
#define N_TRAIN_FIELDS  4
#define N_TAG_FIELDS    3
#define BEAM_SIZE       4
#define TAGGER_NAME     "suc_ne"
#define PyInit_TAGGER_NAME PyInit_suc_ne

#include <stdint.h>

typedef uint32_t   partial_hash_t;
typedef uint32_t   feat_hash_t;
typedef uint32_t   label;

#ifndef __HASH_C__
#define __HASH_C__

#include <stdint.h>
#include <string.h>
#include <stddef.h>

// Code adapted from: http://bjoern.hoehrmann.de/utf-8/decoder/dfa/

#define UTF8_ACCEPT 0
#define UTF8_REJECT 12

static const uint8_t utf8d[] = {
  // The first part of the table maps bytes to character classes that
  // to reduce the size of the transition table and create bitmasks.
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
   7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
   8,8,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  10,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3, 11,6,6,6,5,8,8,8,8,8,8,8,8,8,8,8,

  // The second part is a transition table that maps a combination
  // of a state of the automaton and a character class to a state.
   0,12,24,36,60,96,84,12,12,12,48,72, 12,12,12,12,12,12,12,12,12,12,12,12,
  12, 0,12,12,12,12,12, 0,12, 0,12,12, 12,24,12,12,12,12,12,24,12,24,12,12,
  12,12,12,12,12,12,12,24,12,12,12,12, 12,24,12,12,12,12,12,12,12,24,12,12,
  12,12,12,12,12,12,12,36,12,36,12,12, 12,36,12,12,12,12,12,36,12,36,12,12,
  12,36,12,12,12,12,12,12,12,12,12,12, 
};

static inline void utf8_step(uint32_t *state, uint32_t *codep, uint32_t byte) {
    const uint32_t type = utf8d[byte];
    
    *codep = (*state != UTF8_ACCEPT) ?
        (byte & 0x3fu) | (*codep << 6) :
        (0xff >> type) & (byte);
    *state = utf8d[256 + *state + type];
}

static inline int utf8_decode(
        uint32_t *dest,
        size_t *dest_len,
        const uint8_t *src,
        size_t src_len)
{
    size_t i;
    uint32_t state = 0;
    uint32_t codep;
    size_t j = 0;
    for (i=0; i<src_len; i++) {
        utf8_step(&state, &codep, src[i]);
        if (!state) {
            dest[j++] = codep;
            if (j >= *dest_len) return -1;
        }
    }
    *dest_len = j;
    if (state != UTF8_ACCEPT) return -1;
    return 0;
}

static inline uint32_t rotl32(uint32_t x, int r) {
    return (x << r) | (x >> (32 - r));
}

static inline uint64_t rotl64(uint64_t x, int r) {
    return (x << r) | (x >> (64 - r));
}

static inline uint32_t hash32_mix(uint32_t x, uint32_t y) {
    x = rotl32(x * 0xcc9e2d51, 15) * 0x1b873593;
    y = rotl32(y ^ x, 13) * 5 + 0xe6546b64;
    return y;
}

static inline uint64_t hash64_mix(uint64_t x, uint64_t y) {
    x = rotl64(x * 14029467366897019727ULL, 31) * 11400714785074694791ULL;
    y = rotl64(y ^ x, 31) * 5 + 0xbdef9f91b243c6e6ULL;
    return y;
}

static inline uint32_t hash32_mix_tail(uint32_t x, uint32_t y) {
    x = rotl32(x * 0xcc9e2d51, 15) * 0x1b873593;
    return y ^ x;
}

static inline uint64_t hash64_mix_tail(uint64_t x, uint64_t y) {
    x = rotl64(x * 14029467366897019727ULL, 31) * 11400714785074694791ULL;
    return y ^ x;
}

static inline uint32_t hash32_fmix(uint32_t x) {
    x = 0x85ebca6b * (x ^ (x >> 16));
    x = 0xc2b2ae35 * (x ^ (x >> 13));
    return x ^ (x >> 16);
}

static inline uint64_t hash64_fmix(uint64_t x) {
    x = (x ^ (x >> 33)) * 14029467366897019727ULL;
    x = (x ^ (x >> 29)) * 1609587929392839161ULL;
    return x ^ (x >> 32);
}

static inline uint32_t hash32_partial_unicode(
        const uint32_t *p,
        size_t len)
{
    if (len == 0) return 0x3a5c441;

    uint32_t h1 = p[0];
    size_t i;

    for (i=1; i<len; i++)
        h1 = hash32_mix(p[i], h1);

    return h1 ^ (uint32_t)len;
}

static inline uint64_t hash64_partial_unicode(
        const uint32_t *p,
        size_t len)
{
    if (len == 0) return 0x7fb838a8a0a95046ULL;

    uint64_t h1 = (len == 1)? (uint64_t)p[0] :
                              ((uint64_t)p[0] | ((uint64_t)p[1] << 32));
    size_t i;

    for (i=1; i<len/2; i++)
        h1 = hash64_mix((uint64_t)p[i*2] | ((uint64_t)p[i*2+1] << 32), h1);

    if (len % 2)
        h1 = hash64_mix((uint64_t)p[len-1], h1);

    return h1 ^ (uint64_t)len;
}

static inline uint32_t hash32_partial_unicode_suffix(
        const uint32_t *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    if (min_stem + suffix_len > len) return 0x34b020cc;
    return hash32_partial_unicode(p+(len-suffix_len), suffix_len);
}

static inline uint64_t hash64_partial_unicode_suffix(
        const uint32_t *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    if (min_stem + suffix_len > len) return 0xb9d9d9fb4440f7bbULL;
    return hash64_partial_unicode(p+(len-suffix_len), suffix_len);
}

static inline uint32_t hash32_partial_unicode_prefix(
        const uint32_t *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    if (min_stem + prefix_len > len) return 0x719986aa;
    return hash32_partial_unicode(p, prefix_len);
}

static inline uint64_t hash64_partial_unicode_prefix(
        const uint32_t *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    if (min_stem + prefix_len > len) return 0xc1a7bd3b4e853fc9ULL;
    return hash64_partial_unicode(p, prefix_len);
}

static inline uint32_t read32(const void *p) {
    uint32_t x;
    memcpy(&x, p, sizeof(x));
    return x;
}

static inline uint64_t read64(const void *p) {
    uint64_t x;
    memcpy(&x, p, sizeof(x));
    return x;
}

static inline uint32_t read32_part(const void *p, size_t len) {
    const uint8_t *b = p;
    if (len == 3) return b[0] | (b[1] << 8) | (b[2] << 16);
    else if (len == 2) return b[0] | (b[1] << 8);
    return b[0];
}

static inline uint64_t read64_part(const void *p, size_t len) {
    const uint8_t *b = p;
    uint64_t x = b[0];
    switch (len) {
        case 6: x |= (uint64_t)b[6] << 48;
        case 5: x |= (uint64_t)b[5] << 40;
        case 4: x |= (uint64_t)b[4] << 32;
        case 3: x |= (uint64_t)b[3] << 24;
        case 2: x |= (uint64_t)b[2] << 16;
        case 1: x |= (uint64_t)b[1] << 8;
    }
    return x;
}

static uint32_t hash32_data(uint32_t seed, const void *ptr, size_t len) {
    uint32_t h1 = seed;
    size_t i;

    for (i=0; i+4<=len; i+=4)
        h1 = hash32_mix(read32(ptr+i), h1);

    if (len-i)
        h1 = hash32_mix_tail(read32_part(ptr+i, len-i), h1);

    return hash32_fmix(h1 ^ (uint32_t)len);
}

static uint64_t hash64_data(uint64_t seed, const void *ptr, size_t len) {
    uint64_t h1 = seed;
    size_t i;

    for (i=0; i+8<=len; i+=4)
        h1 = hash64_mix(read64(ptr+i), h1);

    if (len-i)
        h1 = hash64_mix_tail(read64_part(ptr+i, len-i), h1);

    return hash64_fmix(h1 ^ (uint64_t)len);
}

// TODO: less naive implementation, if necessary
static uint32_t hash32_utf8_prefix(
        const void *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    uint32_t buf[len];
    size_t buf_len = len;
    utf8_decode(buf, &buf_len, p, len);
    return hash32_fmix(
            hash32_partial_unicode_prefix(buf, buf_len, prefix_len, min_stem));
}

// TODO: less naive implementation, if necessary
static uint32_t hash32_utf8_suffix(
        const void *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    uint32_t buf[len];
    size_t buf_len = len;
    utf8_decode(buf, &buf_len, p, len);
    return hash32_fmix(
            hash32_partial_unicode_suffix(buf, buf_len, suffix_len, min_stem));
}

#endif

#ifndef __HEADERS_H__
#define __HEADERS_H__

// Using branching hints gives a significant (~10%) speed improvement on my
// system since the innermost loop uses plenty of them.
#define likely(x)       __builtin_expect((x),1)
#define unlikely(x)     __builtin_expect((x),0)

#define MAX_STR         0x1000

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include <float.h>

// Uncomment this to do perform separate training over a number of weight
// vector lengths, choosing the best model (on the dev set):
//#define MIN_WEIGHTS_LEN     0x40000
//#define MAX_WEIGHTS_LEN     0x4000000

// Or just fix the length:
#define MIN_WEIGHTS_LEN     0x4000000
#define MAX_WEIGHTS_LEN     0x4000000

// Uncomment this to compress the weight vector after training:
#define POST_TRAINING_COMPRESSION

// A non-zero value means dropout is used, but empirically this doesn't seem
// to help
#define DROPOUT_RATE        0
#define DROPOUT_CONSTANT    ((uint32_t)((double)DROPOUT_RATE*4294967296.0))

typedef float real;
#define REAL_MAX        FLT_MAX

typedef struct {
    uint32_t hash;
    const void *value;
} hash32_kv;

typedef struct {
    uint64_t hash;
    const void *value;
} hash64_kv;

typedef struct {
    uint32_t hash;
    label value;
} hash32_kv_label;

typedef struct {
    uint64_t hash;
    label value;
} hash64_kv_label;

static inline real get_score(
        const feat_hash_t *hashes,
        size_t len,
        const real *weights,
        size_t weights_len,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    size_t i;
    const feat_hash_t mask = (feat_hash_t)weights_len - 1;
    if (use_dropout) {
        real sum = (real)0.0;
        for (i=0; i<len; i++) {
            const feat_hash_t h = hashes[i];
            if (hash32_mix(dropout_seed, h) >= DROPOUT_CONSTANT)
                sum += weights[h & mask];
        }
        // NOTE: since the scale of the feature vector does not matter, we do
        //       not scale the sum here.
        return sum;
    } else {
        real sum = weights[hashes[0] & mask];
        for (i=1; i<len; i++) sum += weights[hashes[i] & mask];
        return sum;
    }
}


#endif

static const uint32_t normalize_tab[] = {
    0,
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
    13,
    14,
    15,
    16,
    17,
    18,
    19,
    20,
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
    37,
    38,
    39,
    40,
    41,
    42,
    43,
    44,
    45,
    46,
    47,
    48,
    49,
    50,
    51,
    52,
    53,
    54,
    55,
    56,
    57,
    58,
    59,
    60,
    61,
    62,
    63,
    64,
    97,
    98,
    99,
    100,
    101,
    102,
    103,
    104,
    105,
    106,
    107,
    108,
    109,
    110,
    111,
    112,
    113,
    114,
    115,
    116,
    117,
    118,
    119,
    120,
    121,
    122,
    91,
    92,
    93,
    94,
    95,
    96,
    97,
    98,
    99,
    100,
    101,
    102,
    103,
    104,
    105,
    106,
    107,
    108,
    109,
    110,
    111,
    112,
    113,
    114,
    115,
    116,
    117,
    118,
    119,
    120,
    121,
    122,
    123,
    124,
    125,
    126,
    127,
    128,
    129,
    130,
    131,
    132,
    133,
    134,
    135,
    136,
    137,
    138,
    139,
    140,
    141,
    142,
    143,
    144,
    145,
    146,
    147,
    148,
    149,
    150,
    151,
    152,
    153,
    154,
    155,
    156,
    157,
    158,
    159,
    160,
    161,
    162,
    163,
    164,
    165,
    166,
    167,
    168,
    169,
    170,
    171,
    172,
    173,
    174,
    175,
    176,
    177,
    178,
    179,
    180,
    181,
    182,
    183,
    184,
    185,
    186,
    187,
    188,
    189,
    190,
    191,
    224,
    225,
    226,
    227,
    228,
    229,
    230,
    231,
    232,
    233,
    234,
    235,
    236,
    237,
    238,
    239,
    240,
    241,
    242,
    243,
    244,
    245,
    246,
    215,
    248,
    249,
    250,
    251,
    252,
    253,
    254,
    223,
    224,
    225,
    226,
    227,
    228,
    229,
    230,
    231,
    232,
    233,
    234,
    235,
    236,
    237,
    238,
    239,
    240,
    241,
    242,
    243,
    244,
    245,
    246,
    247,
    248,
    249,
    250,
    251,
    252,
    253,
    254,
    255,
    257,
    257,
    259,
    259,
    261,
    261,
    263,
    263,
    265,
    265,
    267,
    267,
    269,
    269,
    271,
    271,
    273,
    273,
    275,
    275,
    277,
    277,
    279,
    279,
    281,
    281,
    283,
    283,
    285,
    285,
    287,
    287,
    289,
    289,
    291,
    291,
    293,
    293,
    295,
    295,
    297,
    297,
    299,
    299,
    301,
    301,
    303,
    303,
    105,
    305,
    307,
    307,
    309,
    309,
    311,
    311,
    312,
    314,
    314,
    316,
    316,
    318,
    318,
    320,
    320,
    322,
    322,
    324,
    324,
    326,
    326,
    328,
    328,
    329,
    331,
    331,
    333,
    333,
    335,
    335,
    337,
    337,
    339,
    339,
    341,
    341,
    343,
    343,
    345,
    345,
    347,
    347,
    349,
    349,
    351,
    351,
    353,
    353,
    355,
    355,
    357,
    357,
    359,
    359,
    361,
    361,
    363,
    363,
    365,
    365,
    367,
    367,
    369,
    369,
    371,
    371,
    373,
    373,
    375,
    375,
    255,
    378,
    378,
    380,
    380,
    382,
    382,
    383,
    384,
    595,
    387,
    387,
    389,
    389,
    596,
    392,
    392,
    598,
    599,
    396,
    396,
    397,
    477,
    601,
    603,
    402,
    402,
    608,
    611,
    405,
    617,
    616,
    409,
    409,
    410,
    411,
    623,
    626,
    414,
    629,
    417,
    417,
    419,
    419,
    421,
    421,
    640,
    424,
    424,
    643,
    426,
    427,
    429,
    429,
    648,
    432,
    432,
    650,
    651,
    436,
    436,
    438,
    438,
    658,
    441,
    441,
    442,
    443,
    445,
    445,
    446,
    447,
    448,
    449,
    450,
    451,
    454,
    454,
    454,
    457,
    457,
    457,
    460,
    460,
    460,
    462,
    462,
    464,
    464,
    466,
    466,
    468,
    468,
    470,
    470,
    472,
    472,
    474,
    474,
    476,
    476,
    477,
    479,
    479,
    481,
    481,
    483,
    483,
    485,
    485,
    487,
    487,
    489,
    489,
    491,
    491,
    493,
    493,
    495,
    495,
    496,
    499,
    499,
    499,
    501,
    501,
    405,
    447,
    505,
    505,
    507,
    507,
    509,
    509,
    511,
    511,
    513,
    513,
    515,
    515,
    517,
    517,
    519,
    519,
    521,
    521,
    523,
    523,
    525,
    525,
    527,
    527,
    529,
    529,
    531,
    531,
    533,
    533,
    535,
    535,
    537,
    537,
    539,
    539,
    541,
    541,
    543,
    543,
    414,
    545,
    547,
    547,
    549,
    549,
    551,
    551,
    553,
    553,
    555,
    555,
    557,
    557,
    559,
    559,
    561,
    561,
    563,
    563,
    564,
    565,
    566,
    567,
    568,
    569,
    11365,
    572,
    572,
    410,
    11366,
    575,
    576,
    578,
    578,
    384,
    649,
    652,
    583,
    583,
    585,
    585,
    587,
    587,
    589,
    589,
    591,
    591,
    592,
    593,
    594,
    595,
    596,
    597,
    598,
    599,
    600,
    601,
    602,
    603,
    604,
    605,
    606,
    607,
    608,
    609,
    610,
    611,
    612,
    613,
    614,
    615,
    616,
    617,
    618,
    619,
    620,
    621,
    622,
    623,
    624,
    625,
    626,
    627,
    628,
    629,
    630,
    631,
    632,
    633,
    634,
    635,
    636,
    637,
    638,
    639,
    640,
    641,
    642,
    643,
    644,
    645,
    646,
    647,
    648,
    649,
    650,
    651,
    652,
    653,
    654,
    655,
    656,
    657,
    658,
    659,
    660,
    661,
    662,
    663,
    664,
    665,
    666,
    667,
    668,
    669,
    670,
    671,
    672,
    673,
    674,
    675,
    676,
    677,
    678,
    679,
    680,
    681,
    682,
    683,
    684,
    685,
    686,
    687,
    688,
    689,
    690,
    691,
    692,
    693,
    694,
    695,
    696,
    697,
    698,
    699,
    700,
    701,
    702,
    703,
    704,
    705,
    706,
    707,
    708,
    709,
    710,
    711,
    712,
    713,
    714,
    715,
    716,
    717,
    718,
    719,
    720,
    721,
    722,
    723,
    724,
    725,
    726,
    727,
    728,
    729,
    730,
    731,
    732,
    733,
    734,
    735,
    736,
    737,
    738,
    739,
    740,
    741,
    742,
    743,
    744,
    745,
    746,
    747,
    748,
    749,
    750,
    751,
    752,
    753,
    754,
    755,
    756,
    757,
    758,
    759,
    760,
    761,
    762,
    763,
    764,
    765,
    766,
    767,
    768,
    769,
    770,
    771,
    772,
    773,
    774,
    775,
    776,
    777,
    778,
    779,
    780,
    781,
    782,
    783,
    784,
    785,
    786,
    787,
    788,
    789,
    790,
    791,
    792,
    793,
    794,
    795,
    796,
    797,
    798,
    799,
    800,
    801,
    802,
    803,
    804,
    805,
    806,
    807,
    808,
    809,
    810,
    811,
    812,
    813,
    814,
    815,
    816,
    817,
    818,
    819,
    820,
    821,
    822,
    823,
    824,
    825,
    826,
    827,
    828,
    829,
    830,
    831,
    768,
    769,
    834,
    787,
    776,
    837,
    838,
    839,
    840,
    841,
    842,
    843,
    844,
    845,
    846,
    847,
    848,
    849,
    850,
    851,
    852,
    853,
    854,
    855,
    856,
    857,
    858,
    859,
    860,
    861,
    862,
    863,
    864,
    865,
    866,
    867,
    868,
    869,
    870,
    871,
    872,
    873,
    874,
    875,
    876,
    877,
    878,
    879,
    881,
    881,
    883,
    883,
    697,
    885,
    887,
    887,
    888,
    889,
    890,
    891,
    892,
    893,
    59,
    895,
    896,
    897,
    898,
    899,
    900,
    901,
    940,
    183,
    941,
    942,
    943,
    907,
    972,
    909,
    973,
    974,
    912,
    945,
    946,
    947,
    948,
    949,
    950,
    951,
    952,
    953,
    954,
    955,
    956,
    957,
    958,
    959,
    960,
    961,
    930,
    963,
    964,
    965,
    966,
    967,
    968,
    969,
    970,
    971,
    940,
    941,
    942,
    943,
    944,
    945,
    946,
    947,
    948,
    949,
    950,
    951,
    952,
    953,
    954,
    955,
    956,
    957,
    958,
    959,
    960,
    961,
    962,
    963,
    964,
    965,
    966,
    967,
    968,
    969,
    970,
    971,
    972,
    973,
    974,
    983,
    976,
    977,
    978,
    979,
    980,
    981,
    982,
    983,
    985,
    985,
    987,
    987,
    989,
    989,
    991,
    991,
    993,
    993,
    995,
    995,
    997,
    997,
    999,
    999,
    1001,
    1001,
    1003,
    1003,
    1005,
    1005,
    1007,
    1007,
    1008,
    1009,
    1010,
    1011,
    952,
    1013,
    1014,
    1016,
    1016,
    1010,
    1019,
    1019,
    1020,
    891,
    892,
    893,
    1104,
    1105,
    1106,
    1107,
    1108,
    1109,
    1110,
    1111,
    1112,
    1113,
    1114,
    1115,
    1116,
    1117,
    1118,
    1119,
    1072,
    1073,
    1074,
    1075,
    1076,
    1077,
    1078,
    1079,
    1080,
    1081,
    1082,
    1083,
    1084,
    1085,
    1086,
    1087,
    1088,
    1089,
    1090,
    1091,
    1092,
    1093,
    1094,
    1095,
    1096,
    1097,
    1098,
    1099,
    1100,
    1101,
    1102,
    1103,
    1072,
    1073,
    1074,
    1075,
    1076,
    1077,
    1078,
    1079,
    1080,
    1081,
    1082,
    1083,
    1084,
    1085,
    1086,
    1087,
    1088,
    1089,
    1090,
    1091,
    1092,
    1093,
    1094,
    1095,
    1096,
    1097,
    1098,
    1099,
    1100,
    1101,
    1102,
    1103,
    1104,
    1105,
    1106,
    1107,
    1108,
    1109,
    1110,
    1111,
    1112,
    1113,
    1114,
    1115,
    1116,
    1117,
    1118,
    1119,
    1121,
    1121,
    1123,
    1123,
    1125,
    1125,
    1127,
    1127,
    1129,
    1129,
    1131,
    1131,
    1133,
    1133,
    1135,
    1135,
    1137,
    1137,
    1139,
    1139,
    1141,
    1141,
    1143,
    1143,
    1145,
    1145,
    1147,
    1147,
    1149,
    1149,
    1151,
    1151,
    1153,
    1153,
    1154,
    1155,
    1156,
    1157,
    1158,
    1159,
    1160,
    1161,
    1163,
    1163,
    1165,
    1165,
    1167,
    1167,
    1169,
    1169,
    1171,
    1171,
    1173,
    1173,
    1175,
    1175,
    1177,
    1177,
    1179,
    1179,
    1181,
    1181,
    1183,
    1183,
    1185,
    1185,
    1187,
    1187,
    1189,
    1189,
    1191,
    1191,
    1193,
    1193,
    1195,
    1195,
    1197,
    1197,
    1199,
    1199,
    1201,
    1201,
    1203,
    1203,
    1205,
    1205,
    1207,
    1207,
    1209,
    1209,
    1211,
    1211,
    1213,
    1213,
    1215,
    1215,
    1231,
    1218,
    1218,
    1220,
    1220,
    1222,
    1222,
    1224,
    1224,
    1226,
    1226,
    1228,
    1228,
    1230,
    1230,
    1231,
    1233,
    1233,
    1235,
    1235,
    1237,
    1237,
    1239,
    1239,
    1241,
    1241,
    1243,
    1243,
    1245,
    1245,
    1247,
    1247,
    1249,
    1249,
    1251,
    1251,
    1253,
    1253,
    1255,
    1255,
    1257,
    1257,
    1259,
    1259,
    1261,
    1261,
    1263,
    1263,
    1265,
    1265,
    1267,
    1267,
    1269,
    1269,
    1271,
    1271,
    1273,
    1273,
    1275,
    1275,
    1277,
    1277,
    1279,
    1279,
    1281,
    1281,
    1283,
    1283,
    1285,
    1285,
    1287,
    1287,
    1289,
    1289,
    1291,
    1291,
    1293,
    1293,
    1295,
    1295,
    1297,
    1297,
    1299,
    1299,
    1301,
    1301,
    1303,
    1303,
    1305,
    1305,
    1307,
    1307,
    1309,
    1309,
    1311,
    1311,
    1313,
    1313,
    1315,
    1315,
    1317,
    1317,
    1319,
    1319,
    1320,
    1321,
    1322,
    1323,
    1324,
    1325,
    1326,
    1327
};

static const uint8_t delex_tab[] = {
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    29,
    21,
    21,
    21,
    23,
    21,
    21,
    21,
    22,
    18,
    21,
    25,
    21,
    17,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    25,
    25,
    25,
    21,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    22,
    21,
    18,
    24,
    16,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    22,
    25,
    18,
    25,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    29,
    21,
    23,
    23,
    23,
    23,
    26,
    21,
    24,
    26,
    7,
    20,
    25,
    1,
    26,
    24,
    26,
    25,
    15,
    15,
    24,
    5,
    21,
    21,
    24,
    15,
    7,
    19,
    15,
    15,
    15,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    25,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    25,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    5,
    7,
    9,
    5,
    5,
    5,
    7,
    7,
    7,
    7,
    9,
    8,
    5,
    9,
    8,
    5,
    9,
    8,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    8,
    5,
    9,
    5,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    9,
    5,
    5,
    9,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    7,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    24,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    9,
    5,
    9,
    5,
    6,
    24,
    9,
    5,
    2,
    2,
    6,
    5,
    5,
    5,
    21,
    2,
    2,
    2,
    2,
    2,
    24,
    24,
    9,
    21,
    9,
    9,
    9,
    2,
    9,
    2,
    9,
    9,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    5,
    9,
    9,
    9,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    25,
    9,
    5,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    26,
    12,
    12,
    12,
    12,
    12,
    11,
    11,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    21,
    17,
    2,
    2,
    2,
    2,
    23,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    17,
    12,
    21,
    12,
    12,
    21,
    12,
    12,
    21,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,
    1,
    1,
    1,
    1,
    2,
    25,
    25,
    25,
    21,
    21,
    23,
    21,
    21,
    26,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    1,
    2,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    1,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    6,
    12,
    12,
    26,
    12,
    12,
    12,
    12,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    26,
    26,
    7,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    1,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    6,
    26,
    21,
    21,
    21,
    6,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    2,
    2,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    12,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    2,
    2,
    10,
    10,
    2,
    2,
    10,
    10,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    23,
    23,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    23,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    2,
    12,
    2,
    10,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    12,
    12,
    2,
    2,
    12,
    12,
    12,
    2,
    2,
    2,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    12,
    12,
    7,
    7,
    7,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    10,
    2,
    10,
    10,
    12,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    23,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    12,
    10,
    12,
    12,
    12,
    12,
    2,
    2,
    10,
    10,
    2,
    2,
    10,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    26,
    7,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    10,
    10,
    12,
    10,
    10,
    2,
    2,
    2,
    10,
    10,
    10,
    2,
    10,
    10,
    10,
    12,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    23,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    2,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    2,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    2,
    12,
    10,
    10,
    2,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    2,
    10,
    10,
    10,
    2,
    10,
    10,
    10,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    12,
    12,
    12,
    2,
    12,
    2,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    23,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    6,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    26,
    26,
    26,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    26,
    21,
    26,
    26,
    26,
    12,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    12,
    26,
    12,
    26,
    12,
    22,
    18,
    22,
    18,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    21,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    21,
    21,
    21,
    21,
    21,
    26,
    26,
    26,
    26,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    10,
    10,
    12,
    12,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    7,
    10,
    10,
    10,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    10,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    7,
    10,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    10,
    10,
    10,
    12,
    26,
    26,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    9,
    2,
    2,
    2,
    2,
    2,
    9,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    17,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    29,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    22,
    18,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    14,
    14,
    14,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    23,
    7,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    21,
    21,
    21,
    21,
    12,
    12,
    12,
    1,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    12,
    12,
    10,
    10,
    10,
    2,
    2,
    2,
    2,
    10,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    26,
    2,
    2,
    2,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    10,
    10,
    12,
    2,
    2,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    10,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    12,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    10,
    12,
    10,
    10,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    12,
    10,
    10,
    10,
    12,
    10,
    12,
    12,
    12,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    7,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    6,
    6,
    6,
    6,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    21,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    9,
    2,
    9,
    2,
    9,
    2,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    5,
    24,
    24,
    24,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    24,
    24,
    5,
    5,
    5,
    5,
    2,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    2,
    24,
    24,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    24,
    24,
    24,
    2,
    2,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    24,
    2,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    1,
    1,
    1,
    1,
    1,
    17,
    17,
    17,
    17,
    17,
    17,
    21,
    21,
    20,
    19,
    22,
    20,
    20,
    19,
    22,
    20,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    27,
    28,
    1,
    1,
    1,
    1,
    1,
    29,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    20,
    19,
    21,
    21,
    21,
    21,
    16,
    16,
    21,
    21,
    21,
    25,
    22,
    18,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    25,
    21,
    16,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    29,
    1,
    1,
    1,
    1,
    1,
    2,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    15,
    6,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    25,
    25,
    25,
    22,
    18,
    6,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    25,
    25,
    25,
    22,
    18,
    2,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    2,
    2,
    2,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    11,
    11,
    11,
    11,
    12,
    11,
    11,
    11,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    9,
    26,
    26,
    26,
    26,
    9,
    26,
    26,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    5,
    26,
    9,
    26,
    26,
    25,
    9,
    9,
    9,
    9,
    9,
    26,
    26,
    26,
    26,
    26,
    26,
    9,
    26,
    9,
    26,
    9,
    26,
    9,
    9,
    9,
    9,
    26,
    5,
    9,
    9,
    9,
    9,
    5,
    7,
    7,
    7,
    7,
    5,
    26,
    26,
    5,
    5,
    9,
    9,
    25,
    25,
    25,
    25,
    25,
    9,
    5,
    5,
    5,
    5,
    26,
    25,
    26,
    26,
    5,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    9,
    5,
    14,
    14,
    14,
    14,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    25,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    25,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    22,
    18,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    26,
    26,
    26,
    26,
    26,
    26,
    9,
    5,
    9,
    5,
    12,
    12,
    12,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    15,
    21,
    21,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    5,
    2,
    2,
    2,
    2,
    2,
    5,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    6,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    20,
    19,
    20,
    19,
    21,
    21,
    21,
    20,
    19,
    21,
    20,
    19,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    21,
    21,
    17,
    21,
    20,
    19,
    21,
    21,
    20,
    19,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    17,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    29,
    21,
    21,
    21,
    26,
    6,
    7,
    14,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    26,
    26,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    17,
    22,
    18,
    18,
    26,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    12,
    12,
    12,
    12,
    10,
    10,
    17,
    6,
    6,
    6,
    6,
    6,
    26,
    26,
    14,
    14,
    14,
    6,
    7,
    21,
    26,
    26,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    12,
    24,
    24,
    6,
    6,
    7,
    17,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    6,
    6,
    6,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    26,
    26,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    6,
    6,
    6,
    6,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    7,
    12,
    11,
    11,
    11,
    21,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    6,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    6,
    24,
    24,
    9,
    5,
    9,
    5,
    2,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    6,
    6,
    5,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    10,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    23,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    7,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    10,
    10,
    10,
    10,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    6,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    7,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    12,
    12,
    12,
    7,
    7,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    7,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    6,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    12,
    10,
    10,
    21,
    21,
    7,
    6,
    6,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    10,
    10,
    12,
    10,
    10,
    21,
    10,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    2,
    2,
    2,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    25,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    22,
    18,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    23,
    26,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    22,
    18,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    17,
    17,
    16,
    16,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    22,
    18,
    21,
    21,
    21,
    21,
    16,
    16,
    16,
    21,
    21,
    21,
    2,
    21,
    21,
    21,
    21,
    17,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    21,
    25,
    17,
    25,
    25,
    25,
    2,
    21,
    23,
    21,
    21,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    1,
    2,
    21,
    21,
    21,
    23,
    21,
    21,
    21,
    22,
    18,
    21,
    25,
    21,
    17,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    25,
    25,
    25,
    21,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    22,
    21,
    18,
    24,
    16,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    22,
    25,
    18,
    25,
    22,
    18,
    21,
    22,
    18,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    2,
    2,
    2,
    23,
    23,
    25,
    24,
    26,
    23,
    23,
    2,
    26,
    25,
    25,
    25,
    25,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,
    1,
    1,
    26,
    26,
    2,
    2
};


static inline void unicode_translate_normalize(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, len = (src_len <= *dest_len)? src_len : *dest_len;
    for (i=0; i<len; i++) {
        if (likely(src[i] < 0x530)) dest[i] = normalize_tab[src[i]];
        else dest[i] = src[i];
    }
    *dest_len = len;
}

static inline void unicode_translate_delexicalize(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, len = (src_len <= *dest_len)? src_len : *dest_len;
    for (i=0; i<len; i++) {
        if (likely(src[i] < 0x10000)) dest[i] = delex_tab[src[i]];
        else dest[i] = 30;
    }
    *dest_len = len;
}

static inline void unicode_translate_abstract(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, n = 0;
    uint8_t last = 0xff;

    for (i=0; i<src_len; i++) {
        const uint8_t c = (likely(src[i] < 0x10000))? delex_tab[src[i]] : 30;
        if (c != last) {
            dest[n++] = c;
            last = c;
            if (n >= *dest_len) break;
        }
    }
    *dest_len = n;
}

#define COL_TAG 3
#define suc_ne_N_TAGS 19
#define suc_ne_N_BITS 5
#define N_TAGS suc_ne_N_TAGS
#define tag_str suc_ne_tag_str

static const char *suc_ne_tag_str[suc_ne_N_TAGS] = {
    "B-animal",
    "B-event",
    "B-inst",
    "B-myth",
    "B-other",
    "B-person",
    "B-place",
    "B-product",
    "B-work",
    "I-animal",
    "I-event",
    "I-inst",
    "I-myth",
    "I-other",
    "I-person",
    "I-place",
    "I-product",
    "I-work",
    "O"
};


static const int suc_ne_from_str(const char *s) {
    int i;
    for (i=0; i<suc_ne_N_TAGS; i++) if (!strcmp(s, suc_ne_tag_str[i])) return i;
    return -1;
}

#define tagset_from_str suc_ne_from_str
#define SUC_NE_lexicon_size 0x20000

static const label SUC_NE_lexicon_tags_0[] = { 2, 5, 14 };
static const label SUC_NE_lexicon_tags_1[] = { 1, 2 };
static const label SUC_NE_lexicon_tags_2[] = { 1, 18 };
static const label SUC_NE_lexicon_tags_3[] = { 1, 14 };
static const label SUC_NE_lexicon_tags_4[] = { 1, 6 };
static const label SUC_NE_lexicon_tags_5[] = { 1, 8 };
static const label SUC_NE_lexicon_tags_6[] = { 2, 2, 4 };
static const label SUC_NE_lexicon_tags_7[] = { 1, 5 };
static const label SUC_NE_lexicon_tags_8[] = { 1, 15 };
static const label SUC_NE_lexicon_tags_9[] = { 3, 17, 18, 2 };
static const label SUC_NE_lexicon_tags_10[] = { 1, 1 };
static const label SUC_NE_lexicon_tags_11[] = { 2, 18, 14 };
static const label SUC_NE_lexicon_tags_12[] = { 3, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_13[] = { 2, 6, 2 };
static const label SUC_NE_lexicon_tags_14[] = { 2, 18, 5 };
static const label SUC_NE_lexicon_tags_15[] = { 3, 17, 18, 5 };
static const label SUC_NE_lexicon_tags_16[] = { 4, 11, 18, 2, 8 };
static const label SUC_NE_lexicon_tags_17[] = { 2, 17, 18 };
static const label SUC_NE_lexicon_tags_18[] = { 1, 11 };
static const label SUC_NE_lexicon_tags_19[] = { 1, 17 };
static const label SUC_NE_lexicon_tags_20[] = { 5, 11, 18, 2, 17, 8 };
static const label SUC_NE_lexicon_tags_21[] = { 2, 18, 7 };
static const label SUC_NE_lexicon_tags_22[] = { 2, 6, 5 };
static const label SUC_NE_lexicon_tags_23[] = { 2, 18, 15 };
static const label SUC_NE_lexicon_tags_24[] = { 2, 11, 18 };
static const label SUC_NE_lexicon_tags_25[] = { 2, 6, 18 };
static const label SUC_NE_lexicon_tags_26[] = { 3, 17, 15, 18 };
static const label SUC_NE_lexicon_tags_27[] = { 3, 6, 18, 15 };
static const label SUC_NE_lexicon_tags_28[] = { 2, 5, 8 };
static const label SUC_NE_lexicon_tags_29[] = { 5, 11, 18, 15, 17, 13 };
static const label SUC_NE_lexicon_tags_30[] = { 2, 18, 2 };
static const label SUC_NE_lexicon_tags_31[] = { 2, 2, 7 };
static const label SUC_NE_lexicon_tags_32[] = { 1, 7 };
static const label SUC_NE_lexicon_tags_33[] = { 4, 1, 18, 5, 8 };
static const label SUC_NE_lexicon_tags_34[] = { 2, 18, 8 };
static const label SUC_NE_lexicon_tags_35[] = { 2, 17, 6 };
static const label SUC_NE_lexicon_tags_36[] = { 1, 16 };
static const label SUC_NE_lexicon_tags_37[] = { 3, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_38[] = { 4, 17, 18, 11, 4 };
static const label SUC_NE_lexicon_tags_39[] = { 2, 15, 18 };
static const label SUC_NE_lexicon_tags_40[] = { 1, 4 };
static const label SUC_NE_lexicon_tags_41[] = { 2, 11, 14 };
static const label SUC_NE_lexicon_tags_42[] = { 2, 17, 2 };
static const label SUC_NE_lexicon_tags_43[] = { 4, 11, 18, 15, 17 };
static const label SUC_NE_lexicon_tags_44[] = { 2, 16, 5 };
static const label SUC_NE_lexicon_tags_45[] = { 3, 11, 18, 13 };
static const label SUC_NE_lexicon_tags_46[] = { 3, 17, 18, 11 };
static const label SUC_NE_lexicon_tags_47[] = { 3, 11, 18, 5 };
static const label SUC_NE_lexicon_tags_48[] = { 5, 11, 18, 17, 9, 13 };
static const label SUC_NE_lexicon_tags_49[] = { 3, 17, 18, 8 };
static const label SUC_NE_lexicon_tags_50[] = { 1, 9 };
static const label SUC_NE_lexicon_tags_51[] = { 1, 13 };
static const label SUC_NE_lexicon_tags_52[] = { 2, 1, 18 };
static const label SUC_NE_lexicon_tags_53[] = { 3, 11, 18, 15 };
static const label SUC_NE_lexicon_tags_54[] = { 8, 6, 2, 5, 3, 18, 8, 17, 4 };
static const label SUC_NE_lexicon_tags_55[] = { 3, 18, 4, 13 };
static const label SUC_NE_lexicon_tags_56[] = { 2, 17, 14 };
static const label SUC_NE_lexicon_tags_57[] = { 2, 10, 18 };
static const label SUC_NE_lexicon_tags_58[] = { 2, 18, 4 };
static const label SUC_NE_lexicon_tags_59[] = { 1, 0 };
static const label SUC_NE_lexicon_tags_60[] = { 2, 2, 13 };
static const label SUC_NE_lexicon_tags_61[] = { 3, 6, 15, 18 };
static const label SUC_NE_lexicon_tags_62[] = { 3, 10, 11, 18 };
static const label SUC_NE_lexicon_tags_63[] = { 2, 10, 11 };
static const label SUC_NE_lexicon_tags_64[] = { 2, 2, 5 };
static const label SUC_NE_lexicon_tags_65[] = { 3, 18, 5, 8 };
static const label SUC_NE_lexicon_tags_66[] = { 3, 17, 5, 8 };
static const label SUC_NE_lexicon_tags_67[] = { 4, 17, 18, 2, 8 };
static const label SUC_NE_lexicon_tags_68[] = { 4, 17, 0, 5, 14 };
static const label SUC_NE_lexicon_tags_69[] = { 2, 6, 4 };
static const label SUC_NE_lexicon_tags_70[] = { 3, 18, 13, 4 };
static const label SUC_NE_lexicon_tags_71[] = { 5, 17, 7, 5, 2, 14 };
static const label SUC_NE_lexicon_tags_72[] = { 2, 17, 8 };
static const label SUC_NE_lexicon_tags_73[] = { 9, 2, 5, 18, 1, 14, 8, 17, 11, 4 };
static const label SUC_NE_lexicon_tags_74[] = { 3, 18, 2, 5 };
static const label SUC_NE_lexicon_tags_75[] = { 2, 16, 7 };
static const label SUC_NE_lexicon_tags_76[] = { 2, 18, 3 };
static const label SUC_NE_lexicon_tags_77[] = { 3, 6, 17, 2 };
static const label SUC_NE_lexicon_tags_78[] = { 3, 17, 16, 7 };
static const label SUC_NE_lexicon_tags_79[] = { 5, 10, 11, 18, 17, 9 };
static const label SUC_NE_lexicon_tags_80[] = { 3, 6, 2, 8 };
static const label SUC_NE_lexicon_tags_81[] = { 2, 18, 13 };
static const label SUC_NE_lexicon_tags_82[] = { 10, 15, 2, 5, 16, 18, 14, 8, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_83[] = { 2, 6, 7 };
static const label SUC_NE_lexicon_tags_84[] = { 2, 11, 6 };
static const label SUC_NE_lexicon_tags_85[] = { 2, 11, 17 };
static const label SUC_NE_lexicon_tags_86[] = { 2, 2, 14 };
static const label SUC_NE_lexicon_tags_87[] = { 3, 11, 18, 8 };
static const label SUC_NE_lexicon_tags_88[] = { 4, 11, 15, 18, 5 };
static const label SUC_NE_lexicon_tags_89[] = { 3, 6, 18, 2 };
static const label SUC_NE_lexicon_tags_90[] = { 6, 15, 5, 18, 14, 13, 4 };
static const label SUC_NE_lexicon_tags_91[] = { 1, 3 };
static const label SUC_NE_lexicon_tags_92[] = { 3, 6, 15, 2 };
static const label SUC_NE_lexicon_tags_93[] = { 2, 9, 13 };
static const label SUC_NE_lexicon_tags_94[] = { 4, 18, 2, 5, 14 };
static const label SUC_NE_lexicon_tags_95[] = { 3, 17, 18, 6 };
static const label SUC_NE_lexicon_tags_96[] = { 3, 17, 18, 14 };
static const label SUC_NE_lexicon_tags_97[] = { 3, 11, 2, 13 };
static const label SUC_NE_lexicon_tags_98[] = { 2, 11, 2 };
static const label SUC_NE_lexicon_tags_99[] = { 6, 15, 18, 14, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_100[] = { 3, 10, 18, 13 };
static const label SUC_NE_lexicon_tags_101[] = { 2, 10, 17 };
static const label SUC_NE_lexicon_tags_102[] = { 3, 11, 18, 2 };
static const label SUC_NE_lexicon_tags_103[] = { 2, 18, 1 };
static const label SUC_NE_lexicon_tags_104[] = { 2, 16, 18 };
static const label SUC_NE_lexicon_tags_105[] = { 4, 17, 18, 2, 4 };
static const label SUC_NE_lexicon_tags_106[] = { 3, 6, 18, 17 };
static const label SUC_NE_lexicon_tags_107[] = { 2, 17, 5 };
static const label SUC_NE_lexicon_tags_108[] = { 3, 18, 2, 4 };
static const label SUC_NE_lexicon_tags_109[] = { 2, 18, 16 };
static const label SUC_NE_lexicon_tags_110[] = { 3, 6, 18, 0 };
static const label SUC_NE_lexicon_tags_111[] = { 2, 15, 5 };
static const label SUC_NE_lexicon_tags_112[] = { 2, 9, 14 };
static const label SUC_NE_lexicon_tags_113[] = { 3, 17, 5, 14 };
static const label SUC_NE_lexicon_tags_114[] = { 4, 18, 3, 14, 12 };
static const label SUC_NE_lexicon_tags_115[] = { 4, 17, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_116[] = { 4, 6, 15, 2, 18 };
static const label SUC_NE_lexicon_tags_117[] = { 3, 16, 7, 18 };
static const label SUC_NE_lexicon_tags_118[] = { 3, 7, 5, 14 };
static const label SUC_NE_lexicon_tags_119[] = { 2, 18, 9 };
static const label SUC_NE_lexicon_tags_120[] = { 2, 17, 9 };
static const label SUC_NE_lexicon_tags_121[] = { 3, 6, 11, 2 };
static const label SUC_NE_lexicon_tags_122[] = { 3, 17, 11, 18 };
static const label SUC_NE_lexicon_tags_123[] = { 2, 6, 17 };
static const label SUC_NE_lexicon_tags_124[] = { 4, 10, 11, 18, 2 };
static const label SUC_NE_lexicon_tags_125[] = { 3, 11, 18, 7 };
static const label SUC_NE_lexicon_tags_126[] = { 5, 6, 17, 18, 14, 4 };
static const label SUC_NE_lexicon_tags_127[] = { 3, 6, 2, 4 };
static const label SUC_NE_lexicon_tags_128[] = { 9, 6, 15, 16, 18, 14, 8, 17, 11, 4 };
static const label SUC_NE_lexicon_tags_129[] = { 2, 16, 8 };
static const label SUC_NE_lexicon_tags_130[] = { 2, 11, 15 };
static const label SUC_NE_lexicon_tags_131[] = { 5, 17, 6, 2, 18, 7 };
static const label SUC_NE_lexicon_tags_132[] = { 3, 6, 2, 3 };
static const label SUC_NE_lexicon_tags_133[] = { 4, 17, 18, 15, 6 };
static const label SUC_NE_lexicon_tags_134[] = { 5, 11, 18, 17, 13, 8 };
static const label SUC_NE_lexicon_tags_135[] = { 3, 12, 18, 8 };
static const label SUC_NE_lexicon_tags_136[] = { 7, 2, 18, 8, 17, 11, 7, 4 };
static const label SUC_NE_lexicon_tags_137[] = { 5, 17, 18, 2, 7, 8 };
static const label SUC_NE_lexicon_tags_138[] = { 3, 18, 2, 8 };
static const label SUC_NE_lexicon_tags_139[] = { 5, 17, 18, 2, 13, 8 };
static const label SUC_NE_lexicon_tags_140[] = { 3, 18, 4, 8 };
static const label SUC_NE_lexicon_tags_141[] = { 3, 17, 6, 2 };
static const label SUC_NE_lexicon_tags_142[] = { 4, 5, 3, 13, 8 };
static const label SUC_NE_lexicon_tags_143[] = { 3, 6, 4, 13 };
static const label SUC_NE_lexicon_tags_144[] = { 3, 18, 15, 5 };
static const label SUC_NE_lexicon_tags_145[] = { 2, 6, 11 };
static const label SUC_NE_lexicon_tags_146[] = { 4, 11, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_147[] = { 4, 17, 18, 11, 8 };
static const label SUC_NE_lexicon_tags_148[] = { 3, 17, 18, 7 };
static const label SUC_NE_lexicon_tags_149[] = { 5, 11, 18, 2, 14, 8 };
static const label SUC_NE_lexicon_tags_150[] = { 4, 10, 11, 18, 14 };
static const label SUC_NE_lexicon_tags_151[] = { 3, 6, 18, 14 };
static const label SUC_NE_lexicon_tags_152[] = { 4, 18, 15, 16, 7 };
static const label SUC_NE_lexicon_tags_153[] = { 4, 2, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_154[] = { 3, 11, 17, 18 };
static const label SUC_NE_lexicon_tags_155[] = { 3, 6, 15, 13 };
static const label SUC_NE_lexicon_tags_156[] = { 4, 6, 18, 2, 5 };
static const label SUC_NE_lexicon_tags_157[] = { 4, 17, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_158[] = { 5, 11, 18, 17, 5, 14 };
static const label SUC_NE_lexicon_tags_159[] = { 8, 16, 15, 18, 14, 8, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_160[] = { 2, 7, 2 };
static const label SUC_NE_lexicon_tags_161[] = { 4, 18, 2, 9, 0 };
static const label SUC_NE_lexicon_tags_162[] = { 5, 8, 7, 5, 14, 4 };
static const label SUC_NE_lexicon_tags_163[] = { 2, 6, 1 };
static const label SUC_NE_lexicon_tags_164[] = { 2, 18, 0 };
static const label SUC_NE_lexicon_tags_165[] = { 2, 11, 16 };
static const label SUC_NE_lexicon_tags_166[] = { 3, 18, 13, 14 };
static const label SUC_NE_lexicon_tags_167[] = { 3, 11, 18, 17 };
static const label SUC_NE_lexicon_tags_168[] = { 3, 6, 18, 8 };
static const label SUC_NE_lexicon_tags_169[] = { 4, 6, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_170[] = { 3, 6, 2, 5 };
static const label SUC_NE_lexicon_tags_171[] = { 3, 11, 15, 18 };
static const label SUC_NE_lexicon_tags_172[] = { 4, 6, 18, 2, 8 };
static const label SUC_NE_lexicon_tags_173[] = { 2, 1, 2 };
static const label SUC_NE_lexicon_tags_174[] = { 3, 18, 2, 14 };
static const label SUC_NE_lexicon_tags_175[] = { 3, 7, 4, 14 };
static const label SUC_NE_lexicon_tags_176[] = { 3, 18, 14, 8 };
static const label SUC_NE_lexicon_tags_177[] = { 3, 15, 5, 14 };
static const label SUC_NE_lexicon_tags_178[] = { 3, 5, 14, 4 };
static const label SUC_NE_lexicon_tags_179[] = { 4, 10, 11, 17, 18 };
static const label SUC_NE_lexicon_tags_180[] = { 2, 11, 7 };
static const label SUC_NE_lexicon_tags_181[] = { 4, 17, 18, 1, 13 };
static const label SUC_NE_lexicon_tags_182[] = { 4, 18, 2, 3, 8 };
static const label SUC_NE_lexicon_tags_183[] = { 6, 6, 2, 18, 1, 8, 13 };
static const label SUC_NE_lexicon_tags_184[] = { 3, 11, 5, 14 };
static const label SUC_NE_lexicon_tags_185[] = { 5, 17, 18, 2, 11, 13 };
static const label SUC_NE_lexicon_tags_186[] = { 3, 17, 18, 15 };
static const label SUC_NE_lexicon_tags_187[] = { 3, 11, 18, 14 };
static const label SUC_NE_lexicon_tags_188[] = { 2, 6, 15 };
static const label SUC_NE_lexicon_tags_189[] = { 7, 15, 16, 18, 8, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_190[] = { 4, 18, 5, 14, 4 };
static const label SUC_NE_lexicon_tags_191[] = { 4, 17, 15, 2, 13 };
static const label SUC_NE_lexicon_tags_192[] = { 2, 11, 13 };
static const label SUC_NE_lexicon_tags_193[] = { 3, 18, 2, 7 };
static const label SUC_NE_lexicon_tags_194[] = { 6, 6, 15, 5, 18, 14, 17 };
static const label SUC_NE_lexicon_tags_195[] = { 3, 15, 18, 13 };
static const label SUC_NE_lexicon_tags_196[] = { 3, 0, 3, 8 };
static const label SUC_NE_lexicon_tags_197[] = { 3, 15, 5, 13 };
static const label SUC_NE_lexicon_tags_198[] = { 2, 5, 4 };
static const label SUC_NE_lexicon_tags_199[] = { 6, 6, 2, 18, 1, 14, 8 };
static const label SUC_NE_lexicon_tags_200[] = { 4, 6, 11, 2, 4 };
static const label SUC_NE_lexicon_tags_201[] = { 2, 17, 11 };
static const label SUC_NE_lexicon_tags_202[] = { 5, 17, 18, 11, 13, 8 };
static const label SUC_NE_lexicon_tags_203[] = { 2, 17, 0 };
static const label SUC_NE_lexicon_tags_204[] = { 3, 16, 2, 7 };
static const label SUC_NE_lexicon_tags_205[] = { 4, 10, 6, 18, 2 };
static const label SUC_NE_lexicon_tags_206[] = { 2, 11, 8 };
static const label SUC_NE_lexicon_tags_207[] = { 3, 5, 14, 13 };
static const label SUC_NE_lexicon_tags_208[] = { 2, 1, 3 };
static const label SUC_NE_lexicon_tags_209[] = { 2, 6, 14 };
static const label SUC_NE_lexicon_tags_210[] = { 3, 2, 5, 14 };
static const label SUC_NE_lexicon_tags_211[] = { 6, 6, 18, 14, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_212[] = { 3, 2, 14, 8 };
static const label SUC_NE_lexicon_tags_213[] = { 5, 17, 18, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_214[] = { 2, 7, 5 };
static const label SUC_NE_lexicon_tags_215[] = { 2, 2, 8 };
static const label SUC_NE_lexicon_tags_216[] = { 3, 17, 15, 0 };
static const label SUC_NE_lexicon_tags_217[] = { 5, 6, 11, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_218[] = { 4, 8, 17, 18, 12 };
static const label SUC_NE_lexicon_tags_219[] = { 2, 15, 14 };
static const label SUC_NE_lexicon_tags_220[] = { 1, 10 };
static const label SUC_NE_lexicon_tags_221[] = { 6, 5, 18, 0, 14, 11, 9 };
static const label SUC_NE_lexicon_tags_222[] = { 3, 18, 7, 4 };
static const label SUC_NE_lexicon_tags_223[] = { 3, 11, 18, 4 };
static const label SUC_NE_lexicon_tags_224[] = { 5, 6, 18, 17, 4, 8 };
static const label SUC_NE_lexicon_tags_225[] = { 3, 11, 18, 6 };
static const label SUC_NE_lexicon_tags_226[] = { 2, 2, 9 };
static const label SUC_NE_lexicon_tags_227[] = { 2, 4, 14 };
static const label SUC_NE_lexicon_tags_228[] = { 5, 11, 18, 2, 15, 13 };
static const label SUC_NE_lexicon_tags_229[] = { 3, 17, 18, 4 };
static const label SUC_NE_lexicon_tags_230[] = { 3, 17, 18, 16 };
static const label SUC_NE_lexicon_tags_231[] = { 4, 6, 11, 15, 2 };
static const label SUC_NE_lexicon_tags_232[] = { 5, 18, 7, 5, 14, 4 };
static const label SUC_NE_lexicon_tags_233[] = { 2, 1, 5 };
static const label SUC_NE_lexicon_tags_234[] = { 2, 17, 15 };
static const label SUC_NE_lexicon_tags_235[] = { 3, 15, 5, 8 };
static const label SUC_NE_lexicon_tags_236[] = { 5, 17, 18, 2, 11, 8 };
static const label SUC_NE_lexicon_tags_237[] = { 3, 18, 16, 7 };
static const label SUC_NE_lexicon_tags_238[] = { 5, 18, 2, 5, 9, 14 };
static const label SUC_NE_lexicon_tags_239[] = { 4, 16, 15, 5, 14 };
static const label SUC_NE_lexicon_tags_240[] = { 4, 11, 18, 2, 4 };
static const label SUC_NE_lexicon_tags_241[] = { 7, 6, 2, 5, 18, 14, 7, 4 };
static const label SUC_NE_lexicon_tags_242[] = { 3, 18, 0, 14 };
static const label SUC_NE_lexicon_tags_243[] = { 5, 17, 18, 6, 5, 14 };
static const label SUC_NE_lexicon_tags_244[] = { 7, 16, 5, 18, 0, 14, 17, 7 };
static const label SUC_NE_lexicon_tags_245[] = { 4, 6, 16, 7, 2 };
static const label SUC_NE_lexicon_tags_246[] = { 4, 11, 18, 5, 13 };
static const label SUC_NE_lexicon_tags_247[] = { 2, 7, 14 };
static const label SUC_NE_lexicon_tags_248[] = { 2, 0, 9 };
static const label SUC_NE_lexicon_tags_249[] = { 4, 18, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_250[] = { 3, 0, 5, 14 };
static const label SUC_NE_lexicon_tags_251[] = { 4, 11, 18, 17, 8 };
static const label SUC_NE_lexicon_tags_252[] = { 2, 7, 4 };
static const label SUC_NE_lexicon_tags_253[] = { 5, 6, 1, 2, 11, 5 };
static const label SUC_NE_lexicon_tags_254[] = { 3, 11, 15, 2 };
static const label SUC_NE_lexicon_tags_255[] = { 3, 6, 18, 7 };
static const label SUC_NE_lexicon_tags_256[] = { 4, 17, 18, 15, 13 };
static const label SUC_NE_lexicon_tags_257[] = { 5, 17, 18, 1, 2, 4 };
static const label SUC_NE_lexicon_tags_258[] = { 4, 18, 2, 16, 7 };
static const label SUC_NE_lexicon_tags_259[] = { 5, 18, 2, 5, 14, 4 };
static const label SUC_NE_lexicon_tags_260[] = { 3, 16, 5, 14 };
static const label SUC_NE_lexicon_tags_261[] = { 4, 17, 18, 13, 8 };
static const label SUC_NE_lexicon_tags_262[] = { 7, 10, 2, 18, 8, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_263[] = { 3, 11, 6, 2 };
static const label SUC_NE_lexicon_tags_264[] = { 4, 11, 15, 18, 14 };
static const label SUC_NE_lexicon_tags_265[] = { 2, 10, 8 };
static const label SUC_NE_lexicon_tags_266[] = { 4, 17, 2, 5, 14 };
static const label SUC_NE_lexicon_tags_267[] = { 8, 6, 15, 5, 18, 1, 14, 8, 17 };
static const label SUC_NE_lexicon_tags_268[] = { 3, 18, 14, 4 };
static const label SUC_NE_lexicon_tags_269[] = { 4, 11, 1, 18, 8 };
static const label SUC_NE_lexicon_tags_270[] = { 2, 0, 5 };
static const label SUC_NE_lexicon_tags_271[] = { 4, 18, 15, 9, 14 };
static const label SUC_NE_lexicon_tags_272[] = { 5, 17, 18, 11, 9, 14 };
static const label SUC_NE_lexicon_tags_273[] = { 5, 11, 18, 17, 4, 8 };
static const label SUC_NE_lexicon_tags_274[] = { 2, 16, 0 };
static const label SUC_NE_lexicon_tags_275[] = { 5, 1, 18, 5, 16, 14 };
static const label SUC_NE_lexicon_tags_276[] = { 6, 6, 5, 18, 1, 11, 7 };
static const label SUC_NE_lexicon_tags_277[] = { 4, 17, 18, 2, 11 };
static const label SUC_NE_lexicon_tags_278[] = { 2, 6, 13 };
static const label SUC_NE_lexicon_tags_279[] = { 4, 11, 18, 6, 14 };
static const label SUC_NE_lexicon_tags_280[] = { 4, 11, 18, 2, 15 };
static const label SUC_NE_lexicon_tags_281[] = { 2, 11, 5 };
static const label SUC_NE_lexicon_tags_282[] = { 5, 11, 18, 2, 6, 4 };
static const label SUC_NE_lexicon_tags_283[] = { 4, 11, 18, 2, 1 };
static const label SUC_NE_lexicon_tags_284[] = { 4, 10, 17, 11, 2 };
static const label SUC_NE_lexicon_tags_285[] = { 3, 16, 7, 8 };
static const label SUC_NE_lexicon_tags_286[] = { 3, 2, 9, 5 };
static const label SUC_NE_lexicon_tags_287[] = { 2, 16, 14 };
static const label SUC_NE_lexicon_tags_288[] = { 3, 6, 18, 1 };
static const label SUC_NE_lexicon_tags_289[] = { 3, 5, 13, 14 };
static const label SUC_NE_lexicon_tags_290[] = { 3, 6, 11, 18 };
static const label SUC_NE_lexicon_tags_291[] = { 3, 18, 7, 2 };
static const label SUC_NE_lexicon_tags_292[] = { 4, 6, 18, 15, 13 };
static const label SUC_NE_lexicon_tags_293[] = { 3, 10, 18, 5 };
static const label SUC_NE_lexicon_tags_294[] = { 6, 2, 18, 14, 8, 17, 11 };
static const label SUC_NE_lexicon_tags_295[] = { 3, 11, 6, 14 };
static const label SUC_NE_lexicon_tags_296[] = { 6, 15, 18, 8, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_297[] = { 3, 5, 4, 14 };
static const label SUC_NE_lexicon_tags_298[] = { 4, 17, 18, 3, 8 };
static const label SUC_NE_lexicon_tags_299[] = { 4, 6, 15, 2, 4 };
static const label SUC_NE_lexicon_tags_300[] = { 5, 17, 18, 2, 4, 8 };
static const label SUC_NE_lexicon_tags_301[] = { 4, 10, 11, 18, 17 };
static const label SUC_NE_lexicon_tags_302[] = { 4, 11, 18, 2, 13 };
static const label SUC_NE_lexicon_tags_303[] = { 3, 16, 18, 7 };
static const label SUC_NE_lexicon_tags_304[] = { 2, 18, 12 };
static const label SUC_NE_lexicon_tags_305[] = { 3, 10, 1, 18 };
static const label SUC_NE_lexicon_tags_306[] = { 3, 16, 7, 5 };
static const label SUC_NE_lexicon_tags_307[] = { 2, 16, 2 };
static const label SUC_NE_lexicon_tags_308[] = { 2, 4, 8 };
static const label SUC_NE_lexicon_tags_309[] = { 4, 6, 17, 2, 8 };
static const label SUC_NE_lexicon_tags_310[] = { 4, 17, 11, 2, 5 };
static const label SUC_NE_lexicon_tags_311[] = { 6, 6, 2, 18, 8, 7, 4 };
static const label SUC_NE_lexicon_tags_312[] = { 4, 18, 1, 3, 8 };
static const label SUC_NE_lexicon_tags_313[] = { 2, 7, 8 };
static const label SUC_NE_lexicon_tags_314[] = { 6, 6, 15, 2, 8, 11, 4 };
static const label SUC_NE_lexicon_tags_315[] = { 6, 6, 15, 18, 14, 17, 4 };
static const label SUC_NE_lexicon_tags_316[] = { 4, 6, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_317[] = { 3, 18, 5, 4 };
static const label SUC_NE_lexicon_tags_318[] = { 5, 11, 18, 15, 16, 17 };
static const label SUC_NE_lexicon_tags_319[] = { 5, 18, 2, 5, 7, 8 };
static const label SUC_NE_lexicon_tags_320[] = { 3, 18, 0, 5 };
static const label SUC_NE_lexicon_tags_321[] = { 6, 6, 5, 18, 14, 8, 4 };
static const label SUC_NE_lexicon_tags_322[] = { 4, 18, 5, 14, 13 };
static const label SUC_NE_lexicon_tags_323[] = { 4, 6, 18, 13, 4 };
static const label SUC_NE_lexicon_tags_324[] = { 3, 6, 5, 14 };
static const label SUC_NE_lexicon_tags_325[] = { 4, 10, 17, 11, 18 };
static const label SUC_NE_lexicon_tags_326[] = { 2, 2, 0 };
static const label SUC_NE_lexicon_tags_327[] = { 4, 11, 18, 5, 8 };
static const label SUC_NE_lexicon_tags_328[] = { 4, 17, 18, 6, 8 };
static const label SUC_NE_lexicon_tags_329[] = { 3, 11, 17, 13 };
static const label SUC_NE_lexicon_tags_330[] = { 3, 6, 2, 14 };
static const label SUC_NE_lexicon_tags_331[] = { 3, 11, 2, 14 };
static const label SUC_NE_lexicon_tags_332[] = { 7, 2, 5, 18, 14, 8, 17, 7 };
static const label SUC_NE_lexicon_tags_333[] = { 3, 11, 17, 16 };
static const label SUC_NE_lexicon_tags_334[] = { 4, 17, 18, 4, 13 };
static const label SUC_NE_lexicon_tags_335[] = { 6, 6, 2, 5, 14, 8, 13 };
static const label SUC_NE_lexicon_tags_336[] = { 2, 3, 8 };
static const label SUC_NE_lexicon_tags_337[] = { 3, 17, 18, 13 };
static const label SUC_NE_lexicon_tags_338[] = { 2, 10, 4 };
static const label SUC_NE_lexicon_tags_339[] = { 4, 11, 6, 17, 2 };
static const label SUC_NE_lexicon_tags_340[] = { 9, 6, 2, 5, 18, 8, 17, 13, 11, 4 };
static const label SUC_NE_lexicon_tags_341[] = { 2, 6, 8 };
static const label SUC_NE_lexicon_tags_342[] = { 5, 11, 18, 2, 4, 13 };
static const label SUC_NE_lexicon_tags_343[] = { 4, 11, 16, 15, 17 };
static const label SUC_NE_lexicon_tags_344[] = { 3, 1, 2, 8 };
static const label SUC_NE_lexicon_tags_345[] = { 5, 17, 18, 15, 2, 14 };
static const label SUC_NE_lexicon_tags_346[] = { 3, 16, 18, 5 };
static const label SUC_NE_lexicon_tags_347[] = { 5, 17, 15, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_348[] = { 4, 6, 18, 4, 8 };
static const label SUC_NE_lexicon_tags_349[] = { 4, 6, 18, 5, 13 };
static const label SUC_NE_lexicon_tags_350[] = { 3, 18, 2, 16 };
static const label SUC_NE_lexicon_tags_351[] = { 4, 6, 11, 2, 18 };
static const label SUC_NE_lexicon_tags_352[] = { 2, 17, 7 };
static const label SUC_NE_lexicon_tags_353[] = { 2, 6, 9 };
static const label SUC_NE_lexicon_tags_354[] = { 6, 18, 17, 13, 11, 7, 4 };
static const label SUC_NE_lexicon_tags_355[] = { 5, 17, 15, 5, 14, 13 };
static const label SUC_NE_lexicon_tags_356[] = { 4, 17, 18, 5, 8 };
static const label SUC_NE_lexicon_tags_357[] = { 3, 17, 14, 8 };
static const label SUC_NE_lexicon_tags_358[] = { 2, 11, 9 };
static const label SUC_NE_lexicon_tags_359[] = { 8, 10, 2, 5, 18, 14, 8, 17, 13 };
static const label SUC_NE_lexicon_tags_360[] = { 3, 11, 17, 15 };
static const label SUC_NE_lexicon_tags_361[] = { 2, 17, 16 };
static const label SUC_NE_lexicon_tags_362[] = { 5, 11, 18, 2, 17, 15 };
static const label SUC_NE_lexicon_tags_363[] = { 3, 11, 2, 5 };
static const label SUC_NE_lexicon_tags_364[] = { 7, 6, 2, 18, 8, 17, 11, 4 };
static const label SUC_NE_lexicon_tags_365[] = { 3, 17, 2, 8 };
static const label SUC_NE_lexicon_tags_366[] = { 4, 11, 17, 2, 18 };
static const label SUC_NE_lexicon_tags_367[] = { 3, 6, 1, 18 };
static const label SUC_NE_lexicon_tags_368[] = { 4, 11, 18, 17, 13 };
static const label SUC_NE_lexicon_tags_369[] = { 3, 11, 17, 8 };
static const label SUC_NE_lexicon_tags_370[] = { 3, 18, 9, 5 };
static const label SUC_NE_lexicon_tags_371[] = { 3, 6, 18, 4 };
static const label SUC_NE_lexicon_tags_372[] = { 3, 18, 7, 8 };
static const label SUC_NE_lexicon_tags_373[] = { 3, 11, 18, 16 };
static const label SUC_NE_lexicon_tags_374[] = { 3, 2, 5, 4 };
static const label SUC_NE_lexicon_tags_375[] = { 2, 14, 8 };
static const label SUC_NE_lexicon_tags_376[] = { 5, 17, 18, 16, 5, 14 };
static const label SUC_NE_lexicon_tags_377[] = { 3, 17, 6, 3 };
static const label SUC_NE_lexicon_tags_378[] = { 5, 6, 9, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_379[] = { 3, 11, 2, 7 };
static const label SUC_NE_lexicon_tags_380[] = { 6, 5, 3, 18, 14, 12, 4 };
static const label SUC_NE_lexicon_tags_381[] = { 4, 6, 15, 2, 17 };
static const label SUC_NE_lexicon_tags_382[] = { 7, 10, 2, 5, 18, 14, 8, 17 };
static const label SUC_NE_lexicon_tags_383[] = { 3, 6, 2, 7 };
static const label SUC_NE_lexicon_tags_384[] = { 3, 2, 7, 8 };
static const label SUC_NE_lexicon_tags_385[] = { 3, 6, 15, 3 };
static const label SUC_NE_lexicon_tags_386[] = { 4, 11, 2, 7, 4 };
static const label SUC_NE_lexicon_tags_387[] = { 5, 17, 18, 6, 5, 2 };
static const label SUC_NE_lexicon_tags_388[] = { 2, 5, 13 };
static const label SUC_NE_lexicon_tags_389[] = { 4, 16, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_390[] = { 3, 17, 1, 2 };
static const label SUC_NE_lexicon_tags_391[] = { 5, 17, 18, 15, 9, 8 };
static const label SUC_NE_lexicon_tags_392[] = { 6, 5, 18, 14, 8, 17, 13 };
static const label SUC_NE_lexicon_tags_393[] = { 7, 2, 1, 18, 8, 17, 11, 4 };
static const label SUC_NE_lexicon_tags_394[] = { 5, 6, 18, 2, 17, 11 };
static const label SUC_NE_lexicon_tags_395[] = { 4, 10, 18, 7, 1 };
static const label SUC_NE_lexicon_tags_396[] = { 3, 11, 2, 4 };
static const label SUC_NE_lexicon_tags_397[] = { 3, 7, 9, 3 };
static const label SUC_NE_lexicon_tags_398[] = { 4, 11, 16, 18, 14 };
static const label SUC_NE_lexicon_tags_399[] = { 2, 4, 13 };
static const label SUC_NE_lexicon_tags_400[] = { 4, 6, 2, 5, 14 };
static const label SUC_NE_lexicon_tags_401[] = { 4, 11, 18, 15, 2 };
static const label SUC_NE_lexicon_tags_402[] = { 4, 18, 7, 2, 14 };
static const label SUC_NE_lexicon_tags_403[] = { 4, 11, 18, 17, 2 };
static const label SUC_NE_lexicon_tags_404[] = { 3, 6, 15, 4 };
static const label SUC_NE_lexicon_tags_405[] = { 3, 17, 0, 5 };
static const label SUC_NE_lexicon_tags_406[] = { 6, 2, 5, 18, 14, 8, 17 };
static const label SUC_NE_lexicon_tags_407[] = { 6, 6, 15, 18, 17, 13, 11 };
static const label SUC_NE_lexicon_tags_408[] = { 4, 6, 18, 2, 15 };
static const label SUC_NE_lexicon_tags_409[] = { 5, 17, 18, 11, 5, 8 };
static const label SUC_NE_lexicon_tags_410[] = { 7, 6, 2, 5, 18, 8, 17, 11 };
static const label SUC_NE_lexicon_tags_411[] = { 5, 17, 18, 15, 14, 13 };
static const label SUC_NE_lexicon_tags_412[] = { 3, 18, 1, 8 };
static const label SUC_NE_lexicon_tags_413[] = { 4, 10, 17, 18, 11 };
static const label SUC_NE_lexicon_tags_414[] = { 4, 6, 11, 2, 13 };
static const label SUC_NE_lexicon_tags_415[] = { 3, 18, 15, 14 };
static const label SUC_NE_lexicon_tags_416[] = { 2, 17, 13 };
static const label SUC_NE_lexicon_tags_417[] = { 3, 17, 18, 0 };
static const label SUC_NE_lexicon_tags_418[] = { 4, 11, 18, 0, 8 };
static const label SUC_NE_lexicon_tags_419[] = { 7, 10, 6, 18, 0, 8, 17, 11 };
static const label SUC_NE_lexicon_tags_420[] = { 4, 11, 2, 13, 8 };
static const label SUC_NE_lexicon_tags_421[] = { 4, 14, 5, 4, 8 };
static const label SUC_NE_lexicon_tags_422[] = { 4, 17, 18, 15, 16 };
static const label SUC_NE_lexicon_tags_423[] = { 4, 17, 18, 2, 6 };
static const label SUC_NE_lexicon_tags_424[] = { 5, 8, 1, 5, 3, 12 };
static const label SUC_NE_lexicon_tags_425[] = { 4, 10, 17, 11, 13 };
static const label SUC_NE_lexicon_tags_426[] = { 4, 11, 18, 17, 15 };
static const label SUC_NE_lexicon_tags_427[] = { 3, 10, 17, 18 };
static const label SUC_NE_lexicon_tags_428[] = { 6, 6, 15, 2, 18, 14, 11 };
static const label SUC_NE_lexicon_tags_429[] = { 3, 18, 7, 16 };
static const label SUC_NE_lexicon_tags_430[] = { 3, 5, 4, 8 };
static const label SUC_NE_lexicon_tags_431[] = { 5, 17, 18, 2, 15, 13 };
static const label SUC_NE_lexicon_tags_432[] = { 4, 6, 18, 2, 11 };
static const label SUC_NE_lexicon_tags_433[] = { 3, 5, 14, 12 };
static const label SUC_NE_lexicon_tags_434[] = { 3, 6, 11, 4 };
static const label SUC_NE_lexicon_tags_435[] = { 4, 17, 18, 15, 8 };
static const label SUC_NE_lexicon_tags_436[] = { 3, 11, 14, 8 };
static const label SUC_NE_lexicon_tags_437[] = { 4, 6, 11, 2, 1 };
static const label SUC_NE_lexicon_tags_438[] = { 4, 0, 5, 3, 4 };
static const label SUC_NE_lexicon_tags_439[] = { 5, 17, 16, 2, 7, 5 };
static const label SUC_NE_lexicon_tags_440[] = { 2, 10, 2 };
static const label SUC_NE_lexicon_tags_441[] = { 2, 15, 13 };
static const label SUC_NE_lexicon_tags_442[] = { 4, 17, 15, 5, 14 };
static const label SUC_NE_lexicon_tags_443[] = { 7, 6, 15, 2, 5, 1, 18, 4 };
static const label SUC_NE_lexicon_tags_444[] = { 3, 7, 2, 5 };
static const label SUC_NE_lexicon_tags_445[] = { 4, 18, 5, 3, 8 };
static const label SUC_NE_lexicon_tags_446[] = { 3, 17, 18, 12 };
static const label SUC_NE_lexicon_tags_447[] = { 4, 15, 18, 5, 14 };
static const label SUC_NE_lexicon_tags_448[] = { 5, 6, 18, 2, 17, 8 };
static const label SUC_NE_lexicon_tags_449[] = { 3, 18, 15, 13 };
static const label SUC_NE_lexicon_tags_450[] = { 5, 17, 18, 2, 11, 14 };
static const label SUC_NE_lexicon_tags_451[] = { 3, 6, 4, 8 };
static const label SUC_NE_lexicon_tags_452[] = { 5, 4, 18, 5, 14, 8 };
static const label SUC_NE_lexicon_tags_453[] = { 3, 2, 5, 8 };
static const label SUC_NE_lexicon_tags_454[] = { 5, 6, 15, 18, 5, 8 };
static const label SUC_NE_lexicon_tags_455[] = { 3, 11, 16, 18 };
static const label SUC_NE_lexicon_tags_open[] = { 19, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18 };
static const hash32_kv SUC_NE_lexicon[SUC_NE_lexicon_size] = {
    { 0, NULL },
    { 0, NULL },
    { 0xcf2e0002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91ac0004, SUC_NE_lexicon_tags_2 },
    { 0x13da0005, SUC_NE_lexicon_tags_2 },
    { 0xad40006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22960009, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa00011, SUC_NE_lexicon_tags_2 },
    { 0xe9a0011, SUC_NE_lexicon_tags_2 },
    { 0xef820012, SUC_NE_lexicon_tags_2 },
    { 0xefb20014, SUC_NE_lexicon_tags_2 },
    { 0xdadc0014, SUC_NE_lexicon_tags_2 },
    { 0x48380016, SUC_NE_lexicon_tags_2 },
    { 0x20f80017, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0x1a3a0019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e02001e, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a0021, SUC_NE_lexicon_tags_2 },
    { 0x55b80022, SUC_NE_lexicon_tags_2 },
    { 0xa3a00023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea300026, SUC_NE_lexicon_tags_7 },
    { 0x8bb20027, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdd20029, SUC_NE_lexicon_tags_2 },
    { 0xd7f4002a, SUC_NE_lexicon_tags_2 },
    { 0x4506002b, SUC_NE_lexicon_tags_2 },
    { 0x9b4002b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18002f, SUC_NE_lexicon_tags_2 },
    { 0xd2820030, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eac0032, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71e0035, SUC_NE_lexicon_tags_7 },
    { 0xc5440036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6820038, SUC_NE_lexicon_tags_2 },
    { 0x60380039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba003c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2260040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4880043, SUC_NE_lexicon_tags_2 },
    { 0x7dee0044, SUC_NE_lexicon_tags_2 },
    { 0x15f80045, SUC_NE_lexicon_tags_2 },
    { 0x99da0045, SUC_NE_lexicon_tags_2 },
    { 0xd98e0043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfde0049, SUC_NE_lexicon_tags_2 },
    { 0x53de004a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b68004c, SUC_NE_lexicon_tags_2 },
    { 0x9fbe004d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x318c0050, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2640055, SUC_NE_lexicon_tags_2 },
    { 0x4d600056, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bc005d, SUC_NE_lexicon_tags_2 },
    { 0x4cdc005d, SUC_NE_lexicon_tags_2 },
    { 0xe56a005f, SUC_NE_lexicon_tags_2 },
    { 0xb470005f, SUC_NE_lexicon_tags_4 },
    { 0x44060061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea40064, SUC_NE_lexicon_tags_7 },
    { 0x97b80065, SUC_NE_lexicon_tags_2 },
    { 0x23ca0064, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc020069, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32da006b, SUC_NE_lexicon_tags_2 },
    { 0x6264006b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c26006f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdab60071, SUC_NE_lexicon_tags_28 },
    { 0x84060072, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79260077, SUC_NE_lexicon_tags_2 },
    { 0x25760078, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5678007c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe802007e, SUC_NE_lexicon_tags_2 },
    { 0xb33c007f, SUC_NE_lexicon_tags_4 },
    { 0xf90a0080, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x94980083, SUC_NE_lexicon_tags_2 },
    { 0xfc1e0083, SUC_NE_lexicon_tags_2 },
    { 0x5c0c0085, SUC_NE_lexicon_tags_2 },
    { 0x3eb60086, SUC_NE_lexicon_tags_2 },
    { 0xa8e00085, SUC_NE_lexicon_tags_2 },
    { 0x3b680088, SUC_NE_lexicon_tags_2 },
    { 0x59360089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d82008b, SUC_NE_lexicon_tags_2 },
    { 0x374008c, SUC_NE_lexicon_tags_2 },
    { 0x4b1c008c, SUC_NE_lexicon_tags_2 },
    { 0xd9d4008e, SUC_NE_lexicon_tags_2 },
    { 0x94ee008b, SUC_NE_lexicon_tags_2 },
    { 0x98860090, SUC_NE_lexicon_tags_2 },
    { 0x60ac008d, SUC_NE_lexicon_tags_2 },
    { 0x660c0092, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42000096, SUC_NE_lexicon_tags_4 },
    { 0x9d320097, SUC_NE_lexicon_tags_2 },
    { 0x5a100098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfffe009a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe930009c, SUC_NE_lexicon_tags_2 },
    { 0x78b6009d, SUC_NE_lexicon_tags_2 },
    { 0x2258009e, SUC_NE_lexicon_tags_2 },
    { 0x7e5c009e, SUC_NE_lexicon_tags_0 },
    { 0xc1aa00a0, SUC_NE_lexicon_tags_2 },
    { 0x7fae00a0, SUC_NE_lexicon_tags_24 },
    { 0xa98400a2, SUC_NE_lexicon_tags_2 },
    { 0xd94c00a2, SUC_NE_lexicon_tags_2 },
    { 0x977a009d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cda00a6, SUC_NE_lexicon_tags_2 },
    { 0xf3d000a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab400ab, SUC_NE_lexicon_tags_2 },
    { 0x725000ac, SUC_NE_lexicon_tags_2 },
    { 0x417000ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffcc00af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42e00b2, SUC_NE_lexicon_tags_2 },
    { 0xb9a600b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x660600b6, SUC_NE_lexicon_tags_2 },
    { 0xabca00b7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xcfb000b9, SUC_NE_lexicon_tags_2 },
    { 0x566200ba, SUC_NE_lexicon_tags_2 },
    { 0xfd5c00bb, SUC_NE_lexicon_tags_2 },
    { 0x81ea00bb, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3a00c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495400c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x202800c7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x321c00ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753400ce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b200d1, SUC_NE_lexicon_tags_2 },
    { 0xb3ca00d2, SUC_NE_lexicon_tags_2 },
    { 0xde1000d3, SUC_NE_lexicon_tags_2 },
    { 0x603e00d3, SUC_NE_lexicon_tags_2 },
    { 0x2b0400d3, SUC_NE_lexicon_tags_40 },
    { 0xa4f800d6, SUC_NE_lexicon_tags_2 },
    { 0x25dc00d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3200d9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eec00dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2a00e5, SUC_NE_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59000ea, SUC_NE_lexicon_tags_2 },
    { 0x4aac00eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8e800ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e000f1, SUC_NE_lexicon_tags_3 },
    { 0xc77600f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd800f5, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x399000f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82400fd, SUC_NE_lexicon_tags_2 },
    { 0x9e5600fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5340101, SUC_NE_lexicon_tags_2 },
    { 0xa0101, SUC_NE_lexicon_tags_2 },
    { 0x72180102, SUC_NE_lexicon_tags_2 },
    { 0xc7e20104, SUC_NE_lexicon_tags_2 },
    { 0x441e0105, SUC_NE_lexicon_tags_2 },
    { 0x36560106, SUC_NE_lexicon_tags_2 },
    { 0xdf060106, SUC_NE_lexicon_tags_0 },
    { 0x910a0105, SUC_NE_lexicon_tags_7 },
    { 0xf8260102, SUC_NE_lexicon_tags_2 },
    { 0x9680010a, SUC_NE_lexicon_tags_2 },
    { 0x807e0102, SUC_NE_lexicon_tags_2 },
    { 0xde000108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fe010f, SUC_NE_lexicon_tags_2 },
    { 0x7214010f, SUC_NE_lexicon_tags_2 },
    { 0xfcf00111, SUC_NE_lexicon_tags_2 },
    { 0x8bb60111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75d60114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bce011c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02e0122, SUC_NE_lexicon_tags_2 },
    { 0x2c4e0122, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74640129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9158012c, SUC_NE_lexicon_tags_2 },
    { 0xb1ac012d, SUC_NE_lexicon_tags_101 },
    { 0xa582012e, SUC_NE_lexicon_tags_2 },
    { 0x658e012d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1ea0131, SUC_NE_lexicon_tags_2 },
    { 0x58960132, SUC_NE_lexicon_tags_4 },
    { 0xd6560133, SUC_NE_lexicon_tags_2 },
    { 0x68a0134, SUC_NE_lexicon_tags_2 },
    { 0x64960134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fb80137, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22de0139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78460140, SUC_NE_lexicon_tags_2 },
    { 0x9840141, SUC_NE_lexicon_tags_2 },
    { 0x883c0142, SUC_NE_lexicon_tags_2 },
    { 0xa9000141, SUC_NE_lexicon_tags_4 },
    { 0x87d40144, SUC_NE_lexicon_tags_2 },
    { 0x3d960145, SUC_NE_lexicon_tags_2 },
    { 0x7d2a0146, SUC_NE_lexicon_tags_2 },
    { 0xde100147, SUC_NE_lexicon_tags_2 },
    { 0x452e0147, SUC_NE_lexicon_tags_2 },
    { 0xc7dc0149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1026014b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562e015a, SUC_NE_lexicon_tags_2 },
    { 0x6696015b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf12c015d, SUC_NE_lexicon_tags_2 },
    { 0x95ae015e, SUC_NE_lexicon_tags_2 },
    { 0xb698015d, SUC_NE_lexicon_tags_2 },
    { 0x2b80015e, SUC_NE_lexicon_tags_2 },
    { 0x2912015e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d340169, SUC_NE_lexicon_tags_2 },
    { 0xff4e016a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xde98016d, SUC_NE_lexicon_tags_2 },
    { 0x59a4016d, SUC_NE_lexicon_tags_2 },
    { 0x1c14016f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d80173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cc0176, SUC_NE_lexicon_tags_13 },
    { 0x37f80177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8a0180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a020182, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x127e0184, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6a0189, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd07c018b, SUC_NE_lexicon_tags_2 },
    { 0x88ca018b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9ce018e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aa0192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b60198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd8019e, SUC_NE_lexicon_tags_2 },
    { 0x7814019f, SUC_NE_lexicon_tags_2 },
    { 0x3aac019f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5601a3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc74a01a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed001a8, SUC_NE_lexicon_tags_2 },
    { 0x126c01a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a01ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5e01b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a3801b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54601b8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69601bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14ea01bd, SUC_NE_lexicon_tags_2 },
    { 0xb0b201be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91201c2, SUC_NE_lexicon_tags_4 },
    { 0xc65e01c3, SUC_NE_lexicon_tags_4 },
    { 0x7d0401c4, SUC_NE_lexicon_tags_7 },
    { 0x545801c4, SUC_NE_lexicon_tags_2 },
    { 0x2fa01c6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x222601c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x589a01ca, SUC_NE_lexicon_tags_2 },
    { 0x528001cb, SUC_NE_lexicon_tags_2 },
    { 0xf29801cc, SUC_NE_lexicon_tags_2 },
    { 0xfeb001ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54b601cf, SUC_NE_lexicon_tags_2 },
    { 0x76d001d0, SUC_NE_lexicon_tags_10 },
    { 0x396801d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8201d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637e01d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3ee01db, SUC_NE_lexicon_tags_2 },
    { 0x1dec01db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527e01e0, SUC_NE_lexicon_tags_2 },
    { 0xd9f201e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e401e4, SUC_NE_lexicon_tags_3 },
    { 0x953e01e5, SUC_NE_lexicon_tags_2 },
    { 0x7a2c01e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d401eb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5601ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee601f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44801f4, SUC_NE_lexicon_tags_2 },
    { 0x2e0201f5, SUC_NE_lexicon_tags_0 },
    { 0xc33c01f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320e01fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2b401fe, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d800200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53f00202, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d580205, SUC_NE_lexicon_tags_2 },
    { 0x97780206, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb58020a, SUC_NE_lexicon_tags_4 },
    { 0x2a04020b, SUC_NE_lexicon_tags_2 },
    { 0xb392020a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf25e020e, SUC_NE_lexicon_tags_2 },
    { 0x9798020f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf60212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dba0214, SUC_NE_lexicon_tags_2 },
    { 0x53dc0214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x884a0217, SUC_NE_lexicon_tags_2 },
    { 0x290c0218, SUC_NE_lexicon_tags_2 },
    { 0x95760219, SUC_NE_lexicon_tags_2 },
    { 0xabf20218, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9502021e, SUC_NE_lexicon_tags_0 },
    { 0x2892021f, SUC_NE_lexicon_tags_7 },
    { 0x52da021f, SUC_NE_lexicon_tags_2 },
    { 0x8fee0221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54860226, SUC_NE_lexicon_tags_2 },
    { 0x4f4c0227, SUC_NE_lexicon_tags_2 },
    { 0x22040228, SUC_NE_lexicon_tags_2 },
    { 0xe9d00229, SUC_NE_lexicon_tags_2 },
    { 0x3fec0228, SUC_NE_lexicon_tags_2 },
    { 0x7160022a, SUC_NE_lexicon_tags_2 },
    { 0xfeaa022c, SUC_NE_lexicon_tags_2 },
    { 0xb0c2022b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x127a022f, SUC_NE_lexicon_tags_2 },
    { 0xa75a0230, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61800232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd00235, SUC_NE_lexicon_tags_2 },
    { 0xa5600235, SUC_NE_lexicon_tags_2 },
    { 0xc2620237, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x96d40239, SUC_NE_lexicon_tags_2 },
    { 0xb950023a, SUC_NE_lexicon_tags_2 },
    { 0x3640239, SUC_NE_lexicon_tags_2 },
    { 0xe0c20239, SUC_NE_lexicon_tags_2 },
    { 0x2e30023d, SUC_NE_lexicon_tags_2 },
    { 0xf1ce023c, SUC_NE_lexicon_tags_2 },
    { 0x28e6023f, SUC_NE_lexicon_tags_2 },
    { 0x52be0240, SUC_NE_lexicon_tags_2 },
    { 0xce880240, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91bc0243, SUC_NE_lexicon_tags_22 },
    { 0xa62a0244, SUC_NE_lexicon_tags_2 },
    { 0xc2f80244, SUC_NE_lexicon_tags_2 },
    { 0xcbc40246, SUC_NE_lexicon_tags_2 },
    { 0x674a0243, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d080249, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddc024c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0a024f, SUC_NE_lexicon_tags_2 },
    { 0x2e400250, SUC_NE_lexicon_tags_12 },
    { 0x93a00251, SUC_NE_lexicon_tags_2 },
    { 0x76920252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x687a0254, SUC_NE_lexicon_tags_2 },
    { 0xf8f00255, SUC_NE_lexicon_tags_2 },
    { 0x96580254, SUC_NE_lexicon_tags_2 },
    { 0xccac0256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebc025a, SUC_NE_lexicon_tags_2 },
    { 0x3f82025b, SUC_NE_lexicon_tags_2 },
    { 0xf2a4025c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a025f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5574026a, SUC_NE_lexicon_tags_2 },
    { 0x525c026b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a8026e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc0274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39120279, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x681c027b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09c027e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c060280, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x57e40282, SUC_NE_lexicon_tags_1 },
    { 0xc1c40283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfae0286, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d940288, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ce028b, SUC_NE_lexicon_tags_2 },
    { 0x4d24028c, SUC_NE_lexicon_tags_2 },
    { 0x8868028d, SUC_NE_lexicon_tags_2 },
    { 0xff7c028c, SUC_NE_lexicon_tags_2 },
    { 0xfd4a028f, SUC_NE_lexicon_tags_2 },
    { 0xb446028f, SUC_NE_lexicon_tags_2 },
    { 0x1c0e0291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a100296, SUC_NE_lexicon_tags_199 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf0029c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cc029f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0e02a1, SUC_NE_lexicon_tags_393 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db402a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9a02a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e02ad, SUC_NE_lexicon_tags_2 },
    { 0x1e8c02ae, SUC_NE_lexicon_tags_2 },
    { 0x597c02ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81bc02b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x470a02b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff402b7, SUC_NE_lexicon_tags_2 },
    { 0x9e8802b7, SUC_NE_lexicon_tags_2 },
    { 0x3aae02b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d202bc, SUC_NE_lexicon_tags_2 },
    { 0x23fe02bc, SUC_NE_lexicon_tags_2 },
    { 0x9b0a02bc, SUC_NE_lexicon_tags_2 },
    { 0x331c02be, SUC_NE_lexicon_tags_2 },
    { 0x42802c0, SUC_NE_lexicon_tags_2 },
    { 0x4e1e02c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef8202c3, SUC_NE_lexicon_tags_2 },
    { 0x62c002c4, SUC_NE_lexicon_tags_2 },
    { 0x9b1602c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e02cc, SUC_NE_lexicon_tags_2 },
    { 0x45b202cd, SUC_NE_lexicon_tags_4 },
    { 0x417a02cd, SUC_NE_lexicon_tags_2 },
    { 0x3db402cf, SUC_NE_lexicon_tags_30 },
    { 0xb6c002cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153602d7, SUC_NE_lexicon_tags_2 },
    { 0xdbd002d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da202de, SUC_NE_lexicon_tags_2 },
    { 0x975002df, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa7da02e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ede02e4, SUC_NE_lexicon_tags_2 },
    { 0xf22402e4, SUC_NE_lexicon_tags_2 },
    { 0x887802e5, SUC_NE_lexicon_tags_2 },
    { 0xfb4002e7, SUC_NE_lexicon_tags_2 },
    { 0x179402e8, SUC_NE_lexicon_tags_2 },
    { 0x11ce02e4, SUC_NE_lexicon_tags_0 },
    { 0x30b602ea, SUC_NE_lexicon_tags_3 },
    { 0x5d2602eb, SUC_NE_lexicon_tags_3 },
    { 0x346a02e9, SUC_NE_lexicon_tags_2 },
    { 0xce9602eb, SUC_NE_lexicon_tags_2 },
    { 0xc58002ed, SUC_NE_lexicon_tags_2 },
    { 0x157202ef, SUC_NE_lexicon_tags_59 },
    { 0xde3e02f0, SUC_NE_lexicon_tags_2 },
    { 0x69bc02f1, SUC_NE_lexicon_tags_2 },
    { 0x332602f2, SUC_NE_lexicon_tags_2 },
    { 0x1a1e02ef, SUC_NE_lexicon_tags_2 },
    { 0x221002f1, SUC_NE_lexicon_tags_2 },
    { 0x63a602ef, SUC_NE_lexicon_tags_2 },
    { 0x3f0e02f6, SUC_NE_lexicon_tags_2 },
    { 0x5c002f4, SUC_NE_lexicon_tags_2 },
    { 0xee8c02f5, SUC_NE_lexicon_tags_2 },
    { 0x51a02f6, SUC_NE_lexicon_tags_4 },
    { 0xf3de02f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb3602fc, SUC_NE_lexicon_tags_2 },
    { 0x75e402fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22a0300, SUC_NE_lexicon_tags_2 },
    { 0x5a0a0300, SUC_NE_lexicon_tags_2 },
    { 0x721c0300, SUC_NE_lexicon_tags_2 },
    { 0x63e80303, SUC_NE_lexicon_tags_2 },
    { 0x84420303, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1660308, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x420e030b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb870030d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5400030f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7cc0311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e20316, SUC_NE_lexicon_tags_2 },
    { 0x95700317, SUC_NE_lexicon_tags_2 },
    { 0x872c0316, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb87e031a, SUC_NE_lexicon_tags_2 },
    { 0x12c4031b, SUC_NE_lexicon_tags_2 },
    { 0x73cc031c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b320325, SUC_NE_lexicon_tags_2 },
    { 0x23180326, SUC_NE_lexicon_tags_2 },
    { 0xbc1a0327, SUC_NE_lexicon_tags_4 },
    { 0xa4160327, SUC_NE_lexicon_tags_2 },
    { 0x31c60329, SUC_NE_lexicon_tags_2 },
    { 0xaf860327, SUC_NE_lexicon_tags_2 },
    { 0x6ee032b, SUC_NE_lexicon_tags_2 },
    { 0xd07c032b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa780333, SUC_NE_lexicon_tags_2 },
    { 0xa70e0334, SUC_NE_lexicon_tags_7 },
    { 0xa80335, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa3e80337, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ee033a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe88033c, SUC_NE_lexicon_tags_2 },
    { 0x5e8033d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4474033f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca0346, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1c034b, SUC_NE_lexicon_tags_1 },
    { 0x8cde034c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb24034e, SUC_NE_lexicon_tags_2 },
    { 0xc46a034e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a0354, SUC_NE_lexicon_tags_0 },
    { 0x3aa0355, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1300359, SUC_NE_lexicon_tags_36 },
    { 0x55ba035a, SUC_NE_lexicon_tags_2 },
    { 0x8fca035b, SUC_NE_lexicon_tags_2 },
    { 0x8c60035c, SUC_NE_lexicon_tags_2 },
    { 0x1f70035c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf20366, SUC_NE_lexicon_tags_2 },
    { 0xa360366, SUC_NE_lexicon_tags_2 },
    { 0xecfa0366, SUC_NE_lexicon_tags_2 },
    { 0x60a80369, SUC_NE_lexicon_tags_2 },
    { 0x5fe60368, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75080371, SUC_NE_lexicon_tags_2 },
    { 0xe49c0372, SUC_NE_lexicon_tags_2 },
    { 0xbbce0372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81f20375, SUC_NE_lexicon_tags_2 },
    { 0xf7f40376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd200379, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd472037b, SUC_NE_lexicon_tags_2 },
    { 0xd346037c, SUC_NE_lexicon_tags_2 },
    { 0x9c58037d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd78037f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb00381, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd920385, SUC_NE_lexicon_tags_7 },
    { 0x63440386, SUC_NE_lexicon_tags_7 },
    { 0x470e0385, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0c038a, SUC_NE_lexicon_tags_2 },
    { 0xd9de038b, SUC_NE_lexicon_tags_2 },
    { 0x5826038c, SUC_NE_lexicon_tags_2 },
    { 0xad88038d, SUC_NE_lexicon_tags_2 },
    { 0x2404038d, SUC_NE_lexicon_tags_2 },
    { 0xea84038d, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2260392, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x873a0394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeda80396, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1250039a, SUC_NE_lexicon_tags_2 },
    { 0xd57c039b, SUC_NE_lexicon_tags_2 },
    { 0x60b6039c, SUC_NE_lexicon_tags_2 },
    { 0xc692039d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819403a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbfe03a3, SUC_NE_lexicon_tags_2 },
    { 0xcf7a03a4, SUC_NE_lexicon_tags_2 },
    { 0xdae803a5, SUC_NE_lexicon_tags_2 },
    { 0x3bba03a6, SUC_NE_lexicon_tags_2 },
    { 0x1bde03a6, SUC_NE_lexicon_tags_2 },
    { 0x216a03a5, SUC_NE_lexicon_tags_13 },
    { 0x5d1003a3, SUC_NE_lexicon_tags_2 },
    { 0xcd5603a3, SUC_NE_lexicon_tags_2 },
    { 0xceea03a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dca03ad, SUC_NE_lexicon_tags_2 },
    { 0x737403ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd49403b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d603b5, SUC_NE_lexicon_tags_2 },
    { 0xc9e603b5, SUC_NE_lexicon_tags_2 },
    { 0xba0603b7, SUC_NE_lexicon_tags_2 },
    { 0x503e03b8, SUC_NE_lexicon_tags_2 },
    { 0x2a803b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a003bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21403c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca803c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aa03ca, SUC_NE_lexicon_tags_2 },
    { 0x371c03cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48dc03cd, SUC_NE_lexicon_tags_2 },
    { 0x99d203ce, SUC_NE_lexicon_tags_7 },
    { 0x26e03ce, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1603de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67403e3, SUC_NE_lexicon_tags_2 },
    { 0xb07403e4, SUC_NE_lexicon_tags_2 },
    { 0x33c003e5, SUC_NE_lexicon_tags_2 },
    { 0x109203e6, SUC_NE_lexicon_tags_2 },
    { 0xf78e03e6, SUC_NE_lexicon_tags_2 },
    { 0xf4f403e7, SUC_NE_lexicon_tags_4 },
    { 0x60be03e9, SUC_NE_lexicon_tags_2 },
    { 0x2aba03ea, SUC_NE_lexicon_tags_2 },
    { 0x1d1803e9, SUC_NE_lexicon_tags_2 },
    { 0xf55403ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ebc03ee, SUC_NE_lexicon_tags_2 },
    { 0x254803ee, SUC_NE_lexicon_tags_2 },
    { 0xff5e03ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a203f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69003fb, SUC_NE_lexicon_tags_2 },
    { 0x300603fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cc03ff, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0xde300401, SUC_NE_lexicon_tags_2 },
    { 0xb3780402, SUC_NE_lexicon_tags_2 },
    { 0xfbf40401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f160407, SUC_NE_lexicon_tags_2 },
    { 0x1d840408, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7488040c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x460e040e, SUC_NE_lexicon_tags_2 },
    { 0xde72040f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6900417, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781e041e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x436a0420, SUC_NE_lexicon_tags_2 },
    { 0xce7e0420, SUC_NE_lexicon_tags_376 },
    { 0x231c0422, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd80425, SUC_NE_lexicon_tags_2 },
    { 0xed620425, SUC_NE_lexicon_tags_7 },
    { 0x5b440426, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fd40429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44b4042b, SUC_NE_lexicon_tags_7 },
    { 0xaf0c042c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ca042e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871a0432, SUC_NE_lexicon_tags_2 },
    { 0xa2a80432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752a0437, SUC_NE_lexicon_tags_2 },
    { 0x88d60437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb030043b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5560043d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb40441, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74a0446, SUC_NE_lexicon_tags_2 },
    { 0x791a0446, SUC_NE_lexicon_tags_2 },
    { 0x9de0448, SUC_NE_lexicon_tags_2 },
    { 0xe8b60446, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1816044b, SUC_NE_lexicon_tags_4 },
    { 0x59d2044b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc0458, SUC_NE_lexicon_tags_2 },
    { 0x6dd20458, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d8045c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3900463, SUC_NE_lexicon_tags_7 },
    { 0x25ec0464, SUC_NE_lexicon_tags_2 },
    { 0x53ba0463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd48046a, SUC_NE_lexicon_tags_2 },
    { 0xd66046b, SUC_NE_lexicon_tags_2 },
    { 0x60d6046b, SUC_NE_lexicon_tags_2 },
    { 0x3296046b, SUC_NE_lexicon_tags_4 },
    { 0x7b80046c, SUC_NE_lexicon_tags_2 },
    { 0xdb54046c, SUC_NE_lexicon_tags_2 },
    { 0x4e24046e, SUC_NE_lexicon_tags_2 },
    { 0x5f2e046b, SUC_NE_lexicon_tags_2 },
    { 0x5b7e0472, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5e60474, SUC_NE_lexicon_tags_2 },
    { 0x40dc0474, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8bc0477, SUC_NE_lexicon_tags_2 },
    { 0x77280477, SUC_NE_lexicon_tags_2 },
    { 0x324a0478, SUC_NE_lexicon_tags_2 },
    { 0xa992047a, SUC_NE_lexicon_tags_2 },
    { 0x628e047b, SUC_NE_lexicon_tags_2 },
    { 0x8de8047c, SUC_NE_lexicon_tags_2 },
    { 0x580e047d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb40480, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8c0483, SUC_NE_lexicon_tags_3 },
    { 0x881c0484, SUC_NE_lexicon_tags_2 },
    { 0x6600484, SUC_NE_lexicon_tags_3 },
    { 0xb51c0484, SUC_NE_lexicon_tags_174 },
    { 0, NULL },
    { 0xe87e0488, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b8048b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53e048d, SUC_NE_lexicon_tags_2 },
    { 0xcc84048e, SUC_NE_lexicon_tags_24 },
    { 0x917a048f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x995e0492, SUC_NE_lexicon_tags_2 },
    { 0x503e0493, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67040497, SUC_NE_lexicon_tags_4 },
    { 0x36c00497, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb40049a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aa049d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1804a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3204a3, SUC_NE_lexicon_tags_2 },
    { 0x337c04a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b9e04a6, SUC_NE_lexicon_tags_2 },
    { 0x9bfc04a7, SUC_NE_lexicon_tags_2 },
    { 0x395204a8, SUC_NE_lexicon_tags_2 },
    { 0x2cd004a9, SUC_NE_lexicon_tags_2 },
    { 0x7b1c04a9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc04b0, SUC_NE_lexicon_tags_2 },
    { 0x85bc04b1, SUC_NE_lexicon_tags_2 },
    { 0x946e04b1, SUC_NE_lexicon_tags_2 },
    { 0x3a804b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ba04b9, SUC_NE_lexicon_tags_7 },
    { 0xa17c04ba, SUC_NE_lexicon_tags_2 },
    { 0x5c8004b9, SUC_NE_lexicon_tags_4 },
    { 0x731c04bc, SUC_NE_lexicon_tags_2 },
    { 0x4b6c04bd, SUC_NE_lexicon_tags_2 },
    { 0x34cc04be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2fe04c0, SUC_NE_lexicon_tags_2 },
    { 0xd85c04c0, SUC_NE_lexicon_tags_2 },
    { 0x7efa04c2, SUC_NE_lexicon_tags_2 },
    { 0x421004c2, SUC_NE_lexicon_tags_2 },
    { 0xb48c04c4, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50a04cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda04cf, SUC_NE_lexicon_tags_2 },
    { 0x1e8604d0, SUC_NE_lexicon_tags_2 },
    { 0xc3c404d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdd404d3, SUC_NE_lexicon_tags_2 },
    { 0xb96c04d4, SUC_NE_lexicon_tags_2 },
    { 0xaa0a04d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1e04da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8c04dd, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894804e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6c04e5, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2004e8, SUC_NE_lexicon_tags_2 },
    { 0x976404e8, SUC_NE_lexicon_tags_2 },
    { 0xcf6604ea, SUC_NE_lexicon_tags_2 },
    { 0x319c04ea, SUC_NE_lexicon_tags_2 },
    { 0x87bc04e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510604f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e04f3, SUC_NE_lexicon_tags_34 },
    { 0x15ce04f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52c04f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9604fd, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e340500, SUC_NE_lexicon_tags_2 },
    { 0xec700501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb9c0503, SUC_NE_lexicon_tags_2 },
    { 0x220a0504, SUC_NE_lexicon_tags_2 },
    { 0x2a80505, SUC_NE_lexicon_tags_12 },
    { 0xb4460505, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f2050a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6f4050c, SUC_NE_lexicon_tags_1 },
    { 0xa25c050d, SUC_NE_lexicon_tags_2 },
    { 0xcca2050c, SUC_NE_lexicon_tags_2 },
    { 0x8d3a050d, SUC_NE_lexicon_tags_2 },
    { 0xfaa20510, SUC_NE_lexicon_tags_2 },
    { 0xef4c0510, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5340516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e3e0518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc790051a, SUC_NE_lexicon_tags_2 },
    { 0xab2e051a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9950051f, SUC_NE_lexicon_tags_2 },
    { 0xcc00051f, SUC_NE_lexicon_tags_3 },
    { 0x14440521, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b60528, SUC_NE_lexicon_tags_2 },
    { 0xf10a0529, SUC_NE_lexicon_tags_2 },
    { 0x27c60529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524c0530, SUC_NE_lexicon_tags_2 },
    { 0xd7ee0530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24140533, SUC_NE_lexicon_tags_4 },
    { 0xd9280533, SUC_NE_lexicon_tags_2 },
    { 0x57a40534, SUC_NE_lexicon_tags_2 },
    { 0x6f00536, SUC_NE_lexicon_tags_2 },
    { 0x8fd80536, SUC_NE_lexicon_tags_2 },
    { 0x180a0538, SUC_NE_lexicon_tags_2 },
    { 0xd6240534, SUC_NE_lexicon_tags_2 },
    { 0xc7ac053a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea50053c, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247a0540, SUC_NE_lexicon_tags_0 },
    { 0x5ad00541, SUC_NE_lexicon_tags_2 },
    { 0x21e80540, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a0545, SUC_NE_lexicon_tags_2 },
    { 0x11040546, SUC_NE_lexicon_tags_2 },
    { 0x7f280546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe78054e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556a0552, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1660555, SUC_NE_lexicon_tags_2 },
    { 0xe6cc0556, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f8055b, SUC_NE_lexicon_tags_2 },
    { 0xf758055b, SUC_NE_lexicon_tags_2 },
    { 0xe38a055d, SUC_NE_lexicon_tags_2 },
    { 0x3c96055e, SUC_NE_lexicon_tags_1 },
    { 0x89c055d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7dc0561, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c20564, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x24560566, SUC_NE_lexicon_tags_2 },
    { 0x6a1c0567, SUC_NE_lexicon_tags_2 },
    { 0x754e0568, SUC_NE_lexicon_tags_1 },
    { 0xd3300568, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc626056b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d4056f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29360573, SUC_NE_lexicon_tags_21 },
    { 0xbfd40574, SUC_NE_lexicon_tags_2 },
    { 0x81de0574, SUC_NE_lexicon_tags_2 },
    { 0x287c0573, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25840578, SUC_NE_lexicon_tags_2 },
    { 0x97f40579, SUC_NE_lexicon_tags_2 },
    { 0xfbf4057a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0a057e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d5c0580, SUC_NE_lexicon_tags_2 },
    { 0x1d020581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee40583, SUC_NE_lexicon_tags_2 },
    { 0x3fe00584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc140589, SUC_NE_lexicon_tags_2 },
    { 0x3d120589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4e058d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeee4058f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba600595, SUC_NE_lexicon_tags_17 },
    { 0x18b20595, SUC_NE_lexicon_tags_2 },
    { 0x7cc0595, SUC_NE_lexicon_tags_251 },
    { 0xef440597, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8e059b, SUC_NE_lexicon_tags_2 },
    { 0x5a6059b, SUC_NE_lexicon_tags_2 },
    { 0x9aac059d, SUC_NE_lexicon_tags_2 },
    { 0x8b98059c, SUC_NE_lexicon_tags_2 },
    { 0x907e059f, SUC_NE_lexicon_tags_2 },
    { 0x56dc059c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaa05a4, SUC_NE_lexicon_tags_2 },
    { 0x174c05a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cf205a7, SUC_NE_lexicon_tags_2 },
    { 0x352805a8, SUC_NE_lexicon_tags_2 },
    { 0xcd3405a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2805ac, SUC_NE_lexicon_tags_2 },
    { 0xb54605ad, SUC_NE_lexicon_tags_2 },
    { 0xfa1605ae, SUC_NE_lexicon_tags_2 },
    { 0x63b405ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98e005b1, SUC_NE_lexicon_tags_2 },
    { 0x212805b1, SUC_NE_lexicon_tags_2 },
    { 0x397e05b3, SUC_NE_lexicon_tags_2 },
    { 0xfcd405b2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9e05b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1d605b9, SUC_NE_lexicon_tags_2 },
    { 0x5bc05b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dc05bc, SUC_NE_lexicon_tags_2 },
    { 0x49f405bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd50405bf, SUC_NE_lexicon_tags_2 },
    { 0x4f5c05c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x838005c2, SUC_NE_lexicon_tags_2 },
    { 0x3f0c05c3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02405c8, SUC_NE_lexicon_tags_2 },
    { 0xee8e05c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0405cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x642c05ce, SUC_NE_lexicon_tags_2 },
    { 0x2ba805cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1e05d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xceac05d5, SUC_NE_lexicon_tags_2 },
    { 0x985005d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x116205d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8c05dc, SUC_NE_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7c05df, SUC_NE_lexicon_tags_7 },
    { 0x61a205df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef205e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdece05e6, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174c05ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a005ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa80405ee, SUC_NE_lexicon_tags_2 },
    { 0x45b605ef, SUC_NE_lexicon_tags_2 },
    { 0xf98205f0, SUC_NE_lexicon_tags_2 },
    { 0xeaf605f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11e05f8, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b9605fa, SUC_NE_lexicon_tags_2 },
    { 0xfd9605fb, SUC_NE_lexicon_tags_2 },
    { 0x326c05fa, SUC_NE_lexicon_tags_2 },
    { 0xfa2605fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58600600, SUC_NE_lexicon_tags_2 },
    { 0x94ee0601, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde500603, SUC_NE_lexicon_tags_2 },
    { 0xbfd20604, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc04060a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x585c060c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61c060f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fa0612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b5a0614, SUC_NE_lexicon_tags_2 },
    { 0x22b60615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1e0619, SUC_NE_lexicon_tags_2 },
    { 0xe136061a, SUC_NE_lexicon_tags_2 },
    { 0xf44061a, SUC_NE_lexicon_tags_2 },
    { 0xc83c061c, SUC_NE_lexicon_tags_2 },
    { 0xf260061c, SUC_NE_lexicon_tags_2 },
    { 0x6676061a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56e40620, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa40625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c80628, SUC_NE_lexicon_tags_4 },
    { 0x62140629, SUC_NE_lexicon_tags_2 },
    { 0xf5a6062a, SUC_NE_lexicon_tags_2 },
    { 0x5d520628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc23a062d, SUC_NE_lexicon_tags_2 },
    { 0xc94062e, SUC_NE_lexicon_tags_2 },
    { 0xf230062d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6040635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x933a0637, SUC_NE_lexicon_tags_2 },
    { 0x6c760637, SUC_NE_lexicon_tags_2 },
    { 0x804e0637, SUC_NE_lexicon_tags_30 },
    { 0x388a063a, SUC_NE_lexicon_tags_4 },
    { 0xa544063b, SUC_NE_lexicon_tags_2 },
    { 0xa7920639, SUC_NE_lexicon_tags_2 },
    { 0x17e80638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c940640, SUC_NE_lexicon_tags_2 },
    { 0x7b8c0641, SUC_NE_lexicon_tags_2 },
    { 0x8aa0642, SUC_NE_lexicon_tags_2 },
    { 0xe2540643, SUC_NE_lexicon_tags_2 },
    { 0x6fda0641, SUC_NE_lexicon_tags_2 },
    { 0x963e0645, SUC_NE_lexicon_tags_2 },
    { 0x14d40646, SUC_NE_lexicon_tags_2 },
    { 0xbdc0647, SUC_NE_lexicon_tags_2 },
    { 0x82380648, SUC_NE_lexicon_tags_2 },
    { 0x48cc0649, SUC_NE_lexicon_tags_2 },
    { 0x3c82064a, SUC_NE_lexicon_tags_2 },
    { 0x3838064b, SUC_NE_lexicon_tags_2 },
    { 0xfe7c0648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a0c064e, SUC_NE_lexicon_tags_4 },
    { 0x7a4a064e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7820651, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de0655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af20659, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf762065c, SUC_NE_lexicon_tags_2 },
    { 0xd72a065c, SUC_NE_lexicon_tags_2 },
    { 0x74e0065d, SUC_NE_lexicon_tags_2 },
    { 0x83ec065f, SUC_NE_lexicon_tags_2 },
    { 0x94ee0660, SUC_NE_lexicon_tags_2 },
    { 0xfb720661, SUC_NE_lexicon_tags_2 },
    { 0x5ad6065e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1760664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f80668, SUC_NE_lexicon_tags_2 },
    { 0xc71c0668, SUC_NE_lexicon_tags_2 },
    { 0x8592066a, SUC_NE_lexicon_tags_2 },
    { 0xc9ee066a, SUC_NE_lexicon_tags_2 },
    { 0xec86066c, SUC_NE_lexicon_tags_2 },
    { 0xc6fa066c, SUC_NE_lexicon_tags_2 },
    { 0xb6c8066c, SUC_NE_lexicon_tags_4 },
    { 0x32ca066d, SUC_NE_lexicon_tags_2 },
    { 0xd0e4066c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18740673, SUC_NE_lexicon_tags_2 },
    { 0xbd200674, SUC_NE_lexicon_tags_2 },
    { 0xafc20675, SUC_NE_lexicon_tags_2 },
    { 0x764e0673, SUC_NE_lexicon_tags_2 },
    { 0x19a20677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cdc0679, SUC_NE_lexicon_tags_2 },
    { 0xc98a0679, SUC_NE_lexicon_tags_2 },
    { 0x1f880679, SUC_NE_lexicon_tags_2 },
    { 0xb75a067c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0b0067e, SUC_NE_lexicon_tags_2 },
    { 0x4dcc067f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1a0682, SUC_NE_lexicon_tags_2 },
    { 0x26cc0682, SUC_NE_lexicon_tags_3 },
    { 0x10d20682, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf3e0686, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb3e0688, SUC_NE_lexicon_tags_2 },
    { 0xd9760689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde2068c, SUC_NE_lexicon_tags_2 },
    { 0x1c82068c, SUC_NE_lexicon_tags_4 },
    { 0x551e068c, SUC_NE_lexicon_tags_32 },
    { 0xfc6e068c, SUC_NE_lexicon_tags_2 },
    { 0xbc36068e, SUC_NE_lexicon_tags_4 },
    { 0xe2040691, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x27920693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d540696, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb380698, SUC_NE_lexicon_tags_2 },
    { 0xd50c0698, SUC_NE_lexicon_tags_2 },
    { 0xdfb0069a, SUC_NE_lexicon_tags_2 },
    { 0xca480698, SUC_NE_lexicon_tags_2 },
    { 0x91be069b, SUC_NE_lexicon_tags_2 },
    { 0xb83a069d, SUC_NE_lexicon_tags_2 },
    { 0xded0069e, SUC_NE_lexicon_tags_2 },
    { 0xb36c069f, SUC_NE_lexicon_tags_2 },
    { 0xaf32069d, SUC_NE_lexicon_tags_2 },
    { 0x6eec06a1, SUC_NE_lexicon_tags_2 },
    { 0x9fba06a1, SUC_NE_lexicon_tags_2 },
    { 0x736e06a3, SUC_NE_lexicon_tags_7 },
    { 0xe1e06a3, SUC_NE_lexicon_tags_2 },
    { 0x936e06a5, SUC_NE_lexicon_tags_2 },
    { 0x2e5a06a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c806a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbe06af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba4406b1, SUC_NE_lexicon_tags_2 },
    { 0x4be806b2, SUC_NE_lexicon_tags_2 },
    { 0xe9d206b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa06b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35aa06b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9fa06ba, SUC_NE_lexicon_tags_2 },
    { 0xa6ce06ba, SUC_NE_lexicon_tags_138 },
    { 0xbdd206ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e006c1, SUC_NE_lexicon_tags_2 },
    { 0x9afa06c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46406c6, SUC_NE_lexicon_tags_2 },
    { 0x637c06c6, SUC_NE_lexicon_tags_30 },
    { 0x4e9c06c8, SUC_NE_lexicon_tags_2 },
    { 0x8ce806c9, SUC_NE_lexicon_tags_2 },
    { 0x7a0a06c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01a06ce, SUC_NE_lexicon_tags_2 },
    { 0x829806ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645006d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a806d7, SUC_NE_lexicon_tags_2 },
    { 0x784206d8, SUC_NE_lexicon_tags_2 },
    { 0x536606d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7be06db, SUC_NE_lexicon_tags_7 },
    { 0x16da06dc, SUC_NE_lexicon_tags_2 },
    { 0x2b1e06dd, SUC_NE_lexicon_tags_2 },
    { 0xae7206de, SUC_NE_lexicon_tags_2 },
    { 0x9d6406df, SUC_NE_lexicon_tags_2 },
    { 0x179006de, SUC_NE_lexicon_tags_2 },
    { 0x8dc006e1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c606e4, SUC_NE_lexicon_tags_2 },
    { 0x55ec06e5, SUC_NE_lexicon_tags_1 },
    { 0x89a206e5, SUC_NE_lexicon_tags_2 },
    { 0xed5e06e5, SUC_NE_lexicon_tags_2 },
    { 0xcaa206e6, SUC_NE_lexicon_tags_2 },
    { 0x31a006e6, SUC_NE_lexicon_tags_2 },
    { 0x6f2806ea, SUC_NE_lexicon_tags_2 },
    { 0x1c406e7, SUC_NE_lexicon_tags_0 },
    { 0x3fbe06ec, SUC_NE_lexicon_tags_2 },
    { 0x7ab006ed, SUC_NE_lexicon_tags_2 },
    { 0xf71a06e7, SUC_NE_lexicon_tags_4 },
    { 0x242806ef, SUC_NE_lexicon_tags_2 },
    { 0xeff806f0, SUC_NE_lexicon_tags_2 },
    { 0x9a0606f1, SUC_NE_lexicon_tags_2 },
    { 0x2f5e06e9, SUC_NE_lexicon_tags_2 },
    { 0x83ea06f3, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xcda206f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e06fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x965a06fd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43e0700, SUC_NE_lexicon_tags_2 },
    { 0xc1c60701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a700703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aa0707, SUC_NE_lexicon_tags_2 },
    { 0x1c200707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc46070d, SUC_NE_lexicon_tags_2 },
    { 0xa314070e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7700715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94a071c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b40721, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e0a0723, SUC_NE_lexicon_tags_2 },
    { 0x63fe0723, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7260726, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34c072b, SUC_NE_lexicon_tags_2 },
    { 0x80e4072c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ff0072e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9140733, SUC_NE_lexicon_tags_2 },
    { 0x69320734, SUC_NE_lexicon_tags_2 },
    { 0x34b80734, SUC_NE_lexicon_tags_2 },
    { 0x95220736, SUC_NE_lexicon_tags_2 },
    { 0x8c9c0737, SUC_NE_lexicon_tags_2 },
    { 0x8ea60737, SUC_NE_lexicon_tags_2 },
    { 0xb0fe0738, SUC_NE_lexicon_tags_2 },
    { 0xba980734, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfe5a073c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c60742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d80748, SUC_NE_lexicon_tags_2 },
    { 0x19d20748, SUC_NE_lexicon_tags_2 },
    { 0x890a074a, SUC_NE_lexicon_tags_2 },
    { 0x1a76074b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd69a074d, SUC_NE_lexicon_tags_4 },
    { 0xbf8074e, SUC_NE_lexicon_tags_7 },
    { 0xe768074d, SUC_NE_lexicon_tags_7 },
    { 0x1818074d, SUC_NE_lexicon_tags_2 },
    { 0xb1e0751, SUC_NE_lexicon_tags_2 },
    { 0x9eea0751, SUC_NE_lexicon_tags_7 },
    { 0x57400753, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302e075b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a50075f, SUC_NE_lexicon_tags_2 },
    { 0x86120760, SUC_NE_lexicon_tags_2 },
    { 0xa5c40760, SUC_NE_lexicon_tags_2 },
    { 0x16ea0761, SUC_NE_lexicon_tags_2 },
    { 0xc7a80763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b40766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96380768, SUC_NE_lexicon_tags_2 },
    { 0x77d60769, SUC_NE_lexicon_tags_13 },
    { 0xfde6076a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9240773, SUC_NE_lexicon_tags_2 },
    { 0x2ee40773, SUC_NE_lexicon_tags_2 },
    { 0x406c0774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8460777, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c077c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81dc077e, SUC_NE_lexicon_tags_2 },
    { 0xfe58077e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e940782, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32320786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41740788, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9202078a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa194078d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bca0791, SUC_NE_lexicon_tags_2 },
    { 0x249c0792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30e80794, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf06e0796, SUC_NE_lexicon_tags_2 },
    { 0xaf300797, SUC_NE_lexicon_tags_2 },
    { 0x39d00798, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c079c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdff0079e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2c07a1, SUC_NE_lexicon_tags_2 },
    { 0xe91807a2, SUC_NE_lexicon_tags_3 },
    { 0x92fa07a3, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8c07a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x480a07a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e07b1, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc6407b3, SUC_NE_lexicon_tags_2 },
    { 0x4e5007b4, SUC_NE_lexicon_tags_2 },
    { 0xa57007b5, SUC_NE_lexicon_tags_2 },
    { 0xa05807b5, SUC_NE_lexicon_tags_2 },
    { 0xe9b207b7, SUC_NE_lexicon_tags_2 },
    { 0xc0e07b5, SUC_NE_lexicon_tags_2 },
    { 0xab5207b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1607bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490607c2, SUC_NE_lexicon_tags_2 },
    { 0x431e07c3, SUC_NE_lexicon_tags_2 },
    { 0x699407c4, SUC_NE_lexicon_tags_2 },
    { 0x221207c4, SUC_NE_lexicon_tags_2 },
    { 0xd8807c4, SUC_NE_lexicon_tags_4 },
    { 0xcd4e07c7, SUC_NE_lexicon_tags_2 },
    { 0x168207c8, SUC_NE_lexicon_tags_2 },
    { 0xa76207c8, SUC_NE_lexicon_tags_2 },
    { 0xf4e807ca, SUC_NE_lexicon_tags_2 },
    { 0xd3f807c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b1207cd, SUC_NE_lexicon_tags_2 },
    { 0xea3c07ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cc07d7, SUC_NE_lexicon_tags_7 },
    { 0x73fa07d8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c07db, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e607de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x599c07e0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a07e3, SUC_NE_lexicon_tags_2 },
    { 0xc57807e4, SUC_NE_lexicon_tags_4 },
    { 0x752607e4, SUC_NE_lexicon_tags_2 },
    { 0x6b3207e6, SUC_NE_lexicon_tags_34 },
    { 0x5b4a07e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4c07ea, SUC_NE_lexicon_tags_1 },
    { 0xe14607eb, SUC_NE_lexicon_tags_2 },
    { 0x304a07ec, SUC_NE_lexicon_tags_2 },
    { 0xfbd607ed, SUC_NE_lexicon_tags_2 },
    { 0xf4a007eb, SUC_NE_lexicon_tags_2 },
    { 0x7a4c07ea, SUC_NE_lexicon_tags_2 },
    { 0xd1de07f0, SUC_NE_lexicon_tags_2 },
    { 0x6de07f0, SUC_NE_lexicon_tags_2 },
    { 0xe5aa07f1, SUC_NE_lexicon_tags_2 },
    { 0x1b7407f3, SUC_NE_lexicon_tags_2 },
    { 0x18f807f3, SUC_NE_lexicon_tags_2 },
    { 0x718e07f2, SUC_NE_lexicon_tags_2 },
    { 0x4c1407f6, SUC_NE_lexicon_tags_2 },
    { 0x897007f7, SUC_NE_lexicon_tags_2 },
    { 0x149207f6, SUC_NE_lexicon_tags_3 },
    { 0x2c8c07f6, SUC_NE_lexicon_tags_2 },
    { 0xd6e07f7, SUC_NE_lexicon_tags_2 },
    { 0x44c07f6, SUC_NE_lexicon_tags_2 },
    { 0x6d9a07fc, SUC_NE_lexicon_tags_2 },
    { 0x8f1407fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b00803, SUC_NE_lexicon_tags_2 },
    { 0x6f740804, SUC_NE_lexicon_tags_2 },
    { 0xb0e80805, SUC_NE_lexicon_tags_2 },
    { 0x3c460805, SUC_NE_lexicon_tags_2 },
    { 0x8f000807, SUC_NE_lexicon_tags_2 },
    { 0xcd20807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc010080d, SUC_NE_lexicon_tags_2 },
    { 0xe5cc080e, SUC_NE_lexicon_tags_2 },
    { 0xab5a080e, SUC_NE_lexicon_tags_2 },
    { 0x4cc0080d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11740814, SUC_NE_lexicon_tags_2 },
    { 0xe0ba0815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4204081b, SUC_NE_lexicon_tags_18 },
    { 0xc9da081c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98220820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40040823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4680827, SUC_NE_lexicon_tags_2 },
    { 0x93b60828, SUC_NE_lexicon_tags_2 },
    { 0xc32c0829, SUC_NE_lexicon_tags_2 },
    { 0x3ad6082a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f0082c, SUC_NE_lexicon_tags_2 },
    { 0x4202082d, SUC_NE_lexicon_tags_2 },
    { 0xc4c082d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3020832, SUC_NE_lexicon_tags_4 },
    { 0x70680833, SUC_NE_lexicon_tags_2 },
    { 0x44660834, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c740837, SUC_NE_lexicon_tags_7 },
    { 0x61b80838, SUC_NE_lexicon_tags_2 },
    { 0xa0780838, SUC_NE_lexicon_tags_2 },
    { 0x67920837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb68083c, SUC_NE_lexicon_tags_2 },
    { 0xde0e083d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5254083f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x540841, SUC_NE_lexicon_tags_24 },
    { 0x923c0842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd520847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f400849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67c084d, SUC_NE_lexicon_tags_2 },
    { 0xaad8084d, SUC_NE_lexicon_tags_2 },
    { 0x5c8084f, SUC_NE_lexicon_tags_2 },
    { 0xb0340850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326a0854, SUC_NE_lexicon_tags_2 },
    { 0x3ad80855, SUC_NE_lexicon_tags_2 },
    { 0x83a80856, SUC_NE_lexicon_tags_2 },
    { 0xe1860855, SUC_NE_lexicon_tags_4 },
    { 0x6d60854, SUC_NE_lexicon_tags_2 },
    { 0xa7de0859, SUC_NE_lexicon_tags_2 },
    { 0xc880854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7020085c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ed6085e, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0xe8320860, SUC_NE_lexicon_tags_2 },
    { 0x81700861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a180863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c880866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a160868, SUC_NE_lexicon_tags_13 },
    { 0x12740868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b68086b, SUC_NE_lexicon_tags_2 },
    { 0xcce086c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68e086f, SUC_NE_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc60875, SUC_NE_lexicon_tags_2 },
    { 0xfcda0876, SUC_NE_lexicon_tags_2 },
    { 0x66ee0875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd560087f, SUC_NE_lexicon_tags_2 },
    { 0x75140880, SUC_NE_lexicon_tags_2 },
    { 0xc4ec0881, SUC_NE_lexicon_tags_2 },
    { 0x910a0882, SUC_NE_lexicon_tags_4 },
    { 0x3d1c0883, SUC_NE_lexicon_tags_3 },
    { 0x1308087f, SUC_NE_lexicon_tags_11 },
    { 0x40120880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3858088b, SUC_NE_lexicon_tags_2 },
    { 0xc9a4088b, SUC_NE_lexicon_tags_0 },
    { 0xdbb4088d, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x3d9a088f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2980891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62840895, SUC_NE_lexicon_tags_2 },
    { 0x75f80896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa316089a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x63f2089c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25a08a0, SUC_NE_lexicon_tags_2 },
    { 0xa07408a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248608a3, SUC_NE_lexicon_tags_2 },
    { 0x6c9208a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x199408a6, SUC_NE_lexicon_tags_2 },
    { 0x95f808a6, SUC_NE_lexicon_tags_2 },
    { 0x680808a7, SUC_NE_lexicon_tags_2 },
    { 0xf9b008a9, SUC_NE_lexicon_tags_2 },
    { 0xab1808aa, SUC_NE_lexicon_tags_3 },
    { 0x63e208ab, SUC_NE_lexicon_tags_3 },
    { 0x9a7a08ac, SUC_NE_lexicon_tags_2 },
    { 0xa55008ad, SUC_NE_lexicon_tags_2 },
    { 0xf0ec08a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d4e08b0, SUC_NE_lexicon_tags_2 },
    { 0x418408b0, SUC_NE_lexicon_tags_2 },
    { 0xad1608b0, SUC_NE_lexicon_tags_2 },
    { 0xa3b608b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x220a08b5, SUC_NE_lexicon_tags_2 },
    { 0x581808b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x492a08b8, SUC_NE_lexicon_tags_4 },
    { 0xe4f408b9, SUC_NE_lexicon_tags_273 },
    { 0, NULL },
    { 0xb4ae08bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x551c08be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1de08c0, SUC_NE_lexicon_tags_3 },
    { 0x95d408c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b4208c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4808c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530a08cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1808d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436a08d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6808dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369e08e0, SUC_NE_lexicon_tags_2 },
    { 0x3cda08e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7ca08e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c208e7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21808ea, SUC_NE_lexicon_tags_2 },
    { 0x28bc08eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2efa08ed, SUC_NE_lexicon_tags_2 },
    { 0x389008ed, SUC_NE_lexicon_tags_2 },
    { 0xed2408ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528a08f4, SUC_NE_lexicon_tags_2 },
    { 0x6c6a08f5, SUC_NE_lexicon_tags_24 },
    { 0x421a08f5, SUC_NE_lexicon_tags_2 },
    { 0x7bc008f7, SUC_NE_lexicon_tags_2 },
    { 0xa15208f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f608fb, SUC_NE_lexicon_tags_2 },
    { 0xf85208fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0008ff, SUC_NE_lexicon_tags_2 },
    { 0x34ce08ff, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b60905, SUC_NE_lexicon_tags_2 },
    { 0xf0e00906, SUC_NE_lexicon_tags_2 },
    { 0x23760906, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65b20909, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8090c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfd2090e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73480911, SUC_NE_lexicon_tags_138 },
    { 0x80e00912, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x653c0914, SUC_NE_lexicon_tags_2 },
    { 0x10f60915, SUC_NE_lexicon_tags_2 },
    { 0x66140916, SUC_NE_lexicon_tags_2 },
    { 0x9dc00917, SUC_NE_lexicon_tags_11 },
    { 0x4ab60918, SUC_NE_lexicon_tags_2 },
    { 0xa4d60919, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b98091c, SUC_NE_lexicon_tags_2 },
    { 0xb3a6091d, SUC_NE_lexicon_tags_2 },
    { 0x9f76091d, SUC_NE_lexicon_tags_2 },
    { 0xad02091f, SUC_NE_lexicon_tags_2 },
    { 0xb820920, SUC_NE_lexicon_tags_2 },
    { 0xc3460921, SUC_NE_lexicon_tags_2 },
    { 0x3aa00921, SUC_NE_lexicon_tags_2 },
    { 0x81020921, SUC_NE_lexicon_tags_2 },
    { 0x40520921, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f20929, SUC_NE_lexicon_tags_2 },
    { 0xf46e092a, SUC_NE_lexicon_tags_2 },
    { 0xdf2a092b, SUC_NE_lexicon_tags_2 },
    { 0x7316092c, SUC_NE_lexicon_tags_2 },
    { 0x32da092a, SUC_NE_lexicon_tags_2 },
    { 0xb960092e, SUC_NE_lexicon_tags_2 },
    { 0x617a092a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ec0933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8160936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be093b, SUC_NE_lexicon_tags_1 },
    { 0x5448093b, SUC_NE_lexicon_tags_2 },
    { 0xc752093d, SUC_NE_lexicon_tags_2 },
    { 0xff7a093e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f120941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e40946, SUC_NE_lexicon_tags_2 },
    { 0xaa9a0947, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd848094c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e460950, SUC_NE_lexicon_tags_1 },
    { 0x93e80951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddc0954, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b20958, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29be095d, SUC_NE_lexicon_tags_4 },
    { 0x51f0095d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaca0963, SUC_NE_lexicon_tags_2 },
    { 0xb1c00964, SUC_NE_lexicon_tags_2 },
    { 0xca020964, SUC_NE_lexicon_tags_2 },
    { 0x5e240966, SUC_NE_lexicon_tags_2 },
    { 0xa27c0965, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3714096c, SUC_NE_lexicon_tags_2 },
    { 0xc268096d, SUC_NE_lexicon_tags_4 },
    { 0xb238096c, SUC_NE_lexicon_tags_2 },
    { 0x6af8096f, SUC_NE_lexicon_tags_2 },
    { 0xfbe8096d, SUC_NE_lexicon_tags_2 },
    { 0x71b0096c, SUC_NE_lexicon_tags_2 },
    { 0xdaf00972, SUC_NE_lexicon_tags_2 },
    { 0xfcf00972, SUC_NE_lexicon_tags_2 },
    { 0x1af20974, SUC_NE_lexicon_tags_2 },
    { 0x79400973, SUC_NE_lexicon_tags_2 },
    { 0xde560976, SUC_NE_lexicon_tags_2 },
    { 0xad520976, SUC_NE_lexicon_tags_2 },
    { 0xb5b40978, SUC_NE_lexicon_tags_2 },
    { 0xb1d40977, SUC_NE_lexicon_tags_2 },
    { 0x7520976, SUC_NE_lexicon_tags_2 },
    { 0xb04c097b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x449e097d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421a0981, SUC_NE_lexicon_tags_2 },
    { 0x76cc0982, SUC_NE_lexicon_tags_40 },
    { 0x36a60983, SUC_NE_lexicon_tags_2 },
    { 0x56fe0982, SUC_NE_lexicon_tags_2 },
    { 0xa7ae0985, SUC_NE_lexicon_tags_2 },
    { 0xfa080984, SUC_NE_lexicon_tags_2 },
    { 0x84e20987, SUC_NE_lexicon_tags_2 },
    { 0x74300988, SUC_NE_lexicon_tags_2 },
    { 0x23e80989, SUC_NE_lexicon_tags_2 },
    { 0x14ac0988, SUC_NE_lexicon_tags_2 },
    { 0xbc640983, SUC_NE_lexicon_tags_2 },
    { 0x2c4c0987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11840991, SUC_NE_lexicon_tags_2 },
    { 0x252a0991, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abe0995, SUC_NE_lexicon_tags_7 },
    { 0xbcc80996, SUC_NE_lexicon_tags_2 },
    { 0x8a160997, SUC_NE_lexicon_tags_2 },
    { 0x7fb80998, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded8099d, SUC_NE_lexicon_tags_2 },
    { 0xa946099e, SUC_NE_lexicon_tags_2 },
    { 0xde18099e, SUC_NE_lexicon_tags_2 },
    { 0x2fa009a0, SUC_NE_lexicon_tags_39 },
    { 0xc5a209a0, SUC_NE_lexicon_tags_2 },
    { 0xeb8009a2, SUC_NE_lexicon_tags_2 },
    { 0x62ce09a3, SUC_NE_lexicon_tags_2 },
    { 0x248209a3, SUC_NE_lexicon_tags_30 },
    { 0xcb3609a2, SUC_NE_lexicon_tags_2 },
    { 0x808009a2, SUC_NE_lexicon_tags_2 },
    { 0x862809a7, SUC_NE_lexicon_tags_102 },
    { 0xc3ac09a7, SUC_NE_lexicon_tags_2 },
    { 0x8a6609a5, SUC_NE_lexicon_tags_2 },
    { 0x1f409a2, SUC_NE_lexicon_tags_432 },
    { 0x706009a1, SUC_NE_lexicon_tags_2 },
    { 0x8b0609ac, SUC_NE_lexicon_tags_2 },
    { 0xbd4209ac, SUC_NE_lexicon_tags_2 },
    { 0xfab209ae, SUC_NE_lexicon_tags_2 },
    { 0x974409ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2d609b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cac09b7, SUC_NE_lexicon_tags_2 },
    { 0x356c09b7, SUC_NE_lexicon_tags_2 },
    { 0x295409b9, SUC_NE_lexicon_tags_4 },
    { 0x1f1a09ba, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x753809bc, SUC_NE_lexicon_tags_2 },
    { 0x4fae09bd, SUC_NE_lexicon_tags_2 },
    { 0x324609bd, SUC_NE_lexicon_tags_2 },
    { 0x5ea009bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x941809c2, SUC_NE_lexicon_tags_2 },
    { 0xc79809c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb6e09c5, SUC_NE_lexicon_tags_2 },
    { 0x1aa809c6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fc09ca, SUC_NE_lexicon_tags_2 },
    { 0xc21009ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc009cf, SUC_NE_lexicon_tags_2 },
    { 0x364409d0, SUC_NE_lexicon_tags_2 },
    { 0x64609d1, SUC_NE_lexicon_tags_2 },
    { 0x39f209cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6009d6, SUC_NE_lexicon_tags_2 },
    { 0xad9a09d7, SUC_NE_lexicon_tags_2 },
    { 0xdada09d8, SUC_NE_lexicon_tags_2 },
    { 0x374209d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ec09dc, SUC_NE_lexicon_tags_2 },
    { 0xe87009dc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143009e1, SUC_NE_lexicon_tags_2 },
    { 0x16b209e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285c09e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e809eb, SUC_NE_lexicon_tags_2 },
    { 0xb8f609ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7e09f1, SUC_NE_lexicon_tags_4 },
    { 0x2ff809f2, SUC_NE_lexicon_tags_2 },
    { 0x909609f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0209f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634609fe, SUC_NE_lexicon_tags_2 },
    { 0xcf6c09ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac00a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce00a09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8340a0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed140a0f, SUC_NE_lexicon_tags_2 },
    { 0x87480a10, SUC_NE_lexicon_tags_2 },
    { 0x95a60a0f, SUC_NE_lexicon_tags_2 },
    { 0xc9a00a0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x461a0a14, SUC_NE_lexicon_tags_2 },
    { 0xe3600a15, SUC_NE_lexicon_tags_3 },
    { 0x72d80a15, SUC_NE_lexicon_tags_2 },
    { 0x51aa0a17, SUC_NE_lexicon_tags_2 },
    { 0xa8bc0a17, SUC_NE_lexicon_tags_2 },
    { 0x79720a19, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f20a1c, SUC_NE_lexicon_tags_2 },
    { 0xfad60a1d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd20a21, SUC_NE_lexicon_tags_2 },
    { 0xf9480a22, SUC_NE_lexicon_tags_2 },
    { 0xda00a23, SUC_NE_lexicon_tags_2 },
    { 0xb85c0a24, SUC_NE_lexicon_tags_2 },
    { 0x9a1e0a25, SUC_NE_lexicon_tags_2 },
    { 0xace00a22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bc0a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4940a2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88a0a30, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b660a34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5e0a36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7840a38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2180a3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c000a3f, SUC_NE_lexicon_tags_2 },
    { 0x81220a3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17940a42, SUC_NE_lexicon_tags_2 },
    { 0xd50e0a43, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x680c0a46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74e0a49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9640a4b, SUC_NE_lexicon_tags_2 },
    { 0xc0880a4c, SUC_NE_lexicon_tags_3 },
    { 0xde2e0a4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabde0a51, SUC_NE_lexicon_tags_2 },
    { 0x67e20a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x515e0a54, SUC_NE_lexicon_tags_2 },
    { 0x816a0a55, SUC_NE_lexicon_tags_2 },
    { 0x4040a54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9380a58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cfa0a5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba40a5c, SUC_NE_lexicon_tags_2 },
    { 0xf8920a5d, SUC_NE_lexicon_tags_2 },
    { 0xdba40a5e, SUC_NE_lexicon_tags_2 },
    { 0xcfe0a5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8a00a61, SUC_NE_lexicon_tags_2 },
    { 0xc9a0a62, SUC_NE_lexicon_tags_2 },
    { 0x177a0a63, SUC_NE_lexicon_tags_2 },
    { 0xc3f80a62, SUC_NE_lexicon_tags_2 },
    { 0x7a020a65, SUC_NE_lexicon_tags_2 },
    { 0x98300a64, SUC_NE_lexicon_tags_2 },
    { 0xb1700a63, SUC_NE_lexicon_tags_2 },
    { 0xf340a68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7420a6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1260a6d, SUC_NE_lexicon_tags_2 },
    { 0xe6620a6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e80a76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ff40a78, SUC_NE_lexicon_tags_0 },
    { 0xac640a79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ee0a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b820a84, SUC_NE_lexicon_tags_2 },
    { 0xdec60a85, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf760a89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ed60a8b, SUC_NE_lexicon_tags_2 },
    { 0x34260a8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc600a8e, SUC_NE_lexicon_tags_2 },
    { 0x945a0a8f, SUC_NE_lexicon_tags_2 },
    { 0xdc800a90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef100a92, SUC_NE_lexicon_tags_2 },
    { 0xd2620a92, SUC_NE_lexicon_tags_2 },
    { 0xac5a0a93, SUC_NE_lexicon_tags_2 },
    { 0x8bc20a95, SUC_NE_lexicon_tags_24 },
    { 0x54720a92, SUC_NE_lexicon_tags_7 },
    { 0x4a420a97, SUC_NE_lexicon_tags_2 },
    { 0xb2160a98, SUC_NE_lexicon_tags_1 },
    { 0xefb40a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade0a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33c0aa1, SUC_NE_lexicon_tags_2 },
    { 0xc2ee0aa2, SUC_NE_lexicon_tags_5 },
    { 0x8c3e0aa2, SUC_NE_lexicon_tags_2 },
    { 0x5e560aa2, SUC_NE_lexicon_tags_2 },
    { 0x62c60aa5, SUC_NE_lexicon_tags_2 },
    { 0x77780aa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee0aa9, SUC_NE_lexicon_tags_2 },
    { 0xa33a0aa9, SUC_NE_lexicon_tags_4 },
    { 0x28a00aab, SUC_NE_lexicon_tags_2 },
    { 0x40320aac, SUC_NE_lexicon_tags_2 },
    { 0xac640aac, SUC_NE_lexicon_tags_2 },
    { 0x1e680aad, SUC_NE_lexicon_tags_2 },
    { 0xa7500aae, SUC_NE_lexicon_tags_2 },
    { 0xe6040ab0, SUC_NE_lexicon_tags_2 },
    { 0x4f9c0aaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d480ab4, SUC_NE_lexicon_tags_2 },
    { 0x71c00ab5, SUC_NE_lexicon_tags_2 },
    { 0x92d00ab6, SUC_NE_lexicon_tags_2 },
    { 0x14d20ab7, SUC_NE_lexicon_tags_2 },
    { 0xea2c0ab5, SUC_NE_lexicon_tags_2 },
    { 0x83e20ab8, SUC_NE_lexicon_tags_2 },
    { 0x4f2a0ab6, SUC_NE_lexicon_tags_2 },
    { 0xbaec0ab8, SUC_NE_lexicon_tags_2 },
    { 0xd4140abc, SUC_NE_lexicon_tags_2 },
    { 0x35f80ab9, SUC_NE_lexicon_tags_1 },
    { 0xb3dc0abb, SUC_NE_lexicon_tags_2 },
    { 0x1b040abf, SUC_NE_lexicon_tags_2 },
    { 0x3b400ac0, SUC_NE_lexicon_tags_2 },
    { 0x91780ab9, SUC_NE_lexicon_tags_2 },
    { 0x9a80ab4, SUC_NE_lexicon_tags_2 },
    { 0x1f660abe, SUC_NE_lexicon_tags_2 },
    { 0x1cb20ac4, SUC_NE_lexicon_tags_2 },
    { 0x80c20ac4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9a0ac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b40ad6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca20adb, SUC_NE_lexicon_tags_2 },
    { 0x47260adb, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xe00c0ade, SUC_NE_lexicon_tags_2 },
    { 0x5a560adf, SUC_NE_lexicon_tags_2 },
    { 0x497e0adf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e100ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72c0ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f240ae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a200ae9, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0c0aef, SUC_NE_lexicon_tags_1 },
    { 0x7dfc0aef, SUC_NE_lexicon_tags_2 },
    { 0xb0fe0af1, SUC_NE_lexicon_tags_2 },
    { 0x8ad60af1, SUC_NE_lexicon_tags_4 },
    { 0x9ff20af3, SUC_NE_lexicon_tags_1 },
    { 0x82520af1, SUC_NE_lexicon_tags_2 },
    { 0xeece0af4, SUC_NE_lexicon_tags_11 },
    { 0x408a0af1, SUC_NE_lexicon_tags_2 },
    { 0x9500af1, SUC_NE_lexicon_tags_2 },
    { 0x2ae20af8, SUC_NE_lexicon_tags_2 },
    { 0x9ebe0af4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab720afb, SUC_NE_lexicon_tags_2 },
    { 0xe05a0afb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84700b00, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb100b02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20b07, SUC_NE_lexicon_tags_2 },
    { 0x4cf00b08, SUC_NE_lexicon_tags_2 },
    { 0x5b280b07, SUC_NE_lexicon_tags_2 },
    { 0xbd180b0a, SUC_NE_lexicon_tags_2 },
    { 0x716a0b09, SUC_NE_lexicon_tags_115 },
    { 0, NULL },
    { 0xc76a0b0d, SUC_NE_lexicon_tags_3 },
    { 0x69860b0e, SUC_NE_lexicon_tags_2 },
    { 0x4c660b0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaea0b12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54dc0b14, SUC_NE_lexicon_tags_2 },
    { 0xc4ac0b15, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60180b18, SUC_NE_lexicon_tags_2 },
    { 0x6ee80b18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef80b1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba060b1e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xced40b20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c6e0b22, SUC_NE_lexicon_tags_2 },
    { 0xe0b20b23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d960b25, SUC_NE_lexicon_tags_7 },
    { 0xffae0b26, SUC_NE_lexicon_tags_2 },
    { 0x98340b27, SUC_NE_lexicon_tags_276 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95320b30, SUC_NE_lexicon_tags_2 },
    { 0x8e800b31, SUC_NE_lexicon_tags_2 },
    { 0x2cdc0b31, SUC_NE_lexicon_tags_7 },
    { 0xacc80b33, SUC_NE_lexicon_tags_4 },
    { 0xf6ea0b34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7780b36, SUC_NE_lexicon_tags_2 },
    { 0x9b9e0b37, SUC_NE_lexicon_tags_2 },
    { 0xf85e0b38, SUC_NE_lexicon_tags_2 },
    { 0x75320b36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709a0b3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfc0b45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae900b4a, SUC_NE_lexicon_tags_2 },
    { 0x656c0b4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf00b4e, SUC_NE_lexicon_tags_2 },
    { 0xff940b4f, SUC_NE_lexicon_tags_2 },
    { 0x9a800b4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d8e0b52, SUC_NE_lexicon_tags_3 },
    { 0xbcc40b53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf00b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d60b59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a0b5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a680b60, SUC_NE_lexicon_tags_2 },
    { 0xe12e0b61, SUC_NE_lexicon_tags_2 },
    { 0x213e0b61, SUC_NE_lexicon_tags_2 },
    { 0x79e00b63, SUC_NE_lexicon_tags_2 },
    { 0x6940b64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d80b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x524a0b6a, SUC_NE_lexicon_tags_2 },
    { 0xa7700b6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea80b72, SUC_NE_lexicon_tags_2 },
    { 0x26380b72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb9e0b75, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd20b78, SUC_NE_lexicon_tags_2 },
    { 0xaf320b79, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x680a0b7b, SUC_NE_lexicon_tags_2 },
    { 0x323e0b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0160b81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3e40b83, SUC_NE_lexicon_tags_2 },
    { 0x93760b84, SUC_NE_lexicon_tags_40 },
    { 0xafc00b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea40b8b, SUC_NE_lexicon_tags_2 },
    { 0x7960b8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd340b8f, SUC_NE_lexicon_tags_2 },
    { 0x9d4c0b90, SUC_NE_lexicon_tags_2 },
    { 0x90260b90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c140b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18dc0b95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e60b98, SUC_NE_lexicon_tags_2 },
    { 0x5e2c0b98, SUC_NE_lexicon_tags_2 },
    { 0xbdc20b9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd600b9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10400b9e, SUC_NE_lexicon_tags_2 },
    { 0xe46e0b9f, SUC_NE_lexicon_tags_2 },
    { 0x957c0ba0, SUC_NE_lexicon_tags_2 },
    { 0xe7220ba1, SUC_NE_lexicon_tags_2 },
    { 0x4d3a0ba2, SUC_NE_lexicon_tags_2 },
    { 0x97ba0ba3, SUC_NE_lexicon_tags_2 },
    { 0x2d360ba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfde0ba8, SUC_NE_lexicon_tags_2 },
    { 0x72fa0ba9, SUC_NE_lexicon_tags_2 },
    { 0x25480ba8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa000bac, SUC_NE_lexicon_tags_0 },
    { 0x9c20bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b7a0baf, SUC_NE_lexicon_tags_2 },
    { 0xbd0c0bb0, SUC_NE_lexicon_tags_2 },
    { 0x36c0bb0, SUC_NE_lexicon_tags_2 },
    { 0x20f40bb2, SUC_NE_lexicon_tags_2 },
    { 0x5cdc0bb3, SUC_NE_lexicon_tags_2 },
    { 0x68ec0baf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba0bb7, SUC_NE_lexicon_tags_2 },
    { 0xe9cc0bb8, SUC_NE_lexicon_tags_15 },
    { 0x8a380bb7, SUC_NE_lexicon_tags_2 },
    { 0x48c80bba, SUC_NE_lexicon_tags_1 },
    { 0x28260bb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c20bc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434e0bcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90900bcd, SUC_NE_lexicon_tags_2 },
    { 0x5600bce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997e0bd4, SUC_NE_lexicon_tags_1 },
    { 0x52340bd4, SUC_NE_lexicon_tags_2 },
    { 0xd1360bd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131c0bdc, SUC_NE_lexicon_tags_4 },
    { 0x160c0bdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5a20bdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d60be5, SUC_NE_lexicon_tags_2 },
    { 0x16d80be5, SUC_NE_lexicon_tags_3 },
    { 0x23c60be7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a0bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc72a0bee, SUC_NE_lexicon_tags_2 },
    { 0xe22a0bef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a20bf2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c0bf6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x16bc0bf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf8c0bfa, SUC_NE_lexicon_tags_2 },
    { 0x94400bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a5c0bfd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb6680bff, SUC_NE_lexicon_tags_2 },
    { 0xcb7a0bff, SUC_NE_lexicon_tags_2 },
    { 0x9ef40c00, SUC_NE_lexicon_tags_2 },
    { 0xb93e0c02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11b40c04, SUC_NE_lexicon_tags_2 },
    { 0x24080c05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8780c07, SUC_NE_lexicon_tags_2 },
    { 0xd4300c08, SUC_NE_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24c0c0b, SUC_NE_lexicon_tags_252 },
    { 0x57980c0c, SUC_NE_lexicon_tags_2 },
    { 0x289e0c0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c40c12, SUC_NE_lexicon_tags_2 },
    { 0xf8620c12, SUC_NE_lexicon_tags_2 },
    { 0xcf0e0c14, SUC_NE_lexicon_tags_2 },
    { 0x77660c12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91100c1a, SUC_NE_lexicon_tags_2 },
    { 0x77ee0c1b, SUC_NE_lexicon_tags_2 },
    { 0xc0600c1b, SUC_NE_lexicon_tags_2 },
    { 0xbb80c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f60c20, SUC_NE_lexicon_tags_2 },
    { 0xd380c21, SUC_NE_lexicon_tags_2 },
    { 0xe420c20, SUC_NE_lexicon_tags_2 },
    { 0xfb1c0c23, SUC_NE_lexicon_tags_7 },
    { 0x58ce0c21, SUC_NE_lexicon_tags_121 },
    { 0xccf00c23, SUC_NE_lexicon_tags_19 },
    { 0x45360c26, SUC_NE_lexicon_tags_2 },
    { 0x735e0c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa760c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9940c2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc2a0c2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd60c37, SUC_NE_lexicon_tags_3 },
    { 0xef6a0c38, SUC_NE_lexicon_tags_2 },
    { 0x33ce0c39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc0c3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c060c40, SUC_NE_lexicon_tags_2 },
    { 0x734a0c41, SUC_NE_lexicon_tags_2 },
    { 0xed7e0c40, SUC_NE_lexicon_tags_2 },
    { 0x9d40c43, SUC_NE_lexicon_tags_2 },
    { 0xa2d00c44, SUC_NE_lexicon_tags_1 },
    { 0x3c220c41, SUC_NE_lexicon_tags_2 },
    { 0xc59e0c46, SUC_NE_lexicon_tags_2 },
    { 0x3bd00c44, SUC_NE_lexicon_tags_2 },
    { 0x9e460c42, SUC_NE_lexicon_tags_2 },
    { 0xafd60c49, SUC_NE_lexicon_tags_2 },
    { 0x52a40c45, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd3d00c4c, SUC_NE_lexicon_tags_2 },
    { 0x57500c4d, SUC_NE_lexicon_tags_2 },
    { 0xd4da0c4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd000c50, SUC_NE_lexicon_tags_2 },
    { 0x8e400c51, SUC_NE_lexicon_tags_2 },
    { 0xe77a0c51, SUC_NE_lexicon_tags_2 },
    { 0x78540c50, SUC_NE_lexicon_tags_2 },
    { 0xda080c54, SUC_NE_lexicon_tags_2 },
    { 0x2be60c55, SUC_NE_lexicon_tags_2 },
    { 0xac900c56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a800c59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa80c63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805a0c6a, SUC_NE_lexicon_tags_2 },
    { 0xbaea0c6b, SUC_NE_lexicon_tags_4 },
    { 0xe12c0c6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fc0c72, SUC_NE_lexicon_tags_2 },
    { 0x1c8a0c72, SUC_NE_lexicon_tags_2 },
    { 0x95f20c73, SUC_NE_lexicon_tags_2 },
    { 0x853c0c75, SUC_NE_lexicon_tags_2 },
    { 0x4e500c76, SUC_NE_lexicon_tags_2 },
    { 0x2e560c73, SUC_NE_lexicon_tags_2 },
    { 0xf6a60c78, SUC_NE_lexicon_tags_5 },
    { 0x58820c75, SUC_NE_lexicon_tags_18 },
    { 0x46480c78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x449c0c7c, SUC_NE_lexicon_tags_2 },
    { 0xf1da0c7d, SUC_NE_lexicon_tags_51 },
    { 0x655e0c7e, SUC_NE_lexicon_tags_2 },
    { 0x2fe00c7c, SUC_NE_lexicon_tags_2 },
    { 0x605c0c80, SUC_NE_lexicon_tags_2 },
    { 0x9a1a0c7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8820c86, SUC_NE_lexicon_tags_2 },
    { 0x4f9e0c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba00c89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7e0c8c, SUC_NE_lexicon_tags_18 },
    { 0xb75e0c8d, SUC_NE_lexicon_tags_2 },
    { 0x86ec0c8e, SUC_NE_lexicon_tags_2 },
    { 0xfdfa0c8f, SUC_NE_lexicon_tags_2 },
    { 0xe3e0c8f, SUC_NE_lexicon_tags_3 },
    { 0x73d80c91, SUC_NE_lexicon_tags_2 },
    { 0xb69a0c91, SUC_NE_lexicon_tags_2 },
    { 0xcc800c91, SUC_NE_lexicon_tags_40 },
    { 0xe10a0c91, SUC_NE_lexicon_tags_2 },
    { 0x7faa0c91, SUC_NE_lexicon_tags_2 },
    { 0xb1020c92, SUC_NE_lexicon_tags_2 },
    { 0x77340c97, SUC_NE_lexicon_tags_2 },
    { 0x43780c97, SUC_NE_lexicon_tags_14 },
    { 0xa7720c92, SUC_NE_lexicon_tags_2 },
    { 0xe7d60c96, SUC_NE_lexicon_tags_2 },
    { 0x3b720c9b, SUC_NE_lexicon_tags_2 },
    { 0x71600c9c, SUC_NE_lexicon_tags_2 },
    { 0xaba60c9d, SUC_NE_lexicon_tags_2 },
    { 0x21b20c9d, SUC_NE_lexicon_tags_0 },
    { 0xabda0c9f, SUC_NE_lexicon_tags_2 },
    { 0x35f80ca0, SUC_NE_lexicon_tags_2 },
    { 0xcea00c98, SUC_NE_lexicon_tags_2 },
    { 0xe1160c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d40ca9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f240cab, SUC_NE_lexicon_tags_2 },
    { 0xfbbc0cac, SUC_NE_lexicon_tags_2 },
    { 0x4ae00cac, SUC_NE_lexicon_tags_242 },
    { 0xe8740cae, SUC_NE_lexicon_tags_2 },
    { 0x143c0caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea900cb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f80cbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5140cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc48e0cbf, SUC_NE_lexicon_tags_2 },
    { 0x1320cc0, SUC_NE_lexicon_tags_2 },
    { 0x9fb80cc0, SUC_NE_lexicon_tags_2 },
    { 0xb8420cc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95120cc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8420cca, SUC_NE_lexicon_tags_7 },
    { 0x9ebe0ccb, SUC_NE_lexicon_tags_2 },
    { 0x9b6c0cca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99960ccf, SUC_NE_lexicon_tags_7 },
    { 0x8e7e0cd0, SUC_NE_lexicon_tags_2 },
    { 0x52ea0cd1, SUC_NE_lexicon_tags_2 },
    { 0x48280cd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe200cd4, SUC_NE_lexicon_tags_2 },
    { 0x6b200cd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x783a0cd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe4a0cd9, SUC_NE_lexicon_tags_2 },
    { 0x44180cda, SUC_NE_lexicon_tags_2 },
    { 0x48f20cd9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca40cde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40500ce6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26480cec, SUC_NE_lexicon_tags_2 },
    { 0x95080ced, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f80cf3, SUC_NE_lexicon_tags_2 },
    { 0x35700cf4, SUC_NE_lexicon_tags_2 },
    { 0x8e7e0cf5, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0xae4e0cf7, SUC_NE_lexicon_tags_7 },
    { 0x15980cf7, SUC_NE_lexicon_tags_2 },
    { 0x987a0cf7, SUC_NE_lexicon_tags_2 },
    { 0xd3600cf8, SUC_NE_lexicon_tags_2 },
    { 0x925c0cfb, SUC_NE_lexicon_tags_2 },
    { 0x49d00cfc, SUC_NE_lexicon_tags_2 },
    { 0x23960cfb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e820d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c480d02, SUC_NE_lexicon_tags_2 },
    { 0xe6600d03, SUC_NE_lexicon_tags_2 },
    { 0x21380d03, SUC_NE_lexicon_tags_2 },
    { 0x8e380d04, SUC_NE_lexicon_tags_2 },
    { 0x8dac0d06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b00d0a, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x357a0d0c, SUC_NE_lexicon_tags_7 },
    { 0xd58a0d0d, SUC_NE_lexicon_tags_2 },
    { 0x34b00d0d, SUC_NE_lexicon_tags_2 },
    { 0x64e0d0f, SUC_NE_lexicon_tags_2 },
    { 0xd36c0d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55140d12, SUC_NE_lexicon_tags_27 },
    { 0x12c0d12, SUC_NE_lexicon_tags_7 },
    { 0x3b2c0d14, SUC_NE_lexicon_tags_2 },
    { 0x314c0d14, SUC_NE_lexicon_tags_2 },
    { 0x5d3e0d12, SUC_NE_lexicon_tags_2 },
    { 0x7f360d13, SUC_NE_lexicon_tags_2 },
    { 0x25ae0d18, SUC_NE_lexicon_tags_2 },
    { 0x737a0d19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578c0d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58a0d23, SUC_NE_lexicon_tags_2 },
    { 0x584c0d24, SUC_NE_lexicon_tags_2 },
    { 0xcfde0d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fa0d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38680d2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed820d2d, SUC_NE_lexicon_tags_2 },
    { 0x479a0d2e, SUC_NE_lexicon_tags_2 },
    { 0x19380d2e, SUC_NE_lexicon_tags_3 },
    { 0xecce0d30, SUC_NE_lexicon_tags_2 },
    { 0x8ba40d2f, SUC_NE_lexicon_tags_2 },
    { 0x9e8e0d2e, SUC_NE_lexicon_tags_2 },
    { 0x1ae00d2e, SUC_NE_lexicon_tags_2 },
    { 0xdee60d2d, SUC_NE_lexicon_tags_2 },
    { 0x6fc00d2d, SUC_NE_lexicon_tags_2 },
    { 0xa41e0d30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2260d3b, SUC_NE_lexicon_tags_2 },
    { 0xa9360d3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa0d41, SUC_NE_lexicon_tags_2 },
    { 0xe5b20d42, SUC_NE_lexicon_tags_2 },
    { 0xcb040d43, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xcaec0d45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca0d48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e480d4b, SUC_NE_lexicon_tags_2 },
    { 0x2a840d4b, SUC_NE_lexicon_tags_2 },
    { 0x63440d4d, SUC_NE_lexicon_tags_2 },
    { 0xf8560d4e, SUC_NE_lexicon_tags_2 },
    { 0x1c6a0d4b, SUC_NE_lexicon_tags_2 },
    { 0x390c0d50, SUC_NE_lexicon_tags_17 },
    { 0x36c0d51, SUC_NE_lexicon_tags_2 },
    { 0x8d3a0d50, SUC_NE_lexicon_tags_2 },
    { 0xf8240d53, SUC_NE_lexicon_tags_2 },
    { 0x6d4c0d51, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6c00d56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7e0d59, SUC_NE_lexicon_tags_18 },
    { 0x55500d5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a740d5e, SUC_NE_lexicon_tags_2 },
    { 0xf82c0d5e, SUC_NE_lexicon_tags_2 },
    { 0x1dba0d5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62d00d62, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6e0d65, SUC_NE_lexicon_tags_2 },
    { 0xa4840d66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd20d6a, SUC_NE_lexicon_tags_2 },
    { 0x76280d6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58140d6e, SUC_NE_lexicon_tags_39 },
    { 0xf1500d6e, SUC_NE_lexicon_tags_2 },
    { 0xb04c0d70, SUC_NE_lexicon_tags_2 },
    { 0x81420d6f, SUC_NE_lexicon_tags_2 },
    { 0x7a7a0d6e, SUC_NE_lexicon_tags_2 },
    { 0x61800d73, SUC_NE_lexicon_tags_2 },
    { 0x5c540d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea60d79, SUC_NE_lexicon_tags_2 },
    { 0x9eb00d7a, SUC_NE_lexicon_tags_2 },
    { 0x1eb40d7b, SUC_NE_lexicon_tags_2 },
    { 0x4e4e0d7c, SUC_NE_lexicon_tags_2 },
    { 0xbdb40d7d, SUC_NE_lexicon_tags_2 },
    { 0x10e40d7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5c0d82, SUC_NE_lexicon_tags_2 },
    { 0x619a0d82, SUC_NE_lexicon_tags_2 },
    { 0x10f40d83, SUC_NE_lexicon_tags_2 },
    { 0xf91e0d84, SUC_NE_lexicon_tags_2 },
    { 0x95660d84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b860d8d, SUC_NE_lexicon_tags_2 },
    { 0x1fa60d8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d20d96, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3400d9e, SUC_NE_lexicon_tags_2 },
    { 0x80700d9e, SUC_NE_lexicon_tags_2 },
    { 0xa4040da0, SUC_NE_lexicon_tags_2 },
    { 0xe5a40da1, SUC_NE_lexicon_tags_2 },
    { 0x6fae0da2, SUC_NE_lexicon_tags_2 },
    { 0x2e9c0da0, SUC_NE_lexicon_tags_2 },
    { 0xd3320da4, SUC_NE_lexicon_tags_2 },
    { 0xd9be0da4, SUC_NE_lexicon_tags_2 },
    { 0xe6800da4, SUC_NE_lexicon_tags_2 },
    { 0x87ee0da0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a360daa, SUC_NE_lexicon_tags_2 },
    { 0xc1c20dab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ca0db0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30160db2, SUC_NE_lexicon_tags_0 },
    { 0xfbbc0db3, SUC_NE_lexicon_tags_2 },
    { 0x3d1e0db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff8a0db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x730e0db9, SUC_NE_lexicon_tags_2 },
    { 0x6100dba, SUC_NE_lexicon_tags_2 },
    { 0x56700dbb, SUC_NE_lexicon_tags_2 },
    { 0x5b260dba, SUC_NE_lexicon_tags_2 },
    { 0x4eaa0dbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a60dc0, SUC_NE_lexicon_tags_2 },
    { 0xa8440dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7240dc6, SUC_NE_lexicon_tags_7 },
    { 0x1e120dc6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x71fa0dc9, SUC_NE_lexicon_tags_160 },
    { 0x64160dca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33980dcf, SUC_NE_lexicon_tags_2 },
    { 0xbb60dd0, SUC_NE_lexicon_tags_34 },
    { 0xaef20dd1, SUC_NE_lexicon_tags_2 },
    { 0x31ee0dd2, SUC_NE_lexicon_tags_2 },
    { 0x2c2e0dd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704e0dd7, SUC_NE_lexicon_tags_2 },
    { 0xf9a0dd8, SUC_NE_lexicon_tags_2 },
    { 0x53a20dd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a0ddc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61720de0, SUC_NE_lexicon_tags_19 },
    { 0xe0da0de1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d40de4, SUC_NE_lexicon_tags_2 },
    { 0x4a560de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba860de7, SUC_NE_lexicon_tags_2 },
    { 0x29480de8, SUC_NE_lexicon_tags_2 },
    { 0x34500de8, SUC_NE_lexicon_tags_2 },
    { 0x5b780dea, SUC_NE_lexicon_tags_2 },
    { 0x13520deb, SUC_NE_lexicon_tags_2 },
    { 0x62a80deb, SUC_NE_lexicon_tags_2 },
    { 0xa17e0ded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9460def, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6c0df3, SUC_NE_lexicon_tags_2 },
    { 0xe0d00df3, SUC_NE_lexicon_tags_2 },
    { 0x708c0df4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc0dfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4a0dfd, SUC_NE_lexicon_tags_2 },
    { 0x69380dfd, SUC_NE_lexicon_tags_410 },
    { 0xaf7a0dff, SUC_NE_lexicon_tags_7 },
    { 0xeff20e00, SUC_NE_lexicon_tags_2 },
    { 0x1100dfe, SUC_NE_lexicon_tags_2 },
    { 0xfe3c0e02, SUC_NE_lexicon_tags_232 },
    { 0x37700e03, SUC_NE_lexicon_tags_2 },
    { 0x17220e04, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x55200e06, SUC_NE_lexicon_tags_2 },
    { 0x6f460e07, SUC_NE_lexicon_tags_2 },
    { 0xbb320e08, SUC_NE_lexicon_tags_4 },
    { 0xcca00e09, SUC_NE_lexicon_tags_2 },
    { 0xadb80e0a, SUC_NE_lexicon_tags_3 },
    { 0xba9e0e08, SUC_NE_lexicon_tags_2 },
    { 0xcf6a0e08, SUC_NE_lexicon_tags_2 },
    { 0x19160e09, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5320e10, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb40e15, SUC_NE_lexicon_tags_51 },
    { 0x34fa0e16, SUC_NE_lexicon_tags_2 },
    { 0x92640e15, SUC_NE_lexicon_tags_2 },
    { 0x11500e18, SUC_NE_lexicon_tags_2 },
    { 0x7a4a0e18, SUC_NE_lexicon_tags_0 },
    { 0x2abe0e1a, SUC_NE_lexicon_tags_2 },
    { 0xbfdc0e18, SUC_NE_lexicon_tags_2 },
    { 0xb9260e1a, SUC_NE_lexicon_tags_2 },
    { 0xb3fc0e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84440e21, SUC_NE_lexicon_tags_7 },
    { 0xd03a0e22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe40e27, SUC_NE_lexicon_tags_192 },
    { 0x25d00e28, SUC_NE_lexicon_tags_2 },
    { 0xf08a0e27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2c0e2b, SUC_NE_lexicon_tags_2 },
    { 0x4f4a0e2c, SUC_NE_lexicon_tags_2 },
    { 0x609e0e2c, SUC_NE_lexicon_tags_2 },
    { 0x47960e2d, SUC_NE_lexicon_tags_2 },
    { 0x8cfe0e2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd740e33, SUC_NE_lexicon_tags_11 },
    { 0x655a0e33, SUC_NE_lexicon_tags_2 },
    { 0x16be0e34, SUC_NE_lexicon_tags_2 },
    { 0xa8400e33, SUC_NE_lexicon_tags_2 },
    { 0xf3b20e36, SUC_NE_lexicon_tags_2 },
    { 0xa5960e38, SUC_NE_lexicon_tags_3 },
    { 0x7f9a0e34, SUC_NE_lexicon_tags_2 },
    { 0x7cd40e39, SUC_NE_lexicon_tags_49 },
    { 0x2d4c0e3b, SUC_NE_lexicon_tags_2 },
    { 0x415e0e36, SUC_NE_lexicon_tags_3 },
    { 0xa49c0e3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c880e40, SUC_NE_lexicon_tags_2 },
    { 0xa3340e41, SUC_NE_lexicon_tags_2 },
    { 0x1bb00e42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd40e48, SUC_NE_lexicon_tags_2 },
    { 0x84080e48, SUC_NE_lexicon_tags_2 },
    { 0xb6a00e48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97fe0e4c, SUC_NE_lexicon_tags_1 },
    { 0xe1f20e4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74d60e4f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47080e53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x211a0e55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d940e57, SUC_NE_lexicon_tags_2 },
    { 0xaa720e57, SUC_NE_lexicon_tags_2 },
    { 0xcf420e58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58a40e5b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba0e5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61580e64, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802e0e6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b500e6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11720e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24980e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x111e0e75, SUC_NE_lexicon_tags_2 },
    { 0x39360e76, SUC_NE_lexicon_tags_2 },
    { 0x6ea60e75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8160e7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb80e7e, SUC_NE_lexicon_tags_2 },
    { 0xa2620e7f, SUC_NE_lexicon_tags_2 },
    { 0x4bdc0e7f, SUC_NE_lexicon_tags_2 },
    { 0x3d320e7f, SUC_NE_lexicon_tags_2 },
    { 0xd58a0e82, SUC_NE_lexicon_tags_2 },
    { 0xbbb00e83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ac0e88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef920e8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c0e8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde720e92, SUC_NE_lexicon_tags_2 },
    { 0xe8ce0e93, SUC_NE_lexicon_tags_2 },
    { 0x970e0e92, SUC_NE_lexicon_tags_2 },
    { 0x5f100e95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08c0ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27f00ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c0ea6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2e80ea8, SUC_NE_lexicon_tags_2 },
    { 0x9ede0ea9, SUC_NE_lexicon_tags_2 },
    { 0x7f100eaa, SUC_NE_lexicon_tags_2 },
    { 0x71900eab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100e0eb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467a0eb7, SUC_NE_lexicon_tags_2 },
    { 0x27820eb8, SUC_NE_lexicon_tags_4 },
    { 0xa7400eb7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x283c0ebb, SUC_NE_lexicon_tags_2 },
    { 0x15660ebc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6840ebf, SUC_NE_lexicon_tags_2 },
    { 0xe6ee0ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adc0ec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8340ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x821a0ec9, SUC_NE_lexicon_tags_2 },
    { 0x5a840eca, SUC_NE_lexicon_tags_2 },
    { 0xf9680ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a40ece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee0ed1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41000ed5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xec660ed7, SUC_NE_lexicon_tags_2 },
    { 0x330c0ed8, SUC_NE_lexicon_tags_2 },
    { 0x5c8c0ed9, SUC_NE_lexicon_tags_2 },
    { 0xb8720eda, SUC_NE_lexicon_tags_209 },
    { 0xcfde0ed9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x20d80edd, SUC_NE_lexicon_tags_0 },
    { 0x40ba0ede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91ec0ee0, SUC_NE_lexicon_tags_2 },
    { 0x35560ee1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x737a0ee3, SUC_NE_lexicon_tags_2 },
    { 0x5d440ee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c40eea, SUC_NE_lexicon_tags_2 },
    { 0x2d220eeb, SUC_NE_lexicon_tags_7 },
    { 0x1d020eea, SUC_NE_lexicon_tags_2 },
    { 0x839c0eed, SUC_NE_lexicon_tags_2 },
    { 0x7020eed, SUC_NE_lexicon_tags_2 },
    { 0x20b80eef, SUC_NE_lexicon_tags_7 },
    { 0x65680ef0, SUC_NE_lexicon_tags_2 },
    { 0x7a140ef0, SUC_NE_lexicon_tags_3 },
    { 0xe2b80eea, SUC_NE_lexicon_tags_2 },
    { 0xb0980ef3, SUC_NE_lexicon_tags_2 },
    { 0xf3c20ef4, SUC_NE_lexicon_tags_2 },
    { 0x67720ef5, SUC_NE_lexicon_tags_2 },
    { 0xeeaa0ef6, SUC_NE_lexicon_tags_2 },
    { 0x4c340ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42c0efb, SUC_NE_lexicon_tags_2 },
    { 0x4da60efb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f40f00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadfa0f02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ac0f09, SUC_NE_lexicon_tags_2 },
    { 0x98e40f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d640f0e, SUC_NE_lexicon_tags_2 },
    { 0x5b740f0f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21520f12, SUC_NE_lexicon_tags_268 },
    { 0, NULL },
    { 0x4f5c0f14, SUC_NE_lexicon_tags_2 },
    { 0x1e0a0f14, SUC_NE_lexicon_tags_2 },
    { 0x823c0f16, SUC_NE_lexicon_tags_51 },
    { 0xa90e0f17, SUC_NE_lexicon_tags_2 },
    { 0xb32e0f18, SUC_NE_lexicon_tags_2 },
    { 0x5820f19, SUC_NE_lexicon_tags_3 },
    { 0xd980f1a, SUC_NE_lexicon_tags_2 },
    { 0xd9340f1b, SUC_NE_lexicon_tags_2 },
    { 0x2ca80f1b, SUC_NE_lexicon_tags_4 },
    { 0x66460f1a, SUC_NE_lexicon_tags_2 },
    { 0x7c580f1e, SUC_NE_lexicon_tags_389 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60120f23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f860f29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c680f2b, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x1bf60f2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe3a0f2f, SUC_NE_lexicon_tags_2 },
    { 0xdf500f30, SUC_NE_lexicon_tags_2 },
    { 0x551a0f31, SUC_NE_lexicon_tags_2 },
    { 0x5b700f31, SUC_NE_lexicon_tags_2 },
    { 0x8d7e0f33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19d60f35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60900f37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c400f3d, SUC_NE_lexicon_tags_2 },
    { 0x86aa0f3e, SUC_NE_lexicon_tags_2 },
    { 0x1ee00f3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c440f41, SUC_NE_lexicon_tags_2 },
    { 0x9b700f41, SUC_NE_lexicon_tags_2 },
    { 0xbc180f43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe0f48, SUC_NE_lexicon_tags_4 },
    { 0x43d20f49, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2460f4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a0f50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46cc0f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f20f55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x327e0f58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9bc0f5a, SUC_NE_lexicon_tags_2 },
    { 0x480a0f5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc0f5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b200f60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728c0f68, SUC_NE_lexicon_tags_2 },
    { 0xd9d00f68, SUC_NE_lexicon_tags_2 },
    { 0x963c0f69, SUC_NE_lexicon_tags_2 },
    { 0x48a40f6b, SUC_NE_lexicon_tags_2 },
    { 0xaefe0f6c, SUC_NE_lexicon_tags_17 },
    { 0x18d20f69, SUC_NE_lexicon_tags_2 },
    { 0xe18c0f6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a80f74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b500f76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3440f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a40f7e, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfa0f86, SUC_NE_lexicon_tags_2 },
    { 0x4a940f87, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c040f97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeea80f99, SUC_NE_lexicon_tags_2 },
    { 0x3fac0f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b00f9d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a5e0f9f, SUC_NE_lexicon_tags_3 },
    { 0xf280fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79da0fa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04e0fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19980fa8, SUC_NE_lexicon_tags_2 },
    { 0xde340fa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa220fab, SUC_NE_lexicon_tags_2 },
    { 0xd1360fac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c400faf, SUC_NE_lexicon_tags_2 },
    { 0x4b480fb0, SUC_NE_lexicon_tags_2 },
    { 0xa0240fb1, SUC_NE_lexicon_tags_2 },
    { 0x57640faf, SUC_NE_lexicon_tags_2 },
    { 0x433e0fb3, SUC_NE_lexicon_tags_2 },
    { 0x470c0fb4, SUC_NE_lexicon_tags_3 },
    { 0xe9480fb4, SUC_NE_lexicon_tags_2 },
    { 0x6db40fb6, SUC_NE_lexicon_tags_2 },
    { 0xb7720fb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb0a0fb9, SUC_NE_lexicon_tags_2 },
    { 0xf0da0fba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b320fc0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc20fc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x408a0fc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61800fca, SUC_NE_lexicon_tags_2 },
    { 0x12100fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609c0fd0, SUC_NE_lexicon_tags_2 },
    { 0x16a80fd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67c0fd3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ae0fd9, SUC_NE_lexicon_tags_2 },
    { 0x261a0fda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b00fdd, SUC_NE_lexicon_tags_2 },
    { 0x89860fdd, SUC_NE_lexicon_tags_3 },
    { 0x79b80fde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98400fe1, SUC_NE_lexicon_tags_2 },
    { 0x35c0fe2, SUC_NE_lexicon_tags_2 },
    { 0x51600fe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9c0fe8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877c0fec, SUC_NE_lexicon_tags_2 },
    { 0x1dfa0fed, SUC_NE_lexicon_tags_2 },
    { 0x22dc0fee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969e0ff3, SUC_NE_lexicon_tags_2 },
    { 0x329c0ff4, SUC_NE_lexicon_tags_2 },
    { 0xd6c80ff3, SUC_NE_lexicon_tags_2 },
    { 0xcdd80ff3, SUC_NE_lexicon_tags_2 },
    { 0xc6a0ff3, SUC_NE_lexicon_tags_13 },
    { 0x7db40ff8, SUC_NE_lexicon_tags_2 },
    { 0xd1c40ff7, SUC_NE_lexicon_tags_2 },
    { 0xa4a40ff6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1580ffd, SUC_NE_lexicon_tags_2 },
    { 0xd0120ffd, SUC_NE_lexicon_tags_2 },
    { 0x650e0ffe, SUC_NE_lexicon_tags_2 },
    { 0x2a1c1000, SUC_NE_lexicon_tags_2 },
    { 0xcd9a0ffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x385a1004, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8781007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c100a, SUC_NE_lexicon_tags_2 },
    { 0x6fe8100b, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1a100d, SUC_NE_lexicon_tags_2 },
    { 0xcfe2100d, SUC_NE_lexicon_tags_2 },
    { 0xd18100f, SUC_NE_lexicon_tags_2 },
    { 0xfe38100f, SUC_NE_lexicon_tags_4 },
    { 0x642100f, SUC_NE_lexicon_tags_2 },
    { 0xd1741012, SUC_NE_lexicon_tags_2 },
    { 0x9d46100f, SUC_NE_lexicon_tags_2 },
    { 0x9b5e1014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52781016, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcac101a, SUC_NE_lexicon_tags_2 },
    { 0xbc16101b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e1021, SUC_NE_lexicon_tags_2 },
    { 0x1bf81021, SUC_NE_lexicon_tags_2 },
    { 0x72021021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc64c1025, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x481a1027, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b72102c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27c1031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f01037, SUC_NE_lexicon_tags_2 },
    { 0x70cc1037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce0103c, SUC_NE_lexicon_tags_2 },
    { 0xf544103c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09e1040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce61044, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3641047, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x9961049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba104c, SUC_NE_lexicon_tags_2 },
    { 0xa854104d, SUC_NE_lexicon_tags_3 },
    { 0x487c104d, SUC_NE_lexicon_tags_2 },
    { 0xe96e104f, SUC_NE_lexicon_tags_2 },
    { 0xe35a1050, SUC_NE_lexicon_tags_2 },
    { 0x29ea1051, SUC_NE_lexicon_tags_2 },
    { 0xb1e2104c, SUC_NE_lexicon_tags_7 },
    { 0x42e21053, SUC_NE_lexicon_tags_7 },
    { 0x6fb61053, SUC_NE_lexicon_tags_3 },
    { 0xa6f4104e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fdc1057, SUC_NE_lexicon_tags_2 },
    { 0xfb581057, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eca105c, SUC_NE_lexicon_tags_2 },
    { 0x1a06105c, SUC_NE_lexicon_tags_2 },
    { 0x23b0105d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a881061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec381068, SUC_NE_lexicon_tags_2 },
    { 0x56221069, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dc106d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d56106f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f541073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa301075, SUC_NE_lexicon_tags_2 },
    { 0x5c741076, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22e107a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0181080, SUC_NE_lexicon_tags_2 },
    { 0x53301081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f01084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38e1087, SUC_NE_lexicon_tags_2 },
    { 0xab321088, SUC_NE_lexicon_tags_2 },
    { 0x29ac1089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1444108d, SUC_NE_lexicon_tags_2 },
    { 0x77ca108d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f181090, SUC_NE_lexicon_tags_2 },
    { 0xc2c1090, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13a21093, SUC_NE_lexicon_tags_2 },
    { 0x67da1093, SUC_NE_lexicon_tags_2 },
    { 0x9ce21095, SUC_NE_lexicon_tags_2 },
    { 0x97dc1094, SUC_NE_lexicon_tags_2 },
    { 0xe8ee1097, SUC_NE_lexicon_tags_2 },
    { 0xaf1a1097, SUC_NE_lexicon_tags_34 },
    { 0x8cfc1095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882e109d, SUC_NE_lexicon_tags_2 },
    { 0x86c4109e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3a10a2, SUC_NE_lexicon_tags_2 },
    { 0x59b410a3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f5a10a5, SUC_NE_lexicon_tags_2 },
    { 0x720c10a6, SUC_NE_lexicon_tags_2 },
    { 0xfa4810a6, SUC_NE_lexicon_tags_3 },
    { 0x75dc10a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac6210aa, SUC_NE_lexicon_tags_7 },
    { 0x4a9410aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cc010ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3f010af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239610b3, SUC_NE_lexicon_tags_5 },
    { 0x12d210b4, SUC_NE_lexicon_tags_2 },
    { 0x12ce10b5, SUC_NE_lexicon_tags_2 },
    { 0x590610b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2410b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3c210bb, SUC_NE_lexicon_tags_7 },
    { 0x87e210bc, SUC_NE_lexicon_tags_2 },
    { 0x86210bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf68c10bf, SUC_NE_lexicon_tags_8 },
    { 0x6f7a10c0, SUC_NE_lexicon_tags_2 },
    { 0xc25810c0, SUC_NE_lexicon_tags_2 },
    { 0xb9f810c2, SUC_NE_lexicon_tags_2 },
    { 0x784810c3, SUC_NE_lexicon_tags_2 },
    { 0x312a10c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690210c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180e10cd, SUC_NE_lexicon_tags_2 },
    { 0x119810cd, SUC_NE_lexicon_tags_2 },
    { 0x941810cf, SUC_NE_lexicon_tags_2 },
    { 0xed2810d0, SUC_NE_lexicon_tags_2 },
    { 0x29f210d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f6c10d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe58c10d5, SUC_NE_lexicon_tags_2 },
    { 0x601410d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41610d9, SUC_NE_lexicon_tags_2 },
    { 0x300c10d9, SUC_NE_lexicon_tags_2 },
    { 0x657210d9, SUC_NE_lexicon_tags_2 },
    { 0x56b810dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e10e0, SUC_NE_lexicon_tags_2 },
    { 0x852a10e0, SUC_NE_lexicon_tags_2 },
    { 0x77810e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45a810e4, SUC_NE_lexicon_tags_0 },
    { 0x96b010e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430610f2, SUC_NE_lexicon_tags_2 },
    { 0x9a5810f3, SUC_NE_lexicon_tags_107 },
    { 0x737210f4, SUC_NE_lexicon_tags_2 },
    { 0x6a6a10f3, SUC_NE_lexicon_tags_2 },
    { 0xd4fc10f6, SUC_NE_lexicon_tags_2 },
    { 0x2d410f7, SUC_NE_lexicon_tags_2 },
    { 0x38f610f6, SUC_NE_lexicon_tags_2 },
    { 0x219210f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x153e10fc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a81106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8310110d, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e01112, SUC_NE_lexicon_tags_3 },
    { 0x591c1112, SUC_NE_lexicon_tags_2 },
    { 0xd4061113, SUC_NE_lexicon_tags_2 },
    { 0x19661115, SUC_NE_lexicon_tags_2 },
    { 0x61b01116, SUC_NE_lexicon_tags_2 },
    { 0x4721117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a72111f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13c1124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c2112a, SUC_NE_lexicon_tags_2 },
    { 0xa8c4112b, SUC_NE_lexicon_tags_2 },
    { 0x36c4112b, SUC_NE_lexicon_tags_7 },
    { 0x67cc112d, SUC_NE_lexicon_tags_2 },
    { 0x772a112e, SUC_NE_lexicon_tags_2 },
    { 0x91b4112f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf2cc1131, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7301134, SUC_NE_lexicon_tags_2 },
    { 0xc2a61135, SUC_NE_lexicon_tags_2 },
    { 0xaec1134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba2113a, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fc1141, SUC_NE_lexicon_tags_2 },
    { 0x477e1142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6121146, SUC_NE_lexicon_tags_2 },
    { 0x58d81147, SUC_NE_lexicon_tags_2 },
    { 0x3bb81148, SUC_NE_lexicon_tags_4 },
    { 0x2f121149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadde114f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84981154, SUC_NE_lexicon_tags_4 },
    { 0x2e4a1154, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c1158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54da115b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6190115e, SUC_NE_lexicon_tags_2 },
    { 0x8ff2115e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb94c1161, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xa80a1163, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20a1168, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a4e116a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b64116f, SUC_NE_lexicon_tags_2 },
    { 0x4fa116f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ff81172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e701175, SUC_NE_lexicon_tags_7 },
    { 0x469c1175, SUC_NE_lexicon_tags_2 },
    { 0x4f6e1175, SUC_NE_lexicon_tags_7 },
    { 0x9b6a1178, SUC_NE_lexicon_tags_2 },
    { 0xb8401179, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aa117d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e861182, SUC_NE_lexicon_tags_2 },
    { 0xecea1183, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc20c1185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dbc1187, SUC_NE_lexicon_tags_2 },
    { 0x82541187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b81191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa401196, SUC_NE_lexicon_tags_2 },
    { 0xb5241197, SUC_NE_lexicon_tags_2 },
    { 0xb1b61198, SUC_NE_lexicon_tags_2 },
    { 0x51041198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a4119d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1211a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4df411a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4011a7, SUC_NE_lexicon_tags_2 },
    { 0xa4a011a7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa88211aa, SUC_NE_lexicon_tags_2 },
    { 0xfd6a11ab, SUC_NE_lexicon_tags_8 },
    { 0x7dbc11ac, SUC_NE_lexicon_tags_2 },
    { 0x9b9011ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x971411b0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe01611b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6811b4, SUC_NE_lexicon_tags_2 },
    { 0x9e1811b5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd211b8, SUC_NE_lexicon_tags_2 },
    { 0xdfe811b9, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aa11bf, SUC_NE_lexicon_tags_2 },
    { 0x25b011c0, SUC_NE_lexicon_tags_2 },
    { 0xe3c11c0, SUC_NE_lexicon_tags_2 },
    { 0xa27611c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f411c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9611cd, SUC_NE_lexicon_tags_2 },
    { 0x50e011ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ee11d1, SUC_NE_lexicon_tags_2 },
    { 0xd5f811d2, SUC_NE_lexicon_tags_2 },
    { 0x3eb411d2, SUC_NE_lexicon_tags_2 },
    { 0xd3c011d1, SUC_NE_lexicon_tags_36 },
    { 0x57fa11d5, SUC_NE_lexicon_tags_2 },
    { 0x203a11d2, SUC_NE_lexicon_tags_2 },
    { 0x43de11d4, SUC_NE_lexicon_tags_2 },
    { 0x53c211d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0211db, SUC_NE_lexicon_tags_0 },
    { 0x5efc11db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7411e3, SUC_NE_lexicon_tags_2 },
    { 0x790011e4, SUC_NE_lexicon_tags_2 },
    { 0x8af011e5, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503411ee, SUC_NE_lexicon_tags_2 },
    { 0xac2c11ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5811fa, SUC_NE_lexicon_tags_2 },
    { 0xbb811fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74011ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af81202, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5681205, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f6120d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ac1211, SUC_NE_lexicon_tags_2 },
    { 0xc73a1212, SUC_NE_lexicon_tags_2 },
    { 0xc5aa1212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fa1217, SUC_NE_lexicon_tags_7 },
    { 0x96461217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496121f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb52c1221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebbe1223, SUC_NE_lexicon_tags_2 },
    { 0x97e21224, SUC_NE_lexicon_tags_28 },
    { 0x3b2c1224, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4541227, SUC_NE_lexicon_tags_2 },
    { 0x34681228, SUC_NE_lexicon_tags_7 },
    { 0xda581229, SUC_NE_lexicon_tags_2 },
    { 0x5f42122a, SUC_NE_lexicon_tags_2 },
    { 0x593e122b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf881230, SUC_NE_lexicon_tags_23 },
    { 0xc45e1231, SUC_NE_lexicon_tags_2 },
    { 0xb6e21231, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba701235, SUC_NE_lexicon_tags_2 },
    { 0x97321236, SUC_NE_lexicon_tags_2 },
    { 0x32541237, SUC_NE_lexicon_tags_2 },
    { 0xe8dc1235, SUC_NE_lexicon_tags_2 },
    { 0x4c8e1235, SUC_NE_lexicon_tags_2 },
    { 0x1d4c123a, SUC_NE_lexicon_tags_2 },
    { 0x7a8c123b, SUC_NE_lexicon_tags_2 },
    { 0xc9ae123b, SUC_NE_lexicon_tags_18 },
    { 0xffe0123d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebdc123f, SUC_NE_lexicon_tags_2 },
    { 0xc7f21240, SUC_NE_lexicon_tags_2 },
    { 0x15301241, SUC_NE_lexicon_tags_2 },
    { 0xeffa1241, SUC_NE_lexicon_tags_2 },
    { 0x9c241243, SUC_NE_lexicon_tags_2 },
    { 0x95261243, SUC_NE_lexicon_tags_2 },
    { 0xf6121243, SUC_NE_lexicon_tags_2 },
    { 0xcc5c1246, SUC_NE_lexicon_tags_7 },
    { 0x63cc1246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9e81249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb661253, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ae1255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67641258, SUC_NE_lexicon_tags_2 },
    { 0x57261258, SUC_NE_lexicon_tags_2 },
    { 0xecb8125a, SUC_NE_lexicon_tags_2 },
    { 0x33c4125a, SUC_NE_lexicon_tags_2 },
    { 0xf2461259, SUC_NE_lexicon_tags_2 },
    { 0xade6125c, SUC_NE_lexicon_tags_2 },
    { 0x1196125c, SUC_NE_lexicon_tags_2 },
    { 0x4f2e125d, SUC_NE_lexicon_tags_3 },
    { 0x6e36125a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xec161262, SUC_NE_lexicon_tags_1 },
    { 0xe34e1262, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf01266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa101269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6161276, SUC_NE_lexicon_tags_2 },
    { 0x79b01276, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1c01279, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a127c, SUC_NE_lexicon_tags_2 },
    { 0x16fe127d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79801284, SUC_NE_lexicon_tags_2 },
    { 0xf0901285, SUC_NE_lexicon_tags_2 },
    { 0x5a641284, SUC_NE_lexicon_tags_2 },
    { 0x119c1287, SUC_NE_lexicon_tags_2 },
    { 0x6e981287, SUC_NE_lexicon_tags_3 },
    { 0xd9ac1288, SUC_NE_lexicon_tags_2 },
    { 0x6e86128a, SUC_NE_lexicon_tags_2 },
    { 0x618c128b, SUC_NE_lexicon_tags_2 },
    { 0x7faa1285, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc128f, SUC_NE_lexicon_tags_2 },
    { 0xc7221290, SUC_NE_lexicon_tags_2 },
    { 0xb5321291, SUC_NE_lexicon_tags_2 },
    { 0x91721292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb41295, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ee1298, SUC_NE_lexicon_tags_2 },
    { 0xc0661299, SUC_NE_lexicon_tags_2 },
    { 0x91681299, SUC_NE_lexicon_tags_4 },
    { 0xa2a4129b, SUC_NE_lexicon_tags_30 },
    { 0x7d76129c, SUC_NE_lexicon_tags_2 },
    { 0xcc3c129d, SUC_NE_lexicon_tags_2 },
    { 0x1ca21298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdc012a0, SUC_NE_lexicon_tags_2 },
    { 0x893212a1, SUC_NE_lexicon_tags_2 },
    { 0x5d212a2, SUC_NE_lexicon_tags_24 },
    { 0x79a612a2, SUC_NE_lexicon_tags_2 },
    { 0x95c212a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc71612a6, SUC_NE_lexicon_tags_2 },
    { 0x998412a7, SUC_NE_lexicon_tags_4 },
    { 0x949012a7, SUC_NE_lexicon_tags_2 },
    { 0x734e12a9, SUC_NE_lexicon_tags_2 },
    { 0x3dcc12a9, SUC_NE_lexicon_tags_392 },
    { 0, NULL },
    { 0x574a12ac, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06412b0, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c12bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38e12c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d612c5, SUC_NE_lexicon_tags_2 },
    { 0xc8e612c5, SUC_NE_lexicon_tags_2 },
    { 0xc82012c5, SUC_NE_lexicon_tags_2 },
    { 0xac5612c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87012cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b412ce, SUC_NE_lexicon_tags_2 },
    { 0xcbca12cf, SUC_NE_lexicon_tags_1 },
    { 0xd5e012d0, SUC_NE_lexicon_tags_2 },
    { 0xd59c12d0, SUC_NE_lexicon_tags_2 },
    { 0xc41412cf, SUC_NE_lexicon_tags_2 },
    { 0x322a12cf, SUC_NE_lexicon_tags_2 },
    { 0xea3212d4, SUC_NE_lexicon_tags_2 },
    { 0x42f812d5, SUC_NE_lexicon_tags_41 },
    { 0x601412d6, SUC_NE_lexicon_tags_2 },
    { 0xdc2c12d7, SUC_NE_lexicon_tags_2 },
    { 0xa18612d8, SUC_NE_lexicon_tags_2 },
    { 0x298012d5, SUC_NE_lexicon_tags_17 },
    { 0x7ca012da, SUC_NE_lexicon_tags_2 },
    { 0xe34612d8, SUC_NE_lexicon_tags_2 },
    { 0xad2a12d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf70e12de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831a12e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87e612e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de212ec, SUC_NE_lexicon_tags_2 },
    { 0x2f4212ed, SUC_NE_lexicon_tags_2 },
    { 0xb9ec12ee, SUC_NE_lexicon_tags_56 },
    { 0xdc6812ef, SUC_NE_lexicon_tags_2 },
    { 0xd2e412ef, SUC_NE_lexicon_tags_2 },
    { 0x824412f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2ee12f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25212f6, SUC_NE_lexicon_tags_2 },
    { 0xc93612f7, SUC_NE_lexicon_tags_2 },
    { 0x5e6212f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaad212fa, SUC_NE_lexicon_tags_2 },
    { 0xe8e412fb, SUC_NE_lexicon_tags_7 },
    { 0x74612fc, SUC_NE_lexicon_tags_2 },
    { 0x29b812fc, SUC_NE_lexicon_tags_2 },
    { 0xa8a612fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c1301, SUC_NE_lexicon_tags_3 },
    { 0x54b01302, SUC_NE_lexicon_tags_2 },
    { 0x83241303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf081305, SUC_NE_lexicon_tags_30 },
    { 0xd3901306, SUC_NE_lexicon_tags_2 },
    { 0xcc01307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902130c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x741c130f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf06c1311, SUC_NE_lexicon_tags_2 },
    { 0xa0f21312, SUC_NE_lexicon_tags_2 },
    { 0x5d021312, SUC_NE_lexicon_tags_3 },
    { 0x7ca81314, SUC_NE_lexicon_tags_3 },
    { 0x95581311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b81318, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fac131a, SUC_NE_lexicon_tags_2 },
    { 0xf554131b, SUC_NE_lexicon_tags_2 },
    { 0xd662131b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa131f, SUC_NE_lexicon_tags_2 },
    { 0x57221320, SUC_NE_lexicon_tags_2 },
    { 0x3f441320, SUC_NE_lexicon_tags_3 },
    { 0x8c80131f, SUC_NE_lexicon_tags_2 },
    { 0x96bc1321, SUC_NE_lexicon_tags_2 },
    { 0x86fc1320, SUC_NE_lexicon_tags_2 },
    { 0x68de1324, SUC_NE_lexicon_tags_2 },
    { 0x61001323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7e41328, SUC_NE_lexicon_tags_1 },
    { 0xf1301329, SUC_NE_lexicon_tags_2 },
    { 0x3e41329, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcae8132c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc0132e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c021333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83161335, SUC_NE_lexicon_tags_2 },
    { 0x31761336, SUC_NE_lexicon_tags_2 },
    { 0x7c541336, SUC_NE_lexicon_tags_2 },
    { 0xa1481338, SUC_NE_lexicon_tags_2 },
    { 0xe2601339, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a133f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe81347, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb018134c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe134f, SUC_NE_lexicon_tags_2 },
    { 0x2184134f, SUC_NE_lexicon_tags_2 },
    { 0xe8021351, SUC_NE_lexicon_tags_2 },
    { 0x28a81352, SUC_NE_lexicon_tags_2 },
    { 0xb9c81353, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c4135a, SUC_NE_lexicon_tags_2 },
    { 0xd1c135b, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace01363, SUC_NE_lexicon_tags_4 },
    { 0x5e1364, SUC_NE_lexicon_tags_2 },
    { 0xf33a1364, SUC_NE_lexicon_tags_2 },
    { 0xb9ba1366, SUC_NE_lexicon_tags_7 },
    { 0x1cb41367, SUC_NE_lexicon_tags_2 },
    { 0x789e1364, SUC_NE_lexicon_tags_2 },
    { 0xf5161365, SUC_NE_lexicon_tags_2 },
    { 0x46f6136a, SUC_NE_lexicon_tags_2 },
    { 0xdcc8136a, SUC_NE_lexicon_tags_2 },
    { 0x22f6136a, SUC_NE_lexicon_tags_2 },
    { 0x126c136d, SUC_NE_lexicon_tags_2 },
    { 0x3d4136e, SUC_NE_lexicon_tags_2 },
    { 0x540a136f, SUC_NE_lexicon_tags_2 },
    { 0x6a321370, SUC_NE_lexicon_tags_2 },
    { 0x96c01371, SUC_NE_lexicon_tags_0 },
    { 0x422c1371, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f481374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68161376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489a137a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb574137c, SUC_NE_lexicon_tags_2 },
    { 0x6f64137d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf690137f, SUC_NE_lexicon_tags_2 },
    { 0x6bc41380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0221387, SUC_NE_lexicon_tags_40 },
    { 0x4eda1388, SUC_NE_lexicon_tags_2 },
    { 0xd9341389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92c0138b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9900138e, SUC_NE_lexicon_tags_2 },
    { 0x1598138e, SUC_NE_lexicon_tags_2 },
    { 0x653e138e, SUC_NE_lexicon_tags_2 },
    { 0xa2361390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d961395, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f001398, SUC_NE_lexicon_tags_2 },
    { 0xc9a61399, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb226139f, SUC_NE_lexicon_tags_2 },
    { 0x5bb813a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2a13a3, SUC_NE_lexicon_tags_2 },
    { 0x610413a4, SUC_NE_lexicon_tags_2 },
    { 0x8cc213a5, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca13a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1013af, SUC_NE_lexicon_tags_2 },
    { 0x1d7213af, SUC_NE_lexicon_tags_2 },
    { 0xa7a613af, SUC_NE_lexicon_tags_2 },
    { 0xe6c413b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f613b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b813b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff213be, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac13c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29413c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcee613c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17213cd, SUC_NE_lexicon_tags_2 },
    { 0xa4b613ce, SUC_NE_lexicon_tags_2 },
    { 0x17ba13ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x792213d1, SUC_NE_lexicon_tags_2 },
    { 0xcc7813d2, SUC_NE_lexicon_tags_4 },
    { 0xe63413d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89dc13d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738813da, SUC_NE_lexicon_tags_2 },
    { 0x421813db, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449013e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed013e8, SUC_NE_lexicon_tags_2 },
    { 0x394013e9, SUC_NE_lexicon_tags_2 },
    { 0xb1ba13ea, SUC_NE_lexicon_tags_2 },
    { 0xc48413eb, SUC_NE_lexicon_tags_2 },
    { 0xd1e413ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x793a13ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de613f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x842213fa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x191e13fc, SUC_NE_lexicon_tags_2 },
    { 0x166213fd, SUC_NE_lexicon_tags_344 },
    { 0x9da013fe, SUC_NE_lexicon_tags_3 },
    { 0xdc4a13ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec1401, SUC_NE_lexicon_tags_2 },
    { 0x61da1402, SUC_NE_lexicon_tags_2 },
    { 0x65ec1402, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55121406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x718c1408, SUC_NE_lexicon_tags_2 },
    { 0xb9c1408, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe984140c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd008140e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7ce1410, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x355a1413, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cc141c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb4141f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f61422, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae181426, SUC_NE_lexicon_tags_56 },
    { 0, NULL },
    { 0x69761428, SUC_NE_lexicon_tags_2 },
    { 0xd3ce1429, SUC_NE_lexicon_tags_25 },
    { 0xc3fc1429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x438a142c, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x1c94142e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb521433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca1437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408143d, SUC_NE_lexicon_tags_2 },
    { 0x80e0143e, SUC_NE_lexicon_tags_2 },
    { 0x9482143e, SUC_NE_lexicon_tags_2 },
    { 0x89301440, SUC_NE_lexicon_tags_2 },
    { 0xf09c1440, SUC_NE_lexicon_tags_2 },
    { 0xe9181442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb461448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x135a144b, SUC_NE_lexicon_tags_2 },
    { 0x9092144b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf0144f, SUC_NE_lexicon_tags_2 },
    { 0x7381450, SUC_NE_lexicon_tags_2 },
    { 0x6abe1451, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x814a1453, SUC_NE_lexicon_tags_2 },
    { 0xac881454, SUC_NE_lexicon_tags_7 },
    { 0x70e1455, SUC_NE_lexicon_tags_2 },
    { 0x696e1454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc145c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bde1463, SUC_NE_lexicon_tags_2 },
    { 0x777e1463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88401467, SUC_NE_lexicon_tags_2 },
    { 0x6d981468, SUC_NE_lexicon_tags_2 },
    { 0x69ee1469, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c01475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37fc1477, SUC_NE_lexicon_tags_2 },
    { 0x63c01477, SUC_NE_lexicon_tags_2 },
    { 0x91e61479, SUC_NE_lexicon_tags_2 },
    { 0x5f1a1479, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f10147c, SUC_NE_lexicon_tags_2 },
    { 0x9356147c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9bc147f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba941483, SUC_NE_lexicon_tags_2 },
    { 0x4ab81484, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e1489, SUC_NE_lexicon_tags_2 },
    { 0x9a8a148a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5f6148c, SUC_NE_lexicon_tags_2 },
    { 0x524148c, SUC_NE_lexicon_tags_2 },
    { 0x12b8148d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7df61490, SUC_NE_lexicon_tags_0 },
    { 0x29e61491, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c81494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccca1496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ae149d, SUC_NE_lexicon_tags_0 },
    { 0x8f7c149e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ec14a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf75e14a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78814a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b014ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbc14b0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70014b3, SUC_NE_lexicon_tags_2 },
    { 0x771a14b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c614b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ee614bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4814bf, SUC_NE_lexicon_tags_2 },
    { 0x714c14bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c0414c2, SUC_NE_lexicon_tags_2 },
    { 0x896e14c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f014c6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1014c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87814cc, SUC_NE_lexicon_tags_2 },
    { 0xce2214cd, SUC_NE_lexicon_tags_2 },
    { 0xf85a14ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d414d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8414d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7014de, SUC_NE_lexicon_tags_2 },
    { 0xc87a14df, SUC_NE_lexicon_tags_2 },
    { 0x727c14df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a4e14e2, SUC_NE_lexicon_tags_2 },
    { 0xba414e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x288614e6, SUC_NE_lexicon_tags_2 },
    { 0x927614e7, SUC_NE_lexicon_tags_2 },
    { 0x4ff814e8, SUC_NE_lexicon_tags_2 },
    { 0xc5e214e9, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0214ec, SUC_NE_lexicon_tags_25 },
    { 0x390614ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43414ef, SUC_NE_lexicon_tags_2 },
    { 0xd79a14f0, SUC_NE_lexicon_tags_2 },
    { 0x21f414f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b6214f3, SUC_NE_lexicon_tags_2 },
    { 0xfdb414f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10b214f6, SUC_NE_lexicon_tags_2 },
    { 0x4f0814f7, SUC_NE_lexicon_tags_2 },
    { 0xd10014f8, SUC_NE_lexicon_tags_4 },
    { 0xf30414f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110a14ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a61502, SUC_NE_lexicon_tags_2 },
    { 0x2c0e1502, SUC_NE_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270e1507, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dec150a, SUC_NE_lexicon_tags_4 },
    { 0xcb6150b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bc150e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13701510, SUC_NE_lexicon_tags_2 },
    { 0x32d81511, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x568c1513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489a1520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef41523, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a1526, SUC_NE_lexicon_tags_3 },
    { 0x6a5e1527, SUC_NE_lexicon_tags_2 },
    { 0xfbe1528, SUC_NE_lexicon_tags_2 },
    { 0xa9721529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd688152b, SUC_NE_lexicon_tags_2 },
    { 0xb83a152c, SUC_NE_lexicon_tags_2 },
    { 0xdf86152d, SUC_NE_lexicon_tags_2 },
    { 0xc258152e, SUC_NE_lexicon_tags_2 },
    { 0xfc6e152f, SUC_NE_lexicon_tags_2 },
    { 0xfa0e1530, SUC_NE_lexicon_tags_2 },
    { 0x742c152e, SUC_NE_lexicon_tags_2 },
    { 0xd96152d, SUC_NE_lexicon_tags_2 },
    { 0x8fce1533, SUC_NE_lexicon_tags_2 },
    { 0x7ae21533, SUC_NE_lexicon_tags_4 },
    { 0x31361535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e81538, SUC_NE_lexicon_tags_2 },
    { 0x87da1538, SUC_NE_lexicon_tags_2 },
    { 0x54b01539, SUC_NE_lexicon_tags_2 },
    { 0x9db4153b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98f4153d, SUC_NE_lexicon_tags_2 },
    { 0x7696153d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ba1541, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8e1546, SUC_NE_lexicon_tags_2 },
    { 0xb1821546, SUC_NE_lexicon_tags_2 },
    { 0xd89c1547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c02154a, SUC_NE_lexicon_tags_2 },
    { 0xfc10154b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60154f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7a1553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62041555, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7de1557, SUC_NE_lexicon_tags_2 },
    { 0xc2a61558, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0c155c, SUC_NE_lexicon_tags_87 },
    { 0x4ac4155d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5901561, SUC_NE_lexicon_tags_2 },
    { 0x98ce1562, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fe1565, SUC_NE_lexicon_tags_2 },
    { 0x3aee1565, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3710156b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5e1570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e181572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836c1577, SUC_NE_lexicon_tags_2 },
    { 0x2c421578, SUC_NE_lexicon_tags_19 },
    { 0xd1da1577, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bc157e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5e1581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90881588, SUC_NE_lexicon_tags_2 },
    { 0x70021589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe46a158b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf41592, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36901595, SUC_NE_lexicon_tags_2 },
    { 0x9d2c1596, SUC_NE_lexicon_tags_2 },
    { 0x68181597, SUC_NE_lexicon_tags_2 },
    { 0xd8681598, SUC_NE_lexicon_tags_162 },
    { 0xb2a41595, SUC_NE_lexicon_tags_2 },
    { 0x13741597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bba159c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x787e159e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0815a4, SUC_NE_lexicon_tags_2 },
    { 0xe42a15a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964615ae, SUC_NE_lexicon_tags_2 },
    { 0xdb7a15af, SUC_NE_lexicon_tags_2 },
    { 0x29c015b0, SUC_NE_lexicon_tags_7 },
    { 0x495215b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5415b4, SUC_NE_lexicon_tags_2 },
    { 0x2b1215b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34615c4, SUC_NE_lexicon_tags_2 },
    { 0x4db415c5, SUC_NE_lexicon_tags_2 },
    { 0x40a615c6, SUC_NE_lexicon_tags_2 },
    { 0xf44015c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec15ce, SUC_NE_lexicon_tags_2 },
    { 0x161015ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7215d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49015d7, SUC_NE_lexicon_tags_19 },
    { 0x2e4415d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbe15dc, SUC_NE_lexicon_tags_2 },
    { 0xd3dc15dd, SUC_NE_lexicon_tags_2 },
    { 0x7e9a15dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb815e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a015e5, SUC_NE_lexicon_tags_2 },
    { 0x951e15e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cce15e8, SUC_NE_lexicon_tags_2 },
    { 0xd37015e8, SUC_NE_lexicon_tags_2 },
    { 0x45a615ea, SUC_NE_lexicon_tags_2 },
    { 0xa2fe15ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87815ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621615f6, SUC_NE_lexicon_tags_2 },
    { 0x59ee15f6, SUC_NE_lexicon_tags_2 },
    { 0xbf8215f6, SUC_NE_lexicon_tags_2 },
    { 0x742415f9, SUC_NE_lexicon_tags_2 },
    { 0x697415f9, SUC_NE_lexicon_tags_2 },
    { 0x4af415f6, SUC_NE_lexicon_tags_2 },
    { 0x4e1015fc, SUC_NE_lexicon_tags_2 },
    { 0x51ca15fd, SUC_NE_lexicon_tags_2 },
    { 0x86b415fd, SUC_NE_lexicon_tags_2 },
    { 0x84e015fd, SUC_NE_lexicon_tags_2 },
    { 0x315e1600, SUC_NE_lexicon_tags_2 },
    { 0x9ee41600, SUC_NE_lexicon_tags_2 },
    { 0x1de215fb, SUC_NE_lexicon_tags_2 },
    { 0xb0941603, SUC_NE_lexicon_tags_2 },
    { 0x3d201604, SUC_NE_lexicon_tags_2 },
    { 0x64a61605, SUC_NE_lexicon_tags_2 },
    { 0xeb8e1606, SUC_NE_lexicon_tags_2 },
    { 0x7db61606, SUC_NE_lexicon_tags_108 },
    { 0, NULL },
    { 0xf0841609, SUC_NE_lexicon_tags_2 },
    { 0xf492160a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf8c160c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ae8160e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52261610, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87481612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dc1616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa7a1618, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6728161a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf36161c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4161f, SUC_NE_lexicon_tags_2 },
    { 0xa34a1620, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x52621623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9e1627, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba162d, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a1637, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aa163c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3372163e, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdc1641, SUC_NE_lexicon_tags_3 },
    { 0x59b41642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3a1647, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x272c164a, SUC_NE_lexicon_tags_2 },
    { 0xda2c164b, SUC_NE_lexicon_tags_2 },
    { 0xd824164c, SUC_NE_lexicon_tags_2 },
    { 0xdf0a164c, SUC_NE_lexicon_tags_2 },
    { 0x8d16164e, SUC_NE_lexicon_tags_2 },
    { 0xc2a4164f, SUC_NE_lexicon_tags_2 },
    { 0xaff01650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf881652, SUC_NE_lexicon_tags_2 },
    { 0x514a1653, SUC_NE_lexicon_tags_2 },
    { 0xc6a01653, SUC_NE_lexicon_tags_2 },
    { 0x3cf41654, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f61658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x350e165a, SUC_NE_lexicon_tags_2 },
    { 0x12c0165b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ca165f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f81664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7e1668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b0166e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd61671, SUC_NE_lexicon_tags_2 },
    { 0x684a1671, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b01675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb948167c, SUC_NE_lexicon_tags_2 },
    { 0x4024167d, SUC_NE_lexicon_tags_2 },
    { 0xef72167e, SUC_NE_lexicon_tags_12 },
    { 0xca24167d, SUC_NE_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a1683, SUC_NE_lexicon_tags_2 },
    { 0x9b8c1684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40c1687, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88e168a, SUC_NE_lexicon_tags_2 },
    { 0x26d4168b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8341691, SUC_NE_lexicon_tags_2 },
    { 0x229c1692, SUC_NE_lexicon_tags_2 },
    { 0x8d9c1693, SUC_NE_lexicon_tags_2 },
    { 0x5761693, SUC_NE_lexicon_tags_1 },
    { 0x90a1695, SUC_NE_lexicon_tags_2 },
    { 0x7ba1696, SUC_NE_lexicon_tags_2 },
    { 0xd75c1697, SUC_NE_lexicon_tags_2 },
    { 0x82521698, SUC_NE_lexicon_tags_34 },
    { 0xe1ca1698, SUC_NE_lexicon_tags_2 },
    { 0xaffe1691, SUC_NE_lexicon_tags_7 },
    { 0xb48c1699, SUC_NE_lexicon_tags_2 },
    { 0x6678169c, SUC_NE_lexicon_tags_2 },
    { 0x956c169c, SUC_NE_lexicon_tags_5 },
    { 0x8ecc169d, SUC_NE_lexicon_tags_2 },
    { 0xa382169f, SUC_NE_lexicon_tags_274 },
    { 0x739416a0, SUC_NE_lexicon_tags_7 },
    { 0x687616a0, SUC_NE_lexicon_tags_2 },
    { 0x97a6169c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cca16a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e16a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc16ab, SUC_NE_lexicon_tags_2 },
    { 0xf35016ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fc16b4, SUC_NE_lexicon_tags_7 },
    { 0x6fea16b4, SUC_NE_lexicon_tags_2 },
    { 0xe42616b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd94e16b8, SUC_NE_lexicon_tags_2 },
    { 0x77da16b9, SUC_NE_lexicon_tags_2 },
    { 0xaea16ba, SUC_NE_lexicon_tags_2 },
    { 0x8a5c16bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725016c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7016c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941c16c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c416ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x901416d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7a16d3, SUC_NE_lexicon_tags_2 },
    { 0x9aa616d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x990e16d6, SUC_NE_lexicon_tags_2 },
    { 0xbff416d7, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c616da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913416de, SUC_NE_lexicon_tags_2 },
    { 0x32d416df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7aae16e1, SUC_NE_lexicon_tags_2 },
    { 0x4fe216e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x308216e4, SUC_NE_lexicon_tags_3 },
    { 0xf1ba16e5, SUC_NE_lexicon_tags_31 },
    { 0x706e16e5, SUC_NE_lexicon_tags_2 },
    { 0x931816e7, SUC_NE_lexicon_tags_2 },
    { 0xc9a16e8, SUC_NE_lexicon_tags_2 },
    { 0xd64416e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8ec16eb, SUC_NE_lexicon_tags_2 },
    { 0x15e816ec, SUC_NE_lexicon_tags_2 },
    { 0x2fd016ed, SUC_NE_lexicon_tags_17 },
    { 0x8ff616ee, SUC_NE_lexicon_tags_2 },
    { 0x533c16ef, SUC_NE_lexicon_tags_2 },
    { 0x65d416ef, SUC_NE_lexicon_tags_2 },
    { 0x475a16f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94616f4, SUC_NE_lexicon_tags_7 },
    { 0xe21e16f5, SUC_NE_lexicon_tags_2 },
    { 0x3e9816f5, SUC_NE_lexicon_tags_2 },
    { 0x5b9616f7, SUC_NE_lexicon_tags_2 },
    { 0x895e16f5, SUC_NE_lexicon_tags_2 },
    { 0xa3f216f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x661816fb, SUC_NE_lexicon_tags_2 },
    { 0xa3c616fc, SUC_NE_lexicon_tags_2 },
    { 0x681e16fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d9016ff, SUC_NE_lexicon_tags_8 },
    { 0xee7e1700, SUC_NE_lexicon_tags_2 },
    { 0x2e521701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdc1706, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a170d, SUC_NE_lexicon_tags_11 },
    { 0x604c170e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe1714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be61718, SUC_NE_lexicon_tags_2 },
    { 0x3f201718, SUC_NE_lexicon_tags_2 },
    { 0xb1ce171a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6c171e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12081724, SUC_NE_lexicon_tags_2 },
    { 0x1bf81725, SUC_NE_lexicon_tags_3 },
    { 0x36461724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x296e1728, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e36172b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa69a172d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bca1731, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f01734, SUC_NE_lexicon_tags_2 },
    { 0xa30c1735, SUC_NE_lexicon_tags_2 },
    { 0xeb0c1736, SUC_NE_lexicon_tags_2 },
    { 0x3f081736, SUC_NE_lexicon_tags_2 },
    { 0xd901737, SUC_NE_lexicon_tags_2 },
    { 0xcd3e1739, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c8173d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba21740, SUC_NE_lexicon_tags_2 },
    { 0xe5ee1741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fa1746, SUC_NE_lexicon_tags_7 },
    { 0xb7881747, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c98174c, SUC_NE_lexicon_tags_2 },
    { 0x75a4174d, SUC_NE_lexicon_tags_2 },
    { 0x72f0174d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28101750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea561754, SUC_NE_lexicon_tags_2 },
    { 0x943a1755, SUC_NE_lexicon_tags_2 },
    { 0x475c1754, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x636e1758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a76175b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cc175e, SUC_NE_lexicon_tags_4 },
    { 0xe5f0175e, SUC_NE_lexicon_tags_49 },
    { 0x2f10175e, SUC_NE_lexicon_tags_2 },
    { 0xaf341761, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccc41763, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a5a1765, SUC_NE_lexicon_tags_2 },
    { 0xb99a1766, SUC_NE_lexicon_tags_2 },
    { 0xf7be1766, SUC_NE_lexicon_tags_2 },
    { 0xdc3e1766, SUC_NE_lexicon_tags_2 },
    { 0x60741767, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e176c, SUC_NE_lexicon_tags_2 },
    { 0x58a0176d, SUC_NE_lexicon_tags_2 },
    { 0xb9a2176e, SUC_NE_lexicon_tags_7 },
    { 0x8276176f, SUC_NE_lexicon_tags_2 },
    { 0xd6d8176f, SUC_NE_lexicon_tags_2 },
    { 0x589c1771, SUC_NE_lexicon_tags_24 },
    { 0x251e1771, SUC_NE_lexicon_tags_2 },
    { 0xa6241773, SUC_NE_lexicon_tags_2 },
    { 0xc1961774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29b61776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4a177a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x140177c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4b8177e, SUC_NE_lexicon_tags_2 },
    { 0x71e0177f, SUC_NE_lexicon_tags_3 },
    { 0xa5f01780, SUC_NE_lexicon_tags_75 },
    { 0x13da1781, SUC_NE_lexicon_tags_2 },
    { 0x319c1781, SUC_NE_lexicon_tags_7 },
    { 0x43d21782, SUC_NE_lexicon_tags_2 },
    { 0x85061783, SUC_NE_lexicon_tags_2 },
    { 0x7f86177e, SUC_NE_lexicon_tags_30 },
    { 0xbc201786, SUC_NE_lexicon_tags_2 },
    { 0x8f401786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10721789, SUC_NE_lexicon_tags_2 },
    { 0x6a2c178a, SUC_NE_lexicon_tags_2 },
    { 0xc4c4178b, SUC_NE_lexicon_tags_2 },
    { 0xd094178c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41d4178e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x438e1790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa81793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf81799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ee179c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6a17a0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x746217a3, SUC_NE_lexicon_tags_2 },
    { 0xdc8e17a4, SUC_NE_lexicon_tags_4 },
    { 0x3b1a17a5, SUC_NE_lexicon_tags_2 },
    { 0x646017a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a017b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd217bb, SUC_NE_lexicon_tags_2 },
    { 0xa83017bc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11617c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b9e17c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x342417ca, SUC_NE_lexicon_tags_153 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d017dd, SUC_NE_lexicon_tags_2 },
    { 0x4e3c17de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc617e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabac17e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27b017e5, SUC_NE_lexicon_tags_2 },
    { 0x63e17e6, SUC_NE_lexicon_tags_2 },
    { 0x283817e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767017ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b417ef, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x227017f1, SUC_NE_lexicon_tags_2 },
    { 0xe017f2, SUC_NE_lexicon_tags_2 },
    { 0x774017f2, SUC_NE_lexicon_tags_2 },
    { 0xbea417f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc17f7, SUC_NE_lexicon_tags_2 },
    { 0x300817f8, SUC_NE_lexicon_tags_2 },
    { 0x9ac17f9, SUC_NE_lexicon_tags_2 },
    { 0x3eca17fa, SUC_NE_lexicon_tags_2 },
    { 0x396817fa, SUC_NE_lexicon_tags_2 },
    { 0xeaba17fa, SUC_NE_lexicon_tags_2 },
    { 0x50fc17f9, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x874e1800, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72941803, SUC_NE_lexicon_tags_30 },
    { 0xb88a1803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x959a1806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ca1809, SUC_NE_lexicon_tags_2 },
    { 0x87b6180a, SUC_NE_lexicon_tags_2 },
    { 0xe05c1809, SUC_NE_lexicon_tags_2 },
    { 0xfae6180c, SUC_NE_lexicon_tags_2 },
    { 0xe764180d, SUC_NE_lexicon_tags_2 },
    { 0xdc3c180e, SUC_NE_lexicon_tags_2 },
    { 0x19ae180f, SUC_NE_lexicon_tags_2 },
    { 0x8928180e, SUC_NE_lexicon_tags_3 },
    { 0x4bf0180c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab56181a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa4a2181c, SUC_NE_lexicon_tags_2 },
    { 0xfde2181d, SUC_NE_lexicon_tags_1 },
    { 0xd12e181d, SUC_NE_lexicon_tags_24 },
    { 0xc4f8181f, SUC_NE_lexicon_tags_2 },
    { 0xf3d21820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da1824, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16241826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aa182e, SUC_NE_lexicon_tags_12 },
    { 0x6a4c182f, SUC_NE_lexicon_tags_2 },
    { 0xf84e1830, SUC_NE_lexicon_tags_2 },
    { 0x5f361830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab3c1833, SUC_NE_lexicon_tags_2 },
    { 0xeed61834, SUC_NE_lexicon_tags_2 },
    { 0xe0881833, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53a41837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30681839, SUC_NE_lexicon_tags_2 },
    { 0xc54e183a, SUC_NE_lexicon_tags_19 },
    { 0xed5c183b, SUC_NE_lexicon_tags_2 },
    { 0x6644183c, SUC_NE_lexicon_tags_2 },
    { 0xaf3a183c, SUC_NE_lexicon_tags_2 },
    { 0xccfc183c, SUC_NE_lexicon_tags_2 },
    { 0x74c2183b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46e1842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f281845, SUC_NE_lexicon_tags_2 },
    { 0xf3e01846, SUC_NE_lexicon_tags_2 },
    { 0x72421845, SUC_NE_lexicon_tags_59 },
    { 0x60181845, SUC_NE_lexicon_tags_2 },
    { 0x9b7e1849, SUC_NE_lexicon_tags_2 },
    { 0x22221847, SUC_NE_lexicon_tags_2 },
    { 0xc70a184a, SUC_NE_lexicon_tags_2 },
    { 0x12c6184c, SUC_NE_lexicon_tags_2 },
    { 0x8e181846, SUC_NE_lexicon_tags_2 },
    { 0xad4c184a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4e01850, SUC_NE_lexicon_tags_2 },
    { 0xb5f81851, SUC_NE_lexicon_tags_2 },
    { 0x62ac1852, SUC_NE_lexicon_tags_2 },
    { 0xa1c1853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a221855, SUC_NE_lexicon_tags_2 },
    { 0x69fe1856, SUC_NE_lexicon_tags_2 },
    { 0x64d41857, SUC_NE_lexicon_tags_2 },
    { 0x24ca1858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a1862, SUC_NE_lexicon_tags_2 },
    { 0x9b7c1863, SUC_NE_lexicon_tags_343 },
    { 0x3c061864, SUC_NE_lexicon_tags_2 },
    { 0x523e1863, SUC_NE_lexicon_tags_2 },
    { 0x1e921866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cc1868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84a186c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280e1870, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d721878, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddba187b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6ea187d, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30861881, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c081885, SUC_NE_lexicon_tags_2 },
    { 0xb7d81886, SUC_NE_lexicon_tags_2 },
    { 0x4b741887, SUC_NE_lexicon_tags_2 },
    { 0x37661888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa2188c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c188f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5461892, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f61899, SUC_NE_lexicon_tags_2 },
    { 0xd20c189a, SUC_NE_lexicon_tags_2 },
    { 0xae72189a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885418a0, SUC_NE_lexicon_tags_2 },
    { 0xe19418a1, SUC_NE_lexicon_tags_2 },
    { 0xb81c18a1, SUC_NE_lexicon_tags_2 },
    { 0x14a618a3, SUC_NE_lexicon_tags_2 },
    { 0x38f618a3, SUC_NE_lexicon_tags_2 },
    { 0x3bcc18a5, SUC_NE_lexicon_tags_2 },
    { 0xb0aa18a6, SUC_NE_lexicon_tags_2 },
    { 0xfc3c18a3, SUC_NE_lexicon_tags_4 },
    { 0x95b818a1, SUC_NE_lexicon_tags_2 },
    { 0xc9be18a9, SUC_NE_lexicon_tags_2 },
    { 0x9a2618a9, SUC_NE_lexicon_tags_2 },
    { 0x13e618a9, SUC_NE_lexicon_tags_2 },
    { 0xd1618ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8618b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x535618b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f2a18b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c18ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8e18bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdae18c3, SUC_NE_lexicon_tags_409 },
    { 0, NULL },
    { 0x645018c5, SUC_NE_lexicon_tags_14 },
    { 0x75418c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48818d5, SUC_NE_lexicon_tags_2 },
    { 0xa11e18d5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2818df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf36018e1, SUC_NE_lexicon_tags_2 },
    { 0xa73e18e2, SUC_NE_lexicon_tags_2 },
    { 0x224618e3, SUC_NE_lexicon_tags_18 },
    { 0xff2218e3, SUC_NE_lexicon_tags_2 },
    { 0x65ec18e5, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc18eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e418ee, SUC_NE_lexicon_tags_2 },
    { 0xc3ac18ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc8818f1, SUC_NE_lexicon_tags_2 },
    { 0x5a7618f1, SUC_NE_lexicon_tags_34 },
    { 0x1cc218f1, SUC_NE_lexicon_tags_2 },
    { 0xa37618f2, SUC_NE_lexicon_tags_2 },
    { 0x1f3e18f5, SUC_NE_lexicon_tags_2 },
    { 0x2f1618f6, SUC_NE_lexicon_tags_2 },
    { 0x6e6618f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4218f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4c18fd, SUC_NE_lexicon_tags_2 },
    { 0x9d4218fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6a1903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f401906, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98a41908, SUC_NE_lexicon_tags_2 },
    { 0xf2e01908, SUC_NE_lexicon_tags_2 },
    { 0xab32190a, SUC_NE_lexicon_tags_2 },
    { 0x307e190b, SUC_NE_lexicon_tags_2 },
    { 0xaacc190b, SUC_NE_lexicon_tags_2 },
    { 0xc692190d, SUC_NE_lexicon_tags_2 },
    { 0x14d4190e, SUC_NE_lexicon_tags_17 },
    { 0x5c5e190f, SUC_NE_lexicon_tags_2 },
    { 0x361c1910, SUC_NE_lexicon_tags_2 },
    { 0xcf381908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508e1916, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xecc81918, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac14191b, SUC_NE_lexicon_tags_2 },
    { 0x1a5e191b, SUC_NE_lexicon_tags_4 },
    { 0x36f8191d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b61925, SUC_NE_lexicon_tags_2 },
    { 0x2fa61926, SUC_NE_lexicon_tags_2 },
    { 0xe9c81927, SUC_NE_lexicon_tags_2 },
    { 0x33a1927, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a192c, SUC_NE_lexicon_tags_2 },
    { 0xb80c192d, SUC_NE_lexicon_tags_2 },
    { 0x6270192e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x893e1930, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35321933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a61936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1c193a, SUC_NE_lexicon_tags_2 },
    { 0x7a0a193b, SUC_NE_lexicon_tags_2 },
    { 0x8634193c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6864193f, SUC_NE_lexicon_tags_2 },
    { 0xb7701940, SUC_NE_lexicon_tags_2 },
    { 0x95f21940, SUC_NE_lexicon_tags_2 },
    { 0xca4e1942, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb34c1944, SUC_NE_lexicon_tags_2 },
    { 0x288a1945, SUC_NE_lexicon_tags_2 },
    { 0x183a1946, SUC_NE_lexicon_tags_19 },
    { 0xc2961945, SUC_NE_lexicon_tags_2 },
    { 0xf3921947, SUC_NE_lexicon_tags_2 },
    { 0x997e1949, SUC_NE_lexicon_tags_2 },
    { 0x157e1947, SUC_NE_lexicon_tags_2 },
    { 0x66381947, SUC_NE_lexicon_tags_2 },
    { 0x2a261949, SUC_NE_lexicon_tags_2 },
    { 0x2f8c194a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bf2194f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cf21951, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c41955, SUC_NE_lexicon_tags_2 },
    { 0x6ccc1955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7e195c, SUC_NE_lexicon_tags_2 },
    { 0x2d72195d, SUC_NE_lexicon_tags_2 },
    { 0x746195c, SUC_NE_lexicon_tags_19 },
    { 0x9dd0195f, SUC_NE_lexicon_tags_2 },
    { 0x6f0a195f, SUC_NE_lexicon_tags_2 },
    { 0x14261960, SUC_NE_lexicon_tags_2 },
    { 0x9cd8195c, SUC_NE_lexicon_tags_2 },
    { 0x79361963, SUC_NE_lexicon_tags_2 },
    { 0x4cac1960, SUC_NE_lexicon_tags_2 },
    { 0xf3f21965, SUC_NE_lexicon_tags_3 },
    { 0xad941965, SUC_NE_lexicon_tags_2 },
    { 0x66881967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d74196a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f0196d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48de1970, SUC_NE_lexicon_tags_2 },
    { 0x675a1971, SUC_NE_lexicon_tags_2 },
    { 0xcc781970, SUC_NE_lexicon_tags_2 },
    { 0x84fe1973, SUC_NE_lexicon_tags_2 },
    { 0xdd561970, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16a01976, SUC_NE_lexicon_tags_2 },
    { 0x7b0a1976, SUC_NE_lexicon_tags_2 },
    { 0x49501978, SUC_NE_lexicon_tags_2 },
    { 0xa01a1978, SUC_NE_lexicon_tags_2 },
    { 0x328e197a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe8a197c, SUC_NE_lexicon_tags_2 },
    { 0x2ef6197d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x510c1980, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb61984, SUC_NE_lexicon_tags_2 },
    { 0x326e1984, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a361989, SUC_NE_lexicon_tags_7 },
    { 0x9b421989, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b04198d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9540198f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81da1991, SUC_NE_lexicon_tags_2 },
    { 0x37bc1992, SUC_NE_lexicon_tags_2 },
    { 0x8bb41992, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d521995, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8801997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d2199a, SUC_NE_lexicon_tags_4 },
    { 0xb616199a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe199e, SUC_NE_lexicon_tags_2 },
    { 0x8b9e199e, SUC_NE_lexicon_tags_2 },
    { 0x4f5a199f, SUC_NE_lexicon_tags_2 },
    { 0x116e199e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x299419a3, SUC_NE_lexicon_tags_2 },
    { 0x319a19a3, SUC_NE_lexicon_tags_2 },
    { 0xdf1e19a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed7619a7, SUC_NE_lexicon_tags_2 },
    { 0xead819a8, SUC_NE_lexicon_tags_2 },
    { 0x13ba19a9, SUC_NE_lexicon_tags_2 },
    { 0xff5419a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49c19af, SUC_NE_lexicon_tags_2 },
    { 0xc1d219af, SUC_NE_lexicon_tags_2 },
    { 0xe2c019b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x911419b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x355619b6, SUC_NE_lexicon_tags_2 },
    { 0xe5f419b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27819ba, SUC_NE_lexicon_tags_2 },
    { 0xbb1819ba, SUC_NE_lexicon_tags_4 },
    { 0x3e4a19bc, SUC_NE_lexicon_tags_17 },
    { 0x80a419bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0219c1, SUC_NE_lexicon_tags_2 },
    { 0x485a19c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x190019c5, SUC_NE_lexicon_tags_2 },
    { 0x3c6219c6, SUC_NE_lexicon_tags_2 },
    { 0x18f019c7, SUC_NE_lexicon_tags_263 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617a19ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdca419d0, SUC_NE_lexicon_tags_2 },
    { 0xe0c619d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x489c19d3, SUC_NE_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc19de, SUC_NE_lexicon_tags_2 },
    { 0xb2e219de, SUC_NE_lexicon_tags_2 },
    { 0x340819e0, SUC_NE_lexicon_tags_2 },
    { 0x491019e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3e19e4, SUC_NE_lexicon_tags_2 },
    { 0xb3a219e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9219e7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xafe219e9, SUC_NE_lexicon_tags_2 },
    { 0x810019e9, SUC_NE_lexicon_tags_2 },
    { 0x900019ea, SUC_NE_lexicon_tags_2 },
    { 0x2e7e19ea, SUC_NE_lexicon_tags_2 },
    { 0xa4e219ed, SUC_NE_lexicon_tags_2 },
    { 0xa22819ee, SUC_NE_lexicon_tags_2 },
    { 0xfc9419ed, SUC_NE_lexicon_tags_17 },
    { 0x3f0219eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c5019f2, SUC_NE_lexicon_tags_2 },
    { 0x427a19f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358419fb, SUC_NE_lexicon_tags_2 },
    { 0x26a19fc, SUC_NE_lexicon_tags_4 },
    { 0xa04619fc, SUC_NE_lexicon_tags_2 },
    { 0xf73819fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd21a02, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c61a05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba1a09, SUC_NE_lexicon_tags_2 },
    { 0x358e1a09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26701a0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb08c1a0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6221a11, SUC_NE_lexicon_tags_2 },
    { 0xcce1a12, SUC_NE_lexicon_tags_2 },
    { 0x7e41a13, SUC_NE_lexicon_tags_4 },
    { 0xe5ec1a14, SUC_NE_lexicon_tags_2 },
    { 0xcb641a15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92721a18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccf81a1a, SUC_NE_lexicon_tags_2 },
    { 0x93481a1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe521a1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe41a23, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c61a27, SUC_NE_lexicon_tags_2 },
    { 0x1de81a28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9c1a2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e1a2f, SUC_NE_lexicon_tags_2 },
    { 0xb45c1a2f, SUC_NE_lexicon_tags_2 },
    { 0x71c41a31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0221a33, SUC_NE_lexicon_tags_2 },
    { 0x37081a34, SUC_NE_lexicon_tags_2 },
    { 0x59621a35, SUC_NE_lexicon_tags_2 },
    { 0x351a1a35, SUC_NE_lexicon_tags_2 },
    { 0x522e1a37, SUC_NE_lexicon_tags_2 },
    { 0x4381a38, SUC_NE_lexicon_tags_5 },
    { 0xf5a01a39, SUC_NE_lexicon_tags_2 },
    { 0xe1701a37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x813c1a3d, SUC_NE_lexicon_tags_13 },
    { 0x371a1a3e, SUC_NE_lexicon_tags_2 },
    { 0xb4f81a3d, SUC_NE_lexicon_tags_2 },
    { 0x8261a40, SUC_NE_lexicon_tags_2 },
    { 0xd6861a41, SUC_NE_lexicon_tags_2 },
    { 0x8aa21a42, SUC_NE_lexicon_tags_2 },
    { 0x2d761a43, SUC_NE_lexicon_tags_2 },
    { 0x1c1a1a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b8c1a46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50e1a49, SUC_NE_lexicon_tags_2 },
    { 0x4961a49, SUC_NE_lexicon_tags_2 },
    { 0x51481a4b, SUC_NE_lexicon_tags_2 },
    { 0xc1aa1a4c, SUC_NE_lexicon_tags_2 },
    { 0x89ec1a4d, SUC_NE_lexicon_tags_3 },
    { 0x34f01a4e, SUC_NE_lexicon_tags_13 },
    { 0xb6061a4f, SUC_NE_lexicon_tags_4 },
    { 0xeb7a1a4f, SUC_NE_lexicon_tags_2 },
    { 0x31621a4f, SUC_NE_lexicon_tags_2 },
    { 0xb2fe1a51, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9261a55, SUC_NE_lexicon_tags_2 },
    { 0x48861a56, SUC_NE_lexicon_tags_2 },
    { 0x140e1a57, SUC_NE_lexicon_tags_2 },
    { 0xce6c1a58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdede1a5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde981a5e, SUC_NE_lexicon_tags_7 },
    { 0xdb901a5f, SUC_NE_lexicon_tags_2 },
    { 0x55201a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x401c1a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ec1a68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b781a6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c141a6e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xed2a1a70, SUC_NE_lexicon_tags_2 },
    { 0xabc1a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399e1a75, SUC_NE_lexicon_tags_2 },
    { 0x3fd41a76, SUC_NE_lexicon_tags_2 },
    { 0x613e1a75, SUC_NE_lexicon_tags_2 },
    { 0xca401a78, SUC_NE_lexicon_tags_17 },
    { 0x8de81a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b41a7f, SUC_NE_lexicon_tags_2 },
    { 0xfa1c1a7f, SUC_NE_lexicon_tags_2 },
    { 0x2b7a1a7f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea01a84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b2e1a86, SUC_NE_lexicon_tags_4 },
    { 0xdb3c1a87, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x93e41a89, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x328a1a8c, SUC_NE_lexicon_tags_2 },
    { 0xcb721a8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffa1a90, SUC_NE_lexicon_tags_2 },
    { 0xb54a1a91, SUC_NE_lexicon_tags_2 },
    { 0xe84a1a91, SUC_NE_lexicon_tags_2 },
    { 0xece01a91, SUC_NE_lexicon_tags_2 },
    { 0x4ca41a91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ae1a97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e7a1a99, SUC_NE_lexicon_tags_3 },
    { 0x8561a9a, SUC_NE_lexicon_tags_2 },
    { 0xbca21a9b, SUC_NE_lexicon_tags_2 },
    { 0x74a81a9c, SUC_NE_lexicon_tags_2 },
    { 0x552a1a9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92c1aa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0461aa5, SUC_NE_lexicon_tags_2 },
    { 0x9eda1aa5, SUC_NE_lexicon_tags_2 },
    { 0xc3541aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf601aa9, SUC_NE_lexicon_tags_2 },
    { 0x22981aa9, SUC_NE_lexicon_tags_2 },
    { 0xc17c1aab, SUC_NE_lexicon_tags_2 },
    { 0x13c01aac, SUC_NE_lexicon_tags_2 },
    { 0xb6541aab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3141aaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c21ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f321abc, SUC_NE_lexicon_tags_2 },
    { 0x9ebe1abd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97901ac0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d5a1ac2, SUC_NE_lexicon_tags_2 },
    { 0x3dc1ac3, SUC_NE_lexicon_tags_2 },
    { 0x4a1c1ac4, SUC_NE_lexicon_tags_2 },
    { 0x8e521ac4, SUC_NE_lexicon_tags_2 },
    { 0x6b2e1ac4, SUC_NE_lexicon_tags_2 },
    { 0xf6ea1ac3, SUC_NE_lexicon_tags_2 },
    { 0x6c9a1ac8, SUC_NE_lexicon_tags_2 },
    { 0x97e01ac6, SUC_NE_lexicon_tags_2 },
    { 0xf43c1aca, SUC_NE_lexicon_tags_2 },
    { 0xb42a1ac9, SUC_NE_lexicon_tags_17 },
    { 0x22d61ac2, SUC_NE_lexicon_tags_174 },
    { 0x655a1acd, SUC_NE_lexicon_tags_2 },
    { 0x45441ac5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba1ad2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a41ad5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70c21ad7, SUC_NE_lexicon_tags_2 },
    { 0x41cc1ad8, SUC_NE_lexicon_tags_2 },
    { 0xdbd81ad8, SUC_NE_lexicon_tags_2 },
    { 0x1e1c1ada, SUC_NE_lexicon_tags_2 },
    { 0xfdb81adb, SUC_NE_lexicon_tags_2 },
    { 0x5bdc1adb, SUC_NE_lexicon_tags_2 },
    { 0xc0241add, SUC_NE_lexicon_tags_7 },
    { 0xcd421add, SUC_NE_lexicon_tags_11 },
    { 0x969c1adf, SUC_NE_lexicon_tags_2 },
    { 0x47261ae0, SUC_NE_lexicon_tags_2 },
    { 0x9e601ad9, SUC_NE_lexicon_tags_2 },
    { 0x7f761ae0, SUC_NE_lexicon_tags_4 },
    { 0x323e1ae3, SUC_NE_lexicon_tags_2 },
    { 0xde8e1adb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30981ae6, SUC_NE_lexicon_tags_30 },
    { 0x1e6e1ae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3201ae9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea1aef, SUC_NE_lexicon_tags_2 },
    { 0x3ab81aef, SUC_NE_lexicon_tags_2 },
    { 0x6f081af1, SUC_NE_lexicon_tags_7 },
    { 0xfe501af2, SUC_NE_lexicon_tags_2 },
    { 0xd6961af2, SUC_NE_lexicon_tags_2 },
    { 0x14561af4, SUC_NE_lexicon_tags_2 },
    { 0xfbe81af3, SUC_NE_lexicon_tags_2 },
    { 0x47e21af6, SUC_NE_lexicon_tags_2 },
    { 0xf7c81af1, SUC_NE_lexicon_tags_4 },
    { 0xd9a1af8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x761c1afb, SUC_NE_lexicon_tags_2 },
    { 0x4aca1afc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41341afe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde181b01, SUC_NE_lexicon_tags_2 },
    { 0x20161b01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d41b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafac1b09, SUC_NE_lexicon_tags_2 },
    { 0x8b241b09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20ec1b0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e4c1b0e, SUC_NE_lexicon_tags_2 },
    { 0x8b721b0f, SUC_NE_lexicon_tags_2 },
    { 0xf8821b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d881b17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b761b1b, SUC_NE_lexicon_tags_2 },
    { 0xc5681b1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d361b24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fe1b27, SUC_NE_lexicon_tags_2 },
    { 0x451e1b27, SUC_NE_lexicon_tags_2 },
    { 0xf8c1b27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e541b2c, SUC_NE_lexicon_tags_2 },
    { 0x3eca1b2d, SUC_NE_lexicon_tags_2 },
    { 0x3bce1b2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x829a1b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31321b32, SUC_NE_lexicon_tags_2 },
    { 0x19c21b33, SUC_NE_lexicon_tags_2 },
    { 0x36e21b34, SUC_NE_lexicon_tags_2 },
    { 0xa0f41b35, SUC_NE_lexicon_tags_1 },
    { 0xc5901b34, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a81b39, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51c1b40, SUC_NE_lexicon_tags_2 },
    { 0xf37c1b40, SUC_NE_lexicon_tags_2 },
    { 0xfc081b42, SUC_NE_lexicon_tags_2 },
    { 0xf51e1b43, SUC_NE_lexicon_tags_143 },
    { 0x6f6c1b43, SUC_NE_lexicon_tags_7 },
    { 0xe41a1b44, SUC_NE_lexicon_tags_2 },
    { 0xc5d61b45, SUC_NE_lexicon_tags_2 },
    { 0xe7781b44, SUC_NE_lexicon_tags_2 },
    { 0x18861b48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ee1b4d, SUC_NE_lexicon_tags_13 },
    { 0x529c1b4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac21b52, SUC_NE_lexicon_tags_2 },
    { 0xf2e61b52, SUC_NE_lexicon_tags_2 },
    { 0xa4381b54, SUC_NE_lexicon_tags_2 },
    { 0xf2481b55, SUC_NE_lexicon_tags_32 },
    { 0xdce41b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41a1b59, SUC_NE_lexicon_tags_2 },
    { 0x9e101b5a, SUC_NE_lexicon_tags_2 },
    { 0xb99c1b5a, SUC_NE_lexicon_tags_2 },
    { 0x5afc1b5b, SUC_NE_lexicon_tags_2 },
    { 0x68c1b5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b01b65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d61b69, SUC_NE_lexicon_tags_1 },
    { 0x3c001b6a, SUC_NE_lexicon_tags_2 },
    { 0x78ac1b6a, SUC_NE_lexicon_tags_2 },
    { 0xfe161b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0e1b6f, SUC_NE_lexicon_tags_2 },
    { 0x38f21b70, SUC_NE_lexicon_tags_2 },
    { 0xa4f21b71, SUC_NE_lexicon_tags_6 },
    { 0x9ab61b72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16601b75, SUC_NE_lexicon_tags_2 },
    { 0xd9281b76, SUC_NE_lexicon_tags_2 },
    { 0x11641b75, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75921b7b, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x48a61b7d, SUC_NE_lexicon_tags_2 },
    { 0xffee1b7e, SUC_NE_lexicon_tags_2 },
    { 0xc98e1b7f, SUC_NE_lexicon_tags_2 },
    { 0x50141b7f, SUC_NE_lexicon_tags_2 },
    { 0x50c01b81, SUC_NE_lexicon_tags_4 },
    { 0x5d9a1b82, SUC_NE_lexicon_tags_18 },
    { 0x52b81b83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb61b86, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2c1b89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad141b8c, SUC_NE_lexicon_tags_2 },
    { 0x733e1b8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf181b97, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x217c1b99, SUC_NE_lexicon_tags_2 },
    { 0xe9ea1b9a, SUC_NE_lexicon_tags_2 },
    { 0x7f1a1b9b, SUC_NE_lexicon_tags_2 },
    { 0x8bae1b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x539c1b9f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0a1ba4, SUC_NE_lexicon_tags_2 },
    { 0xc57a1ba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13d41ba7, SUC_NE_lexicon_tags_2 },
    { 0x37b41ba8, SUC_NE_lexicon_tags_2 },
    { 0xc02a1ba9, SUC_NE_lexicon_tags_2 },
    { 0x60181baa, SUC_NE_lexicon_tags_2 },
    { 0x929c1bab, SUC_NE_lexicon_tags_2 },
    { 0x2f761bac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98da1baf, SUC_NE_lexicon_tags_8 },
    { 0xbf021bb0, SUC_NE_lexicon_tags_1 },
    { 0x86d81bb1, SUC_NE_lexicon_tags_2 },
    { 0xb06c1bb1, SUC_NE_lexicon_tags_8 },
    { 0xd421baf, SUC_NE_lexicon_tags_2 },
    { 0xe8ae1bb1, SUC_NE_lexicon_tags_7 },
    { 0x3b741bb5, SUC_NE_lexicon_tags_2 },
    { 0xd4b21bb6, SUC_NE_lexicon_tags_2 },
    { 0x2e881bb5, SUC_NE_lexicon_tags_2 },
    { 0x9a341bb5, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f61bbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca1bbe, SUC_NE_lexicon_tags_2 },
    { 0x9acc1bbe, SUC_NE_lexicon_tags_2 },
    { 0x8d061bc0, SUC_NE_lexicon_tags_2 },
    { 0x925c1bc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c81bc4, SUC_NE_lexicon_tags_2 },
    { 0xa861bc5, SUC_NE_lexicon_tags_2 },
    { 0xaeb41bc6, SUC_NE_lexicon_tags_0 },
    { 0x90841bc7, SUC_NE_lexicon_tags_2 },
    { 0x95621bc5, SUC_NE_lexicon_tags_2 },
    { 0x1bde1bc9, SUC_NE_lexicon_tags_2 },
    { 0x7c3a1bca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb41bce, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c01bd5, SUC_NE_lexicon_tags_2 },
    { 0xac3e1bd6, SUC_NE_lexicon_tags_2 },
    { 0xfb121bd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac961bdd, SUC_NE_lexicon_tags_2 },
    { 0x40181bde, SUC_NE_lexicon_tags_2 },
    { 0x13861bdf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a881be3, SUC_NE_lexicon_tags_49 },
    { 0x48ec1be4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf421be8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc381beb, SUC_NE_lexicon_tags_2 },
    { 0x5f641bec, SUC_NE_lexicon_tags_2 },
    { 0xa001bed, SUC_NE_lexicon_tags_2 },
    { 0x92181bee, SUC_NE_lexicon_tags_2 },
    { 0xa9fe1bef, SUC_NE_lexicon_tags_2 },
    { 0xbff61bee, SUC_NE_lexicon_tags_2 },
    { 0x7e641bed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29321bf4, SUC_NE_lexicon_tags_2 },
    { 0x816a1bf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c81bf9, SUC_NE_lexicon_tags_2 },
    { 0xa5081bfa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xa421bfc, SUC_NE_lexicon_tags_188 },
    { 0x82e61bfd, SUC_NE_lexicon_tags_2 },
    { 0x53941bfe, SUC_NE_lexicon_tags_2 },
    { 0x28f21bff, SUC_NE_lexicon_tags_2 },
    { 0xfa321bff, SUC_NE_lexicon_tags_2 },
    { 0x12921c01, SUC_NE_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c21c04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4c1c07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16e1c0a, SUC_NE_lexicon_tags_2 },
    { 0xd05e1c0b, SUC_NE_lexicon_tags_2 },
    { 0xddea1c0b, SUC_NE_lexicon_tags_2 },
    { 0x4c2e1c0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dac1c0f, SUC_NE_lexicon_tags_2 },
    { 0x44d81c10, SUC_NE_lexicon_tags_2 },
    { 0xb0ee1c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53a1c17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xedae1c19, SUC_NE_lexicon_tags_2 },
    { 0x86f81c1a, SUC_NE_lexicon_tags_2 },
    { 0xc0e21c1b, SUC_NE_lexicon_tags_2 },
    { 0x63da1c1a, SUC_NE_lexicon_tags_2 },
    { 0x46f61c1b, SUC_NE_lexicon_tags_1 },
    { 0x495e1c19, SUC_NE_lexicon_tags_2 },
    { 0xc3621c19, SUC_NE_lexicon_tags_7 },
    { 0xe3541c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8a1c24, SUC_NE_lexicon_tags_2 },
    { 0xe3401c25, SUC_NE_lexicon_tags_2 },
    { 0xeb541c26, SUC_NE_lexicon_tags_2 },
    { 0xef41c26, SUC_NE_lexicon_tags_7 },
    { 0x2b5c1c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ea1c2c, SUC_NE_lexicon_tags_2 },
    { 0x94ba1c2c, SUC_NE_lexicon_tags_2 },
    { 0x6b941c2e, SUC_NE_lexicon_tags_1 },
    { 0xd0e81c2e, SUC_NE_lexicon_tags_2 },
    { 0xd60a1c30, SUC_NE_lexicon_tags_2 },
    { 0x4b9a1c31, SUC_NE_lexicon_tags_2 },
    { 0xb1581c2d, SUC_NE_lexicon_tags_2 },
    { 0x5f821c33, SUC_NE_lexicon_tags_2 },
    { 0x273e1c2c, SUC_NE_lexicon_tags_17 },
    { 0xca021c35, SUC_NE_lexicon_tags_2 },
    { 0x62c01c35, SUC_NE_lexicon_tags_2 },
    { 0x278a1c36, SUC_NE_lexicon_tags_2 },
    { 0x8bb81c38, SUC_NE_lexicon_tags_2 },
    { 0xe581c2c, SUC_NE_lexicon_tags_2 },
    { 0xeaca1c35, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b701c42, SUC_NE_lexicon_tags_2 },
    { 0xeb841c43, SUC_NE_lexicon_tags_1 },
    { 0x79021c42, SUC_NE_lexicon_tags_57 },
    { 0xe5e61c44, SUC_NE_lexicon_tags_2 },
    { 0xd30e1c46, SUC_NE_lexicon_tags_2 },
    { 0x6e101c43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c301c49, SUC_NE_lexicon_tags_2 },
    { 0xf8061c4a, SUC_NE_lexicon_tags_2 },
    { 0x24b61c4a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x40881c4d, SUC_NE_lexicon_tags_2 },
    { 0x822c1c4e, SUC_NE_lexicon_tags_2 },
    { 0x9b8e1c4f, SUC_NE_lexicon_tags_2 },
    { 0x4e4c1c50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26401c52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78821c56, SUC_NE_lexicon_tags_5 },
    { 0x2b261c57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6a1c59, SUC_NE_lexicon_tags_2 },
    { 0x23e01c59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f41c5f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aba1c62, SUC_NE_lexicon_tags_2 },
    { 0x16141c63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c41c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef481c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a81c6a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30081c6f, SUC_NE_lexicon_tags_2 },
    { 0x61d81c6f, SUC_NE_lexicon_tags_2 },
    { 0xfade1c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5541c78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22241c7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75c1c82, SUC_NE_lexicon_tags_3 },
    { 0x769a1c82, SUC_NE_lexicon_tags_2 },
    { 0x45f21c84, SUC_NE_lexicon_tags_2 },
    { 0x95621c85, SUC_NE_lexicon_tags_2 },
    { 0x64de1c86, SUC_NE_lexicon_tags_2 },
    { 0x6341c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87921c89, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cc1c8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18521c8e, SUC_NE_lexicon_tags_0 },
    { 0x97721c8f, SUC_NE_lexicon_tags_2 },
    { 0xd90a1c90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47321c94, SUC_NE_lexicon_tags_2 },
    { 0x3c341c94, SUC_NE_lexicon_tags_2 },
    { 0xb5be1c96, SUC_NE_lexicon_tags_2 },
    { 0x71e61c97, SUC_NE_lexicon_tags_2 },
    { 0x83161c98, SUC_NE_lexicon_tags_2 },
    { 0x2f2c1c96, SUC_NE_lexicon_tags_2 },
    { 0x4cac1c9a, SUC_NE_lexicon_tags_2 },
    { 0xf4181c9a, SUC_NE_lexicon_tags_2 },
    { 0xcac21c9c, SUC_NE_lexicon_tags_2 },
    { 0x9f6c1c9c, SUC_NE_lexicon_tags_2 },
    { 0xd8e1c9e, SUC_NE_lexicon_tags_17 },
    { 0xd9a61c96, SUC_NE_lexicon_tags_2 },
    { 0x751a1c97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557e1ca4, SUC_NE_lexicon_tags_18 },
    { 0xa8f61ca4, SUC_NE_lexicon_tags_2 },
    { 0xc1ae1ca4, SUC_NE_lexicon_tags_3 },
    { 0x26ce1ca7, SUC_NE_lexicon_tags_2 },
    { 0x64121ca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x649c1cab, SUC_NE_lexicon_tags_2 },
    { 0x8db41cac, SUC_NE_lexicon_tags_2 },
    { 0xd3021cad, SUC_NE_lexicon_tags_2 },
    { 0x4c661cac, SUC_NE_lexicon_tags_3 },
    { 0xcf621caf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1f7a1cb1, SUC_NE_lexicon_tags_2 },
    { 0x74301cb2, SUC_NE_lexicon_tags_2 },
    { 0x75e1cb3, SUC_NE_lexicon_tags_2 },
    { 0xe5c61cb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa01cb7, SUC_NE_lexicon_tags_2 },
    { 0x4d961cb8, SUC_NE_lexicon_tags_2 },
    { 0xc31c1cb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36ba1cbb, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74461cc1, SUC_NE_lexicon_tags_2 },
    { 0x71fa1cc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48c1cc5, SUC_NE_lexicon_tags_2 },
    { 0xf9d01cc6, SUC_NE_lexicon_tags_2 },
    { 0xf01e1cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250c1ccb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561cd4, SUC_NE_lexicon_tags_2 },
    { 0x23281cd4, SUC_NE_lexicon_tags_2 },
    { 0x8f2c1cd6, SUC_NE_lexicon_tags_2 },
    { 0x3e0c1cd7, SUC_NE_lexicon_tags_2 },
    { 0x323e1cd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa61cdc, SUC_NE_lexicon_tags_2 },
    { 0xad461cdd, SUC_NE_lexicon_tags_30 },
    { 0xd71a1cde, SUC_NE_lexicon_tags_2 },
    { 0x54301cdd, SUC_NE_lexicon_tags_2 },
    { 0x854a1ce0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4701ce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90361ce5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8121ce7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b421cea, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b681ced, SUC_NE_lexicon_tags_2 },
    { 0xb6ba1cee, SUC_NE_lexicon_tags_0 },
    { 0xdac61cef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc281cf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3281cf4, SUC_NE_lexicon_tags_2 },
    { 0x1b41cf5, SUC_NE_lexicon_tags_2 },
    { 0x19c81cf6, SUC_NE_lexicon_tags_2 },
    { 0x8d221cf7, SUC_NE_lexicon_tags_2 },
    { 0xe0741cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725c1cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe1d02, SUC_NE_lexicon_tags_102 },
    { 0x95ee1d03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a921d05, SUC_NE_lexicon_tags_2 },
    { 0x39c81d06, SUC_NE_lexicon_tags_4 },
    { 0xfe1d06, SUC_NE_lexicon_tags_2 },
    { 0x22281d08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34fa1d0a, SUC_NE_lexicon_tags_3 },
    { 0x8c8e1d0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c41d0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a161d0f, SUC_NE_lexicon_tags_2 },
    { 0x81761d10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5601d15, SUC_NE_lexicon_tags_2 },
    { 0xa8701d16, SUC_NE_lexicon_tags_4 },
    { 0x16081d17, SUC_NE_lexicon_tags_2 },
    { 0x79881d17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2301d20, SUC_NE_lexicon_tags_2 },
    { 0x77a21d21, SUC_NE_lexicon_tags_2 },
    { 0x19d21d22, SUC_NE_lexicon_tags_2 },
    { 0xf4601d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83901d25, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21881d29, SUC_NE_lexicon_tags_2 },
    { 0x82961d2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a021d2d, SUC_NE_lexicon_tags_7 },
    { 0x6f161d2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf441d31, SUC_NE_lexicon_tags_2 },
    { 0x3efe1d32, SUC_NE_lexicon_tags_2 },
    { 0x7cea1d33, SUC_NE_lexicon_tags_2 },
    { 0xdf2a1d33, SUC_NE_lexicon_tags_34 },
    { 0x3001d35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66981d37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36b01d39, SUC_NE_lexicon_tags_2 },
    { 0xee281d3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca01d40, SUC_NE_lexicon_tags_2 },
    { 0x80f81d41, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0x740a1d43, SUC_NE_lexicon_tags_2 },
    { 0x3ec61d44, SUC_NE_lexicon_tags_2 },
    { 0xac321d45, SUC_NE_lexicon_tags_2 },
    { 0xb0821d46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b181d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c1d50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9b81d52, SUC_NE_lexicon_tags_2 },
    { 0x5bac1d53, SUC_NE_lexicon_tags_2 },
    { 0xb10c1d54, SUC_NE_lexicon_tags_2 },
    { 0x551a1d53, SUC_NE_lexicon_tags_2 },
    { 0x437a1d56, SUC_NE_lexicon_tags_2 },
    { 0x99b81d57, SUC_NE_lexicon_tags_3 },
    { 0x99281d52, SUC_NE_lexicon_tags_2 },
    { 0x95301d59, SUC_NE_lexicon_tags_17 },
    { 0xe5261d5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5361d5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd85c1d60, SUC_NE_lexicon_tags_2 },
    { 0x1d421d61, SUC_NE_lexicon_tags_2 },
    { 0x15901d62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80b81d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5c1d67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42d41d69, SUC_NE_lexicon_tags_2 },
    { 0xb5c41d6a, SUC_NE_lexicon_tags_2 },
    { 0xc66a1d6b, SUC_NE_lexicon_tags_2 },
    { 0x32801d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cd61d6e, SUC_NE_lexicon_tags_2 },
    { 0xd1201d6f, SUC_NE_lexicon_tags_2 },
    { 0x849a1d70, SUC_NE_lexicon_tags_2 },
    { 0xeb281d70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62801d76, SUC_NE_lexicon_tags_2 },
    { 0xe7ee1d77, SUC_NE_lexicon_tags_24 },
    { 0x701e1d78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a21d80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x493c1d82, SUC_NE_lexicon_tags_2 },
    { 0xc7141d83, SUC_NE_lexicon_tags_3 },
    { 0x4a6e1d84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ba1d8a, SUC_NE_lexicon_tags_2 },
    { 0xdc881d8b, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ca1d91, SUC_NE_lexicon_tags_2 },
    { 0x97901d92, SUC_NE_lexicon_tags_2 },
    { 0x60341d93, SUC_NE_lexicon_tags_2 },
    { 0x5a761d92, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54a1d9a, SUC_NE_lexicon_tags_2 },
    { 0xa1e41d9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5901d9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9081da2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59361da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f0a1da8, SUC_NE_lexicon_tags_17 },
    { 0x1e781da8, SUC_NE_lexicon_tags_3 },
    { 0xe5b61da9, SUC_NE_lexicon_tags_2 },
    { 0x9e41da9, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0xa2421dad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30961db3, SUC_NE_lexicon_tags_50 },
    { 0x94201db3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7321db7, SUC_NE_lexicon_tags_2 },
    { 0xfa81db8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe1dc4, SUC_NE_lexicon_tags_2 },
    { 0xa09e1dc5, SUC_NE_lexicon_tags_2 },
    { 0x318a1dc6, SUC_NE_lexicon_tags_19 },
    { 0xfc4a1dc6, SUC_NE_lexicon_tags_69 },
    { 0x12081dc6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512e1dd2, SUC_NE_lexicon_tags_3 },
    { 0x7cba1dd3, SUC_NE_lexicon_tags_2 },
    { 0x25141dd4, SUC_NE_lexicon_tags_2 },
    { 0xf9bc1dd5, SUC_NE_lexicon_tags_2 },
    { 0xa2d61dd6, SUC_NE_lexicon_tags_3 },
    { 0x362c1dd5, SUC_NE_lexicon_tags_2 },
    { 0x2e201dd6, SUC_NE_lexicon_tags_4 },
    { 0x6fce1dd9, SUC_NE_lexicon_tags_2 },
    { 0x65121dda, SUC_NE_lexicon_tags_3 },
    { 0x4a0e1dd6, SUC_NE_lexicon_tags_2 },
    { 0xf9c61dd3, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4741de1, SUC_NE_lexicon_tags_2 },
    { 0xde781de2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db21de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c61dea, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89c1ded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b81df6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6c1df8, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x17221dfa, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x96e1dfc, SUC_NE_lexicon_tags_2 },
    { 0x1f781dfd, SUC_NE_lexicon_tags_2 },
    { 0xf3e1dfe, SUC_NE_lexicon_tags_2 },
    { 0x4a261dff, SUC_NE_lexicon_tags_2 },
    { 0xc9821e00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33081e06, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea1e09, SUC_NE_lexicon_tags_2 },
    { 0x84a21e09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x720e1e0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62941e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d61e1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71941e23, SUC_NE_lexicon_tags_2 },
    { 0xe78a1e24, SUC_NE_lexicon_tags_2 },
    { 0x27781e25, SUC_NE_lexicon_tags_1 },
    { 0xba021e23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91381e2a, SUC_NE_lexicon_tags_2 },
    { 0x9c2a1e2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1e41e2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e01e30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca1e36, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f81e39, SUC_NE_lexicon_tags_2 },
    { 0x7f581e3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7e1e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x949a1e3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0c1e44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed721e48, SUC_NE_lexicon_tags_2 },
    { 0xaca81e49, SUC_NE_lexicon_tags_2 },
    { 0xcd181e4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e401e4c, SUC_NE_lexicon_tags_2 },
    { 0xfece1e4d, SUC_NE_lexicon_tags_2 },
    { 0x81181e4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192e1e54, SUC_NE_lexicon_tags_2 },
    { 0xbc981e55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x312e1e58, SUC_NE_lexicon_tags_2 },
    { 0xe68a1e59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb421e5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47be1e5f, SUC_NE_lexicon_tags_2 },
    { 0x78821e5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa501e62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f701e66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0e1e69, SUC_NE_lexicon_tags_2 },
    { 0xb4741e69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d701e6c, SUC_NE_lexicon_tags_2 },
    { 0x61701e6c, SUC_NE_lexicon_tags_2 },
    { 0x79781e6e, SUC_NE_lexicon_tags_2 },
    { 0xdec21e6e, SUC_NE_lexicon_tags_2 },
    { 0x80ba1e70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1821e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0921e77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4681e79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e461e7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53561e87, SUC_NE_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881a1e8d, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x55541e90, SUC_NE_lexicon_tags_2 },
    { 0x28661e90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x383c1e93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec81e96, SUC_NE_lexicon_tags_2 },
    { 0x51e41e97, SUC_NE_lexicon_tags_2 },
    { 0x9b421e97, SUC_NE_lexicon_tags_2 },
    { 0xae1a1e99, SUC_NE_lexicon_tags_2 },
    { 0x7c281e99, SUC_NE_lexicon_tags_2 },
    { 0x71921e9b, SUC_NE_lexicon_tags_2 },
    { 0x8f5c1e9c, SUC_NE_lexicon_tags_2 },
    { 0x38621e98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dc1ea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55241ea4, SUC_NE_lexicon_tags_2 },
    { 0x7e01ea4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf821eaa, SUC_NE_lexicon_tags_7 },
    { 0x2e361eab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c21eae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a1eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x185a1eb4, SUC_NE_lexicon_tags_2 },
    { 0x9c01eb5, SUC_NE_lexicon_tags_2 },
    { 0x8b861eb5, SUC_NE_lexicon_tags_230 },
    { 0xcc881eb4, SUC_NE_lexicon_tags_17 },
    { 0x8e1e1eb8, SUC_NE_lexicon_tags_2 },
    { 0x46001eb8, SUC_NE_lexicon_tags_2 },
    { 0x84e21eb7, SUC_NE_lexicon_tags_2 },
    { 0xb3c61eb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335c1ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8a1ec6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7621ec9, SUC_NE_lexicon_tags_2 },
    { 0x76901eca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b861ecd, SUC_NE_lexicon_tags_2 },
    { 0xd03e1ece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea681ed0, SUC_NE_lexicon_tags_447 },
    { 0xc4001ed1, SUC_NE_lexicon_tags_2 },
    { 0xcfa61ed2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b01ed5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75d61ed7, SUC_NE_lexicon_tags_2 },
    { 0x514e1ed8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b61edd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6561edf, SUC_NE_lexicon_tags_5 },
    { 0x899a1ee0, SUC_NE_lexicon_tags_2 },
    { 0x17a41ee0, SUC_NE_lexicon_tags_2 },
    { 0x49301ee2, SUC_NE_lexicon_tags_3 },
    { 0xc66e1ee1, SUC_NE_lexicon_tags_2 },
    { 0x4021ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2a1ee6, SUC_NE_lexicon_tags_5 },
    { 0x10381ee7, SUC_NE_lexicon_tags_2 },
    { 0xa2b21ee8, SUC_NE_lexicon_tags_2 },
    { 0xa63a1ee8, SUC_NE_lexicon_tags_2 },
    { 0x9b221ee9, SUC_NE_lexicon_tags_2 },
    { 0x152e1ee9, SUC_NE_lexicon_tags_3 },
    { 0xa9341ee8, SUC_NE_lexicon_tags_2 },
    { 0xeb521eea, SUC_NE_lexicon_tags_2 },
    { 0x9ec21ee9, SUC_NE_lexicon_tags_7 },
    { 0x35e81eef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf81ef2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x720e1ef4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x26901ef6, SUC_NE_lexicon_tags_2 },
    { 0xd6fa1ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9e61ef9, SUC_NE_lexicon_tags_2 },
    { 0x204e1efa, SUC_NE_lexicon_tags_2 },
    { 0x4db61efb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40121f00, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60941f04, SUC_NE_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10c1f0e, SUC_NE_lexicon_tags_3 },
    { 0x293e1f0e, SUC_NE_lexicon_tags_2 },
    { 0x56741f0e, SUC_NE_lexicon_tags_2 },
    { 0xb4e41f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b881f1a, SUC_NE_lexicon_tags_2 },
    { 0x88861f1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36e01f1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99e1f22, SUC_NE_lexicon_tags_10 },
    { 0xa0f01f23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ea1f26, SUC_NE_lexicon_tags_2 },
    { 0x21481f27, SUC_NE_lexicon_tags_197 },
    { 0xdb721f26, SUC_NE_lexicon_tags_3 },
    { 0xeff81f29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ba1f2c, SUC_NE_lexicon_tags_2 },
    { 0xc2ae1f2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5e1f2f, SUC_NE_lexicon_tags_3 },
    { 0xf2761f30, SUC_NE_lexicon_tags_2 },
    { 0x96ec1f30, SUC_NE_lexicon_tags_2 },
    { 0x3b4a1f32, SUC_NE_lexicon_tags_2 },
    { 0x3a1c1f32, SUC_NE_lexicon_tags_2 },
    { 0x2be41f33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74da1f3e, SUC_NE_lexicon_tags_2 },
    { 0xd5621f3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x657a1f42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f81f4e, SUC_NE_lexicon_tags_2 },
    { 0xcda1f4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab1e1f51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96281f54, SUC_NE_lexicon_tags_2 },
    { 0xbfa1f55, SUC_NE_lexicon_tags_2 },
    { 0xb1301f56, SUC_NE_lexicon_tags_0 },
    { 0xb521f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x421f5a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xaefe1f5c, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41e1f5f, SUC_NE_lexicon_tags_2 },
    { 0x1fd61f60, SUC_NE_lexicon_tags_2 },
    { 0xccb01f61, SUC_NE_lexicon_tags_2 },
    { 0x638a1f61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7701f69, SUC_NE_lexicon_tags_2 },
    { 0xb8401f6a, SUC_NE_lexicon_tags_354 },
    { 0, NULL },
    { 0xcb0e1f6c, SUC_NE_lexicon_tags_2 },
    { 0xddce1f6c, SUC_NE_lexicon_tags_2 },
    { 0x86521f6e, SUC_NE_lexicon_tags_2 },
    { 0xbc2a1f6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c1f75, SUC_NE_lexicon_tags_3 },
    { 0x1aca1f76, SUC_NE_lexicon_tags_17 },
    { 0x6b1c1f77, SUC_NE_lexicon_tags_2 },
    { 0xbd641f77, SUC_NE_lexicon_tags_2 },
    { 0x57a81f79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b6a1f7b, SUC_NE_lexicon_tags_2 },
    { 0x3d281f7b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x62501f7e, SUC_NE_lexicon_tags_2 },
    { 0x2f41f7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3da1f81, SUC_NE_lexicon_tags_1 },
    { 0x77261f82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef41f85, SUC_NE_lexicon_tags_2 },
    { 0x49c81f86, SUC_NE_lexicon_tags_4 },
    { 0x422c1f87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x345c1f89, SUC_NE_lexicon_tags_2 },
    { 0x4b3a1f8a, SUC_NE_lexicon_tags_2 },
    { 0xbc9a1f8b, SUC_NE_lexicon_tags_2 },
    { 0xdcaa1f8a, SUC_NE_lexicon_tags_2 },
    { 0x29de1f8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc41f90, SUC_NE_lexicon_tags_2 },
    { 0xc6d61f91, SUC_NE_lexicon_tags_2 },
    { 0xa07e1f92, SUC_NE_lexicon_tags_2 },
    { 0x6d721f92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b01f95, SUC_NE_lexicon_tags_2 },
    { 0xa8c81f95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b681f9c, SUC_NE_lexicon_tags_2 },
    { 0xe7a61f9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6bc1f9f, SUC_NE_lexicon_tags_2 },
    { 0x33e41fa0, SUC_NE_lexicon_tags_2 },
    { 0x9ffc1fa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c41fa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32ca1fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f681fa9, SUC_NE_lexicon_tags_2 },
    { 0x5641faa, SUC_NE_lexicon_tags_4 },
    { 0xcbc21fab, SUC_NE_lexicon_tags_12 },
    { 0xa72a1fab, SUC_NE_lexicon_tags_2 },
    { 0xc3e81fad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d421fb0, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a61fb4, SUC_NE_lexicon_tags_2 },
    { 0x27741fb5, SUC_NE_lexicon_tags_2 },
    { 0xbb321fb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6de1fbc, SUC_NE_lexicon_tags_2 },
    { 0x53281fbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad021fc3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6a1fcb, SUC_NE_lexicon_tags_2 },
    { 0xfffc1fcc, SUC_NE_lexicon_tags_2 },
    { 0xb55a1fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46a1fd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b6c1fd3, SUC_NE_lexicon_tags_1 },
    { 0x3ea21fd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1941fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e761fdb, SUC_NE_lexicon_tags_6 },
    { 0, NULL },
    { 0xbaae1fdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42061fdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x438e1fe1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa6001fe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03e1fe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6961fe9, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4e1fef, SUC_NE_lexicon_tags_1 },
    { 0x4b641ff0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4c1ff3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc41ff6, SUC_NE_lexicon_tags_2 },
    { 0xce361ff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25361ff9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ba1ffc, SUC_NE_lexicon_tags_2 },
    { 0x79641ffd, SUC_NE_lexicon_tags_2 },
    { 0xe781ffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc002001, SUC_NE_lexicon_tags_2 },
    { 0x5a8e2002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34282005, SUC_NE_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e982008, SUC_NE_lexicon_tags_2 },
    { 0x97662008, SUC_NE_lexicon_tags_2 },
    { 0x58822008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc8200d, SUC_NE_lexicon_tags_4 },
    { 0x763a200e, SUC_NE_lexicon_tags_32 },
    { 0x3a2200f, SUC_NE_lexicon_tags_2 },
    { 0x2e82010, SUC_NE_lexicon_tags_2 },
    { 0x5cdc200f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a02014, SUC_NE_lexicon_tags_2 },
    { 0xc74c2015, SUC_NE_lexicon_tags_2 },
    { 0x761e2014, SUC_NE_lexicon_tags_34 },
    { 0xfa3c2017, SUC_NE_lexicon_tags_2 },
    { 0x7c5a2015, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7400201a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bf0201c, SUC_NE_lexicon_tags_34 },
    { 0x34ae201c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b122022, SUC_NE_lexicon_tags_2 },
    { 0xea302023, SUC_NE_lexicon_tags_5 },
    { 0x175e2022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9342026, SUC_NE_lexicon_tags_7 },
    { 0x4bf82027, SUC_NE_lexicon_tags_2 },
    { 0x73602028, SUC_NE_lexicon_tags_2 },
    { 0xb04a2029, SUC_NE_lexicon_tags_2 },
    { 0x64242026, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2352202f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc262034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9a2039, SUC_NE_lexicon_tags_2 },
    { 0xc884203a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cc2040, SUC_NE_lexicon_tags_2 },
    { 0xa5302041, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x81922043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccc204b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d2204f, SUC_NE_lexicon_tags_2 },
    { 0x72d82050, SUC_NE_lexicon_tags_2 },
    { 0xfa482051, SUC_NE_lexicon_tags_2 },
    { 0x6ffa204f, SUC_NE_lexicon_tags_2 },
    { 0x3eca2053, SUC_NE_lexicon_tags_2 },
    { 0x48ce2054, SUC_NE_lexicon_tags_2 },
    { 0xce5e2055, SUC_NE_lexicon_tags_2 },
    { 0xd4a02056, SUC_NE_lexicon_tags_34 },
    { 0x600c204f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706a205b, SUC_NE_lexicon_tags_2 },
    { 0xed3c205b, SUC_NE_lexicon_tags_2 },
    { 0x8690205d, SUC_NE_lexicon_tags_2 },
    { 0x5818205d, SUC_NE_lexicon_tags_2 },
    { 0xd1b2205b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x583e2061, SUC_NE_lexicon_tags_2 },
    { 0x96ec2061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac182064, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd982069, SUC_NE_lexicon_tags_2 },
    { 0x8e6206a, SUC_NE_lexicon_tags_2 },
    { 0x8db4206b, SUC_NE_lexicon_tags_2 },
    { 0xae402069, SUC_NE_lexicon_tags_5 },
    { 0xcc6a206d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dd8206f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7c2072, SUC_NE_lexicon_tags_17 },
    { 0xac382072, SUC_NE_lexicon_tags_2 },
    { 0xb0e22074, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81e207a, SUC_NE_lexicon_tags_0 },
    { 0x17dc207b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bde207d, SUC_NE_lexicon_tags_2 },
    { 0xd4d8207e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc642080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc2083, SUC_NE_lexicon_tags_2 },
    { 0xe2502084, SUC_NE_lexicon_tags_2 },
    { 0xa83a2085, SUC_NE_lexicon_tags_30 },
    { 0x50a42086, SUC_NE_lexicon_tags_2 },
    { 0x74fe2087, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9742208c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x309a208f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d6e2091, SUC_NE_lexicon_tags_2 },
    { 0xc3282091, SUC_NE_lexicon_tags_2 },
    { 0x44322093, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9ca2095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418a209a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e96209d, SUC_NE_lexicon_tags_2 },
    { 0x8f56209d, SUC_NE_lexicon_tags_1 },
    { 0x7ec6209d, SUC_NE_lexicon_tags_2 },
    { 0x2d34209f, SUC_NE_lexicon_tags_2 },
    { 0xed12209e, SUC_NE_lexicon_tags_2 },
    { 0xb9c209f, SUC_NE_lexicon_tags_2 },
    { 0x58ee20a3, SUC_NE_lexicon_tags_3 },
    { 0xcc3420a3, SUC_NE_lexicon_tags_2 },
    { 0x7a420a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1e20aa, SUC_NE_lexicon_tags_1 },
    { 0x8b2e20aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce9620ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636020b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11d420b9, SUC_NE_lexicon_tags_2 },
    { 0xda4820ba, SUC_NE_lexicon_tags_2 },
    { 0x14a620ba, SUC_NE_lexicon_tags_2 },
    { 0x408020b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162c20c3, SUC_NE_lexicon_tags_2 },
    { 0xee1a20c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6c20c7, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xc01c20c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430a20ce, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2220d1, SUC_NE_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6c20d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d620da, SUC_NE_lexicon_tags_2 },
    { 0xe9de20db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x372220dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110a20e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fe20ea, SUC_NE_lexicon_tags_1 },
    { 0x3420ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa820f0, SUC_NE_lexicon_tags_2 },
    { 0x79fe20f1, SUC_NE_lexicon_tags_30 },
    { 0x4b2420f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x340020f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a420f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb27020f9, SUC_NE_lexicon_tags_2 },
    { 0xaf9020f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a620fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab222102, SUC_NE_lexicon_tags_2 },
    { 0x49522102, SUC_NE_lexicon_tags_2 },
    { 0x448a2104, SUC_NE_lexicon_tags_2 },
    { 0x3c22103, SUC_NE_lexicon_tags_2 },
    { 0xd8662102, SUC_NE_lexicon_tags_2 },
    { 0x1acc2102, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x951e2109, SUC_NE_lexicon_tags_2 },
    { 0xa8a62109, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e382110, SUC_NE_lexicon_tags_2 },
    { 0x4e962110, SUC_NE_lexicon_tags_2 },
    { 0xf1f02112, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e342114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f322118, SUC_NE_lexicon_tags_2 },
    { 0xf1f02118, SUC_NE_lexicon_tags_2 },
    { 0x45a82118, SUC_NE_lexicon_tags_2 },
    { 0x7e1c211a, SUC_NE_lexicon_tags_2 },
    { 0x17022119, SUC_NE_lexicon_tags_2 },
    { 0x5b7e211d, SUC_NE_lexicon_tags_17 },
    { 0x6e6211a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7082120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5ba2122, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710e2127, SUC_NE_lexicon_tags_2 },
    { 0x5f1c2128, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa168212b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8020212d, SUC_NE_lexicon_tags_2 },
    { 0xd598212e, SUC_NE_lexicon_tags_3 },
    { 0xee46212e, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xe2b22131, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82302134, SUC_NE_lexicon_tags_210 },
    { 0, NULL },
    { 0x58402136, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f802138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf8213d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e62143, SUC_NE_lexicon_tags_2 },
    { 0x43d22143, SUC_NE_lexicon_tags_2 },
    { 0x7fde2145, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2a2148, SUC_NE_lexicon_tags_2 },
    { 0xa7742148, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5904214b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4580214d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0742150, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe882156, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c6215b, SUC_NE_lexicon_tags_2 },
    { 0x5920215c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af62160, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9662168, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5546216a, SUC_NE_lexicon_tags_2 },
    { 0xfae216b, SUC_NE_lexicon_tags_2 },
    { 0x3dea216c, SUC_NE_lexicon_tags_2 },
    { 0x19a216c, SUC_NE_lexicon_tags_2 },
    { 0x7672216e, SUC_NE_lexicon_tags_2 },
    { 0x456c216e, SUC_NE_lexicon_tags_2 },
    { 0xd4e6216b, SUC_NE_lexicon_tags_2 },
    { 0x8770216d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5ea2173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d9c2175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a42178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd6217c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1b0217e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30742187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2702189, SUC_NE_lexicon_tags_2 },
    { 0x9754218a, SUC_NE_lexicon_tags_2 },
    { 0x8660218b, SUC_NE_lexicon_tags_2 },
    { 0x69e8218c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf616218e, SUC_NE_lexicon_tags_2 },
    { 0x8cf2218f, SUC_NE_lexicon_tags_2 },
    { 0x2d922190, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x35822192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bca2194, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86fc2196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07e219a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e6219d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf2d4219f, SUC_NE_lexicon_tags_2 },
    { 0x46d821a0, SUC_NE_lexicon_tags_5 },
    { 0xccb021a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1da21a6, SUC_NE_lexicon_tags_2 },
    { 0x3dfa21a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x950021a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4021ac, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x46221ae, SUC_NE_lexicon_tags_17 },
    { 0x9a8421af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217a21b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba21b7, SUC_NE_lexicon_tags_34 },
    { 0x6d1621b8, SUC_NE_lexicon_tags_2 },
    { 0x8e3421b9, SUC_NE_lexicon_tags_2 },
    { 0x5e6421b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x227a21bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7221c1, SUC_NE_lexicon_tags_2 },
    { 0xbbc221c1, SUC_NE_lexicon_tags_2 },
    { 0xf52a21c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2a21c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d821cb, SUC_NE_lexicon_tags_1 },
    { 0x243c21cc, SUC_NE_lexicon_tags_2 },
    { 0x8e4a21cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43b421cf, SUC_NE_lexicon_tags_2 },
    { 0x244021cf, SUC_NE_lexicon_tags_2 },
    { 0x800821cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5221d8, SUC_NE_lexicon_tags_2 },
    { 0x3be021d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x529821db, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b221de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1a21e1, SUC_NE_lexicon_tags_2 },
    { 0xa3621e2, SUC_NE_lexicon_tags_2 },
    { 0x5c1221e3, SUC_NE_lexicon_tags_2 },
    { 0x33a221e3, SUC_NE_lexicon_tags_2 },
    { 0x6b9821e2, SUC_NE_lexicon_tags_2 },
    { 0xa13a21e6, SUC_NE_lexicon_tags_2 },
    { 0xb56221e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9821ec, SUC_NE_lexicon_tags_0 },
    { 0xfc0821ed, SUC_NE_lexicon_tags_2 },
    { 0x5c2221ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524621f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22e21f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37c21f9, SUC_NE_lexicon_tags_2 },
    { 0x729221fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985821fe, SUC_NE_lexicon_tags_2 },
    { 0xe13621fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4afc2201, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7422206, SUC_NE_lexicon_tags_2 },
    { 0x5102207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e822209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ec220b, SUC_NE_lexicon_tags_2 },
    { 0x8d56220c, SUC_NE_lexicon_tags_2 },
    { 0xc5ee220d, SUC_NE_lexicon_tags_2 },
    { 0x1efe220e, SUC_NE_lexicon_tags_24 },
    { 0x8f4e220d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d402211, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3082216, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81682219, SUC_NE_lexicon_tags_2 },
    { 0x53ae221a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd602221e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dec2220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f62227, SUC_NE_lexicon_tags_2 },
    { 0xdba22228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abc222c, SUC_NE_lexicon_tags_2 },
    { 0xf822222d, SUC_NE_lexicon_tags_2 },
    { 0xb642222d, SUC_NE_lexicon_tags_2 },
    { 0xfc08222f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdc2235, SUC_NE_lexicon_tags_2 },
    { 0x5f1e2236, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba662239, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256223d, SUC_NE_lexicon_tags_2 },
    { 0xaae223e, SUC_NE_lexicon_tags_2 },
    { 0xf916223f, SUC_NE_lexicon_tags_2 },
    { 0x1296223f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f1a2242, SUC_NE_lexicon_tags_49 },
    { 0xbc722243, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a42248, SUC_NE_lexicon_tags_2 },
    { 0x8a502249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1a224c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7922251, SUC_NE_lexicon_tags_2 },
    { 0xb6e82252, SUC_NE_lexicon_tags_2 },
    { 0x5c4e2251, SUC_NE_lexicon_tags_2 },
    { 0x43e42253, SUC_NE_lexicon_tags_2 },
    { 0x95d62252, SUC_NE_lexicon_tags_2 },
    { 0x8bec2251, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb978225a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dce225c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff38225e, SUC_NE_lexicon_tags_2 },
    { 0x31a225f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ee22261, SUC_NE_lexicon_tags_2 },
    { 0x73462262, SUC_NE_lexicon_tags_2 },
    { 0xf21a2263, SUC_NE_lexicon_tags_2 },
    { 0x3cfa2264, SUC_NE_lexicon_tags_2 },
    { 0xb9902264, SUC_NE_lexicon_tags_2 },
    { 0x66162264, SUC_NE_lexicon_tags_17 },
    { 0xfe402264, SUC_NE_lexicon_tags_2 },
    { 0xf8cc2261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5fc226a, SUC_NE_lexicon_tags_2 },
    { 0xe622226b, SUC_NE_lexicon_tags_2 },
    { 0x5350226b, SUC_NE_lexicon_tags_2 },
    { 0xe126226d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f02271, SUC_NE_lexicon_tags_2 },
    { 0x36c42272, SUC_NE_lexicon_tags_2 },
    { 0x88fa2273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a682278, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe227b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3682280, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d82283, SUC_NE_lexicon_tags_2 },
    { 0x8b7a2284, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d22287, SUC_NE_lexicon_tags_2 },
    { 0x55962287, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c30228b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4c228e, SUC_NE_lexicon_tags_2 },
    { 0xc94a228f, SUC_NE_lexicon_tags_2 },
    { 0x470a228f, SUC_NE_lexicon_tags_2 },
    { 0x719c2291, SUC_NE_lexicon_tags_2 },
    { 0xb78c2292, SUC_NE_lexicon_tags_2 },
    { 0xa0c02293, SUC_NE_lexicon_tags_2 },
    { 0x571e2294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85ee2296, SUC_NE_lexicon_tags_2 },
    { 0x79f82297, SUC_NE_lexicon_tags_2 },
    { 0xd1962298, SUC_NE_lexicon_tags_2 },
    { 0xbff22299, SUC_NE_lexicon_tags_2 },
    { 0x2dd42299, SUC_NE_lexicon_tags_2 },
    { 0x4808229b, SUC_NE_lexicon_tags_4 },
    { 0x9ab0229c, SUC_NE_lexicon_tags_2 },
    { 0x5db82296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa22a0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ca22a3, SUC_NE_lexicon_tags_2 },
    { 0x425222a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b2a22a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858022ab, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5a22af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7022b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b222b9, SUC_NE_lexicon_tags_2 },
    { 0x736422b9, SUC_NE_lexicon_tags_2 },
    { 0x54b222bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x996222bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36c022bf, SUC_NE_lexicon_tags_0 },
    { 0xed4e22c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9c22c2, SUC_NE_lexicon_tags_2 },
    { 0xcce822c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44be22c5, SUC_NE_lexicon_tags_2 },
    { 0xbf222c6, SUC_NE_lexicon_tags_2 },
    { 0x50c22c6, SUC_NE_lexicon_tags_2 },
    { 0x289e22c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4422cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88d222cd, SUC_NE_lexicon_tags_2 },
    { 0xc98422ce, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x4c5822d0, SUC_NE_lexicon_tags_2 },
    { 0xea8022d1, SUC_NE_lexicon_tags_2 },
    { 0xaa8222d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0422d7, SUC_NE_lexicon_tags_2 },
    { 0x303622d8, SUC_NE_lexicon_tags_4 },
    { 0x4f0422d9, SUC_NE_lexicon_tags_13 },
    { 0xfbe622d9, SUC_NE_lexicon_tags_1 },
    { 0x79cc22db, SUC_NE_lexicon_tags_2 },
    { 0x921c22dc, SUC_NE_lexicon_tags_2 },
    { 0xbe822dd, SUC_NE_lexicon_tags_2 },
    { 0x77fc22de, SUC_NE_lexicon_tags_2 },
    { 0x78ec22df, SUC_NE_lexicon_tags_2 },
    { 0x937422e0, SUC_NE_lexicon_tags_7 },
    { 0xf11222e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63b422e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281822ea, SUC_NE_lexicon_tags_2 },
    { 0x62b622eb, SUC_NE_lexicon_tags_1 },
    { 0xadc022ec, SUC_NE_lexicon_tags_2 },
    { 0xebe022ec, SUC_NE_lexicon_tags_4 },
    { 0xf7ba22ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d2e22f0, SUC_NE_lexicon_tags_2 },
    { 0x4b622f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4222f6, SUC_NE_lexicon_tags_2 },
    { 0x7ad022f7, SUC_NE_lexicon_tags_2 },
    { 0x7b6a22f8, SUC_NE_lexicon_tags_3 },
    { 0x7e4c22f9, SUC_NE_lexicon_tags_2 },
    { 0x120222fa, SUC_NE_lexicon_tags_2 },
    { 0x4c3c22f9, SUC_NE_lexicon_tags_2 },
    { 0x646c22f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3902301, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87822305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf688230b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf790230d, SUC_NE_lexicon_tags_2 },
    { 0x7c90230e, SUC_NE_lexicon_tags_2 },
    { 0x5dfc230f, SUC_NE_lexicon_tags_2 },
    { 0xc15c230e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0a2313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d2e2315, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92622318, SUC_NE_lexicon_tags_2 },
    { 0x81cc2319, SUC_NE_lexicon_tags_2 },
    { 0xee24231a, SUC_NE_lexicon_tags_2 },
    { 0xcb54231a, SUC_NE_lexicon_tags_2 },
    { 0x4010231b, SUC_NE_lexicon_tags_2 },
    { 0x552a2318, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2842320, SUC_NE_lexicon_tags_2 },
    { 0xde02321, SUC_NE_lexicon_tags_2 },
    { 0x7c6e2321, SUC_NE_lexicon_tags_2 },
    { 0xcdb82323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc2326, SUC_NE_lexicon_tags_2 },
    { 0x873a2326, SUC_NE_lexicon_tags_4 },
    { 0x7c542327, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab72232b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e12232d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dc2330, SUC_NE_lexicon_tags_2 },
    { 0x388a2331, SUC_NE_lexicon_tags_2 },
    { 0x3d002332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c82334, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaed42336, SUC_NE_lexicon_tags_11 },
    { 0x3fb22337, SUC_NE_lexicon_tags_1 },
    { 0x2ad02338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa670233c, SUC_NE_lexicon_tags_2 },
    { 0x898c233d, SUC_NE_lexicon_tags_2 },
    { 0xf752233e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fe2342, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x252a2345, SUC_NE_lexicon_tags_2 },
    { 0x4d1e2346, SUC_NE_lexicon_tags_2 },
    { 0x181a2345, SUC_NE_lexicon_tags_2 },
    { 0x3cea2348, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdc2350, SUC_NE_lexicon_tags_2 },
    { 0x1e242351, SUC_NE_lexicon_tags_2 },
    { 0xf0302352, SUC_NE_lexicon_tags_2 },
    { 0xe7442352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc078235a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e08235e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc902366, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccba236c, SUC_NE_lexicon_tags_2 },
    { 0xf014236c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe28e236f, SUC_NE_lexicon_tags_2 },
    { 0x79462370, SUC_NE_lexicon_tags_2 },
    { 0x83ae2371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c62376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae237a, SUC_NE_lexicon_tags_2 },
    { 0x1324237b, SUC_NE_lexicon_tags_2 },
    { 0xcb8a237c, SUC_NE_lexicon_tags_2 },
    { 0xc8b2237d, SUC_NE_lexicon_tags_2 },
    { 0x52a0237d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f122384, SUC_NE_lexicon_tags_53 },
    { 0x155e2385, SUC_NE_lexicon_tags_2 },
    { 0x70762386, SUC_NE_lexicon_tags_51 },
    { 0x9c3a2387, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea238a, SUC_NE_lexicon_tags_18 },
    { 0x5d08238b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab76238f, SUC_NE_lexicon_tags_2 },
    { 0x53f42390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34842392, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdea2394, SUC_NE_lexicon_tags_2 },
    { 0xeb02395, SUC_NE_lexicon_tags_34 },
    { 0x36262396, SUC_NE_lexicon_tags_2 },
    { 0xbdaa2397, SUC_NE_lexicon_tags_2 },
    { 0x608c2398, SUC_NE_lexicon_tags_98 },
    { 0x203c2399, SUC_NE_lexicon_tags_2 },
    { 0xc822399, SUC_NE_lexicon_tags_2 },
    { 0x358e239b, SUC_NE_lexicon_tags_2 },
    { 0x90a0239c, SUC_NE_lexicon_tags_2 },
    { 0xb094239d, SUC_NE_lexicon_tags_2 },
    { 0x59a22396, SUC_NE_lexicon_tags_7 },
    { 0x549c239f, SUC_NE_lexicon_tags_2 },
    { 0x811623a0, SUC_NE_lexicon_tags_7 },
    { 0x941623a0, SUC_NE_lexicon_tags_7 },
    { 0x3a8e239c, SUC_NE_lexicon_tags_2 },
    { 0xe34023a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x193823a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933223aa, SUC_NE_lexicon_tags_2 },
    { 0x474a23ab, SUC_NE_lexicon_tags_2 },
    { 0x7ff023ac, SUC_NE_lexicon_tags_2 },
    { 0x48dc23ac, SUC_NE_lexicon_tags_284 },
    { 0, NULL },
    { 0x2bbe23af, SUC_NE_lexicon_tags_2 },
    { 0x244023af, SUC_NE_lexicon_tags_2 },
    { 0x835023b1, SUC_NE_lexicon_tags_4 },
    { 0x5f2623b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecc23b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a23b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc38023ba, SUC_NE_lexicon_tags_2 },
    { 0x46fc23ba, SUC_NE_lexicon_tags_32 },
    { 0xcdc023bc, SUC_NE_lexicon_tags_2 },
    { 0x5c1623bd, SUC_NE_lexicon_tags_2 },
    { 0x2ac823be, SUC_NE_lexicon_tags_89 },
    { 0x7b4a23be, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fc23c2, SUC_NE_lexicon_tags_2 },
    { 0xfbe823c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb823c6, SUC_NE_lexicon_tags_7 },
    { 0xcd2423c6, SUC_NE_lexicon_tags_0 },
    { 0xebcc23c8, SUC_NE_lexicon_tags_2 },
    { 0x12bc23c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e23cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6823d2, SUC_NE_lexicon_tags_2 },
    { 0x9f8023d3, SUC_NE_lexicon_tags_2 },
    { 0x3f2a23d4, SUC_NE_lexicon_tags_2 },
    { 0x8d8223d5, SUC_NE_lexicon_tags_2 },
    { 0x13de23d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x481e23d8, SUC_NE_lexicon_tags_2 },
    { 0xd9cc23d9, SUC_NE_lexicon_tags_75 },
    { 0x9d1423d9, SUC_NE_lexicon_tags_2 },
    { 0xe0ba23d8, SUC_NE_lexicon_tags_2 },
    { 0x940e23d9, SUC_NE_lexicon_tags_7 },
    { 0x78fc23dc, SUC_NE_lexicon_tags_2 },
    { 0x434223de, SUC_NE_lexicon_tags_2 },
    { 0x28d023d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfed623e1, SUC_NE_lexicon_tags_2 },
    { 0x2fa023e2, SUC_NE_lexicon_tags_2 },
    { 0x14cc23e3, SUC_NE_lexicon_tags_2 },
    { 0x25d623e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd2623e6, SUC_NE_lexicon_tags_3 },
    { 0x6f2c23e6, SUC_NE_lexicon_tags_2 },
    { 0x512a23e8, SUC_NE_lexicon_tags_2 },
    { 0xd43023e9, SUC_NE_lexicon_tags_2 },
    { 0xb9f023e8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa4023ec, SUC_NE_lexicon_tags_2 },
    { 0xbf1a23ed, SUC_NE_lexicon_tags_2 },
    { 0xc46e23ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ecc23f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab023f3, SUC_NE_lexicon_tags_2 },
    { 0x2b6223f4, SUC_NE_lexicon_tags_1 },
    { 0xd7c23f5, SUC_NE_lexicon_tags_2 },
    { 0x79ae23f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c9223f8, SUC_NE_lexicon_tags_2 },
    { 0x7b7023f9, SUC_NE_lexicon_tags_2 },
    { 0xde7623fa, SUC_NE_lexicon_tags_2 },
    { 0xe4d223fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f0c23fd, SUC_NE_lexicon_tags_2 },
    { 0xc65223fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4002403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4602406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd992240a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e42410, SUC_NE_lexicon_tags_2 },
    { 0x7d682410, SUC_NE_lexicon_tags_2 },
    { 0xc9ee2411, SUC_NE_lexicon_tags_42 },
    { 0xae842413, SUC_NE_lexicon_tags_2 },
    { 0x89382414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebe241e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14522421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88102424, SUC_NE_lexicon_tags_2 },
    { 0xd12c2424, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26742427, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb482430, SUC_NE_lexicon_tags_2 },
    { 0x1b3e2431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefbe2433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27582435, SUC_NE_lexicon_tags_2 },
    { 0xcaa82436, SUC_NE_lexicon_tags_2 },
    { 0xd6da2437, SUC_NE_lexicon_tags_2 },
    { 0x6b262437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb170243c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2a2445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19622448, SUC_NE_lexicon_tags_2 },
    { 0xb6d82449, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140244e, SUC_NE_lexicon_tags_2 },
    { 0x5558244e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93002451, SUC_NE_lexicon_tags_2 },
    { 0x31f42452, SUC_NE_lexicon_tags_2 },
    { 0x3722453, SUC_NE_lexicon_tags_2 },
    { 0xfeda2452, SUC_NE_lexicon_tags_2 },
    { 0xc2454, SUC_NE_lexicon_tags_2 },
    { 0x4a402454, SUC_NE_lexicon_tags_2 },
    { 0x25f82457, SUC_NE_lexicon_tags_2 },
    { 0xecb62457, SUC_NE_lexicon_tags_2 },
    { 0xb2582457, SUC_NE_lexicon_tags_2 },
    { 0xeb9c2457, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34de245c, SUC_NE_lexicon_tags_2 },
    { 0x2f84245c, SUC_NE_lexicon_tags_2 },
    { 0xc082245e, SUC_NE_lexicon_tags_2 },
    { 0x5730245f, SUC_NE_lexicon_tags_2 },
    { 0xf3c245f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f22462, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba842466, SUC_NE_lexicon_tags_2 },
    { 0x2d6c2466, SUC_NE_lexicon_tags_2 },
    { 0xd3442468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8510246a, SUC_NE_lexicon_tags_2 },
    { 0x9dc6246b, SUC_NE_lexicon_tags_2 },
    { 0xe2ba246c, SUC_NE_lexicon_tags_2 },
    { 0xcc96246d, SUC_NE_lexicon_tags_2 },
    { 0x2b9a246c, SUC_NE_lexicon_tags_2 },
    { 0xddca246d, SUC_NE_lexicon_tags_2 },
    { 0xbe38246d, SUC_NE_lexicon_tags_2 },
    { 0x694a2471, SUC_NE_lexicon_tags_2 },
    { 0xa9242472, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae02477, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x822a247a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc84a247c, SUC_NE_lexicon_tags_2 },
    { 0xef72247d, SUC_NE_lexicon_tags_0 },
    { 0x7624247d, SUC_NE_lexicon_tags_2 },
    { 0x4010247f, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x87182482, SUC_NE_lexicon_tags_2 },
    { 0xca6a2483, SUC_NE_lexicon_tags_2 },
    { 0x39e62484, SUC_NE_lexicon_tags_2 },
    { 0xe3242483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3a2488, SUC_NE_lexicon_tags_2 },
    { 0x19742489, SUC_NE_lexicon_tags_2 },
    { 0xb20c248a, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc2491, SUC_NE_lexicon_tags_2 },
    { 0x539e2492, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x554a2494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d5c2496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8249a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab6249d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555624a1, SUC_NE_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fa24a5, SUC_NE_lexicon_tags_2 },
    { 0x645224a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8e24a9, SUC_NE_lexicon_tags_2 },
    { 0x82ba24a9, SUC_NE_lexicon_tags_2 },
    { 0x660424ab, SUC_NE_lexicon_tags_2 },
    { 0x7b9a24ac, SUC_NE_lexicon_tags_2 },
    { 0x5ae824ad, SUC_NE_lexicon_tags_2 },
    { 0x4e3024ac, SUC_NE_lexicon_tags_2 },
    { 0x6fe624ac, SUC_NE_lexicon_tags_2 },
    { 0x90cc24b0, SUC_NE_lexicon_tags_2 },
    { 0x821424b1, SUC_NE_lexicon_tags_2 },
    { 0xdc3e24ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad024b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a9e24b9, SUC_NE_lexicon_tags_2 },
    { 0xe9ce24ba, SUC_NE_lexicon_tags_2 },
    { 0x833e24bb, SUC_NE_lexicon_tags_19 },
    { 0x1e2e24bc, SUC_NE_lexicon_tags_2 },
    { 0xfdee24bd, SUC_NE_lexicon_tags_4 },
    { 0x569424bc, SUC_NE_lexicon_tags_2 },
    { 0x4ede24bf, SUC_NE_lexicon_tags_51 },
    { 0x3a4624c0, SUC_NE_lexicon_tags_2 },
    { 0x822624ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ee24c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x730c24c9, SUC_NE_lexicon_tags_2 },
    { 0xf2624ca, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853024d8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb28a24da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9224dd, SUC_NE_lexicon_tags_2 },
    { 0x13de24de, SUC_NE_lexicon_tags_2 },
    { 0x2c5824de, SUC_NE_lexicon_tags_2 },
    { 0xb8a824e0, SUC_NE_lexicon_tags_2 },
    { 0x450424e1, SUC_NE_lexicon_tags_2 },
    { 0xfef624e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd424e6, SUC_NE_lexicon_tags_4 },
    { 0x284c24e7, SUC_NE_lexicon_tags_2 },
    { 0xd76624e8, SUC_NE_lexicon_tags_2 },
    { 0x182024e6, SUC_NE_lexicon_tags_2 },
    { 0xea5824e6, SUC_NE_lexicon_tags_2 },
    { 0x77624eb, SUC_NE_lexicon_tags_2 },
    { 0x6b6424ec, SUC_NE_lexicon_tags_2 },
    { 0xf60024e8, SUC_NE_lexicon_tags_2 },
    { 0x935024e7, SUC_NE_lexicon_tags_2 },
    { 0x6bf624ed, SUC_NE_lexicon_tags_2 },
    { 0x253024eb, SUC_NE_lexicon_tags_2 },
    { 0xc01c24f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0e24f7, SUC_NE_lexicon_tags_2 },
    { 0x5a4424f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc79824fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efa24fd, SUC_NE_lexicon_tags_2 },
    { 0xba7a24fd, SUC_NE_lexicon_tags_1 },
    { 0x734e24ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95562503, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e82506, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e082509, SUC_NE_lexicon_tags_2 },
    { 0xdc762509, SUC_NE_lexicon_tags_2 },
    { 0x164e250b, SUC_NE_lexicon_tags_2 },
    { 0xce52250c, SUC_NE_lexicon_tags_2 },
    { 0x4f38250d, SUC_NE_lexicon_tags_2 },
    { 0x1a1c250d, SUC_NE_lexicon_tags_2 },
    { 0xcffa250b, SUC_NE_lexicon_tags_14 },
    { 0xf5482510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf202517, SUC_NE_lexicon_tags_2 },
    { 0x2e542518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9014251c, SUC_NE_lexicon_tags_2 },
    { 0x40aa251d, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0x47c4251f, SUC_NE_lexicon_tags_2 },
    { 0x94962520, SUC_NE_lexicon_tags_30 },
    { 0xf0122521, SUC_NE_lexicon_tags_4 },
    { 0x825a2522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d42524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b62526, SUC_NE_lexicon_tags_2 },
    { 0xca022527, SUC_NE_lexicon_tags_2 },
    { 0xbc9e2527, SUC_NE_lexicon_tags_2 },
    { 0x8ad82529, SUC_NE_lexicon_tags_2 },
    { 0xc362529, SUC_NE_lexicon_tags_2 },
    { 0xa7262528, SUC_NE_lexicon_tags_2 },
    { 0x3d12252b, SUC_NE_lexicon_tags_2 },
    { 0xd1ca252d, SUC_NE_lexicon_tags_2 },
    { 0xea0a252e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b422531, SUC_NE_lexicon_tags_58 },
    { 0xfc6c2532, SUC_NE_lexicon_tags_2 },
    { 0x4a0a2533, SUC_NE_lexicon_tags_2 },
    { 0xfa802534, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95e2538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0253c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d18253e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1b62540, SUC_NE_lexicon_tags_2 },
    { 0x8a3e2541, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3a22543, SUC_NE_lexicon_tags_2 },
    { 0xf7962543, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80322546, SUC_NE_lexicon_tags_2 },
    { 0xf80c2546, SUC_NE_lexicon_tags_446 },
    { 0xec02548, SUC_NE_lexicon_tags_2 },
    { 0xbe5c2549, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b6254c, SUC_NE_lexicon_tags_2 },
    { 0xb38254c, SUC_NE_lexicon_tags_4 },
    { 0x3fe2254c, SUC_NE_lexicon_tags_0 },
    { 0x75be254d, SUC_NE_lexicon_tags_2 },
    { 0x56a8254c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70442552, SUC_NE_lexicon_tags_50 },
    { 0xe69a2553, SUC_NE_lexicon_tags_2 },
    { 0x1fa42554, SUC_NE_lexicon_tags_4 },
    { 0x73602555, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9f02557, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ca255b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb598255d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fee255f, SUC_NE_lexicon_tags_2 },
    { 0x5d4e2560, SUC_NE_lexicon_tags_2 },
    { 0xf3b4255f, SUC_NE_lexicon_tags_2 },
    { 0x91362562, SUC_NE_lexicon_tags_2 },
    { 0xb82e2562, SUC_NE_lexicon_tags_2 },
    { 0x9cb42564, SUC_NE_lexicon_tags_3 },
    { 0x11ac2562, SUC_NE_lexicon_tags_3 },
    { 0x2ca2566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc42569, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x53fc256b, SUC_NE_lexicon_tags_2 },
    { 0xca24256c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf506256e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6a2572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8d82574, SUC_NE_lexicon_tags_2 },
    { 0xa5de2574, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44682577, SUC_NE_lexicon_tags_2 },
    { 0x9922578, SUC_NE_lexicon_tags_2 },
    { 0xb43a2579, SUC_NE_lexicon_tags_2 },
    { 0xd0582578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc390257c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d662581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe022584, SUC_NE_lexicon_tags_2 },
    { 0xf3e62585, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6242588, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6708258e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5702590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda2595, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe962599, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910259e, SUC_NE_lexicon_tags_2 },
    { 0x80ba259f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12de25a1, SUC_NE_lexicon_tags_2 },
    { 0x98a25a2, SUC_NE_lexicon_tags_2 },
    { 0xf625a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709c25a8, SUC_NE_lexicon_tags_14 },
    { 0x27ac25a9, SUC_NE_lexicon_tags_2 },
    { 0x603025aa, SUC_NE_lexicon_tags_2 },
    { 0x634825aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8e425ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba625b1, SUC_NE_lexicon_tags_2 },
    { 0x42b225b2, SUC_NE_lexicon_tags_2 },
    { 0xae6625b1, SUC_NE_lexicon_tags_2 },
    { 0xf54225b4, SUC_NE_lexicon_tags_2 },
    { 0x897825b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ba25b8, SUC_NE_lexicon_tags_8 },
    { 0x1d3c25b9, SUC_NE_lexicon_tags_2 },
    { 0x9cdc25ba, SUC_NE_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec225bf, SUC_NE_lexicon_tags_2 },
    { 0xa84425c0, SUC_NE_lexicon_tags_3 },
    { 0xfb8825c1, SUC_NE_lexicon_tags_0 },
    { 0x6c2225c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaed825c4, SUC_NE_lexicon_tags_2 },
    { 0xafc25c5, SUC_NE_lexicon_tags_2 },
    { 0x6ad825c5, SUC_NE_lexicon_tags_2 },
    { 0xbab625c5, SUC_NE_lexicon_tags_2 },
    { 0xeb5625c6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x237225cb, SUC_NE_lexicon_tags_2 },
    { 0xc61025cb, SUC_NE_lexicon_tags_2 },
    { 0x2dc825cd, SUC_NE_lexicon_tags_93 },
    { 0xf13225cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4625d1, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3625d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc25db, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e25de, SUC_NE_lexicon_tags_0 },
    { 0x561025df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f625e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfa25ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x864e25ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498625f7, SUC_NE_lexicon_tags_2 },
    { 0xc21425f8, SUC_NE_lexicon_tags_4 },
    { 0x137e25f7, SUC_NE_lexicon_tags_2 },
    { 0x278a25fa, SUC_NE_lexicon_tags_2 },
    { 0x155025fa, SUC_NE_lexicon_tags_2 },
    { 0xca8425fc, SUC_NE_lexicon_tags_2 },
    { 0xc21825fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1462600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef42602, SUC_NE_lexicon_tags_2 },
    { 0xe662603, SUC_NE_lexicon_tags_2 },
    { 0xf4dc2603, SUC_NE_lexicon_tags_2 },
    { 0x45ac2605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x645e2607, SUC_NE_lexicon_tags_2 },
    { 0xfbf82607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa750260b, SUC_NE_lexicon_tags_2 },
    { 0x5d5a260c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7462613, SUC_NE_lexicon_tags_11 },
    { 0x3ade2613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4442617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a02619, SUC_NE_lexicon_tags_2 },
    { 0x887e261a, SUC_NE_lexicon_tags_2 },
    { 0xee32261a, SUC_NE_lexicon_tags_2 },
    { 0x4b04261b, SUC_NE_lexicon_tags_2 },
    { 0xf4e6261d, SUC_NE_lexicon_tags_2 },
    { 0x3a20261e, SUC_NE_lexicon_tags_2 },
    { 0xf776261f, SUC_NE_lexicon_tags_2 },
    { 0xc28c261e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x367a2622, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f02627, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4f82629, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330e262e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec422632, SUC_NE_lexicon_tags_2 },
    { 0xadf82633, SUC_NE_lexicon_tags_2 },
    { 0xc8ba2632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20442637, SUC_NE_lexicon_tags_2 },
    { 0x79202637, SUC_NE_lexicon_tags_57 },
    { 0x81282637, SUC_NE_lexicon_tags_2 },
    { 0x61b22638, SUC_NE_lexicon_tags_2 },
    { 0x2628263b, SUC_NE_lexicon_tags_2 },
    { 0xf462263c, SUC_NE_lexicon_tags_2 },
    { 0x6d62263d, SUC_NE_lexicon_tags_2 },
    { 0x7256263d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ce2642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56282646, SUC_NE_lexicon_tags_2 },
    { 0x7bf42647, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x512e2649, SUC_NE_lexicon_tags_2 },
    { 0xf3222649, SUC_NE_lexicon_tags_2 },
    { 0xcb88264a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1b0264d, SUC_NE_lexicon_tags_19 },
    { 0x8fd6264d, SUC_NE_lexicon_tags_2 },
    { 0x70fa264d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e2653, SUC_NE_lexicon_tags_2 },
    { 0x8fea2654, SUC_NE_lexicon_tags_2 },
    { 0xac282655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b82658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f06265a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe06265c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd754265e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2222663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a2667, SUC_NE_lexicon_tags_2 },
    { 0x3c002668, SUC_NE_lexicon_tags_2 },
    { 0xee922669, SUC_NE_lexicon_tags_2 },
    { 0xe0042669, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46de266c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6266f, SUC_NE_lexicon_tags_2 },
    { 0x2e02670, SUC_NE_lexicon_tags_2 },
    { 0xb7f02671, SUC_NE_lexicon_tags_2 },
    { 0x3c2c2670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d882674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af0267b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6744267f, SUC_NE_lexicon_tags_2 },
    { 0x25a2267f, SUC_NE_lexicon_tags_2 },
    { 0x48c0267f, SUC_NE_lexicon_tags_2 },
    { 0xf4922682, SUC_NE_lexicon_tags_2 },
    { 0x8e9a2683, SUC_NE_lexicon_tags_1 },
    { 0xd95e2680, SUC_NE_lexicon_tags_58 },
    { 0xb102680, SUC_NE_lexicon_tags_2 },
    { 0xfa0e2681, SUC_NE_lexicon_tags_2 },
    { 0x47f4267f, SUC_NE_lexicon_tags_2 },
    { 0xd7062688, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20ae268a, SUC_NE_lexicon_tags_2 },
    { 0x1ae4268b, SUC_NE_lexicon_tags_2 },
    { 0x74f8268c, SUC_NE_lexicon_tags_30 },
    { 0x86f4268d, SUC_NE_lexicon_tags_2 },
    { 0xcb92268a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71da2690, SUC_NE_lexicon_tags_2 },
    { 0xf2e82691, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8662693, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x27e2695, SUC_NE_lexicon_tags_2 },
    { 0x2f082695, SUC_NE_lexicon_tags_2 },
    { 0x28a82697, SUC_NE_lexicon_tags_2 },
    { 0xf8742698, SUC_NE_lexicon_tags_2 },
    { 0x502699, SUC_NE_lexicon_tags_2 },
    { 0x84622699, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb10269d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d026a1, SUC_NE_lexicon_tags_2 },
    { 0x70bc26a2, SUC_NE_lexicon_tags_2 },
    { 0x31a26a2, SUC_NE_lexicon_tags_2 },
    { 0xfd7626a4, SUC_NE_lexicon_tags_2 },
    { 0x1d2c26a5, SUC_NE_lexicon_tags_4 },
    { 0x35ec26a5, SUC_NE_lexicon_tags_2 },
    { 0xb16626a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fc26ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18d426ae, SUC_NE_lexicon_tags_2 },
    { 0xc34a26ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78e026b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3426b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9e26bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2026c2, SUC_NE_lexicon_tags_2 },
    { 0xb68426c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x957226c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x459c26c8, SUC_NE_lexicon_tags_2 },
    { 0x8e2a26c9, SUC_NE_lexicon_tags_2 },
    { 0xd05826ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2026cd, SUC_NE_lexicon_tags_3 },
    { 0xc9d226cd, SUC_NE_lexicon_tags_2 },
    { 0xc70826cd, SUC_NE_lexicon_tags_2 },
    { 0xaba626d0, SUC_NE_lexicon_tags_2 },
    { 0x6d8026d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2e26d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff2226d6, SUC_NE_lexicon_tags_2 },
    { 0x8fc626d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ce26da, SUC_NE_lexicon_tags_2 },
    { 0xf97c26db, SUC_NE_lexicon_tags_4 },
    { 0x722826dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfea26e6, SUC_NE_lexicon_tags_2 },
    { 0xea6826e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x174a26ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40e26ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11da26f0, SUC_NE_lexicon_tags_7 },
    { 0x633026f1, SUC_NE_lexicon_tags_2 },
    { 0x7ccc26f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c226f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a026f8, SUC_NE_lexicon_tags_2 },
    { 0x8c8826f9, SUC_NE_lexicon_tags_2 },
    { 0xf55e26f9, SUC_NE_lexicon_tags_2 },
    { 0xa72026fb, SUC_NE_lexicon_tags_2 },
    { 0xe4d426fb, SUC_NE_lexicon_tags_2 },
    { 0xe57e26fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538a2701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a2705, SUC_NE_lexicon_tags_2 },
    { 0x90fc2706, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1562709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3e270c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa638270e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e42715, SUC_NE_lexicon_tags_2 },
    { 0xcff02716, SUC_NE_lexicon_tags_2 },
    { 0x40b82716, SUC_NE_lexicon_tags_2 },
    { 0xb562717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ee271b, SUC_NE_lexicon_tags_2 },
    { 0xa51e271c, SUC_NE_lexicon_tags_3 },
    { 0xe9ac271d, SUC_NE_lexicon_tags_7 },
    { 0xd236271e, SUC_NE_lexicon_tags_2 },
    { 0x153a271f, SUC_NE_lexicon_tags_2 },
    { 0x155c271c, SUC_NE_lexicon_tags_2 },
    { 0xf7c2721, SUC_NE_lexicon_tags_2 },
    { 0xcc082722, SUC_NE_lexicon_tags_2 },
    { 0x4b202721, SUC_NE_lexicon_tags_2 },
    { 0xbc422724, SUC_NE_lexicon_tags_2 },
    { 0xb9942725, SUC_NE_lexicon_tags_2 },
    { 0x6cda2722, SUC_NE_lexicon_tags_2 },
    { 0x491c2727, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345e272b, SUC_NE_lexicon_tags_2 },
    { 0x5522272c, SUC_NE_lexicon_tags_3 },
    { 0x98de272d, SUC_NE_lexicon_tags_2 },
    { 0xc032272e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3f02730, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x5da42732, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb20273b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1c273e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9322745, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d42748, SUC_NE_lexicon_tags_4 },
    { 0x26802748, SUC_NE_lexicon_tags_2 },
    { 0x944e274a, SUC_NE_lexicon_tags_2 },
    { 0xa2ca274b, SUC_NE_lexicon_tags_2 },
    { 0x53d0274c, SUC_NE_lexicon_tags_2 },
    { 0x1cb4274d, SUC_NE_lexicon_tags_2 },
    { 0xb66274e, SUC_NE_lexicon_tags_2 },
    { 0x1d50274e, SUC_NE_lexicon_tags_2 },
    { 0xc7b4274e, SUC_NE_lexicon_tags_1 },
    { 0x5c7c2751, SUC_NE_lexicon_tags_2 },
    { 0x6bf62751, SUC_NE_lexicon_tags_2 },
    { 0x547e2751, SUC_NE_lexicon_tags_2 },
    { 0x6ecc274b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fce2757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2e02759, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0275c, SUC_NE_lexicon_tags_2 },
    { 0xf7fe275d, SUC_NE_lexicon_tags_2 },
    { 0xc746275d, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0xf54c2760, SUC_NE_lexicon_tags_2 },
    { 0xc76e2760, SUC_NE_lexicon_tags_2 },
    { 0xe73c2762, SUC_NE_lexicon_tags_91 },
    { 0x1b0c2763, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe42766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd162768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1508276b, SUC_NE_lexicon_tags_2 },
    { 0x22a276b, SUC_NE_lexicon_tags_2 },
    { 0xdcbc276d, SUC_NE_lexicon_tags_2 },
    { 0x3618276b, SUC_NE_lexicon_tags_2 },
    { 0xd8b6276f, SUC_NE_lexicon_tags_2 },
    { 0x1aa42770, SUC_NE_lexicon_tags_2 },
    { 0x4a1e2770, SUC_NE_lexicon_tags_2 },
    { 0x5e9a276d, SUC_NE_lexicon_tags_2 },
    { 0x1002773, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b982776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a882778, SUC_NE_lexicon_tags_2 },
    { 0x656a2779, SUC_NE_lexicon_tags_2 },
    { 0x50bc277a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xba82277c, SUC_NE_lexicon_tags_5 },
    { 0x3114277d, SUC_NE_lexicon_tags_24 },
    { 0x6508277e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x80082780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd45a2782, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56762786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa942789, SUC_NE_lexicon_tags_2 },
    { 0xd524278a, SUC_NE_lexicon_tags_4 },
    { 0x8970278a, SUC_NE_lexicon_tags_2 },
    { 0xba142789, SUC_NE_lexicon_tags_37 },
    { 0xcd3a278a, SUC_NE_lexicon_tags_2 },
    { 0x53ec278b, SUC_NE_lexicon_tags_2 },
    { 0xbd7e278f, SUC_NE_lexicon_tags_2 },
    { 0xe2262790, SUC_NE_lexicon_tags_2 },
    { 0xb8822791, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf962793, SUC_NE_lexicon_tags_2 },
    { 0x747c2794, SUC_NE_lexicon_tags_2 },
    { 0xdd0c2795, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaba279a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc68c279c, SUC_NE_lexicon_tags_2 },
    { 0x5d8a279d, SUC_NE_lexicon_tags_17 },
    { 0xdb94279e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31d427a0, SUC_NE_lexicon_tags_2 },
    { 0x4af627a1, SUC_NE_lexicon_tags_2 },
    { 0x8e6827a2, SUC_NE_lexicon_tags_2 },
    { 0xaec627a3, SUC_NE_lexicon_tags_17 },
    { 0xd49627a4, SUC_NE_lexicon_tags_2 },
    { 0xd11027a5, SUC_NE_lexicon_tags_2 },
    { 0x9e7a27a5, SUC_NE_lexicon_tags_2 },
    { 0xeb2627a1, SUC_NE_lexicon_tags_2 },
    { 0x728a27a0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76827ad, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892c27b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4027b6, SUC_NE_lexicon_tags_2 },
    { 0x56d827b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef027ba, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x680227bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x397027c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d627c3, SUC_NE_lexicon_tags_2 },
    { 0xa19e27c4, SUC_NE_lexicon_tags_3 },
    { 0x9cd427c4, SUC_NE_lexicon_tags_2 },
    { 0xe98c27c6, SUC_NE_lexicon_tags_2 },
    { 0x7b0627c6, SUC_NE_lexicon_tags_4 },
    { 0x352627c7, SUC_NE_lexicon_tags_2 },
    { 0xb6a27c3, SUC_NE_lexicon_tags_2 },
    { 0xc44827ca, SUC_NE_lexicon_tags_2 },
    { 0x709a27cb, SUC_NE_lexicon_tags_2 },
    { 0x11dc27ca, SUC_NE_lexicon_tags_2 },
    { 0xa4ee27ca, SUC_NE_lexicon_tags_2 },
    { 0xd7a827ce, SUC_NE_lexicon_tags_2 },
    { 0x99827cf, SUC_NE_lexicon_tags_2 },
    { 0xefb827d0, SUC_NE_lexicon_tags_2 },
    { 0xa2fc27d1, SUC_NE_lexicon_tags_2 },
    { 0xffb627cb, SUC_NE_lexicon_tags_2 },
    { 0xf96627d3, SUC_NE_lexicon_tags_2 },
    { 0x329c27ce, SUC_NE_lexicon_tags_448 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5e27d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xceea27db, SUC_NE_lexicon_tags_2 },
    { 0x75aa27dc, SUC_NE_lexicon_tags_4 },
    { 0xc36227dd, SUC_NE_lexicon_tags_2 },
    { 0xf8e827de, SUC_NE_lexicon_tags_2 },
    { 0xd31827dc, SUC_NE_lexicon_tags_2 },
    { 0xb41427e0, SUC_NE_lexicon_tags_2 },
    { 0x673827e0, SUC_NE_lexicon_tags_2 },
    { 0xf5e227de, SUC_NE_lexicon_tags_2 },
    { 0xd06427dc, SUC_NE_lexicon_tags_2 },
    { 0xbbf027e0, SUC_NE_lexicon_tags_2 },
    { 0xc99027e5, SUC_NE_lexicon_tags_11 },
    { 0x11bc27e6, SUC_NE_lexicon_tags_7 },
    { 0x2f5227e1, SUC_NE_lexicon_tags_2 },
    { 0x353027e8, SUC_NE_lexicon_tags_2 },
    { 0x774a27e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef027ec, SUC_NE_lexicon_tags_2 },
    { 0x8f6c27ed, SUC_NE_lexicon_tags_2 },
    { 0x4f4a27ee, SUC_NE_lexicon_tags_2 },
    { 0x6a9c27ed, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2dae27f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5ce27f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d3027f5, SUC_NE_lexicon_tags_2 },
    { 0xefee27f6, SUC_NE_lexicon_tags_2 },
    { 0x8e2e27f7, SUC_NE_lexicon_tags_2 },
    { 0x94e227f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x727827fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbc2801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7c2806, SUC_NE_lexicon_tags_1 },
    { 0x234e2807, SUC_NE_lexicon_tags_2 },
    { 0x655c2807, SUC_NE_lexicon_tags_2 },
    { 0x63a02809, SUC_NE_lexicon_tags_19 },
    { 0xced6280a, SUC_NE_lexicon_tags_2 },
    { 0x4b46280b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe44280d, SUC_NE_lexicon_tags_2 },
    { 0x5c2a280e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23f22810, SUC_NE_lexicon_tags_3 },
    { 0xea3a2810, SUC_NE_lexicon_tags_2 },
    { 0x48062812, SUC_NE_lexicon_tags_2 },
    { 0x982a2812, SUC_NE_lexicon_tags_2 },
    { 0x16522810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7362816, SUC_NE_lexicon_tags_2 },
    { 0xab6c2817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1882819, SUC_NE_lexicon_tags_18 },
    { 0x8fa62819, SUC_NE_lexicon_tags_2 },
    { 0x729e281b, SUC_NE_lexicon_tags_2 },
    { 0x1e82281c, SUC_NE_lexicon_tags_2 },
    { 0xc5b6281d, SUC_NE_lexicon_tags_2 },
    { 0xedc2281a, SUC_NE_lexicon_tags_2 },
    { 0x41d6281f, SUC_NE_lexicon_tags_2 },
    { 0x9bcc281a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78282822, SUC_NE_lexicon_tags_89 },
    { 0xe0962822, SUC_NE_lexicon_tags_2 },
    { 0xb5442824, SUC_NE_lexicon_tags_2 },
    { 0x9afc2825, SUC_NE_lexicon_tags_2 },
    { 0x6e342824, SUC_NE_lexicon_tags_2 },
    { 0x8dbc2827, SUC_NE_lexicon_tags_2 },
    { 0x787e2824, SUC_NE_lexicon_tags_2 },
    { 0x89502826, SUC_NE_lexicon_tags_178 },
    { 0x2bd2282a, SUC_NE_lexicon_tags_2 },
    { 0xbde0282b, SUC_NE_lexicon_tags_7 },
    { 0x3672282b, SUC_NE_lexicon_tags_2 },
    { 0xb254282a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22942831, SUC_NE_lexicon_tags_2 },
    { 0x70702832, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9e2836, SUC_NE_lexicon_tags_2 },
    { 0xc4c02836, SUC_NE_lexicon_tags_2 },
    { 0x66ba2838, SUC_NE_lexicon_tags_7 },
    { 0x990e2838, SUC_NE_lexicon_tags_2 },
    { 0x9484283a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd992283c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc762840, SUC_NE_lexicon_tags_2 },
    { 0x6e702840, SUC_NE_lexicon_tags_2 },
    { 0xf7b82842, SUC_NE_lexicon_tags_2 },
    { 0xf83e2842, SUC_NE_lexicon_tags_2 },
    { 0x6822844, SUC_NE_lexicon_tags_4 },
    { 0xfae2845, SUC_NE_lexicon_tags_1 },
    { 0xf1b62845, SUC_NE_lexicon_tags_2 },
    { 0x4e622847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f0284a, SUC_NE_lexicon_tags_2 },
    { 0x2418284b, SUC_NE_lexicon_tags_1 },
    { 0xcff4284c, SUC_NE_lexicon_tags_34 },
    { 0xdffc284d, SUC_NE_lexicon_tags_7 },
    { 0xa272284e, SUC_NE_lexicon_tags_2 },
    { 0x622a284c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1262853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ce22855, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc2285a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2be6285c, SUC_NE_lexicon_tags_2 },
    { 0xd884285d, SUC_NE_lexicon_tags_2 },
    { 0xcbe2285d, SUC_NE_lexicon_tags_2 },
    { 0x22e4285e, SUC_NE_lexicon_tags_220 },
    { 0x81c02860, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2fc2862, SUC_NE_lexicon_tags_2 },
    { 0x11b42863, SUC_NE_lexicon_tags_2 },
    { 0x3fde2863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x215a2867, SUC_NE_lexicon_tags_2 },
    { 0x876e2867, SUC_NE_lexicon_tags_5 },
    { 0x18ca2868, SUC_NE_lexicon_tags_2 },
    { 0xfac6286a, SUC_NE_lexicon_tags_3 },
    { 0xb78a286b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf954286e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5df42870, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e22872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10742874, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc2877, SUC_NE_lexicon_tags_2 },
    { 0xd9122877, SUC_NE_lexicon_tags_2 },
    { 0xedb42879, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd5a287b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3662880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4322288b, SUC_NE_lexicon_tags_2 },
    { 0x5824288c, SUC_NE_lexicon_tags_2 },
    { 0x394c288d, SUC_NE_lexicon_tags_30 },
    { 0x86a4288d, SUC_NE_lexicon_tags_2 },
    { 0x54c0288f, SUC_NE_lexicon_tags_2 },
    { 0x775e288f, SUC_NE_lexicon_tags_3 },
    { 0xd88c2891, SUC_NE_lexicon_tags_2 },
    { 0x5f98288b, SUC_NE_lexicon_tags_2 },
    { 0xefb22893, SUC_NE_lexicon_tags_2 },
    { 0xd9fc288f, SUC_NE_lexicon_tags_2 },
    { 0x3202288b, SUC_NE_lexicon_tags_4 },
    { 0x13f22896, SUC_NE_lexicon_tags_2 },
    { 0x7d6a2896, SUC_NE_lexicon_tags_2 },
    { 0x466c2891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880a289d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69028a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89428a6, SUC_NE_lexicon_tags_8 },
    { 0x89c428a7, SUC_NE_lexicon_tags_2 },
    { 0xc6f628a7, SUC_NE_lexicon_tags_2 },
    { 0xd03028a7, SUC_NE_lexicon_tags_2 },
    { 0xbcfa28a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x537428ac, SUC_NE_lexicon_tags_2 },
    { 0x5de628ac, SUC_NE_lexicon_tags_3 },
    { 0x4a2c28ae, SUC_NE_lexicon_tags_2 },
    { 0xe85228ae, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x882a28b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a28b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69fc28b7, SUC_NE_lexicon_tags_439 },
    { 0x11ec28b8, SUC_NE_lexicon_tags_2 },
    { 0x858a28b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fe628bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d428c1, SUC_NE_lexicon_tags_2 },
    { 0xc6dc28c1, SUC_NE_lexicon_tags_2 },
    { 0x23c428c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb228c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2428cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x425228ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39a28d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe14428d3, SUC_NE_lexicon_tags_2 },
    { 0x48c428d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d828d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34d628db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fb228dd, SUC_NE_lexicon_tags_2 },
    { 0xdeee28de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769428e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8628ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x876428ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be028ef, SUC_NE_lexicon_tags_2 },
    { 0x3cd828f0, SUC_NE_lexicon_tags_2 },
    { 0xb1a828f0, SUC_NE_lexicon_tags_0 },
    { 0xe62628f2, SUC_NE_lexicon_tags_2 },
    { 0x33a028f3, SUC_NE_lexicon_tags_2 },
    { 0x293028f4, SUC_NE_lexicon_tags_2 },
    { 0x9ff228f0, SUC_NE_lexicon_tags_2 },
    { 0x9fe828f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x811428f8, SUC_NE_lexicon_tags_2 },
    { 0x35a828f9, SUC_NE_lexicon_tags_24 },
    { 0xac6828f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a28fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x475e28ff, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb38a2901, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x550e2903, SUC_NE_lexicon_tags_2 },
    { 0xfa622904, SUC_NE_lexicon_tags_2 },
    { 0x68e02905, SUC_NE_lexicon_tags_2 },
    { 0x52382905, SUC_NE_lexicon_tags_2 },
    { 0xd8922906, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708c290c, SUC_NE_lexicon_tags_2 },
    { 0x8442290c, SUC_NE_lexicon_tags_2 },
    { 0xbb72290e, SUC_NE_lexicon_tags_1 },
    { 0x59d4290e, SUC_NE_lexicon_tags_2 },
    { 0x9d8c290e, SUC_NE_lexicon_tags_4 },
    { 0x70aa290c, SUC_NE_lexicon_tags_7 },
    { 0x745e2912, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a52291b, SUC_NE_lexicon_tags_2 },
    { 0xb0dc291c, SUC_NE_lexicon_tags_2 },
    { 0xb80a291d, SUC_NE_lexicon_tags_2 },
    { 0x7cb4291d, SUC_NE_lexicon_tags_2 },
    { 0x9f90291f, SUC_NE_lexicon_tags_3 },
    { 0x63a62920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd762926, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef6292f, SUC_NE_lexicon_tags_2 },
    { 0xb9942930, SUC_NE_lexicon_tags_2 },
    { 0xe14c2931, SUC_NE_lexicon_tags_2 },
    { 0xc6b22930, SUC_NE_lexicon_tags_198 },
    { 0x8cbe2930, SUC_NE_lexicon_tags_2 },
    { 0x10122934, SUC_NE_lexicon_tags_5 },
    { 0x38642935, SUC_NE_lexicon_tags_2 },
    { 0x3fd82936, SUC_NE_lexicon_tags_2 },
    { 0x8da82936, SUC_NE_lexicon_tags_7 },
    { 0xe84e2933, SUC_NE_lexicon_tags_2 },
    { 0xf3fa2939, SUC_NE_lexicon_tags_2 },
    { 0x9e7e293a, SUC_NE_lexicon_tags_2 },
    { 0xfbc4293b, SUC_NE_lexicon_tags_2 },
    { 0x696e293c, SUC_NE_lexicon_tags_2 },
    { 0xb0842939, SUC_NE_lexicon_tags_2 },
    { 0x43462932, SUC_NE_lexicon_tags_2 },
    { 0x7ec02935, SUC_NE_lexicon_tags_2 },
    { 0xbac42940, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fe2944, SUC_NE_lexicon_tags_2 },
    { 0x2a402945, SUC_NE_lexicon_tags_2 },
    { 0xf4ca2946, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20342949, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac46294b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d4294e, SUC_NE_lexicon_tags_2 },
    { 0x8ed8294e, SUC_NE_lexicon_tags_2 },
    { 0x4b982950, SUC_NE_lexicon_tags_2 },
    { 0xc4d02951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1efe2953, SUC_NE_lexicon_tags_2 },
    { 0x1762954, SUC_NE_lexicon_tags_2 },
    { 0xdab62953, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x203c2957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220e295b, SUC_NE_lexicon_tags_2 },
    { 0x797a295c, SUC_NE_lexicon_tags_2 },
    { 0x98c0295d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d9a295f, SUC_NE_lexicon_tags_2 },
    { 0x65d42960, SUC_NE_lexicon_tags_2 },
    { 0xc6fe2960, SUC_NE_lexicon_tags_2 },
    { 0x110e295f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80362964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b66296a, SUC_NE_lexicon_tags_2 },
    { 0x8c02296b, SUC_NE_lexicon_tags_2 },
    { 0xa2c2296c, SUC_NE_lexicon_tags_2 },
    { 0x7996296c, SUC_NE_lexicon_tags_2 },
    { 0x9f8e296e, SUC_NE_lexicon_tags_2 },
    { 0x6a44296f, SUC_NE_lexicon_tags_2 },
    { 0xece0296b, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x327a2972, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535a2979, SUC_NE_lexicon_tags_2 },
    { 0x1af0297a, SUC_NE_lexicon_tags_2 },
    { 0x7a4e297b, SUC_NE_lexicon_tags_2 },
    { 0x564a297a, SUC_NE_lexicon_tags_2 },
    { 0xf748297d, SUC_NE_lexicon_tags_2 },
    { 0xc406297a, SUC_NE_lexicon_tags_1 },
    { 0x95e6297f, SUC_NE_lexicon_tags_2 },
    { 0xf910297f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc62983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56622985, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8616298a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c1e298c, SUC_NE_lexicon_tags_4 },
    { 0xb594298d, SUC_NE_lexicon_tags_2 },
    { 0x8d98298e, SUC_NE_lexicon_tags_7 },
    { 0x84a0298e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b5c2991, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da2994, SUC_NE_lexicon_tags_2 },
    { 0x791e2995, SUC_NE_lexicon_tags_2 },
    { 0xcbe2994, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97902998, SUC_NE_lexicon_tags_2 },
    { 0xee582999, SUC_NE_lexicon_tags_2 },
    { 0xc7d4299a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56299d, SUC_NE_lexicon_tags_2 },
    { 0xbba0299d, SUC_NE_lexicon_tags_2 },
    { 0x2714299f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadc29a6, SUC_NE_lexicon_tags_2 },
    { 0xbd4229a7, SUC_NE_lexicon_tags_1 },
    { 0x192829a6, SUC_NE_lexicon_tags_2 },
    { 0x495229a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b829ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x580229af, SUC_NE_lexicon_tags_7 },
    { 0x2c0429b0, SUC_NE_lexicon_tags_2 },
    { 0xeb429b0, SUC_NE_lexicon_tags_2 },
    { 0xf09c29b1, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x3e9629b4, SUC_NE_lexicon_tags_18 },
    { 0x886029b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x480229b8, SUC_NE_lexicon_tags_18 },
    { 0xc7bc29b9, SUC_NE_lexicon_tags_3 },
    { 0xd7f429ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36029bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27e829bf, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x704629c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x199829c3, SUC_NE_lexicon_tags_2 },
    { 0xf20e29c3, SUC_NE_lexicon_tags_2 },
    { 0x640629c5, SUC_NE_lexicon_tags_2 },
    { 0x89f029c5, SUC_NE_lexicon_tags_2 },
    { 0xf17029c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabd829c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x226a29cc, SUC_NE_lexicon_tags_11 },
    { 0x27bc29cd, SUC_NE_lexicon_tags_2 },
    { 0xc55a29ce, SUC_NE_lexicon_tags_2 },
    { 0x407e29cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bd629d1, SUC_NE_lexicon_tags_4 },
    { 0x2d629d1, SUC_NE_lexicon_tags_13 },
    { 0xc829d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ea29d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x550829d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9d029db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x722829dd, SUC_NE_lexicon_tags_2 },
    { 0xe56429de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88c29e1, SUC_NE_lexicon_tags_2 },
    { 0x109029e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0229e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26b229e7, SUC_NE_lexicon_tags_3 },
    { 0x9ece29e7, SUC_NE_lexicon_tags_2 },
    { 0xff9229e9, SUC_NE_lexicon_tags_32 },
    { 0x688629ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ee29f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb90a29f2, SUC_NE_lexicon_tags_2 },
    { 0x953029f3, SUC_NE_lexicon_tags_2 },
    { 0xdcec29f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76829f7, SUC_NE_lexicon_tags_2 },
    { 0xe6629f8, SUC_NE_lexicon_tags_2 },
    { 0xd6b829f9, SUC_NE_lexicon_tags_2 },
    { 0x84fa29fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x603e29fc, SUC_NE_lexicon_tags_2 },
    { 0xcfcc29fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e002a04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afe2a09, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ed42a0b, SUC_NE_lexicon_tags_2 },
    { 0x8a1c2a0c, SUC_NE_lexicon_tags_2 },
    { 0x3fc82a0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdece2a14, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0e2a18, SUC_NE_lexicon_tags_2 },
    { 0xbde82a19, SUC_NE_lexicon_tags_2 },
    { 0xc4c62a18, SUC_NE_lexicon_tags_7 },
    { 0x26742a1b, SUC_NE_lexicon_tags_4 },
    { 0x3e022a1b, SUC_NE_lexicon_tags_2 },
    { 0xee562a1d, SUC_NE_lexicon_tags_2 },
    { 0xad5a2a1e, SUC_NE_lexicon_tags_2 },
    { 0x42222a1e, SUC_NE_lexicon_tags_2 },
    { 0x42562a1e, SUC_NE_lexicon_tags_2 },
    { 0x8f582a21, SUC_NE_lexicon_tags_2 },
    { 0x32fa2a21, SUC_NE_lexicon_tags_2 },
    { 0x6ee82a21, SUC_NE_lexicon_tags_2 },
    { 0xefd02a24, SUC_NE_lexicon_tags_2 },
    { 0xa7f22a24, SUC_NE_lexicon_tags_2 },
    { 0x19822a26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c22a2a, SUC_NE_lexicon_tags_5 },
    { 0x8d382a2b, SUC_NE_lexicon_tags_2 },
    { 0xc24a2a2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3522a2f, SUC_NE_lexicon_tags_348 },
    { 0x5de42a30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dc42a32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8e2a37, SUC_NE_lexicon_tags_2 },
    { 0x5b802a37, SUC_NE_lexicon_tags_2 },
    { 0x47482a39, SUC_NE_lexicon_tags_2 },
    { 0x299e2a3a, SUC_NE_lexicon_tags_2 },
    { 0xad2e2a39, SUC_NE_lexicon_tags_2 },
    { 0xed982a39, SUC_NE_lexicon_tags_2 },
    { 0x9af22a38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642a42, SUC_NE_lexicon_tags_2 },
    { 0x7cb62a43, SUC_NE_lexicon_tags_2 },
    { 0x25c2a44, SUC_NE_lexicon_tags_2 },
    { 0xed8e2a43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31182a47, SUC_NE_lexicon_tags_7 },
    { 0xd8582a48, SUC_NE_lexicon_tags_2 },
    { 0x60c82a49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8882a4b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca42a4e, SUC_NE_lexicon_tags_2 },
    { 0xf7982a4e, SUC_NE_lexicon_tags_2 },
    { 0x6cb62a50, SUC_NE_lexicon_tags_2 },
    { 0xfec2a51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ab62a53, SUC_NE_lexicon_tags_7 },
    { 0x8aa02a53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86b82a56, SUC_NE_lexicon_tags_2 },
    { 0xfb942a57, SUC_NE_lexicon_tags_2 },
    { 0x50c22a58, SUC_NE_lexicon_tags_2 },
    { 0x803c2a59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1082a5c, SUC_NE_lexicon_tags_2 },
    { 0xbf3c2a5d, SUC_NE_lexicon_tags_7 },
    { 0x36f42a5e, SUC_NE_lexicon_tags_2 },
    { 0x1b342a5f, SUC_NE_lexicon_tags_2 },
    { 0x872e2a60, SUC_NE_lexicon_tags_2 },
    { 0x20c2a5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d42a64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x397a2a66, SUC_NE_lexicon_tags_2 },
    { 0xab8a2a67, SUC_NE_lexicon_tags_2 },
    { 0x1abe2a66, SUC_NE_lexicon_tags_2 },
    { 0x2ba62a69, SUC_NE_lexicon_tags_2 },
    { 0xc9b82a67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f82a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f482a73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x337e2a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af82a78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d82a7f, SUC_NE_lexicon_tags_2 },
    { 0x30f22a80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd82a85, SUC_NE_lexicon_tags_2 },
    { 0x6aa2a86, SUC_NE_lexicon_tags_2 },
    { 0xeaba2a87, SUC_NE_lexicon_tags_2 },
    { 0x12722a88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5a2a8c, SUC_NE_lexicon_tags_2 },
    { 0xcc922a8d, SUC_NE_lexicon_tags_36 },
    { 0xa6862a8e, SUC_NE_lexicon_tags_2 },
    { 0xb4de2a8f, SUC_NE_lexicon_tags_2 },
    { 0x982a8e, SUC_NE_lexicon_tags_2 },
    { 0x60922a91, SUC_NE_lexicon_tags_2 },
    { 0xeb2a2a92, SUC_NE_lexicon_tags_2 },
    { 0xb6742a8d, SUC_NE_lexicon_tags_2 },
    { 0x51c62a94, SUC_NE_lexicon_tags_2 },
    { 0x3ac22a94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7702a98, SUC_NE_lexicon_tags_2 },
    { 0xdebe2a99, SUC_NE_lexicon_tags_2 },
    { 0x14d42a9a, SUC_NE_lexicon_tags_2 },
    { 0x29b22a9b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302a2aa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bc2aa4, SUC_NE_lexicon_tags_2 },
    { 0x71b62aa5, SUC_NE_lexicon_tags_2 },
    { 0x85762aa6, SUC_NE_lexicon_tags_2 },
    { 0x76ca2aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x782c2aaa, SUC_NE_lexicon_tags_2 },
    { 0x45f82aab, SUC_NE_lexicon_tags_2 },
    { 0x9e562aac, SUC_NE_lexicon_tags_2 },
    { 0x93302aab, SUC_NE_lexicon_tags_2 },
    { 0x7fcc2aab, SUC_NE_lexicon_tags_2 },
    { 0x9df62aaf, SUC_NE_lexicon_tags_19 },
    { 0x9adc2ab0, SUC_NE_lexicon_tags_2 },
    { 0xd4762ab0, SUC_NE_lexicon_tags_2 },
    { 0xa8f62ab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84082ab5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca422ab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf062abb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x40282abd, SUC_NE_lexicon_tags_2 },
    { 0xed7a2abd, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xac5e2ac0, SUC_NE_lexicon_tags_2 },
    { 0x11742ac1, SUC_NE_lexicon_tags_2 },
    { 0xe3302ac2, SUC_NE_lexicon_tags_2 },
    { 0x84102ac3, SUC_NE_lexicon_tags_2 },
    { 0x3fae2ac3, SUC_NE_lexicon_tags_102 },
    { 0xe7e22ac5, SUC_NE_lexicon_tags_2 },
    { 0xc95a2ac5, SUC_NE_lexicon_tags_7 },
    { 0x189a2ac5, SUC_NE_lexicon_tags_2 },
    { 0x874e2ac8, SUC_NE_lexicon_tags_2 },
    { 0x80b02ac9, SUC_NE_lexicon_tags_3 },
    { 0x994a2aca, SUC_NE_lexicon_tags_7 },
    { 0x44a42acb, SUC_NE_lexicon_tags_2 },
    { 0xdaf02acb, SUC_NE_lexicon_tags_2 },
    { 0xd50e2acd, SUC_NE_lexicon_tags_2 },
    { 0x82ee2acd, SUC_NE_lexicon_tags_2 },
    { 0x81722ac3, SUC_NE_lexicon_tags_2 },
    { 0x88ae2ad0, SUC_NE_lexicon_tags_2 },
    { 0x34d42ad0, SUC_NE_lexicon_tags_2 },
    { 0xe9362ad0, SUC_NE_lexicon_tags_2 },
    { 0xf67c2ad3, SUC_NE_lexicon_tags_2 },
    { 0x42a2ad3, SUC_NE_lexicon_tags_2 },
    { 0x52662ad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc022ad7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9002adc, SUC_NE_lexicon_tags_2 },
    { 0xb7b22add, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9262ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99be2ae4, SUC_NE_lexicon_tags_2 },
    { 0x41662ae4, SUC_NE_lexicon_tags_14 },
    { 0x24542ae5, SUC_NE_lexicon_tags_2 },
    { 0xdc2c2ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342af2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76202af4, SUC_NE_lexicon_tags_2 },
    { 0x420c2af5, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583c2afa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e62afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd682b00, SUC_NE_lexicon_tags_2 },
    { 0xef162b00, SUC_NE_lexicon_tags_2 },
    { 0xd4762b02, SUC_NE_lexicon_tags_2 },
    { 0xf9bc2b02, SUC_NE_lexicon_tags_2 },
    { 0xe9582b04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c82b09, SUC_NE_lexicon_tags_2 },
    { 0xe2ae2b09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde62b0d, SUC_NE_lexicon_tags_4 },
    { 0x9efe2b0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbfe2b10, SUC_NE_lexicon_tags_7 },
    { 0xb4b22b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213e2b15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc15c2b17, SUC_NE_lexicon_tags_2 },
    { 0xa9242b18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e7a2b1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca002b1e, SUC_NE_lexicon_tags_2 },
    { 0x8c82b1f, SUC_NE_lexicon_tags_2 },
    { 0x37342b20, SUC_NE_lexicon_tags_2 },
    { 0x993c2b21, SUC_NE_lexicon_tags_2 },
    { 0xeec2b21, SUC_NE_lexicon_tags_2 },
    { 0x158a2b23, SUC_NE_lexicon_tags_2 },
    { 0x59042b22, SUC_NE_lexicon_tags_2 },
    { 0xb1fc2b25, SUC_NE_lexicon_tags_2 },
    { 0x978c2b25, SUC_NE_lexicon_tags_0 },
    { 0x48c82b27, SUC_NE_lexicon_tags_2 },
    { 0x60862b1f, SUC_NE_lexicon_tags_2 },
    { 0x394a2b29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4802b2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65562b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3a2b33, SUC_NE_lexicon_tags_2 },
    { 0x298a2b33, SUC_NE_lexicon_tags_2 },
    { 0x58f22b33, SUC_NE_lexicon_tags_2 },
    { 0x3f082b36, SUC_NE_lexicon_tags_1 },
    { 0xefa42b36, SUC_NE_lexicon_tags_2 },
    { 0x23182b38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4c82b3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fa02b3c, SUC_NE_lexicon_tags_0 },
    { 0xd8bc2b3c, SUC_NE_lexicon_tags_2 },
    { 0x6be02b3e, SUC_NE_lexicon_tags_2 },
    { 0x3c302b3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c5c2b41, SUC_NE_lexicon_tags_2 },
    { 0xe8082b42, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0de2b46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4802b49, SUC_NE_lexicon_tags_2 },
    { 0xe7262b4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c082b4d, SUC_NE_lexicon_tags_7 },
    { 0x13402b4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a02b51, SUC_NE_lexicon_tags_2 },
    { 0x92a2b51, SUC_NE_lexicon_tags_2 },
    { 0xcf182b53, SUC_NE_lexicon_tags_2 },
    { 0x44b22b53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e82b59, SUC_NE_lexicon_tags_2 },
    { 0x1ce22b5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94082b60, SUC_NE_lexicon_tags_2 },
    { 0xe87a2b61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de2b65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30822b6a, SUC_NE_lexicon_tags_2 },
    { 0xb5b82b6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc22b6e, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0x79442b70, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1362b75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9102b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a2b80, SUC_NE_lexicon_tags_2 },
    { 0xcda42b81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67742b83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x764e2b85, SUC_NE_lexicon_tags_2 },
    { 0x3b542b86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ff42b88, SUC_NE_lexicon_tags_2 },
    { 0xaed62b89, SUC_NE_lexicon_tags_2 },
    { 0x63122b88, SUC_NE_lexicon_tags_2 },
    { 0xe7c02b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9be2b8d, SUC_NE_lexicon_tags_2 },
    { 0xe8aa2b8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94602b90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62522b92, SUC_NE_lexicon_tags_2 },
    { 0x59dc2b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b42b96, SUC_NE_lexicon_tags_2 },
    { 0xb4842b96, SUC_NE_lexicon_tags_2 },
    { 0xa8b62b98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c82b9e, SUC_NE_lexicon_tags_2 },
    { 0x27862b9e, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9c2ba7, SUC_NE_lexicon_tags_2 },
    { 0x2ea02ba7, SUC_NE_lexicon_tags_2 },
    { 0x7c862ba7, SUC_NE_lexicon_tags_2 },
    { 0x4c0e2ba9, SUC_NE_lexicon_tags_2 },
    { 0x49002baa, SUC_NE_lexicon_tags_7 },
    { 0x3a422ba9, SUC_NE_lexicon_tags_2 },
    { 0xccbc2bad, SUC_NE_lexicon_tags_4 },
    { 0x55b42bae, SUC_NE_lexicon_tags_2 },
    { 0x3d9a2bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8e2bb2, SUC_NE_lexicon_tags_2 },
    { 0x14842bb3, SUC_NE_lexicon_tags_2 },
    { 0x935c2bb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8b82bb6, SUC_NE_lexicon_tags_2 },
    { 0x75f62bb7, SUC_NE_lexicon_tags_4 },
    { 0x783a2bb6, SUC_NE_lexicon_tags_2 },
    { 0x7aac2bb9, SUC_NE_lexicon_tags_2 },
    { 0x646e2bb9, SUC_NE_lexicon_tags_2 },
    { 0x54202bbb, SUC_NE_lexicon_tags_2 },
    { 0x41342bbb, SUC_NE_lexicon_tags_2 },
    { 0x1ec42bbc, SUC_NE_lexicon_tags_2 },
    { 0xdd2a2bbb, SUC_NE_lexicon_tags_2 },
    { 0x8702bbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8082bc1, SUC_NE_lexicon_tags_2 },
    { 0x1b7c2bc2, SUC_NE_lexicon_tags_2 },
    { 0xf07e2bc3, SUC_NE_lexicon_tags_2 },
    { 0x2d442bc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65a42bc6, SUC_NE_lexicon_tags_2 },
    { 0x51ae2bc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4e2bca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ee2bd1, SUC_NE_lexicon_tags_2 },
    { 0xa32a2bd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2942bd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac02bde, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e2e2be0, SUC_NE_lexicon_tags_2 },
    { 0x1f342be1, SUC_NE_lexicon_tags_2 },
    { 0x98a02be1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc2be9, SUC_NE_lexicon_tags_2 },
    { 0xa5b42bea, SUC_NE_lexicon_tags_2 },
    { 0x2d02bea, SUC_NE_lexicon_tags_2 },
    { 0x33b82bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f02bf0, SUC_NE_lexicon_tags_2 },
    { 0x90942bf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b82bf4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x898a2bf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4742bf9, SUC_NE_lexicon_tags_2 },
    { 0x6f762bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98322bfd, SUC_NE_lexicon_tags_2 },
    { 0xd3ca2bfd, SUC_NE_lexicon_tags_2 },
    { 0x52962bff, SUC_NE_lexicon_tags_18 },
    { 0x73602bfd, SUC_NE_lexicon_tags_2 },
    { 0x7d9a2c01, SUC_NE_lexicon_tags_25 },
    { 0xd8b02c00, SUC_NE_lexicon_tags_2 },
    { 0xe7d22c03, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48a2c06, SUC_NE_lexicon_tags_2 },
    { 0x95142c07, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a902c0c, SUC_NE_lexicon_tags_2 },
    { 0x80682c0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a22c11, SUC_NE_lexicon_tags_2 },
    { 0xa4d82c12, SUC_NE_lexicon_tags_2 },
    { 0x95ec2c13, SUC_NE_lexicon_tags_2 },
    { 0xb33c2c11, SUC_NE_lexicon_tags_2 },
    { 0xb53e2c15, SUC_NE_lexicon_tags_2 },
    { 0x48602c16, SUC_NE_lexicon_tags_2 },
    { 0xbc942c17, SUC_NE_lexicon_tags_2 },
    { 0x59582c14, SUC_NE_lexicon_tags_2 },
    { 0x8eae2c19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca882c1b, SUC_NE_lexicon_tags_2 },
    { 0x62282c1c, SUC_NE_lexicon_tags_2 },
    { 0x1a022c1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc862c21, SUC_NE_lexicon_tags_2 },
    { 0x5ffe2c22, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e762c28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23ce2c2a, SUC_NE_lexicon_tags_2 },
    { 0x64c62c2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda2c2d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d862c2f, SUC_NE_lexicon_tags_2 },
    { 0xd76a2c30, SUC_NE_lexicon_tags_46 },
    { 0xfefe2c2f, SUC_NE_lexicon_tags_2 },
    { 0xf2c42c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba762c35, SUC_NE_lexicon_tags_2 },
    { 0xc5b42c36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75242c38, SUC_NE_lexicon_tags_2 },
    { 0x70762c39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0c2c3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4042c41, SUC_NE_lexicon_tags_2 },
    { 0xa102c42, SUC_NE_lexicon_tags_18 },
    { 0xe0002c43, SUC_NE_lexicon_tags_0 },
    { 0x78d42c44, SUC_NE_lexicon_tags_2 },
    { 0xc3c42c42, SUC_NE_lexicon_tags_2 },
    { 0x67342c42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17da2c49, SUC_NE_lexicon_tags_2 },
    { 0x32282c4a, SUC_NE_lexicon_tags_2 },
    { 0x5cfe2c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11b62c4d, SUC_NE_lexicon_tags_252 },
    { 0x5c82c4e, SUC_NE_lexicon_tags_2 },
    { 0xa6c22c4f, SUC_NE_lexicon_tags_2 },
    { 0x264e2c4f, SUC_NE_lexicon_tags_7 },
    { 0x44402c4e, SUC_NE_lexicon_tags_2 },
    { 0x94742c52, SUC_NE_lexicon_tags_2 },
    { 0xc6162c53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c42c56, SUC_NE_lexicon_tags_2 },
    { 0x26d02c57, SUC_NE_lexicon_tags_2 },
    { 0x73b82c57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb02c5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb062c5e, SUC_NE_lexicon_tags_2 },
    { 0xd0562c5e, SUC_NE_lexicon_tags_34 },
    { 0xeec02c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29002c62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2cc2c64, SUC_NE_lexicon_tags_2 },
    { 0xb522c65, SUC_NE_lexicon_tags_1 },
    { 0x48e62c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a762c69, SUC_NE_lexicon_tags_0 },
    { 0xda82c69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f322c6c, SUC_NE_lexicon_tags_2 },
    { 0x6cce2c6c, SUC_NE_lexicon_tags_2 },
    { 0xd8762c6d, SUC_NE_lexicon_tags_2 },
    { 0xc3342c6f, SUC_NE_lexicon_tags_2 },
    { 0x55522c6f, SUC_NE_lexicon_tags_2 },
    { 0x5dca2c71, SUC_NE_lexicon_tags_3 },
    { 0xe482c72, SUC_NE_lexicon_tags_2 },
    { 0x99242c73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa61a2c75, SUC_NE_lexicon_tags_2 },
    { 0x2fc82c76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x430a2c79, SUC_NE_lexicon_tags_2 },
    { 0x26d82c7a, SUC_NE_lexicon_tags_2 },
    { 0xaab22c7a, SUC_NE_lexicon_tags_1 },
    { 0x596e2c7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc82a2c7e, SUC_NE_lexicon_tags_2 },
    { 0xb7e82c7f, SUC_NE_lexicon_tags_2 },
    { 0x9e3e2c80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a262c83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a82c86, SUC_NE_lexicon_tags_2 },
    { 0xb1bc2c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0742c89, SUC_NE_lexicon_tags_2 },
    { 0xe1342c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7702c8d, SUC_NE_lexicon_tags_2 },
    { 0x53e42c8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc22c92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a842c94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2a62c96, SUC_NE_lexicon_tags_2 },
    { 0x5e1c2c96, SUC_NE_lexicon_tags_2 },
    { 0x2e6e2c97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc82c9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de82c9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4882ca2, SUC_NE_lexicon_tags_7 },
    { 0x1f6c2ca3, SUC_NE_lexicon_tags_2 },
    { 0xe3802ca4, SUC_NE_lexicon_tags_2 },
    { 0x86242ca4, SUC_NE_lexicon_tags_2 },
    { 0x34842ca6, SUC_NE_lexicon_tags_2 },
    { 0x1ff22ca3, SUC_NE_lexicon_tags_2 },
    { 0x50e82ca8, SUC_NE_lexicon_tags_2 },
    { 0xeb002ca9, SUC_NE_lexicon_tags_2 },
    { 0x96962ca9, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea62cae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2cb2, SUC_NE_lexicon_tags_2 },
    { 0x63542cb2, SUC_NE_lexicon_tags_2 },
    { 0x322cb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe68a2cb7, SUC_NE_lexicon_tags_2 },
    { 0x6fd22cb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0342cba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99622cbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x982cc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85842cc2, SUC_NE_lexicon_tags_2 },
    { 0x8eb22cc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ba2cc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec62ccb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0762cce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf042cd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78442cda, SUC_NE_lexicon_tags_2 },
    { 0x66822cdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8a2ce0, SUC_NE_lexicon_tags_2 },
    { 0x7e922ce0, SUC_NE_lexicon_tags_1 },
    { 0x7af02ce2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a262ce6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f02ce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa22ceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x229e2cee, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137e2cf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2be82cf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa42cf9, SUC_NE_lexicon_tags_2 },
    { 0x8f622cfa, SUC_NE_lexicon_tags_2 },
    { 0xffa82cfb, SUC_NE_lexicon_tags_2 },
    { 0x234e2cfa, SUC_NE_lexicon_tags_18 },
    { 0xc8bc2cfd, SUC_NE_lexicon_tags_2 },
    { 0xf4ea2cfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e622d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91c82d02, SUC_NE_lexicon_tags_2 },
    { 0x234c2d03, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44482d07, SUC_NE_lexicon_tags_2 },
    { 0x2c022d08, SUC_NE_lexicon_tags_2 },
    { 0x8b7a2d08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89e22d0b, SUC_NE_lexicon_tags_2 },
    { 0x7f822d0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7282d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c62d15, SUC_NE_lexicon_tags_2 },
    { 0xe962d16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30c2d1c, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec42d20, SUC_NE_lexicon_tags_2 },
    { 0xfe302d21, SUC_NE_lexicon_tags_2 },
    { 0xf5f22d20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d162d29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35c02d2b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe6ec2d2d, SUC_NE_lexicon_tags_2 },
    { 0x9a6c2d2d, SUC_NE_lexicon_tags_2 },
    { 0x83422d2f, SUC_NE_lexicon_tags_2 },
    { 0xec282d2f, SUC_NE_lexicon_tags_2 },
    { 0xe4562d31, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4862d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89c2d37, SUC_NE_lexicon_tags_2 },
    { 0x60982d37, SUC_NE_lexicon_tags_2 },
    { 0x3b82d39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8742d3d, SUC_NE_lexicon_tags_2 },
    { 0xc8f62d3e, SUC_NE_lexicon_tags_2 },
    { 0x68ba2d3f, SUC_NE_lexicon_tags_2 },
    { 0x574e2d40, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x58b02d42, SUC_NE_lexicon_tags_2 },
    { 0x42562d43, SUC_NE_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7942d47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48242d49, SUC_NE_lexicon_tags_2 },
    { 0xafe2d4a, SUC_NE_lexicon_tags_32 },
    { 0x5bbc2d4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b522d4e, SUC_NE_lexicon_tags_2 },
    { 0x1342d4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a022d53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90882d56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fa2d5a, SUC_NE_lexicon_tags_2 },
    { 0x59a22d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaa2d5e, SUC_NE_lexicon_tags_24 },
    { 0x36de2d5f, SUC_NE_lexicon_tags_2 },
    { 0x4bc2d60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf5c2d62, SUC_NE_lexicon_tags_2 },
    { 0x34122d63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7102d67, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9502d69, SUC_NE_lexicon_tags_2 },
    { 0x9ee62d6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a462d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0e2d70, SUC_NE_lexicon_tags_2 },
    { 0xc3682d71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe202d73, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x16362d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35442d77, SUC_NE_lexicon_tags_2 },
    { 0x8f582d78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38442d7a, SUC_NE_lexicon_tags_2 },
    { 0xb4082d7a, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102d7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a02d86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a82d8a, SUC_NE_lexicon_tags_2 },
    { 0x267c2d8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d442d8f, SUC_NE_lexicon_tags_2 },
    { 0x792a2d8f, SUC_NE_lexicon_tags_3 },
    { 0x2d302d90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78122d97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e42d9f, SUC_NE_lexicon_tags_2 },
    { 0x52cc2d9f, SUC_NE_lexicon_tags_2 },
    { 0xf2f42da1, SUC_NE_lexicon_tags_2 },
    { 0xd7122da0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde122da5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x44982da7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ebe2da9, SUC_NE_lexicon_tags_2 },
    { 0xaa762daa, SUC_NE_lexicon_tags_2 },
    { 0x48382dab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x205c2dae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbee22db0, SUC_NE_lexicon_tags_2 },
    { 0x76662db1, SUC_NE_lexicon_tags_2 },
    { 0xfbf62db1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5542db8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e22dbb, SUC_NE_lexicon_tags_2 },
    { 0x8a982dbc, SUC_NE_lexicon_tags_2 },
    { 0x98a82dbb, SUC_NE_lexicon_tags_2 },
    { 0x168c2dbe, SUC_NE_lexicon_tags_2 },
    { 0xb9482dbf, SUC_NE_lexicon_tags_2 },
    { 0x4ec82dc0, SUC_NE_lexicon_tags_2 },
    { 0x1dae2dc1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5802dc3, SUC_NE_lexicon_tags_2 },
    { 0x92082dc3, SUC_NE_lexicon_tags_2 },
    { 0xf182dc5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12342dc8, SUC_NE_lexicon_tags_2 },
    { 0xbb142dc9, SUC_NE_lexicon_tags_2 },
    { 0xcd682dc9, SUC_NE_lexicon_tags_2 },
    { 0x967e2dc9, SUC_NE_lexicon_tags_2 },
    { 0x9622dcb, SUC_NE_lexicon_tags_59 },
    { 0x706e2dca, SUC_NE_lexicon_tags_2 },
    { 0xe9962dc9, SUC_NE_lexicon_tags_2 },
    { 0x1b922dcc, SUC_NE_lexicon_tags_7 },
    { 0xe58a2dd0, SUC_NE_lexicon_tags_2 },
    { 0x3afa2dd1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9662dd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3f02dd8, SUC_NE_lexicon_tags_5 },
    { 0xc03a2dd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9a62ddb, SUC_NE_lexicon_tags_49 },
    { 0xc102ddc, SUC_NE_lexicon_tags_3 },
    { 0xf0922ddc, SUC_NE_lexicon_tags_1 },
    { 0x808e2ddd, SUC_NE_lexicon_tags_250 },
    { 0x2ea22ddf, SUC_NE_lexicon_tags_2 },
    { 0x84722de0, SUC_NE_lexicon_tags_6 },
    { 0x174e2de1, SUC_NE_lexicon_tags_2 },
    { 0x8a842de2, SUC_NE_lexicon_tags_2 },
    { 0x765e2de2, SUC_NE_lexicon_tags_2 },
    { 0xde322de3, SUC_NE_lexicon_tags_2 },
    { 0xe44c2de3, SUC_NE_lexicon_tags_2 },
    { 0x807e2de4, SUC_NE_lexicon_tags_2 },
    { 0xbed02de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b162de9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962c2df1, SUC_NE_lexicon_tags_2 },
    { 0x28ca2df1, SUC_NE_lexicon_tags_19 },
    { 0xdb302df3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8182df5, SUC_NE_lexicon_tags_2 },
    { 0xb4ae2df6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabbe2df8, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fa2dfa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61e2dff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67622e08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x971e2e0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9062e0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x806a2e10, SUC_NE_lexicon_tags_2 },
    { 0x20ea2e11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80a02e13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33782e18, SUC_NE_lexicon_tags_2 },
    { 0x25f42e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0a2e1f, SUC_NE_lexicon_tags_7 },
    { 0x431e2e20, SUC_NE_lexicon_tags_2 },
    { 0xa0582e21, SUC_NE_lexicon_tags_445 },
    { 0x44722e22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5362e24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84022e2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde62e31, SUC_NE_lexicon_tags_2 },
    { 0xd1182e32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd58a2e34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa962e36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f282e3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91f62e3e, SUC_NE_lexicon_tags_2 },
    { 0xd93a2e3e, SUC_NE_lexicon_tags_2 },
    { 0x78b42e3e, SUC_NE_lexicon_tags_2 },
    { 0xfa442e3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5442e47, SUC_NE_lexicon_tags_2 },
    { 0x60fa2e47, SUC_NE_lexicon_tags_2 },
    { 0x79ec2e47, SUC_NE_lexicon_tags_2 },
    { 0xfa262e49, SUC_NE_lexicon_tags_2 },
    { 0x62002e4b, SUC_NE_lexicon_tags_2 },
    { 0x1eac2e4c, SUC_NE_lexicon_tags_2 },
    { 0x46482e4d, SUC_NE_lexicon_tags_2 },
    { 0xc8142e4d, SUC_NE_lexicon_tags_2 },
    { 0x84922e4f, SUC_NE_lexicon_tags_7 },
    { 0x78c42e4b, SUC_NE_lexicon_tags_2 },
    { 0x86a22e51, SUC_NE_lexicon_tags_2 },
    { 0x43b62e52, SUC_NE_lexicon_tags_2 },
    { 0x6db82e53, SUC_NE_lexicon_tags_81 },
    { 0xbace2e54, SUC_NE_lexicon_tags_2 },
    { 0xa3462e55, SUC_NE_lexicon_tags_2 },
    { 0x122c2e56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f02e5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e62e5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f62e61, SUC_NE_lexicon_tags_2 },
    { 0xf8822e62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ca2e65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88322e6a, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e62e6d, SUC_NE_lexicon_tags_2 },
    { 0xe6982e6d, SUC_NE_lexicon_tags_25 },
    { 0x5542e6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c2e72, SUC_NE_lexicon_tags_2 },
    { 0xb4102e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda242e79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72b02e7b, SUC_NE_lexicon_tags_2 },
    { 0x6c4a2e7c, SUC_NE_lexicon_tags_7 },
    { 0x2d202e7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35222e7f, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11a2e88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8a2e8f, SUC_NE_lexicon_tags_2 },
    { 0xb02c2e8f, SUC_NE_lexicon_tags_2 },
    { 0xca102e91, SUC_NE_lexicon_tags_2 },
    { 0xf2e22e92, SUC_NE_lexicon_tags_2 },
    { 0xfeca2e92, SUC_NE_lexicon_tags_2 },
    { 0x5d5c2e94, SUC_NE_lexicon_tags_2 },
    { 0xd5fa2e95, SUC_NE_lexicon_tags_2 },
    { 0x41da2e95, SUC_NE_lexicon_tags_2 },
    { 0x23ce2e96, SUC_NE_lexicon_tags_3 },
    { 0x34282e98, SUC_NE_lexicon_tags_2 },
    { 0xcee22e99, SUC_NE_lexicon_tags_2 },
    { 0x9202e9a, SUC_NE_lexicon_tags_102 },
    { 0x57ba2e9b, SUC_NE_lexicon_tags_2 },
    { 0x55262e9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12282e9e, SUC_NE_lexicon_tags_2 },
    { 0x48b62e9f, SUC_NE_lexicon_tags_2 },
    { 0x2c8e2ea0, SUC_NE_lexicon_tags_2 },
    { 0x291c2ea1, SUC_NE_lexicon_tags_2 },
    { 0xaede2e9f, SUC_NE_lexicon_tags_2 },
    { 0x6aea2e9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x496a2ea6, SUC_NE_lexicon_tags_2 },
    { 0xde802ea6, SUC_NE_lexicon_tags_2 },
    { 0x91e22ea8, SUC_NE_lexicon_tags_2 },
    { 0x6fb02ea6, SUC_NE_lexicon_tags_32 },
    { 0x3d922eaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea442ead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e02eb0, SUC_NE_lexicon_tags_2 },
    { 0x32982eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47e2eb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1002eba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x911c2ebd, SUC_NE_lexicon_tags_2 },
    { 0x558c2ebe, SUC_NE_lexicon_tags_2 },
    { 0xae5c2ebe, SUC_NE_lexicon_tags_2 },
    { 0xe4802ec0, SUC_NE_lexicon_tags_2 },
    { 0xcbcc2ec0, SUC_NE_lexicon_tags_2 },
    { 0x313c2ec2, SUC_NE_lexicon_tags_2 },
    { 0xd0822ec3, SUC_NE_lexicon_tags_2 },
    { 0xb79e2ec4, SUC_NE_lexicon_tags_2 },
    { 0x8a322ec5, SUC_NE_lexicon_tags_14 },
    { 0x46142ec5, SUC_NE_lexicon_tags_2 },
    { 0x75542ec5, SUC_NE_lexicon_tags_2 },
    { 0x55442ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3f02eca, SUC_NE_lexicon_tags_2 },
    { 0x95602ecb, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x1a2a2ecd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76162ecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73482ed1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d002ed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab62ed7, SUC_NE_lexicon_tags_2 },
    { 0x38ae2ed8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffc2edc, SUC_NE_lexicon_tags_2 },
    { 0xd7902edd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a82ee1, SUC_NE_lexicon_tags_2 },
    { 0xd2082ee2, SUC_NE_lexicon_tags_0 },
    { 0xf3aa2ee1, SUC_NE_lexicon_tags_4 },
    { 0x77ae2ee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31302eeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cd42eed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52e02eef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6342ef6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x310c2ef9, SUC_NE_lexicon_tags_2 },
    { 0x12d02efa, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x306a2efc, SUC_NE_lexicon_tags_2 },
    { 0xdc162efd, SUC_NE_lexicon_tags_2 },
    { 0xfacc2efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ac2f01, SUC_NE_lexicon_tags_2 },
    { 0xba8e2f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd72a2f04, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x1182f07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79c2f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c742f0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e02f0e, SUC_NE_lexicon_tags_2 },
    { 0xeb602f0f, SUC_NE_lexicon_tags_2 },
    { 0xa0f02f10, SUC_NE_lexicon_tags_40 },
    { 0xec802f0f, SUC_NE_lexicon_tags_2 },
    { 0x80102f12, SUC_NE_lexicon_tags_2 },
    { 0xab902f0f, SUC_NE_lexicon_tags_2 },
    { 0xada2f14, SUC_NE_lexicon_tags_2 },
    { 0x2ce62f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62322f18, SUC_NE_lexicon_tags_2 },
    { 0xd8ce2f19, SUC_NE_lexicon_tags_2 },
    { 0x94622f19, SUC_NE_lexicon_tags_2 },
    { 0x22e02f19, SUC_NE_lexicon_tags_2 },
    { 0xd41e2f1a, SUC_NE_lexicon_tags_2 },
    { 0x34fa2f1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3982f21, SUC_NE_lexicon_tags_2 },
    { 0x9d5e2f21, SUC_NE_lexicon_tags_2 },
    { 0xda8a2f22, SUC_NE_lexicon_tags_2 },
    { 0x1f342f22, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x44f22f26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70222f2a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1442f32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7f42f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a2f38, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3842f3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e002f3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7abe2f3f, SUC_NE_lexicon_tags_2 },
    { 0x22702f40, SUC_NE_lexicon_tags_2 },
    { 0x3c722f40, SUC_NE_lexicon_tags_2 },
    { 0xec942f40, SUC_NE_lexicon_tags_2 },
    { 0x7ee02f43, SUC_NE_lexicon_tags_2 },
    { 0x87a42f43, SUC_NE_lexicon_tags_2 },
    { 0xc6ca2f45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440c2f4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992a2f4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a62f53, SUC_NE_lexicon_tags_2 },
    { 0x9aba2f53, SUC_NE_lexicon_tags_2 },
    { 0x25fc2f54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca482f58, SUC_NE_lexicon_tags_1 },
    { 0x34e2f59, SUC_NE_lexicon_tags_2 },
    { 0xea782f5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae422f5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86882f60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7582f62, SUC_NE_lexicon_tags_2 },
    { 0x88822f63, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x747a2f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3a2f68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13042f6c, SUC_NE_lexicon_tags_2 },
    { 0xc8aa2f6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ba2f7b, SUC_NE_lexicon_tags_328 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ea2f7e, SUC_NE_lexicon_tags_7 },
    { 0x27aa2f7e, SUC_NE_lexicon_tags_2 },
    { 0xe4f02f80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88bc2f82, SUC_NE_lexicon_tags_2 },
    { 0x615c2f83, SUC_NE_lexicon_tags_81 },
    { 0x79442f84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999c2f89, SUC_NE_lexicon_tags_2 },
    { 0xd74e2f8a, SUC_NE_lexicon_tags_19 },
    { 0xa602f8b, SUC_NE_lexicon_tags_2 },
    { 0xb48e2f8b, SUC_NE_lexicon_tags_2 },
    { 0x62fc2f8d, SUC_NE_lexicon_tags_2 },
    { 0x42d22f8b, SUC_NE_lexicon_tags_2 },
    { 0x46982f8f, SUC_NE_lexicon_tags_2 },
    { 0xe5fa2f90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf362f94, SUC_NE_lexicon_tags_270 },
    { 0xb3be2f95, SUC_NE_lexicon_tags_2 },
    { 0x54e2f95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30682f99, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd1e82f9b, SUC_NE_lexicon_tags_2 },
    { 0x26162f9b, SUC_NE_lexicon_tags_2 },
    { 0x62cc2f9d, SUC_NE_lexicon_tags_2 },
    { 0x241e2f9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27722fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5542fa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bcc2fa4, SUC_NE_lexicon_tags_2 },
    { 0xe7702fa5, SUC_NE_lexicon_tags_2 },
    { 0x99f42fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93422fa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7042fac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b562faf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14c2fb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94402fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb62fb7, SUC_NE_lexicon_tags_2 },
    { 0xcdc62fb8, SUC_NE_lexicon_tags_3 },
    { 0x65382fb9, SUC_NE_lexicon_tags_2 },
    { 0x798a2fb9, SUC_NE_lexicon_tags_3 },
    { 0x8b82fb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5d62fbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6042fc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8202fc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83342fc8, SUC_NE_lexicon_tags_2 },
    { 0x45162fc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5c2fce, SUC_NE_lexicon_tags_2 },
    { 0x27082fcf, SUC_NE_lexicon_tags_2 },
    { 0xe22e2fd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62402fd3, SUC_NE_lexicon_tags_2 },
    { 0xeb82fd4, SUC_NE_lexicon_tags_2 },
    { 0x5b8e2fd5, SUC_NE_lexicon_tags_2 },
    { 0x5cca2fd6, SUC_NE_lexicon_tags_2 },
    { 0xfb02fd4, SUC_NE_lexicon_tags_2 },
    { 0xb9762fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406c2fdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c5c2fdf, SUC_NE_lexicon_tags_2 },
    { 0x37b02fe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69242fe3, SUC_NE_lexicon_tags_2 },
    { 0x69b82fe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7a2fe6, SUC_NE_lexicon_tags_2 },
    { 0x5b362fe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70482fe9, SUC_NE_lexicon_tags_2 },
    { 0x954e2fe9, SUC_NE_lexicon_tags_25 },
    { 0xd8322fea, SUC_NE_lexicon_tags_2 },
    { 0x5ee2fec, SUC_NE_lexicon_tags_2 },
    { 0xdd2c2fed, SUC_NE_lexicon_tags_2 },
    { 0xe0242fed, SUC_NE_lexicon_tags_2 },
    { 0x56522fed, SUC_NE_lexicon_tags_5 },
    { 0x96be2ff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c3e2ff2, SUC_NE_lexicon_tags_2 },
    { 0xcbe82ff2, SUC_NE_lexicon_tags_2 },
    { 0xf5842ff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51602ff8, SUC_NE_lexicon_tags_2 },
    { 0x13882ff8, SUC_NE_lexicon_tags_2 },
    { 0x3a2e2ff9, SUC_NE_lexicon_tags_2 },
    { 0x49542ffb, SUC_NE_lexicon_tags_2 },
    { 0x2b9e2ffc, SUC_NE_lexicon_tags_7 },
    { 0xe3742ffb, SUC_NE_lexicon_tags_0 },
    { 0xf5b02ffe, SUC_NE_lexicon_tags_2 },
    { 0x439c2ffe, SUC_NE_lexicon_tags_2 },
    { 0x56cc2ffe, SUC_NE_lexicon_tags_2 },
    { 0xf8c2ffd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864c3005, SUC_NE_lexicon_tags_2 },
    { 0xe183006, SUC_NE_lexicon_tags_2 },
    { 0xd97e3006, SUC_NE_lexicon_tags_2 },
    { 0xa423007, SUC_NE_lexicon_tags_2 },
    { 0x56643008, SUC_NE_lexicon_tags_2 },
    { 0xd460300a, SUC_NE_lexicon_tags_2 },
    { 0xfc72300a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xa822300d, SUC_NE_lexicon_tags_2 },
    { 0x3a96300d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73163011, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f383013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a83016, SUC_NE_lexicon_tags_2 },
    { 0x8a6e3016, SUC_NE_lexicon_tags_2 },
    { 0x9a2e3017, SUC_NE_lexicon_tags_2 },
    { 0x5eac3017, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5ac0301b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e62301e, SUC_NE_lexicon_tags_2 },
    { 0xf496301f, SUC_NE_lexicon_tags_2 },
    { 0x3fa03020, SUC_NE_lexicon_tags_2 },
    { 0x809a3020, SUC_NE_lexicon_tags_2 },
    { 0x939e3020, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1003024, SUC_NE_lexicon_tags_2 },
    { 0xb80e3024, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b843028, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba302c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1656302e, SUC_NE_lexicon_tags_2 },
    { 0x9d2e302f, SUC_NE_lexicon_tags_2 },
    { 0xa52c3030, SUC_NE_lexicon_tags_2 },
    { 0xb7183030, SUC_NE_lexicon_tags_2 },
    { 0x6464302f, SUC_NE_lexicon_tags_7 },
    { 0xcd6e3032, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea3038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67aa303b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7238303e, SUC_NE_lexicon_tags_2 },
    { 0xed20303f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fc63041, SUC_NE_lexicon_tags_2 },
    { 0xbbf03041, SUC_NE_lexicon_tags_2 },
    { 0x87b03043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65f43045, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x76423047, SUC_NE_lexicon_tags_4 },
    { 0x683048, SUC_NE_lexicon_tags_2 },
    { 0x90083048, SUC_NE_lexicon_tags_2 },
    { 0x50ea304a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf818304d, SUC_NE_lexicon_tags_2 },
    { 0xc3b2304d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fc3054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9023056, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61983059, SUC_NE_lexicon_tags_2 },
    { 0x8dc8305a, SUC_NE_lexicon_tags_2 },
    { 0x37c4305b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x550305e, SUC_NE_lexicon_tags_2 },
    { 0x43ca305f, SUC_NE_lexicon_tags_2 },
    { 0x8c363060, SUC_NE_lexicon_tags_2 },
    { 0x52b4305f, SUC_NE_lexicon_tags_2 },
    { 0x81583060, SUC_NE_lexicon_tags_4 },
    { 0xbb8e3063, SUC_NE_lexicon_tags_76 },
    { 0x302c3063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd234306a, SUC_NE_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xe832306d, SUC_NE_lexicon_tags_2 },
    { 0xa5e2306e, SUC_NE_lexicon_tags_2 },
    { 0x6eb6306f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd983072, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b503079, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38ea307b, SUC_NE_lexicon_tags_2 },
    { 0xb816307c, SUC_NE_lexicon_tags_2 },
    { 0x36f2307c, SUC_NE_lexicon_tags_2 },
    { 0xa8e307b, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0xaed83080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ec3086, SUC_NE_lexicon_tags_2 },
    { 0xdf0a3087, SUC_NE_lexicon_tags_2 },
    { 0xfc983086, SUC_NE_lexicon_tags_2 },
    { 0xc6a23086, SUC_NE_lexicon_tags_7 },
    { 0x8a60308a, SUC_NE_lexicon_tags_2 },
    { 0x9656308b, SUC_NE_lexicon_tags_2 },
    { 0x8890308c, SUC_NE_lexicon_tags_2 },
    { 0x77643086, SUC_NE_lexicon_tags_2 },
    { 0xe4fc3088, SUC_NE_lexicon_tags_2 },
    { 0xd9ee308f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdc3097, SUC_NE_lexicon_tags_2 },
    { 0x87ba3098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca90309c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71be309e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47630b1, SUC_NE_lexicon_tags_2 },
    { 0x598430b1, SUC_NE_lexicon_tags_2 },
    { 0x639e30b3, SUC_NE_lexicon_tags_2 },
    { 0x474030b2, SUC_NE_lexicon_tags_2 },
    { 0xefee30b5, SUC_NE_lexicon_tags_2 },
    { 0x872a30b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4c430b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1030bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dc30c2, SUC_NE_lexicon_tags_30 },
    { 0x9ed430c3, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f430c7, SUC_NE_lexicon_tags_2 },
    { 0xb2be30c8, SUC_NE_lexicon_tags_19 },
    { 0xeae630c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5a30cc, SUC_NE_lexicon_tags_4 },
    { 0x27f830cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f430d0, SUC_NE_lexicon_tags_2 },
    { 0xa25830d0, SUC_NE_lexicon_tags_2 },
    { 0xd49630d0, SUC_NE_lexicon_tags_2 },
    { 0xb4e230d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb830d5, SUC_NE_lexicon_tags_2 },
    { 0x80ac30d5, SUC_NE_lexicon_tags_2 },
    { 0x6ac430d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b9430d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5230e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d1c30e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa6a30e7, SUC_NE_lexicon_tags_32 },
    { 0xe9de30e8, SUC_NE_lexicon_tags_2 },
    { 0xbdbe30e8, SUC_NE_lexicon_tags_2 },
    { 0x444a30e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687630f1, SUC_NE_lexicon_tags_7 },
    { 0x138630f2, SUC_NE_lexicon_tags_2 },
    { 0x9b630f3, SUC_NE_lexicon_tags_2 },
    { 0xc38030f3, SUC_NE_lexicon_tags_2 },
    { 0xad8030f4, SUC_NE_lexicon_tags_2 },
    { 0x6ff630f4, SUC_NE_lexicon_tags_2 },
    { 0x5c6830f7, SUC_NE_lexicon_tags_2 },
    { 0xde0c30f8, SUC_NE_lexicon_tags_24 },
    { 0x65a430f7, SUC_NE_lexicon_tags_2 },
    { 0xd28230fa, SUC_NE_lexicon_tags_2 },
    { 0x5c630fb, SUC_NE_lexicon_tags_2 },
    { 0x932630f9, SUC_NE_lexicon_tags_2 },
    { 0xe84030fb, SUC_NE_lexicon_tags_2 },
    { 0x420230f7, SUC_NE_lexicon_tags_2 },
    { 0xbeee30ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef63103, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d823106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203a310a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa38310e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3111, SUC_NE_lexicon_tags_2 },
    { 0x442e3112, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b383114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6723116, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9e311f, SUC_NE_lexicon_tags_2 },
    { 0xcc7c3120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfae83122, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf544312a, SUC_NE_lexicon_tags_2 },
    { 0xeac0312a, SUC_NE_lexicon_tags_2 },
    { 0xcbfe312c, SUC_NE_lexicon_tags_2 },
    { 0xa4d6312d, SUC_NE_lexicon_tags_2 },
    { 0x366312d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ace3132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3563134, SUC_NE_lexicon_tags_2 },
    { 0xb02c3134, SUC_NE_lexicon_tags_2 },
    { 0xd623136, SUC_NE_lexicon_tags_2 },
    { 0xa43e3137, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ca313b, SUC_NE_lexicon_tags_2 },
    { 0x2432313c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e3140, SUC_NE_lexicon_tags_7 },
    { 0x5c3c3141, SUC_NE_lexicon_tags_2 },
    { 0x4263141, SUC_NE_lexicon_tags_2 },
    { 0x93ba3143, SUC_NE_lexicon_tags_1 },
    { 0x818c3143, SUC_NE_lexicon_tags_2 },
    { 0xdd43141, SUC_NE_lexicon_tags_2 },
    { 0x4d523146, SUC_NE_lexicon_tags_3 },
    { 0xebc23141, SUC_NE_lexicon_tags_2 },
    { 0xf7be3142, SUC_NE_lexicon_tags_2 },
    { 0x40363140, SUC_NE_lexicon_tags_2 },
    { 0xd6c03141, SUC_NE_lexicon_tags_2 },
    { 0x654e314b, SUC_NE_lexicon_tags_2 },
    { 0xd2b4314c, SUC_NE_lexicon_tags_2 },
    { 0x76ea314d, SUC_NE_lexicon_tags_2 },
    { 0xbb22314e, SUC_NE_lexicon_tags_37 },
    { 0x1d86314f, SUC_NE_lexicon_tags_2 },
    { 0xd694314b, SUC_NE_lexicon_tags_34 },
    { 0x6ab03151, SUC_NE_lexicon_tags_2 },
    { 0x77a63151, SUC_NE_lexicon_tags_2 },
    { 0x9366314e, SUC_NE_lexicon_tags_2 },
    { 0x85b63148, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33c3156, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d6315d, SUC_NE_lexicon_tags_2 },
    { 0x1fb4315d, SUC_NE_lexicon_tags_2 },
    { 0xc476315f, SUC_NE_lexicon_tags_2 },
    { 0x3410315f, SUC_NE_lexicon_tags_2 },
    { 0x1040315e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e83164, SUC_NE_lexicon_tags_2 },
    { 0x2b4e3165, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34723167, SUC_NE_lexicon_tags_2 },
    { 0x361c3168, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bd4316a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e98316c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb75e316e, SUC_NE_lexicon_tags_2 },
    { 0x212a316f, SUC_NE_lexicon_tags_2 },
    { 0x712a3170, SUC_NE_lexicon_tags_2 },
    { 0x158c3170, SUC_NE_lexicon_tags_11 },
    { 0x77b43170, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x488a3174, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33e3176, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd38317d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe336317f, SUC_NE_lexicon_tags_2 },
    { 0x23443180, SUC_NE_lexicon_tags_2 },
    { 0x98643181, SUC_NE_lexicon_tags_2 },
    { 0xf8b23182, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x31183185, SUC_NE_lexicon_tags_2 },
    { 0xaa7c3185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6a318c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ce318f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56763192, SUC_NE_lexicon_tags_7 },
    { 0x48ae3193, SUC_NE_lexicon_tags_2 },
    { 0xe1663194, SUC_NE_lexicon_tags_2 },
    { 0xd1b03194, SUC_NE_lexicon_tags_7 },
    { 0xbcda3195, SUC_NE_lexicon_tags_2 },
    { 0xbb583192, SUC_NE_lexicon_tags_2 },
    { 0x7e883198, SUC_NE_lexicon_tags_2 },
    { 0x42e43192, SUC_NE_lexicon_tags_2 },
    { 0x74c3193, SUC_NE_lexicon_tags_2 },
    { 0xf4463198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1476319d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f831a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe96631a2, SUC_NE_lexicon_tags_59 },
    { 0x27f631a2, SUC_NE_lexicon_tags_0 },
    { 0x614431a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3c631a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41231a8, SUC_NE_lexicon_tags_2 },
    { 0x915831a9, SUC_NE_lexicon_tags_2 },
    { 0xaf3c31a9, SUC_NE_lexicon_tags_13 },
    { 0xae0431aa, SUC_NE_lexicon_tags_2 },
    { 0x991c31ac, SUC_NE_lexicon_tags_2 },
    { 0x317431ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3831b0, SUC_NE_lexicon_tags_2 },
    { 0x7dca31b1, SUC_NE_lexicon_tags_2 },
    { 0x510e31b0, SUC_NE_lexicon_tags_37 },
    { 0x437631b3, SUC_NE_lexicon_tags_2 },
    { 0x3b4c31b3, SUC_NE_lexicon_tags_2 },
    { 0x679e31b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x429631b7, SUC_NE_lexicon_tags_2 },
    { 0x369631b8, SUC_NE_lexicon_tags_5 },
    { 0x510a31b7, SUC_NE_lexicon_tags_2 },
    { 0xfb6a31ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c431bd, SUC_NE_lexicon_tags_2 },
    { 0xef7c31bd, SUC_NE_lexicon_tags_2 },
    { 0x772831bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38e31c7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13631cc, SUC_NE_lexicon_tags_30 },
    { 0xedcc31cd, SUC_NE_lexicon_tags_2 },
    { 0x609031ce, SUC_NE_lexicon_tags_5 },
    { 0x634c31cf, SUC_NE_lexicon_tags_2 },
    { 0x731631d0, SUC_NE_lexicon_tags_2 },
    { 0x2ed031d1, SUC_NE_lexicon_tags_89 },
    { 0xe03a31d0, SUC_NE_lexicon_tags_2 },
    { 0x3a4c31d3, SUC_NE_lexicon_tags_2 },
    { 0x8fd631d4, SUC_NE_lexicon_tags_2 },
    { 0x45d831cf, SUC_NE_lexicon_tags_4 },
    { 0x95031d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6a31d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0431dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x250431de, SUC_NE_lexicon_tags_2 },
    { 0x8bf631de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f831e5, SUC_NE_lexicon_tags_2 },
    { 0x6c9a31e5, SUC_NE_lexicon_tags_2 },
    { 0xbc7231e7, SUC_NE_lexicon_tags_3 },
    { 0x16a231e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cec31ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcac31f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca231f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c31f7, SUC_NE_lexicon_tags_19 },
    { 0x35031f8, SUC_NE_lexicon_tags_2 },
    { 0x2dfe31f9, SUC_NE_lexicon_tags_2 },
    { 0xcaf831f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e2e31fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78183201, SUC_NE_lexicon_tags_2 },
    { 0x147c3202, SUC_NE_lexicon_tags_1 },
    { 0x778e3203, SUC_NE_lexicon_tags_2 },
    { 0x52bc3204, SUC_NE_lexicon_tags_2 },
    { 0x3fc03201, SUC_NE_lexicon_tags_2 },
    { 0xe89c3206, SUC_NE_lexicon_tags_2 },
    { 0xbb903207, SUC_NE_lexicon_tags_3 },
    { 0x3fa03203, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x647a320a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c20320c, SUC_NE_lexicon_tags_2 },
    { 0x66b6320d, SUC_NE_lexicon_tags_2 },
    { 0x51e2320e, SUC_NE_lexicon_tags_2 },
    { 0xe426320e, SUC_NE_lexicon_tags_2 },
    { 0xb2023210, SUC_NE_lexicon_tags_2 },
    { 0x8f683210, SUC_NE_lexicon_tags_127 },
    { 0xc003212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa603214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9643219, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3c321f, SUC_NE_lexicon_tags_2 },
    { 0xc43220, SUC_NE_lexicon_tags_25 },
    { 0x43e83221, SUC_NE_lexicon_tags_2 },
    { 0x2b3c3221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc23225, SUC_NE_lexicon_tags_2 },
    { 0x18303225, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2e322a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd956322c, SUC_NE_lexicon_tags_2 },
    { 0x767e322d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787e3231, SUC_NE_lexicon_tags_2 },
    { 0xe6543231, SUC_NE_lexicon_tags_2 },
    { 0x1fda3233, SUC_NE_lexicon_tags_2 },
    { 0x11a23234, SUC_NE_lexicon_tags_241 },
    { 0x7f9e3232, SUC_NE_lexicon_tags_3 },
    { 0xa3543231, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8b8a3238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d8323e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43143240, SUC_NE_lexicon_tags_2 },
    { 0x78703241, SUC_NE_lexicon_tags_2 },
    { 0x18883242, SUC_NE_lexicon_tags_2 },
    { 0x990a3242, SUC_NE_lexicon_tags_2 },
    { 0x5c183244, SUC_NE_lexicon_tags_2 },
    { 0x96763242, SUC_NE_lexicon_tags_2 },
    { 0x7c1e3241, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8508324a, SUC_NE_lexicon_tags_2 },
    { 0x46c0324b, SUC_NE_lexicon_tags_4 },
    { 0x1aea324c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3c3251, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26263254, SUC_NE_lexicon_tags_2 },
    { 0xf2863255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af83258, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3e325b, SUC_NE_lexicon_tags_2 },
    { 0xf198325c, SUC_NE_lexicon_tags_34 },
    { 0x67cc325c, SUC_NE_lexicon_tags_18 },
    { 0x1ad4325e, SUC_NE_lexicon_tags_2 },
    { 0xaf2325f, SUC_NE_lexicon_tags_2 },
    { 0xce43260, SUC_NE_lexicon_tags_3 },
    { 0x64a3260, SUC_NE_lexicon_tags_2 },
    { 0x9672325f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2c3265, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x410e3268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878e326f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19fc3271, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7842327b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03e3282, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f63285, SUC_NE_lexicon_tags_2 },
    { 0x31023285, SUC_NE_lexicon_tags_2 },
    { 0xf2f83287, SUC_NE_lexicon_tags_40 },
    { 0x590c3288, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc164328a, SUC_NE_lexicon_tags_0 },
    { 0x8a70328b, SUC_NE_lexicon_tags_2 },
    { 0x9e04328b, SUC_NE_lexicon_tags_2 },
    { 0xc274328d, SUC_NE_lexicon_tags_2 },
    { 0x8162328e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e03295, SUC_NE_lexicon_tags_2 },
    { 0x1fa03295, SUC_NE_lexicon_tags_2 },
    { 0xea8a3297, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4c329e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f1e32a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afc32a7, SUC_NE_lexicon_tags_2 },
    { 0x3cd232a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a232ac, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b232af, SUC_NE_lexicon_tags_2 },
    { 0xd7a032b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec4a32b2, SUC_NE_lexicon_tags_2 },
    { 0xba5c32b3, SUC_NE_lexicon_tags_180 },
    { 0, NULL },
    { 0xa89232b5, SUC_NE_lexicon_tags_2 },
    { 0x2cee32b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x925032b8, SUC_NE_lexicon_tags_2 },
    { 0x283a32b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1a32c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f8832c2, SUC_NE_lexicon_tags_2 },
    { 0xb5e832c3, SUC_NE_lexicon_tags_2 },
    { 0x7c032c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360e32c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19632cf, SUC_NE_lexicon_tags_2 },
    { 0xff632d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe032d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ca832d6, SUC_NE_lexicon_tags_2 },
    { 0x6f8832d7, SUC_NE_lexicon_tags_2 },
    { 0xde6032d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7632dc, SUC_NE_lexicon_tags_2 },
    { 0x85e632dd, SUC_NE_lexicon_tags_2 },
    { 0x7fd032dc, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208a32e5, SUC_NE_lexicon_tags_2 },
    { 0x218832e5, SUC_NE_lexicon_tags_1 },
    { 0x53c432e6, SUC_NE_lexicon_tags_2 },
    { 0x481a32e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c032ea, SUC_NE_lexicon_tags_2 },
    { 0x70de32eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab432f0, SUC_NE_lexicon_tags_2 },
    { 0xcc0c32f1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9632f5, SUC_NE_lexicon_tags_2 },
    { 0x6fb432f6, SUC_NE_lexicon_tags_2 },
    { 0xd30232f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8632fc, SUC_NE_lexicon_tags_2 },
    { 0x94d832fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedba3302, SUC_NE_lexicon_tags_2 },
    { 0x40a23303, SUC_NE_lexicon_tags_2 },
    { 0xb1523304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa463308, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5118330a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67de330c, SUC_NE_lexicon_tags_2 },
    { 0x9770330d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbe3310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc05a3312, SUC_NE_lexicon_tags_2 },
    { 0x32563312, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3710331f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8332b, SUC_NE_lexicon_tags_2 },
    { 0x4388332c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e36332f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07a3333, SUC_NE_lexicon_tags_138 },
    { 0xb1503334, SUC_NE_lexicon_tags_2 },
    { 0x52643335, SUC_NE_lexicon_tags_2 },
    { 0xd08c3336, SUC_NE_lexicon_tags_69 },
    { 0x978c3337, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d98333a, SUC_NE_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd96333f, SUC_NE_lexicon_tags_2 },
    { 0xb143340, SUC_NE_lexicon_tags_2 },
    { 0xb4923341, SUC_NE_lexicon_tags_2 },
    { 0xd3ca3342, SUC_NE_lexicon_tags_2 },
    { 0xa16e3342, SUC_NE_lexicon_tags_2 },
    { 0x253a3343, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c323347, SUC_NE_lexicon_tags_2 },
    { 0x1063348, SUC_NE_lexicon_tags_2 },
    { 0xfcf23347, SUC_NE_lexicon_tags_2 },
    { 0xd778334a, SUC_NE_lexicon_tags_7 },
    { 0x70bc3348, SUC_NE_lexicon_tags_2 },
    { 0x23703348, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acc334f, SUC_NE_lexicon_tags_2 },
    { 0x7d12334f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54803354, SUC_NE_lexicon_tags_4 },
    { 0x11223354, SUC_NE_lexicon_tags_2 },
    { 0xdc9a3354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e023361, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb263363, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120c3367, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ea336a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bf6336c, SUC_NE_lexicon_tags_2 },
    { 0x40e8336c, SUC_NE_lexicon_tags_2 },
    { 0xddec336c, SUC_NE_lexicon_tags_2 },
    { 0x2e0c336c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e6a3371, SUC_NE_lexicon_tags_7 },
    { 0x950c3372, SUC_NE_lexicon_tags_2 },
    { 0xe9323372, SUC_NE_lexicon_tags_2 },
    { 0x29e63372, SUC_NE_lexicon_tags_2 },
    { 0x1dc3375, SUC_NE_lexicon_tags_2 },
    { 0x62203376, SUC_NE_lexicon_tags_1 },
    { 0xb88c3377, SUC_NE_lexicon_tags_2 },
    { 0xa2cc3378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x457c337b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49fa337d, SUC_NE_lexicon_tags_4 },
    { 0x973a337e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e883387, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac43390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x448c3392, SUC_NE_lexicon_tags_2 },
    { 0xd3223393, SUC_NE_lexicon_tags_2 },
    { 0x29103394, SUC_NE_lexicon_tags_1 },
    { 0x7f003393, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfc3398, SUC_NE_lexicon_tags_2 },
    { 0x58643399, SUC_NE_lexicon_tags_2 },
    { 0xe0d6339a, SUC_NE_lexicon_tags_2 },
    { 0x51983398, SUC_NE_lexicon_tags_2 },
    { 0xfabc339b, SUC_NE_lexicon_tags_2 },
    { 0x2f82339d, SUC_NE_lexicon_tags_2 },
    { 0x89e6339d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x387e33a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bba33a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1c33a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8833ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df433b2, SUC_NE_lexicon_tags_2 },
    { 0x3f0e33b2, SUC_NE_lexicon_tags_2 },
    { 0xa41233b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a0833b6, SUC_NE_lexicon_tags_2 },
    { 0xad0233b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x295233b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb633c0, SUC_NE_lexicon_tags_2 },
    { 0xcdfa33c1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e433c4, SUC_NE_lexicon_tags_2 },
    { 0xac9033c4, SUC_NE_lexicon_tags_2 },
    { 0x3c1433c6, SUC_NE_lexicon_tags_2 },
    { 0xf2ba33c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26233ce, SUC_NE_lexicon_tags_2 },
    { 0x5c5a33cf, SUC_NE_lexicon_tags_2 },
    { 0xb5a033cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc9233d2, SUC_NE_lexicon_tags_2 },
    { 0x3df633d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7c33da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4233de, SUC_NE_lexicon_tags_2 },
    { 0xb0dc33de, SUC_NE_lexicon_tags_2 },
    { 0xb98233df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a233e4, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f833ea, SUC_NE_lexicon_tags_2 },
    { 0xab033eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9633f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4a33f9, SUC_NE_lexicon_tags_2 },
    { 0xc46633f9, SUC_NE_lexicon_tags_2 },
    { 0x104833fb, SUC_NE_lexicon_tags_2 },
    { 0x21ce33fc, SUC_NE_lexicon_tags_2 },
    { 0x6aac33fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48863400, SUC_NE_lexicon_tags_2 },
    { 0xc7e63401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde43405, SUC_NE_lexicon_tags_2 },
    { 0xdf523406, SUC_NE_lexicon_tags_2 },
    { 0xb8ea3407, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e8340c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64e340e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc563410, SUC_NE_lexicon_tags_49 },
    { 0x2f4e3411, SUC_NE_lexicon_tags_2 },
    { 0xea9c3412, SUC_NE_lexicon_tags_30 },
    { 0x54c83413, SUC_NE_lexicon_tags_98 },
    { 0xd30c3414, SUC_NE_lexicon_tags_2 },
    { 0x171c3415, SUC_NE_lexicon_tags_2 },
    { 0x3c6c3416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2483418, SUC_NE_lexicon_tags_2 },
    { 0x47e23419, SUC_NE_lexicon_tags_2 },
    { 0xcb9a341a, SUC_NE_lexicon_tags_2 },
    { 0xdcb0341a, SUC_NE_lexicon_tags_2 },
    { 0x38a341c, SUC_NE_lexicon_tags_2 },
    { 0xa520341d, SUC_NE_lexicon_tags_2 },
    { 0xf7463419, SUC_NE_lexicon_tags_2 },
    { 0xc2a4341f, SUC_NE_lexicon_tags_2 },
    { 0xdd803420, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fe3423, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e83427, SUC_NE_lexicon_tags_4 },
    { 0x3f43428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc74342a, SUC_NE_lexicon_tags_7 },
    { 0x2714342b, SUC_NE_lexicon_tags_2 },
    { 0x2012342b, SUC_NE_lexicon_tags_122 },
    { 0xc8be342d, SUC_NE_lexicon_tags_380 },
    { 0, NULL },
    { 0xf54a342f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba823433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e943435, SUC_NE_lexicon_tags_2 },
    { 0xfa6c3436, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a1a3438, SUC_NE_lexicon_tags_2 },
    { 0xca523439, SUC_NE_lexicon_tags_2 },
    { 0x4236343a, SUC_NE_lexicon_tags_2 },
    { 0xa4a0343b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3880343d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e3440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e23443, SUC_NE_lexicon_tags_1 },
    { 0x1cda3444, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd64344a, SUC_NE_lexicon_tags_2 },
    { 0x71d6344a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08344e, SUC_NE_lexicon_tags_4 },
    { 0x40bc344f, SUC_NE_lexicon_tags_2 },
    { 0x77c0344f, SUC_NE_lexicon_tags_34 },
    { 0x1bbe3451, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xde063453, SUC_NE_lexicon_tags_2 },
    { 0x4dfc3453, SUC_NE_lexicon_tags_2 },
    { 0xb2163453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c345b, SUC_NE_lexicon_tags_1 },
    { 0x7e2a345c, SUC_NE_lexicon_tags_2 },
    { 0x74d2345c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9148345f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85083464, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa143467, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d4346a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7592346d, SUC_NE_lexicon_tags_2 },
    { 0xb8d0346e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b63471, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ea3474, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dea3476, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ad03478, SUC_NE_lexicon_tags_14 },
    { 0x93aa3478, SUC_NE_lexicon_tags_2 },
    { 0x43a0347a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac347d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcbc347f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462e3485, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a483488, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c74348b, SUC_NE_lexicon_tags_2 },
    { 0x8f20348c, SUC_NE_lexicon_tags_2 },
    { 0xa34348d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1423490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f43497, SUC_NE_lexicon_tags_2 },
    { 0xa8823497, SUC_NE_lexicon_tags_2 },
    { 0x65f83499, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xaae4349b, SUC_NE_lexicon_tags_2 },
    { 0x1a38349c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1634a0, SUC_NE_lexicon_tags_2 },
    { 0x219234a1, SUC_NE_lexicon_tags_2 },
    { 0xb94234a2, SUC_NE_lexicon_tags_2 },
    { 0x64d434a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437434a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3c34ab, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdca34ae, SUC_NE_lexicon_tags_2 },
    { 0xf11a34af, SUC_NE_lexicon_tags_2 },
    { 0xf80c34ae, SUC_NE_lexicon_tags_2 },
    { 0xf4a834b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd27e34b3, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74234b8, SUC_NE_lexicon_tags_2 },
    { 0x950c34b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1c34c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56634cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e634ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x818234d0, SUC_NE_lexicon_tags_2 },
    { 0x47fe34d1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8434d5, SUC_NE_lexicon_tags_2 },
    { 0x9dfe34d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x702634d8, SUC_NE_lexicon_tags_2 },
    { 0x9e0434d9, SUC_NE_lexicon_tags_2 },
    { 0xc95e34da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12834dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba6a34de, SUC_NE_lexicon_tags_2 },
    { 0xa3c34df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40834e2, SUC_NE_lexicon_tags_2 },
    { 0x9aec34e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd6634e5, SUC_NE_lexicon_tags_2 },
    { 0xa0f034e6, SUC_NE_lexicon_tags_2 },
    { 0xd88434e7, SUC_NE_lexicon_tags_2 },
    { 0x62d234e8, SUC_NE_lexicon_tags_2 },
    { 0x651a34e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2634f2, SUC_NE_lexicon_tags_3 },
    { 0x24034f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245234f8, SUC_NE_lexicon_tags_2 },
    { 0xe1d234f9, SUC_NE_lexicon_tags_4 },
    { 0x48d034fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53dc34fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ba3501, SUC_NE_lexicon_tags_13 },
    { 0x9dbe3502, SUC_NE_lexicon_tags_4 },
    { 0xefa3502, SUC_NE_lexicon_tags_2 },
    { 0xcd603502, SUC_NE_lexicon_tags_2 },
    { 0x75403505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe382350a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fc350d, SUC_NE_lexicon_tags_2 },
    { 0x6c70350e, SUC_NE_lexicon_tags_2 },
    { 0xb4ca350d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97ca3511, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be3514, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d34351b, SUC_NE_lexicon_tags_2 },
    { 0xe544351c, SUC_NE_lexicon_tags_2 },
    { 0x13ea351c, SUC_NE_lexicon_tags_2 },
    { 0xa004351e, SUC_NE_lexicon_tags_2 },
    { 0x72f0351d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ee43521, SUC_NE_lexicon_tags_2 },
    { 0x41cc3522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b183525, SUC_NE_lexicon_tags_0 },
    { 0x390c3525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbba83528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8ca352a, SUC_NE_lexicon_tags_2 },
    { 0x5dea352b, SUC_NE_lexicon_tags_17 },
    { 0x55ae352c, SUC_NE_lexicon_tags_2 },
    { 0x9d78352c, SUC_NE_lexicon_tags_4 },
    { 0xa1c4352c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65623534, SUC_NE_lexicon_tags_59 },
    { 0x34a23535, SUC_NE_lexicon_tags_2 },
    { 0x9a083536, SUC_NE_lexicon_tags_2 },
    { 0xd87a3537, SUC_NE_lexicon_tags_2 },
    { 0xe8d43538, SUC_NE_lexicon_tags_17 },
    { 0x20b63536, SUC_NE_lexicon_tags_2 },
    { 0xa66e3537, SUC_NE_lexicon_tags_2 },
    { 0x8a26353b, SUC_NE_lexicon_tags_13 },
    { 0x7802353b, SUC_NE_lexicon_tags_2 },
    { 0xe2e0353d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec3541, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3103545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80163547, SUC_NE_lexicon_tags_19 },
    { 0x78943548, SUC_NE_lexicon_tags_2 },
    { 0x87563549, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5d4354b, SUC_NE_lexicon_tags_2 },
    { 0x979a354b, SUC_NE_lexicon_tags_2 },
    { 0x743a354d, SUC_NE_lexicon_tags_2 },
    { 0x499e354e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7e3554, SUC_NE_lexicon_tags_4 },
    { 0x5a123554, SUC_NE_lexicon_tags_2 },
    { 0xd8c3556, SUC_NE_lexicon_tags_2 },
    { 0xe38c3557, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d28355b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7706355f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55a03561, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x895e3563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x326e3565, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a82356a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80ca356c, SUC_NE_lexicon_tags_2 },
    { 0x879e356d, SUC_NE_lexicon_tags_2 },
    { 0xba04356e, SUC_NE_lexicon_tags_2 },
    { 0x5056356e, SUC_NE_lexicon_tags_2 },
    { 0xa34a3570, SUC_NE_lexicon_tags_2 },
    { 0x53e0356f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68143573, SUC_NE_lexicon_tags_2 },
    { 0xb6063574, SUC_NE_lexicon_tags_2 },
    { 0x2e3e3575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75f03577, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfca43579, SUC_NE_lexicon_tags_2 },
    { 0xa2a03579, SUC_NE_lexicon_tags_2 },
    { 0x1b38357a, SUC_NE_lexicon_tags_2 },
    { 0xb4e3579, SUC_NE_lexicon_tags_2 },
    { 0x2332357d, SUC_NE_lexicon_tags_4 },
    { 0xb1a8357d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e43582, SUC_NE_lexicon_tags_2 },
    { 0x10da3583, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c358a, SUC_NE_lexicon_tags_2 },
    { 0x451c358b, SUC_NE_lexicon_tags_2 },
    { 0xa458358c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58c358f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeac3592, SUC_NE_lexicon_tags_2 },
    { 0xaea83593, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c63598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7278359d, SUC_NE_lexicon_tags_2 },
    { 0xef16359e, SUC_NE_lexicon_tags_2 },
    { 0x5068359d, SUC_NE_lexicon_tags_2 },
    { 0xfbf635a0, SUC_NE_lexicon_tags_2 },
    { 0x117835a1, SUC_NE_lexicon_tags_2 },
    { 0xb1c435a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb6035a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27de35a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e7635a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5b635ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe235ad, SUC_NE_lexicon_tags_2 },
    { 0x300235ad, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc235b3, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xe15635b5, SUC_NE_lexicon_tags_2 },
    { 0x458235b6, SUC_NE_lexicon_tags_2 },
    { 0xc44235b5, SUC_NE_lexicon_tags_2 },
    { 0xad7835b8, SUC_NE_lexicon_tags_2 },
    { 0xae5835b9, SUC_NE_lexicon_tags_2 },
    { 0x1a7435b8, SUC_NE_lexicon_tags_3 },
    { 0x598835bb, SUC_NE_lexicon_tags_2 },
    { 0x170035bc, SUC_NE_lexicon_tags_2 },
    { 0x7d0235bd, SUC_NE_lexicon_tags_2 },
    { 0x67035b8, SUC_NE_lexicon_tags_2 },
    { 0xbb3c35bf, SUC_NE_lexicon_tags_2 },
    { 0x6f2435bf, SUC_NE_lexicon_tags_5 },
    { 0xf5fe35ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74cc35c3, SUC_NE_lexicon_tags_2 },
    { 0x254235c3, SUC_NE_lexicon_tags_2 },
    { 0x92235c5, SUC_NE_lexicon_tags_2 },
    { 0x1ec235c5, SUC_NE_lexicon_tags_2 },
    { 0x9ffe35c7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a35cc, SUC_NE_lexicon_tags_19 },
    { 0xde9235cd, SUC_NE_lexicon_tags_2 },
    { 0x3ffa35cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea835d2, SUC_NE_lexicon_tags_2 },
    { 0x335035d2, SUC_NE_lexicon_tags_2 },
    { 0xe1dc35d4, SUC_NE_lexicon_tags_2 },
    { 0xa4635d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a035d8, SUC_NE_lexicon_tags_2 },
    { 0x3af435d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe81635db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8d635dd, SUC_NE_lexicon_tags_2 },
    { 0x8c4a35de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c835e2, SUC_NE_lexicon_tags_2 },
    { 0xe26035e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20dc35e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07835eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c35ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9c35f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ac35f9, SUC_NE_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241a35fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b23600, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4103603, SUC_NE_lexicon_tags_2 },
    { 0xe9963603, SUC_NE_lexicon_tags_2 },
    { 0xa7683605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa4a3607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd43611, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70143613, SUC_NE_lexicon_tags_2 },
    { 0xa9ee3614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16ea3616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733e361d, SUC_NE_lexicon_tags_2 },
    { 0x7020361d, SUC_NE_lexicon_tags_2 },
    { 0x3f24361e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd43622, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1503626, SUC_NE_lexicon_tags_2 },
    { 0xcc643627, SUC_NE_lexicon_tags_2 },
    { 0x49623628, SUC_NE_lexicon_tags_2 },
    { 0x2aba3628, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf832362c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d023630, SUC_NE_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c03635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9b03637, SUC_NE_lexicon_tags_2 },
    { 0xcbea3638, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc2363e, SUC_NE_lexicon_tags_2 },
    { 0x7118363f, SUC_NE_lexicon_tags_2 },
    { 0x8996363f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef43643, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b83646, SUC_NE_lexicon_tags_2 },
    { 0x8a23647, SUC_NE_lexicon_tags_2 },
    { 0xf3e03648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de364c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47303650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea343656, SUC_NE_lexicon_tags_4 },
    { 0x13ba3657, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6a03659, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b2e365b, SUC_NE_lexicon_tags_4 },
    { 0x13c6365c, SUC_NE_lexicon_tags_2 },
    { 0x6cf6365d, SUC_NE_lexicon_tags_2 },
    { 0x7984365e, SUC_NE_lexicon_tags_2 },
    { 0xbdda365f, SUC_NE_lexicon_tags_2 },
    { 0xc6b4365b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70343663, SUC_NE_lexicon_tags_2 },
    { 0xf4a83664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db2366c, SUC_NE_lexicon_tags_2 },
    { 0x1460366d, SUC_NE_lexicon_tags_4 },
    { 0x12ac366e, SUC_NE_lexicon_tags_4 },
    { 0xf354366e, SUC_NE_lexicon_tags_4 },
    { 0xd6a23670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db43673, SUC_NE_lexicon_tags_2 },
    { 0x79223674, SUC_NE_lexicon_tags_2 },
    { 0xde843674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab363678, SUC_NE_lexicon_tags_3 },
    { 0x607e3679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d84367b, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b24367f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83ec3681, SUC_NE_lexicon_tags_2 },
    { 0x26043682, SUC_NE_lexicon_tags_2 },
    { 0x35fe3683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b7e3685, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x59643688, SUC_NE_lexicon_tags_2 },
    { 0x3aa23689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c368c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59683691, SUC_NE_lexicon_tags_2 },
    { 0x55de3691, SUC_NE_lexicon_tags_2 },
    { 0x16523693, SUC_NE_lexicon_tags_2 },
    { 0x5a4c3694, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0623696, SUC_NE_lexicon_tags_2 },
    { 0x4d8c3696, SUC_NE_lexicon_tags_3 },
    { 0xa5d63698, SUC_NE_lexicon_tags_11 },
    { 0x95da3698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b8369c, SUC_NE_lexicon_tags_2 },
    { 0x61d2369d, SUC_NE_lexicon_tags_5 },
    { 0x6464369e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f636a1, SUC_NE_lexicon_tags_18 },
    { 0x928a36a2, SUC_NE_lexicon_tags_283 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41636a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x818e36a8, SUC_NE_lexicon_tags_2 },
    { 0x9ad036a8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90e36af, SUC_NE_lexicon_tags_2 },
    { 0x8c3636b0, SUC_NE_lexicon_tags_2 },
    { 0x136a36b0, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b6036b3, SUC_NE_lexicon_tags_2 },
    { 0x641836b4, SUC_NE_lexicon_tags_2 },
    { 0x2a8e36b3, SUC_NE_lexicon_tags_2 },
    { 0x301236b6, SUC_NE_lexicon_tags_2 },
    { 0x49b836b6, SUC_NE_lexicon_tags_2 },
    { 0x8cbc36b8, SUC_NE_lexicon_tags_2 },
    { 0x82d636b5, SUC_NE_lexicon_tags_2 },
    { 0xc57436b8, SUC_NE_lexicon_tags_2 },
    { 0x625636bb, SUC_NE_lexicon_tags_2 },
    { 0xd26236bc, SUC_NE_lexicon_tags_2 },
    { 0xaa1836ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a7236bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d836c7, SUC_NE_lexicon_tags_2 },
    { 0x226236c8, SUC_NE_lexicon_tags_2 },
    { 0xf10a36c9, SUC_NE_lexicon_tags_8 },
    { 0xe3f436c9, SUC_NE_lexicon_tags_2 },
    { 0x5c2436cb, SUC_NE_lexicon_tags_7 },
    { 0x516a36c7, SUC_NE_lexicon_tags_2 },
    { 0x2a2c36cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e436d1, SUC_NE_lexicon_tags_2 },
    { 0x26a836d2, SUC_NE_lexicon_tags_2 },
    { 0x708c36d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59ec36d5, SUC_NE_lexicon_tags_2 },
    { 0xc8d436d5, SUC_NE_lexicon_tags_2 },
    { 0x182e36d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a836da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca836e1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e9636e3, SUC_NE_lexicon_tags_2 },
    { 0x158836e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e5236e6, SUC_NE_lexicon_tags_17 },
    { 0xbca036e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfa836e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12a36ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff7436ee, SUC_NE_lexicon_tags_19 },
    { 0xeb7c36ef, SUC_NE_lexicon_tags_13 },
    { 0x7bac36f0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c4c36f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x458036f5, SUC_NE_lexicon_tags_2 },
    { 0xfd9c36f6, SUC_NE_lexicon_tags_2 },
    { 0x388436f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8e836f9, SUC_NE_lexicon_tags_2 },
    { 0xd26036fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be36ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9fe3701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23fc3703, SUC_NE_lexicon_tags_2 },
    { 0x40c43704, SUC_NE_lexicon_tags_2 },
    { 0x6bac3705, SUC_NE_lexicon_tags_2 },
    { 0xf11a3706, SUC_NE_lexicon_tags_2 },
    { 0xd2c43707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x500e370a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491c370f, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433c3714, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x7afe3716, SUC_NE_lexicon_tags_1 },
    { 0xd4b83717, SUC_NE_lexicon_tags_2 },
    { 0x6c003718, SUC_NE_lexicon_tags_2 },
    { 0x22e23719, SUC_NE_lexicon_tags_2 },
    { 0x204e3718, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d8371f, SUC_NE_lexicon_tags_2 },
    { 0x5f5c371f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4223723, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6583726, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92ca3728, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972372e, SUC_NE_lexicon_tags_2 },
    { 0x370372f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd23735, SUC_NE_lexicon_tags_2 },
    { 0x35743735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c83739, SUC_NE_lexicon_tags_2 },
    { 0xf5ee373a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f08373c, SUC_NE_lexicon_tags_2 },
    { 0xb37e373c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65e3741, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ba3745, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67743748, SUC_NE_lexicon_tags_2 },
    { 0xac863748, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86e374c, SUC_NE_lexicon_tags_2 },
    { 0x8114374d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x19e2374f, SUC_NE_lexicon_tags_2 },
    { 0x9030374f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x27bc3752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ace3757, SUC_NE_lexicon_tags_2 },
    { 0x83103758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926c375c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e8376c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3556376f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x502c3771, SUC_NE_lexicon_tags_261 },
    { 0x3ba3772, SUC_NE_lexicon_tags_2 },
    { 0xf3463773, SUC_NE_lexicon_tags_2 },
    { 0x702e3772, SUC_NE_lexicon_tags_2 },
    { 0x50d83775, SUC_NE_lexicon_tags_2 },
    { 0x78463776, SUC_NE_lexicon_tags_2 },
    { 0xabe3776, SUC_NE_lexicon_tags_2 },
    { 0xe85e3776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e82377a, SUC_NE_lexicon_tags_2 },
    { 0x4642377b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3458377f, SUC_NE_lexicon_tags_113 },
    { 0xdc303780, SUC_NE_lexicon_tags_2 },
    { 0x7ed43781, SUC_NE_lexicon_tags_2 },
    { 0x53da3781, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc032378a, SUC_NE_lexicon_tags_2 },
    { 0x5a88378b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1754378d, SUC_NE_lexicon_tags_2 },
    { 0x68d8378e, SUC_NE_lexicon_tags_2 },
    { 0xdf24378e, SUC_NE_lexicon_tags_2 },
    { 0xd96378f, SUC_NE_lexicon_tags_2 },
    { 0x53e43791, SUC_NE_lexicon_tags_2 },
    { 0x24883792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadb23794, SUC_NE_lexicon_tags_2 },
    { 0x48963794, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa796379b, SUC_NE_lexicon_tags_2 },
    { 0x9392379c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e18379e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfc37a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b237a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee3e37aa, SUC_NE_lexicon_tags_2 },
    { 0xc7e237aa, SUC_NE_lexicon_tags_2 },
    { 0x45637ac, SUC_NE_lexicon_tags_18 },
    { 0x1b7a37aa, SUC_NE_lexicon_tags_2 },
    { 0x7e0037ab, SUC_NE_lexicon_tags_2 },
    { 0x3f6437ad, SUC_NE_lexicon_tags_2 },
    { 0x3bd037ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5a37b4, SUC_NE_lexicon_tags_2 },
    { 0xdf1037b5, SUC_NE_lexicon_tags_2 },
    { 0x4b5237b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bda37bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32cc37bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80e37bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420037c4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f637cc, SUC_NE_lexicon_tags_69 },
    { 0xcd837cd, SUC_NE_lexicon_tags_2 },
    { 0x3de437cd, SUC_NE_lexicon_tags_2 },
    { 0x258037cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567e37d8, SUC_NE_lexicon_tags_2 },
    { 0xec0237d8, SUC_NE_lexicon_tags_2 },
    { 0x7fca37d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81c37dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a637e1, SUC_NE_lexicon_tags_2 },
    { 0x8b4237e2, SUC_NE_lexicon_tags_2 },
    { 0x905037e3, SUC_NE_lexicon_tags_2 },
    { 0xd02037e4, SUC_NE_lexicon_tags_2 },
    { 0x335c37e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba3c37e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaafc37e9, SUC_NE_lexicon_tags_2 },
    { 0x630a37ea, SUC_NE_lexicon_tags_2 },
    { 0x550c37eb, SUC_NE_lexicon_tags_2 },
    { 0xceca37ec, SUC_NE_lexicon_tags_18 },
    { 0xce6c37ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d037f4, SUC_NE_lexicon_tags_2 },
    { 0xc77e37f5, SUC_NE_lexicon_tags_2 },
    { 0x14b637f6, SUC_NE_lexicon_tags_2 },
    { 0x2ea37f6, SUC_NE_lexicon_tags_2 },
    { 0xbe5437f8, SUC_NE_lexicon_tags_2 },
    { 0x6de237f9, SUC_NE_lexicon_tags_2 },
    { 0xc0637f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd05237fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ac3801, SUC_NE_lexicon_tags_2 },
    { 0xce9a3802, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d463808, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x765e380b, SUC_NE_lexicon_tags_2 },
    { 0x3d6380c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4c380f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dfc3811, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x878c3814, SUC_NE_lexicon_tags_0 },
    { 0xea2e3815, SUC_NE_lexicon_tags_2 },
    { 0xd1f03816, SUC_NE_lexicon_tags_2 },
    { 0x88763816, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4c381a, SUC_NE_lexicon_tags_8 },
    { 0x6e22381a, SUC_NE_lexicon_tags_7 },
    { 0x2e56381c, SUC_NE_lexicon_tags_2 },
    { 0x6d98381c, SUC_NE_lexicon_tags_2 },
    { 0xf4f8381e, SUC_NE_lexicon_tags_2 },
    { 0xeecc381b, SUC_NE_lexicon_tags_2 },
    { 0x96ba381c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb5e3822, SUC_NE_lexicon_tags_2 },
    { 0x49303822, SUC_NE_lexicon_tags_2 },
    { 0x87da3823, SUC_NE_lexicon_tags_2 },
    { 0x5ff23825, SUC_NE_lexicon_tags_18 },
    { 0xbd023826, SUC_NE_lexicon_tags_2 },
    { 0x92fe3827, SUC_NE_lexicon_tags_2 },
    { 0x752e3826, SUC_NE_lexicon_tags_2 },
    { 0xb34c3828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94d2382b, SUC_NE_lexicon_tags_2 },
    { 0x98dc382c, SUC_NE_lexicon_tags_2 },
    { 0x8cec382b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a3831, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3223835, SUC_NE_lexicon_tags_2 },
    { 0x62bc3836, SUC_NE_lexicon_tags_2 },
    { 0x496c3836, SUC_NE_lexicon_tags_1 },
    { 0x7863836, SUC_NE_lexicon_tags_2 },
    { 0x4b983839, SUC_NE_lexicon_tags_2 },
    { 0x32283836, SUC_NE_lexicon_tags_2 },
    { 0xa1343839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18fc383d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x720a3840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89763842, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ee3845, SUC_NE_lexicon_tags_2 },
    { 0x5e3e3846, SUC_NE_lexicon_tags_2 },
    { 0xd0103847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401e384b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181c3850, SUC_NE_lexicon_tags_4 },
    { 0x76c43850, SUC_NE_lexicon_tags_2 },
    { 0xf8103852, SUC_NE_lexicon_tags_2 },
    { 0xebc83853, SUC_NE_lexicon_tags_2 },
    { 0xb8ee3854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf83857, SUC_NE_lexicon_tags_5 },
    { 0xd2fe3857, SUC_NE_lexicon_tags_2 },
    { 0xe57c3859, SUC_NE_lexicon_tags_2 },
    { 0xe470385a, SUC_NE_lexicon_tags_2 },
    { 0xe638385b, SUC_NE_lexicon_tags_4 },
    { 0x82ee385b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1c385f, SUC_NE_lexicon_tags_2 },
    { 0x50d63860, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f403863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5c3866, SUC_NE_lexicon_tags_2 },
    { 0x3fd63866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbdc3869, SUC_NE_lexicon_tags_2 },
    { 0x592c3869, SUC_NE_lexicon_tags_2 },
    { 0xe0d8386b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7882386e, SUC_NE_lexicon_tags_2 },
    { 0x1eb4386f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72ba3871, SUC_NE_lexicon_tags_2 },
    { 0x3a223872, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b03875, SUC_NE_lexicon_tags_2 },
    { 0xa5243876, SUC_NE_lexicon_tags_2 },
    { 0x518a3876, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3c387b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a8387e, SUC_NE_lexicon_tags_2 },
    { 0xea52387f, SUC_NE_lexicon_tags_17 },
    { 0x84b83880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b03884, SUC_NE_lexicon_tags_2 },
    { 0xa803885, SUC_NE_lexicon_tags_2 },
    { 0x84263886, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ae388f, SUC_NE_lexicon_tags_4 },
    { 0xcf63890, SUC_NE_lexicon_tags_2 },
    { 0xd90a3890, SUC_NE_lexicon_tags_2 },
    { 0x1f423892, SUC_NE_lexicon_tags_2 },
    { 0xfe363893, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9143895, SUC_NE_lexicon_tags_2 },
    { 0xf9603896, SUC_NE_lexicon_tags_2 },
    { 0x7d183897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa392389b, SUC_NE_lexicon_tags_2 },
    { 0x6f48389c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d238a3, SUC_NE_lexicon_tags_2 },
    { 0xf9fa38a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd68a38a6, SUC_NE_lexicon_tags_2 },
    { 0xc73a38a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd238ac, SUC_NE_lexicon_tags_2 },
    { 0x6f1c38ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45be38b4, SUC_NE_lexicon_tags_2 },
    { 0x35da38b5, SUC_NE_lexicon_tags_7 },
    { 0xc94a38b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d2a38b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x491a38ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe81638bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f5038be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x438c1, SUC_NE_lexicon_tags_2 },
    { 0xdd5c38c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da038c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e38d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x498038d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3438d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee038d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x309438dc, SUC_NE_lexicon_tags_2 },
    { 0x9d8638dc, SUC_NE_lexicon_tags_2 },
    { 0x9c1438dc, SUC_NE_lexicon_tags_2 },
    { 0x608438df, SUC_NE_lexicon_tags_2 },
    { 0x796838e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f838e3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a238e6, SUC_NE_lexicon_tags_4 },
    { 0xae3638e7, SUC_NE_lexicon_tags_2 },
    { 0x600c38e8, SUC_NE_lexicon_tags_2 },
    { 0xd9fc38e9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb4638eb, SUC_NE_lexicon_tags_2 },
    { 0xbc6438ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7c238ee, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e38f2, SUC_NE_lexicon_tags_2 },
    { 0x4f3438f3, SUC_NE_lexicon_tags_14 },
    { 0x31b438f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7038f9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f9238fb, SUC_NE_lexicon_tags_1 },
    { 0xd19438fc, SUC_NE_lexicon_tags_121 },
    { 0xd10038fc, SUC_NE_lexicon_tags_2 },
    { 0x525038fe, SUC_NE_lexicon_tags_2 },
    { 0xf1838ff, SUC_NE_lexicon_tags_2 },
    { 0x346c3900, SUC_NE_lexicon_tags_2 },
    { 0x430e3901, SUC_NE_lexicon_tags_2 },
    { 0xdcd23902, SUC_NE_lexicon_tags_2 },
    { 0x74a03902, SUC_NE_lexicon_tags_40 },
    { 0x63903904, SUC_NE_lexicon_tags_3 },
    { 0x37383904, SUC_NE_lexicon_tags_2 },
    { 0xbe1e3905, SUC_NE_lexicon_tags_2 },
    { 0x185438ff, SUC_NE_lexicon_tags_2 },
    { 0xb6503902, SUC_NE_lexicon_tags_2 },
    { 0xdd123909, SUC_NE_lexicon_tags_2 },
    { 0x5af2390a, SUC_NE_lexicon_tags_2 },
    { 0x5f4e390a, SUC_NE_lexicon_tags_2 },
    { 0x9288390c, SUC_NE_lexicon_tags_2 },
    { 0x3bf6390d, SUC_NE_lexicon_tags_2 },
    { 0x3d6e390e, SUC_NE_lexicon_tags_2 },
    { 0x8bc4390d, SUC_NE_lexicon_tags_2 },
    { 0xb0ee3910, SUC_NE_lexicon_tags_2 },
    { 0x7a6c3911, SUC_NE_lexicon_tags_2 },
    { 0x65ae3912, SUC_NE_lexicon_tags_7 },
    { 0xb3343912, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xd00e3915, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84243919, SUC_NE_lexicon_tags_2 },
    { 0xc6b2391a, SUC_NE_lexicon_tags_2 },
    { 0x1922391b, SUC_NE_lexicon_tags_2 },
    { 0xefee391c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a78391f, SUC_NE_lexicon_tags_2 },
    { 0xd47c3920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31823923, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a183929, SUC_NE_lexicon_tags_2 },
    { 0x90a8392a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc712392e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0a3930, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3943934, SUC_NE_lexicon_tags_2 },
    { 0x11d83935, SUC_NE_lexicon_tags_2 },
    { 0xa16e3934, SUC_NE_lexicon_tags_2 },
    { 0x96783937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b34393c, SUC_NE_lexicon_tags_2 },
    { 0xcb72393c, SUC_NE_lexicon_tags_2 },
    { 0xba80393e, SUC_NE_lexicon_tags_2 },
    { 0xe552393f, SUC_NE_lexicon_tags_3 },
    { 0x65f23940, SUC_NE_lexicon_tags_2 },
    { 0x7d143940, SUC_NE_lexicon_tags_2 },
    { 0xd0723942, SUC_NE_lexicon_tags_2 },
    { 0xe716393d, SUC_NE_lexicon_tags_1 },
    { 0xc2e83944, SUC_NE_lexicon_tags_2 },
    { 0xbdd83943, SUC_NE_lexicon_tags_2 },
    { 0x2ac2393c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7463948, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4896394a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x859a394c, SUC_NE_lexicon_tags_2 },
    { 0xd1c8394d, SUC_NE_lexicon_tags_2 },
    { 0x8e9c394e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25b83950, SUC_NE_lexicon_tags_7 },
    { 0x2dd63951, SUC_NE_lexicon_tags_2 },
    { 0xb4783951, SUC_NE_lexicon_tags_2 },
    { 0x5a4e3953, SUC_NE_lexicon_tags_2 },
    { 0x1f663954, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd723956, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0de395c, SUC_NE_lexicon_tags_2 },
    { 0xb8ae395d, SUC_NE_lexicon_tags_2 },
    { 0x2c7c395c, SUC_NE_lexicon_tags_3 },
    { 0xe794395d, SUC_NE_lexicon_tags_2 },
    { 0x789c3960, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8583967, SUC_NE_lexicon_tags_2 },
    { 0x17a43968, SUC_NE_lexicon_tags_2 },
    { 0x53cc3969, SUC_NE_lexicon_tags_18 },
    { 0xddc8396a, SUC_NE_lexicon_tags_2 },
    { 0xcdec3969, SUC_NE_lexicon_tags_2 },
    { 0xb4d8396a, SUC_NE_lexicon_tags_2 },
    { 0x4fec396d, SUC_NE_lexicon_tags_2 },
    { 0xe1a4396b, SUC_NE_lexicon_tags_2 },
    { 0x89cc396f, SUC_NE_lexicon_tags_2 },
    { 0xa2223970, SUC_NE_lexicon_tags_2 },
    { 0x87343970, SUC_NE_lexicon_tags_2 },
    { 0x2f08396a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b963975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33e3979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4ec397b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4c3981, SUC_NE_lexicon_tags_124 },
    { 0x3e5c3981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc19a3984, SUC_NE_lexicon_tags_4 },
    { 0xb2f43985, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c22398c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb82e398e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ec3994, SUC_NE_lexicon_tags_2 },
    { 0x80063995, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb803997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38399a, SUC_NE_lexicon_tags_2 },
    { 0xc59e399a, SUC_NE_lexicon_tags_2 },
    { 0xc134399a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ca39a1, SUC_NE_lexicon_tags_2 },
    { 0x7e1c39a2, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6c39a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80839ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12039b1, SUC_NE_lexicon_tags_2 },
    { 0x7e5c39b2, SUC_NE_lexicon_tags_2 },
    { 0x638839b3, SUC_NE_lexicon_tags_2 },
    { 0x35c239b3, SUC_NE_lexicon_tags_2 },
    { 0xa45e39b5, SUC_NE_lexicon_tags_2 },
    { 0xe5e439b6, SUC_NE_lexicon_tags_2 },
    { 0x9b8839b7, SUC_NE_lexicon_tags_2 },
    { 0xf98239b6, SUC_NE_lexicon_tags_2 },
    { 0xbff039b6, SUC_NE_lexicon_tags_2 },
    { 0xd78a39b9, SUC_NE_lexicon_tags_2 },
    { 0xf07239b7, SUC_NE_lexicon_tags_2 },
    { 0x303839ba, SUC_NE_lexicon_tags_2 },
    { 0xceca39bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e39c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7839c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d039cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8e39d0, SUC_NE_lexicon_tags_2 },
    { 0xea4839d1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762839d5, SUC_NE_lexicon_tags_2 },
    { 0xdd1239d6, SUC_NE_lexicon_tags_2 },
    { 0x36a839d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f6e39d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee639dc, SUC_NE_lexicon_tags_2 },
    { 0xf82439dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x529039df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa79639e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55239e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da639ee, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc439f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7439f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa9839f7, SUC_NE_lexicon_tags_2 },
    { 0x19d039f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0a239fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaac39fe, SUC_NE_lexicon_tags_2 },
    { 0x9be239ff, SUC_NE_lexicon_tags_2 },
    { 0xa16e39ff, SUC_NE_lexicon_tags_2 },
    { 0xa2983a01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe68c3a03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37903a06, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde43a0b, SUC_NE_lexicon_tags_2 },
    { 0xb0903a0c, SUC_NE_lexicon_tags_2 },
    { 0x14f83a0c, SUC_NE_lexicon_tags_2 },
    { 0xe2d43a0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4e3a15, SUC_NE_lexicon_tags_2 },
    { 0x8383a16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a663a1d, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0a3a20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80583a23, SUC_NE_lexicon_tags_2 },
    { 0x8c9e3a24, SUC_NE_lexicon_tags_2 },
    { 0xb8043a25, SUC_NE_lexicon_tags_2 },
    { 0xcd4c3a24, SUC_NE_lexicon_tags_2 },
    { 0xa42a3a24, SUC_NE_lexicon_tags_2 },
    { 0xcc623a28, SUC_NE_lexicon_tags_2 },
    { 0x6fec3a23, SUC_NE_lexicon_tags_2 },
    { 0xb6843a2a, SUC_NE_lexicon_tags_2 },
    { 0xc8f63a2b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2f923a2d, SUC_NE_lexicon_tags_2 },
    { 0xaaf43a2e, SUC_NE_lexicon_tags_2 },
    { 0x7703a2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66723a33, SUC_NE_lexicon_tags_2 },
    { 0x7c703a34, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81a3a38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb363a3d, SUC_NE_lexicon_tags_2 },
    { 0x5b703a3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab103a40, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe1323a42, SUC_NE_lexicon_tags_2 },
    { 0xa3283a43, SUC_NE_lexicon_tags_2 },
    { 0xd363a44, SUC_NE_lexicon_tags_2 },
    { 0x8bde3a45, SUC_NE_lexicon_tags_2 },
    { 0x93803a44, SUC_NE_lexicon_tags_2 },
    { 0x57383a47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c483a4c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e6a3a4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15203a51, SUC_NE_lexicon_tags_2 },
    { 0x83543a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6a3a55, SUC_NE_lexicon_tags_2 },
    { 0xc0ae3a56, SUC_NE_lexicon_tags_2 },
    { 0xd3d83a57, SUC_NE_lexicon_tags_2 },
    { 0x8f643a58, SUC_NE_lexicon_tags_2 },
    { 0x84a43a55, SUC_NE_lexicon_tags_2 },
    { 0x65bc3a57, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2a043a5c, SUC_NE_lexicon_tags_1 },
    { 0x291e3a5d, SUC_NE_lexicon_tags_2 },
    { 0xfb083a5e, SUC_NE_lexicon_tags_4 },
    { 0xbaae3a5f, SUC_NE_lexicon_tags_2 },
    { 0xa2fe3a60, SUC_NE_lexicon_tags_2 },
    { 0xa1043a61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b503a6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef63a6d, SUC_NE_lexicon_tags_2 },
    { 0x59523a6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75a3a75, SUC_NE_lexicon_tags_2 },
    { 0x250a3a76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9a3a7a, SUC_NE_lexicon_tags_5 },
    { 0xa69e3a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dc3a7f, SUC_NE_lexicon_tags_2 },
    { 0x16343a80, SUC_NE_lexicon_tags_2 },
    { 0x44cc3a80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76d43a83, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d43a86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e2a3a88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87063a8a, SUC_NE_lexicon_tags_17 },
    { 0xc98e3a8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c9a3a8d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x47763a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599c3a9d, SUC_NE_lexicon_tags_2 },
    { 0xdaf23a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85763aa2, SUC_NE_lexicon_tags_19 },
    { 0x3f6c3aa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48363aa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6c03aa7, SUC_NE_lexicon_tags_2 },
    { 0x6c543aa8, SUC_NE_lexicon_tags_2 },
    { 0xe28a3aa8, SUC_NE_lexicon_tags_2 },
    { 0xa5963aaa, SUC_NE_lexicon_tags_52 },
    { 0x4a303aab, SUC_NE_lexicon_tags_2 },
    { 0x37703aac, SUC_NE_lexicon_tags_2 },
    { 0x8bf23aaa, SUC_NE_lexicon_tags_2 },
    { 0x19303aae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10dc3ab0, SUC_NE_lexicon_tags_2 },
    { 0xb0163ab1, SUC_NE_lexicon_tags_2 },
    { 0x5f8a3ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b303ab4, SUC_NE_lexicon_tags_2 },
    { 0x25e83ab5, SUC_NE_lexicon_tags_2 },
    { 0xd0fe3ab6, SUC_NE_lexicon_tags_34 },
    { 0x721e3ab5, SUC_NE_lexicon_tags_2 },
    { 0xb823ab5, SUC_NE_lexicon_tags_2 },
    { 0x33a43ab9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa05e3abb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99043abe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d203ac2, SUC_NE_lexicon_tags_3 },
    { 0xcb9a3ac2, SUC_NE_lexicon_tags_2 },
    { 0xc2103ac3, SUC_NE_lexicon_tags_2 },
    { 0xad083ac5, SUC_NE_lexicon_tags_2 },
    { 0x3ac43ac6, SUC_NE_lexicon_tags_2 },
    { 0x4f4c3ac7, SUC_NE_lexicon_tags_2 },
    { 0xb21c3ac7, SUC_NE_lexicon_tags_2 },
    { 0x375e3ac8, SUC_NE_lexicon_tags_2 },
    { 0x6ce83ac7, SUC_NE_lexicon_tags_2 },
    { 0x5aae3acb, SUC_NE_lexicon_tags_2 },
    { 0x718c3ac8, SUC_NE_lexicon_tags_2 },
    { 0xfe9a3ac3, SUC_NE_lexicon_tags_1 },
    { 0x4c1e3ac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ade3ad0, SUC_NE_lexicon_tags_2 },
    { 0xe9c03ad1, SUC_NE_lexicon_tags_2 },
    { 0x445c3ad1, SUC_NE_lexicon_tags_2 },
    { 0x214c3ad3, SUC_NE_lexicon_tags_2 },
    { 0x2a143ad2, SUC_NE_lexicon_tags_2 },
    { 0xe4683ad1, SUC_NE_lexicon_tags_2 },
    { 0xd5563ad6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeae43ad8, SUC_NE_lexicon_tags_2 },
    { 0x29983ad9, SUC_NE_lexicon_tags_2 },
    { 0xde083ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c543add, SUC_NE_lexicon_tags_2 },
    { 0x34683ade, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f83ae0, SUC_NE_lexicon_tags_2 },
    { 0x95023ae1, SUC_NE_lexicon_tags_0 },
    { 0x7e723ae2, SUC_NE_lexicon_tags_2 },
    { 0xabe83ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a423ae5, SUC_NE_lexicon_tags_2 },
    { 0x7c443ae6, SUC_NE_lexicon_tags_2 },
    { 0xa243ae7, SUC_NE_lexicon_tags_66 },
    { 0, NULL },
    { 0xa75c3ae9, SUC_NE_lexicon_tags_2 },
    { 0xcca03aea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a23af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcc3af6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1f63af8, SUC_NE_lexicon_tags_13 },
    { 0x926c3af8, SUC_NE_lexicon_tags_2 },
    { 0xfb9a3af8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93ac3afc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x304c3aff, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0x69643b01, SUC_NE_lexicon_tags_2 },
    { 0xa0263b02, SUC_NE_lexicon_tags_2 },
    { 0x43bc3b03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f63b0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5183b0d, SUC_NE_lexicon_tags_4 },
    { 0xbdb83b0d, SUC_NE_lexicon_tags_8 },
    { 0xee4c3b0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38223b13, SUC_NE_lexicon_tags_2 },
    { 0x110e3b13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bc3b1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94243b20, SUC_NE_lexicon_tags_2 },
    { 0xb9383b21, SUC_NE_lexicon_tags_1 },
    { 0x43ee3b20, SUC_NE_lexicon_tags_2 },
    { 0x80463b23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x161a3b25, SUC_NE_lexicon_tags_2 },
    { 0x30ee3b26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca643b2a, SUC_NE_lexicon_tags_2 },
    { 0x69ea3b2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20563b2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd743b33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x663b36, SUC_NE_lexicon_tags_2 },
    { 0x72bc3b37, SUC_NE_lexicon_tags_2 },
    { 0x33183b38, SUC_NE_lexicon_tags_2 },
    { 0xe6dc3b38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d63b3c, SUC_NE_lexicon_tags_2 },
    { 0x7af43b3d, SUC_NE_lexicon_tags_2 },
    { 0x5a823b3e, SUC_NE_lexicon_tags_2 },
    { 0x47ae3b3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5623b42, SUC_NE_lexicon_tags_2 },
    { 0xb4fe3b43, SUC_NE_lexicon_tags_2 },
    { 0x524e3b44, SUC_NE_lexicon_tags_2 },
    { 0x59003b43, SUC_NE_lexicon_tags_2 },
    { 0x7fb83b46, SUC_NE_lexicon_tags_4 },
    { 0x7ccc3b47, SUC_NE_lexicon_tags_2 },
    { 0x3cca3b48, SUC_NE_lexicon_tags_2 },
    { 0xb8a23b49, SUC_NE_lexicon_tags_2 },
    { 0x45de3b4a, SUC_NE_lexicon_tags_0 },
    { 0x8eb23b4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf43b4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46043b52, SUC_NE_lexicon_tags_2 },
    { 0xac623b53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75563b55, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ae3b58, SUC_NE_lexicon_tags_2 },
    { 0xef443b59, SUC_NE_lexicon_tags_2 },
    { 0x96a63b58, SUC_NE_lexicon_tags_2 },
    { 0xc19e3b5b, SUC_NE_lexicon_tags_13 },
    { 0xb503b5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86443b64, SUC_NE_lexicon_tags_102 },
    { 0x51b23b64, SUC_NE_lexicon_tags_2 },
    { 0x4723b66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc603b68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58aa3b6c, SUC_NE_lexicon_tags_8 },
    { 0x503b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f003b6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69e83b71, SUC_NE_lexicon_tags_2 },
    { 0x2b123b71, SUC_NE_lexicon_tags_49 },
    { 0xaafe3b73, SUC_NE_lexicon_tags_2 },
    { 0xce583b74, SUC_NE_lexicon_tags_2 },
    { 0x85923b75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a23b78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33523b7a, SUC_NE_lexicon_tags_2 },
    { 0xb7723b7b, SUC_NE_lexicon_tags_8 },
    { 0x5d543b7b, SUC_NE_lexicon_tags_2 },
    { 0x7f9e3b7d, SUC_NE_lexicon_tags_2 },
    { 0xec183b7e, SUC_NE_lexicon_tags_2 },
    { 0x2003b7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1883b85, SUC_NE_lexicon_tags_2 },
    { 0xd0183b86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947a3b8a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f83b91, SUC_NE_lexicon_tags_2 },
    { 0x5e0e3b92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6c63b94, SUC_NE_lexicon_tags_3 },
    { 0x82183b95, SUC_NE_lexicon_tags_3 },
    { 0xbb3e3b96, SUC_NE_lexicon_tags_2 },
    { 0x24023b94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd12c3b99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5c3b9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6e3b9f, SUC_NE_lexicon_tags_2 },
    { 0x43a23ba0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45363ba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e283ba6, SUC_NE_lexicon_tags_2 },
    { 0x44283ba6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b483ba9, SUC_NE_lexicon_tags_2 },
    { 0xbfc23baa, SUC_NE_lexicon_tags_2 },
    { 0x63383baa, SUC_NE_lexicon_tags_2 },
    { 0x2bf83bac, SUC_NE_lexicon_tags_18 },
    { 0x95aa3bad, SUC_NE_lexicon_tags_2 },
    { 0xb9ac3bae, SUC_NE_lexicon_tags_2 },
    { 0x738e3baf, SUC_NE_lexicon_tags_2 },
    { 0xeb7c3bae, SUC_NE_lexicon_tags_0 },
    { 0xefce3bb1, SUC_NE_lexicon_tags_4 },
    { 0xb65e3bb2, SUC_NE_lexicon_tags_2 },
    { 0xc73c3bb2, SUC_NE_lexicon_tags_2 },
    { 0x24ba3bb4, SUC_NE_lexicon_tags_2 },
    { 0xa0fe3bb5, SUC_NE_lexicon_tags_2 },
    { 0x12a3baf, SUC_NE_lexicon_tags_2 },
    { 0x569e3bb7, SUC_NE_lexicon_tags_2 },
    { 0x411e3bb7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8763bc1, SUC_NE_lexicon_tags_2 },
    { 0x11ca3bc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a3bc5, SUC_NE_lexicon_tags_2 },
    { 0x4c363bc6, SUC_NE_lexicon_tags_4 },
    { 0xe1b23bc7, SUC_NE_lexicon_tags_2 },
    { 0xba1e3bc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad983bca, SUC_NE_lexicon_tags_2 },
    { 0x3cc23bca, SUC_NE_lexicon_tags_2 },
    { 0x49a43bcb, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834c3bd0, SUC_NE_lexicon_tags_2 },
    { 0x12ba3bd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39f23bd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51423bd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c803bdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x307a3be1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8163be6, SUC_NE_lexicon_tags_2 },
    { 0xbc403be6, SUC_NE_lexicon_tags_2 },
    { 0xaf9a3be8, SUC_NE_lexicon_tags_2 },
    { 0x1b03be9, SUC_NE_lexicon_tags_2 },
    { 0x18c03bea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6023bec, SUC_NE_lexicon_tags_2 },
    { 0x5543bed, SUC_NE_lexicon_tags_2 },
    { 0xf0d03bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233a3bf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d143bf6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53483bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad8c3bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4183bfe, SUC_NE_lexicon_tags_2 },
    { 0x2dc03bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b63c03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbd23c05, SUC_NE_lexicon_tags_2 },
    { 0x3f463c05, SUC_NE_lexicon_tags_2 },
    { 0x1dc23c05, SUC_NE_lexicon_tags_2 },
    { 0x6dcc3c06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f63c0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76203c0e, SUC_NE_lexicon_tags_1 },
    { 0x415a3c0f, SUC_NE_lexicon_tags_2 },
    { 0xc0063c0e, SUC_NE_lexicon_tags_2 },
    { 0xb0683c11, SUC_NE_lexicon_tags_2 },
    { 0x33b63c12, SUC_NE_lexicon_tags_59 },
    { 0x33203c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d643c1e, SUC_NE_lexicon_tags_2 },
    { 0x88483c1e, SUC_NE_lexicon_tags_2 },
    { 0x8e943c1f, SUC_NE_lexicon_tags_2 },
    { 0x80863c1e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe903c24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87603c28, SUC_NE_lexicon_tags_2 },
    { 0xed43c29, SUC_NE_lexicon_tags_2 },
    { 0x4dde3c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf63c2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49d63c2f, SUC_NE_lexicon_tags_2 },
    { 0xbad03c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40843c32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f43c35, SUC_NE_lexicon_tags_2 },
    { 0xf0d63c36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3dc3c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf63c43, SUC_NE_lexicon_tags_2 },
    { 0xe8da3c44, SUC_NE_lexicon_tags_2 },
    { 0x85f43c45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1623c47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c3c4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x126c3c4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76783c51, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894e3c56, SUC_NE_lexicon_tags_2 },
    { 0xedaa3c56, SUC_NE_lexicon_tags_2 },
    { 0x72843c58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaade3c5c, SUC_NE_lexicon_tags_2 },
    { 0x65383c5d, SUC_NE_lexicon_tags_2 },
    { 0x67483c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83043c63, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27043c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95e3c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb16a3c6a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b203c6e, SUC_NE_lexicon_tags_2 },
    { 0x5f3e3c6f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e143c74, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b223c76, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xddfa3c79, SUC_NE_lexicon_tags_2 },
    { 0x430a3c7a, SUC_NE_lexicon_tags_2 },
    { 0x4b023c7b, SUC_NE_lexicon_tags_40 },
    { 0x67ea3c7c, SUC_NE_lexicon_tags_2 },
    { 0xb67a3c7d, SUC_NE_lexicon_tags_2 },
    { 0x95323c7e, SUC_NE_lexicon_tags_2 },
    { 0xa57c3c7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f703c81, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14e3c85, SUC_NE_lexicon_tags_2 },
    { 0xb0da3c86, SUC_NE_lexicon_tags_2 },
    { 0xa4e43c85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cca3c8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde3c8e, SUC_NE_lexicon_tags_2 },
    { 0x9e0c3c8e, SUC_NE_lexicon_tags_2 },
    { 0x5bf23c90, SUC_NE_lexicon_tags_2 },
    { 0x4ec43c90, SUC_NE_lexicon_tags_2 },
    { 0x355a3c92, SUC_NE_lexicon_tags_7 },
    { 0x7b83c93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x352e3c95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c63c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc623c9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2743c9d, SUC_NE_lexicon_tags_2 },
    { 0x5eac3c9e, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b43ca1, SUC_NE_lexicon_tags_2 },
    { 0xe1b03ca2, SUC_NE_lexicon_tags_2 },
    { 0x7a6a3ca3, SUC_NE_lexicon_tags_2 },
    { 0xc8103ca4, SUC_NE_lexicon_tags_2 },
    { 0x1db63ca4, SUC_NE_lexicon_tags_2 },
    { 0x99be3ca6, SUC_NE_lexicon_tags_2 },
    { 0xc7bc3ca2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63383caa, SUC_NE_lexicon_tags_2 },
    { 0x43fe3cab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe303cae, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be83cb1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x71a3cb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0543cb5, SUC_NE_lexicon_tags_2 },
    { 0x2f683cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996a3cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdef83cbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2783cc4, SUC_NE_lexicon_tags_2 },
    { 0x13b63cc5, SUC_NE_lexicon_tags_2 },
    { 0x74d43cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c23cca, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc03c3ccc, SUC_NE_lexicon_tags_2 },
    { 0xe9823ccc, SUC_NE_lexicon_tags_2 },
    { 0x4fce3cce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16b63cd0, SUC_NE_lexicon_tags_2 },
    { 0x2e1a3cd1, SUC_NE_lexicon_tags_2 },
    { 0x80ae3cd2, SUC_NE_lexicon_tags_7 },
    { 0xbc663cd3, SUC_NE_lexicon_tags_2 },
    { 0xde883cd4, SUC_NE_lexicon_tags_2 },
    { 0xc77c3cd1, SUC_NE_lexicon_tags_3 },
    { 0x1bfc3cd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad583cda, SUC_NE_lexicon_tags_2 },
    { 0x744c3cdb, SUC_NE_lexicon_tags_2 },
    { 0x57163cdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e0a3cde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76c63ce0, SUC_NE_lexicon_tags_2 },
    { 0x607e3ce0, SUC_NE_lexicon_tags_2 },
    { 0x84ea3ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71ba3ce4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfc23ce6, SUC_NE_lexicon_tags_1 },
    { 0x912e3ce6, SUC_NE_lexicon_tags_2 },
    { 0x2cda3ce8, SUC_NE_lexicon_tags_2 },
    { 0xadb23ce9, SUC_NE_lexicon_tags_2 },
    { 0xd9a3ce8, SUC_NE_lexicon_tags_2 },
    { 0xdbd63ceb, SUC_NE_lexicon_tags_2 },
    { 0xf2643cea, SUC_NE_lexicon_tags_0 },
    { 0xfeee3ced, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x407c3cf0, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x48863cf3, SUC_NE_lexicon_tags_3 },
    { 0xd2fe3cf4, SUC_NE_lexicon_tags_2 },
    { 0x1783cf4, SUC_NE_lexicon_tags_2 },
    { 0xf48c3cf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd83cfb, SUC_NE_lexicon_tags_2 },
    { 0xee7c3cfb, SUC_NE_lexicon_tags_2 },
    { 0xfc223cfc, SUC_NE_lexicon_tags_1 },
    { 0x31c23cfe, SUC_NE_lexicon_tags_4 },
    { 0xef943cfd, SUC_NE_lexicon_tags_2 },
    { 0x35be3cfc, SUC_NE_lexicon_tags_7 },
    { 0xd64c3cfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc63d08, SUC_NE_lexicon_tags_2 },
    { 0x862c3d09, SUC_NE_lexicon_tags_2 },
    { 0x26383d0a, SUC_NE_lexicon_tags_19 },
    { 0x35e03d0b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a223d0f, SUC_NE_lexicon_tags_2 },
    { 0xa9ba3d0f, SUC_NE_lexicon_tags_2 },
    { 0x567a3d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8943d1b, SUC_NE_lexicon_tags_2 },
    { 0x21e03d1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x519e3d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a63d22, SUC_NE_lexicon_tags_2 },
    { 0xa3f83d23, SUC_NE_lexicon_tags_4 },
    { 0xc02a3d24, SUC_NE_lexicon_tags_2 },
    { 0xd8ca3d25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a443d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a3d2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583a3d33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c683d35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2183d38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e143d3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1323d41, SUC_NE_lexicon_tags_2 },
    { 0x3e923d41, SUC_NE_lexicon_tags_2 },
    { 0x88de3d41, SUC_NE_lexicon_tags_2 },
    { 0x55483d44, SUC_NE_lexicon_tags_18 },
    { 0x36023d44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7123d47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e43d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff63d4f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66e3d57, SUC_NE_lexicon_tags_2 },
    { 0x73c83d58, SUC_NE_lexicon_tags_12 },
    { 0x4eac3d59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83263d5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa01a3d5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb23d63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e2e3d65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13503d69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad83d6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a3d71, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8383d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb43d79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f223d83, SUC_NE_lexicon_tags_2 },
    { 0x2ade3d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9da3d89, SUC_NE_lexicon_tags_2 },
    { 0xb8c83d8a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fe3d8f, SUC_NE_lexicon_tags_2 },
    { 0xe8723d90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c23d94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56403d96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8803d9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6583d9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1223da2, SUC_NE_lexicon_tags_3 },
    { 0x73e03da3, SUC_NE_lexicon_tags_2 },
    { 0xe4323da4, SUC_NE_lexicon_tags_2 },
    { 0x90183da3, SUC_NE_lexicon_tags_2 },
    { 0xeb1e3da6, SUC_NE_lexicon_tags_2 },
    { 0xbe843da7, SUC_NE_lexicon_tags_2 },
    { 0xacdc3da6, SUC_NE_lexicon_tags_2 },
    { 0x54103da4, SUC_NE_lexicon_tags_2 },
    { 0x54b43daa, SUC_NE_lexicon_tags_4 },
    { 0x397a3da2, SUC_NE_lexicon_tags_2 },
    { 0xbe9e3da8, SUC_NE_lexicon_tags_2 },
    { 0x93063da7, SUC_NE_lexicon_tags_2 },
    { 0xb25a3da2, SUC_NE_lexicon_tags_2 },
    { 0xddaa3dad, SUC_NE_lexicon_tags_2 },
    { 0x65443db0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c243db2, SUC_NE_lexicon_tags_2 },
    { 0xfdd63db3, SUC_NE_lexicon_tags_2 },
    { 0x11f23db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e3db7, SUC_NE_lexicon_tags_2 },
    { 0xcb803db7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190e3dbc, SUC_NE_lexicon_tags_2 },
    { 0xbb363dbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a223dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d63dc3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x1c63dc5, SUC_NE_lexicon_tags_34 },
    { 0xa7ec3dc6, SUC_NE_lexicon_tags_2 },
    { 0x986c3dc5, SUC_NE_lexicon_tags_2 },
    { 0xff6a3dc8, SUC_NE_lexicon_tags_2 },
    { 0x875c3dc9, SUC_NE_lexicon_tags_2 },
    { 0xadd23dca, SUC_NE_lexicon_tags_2 },
    { 0x2cce3dc6, SUC_NE_lexicon_tags_2 },
    { 0xbf4c3dc5, SUC_NE_lexicon_tags_2 },
    { 0x25f03dcd, SUC_NE_lexicon_tags_2 },
    { 0x522c3dce, SUC_NE_lexicon_tags_2 },
    { 0xe9a43dcf, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x65a83dd1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c183dd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c783dd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20843dd9, SUC_NE_lexicon_tags_2 },
    { 0xd5ba3dda, SUC_NE_lexicon_tags_2 },
    { 0x55c03ddb, SUC_NE_lexicon_tags_2 },
    { 0x11d63ddc, SUC_NE_lexicon_tags_2 },
    { 0x43763ddd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54fc3ddf, SUC_NE_lexicon_tags_7 },
    { 0x271e3de0, SUC_NE_lexicon_tags_2 },
    { 0x62843de1, SUC_NE_lexicon_tags_22 },
    { 0xbd083de2, SUC_NE_lexicon_tags_2 },
    { 0xd81c3de2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a423de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef63de8, SUC_NE_lexicon_tags_2 },
    { 0x5be23de9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69a3df0, SUC_NE_lexicon_tags_2 },
    { 0x59d23df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7a3df6, SUC_NE_lexicon_tags_2 },
    { 0xf283df7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e9c3df9, SUC_NE_lexicon_tags_4 },
    { 0x90323dfa, SUC_NE_lexicon_tags_2 },
    { 0x8f183dfb, SUC_NE_lexicon_tags_1 },
    { 0x75943df9, SUC_NE_lexicon_tags_2 },
    { 0xfeba3dfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51363e01, SUC_NE_lexicon_tags_7 },
    { 0xf17a3e01, SUC_NE_lexicon_tags_2 },
    { 0x21a83e02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eae3e08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd05e3e0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b63e0c, SUC_NE_lexicon_tags_2 },
    { 0x4c903e0c, SUC_NE_lexicon_tags_2 },
    { 0xbe523e0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3423e13, SUC_NE_lexicon_tags_2 },
    { 0x1fce3e13, SUC_NE_lexicon_tags_2 },
    { 0xe9aa3e13, SUC_NE_lexicon_tags_2 },
    { 0x16743e16, SUC_NE_lexicon_tags_2 },
    { 0x20e43e15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff9a3e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11883e1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a943e1e, SUC_NE_lexicon_tags_2 },
    { 0x4dec3e1f, SUC_NE_lexicon_tags_2 },
    { 0x3f3a3e1f, SUC_NE_lexicon_tags_2 },
    { 0xbcde3e1f, SUC_NE_lexicon_tags_2 },
    { 0x2aba3e22, SUC_NE_lexicon_tags_2 },
    { 0xca83e23, SUC_NE_lexicon_tags_2 },
    { 0xd0e3e24, SUC_NE_lexicon_tags_2 },
    { 0xee903e1f, SUC_NE_lexicon_tags_2 },
    { 0x9b763e1f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b943e29, SUC_NE_lexicon_tags_2 },
    { 0x8dbe3e29, SUC_NE_lexicon_tags_2 },
    { 0xca4a3e2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39023e2d, SUC_NE_lexicon_tags_3 },
    { 0xaeb83e2e, SUC_NE_lexicon_tags_2 },
    { 0x79623e2f, SUC_NE_lexicon_tags_2 },
    { 0x68703e2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8663e33, SUC_NE_lexicon_tags_2 },
    { 0x64ce3e34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d5a3e36, SUC_NE_lexicon_tags_2 },
    { 0xc3323e37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63563e39, SUC_NE_lexicon_tags_2 },
    { 0xc01e3e39, SUC_NE_lexicon_tags_2 },
    { 0xa6ce3e3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a743e3d, SUC_NE_lexicon_tags_2 },
    { 0xd8583e3e, SUC_NE_lexicon_tags_109 },
    { 0x3d423e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1a3e46, SUC_NE_lexicon_tags_2 },
    { 0xd3e03e46, SUC_NE_lexicon_tags_2 },
    { 0x76a03e48, SUC_NE_lexicon_tags_2 },
    { 0x1b863e49, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6243e4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a3e50, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70923e55, SUC_NE_lexicon_tags_2 },
    { 0xc8523e55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f5c3e58, SUC_NE_lexicon_tags_2 },
    { 0xf2ba3e59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e63e61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52603e63, SUC_NE_lexicon_tags_2 },
    { 0x390c3e64, SUC_NE_lexicon_tags_2 },
    { 0xfba43e64, SUC_NE_lexicon_tags_2 },
    { 0xef263e64, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52a3e69, SUC_NE_lexicon_tags_2 },
    { 0x247a3e6a, SUC_NE_lexicon_tags_2 },
    { 0x1e5e3e6a, SUC_NE_lexicon_tags_2 },
    { 0x94de3e6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0e3e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8883e78, SUC_NE_lexicon_tags_2 },
    { 0x93aa3e79, SUC_NE_lexicon_tags_4 },
    { 0x254c3e7a, SUC_NE_lexicon_tags_2 },
    { 0xbd123e7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d03e7e, SUC_NE_lexicon_tags_2 },
    { 0x53523e7f, SUC_NE_lexicon_tags_24 },
    { 0x2d103e7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x472c3e82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24143e85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd143e88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa3e8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca3e93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39603e95, SUC_NE_lexicon_tags_2 },
    { 0x1a63e96, SUC_NE_lexicon_tags_2 },
    { 0x43523e97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53ae3e99, SUC_NE_lexicon_tags_2 },
    { 0x46f83e99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb943e9d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe43ea1, SUC_NE_lexicon_tags_7 },
    { 0xe3c43ea2, SUC_NE_lexicon_tags_3 },
    { 0x9c8a3ea3, SUC_NE_lexicon_tags_2 },
    { 0xfa3ea2, SUC_NE_lexicon_tags_2 },
    { 0x5f923ea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba6c3ea7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3163ea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a3eae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a663eb0, SUC_NE_lexicon_tags_5 },
    { 0xa0bc3eb1, SUC_NE_lexicon_tags_5 },
    { 0x20583eb2, SUC_NE_lexicon_tags_2 },
    { 0xfdd63eb2, SUC_NE_lexicon_tags_2 },
    { 0x872c3eb4, SUC_NE_lexicon_tags_34 },
    { 0xac703eb5, SUC_NE_lexicon_tags_2 },
    { 0x6f4c3eb4, SUC_NE_lexicon_tags_2 },
    { 0x79a23eb0, SUC_NE_lexicon_tags_0 },
    { 0x495e3eb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde943ebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x409e3ebd, SUC_NE_lexicon_tags_2 },
    { 0xde2a3ebe, SUC_NE_lexicon_tags_2 },
    { 0xe7483ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x803c3ec1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85c3ec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b23eca, SUC_NE_lexicon_tags_2 },
    { 0x3dce3eca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9183ecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39903ed1, SUC_NE_lexicon_tags_2 },
    { 0xd2743ed2, SUC_NE_lexicon_tags_2 },
    { 0x3ab83ed3, SUC_NE_lexicon_tags_2 },
    { 0x2bea3ed2, SUC_NE_lexicon_tags_12 },
    { 0x987a3ed5, SUC_NE_lexicon_tags_2 },
    { 0xe683ed4, SUC_NE_lexicon_tags_2 },
    { 0x8ee03ed5, SUC_NE_lexicon_tags_2 },
    { 0x67e23ed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4523eda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46863edc, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xd463ede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac63ee1, SUC_NE_lexicon_tags_2 },
    { 0xe0263ee2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e03ee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6223ee6, SUC_NE_lexicon_tags_2 },
    { 0xbd63ee7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9e3eec, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x86403eee, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c3ef1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c3ef4, SUC_NE_lexicon_tags_2 },
    { 0x7f5a3ef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a3ef8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96f83efa, SUC_NE_lexicon_tags_2 },
    { 0x9b563efb, SUC_NE_lexicon_tags_230 },
    { 0xd1e83efb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d83efe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96183f00, SUC_NE_lexicon_tags_2 },
    { 0xbb5c3f01, SUC_NE_lexicon_tags_19 },
    { 0x583f02, SUC_NE_lexicon_tags_2 },
    { 0xf9243f03, SUC_NE_lexicon_tags_2 },
    { 0x27a23f03, SUC_NE_lexicon_tags_2 },
    { 0x41e63f04, SUC_NE_lexicon_tags_2 },
    { 0xf2c3f06, SUC_NE_lexicon_tags_2 },
    { 0x7b543f07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae203f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe17e3f0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd23f12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87a63f14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb763f19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7603f20, SUC_NE_lexicon_tags_2 },
    { 0x837e3f21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfbe3f23, SUC_NE_lexicon_tags_2 },
    { 0xa1b83f24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2203f2a, SUC_NE_lexicon_tags_30 },
    { 0x945c3f2b, SUC_NE_lexicon_tags_2 },
    { 0xb6f03f2b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e23f2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x871c3f31, SUC_NE_lexicon_tags_2 },
    { 0x6aee3f31, SUC_NE_lexicon_tags_2 },
    { 0x981e3f31, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b83f36, SUC_NE_lexicon_tags_2 },
    { 0x37ac3f37, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f23f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa63f42, SUC_NE_lexicon_tags_2 },
    { 0x81aa3f42, SUC_NE_lexicon_tags_2 },
    { 0xf283f44, SUC_NE_lexicon_tags_2 },
    { 0xcf3c3f43, SUC_NE_lexicon_tags_2 },
    { 0x639a3f46, SUC_NE_lexicon_tags_2 },
    { 0x30a43f46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x138e3f4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf23f4c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7e3f4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd15a3f51, SUC_NE_lexicon_tags_2 },
    { 0x4ce83f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c3f5b, SUC_NE_lexicon_tags_2 },
    { 0xef603f5c, SUC_NE_lexicon_tags_2 },
    { 0x47803f5c, SUC_NE_lexicon_tags_1 },
    { 0xedd43f5e, SUC_NE_lexicon_tags_2 },
    { 0x830e3f5f, SUC_NE_lexicon_tags_2 },
    { 0x5a843f60, SUC_NE_lexicon_tags_3 },
    { 0x45083f60, SUC_NE_lexicon_tags_2 },
    { 0xdd983f62, SUC_NE_lexicon_tags_2 },
    { 0x1d43f5f, SUC_NE_lexicon_tags_2 },
    { 0x95483f63, SUC_NE_lexicon_tags_34 },
    { 0x88ec3f65, SUC_NE_lexicon_tags_2 },
    { 0x9cf43f5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41823f68, SUC_NE_lexicon_tags_3 },
    { 0x96623f68, SUC_NE_lexicon_tags_2 },
    { 0x51043f6a, SUC_NE_lexicon_tags_3 },
    { 0x19bc3f6b, SUC_NE_lexicon_tags_2 },
    { 0x35ba3f68, SUC_NE_lexicon_tags_7 },
    { 0xb2023f69, SUC_NE_lexicon_tags_2 },
    { 0x87a3f69, SUC_NE_lexicon_tags_2 },
    { 0x88d83f6a, SUC_NE_lexicon_tags_7 },
    { 0xfdee3f70, SUC_NE_lexicon_tags_2 },
    { 0x65ac3f70, SUC_NE_lexicon_tags_2 },
    { 0x26d63f72, SUC_NE_lexicon_tags_2 },
    { 0x42043f70, SUC_NE_lexicon_tags_7 },
    { 0x55ae3f74, SUC_NE_lexicon_tags_2 },
    { 0xf0d63f71, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xde703f77, SUC_NE_lexicon_tags_2 },
    { 0xa05c3f78, SUC_NE_lexicon_tags_2 },
    { 0xfe083f79, SUC_NE_lexicon_tags_2 },
    { 0xdfcc3f7a, SUC_NE_lexicon_tags_2 },
    { 0xc02e3f7a, SUC_NE_lexicon_tags_2 },
    { 0xf9a63f7b, SUC_NE_lexicon_tags_7 },
    { 0xbbb63f7a, SUC_NE_lexicon_tags_2 },
    { 0x15663f7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76583f81, SUC_NE_lexicon_tags_19 },
    { 0x56da3f82, SUC_NE_lexicon_tags_2 },
    { 0x76983f82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff43f8a, SUC_NE_lexicon_tags_2 },
    { 0x643e3f8b, SUC_NE_lexicon_tags_3 },
    { 0x215e3f8c, SUC_NE_lexicon_tags_3 },
    { 0x8d423f8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56023f8f, SUC_NE_lexicon_tags_2 },
    { 0x63bc3f8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583c3f97, SUC_NE_lexicon_tags_2 },
    { 0x1b923f97, SUC_NE_lexicon_tags_2 },
    { 0x39c03f98, SUC_NE_lexicon_tags_2 },
    { 0xb0c63f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e123fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b63fa3, SUC_NE_lexicon_tags_2 },
    { 0x2abe3fa4, SUC_NE_lexicon_tags_2 },
    { 0x9e183fa4, SUC_NE_lexicon_tags_2 },
    { 0xda23fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee823fa9, SUC_NE_lexicon_tags_2 },
    { 0xfa603faa, SUC_NE_lexicon_tags_2 },
    { 0xb3223faa, SUC_NE_lexicon_tags_7 },
    { 0x63883fac, SUC_NE_lexicon_tags_7 },
    { 0xdce63fa9, SUC_NE_lexicon_tags_2 },
    { 0x4fdc3fae, SUC_NE_lexicon_tags_2 },
    { 0xc05e3faf, SUC_NE_lexicon_tags_7 },
    { 0xa8123fae, SUC_NE_lexicon_tags_2 },
    { 0xc4403faa, SUC_NE_lexicon_tags_2 },
    { 0xcc803faa, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xba943fb4, SUC_NE_lexicon_tags_2 },
    { 0x94363fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37e3fbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ee3fc3, SUC_NE_lexicon_tags_2 },
    { 0x88ba3fc4, SUC_NE_lexicon_tags_2 },
    { 0x19d03fc3, SUC_NE_lexicon_tags_2 },
    { 0x96bc3fc3, SUC_NE_lexicon_tags_0 },
    { 0xb823fc7, SUC_NE_lexicon_tags_2 },
    { 0xf71c3fc7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x429c3fca, SUC_NE_lexicon_tags_2 },
    { 0x88c03fcb, SUC_NE_lexicon_tags_2 },
    { 0xa1783fcb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23623fd1, SUC_NE_lexicon_tags_2 },
    { 0xa5c43fd1, SUC_NE_lexicon_tags_2 },
    { 0x22bc3fd3, SUC_NE_lexicon_tags_2 },
    { 0x1ca83fd1, SUC_NE_lexicon_tags_2 },
    { 0xe3e83fd4, SUC_NE_lexicon_tags_7 },
    { 0x4f8c3fd6, SUC_NE_lexicon_tags_2 },
    { 0x78c43fd6, SUC_NE_lexicon_tags_2 },
    { 0xd383fd6, SUC_NE_lexicon_tags_2 },
    { 0x8cbc3fd7, SUC_NE_lexicon_tags_2 },
    { 0xefb83fd9, SUC_NE_lexicon_tags_3 },
    { 0x920a3fd9, SUC_NE_lexicon_tags_2 },
    { 0xf0263fdc, SUC_NE_lexicon_tags_1 },
    { 0xbeb43fdd, SUC_NE_lexicon_tags_40 },
    { 0xc4da3fde, SUC_NE_lexicon_tags_2 },
    { 0x7423fdf, SUC_NE_lexicon_tags_341 },
    { 0xd1aa3fde, SUC_NE_lexicon_tags_2 },
    { 0xea603fd1, SUC_NE_lexicon_tags_11 },
    { 0xaef43fe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe243fe4, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x32a03fe6, SUC_NE_lexicon_tags_2 },
    { 0x7f623fe6, SUC_NE_lexicon_tags_2 },
    { 0xf7043fe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748c3fed, SUC_NE_lexicon_tags_2 },
    { 0x7bd03fee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f63ff2, SUC_NE_lexicon_tags_2 },
    { 0xf17e3ff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5a3ff6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b63ffa, SUC_NE_lexicon_tags_2 },
    { 0xd683ffb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60283ffe, SUC_NE_lexicon_tags_2 },
    { 0x1eea3fff, SUC_NE_lexicon_tags_2 },
    { 0x25183ffe, SUC_NE_lexicon_tags_2 },
    { 0x41d83fff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed64003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf86a4005, SUC_NE_lexicon_tags_2 },
    { 0xa7344006, SUC_NE_lexicon_tags_2 },
    { 0x27684007, SUC_NE_lexicon_tags_2 },
    { 0x4f64006, SUC_NE_lexicon_tags_2 },
    { 0x75504009, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe180400c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa524400e, SUC_NE_lexicon_tags_0 },
    { 0xaab0400f, SUC_NE_lexicon_tags_2 },
    { 0x2ba84010, SUC_NE_lexicon_tags_0 },
    { 0xf4cc4011, SUC_NE_lexicon_tags_2 },
    { 0x27064010, SUC_NE_lexicon_tags_2 },
    { 0xca844011, SUC_NE_lexicon_tags_2 },
    { 0xe4624014, SUC_NE_lexicon_tags_3 },
    { 0x5f344014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1c401b, SUC_NE_lexicon_tags_2 },
    { 0x625c401c, SUC_NE_lexicon_tags_89 },
    { 0, NULL },
    { 0x6abc401e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x397e4021, SUC_NE_lexicon_tags_2 },
    { 0x45384022, SUC_NE_lexicon_tags_3 },
    { 0xe3f24023, SUC_NE_lexicon_tags_2 },
    { 0xdf8c4022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c544027, SUC_NE_lexicon_tags_2 },
    { 0x41ce4028, SUC_NE_lexicon_tags_40 },
    { 0xd8044027, SUC_NE_lexicon_tags_2 },
    { 0xf0024028, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4070402d, SUC_NE_lexicon_tags_2 },
    { 0xaac6402e, SUC_NE_lexicon_tags_2 },
    { 0xf06c402f, SUC_NE_lexicon_tags_2 },
    { 0xbfa04030, SUC_NE_lexicon_tags_2 },
    { 0xe7864030, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48f24033, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb764035, SUC_NE_lexicon_tags_2 },
    { 0xcab84036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b26403c, SUC_NE_lexicon_tags_1 },
    { 0x3b2a403c, SUC_NE_lexicon_tags_0 },
    { 0x5bb4403e, SUC_NE_lexicon_tags_2 },
    { 0x4e84403d, SUC_NE_lexicon_tags_2 },
    { 0x24804040, SUC_NE_lexicon_tags_2 },
    { 0x1a624041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b924044, SUC_NE_lexicon_tags_2 },
    { 0xa3b84045, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa404e, SUC_NE_lexicon_tags_2 },
    { 0xe2b0404f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad004054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bde4057, SUC_NE_lexicon_tags_2 },
    { 0xafa84058, SUC_NE_lexicon_tags_2 },
    { 0xc5224059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4284405c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d4405e, SUC_NE_lexicon_tags_2 },
    { 0xd02e405f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8964062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb44066, SUC_NE_lexicon_tags_2 },
    { 0xdb904067, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2624069, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x602406b, SUC_NE_lexicon_tags_2 },
    { 0x72a6406b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33a406e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e84074, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc9a44076, SUC_NE_lexicon_tags_7 },
    { 0xe8324076, SUC_NE_lexicon_tags_2 },
    { 0xb5a4077, SUC_NE_lexicon_tags_2 },
    { 0x4f564079, SUC_NE_lexicon_tags_7 },
    { 0x169e407a, SUC_NE_lexicon_tags_2 },
    { 0x8b824077, SUC_NE_lexicon_tags_2 },
    { 0xe85e407c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ae4080, SUC_NE_lexicon_tags_4 },
    { 0xeb3a4081, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x8fec4083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3de4085, SUC_NE_lexicon_tags_13 },
    { 0x87504086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0408e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c44094, SUC_NE_lexicon_tags_28 },
    { 0xe1aa4095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf440a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49240a8, SUC_NE_lexicon_tags_17 },
    { 0x14ee40a9, SUC_NE_lexicon_tags_2 },
    { 0x382640a9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9b840ac, SUC_NE_lexicon_tags_2 },
    { 0x36840ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2240b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa36840b4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x693440b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae840b9, SUC_NE_lexicon_tags_2 },
    { 0xf3d440b9, SUC_NE_lexicon_tags_7 },
    { 0xce9040bb, SUC_NE_lexicon_tags_2 },
    { 0xe34e40bc, SUC_NE_lexicon_tags_5 },
    { 0x1f0e40b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9640c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7040c4, SUC_NE_lexicon_tags_2 },
    { 0x1b1440c4, SUC_NE_lexicon_tags_2 },
    { 0xb2f040c6, SUC_NE_lexicon_tags_2 },
    { 0x5f0840c7, SUC_NE_lexicon_tags_2 },
    { 0x7eaa40c8, SUC_NE_lexicon_tags_2 },
    { 0x368e40c6, SUC_NE_lexicon_tags_2 },
    { 0x953e40c6, SUC_NE_lexicon_tags_1 },
    { 0x1e640cb, SUC_NE_lexicon_tags_2 },
    { 0xbd5040cc, SUC_NE_lexicon_tags_2 },
    { 0x358440cb, SUC_NE_lexicon_tags_4 },
    { 0x65ca40ce, SUC_NE_lexicon_tags_2 },
    { 0x7d6440cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685840d6, SUC_NE_lexicon_tags_2 },
    { 0xd4fa40d7, SUC_NE_lexicon_tags_2 },
    { 0xbdae40d8, SUC_NE_lexicon_tags_2 },
    { 0x5b1a40d6, SUC_NE_lexicon_tags_2 },
    { 0xef1840d9, SUC_NE_lexicon_tags_2 },
    { 0x5fe240db, SUC_NE_lexicon_tags_2 },
    { 0x5c5440d7, SUC_NE_lexicon_tags_2 },
    { 0x322440dc, SUC_NE_lexicon_tags_2 },
    { 0x124a40d8, SUC_NE_lexicon_tags_2 },
    { 0xd2e40db, SUC_NE_lexicon_tags_2 },
    { 0x10e840db, SUC_NE_lexicon_tags_2 },
    { 0xfe9840e1, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac40e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa67c40e6, SUC_NE_lexicon_tags_2 },
    { 0x708040e6, SUC_NE_lexicon_tags_2 },
    { 0x7e2840e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e640eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x190840ed, SUC_NE_lexicon_tags_2 },
    { 0x573440ed, SUC_NE_lexicon_tags_2 },
    { 0xdbee40ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952240f3, SUC_NE_lexicon_tags_4 },
    { 0xd5d040f4, SUC_NE_lexicon_tags_2 },
    { 0x748a40f5, SUC_NE_lexicon_tags_2 },
    { 0xfdea40f5, SUC_NE_lexicon_tags_0 },
    { 0xe79a40f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0c40fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d584100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae24105, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x267a4108, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa6c410a, SUC_NE_lexicon_tags_2 },
    { 0x9034410b, SUC_NE_lexicon_tags_2 },
    { 0x9356410c, SUC_NE_lexicon_tags_2 },
    { 0x9824410d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd864111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12964116, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0xeca24118, SUC_NE_lexicon_tags_2 },
    { 0xbb124119, SUC_NE_lexicon_tags_2 },
    { 0x9e8e4119, SUC_NE_lexicon_tags_2 },
    { 0x1644411a, SUC_NE_lexicon_tags_2 },
    { 0x3cb6411c, SUC_NE_lexicon_tags_2 },
    { 0xb2aa411d, SUC_NE_lexicon_tags_2 },
    { 0x8352411c, SUC_NE_lexicon_tags_2 },
    { 0xa61c411c, SUC_NE_lexicon_tags_2 },
    { 0xb75c411f, SUC_NE_lexicon_tags_1 },
    { 0x80e6411e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x641a4124, SUC_NE_lexicon_tags_2 },
    { 0x35ea4124, SUC_NE_lexicon_tags_2 },
    { 0x98584126, SUC_NE_lexicon_tags_2 },
    { 0xbc504125, SUC_NE_lexicon_tags_2 },
    { 0xb4e44128, SUC_NE_lexicon_tags_2 },
    { 0x6904129, SUC_NE_lexicon_tags_4 },
    { 0x1f96412a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc12e412c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46b4412e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2ee4130, SUC_NE_lexicon_tags_2 },
    { 0xdad04130, SUC_NE_lexicon_tags_2 },
    { 0xc9884130, SUC_NE_lexicon_tags_2 },
    { 0xe5404131, SUC_NE_lexicon_tags_2 },
    { 0xccd64134, SUC_NE_lexicon_tags_5 },
    { 0xd95c4135, SUC_NE_lexicon_tags_2 },
    { 0x8bce4133, SUC_NE_lexicon_tags_2 },
    { 0x79484134, SUC_NE_lexicon_tags_2 },
    { 0x5a384134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd0413a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ea413d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4a4142, SUC_NE_lexicon_tags_26 },
    { 0x27dc4143, SUC_NE_lexicon_tags_2 },
    { 0x734e4144, SUC_NE_lexicon_tags_2 },
    { 0xaf5c4145, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa1d04147, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4414c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4a4150, SUC_NE_lexicon_tags_2 },
    { 0x55ec4151, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f64155, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c76415a, SUC_NE_lexicon_tags_2 },
    { 0x2916415b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e0415f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a664162, SUC_NE_lexicon_tags_2 },
    { 0xe9b84162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf404165, SUC_NE_lexicon_tags_2 },
    { 0x8f804165, SUC_NE_lexicon_tags_2 },
    { 0x8a804167, SUC_NE_lexicon_tags_2 },
    { 0xcda84166, SUC_NE_lexicon_tags_2 },
    { 0xca244169, SUC_NE_lexicon_tags_2 },
    { 0xa226416a, SUC_NE_lexicon_tags_2 },
    { 0x97ec416a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20144170, SUC_NE_lexicon_tags_2 },
    { 0xedc84171, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c044174, SUC_NE_lexicon_tags_2 },
    { 0x2f9a4175, SUC_NE_lexicon_tags_2 },
    { 0x1ce24176, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69e4179, SUC_NE_lexicon_tags_2 },
    { 0x3bf0417a, SUC_NE_lexicon_tags_2 },
    { 0x8aa4179, SUC_NE_lexicon_tags_2 },
    { 0x3542417a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde64417f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed324187, SUC_NE_lexicon_tags_2 },
    { 0xe4188, SUC_NE_lexicon_tags_2 },
    { 0x7c844188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x871c418b, SUC_NE_lexicon_tags_2 },
    { 0x9638418c, SUC_NE_lexicon_tags_2 },
    { 0xc45c418c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cdc418f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9f04191, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbeea4193, SUC_NE_lexicon_tags_2 },
    { 0x19ae4194, SUC_NE_lexicon_tags_2 },
    { 0x8d984195, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50a64197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc419d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fc41a2, SUC_NE_lexicon_tags_2 },
    { 0xec7c41a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef441a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54641a9, SUC_NE_lexicon_tags_7 },
    { 0x9ff041aa, SUC_NE_lexicon_tags_3 },
    { 0x3b6e41ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c41b0, SUC_NE_lexicon_tags_2 },
    { 0x73641b1, SUC_NE_lexicon_tags_2 },
    { 0x4f2841b2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54841bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65841bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372041ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd1a41cc, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589441d2, SUC_NE_lexicon_tags_2 },
    { 0x5c41d3, SUC_NE_lexicon_tags_2 },
    { 0x20d841d4, SUC_NE_lexicon_tags_2 },
    { 0x748641d4, SUC_NE_lexicon_tags_2 },
    { 0xf81441d6, SUC_NE_lexicon_tags_2 },
    { 0xa1c841d4, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e41dc, SUC_NE_lexicon_tags_2 },
    { 0x56a641dd, SUC_NE_lexicon_tags_2 },
    { 0x5e1041de, SUC_NE_lexicon_tags_2 },
    { 0x4d7c41df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb241e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222441e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5f841e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x957441eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2041f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc641f8, SUC_NE_lexicon_tags_4 },
    { 0x10e641f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5a241fb, SUC_NE_lexicon_tags_2 },
    { 0x724841fc, SUC_NE_lexicon_tags_2 },
    { 0xc1ac41fd, SUC_NE_lexicon_tags_2 },
    { 0x67041fd, SUC_NE_lexicon_tags_2 },
    { 0x506e41ff, SUC_NE_lexicon_tags_2 },
    { 0x549841ff, SUC_NE_lexicon_tags_2 },
    { 0xf3b64201, SUC_NE_lexicon_tags_2 },
    { 0xe8f84202, SUC_NE_lexicon_tags_2 },
    { 0xaa8a4203, SUC_NE_lexicon_tags_2 },
    { 0x6c7c4203, SUC_NE_lexicon_tags_2 },
    { 0xe65441fb, SUC_NE_lexicon_tags_0 },
    { 0x31e84200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4484208, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30aa420c, SUC_NE_lexicon_tags_2 },
    { 0xe778420d, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x1078420f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8844212, SUC_NE_lexicon_tags_2 },
    { 0xea844213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc84216, SUC_NE_lexicon_tags_2 },
    { 0x806a4217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a98421c, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08421f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83e4222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33064224, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f32422a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf6422f, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b24232, SUC_NE_lexicon_tags_2 },
    { 0x81104232, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78e4237, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4df24239, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b68423d, SUC_NE_lexicon_tags_2 },
    { 0x28c423e, SUC_NE_lexicon_tags_2 },
    { 0xd2a2423e, SUC_NE_lexicon_tags_14 },
    { 0x640423f, SUC_NE_lexicon_tags_25 },
    { 0x90504241, SUC_NE_lexicon_tags_210 },
    { 0x15584242, SUC_NE_lexicon_tags_2 },
    { 0xe6864243, SUC_NE_lexicon_tags_2 },
    { 0xfc444243, SUC_NE_lexicon_tags_18 },
    { 0x3dc44243, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9244247, SUC_NE_lexicon_tags_3 },
    { 0x88f24247, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9692424a, SUC_NE_lexicon_tags_2 },
    { 0x9674424a, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0xfe8424d, SUC_NE_lexicon_tags_2 },
    { 0x56d6424d, SUC_NE_lexicon_tags_2 },
    { 0xb732424e, SUC_NE_lexicon_tags_2 },
    { 0x2a844250, SUC_NE_lexicon_tags_2 },
    { 0x99484250, SUC_NE_lexicon_tags_2 },
    { 0xb1004252, SUC_NE_lexicon_tags_2 },
    { 0x24e04253, SUC_NE_lexicon_tags_2 },
    { 0xcde84253, SUC_NE_lexicon_tags_2 },
    { 0xca1c4255, SUC_NE_lexicon_tags_1 },
    { 0xab8a4256, SUC_NE_lexicon_tags_2 },
    { 0x17644257, SUC_NE_lexicon_tags_2 },
    { 0x4c044257, SUC_NE_lexicon_tags_2 },
    { 0xe6444259, SUC_NE_lexicon_tags_2 },
    { 0x1360425a, SUC_NE_lexicon_tags_2 },
    { 0xaed64257, SUC_NE_lexicon_tags_2 },
    { 0x6dec4256, SUC_NE_lexicon_tags_2 },
    { 0x2dea4259, SUC_NE_lexicon_tags_2 },
    { 0x6e9c4258, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36244261, SUC_NE_lexicon_tags_2 },
    { 0x6ace4261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99204264, SUC_NE_lexicon_tags_2 },
    { 0x106e4265, SUC_NE_lexicon_tags_2 },
    { 0xe8d24266, SUC_NE_lexicon_tags_2 },
    { 0xf2c4266, SUC_NE_lexicon_tags_2 },
    { 0x86fc4268, SUC_NE_lexicon_tags_337 },
    { 0, NULL },
    { 0, NULL },
    { 0x367c426b, SUC_NE_lexicon_tags_2 },
    { 0x15a2426c, SUC_NE_lexicon_tags_24 },
    { 0x45ac426d, SUC_NE_lexicon_tags_2 },
    { 0xd268426e, SUC_NE_lexicon_tags_2 },
    { 0x24ec426e, SUC_NE_lexicon_tags_2 },
    { 0xdcbc426b, SUC_NE_lexicon_tags_2 },
    { 0x66584270, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c244273, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d4427a, SUC_NE_lexicon_tags_4 },
    { 0x2098427b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8a427f, SUC_NE_lexicon_tags_7 },
    { 0xeb524280, SUC_NE_lexicon_tags_1 },
    { 0x8c984281, SUC_NE_lexicon_tags_2 },
    { 0x98224280, SUC_NE_lexicon_tags_2 },
    { 0x24b44283, SUC_NE_lexicon_tags_2 },
    { 0xb7ac4284, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a64289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x523c428b, SUC_NE_lexicon_tags_188 },
    { 0x6c72428c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b76428e, SUC_NE_lexicon_tags_2 },
    { 0xe9dc428e, SUC_NE_lexicon_tags_2 },
    { 0xbff04290, SUC_NE_lexicon_tags_2 },
    { 0x56884290, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x724294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e124296, SUC_NE_lexicon_tags_2 },
    { 0x92764297, SUC_NE_lexicon_tags_4 },
    { 0xfe424298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d9c429a, SUC_NE_lexicon_tags_2 },
    { 0x5c74429b, SUC_NE_lexicon_tags_2 },
    { 0x6b2429c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36d8429e, SUC_NE_lexicon_tags_2 },
    { 0x6a3a429f, SUC_NE_lexicon_tags_2 },
    { 0x2862429e, SUC_NE_lexicon_tags_2 },
    { 0x2a2642a1, SUC_NE_lexicon_tags_2 },
    { 0x204a42a0, SUC_NE_lexicon_tags_4 },
    { 0xd1642a3, SUC_NE_lexicon_tags_2 },
    { 0xd79242a4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x38ae42a6, SUC_NE_lexicon_tags_2 },
    { 0x63b642a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d042aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02442ad, SUC_NE_lexicon_tags_17 },
    { 0x354242ae, SUC_NE_lexicon_tags_34 },
    { 0x69ee42af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a9642b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3042b3, SUC_NE_lexicon_tags_2 },
    { 0x115242b4, SUC_NE_lexicon_tags_4 },
    { 0xc30242b5, SUC_NE_lexicon_tags_2 },
    { 0xfdd242b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac042bc, SUC_NE_lexicon_tags_2 },
    { 0xd2f242bc, SUC_NE_lexicon_tags_2 },
    { 0x94c42be, SUC_NE_lexicon_tags_2 },
    { 0x868c42be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcde242c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0042cb, SUC_NE_lexicon_tags_2 },
    { 0x6eea42cc, SUC_NE_lexicon_tags_4 },
    { 0x295a42cd, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7242d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x811042d2, SUC_NE_lexicon_tags_2 },
    { 0xbfee42d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc42d8, SUC_NE_lexicon_tags_2 },
    { 0x805e42d9, SUC_NE_lexicon_tags_2 },
    { 0xd09c42da, SUC_NE_lexicon_tags_2 },
    { 0x840042db, SUC_NE_lexicon_tags_2 },
    { 0xc3ec42da, SUC_NE_lexicon_tags_2 },
    { 0x2a0042dd, SUC_NE_lexicon_tags_2 },
    { 0x9e0042d9, SUC_NE_lexicon_tags_2 },
    { 0xa66042df, SUC_NE_lexicon_tags_2 },
    { 0x6d9242df, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0042e5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x917242e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d042eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903042ef, SUC_NE_lexicon_tags_0 },
    { 0xea2042f0, SUC_NE_lexicon_tags_2 },
    { 0x1e8242ef, SUC_NE_lexicon_tags_2 },
    { 0x33a642f2, SUC_NE_lexicon_tags_2 },
    { 0xf97842f2, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837c42f7, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a6242f9, SUC_NE_lexicon_tags_2 },
    { 0xb32e42fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa66842fc, SUC_NE_lexicon_tags_2 },
    { 0x6f9e42fd, SUC_NE_lexicon_tags_2 },
    { 0xb2ec42fe, SUC_NE_lexicon_tags_2 },
    { 0x2b1042fe, SUC_NE_lexicon_tags_2 },
    { 0x9ebc42fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x723c4302, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0xbdea4304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc344306, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba424309, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c6430c, SUC_NE_lexicon_tags_7 },
    { 0xe568430d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4264312, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x915c4314, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc224317, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6431b, SUC_NE_lexicon_tags_2 },
    { 0x1fbe431c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xda00431e, SUC_NE_lexicon_tags_2 },
    { 0xdee8431e, SUC_NE_lexicon_tags_2 },
    { 0x7c8e4320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5e04322, SUC_NE_lexicon_tags_2 },
    { 0xc95c4323, SUC_NE_lexicon_tags_2 },
    { 0x68944324, SUC_NE_lexicon_tags_2 },
    { 0x366c4325, SUC_NE_lexicon_tags_2 },
    { 0x79f04322, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18be4328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x261a432a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a84335, SUC_NE_lexicon_tags_2 },
    { 0xdd1a4336, SUC_NE_lexicon_tags_0 },
    { 0xb04337, SUC_NE_lexicon_tags_2 },
    { 0xff524336, SUC_NE_lexicon_tags_7 },
    { 0xce404339, SUC_NE_lexicon_tags_2 },
    { 0xed044339, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x231c433c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13a4433e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb484340, SUC_NE_lexicon_tags_2 },
    { 0xfacc4341, SUC_NE_lexicon_tags_281 },
    { 0xeefa4342, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1424347, SUC_NE_lexicon_tags_2 },
    { 0xea6c4348, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8006434f, SUC_NE_lexicon_tags_2 },
    { 0xfb36434f, SUC_NE_lexicon_tags_2 },
    { 0xbdb6434f, SUC_NE_lexicon_tags_2 },
    { 0x8bd64350, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f24356, SUC_NE_lexicon_tags_2 },
    { 0x4d344357, SUC_NE_lexicon_tags_4 },
    { 0xb1ea4358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x745c435a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e74435d, SUC_NE_lexicon_tags_2 },
    { 0xbc84435e, SUC_NE_lexicon_tags_2 },
    { 0x2cd8435f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33864361, SUC_NE_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xea344364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fc4368, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x88dc436a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9a436d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ee4371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36664373, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dea4375, SUC_NE_lexicon_tags_2 },
    { 0xba644376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0437a, SUC_NE_lexicon_tags_2 },
    { 0x7e78437b, SUC_NE_lexicon_tags_2 },
    { 0x1664437a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7844384, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92d44386, SUC_NE_lexicon_tags_4 },
    { 0xc62a4386, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24704389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586c438e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c04391, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c64394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15764397, SUC_NE_lexicon_tags_2 },
    { 0x9db84398, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248c439a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d0439c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a2439f, SUC_NE_lexicon_tags_2 },
    { 0x5eb643a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe61e43a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38243a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6443a8, SUC_NE_lexicon_tags_2 },
    { 0xb4aa43a9, SUC_NE_lexicon_tags_4 },
    { 0x26b643a9, SUC_NE_lexicon_tags_2 },
    { 0xdba843ab, SUC_NE_lexicon_tags_2 },
    { 0xfa9e43a9, SUC_NE_lexicon_tags_2 },
    { 0xf66a43ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8a43b1, SUC_NE_lexicon_tags_5 },
    { 0x55d243b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x555443b5, SUC_NE_lexicon_tags_7 },
    { 0xe87c43b6, SUC_NE_lexicon_tags_2 },
    { 0x13ba43b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45643ba, SUC_NE_lexicon_tags_2 },
    { 0xc5cc43bb, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6e43c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4a43c3, SUC_NE_lexicon_tags_2 },
    { 0xe89a43c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47d443c6, SUC_NE_lexicon_tags_2 },
    { 0x24d043c7, SUC_NE_lexicon_tags_53 },
    { 0xe34843c6, SUC_NE_lexicon_tags_2 },
    { 0x2f2843c9, SUC_NE_lexicon_tags_0 },
    { 0xd03043c9, SUC_NE_lexicon_tags_2 },
    { 0x72de43c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a6443cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a843d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x124243d2, SUC_NE_lexicon_tags_2 },
    { 0x3f0643d2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb7643d5, SUC_NE_lexicon_tags_2 },
    { 0x539043d5, SUC_NE_lexicon_tags_2 },
    { 0xec8043d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456643dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05a43df, SUC_NE_lexicon_tags_2 },
    { 0x109443e0, SUC_NE_lexicon_tags_0 },
    { 0x605e43e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc7c43e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbc43ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc43ed, SUC_NE_lexicon_tags_2 },
    { 0xbe8e43ee, SUC_NE_lexicon_tags_2 },
    { 0xede643ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567043f3, SUC_NE_lexicon_tags_2 },
    { 0xec5043f4, SUC_NE_lexicon_tags_2 },
    { 0x6eae43f5, SUC_NE_lexicon_tags_2 },
    { 0x8f4043f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x450443f8, SUC_NE_lexicon_tags_2 },
    { 0x767c43f8, SUC_NE_lexicon_tags_2 },
    { 0xf1e043fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fda43fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc8c43fe, SUC_NE_lexicon_tags_2 },
    { 0xae5a43fe, SUC_NE_lexicon_tags_2 },
    { 0xc5c4400, SUC_NE_lexicon_tags_0 },
    { 0xf4fe4400, SUC_NE_lexicon_tags_2 },
    { 0xf3964402, SUC_NE_lexicon_tags_2 },
    { 0x349c4400, SUC_NE_lexicon_tags_2 },
    { 0xbe0c4404, SUC_NE_lexicon_tags_2 },
    { 0xa8b24405, SUC_NE_lexicon_tags_2 },
    { 0x31a4406, SUC_NE_lexicon_tags_2 },
    { 0xcb744402, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x528a4409, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44c2440b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5baa440d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07c4416, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x487a4418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa310441a, SUC_NE_lexicon_tags_2 },
    { 0x1930441b, SUC_NE_lexicon_tags_2 },
    { 0x66c2441b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe208441f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf6c4421, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248a4425, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead04429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeec4431, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0xd244433, SUC_NE_lexicon_tags_30 },
    { 0x4b604434, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93a24436, SUC_NE_lexicon_tags_2 },
    { 0x38c84437, SUC_NE_lexicon_tags_2 },
    { 0x55f64437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1a443c, SUC_NE_lexicon_tags_24 },
    { 0xa6a8443d, SUC_NE_lexicon_tags_2 },
    { 0x9cee443e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a84441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d664443, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b04447, SUC_NE_lexicon_tags_3 },
    { 0x9d764448, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9e444c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12424450, SUC_NE_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c44453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4456, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d04459, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde56445c, SUC_NE_lexicon_tags_2 },
    { 0xc2b6445d, SUC_NE_lexicon_tags_2 },
    { 0x1436445e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5804460, SUC_NE_lexicon_tags_2 },
    { 0x936e4461, SUC_NE_lexicon_tags_3 },
    { 0xde004462, SUC_NE_lexicon_tags_7 },
    { 0x55904462, SUC_NE_lexicon_tags_2 },
    { 0x15a84462, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee6446a, SUC_NE_lexicon_tags_2 },
    { 0x640446b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcc8446d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fba4475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc4479, SUC_NE_lexicon_tags_2 },
    { 0xe64c4479, SUC_NE_lexicon_tags_2 },
    { 0xb03a447b, SUC_NE_lexicon_tags_2 },
    { 0xc254447a, SUC_NE_lexicon_tags_2 },
    { 0xf954447d, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fe4487, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9368448a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fa448d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa2448f, SUC_NE_lexicon_tags_2 },
    { 0x6ee4490, SUC_NE_lexicon_tags_2 },
    { 0x89144491, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47864496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x570a4498, SUC_NE_lexicon_tags_2 },
    { 0x9ec04498, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23b0449b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e80449d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5044a0, SUC_NE_lexicon_tags_2 },
    { 0x166e44a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab1e44a3, SUC_NE_lexicon_tags_0 },
    { 0x781844a4, SUC_NE_lexicon_tags_2 },
    { 0x6eda44a5, SUC_NE_lexicon_tags_2 },
    { 0x1ffa44a5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841a44aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2afc44ac, SUC_NE_lexicon_tags_2 },
    { 0x17244ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09044b2, SUC_NE_lexicon_tags_2 },
    { 0xc7ba44b3, SUC_NE_lexicon_tags_2 },
    { 0x3ab444b3, SUC_NE_lexicon_tags_2 },
    { 0x999a44b5, SUC_NE_lexicon_tags_2 },
    { 0xba4a44b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636444bf, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0a44c5, SUC_NE_lexicon_tags_2 },
    { 0x93e844c6, SUC_NE_lexicon_tags_7 },
    { 0xedca44c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c244ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ec44d1, SUC_NE_lexicon_tags_2 },
    { 0xbb9a44d2, SUC_NE_lexicon_tags_2 },
    { 0x618c44d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6244d6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a044dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6044e5, SUC_NE_lexicon_tags_2 },
    { 0xf7aa44e6, SUC_NE_lexicon_tags_2 },
    { 0xd3244e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x537844e9, SUC_NE_lexicon_tags_2 },
    { 0x961244ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd444ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baa44f4, SUC_NE_lexicon_tags_2 },
    { 0x313444f5, SUC_NE_lexicon_tags_2 },
    { 0xe7f844f5, SUC_NE_lexicon_tags_2 },
    { 0xefe844f7, SUC_NE_lexicon_tags_2 },
    { 0xbd2444f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5a44fd, SUC_NE_lexicon_tags_2 },
    { 0x11d444fd, SUC_NE_lexicon_tags_2 },
    { 0x93d244ff, SUC_NE_lexicon_tags_2 },
    { 0xb4ae44ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf3c4502, SUC_NE_lexicon_tags_2 },
    { 0x21f44503, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32c4509, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b68450b, SUC_NE_lexicon_tags_2 },
    { 0x56de450c, SUC_NE_lexicon_tags_2 },
    { 0xaba0450c, SUC_NE_lexicon_tags_2 },
    { 0x51ac450e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71244510, SUC_NE_lexicon_tags_4 },
    { 0xa1564511, SUC_NE_lexicon_tags_40 },
    { 0xe4e84512, SUC_NE_lexicon_tags_2 },
    { 0xbbe64513, SUC_NE_lexicon_tags_2 },
    { 0x6d5a4514, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e4519, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5044523, SUC_NE_lexicon_tags_5 },
    { 0xbca64524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732c4528, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xcb88452a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c32452d, SUC_NE_lexicon_tags_2 },
    { 0x8c6e452d, SUC_NE_lexicon_tags_2 },
    { 0xe794452f, SUC_NE_lexicon_tags_2 },
    { 0x9d28452e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35704535, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a3e4537, SUC_NE_lexicon_tags_2 },
    { 0xdc224538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94e453e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a6c4540, SUC_NE_lexicon_tags_2 },
    { 0x6bd24540, SUC_NE_lexicon_tags_2 },
    { 0x94624542, SUC_NE_lexicon_tags_2 },
    { 0x4d6a4543, SUC_NE_lexicon_tags_2 },
    { 0xad224544, SUC_NE_lexicon_tags_2 },
    { 0xc4f04545, SUC_NE_lexicon_tags_4 },
    { 0xf3b64545, SUC_NE_lexicon_tags_2 },
    { 0x69da4543, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c454b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189a4555, SUC_NE_lexicon_tags_2 },
    { 0xed544556, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b64559, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6be2455b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57da4560, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db44564, SUC_NE_lexicon_tags_385 },
    { 0xcf964565, SUC_NE_lexicon_tags_2 },
    { 0x7ef04566, SUC_NE_lexicon_tags_2 },
    { 0x2d584566, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f224569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a4456c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130c4570, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0804575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12504577, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c724579, SUC_NE_lexicon_tags_2 },
    { 0x1eb8457a, SUC_NE_lexicon_tags_2 },
    { 0xdb0e457b, SUC_NE_lexicon_tags_4 },
    { 0x42cc457a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27564582, SUC_NE_lexicon_tags_8 },
    { 0xa8264583, SUC_NE_lexicon_tags_2 },
    { 0xf1ec4583, SUC_NE_lexicon_tags_2 },
    { 0x7b9c4582, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd4458d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f14458f, SUC_NE_lexicon_tags_2 },
    { 0x61d24590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada24595, SUC_NE_lexicon_tags_1 },
    { 0x1a584596, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff8459e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8845a1, SUC_NE_lexicon_tags_18 },
    { 0xc48445a1, SUC_NE_lexicon_tags_2 },
    { 0xf07045a3, SUC_NE_lexicon_tags_2 },
    { 0x276e45a4, SUC_NE_lexicon_tags_117 },
    { 0xdb0445a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40c45a8, SUC_NE_lexicon_tags_27 },
    { 0x936045a9, SUC_NE_lexicon_tags_2 },
    { 0xc10c45a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7e845ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d445af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c0245b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57845b3, SUC_NE_lexicon_tags_2 },
    { 0x24445b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1a45b8, SUC_NE_lexicon_tags_2 },
    { 0xb1c445b9, SUC_NE_lexicon_tags_2 },
    { 0x76c045ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1445bf, SUC_NE_lexicon_tags_4 },
    { 0x487645c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaec45c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96e245c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x914445c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482445cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2645d0, SUC_NE_lexicon_tags_14 },
    { 0x6b3045d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d0645d3, SUC_NE_lexicon_tags_2 },
    { 0x94a845d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbc45d7, SUC_NE_lexicon_tags_2 },
    { 0x29ac45d8, SUC_NE_lexicon_tags_1 },
    { 0xc27245d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe45db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d0045dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93245e1, SUC_NE_lexicon_tags_2 },
    { 0xe71e45e1, SUC_NE_lexicon_tags_109 },
    { 0x215a45e3, SUC_NE_lexicon_tags_2 },
    { 0xa89645e4, SUC_NE_lexicon_tags_2 },
    { 0x27f045e2, SUC_NE_lexicon_tags_2 },
    { 0x489a45e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6045eb, SUC_NE_lexicon_tags_2 },
    { 0x835e45eb, SUC_NE_lexicon_tags_2 },
    { 0xc63045eb, SUC_NE_lexicon_tags_2 },
    { 0x33d445ec, SUC_NE_lexicon_tags_2 },
    { 0x389045ef, SUC_NE_lexicon_tags_2 },
    { 0xa31c45f0, SUC_NE_lexicon_tags_3 },
    { 0x7ca845f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8045f4, SUC_NE_lexicon_tags_4 },
    { 0x35e645f5, SUC_NE_lexicon_tags_2 },
    { 0x621245f6, SUC_NE_lexicon_tags_2 },
    { 0xf4ee45f5, SUC_NE_lexicon_tags_2 },
    { 0x44a045f5, SUC_NE_lexicon_tags_2 },
    { 0x1f4445f9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x163445fc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe045fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f484600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d124602, SUC_NE_lexicon_tags_2 },
    { 0x61da4603, SUC_NE_lexicon_tags_2 },
    { 0xb90c4604, SUC_NE_lexicon_tags_2 },
    { 0x4244605, SUC_NE_lexicon_tags_198 },
    { 0xacae4606, SUC_NE_lexicon_tags_2 },
    { 0x91484606, SUC_NE_lexicon_tags_2 },
    { 0x52704608, SUC_NE_lexicon_tags_2 },
    { 0x1c964606, SUC_NE_lexicon_tags_2 },
    { 0x4b26460a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a84610, SUC_NE_lexicon_tags_57 },
    { 0x128e4611, SUC_NE_lexicon_tags_2 },
    { 0x333a4612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39064616, SUC_NE_lexicon_tags_2 },
    { 0x74e64617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6461a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x516c461d, SUC_NE_lexicon_tags_2 },
    { 0xb792461d, SUC_NE_lexicon_tags_2 },
    { 0xbc20461f, SUC_NE_lexicon_tags_2 },
    { 0x890a461f, SUC_NE_lexicon_tags_2 },
    { 0xfafe4621, SUC_NE_lexicon_tags_2 },
    { 0x19464622, SUC_NE_lexicon_tags_2 },
    { 0xb6844623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd11c4625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dc4628, SUC_NE_lexicon_tags_4 },
    { 0x24e44628, SUC_NE_lexicon_tags_2 },
    { 0x597c462a, SUC_NE_lexicon_tags_2 },
    { 0x25e462b, SUC_NE_lexicon_tags_2 },
    { 0xe9b2462c, SUC_NE_lexicon_tags_2 },
    { 0x640c462d, SUC_NE_lexicon_tags_2 },
    { 0xbf00462b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9584630, SUC_NE_lexicon_tags_19 },
    { 0xd3284631, SUC_NE_lexicon_tags_2 },
    { 0xad3c4632, SUC_NE_lexicon_tags_2 },
    { 0x6f804630, SUC_NE_lexicon_tags_0 },
    { 0x54fe4633, SUC_NE_lexicon_tags_2 },
    { 0x51184635, SUC_NE_lexicon_tags_1 },
    { 0xb0444634, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd8463a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b94463c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97a463f, SUC_NE_lexicon_tags_2 },
    { 0x81b04640, SUC_NE_lexicon_tags_2 },
    { 0xbfbe4640, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94204645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94024648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cca464a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb79a464c, SUC_NE_lexicon_tags_2 },
    { 0xcd38464d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb624653, SUC_NE_lexicon_tags_2 },
    { 0x4d04653, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c4658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbba465a, SUC_NE_lexicon_tags_2 },
    { 0xf894465b, SUC_NE_lexicon_tags_2 },
    { 0xf44465c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b6e465e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d164661, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab424665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d724668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2a4670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31924672, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6284674, SUC_NE_lexicon_tags_2 },
    { 0xcdb04675, SUC_NE_lexicon_tags_2 },
    { 0x65ea4674, SUC_NE_lexicon_tags_2 },
    { 0x9f5a4677, SUC_NE_lexicon_tags_2 },
    { 0x4b8e4678, SUC_NE_lexicon_tags_14 },
    { 0x89824677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4467c, SUC_NE_lexicon_tags_30 },
    { 0x5b7c467d, SUC_NE_lexicon_tags_2 },
    { 0xb1dc467e, SUC_NE_lexicon_tags_2 },
    { 0xa3d0467e, SUC_NE_lexicon_tags_2 },
    { 0x1dca467f, SUC_NE_lexicon_tags_2 },
    { 0x4f344681, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c844688, SUC_NE_lexicon_tags_2 },
    { 0x3eca4688, SUC_NE_lexicon_tags_2 },
    { 0x340c4689, SUC_NE_lexicon_tags_2 },
    { 0xaff4468b, SUC_NE_lexicon_tags_2 },
    { 0xda82468a, SUC_NE_lexicon_tags_2 },
    { 0x7be4468d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb624690, SUC_NE_lexicon_tags_2 },
    { 0xf5d04690, SUC_NE_lexicon_tags_2 },
    { 0xdd884692, SUC_NE_lexicon_tags_2 },
    { 0xf26c4692, SUC_NE_lexicon_tags_2 },
    { 0x61724693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5864699, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca6469d, SUC_NE_lexicon_tags_2 },
    { 0xeb6e469d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47e46a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24e46a4, SUC_NE_lexicon_tags_40 },
    { 0x11fe46a4, SUC_NE_lexicon_tags_2 },
    { 0x858646a6, SUC_NE_lexicon_tags_2 },
    { 0xce6e46a7, SUC_NE_lexicon_tags_2 },
    { 0x306446a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e46ae, SUC_NE_lexicon_tags_2 },
    { 0x473846af, SUC_NE_lexicon_tags_2 },
    { 0xc96046b0, SUC_NE_lexicon_tags_2 },
    { 0x1ed046b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x837046b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc15e46b5, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71646ba, SUC_NE_lexicon_tags_2 },
    { 0xcffa46bb, SUC_NE_lexicon_tags_2 },
    { 0x14ea46ba, SUC_NE_lexicon_tags_2 },
    { 0xdd3646bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x768446bf, SUC_NE_lexicon_tags_2 },
    { 0x304646c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a9a46c2, SUC_NE_lexicon_tags_2 },
    { 0xf50e46c3, SUC_NE_lexicon_tags_4 },
    { 0x55e646c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81da46c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c46c9, SUC_NE_lexicon_tags_2 },
    { 0xec7446c9, SUC_NE_lexicon_tags_2 },
    { 0xebc446c9, SUC_NE_lexicon_tags_2 },
    { 0xf36646cc, SUC_NE_lexicon_tags_2 },
    { 0xbe0c46cd, SUC_NE_lexicon_tags_2 },
    { 0x846046cd, SUC_NE_lexicon_tags_158 },
    { 0x76c046cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fd646d1, SUC_NE_lexicon_tags_2 },
    { 0xe60646d2, SUC_NE_lexicon_tags_2 },
    { 0xad9c46d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3446d6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94c46db, SUC_NE_lexicon_tags_2 },
    { 0xeffc46db, SUC_NE_lexicon_tags_4 },
    { 0xe72446db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x527046df, SUC_NE_lexicon_tags_2 },
    { 0x667646e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x394a46e2, SUC_NE_lexicon_tags_3 },
    { 0x710446e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd92a46e5, SUC_NE_lexicon_tags_2 },
    { 0x85aa46e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09a46ea, SUC_NE_lexicon_tags_2 },
    { 0x2e1c46eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e7a46ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf846f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e846f9, SUC_NE_lexicon_tags_2 },
    { 0xab9c46fa, SUC_NE_lexicon_tags_2 },
    { 0x155646fa, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x67da46fd, SUC_NE_lexicon_tags_2 },
    { 0x256446fe, SUC_NE_lexicon_tags_2 },
    { 0xfade46ff, SUC_NE_lexicon_tags_2 },
    { 0x665a46fe, SUC_NE_lexicon_tags_2 },
    { 0xa21046ff, SUC_NE_lexicon_tags_2 },
    { 0x839a4702, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838a4706, SUC_NE_lexicon_tags_2 },
    { 0x8e24707, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x173e470a, SUC_NE_lexicon_tags_2 },
    { 0xab5a470b, SUC_NE_lexicon_tags_3 },
    { 0x3608470b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04e4710, SUC_NE_lexicon_tags_2 },
    { 0x13fa4711, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2c4715, SUC_NE_lexicon_tags_2 },
    { 0xabae4715, SUC_NE_lexicon_tags_2 },
    { 0xca8a4715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b6471f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2ea64721, SUC_NE_lexicon_tags_15 },
    { 0xbc0e4721, SUC_NE_lexicon_tags_2 },
    { 0x36324723, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb724725, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xfd844727, SUC_NE_lexicon_tags_2 },
    { 0x4d884728, SUC_NE_lexicon_tags_2 },
    { 0x8b884729, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e84472b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd644472e, SUC_NE_lexicon_tags_2 },
    { 0xfc5c472f, SUC_NE_lexicon_tags_2 },
    { 0x87c84730, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47264734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b98473d, SUC_NE_lexicon_tags_2 },
    { 0xa7ae473e, SUC_NE_lexicon_tags_2 },
    { 0xf69e473f, SUC_NE_lexicon_tags_2 },
    { 0x4864740, SUC_NE_lexicon_tags_2 },
    { 0x23e64741, SUC_NE_lexicon_tags_2 },
    { 0xdf62473d, SUC_NE_lexicon_tags_2 },
    { 0xfcce4743, SUC_NE_lexicon_tags_3 },
    { 0x99f2473d, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e04749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57c474e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e424752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20824755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd628475b, SUC_NE_lexicon_tags_2 },
    { 0xdb4e475c, SUC_NE_lexicon_tags_2 },
    { 0x372c475c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6644760, SUC_NE_lexicon_tags_2 },
    { 0xaa44761, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1984763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3464765, SUC_NE_lexicon_tags_36 },
    { 0xed8a4766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78aa4768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea54476b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x372c476d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be4771, SUC_NE_lexicon_tags_2 },
    { 0xd0d24772, SUC_NE_lexicon_tags_17 },
    { 0x20d64773, SUC_NE_lexicon_tags_2 },
    { 0x538a4774, SUC_NE_lexicon_tags_2 },
    { 0x5a484773, SUC_NE_lexicon_tags_2 },
    { 0x7ac44776, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49da4779, SUC_NE_lexicon_tags_138 },
    { 0x9f484779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a8477d, SUC_NE_lexicon_tags_2 },
    { 0x3354477d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdea4781, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4bc4783, SUC_NE_lexicon_tags_2 },
    { 0x14f44783, SUC_NE_lexicon_tags_3 },
    { 0x431a4784, SUC_NE_lexicon_tags_2 },
    { 0xad944786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd0478a, SUC_NE_lexicon_tags_2 },
    { 0x3374478b, SUC_NE_lexicon_tags_2 },
    { 0xa6ce478c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb350478f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c84792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f04795, SUC_NE_lexicon_tags_2 },
    { 0xc9124796, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356479f, SUC_NE_lexicon_tags_2 },
    { 0xb11e47a0, SUC_NE_lexicon_tags_2 },
    { 0xcfce479f, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41e47a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x828647a8, SUC_NE_lexicon_tags_2 },
    { 0xc22647a8, SUC_NE_lexicon_tags_2 },
    { 0xe28e47aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713247ae, SUC_NE_lexicon_tags_2 },
    { 0x7c8e47ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf62a47b1, SUC_NE_lexicon_tags_2 },
    { 0xccba47b2, SUC_NE_lexicon_tags_2 },
    { 0x372047b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81c47b6, SUC_NE_lexicon_tags_2 },
    { 0xf4a847b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797847bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ff647be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ee47c2, SUC_NE_lexicon_tags_2 },
    { 0x4a0047c3, SUC_NE_lexicon_tags_2 },
    { 0x626e47c3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6247c9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fb447cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80247d2, SUC_NE_lexicon_tags_2 },
    { 0xe96a47d2, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3a47d8, SUC_NE_lexicon_tags_2 },
    { 0xb8ae47d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb43247db, SUC_NE_lexicon_tags_2 },
    { 0x856447dc, SUC_NE_lexicon_tags_2 },
    { 0x117e47dd, SUC_NE_lexicon_tags_2 },
    { 0x7c9447dc, SUC_NE_lexicon_tags_2 },
    { 0x91f247dc, SUC_NE_lexicon_tags_2 },
    { 0xc2b647dd, SUC_NE_lexicon_tags_2 },
    { 0xd37847e1, SUC_NE_lexicon_tags_2 },
    { 0xc59e47e0, SUC_NE_lexicon_tags_2 },
    { 0x3c2247e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x828247e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2047e8, SUC_NE_lexicon_tags_2 },
    { 0xc56447e9, SUC_NE_lexicon_tags_2 },
    { 0xd88c47ea, SUC_NE_lexicon_tags_2 },
    { 0x3c1c47ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cd647ed, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x863647ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe447f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6e47f5, SUC_NE_lexicon_tags_2 },
    { 0xf59a47f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf3647f8, SUC_NE_lexicon_tags_2 },
    { 0xa8647f8, SUC_NE_lexicon_tags_2 },
    { 0x37a47fa, SUC_NE_lexicon_tags_2 },
    { 0x605647fa, SUC_NE_lexicon_tags_2 },
    { 0xf23047f9, SUC_NE_lexicon_tags_2 },
    { 0x3a0c47fd, SUC_NE_lexicon_tags_2 },
    { 0x68247fd, SUC_NE_lexicon_tags_3 },
    { 0x8f4847fe, SUC_NE_lexicon_tags_8 },
    { 0xc6da4800, SUC_NE_lexicon_tags_2 },
    { 0xfda04801, SUC_NE_lexicon_tags_1 },
    { 0x8dc44802, SUC_NE_lexicon_tags_2 },
    { 0x1ab84802, SUC_NE_lexicon_tags_2 },
    { 0x66f04804, SUC_NE_lexicon_tags_2 },
    { 0x3c924804, SUC_NE_lexicon_tags_2 },
    { 0xb6c04805, SUC_NE_lexicon_tags_4 },
    { 0x76de4807, SUC_NE_lexicon_tags_4 },
    { 0x2adc4808, SUC_NE_lexicon_tags_2 },
    { 0xe6724809, SUC_NE_lexicon_tags_2 },
    { 0x2fda4809, SUC_NE_lexicon_tags_2 },
    { 0x3e58480b, SUC_NE_lexicon_tags_0 },
    { 0x42c24806, SUC_NE_lexicon_tags_2 },
    { 0x29a0480d, SUC_NE_lexicon_tags_2 },
    { 0x6c58480e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x888a4811, SUC_NE_lexicon_tags_2 },
    { 0x5bb44812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e04815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ac4819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab14481b, SUC_NE_lexicon_tags_2 },
    { 0xee7a481c, SUC_NE_lexicon_tags_2 },
    { 0xefd2481d, SUC_NE_lexicon_tags_2 },
    { 0x7e64481e, SUC_NE_lexicon_tags_2 },
    { 0x7714481f, SUC_NE_lexicon_tags_2 },
    { 0xf30e4820, SUC_NE_lexicon_tags_2 },
    { 0x30ba4820, SUC_NE_lexicon_tags_2 },
    { 0xe4044821, SUC_NE_lexicon_tags_2 },
    { 0xc5044822, SUC_NE_lexicon_tags_2 },
    { 0xbbe481e, SUC_NE_lexicon_tags_2 },
    { 0x678a4825, SUC_NE_lexicon_tags_2 },
    { 0x1a76481d, SUC_NE_lexicon_tags_2 },
    { 0x6c84820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d60482c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50482f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf544833, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa60a4835, SUC_NE_lexicon_tags_2 },
    { 0xbb6e4836, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xabc64838, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xbc6c483a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0e2483c, SUC_NE_lexicon_tags_198 },
    { 0x1cda483d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x434a4840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f1a4842, SUC_NE_lexicon_tags_2 },
    { 0x3fe84843, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44524845, SUC_NE_lexicon_tags_39 },
    { 0x2e964846, SUC_NE_lexicon_tags_3 },
    { 0x10f84847, SUC_NE_lexicon_tags_2 },
    { 0x3c044847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x500c484a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab544850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64a4853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92144855, SUC_NE_lexicon_tags_2 },
    { 0x8da24856, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514e485a, SUC_NE_lexicon_tags_2 },
    { 0x3288485b, SUC_NE_lexicon_tags_2 },
    { 0x3e3c485a, SUC_NE_lexicon_tags_2 },
    { 0x81ee485a, SUC_NE_lexicon_tags_2 },
    { 0x1a0e485c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8e4869, SUC_NE_lexicon_tags_2 },
    { 0x6f26486a, SUC_NE_lexicon_tags_2 },
    { 0x47b8486a, SUC_NE_lexicon_tags_2 },
    { 0x37d4486b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e30486e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea24871, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7c44873, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb55c4875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206e4879, SUC_NE_lexicon_tags_2 },
    { 0x6442487a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264c4880, SUC_NE_lexicon_tags_2 },
    { 0x89a84881, SUC_NE_lexicon_tags_2 },
    { 0x42944882, SUC_NE_lexicon_tags_2 },
    { 0xfdd44883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e488b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x922e488d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x398e488f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1a24891, SUC_NE_lexicon_tags_7 },
    { 0x58524892, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a904895, SUC_NE_lexicon_tags_2 },
    { 0x7cec4895, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2dc4898, SUC_NE_lexicon_tags_2 },
    { 0x6e904899, SUC_NE_lexicon_tags_2 },
    { 0xbe4489a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44f4489c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x734a489e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbc48a1, SUC_NE_lexicon_tags_2 },
    { 0xeff048a2, SUC_NE_lexicon_tags_2 },
    { 0x3ee048a2, SUC_NE_lexicon_tags_2 },
    { 0xc73248a4, SUC_NE_lexicon_tags_2 },
    { 0x6dba48a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc51c48a7, SUC_NE_lexicon_tags_2 },
    { 0xbd7c48a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cc248aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefe048ac, SUC_NE_lexicon_tags_2 },
    { 0xf3e648ad, SUC_NE_lexicon_tags_40 },
    { 0xa55448ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7648b5, SUC_NE_lexicon_tags_2 },
    { 0x26c248b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be48b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e048be, SUC_NE_lexicon_tags_2 },
    { 0x8bd448bf, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86448c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x826848c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x330a48ca, SUC_NE_lexicon_tags_2 },
    { 0x3ec448cb, SUC_NE_lexicon_tags_3 },
    { 0x1ba448cc, SUC_NE_lexicon_tags_2 },
    { 0x3df248cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x377448cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae4248d1, SUC_NE_lexicon_tags_2 },
    { 0xe57e48d1, SUC_NE_lexicon_tags_2 },
    { 0xcf8048d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58da48d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a3e48d8, SUC_NE_lexicon_tags_2 },
    { 0x218c48d9, SUC_NE_lexicon_tags_2 },
    { 0x8adc48da, SUC_NE_lexicon_tags_2 },
    { 0xb69048db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0848df, SUC_NE_lexicon_tags_2 },
    { 0xb2a448e0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1048e4, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247248e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8a48eb, SUC_NE_lexicon_tags_2 },
    { 0xf6e448ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553248f0, SUC_NE_lexicon_tags_2 },
    { 0x5e2e48f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bc48f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786448f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c248fb, SUC_NE_lexicon_tags_2 },
    { 0xff5a48fc, SUC_NE_lexicon_tags_3 },
    { 0x7afc48fb, SUC_NE_lexicon_tags_2 },
    { 0x488848fe, SUC_NE_lexicon_tags_19 },
    { 0x88b248ff, SUC_NE_lexicon_tags_2 },
    { 0xd6ac4900, SUC_NE_lexicon_tags_2 },
    { 0x49e4901, SUC_NE_lexicon_tags_3 },
    { 0xfa44902, SUC_NE_lexicon_tags_2 },
    { 0xc46648fe, SUC_NE_lexicon_tags_2 },
    { 0xbac848fe, SUC_NE_lexicon_tags_2 },
    { 0xaa324905, SUC_NE_lexicon_tags_170 },
    { 0xa9e04905, SUC_NE_lexicon_tags_2 },
    { 0x4a8848ff, SUC_NE_lexicon_tags_2 },
    { 0x28764901, SUC_NE_lexicon_tags_1 },
    { 0x1a904909, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9204912, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x30844914, SUC_NE_lexicon_tags_2 },
    { 0xa9f64914, SUC_NE_lexicon_tags_2 },
    { 0x6ee24915, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be84919, SUC_NE_lexicon_tags_2 },
    { 0xfe584919, SUC_NE_lexicon_tags_2 },
    { 0x94de491b, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0x2dc8491d, SUC_NE_lexicon_tags_2 },
    { 0x21e4491e, SUC_NE_lexicon_tags_2 },
    { 0xbe5c491f, SUC_NE_lexicon_tags_1 },
    { 0xcd4c491e, SUC_NE_lexicon_tags_2 },
    { 0xa54a4920, SUC_NE_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a264924, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fa4928, SUC_NE_lexicon_tags_2 },
    { 0x15ac4929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad164933, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f664936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382c493c, SUC_NE_lexicon_tags_18 },
    { 0x8d94493c, SUC_NE_lexicon_tags_2 },
    { 0x377e493d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12744943, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89764945, SUC_NE_lexicon_tags_2 },
    { 0x1eac4945, SUC_NE_lexicon_tags_49 },
    { 0xfa764947, SUC_NE_lexicon_tags_2 },
    { 0x43824948, SUC_NE_lexicon_tags_2 },
    { 0x7d1c4949, SUC_NE_lexicon_tags_2 },
    { 0xe810494a, SUC_NE_lexicon_tags_2 },
    { 0x8886494a, SUC_NE_lexicon_tags_2 },
    { 0xcf66494b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e24950, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77244952, SUC_NE_lexicon_tags_2 },
    { 0x7b4c4952, SUC_NE_lexicon_tags_7 },
    { 0xc03e4953, SUC_NE_lexicon_tags_2 },
    { 0xf8184955, SUC_NE_lexicon_tags_2 },
    { 0xbaba4956, SUC_NE_lexicon_tags_2 },
    { 0x5f604957, SUC_NE_lexicon_tags_2 },
    { 0x64884957, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cc495c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a78495e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99964963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0e4967, SUC_NE_lexicon_tags_2 },
    { 0xdda04967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe496d, SUC_NE_lexicon_tags_49 },
    { 0x8894496e, SUC_NE_lexicon_tags_441 },
    { 0x43ba496e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b464971, SUC_NE_lexicon_tags_2 },
    { 0x3584971, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa64975, SUC_NE_lexicon_tags_2 },
    { 0x4bca4975, SUC_NE_lexicon_tags_2 },
    { 0x4f984977, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc497a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d0497e, SUC_NE_lexicon_tags_5 },
    { 0xa87e497e, SUC_NE_lexicon_tags_2 },
    { 0xc1c4980, SUC_NE_lexicon_tags_2 },
    { 0x2c10497f, SUC_NE_lexicon_tags_2 },
    { 0x509a497e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc34a4984, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff424987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd146498f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11ac4991, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaec4994, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b704996, SUC_NE_lexicon_tags_2 },
    { 0x72924996, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8a499a, SUC_NE_lexicon_tags_17 },
    { 0xfbd6499b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9a499f, SUC_NE_lexicon_tags_2 },
    { 0x6fd0499f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afa49a7, SUC_NE_lexicon_tags_4 },
    { 0xe4da49a8, SUC_NE_lexicon_tags_1 },
    { 0x751849a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x465a49ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaeaa49ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864a49b2, SUC_NE_lexicon_tags_2 },
    { 0x451849b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2849b7, SUC_NE_lexicon_tags_2 },
    { 0x95d249b8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b249bd, SUC_NE_lexicon_tags_0 },
    { 0x7e3a49bd, SUC_NE_lexicon_tags_2 },
    { 0x429249bf, SUC_NE_lexicon_tags_2 },
    { 0x2d4049bf, SUC_NE_lexicon_tags_2 },
    { 0xec1a49c1, SUC_NE_lexicon_tags_2 },
    { 0xdd4449c2, SUC_NE_lexicon_tags_2 },
    { 0xb93e49c2, SUC_NE_lexicon_tags_2 },
    { 0xc7ce49c4, SUC_NE_lexicon_tags_2 },
    { 0xb2c49c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1249ca, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83e49d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87849d6, SUC_NE_lexicon_tags_2 },
    { 0x1b3a49d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eb049d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e49de, SUC_NE_lexicon_tags_2 },
    { 0xd9b049df, SUC_NE_lexicon_tags_2 },
    { 0x501449de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf7649e2, SUC_NE_lexicon_tags_1 },
    { 0x26cc49e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d049eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c049f0, SUC_NE_lexicon_tags_2 },
    { 0x6c2449f0, SUC_NE_lexicon_tags_19 },
    { 0x569049f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e449f6, SUC_NE_lexicon_tags_2 },
    { 0x3cba49f6, SUC_NE_lexicon_tags_2 },
    { 0x449c49f6, SUC_NE_lexicon_tags_2 },
    { 0x12249f9, SUC_NE_lexicon_tags_2 },
    { 0xefc649fa, SUC_NE_lexicon_tags_2 },
    { 0xa68e49f7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e649fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574e4a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8184a09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab3a4a0b, SUC_NE_lexicon_tags_2 },
    { 0xf2504a0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6bc4a0e, SUC_NE_lexicon_tags_313 },
    { 0x5f2e4a0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2064a1f, SUC_NE_lexicon_tags_1 },
    { 0x93084a1f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc64a27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fba4a29, SUC_NE_lexicon_tags_2 },
    { 0x4a304a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bac4a2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1a4a32, SUC_NE_lexicon_tags_181 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c4a38, SUC_NE_lexicon_tags_2 },
    { 0x1a304a39, SUC_NE_lexicon_tags_2 },
    { 0x9fea4a3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5a84a3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb44a40, SUC_NE_lexicon_tags_2 },
    { 0xd4804a41, SUC_NE_lexicon_tags_2 },
    { 0x9c2e4a42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa44a47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2c4a4d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfcc04a4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c84a55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30844a5b, SUC_NE_lexicon_tags_2 },
    { 0xa35a4a5c, SUC_NE_lexicon_tags_2 },
    { 0xbf484a5d, SUC_NE_lexicon_tags_2 },
    { 0x5d1e4a5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54da4a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abe4a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261c4a68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814e4a6c, SUC_NE_lexicon_tags_7 },
    { 0xb9c44a6d, SUC_NE_lexicon_tags_2 },
    { 0x65c24a6e, SUC_NE_lexicon_tags_2 },
    { 0xf9404a6f, SUC_NE_lexicon_tags_3 },
    { 0x6ac84a70, SUC_NE_lexicon_tags_2 },
    { 0x32b24a6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4424a73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ed24a75, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf504a78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0c4a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafe4a80, SUC_NE_lexicon_tags_2 },
    { 0xa6f44a81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ea4a86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cc44a88, SUC_NE_lexicon_tags_1 },
    { 0x9dee4a89, SUC_NE_lexicon_tags_8 },
    { 0x38284a8a, SUC_NE_lexicon_tags_3 },
    { 0xbaf84a89, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xbe304a8d, SUC_NE_lexicon_tags_19 },
    { 0x54f64a8d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74c4a96, SUC_NE_lexicon_tags_2 },
    { 0xfd024a96, SUC_NE_lexicon_tags_2 },
    { 0x4dc04a98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bc4a9b, SUC_NE_lexicon_tags_2 },
    { 0xce524a9c, SUC_NE_lexicon_tags_2 },
    { 0xb18c4a9d, SUC_NE_lexicon_tags_2 },
    { 0x9d644a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd124aa1, SUC_NE_lexicon_tags_2 },
    { 0x97944aa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc984aa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a64aa7, SUC_NE_lexicon_tags_2 },
    { 0x65d84aa8, SUC_NE_lexicon_tags_2 },
    { 0xcf9a4aa9, SUC_NE_lexicon_tags_2 },
    { 0x583a4aa9, SUC_NE_lexicon_tags_4 },
    { 0x100a4aa9, SUC_NE_lexicon_tags_2 },
    { 0x84e64aac, SUC_NE_lexicon_tags_2 },
    { 0x5fe84aa9, SUC_NE_lexicon_tags_2 },
    { 0xecbc4aac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72044ab3, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xd1604ab5, SUC_NE_lexicon_tags_2 },
    { 0x98484ab6, SUC_NE_lexicon_tags_2 },
    { 0x8a824ab6, SUC_NE_lexicon_tags_2 },
    { 0xfdf84ab5, SUC_NE_lexicon_tags_2 },
    { 0x51864ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2364abb, SUC_NE_lexicon_tags_2 },
    { 0x57ac4abb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80984ac2, SUC_NE_lexicon_tags_2 },
    { 0xff04ac2, SUC_NE_lexicon_tags_2 },
    { 0x53f84ac4, SUC_NE_lexicon_tags_2 },
    { 0xd6464ac2, SUC_NE_lexicon_tags_2 },
    { 0x54404ac6, SUC_NE_lexicon_tags_2 },
    { 0xebf84ac7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fa4aca, SUC_NE_lexicon_tags_2 },
    { 0xb55c4acb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd04acd, SUC_NE_lexicon_tags_2 },
    { 0xafe04acd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bc64ad0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa4ad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3964adc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a204ade, SUC_NE_lexicon_tags_2 },
    { 0xc86e4adf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2004ae6, SUC_NE_lexicon_tags_2 },
    { 0x2f484ae7, SUC_NE_lexicon_tags_2 },
    { 0x6ef44ae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a24af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb04af4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc3a4af6, SUC_NE_lexicon_tags_2 },
    { 0xe01c4af6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb04afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c44aff, SUC_NE_lexicon_tags_2 },
    { 0xa8e4aff, SUC_NE_lexicon_tags_30 },
    { 0xa6e84b01, SUC_NE_lexicon_tags_2 },
    { 0xa2904b02, SUC_NE_lexicon_tags_2 },
    { 0x98a24b03, SUC_NE_lexicon_tags_2 },
    { 0xf9924b04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5184b06, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x78224b09, SUC_NE_lexicon_tags_2 },
    { 0xe6bc4b0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa32a4b0c, SUC_NE_lexicon_tags_36 },
    { 0x15904b0c, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b84b11, SUC_NE_lexicon_tags_8 },
    { 0xe5424b11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd84b15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e4b18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8e4b23, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x91824b26, SUC_NE_lexicon_tags_2 },
    { 0x22ee4b26, SUC_NE_lexicon_tags_2 },
    { 0x62664b27, SUC_NE_lexicon_tags_2 },
    { 0x6cf64b29, SUC_NE_lexicon_tags_2 },
    { 0xb8c44b2a, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d04b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63404b32, SUC_NE_lexicon_tags_2 },
    { 0x14804b33, SUC_NE_lexicon_tags_0 },
    { 0x9e924b34, SUC_NE_lexicon_tags_2 },
    { 0x6daa4b35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c64b39, SUC_NE_lexicon_tags_2 },
    { 0xcee4b39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cca4b3e, SUC_NE_lexicon_tags_2 },
    { 0xa9324b3f, SUC_NE_lexicon_tags_2 },
    { 0x9c084b40, SUC_NE_lexicon_tags_2 },
    { 0x65a4b40, SUC_NE_lexicon_tags_2 },
    { 0x9cd44b42, SUC_NE_lexicon_tags_2 },
    { 0x3d4e4b42, SUC_NE_lexicon_tags_2 },
    { 0x3a364b3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc04b4a, SUC_NE_lexicon_tags_2 },
    { 0x20e04b4a, SUC_NE_lexicon_tags_2 },
    { 0xe5a64b4c, SUC_NE_lexicon_tags_2 },
    { 0xf1ae4b4d, SUC_NE_lexicon_tags_2 },
    { 0xaf6e4b4a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x92384b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb24b57, SUC_NE_lexicon_tags_2 },
    { 0x3f0c4b57, SUC_NE_lexicon_tags_2 },
    { 0xf1b44b57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a4b5c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c44b62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90064b64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53f84b66, SUC_NE_lexicon_tags_2 },
    { 0xaa3a4b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ce04b69, SUC_NE_lexicon_tags_2 },
    { 0x29444b6a, SUC_NE_lexicon_tags_14 },
    { 0x48604b6a, SUC_NE_lexicon_tags_3 },
    { 0x946e4b6c, SUC_NE_lexicon_tags_7 },
    { 0xdc184b6b, SUC_NE_lexicon_tags_2 },
    { 0xfc64b6e, SUC_NE_lexicon_tags_2 },
    { 0x110a4b6e, SUC_NE_lexicon_tags_2 },
    { 0xe03c4b70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ec4b76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x531e4b79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9eb64b7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8c4b7d, SUC_NE_lexicon_tags_59 },
    { 0xb94a4b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb844b81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d384b85, SUC_NE_lexicon_tags_4 },
    { 0xf98e4b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8e84b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19464b8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53464b8d, SUC_NE_lexicon_tags_2 },
    { 0xae844b8d, SUC_NE_lexicon_tags_3 },
    { 0x8c524b8e, SUC_NE_lexicon_tags_2 },
    { 0x2a484b8e, SUC_NE_lexicon_tags_34 },
    { 0xca204b90, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xae5e4b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdede4b97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c04b9d, SUC_NE_lexicon_tags_2 },
    { 0x30084b9d, SUC_NE_lexicon_tags_2 },
    { 0x8c964b9f, SUC_NE_lexicon_tags_2 },
    { 0xae424b9f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ed04ba2, SUC_NE_lexicon_tags_2 },
    { 0xa98e4ba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee64ba5, SUC_NE_lexicon_tags_2 },
    { 0x55f04ba6, SUC_NE_lexicon_tags_2 },
    { 0xc8164ba6, SUC_NE_lexicon_tags_2 },
    { 0xf9004ba8, SUC_NE_lexicon_tags_2 },
    { 0x1a704ba8, SUC_NE_lexicon_tags_2 },
    { 0xcb84ba6, SUC_NE_lexicon_tags_2 },
    { 0x901e4bab, SUC_NE_lexicon_tags_2 },
    { 0xbd7a4bac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bc4baf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e4bb6, SUC_NE_lexicon_tags_17 },
    { 0x77c44bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf84bba, SUC_NE_lexicon_tags_2 },
    { 0x8d6a4bba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c44bbe, SUC_NE_lexicon_tags_2 },
    { 0xb2464bbf, SUC_NE_lexicon_tags_2 },
    { 0x7d864bc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefc4bc4, SUC_NE_lexicon_tags_2 },
    { 0xde4a4bc5, SUC_NE_lexicon_tags_2 },
    { 0xd53e4bc6, SUC_NE_lexicon_tags_2 },
    { 0x265e4bc7, SUC_NE_lexicon_tags_2 },
    { 0x62324bc7, SUC_NE_lexicon_tags_2 },
    { 0xca6c4bc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11744bd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e4bd3, SUC_NE_lexicon_tags_2 },
    { 0x39f84bd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d864bd6, SUC_NE_lexicon_tags_5 },
    { 0xe46e4bd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd44bda, SUC_NE_lexicon_tags_2 },
    { 0x7af44bdb, SUC_NE_lexicon_tags_2 },
    { 0x41264bdb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1844bde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d6a4be0, SUC_NE_lexicon_tags_40 },
    { 0x99a64be1, SUC_NE_lexicon_tags_2 },
    { 0xc80a4be2, SUC_NE_lexicon_tags_2 },
    { 0xc5364be3, SUC_NE_lexicon_tags_2 },
    { 0x3ebc4be2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bec4be6, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6e4bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6baa4bef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30bc4bf1, SUC_NE_lexicon_tags_2 },
    { 0x1aaa4bf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffe4bf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235c4bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ffe4bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932e4c01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a44c04, SUC_NE_lexicon_tags_2 },
    { 0x75ec4c05, SUC_NE_lexicon_tags_2 },
    { 0xa2444c04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a64c09, SUC_NE_lexicon_tags_2 },
    { 0x2cd44c0a, SUC_NE_lexicon_tags_2 },
    { 0x7a804c0b, SUC_NE_lexicon_tags_2 },
    { 0x63ea4c0a, SUC_NE_lexicon_tags_2 },
    { 0x494c4c0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27484c10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdc4c16, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b64c1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6984c1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43624c22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c9c4c24, SUC_NE_lexicon_tags_2 },
    { 0xc1024c25, SUC_NE_lexicon_tags_36 },
    { 0xa26c4c26, SUC_NE_lexicon_tags_2 },
    { 0xa0e64c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8064c2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46c64c2d, SUC_NE_lexicon_tags_2 },
    { 0xa4644c2e, SUC_NE_lexicon_tags_3 },
    { 0x4a284c2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3484c31, SUC_NE_lexicon_tags_2 },
    { 0x393a4c32, SUC_NE_lexicon_tags_65 },
    { 0xad404c33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d24c35, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18304c39, SUC_NE_lexicon_tags_2 },
    { 0x26e24c3a, SUC_NE_lexicon_tags_2 },
    { 0xd1644c3b, SUC_NE_lexicon_tags_2 },
    { 0x88824c3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b04c42, SUC_NE_lexicon_tags_2 },
    { 0x77404c42, SUC_NE_lexicon_tags_2 },
    { 0xff5a4c44, SUC_NE_lexicon_tags_2 },
    { 0x7f4c4c45, SUC_NE_lexicon_tags_2 },
    { 0x15d84c46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67da4c49, SUC_NE_lexicon_tags_2 },
    { 0xd0d04c4a, SUC_NE_lexicon_tags_2 },
    { 0xcd24c4a, SUC_NE_lexicon_tags_2 },
    { 0x965e4c4a, SUC_NE_lexicon_tags_2 },
    { 0xac204c4a, SUC_NE_lexicon_tags_2 },
    { 0x93dc4c4e, SUC_NE_lexicon_tags_2 },
    { 0x81304c4f, SUC_NE_lexicon_tags_2 },
    { 0xfb704c50, SUC_NE_lexicon_tags_2 },
    { 0x12c24c4f, SUC_NE_lexicon_tags_2 },
    { 0x4c484c52, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x79b84c54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ac4c5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffa4c5d, SUC_NE_lexicon_tags_2 },
    { 0xdce84c5e, SUC_NE_lexicon_tags_2 },
    { 0xd0f64c5f, SUC_NE_lexicon_tags_2 },
    { 0x96e44c60, SUC_NE_lexicon_tags_7 },
    { 0xcbbc4c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b264c63, SUC_NE_lexicon_tags_2 },
    { 0x64a4c63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a544c68, SUC_NE_lexicon_tags_2 },
    { 0x5f0a4c69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc764c6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a324c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa304c79, SUC_NE_lexicon_tags_2 },
    { 0x8e5c4c7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8eda4c7c, SUC_NE_lexicon_tags_2 },
    { 0xc76c4c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a384c81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54ea4c83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3684c88, SUC_NE_lexicon_tags_2 },
    { 0xd7364c89, SUC_NE_lexicon_tags_2 },
    { 0x8bb24c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff584c8f, SUC_NE_lexicon_tags_2 },
    { 0xd0364c90, SUC_NE_lexicon_tags_7 },
    { 0x83b44c91, SUC_NE_lexicon_tags_17 },
    { 0xb0aa4c91, SUC_NE_lexicon_tags_2 },
    { 0x19624c91, SUC_NE_lexicon_tags_2 },
    { 0x4dbc4c90, SUC_NE_lexicon_tags_10 },
    { 0x41904c8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0624c99, SUC_NE_lexicon_tags_2 },
    { 0x2c744c9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfda4c9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa74a4ca1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x76a04ca3, SUC_NE_lexicon_tags_2 },
    { 0x4f544ca3, SUC_NE_lexicon_tags_3 },
    { 0x91924ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79a4ca8, SUC_NE_lexicon_tags_2 },
    { 0x29d84ca9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34c44cab, SUC_NE_lexicon_tags_105 },
    { 0xdffc4cac, SUC_NE_lexicon_tags_2 },
    { 0x74764cad, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d224caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50284cb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431e4cb7, SUC_NE_lexicon_tags_2 },
    { 0xff284cb8, SUC_NE_lexicon_tags_2 },
    { 0x952a4cb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0f24cbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7784cc0, SUC_NE_lexicon_tags_2 },
    { 0x878c4cc1, SUC_NE_lexicon_tags_2 },
    { 0xfb724cc2, SUC_NE_lexicon_tags_2 },
    { 0x6644cc3, SUC_NE_lexicon_tags_2 },
    { 0x833e4cc4, SUC_NE_lexicon_tags_2 },
    { 0x93024cc1, SUC_NE_lexicon_tags_2 },
    { 0x20224cc6, SUC_NE_lexicon_tags_2 },
    { 0x5e344cc6, SUC_NE_lexicon_tags_2 },
    { 0xd4c04cc7, SUC_NE_lexicon_tags_2 },
    { 0x33524cc3, SUC_NE_lexicon_tags_17 },
    { 0xe8864cc8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e04cce, SUC_NE_lexicon_tags_2 },
    { 0xa22e4ccf, SUC_NE_lexicon_tags_18 },
    { 0x4d1c4cd0, SUC_NE_lexicon_tags_2 },
    { 0x396c4ccf, SUC_NE_lexicon_tags_1 },
    { 0xc95c4cd0, SUC_NE_lexicon_tags_2 },
    { 0xf89a4cd3, SUC_NE_lexicon_tags_2 },
    { 0xaa344cd4, SUC_NE_lexicon_tags_2 },
    { 0x14a84cd4, SUC_NE_lexicon_tags_2 },
    { 0xa8ca4cd1, SUC_NE_lexicon_tags_5 },
    { 0x592a4cd5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa12c4cd9, SUC_NE_lexicon_tags_7 },
    { 0xfb3e4cda, SUC_NE_lexicon_tags_2 },
    { 0x7a704cdb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81424ce2, SUC_NE_lexicon_tags_17 },
    { 0x63404ce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297c4ce7, SUC_NE_lexicon_tags_2 },
    { 0x77f84ce7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92de4ced, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ac4cf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41a84cf4, SUC_NE_lexicon_tags_7 },
    { 0x51624cf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc4cf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef44cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99804d01, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac64d04, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0004d06, SUC_NE_lexicon_tags_2 },
    { 0x721e4d07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36944d09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a4d0d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x50a44d0f, SUC_NE_lexicon_tags_229 },
    { 0x27584d10, SUC_NE_lexicon_tags_7 },
    { 0xe4c44d10, SUC_NE_lexicon_tags_7 },
    { 0xf08c4d10, SUC_NE_lexicon_tags_3 },
    { 0x56664d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81c4d1b, SUC_NE_lexicon_tags_2 },
    { 0xa0f44d1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8664d1f, SUC_NE_lexicon_tags_2 },
    { 0x49a84d1f, SUC_NE_lexicon_tags_2 },
    { 0x9044d21, SUC_NE_lexicon_tags_2 },
    { 0xfaf04d22, SUC_NE_lexicon_tags_2 },
    { 0xc58c4d21, SUC_NE_lexicon_tags_4 },
    { 0xa8584d20, SUC_NE_lexicon_tags_2 },
    { 0xdcb04d25, SUC_NE_lexicon_tags_2 },
    { 0x43304d26, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x714c4d28, SUC_NE_lexicon_tags_2 },
    { 0xdfea4d29, SUC_NE_lexicon_tags_2 },
    { 0x26ca4d29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x945c4d2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3c4d2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12a4d33, SUC_NE_lexicon_tags_0 },
    { 0x1a1a4d33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74944d36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bc4d39, SUC_NE_lexicon_tags_11 },
    { 0x2564d3a, SUC_NE_lexicon_tags_2 },
    { 0xf5604d3b, SUC_NE_lexicon_tags_7 },
    { 0x2b24d3c, SUC_NE_lexicon_tags_2 },
    { 0x6c384d39, SUC_NE_lexicon_tags_2 },
    { 0x8c064d3c, SUC_NE_lexicon_tags_2 },
    { 0xba9e4d3f, SUC_NE_lexicon_tags_2 },
    { 0x9a5a4d3e, SUC_NE_lexicon_tags_2 },
    { 0x62744d3b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b44d44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3e04d46, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x16c84d48, SUC_NE_lexicon_tags_2 },
    { 0x287e4d49, SUC_NE_lexicon_tags_2 },
    { 0x9d924d4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9a4d50, SUC_NE_lexicon_tags_2 },
    { 0x88164d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e4d55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c24d5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8a4d5c, SUC_NE_lexicon_tags_2 },
    { 0x3b004d5c, SUC_NE_lexicon_tags_2 },
    { 0x95464d5d, SUC_NE_lexicon_tags_4 },
    { 0x46284d5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ac64d61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10a4d65, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76204d6c, SUC_NE_lexicon_tags_2 },
    { 0xb4724d6c, SUC_NE_lexicon_tags_3 },
    { 0x72544d6e, SUC_NE_lexicon_tags_2 },
    { 0xf1ae4d6f, SUC_NE_lexicon_tags_19 },
    { 0x8d144d6d, SUC_NE_lexicon_tags_3 },
    { 0xb5b64d6f, SUC_NE_lexicon_tags_2 },
    { 0x26364d72, SUC_NE_lexicon_tags_2 },
    { 0xe4c84d71, SUC_NE_lexicon_tags_2 },
    { 0x7b2c4d6c, SUC_NE_lexicon_tags_2 },
    { 0x825a4d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb964d77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74284d7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ac4d80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2ac4d82, SUC_NE_lexicon_tags_17 },
    { 0x8f464d82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14644d85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab64d8b, SUC_NE_lexicon_tags_3 },
    { 0x63864d8c, SUC_NE_lexicon_tags_2 },
    { 0xa36e4d8c, SUC_NE_lexicon_tags_2 },
    { 0xc0884d8e, SUC_NE_lexicon_tags_2 },
    { 0x1a464d8f, SUC_NE_lexicon_tags_2 },
    { 0xf8204d8c, SUC_NE_lexicon_tags_2 },
    { 0x5f3c4d91, SUC_NE_lexicon_tags_2 },
    { 0x88904d8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8a4d95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8b24d97, SUC_NE_lexicon_tags_2 },
    { 0xc2c64d98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84d9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f884da1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f44da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e344da8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15ac4daa, SUC_NE_lexicon_tags_2 },
    { 0xcd144dab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x832e4dad, SUC_NE_lexicon_tags_2 },
    { 0x97ee4dad, SUC_NE_lexicon_tags_2 },
    { 0xc7ec4daf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34504db4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e0e4db6, SUC_NE_lexicon_tags_2 },
    { 0x2f324db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61444db9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf81e4dbb, SUC_NE_lexicon_tags_2 },
    { 0xcf804dbc, SUC_NE_lexicon_tags_2 },
    { 0xde424dbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef384dc2, SUC_NE_lexicon_tags_2 },
    { 0xe60e4dc2, SUC_NE_lexicon_tags_2 },
    { 0x9b524dc4, SUC_NE_lexicon_tags_2 },
    { 0xe3524dc5, SUC_NE_lexicon_tags_2 },
    { 0x26ea4dc6, SUC_NE_lexicon_tags_2 },
    { 0xb5f04dc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72ac4dc9, SUC_NE_lexicon_tags_3 },
    { 0x42b84dca, SUC_NE_lexicon_tags_2 },
    { 0x3d764dca, SUC_NE_lexicon_tags_2 },
    { 0x28c4dca, SUC_NE_lexicon_tags_2 },
    { 0x6afa4dcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef64dcf, SUC_NE_lexicon_tags_2 },
    { 0x756e4dcf, SUC_NE_lexicon_tags_2 },
    { 0xcafc4dd1, SUC_NE_lexicon_tags_2 },
    { 0xecea4dcf, SUC_NE_lexicon_tags_2 },
    { 0x87784dd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28e4dd5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb9ae4dd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc4ddc, SUC_NE_lexicon_tags_3 },
    { 0xb6804ddd, SUC_NE_lexicon_tags_2 },
    { 0x64844dde, SUC_NE_lexicon_tags_109 },
    { 0x15b64ddf, SUC_NE_lexicon_tags_2 },
    { 0xcee64ddf, SUC_NE_lexicon_tags_2 },
    { 0x51a84de1, SUC_NE_lexicon_tags_2 },
    { 0xaa604ddf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1ec4de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a4de8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b64df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6364df7, SUC_NE_lexicon_tags_2 },
    { 0xa75a4df7, SUC_NE_lexicon_tags_2 },
    { 0x74904df9, SUC_NE_lexicon_tags_2 },
    { 0xfe5a4df9, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x2f4a4dfc, SUC_NE_lexicon_tags_2 },
    { 0xe2e04dfd, SUC_NE_lexicon_tags_2 },
    { 0x3d064dfe, SUC_NE_lexicon_tags_2 },
    { 0x41da4dff, SUC_NE_lexicon_tags_2 },
    { 0xe4ee4e00, SUC_NE_lexicon_tags_2 },
    { 0xb53e4e01, SUC_NE_lexicon_tags_2 },
    { 0x81b44e02, SUC_NE_lexicon_tags_2 },
    { 0x94664e03, SUC_NE_lexicon_tags_2 },
    { 0xa24e04, SUC_NE_lexicon_tags_2 },
    { 0x86c84e03, SUC_NE_lexicon_tags_2 },
    { 0xbab64e06, SUC_NE_lexicon_tags_2 },
    { 0xad44e07, SUC_NE_lexicon_tags_2 },
    { 0xc244e07, SUC_NE_lexicon_tags_2 },
    { 0xdd444e09, SUC_NE_lexicon_tags_2 },
    { 0x3c224e09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34404e0e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22544e13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf984e16, SUC_NE_lexicon_tags_2 },
    { 0x64e64e16, SUC_NE_lexicon_tags_2 },
    { 0x53484e18, SUC_NE_lexicon_tags_2 },
    { 0x60364e19, SUC_NE_lexicon_tags_2 },
    { 0xf9bc4e17, SUC_NE_lexicon_tags_2 },
    { 0xd4644e1b, SUC_NE_lexicon_tags_2 },
    { 0x794e4e1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27c4e24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3e4e28, SUC_NE_lexicon_tags_2 },
    { 0xa92e4e29, SUC_NE_lexicon_tags_2 },
    { 0xc8a84e2a, SUC_NE_lexicon_tags_2 },
    { 0x70f84e2b, SUC_NE_lexicon_tags_2 },
    { 0x5f6e4e2c, SUC_NE_lexicon_tags_2 },
    { 0xeb184e2d, SUC_NE_lexicon_tags_2 },
    { 0x21844e29, SUC_NE_lexicon_tags_2 },
    { 0x72ae4e28, SUC_NE_lexicon_tags_4 },
    { 0xf3cc4e28, SUC_NE_lexicon_tags_2 },
    { 0x88c04e31, SUC_NE_lexicon_tags_2 },
    { 0x13504e32, SUC_NE_lexicon_tags_2 },
    { 0xe9ae4e32, SUC_NE_lexicon_tags_2 },
    { 0x62504e32, SUC_NE_lexicon_tags_3 },
    { 0x70064e33, SUC_NE_lexicon_tags_7 },
    { 0xd50e4e36, SUC_NE_lexicon_tags_4 },
    { 0x1db24e37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87ea4e39, SUC_NE_lexicon_tags_4 },
    { 0xf25c4e3a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x996a4e3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53684e3f, SUC_NE_lexicon_tags_2 },
    { 0xcb164e3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e764e43, SUC_NE_lexicon_tags_2 },
    { 0xbebc4e43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9a84e46, SUC_NE_lexicon_tags_2 },
    { 0x36d24e47, SUC_NE_lexicon_tags_2 },
    { 0xdb9e4e47, SUC_NE_lexicon_tags_13 },
    { 0xcfb84e46, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c64e4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x666a4e50, SUC_NE_lexicon_tags_30 },
    { 0xbd1e4e51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4924e55, SUC_NE_lexicon_tags_2 },
    { 0x68a64e56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55024e58, SUC_NE_lexicon_tags_2 },
    { 0x1684e59, SUC_NE_lexicon_tags_2 },
    { 0xf2944e58, SUC_NE_lexicon_tags_2 },
    { 0x33ba4e5b, SUC_NE_lexicon_tags_2 },
    { 0x21d04e58, SUC_NE_lexicon_tags_34 },
    { 0xf2d24e5b, SUC_NE_lexicon_tags_1 },
    { 0x28fc4e5e, SUC_NE_lexicon_tags_2 },
    { 0x1b6e4e5e, SUC_NE_lexicon_tags_2 },
    { 0x5e024e5c, SUC_NE_lexicon_tags_2 },
    { 0x25804e61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a144e65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3424e67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bc04e69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd76c4e6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50b64e6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b964e73, SUC_NE_lexicon_tags_2 },
    { 0x9d924e73, SUC_NE_lexicon_tags_2 },
    { 0x35f04e74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba84e78, SUC_NE_lexicon_tags_2 },
    { 0xd29c4e78, SUC_NE_lexicon_tags_2 },
    { 0x9eea4e78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e204e7d, SUC_NE_lexicon_tags_2 },
    { 0xe6ea4e7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bd64e80, SUC_NE_lexicon_tags_2 },
    { 0xc0144e81, SUC_NE_lexicon_tags_7 },
    { 0xd3724e82, SUC_NE_lexicon_tags_2 },
    { 0xf3844e81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c84e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x128c4e8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c04e8e, SUC_NE_lexicon_tags_2 },
    { 0x20fe4e8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33f64e91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea524e96, SUC_NE_lexicon_tags_2 },
    { 0xc41a4e97, SUC_NE_lexicon_tags_2 },
    { 0xa9864e97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6824e9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f3a4e9c, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18564ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a104ea5, SUC_NE_lexicon_tags_2 },
    { 0x8a4a4ea6, SUC_NE_lexicon_tags_2 },
    { 0xdf404ea7, SUC_NE_lexicon_tags_2 },
    { 0x626a4ea7, SUC_NE_lexicon_tags_2 },
    { 0xedf64ea8, SUC_NE_lexicon_tags_2 },
    { 0xd7e44ea8, SUC_NE_lexicon_tags_2 },
    { 0x49064ea9, SUC_NE_lexicon_tags_2 },
    { 0x5d204eac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10824eae, SUC_NE_lexicon_tags_24 },
    { 0x30964eaf, SUC_NE_lexicon_tags_2 },
    { 0x50ca4eaf, SUC_NE_lexicon_tags_2 },
    { 0x27864eb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f604eb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9c4eb7, SUC_NE_lexicon_tags_2 },
    { 0xb7b84eb8, SUC_NE_lexicon_tags_11 },
    { 0xb8a4eb9, SUC_NE_lexicon_tags_2 },
    { 0x3d404eb9, SUC_NE_lexicon_tags_2 },
    { 0x58b64ebb, SUC_NE_lexicon_tags_2 },
    { 0x8f724ebb, SUC_NE_lexicon_tags_2 },
    { 0x90e84ebc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30204ebf, SUC_NE_lexicon_tags_2 },
    { 0x374a4ec0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d24ec3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88004ec6, SUC_NE_lexicon_tags_2 },
    { 0xc3b24ec7, SUC_NE_lexicon_tags_2 },
    { 0x17024ec8, SUC_NE_lexicon_tags_2 },
    { 0x387a4ec8, SUC_NE_lexicon_tags_2 },
    { 0x700a4eca, SUC_NE_lexicon_tags_7 },
    { 0x8f564ecb, SUC_NE_lexicon_tags_2 },
    { 0xdbca4ecc, SUC_NE_lexicon_tags_2 },
    { 0x1e1e4ecd, SUC_NE_lexicon_tags_2 },
    { 0x3dac4ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeba4ed0, SUC_NE_lexicon_tags_5 },
    { 0x7f5a4ed1, SUC_NE_lexicon_tags_2 },
    { 0xbfba4ed2, SUC_NE_lexicon_tags_2 },
    { 0x577c4ed0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a464ed5, SUC_NE_lexicon_tags_2 },
    { 0x39304ed5, SUC_NE_lexicon_tags_2 },
    { 0xc5de4ed7, SUC_NE_lexicon_tags_7 },
    { 0xb3824ed8, SUC_NE_lexicon_tags_2 },
    { 0x4bee4ed9, SUC_NE_lexicon_tags_2 },
    { 0x1c2a4ed9, SUC_NE_lexicon_tags_2 },
    { 0x43ce4eda, SUC_NE_lexicon_tags_2 },
    { 0x757c4edc, SUC_NE_lexicon_tags_2 },
    { 0xd9da4edd, SUC_NE_lexicon_tags_2 },
    { 0xb94a4edc, SUC_NE_lexicon_tags_2 },
    { 0x9ae24edc, SUC_NE_lexicon_tags_2 },
    { 0xb02a4edf, SUC_NE_lexicon_tags_2 },
    { 0xe4664ee0, SUC_NE_lexicon_tags_4 },
    { 0x59e24ee2, SUC_NE_lexicon_tags_2 },
    { 0x146c4ee3, SUC_NE_lexicon_tags_2 },
    { 0x3b7a4ee4, SUC_NE_lexicon_tags_2 },
    { 0xd5464ee1, SUC_NE_lexicon_tags_2 },
    { 0x46764ee6, SUC_NE_lexicon_tags_2 },
    { 0x36d04ee7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98a4eea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1a4eed, SUC_NE_lexicon_tags_2 },
    { 0x6ffe4eee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29c4ef7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x106c4ef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd84efc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b84f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8504f05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f384f07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x107c4f09, SUC_NE_lexicon_tags_1 },
    { 0x800a4f0a, SUC_NE_lexicon_tags_0 },
    { 0xb7924f0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0a4f0e, SUC_NE_lexicon_tags_2 },
    { 0xc5a64f0f, SUC_NE_lexicon_tags_2 },
    { 0x8a1e4f0f, SUC_NE_lexicon_tags_2 },
    { 0x65064f11, SUC_NE_lexicon_tags_3 },
    { 0x4cc84f12, SUC_NE_lexicon_tags_2 },
    { 0xdde4f11, SUC_NE_lexicon_tags_2 },
    { 0x4fe44f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7424f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f84f1d, SUC_NE_lexicon_tags_2 },
    { 0x61e64f1e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897a4f22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35904f26, SUC_NE_lexicon_tags_2 },
    { 0xd8924f27, SUC_NE_lexicon_tags_2 },
    { 0x659e4f27, SUC_NE_lexicon_tags_2 },
    { 0xd0f84f26, SUC_NE_lexicon_tags_2 },
    { 0x68b44f2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72f44f2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e844f2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddce4f32, SUC_NE_lexicon_tags_2 },
    { 0x6044f32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e184f36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d24f39, SUC_NE_lexicon_tags_4 },
    { 0x173e4f3a, SUC_NE_lexicon_tags_7 },
    { 0x3d9c4f39, SUC_NE_lexicon_tags_2 },
    { 0x3eb84f3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a524f3f, SUC_NE_lexicon_tags_2 },
    { 0x34ca4f40, SUC_NE_lexicon_tags_2 },
    { 0xe8b24f3f, SUC_NE_lexicon_tags_2 },
    { 0x130a4f42, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece4f46, SUC_NE_lexicon_tags_2 },
    { 0xf7104f46, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x5ed84f49, SUC_NE_lexicon_tags_2 },
    { 0x377c4f4a, SUC_NE_lexicon_tags_2 },
    { 0x72c4f49, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x36184f4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df24f50, SUC_NE_lexicon_tags_2 },
    { 0x3d04f51, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aa4f55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x213a4f57, SUC_NE_lexicon_tags_2 },
    { 0xb8e84f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a4f5b, SUC_NE_lexicon_tags_7 },
    { 0xa62a4f5c, SUC_NE_lexicon_tags_18 },
    { 0x52644f5d, SUC_NE_lexicon_tags_2 },
    { 0xa7e24f5e, SUC_NE_lexicon_tags_40 },
    { 0x33844f5d, SUC_NE_lexicon_tags_2 },
    { 0x240a4f60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf62a4f62, SUC_NE_lexicon_tags_2 },
    { 0xd00e4f63, SUC_NE_lexicon_tags_2 },
    { 0x25564f63, SUC_NE_lexicon_tags_2 },
    { 0xbcc64f65, SUC_NE_lexicon_tags_2 },
    { 0x6c284f65, SUC_NE_lexicon_tags_2 },
    { 0x3ad84f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9704f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b164f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x687a4f72, SUC_NE_lexicon_tags_0 },
    { 0x1944f72, SUC_NE_lexicon_tags_2 },
    { 0x4bb64f74, SUC_NE_lexicon_tags_4 },
    { 0x5e9a4f75, SUC_NE_lexicon_tags_2 },
    { 0xc4264f73, SUC_NE_lexicon_tags_2 },
    { 0x6eee4f77, SUC_NE_lexicon_tags_19 },
    { 0x644a4f72, SUC_NE_lexicon_tags_187 },
    { 0, NULL },
    { 0x19d24f7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957e4f81, SUC_NE_lexicon_tags_2 },
    { 0x47384f82, SUC_NE_lexicon_tags_2 },
    { 0xeaca4f83, SUC_NE_lexicon_tags_2 },
    { 0xe0484f83, SUC_NE_lexicon_tags_2 },
    { 0xbad44f84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59884f89, SUC_NE_lexicon_tags_2 },
    { 0x6c224f89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aa4f8e, SUC_NE_lexicon_tags_2 },
    { 0xd01a4f8e, SUC_NE_lexicon_tags_2 },
    { 0xa8c64f90, SUC_NE_lexicon_tags_2 },
    { 0x41144f91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6fa4f93, SUC_NE_lexicon_tags_2 },
    { 0x6d864f94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b0a4f96, SUC_NE_lexicon_tags_2 },
    { 0x36044f97, SUC_NE_lexicon_tags_2 },
    { 0xa7a24f98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868e4fa0, SUC_NE_lexicon_tags_2 },
    { 0x684fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9144fa4, SUC_NE_lexicon_tags_2 },
    { 0x51b04fa5, SUC_NE_lexicon_tags_2 },
    { 0xcf5e4fa6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf464fa8, SUC_NE_lexicon_tags_2 },
    { 0x8b384fa9, SUC_NE_lexicon_tags_2 },
    { 0x268a4faa, SUC_NE_lexicon_tags_2 },
    { 0x2f24fab, SUC_NE_lexicon_tags_2 },
    { 0x32364faa, SUC_NE_lexicon_tags_18 },
    { 0xe7304fad, SUC_NE_lexicon_tags_2 },
    { 0x32fa4fae, SUC_NE_lexicon_tags_2 },
    { 0x3b0a4faf, SUC_NE_lexicon_tags_2 },
    { 0x76264fb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f04fb8, SUC_NE_lexicon_tags_2 },
    { 0x6c304fb8, SUC_NE_lexicon_tags_2 },
    { 0xf0804fba, SUC_NE_lexicon_tags_2 },
    { 0x7c344fbb, SUC_NE_lexicon_tags_2 },
    { 0xeb1e4fbc, SUC_NE_lexicon_tags_2 },
    { 0x2f584fbd, SUC_NE_lexicon_tags_2 },
    { 0x7dd44fbe, SUC_NE_lexicon_tags_2 },
    { 0xe36e4fbf, SUC_NE_lexicon_tags_2 },
    { 0x66f04fc0, SUC_NE_lexicon_tags_2 },
    { 0x45e04fc1, SUC_NE_lexicon_tags_2 },
    { 0x40424fc2, SUC_NE_lexicon_tags_2 },
    { 0x26124fb8, SUC_NE_lexicon_tags_2 },
    { 0xf4864fbc, SUC_NE_lexicon_tags_2 },
    { 0x3f5e4fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c64fc8, SUC_NE_lexicon_tags_2 },
    { 0x64564fc8, SUC_NE_lexicon_tags_2 },
    { 0xff264fc8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcae4fcd, SUC_NE_lexicon_tags_2 },
    { 0xf4e64fce, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcc4fd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da44fd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x555e4fd6, SUC_NE_lexicon_tags_7 },
    { 0xf23c4fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13064fdb, SUC_NE_lexicon_tags_2 },
    { 0x5a4fdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47664fdf, SUC_NE_lexicon_tags_2 },
    { 0xec4e4fdf, SUC_NE_lexicon_tags_2 },
    { 0xc5e64fe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x228a4fe4, SUC_NE_lexicon_tags_2 },
    { 0x97dc4fe5, SUC_NE_lexicon_tags_2 },
    { 0x6e284fe5, SUC_NE_lexicon_tags_2 },
    { 0xabee4fe5, SUC_NE_lexicon_tags_2 },
    { 0x39644fe8, SUC_NE_lexicon_tags_2 },
    { 0x341e4fe8, SUC_NE_lexicon_tags_2 },
    { 0x6cf84fea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12964fee, SUC_NE_lexicon_tags_2 },
    { 0x2d724fee, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f3e4ff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b304ff3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfa4fff, SUC_NE_lexicon_tags_4 },
    { 0x30d85000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf085002, SUC_NE_lexicon_tags_2 },
    { 0xf2545002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa25009, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab26500b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad145011, SUC_NE_lexicon_tags_2 },
    { 0xeedc5012, SUC_NE_lexicon_tags_2 },
    { 0xdaa45013, SUC_NE_lexicon_tags_0 },
    { 0x1d1a5013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf85019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13a5020, SUC_NE_lexicon_tags_2 },
    { 0x8f485021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e05024, SUC_NE_lexicon_tags_2 },
    { 0x11445025, SUC_NE_lexicon_tags_2 },
    { 0xcf265026, SUC_NE_lexicon_tags_2 },
    { 0xbde45026, SUC_NE_lexicon_tags_2 },
    { 0x2dce5026, SUC_NE_lexicon_tags_2 },
    { 0x618e5025, SUC_NE_lexicon_tags_1 },
    { 0xe9ec5026, SUC_NE_lexicon_tags_2 },
    { 0x73d65024, SUC_NE_lexicon_tags_2 },
    { 0xad965027, SUC_NE_lexicon_tags_2 },
    { 0x392a502d, SUC_NE_lexicon_tags_2 },
    { 0x22dc502e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75645030, SUC_NE_lexicon_tags_2 },
    { 0x2eb25030, SUC_NE_lexicon_tags_2 },
    { 0x3c645030, SUC_NE_lexicon_tags_2 },
    { 0xec4c5031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c5035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e5039, SUC_NE_lexicon_tags_2 },
    { 0x19f85039, SUC_NE_lexicon_tags_4 },
    { 0x7a84503b, SUC_NE_lexicon_tags_7 },
    { 0x123a503c, SUC_NE_lexicon_tags_2 },
    { 0x4540503d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6e5045, SUC_NE_lexicon_tags_2 },
    { 0xde705045, SUC_NE_lexicon_tags_2 },
    { 0xe6425047, SUC_NE_lexicon_tags_2 },
    { 0x76a45046, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d745050, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fa5055, SUC_NE_lexicon_tags_98 },
    { 0xc2545056, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a5059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x683e505c, SUC_NE_lexicon_tags_2 },
    { 0x12de505c, SUC_NE_lexicon_tags_2 },
    { 0x2816505e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5525062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb265064, SUC_NE_lexicon_tags_2 },
    { 0xbcd05064, SUC_NE_lexicon_tags_2 },
    { 0x88485066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f485069, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb724506f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5c5073, SUC_NE_lexicon_tags_2 },
    { 0xfa4c5073, SUC_NE_lexicon_tags_2 },
    { 0xaac45075, SUC_NE_lexicon_tags_2 },
    { 0x1b725076, SUC_NE_lexicon_tags_2 },
    { 0x3a545074, SUC_NE_lexicon_tags_17 },
    { 0xa97a5073, SUC_NE_lexicon_tags_2 },
    { 0xd10e5077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3242507d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1305080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0ce5082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8945084, SUC_NE_lexicon_tags_7 },
    { 0xb8065084, SUC_NE_lexicon_tags_1 },
    { 0x814c5086, SUC_NE_lexicon_tags_2 },
    { 0xd1b45087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa606508c, SUC_NE_lexicon_tags_2 },
    { 0xc168508c, SUC_NE_lexicon_tags_2 },
    { 0xf22c508d, SUC_NE_lexicon_tags_2 },
    { 0xd10c508f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b425095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4d05097, SUC_NE_lexicon_tags_2 },
    { 0xeee5098, SUC_NE_lexicon_tags_2 },
    { 0x47325097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2a50a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec850a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd30e50a6, SUC_NE_lexicon_tags_2 },
    { 0x487e50a7, SUC_NE_lexicon_tags_2 },
    { 0xc47a50a7, SUC_NE_lexicon_tags_7 },
    { 0xfacc50a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05650ac, SUC_NE_lexicon_tags_3 },
    { 0x2d5a50ad, SUC_NE_lexicon_tags_2 },
    { 0x30e050ad, SUC_NE_lexicon_tags_2 },
    { 0x89f850af, SUC_NE_lexicon_tags_2 },
    { 0x660a50b0, SUC_NE_lexicon_tags_2 },
    { 0xb55450ae, SUC_NE_lexicon_tags_403 },
    { 0x16f650b2, SUC_NE_lexicon_tags_2 },
    { 0x467650b1, SUC_NE_lexicon_tags_2 },
    { 0xf7e050b4, SUC_NE_lexicon_tags_2 },
    { 0xe0aa50b4, SUC_NE_lexicon_tags_1 },
    { 0x907450b5, SUC_NE_lexicon_tags_2 },
    { 0xa07650b7, SUC_NE_lexicon_tags_2 },
    { 0x3b9050b8, SUC_NE_lexicon_tags_2 },
    { 0x5e6650b2, SUC_NE_lexicon_tags_2 },
    { 0xb87850b8, SUC_NE_lexicon_tags_2 },
    { 0x1b3450bb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd650bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499850c5, SUC_NE_lexicon_tags_2 },
    { 0x615850c6, SUC_NE_lexicon_tags_2 },
    { 0xeebe50c7, SUC_NE_lexicon_tags_2 },
    { 0x9e9e50c8, SUC_NE_lexicon_tags_2 },
    { 0xadd450c9, SUC_NE_lexicon_tags_2 },
    { 0x7c9450c9, SUC_NE_lexicon_tags_2 },
    { 0xf2b650c9, SUC_NE_lexicon_tags_2 },
    { 0x66dc50c9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfa50cf, SUC_NE_lexicon_tags_50 },
    { 0x311450d0, SUC_NE_lexicon_tags_2 },
    { 0x5d6c50d1, SUC_NE_lexicon_tags_4 },
    { 0xb01650d1, SUC_NE_lexicon_tags_2 },
    { 0x8f5c50d0, SUC_NE_lexicon_tags_2 },
    { 0x3b9e50d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x773450d7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467850db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc850de, SUC_NE_lexicon_tags_2 },
    { 0x650c50de, SUC_NE_lexicon_tags_130 },
    { 0xd51e50e0, SUC_NE_lexicon_tags_2 },
    { 0x775450de, SUC_NE_lexicon_tags_2 },
    { 0x826e50e2, SUC_NE_lexicon_tags_2 },
    { 0x959c50e0, SUC_NE_lexicon_tags_2 },
    { 0x99250de, SUC_NE_lexicon_tags_2 },
    { 0x899650e5, SUC_NE_lexicon_tags_2 },
    { 0xd17a50e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06c50e9, SUC_NE_lexicon_tags_2 },
    { 0x86c450ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x159c50ec, SUC_NE_lexicon_tags_2 },
    { 0x291250ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ba50f0, SUC_NE_lexicon_tags_2 },
    { 0x973e50f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x184650f3, SUC_NE_lexicon_tags_2 },
    { 0xd88e50f4, SUC_NE_lexicon_tags_2 },
    { 0xdb3450f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa250f9, SUC_NE_lexicon_tags_2 },
    { 0x9b2e50f9, SUC_NE_lexicon_tags_2 },
    { 0x29b850f9, SUC_NE_lexicon_tags_2 },
    { 0x23aa50fc, SUC_NE_lexicon_tags_2 },
    { 0xb1a050f9, SUC_NE_lexicon_tags_2 },
    { 0x6de250fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a325100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8e65102, SUC_NE_lexicon_tags_4 },
    { 0xc4545103, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe69e5105, SUC_NE_lexicon_tags_25 },
    { 0xb4185106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb87e5108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dc510b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4225111, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c25114, SUC_NE_lexicon_tags_2 },
    { 0x52945115, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cb05117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a2511a, SUC_NE_lexicon_tags_2 },
    { 0xd5f6511a, SUC_NE_lexicon_tags_2 },
    { 0xfc1a511a, SUC_NE_lexicon_tags_2 },
    { 0x782a511b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a985121, SUC_NE_lexicon_tags_2 },
    { 0xbcca5121, SUC_NE_lexicon_tags_1 },
    { 0x1a9c5123, SUC_NE_lexicon_tags_2 },
    { 0xe1ac5124, SUC_NE_lexicon_tags_2 },
    { 0xcb2c5125, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc125127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b88512a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3e6512c, SUC_NE_lexicon_tags_2 },
    { 0xb1f4512c, SUC_NE_lexicon_tags_2 },
    { 0x3c7a512e, SUC_NE_lexicon_tags_2 },
    { 0x8b4c512e, SUC_NE_lexicon_tags_2 },
    { 0x9f805130, SUC_NE_lexicon_tags_2 },
    { 0x455e5130, SUC_NE_lexicon_tags_2 },
    { 0xb685132, SUC_NE_lexicon_tags_2 },
    { 0x69565130, SUC_NE_lexicon_tags_51 },
    { 0x2a945134, SUC_NE_lexicon_tags_2 },
    { 0xf2a5133, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x756e5138, SUC_NE_lexicon_tags_2 },
    { 0x130e5139, SUC_NE_lexicon_tags_2 },
    { 0xc42513a, SUC_NE_lexicon_tags_2 },
    { 0xd7b65139, SUC_NE_lexicon_tags_2 },
    { 0xdc26513c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0105142, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62185145, SUC_NE_lexicon_tags_2 },
    { 0xf13a5146, SUC_NE_lexicon_tags_2 },
    { 0x67ca5145, SUC_NE_lexicon_tags_2 },
    { 0xdcae5147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6514b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae514e, SUC_NE_lexicon_tags_2 },
    { 0xe006514f, SUC_NE_lexicon_tags_2 },
    { 0xb9385150, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f765154, SUC_NE_lexicon_tags_3 },
    { 0xea9a5154, SUC_NE_lexicon_tags_2 },
    { 0x116c5156, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c56515a, SUC_NE_lexicon_tags_2 },
    { 0x4a26515a, SUC_NE_lexicon_tags_2 },
    { 0x4498515c, SUC_NE_lexicon_tags_2 },
    { 0x8d96515d, SUC_NE_lexicon_tags_2 },
    { 0xa06c515e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f485161, SUC_NE_lexicon_tags_2 },
    { 0xa3765162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89165165, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f265167, SUC_NE_lexicon_tags_2 },
    { 0xaaf05167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc516f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47465173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x299c5176, SUC_NE_lexicon_tags_2 },
    { 0xda4e5177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73a517d, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5625182, SUC_NE_lexicon_tags_2 },
    { 0x863c5183, SUC_NE_lexicon_tags_2 },
    { 0xe7b85183, SUC_NE_lexicon_tags_2 },
    { 0x85345185, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e4518a, SUC_NE_lexicon_tags_2 },
    { 0xe4f6518a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc30518d, SUC_NE_lexicon_tags_2 },
    { 0xa0e4518e, SUC_NE_lexicon_tags_2 },
    { 0x43b2518f, SUC_NE_lexicon_tags_2 },
    { 0x4bda518d, SUC_NE_lexicon_tags_2 },
    { 0x8e445191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabd25193, SUC_NE_lexicon_tags_2 },
    { 0x157e5194, SUC_NE_lexicon_tags_2 },
    { 0xbee65194, SUC_NE_lexicon_tags_2 },
    { 0xd4225193, SUC_NE_lexicon_tags_2 },
    { 0x36385197, SUC_NE_lexicon_tags_15 },
    { 0xe2945196, SUC_NE_lexicon_tags_2 },
    { 0xa5b05199, SUC_NE_lexicon_tags_2 },
    { 0xa04519a, SUC_NE_lexicon_tags_36 },
    { 0xf2045196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77c8519d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x866e519f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36dc51a1, SUC_NE_lexicon_tags_3 },
    { 0x48d851a2, SUC_NE_lexicon_tags_2 },
    { 0x9bee51a2, SUC_NE_lexicon_tags_2 },
    { 0x6b4851a4, SUC_NE_lexicon_tags_186 },
    { 0x40ea51a1, SUC_NE_lexicon_tags_2 },
    { 0x1f3451a6, SUC_NE_lexicon_tags_2 },
    { 0xd8ba51a6, SUC_NE_lexicon_tags_2 },
    { 0xacf251a7, SUC_NE_lexicon_tags_2 },
    { 0x78ac51a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ea51b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc29e51b3, SUC_NE_lexicon_tags_2 },
    { 0x3fc651b4, SUC_NE_lexicon_tags_2 },
    { 0x633a51b3, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xff6c51b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f651b9, SUC_NE_lexicon_tags_2 },
    { 0xf6e51ba, SUC_NE_lexicon_tags_18 },
    { 0x2fca51bb, SUC_NE_lexicon_tags_2 },
    { 0x528451bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x685251be, SUC_NE_lexicon_tags_2 },
    { 0x1ab251bf, SUC_NE_lexicon_tags_2 },
    { 0x62d051be, SUC_NE_lexicon_tags_2 },
    { 0x19e851c1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3451c4, SUC_NE_lexicon_tags_40 },
    { 0x387851c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4851d7, SUC_NE_lexicon_tags_2 },
    { 0x601451d8, SUC_NE_lexicon_tags_2 },
    { 0xff651d7, SUC_NE_lexicon_tags_7 },
    { 0xa49c51da, SUC_NE_lexicon_tags_2 },
    { 0x704651db, SUC_NE_lexicon_tags_2 },
    { 0x4c9e51d9, SUC_NE_lexicon_tags_2 },
    { 0x278051dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x339051df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3051e3, SUC_NE_lexicon_tags_2 },
    { 0xd91251e4, SUC_NE_lexicon_tags_2 },
    { 0xeac51e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840c51ed, SUC_NE_lexicon_tags_2 },
    { 0x297e51ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec051f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466a51fa, SUC_NE_lexicon_tags_2 },
    { 0xf7b851fa, SUC_NE_lexicon_tags_2 },
    { 0xa57651fa, SUC_NE_lexicon_tags_2 },
    { 0x8a0e51fd, SUC_NE_lexicon_tags_2 },
    { 0xfe6651fe, SUC_NE_lexicon_tags_2 },
    { 0xa15e51fe, SUC_NE_lexicon_tags_2 },
    { 0xd79651fa, SUC_NE_lexicon_tags_2 },
    { 0x737051fd, SUC_NE_lexicon_tags_2 },
    { 0x53e451fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4545204, SUC_NE_lexicon_tags_2 },
    { 0x2aaa5205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3725207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94365209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6108520c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1648520e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77fc5210, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4d05212, SUC_NE_lexicon_tags_2 },
    { 0x90205213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b04521c, SUC_NE_lexicon_tags_2 },
    { 0xa42521d, SUC_NE_lexicon_tags_2 },
    { 0x3cc521e, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8365222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8945224, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6a5227, SUC_NE_lexicon_tags_2 },
    { 0x9dc85228, SUC_NE_lexicon_tags_24 },
    { 0xaf625229, SUC_NE_lexicon_tags_2 },
    { 0x4006522a, SUC_NE_lexicon_tags_2 },
    { 0x7ec6522b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28ac522d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0e5233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe0523e, SUC_NE_lexicon_tags_2 },
    { 0x958c523f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc1c5241, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e5247, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e45250, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x188a5252, SUC_NE_lexicon_tags_2 },
    { 0x76bc5253, SUC_NE_lexicon_tags_2 },
    { 0x3f005253, SUC_NE_lexicon_tags_27 },
    { 0xc88a5254, SUC_NE_lexicon_tags_2 },
    { 0x8ee05256, SUC_NE_lexicon_tags_2 },
    { 0x449c5257, SUC_NE_lexicon_tags_2 },
    { 0x40d25258, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc525b, SUC_NE_lexicon_tags_2 },
    { 0x8112525c, SUC_NE_lexicon_tags_2 },
    { 0x12a6525c, SUC_NE_lexicon_tags_0 },
    { 0x936a525e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb03a5260, SUC_NE_lexicon_tags_2 },
    { 0xcdea5261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b205266, SUC_NE_lexicon_tags_4 },
    { 0xee8a5267, SUC_NE_lexicon_tags_2 },
    { 0xdb6e5268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb880526c, SUC_NE_lexicon_tags_2 },
    { 0x2c32526c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2685272, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x253e5274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x899a5276, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd204527a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779c5281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f105286, SUC_NE_lexicon_tags_2 },
    { 0x92965287, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda385289, SUC_NE_lexicon_tags_2 },
    { 0xaf90528a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9e50528c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fae528f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d565292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b05295, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa65298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x527e529a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf252a7, SUC_NE_lexicon_tags_4 },
    { 0xe11a52a8, SUC_NE_lexicon_tags_2 },
    { 0xc61052a9, SUC_NE_lexicon_tags_2 },
    { 0xa73a52aa, SUC_NE_lexicon_tags_2 },
    { 0xff0e52a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x270052ad, SUC_NE_lexicon_tags_2 },
    { 0xd72452ae, SUC_NE_lexicon_tags_2 },
    { 0x917052ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1a52b2, SUC_NE_lexicon_tags_2 },
    { 0x92b652b3, SUC_NE_lexicon_tags_4 },
    { 0x590252b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dc52bc, SUC_NE_lexicon_tags_2 },
    { 0xcc3652bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3652c4, SUC_NE_lexicon_tags_2 },
    { 0xfe2a52c4, SUC_NE_lexicon_tags_225 },
    { 0, NULL },
    { 0, NULL },
    { 0x292052c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa39a52ca, SUC_NE_lexicon_tags_7 },
    { 0xc17452cb, SUC_NE_lexicon_tags_2 },
    { 0x3c4252cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe3e52ce, SUC_NE_lexicon_tags_2 },
    { 0xac1e52cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f452d2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978a52dc, SUC_NE_lexicon_tags_2 },
    { 0x363a52dc, SUC_NE_lexicon_tags_2 },
    { 0xb7e452dd, SUC_NE_lexicon_tags_2 },
    { 0x50452dd, SUC_NE_lexicon_tags_2 },
    { 0xa17052dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da452e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x276852e7, SUC_NE_lexicon_tags_2 },
    { 0x9eec52e7, SUC_NE_lexicon_tags_2 },
    { 0xce0252e7, SUC_NE_lexicon_tags_2 },
    { 0x7a3252e8, SUC_NE_lexicon_tags_2 },
    { 0x435252eb, SUC_NE_lexicon_tags_2 },
    { 0xdf4a52ec, SUC_NE_lexicon_tags_2 },
    { 0x22852ed, SUC_NE_lexicon_tags_2 },
    { 0x8f0052ed, SUC_NE_lexicon_tags_215 },
    { 0x278a52ec, SUC_NE_lexicon_tags_2 },
    { 0x330e52ee, SUC_NE_lexicon_tags_2 },
    { 0x34bc52f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51ca52f3, SUC_NE_lexicon_tags_2 },
    { 0xa7e52f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x676052f6, SUC_NE_lexicon_tags_2 },
    { 0x2e2052f7, SUC_NE_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d652fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b052fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cfa5300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f8530b, SUC_NE_lexicon_tags_2 },
    { 0x5c10530c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe840530f, SUC_NE_lexicon_tags_2 },
    { 0x13cc5310, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f45315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57585319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebc531c, SUC_NE_lexicon_tags_2 },
    { 0x4d28531d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d05320, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5445322, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xebaa5324, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375e5329, SUC_NE_lexicon_tags_4 },
    { 0x78bc532a, SUC_NE_lexicon_tags_2 },
    { 0xd6da532b, SUC_NE_lexicon_tags_2 },
    { 0xa950532a, SUC_NE_lexicon_tags_2 },
    { 0x8376532b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaea2532f, SUC_NE_lexicon_tags_2 },
    { 0x30045330, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f0c5332, SUC_NE_lexicon_tags_2 },
    { 0xd26e5333, SUC_NE_lexicon_tags_2 },
    { 0x231e5334, SUC_NE_lexicon_tags_2 },
    { 0x6cde5334, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aa533a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x197e533d, SUC_NE_lexicon_tags_25 },
    { 0x8018533d, SUC_NE_lexicon_tags_2 },
    { 0x6d6e533f, SUC_NE_lexicon_tags_1 },
    { 0xcf48533f, SUC_NE_lexicon_tags_2 },
    { 0x29705340, SUC_NE_lexicon_tags_2 },
    { 0x920c5340, SUC_NE_lexicon_tags_1 },
    { 0x9dd25340, SUC_NE_lexicon_tags_2 },
    { 0xa3f25344, SUC_NE_lexicon_tags_2 },
    { 0x11c85345, SUC_NE_lexicon_tags_2 },
    { 0x8db85345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17aa5348, SUC_NE_lexicon_tags_2 },
    { 0x64025349, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed2534c, SUC_NE_lexicon_tags_2 },
    { 0xe4a0534c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38a8534f, SUC_NE_lexicon_tags_2 },
    { 0x8e045350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac485354, SUC_NE_lexicon_tags_4 },
    { 0x3e3c5354, SUC_NE_lexicon_tags_2 },
    { 0x9d9a5354, SUC_NE_lexicon_tags_2 },
    { 0xc17a5355, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2520535a, SUC_NE_lexicon_tags_2 },
    { 0xd574535b, SUC_NE_lexicon_tags_3 },
    { 0x3588535a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f4535f, SUC_NE_lexicon_tags_2 },
    { 0x9b705360, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4525362, SUC_NE_lexicon_tags_2 },
    { 0x89b65363, SUC_NE_lexicon_tags_2 },
    { 0x1b385364, SUC_NE_lexicon_tags_2 },
    { 0xd5e5365, SUC_NE_lexicon_tags_2 },
    { 0xa8185366, SUC_NE_lexicon_tags_30 },
    { 0xeefc5366, SUC_NE_lexicon_tags_2 },
    { 0xe0de5366, SUC_NE_lexicon_tags_2 },
    { 0xa5e85369, SUC_NE_lexicon_tags_2 },
    { 0x8414536a, SUC_NE_lexicon_tags_3 },
    { 0xe5825368, SUC_NE_lexicon_tags_2 },
    { 0xb5145368, SUC_NE_lexicon_tags_2 },
    { 0x1c36536d, SUC_NE_lexicon_tags_17 },
    { 0xf834536e, SUC_NE_lexicon_tags_2 },
    { 0x7304536f, SUC_NE_lexicon_tags_2 },
    { 0x11dc536d, SUC_NE_lexicon_tags_2 },
    { 0x7385371, SUC_NE_lexicon_tags_2 },
    { 0x29305372, SUC_NE_lexicon_tags_3 },
    { 0xd90a5373, SUC_NE_lexicon_tags_2 },
    { 0x4d745373, SUC_NE_lexicon_tags_19 },
    { 0xb41c5368, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd432537a, SUC_NE_lexicon_tags_2 },
    { 0xa85c537b, SUC_NE_lexicon_tags_2 },
    { 0x34e537b, SUC_NE_lexicon_tags_4 },
    { 0x118c537b, SUC_NE_lexicon_tags_2 },
    { 0xf728537d, SUC_NE_lexicon_tags_2 },
    { 0x93f0537e, SUC_NE_lexicon_tags_3 },
    { 0xb27c5380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8be5383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26605387, SUC_NE_lexicon_tags_2 },
    { 0x3c7e5387, SUC_NE_lexicon_tags_2 },
    { 0x406c5388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac0538c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1225390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1ea5392, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x70185395, SUC_NE_lexicon_tags_4 },
    { 0x306e5396, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481c539c, SUC_NE_lexicon_tags_2 },
    { 0xde3c539c, SUC_NE_lexicon_tags_2 },
    { 0x1cd8539c, SUC_NE_lexicon_tags_2 },
    { 0x5a18539c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xef4c53a1, SUC_NE_lexicon_tags_2 },
    { 0x22653a1, SUC_NE_lexicon_tags_2 },
    { 0x96aa53a3, SUC_NE_lexicon_tags_2 },
    { 0xf7cc53a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4f253a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6053af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e853b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7e53b9, SUC_NE_lexicon_tags_2 },
    { 0xda7c53ba, SUC_NE_lexicon_tags_7 },
    { 0xc66053bb, SUC_NE_lexicon_tags_2 },
    { 0x69cc53bc, SUC_NE_lexicon_tags_2 },
    { 0x7ef653bd, SUC_NE_lexicon_tags_2 },
    { 0x5cbe53bb, SUC_NE_lexicon_tags_2 },
    { 0x829a53bf, SUC_NE_lexicon_tags_2 },
    { 0xe45c53c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b5653c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1a53c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a2453c7, SUC_NE_lexicon_tags_2 },
    { 0xef5c53c8, SUC_NE_lexicon_tags_74 },
    { 0x533e53c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ee53cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x508e53cf, SUC_NE_lexicon_tags_2 },
    { 0xcaaa53cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5a853d2, SUC_NE_lexicon_tags_2 },
    { 0x9a9a53d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f053d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc6853d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83053dd, SUC_NE_lexicon_tags_2 },
    { 0x5a2853dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f053e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x720653e4, SUC_NE_lexicon_tags_2 },
    { 0xafe853e4, SUC_NE_lexicon_tags_2 },
    { 0xfa6053e6, SUC_NE_lexicon_tags_2 },
    { 0xcc0653e7, SUC_NE_lexicon_tags_2 },
    { 0x7b0a53e8, SUC_NE_lexicon_tags_2 },
    { 0xf9053e8, SUC_NE_lexicon_tags_2 },
    { 0x1c4453e5, SUC_NE_lexicon_tags_7 },
    { 0xdfe53e9, SUC_NE_lexicon_tags_2 },
    { 0x223e53e4, SUC_NE_lexicon_tags_2 },
    { 0x445253ed, SUC_NE_lexicon_tags_2 },
    { 0x7a4653ea, SUC_NE_lexicon_tags_2 },
    { 0xe2c053ef, SUC_NE_lexicon_tags_2 },
    { 0x47853f0, SUC_NE_lexicon_tags_2 },
    { 0xc8d253f0, SUC_NE_lexicon_tags_2 },
    { 0xf34e53f2, SUC_NE_lexicon_tags_0 },
    { 0x286a53ef, SUC_NE_lexicon_tags_40 },
    { 0xb8da53f4, SUC_NE_lexicon_tags_2 },
    { 0x398a53f2, SUC_NE_lexicon_tags_2 },
    { 0xece853f6, SUC_NE_lexicon_tags_2 },
    { 0x8ebe53f6, SUC_NE_lexicon_tags_2 },
    { 0xb1d853f8, SUC_NE_lexicon_tags_2 },
    { 0xbba253f8, SUC_NE_lexicon_tags_2 },
    { 0x840e53f9, SUC_NE_lexicon_tags_1 },
    { 0xc63c53fb, SUC_NE_lexicon_tags_1 },
    { 0xc86453fa, SUC_NE_lexicon_tags_2 },
    { 0xd7ae53fd, SUC_NE_lexicon_tags_19 },
    { 0x469c53f4, SUC_NE_lexicon_tags_2 },
    { 0xc21853f1, SUC_NE_lexicon_tags_2 },
    { 0xb7cc53eb, SUC_NE_lexicon_tags_2 },
    { 0xac5c53fa, SUC_NE_lexicon_tags_2 },
    { 0x76ec5402, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1465406, SUC_NE_lexicon_tags_2 },
    { 0xb6625407, SUC_NE_lexicon_tags_2 },
    { 0x2f485408, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4642540a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x540e540c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed145410, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x61865412, SUC_NE_lexicon_tags_2 },
    { 0x29ce5412, SUC_NE_lexicon_tags_85 },
    { 0x39e5414, SUC_NE_lexicon_tags_2 },
    { 0x5ec05415, SUC_NE_lexicon_tags_2 },
    { 0x6d625415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8345419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d2541c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x687a541e, SUC_NE_lexicon_tags_2 },
    { 0x10a0541f, SUC_NE_lexicon_tags_2 },
    { 0x91365420, SUC_NE_lexicon_tags_2 },
    { 0xb1f25421, SUC_NE_lexicon_tags_2 },
    { 0xbc2a5421, SUC_NE_lexicon_tags_2 },
    { 0x20e25421, SUC_NE_lexicon_tags_2 },
    { 0xa2385424, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd25428, SUC_NE_lexicon_tags_7 },
    { 0x41485429, SUC_NE_lexicon_tags_7 },
    { 0x2d3a5428, SUC_NE_lexicon_tags_2 },
    { 0x433c542b, SUC_NE_lexicon_tags_2 },
    { 0xfda4542c, SUC_NE_lexicon_tags_0 },
    { 0xb738542a, SUC_NE_lexicon_tags_17 },
    { 0x421a542e, SUC_NE_lexicon_tags_2 },
    { 0x5130542f, SUC_NE_lexicon_tags_2 },
    { 0x7d16542e, SUC_NE_lexicon_tags_3 },
    { 0xf9f05431, SUC_NE_lexicon_tags_2 },
    { 0x24c85431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x651a5434, SUC_NE_lexicon_tags_2 },
    { 0xaf985435, SUC_NE_lexicon_tags_2 },
    { 0x78bc5435, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b6543d, SUC_NE_lexicon_tags_2 },
    { 0xaf8e543d, SUC_NE_lexicon_tags_2 },
    { 0x442e543f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a5442, SUC_NE_lexicon_tags_2 },
    { 0x4d9a5443, SUC_NE_lexicon_tags_2 },
    { 0xd83e5443, SUC_NE_lexicon_tags_2 },
    { 0x57c65445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebea5448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97fc544a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f05457, SUC_NE_lexicon_tags_2 },
    { 0xebca5458, SUC_NE_lexicon_tags_2 },
    { 0x47005459, SUC_NE_lexicon_tags_2 },
    { 0xba56545a, SUC_NE_lexicon_tags_188 },
    { 0x265c545b, SUC_NE_lexicon_tags_2 },
    { 0x4980545c, SUC_NE_lexicon_tags_2 },
    { 0x589e545d, SUC_NE_lexicon_tags_3 },
    { 0xc66e545d, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c45461, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a25464, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fa5467, SUC_NE_lexicon_tags_1 },
    { 0xae545468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3e4546a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d34546c, SUC_NE_lexicon_tags_2 },
    { 0x831e546d, SUC_NE_lexicon_tags_2 },
    { 0x2a70546d, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa45471, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xd82a5473, SUC_NE_lexicon_tags_0 },
    { 0xae865473, SUC_NE_lexicon_tags_0 },
    { 0x4ac05474, SUC_NE_lexicon_tags_2 },
    { 0xc3a25476, SUC_NE_lexicon_tags_2 },
    { 0xebf85477, SUC_NE_lexicon_tags_2 },
    { 0xa8c5477, SUC_NE_lexicon_tags_3 },
    { 0xe6685479, SUC_NE_lexicon_tags_2 },
    { 0x26d8547a, SUC_NE_lexicon_tags_2 },
    { 0xc0f2547a, SUC_NE_lexicon_tags_2 },
    { 0x8480547c, SUC_NE_lexicon_tags_2 },
    { 0x4a36547d, SUC_NE_lexicon_tags_7 },
    { 0x74ce547b, SUC_NE_lexicon_tags_2 },
    { 0xcd94547f, SUC_NE_lexicon_tags_2 },
    { 0x1d1c547e, SUC_NE_lexicon_tags_4 },
    { 0xbab45481, SUC_NE_lexicon_tags_2 },
    { 0xfc465482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b45486, SUC_NE_lexicon_tags_2 },
    { 0x33325486, SUC_NE_lexicon_tags_2 },
    { 0x59965488, SUC_NE_lexicon_tags_2 },
    { 0x4d225489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf8548c, SUC_NE_lexicon_tags_3 },
    { 0x7504548d, SUC_NE_lexicon_tags_2 },
    { 0xe3ac548d, SUC_NE_lexicon_tags_2 },
    { 0x39c6548f, SUC_NE_lexicon_tags_2 },
    { 0x3ab2548c, SUC_NE_lexicon_tags_2 },
    { 0x1205491, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c65496, SUC_NE_lexicon_tags_2 },
    { 0xd8ea5497, SUC_NE_lexicon_tags_2 },
    { 0xbfa5497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7096549a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87c549e, SUC_NE_lexicon_tags_2 },
    { 0x3e44549e, SUC_NE_lexicon_tags_2 },
    { 0xf752549e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60e854a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396054a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aca54a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10054ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb05254ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x964054b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bc054b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac54b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f054b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853454bd, SUC_NE_lexicon_tags_2 },
    { 0x7dd054be, SUC_NE_lexicon_tags_2 },
    { 0x34e454bf, SUC_NE_lexicon_tags_2 },
    { 0x2a3454bd, SUC_NE_lexicon_tags_2 },
    { 0xa86e54be, SUC_NE_lexicon_tags_2 },
    { 0x732654c2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1454c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x471254ca, SUC_NE_lexicon_tags_2 },
    { 0x8d5c54cb, SUC_NE_lexicon_tags_2 },
    { 0xba4a54cc, SUC_NE_lexicon_tags_2 },
    { 0xec7054cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b454d2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe31454d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b2e54d6, SUC_NE_lexicon_tags_2 },
    { 0x4d654d7, SUC_NE_lexicon_tags_4 },
    { 0xf08054d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fce54db, SUC_NE_lexicon_tags_1 },
    { 0xd19454db, SUC_NE_lexicon_tags_2 },
    { 0xc15054dc, SUC_NE_lexicon_tags_2 },
    { 0x36ee54de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x765054e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31e54e5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x314c54e7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3454ea, SUC_NE_lexicon_tags_2 },
    { 0x3ae854ea, SUC_NE_lexicon_tags_2 },
    { 0xc33254eb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df254f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70da54f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a454f8, SUC_NE_lexicon_tags_2 },
    { 0x7a1854f8, SUC_NE_lexicon_tags_2 },
    { 0xc7ac54f8, SUC_NE_lexicon_tags_3 },
    { 0x9b1454fb, SUC_NE_lexicon_tags_2 },
    { 0xf46054fc, SUC_NE_lexicon_tags_0 },
    { 0x700c54fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x783a54ff, SUC_NE_lexicon_tags_4 },
    { 0x69f654ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27d25502, SUC_NE_lexicon_tags_2 },
    { 0xf1b25503, SUC_NE_lexicon_tags_2 },
    { 0x6bda5502, SUC_NE_lexicon_tags_2 },
    { 0x56ca5505, SUC_NE_lexicon_tags_2 },
    { 0x53de5502, SUC_NE_lexicon_tags_2 },
    { 0x41d25507, SUC_NE_lexicon_tags_2 },
    { 0xc6c25508, SUC_NE_lexicon_tags_3 },
    { 0x47ca5504, SUC_NE_lexicon_tags_2 },
    { 0x6d2c550a, SUC_NE_lexicon_tags_2 },
    { 0x1b6550a, SUC_NE_lexicon_tags_2 },
    { 0x8d9a5509, SUC_NE_lexicon_tags_2 },
    { 0xb7a25506, SUC_NE_lexicon_tags_2 },
    { 0x2ca6550e, SUC_NE_lexicon_tags_2 },
    { 0xe38550f, SUC_NE_lexicon_tags_2 },
    { 0x37e25510, SUC_NE_lexicon_tags_2 },
    { 0x609e5510, SUC_NE_lexicon_tags_2 },
    { 0x3fb25512, SUC_NE_lexicon_tags_13 },
    { 0xb9665511, SUC_NE_lexicon_tags_2 },
    { 0xc3805514, SUC_NE_lexicon_tags_2 },
    { 0x79be5515, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0a5518, SUC_NE_lexicon_tags_2 },
    { 0x10e25519, SUC_NE_lexicon_tags_2 },
    { 0x75fe5519, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaac0551c, SUC_NE_lexicon_tags_2 },
    { 0x1682551c, SUC_NE_lexicon_tags_2 },
    { 0x4968551d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa65c5520, SUC_NE_lexicon_tags_2 },
    { 0xd0a25521, SUC_NE_lexicon_tags_2 },
    { 0xa67e5521, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07a5525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0c5529, SUC_NE_lexicon_tags_2 },
    { 0xf19e5529, SUC_NE_lexicon_tags_2 },
    { 0x8e25529, SUC_NE_lexicon_tags_2 },
    { 0x8694552c, SUC_NE_lexicon_tags_2 },
    { 0x78be552d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x990e5530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2385534, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f42553c, SUC_NE_lexicon_tags_2 },
    { 0x1e18553d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74e4553f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9785541, SUC_NE_lexicon_tags_2 },
    { 0x3a305542, SUC_NE_lexicon_tags_2 },
    { 0xb3be5543, SUC_NE_lexicon_tags_2 },
    { 0x860e5541, SUC_NE_lexicon_tags_0 },
    { 0x3dec5545, SUC_NE_lexicon_tags_2 },
    { 0xdc945546, SUC_NE_lexicon_tags_0 },
    { 0xc9ee5546, SUC_NE_lexicon_tags_2 },
    { 0x28365547, SUC_NE_lexicon_tags_2 },
    { 0xcde65545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3c4554b, SUC_NE_lexicon_tags_1 },
    { 0x8b3e554c, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447c5550, SUC_NE_lexicon_tags_2 },
    { 0x8ebc5550, SUC_NE_lexicon_tags_2 },
    { 0xf1145551, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b8c5554, SUC_NE_lexicon_tags_2 },
    { 0xbe725555, SUC_NE_lexicon_tags_2 },
    { 0x4b8a5554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60e555b, SUC_NE_lexicon_tags_2 },
    { 0xfc70555b, SUC_NE_lexicon_tags_2 },
    { 0xdb4e555b, SUC_NE_lexicon_tags_2 },
    { 0xb4f2555d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe55c5560, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b925562, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1de25564, SUC_NE_lexicon_tags_2 },
    { 0xbc745565, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80fc5567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef08556a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce02556d, SUC_NE_lexicon_tags_2 },
    { 0x45a556e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a25570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1f25572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2c85574, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0765578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9310557a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f4557f, SUC_NE_lexicon_tags_3 },
    { 0x60cc557f, SUC_NE_lexicon_tags_2 },
    { 0xf7e25581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae5586, SUC_NE_lexicon_tags_2 },
    { 0xf185586, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79e85589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1842558d, SUC_NE_lexicon_tags_2 },
    { 0xf77a558e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d505595, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf805597, SUC_NE_lexicon_tags_2 },
    { 0xb62c5598, SUC_NE_lexicon_tags_2 },
    { 0xb9cc5599, SUC_NE_lexicon_tags_2 },
    { 0x90405597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0855a3, SUC_NE_lexicon_tags_4 },
    { 0x9fbc55a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x474855a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a1455a9, SUC_NE_lexicon_tags_2 },
    { 0x22f455aa, SUC_NE_lexicon_tags_2 },
    { 0x8df455ab, SUC_NE_lexicon_tags_2 },
    { 0x417855ab, SUC_NE_lexicon_tags_2 },
    { 0x532a55a9, SUC_NE_lexicon_tags_2 },
    { 0x15a055a9, SUC_NE_lexicon_tags_2 },
    { 0xcc3a55af, SUC_NE_lexicon_tags_2 },
    { 0x4f2655af, SUC_NE_lexicon_tags_2 },
    { 0x9a1655b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdc655b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2a455b5, SUC_NE_lexicon_tags_2 },
    { 0xd12c55b6, SUC_NE_lexicon_tags_2 },
    { 0x91ce55b7, SUC_NE_lexicon_tags_2 },
    { 0x83e055b7, SUC_NE_lexicon_tags_2 },
    { 0xca4855b9, SUC_NE_lexicon_tags_2 },
    { 0x4c5455ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df855bd, SUC_NE_lexicon_tags_2 },
    { 0x1dae55be, SUC_NE_lexicon_tags_2 },
    { 0x780055bf, SUC_NE_lexicon_tags_2 },
    { 0x504855be, SUC_NE_lexicon_tags_2 },
    { 0xc73655c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6655c7, SUC_NE_lexicon_tags_2 },
    { 0xdcbe55c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867855cc, SUC_NE_lexicon_tags_2 },
    { 0x675e55cd, SUC_NE_lexicon_tags_2 },
    { 0xa52c55cd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc55d7, SUC_NE_lexicon_tags_2 },
    { 0xd1ea55d8, SUC_NE_lexicon_tags_256 },
    { 0, NULL },
    { 0x6f4055da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ac55df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d0a55e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87255e4, SUC_NE_lexicon_tags_2 },
    { 0xaf7c55e5, SUC_NE_lexicon_tags_2 },
    { 0x757455e6, SUC_NE_lexicon_tags_2 },
    { 0xb5d055e7, SUC_NE_lexicon_tags_2 },
    { 0xda2855e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905455ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d455f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4255f3, SUC_NE_lexicon_tags_2 },
    { 0xd6f855f3, SUC_NE_lexicon_tags_2 },
    { 0xf4c855f5, SUC_NE_lexicon_tags_8 },
    { 0xfb5a55f6, SUC_NE_lexicon_tags_2 },
    { 0x785255f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x156655f9, SUC_NE_lexicon_tags_2 },
    { 0x6b1455fa, SUC_NE_lexicon_tags_2 },
    { 0x659855fb, SUC_NE_lexicon_tags_2 },
    { 0xa29255fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3285600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x279e5602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff05605, SUC_NE_lexicon_tags_2 },
    { 0x464e5606, SUC_NE_lexicon_tags_2 },
    { 0xc88a5607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96a560a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cc5611, SUC_NE_lexicon_tags_2 },
    { 0x23c05612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x360c5615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd0561b, SUC_NE_lexicon_tags_7 },
    { 0x98bc561c, SUC_NE_lexicon_tags_2 },
    { 0xa92c561d, SUC_NE_lexicon_tags_2 },
    { 0xb1ea561d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9a5622, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4be5624, SUC_NE_lexicon_tags_2 },
    { 0x3e4e5625, SUC_NE_lexicon_tags_2 },
    { 0x6fca5625, SUC_NE_lexicon_tags_2 },
    { 0xabc25624, SUC_NE_lexicon_tags_2 },
    { 0x2a8a5628, SUC_NE_lexicon_tags_7 },
    { 0x4f65629, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe548562b, SUC_NE_lexicon_tags_2 },
    { 0xa258562c, SUC_NE_lexicon_tags_2 },
    { 0x274a562d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30645632, SUC_NE_lexicon_tags_17 },
    { 0x34265632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4725637, SUC_NE_lexicon_tags_2 },
    { 0x9a0c5638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281e563e, SUC_NE_lexicon_tags_91 },
    { 0x99ea563f, SUC_NE_lexicon_tags_2 },
    { 0x5826563f, SUC_NE_lexicon_tags_2 },
    { 0xb6dc5640, SUC_NE_lexicon_tags_2 },
    { 0x7cb4563f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef45647, SUC_NE_lexicon_tags_2 },
    { 0xd6005648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b06564a, SUC_NE_lexicon_tags_7 },
    { 0x145c564a, SUC_NE_lexicon_tags_2 },
    { 0x93ec564c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8ea4564e, SUC_NE_lexicon_tags_2 },
    { 0xb20564f, SUC_NE_lexicon_tags_2 },
    { 0xa86e564f, SUC_NE_lexicon_tags_2 },
    { 0xe624564f, SUC_NE_lexicon_tags_7 },
    { 0x93c05652, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1725659, SUC_NE_lexicon_tags_2 },
    { 0xc960565a, SUC_NE_lexicon_tags_2 },
    { 0x2faa5659, SUC_NE_lexicon_tags_2 },
    { 0xaf10565c, SUC_NE_lexicon_tags_2 },
    { 0x8cce565d, SUC_NE_lexicon_tags_2 },
    { 0xcf1e565e, SUC_NE_lexicon_tags_2 },
    { 0x5d22565f, SUC_NE_lexicon_tags_2 },
    { 0xd7a8565d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a25663, SUC_NE_lexicon_tags_2 },
    { 0xa6f45664, SUC_NE_lexicon_tags_32 },
    { 0x2ca65665, SUC_NE_lexicon_tags_18 },
    { 0x449a5666, SUC_NE_lexicon_tags_2 },
    { 0xb9865665, SUC_NE_lexicon_tags_2 },
    { 0xafd05668, SUC_NE_lexicon_tags_2 },
    { 0xac345669, SUC_NE_lexicon_tags_2 },
    { 0x4a685669, SUC_NE_lexicon_tags_2 },
    { 0xd8c4566b, SUC_NE_lexicon_tags_2 },
    { 0xe654566a, SUC_NE_lexicon_tags_2 },
    { 0x94945669, SUC_NE_lexicon_tags_2 },
    { 0x7cee566e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb305670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bbe5672, SUC_NE_lexicon_tags_2 },
    { 0x5265672, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a45676, SUC_NE_lexicon_tags_2 },
    { 0xf8585677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab8567b, SUC_NE_lexicon_tags_7 },
    { 0xe7a8567c, SUC_NE_lexicon_tags_2 },
    { 0x9a40567d, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x876e567f, SUC_NE_lexicon_tags_2 },
    { 0x3b785680, SUC_NE_lexicon_tags_2 },
    { 0x21ee5681, SUC_NE_lexicon_tags_2 },
    { 0x92445682, SUC_NE_lexicon_tags_2 },
    { 0x33ac5682, SUC_NE_lexicon_tags_3 },
    { 0xbc4e5684, SUC_NE_lexicon_tags_2 },
    { 0x4e7c5685, SUC_NE_lexicon_tags_2 },
    { 0xe2365686, SUC_NE_lexicon_tags_2 },
    { 0x52a5687, SUC_NE_lexicon_tags_2 },
    { 0xf4345686, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f8568b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b9e568d, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xa6fe568f, SUC_NE_lexicon_tags_2 },
    { 0x3f425690, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92485693, SUC_NE_lexicon_tags_2 },
    { 0x9b65693, SUC_NE_lexicon_tags_2 },
    { 0x6af85694, SUC_NE_lexicon_tags_2 },
    { 0xf8ac5696, SUC_NE_lexicon_tags_2 },
    { 0x20725696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6605699, SUC_NE_lexicon_tags_3 },
    { 0x50a65699, SUC_NE_lexicon_tags_2 },
    { 0x82de569b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17cc569e, SUC_NE_lexicon_tags_2 },
    { 0x731e569e, SUC_NE_lexicon_tags_24 },
    { 0x8e72569f, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18a56a4, SUC_NE_lexicon_tags_2 },
    { 0xe92e56a5, SUC_NE_lexicon_tags_2 },
    { 0x4c3a56a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x485c56a8, SUC_NE_lexicon_tags_2 },
    { 0x22e656a9, SUC_NE_lexicon_tags_2 },
    { 0x647856a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc56ad, SUC_NE_lexicon_tags_2 },
    { 0xaa6456ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978e56b3, SUC_NE_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfa56bd, SUC_NE_lexicon_tags_2 },
    { 0xd32056bd, SUC_NE_lexicon_tags_2 },
    { 0xc4e456bf, SUC_NE_lexicon_tags_2 },
    { 0xb31e56be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c8056c2, SUC_NE_lexicon_tags_2 },
    { 0x36ca56c3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1a56c6, SUC_NE_lexicon_tags_2 },
    { 0x7e6056c7, SUC_NE_lexicon_tags_102 },
    { 0x446e56c8, SUC_NE_lexicon_tags_2 },
    { 0x2b0e56c9, SUC_NE_lexicon_tags_2 },
    { 0x8ff256c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa456cd, SUC_NE_lexicon_tags_2 },
    { 0x7f5856cd, SUC_NE_lexicon_tags_2 },
    { 0x2a5456cf, SUC_NE_lexicon_tags_2 },
    { 0xdac856d0, SUC_NE_lexicon_tags_2 },
    { 0x454056d1, SUC_NE_lexicon_tags_2 },
    { 0x381256d2, SUC_NE_lexicon_tags_2 },
    { 0x64bc56d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b2a56d5, SUC_NE_lexicon_tags_2 },
    { 0x57056d6, SUC_NE_lexicon_tags_2 },
    { 0x605c56d7, SUC_NE_lexicon_tags_2 },
    { 0xe12256d6, SUC_NE_lexicon_tags_2 },
    { 0xb4a56d7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846456e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb456e4, SUC_NE_lexicon_tags_212 },
    { 0x5b6256e5, SUC_NE_lexicon_tags_19 },
    { 0x57b056e6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce56ea, SUC_NE_lexicon_tags_2 },
    { 0xba3c56eb, SUC_NE_lexicon_tags_2 },
    { 0xec6256ea, SUC_NE_lexicon_tags_85 },
    { 0x133e56ec, SUC_NE_lexicon_tags_2 },
    { 0x13c256ee, SUC_NE_lexicon_tags_3 },
    { 0xa21656ee, SUC_NE_lexicon_tags_2 },
    { 0x3b256ee, SUC_NE_lexicon_tags_2 },
    { 0x600856f1, SUC_NE_lexicon_tags_2 },
    { 0xb60c56f2, SUC_NE_lexicon_tags_2 },
    { 0x9d3856f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c4456f5, SUC_NE_lexicon_tags_2 },
    { 0xc12456f5, SUC_NE_lexicon_tags_2 },
    { 0xf65e56f7, SUC_NE_lexicon_tags_2 },
    { 0x93a56f7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b656fb, SUC_NE_lexicon_tags_2 },
    { 0xf90e56fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaed656fe, SUC_NE_lexicon_tags_2 },
    { 0x4a5a56ff, SUC_NE_lexicon_tags_2 },
    { 0x961c5700, SUC_NE_lexicon_tags_24 },
    { 0x64a5701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22885704, SUC_NE_lexicon_tags_2 },
    { 0x8ef25705, SUC_NE_lexicon_tags_2 },
    { 0xa9c5706, SUC_NE_lexicon_tags_2 },
    { 0xb1365707, SUC_NE_lexicon_tags_2 },
    { 0xa4765708, SUC_NE_lexicon_tags_2 },
    { 0x88c45709, SUC_NE_lexicon_tags_2 },
    { 0x6f565709, SUC_NE_lexicon_tags_2 },
    { 0xe73e570b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f72570d, SUC_NE_lexicon_tags_2 },
    { 0x65a2570e, SUC_NE_lexicon_tags_3 },
    { 0xa3ac570d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced85713, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3825716, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3e571d, SUC_NE_lexicon_tags_2 },
    { 0xb522571e, SUC_NE_lexicon_tags_3 },
    { 0xd96571e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xeabe5721, SUC_NE_lexicon_tags_2 },
    { 0xd3105722, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbee5727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ce572a, SUC_NE_lexicon_tags_2 },
    { 0xcbde572b, SUC_NE_lexicon_tags_2 },
    { 0x3216572b, SUC_NE_lexicon_tags_2 },
    { 0x5086572c, SUC_NE_lexicon_tags_2 },
    { 0x38f4572d, SUC_NE_lexicon_tags_2 },
    { 0xff3e572e, SUC_NE_lexicon_tags_2 },
    { 0xc870572e, SUC_NE_lexicon_tags_2 },
    { 0x1aa05731, SUC_NE_lexicon_tags_2 },
    { 0x3825732, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd6565734, SUC_NE_lexicon_tags_2 },
    { 0x28a45735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42a573a, SUC_NE_lexicon_tags_2 },
    { 0x585e573b, SUC_NE_lexicon_tags_2 },
    { 0x699e573a, SUC_NE_lexicon_tags_19 },
    { 0x115c573c, SUC_NE_lexicon_tags_2 },
    { 0xaa0e573e, SUC_NE_lexicon_tags_3 },
    { 0x58c4573a, SUC_NE_lexicon_tags_2 },
    { 0x51065740, SUC_NE_lexicon_tags_2 },
    { 0x8e2c5741, SUC_NE_lexicon_tags_2 },
    { 0xcc385741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ae5746, SUC_NE_lexicon_tags_2 },
    { 0x42185747, SUC_NE_lexicon_tags_2 },
    { 0x1e365747, SUC_NE_lexicon_tags_2 },
    { 0xce9c5749, SUC_NE_lexicon_tags_2 },
    { 0xff105749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b68574d, SUC_NE_lexicon_tags_2 },
    { 0x1754574d, SUC_NE_lexicon_tags_2 },
    { 0x17ae574f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ac5753, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0x73c45755, SUC_NE_lexicon_tags_1 },
    { 0x8b0a5756, SUC_NE_lexicon_tags_2 },
    { 0x9eba5756, SUC_NE_lexicon_tags_2 },
    { 0x660a5758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6058575a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828575f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b6e5761, SUC_NE_lexicon_tags_2 },
    { 0x67e25762, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73525767, SUC_NE_lexicon_tags_2 },
    { 0x5b385768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29c4576a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fec5771, SUC_NE_lexicon_tags_2 },
    { 0x9ebc5772, SUC_NE_lexicon_tags_2 },
    { 0xec545773, SUC_NE_lexicon_tags_2 },
    { 0xabac5774, SUC_NE_lexicon_tags_17 },
    { 0xc74a5774, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d005779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cca577d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da8577f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ff85781, SUC_NE_lexicon_tags_2 },
    { 0xe8965782, SUC_NE_lexicon_tags_2 },
    { 0x207e5783, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xb0025785, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x11085788, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4578d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73e5793, SUC_NE_lexicon_tags_2 },
    { 0xabb25793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11545799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf186579d, SUC_NE_lexicon_tags_2 },
    { 0x104a579e, SUC_NE_lexicon_tags_2 },
    { 0xef5c579d, SUC_NE_lexicon_tags_2 },
    { 0xacd2579d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e57a4, SUC_NE_lexicon_tags_34 },
    { 0xe5c857a4, SUC_NE_lexicon_tags_2 },
    { 0xf21e57a4, SUC_NE_lexicon_tags_2 },
    { 0x4fe057a7, SUC_NE_lexicon_tags_2 },
    { 0xdf6c57a7, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ec57ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x461a57ad, SUC_NE_lexicon_tags_2 },
    { 0x4c0a57ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x877857b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a457b2, SUC_NE_lexicon_tags_9 },
    { 0x224057b2, SUC_NE_lexicon_tags_7 },
    { 0xdfee57b3, SUC_NE_lexicon_tags_4 },
    { 0xb07657b3, SUC_NE_lexicon_tags_18 },
    { 0xde1457b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d457b9, SUC_NE_lexicon_tags_2 },
    { 0xd24257ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cc57c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8e57cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aa57d1, SUC_NE_lexicon_tags_2 },
    { 0x122e57d2, SUC_NE_lexicon_tags_2 },
    { 0x2c6457d1, SUC_NE_lexicon_tags_2 },
    { 0x6e5657d4, SUC_NE_lexicon_tags_86 },
    { 0x348657d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe44857d7, SUC_NE_lexicon_tags_2 },
    { 0x914a57d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e857db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95c57df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04857e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67ac57e5, SUC_NE_lexicon_tags_2 },
    { 0xe2a057e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8457e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66b657eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3a57f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0057f5, SUC_NE_lexicon_tags_2 },
    { 0x4fd857f6, SUC_NE_lexicon_tags_8 },
    { 0xb79857f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afe57fa, SUC_NE_lexicon_tags_2 },
    { 0xd24057fb, SUC_NE_lexicon_tags_2 },
    { 0x34ea57fc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x530857fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15985801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc385806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d265809, SUC_NE_lexicon_tags_2 },
    { 0x27de580a, SUC_NE_lexicon_tags_2 },
    { 0x996580a, SUC_NE_lexicon_tags_19 },
    { 0xa1a0580a, SUC_NE_lexicon_tags_2 },
    { 0x58d45809, SUC_NE_lexicon_tags_2 },
    { 0x7c32580c, SUC_NE_lexicon_tags_1 },
    { 0x62a6580f, SUC_NE_lexicon_tags_2 },
    { 0xc6e0580b, SUC_NE_lexicon_tags_2 },
    { 0x96e5811, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6b25813, SUC_NE_lexicon_tags_2 },
    { 0xe82a5814, SUC_NE_lexicon_tags_7 },
    { 0xd3945813, SUC_NE_lexicon_tags_2 },
    { 0x6e85814, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fa581b, SUC_NE_lexicon_tags_2 },
    { 0x5aa581c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df6581f, SUC_NE_lexicon_tags_2 },
    { 0xd552581f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x163a5823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b265825, SUC_NE_lexicon_tags_2 },
    { 0x48c25825, SUC_NE_lexicon_tags_2 },
    { 0xa9805825, SUC_NE_lexicon_tags_2 },
    { 0x2b185828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x719a582a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36fe582c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09e5831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bc5835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76e65837, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c8583a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcbe583c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9638583e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e825840, SUC_NE_lexicon_tags_2 },
    { 0xb5e25841, SUC_NE_lexicon_tags_2 },
    { 0xd31a5842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2a5846, SUC_NE_lexicon_tags_2 },
    { 0x55f85847, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0x36165849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72c5851, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6925854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5de5856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc25859, SUC_NE_lexicon_tags_2 },
    { 0x10cc5859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ac585d, SUC_NE_lexicon_tags_2 },
    { 0xc8b2585e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f365861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e705864, SUC_NE_lexicon_tags_2 },
    { 0xd0365864, SUC_NE_lexicon_tags_2 },
    { 0xabea5866, SUC_NE_lexicon_tags_5 },
    { 0x45c85867, SUC_NE_lexicon_tags_2 },
    { 0xc36e5868, SUC_NE_lexicon_tags_1 },
    { 0xc1a85869, SUC_NE_lexicon_tags_2 },
    { 0x79805869, SUC_NE_lexicon_tags_2 },
    { 0xa42a5864, SUC_NE_lexicon_tags_57 },
    { 0xc428586b, SUC_NE_lexicon_tags_2 },
    { 0x64d8586c, SUC_NE_lexicon_tags_2 },
    { 0x5474586e, SUC_NE_lexicon_tags_10 },
    { 0x57ce586e, SUC_NE_lexicon_tags_2 },
    { 0xf2de5870, SUC_NE_lexicon_tags_2 },
    { 0xdc105871, SUC_NE_lexicon_tags_2 },
    { 0x2d50586a, SUC_NE_lexicon_tags_2 },
    { 0x18925873, SUC_NE_lexicon_tags_2 },
    { 0x82ae5866, SUC_NE_lexicon_tags_2 },
    { 0x3cfc5875, SUC_NE_lexicon_tags_2 },
    { 0x1b5a5875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4ee5878, SUC_NE_lexicon_tags_2 },
    { 0xf73a5879, SUC_NE_lexicon_tags_32 },
    { 0x87be587a, SUC_NE_lexicon_tags_2 },
    { 0x8b56587b, SUC_NE_lexicon_tags_2 },
    { 0xed84587c, SUC_NE_lexicon_tags_2 },
    { 0x296e587d, SUC_NE_lexicon_tags_2 },
    { 0x3b6e587e, SUC_NE_lexicon_tags_2 },
    { 0x4cc25878, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf105881, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0505884, SUC_NE_lexicon_tags_2 },
    { 0x2e205885, SUC_NE_lexicon_tags_2 },
    { 0x94945886, SUC_NE_lexicon_tags_2 },
    { 0x316a5887, SUC_NE_lexicon_tags_2 },
    { 0xc1c85888, SUC_NE_lexicon_tags_2 },
    { 0x4d625888, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x71da588c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21a5891, SUC_NE_lexicon_tags_2 },
    { 0xbad45892, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa185896, SUC_NE_lexicon_tags_2 },
    { 0x486c5897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dfa5899, SUC_NE_lexicon_tags_2 },
    { 0x6e465899, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1820589c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aca589e, SUC_NE_lexicon_tags_4 },
    { 0x3e6e589f, SUC_NE_lexicon_tags_2 },
    { 0x566258a0, SUC_NE_lexicon_tags_2 },
    { 0xa6d058a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee58a5, SUC_NE_lexicon_tags_2 },
    { 0x310858a5, SUC_NE_lexicon_tags_2 },
    { 0x9b7458a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0858aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d6e58ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x273258af, SUC_NE_lexicon_tags_2 },
    { 0x7d3c58b0, SUC_NE_lexicon_tags_2 },
    { 0x670c58b1, SUC_NE_lexicon_tags_2 },
    { 0xcac58b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02e58b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86e258ba, SUC_NE_lexicon_tags_2 },
    { 0x544658bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x283458be, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0x731c58c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x431058c3, SUC_NE_lexicon_tags_373 },
    { 0x458858c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18258ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8a58d3, SUC_NE_lexicon_tags_2 },
    { 0xa73e58d4, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed858d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff3c58da, SUC_NE_lexicon_tags_49 },
    { 0x52b658db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ad858dd, SUC_NE_lexicon_tags_2 },
    { 0x9bce58de, SUC_NE_lexicon_tags_2 },
    { 0xa53458de, SUC_NE_lexicon_tags_13 },
    { 0xd26e58e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b458e6, SUC_NE_lexicon_tags_2 },
    { 0xd9c258e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa06c58e9, SUC_NE_lexicon_tags_2 },
    { 0x814c58ea, SUC_NE_lexicon_tags_1 },
    { 0x369658ea, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x3f0a58ed, SUC_NE_lexicon_tags_2 },
    { 0x3dde58ed, SUC_NE_lexicon_tags_2 },
    { 0xbdcc58ee, SUC_NE_lexicon_tags_2 },
    { 0x147e58f0, SUC_NE_lexicon_tags_2 },
    { 0x4f3858f0, SUC_NE_lexicon_tags_2 },
    { 0xf25458ef, SUC_NE_lexicon_tags_2 },
    { 0xd01058f0, SUC_NE_lexicon_tags_2 },
    { 0x352858f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcc458f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a58f9, SUC_NE_lexicon_tags_2 },
    { 0xb04058fa, SUC_NE_lexicon_tags_2 },
    { 0x68de58fa, SUC_NE_lexicon_tags_2 },
    { 0x9c7a58fb, SUC_NE_lexicon_tags_2 },
    { 0xf93c58fd, SUC_NE_lexicon_tags_2 },
    { 0x1f6c58fe, SUC_NE_lexicon_tags_2 },
    { 0x566058f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce85903, SUC_NE_lexicon_tags_2 },
    { 0xfc4c5903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61045907, SUC_NE_lexicon_tags_2 },
    { 0xc26c5908, SUC_NE_lexicon_tags_2 },
    { 0xe4905908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0590d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb160590f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45aa5916, SUC_NE_lexicon_tags_2 },
    { 0x9dc85916, SUC_NE_lexicon_tags_2 },
    { 0x64265918, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a2591b, SUC_NE_lexicon_tags_17 },
    { 0x6cfe591c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x591e591e, SUC_NE_lexicon_tags_2 },
    { 0x74ec591f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xddb65921, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44e25923, SUC_NE_lexicon_tags_2 },
    { 0xf81c5923, SUC_NE_lexicon_tags_2 },
    { 0x462c5925, SUC_NE_lexicon_tags_2 },
    { 0x850c5924, SUC_NE_lexicon_tags_2 },
    { 0x23745927, SUC_NE_lexicon_tags_2 },
    { 0x471e5928, SUC_NE_lexicon_tags_341 },
    { 0x2c6e5928, SUC_NE_lexicon_tags_2 },
    { 0x40fa5924, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24a0592c, SUC_NE_lexicon_tags_2 },
    { 0x71c0592d, SUC_NE_lexicon_tags_2 },
    { 0xf1c6592e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf65932, SUC_NE_lexicon_tags_2 },
    { 0xb1d85932, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c5936, SUC_NE_lexicon_tags_30 },
    { 0xc8ee5937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63645939, SUC_NE_lexicon_tags_2 },
    { 0x4966593a, SUC_NE_lexicon_tags_2 },
    { 0xd2f4593b, SUC_NE_lexicon_tags_2 },
    { 0xa47e5939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcef0593e, SUC_NE_lexicon_tags_2 },
    { 0x64ce593f, SUC_NE_lexicon_tags_2 },
    { 0x50185940, SUC_NE_lexicon_tags_2 },
    { 0x9444593f, SUC_NE_lexicon_tags_2 },
    { 0xcc425942, SUC_NE_lexicon_tags_2 },
    { 0x2cc0593f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab45947, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc4594a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87de594c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856c5950, SUC_NE_lexicon_tags_7 },
    { 0x44de5950, SUC_NE_lexicon_tags_2 },
    { 0x65a5950, SUC_NE_lexicon_tags_2 },
    { 0x87385953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28b05955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e25959, SUC_NE_lexicon_tags_160 },
    { 0x8880595a, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x4b5a595c, SUC_NE_lexicon_tags_2 },
    { 0x3002595d, SUC_NE_lexicon_tags_2 },
    { 0x48ae595e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1785967, SUC_NE_lexicon_tags_2 },
    { 0x837e5968, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5c5974, SUC_NE_lexicon_tags_2 },
    { 0x7fee5975, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8094597a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12597e, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xd805980, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9e5983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d65989, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f74598b, SUC_NE_lexicon_tags_2 },
    { 0x78ae598c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7465992, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca5998, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe614599a, SUC_NE_lexicon_tags_2 },
    { 0x4332599b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4c59a0, SUC_NE_lexicon_tags_2 },
    { 0xb14a59a0, SUC_NE_lexicon_tags_24 },
    { 0x5d059a2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb259a7, SUC_NE_lexicon_tags_2 },
    { 0x1ab659a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5c259aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fb659ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7a59b0, SUC_NE_lexicon_tags_2 },
    { 0xb18259b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c6459b3, SUC_NE_lexicon_tags_2 },
    { 0xd8a259b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce059b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b059be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518659c2, SUC_NE_lexicon_tags_2 },
    { 0xbedc59c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c059c7, SUC_NE_lexicon_tags_2 },
    { 0x618259c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42da59ca, SUC_NE_lexicon_tags_121 },
    { 0, NULL },
    { 0x356059cc, SUC_NE_lexicon_tags_2 },
    { 0x19b859cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb6459cf, SUC_NE_lexicon_tags_2 },
    { 0x8c5659d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc39059d2, SUC_NE_lexicon_tags_2 },
    { 0x317259d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c0a59d5, SUC_NE_lexicon_tags_2 },
    { 0x793e59d6, SUC_NE_lexicon_tags_103 },
    { 0x3a259d5, SUC_NE_lexicon_tags_2 },
    { 0xf6b459d7, SUC_NE_lexicon_tags_2 },
    { 0x53de59d5, SUC_NE_lexicon_tags_30 },
    { 0xfe2459da, SUC_NE_lexicon_tags_2 },
    { 0x46f459db, SUC_NE_lexicon_tags_2 },
    { 0x8d0859dc, SUC_NE_lexicon_tags_2 },
    { 0x8e7e59dd, SUC_NE_lexicon_tags_2 },
    { 0xed0a59dd, SUC_NE_lexicon_tags_2 },
    { 0x1fe459df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca59e5, SUC_NE_lexicon_tags_178 },
    { 0, NULL },
    { 0xaff859e7, SUC_NE_lexicon_tags_2 },
    { 0x270a59e7, SUC_NE_lexicon_tags_2 },
    { 0xace459e7, SUC_NE_lexicon_tags_2 },
    { 0xa7859ea, SUC_NE_lexicon_tags_104 },
    { 0xc7d659e9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ec59f0, SUC_NE_lexicon_tags_2 },
    { 0x131059f1, SUC_NE_lexicon_tags_5 },
    { 0xc49c59f1, SUC_NE_lexicon_tags_2 },
    { 0x1e5859f3, SUC_NE_lexicon_tags_2 },
    { 0xc6459f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18859f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658259fb, SUC_NE_lexicon_tags_2 },
    { 0x41d859fc, SUC_NE_lexicon_tags_2 },
    { 0x151859fd, SUC_NE_lexicon_tags_2 },
    { 0x98ec59fb, SUC_NE_lexicon_tags_2 },
    { 0x558e59ff, SUC_NE_lexicon_tags_2 },
    { 0x96e659fc, SUC_NE_lexicon_tags_2 },
    { 0x97f85a01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x391c5a04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab65a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33685a0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1385a11, SUC_NE_lexicon_tags_3 },
    { 0x13505a12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58e85a14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aa5a1a, SUC_NE_lexicon_tags_2 },
    { 0xc0765a1b, SUC_NE_lexicon_tags_2 },
    { 0x717c5a1c, SUC_NE_lexicon_tags_2 },
    { 0x13ec5a1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b45a20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x327a5a22, SUC_NE_lexicon_tags_2 },
    { 0x4ed05a23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd225a26, SUC_NE_lexicon_tags_2 },
    { 0x2dfa5a26, SUC_NE_lexicon_tags_2 },
    { 0x4a525a27, SUC_NE_lexicon_tags_2 },
    { 0xe9f05a28, SUC_NE_lexicon_tags_7 },
    { 0x535e5a2a, SUC_NE_lexicon_tags_2 },
    { 0xf10e5a2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e25a2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76845a30, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe25a33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35685a3b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x85785a3d, SUC_NE_lexicon_tags_2 },
    { 0xda05a3e, SUC_NE_lexicon_tags_3 },
    { 0x10485a3e, SUC_NE_lexicon_tags_2 },
    { 0x91aa5a3f, SUC_NE_lexicon_tags_2 },
    { 0x13be5a3f, SUC_NE_lexicon_tags_7 },
    { 0x6e765a3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5f05a44, SUC_NE_lexicon_tags_7 },
    { 0x14bc5a45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96485a48, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e5a4b, SUC_NE_lexicon_tags_2 },
    { 0xb08a5a4c, SUC_NE_lexicon_tags_2 },
    { 0xc14e5a4b, SUC_NE_lexicon_tags_2 },
    { 0x702a5a4e, SUC_NE_lexicon_tags_7 },
    { 0x72c25a4f, SUC_NE_lexicon_tags_2 },
    { 0x94da5a4f, SUC_NE_lexicon_tags_2 },
    { 0x97425a51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba5a54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fee5a58, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x30705a5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7c5a62, SUC_NE_lexicon_tags_191 },
    { 0x64fe5a62, SUC_NE_lexicon_tags_2 },
    { 0x3f765a64, SUC_NE_lexicon_tags_9 },
    { 0xc6f05a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f325a68, SUC_NE_lexicon_tags_3 },
    { 0xfa065a69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1825a6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12765a73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc585a7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0405a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ff65a7f, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5a5a83, SUC_NE_lexicon_tags_2 },
    { 0x68d25a84, SUC_NE_lexicon_tags_2 },
    { 0x18a5a83, SUC_NE_lexicon_tags_2 },
    { 0x1fae5a86, SUC_NE_lexicon_tags_2 },
    { 0x4cc45a84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83c5a8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61e25a8c, SUC_NE_lexicon_tags_2 },
    { 0x56705a8c, SUC_NE_lexicon_tags_4 },
    { 0x38be5a8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58e5a90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a85a95, SUC_NE_lexicon_tags_2 },
    { 0xd70e5a95, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd05a9a, SUC_NE_lexicon_tags_2 },
    { 0x3845a9b, SUC_NE_lexicon_tags_2 },
    { 0x15125a9c, SUC_NE_lexicon_tags_7 },
    { 0x57e25a9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4685aa0, SUC_NE_lexicon_tags_2 },
    { 0xe66e5aa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64f85aa3, SUC_NE_lexicon_tags_2 },
    { 0x21305aa3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea85aa8, SUC_NE_lexicon_tags_19 },
    { 0xc6fa5aa9, SUC_NE_lexicon_tags_2 },
    { 0x97f45aa9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec5ab0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0c5ab3, SUC_NE_lexicon_tags_2 },
    { 0xdf045ab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83e25ab6, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7385ab9, SUC_NE_lexicon_tags_2 },
    { 0x3af05ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb585abc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a085abe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e765ac4, SUC_NE_lexicon_tags_2 },
    { 0x9b745ac4, SUC_NE_lexicon_tags_2 },
    { 0xfef05ac4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a45ac9, SUC_NE_lexicon_tags_2 },
    { 0x5b3c5ac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f45ad2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf2e5ad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4205ad6, SUC_NE_lexicon_tags_2 },
    { 0x1005ad7, SUC_NE_lexicon_tags_2 },
    { 0xd3445ad8, SUC_NE_lexicon_tags_2 },
    { 0x2245ad6, SUC_NE_lexicon_tags_2 },
    { 0xe5aa5ada, SUC_NE_lexicon_tags_2 },
    { 0x5f3a5adb, SUC_NE_lexicon_tags_2 },
    { 0xde385ad6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c425adf, SUC_NE_lexicon_tags_2 },
    { 0xef325ae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aa5ae4, SUC_NE_lexicon_tags_2 },
    { 0x704c5ae4, SUC_NE_lexicon_tags_2 },
    { 0x1c085ae6, SUC_NE_lexicon_tags_2 },
    { 0x2ffa5ae6, SUC_NE_lexicon_tags_2 },
    { 0xa0525ae8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fe5aed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29e5af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68c05af3, SUC_NE_lexicon_tags_2 },
    { 0xc85a5af3, SUC_NE_lexicon_tags_2 },
    { 0x38fe5af5, SUC_NE_lexicon_tags_7 },
    { 0xb0285af6, SUC_NE_lexicon_tags_2 },
    { 0x53185af7, SUC_NE_lexicon_tags_94 },
    { 0xe57a5af7, SUC_NE_lexicon_tags_2 },
    { 0xa5f25af9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe205afb, SUC_NE_lexicon_tags_2 },
    { 0x7285afc, SUC_NE_lexicon_tags_2 },
    { 0x9a285afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1105b01, SUC_NE_lexicon_tags_2 },
    { 0x324c5b02, SUC_NE_lexicon_tags_2 },
    { 0x630a5b01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x682c5b05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ef85b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9625b0a, SUC_NE_lexicon_tags_2 },
    { 0x49645b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x852e5b0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c05b12, SUC_NE_lexicon_tags_3 },
    { 0xa7c05b13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d945b18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50225b20, SUC_NE_lexicon_tags_2 },
    { 0x65dc5b21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43205b23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b325b26, SUC_NE_lexicon_tags_2 },
    { 0xf1125b27, SUC_NE_lexicon_tags_2 },
    { 0x3df05b26, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x7f865b2a, SUC_NE_lexicon_tags_2 },
    { 0x10aa5b2b, SUC_NE_lexicon_tags_2 },
    { 0x7a25b2c, SUC_NE_lexicon_tags_2 },
    { 0xcf205b2c, SUC_NE_lexicon_tags_2 },
    { 0xae845b2d, SUC_NE_lexicon_tags_2 },
    { 0xe2da5b2b, SUC_NE_lexicon_tags_2 },
    { 0xb77e5b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e5b34, SUC_NE_lexicon_tags_2 },
    { 0x39cc5b35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9345b38, SUC_NE_lexicon_tags_2 },
    { 0x973c5b39, SUC_NE_lexicon_tags_2 },
    { 0x8cae5b3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9765b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28425b3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x364c5b41, SUC_NE_lexicon_tags_2 },
    { 0x6d685b42, SUC_NE_lexicon_tags_2 },
    { 0x1c4a5b43, SUC_NE_lexicon_tags_2 },
    { 0x76365b41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336e5b49, SUC_NE_lexicon_tags_2 },
    { 0xd2da5b4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63085b4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x217a5b50, SUC_NE_lexicon_tags_2 },
    { 0x53065b51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa485b54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b45b5a, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a45b5c, SUC_NE_lexicon_tags_3 },
    { 0x17c85b5d, SUC_NE_lexicon_tags_2 },
    { 0x56365b5c, SUC_NE_lexicon_tags_2 },
    { 0x14e65b5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f25b65, SUC_NE_lexicon_tags_2 },
    { 0xb6945b66, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30a5b6a, SUC_NE_lexicon_tags_2 },
    { 0x4d5e5b6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0c65b6d, SUC_NE_lexicon_tags_4 },
    { 0x6ec65b6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbac5b73, SUC_NE_lexicon_tags_2 },
    { 0x37dc5b73, SUC_NE_lexicon_tags_2 },
    { 0xa90e5b75, SUC_NE_lexicon_tags_2 },
    { 0xbe6a5b76, SUC_NE_lexicon_tags_2 },
    { 0xe36a5b75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f685b7a, SUC_NE_lexicon_tags_98 },
    { 0xf3ec5b7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a385b81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b025b83, SUC_NE_lexicon_tags_2 },
    { 0x57dc5b84, SUC_NE_lexicon_tags_2 },
    { 0x757a5b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d65b8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98745b90, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a2a5b92, SUC_NE_lexicon_tags_2 },
    { 0x93225b92, SUC_NE_lexicon_tags_2 },
    { 0xa3045b94, SUC_NE_lexicon_tags_2 },
    { 0x942a5b92, SUC_NE_lexicon_tags_1 },
    { 0x76a65b96, SUC_NE_lexicon_tags_2 },
    { 0x3ada5b97, SUC_NE_lexicon_tags_18 },
    { 0x471c5b97, SUC_NE_lexicon_tags_2 },
    { 0x8e205b99, SUC_NE_lexicon_tags_2 },
    { 0x42c65b96, SUC_NE_lexicon_tags_2 },
    { 0xcc565b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cc5ba3, SUC_NE_lexicon_tags_2 },
    { 0xc4cc5ba3, SUC_NE_lexicon_tags_1 },
    { 0x2bb65ba3, SUC_NE_lexicon_tags_7 },
    { 0x225ba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c85ba9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37085bab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9785bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25e5bb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8c5bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c85bba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39c25bbc, SUC_NE_lexicon_tags_19 },
    { 0x531a5bbd, SUC_NE_lexicon_tags_1 },
    { 0x3c145bbe, SUC_NE_lexicon_tags_2 },
    { 0xba105bbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a5bc3, SUC_NE_lexicon_tags_2 },
    { 0x2ad05bc4, SUC_NE_lexicon_tags_2 },
    { 0x769a5bc4, SUC_NE_lexicon_tags_2 },
    { 0x511a5bc5, SUC_NE_lexicon_tags_2 },
    { 0x99325bc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e25bca, SUC_NE_lexicon_tags_2 },
    { 0x7b905bcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ec5bd3, SUC_NE_lexicon_tags_2 },
    { 0x24c5bd3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c5bda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0945bde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2085be3, SUC_NE_lexicon_tags_2 },
    { 0xd1f25be4, SUC_NE_lexicon_tags_2 },
    { 0xefc5be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8a5be8, SUC_NE_lexicon_tags_4 },
    { 0x48245be9, SUC_NE_lexicon_tags_2 },
    { 0x59f85be9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c125bed, SUC_NE_lexicon_tags_2 },
    { 0xece45bee, SUC_NE_lexicon_tags_2 },
    { 0xaa25bee, SUC_NE_lexicon_tags_2 },
    { 0xc5985bf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44125bf2, SUC_NE_lexicon_tags_4 },
    { 0x158c5bf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7845bf6, SUC_NE_lexicon_tags_61 },
    { 0x28ac5bf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b265bfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4e5bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3efc5c00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86365c04, SUC_NE_lexicon_tags_2 },
    { 0x7d865c05, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19825c09, SUC_NE_lexicon_tags_2 },
    { 0x10305c0a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ae5c0d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a05c10, SUC_NE_lexicon_tags_2 },
    { 0x70b25c11, SUC_NE_lexicon_tags_2 },
    { 0x3c665c12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xddda5c14, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c05c1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a85c22, SUC_NE_lexicon_tags_2 },
    { 0xf46e5c23, SUC_NE_lexicon_tags_2 },
    { 0x45845c23, SUC_NE_lexicon_tags_2 },
    { 0xf6e5c25, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x52545c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36425c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee905c2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb045c30, SUC_NE_lexicon_tags_2 },
    { 0x9e965c31, SUC_NE_lexicon_tags_3 },
    { 0xbc405c32, SUC_NE_lexicon_tags_2 },
    { 0xb2bc5c32, SUC_NE_lexicon_tags_2 },
    { 0xec945c34, SUC_NE_lexicon_tags_2 },
    { 0xa6a85c35, SUC_NE_lexicon_tags_2 },
    { 0x60ce5c36, SUC_NE_lexicon_tags_2 },
    { 0xeeec5c32, SUC_NE_lexicon_tags_2 },
    { 0xd3545c31, SUC_NE_lexicon_tags_2 },
    { 0x3f7e5c33, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b25c3c, SUC_NE_lexicon_tags_2 },
    { 0xe2f05c3d, SUC_NE_lexicon_tags_2 },
    { 0xf23c5c3e, SUC_NE_lexicon_tags_2 },
    { 0xa3f05c3e, SUC_NE_lexicon_tags_222 },
    { 0xccf45c40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac7a5c42, SUC_NE_lexicon_tags_2 },
    { 0xf1265c42, SUC_NE_lexicon_tags_2 },
    { 0xdf7c5c42, SUC_NE_lexicon_tags_2 },
    { 0xcce5c44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95c65c47, SUC_NE_lexicon_tags_2 },
    { 0x244a5c47, SUC_NE_lexicon_tags_2 },
    { 0x4e105c49, SUC_NE_lexicon_tags_0 },
    { 0x53725c49, SUC_NE_lexicon_tags_2 },
    { 0x7af85c4b, SUC_NE_lexicon_tags_2 },
    { 0x6dea5c4b, SUC_NE_lexicon_tags_2 },
    { 0xe945c4a, SUC_NE_lexicon_tags_2 },
    { 0x8b1a5c49, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f425c52, SUC_NE_lexicon_tags_2 },
    { 0x542e5c53, SUC_NE_lexicon_tags_2 },
    { 0xa3a5c52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46e65c56, SUC_NE_lexicon_tags_3 },
    { 0xb00e5c57, SUC_NE_lexicon_tags_2 },
    { 0x71965c57, SUC_NE_lexicon_tags_2 },
    { 0x7ec65c58, SUC_NE_lexicon_tags_2 },
    { 0xe2185c59, SUC_NE_lexicon_tags_4 },
    { 0xe1045c5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6465c5d, SUC_NE_lexicon_tags_2 },
    { 0x196c5c5d, SUC_NE_lexicon_tags_3 },
    { 0x85805c5d, SUC_NE_lexicon_tags_7 },
    { 0x500a5c60, SUC_NE_lexicon_tags_2 },
    { 0xfb025c61, SUC_NE_lexicon_tags_2 },
    { 0xe1345c61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe19e5c64, SUC_NE_lexicon_tags_3 },
    { 0x20045c65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb805c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a05c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c85c70, SUC_NE_lexicon_tags_0 },
    { 0x55325c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d45c74, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f845c76, SUC_NE_lexicon_tags_40 },
    { 0xfaf25c76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd07a5c79, SUC_NE_lexicon_tags_2 },
    { 0x255c5c7a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa785c7c, SUC_NE_lexicon_tags_2 },
    { 0xacc25c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb25c82, SUC_NE_lexicon_tags_2 },
    { 0xf0fa5c83, SUC_NE_lexicon_tags_2 },
    { 0x227a5c83, SUC_NE_lexicon_tags_2 },
    { 0x6f6c5c82, SUC_NE_lexicon_tags_2 },
    { 0xfc765c86, SUC_NE_lexicon_tags_2 },
    { 0xcada5c82, SUC_NE_lexicon_tags_2 },
    { 0x434a5c88, SUC_NE_lexicon_tags_2 },
    { 0x46185c89, SUC_NE_lexicon_tags_2 },
    { 0xf13c5c8a, SUC_NE_lexicon_tags_7 },
    { 0xdbaa5c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d45c8f, SUC_NE_lexicon_tags_2 },
    { 0x73465c90, SUC_NE_lexicon_tags_2 },
    { 0x91665c91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101e5c96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44725c98, SUC_NE_lexicon_tags_2 },
    { 0xe4f25c99, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fa5c9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22d65c9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8c5ca3, SUC_NE_lexicon_tags_2 },
    { 0x4f005ca3, SUC_NE_lexicon_tags_2 },
    { 0x1a2e5ca5, SUC_NE_lexicon_tags_103 },
    { 0x1d345ca5, SUC_NE_lexicon_tags_2 },
    { 0x80b85ca7, SUC_NE_lexicon_tags_2 },
    { 0x41ee5ca8, SUC_NE_lexicon_tags_2 },
    { 0x1a6c5ca9, SUC_NE_lexicon_tags_2 },
    { 0x41fa5caa, SUC_NE_lexicon_tags_2 },
    { 0x9d885cab, SUC_NE_lexicon_tags_2 },
    { 0x6d6a5cac, SUC_NE_lexicon_tags_2 },
    { 0x8da85cad, SUC_NE_lexicon_tags_1 },
    { 0xb1aa5cae, SUC_NE_lexicon_tags_2 },
    { 0x2a9e5caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedba5cb5, SUC_NE_lexicon_tags_2 },
    { 0x196a5cb5, SUC_NE_lexicon_tags_2 },
    { 0xd9265cb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94205cbd, SUC_NE_lexicon_tags_2 },
    { 0x28865cbd, SUC_NE_lexicon_tags_2 },
    { 0xd5245cbd, SUC_NE_lexicon_tags_51 },
    { 0xfd85cc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6c5cc6, SUC_NE_lexicon_tags_2 },
    { 0xacce5cc7, SUC_NE_lexicon_tags_3 },
    { 0x66f45cc8, SUC_NE_lexicon_tags_2 },
    { 0x9c5a5cc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa00e5ccb, SUC_NE_lexicon_tags_5 },
    { 0x5f7c5ccb, SUC_NE_lexicon_tags_30 },
    { 0xfc9c5ccd, SUC_NE_lexicon_tags_2 },
    { 0xacde5ccb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23305cd0, SUC_NE_lexicon_tags_2 },
    { 0x24525cd1, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69165cdc, SUC_NE_lexicon_tags_2 },
    { 0x15ea5cdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a5ce2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1645ce6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d5e5ce8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdce5cea, SUC_NE_lexicon_tags_243 },
    { 0xa1be5cea, SUC_NE_lexicon_tags_2 },
    { 0xec1a5ceb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ae5cef, SUC_NE_lexicon_tags_2 },
    { 0x9c665cef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5445cf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb025cf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc625cfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892c5cff, SUC_NE_lexicon_tags_2 },
    { 0x83825cff, SUC_NE_lexicon_tags_3 },
    { 0x1cb65d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77765d03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e5d06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c65d0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e05d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24985d12, SUC_NE_lexicon_tags_2 },
    { 0x54025d12, SUC_NE_lexicon_tags_2 },
    { 0x27cc5d14, SUC_NE_lexicon_tags_0 },
    { 0xfbe85d15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x233a5d17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa8a5d19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda045d1d, SUC_NE_lexicon_tags_2 },
    { 0x34dc5d1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff8a5d20, SUC_NE_lexicon_tags_2 },
    { 0x45ac5d20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48165d24, SUC_NE_lexicon_tags_2 },
    { 0xef585d25, SUC_NE_lexicon_tags_2 },
    { 0xf57a5d26, SUC_NE_lexicon_tags_2 },
    { 0xbcae5d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbae65d29, SUC_NE_lexicon_tags_2 },
    { 0x89265d2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e485d2c, SUC_NE_lexicon_tags_58 },
    { 0x30665d2d, SUC_NE_lexicon_tags_2 },
    { 0x528e5d2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce5d38, SUC_NE_lexicon_tags_2 },
    { 0x61ca5d39, SUC_NE_lexicon_tags_2 },
    { 0xf69c5d3a, SUC_NE_lexicon_tags_138 },
    { 0x32285d3b, SUC_NE_lexicon_tags_2 },
    { 0x1ef05d3c, SUC_NE_lexicon_tags_3 },
    { 0x8245d3d, SUC_NE_lexicon_tags_2 },
    { 0xbb0a5d39, SUC_NE_lexicon_tags_2 },
    { 0xddfc5d3b, SUC_NE_lexicon_tags_2 },
    { 0x5b285d3e, SUC_NE_lexicon_tags_2 },
    { 0x94b65d3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec3a5d43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d365d47, SUC_NE_lexicon_tags_2 },
    { 0x428a5d47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12405d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ea5d4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0585d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x120e5d53, SUC_NE_lexicon_tags_2 },
    { 0xd6b05d53, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x44825d56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786c5d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6be45d5d, SUC_NE_lexicon_tags_2 },
    { 0x23085d5d, SUC_NE_lexicon_tags_2 },
    { 0x208e5d5f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x39fe5d61, SUC_NE_lexicon_tags_7 },
    { 0x7ece5d62, SUC_NE_lexicon_tags_17 },
    { 0x9eec5d63, SUC_NE_lexicon_tags_2 },
    { 0x7bb25d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc245d67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x269a5d6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277a5d72, SUC_NE_lexicon_tags_2 },
    { 0xcc6a5d73, SUC_NE_lexicon_tags_2 },
    { 0x56fc5d73, SUC_NE_lexicon_tags_2 },
    { 0x8d2e5d75, SUC_NE_lexicon_tags_2 },
    { 0x59ac5d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97b45d78, SUC_NE_lexicon_tags_2 },
    { 0x72fc5d79, SUC_NE_lexicon_tags_2 },
    { 0x93e45d7a, SUC_NE_lexicon_tags_2 },
    { 0xd6aa5d7a, SUC_NE_lexicon_tags_2 },
    { 0x15925d7a, SUC_NE_lexicon_tags_2 },
    { 0x969a5d7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f45d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec445d88, SUC_NE_lexicon_tags_2 },
    { 0xc07a5d89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc725d91, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84185d94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49b65d96, SUC_NE_lexicon_tags_2 },
    { 0xbdee5d96, SUC_NE_lexicon_tags_2 },
    { 0xba9a5d97, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1e5d9b, SUC_NE_lexicon_tags_2 },
    { 0x73a05d9c, SUC_NE_lexicon_tags_2 },
    { 0xc6945d9d, SUC_NE_lexicon_tags_2 },
    { 0x5f445d9e, SUC_NE_lexicon_tags_2 },
    { 0x42cc5d9f, SUC_NE_lexicon_tags_19 },
    { 0x128c5da0, SUC_NE_lexicon_tags_2 },
    { 0x78685da1, SUC_NE_lexicon_tags_2 },
    { 0xb0005d9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9365da6, SUC_NE_lexicon_tags_11 },
    { 0x34c65da6, SUC_NE_lexicon_tags_2 },
    { 0x5ba85da8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61845daa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a025daf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46845db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d545db6, SUC_NE_lexicon_tags_2 },
    { 0xcb5a5db7, SUC_NE_lexicon_tags_2 },
    { 0x895a5db8, SUC_NE_lexicon_tags_2 },
    { 0x25f65db9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4a5dbe, SUC_NE_lexicon_tags_2 },
    { 0x90a5dbf, SUC_NE_lexicon_tags_2 },
    { 0x22465dbf, SUC_NE_lexicon_tags_2 },
    { 0x7fb65dc1, SUC_NE_lexicon_tags_2 },
    { 0x4f1c5dc2, SUC_NE_lexicon_tags_3 },
    { 0xc665dc3, SUC_NE_lexicon_tags_36 },
    { 0x247a5dc4, SUC_NE_lexicon_tags_2 },
    { 0xb4925dc4, SUC_NE_lexicon_tags_2 },
    { 0xbfd65dc5, SUC_NE_lexicon_tags_2 },
    { 0x825c5dc2, SUC_NE_lexicon_tags_36 },
    { 0xa35e5dc7, SUC_NE_lexicon_tags_2 },
    { 0x44bc5dc9, SUC_NE_lexicon_tags_0 },
    { 0x738a5dc9, SUC_NE_lexicon_tags_2 },
    { 0xb50c5dc9, SUC_NE_lexicon_tags_2 },
    { 0x51905dcb, SUC_NE_lexicon_tags_2 },
    { 0x871c5dcc, SUC_NE_lexicon_tags_2 },
    { 0x5e645dcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4085dd3, SUC_NE_lexicon_tags_2 },
    { 0xa2d85dd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ac5dd8, SUC_NE_lexicon_tags_2 },
    { 0xda285dd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5285ddf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b25de2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d65de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0185de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9c5de9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0785ded, SUC_NE_lexicon_tags_2 },
    { 0xc85e5ded, SUC_NE_lexicon_tags_2 },
    { 0xda925ded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3e5df2, SUC_NE_lexicon_tags_2 },
    { 0xc54a5df3, SUC_NE_lexicon_tags_2 },
    { 0xd9f25df3, SUC_NE_lexicon_tags_13 },
    { 0xb4fe5df3, SUC_NE_lexicon_tags_2 },
    { 0x5c7a5df6, SUC_NE_lexicon_tags_2 },
    { 0x71d65df7, SUC_NE_lexicon_tags_2 },
    { 0xce705df3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50c5dfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7a5dfe, SUC_NE_lexicon_tags_2 },
    { 0xe67e5dfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22a05e01, SUC_NE_lexicon_tags_1 },
    { 0x9a245e02, SUC_NE_lexicon_tags_2 },
    { 0xb45c5e03, SUC_NE_lexicon_tags_2 },
    { 0xd1be5e03, SUC_NE_lexicon_tags_2 },
    { 0xcf4a5e04, SUC_NE_lexicon_tags_2 },
    { 0xd5da5e06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33945e0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8485e12, SUC_NE_lexicon_tags_2 },
    { 0xd7c05e13, SUC_NE_lexicon_tags_2 },
    { 0x46025e14, SUC_NE_lexicon_tags_2 },
    { 0x32565e14, SUC_NE_lexicon_tags_2 },
    { 0x950a5e14, SUC_NE_lexicon_tags_2 },
    { 0x61425e17, SUC_NE_lexicon_tags_2 },
    { 0xf83e5e17, SUC_NE_lexicon_tags_2 },
    { 0xcffa5e14, SUC_NE_lexicon_tags_364 },
    { 0xd4865e1a, SUC_NE_lexicon_tags_2 },
    { 0x65545e1a, SUC_NE_lexicon_tags_2 },
    { 0x948e5e18, SUC_NE_lexicon_tags_2 },
    { 0x8e5a5e14, SUC_NE_lexicon_tags_2 },
    { 0x1f645e17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87985e21, SUC_NE_lexicon_tags_2 },
    { 0x71ee5e22, SUC_NE_lexicon_tags_2 },
    { 0x54ee5e22, SUC_NE_lexicon_tags_2 },
    { 0x34405e24, SUC_NE_lexicon_tags_0 },
    { 0x13c65e25, SUC_NE_lexicon_tags_0 },
    { 0xca0e5e24, SUC_NE_lexicon_tags_1 },
    { 0x58a5e27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x238e5e2a, SUC_NE_lexicon_tags_2 },
    { 0xab865e2b, SUC_NE_lexicon_tags_2 },
    { 0xd3ce5e2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ba5e2f, SUC_NE_lexicon_tags_2 },
    { 0xa41c5e30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82705e35, SUC_NE_lexicon_tags_2 },
    { 0xdc1c5e35, SUC_NE_lexicon_tags_2 },
    { 0xcffc5e37, SUC_NE_lexicon_tags_2 },
    { 0xf4d25e37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dc85e3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8085e3e, SUC_NE_lexicon_tags_2 },
    { 0x79f25e3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x224a5e42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38125e44, SUC_NE_lexicon_tags_3 },
    { 0x908e5e45, SUC_NE_lexicon_tags_2 },
    { 0x483c5e44, SUC_NE_lexicon_tags_2 },
    { 0x99c85e46, SUC_NE_lexicon_tags_18 },
    { 0x9c785e44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84e5e4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78525e54, SUC_NE_lexicon_tags_251 },
    { 0, NULL },
    { 0x31005e56, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xd2ee5e58, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9c5e62, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f85e6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcee45e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86ee5e70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b25e76, SUC_NE_lexicon_tags_2 },
    { 0x1fa45e76, SUC_NE_lexicon_tags_2 },
    { 0x33ae5e78, SUC_NE_lexicon_tags_2 },
    { 0x72e65e78, SUC_NE_lexicon_tags_2 },
    { 0xb27e5e79, SUC_NE_lexicon_tags_2 },
    { 0x572c5e77, SUC_NE_lexicon_tags_2 },
    { 0xe0fa5e7c, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88305e80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3c5e84, SUC_NE_lexicon_tags_2 },
    { 0xddd65e84, SUC_NE_lexicon_tags_2 },
    { 0x93bc5e86, SUC_NE_lexicon_tags_2 },
    { 0x82fc5e87, SUC_NE_lexicon_tags_2 },
    { 0x2a4c5e88, SUC_NE_lexicon_tags_2 },
    { 0x73d25e89, SUC_NE_lexicon_tags_2 },
    { 0xe9a85e8a, SUC_NE_lexicon_tags_2 },
    { 0x5c765e8b, SUC_NE_lexicon_tags_2 },
    { 0x2b105e8c, SUC_NE_lexicon_tags_4 },
    { 0x57b05e85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82445e92, SUC_NE_lexicon_tags_2 },
    { 0xbbd85e92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa165e96, SUC_NE_lexicon_tags_2 },
    { 0xbe45e97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d25e9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe265e9e, SUC_NE_lexicon_tags_17 },
    { 0x67905e9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb725ea4, SUC_NE_lexicon_tags_2 },
    { 0xacb45ea4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3925ea7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6a5ea9, SUC_NE_lexicon_tags_2 },
    { 0xc7625ea9, SUC_NE_lexicon_tags_2 },
    { 0x69065eab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf425eae, SUC_NE_lexicon_tags_2 },
    { 0x60e45eaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f345eb1, SUC_NE_lexicon_tags_32 },
    { 0x6c8c5eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x464e5eb4, SUC_NE_lexicon_tags_14 },
    { 0x3ab85eb4, SUC_NE_lexicon_tags_2 },
    { 0xbcdc5eb6, SUC_NE_lexicon_tags_2 },
    { 0x5eb05eb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce65ebc, SUC_NE_lexicon_tags_3 },
    { 0x1ffa5ebd, SUC_NE_lexicon_tags_2 },
    { 0x23a45ebd, SUC_NE_lexicon_tags_2 },
    { 0xcc305ebe, SUC_NE_lexicon_tags_2 },
    { 0x218a5ec0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85685ec2, SUC_NE_lexicon_tags_2 },
    { 0x581e5ec3, SUC_NE_lexicon_tags_220 },
    { 0x21ac5ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0c5ec7, SUC_NE_lexicon_tags_2 },
    { 0x171c5ec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed245ecb, SUC_NE_lexicon_tags_2 },
    { 0xe2a45ecc, SUC_NE_lexicon_tags_2 },
    { 0xe1a25ecd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5e5ed2, SUC_NE_lexicon_tags_2 },
    { 0xb4665ed3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac25ed7, SUC_NE_lexicon_tags_2 },
    { 0x2b285ed7, SUC_NE_lexicon_tags_2 },
    { 0xaeba5ed9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x613e5edc, SUC_NE_lexicon_tags_2 },
    { 0xd9525edd, SUC_NE_lexicon_tags_2 },
    { 0x85d45ede, SUC_NE_lexicon_tags_2 },
    { 0x22fa5edf, SUC_NE_lexicon_tags_4 },
    { 0xb9865ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1525ee2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc25ee6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x475a5ee9, SUC_NE_lexicon_tags_2 },
    { 0xe8625ee9, SUC_NE_lexicon_tags_2 },
    { 0x281a5eea, SUC_NE_lexicon_tags_2 },
    { 0x27245eec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd745eef, SUC_NE_lexicon_tags_2 },
    { 0xe0aa5ef0, SUC_NE_lexicon_tags_2 },
    { 0x3ffa5ef1, SUC_NE_lexicon_tags_2 },
    { 0x4e1a5ef2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x63245ef4, SUC_NE_lexicon_tags_2 },
    { 0x7f2e5ef5, SUC_NE_lexicon_tags_2 },
    { 0x56925ef5, SUC_NE_lexicon_tags_2 },
    { 0x26c05ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6b25ef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8305efb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30ec5efd, SUC_NE_lexicon_tags_2 },
    { 0xb2465efe, SUC_NE_lexicon_tags_2 },
    { 0xd0145efe, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe045f01, SUC_NE_lexicon_tags_2 },
    { 0xf5805f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba85f08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3285f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7625f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71885f1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f85f21, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75145f24, SUC_NE_lexicon_tags_2 },
    { 0xe6645f24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96705f27, SUC_NE_lexicon_tags_2 },
    { 0xef9e5f28, SUC_NE_lexicon_tags_2 },
    { 0xdeaa5f29, SUC_NE_lexicon_tags_2 },
    { 0xa1b25f2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849a5f2f, SUC_NE_lexicon_tags_2 },
    { 0xabdc5f2f, SUC_NE_lexicon_tags_2 },
    { 0xc20e5f31, SUC_NE_lexicon_tags_2 },
    { 0x87745f32, SUC_NE_lexicon_tags_2 },
    { 0x25525f33, SUC_NE_lexicon_tags_2 },
    { 0xc5485f32, SUC_NE_lexicon_tags_2 },
    { 0xe5825f35, SUC_NE_lexicon_tags_2 },
    { 0xcf05f36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fe5f3a, SUC_NE_lexicon_tags_2 },
    { 0xe1785f3a, SUC_NE_lexicon_tags_2 },
    { 0x35ac5f3c, SUC_NE_lexicon_tags_1 },
    { 0x88d25f3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1225f3f, SUC_NE_lexicon_tags_2 },
    { 0xed125f40, SUC_NE_lexicon_tags_178 },
    { 0xe1065f41, SUC_NE_lexicon_tags_2 },
    { 0x395a5f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc91c5f44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13525f46, SUC_NE_lexicon_tags_2 },
    { 0x4ed45f47, SUC_NE_lexicon_tags_2 },
    { 0x67fe5f48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3705f4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf25f55, SUC_NE_lexicon_tags_7 },
    { 0xe4325f56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c325f59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84c5f5c, SUC_NE_lexicon_tags_2 },
    { 0x75ec5f5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f425f60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4e5f63, SUC_NE_lexicon_tags_2 },
    { 0x3cc65f64, SUC_NE_lexicon_tags_2 },
    { 0x55125f65, SUC_NE_lexicon_tags_2 },
    { 0x4fc65f65, SUC_NE_lexicon_tags_2 },
    { 0x12b25f67, SUC_NE_lexicon_tags_2 },
    { 0xac0a5f66, SUC_NE_lexicon_tags_2 },
    { 0x3db45f63, SUC_NE_lexicon_tags_2 },
    { 0xaa585f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4c25f6c, SUC_NE_lexicon_tags_2 },
    { 0x660c5f6d, SUC_NE_lexicon_tags_0 },
    { 0x8a0e5f6d, SUC_NE_lexicon_tags_4 },
    { 0x2e4e5f6f, SUC_NE_lexicon_tags_2 },
    { 0x62165f6e, SUC_NE_lexicon_tags_2 },
    { 0x5305f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x924e5f74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf305f77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee4c5f79, SUC_NE_lexicon_tags_2 },
    { 0xcbc05f79, SUC_NE_lexicon_tags_2 },
    { 0x6bba5f7b, SUC_NE_lexicon_tags_4 },
    { 0x90da5f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26c5f7f, SUC_NE_lexicon_tags_2 },
    { 0x86b25f7f, SUC_NE_lexicon_tags_2 },
    { 0x1aa45f81, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1945f85, SUC_NE_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a5f89, SUC_NE_lexicon_tags_2 },
    { 0x78565f8a, SUC_NE_lexicon_tags_7 },
    { 0x4505f8a, SUC_NE_lexicon_tags_7 },
    { 0x2f345f8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74865f8f, SUC_NE_lexicon_tags_2 },
    { 0x66805f8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafc5f93, SUC_NE_lexicon_tags_2 },
    { 0x29025f94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4e85f96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de45f99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa785f9b, SUC_NE_lexicon_tags_2 },
    { 0x473e5f9b, SUC_NE_lexicon_tags_2 },
    { 0x4f705f9b, SUC_NE_lexicon_tags_2 },
    { 0x274a5f9b, SUC_NE_lexicon_tags_2 },
    { 0x9bec5f9e, SUC_NE_lexicon_tags_2 },
    { 0x95e45fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb285fa2, SUC_NE_lexicon_tags_2 },
    { 0x16a25fa3, SUC_NE_lexicon_tags_2 },
    { 0x3cc45fa4, SUC_NE_lexicon_tags_2 },
    { 0xcc7a5fa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb8c5fa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x968c5fa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dde5fad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93e5fb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd8e5fb2, SUC_NE_lexicon_tags_4 },
    { 0x8a645fb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d85fb6, SUC_NE_lexicon_tags_2 },
    { 0x2d9c5fb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a05fbc, SUC_NE_lexicon_tags_2 },
    { 0x7a9a5fbc, SUC_NE_lexicon_tags_2 },
    { 0x46485fbe, SUC_NE_lexicon_tags_2 },
    { 0x9cc65fbe, SUC_NE_lexicon_tags_7 },
    { 0x5f505fc0, SUC_NE_lexicon_tags_2 },
    { 0x1b065fc0, SUC_NE_lexicon_tags_2 },
    { 0x30945fc2, SUC_NE_lexicon_tags_2 },
    { 0xff2e5fc3, SUC_NE_lexicon_tags_2 },
    { 0xc95e5fc3, SUC_NE_lexicon_tags_21 },
    { 0x64005fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9345fcb, SUC_NE_lexicon_tags_7 },
    { 0x4bda5fcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3885fd0, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8465fd4, SUC_NE_lexicon_tags_2 },
    { 0x17ce5fd4, SUC_NE_lexicon_tags_2 },
    { 0xbd685fd6, SUC_NE_lexicon_tags_2 },
    { 0xca805fd7, SUC_NE_lexicon_tags_2 },
    { 0x576a5fd6, SUC_NE_lexicon_tags_2 },
    { 0x7e585fd7, SUC_NE_lexicon_tags_2 },
    { 0x94365fd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc25fdd, SUC_NE_lexicon_tags_2 },
    { 0x53fe5fdd, SUC_NE_lexicon_tags_2 },
    { 0x95305fdf, SUC_NE_lexicon_tags_2 },
    { 0x109e5fe0, SUC_NE_lexicon_tags_2 },
    { 0xe2ae5fe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44845fe6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b65fea, SUC_NE_lexicon_tags_2 },
    { 0xd1c65fea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x818c5fee, SUC_NE_lexicon_tags_2 },
    { 0x48545fee, SUC_NE_lexicon_tags_0 },
    { 0x5b265fee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e5ff6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x43ee5ff8, SUC_NE_lexicon_tags_2 },
    { 0xadc5ff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18785ffb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca5fff, SUC_NE_lexicon_tags_30 },
    { 0xd3f46000, SUC_NE_lexicon_tags_2 },
    { 0x10566001, SUC_NE_lexicon_tags_2 },
    { 0xcb686000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc6006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72cc6008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe378600a, SUC_NE_lexicon_tags_2 },
    { 0x849a600a, SUC_NE_lexicon_tags_2 },
    { 0xb38c600c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80286010, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb3466012, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86626014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb866016, SUC_NE_lexicon_tags_2 },
    { 0xbdf86017, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777e601d, SUC_NE_lexicon_tags_7 },
    { 0x7e18601e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36be6021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ce6029, SUC_NE_lexicon_tags_2 },
    { 0x442602a, SUC_NE_lexicon_tags_14 },
    { 0x8f9e602b, SUC_NE_lexicon_tags_2 },
    { 0x1d90602c, SUC_NE_lexicon_tags_24 },
    { 0xee72602d, SUC_NE_lexicon_tags_2 },
    { 0xa8786029, SUC_NE_lexicon_tags_2 },
    { 0xa18602f, SUC_NE_lexicon_tags_2 },
    { 0x5f2e6030, SUC_NE_lexicon_tags_2 },
    { 0xa9026030, SUC_NE_lexicon_tags_2 },
    { 0x81846032, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29286038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823c603f, SUC_NE_lexicon_tags_2 },
    { 0x34b86040, SUC_NE_lexicon_tags_36 },
    { 0x385a6041, SUC_NE_lexicon_tags_2 },
    { 0xe0d26041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78a6045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11146047, SUC_NE_lexicon_tags_2 },
    { 0xaf06048, SUC_NE_lexicon_tags_2 },
    { 0x75746047, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f8604c, SUC_NE_lexicon_tags_19 },
    { 0xcf2c604c, SUC_NE_lexicon_tags_2 },
    { 0x2436604c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbca6050, SUC_NE_lexicon_tags_2 },
    { 0xd7ac6051, SUC_NE_lexicon_tags_7 },
    { 0x66ae6052, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc86056, SUC_NE_lexicon_tags_2 },
    { 0xde8e6057, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e605a, SUC_NE_lexicon_tags_2 },
    { 0xa6c605b, SUC_NE_lexicon_tags_2 },
    { 0xcda0605c, SUC_NE_lexicon_tags_2 },
    { 0x6f2a605d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35fa605f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92126062, SUC_NE_lexicon_tags_2 },
    { 0xf0566063, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c0a6065, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ca6069, SUC_NE_lexicon_tags_2 },
    { 0xe64a6069, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aa606e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c086073, SUC_NE_lexicon_tags_2 },
    { 0x20ba6074, SUC_NE_lexicon_tags_2 },
    { 0xf0ec6073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f346077, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb90a6079, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1e607e, SUC_NE_lexicon_tags_2 },
    { 0x701a607e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ba6082, SUC_NE_lexicon_tags_1 },
    { 0x5a426083, SUC_NE_lexicon_tags_2 },
    { 0x48706084, SUC_NE_lexicon_tags_2 },
    { 0x99cc6085, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x570e6088, SUC_NE_lexicon_tags_2 },
    { 0x7b4e6089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8710608b, SUC_NE_lexicon_tags_2 },
    { 0x32fa608c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce7e608e, SUC_NE_lexicon_tags_17 },
    { 0x17e4608f, SUC_NE_lexicon_tags_2 },
    { 0xcb46090, SUC_NE_lexicon_tags_2 },
    { 0x7506608f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fd26093, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a306097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3166099, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xe9e6609b, SUC_NE_lexicon_tags_2 },
    { 0xa9c6609c, SUC_NE_lexicon_tags_2 },
    { 0x344c609c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9204609f, SUC_NE_lexicon_tags_2 },
    { 0x71dc60a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5660a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784260a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d060ad, SUC_NE_lexicon_tags_7 },
    { 0xded460ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a5060b0, SUC_NE_lexicon_tags_3 },
    { 0x424060b1, SUC_NE_lexicon_tags_2 },
    { 0xb42060b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x704c60b4, SUC_NE_lexicon_tags_4 },
    { 0xbcbc60b4, SUC_NE_lexicon_tags_2 },
    { 0x399260b6, SUC_NE_lexicon_tags_2 },
    { 0xf4e260b4, SUC_NE_lexicon_tags_2 },
    { 0x635c60b8, SUC_NE_lexicon_tags_2 },
    { 0x6460b9, SUC_NE_lexicon_tags_2 },
    { 0x622e60b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332460c3, SUC_NE_lexicon_tags_7 },
    { 0x494060c4, SUC_NE_lexicon_tags_2 },
    { 0xa36a60c4, SUC_NE_lexicon_tags_2 },
    { 0xa65260c6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242c60cb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ea60d4, SUC_NE_lexicon_tags_2 },
    { 0x2d3860d4, SUC_NE_lexicon_tags_2 },
    { 0x93260d6, SUC_NE_lexicon_tags_2 },
    { 0x666660d7, SUC_NE_lexicon_tags_14 },
    { 0xa2ca60d7, SUC_NE_lexicon_tags_2 },
    { 0xd1a460d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b460dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d460e0, SUC_NE_lexicon_tags_2 },
    { 0xa4d260e0, SUC_NE_lexicon_tags_7 },
    { 0x7d0860e1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a060e7, SUC_NE_lexicon_tags_4 },
    { 0x7b6a60e8, SUC_NE_lexicon_tags_2 },
    { 0x800e60e8, SUC_NE_lexicon_tags_2 },
    { 0x98ae60e7, SUC_NE_lexicon_tags_2 },
    { 0x644e60eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5060ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb72c60f1, SUC_NE_lexicon_tags_2 },
    { 0x756660f1, SUC_NE_lexicon_tags_2 },
    { 0x43d660f3, SUC_NE_lexicon_tags_2 },
    { 0x1a2260f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1860fb, SUC_NE_lexicon_tags_2 },
    { 0x32060fc, SUC_NE_lexicon_tags_2 },
    { 0x4e7060fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e926100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce766103, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79506108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1a610e, SUC_NE_lexicon_tags_3 },
    { 0xa3c610f, SUC_NE_lexicon_tags_2 },
    { 0x5622610e, SUC_NE_lexicon_tags_2 },
    { 0xb6a86111, SUC_NE_lexicon_tags_0 },
    { 0x8f1a6111, SUC_NE_lexicon_tags_2 },
    { 0x58b26112, SUC_NE_lexicon_tags_2 },
    { 0x1bf86114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x802c6116, SUC_NE_lexicon_tags_2 },
    { 0x2a6c6117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e66119, SUC_NE_lexicon_tags_2 },
    { 0xd7b46119, SUC_NE_lexicon_tags_2 },
    { 0xb2ea611b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d02611f, SUC_NE_lexicon_tags_2 },
    { 0xbc40611f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13a6123, SUC_NE_lexicon_tags_40 },
    { 0xd7a06123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997a6129, SUC_NE_lexicon_tags_2 },
    { 0x641c6129, SUC_NE_lexicon_tags_2 },
    { 0xb6d6612b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xead4612d, SUC_NE_lexicon_tags_2 },
    { 0x5d52612e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe146131, SUC_NE_lexicon_tags_2 },
    { 0xb6266132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b026138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90f4613a, SUC_NE_lexicon_tags_2 },
    { 0xd77a613b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3338613e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ba6142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53da6147, SUC_NE_lexicon_tags_2 },
    { 0xcd146148, SUC_NE_lexicon_tags_2 },
    { 0x973e6149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7904614f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5e6152, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bf26154, SUC_NE_lexicon_tags_2 },
    { 0xf6a06155, SUC_NE_lexicon_tags_2 },
    { 0xf7666155, SUC_NE_lexicon_tags_2 },
    { 0x74586155, SUC_NE_lexicon_tags_2 },
    { 0x957e6158, SUC_NE_lexicon_tags_2 },
    { 0xe9f66158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a38615b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabe6160, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72466163, SUC_NE_lexicon_tags_0 },
    { 0x49be6163, SUC_NE_lexicon_tags_2 },
    { 0x42c86163, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40526167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf666169, SUC_NE_lexicon_tags_2 },
    { 0xc8a4616a, SUC_NE_lexicon_tags_2 },
    { 0x46da616a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x96b4616d, SUC_NE_lexicon_tags_2 },
    { 0x9b4c616d, SUC_NE_lexicon_tags_2 },
    { 0x6bcc616f, SUC_NE_lexicon_tags_2 },
    { 0x40946170, SUC_NE_lexicon_tags_2 },
    { 0x7fee6171, SUC_NE_lexicon_tags_2 },
    { 0xcff46170, SUC_NE_lexicon_tags_2 },
    { 0x275e6173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf586175, SUC_NE_lexicon_tags_2 },
    { 0xc1546176, SUC_NE_lexicon_tags_2 },
    { 0xe4a26175, SUC_NE_lexicon_tags_2 },
    { 0x47346178, SUC_NE_lexicon_tags_2 },
    { 0xc3546178, SUC_NE_lexicon_tags_11 },
    { 0x6d3a6179, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xebba617d, SUC_NE_lexicon_tags_27 },
    { 0x8180617e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2866184, SUC_NE_lexicon_tags_2 },
    { 0xb3c66185, SUC_NE_lexicon_tags_2 },
    { 0x4ae46186, SUC_NE_lexicon_tags_2 },
    { 0x579a6186, SUC_NE_lexicon_tags_2 },
    { 0x6b226188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1900618a, SUC_NE_lexicon_tags_2 },
    { 0x2e4618b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8a618f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1a6192, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dc86194, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35206196, SUC_NE_lexicon_tags_19 },
    { 0x8b166196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f86199, SUC_NE_lexicon_tags_2 },
    { 0xf29a6199, SUC_NE_lexicon_tags_2 },
    { 0x6e8e619a, SUC_NE_lexicon_tags_2 },
    { 0xd46619c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6fe619e, SUC_NE_lexicon_tags_2 },
    { 0xe350619e, SUC_NE_lexicon_tags_2 },
    { 0xabba61a0, SUC_NE_lexicon_tags_157 },
    { 0x4fc661a1, SUC_NE_lexicon_tags_2 },
    { 0x698619e, SUC_NE_lexicon_tags_22 },
    { 0xe55261a3, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391061a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7e61ae, SUC_NE_lexicon_tags_2 },
    { 0x99cc61af, SUC_NE_lexicon_tags_2 },
    { 0xa90461ae, SUC_NE_lexicon_tags_2 },
    { 0x9ea261b1, SUC_NE_lexicon_tags_2 },
    { 0xc0b461ae, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d661b7, SUC_NE_lexicon_tags_2 },
    { 0x59cc61b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0061bb, SUC_NE_lexicon_tags_17 },
    { 0xf6d261bb, SUC_NE_lexicon_tags_2 },
    { 0x8cd661bc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e61c1, SUC_NE_lexicon_tags_1 },
    { 0xd37061c2, SUC_NE_lexicon_tags_3 },
    { 0x9ea61c3, SUC_NE_lexicon_tags_2 },
    { 0x8d2061c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819261c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6661cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe861d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfc61d4, SUC_NE_lexicon_tags_2 },
    { 0x7f2261d5, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdaa61d7, SUC_NE_lexicon_tags_1 },
    { 0xd49a61d8, SUC_NE_lexicon_tags_1 },
    { 0xba861d9, SUC_NE_lexicon_tags_2 },
    { 0x7a061da, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aa61de, SUC_NE_lexicon_tags_2 },
    { 0xd99461df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42c61e1, SUC_NE_lexicon_tags_34 },
    { 0xba8661e2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955661e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19461e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51a261eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x150261ee, SUC_NE_lexicon_tags_2 },
    { 0xad3061ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab261f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d3e61f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353c61fc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x462261ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e06206, SUC_NE_lexicon_tags_50 },
    { 0xdf406207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88620e, SUC_NE_lexicon_tags_7 },
    { 0x1bdc620e, SUC_NE_lexicon_tags_2 },
    { 0xeaae620f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40006215, SUC_NE_lexicon_tags_2 },
    { 0x117a6216, SUC_NE_lexicon_tags_2 },
    { 0xb4226217, SUC_NE_lexicon_tags_2 },
    { 0xfb826217, SUC_NE_lexicon_tags_2 },
    { 0x2dc06219, SUC_NE_lexicon_tags_2 },
    { 0xefe66218, SUC_NE_lexicon_tags_2 },
    { 0xe0b06219, SUC_NE_lexicon_tags_2 },
    { 0x5a1e621c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64c621e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f9e6220, SUC_NE_lexicon_tags_2 },
    { 0x56be6220, SUC_NE_lexicon_tags_2 },
    { 0x86126221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2a6228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76d6622a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed46235, SUC_NE_lexicon_tags_2 },
    { 0x78e06236, SUC_NE_lexicon_tags_2 },
    { 0x78266237, SUC_NE_lexicon_tags_2 },
    { 0xccf06235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd51a623a, SUC_NE_lexicon_tags_19 },
    { 0x211c623a, SUC_NE_lexicon_tags_2 },
    { 0x9144623b, SUC_NE_lexicon_tags_2 },
    { 0xcc7a623d, SUC_NE_lexicon_tags_2 },
    { 0x39e8623a, SUC_NE_lexicon_tags_7 },
    { 0x90bc623f, SUC_NE_lexicon_tags_3 },
    { 0xe59a6240, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e66244, SUC_NE_lexicon_tags_2 },
    { 0x70466245, SUC_NE_lexicon_tags_2 },
    { 0x32ec6246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3386624e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60aa6250, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4de6253, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f06256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe208625b, SUC_NE_lexicon_tags_2 },
    { 0x72ea625b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e46265, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d86268, SUC_NE_lexicon_tags_7 },
    { 0x54a26269, SUC_NE_lexicon_tags_2 },
    { 0x4390626a, SUC_NE_lexicon_tags_2 },
    { 0x6172626a, SUC_NE_lexicon_tags_2 },
    { 0x77c8626c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x82da626e, SUC_NE_lexicon_tags_1 },
    { 0x597a626f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67786273, SUC_NE_lexicon_tags_2 },
    { 0x8cb66273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e06277, SUC_NE_lexicon_tags_5 },
    { 0xf9246278, SUC_NE_lexicon_tags_2 },
    { 0x78166279, SUC_NE_lexicon_tags_2 },
    { 0x5c186279, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ea627f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7fc6281, SUC_NE_lexicon_tags_2 },
    { 0x7d1e6281, SUC_NE_lexicon_tags_2 },
    { 0xd7966283, SUC_NE_lexicon_tags_1 },
    { 0x1d186284, SUC_NE_lexicon_tags_2 },
    { 0x381c6281, SUC_NE_lexicon_tags_2 },
    { 0x4c666282, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6a628a, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0xdf54628c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f226291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6c6295, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e6629a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b862a1, SUC_NE_lexicon_tags_18 },
    { 0x42b462a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd68862a4, SUC_NE_lexicon_tags_2 },
    { 0xb08062a4, SUC_NE_lexicon_tags_2 },
    { 0xd29662a6, SUC_NE_lexicon_tags_2 },
    { 0x8e3862a6, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a462aa, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d462ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdc62b0, SUC_NE_lexicon_tags_7 },
    { 0x10a662b0, SUC_NE_lexicon_tags_2 },
    { 0xb24462b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942862b6, SUC_NE_lexicon_tags_23 },
    { 0x595e62b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48a62bb, SUC_NE_lexicon_tags_18 },
    { 0x710662bc, SUC_NE_lexicon_tags_2 },
    { 0x8b262bd, SUC_NE_lexicon_tags_51 },
    { 0x5b0262be, SUC_NE_lexicon_tags_2 },
    { 0x493862bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x543062c1, SUC_NE_lexicon_tags_2 },
    { 0xaa262c2, SUC_NE_lexicon_tags_2 },
    { 0x3a1a62c2, SUC_NE_lexicon_tags_2 },
    { 0x6c4462c4, SUC_NE_lexicon_tags_83 },
    { 0x8e6e62c3, SUC_NE_lexicon_tags_2 },
    { 0x83f262c6, SUC_NE_lexicon_tags_2 },
    { 0x108a62c5, SUC_NE_lexicon_tags_2 },
    { 0xac7e62c8, SUC_NE_lexicon_tags_18 },
    { 0xdd1262c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae662cb, SUC_NE_lexicon_tags_3 },
    { 0x62cb, SUC_NE_lexicon_tags_2 },
    { 0x3c6a62cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a62d3, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x15bc62d5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd2d462d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222462db, SUC_NE_lexicon_tags_2 },
    { 0x814c62dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94a62e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2262e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63662f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b6262f4, SUC_NE_lexicon_tags_2 },
    { 0xea3462f5, SUC_NE_lexicon_tags_2 },
    { 0x862662f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a9262f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23b862fa, SUC_NE_lexicon_tags_2 },
    { 0x21d062fb, SUC_NE_lexicon_tags_2 },
    { 0xdfca62fb, SUC_NE_lexicon_tags_2 },
    { 0x5d1262fb, SUC_NE_lexicon_tags_7 },
    { 0x9a2862fd, SUC_NE_lexicon_tags_2 },
    { 0xe34462ff, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0a6302, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3226309, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fba630c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78f2630e, SUC_NE_lexicon_tags_7 },
    { 0x29c630f, SUC_NE_lexicon_tags_2 },
    { 0x89a06310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0e6313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea846316, SUC_NE_lexicon_tags_7 },
    { 0x8a4e6317, SUC_NE_lexicon_tags_4 },
    { 0x38c46317, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa354631a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x294e631d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x540631f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2426321, SUC_NE_lexicon_tags_2 },
    { 0x3bf06321, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb66329, SUC_NE_lexicon_tags_2 },
    { 0x8a066329, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bc0632c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0e6331, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d986334, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xaff86336, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a8633a, SUC_NE_lexicon_tags_2 },
    { 0xbbc0633a, SUC_NE_lexicon_tags_1 },
    { 0x6dd4633a, SUC_NE_lexicon_tags_2 },
    { 0xee9e633b, SUC_NE_lexicon_tags_2 },
    { 0xe68e633e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38e6340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47046346, SUC_NE_lexicon_tags_1 },
    { 0x11fe6346, SUC_NE_lexicon_tags_7 },
    { 0x5c106348, SUC_NE_lexicon_tags_2 },
    { 0x6dfc6348, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd336634b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb864634d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90486350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18066358, SUC_NE_lexicon_tags_2 },
    { 0xda8a6358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x139a635b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2950635e, SUC_NE_lexicon_tags_2 },
    { 0x6d18635e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc126361, SUC_NE_lexicon_tags_2 },
    { 0x6d506362, SUC_NE_lexicon_tags_4 },
    { 0x7d666363, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5606366, SUC_NE_lexicon_tags_2 },
    { 0xd2026367, SUC_NE_lexicon_tags_2 },
    { 0x53ae6367, SUC_NE_lexicon_tags_2 },
    { 0xb366369, SUC_NE_lexicon_tags_2 },
    { 0x18666366, SUC_NE_lexicon_tags_17 },
    { 0xc0ca6367, SUC_NE_lexicon_tags_2 },
    { 0xab44636c, SUC_NE_lexicon_tags_2 },
    { 0xdc4e636c, SUC_NE_lexicon_tags_2 },
    { 0xb576636d, SUC_NE_lexicon_tags_1 },
    { 0x65c2636b, SUC_NE_lexicon_tags_2 },
    { 0x9daa6370, SUC_NE_lexicon_tags_2 },
    { 0x474c6371, SUC_NE_lexicon_tags_2 },
    { 0x88a6372, SUC_NE_lexicon_tags_2 },
    { 0xce846371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182c6377, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64637a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21e0637c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56886381, SUC_NE_lexicon_tags_91 },
    { 0xb67c6382, SUC_NE_lexicon_tags_7 },
    { 0x1d3a6381, SUC_NE_lexicon_tags_2 },
    { 0xf4be6381, SUC_NE_lexicon_tags_2 },
    { 0x25926382, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25706388, SUC_NE_lexicon_tags_2 },
    { 0x70f06389, SUC_NE_lexicon_tags_2 },
    { 0xc0246389, SUC_NE_lexicon_tags_7 },
    { 0x6bec638b, SUC_NE_lexicon_tags_2 },
    { 0x73e638b, SUC_NE_lexicon_tags_2 },
    { 0x148638b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab2638f, SUC_NE_lexicon_tags_2 },
    { 0x63f46390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd86393, SUC_NE_lexicon_tags_2 },
    { 0x994a6394, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x27586397, SUC_NE_lexicon_tags_2 },
    { 0xf2226398, SUC_NE_lexicon_tags_18 },
    { 0xff766398, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80639d, SUC_NE_lexicon_tags_1 },
    { 0xd198639d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd1063a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x869063a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe8663a4, SUC_NE_lexicon_tags_2 },
    { 0xec0863a5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x843a63a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad263ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e663b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239e63b6, SUC_NE_lexicon_tags_2 },
    { 0xf25063b6, SUC_NE_lexicon_tags_2 },
    { 0x67c863b7, SUC_NE_lexicon_tags_2 },
    { 0xf76c63b9, SUC_NE_lexicon_tags_2 },
    { 0x423663ba, SUC_NE_lexicon_tags_2 },
    { 0x55b063bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e63be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83cc63c0, SUC_NE_lexicon_tags_2 },
    { 0x450e63c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x190463c3, SUC_NE_lexicon_tags_2 },
    { 0xea3063c4, SUC_NE_lexicon_tags_2 },
    { 0x737c63c4, SUC_NE_lexicon_tags_2 },
    { 0xd22263c3, SUC_NE_lexicon_tags_2 },
    { 0xcb863c5, SUC_NE_lexicon_tags_2 },
    { 0xf5ce63c8, SUC_NE_lexicon_tags_2 },
    { 0x2c4263c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6263cb, SUC_NE_lexicon_tags_2 },
    { 0xbb9663cc, SUC_NE_lexicon_tags_2 },
    { 0xb87c63cd, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6463d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af863d5, SUC_NE_lexicon_tags_2 },
    { 0xf9e63d6, SUC_NE_lexicon_tags_2 },
    { 0x814863d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7bc63d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59063dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2e63e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b663ec, SUC_NE_lexicon_tags_2 },
    { 0xf75063ed, SUC_NE_lexicon_tags_2 },
    { 0xf9be63ee, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad063f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171463fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x452863fd, SUC_NE_lexicon_tags_2 },
    { 0xf49863fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5606400, SUC_NE_lexicon_tags_2 },
    { 0x1b326401, SUC_NE_lexicon_tags_2 },
    { 0xd4a66401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42e6404, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80386406, SUC_NE_lexicon_tags_2 },
    { 0x551a6406, SUC_NE_lexicon_tags_2 },
    { 0x35ca6406, SUC_NE_lexicon_tags_2 },
    { 0xa7106409, SUC_NE_lexicon_tags_2 },
    { 0xc51e640a, SUC_NE_lexicon_tags_2 },
    { 0x793c640b, SUC_NE_lexicon_tags_2 },
    { 0xb856640c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20126412, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5606414, SUC_NE_lexicon_tags_2 },
    { 0xe44a6414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c06418, SUC_NE_lexicon_tags_2 },
    { 0x7db46418, SUC_NE_lexicon_tags_0 },
    { 0xdc906418, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4a40641c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa146425, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdd6c6427, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cc642a, SUC_NE_lexicon_tags_2 },
    { 0x45ba642a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3440642d, SUC_NE_lexicon_tags_1 },
    { 0x460e642d, SUC_NE_lexicon_tags_2 },
    { 0x9664642e, SUC_NE_lexicon_tags_2 },
    { 0x22986430, SUC_NE_lexicon_tags_2 },
    { 0x9e2e6431, SUC_NE_lexicon_tags_2 },
    { 0xc2aa6432, SUC_NE_lexicon_tags_2 },
    { 0x7c566432, SUC_NE_lexicon_tags_4 },
    { 0xbffc6432, SUC_NE_lexicon_tags_34 },
    { 0x51906433, SUC_NE_lexicon_tags_7 },
    { 0x296c6436, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe706438, SUC_NE_lexicon_tags_2 },
    { 0x794c6438, SUC_NE_lexicon_tags_2 },
    { 0x6abc6439, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x925c643d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7ce643f, SUC_NE_lexicon_tags_2 },
    { 0xd8bc6440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb566444, SUC_NE_lexicon_tags_2 },
    { 0x30006445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e46448, SUC_NE_lexicon_tags_3 },
    { 0x6a5e6449, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b36644b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91126450, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce86457, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dce6459, SUC_NE_lexicon_tags_2 },
    { 0x4ffe6459, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d8645d, SUC_NE_lexicon_tags_2 },
    { 0x8f08645e, SUC_NE_lexicon_tags_2 },
    { 0x864c645e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6114646e, SUC_NE_lexicon_tags_2 },
    { 0xf788646f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d086472, SUC_NE_lexicon_tags_13 },
    { 0x6e046472, SUC_NE_lexicon_tags_2 },
    { 0xa36c6474, SUC_NE_lexicon_tags_2 },
    { 0x825c6473, SUC_NE_lexicon_tags_2 },
    { 0x7cd66474, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2c647a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x348c647c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fe6481, SUC_NE_lexicon_tags_7 },
    { 0x5e046482, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc386648a, SUC_NE_lexicon_tags_13 },
    { 0xe12648a, SUC_NE_lexicon_tags_2 },
    { 0x4d98648b, SUC_NE_lexicon_tags_2 },
    { 0x67ea648c, SUC_NE_lexicon_tags_2 },
    { 0x1e26648e, SUC_NE_lexicon_tags_2 },
    { 0xd208648f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcea86491, SUC_NE_lexicon_tags_2 },
    { 0x71d26491, SUC_NE_lexicon_tags_2 },
    { 0x53da6493, SUC_NE_lexicon_tags_2 },
    { 0x6b6a6494, SUC_NE_lexicon_tags_24 },
    { 0x413a6493, SUC_NE_lexicon_tags_0 },
    { 0x58f66496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e649a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f664a0, SUC_NE_lexicon_tags_2 },
    { 0x73a064a1, SUC_NE_lexicon_tags_2 },
    { 0xb8ac64a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1264a6, SUC_NE_lexicon_tags_2 },
    { 0x83a064a6, SUC_NE_lexicon_tags_2 },
    { 0xabaa64a8, SUC_NE_lexicon_tags_2 },
    { 0x8a64a9, SUC_NE_lexicon_tags_2 },
    { 0x251064aa, SUC_NE_lexicon_tags_2 },
    { 0x79f664ab, SUC_NE_lexicon_tags_2 },
    { 0xfede64ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66864af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8264b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d464b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadba64bb, SUC_NE_lexicon_tags_24 },
    { 0x801a64bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84264c0, SUC_NE_lexicon_tags_3 },
    { 0xb50864c1, SUC_NE_lexicon_tags_2 },
    { 0xbc1264c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6e864c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7c64cb, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ac264cd, SUC_NE_lexicon_tags_2 },
    { 0xce5a64ce, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xd07664d0, SUC_NE_lexicon_tags_18 },
    { 0x6dc64d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1caa64d3, SUC_NE_lexicon_tags_0 },
    { 0x539064d4, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x43b864d6, SUC_NE_lexicon_tags_4 },
    { 0xb2f264d7, SUC_NE_lexicon_tags_18 },
    { 0xaab664d6, SUC_NE_lexicon_tags_1 },
    { 0xcb4c64d9, SUC_NE_lexicon_tags_2 },
    { 0x3bce64da, SUC_NE_lexicon_tags_2 },
    { 0x22c864db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341c64df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b664e8, SUC_NE_lexicon_tags_4 },
    { 0x485c64e9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x784e64ec, SUC_NE_lexicon_tags_13 },
    { 0x80ca64ed, SUC_NE_lexicon_tags_2 },
    { 0x6a9664ed, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xca0e64f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e3464f2, SUC_NE_lexicon_tags_2 },
    { 0x598464f3, SUC_NE_lexicon_tags_2 },
    { 0x264064f4, SUC_NE_lexicon_tags_2 },
    { 0xfdb264f5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e264f9, SUC_NE_lexicon_tags_2 },
    { 0x4eb864fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x591a64fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac6503, SUC_NE_lexicon_tags_2 },
    { 0x1e3c6504, SUC_NE_lexicon_tags_13 },
    { 0xd3fa6505, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x789e6507, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dba6509, SUC_NE_lexicon_tags_2 },
    { 0x2fb86509, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e650d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a26510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94a66512, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb46515, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c6a6517, SUC_NE_lexicon_tags_2 },
    { 0x80aa6518, SUC_NE_lexicon_tags_2 },
    { 0xc9706519, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5a651c, SUC_NE_lexicon_tags_3 },
    { 0x5896651c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa056651f, SUC_NE_lexicon_tags_2 },
    { 0x76a8651f, SUC_NE_lexicon_tags_2 },
    { 0x2dea6521, SUC_NE_lexicon_tags_2 },
    { 0x44d86522, SUC_NE_lexicon_tags_2 },
    { 0x28ea6523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b06526, SUC_NE_lexicon_tags_2 },
    { 0x447e6527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a246529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9700652d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eca6532, SUC_NE_lexicon_tags_2 },
    { 0x27a46532, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c6a6535, SUC_NE_lexicon_tags_2 },
    { 0x86466536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56c06538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ac653a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb50653e, SUC_NE_lexicon_tags_2 },
    { 0xcae0653f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ea06541, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xec26543, SUC_NE_lexicon_tags_2 },
    { 0xc5346544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68a66546, SUC_NE_lexicon_tags_4 },
    { 0x55726546, SUC_NE_lexicon_tags_8 },
    { 0x99306548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x903c654a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a8654c, SUC_NE_lexicon_tags_4 },
    { 0x2b7c654d, SUC_NE_lexicon_tags_2 },
    { 0x3636654c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15946551, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbcd06553, SUC_NE_lexicon_tags_2 },
    { 0xebc46554, SUC_NE_lexicon_tags_2 },
    { 0x65706554, SUC_NE_lexicon_tags_2 },
    { 0xaaa86556, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf46559, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33bc655b, SUC_NE_lexicon_tags_2 },
    { 0x7da8655c, SUC_NE_lexicon_tags_4 },
    { 0xf90e655d, SUC_NE_lexicon_tags_2 },
    { 0x881e655e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e926560, SUC_NE_lexicon_tags_2 },
    { 0xe3fc6561, SUC_NE_lexicon_tags_2 },
    { 0xc7306562, SUC_NE_lexicon_tags_228 },
    { 0xe4f66563, SUC_NE_lexicon_tags_2 },
    { 0xa8c06564, SUC_NE_lexicon_tags_1 },
    { 0x32046565, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba606567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74d06569, SUC_NE_lexicon_tags_2 },
    { 0x81b06569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2656e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ca6571, SUC_NE_lexicon_tags_2 },
    { 0xbddc6572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60c6575, SUC_NE_lexicon_tags_7 },
    { 0x122e6576, SUC_NE_lexicon_tags_2 },
    { 0xfc726577, SUC_NE_lexicon_tags_4 },
    { 0x166a6576, SUC_NE_lexicon_tags_2 },
    { 0x28186579, SUC_NE_lexicon_tags_2 },
    { 0x3204657a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd83e657c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0206582, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fe6586, SUC_NE_lexicon_tags_2 },
    { 0x9b546587, SUC_NE_lexicon_tags_2 },
    { 0x765e6588, SUC_NE_lexicon_tags_2 },
    { 0x2b2e6587, SUC_NE_lexicon_tags_2 },
    { 0x97a658a, SUC_NE_lexicon_tags_2 },
    { 0x56fc658b, SUC_NE_lexicon_tags_4 },
    { 0x7e76658c, SUC_NE_lexicon_tags_2 },
    { 0x2f7a658d, SUC_NE_lexicon_tags_2 },
    { 0x1eaa658a, SUC_NE_lexicon_tags_2 },
    { 0x82d8658e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3166591, SUC_NE_lexicon_tags_1 },
    { 0x34dc6591, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba06595, SUC_NE_lexicon_tags_2 },
    { 0xa66595, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9c6599, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730c659e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7665a3, SUC_NE_lexicon_tags_2 },
    { 0x559465a4, SUC_NE_lexicon_tags_2 },
    { 0x86f265a3, SUC_NE_lexicon_tags_2 },
    { 0x255065a6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87e65ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2665b3, SUC_NE_lexicon_tags_2 },
    { 0x63d265b4, SUC_NE_lexicon_tags_5 },
    { 0x179265b5, SUC_NE_lexicon_tags_2 },
    { 0x5c8a65b5, SUC_NE_lexicon_tags_2 },
    { 0x2acc65b3, SUC_NE_lexicon_tags_3 },
    { 0x37cc65b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8a65c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295465cd, SUC_NE_lexicon_tags_2 },
    { 0x461a65ce, SUC_NE_lexicon_tags_2 },
    { 0x105c65ce, SUC_NE_lexicon_tags_32 },
    { 0x793465cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f265d3, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1065d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57665dd, SUC_NE_lexicon_tags_2 },
    { 0x5d3a65de, SUC_NE_lexicon_tags_2 },
    { 0xbe3465df, SUC_NE_lexicon_tags_2 },
    { 0xdb3065e0, SUC_NE_lexicon_tags_2 },
    { 0x20ec65e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13865e4, SUC_NE_lexicon_tags_2 },
    { 0x47a065e5, SUC_NE_lexicon_tags_2 },
    { 0xd2fa65e6, SUC_NE_lexicon_tags_2 },
    { 0x637065e7, SUC_NE_lexicon_tags_2 },
    { 0x82d465e8, SUC_NE_lexicon_tags_2 },
    { 0xd5865e9, SUC_NE_lexicon_tags_2 },
    { 0xad5c65e8, SUC_NE_lexicon_tags_2 },
    { 0x774465e9, SUC_NE_lexicon_tags_2 },
    { 0x67ee65ec, SUC_NE_lexicon_tags_2 },
    { 0x58c465ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e9a65ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1065f2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7bfc65f4, SUC_NE_lexicon_tags_2 },
    { 0x5a2e65f4, SUC_NE_lexicon_tags_2 },
    { 0x30e465f6, SUC_NE_lexicon_tags_32 },
    { 0x663865f7, SUC_NE_lexicon_tags_3 },
    { 0x973265f5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc65fb, SUC_NE_lexicon_tags_2 },
    { 0x299665fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1546600, SUC_NE_lexicon_tags_2 },
    { 0xee586600, SUC_NE_lexicon_tags_2 },
    { 0x232a6601, SUC_NE_lexicon_tags_2 },
    { 0x18be6603, SUC_NE_lexicon_tags_2 },
    { 0xe7c46601, SUC_NE_lexicon_tags_2 },
    { 0x44286605, SUC_NE_lexicon_tags_2 },
    { 0xbc7e6605, SUC_NE_lexicon_tags_2 },
    { 0x539e6607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2786660a, SUC_NE_lexicon_tags_2 },
    { 0x3e82660b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1e660e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de661b, SUC_NE_lexicon_tags_2 },
    { 0xfe86661c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba661f, SUC_NE_lexicon_tags_4 },
    { 0x750e6620, SUC_NE_lexicon_tags_2 },
    { 0x82f86621, SUC_NE_lexicon_tags_2 },
    { 0x83126621, SUC_NE_lexicon_tags_3 },
    { 0xd0d86621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63ae6625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d46628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe44c662a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b46630, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc06633, SUC_NE_lexicon_tags_2 },
    { 0x4186633, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x352e6636, SUC_NE_lexicon_tags_2 },
    { 0x88226637, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73d26639, SUC_NE_lexicon_tags_2 },
    { 0x9b746639, SUC_NE_lexicon_tags_2 },
    { 0xc270663b, SUC_NE_lexicon_tags_2 },
    { 0x4458663c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc65a663e, SUC_NE_lexicon_tags_2 },
    { 0x9f22663f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x565a6642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36286645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38be664c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x4066664e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b186651, SUC_NE_lexicon_tags_2 },
    { 0x18a66652, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0406656, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x71806659, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e36665b, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12e665e, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc6661, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e06664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14a26666, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf938666b, SUC_NE_lexicon_tags_2 },
    { 0x346c666c, SUC_NE_lexicon_tags_7 },
    { 0x676c666c, SUC_NE_lexicon_tags_2 },
    { 0x381e666c, SUC_NE_lexicon_tags_2 },
    { 0x9d56666b, SUC_NE_lexicon_tags_2 },
    { 0x5dce6670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118e6674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f66678, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dc4667a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24ca667c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60a667f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78646683, SUC_NE_lexicon_tags_2 },
    { 0xbc5a6684, SUC_NE_lexicon_tags_4 },
    { 0x55dc6684, SUC_NE_lexicon_tags_34 },
    { 0xd9286686, SUC_NE_lexicon_tags_2 },
    { 0xcf366687, SUC_NE_lexicon_tags_0 },
    { 0x24fe6688, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf804668a, SUC_NE_lexicon_tags_2 },
    { 0xf5b6668b, SUC_NE_lexicon_tags_2 },
    { 0x9e56668c, SUC_NE_lexicon_tags_4 },
    { 0x60f4668c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1914668f, SUC_NE_lexicon_tags_2 },
    { 0x1f4e6690, SUC_NE_lexicon_tags_2 },
    { 0x1ada668f, SUC_NE_lexicon_tags_160 },
    { 0xe20a668f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e6696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e6669c, SUC_NE_lexicon_tags_2 },
    { 0xb47a669d, SUC_NE_lexicon_tags_2 },
    { 0xfd28669e, SUC_NE_lexicon_tags_2 },
    { 0xd138669f, SUC_NE_lexicon_tags_4 },
    { 0x177666a0, SUC_NE_lexicon_tags_18 },
    { 0x652866a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0c66a5, SUC_NE_lexicon_tags_2 },
    { 0x427266a6, SUC_NE_lexicon_tags_2 },
    { 0xe7a466a7, SUC_NE_lexicon_tags_2 },
    { 0xa1b066a5, SUC_NE_lexicon_tags_2 },
    { 0xf12666a5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xdcbe66ab, SUC_NE_lexicon_tags_7 },
    { 0x7c7866ac, SUC_NE_lexicon_tags_2 },
    { 0x2fd066ad, SUC_NE_lexicon_tags_167 },
    { 0xbd8866ad, SUC_NE_lexicon_tags_121 },
    { 0x41666ae, SUC_NE_lexicon_tags_2 },
    { 0x461e66b0, SUC_NE_lexicon_tags_2 },
    { 0x988c66ae, SUC_NE_lexicon_tags_7 },
    { 0x561066b2, SUC_NE_lexicon_tags_2 },
    { 0x856066b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecb266b5, SUC_NE_lexicon_tags_2 },
    { 0xe4c866b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd95066b8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x15fc66ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5666bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a666c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc04a66c2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69466c6, SUC_NE_lexicon_tags_2 },
    { 0x810466c7, SUC_NE_lexicon_tags_2 },
    { 0xe25066c8, SUC_NE_lexicon_tags_2 },
    { 0x56d666c9, SUC_NE_lexicon_tags_4 },
    { 0x899266ca, SUC_NE_lexicon_tags_2 },
    { 0xfcc066cb, SUC_NE_lexicon_tags_2 },
    { 0x6ad666ca, SUC_NE_lexicon_tags_2 },
    { 0xa40866cb, SUC_NE_lexicon_tags_2 },
    { 0x8aaa66cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66d0, SUC_NE_lexicon_tags_7 },
    { 0x98d266d1, SUC_NE_lexicon_tags_2 },
    { 0x784666d1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x933466d5, SUC_NE_lexicon_tags_2 },
    { 0xd29a66d5, SUC_NE_lexicon_tags_2 },
    { 0xce066d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2266db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ee66e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27a66e7, SUC_NE_lexicon_tags_2 },
    { 0xef7266e8, SUC_NE_lexicon_tags_2 },
    { 0x638666e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe89666eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc07866ed, SUC_NE_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29666f0, SUC_NE_lexicon_tags_2 },
    { 0x6aee66f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dfc66f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868866fa, SUC_NE_lexicon_tags_2 },
    { 0xeba866fa, SUC_NE_lexicon_tags_2 },
    { 0xc3e666fa, SUC_NE_lexicon_tags_2 },
    { 0x821666fd, SUC_NE_lexicon_tags_2 },
    { 0xf83c66fd, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b746704, SUC_NE_lexicon_tags_2 },
    { 0xcf86705, SUC_NE_lexicon_tags_2 },
    { 0xdfaa6705, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60926709, SUC_NE_lexicon_tags_2 },
    { 0xdc54670a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab4670d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b726711, SUC_NE_lexicon_tags_2 },
    { 0x98c06712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d66715, SUC_NE_lexicon_tags_2 },
    { 0xdcc6716, SUC_NE_lexicon_tags_19 },
    { 0x12d46717, SUC_NE_lexicon_tags_2 },
    { 0x77486718, SUC_NE_lexicon_tags_2 },
    { 0xdc666718, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe45a671b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6a671e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43a46720, SUC_NE_lexicon_tags_2 },
    { 0x46fc6720, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e06725, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8966727, SUC_NE_lexicon_tags_2 },
    { 0xbf626728, SUC_NE_lexicon_tags_2 },
    { 0xcab06727, SUC_NE_lexicon_tags_2 },
    { 0xbbc4672a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9096672c, SUC_NE_lexicon_tags_2 },
    { 0x5418672d, SUC_NE_lexicon_tags_2 },
    { 0x6144672d, SUC_NE_lexicon_tags_2 },
    { 0xdcd0672c, SUC_NE_lexicon_tags_2 },
    { 0xa0086730, SUC_NE_lexicon_tags_7 },
    { 0xfef66731, SUC_NE_lexicon_tags_13 },
    { 0x47b66732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92066735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67a6738, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffd8673a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d22673e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c746747, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8c674a, SUC_NE_lexicon_tags_2 },
    { 0x5b6674b, SUC_NE_lexicon_tags_2 },
    { 0x4b74674c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42c674f, SUC_NE_lexicon_tags_2 },
    { 0x27ea6750, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc6ce6752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a306756, SUC_NE_lexicon_tags_4 },
    { 0xee386757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe898675e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7c6766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c86769, SUC_NE_lexicon_tags_2 },
    { 0x31b66769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68c676f, SUC_NE_lexicon_tags_2 },
    { 0x9cf06770, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec946776, SUC_NE_lexicon_tags_2 },
    { 0xffc26776, SUC_NE_lexicon_tags_2 },
    { 0x5e2c6776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fa677d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed26785, SUC_NE_lexicon_tags_2 },
    { 0x53246786, SUC_NE_lexicon_tags_2 },
    { 0x5c4e6786, SUC_NE_lexicon_tags_19 },
    { 0x19506788, SUC_NE_lexicon_tags_4 },
    { 0x32646789, SUC_NE_lexicon_tags_2 },
    { 0xb046678a, SUC_NE_lexicon_tags_2 },
    { 0xa6f46788, SUC_NE_lexicon_tags_2 },
    { 0xd6ec6786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdc6790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63806793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62106796, SUC_NE_lexicon_tags_2 },
    { 0x42686797, SUC_NE_lexicon_tags_2 },
    { 0x90c86798, SUC_NE_lexicon_tags_2 },
    { 0x7df86798, SUC_NE_lexicon_tags_2 },
    { 0x7f8e6797, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d80679c, SUC_NE_lexicon_tags_2 },
    { 0xb6f2679d, SUC_NE_lexicon_tags_2 },
    { 0x628c679d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dec67a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc67a5, SUC_NE_lexicon_tags_56 },
    { 0, NULL },
    { 0x638867a7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4b667a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df467ac, SUC_NE_lexicon_tags_2 },
    { 0xdfa067ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf1e67af, SUC_NE_lexicon_tags_2 },
    { 0x8f2067b0, SUC_NE_lexicon_tags_2 },
    { 0xa6fe67b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd467b4, SUC_NE_lexicon_tags_2 },
    { 0xa77667b5, SUC_NE_lexicon_tags_2 },
    { 0x134c67b6, SUC_NE_lexicon_tags_2 },
    { 0x5ff867b7, SUC_NE_lexicon_tags_2 },
    { 0x1d6067b4, SUC_NE_lexicon_tags_1 },
    { 0x619867b9, SUC_NE_lexicon_tags_2 },
    { 0x9c8067b7, SUC_NE_lexicon_tags_2 },
    { 0x244e67bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x784a67bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x656467c0, SUC_NE_lexicon_tags_2 },
    { 0xe92067c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ba67c3, SUC_NE_lexicon_tags_7 },
    { 0xbbea67c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7ee67c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96a67c9, SUC_NE_lexicon_tags_2 },
    { 0xf1bc67ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42fe67cc, SUC_NE_lexicon_tags_4 },
    { 0xc69667cc, SUC_NE_lexicon_tags_2 },
    { 0xbda867ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac667d3, SUC_NE_lexicon_tags_2 },
    { 0x60ba67d4, SUC_NE_lexicon_tags_10 },
    { 0x419067d4, SUC_NE_lexicon_tags_2 },
    { 0x4c9667d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0e467d8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x782867da, SUC_NE_lexicon_tags_2 },
    { 0x127e67db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ba67e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2067e7, SUC_NE_lexicon_tags_2 },
    { 0xdb9a67e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2e67eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c5e67ed, SUC_NE_lexicon_tags_2 },
    { 0xa33267ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0a067f0, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00667f3, SUC_NE_lexicon_tags_7 },
    { 0xa69067f4, SUC_NE_lexicon_tags_2 },
    { 0x37a67f3, SUC_NE_lexicon_tags_3 },
    { 0x154067f3, SUC_NE_lexicon_tags_2 },
    { 0x830867f7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x35fa67f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d067ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e2c6801, SUC_NE_lexicon_tags_2 },
    { 0xfef06801, SUC_NE_lexicon_tags_2 },
    { 0x1a26801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x803e6805, SUC_NE_lexicon_tags_2 },
    { 0xd9566806, SUC_NE_lexicon_tags_2 },
    { 0x76406806, SUC_NE_lexicon_tags_4 },
    { 0xb8c86808, SUC_NE_lexicon_tags_2 },
    { 0x73386808, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x814a680b, SUC_NE_lexicon_tags_2 },
    { 0x28e680c, SUC_NE_lexicon_tags_2 },
    { 0x7c56680d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ee6811, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d46815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa6818, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xe2ec681a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4681c, SUC_NE_lexicon_tags_2 },
    { 0x8f4c681d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61506821, SUC_NE_lexicon_tags_2 },
    { 0x89ce6822, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ba86824, SUC_NE_lexicon_tags_2 },
    { 0xf86825, SUC_NE_lexicon_tags_2 },
    { 0x2a326826, SUC_NE_lexicon_tags_2 },
    { 0x91746827, SUC_NE_lexicon_tags_2 },
    { 0x352c6825, SUC_NE_lexicon_tags_3 },
    { 0xad506829, SUC_NE_lexicon_tags_2 },
    { 0x90846829, SUC_NE_lexicon_tags_2 },
    { 0x7b3e682b, SUC_NE_lexicon_tags_2 },
    { 0x15c86824, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1c6832, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb46835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbda6837, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x282c683a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x153e683c, SUC_NE_lexicon_tags_0 },
    { 0x1de8683d, SUC_NE_lexicon_tags_2 },
    { 0x6820683e, SUC_NE_lexicon_tags_2 },
    { 0xc0cc683f, SUC_NE_lexicon_tags_2 },
    { 0x500c6840, SUC_NE_lexicon_tags_2 },
    { 0x4b06840, SUC_NE_lexicon_tags_2 },
    { 0x2d6c6842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd546845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96d26847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840684c, SUC_NE_lexicon_tags_2 },
    { 0xb3f8684d, SUC_NE_lexicon_tags_2 },
    { 0xa330684e, SUC_NE_lexicon_tags_2 },
    { 0x7988684f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cc66851, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc806856, SUC_NE_lexicon_tags_2 },
    { 0x70c06857, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x6c266859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8685c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x251c685e, SUC_NE_lexicon_tags_2 },
    { 0xadc2685f, SUC_NE_lexicon_tags_36 },
    { 0x3e9a685f, SUC_NE_lexicon_tags_2 },
    { 0x590a685e, SUC_NE_lexicon_tags_2 },
    { 0x4a66862, SUC_NE_lexicon_tags_7 },
    { 0xc5bc6863, SUC_NE_lexicon_tags_19 },
    { 0xf1c26864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdbc6866, SUC_NE_lexicon_tags_2 },
    { 0xb8d06867, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539c686c, SUC_NE_lexicon_tags_4 },
    { 0xe3c0686c, SUC_NE_lexicon_tags_2 },
    { 0x402a686e, SUC_NE_lexicon_tags_1 },
    { 0x5854686e, SUC_NE_lexicon_tags_2 },
    { 0xa3f4686f, SUC_NE_lexicon_tags_2 },
    { 0x4e4e6870, SUC_NE_lexicon_tags_2 },
    { 0x663a6872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x435c6877, SUC_NE_lexicon_tags_2 },
    { 0x32a06878, SUC_NE_lexicon_tags_2 },
    { 0x2c1a6879, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2754687d, SUC_NE_lexicon_tags_2 },
    { 0x41d0687d, SUC_NE_lexicon_tags_50 },
    { 0xe72e687f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48146881, SUC_NE_lexicon_tags_2 },
    { 0x9c166882, SUC_NE_lexicon_tags_2 },
    { 0xa3406883, SUC_NE_lexicon_tags_4 },
    { 0x29166881, SUC_NE_lexicon_tags_3 },
    { 0xe4226884, SUC_NE_lexicon_tags_225 },
    { 0x936a6881, SUC_NE_lexicon_tags_2 },
    { 0x53d26887, SUC_NE_lexicon_tags_2 },
    { 0x8f766887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e4688b, SUC_NE_lexicon_tags_2 },
    { 0xf1ca688c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd58688e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c726890, SUC_NE_lexicon_tags_2 },
    { 0xdad66891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc6896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514c689a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c78689c, SUC_NE_lexicon_tags_2 },
    { 0x84ba689c, SUC_NE_lexicon_tags_2 },
    { 0x27ec689e, SUC_NE_lexicon_tags_5 },
    { 0x1314689e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d9e68a1, SUC_NE_lexicon_tags_2 },
    { 0xb6c868a1, SUC_NE_lexicon_tags_2 },
    { 0x177868a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f4268a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x643068a7, SUC_NE_lexicon_tags_2 },
    { 0x3aea68a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e8868aa, SUC_NE_lexicon_tags_2 },
    { 0xd07868aa, SUC_NE_lexicon_tags_2 },
    { 0xf7c68ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89e68b2, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x341668b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58f868b7, SUC_NE_lexicon_tags_2 },
    { 0x99b868b8, SUC_NE_lexicon_tags_2 },
    { 0x7e5268b9, SUC_NE_lexicon_tags_0 },
    { 0xabe468ba, SUC_NE_lexicon_tags_2 },
    { 0xb71e68b9, SUC_NE_lexicon_tags_2 },
    { 0x447268bc, SUC_NE_lexicon_tags_2 },
    { 0xbb2268bd, SUC_NE_lexicon_tags_2 },
    { 0x70f668be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f3068c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06468c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ac68c7, SUC_NE_lexicon_tags_2 },
    { 0x27c668c8, SUC_NE_lexicon_tags_2 },
    { 0xa56068c9, SUC_NE_lexicon_tags_2 },
    { 0x4b2068ca, SUC_NE_lexicon_tags_2 },
    { 0x6be668cb, SUC_NE_lexicon_tags_2 },
    { 0xdd0868cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f268cf, SUC_NE_lexicon_tags_2 },
    { 0xc09668d0, SUC_NE_lexicon_tags_3 },
    { 0xed1c68d0, SUC_NE_lexicon_tags_2 },
    { 0x227468cf, SUC_NE_lexicon_tags_2 },
    { 0x653c68d3, SUC_NE_lexicon_tags_2 },
    { 0x70d668d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x423668d6, SUC_NE_lexicon_tags_2 },
    { 0x32de68d7, SUC_NE_lexicon_tags_2 },
    { 0x7b4868d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ad868da, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05e68de, SUC_NE_lexicon_tags_2 },
    { 0x262868df, SUC_NE_lexicon_tags_2 },
    { 0xbe9068e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba2868e2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36268e8, SUC_NE_lexicon_tags_2 },
    { 0x92ea68e9, SUC_NE_lexicon_tags_2 },
    { 0x964e68ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x164e68ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f468f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2268f3, SUC_NE_lexicon_tags_2 },
    { 0xec2868f4, SUC_NE_lexicon_tags_7 },
    { 0xb65a68f4, SUC_NE_lexicon_tags_2 },
    { 0x799668f6, SUC_NE_lexicon_tags_2 },
    { 0x25ce68f6, SUC_NE_lexicon_tags_17 },
    { 0x333e68f8, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x162268fb, SUC_NE_lexicon_tags_2 },
    { 0xb2fa68fb, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5ce68fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e206902, SUC_NE_lexicon_tags_2 },
    { 0x354e6903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a266905, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xdb246907, SUC_NE_lexicon_tags_2 },
    { 0x4ba66908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd246690c, SUC_NE_lexicon_tags_2 },
    { 0x6f00690d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2696690f, SUC_NE_lexicon_tags_2 },
    { 0xe8ea6910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00e6915, SUC_NE_lexicon_tags_2 },
    { 0xf1e26915, SUC_NE_lexicon_tags_2 },
    { 0x51ce6917, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646691c, SUC_NE_lexicon_tags_2 },
    { 0x853a691c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74c4691f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x364a6921, SUC_NE_lexicon_tags_2 },
    { 0x3a446921, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbf5c6924, SUC_NE_lexicon_tags_2 },
    { 0x85b26925, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25286929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0e2692b, SUC_NE_lexicon_tags_2 },
    { 0xb3e6692c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1e6933, SUC_NE_lexicon_tags_2 },
    { 0xed6c6933, SUC_NE_lexicon_tags_2 },
    { 0x56f66933, SUC_NE_lexicon_tags_2 },
    { 0xfbae6936, SUC_NE_lexicon_tags_2 },
    { 0x29446934, SUC_NE_lexicon_tags_2 },
    { 0xa9326935, SUC_NE_lexicon_tags_2 },
    { 0x265e6935, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56ce693b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4686940, SUC_NE_lexicon_tags_2 },
    { 0x9d906941, SUC_NE_lexicon_tags_2 },
    { 0x5efe6941, SUC_NE_lexicon_tags_2 },
    { 0x72786943, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c926945, SUC_NE_lexicon_tags_2 },
    { 0x627e6946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d38694d, SUC_NE_lexicon_tags_2 },
    { 0x455c694e, SUC_NE_lexicon_tags_2 },
    { 0x9e3c694f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da86957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x797e6959, SUC_NE_lexicon_tags_2 },
    { 0xd120695a, SUC_NE_lexicon_tags_2 },
    { 0xaa0e695b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd66695e, SUC_NE_lexicon_tags_2 },
    { 0xb33a695f, SUC_NE_lexicon_tags_2 },
    { 0xb20c695f, SUC_NE_lexicon_tags_28 },
    { 0x2eb26960, SUC_NE_lexicon_tags_2 },
    { 0xf0b86962, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fda6965, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cda6969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e32696b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a26973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37ee6975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1976697d, SUC_NE_lexicon_tags_2 },
    { 0x3bf0697e, SUC_NE_lexicon_tags_2 },
    { 0xe0e6697f, SUC_NE_lexicon_tags_2 },
    { 0x75d66980, SUC_NE_lexicon_tags_2 },
    { 0x6fe26981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea066983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2906986, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a626989, SUC_NE_lexicon_tags_2 },
    { 0xbb96698a, SUC_NE_lexicon_tags_2 },
    { 0xf496698b, SUC_NE_lexicon_tags_2 },
    { 0xb7cc698c, SUC_NE_lexicon_tags_2 },
    { 0x9b46698c, SUC_NE_lexicon_tags_2 },
    { 0xa17e698e, SUC_NE_lexicon_tags_2 },
    { 0x4b5c698f, SUC_NE_lexicon_tags_2 },
    { 0x421c6990, SUC_NE_lexicon_tags_2 },
    { 0x11f8698f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f606994, SUC_NE_lexicon_tags_2 },
    { 0xca3e6994, SUC_NE_lexicon_tags_2 },
    { 0x50fa6996, SUC_NE_lexicon_tags_2 },
    { 0xba086997, SUC_NE_lexicon_tags_2 },
    { 0xbe506997, SUC_NE_lexicon_tags_2 },
    { 0x9a806997, SUC_NE_lexicon_tags_4 },
    { 0x15f0699a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a699d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5eba699f, SUC_NE_lexicon_tags_2 },
    { 0xeab669a0, SUC_NE_lexicon_tags_2 },
    { 0x650069a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a69a4, SUC_NE_lexicon_tags_2 },
    { 0x793c69a5, SUC_NE_lexicon_tags_2 },
    { 0x206e69a6, SUC_NE_lexicon_tags_2 },
    { 0xe7669a7, SUC_NE_lexicon_tags_7 },
    { 0x75e269a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aa69ae, SUC_NE_lexicon_tags_2 },
    { 0xa21869ae, SUC_NE_lexicon_tags_2 },
    { 0x7a1c69b0, SUC_NE_lexicon_tags_2 },
    { 0x18f869b1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb09a69b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae669b7, SUC_NE_lexicon_tags_2 },
    { 0x8b9e69b8, SUC_NE_lexicon_tags_3 },
    { 0x205e69b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282e69c0, SUC_NE_lexicon_tags_2 },
    { 0xae3869c1, SUC_NE_lexicon_tags_2 },
    { 0x114869c2, SUC_NE_lexicon_tags_2 },
    { 0xb1fe69c2, SUC_NE_lexicon_tags_2 },
    { 0xbfaa69c2, SUC_NE_lexicon_tags_2 },
    { 0xc8f869c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988269ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x902469cd, SUC_NE_lexicon_tags_2 },
    { 0x6ac269ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x477469d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x995069d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x173669d6, SUC_NE_lexicon_tags_2 },
    { 0xae0e69d7, SUC_NE_lexicon_tags_2 },
    { 0x3c0669d8, SUC_NE_lexicon_tags_348 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b669dc, SUC_NE_lexicon_tags_1 },
    { 0x88b069dd, SUC_NE_lexicon_tags_2 },
    { 0x809a69dc, SUC_NE_lexicon_tags_2 },
    { 0xde2e69df, SUC_NE_lexicon_tags_2 },
    { 0xd1869df, SUC_NE_lexicon_tags_2 },
    { 0x4a7469e1, SUC_NE_lexicon_tags_2 },
    { 0x63aa69e1, SUC_NE_lexicon_tags_192 },
    { 0x1ca069dc, SUC_NE_lexicon_tags_2 },
    { 0x1e9e69e2, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded069ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab269ee, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ea69f2, SUC_NE_lexicon_tags_7 },
    { 0x99c669f2, SUC_NE_lexicon_tags_4 },
    { 0x4bd469f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda69f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63069fd, SUC_NE_lexicon_tags_1 },
    { 0x667469fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4226a05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe57a6a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ce6a0c, SUC_NE_lexicon_tags_2 },
    { 0xd6486a0c, SUC_NE_lexicon_tags_2 },
    { 0x2e8a6a0e, SUC_NE_lexicon_tags_2 },
    { 0x8b1e6a0e, SUC_NE_lexicon_tags_2 },
    { 0xf77c6a0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5746a12, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b66a16, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x154c6a18, SUC_NE_lexicon_tags_2 },
    { 0xf1d66a19, SUC_NE_lexicon_tags_2 },
    { 0x6f2e6a18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5c6a1d, SUC_NE_lexicon_tags_2 },
    { 0x9ee6a1d, SUC_NE_lexicon_tags_7 },
    { 0x1b866a1e, SUC_NE_lexicon_tags_2 },
    { 0xb2e06a20, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8546a28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac166a2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78de6a2e, SUC_NE_lexicon_tags_2 },
    { 0xb766a2f, SUC_NE_lexicon_tags_2 },
    { 0x81826a2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfaae6a32, SUC_NE_lexicon_tags_2 },
    { 0xddd26a33, SUC_NE_lexicon_tags_2 },
    { 0xd0e06a34, SUC_NE_lexicon_tags_2 },
    { 0x338a6a35, SUC_NE_lexicon_tags_2 },
    { 0x86a86a36, SUC_NE_lexicon_tags_214 },
    { 0xbdc46a33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c6a3b, SUC_NE_lexicon_tags_2 },
    { 0xcdd46a3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd506a3e, SUC_NE_lexicon_tags_2 },
    { 0x77526a3f, SUC_NE_lexicon_tags_2 },
    { 0xbd8c6a3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc66a45, SUC_NE_lexicon_tags_2 },
    { 0x45166a46, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb086a48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a046a4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf06a50, SUC_NE_lexicon_tags_2 },
    { 0x8b646a51, SUC_NE_lexicon_tags_2 },
    { 0x1ee66a50, SUC_NE_lexicon_tags_2 },
    { 0x99bc6a53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75266a55, SUC_NE_lexicon_tags_2 },
    { 0xf4aa6a56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0a6a5a, SUC_NE_lexicon_tags_4 },
    { 0xb8226a5b, SUC_NE_lexicon_tags_2 },
    { 0x3de06a5c, SUC_NE_lexicon_tags_1 },
    { 0xa75c6a5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf926a61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd46a64, SUC_NE_lexicon_tags_2 },
    { 0x44626a65, SUC_NE_lexicon_tags_2 },
    { 0x39e46a65, SUC_NE_lexicon_tags_2 },
    { 0xbd706a64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57446a6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80546a6f, SUC_NE_lexicon_tags_2 },
    { 0xd55a6a70, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b866a72, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x6bb06a74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b46a78, SUC_NE_lexicon_tags_2 },
    { 0x366e6a79, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xec9e6a7b, SUC_NE_lexicon_tags_2 },
    { 0x70386a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf766a84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe54a6a86, SUC_NE_lexicon_tags_2 },
    { 0x1ae66a86, SUC_NE_lexicon_tags_2 },
    { 0xec8c6a88, SUC_NE_lexicon_tags_2 },
    { 0xd1b46a87, SUC_NE_lexicon_tags_2 },
    { 0x13e26a87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2126a8c, SUC_NE_lexicon_tags_7 },
    { 0x185e6a8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc68a6a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc926a91, SUC_NE_lexicon_tags_1 },
    { 0xa7a06a92, SUC_NE_lexicon_tags_2 },
    { 0x9966a93, SUC_NE_lexicon_tags_2 },
    { 0xf2fa6a93, SUC_NE_lexicon_tags_4 },
    { 0x875c6a95, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16c6a9b, SUC_NE_lexicon_tags_2 },
    { 0xcce46a9c, SUC_NE_lexicon_tags_2 },
    { 0x48d26a9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0c6aa0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a426aa8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2826aab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7e6aae, SUC_NE_lexicon_tags_2 },
    { 0x56c06aae, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x33366ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae6a6ab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc286ab5, SUC_NE_lexicon_tags_2 },
    { 0x96626ab6, SUC_NE_lexicon_tags_2 },
    { 0xfd826ab6, SUC_NE_lexicon_tags_2 },
    { 0x19da6ab8, SUC_NE_lexicon_tags_2 },
    { 0x1edc6ab8, SUC_NE_lexicon_tags_2 },
    { 0xbcdc6aba, SUC_NE_lexicon_tags_2 },
    { 0x10966ab8, SUC_NE_lexicon_tags_17 },
    { 0x841c6abc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef66ac1, SUC_NE_lexicon_tags_2 },
    { 0x11f66ac2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47206ac4, SUC_NE_lexicon_tags_2 },
    { 0x4dfa6ac5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x960c6ac7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce06acb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12a06acd, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f66ad0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b66ad4, SUC_NE_lexicon_tags_2 },
    { 0xde646ad4, SUC_NE_lexicon_tags_2 },
    { 0x30086ad6, SUC_NE_lexicon_tags_2 },
    { 0x10e66ad7, SUC_NE_lexicon_tags_2 },
    { 0x1c446ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48146adc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe726ade, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d9c6ae0, SUC_NE_lexicon_tags_2 },
    { 0x93e66ae1, SUC_NE_lexicon_tags_2 },
    { 0x4f266ae2, SUC_NE_lexicon_tags_3 },
    { 0x4c766ae2, SUC_NE_lexicon_tags_2 },
    { 0xc95e6ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c266ae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbc46ae8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e26aec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bc6af0, SUC_NE_lexicon_tags_2 },
    { 0xbd1e6af1, SUC_NE_lexicon_tags_22 },
    { 0x85c26af2, SUC_NE_lexicon_tags_2 },
    { 0xf2c6af2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x183c6af5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9c6af8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8cc6afa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af46afd, SUC_NE_lexicon_tags_2 },
    { 0xd6286afe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3046b05, SUC_NE_lexicon_tags_2 },
    { 0x81d06b05, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e9e6b08, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf46b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69726b0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f26b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2626b12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ba6b15, SUC_NE_lexicon_tags_2 },
    { 0x411e6b15, SUC_NE_lexicon_tags_2 },
    { 0xe16a6b16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e8a6b19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7a6b1f, SUC_NE_lexicon_tags_2 },
    { 0xc2186b1f, SUC_NE_lexicon_tags_2 },
    { 0x99806b20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d66b26, SUC_NE_lexicon_tags_2 },
    { 0xb5966b27, SUC_NE_lexicon_tags_2 },
    { 0x2c7c6b26, SUC_NE_lexicon_tags_2 },
    { 0x359a6b29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79046b2c, SUC_NE_lexicon_tags_34 },
    { 0x59626b2d, SUC_NE_lexicon_tags_2 },
    { 0xf1286b2e, SUC_NE_lexicon_tags_2 },
    { 0x10486b2f, SUC_NE_lexicon_tags_2 },
    { 0x83f86b2e, SUC_NE_lexicon_tags_2 },
    { 0xfbac6b2c, SUC_NE_lexicon_tags_2 },
    { 0x1c46b31, SUC_NE_lexicon_tags_168 },
    { 0, NULL },
    { 0x16ba6b34, SUC_NE_lexicon_tags_2 },
    { 0x319e6b35, SUC_NE_lexicon_tags_2 },
    { 0xc73c6b36, SUC_NE_lexicon_tags_2 },
    { 0x96f66b37, SUC_NE_lexicon_tags_2 },
    { 0xa6066b37, SUC_NE_lexicon_tags_2 },
    { 0x5b5e6b34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241e6b41, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c66b44, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42986b48, SUC_NE_lexicon_tags_18 },
    { 0x31786b49, SUC_NE_lexicon_tags_2 },
    { 0x7a8c6b4a, SUC_NE_lexicon_tags_2 },
    { 0x4bba6b4a, SUC_NE_lexicon_tags_2 },
    { 0x8c1c6b4c, SUC_NE_lexicon_tags_24 },
    { 0xcb0a6b4b, SUC_NE_lexicon_tags_22 },
    { 0x8dd26b4c, SUC_NE_lexicon_tags_2 },
    { 0xa19a6b4f, SUC_NE_lexicon_tags_2 },
    { 0xd9626b4c, SUC_NE_lexicon_tags_2 },
    { 0x47ec6b4b, SUC_NE_lexicon_tags_2 },
    { 0xce4a6b52, SUC_NE_lexicon_tags_2 },
    { 0xa7a46b53, SUC_NE_lexicon_tags_2 },
    { 0x35946b54, SUC_NE_lexicon_tags_2 },
    { 0x278c6b4e, SUC_NE_lexicon_tags_2 },
    { 0xf3a26b4f, SUC_NE_lexicon_tags_2 },
    { 0x1b5c6b52, SUC_NE_lexicon_tags_2 },
    { 0x501e6b58, SUC_NE_lexicon_tags_2 },
    { 0x78526b59, SUC_NE_lexicon_tags_2 },
    { 0x3d806b5a, SUC_NE_lexicon_tags_2 },
    { 0xb48c6b4a, SUC_NE_lexicon_tags_18 },
    { 0x2c6e6b5c, SUC_NE_lexicon_tags_2 },
    { 0xd58c6b5d, SUC_NE_lexicon_tags_2 },
    { 0x42f86b5d, SUC_NE_lexicon_tags_2 },
    { 0xcc086b5c, SUC_NE_lexicon_tags_2 },
    { 0x9b8e6b60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae6b64, SUC_NE_lexicon_tags_2 },
    { 0x35ca6b65, SUC_NE_lexicon_tags_2 },
    { 0xed846b64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4a6b6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ebc6b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b06b74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57686b78, SUC_NE_lexicon_tags_2 },
    { 0x59886b79, SUC_NE_lexicon_tags_2 },
    { 0x18c46b78, SUC_NE_lexicon_tags_2 },
    { 0x3d026b7b, SUC_NE_lexicon_tags_2 },
    { 0xd3cc6b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bae6b7e, SUC_NE_lexicon_tags_2 },
    { 0x314a6b7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b246b85, SUC_NE_lexicon_tags_2 },
    { 0xc5646b86, SUC_NE_lexicon_tags_2 },
    { 0xb326b85, SUC_NE_lexicon_tags_2 },
    { 0xece66b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4e6b8f, SUC_NE_lexicon_tags_2 },
    { 0x9d0e6b8f, SUC_NE_lexicon_tags_2 },
    { 0x35d26b90, SUC_NE_lexicon_tags_2 },
    { 0xca86b92, SUC_NE_lexicon_tags_2 },
    { 0xe9606b92, SUC_NE_lexicon_tags_2 },
    { 0xf9246b94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x966c6b96, SUC_NE_lexicon_tags_2 },
    { 0x44246b96, SUC_NE_lexicon_tags_2 },
    { 0x673c6b97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dc6b9b, SUC_NE_lexicon_tags_2 },
    { 0xba946b9b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6a6ba0, SUC_NE_lexicon_tags_2 },
    { 0x5eee6ba0, SUC_NE_lexicon_tags_3 },
    { 0x322e6ba2, SUC_NE_lexicon_tags_2 },
    { 0x9fac6ba2, SUC_NE_lexicon_tags_2 },
    { 0xa20c6ba0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32c66ba6, SUC_NE_lexicon_tags_2 },
    { 0x65366ba6, SUC_NE_lexicon_tags_2 },
    { 0xa5066ba8, SUC_NE_lexicon_tags_2 },
    { 0xf60a6ba9, SUC_NE_lexicon_tags_2 },
    { 0x4a546baa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c6bad, SUC_NE_lexicon_tags_2 },
    { 0x46e46bae, SUC_NE_lexicon_tags_2 },
    { 0x53be6baf, SUC_NE_lexicon_tags_2 },
    { 0x68dc6bb0, SUC_NE_lexicon_tags_2 },
    { 0xdf1e6bb1, SUC_NE_lexicon_tags_2 },
    { 0x5b806bb0, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae6bb8, SUC_NE_lexicon_tags_1 },
    { 0x6af06bb9, SUC_NE_lexicon_tags_2 },
    { 0xfb826bba, SUC_NE_lexicon_tags_2 },
    { 0xa6746bbb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81946bc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b4a6bc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc806bc6, SUC_NE_lexicon_tags_2 },
    { 0xa7726bc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157a6bd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e4e6bd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6306bd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c2e6bd6, SUC_NE_lexicon_tags_2 },
    { 0x519c6bd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf766bd9, SUC_NE_lexicon_tags_165 },
    { 0x51606bda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x246a6bdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc346bdf, SUC_NE_lexicon_tags_7 },
    { 0x21686bdf, SUC_NE_lexicon_tags_2 },
    { 0x57586be1, SUC_NE_lexicon_tags_2 },
    { 0xe1226bdf, SUC_NE_lexicon_tags_2 },
    { 0xaf86be1, SUC_NE_lexicon_tags_2 },
    { 0x9a2a6be4, SUC_NE_lexicon_tags_2 },
    { 0x9e626be1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d86bea, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b26bee, SUC_NE_lexicon_tags_17 },
    { 0x24166bef, SUC_NE_lexicon_tags_2 },
    { 0x8dc66bf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61826bf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e706bf4, SUC_NE_lexicon_tags_2 },
    { 0xcea66bf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50046bf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18626bfc, SUC_NE_lexicon_tags_2 },
    { 0x69106bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36da6bff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75886c06, SUC_NE_lexicon_tags_2 },
    { 0x39f66c07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x786e6c09, SUC_NE_lexicon_tags_2 },
    { 0x6eb66c09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b26c0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ec6c11, SUC_NE_lexicon_tags_2 },
    { 0x27906c12, SUC_NE_lexicon_tags_2 },
    { 0x6f606c11, SUC_NE_lexicon_tags_2 },
    { 0x3cb86c14, SUC_NE_lexicon_tags_1 },
    { 0xf18e6c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cce6c1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a426c1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4306c1e, SUC_NE_lexicon_tags_2 },
    { 0xdcba6c1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95146c23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8006c27, SUC_NE_lexicon_tags_2 },
    { 0x16606c28, SUC_NE_lexicon_tags_2 },
    { 0x9f526c28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51346c2b, SUC_NE_lexicon_tags_2 },
    { 0xcae66c2c, SUC_NE_lexicon_tags_2 },
    { 0xb09c6c2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ac6c31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba26c34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6846c37, SUC_NE_lexicon_tags_2 },
    { 0x626e6c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61346c3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28ec6c3c, SUC_NE_lexicon_tags_2 },
    { 0x26c26c3d, SUC_NE_lexicon_tags_1 },
    { 0xa76a6c3e, SUC_NE_lexicon_tags_2 },
    { 0x322a6c3f, SUC_NE_lexicon_tags_2 },
    { 0x1c66c40, SUC_NE_lexicon_tags_2 },
    { 0x48c06c40, SUC_NE_lexicon_tags_2 },
    { 0x3c7c6c40, SUC_NE_lexicon_tags_2 },
    { 0x70a26c43, SUC_NE_lexicon_tags_2 },
    { 0x729c6c3e, SUC_NE_lexicon_tags_3 },
    { 0x88fe6c3d, SUC_NE_lexicon_tags_2 },
    { 0xc66c6c44, SUC_NE_lexicon_tags_0 },
    { 0x638c6c47, SUC_NE_lexicon_tags_3 },
    { 0x70606c48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c6c4e, SUC_NE_lexicon_tags_2 },
    { 0x82266c4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68ba6c51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e06c54, SUC_NE_lexicon_tags_2 },
    { 0x50066c54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f066c57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832a6c5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d9a6c5e, SUC_NE_lexicon_tags_2 },
    { 0xd5706c5e, SUC_NE_lexicon_tags_4 },
    { 0x39486c5f, SUC_NE_lexicon_tags_2 },
    { 0xd5c26c5f, SUC_NE_lexicon_tags_2 },
    { 0xf9be6c5e, SUC_NE_lexicon_tags_2 },
    { 0xfa186c62, SUC_NE_lexicon_tags_2 },
    { 0x3246c64, SUC_NE_lexicon_tags_2 },
    { 0x97e26c62, SUC_NE_lexicon_tags_2 },
    { 0x41ca6c60, SUC_NE_lexicon_tags_2 },
    { 0xc8b26c66, SUC_NE_lexicon_tags_2 },
    { 0xde946c68, SUC_NE_lexicon_tags_2 },
    { 0xccf66c68, SUC_NE_lexicon_tags_2 },
    { 0x4986c6a, SUC_NE_lexicon_tags_2 },
    { 0x96566c6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ac6c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd6a6c6f, SUC_NE_lexicon_tags_2 },
    { 0x80b06c70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51ea6c72, SUC_NE_lexicon_tags_2 },
    { 0x22d26c73, SUC_NE_lexicon_tags_2 },
    { 0x49426c73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3d86c76, SUC_NE_lexicon_tags_2 },
    { 0x29e26c77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdad66c79, SUC_NE_lexicon_tags_32 },
    { 0x5b746c79, SUC_NE_lexicon_tags_2 },
    { 0xe3fa6c79, SUC_NE_lexicon_tags_2 },
    { 0xb3706c7c, SUC_NE_lexicon_tags_2 },
    { 0xc6ee6c7b, SUC_NE_lexicon_tags_2 },
    { 0xe8106c7e, SUC_NE_lexicon_tags_2 },
    { 0xb2586c7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e86c82, SUC_NE_lexicon_tags_2 },
    { 0xff886c83, SUC_NE_lexicon_tags_2 },
    { 0x26b46c84, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a06c89, SUC_NE_lexicon_tags_2 },
    { 0xcff06c8a, SUC_NE_lexicon_tags_2 },
    { 0x89fc6c8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d66c8e, SUC_NE_lexicon_tags_2 },
    { 0x65b06c8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6e6c96, SUC_NE_lexicon_tags_2 },
    { 0x40866c97, SUC_NE_lexicon_tags_2 },
    { 0x80146c98, SUC_NE_lexicon_tags_2 },
    { 0x87d26c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x834e6c9b, SUC_NE_lexicon_tags_2 },
    { 0x7f306c9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9e6ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a6ca6, SUC_NE_lexicon_tags_2 },
    { 0x10d46ca6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f46caa, SUC_NE_lexicon_tags_8 },
    { 0xb7b86cab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f746cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6aa6caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ea6cb2, SUC_NE_lexicon_tags_2 },
    { 0x455a6cb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ade6cb8, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0xbee26cba, SUC_NE_lexicon_tags_2 },
    { 0x9b906cbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa05e6cbd, SUC_NE_lexicon_tags_2 },
    { 0xc1006cbd, SUC_NE_lexicon_tags_2 },
    { 0x872a6cbf, SUC_NE_lexicon_tags_2 },
    { 0x1e366cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35bc6cc2, SUC_NE_lexicon_tags_2 },
    { 0xfdde6cc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50546cc5, SUC_NE_lexicon_tags_2 },
    { 0xa25c6cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b166cc9, SUC_NE_lexicon_tags_2 },
    { 0x3c1e6cc9, SUC_NE_lexicon_tags_2 },
    { 0xa0c6ccb, SUC_NE_lexicon_tags_2 },
    { 0x90bc6cc9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c46cce, SUC_NE_lexicon_tags_2 },
    { 0xbfac6cce, SUC_NE_lexicon_tags_2 },
    { 0xdc4e6ccf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebc66cd2, SUC_NE_lexicon_tags_2 },
    { 0x9f4c6cd3, SUC_NE_lexicon_tags_2 },
    { 0x6a666cd4, SUC_NE_lexicon_tags_11 },
    { 0x9d966cd3, SUC_NE_lexicon_tags_18 },
    { 0x6e246cd6, SUC_NE_lexicon_tags_2 },
    { 0xda166cd7, SUC_NE_lexicon_tags_2 },
    { 0x2bac6cd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f26cdb, SUC_NE_lexicon_tags_2 },
    { 0x24f26cdc, SUC_NE_lexicon_tags_4 },
    { 0x6cae6cdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb55a6cdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6046ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa6ce4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacce6ce8, SUC_NE_lexicon_tags_2 },
    { 0xa19e6ce8, SUC_NE_lexicon_tags_2 },
    { 0x355a6cea, SUC_NE_lexicon_tags_2 },
    { 0x7a0c6ceb, SUC_NE_lexicon_tags_2 },
    { 0x10cc6cec, SUC_NE_lexicon_tags_2 },
    { 0x725a6ced, SUC_NE_lexicon_tags_2 },
    { 0xbb586ce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e866cf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60666cf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40e6d00, SUC_NE_lexicon_tags_2 },
    { 0x686e6d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5066d03, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2a6d0a, SUC_NE_lexicon_tags_2 },
    { 0xb3c26d0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8a6d0e, SUC_NE_lexicon_tags_2 },
    { 0xbf2a6d0f, SUC_NE_lexicon_tags_2 },
    { 0x195c6d10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31e6d14, SUC_NE_lexicon_tags_2 },
    { 0x7b626d15, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7a6d18, SUC_NE_lexicon_tags_2 },
    { 0xf44a6d19, SUC_NE_lexicon_tags_2 },
    { 0x5eb46d19, SUC_NE_lexicon_tags_2 },
    { 0x7606d1b, SUC_NE_lexicon_tags_2 },
    { 0x4ef06d1c, SUC_NE_lexicon_tags_40 },
    { 0xbc226d1d, SUC_NE_lexicon_tags_2 },
    { 0x5ace6d1d, SUC_NE_lexicon_tags_2 },
    { 0x4a3a6d1f, SUC_NE_lexicon_tags_2 },
    { 0xdee6d19, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cc6d25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaba6d27, SUC_NE_lexicon_tags_2 },
    { 0xcf7c6d28, SUC_NE_lexicon_tags_2 },
    { 0x68c86d29, SUC_NE_lexicon_tags_2 },
    { 0x16426d28, SUC_NE_lexicon_tags_2 },
    { 0xae9c6d28, SUC_NE_lexicon_tags_2 },
    { 0xad966d28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7526d2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45fa6d30, SUC_NE_lexicon_tags_2 },
    { 0xa1dc6d31, SUC_NE_lexicon_tags_32 },
    { 0x6c146d32, SUC_NE_lexicon_tags_2 },
    { 0x27286d33, SUC_NE_lexicon_tags_17 },
    { 0xd0c46d33, SUC_NE_lexicon_tags_0 },
    { 0xa8026d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10f86d37, SUC_NE_lexicon_tags_2 },
    { 0x9c9c6d38, SUC_NE_lexicon_tags_2 },
    { 0x57966d38, SUC_NE_lexicon_tags_2 },
    { 0xd72c6d38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef26d3d, SUC_NE_lexicon_tags_2 },
    { 0x85946d3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60fe6d40, SUC_NE_lexicon_tags_2 },
    { 0x94166d40, SUC_NE_lexicon_tags_2 },
    { 0xabde6d42, SUC_NE_lexicon_tags_2 },
    { 0x4b846d41, SUC_NE_lexicon_tags_2 },
    { 0x64566d40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a06d48, SUC_NE_lexicon_tags_2 },
    { 0xe7fe6d48, SUC_NE_lexicon_tags_2 },
    { 0x65ba6d4a, SUC_NE_lexicon_tags_2 },
    { 0x2c246d4b, SUC_NE_lexicon_tags_2 },
    { 0xb086d4b, SUC_NE_lexicon_tags_2 },
    { 0x58c86d48, SUC_NE_lexicon_tags_2 },
    { 0x65506d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13866d54, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c6d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc846d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c86d5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d26d62, SUC_NE_lexicon_tags_2 },
    { 0xb16c6d63, SUC_NE_lexicon_tags_2 },
    { 0x521e6d62, SUC_NE_lexicon_tags_2 },
    { 0xbf866d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec5e6d67, SUC_NE_lexicon_tags_2 },
    { 0x4cc06d68, SUC_NE_lexicon_tags_19 },
    { 0x62e86d69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45086d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3166d6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f606d73, SUC_NE_lexicon_tags_2 },
    { 0x5e006d73, SUC_NE_lexicon_tags_2 },
    { 0xba26d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbbe6d77, SUC_NE_lexicon_tags_13 },
    { 0xedb06d78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe366d7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c06d7e, SUC_NE_lexicon_tags_2 },
    { 0x3326d7f, SUC_NE_lexicon_tags_2 },
    { 0x2dc06d80, SUC_NE_lexicon_tags_2 },
    { 0xa0e06d81, SUC_NE_lexicon_tags_2 },
    { 0x496a6d7f, SUC_NE_lexicon_tags_2 },
    { 0xbe406d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46106d88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe17a6d8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf66d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8606d8f, SUC_NE_lexicon_tags_4 },
    { 0xf7606d90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2646d93, SUC_NE_lexicon_tags_2 },
    { 0x4fcc6d94, SUC_NE_lexicon_tags_2 },
    { 0x1e966d95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf66d9a, SUC_NE_lexicon_tags_0 },
    { 0x2ef86d9a, SUC_NE_lexicon_tags_2 },
    { 0x11b06d9c, SUC_NE_lexicon_tags_58 },
    { 0x59dc6d9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf86d9f, SUC_NE_lexicon_tags_2 },
    { 0x292c6da0, SUC_NE_lexicon_tags_2 },
    { 0xd55e6da1, SUC_NE_lexicon_tags_2 },
    { 0x73606da2, SUC_NE_lexicon_tags_2 },
    { 0xf9c06da3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ec6da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd66da9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26c6dae, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x7c746db0, SUC_NE_lexicon_tags_2 },
    { 0xa0766db1, SUC_NE_lexicon_tags_30 },
    { 0x23ae6db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1e6db7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62886dbf, SUC_NE_lexicon_tags_3 },
    { 0xf9d46dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc88a6dc2, SUC_NE_lexicon_tags_2 },
    { 0x9fae6dc3, SUC_NE_lexicon_tags_2 },
    { 0x56426dc4, SUC_NE_lexicon_tags_2 },
    { 0xb2246dc4, SUC_NE_lexicon_tags_2 },
    { 0x37b46dc6, SUC_NE_lexicon_tags_2 },
    { 0x2c1e6dc7, SUC_NE_lexicon_tags_2 },
    { 0xd5306dc8, SUC_NE_lexicon_tags_2 },
    { 0x1bac6dc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e6dcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d66dd1, SUC_NE_lexicon_tags_2 },
    { 0xf4726dd2, SUC_NE_lexicon_tags_2 },
    { 0xa0766dd1, SUC_NE_lexicon_tags_2 },
    { 0x63d06dd4, SUC_NE_lexicon_tags_2 },
    { 0xa9886dd5, SUC_NE_lexicon_tags_57 },
    { 0xcf1c6dd4, SUC_NE_lexicon_tags_2 },
    { 0xaabc6dd7, SUC_NE_lexicon_tags_2 },
    { 0x447a6dd1, SUC_NE_lexicon_tags_3 },
    { 0x8d5e6dd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3946de1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a1a6de3, SUC_NE_lexicon_tags_2 },
    { 0xb6406de4, SUC_NE_lexicon_tags_2 },
    { 0x95c86de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13a86de7, SUC_NE_lexicon_tags_2 },
    { 0x96de6de8, SUC_NE_lexicon_tags_2 },
    { 0x26426de9, SUC_NE_lexicon_tags_2 },
    { 0xe3c26dea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d566df2, SUC_NE_lexicon_tags_8 },
    { 0xc3a26df3, SUC_NE_lexicon_tags_2 },
    { 0x70206df4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cc6e00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd36e6e02, SUC_NE_lexicon_tags_2 },
    { 0x51dc6e03, SUC_NE_lexicon_tags_2 },
    { 0xb43c6e04, SUC_NE_lexicon_tags_13 },
    { 0xde6e03, SUC_NE_lexicon_tags_58 },
    { 0xa2f46e06, SUC_NE_lexicon_tags_3 },
    { 0x8c206e07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ba6e0b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd26e10, SUC_NE_lexicon_tags_2 },
    { 0xaea66e11, SUC_NE_lexicon_tags_11 },
    { 0xed926e11, SUC_NE_lexicon_tags_2 },
    { 0xdce06e13, SUC_NE_lexicon_tags_2 },
    { 0x16c46e11, SUC_NE_lexicon_tags_2 },
    { 0xc35e6e15, SUC_NE_lexicon_tags_2 },
    { 0xc9146e12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc6e1a, SUC_NE_lexicon_tags_32 },
    { 0x56246e1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12606e20, SUC_NE_lexicon_tags_7 },
    { 0xfb0c6e21, SUC_NE_lexicon_tags_2 },
    { 0x87386e20, SUC_NE_lexicon_tags_2 },
    { 0x8eb06e21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d006e26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcc6e29, SUC_NE_lexicon_tags_36 },
    { 0x1f826e2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1166e2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34046e2f, SUC_NE_lexicon_tags_4 },
    { 0x9e486e30, SUC_NE_lexicon_tags_2 },
    { 0xad2c6e31, SUC_NE_lexicon_tags_2 },
    { 0xea8a6e32, SUC_NE_lexicon_tags_2 },
    { 0x616e6e33, SUC_NE_lexicon_tags_2 },
    { 0xcd326e34, SUC_NE_lexicon_tags_2 },
    { 0x6da66e35, SUC_NE_lexicon_tags_2 },
    { 0xf6306e33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb13e6e38, SUC_NE_lexicon_tags_2 },
    { 0x49586e39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce86e3e, SUC_NE_lexicon_tags_2 },
    { 0x7e106e3f, SUC_NE_lexicon_tags_2 },
    { 0xe23e6e40, SUC_NE_lexicon_tags_2 },
    { 0xdb3e6e3f, SUC_NE_lexicon_tags_2 },
    { 0x65f26e42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x528e6e44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ee6e47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee6e4a, SUC_NE_lexicon_tags_2 },
    { 0x501e6e4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16966e57, SUC_NE_lexicon_tags_2 },
    { 0xe4a46e57, SUC_NE_lexicon_tags_2 },
    { 0xda06e58, SUC_NE_lexicon_tags_59 },
    { 0x99a86e5a, SUC_NE_lexicon_tags_2 },
    { 0x7c206e5a, SUC_NE_lexicon_tags_2 },
    { 0xc6ba6e5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48606e62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x551c6e65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1006e67, SUC_NE_lexicon_tags_2 },
    { 0x82cc6e68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a246e6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7186e6c, SUC_NE_lexicon_tags_2 },
    { 0x81386e6c, SUC_NE_lexicon_tags_2 },
    { 0x13c46e6e, SUC_NE_lexicon_tags_2 },
    { 0x1e206e6f, SUC_NE_lexicon_tags_2 },
    { 0xb10a6e70, SUC_NE_lexicon_tags_2 },
    { 0x83de6e71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488c6e75, SUC_NE_lexicon_tags_2 },
    { 0xec9a6e75, SUC_NE_lexicon_tags_7 },
    { 0x258e6e76, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93e6e7e, SUC_NE_lexicon_tags_2 },
    { 0x3e186e7f, SUC_NE_lexicon_tags_2 },
    { 0x9c3c6e80, SUC_NE_lexicon_tags_2 },
    { 0xad6e6e7e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x29ee6e83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8e6e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35106e8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f86e91, SUC_NE_lexicon_tags_7 },
    { 0xcd486e92, SUC_NE_lexicon_tags_2 },
    { 0xb8606e93, SUC_NE_lexicon_tags_2 },
    { 0x91ea6e94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65746e97, SUC_NE_lexicon_tags_1 },
    { 0xe3666e97, SUC_NE_lexicon_tags_2 },
    { 0xd2fa6e99, SUC_NE_lexicon_tags_2 },
    { 0xc2286e9a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xae4e6e9c, SUC_NE_lexicon_tags_2 },
    { 0xea586e9d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9d766e9f, SUC_NE_lexicon_tags_2 },
    { 0x1d566ea0, SUC_NE_lexicon_tags_2 },
    { 0xdfae6ea1, SUC_NE_lexicon_tags_2 },
    { 0xee466ea1, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7146ea5, SUC_NE_lexicon_tags_2 },
    { 0x8e166ea6, SUC_NE_lexicon_tags_2 },
    { 0x83626ea7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab026eaa, SUC_NE_lexicon_tags_2 },
    { 0x5d8a6eab, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25e6eae, SUC_NE_lexicon_tags_2 },
    { 0xcafa6eae, SUC_NE_lexicon_tags_270 },
    { 0x44a6eb0, SUC_NE_lexicon_tags_8 },
    { 0xecb06eaf, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0106eb4, SUC_NE_lexicon_tags_2 },
    { 0x41926eb5, SUC_NE_lexicon_tags_2 },
    { 0x4e646eb6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x51d66eb8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c86ebb, SUC_NE_lexicon_tags_2 },
    { 0x77926ebc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f2a6ebe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc846ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33d66ec6, SUC_NE_lexicon_tags_2 },
    { 0xc9306ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63b46ec9, SUC_NE_lexicon_tags_293 },
    { 0xc29a6eca, SUC_NE_lexicon_tags_2 },
    { 0x19326eca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0c6ed2, SUC_NE_lexicon_tags_2 },
    { 0xa7766ed3, SUC_NE_lexicon_tags_2 },
    { 0xbcc6ed4, SUC_NE_lexicon_tags_2 },
    { 0xf8726ed4, SUC_NE_lexicon_tags_2 },
    { 0x93be6ed6, SUC_NE_lexicon_tags_2 },
    { 0xa3366ed6, SUC_NE_lexicon_tags_2 },
    { 0x477c6ed8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c386ede, SUC_NE_lexicon_tags_2 },
    { 0xfabe6edf, SUC_NE_lexicon_tags_2 },
    { 0xb3326ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a6ee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c26ee7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71626eeb, SUC_NE_lexicon_tags_2 },
    { 0xb9246eec, SUC_NE_lexicon_tags_2 },
    { 0x70ea6eeb, SUC_NE_lexicon_tags_2 },
    { 0x46766eee, SUC_NE_lexicon_tags_0 },
    { 0xf4ca6eef, SUC_NE_lexicon_tags_2 },
    { 0x31e86eef, SUC_NE_lexicon_tags_2 },
    { 0xc0386eef, SUC_NE_lexicon_tags_2 },
    { 0xac3c6ef2, SUC_NE_lexicon_tags_2 },
    { 0x6ee66ef1, SUC_NE_lexicon_tags_2 },
    { 0x80a66ef2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ba6ef6, SUC_NE_lexicon_tags_2 },
    { 0x87286ef6, SUC_NE_lexicon_tags_2 },
    { 0x252c6ef6, SUC_NE_lexicon_tags_2 },
    { 0xd05e6ef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8a6efc, SUC_NE_lexicon_tags_2 },
    { 0x3c386efc, SUC_NE_lexicon_tags_4 },
    { 0x91e6efc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d246f03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa66f05, SUC_NE_lexicon_tags_2 },
    { 0xb6986f06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d506f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb66f0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2b26f0e, SUC_NE_lexicon_tags_2 },
    { 0x26ca6f0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f426f12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89e6f15, SUC_NE_lexicon_tags_2 },
    { 0x37c66f16, SUC_NE_lexicon_tags_2 },
    { 0x96706f16, SUC_NE_lexicon_tags_2 },
    { 0xc7a06f17, SUC_NE_lexicon_tags_2 },
    { 0xc6c6f15, SUC_NE_lexicon_tags_2 },
    { 0xb1f86f1a, SUC_NE_lexicon_tags_2 },
    { 0x4246f1a, SUC_NE_lexicon_tags_2 },
    { 0xcbf26f1b, SUC_NE_lexicon_tags_2 },
    { 0x5ac46f1c, SUC_NE_lexicon_tags_2 },
    { 0x71a06f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0a6f29, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b26f2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d46f31, SUC_NE_lexicon_tags_3 },
    { 0x15c86f32, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x10826f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c86f37, SUC_NE_lexicon_tags_2 },
    { 0xdda26f38, SUC_NE_lexicon_tags_2 },
    { 0xf5ea6f38, SUC_NE_lexicon_tags_2 },
    { 0xc3986f38, SUC_NE_lexicon_tags_2 },
    { 0xa23c6f39, SUC_NE_lexicon_tags_2 },
    { 0x8f466f39, SUC_NE_lexicon_tags_2 },
    { 0xf75a6f37, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb66f41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03a6f45, SUC_NE_lexicon_tags_7 },
    { 0x7e526f45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29a6f4a, SUC_NE_lexicon_tags_19 },
    { 0xec906f4b, SUC_NE_lexicon_tags_2 },
    { 0x92006f4c, SUC_NE_lexicon_tags_2 },
    { 0x88d26f4d, SUC_NE_lexicon_tags_2 },
    { 0xa76c6f4d, SUC_NE_lexicon_tags_19 },
    { 0x72e86f4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfe46f51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c906f53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc46f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5166f68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5aa6f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83b46f6c, SUC_NE_lexicon_tags_2 },
    { 0x49ae6f6d, SUC_NE_lexicon_tags_2 },
    { 0xf6866f6d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85aa6f73, SUC_NE_lexicon_tags_2 },
    { 0xdf7e6f74, SUC_NE_lexicon_tags_2 },
    { 0x64566f75, SUC_NE_lexicon_tags_2 },
    { 0x1c6e6f76, SUC_NE_lexicon_tags_2 },
    { 0xdb146f77, SUC_NE_lexicon_tags_2 },
    { 0xa64a6f77, SUC_NE_lexicon_tags_2 },
    { 0x2e2a6f79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2086f7b, SUC_NE_lexicon_tags_2 },
    { 0x31b06f7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d46f7e, SUC_NE_lexicon_tags_2 },
    { 0x2cc6f7e, SUC_NE_lexicon_tags_2 },
    { 0x7a506f80, SUC_NE_lexicon_tags_2 },
    { 0xf4586f81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebe6f83, SUC_NE_lexicon_tags_2 },
    { 0x65146f84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe046f86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a86f8a, SUC_NE_lexicon_tags_0 },
    { 0x43286f8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b86f8e, SUC_NE_lexicon_tags_2 },
    { 0xec66f8e, SUC_NE_lexicon_tags_2 },
    { 0x1426f8e, SUC_NE_lexicon_tags_2 },
    { 0x922c6f91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x196e6f93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x409e6f96, SUC_NE_lexicon_tags_2 },
    { 0xc71a6f97, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e446f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12606f9c, SUC_NE_lexicon_tags_2 },
    { 0x43f66f9c, SUC_NE_lexicon_tags_2 },
    { 0x3986f9c, SUC_NE_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3766fa1, SUC_NE_lexicon_tags_22 },
    { 0xee9e6fa2, SUC_NE_lexicon_tags_2 },
    { 0xda0a6fa3, SUC_NE_lexicon_tags_2 },
    { 0x61586fa1, SUC_NE_lexicon_tags_1 },
    { 0x3c4c6fa5, SUC_NE_lexicon_tags_2 },
    { 0x88006fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae106fa9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fa6fae, SUC_NE_lexicon_tags_7 },
    { 0xba4a6faf, SUC_NE_lexicon_tags_2 },
    { 0x3e126fae, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35686fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17766fb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a06fbb, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4c6fc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa066fc2, SUC_NE_lexicon_tags_2 },
    { 0xf9ee6fc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca6fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc26fca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20626fce, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb86fd1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ac6fd4, SUC_NE_lexicon_tags_2 },
    { 0x840c6fd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65486fd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa04e6fdb, SUC_NE_lexicon_tags_4 },
    { 0x14e46fdc, SUC_NE_lexicon_tags_51 },
    { 0x2bf86fdb, SUC_NE_lexicon_tags_2 },
    { 0xc1d46fdb, SUC_NE_lexicon_tags_2 },
    { 0xd3606fdf, SUC_NE_lexicon_tags_2 },
    { 0x44886fe0, SUC_NE_lexicon_tags_2 },
    { 0xc1b66fe1, SUC_NE_lexicon_tags_2 },
    { 0xf4be6fe0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbe6fe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde6fea, SUC_NE_lexicon_tags_2 },
    { 0x247c6feb, SUC_NE_lexicon_tags_2 },
    { 0x2e886fea, SUC_NE_lexicon_tags_2 },
    { 0x722c6fed, SUC_NE_lexicon_tags_3 },
    { 0x484a6fed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3a46ff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aac6ff4, SUC_NE_lexicon_tags_2 },
    { 0x67226ff5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50146ff7, SUC_NE_lexicon_tags_2 },
    { 0x51046ff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda26ffb, SUC_NE_lexicon_tags_2 },
    { 0xc1e86ffc, SUC_NE_lexicon_tags_2 },
    { 0x9c86ffb, SUC_NE_lexicon_tags_2 },
    { 0x74a86ffc, SUC_NE_lexicon_tags_24 },
    { 0x75166fff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf647004, SUC_NE_lexicon_tags_2 },
    { 0x59087004, SUC_NE_lexicon_tags_17 },
    { 0x532c7004, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x211a7009, SUC_NE_lexicon_tags_2 },
    { 0x66547009, SUC_NE_lexicon_tags_2 },
    { 0x1a44700a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1747013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78667017, SUC_NE_lexicon_tags_2 },
    { 0x91007017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c08701e, SUC_NE_lexicon_tags_2 },
    { 0xb0fa701f, SUC_NE_lexicon_tags_3 },
    { 0xb7847020, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecde7026, SUC_NE_lexicon_tags_7 },
    { 0x12c47027, SUC_NE_lexicon_tags_2 },
    { 0xc87e7028, SUC_NE_lexicon_tags_2 },
    { 0xe1c47026, SUC_NE_lexicon_tags_2 },
    { 0xe2f87026, SUC_NE_lexicon_tags_2 },
    { 0x6836702b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa6c702d, SUC_NE_lexicon_tags_2 },
    { 0x6a56702e, SUC_NE_lexicon_tags_2 },
    { 0x18e702e, SUC_NE_lexicon_tags_8 },
    { 0x46a4702d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b807032, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde0c7034, SUC_NE_lexicon_tags_2 },
    { 0x57667035, SUC_NE_lexicon_tags_32 },
    { 0x91967036, SUC_NE_lexicon_tags_2 },
    { 0xb2287036, SUC_NE_lexicon_tags_2 },
    { 0x6747037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd722703a, SUC_NE_lexicon_tags_2 },
    { 0x7d2c703b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x48a2703d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29647041, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d47048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d8704c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69727053, SUC_NE_lexicon_tags_2 },
    { 0xda907053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52705b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dc7062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca47068, SUC_NE_lexicon_tags_2 },
    { 0x4a27068, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3248706d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x493e706f, SUC_NE_lexicon_tags_2 },
    { 0x6d5c7070, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb627075, SUC_NE_lexicon_tags_2 },
    { 0x60ce7075, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2648707a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a0707f, SUC_NE_lexicon_tags_2 },
    { 0xec72707f, SUC_NE_lexicon_tags_2 },
    { 0xba4e7081, SUC_NE_lexicon_tags_2 },
    { 0x870e7082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee847084, SUC_NE_lexicon_tags_2 },
    { 0xb11e7085, SUC_NE_lexicon_tags_2 },
    { 0x2f9e7086, SUC_NE_lexicon_tags_2 },
    { 0xf3c7087, SUC_NE_lexicon_tags_3 },
    { 0x82567087, SUC_NE_lexicon_tags_2 },
    { 0xcbf67089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1e708d, SUC_NE_lexicon_tags_7 },
    { 0x73de708e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ec7091, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe882709a, SUC_NE_lexicon_tags_2 },
    { 0x50fa709b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ebe709d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6596709f, SUC_NE_lexicon_tags_36 },
    { 0x987870a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aa70a3, SUC_NE_lexicon_tags_2 },
    { 0xe8c670a3, SUC_NE_lexicon_tags_2 },
    { 0x653a70a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5470a9, SUC_NE_lexicon_tags_2 },
    { 0x489070aa, SUC_NE_lexicon_tags_2 },
    { 0x371a70ab, SUC_NE_lexicon_tags_2 },
    { 0xb57c70ac, SUC_NE_lexicon_tags_2 },
    { 0xe48a70ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bc70b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ce70b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849c70b8, SUC_NE_lexicon_tags_2 },
    { 0x4bb870b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x787470bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8670bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x292670c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88470c7, SUC_NE_lexicon_tags_2 },
    { 0xa16e70c8, SUC_NE_lexicon_tags_2 },
    { 0x868270c9, SUC_NE_lexicon_tags_2 },
    { 0xd52070c8, SUC_NE_lexicon_tags_2 },
    { 0xe5a670cb, SUC_NE_lexicon_tags_3 },
    { 0x4f3e70cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x211670cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b1c70d1, SUC_NE_lexicon_tags_2 },
    { 0x53de70d1, SUC_NE_lexicon_tags_3 },
    { 0x5d2070d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e70dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4270e2, SUC_NE_lexicon_tags_2 },
    { 0x49e670e3, SUC_NE_lexicon_tags_2 },
    { 0x212a70e4, SUC_NE_lexicon_tags_2 },
    { 0x6a1a70e5, SUC_NE_lexicon_tags_7 },
    { 0xdf9a70e2, SUC_NE_lexicon_tags_2 },
    { 0x72e70e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aae70ec, SUC_NE_lexicon_tags_2 },
    { 0x9d0470ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x920670ef, SUC_NE_lexicon_tags_2 },
    { 0x293070ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348470f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa45870f6, SUC_NE_lexicon_tags_2 },
    { 0x917470f7, SUC_NE_lexicon_tags_2 },
    { 0x73f670f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe070fc, SUC_NE_lexicon_tags_2 },
    { 0x805470fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x897070ff, SUC_NE_lexicon_tags_2 },
    { 0x53807100, SUC_NE_lexicon_tags_2 },
    { 0x4b3a7101, SUC_NE_lexicon_tags_2 },
    { 0x3b307102, SUC_NE_lexicon_tags_1 },
    { 0xad907103, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a7107, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x432c710a, SUC_NE_lexicon_tags_2 },
    { 0x8c3c710b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31747111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe7116, SUC_NE_lexicon_tags_2 },
    { 0xf8a07117, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7220711a, SUC_NE_lexicon_tags_2 },
    { 0xaea0711b, SUC_NE_lexicon_tags_2 },
    { 0x7292711a, SUC_NE_lexicon_tags_30 },
    { 0xe5fc711d, SUC_NE_lexicon_tags_2 },
    { 0x1e3a711e, SUC_NE_lexicon_tags_2 },
    { 0xc990711f, SUC_NE_lexicon_tags_2 },
    { 0xea127120, SUC_NE_lexicon_tags_24 },
    { 0xbf07120, SUC_NE_lexicon_tags_4 },
    { 0xed8e7122, SUC_NE_lexicon_tags_1 },
    { 0xec07123, SUC_NE_lexicon_tags_2 },
    { 0xc4b47122, SUC_NE_lexicon_tags_13 },
    { 0x8ecc7123, SUC_NE_lexicon_tags_2 },
    { 0x6100711d, SUC_NE_lexicon_tags_2 },
    { 0xffb0711d, SUC_NE_lexicon_tags_2 },
    { 0x46147126, SUC_NE_lexicon_tags_4 },
    { 0x94167129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703e7130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea1e7132, SUC_NE_lexicon_tags_2 },
    { 0x670c7133, SUC_NE_lexicon_tags_2 },
    { 0xecb67132, SUC_NE_lexicon_tags_2 },
    { 0x65bc7135, SUC_NE_lexicon_tags_2 },
    { 0x68c47134, SUC_NE_lexicon_tags_2 },
    { 0x39bc7137, SUC_NE_lexicon_tags_2 },
    { 0x745e7138, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282e713c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2713f, SUC_NE_lexicon_tags_2 },
    { 0x138e7140, SUC_NE_lexicon_tags_5 },
    { 0x6e367140, SUC_NE_lexicon_tags_2 },
    { 0x87bc7142, SUC_NE_lexicon_tags_2 },
    { 0xd6ee713f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97c67145, SUC_NE_lexicon_tags_2 },
    { 0xbfbc7146, SUC_NE_lexicon_tags_2 },
    { 0x535a7145, SUC_NE_lexicon_tags_2 },
    { 0x40227148, SUC_NE_lexicon_tags_2 },
    { 0x6ad07147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e94714d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0707151, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c587153, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x35647156, SUC_NE_lexicon_tags_2 },
    { 0xa627157, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b2c7159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76715c, SUC_NE_lexicon_tags_2 },
    { 0x452a715c, SUC_NE_lexicon_tags_3 },
    { 0xc932715e, SUC_NE_lexicon_tags_2 },
    { 0x99ac715d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6de7162, SUC_NE_lexicon_tags_2 },
    { 0x8afa7163, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5307167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4906716d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7682716f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43b07171, SUC_NE_lexicon_tags_2 },
    { 0xc3167171, SUC_NE_lexicon_tags_6 },
    { 0x70b67173, SUC_NE_lexicon_tags_2 },
    { 0xf14a7173, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x36187176, SUC_NE_lexicon_tags_2 },
    { 0x145e7177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27e47179, SUC_NE_lexicon_tags_4 },
    { 0x81dc717a, SUC_NE_lexicon_tags_2 },
    { 0x89787179, SUC_NE_lexicon_tags_2 },
    { 0x30167179, SUC_NE_lexicon_tags_4 },
    { 0xbeb8717d, SUC_NE_lexicon_tags_2 },
    { 0xef6e717e, SUC_NE_lexicon_tags_2 },
    { 0x3198717f, SUC_NE_lexicon_tags_2 },
    { 0xc430717e, SUC_NE_lexicon_tags_2 },
    { 0xadaa7181, SUC_NE_lexicon_tags_2 },
    { 0x110c717e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f727186, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3287189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ea718c, SUC_NE_lexicon_tags_2 },
    { 0xdf56718d, SUC_NE_lexicon_tags_2 },
    { 0x5240718d, SUC_NE_lexicon_tags_4 },
    { 0x56c8718f, SUC_NE_lexicon_tags_2 },
    { 0x7f87190, SUC_NE_lexicon_tags_2 },
    { 0x10927190, SUC_NE_lexicon_tags_2 },
    { 0x78aa718e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb0a7194, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c787196, SUC_NE_lexicon_tags_2 },
    { 0xcee87196, SUC_NE_lexicon_tags_2 },
    { 0x7dc67196, SUC_NE_lexicon_tags_2 },
    { 0x70807197, SUC_NE_lexicon_tags_2 },
    { 0xaa68719a, SUC_NE_lexicon_tags_2 },
    { 0x2d8a719b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71e4719d, SUC_NE_lexicon_tags_2 },
    { 0x5c82719e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd6471a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bc71a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe48e71a5, SUC_NE_lexicon_tags_3 },
    { 0xd9ac71a6, SUC_NE_lexicon_tags_2 },
    { 0x635e71a7, SUC_NE_lexicon_tags_2 },
    { 0x424471a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc10671aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84b871ac, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xe1a71ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa71b1, SUC_NE_lexicon_tags_2 },
    { 0xdb5071b2, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x31b671b4, SUC_NE_lexicon_tags_2 },
    { 0x51c71b5, SUC_NE_lexicon_tags_2 },
    { 0xa57a71b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248e71ba, SUC_NE_lexicon_tags_2 },
    { 0xf9be71bb, SUC_NE_lexicon_tags_2 },
    { 0x25ac71bb, SUC_NE_lexicon_tags_2 },
    { 0x3f8471ba, SUC_NE_lexicon_tags_2 },
    { 0xceea71be, SUC_NE_lexicon_tags_2 },
    { 0xcde671bf, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xef8671c1, SUC_NE_lexicon_tags_2 },
    { 0x4cd871c2, SUC_NE_lexicon_tags_2 },
    { 0x948871c3, SUC_NE_lexicon_tags_2 },
    { 0xc4ac71c3, SUC_NE_lexicon_tags_2 },
    { 0xbec471c5, SUC_NE_lexicon_tags_17 },
    { 0x3baa71c6, SUC_NE_lexicon_tags_2 },
    { 0xd3de71c7, SUC_NE_lexicon_tags_2 },
    { 0xe07471c8, SUC_NE_lexicon_tags_3 },
    { 0x7cc271c9, SUC_NE_lexicon_tags_2 },
    { 0x423271ca, SUC_NE_lexicon_tags_2 },
    { 0x7b271c9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99a71d3, SUC_NE_lexicon_tags_317 },
    { 0, NULL },
    { 0x431871d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3e71da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d271dd, SUC_NE_lexicon_tags_2 },
    { 0xcb1071de, SUC_NE_lexicon_tags_2 },
    { 0x51f471df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b071e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8871e6, SUC_NE_lexicon_tags_2 },
    { 0xde1c71e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08c71ea, SUC_NE_lexicon_tags_2 },
    { 0xa3dc71eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb9a71ed, SUC_NE_lexicon_tags_2 },
    { 0x2c5071ee, SUC_NE_lexicon_tags_2 },
    { 0x4f2c71ee, SUC_NE_lexicon_tags_3 },
    { 0x6eb871ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c071f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925071fb, SUC_NE_lexicon_tags_2 },
    { 0xbe0071fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6fe71fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7a7201, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bf07203, SUC_NE_lexicon_tags_2 },
    { 0xd9f27203, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x90227206, SUC_NE_lexicon_tags_2 },
    { 0xdd747206, SUC_NE_lexicon_tags_2 },
    { 0x76c27207, SUC_NE_lexicon_tags_18 },
    { 0x909c7209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73da720c, SUC_NE_lexicon_tags_1 },
    { 0x84be720c, SUC_NE_lexicon_tags_3 },
    { 0x94ca720e, SUC_NE_lexicon_tags_2 },
    { 0xf2fa720f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3627211, SUC_NE_lexicon_tags_2 },
    { 0x13ea7212, SUC_NE_lexicon_tags_2 },
    { 0x4adc7213, SUC_NE_lexicon_tags_2 },
    { 0x4aa27214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43067218, SUC_NE_lexicon_tags_2 },
    { 0x2a227218, SUC_NE_lexicon_tags_2 },
    { 0xc696721a, SUC_NE_lexicon_tags_40 },
    { 0xe376721a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d6721e, SUC_NE_lexicon_tags_2 },
    { 0x5df8721e, SUC_NE_lexicon_tags_2 },
    { 0xd4a2721f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97067226, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf34722c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b27233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb2723a, SUC_NE_lexicon_tags_2 },
    { 0x4f1a723b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c68723d, SUC_NE_lexicon_tags_2 },
    { 0x2c90723e, SUC_NE_lexicon_tags_2 },
    { 0xc9b6723f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x500a7241, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9e7246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8614724c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e0724f, SUC_NE_lexicon_tags_2 },
    { 0x8a1e724f, SUC_NE_lexicon_tags_4 },
    { 0xe6a67251, SUC_NE_lexicon_tags_3 },
    { 0xc7827252, SUC_NE_lexicon_tags_2 },
    { 0xb70c7252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17a47255, SUC_NE_lexicon_tags_2 },
    { 0xb387256, SUC_NE_lexicon_tags_2 },
    { 0x1db07257, SUC_NE_lexicon_tags_2 },
    { 0xb28a7257, SUC_NE_lexicon_tags_2 },
    { 0x4d267258, SUC_NE_lexicon_tags_2 },
    { 0x54ae725a, SUC_NE_lexicon_tags_2 },
    { 0x7306725b, SUC_NE_lexicon_tags_2 },
    { 0x2cb27255, SUC_NE_lexicon_tags_7 },
    { 0xa84725a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37e2725f, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0x87e67261, SUC_NE_lexicon_tags_2 },
    { 0x86b07262, SUC_NE_lexicon_tags_2 },
    { 0xe79a7263, SUC_NE_lexicon_tags_2 },
    { 0xe487263, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f387267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf7c7269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa05e726b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a726f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce7272, SUC_NE_lexicon_tags_2 },
    { 0xd5047273, SUC_NE_lexicon_tags_2 },
    { 0xf7c27272, SUC_NE_lexicon_tags_2 },
    { 0x3afa7275, SUC_NE_lexicon_tags_2 },
    { 0xc7a67275, SUC_NE_lexicon_tags_2 },
    { 0xadde7277, SUC_NE_lexicon_tags_2 },
    { 0x63de7278, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x616727b, SUC_NE_lexicon_tags_2 },
    { 0xa672727b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xceea727f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36007284, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb67289, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd722728b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb8728e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cf27290, SUC_NE_lexicon_tags_2 },
    { 0xee107290, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x477e7294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296a7298, SUC_NE_lexicon_tags_4 },
    { 0x10dc7299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786729e, SUC_NE_lexicon_tags_7 },
    { 0x8492729e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb4872a1, SUC_NE_lexicon_tags_2 },
    { 0x6c5072a2, SUC_NE_lexicon_tags_2 },
    { 0xb80e72a3, SUC_NE_lexicon_tags_7 },
    { 0xabf472a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25672a9, SUC_NE_lexicon_tags_2 },
    { 0xe66072a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43e072ac, SUC_NE_lexicon_tags_2 },
    { 0xc51472ad, SUC_NE_lexicon_tags_2 },
    { 0x8d0072ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715472b0, SUC_NE_lexicon_tags_57 },
    { 0x89ce72b1, SUC_NE_lexicon_tags_2 },
    { 0xb2f072b2, SUC_NE_lexicon_tags_2 },
    { 0x65b872b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac72bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453872c1, SUC_NE_lexicon_tags_2 },
    { 0xbc1a72c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e72c7, SUC_NE_lexicon_tags_2 },
    { 0xf7e872c8, SUC_NE_lexicon_tags_2 },
    { 0x73b672c9, SUC_NE_lexicon_tags_2 },
    { 0x9cf072ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaea72cc, SUC_NE_lexicon_tags_2 },
    { 0x97672cc, SUC_NE_lexicon_tags_2 },
    { 0x507e72ce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303072d2, SUC_NE_lexicon_tags_2 },
    { 0x461072d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897c72d7, SUC_NE_lexicon_tags_2 },
    { 0xa21872d8, SUC_NE_lexicon_tags_2 },
    { 0xdc8a72d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda672dd, SUC_NE_lexicon_tags_2 },
    { 0x797672de, SUC_NE_lexicon_tags_7 },
    { 0x92fe72df, SUC_NE_lexicon_tags_2 },
    { 0x590872df, SUC_NE_lexicon_tags_2 },
    { 0x469272e1, SUC_NE_lexicon_tags_3 },
    { 0x527072df, SUC_NE_lexicon_tags_2 },
    { 0x4a6a72e3, SUC_NE_lexicon_tags_2 },
    { 0xd00c72e4, SUC_NE_lexicon_tags_2 },
    { 0x69ee72e5, SUC_NE_lexicon_tags_2 },
    { 0xafc672e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x978072e9, SUC_NE_lexicon_tags_3 },
    { 0xfe1272e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe22872ec, SUC_NE_lexicon_tags_2 },
    { 0xd58272ed, SUC_NE_lexicon_tags_2 },
    { 0xcf0a72ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d7c72f0, SUC_NE_lexicon_tags_2 },
    { 0x1e7672f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a6872f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7e72f8, SUC_NE_lexicon_tags_2 },
    { 0xabc872f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a072fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bca72fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35387300, SUC_NE_lexicon_tags_1 },
    { 0x16287300, SUC_NE_lexicon_tags_1 },
    { 0x3ce07300, SUC_NE_lexicon_tags_7 },
    { 0x4fc27302, SUC_NE_lexicon_tags_2 },
    { 0x221c7302, SUC_NE_lexicon_tags_18 },
    { 0x57f87305, SUC_NE_lexicon_tags_2 },
    { 0x277e7300, SUC_NE_lexicon_tags_2 },
    { 0x50f27304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x505c730a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bc730d, SUC_NE_lexicon_tags_2 },
    { 0x856c730d, SUC_NE_lexicon_tags_2 },
    { 0x3b56730f, SUC_NE_lexicon_tags_2 },
    { 0x7f5e730f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e667312, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60807314, SUC_NE_lexicon_tags_2 },
    { 0x87f27315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x811c7318, SUC_NE_lexicon_tags_2 },
    { 0xd9507319, SUC_NE_lexicon_tags_2 },
    { 0x845a731a, SUC_NE_lexicon_tags_2 },
    { 0xe6de731b, SUC_NE_lexicon_tags_2 },
    { 0x5b36731c, SUC_NE_lexicon_tags_49 },
    { 0x4854731d, SUC_NE_lexicon_tags_3 },
    { 0xbad4731e, SUC_NE_lexicon_tags_2 },
    { 0x35ac731d, SUC_NE_lexicon_tags_2 },
    { 0x2a2e7320, SUC_NE_lexicon_tags_2 },
    { 0xefc27319, SUC_NE_lexicon_tags_2 },
    { 0x359e7320, SUC_NE_lexicon_tags_2 },
    { 0x1e7c7323, SUC_NE_lexicon_tags_2 },
    { 0x66bc731c, SUC_NE_lexicon_tags_2 },
    { 0xd4a67325, SUC_NE_lexicon_tags_2 },
    { 0x224a7320, SUC_NE_lexicon_tags_2 },
    { 0xe5587327, SUC_NE_lexicon_tags_2 },
    { 0x63bc7328, SUC_NE_lexicon_tags_2 },
    { 0xb52e7327, SUC_NE_lexicon_tags_2 },
    { 0xb672731a, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e732f, SUC_NE_lexicon_tags_4 },
    { 0x8d647330, SUC_NE_lexicon_tags_2 },
    { 0xc1ee732f, SUC_NE_lexicon_tags_2 },
    { 0xb8b07332, SUC_NE_lexicon_tags_2 },
    { 0xc63a732f, SUC_NE_lexicon_tags_4 },
    { 0xbf447334, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec733f, SUC_NE_lexicon_tags_0 },
    { 0x4ce47340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85027343, SUC_NE_lexicon_tags_2 },
    { 0x33047344, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93447346, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72e87348, SUC_NE_lexicon_tags_2 },
    { 0x8a227348, SUC_NE_lexicon_tags_2 },
    { 0xf5c6734a, SUC_NE_lexicon_tags_2 },
    { 0x23ba734b, SUC_NE_lexicon_tags_2 },
    { 0x996e734c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0734f, SUC_NE_lexicon_tags_3 },
    { 0x6ef27350, SUC_NE_lexicon_tags_2 },
    { 0x1fce7351, SUC_NE_lexicon_tags_117 },
    { 0x1b5e7350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81a7354, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e7357, SUC_NE_lexicon_tags_40 },
    { 0x2e387357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf184735a, SUC_NE_lexicon_tags_2 },
    { 0xe34e735a, SUC_NE_lexicon_tags_2 },
    { 0x4c8e735c, SUC_NE_lexicon_tags_2 },
    { 0xd422735b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa67360, SUC_NE_lexicon_tags_2 },
    { 0x2da7360, SUC_NE_lexicon_tags_2 },
    { 0xa8f67362, SUC_NE_lexicon_tags_2 },
    { 0x55827363, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x3847365, SUC_NE_lexicon_tags_2 },
    { 0x88087366, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd61c7368, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8b6736a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd818736c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8bc736e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc67374, SUC_NE_lexicon_tags_2 },
    { 0x7a2a7374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e8737c, SUC_NE_lexicon_tags_2 },
    { 0xea7e737d, SUC_NE_lexicon_tags_1 },
    { 0xa51c737d, SUC_NE_lexicon_tags_2 },
    { 0xc2a2737d, SUC_NE_lexicon_tags_2 },
    { 0xd696737d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58047384, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77167389, SUC_NE_lexicon_tags_2 },
    { 0x58738a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a8738d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e447390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c7394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a67397, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d06739a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf776739d, SUC_NE_lexicon_tags_2 },
    { 0xb2c739e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623673a3, SUC_NE_lexicon_tags_2 },
    { 0x33d873a3, SUC_NE_lexicon_tags_2 },
    { 0xef5273a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8873a7, SUC_NE_lexicon_tags_2 },
    { 0x3cd273a8, SUC_NE_lexicon_tags_2 },
    { 0xae6a73a7, SUC_NE_lexicon_tags_2 },
    { 0xe99473aa, SUC_NE_lexicon_tags_2 },
    { 0x50cc73ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a73b2, SUC_NE_lexicon_tags_2 },
    { 0xa8ec73b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232a73ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x525073bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823673c2, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x83fe73c4, SUC_NE_lexicon_tags_2 },
    { 0xe54473c5, SUC_NE_lexicon_tags_2 },
    { 0xeca473c5, SUC_NE_lexicon_tags_2 },
    { 0x922273c7, SUC_NE_lexicon_tags_2 },
    { 0x6b6a73c7, SUC_NE_lexicon_tags_2 },
    { 0xc3c673c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a073cc, SUC_NE_lexicon_tags_2 },
    { 0x4a4273cd, SUC_NE_lexicon_tags_2 },
    { 0x5b473cd, SUC_NE_lexicon_tags_0 },
    { 0xbd073cf, SUC_NE_lexicon_tags_2 },
    { 0xd97873d0, SUC_NE_lexicon_tags_2 },
    { 0xaa9c73cc, SUC_NE_lexicon_tags_2 },
    { 0xfe5a73cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x957873d4, SUC_NE_lexicon_tags_17 },
    { 0x5c5273d5, SUC_NE_lexicon_tags_4 },
    { 0xd11a73d6, SUC_NE_lexicon_tags_2 },
    { 0x6b5873d7, SUC_NE_lexicon_tags_2 },
    { 0x3f4273d8, SUC_NE_lexicon_tags_2 },
    { 0xc18473d6, SUC_NE_lexicon_tags_4 },
    { 0x539a73d8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d873dd, SUC_NE_lexicon_tags_2 },
    { 0x542473dd, SUC_NE_lexicon_tags_2 },
    { 0x46a673dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ae73e6, SUC_NE_lexicon_tags_2 },
    { 0xbab673e7, SUC_NE_lexicon_tags_30 },
    { 0x7b9a73e8, SUC_NE_lexicon_tags_2 },
    { 0x8d1273e9, SUC_NE_lexicon_tags_3 },
    { 0xc0b873e6, SUC_NE_lexicon_tags_2 },
    { 0x956873eb, SUC_NE_lexicon_tags_2 },
    { 0xe60073e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24273f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x584273f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x139673f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b873fb, SUC_NE_lexicon_tags_2 },
    { 0x42b473fb, SUC_NE_lexicon_tags_2 },
    { 0xfe8a73fd, SUC_NE_lexicon_tags_2 },
    { 0xfb0673fe, SUC_NE_lexicon_tags_1 },
    { 0xc61073ff, SUC_NE_lexicon_tags_2 },
    { 0x1acc73fd, SUC_NE_lexicon_tags_18 },
    { 0x28367401, SUC_NE_lexicon_tags_4 },
    { 0x96547402, SUC_NE_lexicon_tags_2 },
    { 0x139c7400, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249e7407, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8c740a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86a4740c, SUC_NE_lexicon_tags_2 },
    { 0xda8740d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a04740f, SUC_NE_lexicon_tags_83 },
    { 0x8a20740f, SUC_NE_lexicon_tags_2 },
    { 0x4834740f, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xb967413, SUC_NE_lexicon_tags_2 },
    { 0x16a67413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c67419, SUC_NE_lexicon_tags_2 },
    { 0x1fc6741a, SUC_NE_lexicon_tags_2 },
    { 0xff98741b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8aa741d, SUC_NE_lexicon_tags_3 },
    { 0x7bba741d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e87427, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662742c, SUC_NE_lexicon_tags_2 },
    { 0xed0c742d, SUC_NE_lexicon_tags_2 },
    { 0xada2742e, SUC_NE_lexicon_tags_4 },
    { 0x5556742e, SUC_NE_lexicon_tags_17 },
    { 0x6850742d, SUC_NE_lexicon_tags_2 },
    { 0xba747430, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b47439, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7c743c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x38ae743e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbb27440, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa67445, SUC_NE_lexicon_tags_2 },
    { 0xafe47446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6987449, SUC_NE_lexicon_tags_278 },
    { 0x22e8744a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a744e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26c7450, SUC_NE_lexicon_tags_2 },
    { 0x4dd67451, SUC_NE_lexicon_tags_2 },
    { 0x29747451, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa234745a, SUC_NE_lexicon_tags_2 },
    { 0xd77e745a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac3a745d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bfc745f, SUC_NE_lexicon_tags_2 },
    { 0xca0c745f, SUC_NE_lexicon_tags_34 },
    { 0x5e027461, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x5ffc7463, SUC_NE_lexicon_tags_2 },
    { 0x13e67464, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e627466, SUC_NE_lexicon_tags_7 },
    { 0x9fe27467, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f8746c, SUC_NE_lexicon_tags_2 },
    { 0x6d32746c, SUC_NE_lexicon_tags_2 },
    { 0x1398746e, SUC_NE_lexicon_tags_2 },
    { 0x2e56746f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x67f07471, SUC_NE_lexicon_tags_2 },
    { 0x6d07471, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63487477, SUC_NE_lexicon_tags_7 },
    { 0xb1a87478, SUC_NE_lexicon_tags_2 },
    { 0xaea27479, SUC_NE_lexicon_tags_2 },
    { 0x7f467478, SUC_NE_lexicon_tags_3 },
    { 0xdc5a747b, SUC_NE_lexicon_tags_319 },
    { 0xb97c747c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12f0747e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fda7480, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x653e7483, SUC_NE_lexicon_tags_102 },
    { 0xecac7484, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a527489, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d8748e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4527491, SUC_NE_lexicon_tags_2 },
    { 0x1a527492, SUC_NE_lexicon_tags_2 },
    { 0x2ac7493, SUC_NE_lexicon_tags_2 },
    { 0xfcc07492, SUC_NE_lexicon_tags_2 },
    { 0x4e1c7492, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42927498, SUC_NE_lexicon_tags_2 },
    { 0xabde7499, SUC_NE_lexicon_tags_2 },
    { 0xc1ba749a, SUC_NE_lexicon_tags_2 },
    { 0x879c749b, SUC_NE_lexicon_tags_2 },
    { 0xda3a749a, SUC_NE_lexicon_tags_2 },
    { 0xddf0749c, SUC_NE_lexicon_tags_4 },
    { 0xd3f0749e, SUC_NE_lexicon_tags_2 },
    { 0xa74a749f, SUC_NE_lexicon_tags_2 },
    { 0xda2474a0, SUC_NE_lexicon_tags_2 },
    { 0x16f674a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cc74a4, SUC_NE_lexicon_tags_2 },
    { 0xf65e74a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb29e74a7, SUC_NE_lexicon_tags_8 },
    { 0xc7c074a8, SUC_NE_lexicon_tags_2 },
    { 0x8cc674a8, SUC_NE_lexicon_tags_2 },
    { 0x663274aa, SUC_NE_lexicon_tags_2 },
    { 0x752274ab, SUC_NE_lexicon_tags_7 },
    { 0x127874aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf0c74ae, SUC_NE_lexicon_tags_2 },
    { 0xc05874ae, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7c074b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f474b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x771e74b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f6c74bb, SUC_NE_lexicon_tags_2 },
    { 0xbaec74bc, SUC_NE_lexicon_tags_2 },
    { 0xb50a74bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c874c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x340274c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef474cb, SUC_NE_lexicon_tags_2 },
    { 0x259e74cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e674d3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663e74db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218474e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x911e74e5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x959874e8, SUC_NE_lexicon_tags_2 },
    { 0xe1b474e9, SUC_NE_lexicon_tags_32 },
    { 0x4a4874ea, SUC_NE_lexicon_tags_2 },
    { 0xbab874eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe64074ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c74f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4274f3, SUC_NE_lexicon_tags_2 },
    { 0x5ac474f3, SUC_NE_lexicon_tags_2 },
    { 0x2e3a74f4, SUC_NE_lexicon_tags_2 },
    { 0xdac874f6, SUC_NE_lexicon_tags_1 },
    { 0x5ed674f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x918674f9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d5074fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83e74fd, SUC_NE_lexicon_tags_2 },
    { 0xf7fa74fe, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0a7501, SUC_NE_lexicon_tags_2 },
    { 0x4c647501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf027506, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91ae7508, SUC_NE_lexicon_tags_2 },
    { 0xe6087509, SUC_NE_lexicon_tags_2 },
    { 0x9dee7508, SUC_NE_lexicon_tags_2 },
    { 0x2ae87508, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97627510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef27515, SUC_NE_lexicon_tags_32 },
    { 0x47867516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63b67518, SUC_NE_lexicon_tags_2 },
    { 0x87827518, SUC_NE_lexicon_tags_3 },
    { 0x162c7519, SUC_NE_lexicon_tags_4 },
    { 0x3f967518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82167520, SUC_NE_lexicon_tags_4 },
    { 0x449e7521, SUC_NE_lexicon_tags_2 },
    { 0xae127522, SUC_NE_lexicon_tags_2 },
    { 0x1cec7523, SUC_NE_lexicon_tags_2 },
    { 0x49c27524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dc7527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b8752b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8752f, SUC_NE_lexicon_tags_2 },
    { 0xb1e7530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3e7533, SUC_NE_lexicon_tags_2 },
    { 0xcbc87534, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90347536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac6753a, SUC_NE_lexicon_tags_3 },
    { 0xfd90753b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58ee753d, SUC_NE_lexicon_tags_2 },
    { 0x6b8e753e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8827540, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89e7543, SUC_NE_lexicon_tags_2 },
    { 0xd7807544, SUC_NE_lexicon_tags_2 },
    { 0x974a7545, SUC_NE_lexicon_tags_2 },
    { 0xb14a7545, SUC_NE_lexicon_tags_2 },
    { 0x6987547, SUC_NE_lexicon_tags_2 },
    { 0x17867547, SUC_NE_lexicon_tags_2 },
    { 0x45fa7549, SUC_NE_lexicon_tags_2 },
    { 0x60fa7549, SUC_NE_lexicon_tags_2 },
    { 0x7f6c754b, SUC_NE_lexicon_tags_2 },
    { 0xcc74754c, SUC_NE_lexicon_tags_2 },
    { 0xb9aa754a, SUC_NE_lexicon_tags_2 },
    { 0xd04c7545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba707551, SUC_NE_lexicon_tags_2 },
    { 0x397e7552, SUC_NE_lexicon_tags_2 },
    { 0xf3627552, SUC_NE_lexicon_tags_2 },
    { 0xb6d47553, SUC_NE_lexicon_tags_2 },
    { 0x2f287555, SUC_NE_lexicon_tags_1 },
    { 0xe2207551, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ec7559, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4c755c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f4755f, SUC_NE_lexicon_tags_17 },
    { 0xe79a7560, SUC_NE_lexicon_tags_23 },
    { 0xbaea7561, SUC_NE_lexicon_tags_2 },
    { 0xbafc7562, SUC_NE_lexicon_tags_2 },
    { 0x326e7562, SUC_NE_lexicon_tags_2 },
    { 0x6ca07564, SUC_NE_lexicon_tags_2 },
    { 0x19527561, SUC_NE_lexicon_tags_2 },
    { 0x42dc7566, SUC_NE_lexicon_tags_0 },
    { 0xb91c7567, SUC_NE_lexicon_tags_2 },
    { 0xfa1a7567, SUC_NE_lexicon_tags_2 },
    { 0xf6b67569, SUC_NE_lexicon_tags_2 },
    { 0xe138756a, SUC_NE_lexicon_tags_14 },
    { 0xddb27569, SUC_NE_lexicon_tags_2 },
    { 0x5c18756b, SUC_NE_lexicon_tags_2 },
    { 0xb574756d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46a7571, SUC_NE_lexicon_tags_2 },
    { 0x9cf67572, SUC_NE_lexicon_tags_2 },
    { 0x4af67573, SUC_NE_lexicon_tags_2 },
    { 0x1dfc7574, SUC_NE_lexicon_tags_2 },
    { 0x24287574, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754c7579, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57a757d, SUC_NE_lexicon_tags_10 },
    { 0x3b2e757e, SUC_NE_lexicon_tags_2 },
    { 0x7536757d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x910c7581, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2d467583, SUC_NE_lexicon_tags_2 },
    { 0x9db27584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc7589, SUC_NE_lexicon_tags_2 },
    { 0x965e758a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab68758d, SUC_NE_lexicon_tags_2 },
    { 0xadbe758e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0747590, SUC_NE_lexicon_tags_2 },
    { 0x80a67590, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba7596, SUC_NE_lexicon_tags_2 },
    { 0xecf87597, SUC_NE_lexicon_tags_2 },
    { 0x5c387598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe916759c, SUC_NE_lexicon_tags_2 },
    { 0xfd8e759d, SUC_NE_lexicon_tags_2 },
    { 0xe3ba759e, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xadd875a0, SUC_NE_lexicon_tags_2 },
    { 0x9f6075a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec675a4, SUC_NE_lexicon_tags_2 },
    { 0xbb9675a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec75a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a75ab, SUC_NE_lexicon_tags_2 },
    { 0xfcbc75ab, SUC_NE_lexicon_tags_2 },
    { 0x79f875ad, SUC_NE_lexicon_tags_2 },
    { 0x854875ae, SUC_NE_lexicon_tags_2 },
    { 0x71a75af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781a75b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d675bb, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3475c0, SUC_NE_lexicon_tags_2 },
    { 0xfafc75c0, SUC_NE_lexicon_tags_2 },
    { 0x457675c2, SUC_NE_lexicon_tags_4 },
    { 0x167c75c1, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x76a475c5, SUC_NE_lexicon_tags_2 },
    { 0x3b5875c6, SUC_NE_lexicon_tags_2 },
    { 0xf0c875c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f1875c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x625475cb, SUC_NE_lexicon_tags_2 },
    { 0x69e675cc, SUC_NE_lexicon_tags_2 },
    { 0xef9275cd, SUC_NE_lexicon_tags_220 },
    { 0xc0875cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e75d3, SUC_NE_lexicon_tags_2 },
    { 0x8be875d4, SUC_NE_lexicon_tags_2 },
    { 0x4fa875d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb05075d7, SUC_NE_lexicon_tags_2 },
    { 0x57ee75d7, SUC_NE_lexicon_tags_2 },
    { 0x669a75d9, SUC_NE_lexicon_tags_2 },
    { 0xa6b275d8, SUC_NE_lexicon_tags_2 },
    { 0xe85875d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e275e1, SUC_NE_lexicon_tags_2 },
    { 0xc1d475e1, SUC_NE_lexicon_tags_2 },
    { 0xd78a75e3, SUC_NE_lexicon_tags_0 },
    { 0xc60275e1, SUC_NE_lexicon_tags_34 },
    { 0xdade75e5, SUC_NE_lexicon_tags_2 },
    { 0xd0ca75e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c7675e8, SUC_NE_lexicon_tags_2 },
    { 0x41ba75e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f875ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49e75ee, SUC_NE_lexicon_tags_2 },
    { 0x6ef475ef, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1e75f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63275f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40075f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d875fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d675ff, SUC_NE_lexicon_tags_2 },
    { 0x3fcc75ff, SUC_NE_lexicon_tags_2 },
    { 0xb42e7601, SUC_NE_lexicon_tags_18 },
    { 0xe3b07601, SUC_NE_lexicon_tags_5 },
    { 0x11767603, SUC_NE_lexicon_tags_2 },
    { 0x967e7604, SUC_NE_lexicon_tags_2 },
    { 0xfb867601, SUC_NE_lexicon_tags_2 },
    { 0x59807606, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88647608, SUC_NE_lexicon_tags_2 },
    { 0xd4ec7609, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x762e760b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18760e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50767615, SUC_NE_lexicon_tags_2 },
    { 0xcb347616, SUC_NE_lexicon_tags_2 },
    { 0xe4a67615, SUC_NE_lexicon_tags_2 },
    { 0x95947618, SUC_NE_lexicon_tags_2 },
    { 0xde2e7619, SUC_NE_lexicon_tags_2 },
    { 0xbd047619, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb968761c, SUC_NE_lexicon_tags_4 },
    { 0x94a6761d, SUC_NE_lexicon_tags_2 },
    { 0x3e62761c, SUC_NE_lexicon_tags_2 },
    { 0xaa90761f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe87623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6007625, SUC_NE_lexicon_tags_2 },
    { 0xf3887626, SUC_NE_lexicon_tags_2 },
    { 0x9087626, SUC_NE_lexicon_tags_2 },
    { 0x675c7628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9ea762a, SUC_NE_lexicon_tags_13 },
    { 0x246c762a, SUC_NE_lexicon_tags_2 },
    { 0x8b9a762c, SUC_NE_lexicon_tags_2 },
    { 0x3460762d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc447633, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf787636, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f8763a, SUC_NE_lexicon_tags_7 },
    { 0x4a02763a, SUC_NE_lexicon_tags_5 },
    { 0xe4c6763c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9027641, SUC_NE_lexicon_tags_2 },
    { 0xe21e7642, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5a7646, SUC_NE_lexicon_tags_2 },
    { 0x41527646, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ca764c, SUC_NE_lexicon_tags_2 },
    { 0x4d0c764d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32327653, SUC_NE_lexicon_tags_2 },
    { 0x1acc7654, SUC_NE_lexicon_tags_2 },
    { 0x5c6e7655, SUC_NE_lexicon_tags_2 },
    { 0xa7567656, SUC_NE_lexicon_tags_2 },
    { 0x3f267656, SUC_NE_lexicon_tags_2 },
    { 0x827c7654, SUC_NE_lexicon_tags_2 },
    { 0xb8427659, SUC_NE_lexicon_tags_2 },
    { 0x5de67659, SUC_NE_lexicon_tags_2 },
    { 0xd324765b, SUC_NE_lexicon_tags_2 },
    { 0xdf4765c, SUC_NE_lexicon_tags_2 },
    { 0xfcec765d, SUC_NE_lexicon_tags_2 },
    { 0x1bda765e, SUC_NE_lexicon_tags_0 },
    { 0xc7ec765f, SUC_NE_lexicon_tags_2 },
    { 0x83ea765e, SUC_NE_lexicon_tags_2 },
    { 0x6ba87659, SUC_NE_lexicon_tags_3 },
    { 0x90e07662, SUC_NE_lexicon_tags_4 },
    { 0xef907662, SUC_NE_lexicon_tags_2 },
    { 0xa82a765a, SUC_NE_lexicon_tags_2 },
    { 0x2e765e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbbe7667, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78f67669, SUC_NE_lexicon_tags_2 },
    { 0xcf1c766a, SUC_NE_lexicon_tags_2 },
    { 0xa99a7669, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6e74766d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5974766f, SUC_NE_lexicon_tags_2 },
    { 0x7c68766f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63027672, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea27675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b47679, SUC_NE_lexicon_tags_2 },
    { 0x72d87679, SUC_NE_lexicon_tags_2 },
    { 0x44667679, SUC_NE_lexicon_tags_2 },
    { 0x7d3e767c, SUC_NE_lexicon_tags_2 },
    { 0x8ea767d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10c8767f, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be7684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f927686, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7324768c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7396768e, SUC_NE_lexicon_tags_2 },
    { 0xe11a768f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eb47691, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b5c7693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52867697, SUC_NE_lexicon_tags_2 },
    { 0x8167698, SUC_NE_lexicon_tags_1 },
    { 0x1b1e7699, SUC_NE_lexicon_tags_2 },
    { 0xf3ba769a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3c769d, SUC_NE_lexicon_tags_2 },
    { 0x15a0769e, SUC_NE_lexicon_tags_2 },
    { 0x552e769e, SUC_NE_lexicon_tags_1 },
    { 0xe1f076a0, SUC_NE_lexicon_tags_2 },
    { 0xc5d476a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06c76a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcce676a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c76af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ca76b2, SUC_NE_lexicon_tags_2 },
    { 0xbf6276b3, SUC_NE_lexicon_tags_19 },
    { 0xbc8076b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf57076b6, SUC_NE_lexicon_tags_48 },
    { 0x4df876b6, SUC_NE_lexicon_tags_7 },
    { 0xe9d276b8, SUC_NE_lexicon_tags_2 },
    { 0xd53276b8, SUC_NE_lexicon_tags_2 },
    { 0x869276ba, SUC_NE_lexicon_tags_2 },
    { 0xdbce76bb, SUC_NE_lexicon_tags_2 },
    { 0x5abe76b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce076c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c476ca, SUC_NE_lexicon_tags_2 },
    { 0xf43c76ca, SUC_NE_lexicon_tags_2 },
    { 0x49e076cc, SUC_NE_lexicon_tags_2 },
    { 0x8b1276cb, SUC_NE_lexicon_tags_2 },
    { 0xaeb676cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3476d2, SUC_NE_lexicon_tags_2 },
    { 0xc36876d2, SUC_NE_lexicon_tags_2 },
    { 0x2c4c76d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe40076d6, SUC_NE_lexicon_tags_2 },
    { 0x3a0c76d7, SUC_NE_lexicon_tags_2 },
    { 0x47fc76d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e076da, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fe76de, SUC_NE_lexicon_tags_2 },
    { 0x886a76de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52676e3, SUC_NE_lexicon_tags_2 },
    { 0x98ac76e3, SUC_NE_lexicon_tags_2 },
    { 0x998e76e5, SUC_NE_lexicon_tags_2 },
    { 0x823c76e6, SUC_NE_lexicon_tags_2 },
    { 0xc27676e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd25276e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c8a76eb, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0xe55876ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x943476f0, SUC_NE_lexicon_tags_24 },
    { 0x83b676f1, SUC_NE_lexicon_tags_2 },
    { 0xadae76f2, SUC_NE_lexicon_tags_67 },
    { 0x41d876f1, SUC_NE_lexicon_tags_2 },
    { 0x790a76f4, SUC_NE_lexicon_tags_2 },
    { 0x6a876f4, SUC_NE_lexicon_tags_1 },
    { 0x9476f6, SUC_NE_lexicon_tags_2 },
    { 0xecf676f5, SUC_NE_lexicon_tags_2 },
    { 0xd31876f8, SUC_NE_lexicon_tags_2 },
    { 0x91a076f9, SUC_NE_lexicon_tags_2 },
    { 0xf1ec76fa, SUC_NE_lexicon_tags_2 },
    { 0xa4dc76fa, SUC_NE_lexicon_tags_2 },
    { 0xbc5e76fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc67700, SUC_NE_lexicon_tags_5 },
    { 0x322a7701, SUC_NE_lexicon_tags_5 },
    { 0xd9ea7701, SUC_NE_lexicon_tags_3 },
    { 0x97c67702, SUC_NE_lexicon_tags_2 },
    { 0x47ae7704, SUC_NE_lexicon_tags_5 },
    { 0x7a647705, SUC_NE_lexicon_tags_2 },
    { 0xdb547705, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff4770b, SUC_NE_lexicon_tags_4 },
    { 0xb87a770b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec47711, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeac7714, SUC_NE_lexicon_tags_2 },
    { 0x57587715, SUC_NE_lexicon_tags_2 },
    { 0x218e7716, SUC_NE_lexicon_tags_2 },
    { 0xdea7717, SUC_NE_lexicon_tags_2 },
    { 0x775c7715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ee771d, SUC_NE_lexicon_tags_2 },
    { 0xafc2771d, SUC_NE_lexicon_tags_5 },
    { 0xe70a771e, SUC_NE_lexicon_tags_2 },
    { 0x1ec7720, SUC_NE_lexicon_tags_2 },
    { 0xb00a7720, SUC_NE_lexicon_tags_2 },
    { 0x438e7720, SUC_NE_lexicon_tags_2 },
    { 0xb4e27723, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381c7729, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5306772c, SUC_NE_lexicon_tags_2 },
    { 0xfcf0772c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9327732, SUC_NE_lexicon_tags_2 },
    { 0xe8b47733, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1c7739, SUC_NE_lexicon_tags_2 },
    { 0x8194773a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4c7740, SUC_NE_lexicon_tags_2 },
    { 0x959e7740, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9d67743, SUC_NE_lexicon_tags_2 },
    { 0xbda87744, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc7749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf482774d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15167751, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed27754, SUC_NE_lexicon_tags_2 },
    { 0x75e07755, SUC_NE_lexicon_tags_2 },
    { 0x552e7755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa27c7758, SUC_NE_lexicon_tags_2 },
    { 0x42f47759, SUC_NE_lexicon_tags_2 },
    { 0x29dc775a, SUC_NE_lexicon_tags_2 },
    { 0x877c775b, SUC_NE_lexicon_tags_2 },
    { 0xcd687759, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a4775f, SUC_NE_lexicon_tags_2 },
    { 0x1fea7760, SUC_NE_lexicon_tags_184 },
    { 0xe6c87761, SUC_NE_lexicon_tags_2 },
    { 0x1fd87762, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa47765, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9afa7767, SUC_NE_lexicon_tags_2 },
    { 0x25b87767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d87774, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x36e47776, SUC_NE_lexicon_tags_34 },
    { 0xf5c7777, SUC_NE_lexicon_tags_2 },
    { 0x5c9e7777, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc350777b, SUC_NE_lexicon_tags_2 },
    { 0x6f64777c, SUC_NE_lexicon_tags_2 },
    { 0xabe8777c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d607782, SUC_NE_lexicon_tags_18 },
    { 0x7bfa7782, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43787786, SUC_NE_lexicon_tags_0 },
    { 0x56b27787, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1227789, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea778c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f48778e, SUC_NE_lexicon_tags_2 },
    { 0x9716778e, SUC_NE_lexicon_tags_2 },
    { 0x7d967790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcc27792, SUC_NE_lexicon_tags_2 },
    { 0x6ece7793, SUC_NE_lexicon_tags_2 },
    { 0xb4a27794, SUC_NE_lexicon_tags_2 },
    { 0x1667792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a07799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52e6779b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21ec779d, SUC_NE_lexicon_tags_2 },
    { 0xef26779d, SUC_NE_lexicon_tags_2 },
    { 0x7f2779e, SUC_NE_lexicon_tags_2 },
    { 0x51e677a0, SUC_NE_lexicon_tags_2 },
    { 0xcf90779f, SUC_NE_lexicon_tags_2 },
    { 0xb3477a2, SUC_NE_lexicon_tags_2 },
    { 0xdc7877a2, SUC_NE_lexicon_tags_2 },
    { 0x381277a4, SUC_NE_lexicon_tags_2 },
    { 0xa16877a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x615c77a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8a077aa, SUC_NE_lexicon_tags_2 },
    { 0xd94a77ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbe77b0, SUC_NE_lexicon_tags_7 },
    { 0x94da77b1, SUC_NE_lexicon_tags_2 },
    { 0xae9877b2, SUC_NE_lexicon_tags_2 },
    { 0x5b3877b2, SUC_NE_lexicon_tags_2 },
    { 0x840077b4, SUC_NE_lexicon_tags_2 },
    { 0xa63677b5, SUC_NE_lexicon_tags_2 },
    { 0xc5e77b6, SUC_NE_lexicon_tags_2 },
    { 0x3ee277b6, SUC_NE_lexicon_tags_2 },
    { 0xd24e77b8, SUC_NE_lexicon_tags_2 },
    { 0xa2e277b9, SUC_NE_lexicon_tags_2 },
    { 0x3d0c77ba, SUC_NE_lexicon_tags_2 },
    { 0x131c77b5, SUC_NE_lexicon_tags_2 },
    { 0x925877bc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x117e77be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x641477c0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cc77c6, SUC_NE_lexicon_tags_40 },
    { 0x93677c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0fe77c9, SUC_NE_lexicon_tags_2 },
    { 0x64ac77ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaaa077cc, SUC_NE_lexicon_tags_4 },
    { 0xd0077cc, SUC_NE_lexicon_tags_2 },
    { 0x94e677ce, SUC_NE_lexicon_tags_2 },
    { 0x900a77ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16077d2, SUC_NE_lexicon_tags_2 },
    { 0x86ac77d2, SUC_NE_lexicon_tags_2 },
    { 0xf57a77d2, SUC_NE_lexicon_tags_2 },
    { 0x5e8477d4, SUC_NE_lexicon_tags_2 },
    { 0x208077d4, SUC_NE_lexicon_tags_2 },
    { 0xe75877d7, SUC_NE_lexicon_tags_2 },
    { 0xe03c77d6, SUC_NE_lexicon_tags_2 },
    { 0x4a0c77d5, SUC_NE_lexicon_tags_2 },
    { 0x29e077da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8877dd, SUC_NE_lexicon_tags_2 },
    { 0x747277de, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc77e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf9677e3, SUC_NE_lexicon_tags_2 },
    { 0x789a77e4, SUC_NE_lexicon_tags_2 },
    { 0x58c077e5, SUC_NE_lexicon_tags_2 },
    { 0x4d2e77e5, SUC_NE_lexicon_tags_2 },
    { 0x8bac77e6, SUC_NE_lexicon_tags_2 },
    { 0x14fc77e7, SUC_NE_lexicon_tags_2 },
    { 0x207477e9, SUC_NE_lexicon_tags_2 },
    { 0xc57a77ea, SUC_NE_lexicon_tags_7 },
    { 0x483a77eb, SUC_NE_lexicon_tags_2 },
    { 0xbc8277eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e3a77ee, SUC_NE_lexicon_tags_2 },
    { 0x706c77ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x590477f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77bc77f3, SUC_NE_lexicon_tags_2 },
    { 0xea8a77f4, SUC_NE_lexicon_tags_2 },
    { 0x650477f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9277f8, SUC_NE_lexicon_tags_3 },
    { 0x3a5077f9, SUC_NE_lexicon_tags_2 },
    { 0x4e1877f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e87800, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c747803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x596c7805, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc780d, SUC_NE_lexicon_tags_2 },
    { 0xb5cc780e, SUC_NE_lexicon_tags_2 },
    { 0x297a780e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f27812, SUC_NE_lexicon_tags_2 },
    { 0x93487813, SUC_NE_lexicon_tags_2 },
    { 0xf5e87813, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d07816, SUC_NE_lexicon_tags_185 },
    { 0x59f67817, SUC_NE_lexicon_tags_2 },
    { 0x948a7817, SUC_NE_lexicon_tags_2 },
    { 0x9ad07818, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xe1bc781b, SUC_NE_lexicon_tags_2 },
    { 0x2048781c, SUC_NE_lexicon_tags_2 },
    { 0x5156781b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24047820, SUC_NE_lexicon_tags_2 },
    { 0xc5d47821, SUC_NE_lexicon_tags_2 },
    { 0x6eb87822, SUC_NE_lexicon_tags_2 },
    { 0xae347823, SUC_NE_lexicon_tags_2 },
    { 0x8b07823, SUC_NE_lexicon_tags_30 },
    { 0xc8f07825, SUC_NE_lexicon_tags_2 },
    { 0xda307826, SUC_NE_lexicon_tags_2 },
    { 0x19467826, SUC_NE_lexicon_tags_2 },
    { 0x45c07828, SUC_NE_lexicon_tags_2 },
    { 0x4b7c7829, SUC_NE_lexicon_tags_2 },
    { 0x85107829, SUC_NE_lexicon_tags_2 },
    { 0x396782b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e87830, SUC_NE_lexicon_tags_2 },
    { 0xe0367831, SUC_NE_lexicon_tags_2 },
    { 0x7d847832, SUC_NE_lexicon_tags_2 },
    { 0xd8207833, SUC_NE_lexicon_tags_2 },
    { 0x6a647833, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8a7839, SUC_NE_lexicon_tags_2 },
    { 0x2a7a7839, SUC_NE_lexicon_tags_2 },
    { 0xe11c783b, SUC_NE_lexicon_tags_2 },
    { 0xcb0a783a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b87840, SUC_NE_lexicon_tags_2 },
    { 0x61dc7840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35387844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2a7849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223e784f, SUC_NE_lexicon_tags_2 },
    { 0x3a0e784f, SUC_NE_lexicon_tags_2 },
    { 0xb6907851, SUC_NE_lexicon_tags_2 },
    { 0x9d9c7852, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8127854, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a367857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7936785a, SUC_NE_lexicon_tags_2 },
    { 0x32b6785b, SUC_NE_lexicon_tags_2 },
    { 0x8f22785c, SUC_NE_lexicon_tags_2 },
    { 0x4136785a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94247862, SUC_NE_lexicon_tags_4 },
    { 0x84247863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589c7868, SUC_NE_lexicon_tags_2 },
    { 0x87ba7869, SUC_NE_lexicon_tags_2 },
    { 0x161e786a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe96786d, SUC_NE_lexicon_tags_2 },
    { 0x895c786e, SUC_NE_lexicon_tags_2 },
    { 0x81d4786f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41f47871, SUC_NE_lexicon_tags_2 },
    { 0x76a47872, SUC_NE_lexicon_tags_2 },
    { 0x2a27873, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c7876, SUC_NE_lexicon_tags_2 },
    { 0x3ea87877, SUC_NE_lexicon_tags_2 },
    { 0x63487878, SUC_NE_lexicon_tags_2 },
    { 0x75027879, SUC_NE_lexicon_tags_17 },
    { 0x6b8c7878, SUC_NE_lexicon_tags_2 },
    { 0xe3d07877, SUC_NE_lexicon_tags_75 },
    { 0x7c727879, SUC_NE_lexicon_tags_2 },
    { 0x9dfc7877, SUC_NE_lexicon_tags_2 },
    { 0x536c787e, SUC_NE_lexicon_tags_2 },
    { 0xb5fc787f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a87886, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d4788c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd147892, SUC_NE_lexicon_tags_2 },
    { 0x99ce7893, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc98c7895, SUC_NE_lexicon_tags_2 },
    { 0xb26a7896, SUC_NE_lexicon_tags_2 },
    { 0x3a487897, SUC_NE_lexicon_tags_2 },
    { 0xa2467896, SUC_NE_lexicon_tags_2 },
    { 0xff947895, SUC_NE_lexicon_tags_2 },
    { 0x4992789a, SUC_NE_lexicon_tags_2 },
    { 0x5404789b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e4789e, SUC_NE_lexicon_tags_7 },
    { 0x7530789f, SUC_NE_lexicon_tags_2 },
    { 0x860c78a0, SUC_NE_lexicon_tags_2 },
    { 0x761e78a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee278a6, SUC_NE_lexicon_tags_40 },
    { 0x563678a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6478ab, SUC_NE_lexicon_tags_30 },
    { 0x5ed278ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb6678ae, SUC_NE_lexicon_tags_51 },
    { 0x89c78ae, SUC_NE_lexicon_tags_2 },
    { 0xd33678b0, SUC_NE_lexicon_tags_2 },
    { 0x10d878b1, SUC_NE_lexicon_tags_2 },
    { 0x671478b2, SUC_NE_lexicon_tags_184 },
    { 0x966478b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4e78b6, SUC_NE_lexicon_tags_2 },
    { 0xfe0478b6, SUC_NE_lexicon_tags_2 },
    { 0x50f478b8, SUC_NE_lexicon_tags_2 },
    { 0x10b278b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x818278bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcee78c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ae78ce, SUC_NE_lexicon_tags_2 },
    { 0x8c3878ce, SUC_NE_lexicon_tags_2 },
    { 0xec0878cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b278d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55278d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91878dc, SUC_NE_lexicon_tags_2 },
    { 0x7c6278dd, SUC_NE_lexicon_tags_2 },
    { 0xa9ea78de, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0xbeec78e0, SUC_NE_lexicon_tags_2 },
    { 0xbe1278e1, SUC_NE_lexicon_tags_2 },
    { 0x1b0078e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x638a78e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cc78e9, SUC_NE_lexicon_tags_2 },
    { 0xdbaa78ea, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a078ef, SUC_NE_lexicon_tags_3 },
    { 0x5ebc78f0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x991c78f2, SUC_NE_lexicon_tags_2 },
    { 0xdc8478f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c78fb, SUC_NE_lexicon_tags_2 },
    { 0x296478fb, SUC_NE_lexicon_tags_2 },
    { 0x63ea78fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c7905, SUC_NE_lexicon_tags_2 },
    { 0xf37a7906, SUC_NE_lexicon_tags_2 },
    { 0xa5647905, SUC_NE_lexicon_tags_36 },
    { 0x8cfc7908, SUC_NE_lexicon_tags_2 },
    { 0x1e2e7908, SUC_NE_lexicon_tags_30 },
    { 0x2f867908, SUC_NE_lexicon_tags_2 },
    { 0x1dd87907, SUC_NE_lexicon_tags_2 },
    { 0x1ee2790c, SUC_NE_lexicon_tags_2 },
    { 0xe176790c, SUC_NE_lexicon_tags_13 },
    { 0x7f927905, SUC_NE_lexicon_tags_4 },
    { 0x4abc790e, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x75367911, SUC_NE_lexicon_tags_2 },
    { 0xdd807912, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x984a7914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963c7918, SUC_NE_lexicon_tags_2 },
    { 0x2d067919, SUC_NE_lexicon_tags_2 },
    { 0xf0ac791a, SUC_NE_lexicon_tags_2 },
    { 0xd2ac791b, SUC_NE_lexicon_tags_2 },
    { 0x60187919, SUC_NE_lexicon_tags_2 },
    { 0x81e2791c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f50791f, SUC_NE_lexicon_tags_2 },
    { 0xf6027920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe287923, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65987928, SUC_NE_lexicon_tags_2 },
    { 0x3f7e7928, SUC_NE_lexicon_tags_2 },
    { 0x940e792a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2782792e, SUC_NE_lexicon_tags_2 },
    { 0x35fa792f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x762a7931, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e47936, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x64de7938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed42793b, SUC_NE_lexicon_tags_1 },
    { 0xb7f0793b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d27944, SUC_NE_lexicon_tags_2 },
    { 0x82cc7944, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b67949, SUC_NE_lexicon_tags_0 },
    { 0xccc0794a, SUC_NE_lexicon_tags_2 },
    { 0x2d42794b, SUC_NE_lexicon_tags_2 },
    { 0x5c84794a, SUC_NE_lexicon_tags_32 },
    { 0xaa5e794d, SUC_NE_lexicon_tags_4 },
    { 0xdf60794e, SUC_NE_lexicon_tags_2 },
    { 0xcfd6794e, SUC_NE_lexicon_tags_2 },
    { 0xe8da794e, SUC_NE_lexicon_tags_2 },
    { 0xdc027950, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x369c7954, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x21727956, SUC_NE_lexicon_tags_17 },
    { 0xb3e47957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe795a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc42a795c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e47964, SUC_NE_lexicon_tags_2 },
    { 0xb5907964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b687967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88da7969, SUC_NE_lexicon_tags_2 },
    { 0xccea796a, SUC_NE_lexicon_tags_2 },
    { 0xd376796b, SUC_NE_lexicon_tags_2 },
    { 0xda4a796b, SUC_NE_lexicon_tags_2 },
    { 0xf6bc796b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844a7972, SUC_NE_lexicon_tags_2 },
    { 0xa2127973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40d87975, SUC_NE_lexicon_tags_2 },
    { 0xda7c7976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7487978, SUC_NE_lexicon_tags_2 },
    { 0x17c67979, SUC_NE_lexicon_tags_2 },
    { 0xcfa797a, SUC_NE_lexicon_tags_2 },
    { 0x7f24797a, SUC_NE_lexicon_tags_2 },
    { 0x7e12797c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7aae797e, SUC_NE_lexicon_tags_18 },
    { 0x466e797f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3747989, SUC_NE_lexicon_tags_2 },
    { 0x2462798a, SUC_NE_lexicon_tags_2 },
    { 0x420c798b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a58798d, SUC_NE_lexicon_tags_2 },
    { 0xa07e798e, SUC_NE_lexicon_tags_13 },
    { 0x5d1c798f, SUC_NE_lexicon_tags_17 },
    { 0xbe40798f, SUC_NE_lexicon_tags_2 },
    { 0xfea7990, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fe27993, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xded87995, SUC_NE_lexicon_tags_2 },
    { 0xc36e7995, SUC_NE_lexicon_tags_17 },
    { 0xcdc27997, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bec799d, SUC_NE_lexicon_tags_0 },
    { 0x582799e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49279a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0e79a7, SUC_NE_lexicon_tags_2 },
    { 0x405c79a7, SUC_NE_lexicon_tags_4 },
    { 0x2cda79a9, SUC_NE_lexicon_tags_2 },
    { 0x856c79aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a5e79ac, SUC_NE_lexicon_tags_2 },
    { 0x8e7079ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd15079af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x412e79b2, SUC_NE_lexicon_tags_2 },
    { 0x7c0a79b3, SUC_NE_lexicon_tags_2 },
    { 0x101879b4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x27d679b6, SUC_NE_lexicon_tags_2 },
    { 0xc99279b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbd679b9, SUC_NE_lexicon_tags_2 },
    { 0x6d4279ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628e79be, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9079c3, SUC_NE_lexicon_tags_2 },
    { 0xb31e79c4, SUC_NE_lexicon_tags_2 },
    { 0xe47e79c5, SUC_NE_lexicon_tags_2 },
    { 0x99479c6, SUC_NE_lexicon_tags_7 },
    { 0x1b2479c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721079ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ca279d0, SUC_NE_lexicon_tags_2 },
    { 0x98ae79d0, SUC_NE_lexicon_tags_2 },
    { 0xe39479d0, SUC_NE_lexicon_tags_2 },
    { 0x4a6479d3, SUC_NE_lexicon_tags_2 },
    { 0x67c279d4, SUC_NE_lexicon_tags_2 },
    { 0xc5f279d3, SUC_NE_lexicon_tags_2 },
    { 0xfee679d6, SUC_NE_lexicon_tags_2 },
    { 0xfde79d3, SUC_NE_lexicon_tags_2 },
    { 0xa6479d8, SUC_NE_lexicon_tags_4 },
    { 0x5dfe79d8, SUC_NE_lexicon_tags_2 },
    { 0x5a9a79d6, SUC_NE_lexicon_tags_2 },
    { 0x822c79db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a79df, SUC_NE_lexicon_tags_2 },
    { 0x2ca679e0, SUC_NE_lexicon_tags_2 },
    { 0xa2c279e1, SUC_NE_lexicon_tags_2 },
    { 0x8be679e2, SUC_NE_lexicon_tags_2 },
    { 0x474879e1, SUC_NE_lexicon_tags_2 },
    { 0xd58a79e4, SUC_NE_lexicon_tags_2 },
    { 0x7cb679e4, SUC_NE_lexicon_tags_2 },
    { 0xa04079e2, SUC_NE_lexicon_tags_2 },
    { 0xfa2279e7, SUC_NE_lexicon_tags_2 },
    { 0x601879e8, SUC_NE_lexicon_tags_2 },
    { 0xbe8279e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd879eb, SUC_NE_lexicon_tags_3 },
    { 0xb65879ec, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x57da79ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0879f3, SUC_NE_lexicon_tags_2 },
    { 0x76aa79f4, SUC_NE_lexicon_tags_17 },
    { 0x52ca79f5, SUC_NE_lexicon_tags_2 },
    { 0x1a0679f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfa79f9, SUC_NE_lexicon_tags_2 },
    { 0xdbb479fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8c79fc, SUC_NE_lexicon_tags_2 },
    { 0x5d879fd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x609a79ff, SUC_NE_lexicon_tags_2 },
    { 0xc5e07a00, SUC_NE_lexicon_tags_2 },
    { 0x5a2c79ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e87a07, SUC_NE_lexicon_tags_2 },
    { 0x5067a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x852c7a0a, SUC_NE_lexicon_tags_1 },
    { 0x75fa7a0a, SUC_NE_lexicon_tags_2 },
    { 0x85467a0c, SUC_NE_lexicon_tags_1 },
    { 0xa8867a0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf707a0f, SUC_NE_lexicon_tags_2 },
    { 0x461a7a0f, SUC_NE_lexicon_tags_2 },
    { 0xc847a11, SUC_NE_lexicon_tags_2 },
    { 0x862a7a10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccb87a14, SUC_NE_lexicon_tags_2 },
    { 0xe36c7a14, SUC_NE_lexicon_tags_0 },
    { 0xf8547a16, SUC_NE_lexicon_tags_4 },
    { 0x8647a16, SUC_NE_lexicon_tags_2 },
    { 0xb7627a16, SUC_NE_lexicon_tags_2 },
    { 0x995e7a19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75387a1c, SUC_NE_lexicon_tags_2 },
    { 0x3d887a1c, SUC_NE_lexicon_tags_1 },
    { 0x31547a1e, SUC_NE_lexicon_tags_2 },
    { 0x950a7a1f, SUC_NE_lexicon_tags_2 },
    { 0xc0d87a1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf27a25, SUC_NE_lexicon_tags_281 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd47a28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ec47a2a, SUC_NE_lexicon_tags_2 },
    { 0xfbdc7a2b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58c7a32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbc7a34, SUC_NE_lexicon_tags_2 },
    { 0xed9a7a35, SUC_NE_lexicon_tags_2 },
    { 0x21f47a36, SUC_NE_lexicon_tags_2 },
    { 0x13307a36, SUC_NE_lexicon_tags_2 },
    { 0xa947a37, SUC_NE_lexicon_tags_2 },
    { 0x8f67a38, SUC_NE_lexicon_tags_2 },
    { 0x181a7a3a, SUC_NE_lexicon_tags_2 },
    { 0xaa8c7a3a, SUC_NE_lexicon_tags_2 },
    { 0x105e7a36, SUC_NE_lexicon_tags_2 },
    { 0x4b287a38, SUC_NE_lexicon_tags_2 },
    { 0x67e27a3e, SUC_NE_lexicon_tags_2 },
    { 0x516a7a3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d47a43, SUC_NE_lexicon_tags_2 },
    { 0x515a7a44, SUC_NE_lexicon_tags_0 },
    { 0xc7b07a44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2a7a49, SUC_NE_lexicon_tags_2 },
    { 0x67c87a4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba07a4c, SUC_NE_lexicon_tags_2 },
    { 0xe8b47a4d, SUC_NE_lexicon_tags_2 },
    { 0xbbba7a4e, SUC_NE_lexicon_tags_2 },
    { 0x3f167a4f, SUC_NE_lexicon_tags_2 },
    { 0x7e087a50, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xb3bc7a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d07a55, SUC_NE_lexicon_tags_2 },
    { 0x22287a55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c7a5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c47a5f, SUC_NE_lexicon_tags_390 },
    { 0x54907a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e67a69, SUC_NE_lexicon_tags_2 },
    { 0x5afa7a69, SUC_NE_lexicon_tags_2 },
    { 0x50827a69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc1e7a6d, SUC_NE_lexicon_tags_2 },
    { 0x9e267a6e, SUC_NE_lexicon_tags_2 },
    { 0xc4707a6d, SUC_NE_lexicon_tags_19 },
    { 0xdf667a70, SUC_NE_lexicon_tags_3 },
    { 0xb0a87a71, SUC_NE_lexicon_tags_2 },
    { 0x21907a72, SUC_NE_lexicon_tags_2 },
    { 0x95d47a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62767a78, SUC_NE_lexicon_tags_2 },
    { 0x37c7a79, SUC_NE_lexicon_tags_2 },
    { 0x2c67a7a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf87a7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02e7a89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca07a8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5287a90, SUC_NE_lexicon_tags_7 },
    { 0x93fa7a90, SUC_NE_lexicon_tags_2 },
    { 0xd8ac7a91, SUC_NE_lexicon_tags_2 },
    { 0xc3827a93, SUC_NE_lexicon_tags_2 },
    { 0x5ba07a93, SUC_NE_lexicon_tags_40 },
    { 0xe6e7a95, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x33de7a98, SUC_NE_lexicon_tags_2 },
    { 0x62ee7a98, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0c7a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2827aa5, SUC_NE_lexicon_tags_40 },
    { 0x8d407aa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b07aa9, SUC_NE_lexicon_tags_2 },
    { 0x7e6e7aaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff67aae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b187ab0, SUC_NE_lexicon_tags_2 },
    { 0x20947ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43ac7ab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75c7ab7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b847aba, SUC_NE_lexicon_tags_2 },
    { 0xf0287abb, SUC_NE_lexicon_tags_2 },
    { 0x7dce7aba, SUC_NE_lexicon_tags_2 },
    { 0xa7ee7abd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88167abf, SUC_NE_lexicon_tags_2 },
    { 0x1ed87ac0, SUC_NE_lexicon_tags_2 },
    { 0x215a7ac1, SUC_NE_lexicon_tags_2 },
    { 0x9a247ac1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x924c7ac4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6087ac7, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5827acb, SUC_NE_lexicon_tags_2 },
    { 0xc42c7acc, SUC_NE_lexicon_tags_2 },
    { 0xd8ea7acd, SUC_NE_lexicon_tags_2 },
    { 0x30727ace, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97927ad4, SUC_NE_lexicon_tags_3 },
    { 0x21de7ad5, SUC_NE_lexicon_tags_2 },
    { 0x5a467ad5, SUC_NE_lexicon_tags_2 },
    { 0xe4e07ad6, SUC_NE_lexicon_tags_451 },
    { 0x8c627ad5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc687ada, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aac7adc, SUC_NE_lexicon_tags_2 },
    { 0x4eae7adc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x792a7adf, SUC_NE_lexicon_tags_2 },
    { 0xdd687adf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a7ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac707ae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1a7aeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c7af0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377e7afc, SUC_NE_lexicon_tags_2 },
    { 0x45c47afc, SUC_NE_lexicon_tags_2 },
    { 0x56cc7afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46067b00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aa7b05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f987b08, SUC_NE_lexicon_tags_2 },
    { 0xce907b09, SUC_NE_lexicon_tags_2 },
    { 0x8ab87b0a, SUC_NE_lexicon_tags_2 },
    { 0xa7f67b0b, SUC_NE_lexicon_tags_2 },
    { 0x79dc7b0c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbe7b10, SUC_NE_lexicon_tags_2 },
    { 0xe5de7b10, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x40ae7b13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a07b16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d87b1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39787b1c, SUC_NE_lexicon_tags_2 },
    { 0x53647b1d, SUC_NE_lexicon_tags_2 },
    { 0x31487b1e, SUC_NE_lexicon_tags_2 },
    { 0x622e7b1f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x43467b21, SUC_NE_lexicon_tags_2 },
    { 0x934a7b22, SUC_NE_lexicon_tags_2 },
    { 0x88b07b22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62b07b25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24c7b29, SUC_NE_lexicon_tags_2 },
    { 0xb1c7b29, SUC_NE_lexicon_tags_2 },
    { 0x58f47b29, SUC_NE_lexicon_tags_2 },
    { 0xc0ac7b2c, SUC_NE_lexicon_tags_2 },
    { 0x10e07b2d, SUC_NE_lexicon_tags_2 },
    { 0x30007b2d, SUC_NE_lexicon_tags_2 },
    { 0xa8be7b2e, SUC_NE_lexicon_tags_5 },
    { 0x30287b30, SUC_NE_lexicon_tags_2 },
    { 0x478e7b31, SUC_NE_lexicon_tags_2 },
    { 0xc4c47b31, SUC_NE_lexicon_tags_2 },
    { 0xcaba7b2e, SUC_NE_lexicon_tags_2 },
    { 0xa6ae7b2c, SUC_NE_lexicon_tags_7 },
    { 0xbdc67b33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1d67b37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7507b3c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8fa87b3e, SUC_NE_lexicon_tags_2 },
    { 0x1f407b3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162a7b44, SUC_NE_lexicon_tags_2 },
    { 0x4d7c7b45, SUC_NE_lexicon_tags_2 },
    { 0xcc0c7b46, SUC_NE_lexicon_tags_2 },
    { 0xed307b47, SUC_NE_lexicon_tags_2 },
    { 0x450e7b45, SUC_NE_lexicon_tags_2 },
    { 0x90da7b44, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cc7b50, SUC_NE_lexicon_tags_2 },
    { 0x11147b50, SUC_NE_lexicon_tags_2 },
    { 0x84c67b52, SUC_NE_lexicon_tags_2 },
    { 0xf5367b53, SUC_NE_lexicon_tags_2 },
    { 0x5f027b54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38f47b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a7b59, SUC_NE_lexicon_tags_2 },
    { 0xba3a7b5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e287b5d, SUC_NE_lexicon_tags_2 },
    { 0x7fb67b5e, SUC_NE_lexicon_tags_2 },
    { 0x61bc7b5d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa187b62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e947b65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a467b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x811a7b69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c7b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7c7b7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84207b7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac587b7f, SUC_NE_lexicon_tags_2 },
    { 0xd36a7b80, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5aa7b83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea7b87, SUC_NE_lexicon_tags_2 },
    { 0xec327b87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x300c7b8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x344e7b8d, SUC_NE_lexicon_tags_2 },
    { 0x6ee7b8e, SUC_NE_lexicon_tags_5 },
    { 0x7ccc7b8f, SUC_NE_lexicon_tags_2 },
    { 0xfb07b8e, SUC_NE_lexicon_tags_2 },
    { 0x61c87b8d, SUC_NE_lexicon_tags_2 },
    { 0x9e147b8d, SUC_NE_lexicon_tags_2 },
    { 0x2fce7b92, SUC_NE_lexicon_tags_2 },
    { 0x2c3e7b94, SUC_NE_lexicon_tags_30 },
    { 0x24ae7b92, SUC_NE_lexicon_tags_2 },
    { 0x2b1e7b96, SUC_NE_lexicon_tags_2 },
    { 0x81667b96, SUC_NE_lexicon_tags_2 },
    { 0x63187b98, SUC_NE_lexicon_tags_2 },
    { 0x31e67b99, SUC_NE_lexicon_tags_2 },
    { 0x1bb67b9a, SUC_NE_lexicon_tags_2 },
    { 0xddec7b98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee07b9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29ac7ba0, SUC_NE_lexicon_tags_2 },
    { 0x4a987ba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fca7ba3, SUC_NE_lexicon_tags_164 },
    { 0xccda7ba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62aa7ba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892a7bad, SUC_NE_lexicon_tags_2 },
    { 0x2a167bae, SUC_NE_lexicon_tags_2 },
    { 0x5b047baf, SUC_NE_lexicon_tags_2 },
    { 0x33527bad, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x93947bb2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f27bb6, SUC_NE_lexicon_tags_2 },
    { 0xfeca7bb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15507bba, SUC_NE_lexicon_tags_2 },
    { 0xd92e7bba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a207bbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebe7bc3, SUC_NE_lexicon_tags_2 },
    { 0x9d547bc3, SUC_NE_lexicon_tags_0 },
    { 0x9067bc5, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa207bc8, SUC_NE_lexicon_tags_2 },
    { 0x5f227bc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8e7bcc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x81e87bce, SUC_NE_lexicon_tags_2 },
    { 0x56ec7bcf, SUC_NE_lexicon_tags_2 },
    { 0xb067bd0, SUC_NE_lexicon_tags_19 },
    { 0x88c67bd0, SUC_NE_lexicon_tags_2 },
    { 0xb8cc7bcf, SUC_NE_lexicon_tags_2 },
    { 0xf3de7bd3, SUC_NE_lexicon_tags_2 },
    { 0xa27e7bd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70467bd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e8e7bd9, SUC_NE_lexicon_tags_2 },
    { 0xc2c7bda, SUC_NE_lexicon_tags_2 },
    { 0xefea7bdb, SUC_NE_lexicon_tags_2 },
    { 0xb7ee7bdc, SUC_NE_lexicon_tags_2 },
    { 0x5caa7bdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46727bdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8c47be1, SUC_NE_lexicon_tags_2 },
    { 0xf8f27be2, SUC_NE_lexicon_tags_2 },
    { 0x4d6a7be3, SUC_NE_lexicon_tags_2 },
    { 0x172a7be4, SUC_NE_lexicon_tags_2 },
    { 0xafa47be2, SUC_NE_lexicon_tags_2 },
    { 0xc6da7be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3a07be8, SUC_NE_lexicon_tags_2 },
    { 0x68f07be9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef387bf0, SUC_NE_lexicon_tags_2 },
    { 0xf4667bf1, SUC_NE_lexicon_tags_3 },
    { 0xf4d07bf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaea7bf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17067bf8, SUC_NE_lexicon_tags_2 },
    { 0x995e7bf8, SUC_NE_lexicon_tags_2 },
    { 0x577c7bf8, SUC_NE_lexicon_tags_2 },
    { 0x5d427bf8, SUC_NE_lexicon_tags_19 },
    { 0x1d4a7bfc, SUC_NE_lexicon_tags_2 },
    { 0x7caa7bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x885c7bff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81807c01, SUC_NE_lexicon_tags_0 },
    { 0x39587c02, SUC_NE_lexicon_tags_2 },
    { 0xd0627c03, SUC_NE_lexicon_tags_2 },
    { 0xfbfc7c04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31807c08, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x212a7c0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee627c0d, SUC_NE_lexicon_tags_2 },
    { 0x6f087c0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ec7c10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a227c15, SUC_NE_lexicon_tags_2 },
    { 0x9e547c15, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b087c1a, SUC_NE_lexicon_tags_2 },
    { 0xf8807c1a, SUC_NE_lexicon_tags_2 },
    { 0x20f47c1b, SUC_NE_lexicon_tags_2 },
    { 0xf767c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e47c23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b347c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x464a7c32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65d87c34, SUC_NE_lexicon_tags_2 },
    { 0x61b47c35, SUC_NE_lexicon_tags_2 },
    { 0xc3967c36, SUC_NE_lexicon_tags_2 },
    { 0x6ac67c36, SUC_NE_lexicon_tags_2 },
    { 0xa4627c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda7c3c, SUC_NE_lexicon_tags_8 },
    { 0xd31a7c3d, SUC_NE_lexicon_tags_3 },
    { 0x5cc67c3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d5a7c40, SUC_NE_lexicon_tags_2 },
    { 0xaac07c40, SUC_NE_lexicon_tags_2 },
    { 0xf41e7c42, SUC_NE_lexicon_tags_3 },
    { 0x190e7c42, SUC_NE_lexicon_tags_2 },
    { 0xcb8a7c44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da87c47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ae7c4b, SUC_NE_lexicon_tags_2 },
    { 0x999e7c4c, SUC_NE_lexicon_tags_2 },
    { 0xb95c7c4d, SUC_NE_lexicon_tags_2 },
    { 0x22527c4c, SUC_NE_lexicon_tags_2 },
    { 0xa06e7c4f, SUC_NE_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b387c52, SUC_NE_lexicon_tags_2 },
    { 0x8da07c52, SUC_NE_lexicon_tags_151 },
    { 0x5da47c54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c0c7c56, SUC_NE_lexicon_tags_2 },
    { 0xb38e7c57, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c47c5a, SUC_NE_lexicon_tags_2 },
    { 0x12187c5b, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xde87c5d, SUC_NE_lexicon_tags_2 },
    { 0x3aec7c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf887c62, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43e7c65, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x16007c67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b987c69, SUC_NE_lexicon_tags_17 },
    { 0x868a7c6a, SUC_NE_lexicon_tags_2 },
    { 0x69b67c6b, SUC_NE_lexicon_tags_7 },
    { 0x5207c69, SUC_NE_lexicon_tags_7 },
    { 0x936e7c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda207c70, SUC_NE_lexicon_tags_2 },
    { 0x73467c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8647c74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c327c7c, SUC_NE_lexicon_tags_2 },
    { 0xb7887c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb67c80, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2b07c82, SUC_NE_lexicon_tags_2 },
    { 0x6c6a7c82, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x254a7c85, SUC_NE_lexicon_tags_4 },
    { 0xd46e7c86, SUC_NE_lexicon_tags_34 },
    { 0xf8467c86, SUC_NE_lexicon_tags_2 },
    { 0xad2a7c88, SUC_NE_lexicon_tags_2 },
    { 0x86ba7c89, SUC_NE_lexicon_tags_2 },
    { 0x5fca7c85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x222e7c8d, SUC_NE_lexicon_tags_2 },
    { 0xa6b27c8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d707c90, SUC_NE_lexicon_tags_2 },
    { 0x34827c91, SUC_NE_lexicon_tags_2 },
    { 0x83aa7c92, SUC_NE_lexicon_tags_2 },
    { 0x49467c90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc7c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbd47c9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7ca0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8567ca2, SUC_NE_lexicon_tags_2 },
    { 0x90a07ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d67ca7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb87cab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57a87cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13447cb1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2c7cb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98c7cb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1e7cbb, SUC_NE_lexicon_tags_1 },
    { 0xd18c7cbc, SUC_NE_lexicon_tags_2 },
    { 0xface7cbb, SUC_NE_lexicon_tags_2 },
    { 0x911e7cbd, SUC_NE_lexicon_tags_0 },
    { 0xcd007cbe, SUC_NE_lexicon_tags_2 },
    { 0x6f187cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14527cc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e3c7ccb, SUC_NE_lexicon_tags_2 },
    { 0xa5dc7ccb, SUC_NE_lexicon_tags_2 },
    { 0xf9d47ccc, SUC_NE_lexicon_tags_2 },
    { 0xf1047cce, SUC_NE_lexicon_tags_2 },
    { 0xb93e7ccf, SUC_NE_lexicon_tags_2 },
    { 0xb1da7cd0, SUC_NE_lexicon_tags_2 },
    { 0x49847cd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b527cd4, SUC_NE_lexicon_tags_2 },
    { 0x1f0e7cd4, SUC_NE_lexicon_tags_2 },
    { 0x7a07cd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e87cda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2e7ce1, SUC_NE_lexicon_tags_2 },
    { 0x817a7ce2, SUC_NE_lexicon_tags_2 },
    { 0x71267ce2, SUC_NE_lexicon_tags_2 },
    { 0x9dba7ce3, SUC_NE_lexicon_tags_2 },
    { 0x1d3c7ce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c27ce8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3087cf1, SUC_NE_lexicon_tags_2 },
    { 0x9f6e7cf2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fa7cf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f5a7cfb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8a7cfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7907d05, SUC_NE_lexicon_tags_7 },
    { 0xe2d67d05, SUC_NE_lexicon_tags_2 },
    { 0x5d8c7d07, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x936e7d09, SUC_NE_lexicon_tags_2 },
    { 0x82027d0a, SUC_NE_lexicon_tags_2 },
    { 0xdd867d0b, SUC_NE_lexicon_tags_2 },
    { 0x801a7d0b, SUC_NE_lexicon_tags_2 },
    { 0x9ac27d0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a287d0f, SUC_NE_lexicon_tags_2 },
    { 0xbd647d0f, SUC_NE_lexicon_tags_2 },
    { 0xce287d11, SUC_NE_lexicon_tags_2 },
    { 0xbe287d11, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661c7d16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7327d19, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x90527d1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68a7d1e, SUC_NE_lexicon_tags_7 },
    { 0x96c87d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e7a7d21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61de7d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ca7d28, SUC_NE_lexicon_tags_2 },
    { 0x63a27d29, SUC_NE_lexicon_tags_2 },
    { 0x47887d2a, SUC_NE_lexicon_tags_2 },
    { 0x2ba27d28, SUC_NE_lexicon_tags_0 },
    { 0x841a7d28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eae7d33, SUC_NE_lexicon_tags_7 },
    { 0x5cb47d34, SUC_NE_lexicon_tags_2 },
    { 0x4c647d35, SUC_NE_lexicon_tags_2 },
    { 0x5f707d36, SUC_NE_lexicon_tags_2 },
    { 0x7bf67d37, SUC_NE_lexicon_tags_0 },
    { 0x50e47d38, SUC_NE_lexicon_tags_3 },
    { 0x2c0a7d39, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe07d3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa2a7d3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c7d46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32827d4a, SUC_NE_lexicon_tags_2 },
    { 0x2cea7d4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x701e7d4d, SUC_NE_lexicon_tags_2 },
    { 0x74d87d4d, SUC_NE_lexicon_tags_2 },
    { 0x73a87d4f, SUC_NE_lexicon_tags_67 },
    { 0xb5407d4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed87d53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe24e7d55, SUC_NE_lexicon_tags_2 },
    { 0x148c7d55, SUC_NE_lexicon_tags_7 },
    { 0xbc767d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef527d5a, SUC_NE_lexicon_tags_2 },
    { 0xb7e27d5b, SUC_NE_lexicon_tags_2 },
    { 0xf83c7d5c, SUC_NE_lexicon_tags_2 },
    { 0x283c7d5d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8a7d62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d67d66, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7027d6d, SUC_NE_lexicon_tags_2 },
    { 0x4a707d6d, SUC_NE_lexicon_tags_2 },
    { 0x506a7d6f, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c7d72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde27d76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7027d79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3127d7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e267d7d, SUC_NE_lexicon_tags_7 },
    { 0x8607d7e, SUC_NE_lexicon_tags_2 },
    { 0x4b47d7e, SUC_NE_lexicon_tags_2 },
    { 0xf6967d7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62287d82, SUC_NE_lexicon_tags_2 },
    { 0x347c7d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c5c7d85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b47d88, SUC_NE_lexicon_tags_2 },
    { 0xd187d88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea707d8c, SUC_NE_lexicon_tags_1 },
    { 0xe66a7d8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4667d93, SUC_NE_lexicon_tags_2 },
    { 0x19667d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6487d97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4567d99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2567d9c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7c7da3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1207da6, SUC_NE_lexicon_tags_2 },
    { 0xcb5a7da6, SUC_NE_lexicon_tags_2 },
    { 0xdc6a7da8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e87dab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75187dad, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a47db0, SUC_NE_lexicon_tags_2 },
    { 0xeb727db1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4967db3, SUC_NE_lexicon_tags_3 },
    { 0x80b07db4, SUC_NE_lexicon_tags_2 },
    { 0x61747db3, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dc7dba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91407dbd, SUC_NE_lexicon_tags_3 },
    { 0xafd87dbe, SUC_NE_lexicon_tags_18 },
    { 0xe2f27dbe, SUC_NE_lexicon_tags_2 },
    { 0x75de7dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61807dc2, SUC_NE_lexicon_tags_2 },
    { 0x267e7dc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7547dc8, SUC_NE_lexicon_tags_2 },
    { 0x7fba7dc8, SUC_NE_lexicon_tags_2 },
    { 0xbeb67dca, SUC_NE_lexicon_tags_2 },
    { 0x55ae7dca, SUC_NE_lexicon_tags_2 },
    { 0x13a07dcb, SUC_NE_lexicon_tags_2 },
    { 0x933e7dcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaeda7dcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c07dd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14727dd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5907dd7, SUC_NE_lexicon_tags_2 },
    { 0x96e87dd8, SUC_NE_lexicon_tags_18 },
    { 0x90787dd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6767ddb, SUC_NE_lexicon_tags_2 },
    { 0x73c7ddc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3587dde, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22c7de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22da7de6, SUC_NE_lexicon_tags_2 },
    { 0xa2d27de7, SUC_NE_lexicon_tags_1 },
    { 0x744a7de7, SUC_NE_lexicon_tags_2 },
    { 0xe8447de7, SUC_NE_lexicon_tags_2 },
    { 0x77987dea, SUC_NE_lexicon_tags_2 },
    { 0x48627de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x767a7dee, SUC_NE_lexicon_tags_2 },
    { 0x982a7dee, SUC_NE_lexicon_tags_2 },
    { 0x5f0c7df0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x80727df2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd07df6, SUC_NE_lexicon_tags_2 },
    { 0x3b47df6, SUC_NE_lexicon_tags_2 },
    { 0xf2947df8, SUC_NE_lexicon_tags_2 },
    { 0x7fce7df7, SUC_NE_lexicon_tags_2 },
    { 0xffa7dfa, SUC_NE_lexicon_tags_2 },
    { 0xdbb87dfa, SUC_NE_lexicon_tags_2 },
    { 0x67607dfa, SUC_NE_lexicon_tags_2 },
    { 0x82067df6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68127e01, SUC_NE_lexicon_tags_2 },
    { 0x59767e02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8567e05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x532e7e07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a87e0b, SUC_NE_lexicon_tags_2 },
    { 0x2e587e0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef87e10, SUC_NE_lexicon_tags_32 },
    { 0x46547e10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2a7e14, SUC_NE_lexicon_tags_2 },
    { 0x7ef67e15, SUC_NE_lexicon_tags_2 },
    { 0xc567e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c087e18, SUC_NE_lexicon_tags_2 },
    { 0xfde47e19, SUC_NE_lexicon_tags_2 },
    { 0xb79e7e19, SUC_NE_lexicon_tags_139 },
    { 0x94be7e18, SUC_NE_lexicon_tags_2 },
    { 0x6d047e1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23287e1e, SUC_NE_lexicon_tags_2 },
    { 0xdee07e1f, SUC_NE_lexicon_tags_7 },
    { 0xe02a7e1f, SUC_NE_lexicon_tags_2 },
    { 0x5c2e7e21, SUC_NE_lexicon_tags_2 },
    { 0x85de7e22, SUC_NE_lexicon_tags_2 },
    { 0x4e847e23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e47e29, SUC_NE_lexicon_tags_2 },
    { 0xf34c7e2a, SUC_NE_lexicon_tags_2 },
    { 0xdc3c7e29, SUC_NE_lexicon_tags_4 },
    { 0x5d2a7e2a, SUC_NE_lexicon_tags_3 },
    { 0x77fe7e2d, SUC_NE_lexicon_tags_2 },
    { 0x68c87e2d, SUC_NE_lexicon_tags_2 },
    { 0x4b767e2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x906c7e31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xceb27e33, SUC_NE_lexicon_tags_2 },
    { 0x18aa7e33, SUC_NE_lexicon_tags_2 },
    { 0xdb67e35, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0x26867e37, SUC_NE_lexicon_tags_2 },
    { 0x302e7e38, SUC_NE_lexicon_tags_2 },
    { 0x58b07e38, SUC_NE_lexicon_tags_2 },
    { 0x7ec67e3a, SUC_NE_lexicon_tags_2 },
    { 0xb0587e3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d427e3f, SUC_NE_lexicon_tags_2 },
    { 0x72627e40, SUC_NE_lexicon_tags_2 },
    { 0x84387e3f, SUC_NE_lexicon_tags_2 },
    { 0x52207e42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01e7e46, SUC_NE_lexicon_tags_0 },
    { 0xc8c27e47, SUC_NE_lexicon_tags_2 },
    { 0x416e7e48, SUC_NE_lexicon_tags_2 },
    { 0x9c607e49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f227e4f, SUC_NE_lexicon_tags_2 },
    { 0xe8347e50, SUC_NE_lexicon_tags_2 },
    { 0x4e7a7e50, SUC_NE_lexicon_tags_2 },
    { 0x5bb87e51, SUC_NE_lexicon_tags_2 },
    { 0xb6e27e52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3e7e56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fca7e58, SUC_NE_lexicon_tags_2 },
    { 0x11287e59, SUC_NE_lexicon_tags_4 },
    { 0x69527e5a, SUC_NE_lexicon_tags_19 },
    { 0x4ca7e5b, SUC_NE_lexicon_tags_2 },
    { 0xc0ee7e5c, SUC_NE_lexicon_tags_2 },
    { 0xe8267e5d, SUC_NE_lexicon_tags_2 },
    { 0xc0307e5b, SUC_NE_lexicon_tags_2 },
    { 0x21c47e5e, SUC_NE_lexicon_tags_2 },
    { 0xc8ca7e60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0e47e62, SUC_NE_lexicon_tags_2 },
    { 0xdd187e62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a4c7e65, SUC_NE_lexicon_tags_2 },
    { 0x41107e66, SUC_NE_lexicon_tags_2 },
    { 0x73067e67, SUC_NE_lexicon_tags_2 },
    { 0xcc207e68, SUC_NE_lexicon_tags_2 },
    { 0xa4247e67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e87e6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x115c7e6f, SUC_NE_lexicon_tags_2 },
    { 0x64527e70, SUC_NE_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e67e78, SUC_NE_lexicon_tags_2 },
    { 0xa73c7e79, SUC_NE_lexicon_tags_2 },
    { 0x98dc7e79, SUC_NE_lexicon_tags_2 },
    { 0xad27e7b, SUC_NE_lexicon_tags_2 },
    { 0x8a047e7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88507e84, SUC_NE_lexicon_tags_2 },
    { 0x525e7e85, SUC_NE_lexicon_tags_2 },
    { 0xe4c67e86, SUC_NE_lexicon_tags_2 },
    { 0xe6c07e87, SUC_NE_lexicon_tags_2 },
    { 0xeb747e85, SUC_NE_lexicon_tags_2 },
    { 0xaa47e89, SUC_NE_lexicon_tags_3 },
    { 0x9b2a7e8a, SUC_NE_lexicon_tags_2 },
    { 0x9d9a7e8b, SUC_NE_lexicon_tags_2 },
    { 0xd2da7e8b, SUC_NE_lexicon_tags_2 },
    { 0x572c7e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8c7e94, SUC_NE_lexicon_tags_2 },
    { 0x70a67e95, SUC_NE_lexicon_tags_2 },
    { 0xf7207e95, SUC_NE_lexicon_tags_7 },
    { 0x4e8c7e94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da07e9a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c7e9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38c7ea2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe9a7ea4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d67ea7, SUC_NE_lexicon_tags_3 },
    { 0x6afc7ea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99b27eaa, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc887ead, SUC_NE_lexicon_tags_2 },
    { 0x14bc7ead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191a7eb5, SUC_NE_lexicon_tags_2 },
    { 0x6c347eb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1327eb8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39be7ebc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84e7ec0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63ba7ec2, SUC_NE_lexicon_tags_2 },
    { 0x3cec7ec3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16047ec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11e7ec9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f147ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30de7ece, SUC_NE_lexicon_tags_2 },
    { 0x32927ecf, SUC_NE_lexicon_tags_2 },
    { 0xfca07ed0, SUC_NE_lexicon_tags_2 },
    { 0x2dce7ed1, SUC_NE_lexicon_tags_2 },
    { 0x92ba7ed2, SUC_NE_lexicon_tags_7 },
    { 0x1a07ed3, SUC_NE_lexicon_tags_2 },
    { 0xcc827ed1, SUC_NE_lexicon_tags_2 },
    { 0xa29e7ed4, SUC_NE_lexicon_tags_2 },
    { 0x43487ed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45587edc, SUC_NE_lexicon_tags_2 },
    { 0x78287edd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a987edf, SUC_NE_lexicon_tags_2 },
    { 0x872a7ee0, SUC_NE_lexicon_tags_2 },
    { 0xf15c7ee0, SUC_NE_lexicon_tags_2 },
    { 0xc9d47ee2, SUC_NE_lexicon_tags_2 },
    { 0xaf247edf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b527ee7, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x81b27ee9, SUC_NE_lexicon_tags_2 },
    { 0xd9d67ee9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b67eef, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88347ef4, SUC_NE_lexicon_tags_2 },
    { 0x75bc7ef4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc147ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6707efa, SUC_NE_lexicon_tags_2 },
    { 0x9f3c7efb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd7247efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe67f04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49cc7f06, SUC_NE_lexicon_tags_2 },
    { 0x48727f07, SUC_NE_lexicon_tags_2 },
    { 0x16d87f06, SUC_NE_lexicon_tags_2 },
    { 0xeb027f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e87f0f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194c7f14, SUC_NE_lexicon_tags_2 },
    { 0x31ea7f15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675e7f1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa7f23, SUC_NE_lexicon_tags_2 },
    { 0x66007f23, SUC_NE_lexicon_tags_2 },
    { 0x31267f23, SUC_NE_lexicon_tags_2 },
    { 0xb0fe7f26, SUC_NE_lexicon_tags_2 },
    { 0x27727f26, SUC_NE_lexicon_tags_2 },
    { 0xe7bc7f28, SUC_NE_lexicon_tags_2 },
    { 0xf91e7f28, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7907f2b, SUC_NE_lexicon_tags_2 },
    { 0x42b87f2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4a27f2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40a7f31, SUC_NE_lexicon_tags_2 },
    { 0x9f307f31, SUC_NE_lexicon_tags_31 },
    { 0x6f6e7f33, SUC_NE_lexicon_tags_2 },
    { 0x5a187f33, SUC_NE_lexicon_tags_2 },
    { 0xb807f34, SUC_NE_lexicon_tags_2 },
    { 0xf3007f35, SUC_NE_lexicon_tags_2 },
    { 0xe7447f37, SUC_NE_lexicon_tags_2 },
    { 0x257c7f33, SUC_NE_lexicon_tags_2 },
    { 0xcefe7f39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94a87f3b, SUC_NE_lexicon_tags_2 },
    { 0x676a7f3c, SUC_NE_lexicon_tags_2 },
    { 0xe31c7f3d, SUC_NE_lexicon_tags_2 },
    { 0xcbec7f3d, SUC_NE_lexicon_tags_2 },
    { 0xd9f67f3d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9047f43, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb87f4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b047f4f, SUC_NE_lexicon_tags_2 },
    { 0x54c27f50, SUC_NE_lexicon_tags_2 },
    { 0x17307f51, SUC_NE_lexicon_tags_2 },
    { 0x5dcc7f51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2327f54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e27f59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ae7f5b, SUC_NE_lexicon_tags_2 },
    { 0xf5447f5c, SUC_NE_lexicon_tags_2 },
    { 0x5fb27f5d, SUC_NE_lexicon_tags_2 },
    { 0xef827f5e, SUC_NE_lexicon_tags_2 },
    { 0x13607f5d, SUC_NE_lexicon_tags_2 },
    { 0xf727f5e, SUC_NE_lexicon_tags_2 },
    { 0x5c027f61, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x38367f64, SUC_NE_lexicon_tags_2 },
    { 0xceba7f64, SUC_NE_lexicon_tags_2 },
    { 0x14067f64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x349e7f68, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694c7f6f, SUC_NE_lexicon_tags_3 },
    { 0xc25a7f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3da7f73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28a7f75, SUC_NE_lexicon_tags_2 },
    { 0xae6c7f76, SUC_NE_lexicon_tags_2 },
    { 0x629e7f77, SUC_NE_lexicon_tags_2 },
    { 0xeb787f77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x510c7f7b, SUC_NE_lexicon_tags_34 },
    { 0xbbcc7f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfa07f7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe887f80, SUC_NE_lexicon_tags_2 },
    { 0x54c07f81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee7f84, SUC_NE_lexicon_tags_2 },
    { 0x1ad67f85, SUC_NE_lexicon_tags_2 },
    { 0x6a087f85, SUC_NE_lexicon_tags_2 },
    { 0xeeb67f84, SUC_NE_lexicon_tags_2 },
    { 0x3e8c7f84, SUC_NE_lexicon_tags_2 },
    { 0x9da7f87, SUC_NE_lexicon_tags_2 },
    { 0x7b47f8a, SUC_NE_lexicon_tags_2 },
    { 0x27f47f8a, SUC_NE_lexicon_tags_2 },
    { 0xec4a7f8b, SUC_NE_lexicon_tags_2 },
    { 0x65dc7f8d, SUC_NE_lexicon_tags_2 },
    { 0x9fbc7f8e, SUC_NE_lexicon_tags_2 },
    { 0x91127f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce7f91, SUC_NE_lexicon_tags_2 },
    { 0x29d87f92, SUC_NE_lexicon_tags_2 },
    { 0x23887f93, SUC_NE_lexicon_tags_2 },
    { 0x36ba7f91, SUC_NE_lexicon_tags_2 },
    { 0x49b87f91, SUC_NE_lexicon_tags_2 },
    { 0x87d87f96, SUC_NE_lexicon_tags_4 },
    { 0xf5e27f97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d447f9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce27f9f, SUC_NE_lexicon_tags_4 },
    { 0x71847fa0, SUC_NE_lexicon_tags_2 },
    { 0x247e7fa1, SUC_NE_lexicon_tags_2 },
    { 0x997e7fa2, SUC_NE_lexicon_tags_2 },
    { 0x86347f9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb47fab, SUC_NE_lexicon_tags_2 },
    { 0x7b207fac, SUC_NE_lexicon_tags_50 },
    { 0xa6267fab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b87fb0, SUC_NE_lexicon_tags_2 },
    { 0x74a87fb1, SUC_NE_lexicon_tags_2 },
    { 0x42f87fb2, SUC_NE_lexicon_tags_2 },
    { 0xfa0a7fb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa7fb7, SUC_NE_lexicon_tags_2 },
    { 0x1a0c7fb8, SUC_NE_lexicon_tags_49 },
    { 0xb2d67fb9, SUC_NE_lexicon_tags_2 },
    { 0xabc47fba, SUC_NE_lexicon_tags_2 },
    { 0xbc0a7fb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf187fbd, SUC_NE_lexicon_tags_2 },
    { 0xac9c7fbe, SUC_NE_lexicon_tags_2 },
    { 0x2bf47fbe, SUC_NE_lexicon_tags_2 },
    { 0x3cd07fbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x842e7fc3, SUC_NE_lexicon_tags_2 },
    { 0xb3407fc4, SUC_NE_lexicon_tags_0 },
    { 0x9a4c7fc4, SUC_NE_lexicon_tags_2 },
    { 0x687a7fc5, SUC_NE_lexicon_tags_2 },
    { 0xed807fc7, SUC_NE_lexicon_tags_3 },
    { 0xd3847fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1447fcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89da7fd0, SUC_NE_lexicon_tags_2 },
    { 0xa2327fd1, SUC_NE_lexicon_tags_2 },
    { 0x8ac67fd2, SUC_NE_lexicon_tags_2 },
    { 0xa1427fd3, SUC_NE_lexicon_tags_2 },
    { 0xb6027fd4, SUC_NE_lexicon_tags_0 },
    { 0x4ad47fd3, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf187fda, SUC_NE_lexicon_tags_2 },
    { 0x1e27fda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d527fe3, SUC_NE_lexicon_tags_2 },
    { 0x29887fe4, SUC_NE_lexicon_tags_2 },
    { 0xf6767fe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef07fe8, SUC_NE_lexicon_tags_2 },
    { 0x9b887fe9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87887feb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46c7ff1, SUC_NE_lexicon_tags_3 },
    { 0xc40e7ff2, SUC_NE_lexicon_tags_2 },
    { 0xc91a7ff3, SUC_NE_lexicon_tags_2 },
    { 0x6ab67ff3, SUC_NE_lexicon_tags_3 },
    { 0xeb287ff5, SUC_NE_lexicon_tags_2 },
    { 0xf28e7ff5, SUC_NE_lexicon_tags_1 },
    { 0x4c2c7ff7, SUC_NE_lexicon_tags_2 },
    { 0xf38a7ff8, SUC_NE_lexicon_tags_0 },
    { 0xf5347ff9, SUC_NE_lexicon_tags_4 },
    { 0xeefa7ff9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b28000, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x37ea8002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbc08004, SUC_NE_lexicon_tags_2 },
    { 0x5ed08004, SUC_NE_lexicon_tags_2 },
    { 0x4cac8006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0688008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf96800b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8348011, SUC_NE_lexicon_tags_2 },
    { 0xc5b28012, SUC_NE_lexicon_tags_2 },
    { 0xebcc8013, SUC_NE_lexicon_tags_2 },
    { 0x2c308013, SUC_NE_lexicon_tags_2 },
    { 0xe0248014, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6da8017, SUC_NE_lexicon_tags_1 },
    { 0xd0f08018, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc548801e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e8022, SUC_NE_lexicon_tags_2 },
    { 0xf3da8022, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be88027, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c468029, SUC_NE_lexicon_tags_2 },
    { 0xf66802a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b02802c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x516802e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba488031, SUC_NE_lexicon_tags_2 },
    { 0xa8388031, SUC_NE_lexicon_tags_2 },
    { 0x7aa28031, SUC_NE_lexicon_tags_2 },
    { 0xb8b08031, SUC_NE_lexicon_tags_2 },
    { 0x54008035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6288037, SUC_NE_lexicon_tags_4 },
    { 0xc8508037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9af0803a, SUC_NE_lexicon_tags_2 },
    { 0x16f4803b, SUC_NE_lexicon_tags_2 },
    { 0x1ed0803b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2a8040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x694c8042, SUC_NE_lexicon_tags_2 },
    { 0xe7508043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12008047, SUC_NE_lexicon_tags_2 },
    { 0xfff48048, SUC_NE_lexicon_tags_2 },
    { 0xcc448049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e76804e, SUC_NE_lexicon_tags_2 },
    { 0xe388804e, SUC_NE_lexicon_tags_7 },
    { 0x9a7e8050, SUC_NE_lexicon_tags_2 },
    { 0xf7168050, SUC_NE_lexicon_tags_2 },
    { 0x7f848052, SUC_NE_lexicon_tags_2 },
    { 0x8db28053, SUC_NE_lexicon_tags_2 },
    { 0x2d468054, SUC_NE_lexicon_tags_2 },
    { 0xafbc8054, SUC_NE_lexicon_tags_2 },
    { 0xf37a8052, SUC_NE_lexicon_tags_2 },
    { 0x47948057, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3248060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3828062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2a8065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59388069, SUC_NE_lexicon_tags_2 },
    { 0x2a6e806a, SUC_NE_lexicon_tags_2 },
    { 0x10fc8069, SUC_NE_lexicon_tags_2 },
    { 0xa5f2806c, SUC_NE_lexicon_tags_2 },
    { 0xdd70806d, SUC_NE_lexicon_tags_2 },
    { 0xe102806e, SUC_NE_lexicon_tags_2 },
    { 0x1230806c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265c8074, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe468077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd84807b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2700807d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x4b9a807f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f888082, SUC_NE_lexicon_tags_2 },
    { 0x45be8083, SUC_NE_lexicon_tags_2 },
    { 0x81be8084, SUC_NE_lexicon_tags_2 },
    { 0x31008083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9b08087, SUC_NE_lexicon_tags_64 },
    { 0xcd308088, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e5e808a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f2808c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4908808f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a48093, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef08096, SUC_NE_lexicon_tags_2 },
    { 0x29ea8097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf49a8099, SUC_NE_lexicon_tags_9 },
    { 0x1a12809a, SUC_NE_lexicon_tags_2 },
    { 0x31e6809b, SUC_NE_lexicon_tags_2 },
    { 0x6f58809a, SUC_NE_lexicon_tags_36 },
    { 0xf2e2809d, SUC_NE_lexicon_tags_2 },
    { 0x5e22809a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5a80a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aa680aa, SUC_NE_lexicon_tags_2 },
    { 0xc25880aa, SUC_NE_lexicon_tags_2 },
    { 0xfc6880ac, SUC_NE_lexicon_tags_2 },
    { 0x5ba480ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecc80b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99ca80b4, SUC_NE_lexicon_tags_7 },
    { 0x46cc80b4, SUC_NE_lexicon_tags_2 },
    { 0x186c80b6, SUC_NE_lexicon_tags_2 },
    { 0x773880b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x971a80b9, SUC_NE_lexicon_tags_2 },
    { 0xf5d880ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x802480bc, SUC_NE_lexicon_tags_2 },
    { 0x45da80bc, SUC_NE_lexicon_tags_2 },
    { 0x1db480be, SUC_NE_lexicon_tags_2 },
    { 0xb34680bf, SUC_NE_lexicon_tags_1 },
    { 0xd96680c0, SUC_NE_lexicon_tags_2 },
    { 0xa08e80c0, SUC_NE_lexicon_tags_2 },
    { 0x6d8480c0, SUC_NE_lexicon_tags_4 },
    { 0xa6ea80c3, SUC_NE_lexicon_tags_2 },
    { 0xf5e880c0, SUC_NE_lexicon_tags_2 },
    { 0x2db480c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcf280c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2880cc, SUC_NE_lexicon_tags_58 },
    { 0x472e80cd, SUC_NE_lexicon_tags_208 },
    { 0x84d880cd, SUC_NE_lexicon_tags_2 },
    { 0x5b6080cd, SUC_NE_lexicon_tags_51 },
    { 0xbd2c80d0, SUC_NE_lexicon_tags_2 },
    { 0xab7680cc, SUC_NE_lexicon_tags_2 },
    { 0x83d480d0, SUC_NE_lexicon_tags_2 },
    { 0xa01c80d3, SUC_NE_lexicon_tags_2 },
    { 0xaec880d0, SUC_NE_lexicon_tags_2 },
    { 0xd39480d5, SUC_NE_lexicon_tags_2 },
    { 0x3c6880d6, SUC_NE_lexicon_tags_2 },
    { 0xc6ca80d4, SUC_NE_lexicon_tags_2 },
    { 0x82bc80ce, SUC_NE_lexicon_tags_2 },
    { 0x319280d1, SUC_NE_lexicon_tags_2 },
    { 0x447e80cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x870280dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9c080df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad9480e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x473080e3, SUC_NE_lexicon_tags_2 },
    { 0xcb880e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50d680e6, SUC_NE_lexicon_tags_4 },
    { 0xa9c280e7, SUC_NE_lexicon_tags_2 },
    { 0xe05280e8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a880ee, SUC_NE_lexicon_tags_2 },
    { 0x877e80ef, SUC_NE_lexicon_tags_2 },
    { 0x3a3080f0, SUC_NE_lexicon_tags_19 },
    { 0xfd6a80f1, SUC_NE_lexicon_tags_2 },
    { 0x319680f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f2680f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0a80fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6c80fe, SUC_NE_lexicon_tags_4 },
    { 0xaf5080ff, SUC_NE_lexicon_tags_2 },
    { 0x7cb48100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24a8104, SUC_NE_lexicon_tags_2 },
    { 0xf5ec8105, SUC_NE_lexicon_tags_2 },
    { 0x5cac8106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fc810e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf688111, SUC_NE_lexicon_tags_3 },
    { 0xad928111, SUC_NE_lexicon_tags_2 },
    { 0x53a88113, SUC_NE_lexicon_tags_2 },
    { 0x6f9e8114, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a68118, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce811c, SUC_NE_lexicon_tags_2 },
    { 0x7884811d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a6c811f, SUC_NE_lexicon_tags_2 },
    { 0x1b80811f, SUC_NE_lexicon_tags_2 },
    { 0xa4ee8121, SUC_NE_lexicon_tags_2 },
    { 0xc0428122, SUC_NE_lexicon_tags_2 },
    { 0x779c8121, SUC_NE_lexicon_tags_2 },
    { 0xbe308121, SUC_NE_lexicon_tags_2 },
    { 0x3b2c8125, SUC_NE_lexicon_tags_2 },
    { 0x85328123, SUC_NE_lexicon_tags_2 },
    { 0xe9e88127, SUC_NE_lexicon_tags_2 },
    { 0xaafc8122, SUC_NE_lexicon_tags_2 },
    { 0x6f4c8128, SUC_NE_lexicon_tags_2 },
    { 0x2058812a, SUC_NE_lexicon_tags_2 },
    { 0x18d68121, SUC_NE_lexicon_tags_2 },
    { 0xbf8c812c, SUC_NE_lexicon_tags_2 },
    { 0xf2a812d, SUC_NE_lexicon_tags_1 },
    { 0x2acc812e, SUC_NE_lexicon_tags_2 },
    { 0x1b6a812f, SUC_NE_lexicon_tags_2 },
    { 0xac6e812e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e48135, SUC_NE_lexicon_tags_2 },
    { 0x1a7c8136, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde428138, SUC_NE_lexicon_tags_337 },
    { 0x5a9e8139, SUC_NE_lexicon_tags_2 },
    { 0x2a06813a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89ce813c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb38813e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8eb08140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aea8144, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd08147, SUC_NE_lexicon_tags_2 },
    { 0x5fec8148, SUC_NE_lexicon_tags_2 },
    { 0x32e48149, SUC_NE_lexicon_tags_2 },
    { 0x14348149, SUC_NE_lexicon_tags_2 },
    { 0x4312814b, SUC_NE_lexicon_tags_2 },
    { 0xff8c8148, SUC_NE_lexicon_tags_2 },
    { 0xcf90814d, SUC_NE_lexicon_tags_2 },
    { 0x95a8814e, SUC_NE_lexicon_tags_2 },
    { 0xe8a4814d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fc48151, SUC_NE_lexicon_tags_2 },
    { 0x182e8151, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0aa8154, SUC_NE_lexicon_tags_2 },
    { 0xd3648154, SUC_NE_lexicon_tags_23 },
    { 0x75de8156, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de815a, SUC_NE_lexicon_tags_2 },
    { 0xf9a0815b, SUC_NE_lexicon_tags_4 },
    { 0xba08815c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaaec815e, SUC_NE_lexicon_tags_2 },
    { 0x7fec815e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af48162, SUC_NE_lexicon_tags_2 },
    { 0x61368163, SUC_NE_lexicon_tags_67 },
    { 0x5c528163, SUC_NE_lexicon_tags_2 },
    { 0x69128165, SUC_NE_lexicon_tags_2 },
    { 0x4a9e8162, SUC_NE_lexicon_tags_2 },
    { 0x74da8165, SUC_NE_lexicon_tags_2 },
    { 0xf4d68168, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe744816a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ae816e, SUC_NE_lexicon_tags_2 },
    { 0x2dca816f, SUC_NE_lexicon_tags_2 },
    { 0x743e8170, SUC_NE_lexicon_tags_2 },
    { 0xb6d08170, SUC_NE_lexicon_tags_2 },
    { 0xe2308172, SUC_NE_lexicon_tags_2 },
    { 0x7e5a8172, SUC_NE_lexicon_tags_2 },
    { 0x5ede8173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ed68176, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe4817b, SUC_NE_lexicon_tags_2 },
    { 0x2e2e817c, SUC_NE_lexicon_tags_2 },
    { 0x676a817d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902a8181, SUC_NE_lexicon_tags_2 },
    { 0xfdac8182, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535a8186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf5e8188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefa818a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d8818e, SUC_NE_lexicon_tags_2 },
    { 0x69a8818e, SUC_NE_lexicon_tags_2 },
    { 0xd4b48190, SUC_NE_lexicon_tags_2 },
    { 0xfe0c8191, SUC_NE_lexicon_tags_2 },
    { 0x6c66818e, SUC_NE_lexicon_tags_2 },
    { 0xb7be8193, SUC_NE_lexicon_tags_2 },
    { 0x9fb68193, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87788198, SUC_NE_lexicon_tags_237 },
    { 0, NULL },
    { 0x376e819a, SUC_NE_lexicon_tags_1 },
    { 0x9e22819b, SUC_NE_lexicon_tags_2 },
    { 0x230819c, SUC_NE_lexicon_tags_2 },
    { 0x3e82819d, SUC_NE_lexicon_tags_2 },
    { 0xeb16819e, SUC_NE_lexicon_tags_2 },
    { 0x47e8819d, SUC_NE_lexicon_tags_2 },
    { 0xfc7281a0, SUC_NE_lexicon_tags_2 },
    { 0xd42819f, SUC_NE_lexicon_tags_2 },
    { 0x5c88819f, SUC_NE_lexicon_tags_2 },
    { 0x4da281a2, SUC_NE_lexicon_tags_2 },
    { 0xfd2a81a4, SUC_NE_lexicon_tags_2 },
    { 0x912081a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25c81aa, SUC_NE_lexicon_tags_2 },
    { 0x721a81ab, SUC_NE_lexicon_tags_19 },
    { 0x8a1081ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c881b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1a81b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7a81bb, SUC_NE_lexicon_tags_2 },
    { 0x2ff481bc, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x6b3281be, SUC_NE_lexicon_tags_2 },
    { 0x8ffa81be, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x73f681c1, SUC_NE_lexicon_tags_2 },
    { 0x873a81c1, SUC_NE_lexicon_tags_2 },
    { 0xae81c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd98681c5, SUC_NE_lexicon_tags_2 },
    { 0xb8a881c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19ce81c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e081cc, SUC_NE_lexicon_tags_2 },
    { 0x8bf681cc, SUC_NE_lexicon_tags_2 },
    { 0xf8881ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e81d1, SUC_NE_lexicon_tags_2 },
    { 0x57aa81d2, SUC_NE_lexicon_tags_2 },
    { 0xaf7081d3, SUC_NE_lexicon_tags_2 },
    { 0x18e81d4, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb881db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320081e2, SUC_NE_lexicon_tags_2 },
    { 0x289a81e3, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1c81e6, SUC_NE_lexicon_tags_2 },
    { 0x9eec81e6, SUC_NE_lexicon_tags_2 },
    { 0xb08081e6, SUC_NE_lexicon_tags_2 },
    { 0xadb081e6, SUC_NE_lexicon_tags_2 },
    { 0xea2681e9, SUC_NE_lexicon_tags_2 },
    { 0x44ae81e6, SUC_NE_lexicon_tags_2 },
    { 0x9cb681e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132881f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41a881f9, SUC_NE_lexicon_tags_4 },
    { 0xaf7481f9, SUC_NE_lexicon_tags_2 },
    { 0x80b681fb, SUC_NE_lexicon_tags_2 },
    { 0x1cfc81fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbc81ff, SUC_NE_lexicon_tags_3 },
    { 0x84ae8200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64c8205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebb68207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9644820a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8e8210, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ec8213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5448821d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245a8221, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc28224, SUC_NE_lexicon_tags_2 },
    { 0x3ab28225, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3e822a, SUC_NE_lexicon_tags_2 },
    { 0x9316822a, SUC_NE_lexicon_tags_2 },
    { 0x5352822b, SUC_NE_lexicon_tags_2 },
    { 0xf09e822a, SUC_NE_lexicon_tags_2 },
    { 0xe3e4822e, SUC_NE_lexicon_tags_2 },
    { 0x102a822f, SUC_NE_lexicon_tags_2 },
    { 0xe33a8230, SUC_NE_lexicon_tags_2 },
    { 0xb2988230, SUC_NE_lexicon_tags_2 },
    { 0xe1708231, SUC_NE_lexicon_tags_7 },
    { 0x47ce8233, SUC_NE_lexicon_tags_2 },
    { 0x1442822d, SUC_NE_lexicon_tags_2 },
    { 0x8b18822e, SUC_NE_lexicon_tags_2 },
    { 0xe772822e, SUC_NE_lexicon_tags_2 },
    { 0x4ff88232, SUC_NE_lexicon_tags_2 },
    { 0xc0ea822f, SUC_NE_lexicon_tags_2 },
    { 0x24288239, SUC_NE_lexicon_tags_3 },
    { 0x23c08239, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe936823f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d48241, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26268244, SUC_NE_lexicon_tags_2 },
    { 0x63968245, SUC_NE_lexicon_tags_34 },
    { 0xd7248246, SUC_NE_lexicon_tags_2 },
    { 0xd0108246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb834824a, SUC_NE_lexicon_tags_2 },
    { 0xe672824b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c04824f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786a8253, SUC_NE_lexicon_tags_2 },
    { 0x8d168254, SUC_NE_lexicon_tags_13 },
    { 0x566c8255, SUC_NE_lexicon_tags_2 },
    { 0x17088254, SUC_NE_lexicon_tags_2 },
    { 0x30288257, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4a825a, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c68825f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646a8266, SUC_NE_lexicon_tags_2 },
    { 0x9ba48266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe744826a, SUC_NE_lexicon_tags_2 },
    { 0xc722826b, SUC_NE_lexicon_tags_2 },
    { 0x681a826c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1118826f, SUC_NE_lexicon_tags_2 },
    { 0x52d48270, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadda8274, SUC_NE_lexicon_tags_2 },
    { 0x3a968275, SUC_NE_lexicon_tags_2 },
    { 0xe6de8276, SUC_NE_lexicon_tags_2 },
    { 0xa5c88277, SUC_NE_lexicon_tags_2 },
    { 0x96de8277, SUC_NE_lexicon_tags_2 },
    { 0x78048279, SUC_NE_lexicon_tags_2 },
    { 0xd2a8827a, SUC_NE_lexicon_tags_2 },
    { 0x429e827a, SUC_NE_lexicon_tags_13 },
    { 0xb640827b, SUC_NE_lexicon_tags_2 },
    { 0x49108277, SUC_NE_lexicon_tags_2 },
    { 0x6a14827e, SUC_NE_lexicon_tags_2 },
    { 0xb632827f, SUC_NE_lexicon_tags_7 },
    { 0xf02827e, SUC_NE_lexicon_tags_2 },
    { 0x27588281, SUC_NE_lexicon_tags_2 },
    { 0xd5f2827f, SUC_NE_lexicon_tags_2 },
    { 0x9b88283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c68285, SUC_NE_lexicon_tags_2 },
    { 0x26ee8285, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3c6a8288, SUC_NE_lexicon_tags_2 },
    { 0xb9c8289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce88828d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6aaa828f, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503e8294, SUC_NE_lexicon_tags_2 },
    { 0x45ec8294, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99608299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6692829c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b2829f, SUC_NE_lexicon_tags_1 },
    { 0xb282a0, SUC_NE_lexicon_tags_2 },
    { 0xcd2c82a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x992482a3, SUC_NE_lexicon_tags_2 },
    { 0x5b4282a4, SUC_NE_lexicon_tags_2 },
    { 0xb7e682a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x865482a8, SUC_NE_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31a82af, SUC_NE_lexicon_tags_2 },
    { 0xa8c482b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46b282b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88282b6, SUC_NE_lexicon_tags_2 },
    { 0xfb1c82b6, SUC_NE_lexicon_tags_2 },
    { 0x551c82b7, SUC_NE_lexicon_tags_2 },
    { 0xe2ac82b9, SUC_NE_lexicon_tags_2 },
    { 0x5f6a82b7, SUC_NE_lexicon_tags_12 },
    { 0x380c82b8, SUC_NE_lexicon_tags_2 },
    { 0x7a0082bc, SUC_NE_lexicon_tags_0 },
    { 0x197082bd, SUC_NE_lexicon_tags_2 },
    { 0x710c82be, SUC_NE_lexicon_tags_2 },
    { 0xc3aa82bf, SUC_NE_lexicon_tags_96 },
    { 0x4e2a82bc, SUC_NE_lexicon_tags_2 },
    { 0x4fe482b7, SUC_NE_lexicon_tags_3 },
    { 0x82282c2, SUC_NE_lexicon_tags_2 },
    { 0x36a282c2, SUC_NE_lexicon_tags_2 },
    { 0xb88a82b7, SUC_NE_lexicon_tags_2 },
    { 0x4d5482c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14282c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf06c82ca, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42882ce, SUC_NE_lexicon_tags_2 },
    { 0x723a82cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x864a82d1, SUC_NE_lexicon_tags_2 },
    { 0x671682d2, SUC_NE_lexicon_tags_2 },
    { 0x8fe882d2, SUC_NE_lexicon_tags_0 },
    { 0x3fde82d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76282d8, SUC_NE_lexicon_tags_2 },
    { 0x2a4682d8, SUC_NE_lexicon_tags_2 },
    { 0x486c82da, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4c82df, SUC_NE_lexicon_tags_2 },
    { 0x4d8682e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x864082e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x945082e5, SUC_NE_lexicon_tags_2 },
    { 0xec6682e5, SUC_NE_lexicon_tags_2 },
    { 0x4cfc82e7, SUC_NE_lexicon_tags_2 },
    { 0xe6d282e8, SUC_NE_lexicon_tags_2 },
    { 0x7a1282e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadda82ec, SUC_NE_lexicon_tags_2 },
    { 0x3b5282ed, SUC_NE_lexicon_tags_2 },
    { 0x266e82ee, SUC_NE_lexicon_tags_2 },
    { 0x844082ee, SUC_NE_lexicon_tags_7 },
    { 0xe84c82f0, SUC_NE_lexicon_tags_2 },
    { 0x20b682f0, SUC_NE_lexicon_tags_2 },
    { 0x992282ee, SUC_NE_lexicon_tags_2 },
    { 0xcef882f2, SUC_NE_lexicon_tags_2 },
    { 0xe8e82f2, SUC_NE_lexicon_tags_2 },
    { 0x7b4a82f5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf57482f7, SUC_NE_lexicon_tags_2 },
    { 0x575c82f7, SUC_NE_lexicon_tags_4 },
    { 0xe1ca82f8, SUC_NE_lexicon_tags_2 },
    { 0xf5ae82f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715082fc, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x765882ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e8302, SUC_NE_lexicon_tags_2 },
    { 0x6a908303, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x69e68305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc88307, SUC_NE_lexicon_tags_2 },
    { 0x8848308, SUC_NE_lexicon_tags_2 },
    { 0xe93e8309, SUC_NE_lexicon_tags_17 },
    { 0x7a70830a, SUC_NE_lexicon_tags_2 },
    { 0x6912830a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd44830f, SUC_NE_lexicon_tags_2 },
    { 0x6b1e8310, SUC_NE_lexicon_tags_1 },
    { 0x66f88311, SUC_NE_lexicon_tags_2 },
    { 0x50ae8312, SUC_NE_lexicon_tags_2 },
    { 0x4e1e8313, SUC_NE_lexicon_tags_2 },
    { 0x648e8310, SUC_NE_lexicon_tags_2 },
    { 0x699c8312, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x65a68317, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7e831a, SUC_NE_lexicon_tags_2 },
    { 0xce40831b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e2831e, SUC_NE_lexicon_tags_2 },
    { 0x5106831f, SUC_NE_lexicon_tags_2 },
    { 0x362e8320, SUC_NE_lexicon_tags_2 },
    { 0xaf728321, SUC_NE_lexicon_tags_7 },
    { 0x96508321, SUC_NE_lexicon_tags_2 },
    { 0xa8268322, SUC_NE_lexicon_tags_2 },
    { 0x276e8324, SUC_NE_lexicon_tags_2 },
    { 0x42208325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0228328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ad0832a, SUC_NE_lexicon_tags_2 },
    { 0xbb40832a, SUC_NE_lexicon_tags_23 },
    { 0xa004832b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68428331, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab888334, SUC_NE_lexicon_tags_2 },
    { 0xffe08335, SUC_NE_lexicon_tags_2 },
    { 0xe0908336, SUC_NE_lexicon_tags_2 },
    { 0x617c8336, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36fa8339, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c4833d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217c8342, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71148345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63808347, SUC_NE_lexicon_tags_4 },
    { 0xd50c8347, SUC_NE_lexicon_tags_2 },
    { 0x10068348, SUC_NE_lexicon_tags_2 },
    { 0x3972834a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6992834d, SUC_NE_lexicon_tags_2 },
    { 0x2f7a834d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x243c8350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1a8355, SUC_NE_lexicon_tags_2 },
    { 0xb5c88356, SUC_NE_lexicon_tags_2 },
    { 0xbe3a8355, SUC_NE_lexicon_tags_2 },
    { 0xcf148358, SUC_NE_lexicon_tags_2 },
    { 0x10268359, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x9994835b, SUC_NE_lexicon_tags_2 },
    { 0x59e0835c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab70835e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9e8361, SUC_NE_lexicon_tags_2 },
    { 0xb7b48362, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0xb0c88364, SUC_NE_lexicon_tags_2 },
    { 0x87208365, SUC_NE_lexicon_tags_2 },
    { 0xa28a8364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17008369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8296836d, SUC_NE_lexicon_tags_1 },
    { 0x7376836d, SUC_NE_lexicon_tags_2 },
    { 0x90836d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec8372, SUC_NE_lexicon_tags_2 },
    { 0xaf9a8373, SUC_NE_lexicon_tags_2 },
    { 0x76548372, SUC_NE_lexicon_tags_2 },
    { 0xdcb08373, SUC_NE_lexicon_tags_2 },
    { 0x7f4a8376, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xbdd48378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667c8382, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e28385, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be28388, SUC_NE_lexicon_tags_2 },
    { 0xd5e48388, SUC_NE_lexicon_tags_2 },
    { 0xbd028388, SUC_NE_lexicon_tags_2 },
    { 0x898a838b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c88390, SUC_NE_lexicon_tags_2 },
    { 0xfd288391, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x15788394, SUC_NE_lexicon_tags_30 },
    { 0xac1a8395, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5e8399, SUC_NE_lexicon_tags_1 },
    { 0xec6839a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb27c839c, SUC_NE_lexicon_tags_2 },
    { 0xb712839d, SUC_NE_lexicon_tags_2 },
    { 0xfdfe839e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x135883a0, SUC_NE_lexicon_tags_2 },
    { 0x473c83a0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5683a4, SUC_NE_lexicon_tags_2 },
    { 0xf70483a4, SUC_NE_lexicon_tags_2 },
    { 0x55d883a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39a683a8, SUC_NE_lexicon_tags_2 },
    { 0x75da83a9, SUC_NE_lexicon_tags_2 },
    { 0xf10e83aa, SUC_NE_lexicon_tags_2 },
    { 0x1ca283aa, SUC_NE_lexicon_tags_2 },
    { 0x5b3c83ab, SUC_NE_lexicon_tags_2 },
    { 0x815683a8, SUC_NE_lexicon_tags_2 },
    { 0x7ffc83ae, SUC_NE_lexicon_tags_2 },
    { 0xb7ee83ae, SUC_NE_lexicon_tags_2 },
    { 0xf8d083b0, SUC_NE_lexicon_tags_59 },
    { 0xaeee83b1, SUC_NE_lexicon_tags_2 },
    { 0x56a883b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168283b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2e83b9, SUC_NE_lexicon_tags_2 },
    { 0xba2283ba, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66c83c3, SUC_NE_lexicon_tags_2 },
    { 0x611483c4, SUC_NE_lexicon_tags_2 },
    { 0x3a083c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fe83ca, SUC_NE_lexicon_tags_2 },
    { 0xfcec83ca, SUC_NE_lexicon_tags_2 },
    { 0x4f3283cb, SUC_NE_lexicon_tags_2 },
    { 0xad5483cb, SUC_NE_lexicon_tags_2 },
    { 0xd56283ca, SUC_NE_lexicon_tags_2 },
    { 0xb04e83cc, SUC_NE_lexicon_tags_2 },
    { 0x848483cb, SUC_NE_lexicon_tags_2 },
    { 0xee7a83cc, SUC_NE_lexicon_tags_24 },
    { 0xd41283d2, SUC_NE_lexicon_tags_2 },
    { 0x7dde83d3, SUC_NE_lexicon_tags_2 },
    { 0x4e6683d4, SUC_NE_lexicon_tags_2 },
    { 0xfb8283d5, SUC_NE_lexicon_tags_2 },
    { 0xbe8483d6, SUC_NE_lexicon_tags_4 },
    { 0x606e83d5, SUC_NE_lexicon_tags_2 },
    { 0xfa2683d8, SUC_NE_lexicon_tags_2 },
    { 0xeafe83d2, SUC_NE_lexicon_tags_2 },
    { 0xb8de83d5, SUC_NE_lexicon_tags_2 },
    { 0x18f283db, SUC_NE_lexicon_tags_2 },
    { 0x4c8083dc, SUC_NE_lexicon_tags_160 },
    { 0x927483dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05e83e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2083e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5283e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85c883e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9883ec, SUC_NE_lexicon_tags_2 },
    { 0xfbae83ed, SUC_NE_lexicon_tags_2 },
    { 0x201283ed, SUC_NE_lexicon_tags_2 },
    { 0x781683ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633a83f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x285083fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22a83fd, SUC_NE_lexicon_tags_2 },
    { 0xd9da83fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3a68400, SUC_NE_lexicon_tags_2 },
    { 0x1f3a8401, SUC_NE_lexicon_tags_2 },
    { 0xaf748402, SUC_NE_lexicon_tags_2 },
    { 0x11028403, SUC_NE_lexicon_tags_17 },
    { 0x97908404, SUC_NE_lexicon_tags_7 },
    { 0xd9648405, SUC_NE_lexicon_tags_2 },
    { 0xe4868403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4428408, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18968416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886841c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbe841e, SUC_NE_lexicon_tags_2 },
    { 0xb2a2841f, SUC_NE_lexicon_tags_2 },
    { 0xc7708420, SUC_NE_lexicon_tags_3 },
    { 0x31ac841e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ba8427, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a842d, SUC_NE_lexicon_tags_2 },
    { 0x4a2e842d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee8430, SUC_NE_lexicon_tags_2 },
    { 0x81088431, SUC_NE_lexicon_tags_2 },
    { 0xfa288431, SUC_NE_lexicon_tags_2 },
    { 0x28f68433, SUC_NE_lexicon_tags_2 },
    { 0x70c8430, SUC_NE_lexicon_tags_2 },
    { 0x75848430, SUC_NE_lexicon_tags_2 },
    { 0xb9448435, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc140843b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cc8440, SUC_NE_lexicon_tags_2 },
    { 0xcf628440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92888443, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35ba8445, SUC_NE_lexicon_tags_3 },
    { 0xaa08446, SUC_NE_lexicon_tags_3 },
    { 0xa0a28446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b08844a, SUC_NE_lexicon_tags_2 },
    { 0xd678844b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0da8452, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74b68454, SUC_NE_lexicon_tags_224 },
    { 0x712a8455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650845a, SUC_NE_lexicon_tags_2 },
    { 0x6594845a, SUC_NE_lexicon_tags_2 },
    { 0xbec0845c, SUC_NE_lexicon_tags_2 },
    { 0xaa16845a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x294a8460, SUC_NE_lexicon_tags_2 },
    { 0x7b1c8461, SUC_NE_lexicon_tags_17 },
    { 0x177e8462, SUC_NE_lexicon_tags_2 },
    { 0xefca8463, SUC_NE_lexicon_tags_2 },
    { 0x520a8462, SUC_NE_lexicon_tags_2 },
    { 0x124e8460, SUC_NE_lexicon_tags_2 },
    { 0x671e8464, SUC_NE_lexicon_tags_248 },
    { 0xc6ea8464, SUC_NE_lexicon_tags_2 },
    { 0xbb28468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a2846c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe048474, SUC_NE_lexicon_tags_24 },
    { 0xc05c8474, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82a8478, SUC_NE_lexicon_tags_4 },
    { 0x9a128479, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dc847d, SUC_NE_lexicon_tags_2 },
    { 0xf5b2847e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da08482, SUC_NE_lexicon_tags_2 },
    { 0xccb28483, SUC_NE_lexicon_tags_2 },
    { 0xdbc28483, SUC_NE_lexicon_tags_2 },
    { 0x55508485, SUC_NE_lexicon_tags_2 },
    { 0xdc2e8485, SUC_NE_lexicon_tags_2 },
    { 0xe9f08487, SUC_NE_lexicon_tags_2 },
    { 0xe2a48488, SUC_NE_lexicon_tags_2 },
    { 0x740a8486, SUC_NE_lexicon_tags_2 },
    { 0xee20848a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf44848e, SUC_NE_lexicon_tags_2 },
    { 0x8c94848f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa8494, SUC_NE_lexicon_tags_2 },
    { 0xae48495, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c848497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10e849c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2084a1, SUC_NE_lexicon_tags_2 },
    { 0x29fc84a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ac884a4, SUC_NE_lexicon_tags_2 },
    { 0x62a484a5, SUC_NE_lexicon_tags_2 },
    { 0x238a84a5, SUC_NE_lexicon_tags_2 },
    { 0x37684a6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x742284aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae884b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346e84b5, SUC_NE_lexicon_tags_2 },
    { 0x3dd484b5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd15284b8, SUC_NE_lexicon_tags_2 },
    { 0x8db884b9, SUC_NE_lexicon_tags_1 },
    { 0x550684b8, SUC_NE_lexicon_tags_2 },
    { 0xb70284bb, SUC_NE_lexicon_tags_2 },
    { 0xadea84bb, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640484c0, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2684c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x416e84c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9684cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd884d1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ee84db, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x749684dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622284e1, SUC_NE_lexicon_tags_3 },
    { 0xa7a284e2, SUC_NE_lexicon_tags_2 },
    { 0x76aa84e2, SUC_NE_lexicon_tags_2 },
    { 0xbb7e84e4, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7484e7, SUC_NE_lexicon_tags_2 },
    { 0x4f8484e7, SUC_NE_lexicon_tags_2 },
    { 0xca2684e9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc72284eb, SUC_NE_lexicon_tags_2 },
    { 0xe88084ec, SUC_NE_lexicon_tags_2 },
    { 0x7dee84ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3684f0, SUC_NE_lexicon_tags_2 },
    { 0xe2e484f0, SUC_NE_lexicon_tags_2 },
    { 0x8cca84f2, SUC_NE_lexicon_tags_2 },
    { 0x59be84f3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b484fb, SUC_NE_lexicon_tags_2 },
    { 0xf9fa84fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe684fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1a08500, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0850a, SUC_NE_lexicon_tags_2 },
    { 0x44cc850b, SUC_NE_lexicon_tags_2 },
    { 0x7dc4850b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fa8513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b68516, SUC_NE_lexicon_tags_2 },
    { 0xd1fe8517, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce2851b, SUC_NE_lexicon_tags_2 },
    { 0xdf62851b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e28520, SUC_NE_lexicon_tags_3 },
    { 0xf61c8521, SUC_NE_lexicon_tags_2 },
    { 0x99468520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb928524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a948528, SUC_NE_lexicon_tags_2 },
    { 0xe19e8529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8822852b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8852e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfad48530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca08533, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c3e8535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1228538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c8542, SUC_NE_lexicon_tags_2 },
    { 0x7a608543, SUC_NE_lexicon_tags_2 },
    { 0x7f5a8544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bea8546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa48548, SUC_NE_lexicon_tags_2 },
    { 0xd1be8549, SUC_NE_lexicon_tags_2 },
    { 0xaf70854a, SUC_NE_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x255c854d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x638c854f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75208553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d88558, SUC_NE_lexicon_tags_2 },
    { 0xa488559, SUC_NE_lexicon_tags_2 },
    { 0x79528559, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde7a855c, SUC_NE_lexicon_tags_2 },
    { 0xe824855d, SUC_NE_lexicon_tags_2 },
    { 0xd590855d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a968563, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd908566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc468568, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x333c856a, SUC_NE_lexicon_tags_17 },
    { 0x1d0c856a, SUC_NE_lexicon_tags_2 },
    { 0xe0f8856c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a36856e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a08573, SUC_NE_lexicon_tags_2 },
    { 0x29fe8574, SUC_NE_lexicon_tags_2 },
    { 0x12a68574, SUC_NE_lexicon_tags_10 },
    { 0x207e8575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fc857b, SUC_NE_lexicon_tags_0 },
    { 0xe18c857c, SUC_NE_lexicon_tags_2 },
    { 0xa870857d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed68580, SUC_NE_lexicon_tags_2 },
    { 0x7f888580, SUC_NE_lexicon_tags_2 },
    { 0xce908581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9a858b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1040858d, SUC_NE_lexicon_tags_2 },
    { 0xb830858e, SUC_NE_lexicon_tags_2 },
    { 0x29a6858e, SUC_NE_lexicon_tags_2 },
    { 0x91f48590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe68c8592, SUC_NE_lexicon_tags_2 },
    { 0x40648593, SUC_NE_lexicon_tags_2 },
    { 0x2f428594, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641c8599, SUC_NE_lexicon_tags_2 },
    { 0x1c12859a, SUC_NE_lexicon_tags_2 },
    { 0x70a8859b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa316859d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x4108859f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x305e85a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x513285a4, SUC_NE_lexicon_tags_2 },
    { 0xbe0285a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5685a8, SUC_NE_lexicon_tags_2 },
    { 0x429885a9, SUC_NE_lexicon_tags_2 },
    { 0xdeda85aa, SUC_NE_lexicon_tags_2 },
    { 0xbc5c85a9, SUC_NE_lexicon_tags_2 },
    { 0x61f285ab, SUC_NE_lexicon_tags_2 },
    { 0xc14285a9, SUC_NE_lexicon_tags_2 },
    { 0xdd7885ae, SUC_NE_lexicon_tags_2 },
    { 0xd42685af, SUC_NE_lexicon_tags_2 },
    { 0x667685b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c85b4, SUC_NE_lexicon_tags_2 },
    { 0xab8e85b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a85bb, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701285bf, SUC_NE_lexicon_tags_2 },
    { 0x26be85c0, SUC_NE_lexicon_tags_2 },
    { 0xf30c85c1, SUC_NE_lexicon_tags_2 },
    { 0x213085c0, SUC_NE_lexicon_tags_2 },
    { 0x130485c1, SUC_NE_lexicon_tags_2 },
    { 0x3f5a85c2, SUC_NE_lexicon_tags_2 },
    { 0x552085c1, SUC_NE_lexicon_tags_2 },
    { 0xb2085c4, SUC_NE_lexicon_tags_2 },
    { 0x19b285c7, SUC_NE_lexicon_tags_13 },
    { 0x95d085c7, SUC_NE_lexicon_tags_19 },
    { 0x7aaa85c1, SUC_NE_lexicon_tags_80 },
    { 0x6a9085ca, SUC_NE_lexicon_tags_2 },
    { 0x4e8285c7, SUC_NE_lexicon_tags_2 },
    { 0x8a0285cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b885d0, SUC_NE_lexicon_tags_2 },
    { 0x24dc85d1, SUC_NE_lexicon_tags_2 },
    { 0x408c85d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8a85d6, SUC_NE_lexicon_tags_2 },
    { 0x75ac85d7, SUC_NE_lexicon_tags_2 },
    { 0xfe8c85d6, SUC_NE_lexicon_tags_220 },
    { 0xe2a85d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc85dd, SUC_NE_lexicon_tags_34 },
    { 0x92a485de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0685e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcea285e3, SUC_NE_lexicon_tags_1 },
    { 0xb13885e4, SUC_NE_lexicon_tags_147 },
    { 0x731485e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb285eb, SUC_NE_lexicon_tags_2 },
    { 0xb84285ec, SUC_NE_lexicon_tags_4 },
    { 0xab2e85eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24d085ef, SUC_NE_lexicon_tags_2 },
    { 0xd7bc85f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404285f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91f485f7, SUC_NE_lexicon_tags_2 },
    { 0x72285f8, SUC_NE_lexicon_tags_3 },
    { 0xf1b085f9, SUC_NE_lexicon_tags_2 },
    { 0x9e8685f9, SUC_NE_lexicon_tags_2 },
    { 0x7d085f9, SUC_NE_lexicon_tags_2 },
    { 0x27be85fb, SUC_NE_lexicon_tags_2 },
    { 0xaf5285fa, SUC_NE_lexicon_tags_2 },
    { 0x604685fe, SUC_NE_lexicon_tags_13 },
    { 0x124e85f8, SUC_NE_lexicon_tags_2 },
    { 0xab208600, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x14748603, SUC_NE_lexicon_tags_2 },
    { 0x2d728603, SUC_NE_lexicon_tags_4 },
    { 0x45108605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x492e8607, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9082860a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257e860f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0a8612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c68616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c48619, SUC_NE_lexicon_tags_2 },
    { 0x27b8861a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb56861c, SUC_NE_lexicon_tags_4 },
    { 0xa938861d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8888624, SUC_NE_lexicon_tags_2 },
    { 0xd73e8625, SUC_NE_lexicon_tags_2 },
    { 0xdd048625, SUC_NE_lexicon_tags_2 },
    { 0x65548627, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73c68629, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfe4862b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c4862f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f388631, SUC_NE_lexicon_tags_2 },
    { 0x98cc8631, SUC_NE_lexicon_tags_2 },
    { 0x25fe8631, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e8636, SUC_NE_lexicon_tags_2 },
    { 0x39fe8637, SUC_NE_lexicon_tags_2 },
    { 0xb71a8638, SUC_NE_lexicon_tags_7 },
    { 0x64848638, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4a863c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6154863e, SUC_NE_lexicon_tags_2 },
    { 0x8706863e, SUC_NE_lexicon_tags_1 },
    { 0xc38a8640, SUC_NE_lexicon_tags_2 },
    { 0xff088641, SUC_NE_lexicon_tags_2 },
    { 0x9c5e8642, SUC_NE_lexicon_tags_2 },
    { 0xf7e88640, SUC_NE_lexicon_tags_2 },
    { 0x7e7e8644, SUC_NE_lexicon_tags_2 },
    { 0x5e0c8645, SUC_NE_lexicon_tags_3 },
    { 0xde68646, SUC_NE_lexicon_tags_2 },
    { 0x4a388646, SUC_NE_lexicon_tags_0 },
    { 0xe1968644, SUC_NE_lexicon_tags_2 },
    { 0xf2668648, SUC_NE_lexicon_tags_2 },
    { 0xfa864a, SUC_NE_lexicon_tags_2 },
    { 0xb2368646, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefae864d, SUC_NE_lexicon_tags_2 },
    { 0xcdc2864d, SUC_NE_lexicon_tags_4 },
    { 0xd842864e, SUC_NE_lexicon_tags_2 },
    { 0x1756864d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d008652, SUC_NE_lexicon_tags_2 },
    { 0x8b588653, SUC_NE_lexicon_tags_4 },
    { 0x43468654, SUC_NE_lexicon_tags_2 },
    { 0x30ea8652, SUC_NE_lexicon_tags_2 },
    { 0xe90a8656, SUC_NE_lexicon_tags_2 },
    { 0xbd608657, SUC_NE_lexicon_tags_2 },
    { 0x13a28654, SUC_NE_lexicon_tags_4 },
    { 0xe9788652, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9bc0865b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8726865d, SUC_NE_lexicon_tags_2 },
    { 0x4ca0865d, SUC_NE_lexicon_tags_2 },
    { 0x1e6c865f, SUC_NE_lexicon_tags_2 },
    { 0x41d68660, SUC_NE_lexicon_tags_2 },
    { 0xfb068661, SUC_NE_lexicon_tags_83 },
    { 0x6a0a8661, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4108668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6562866c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa75c866e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4728670, SUC_NE_lexicon_tags_2 },
    { 0xcd648671, SUC_NE_lexicon_tags_2 },
    { 0x15168672, SUC_NE_lexicon_tags_0 },
    { 0x94ec8671, SUC_NE_lexicon_tags_2 },
    { 0x1d748673, SUC_NE_lexicon_tags_2 },
    { 0x459e8675, SUC_NE_lexicon_tags_2 },
    { 0x18348676, SUC_NE_lexicon_tags_2 },
    { 0x669c8670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7984867b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b2867f, SUC_NE_lexicon_tags_2 },
    { 0xc6ec8680, SUC_NE_lexicon_tags_7 },
    { 0xcf9a8680, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75aa8683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8db28685, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8148689, SUC_NE_lexicon_tags_2 },
    { 0x5a9e8689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5c0868c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44868f, SUC_NE_lexicon_tags_69 },
    { 0xb5c88690, SUC_NE_lexicon_tags_2 },
    { 0xf9708691, SUC_NE_lexicon_tags_4 },
    { 0xf87a8691, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f868698, SUC_NE_lexicon_tags_2 },
    { 0x9cca8699, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a0869c, SUC_NE_lexicon_tags_2 },
    { 0xef88869d, SUC_NE_lexicon_tags_2 },
    { 0x4d94869e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4486a1, SUC_NE_lexicon_tags_2 },
    { 0x81d086a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bec86a5, SUC_NE_lexicon_tags_2 },
    { 0x20b686a6, SUC_NE_lexicon_tags_2 },
    { 0x1a0286a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e86aa, SUC_NE_lexicon_tags_2 },
    { 0x37de86ab, SUC_NE_lexicon_tags_2 },
    { 0x30de86aa, SUC_NE_lexicon_tags_2 },
    { 0xe85086aa, SUC_NE_lexicon_tags_2 },
    { 0x81bc86ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd9086b0, SUC_NE_lexicon_tags_17 },
    { 0xb18e86b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dc86b8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2886be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38ec86c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4286c3, SUC_NE_lexicon_tags_2 },
    { 0xa74486c4, SUC_NE_lexicon_tags_2 },
    { 0x6ca086c5, SUC_NE_lexicon_tags_2 },
    { 0xdd2a86c6, SUC_NE_lexicon_tags_2 },
    { 0x631286c5, SUC_NE_lexicon_tags_2 },
    { 0x1b8c86c8, SUC_NE_lexicon_tags_2 },
    { 0x24dc86c6, SUC_NE_lexicon_tags_2 },
    { 0x8dd886c3, SUC_NE_lexicon_tags_2 },
    { 0xe42086cb, SUC_NE_lexicon_tags_2 },
    { 0xbcc886cc, SUC_NE_lexicon_tags_2 },
    { 0xf57886cd, SUC_NE_lexicon_tags_1 },
    { 0x447c86cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109486d2, SUC_NE_lexicon_tags_2 },
    { 0xf93c86d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ec86d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143c86dc, SUC_NE_lexicon_tags_2 },
    { 0x895c86dd, SUC_NE_lexicon_tags_2 },
    { 0xcfde86de, SUC_NE_lexicon_tags_18 },
    { 0xdb6c86df, SUC_NE_lexicon_tags_2 },
    { 0x980686e0, SUC_NE_lexicon_tags_2 },
    { 0xd0e86e1, SUC_NE_lexicon_tags_2 },
    { 0xcf7686e2, SUC_NE_lexicon_tags_2 },
    { 0x90c486e0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x551686e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c86ea, SUC_NE_lexicon_tags_2 },
    { 0x785086ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x503086ee, SUC_NE_lexicon_tags_2 },
    { 0x147686ef, SUC_NE_lexicon_tags_2 },
    { 0xb6fe86f0, SUC_NE_lexicon_tags_1 },
    { 0xb3ae86f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6086f4, SUC_NE_lexicon_tags_2 },
    { 0xb0e486f5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d486f8, SUC_NE_lexicon_tags_2 },
    { 0x9ad686f9, SUC_NE_lexicon_tags_2 },
    { 0xb46a86f8, SUC_NE_lexicon_tags_2 },
    { 0xd48a86fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6c8701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b08706, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e08709, SUC_NE_lexicon_tags_2 },
    { 0x950870a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf384870c, SUC_NE_lexicon_tags_2 },
    { 0xce5a870d, SUC_NE_lexicon_tags_2 },
    { 0xb6e4870e, SUC_NE_lexicon_tags_7 },
    { 0xf594870e, SUC_NE_lexicon_tags_5 },
    { 0x272c8710, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5a88712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fc8715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabfc8717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d0871f, SUC_NE_lexicon_tags_24 },
    { 0xd5e48720, SUC_NE_lexicon_tags_2 },
    { 0xa5188721, SUC_NE_lexicon_tags_2 },
    { 0xa2708722, SUC_NE_lexicon_tags_2 },
    { 0xa2508723, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x30848726, SUC_NE_lexicon_tags_2 },
    { 0xed7e8727, SUC_NE_lexicon_tags_2 },
    { 0x17b68727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fc872b, SUC_NE_lexicon_tags_4 },
    { 0x4908872b, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0xcf2e872e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91168732, SUC_NE_lexicon_tags_2 },
    { 0xfbfa8732, SUC_NE_lexicon_tags_3 },
    { 0xad2c8734, SUC_NE_lexicon_tags_2 },
    { 0xae5a8735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ee8738, SUC_NE_lexicon_tags_2 },
    { 0xef708738, SUC_NE_lexicon_tags_7 },
    { 0xcd24873a, SUC_NE_lexicon_tags_4 },
    { 0xeb72873b, SUC_NE_lexicon_tags_346 },
    { 0x8cd0873b, SUC_NE_lexicon_tags_1 },
    { 0xe776873d, SUC_NE_lexicon_tags_2 },
    { 0xb51a873e, SUC_NE_lexicon_tags_2 },
    { 0xbb2873b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08c8742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92a8744, SUC_NE_lexicon_tags_2 },
    { 0x305c8744, SUC_NE_lexicon_tags_2 },
    { 0x7e0e8744, SUC_NE_lexicon_tags_2 },
    { 0xa6188747, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cde8749, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x77b6874b, SUC_NE_lexicon_tags_2 },
    { 0xf292874c, SUC_NE_lexicon_tags_3 },
    { 0xf576874c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4004874f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14888754, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb668756, SUC_NE_lexicon_tags_2 },
    { 0xea388756, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe6875b, SUC_NE_lexicon_tags_2 },
    { 0xf950875c, SUC_NE_lexicon_tags_17 },
    { 0x8e0a875d, SUC_NE_lexicon_tags_2 },
    { 0x6fa875d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2528762, SUC_NE_lexicon_tags_2 },
    { 0x869c8763, SUC_NE_lexicon_tags_2 },
    { 0x70e28764, SUC_NE_lexicon_tags_7 },
    { 0xe08c8765, SUC_NE_lexicon_tags_56 },
    { 0x465a8766, SUC_NE_lexicon_tags_2 },
    { 0x19ce8767, SUC_NE_lexicon_tags_2 },
    { 0x4c388763, SUC_NE_lexicon_tags_4 },
    { 0xe88a8765, SUC_NE_lexicon_tags_2 },
    { 0x46c6876a, SUC_NE_lexicon_tags_3 },
    { 0x61b2876b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xae18876e, SUC_NE_lexicon_tags_2 },
    { 0xc994876e, SUC_NE_lexicon_tags_2 },
    { 0xb3da8770, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e8776, SUC_NE_lexicon_tags_7 },
    { 0x63a08777, SUC_NE_lexicon_tags_2 },
    { 0x367c8778, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc4877d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97748782, SUC_NE_lexicon_tags_2 },
    { 0x1b328782, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39e8788, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xda18878a, SUC_NE_lexicon_tags_4 },
    { 0x170c878b, SUC_NE_lexicon_tags_2 },
    { 0xdf6a878c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9088790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96de8794, SUC_NE_lexicon_tags_2 },
    { 0x78b88795, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d5a8797, SUC_NE_lexicon_tags_2 },
    { 0x4a748798, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c8879b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b8879e, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d287a0, SUC_NE_lexicon_tags_2 },
    { 0x3a2687a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x548e87a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74087a7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6e87ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20087b3, SUC_NE_lexicon_tags_2 },
    { 0x9a6287b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe90687b6, SUC_NE_lexicon_tags_2 },
    { 0x39ca87b6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6b687b9, SUC_NE_lexicon_tags_2 },
    { 0xc35287b9, SUC_NE_lexicon_tags_2 },
    { 0x3ac87bb, SUC_NE_lexicon_tags_2 },
    { 0x6c5687bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd65487be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x472087c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x757887c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x983c87c5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x778087c7, SUC_NE_lexicon_tags_2 },
    { 0xf7a887c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc487cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef3c87cd, SUC_NE_lexicon_tags_2 },
    { 0xbca887ce, SUC_NE_lexicon_tags_2 },
    { 0xcdae87cf, SUC_NE_lexicon_tags_2 },
    { 0xf96e87cd, SUC_NE_lexicon_tags_2 },
    { 0x88687cf, SUC_NE_lexicon_tags_1 },
    { 0xbcea87d0, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a087d6, SUC_NE_lexicon_tags_3 },
    { 0xa8ba87d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad2287d9, SUC_NE_lexicon_tags_2 },
    { 0xf94487da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34e087dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07487df, SUC_NE_lexicon_tags_2 },
    { 0x5ae487df, SUC_NE_lexicon_tags_49 },
    { 0x2d9887e1, SUC_NE_lexicon_tags_18 },
    { 0xb60a87e2, SUC_NE_lexicon_tags_2 },
    { 0xab4687e3, SUC_NE_lexicon_tags_81 },
    { 0xcb0e87e0, SUC_NE_lexicon_tags_2 },
    { 0x4e87e2, SUC_NE_lexicon_tags_2 },
    { 0x160e87e6, SUC_NE_lexicon_tags_2 },
    { 0x18b687e7, SUC_NE_lexicon_tags_4 },
    { 0xa9b887e8, SUC_NE_lexicon_tags_2 },
    { 0xb98487e9, SUC_NE_lexicon_tags_2 },
    { 0x2c2c87ea, SUC_NE_lexicon_tags_2 },
    { 0x585487ea, SUC_NE_lexicon_tags_2 },
    { 0x3c6887e3, SUC_NE_lexicon_tags_1 },
    { 0x8c4287ec, SUC_NE_lexicon_tags_2 },
    { 0x76e87ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa80887f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26f087f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7a87f7, SUC_NE_lexicon_tags_7 },
    { 0x94f887f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24dc87fa, SUC_NE_lexicon_tags_2 },
    { 0xc24687fa, SUC_NE_lexicon_tags_2 },
    { 0x695087fc, SUC_NE_lexicon_tags_2 },
    { 0xa2d087fd, SUC_NE_lexicon_tags_2 },
    { 0xcb3a87fe, SUC_NE_lexicon_tags_2 },
    { 0xae6287fb, SUC_NE_lexicon_tags_2 },
    { 0x5d5287ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc048807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e048809, SUC_NE_lexicon_tags_2 },
    { 0x34828809, SUC_NE_lexicon_tags_2 },
    { 0x9ae0880a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dbe880d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa88810, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x99c88812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32768819, SUC_NE_lexicon_tags_2 },
    { 0x328c881a, SUC_NE_lexicon_tags_2 },
    { 0xe936881a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafa881e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd14a8820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c48826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9188828, SUC_NE_lexicon_tags_2 },
    { 0x2df08829, SUC_NE_lexicon_tags_2 },
    { 0xbbb6882a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xf408882c, SUC_NE_lexicon_tags_2 },
    { 0xfaa6882d, SUC_NE_lexicon_tags_2 },
    { 0xb154882d, SUC_NE_lexicon_tags_235 },
    { 0xbcec882e, SUC_NE_lexicon_tags_2 },
    { 0x596e8830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a148832, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a028835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe828838, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a883c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd228840, SUC_NE_lexicon_tags_2 },
    { 0x8ad08841, SUC_NE_lexicon_tags_2 },
    { 0x8d6a8841, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71248846, SUC_NE_lexicon_tags_2 },
    { 0x721c8847, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e1c8849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e9a884b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe248884f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcab88851, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8885c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ade885e, SUC_NE_lexicon_tags_2 },
    { 0xc630885e, SUC_NE_lexicon_tags_2 },
    { 0x80ae885e, SUC_NE_lexicon_tags_2 },
    { 0x25f88860, SUC_NE_lexicon_tags_2 },
    { 0x19548862, SUC_NE_lexicon_tags_4 },
    { 0x365c8863, SUC_NE_lexicon_tags_2 },
    { 0x9f308862, SUC_NE_lexicon_tags_2 },
    { 0x73aa8865, SUC_NE_lexicon_tags_2 },
    { 0x40b28864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0e28868, SUC_NE_lexicon_tags_58 },
    { 0x84028868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe522886c, SUC_NE_lexicon_tags_17 },
    { 0x5f1a886d, SUC_NE_lexicon_tags_2 },
    { 0xe18886d, SUC_NE_lexicon_tags_2 },
    { 0x55a2886f, SUC_NE_lexicon_tags_4 },
    { 0xaefa886f, SUC_NE_lexicon_tags_2 },
    { 0x74ca886e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f48876, SUC_NE_lexicon_tags_2 },
    { 0x7b248876, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x486a8879, SUC_NE_lexicon_tags_2 },
    { 0x7fb88879, SUC_NE_lexicon_tags_2 },
    { 0xc6768879, SUC_NE_lexicon_tags_2 },
    { 0x56b8887b, SUC_NE_lexicon_tags_2 },
    { 0x7bd8887d, SUC_NE_lexicon_tags_2 },
    { 0xb3ee887d, SUC_NE_lexicon_tags_2 },
    { 0x88688879, SUC_NE_lexicon_tags_2 },
    { 0x96e48880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19bc8882, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caa8885, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafbc8887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97aa8889, SUC_NE_lexicon_tags_2 },
    { 0xf4be888a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e728890, SUC_NE_lexicon_tags_3 },
    { 0xc9468890, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda308899, SUC_NE_lexicon_tags_2 },
    { 0x3bfa889a, SUC_NE_lexicon_tags_2 },
    { 0x9baa889b, SUC_NE_lexicon_tags_0 },
    { 0x7c5e889b, SUC_NE_lexicon_tags_2 },
    { 0x9418889d, SUC_NE_lexicon_tags_2 },
    { 0x5b16889c, SUC_NE_lexicon_tags_2 },
    { 0x6770889c, SUC_NE_lexicon_tags_4 },
    { 0xe52c889a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2888a2, SUC_NE_lexicon_tags_19 },
    { 0x521688a2, SUC_NE_lexicon_tags_5 },
    { 0x6d6088a3, SUC_NE_lexicon_tags_2 },
    { 0x796088a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x163288a7, SUC_NE_lexicon_tags_2 },
    { 0x266488a8, SUC_NE_lexicon_tags_2 },
    { 0x241488a9, SUC_NE_lexicon_tags_2 },
    { 0x99bc88aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2a88ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578c88b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600288b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x763488b9, SUC_NE_lexicon_tags_2 },
    { 0x6fe288ba, SUC_NE_lexicon_tags_19 },
    { 0xdfe488b9, SUC_NE_lexicon_tags_2 },
    { 0x22ec88b9, SUC_NE_lexicon_tags_2 },
    { 0xddce88bd, SUC_NE_lexicon_tags_2 },
    { 0xce8688bc, SUC_NE_lexicon_tags_2 },
    { 0xee5688bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca88c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388888c8, SUC_NE_lexicon_tags_2 },
    { 0x3b4c88c8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ca88cc, SUC_NE_lexicon_tags_2 },
    { 0x28d688cc, SUC_NE_lexicon_tags_2 },
    { 0x1d5e88ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52088d3, SUC_NE_lexicon_tags_2 },
    { 0xdaa88d4, SUC_NE_lexicon_tags_2 },
    { 0xca7688d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bea88d9, SUC_NE_lexicon_tags_17 },
    { 0x9a8a88d9, SUC_NE_lexicon_tags_2 },
    { 0x980288d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb13288dd, SUC_NE_lexicon_tags_2 },
    { 0x2da88de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f688e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb888e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96be88e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372288ec, SUC_NE_lexicon_tags_2 },
    { 0x1a4888ec, SUC_NE_lexicon_tags_2 },
    { 0x43e888ee, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ab688f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77688f3, SUC_NE_lexicon_tags_7 },
    { 0x87ee88f3, SUC_NE_lexicon_tags_2 },
    { 0x21ba88f5, SUC_NE_lexicon_tags_2 },
    { 0x3b2288f4, SUC_NE_lexicon_tags_2 },
    { 0x56e488f7, SUC_NE_lexicon_tags_2 },
    { 0xcfcc88f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417e88fe, SUC_NE_lexicon_tags_7 },
    { 0x10b488ff, SUC_NE_lexicon_tags_2 },
    { 0x7ca88ff, SUC_NE_lexicon_tags_1 },
    { 0xc1948900, SUC_NE_lexicon_tags_2 },
    { 0x9a68900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ae8904, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd8c8906, SUC_NE_lexicon_tags_2 },
    { 0xa5088907, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf378890b, SUC_NE_lexicon_tags_2 },
    { 0x774c890b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae748910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e08913, SUC_NE_lexicon_tags_2 },
    { 0x72a8914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ca8917, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3da48919, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d4891c, SUC_NE_lexicon_tags_2 },
    { 0xdd00891c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964a8921, SUC_NE_lexicon_tags_2 },
    { 0xdc0c8922, SUC_NE_lexicon_tags_2 },
    { 0x83888921, SUC_NE_lexicon_tags_2 },
    { 0x78988924, SUC_NE_lexicon_tags_2 },
    { 0x97948923, SUC_NE_lexicon_tags_2 },
    { 0x5a508926, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d72892a, SUC_NE_lexicon_tags_2 },
    { 0xe87e892a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3912892e, SUC_NE_lexicon_tags_2 },
    { 0x1280892f, SUC_NE_lexicon_tags_2 },
    { 0xde84892f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd068934, SUC_NE_lexicon_tags_2 },
    { 0x84e8935, SUC_NE_lexicon_tags_2 },
    { 0x61288935, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x9de68938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2f0893a, SUC_NE_lexicon_tags_2 },
    { 0x8072893a, SUC_NE_lexicon_tags_2 },
    { 0x11de893c, SUC_NE_lexicon_tags_2 },
    { 0xe3c893d, SUC_NE_lexicon_tags_7 },
    { 0xd8ba893c, SUC_NE_lexicon_tags_17 },
    { 0x1f8893f, SUC_NE_lexicon_tags_2 },
    { 0x2a36893f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b48948, SUC_NE_lexicon_tags_2 },
    { 0xe70c8949, SUC_NE_lexicon_tags_2 },
    { 0x8dd2894a, SUC_NE_lexicon_tags_2 },
    { 0x5b02894a, SUC_NE_lexicon_tags_40 },
    { 0x9c60894c, SUC_NE_lexicon_tags_2 },
    { 0xd9ea8949, SUC_NE_lexicon_tags_98 },
    { 0xefc6894b, SUC_NE_lexicon_tags_2 },
    { 0xc17c894f, SUC_NE_lexicon_tags_2 },
    { 0x8c38894f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87948953, SUC_NE_lexicon_tags_2 },
    { 0x27288953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95c8958, SUC_NE_lexicon_tags_2 },
    { 0xb1948959, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46895f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d288961, SUC_NE_lexicon_tags_2 },
    { 0x85c48961, SUC_NE_lexicon_tags_17 },
    { 0x9f0a8963, SUC_NE_lexicon_tags_2 },
    { 0x73668964, SUC_NE_lexicon_tags_2 },
    { 0x62ae8962, SUC_NE_lexicon_tags_2 },
    { 0x41428966, SUC_NE_lexicon_tags_2 },
    { 0x7e028962, SUC_NE_lexicon_tags_2 },
    { 0xe47a8968, SUC_NE_lexicon_tags_2 },
    { 0xf00a8964, SUC_NE_lexicon_tags_2 },
    { 0x5dce896a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1778896d, SUC_NE_lexicon_tags_2 },
    { 0x978c896d, SUC_NE_lexicon_tags_2 },
    { 0xf42a896d, SUC_NE_lexicon_tags_2 },
    { 0x5928896f, SUC_NE_lexicon_tags_2 },
    { 0x35408971, SUC_NE_lexicon_tags_7 },
    { 0x99da8972, SUC_NE_lexicon_tags_9 },
    { 0x41bc8973, SUC_NE_lexicon_tags_2 },
    { 0x4bc48973, SUC_NE_lexicon_tags_2 },
    { 0x466896f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ec8978, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d4897c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e02897f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9b3e8981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece8985, SUC_NE_lexicon_tags_2 },
    { 0x357c8986, SUC_NE_lexicon_tags_5 },
    { 0x4d6a8987, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6e8999, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81a6899b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca489a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac1889a2, SUC_NE_lexicon_tags_2 },
    { 0xeab889a3, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d489a6, SUC_NE_lexicon_tags_2 },
    { 0xca9e89a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f8e89a9, SUC_NE_lexicon_tags_2 },
    { 0x2f489a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dc89af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3489b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5489b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf16a89ba, SUC_NE_lexicon_tags_2 },
    { 0x24c889bb, SUC_NE_lexicon_tags_4 },
    { 0x68a489bb, SUC_NE_lexicon_tags_2 },
    { 0x91e689bd, SUC_NE_lexicon_tags_2 },
    { 0xbac689bc, SUC_NE_lexicon_tags_2 },
    { 0x652489bf, SUC_NE_lexicon_tags_2 },
    { 0xc62689c0, SUC_NE_lexicon_tags_2 },
    { 0x1d9e89be, SUC_NE_lexicon_tags_2 },
    { 0x475c89c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c089c5, SUC_NE_lexicon_tags_2 },
    { 0x616089c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23e289c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b1c89ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252a89cf, SUC_NE_lexicon_tags_2 },
    { 0x7ab089cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781889d2, SUC_NE_lexicon_tags_2 },
    { 0xe6a89d3, SUC_NE_lexicon_tags_2 },
    { 0xc8b089d4, SUC_NE_lexicon_tags_2 },
    { 0xbcba89d5, SUC_NE_lexicon_tags_2 },
    { 0x5e6089d5, SUC_NE_lexicon_tags_7 },
    { 0xc33089d6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733689db, SUC_NE_lexicon_tags_2 },
    { 0x712a89dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9689e0, SUC_NE_lexicon_tags_2 },
    { 0xba6c89e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5289e4, SUC_NE_lexicon_tags_7 },
    { 0x1e3c89e5, SUC_NE_lexicon_tags_2 },
    { 0xfea489e6, SUC_NE_lexicon_tags_7 },
    { 0x5e8689e7, SUC_NE_lexicon_tags_2 },
    { 0x18f689e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x168689ea, SUC_NE_lexicon_tags_2 },
    { 0xae5e89eb, SUC_NE_lexicon_tags_2 },
    { 0x13b289ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3689ef, SUC_NE_lexicon_tags_2 },
    { 0x2b7289ef, SUC_NE_lexicon_tags_2 },
    { 0x6fa489ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd78e89f3, SUC_NE_lexicon_tags_2 },
    { 0x5dba89f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c1689f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993089fc, SUC_NE_lexicon_tags_2 },
    { 0xcb9e89fc, SUC_NE_lexicon_tags_3 },
    { 0x492c89fd, SUC_NE_lexicon_tags_2 },
    { 0x269089fe, SUC_NE_lexicon_tags_24 },
    { 0x178a89fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3a88a02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bda8a04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aa8a06, SUC_NE_lexicon_tags_2 },
    { 0x68688a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae8e8a09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84ec8a0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d28a10, SUC_NE_lexicon_tags_2 },
    { 0xc48c8a11, SUC_NE_lexicon_tags_2 },
    { 0xe6368a11, SUC_NE_lexicon_tags_19 },
    { 0x7f2e8a13, SUC_NE_lexicon_tags_3 },
    { 0x41bc8a11, SUC_NE_lexicon_tags_2 },
    { 0x4ed28a15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c368a17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff828a19, SUC_NE_lexicon_tags_2 },
    { 0x6a5a8a1a, SUC_NE_lexicon_tags_4 },
    { 0x6d808a1b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa8a1e, SUC_NE_lexicon_tags_2 },
    { 0x689e8a1e, SUC_NE_lexicon_tags_2 },
    { 0xb7e28a20, SUC_NE_lexicon_tags_2 },
    { 0x4a68a21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe028a24, SUC_NE_lexicon_tags_2 },
    { 0x98c8a25, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x68e48a27, SUC_NE_lexicon_tags_2 },
    { 0xe2808a28, SUC_NE_lexicon_tags_3 },
    { 0x96f08a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9e48a2b, SUC_NE_lexicon_tags_2 },
    { 0x7abc8a2c, SUC_NE_lexicon_tags_2 },
    { 0xa9188a2d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e8a31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ac8a35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47068a37, SUC_NE_lexicon_tags_2 },
    { 0x354c8a38, SUC_NE_lexicon_tags_59 },
    { 0xc6228a37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3068a3d, SUC_NE_lexicon_tags_2 },
    { 0x8bf28a3e, SUC_NE_lexicon_tags_2 },
    { 0x9ada8a3f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04c8a44, SUC_NE_lexicon_tags_2 },
    { 0xad8e8a45, SUC_NE_lexicon_tags_2 },
    { 0x91f88a46, SUC_NE_lexicon_tags_11 },
    { 0xed0c8a46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a68a4d, SUC_NE_lexicon_tags_2 },
    { 0xfaa48a4e, SUC_NE_lexicon_tags_2 },
    { 0x84f48a4f, SUC_NE_lexicon_tags_2 },
    { 0xa0668a4e, SUC_NE_lexicon_tags_2 },
    { 0xdbd68a4f, SUC_NE_lexicon_tags_2 },
    { 0x14208a50, SUC_NE_lexicon_tags_2 },
    { 0xe6928a53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30648a56, SUC_NE_lexicon_tags_17 },
    { 0x50fa8a57, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3868a5a, SUC_NE_lexicon_tags_2 },
    { 0xcab88a5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde8a60, SUC_NE_lexicon_tags_2 },
    { 0xe4b88a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ce8a63, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2f328a65, SUC_NE_lexicon_tags_2 },
    { 0xb79a8a66, SUC_NE_lexicon_tags_2 },
    { 0xade88a67, SUC_NE_lexicon_tags_2 },
    { 0x79988a66, SUC_NE_lexicon_tags_2 },
    { 0x937e8a65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2e8a6c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa168a6f, SUC_NE_lexicon_tags_2 },
    { 0xf5a08a70, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d08a73, SUC_NE_lexicon_tags_2 },
    { 0xf7188a74, SUC_NE_lexicon_tags_2 },
    { 0x93a8a75, SUC_NE_lexicon_tags_1 },
    { 0xc788a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6668a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc7e8a7d, SUC_NE_lexicon_tags_2 },
    { 0x40cc8a7e, SUC_NE_lexicon_tags_4 },
    { 0x9b5e8a7e, SUC_NE_lexicon_tags_2 },
    { 0xb15a8a80, SUC_NE_lexicon_tags_2 },
    { 0xec748a81, SUC_NE_lexicon_tags_2 },
    { 0x5d268a82, SUC_NE_lexicon_tags_2 },
    { 0x9328a83, SUC_NE_lexicon_tags_2 },
    { 0x6a6e8a83, SUC_NE_lexicon_tags_2 },
    { 0x2648a81, SUC_NE_lexicon_tags_2 },
    { 0x906c8a86, SUC_NE_lexicon_tags_2 },
    { 0x86ca8a87, SUC_NE_lexicon_tags_2 },
    { 0x2a48a86, SUC_NE_lexicon_tags_7 },
    { 0x35788a89, SUC_NE_lexicon_tags_2 },
    { 0x64388a89, SUC_NE_lexicon_tags_2 },
    { 0xc6868a8b, SUC_NE_lexicon_tags_5 },
    { 0xd6408a89, SUC_NE_lexicon_tags_2 },
    { 0xd8cc8a8b, SUC_NE_lexicon_tags_2 },
    { 0xefc68a8e, SUC_NE_lexicon_tags_2 },
    { 0xcec68a8f, SUC_NE_lexicon_tags_2 },
    { 0xd9ba8a90, SUC_NE_lexicon_tags_2 },
    { 0x4a2c8a90, SUC_NE_lexicon_tags_2 },
    { 0x7ba88a90, SUC_NE_lexicon_tags_2 },
    { 0x29188a93, SUC_NE_lexicon_tags_2 },
    { 0xdcb28a94, SUC_NE_lexicon_tags_2 },
    { 0x88b08a93, SUC_NE_lexicon_tags_2 },
    { 0x83908a94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5028a98, SUC_NE_lexicon_tags_2 },
    { 0x624c8a99, SUC_NE_lexicon_tags_2 },
    { 0x1f6e8a9a, SUC_NE_lexicon_tags_2 },
    { 0x350c8a98, SUC_NE_lexicon_tags_2 },
    { 0xba468a9c, SUC_NE_lexicon_tags_30 },
    { 0x8d628a9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc87e8a9f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f008aa1, SUC_NE_lexicon_tags_2 },
    { 0xfd088aa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fc48aa4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b28aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfe8aaf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc2e48ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e308ab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e8aba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76e8ac4, SUC_NE_lexicon_tags_2 },
    { 0xda148ac5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb168ac8, SUC_NE_lexicon_tags_4 },
    { 0xc3f88ac9, SUC_NE_lexicon_tags_2 },
    { 0xab648ac8, SUC_NE_lexicon_tags_0 },
    { 0x84f48acb, SUC_NE_lexicon_tags_2 },
    { 0xb0ac8acb, SUC_NE_lexicon_tags_2 },
    { 0x44728acd, SUC_NE_lexicon_tags_2 },
    { 0x5cfa8ace, SUC_NE_lexicon_tags_2 },
    { 0xbfd68acc, SUC_NE_lexicon_tags_2 },
    { 0xe1808ad0, SUC_NE_lexicon_tags_2 },
    { 0x98628acc, SUC_NE_lexicon_tags_40 },
    { 0xacfc8acb, SUC_NE_lexicon_tags_2 },
    { 0x276c8acf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51548ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f28adc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7f08ade, SUC_NE_lexicon_tags_2 },
    { 0xd1488adf, SUC_NE_lexicon_tags_96 },
    { 0x6f908adf, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x121a8ae3, SUC_NE_lexicon_tags_2 },
    { 0x664c8ae4, SUC_NE_lexicon_tags_2 },
    { 0x9a648ae4, SUC_NE_lexicon_tags_4 },
    { 0x1fb48ae3, SUC_NE_lexicon_tags_57 },
    { 0x770e8ae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962c8aeb, SUC_NE_lexicon_tags_2 },
    { 0xe24a8aec, SUC_NE_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf228aef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0668af2, SUC_NE_lexicon_tags_2 },
    { 0xf6628af3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b288af8, SUC_NE_lexicon_tags_3 },
    { 0x639c8af9, SUC_NE_lexicon_tags_2 },
    { 0x9b8a8af9, SUC_NE_lexicon_tags_2 },
    { 0xd6388af9, SUC_NE_lexicon_tags_2 },
    { 0x82688afc, SUC_NE_lexicon_tags_2 },
    { 0xac968afb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdee8b02, SUC_NE_lexicon_tags_2 },
    { 0x59508b03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6d08b05, SUC_NE_lexicon_tags_2 },
    { 0xd6948b05, SUC_NE_lexicon_tags_2 },
    { 0x1f568b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a968b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x321c8b12, SUC_NE_lexicon_tags_383 },
    { 0x46488b13, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dbe8b15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79488b18, SUC_NE_lexicon_tags_2 },
    { 0xd1968b18, SUC_NE_lexicon_tags_3 },
    { 0xc49e8b1a, SUC_NE_lexicon_tags_2 },
    { 0xa51e8b19, SUC_NE_lexicon_tags_2 },
    { 0x10388b1c, SUC_NE_lexicon_tags_2 },
    { 0xf5ba8b1c, SUC_NE_lexicon_tags_2 },
    { 0xa0aa8b18, SUC_NE_lexicon_tags_2 },
    { 0x26ba8b1f, SUC_NE_lexicon_tags_2 },
    { 0xe3b28b20, SUC_NE_lexicon_tags_2 },
    { 0xc0c68b18, SUC_NE_lexicon_tags_2 },
    { 0x12dc8b22, SUC_NE_lexicon_tags_2 },
    { 0x2bb68b23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eae8b26, SUC_NE_lexicon_tags_2 },
    { 0x59ca8b27, SUC_NE_lexicon_tags_0 },
    { 0x41568b28, SUC_NE_lexicon_tags_2 },
    { 0xb9ea8b29, SUC_NE_lexicon_tags_2 },
    { 0x5c348b2a, SUC_NE_lexicon_tags_30 },
    { 0x8a748b2a, SUC_NE_lexicon_tags_2 },
    { 0xd5b08b26, SUC_NE_lexicon_tags_2 },
    { 0xcb9a8b2c, SUC_NE_lexicon_tags_8 },
    { 0x15888b2b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1e8b31, SUC_NE_lexicon_tags_8 },
    { 0x8ab28b31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff668b34, SUC_NE_lexicon_tags_2 },
    { 0xf2da8b34, SUC_NE_lexicon_tags_2 },
    { 0xda7a8b36, SUC_NE_lexicon_tags_2 },
    { 0x92648b37, SUC_NE_lexicon_tags_2 },
    { 0x45828b38, SUC_NE_lexicon_tags_2 },
    { 0x1cde8b39, SUC_NE_lexicon_tags_3 },
    { 0x5c8e8b3a, SUC_NE_lexicon_tags_4 },
    { 0xdb888b38, SUC_NE_lexicon_tags_2 },
    { 0x21f88b3c, SUC_NE_lexicon_tags_2 },
    { 0x19c68b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49488b41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f28b45, SUC_NE_lexicon_tags_17 },
    { 0x5dca8b46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2c8b4c, SUC_NE_lexicon_tags_2 },
    { 0xc8988b4c, SUC_NE_lexicon_tags_2 },
    { 0x12488b4e, SUC_NE_lexicon_tags_2 },
    { 0xaef68b4f, SUC_NE_lexicon_tags_2 },
    { 0xa8e48b50, SUC_NE_lexicon_tags_2 },
    { 0x6c2c8b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304c8b55, SUC_NE_lexicon_tags_2 },
    { 0x16408b55, SUC_NE_lexicon_tags_2 },
    { 0x500e8b57, SUC_NE_lexicon_tags_2 },
    { 0x5a5a8b56, SUC_NE_lexicon_tags_2 },
    { 0x93268b59, SUC_NE_lexicon_tags_2 },
    { 0xcffc8b59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x150c8b5c, SUC_NE_lexicon_tags_2 },
    { 0x4c268b5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ec8b63, SUC_NE_lexicon_tags_2 },
    { 0xa2788b64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9508b68, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ac8b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cc8b6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad08b72, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaea8b74, SUC_NE_lexicon_tags_19 },
    { 0xff08b75, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d848b7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c68b7c, SUC_NE_lexicon_tags_2 },
    { 0x56868b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1f68b7f, SUC_NE_lexicon_tags_2 },
    { 0x9a4a8b7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x275e8b82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c8b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f88b8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f208b8e, SUC_NE_lexicon_tags_2 },
    { 0xc2ea8b8f, SUC_NE_lexicon_tags_2 },
    { 0x617e8b90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dfa8b92, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e08b95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f5c8b97, SUC_NE_lexicon_tags_2 },
    { 0xaa08b98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe65e8b9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c08ba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3568ba9, SUC_NE_lexicon_tags_2 },
    { 0x37768ba9, SUC_NE_lexicon_tags_2 },
    { 0xb6dc8bab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52948bad, SUC_NE_lexicon_tags_2 },
    { 0x92f68bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6588bb0, SUC_NE_lexicon_tags_2 },
    { 0xa8428bb1, SUC_NE_lexicon_tags_2 },
    { 0x6de08bb0, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb468bb8, SUC_NE_lexicon_tags_2 },
    { 0x3a5c8bb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x444a8bbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc88bbd, SUC_NE_lexicon_tags_2 },
    { 0xbb608bbe, SUC_NE_lexicon_tags_2 },
    { 0xa4ca8bbf, SUC_NE_lexicon_tags_2 },
    { 0xbae28bc0, SUC_NE_lexicon_tags_19 },
    { 0xbd7e8bc1, SUC_NE_lexicon_tags_2 },
    { 0xba248bc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e8bc5, SUC_NE_lexicon_tags_2 },
    { 0x9f308bc6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x706c8bc8, SUC_NE_lexicon_tags_2 },
    { 0x68d08bc9, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab48bd0, SUC_NE_lexicon_tags_2 },
    { 0x80f88bd0, SUC_NE_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e448bd4, SUC_NE_lexicon_tags_2 },
    { 0xe7308bd5, SUC_NE_lexicon_tags_30 },
    { 0xeae88bd6, SUC_NE_lexicon_tags_2 },
    { 0x1a28bd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ba48bd9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dda8be0, SUC_NE_lexicon_tags_2 },
    { 0xcda08be1, SUC_NE_lexicon_tags_2 },
    { 0x81c08be2, SUC_NE_lexicon_tags_2 },
    { 0x237c8be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeae8be5, SUC_NE_lexicon_tags_2 },
    { 0x29e08be6, SUC_NE_lexicon_tags_2 },
    { 0xef668be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5c8bee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4228bf1, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916a8bfc, SUC_NE_lexicon_tags_2 },
    { 0xdc128bfd, SUC_NE_lexicon_tags_2 },
    { 0x30528bfd, SUC_NE_lexicon_tags_2 },
    { 0x225c8bff, SUC_NE_lexicon_tags_56 },
    { 0x1dd28c00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52968c04, SUC_NE_lexicon_tags_2 },
    { 0x83ee8c05, SUC_NE_lexicon_tags_2 },
    { 0xe35a8c06, SUC_NE_lexicon_tags_2 },
    { 0x38908c07, SUC_NE_lexicon_tags_2 },
    { 0xea8c8c08, SUC_NE_lexicon_tags_2 },
    { 0x5a5e8c09, SUC_NE_lexicon_tags_2 },
    { 0x8ab48c09, SUC_NE_lexicon_tags_2 },
    { 0x34648c0a, SUC_NE_lexicon_tags_453 },
    { 0, NULL },
    { 0x47648c0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c88c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x839c8c14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2508c1a, SUC_NE_lexicon_tags_2 },
    { 0xd94a8c1b, SUC_NE_lexicon_tags_2 },
    { 0x943e8c1c, SUC_NE_lexicon_tags_2 },
    { 0x1dd08c1d, SUC_NE_lexicon_tags_2 },
    { 0x24108c1d, SUC_NE_lexicon_tags_2 },
    { 0x575e8c1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x588c8c21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1dc8c23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79228c25, SUC_NE_lexicon_tags_2 },
    { 0x179e8c26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d668c28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32d08c2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdda8c2d, SUC_NE_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c8c30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc8c34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56468c37, SUC_NE_lexicon_tags_2 },
    { 0xf6388c37, SUC_NE_lexicon_tags_2 },
    { 0x14208c37, SUC_NE_lexicon_tags_2 },
    { 0xb6c68c3a, SUC_NE_lexicon_tags_2 },
    { 0x802e8c3b, SUC_NE_lexicon_tags_2 },
    { 0xa2a8c38, SUC_NE_lexicon_tags_2 },
    { 0x79ce8c38, SUC_NE_lexicon_tags_2 },
    { 0xcf2c8c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d68c47, SUC_NE_lexicon_tags_2 },
    { 0xa9b88c48, SUC_NE_lexicon_tags_2 },
    { 0xc9f08c49, SUC_NE_lexicon_tags_14 },
    { 0x501a8c4a, SUC_NE_lexicon_tags_2 },
    { 0xdeac8c4b, SUC_NE_lexicon_tags_2 },
    { 0x56028c4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746a8c52, SUC_NE_lexicon_tags_2 },
    { 0x7a7e8c52, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb7c8c55, SUC_NE_lexicon_tags_2 },
    { 0x8e5a8c56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4248c5a, SUC_NE_lexicon_tags_2 },
    { 0x1ce88c5b, SUC_NE_lexicon_tags_2 },
    { 0xe1e8c5c, SUC_NE_lexicon_tags_132 },
    { 0x20c8c5b, SUC_NE_lexicon_tags_2 },
    { 0x78fc8c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaf68c60, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6908c66, SUC_NE_lexicon_tags_4 },
    { 0x5d268c67, SUC_NE_lexicon_tags_2 },
    { 0xcac88c68, SUC_NE_lexicon_tags_2 },
    { 0x7c2e8c68, SUC_NE_lexicon_tags_2 },
    { 0x9d9c8c6a, SUC_NE_lexicon_tags_2 },
    { 0x12d28c6b, SUC_NE_lexicon_tags_2 },
    { 0x29de8c6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a868c75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20cc8c77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90948c7b, SUC_NE_lexicon_tags_2 },
    { 0xd8f08c7b, SUC_NE_lexicon_tags_4 },
    { 0xa8428c7d, SUC_NE_lexicon_tags_2 },
    { 0xeda68c7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27ea8c80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0e8c84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21b88c86, SUC_NE_lexicon_tags_2 },
    { 0x6b648c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7988c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedda8c8f, SUC_NE_lexicon_tags_3 },
    { 0xae948c8f, SUC_NE_lexicon_tags_2 },
    { 0x55068c91, SUC_NE_lexicon_tags_2 },
    { 0x7d228c92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679e8c97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520c8c9b, SUC_NE_lexicon_tags_2 },
    { 0xf5028c9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe528c9f, SUC_NE_lexicon_tags_2 },
    { 0xdaac8ca0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112e8caa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c68cad, SUC_NE_lexicon_tags_2 },
    { 0xf3a48cad, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xad768cb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x915c8cb2, SUC_NE_lexicon_tags_2 },
    { 0x66768cb3, SUC_NE_lexicon_tags_2 },
    { 0x21768cb4, SUC_NE_lexicon_tags_2 },
    { 0x8fca8cb5, SUC_NE_lexicon_tags_12 },
    { 0xec4a8cb6, SUC_NE_lexicon_tags_2 },
    { 0xf7588cb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e2a8cb9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc88cbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f08cc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62508cc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a68ccb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f688cd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb88cd5, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36348cdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b468ce0, SUC_NE_lexicon_tags_2 },
    { 0x5dec8ce1, SUC_NE_lexicon_tags_2 },
    { 0x384e8ce1, SUC_NE_lexicon_tags_2 },
    { 0x433c8ce2, SUC_NE_lexicon_tags_2 },
    { 0xc0e8ce3, SUC_NE_lexicon_tags_2 },
    { 0xabb28ce5, SUC_NE_lexicon_tags_2 },
    { 0xb59e8ce6, SUC_NE_lexicon_tags_2 },
    { 0xe1ca8ce7, SUC_NE_lexicon_tags_2 },
    { 0xbec28ce6, SUC_NE_lexicon_tags_2 },
    { 0xdae88ce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5208cec, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d348cf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f48cf6, SUC_NE_lexicon_tags_2 },
    { 0x1a048cf7, SUC_NE_lexicon_tags_36 },
    { 0x31b28cf8, SUC_NE_lexicon_tags_4 },
    { 0x19a48cf7, SUC_NE_lexicon_tags_4 },
    { 0x1cc08cf8, SUC_NE_lexicon_tags_2 },
    { 0x79d88cfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde848cfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a228cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7428d05, SUC_NE_lexicon_tags_2 },
    { 0x4d4a8d05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac468d0d, SUC_NE_lexicon_tags_2 },
    { 0x3b5e8d0e, SUC_NE_lexicon_tags_2 },
    { 0xf03e8d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7028d11, SUC_NE_lexicon_tags_1 },
    { 0x32668d12, SUC_NE_lexicon_tags_2 },
    { 0x9b848d13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ae8d16, SUC_NE_lexicon_tags_4 },
    { 0x94628d17, SUC_NE_lexicon_tags_2 },
    { 0xc2ba8d16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d188d1c, SUC_NE_lexicon_tags_2 },
    { 0xb3368d1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50568d25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a48d27, SUC_NE_lexicon_tags_2 },
    { 0x1bde8d28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed08d2c, SUC_NE_lexicon_tags_2 },
    { 0x56be8d2d, SUC_NE_lexicon_tags_406 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a8d32, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2e08d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49b08d36, SUC_NE_lexicon_tags_2 },
    { 0xf1268d37, SUC_NE_lexicon_tags_2 },
    { 0xe4388d36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb108d3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7c8d3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89668d40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40928d44, SUC_NE_lexicon_tags_2 },
    { 0x59e28d45, SUC_NE_lexicon_tags_2 },
    { 0x1c688d46, SUC_NE_lexicon_tags_2 },
    { 0x9928d45, SUC_NE_lexicon_tags_1 },
    { 0x21d68d48, SUC_NE_lexicon_tags_2 },
    { 0xa70e8d49, SUC_NE_lexicon_tags_2 },
    { 0x6e0c8d49, SUC_NE_lexicon_tags_2 },
    { 0xbe2a8d48, SUC_NE_lexicon_tags_2 },
    { 0x43f28d4c, SUC_NE_lexicon_tags_2 },
    { 0x11a08d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb008d50, SUC_NE_lexicon_tags_19 },
    { 0xe40e8d50, SUC_NE_lexicon_tags_17 },
    { 0x7748d52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fdc8d54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11c8d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5a8d5a, SUC_NE_lexicon_tags_0 },
    { 0x326c8d5a, SUC_NE_lexicon_tags_7 },
    { 0x2ce28d5c, SUC_NE_lexicon_tags_2 },
    { 0x97348d5d, SUC_NE_lexicon_tags_2 },
    { 0x66dc8d5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a3e8d60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df08d66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60568d68, SUC_NE_lexicon_tags_2 },
    { 0x57468d69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff08d6e, SUC_NE_lexicon_tags_2 },
    { 0xffaa8d6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73248d74, SUC_NE_lexicon_tags_2 },
    { 0xc53c8d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e68d7c, SUC_NE_lexicon_tags_2 },
    { 0xac968d7c, SUC_NE_lexicon_tags_2 },
    { 0x59788d7c, SUC_NE_lexicon_tags_2 },
    { 0x31de8d7f, SUC_NE_lexicon_tags_0 },
    { 0xb9cc8d80, SUC_NE_lexicon_tags_2 },
    { 0xa39a8d81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdae8d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51368d86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff28d89, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c28d8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b68d99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13e48d9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1e8d9f, SUC_NE_lexicon_tags_2 },
    { 0xfe388da0, SUC_NE_lexicon_tags_216 },
    { 0xf2fa8da0, SUC_NE_lexicon_tags_2 },
    { 0xbbd48da0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6248da6, SUC_NE_lexicon_tags_2 },
    { 0x94288da7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae9a8da9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d988dac, SUC_NE_lexicon_tags_3 },
    { 0xe86e8dad, SUC_NE_lexicon_tags_2 },
    { 0x12488dad, SUC_NE_lexicon_tags_2 },
    { 0x20788daf, SUC_NE_lexicon_tags_2 },
    { 0xfa228db0, SUC_NE_lexicon_tags_2 },
    { 0xc8d88dac, SUC_NE_lexicon_tags_2 },
    { 0x37e8dad, SUC_NE_lexicon_tags_2 },
    { 0x341e8db3, SUC_NE_lexicon_tags_2 },
    { 0x5de08db3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x373a8db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c48db9, SUC_NE_lexicon_tags_7 },
    { 0x2e988dba, SUC_NE_lexicon_tags_2 },
    { 0x71c88dbb, SUC_NE_lexicon_tags_2 },
    { 0x4c408dbc, SUC_NE_lexicon_tags_2 },
    { 0x7aaa8dbc, SUC_NE_lexicon_tags_2 },
    { 0x3af48dbe, SUC_NE_lexicon_tags_2 },
    { 0x732a8dbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ec88dc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x359c8dc4, SUC_NE_lexicon_tags_2 },
    { 0xc268dc5, SUC_NE_lexicon_tags_2 },
    { 0x99488dc6, SUC_NE_lexicon_tags_2 },
    { 0x4b6a8dc4, SUC_NE_lexicon_tags_17 },
    { 0x70448dc8, SUC_NE_lexicon_tags_2 },
    { 0x9c688dc8, SUC_NE_lexicon_tags_2 },
    { 0xf2048dca, SUC_NE_lexicon_tags_2 },
    { 0x96b48dcb, SUC_NE_lexicon_tags_2 },
    { 0xb4f28dc8, SUC_NE_lexicon_tags_2 },
    { 0xacfc8dcd, SUC_NE_lexicon_tags_2 },
    { 0xebba8dcd, SUC_NE_lexicon_tags_19 },
    { 0x93728dcf, SUC_NE_lexicon_tags_2 },
    { 0x343a8dc9, SUC_NE_lexicon_tags_2 },
    { 0x6fc08dc5, SUC_NE_lexicon_tags_2 },
    { 0xfe788dca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a8dd6, SUC_NE_lexicon_tags_2 },
    { 0x69948dd6, SUC_NE_lexicon_tags_2 },
    { 0x3db88dd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1e8ddb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd888de0, SUC_NE_lexicon_tags_2 },
    { 0xa58a8de1, SUC_NE_lexicon_tags_2 },
    { 0xb7588de0, SUC_NE_lexicon_tags_2 },
    { 0xb6348de3, SUC_NE_lexicon_tags_2 },
    { 0xd78c8de3, SUC_NE_lexicon_tags_17 },
    { 0x19e08de0, SUC_NE_lexicon_tags_2 },
    { 0x6de88de3, SUC_NE_lexicon_tags_2 },
    { 0x82c88de0, SUC_NE_lexicon_tags_2 },
    { 0xff488de8, SUC_NE_lexicon_tags_2 },
    { 0x4f728de8, SUC_NE_lexicon_tags_2 },
    { 0x27a88de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f848df0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fc8df4, SUC_NE_lexicon_tags_2 },
    { 0xff568df5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7a8df8, SUC_NE_lexicon_tags_2 },
    { 0xaed48df9, SUC_NE_lexicon_tags_1 },
    { 0x79f08dfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eec8dfe, SUC_NE_lexicon_tags_2 },
    { 0x55588dff, SUC_NE_lexicon_tags_2 },
    { 0x74648e00, SUC_NE_lexicon_tags_2 },
    { 0x72508dff, SUC_NE_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x88548e04, SUC_NE_lexicon_tags_2 },
    { 0x62188e05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8e8e09, SUC_NE_lexicon_tags_0 },
    { 0xdc488e09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd48e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac3e8e1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd548e1d, SUC_NE_lexicon_tags_2 },
    { 0x12e8e1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a588e20, SUC_NE_lexicon_tags_18 },
    { 0x29068e20, SUC_NE_lexicon_tags_2 },
    { 0xf1b88e20, SUC_NE_lexicon_tags_2 },
    { 0x346e8e23, SUC_NE_lexicon_tags_2 },
    { 0x5dc08e24, SUC_NE_lexicon_tags_19 },
    { 0xffd68e23, SUC_NE_lexicon_tags_17 },
    { 0x77468e25, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadee8e2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78a8e2e, SUC_NE_lexicon_tags_2 },
    { 0x592e8e2f, SUC_NE_lexicon_tags_59 },
    { 0x4eb08e2e, SUC_NE_lexicon_tags_2 },
    { 0x620e8e2f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7d448e33, SUC_NE_lexicon_tags_40 },
    { 0x85148e33, SUC_NE_lexicon_tags_25 },
    { 0x88528e34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd628e38, SUC_NE_lexicon_tags_2 },
    { 0xe4d28e39, SUC_NE_lexicon_tags_2 },
    { 0xccae8e38, SUC_NE_lexicon_tags_2 },
    { 0xe02c8e3b, SUC_NE_lexicon_tags_2 },
    { 0x56148e3c, SUC_NE_lexicon_tags_2 },
    { 0xa6f48e3d, SUC_NE_lexicon_tags_2 },
    { 0x655c8e3b, SUC_NE_lexicon_tags_2 },
    { 0xbb568e3c, SUC_NE_lexicon_tags_324 },
    { 0xaa2c8e3c, SUC_NE_lexicon_tags_2 },
    { 0x1d608e41, SUC_NE_lexicon_tags_2 },
    { 0x8b1e8e41, SUC_NE_lexicon_tags_2 },
    { 0x97e08e3b, SUC_NE_lexicon_tags_2 },
    { 0x7368e44, SUC_NE_lexicon_tags_3 },
    { 0x75bc8e45, SUC_NE_lexicon_tags_2 },
    { 0x8ebc8e44, SUC_NE_lexicon_tags_2 },
    { 0xa0208e47, SUC_NE_lexicon_tags_184 },
    { 0x5bbe8e48, SUC_NE_lexicon_tags_2 },
    { 0x25628e44, SUC_NE_lexicon_tags_2 },
    { 0x961c8e4a, SUC_NE_lexicon_tags_2 },
    { 0x9e4a8e41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde448e4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d528e53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24488e56, SUC_NE_lexicon_tags_2 },
    { 0x8bd08e57, SUC_NE_lexicon_tags_0 },
    { 0x78d48e58, SUC_NE_lexicon_tags_2 },
    { 0xb92a8e59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53408e5b, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xd5e8e5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x457e8e60, SUC_NE_lexicon_tags_7 },
    { 0xfe3e8e61, SUC_NE_lexicon_tags_2 },
    { 0x6fc48e60, SUC_NE_lexicon_tags_2 },
    { 0xe8568e61, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87248e67, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7108e6b, SUC_NE_lexicon_tags_2 },
    { 0xdc5c8e6c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc828e71, SUC_NE_lexicon_tags_2 },
    { 0xda2c8e72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff28e75, SUC_NE_lexicon_tags_2 },
    { 0x95c88e76, SUC_NE_lexicon_tags_2 },
    { 0x366e8e76, SUC_NE_lexicon_tags_2 },
    { 0xcaee8e78, SUC_NE_lexicon_tags_2 },
    { 0xc2d68e75, SUC_NE_lexicon_tags_2 },
    { 0x5f8e8e79, SUC_NE_lexicon_tags_2 },
    { 0xc81c8e76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d7e8e7d, SUC_NE_lexicon_tags_2 },
    { 0x6e4a8e7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4028e80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688e8e85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x890e8e87, SUC_NE_lexicon_tags_4 },
    { 0xa5928e87, SUC_NE_lexicon_tags_50 },
    { 0x24768e87, SUC_NE_lexicon_tags_2 },
    { 0xe5388e8a, SUC_NE_lexicon_tags_7 },
    { 0x4f388e89, SUC_NE_lexicon_tags_3 },
    { 0x901c8e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d08e92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e88e9a, SUC_NE_lexicon_tags_2 },
    { 0x3d788e9a, SUC_NE_lexicon_tags_2 },
    { 0x81408e9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x200a8e9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49228ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf68ea4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd228eaa, SUC_NE_lexicon_tags_2 },
    { 0x74788eab, SUC_NE_lexicon_tags_2 },
    { 0xc0c08eac, SUC_NE_lexicon_tags_2 },
    { 0x8d908ead, SUC_NE_lexicon_tags_2 },
    { 0x9b928eac, SUC_NE_lexicon_tags_2 },
    { 0xf2b48eaa, SUC_NE_lexicon_tags_2 },
    { 0x7db88eb0, SUC_NE_lexicon_tags_2 },
    { 0xb8088eb1, SUC_NE_lexicon_tags_2 },
    { 0x994e8eb2, SUC_NE_lexicon_tags_2 },
    { 0x34be8ead, SUC_NE_lexicon_tags_2 },
    { 0xcc528eaf, SUC_NE_lexicon_tags_2 },
    { 0x9b308eb5, SUC_NE_lexicon_tags_2 },
    { 0xca848eb5, SUC_NE_lexicon_tags_2 },
    { 0xcade8eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cc8eba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef528ebd, SUC_NE_lexicon_tags_2 },
    { 0xe8988ebe, SUC_NE_lexicon_tags_2 },
    { 0x89208ebf, SUC_NE_lexicon_tags_2 },
    { 0x2a7a8ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33868ec3, SUC_NE_lexicon_tags_2 },
    { 0xbb988ec4, SUC_NE_lexicon_tags_2 },
    { 0xcf6a8ec5, SUC_NE_lexicon_tags_2 },
    { 0x64708ec6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9da8ec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a28ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbb68ecd, SUC_NE_lexicon_tags_2 },
    { 0xdf0e8ece, SUC_NE_lexicon_tags_2 },
    { 0xea8a8ecf, SUC_NE_lexicon_tags_2 },
    { 0xb7ae8ed0, SUC_NE_lexicon_tags_2 },
    { 0xfc48ed1, SUC_NE_lexicon_tags_50 },
    { 0x71ba8ed2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e928ed4, SUC_NE_lexicon_tags_2 },
    { 0xf49a8ed5, SUC_NE_lexicon_tags_2 },
    { 0xf98e8ed6, SUC_NE_lexicon_tags_2 },
    { 0xb968ed7, SUC_NE_lexicon_tags_2 },
    { 0x2248ed7, SUC_NE_lexicon_tags_2 },
    { 0xfa6e8ed9, SUC_NE_lexicon_tags_2 },
    { 0x6bbc8ed5, SUC_NE_lexicon_tags_2 },
    { 0xde808ed5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29e8ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e8ee3, SUC_NE_lexicon_tags_2 },
    { 0x5e408ee4, SUC_NE_lexicon_tags_2 },
    { 0xb0288ee5, SUC_NE_lexicon_tags_2 },
    { 0x16948ee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48be8eed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ec28eef, SUC_NE_lexicon_tags_2 },
    { 0x7d288ef0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49ca8ef2, SUC_NE_lexicon_tags_0 },
    { 0x7e708ef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e88efa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221c8efe, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa7288f00, SUC_NE_lexicon_tags_3 },
    { 0x7d408f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7028f05, SUC_NE_lexicon_tags_2 },
    { 0x2dd08f05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94e68f08, SUC_NE_lexicon_tags_34 },
    { 0x5f028f09, SUC_NE_lexicon_tags_2 },
    { 0x8ba88f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0fc8f0c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55468f10, SUC_NE_lexicon_tags_2 },
    { 0xee828f10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4c8f16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x728c8f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15228f1f, SUC_NE_lexicon_tags_24 },
    { 0x378c8f20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be68f2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35468f31, SUC_NE_lexicon_tags_2 },
    { 0x59a48f32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x629c8f35, SUC_NE_lexicon_tags_2 },
    { 0x85408f36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62e68f38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a08f3e, SUC_NE_lexicon_tags_2 },
    { 0x22da8f3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259e8f45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71a68f47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e6a8f49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8908f4c, SUC_NE_lexicon_tags_18 },
    { 0x7e028f4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e28f50, SUC_NE_lexicon_tags_2 },
    { 0xf3c48f51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e8f55, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4d6e8f57, SUC_NE_lexicon_tags_2 },
    { 0xab228f58, SUC_NE_lexicon_tags_2 },
    { 0x72d88f59, SUC_NE_lexicon_tags_2 },
    { 0x3a928f5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9708f5c, SUC_NE_lexicon_tags_2 },
    { 0x578a8f5d, SUC_NE_lexicon_tags_2 },
    { 0xa2c88f5e, SUC_NE_lexicon_tags_2 },
    { 0xd2c88f5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a248f62, SUC_NE_lexicon_tags_2 },
    { 0xbaa48f62, SUC_NE_lexicon_tags_2 },
    { 0x58fa8f64, SUC_NE_lexicon_tags_2 },
    { 0x4708f65, SUC_NE_lexicon_tags_2 },
    { 0x666a8f66, SUC_NE_lexicon_tags_2 },
    { 0x841e8f64, SUC_NE_lexicon_tags_2 },
    { 0x46fc8f67, SUC_NE_lexicon_tags_2 },
    { 0x5b328f64, SUC_NE_lexicon_tags_2 },
    { 0x9d908f6a, SUC_NE_lexicon_tags_2 },
    { 0x39b08f6a, SUC_NE_lexicon_tags_2 },
    { 0x24d88f6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78688f6f, SUC_NE_lexicon_tags_2 },
    { 0xfb428f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f48f77, SUC_NE_lexicon_tags_2 },
    { 0xe75a8f78, SUC_NE_lexicon_tags_2 },
    { 0x59da8f79, SUC_NE_lexicon_tags_2 },
    { 0xc9c08f79, SUC_NE_lexicon_tags_2 },
    { 0x20388f7b, SUC_NE_lexicon_tags_2 },
    { 0x82628f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b388f81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e88f89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecde8f8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb88f8e, SUC_NE_lexicon_tags_2 },
    { 0x3b508f8f, SUC_NE_lexicon_tags_2 },
    { 0x10e48f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1da8f92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2108f95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf248f97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x996e8f99, SUC_NE_lexicon_tags_2 },
    { 0x11dc8f9a, SUC_NE_lexicon_tags_2 },
    { 0xf8628f9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd08fa0, SUC_NE_lexicon_tags_2 },
    { 0x50208fa1, SUC_NE_lexicon_tags_3 },
    { 0xfeb28fa0, SUC_NE_lexicon_tags_2 },
    { 0x38368fa0, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x98348fa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4a8fac, SUC_NE_lexicon_tags_5 },
    { 0x1c748fad, SUC_NE_lexicon_tags_2 },
    { 0xadae8fae, SUC_NE_lexicon_tags_3 },
    { 0x3bd48fad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfce08fb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd48fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59c28fb6, SUC_NE_lexicon_tags_2 },
    { 0x83428fb7, SUC_NE_lexicon_tags_17 },
    { 0x7a6e8fb8, SUC_NE_lexicon_tags_2 },
    { 0x1e488fb9, SUC_NE_lexicon_tags_2 },
    { 0xfe088fba, SUC_NE_lexicon_tags_2 },
    { 0x88848fb6, SUC_NE_lexicon_tags_2 },
    { 0x834a8fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43608fbe, SUC_NE_lexicon_tags_2 },
    { 0xaca88fbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1288fc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc12e8fc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85a8fc8, SUC_NE_lexicon_tags_2 },
    { 0xd6948fc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6288fcc, SUC_NE_lexicon_tags_2 },
    { 0xe9e28fcd, SUC_NE_lexicon_tags_2 },
    { 0x29c28fce, SUC_NE_lexicon_tags_2 },
    { 0x7efe8fcf, SUC_NE_lexicon_tags_36 },
    { 0xa48c8fcf, SUC_NE_lexicon_tags_2 },
    { 0x30a8fd1, SUC_NE_lexicon_tags_2 },
    { 0x488fd2, SUC_NE_lexicon_tags_2 },
    { 0x3b328fcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b28fd9, SUC_NE_lexicon_tags_96 },
    { 0xcbc8fda, SUC_NE_lexicon_tags_2 },
    { 0x35968fdb, SUC_NE_lexicon_tags_2 },
    { 0x33528fdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c148fe4, SUC_NE_lexicon_tags_2 },
    { 0x30a08fe4, SUC_NE_lexicon_tags_2 },
    { 0x5b948fe6, SUC_NE_lexicon_tags_2 },
    { 0xa21e8fe7, SUC_NE_lexicon_tags_0 },
    { 0xedb08fe6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b988fed, SUC_NE_lexicon_tags_2 },
    { 0x41c8fed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x505c8ff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3b88ff2, SUC_NE_lexicon_tags_2 },
    { 0x32fc8ff3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e8ff8, SUC_NE_lexicon_tags_2 },
    { 0x7ee48ff9, SUC_NE_lexicon_tags_4 },
    { 0xd5d68ff8, SUC_NE_lexicon_tags_2 },
    { 0xb1088ffb, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x34d08ffd, SUC_NE_lexicon_tags_2 },
    { 0x476c8ffd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80689001, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a8900a, SUC_NE_lexicon_tags_2 },
    { 0xd7f8900b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04c9010, SUC_NE_lexicon_tags_2 },
    { 0x14429010, SUC_NE_lexicon_tags_2 },
    { 0x4c989012, SUC_NE_lexicon_tags_2 },
    { 0x67109013, SUC_NE_lexicon_tags_2 },
    { 0x2ef29014, SUC_NE_lexicon_tags_2 },
    { 0x58529015, SUC_NE_lexicon_tags_2 },
    { 0xe8389014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47829018, SUC_NE_lexicon_tags_2 },
    { 0xa2c09019, SUC_NE_lexicon_tags_2 },
    { 0x22e8901a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9128901c, SUC_NE_lexicon_tags_2 },
    { 0x77da901c, SUC_NE_lexicon_tags_2 },
    { 0xf972901c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a9022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c4902a, SUC_NE_lexicon_tags_2 },
    { 0xed1e902b, SUC_NE_lexicon_tags_2 },
    { 0x34c8902b, SUC_NE_lexicon_tags_2 },
    { 0x92ae902d, SUC_NE_lexicon_tags_2 },
    { 0x8926902e, SUC_NE_lexicon_tags_2 },
    { 0x6476902c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32169034, SUC_NE_lexicon_tags_2 },
    { 0x45a89035, SUC_NE_lexicon_tags_2 },
    { 0xb5889036, SUC_NE_lexicon_tags_2 },
    { 0x8b949035, SUC_NE_lexicon_tags_2 },
    { 0x415e9037, SUC_NE_lexicon_tags_2 },
    { 0x4b49039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b4903e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc3e9040, SUC_NE_lexicon_tags_2 },
    { 0x2c249041, SUC_NE_lexicon_tags_2 },
    { 0x283a9042, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d609048, SUC_NE_lexicon_tags_2 },
    { 0x31609049, SUC_NE_lexicon_tags_2 },
    { 0x16049049, SUC_NE_lexicon_tags_2 },
    { 0x86089049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cc904e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27069050, SUC_NE_lexicon_tags_2 },
    { 0x10ac9051, SUC_NE_lexicon_tags_2 },
    { 0xe5ea9052, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87069058, SUC_NE_lexicon_tags_2 },
    { 0xf2fe9059, SUC_NE_lexicon_tags_2 },
    { 0xcbd49058, SUC_NE_lexicon_tags_18 },
    { 0xffca905b, SUC_NE_lexicon_tags_32 },
    { 0x1880905c, SUC_NE_lexicon_tags_2 },
    { 0x826a905b, SUC_NE_lexicon_tags_2 },
    { 0xc362905e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e29061, SUC_NE_lexicon_tags_2 },
    { 0x74629061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89209065, SUC_NE_lexicon_tags_114 },
    { 0xac129066, SUC_NE_lexicon_tags_17 },
    { 0xa66e9066, SUC_NE_lexicon_tags_2 },
    { 0xaa09066, SUC_NE_lexicon_tags_2 },
    { 0xde709066, SUC_NE_lexicon_tags_2 },
    { 0x7ffc906a, SUC_NE_lexicon_tags_2 },
    { 0xbe0c906a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5692906e, SUC_NE_lexicon_tags_2 },
    { 0x6d96906f, SUC_NE_lexicon_tags_32 },
    { 0x802a9070, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ba9077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6054907b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x544a907e, SUC_NE_lexicon_tags_2 },
    { 0xcb3c907e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fc9082, SUC_NE_lexicon_tags_40 },
    { 0x3ecc9083, SUC_NE_lexicon_tags_32 },
    { 0x56229084, SUC_NE_lexicon_tags_5 },
    { 0x2f669085, SUC_NE_lexicon_tags_2 },
    { 0xfdea9086, SUC_NE_lexicon_tags_2 },
    { 0x3d69084, SUC_NE_lexicon_tags_2 },
    { 0x7c2a9088, SUC_NE_lexicon_tags_19 },
    { 0x93329086, SUC_NE_lexicon_tags_8 },
    { 0xea9085, SUC_NE_lexicon_tags_2 },
    { 0xd96c9083, SUC_NE_lexicon_tags_2 },
    { 0x634908c, SUC_NE_lexicon_tags_2 },
    { 0xc098908d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98889096, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7149099, SUC_NE_lexicon_tags_5 },
    { 0x5d40909a, SUC_NE_lexicon_tags_4 },
    { 0xbc22909b, SUC_NE_lexicon_tags_2 },
    { 0xf2a6909c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7490a3, SUC_NE_lexicon_tags_2 },
    { 0x927290a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15fc90a6, SUC_NE_lexicon_tags_2 },
    { 0x49e890a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31da90a9, SUC_NE_lexicon_tags_2 },
    { 0x992090aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db290ae, SUC_NE_lexicon_tags_2 },
    { 0x2f2090af, SUC_NE_lexicon_tags_2 },
    { 0xc6f090b0, SUC_NE_lexicon_tags_7 },
    { 0x899290b0, SUC_NE_lexicon_tags_2 },
    { 0x86c690ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcba90b7, SUC_NE_lexicon_tags_2 },
    { 0xbb0e90b8, SUC_NE_lexicon_tags_3 },
    { 0x5a2a90b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1090bc, SUC_NE_lexicon_tags_17 },
    { 0x5d890bc, SUC_NE_lexicon_tags_7 },
    { 0xa96690be, SUC_NE_lexicon_tags_2 },
    { 0xf7f890bf, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4e90c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1c90c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25090c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328490ce, SUC_NE_lexicon_tags_2 },
    { 0x451690cf, SUC_NE_lexicon_tags_2 },
    { 0x997890d0, SUC_NE_lexicon_tags_12 },
    { 0x6d6c90ce, SUC_NE_lexicon_tags_2 },
    { 0x2ace90cf, SUC_NE_lexicon_tags_2 },
    { 0x2ec890d3, SUC_NE_lexicon_tags_2 },
    { 0x113490d3, SUC_NE_lexicon_tags_1 },
    { 0x835890d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa87090d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5ec90d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282e90dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ac090df, SUC_NE_lexicon_tags_2 },
    { 0xf37090df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d890e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7890e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6de90e9, SUC_NE_lexicon_tags_2 },
    { 0xb65490ea, SUC_NE_lexicon_tags_81 },
    { 0xe0e490ea, SUC_NE_lexicon_tags_2 },
    { 0xa78090ec, SUC_NE_lexicon_tags_2 },
    { 0x157a90ea, SUC_NE_lexicon_tags_2 },
    { 0xfc9290ee, SUC_NE_lexicon_tags_2 },
    { 0xa62a90ee, SUC_NE_lexicon_tags_2 },
    { 0x8ebc90f0, SUC_NE_lexicon_tags_2 },
    { 0xb84e90e9, SUC_NE_lexicon_tags_2 },
    { 0xc4ae90f2, SUC_NE_lexicon_tags_2 },
    { 0xc41290f3, SUC_NE_lexicon_tags_2 },
    { 0x688e90f3, SUC_NE_lexicon_tags_2 },
    { 0xacd890f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37290fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83d490fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x662490ff, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966a9103, SUC_NE_lexicon_tags_2 },
    { 0xb8409104, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ba9107, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9de9109, SUC_NE_lexicon_tags_2 },
    { 0x3fce910a, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cf2910c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea910f, SUC_NE_lexicon_tags_2 },
    { 0x75609110, SUC_NE_lexicon_tags_2 },
    { 0x8cba9111, SUC_NE_lexicon_tags_40 },
    { 0xb916910f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31449119, SUC_NE_lexicon_tags_2 },
    { 0xf7a69119, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa672911c, SUC_NE_lexicon_tags_19 },
    { 0xa78e911d, SUC_NE_lexicon_tags_2 },
    { 0xd066911c, SUC_NE_lexicon_tags_2 },
    { 0x8902911f, SUC_NE_lexicon_tags_2 },
    { 0x7d4911c, SUC_NE_lexicon_tags_2 },
    { 0xa1a9121, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2c49123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbc69125, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59369127, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef04912b, SUC_NE_lexicon_tags_2 },
    { 0x9b9e912c, SUC_NE_lexicon_tags_2 },
    { 0xeb62912c, SUC_NE_lexicon_tags_2 },
    { 0xfbd8912c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60349130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d589132, SUC_NE_lexicon_tags_2 },
    { 0xc7289132, SUC_NE_lexicon_tags_2 },
    { 0xd5ee9134, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac9137, SUC_NE_lexicon_tags_2 },
    { 0xd1c29138, SUC_NE_lexicon_tags_2 },
    { 0x3fda9138, SUC_NE_lexicon_tags_1 },
    { 0x61b49139, SUC_NE_lexicon_tags_2 },
    { 0xd8229138, SUC_NE_lexicon_tags_24 },
    { 0xa3309138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95f8913e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43749140, SUC_NE_lexicon_tags_2 },
    { 0x7bb49141, SUC_NE_lexicon_tags_2 },
    { 0x1d69142, SUC_NE_lexicon_tags_2 },
    { 0xa87a9140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32549147, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903c914d, SUC_NE_lexicon_tags_2 },
    { 0x57fc914e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50c9155, SUC_NE_lexicon_tags_2 },
    { 0x48329155, SUC_NE_lexicon_tags_7 },
    { 0x8d409156, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65f09159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb46915b, SUC_NE_lexicon_tags_2 },
    { 0x9d2a915c, SUC_NE_lexicon_tags_17 },
    { 0x6614915b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c29163, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf6e9165, SUC_NE_lexicon_tags_7 },
    { 0xb5069166, SUC_NE_lexicon_tags_2 },
    { 0x8c6e9166, SUC_NE_lexicon_tags_2 },
    { 0x4c969168, SUC_NE_lexicon_tags_7 },
    { 0x903e9168, SUC_NE_lexicon_tags_2 },
    { 0xf1de916a, SUC_NE_lexicon_tags_2 },
    { 0x34c89165, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x532e916d, SUC_NE_lexicon_tags_2 },
    { 0x2110916e, SUC_NE_lexicon_tags_2 },
    { 0x7932916f, SUC_NE_lexicon_tags_2 },
    { 0x6bf4916d, SUC_NE_lexicon_tags_2 },
    { 0xe73c9171, SUC_NE_lexicon_tags_2 },
    { 0xb534916f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e7a9174, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1df29176, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc549178, SUC_NE_lexicon_tags_2 },
    { 0xe6b89178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee917d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9e9181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c49185, SUC_NE_lexicon_tags_87 },
    { 0x407c9186, SUC_NE_lexicon_tags_4 },
    { 0xd02a9187, SUC_NE_lexicon_tags_2 },
    { 0x43189187, SUC_NE_lexicon_tags_2 },
    { 0x52589189, SUC_NE_lexicon_tags_2 },
    { 0xb18c918a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a6918c, SUC_NE_lexicon_tags_2 },
    { 0x228e918c, SUC_NE_lexicon_tags_40 },
    { 0xe400918c, SUC_NE_lexicon_tags_2 },
    { 0x967e918e, SUC_NE_lexicon_tags_2 },
    { 0x7352918e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78089192, SUC_NE_lexicon_tags_2 },
    { 0xec6e9192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65309197, SUC_NE_lexicon_tags_2 },
    { 0xc1a69198, SUC_NE_lexicon_tags_2 },
    { 0xb10e9198, SUC_NE_lexicon_tags_2 },
    { 0x5eb69199, SUC_NE_lexicon_tags_2 },
    { 0x95cc919b, SUC_NE_lexicon_tags_2 },
    { 0x5c92919b, SUC_NE_lexicon_tags_28 },
    { 0x632e919b, SUC_NE_lexicon_tags_24 },
    { 0xf420919e, SUC_NE_lexicon_tags_1 },
    { 0x9b02919e, SUC_NE_lexicon_tags_0 },
    { 0x658091a0, SUC_NE_lexicon_tags_2 },
    { 0xa18e919e, SUC_NE_lexicon_tags_2 },
    { 0x797091a2, SUC_NE_lexicon_tags_2 },
    { 0x78891a2, SUC_NE_lexicon_tags_14 },
    { 0x3d6491a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d491a9, SUC_NE_lexicon_tags_2 },
    { 0xd46291aa, SUC_NE_lexicon_tags_7 },
    { 0x97c091ab, SUC_NE_lexicon_tags_2 },
    { 0xdc4491ac, SUC_NE_lexicon_tags_2 },
    { 0x32c891ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82491b0, SUC_NE_lexicon_tags_2 },
    { 0x1ce091b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55691b4, SUC_NE_lexicon_tags_2 },
    { 0xcf7291b5, SUC_NE_lexicon_tags_4 },
    { 0x645891b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a6291b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd77a91ba, SUC_NE_lexicon_tags_2 },
    { 0xed2891ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f091bd, SUC_NE_lexicon_tags_2 },
    { 0xd3c091be, SUC_NE_lexicon_tags_2 },
    { 0xee4c91be, SUC_NE_lexicon_tags_2 },
    { 0x41c91c0, SUC_NE_lexicon_tags_4 },
    { 0xecde91c0, SUC_NE_lexicon_tags_2 },
    { 0xcb291c2, SUC_NE_lexicon_tags_2 },
    { 0x83bc91c3, SUC_NE_lexicon_tags_2 },
    { 0x9c5e91c1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5cb891c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab491cc, SUC_NE_lexicon_tags_2 },
    { 0xc4da91cc, SUC_NE_lexicon_tags_2 },
    { 0xbb5291cc, SUC_NE_lexicon_tags_2 },
    { 0xcdec91cc, SUC_NE_lexicon_tags_4 },
    { 0xa78c91cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5491d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2891de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c91e3, SUC_NE_lexicon_tags_2 },
    { 0xee7a91e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x359a91e6, SUC_NE_lexicon_tags_2 },
    { 0x691e6, SUC_NE_lexicon_tags_7 },
    { 0x1fa491e8, SUC_NE_lexicon_tags_2 },
    { 0x8cc491e9, SUC_NE_lexicon_tags_2 },
    { 0xa1be91ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x903691ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc3291ee, SUC_NE_lexicon_tags_4 },
    { 0xc7a491ef, SUC_NE_lexicon_tags_0 },
    { 0x97b091ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aa91f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28091fa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f291fe, SUC_NE_lexicon_tags_5 },
    { 0x349891ff, SUC_NE_lexicon_tags_2 },
    { 0x7629200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb49203, SUC_NE_lexicon_tags_2 },
    { 0x5f309204, SUC_NE_lexicon_tags_2 },
    { 0xc8429204, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0289208, SUC_NE_lexicon_tags_2 },
    { 0x21709209, SUC_NE_lexicon_tags_2 },
    { 0xcd0920a, SUC_NE_lexicon_tags_2 },
    { 0x74dc920b, SUC_NE_lexicon_tags_2 },
    { 0xbcac920c, SUC_NE_lexicon_tags_2 },
    { 0xcec920c, SUC_NE_lexicon_tags_3 },
    { 0xc78e920e, SUC_NE_lexicon_tags_2 },
    { 0x24a4920f, SUC_NE_lexicon_tags_92 },
    { 0x3b10920f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12149212, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xeffc9214, SUC_NE_lexicon_tags_3 },
    { 0x9b889215, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f789219, SUC_NE_lexicon_tags_2 },
    { 0x521a9219, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffa921d, SUC_NE_lexicon_tags_2 },
    { 0x143c921d, SUC_NE_lexicon_tags_2 },
    { 0x2e6e921d, SUC_NE_lexicon_tags_2 },
    { 0xc2529220, SUC_NE_lexicon_tags_2 },
    { 0x77969221, SUC_NE_lexicon_tags_2 },
    { 0x7b5e9222, SUC_NE_lexicon_tags_2 },
    { 0xe2569223, SUC_NE_lexicon_tags_2 },
    { 0x2c209221, SUC_NE_lexicon_tags_18 },
    { 0x16a2921e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8469227, SUC_NE_lexicon_tags_2 },
    { 0x75c69227, SUC_NE_lexicon_tags_3 },
    { 0x734e9229, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19e8922b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cc9230, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25369233, SUC_NE_lexicon_tags_2 },
    { 0xef8a9233, SUC_NE_lexicon_tags_24 },
    { 0x20f89234, SUC_NE_lexicon_tags_2 },
    { 0x95229236, SUC_NE_lexicon_tags_3 },
    { 0xe59c9234, SUC_NE_lexicon_tags_7 },
    { 0x2c489238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd244923e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x758e9241, SUC_NE_lexicon_tags_2 },
    { 0x44c9242, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x996a9244, SUC_NE_lexicon_tags_7 },
    { 0xfa29244, SUC_NE_lexicon_tags_2 },
    { 0x8589246, SUC_NE_lexicon_tags_2 },
    { 0x11c49247, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6569249, SUC_NE_lexicon_tags_2 },
    { 0x14a0924a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcace924d, SUC_NE_lexicon_tags_2 },
    { 0x9256924d, SUC_NE_lexicon_tags_2 },
    { 0x2462924f, SUC_NE_lexicon_tags_2 },
    { 0x1162924f, SUC_NE_lexicon_tags_39 },
    { 0x4f56924e, SUC_NE_lexicon_tags_4 },
    { 0xb8049252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa49259, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c925e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c069261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e449265, SUC_NE_lexicon_tags_2 },
    { 0xd5f09265, SUC_NE_lexicon_tags_34 },
    { 0x273e9267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca3e9269, SUC_NE_lexicon_tags_2 },
    { 0x2b66926a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83e926e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a29273, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcc9278, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bc927d, SUC_NE_lexicon_tags_2 },
    { 0xfe1e927e, SUC_NE_lexicon_tags_2 },
    { 0x2f4a927d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98349287, SUC_NE_lexicon_tags_2 },
    { 0xda8c9288, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559e928d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80849291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11829295, SUC_NE_lexicon_tags_2 },
    { 0x942e9296, SUC_NE_lexicon_tags_2 },
    { 0xbde9297, SUC_NE_lexicon_tags_2 },
    { 0xfa389295, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da929e, SUC_NE_lexicon_tags_2 },
    { 0xb958929f, SUC_NE_lexicon_tags_3 },
    { 0x360892a0, SUC_NE_lexicon_tags_2 },
    { 0x68e292a0, SUC_NE_lexicon_tags_2 },
    { 0x5a1092a1, SUC_NE_lexicon_tags_3 },
    { 0x70e92a2, SUC_NE_lexicon_tags_2 },
    { 0xbfca92a2, SUC_NE_lexicon_tags_2 },
    { 0x613892a1, SUC_NE_lexicon_tags_2 },
    { 0x9ece92a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1e92ab, SUC_NE_lexicon_tags_2 },
    { 0x28f292ac, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d292af, SUC_NE_lexicon_tags_2 },
    { 0x3b2a92af, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d892b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1ca92b5, SUC_NE_lexicon_tags_2 },
    { 0xd03e92b6, SUC_NE_lexicon_tags_2 },
    { 0x6bbe92b7, SUC_NE_lexicon_tags_2 },
    { 0xb03c92b6, SUC_NE_lexicon_tags_1 },
    { 0xd4c092b9, SUC_NE_lexicon_tags_2 },
    { 0x6b3292b9, SUC_NE_lexicon_tags_2 },
    { 0x225692b6, SUC_NE_lexicon_tags_2 },
    { 0xad2e92b6, SUC_NE_lexicon_tags_7 },
    { 0x447292bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc92c0, SUC_NE_lexicon_tags_0 },
    { 0x613892c0, SUC_NE_lexicon_tags_2 },
    { 0x904a92c2, SUC_NE_lexicon_tags_2 },
    { 0x6b5892c3, SUC_NE_lexicon_tags_17 },
    { 0x5ba292c0, SUC_NE_lexicon_tags_7 },
    { 0xc9ca92c5, SUC_NE_lexicon_tags_13 },
    { 0x9ef492c6, SUC_NE_lexicon_tags_2 },
    { 0xafe292c7, SUC_NE_lexicon_tags_2 },
    { 0x218092c6, SUC_NE_lexicon_tags_2 },
    { 0x8d292c9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x67a292cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58d892cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d492d1, SUC_NE_lexicon_tags_2 },
    { 0x2a2892d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab292d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8a92da, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7892e2, SUC_NE_lexicon_tags_2 },
    { 0xd6492e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961c92e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc54e92eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c092f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc47892f7, SUC_NE_lexicon_tags_1 },
    { 0x651e92f7, SUC_NE_lexicon_tags_2 },
    { 0x29d692f7, SUC_NE_lexicon_tags_4 },
    { 0x419e92f9, SUC_NE_lexicon_tags_2 },
    { 0x9bfe92fb, SUC_NE_lexicon_tags_2 },
    { 0xe59492fc, SUC_NE_lexicon_tags_2 },
    { 0x29c892fd, SUC_NE_lexicon_tags_4 },
    { 0x6f0292fd, SUC_NE_lexicon_tags_3 },
    { 0x1bf292ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41309304, SUC_NE_lexicon_tags_2 },
    { 0x56ee9305, SUC_NE_lexicon_tags_2 },
    { 0x103c9304, SUC_NE_lexicon_tags_2 },
    { 0x84309304, SUC_NE_lexicon_tags_2 },
    { 0xec969308, SUC_NE_lexicon_tags_2 },
    { 0x687a9308, SUC_NE_lexicon_tags_2 },
    { 0xb676930a, SUC_NE_lexicon_tags_2 },
    { 0x6c04930b, SUC_NE_lexicon_tags_0 },
    { 0x73e0930a, SUC_NE_lexicon_tags_2 },
    { 0x655e9309, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a9311, SUC_NE_lexicon_tags_4 },
    { 0x814e9312, SUC_NE_lexicon_tags_2 },
    { 0xd8449312, SUC_NE_lexicon_tags_2 },
    { 0xcf589314, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a9317, SUC_NE_lexicon_tags_2 },
    { 0x77cc9317, SUC_NE_lexicon_tags_2 },
    { 0x22549318, SUC_NE_lexicon_tags_2 },
    { 0xfbea9319, SUC_NE_lexicon_tags_3 },
    { 0x1c8a9319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec29320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87c09322, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b89325, SUC_NE_lexicon_tags_2 },
    { 0xdf309326, SUC_NE_lexicon_tags_2 },
    { 0x24669326, SUC_NE_lexicon_tags_2 },
    { 0xeac49328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x599c932a, SUC_NE_lexicon_tags_4 },
    { 0x919c932a, SUC_NE_lexicon_tags_2 },
    { 0xb9b0932b, SUC_NE_lexicon_tags_2 },
    { 0x3a4a932d, SUC_NE_lexicon_tags_2 },
    { 0x3d3a932e, SUC_NE_lexicon_tags_2 },
    { 0xd86a932e, SUC_NE_lexicon_tags_1 },
    { 0xbd5c9330, SUC_NE_lexicon_tags_2 },
    { 0xd2fe9331, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0x27909333, SUC_NE_lexicon_tags_2 },
    { 0xf42e9334, SUC_NE_lexicon_tags_2 },
    { 0xb5509335, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98933a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff26933d, SUC_NE_lexicon_tags_2 },
    { 0x10d0933d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59549340, SUC_NE_lexicon_tags_2 },
    { 0xc0b89340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87be9344, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74a934e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93a9352, SUC_NE_lexicon_tags_2 },
    { 0xa7c69352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd4e9355, SUC_NE_lexicon_tags_2 },
    { 0x1c789355, SUC_NE_lexicon_tags_2 },
    { 0xd5469357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2e935d, SUC_NE_lexicon_tags_2 },
    { 0x87ae935e, SUC_NE_lexicon_tags_56 },
    { 0x552a935e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc309366, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ca9369, SUC_NE_lexicon_tags_2 },
    { 0xc112936a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c936e, SUC_NE_lexicon_tags_2 },
    { 0x1706936f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ba9372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16149378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb244937f, SUC_NE_lexicon_tags_2 },
    { 0x632c9380, SUC_NE_lexicon_tags_1 },
    { 0x95589381, SUC_NE_lexicon_tags_4 },
    { 0x56da9381, SUC_NE_lexicon_tags_201 },
    { 0x4f889382, SUC_NE_lexicon_tags_2 },
    { 0xbcc89384, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde9387, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff169389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e0938d, SUC_NE_lexicon_tags_2 },
    { 0x4c18938e, SUC_NE_lexicon_tags_2 },
    { 0xbcae938f, SUC_NE_lexicon_tags_2 },
    { 0x766c938f, SUC_NE_lexicon_tags_2 },
    { 0x7d4e9391, SUC_NE_lexicon_tags_2 },
    { 0x7b60938d, SUC_NE_lexicon_tags_2 },
    { 0x8f289393, SUC_NE_lexicon_tags_4 },
    { 0xa0f89390, SUC_NE_lexicon_tags_2 },
    { 0x1729395, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb0939b, SUC_NE_lexicon_tags_2 },
    { 0x67c6939c, SUC_NE_lexicon_tags_4 },
    { 0xd816939d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c93a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x881693a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a7c93a5, SUC_NE_lexicon_tags_2 },
    { 0x8e2693a6, SUC_NE_lexicon_tags_1 },
    { 0x30d493a7, SUC_NE_lexicon_tags_2 },
    { 0x9cfc93a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa6493aa, SUC_NE_lexicon_tags_2 },
    { 0x908293aa, SUC_NE_lexicon_tags_4 },
    { 0x15c093ac, SUC_NE_lexicon_tags_2 },
    { 0xfc7693ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4ca93af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0b093b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22be93b5, SUC_NE_lexicon_tags_3 },
    { 0xadd693b6, SUC_NE_lexicon_tags_2 },
    { 0x337a93b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a893ba, SUC_NE_lexicon_tags_2 },
    { 0x593493bb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb09293bd, SUC_NE_lexicon_tags_2 },
    { 0xee3e93bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8493c1, SUC_NE_lexicon_tags_2 },
    { 0xf3b893c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x107093c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4ee93c6, SUC_NE_lexicon_tags_2 },
    { 0x952a93c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a8093c9, SUC_NE_lexicon_tags_2 },
    { 0xf8693ca, SUC_NE_lexicon_tags_18 },
    { 0x69493cb, SUC_NE_lexicon_tags_2 },
    { 0x844e93cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31093cf, SUC_NE_lexicon_tags_2 },
    { 0x25d893d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90493d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5c93dc, SUC_NE_lexicon_tags_2 },
    { 0x569a93dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f093e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac5e93e2, SUC_NE_lexicon_tags_2 },
    { 0xb8b293e3, SUC_NE_lexicon_tags_2 },
    { 0xeb9293e4, SUC_NE_lexicon_tags_2 },
    { 0x7c5093e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5293e7, SUC_NE_lexicon_tags_30 },
    { 0x7f7693e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507893ed, SUC_NE_lexicon_tags_2 },
    { 0xb57a93ed, SUC_NE_lexicon_tags_2 },
    { 0xc5ac93ef, SUC_NE_lexicon_tags_2 },
    { 0xa78e93ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x737093f2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8493f5, SUC_NE_lexicon_tags_2 },
    { 0xe26693f6, SUC_NE_lexicon_tags_2 },
    { 0xfde693f7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa6e893f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ce93fc, SUC_NE_lexicon_tags_2 },
    { 0x4ca693fd, SUC_NE_lexicon_tags_2 },
    { 0xfeaa93fd, SUC_NE_lexicon_tags_2 },
    { 0xcd5293fd, SUC_NE_lexicon_tags_2 },
    { 0x803093ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f769403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeee09405, SUC_NE_lexicon_tags_2 },
    { 0x84fe9405, SUC_NE_lexicon_tags_2 },
    { 0x41849407, SUC_NE_lexicon_tags_2 },
    { 0xff669406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04c9412, SUC_NE_lexicon_tags_2 },
    { 0x41ec9413, SUC_NE_lexicon_tags_2 },
    { 0xdb749414, SUC_NE_lexicon_tags_2 },
    { 0xaf8a9415, SUC_NE_lexicon_tags_2 },
    { 0x573a9414, SUC_NE_lexicon_tags_1 },
    { 0x2a969417, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc941a, SUC_NE_lexicon_tags_2 },
    { 0xc546941b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc782941e, SUC_NE_lexicon_tags_2 },
    { 0x7b9e941f, SUC_NE_lexicon_tags_2 },
    { 0x251e9420, SUC_NE_lexicon_tags_2 },
    { 0x21009421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb25c9423, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b89427, SUC_NE_lexicon_tags_2 },
    { 0x14889427, SUC_NE_lexicon_tags_2 },
    { 0xb6f89429, SUC_NE_lexicon_tags_2 },
    { 0x7d42942a, SUC_NE_lexicon_tags_2 },
    { 0xfcf69428, SUC_NE_lexicon_tags_2 },
    { 0x7274942c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6148942e, SUC_NE_lexicon_tags_2 },
    { 0xdcfa942e, SUC_NE_lexicon_tags_2 },
    { 0xee46942f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90e9434, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7c943a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc62943c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c69440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b4c9442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e09445, SUC_NE_lexicon_tags_2 },
    { 0xc7709446, SUC_NE_lexicon_tags_2 },
    { 0xb67a9447, SUC_NE_lexicon_tags_2 },
    { 0x66289445, SUC_NE_lexicon_tags_2 },
    { 0x34e69449, SUC_NE_lexicon_tags_2 },
    { 0xc24e944a, SUC_NE_lexicon_tags_2 },
    { 0x94ac944a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a7a944d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed589450, SUC_NE_lexicon_tags_2 },
    { 0x8ad49450, SUC_NE_lexicon_tags_2 },
    { 0x7c129451, SUC_NE_lexicon_tags_2 },
    { 0xd2ac9453, SUC_NE_lexicon_tags_2 },
    { 0x66c9454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24849456, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e6945a, SUC_NE_lexicon_tags_1 },
    { 0x9e5a945b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4336945e, SUC_NE_lexicon_tags_2 },
    { 0x8b68945f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb269466, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaa49468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x607a946b, SUC_NE_lexicon_tags_2 },
    { 0x13ce946c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8306946e, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee809471, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82da9475, SUC_NE_lexicon_tags_2 },
    { 0xf4989476, SUC_NE_lexicon_tags_18 },
    { 0x7fba9477, SUC_NE_lexicon_tags_2 },
    { 0xdaf89478, SUC_NE_lexicon_tags_2 },
    { 0x95529475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e947c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eca9481, SUC_NE_lexicon_tags_34 },
    { 0x49309482, SUC_NE_lexicon_tags_11 },
    { 0xfbcc9482, SUC_NE_lexicon_tags_2 },
    { 0x74c9483, SUC_NE_lexicon_tags_4 },
    { 0xebce9485, SUC_NE_lexicon_tags_2 },
    { 0xd4209483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60da948d, SUC_NE_lexicon_tags_23 },
    { 0xe934948d, SUC_NE_lexicon_tags_4 },
    { 0x3f3c948f, SUC_NE_lexicon_tags_2 },
    { 0xb87e948d, SUC_NE_lexicon_tags_23 },
    { 0x81b49491, SUC_NE_lexicon_tags_2 },
    { 0x2d409491, SUC_NE_lexicon_tags_2 },
    { 0x10d4948f, SUC_NE_lexicon_tags_2 },
    { 0xbfe09494, SUC_NE_lexicon_tags_17 },
    { 0x6e049495, SUC_NE_lexicon_tags_2 },
    { 0x66309492, SUC_NE_lexicon_tags_2 },
    { 0xb1449497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1676949d, SUC_NE_lexicon_tags_2 },
    { 0xe3e0949e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaf694a0, SUC_NE_lexicon_tags_2 },
    { 0xbe494a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8294a5, SUC_NE_lexicon_tags_2 },
    { 0x240894a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913e94ab, SUC_NE_lexicon_tags_2 },
    { 0x6dcc94ab, SUC_NE_lexicon_tags_3 },
    { 0x9a2894ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x439c94af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff094b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d7294b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ba294b7, SUC_NE_lexicon_tags_2 },
    { 0x104094b8, SUC_NE_lexicon_tags_2 },
    { 0x276294b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5294bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cc94c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb77e94c2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc88294c4, SUC_NE_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c694c7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1294ca, SUC_NE_lexicon_tags_2 },
    { 0x945894cb, SUC_NE_lexicon_tags_7 },
    { 0x1ac494cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4094cf, SUC_NE_lexicon_tags_2 },
    { 0xc08294d0, SUC_NE_lexicon_tags_2 },
    { 0x4faa94cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50494d4, SUC_NE_lexicon_tags_2 },
    { 0x912094d5, SUC_NE_lexicon_tags_2 },
    { 0x3e2294d5, SUC_NE_lexicon_tags_2 },
    { 0x8b3694d4, SUC_NE_lexicon_tags_2 },
    { 0xca9c94d6, SUC_NE_lexicon_tags_2 },
    { 0xa01294d8, SUC_NE_lexicon_tags_2 },
    { 0xfd9894d8, SUC_NE_lexicon_tags_2 },
    { 0x942a94db, SUC_NE_lexicon_tags_2 },
    { 0x32c494da, SUC_NE_lexicon_tags_2 },
    { 0x5aae94dd, SUC_NE_lexicon_tags_2 },
    { 0xd86494de, SUC_NE_lexicon_tags_25 },
    { 0xce3894dd, SUC_NE_lexicon_tags_2 },
    { 0x2abc94dd, SUC_NE_lexicon_tags_2 },
    { 0xa43894e1, SUC_NE_lexicon_tags_2 },
    { 0x56c694e1, SUC_NE_lexicon_tags_2 },
    { 0x3f2694da, SUC_NE_lexicon_tags_7 },
    { 0x7e0694e4, SUC_NE_lexicon_tags_2 },
    { 0x874494df, SUC_NE_lexicon_tags_2 },
    { 0xac4c94de, SUC_NE_lexicon_tags_2 },
    { 0x899294e1, SUC_NE_lexicon_tags_2 },
    { 0xe59094e8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c94f3, SUC_NE_lexicon_tags_21 },
    { 0x662a94f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7e94fe, SUC_NE_lexicon_tags_2 },
    { 0xf00894ff, SUC_NE_lexicon_tags_2 },
    { 0xc2c09500, SUC_NE_lexicon_tags_2 },
    { 0xf91c9501, SUC_NE_lexicon_tags_2 },
    { 0xc3fc9500, SUC_NE_lexicon_tags_2 },
    { 0x301694ff, SUC_NE_lexicon_tags_2 },
    { 0x834a9504, SUC_NE_lexicon_tags_34 },
    { 0x2a949505, SUC_NE_lexicon_tags_2 },
    { 0x20a69506, SUC_NE_lexicon_tags_5 },
    { 0xf4189507, SUC_NE_lexicon_tags_2 },
    { 0x172c9504, SUC_NE_lexicon_tags_2 },
    { 0x6bfa9507, SUC_NE_lexicon_tags_2 },
    { 0x38629506, SUC_NE_lexicon_tags_2 },
    { 0x10aa950b, SUC_NE_lexicon_tags_4 },
    { 0xbf0c9505, SUC_NE_lexicon_tags_2 },
    { 0x88f69504, SUC_NE_lexicon_tags_2 },
    { 0x2c9a950a, SUC_NE_lexicon_tags_2 },
    { 0x1ff6950f, SUC_NE_lexicon_tags_2 },
    { 0xfbe0950d, SUC_NE_lexicon_tags_2 },
    { 0x742e9511, SUC_NE_lexicon_tags_3 },
    { 0xeb089512, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa49515, SUC_NE_lexicon_tags_40 },
    { 0xaafa9516, SUC_NE_lexicon_tags_3 },
    { 0xb1149517, SUC_NE_lexicon_tags_2 },
    { 0xc4009518, SUC_NE_lexicon_tags_2 },
    { 0x8ece9517, SUC_NE_lexicon_tags_2 },
    { 0x9bfc951a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc29521, SUC_NE_lexicon_tags_1 },
    { 0x610a9522, SUC_NE_lexicon_tags_2 },
    { 0x9a409522, SUC_NE_lexicon_tags_2 },
    { 0xc28a9522, SUC_NE_lexicon_tags_7 },
    { 0x50d49525, SUC_NE_lexicon_tags_2 },
    { 0x3b269525, SUC_NE_lexicon_tags_2 },
    { 0x20829527, SUC_NE_lexicon_tags_2 },
    { 0x9e09528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b48952a, SUC_NE_lexicon_tags_2 },
    { 0x1cc0952b, SUC_NE_lexicon_tags_4 },
    { 0xe12a952b, SUC_NE_lexicon_tags_2 },
    { 0x2756952d, SUC_NE_lexicon_tags_1 },
    { 0x6e82952c, SUC_NE_lexicon_tags_2 },
    { 0xe9f4952f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23889531, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e9535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd398953c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20e6953e, SUC_NE_lexicon_tags_2 },
    { 0xe648953f, SUC_NE_lexicon_tags_2 },
    { 0xe8ba9540, SUC_NE_lexicon_tags_2 },
    { 0xa5e09541, SUC_NE_lexicon_tags_4 },
    { 0x33fc9541, SUC_NE_lexicon_tags_4 },
    { 0xbe4e9543, SUC_NE_lexicon_tags_2 },
    { 0x40dc9544, SUC_NE_lexicon_tags_4 },
    { 0xfff29545, SUC_NE_lexicon_tags_2 },
    { 0x94569545, SUC_NE_lexicon_tags_2 },
    { 0x7aa9546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adc954e, SUC_NE_lexicon_tags_2 },
    { 0x3814954f, SUC_NE_lexicon_tags_2 },
    { 0x7a16954f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fa9554, SUC_NE_lexicon_tags_2 },
    { 0x90fc9554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f989559, SUC_NE_lexicon_tags_2 },
    { 0xcc4e955a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa874955c, SUC_NE_lexicon_tags_3 },
    { 0x1482955d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e389560, SUC_NE_lexicon_tags_2 },
    { 0x51d49560, SUC_NE_lexicon_tags_2 },
    { 0x97869562, SUC_NE_lexicon_tags_4 },
    { 0x6f729563, SUC_NE_lexicon_tags_2 },
    { 0x24d09560, SUC_NE_lexicon_tags_2 },
    { 0x55e89564, SUC_NE_lexicon_tags_3 },
    { 0x90169566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55649569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5364956b, SUC_NE_lexicon_tags_2 },
    { 0xd962956c, SUC_NE_lexicon_tags_2 },
    { 0x5706956b, SUC_NE_lexicon_tags_2 },
    { 0x1612956b, SUC_NE_lexicon_tags_2 },
    { 0x792c956f, SUC_NE_lexicon_tags_2 },
    { 0xcd18956f, SUC_NE_lexicon_tags_2 },
    { 0x34829571, SUC_NE_lexicon_tags_7 },
    { 0x15e29572, SUC_NE_lexicon_tags_2 },
    { 0xdd2c956d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c409579, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff70957e, SUC_NE_lexicon_tags_2 },
    { 0xd290957f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad289582, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee29586, SUC_NE_lexicon_tags_2 },
    { 0x51b69587, SUC_NE_lexicon_tags_2 },
    { 0xfeba9588, SUC_NE_lexicon_tags_34 },
    { 0xa6449589, SUC_NE_lexicon_tags_2 },
    { 0xc8329587, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60549595, SUC_NE_lexicon_tags_2 },
    { 0xa4589596, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61d69598, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffe959d, SUC_NE_lexicon_tags_2 },
    { 0x5864959d, SUC_NE_lexicon_tags_2 },
    { 0x7ff6959f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a695a3, SUC_NE_lexicon_tags_22 },
    { 0x4f1295a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fa95a8, SUC_NE_lexicon_tags_2 },
    { 0x4a0495a8, SUC_NE_lexicon_tags_2 },
    { 0xea695a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b895af, SUC_NE_lexicon_tags_2 },
    { 0x36ac95b0, SUC_NE_lexicon_tags_2 },
    { 0x42d295b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29295b4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe09095b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b295bc, SUC_NE_lexicon_tags_4 },
    { 0xc9b895bc, SUC_NE_lexicon_tags_2 },
    { 0xbbe495be, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x365c95c0, SUC_NE_lexicon_tags_2 },
    { 0x630495c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24b095c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9c95cb, SUC_NE_lexicon_tags_2 },
    { 0xba8e95cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a095cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78895d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x698095d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc7895da, SUC_NE_lexicon_tags_2 },
    { 0xee6695da, SUC_NE_lexicon_tags_2 },
    { 0xc47095db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47895e1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x518095e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f0695e5, SUC_NE_lexicon_tags_2 },
    { 0x126295e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9d695e8, SUC_NE_lexicon_tags_2 },
    { 0x17e895e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137e95ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20d495f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7695f8, SUC_NE_lexicon_tags_122 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ce95fc, SUC_NE_lexicon_tags_2 },
    { 0xc48895fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2de95ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445e9603, SUC_NE_lexicon_tags_2 },
    { 0xf6a49603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef769607, SUC_NE_lexicon_tags_10 },
    { 0x5db09608, SUC_NE_lexicon_tags_2 },
    { 0xb6e29609, SUC_NE_lexicon_tags_2 },
    { 0x9838960a, SUC_NE_lexicon_tags_2 },
    { 0x3a88960a, SUC_NE_lexicon_tags_2 },
    { 0x3f909607, SUC_NE_lexicon_tags_2 },
    { 0xde4960d, SUC_NE_lexicon_tags_2 },
    { 0xa3da960d, SUC_NE_lexicon_tags_49 },
    { 0xac4a960e, SUC_NE_lexicon_tags_2 },
    { 0xcc8a9610, SUC_NE_lexicon_tags_2 },
    { 0xb309610, SUC_NE_lexicon_tags_1 },
    { 0xa7f09610, SUC_NE_lexicon_tags_2 },
    { 0x614e9613, SUC_NE_lexicon_tags_2 },
    { 0xb944960e, SUC_NE_lexicon_tags_2 },
    { 0x543e960c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56329617, SUC_NE_lexicon_tags_2 },
    { 0xbd7e9618, SUC_NE_lexicon_tags_2 },
    { 0x7bee9618, SUC_NE_lexicon_tags_7 },
    { 0xf656961a, SUC_NE_lexicon_tags_2 },
    { 0x8dce9618, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcada961d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71c9623, SUC_NE_lexicon_tags_2 },
    { 0x7fa9624, SUC_NE_lexicon_tags_2 },
    { 0x27e89623, SUC_NE_lexicon_tags_2 },
    { 0x40fa9626, SUC_NE_lexicon_tags_2 },
    { 0xd59a9625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6e69629, SUC_NE_lexicon_tags_2 },
    { 0xb5049629, SUC_NE_lexicon_tags_4 },
    { 0x36d6962b, SUC_NE_lexicon_tags_2 },
    { 0x2a06962c, SUC_NE_lexicon_tags_2 },
    { 0x3f12962c, SUC_NE_lexicon_tags_2 },
    { 0xfbde962d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5089632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc09635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb2963b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d0963e, SUC_NE_lexicon_tags_14 },
    { 0x6cf8963f, SUC_NE_lexicon_tags_49 },
    { 0x225c963f, SUC_NE_lexicon_tags_2 },
    { 0x98449640, SUC_NE_lexicon_tags_2 },
    { 0xd0509642, SUC_NE_lexicon_tags_2 },
    { 0xc9a69642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69589649, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd12964c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7140964f, SUC_NE_lexicon_tags_2 },
    { 0xe0e89650, SUC_NE_lexicon_tags_2 },
    { 0x9222964f, SUC_NE_lexicon_tags_4 },
    { 0x43729652, SUC_NE_lexicon_tags_1 },
    { 0xd5669653, SUC_NE_lexicon_tags_2 },
    { 0x77489654, SUC_NE_lexicon_tags_2 },
    { 0x27449654, SUC_NE_lexicon_tags_2 },
    { 0xd28a9656, SUC_NE_lexicon_tags_3 },
    { 0x1249657, SUC_NE_lexicon_tags_2 },
    { 0xf3d89657, SUC_NE_lexicon_tags_7 },
    { 0x41e29654, SUC_NE_lexicon_tags_2 },
    { 0x74cc9659, SUC_NE_lexicon_tags_2 },
    { 0x39569658, SUC_NE_lexicon_tags_2 },
    { 0xf9ba9655, SUC_NE_lexicon_tags_12 },
    { 0xfede9658, SUC_NE_lexicon_tags_2 },
    { 0xda6965e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ec9661, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa23e9663, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcc9666, SUC_NE_lexicon_tags_2 },
    { 0x56129667, SUC_NE_lexicon_tags_5 },
    { 0x99ec9668, SUC_NE_lexicon_tags_3 },
    { 0xf1369666, SUC_NE_lexicon_tags_3 },
    { 0x383c966a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e3e966c, SUC_NE_lexicon_tags_2 },
    { 0x2f82966d, SUC_NE_lexicon_tags_13 },
    { 0xf4b4966c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b109670, SUC_NE_lexicon_tags_2 },
    { 0xc8389671, SUC_NE_lexicon_tags_2 },
    { 0x792c9671, SUC_NE_lexicon_tags_2 },
    { 0xbdd69672, SUC_NE_lexicon_tags_2 },
    { 0xbae69674, SUC_NE_lexicon_tags_2 },
    { 0x22189672, SUC_NE_lexicon_tags_2 },
    { 0x74449673, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785c967d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee49681, SUC_NE_lexicon_tags_2 },
    { 0x496e9681, SUC_NE_lexicon_tags_2 },
    { 0x759c9683, SUC_NE_lexicon_tags_2 },
    { 0xf3109681, SUC_NE_lexicon_tags_2 },
    { 0x33bc9685, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df09689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x697e968c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94ca968e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd34969b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa40969d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x886a96a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48496a5, SUC_NE_lexicon_tags_2 },
    { 0x87b096a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42da96aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84296ae, SUC_NE_lexicon_tags_2 },
    { 0x79f496af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd71496b1, SUC_NE_lexicon_tags_2 },
    { 0x72c96b2, SUC_NE_lexicon_tags_2 },
    { 0x49ee96b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d896bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f696c7, SUC_NE_lexicon_tags_2 },
    { 0x2fac96c8, SUC_NE_lexicon_tags_2 },
    { 0x52d496c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb096cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d8296ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe06896d0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x414c96d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278896d9, SUC_NE_lexicon_tags_2 },
    { 0x663696da, SUC_NE_lexicon_tags_2 },
    { 0x40f096db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x548696de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75ac96e0, SUC_NE_lexicon_tags_2 },
    { 0x843296e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x676e96e3, SUC_NE_lexicon_tags_2 },
    { 0x8fba96e3, SUC_NE_lexicon_tags_2 },
    { 0xfaa496e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb17a96e7, SUC_NE_lexicon_tags_2 },
    { 0xebae96e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ddc96ea, SUC_NE_lexicon_tags_2 },
    { 0x5d5696eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95c96ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec096f2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de296f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65496fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d929700, SUC_NE_lexicon_tags_2 },
    { 0xb4c09701, SUC_NE_lexicon_tags_2 },
    { 0x8a329702, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xf8b69704, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a9708, SUC_NE_lexicon_tags_2 },
    { 0x31169709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390970f, SUC_NE_lexicon_tags_3 },
    { 0xd5ca9710, SUC_NE_lexicon_tags_2 },
    { 0xf4c69710, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd89714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d69721, SUC_NE_lexicon_tags_2 },
    { 0x74c69721, SUC_NE_lexicon_tags_37 },
    { 0xaba09723, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cec9725, SUC_NE_lexicon_tags_8 },
    { 0x61309725, SUC_NE_lexicon_tags_2 },
    { 0xd3ba9727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8689729, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936e9731, SUC_NE_lexicon_tags_2 },
    { 0x7a909732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43629734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9737, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5748973a, SUC_NE_lexicon_tags_2 },
    { 0x78ee973a, SUC_NE_lexicon_tags_2 },
    { 0x271c973c, SUC_NE_lexicon_tags_402 },
    { 0x6c60973d, SUC_NE_lexicon_tags_2 },
    { 0xecb2973e, SUC_NE_lexicon_tags_2 },
    { 0x55e0973f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46c9743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82849747, SUC_NE_lexicon_tags_3 },
    { 0xccba9748, SUC_NE_lexicon_tags_3 },
    { 0xa7e69749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf132974c, SUC_NE_lexicon_tags_2 },
    { 0x7cbc974d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e98974f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9229753, SUC_NE_lexicon_tags_2 },
    { 0xe1f69754, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd8c9756, SUC_NE_lexicon_tags_2 },
    { 0x769a9756, SUC_NE_lexicon_tags_2 },
    { 0x13109758, SUC_NE_lexicon_tags_2 },
    { 0x56029759, SUC_NE_lexicon_tags_2 },
    { 0x1eb0975a, SUC_NE_lexicon_tags_2 },
    { 0xb3a69756, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22b2975d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c9761, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1289766, SUC_NE_lexicon_tags_2 },
    { 0xb46c9766, SUC_NE_lexicon_tags_2 },
    { 0x71149767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf36c976a, SUC_NE_lexicon_tags_2 },
    { 0xe34a976b, SUC_NE_lexicon_tags_2 },
    { 0x9884976b, SUC_NE_lexicon_tags_2 },
    { 0x9722976d, SUC_NE_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af49772, SUC_NE_lexicon_tags_2 },
    { 0xa8d89773, SUC_NE_lexicon_tags_2 },
    { 0x7d0c9774, SUC_NE_lexicon_tags_2 },
    { 0xc6569775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca8977d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecda9780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd89784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce9787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60c09789, SUC_NE_lexicon_tags_2 },
    { 0x1c12978a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d29790, SUC_NE_lexicon_tags_2 },
    { 0x9a449790, SUC_NE_lexicon_tags_2 },
    { 0x78309791, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa1c9794, SUC_NE_lexicon_tags_2 },
    { 0xc75e9795, SUC_NE_lexicon_tags_2 },
    { 0xee9796, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8df29798, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a2979c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa34979e, SUC_NE_lexicon_tags_7 },
    { 0xe92a979f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fe97a6, SUC_NE_lexicon_tags_2 },
    { 0xcd8a97a7, SUC_NE_lexicon_tags_2 },
    { 0x530c97a8, SUC_NE_lexicon_tags_4 },
    { 0x21297a8, SUC_NE_lexicon_tags_2 },
    { 0x30d697a6, SUC_NE_lexicon_tags_2 },
    { 0x147e97aa, SUC_NE_lexicon_tags_2 },
    { 0xb99697ab, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348c97b0, SUC_NE_lexicon_tags_2 },
    { 0xc35897b0, SUC_NE_lexicon_tags_2 },
    { 0x268297b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a497b8, SUC_NE_lexicon_tags_2 },
    { 0xabce97b9, SUC_NE_lexicon_tags_2 },
    { 0xc87497ba, SUC_NE_lexicon_tags_2 },
    { 0x94ae97bb, SUC_NE_lexicon_tags_2 },
    { 0xf8497bc, SUC_NE_lexicon_tags_2 },
    { 0x523297bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79297c1, SUC_NE_lexicon_tags_2 },
    { 0x4afc97c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9097cb, SUC_NE_lexicon_tags_2 },
    { 0x3e0497cb, SUC_NE_lexicon_tags_2 },
    { 0x14b097cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe14097cf, SUC_NE_lexicon_tags_2 },
    { 0x665497d0, SUC_NE_lexicon_tags_2 },
    { 0xbc3697d1, SUC_NE_lexicon_tags_4 },
    { 0x988497d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c297d5, SUC_NE_lexicon_tags_13 },
    { 0xa55497d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78a897d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1297dc, SUC_NE_lexicon_tags_4 },
    { 0xf3b697dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf39c97df, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49297e9, SUC_NE_lexicon_tags_2 },
    { 0x563297ea, SUC_NE_lexicon_tags_2 },
    { 0xcaa97eb, SUC_NE_lexicon_tags_2 },
    { 0xb76297ec, SUC_NE_lexicon_tags_2 },
    { 0xb18897eb, SUC_NE_lexicon_tags_4 },
    { 0x83b897ee, SUC_NE_lexicon_tags_2 },
    { 0xfeca97ee, SUC_NE_lexicon_tags_2 },
    { 0xde6c97ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1497f5, SUC_NE_lexicon_tags_2 },
    { 0x707097f6, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x150497f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x609697fb, SUC_NE_lexicon_tags_2 },
    { 0xd6fe97fc, SUC_NE_lexicon_tags_1 },
    { 0x25d497fc, SUC_NE_lexicon_tags_3 },
    { 0xb52c97fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a09804, SUC_NE_lexicon_tags_2 },
    { 0xb4649805, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b3e9807, SUC_NE_lexicon_tags_2 },
    { 0x75889807, SUC_NE_lexicon_tags_2 },
    { 0xa2bc9809, SUC_NE_lexicon_tags_2 },
    { 0x1ca2980a, SUC_NE_lexicon_tags_2 },
    { 0xf7e8980b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb68a980d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x609c980f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848a9814, SUC_NE_lexicon_tags_2 },
    { 0x70009815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee789819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd768981b, SUC_NE_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa49820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cac9825, SUC_NE_lexicon_tags_2 },
    { 0x4ee09826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7a69828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9a982b, SUC_NE_lexicon_tags_2 },
    { 0x925a982c, SUC_NE_lexicon_tags_2 },
    { 0x66a0982b, SUC_NE_lexicon_tags_2 },
    { 0xef9e982d, SUC_NE_lexicon_tags_2 },
    { 0x722e982f, SUC_NE_lexicon_tags_2 },
    { 0x2cfa9830, SUC_NE_lexicon_tags_2 },
    { 0xbbe29830, SUC_NE_lexicon_tags_2 },
    { 0xdf529832, SUC_NE_lexicon_tags_2 },
    { 0xc65a9831, SUC_NE_lexicon_tags_2 },
    { 0x9169834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x645e9837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb664983a, SUC_NE_lexicon_tags_2 },
    { 0xeb7e983a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd49840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5189842, SUC_NE_lexicon_tags_2 },
    { 0x8df49842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1449845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d009848, SUC_NE_lexicon_tags_2 },
    { 0x2c789849, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x55a8984b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1054984f, SUC_NE_lexicon_tags_2 },
    { 0xdf7c9850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79f69852, SUC_NE_lexicon_tags_2 },
    { 0x5e5e9852, SUC_NE_lexicon_tags_2 },
    { 0x50de9854, SUC_NE_lexicon_tags_2 },
    { 0x9e0e9854, SUC_NE_lexicon_tags_2 },
    { 0xf169856, SUC_NE_lexicon_tags_2 },
    { 0xef529857, SUC_NE_lexicon_tags_2 },
    { 0xf2b69858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae32985a, SUC_NE_lexicon_tags_2 },
    { 0xacfc985b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3789864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2609867, SUC_NE_lexicon_tags_0 },
    { 0x3a7a9868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc482986d, SUC_NE_lexicon_tags_2 },
    { 0x8334986e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8da09870, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03a9873, SUC_NE_lexicon_tags_2 },
    { 0x73789873, SUC_NE_lexicon_tags_2 },
    { 0x134a9875, SUC_NE_lexicon_tags_2 },
    { 0x7b09876, SUC_NE_lexicon_tags_2 },
    { 0xd2aa9877, SUC_NE_lexicon_tags_2 },
    { 0xa2c69875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c30987a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477e987f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11f49881, SUC_NE_lexicon_tags_2 },
    { 0xb1349881, SUC_NE_lexicon_tags_2 },
    { 0xdffa9882, SUC_NE_lexicon_tags_2 },
    { 0xebc9883, SUC_NE_lexicon_tags_2 },
    { 0xcfd29885, SUC_NE_lexicon_tags_2 },
    { 0x93189882, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x516e9889, SUC_NE_lexicon_tags_2 },
    { 0x1480988a, SUC_NE_lexicon_tags_2 },
    { 0xc626988a, SUC_NE_lexicon_tags_2 },
    { 0x74129889, SUC_NE_lexicon_tags_2 },
    { 0x1824988d, SUC_NE_lexicon_tags_2 },
    { 0xbd4e988e, SUC_NE_lexicon_tags_2 },
    { 0xeeb8988e, SUC_NE_lexicon_tags_3 },
    { 0x3daa988c, SUC_NE_lexicon_tags_2 },
    { 0x7c6c9891, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbe9897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac989d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a62989f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ee98a2, SUC_NE_lexicon_tags_2 },
    { 0x299898a3, SUC_NE_lexicon_tags_2 },
    { 0xa49a98a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5a098a6, SUC_NE_lexicon_tags_4 },
    { 0x291898a7, SUC_NE_lexicon_tags_2 },
    { 0x362a98a6, SUC_NE_lexicon_tags_2 },
    { 0xbd5498a9, SUC_NE_lexicon_tags_2 },
    { 0x85e498a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee9498ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaea698ae, SUC_NE_lexicon_tags_2 },
    { 0x5aa298ae, SUC_NE_lexicon_tags_7 },
    { 0x312498b0, SUC_NE_lexicon_tags_2 },
    { 0x95a98b1, SUC_NE_lexicon_tags_2 },
    { 0xc4c498b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a898b5, SUC_NE_lexicon_tags_2 },
    { 0x754298b6, SUC_NE_lexicon_tags_2 },
    { 0x653c98b5, SUC_NE_lexicon_tags_2 },
    { 0xc51e98b8, SUC_NE_lexicon_tags_1 },
    { 0x62a498b9, SUC_NE_lexicon_tags_2 },
    { 0x46ae98ba, SUC_NE_lexicon_tags_2 },
    { 0xe09a98bb, SUC_NE_lexicon_tags_2 },
    { 0x841098bb, SUC_NE_lexicon_tags_2 },
    { 0xe36498b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae098c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b9a98c3, SUC_NE_lexicon_tags_2 },
    { 0x572298c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8b498c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9698c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf87498cb, SUC_NE_lexicon_tags_2 },
    { 0xa1c698cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ee98cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaa98d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d698d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fa98dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4de98de, SUC_NE_lexicon_tags_8 },
    { 0x6d4698de, SUC_NE_lexicon_tags_2 },
    { 0x460e98e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dc98e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5e98eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c298ef, SUC_NE_lexicon_tags_2 },
    { 0xc5e498ef, SUC_NE_lexicon_tags_2 },
    { 0x2caa98f1, SUC_NE_lexicon_tags_2 },
    { 0xae6898f2, SUC_NE_lexicon_tags_3 },
    { 0x4ede98f2, SUC_NE_lexicon_tags_2 },
    { 0x208298f1, SUC_NE_lexicon_tags_2 },
    { 0x57298f5, SUC_NE_lexicon_tags_2 },
    { 0xe4e498f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f5a98f8, SUC_NE_lexicon_tags_138 },
    { 0x934298f8, SUC_NE_lexicon_tags_2 },
    { 0x243e98f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d498fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa02698ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65269901, SUC_NE_lexicon_tags_2 },
    { 0x32fa9902, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8c69904, SUC_NE_lexicon_tags_2 },
    { 0xb79e9905, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b6c9907, SUC_NE_lexicon_tags_2 },
    { 0x6369907, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b8990b, SUC_NE_lexicon_tags_32 },
    { 0xbd32990b, SUC_NE_lexicon_tags_2 },
    { 0x59e2990b, SUC_NE_lexicon_tags_2 },
    { 0x2fd8990e, SUC_NE_lexicon_tags_2 },
    { 0x148990f, SUC_NE_lexicon_tags_2 },
    { 0xfa969910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c9913, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742e9917, SUC_NE_lexicon_tags_2 },
    { 0x26809918, SUC_NE_lexicon_tags_2 },
    { 0xbea9918, SUC_NE_lexicon_tags_2 },
    { 0x41ae991a, SUC_NE_lexicon_tags_2 },
    { 0x730a991b, SUC_NE_lexicon_tags_2 },
    { 0x4166991c, SUC_NE_lexicon_tags_2 },
    { 0x6f14991c, SUC_NE_lexicon_tags_2 },
    { 0xc7b0991a, SUC_NE_lexicon_tags_2 },
    { 0x3258991f, SUC_NE_lexicon_tags_2 },
    { 0x2542991f, SUC_NE_lexicon_tags_2 },
    { 0xb1d69919, SUC_NE_lexicon_tags_2 },
    { 0xcb06991e, SUC_NE_lexicon_tags_2 },
    { 0x1092991a, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0x7a469925, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73469928, SUC_NE_lexicon_tags_2 },
    { 0x8e5a9929, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf738992d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74e9930, SUC_NE_lexicon_tags_188 },
    { 0xe3ee9931, SUC_NE_lexicon_tags_2 },
    { 0x6d1e9932, SUC_NE_lexicon_tags_2 },
    { 0xf5d29933, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ec9937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6489940, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e89945, SUC_NE_lexicon_tags_2 },
    { 0x776a9945, SUC_NE_lexicon_tags_3 },
    { 0x36a29945, SUC_NE_lexicon_tags_2 },
    { 0xbcfe9946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8684994a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe55c994c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc69951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1049954, SUC_NE_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3249959, SUC_NE_lexicon_tags_2 },
    { 0x7b9a9959, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa06995c, SUC_NE_lexicon_tags_7 },
    { 0x3226995d, SUC_NE_lexicon_tags_2 },
    { 0x8e36995e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb8a9960, SUC_NE_lexicon_tags_2 },
    { 0xf3109961, SUC_NE_lexicon_tags_2 },
    { 0x122a9962, SUC_NE_lexicon_tags_2 },
    { 0xf2389963, SUC_NE_lexicon_tags_2 },
    { 0x1f409963, SUC_NE_lexicon_tags_2 },
    { 0x21e89963, SUC_NE_lexicon_tags_2 },
    { 0xa02e9961, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2789969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e0996c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c89971, SUC_NE_lexicon_tags_2 },
    { 0x7cce9972, SUC_NE_lexicon_tags_7 },
    { 0xa1e29973, SUC_NE_lexicon_tags_2 },
    { 0xd6f09974, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd649976, SUC_NE_lexicon_tags_2 },
    { 0x91e29976, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceae997d, SUC_NE_lexicon_tags_2 },
    { 0xf488997d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21c89980, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d49984, SUC_NE_lexicon_tags_2 },
    { 0xa9269985, SUC_NE_lexicon_tags_2 },
    { 0x3b8c9986, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce09989, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8998d, SUC_NE_lexicon_tags_2 },
    { 0xba20998e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46409993, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9129997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x308e999a, SUC_NE_lexicon_tags_2 },
    { 0xed82999b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92999d, SUC_NE_lexicon_tags_2 },
    { 0x312e999e, SUC_NE_lexicon_tags_2 },
    { 0x6e7a999f, SUC_NE_lexicon_tags_19 },
    { 0x14a499a0, SUC_NE_lexicon_tags_2 },
    { 0xab7c99a0, SUC_NE_lexicon_tags_17 },
    { 0xba84999d, SUC_NE_lexicon_tags_19 },
    { 0x19aa99a3, SUC_NE_lexicon_tags_2 },
    { 0xf7ae99a3, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2a99ae, SUC_NE_lexicon_tags_2 },
    { 0x5a7099af, SUC_NE_lexicon_tags_2 },
    { 0x5e8499ae, SUC_NE_lexicon_tags_2 },
    { 0x912299af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4899b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa99ba, SUC_NE_lexicon_tags_1 },
    { 0xa69699bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2d099bd, SUC_NE_lexicon_tags_2 },
    { 0xb4e99be, SUC_NE_lexicon_tags_25 },
    { 0x634099bd, SUC_NE_lexicon_tags_2 },
    { 0x329099bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6299c3, SUC_NE_lexicon_tags_2 },
    { 0xb1ee99c4, SUC_NE_lexicon_tags_2 },
    { 0xd63a99c4, SUC_NE_lexicon_tags_2 },
    { 0x1c899c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe59e99c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbe99cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29b899cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14899d0, SUC_NE_lexicon_tags_2 },
    { 0x220c99d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a099d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x255899d7, SUC_NE_lexicon_tags_2 },
    { 0xfcb099d7, SUC_NE_lexicon_tags_2 },
    { 0x7c7699d9, SUC_NE_lexicon_tags_75 },
    { 0xc0099d8, SUC_NE_lexicon_tags_2 },
    { 0xa55699d8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x83fa99dd, SUC_NE_lexicon_tags_2 },
    { 0xb31099de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1699e5, SUC_NE_lexicon_tags_2 },
    { 0xe68a99e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe85299e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33e699ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9b699ec, SUC_NE_lexicon_tags_7 },
    { 0x601a99ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bde99f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf19099f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96899fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5589a00, SUC_NE_lexicon_tags_2 },
    { 0x1d369a01, SUC_NE_lexicon_tags_2 },
    { 0xfd509a00, SUC_NE_lexicon_tags_2 },
    { 0x81769a03, SUC_NE_lexicon_tags_17 },
    { 0xb8d69a03, SUC_NE_lexicon_tags_2 },
    { 0x8f609a05, SUC_NE_lexicon_tags_2 },
    { 0xf3009a05, SUC_NE_lexicon_tags_2 },
    { 0x15029a07, SUC_NE_lexicon_tags_2 },
    { 0xf0249a04, SUC_NE_lexicon_tags_2 },
    { 0x43589a09, SUC_NE_lexicon_tags_2 },
    { 0xcb4c9a0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e7c9a0c, SUC_NE_lexicon_tags_7 },
    { 0x81889a0d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7a9a17, SUC_NE_lexicon_tags_2 },
    { 0x3eb09a17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7609a1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c809a22, SUC_NE_lexicon_tags_2 },
    { 0x51bc9a23, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf7669a25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x934e9a27, SUC_NE_lexicon_tags_2 },
    { 0x5f949a28, SUC_NE_lexicon_tags_32 },
    { 0x69509a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5909a2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64fa9a2d, SUC_NE_lexicon_tags_2 },
    { 0x5169a2e, SUC_NE_lexicon_tags_2 },
    { 0xdaa89a2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea9a32, SUC_NE_lexicon_tags_2 },
    { 0xaec9a33, SUC_NE_lexicon_tags_2 },
    { 0x59329a34, SUC_NE_lexicon_tags_2 },
    { 0x837c9a35, SUC_NE_lexicon_tags_13 },
    { 0xe1369a36, SUC_NE_lexicon_tags_12 },
    { 0xeb849a37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e9a3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c989a3d, SUC_NE_lexicon_tags_2 },
    { 0xf7509a3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13429a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f709a43, SUC_NE_lexicon_tags_2 },
    { 0x39ae9a44, SUC_NE_lexicon_tags_17 },
    { 0xbaf69a45, SUC_NE_lexicon_tags_2 },
    { 0x5f869a44, SUC_NE_lexicon_tags_297 },
    { 0x62909a45, SUC_NE_lexicon_tags_7 },
    { 0xab969a46, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b09a4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4049a53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11fe9a55, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2289a57, SUC_NE_lexicon_tags_4 },
    { 0x12189a58, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b609a5b, SUC_NE_lexicon_tags_2 },
    { 0x54829a5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71749a5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ea49a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a389a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78189a6a, SUC_NE_lexicon_tags_2 },
    { 0xa3f49a6b, SUC_NE_lexicon_tags_5 },
    { 0xa3209a6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b369a6f, SUC_NE_lexicon_tags_2 },
    { 0x44489a70, SUC_NE_lexicon_tags_2 },
    { 0x3c109a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70109a74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ece9a78, SUC_NE_lexicon_tags_2 },
    { 0x8369a78, SUC_NE_lexicon_tags_2 },
    { 0xa1149a7a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x39aa9a7d, SUC_NE_lexicon_tags_2 },
    { 0xebe9a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972e9a82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a709a84, SUC_NE_lexicon_tags_2 },
    { 0x32c09a85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfc9a8b, SUC_NE_lexicon_tags_2 },
    { 0x21f49a8c, SUC_NE_lexicon_tags_2 },
    { 0x10b49a8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e69a90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c49a92, SUC_NE_lexicon_tags_1 },
    { 0xdbe69a92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b29a96, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78049a9e, SUC_NE_lexicon_tags_2 },
    { 0x2fbc9a9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x684e9aa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x155a9aa5, SUC_NE_lexicon_tags_2 },
    { 0x14449aa5, SUC_NE_lexicon_tags_13 },
    { 0xfda9aa7, SUC_NE_lexicon_tags_2 },
    { 0x49ce9aa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9429aaa, SUC_NE_lexicon_tags_28 },
    { 0xc9349aab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8029ab0, SUC_NE_lexicon_tags_2 },
    { 0xea1c9ab0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf69ab6, SUC_NE_lexicon_tags_2 },
    { 0x49e89ab6, SUC_NE_lexicon_tags_2 },
    { 0x90149ab7, SUC_NE_lexicon_tags_2 },
    { 0x5b089ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47409abc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bee9abf, SUC_NE_lexicon_tags_335 },
    { 0, NULL },
    { 0x13bc9ac1, SUC_NE_lexicon_tags_3 },
    { 0x48c29ac2, SUC_NE_lexicon_tags_2 },
    { 0x8c789ac3, SUC_NE_lexicon_tags_2 },
    { 0x86de9ac4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc89ac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d09acb, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d189ad2, SUC_NE_lexicon_tags_2 },
    { 0xd63c9ad3, SUC_NE_lexicon_tags_2 },
    { 0x124e9ad4, SUC_NE_lexicon_tags_2 },
    { 0x75be9ad3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9a9ad8, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a9adb, SUC_NE_lexicon_tags_2 },
    { 0xf18e9adc, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5da9ae1, SUC_NE_lexicon_tags_2 },
    { 0xc9d89ae2, SUC_NE_lexicon_tags_2 },
    { 0xbf129ae3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e9ae8, SUC_NE_lexicon_tags_2 },
    { 0x26c09ae9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cc9af8, SUC_NE_lexicon_tags_2 },
    { 0x50709af9, SUC_NE_lexicon_tags_2 },
    { 0x7dfa9af9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4c9b00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b05, SUC_NE_lexicon_tags_2 },
    { 0x17969b06, SUC_NE_lexicon_tags_2 },
    { 0x90089b06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0609b0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968a9b12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76849b14, SUC_NE_lexicon_tags_2 },
    { 0xe049b14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d329b19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa57e9b1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4a9b1d, SUC_NE_lexicon_tags_2 },
    { 0x36d29b1d, SUC_NE_lexicon_tags_2 },
    { 0x3a429b1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c409b22, SUC_NE_lexicon_tags_2 },
    { 0xe4ac9b23, SUC_NE_lexicon_tags_5 },
    { 0x21129b23, SUC_NE_lexicon_tags_2 },
    { 0x38489b23, SUC_NE_lexicon_tags_2 },
    { 0x80f89b23, SUC_NE_lexicon_tags_2 },
    { 0x95169b27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc29b2b, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xc45e9b2d, SUC_NE_lexicon_tags_2 },
    { 0x608a9b2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3fe9b30, SUC_NE_lexicon_tags_2 },
    { 0x34f09b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ce9b35, SUC_NE_lexicon_tags_2 },
    { 0x5bea9b35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c309b39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2089b3b, SUC_NE_lexicon_tags_2 },
    { 0xf95a9b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77749b42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83e9b45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb1c9b47, SUC_NE_lexicon_tags_2 },
    { 0x38249b48, SUC_NE_lexicon_tags_3 },
    { 0x12c69b49, SUC_NE_lexicon_tags_2 },
    { 0x2c3e9b48, SUC_NE_lexicon_tags_7 },
    { 0x3b789b4b, SUC_NE_lexicon_tags_2 },
    { 0x73b09b4b, SUC_NE_lexicon_tags_2 },
    { 0x748a9b4d, SUC_NE_lexicon_tags_2 },
    { 0x8a8a9b4c, SUC_NE_lexicon_tags_2 },
    { 0x4c2c9b4e, SUC_NE_lexicon_tags_2 },
    { 0x86109b50, SUC_NE_lexicon_tags_2 },
    { 0xac749b4c, SUC_NE_lexicon_tags_30 },
    { 0xfa409b52, SUC_NE_lexicon_tags_2 },
    { 0xc6ce9b50, SUC_NE_lexicon_tags_13 },
    { 0xc12e9b54, SUC_NE_lexicon_tags_2 },
    { 0x367e9b4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be29b59, SUC_NE_lexicon_tags_2 },
    { 0x61249b59, SUC_NE_lexicon_tags_2 },
    { 0xaaba9b5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x937e9b5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc89b62, SUC_NE_lexicon_tags_2 },
    { 0x993c9b63, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x559a9b66, SUC_NE_lexicon_tags_2 },
    { 0xa60a9b67, SUC_NE_lexicon_tags_2 },
    { 0x611a9b68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b129b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e49b6f, SUC_NE_lexicon_tags_2 },
    { 0x377c9b6f, SUC_NE_lexicon_tags_2 },
    { 0x1e909b71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x135a9b74, SUC_NE_lexicon_tags_2 },
    { 0x81669b75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9949b78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6629b7a, SUC_NE_lexicon_tags_18 },
    { 0xe92e9b7b, SUC_NE_lexicon_tags_2 },
    { 0xe5e69b7b, SUC_NE_lexicon_tags_2 },
    { 0xd9409b7d, SUC_NE_lexicon_tags_2 },
    { 0x87889b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8609b8a, SUC_NE_lexicon_tags_3 },
    { 0xe3ec9b8a, SUC_NE_lexicon_tags_2 },
    { 0xe1169b8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a229b92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54729b95, SUC_NE_lexicon_tags_2 },
    { 0x10329b95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5829b98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6b69b9a, SUC_NE_lexicon_tags_2 },
    { 0x29c9b9a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x3bc69b9d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e29ba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac49ba8, SUC_NE_lexicon_tags_2 },
    { 0xa10a9ba8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54349bac, SUC_NE_lexicon_tags_2 },
    { 0xb9c49bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89709baf, SUC_NE_lexicon_tags_2 },
    { 0xc6109bb0, SUC_NE_lexicon_tags_2 },
    { 0xebba9bb0, SUC_NE_lexicon_tags_2 },
    { 0xf6f89bb2, SUC_NE_lexicon_tags_2 },
    { 0x4f9e9bb3, SUC_NE_lexicon_tags_2 },
    { 0x21189bb2, SUC_NE_lexicon_tags_103 },
    { 0x6f369bb5, SUC_NE_lexicon_tags_2 },
    { 0x51ac9bb5, SUC_NE_lexicon_tags_2 },
    { 0xb9f49bb2, SUC_NE_lexicon_tags_2 },
    { 0x28ea9bb8, SUC_NE_lexicon_tags_2 },
    { 0x15609bb9, SUC_NE_lexicon_tags_2 },
    { 0x10a09bb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db09bbe, SUC_NE_lexicon_tags_2 },
    { 0xbdb49bbf, SUC_NE_lexicon_tags_2 },
    { 0x70889bbf, SUC_NE_lexicon_tags_2 },
    { 0x870a9bc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22f09bc3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9e9bc6, SUC_NE_lexicon_tags_12 },
    { 0xa12a9bc7, SUC_NE_lexicon_tags_2 },
    { 0x869a9bc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13ce9bca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1489bcc, SUC_NE_lexicon_tags_2 },
    { 0xe9149bcc, SUC_NE_lexicon_tags_2 },
    { 0xe69c9bce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec89bd0, SUC_NE_lexicon_tags_2 },
    { 0x2d809bd1, SUC_NE_lexicon_tags_2 },
    { 0x14289bd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba829bd5, SUC_NE_lexicon_tags_2 },
    { 0xa28a9bd6, SUC_NE_lexicon_tags_2 },
    { 0xc32c9bd6, SUC_NE_lexicon_tags_4 },
    { 0x21f89bd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e309be3, SUC_NE_lexicon_tags_2 },
    { 0xaa5a9be4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x32ce9be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55169beb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x588c9bed, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf09bf2, SUC_NE_lexicon_tags_4 },
    { 0xf0b49bf3, SUC_NE_lexicon_tags_2 },
    { 0x883c9bf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a9bfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe93a9bfd, SUC_NE_lexicon_tags_2 },
    { 0xda229bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09c05, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed929c08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9309c0a, SUC_NE_lexicon_tags_2 },
    { 0xe9549c0a, SUC_NE_lexicon_tags_2 },
    { 0x99609c0c, SUC_NE_lexicon_tags_13 },
    { 0x6f549c0d, SUC_NE_lexicon_tags_2 },
    { 0xb68e9c0d, SUC_NE_lexicon_tags_2 },
    { 0x70b89c0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d09c12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e9c19, SUC_NE_lexicon_tags_2 },
    { 0xc5da9c1a, SUC_NE_lexicon_tags_2 },
    { 0x1af49c1b, SUC_NE_lexicon_tags_2 },
    { 0x8a5e9c1c, SUC_NE_lexicon_tags_2 },
    { 0xd8da9c1b, SUC_NE_lexicon_tags_2 },
    { 0xdf269c1c, SUC_NE_lexicon_tags_2 },
    { 0x6c689c1f, SUC_NE_lexicon_tags_2 },
    { 0xcf7a9c20, SUC_NE_lexicon_tags_17 },
    { 0x7c89c21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde589c24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c9c28, SUC_NE_lexicon_tags_2 },
    { 0x83809c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b469c2b, SUC_NE_lexicon_tags_4 },
    { 0x5889c2b, SUC_NE_lexicon_tags_2 },
    { 0xbbac9c2d, SUC_NE_lexicon_tags_2 },
    { 0xcc2e9c2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f049c30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4309c33, SUC_NE_lexicon_tags_2 },
    { 0xc1749c34, SUC_NE_lexicon_tags_7 },
    { 0x2bba9c35, SUC_NE_lexicon_tags_2 },
    { 0x619c9c35, SUC_NE_lexicon_tags_2 },
    { 0xdb49c37, SUC_NE_lexicon_tags_2 },
    { 0xe6ba9c37, SUC_NE_lexicon_tags_361 },
    { 0xc6469c34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e69c3c, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9c9c3f, SUC_NE_lexicon_tags_2 },
    { 0xb82e9c40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x629e9c43, SUC_NE_lexicon_tags_2 },
    { 0xb7889c44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258c9c4d, SUC_NE_lexicon_tags_2 },
    { 0x5ec09c4e, SUC_NE_lexicon_tags_2 },
    { 0x7f6e9c4d, SUC_NE_lexicon_tags_2 },
    { 0xee309c4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0fc9c52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8109c57, SUC_NE_lexicon_tags_2 },
    { 0x353a9c58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1909c5c, SUC_NE_lexicon_tags_2 },
    { 0x803c9c5d, SUC_NE_lexicon_tags_17 },
    { 0xaa3a9c5c, SUC_NE_lexicon_tags_2 },
    { 0xf27c9c5f, SUC_NE_lexicon_tags_4 },
    { 0x85a29c60, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33289c65, SUC_NE_lexicon_tags_2 },
    { 0x9a189c66, SUC_NE_lexicon_tags_2 },
    { 0xedea9c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3c9c6c, SUC_NE_lexicon_tags_2 },
    { 0x944a9c6c, SUC_NE_lexicon_tags_2 },
    { 0xf9ee9c6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31849c70, SUC_NE_lexicon_tags_2 },
    { 0x6aea9c71, SUC_NE_lexicon_tags_2 },
    { 0x109c9c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17749c77, SUC_NE_lexicon_tags_2 },
    { 0xd2529c77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bc9c83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b449c86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b689c8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6a9c91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x197a9c93, SUC_NE_lexicon_tags_2 },
    { 0x7aae9c93, SUC_NE_lexicon_tags_40 },
    { 0xa4f89c94, SUC_NE_lexicon_tags_3 },
    { 0xab369c96, SUC_NE_lexicon_tags_2 },
    { 0x1cca9c94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23dc9c99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63989c9c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0849ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61ba9ca5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe88c9ca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21169ca9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6609cac, SUC_NE_lexicon_tags_24 },
    { 0x21029cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d309caf, SUC_NE_lexicon_tags_2 },
    { 0x2ccc9cb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd71c9cb2, SUC_NE_lexicon_tags_1 },
    { 0xf7e69cb3, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d69cba, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76a9cbd, SUC_NE_lexicon_tags_1 },
    { 0x96ca9cbe, SUC_NE_lexicon_tags_2 },
    { 0x7f669cbe, SUC_NE_lexicon_tags_2 },
    { 0xe3cc9cc0, SUC_NE_lexicon_tags_109 },
    { 0x5ffc9cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d89cc4, SUC_NE_lexicon_tags_2 },
    { 0xa2189cc5, SUC_NE_lexicon_tags_2 },
    { 0x4c929cc6, SUC_NE_lexicon_tags_2 },
    { 0xd2a29cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff029cca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x261e9ccd, SUC_NE_lexicon_tags_2 },
    { 0xf1369ccd, SUC_NE_lexicon_tags_2 },
    { 0xc2669ccf, SUC_NE_lexicon_tags_2 },
    { 0x12b49ccf, SUC_NE_lexicon_tags_2 },
    { 0x5a809cd1, SUC_NE_lexicon_tags_13 },
    { 0x1d909cd2, SUC_NE_lexicon_tags_2 },
    { 0x2f349cd0, SUC_NE_lexicon_tags_2 },
    { 0x53129cd4, SUC_NE_lexicon_tags_2 },
    { 0xd43e9cd5, SUC_NE_lexicon_tags_173 },
    { 0xa12c9cd6, SUC_NE_lexicon_tags_111 },
    { 0xddc9ccf, SUC_NE_lexicon_tags_2 },
    { 0xd2769cd8, SUC_NE_lexicon_tags_2 },
    { 0x33e69cd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c149cdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fa9ce0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c49ce3, SUC_NE_lexicon_tags_2 },
    { 0x92329ce4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6c9cec, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x739e9cef, SUC_NE_lexicon_tags_2 },
    { 0xff5e9cef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x277e9cf3, SUC_NE_lexicon_tags_57 },
    { 0x28309cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a2e9cf6, SUC_NE_lexicon_tags_2 },
    { 0x6f3c9cf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dc9cff, SUC_NE_lexicon_tags_2 },
    { 0xf2a9d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c049d02, SUC_NE_lexicon_tags_2 },
    { 0xbbf89d02, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6c9d07, SUC_NE_lexicon_tags_2 },
    { 0x475c9d08, SUC_NE_lexicon_tags_59 },
    { 0x53c89d09, SUC_NE_lexicon_tags_2 },
    { 0x8d409d09, SUC_NE_lexicon_tags_3 },
    { 0x882e9d0b, SUC_NE_lexicon_tags_2 },
    { 0x2fc89d08, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae9d0f, SUC_NE_lexicon_tags_2 },
    { 0xb9369d0f, SUC_NE_lexicon_tags_2 },
    { 0xfe549d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a9d14, SUC_NE_lexicon_tags_2 },
    { 0x57ce9d15, SUC_NE_lexicon_tags_2 },
    { 0x11c89d15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e429d18, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x727a9d1a, SUC_NE_lexicon_tags_2 },
    { 0x90f49d1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7409d1e, SUC_NE_lexicon_tags_2 },
    { 0xe5f69d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55089d21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ba9d25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b049d27, SUC_NE_lexicon_tags_2 },
    { 0xcfd49d28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c89d2a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb529d32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f149d36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba9d3b, SUC_NE_lexicon_tags_2 },
    { 0xabec9d3b, SUC_NE_lexicon_tags_2 },
    { 0x608a9d3b, SUC_NE_lexicon_tags_2 },
    { 0xaeba9d3d, SUC_NE_lexicon_tags_2 },
    { 0x7f8a9d3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5ee9d41, SUC_NE_lexicon_tags_2 },
    { 0xe1109d42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ec9d45, SUC_NE_lexicon_tags_2 },
    { 0xefb49d45, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89be9d4a, SUC_NE_lexicon_tags_2 },
    { 0x9c4a9d4a, SUC_NE_lexicon_tags_2 },
    { 0xe1e49d4c, SUC_NE_lexicon_tags_2 },
    { 0xdf249d4d, SUC_NE_lexicon_tags_2 },
    { 0x2df69d4b, SUC_NE_lexicon_tags_2 },
    { 0x1e589d4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee49d57, SUC_NE_lexicon_tags_2 },
    { 0xbb889d58, SUC_NE_lexicon_tags_2 },
    { 0x5409d59, SUC_NE_lexicon_tags_2 },
    { 0xc7e89d59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed2a9d5c, SUC_NE_lexicon_tags_2 },
    { 0x579e9d5d, SUC_NE_lexicon_tags_4 },
    { 0x1ed89d5c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e269d63, SUC_NE_lexicon_tags_2 },
    { 0x5d929d64, SUC_NE_lexicon_tags_2 },
    { 0x7d9c9d63, SUC_NE_lexicon_tags_2 },
    { 0x10d49d66, SUC_NE_lexicon_tags_2 },
    { 0x5ff29d63, SUC_NE_lexicon_tags_2 },
    { 0xa09a9d68, SUC_NE_lexicon_tags_2 },
    { 0x82ce9d68, SUC_NE_lexicon_tags_2 },
    { 0x29ee9d69, SUC_NE_lexicon_tags_2 },
    { 0xbd209d6b, SUC_NE_lexicon_tags_2 },
    { 0xf29e9d6c, SUC_NE_lexicon_tags_2 },
    { 0x95809d6d, SUC_NE_lexicon_tags_2 },
    { 0xf20a9d6e, SUC_NE_lexicon_tags_7 },
    { 0xacae9d6d, SUC_NE_lexicon_tags_2 },
    { 0x71369d6f, SUC_NE_lexicon_tags_2 },
    { 0xe3b09d70, SUC_NE_lexicon_tags_2 },
    { 0x565c9d65, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e5e9d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cf69d76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb49d7a, SUC_NE_lexicon_tags_2 },
    { 0xf1da9d7a, SUC_NE_lexicon_tags_2 },
    { 0x49fe9d7c, SUC_NE_lexicon_tags_4 },
    { 0x40f69d7c, SUC_NE_lexicon_tags_2 },
    { 0xbae9d7e, SUC_NE_lexicon_tags_2 },
    { 0x32ae9d7a, SUC_NE_lexicon_tags_2 },
    { 0x5ee9d7e, SUC_NE_lexicon_tags_7 },
    { 0x1cd29d81, SUC_NE_lexicon_tags_2 },
    { 0x1bbe9d81, SUC_NE_lexicon_tags_3 },
    { 0x521c9d83, SUC_NE_lexicon_tags_74 },
    { 0xb52c9d84, SUC_NE_lexicon_tags_2 },
    { 0x4e1a9d81, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28349d89, SUC_NE_lexicon_tags_2 },
    { 0xf0289d89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d809d90, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x55809d92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d09d97, SUC_NE_lexicon_tags_2 },
    { 0x5a109d98, SUC_NE_lexicon_tags_2 },
    { 0x876e9d99, SUC_NE_lexicon_tags_2 },
    { 0x71549d9a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c09da3, SUC_NE_lexicon_tags_2 },
    { 0x62869da3, SUC_NE_lexicon_tags_2 },
    { 0xc1249da3, SUC_NE_lexicon_tags_4 },
    { 0x6e869da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8349da8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d29dac, SUC_NE_lexicon_tags_2 },
    { 0xd1869dad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b969db0, SUC_NE_lexicon_tags_2 },
    { 0xd6be9db1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96169db5, SUC_NE_lexicon_tags_2 },
    { 0xdf849db5, SUC_NE_lexicon_tags_2 },
    { 0x153a9db5, SUC_NE_lexicon_tags_2 },
    { 0xa50c9db6, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4089dbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e629dc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4e89dc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22909dc6, SUC_NE_lexicon_tags_2 },
    { 0x1b6e9dc7, SUC_NE_lexicon_tags_2 },
    { 0xfcfe9dc8, SUC_NE_lexicon_tags_4 },
    { 0x17f49dc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8769dcc, SUC_NE_lexicon_tags_2 },
    { 0xcbce9dcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6709dd1, SUC_NE_lexicon_tags_2 },
    { 0x2149dd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e9dd7, SUC_NE_lexicon_tags_2 },
    { 0x55309dd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9029dda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40a29ddc, SUC_NE_lexicon_tags_2 },
    { 0x8d389ddd, SUC_NE_lexicon_tags_2 },
    { 0xe1109dde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795e9de2, SUC_NE_lexicon_tags_2 },
    { 0x6af09de3, SUC_NE_lexicon_tags_89 },
    { 0, NULL },
    { 0x72929de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fba9dea, SUC_NE_lexicon_tags_2 },
    { 0xcd809deb, SUC_NE_lexicon_tags_2 },
    { 0x5d5c9dec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d29df1, SUC_NE_lexicon_tags_2 },
    { 0x7e129df2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53969df5, SUC_NE_lexicon_tags_2 },
    { 0x6eea9df6, SUC_NE_lexicon_tags_2 },
    { 0xa11e9df6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec329e01, SUC_NE_lexicon_tags_5 },
    { 0xf74a9e02, SUC_NE_lexicon_tags_2 },
    { 0x17469e03, SUC_NE_lexicon_tags_2 },
    { 0x25629e04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c9e07, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614a9e0b, SUC_NE_lexicon_tags_2 },
    { 0x47389e0c, SUC_NE_lexicon_tags_2 },
    { 0x22649e0c, SUC_NE_lexicon_tags_2 },
    { 0x5f769e0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f229e10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9de9e12, SUC_NE_lexicon_tags_2 },
    { 0xb8749e13, SUC_NE_lexicon_tags_2 },
    { 0x10fc9e13, SUC_NE_lexicon_tags_2 },
    { 0x4ffe9e15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f69e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17e09e1b, SUC_NE_lexicon_tags_2 },
    { 0x50089e1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd09e20, SUC_NE_lexicon_tags_11 },
    { 0x3b5c9e21, SUC_NE_lexicon_tags_8 },
    { 0xf8629e22, SUC_NE_lexicon_tags_2 },
    { 0xef789e22, SUC_NE_lexicon_tags_2 },
    { 0xc38c9e21, SUC_NE_lexicon_tags_2 },
    { 0x196e9e25, SUC_NE_lexicon_tags_2 },
    { 0xd0c09e25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d369e28, SUC_NE_lexicon_tags_2 },
    { 0xd23e9e29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24089e2c, SUC_NE_lexicon_tags_2 },
    { 0x95fe9e2d, SUC_NE_lexicon_tags_2 },
    { 0xe5f89e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82e69e30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea09e39, SUC_NE_lexicon_tags_2 },
    { 0xb49a9e39, SUC_NE_lexicon_tags_2 },
    { 0x49ca9e3b, SUC_NE_lexicon_tags_2 },
    { 0x15529e3c, SUC_NE_lexicon_tags_14 },
    { 0x74a49e3a, SUC_NE_lexicon_tags_2 },
    { 0x1e529e3e, SUC_NE_lexicon_tags_2 },
    { 0x12ba9e3f, SUC_NE_lexicon_tags_2 },
    { 0xaf029e40, SUC_NE_lexicon_tags_2 },
    { 0x97f29e3e, SUC_NE_lexicon_tags_2 },
    { 0x26c89e3e, SUC_NE_lexicon_tags_2 },
    { 0x8ce49e43, SUC_NE_lexicon_tags_2 },
    { 0xf2989e3c, SUC_NE_lexicon_tags_2 },
    { 0x72389e41, SUC_NE_lexicon_tags_2 },
    { 0x3b09e42, SUC_NE_lexicon_tags_0 },
    { 0xcb089e47, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7f69e49, SUC_NE_lexicon_tags_2 },
    { 0x49609e4a, SUC_NE_lexicon_tags_4 },
    { 0x5fc69e4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f109e4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9569e53, SUC_NE_lexicon_tags_3 },
    { 0xa8789e54, SUC_NE_lexicon_tags_2 },
    { 0xaf509e55, SUC_NE_lexicon_tags_2 },
    { 0x89269e56, SUC_NE_lexicon_tags_2 },
    { 0xcdec9e57, SUC_NE_lexicon_tags_49 },
    { 0x7b6e9e57, SUC_NE_lexicon_tags_2 },
    { 0x5ee09e59, SUC_NE_lexicon_tags_4 },
    { 0x76089e53, SUC_NE_lexicon_tags_2 },
    { 0x1a5a9e54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13349e5f, SUC_NE_lexicon_tags_2 },
    { 0x4c929e5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf09e64, SUC_NE_lexicon_tags_2 },
    { 0x285a9e64, SUC_NE_lexicon_tags_5 },
    { 0xa9789e66, SUC_NE_lexicon_tags_2 },
    { 0xab8a9e64, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bc9e6a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a9e6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d729e70, SUC_NE_lexicon_tags_14 },
    { 0xe9fc9e71, SUC_NE_lexicon_tags_2 },
    { 0xee1c9e72, SUC_NE_lexicon_tags_0 },
    { 0xb8249e71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdc49e75, SUC_NE_lexicon_tags_3 },
    { 0x4e229e76, SUC_NE_lexicon_tags_18 },
    { 0xdeee9e77, SUC_NE_lexicon_tags_2 },
    { 0x466a9e77, SUC_NE_lexicon_tags_2 },
    { 0x29aa9e77, SUC_NE_lexicon_tags_2 },
    { 0xeb729e77, SUC_NE_lexicon_tags_2 },
    { 0x78249e7b, SUC_NE_lexicon_tags_2 },
    { 0xcaa49e7c, SUC_NE_lexicon_tags_2 },
    { 0x6c189e7c, SUC_NE_lexicon_tags_2 },
    { 0xd5569e7d, SUC_NE_lexicon_tags_2 },
    { 0x9fb49e7f, SUC_NE_lexicon_tags_2 },
    { 0x20ce9e78, SUC_NE_lexicon_tags_3 },
    { 0x9c029e75, SUC_NE_lexicon_tags_2 },
    { 0x6249e82, SUC_NE_lexicon_tags_7 },
    { 0xdf69e83, SUC_NE_lexicon_tags_50 },
    { 0x43de9e81, SUC_NE_lexicon_tags_2 },
    { 0x8c5c9e85, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb69e8d, SUC_NE_lexicon_tags_1 },
    { 0xd9949e8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc669e90, SUC_NE_lexicon_tags_2 },
    { 0xd53c9e91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaab09e93, SUC_NE_lexicon_tags_2 },
    { 0x60fa9e94, SUC_NE_lexicon_tags_2 },
    { 0x9b729e94, SUC_NE_lexicon_tags_2 },
    { 0x91e29e96, SUC_NE_lexicon_tags_17 },
    { 0x98609e97, SUC_NE_lexicon_tags_2 },
    { 0x635a9e97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d989e9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed69ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba29ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1149ea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef89eaa, SUC_NE_lexicon_tags_336 },
    { 0x6d2c9eab, SUC_NE_lexicon_tags_2 },
    { 0x2a549eaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc89eb0, SUC_NE_lexicon_tags_3 },
    { 0xe5529eb0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x82aa9eb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x901e9eb5, SUC_NE_lexicon_tags_2 },
    { 0xd809eb6, SUC_NE_lexicon_tags_2 },
    { 0x4e749eb7, SUC_NE_lexicon_tags_2 },
    { 0xa1829eb7, SUC_NE_lexicon_tags_2 },
    { 0xdd9e9eb9, SUC_NE_lexicon_tags_2 },
    { 0xa3ee9eba, SUC_NE_lexicon_tags_21 },
    { 0xb29c9eba, SUC_NE_lexicon_tags_4 },
    { 0xb0fc9eb8, SUC_NE_lexicon_tags_2 },
    { 0x52969ebc, SUC_NE_lexicon_tags_2 },
    { 0xee0c9eb8, SUC_NE_lexicon_tags_3 },
    { 0xd3ca9ebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35c9ec3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61509ec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5249ecc, SUC_NE_lexicon_tags_2 },
    { 0x1749ecc, SUC_NE_lexicon_tags_2 },
    { 0xbfc49ece, SUC_NE_lexicon_tags_2 },
    { 0xbff29ecd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9989ed1, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x3b529ed3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x983c9ed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49edb, SUC_NE_lexicon_tags_2 },
    { 0x9de69edb, SUC_NE_lexicon_tags_2 },
    { 0xaa0a9edd, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x90689edf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc929ee3, SUC_NE_lexicon_tags_3 },
    { 0x9fae9ee4, SUC_NE_lexicon_tags_7 },
    { 0xc669ee5, SUC_NE_lexicon_tags_2 },
    { 0x83ba9ee6, SUC_NE_lexicon_tags_2 },
    { 0xf5169ee7, SUC_NE_lexicon_tags_2 },
    { 0xa7149ee8, SUC_NE_lexicon_tags_5 },
    { 0xfeb49ee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aa29eeb, SUC_NE_lexicon_tags_2 },
    { 0x70a9eec, SUC_NE_lexicon_tags_2 },
    { 0x84569eec, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53929ef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e9ef9, SUC_NE_lexicon_tags_3 },
    { 0x52869efa, SUC_NE_lexicon_tags_2 },
    { 0x8e0e9efb, SUC_NE_lexicon_tags_2 },
    { 0xc9ae9ef9, SUC_NE_lexicon_tags_19 },
    { 0xeb149efd, SUC_NE_lexicon_tags_2 },
    { 0xeab09efb, SUC_NE_lexicon_tags_2 },
    { 0x5bb69eff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a529f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda9f07, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce509f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fc9f0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30889f14, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x585c9f16, SUC_NE_lexicon_tags_5 },
    { 0xb02a9f17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0869f1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9be9f1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6d49f1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2e69f21, SUC_NE_lexicon_tags_2 },
    { 0x9ca49f22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c469f26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6089f2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d969f2e, SUC_NE_lexicon_tags_2 },
    { 0xaf749f2e, SUC_NE_lexicon_tags_290 },
    { 0xfabe9f30, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99949f33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf909f35, SUC_NE_lexicon_tags_2 },
    { 0x90c69f35, SUC_NE_lexicon_tags_2 },
    { 0xb57e9f37, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x1309f39, SUC_NE_lexicon_tags_2 },
    { 0x683a9f3a, SUC_NE_lexicon_tags_2 },
    { 0x10589f3b, SUC_NE_lexicon_tags_2 },
    { 0x26509f3c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd009f3f, SUC_NE_lexicon_tags_2 },
    { 0x79369f40, SUC_NE_lexicon_tags_2 },
    { 0x421c9f3f, SUC_NE_lexicon_tags_2 },
    { 0x5829f42, SUC_NE_lexicon_tags_2 },
    { 0xcae69f43, SUC_NE_lexicon_tags_74 },
    { 0x2ff09f44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1629f48, SUC_NE_lexicon_tags_2 },
    { 0xced29f49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c89f52, SUC_NE_lexicon_tags_2 },
    { 0x8b809f52, SUC_NE_lexicon_tags_2 },
    { 0x7a809f54, SUC_NE_lexicon_tags_2 },
    { 0xd1aa9f54, SUC_NE_lexicon_tags_2 },
    { 0x67549f52, SUC_NE_lexicon_tags_2 },
    { 0x77929f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76b89f59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x774e9f5c, SUC_NE_lexicon_tags_2 },
    { 0x45aa9f5d, SUC_NE_lexicon_tags_2 },
    { 0x26f29f5e, SUC_NE_lexicon_tags_2 },
    { 0x83c9f5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5949f61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3c9f65, SUC_NE_lexicon_tags_2 },
    { 0xf94a9f66, SUC_NE_lexicon_tags_2 },
    { 0xd4e29f67, SUC_NE_lexicon_tags_2 },
    { 0xa73a9f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20fa9f6a, SUC_NE_lexicon_tags_2 },
    { 0xaa9c9f6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae209f6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe09f74, SUC_NE_lexicon_tags_2 },
    { 0xaacc9f74, SUC_NE_lexicon_tags_2 },
    { 0x690a9f75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ea9f7a, SUC_NE_lexicon_tags_112 },
    { 0x50969f7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0469f7e, SUC_NE_lexicon_tags_5 },
    { 0x4149f7f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bc9f86, SUC_NE_lexicon_tags_2 },
    { 0x35d29f87, SUC_NE_lexicon_tags_2 },
    { 0x65309f87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf189f8a, SUC_NE_lexicon_tags_5 },
    { 0x12f29f8a, SUC_NE_lexicon_tags_2 },
    { 0xf7e09f8c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe9089f8e, SUC_NE_lexicon_tags_2 },
    { 0xa9469f8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb209f93, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xfcd69f95, SUC_NE_lexicon_tags_2 },
    { 0xb8d89f96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba49f9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x919e9fa0, SUC_NE_lexicon_tags_2 },
    { 0xa4c29fa1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x39409fa4, SUC_NE_lexicon_tags_2 },
    { 0x3c7a9fa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0489fa7, SUC_NE_lexicon_tags_2 },
    { 0xa3ee9fa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b49fab, SUC_NE_lexicon_tags_2 },
    { 0xb84a9fac, SUC_NE_lexicon_tags_2 },
    { 0x71409fad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88929faf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e69fb2, SUC_NE_lexicon_tags_2 },
    { 0x18489fb2, SUC_NE_lexicon_tags_7 },
    { 0x81a69fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79649fb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6669fba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bee9fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69fbf, SUC_NE_lexicon_tags_2 },
    { 0x59849fbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15149fc5, SUC_NE_lexicon_tags_2 },
    { 0x898e9fc6, SUC_NE_lexicon_tags_2 },
    { 0x17e9fc7, SUC_NE_lexicon_tags_3 },
    { 0xdfd29fc8, SUC_NE_lexicon_tags_2 },
    { 0xa5889fc9, SUC_NE_lexicon_tags_8 },
    { 0x61b89fc5, SUC_NE_lexicon_tags_2 },
    { 0x76c69fcb, SUC_NE_lexicon_tags_7 },
    { 0x39149fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75849fce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45209fd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de9fd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8c9fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea629fd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f829fdb, SUC_NE_lexicon_tags_0 },
    { 0x4f069fdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17169fe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a809fea, SUC_NE_lexicon_tags_2 },
    { 0x38289fea, SUC_NE_lexicon_tags_2 },
    { 0xd2589fec, SUC_NE_lexicon_tags_2 },
    { 0x621c9fed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2189fef, SUC_NE_lexicon_tags_2 },
    { 0x34a9ff0, SUC_NE_lexicon_tags_2 },
    { 0x66e89ff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd709ff6, SUC_NE_lexicon_tags_2 },
    { 0x25ca9ff7, SUC_NE_lexicon_tags_2 },
    { 0xe9a49ff8, SUC_NE_lexicon_tags_2 },
    { 0xd9929ff9, SUC_NE_lexicon_tags_2 },
    { 0x14689ffa, SUC_NE_lexicon_tags_30 },
    { 0x6e009ff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53a9ffe, SUC_NE_lexicon_tags_17 },
    { 0x63d69fff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x412ea001, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbca007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eea00e, SUC_NE_lexicon_tags_2 },
    { 0xf854a00e, SUC_NE_lexicon_tags_40 },
    { 0x95e4a00e, SUC_NE_lexicon_tags_2 },
    { 0xb2eea00f, SUC_NE_lexicon_tags_2 },
    { 0x5b1ea012, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb86ea014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f58a016, SUC_NE_lexicon_tags_2 },
    { 0x1674a016, SUC_NE_lexicon_tags_2 },
    { 0x4ff2a017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18a01d, SUC_NE_lexicon_tags_2 },
    { 0xa8f6a01e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d9ea020, SUC_NE_lexicon_tags_7 },
    { 0x88f6a021, SUC_NE_lexicon_tags_2 },
    { 0xbbfca022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ca02c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0caa030, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac0a035, SUC_NE_lexicon_tags_7 },
    { 0x6b98a036, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb8a039, SUC_NE_lexicon_tags_2 },
    { 0xfb70a03a, SUC_NE_lexicon_tags_2 },
    { 0xed76a03b, SUC_NE_lexicon_tags_2 },
    { 0xc6e4a03c, SUC_NE_lexicon_tags_2 },
    { 0x3844a03d, SUC_NE_lexicon_tags_3 },
    { 0x97dea03e, SUC_NE_lexicon_tags_2 },
    { 0x971ca03f, SUC_NE_lexicon_tags_2 },
    { 0x824aa03b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd19aa043, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa286a047, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1aa04a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9102a04c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a0a050, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24a053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86aca055, SUC_NE_lexicon_tags_2 },
    { 0xcedca055, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde92a058, SUC_NE_lexicon_tags_58 },
    { 0x544ca058, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dca05d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4a060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa80ea062, SUC_NE_lexicon_tags_2 },
    { 0x761aa063, SUC_NE_lexicon_tags_2 },
    { 0xfec8a063, SUC_NE_lexicon_tags_2 },
    { 0x43b4a065, SUC_NE_lexicon_tags_7 },
    { 0x6df8a066, SUC_NE_lexicon_tags_2 },
    { 0x92b6a065, SUC_NE_lexicon_tags_2 },
    { 0x16e6a068, SUC_NE_lexicon_tags_2 },
    { 0x7f14a066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38aca06b, SUC_NE_lexicon_tags_2 },
    { 0x56bca06c, SUC_NE_lexicon_tags_2 },
    { 0x42c8a06d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x425ea06f, SUC_NE_lexicon_tags_2 },
    { 0x3f14a070, SUC_NE_lexicon_tags_2 },
    { 0x1bf2a071, SUC_NE_lexicon_tags_2 },
    { 0xdbfaa071, SUC_NE_lexicon_tags_2 },
    { 0x20aaa070, SUC_NE_lexicon_tags_2 },
    { 0x96aca070, SUC_NE_lexicon_tags_2 },
    { 0xaf0ea073, SUC_NE_lexicon_tags_2 },
    { 0xa43ea070, SUC_NE_lexicon_tags_2 },
    { 0xd198a073, SUC_NE_lexicon_tags_2 },
    { 0x8c68a078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe62aa07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3aa07e, SUC_NE_lexicon_tags_2 },
    { 0xd698a07e, SUC_NE_lexicon_tags_2 },
    { 0xb41ca080, SUC_NE_lexicon_tags_2 },
    { 0xd16ca081, SUC_NE_lexicon_tags_2 },
    { 0x5e3ca082, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4a086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240a08a, SUC_NE_lexicon_tags_2 },
    { 0x4ae4a08b, SUC_NE_lexicon_tags_2 },
    { 0x6a02a08b, SUC_NE_lexicon_tags_2 },
    { 0x3344a08d, SUC_NE_lexicon_tags_2 },
    { 0xe0d4a08e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cca095, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe38ea097, SUC_NE_lexicon_tags_14 },
    { 0xe9f8a098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8aa09d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cea0a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e96a0ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee38a0af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb96a0b1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf7d4a0b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980ea0b7, SUC_NE_lexicon_tags_2 },
    { 0xbc22a0b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc44a0ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2ea0bf, SUC_NE_lexicon_tags_2 },
    { 0x99c6a0bf, SUC_NE_lexicon_tags_2 },
    { 0x436ea0c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93ea0c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dd4a0c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f16a0c9, SUC_NE_lexicon_tags_2 },
    { 0xae1ea0c9, SUC_NE_lexicon_tags_405 },
    { 0x6aaaa0cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd38a0cd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffea0d0, SUC_NE_lexicon_tags_2 },
    { 0x3030a0d1, SUC_NE_lexicon_tags_19 },
    { 0xee6a0d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2778a0d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e8a0d7, SUC_NE_lexicon_tags_2 },
    { 0x34e4a0d8, SUC_NE_lexicon_tags_2 },
    { 0xc01aa0d9, SUC_NE_lexicon_tags_167 },
    { 0x1ddaa0da, SUC_NE_lexicon_tags_2 },
    { 0x418ea0d9, SUC_NE_lexicon_tags_7 },
    { 0xa520a0dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfda0a0de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc44a0e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3016a0e4, SUC_NE_lexicon_tags_2 },
    { 0x70daa0e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903ca0ea, SUC_NE_lexicon_tags_2 },
    { 0xc6a6a0eb, SUC_NE_lexicon_tags_3 },
    { 0x3452a0eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170aa0f3, SUC_NE_lexicon_tags_2 },
    { 0xf598a0f3, SUC_NE_lexicon_tags_2 },
    { 0xcce2a0f4, SUC_NE_lexicon_tags_2 },
    { 0xf33aa0f6, SUC_NE_lexicon_tags_2 },
    { 0x42ea0f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3660a0fd, SUC_NE_lexicon_tags_2 },
    { 0x74e4a0fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf540a100, SUC_NE_lexicon_tags_2 },
    { 0xad02a101, SUC_NE_lexicon_tags_2 },
    { 0x7930a100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b4a105, SUC_NE_lexicon_tags_2 },
    { 0x50fca105, SUC_NE_lexicon_tags_1 },
    { 0xe28a107, SUC_NE_lexicon_tags_2 },
    { 0x995aa108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeca10c, SUC_NE_lexicon_tags_2 },
    { 0x18d8a10c, SUC_NE_lexicon_tags_2 },
    { 0x1104a10e, SUC_NE_lexicon_tags_2 },
    { 0x2a40a10d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc2a11a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6ea11e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90faa124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0a127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x411aa12a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d6a12e, SUC_NE_lexicon_tags_2 },
    { 0xb76a12f, SUC_NE_lexicon_tags_2 },
    { 0x9702a130, SUC_NE_lexicon_tags_2 },
    { 0x9a66a131, SUC_NE_lexicon_tags_188 },
    { 0xf58a132, SUC_NE_lexicon_tags_2 },
    { 0x281aa132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105ea13a, SUC_NE_lexicon_tags_2 },
    { 0x621ea13b, SUC_NE_lexicon_tags_2 },
    { 0x94a0a13b, SUC_NE_lexicon_tags_4 },
    { 0xa0c4a13d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2aa140, SUC_NE_lexicon_tags_2 },
    { 0x77a8a141, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47daa145, SUC_NE_lexicon_tags_2 },
    { 0x4294a146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecaa14a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777aa152, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596aa157, SUC_NE_lexicon_tags_2 },
    { 0x3c04a157, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33ca15d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc56a15f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec92a162, SUC_NE_lexicon_tags_2 },
    { 0x3f38a163, SUC_NE_lexicon_tags_2 },
    { 0x455ea162, SUC_NE_lexicon_tags_2 },
    { 0x416ea165, SUC_NE_lexicon_tags_2 },
    { 0xa2e8a166, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9038a16a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1edea16c, SUC_NE_lexicon_tags_2 },
    { 0x1008a16d, SUC_NE_lexicon_tags_2 },
    { 0x3a9ea16e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffaca171, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9374a173, SUC_NE_lexicon_tags_2 },
    { 0xc966a173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5058a177, SUC_NE_lexicon_tags_2 },
    { 0x2dd2a178, SUC_NE_lexicon_tags_25 },
    { 0xc678a178, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a0a17c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a17f, SUC_NE_lexicon_tags_2 },
    { 0x6ceca180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bca183, SUC_NE_lexicon_tags_2 },
    { 0x6fb4a183, SUC_NE_lexicon_tags_2 },
    { 0x89f8a185, SUC_NE_lexicon_tags_37 },
    { 0xff9ea185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3ea189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe418a18b, SUC_NE_lexicon_tags_2 },
    { 0xeeb6a18c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf398a18e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe2a191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b2ea193, SUC_NE_lexicon_tags_2 },
    { 0x4d98a194, SUC_NE_lexicon_tags_2 },
    { 0xc246a193, SUC_NE_lexicon_tags_214 },
    { 0x2342a196, SUC_NE_lexicon_tags_18 },
    { 0xfc74a197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8590a19a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b2ca19c, SUC_NE_lexicon_tags_2 },
    { 0x9e22a19d, SUC_NE_lexicon_tags_2 },
    { 0x45f8a19e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276ea1a2, SUC_NE_lexicon_tags_2 },
    { 0xbd54a1a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b14a1a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa1a8, SUC_NE_lexicon_tags_2 },
    { 0x5094a1a9, SUC_NE_lexicon_tags_2 },
    { 0x2c74a1aa, SUC_NE_lexicon_tags_2 },
    { 0xcee0a1ab, SUC_NE_lexicon_tags_2 },
    { 0xa806a1ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89faa1ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0a1b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0aa1ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x464aa1bd, SUC_NE_lexicon_tags_2 },
    { 0x4feca1bd, SUC_NE_lexicon_tags_2 },
    { 0x2ccca1bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x188a1c2, SUC_NE_lexicon_tags_2 },
    { 0xf7b4a1c3, SUC_NE_lexicon_tags_2 },
    { 0xb19aa1c2, SUC_NE_lexicon_tags_2 },
    { 0x672a1c5, SUC_NE_lexicon_tags_2 },
    { 0x386a1c6, SUC_NE_lexicon_tags_2 },
    { 0xbc70a1c5, SUC_NE_lexicon_tags_30 },
    { 0xeaf8a1c5, SUC_NE_lexicon_tags_2 },
    { 0x4cc2a1c3, SUC_NE_lexicon_tags_7 },
    { 0xd8b6a1c5, SUC_NE_lexicon_tags_2 },
    { 0x18ca1c9, SUC_NE_lexicon_tags_2 },
    { 0xf01aa1cc, SUC_NE_lexicon_tags_2 },
    { 0x41b6a1cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1744a1cf, SUC_NE_lexicon_tags_2 },
    { 0x2faea1cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x167ca1d2, SUC_NE_lexicon_tags_2 },
    { 0xb2aea1d3, SUC_NE_lexicon_tags_1 },
    { 0x83ca1d4, SUC_NE_lexicon_tags_2 },
    { 0x1696a1d5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf040a1d9, SUC_NE_lexicon_tags_2 },
    { 0xac4a1da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbcaa1dc, SUC_NE_lexicon_tags_2 },
    { 0x73ca1dd, SUC_NE_lexicon_tags_2 },
    { 0x2f5aa1dd, SUC_NE_lexicon_tags_2 },
    { 0x5598a1df, SUC_NE_lexicon_tags_17 },
    { 0xd196a1e0, SUC_NE_lexicon_tags_2 },
    { 0x60aea1df, SUC_NE_lexicon_tags_4 },
    { 0x20aea1e2, SUC_NE_lexicon_tags_2 },
    { 0xc568a1e3, SUC_NE_lexicon_tags_2 },
    { 0xb7d4a1e3, SUC_NE_lexicon_tags_2 },
    { 0x120ea1e3, SUC_NE_lexicon_tags_1 },
    { 0xbc9ca1e6, SUC_NE_lexicon_tags_2 },
    { 0xb150a1e7, SUC_NE_lexicon_tags_24 },
    { 0xfe90a1e8, SUC_NE_lexicon_tags_2 },
    { 0x478ea1e8, SUC_NE_lexicon_tags_2 },
    { 0x92cea1e5, SUC_NE_lexicon_tags_2 },
    { 0x143ca1eb, SUC_NE_lexicon_tags_2 },
    { 0x7c72a1ec, SUC_NE_lexicon_tags_2 },
    { 0x1656a1ec, SUC_NE_lexicon_tags_36 },
    { 0xd35ea1e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x689aa1f0, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d54a1f2, SUC_NE_lexicon_tags_2 },
    { 0x96fca1f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2b8a1f5, SUC_NE_lexicon_tags_2 },
    { 0xf48ea1f6, SUC_NE_lexicon_tags_2 },
    { 0x241ea1f5, SUC_NE_lexicon_tags_2 },
    { 0xc418a1f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf520a1fa, SUC_NE_lexicon_tags_2 },
    { 0x2b14a1fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe52a1fd, SUC_NE_lexicon_tags_2 },
    { 0x3974a1fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4650a200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b12a207, SUC_NE_lexicon_tags_2 },
    { 0x887aa208, SUC_NE_lexicon_tags_2 },
    { 0x919ca209, SUC_NE_lexicon_tags_2 },
    { 0x682ca209, SUC_NE_lexicon_tags_2 },
    { 0x3f44a20b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b24a20e, SUC_NE_lexicon_tags_2 },
    { 0x5ef2a20f, SUC_NE_lexicon_tags_2 },
    { 0x3168a20f, SUC_NE_lexicon_tags_2 },
    { 0xbcf4a211, SUC_NE_lexicon_tags_2 },
    { 0x77e2a210, SUC_NE_lexicon_tags_2 },
    { 0x4a8a210, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70eaa218, SUC_NE_lexicon_tags_2 },
    { 0xb0a0a218, SUC_NE_lexicon_tags_7 },
    { 0x25d0a21a, SUC_NE_lexicon_tags_7 },
    { 0xb3b6a21b, SUC_NE_lexicon_tags_2 },
    { 0xb820a21a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc068a21e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc440a221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d74a227, SUC_NE_lexicon_tags_2 },
    { 0x2040a228, SUC_NE_lexicon_tags_2 },
    { 0xb26ea229, SUC_NE_lexicon_tags_2 },
    { 0x19fea22a, SUC_NE_lexicon_tags_2 },
    { 0xe40ca22a, SUC_NE_lexicon_tags_2 },
    { 0x5d0aa22c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc762a22e, SUC_NE_lexicon_tags_17 },
    { 0xc23aa22e, SUC_NE_lexicon_tags_2 },
    { 0xa94aa22e, SUC_NE_lexicon_tags_145 },
    { 0xbc92a22e, SUC_NE_lexicon_tags_1 },
    { 0x31dca231, SUC_NE_lexicon_tags_2 },
    { 0xb878a233, SUC_NE_lexicon_tags_2 },
    { 0xb24a234, SUC_NE_lexicon_tags_2 },
    { 0xc2b4a233, SUC_NE_lexicon_tags_2 },
    { 0xa956a236, SUC_NE_lexicon_tags_2 },
    { 0x40aa232, SUC_NE_lexicon_tags_2 },
    { 0xfe14a235, SUC_NE_lexicon_tags_2 },
    { 0x61d2a239, SUC_NE_lexicon_tags_2 },
    { 0xa38ea239, SUC_NE_lexicon_tags_2 },
    { 0x41d8a236, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f0a243, SUC_NE_lexicon_tags_7 },
    { 0x182ea244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fca248, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c4a24b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x4c2aa24d, SUC_NE_lexicon_tags_2 },
    { 0x9e34a24e, SUC_NE_lexicon_tags_2 },
    { 0x7a22a24f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbf6a251, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa432a253, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x437ca255, SUC_NE_lexicon_tags_2 },
    { 0x4e20a256, SUC_NE_lexicon_tags_2 },
    { 0x6412a257, SUC_NE_lexicon_tags_2 },
    { 0x1d40a257, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa4aa25a, SUC_NE_lexicon_tags_2 },
    { 0xf104a25a, SUC_NE_lexicon_tags_2 },
    { 0x502ca25c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a260, SUC_NE_lexicon_tags_2 },
    { 0x228a261, SUC_NE_lexicon_tags_2 },
    { 0xe4aa262, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf284a265, SUC_NE_lexicon_tags_2 },
    { 0xad5ca266, SUC_NE_lexicon_tags_2 },
    { 0x4c84a267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb638a269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d4ea26b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3260a26d, SUC_NE_lexicon_tags_2 },
    { 0x752ea26e, SUC_NE_lexicon_tags_2 },
    { 0xa53aa26d, SUC_NE_lexicon_tags_2 },
    { 0xa73ca26e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eea273, SUC_NE_lexicon_tags_2 },
    { 0x328ca274, SUC_NE_lexicon_tags_2 },
    { 0x7d22a275, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0dea277, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecc8a279, SUC_NE_lexicon_tags_2 },
    { 0x6d42a27a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe872a27c, SUC_NE_lexicon_tags_30 },
    { 0x4bf4a27d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa850a285, SUC_NE_lexicon_tags_2 },
    { 0x1adea286, SUC_NE_lexicon_tags_2 },
    { 0x9090a287, SUC_NE_lexicon_tags_4 },
    { 0xf990a288, SUC_NE_lexicon_tags_2 },
    { 0xd18a288, SUC_NE_lexicon_tags_2 },
    { 0x88e0a28a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6762a28d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea291, SUC_NE_lexicon_tags_2 },
    { 0x95c4a292, SUC_NE_lexicon_tags_2 },
    { 0x3202a293, SUC_NE_lexicon_tags_2 },
    { 0x4daaa291, SUC_NE_lexicon_tags_2 },
    { 0x1a18a295, SUC_NE_lexicon_tags_2 },
    { 0x853ca293, SUC_NE_lexicon_tags_5 },
    { 0x9dc0a297, SUC_NE_lexicon_tags_2 },
    { 0xcb9ea298, SUC_NE_lexicon_tags_2 },
    { 0x15faa299, SUC_NE_lexicon_tags_2 },
    { 0xf9ea29a, SUC_NE_lexicon_tags_1 },
    { 0xf1fea29a, SUC_NE_lexicon_tags_7 },
    { 0xea6a29c, SUC_NE_lexicon_tags_3 },
    { 0x4d56a29d, SUC_NE_lexicon_tags_4 },
    { 0xff04a291, SUC_NE_lexicon_tags_40 },
    { 0x366ca296, SUC_NE_lexicon_tags_2 },
    { 0xa5aca29d, SUC_NE_lexicon_tags_7 },
    { 0xfd38a2a1, SUC_NE_lexicon_tags_86 },
    { 0x4524a2a2, SUC_NE_lexicon_tags_2 },
    { 0x67faa2a3, SUC_NE_lexicon_tags_2 },
    { 0xbc6ea2a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e66a2a7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd286a2aa, SUC_NE_lexicon_tags_4 },
    { 0xc7d0a2aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef32a2ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cca2b1, SUC_NE_lexicon_tags_2 },
    { 0x411aa2b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3392a2b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8400a2b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e6a2b9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12aea2bc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6a2bf, SUC_NE_lexicon_tags_2 },
    { 0xe06a2c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd132a2c2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe76a2c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ca2c8, SUC_NE_lexicon_tags_2 },
    { 0x696a2c8, SUC_NE_lexicon_tags_2 },
    { 0x55a4a2ca, SUC_NE_lexicon_tags_2 },
    { 0xeac6a2c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ca2d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60faa2d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a2a2d6, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27ca2d9, SUC_NE_lexicon_tags_2 },
    { 0x4338a2da, SUC_NE_lexicon_tags_2 },
    { 0x9e1ca2db, SUC_NE_lexicon_tags_2 },
    { 0x98c2a2dc, SUC_NE_lexicon_tags_2 },
    { 0xc2baa2d9, SUC_NE_lexicon_tags_2 },
    { 0xdffaa2de, SUC_NE_lexicon_tags_2 },
    { 0x9dea2de, SUC_NE_lexicon_tags_2 },
    { 0x9dbca2e0, SUC_NE_lexicon_tags_2 },
    { 0xa9e2a2e0, SUC_NE_lexicon_tags_2 },
    { 0x55ea2e2, SUC_NE_lexicon_tags_2 },
    { 0xe47ea2e2, SUC_NE_lexicon_tags_2 },
    { 0x945ea2dd, SUC_NE_lexicon_tags_2 },
    { 0xe3fea2e5, SUC_NE_lexicon_tags_2 },
    { 0x9106a2e6, SUC_NE_lexicon_tags_2 },
    { 0xc042a2e5, SUC_NE_lexicon_tags_2 },
    { 0x410aa2db, SUC_NE_lexicon_tags_4 },
    { 0x13d6a2e9, SUC_NE_lexicon_tags_2 },
    { 0xa804a2dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18a2f1, SUC_NE_lexicon_tags_2 },
    { 0x3fe6a2f2, SUC_NE_lexicon_tags_1 },
    { 0x6388a2f3, SUC_NE_lexicon_tags_2 },
    { 0x40a0a2f2, SUC_NE_lexicon_tags_2 },
    { 0xa616a2f3, SUC_NE_lexicon_tags_2 },
    { 0xa5aa2f3, SUC_NE_lexicon_tags_40 },
    { 0x9af0a2f6, SUC_NE_lexicon_tags_2 },
    { 0xff12a2f8, SUC_NE_lexicon_tags_2 },
    { 0xaaf0a2f9, SUC_NE_lexicon_tags_23 },
    { 0xe410a2f9, SUC_NE_lexicon_tags_2 },
    { 0x454ca2fb, SUC_NE_lexicon_tags_2 },
    { 0x8090a2fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb594a2fe, SUC_NE_lexicon_tags_2 },
    { 0x8b76a2fe, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4478a302, SUC_NE_lexicon_tags_2 },
    { 0xf6a2a302, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7244a305, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x6b02a307, SUC_NE_lexicon_tags_2 },
    { 0xb948a308, SUC_NE_lexicon_tags_2 },
    { 0x5180a307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe04a30b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4730a30e, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1092a313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65b8a315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd774a318, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb476a31e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa140a322, SUC_NE_lexicon_tags_2 },
    { 0x792a323, SUC_NE_lexicon_tags_2 },
    { 0x2c74a322, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9432a327, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86f8a329, SUC_NE_lexicon_tags_2 },
    { 0xdf20a32a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0a32d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2882a32f, SUC_NE_lexicon_tags_2 },
    { 0xbeb2a32f, SUC_NE_lexicon_tags_2 },
    { 0xc232a331, SUC_NE_lexicon_tags_2 },
    { 0x2e32a32f, SUC_NE_lexicon_tags_2 },
    { 0xb3bea330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe32a337, SUC_NE_lexicon_tags_2 },
    { 0x7f3ea338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23dea33a, SUC_NE_lexicon_tags_2 },
    { 0xf4baa33a, SUC_NE_lexicon_tags_2 },
    { 0x77baa33a, SUC_NE_lexicon_tags_2 },
    { 0xe01aa33d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00ea340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x580ca343, SUC_NE_lexicon_tags_2 },
    { 0x2ba6a343, SUC_NE_lexicon_tags_2 },
    { 0x21bca343, SUC_NE_lexicon_tags_2 },
    { 0xee8aa343, SUC_NE_lexicon_tags_2 },
    { 0x7d8ea343, SUC_NE_lexicon_tags_2 },
    { 0x98fca348, SUC_NE_lexicon_tags_2 },
    { 0xb5baa346, SUC_NE_lexicon_tags_2 },
    { 0xc14a347, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f7aa34c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b00a358, SUC_NE_lexicon_tags_2 },
    { 0xa6cea359, SUC_NE_lexicon_tags_2 },
    { 0xdb90a358, SUC_NE_lexicon_tags_2 },
    { 0x9e0ea35b, SUC_NE_lexicon_tags_2 },
    { 0x1426a35c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692aa361, SUC_NE_lexicon_tags_2 },
    { 0xc012a362, SUC_NE_lexicon_tags_2 },
    { 0x9b34a362, SUC_NE_lexicon_tags_2 },
    { 0x8bb6a362, SUC_NE_lexicon_tags_2 },
    { 0xb6d8a362, SUC_NE_lexicon_tags_2 },
    { 0x720ca365, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf532a369, SUC_NE_lexicon_tags_2 },
    { 0x3e48a36a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9de2a36c, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead2a375, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe18a378, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247ca37d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x75eaa37f, SUC_NE_lexicon_tags_2 },
    { 0xd234a37f, SUC_NE_lexicon_tags_2 },
    { 0x67e4a381, SUC_NE_lexicon_tags_2 },
    { 0x4528a382, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dca385, SUC_NE_lexicon_tags_2 },
    { 0x15aea385, SUC_NE_lexicon_tags_2 },
    { 0xc91ea387, SUC_NE_lexicon_tags_2 },
    { 0x3ab2a388, SUC_NE_lexicon_tags_2 },
    { 0xa13ca388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e0a38e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3aa392, SUC_NE_lexicon_tags_0 },
    { 0x2d0ea393, SUC_NE_lexicon_tags_2 },
    { 0x90aea393, SUC_NE_lexicon_tags_2 },
    { 0xe09ea395, SUC_NE_lexicon_tags_2 },
    { 0xd3e6a396, SUC_NE_lexicon_tags_2 },
    { 0xbb92a393, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x514ca399, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5ea3a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fca3a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b74a3a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91ca3ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2aa3b1, SUC_NE_lexicon_tags_2 },
    { 0xdf02a3b2, SUC_NE_lexicon_tags_2 },
    { 0xbc92a3b2, SUC_NE_lexicon_tags_2 },
    { 0x62c6a3b3, SUC_NE_lexicon_tags_7 },
    { 0x2aeca3b2, SUC_NE_lexicon_tags_2 },
    { 0xc56ea3b3, SUC_NE_lexicon_tags_2 },
    { 0x9e2a3b7, SUC_NE_lexicon_tags_2 },
    { 0x640a3b8, SUC_NE_lexicon_tags_2 },
    { 0xec88a3b9, SUC_NE_lexicon_tags_2 },
    { 0x52dea3b7, SUC_NE_lexicon_tags_2 },
    { 0xafd8a3bb, SUC_NE_lexicon_tags_2 },
    { 0xb91ca3b7, SUC_NE_lexicon_tags_2 },
    { 0x2d88a3b9, SUC_NE_lexicon_tags_4 },
    { 0x7b4a3be, SUC_NE_lexicon_tags_3 },
    { 0x2714a3bf, SUC_NE_lexicon_tags_2 },
    { 0xb85ea3bc, SUC_NE_lexicon_tags_2 },
    { 0x59dea3c1, SUC_NE_lexicon_tags_4 },
    { 0xfec4a3c2, SUC_NE_lexicon_tags_3 },
    { 0xaae4a3c3, SUC_NE_lexicon_tags_2 },
    { 0x6b98a3c0, SUC_NE_lexicon_tags_2 },
    { 0xfd74a3c5, SUC_NE_lexicon_tags_2 },
    { 0x85b4a3c5, SUC_NE_lexicon_tags_2 },
    { 0x8be0a3c5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf64ea3c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a82a3cd, SUC_NE_lexicon_tags_2 },
    { 0xce84a3ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb16a3d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x156ca3d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x464a3d5, SUC_NE_lexicon_tags_2 },
    { 0x6be4a3d6, SUC_NE_lexicon_tags_2 },
    { 0xb548a3d7, SUC_NE_lexicon_tags_2 },
    { 0xecc2a3d8, SUC_NE_lexicon_tags_2 },
    { 0x7deaa3d8, SUC_NE_lexicon_tags_19 },
    { 0xd628a3d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18b6a3dc, SUC_NE_lexicon_tags_4 },
    { 0x2ea8a3dd, SUC_NE_lexicon_tags_2 },
    { 0x1f2a3de, SUC_NE_lexicon_tags_2 },
    { 0x3640a3df, SUC_NE_lexicon_tags_2 },
    { 0x95b0a3de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a4ea3e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb2a3e4, SUC_NE_lexicon_tags_2 },
    { 0x7754a3e5, SUC_NE_lexicon_tags_2 },
    { 0xc4c6a3e6, SUC_NE_lexicon_tags_4 },
    { 0x63b6a3e7, SUC_NE_lexicon_tags_2 },
    { 0x5d62a3e8, SUC_NE_lexicon_tags_2 },
    { 0x2128a3e7, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b54a3ed, SUC_NE_lexicon_tags_2 },
    { 0x3128a3ee, SUC_NE_lexicon_tags_2 },
    { 0xb73aa3ef, SUC_NE_lexicon_tags_2 },
    { 0xbbe0a3ee, SUC_NE_lexicon_tags_2 },
    { 0x7d86a3f1, SUC_NE_lexicon_tags_2 },
    { 0xa59ea3ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4482a3f7, SUC_NE_lexicon_tags_2 },
    { 0xc74aa3f7, SUC_NE_lexicon_tags_4 },
    { 0x8a62a3f9, SUC_NE_lexicon_tags_11 },
    { 0xc9eca3fa, SUC_NE_lexicon_tags_2 },
    { 0x85c4a3fb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dea3fe, SUC_NE_lexicon_tags_264 },
    { 0xc4ca3ff, SUC_NE_lexicon_tags_2 },
    { 0x3de0a400, SUC_NE_lexicon_tags_2 },
    { 0x769ea3fe, SUC_NE_lexicon_tags_2 },
    { 0xad46a400, SUC_NE_lexicon_tags_2 },
    { 0xf502a403, SUC_NE_lexicon_tags_1 },
    { 0x5aeea403, SUC_NE_lexicon_tags_7 },
    { 0x6ebaa405, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaa409, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0aa40e, SUC_NE_lexicon_tags_2 },
    { 0xd22aa40f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3994a413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde8a416, SUC_NE_lexicon_tags_2 },
    { 0xd0faa416, SUC_NE_lexicon_tags_2 },
    { 0xbb38a416, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x536ea41b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea04a41f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1204a421, SUC_NE_lexicon_tags_4 },
    { 0x5c24a422, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eaa426, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x382aa428, SUC_NE_lexicon_tags_2 },
    { 0x1a2ca429, SUC_NE_lexicon_tags_0 },
    { 0x5212a42a, SUC_NE_lexicon_tags_2 },
    { 0x3b7aa42a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d20a42e, SUC_NE_lexicon_tags_2 },
    { 0x125ca42f, SUC_NE_lexicon_tags_2 },
    { 0x6468a430, SUC_NE_lexicon_tags_2 },
    { 0xb3bea430, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb68ca433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5116a436, SUC_NE_lexicon_tags_2 },
    { 0xdafca437, SUC_NE_lexicon_tags_2 },
    { 0x394aa438, SUC_NE_lexicon_tags_2 },
    { 0x5a7ca437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7942a43b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2d2a43d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250ca444, SUC_NE_lexicon_tags_2 },
    { 0x4dc4a445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xede4a448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc022a44b, SUC_NE_lexicon_tags_2 },
    { 0x4d94a44b, SUC_NE_lexicon_tags_2 },
    { 0x39f2a44d, SUC_NE_lexicon_tags_2 },
    { 0xc94ca44e, SUC_NE_lexicon_tags_2 },
    { 0x7f72a44f, SUC_NE_lexicon_tags_2 },
    { 0x2f16a450, SUC_NE_lexicon_tags_2 },
    { 0xbceca450, SUC_NE_lexicon_tags_17 },
    { 0x8148a450, SUC_NE_lexicon_tags_1 },
    { 0xb448a452, SUC_NE_lexicon_tags_2 },
    { 0x3732a44c, SUC_NE_lexicon_tags_2 },
    { 0x5f7ea44f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2358a459, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418a45d, SUC_NE_lexicon_tags_2 },
    { 0xda54a45e, SUC_NE_lexicon_tags_2 },
    { 0xf90aa45f, SUC_NE_lexicon_tags_2 },
    { 0x129ea45d, SUC_NE_lexicon_tags_2 },
    { 0xfdf0a461, SUC_NE_lexicon_tags_2 },
    { 0x3f36a462, SUC_NE_lexicon_tags_2 },
    { 0x8448a461, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d8a469, SUC_NE_lexicon_tags_2 },
    { 0xca4a46a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78c6a46c, SUC_NE_lexicon_tags_2 },
    { 0x100ca46d, SUC_NE_lexicon_tags_2 },
    { 0x9a82a46e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a54a472, SUC_NE_lexicon_tags_3 },
    { 0x905ca473, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b4a479, SUC_NE_lexicon_tags_2 },
    { 0xc38ea47a, SUC_NE_lexicon_tags_53 },
    { 0xa012a47a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe546a47f, SUC_NE_lexicon_tags_2 },
    { 0xeb6aa480, SUC_NE_lexicon_tags_2 },
    { 0x1324a480, SUC_NE_lexicon_tags_2 },
    { 0xd326a482, SUC_NE_lexicon_tags_2 },
    { 0xa51ca483, SUC_NE_lexicon_tags_2 },
    { 0xe9e0a484, SUC_NE_lexicon_tags_2 },
    { 0x90dca484, SUC_NE_lexicon_tags_2 },
    { 0x72aea484, SUC_NE_lexicon_tags_2 },
    { 0x310aa481, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58daa48b, SUC_NE_lexicon_tags_3 },
    { 0xf030a48c, SUC_NE_lexicon_tags_2 },
    { 0x45dca48c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6ea490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac56a496, SUC_NE_lexicon_tags_17 },
    { 0xe244a496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6198a499, SUC_NE_lexicon_tags_4 },
    { 0xae8ea49a, SUC_NE_lexicon_tags_2 },
    { 0x85b2a49a, SUC_NE_lexicon_tags_2 },
    { 0xfd98a49a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b26a49f, SUC_NE_lexicon_tags_2 },
    { 0xf038a4a0, SUC_NE_lexicon_tags_2 },
    { 0xb110a4a1, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dea4a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a3ea4a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e0a4aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd82a4ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe60ea4b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2520a4b3, SUC_NE_lexicon_tags_3 },
    { 0x7320a4b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe69ca4b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe15ca4b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe74ea4ba, SUC_NE_lexicon_tags_3 },
    { 0x6c5ca4ba, SUC_NE_lexicon_tags_2 },
    { 0x4e78a4bc, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c08a4c6, SUC_NE_lexicon_tags_2 },
    { 0x320a4c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53f0a4c9, SUC_NE_lexicon_tags_2 },
    { 0x6ee0a4ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0e6a4cc, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a00a4cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba0ea4d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50aea4d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2312a4d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7214a4db, SUC_NE_lexicon_tags_2 },
    { 0xc756a4dc, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9558a4df, SUC_NE_lexicon_tags_2 },
    { 0xd41ca4df, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a3ca4e2, SUC_NE_lexicon_tags_225 },
    { 0, NULL },
    { 0xb912a4e4, SUC_NE_lexicon_tags_2 },
    { 0xd332a4e5, SUC_NE_lexicon_tags_58 },
    { 0xf000a4e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77e0a4e8, SUC_NE_lexicon_tags_2 },
    { 0x72c2a4e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1baca4eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dcca4ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefd4a4ef, SUC_NE_lexicon_tags_33 },
    { 0, NULL },
    { 0x12e8a4f1, SUC_NE_lexicon_tags_2 },
    { 0xb6a6a4f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2a4f9, SUC_NE_lexicon_tags_113 },
    { 0x8072a4f9, SUC_NE_lexicon_tags_18 },
    { 0xf1f4a4f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e22a501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26aa504, SUC_NE_lexicon_tags_2 },
    { 0x9ff6a505, SUC_NE_lexicon_tags_2 },
    { 0xd14ea506, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb22a509, SUC_NE_lexicon_tags_2 },
    { 0xce64a50a, SUC_NE_lexicon_tags_2 },
    { 0xa208a50b, SUC_NE_lexicon_tags_2 },
    { 0xb826a50c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc12a50e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c2a511, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bcca513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f28a516, SUC_NE_lexicon_tags_2 },
    { 0xb6baa516, SUC_NE_lexicon_tags_2 },
    { 0x76a2a518, SUC_NE_lexicon_tags_2 },
    { 0xc750a517, SUC_NE_lexicon_tags_2 },
    { 0x52e0a51a, SUC_NE_lexicon_tags_2 },
    { 0xa22ea51b, SUC_NE_lexicon_tags_2 },
    { 0xe0f4a518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ca520, SUC_NE_lexicon_tags_127 },
    { 0, NULL },
    { 0x53fea522, SUC_NE_lexicon_tags_2 },
    { 0x53aea523, SUC_NE_lexicon_tags_2 },
    { 0x582ca524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61aa527, SUC_NE_lexicon_tags_2 },
    { 0xea4ea528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bea52e, SUC_NE_lexicon_tags_2 },
    { 0xce82a52f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa168a533, SUC_NE_lexicon_tags_455 },
    { 0, NULL },
    { 0x1882a535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d0a537, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3920a53a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d4a540, SUC_NE_lexicon_tags_2 },
    { 0x97b4a541, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xecc8a543, SUC_NE_lexicon_tags_2 },
    { 0x6242a544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7b6a546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fea54a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadaaa54d, SUC_NE_lexicon_tags_2 },
    { 0xb4baa54d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a6a551, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x604ea553, SUC_NE_lexicon_tags_2 },
    { 0x2060a553, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9922a55f, SUC_NE_lexicon_tags_2 },
    { 0x4bfea560, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fea564, SUC_NE_lexicon_tags_2 },
    { 0x74bea565, SUC_NE_lexicon_tags_2 },
    { 0xc9baa564, SUC_NE_lexicon_tags_2 },
    { 0xb06aa567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1610a569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d04a572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65baa575, SUC_NE_lexicon_tags_2 },
    { 0x99dca576, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fd0a578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644a57e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121ca58b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb460a591, SUC_NE_lexicon_tags_2 },
    { 0x4650a592, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56eaa596, SUC_NE_lexicon_tags_2 },
    { 0x59ca597, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb9d2a599, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf8a59c, SUC_NE_lexicon_tags_2 },
    { 0x3bcaa59c, SUC_NE_lexicon_tags_2 },
    { 0xa4caa59c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8360a5a4, SUC_NE_lexicon_tags_2 },
    { 0x2cbaa5a5, SUC_NE_lexicon_tags_7 },
    { 0xb6eaa5a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d8a5a8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x4690a5aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacaa5ac, SUC_NE_lexicon_tags_5 },
    { 0x7e8ca5ad, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b6ea5af, SUC_NE_lexicon_tags_2 },
    { 0x6e70a5b0, SUC_NE_lexicon_tags_2 },
    { 0x5366a5b0, SUC_NE_lexicon_tags_2 },
    { 0x6086a5b2, SUC_NE_lexicon_tags_2 },
    { 0xd09ca5b3, SUC_NE_lexicon_tags_2 },
    { 0xf610a5b4, SUC_NE_lexicon_tags_2 },
    { 0x69bea5b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa00a5bd, SUC_NE_lexicon_tags_4 },
    { 0x7c4ea5be, SUC_NE_lexicon_tags_2 },
    { 0x70aaa5bd, SUC_NE_lexicon_tags_7 },
    { 0x41d6a5c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e60a5c8, SUC_NE_lexicon_tags_2 },
    { 0x52cea5c9, SUC_NE_lexicon_tags_19 },
    { 0x1484a5ca, SUC_NE_lexicon_tags_2 },
    { 0x1d54a5cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf920a5cd, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6ed2a5cf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aaa5d2, SUC_NE_lexicon_tags_2 },
    { 0x2ddaa5d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d4a5d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190aa5db, SUC_NE_lexicon_tags_2 },
    { 0xe552a5dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aaa5e1, SUC_NE_lexicon_tags_4 },
    { 0x9b16a5e2, SUC_NE_lexicon_tags_2 },
    { 0x1680a5e1, SUC_NE_lexicon_tags_3 },
    { 0x104ca5e4, SUC_NE_lexicon_tags_2 },
    { 0xab5aa5e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84ea5eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd1ca5ed, SUC_NE_lexicon_tags_3 },
    { 0xd166a5ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb886a5f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28ca5f2, SUC_NE_lexicon_tags_2 },
    { 0x6310a5f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7daa5f9, SUC_NE_lexicon_tags_2 },
    { 0x1b72a5fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3aa5fd, SUC_NE_lexicon_tags_2 },
    { 0xf122a5fd, SUC_NE_lexicon_tags_2 },
    { 0xb34ca5ff, SUC_NE_lexicon_tags_2 },
    { 0xf036a5ff, SUC_NE_lexicon_tags_1 },
    { 0xbfea5fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6ca605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acaa60c, SUC_NE_lexicon_tags_2 },
    { 0xa854a60d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac54a612, SUC_NE_lexicon_tags_2 },
    { 0x1800a612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe178a617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f96a61b, SUC_NE_lexicon_tags_2 },
    { 0xb660a61c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a10a61e, SUC_NE_lexicon_tags_2 },
    { 0x87a4a61f, SUC_NE_lexicon_tags_2 },
    { 0xc258a61f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf82a626, SUC_NE_lexicon_tags_2 },
    { 0x48e2a627, SUC_NE_lexicon_tags_2 },
    { 0x9feca626, SUC_NE_lexicon_tags_2 },
    { 0x55d0a629, SUC_NE_lexicon_tags_2 },
    { 0x3f80a62a, SUC_NE_lexicon_tags_7 },
    { 0x191aa62b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9baa62e, SUC_NE_lexicon_tags_4 },
    { 0xd4c8a62e, SUC_NE_lexicon_tags_59 },
    { 0xe5f4a630, SUC_NE_lexicon_tags_2 },
    { 0xda8ca631, SUC_NE_lexicon_tags_2 },
    { 0xa830a631, SUC_NE_lexicon_tags_19 },
    { 0xf4f2a633, SUC_NE_lexicon_tags_3 },
    { 0x70d6a630, SUC_NE_lexicon_tags_3 },
    { 0x1c24a635, SUC_NE_lexicon_tags_2 },
    { 0xf1fca636, SUC_NE_lexicon_tags_2 },
    { 0xc8e4a636, SUC_NE_lexicon_tags_1 },
    { 0x4382a638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a14a63a, SUC_NE_lexicon_tags_2 },
    { 0xe330a63b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9eaa63d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4160a63f, SUC_NE_lexicon_tags_2 },
    { 0xa070a63f, SUC_NE_lexicon_tags_2 },
    { 0x2320a640, SUC_NE_lexicon_tags_2 },
    { 0x2254a642, SUC_NE_lexicon_tags_2 },
    { 0xd782a643, SUC_NE_lexicon_tags_2 },
    { 0xb840a644, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ccaa646, SUC_NE_lexicon_tags_2 },
    { 0x3af8a647, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd04a64b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb32a64e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba3ea650, SUC_NE_lexicon_tags_2 },
    { 0x6964a650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x456aa654, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bea659, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7daa65f, SUC_NE_lexicon_tags_2 },
    { 0x9f76a660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f42a662, SUC_NE_lexicon_tags_2 },
    { 0x3d06a663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ccaa665, SUC_NE_lexicon_tags_2 },
    { 0xcca2a665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2d6a668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd34a66c, SUC_NE_lexicon_tags_2 },
    { 0x43f4a66d, SUC_NE_lexicon_tags_2 },
    { 0xd2d0a66e, SUC_NE_lexicon_tags_2 },
    { 0x8c7ea66e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb0a676, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4a67c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc760a683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41caa685, SUC_NE_lexicon_tags_2 },
    { 0x522ea686, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa796a689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7aa68c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bca68f, SUC_NE_lexicon_tags_2 },
    { 0xc7ea690, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x542aa692, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98a696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476a69a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80a69d, SUC_NE_lexicon_tags_2 },
    { 0xa10ca69e, SUC_NE_lexicon_tags_2 },
    { 0x9904a69f, SUC_NE_lexicon_tags_2 },
    { 0xb1cca69f, SUC_NE_lexicon_tags_2 },
    { 0x87b6a6a0, SUC_NE_lexicon_tags_2 },
    { 0xabc8a6a2, SUC_NE_lexicon_tags_19 },
    { 0x41d8a6a2, SUC_NE_lexicon_tags_2 },
    { 0x6458a6a4, SUC_NE_lexicon_tags_2 },
    { 0xe536a6a4, SUC_NE_lexicon_tags_2 },
    { 0xabeca6a4, SUC_NE_lexicon_tags_2 },
    { 0xda5ca69d, SUC_NE_lexicon_tags_2 },
    { 0xa9b4a69f, SUC_NE_lexicon_tags_2 },
    { 0xc58aa6a9, SUC_NE_lexicon_tags_2 },
    { 0xb654a6aa, SUC_NE_lexicon_tags_2 },
    { 0xf6d4a6aa, SUC_NE_lexicon_tags_2 },
    { 0x1584a6ac, SUC_NE_lexicon_tags_2 },
    { 0x80bea6ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe778a6af, SUC_NE_lexicon_tags_2 },
    { 0xad78a6af, SUC_NE_lexicon_tags_2 },
    { 0xdb56a6b0, SUC_NE_lexicon_tags_2 },
    { 0x25f6a6b2, SUC_NE_lexicon_tags_2 },
    { 0xb412a6b2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e58a6ba, SUC_NE_lexicon_tags_2 },
    { 0x2f4a6ba, SUC_NE_lexicon_tags_2 },
    { 0x4926a6bc, SUC_NE_lexicon_tags_307 },
    { 0xabd4a6bd, SUC_NE_lexicon_tags_2 },
    { 0xa9d4a6bd, SUC_NE_lexicon_tags_2 },
    { 0x3ce4a6be, SUC_NE_lexicon_tags_2 },
    { 0x9fc4a6c0, SUC_NE_lexicon_tags_109 },
    { 0x48a6c1, SUC_NE_lexicon_tags_2 },
    { 0xca40a6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf42a6c4, SUC_NE_lexicon_tags_2 },
    { 0x34bea6c4, SUC_NE_lexicon_tags_3 },
    { 0x4690a6c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8aa6cb, SUC_NE_lexicon_tags_17 },
    { 0x37b6a6cc, SUC_NE_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a22a6d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x642a6d6, SUC_NE_lexicon_tags_7 },
    { 0x4ed2a6d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa0a6db, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7156a6de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9518a6e4, SUC_NE_lexicon_tags_2 },
    { 0xb630a6e4, SUC_NE_lexicon_tags_7 },
    { 0x7406a6e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75f4a6e8, SUC_NE_lexicon_tags_2 },
    { 0x2aa6a6e8, SUC_NE_lexicon_tags_19 },
    { 0x158aa6ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68d4a6ec, SUC_NE_lexicon_tags_18 },
    { 0xd70ca6ed, SUC_NE_lexicon_tags_1 },
    { 0xbd14a6ee, SUC_NE_lexicon_tags_2 },
    { 0xcc4a6ee, SUC_NE_lexicon_tags_4 },
    { 0xcf48a6f0, SUC_NE_lexicon_tags_2 },
    { 0x8ae6a6f0, SUC_NE_lexicon_tags_2 },
    { 0xa140a6f2, SUC_NE_lexicon_tags_2 },
    { 0xf7e2a6f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78a6f7, SUC_NE_lexicon_tags_2 },
    { 0x634a6f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f42a6fb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4704a6ff, SUC_NE_lexicon_tags_2 },
    { 0xa4ea700, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb62a702, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4ea707, SUC_NE_lexicon_tags_3 },
    { 0x260ca708, SUC_NE_lexicon_tags_2 },
    { 0xf22ea709, SUC_NE_lexicon_tags_2 },
    { 0x1e58a70a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82fca70c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df4a712, SUC_NE_lexicon_tags_4 },
    { 0x2ccca713, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x569ca715, SUC_NE_lexicon_tags_30 },
    { 0x6d8aa716, SUC_NE_lexicon_tags_2 },
    { 0x9c0ea717, SUC_NE_lexicon_tags_2 },
    { 0xed2aa718, SUC_NE_lexicon_tags_2 },
    { 0x1e4ea719, SUC_NE_lexicon_tags_2 },
    { 0x35eea718, SUC_NE_lexicon_tags_2 },
    { 0xab4ca718, SUC_NE_lexicon_tags_30 },
    { 0x1baa71c, SUC_NE_lexicon_tags_2 },
    { 0xb89ea71c, SUC_NE_lexicon_tags_2 },
    { 0x8fb2a719, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ca721, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa340a727, SUC_NE_lexicon_tags_2 },
    { 0x98e2a728, SUC_NE_lexicon_tags_2 },
    { 0x48a0a727, SUC_NE_lexicon_tags_4 },
    { 0x3e4ea72a, SUC_NE_lexicon_tags_2 },
    { 0xf1aa72b, SUC_NE_lexicon_tags_2 },
    { 0xc748a72b, SUC_NE_lexicon_tags_34 },
    { 0x2d44a72c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d2a731, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aa737, SUC_NE_lexicon_tags_4 },
    { 0x7a6a737, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f08a73f, SUC_NE_lexicon_tags_2 },
    { 0x89eaa740, SUC_NE_lexicon_tags_17 },
    { 0x63eca741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e0a746, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x256aa749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbea750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc794a755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9aa759, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af0a75e, SUC_NE_lexicon_tags_2 },
    { 0x9eb6a75f, SUC_NE_lexicon_tags_2 },
    { 0x132ca75f, SUC_NE_lexicon_tags_2 },
    { 0x249ca75f, SUC_NE_lexicon_tags_2 },
    { 0x8fa6a762, SUC_NE_lexicon_tags_2 },
    { 0xbd6ea75e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62faa765, SUC_NE_lexicon_tags_1 },
    { 0x6ea0a765, SUC_NE_lexicon_tags_2 },
    { 0x2d82a765, SUC_NE_lexicon_tags_2 },
    { 0x3d9aa768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97eaa76a, SUC_NE_lexicon_tags_2 },
    { 0x8c68a76b, SUC_NE_lexicon_tags_2 },
    { 0x53a0a76b, SUC_NE_lexicon_tags_2 },
    { 0xf9f0a76b, SUC_NE_lexicon_tags_2 },
    { 0xc734a76b, SUC_NE_lexicon_tags_7 },
    { 0x7ef4a76f, SUC_NE_lexicon_tags_2 },
    { 0x1722a770, SUC_NE_lexicon_tags_2 },
    { 0x440ca770, SUC_NE_lexicon_tags_2 },
    { 0x245aa76c, SUC_NE_lexicon_tags_2 },
    { 0xb2fca773, SUC_NE_lexicon_tags_2 },
    { 0x4ee4a774, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5faa777, SUC_NE_lexicon_tags_2 },
    { 0xaa4aa778, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7caa77f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7450a784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e9aa786, SUC_NE_lexicon_tags_219 },
    { 0xea10a786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe436a789, SUC_NE_lexicon_tags_2 },
    { 0xf7cca78a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2a78d, SUC_NE_lexicon_tags_2 },
    { 0x8e66a78e, SUC_NE_lexicon_tags_2 },
    { 0x1084a78e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1392a791, SUC_NE_lexicon_tags_2 },
    { 0x1164a791, SUC_NE_lexicon_tags_2 },
    { 0xd35aa793, SUC_NE_lexicon_tags_2 },
    { 0xe28ca791, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff8a79a, SUC_NE_lexicon_tags_4 },
    { 0x3aa2a79a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb6a7a2, SUC_NE_lexicon_tags_2 },
    { 0xc21ca7a3, SUC_NE_lexicon_tags_2 },
    { 0x104aa7a4, SUC_NE_lexicon_tags_2 },
    { 0xda50a7a5, SUC_NE_lexicon_tags_2 },
    { 0xb3eaa7a6, SUC_NE_lexicon_tags_2 },
    { 0x235ca7a2, SUC_NE_lexicon_tags_0 },
    { 0xb406a7a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9afea7aa, SUC_NE_lexicon_tags_2 },
    { 0x405ca7ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe6a7b0, SUC_NE_lexicon_tags_2 },
    { 0xa5a2a7b1, SUC_NE_lexicon_tags_2 },
    { 0xbbf8a7b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15b8a7b4, SUC_NE_lexicon_tags_40 },
    { 0x4828a7b4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa9ea7b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbe2a7b9, SUC_NE_lexicon_tags_2 },
    { 0x3168a7ba, SUC_NE_lexicon_tags_2 },
    { 0xe1ea7bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe240a7bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9664a7c1, SUC_NE_lexicon_tags_2 },
    { 0x108a7c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08aa7c7, SUC_NE_lexicon_tags_2 },
    { 0xa378a7c8, SUC_NE_lexicon_tags_2 },
    { 0xb5b2a7c9, SUC_NE_lexicon_tags_2 },
    { 0x5dbaa7ca, SUC_NE_lexicon_tags_2 },
    { 0x5d9ca7cb, SUC_NE_lexicon_tags_2 },
    { 0x2610a7ca, SUC_NE_lexicon_tags_2 },
    { 0x7202a7cd, SUC_NE_lexicon_tags_2 },
    { 0xcf6ea7c7, SUC_NE_lexicon_tags_2 },
    { 0x880ca7c7, SUC_NE_lexicon_tags_2 },
    { 0xe1e6a7d0, SUC_NE_lexicon_tags_2 },
    { 0x5056a7d1, SUC_NE_lexicon_tags_2 },
    { 0x70d0a7c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ca7d5, SUC_NE_lexicon_tags_2 },
    { 0xfb6ca7d6, SUC_NE_lexicon_tags_2 },
    { 0x73bca7d7, SUC_NE_lexicon_tags_2 },
    { 0xb42ca7d8, SUC_NE_lexicon_tags_2 },
    { 0x3bbea7d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x639aa7db, SUC_NE_lexicon_tags_2 },
    { 0x253aa7dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc778a7df, SUC_NE_lexicon_tags_5 },
    { 0x4898a7df, SUC_NE_lexicon_tags_2 },
    { 0x6554a7df, SUC_NE_lexicon_tags_2 },
    { 0x8070a7e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757aa7e6, SUC_NE_lexicon_tags_2 },
    { 0x2144a7e6, SUC_NE_lexicon_tags_7 },
    { 0x5c38a7e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea4a7ec, SUC_NE_lexicon_tags_2 },
    { 0x7ce2a7ed, SUC_NE_lexicon_tags_2 },
    { 0x6eeca7ee, SUC_NE_lexicon_tags_13 },
    { 0xeb36a7ef, SUC_NE_lexicon_tags_2 },
    { 0x93f0a7f0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc364a7f5, SUC_NE_lexicon_tags_2 },
    { 0x4900a7f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4aa7fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce8a7fd, SUC_NE_lexicon_tags_2 },
    { 0x5b92a7fd, SUC_NE_lexicon_tags_2 },
    { 0x7f0ea7ff, SUC_NE_lexicon_tags_2 },
    { 0x83aa7fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d4a802, SUC_NE_lexicon_tags_2 },
    { 0x516a802, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3c6a805, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e4a809, SUC_NE_lexicon_tags_2 },
    { 0xabeaa809, SUC_NE_lexicon_tags_2 },
    { 0xf580a80b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e36a80e, SUC_NE_lexicon_tags_17 },
    { 0x4dd2a80f, SUC_NE_lexicon_tags_21 },
    { 0x7392a810, SUC_NE_lexicon_tags_2 },
    { 0xcaf4a80f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5144a814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb6a81a, SUC_NE_lexicon_tags_2 },
    { 0x18b4a81b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff4a81e, SUC_NE_lexicon_tags_2 },
    { 0x66aca81f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x644a821, SUC_NE_lexicon_tags_2 },
    { 0x60eca821, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c2a826, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ca82a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ea82f, SUC_NE_lexicon_tags_2 },
    { 0x3954a82f, SUC_NE_lexicon_tags_2 },
    { 0x3348a831, SUC_NE_lexicon_tags_2 },
    { 0x4792a830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b78a834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e0a837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x883ea83a, SUC_NE_lexicon_tags_2 },
    { 0xf8d8a83a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa2a83e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181ca843, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1038a845, SUC_NE_lexicon_tags_2 },
    { 0x29baa846, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x30a4a848, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2188a84e, SUC_NE_lexicon_tags_2 },
    { 0xc462a84f, SUC_NE_lexicon_tags_2 },
    { 0x42e2a850, SUC_NE_lexicon_tags_2 },
    { 0x8ffea851, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a42a858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de2a85b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41f0a85d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea76a862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff6a865, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32c6a867, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd7ea869, SUC_NE_lexicon_tags_2 },
    { 0x94a2a86a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74a86d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26a870, SUC_NE_lexicon_tags_3 },
    { 0xf6e4a871, SUC_NE_lexicon_tags_2 },
    { 0x78caa871, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4484a875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e2a879, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e02a87f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248a881, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc01aa883, SUC_NE_lexicon_tags_2 },
    { 0x2dc6a884, SUC_NE_lexicon_tags_2 },
    { 0xe5e4a883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2676a888, SUC_NE_lexicon_tags_2 },
    { 0x243aa888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2aa88c, SUC_NE_lexicon_tags_2 },
    { 0x15fea88c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd442a891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c8a894, SUC_NE_lexicon_tags_2 },
    { 0x8a92a895, SUC_NE_lexicon_tags_2 },
    { 0x4b7ca896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe086a898, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5818a89a, SUC_NE_lexicon_tags_4 },
    { 0xb954a89b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a90a89d, SUC_NE_lexicon_tags_3 },
    { 0xe888a89e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca6ca8a0, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f4a8a7, SUC_NE_lexicon_tags_2 },
    { 0x26f8a8a8, SUC_NE_lexicon_tags_17 },
    { 0xb35aa8a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2058a8ae, SUC_NE_lexicon_tags_2 },
    { 0x2ad0a8af, SUC_NE_lexicon_tags_2 },
    { 0xeb8ca8b0, SUC_NE_lexicon_tags_2 },
    { 0x10cca8b1, SUC_NE_lexicon_tags_2 },
    { 0xf316a8af, SUC_NE_lexicon_tags_2 },
    { 0xc3fea8af, SUC_NE_lexicon_tags_2 },
    { 0xfb18a8b1, SUC_NE_lexicon_tags_2 },
    { 0x1bb0a8b5, SUC_NE_lexicon_tags_1 },
    { 0x807ea8b6, SUC_NE_lexicon_tags_2 },
    { 0xbd86a8b7, SUC_NE_lexicon_tags_2 },
    { 0xa4e8a8b8, SUC_NE_lexicon_tags_2 },
    { 0x4d08a8b9, SUC_NE_lexicon_tags_2 },
    { 0x6e42a8ba, SUC_NE_lexicon_tags_7 },
    { 0x7b5ca8bb, SUC_NE_lexicon_tags_2 },
    { 0xa3caa8ba, SUC_NE_lexicon_tags_2 },
    { 0xd52aa8bd, SUC_NE_lexicon_tags_2 },
    { 0x40a8a8be, SUC_NE_lexicon_tags_2 },
    { 0x6c6a8be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b2a8c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b00a8c7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c12a8cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27fea8cf, SUC_NE_lexicon_tags_2 },
    { 0xa9b0a8cf, SUC_NE_lexicon_tags_2 },
    { 0x4dc4a8d1, SUC_NE_lexicon_tags_2 },
    { 0x3934a8d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x170ea8d4, SUC_NE_lexicon_tags_2 },
    { 0x131aa8d5, SUC_NE_lexicon_tags_2 },
    { 0x5298a8d6, SUC_NE_lexicon_tags_98 },
    { 0x281ea8d7, SUC_NE_lexicon_tags_2 },
    { 0xc350a8d8, SUC_NE_lexicon_tags_2 },
    { 0x6fe8a8d9, SUC_NE_lexicon_tags_2 },
    { 0x5202a8d8, SUC_NE_lexicon_tags_2 },
    { 0xc5faa8db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc32ca8dd, SUC_NE_lexicon_tags_2 },
    { 0xc08ea8de, SUC_NE_lexicon_tags_2 },
    { 0x4a82a8df, SUC_NE_lexicon_tags_2 },
    { 0x4908a8df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35b6a8e2, SUC_NE_lexicon_tags_2 },
    { 0x218a8e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65d2a8e5, SUC_NE_lexicon_tags_2 },
    { 0xc51aa8e6, SUC_NE_lexicon_tags_2 },
    { 0xc620a8e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95f2a8e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdca8ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa710a8ef, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce8a8f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c8a8fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429aa900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x260ea902, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b3aa904, SUC_NE_lexicon_tags_2 },
    { 0xfee2a905, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa206a908, SUC_NE_lexicon_tags_2 },
    { 0x7deaa908, SUC_NE_lexicon_tags_2 },
    { 0x9eeea90a, SUC_NE_lexicon_tags_2 },
    { 0x177ea90b, SUC_NE_lexicon_tags_24 },
    { 0xb34ea90c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b8a910, SUC_NE_lexicon_tags_4 },
    { 0xbbaa911, SUC_NE_lexicon_tags_2 },
    { 0xc55ea911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeca916, SUC_NE_lexicon_tags_2 },
    { 0xb3cca917, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdca91b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444a91f, SUC_NE_lexicon_tags_2 },
    { 0x6494a91f, SUC_NE_lexicon_tags_2 },
    { 0xee9aa921, SUC_NE_lexicon_tags_2 },
    { 0x7c46a91f, SUC_NE_lexicon_tags_2 },
    { 0xa360a91f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14a0a925, SUC_NE_lexicon_tags_2 },
    { 0xa74aa926, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65ea931, SUC_NE_lexicon_tags_2 },
    { 0xc176a932, SUC_NE_lexicon_tags_2 },
    { 0xda28a931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea44a936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe40a938, SUC_NE_lexicon_tags_3 },
    { 0x2558a939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdea940, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13aca942, SUC_NE_lexicon_tags_2 },
    { 0xd398a943, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc816a94a, SUC_NE_lexicon_tags_2 },
    { 0x9f04a94b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd4ca94d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad3ea94f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8a953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cd8a955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8452a959, SUC_NE_lexicon_tags_2 },
    { 0x9784a95a, SUC_NE_lexicon_tags_2 },
    { 0x7240a95b, SUC_NE_lexicon_tags_2 },
    { 0xbf0aa95a, SUC_NE_lexicon_tags_2 },
    { 0xb714a95a, SUC_NE_lexicon_tags_2 },
    { 0x548a95c, SUC_NE_lexicon_tags_2 },
    { 0x34b0a95f, SUC_NE_lexicon_tags_2 },
    { 0xfb4ca960, SUC_NE_lexicon_tags_2 },
    { 0xc038a961, SUC_NE_lexicon_tags_2 },
    { 0x5486a962, SUC_NE_lexicon_tags_2 },
    { 0xbbc6a960, SUC_NE_lexicon_tags_2 },
    { 0x3ccaa961, SUC_NE_lexicon_tags_2 },
    { 0x1840a95f, SUC_NE_lexicon_tags_2 },
    { 0xf2baa960, SUC_NE_lexicon_tags_2 },
    { 0xb03aa967, SUC_NE_lexicon_tags_2 },
    { 0x6c66a963, SUC_NE_lexicon_tags_1 },
    { 0xebd2a963, SUC_NE_lexicon_tags_2 },
    { 0xe8dea965, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe850a96e, SUC_NE_lexicon_tags_2 },
    { 0xba5ea96e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90aa974, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbaea976, SUC_NE_lexicon_tags_2 },
    { 0xa0d4a976, SUC_NE_lexicon_tags_2 },
    { 0x54baa978, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c2a97b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc902a990, SUC_NE_lexicon_tags_0 },
    { 0x1d38a991, SUC_NE_lexicon_tags_187 },
    { 0x196aa991, SUC_NE_lexicon_tags_3 },
    { 0xf888a990, SUC_NE_lexicon_tags_2 },
    { 0xc028a993, SUC_NE_lexicon_tags_2 },
    { 0xf908a995, SUC_NE_lexicon_tags_2 },
    { 0xf68ea992, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf894a998, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf54aa99a, SUC_NE_lexicon_tags_2 },
    { 0x8e4aa99b, SUC_NE_lexicon_tags_2 },
    { 0xbdd8a99c, SUC_NE_lexicon_tags_2 },
    { 0xa1c4a99d, SUC_NE_lexicon_tags_2 },
    { 0x3aaa99e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6a9a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472a9a5, SUC_NE_lexicon_tags_2 },
    { 0x846ea9a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdea9aa, SUC_NE_lexicon_tags_2 },
    { 0xe86ca9ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc118a9ae, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f4a9b1, SUC_NE_lexicon_tags_2 },
    { 0xe1aca9b1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527ea9b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eea9bc, SUC_NE_lexicon_tags_2 },
    { 0x79a8a9bd, SUC_NE_lexicon_tags_2 },
    { 0x2010a9be, SUC_NE_lexicon_tags_2 },
    { 0xf370a9bd, SUC_NE_lexicon_tags_2 },
    { 0xf72a9be, SUC_NE_lexicon_tags_2 },
    { 0x64aca9c1, SUC_NE_lexicon_tags_2 },
    { 0x7772a9c1, SUC_NE_lexicon_tags_2 },
    { 0xf998a9c3, SUC_NE_lexicon_tags_58 },
    { 0xcca4a9c3, SUC_NE_lexicon_tags_3 },
    { 0x734aa9c3, SUC_NE_lexicon_tags_7 },
    { 0xc6e0a9c6, SUC_NE_lexicon_tags_251 },
    { 0xd1f0a9c7, SUC_NE_lexicon_tags_2 },
    { 0xaf38a9c5, SUC_NE_lexicon_tags_2 },
    { 0x1794a9c5, SUC_NE_lexicon_tags_2 },
    { 0xf8f8a9ca, SUC_NE_lexicon_tags_2 },
    { 0x5712a9c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a12a9cd, SUC_NE_lexicon_tags_2 },
    { 0x9a0ca9cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473ca9d3, SUC_NE_lexicon_tags_2 },
    { 0x31daa9d4, SUC_NE_lexicon_tags_3 },
    { 0x5264a9d5, SUC_NE_lexicon_tags_2 },
    { 0xd23ea9d4, SUC_NE_lexicon_tags_17 },
    { 0xc7eaa9d6, SUC_NE_lexicon_tags_2 },
    { 0xf0cea9d5, SUC_NE_lexicon_tags_2 },
    { 0x294ca9d9, SUC_NE_lexicon_tags_2 },
    { 0x221ca9d5, SUC_NE_lexicon_tags_22 },
    { 0xa90aa9da, SUC_NE_lexicon_tags_2 },
    { 0x7d26a9d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6456a9e3, SUC_NE_lexicon_tags_2 },
    { 0xf55ca9e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8354a9e6, SUC_NE_lexicon_tags_2 },
    { 0x32c6a9e7, SUC_NE_lexicon_tags_2 },
    { 0x6f0a9e7, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b4a9ee, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0xab4aa9f0, SUC_NE_lexicon_tags_2 },
    { 0x3a98a9f1, SUC_NE_lexicon_tags_2 },
    { 0x93ca9f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e58a9f4, SUC_NE_lexicon_tags_2 },
    { 0x77b2a9f5, SUC_NE_lexicon_tags_2 },
    { 0x5e40a9f4, SUC_NE_lexicon_tags_2 },
    { 0x7326a9f4, SUC_NE_lexicon_tags_2 },
    { 0xa8a8a9f8, SUC_NE_lexicon_tags_2 },
    { 0x1794a9f8, SUC_NE_lexicon_tags_2 },
    { 0x316ca9fa, SUC_NE_lexicon_tags_2 },
    { 0x512a9fb, SUC_NE_lexicon_tags_2 },
    { 0x8934a9fc, SUC_NE_lexicon_tags_2 },
    { 0x2f2ea9fb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x212aaa00, SUC_NE_lexicon_tags_2 },
    { 0x37dcaa00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6152aa05, SUC_NE_lexicon_tags_2 },
    { 0xf92aa06, SUC_NE_lexicon_tags_2 },
    { 0xf992aa05, SUC_NE_lexicon_tags_7 },
    { 0xc998aa06, SUC_NE_lexicon_tags_2 },
    { 0x36f2aa09, SUC_NE_lexicon_tags_3 },
    { 0x4f1aaa0a, SUC_NE_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad6aa13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586caa18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4beaa1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9beaa1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a44aa20, SUC_NE_lexicon_tags_2 },
    { 0xd020aa20, SUC_NE_lexicon_tags_3 },
    { 0xa7caaa21, SUC_NE_lexicon_tags_2 },
    { 0x3868aa23, SUC_NE_lexicon_tags_2 },
    { 0x6a2aaa23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87f8aa26, SUC_NE_lexicon_tags_2 },
    { 0x66c2aa27, SUC_NE_lexicon_tags_2 },
    { 0x48deaa28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587caa2e, SUC_NE_lexicon_tags_2 },
    { 0xb23caa2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x126caa31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2feaa33, SUC_NE_lexicon_tags_19 },
    { 0xde4eaa33, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602aaa3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6520aa3e, SUC_NE_lexicon_tags_2 },
    { 0xfe58aa3f, SUC_NE_lexicon_tags_2 },
    { 0x1b98aa40, SUC_NE_lexicon_tags_2 },
    { 0xc8ecaa41, SUC_NE_lexicon_tags_3 },
    { 0xf5dcaa42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978aa47, SUC_NE_lexicon_tags_2 },
    { 0xe4c2aa47, SUC_NE_lexicon_tags_2 },
    { 0xb7f0aa49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2aa4d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc504aa56, SUC_NE_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a2aa5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca6aa63, SUC_NE_lexicon_tags_36 },
    { 0x3364aa64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b4aa67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x564eaa69, SUC_NE_lexicon_tags_2 },
    { 0x1f3aaa6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30caa71, SUC_NE_lexicon_tags_2 },
    { 0x8780aa71, SUC_NE_lexicon_tags_2 },
    { 0x86eaaa73, SUC_NE_lexicon_tags_58 },
    { 0x992aa72, SUC_NE_lexicon_tags_2 },
    { 0x1e1caa73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8aa79, SUC_NE_lexicon_tags_2 },
    { 0xa554aa79, SUC_NE_lexicon_tags_2 },
    { 0xb494aa7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4614aa7f, SUC_NE_lexicon_tags_2 },
    { 0x2a26aa80, SUC_NE_lexicon_tags_18 },
    { 0xabceaa81, SUC_NE_lexicon_tags_2 },
    { 0x88b0aa82, SUC_NE_lexicon_tags_2 },
    { 0x2016aa82, SUC_NE_lexicon_tags_2 },
    { 0x9cf2aa83, SUC_NE_lexicon_tags_2 },
    { 0x8a58aa83, SUC_NE_lexicon_tags_2 },
    { 0x9f7eaa85, SUC_NE_lexicon_tags_2 },
    { 0x1b7caa86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92aaa97, SUC_NE_lexicon_tags_2 },
    { 0x83d2aa98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae60aa9a, SUC_NE_lexicon_tags_2 },
    { 0xbbecaa9a, SUC_NE_lexicon_tags_0 },
    { 0xe63eaa9c, SUC_NE_lexicon_tags_10 },
    { 0x8e10aa9d, SUC_NE_lexicon_tags_2 },
    { 0xed30aa9c, SUC_NE_lexicon_tags_2 },
    { 0x9a8caa9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebe0aaa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7af0aaa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b6aaa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7eaaaa, SUC_NE_lexicon_tags_2 },
    { 0xe308aaab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2caaae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc6aab2, SUC_NE_lexicon_tags_2 },
    { 0xb254aab2, SUC_NE_lexicon_tags_2 },
    { 0xff94aab4, SUC_NE_lexicon_tags_2 },
    { 0x2274aab5, SUC_NE_lexicon_tags_2 },
    { 0x3f3caab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5c4aab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5b6aaba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb2aac1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe028aac3, SUC_NE_lexicon_tags_2 },
    { 0xc894aac3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b28aac7, SUC_NE_lexicon_tags_2 },
    { 0x1d8caac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cceaad1, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedcaad4, SUC_NE_lexicon_tags_2 },
    { 0xc238aad4, SUC_NE_lexicon_tags_2 },
    { 0x5a34aad6, SUC_NE_lexicon_tags_2 },
    { 0x13aaad7, SUC_NE_lexicon_tags_2 },
    { 0xe13eaad8, SUC_NE_lexicon_tags_2 },
    { 0x9d20aad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518aaadf, SUC_NE_lexicon_tags_2 },
    { 0x2fb4aadf, SUC_NE_lexicon_tags_2 },
    { 0x28e2aae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f0aae5, SUC_NE_lexicon_tags_23 },
    { 0xa1a4aae5, SUC_NE_lexicon_tags_2 },
    { 0x865eaae7, SUC_NE_lexicon_tags_2 },
    { 0x7feaaae6, SUC_NE_lexicon_tags_2 },
    { 0x6b60aae9, SUC_NE_lexicon_tags_1 },
    { 0x2a04aaea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee4aaaec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea8aaf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c6aaf8, SUC_NE_lexicon_tags_2 },
    { 0x3aaaaf9, SUC_NE_lexicon_tags_2 },
    { 0x225caaf8, SUC_NE_lexicon_tags_2 },
    { 0x4754aaf8, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x298aafd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82caaaff, SUC_NE_lexicon_tags_2 },
    { 0x7f3aaaff, SUC_NE_lexicon_tags_2 },
    { 0xd568ab00, SUC_NE_lexicon_tags_2 },
    { 0xc0e4ab02, SUC_NE_lexicon_tags_2 },
    { 0x894cab03, SUC_NE_lexicon_tags_2 },
    { 0xb0d0ab04, SUC_NE_lexicon_tags_2 },
    { 0xd45eab04, SUC_NE_lexicon_tags_2 },
    { 0xa1a8ab05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7eab0d, SUC_NE_lexicon_tags_2 },
    { 0xbe7eab0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f0ab1f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4ab22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c18ab24, SUC_NE_lexicon_tags_2 },
    { 0xfa34ab25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2910ab27, SUC_NE_lexicon_tags_2 },
    { 0x3908ab28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81eab2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x584aab2d, SUC_NE_lexicon_tags_2 },
    { 0xf7c2ab2e, SUC_NE_lexicon_tags_2 },
    { 0x1d78ab2e, SUC_NE_lexicon_tags_2 },
    { 0x20aaab30, SUC_NE_lexicon_tags_2 },
    { 0x5508ab31, SUC_NE_lexicon_tags_2 },
    { 0x376aab32, SUC_NE_lexicon_tags_2 },
    { 0xab94ab2d, SUC_NE_lexicon_tags_2 },
    { 0xc73aab34, SUC_NE_lexicon_tags_2 },
    { 0x9836ab30, SUC_NE_lexicon_tags_2 },
    { 0x1b1eab36, SUC_NE_lexicon_tags_17 },
    { 0xf9e0ab37, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x64b8ab39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bfaab3b, SUC_NE_lexicon_tags_2 },
    { 0x6752ab3b, SUC_NE_lexicon_tags_2 },
    { 0xdf60ab3b, SUC_NE_lexicon_tags_2 },
    { 0xc60cab3e, SUC_NE_lexicon_tags_2 },
    { 0x5f1aab3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x837eab41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc058ab45, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b84ab4a, SUC_NE_lexicon_tags_2 },
    { 0xf876ab4b, SUC_NE_lexicon_tags_2 },
    { 0x9812ab4b, SUC_NE_lexicon_tags_2 },
    { 0x15beab4d, SUC_NE_lexicon_tags_2 },
    { 0x54eaab4a, SUC_NE_lexicon_tags_2 },
    { 0xb3e6ab4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f34ab51, SUC_NE_lexicon_tags_2 },
    { 0x51b2ab52, SUC_NE_lexicon_tags_202 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39aab55, SUC_NE_lexicon_tags_2 },
    { 0x29bcab56, SUC_NE_lexicon_tags_2 },
    { 0xa48aab56, SUC_NE_lexicon_tags_2 },
    { 0x383cab58, SUC_NE_lexicon_tags_2 },
    { 0x8c9eab58, SUC_NE_lexicon_tags_2 },
    { 0x6ab5a, SUC_NE_lexicon_tags_2 },
    { 0xbed2ab5b, SUC_NE_lexicon_tags_2 },
    { 0xe038ab5c, SUC_NE_lexicon_tags_2 },
    { 0x921cab5b, SUC_NE_lexicon_tags_2 },
    { 0xe69aab5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3236ab60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79beab62, SUC_NE_lexicon_tags_2 },
    { 0x6c58ab63, SUC_NE_lexicon_tags_2 },
    { 0x13caab64, SUC_NE_lexicon_tags_13 },
    { 0xa790ab65, SUC_NE_lexicon_tags_5 },
    { 0xb6fcab65, SUC_NE_lexicon_tags_2 },
    { 0xabc4ab67, SUC_NE_lexicon_tags_2 },
    { 0x8220ab64, SUC_NE_lexicon_tags_2 },
    { 0x2298ab69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52f8ab6b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x17daab6d, SUC_NE_lexicon_tags_2 },
    { 0x9eab6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x447aab70, SUC_NE_lexicon_tags_2 },
    { 0x4e18ab71, SUC_NE_lexicon_tags_2 },
    { 0xd65eab71, SUC_NE_lexicon_tags_2 },
    { 0x8482ab73, SUC_NE_lexicon_tags_2 },
    { 0x76d8ab70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f4ab7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x554eab82, SUC_NE_lexicon_tags_2 },
    { 0x5adaab83, SUC_NE_lexicon_tags_2 },
    { 0x88eaab83, SUC_NE_lexicon_tags_2 },
    { 0x854aab85, SUC_NE_lexicon_tags_2 },
    { 0x6416ab82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aeaab88, SUC_NE_lexicon_tags_2 },
    { 0x1362ab88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x430aab8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16b4ab8e, SUC_NE_lexicon_tags_2 },
    { 0x5efcab8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4eab94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355cab99, SUC_NE_lexicon_tags_2 },
    { 0xd50aab99, SUC_NE_lexicon_tags_2 },
    { 0x2744ab99, SUC_NE_lexicon_tags_2 },
    { 0x6000ab9c, SUC_NE_lexicon_tags_2 },
    { 0x14aaab9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893caba5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x16dcaba8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2336abb0, SUC_NE_lexicon_tags_1 },
    { 0xc996abb0, SUC_NE_lexicon_tags_2 },
    { 0x800cabb2, SUC_NE_lexicon_tags_123 },
    { 0x87dcabb3, SUC_NE_lexicon_tags_2 },
    { 0x1a00abb4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x629eabb6, SUC_NE_lexicon_tags_3 },
    { 0xfc18abb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabbc, SUC_NE_lexicon_tags_2 },
    { 0x44a2abbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc334abc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x876cabc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3728abc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1b8abcb, SUC_NE_lexicon_tags_1 },
    { 0x5682abcc, SUC_NE_lexicon_tags_2 },
    { 0x56eabcb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3442abd1, SUC_NE_lexicon_tags_2 },
    { 0xf87aabd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf62abd4, SUC_NE_lexicon_tags_2 },
    { 0x20c0abd5, SUC_NE_lexicon_tags_2 },
    { 0x93eeabd6, SUC_NE_lexicon_tags_2 },
    { 0xe71cabd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a2abdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa576abdf, SUC_NE_lexicon_tags_2 },
    { 0xc216abe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5aabe4, SUC_NE_lexicon_tags_2 },
    { 0x8e02abe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c8abe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18aabea, SUC_NE_lexicon_tags_3 },
    { 0x6a08abeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91baabed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56aabf0, SUC_NE_lexicon_tags_2 },
    { 0xd504abf0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x283eabf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbc8abf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cabff, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x6378ac01, SUC_NE_lexicon_tags_2 },
    { 0xa23cac02, SUC_NE_lexicon_tags_0 },
    { 0x92c0ac03, SUC_NE_lexicon_tags_2 },
    { 0x4a42ac03, SUC_NE_lexicon_tags_2 },
    { 0xce96ac02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b6ac08, SUC_NE_lexicon_tags_2 },
    { 0x2282ac09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61aeac0b, SUC_NE_lexicon_tags_2 },
    { 0xa63cac0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2caac0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdceac10, SUC_NE_lexicon_tags_2 },
    { 0x535eac10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa41eac13, SUC_NE_lexicon_tags_2 },
    { 0x5918ac13, SUC_NE_lexicon_tags_7 },
    { 0xf3cac15, SUC_NE_lexicon_tags_2 },
    { 0xe510ac15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640ac1c, SUC_NE_lexicon_tags_2 },
    { 0x29c6ac1c, SUC_NE_lexicon_tags_17 },
    { 0x1170ac1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d72ac24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997cac28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd52ac2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2980ac31, SUC_NE_lexicon_tags_2 },
    { 0xfddaac32, SUC_NE_lexicon_tags_2 },
    { 0x23b4ac33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd094ac36, SUC_NE_lexicon_tags_2 },
    { 0x3e5aac36, SUC_NE_lexicon_tags_2 },
    { 0xadc0ac38, SUC_NE_lexicon_tags_2 },
    { 0xf566ac39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x959eac3b, SUC_NE_lexicon_tags_2 },
    { 0xd7caac3b, SUC_NE_lexicon_tags_2 },
    { 0x1348ac3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb868ac3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca8ac41, SUC_NE_lexicon_tags_2 },
    { 0xb03cac41, SUC_NE_lexicon_tags_2 },
    { 0xf714ac43, SUC_NE_lexicon_tags_4 },
    { 0xc838ac44, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd04ac48, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd932ac4b, SUC_NE_lexicon_tags_2 },
    { 0x5e6eac4c, SUC_NE_lexicon_tags_2 },
    { 0x8390ac4d, SUC_NE_lexicon_tags_2 },
    { 0x5ae6ac4d, SUC_NE_lexicon_tags_2 },
    { 0x77e0ac4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6590ac52, SUC_NE_lexicon_tags_2 },
    { 0xac76ac52, SUC_NE_lexicon_tags_2 },
    { 0xf95aac54, SUC_NE_lexicon_tags_2 },
    { 0x79a4ac55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa74aac57, SUC_NE_lexicon_tags_2 },
    { 0xfb34ac58, SUC_NE_lexicon_tags_2 },
    { 0xf9f6ac58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf26ac61, SUC_NE_lexicon_tags_2 },
    { 0x367cac61, SUC_NE_lexicon_tags_2 },
    { 0x6bc4ac61, SUC_NE_lexicon_tags_2 },
    { 0x3000ac64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39aaac66, SUC_NE_lexicon_tags_2 },
    { 0x895aac67, SUC_NE_lexicon_tags_36 },
    { 0x62d4ac68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eaac6b, SUC_NE_lexicon_tags_2 },
    { 0x69a6ac6b, SUC_NE_lexicon_tags_2 },
    { 0x8202ac6b, SUC_NE_lexicon_tags_3 },
    { 0xf86ac6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4faac71, SUC_NE_lexicon_tags_2 },
    { 0xfc44ac71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x306eac75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b58ac77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf42ac79, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9884ac7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38d2ac7e, SUC_NE_lexicon_tags_2 },
    { 0x6c98ac7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde6ac81, SUC_NE_lexicon_tags_31 },
    { 0x2b66ac82, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf308ac84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec4ac86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da6ac89, SUC_NE_lexicon_tags_2 },
    { 0x32f4ac8a, SUC_NE_lexicon_tags_2 },
    { 0xa80aac8a, SUC_NE_lexicon_tags_2 },
    { 0x8f44ac8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802eac91, SUC_NE_lexicon_tags_2 },
    { 0xd6aaac91, SUC_NE_lexicon_tags_7 },
    { 0x950eac93, SUC_NE_lexicon_tags_7 },
    { 0x4becac94, SUC_NE_lexicon_tags_2 },
    { 0x11d6ac93, SUC_NE_lexicon_tags_2 },
    { 0x6622ac92, SUC_NE_lexicon_tags_2 },
    { 0xd46aac97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62cac9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf930aca1, SUC_NE_lexicon_tags_2 },
    { 0xedbcaca2, SUC_NE_lexicon_tags_2 },
    { 0xdbbcaca3, SUC_NE_lexicon_tags_2 },
    { 0xa07caca3, SUC_NE_lexicon_tags_2 },
    { 0xb714aca4, SUC_NE_lexicon_tags_2 },
    { 0x7410aca5, SUC_NE_lexicon_tags_2 },
    { 0xec24aca7, SUC_NE_lexicon_tags_2 },
    { 0x9348aca8, SUC_NE_lexicon_tags_3 },
    { 0xfb8aca9, SUC_NE_lexicon_tags_2 },
    { 0xc3baacaa, SUC_NE_lexicon_tags_2 },
    { 0x2e9caca9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa846acad, SUC_NE_lexicon_tags_2 },
    { 0xf40acae, SUC_NE_lexicon_tags_2 },
    { 0xfa3eacad, SUC_NE_lexicon_tags_2 },
    { 0x4324acb0, SUC_NE_lexicon_tags_2 },
    { 0xcf66acb1, SUC_NE_lexicon_tags_2 },
    { 0x291eacb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x474aacb4, SUC_NE_lexicon_tags_2 },
    { 0x3054acb5, SUC_NE_lexicon_tags_2 },
    { 0x6436acb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe388acb8, SUC_NE_lexicon_tags_2 },
    { 0xbe08acb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70b8acbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18fcacbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea32acbf, SUC_NE_lexicon_tags_2 },
    { 0x78f2acbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c0acc3, SUC_NE_lexicon_tags_2 },
    { 0x1532acc4, SUC_NE_lexicon_tags_2 },
    { 0x29ccacc5, SUC_NE_lexicon_tags_2 },
    { 0x48f8acc6, SUC_NE_lexicon_tags_2 },
    { 0xcbfcacc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd496acca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2762acce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b6acd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78c6acd6, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9aacd9, SUC_NE_lexicon_tags_2 },
    { 0x59faacd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eeacde, SUC_NE_lexicon_tags_2 },
    { 0xdcf0acdf, SUC_NE_lexicon_tags_2 },
    { 0x20d2ace0, SUC_NE_lexicon_tags_2 },
    { 0x6686ace1, SUC_NE_lexicon_tags_13 },
    { 0x5e4aace1, SUC_NE_lexicon_tags_2 },
    { 0x520ace3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3cace7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x9614ace9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe94aceb, SUC_NE_lexicon_tags_2 },
    { 0x9518acec, SUC_NE_lexicon_tags_2 },
    { 0x2dfcaceb, SUC_NE_lexicon_tags_2 },
    { 0xe742aced, SUC_NE_lexicon_tags_2 },
    { 0x6608acee, SUC_NE_lexicon_tags_2 },
    { 0x147aaceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1216acf3, SUC_NE_lexicon_tags_32 },
    { 0x478acf4, SUC_NE_lexicon_tags_2 },
    { 0xd870acf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76acf8, SUC_NE_lexicon_tags_7 },
    { 0xc762acf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x440eacfb, SUC_NE_lexicon_tags_2 },
    { 0x18acfc, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0xe824acfe, SUC_NE_lexicon_tags_2 },
    { 0xffacacff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128ead05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab82ad08, SUC_NE_lexicon_tags_2 },
    { 0x7e32ad09, SUC_NE_lexicon_tags_2 },
    { 0x2606ad09, SUC_NE_lexicon_tags_11 },
    { 0x800ead0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x162ead0d, SUC_NE_lexicon_tags_2 },
    { 0x3e3aad0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd624ad10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d8ad13, SUC_NE_lexicon_tags_2 },
    { 0xaddcad13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1cad17, SUC_NE_lexicon_tags_2 },
    { 0x749cad17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fb8ad1a, SUC_NE_lexicon_tags_2 },
    { 0xda90ad1b, SUC_NE_lexicon_tags_2 },
    { 0x98aaad1b, SUC_NE_lexicon_tags_2 },
    { 0x579cad1d, SUC_NE_lexicon_tags_2 },
    { 0x440cad1d, SUC_NE_lexicon_tags_2 },
    { 0x78b2ad1d, SUC_NE_lexicon_tags_2 },
    { 0xf5eead1f, SUC_NE_lexicon_tags_2 },
    { 0x2498ad20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efead26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5770ad2b, SUC_NE_lexicon_tags_2 },
    { 0xbf0ead2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x329ead2e, SUC_NE_lexicon_tags_2 },
    { 0x8a02ad2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2340ad33, SUC_NE_lexicon_tags_2 },
    { 0xef76ad34, SUC_NE_lexicon_tags_2 },
    { 0xf258ad35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9616ad37, SUC_NE_lexicon_tags_30 },
    { 0x67ecad37, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8354ad3a, SUC_NE_lexicon_tags_7 },
    { 0xf74cad3b, SUC_NE_lexicon_tags_2 },
    { 0x318ead3a, SUC_NE_lexicon_tags_2 },
    { 0xd772ad3d, SUC_NE_lexicon_tags_2 },
    { 0x9528ad3e, SUC_NE_lexicon_tags_2 },
    { 0xad4aad3f, SUC_NE_lexicon_tags_2 },
    { 0x43b6ad3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14c6ad42, SUC_NE_lexicon_tags_2 },
    { 0xd382ad42, SUC_NE_lexicon_tags_2 },
    { 0x9266ad44, SUC_NE_lexicon_tags_2 },
    { 0x2518ad45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4cad48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d6ad4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d6ad51, SUC_NE_lexicon_tags_2 },
    { 0x38f2ad52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01cad57, SUC_NE_lexicon_tags_2 },
    { 0x2990ad57, SUC_NE_lexicon_tags_2 },
    { 0x9c24ad59, SUC_NE_lexicon_tags_2 },
    { 0x8196ad5a, SUC_NE_lexicon_tags_2 },
    { 0x9650ad5b, SUC_NE_lexicon_tags_2 },
    { 0xafcead5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd6ad62, SUC_NE_lexicon_tags_2 },
    { 0x21b4ad63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1618ad66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6458ad69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4600ad6c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a4ad72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37aad76, SUC_NE_lexicon_tags_4 },
    { 0xb494ad77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34ad7d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be0ad80, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6db0ad82, SUC_NE_lexicon_tags_2 },
    { 0x9220ad83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b8ad86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2ad8d, SUC_NE_lexicon_tags_2 },
    { 0x3f3cad8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe00aad90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1d2ad92, SUC_NE_lexicon_tags_2 },
    { 0xdceaad93, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7034ad97, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc99ead99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862eada1, SUC_NE_lexicon_tags_2 },
    { 0xe516ada2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6aada5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1820adae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8872adb2, SUC_NE_lexicon_tags_2 },
    { 0x5018adb3, SUC_NE_lexicon_tags_2 },
    { 0x8a0cadb4, SUC_NE_lexicon_tags_32 },
    { 0x633cadb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x612eadb8, SUC_NE_lexicon_tags_2 },
    { 0x102cadb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f38adbd, SUC_NE_lexicon_tags_2 },
    { 0x6dd0adbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642aadc3, SUC_NE_lexicon_tags_2 },
    { 0x72e4adc4, SUC_NE_lexicon_tags_2 },
    { 0xbd8aadc5, SUC_NE_lexicon_tags_2 },
    { 0x3314adc6, SUC_NE_lexicon_tags_2 },
    { 0xf41cadc5, SUC_NE_lexicon_tags_2 },
    { 0x4b56adc5, SUC_NE_lexicon_tags_2 },
    { 0x154aadc9, SUC_NE_lexicon_tags_4 },
    { 0xbaaadca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc60adcc, SUC_NE_lexicon_tags_2 },
    { 0x6e4aadcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47eaadcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x961eadd1, SUC_NE_lexicon_tags_2 },
    { 0x2452add2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c2cadd4, SUC_NE_lexicon_tags_2 },
    { 0xea36add4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3238add9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaddc, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x934caddf, SUC_NE_lexicon_tags_2 },
    { 0x4792addf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4292ade3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x877eade5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab8adea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f16aded, SUC_NE_lexicon_tags_2 },
    { 0x1fb8adee, SUC_NE_lexicon_tags_4 },
    { 0x2c0adef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9962adf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c2adf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb032adf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4756adfa, SUC_NE_lexicon_tags_2 },
    { 0x26c2adfb, SUC_NE_lexicon_tags_2 },
    { 0x60f8adfb, SUC_NE_lexicon_tags_2 },
    { 0x8fccadfd, SUC_NE_lexicon_tags_4 },
    { 0x1c42adfe, SUC_NE_lexicon_tags_2 },
    { 0x9deaadfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x305eae01, SUC_NE_lexicon_tags_2 },
    { 0xf432ae01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fc4ae04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeeae0a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb3caae0c, SUC_NE_lexicon_tags_2 },
    { 0x7026ae0c, SUC_NE_lexicon_tags_2 },
    { 0x9becae0c, SUC_NE_lexicon_tags_2 },
    { 0x10b8ae0f, SUC_NE_lexicon_tags_2 },
    { 0xef10ae10, SUC_NE_lexicon_tags_2 },
    { 0xa128ae11, SUC_NE_lexicon_tags_2 },
    { 0x92e2ae12, SUC_NE_lexicon_tags_2 },
    { 0xd712ae0c, SUC_NE_lexicon_tags_2 },
    { 0xbef2ae0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f42ae17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c22ae19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f40ae22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5c8ae24, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x2488ae26, SUC_NE_lexicon_tags_2 },
    { 0x43c4ae27, SUC_NE_lexicon_tags_2 },
    { 0x3c84ae28, SUC_NE_lexicon_tags_2 },
    { 0x7f4eae26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x660cae2c, SUC_NE_lexicon_tags_2 },
    { 0xd512ae2d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7d72ae2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ad4ae31, SUC_NE_lexicon_tags_2 },
    { 0xcba2ae32, SUC_NE_lexicon_tags_4 },
    { 0x380cae33, SUC_NE_lexicon_tags_102 },
    { 0xa18aae33, SUC_NE_lexicon_tags_2 },
    { 0x4f92ae35, SUC_NE_lexicon_tags_203 },
    { 0xae0ae36, SUC_NE_lexicon_tags_2 },
    { 0x827aae37, SUC_NE_lexicon_tags_2 },
    { 0xfb1eae37, SUC_NE_lexicon_tags_2 },
    { 0x49eaae39, SUC_NE_lexicon_tags_4 },
    { 0x9de4ae35, SUC_NE_lexicon_tags_2 },
    { 0xc770ae3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c4ae3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28ae42, SUC_NE_lexicon_tags_2 },
    { 0xba50ae43, SUC_NE_lexicon_tags_2 },
    { 0xf0d0ae42, SUC_NE_lexicon_tags_2 },
    { 0x382ae45, SUC_NE_lexicon_tags_2 },
    { 0xf944ae45, SUC_NE_lexicon_tags_2 },
    { 0xc462ae47, SUC_NE_lexicon_tags_2 },
    { 0x3296ae48, SUC_NE_lexicon_tags_2 },
    { 0x288eae49, SUC_NE_lexicon_tags_7 },
    { 0xb4f6ae45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x511eae4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4682ae4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8cae52, SUC_NE_lexicon_tags_2 },
    { 0x80f6ae53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24aae58, SUC_NE_lexicon_tags_2 },
    { 0xf890ae59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e86ae5b, SUC_NE_lexicon_tags_0 },
    { 0x5f12ae5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1eae5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6390ae61, SUC_NE_lexicon_tags_2 },
    { 0xdc46ae62, SUC_NE_lexicon_tags_2 },
    { 0x3e46ae62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312eae67, SUC_NE_lexicon_tags_2 },
    { 0xc5eaae68, SUC_NE_lexicon_tags_7 },
    { 0xa86ae67, SUC_NE_lexicon_tags_2 },
    { 0x7b62ae6a, SUC_NE_lexicon_tags_3 },
    { 0x8014ae6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f64ae70, SUC_NE_lexicon_tags_2 },
    { 0x433aae71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c32ae73, SUC_NE_lexicon_tags_2 },
    { 0xf83aae74, SUC_NE_lexicon_tags_2 },
    { 0x95f2ae73, SUC_NE_lexicon_tags_2 },
    { 0x3318ae75, SUC_NE_lexicon_tags_2 },
    { 0xcc76ae76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4ae7a, SUC_NE_lexicon_tags_19 },
    { 0xd5deae7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ec0ae7d, SUC_NE_lexicon_tags_34 },
    { 0xff08ae7e, SUC_NE_lexicon_tags_2 },
    { 0x9e4eae7f, SUC_NE_lexicon_tags_2 },
    { 0xe94ae7e, SUC_NE_lexicon_tags_2 },
    { 0x2006ae7d, SUC_NE_lexicon_tags_2 },
    { 0x9a62ae7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9eae84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbaae88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148aae8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed92ae8f, SUC_NE_lexicon_tags_2 },
    { 0x3cd4ae90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640cae9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67baae9f, SUC_NE_lexicon_tags_7 },
    { 0xf3eeaea0, SUC_NE_lexicon_tags_7 },
    { 0x8164aea1, SUC_NE_lexicon_tags_2 },
    { 0xa530aea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a6aea5, SUC_NE_lexicon_tags_2 },
    { 0xac8aaea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa26aea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4ecaeaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa8aead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c8aeb3, SUC_NE_lexicon_tags_2 },
    { 0xc310aeb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5174aeb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d14aeb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef16aebb, SUC_NE_lexicon_tags_2 },
    { 0xe648aebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92faaebe, SUC_NE_lexicon_tags_2 },
    { 0x80feaebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0aaec4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc542aec7, SUC_NE_lexicon_tags_2 },
    { 0xcd6aaec7, SUC_NE_lexicon_tags_2 },
    { 0x13eeaec9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34caece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf2eaed0, SUC_NE_lexicon_tags_2 },
    { 0x10deaed1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c4aed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7baaedc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a68aee4, SUC_NE_lexicon_tags_2 },
    { 0x985caee5, SUC_NE_lexicon_tags_2 },
    { 0x23feaee6, SUC_NE_lexicon_tags_14 },
    { 0xb53caee7, SUC_NE_lexicon_tags_2 },
    { 0xdcc0aee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9852aeea, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4dcaeec, SUC_NE_lexicon_tags_2 },
    { 0xdb46aeed, SUC_NE_lexicon_tags_2 },
    { 0xe644aeed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b2aef1, SUC_NE_lexicon_tags_2 },
    { 0x416aaef2, SUC_NE_lexicon_tags_3 },
    { 0xe85caef2, SUC_NE_lexicon_tags_2 },
    { 0xce88aef3, SUC_NE_lexicon_tags_3 },
    { 0x38aef3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e76aef7, SUC_NE_lexicon_tags_2 },
    { 0x2abeaef8, SUC_NE_lexicon_tags_2 },
    { 0x7a2caef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70deaefc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47faaeff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fbaaf01, SUC_NE_lexicon_tags_2 },
    { 0x1754af01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd956af05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa48af0b, SUC_NE_lexicon_tags_18 },
    { 0x1b30af0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8008af0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11caaf10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0eaf15, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f4af1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d0af1e, SUC_NE_lexicon_tags_2 },
    { 0x7f48af1f, SUC_NE_lexicon_tags_2 },
    { 0x7918af20, SUC_NE_lexicon_tags_2 },
    { 0xc0ccaf21, SUC_NE_lexicon_tags_2 },
    { 0xb61caf22, SUC_NE_lexicon_tags_2 },
    { 0xf2f8af22, SUC_NE_lexicon_tags_2 },
    { 0x30e4af24, SUC_NE_lexicon_tags_2 },
    { 0x440caf25, SUC_NE_lexicon_tags_2 },
    { 0x41eaaf26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de4af2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x548af2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50f6af31, SUC_NE_lexicon_tags_2 },
    { 0x66e6af32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e0af34, SUC_NE_lexicon_tags_2 },
    { 0x818aaf35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3aaf39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7eaf3c, SUC_NE_lexicon_tags_2 },
    { 0x2446af3d, SUC_NE_lexicon_tags_2 },
    { 0x3518af3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aeaaf40, SUC_NE_lexicon_tags_2 },
    { 0x68f2af41, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f46af43, SUC_NE_lexicon_tags_2 },
    { 0xa7d0af44, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf172af47, SUC_NE_lexicon_tags_2 },
    { 0xcff6af47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b78af4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12caf4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x112af51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8feaaf55, SUC_NE_lexicon_tags_17 },
    { 0xb5eaaf55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87beaf5c, SUC_NE_lexicon_tags_2 },
    { 0xccfcaf5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d44af5f, SUC_NE_lexicon_tags_2 },
    { 0x9ad2af5f, SUC_NE_lexicon_tags_2 },
    { 0x7b3aaf61, SUC_NE_lexicon_tags_23 },
    { 0x4a2caf61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd82aaf64, SUC_NE_lexicon_tags_2 },
    { 0xa7b0af65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc98af67, SUC_NE_lexicon_tags_2 },
    { 0x994eaf67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x368caf6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe18af6d, SUC_NE_lexicon_tags_2 },
    { 0x7c3aaf6d, SUC_NE_lexicon_tags_2 },
    { 0xf74eaf6f, SUC_NE_lexicon_tags_2 },
    { 0x460aaf6d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755caf74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf294af80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6af83, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0x8a24af85, SUC_NE_lexicon_tags_2 },
    { 0x94d2af86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3172af89, SUC_NE_lexicon_tags_17 },
    { 0xb0b2af89, SUC_NE_lexicon_tags_2 },
    { 0xe140af8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1176af8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a6af91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9aaf95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb4eaf97, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc4af9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf2afa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c4afa4, SUC_NE_lexicon_tags_2 },
    { 0x6b66afa5, SUC_NE_lexicon_tags_2 },
    { 0x4d62afa5, SUC_NE_lexicon_tags_2 },
    { 0xaebaafa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x762aafa9, SUC_NE_lexicon_tags_2 },
    { 0xd99eafaa, SUC_NE_lexicon_tags_2 },
    { 0xc356afab, SUC_NE_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6888afb3, SUC_NE_lexicon_tags_2 },
    { 0x401eafb4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x92feafb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed02afb8, SUC_NE_lexicon_tags_2 },
    { 0xdabcafb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc70afbc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cafc5, SUC_NE_lexicon_tags_2 },
    { 0x9cceafc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x920aafc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36aafcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdecaafcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd93cafd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd452afd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x434cafd6, SUC_NE_lexicon_tags_2 },
    { 0x167aafd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8aeafd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3634afdd, SUC_NE_lexicon_tags_2 },
    { 0x9b14afde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e4afe4, SUC_NE_lexicon_tags_5 },
    { 0x229aafe4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa990afe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb780afea, SUC_NE_lexicon_tags_11 },
    { 0xa9e0afeb, SUC_NE_lexicon_tags_25 },
    { 0xb5b6afeb, SUC_NE_lexicon_tags_2 },
    { 0x1da6afed, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xea0cafef, SUC_NE_lexicon_tags_2 },
    { 0x838aafef, SUC_NE_lexicon_tags_2 },
    { 0x77faaff1, SUC_NE_lexicon_tags_2 },
    { 0x63d0aff0, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0x8b58aff4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff0affd, SUC_NE_lexicon_tags_0 },
    { 0x9bdeaffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8434b000, SUC_NE_lexicon_tags_2 },
    { 0x5d7eb000, SUC_NE_lexicon_tags_316 },
    { 0x979eb000, SUC_NE_lexicon_tags_2 },
    { 0xaba8b001, SUC_NE_lexicon_tags_7 },
    { 0x4732b000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04cb007, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x888cb00a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb15eb00c, SUC_NE_lexicon_tags_19 },
    { 0x4814b00c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x22feb00f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9eab011, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa548b018, SUC_NE_lexicon_tags_2 },
    { 0x2b6b018, SUC_NE_lexicon_tags_0 },
    { 0xef4eb01a, SUC_NE_lexicon_tags_2 },
    { 0x5c14b01b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7936b01f, SUC_NE_lexicon_tags_2 },
    { 0xf9f2b020, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6b023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf960b025, SUC_NE_lexicon_tags_2 },
    { 0xa2ceb026, SUC_NE_lexicon_tags_2 },
    { 0xe088b027, SUC_NE_lexicon_tags_4 },
    { 0xdbacb026, SUC_NE_lexicon_tags_2 },
    { 0xa70b028, SUC_NE_lexicon_tags_2 },
    { 0x6a56b025, SUC_NE_lexicon_tags_2 },
    { 0x541cb02b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6292b02d, SUC_NE_lexicon_tags_2 },
    { 0x5a66b02d, SUC_NE_lexicon_tags_2 },
    { 0xeaacb02f, SUC_NE_lexicon_tags_2 },
    { 0x9db2b02e, SUC_NE_lexicon_tags_2 },
    { 0xef0ab02e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebcb034, SUC_NE_lexicon_tags_2 },
    { 0xc8b8b035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2696b037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3f6b039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf326b03c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x324cb03e, SUC_NE_lexicon_tags_2 },
    { 0x1d1eb03f, SUC_NE_lexicon_tags_2 },
    { 0x67b4b03f, SUC_NE_lexicon_tags_2 },
    { 0x9404b041, SUC_NE_lexicon_tags_2 },
    { 0x582cb042, SUC_NE_lexicon_tags_2 },
    { 0x5c04b043, SUC_NE_lexicon_tags_5 },
    { 0x21ecb044, SUC_NE_lexicon_tags_2 },
    { 0x3776b045, SUC_NE_lexicon_tags_2 },
    { 0xefccb046, SUC_NE_lexicon_tags_215 },
    { 0x64b2b047, SUC_NE_lexicon_tags_2 },
    { 0x7f72b048, SUC_NE_lexicon_tags_4 },
    { 0x96a8b049, SUC_NE_lexicon_tags_2 },
    { 0xea30b04a, SUC_NE_lexicon_tags_14 },
    { 0xe7ceb04b, SUC_NE_lexicon_tags_2 },
    { 0x2a52b04c, SUC_NE_lexicon_tags_2 },
    { 0xeccb04d, SUC_NE_lexicon_tags_2 },
    { 0x6e2cb040, SUC_NE_lexicon_tags_2 },
    { 0x6388b04f, SUC_NE_lexicon_tags_2 },
    { 0x5964b050, SUC_NE_lexicon_tags_2 },
    { 0x93ecb051, SUC_NE_lexicon_tags_2 },
    { 0x2edab052, SUC_NE_lexicon_tags_2 },
    { 0x800b053, SUC_NE_lexicon_tags_2 },
    { 0x443cb054, SUC_NE_lexicon_tags_2 },
    { 0xeca0b055, SUC_NE_lexicon_tags_2 },
    { 0xf1c8b055, SUC_NE_lexicon_tags_2 },
    { 0x3cf0b057, SUC_NE_lexicon_tags_2 },
    { 0x7310b040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cdab05a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671eb062, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c38b065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae26b067, SUC_NE_lexicon_tags_2 },
    { 0xd22eb068, SUC_NE_lexicon_tags_2 },
    { 0x8bc0b069, SUC_NE_lexicon_tags_2 },
    { 0xbceeb067, SUC_NE_lexicon_tags_2 },
    { 0x3146b06b, SUC_NE_lexicon_tags_2 },
    { 0xc6b2b06b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x76feb06e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3e72b070, SUC_NE_lexicon_tags_2 },
    { 0xa124b071, SUC_NE_lexicon_tags_2 },
    { 0x93f8b071, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1238b074, SUC_NE_lexicon_tags_2 },
    { 0x6316b075, SUC_NE_lexicon_tags_2 },
    { 0xcc6ab075, SUC_NE_lexicon_tags_7 },
    { 0x3b02b077, SUC_NE_lexicon_tags_1 },
    { 0x3586b078, SUC_NE_lexicon_tags_2 },
    { 0xacccb079, SUC_NE_lexicon_tags_2 },
    { 0x4dc0b074, SUC_NE_lexicon_tags_2 },
    { 0x5e98b077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b84b082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x535ab084, SUC_NE_lexicon_tags_2 },
    { 0x39e2b085, SUC_NE_lexicon_tags_2 },
    { 0x12cab084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4cb08a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x451ab08c, SUC_NE_lexicon_tags_3 },
    { 0xe388b08c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6de2b08f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf024b096, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1648b099, SUC_NE_lexicon_tags_2 },
    { 0x663cb099, SUC_NE_lexicon_tags_4 },
    { 0x9b4cb09b, SUC_NE_lexicon_tags_2 },
    { 0x3b14b09c, SUC_NE_lexicon_tags_7 },
    { 0xf8acb09d, SUC_NE_lexicon_tags_2 },
    { 0x6d88b09e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5eb0a4, SUC_NE_lexicon_tags_2 },
    { 0x8b2b0a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb366b0a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e24b0ab, SUC_NE_lexicon_tags_2 },
    { 0x72d8b0ac, SUC_NE_lexicon_tags_2 },
    { 0x8890b0ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50c0b0af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34aab0b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523ab0b8, SUC_NE_lexicon_tags_2 },
    { 0x10d4b0b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26b0bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd052b0c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dab0c6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dab0c9, SUC_NE_lexicon_tags_0 },
    { 0xa176b0ca, SUC_NE_lexicon_tags_2 },
    { 0xb388b0cb, SUC_NE_lexicon_tags_2 },
    { 0x4b40b0cc, SUC_NE_lexicon_tags_2 },
    { 0xb7a6b0cc, SUC_NE_lexicon_tags_7 },
    { 0x786cb0cc, SUC_NE_lexicon_tags_2 },
    { 0xcd28b0cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb260b0d1, SUC_NE_lexicon_tags_2 },
    { 0xe014b0d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9338b0d9, SUC_NE_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x6058b0dc, SUC_NE_lexicon_tags_2 },
    { 0x7e9cb0dd, SUC_NE_lexicon_tags_2 },
    { 0xb4ecb0de, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfff2b0e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15deb0e5, SUC_NE_lexicon_tags_2 },
    { 0x89b8b0e6, SUC_NE_lexicon_tags_2 },
    { 0x7bacb0e7, SUC_NE_lexicon_tags_2 },
    { 0xc400b0e7, SUC_NE_lexicon_tags_2 },
    { 0xc6dab0e9, SUC_NE_lexicon_tags_1 },
    { 0x3bc2b0e5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbeb0ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b74b0f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4b0f4, SUC_NE_lexicon_tags_2 },
    { 0x9378b0f5, SUC_NE_lexicon_tags_0 },
    { 0x2fbab0f4, SUC_NE_lexicon_tags_2 },
    { 0xadf0b0f7, SUC_NE_lexicon_tags_178 },
    { 0xd214b0f8, SUC_NE_lexicon_tags_2 },
    { 0x2f18b0f8, SUC_NE_lexicon_tags_7 },
    { 0x1c5cb0fa, SUC_NE_lexicon_tags_2 },
    { 0xe24cb0f5, SUC_NE_lexicon_tags_7 },
    { 0x8bd2b0fb, SUC_NE_lexicon_tags_2 },
    { 0x6000b0fd, SUC_NE_lexicon_tags_2 },
    { 0x1074b0f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907eb104, SUC_NE_lexicon_tags_2 },
    { 0x778b105, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4cb108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bcb10b, SUC_NE_lexicon_tags_2 },
    { 0x3a26b10c, SUC_NE_lexicon_tags_2 },
    { 0x4e30b10c, SUC_NE_lexicon_tags_2 },
    { 0x1dfeb10e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee2b111, SUC_NE_lexicon_tags_2 },
    { 0x2da4b111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb11cb114, SUC_NE_lexicon_tags_2 },
    { 0x98dcb115, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a22b118, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15a4b11a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdeb11d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6880b120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e12b124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x881eb127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4850b129, SUC_NE_lexicon_tags_2 },
    { 0xee3eb12a, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcccb12e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ab137, SUC_NE_lexicon_tags_2 },
    { 0xb9d2b138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f12b13b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd776b140, SUC_NE_lexicon_tags_2 },
    { 0x427ab141, SUC_NE_lexicon_tags_87 },
    { 0xf20ab140, SUC_NE_lexicon_tags_2 },
    { 0x2f9ab142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd2ab145, SUC_NE_lexicon_tags_2 },
    { 0xef8cb146, SUC_NE_lexicon_tags_2 },
    { 0x327eb147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2b14c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a20b14e, SUC_NE_lexicon_tags_2 },
    { 0xe018b14f, SUC_NE_lexicon_tags_2 },
    { 0x9c02b14e, SUC_NE_lexicon_tags_2 },
    { 0xb04ab151, SUC_NE_lexicon_tags_2 },
    { 0xb8b2b152, SUC_NE_lexicon_tags_2 },
    { 0xd7beb153, SUC_NE_lexicon_tags_7 },
    { 0x6454b154, SUC_NE_lexicon_tags_277 },
    { 0x8cccb155, SUC_NE_lexicon_tags_2 },
    { 0x64acb14f, SUC_NE_lexicon_tags_19 },
    { 0x60c0b151, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2266b159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf102b15b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5542b162, SUC_NE_lexicon_tags_2 },
    { 0xc2c6b162, SUC_NE_lexicon_tags_2 },
    { 0xe298b164, SUC_NE_lexicon_tags_200 },
    { 0, NULL },
    { 0xdd9eb166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2feeb168, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0x52dab16a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d3ab16c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63e2b16e, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4772b172, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8eb17e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2ab184, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x953ab186, SUC_NE_lexicon_tags_2 },
    { 0x90ceb187, SUC_NE_lexicon_tags_2 },
    { 0x1b52b186, SUC_NE_lexicon_tags_2 },
    { 0xea36b189, SUC_NE_lexicon_tags_2 },
    { 0xb888b189, SUC_NE_lexicon_tags_4 },
    { 0xd672b186, SUC_NE_lexicon_tags_2 },
    { 0x8cc4b18a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8d2b18e, SUC_NE_lexicon_tags_424 },
    { 0x7e0b18f, SUC_NE_lexicon_tags_2 },
    { 0x3f56b18f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x743eb192, SUC_NE_lexicon_tags_2 },
    { 0xea86b193, SUC_NE_lexicon_tags_2 },
    { 0x74dcb192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb6b196, SUC_NE_lexicon_tags_2 },
    { 0xb1a2b197, SUC_NE_lexicon_tags_2 },
    { 0x84d0b196, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x2886b19a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe996b19c, SUC_NE_lexicon_tags_2 },
    { 0x6236b19d, SUC_NE_lexicon_tags_2 },
    { 0xc418b19e, SUC_NE_lexicon_tags_3 },
    { 0x7fa8b19e, SUC_NE_lexicon_tags_2 },
    { 0x8d94b19c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42ab1a5, SUC_NE_lexicon_tags_397 },
    { 0x3668b1a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ab1a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2108b1ae, SUC_NE_lexicon_tags_2 },
    { 0xb460b1af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5eb0b1b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f90b1b3, SUC_NE_lexicon_tags_2 },
    { 0x6ff4b1b4, SUC_NE_lexicon_tags_2 },
    { 0xc45cb1b5, SUC_NE_lexicon_tags_2 },
    { 0x5666b1b4, SUC_NE_lexicon_tags_2 },
    { 0x1df2b1b7, SUC_NE_lexicon_tags_2 },
    { 0x6d90b1b7, SUC_NE_lexicon_tags_2 },
    { 0x225cb1b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7eb1c1, SUC_NE_lexicon_tags_138 },
    { 0x68c0b1c1, SUC_NE_lexicon_tags_2 },
    { 0xbde6b1c1, SUC_NE_lexicon_tags_2 },
    { 0x5488b1c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa96b1c9, SUC_NE_lexicon_tags_19 },
    { 0x4f32b1ca, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xa8ccb1cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf726b1ce, SUC_NE_lexicon_tags_2 },
    { 0x12acb1cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2cb1d5, SUC_NE_lexicon_tags_2 },
    { 0xdb42b1d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9864b1dc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4b1df, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d4b1e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12b1ea, SUC_NE_lexicon_tags_2 },
    { 0x51aeb1ea, SUC_NE_lexicon_tags_2 },
    { 0x3926b1ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7978b1f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0d6b1f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x792b1f8, SUC_NE_lexicon_tags_2 },
    { 0x88b1f9, SUC_NE_lexicon_tags_2 },
    { 0xfb12b1fa, SUC_NE_lexicon_tags_2 },
    { 0x5a0b1fb, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x60acb1fd, SUC_NE_lexicon_tags_2 },
    { 0xd08b1fd, SUC_NE_lexicon_tags_3 },
    { 0xf63cb1ff, SUC_NE_lexicon_tags_2 },
    { 0xe846b1ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x272b202, SUC_NE_lexicon_tags_2 },
    { 0x4f78b203, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d60b209, SUC_NE_lexicon_tags_2 },
    { 0x2c94b20a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75feb20e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c76b211, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214eb215, SUC_NE_lexicon_tags_2 },
    { 0x1798b216, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf602b21a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16cb220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x390eb223, SUC_NE_lexicon_tags_1 },
    { 0xf60eb224, SUC_NE_lexicon_tags_2 },
    { 0x67bab225, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1688b229, SUC_NE_lexicon_tags_17 },
    { 0x11bab22a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11eb22d, SUC_NE_lexicon_tags_7 },
    { 0x7622b22e, SUC_NE_lexicon_tags_2 },
    { 0xfa12b22f, SUC_NE_lexicon_tags_3 },
    { 0x2358b230, SUC_NE_lexicon_tags_2 },
    { 0x5d3eb231, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x61c0b233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x748cb235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe11cb237, SUC_NE_lexicon_tags_3 },
    { 0x66d2b238, SUC_NE_lexicon_tags_2 },
    { 0x6764b239, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bcb23d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x51eeb23f, SUC_NE_lexicon_tags_2 },
    { 0xa99cb23f, SUC_NE_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dab244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeb6b246, SUC_NE_lexicon_tags_2 },
    { 0xccc4b246, SUC_NE_lexicon_tags_2 },
    { 0x6cf6b246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ceb24b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca98b24e, SUC_NE_lexicon_tags_5 },
    { 0x2188b24f, SUC_NE_lexicon_tags_2 },
    { 0xef9cb250, SUC_NE_lexicon_tags_2 },
    { 0xc7c6b251, SUC_NE_lexicon_tags_2 },
    { 0xf304b252, SUC_NE_lexicon_tags_2 },
    { 0x4ef8b250, SUC_NE_lexicon_tags_2 },
    { 0x9c10b254, SUC_NE_lexicon_tags_2 },
    { 0xf236b255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb09eb257, SUC_NE_lexicon_tags_2 },
    { 0x5744b258, SUC_NE_lexicon_tags_2 },
    { 0xbfaeb257, SUC_NE_lexicon_tags_2 },
    { 0x5418b259, SUC_NE_lexicon_tags_2 },
    { 0xe2eb258, SUC_NE_lexicon_tags_7 },
    { 0xa804b25b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8810b25e, SUC_NE_lexicon_tags_2 },
    { 0xa9cab25e, SUC_NE_lexicon_tags_2 },
    { 0xf4b0b260, SUC_NE_lexicon_tags_2 },
    { 0xb768b261, SUC_NE_lexicon_tags_2 },
    { 0x94deb262, SUC_NE_lexicon_tags_1 },
    { 0x4802b263, SUC_NE_lexicon_tags_2 },
    { 0x580b264, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaafcb266, SUC_NE_lexicon_tags_2 },
    { 0xd64eb266, SUC_NE_lexicon_tags_2 },
    { 0x638b268, SUC_NE_lexicon_tags_2 },
    { 0xfb40b268, SUC_NE_lexicon_tags_2 },
    { 0xe462b266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cb2b26c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c8b273, SUC_NE_lexicon_tags_2 },
    { 0xe376b274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffecb27a, SUC_NE_lexicon_tags_4 },
    { 0x61eab27b, SUC_NE_lexicon_tags_2 },
    { 0xb8fcb27b, SUC_NE_lexicon_tags_4 },
    { 0x5aa8b27b, SUC_NE_lexicon_tags_2 },
    { 0x6b70b27e, SUC_NE_lexicon_tags_2 },
    { 0xbdc6b27f, SUC_NE_lexicon_tags_2 },
    { 0xaabcb280, SUC_NE_lexicon_tags_2 },
    { 0x574b280, SUC_NE_lexicon_tags_2 },
    { 0xd5d6b280, SUC_NE_lexicon_tags_2 },
    { 0x78eab27e, SUC_NE_lexicon_tags_2 },
    { 0x8cc4b284, SUC_NE_lexicon_tags_2 },
    { 0x5334b284, SUC_NE_lexicon_tags_2 },
    { 0xc2c2b285, SUC_NE_lexicon_tags_2 },
    { 0xdf26b287, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa1eb289, SUC_NE_lexicon_tags_2 },
    { 0xf1acb289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe62b28c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bcb292, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3154b294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b8b2a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f6b2a5, SUC_NE_lexicon_tags_2 },
    { 0xf0e0b2a6, SUC_NE_lexicon_tags_2 },
    { 0x624cb2a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bab2a9, SUC_NE_lexicon_tags_2 },
    { 0xaa12b2aa, SUC_NE_lexicon_tags_2 },
    { 0x4cecb2a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbcb2ad, SUC_NE_lexicon_tags_2 },
    { 0x3112b2ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47ab2b2, SUC_NE_lexicon_tags_2 },
    { 0x135ab2b3, SUC_NE_lexicon_tags_2 },
    { 0xf9e0b2b4, SUC_NE_lexicon_tags_2 },
    { 0x4836b2b5, SUC_NE_lexicon_tags_4 },
    { 0x788b2b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96ab2b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e40b2be, SUC_NE_lexicon_tags_2 },
    { 0x7bdcb2bf, SUC_NE_lexicon_tags_2 },
    { 0xd4a8b2bf, SUC_NE_lexicon_tags_2 },
    { 0xc43cb2bf, SUC_NE_lexicon_tags_2 },
    { 0x33beb2c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb746b2c4, SUC_NE_lexicon_tags_2 },
    { 0xa70ab2c5, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d98b2cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad00b2d0, SUC_NE_lexicon_tags_1 },
    { 0x5864b2d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58c2b2d3, SUC_NE_lexicon_tags_2 },
    { 0x4d50b2d4, SUC_NE_lexicon_tags_2 },
    { 0xe36b2d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c6b2da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80ab2dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f6b2e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0cb2e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fbcb2e5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54b2e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2034b2ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b8b2f0, SUC_NE_lexicon_tags_2 },
    { 0x4254b2f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8f4b2f3, SUC_NE_lexicon_tags_2 },
    { 0x41c6b2f3, SUC_NE_lexicon_tags_2 },
    { 0x5590b2f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4cb2fb, SUC_NE_lexicon_tags_2 },
    { 0x34dab2fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e74b300, SUC_NE_lexicon_tags_2 },
    { 0x20bab301, SUC_NE_lexicon_tags_2 },
    { 0x6126b300, SUC_NE_lexicon_tags_2 },
    { 0x24c0b303, SUC_NE_lexicon_tags_2 },
    { 0xe4dcb304, SUC_NE_lexicon_tags_2 },
    { 0xf132b305, SUC_NE_lexicon_tags_2 },
    { 0xf432b303, SUC_NE_lexicon_tags_2 },
    { 0x849cb300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6322b30a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0c6b30c, SUC_NE_lexicon_tags_2 },
    { 0x3f8cb30c, SUC_NE_lexicon_tags_4 },
    { 0xe972b30e, SUC_NE_lexicon_tags_2 },
    { 0x667ab30f, SUC_NE_lexicon_tags_2 },
    { 0x23c0b310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa296b312, SUC_NE_lexicon_tags_2 },
    { 0x8024b312, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x675cb316, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc32eb318, SUC_NE_lexicon_tags_2 },
    { 0x4caab318, SUC_NE_lexicon_tags_2 },
    { 0x5d50b319, SUC_NE_lexicon_tags_2 },
    { 0x6ab8b31a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933eb320, SUC_NE_lexicon_tags_2 },
    { 0xf494b320, SUC_NE_lexicon_tags_2 },
    { 0x18eeb322, SUC_NE_lexicon_tags_2 },
    { 0x95d6b323, SUC_NE_lexicon_tags_2 },
    { 0x78deb322, SUC_NE_lexicon_tags_2 },
    { 0x5570b323, SUC_NE_lexicon_tags_2 },
    { 0xca54b320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf4b328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab92b32d, SUC_NE_lexicon_tags_2 },
    { 0x40a4b32e, SUC_NE_lexicon_tags_7 },
    { 0x7878b32e, SUC_NE_lexicon_tags_2 },
    { 0xf484b330, SUC_NE_lexicon_tags_2 },
    { 0xa386b330, SUC_NE_lexicon_tags_3 },
    { 0x5c2b32d, SUC_NE_lexicon_tags_3 },
    { 0x357eb333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6562b338, SUC_NE_lexicon_tags_2 },
    { 0x58bcb339, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc0b33e, SUC_NE_lexicon_tags_2 },
    { 0xa142b33f, SUC_NE_lexicon_tags_2 },
    { 0xcc7ab340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebcb345, SUC_NE_lexicon_tags_13 },
    { 0x6f92b346, SUC_NE_lexicon_tags_2 },
    { 0x3de4b347, SUC_NE_lexicon_tags_2 },
    { 0x4c9cb345, SUC_NE_lexicon_tags_2 },
    { 0xdac8b349, SUC_NE_lexicon_tags_2 },
    { 0x38b8b34a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fcb34f, SUC_NE_lexicon_tags_2 },
    { 0xea06b34f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb354, SUC_NE_lexicon_tags_2 },
    { 0xed42b354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe778b35b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aeb35f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c8b362, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab2b364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x495eb367, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6e2b369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40cb36c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xce54b36e, SUC_NE_lexicon_tags_4 },
    { 0xf264b36f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5ab374, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe39cb376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabf6b378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x722eb37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef40b37f, SUC_NE_lexicon_tags_2 },
    { 0x69f8b37f, SUC_NE_lexicon_tags_8 },
    { 0x9bf0b381, SUC_NE_lexicon_tags_2 },
    { 0x7db8b37f, SUC_NE_lexicon_tags_2 },
    { 0x21e6b383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f82b385, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194eb38b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8018b38d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c40b38f, SUC_NE_lexicon_tags_2 },
    { 0xc586b390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12ab392, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fab395, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c2b39a, SUC_NE_lexicon_tags_2 },
    { 0xbd32b39b, SUC_NE_lexicon_tags_2 },
    { 0x9d84b39c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d06b3a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfeb3b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca30b3b6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7cb3ba, SUC_NE_lexicon_tags_2 },
    { 0x2c70b3bb, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d9eb3bd, SUC_NE_lexicon_tags_50 },
    { 0x616cb3be, SUC_NE_lexicon_tags_2 },
    { 0x52f0b3bf, SUC_NE_lexicon_tags_2 },
    { 0xb58eb3be, SUC_NE_lexicon_tags_2 },
    { 0x444b3c1, SUC_NE_lexicon_tags_2 },
    { 0x6cb3c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b8b3c5, SUC_NE_lexicon_tags_2 },
    { 0x8e1eb3c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32b3cf, SUC_NE_lexicon_tags_2 },
    { 0x80f2b3d0, SUC_NE_lexicon_tags_2 },
    { 0x3948b3d0, SUC_NE_lexicon_tags_2 },
    { 0xa770b3d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xded4b3d5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26eab3d8, SUC_NE_lexicon_tags_2 },
    { 0xa90cb3d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b04b3de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb2b3e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x727ab3e6, SUC_NE_lexicon_tags_2 },
    { 0xa22ab3e7, SUC_NE_lexicon_tags_2 },
    { 0xf18cb3e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaab3ed, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x531eb3ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f76b3f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc468b3fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd722b3ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cdcb401, SUC_NE_lexicon_tags_2 },
    { 0x19dab402, SUC_NE_lexicon_tags_2 },
    { 0x2626b403, SUC_NE_lexicon_tags_2 },
    { 0x2952b401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9444b40a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee4b40d, SUC_NE_lexicon_tags_2 },
    { 0x11e0b40d, SUC_NE_lexicon_tags_2 },
    { 0xf7eab40f, SUC_NE_lexicon_tags_2 },
    { 0xa042b410, SUC_NE_lexicon_tags_2 },
    { 0x7a42b411, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f6b414, SUC_NE_lexicon_tags_2 },
    { 0x5eb4b415, SUC_NE_lexicon_tags_2 },
    { 0xe690b415, SUC_NE_lexicon_tags_4 },
    { 0x9622b417, SUC_NE_lexicon_tags_0 },
    { 0x9f2eb417, SUC_NE_lexicon_tags_2 },
    { 0x2242b419, SUC_NE_lexicon_tags_2 },
    { 0xd320b419, SUC_NE_lexicon_tags_2 },
    { 0x1776b41b, SUC_NE_lexicon_tags_2 },
    { 0x8b2b41c, SUC_NE_lexicon_tags_2 },
    { 0xeaf6b41d, SUC_NE_lexicon_tags_2 },
    { 0x21f8b41e, SUC_NE_lexicon_tags_2 },
    { 0x862b41f, SUC_NE_lexicon_tags_17 },
    { 0xba24b420, SUC_NE_lexicon_tags_2 },
    { 0x2566b418, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8eeb425, SUC_NE_lexicon_tags_2 },
    { 0x2b34b425, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b4ab428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ba2b42a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1184b433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc58b436, SUC_NE_lexicon_tags_2 },
    { 0xce2ab437, SUC_NE_lexicon_tags_2 },
    { 0xa376b438, SUC_NE_lexicon_tags_2 },
    { 0x7ac8b437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf60b43b, SUC_NE_lexicon_tags_2 },
    { 0x49b8b43c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30eb43e, SUC_NE_lexicon_tags_2 },
    { 0xc38b43e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x557eb441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a96b443, SUC_NE_lexicon_tags_2 },
    { 0x35d8b444, SUC_NE_lexicon_tags_2 },
    { 0x6e74b444, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410eb44a, SUC_NE_lexicon_tags_2 },
    { 0x2946b44b, SUC_NE_lexicon_tags_2 },
    { 0x6886b44b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x794b44e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x431cb450, SUC_NE_lexicon_tags_2 },
    { 0x796b450, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3deb454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1562b458, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c52b45a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48acb45d, SUC_NE_lexicon_tags_2 },
    { 0x7a56b45e, SUC_NE_lexicon_tags_2 },
    { 0x33d2b45f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca4b463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ab467, SUC_NE_lexicon_tags_2 },
    { 0x374eb468, SUC_NE_lexicon_tags_2 },
    { 0x1772b469, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb33eb46b, SUC_NE_lexicon_tags_2 },
    { 0x2140b46b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa772b46e, SUC_NE_lexicon_tags_2 },
    { 0xbb6eb46f, SUC_NE_lexicon_tags_2 },
    { 0xd4b8b46f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12e6b472, SUC_NE_lexicon_tags_2 },
    { 0x85a4b472, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae6b478, SUC_NE_lexicon_tags_7 },
    { 0x72d8b478, SUC_NE_lexicon_tags_2 },
    { 0x4790b478, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4eb481, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8676b483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124b48b, SUC_NE_lexicon_tags_2 },
    { 0x9aeb48c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b48f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ceb494, SUC_NE_lexicon_tags_4 },
    { 0x3ac4b494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a74b498, SUC_NE_lexicon_tags_2 },
    { 0xf57cb499, SUC_NE_lexicon_tags_2 },
    { 0x3a00b49a, SUC_NE_lexicon_tags_2 },
    { 0xf88b498, SUC_NE_lexicon_tags_2 },
    { 0xaf28b49c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x789ab49f, SUC_NE_lexicon_tags_2 },
    { 0x7e56b4a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef78b4a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x198cb4a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7d6b4a6, SUC_NE_lexicon_tags_2 },
    { 0x2fa0b4a6, SUC_NE_lexicon_tags_7 },
    { 0x6cc2b4a8, SUC_NE_lexicon_tags_2 },
    { 0x5516b4a9, SUC_NE_lexicon_tags_2 },
    { 0x9d2cb4a9, SUC_NE_lexicon_tags_2 },
    { 0x9ab2b4a9, SUC_NE_lexicon_tags_2 },
    { 0x4d2b4a9, SUC_NE_lexicon_tags_2 },
    { 0x392b4a9, SUC_NE_lexicon_tags_2 },
    { 0x202eb4ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8c2b4b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb2b4b3, SUC_NE_lexicon_tags_2 },
    { 0x3dfcb4b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780b4ba, SUC_NE_lexicon_tags_2 },
    { 0xdcf2b4bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x910cb4bd, SUC_NE_lexicon_tags_2 },
    { 0x2df6b4bd, SUC_NE_lexicon_tags_7 },
    { 0xed5eb4bf, SUC_NE_lexicon_tags_2 },
    { 0x1e92b4bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9798b4c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eab4c9, SUC_NE_lexicon_tags_4 },
    { 0x68b8b4ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1118b4cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad70b4ce, SUC_NE_lexicon_tags_170 },
    { 0x47f6b4cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab4d4, SUC_NE_lexicon_tags_2 },
    { 0x9458b4d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879eb4dc, SUC_NE_lexicon_tags_2 },
    { 0xe9e8b4dd, SUC_NE_lexicon_tags_2 },
    { 0xbb22b4de, SUC_NE_lexicon_tags_2 },
    { 0xef36b4de, SUC_NE_lexicon_tags_2 },
    { 0x352cb4e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x141cb4e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7d4b4e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d0b4ea, SUC_NE_lexicon_tags_2 },
    { 0x9250b4eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d2b4ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fab4f1, SUC_NE_lexicon_tags_2 },
    { 0xe540b4f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224cb4f8, SUC_NE_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x1070b4fb, SUC_NE_lexicon_tags_2 },
    { 0x1fe2b4fc, SUC_NE_lexicon_tags_2 },
    { 0xf1bab4fb, SUC_NE_lexicon_tags_2 },
    { 0x434eb4fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4072b500, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffeb502, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c8b505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3804b507, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x700eb509, SUC_NE_lexicon_tags_2 },
    { 0xaf0b509, SUC_NE_lexicon_tags_2 },
    { 0x148ab50a, SUC_NE_lexicon_tags_2 },
    { 0xa244b50b, SUC_NE_lexicon_tags_2 },
    { 0xcc0ab50c, SUC_NE_lexicon_tags_2 },
    { 0xa82b50c, SUC_NE_lexicon_tags_2 },
    { 0x7276b50f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb8b514, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf8b51c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93ab523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33eb528, SUC_NE_lexicon_tags_2 },
    { 0xc4aab529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17ab52c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d34b52e, SUC_NE_lexicon_tags_4 },
    { 0xc104b52e, SUC_NE_lexicon_tags_2 },
    { 0xcb22b52f, SUC_NE_lexicon_tags_2 },
    { 0x6ee2b531, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca4b533, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1eb536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d8b539, SUC_NE_lexicon_tags_2 },
    { 0x4b0ab53a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e3cb53c, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4ab540, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490b548, SUC_NE_lexicon_tags_2 },
    { 0x434ab548, SUC_NE_lexicon_tags_18 },
    { 0xe20cb54a, SUC_NE_lexicon_tags_2 },
    { 0xb3a2b548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8dcb54d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e76b54f, SUC_NE_lexicon_tags_2 },
    { 0xc374b550, SUC_NE_lexicon_tags_24 },
    { 0x6daeb551, SUC_NE_lexicon_tags_2 },
    { 0x99d4b552, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4074b554, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb5aab556, SUC_NE_lexicon_tags_2 },
    { 0x8cbab557, SUC_NE_lexicon_tags_2 },
    { 0xd0feb557, SUC_NE_lexicon_tags_2 },
    { 0x312eb559, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8b55c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeeb562, SUC_NE_lexicon_tags_2 },
    { 0x32f8b562, SUC_NE_lexicon_tags_2 },
    { 0x6da2b564, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40ccb566, SUC_NE_lexicon_tags_2 },
    { 0x31b8b567, SUC_NE_lexicon_tags_2 },
    { 0x415eb568, SUC_NE_lexicon_tags_2 },
    { 0xcd26b569, SUC_NE_lexicon_tags_2 },
    { 0x75a4b56a, SUC_NE_lexicon_tags_3 },
    { 0xfb4cb567, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf032b56f, SUC_NE_lexicon_tags_2 },
    { 0xf892b570, SUC_NE_lexicon_tags_2 },
    { 0x9064b571, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2026b574, SUC_NE_lexicon_tags_2 },
    { 0xd00cb574, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35cb578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697eb57e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc6aeb580, SUC_NE_lexicon_tags_2 },
    { 0x949ab581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cfab583, SUC_NE_lexicon_tags_11 },
    { 0x16aeb583, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d04b588, SUC_NE_lexicon_tags_2 },
    { 0xe210b588, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0x69d2b58b, SUC_NE_lexicon_tags_2 },
    { 0x5786b58c, SUC_NE_lexicon_tags_2 },
    { 0xe358b58c, SUC_NE_lexicon_tags_2 },
    { 0xa922b58d, SUC_NE_lexicon_tags_2 },
    { 0x73e2b58f, SUC_NE_lexicon_tags_2 },
    { 0x4954b58f, SUC_NE_lexicon_tags_17 },
    { 0x542ab591, SUC_NE_lexicon_tags_1 },
    { 0xf6c6b590, SUC_NE_lexicon_tags_2 },
    { 0x27ceb58c, SUC_NE_lexicon_tags_2 },
    { 0xb246b594, SUC_NE_lexicon_tags_4 },
    { 0x6ac0b595, SUC_NE_lexicon_tags_2 },
    { 0xdb70b596, SUC_NE_lexicon_tags_2 },
    { 0x6bdab597, SUC_NE_lexicon_tags_2 },
    { 0xa808b58e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80e0b59a, SUC_NE_lexicon_tags_2 },
    { 0x9792b59a, SUC_NE_lexicon_tags_2 },
    { 0xc6f0b59b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedceb5a1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3ea0b5a3, SUC_NE_lexicon_tags_2 },
    { 0x38b8b5a4, SUC_NE_lexicon_tags_2 },
    { 0x968cb5a4, SUC_NE_lexicon_tags_0 },
    { 0x7572b5a5, SUC_NE_lexicon_tags_2 },
    { 0xb3e4b5a7, SUC_NE_lexicon_tags_2 },
    { 0xb676b5a8, SUC_NE_lexicon_tags_2 },
    { 0x402cb5a9, SUC_NE_lexicon_tags_2 },
    { 0xa934b5a4, SUC_NE_lexicon_tags_2 },
    { 0x7608b5ab, SUC_NE_lexicon_tags_2 },
    { 0x8446b5ab, SUC_NE_lexicon_tags_2 },
    { 0x1142b5ad, SUC_NE_lexicon_tags_2 },
    { 0xa40eb5ae, SUC_NE_lexicon_tags_4 },
    { 0x92ecb5af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb598b5b5, SUC_NE_lexicon_tags_2 },
    { 0xab30b5b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7f8b5b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd536b5bd, SUC_NE_lexicon_tags_2 },
    { 0xca10b5bd, SUC_NE_lexicon_tags_1 },
    { 0xc76eb5bf, SUC_NE_lexicon_tags_2 },
    { 0x8226b5be, SUC_NE_lexicon_tags_2 },
    { 0xf4b8b5c0, SUC_NE_lexicon_tags_2 },
    { 0x49eeb5c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabacb5c6, SUC_NE_lexicon_tags_2 },
    { 0xb2b0b5c6, SUC_NE_lexicon_tags_109 },
    { 0xe3dab5c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1980b5cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc724b5d1, SUC_NE_lexicon_tags_4 },
    { 0x837cb5d2, SUC_NE_lexicon_tags_2 },
    { 0xf756b5d3, SUC_NE_lexicon_tags_2 },
    { 0x48a2b5d4, SUC_NE_lexicon_tags_2 },
    { 0x301eb5d5, SUC_NE_lexicon_tags_2 },
    { 0x70f4b5d5, SUC_NE_lexicon_tags_18 },
    { 0x1102b5d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fab5da, SUC_NE_lexicon_tags_2 },
    { 0x201cb5da, SUC_NE_lexicon_tags_2 },
    { 0xd2b2b5db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326ab5e0, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3792b5e3, SUC_NE_lexicon_tags_2 },
    { 0x5f00b5e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9006b5e6, SUC_NE_lexicon_tags_50 },
    { 0xa2beb5e6, SUC_NE_lexicon_tags_14 },
    { 0xa9f0b5e6, SUC_NE_lexicon_tags_2 },
    { 0x53c6b5e9, SUC_NE_lexicon_tags_2 },
    { 0x4366b5e9, SUC_NE_lexicon_tags_2 },
    { 0xf4ab5e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x481ab5ed, SUC_NE_lexicon_tags_7 },
    { 0xf2f2b5ed, SUC_NE_lexicon_tags_2 },
    { 0x491cb5ee, SUC_NE_lexicon_tags_411 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebcb5f2, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xd8a0b5f4, SUC_NE_lexicon_tags_18 },
    { 0xb18ab5f4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x199cb5f7, SUC_NE_lexicon_tags_2 },
    { 0x2b8cb5f7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b5fb, SUC_NE_lexicon_tags_3 },
    { 0x1c5eb5fb, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d14b601, SUC_NE_lexicon_tags_2 },
    { 0x7704b602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66acb604, SUC_NE_lexicon_tags_1 },
    { 0x5f20b605, SUC_NE_lexicon_tags_2 },
    { 0xc2d4b606, SUC_NE_lexicon_tags_2 },
    { 0x9354b607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc91ab609, SUC_NE_lexicon_tags_30 },
    { 0x3720b60a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab4b60e, SUC_NE_lexicon_tags_4 },
    { 0x4c9cb60f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb34b611, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb608b613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f1cb615, SUC_NE_lexicon_tags_2 },
    { 0x5f0eb616, SUC_NE_lexicon_tags_2 },
    { 0xe4b4b617, SUC_NE_lexicon_tags_2 },
    { 0x6a6ab615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b8b61e, SUC_NE_lexicon_tags_2 },
    { 0x86d4b61f, SUC_NE_lexicon_tags_2 },
    { 0xf48cb61f, SUC_NE_lexicon_tags_2 },
    { 0x1286b621, SUC_NE_lexicon_tags_2 },
    { 0x37d8b620, SUC_NE_lexicon_tags_2 },
    { 0x20ceb622, SUC_NE_lexicon_tags_24 },
    { 0x69d8b623, SUC_NE_lexicon_tags_2 },
    { 0x1f4b61e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7212b628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf718b62a, SUC_NE_lexicon_tags_2 },
    { 0x55e2b62b, SUC_NE_lexicon_tags_2 },
    { 0xdc5eb62c, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f44b630, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf568b632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192cb636, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb639, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75ab63c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa234b63f, SUC_NE_lexicon_tags_2 },
    { 0xee00b640, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0b0b642, SUC_NE_lexicon_tags_2 },
    { 0xeca4b642, SUC_NE_lexicon_tags_2 },
    { 0x5ba4b643, SUC_NE_lexicon_tags_2 },
    { 0xffa8b645, SUC_NE_lexicon_tags_2 },
    { 0x212cb645, SUC_NE_lexicon_tags_2 },
    { 0xa9c4b642, SUC_NE_lexicon_tags_2 },
    { 0x54eab647, SUC_NE_lexicon_tags_2 },
    { 0x2730b649, SUC_NE_lexicon_tags_2 },
    { 0x8c5ab645, SUC_NE_lexicon_tags_2 },
    { 0x676ab648, SUC_NE_lexicon_tags_2 },
    { 0x74aeb64c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4902b652, SUC_NE_lexicon_tags_2 },
    { 0xc406b652, SUC_NE_lexicon_tags_2 },
    { 0x63f0b654, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b8b657, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35feb65d, SUC_NE_lexicon_tags_2 },
    { 0xa28ab65e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2878b661, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1948b663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f0b666, SUC_NE_lexicon_tags_2 },
    { 0xa9e4b667, SUC_NE_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x964cb66a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x909ab66c, SUC_NE_lexicon_tags_2 },
    { 0xddf2b66d, SUC_NE_lexicon_tags_2 },
    { 0x7bd2b66d, SUC_NE_lexicon_tags_3 },
    { 0xb596b66c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbce2b671, SUC_NE_lexicon_tags_17 },
    { 0x387eb671, SUC_NE_lexicon_tags_7 },
    { 0x26eeb672, SUC_NE_lexicon_tags_17 },
    { 0x64feb673, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5174b676, SUC_NE_lexicon_tags_3 },
    { 0x40aeb677, SUC_NE_lexicon_tags_2 },
    { 0x6f3cb678, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5070b67b, SUC_NE_lexicon_tags_2 },
    { 0x8df6b67c, SUC_NE_lexicon_tags_2 },
    { 0x75e0b67b, SUC_NE_lexicon_tags_1 },
    { 0xd60cb67d, SUC_NE_lexicon_tags_2 },
    { 0x628cb67f, SUC_NE_lexicon_tags_2 },
    { 0x15feb680, SUC_NE_lexicon_tags_2 },
    { 0x1142b67d, SUC_NE_lexicon_tags_32 },
    { 0x8ebeb67c, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad2b686, SUC_NE_lexicon_tags_2 },
    { 0x80a4b686, SUC_NE_lexicon_tags_2 },
    { 0x1600b687, SUC_NE_lexicon_tags_102 },
    { 0x4364b688, SUC_NE_lexicon_tags_2 },
    { 0x140b68a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d42b68e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e8b692, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a0b696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a4b69a, SUC_NE_lexicon_tags_2 },
    { 0x5526b69b, SUC_NE_lexicon_tags_2 },
    { 0x6248b69c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe030b6a0, SUC_NE_lexicon_tags_2 },
    { 0xe72eb6a1, SUC_NE_lexicon_tags_2 },
    { 0x7126b6a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5cb6a8, SUC_NE_lexicon_tags_2 },
    { 0x6424b6a8, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaab6ad, SUC_NE_lexicon_tags_49 },
    { 0x968eb6ae, SUC_NE_lexicon_tags_2 },
    { 0x4b32b6ad, SUC_NE_lexicon_tags_2 },
    { 0xa980b6ad, SUC_NE_lexicon_tags_13 },
    { 0xe344b6b1, SUC_NE_lexicon_tags_2 },
    { 0xc7d0b6b2, SUC_NE_lexicon_tags_0 },
    { 0xce5eb6b3, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e56b6b6, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x2186b6b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa1ab6ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8752b6bc, SUC_NE_lexicon_tags_7 },
    { 0xc0a4b6bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478ab6c2, SUC_NE_lexicon_tags_2 },
    { 0x3fc6b6c3, SUC_NE_lexicon_tags_2 },
    { 0x8564b6c4, SUC_NE_lexicon_tags_260 },
    { 0xf4e2b6c5, SUC_NE_lexicon_tags_2 },
    { 0x5172b6c5, SUC_NE_lexicon_tags_2 },
    { 0x8deb6c4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fab6ca, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xbadab6cc, SUC_NE_lexicon_tags_2 },
    { 0xc9fcb6cd, SUC_NE_lexicon_tags_2 },
    { 0x8134b6cd, SUC_NE_lexicon_tags_2 },
    { 0xd192b6cd, SUC_NE_lexicon_tags_2 },
    { 0x1dd4b6d0, SUC_NE_lexicon_tags_2 },
    { 0xbdd8b6d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x852b6d3, SUC_NE_lexicon_tags_2 },
    { 0x7cd2b6d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cc8b6d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaab6dc, SUC_NE_lexicon_tags_2 },
    { 0x2724b6dd, SUC_NE_lexicon_tags_1 },
    { 0x6314b6de, SUC_NE_lexicon_tags_2 },
    { 0x9b5cb6df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa8b6e3, SUC_NE_lexicon_tags_2 },
    { 0x5c60b6e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9994b6e6, SUC_NE_lexicon_tags_2 },
    { 0x3b30b6e7, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883ab6f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c36b6f6, SUC_NE_lexicon_tags_2 },
    { 0x2000b6f7, SUC_NE_lexicon_tags_110 },
    { 0xa1bab6f8, SUC_NE_lexicon_tags_2 },
    { 0x4d4ab6f9, SUC_NE_lexicon_tags_2 },
    { 0x4910b6fa, SUC_NE_lexicon_tags_2 },
    { 0xc06cb6f9, SUC_NE_lexicon_tags_2 },
    { 0x8618b6fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e94b6ff, SUC_NE_lexicon_tags_2 },
    { 0x9beab700, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66b709, SUC_NE_lexicon_tags_13 },
    { 0x3558b70a, SUC_NE_lexicon_tags_36 },
    { 0xcf4cb70b, SUC_NE_lexicon_tags_2 },
    { 0x2f1eb70b, SUC_NE_lexicon_tags_2 },
    { 0xbc10b70d, SUC_NE_lexicon_tags_2 },
    { 0x49ccb70e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc14eb710, SUC_NE_lexicon_tags_2 },
    { 0x7356b710, SUC_NE_lexicon_tags_34 },
    { 0xbf04b712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf11eb714, SUC_NE_lexicon_tags_2 },
    { 0x5dd0b714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7084b717, SUC_NE_lexicon_tags_2 },
    { 0x34dcb718, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81cb71a, SUC_NE_lexicon_tags_2 },
    { 0xfbaeb71b, SUC_NE_lexicon_tags_2 },
    { 0x83d6b71c, SUC_NE_lexicon_tags_2 },
    { 0xf66b71d, SUC_NE_lexicon_tags_2 },
    { 0xc580b71e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf4b722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa16b726, SUC_NE_lexicon_tags_2 },
    { 0x1fdab727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c2b72b, SUC_NE_lexicon_tags_2 },
    { 0xf974b72b, SUC_NE_lexicon_tags_2 },
    { 0x880cb72b, SUC_NE_lexicon_tags_2 },
    { 0xd328b72e, SUC_NE_lexicon_tags_2 },
    { 0x170b72f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd2b73a, SUC_NE_lexicon_tags_2 },
    { 0xcb2cb73b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x29aeb73d, SUC_NE_lexicon_tags_1 },
    { 0x318b73e, SUC_NE_lexicon_tags_2 },
    { 0x2830b73f, SUC_NE_lexicon_tags_2 },
    { 0x79beb740, SUC_NE_lexicon_tags_2 },
    { 0x35b2b741, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcab745, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cd6b747, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x584eb74a, SUC_NE_lexicon_tags_2 },
    { 0x51e4b74a, SUC_NE_lexicon_tags_2 },
    { 0x65b6b74c, SUC_NE_lexicon_tags_17 },
    { 0x4160b74d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1164b74f, SUC_NE_lexicon_tags_2 },
    { 0xcc38b750, SUC_NE_lexicon_tags_2 },
    { 0x26e6b751, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c6b757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b8cb759, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x199ab75b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda08b75e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c88b760, SUC_NE_lexicon_tags_2 },
    { 0xa7ecb761, SUC_NE_lexicon_tags_2 },
    { 0x8386b762, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x209ab764, SUC_NE_lexicon_tags_2 },
    { 0x26a0b764, SUC_NE_lexicon_tags_2 },
    { 0x2682b766, SUC_NE_lexicon_tags_2 },
    { 0x4940b767, SUC_NE_lexicon_tags_2 },
    { 0xce8cb767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3616b76b, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x5752b76d, SUC_NE_lexicon_tags_2 },
    { 0x5004b76d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb84eb770, SUC_NE_lexicon_tags_0 },
    { 0xae48b771, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd14b783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bcb786, SUC_NE_lexicon_tags_2 },
    { 0x7a02b787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8b78c, SUC_NE_lexicon_tags_2 },
    { 0x8162b78c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbcb794, SUC_NE_lexicon_tags_2 },
    { 0x179cb794, SUC_NE_lexicon_tags_2 },
    { 0xd6d4b796, SUC_NE_lexicon_tags_2 },
    { 0xe4f0b796, SUC_NE_lexicon_tags_2 },
    { 0x712cb798, SUC_NE_lexicon_tags_2 },
    { 0xf800b798, SUC_NE_lexicon_tags_2 },
    { 0x9850b798, SUC_NE_lexicon_tags_7 },
    { 0xd422b798, SUC_NE_lexicon_tags_2 },
    { 0xf83ab798, SUC_NE_lexicon_tags_2 },
    { 0xbb18b79c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee80b7a0, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b0b7a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x212b7a6, SUC_NE_lexicon_tags_3 },
    { 0x27fab7a6, SUC_NE_lexicon_tags_2 },
    { 0xd61eb7a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeeb7ac, SUC_NE_lexicon_tags_2 },
    { 0xb03eb7ad, SUC_NE_lexicon_tags_17 },
    { 0x75b4b7ad, SUC_NE_lexicon_tags_2 },
    { 0xb7ecb7ae, SUC_NE_lexicon_tags_2 },
    { 0x41a4b7af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7deb7b4, SUC_NE_lexicon_tags_2 },
    { 0xaff2b7b5, SUC_NE_lexicon_tags_2 },
    { 0xbafcb7b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1774b7bb, SUC_NE_lexicon_tags_2 },
    { 0x49bcb7bc, SUC_NE_lexicon_tags_2 },
    { 0x7ff4b7bd, SUC_NE_lexicon_tags_2 },
    { 0x767ab7be, SUC_NE_lexicon_tags_2 },
    { 0xfbb8b7bd, SUC_NE_lexicon_tags_2 },
    { 0x4b0eb7c0, SUC_NE_lexicon_tags_2 },
    { 0xbc44b7c1, SUC_NE_lexicon_tags_2 },
    { 0xf1eb7be, SUC_NE_lexicon_tags_2 },
    { 0xf4ceb7c1, SUC_NE_lexicon_tags_3 },
    { 0x8216b7bc, SUC_NE_lexicon_tags_2 },
    { 0xa588b7c0, SUC_NE_lexicon_tags_2 },
    { 0xb542b7c6, SUC_NE_lexicon_tags_2 },
    { 0xed6b7c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f8b7ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4940b7cd, SUC_NE_lexicon_tags_2 },
    { 0x20e8b7cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb896b7d1, SUC_NE_lexicon_tags_2 },
    { 0x4d28b7d2, SUC_NE_lexicon_tags_2 },
    { 0x7160b7d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90c0b7d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x599eb7d7, SUC_NE_lexicon_tags_2 },
    { 0x57a2b7d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d14b7da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf822b7e0, SUC_NE_lexicon_tags_7 },
    { 0x9b9eb7e1, SUC_NE_lexicon_tags_2 },
    { 0x704b7e2, SUC_NE_lexicon_tags_2 },
    { 0x3c90b7e3, SUC_NE_lexicon_tags_2 },
    { 0xace6b7e1, SUC_NE_lexicon_tags_2 },
    { 0x634b7e3, SUC_NE_lexicon_tags_2 },
    { 0x5dfcb7e4, SUC_NE_lexicon_tags_2 },
    { 0x2482b7e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2374b7ea, SUC_NE_lexicon_tags_2 },
    { 0xee6cb7eb, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0d6b7ed, SUC_NE_lexicon_tags_4 },
    { 0x52eab7ed, SUC_NE_lexicon_tags_2 },
    { 0xbc62b7ef, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xf296b7f1, SUC_NE_lexicon_tags_2 },
    { 0x14feb7f2, SUC_NE_lexicon_tags_2 },
    { 0x9eccb7f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64cb7f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77a2b7f8, SUC_NE_lexicon_tags_5 },
    { 0xf4eb7f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf6b801, SUC_NE_lexicon_tags_2 },
    { 0xb3e8b802, SUC_NE_lexicon_tags_2 },
    { 0x80f4b803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f4b80f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf56b811, SUC_NE_lexicon_tags_2 },
    { 0xb32b812, SUC_NE_lexicon_tags_104 },
    { 0x9aa2b812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7556b815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5602b817, SUC_NE_lexicon_tags_2 },
    { 0x3dab817, SUC_NE_lexicon_tags_2 },
    { 0xbaf0b819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98dcb81b, SUC_NE_lexicon_tags_2 },
    { 0xeca8b81c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb8b820, SUC_NE_lexicon_tags_2 },
    { 0x3cfcb821, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468ab826, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4448b829, SUC_NE_lexicon_tags_2 },
    { 0x7e8ab829, SUC_NE_lexicon_tags_3 },
    { 0x923eb82b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cab830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1098b832, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c8b835, SUC_NE_lexicon_tags_2 },
    { 0xe140b835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf01ab838, SUC_NE_lexicon_tags_2 },
    { 0xc306b839, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2714b83c, SUC_NE_lexicon_tags_2 },
    { 0x24eeb83d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2906b83f, SUC_NE_lexicon_tags_2 },
    { 0x4fe6b83f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x436eb842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b02b848, SUC_NE_lexicon_tags_2 },
    { 0x790b848, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66beb84d, SUC_NE_lexicon_tags_2 },
    { 0x4624b84e, SUC_NE_lexicon_tags_2 },
    { 0x6beeb84f, SUC_NE_lexicon_tags_2 },
    { 0xcef6b850, SUC_NE_lexicon_tags_19 },
    { 0x2bfab84e, SUC_NE_lexicon_tags_2 },
    { 0x9ed6b84f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc714b854, SUC_NE_lexicon_tags_2 },
    { 0xd150b855, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed4b858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x698eb85a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7262b85d, SUC_NE_lexicon_tags_2 },
    { 0x1e34b85e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a8b860, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e4b866, SUC_NE_lexicon_tags_2 },
    { 0x3aaeb867, SUC_NE_lexicon_tags_2 },
    { 0x2cfcb868, SUC_NE_lexicon_tags_7 },
    { 0x31dcb866, SUC_NE_lexicon_tags_2 },
    { 0x789ab866, SUC_NE_lexicon_tags_2 },
    { 0x3046b86b, SUC_NE_lexicon_tags_2 },
    { 0xca80b86c, SUC_NE_lexicon_tags_2 },
    { 0x3314b86b, SUC_NE_lexicon_tags_2 },
    { 0xffaeb86e, SUC_NE_lexicon_tags_2 },
    { 0x8d34b869, SUC_NE_lexicon_tags_2 },
    { 0xca1eb86c, SUC_NE_lexicon_tags_2 },
    { 0x6a22b869, SUC_NE_lexicon_tags_2 },
    { 0xa3fcb868, SUC_NE_lexicon_tags_2 },
    { 0x62bcb873, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb162b877, SUC_NE_lexicon_tags_2 },
    { 0x8de4b877, SUC_NE_lexicon_tags_4 },
    { 0x490b878, SUC_NE_lexicon_tags_2 },
    { 0x54bcb87a, SUC_NE_lexicon_tags_2 },
    { 0xda2b878, SUC_NE_lexicon_tags_280 },
    { 0x2e24b878, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0b880, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c0b883, SUC_NE_lexicon_tags_2 },
    { 0xbddcb884, SUC_NE_lexicon_tags_2 },
    { 0x8ef0b885, SUC_NE_lexicon_tags_2 },
    { 0xe294b886, SUC_NE_lexicon_tags_3 },
    { 0x135eb886, SUC_NE_lexicon_tags_18 },
    { 0x4254b888, SUC_NE_lexicon_tags_2 },
    { 0x20d8b889, SUC_NE_lexicon_tags_2 },
    { 0x17c8b88a, SUC_NE_lexicon_tags_1 },
    { 0xd9d4b88b, SUC_NE_lexicon_tags_24 },
    { 0xb4bab88c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f74b894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d4b89b, SUC_NE_lexicon_tags_2 },
    { 0xfd18b89b, SUC_NE_lexicon_tags_1 },
    { 0x1b54b89b, SUC_NE_lexicon_tags_2 },
    { 0x15a0b89e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3eb8a3, SUC_NE_lexicon_tags_2 },
    { 0x3550b8a4, SUC_NE_lexicon_tags_2 },
    { 0xd11ab8a3, SUC_NE_lexicon_tags_40 },
    { 0xb07cb8a6, SUC_NE_lexicon_tags_2 },
    { 0x2c0ab8a6, SUC_NE_lexicon_tags_2 },
    { 0x4ed0b8a7, SUC_NE_lexicon_tags_2 },
    { 0xd5c6b8a9, SUC_NE_lexicon_tags_2 },
    { 0x41aeb8aa, SUC_NE_lexicon_tags_3 },
    { 0x3fdab8ab, SUC_NE_lexicon_tags_2 },
    { 0xad78b8ac, SUC_NE_lexicon_tags_2 },
    { 0xe7d2b8a9, SUC_NE_lexicon_tags_2 },
    { 0x30fcb8ad, SUC_NE_lexicon_tags_2 },
    { 0x1b04b8af, SUC_NE_lexicon_tags_2 },
    { 0xc0f2b8a4, SUC_NE_lexicon_tags_2 },
    { 0x69eeb8aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44b8b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84cb8ba, SUC_NE_lexicon_tags_2 },
    { 0xfd28b8bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd2cb8bd, SUC_NE_lexicon_tags_2 },
    { 0xc8d0b8be, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bb0b8c0, SUC_NE_lexicon_tags_2 },
    { 0x1b6eb8c1, SUC_NE_lexicon_tags_2 },
    { 0xbef8b8c2, SUC_NE_lexicon_tags_2 },
    { 0x1030b8c3, SUC_NE_lexicon_tags_207 },
    { 0x8a9eb8c3, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe766b8c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43d0b8cb, SUC_NE_lexicon_tags_2 },
    { 0x8218b8cc, SUC_NE_lexicon_tags_170 },
    { 0xe5e0b8cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe638b8d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1a6b8d7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b6b8dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x584cb8df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49c2b8e1, SUC_NE_lexicon_tags_2 },
    { 0x5cacb8e1, SUC_NE_lexicon_tags_4 },
    { 0xb462b8e3, SUC_NE_lexicon_tags_2 },
    { 0x1a22b8e4, SUC_NE_lexicon_tags_2 },
    { 0xb8d4b8e4, SUC_NE_lexicon_tags_2 },
    { 0x7be0b8e4, SUC_NE_lexicon_tags_2 },
    { 0x6792b8e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9978b8e9, SUC_NE_lexicon_tags_5 },
    { 0xcbe2b8ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x999eb8ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b1eb8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9aeb8f1, SUC_NE_lexicon_tags_2 },
    { 0x4400b8f2, SUC_NE_lexicon_tags_2 },
    { 0xb656b8f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc792b8fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf676b8fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89e2b8fe, SUC_NE_lexicon_tags_2 },
    { 0x5e88b8ff, SUC_NE_lexicon_tags_17 },
    { 0xcaeb8fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3150b902, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x44eab904, SUC_NE_lexicon_tags_2 },
    { 0xec7eb905, SUC_NE_lexicon_tags_2 },
    { 0xa536b905, SUC_NE_lexicon_tags_2 },
    { 0xd8f6b907, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x575ab90a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbc6b90c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc918b90f, SUC_NE_lexicon_tags_3 },
    { 0xb516b90f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1008b915, SUC_NE_lexicon_tags_4 },
    { 0x182b916, SUC_NE_lexicon_tags_2 },
    { 0xd472b916, SUC_NE_lexicon_tags_2 },
    { 0x4918b916, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85b6b91a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd138b91c, SUC_NE_lexicon_tags_2 },
    { 0x1264b91d, SUC_NE_lexicon_tags_2 },
    { 0xe776b91d, SUC_NE_lexicon_tags_5 },
    { 0xe87cb91f, SUC_NE_lexicon_tags_2 },
    { 0x59bcb91f, SUC_NE_lexicon_tags_2 },
    { 0xeb82b921, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x220eb924, SUC_NE_lexicon_tags_2 },
    { 0x866ab925, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46beb92b, SUC_NE_lexicon_tags_2 },
    { 0x63aeb92b, SUC_NE_lexicon_tags_2 },
    { 0x4d66b92b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b0b934, SUC_NE_lexicon_tags_2 },
    { 0xfbceb935, SUC_NE_lexicon_tags_2 },
    { 0xdc28b935, SUC_NE_lexicon_tags_2 },
    { 0x4e8ab937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab52b93a, SUC_NE_lexicon_tags_2 },
    { 0x95eab93a, SUC_NE_lexicon_tags_2 },
    { 0x3e8cb93c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9e2b93e, SUC_NE_lexicon_tags_2 },
    { 0xdd12b93e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa582b942, SUC_NE_lexicon_tags_2 },
    { 0xcd0cb943, SUC_NE_lexicon_tags_2 },
    { 0x8a62b943, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6b94a, SUC_NE_lexicon_tags_2 },
    { 0x2a96b94b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc24b94d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1944b956, SUC_NE_lexicon_tags_2 },
    { 0x4930b956, SUC_NE_lexicon_tags_2 },
    { 0x3776b956, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1554b95d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f86b962, SUC_NE_lexicon_tags_2 },
    { 0xd332b962, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45eb966, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x548b968, SUC_NE_lexicon_tags_2 },
    { 0x3c6b969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x835eb96b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2e6b96d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa754b971, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d4b975, SUC_NE_lexicon_tags_2 },
    { 0xa114b976, SUC_NE_lexicon_tags_2 },
    { 0x4a7eb977, SUC_NE_lexicon_tags_2 },
    { 0x2302b978, SUC_NE_lexicon_tags_7 },
    { 0x1868b979, SUC_NE_lexicon_tags_2 },
    { 0x8c5cb978, SUC_NE_lexicon_tags_1 },
    { 0xa86ab97a, SUC_NE_lexicon_tags_2 },
    { 0x78dab97a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd892b97f, SUC_NE_lexicon_tags_2 },
    { 0x8f6cb980, SUC_NE_lexicon_tags_2 },
    { 0x8544b981, SUC_NE_lexicon_tags_2 },
    { 0xc43ab980, SUC_NE_lexicon_tags_2 },
    { 0x2a0cb980, SUC_NE_lexicon_tags_2 },
    { 0xd1bab980, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2ab98b, SUC_NE_lexicon_tags_2 },
    { 0x6194b98c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fcb990, SUC_NE_lexicon_tags_4 },
    { 0x6704b991, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86cb994, SUC_NE_lexicon_tags_2 },
    { 0x7514b995, SUC_NE_lexicon_tags_2 },
    { 0xac78b996, SUC_NE_lexicon_tags_3 },
    { 0x9dc0b995, SUC_NE_lexicon_tags_2 },
    { 0xa546b998, SUC_NE_lexicon_tags_2 },
    { 0x3390b995, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x624eb99b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21d4b99d, SUC_NE_lexicon_tags_2 },
    { 0x1eeb99e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2c60b9a0, SUC_NE_lexicon_tags_2 },
    { 0x4dab9a1, SUC_NE_lexicon_tags_2 },
    { 0x8118b9a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11acb9a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9850b9aa, SUC_NE_lexicon_tags_2 },
    { 0xbe6cb9ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962b9b4, SUC_NE_lexicon_tags_2 },
    { 0x9e8b9b4, SUC_NE_lexicon_tags_2 },
    { 0x931eb9b6, SUC_NE_lexicon_tags_2 },
    { 0xbe4b9b7, SUC_NE_lexicon_tags_49 },
    { 0x5786b9b8, SUC_NE_lexicon_tags_2 },
    { 0x7606b9b9, SUC_NE_lexicon_tags_2 },
    { 0x397ab9b8, SUC_NE_lexicon_tags_2 },
    { 0x27bab9b5, SUC_NE_lexicon_tags_2 },
    { 0x7e10b9bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3780b9bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a8b9c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72cb9c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf2cb9c7, SUC_NE_lexicon_tags_2 },
    { 0x25ccb9c8, SUC_NE_lexicon_tags_4 },
    { 0x214b9c8, SUC_NE_lexicon_tags_2 },
    { 0xe73ab9c9, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x111cb9cc, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x90ab9ce, SUC_NE_lexicon_tags_2 },
    { 0x84eb9cf, SUC_NE_lexicon_tags_2 },
    { 0xc5cab9d0, SUC_NE_lexicon_tags_2 },
    { 0x994eb9d0, SUC_NE_lexicon_tags_2 },
    { 0xb482b9d2, SUC_NE_lexicon_tags_2 },
    { 0xd7d2b9cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb444b9d6, SUC_NE_lexicon_tags_2 },
    { 0x54d6b9d7, SUC_NE_lexicon_tags_2 },
    { 0xd9e0b9d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5cb9dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657eb9e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe518b9e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2486b9e8, SUC_NE_lexicon_tags_3 },
    { 0x566eb9e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeeb9ee, SUC_NE_lexicon_tags_2 },
    { 0x3208b9ef, SUC_NE_lexicon_tags_2 },
    { 0x4efab9f0, SUC_NE_lexicon_tags_2 },
    { 0x4966b9f1, SUC_NE_lexicon_tags_2 },
    { 0x1f96b9f2, SUC_NE_lexicon_tags_2 },
    { 0xc0c8b9f1, SUC_NE_lexicon_tags_2 },
    { 0xade6b9f4, SUC_NE_lexicon_tags_2 },
    { 0x3bbab9f5, SUC_NE_lexicon_tags_2 },
    { 0x4d74b9f6, SUC_NE_lexicon_tags_1 },
    { 0xd9a8b9f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x296eb9fa, SUC_NE_lexicon_tags_2 },
    { 0x5b2b9fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb324ba04, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dcba08, SUC_NE_lexicon_tags_2 },
    { 0x5b2cba09, SUC_NE_lexicon_tags_3 },
    { 0xe0eba0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3af0ba0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb202ba13, SUC_NE_lexicon_tags_2 },
    { 0x4ff6ba14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eba1b, SUC_NE_lexicon_tags_2 },
    { 0xc826ba1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9288ba1e, SUC_NE_lexicon_tags_2 },
    { 0xdce4ba1f, SUC_NE_lexicon_tags_8 },
    { 0x82b8ba20, SUC_NE_lexicon_tags_2 },
    { 0x51aba21, SUC_NE_lexicon_tags_2 },
    { 0xe674ba22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x831cba25, SUC_NE_lexicon_tags_2 },
    { 0x73a4ba26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc148ba2a, SUC_NE_lexicon_tags_2 },
    { 0x7362ba2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865aba30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x691aba33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe524ba36, SUC_NE_lexicon_tags_3 },
    { 0xfeb8ba37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4428ba3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69e6ba40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85ceba42, SUC_NE_lexicon_tags_2 },
    { 0xee2cba43, SUC_NE_lexicon_tags_17 },
    { 0x6f2ba42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x628eba46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x892eba48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe618ba4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5414ba55, SUC_NE_lexicon_tags_2 },
    { 0x9c14ba56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d08ba5d, SUC_NE_lexicon_tags_2 },
    { 0x852aba5d, SUC_NE_lexicon_tags_2 },
    { 0x3496ba5e, SUC_NE_lexicon_tags_2 },
    { 0x6d2eba60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc84ba63, SUC_NE_lexicon_tags_7 },
    { 0x87fcba64, SUC_NE_lexicon_tags_2 },
    { 0x1a48ba65, SUC_NE_lexicon_tags_2 },
    { 0x553cba65, SUC_NE_lexicon_tags_2 },
    { 0x221eba67, SUC_NE_lexicon_tags_2 },
    { 0xe65eba65, SUC_NE_lexicon_tags_7 },
    { 0x8ae2ba69, SUC_NE_lexicon_tags_7 },
    { 0xb6a8ba69, SUC_NE_lexicon_tags_2 },
    { 0x7024ba69, SUC_NE_lexicon_tags_174 },
    { 0x3724ba6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea0ba6f, SUC_NE_lexicon_tags_7 },
    { 0xbffaba70, SUC_NE_lexicon_tags_2 },
    { 0xbeb8ba71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6ba74, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d64ba78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e94ba7d, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xfa1cba7f, SUC_NE_lexicon_tags_1 },
    { 0x243aba80, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x3306ba82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3eba85, SUC_NE_lexicon_tags_2 },
    { 0xe752ba86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaceba89, SUC_NE_lexicon_tags_2 },
    { 0xeceaba8a, SUC_NE_lexicon_tags_2 },
    { 0x381eba8b, SUC_NE_lexicon_tags_2 },
    { 0x93d8ba8c, SUC_NE_lexicon_tags_2 },
    { 0x73f2ba8d, SUC_NE_lexicon_tags_4 },
    { 0x9c0ba8e, SUC_NE_lexicon_tags_13 },
    { 0xc4c6ba8e, SUC_NE_lexicon_tags_2 },
    { 0xde80ba8a, SUC_NE_lexicon_tags_2 },
    { 0x2bc2ba8b, SUC_NE_lexicon_tags_1 },
    { 0xc43aba92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81caba94, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf40ba99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8692ba9c, SUC_NE_lexicon_tags_2 },
    { 0x8ba6ba9d, SUC_NE_lexicon_tags_2 },
    { 0x22caba9e, SUC_NE_lexicon_tags_2 },
    { 0xd2aeba9d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd2baa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e8baa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6266baa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80abaad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62aebab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d0bab5, SUC_NE_lexicon_tags_2 },
    { 0x248cbab5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5686babc, SUC_NE_lexicon_tags_2 },
    { 0xff3ababc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6acbabf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb6bac5, SUC_NE_lexicon_tags_64 },
    { 0x53a8bac6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52bcbac8, SUC_NE_lexicon_tags_4 },
    { 0x81eebac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e42bacb, SUC_NE_lexicon_tags_2 },
    { 0xd48ebacc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde4bad1, SUC_NE_lexicon_tags_2 },
    { 0xcb8bad1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc016bad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67dcbad6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x210ebad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac2badb, SUC_NE_lexicon_tags_244 },
    { 0x5484badc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfebae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xced0bae2, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9618bae7, SUC_NE_lexicon_tags_2 },
    { 0x858bae8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x223abaea, SUC_NE_lexicon_tags_14 },
    { 0x3ff2baeb, SUC_NE_lexicon_tags_2 },
    { 0xc9d4baea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d1abaee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a4baf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e4baf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6382bafb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3112bafe, SUC_NE_lexicon_tags_2 },
    { 0x2564baff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9216bb01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd394bb06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb2bb0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb858bb0d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x7570bb0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf976bb15, SUC_NE_lexicon_tags_2 },
    { 0x5c1ebb16, SUC_NE_lexicon_tags_2 },
    { 0x49b6bb17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf7ebb19, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176abb1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03cbb22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebb24, SUC_NE_lexicon_tags_4 },
    { 0xfdb8bb24, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ebb2a, SUC_NE_lexicon_tags_2 },
    { 0x8626bb2a, SUC_NE_lexicon_tags_2 },
    { 0xf76bb2c, SUC_NE_lexicon_tags_24 },
    { 0x5302bb2d, SUC_NE_lexicon_tags_2 },
    { 0xb5c6bb2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aabb32, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x135ebb35, SUC_NE_lexicon_tags_2 },
    { 0xc4a2bb35, SUC_NE_lexicon_tags_2 },
    { 0x4428bb37, SUC_NE_lexicon_tags_2 },
    { 0x2e3ebb38, SUC_NE_lexicon_tags_2 },
    { 0xc536bb37, SUC_NE_lexicon_tags_2 },
    { 0xd8ccbb3a, SUC_NE_lexicon_tags_2 },
    { 0x80e4bb37, SUC_NE_lexicon_tags_5 },
    { 0xc556bb3c, SUC_NE_lexicon_tags_2 },
    { 0xbd22bb3a, SUC_NE_lexicon_tags_2 },
    { 0x4f8cbb3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f20bb45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6902bb48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa426bb4a, SUC_NE_lexicon_tags_2 },
    { 0x7f48bb4b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794bb50, SUC_NE_lexicon_tags_2 },
    { 0x2da8bb51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c80bb54, SUC_NE_lexicon_tags_3 },
    { 0xeaa6bb55, SUC_NE_lexicon_tags_14 },
    { 0xb6f0bb56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d2abb58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aebb5c, SUC_NE_lexicon_tags_2 },
    { 0xd34bb5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd03abb5f, SUC_NE_lexicon_tags_2 },
    { 0xfd8bb60, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f86bb6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1464bb70, SUC_NE_lexicon_tags_2 },
    { 0x6982bb70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x436abb74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd8cbb76, SUC_NE_lexicon_tags_2 },
    { 0xd86abb77, SUC_NE_lexicon_tags_4 },
    { 0xdaa2bb77, SUC_NE_lexicon_tags_2 },
    { 0x2d94bb78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbeabb7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6470bb7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed30bb86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dabb8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec0bb8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc14bb8f, SUC_NE_lexicon_tags_10 },
    { 0x2a70bb8f, SUC_NE_lexicon_tags_4 },
    { 0xd1a2bb90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5118bb94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x822cbb97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb34bb99, SUC_NE_lexicon_tags_2 },
    { 0x30c2bb99, SUC_NE_lexicon_tags_2 },
    { 0x2386bb9b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e56bb9e, SUC_NE_lexicon_tags_2 },
    { 0x2ca6bb9f, SUC_NE_lexicon_tags_2 },
    { 0x7b2abb9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8452bba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1c2bba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc930bbab, SUC_NE_lexicon_tags_2 },
    { 0x8462bbab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfc4bbae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc34bbb4, SUC_NE_lexicon_tags_2 },
    { 0xc05ebbb5, SUC_NE_lexicon_tags_1 },
    { 0x9a40bbb6, SUC_NE_lexicon_tags_2 },
    { 0x2882bbb7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a48bbb9, SUC_NE_lexicon_tags_2 },
    { 0xdba4bbb9, SUC_NE_lexicon_tags_3 },
    { 0xded0bbbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b6bbc3, SUC_NE_lexicon_tags_7 },
    { 0xd4cbbc4, SUC_NE_lexicon_tags_2 },
    { 0x3f7ebbc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bebbca, SUC_NE_lexicon_tags_2 },
    { 0x24d2bbcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcaabbcd, SUC_NE_lexicon_tags_2 },
    { 0xca46bbce, SUC_NE_lexicon_tags_2 },
    { 0xf432bbce, SUC_NE_lexicon_tags_2 },
    { 0xf72bbd0, SUC_NE_lexicon_tags_2 },
    { 0xcb48bbd1, SUC_NE_lexicon_tags_2 },
    { 0xc50ebbd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ebbd9, SUC_NE_lexicon_tags_2 },
    { 0x415ebbda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba98bbdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8978bbe3, SUC_NE_lexicon_tags_2 },
    { 0x929ebbe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a6bbeb, SUC_NE_lexicon_tags_2 },
    { 0x7efabbec, SUC_NE_lexicon_tags_2 },
    { 0x3a78bbed, SUC_NE_lexicon_tags_2 },
    { 0xef4abbec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ebbf3, SUC_NE_lexicon_tags_2 },
    { 0x341abbf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c3cbbf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aa2bbf8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xca02bbfb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x561ebbfe, SUC_NE_lexicon_tags_2 },
    { 0xa70bbfe, SUC_NE_lexicon_tags_2 },
    { 0xee9abc00, SUC_NE_lexicon_tags_0 },
    { 0xc58cbc01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a72bc05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef82bc07, SUC_NE_lexicon_tags_2 },
    { 0x8fa8bc08, SUC_NE_lexicon_tags_2 },
    { 0x10e8bc08, SUC_NE_lexicon_tags_2 },
    { 0x1a82bc0a, SUC_NE_lexicon_tags_3 },
    { 0xacb2bc09, SUC_NE_lexicon_tags_2 },
    { 0x8a98bc0b, SUC_NE_lexicon_tags_2 },
    { 0xdaa8bc0b, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bebc12, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9ebc18, SUC_NE_lexicon_tags_2 },
    { 0x50ebc19, SUC_NE_lexicon_tags_1 },
    { 0xbd1abc18, SUC_NE_lexicon_tags_3 },
    { 0x73d6bc1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5186bc1f, SUC_NE_lexicon_tags_2 },
    { 0xc44ebc20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450ebc25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fabc29, SUC_NE_lexicon_tags_2 },
    { 0x34d0bc29, SUC_NE_lexicon_tags_2 },
    { 0xbcdcbc2b, SUC_NE_lexicon_tags_2 },
    { 0xfe02bc2a, SUC_NE_lexicon_tags_2 },
    { 0x8d8bc2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf8bc30, SUC_NE_lexicon_tags_2 },
    { 0xfd06bc31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74cbc34, SUC_NE_lexicon_tags_2 },
    { 0xe3b2bc35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cacbc39, SUC_NE_lexicon_tags_3 },
    { 0x4854bc3a, SUC_NE_lexicon_tags_2 },
    { 0x4a56bc3b, SUC_NE_lexicon_tags_5 },
    { 0x4b0ebc3c, SUC_NE_lexicon_tags_2 },
    { 0xad44bc3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6892bc3f, SUC_NE_lexicon_tags_2 },
    { 0xbf4abc3f, SUC_NE_lexicon_tags_2 },
    { 0x8d54bc41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a4bc47, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xefa6bc49, SUC_NE_lexicon_tags_2 },
    { 0x230cbc4a, SUC_NE_lexicon_tags_2 },
    { 0xc058bc49, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4222bc4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2a6bc51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc640bc53, SUC_NE_lexicon_tags_19 },
    { 0x48d0bc53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c2bc57, SUC_NE_lexicon_tags_2 },
    { 0x7e34bc57, SUC_NE_lexicon_tags_2 },
    { 0x9e26bc59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c8bc5d, SUC_NE_lexicon_tags_2 },
    { 0x1150bc5d, SUC_NE_lexicon_tags_2 },
    { 0x59f6bc5f, SUC_NE_lexicon_tags_2 },
    { 0x8faabc5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1abc63, SUC_NE_lexicon_tags_2 },
    { 0x92dcbc64, SUC_NE_lexicon_tags_2 },
    { 0xc5aebc64, SUC_NE_lexicon_tags_2 },
    { 0x3836bc66, SUC_NE_lexicon_tags_2 },
    { 0xf8b6bc67, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ccbc6d, SUC_NE_lexicon_tags_2 },
    { 0x429cbc6e, SUC_NE_lexicon_tags_1 },
    { 0xc166bc6f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe646bc71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd356bc73, SUC_NE_lexicon_tags_42 },
    { 0xd8abc74, SUC_NE_lexicon_tags_2 },
    { 0xdb6abc75, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x9bbabc77, SUC_NE_lexicon_tags_2 },
    { 0x2232bc78, SUC_NE_lexicon_tags_7 },
    { 0xf4b4bc79, SUC_NE_lexicon_tags_2 },
    { 0x8e1abc7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe038bc7d, SUC_NE_lexicon_tags_253 },
    { 0x56aebc7e, SUC_NE_lexicon_tags_3 },
    { 0x2384bc7d, SUC_NE_lexicon_tags_2 },
    { 0xc3febc7e, SUC_NE_lexicon_tags_2 },
    { 0x3f76bc81, SUC_NE_lexicon_tags_2 },
    { 0xd084bc7f, SUC_NE_lexicon_tags_2 },
    { 0x216bc83, SUC_NE_lexicon_tags_2 },
    { 0x1c2ebc84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x492cbc87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d6bc8e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7ebc99, SUC_NE_lexicon_tags_2 },
    { 0xe93cbc9a, SUC_NE_lexicon_tags_2 },
    { 0xcb50bc9b, SUC_NE_lexicon_tags_18 },
    { 0xb3f0bc9b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x462bc9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3afebca1, SUC_NE_lexicon_tags_2 },
    { 0xee8cbca2, SUC_NE_lexicon_tags_2 },
    { 0x5bf8bca2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ad4bca5, SUC_NE_lexicon_tags_2 },
    { 0x59c4bca6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb390bca8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70bcab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28b2bcad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe04abcaf, SUC_NE_lexicon_tags_1 },
    { 0x8e46bcb0, SUC_NE_lexicon_tags_2 },
    { 0x7bc0bcaf, SUC_NE_lexicon_tags_2 },
    { 0x53ebcb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8ccbcb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a0bcbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc830bcbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6bcc1, SUC_NE_lexicon_tags_2 },
    { 0x7fdabcc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7506bcc5, SUC_NE_lexicon_tags_2 },
    { 0xef96bcc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x435abcc9, SUC_NE_lexicon_tags_2 },
    { 0xc9e8bcca, SUC_NE_lexicon_tags_2 },
    { 0xac44bccb, SUC_NE_lexicon_tags_2 },
    { 0x5e72bccb, SUC_NE_lexicon_tags_2 },
    { 0xdcccbccb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc278bccf, SUC_NE_lexicon_tags_2 },
    { 0xcf8cbcd0, SUC_NE_lexicon_tags_4 },
    { 0x55f8bccf, SUC_NE_lexicon_tags_2 },
    { 0x6682bccf, SUC_NE_lexicon_tags_2 },
    { 0xef7abcd0, SUC_NE_lexicon_tags_2 },
    { 0xcc6cbcd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e2bcd9, SUC_NE_lexicon_tags_2 },
    { 0xec1abcd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a14bcdd, SUC_NE_lexicon_tags_2 },
    { 0x7abebcde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c4bce1, SUC_NE_lexicon_tags_2 },
    { 0x83abce2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8d4bce4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe608bce8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dabceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330bcf6, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0x5832bcf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x133ebcfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72acbcff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x226ebd01, SUC_NE_lexicon_tags_2 },
    { 0x7dc2bd02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb110bd04, SUC_NE_lexicon_tags_2 },
    { 0xb302bd05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa82bd07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116ebd0f, SUC_NE_lexicon_tags_2 },
    { 0xb8aabd0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe482bd12, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb004bd14, SUC_NE_lexicon_tags_2 },
    { 0x5eecbd14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3238bd1b, SUC_NE_lexicon_tags_2 },
    { 0x19eebd1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffebd1f, SUC_NE_lexicon_tags_2 },
    { 0xa188bd20, SUC_NE_lexicon_tags_4 },
    { 0xb842bd21, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x666abd23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb802bd2b, SUC_NE_lexicon_tags_2 },
    { 0x87f2bd2b, SUC_NE_lexicon_tags_30 },
    { 0x92acbd2d, SUC_NE_lexicon_tags_2 },
    { 0x10f8bd2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x340cbd30, SUC_NE_lexicon_tags_2 },
    { 0x64a0bd31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x168ebd33, SUC_NE_lexicon_tags_2 },
    { 0x33d2bd33, SUC_NE_lexicon_tags_2 },
    { 0x9d7cbd34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe0bd39, SUC_NE_lexicon_tags_2 },
    { 0xd036bd39, SUC_NE_lexicon_tags_2 },
    { 0x9112bd39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cecbd40, SUC_NE_lexicon_tags_2 },
    { 0x810ebd40, SUC_NE_lexicon_tags_2 },
    { 0xd7fcbd42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb622bd47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5854bd4b, SUC_NE_lexicon_tags_2 },
    { 0xf53cbd4b, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0xa688bd4e, SUC_NE_lexicon_tags_2 },
    { 0xe30abd4f, SUC_NE_lexicon_tags_2 },
    { 0x370cbd50, SUC_NE_lexicon_tags_215 },
    { 0xac1abd50, SUC_NE_lexicon_tags_2 },
    { 0x1104bd52, SUC_NE_lexicon_tags_2 },
    { 0x916bd4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d5ebd55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77dabd57, SUC_NE_lexicon_tags_2 },
    { 0x5fe8bd57, SUC_NE_lexicon_tags_2 },
    { 0x1018bd59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd606bd63, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d38bd68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74ebd6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd810bd71, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b8bd75, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x897abd77, SUC_NE_lexicon_tags_2 },
    { 0xb9a8bd77, SUC_NE_lexicon_tags_2 },
    { 0x591abd79, SUC_NE_lexicon_tags_2 },
    { 0x5162bd7a, SUC_NE_lexicon_tags_3 },
    { 0x6724bd7b, SUC_NE_lexicon_tags_34 },
    { 0x8872bd7b, SUC_NE_lexicon_tags_2 },
    { 0xc114bd7b, SUC_NE_lexicon_tags_2 },
    { 0x4f76bd7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf2bd85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf2abd87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86e4bd89, SUC_NE_lexicon_tags_4 },
    { 0x3452bd8a, SUC_NE_lexicon_tags_2 },
    { 0x324abd8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40acbd8e, SUC_NE_lexicon_tags_2 },
    { 0xbe38bd8f, SUC_NE_lexicon_tags_2 },
    { 0x2a16bd90, SUC_NE_lexicon_tags_2 },
    { 0x2b7cbd90, SUC_NE_lexicon_tags_35 },
    { 0x79e2bd92, SUC_NE_lexicon_tags_2 },
    { 0x9050bd93, SUC_NE_lexicon_tags_14 },
    { 0x652abd94, SUC_NE_lexicon_tags_2 },
    { 0x40bebd95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa318bd9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaec6bd9e, SUC_NE_lexicon_tags_2 },
    { 0xb14abd9e, SUC_NE_lexicon_tags_2 },
    { 0xce20bd9f, SUC_NE_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd0bda3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33fabda5, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x92e8bda7, SUC_NE_lexicon_tags_2 },
    { 0x11e6bda8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12eabdab, SUC_NE_lexicon_tags_2 },
    { 0xb5c4bdac, SUC_NE_lexicon_tags_2 },
    { 0xcf3ebdad, SUC_NE_lexicon_tags_2 },
    { 0xb540bdae, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe728bdb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e8bdb8, SUC_NE_lexicon_tags_2 },
    { 0x5ea4bdb9, SUC_NE_lexicon_tags_7 },
    { 0x84f4bdb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9376bdc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb54bdc6, SUC_NE_lexicon_tags_2 },
    { 0xbc2abdc6, SUC_NE_lexicon_tags_1 },
    { 0xce72bdc8, SUC_NE_lexicon_tags_59 },
    { 0x3f14bdc8, SUC_NE_lexicon_tags_2 },
    { 0x6356bdca, SUC_NE_lexicon_tags_2 },
    { 0xb19abdc7, SUC_NE_lexicon_tags_2 },
    { 0x144cbdcc, SUC_NE_lexicon_tags_2 },
    { 0xebe0bdcd, SUC_NE_lexicon_tags_2 },
    { 0x7458bdcd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x69ebdd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59f4bdd2, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310ebdd9, SUC_NE_lexicon_tags_2 },
    { 0xea84bdda, SUC_NE_lexicon_tags_2 },
    { 0xe35cbddb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dcbde6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9b6bde8, SUC_NE_lexicon_tags_2 },
    { 0x7c30bde9, SUC_NE_lexicon_tags_2 },
    { 0x7f00bde9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18ebdf1, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ecbdf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf72bdf6, SUC_NE_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3cbdf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9e4bdfb, SUC_NE_lexicon_tags_0 },
    { 0xf8eabdfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed36bdfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cf6be00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9372be02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65abe07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x327abe09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x984abe0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa72ebe0d, SUC_NE_lexicon_tags_2 },
    { 0xbaa4be0e, SUC_NE_lexicon_tags_2 },
    { 0x85aabe0f, SUC_NE_lexicon_tags_2 },
    { 0x7c5cbe0f, SUC_NE_lexicon_tags_10 },
    { 0x9616be11, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xddb8be13, SUC_NE_lexicon_tags_2 },
    { 0x452ebe13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe31abe16, SUC_NE_lexicon_tags_2 },
    { 0x62e0be17, SUC_NE_lexicon_tags_2 },
    { 0x6678be18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b76be1a, SUC_NE_lexicon_tags_2 },
    { 0x741ebe1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x882ebe1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3febe24, SUC_NE_lexicon_tags_1 },
    { 0x2372be24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6be28, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d2be2e, SUC_NE_lexicon_tags_18 },
    { 0xa1dcbe2e, SUC_NE_lexicon_tags_2 },
    { 0x91b8be30, SUC_NE_lexicon_tags_2 },
    { 0xa004be31, SUC_NE_lexicon_tags_2 },
    { 0xf92cbe32, SUC_NE_lexicon_tags_2 },
    { 0xf0b0be2e, SUC_NE_lexicon_tags_2 },
    { 0x8fdabe33, SUC_NE_lexicon_tags_2 },
    { 0x3160be35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb270be39, SUC_NE_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xc870be3c, SUC_NE_lexicon_tags_2 },
    { 0xa8f0be3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd858be40, SUC_NE_lexicon_tags_2 },
    { 0x6478be40, SUC_NE_lexicon_tags_2 },
    { 0x9410be41, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7310be44, SUC_NE_lexicon_tags_2 },
    { 0x31cebe45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7262be48, SUC_NE_lexicon_tags_2 },
    { 0x947ebe49, SUC_NE_lexicon_tags_2 },
    { 0x5448be4a, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce0be56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad0ebe58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2be5c, SUC_NE_lexicon_tags_2 },
    { 0xf846be5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e4be65, SUC_NE_lexicon_tags_2 },
    { 0xe9d0be66, SUC_NE_lexicon_tags_2 },
    { 0xf83abe65, SUC_NE_lexicon_tags_2 },
    { 0x6a2abe68, SUC_NE_lexicon_tags_2 },
    { 0xa7d2be69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90c2be6b, SUC_NE_lexicon_tags_2 },
    { 0x9f66be6b, SUC_NE_lexicon_tags_2 },
    { 0x61d6be6c, SUC_NE_lexicon_tags_2 },
    { 0x34f8be6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e62be70, SUC_NE_lexicon_tags_2 },
    { 0x171ebe71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd52be73, SUC_NE_lexicon_tags_2 },
    { 0x7694be73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29b2be76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4cbe7a, SUC_NE_lexicon_tags_2 },
    { 0x37aabe7b, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2be83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6abe86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c4be8a, SUC_NE_lexicon_tags_2 },
    { 0x562abe8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00ebe8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9ccbe90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x656be92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e6be94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6942be97, SUC_NE_lexicon_tags_2 },
    { 0xc5febe98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1756be9a, SUC_NE_lexicon_tags_2 },
    { 0x445ebe9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x510ebe9d, SUC_NE_lexicon_tags_2 },
    { 0x4508be9e, SUC_NE_lexicon_tags_104 },
    { 0x2e7cbe9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0abea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bd2bea4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba52bea6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e00beb1, SUC_NE_lexicon_tags_2 },
    { 0x371cbeb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dbebeb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca4beb7, SUC_NE_lexicon_tags_2 },
    { 0xca70beb8, SUC_NE_lexicon_tags_2 },
    { 0xa340beb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f58bebd, SUC_NE_lexicon_tags_2 },
    { 0xfbf4bebe, SUC_NE_lexicon_tags_2 },
    { 0x6174bebe, SUC_NE_lexicon_tags_2 },
    { 0x7340bec0, SUC_NE_lexicon_tags_5 },
    { 0x23c8bec1, SUC_NE_lexicon_tags_2 },
    { 0xb262bec2, SUC_NE_lexicon_tags_2 },
    { 0x5c7abec3, SUC_NE_lexicon_tags_2 },
    { 0xaa2abec4, SUC_NE_lexicon_tags_2 },
    { 0x7d6bec5, SUC_NE_lexicon_tags_2 },
    { 0x67b6bebf, SUC_NE_lexicon_tags_2 },
    { 0x423ebec7, SUC_NE_lexicon_tags_2 },
    { 0x2c1abec7, SUC_NE_lexicon_tags_2 },
    { 0x20febec2, SUC_NE_lexicon_tags_2 },
    { 0x1a90beca, SUC_NE_lexicon_tags_2 },
    { 0x26c6becb, SUC_NE_lexicon_tags_2 },
    { 0x7f6bec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92a2bece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b4bed1, SUC_NE_lexicon_tags_2 },
    { 0x3d4cbed2, SUC_NE_lexicon_tags_2 },
    { 0x97ccbed3, SUC_NE_lexicon_tags_2 },
    { 0x4b22bed4, SUC_NE_lexicon_tags_2 },
    { 0xa492bed5, SUC_NE_lexicon_tags_4 },
    { 0xacdebed5, SUC_NE_lexicon_tags_2 },
    { 0x92f4bed7, SUC_NE_lexicon_tags_2 },
    { 0x1c76bed3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a52beda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d56bedc, SUC_NE_lexicon_tags_2 },
    { 0x6f2cbedc, SUC_NE_lexicon_tags_2 },
    { 0xfd58bede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23abee1, SUC_NE_lexicon_tags_17 },
    { 0x5e24bee2, SUC_NE_lexicon_tags_3 },
    { 0x685cbee2, SUC_NE_lexicon_tags_2 },
    { 0xd336bee4, SUC_NE_lexicon_tags_7 },
    { 0x2ef8bee2, SUC_NE_lexicon_tags_2 },
    { 0x8266bee4, SUC_NE_lexicon_tags_2 },
    { 0x4de2bee5, SUC_NE_lexicon_tags_0 },
    { 0xae98bee7, SUC_NE_lexicon_tags_2 },
    { 0x4b6bee1, SUC_NE_lexicon_tags_2 },
    { 0x5500bee4, SUC_NE_lexicon_tags_2 },
    { 0xfab4beeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6784beee, SUC_NE_lexicon_tags_2 },
    { 0xecbabeee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c58bef1, SUC_NE_lexicon_tags_2 },
    { 0x6876bef1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d84bef4, SUC_NE_lexicon_tags_2 },
    { 0xfeb2bef4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ad2bef7, SUC_NE_lexicon_tags_2 },
    { 0xb9c0bef7, SUC_NE_lexicon_tags_2 },
    { 0x76f4bef9, SUC_NE_lexicon_tags_2 },
    { 0x4882befa, SUC_NE_lexicon_tags_2 },
    { 0xfe8ebef7, SUC_NE_lexicon_tags_2 },
    { 0xb11ebefc, SUC_NE_lexicon_tags_2 },
    { 0xb3e4befd, SUC_NE_lexicon_tags_2 },
    { 0x51cbefc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ccbf01, SUC_NE_lexicon_tags_2 },
    { 0x92d4bf02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eebf05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbcbf0a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x98febf0d, SUC_NE_lexicon_tags_2 },
    { 0xf664bf0e, SUC_NE_lexicon_tags_220 },
    { 0x518cbf0f, SUC_NE_lexicon_tags_2 },
    { 0xce9abf0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a2bf13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea4bf17, SUC_NE_lexicon_tags_2 },
    { 0xf072bf17, SUC_NE_lexicon_tags_2 },
    { 0x4fd8bf19, SUC_NE_lexicon_tags_2 },
    { 0xf1babf17, SUC_NE_lexicon_tags_2 },
    { 0xf61cbf1a, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5632bf1e, SUC_NE_lexicon_tags_18 },
    { 0x46e4bf1f, SUC_NE_lexicon_tags_2 },
    { 0x51e8bf20, SUC_NE_lexicon_tags_2 },
    { 0x6e22bf21, SUC_NE_lexicon_tags_2 },
    { 0x55a2bf21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67abf2d, SUC_NE_lexicon_tags_2 },
    { 0x1406bf2e, SUC_NE_lexicon_tags_2 },
    { 0x6e28bf2f, SUC_NE_lexicon_tags_2 },
    { 0xef6cbf2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe10abf32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3092bf38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8abf3b, SUC_NE_lexicon_tags_2 },
    { 0x716abf3b, SUC_NE_lexicon_tags_2 },
    { 0x7aa4bf3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3030bf3f, SUC_NE_lexicon_tags_2 },
    { 0xd13abf40, SUC_NE_lexicon_tags_2 },
    { 0x3fb8bf40, SUC_NE_lexicon_tags_2 },
    { 0xfbf4bf42, SUC_NE_lexicon_tags_14 },
    { 0xf8f8bf3f, SUC_NE_lexicon_tags_2 },
    { 0xd47ebf41, SUC_NE_lexicon_tags_2 },
    { 0x28f8bf45, SUC_NE_lexicon_tags_18 },
    { 0x29d4bf45, SUC_NE_lexicon_tags_2 },
    { 0xce9cbf47, SUC_NE_lexicon_tags_2 },
    { 0x3f0abf43, SUC_NE_lexicon_tags_2 },
    { 0x75c8bf43, SUC_NE_lexicon_tags_40 },
    { 0xf614bf4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5088bf4d, SUC_NE_lexicon_tags_2 },
    { 0xb7a2bf4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83dabf50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4bf52, SUC_NE_lexicon_tags_298 },
    { 0x4d30bf53, SUC_NE_lexicon_tags_2 },
    { 0x1930bf54, SUC_NE_lexicon_tags_2 },
    { 0x3058bf55, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x344cbf57, SUC_NE_lexicon_tags_4 },
    { 0x6a3ebf57, SUC_NE_lexicon_tags_2 },
    { 0xc594bf59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a2bf5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc84bf61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d28bf63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf000bf65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25a4bf67, SUC_NE_lexicon_tags_2 },
    { 0x1da4bf68, SUC_NE_lexicon_tags_2 },
    { 0xe6d0bf68, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cbf6c, SUC_NE_lexicon_tags_4 },
    { 0x421ebf6c, SUC_NE_lexicon_tags_2 },
    { 0x8f96bf6e, SUC_NE_lexicon_tags_2 },
    { 0xd610bf6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccbf72, SUC_NE_lexicon_tags_2 },
    { 0xbf4abf73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf308bf77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d0bf7b, SUC_NE_lexicon_tags_2 },
    { 0x573cbf7c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a8bf7f, SUC_NE_lexicon_tags_2 },
    { 0xab2bf80, SUC_NE_lexicon_tags_2 },
    { 0xa5ebf81, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9e8bf83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda6bf87, SUC_NE_lexicon_tags_2 },
    { 0x31f6bf88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae84bf8e, SUC_NE_lexicon_tags_118 },
    { 0xb8b2bf8f, SUC_NE_lexicon_tags_2 },
    { 0x6ad0bf90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc048bf93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4adebf95, SUC_NE_lexicon_tags_2 },
    { 0x1a76bf95, SUC_NE_lexicon_tags_2 },
    { 0x5102bf97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8ebf9d, SUC_NE_lexicon_tags_2 },
    { 0xc1a8bf9e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4dcbfa0, SUC_NE_lexicon_tags_2 },
    { 0xc6f6bfa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2bfa4, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xc694bfa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86ebfaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a2bfad, SUC_NE_lexicon_tags_2 },
    { 0x9a4cbfae, SUC_NE_lexicon_tags_2 },
    { 0x7348bfae, SUC_NE_lexicon_tags_2 },
    { 0x4674bfaf, SUC_NE_lexicon_tags_2 },
    { 0xcaa8bfad, SUC_NE_lexicon_tags_1 },
    { 0xb7f2bfb2, SUC_NE_lexicon_tags_2 },
    { 0xbfcabfaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba6bfb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a44bfb8, SUC_NE_lexicon_tags_3 },
    { 0x4726bfb9, SUC_NE_lexicon_tags_2 },
    { 0xb914bfb9, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x32cabfbd, SUC_NE_lexicon_tags_7 },
    { 0xf67abfbe, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f0bfc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbcabfc9, SUC_NE_lexicon_tags_7 },
    { 0x114bfc9, SUC_NE_lexicon_tags_270 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48bfcf, SUC_NE_lexicon_tags_2 },
    { 0x85d6bfcf, SUC_NE_lexicon_tags_2 },
    { 0xb5d0bfd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x297ebfd4, SUC_NE_lexicon_tags_2 },
    { 0x5690bfd5, SUC_NE_lexicon_tags_2 },
    { 0xb3abfd6, SUC_NE_lexicon_tags_2 },
    { 0x3ee4bfd5, SUC_NE_lexicon_tags_2 },
    { 0x7f26bfd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0bfda, SUC_NE_lexicon_tags_7 },
    { 0x626ebfdb, SUC_NE_lexicon_tags_2 },
    { 0x2b26bfdc, SUC_NE_lexicon_tags_2 },
    { 0x2cbebfdd, SUC_NE_lexicon_tags_36 },
    { 0x696abfdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc106bfe1, SUC_NE_lexicon_tags_3 },
    { 0x5aacbfe1, SUC_NE_lexicon_tags_2 },
    { 0x1192bfe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cbfe7, SUC_NE_lexicon_tags_12 },
    { 0x6c10bfe8, SUC_NE_lexicon_tags_2 },
    { 0xd29cbfe9, SUC_NE_lexicon_tags_2 },
    { 0xf884bfea, SUC_NE_lexicon_tags_2 },
    { 0x514bfeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aebfee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd8bfff, SUC_NE_lexicon_tags_2 },
    { 0x1c6ec000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf32ac002, SUC_NE_lexicon_tags_2 },
    { 0xc33ac003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cac009, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c6cc00b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x358ac00d, SUC_NE_lexicon_tags_2 },
    { 0xf478c00e, SUC_NE_lexicon_tags_2 },
    { 0x50c0c00e, SUC_NE_lexicon_tags_2 },
    { 0x260cc010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9096c012, SUC_NE_lexicon_tags_2 },
    { 0x7996c013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7410c018, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69b0c01a, SUC_NE_lexicon_tags_2 },
    { 0x9768c01b, SUC_NE_lexicon_tags_2 },
    { 0x7e08c01b, SUC_NE_lexicon_tags_2 },
    { 0x16cac01d, SUC_NE_lexicon_tags_2 },
    { 0x7e0ec01c, SUC_NE_lexicon_tags_2 },
    { 0x89b4c01b, SUC_NE_lexicon_tags_2 },
    { 0x7504c01d, SUC_NE_lexicon_tags_3 },
    { 0xfa4ac021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5acc023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffec026, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x104ac028, SUC_NE_lexicon_tags_2 },
    { 0xbf9ec028, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5ec02c, SUC_NE_lexicon_tags_2 },
    { 0x6c12c02c, SUC_NE_lexicon_tags_2 },
    { 0xa59ec02e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5ac038, SUC_NE_lexicon_tags_2 },
    { 0x9786c038, SUC_NE_lexicon_tags_2 },
    { 0x4a3ac038, SUC_NE_lexicon_tags_2 },
    { 0x7894c03a, SUC_NE_lexicon_tags_7 },
    { 0xdaccc03c, SUC_NE_lexicon_tags_2 },
    { 0x763ec03c, SUC_NE_lexicon_tags_2 },
    { 0xf30ac03e, SUC_NE_lexicon_tags_2 },
    { 0x91cac03a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54cc049, SUC_NE_lexicon_tags_2 },
    { 0x9b34c04a, SUC_NE_lexicon_tags_2 },
    { 0x100c04a, SUC_NE_lexicon_tags_2 },
    { 0x83b6c049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e0c04e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a86c055, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b0c059, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x478cc05b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63fec05d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a22c060, SUC_NE_lexicon_tags_2 },
    { 0x24b8c061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb12c063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37a4c065, SUC_NE_lexicon_tags_2 },
    { 0xe86ec066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ec06c, SUC_NE_lexicon_tags_2 },
    { 0x2802c06d, SUC_NE_lexicon_tags_2 },
    { 0xcb2ec06e, SUC_NE_lexicon_tags_2 },
    { 0x8a0cc06d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0a0c071, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2980c075, SUC_NE_lexicon_tags_2 },
    { 0xf212c076, SUC_NE_lexicon_tags_2 },
    { 0x23a6c075, SUC_NE_lexicon_tags_16 },
    { 0xc16cc075, SUC_NE_lexicon_tags_2 },
    { 0x466cc079, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b4c07b, SUC_NE_lexicon_tags_2 },
    { 0x32c8c07c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff8c07f, SUC_NE_lexicon_tags_3 },
    { 0xd8f2c080, SUC_NE_lexicon_tags_2 },
    { 0xe138c081, SUC_NE_lexicon_tags_2 },
    { 0xb26cc082, SUC_NE_lexicon_tags_2 },
    { 0x98dac083, SUC_NE_lexicon_tags_2 },
    { 0x6e30c083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x473ec086, SUC_NE_lexicon_tags_2 },
    { 0x822ac087, SUC_NE_lexicon_tags_2 },
    { 0xc174c088, SUC_NE_lexicon_tags_2 },
    { 0x1a2ec087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6cc08d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0ec08f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52d4c091, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe12c095, SUC_NE_lexicon_tags_2 },
    { 0x2164c096, SUC_NE_lexicon_tags_3 },
    { 0xdd28c097, SUC_NE_lexicon_tags_2 },
    { 0xf820c098, SUC_NE_lexicon_tags_2 },
    { 0xb1ec099, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8e0c09b, SUC_NE_lexicon_tags_2 },
    { 0xe6ecc09b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5892c0a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff6c0a9, SUC_NE_lexicon_tags_2 },
    { 0xa4d6c0aa, SUC_NE_lexicon_tags_2 },
    { 0x8edec0ab, SUC_NE_lexicon_tags_2 },
    { 0xb0bec0ab, SUC_NE_lexicon_tags_2 },
    { 0x5e48c0ad, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x280cc0b0, SUC_NE_lexicon_tags_2 },
    { 0x3f64c0b1, SUC_NE_lexicon_tags_2 },
    { 0x647cc0b2, SUC_NE_lexicon_tags_1 },
    { 0x7090c0b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e6c0b7, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5cc0bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ac0be, SUC_NE_lexicon_tags_2 },
    { 0x3398c0be, SUC_NE_lexicon_tags_2 },
    { 0x268cc0bf, SUC_NE_lexicon_tags_2 },
    { 0x140c0c1, SUC_NE_lexicon_tags_2 },
    { 0x78eec0bf, SUC_NE_lexicon_tags_2 },
    { 0xf234c0c1, SUC_NE_lexicon_tags_2 },
    { 0xba6ac0c0, SUC_NE_lexicon_tags_2 },
    { 0x7b5cc0c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef2c0c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc0ce, SUC_NE_lexicon_tags_2 },
    { 0x6d8cc0cf, SUC_NE_lexicon_tags_2 },
    { 0xbf6c0d0, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ea4c0d2, SUC_NE_lexicon_tags_2 },
    { 0xc718c0d3, SUC_NE_lexicon_tags_2 },
    { 0xe8ec0d4, SUC_NE_lexicon_tags_2 },
    { 0xeb9ac0d4, SUC_NE_lexicon_tags_2 },
    { 0x2ddac0d6, SUC_NE_lexicon_tags_2 },
    { 0x993cc0d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x684ec0d9, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14acc0de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb450c0e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e64c0e7, SUC_NE_lexicon_tags_2 },
    { 0xf16ec0e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe090c0ea, SUC_NE_lexicon_tags_2 },
    { 0x38f2c0eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f3ac0ed, SUC_NE_lexicon_tags_2 },
    { 0x383ec0ed, SUC_NE_lexicon_tags_2 },
    { 0xb084c0ed, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0xba80c0f1, SUC_NE_lexicon_tags_19 },
    { 0x4e10c0f2, SUC_NE_lexicon_tags_6 },
    { 0xc9bcc0f2, SUC_NE_lexicon_tags_2 },
    { 0xb7bcc0f1, SUC_NE_lexicon_tags_2 },
    { 0x4c4ac0f4, SUC_NE_lexicon_tags_2 },
    { 0x1a74c0f2, SUC_NE_lexicon_tags_2 },
    { 0xd4acc0f7, SUC_NE_lexicon_tags_2 },
    { 0xbc02c0f8, SUC_NE_lexicon_tags_2 },
    { 0x9d18c0f9, SUC_NE_lexicon_tags_2 },
    { 0x6622c0f6, SUC_NE_lexicon_tags_2 },
    { 0xaa46c0f9, SUC_NE_lexicon_tags_2 },
    { 0x77ccc0fc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa09ec0fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b2c101, SUC_NE_lexicon_tags_3 },
    { 0xa8f4c102, SUC_NE_lexicon_tags_2 },
    { 0x69dcc103, SUC_NE_lexicon_tags_2 },
    { 0xdc3ec104, SUC_NE_lexicon_tags_2 },
    { 0x41f4c104, SUC_NE_lexicon_tags_2 },
    { 0xba36c105, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4f2c108, SUC_NE_lexicon_tags_2 },
    { 0x4ff0c109, SUC_NE_lexicon_tags_2 },
    { 0xfceec108, SUC_NE_lexicon_tags_336 },
    { 0, NULL },
    { 0xa498c10c, SUC_NE_lexicon_tags_1 },
    { 0x3c18c10d, SUC_NE_lexicon_tags_2 },
    { 0xb89ac10c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ecc111, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x345cc113, SUC_NE_lexicon_tags_2 },
    { 0x957cc113, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd12c116, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fac11c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa754c11e, SUC_NE_lexicon_tags_2 },
    { 0x8c4ec11f, SUC_NE_lexicon_tags_2 },
    { 0x207ec11f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ec129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf81ec12b, SUC_NE_lexicon_tags_2 },
    { 0x6ceac12c, SUC_NE_lexicon_tags_7 },
    { 0xf392c12b, SUC_NE_lexicon_tags_2 },
    { 0x2110c12e, SUC_NE_lexicon_tags_3 },
    { 0xf81ec12f, SUC_NE_lexicon_tags_2 },
    { 0x993cc130, SUC_NE_lexicon_tags_2 },
    { 0xd44ac131, SUC_NE_lexicon_tags_2 },
    { 0x3874c12f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de0c135, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1ec13a, SUC_NE_lexicon_tags_1 },
    { 0xe6dcc13a, SUC_NE_lexicon_tags_2 },
    { 0x2ca4c13c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfec13f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8cc142, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696ec146, SUC_NE_lexicon_tags_2 },
    { 0xbff0c147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a96c14a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4b8c14c, SUC_NE_lexicon_tags_2 },
    { 0x1e0ec14c, SUC_NE_lexicon_tags_2 },
    { 0x427ac14e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f6c153, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c157, SUC_NE_lexicon_tags_3 },
    { 0x2904c158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefd4c15a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5198c15c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab2ec15e, SUC_NE_lexicon_tags_3 },
    { 0xed8c15e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd176c161, SUC_NE_lexicon_tags_2 },
    { 0x8746c161, SUC_NE_lexicon_tags_2 },
    { 0x2b88c163, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38ac166, SUC_NE_lexicon_tags_0 },
    { 0xd612c167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2146c169, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f6c16c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb7cc16e, SUC_NE_lexicon_tags_2 },
    { 0xaee4c16e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x354cc172, SUC_NE_lexicon_tags_2 },
    { 0x5a20c172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157cc177, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c96c179, SUC_NE_lexicon_tags_2 },
    { 0x79aac17a, SUC_NE_lexicon_tags_2 },
    { 0xdd78c179, SUC_NE_lexicon_tags_2 },
    { 0x7dd2c17c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1962c181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x541ec183, SUC_NE_lexicon_tags_2 },
    { 0xfb40c184, SUC_NE_lexicon_tags_2 },
    { 0x5d6ac183, SUC_NE_lexicon_tags_2 },
    { 0xf912c186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b8c190, SUC_NE_lexicon_tags_2 },
    { 0x96e4c190, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5456c193, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8992c198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc19f, SUC_NE_lexicon_tags_188 },
    { 0x8ef8c1a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4538c1a5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2ebcc1a7, SUC_NE_lexicon_tags_2 },
    { 0xae9ac1a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce4c1aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9cc1ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07ac1b6, SUC_NE_lexicon_tags_2 },
    { 0x2420c1b7, SUC_NE_lexicon_tags_2 },
    { 0xe71ac1b7, SUC_NE_lexicon_tags_2 },
    { 0xd2f4c1b6, SUC_NE_lexicon_tags_2 },
    { 0x153cc1b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe250c1bf, SUC_NE_lexicon_tags_2 },
    { 0x70c2c1c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x193ac1c2, SUC_NE_lexicon_tags_2 },
    { 0xad5cc1c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c4c1c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e2c1ca, SUC_NE_lexicon_tags_14 },
    { 0xc3c6c1cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x623ac1cd, SUC_NE_lexicon_tags_2 },
    { 0x9bc0c1cd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x81d6c1d0, SUC_NE_lexicon_tags_4 },
    { 0xbe70c1d0, SUC_NE_lexicon_tags_2 },
    { 0x7a3ec1d2, SUC_NE_lexicon_tags_24 },
    { 0xaa2c1d2, SUC_NE_lexicon_tags_2 },
    { 0x32f8c1d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91bcc1d6, SUC_NE_lexicon_tags_2 },
    { 0xbe42c1d7, SUC_NE_lexicon_tags_2 },
    { 0x3368c1d8, SUC_NE_lexicon_tags_2 },
    { 0x17bac1d9, SUC_NE_lexicon_tags_2 },
    { 0xb81ec1d9, SUC_NE_lexicon_tags_2 },
    { 0x280c1db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ecc1de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f4c1e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e86c1e9, SUC_NE_lexicon_tags_2 },
    { 0xa1a6c1ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48ec1ed, SUC_NE_lexicon_tags_2 },
    { 0x19a4c1ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b34c1f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8442c1f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdec1fa, SUC_NE_lexicon_tags_2 },
    { 0xeb76c1fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce96c202, SUC_NE_lexicon_tags_2 },
    { 0x220ac203, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfe8c205, SUC_NE_lexicon_tags_2 },
    { 0xbe98c205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cec20a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7ec20d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a96c212, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x296cc215, SUC_NE_lexicon_tags_2 },
    { 0xfb08c215, SUC_NE_lexicon_tags_2 },
    { 0x79ac217, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x211cc219, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1104c21e, SUC_NE_lexicon_tags_2 },
    { 0xacdcc21e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaec223, SUC_NE_lexicon_tags_2 },
    { 0x6a24c224, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4ac22c, SUC_NE_lexicon_tags_209 },
    { 0, NULL },
    { 0, NULL },
    { 0x139cc22f, SUC_NE_lexicon_tags_2 },
    { 0xee84c230, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1008c239, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abcc23f, SUC_NE_lexicon_tags_2 },
    { 0xd402c23f, SUC_NE_lexicon_tags_17 },
    { 0x4fa0c241, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x340c244, SUC_NE_lexicon_tags_2 },
    { 0x21bec245, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54d4c247, SUC_NE_lexicon_tags_2 },
    { 0x509cc247, SUC_NE_lexicon_tags_2 },
    { 0xe13ec249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f2c255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a40c257, SUC_NE_lexicon_tags_2 },
    { 0xd514c258, SUC_NE_lexicon_tags_2 },
    { 0xb932c259, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf04c25b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5ec25e, SUC_NE_lexicon_tags_2 },
    { 0xf74ec25e, SUC_NE_lexicon_tags_2 },
    { 0x7614c25e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0c263, SUC_NE_lexicon_tags_2 },
    { 0x3820c264, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb2c267, SUC_NE_lexicon_tags_2 },
    { 0xde84c268, SUC_NE_lexicon_tags_2 },
    { 0x4df0c269, SUC_NE_lexicon_tags_2 },
    { 0x8be6c269, SUC_NE_lexicon_tags_2 },
    { 0x5dcc269, SUC_NE_lexicon_tags_2 },
    { 0xe3eec26a, SUC_NE_lexicon_tags_186 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60eac272, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4910c274, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738cc27a, SUC_NE_lexicon_tags_2 },
    { 0xb592c27b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb90c27d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c14c280, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e2c284, SUC_NE_lexicon_tags_2 },
    { 0x108ac285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37ac288, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e0c28c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748c291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf224c293, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb4c296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5014c298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b52c29a, SUC_NE_lexicon_tags_2 },
    { 0x5e1ac29a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x833ec29e, SUC_NE_lexicon_tags_2 },
    { 0x8a30c29e, SUC_NE_lexicon_tags_2 },
    { 0x12bcc2a0, SUC_NE_lexicon_tags_2 },
    { 0x431ec2a0, SUC_NE_lexicon_tags_2 },
    { 0x43f2c2a2, SUC_NE_lexicon_tags_2 },
    { 0x820ac2a3, SUC_NE_lexicon_tags_1 },
    { 0xedc2c2a3, SUC_NE_lexicon_tags_2 },
    { 0xad7ac29f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4ce0c2a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c6c2ac, SUC_NE_lexicon_tags_2 },
    { 0xf014c2ad, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91ec2b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f2c2b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c5ac2b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5598c2bd, SUC_NE_lexicon_tags_2 },
    { 0xca50c2be, SUC_NE_lexicon_tags_0 },
    { 0xd76cc2bf, SUC_NE_lexicon_tags_2 },
    { 0x7062c2c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4dcc2c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f0c2c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3704c2c7, SUC_NE_lexicon_tags_2 },
    { 0x596ac2c8, SUC_NE_lexicon_tags_2 },
    { 0xafccc2c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8612c2cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9cc2d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f72c2d4, SUC_NE_lexicon_tags_2 },
    { 0xe90ec2d5, SUC_NE_lexicon_tags_2 },
    { 0x9e70c2d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ec2de, SUC_NE_lexicon_tags_2 },
    { 0x648c2de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c60c2e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6920c2e8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x468c2ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2e8c2ec, SUC_NE_lexicon_tags_2 },
    { 0xc5a0c2ec, SUC_NE_lexicon_tags_2 },
    { 0xadc0c2ee, SUC_NE_lexicon_tags_2 },
    { 0x93a6c2ef, SUC_NE_lexicon_tags_2 },
    { 0x42eac2f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3052c2f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2258c2f5, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0xc0bcc2f7, SUC_NE_lexicon_tags_2 },
    { 0x9dbcc2f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1966c2fa, SUC_NE_lexicon_tags_2 },
    { 0x86dcc2fb, SUC_NE_lexicon_tags_2 },
    { 0x129ec2fb, SUC_NE_lexicon_tags_2 },
    { 0x8beac2fd, SUC_NE_lexicon_tags_40 },
    { 0x8a98c2fd, SUC_NE_lexicon_tags_2 },
    { 0xe0aac2fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1ccc301, SUC_NE_lexicon_tags_2 },
    { 0xb67cc302, SUC_NE_lexicon_tags_2 },
    { 0xeb4ec303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae0c306, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205cc30a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cec311, SUC_NE_lexicon_tags_2 },
    { 0x19e8c312, SUC_NE_lexicon_tags_2 },
    { 0x60b6c311, SUC_NE_lexicon_tags_2 },
    { 0x2a0c314, SUC_NE_lexicon_tags_2 },
    { 0xc084c315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa322c319, SUC_NE_lexicon_tags_2 },
    { 0x4adcc31a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28ec31c, SUC_NE_lexicon_tags_2 },
    { 0x360cc31d, SUC_NE_lexicon_tags_2 },
    { 0x393cc31d, SUC_NE_lexicon_tags_2 },
    { 0xcd30c31f, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f66c322, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dac327, SUC_NE_lexicon_tags_2 },
    { 0x8098c328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf36c32a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9020c32e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7cc330, SUC_NE_lexicon_tags_2 },
    { 0xf276c330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e22c334, SUC_NE_lexicon_tags_2 },
    { 0x6bb2c335, SUC_NE_lexicon_tags_2 },
    { 0x72cc334, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc8c33d, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd4c340, SUC_NE_lexicon_tags_2 },
    { 0xd4eac341, SUC_NE_lexicon_tags_2 },
    { 0xfe50c341, SUC_NE_lexicon_tags_2 },
    { 0xd2dac340, SUC_NE_lexicon_tags_2 },
    { 0xd8cac341, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xde44c346, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe750c348, SUC_NE_lexicon_tags_2 },
    { 0x993ec349, SUC_NE_lexicon_tags_2 },
    { 0xda98c34a, SUC_NE_lexicon_tags_3 },
    { 0x4b0ec349, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9236c352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1ec356, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x294c358, SUC_NE_lexicon_tags_2 },
    { 0x6c0ec358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc386c35c, SUC_NE_lexicon_tags_1 },
    { 0x55a8c35c, SUC_NE_lexicon_tags_2 },
    { 0x97bac35c, SUC_NE_lexicon_tags_2 },
    { 0x9cecc35f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a2c367, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c92c369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a2c36c, SUC_NE_lexicon_tags_2 },
    { 0xc7d6c36c, SUC_NE_lexicon_tags_3 },
    { 0xeb92c36c, SUC_NE_lexicon_tags_2 },
    { 0x5ff8c36c, SUC_NE_lexicon_tags_2 },
    { 0x21e4c370, SUC_NE_lexicon_tags_2 },
    { 0xccf8c371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1ec37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0ec37e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51c6c380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c96c383, SUC_NE_lexicon_tags_2 },
    { 0x7804c384, SUC_NE_lexicon_tags_3 },
    { 0xe2f8c383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbec388, SUC_NE_lexicon_tags_2 },
    { 0x868ec388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f0cc38b, SUC_NE_lexicon_tags_4 },
    { 0x9ca8c38b, SUC_NE_lexicon_tags_2 },
    { 0xd294c38d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6fcc38f, SUC_NE_lexicon_tags_2 },
    { 0x1cc38f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26ec398, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x620cc39a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x789cc39c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a4ec39e, SUC_NE_lexicon_tags_2 },
    { 0xdb44c39f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7236c3a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1558c3a8, SUC_NE_lexicon_tags_2 },
    { 0xd17ac3a9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x38fcc3ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ac3af, SUC_NE_lexicon_tags_2 },
    { 0x858cc3b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ac3b7, SUC_NE_lexicon_tags_7 },
    { 0x2cb4c3b8, SUC_NE_lexicon_tags_2 },
    { 0xbe74c3b8, SUC_NE_lexicon_tags_229 },
    { 0xa94cc3ba, SUC_NE_lexicon_tags_2 },
    { 0xc318c3ba, SUC_NE_lexicon_tags_2 },
    { 0xbbacc3ba, SUC_NE_lexicon_tags_2 },
    { 0x3a5ec3bd, SUC_NE_lexicon_tags_2 },
    { 0x2c5ec3bd, SUC_NE_lexicon_tags_2 },
    { 0xe6e0c3bd, SUC_NE_lexicon_tags_2 },
    { 0x962ec3bc, SUC_NE_lexicon_tags_2 },
    { 0xf85ac3b9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1020c3c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x824ec3c7, SUC_NE_lexicon_tags_2 },
    { 0xa16ec3c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfac3cc, SUC_NE_lexicon_tags_2 },
    { 0x98f8c3cc, SUC_NE_lexicon_tags_2 },
    { 0x2c8cc3ce, SUC_NE_lexicon_tags_2 },
    { 0x5934c3ce, SUC_NE_lexicon_tags_2 },
    { 0x8ce2c3d0, SUC_NE_lexicon_tags_2 },
    { 0x1776c3cf, SUC_NE_lexicon_tags_3 },
    { 0x9948c3d2, SUC_NE_lexicon_tags_2 },
    { 0xb97cc3d1, SUC_NE_lexicon_tags_2 },
    { 0xa960c3ce, SUC_NE_lexicon_tags_2 },
    { 0x5424c3d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ec3db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c32c3de, SUC_NE_lexicon_tags_2 },
    { 0xf924c3df, SUC_NE_lexicon_tags_2 },
    { 0xc108c3de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebac3e5, SUC_NE_lexicon_tags_2 },
    { 0x573cc3e6, SUC_NE_lexicon_tags_2 },
    { 0x57c6c3e7, SUC_NE_lexicon_tags_2 },
    { 0xcc84c3e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe28c3eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00c3ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622c3f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca6c3f7, SUC_NE_lexicon_tags_2 },
    { 0x2cdac3f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd722c3fa, SUC_NE_lexicon_tags_7 },
    { 0xcc3ec3fb, SUC_NE_lexicon_tags_2 },
    { 0x2702c3fc, SUC_NE_lexicon_tags_2 },
    { 0xc130c3fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835ec401, SUC_NE_lexicon_tags_2 },
    { 0x3a80c401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8524c406, SUC_NE_lexicon_tags_2 },
    { 0x60bac407, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666c40d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6644c410, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa100c412, SUC_NE_lexicon_tags_2 },
    { 0xadc0c413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ecc416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffac419, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede2c41e, SUC_NE_lexicon_tags_2 },
    { 0x3c6cc41e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a8c424, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbce8c426, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5db6c428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fcc42d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfcc432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114c438, SUC_NE_lexicon_tags_2 },
    { 0x1b94c438, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6952c43b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cc444, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf640c448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf714c44a, SUC_NE_lexicon_tags_2 },
    { 0x26a4c44b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d4c44f, SUC_NE_lexicon_tags_2 },
    { 0x285cc44f, SUC_NE_lexicon_tags_2 },
    { 0x37cac450, SUC_NE_lexicon_tags_2 },
    { 0x59d0c452, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cc457, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe568c459, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x6d2cc45b, SUC_NE_lexicon_tags_2 },
    { 0xee94c45b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac4c460, SUC_NE_lexicon_tags_2 },
    { 0xea0cc461, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc06c463, SUC_NE_lexicon_tags_2 },
    { 0xb458c463, SUC_NE_lexicon_tags_2 },
    { 0x24c4c465, SUC_NE_lexicon_tags_2 },
    { 0x9b50c466, SUC_NE_lexicon_tags_4 },
    { 0xe146c465, SUC_NE_lexicon_tags_2 },
    { 0x2e48c468, SUC_NE_lexicon_tags_2 },
    { 0x38f4c467, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945cc474, SUC_NE_lexicon_tags_2 },
    { 0xd028c475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6c479, SUC_NE_lexicon_tags_2 },
    { 0x70eac47a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dac480, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf66c485, SUC_NE_lexicon_tags_2 },
    { 0x4d62c486, SUC_NE_lexicon_tags_2 },
    { 0x4718c487, SUC_NE_lexicon_tags_4 },
    { 0xc77cc488, SUC_NE_lexicon_tags_2 },
    { 0xde8ec489, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e18c48f, SUC_NE_lexicon_tags_2 },
    { 0xdc1ac490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e46c492, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6478c497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x104cc49a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x953ec49d, SUC_NE_lexicon_tags_2 },
    { 0x69a0c49d, SUC_NE_lexicon_tags_2 },
    { 0x6540c49f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7832c4a2, SUC_NE_lexicon_tags_2 },
    { 0x38e6c4a2, SUC_NE_lexicon_tags_2 },
    { 0xc8d0c4a4, SUC_NE_lexicon_tags_2 },
    { 0x76bcc4a5, SUC_NE_lexicon_tags_2 },
    { 0x8b04c4a5, SUC_NE_lexicon_tags_2 },
    { 0xccf0c4a2, SUC_NE_lexicon_tags_2 },
    { 0x4e2ac4a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe918c4ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4e4c4ae, SUC_NE_lexicon_tags_2 },
    { 0xd328c4ae, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63ac4b2, SUC_NE_lexicon_tags_2 },
    { 0x254ac4b3, SUC_NE_lexicon_tags_2 },
    { 0xdf1cc4b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf74c4b6, SUC_NE_lexicon_tags_2 },
    { 0x15dcc4b6, SUC_NE_lexicon_tags_2 },
    { 0x595ac4b6, SUC_NE_lexicon_tags_2 },
    { 0x6d18c4b7, SUC_NE_lexicon_tags_2 },
    { 0x16e8c4b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8690c4bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc466c4c7, SUC_NE_lexicon_tags_2 },
    { 0xa95cc4c7, SUC_NE_lexicon_tags_2 },
    { 0xd4b8c4c8, SUC_NE_lexicon_tags_3 },
    { 0xf6ec4c9, SUC_NE_lexicon_tags_2 },
    { 0x9bfac4cb, SUC_NE_lexicon_tags_2 },
    { 0x465ac4cc, SUC_NE_lexicon_tags_2 },
    { 0x9ebec4cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfec4d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d64c4d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6974c4d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3382c4d8, SUC_NE_lexicon_tags_2 },
    { 0x9594c4d9, SUC_NE_lexicon_tags_2 },
    { 0x2adac4da, SUC_NE_lexicon_tags_2 },
    { 0x52fec4db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa94c4de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5114c4e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6c4ee, SUC_NE_lexicon_tags_2 },
    { 0xc81ac4ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cac4fc, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x682ac4fe, SUC_NE_lexicon_tags_7 },
    { 0xf140c4ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xedd2c501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611ec505, SUC_NE_lexicon_tags_2 },
    { 0x7d98c505, SUC_NE_lexicon_tags_2 },
    { 0xd106c507, SUC_NE_lexicon_tags_2 },
    { 0x6370c508, SUC_NE_lexicon_tags_2 },
    { 0xc736c507, SUC_NE_lexicon_tags_2 },
    { 0x8e44c50a, SUC_NE_lexicon_tags_2 },
    { 0xb1dcc50a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3656c511, SUC_NE_lexicon_tags_2 },
    { 0x25dec512, SUC_NE_lexicon_tags_2 },
    { 0x107cc513, SUC_NE_lexicon_tags_2 },
    { 0x5c4cc513, SUC_NE_lexicon_tags_2 },
    { 0xd088c515, SUC_NE_lexicon_tags_2 },
    { 0x8f2c514, SUC_NE_lexicon_tags_4 },
    { 0x7c9cc517, SUC_NE_lexicon_tags_2 },
    { 0x7da4c518, SUC_NE_lexicon_tags_1 },
    { 0x13aac519, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a32c51c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dacc526, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x477ac528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30bcc52a, SUC_NE_lexicon_tags_2 },
    { 0x957cc52b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb76ec52d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f6c531, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668ac535, SUC_NE_lexicon_tags_2 },
    { 0x26e2c536, SUC_NE_lexicon_tags_2 },
    { 0xf6b2c535, SUC_NE_lexicon_tags_2 },
    { 0x2b5ac538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x952c53b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x902c53d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7fcc53f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x524ac541, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa286c543, SUC_NE_lexicon_tags_2 },
    { 0xbe04c544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70cc547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9fec549, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa43ec54b, SUC_NE_lexicon_tags_2 },
    { 0xb70c54c, SUC_NE_lexicon_tags_2 },
    { 0xaec2c54b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ae0c54f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8972c552, SUC_NE_lexicon_tags_2 },
    { 0x2ed6c553, SUC_NE_lexicon_tags_2 },
    { 0xc204c554, SUC_NE_lexicon_tags_2 },
    { 0xfc9ac553, SUC_NE_lexicon_tags_2 },
    { 0xc686c556, SUC_NE_lexicon_tags_2 },
    { 0xb4dec557, SUC_NE_lexicon_tags_2 },
    { 0x1c36c557, SUC_NE_lexicon_tags_2 },
    { 0xc22c559, SUC_NE_lexicon_tags_2 },
    { 0xf54cc55a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6c55d, SUC_NE_lexicon_tags_2 },
    { 0x12aec55d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7650c562, SUC_NE_lexicon_tags_19 },
    { 0x93d0c562, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92aec566, SUC_NE_lexicon_tags_2 },
    { 0xe3acc567, SUC_NE_lexicon_tags_2 },
    { 0x3f76c567, SUC_NE_lexicon_tags_2 },
    { 0x2e74c567, SUC_NE_lexicon_tags_2 },
    { 0x1290c56a, SUC_NE_lexicon_tags_2 },
    { 0xf338c56b, SUC_NE_lexicon_tags_2 },
    { 0x85c4c56c, SUC_NE_lexicon_tags_2 },
    { 0x1c16c56d, SUC_NE_lexicon_tags_98 },
    { 0x7ef4c56d, SUC_NE_lexicon_tags_2 },
    { 0xf08ec56f, SUC_NE_lexicon_tags_2 },
    { 0x9356c570, SUC_NE_lexicon_tags_2 },
    { 0xad32c56f, SUC_NE_lexicon_tags_2 },
    { 0x239cc570, SUC_NE_lexicon_tags_75 },
    { 0x92aac56b, SUC_NE_lexicon_tags_2 },
    { 0x4012c572, SUC_NE_lexicon_tags_2 },
    { 0xb7a2c574, SUC_NE_lexicon_tags_5 },
    { 0x5864c576, SUC_NE_lexicon_tags_2 },
    { 0xd88c577, SUC_NE_lexicon_tags_2 },
    { 0xb980c577, SUC_NE_lexicon_tags_2 },
    { 0x283ec579, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0acc57b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb0c584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe214c58b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c82c58f, SUC_NE_lexicon_tags_2 },
    { 0x4ea0c590, SUC_NE_lexicon_tags_0 },
    { 0x948ac590, SUC_NE_lexicon_tags_2 },
    { 0xbb02c58f, SUC_NE_lexicon_tags_2 },
    { 0x8ffec590, SUC_NE_lexicon_tags_2 },
    { 0x2bb8c594, SUC_NE_lexicon_tags_2 },
    { 0xff0c595, SUC_NE_lexicon_tags_2 },
    { 0x2694c596, SUC_NE_lexicon_tags_2 },
    { 0x315ac596, SUC_NE_lexicon_tags_8 },
    { 0xe548c597, SUC_NE_lexicon_tags_414 },
    { 0x2732c599, SUC_NE_lexicon_tags_2 },
    { 0x3752c590, SUC_NE_lexicon_tags_2 },
    { 0x87aec59b, SUC_NE_lexicon_tags_2 },
    { 0x734ac59c, SUC_NE_lexicon_tags_2 },
    { 0x2a04c59d, SUC_NE_lexicon_tags_2 },
    { 0x8b0ec59c, SUC_NE_lexicon_tags_2 },
    { 0xad34c59f, SUC_NE_lexicon_tags_2 },
    { 0x3134c59b, SUC_NE_lexicon_tags_2 },
    { 0x920ac597, SUC_NE_lexicon_tags_2 },
    { 0xf562c5a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d28c5a9, SUC_NE_lexicon_tags_2 },
    { 0x2658c5a9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532c5af, SUC_NE_lexicon_tags_4 },
    { 0x2f38c5af, SUC_NE_lexicon_tags_7 },
    { 0xf64c5b1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86ec5b6, SUC_NE_lexicon_tags_164 },
    { 0x608cc5b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb978c5bb, SUC_NE_lexicon_tags_2 },
    { 0xdd42c5bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb12cc5be, SUC_NE_lexicon_tags_2 },
    { 0xce4c5be, SUC_NE_lexicon_tags_2 },
    { 0x8700c5bf, SUC_NE_lexicon_tags_2 },
    { 0x1f90c5c1, SUC_NE_lexicon_tags_2 },
    { 0x8b2cc5c2, SUC_NE_lexicon_tags_2 },
    { 0x4190c5c3, SUC_NE_lexicon_tags_2 },
    { 0xa4f4c5c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bec5c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a28c5cb, SUC_NE_lexicon_tags_2 },
    { 0xd2acc5cb, SUC_NE_lexicon_tags_3 },
    { 0x853ec5cd, SUC_NE_lexicon_tags_7 },
    { 0x6df8c5cd, SUC_NE_lexicon_tags_2 },
    { 0xc446c5cf, SUC_NE_lexicon_tags_2 },
    { 0x3e82c5d0, SUC_NE_lexicon_tags_37 },
    { 0x45ecc5d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc972c5d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa908c5d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeab0c5da, SUC_NE_lexicon_tags_2 },
    { 0x6b88c5da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf144c5de, SUC_NE_lexicon_tags_7 },
    { 0x405cc5df, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60cc5e2, SUC_NE_lexicon_tags_259 },
    { 0, NULL },
    { 0xbb3cc5e4, SUC_NE_lexicon_tags_2 },
    { 0x71c2c5e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa8ac5e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c4c5ec, SUC_NE_lexicon_tags_2 },
    { 0xe9d4c5ec, SUC_NE_lexicon_tags_2 },
    { 0x308c5ec, SUC_NE_lexicon_tags_2 },
    { 0x6dd0c5ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5070c5f3, SUC_NE_lexicon_tags_2 },
    { 0xe31ac5f4, SUC_NE_lexicon_tags_7 },
    { 0x47e2c5f3, SUC_NE_lexicon_tags_2 },
    { 0x45d8c5f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x660cc5f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28c5fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c0c5fe, SUC_NE_lexicon_tags_2 },
    { 0x85a4c5fe, SUC_NE_lexicon_tags_70 },
    { 0xf08cc5fe, SUC_NE_lexicon_tags_2 },
    { 0x1c50c601, SUC_NE_lexicon_tags_2 },
    { 0xf1b2c602, SUC_NE_lexicon_tags_2 },
    { 0x76dac603, SUC_NE_lexicon_tags_2 },
    { 0x9ba2c603, SUC_NE_lexicon_tags_2 },
    { 0x8fdcc604, SUC_NE_lexicon_tags_2 },
    { 0x48c601, SUC_NE_lexicon_tags_2 },
    { 0x7f08c601, SUC_NE_lexicon_tags_4 },
    { 0x8f14c606, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b8c60e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xce7ac610, SUC_NE_lexicon_tags_2 },
    { 0x2dfec611, SUC_NE_lexicon_tags_2 },
    { 0x769cc610, SUC_NE_lexicon_tags_2 },
    { 0xed6ac611, SUC_NE_lexicon_tags_2 },
    { 0xafb8c614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2882c616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56f0c618, SUC_NE_lexicon_tags_34 },
    { 0x525ac618, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a6cc61b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2c61f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82e0c621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5542c624, SUC_NE_lexicon_tags_1 },
    { 0xfd62c625, SUC_NE_lexicon_tags_2 },
    { 0x46c8c626, SUC_NE_lexicon_tags_2 },
    { 0x3b16c625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd6ec629, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xc3e8c62b, SUC_NE_lexicon_tags_2 },
    { 0xcd00c62c, SUC_NE_lexicon_tags_2 },
    { 0x22d2c62c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e7ec62f, SUC_NE_lexicon_tags_2 },
    { 0xb0e8c630, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2880c633, SUC_NE_lexicon_tags_18 },
    { 0x96bcc634, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5528c637, SUC_NE_lexicon_tags_2 },
    { 0x43e2c638, SUC_NE_lexicon_tags_2 },
    { 0x3016c638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1cc640, SUC_NE_lexicon_tags_2 },
    { 0x6ddec640, SUC_NE_lexicon_tags_2 },
    { 0x42eac642, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd2c646, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798ec64a, SUC_NE_lexicon_tags_2 },
    { 0x11d2c64b, SUC_NE_lexicon_tags_2 },
    { 0x8afec64b, SUC_NE_lexicon_tags_2 },
    { 0x2a1ac64d, SUC_NE_lexicon_tags_2 },
    { 0xc0ac64b, SUC_NE_lexicon_tags_2 },
    { 0xedf0c64a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4694c652, SUC_NE_lexicon_tags_40 },
    { 0x11a0c652, SUC_NE_lexicon_tags_2 },
    { 0xff28c654, SUC_NE_lexicon_tags_2 },
    { 0x3566c655, SUC_NE_lexicon_tags_2 },
    { 0xf7d4c655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ed0c658, SUC_NE_lexicon_tags_1 },
    { 0x6774c659, SUC_NE_lexicon_tags_7 },
    { 0x2350c65a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23c0c65c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eec660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b8ac662, SUC_NE_lexicon_tags_2 },
    { 0x7cecc663, SUC_NE_lexicon_tags_2 },
    { 0xa038c664, SUC_NE_lexicon_tags_2 },
    { 0xe7c6c662, SUC_NE_lexicon_tags_13 },
    { 0xff20c666, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ccc66a, SUC_NE_lexicon_tags_2 },
    { 0x4554c66b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7910c66d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1e2c66f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0ac671, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x693ec673, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630ec678, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c67c, SUC_NE_lexicon_tags_2 },
    { 0x534ec67d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3ac681, SUC_NE_lexicon_tags_2 },
    { 0xe508c682, SUC_NE_lexicon_tags_2 },
    { 0xc98cc682, SUC_NE_lexicon_tags_1 },
    { 0x3968c682, SUC_NE_lexicon_tags_2 },
    { 0xc442c685, SUC_NE_lexicon_tags_2 },
    { 0x416c685, SUC_NE_lexicon_tags_2 },
    { 0x3120c685, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef2c68c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3002c68f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2ac693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ec696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc56c698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb6c69c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x295ac69e, SUC_NE_lexicon_tags_2 },
    { 0x318ac69e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89a8c6a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ac6a4, SUC_NE_lexicon_tags_248 },
    { 0xb9aec6a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c36c6a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84b8c6a9, SUC_NE_lexicon_tags_2 },
    { 0xddb6c6aa, SUC_NE_lexicon_tags_2 },
    { 0x87f4c6aa, SUC_NE_lexicon_tags_2 },
    { 0x8bb0c6aa, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0xbd84c6ae, SUC_NE_lexicon_tags_2 },
    { 0x5aaec6af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdcc6b4, SUC_NE_lexicon_tags_2 },
    { 0xfe02c6b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59dac6b7, SUC_NE_lexicon_tags_3 },
    { 0x9772c6b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8f6c6ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f86c6c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5154c6c4, SUC_NE_lexicon_tags_2 },
    { 0x258ec6c5, SUC_NE_lexicon_tags_2 },
    { 0x34a8c6c5, SUC_NE_lexicon_tags_2 },
    { 0xa770c6c4, SUC_NE_lexicon_tags_2 },
    { 0x954ec6c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaf0c6ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7906c6d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ac6d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb62c6da, SUC_NE_lexicon_tags_2 },
    { 0x932c6da, SUC_NE_lexicon_tags_7 },
    { 0x726ac6dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e4c6e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8a8c6e2, SUC_NE_lexicon_tags_2 },
    { 0xc834c6e3, SUC_NE_lexicon_tags_2 },
    { 0xe6e6c6e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c8cc6e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d8c6e9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x87cc6eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41cc6f0, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0c6f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96d0c6f5, SUC_NE_lexicon_tags_4 },
    { 0x9c68c6f6, SUC_NE_lexicon_tags_2 },
    { 0xf504c6f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6976c6fc, SUC_NE_lexicon_tags_2 },
    { 0xa624c6fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9340c6ff, SUC_NE_lexicon_tags_2 },
    { 0x27eec700, SUC_NE_lexicon_tags_2 },
    { 0x7caac701, SUC_NE_lexicon_tags_2 },
    { 0x8efcc702, SUC_NE_lexicon_tags_2 },
    { 0xe432c703, SUC_NE_lexicon_tags_2 },
    { 0xece4c704, SUC_NE_lexicon_tags_2 },
    { 0xeeac702, SUC_NE_lexicon_tags_2 },
    { 0xa2f6c703, SUC_NE_lexicon_tags_2 },
    { 0xfc66c705, SUC_NE_lexicon_tags_2 },
    { 0x876cc708, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fd0c70a, SUC_NE_lexicon_tags_2 },
    { 0x5b6c70b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebe6c70d, SUC_NE_lexicon_tags_2 },
    { 0x387ec70d, SUC_NE_lexicon_tags_2 },
    { 0xb0c8c70f, SUC_NE_lexicon_tags_3 },
    { 0xe79cc710, SUC_NE_lexicon_tags_2 },
    { 0xe6a6c711, SUC_NE_lexicon_tags_2 },
    { 0xe9a0c711, SUC_NE_lexicon_tags_2 },
    { 0x4464c70e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef62c715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4948c718, SUC_NE_lexicon_tags_2 },
    { 0xac18c719, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfec721, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca74c725, SUC_NE_lexicon_tags_2 },
    { 0x4386c726, SUC_NE_lexicon_tags_2 },
    { 0x34ecc727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3404c729, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd122c72d, SUC_NE_lexicon_tags_2 },
    { 0x1c74c72e, SUC_NE_lexicon_tags_2 },
    { 0xa106c72f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6ec733, SUC_NE_lexicon_tags_2 },
    { 0x331ac734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3004c736, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaecc741, SUC_NE_lexicon_tags_2 },
    { 0x9c0ac742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf222c744, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf02c749, SUC_NE_lexicon_tags_2 },
    { 0x56d2c749, SUC_NE_lexicon_tags_2 },
    { 0x350cc74b, SUC_NE_lexicon_tags_2 },
    { 0x529ac74c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d4c755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2daec75a, SUC_NE_lexicon_tags_3 },
    { 0xe298c75b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc526c75d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964ec761, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f42c763, SUC_NE_lexicon_tags_2 },
    { 0x4760c764, SUC_NE_lexicon_tags_2 },
    { 0x9aacc764, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa36ec767, SUC_NE_lexicon_tags_2 },
    { 0xb80ac767, SUC_NE_lexicon_tags_2 },
    { 0x1a5ec769, SUC_NE_lexicon_tags_2 },
    { 0x68f8c769, SUC_NE_lexicon_tags_2 },
    { 0x583cc76b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa448c76f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7588c771, SUC_NE_lexicon_tags_2 },
    { 0x52d2c772, SUC_NE_lexicon_tags_2 },
    { 0xb8eec773, SUC_NE_lexicon_tags_2 },
    { 0xb2d2c774, SUC_NE_lexicon_tags_2 },
    { 0x3e22c775, SUC_NE_lexicon_tags_2 },
    { 0x202ec776, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fcc77c, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ecc77f, SUC_NE_lexicon_tags_2 },
    { 0x8208c780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0ec787, SUC_NE_lexicon_tags_2 },
    { 0xbe06c788, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa0c78b, SUC_NE_lexicon_tags_2 },
    { 0x6cc8c78c, SUC_NE_lexicon_tags_2 },
    { 0x4dbac78d, SUC_NE_lexicon_tags_2 },
    { 0xd346c78e, SUC_NE_lexicon_tags_311 },
    { 0xf6f2c78f, SUC_NE_lexicon_tags_17 },
    { 0x5f08c78f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc84ac792, SUC_NE_lexicon_tags_2 },
    { 0x1eb0c793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee54c798, SUC_NE_lexicon_tags_2 },
    { 0x601ac799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c08c79b, SUC_NE_lexicon_tags_2 },
    { 0x2332c79b, SUC_NE_lexicon_tags_2 },
    { 0x64d6c79d, SUC_NE_lexicon_tags_2 },
    { 0xa34ec79e, SUC_NE_lexicon_tags_2 },
    { 0x27d0c79f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87f2c7a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48ac7a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4c7a8, SUC_NE_lexicon_tags_2 },
    { 0xbf4ec7a8, SUC_NE_lexicon_tags_2 },
    { 0xed9ec7aa, SUC_NE_lexicon_tags_2 },
    { 0xeedec7a9, SUC_NE_lexicon_tags_7 },
    { 0xd6f2c7ac, SUC_NE_lexicon_tags_2 },
    { 0xd8e8c7ab, SUC_NE_lexicon_tags_31 },
    { 0x1530c7a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b8c7b1, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc2c7b4, SUC_NE_lexicon_tags_2 },
    { 0xb658c7b5, SUC_NE_lexicon_tags_2 },
    { 0x60dec7b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd482c7b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddac7be, SUC_NE_lexicon_tags_2 },
    { 0xaf0ac7bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba76c7c1, SUC_NE_lexicon_tags_2 },
    { 0x68a0c7c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1aac7c4, SUC_NE_lexicon_tags_2 },
    { 0xb6bac7c4, SUC_NE_lexicon_tags_3 },
    { 0xd7a4c7c6, SUC_NE_lexicon_tags_2 },
    { 0x27d0c7c7, SUC_NE_lexicon_tags_2 },
    { 0x699ec7c8, SUC_NE_lexicon_tags_2 },
    { 0x6180c7c8, SUC_NE_lexicon_tags_2 },
    { 0xb8dcc7ca, SUC_NE_lexicon_tags_10 },
    { 0xcc76c7ca, SUC_NE_lexicon_tags_2 },
    { 0xa7bac7c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbf6c7ce, SUC_NE_lexicon_tags_2 },
    { 0x370cc7ce, SUC_NE_lexicon_tags_46 },
    { 0xe140c7ce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f4c7d3, SUC_NE_lexicon_tags_2 },
    { 0xc502c7d4, SUC_NE_lexicon_tags_2 },
    { 0x1bfac7d4, SUC_NE_lexicon_tags_2 },
    { 0xbcbac7d4, SUC_NE_lexicon_tags_7 },
    { 0x3842c7d4, SUC_NE_lexicon_tags_2 },
    { 0x6310c7d8, SUC_NE_lexicon_tags_2 },
    { 0xcec4c7d6, SUC_NE_lexicon_tags_2 },
    { 0x2454c7da, SUC_NE_lexicon_tags_2 },
    { 0xd178c7db, SUC_NE_lexicon_tags_2 },
    { 0x71aac7db, SUC_NE_lexicon_tags_2 },
    { 0xc144c7db, SUC_NE_lexicon_tags_2 },
    { 0x1806c7db, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc188c7e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3570c7e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e74c7e7, SUC_NE_lexicon_tags_165 },
    { 0x8220c7e7, SUC_NE_lexicon_tags_2 },
    { 0x5268c7e8, SUC_NE_lexicon_tags_2 },
    { 0x47d0c7ea, SUC_NE_lexicon_tags_2 },
    { 0x959ac7e8, SUC_NE_lexicon_tags_2 },
    { 0x4596c7ec, SUC_NE_lexicon_tags_2 },
    { 0x3802c7eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9ec7f2, SUC_NE_lexicon_tags_2 },
    { 0x64e0c7f3, SUC_NE_lexicon_tags_109 },
    { 0x402ac7f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf50c7f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2060c7fb, SUC_NE_lexicon_tags_2 },
    { 0xc438c7fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b8c7ff, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4c4c801, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f2c805, SUC_NE_lexicon_tags_2 },
    { 0x216ac806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebcc808, SUC_NE_lexicon_tags_64 },
    { 0x9be8c809, SUC_NE_lexicon_tags_2 },
    { 0x3c90c80a, SUC_NE_lexicon_tags_2 },
    { 0xaccec809, SUC_NE_lexicon_tags_2 },
    { 0x9a6c80c, SUC_NE_lexicon_tags_2 },
    { 0xd2b6c80d, SUC_NE_lexicon_tags_2 },
    { 0xe9cac80d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4142c810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60a8c812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aeac814, SUC_NE_lexicon_tags_2 },
    { 0xd98c815, SUC_NE_lexicon_tags_2 },
    { 0x7b66c816, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40c8c818, SUC_NE_lexicon_tags_2 },
    { 0xf7fcc818, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cc81c, SUC_NE_lexicon_tags_2 },
    { 0x4c56c81c, SUC_NE_lexicon_tags_1 },
    { 0x3e0ac81c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ec824, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9988c82a, SUC_NE_lexicon_tags_2 },
    { 0x8accc82b, SUC_NE_lexicon_tags_2 },
    { 0x114cc82c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8eec82f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7cc835, SUC_NE_lexicon_tags_2 },
    { 0x983ec836, SUC_NE_lexicon_tags_2 },
    { 0x11d0c836, SUC_NE_lexicon_tags_2 },
    { 0x5f8cc838, SUC_NE_lexicon_tags_1 },
    { 0xc040c839, SUC_NE_lexicon_tags_156 },
    { 0xe6c8c838, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dcc840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972ec847, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ecc84a, SUC_NE_lexicon_tags_2 },
    { 0x6d2cc84b, SUC_NE_lexicon_tags_2 },
    { 0xc294c84b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34c84f, SUC_NE_lexicon_tags_2 },
    { 0xf3c2c850, SUC_NE_lexicon_tags_2 },
    { 0x7b6c850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb0c854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x106ac857, SUC_NE_lexicon_tags_2 },
    { 0xdc06c858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eec85b, SUC_NE_lexicon_tags_2 },
    { 0x8394c85c, SUC_NE_lexicon_tags_2 },
    { 0xd898c85d, SUC_NE_lexicon_tags_2 },
    { 0xb0e8c85c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4c862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb442c868, SUC_NE_lexicon_tags_2 },
    { 0xb970c868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa624c86b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e04c86d, SUC_NE_lexicon_tags_2 },
    { 0x89e2c86e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b12c871, SUC_NE_lexicon_tags_2 },
    { 0x91ecc872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3724c874, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6c878, SUC_NE_lexicon_tags_7 },
    { 0x2ba8c879, SUC_NE_lexicon_tags_2 },
    { 0x30c0c87a, SUC_NE_lexicon_tags_2 },
    { 0xacf0c87a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dec87f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c0c887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x533ec889, SUC_NE_lexicon_tags_13 },
    { 0xb4e8c88a, SUC_NE_lexicon_tags_2 },
    { 0x9ec2c88b, SUC_NE_lexicon_tags_2 },
    { 0x36dec88c, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc8c88f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484cc893, SUC_NE_lexicon_tags_19 },
    { 0xa456c893, SUC_NE_lexicon_tags_2 },
    { 0xc7cec895, SUC_NE_lexicon_tags_2 },
    { 0x81b8c894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3cc89d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c0c8a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e5cc8a2, SUC_NE_lexicon_tags_148 },
    { 0, NULL },
    { 0xecc2c8a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7554c8a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb290c8ab, SUC_NE_lexicon_tags_2 },
    { 0x9a16c8ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1750c8b0, SUC_NE_lexicon_tags_2 },
    { 0x9ac0c8b1, SUC_NE_lexicon_tags_360 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf578c8b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f48c8bc, SUC_NE_lexicon_tags_2 },
    { 0xc7b6c8bd, SUC_NE_lexicon_tags_2 },
    { 0xe652c8be, SUC_NE_lexicon_tags_2 },
    { 0xb88cc8bf, SUC_NE_lexicon_tags_2 },
    { 0x2bc8c8c0, SUC_NE_lexicon_tags_2 },
    { 0x388c8c1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x526c8c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ecc8c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16c4c8c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9206c8ce, SUC_NE_lexicon_tags_2 },
    { 0xd560c8cf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af4c8d2, SUC_NE_lexicon_tags_2 },
    { 0xf3a0c8d3, SUC_NE_lexicon_tags_2 },
    { 0x3a98c8d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1088c8d9, SUC_NE_lexicon_tags_2 },
    { 0x3f9ac8da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac8e1, SUC_NE_lexicon_tags_18 },
    { 0x57acc8e2, SUC_NE_lexicon_tags_2 },
    { 0x8f76c8e3, SUC_NE_lexicon_tags_2 },
    { 0xce9ec8e3, SUC_NE_lexicon_tags_2 },
    { 0x4810c8e3, SUC_NE_lexicon_tags_19 },
    { 0x4338c8e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cac8ec, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f48c8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa128c8f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777ec8f9, SUC_NE_lexicon_tags_2 },
    { 0x9a98c8f9, SUC_NE_lexicon_tags_4 },
    { 0x81fec8fb, SUC_NE_lexicon_tags_2 },
    { 0x5f58c8fc, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x426cc8ff, SUC_NE_lexicon_tags_2 },
    { 0x8a0ac900, SUC_NE_lexicon_tags_2 },
    { 0x3ee8c901, SUC_NE_lexicon_tags_2 },
    { 0x1df6c8ff, SUC_NE_lexicon_tags_2 },
    { 0xd1ecc900, SUC_NE_lexicon_tags_10 },
    { 0x97bcc904, SUC_NE_lexicon_tags_2 },
    { 0x7a5ec904, SUC_NE_lexicon_tags_2 },
    { 0x96a8c906, SUC_NE_lexicon_tags_8 },
    { 0x604c907, SUC_NE_lexicon_tags_2 },
    { 0xa9e6c8ff, SUC_NE_lexicon_tags_3 },
    { 0xf760c909, SUC_NE_lexicon_tags_2 },
    { 0x63a4c90a, SUC_NE_lexicon_tags_0 },
    { 0x38eac90b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa684c90d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd4c911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7014c913, SUC_NE_lexicon_tags_2 },
    { 0x82a0c914, SUC_NE_lexicon_tags_2 },
    { 0x55a6c914, SUC_NE_lexicon_tags_2 },
    { 0xf3f2c916, SUC_NE_lexicon_tags_4 },
    { 0xfde2c917, SUC_NE_lexicon_tags_2 },
    { 0x2eecc915, SUC_NE_lexicon_tags_2 },
    { 0x7efac919, SUC_NE_lexicon_tags_2 },
    { 0x8bc2c91a, SUC_NE_lexicon_tags_2 },
    { 0x5ec0c919, SUC_NE_lexicon_tags_2 },
    { 0xf41cc919, SUC_NE_lexicon_tags_7 },
    { 0x770ec91d, SUC_NE_lexicon_tags_2 },
    { 0x5120c91e, SUC_NE_lexicon_tags_2 },
    { 0xd1e8c91f, SUC_NE_lexicon_tags_2 },
    { 0x3b5ec920, SUC_NE_lexicon_tags_2 },
    { 0x6330c91f, SUC_NE_lexicon_tags_23 },
    { 0x795ec922, SUC_NE_lexicon_tags_24 },
    { 0x1158c923, SUC_NE_lexicon_tags_2 },
    { 0x7716c91e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ec6c926, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebec929, SUC_NE_lexicon_tags_4 },
    { 0xda08c92a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7754c92f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2db6c931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca32c933, SUC_NE_lexicon_tags_4 },
    { 0x9cac934, SUC_NE_lexicon_tags_7 },
    { 0x27acc933, SUC_NE_lexicon_tags_2 },
    { 0xc0a8c936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814cc93c, SUC_NE_lexicon_tags_0 },
    { 0x9c6c93c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4158c93f, SUC_NE_lexicon_tags_2 },
    { 0xc40ac93f, SUC_NE_lexicon_tags_2 },
    { 0x1eac941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9316c944, SUC_NE_lexicon_tags_17 },
    { 0xcaf2c945, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ec948, SUC_NE_lexicon_tags_2 },
    { 0x5186c949, SUC_NE_lexicon_tags_2 },
    { 0xd106c94a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aac94e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd1ac950, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6250c952, SUC_NE_lexicon_tags_2 },
    { 0xe222c953, SUC_NE_lexicon_tags_2 },
    { 0x5daec954, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c6c958, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f8c95a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75cc95d, SUC_NE_lexicon_tags_2 },
    { 0xfdbcc95e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9472c961, SUC_NE_lexicon_tags_2 },
    { 0x3c5ac961, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x652ac964, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x177ac966, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf692c96b, SUC_NE_lexicon_tags_2 },
    { 0xa4fcc96c, SUC_NE_lexicon_tags_2 },
    { 0x8780c96d, SUC_NE_lexicon_tags_2 },
    { 0xccc8c96d, SUC_NE_lexicon_tags_2 },
    { 0x54e0c96c, SUC_NE_lexicon_tags_1 },
    { 0x2d02c96c, SUC_NE_lexicon_tags_2 },
    { 0x83f6c971, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ac975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cc979, SUC_NE_lexicon_tags_2 },
    { 0xadb8c97a, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aec97d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a2c980, SUC_NE_lexicon_tags_2 },
    { 0xb4aac981, SUC_NE_lexicon_tags_2 },
    { 0xaac6c982, SUC_NE_lexicon_tags_2 },
    { 0x7e88c983, SUC_NE_lexicon_tags_2 },
    { 0x9abec984, SUC_NE_lexicon_tags_2 },
    { 0xed00c981, SUC_NE_lexicon_tags_2 },
    { 0x9c4c986, SUC_NE_lexicon_tags_2 },
    { 0x2732c987, SUC_NE_lexicon_tags_2 },
    { 0x4cf8c988, SUC_NE_lexicon_tags_2 },
    { 0xfb8cc987, SUC_NE_lexicon_tags_2 },
    { 0xf67ac987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x446cc98d, SUC_NE_lexicon_tags_4 },
    { 0xd4a2c98e, SUC_NE_lexicon_tags_2 },
    { 0xbc8ec98f, SUC_NE_lexicon_tags_7 },
    { 0xa99cc98d, SUC_NE_lexicon_tags_2 },
    { 0x358ec991, SUC_NE_lexicon_tags_4 },
    { 0xb000c992, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd152c99a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56cc99c, SUC_NE_lexicon_tags_8 },
    { 0xf112c99d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2c9a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x684ac9a3, SUC_NE_lexicon_tags_2 },
    { 0x2522c9a4, SUC_NE_lexicon_tags_2 },
    { 0xbda0c9a4, SUC_NE_lexicon_tags_2 },
    { 0x5960c9a4, SUC_NE_lexicon_tags_2 },
    { 0x86d0c9a6, SUC_NE_lexicon_tags_2 },
    { 0xc494c9a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x302cc9aa, SUC_NE_lexicon_tags_2 },
    { 0xb462c9aa, SUC_NE_lexicon_tags_7 },
    { 0xde92c9aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd4c9b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2816c9b2, SUC_NE_lexicon_tags_2 },
    { 0x1e0c9b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b72c9b6, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d54c9ba, SUC_NE_lexicon_tags_0 },
    { 0x64e6c9bb, SUC_NE_lexicon_tags_2 },
    { 0xe43ac9bc, SUC_NE_lexicon_tags_7 },
    { 0xd094c9bb, SUC_NE_lexicon_tags_2 },
    { 0xa4d0c9be, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x7464c9c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3426c9c4, SUC_NE_lexicon_tags_2 },
    { 0x6e58c9c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cc9c8, SUC_NE_lexicon_tags_2 },
    { 0x706cc9c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeccc9cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7638c9ce, SUC_NE_lexicon_tags_1 },
    { 0x7a5cc9ce, SUC_NE_lexicon_tags_2 },
    { 0x29ecc9d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8f2c9d2, SUC_NE_lexicon_tags_2 },
    { 0x7ddec9d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x849cc9d6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9b68c9d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcec9db, SUC_NE_lexicon_tags_2 },
    { 0x3a5cc9db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb536c9de, SUC_NE_lexicon_tags_2 },
    { 0x4eaec9df, SUC_NE_lexicon_tags_2 },
    { 0xe1ec9df, SUC_NE_lexicon_tags_2 },
    { 0x396ac9df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8ac9e7, SUC_NE_lexicon_tags_2 },
    { 0x9cd8c9e7, SUC_NE_lexicon_tags_2 },
    { 0xb996c9e9, SUC_NE_lexicon_tags_2 },
    { 0x9d00c9e7, SUC_NE_lexicon_tags_2 },
    { 0x439ec9ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a04c9ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x947cc9ef, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xbc86c9f1, SUC_NE_lexicon_tags_2 },
    { 0xc216c9f1, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xec96c9f4, SUC_NE_lexicon_tags_2 },
    { 0x75b4c9f5, SUC_NE_lexicon_tags_2 },
    { 0x9af0c9f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa962c9fa, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8662ca04, SUC_NE_lexicon_tags_2 },
    { 0x25c2ca04, SUC_NE_lexicon_tags_2 },
    { 0x3182ca06, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e0ca0a, SUC_NE_lexicon_tags_2 },
    { 0xcd0eca0b, SUC_NE_lexicon_tags_2 },
    { 0xe072ca0c, SUC_NE_lexicon_tags_2 },
    { 0x5df2ca0d, SUC_NE_lexicon_tags_2 },
    { 0xdde2ca0e, SUC_NE_lexicon_tags_2 },
    { 0xb130ca0f, SUC_NE_lexicon_tags_2 },
    { 0x782ca10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db8ca14, SUC_NE_lexicon_tags_2 },
    { 0xb7cca15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c0ca22, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa00ca25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46eca28, SUC_NE_lexicon_tags_2 },
    { 0xe810ca29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e2ca2c, SUC_NE_lexicon_tags_2 },
    { 0xe1ecca2d, SUC_NE_lexicon_tags_2 },
    { 0xb828ca2c, SUC_NE_lexicon_tags_7 },
    { 0x96a8ca2f, SUC_NE_lexicon_tags_5 },
    { 0x2920ca30, SUC_NE_lexicon_tags_2 },
    { 0xe59cca30, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x167aca33, SUC_NE_lexicon_tags_2 },
    { 0x4df8ca34, SUC_NE_lexicon_tags_2 },
    { 0x7a12ca34, SUC_NE_lexicon_tags_2 },
    { 0x4a98ca35, SUC_NE_lexicon_tags_2 },
    { 0x4fcca37, SUC_NE_lexicon_tags_3 },
    { 0xdc8cca37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972ca3c, SUC_NE_lexicon_tags_2 },
    { 0xef0ca3d, SUC_NE_lexicon_tags_58 },
    { 0xca38ca3e, SUC_NE_lexicon_tags_2 },
    { 0xac02ca3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x681aca42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5790ca48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3214ca4c, SUC_NE_lexicon_tags_2 },
    { 0xa81eca4d, SUC_NE_lexicon_tags_2 },
    { 0x4d8aca4e, SUC_NE_lexicon_tags_2 },
    { 0xb25cca4e, SUC_NE_lexicon_tags_2 },
    { 0x99c4ca4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe42ca55, SUC_NE_lexicon_tags_2 },
    { 0x66e0ca55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca58, SUC_NE_lexicon_tags_2 },
    { 0x3d40ca58, SUC_NE_lexicon_tags_2 },
    { 0xab80ca58, SUC_NE_lexicon_tags_2 },
    { 0x3f38ca5b, SUC_NE_lexicon_tags_2 },
    { 0x7a74ca5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b1eca5e, SUC_NE_lexicon_tags_2 },
    { 0xbcb6ca5e, SUC_NE_lexicon_tags_2 },
    { 0x51b6ca5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6122ca68, SUC_NE_lexicon_tags_2 },
    { 0x7d06ca69, SUC_NE_lexicon_tags_7 },
    { 0x3eca68, SUC_NE_lexicon_tags_2 },
    { 0x8000ca6b, SUC_NE_lexicon_tags_3 },
    { 0xd682ca6c, SUC_NE_lexicon_tags_2 },
    { 0x805eca6d, SUC_NE_lexicon_tags_2 },
    { 0x907cca6e, SUC_NE_lexicon_tags_2 },
    { 0x2d74ca6f, SUC_NE_lexicon_tags_2 },
    { 0x88fcca70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3cca75, SUC_NE_lexicon_tags_7 },
    { 0x4e88ca75, SUC_NE_lexicon_tags_2 },
    { 0x810cca77, SUC_NE_lexicon_tags_2 },
    { 0x7cc4ca77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d2ca7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7f4ca7c, SUC_NE_lexicon_tags_2 },
    { 0xf03cca7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x353aca80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5104ca85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e84ca87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9008ca89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993aca8e, SUC_NE_lexicon_tags_2 },
    { 0x1134ca8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x587aca92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc54ca94, SUC_NE_lexicon_tags_6 },
    { 0xb94cca95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6346ca97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ceca9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe292ca9e, SUC_NE_lexicon_tags_1 },
    { 0xdb24ca9e, SUC_NE_lexicon_tags_2 },
    { 0xef2cca9e, SUC_NE_lexicon_tags_2 },
    { 0x6cf0caa1, SUC_NE_lexicon_tags_2 },
    { 0x665cca9f, SUC_NE_lexicon_tags_2 },
    { 0x7152caa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ecaa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa58caaa, SUC_NE_lexicon_tags_2 },
    { 0x66a6caaa, SUC_NE_lexicon_tags_2 },
    { 0x8cb6caac, SUC_NE_lexicon_tags_2 },
    { 0xc984caad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaecab0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9840cab2, SUC_NE_lexicon_tags_2 },
    { 0xfb0acab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56cab6, SUC_NE_lexicon_tags_2 },
    { 0x536acab7, SUC_NE_lexicon_tags_2 },
    { 0x1f52cab8, SUC_NE_lexicon_tags_2 },
    { 0x8960cab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41b0cabb, SUC_NE_lexicon_tags_14 },
    { 0xf99acabc, SUC_NE_lexicon_tags_35 },
    { 0xe4f4cabc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eccac0, SUC_NE_lexicon_tags_7 },
    { 0xef06cac1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338ecac5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eecac8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab08cacc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4e0cace, SUC_NE_lexicon_tags_2 },
    { 0xd5aecace, SUC_NE_lexicon_tags_2 },
    { 0xe49ecad0, SUC_NE_lexicon_tags_3 },
    { 0xd498cacf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8acad5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d0cad9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9ccadc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x937ccadf, SUC_NE_lexicon_tags_2 },
    { 0x84facae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3204cae2, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0x38d4cae4, SUC_NE_lexicon_tags_2 },
    { 0xf952cae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b20cae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ccaea, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64acaef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afecaf5, SUC_NE_lexicon_tags_40 },
    { 0xc70caf6, SUC_NE_lexicon_tags_2 },
    { 0x36ecaf5, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5700cafb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fccaff, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c2cb02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x925acb04, SUC_NE_lexicon_tags_2 },
    { 0x6818cb04, SUC_NE_lexicon_tags_4 },
    { 0x6c8cb06, SUC_NE_lexicon_tags_2 },
    { 0xfa14cb05, SUC_NE_lexicon_tags_2 },
    { 0x120acb08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b00cb10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5388cb13, SUC_NE_lexicon_tags_2 },
    { 0x7cd0cb13, SUC_NE_lexicon_tags_2 },
    { 0x264cb15, SUC_NE_lexicon_tags_2 },
    { 0x8bdacb16, SUC_NE_lexicon_tags_2 },
    { 0xbdc0cb15, SUC_NE_lexicon_tags_2 },
    { 0x3d96cb18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fecb1e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x796cb21, SUC_NE_lexicon_tags_2 },
    { 0xea2ccb21, SUC_NE_lexicon_tags_2 },
    { 0x6ed2cb23, SUC_NE_lexicon_tags_2 },
    { 0x855acb21, SUC_NE_lexicon_tags_2 },
    { 0x6f6ccb25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdecb29, SUC_NE_lexicon_tags_2 },
    { 0xcc84cb29, SUC_NE_lexicon_tags_2 },
    { 0x22e4cb29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4cb30, SUC_NE_lexicon_tags_2 },
    { 0x38accb31, SUC_NE_lexicon_tags_2 },
    { 0x4a16cb32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff34cb34, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xa154cb36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd500cb3a, SUC_NE_lexicon_tags_2 },
    { 0xbfa2cb3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd88cb3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f8cb41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1ecb43, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xf970cb45, SUC_NE_lexicon_tags_2 },
    { 0xcf6ecb45, SUC_NE_lexicon_tags_2 },
    { 0x6a68cb47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x395ecb49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf13ecb4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab52cb4d, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c00cb4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aecb55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafb6cb57, SUC_NE_lexicon_tags_2 },
    { 0x348acb58, SUC_NE_lexicon_tags_2 },
    { 0xddf6cb59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6608cb5e, SUC_NE_lexicon_tags_0 },
    { 0x5008cb5f, SUC_NE_lexicon_tags_2 },
    { 0xe00cb60, SUC_NE_lexicon_tags_2 },
    { 0x9c90cb61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a4cb66, SUC_NE_lexicon_tags_2 },
    { 0x39d4cb67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc82cb6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe82ccb6d, SUC_NE_lexicon_tags_2 },
    { 0x5d7ecb6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x370acb71, SUC_NE_lexicon_tags_2 },
    { 0x495ccb72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4ccb75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94e0cb77, SUC_NE_lexicon_tags_2 },
    { 0x7ee6cb77, SUC_NE_lexicon_tags_2 },
    { 0xcc26cb79, SUC_NE_lexicon_tags_2 },
    { 0x340ecb79, SUC_NE_lexicon_tags_2 },
    { 0x43a8cb7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e32cb7d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x87a6cb7f, SUC_NE_lexicon_tags_2 },
    { 0x171ecb80, SUC_NE_lexicon_tags_7 },
    { 0x5f42cb81, SUC_NE_lexicon_tags_2 },
    { 0x748cb7f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xb9f2cb84, SUC_NE_lexicon_tags_2 },
    { 0xdf30cb85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b0cb8f, SUC_NE_lexicon_tags_2 },
    { 0x790ecb90, SUC_NE_lexicon_tags_2 },
    { 0xa250cb91, SUC_NE_lexicon_tags_4 },
    { 0x7aeacb92, SUC_NE_lexicon_tags_2 },
    { 0xe88ccb92, SUC_NE_lexicon_tags_2 },
    { 0xc5c6cb8f, SUC_NE_lexicon_tags_2 },
    { 0xfa88cb95, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ecb9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c40cb9d, SUC_NE_lexicon_tags_2 },
    { 0x587ccb9e, SUC_NE_lexicon_tags_2 },
    { 0x5762cb9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdf0cba1, SUC_NE_lexicon_tags_2 },
    { 0xd27ccba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2826cbab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746ecbb2, SUC_NE_lexicon_tags_2 },
    { 0x47a8cbb2, SUC_NE_lexicon_tags_2 },
    { 0x34b4cbb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x985ccbb6, SUC_NE_lexicon_tags_7 },
    { 0x320ecbb7, SUC_NE_lexicon_tags_325 },
    { 0x549ccbb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb420cbbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7462cbbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b8cbc1, SUC_NE_lexicon_tags_2 },
    { 0x82d0cbc1, SUC_NE_lexicon_tags_2 },
    { 0xb07ccbc3, SUC_NE_lexicon_tags_2 },
    { 0xda34cbc1, SUC_NE_lexicon_tags_0 },
    { 0x127ccbc4, SUC_NE_lexicon_tags_2 },
    { 0xbf34cbc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x348acbc8, SUC_NE_lexicon_tags_116 },
    { 0xa89ccbc9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xdab8cbcb, SUC_NE_lexicon_tags_5 },
    { 0xbab4cbcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8730cbd1, SUC_NE_lexicon_tags_18 },
    { 0x47dacbd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a2acbd4, SUC_NE_lexicon_tags_2 },
    { 0xa388cbd5, SUC_NE_lexicon_tags_2 },
    { 0x43cecbd6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b40cbda, SUC_NE_lexicon_tags_2 },
    { 0x8eb2cbdb, SUC_NE_lexicon_tags_2 },
    { 0x4248cbdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec2cbe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3528cbe9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f18cbeb, SUC_NE_lexicon_tags_2 },
    { 0xc3a6cbec, SUC_NE_lexicon_tags_2 },
    { 0xfb10cbec, SUC_NE_lexicon_tags_1 },
    { 0x3260cbee, SUC_NE_lexicon_tags_7 },
    { 0x7e32cbee, SUC_NE_lexicon_tags_109 },
    { 0x9210cbf0, SUC_NE_lexicon_tags_2 },
    { 0x6c18cbed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eccbf7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b4cbfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eecbfe, SUC_NE_lexicon_tags_2 },
    { 0x60cacbff, SUC_NE_lexicon_tags_2 },
    { 0x5e9ecbff, SUC_NE_lexicon_tags_2 },
    { 0xf2ecc01, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1802cc07, SUC_NE_lexicon_tags_2 },
    { 0xf81acc08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd036cc0d, SUC_NE_lexicon_tags_2 },
    { 0x45c2cc0d, SUC_NE_lexicon_tags_1 },
    { 0x7e48cc0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec02cc12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9366cc14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6cc17, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0xbeacc19, SUC_NE_lexicon_tags_2 },
    { 0xa140cc19, SUC_NE_lexicon_tags_7 },
    { 0x5acccc1b, SUC_NE_lexicon_tags_2 },
    { 0x180ecc1c, SUC_NE_lexicon_tags_2 },
    { 0xd176cc1d, SUC_NE_lexicon_tags_2 },
    { 0x5b32cc1e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec24cc22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1182cc26, SUC_NE_lexicon_tags_2 },
    { 0xca6ecc26, SUC_NE_lexicon_tags_2 },
    { 0xc0facc26, SUC_NE_lexicon_tags_17 },
    { 0x4be2cc26, SUC_NE_lexicon_tags_2 },
    { 0x2d1ecc2a, SUC_NE_lexicon_tags_7 },
    { 0xa270cc27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb320cc2e, SUC_NE_lexicon_tags_2 },
    { 0x3054cc2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x952cc31, SUC_NE_lexicon_tags_2 },
    { 0xef9ccc31, SUC_NE_lexicon_tags_247 },
    { 0x77d4cc31, SUC_NE_lexicon_tags_2 },
    { 0x1862cc32, SUC_NE_lexicon_tags_2 },
    { 0xb778cc35, SUC_NE_lexicon_tags_3 },
    { 0x180cc35, SUC_NE_lexicon_tags_2 },
    { 0xc1a2cc37, SUC_NE_lexicon_tags_2 },
    { 0xda08cc31, SUC_NE_lexicon_tags_2 },
    { 0xf840cc31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb514cc3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x872acc3e, SUC_NE_lexicon_tags_2 },
    { 0xb554cc3f, SUC_NE_lexicon_tags_2 },
    { 0x82b0cc40, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26cc43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd6cc45, SUC_NE_lexicon_tags_0 },
    { 0x21ecc46, SUC_NE_lexicon_tags_2 },
    { 0x9498cc46, SUC_NE_lexicon_tags_2 },
    { 0xa6facc45, SUC_NE_lexicon_tags_2 },
    { 0xfe0ccc49, SUC_NE_lexicon_tags_2 },
    { 0x4e8ecc4a, SUC_NE_lexicon_tags_2 },
    { 0xf82ccc4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0ecc51, SUC_NE_lexicon_tags_2 },
    { 0x4744cc51, SUC_NE_lexicon_tags_2 },
    { 0xb8bccc51, SUC_NE_lexicon_tags_2 },
    { 0xb754cc54, SUC_NE_lexicon_tags_2 },
    { 0xec6cc55, SUC_NE_lexicon_tags_2 },
    { 0x12a8cc56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a58cc58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aacc5c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22acc61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33cacc63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x653ecc65, SUC_NE_lexicon_tags_2 },
    { 0xc538cc66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3836cc6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6786cc70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x882ccc73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f0cc76, SUC_NE_lexicon_tags_2 },
    { 0xa090cc77, SUC_NE_lexicon_tags_2 },
    { 0x98bacc76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50cc7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d6cc80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5148cc82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4710cc84, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb786cc89, SUC_NE_lexicon_tags_2 },
    { 0x331ecc89, SUC_NE_lexicon_tags_2 },
    { 0x1edacc89, SUC_NE_lexicon_tags_2 },
    { 0xa6a2cc8c, SUC_NE_lexicon_tags_2 },
    { 0x5fe0cc8d, SUC_NE_lexicon_tags_2 },
    { 0xf160cc8e, SUC_NE_lexicon_tags_2 },
    { 0x655ccc8f, SUC_NE_lexicon_tags_2 },
    { 0x8082cc90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6cc94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4a0cc96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x576ecc98, SUC_NE_lexicon_tags_2 },
    { 0x39eecc99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16cecc9b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa01ccc9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c84cca6, SUC_NE_lexicon_tags_2 },
    { 0x270cca7, SUC_NE_lexicon_tags_2 },
    { 0x8de2cca8, SUC_NE_lexicon_tags_2 },
    { 0xb1cacca8, SUC_NE_lexicon_tags_2 },
    { 0xa9bcccaa, SUC_NE_lexicon_tags_2 },
    { 0xe580ccaa, SUC_NE_lexicon_tags_2 },
    { 0xfc74cca6, SUC_NE_lexicon_tags_2 },
    { 0x4beccaa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x18ccaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbcccb3, SUC_NE_lexicon_tags_2 },
    { 0x5732ccb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd44ccb6, SUC_NE_lexicon_tags_2 },
    { 0x6f3accb7, SUC_NE_lexicon_tags_16 },
    { 0xf430ccb7, SUC_NE_lexicon_tags_7 },
    { 0x63eaccb9, SUC_NE_lexicon_tags_2 },
    { 0xcbf0ccba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf238ccbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da0ccc2, SUC_NE_lexicon_tags_24 },
    { 0xda4eccc3, SUC_NE_lexicon_tags_2 },
    { 0xa52cccc4, SUC_NE_lexicon_tags_2 },
    { 0x1110ccc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96daccc8, SUC_NE_lexicon_tags_2 },
    { 0xf40eccc9, SUC_NE_lexicon_tags_2 },
    { 0x19accc9, SUC_NE_lexicon_tags_2 },
    { 0x8eaccca, SUC_NE_lexicon_tags_2 },
    { 0x90caccc8, SUC_NE_lexicon_tags_40 },
    { 0x7ed8cccd, SUC_NE_lexicon_tags_2 },
    { 0x67d6ccce, SUC_NE_lexicon_tags_2 },
    { 0x7b38cccf, SUC_NE_lexicon_tags_1 },
    { 0xf574ccce, SUC_NE_lexicon_tags_2 },
    { 0x4194ccd1, SUC_NE_lexicon_tags_2 },
    { 0x863cccd2, SUC_NE_lexicon_tags_2 },
    { 0xb1c8ccd3, SUC_NE_lexicon_tags_2 },
    { 0xeba4ccd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x367eccd6, SUC_NE_lexicon_tags_35 },
    { 0x3ba4ccd6, SUC_NE_lexicon_tags_2 },
    { 0x752cccd7, SUC_NE_lexicon_tags_2 },
    { 0x35faccd9, SUC_NE_lexicon_tags_2 },
    { 0x99aeccda, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x563cccdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa2ccdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6862cce2, SUC_NE_lexicon_tags_2 },
    { 0x91fecce3, SUC_NE_lexicon_tags_2 },
    { 0x7610cce4, SUC_NE_lexicon_tags_2 },
    { 0x218cce5, SUC_NE_lexicon_tags_2 },
    { 0x1c2acce6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5724ccec, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277accf5, SUC_NE_lexicon_tags_2 },
    { 0xc9d2ccf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf638ccf8, SUC_NE_lexicon_tags_3 },
    { 0x7982ccf8, SUC_NE_lexicon_tags_227 },
    { 0x3cf8ccfa, SUC_NE_lexicon_tags_2 },
    { 0x2844ccfb, SUC_NE_lexicon_tags_2 },
    { 0xb04eccfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf50cd01, SUC_NE_lexicon_tags_2 },
    { 0x5b20cd02, SUC_NE_lexicon_tags_2 },
    { 0x77c2cd03, SUC_NE_lexicon_tags_2 },
    { 0xf422cd03, SUC_NE_lexicon_tags_2 },
    { 0x7830cd02, SUC_NE_lexicon_tags_2 },
    { 0x41facd06, SUC_NE_lexicon_tags_2 },
    { 0xf984cd01, SUC_NE_lexicon_tags_2 },
    { 0xd9becd08, SUC_NE_lexicon_tags_2 },
    { 0x12ccd08, SUC_NE_lexicon_tags_18 },
    { 0xd0d0cd0a, SUC_NE_lexicon_tags_13 },
    { 0x7624cd07, SUC_NE_lexicon_tags_2 },
    { 0x82f4cd0c, SUC_NE_lexicon_tags_2 },
    { 0xcc68cd0c, SUC_NE_lexicon_tags_2 },
    { 0xf1c2cd05, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdecd15, SUC_NE_lexicon_tags_2 },
    { 0x4cdacd16, SUC_NE_lexicon_tags_2 },
    { 0x43decd17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf32cd19, SUC_NE_lexicon_tags_2 },
    { 0x4c6cd1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe706cd20, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0xf8ecd22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec38cd24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf140cd26, SUC_NE_lexicon_tags_2 },
    { 0xaf66cd26, SUC_NE_lexicon_tags_2 },
    { 0xcba4cd28, SUC_NE_lexicon_tags_2 },
    { 0x9b42cd29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbb8cd2b, SUC_NE_lexicon_tags_309 },
    { 0x1eeecd2c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bb2cd2e, SUC_NE_lexicon_tags_0 },
    { 0x1c30cd2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab52cd32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6acd34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12dacd36, SUC_NE_lexicon_tags_4 },
    { 0x9fbecd36, SUC_NE_lexicon_tags_2 },
    { 0xeffacd38, SUC_NE_lexicon_tags_2 },
    { 0x64aecd39, SUC_NE_lexicon_tags_13 },
    { 0xdd38cd3a, SUC_NE_lexicon_tags_2 },
    { 0x3cd6cd3a, SUC_NE_lexicon_tags_2 },
    { 0x5986cd3c, SUC_NE_lexicon_tags_2 },
    { 0xd426cd36, SUC_NE_lexicon_tags_2 },
    { 0xfa8ccd3d, SUC_NE_lexicon_tags_2 },
    { 0xb182cd3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12ccd42, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f36cd46, SUC_NE_lexicon_tags_2 },
    { 0x9260cd47, SUC_NE_lexicon_tags_7 },
    { 0x163ecd46, SUC_NE_lexicon_tags_2 },
    { 0x4b84cd47, SUC_NE_lexicon_tags_2 },
    { 0xb39ccd4a, SUC_NE_lexicon_tags_2 },
    { 0x7b3acd4b, SUC_NE_lexicon_tags_2 },
    { 0x70cd4c, SUC_NE_lexicon_tags_2 },
    { 0x36eacd4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab4cd54, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf82ecd56, SUC_NE_lexicon_tags_4 },
    { 0xbfd0cd56, SUC_NE_lexicon_tags_2 },
    { 0x7a28cd58, SUC_NE_lexicon_tags_2 },
    { 0x2d6cd56, SUC_NE_lexicon_tags_2 },
    { 0x694ccd5a, SUC_NE_lexicon_tags_2 },
    { 0x8048cd5b, SUC_NE_lexicon_tags_2 },
    { 0x9184cd57, SUC_NE_lexicon_tags_2 },
    { 0x629ecd5c, SUC_NE_lexicon_tags_2 },
    { 0xaa00cd5e, SUC_NE_lexicon_tags_2 },
    { 0xfcdecd5f, SUC_NE_lexicon_tags_2 },
    { 0x209acd5f, SUC_NE_lexicon_tags_2 },
    { 0x519ecd61, SUC_NE_lexicon_tags_25 },
    { 0x59f2cd62, SUC_NE_lexicon_tags_2 },
    { 0xf04acd5e, SUC_NE_lexicon_tags_2 },
    { 0x5200cd63, SUC_NE_lexicon_tags_2 },
    { 0xc80ccd62, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5ecd69, SUC_NE_lexicon_tags_2 },
    { 0x651ccd6a, SUC_NE_lexicon_tags_2 },
    { 0x9acccd69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b2cd70, SUC_NE_lexicon_tags_2 },
    { 0xd15ccd70, SUC_NE_lexicon_tags_2 },
    { 0x5892cd70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea88cd74, SUC_NE_lexicon_tags_2 },
    { 0xb21ecd75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x119ccd78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x552ecd7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf144cd7f, SUC_NE_lexicon_tags_2 },
    { 0xcc3ccd80, SUC_NE_lexicon_tags_438 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abccd89, SUC_NE_lexicon_tags_2 },
    { 0xe26acd8a, SUC_NE_lexicon_tags_2 },
    { 0xcf40cd8b, SUC_NE_lexicon_tags_2 },
    { 0x646acd8b, SUC_NE_lexicon_tags_2 },
    { 0x2f4cd89, SUC_NE_lexicon_tags_2 },
    { 0x8c2ecd8e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ad0cd90, SUC_NE_lexicon_tags_2 },
    { 0x1c9acd90, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6e2cd93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1292cd98, SUC_NE_lexicon_tags_7 },
    { 0xb380cd98, SUC_NE_lexicon_tags_17 },
    { 0xd186cd9a, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b72cda0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6040cda3, SUC_NE_lexicon_tags_2 },
    { 0xce96cda3, SUC_NE_lexicon_tags_2 },
    { 0xd564cda5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7568cda9, SUC_NE_lexicon_tags_2 },
    { 0x7c90cdaa, SUC_NE_lexicon_tags_4 },
    { 0xf12ccdab, SUC_NE_lexicon_tags_4 },
    { 0x4ac0cdac, SUC_NE_lexicon_tags_2 },
    { 0xf7fccda9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e4cdb1, SUC_NE_lexicon_tags_2 },
    { 0xb28acdb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e24cdb6, SUC_NE_lexicon_tags_7 },
    { 0x7faecdb6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0ecdbb, SUC_NE_lexicon_tags_2 },
    { 0xe748cdbc, SUC_NE_lexicon_tags_2 },
    { 0xe440cdbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ac2cdbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef14cdc6, SUC_NE_lexicon_tags_2 },
    { 0x5e2cdc7, SUC_NE_lexicon_tags_3 },
    { 0xe3d8cdc8, SUC_NE_lexicon_tags_2 },
    { 0x452ccdc6, SUC_NE_lexicon_tags_2 },
    { 0xab8ecdc7, SUC_NE_lexicon_tags_2 },
    { 0xd222cdcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f36cdcd, SUC_NE_lexicon_tags_5 },
    { 0xac1acdce, SUC_NE_lexicon_tags_59 },
    { 0x21f0cdce, SUC_NE_lexicon_tags_2 },
    { 0xa104cdce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa122cdd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x524acdd6, SUC_NE_lexicon_tags_2 },
    { 0xa8dacdd7, SUC_NE_lexicon_tags_2 },
    { 0xd276cdd8, SUC_NE_lexicon_tags_36 },
    { 0x7982cdd9, SUC_NE_lexicon_tags_2 },
    { 0xdd6acdd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60ecde0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd62cde4, SUC_NE_lexicon_tags_2 },
    { 0xa5dacde5, SUC_NE_lexicon_tags_2 },
    { 0x358ccde6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3216cde9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e84cdeb, SUC_NE_lexicon_tags_2 },
    { 0xa87acdec, SUC_NE_lexicon_tags_2 },
    { 0x1768cdec, SUC_NE_lexicon_tags_2 },
    { 0xa726cded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc956cdf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5206cdf5, SUC_NE_lexicon_tags_2 },
    { 0xd0fccdf6, SUC_NE_lexicon_tags_2 },
    { 0x4c46cdf6, SUC_NE_lexicon_tags_2 },
    { 0xcd12cdf8, SUC_NE_lexicon_tags_2 },
    { 0xedaacdf7, SUC_NE_lexicon_tags_34 },
    { 0x7340cdfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d0cdff, SUC_NE_lexicon_tags_2 },
    { 0xd73ecdff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x654ece02, SUC_NE_lexicon_tags_19 },
    { 0x3574ce02, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0xda9cce05, SUC_NE_lexicon_tags_2 },
    { 0x2a8cce06, SUC_NE_lexicon_tags_2 },
    { 0xad4ece06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d4ce10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ace16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7058ce1a, SUC_NE_lexicon_tags_2 },
    { 0xd776ce1b, SUC_NE_lexicon_tags_2 },
    { 0x1e84ce1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd98ace1e, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6642ce28, SUC_NE_lexicon_tags_2 },
    { 0x8590ce29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31a6ce2b, SUC_NE_lexicon_tags_3 },
    { 0xf74ace2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xedb8ce2e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb978ce30, SUC_NE_lexicon_tags_2 },
    { 0xb19ece30, SUC_NE_lexicon_tags_220 },
    { 0xbe14ce32, SUC_NE_lexicon_tags_11 },
    { 0xf08ace33, SUC_NE_lexicon_tags_2 },
    { 0x9d88ce32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5628ce36, SUC_NE_lexicon_tags_2 },
    { 0xac4ece36, SUC_NE_lexicon_tags_2 },
    { 0xefb2ce38, SUC_NE_lexicon_tags_2 },
    { 0x41dcce39, SUC_NE_lexicon_tags_2 },
    { 0x5788ce3a, SUC_NE_lexicon_tags_2 },
    { 0x869cce3b, SUC_NE_lexicon_tags_2 },
    { 0xa89cce3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf84ce3e, SUC_NE_lexicon_tags_2 },
    { 0xc774ce3f, SUC_NE_lexicon_tags_2 },
    { 0x49b6ce40, SUC_NE_lexicon_tags_2 },
    { 0x4252ce41, SUC_NE_lexicon_tags_2 },
    { 0xf25ece42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a98ce44, SUC_NE_lexicon_tags_5 },
    { 0x71a8ce45, SUC_NE_lexicon_tags_2 },
    { 0xe328ce46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x239ece49, SUC_NE_lexicon_tags_18 },
    { 0xce84ce49, SUC_NE_lexicon_tags_2 },
    { 0xf4dece4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed7cce4d, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ecce5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec0ce61, SUC_NE_lexicon_tags_2 },
    { 0xe5cace62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbace65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x536cce68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a9ace6a, SUC_NE_lexicon_tags_2 },
    { 0x2c76ce6b, SUC_NE_lexicon_tags_2 },
    { 0x8ed2ce6c, SUC_NE_lexicon_tags_2 },
    { 0x666ace6d, SUC_NE_lexicon_tags_2 },
    { 0x4ca6ce6e, SUC_NE_lexicon_tags_4 },
    { 0x602cce6c, SUC_NE_lexicon_tags_2 },
    { 0x9c0cce70, SUC_NE_lexicon_tags_2 },
    { 0xa824ce71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cace75, SUC_NE_lexicon_tags_2 },
    { 0x7b8cce75, SUC_NE_lexicon_tags_2 },
    { 0xe44ece76, SUC_NE_lexicon_tags_2 },
    { 0xbb1ace78, SUC_NE_lexicon_tags_2 },
    { 0x9658ce79, SUC_NE_lexicon_tags_2 },
    { 0xe704ce7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa212ce7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1288ce80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1e4ce82, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x10f4ce84, SUC_NE_lexicon_tags_2 },
    { 0x6314ce84, SUC_NE_lexicon_tags_1 },
    { 0xf5ccce86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ece8e, SUC_NE_lexicon_tags_2 },
    { 0xdb60ce8f, SUC_NE_lexicon_tags_2 },
    { 0x9008ce90, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xf216ce92, SUC_NE_lexicon_tags_170 },
    { 0xf168ce93, SUC_NE_lexicon_tags_2 },
    { 0x999ace92, SUC_NE_lexicon_tags_18 },
    { 0x4e3ace92, SUC_NE_lexicon_tags_2 },
    { 0x7160ce95, SUC_NE_lexicon_tags_34 },
    { 0xa358ce95, SUC_NE_lexicon_tags_3 },
    { 0x7944ce97, SUC_NE_lexicon_tags_2 },
    { 0x96cece99, SUC_NE_lexicon_tags_2 },
    { 0x5670ce98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x799ece9c, SUC_NE_lexicon_tags_2 },
    { 0x688ece9c, SUC_NE_lexicon_tags_2 },
    { 0x124cce9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea20cea0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ecea4, SUC_NE_lexicon_tags_2 },
    { 0xc17ccea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61becea7, SUC_NE_lexicon_tags_2 },
    { 0x7c22cea8, SUC_NE_lexicon_tags_0 },
    { 0xb05acea7, SUC_NE_lexicon_tags_2 },
    { 0xb9e6cea8, SUC_NE_lexicon_tags_2 },
    { 0xaabaceab, SUC_NE_lexicon_tags_2 },
    { 0xc0cacea9, SUC_NE_lexicon_tags_2 },
    { 0xe028cead, SUC_NE_lexicon_tags_2 },
    { 0xe38aceae, SUC_NE_lexicon_tags_2 },
    { 0xaedccead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce0ceb4, SUC_NE_lexicon_tags_3 },
    { 0xc43aceb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe600ceba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8ecebd, SUC_NE_lexicon_tags_2 },
    { 0xa0c6cebe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1dacec0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe92ccec2, SUC_NE_lexicon_tags_2 },
    { 0x2666cec2, SUC_NE_lexicon_tags_2 },
    { 0x6948cec4, SUC_NE_lexicon_tags_2 },
    { 0x739ecec4, SUC_NE_lexicon_tags_2 },
    { 0x858ecec3, SUC_NE_lexicon_tags_2 },
    { 0x1b44cec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eececb, SUC_NE_lexicon_tags_4 },
    { 0x615ececc, SUC_NE_lexicon_tags_2 },
    { 0x5bb6cecd, SUC_NE_lexicon_tags_2 },
    { 0x3864cecc, SUC_NE_lexicon_tags_2 },
    { 0x812ccece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c20ced2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf50ced4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eced9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18cedd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf410cee0, SUC_NE_lexicon_tags_2 },
    { 0x56a4cee1, SUC_NE_lexicon_tags_2 },
    { 0xb06acee2, SUC_NE_lexicon_tags_7 },
    { 0xd324cee2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abccee6, SUC_NE_lexicon_tags_421 },
    { 0, NULL },
    { 0x8c0ccee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c5cceea, SUC_NE_lexicon_tags_2 },
    { 0xea6eceea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd44ceee, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58accef2, SUC_NE_lexicon_tags_2 },
    { 0x772ccef3, SUC_NE_lexicon_tags_2 },
    { 0x1cf8cef2, SUC_NE_lexicon_tags_2 },
    { 0xe2f4cef5, SUC_NE_lexicon_tags_2 },
    { 0xf178cef3, SUC_NE_lexicon_tags_2 },
    { 0x4adecef7, SUC_NE_lexicon_tags_2 },
    { 0x3272cef8, SUC_NE_lexicon_tags_2 },
    { 0xbe62cef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b72cefb, SUC_NE_lexicon_tags_2 },
    { 0xb454cefc, SUC_NE_lexicon_tags_2 },
    { 0x186acefd, SUC_NE_lexicon_tags_2 },
    { 0x248ecefd, SUC_NE_lexicon_tags_2 },
    { 0xe4d4cefe, SUC_NE_lexicon_tags_2 },
    { 0xe8b8cefb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b56cf0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda16cf0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbacf12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d78cf15, SUC_NE_lexicon_tags_2 },
    { 0x5850cf16, SUC_NE_lexicon_tags_30 },
    { 0x3856cf17, SUC_NE_lexicon_tags_2 },
    { 0x59a2cf15, SUC_NE_lexicon_tags_2 },
    { 0x409ecf17, SUC_NE_lexicon_tags_2 },
    { 0xcdb0cf1a, SUC_NE_lexicon_tags_2 },
    { 0x8b00cf1b, SUC_NE_lexicon_tags_2 },
    { 0x752cf18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e10cf1e, SUC_NE_lexicon_tags_2 },
    { 0x93a6cf1f, SUC_NE_lexicon_tags_2 },
    { 0x3558cf20, SUC_NE_lexicon_tags_4 },
    { 0x6ea6cf21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc474cf25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd6cf28, SUC_NE_lexicon_tags_2 },
    { 0x56c0cf29, SUC_NE_lexicon_tags_2 },
    { 0xb960cf29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadd2cf2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e6cf30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff2cf33, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe458cf37, SUC_NE_lexicon_tags_2 },
    { 0x97decf38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7130cf3c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x686cf3f, SUC_NE_lexicon_tags_2 },
    { 0x43a8cf40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f62cf44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c14cf46, SUC_NE_lexicon_tags_2 },
    { 0x3b38cf47, SUC_NE_lexicon_tags_2 },
    { 0x7ba2cf48, SUC_NE_lexicon_tags_2 },
    { 0xec0cf47, SUC_NE_lexicon_tags_2 },
    { 0x8ceacf46, SUC_NE_lexicon_tags_2 },
    { 0x3c3ecf4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1750cf4d, SUC_NE_lexicon_tags_7 },
    { 0x7d32cf4e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x446cf51, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x23bacf53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9946cf56, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be2cf59, SUC_NE_lexicon_tags_2 },
    { 0x580cf5a, SUC_NE_lexicon_tags_11 },
    { 0x6e70cf5b, SUC_NE_lexicon_tags_2 },
    { 0x844cf5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x848ccf5e, SUC_NE_lexicon_tags_2 },
    { 0x1b66cf5f, SUC_NE_lexicon_tags_2 },
    { 0x6540cf5e, SUC_NE_lexicon_tags_2 },
    { 0x72f4cf61, SUC_NE_lexicon_tags_2 },
    { 0xe204cf60, SUC_NE_lexicon_tags_2 },
    { 0xcafacf63, SUC_NE_lexicon_tags_3 },
    { 0x5868cf64, SUC_NE_lexicon_tags_2 },
    { 0xcb4ccf63, SUC_NE_lexicon_tags_2 },
    { 0xc7facf62, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecccf70, SUC_NE_lexicon_tags_2 },
    { 0xac5ccf71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d8cf76, SUC_NE_lexicon_tags_2 },
    { 0xe0a8cf77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x519ccf7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f8ccf7c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7acf83, SUC_NE_lexicon_tags_2 },
    { 0x27eacf84, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x1566cf86, SUC_NE_lexicon_tags_2 },
    { 0x38e0cf87, SUC_NE_lexicon_tags_2 },
    { 0xa9a2cf88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7becf8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0cf90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eecf92, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x34b8cf94, SUC_NE_lexicon_tags_2 },
    { 0x892cf94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4cf98, SUC_NE_lexicon_tags_2 },
    { 0xd028cf99, SUC_NE_lexicon_tags_2 },
    { 0x840ccf98, SUC_NE_lexicon_tags_2 },
    { 0x28d2cf9b, SUC_NE_lexicon_tags_2 },
    { 0xebbecf9b, SUC_NE_lexicon_tags_2 },
    { 0x4e3ecf9c, SUC_NE_lexicon_tags_2 },
    { 0x508cf9e, SUC_NE_lexicon_tags_2 },
    { 0xcd60cf9f, SUC_NE_lexicon_tags_19 },
    { 0xc43ecf9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9652cfa3, SUC_NE_lexicon_tags_2 },
    { 0x17b0cfa4, SUC_NE_lexicon_tags_2 },
    { 0xf5decfa3, SUC_NE_lexicon_tags_4 },
    { 0xfb00cfa5, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0x7942cfa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bccfae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4cecfb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dc0cfb2, SUC_NE_lexicon_tags_24 },
    { 0x1af8cfb3, SUC_NE_lexicon_tags_2 },
    { 0xfd3ecfb4, SUC_NE_lexicon_tags_2 },
    { 0x2a96cfb5, SUC_NE_lexicon_tags_2 },
    { 0xa6d0cfb4, SUC_NE_lexicon_tags_2 },
    { 0x184cfb3, SUC_NE_lexicon_tags_2 },
    { 0x5ca2cfb7, SUC_NE_lexicon_tags_102 },
    { 0x986ecfb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3216cfbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83accfc1, SUC_NE_lexicon_tags_2 },
    { 0x7030cfc2, SUC_NE_lexicon_tags_2 },
    { 0x4f00cfc3, SUC_NE_lexicon_tags_1 },
    { 0xcd72cfc1, SUC_NE_lexicon_tags_2 },
    { 0x18aecfc5, SUC_NE_lexicon_tags_2 },
    { 0x56e2cfc4, SUC_NE_lexicon_tags_2 },
    { 0xf5f2cfc7, SUC_NE_lexicon_tags_86 },
    { 0xd022cfc8, SUC_NE_lexicon_tags_2 },
    { 0x7acfc4, SUC_NE_lexicon_tags_2 },
    { 0x43a6cfca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x356acfcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x355ccfcf, SUC_NE_lexicon_tags_3 },
    { 0xc40cfd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f0cfd8, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe4ecfda, SUC_NE_lexicon_tags_2 },
    { 0x2c8ccfdb, SUC_NE_lexicon_tags_2 },
    { 0xa752cfda, SUC_NE_lexicon_tags_2 },
    { 0xec4ccfdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3602cfdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1002cfe4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540acfe8, SUC_NE_lexicon_tags_2 },
    { 0x9d94cfe8, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a0cff2, SUC_NE_lexicon_tags_2 },
    { 0xc824cff3, SUC_NE_lexicon_tags_2 },
    { 0x504ccff4, SUC_NE_lexicon_tags_2 },
    { 0x92f8cff5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff4cff8, SUC_NE_lexicon_tags_2 },
    { 0xc708cff9, SUC_NE_lexicon_tags_2 },
    { 0x527ccffa, SUC_NE_lexicon_tags_2 },
    { 0x6a72cff9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf516cfff, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a12d006, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x733cd008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0ad00d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae4ad00f, SUC_NE_lexicon_tags_2 },
    { 0xe1a6d00f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b00d017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9228d01d, SUC_NE_lexicon_tags_2 },
    { 0x9414d01d, SUC_NE_lexicon_tags_2 },
    { 0xfeb6d01f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x788d021, SUC_NE_lexicon_tags_2 },
    { 0x843cd022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbad027, SUC_NE_lexicon_tags_2 },
    { 0xcdb4d028, SUC_NE_lexicon_tags_4 },
    { 0xde70d029, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4970d02b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6dcd02d, SUC_NE_lexicon_tags_2 },
    { 0x8fb6d02e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9050d034, SUC_NE_lexicon_tags_2 },
    { 0xc816d035, SUC_NE_lexicon_tags_2 },
    { 0x39c0d036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa080d03c, SUC_NE_lexicon_tags_2 },
    { 0xd1ced03c, SUC_NE_lexicon_tags_2 },
    { 0x9c22d03d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdab6d040, SUC_NE_lexicon_tags_2 },
    { 0x61c0d041, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x5f7cd043, SUC_NE_lexicon_tags_2 },
    { 0x93e8d044, SUC_NE_lexicon_tags_2 },
    { 0x68eed045, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838cd049, SUC_NE_lexicon_tags_2 },
    { 0xcc90d04a, SUC_NE_lexicon_tags_2 },
    { 0xf284d04a, SUC_NE_lexicon_tags_2 },
    { 0xcba2d04c, SUC_NE_lexicon_tags_2 },
    { 0x18c4d04d, SUC_NE_lexicon_tags_2 },
    { 0x3ca6d04e, SUC_NE_lexicon_tags_2 },
    { 0x6dd0d04f, SUC_NE_lexicon_tags_2 },
    { 0x954d04e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8cd053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7cd055, SUC_NE_lexicon_tags_2 },
    { 0x6cc2d056, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce7cd058, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4266d061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac98d066, SUC_NE_lexicon_tags_2 },
    { 0x7f36d066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fed06c, SUC_NE_lexicon_tags_2 },
    { 0x783ed06d, SUC_NE_lexicon_tags_2 },
    { 0x99aed06e, SUC_NE_lexicon_tags_2 },
    { 0x919cd06e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c42d072, SUC_NE_lexicon_tags_2 },
    { 0x866ad072, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd4d079, SUC_NE_lexicon_tags_2 },
    { 0xa42d079, SUC_NE_lexicon_tags_2 },
    { 0xe2d6d079, SUC_NE_lexicon_tags_2 },
    { 0x3c32d07c, SUC_NE_lexicon_tags_2 },
    { 0xb232d07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870ad083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e4d086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1acd089, SUC_NE_lexicon_tags_2 },
    { 0x89ead08a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd92cd08c, SUC_NE_lexicon_tags_2 },
    { 0xf8e4d08d, SUC_NE_lexicon_tags_2 },
    { 0x4bb8d08e, SUC_NE_lexicon_tags_2 },
    { 0x511ad08d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26acd098, SUC_NE_lexicon_tags_2 },
    { 0x6c2cd098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x241ed09b, SUC_NE_lexicon_tags_2 },
    { 0x6e14d09c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35f2d09e, SUC_NE_lexicon_tags_2 },
    { 0x7e14d09f, SUC_NE_lexicon_tags_2 },
    { 0x2104d0a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc72d0a5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3326d0aa, SUC_NE_lexicon_tags_2 },
    { 0x2bed0aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7290d0ae, SUC_NE_lexicon_tags_2 },
    { 0x82eed0af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6fed0b1, SUC_NE_lexicon_tags_2 },
    { 0xe318d0b2, SUC_NE_lexicon_tags_2 },
    { 0xd20ad0b3, SUC_NE_lexicon_tags_17 },
    { 0x41f6d0b4, SUC_NE_lexicon_tags_2 },
    { 0x968cd0b5, SUC_NE_lexicon_tags_2 },
    { 0xc09cd0b1, SUC_NE_lexicon_tags_4 },
    { 0x2c46d0b6, SUC_NE_lexicon_tags_34 },
    { 0xc494d0b8, SUC_NE_lexicon_tags_2 },
    { 0xc970d0b8, SUC_NE_lexicon_tags_2 },
    { 0xb90ad0b6, SUC_NE_lexicon_tags_2 },
    { 0x3d8d0b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x724ed0bd, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb070d0bf, SUC_NE_lexicon_tags_2 },
    { 0xea3ad0c0, SUC_NE_lexicon_tags_2 },
    { 0xa552d0c1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe342d0c6, SUC_NE_lexicon_tags_2 },
    { 0x792d0c7, SUC_NE_lexicon_tags_2 },
    { 0x4290d0c8, SUC_NE_lexicon_tags_2 },
    { 0x61ecd0c9, SUC_NE_lexicon_tags_2 },
    { 0x48b8d0ca, SUC_NE_lexicon_tags_3 },
    { 0x90ded0cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16ed0d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484d0d5, SUC_NE_lexicon_tags_2 },
    { 0x6216d0d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4ed0d9, SUC_NE_lexicon_tags_19 },
    { 0xdb26d0da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b4d0dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dad0df, SUC_NE_lexicon_tags_2 },
    { 0xad60d0df, SUC_NE_lexicon_tags_2 },
    { 0x7d62d0df, SUC_NE_lexicon_tags_2 },
    { 0xa900d0e2, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x1528d0e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9ed0e7, SUC_NE_lexicon_tags_2 },
    { 0xee0d0e8, SUC_NE_lexicon_tags_2 },
    { 0xa9ad0e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d82d0ef, SUC_NE_lexicon_tags_2 },
    { 0x844d0f0, SUC_NE_lexicon_tags_2 },
    { 0x8c22d0f1, SUC_NE_lexicon_tags_2 },
    { 0x798cd0f2, SUC_NE_lexicon_tags_2 },
    { 0x92d0ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9914d0f8, SUC_NE_lexicon_tags_2 },
    { 0xaf06d0f9, SUC_NE_lexicon_tags_2 },
    { 0xb298d0f9, SUC_NE_lexicon_tags_2 },
    { 0x37c6d0fb, SUC_NE_lexicon_tags_2 },
    { 0xbc3ad0fc, SUC_NE_lexicon_tags_2 },
    { 0xc9acd0fc, SUC_NE_lexicon_tags_2 },
    { 0x8e16d0fe, SUC_NE_lexicon_tags_2 },
    { 0xbd10d0ff, SUC_NE_lexicon_tags_2 },
    { 0xbf46d100, SUC_NE_lexicon_tags_2 },
    { 0x1f98d0f9, SUC_NE_lexicon_tags_2 },
    { 0xa876d102, SUC_NE_lexicon_tags_2 },
    { 0x3e2ed0ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdac0d105, SUC_NE_lexicon_tags_36 },
    { 0xbe72d106, SUC_NE_lexicon_tags_2 },
    { 0xfcead107, SUC_NE_lexicon_tags_2 },
    { 0x46f4d108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6d10d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1068d10f, SUC_NE_lexicon_tags_2 },
    { 0x97ead110, SUC_NE_lexicon_tags_5 },
    { 0xad10d111, SUC_NE_lexicon_tags_7 },
    { 0x8ba4d112, SUC_NE_lexicon_tags_2 },
    { 0x8686d113, SUC_NE_lexicon_tags_2 },
    { 0x6d96d112, SUC_NE_lexicon_tags_2 },
    { 0xb2e4d115, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa52d117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ecd11b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581cd11f, SUC_NE_lexicon_tags_2 },
    { 0x579ed120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefd0d122, SUC_NE_lexicon_tags_2 },
    { 0x911ed123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8446d128, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeed4d12a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5bcd12c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66d12f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf52d131, SUC_NE_lexicon_tags_2 },
    { 0x4014d131, SUC_NE_lexicon_tags_2 },
    { 0x4d56d133, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb982d13b, SUC_NE_lexicon_tags_81 },
    { 0xe7b8d13c, SUC_NE_lexicon_tags_2 },
    { 0x1dacd13c, SUC_NE_lexicon_tags_2 },
    { 0xde2ed13d, SUC_NE_lexicon_tags_2 },
    { 0xb226d13d, SUC_NE_lexicon_tags_2 },
    { 0x56f0d13d, SUC_NE_lexicon_tags_1 },
    { 0xaf0d13f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23d0d143, SUC_NE_lexicon_tags_2 },
    { 0x8ced143, SUC_NE_lexicon_tags_2 },
    { 0x300ed144, SUC_NE_lexicon_tags_7 },
    { 0x9710d143, SUC_NE_lexicon_tags_2 },
    { 0x6730d144, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718cd14b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ffcd14d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1ed151, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07cd154, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4782d157, SUC_NE_lexicon_tags_2 },
    { 0xc4bcd158, SUC_NE_lexicon_tags_4 },
    { 0x4814d158, SUC_NE_lexicon_tags_2 },
    { 0x6e12d159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d0d15d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a8d162, SUC_NE_lexicon_tags_2 },
    { 0xd368d162, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4226d169, SUC_NE_lexicon_tags_105 },
    { 0x8890d169, SUC_NE_lexicon_tags_7 },
    { 0xa14d16b, SUC_NE_lexicon_tags_2 },
    { 0xb348d16a, SUC_NE_lexicon_tags_2 },
    { 0x27e4d16d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e2d173, SUC_NE_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbad176, SUC_NE_lexicon_tags_2 },
    { 0x770d177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596ed17b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5cd185, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x214cd187, SUC_NE_lexicon_tags_2 },
    { 0x312cd187, SUC_NE_lexicon_tags_2 },
    { 0xfb12d189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f6d18b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7a8d18d, SUC_NE_lexicon_tags_2 },
    { 0x7cded18e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb0d19a, SUC_NE_lexicon_tags_1 },
    { 0xc2d4d19b, SUC_NE_lexicon_tags_2 },
    { 0xbed0d19c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc800d19f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x909ad1a1, SUC_NE_lexicon_tags_2 },
    { 0x4536d1a2, SUC_NE_lexicon_tags_2 },
    { 0xe7ced1a2, SUC_NE_lexicon_tags_2 },
    { 0x899ad1a4, SUC_NE_lexicon_tags_2 },
    { 0x10e2d1a2, SUC_NE_lexicon_tags_2 },
    { 0x5492d1a2, SUC_NE_lexicon_tags_2 },
    { 0x118ad1a2, SUC_NE_lexicon_tags_2 },
    { 0x6634d1a8, SUC_NE_lexicon_tags_2 },
    { 0x5cbcd1a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaed1ad, SUC_NE_lexicon_tags_34 },
    { 0x433ad1ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd0d1b2, SUC_NE_lexicon_tags_2 },
    { 0xd8ccd1b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392ed1ba, SUC_NE_lexicon_tags_2 },
    { 0xdfdcd1bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda04d1bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bed1c3, SUC_NE_lexicon_tags_291 },
    { 0x9704d1c4, SUC_NE_lexicon_tags_2 },
    { 0xb44ad1c5, SUC_NE_lexicon_tags_2 },
    { 0xd112d1c6, SUC_NE_lexicon_tags_2 },
    { 0x5ad8d1c7, SUC_NE_lexicon_tags_2 },
    { 0x6142d1c7, SUC_NE_lexicon_tags_2 },
    { 0x4098d1c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5726d1cd, SUC_NE_lexicon_tags_2 },
    { 0x418ed1cd, SUC_NE_lexicon_tags_30 },
    { 0x301ad1cf, SUC_NE_lexicon_tags_2 },
    { 0xca4d1cf, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x56a6d1d2, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56cd1d6, SUC_NE_lexicon_tags_2 },
    { 0xe4d1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e00d1da, SUC_NE_lexicon_tags_2 },
    { 0x5ce4d1db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12c0d1dd, SUC_NE_lexicon_tags_2 },
    { 0xfe30d1de, SUC_NE_lexicon_tags_4 },
    { 0x7c9ad1de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ccd1e1, SUC_NE_lexicon_tags_2 },
    { 0x3efad1e1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5750d1e6, SUC_NE_lexicon_tags_2 },
    { 0x5ccad1e7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ccd1ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9392d1ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8440d1f0, SUC_NE_lexicon_tags_2 },
    { 0xf9bcd1f1, SUC_NE_lexicon_tags_2 },
    { 0xa42cd1f1, SUC_NE_lexicon_tags_2 },
    { 0x2bead1f3, SUC_NE_lexicon_tags_2 },
    { 0xb94cd1f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee32d1f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9154d1fa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe238d1fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf13cd1ff, SUC_NE_lexicon_tags_2 },
    { 0xb7e6d200, SUC_NE_lexicon_tags_2 },
    { 0xac1ed201, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd860d205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aad208, SUC_NE_lexicon_tags_7 },
    { 0x2e9ad208, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x914ad20b, SUC_NE_lexicon_tags_2 },
    { 0x6888d20b, SUC_NE_lexicon_tags_2 },
    { 0xc27ad20d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca0d212, SUC_NE_lexicon_tags_2 },
    { 0x448ed212, SUC_NE_lexicon_tags_374 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f14d21d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf788d227, SUC_NE_lexicon_tags_2 },
    { 0x2202d228, SUC_NE_lexicon_tags_2 },
    { 0xe9ead229, SUC_NE_lexicon_tags_1 },
    { 0x2b36d22a, SUC_NE_lexicon_tags_2 },
    { 0x27ced228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fed22e, SUC_NE_lexicon_tags_220 },
    { 0x580ed22f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf15ed231, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3decd237, SUC_NE_lexicon_tags_2 },
    { 0xd022d238, SUC_NE_lexicon_tags_2 },
    { 0x7b5ed238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ffed23b, SUC_NE_lexicon_tags_8 },
    { 0x7004d23c, SUC_NE_lexicon_tags_2 },
    { 0xef92d23b, SUC_NE_lexicon_tags_3 },
    { 0xcfeed23e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd944d241, SUC_NE_lexicon_tags_4 },
    { 0xa9e8d242, SUC_NE_lexicon_tags_2 },
    { 0x61aed243, SUC_NE_lexicon_tags_2 },
    { 0x7eead244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x916ad246, SUC_NE_lexicon_tags_2 },
    { 0xcfb0d247, SUC_NE_lexicon_tags_2 },
    { 0xed20d247, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe50d252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ccd258, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bad25c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604ad261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3788d263, SUC_NE_lexicon_tags_1 },
    { 0xe010d264, SUC_NE_lexicon_tags_2 },
    { 0x5536d265, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2388d267, SUC_NE_lexicon_tags_2 },
    { 0x28cad268, SUC_NE_lexicon_tags_2 },
    { 0xaa64d269, SUC_NE_lexicon_tags_2 },
    { 0x845ed26a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x752ad26c, SUC_NE_lexicon_tags_7 },
    { 0x6e22d26c, SUC_NE_lexicon_tags_5 },
    { 0xc6f6d26e, SUC_NE_lexicon_tags_2 },
    { 0xca6ed26d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe410d272, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0xa958d274, SUC_NE_lexicon_tags_2 },
    { 0x60c0d275, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe636d277, SUC_NE_lexicon_tags_2 },
    { 0xf916d278, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ccd27e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2ed281, SUC_NE_lexicon_tags_2 },
    { 0x1ec6d281, SUC_NE_lexicon_tags_2 },
    { 0xee54d282, SUC_NE_lexicon_tags_3 },
    { 0xc11cd284, SUC_NE_lexicon_tags_2 },
    { 0xb266d285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e86d287, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d66d28a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4ed290, SUC_NE_lexicon_tags_2 },
    { 0x61a4d291, SUC_NE_lexicon_tags_2 },
    { 0x1922d292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce5ad294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8a0d296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed82d298, SUC_NE_lexicon_tags_2 },
    { 0xc2bcd299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7e2d29b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33cd29e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb56d2a0, SUC_NE_lexicon_tags_2 },
    { 0xc7c4d2a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1cd2a3, SUC_NE_lexicon_tags_2 },
    { 0xbc9ad2a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa56d2a6, SUC_NE_lexicon_tags_2 },
    { 0x5656d2a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd64d2a9, SUC_NE_lexicon_tags_3 },
    { 0xd844d2aa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b4d2ad, SUC_NE_lexicon_tags_2 },
    { 0x936ad2ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x949cd2b1, SUC_NE_lexicon_tags_2 },
    { 0x45ed2b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18cd2b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3124d2bf, SUC_NE_lexicon_tags_10 },
    { 0xb586d2c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa190d2c2, SUC_NE_lexicon_tags_2 },
    { 0x7eced2c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb674d2c7, SUC_NE_lexicon_tags_2 },
    { 0xfd78d2c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x880ed2ca, SUC_NE_lexicon_tags_3 },
    { 0x95a0d2cb, SUC_NE_lexicon_tags_2 },
    { 0x1a46d2cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xded8d2ce, SUC_NE_lexicon_tags_2 },
    { 0x9276d2cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x113cd2d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ea4d2d3, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b96d2d6, SUC_NE_lexicon_tags_2 },
    { 0xc626d2d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cd2dd, SUC_NE_lexicon_tags_2 },
    { 0xc9d4d2dd, SUC_NE_lexicon_tags_3 },
    { 0xd24ad2df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x982cd2e1, SUC_NE_lexicon_tags_34 },
    { 0xe4f2d2e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7454d2e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2838d2ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb898d2ec, SUC_NE_lexicon_tags_2 },
    { 0xedf6d2ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852ed2f2, SUC_NE_lexicon_tags_2 },
    { 0x900ad2f2, SUC_NE_lexicon_tags_2 },
    { 0xf352d2f4, SUC_NE_lexicon_tags_2 },
    { 0xf3aed2f5, SUC_NE_lexicon_tags_2 },
    { 0xc1f4d2f5, SUC_NE_lexicon_tags_2 },
    { 0x43f4d2f7, SUC_NE_lexicon_tags_2 },
    { 0x6ac0d2f8, SUC_NE_lexicon_tags_2 },
    { 0x1f42d2f7, SUC_NE_lexicon_tags_2 },
    { 0xcaaad2f4, SUC_NE_lexicon_tags_2 },
    { 0x5ae0d2fb, SUC_NE_lexicon_tags_2 },
    { 0x8246d2fc, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220d300, SUC_NE_lexicon_tags_422 },
    { 0xa050d301, SUC_NE_lexicon_tags_2 },
    { 0x3938d301, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8560d305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74aed307, SUC_NE_lexicon_tags_2 },
    { 0x5c3ad308, SUC_NE_lexicon_tags_2 },
    { 0x2632d309, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfced30b, SUC_NE_lexicon_tags_2 },
    { 0xb56cd30b, SUC_NE_lexicon_tags_2 },
    { 0x7886d30d, SUC_NE_lexicon_tags_2 },
    { 0xbe70d30b, SUC_NE_lexicon_tags_2 },
    { 0xbe00d30e, SUC_NE_lexicon_tags_2 },
    { 0x1916d310, SUC_NE_lexicon_tags_2 },
    { 0x73d4d311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bad314, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6ed317, SUC_NE_lexicon_tags_296 },
    { 0, NULL },
    { 0x4170d319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6ed31d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6810d31f, SUC_NE_lexicon_tags_2 },
    { 0x9482d320, SUC_NE_lexicon_tags_2 },
    { 0x160d31f, SUC_NE_lexicon_tags_2 },
    { 0xaf6ed320, SUC_NE_lexicon_tags_2 },
    { 0x144cd320, SUC_NE_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0x2144d326, SUC_NE_lexicon_tags_2 },
    { 0xd9b6d326, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0x2ba8d329, SUC_NE_lexicon_tags_5 },
    { 0x7152d329, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff2d330, SUC_NE_lexicon_tags_2 },
    { 0x5bd6d331, SUC_NE_lexicon_tags_12 },
    { 0x8a86d332, SUC_NE_lexicon_tags_2 },
    { 0x1e40d330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc59ad335, SUC_NE_lexicon_tags_2 },
    { 0x2d64d335, SUC_NE_lexicon_tags_4 },
    { 0x7fb0d337, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5490d339, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e6d33d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb960d33f, SUC_NE_lexicon_tags_2 },
    { 0x7844d340, SUC_NE_lexicon_tags_2 },
    { 0x5e7ed341, SUC_NE_lexicon_tags_2 },
    { 0x787ad341, SUC_NE_lexicon_tags_2 },
    { 0x6dfad343, SUC_NE_lexicon_tags_2 },
    { 0x7a6ad343, SUC_NE_lexicon_tags_2 },
    { 0xe574d345, SUC_NE_lexicon_tags_2 },
    { 0x6464d341, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d14d34c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf262d350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8258d354, SUC_NE_lexicon_tags_2 },
    { 0x6740d354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7aa8d357, SUC_NE_lexicon_tags_300 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ad35c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc01ad35e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x189ed360, SUC_NE_lexicon_tags_2 },
    { 0x49f2d361, SUC_NE_lexicon_tags_2 },
    { 0x4598d362, SUC_NE_lexicon_tags_2 },
    { 0x8bed360, SUC_NE_lexicon_tags_2 },
    { 0x719cd364, SUC_NE_lexicon_tags_2 },
    { 0xec52d365, SUC_NE_lexicon_tags_2 },
    { 0x3e70d365, SUC_NE_lexicon_tags_57 },
    { 0xa6c0d363, SUC_NE_lexicon_tags_2 },
    { 0xce48d366, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea06d36b, SUC_NE_lexicon_tags_183 },
    { 0, NULL },
    { 0xe9aed36d, SUC_NE_lexicon_tags_2 },
    { 0xb9e6d36d, SUC_NE_lexicon_tags_0 },
    { 0xf996d36f, SUC_NE_lexicon_tags_2 },
    { 0x2eccd36f, SUC_NE_lexicon_tags_2 },
    { 0xfc6d371, SUC_NE_lexicon_tags_2 },
    { 0xf072d372, SUC_NE_lexicon_tags_0 },
    { 0xe0bad371, SUC_NE_lexicon_tags_15 },
    { 0x6f7ad374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc574d376, SUC_NE_lexicon_tags_2 },
    { 0x1ef2d377, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18bed379, SUC_NE_lexicon_tags_17 },
    { 0x5aecd37a, SUC_NE_lexicon_tags_2 },
    { 0x5dd2d37b, SUC_NE_lexicon_tags_2 },
    { 0x129cd37c, SUC_NE_lexicon_tags_2 },
    { 0x14c8d37b, SUC_NE_lexicon_tags_2 },
    { 0x6f0d37e, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x7c5cd380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ecd385, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4d38c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1cd38f, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xebdcd391, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8246d393, SUC_NE_lexicon_tags_46 },
    { 0x80c6d394, SUC_NE_lexicon_tags_2 },
    { 0x34c6d395, SUC_NE_lexicon_tags_2 },
    { 0x37ecd394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf4d39b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d86d3a0, SUC_NE_lexicon_tags_2 },
    { 0x5d38d3a1, SUC_NE_lexicon_tags_7 },
    { 0x9122d3a1, SUC_NE_lexicon_tags_2 },
    { 0x9b20d3a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b72d3a7, SUC_NE_lexicon_tags_2 },
    { 0xffaad3a8, SUC_NE_lexicon_tags_4 },
    { 0xcb48d3a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd32d3ac, SUC_NE_lexicon_tags_2 },
    { 0x9e6ad3ad, SUC_NE_lexicon_tags_2 },
    { 0x9fd4d3ad, SUC_NE_lexicon_tags_2 },
    { 0xda4ed3ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b00d3b2, SUC_NE_lexicon_tags_2 },
    { 0x890d3b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474cd3b8, SUC_NE_lexicon_tags_2 },
    { 0xb65ed3b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6cd3bc, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x8678d3be, SUC_NE_lexicon_tags_2 },
    { 0xd72cd3bf, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eed3c2, SUC_NE_lexicon_tags_2 },
    { 0xb3bcd3c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf86d3c7, SUC_NE_lexicon_tags_2 },
    { 0x4b72d3c8, SUC_NE_lexicon_tags_2 },
    { 0x1b26d3c9, SUC_NE_lexicon_tags_2 },
    { 0x2ed0d3ca, SUC_NE_lexicon_tags_2 },
    { 0x24b6d3c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ccd3cf, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2296d3d3, SUC_NE_lexicon_tags_3 },
    { 0x646d3d4, SUC_NE_lexicon_tags_2 },
    { 0x861ad3d3, SUC_NE_lexicon_tags_2 },
    { 0xa19cd3d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ded3db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e8d3df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad6d3e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ad3eb, SUC_NE_lexicon_tags_2 },
    { 0x32f0d3eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0cd3ef, SUC_NE_lexicon_tags_12 },
    { 0xd984d3f0, SUC_NE_lexicon_tags_13 },
    { 0x96a8d3f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2d3fd, SUC_NE_lexicon_tags_2 },
    { 0x48bed3fe, SUC_NE_lexicon_tags_2 },
    { 0x149ad3ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6650d401, SUC_NE_lexicon_tags_17 },
    { 0x6ecad402, SUC_NE_lexicon_tags_0 },
    { 0x8132d403, SUC_NE_lexicon_tags_2 },
    { 0x269ed404, SUC_NE_lexicon_tags_2 },
    { 0xaa42d405, SUC_NE_lexicon_tags_3 },
    { 0x1bcd405, SUC_NE_lexicon_tags_2 },
    { 0x4590d401, SUC_NE_lexicon_tags_2 },
    { 0x14e6d408, SUC_NE_lexicon_tags_2 },
    { 0x4a82d407, SUC_NE_lexicon_tags_2 },
    { 0xe688d40a, SUC_NE_lexicon_tags_2 },
    { 0xa266d404, SUC_NE_lexicon_tags_2 },
    { 0x56a4d40c, SUC_NE_lexicon_tags_2 },
    { 0x9fe6d40d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ad410, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a2d416, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d76d418, SUC_NE_lexicon_tags_2 },
    { 0x966ed419, SUC_NE_lexicon_tags_2 },
    { 0xbcb8d418, SUC_NE_lexicon_tags_4 },
    { 0x31ccd418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabcad41d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbff0d41f, SUC_NE_lexicon_tags_2 },
    { 0x6742d420, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5386d422, SUC_NE_lexicon_tags_7 },
    { 0x9ec2d423, SUC_NE_lexicon_tags_2 },
    { 0x9bcd424, SUC_NE_lexicon_tags_2 },
    { 0x248cd425, SUC_NE_lexicon_tags_2 },
    { 0x69ead424, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c0d428, SUC_NE_lexicon_tags_2 },
    { 0x48aed428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24acd42b, SUC_NE_lexicon_tags_2 },
    { 0x602ad42c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bad431, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9096d433, SUC_NE_lexicon_tags_2 },
    { 0x6bed433, SUC_NE_lexicon_tags_2 },
    { 0xa7ded433, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x456cd437, SUC_NE_lexicon_tags_4 },
    { 0xa58d438, SUC_NE_lexicon_tags_2 },
    { 0xc9fad437, SUC_NE_lexicon_tags_2 },
    { 0x3cc2d43a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x516ad43c, SUC_NE_lexicon_tags_2 },
    { 0xef9ad43c, SUC_NE_lexicon_tags_2 },
    { 0x5c62d43c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d6d441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd8d443, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2562d449, SUC_NE_lexicon_tags_1 },
    { 0x4d9ed44a, SUC_NE_lexicon_tags_2 },
    { 0xe2e2d449, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628ad44f, SUC_NE_lexicon_tags_2 },
    { 0xa1ed450, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e24d453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f0d455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b6d459, SUC_NE_lexicon_tags_2 },
    { 0xa906d459, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7cd45f, SUC_NE_lexicon_tags_17 },
    { 0xa1fed460, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5896d462, SUC_NE_lexicon_tags_2 },
    { 0x4030d463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9aed46a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a2d470, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6d2d472, SUC_NE_lexicon_tags_19 },
    { 0x7608d472, SUC_NE_lexicon_tags_2 },
    { 0x19ead474, SUC_NE_lexicon_tags_2 },
    { 0x1284d475, SUC_NE_lexicon_tags_2 },
    { 0x109cd472, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f62d47a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6ed484, SUC_NE_lexicon_tags_2 },
    { 0xfaf2d484, SUC_NE_lexicon_tags_2 },
    { 0xecead486, SUC_NE_lexicon_tags_2 },
    { 0xf18ed487, SUC_NE_lexicon_tags_2 },
    { 0xfac0d486, SUC_NE_lexicon_tags_2 },
    { 0xfdd0d485, SUC_NE_lexicon_tags_2 },
    { 0x3c96d48a, SUC_NE_lexicon_tags_2 },
    { 0xcad2d48b, SUC_NE_lexicon_tags_2 },
    { 0xf148d48b, SUC_NE_lexicon_tags_2 },
    { 0xfeb6d48b, SUC_NE_lexicon_tags_2 },
    { 0x34acd48e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8718d490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf8ed492, SUC_NE_lexicon_tags_2 },
    { 0x2a44d493, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ced498, SUC_NE_lexicon_tags_2 },
    { 0x6df8d499, SUC_NE_lexicon_tags_2 },
    { 0xeda0d49a, SUC_NE_lexicon_tags_2 },
    { 0xf878d499, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x573ad49d, SUC_NE_lexicon_tags_2 },
    { 0xc460d49e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5902d4a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a8d4a2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd186d4a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ccd4a9, SUC_NE_lexicon_tags_2 },
    { 0x8576d4a9, SUC_NE_lexicon_tags_2 },
    { 0x9be6d4ab, SUC_NE_lexicon_tags_2 },
    { 0x41f6d4ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8ed4af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ee4d4b1, SUC_NE_lexicon_tags_1 },
    { 0x5a0ed4b1, SUC_NE_lexicon_tags_2 },
    { 0x8422d4b3, SUC_NE_lexicon_tags_2 },
    { 0x6828d4b4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x54f2d4b6, SUC_NE_lexicon_tags_2 },
    { 0x1f5ed4b7, SUC_NE_lexicon_tags_2 },
    { 0x3376d4b8, SUC_NE_lexicon_tags_5 },
    { 0x83eed4b6, SUC_NE_lexicon_tags_2 },
    { 0x91fed4ba, SUC_NE_lexicon_tags_2 },
    { 0x456cd4b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf672d4be, SUC_NE_lexicon_tags_2 },
    { 0xc012d4bf, SUC_NE_lexicon_tags_2 },
    { 0x13bcd4bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffad4c9, SUC_NE_lexicon_tags_7 },
    { 0x3608d4ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d4d4d2, SUC_NE_lexicon_tags_0 },
    { 0xb1a2d4d3, SUC_NE_lexicon_tags_2 },
    { 0x650ad4d4, SUC_NE_lexicon_tags_2 },
    { 0xa8c4d4d5, SUC_NE_lexicon_tags_2 },
    { 0x78ccd4d6, SUC_NE_lexicon_tags_2 },
    { 0xb78d4d7, SUC_NE_lexicon_tags_2 },
    { 0x1b32d4d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x133ed4db, SUC_NE_lexicon_tags_2 },
    { 0x9392d4db, SUC_NE_lexicon_tags_2 },
    { 0x9544d4dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18b6d4df, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1230d4e1, SUC_NE_lexicon_tags_17 },
    { 0xfea2d4e1, SUC_NE_lexicon_tags_2 },
    { 0x17b0d4e3, SUC_NE_lexicon_tags_2 },
    { 0x2d1ed4e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bcd4eb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd30d4ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x754ad4ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b2d4f4, SUC_NE_lexicon_tags_2 },
    { 0xb1ccd4f5, SUC_NE_lexicon_tags_2 },
    { 0xc766d4f4, SUC_NE_lexicon_tags_2 },
    { 0xc6ded4f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f6d4fc, SUC_NE_lexicon_tags_340 },
    { 0, NULL },
    { 0xf068d4fe, SUC_NE_lexicon_tags_2 },
    { 0x47e8d4ff, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bed504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f8cd506, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a98d50a, SUC_NE_lexicon_tags_2 },
    { 0x516ed50b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa936d50d, SUC_NE_lexicon_tags_2 },
    { 0x8cccd50e, SUC_NE_lexicon_tags_2 },
    { 0xd254d50e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5722d515, SUC_NE_lexicon_tags_2 },
    { 0x88d4d516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cf2d518, SUC_NE_lexicon_tags_23 },
    { 0x6e6cd518, SUC_NE_lexicon_tags_2 },
    { 0x8b1cd518, SUC_NE_lexicon_tags_2 },
    { 0x8420d51b, SUC_NE_lexicon_tags_2 },
    { 0xc440d51b, SUC_NE_lexicon_tags_2 },
    { 0x19d0d51b, SUC_NE_lexicon_tags_1 },
    { 0x7e50d519, SUC_NE_lexicon_tags_2 },
    { 0x20c4d51b, SUC_NE_lexicon_tags_2 },
    { 0x2716d520, SUC_NE_lexicon_tags_2 },
    { 0x53c0d520, SUC_NE_lexicon_tags_4 },
    { 0xd26ed522, SUC_NE_lexicon_tags_2 },
    { 0xdabcd522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x230ed525, SUC_NE_lexicon_tags_2 },
    { 0x6064d525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c6d528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74d6d52a, SUC_NE_lexicon_tags_2 },
    { 0x574ed52a, SUC_NE_lexicon_tags_2 },
    { 0x96c6d52a, SUC_NE_lexicon_tags_2 },
    { 0xd348d52c, SUC_NE_lexicon_tags_2 },
    { 0x53bed52e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ecd535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4ed53b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94ed540, SUC_NE_lexicon_tags_2 },
    { 0x6ad6d541, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xead4d544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa754d547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85ed549, SUC_NE_lexicon_tags_2 },
    { 0xd704d549, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d64d54d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b84d553, SUC_NE_lexicon_tags_2 },
    { 0xdc8cd554, SUC_NE_lexicon_tags_2 },
    { 0x217cd555, SUC_NE_lexicon_tags_2 },
    { 0xf04cd554, SUC_NE_lexicon_tags_2 },
    { 0x6e94d556, SUC_NE_lexicon_tags_2 },
    { 0xef6ad554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ead55c, SUC_NE_lexicon_tags_2 },
    { 0x8a04d55d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8ad55f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd472d562, SUC_NE_lexicon_tags_2 },
    { 0xf696d563, SUC_NE_lexicon_tags_2 },
    { 0x73ccd563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c66d566, SUC_NE_lexicon_tags_2 },
    { 0x5178d567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4354d569, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9c8d56b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdcd56f, SUC_NE_lexicon_tags_2 },
    { 0xfd0ad570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c8d573, SUC_NE_lexicon_tags_2 },
    { 0x1562d573, SUC_NE_lexicon_tags_2 },
    { 0x4750d575, SUC_NE_lexicon_tags_2 },
    { 0xf6e8d576, SUC_NE_lexicon_tags_2 },
    { 0xff92d575, SUC_NE_lexicon_tags_2 },
    { 0xd7aad576, SUC_NE_lexicon_tags_2 },
    { 0xeafed574, SUC_NE_lexicon_tags_2 },
    { 0x7df2d573, SUC_NE_lexicon_tags_2 },
    { 0x29ded57a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d6d582, SUC_NE_lexicon_tags_2 },
    { 0x218cd583, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9824d585, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcf6d587, SUC_NE_lexicon_tags_2 },
    { 0xb722d588, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd74d58b, SUC_NE_lexicon_tags_19 },
    { 0x36aed58b, SUC_NE_lexicon_tags_2 },
    { 0xec86d58d, SUC_NE_lexicon_tags_2 },
    { 0xcba8d58d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bccd590, SUC_NE_lexicon_tags_2 },
    { 0x6474d590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5ed594, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea56d59a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287ad5a0, SUC_NE_lexicon_tags_2 },
    { 0x26c6d5a1, SUC_NE_lexicon_tags_2 },
    { 0xcd2ed5a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x995ed5a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b6d5a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa360d5a9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd618d5ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3494d5af, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06d5b3, SUC_NE_lexicon_tags_2 },
    { 0xe522d5b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x114cd5b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e6d5ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c0d5be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab0d5c0, SUC_NE_lexicon_tags_2 },
    { 0x9598d5c0, SUC_NE_lexicon_tags_2 },
    { 0x1faed5c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3ad5c6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c14d5c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8cd5cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d4d5cf, SUC_NE_lexicon_tags_2 },
    { 0x9d98d5cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68a2d5d2, SUC_NE_lexicon_tags_2 },
    { 0x8ee0d5d3, SUC_NE_lexicon_tags_2 },
    { 0xe64cd5d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc616d5d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58d4d5da, SUC_NE_lexicon_tags_2 },
    { 0xf44d5db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03cd5de, SUC_NE_lexicon_tags_2 },
    { 0xb01cd5df, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a76d5e2, SUC_NE_lexicon_tags_395 },
    { 0xba56d5e3, SUC_NE_lexicon_tags_2 },
    { 0x186d5e3, SUC_NE_lexicon_tags_3 },
    { 0x4e12d5e5, SUC_NE_lexicon_tags_2 },
    { 0xb626d5e5, SUC_NE_lexicon_tags_7 },
    { 0x6a22d5e3, SUC_NE_lexicon_tags_2 },
    { 0x8ce2d5e8, SUC_NE_lexicon_tags_2 },
    { 0x9b76d5e3, SUC_NE_lexicon_tags_2 },
    { 0x4e4ed5ea, SUC_NE_lexicon_tags_2 },
    { 0xf26ed5eb, SUC_NE_lexicon_tags_2 },
    { 0x8a58d5ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3076d5ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa72d5f3, SUC_NE_lexicon_tags_2 },
    { 0x2200d5f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x656ed5f6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8ce6d5f8, SUC_NE_lexicon_tags_2 },
    { 0x6272d5f9, SUC_NE_lexicon_tags_4 },
    { 0x5ad0d5fa, SUC_NE_lexicon_tags_2 },
    { 0xd674d5fb, SUC_NE_lexicon_tags_2 },
    { 0xcc58d5f9, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xdd5ed5fe, SUC_NE_lexicon_tags_4 },
    { 0x69f8d5fe, SUC_NE_lexicon_tags_2 },
    { 0x1feed5fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x417ed602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8370d604, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6d6d606, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7822d609, SUC_NE_lexicon_tags_2 },
    { 0x7b28d609, SUC_NE_lexicon_tags_1 },
    { 0xc860d60a, SUC_NE_lexicon_tags_2 },
    { 0xf5c2d60c, SUC_NE_lexicon_tags_2 },
    { 0xb2f6d60c, SUC_NE_lexicon_tags_2 },
    { 0xd29cd60d, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6d612, SUC_NE_lexicon_tags_2 },
    { 0x8212d613, SUC_NE_lexicon_tags_2 },
    { 0xf06ad613, SUC_NE_lexicon_tags_4 },
    { 0x1648d615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9564d61c, SUC_NE_lexicon_tags_2 },
    { 0x1042d61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aad621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd448d623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab10d626, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c8d62e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf402d632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d0d636, SUC_NE_lexicon_tags_2 },
    { 0xddd4d637, SUC_NE_lexicon_tags_2 },
    { 0x9cb0d638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x693ad63a, SUC_NE_lexicon_tags_2 },
    { 0x552ed63a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8384d63e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4d2d640, SUC_NE_lexicon_tags_4 },
    { 0x402ed641, SUC_NE_lexicon_tags_2 },
    { 0xdd5ed642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa5cd644, SUC_NE_lexicon_tags_2 },
    { 0x5b46d644, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6cd649, SUC_NE_lexicon_tags_57 },
    { 0x731ed64a, SUC_NE_lexicon_tags_2 },
    { 0xc898d64b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb48d64f, SUC_NE_lexicon_tags_1 },
    { 0x7ad4d64f, SUC_NE_lexicon_tags_2 },
    { 0x4412d651, SUC_NE_lexicon_tags_7 },
    { 0x8a8ed652, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9edad654, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc854d659, SUC_NE_lexicon_tags_2 },
    { 0x660cd65a, SUC_NE_lexicon_tags_7 },
    { 0x9a5cd65a, SUC_NE_lexicon_tags_2 },
    { 0x47e6d65c, SUC_NE_lexicon_tags_4 },
    { 0x278ed65b, SUC_NE_lexicon_tags_2 },
    { 0xc486d65e, SUC_NE_lexicon_tags_2 },
    { 0xf89cd65b, SUC_NE_lexicon_tags_2 },
    { 0xab1cd660, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42cd664, SUC_NE_lexicon_tags_2 },
    { 0xb1e6d665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cead669, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63ed66f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45c8d671, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b8d67f, SUC_NE_lexicon_tags_2 },
    { 0x9824d680, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2046d68e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d0d695, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad6d69b, SUC_NE_lexicon_tags_2 },
    { 0x8120d69c, SUC_NE_lexicon_tags_2 },
    { 0x5b6ad69d, SUC_NE_lexicon_tags_2 },
    { 0x26aad69e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2d6a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c2d6ae, SUC_NE_lexicon_tags_24 },
    { 0x6ed6d6af, SUC_NE_lexicon_tags_2 },
    { 0xc936d6b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7482d6b4, SUC_NE_lexicon_tags_2 },
    { 0x79b2d6b5, SUC_NE_lexicon_tags_2 },
    { 0x5862d6b5, SUC_NE_lexicon_tags_2 },
    { 0x6e98d6b7, SUC_NE_lexicon_tags_2 },
    { 0x5ae4d6b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8504d6bb, SUC_NE_lexicon_tags_25 },
    { 0xf5d6d6bc, SUC_NE_lexicon_tags_2 },
    { 0x395ed6bd, SUC_NE_lexicon_tags_2 },
    { 0x608d6bd, SUC_NE_lexicon_tags_18 },
    { 0x94f0d6be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfedcd6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6b4d6c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb826d6c7, SUC_NE_lexicon_tags_2 },
    { 0xd02d6c7, SUC_NE_lexicon_tags_2 },
    { 0x8e72d6c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d1cd6cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb548d6d2, SUC_NE_lexicon_tags_19 },
    { 0xdc9ed6d2, SUC_NE_lexicon_tags_8 },
    { 0xd59ed6d3, SUC_NE_lexicon_tags_2 },
    { 0x1b1ed6d5, SUC_NE_lexicon_tags_2 },
    { 0xdadcd6d4, SUC_NE_lexicon_tags_2 },
    { 0xe2f2d6d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68ad6e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6b4d6e7, SUC_NE_lexicon_tags_2 },
    { 0x3f5ad6e8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xc938d6ea, SUC_NE_lexicon_tags_2 },
    { 0x4cf4d6eb, SUC_NE_lexicon_tags_2 },
    { 0xaa9ad6ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1338d6ef, SUC_NE_lexicon_tags_2 },
    { 0xef6d6f0, SUC_NE_lexicon_tags_36 },
    { 0x9534d6f0, SUC_NE_lexicon_tags_2 },
    { 0x6014d6f2, SUC_NE_lexicon_tags_2 },
    { 0x76c6d6f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8804d6f5, SUC_NE_lexicon_tags_3 },
    { 0xb136d6f6, SUC_NE_lexicon_tags_2 },
    { 0x3142d6f6, SUC_NE_lexicon_tags_2 },
    { 0xf5ccd6f8, SUC_NE_lexicon_tags_2 },
    { 0x6578d6f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29c2d6fb, SUC_NE_lexicon_tags_2 },
    { 0x33c4d6fc, SUC_NE_lexicon_tags_2 },
    { 0x8d82d6fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3cd701, SUC_NE_lexicon_tags_2 },
    { 0x3584d702, SUC_NE_lexicon_tags_2 },
    { 0xda2d701, SUC_NE_lexicon_tags_2 },
    { 0x46d6d704, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4314d706, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49aad70a, SUC_NE_lexicon_tags_2 },
    { 0x8f50d70b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f12d70f, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ced713, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3cd716, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x21c0d718, SUC_NE_lexicon_tags_2 },
    { 0x36e8d719, SUC_NE_lexicon_tags_2 },
    { 0x417ed718, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe71ad71c, SUC_NE_lexicon_tags_2 },
    { 0x514d71c, SUC_NE_lexicon_tags_2 },
    { 0xcbaad71e, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd2d721, SUC_NE_lexicon_tags_2 },
    { 0x9dbed722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc454d726, SUC_NE_lexicon_tags_2 },
    { 0xfd0ad726, SUC_NE_lexicon_tags_3 },
    { 0xc60ed728, SUC_NE_lexicon_tags_4 },
    { 0xc502d727, SUC_NE_lexicon_tags_2 },
    { 0xe4f0d72a, SUC_NE_lexicon_tags_2 },
    { 0xe52d72a, SUC_NE_lexicon_tags_2 },
    { 0x204ed72c, SUC_NE_lexicon_tags_2 },
    { 0xa14cd726, SUC_NE_lexicon_tags_2 },
    { 0x9990d72a, SUC_NE_lexicon_tags_2 },
    { 0x41d6d72b, SUC_NE_lexicon_tags_2 },
    { 0x4c58d730, SUC_NE_lexicon_tags_2 },
    { 0x49ced731, SUC_NE_lexicon_tags_2 },
    { 0x429ad728, SUC_NE_lexicon_tags_2 },
    { 0x41f6d733, SUC_NE_lexicon_tags_7 },
    { 0x3becd733, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e74d737, SUC_NE_lexicon_tags_2 },
    { 0x99b6d737, SUC_NE_lexicon_tags_2 },
    { 0x11e6d739, SUC_NE_lexicon_tags_2 },
    { 0x87ccd73a, SUC_NE_lexicon_tags_2 },
    { 0x86b2d73a, SUC_NE_lexicon_tags_2 },
    { 0x3d24d73c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x360ad73f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145cd743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5ded745, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x2d8ed747, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x45d8d749, SUC_NE_lexicon_tags_2 },
    { 0x1378d74a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45ecd74c, SUC_NE_lexicon_tags_2 },
    { 0x2532d74c, SUC_NE_lexicon_tags_193 },
    { 0x55fed74d, SUC_NE_lexicon_tags_2 },
    { 0x5690d74f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf598d755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f4cd757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x579cd759, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e6d75c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3910d763, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbcd767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d769, SUC_NE_lexicon_tags_2 },
    { 0xaf76d769, SUC_NE_lexicon_tags_2 },
    { 0x9c54d76b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c18d76e, SUC_NE_lexicon_tags_2 },
    { 0xccd2d76f, SUC_NE_lexicon_tags_2 },
    { 0x411cd76f, SUC_NE_lexicon_tags_2 },
    { 0x1416d771, SUC_NE_lexicon_tags_2 },
    { 0x978ed772, SUC_NE_lexicon_tags_2 },
    { 0xe716d773, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6676d776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e08d778, SUC_NE_lexicon_tags_2 },
    { 0x5298d779, SUC_NE_lexicon_tags_2 },
    { 0x44d778, SUC_NE_lexicon_tags_2 },
    { 0x6870d77b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4d77f, SUC_NE_lexicon_tags_2 },
    { 0xd522d77f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad40d784, SUC_NE_lexicon_tags_2 },
    { 0xba24d784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df2d788, SUC_NE_lexicon_tags_2 },
    { 0xc326d788, SUC_NE_lexicon_tags_2 },
    { 0xe06ed78a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ad78f, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720d794, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9110d796, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x894ad798, SUC_NE_lexicon_tags_2 },
    { 0xd00ad799, SUC_NE_lexicon_tags_2 },
    { 0xc25ed799, SUC_NE_lexicon_tags_2 },
    { 0x8510d79b, SUC_NE_lexicon_tags_2 },
    { 0x82d2d79c, SUC_NE_lexicon_tags_2 },
    { 0x71ed79c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb62cd79f, SUC_NE_lexicon_tags_2 },
    { 0xfccad79f, SUC_NE_lexicon_tags_2 },
    { 0xb41cd79f, SUC_NE_lexicon_tags_2 },
    { 0xce70d7a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5774d7a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f18d7a8, SUC_NE_lexicon_tags_2 },
    { 0xba1ad7a8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218d7ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f4d7b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda8d7b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f2d7bc, SUC_NE_lexicon_tags_2 },
    { 0x50c4d7bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc402d7bf, SUC_NE_lexicon_tags_2 },
    { 0xac7ed7c0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c4cd7c2, SUC_NE_lexicon_tags_2 },
    { 0x4028d7c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e58d7c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8822d7c8, SUC_NE_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c76d7cb, SUC_NE_lexicon_tags_2 },
    { 0xaa00d7cb, SUC_NE_lexicon_tags_1 },
    { 0xdc0ad7cb, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x1982d7cf, SUC_NE_lexicon_tags_2 },
    { 0x4db6d7d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134ad7d4, SUC_NE_lexicon_tags_2 },
    { 0xe454d7d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe86d7d8, SUC_NE_lexicon_tags_2 },
    { 0xbabed7d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93ed7dd, SUC_NE_lexicon_tags_2 },
    { 0xba3cd7de, SUC_NE_lexicon_tags_299 },
    { 0xaa14d7df, SUC_NE_lexicon_tags_2 },
    { 0x4234d7e0, SUC_NE_lexicon_tags_2 },
    { 0xe95ad7e0, SUC_NE_lexicon_tags_9 },
    { 0x705cd7df, SUC_NE_lexicon_tags_2 },
    { 0xa558d7e3, SUC_NE_lexicon_tags_2 },
    { 0x6f64d7e4, SUC_NE_lexicon_tags_2 },
    { 0x728cd7e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bacd7eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52a2d7ed, SUC_NE_lexicon_tags_2 },
    { 0xac72d7ed, SUC_NE_lexicon_tags_2 },
    { 0xb80cd7ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50a4d7f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd56d7f3, SUC_NE_lexicon_tags_2 },
    { 0xae62d7f4, SUC_NE_lexicon_tags_2 },
    { 0x79fcd7f3, SUC_NE_lexicon_tags_2 },
    { 0x9c44d7f3, SUC_NE_lexicon_tags_2 },
    { 0x194d7f7, SUC_NE_lexicon_tags_2 },
    { 0xa34ed7f8, SUC_NE_lexicon_tags_2 },
    { 0xae8cd7f8, SUC_NE_lexicon_tags_2 },
    { 0x6daad7f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd8d801, SUC_NE_lexicon_tags_2 },
    { 0x9e7ed802, SUC_NE_lexicon_tags_2 },
    { 0x3fc4d803, SUC_NE_lexicon_tags_2 },
    { 0xcdf0d803, SUC_NE_lexicon_tags_2 },
    { 0xaf7ed803, SUC_NE_lexicon_tags_2 },
    { 0x7862d806, SUC_NE_lexicon_tags_2 },
    { 0x614ad807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89c6d809, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f0d80d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c6d811, SUC_NE_lexicon_tags_5 },
    { 0x30d812, SUC_NE_lexicon_tags_2 },
    { 0xe4ecd811, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f3cd815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c9ad817, SUC_NE_lexicon_tags_2 },
    { 0x623ed817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaaad81a, SUC_NE_lexicon_tags_2 },
    { 0x1dc2d81b, SUC_NE_lexicon_tags_2 },
    { 0xacc4d81c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe90d81e, SUC_NE_lexicon_tags_2 },
    { 0x6cb6d81e, SUC_NE_lexicon_tags_2 },
    { 0xc7a2d81f, SUC_NE_lexicon_tags_2 },
    { 0x8a72d821, SUC_NE_lexicon_tags_2 },
    { 0x1edad822, SUC_NE_lexicon_tags_2 },
    { 0x3260d81e, SUC_NE_lexicon_tags_2 },
    { 0xcf5ad822, SUC_NE_lexicon_tags_2 },
    { 0x2482d825, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8cd82c, SUC_NE_lexicon_tags_2 },
    { 0x975cd82d, SUC_NE_lexicon_tags_2 },
    { 0xb7fcd82d, SUC_NE_lexicon_tags_2 },
    { 0x3dbad82f, SUC_NE_lexicon_tags_2 },
    { 0x16b8d82d, SUC_NE_lexicon_tags_2 },
    { 0x77a6d82d, SUC_NE_lexicon_tags_2 },
    { 0x46b4d830, SUC_NE_lexicon_tags_2 },
    { 0x4f0ed833, SUC_NE_lexicon_tags_1 },
    { 0x307cd834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8090d83c, SUC_NE_lexicon_tags_2 },
    { 0xdf94d83d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x414d83f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ad842, SUC_NE_lexicon_tags_2 },
    { 0xdb0cd842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6156d845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd092d849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbcd84c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ed850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504ed856, SUC_NE_lexicon_tags_4 },
    { 0xf5fcd857, SUC_NE_lexicon_tags_2 },
    { 0x33c0d858, SUC_NE_lexicon_tags_2 },
    { 0x2994d856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdb8d85b, SUC_NE_lexicon_tags_7 },
    { 0xb0ead85c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd508d85e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc80d860, SUC_NE_lexicon_tags_2 },
    { 0x7da4d861, SUC_NE_lexicon_tags_2 },
    { 0x8a8ad862, SUC_NE_lexicon_tags_2 },
    { 0x27ad862, SUC_NE_lexicon_tags_2 },
    { 0x44b4d861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf590d866, SUC_NE_lexicon_tags_2 },
    { 0xed4ad866, SUC_NE_lexicon_tags_109 },
    { 0x470cd868, SUC_NE_lexicon_tags_2 },
    { 0xbf6ad869, SUC_NE_lexicon_tags_2 },
    { 0xc9acd869, SUC_NE_lexicon_tags_2 },
    { 0xfc1ad86b, SUC_NE_lexicon_tags_2 },
    { 0x7110d86c, SUC_NE_lexicon_tags_2 },
    { 0xa3e8d868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6cd871, SUC_NE_lexicon_tags_2 },
    { 0xde4ad872, SUC_NE_lexicon_tags_2 },
    { 0xc1dcd873, SUC_NE_lexicon_tags_2 },
    { 0x41a6d872, SUC_NE_lexicon_tags_2 },
    { 0xd2b4d875, SUC_NE_lexicon_tags_2 },
    { 0x4f7cd876, SUC_NE_lexicon_tags_2 },
    { 0x5bbad876, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66c8d879, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eed87e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2e6d880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2d885, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd72d887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8150d889, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1786d893, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9388d89f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2d8a2, SUC_NE_lexicon_tags_4 },
    { 0x500cd8a3, SUC_NE_lexicon_tags_2 },
    { 0xc102d8a2, SUC_NE_lexicon_tags_2 },
    { 0x936ad8a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ead8a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4484d8aa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb362d8af, SUC_NE_lexicon_tags_2 },
    { 0xa392d8b0, SUC_NE_lexicon_tags_2 },
    { 0xaef8d8b1, SUC_NE_lexicon_tags_2 },
    { 0xbdd8d8b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b4ed8b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa07ed8b6, SUC_NE_lexicon_tags_7 },
    { 0x251ed8b7, SUC_NE_lexicon_tags_2 },
    { 0xfdbad8b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe69ed8ba, SUC_NE_lexicon_tags_2 },
    { 0x7cf6d8ba, SUC_NE_lexicon_tags_2 },
    { 0xdb0cd8bb, SUC_NE_lexicon_tags_2 },
    { 0xf470d8bc, SUC_NE_lexicon_tags_2 },
    { 0x4c2cd8bb, SUC_NE_lexicon_tags_4 },
    { 0x41d2d8bd, SUC_NE_lexicon_tags_2 },
    { 0x5926d8c0, SUC_NE_lexicon_tags_2 },
    { 0x956d8c1, SUC_NE_lexicon_tags_4 },
    { 0xc4acd8c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0cd8c8, SUC_NE_lexicon_tags_2 },
    { 0x4180d8c8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4d8cc, SUC_NE_lexicon_tags_214 },
    { 0, NULL },
    { 0x6db4d8ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca76d8d0, SUC_NE_lexicon_tags_2 },
    { 0x86ed8d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a6d8d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54aed8d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ac2d8da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fa0d8dc, SUC_NE_lexicon_tags_2 },
    { 0x8f62d8dd, SUC_NE_lexicon_tags_2 },
    { 0x627cd8dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d4d8e1, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2686d8e6, SUC_NE_lexicon_tags_4 },
    { 0xe1b8d8e7, SUC_NE_lexicon_tags_2 },
    { 0x524d8e8, SUC_NE_lexicon_tags_2 },
    { 0xd1e4d8e8, SUC_NE_lexicon_tags_2 },
    { 0x2d78d8e6, SUC_NE_lexicon_tags_2 },
    { 0x2c1ad8eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5ed8ee, SUC_NE_lexicon_tags_2 },
    { 0x8470d8ee, SUC_NE_lexicon_tags_2 },
    { 0x6fe2d8ef, SUC_NE_lexicon_tags_104 },
    { 0xd448d8f1, SUC_NE_lexicon_tags_2 },
    { 0x595ad8f0, SUC_NE_lexicon_tags_7 },
    { 0x4374d8f3, SUC_NE_lexicon_tags_2 },
    { 0x8356d8f4, SUC_NE_lexicon_tags_2 },
    { 0x44c4d8ee, SUC_NE_lexicon_tags_2 },
    { 0x4b9cd8f6, SUC_NE_lexicon_tags_2 },
    { 0xa760d8f0, SUC_NE_lexicon_tags_2 },
    { 0x5386d8f8, SUC_NE_lexicon_tags_2 },
    { 0x7ec4d8f9, SUC_NE_lexicon_tags_2 },
    { 0x28f2d8f5, SUC_NE_lexicon_tags_2 },
    { 0x4630d8fb, SUC_NE_lexicon_tags_1 },
    { 0x52fad8fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe370d8ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe14cd901, SUC_NE_lexicon_tags_4 },
    { 0x378cd902, SUC_NE_lexicon_tags_2 },
    { 0x85ed903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc492d906, SUC_NE_lexicon_tags_4 },
    { 0x866d907, SUC_NE_lexicon_tags_2 },
    { 0xdf5ad907, SUC_NE_lexicon_tags_69 },
    { 0x5fdad909, SUC_NE_lexicon_tags_2 },
    { 0x8c18d90a, SUC_NE_lexicon_tags_157 },
    { 0xbb32d90a, SUC_NE_lexicon_tags_4 },
    { 0x55b2d90c, SUC_NE_lexicon_tags_2 },
    { 0x37c8d90a, SUC_NE_lexicon_tags_2 },
    { 0x35f0d90e, SUC_NE_lexicon_tags_2 },
    { 0xbc7ed906, SUC_NE_lexicon_tags_2 },
    { 0xec30d910, SUC_NE_lexicon_tags_2 },
    { 0x727ad910, SUC_NE_lexicon_tags_2 },
    { 0xb63ed907, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b2d918, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4212d91a, SUC_NE_lexicon_tags_2 },
    { 0x9ab8d91a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f2d920, SUC_NE_lexicon_tags_2 },
    { 0xc37ed920, SUC_NE_lexicon_tags_2 },
    { 0xfdced922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe044d925, SUC_NE_lexicon_tags_2 },
    { 0xa672d926, SUC_NE_lexicon_tags_2 },
    { 0x853ad926, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8d92a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd20d92e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaed932, SUC_NE_lexicon_tags_3 },
    { 0xfc70d932, SUC_NE_lexicon_tags_2 },
    { 0xa99ad934, SUC_NE_lexicon_tags_2 },
    { 0xff84d934, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00d938, SUC_NE_lexicon_tags_2 },
    { 0xfd52d938, SUC_NE_lexicon_tags_2 },
    { 0x2b44d939, SUC_NE_lexicon_tags_32 },
    { 0x31ead93b, SUC_NE_lexicon_tags_2 },
    { 0x9b3cd93c, SUC_NE_lexicon_tags_2 },
    { 0x3a7ed93c, SUC_NE_lexicon_tags_11 },
    { 0x4974d938, SUC_NE_lexicon_tags_2 },
    { 0xaf2ed93f, SUC_NE_lexicon_tags_2 },
    { 0x4dacd93e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c2d945, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3582d947, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ce8d949, SUC_NE_lexicon_tags_2 },
    { 0x7de2d949, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8020d94e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x339ad950, SUC_NE_lexicon_tags_2 },
    { 0xf008d951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c80d958, SUC_NE_lexicon_tags_2 },
    { 0xd902d959, SUC_NE_lexicon_tags_2 },
    { 0xdf80d959, SUC_NE_lexicon_tags_2 },
    { 0xf6dcd95b, SUC_NE_lexicon_tags_2 },
    { 0xff70d95c, SUC_NE_lexicon_tags_2 },
    { 0x214ed95a, SUC_NE_lexicon_tags_2 },
    { 0xd35ed95a, SUC_NE_lexicon_tags_2 },
    { 0x5914d95f, SUC_NE_lexicon_tags_2 },
    { 0xb7fad95f, SUC_NE_lexicon_tags_2 },
    { 0xa888d960, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x784cd963, SUC_NE_lexicon_tags_2 },
    { 0x4012d964, SUC_NE_lexicon_tags_2 },
    { 0x876ed964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4aed967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4d96c, SUC_NE_lexicon_tags_2 },
    { 0xe86ed96c, SUC_NE_lexicon_tags_378 },
    { 0x42ecd96e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x22e8d970, SUC_NE_lexicon_tags_2 },
    { 0xad58d971, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d64d973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e4ed975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe60d977, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4584d979, SUC_NE_lexicon_tags_2 },
    { 0xecb8d97a, SUC_NE_lexicon_tags_2 },
    { 0x357cd97b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf520d97d, SUC_NE_lexicon_tags_18 },
    { 0x1cb6d97d, SUC_NE_lexicon_tags_2 },
    { 0xe134d97f, SUC_NE_lexicon_tags_3 },
    { 0xe9f8d97f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d982, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa172d987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea70d98c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e0d98f, SUC_NE_lexicon_tags_4 },
    { 0x4ae6d98f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4742d994, SUC_NE_lexicon_tags_2 },
    { 0x4c7ad995, SUC_NE_lexicon_tags_2 },
    { 0x8ca2d994, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afad99a, SUC_NE_lexicon_tags_2 },
    { 0xac92d99b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62aad99d, SUC_NE_lexicon_tags_2 },
    { 0x4166d99e, SUC_NE_lexicon_tags_2 },
    { 0x3880d99e, SUC_NE_lexicon_tags_4 },
    { 0x964ed9a0, SUC_NE_lexicon_tags_2 },
    { 0xd824d99d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6304d9a8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x7626d9aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2cd9af, SUC_NE_lexicon_tags_17 },
    { 0x8baed9b0, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ad9b4, SUC_NE_lexicon_tags_4 },
    { 0x4b20d9b5, SUC_NE_lexicon_tags_2 },
    { 0x7a3ad9b6, SUC_NE_lexicon_tags_2 },
    { 0x8056d9b6, SUC_NE_lexicon_tags_2 },
    { 0x32c2d9b8, SUC_NE_lexicon_tags_7 },
    { 0xb606d9b9, SUC_NE_lexicon_tags_2 },
    { 0xae3ad9b6, SUC_NE_lexicon_tags_2 },
    { 0xdc02d9ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd990d9bd, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa0d9c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84cd9cb, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x3706d9cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901ed9d1, SUC_NE_lexicon_tags_2 },
    { 0x6d58d9d2, SUC_NE_lexicon_tags_2 },
    { 0x81a4d9d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e8d9d5, SUC_NE_lexicon_tags_2 },
    { 0xca48d9d6, SUC_NE_lexicon_tags_2 },
    { 0x244cd9d7, SUC_NE_lexicon_tags_2 },
    { 0x720ad9d7, SUC_NE_lexicon_tags_2 },
    { 0x64a2d9d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaccd9dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3272d9df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x205cd9e2, SUC_NE_lexicon_tags_2 },
    { 0xab04d9e2, SUC_NE_lexicon_tags_2 },
    { 0x92f6d9e3, SUC_NE_lexicon_tags_2 },
    { 0xf46ed9e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3aed9e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37ad9f0, SUC_NE_lexicon_tags_2 },
    { 0x9e12d9f0, SUC_NE_lexicon_tags_2 },
    { 0xfae0d9f2, SUC_NE_lexicon_tags_2 },
    { 0x834d9f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d8d9f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ac8d9f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52d0d9fa, SUC_NE_lexicon_tags_7 },
    { 0x6c6d9fa, SUC_NE_lexicon_tags_4 },
    { 0x2ecd9fc, SUC_NE_lexicon_tags_2 },
    { 0xeb30d9fa, SUC_NE_lexicon_tags_2 },
    { 0x274d9fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd8da02, SUC_NE_lexicon_tags_2 },
    { 0x55c0da02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6eda06, SUC_NE_lexicon_tags_2 },
    { 0xd506da06, SUC_NE_lexicon_tags_3 },
    { 0x57d4da08, SUC_NE_lexicon_tags_2 },
    { 0xaa8da08, SUC_NE_lexicon_tags_2 },
    { 0xfc3ada08, SUC_NE_lexicon_tags_2 },
    { 0xfe38da0b, SUC_NE_lexicon_tags_2 },
    { 0x52e2da08, SUC_NE_lexicon_tags_4 },
    { 0xee0cda0d, SUC_NE_lexicon_tags_2 },
    { 0x59ada0e, SUC_NE_lexicon_tags_4 },
    { 0xf23ada0f, SUC_NE_lexicon_tags_2 },
    { 0x6832da0d, SUC_NE_lexicon_tags_67 },
    { 0xb48cda11, SUC_NE_lexicon_tags_2 },
    { 0xed06da0d, SUC_NE_lexicon_tags_2 },
    { 0x6902da13, SUC_NE_lexicon_tags_2 },
    { 0x276da14, SUC_NE_lexicon_tags_2 },
    { 0x370cda15, SUC_NE_lexicon_tags_2 },
    { 0x53eda16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf214da19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x210ada1b, SUC_NE_lexicon_tags_2 },
    { 0x57f6da1c, SUC_NE_lexicon_tags_2 },
    { 0xaaacda1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b96da22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd6da25, SUC_NE_lexicon_tags_2 },
    { 0xffe2da26, SUC_NE_lexicon_tags_2 },
    { 0xa408da27, SUC_NE_lexicon_tags_2 },
    { 0x29deda28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d30da2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6928da2e, SUC_NE_lexicon_tags_2 },
    { 0xc4beda2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14bada31, SUC_NE_lexicon_tags_2 },
    { 0xf312da31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd20ada34, SUC_NE_lexicon_tags_2 },
    { 0x73aeda35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ada3a, SUC_NE_lexicon_tags_2 },
    { 0x1e20da3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf342da3e, SUC_NE_lexicon_tags_2 },
    { 0xba86da3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465eda43, SUC_NE_lexicon_tags_2 },
    { 0x712ada44, SUC_NE_lexicon_tags_2 },
    { 0x5860da45, SUC_NE_lexicon_tags_2 },
    { 0xb954da46, SUC_NE_lexicon_tags_2 },
    { 0x96aeda43, SUC_NE_lexicon_tags_2 },
    { 0xb222da44, SUC_NE_lexicon_tags_1 },
    { 0x4024da49, SUC_NE_lexicon_tags_4 },
    { 0x9a86da4a, SUC_NE_lexicon_tags_2 },
    { 0xb2f8da4b, SUC_NE_lexicon_tags_2 },
    { 0xef68da4b, SUC_NE_lexicon_tags_2 },
    { 0xc156da4d, SUC_NE_lexicon_tags_2 },
    { 0xd456da4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e2da53, SUC_NE_lexicon_tags_2 },
    { 0x4526da54, SUC_NE_lexicon_tags_2 },
    { 0x42bcda54, SUC_NE_lexicon_tags_2 },
    { 0x1684da56, SUC_NE_lexicon_tags_4 },
    { 0x7ffada53, SUC_NE_lexicon_tags_2 },
    { 0x5e36da55, SUC_NE_lexicon_tags_2 },
    { 0x27bada59, SUC_NE_lexicon_tags_2 },
    { 0xb74eda59, SUC_NE_lexicon_tags_2 },
    { 0xdef4da5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd732da62, SUC_NE_lexicon_tags_246 },
    { 0x695eda63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf06da66, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ada6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7cada6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148cda74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15ada77, SUC_NE_lexicon_tags_2 },
    { 0x843cda78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde90da7e, SUC_NE_lexicon_tags_2 },
    { 0x68bada7f, SUC_NE_lexicon_tags_2 },
    { 0x473ada7f, SUC_NE_lexicon_tags_2 },
    { 0xe29eda81, SUC_NE_lexicon_tags_2 },
    { 0x6892da81, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x659eda85, SUC_NE_lexicon_tags_2 },
    { 0x309cda85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebeda8c, SUC_NE_lexicon_tags_2 },
    { 0x69a0da8c, SUC_NE_lexicon_tags_18 },
    { 0x21b0da8e, SUC_NE_lexicon_tags_2 },
    { 0x2d78da8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6792da9c, SUC_NE_lexicon_tags_2 },
    { 0x3484da9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44daa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3674daa6, SUC_NE_lexicon_tags_2 },
    { 0x135cdaa7, SUC_NE_lexicon_tags_18 },
    { 0x7128daa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4f8daaa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b2daae, SUC_NE_lexicon_tags_2 },
    { 0x2796daaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe720dab5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fadaba, SUC_NE_lexicon_tags_2 },
    { 0x8ea0dabb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe4dac2, SUC_NE_lexicon_tags_271 },
    { 0x2452dac3, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xd32adac5, SUC_NE_lexicon_tags_2 },
    { 0x485edac6, SUC_NE_lexicon_tags_5 },
    { 0x2f70dac7, SUC_NE_lexicon_tags_2 },
    { 0xb1f6dac8, SUC_NE_lexicon_tags_2 },
    { 0x296dac9, SUC_NE_lexicon_tags_2 },
    { 0xca44daca, SUC_NE_lexicon_tags_10 },
    { 0x60f8daca, SUC_NE_lexicon_tags_0 },
    { 0x44dacb, SUC_NE_lexicon_tags_2 },
    { 0xd642dacd, SUC_NE_lexicon_tags_2 },
    { 0x9d5cdace, SUC_NE_lexicon_tags_2 },
    { 0xa3a8dacf, SUC_NE_lexicon_tags_2 },
    { 0xd31adacf, SUC_NE_lexicon_tags_2 },
    { 0x5f2daca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x154edad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55cdae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d6dae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15e6dae7, SUC_NE_lexicon_tags_2 },
    { 0x3012dae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b84daea, SUC_NE_lexicon_tags_4 },
    { 0x6afedaeb, SUC_NE_lexicon_tags_4 },
    { 0x52c4daec, SUC_NE_lexicon_tags_2 },
    { 0x8946daed, SUC_NE_lexicon_tags_2 },
    { 0x4c66daee, SUC_NE_lexicon_tags_2 },
    { 0x263adaef, SUC_NE_lexicon_tags_2 },
    { 0xfb4daee, SUC_NE_lexicon_tags_2 },
    { 0xd122daf1, SUC_NE_lexicon_tags_2 },
    { 0xe6a0daea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5422daf6, SUC_NE_lexicon_tags_2 },
    { 0x5e9adaf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc0dafa, SUC_NE_lexicon_tags_2 },
    { 0x2570dafb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6b8dafd, SUC_NE_lexicon_tags_13 },
    { 0xf7cedafd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4deadb03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a4db06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26d4db08, SUC_NE_lexicon_tags_2 },
    { 0xf12cdb09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9660db0c, SUC_NE_lexicon_tags_2 },
    { 0xb964db0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad88db0f, SUC_NE_lexicon_tags_2 },
    { 0x1bb8db10, SUC_NE_lexicon_tags_2 },
    { 0x287cdb11, SUC_NE_lexicon_tags_4 },
    { 0x52a4db10, SUC_NE_lexicon_tags_2 },
    { 0x194adb13, SUC_NE_lexicon_tags_7 },
    { 0xf7cedb0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b4db17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63a4db19, SUC_NE_lexicon_tags_2 },
    { 0x771adb19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d26db1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaedb27, SUC_NE_lexicon_tags_2 },
    { 0xace8db27, SUC_NE_lexicon_tags_2 },
    { 0x1d24db27, SUC_NE_lexicon_tags_2 },
    { 0xd3d8db2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31eedb2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2938db2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaab2db30, SUC_NE_lexicon_tags_2 },
    { 0x4696db31, SUC_NE_lexicon_tags_2 },
    { 0x808adb31, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb54db34, SUC_NE_lexicon_tags_2 },
    { 0x3c22db34, SUC_NE_lexicon_tags_2 },
    { 0xf860db34, SUC_NE_lexicon_tags_2 },
    { 0xc512db37, SUC_NE_lexicon_tags_2 },
    { 0x1bcdb38, SUC_NE_lexicon_tags_24 },
    { 0xbeb0db38, SUC_NE_lexicon_tags_2 },
    { 0x748cdb38, SUC_NE_lexicon_tags_2 },
    { 0x772db3b, SUC_NE_lexicon_tags_2 },
    { 0xf782db3c, SUC_NE_lexicon_tags_5 },
    { 0x50cedb38, SUC_NE_lexicon_tags_2 },
    { 0xc312db3e, SUC_NE_lexicon_tags_7 },
    { 0x220adb3e, SUC_NE_lexicon_tags_2 },
    { 0x7a1edb3e, SUC_NE_lexicon_tags_2 },
    { 0xdb2edb3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a30db44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33edb4b, SUC_NE_lexicon_tags_127 },
    { 0x2416db4c, SUC_NE_lexicon_tags_2 },
    { 0xc2aedb4d, SUC_NE_lexicon_tags_7 },
    { 0x4922db4e, SUC_NE_lexicon_tags_2 },
    { 0xf10edb4e, SUC_NE_lexicon_tags_2 },
    { 0x5192db4d, SUC_NE_lexicon_tags_58 },
    { 0x14d4db51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a2db53, SUC_NE_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x8290db56, SUC_NE_lexicon_tags_2 },
    { 0x297edb56, SUC_NE_lexicon_tags_2 },
    { 0xb0a8db57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a16db5a, SUC_NE_lexicon_tags_2 },
    { 0xd048db5b, SUC_NE_lexicon_tags_2 },
    { 0xf6a8db5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8256db5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e10db61, SUC_NE_lexicon_tags_2 },
    { 0xe1f6db62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b6cdb64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e36db6a, SUC_NE_lexicon_tags_2 },
    { 0x192cdb6a, SUC_NE_lexicon_tags_2 },
    { 0xe33cdb6b, SUC_NE_lexicon_tags_2 },
    { 0x8594db6a, SUC_NE_lexicon_tags_2 },
    { 0x39eadb6a, SUC_NE_lexicon_tags_2 },
    { 0xeb58db6f, SUC_NE_lexicon_tags_2 },
    { 0xcafcdb70, SUC_NE_lexicon_tags_2 },
    { 0x2396db71, SUC_NE_lexicon_tags_2 },
    { 0x751edb72, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0x98b0db74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0adb76, SUC_NE_lexicon_tags_2 },
    { 0x5ededb77, SUC_NE_lexicon_tags_2 },
    { 0x8e50db78, SUC_NE_lexicon_tags_5 },
    { 0xc31adb79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79edb7e, SUC_NE_lexicon_tags_2 },
    { 0x5cfedb7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217cdb83, SUC_NE_lexicon_tags_2 },
    { 0xa2d6db83, SUC_NE_lexicon_tags_2 },
    { 0xa2a6db85, SUC_NE_lexicon_tags_2 },
    { 0x60e8db85, SUC_NE_lexicon_tags_2 },
    { 0x8010db87, SUC_NE_lexicon_tags_2 },
    { 0x74b8db88, SUC_NE_lexicon_tags_2 },
    { 0x4ee0db84, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f90db8c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dcdb8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a70db93, SUC_NE_lexicon_tags_2 },
    { 0x9486db94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3398db9b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44dcdb9e, SUC_NE_lexicon_tags_2 },
    { 0x26d0db9e, SUC_NE_lexicon_tags_2 },
    { 0x87a2db9f, SUC_NE_lexicon_tags_2 },
    { 0xfa76db9f, SUC_NE_lexicon_tags_2 },
    { 0xf89edba2, SUC_NE_lexicon_tags_7 },
    { 0x5d34dba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cedba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e82dbaf, SUC_NE_lexicon_tags_2 },
    { 0xa9f8dbaf, SUC_NE_lexicon_tags_7 },
    { 0x20ccdbaf, SUC_NE_lexicon_tags_2 },
    { 0xaf0edbb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x542edbb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4adbba, SUC_NE_lexicon_tags_2 },
    { 0x49eadbbb, SUC_NE_lexicon_tags_2 },
    { 0xf0ecdbbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdda6dbbe, SUC_NE_lexicon_tags_2 },
    { 0x8e22dbbf, SUC_NE_lexicon_tags_2 },
    { 0x3c84dbc0, SUC_NE_lexicon_tags_4 },
    { 0xb974dbc0, SUC_NE_lexicon_tags_2 },
    { 0xff30dbc2, SUC_NE_lexicon_tags_2 },
    { 0x4a62dbc1, SUC_NE_lexicon_tags_2 },
    { 0x23badbc4, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c2dbca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18dbcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee24dbcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2308dbd2, SUC_NE_lexicon_tags_2 },
    { 0xbf6edbd2, SUC_NE_lexicon_tags_2 },
    { 0x9e78dbd4, SUC_NE_lexicon_tags_2 },
    { 0x7596dbd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14f4dbd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x264cdbd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f6dbdc, SUC_NE_lexicon_tags_2 },
    { 0x8766dbdc, SUC_NE_lexicon_tags_2 },
    { 0xcd5adbde, SUC_NE_lexicon_tags_1 },
    { 0x896dbdf, SUC_NE_lexicon_tags_2 },
    { 0x373edbde, SUC_NE_lexicon_tags_2 },
    { 0xcdaedbe1, SUC_NE_lexicon_tags_2 },
    { 0xaeeedbe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa2dbe6, SUC_NE_lexicon_tags_2 },
    { 0x4092dbe7, SUC_NE_lexicon_tags_2 },
    { 0xe208dbe8, SUC_NE_lexicon_tags_2 },
    { 0x1f8edbe8, SUC_NE_lexicon_tags_2 },
    { 0x25d2dbea, SUC_NE_lexicon_tags_2 },
    { 0xb032dbe6, SUC_NE_lexicon_tags_2 },
    { 0x13c6dbe7, SUC_NE_lexicon_tags_2 },
    { 0x1000dbed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeadbf0, SUC_NE_lexicon_tags_2 },
    { 0x395edbf1, SUC_NE_lexicon_tags_2 },
    { 0xb0f4dbf2, SUC_NE_lexicon_tags_39 },
    { 0x9f62dbf2, SUC_NE_lexicon_tags_3 },
    { 0x7f52dbf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43a8dbf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb9edbf8, SUC_NE_lexicon_tags_3 },
    { 0x4aeedbf8, SUC_NE_lexicon_tags_2 },
    { 0x4946dbf9, SUC_NE_lexicon_tags_2 },
    { 0x4b74dbf8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6adbff, SUC_NE_lexicon_tags_2 },
    { 0x6a56dc00, SUC_NE_lexicon_tags_2 },
    { 0xa9a0dc01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273cdc08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceedc0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d4dc12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe922dc14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1738dc22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244cdc28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4972dc2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4308dc32, SUC_NE_lexicon_tags_2 },
    { 0x5c10dc33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8dc36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc512dc3a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50dc3d, SUC_NE_lexicon_tags_3 },
    { 0x2e14dc3d, SUC_NE_lexicon_tags_2 },
    { 0x19fedc3f, SUC_NE_lexicon_tags_2 },
    { 0x2276dc3f, SUC_NE_lexicon_tags_2 },
    { 0xdbcdc40, SUC_NE_lexicon_tags_2 },
    { 0xf13cdc42, SUC_NE_lexicon_tags_2 },
    { 0x19e4dc43, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c0dc46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe818dc49, SUC_NE_lexicon_tags_2 },
    { 0x9480dc4a, SUC_NE_lexicon_tags_3 },
    { 0x7ca4dc49, SUC_NE_lexicon_tags_17 },
    { 0x7d2edc4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d4cdc4e, SUC_NE_lexicon_tags_2 },
    { 0xcd4edc4f, SUC_NE_lexicon_tags_5 },
    { 0xb120dc50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864adc54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf0dc5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533edc5f, SUC_NE_lexicon_tags_2 },
    { 0x6228dc60, SUC_NE_lexicon_tags_2 },
    { 0x18a4dc60, SUC_NE_lexicon_tags_2 },
    { 0x3a72dc62, SUC_NE_lexicon_tags_2 },
    { 0xe54adc63, SUC_NE_lexicon_tags_19 },
    { 0x1594dc60, SUC_NE_lexicon_tags_2 },
    { 0xe04adc60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe0dc68, SUC_NE_lexicon_tags_2 },
    { 0x63c6dc69, SUC_NE_lexicon_tags_2 },
    { 0x3964dc69, SUC_NE_lexicon_tags_2 },
    { 0x1dd0dc6a, SUC_NE_lexicon_tags_2 },
    { 0x1ce2dc68, SUC_NE_lexicon_tags_2 },
    { 0x888edc6d, SUC_NE_lexicon_tags_2 },
    { 0xb46adc6e, SUC_NE_lexicon_tags_2 },
    { 0xd3cdc6f, SUC_NE_lexicon_tags_2 },
    { 0x64f2dc6f, SUC_NE_lexicon_tags_2 },
    { 0xd38dc68, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2008dc73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6222dc76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b0dc7a, SUC_NE_lexicon_tags_2 },
    { 0x4e42dc7a, SUC_NE_lexicon_tags_2 },
    { 0x7130dc7b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa4dc80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7394dc82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb82dc84, SUC_NE_lexicon_tags_2 },
    { 0xde56dc84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7248dc89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d6dc90, SUC_NE_lexicon_tags_7 },
    { 0xb594dc91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aadc96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf916dc98, SUC_NE_lexicon_tags_2 },
    { 0x315adc99, SUC_NE_lexicon_tags_2 },
    { 0xe0c6dc9a, SUC_NE_lexicon_tags_4 },
    { 0x74ccdc9a, SUC_NE_lexicon_tags_2 },
    { 0xebb6dc9c, SUC_NE_lexicon_tags_2 },
    { 0x8b9adc99, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e96dca1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac4dca5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5806dca7, SUC_NE_lexicon_tags_3 },
    { 0xb10adca7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa310dcab, SUC_NE_lexicon_tags_2 },
    { 0xcfa2dcac, SUC_NE_lexicon_tags_2 },
    { 0xf8b0dcab, SUC_NE_lexicon_tags_2 },
    { 0x7550dcae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5acedcb0, SUC_NE_lexicon_tags_4 },
    { 0x316dcb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddedcb5, SUC_NE_lexicon_tags_2 },
    { 0xc058dcb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0cdcb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ecdcbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5072dcc1, SUC_NE_lexicon_tags_2 },
    { 0x1edcc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee4dcc6, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fadcca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96cdccd, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8a30dccf, SUC_NE_lexicon_tags_3 },
    { 0x9302dcd0, SUC_NE_lexicon_tags_2 },
    { 0xab82dcd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13e8dcd3, SUC_NE_lexicon_tags_2 },
    { 0x3236dcd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9142dcd8, SUC_NE_lexicon_tags_2 },
    { 0x377cdcd8, SUC_NE_lexicon_tags_2 },
    { 0x5520dcda, SUC_NE_lexicon_tags_2 },
    { 0xee60dcd9, SUC_NE_lexicon_tags_34 },
    { 0x52a4dcdc, SUC_NE_lexicon_tags_2 },
    { 0xefd4dcd9, SUC_NE_lexicon_tags_2 },
    { 0x6d1cdcde, SUC_NE_lexicon_tags_2 },
    { 0xad52dcdf, SUC_NE_lexicon_tags_2 },
    { 0xa18dce0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29badce5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c4dcec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad6dcef, SUC_NE_lexicon_tags_2 },
    { 0x9720dcf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc42dcf2, SUC_NE_lexicon_tags_2 },
    { 0x3ce8dcf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda8dcf6, SUC_NE_lexicon_tags_4 },
    { 0xefa2dcf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa388dcfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b2dcfd, SUC_NE_lexicon_tags_2 },
    { 0x71c0dcfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4308dd04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d78dd06, SUC_NE_lexicon_tags_2 },
    { 0x7434dd07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b44dd0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fedd10, SUC_NE_lexicon_tags_12 },
    { 0x6768dd11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a4dd15, SUC_NE_lexicon_tags_0 },
    { 0x9c1cdd16, SUC_NE_lexicon_tags_2 },
    { 0x1540dd15, SUC_NE_lexicon_tags_2 },
    { 0x8aa6dd18, SUC_NE_lexicon_tags_2 },
    { 0xc9e4dd19, SUC_NE_lexicon_tags_4 },
    { 0x33b4dd18, SUC_NE_lexicon_tags_2 },
    { 0x38e0dd1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5370dd1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x614cdd20, SUC_NE_lexicon_tags_2 },
    { 0x6d38dd20, SUC_NE_lexicon_tags_2 },
    { 0x64d6dd22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a88dd26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66badd29, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x2fd8dd2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52d2dd2d, SUC_NE_lexicon_tags_2 },
    { 0x888add2e, SUC_NE_lexicon_tags_2 },
    { 0x59a0dd2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f6dd33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc134dd3a, SUC_NE_lexicon_tags_2 },
    { 0x3ebedd3a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd14dd3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d6dd40, SUC_NE_lexicon_tags_2 },
    { 0x7edcdd40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa484dd46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ab6dd48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc684dd4c, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1296dd4f, SUC_NE_lexicon_tags_2 },
    { 0xa7e2dd4f, SUC_NE_lexicon_tags_2 },
    { 0xde52dd51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac02dd56, SUC_NE_lexicon_tags_14 },
    { 0x1b82dd56, SUC_NE_lexicon_tags_2 },
    { 0x233edd56, SUC_NE_lexicon_tags_2 },
    { 0x76dd59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe764dd5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8add61, SUC_NE_lexicon_tags_2 },
    { 0x70d4dd61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a00dd64, SUC_NE_lexicon_tags_4 },
    { 0x7a56dd64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d6dd6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b7edd6d, SUC_NE_lexicon_tags_2 },
    { 0xe54dd6e, SUC_NE_lexicon_tags_2 },
    { 0x8ee6dd6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9edd74, SUC_NE_lexicon_tags_2 },
    { 0x51eadd75, SUC_NE_lexicon_tags_2 },
    { 0x37aedd76, SUC_NE_lexicon_tags_2 },
    { 0xa994dd75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4dd7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b2dd80, SUC_NE_lexicon_tags_2 },
    { 0x6816dd81, SUC_NE_lexicon_tags_4 },
    { 0xa30dd82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12ecdd84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc036dd87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dfcdd89, SUC_NE_lexicon_tags_2 },
    { 0xc6fcdd89, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4156dd8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53cdd94, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dadd9a, SUC_NE_lexicon_tags_249 },
    { 0, NULL },
    { 0, NULL },
    { 0xa300dd9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x163edd9f, SUC_NE_lexicon_tags_2 },
    { 0xe164dd9f, SUC_NE_lexicon_tags_2 },
    { 0x9074dda1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8acdda3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3210dda5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbedda8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x432cddab, SUC_NE_lexicon_tags_2 },
    { 0x350cddac, SUC_NE_lexicon_tags_2 },
    { 0x1d20ddad, SUC_NE_lexicon_tags_2 },
    { 0x4856ddae, SUC_NE_lexicon_tags_2 },
    { 0xbd8eddaf, SUC_NE_lexicon_tags_2 },
    { 0xada8ddb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f8ddb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5addb9, SUC_NE_lexicon_tags_2 },
    { 0x4e42ddba, SUC_NE_lexicon_tags_2 },
    { 0x8efeddbb, SUC_NE_lexicon_tags_0 },
    { 0x43d8ddbc, SUC_NE_lexicon_tags_2 },
    { 0x4fc8ddb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6a6ddbf, SUC_NE_lexicon_tags_2 },
    { 0xec5cddc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cddc4, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb976ddcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57aeddce, SUC_NE_lexicon_tags_2 },
    { 0xd906ddcf, SUC_NE_lexicon_tags_2 },
    { 0xcfb6ddd0, SUC_NE_lexicon_tags_2 },
    { 0x1b14ddd1, SUC_NE_lexicon_tags_2 },
    { 0xc13addd1, SUC_NE_lexicon_tags_2 },
    { 0xa35cddce, SUC_NE_lexicon_tags_2 },
    { 0xb434ddd3, SUC_NE_lexicon_tags_2 },
    { 0x9d9eddd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23adddb, SUC_NE_lexicon_tags_2 },
    { 0x8322dddc, SUC_NE_lexicon_tags_2 },
    { 0x993adddb, SUC_NE_lexicon_tags_2 },
    { 0xa72addde, SUC_NE_lexicon_tags_2 },
    { 0xfa7adddf, SUC_NE_lexicon_tags_2 },
    { 0xf33cdde0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93acdde7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9addeb, SUC_NE_lexicon_tags_2 },
    { 0x11eaddeb, SUC_NE_lexicon_tags_2 },
    { 0xd43eddec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2072ddef, SUC_NE_lexicon_tags_39 },
    { 0x132addef, SUC_NE_lexicon_tags_2 },
    { 0x24a4ddf0, SUC_NE_lexicon_tags_2 },
    { 0xf106ddf0, SUC_NE_lexicon_tags_2 },
    { 0x1110ddf1, SUC_NE_lexicon_tags_2 },
    { 0xef5eddf4, SUC_NE_lexicon_tags_2 },
    { 0x8acddf3, SUC_NE_lexicon_tags_2 },
    { 0x328cddf6, SUC_NE_lexicon_tags_2 },
    { 0xf082ddf7, SUC_NE_lexicon_tags_2 },
    { 0x8bf8ddf8, SUC_NE_lexicon_tags_2 },
    { 0x738ddf9, SUC_NE_lexicon_tags_2 },
    { 0x3b8ddf6, SUC_NE_lexicon_tags_0 },
    { 0x7f5addfb, SUC_NE_lexicon_tags_2 },
    { 0x2848ddfc, SUC_NE_lexicon_tags_2 },
    { 0xbda6ddf4, SUC_NE_lexicon_tags_2 },
    { 0x45f8ddfe, SUC_NE_lexicon_tags_2 },
    { 0x2aaddfe, SUC_NE_lexicon_tags_2 },
    { 0x88ecddef, SUC_NE_lexicon_tags_0 },
    { 0xd7d6ddff, SUC_NE_lexicon_tags_2 },
    { 0x404cde02, SUC_NE_lexicon_tags_2 },
    { 0x2e8de02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e52de05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68ade08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09cde0b, SUC_NE_lexicon_tags_2 },
    { 0x4b04de0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef2de11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba2de15, SUC_NE_lexicon_tags_2 },
    { 0xb4a8de16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbede1b, SUC_NE_lexicon_tags_2 },
    { 0x8f5ede1c, SUC_NE_lexicon_tags_2 },
    { 0x31a4de1c, SUC_NE_lexicon_tags_2 },
    { 0xfcf2de1b, SUC_NE_lexicon_tags_2 },
    { 0xae0de1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd562de22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b38de25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4acde2d, SUC_NE_lexicon_tags_14 },
    { 0xdec0de2d, SUC_NE_lexicon_tags_2 },
    { 0xaa2de2f, SUC_NE_lexicon_tags_12 },
    { 0xf256de2f, SUC_NE_lexicon_tags_2 },
    { 0xcf0ade31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1278de35, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe684de37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x966de3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5412de3f, SUC_NE_lexicon_tags_2 },
    { 0x22ecde40, SUC_NE_lexicon_tags_2 },
    { 0x6d88de3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd758de48, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8ade4c, SUC_NE_lexicon_tags_2 },
    { 0x7440de4d, SUC_NE_lexicon_tags_2 },
    { 0x4b80de4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2128de53, SUC_NE_lexicon_tags_2 },
    { 0x5f0ade53, SUC_NE_lexicon_tags_2 },
    { 0xbf00de55, SUC_NE_lexicon_tags_2 },
    { 0xc8e2de56, SUC_NE_lexicon_tags_2 },
    { 0xf8dcde57, SUC_NE_lexicon_tags_2 },
    { 0x5be6de54, SUC_NE_lexicon_tags_2 },
    { 0xd782de59, SUC_NE_lexicon_tags_0 },
    { 0x1c94de5a, SUC_NE_lexicon_tags_2 },
    { 0x6570de5b, SUC_NE_lexicon_tags_2 },
    { 0x6922de5c, SUC_NE_lexicon_tags_2 },
    { 0x7bb8de5c, SUC_NE_lexicon_tags_7 },
    { 0x8db2de5e, SUC_NE_lexicon_tags_2 },
    { 0x8178de5b, SUC_NE_lexicon_tags_2 },
    { 0xdd7cde5e, SUC_NE_lexicon_tags_2 },
    { 0xd0dede61, SUC_NE_lexicon_tags_2 },
    { 0x50acde62, SUC_NE_lexicon_tags_2 },
    { 0xf690de62, SUC_NE_lexicon_tags_2 },
    { 0x7d3cde64, SUC_NE_lexicon_tags_2 },
    { 0x652ade5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4de6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e38de70, SUC_NE_lexicon_tags_2 },
    { 0x8f7ede71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f78de75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5acde78, SUC_NE_lexicon_tags_4 },
    { 0x6b88de78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a0de7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0bede7e, SUC_NE_lexicon_tags_2 },
    { 0xb530de7e, SUC_NE_lexicon_tags_2 },
    { 0x436cde7f, SUC_NE_lexicon_tags_184 },
    { 0x8434de7f, SUC_NE_lexicon_tags_2 },
    { 0x8546de7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeab6de84, SUC_NE_lexicon_tags_2 },
    { 0x48e4de85, SUC_NE_lexicon_tags_2 },
    { 0x7ef8de85, SUC_NE_lexicon_tags_2 },
    { 0x4acede87, SUC_NE_lexicon_tags_2 },
    { 0x78aede88, SUC_NE_lexicon_tags_2 },
    { 0xed6cde88, SUC_NE_lexicon_tags_2 },
    { 0x74a6de8a, SUC_NE_lexicon_tags_2 },
    { 0x2b9ade86, SUC_NE_lexicon_tags_2 },
    { 0x927ede85, SUC_NE_lexicon_tags_2 },
    { 0x5438de8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e4de90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a8ade92, SUC_NE_lexicon_tags_2 },
    { 0x301cde93, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xb600de95, SUC_NE_lexicon_tags_2 },
    { 0xe43ade96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc4cde98, SUC_NE_lexicon_tags_2 },
    { 0x1f48de98, SUC_NE_lexicon_tags_2 },
    { 0xb9ecde9a, SUC_NE_lexicon_tags_2 },
    { 0x332ade98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9424de9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xceb0de9f, SUC_NE_lexicon_tags_2 },
    { 0xbf78de9f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf4dea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7452dea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e48dea7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f6deab, SUC_NE_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5adeae, SUC_NE_lexicon_tags_2 },
    { 0x6be8deaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7f0deb1, SUC_NE_lexicon_tags_2 },
    { 0x3f5adeb1, SUC_NE_lexicon_tags_18 },
    { 0x1b08deb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2182deb5, SUC_NE_lexicon_tags_2 },
    { 0x5a0cdeb5, SUC_NE_lexicon_tags_2 },
    { 0x8e70deb7, SUC_NE_lexicon_tags_2 },
    { 0x5e2cdeb7, SUC_NE_lexicon_tags_2 },
    { 0x9974deb5, SUC_NE_lexicon_tags_2 },
    { 0x6c88deba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137cdebf, SUC_NE_lexicon_tags_2 },
    { 0xce2debf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd546decc, SUC_NE_lexicon_tags_2 },
    { 0xfad8decd, SUC_NE_lexicon_tags_7 },
    { 0xcc6adece, SUC_NE_lexicon_tags_2 },
    { 0xb3f4decf, SUC_NE_lexicon_tags_2 },
    { 0x37cded0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xceeeded2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93cded8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465adedd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6458dedf, SUC_NE_lexicon_tags_2 },
    { 0x3bc0dee0, SUC_NE_lexicon_tags_4 },
    { 0xd04adee1, SUC_NE_lexicon_tags_2 },
    { 0xfb88dedf, SUC_NE_lexicon_tags_2 },
    { 0xece8dee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cdee6, SUC_NE_lexicon_tags_2 },
    { 0x88fcdee6, SUC_NE_lexicon_tags_2 },
    { 0xcf32dee7, SUC_NE_lexicon_tags_2 },
    { 0x9f80dee9, SUC_NE_lexicon_tags_2 },
    { 0x931edee7, SUC_NE_lexicon_tags_2 },
    { 0x774cdeeb, SUC_NE_lexicon_tags_2 },
    { 0xf9b4deec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb7adeee, SUC_NE_lexicon_tags_2 },
    { 0x91aedeef, SUC_NE_lexicon_tags_2 },
    { 0x8168deef, SUC_NE_lexicon_tags_2 },
    { 0x8fd4deef, SUC_NE_lexicon_tags_2 },
    { 0x663edef2, SUC_NE_lexicon_tags_2 },
    { 0xe388def3, SUC_NE_lexicon_tags_2 },
    { 0x8bcdef3, SUC_NE_lexicon_tags_2 },
    { 0x269edef5, SUC_NE_lexicon_tags_442 },
    { 0, NULL },
    { 0x5d20def7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89cdefa, SUC_NE_lexicon_tags_40 },
    { 0x3696defb, SUC_NE_lexicon_tags_2 },
    { 0x7c8defb, SUC_NE_lexicon_tags_2 },
    { 0xcca6defd, SUC_NE_lexicon_tags_2 },
    { 0xa294defe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd03cdf00, SUC_NE_lexicon_tags_2 },
    { 0x55ccdf01, SUC_NE_lexicon_tags_2 },
    { 0x91eedf01, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3cdf05, SUC_NE_lexicon_tags_2 },
    { 0xaf60df06, SUC_NE_lexicon_tags_2 },
    { 0x8288df07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bd6df09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14aadf0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c62df10, SUC_NE_lexicon_tags_2 },
    { 0xcf66df10, SUC_NE_lexicon_tags_2 },
    { 0xf82edf12, SUC_NE_lexicon_tags_2 },
    { 0xfeadf12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f38df15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffecdf1b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8df1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a6df2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d40df33, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0df36, SUC_NE_lexicon_tags_2 },
    { 0xcb76df36, SUC_NE_lexicon_tags_2 },
    { 0xf50cdf38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b40df45, SUC_NE_lexicon_tags_2 },
    { 0x2ad0df46, SUC_NE_lexicon_tags_4 },
    { 0x7bf6df47, SUC_NE_lexicon_tags_2 },
    { 0xfdc0df47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf824df52, SUC_NE_lexicon_tags_2 },
    { 0x43eedf52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb608df61, SUC_NE_lexicon_tags_2 },
    { 0x6fe6df62, SUC_NE_lexicon_tags_2 },
    { 0x92c8df63, SUC_NE_lexicon_tags_2 },
    { 0x6792df64, SUC_NE_lexicon_tags_2 },
    { 0xe7c4df65, SUC_NE_lexicon_tags_2 },
    { 0x9ea2df66, SUC_NE_lexicon_tags_2 },
    { 0x3586df66, SUC_NE_lexicon_tags_2 },
    { 0x868adf66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caedf6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e8df6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf24df73, SUC_NE_lexicon_tags_2 },
    { 0x3778df74, SUC_NE_lexicon_tags_2 },
    { 0xac8cdf74, SUC_NE_lexicon_tags_2 },
    { 0x4092df76, SUC_NE_lexicon_tags_5 },
    { 0xc87adf75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf0edf79, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0xeff8df7b, SUC_NE_lexicon_tags_2 },
    { 0xaffadf7c, SUC_NE_lexicon_tags_2 },
    { 0x845cdf7c, SUC_NE_lexicon_tags_2 },
    { 0xed5adf7e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1016df82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711adf89, SUC_NE_lexicon_tags_2 },
    { 0x6100df8a, SUC_NE_lexicon_tags_2 },
    { 0xbdf4df8a, SUC_NE_lexicon_tags_2 },
    { 0x871adf8c, SUC_NE_lexicon_tags_2 },
    { 0xabb6df8c, SUC_NE_lexicon_tags_2 },
    { 0x6802df8e, SUC_NE_lexicon_tags_2 },
    { 0xc446df8f, SUC_NE_lexicon_tags_2 },
    { 0x550df90, SUC_NE_lexicon_tags_2 },
    { 0xc7f6df91, SUC_NE_lexicon_tags_2 },
    { 0xaddedf8e, SUC_NE_lexicon_tags_3 },
    { 0x9e46df91, SUC_NE_lexicon_tags_2 },
    { 0x9338df8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515adf9b, SUC_NE_lexicon_tags_2 },
    { 0x5188df9c, SUC_NE_lexicon_tags_2 },
    { 0xbac0df9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x697adfa0, SUC_NE_lexicon_tags_2 },
    { 0x3776dfa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x590adfa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7080dfa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03adfa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22a0dfab, SUC_NE_lexicon_tags_2 },
    { 0x806cdfac, SUC_NE_lexicon_tags_2 },
    { 0xd57cdfad, SUC_NE_lexicon_tags_2 },
    { 0xc056dfad, SUC_NE_lexicon_tags_2 },
    { 0xd65adfaf, SUC_NE_lexicon_tags_2 },
    { 0xdf30dfb0, SUC_NE_lexicon_tags_1 },
    { 0x7886dfb0, SUC_NE_lexicon_tags_2 },
    { 0x12e0dfb2, SUC_NE_lexicon_tags_2 },
    { 0xb07adfad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8adfb6, SUC_NE_lexicon_tags_2 },
    { 0xbe42dfb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba74dfba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeed2dfbc, SUC_NE_lexicon_tags_2 },
    { 0x7f4edfbc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42dfc0, SUC_NE_lexicon_tags_2 },
    { 0x24aedfc1, SUC_NE_lexicon_tags_8 },
    { 0xbd40dfc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5364dfc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d0dfcc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x964dfcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9936dfd2, SUC_NE_lexicon_tags_2 },
    { 0x3da4dfd2, SUC_NE_lexicon_tags_2 },
    { 0x3532dfd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52d6dfd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b9cdfd8, SUC_NE_lexicon_tags_2 },
    { 0x7ce8dfd9, SUC_NE_lexicon_tags_2 },
    { 0xc018dfd9, SUC_NE_lexicon_tags_2 },
    { 0x570dfdb, SUC_NE_lexicon_tags_2 },
    { 0x5ab6dfdb, SUC_NE_lexicon_tags_2 },
    { 0xb0f2dfdb, SUC_NE_lexicon_tags_2 },
    { 0x5362dfdc, SUC_NE_lexicon_tags_2 },
    { 0x4366dfdf, SUC_NE_lexicon_tags_2 },
    { 0xab8dfdc, SUC_NE_lexicon_tags_2 },
    { 0x7276dfdd, SUC_NE_lexicon_tags_2 },
    { 0x2e52dfe2, SUC_NE_lexicon_tags_1 },
    { 0xf2f8dfe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5448dfe6, SUC_NE_lexicon_tags_2 },
    { 0xb7fadfe7, SUC_NE_lexicon_tags_2 },
    { 0x1abcdfe8, SUC_NE_lexicon_tags_2 },
    { 0x4798dfe9, SUC_NE_lexicon_tags_2 },
    { 0x4506dfe9, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x8e44dfec, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x619edfee, SUC_NE_lexicon_tags_2 },
    { 0xef64dfee, SUC_NE_lexicon_tags_7 },
    { 0xf752dfef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8badff2, SUC_NE_lexicon_tags_2 },
    { 0x441edff2, SUC_NE_lexicon_tags_2 },
    { 0x5fa0dff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9184dff9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293adffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc858e004, SUC_NE_lexicon_tags_2 },
    { 0xf2b2e005, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cdee007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b4e00b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x898ee00e, SUC_NE_lexicon_tags_3 },
    { 0xafcae00f, SUC_NE_lexicon_tags_2 },
    { 0x9f1ce00f, SUC_NE_lexicon_tags_2 },
    { 0x7056e011, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0ae017, SUC_NE_lexicon_tags_2 },
    { 0x9366e017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5804e01c, SUC_NE_lexicon_tags_2 },
    { 0x31d2e01d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa764e01f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd42e028, SUC_NE_lexicon_tags_1 },
    { 0xb33ae029, SUC_NE_lexicon_tags_2 },
    { 0xc076e029, SUC_NE_lexicon_tags_2 },
    { 0x4378e02a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2788e02d, SUC_NE_lexicon_tags_2 },
    { 0x65ee02e, SUC_NE_lexicon_tags_2 },
    { 0xf5f6e02f, SUC_NE_lexicon_tags_4 },
    { 0xd934e030, SUC_NE_lexicon_tags_2 },
    { 0x9d4ae030, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab32e037, SUC_NE_lexicon_tags_0 },
    { 0x18d2e037, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22e03b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d90e03e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94ce044, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6db6e046, SUC_NE_lexicon_tags_2 },
    { 0x3d8e047, SUC_NE_lexicon_tags_2 },
    { 0x2d3ee048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d8e04a, SUC_NE_lexicon_tags_2 },
    { 0xdf74e04b, SUC_NE_lexicon_tags_2 },
    { 0x8bcae04c, SUC_NE_lexicon_tags_2 },
    { 0x2d00e04d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a14e051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c46e055, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc8e058, SUC_NE_lexicon_tags_2 },
    { 0x5902e059, SUC_NE_lexicon_tags_2 },
    { 0x5498e05a, SUC_NE_lexicon_tags_2 },
    { 0x707ae05b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced6e05f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x894e061, SUC_NE_lexicon_tags_117 },
    { 0xbc14e061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf02e064, SUC_NE_lexicon_tags_4 },
    { 0x608e065, SUC_NE_lexicon_tags_2 },
    { 0xe7ace066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1732e069, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7508e06e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82e6e070, SUC_NE_lexicon_tags_7 },
    { 0x6182e071, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8ae076, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe686e07d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb878e07f, SUC_NE_lexicon_tags_2 },
    { 0xf7d0e080, SUC_NE_lexicon_tags_2 },
    { 0x6086e081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21ee086, SUC_NE_lexicon_tags_2 },
    { 0x4a06e086, SUC_NE_lexicon_tags_2 },
    { 0xf7ee088, SUC_NE_lexicon_tags_2 },
    { 0x5750e088, SUC_NE_lexicon_tags_2 },
    { 0xc178e089, SUC_NE_lexicon_tags_2 },
    { 0x7eeee08b, SUC_NE_lexicon_tags_2 },
    { 0xd99ce08c, SUC_NE_lexicon_tags_2 },
    { 0x5ce6e088, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa82e08f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x768ee092, SUC_NE_lexicon_tags_65 },
    { 0xa70ae093, SUC_NE_lexicon_tags_18 },
    { 0x166ae094, SUC_NE_lexicon_tags_2 },
    { 0xbff4e094, SUC_NE_lexicon_tags_2 },
    { 0xa6b4e093, SUC_NE_lexicon_tags_2 },
    { 0xbaaae097, SUC_NE_lexicon_tags_2 },
    { 0xd74ee096, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf448e09a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2474e09c, SUC_NE_lexicon_tags_2 },
    { 0x233ce09d, SUC_NE_lexicon_tags_2 },
    { 0x99c2e09e, SUC_NE_lexicon_tags_4 },
    { 0xe4d2e09f, SUC_NE_lexicon_tags_2 },
    { 0xde86e0a0, SUC_NE_lexicon_tags_2 },
    { 0x9486e0a0, SUC_NE_lexicon_tags_3 },
    { 0xe038e0a1, SUC_NE_lexicon_tags_2 },
    { 0xed4e0a3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xfcdae0a5, SUC_NE_lexicon_tags_2 },
    { 0x9838e0a6, SUC_NE_lexicon_tags_2 },
    { 0x4836e0a7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xefa4e0a9, SUC_NE_lexicon_tags_2 },
    { 0x5866e0aa, SUC_NE_lexicon_tags_2 },
    { 0xc810e0ab, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xca96e0ad, SUC_NE_lexicon_tags_2 },
    { 0x9bc2e0ad, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cae0b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c70e0b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fee0b8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79ce0bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ca2e0be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99eae0c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ae0c3, SUC_NE_lexicon_tags_2 },
    { 0x920ae0c4, SUC_NE_lexicon_tags_85 },
    { 0x78aae0c4, SUC_NE_lexicon_tags_2 },
    { 0xec98e0c6, SUC_NE_lexicon_tags_2 },
    { 0x2abee0c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcae0cb, SUC_NE_lexicon_tags_2 },
    { 0xc908e0cc, SUC_NE_lexicon_tags_2 },
    { 0xe5bce0cb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01ae0d2, SUC_NE_lexicon_tags_2 },
    { 0x6116e0d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cae0d7, SUC_NE_lexicon_tags_18 },
    { 0xb0cce0d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b4e0dd, SUC_NE_lexicon_tags_2 },
    { 0x867ee0de, SUC_NE_lexicon_tags_2 },
    { 0xa94e0df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fae0e6, SUC_NE_lexicon_tags_2 },
    { 0xcc00e0e6, SUC_NE_lexicon_tags_30 },
    { 0xe190e0e8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3ae0eb, SUC_NE_lexicon_tags_2 },
    { 0xe208e0eb, SUC_NE_lexicon_tags_4 },
    { 0xd830e0ed, SUC_NE_lexicon_tags_2 },
    { 0xa85ee0ee, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c2e0f3, SUC_NE_lexicon_tags_2 },
    { 0xf7cce0f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3020e0f7, SUC_NE_lexicon_tags_5 },
    { 0x50fce0f8, SUC_NE_lexicon_tags_3 },
    { 0x15ae0f9, SUC_NE_lexicon_tags_2 },
    { 0x41f8e0fa, SUC_NE_lexicon_tags_2 },
    { 0xd6f8e0fb, SUC_NE_lexicon_tags_2 },
    { 0x3f78e0fc, SUC_NE_lexicon_tags_2 },
    { 0x19c8e0f8, SUC_NE_lexicon_tags_2 },
    { 0x2214e0fa, SUC_NE_lexicon_tags_1 },
    { 0xfa3ae0ff, SUC_NE_lexicon_tags_4 },
    { 0x7a30e100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1ce105, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4ce108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b04e10a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd234e10e, SUC_NE_lexicon_tags_1 },
    { 0xe830e10f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8db0e111, SUC_NE_lexicon_tags_2 },
    { 0xf828e112, SUC_NE_lexicon_tags_2 },
    { 0xa988e112, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc658e119, SUC_NE_lexicon_tags_0 },
    { 0x7d04e11a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42ee11f, SUC_NE_lexicon_tags_2 },
    { 0xc704e11f, SUC_NE_lexicon_tags_0 },
    { 0x8d16e120, SUC_NE_lexicon_tags_2 },
    { 0x5a1ce121, SUC_NE_lexicon_tags_2 },
    { 0xe66ae121, SUC_NE_lexicon_tags_2 },
    { 0x6fbee124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e2ee126, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa8e129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74e12c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbce12e, SUC_NE_lexicon_tags_2 },
    { 0xdcfce12f, SUC_NE_lexicon_tags_7 },
    { 0x1df4e12f, SUC_NE_lexicon_tags_3 },
    { 0x9102e131, SUC_NE_lexicon_tags_2 },
    { 0x7eaee130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb918e136, SUC_NE_lexicon_tags_2 },
    { 0xbce0e137, SUC_NE_lexicon_tags_8 },
    { 0xd0b2e138, SUC_NE_lexicon_tags_2 },
    { 0x7748e139, SUC_NE_lexicon_tags_2 },
    { 0x968ee13a, SUC_NE_lexicon_tags_2 },
    { 0xb84ee13a, SUC_NE_lexicon_tags_2 },
    { 0xfd90e13c, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e94e13f, SUC_NE_lexicon_tags_2 },
    { 0x624e13f, SUC_NE_lexicon_tags_2 },
    { 0x9758e141, SUC_NE_lexicon_tags_2 },
    { 0x9e60e142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe89ce144, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a0e146, SUC_NE_lexicon_tags_2 },
    { 0xbe4ae147, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ae14a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xac9ae14c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3ae14f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfbce151, SUC_NE_lexicon_tags_2 },
    { 0xb94ce151, SUC_NE_lexicon_tags_3 },
    { 0xc434e151, SUC_NE_lexicon_tags_2 },
    { 0xb8f8e152, SUC_NE_lexicon_tags_2 },
    { 0xc05ee152, SUC_NE_lexicon_tags_2 },
    { 0xc4bae152, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286ae15b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3ece15d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x185ce15f, SUC_NE_lexicon_tags_3 },
    { 0x39b6e160, SUC_NE_lexicon_tags_4 },
    { 0xa046e161, SUC_NE_lexicon_tags_2 },
    { 0x7d5ce15f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb6e165, SUC_NE_lexicon_tags_4 },
    { 0x555ce166, SUC_NE_lexicon_tags_2 },
    { 0x8b02e167, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xbec0e169, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc20e16d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x224ae170, SUC_NE_lexicon_tags_2 },
    { 0x7758e171, SUC_NE_lexicon_tags_2 },
    { 0xa760e172, SUC_NE_lexicon_tags_2 },
    { 0xd01ae172, SUC_NE_lexicon_tags_2 },
    { 0xb2f6e172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb102e176, SUC_NE_lexicon_tags_7 },
    { 0x3d3ae177, SUC_NE_lexicon_tags_3 },
    { 0xadaee177, SUC_NE_lexicon_tags_2 },
    { 0x2548e179, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7c6e17b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d4e17e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3426e180, SUC_NE_lexicon_tags_2 },
    { 0x11bee181, SUC_NE_lexicon_tags_2 },
    { 0xf4fee181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f8ce184, SUC_NE_lexicon_tags_2 },
    { 0xe5c8e185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15ce189, SUC_NE_lexicon_tags_2 },
    { 0x6a36e189, SUC_NE_lexicon_tags_2 },
    { 0xcc62e18b, SUC_NE_lexicon_tags_2 },
    { 0xb6f8e18c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2052e18f, SUC_NE_lexicon_tags_2 },
    { 0x1386e190, SUC_NE_lexicon_tags_2 },
    { 0xe20ae191, SUC_NE_lexicon_tags_2 },
    { 0x4e8ee192, SUC_NE_lexicon_tags_2 },
    { 0x4ef6e193, SUC_NE_lexicon_tags_4 },
    { 0xce06e194, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c44e198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13e4e19a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4e1a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb354e1a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f40e1ab, SUC_NE_lexicon_tags_4 },
    { 0xb0dce1ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec1ae1ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f18e1b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde80e1b8, SUC_NE_lexicon_tags_2 },
    { 0x8038e1b9, SUC_NE_lexicon_tags_2 },
    { 0xcbc6e1ba, SUC_NE_lexicon_tags_2 },
    { 0x39d0e1bb, SUC_NE_lexicon_tags_2 },
    { 0x4bb4e1bc, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x246ae1be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396ce1c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6ee1c7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccae1cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8638e1cf, SUC_NE_lexicon_tags_2 },
    { 0x930ce1cf, SUC_NE_lexicon_tags_2 },
    { 0x431ee1d0, SUC_NE_lexicon_tags_7 },
    { 0xea5ae1d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ce1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ee1dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8338e1e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a36e1e4, SUC_NE_lexicon_tags_2 },
    { 0x8040e1e5, SUC_NE_lexicon_tags_2 },
    { 0x9c86e1e6, SUC_NE_lexicon_tags_2 },
    { 0xeadce1e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x481ee1e9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ee1ee, SUC_NE_lexicon_tags_2 },
    { 0xa162e1ee, SUC_NE_lexicon_tags_2 },
    { 0x61ee1f0, SUC_NE_lexicon_tags_2 },
    { 0x2a4ae1f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66cce1f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b04e1f6, SUC_NE_lexicon_tags_2 },
    { 0x19c8e1f7, SUC_NE_lexicon_tags_2 },
    { 0x9cbae1f8, SUC_NE_lexicon_tags_2 },
    { 0xfeb8e1f9, SUC_NE_lexicon_tags_2 },
    { 0xd6f8e1fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e82e1fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ee203, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce0e205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf646e208, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb268e20c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fb6e20e, SUC_NE_lexicon_tags_2 },
    { 0x82cce20f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bbae211, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf0e213, SUC_NE_lexicon_tags_2 },
    { 0x2be8e213, SUC_NE_lexicon_tags_2 },
    { 0xc182e215, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xd99ce217, SUC_NE_lexicon_tags_2 },
    { 0xe524e218, SUC_NE_lexicon_tags_2 },
    { 0x5756e219, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aae21d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3168e21f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5406e225, SUC_NE_lexicon_tags_3 },
    { 0x2702e226, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e26e229, SUC_NE_lexicon_tags_19 },
    { 0x453ce229, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab6e22e, SUC_NE_lexicon_tags_2 },
    { 0xdbb4e22f, SUC_NE_lexicon_tags_2 },
    { 0x6114e22e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5a8e232, SUC_NE_lexicon_tags_40 },
    { 0xfb44e232, SUC_NE_lexicon_tags_2 },
    { 0x3466e234, SUC_NE_lexicon_tags_2 },
    { 0x5318e235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de8e23a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1ee242, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc0e248, SUC_NE_lexicon_tags_2 },
    { 0x96c2e248, SUC_NE_lexicon_tags_2 },
    { 0xe14ce24a, SUC_NE_lexicon_tags_2 },
    { 0x4120e249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ae24f, SUC_NE_lexicon_tags_2 },
    { 0x763ce24f, SUC_NE_lexicon_tags_2 },
    { 0xb9ce251, SUC_NE_lexicon_tags_2 },
    { 0x3692e252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87ee255, SUC_NE_lexicon_tags_2 },
    { 0x785ae256, SUC_NE_lexicon_tags_0 },
    { 0x4870e257, SUC_NE_lexicon_tags_2 },
    { 0xa0d2e257, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xd20ae25a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6190e260, SUC_NE_lexicon_tags_2 },
    { 0xadfae260, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0x62d2e263, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca2e265, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffce268, SUC_NE_lexicon_tags_2 },
    { 0x568e269, SUC_NE_lexicon_tags_164 },
    { 0xf12ce268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa67ce26c, SUC_NE_lexicon_tags_2 },
    { 0xe134e26c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xacaae26f, SUC_NE_lexicon_tags_2 },
    { 0x971ee270, SUC_NE_lexicon_tags_2 },
    { 0x3da4e270, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9614e273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da0e278, SUC_NE_lexicon_tags_2 },
    { 0x2444e278, SUC_NE_lexicon_tags_2 },
    { 0x68b8e278, SUC_NE_lexicon_tags_2 },
    { 0x152ee27b, SUC_NE_lexicon_tags_2 },
    { 0x1fc8e27b, SUC_NE_lexicon_tags_2 },
    { 0x2214e27d, SUC_NE_lexicon_tags_2 },
    { 0x3b98e27e, SUC_NE_lexicon_tags_2 },
    { 0x2ee6e27f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd8e282, SUC_NE_lexicon_tags_2 },
    { 0x2bf4e282, SUC_NE_lexicon_tags_2 },
    { 0x142e284, SUC_NE_lexicon_tags_3 },
    { 0x9ae282, SUC_NE_lexicon_tags_2 },
    { 0xd17ce282, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf264e289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x794e28b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe0ee28d, SUC_NE_lexicon_tags_2 },
    { 0x2bdee28d, SUC_NE_lexicon_tags_2 },
    { 0x4b9ee28d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ee293, SUC_NE_lexicon_tags_17 },
    { 0xb61ce294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9ee29b, SUC_NE_lexicon_tags_2 },
    { 0x46e4e29b, SUC_NE_lexicon_tags_2 },
    { 0xa7b8e29d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bc6e29f, SUC_NE_lexicon_tags_2 },
    { 0xe32e29f, SUC_NE_lexicon_tags_2 },
    { 0x70d8e2a1, SUC_NE_lexicon_tags_2 },
    { 0x958ae2a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1940e2a4, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x5786e2a6, SUC_NE_lexicon_tags_2 },
    { 0xf5e2e2a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0ae2aa, SUC_NE_lexicon_tags_1 },
    { 0xd99ce2ab, SUC_NE_lexicon_tags_2 },
    { 0xcb38e2ac, SUC_NE_lexicon_tags_333 },
    { 0xaa0ee2ad, SUC_NE_lexicon_tags_2 },
    { 0xf434e2ae, SUC_NE_lexicon_tags_2 },
    { 0xf0ace2af, SUC_NE_lexicon_tags_2 },
    { 0x557ee2b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf60e2b2, SUC_NE_lexicon_tags_2 },
    { 0x36fae2b2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x99ace2b5, SUC_NE_lexicon_tags_2 },
    { 0xb7c6e2b6, SUC_NE_lexicon_tags_1 },
    { 0x259ee2b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75f8e2b9, SUC_NE_lexicon_tags_98 },
    { 0x88dae2ba, SUC_NE_lexicon_tags_2 },
    { 0x2466e2bb, SUC_NE_lexicon_tags_2 },
    { 0xb928e2b9, SUC_NE_lexicon_tags_2 },
    { 0xe5e4e2bd, SUC_NE_lexicon_tags_2 },
    { 0xb668e2be, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2ae2c2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cce2c8, SUC_NE_lexicon_tags_2 },
    { 0x70d0e2c9, SUC_NE_lexicon_tags_2 },
    { 0xde12e2c8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2e2cf, SUC_NE_lexicon_tags_2 },
    { 0x2c24e2d0, SUC_NE_lexicon_tags_2 },
    { 0x1fcae2d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cae2d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed96e2d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b06e2d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ba2e2da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6e2e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b88e2e2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ee2e5, SUC_NE_lexicon_tags_100 },
    { 0x7ff8e2e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1eae2e8, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7ee2ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6398e2f0, SUC_NE_lexicon_tags_2 },
    { 0x1798e2f1, SUC_NE_lexicon_tags_2 },
    { 0xd508e2f2, SUC_NE_lexicon_tags_2 },
    { 0x20ace2f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d2e2f8, SUC_NE_lexicon_tags_2 },
    { 0x3dece2f9, SUC_NE_lexicon_tags_2 },
    { 0xd9d8e2fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0eee2fc, SUC_NE_lexicon_tags_2 },
    { 0x53d0e2fc, SUC_NE_lexicon_tags_2 },
    { 0xd9c0e2fd, SUC_NE_lexicon_tags_2 },
    { 0x3bce2fe, SUC_NE_lexicon_tags_2 },
    { 0x57f8e300, SUC_NE_lexicon_tags_2 },
    { 0x4d6ae300, SUC_NE_lexicon_tags_2 },
    { 0xc654e302, SUC_NE_lexicon_tags_2 },
    { 0x3818e303, SUC_NE_lexicon_tags_2 },
    { 0xeb32e303, SUC_NE_lexicon_tags_2 },
    { 0xbf1ee300, SUC_NE_lexicon_tags_2 },
    { 0x6522e306, SUC_NE_lexicon_tags_65 },
    { 0xd0bee307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e64e30a, SUC_NE_lexicon_tags_4 },
    { 0xc770e30b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a2e30f, SUC_NE_lexicon_tags_2 },
    { 0x90c8e310, SUC_NE_lexicon_tags_2 },
    { 0x7096e310, SUC_NE_lexicon_tags_2 },
    { 0x8442e310, SUC_NE_lexicon_tags_2 },
    { 0xc68e313, SUC_NE_lexicon_tags_2 },
    { 0xc568e312, SUC_NE_lexicon_tags_2 },
    { 0xd4dce315, SUC_NE_lexicon_tags_2 },
    { 0xcb1ae316, SUC_NE_lexicon_tags_2 },
    { 0x1d5ae311, SUC_NE_lexicon_tags_2 },
    { 0x2b66e315, SUC_NE_lexicon_tags_2 },
    { 0x7cf4e317, SUC_NE_lexicon_tags_2 },
    { 0xa75ae31a, SUC_NE_lexicon_tags_2 },
    { 0xeccee31b, SUC_NE_lexicon_tags_2 },
    { 0xeb84e318, SUC_NE_lexicon_tags_2 },
    { 0xd79ce317, SUC_NE_lexicon_tags_2 },
    { 0xda5ce311, SUC_NE_lexicon_tags_2 },
    { 0xa870e31f, SUC_NE_lexicon_tags_2 },
    { 0x3312e31f, SUC_NE_lexicon_tags_2 },
    { 0xf916e321, SUC_NE_lexicon_tags_2 },
    { 0xc830e316, SUC_NE_lexicon_tags_2 },
    { 0xcc2ce310, SUC_NE_lexicon_tags_2 },
    { 0x1efce313, SUC_NE_lexicon_tags_2 },
    { 0x3c62e325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dae329, SUC_NE_lexicon_tags_3 },
    { 0x876ce32a, SUC_NE_lexicon_tags_2 },
    { 0xa866e32b, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf946e330, SUC_NE_lexicon_tags_2 },
    { 0xf8d4e330, SUC_NE_lexicon_tags_2 },
    { 0x2c12e332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e6ce334, SUC_NE_lexicon_tags_1 },
    { 0x97eae335, SUC_NE_lexicon_tags_3 },
    { 0x1962e336, SUC_NE_lexicon_tags_2 },
    { 0x77c2e337, SUC_NE_lexicon_tags_2 },
    { 0xc6bce338, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x357ee33a, SUC_NE_lexicon_tags_2 },
    { 0x7b3ce33b, SUC_NE_lexicon_tags_7 },
    { 0x9ebee33c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd44e33e, SUC_NE_lexicon_tags_2 },
    { 0x48b8e33f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27ce343, SUC_NE_lexicon_tags_2 },
    { 0x4512e343, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95ee349, SUC_NE_lexicon_tags_7 },
    { 0x21d8e34a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f2e34e, SUC_NE_lexicon_tags_2 },
    { 0x7d4ee34f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa116e351, SUC_NE_lexicon_tags_2 },
    { 0x67c6e352, SUC_NE_lexicon_tags_2 },
    { 0x7a72e351, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5ee357, SUC_NE_lexicon_tags_2 },
    { 0xb8dae358, SUC_NE_lexicon_tags_2 },
    { 0x9178e358, SUC_NE_lexicon_tags_2 },
    { 0x8d8ce35a, SUC_NE_lexicon_tags_2 },
    { 0x2368e35b, SUC_NE_lexicon_tags_2 },
    { 0x5ae0e35c, SUC_NE_lexicon_tags_2 },
    { 0x31aee35b, SUC_NE_lexicon_tags_2 },
    { 0x34d4e35e, SUC_NE_lexicon_tags_2 },
    { 0xa764e35c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8e368, SUC_NE_lexicon_tags_2 },
    { 0x73a0e369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7ce36d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x6c6ae36f, SUC_NE_lexicon_tags_2 },
    { 0x47c2e36f, SUC_NE_lexicon_tags_2 },
    { 0x4f02e370, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc88e374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba6ce376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb02e378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7ee381, SUC_NE_lexicon_tags_2 },
    { 0xff84e382, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251ce387, SUC_NE_lexicon_tags_2 },
    { 0x376e387, SUC_NE_lexicon_tags_2 },
    { 0xba66e388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc62e392, SUC_NE_lexicon_tags_2 },
    { 0x5cae393, SUC_NE_lexicon_tags_2 },
    { 0xef88e394, SUC_NE_lexicon_tags_7 },
    { 0x605ce394, SUC_NE_lexicon_tags_2 },
    { 0xc18e395, SUC_NE_lexicon_tags_2 },
    { 0xf2eae395, SUC_NE_lexicon_tags_2 },
    { 0x4f6ce398, SUC_NE_lexicon_tags_2 },
    { 0xfeaae396, SUC_NE_lexicon_tags_2 },
    { 0xf808e393, SUC_NE_lexicon_tags_356 },
    { 0, NULL },
    { 0, NULL },
    { 0x7532e39d, SUC_NE_lexicon_tags_2 },
    { 0x7c5ee39e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fae3a2, SUC_NE_lexicon_tags_1 },
    { 0x21fee3a2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc012e3a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66ae3aa, SUC_NE_lexicon_tags_2 },
    { 0x3f20e3aa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9116e3ae, SUC_NE_lexicon_tags_2 },
    { 0x8e72e3ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb4e3b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc2e3b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc3ee3b9, SUC_NE_lexicon_tags_2 },
    { 0x9a00e3b9, SUC_NE_lexicon_tags_2 },
    { 0x11ece3bb, SUC_NE_lexicon_tags_2 },
    { 0x92b0e3bc, SUC_NE_lexicon_tags_2 },
    { 0x890ee3bd, SUC_NE_lexicon_tags_11 },
    { 0x1546e3be, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538ae3c2, SUC_NE_lexicon_tags_2 },
    { 0xee2e3c3, SUC_NE_lexicon_tags_2 },
    { 0x1cb0e3c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94aae3c7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4494e3cb, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbae3ce, SUC_NE_lexicon_tags_2 },
    { 0xf8f6e3cf, SUC_NE_lexicon_tags_7 },
    { 0xc488e3ce, SUC_NE_lexicon_tags_2 },
    { 0x69d4e3cf, SUC_NE_lexicon_tags_2 },
    { 0xab86e3d1, SUC_NE_lexicon_tags_2 },
    { 0x5318e3d3, SUC_NE_lexicon_tags_2 },
    { 0xc070e3d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2e0e3d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ae3dd, SUC_NE_lexicon_tags_2 },
    { 0x747ae3de, SUC_NE_lexicon_tags_2 },
    { 0x40e6e3df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60ee3e3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd630e3e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b98e3e8, SUC_NE_lexicon_tags_2 },
    { 0x1912e3e8, SUC_NE_lexicon_tags_2 },
    { 0x897ce3ea, SUC_NE_lexicon_tags_2 },
    { 0x75fce3eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccaee3ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9cee3f0, SUC_NE_lexicon_tags_17 },
    { 0xc6bce3f1, SUC_NE_lexicon_tags_2 },
    { 0x40c6e3f2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xec9ee3f4, SUC_NE_lexicon_tags_2 },
    { 0xbf54e3f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc70e3f9, SUC_NE_lexicon_tags_3 },
    { 0xd00ae3fa, SUC_NE_lexicon_tags_2 },
    { 0x336e3fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe110e402, SUC_NE_lexicon_tags_2 },
    { 0xfdd0e403, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ce406, SUC_NE_lexicon_tags_2 },
    { 0x2134e407, SUC_NE_lexicon_tags_2 },
    { 0xc6cce407, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50d6e40a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e8e40f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7518e415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4768e417, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567ae41b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e18e422, SUC_NE_lexicon_tags_2 },
    { 0xc192e423, SUC_NE_lexicon_tags_2 },
    { 0x9d6ee422, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cc4e426, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9ae42a, SUC_NE_lexicon_tags_2 },
    { 0xe7aee42b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39ece42d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d28e42f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dae432, SUC_NE_lexicon_tags_3 },
    { 0xa8a4e432, SUC_NE_lexicon_tags_30 },
    { 0x642ae433, SUC_NE_lexicon_tags_1 },
    { 0xd8bae432, SUC_NE_lexicon_tags_3 },
    { 0x1e94e436, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912ee43a, SUC_NE_lexicon_tags_2 },
    { 0x4572e43b, SUC_NE_lexicon_tags_2 },
    { 0x9756e43c, SUC_NE_lexicon_tags_2 },
    { 0x529ce43a, SUC_NE_lexicon_tags_0 },
    { 0xcaee43e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae08e441, SUC_NE_lexicon_tags_2 },
    { 0x61f6e442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ae446, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x249ee448, SUC_NE_lexicon_tags_2 },
    { 0xe248e449, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae94e44d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14b8e44f, SUC_NE_lexicon_tags_2 },
    { 0xdd7ae44f, SUC_NE_lexicon_tags_2 },
    { 0xf036e44f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92cce454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f8e458, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772ce460, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571ee464, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcadae466, SUC_NE_lexicon_tags_2 },
    { 0xa648e466, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd072e46b, SUC_NE_lexicon_tags_2 },
    { 0x7a8ae46c, SUC_NE_lexicon_tags_2 },
    { 0xa44ce46d, SUC_NE_lexicon_tags_2 },
    { 0x274ce46e, SUC_NE_lexicon_tags_2 },
    { 0x7a8ae46d, SUC_NE_lexicon_tags_19 },
    { 0xabfce46b, SUC_NE_lexicon_tags_2 },
    { 0xd33ce471, SUC_NE_lexicon_tags_219 },
    { 0, NULL },
    { 0x1ff0e473, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1922e47c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c18e47e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d74e480, SUC_NE_lexicon_tags_2 },
    { 0xf6a6e481, SUC_NE_lexicon_tags_2 },
    { 0x2e18e482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351ce487, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1dae489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7adce48b, SUC_NE_lexicon_tags_2 },
    { 0x1084e48c, SUC_NE_lexicon_tags_2 },
    { 0xefe0e48b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1ce494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x165ae496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda12e498, SUC_NE_lexicon_tags_7 },
    { 0xe21ce498, SUC_NE_lexicon_tags_2 },
    { 0x661ae49a, SUC_NE_lexicon_tags_0 },
    { 0xbf42e49b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cae4a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828ee4a8, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x668ae4aa, SUC_NE_lexicon_tags_2 },
    { 0x357ce4aa, SUC_NE_lexicon_tags_2 },
    { 0xb1eee4ab, SUC_NE_lexicon_tags_2 },
    { 0xacf2e4ad, SUC_NE_lexicon_tags_2 },
    { 0xb250e4ac, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddee4b2, SUC_NE_lexicon_tags_2 },
    { 0x168ae4b2, SUC_NE_lexicon_tags_2 },
    { 0x395ee4b4, SUC_NE_lexicon_tags_2 },
    { 0x7e3ae4b3, SUC_NE_lexicon_tags_2 },
    { 0x6c5ce4b2, SUC_NE_lexicon_tags_2 },
    { 0x3ef2e4b7, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26ce4ba, SUC_NE_lexicon_tags_2 },
    { 0xab12e4bb, SUC_NE_lexicon_tags_2 },
    { 0x7af8e4bb, SUC_NE_lexicon_tags_2 },
    { 0xe6c6e4bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d80e4c1, SUC_NE_lexicon_tags_1 },
    { 0x32bce4c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a8e4c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd744e4c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a52e4cc, SUC_NE_lexicon_tags_2 },
    { 0x7994e4cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e8e4d1, SUC_NE_lexicon_tags_2 },
    { 0x459ce4d2, SUC_NE_lexicon_tags_2 },
    { 0xc324e4d3, SUC_NE_lexicon_tags_2 },
    { 0xbad6e4d4, SUC_NE_lexicon_tags_2 },
    { 0xdaaee4d5, SUC_NE_lexicon_tags_2 },
    { 0xf26ee4d6, SUC_NE_lexicon_tags_2 },
    { 0xa596e4d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f78e4da, SUC_NE_lexicon_tags_2 },
    { 0x56c0e4db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5022e4dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d7ae4df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81ae4e2, SUC_NE_lexicon_tags_18 },
    { 0x7cb8e4e3, SUC_NE_lexicon_tags_24 },
    { 0xed34e4e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fce4e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08ce4ee, SUC_NE_lexicon_tags_7 },
    { 0x9990e4ee, SUC_NE_lexicon_tags_1 },
    { 0xad0ae4f0, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df0e4f5, SUC_NE_lexicon_tags_2 },
    { 0xf79ee4f6, SUC_NE_lexicon_tags_2 },
    { 0x8f80e4f7, SUC_NE_lexicon_tags_2 },
    { 0xe9cae4f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9594e4fa, SUC_NE_lexicon_tags_2 },
    { 0xcbcae4fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ee502, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x884e504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a50e50a, SUC_NE_lexicon_tags_2 },
    { 0x692ce50b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9ce50e, SUC_NE_lexicon_tags_17 },
    { 0xb530e50f, SUC_NE_lexicon_tags_2 },
    { 0x8c78e510, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6332e513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6298e516, SUC_NE_lexicon_tags_13 },
    { 0x284ee517, SUC_NE_lexicon_tags_2 },
    { 0x3940e518, SUC_NE_lexicon_tags_2 },
    { 0xa39ae519, SUC_NE_lexicon_tags_2 },
    { 0x5d2ce519, SUC_NE_lexicon_tags_2 },
    { 0x1c80e518, SUC_NE_lexicon_tags_2 },
    { 0x693ee516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x169ce51e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4706e522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccb8e524, SUC_NE_lexicon_tags_2 },
    { 0x4cb2e525, SUC_NE_lexicon_tags_3 },
    { 0x3fe2e526, SUC_NE_lexicon_tags_2 },
    { 0xf1f8e527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7076e52b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1674e52f, SUC_NE_lexicon_tags_1 },
    { 0x3540e530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d2e533, SUC_NE_lexicon_tags_13 },
    { 0xa3b4e534, SUC_NE_lexicon_tags_2 },
    { 0xf826e535, SUC_NE_lexicon_tags_1 },
    { 0xd9dae536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3736e538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d78e53a, SUC_NE_lexicon_tags_53 },
    { 0x40cce53a, SUC_NE_lexicon_tags_2 },
    { 0x96c2e53c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce6e53f, SUC_NE_lexicon_tags_2 },
    { 0x9196e53f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ce544, SUC_NE_lexicon_tags_2 },
    { 0xd8b8e545, SUC_NE_lexicon_tags_2 },
    { 0x5206e546, SUC_NE_lexicon_tags_2 },
    { 0xd768e547, SUC_NE_lexicon_tags_2 },
    { 0x990e548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ee54c, SUC_NE_lexicon_tags_2 },
    { 0x1256e54d, SUC_NE_lexicon_tags_2 },
    { 0xf85ce54e, SUC_NE_lexicon_tags_2 },
    { 0x418ae54e, SUC_NE_lexicon_tags_2 },
    { 0x9bfee550, SUC_NE_lexicon_tags_2 },
    { 0x3a1ce551, SUC_NE_lexicon_tags_2 },
    { 0xb69ee54f, SUC_NE_lexicon_tags_2 },
    { 0xe174e553, SUC_NE_lexicon_tags_2 },
    { 0xe320e553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f24e556, SUC_NE_lexicon_tags_2 },
    { 0x3ca8e557, SUC_NE_lexicon_tags_2 },
    { 0x185ce558, SUC_NE_lexicon_tags_2 },
    { 0xe994e556, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x7758e55b, SUC_NE_lexicon_tags_2 },
    { 0xe860e55b, SUC_NE_lexicon_tags_2 },
    { 0x56a8e55d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19d0e55f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd8e563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0ce567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb022e56e, SUC_NE_lexicon_tags_2 },
    { 0x6a9ce56e, SUC_NE_lexicon_tags_2 },
    { 0x846e56f, SUC_NE_lexicon_tags_2 },
    { 0x5fc4e571, SUC_NE_lexicon_tags_2 },
    { 0x3738e56e, SUC_NE_lexicon_tags_18 },
    { 0xfa9ee573, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf274e575, SUC_NE_lexicon_tags_2 },
    { 0x5212e576, SUC_NE_lexicon_tags_2 },
    { 0xcfbee577, SUC_NE_lexicon_tags_2 },
    { 0xfb86e578, SUC_NE_lexicon_tags_2 },
    { 0x61ae578, SUC_NE_lexicon_tags_3 },
    { 0x1198e579, SUC_NE_lexicon_tags_2 },
    { 0xed04e577, SUC_NE_lexicon_tags_2 },
    { 0xa85ee576, SUC_NE_lexicon_tags_2 },
    { 0x88d6e57d, SUC_NE_lexicon_tags_2 },
    { 0xbebce576, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56e581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5926e584, SUC_NE_lexicon_tags_2 },
    { 0xf342e585, SUC_NE_lexicon_tags_2 },
    { 0xe718e586, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9496e58a, SUC_NE_lexicon_tags_2 },
    { 0xac88e58b, SUC_NE_lexicon_tags_2 },
    { 0xd7d0e58c, SUC_NE_lexicon_tags_7 },
    { 0x41a4e58d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c4e590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62a8e592, SUC_NE_lexicon_tags_2 },
    { 0xfc4ee593, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818e597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x619ce59a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8a8e59c, SUC_NE_lexicon_tags_2 },
    { 0xdde6e59d, SUC_NE_lexicon_tags_2 },
    { 0x7a5ee59e, SUC_NE_lexicon_tags_2 },
    { 0x4500e59f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xafdce5a1, SUC_NE_lexicon_tags_2 },
    { 0x5b9ee5a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe036e5a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa68e5a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5ae5aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8484e5af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf28e5b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e5b6, SUC_NE_lexicon_tags_2 },
    { 0x621ae5b6, SUC_NE_lexicon_tags_2 },
    { 0x6dcae5b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x512ee5ba, SUC_NE_lexicon_tags_2 },
    { 0xe8fae5ba, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6382e5bf, SUC_NE_lexicon_tags_2 },
    { 0x3296e5bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x685ce5c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28e5c8, SUC_NE_lexicon_tags_2 },
    { 0x5d3ae5c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7c6e5cb, SUC_NE_lexicon_tags_2 },
    { 0xa276e5cb, SUC_NE_lexicon_tags_13 },
    { 0x7210e5cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc412e5d1, SUC_NE_lexicon_tags_2 },
    { 0xc474e5d2, SUC_NE_lexicon_tags_2 },
    { 0xd49ee5d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe162e5d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f0e5da, SUC_NE_lexicon_tags_2 },
    { 0x7604e5da, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2ae5df, SUC_NE_lexicon_tags_2 },
    { 0x767ee5e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1992e5e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a4e5e6, SUC_NE_lexicon_tags_2 },
    { 0x31b0e5e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd2e5e9, SUC_NE_lexicon_tags_2 },
    { 0x5420e5e9, SUC_NE_lexicon_tags_2 },
    { 0xeb3ae5ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd2e5ee, SUC_NE_lexicon_tags_2 },
    { 0xbcdce5ef, SUC_NE_lexicon_tags_2 },
    { 0xc40e5ef, SUC_NE_lexicon_tags_2 },
    { 0x9296e5f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa912e5f6, SUC_NE_lexicon_tags_2 },
    { 0xe6c4e5f7, SUC_NE_lexicon_tags_2 },
    { 0x488ce5f7, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c88e5fa, SUC_NE_lexicon_tags_2 },
    { 0x44eae5fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e5fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ae602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4092e604, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eace607, SUC_NE_lexicon_tags_7 },
    { 0x4a68e607, SUC_NE_lexicon_tags_2 },
    { 0x67fce607, SUC_NE_lexicon_tags_2 },
    { 0x86d2e609, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fee613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb8ae615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9638e617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a1ce619, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab64e61b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd66ee61d, SUC_NE_lexicon_tags_2 },
    { 0x163ce61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1866e623, SUC_NE_lexicon_tags_2 },
    { 0x483ee624, SUC_NE_lexicon_tags_2 },
    { 0xca1ce623, SUC_NE_lexicon_tags_2 },
    { 0xde6ee626, SUC_NE_lexicon_tags_2 },
    { 0x8e2ce627, SUC_NE_lexicon_tags_2 },
    { 0xb090e628, SUC_NE_lexicon_tags_2 },
    { 0xeca8e628, SUC_NE_lexicon_tags_7 },
    { 0xfa6ae62a, SUC_NE_lexicon_tags_2 },
    { 0xab16e62b, SUC_NE_lexicon_tags_2 },
    { 0xb62ae628, SUC_NE_lexicon_tags_3 },
    { 0x9d22e62d, SUC_NE_lexicon_tags_2 },
    { 0xf5ece62e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b26e635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2364e637, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb962e63a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae76e63c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3c76e63e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1ae642, SUC_NE_lexicon_tags_2 },
    { 0xf1a4e642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcda2e645, SUC_NE_lexicon_tags_3 },
    { 0x12cee646, SUC_NE_lexicon_tags_2 },
    { 0x8542e645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fee64a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x234ee64c, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b4ee64e, SUC_NE_lexicon_tags_2 },
    { 0xe162e64f, SUC_NE_lexicon_tags_2 },
    { 0x808ee650, SUC_NE_lexicon_tags_2 },
    { 0x49dee651, SUC_NE_lexicon_tags_2 },
    { 0xfbc4e652, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2e659, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d6e65b, SUC_NE_lexicon_tags_2 },
    { 0xa42ce65c, SUC_NE_lexicon_tags_59 },
    { 0xeb44e65d, SUC_NE_lexicon_tags_2 },
    { 0x12c8e65d, SUC_NE_lexicon_tags_2 },
    { 0x4188e65e, SUC_NE_lexicon_tags_2 },
    { 0xfb44e65c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6e66a, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74ee66f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x688ce671, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9ee673, SUC_NE_lexicon_tags_2 },
    { 0x751ae674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d8e678, SUC_NE_lexicon_tags_2 },
    { 0x149ce678, SUC_NE_lexicon_tags_2 },
    { 0x9172e67a, SUC_NE_lexicon_tags_2 },
    { 0xba1ee67a, SUC_NE_lexicon_tags_81 },
    { 0x69eee679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a42e67e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4962e682, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xed8e684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d2e687, SUC_NE_lexicon_tags_2 },
    { 0x8018e687, SUC_NE_lexicon_tags_1 },
    { 0xc46ce689, SUC_NE_lexicon_tags_2 },
    { 0x4532e68a, SUC_NE_lexicon_tags_2 },
    { 0x7b66e689, SUC_NE_lexicon_tags_2 },
    { 0xcca8e68b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8fee68e, SUC_NE_lexicon_tags_2 },
    { 0x248ce68f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b0e693, SUC_NE_lexicon_tags_1 },
    { 0x3226e694, SUC_NE_lexicon_tags_2 },
    { 0x5156e695, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf20e698, SUC_NE_lexicon_tags_2 },
    { 0xf5dae699, SUC_NE_lexicon_tags_2 },
    { 0x6140e69a, SUC_NE_lexicon_tags_2 },
    { 0x9864e69b, SUC_NE_lexicon_tags_2 },
    { 0x7274e69a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9afae69e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a6e6a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5518e6a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eae6a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8926e6aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x197ee6ac, SUC_NE_lexicon_tags_2 },
    { 0x8cdce6ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x871ee6af, SUC_NE_lexicon_tags_2 },
    { 0x6596e6b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5554e6b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa590e6b7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb724e6bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bf4e6bd, SUC_NE_lexicon_tags_2 },
    { 0x84f6e6bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952e6c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd30e6c7, SUC_NE_lexicon_tags_2 },
    { 0xb75ce6c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbae6cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc272e6cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b78e6d1, SUC_NE_lexicon_tags_2 },
    { 0x2636e6d2, SUC_NE_lexicon_tags_2 },
    { 0x9f5ee6d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9f0e6d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf710e6d7, SUC_NE_lexicon_tags_2 },
    { 0xdcd2e6d8, SUC_NE_lexicon_tags_2 },
    { 0x52fce6d8, SUC_NE_lexicon_tags_2 },
    { 0x8528e6d7, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b6e6dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9140e6e5, SUC_NE_lexicon_tags_2 },
    { 0x6fd2e6e6, SUC_NE_lexicon_tags_8 },
    { 0xa91ce6e7, SUC_NE_lexicon_tags_2 },
    { 0x2390e6e7, SUC_NE_lexicon_tags_34 },
    { 0xed3ce6e7, SUC_NE_lexicon_tags_2 },
    { 0xddb4e6ea, SUC_NE_lexicon_tags_7 },
    { 0xe288e6ea, SUC_NE_lexicon_tags_25 },
    { 0xdb46e6eb, SUC_NE_lexicon_tags_2 },
    { 0xe4e6ea, SUC_NE_lexicon_tags_4 },
    { 0xe844e6ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4ce6f1, SUC_NE_lexicon_tags_2 },
    { 0x1388e6f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c4e6f6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7d26e6f8, SUC_NE_lexicon_tags_2 },
    { 0x7372e6f9, SUC_NE_lexicon_tags_2 },
    { 0x86ee6fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46f2e6fc, SUC_NE_lexicon_tags_2 },
    { 0xb60ee6fd, SUC_NE_lexicon_tags_2 },
    { 0x8688e6fe, SUC_NE_lexicon_tags_2 },
    { 0x4650e6ff, SUC_NE_lexicon_tags_2 },
    { 0xf57ee6ff, SUC_NE_lexicon_tags_2 },
    { 0x3c1ce6fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1372e704, SUC_NE_lexicon_tags_2 },
    { 0xefb2e705, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc02e70f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd90ce711, SUC_NE_lexicon_tags_2 },
    { 0xd69ae712, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb582e715, SUC_NE_lexicon_tags_2 },
    { 0xcca8e715, SUC_NE_lexicon_tags_2 },
    { 0x7a4ee717, SUC_NE_lexicon_tags_2 },
    { 0x3e80e718, SUC_NE_lexicon_tags_2 },
    { 0x5250e717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c80e71d, SUC_NE_lexicon_tags_2 },
    { 0x2804e71e, SUC_NE_lexicon_tags_2 },
    { 0xe6fae71e, SUC_NE_lexicon_tags_2 },
    { 0x2ace720, SUC_NE_lexicon_tags_2 },
    { 0xc626e721, SUC_NE_lexicon_tags_2 },
    { 0xe10ee721, SUC_NE_lexicon_tags_2 },
    { 0x911ee723, SUC_NE_lexicon_tags_2 },
    { 0x56a8e723, SUC_NE_lexicon_tags_2 },
    { 0xdefae725, SUC_NE_lexicon_tags_2 },
    { 0xe79ae723, SUC_NE_lexicon_tags_2 },
    { 0xd788e721, SUC_NE_lexicon_tags_2 },
    { 0x8ef8e728, SUC_NE_lexicon_tags_2 },
    { 0x939ee71f, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xb7dee72b, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x5244e72d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4298e731, SUC_NE_lexicon_tags_2 },
    { 0x8544e732, SUC_NE_lexicon_tags_2 },
    { 0xf6eee733, SUC_NE_lexicon_tags_2 },
    { 0xb644e734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa858e738, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd656e73a, SUC_NE_lexicon_tags_2 },
    { 0xc7c2e73a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4e73e, SUC_NE_lexicon_tags_2 },
    { 0x8428e73f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4060e741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23c6e743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7ce746, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1456e74c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x144ee74e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6426e750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe82e754, SUC_NE_lexicon_tags_17 },
    { 0xfafae754, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde18e758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601ee75c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e08e75e, SUC_NE_lexicon_tags_2 },
    { 0xc416e75f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9718e763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf428e768, SUC_NE_lexicon_tags_2 },
    { 0x1b20e769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x620ee76c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x207ae76e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eae774, SUC_NE_lexicon_tags_2 },
    { 0x9f94e775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6a8e777, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71cae779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dfee77b, SUC_NE_lexicon_tags_1 },
    { 0x1768e77b, SUC_NE_lexicon_tags_58 },
    { 0xa844e77b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc832e77f, SUC_NE_lexicon_tags_2 },
    { 0xa9b2e780, SUC_NE_lexicon_tags_2 },
    { 0xe978e77f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1e2e783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f9ee785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85ae78a, SUC_NE_lexicon_tags_102 },
    { 0x4960e78a, SUC_NE_lexicon_tags_2 },
    { 0xcdc4e78b, SUC_NE_lexicon_tags_2 },
    { 0x1804e78d, SUC_NE_lexicon_tags_2 },
    { 0xa0d8e78b, SUC_NE_lexicon_tags_2 },
    { 0xff5ee78e, SUC_NE_lexicon_tags_2 },
    { 0x941ce78d, SUC_NE_lexicon_tags_2 },
    { 0x4986e78d, SUC_NE_lexicon_tags_2 },
    { 0x271ce78c, SUC_NE_lexicon_tags_2 },
    { 0xd20e790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa02ae795, SUC_NE_lexicon_tags_2 },
    { 0x1df4e796, SUC_NE_lexicon_tags_2 },
    { 0x1daee796, SUC_NE_lexicon_tags_17 },
    { 0x4486e797, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eee79b, SUC_NE_lexicon_tags_2 },
    { 0x994e79c, SUC_NE_lexicon_tags_2 },
    { 0x7896e79c, SUC_NE_lexicon_tags_2 },
    { 0x85ce79e, SUC_NE_lexicon_tags_2 },
    { 0xb294e79e, SUC_NE_lexicon_tags_2 },
    { 0xe5cae79f, SUC_NE_lexicon_tags_2 },
    { 0x451ae7a1, SUC_NE_lexicon_tags_2 },
    { 0xd0c6e7a2, SUC_NE_lexicon_tags_2 },
    { 0x1b5ae79f, SUC_NE_lexicon_tags_2 },
    { 0xa294e79d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf76ae7a6, SUC_NE_lexicon_tags_2 },
    { 0x6440e7a7, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f8e7ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5210e7ad, SUC_NE_lexicon_tags_2 },
    { 0x6788e7ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106e7b4, SUC_NE_lexicon_tags_2 },
    { 0xb56ae7b5, SUC_NE_lexicon_tags_2 },
    { 0xd5a2e7b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f5ae7b8, SUC_NE_lexicon_tags_2 },
    { 0x4bfae7b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b0e7bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781ee7c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1392e7c5, SUC_NE_lexicon_tags_2 },
    { 0x9d28e7c6, SUC_NE_lexicon_tags_2 },
    { 0xd4f4e7c6, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7736e7cf, SUC_NE_lexicon_tags_2 },
    { 0xb984e7d0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69ce7d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ae7d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x788ee7db, SUC_NE_lexicon_tags_2 },
    { 0x39aae7dc, SUC_NE_lexicon_tags_2 },
    { 0x290e7dd, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b8e7e2, SUC_NE_lexicon_tags_2 },
    { 0x4f24e7e3, SUC_NE_lexicon_tags_2 },
    { 0x5f1ce7e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a2e7e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x886ce7ec, SUC_NE_lexicon_tags_2 },
    { 0xf802e7ec, SUC_NE_lexicon_tags_2 },
    { 0x4564e7ee, SUC_NE_lexicon_tags_2 },
    { 0xc528e7ee, SUC_NE_lexicon_tags_2 },
    { 0xbe9ee7ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd074e7f8, SUC_NE_lexicon_tags_2 },
    { 0x86a0e7f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47cce7fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc708e7fe, SUC_NE_lexicon_tags_2 },
    { 0x69d4e7ff, SUC_NE_lexicon_tags_2 },
    { 0x5914e7ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1138e803, SUC_NE_lexicon_tags_2 },
    { 0xe8cce803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39ce807, SUC_NE_lexicon_tags_2 },
    { 0x7f04e807, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fce80b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd72e80d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x35a0e80f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cae814, SUC_NE_lexicon_tags_2 },
    { 0x95a0e815, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5a2e817, SUC_NE_lexicon_tags_3 },
    { 0x1f44e818, SUC_NE_lexicon_tags_2 },
    { 0x1c8e818, SUC_NE_lexicon_tags_2 },
    { 0xc536e81a, SUC_NE_lexicon_tags_2 },
    { 0x48aae81a, SUC_NE_lexicon_tags_2 },
    { 0xd72e81c, SUC_NE_lexicon_tags_2 },
    { 0xaf8ae81c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6eae81f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc32e821, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50e824, SUC_NE_lexicon_tags_2 },
    { 0x1f70e824, SUC_NE_lexicon_tags_2 },
    { 0x578ce824, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5ae829, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad14e82b, SUC_NE_lexicon_tags_2 },
    { 0xfc2ce82c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1ce82f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc5ae831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae2e835, SUC_NE_lexicon_tags_2 },
    { 0xa018e835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9c0e838, SUC_NE_lexicon_tags_2 },
    { 0x442ae839, SUC_NE_lexicon_tags_2 },
    { 0xc2dae839, SUC_NE_lexicon_tags_2 },
    { 0x1884e83a, SUC_NE_lexicon_tags_2 },
    { 0xa910e83b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f8e840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515ce848, SUC_NE_lexicon_tags_7 },
    { 0xbe76e849, SUC_NE_lexicon_tags_4 },
    { 0xf65ce84a, SUC_NE_lexicon_tags_2 },
    { 0x5f4ae84b, SUC_NE_lexicon_tags_2 },
    { 0x974e84c, SUC_NE_lexicon_tags_2 },
    { 0xa618e84c, SUC_NE_lexicon_tags_2 },
    { 0x1bdee84b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7ee856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1aae858, SUC_NE_lexicon_tags_113 },
    { 0, NULL },
    { 0xb1bee85a, SUC_NE_lexicon_tags_2 },
    { 0x4f9ce85b, SUC_NE_lexicon_tags_2 },
    { 0x514ee85c, SUC_NE_lexicon_tags_2 },
    { 0x9e38e85d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a30e860, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49ee863, SUC_NE_lexicon_tags_2 },
    { 0x177ae864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3688e868, SUC_NE_lexicon_tags_2 },
    { 0x62eee868, SUC_NE_lexicon_tags_2 },
    { 0x1f9ee869, SUC_NE_lexicon_tags_2 },
    { 0x198ae86a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x168ce86d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e875, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xace0e877, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecce87a, SUC_NE_lexicon_tags_2 },
    { 0xcd0ce87a, SUC_NE_lexicon_tags_2 },
    { 0xd11ee87a, SUC_NE_lexicon_tags_2 },
    { 0x3c3ce87a, SUC_NE_lexicon_tags_13 },
    { 0xf5a4e87e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1a6e880, SUC_NE_lexicon_tags_2 },
    { 0x8ea8e881, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8ae884, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6192e888, SUC_NE_lexicon_tags_2 },
    { 0xffd0e888, SUC_NE_lexicon_tags_2 },
    { 0xea5ce88a, SUC_NE_lexicon_tags_2 },
    { 0x28fae88a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f0e893, SUC_NE_lexicon_tags_2 },
    { 0x790ee894, SUC_NE_lexicon_tags_4 },
    { 0x8212e894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x800e897, SUC_NE_lexicon_tags_2 },
    { 0xc256e898, SUC_NE_lexicon_tags_2 },
    { 0x4566e899, SUC_NE_lexicon_tags_2 },
    { 0xc9aee899, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba4e89e, SUC_NE_lexicon_tags_2 },
    { 0x18dae89e, SUC_NE_lexicon_tags_2 },
    { 0x6edae8a0, SUC_NE_lexicon_tags_2 },
    { 0x9d4ce89e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a0e8a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c60e8a7, SUC_NE_lexicon_tags_2 },
    { 0xb254e8a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd34e8aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2252e8ad, SUC_NE_lexicon_tags_2 },
    { 0xdf6e8ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf16e8b0, SUC_NE_lexicon_tags_2 },
    { 0xdcfce8b1, SUC_NE_lexicon_tags_2 },
    { 0x26ee8b2, SUC_NE_lexicon_tags_2 },
    { 0xc194e8b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb77ee8b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8006e8b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcce0e8b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed24e8bf, SUC_NE_lexicon_tags_2 },
    { 0xcba4e8bf, SUC_NE_lexicon_tags_2 },
    { 0x8122e8c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc32e8c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959ae8c7, SUC_NE_lexicon_tags_4 },
    { 0x349ee8c7, SUC_NE_lexicon_tags_2 },
    { 0xa70e8c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa726e8cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6848e8d1, SUC_NE_lexicon_tags_4 },
    { 0x9620e8d2, SUC_NE_lexicon_tags_2 },
    { 0x525ae8d3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c2e8d7, SUC_NE_lexicon_tags_21 },
    { 0x2f62e8d8, SUC_NE_lexicon_tags_2 },
    { 0x2c28e8d9, SUC_NE_lexicon_tags_2 },
    { 0xee58e8da, SUC_NE_lexicon_tags_2 },
    { 0x3c7ee8da, SUC_NE_lexicon_tags_2 },
    { 0xdc5ee8dc, SUC_NE_lexicon_tags_2 },
    { 0x5a88e8dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe314e8e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d00e8e2, SUC_NE_lexicon_tags_2 },
    { 0xaa68e8e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd76e8e5, SUC_NE_lexicon_tags_3 },
    { 0xa5f6e8e6, SUC_NE_lexicon_tags_4 },
    { 0xbf3ce8e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d0e8ed, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb4e8f0, SUC_NE_lexicon_tags_4 },
    { 0xa92e8f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850e8f5, SUC_NE_lexicon_tags_2 },
    { 0xf15ae8f5, SUC_NE_lexicon_tags_2 },
    { 0x7790e8f7, SUC_NE_lexicon_tags_2 },
    { 0xe104e8f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f4e8fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef0e901, SUC_NE_lexicon_tags_2 },
    { 0x87f0e902, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f48e905, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a62e908, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x659ee90a, SUC_NE_lexicon_tags_4 },
    { 0x421ee90a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c0e910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4058e912, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff4e916, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a60e918, SUC_NE_lexicon_tags_14 },
    { 0x66d2e919, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fe2e91b, SUC_NE_lexicon_tags_2 },
    { 0x2c2ce91b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1ee922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe48e92c, SUC_NE_lexicon_tags_2 },
    { 0xfb60e92c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd922e930, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa0e937, SUC_NE_lexicon_tags_2 },
    { 0x972ee938, SUC_NE_lexicon_tags_2 },
    { 0xbbcae939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdae93b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9502e93d, SUC_NE_lexicon_tags_2 },
    { 0xd132e93e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x392ee941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b0ae943, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aee947, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f2ce949, SUC_NE_lexicon_tags_2 },
    { 0xfb22e949, SUC_NE_lexicon_tags_2 },
    { 0x420ae94b, SUC_NE_lexicon_tags_2 },
    { 0x7096e94c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be6e94f, SUC_NE_lexicon_tags_4 },
    { 0xb866e950, SUC_NE_lexicon_tags_2 },
    { 0x75eee951, SUC_NE_lexicon_tags_2 },
    { 0xbf9ae951, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x59a8e954, SUC_NE_lexicon_tags_2 },
    { 0x16ce955, SUC_NE_lexicon_tags_2 },
    { 0xc558e956, SUC_NE_lexicon_tags_2 },
    { 0x550ae955, SUC_NE_lexicon_tags_2 },
    { 0x2a88e958, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2edce95a, SUC_NE_lexicon_tags_2 },
    { 0x7c96e95b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7946e960, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb44e965, SUC_NE_lexicon_tags_2 },
    { 0xafee966, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xb606e969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f2e96d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabb2e96f, SUC_NE_lexicon_tags_52 },
    { 0xe16e970, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b50e972, SUC_NE_lexicon_tags_2 },
    { 0xbfd0e972, SUC_NE_lexicon_tags_2 },
    { 0xb01ee972, SUC_NE_lexicon_tags_2 },
    { 0x1de8e975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3224e978, SUC_NE_lexicon_tags_2 },
    { 0xbf6ee979, SUC_NE_lexicon_tags_2 },
    { 0xfe02e979, SUC_NE_lexicon_tags_2 },
    { 0x9a3ee97b, SUC_NE_lexicon_tags_2 },
    { 0x7036e97c, SUC_NE_lexicon_tags_2 },
    { 0xaa26e97d, SUC_NE_lexicon_tags_0 },
    { 0x95cce97c, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f36e985, SUC_NE_lexicon_tags_2 },
    { 0x9074e985, SUC_NE_lexicon_tags_19 },
    { 0x579ee987, SUC_NE_lexicon_tags_2 },
    { 0x9294e988, SUC_NE_lexicon_tags_2 },
    { 0x3a42e989, SUC_NE_lexicon_tags_2 },
    { 0xcd8e98a, SUC_NE_lexicon_tags_2 },
    { 0xb414e987, SUC_NE_lexicon_tags_2 },
    { 0x71d6e988, SUC_NE_lexicon_tags_2 },
    { 0x1382e98d, SUC_NE_lexicon_tags_2 },
    { 0x1436e98e, SUC_NE_lexicon_tags_2 },
    { 0x36d6e98f, SUC_NE_lexicon_tags_2 },
    { 0xaaaae990, SUC_NE_lexicon_tags_2 },
    { 0x3f3ae990, SUC_NE_lexicon_tags_2 },
    { 0xa554e990, SUC_NE_lexicon_tags_2 },
    { 0x21a6e98c, SUC_NE_lexicon_tags_2 },
    { 0x5fa8e994, SUC_NE_lexicon_tags_2 },
    { 0xa2c6e98a, SUC_NE_lexicon_tags_2 },
    { 0x676ce996, SUC_NE_lexicon_tags_2 },
    { 0xa2ce997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda12e99c, SUC_NE_lexicon_tags_7 },
    { 0x1a86e99c, SUC_NE_lexicon_tags_2 },
    { 0x8a86e99e, SUC_NE_lexicon_tags_2 },
    { 0xa652e99e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e4e9a3, SUC_NE_lexicon_tags_2 },
    { 0x8852e9a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ae9ac, SUC_NE_lexicon_tags_2 },
    { 0x3dce9ad, SUC_NE_lexicon_tags_13 },
    { 0xaa4ce9ac, SUC_NE_lexicon_tags_2 },
    { 0x7daee9af, SUC_NE_lexicon_tags_2 },
    { 0xd458e9b0, SUC_NE_lexicon_tags_4 },
    { 0xfbd0e9b1, SUC_NE_lexicon_tags_2 },
    { 0x5580e9b2, SUC_NE_lexicon_tags_2 },
    { 0x4dd2e9b3, SUC_NE_lexicon_tags_2 },
    { 0x4acae9b4, SUC_NE_lexicon_tags_2 },
    { 0x66cce9b3, SUC_NE_lexicon_tags_2 },
    { 0x3ff4e9b6, SUC_NE_lexicon_tags_2 },
    { 0xcdfce9b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8574e9b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666ce9c1, SUC_NE_lexicon_tags_2 },
    { 0x1b04e9c2, SUC_NE_lexicon_tags_2 },
    { 0xafdae9c2, SUC_NE_lexicon_tags_2 },
    { 0x6878e9c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9feae9c7, SUC_NE_lexicon_tags_30 },
    { 0x804ae9c8, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1540e9cb, SUC_NE_lexicon_tags_2 },
    { 0x52aae9cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e92e9d5, SUC_NE_lexicon_tags_2 },
    { 0xbe04e9d6, SUC_NE_lexicon_tags_2 },
    { 0x341ae9d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b9ce9d9, SUC_NE_lexicon_tags_2 },
    { 0xe466e9d9, SUC_NE_lexicon_tags_2 },
    { 0xe36ce9db, SUC_NE_lexicon_tags_2 },
    { 0xbef8e9db, SUC_NE_lexicon_tags_17 },
    { 0x373ae9dd, SUC_NE_lexicon_tags_2 },
    { 0x31ae9de, SUC_NE_lexicon_tags_2 },
    { 0xfe08e9df, SUC_NE_lexicon_tags_2 },
    { 0x545ae9df, SUC_NE_lexicon_tags_2 },
    { 0xcb10e9e1, SUC_NE_lexicon_tags_2 },
    { 0x8fc2e9df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f8e9e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd526e9ea, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6878e9ed, SUC_NE_lexicon_tags_2 },
    { 0x6f6ae9ee, SUC_NE_lexicon_tags_2 },
    { 0xda42e9ef, SUC_NE_lexicon_tags_2 },
    { 0xf088e9ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe234e9f3, SUC_NE_lexicon_tags_2 },
    { 0x7632e9f3, SUC_NE_lexicon_tags_2 },
    { 0xe59ce9f5, SUC_NE_lexicon_tags_2 },
    { 0x17ee9f5, SUC_NE_lexicon_tags_2 },
    { 0xfc82e9f7, SUC_NE_lexicon_tags_2 },
    { 0x6c1ee9f8, SUC_NE_lexicon_tags_7 },
    { 0xbb50e9f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636ea00, SUC_NE_lexicon_tags_80 },
    { 0, NULL },
    { 0x738aea02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2410ea06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f68ea0a, SUC_NE_lexicon_tags_2 },
    { 0x5ab2ea0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f06ea0d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ccea11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf194ea14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2aea1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242aea20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7aea23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dceea28, SUC_NE_lexicon_tags_2 },
    { 0xd9a8ea29, SUC_NE_lexicon_tags_2 },
    { 0xa7a2ea2a, SUC_NE_lexicon_tags_2 },
    { 0xc796ea28, SUC_NE_lexicon_tags_2 },
    { 0xbf30ea2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0eea31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebbaea33, SUC_NE_lexicon_tags_2 },
    { 0x167cea33, SUC_NE_lexicon_tags_2 },
    { 0xcb9aea33, SUC_NE_lexicon_tags_2 },
    { 0xbd90ea36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222ea3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82ea3f, SUC_NE_lexicon_tags_1 },
    { 0xd1dcea40, SUC_NE_lexicon_tags_2 },
    { 0xa944ea40, SUC_NE_lexicon_tags_7 },
    { 0x5dbeea42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0baea44, SUC_NE_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebcea49, SUC_NE_lexicon_tags_7 },
    { 0x781eea4a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3aea4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e02ea52, SUC_NE_lexicon_tags_2 },
    { 0xae24ea52, SUC_NE_lexicon_tags_2 },
    { 0x4124ea54, SUC_NE_lexicon_tags_2 },
    { 0x7cbcea55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1eea58, SUC_NE_lexicon_tags_2 },
    { 0x2e2aea59, SUC_NE_lexicon_tags_2 },
    { 0x9140ea59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x541aea5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de6ea66, SUC_NE_lexicon_tags_2 },
    { 0x96e6ea66, SUC_NE_lexicon_tags_2 },
    { 0x2868ea68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4640ea6f, SUC_NE_lexicon_tags_2 },
    { 0xf162ea70, SUC_NE_lexicon_tags_2 },
    { 0x972aea71, SUC_NE_lexicon_tags_2 },
    { 0xa874ea71, SUC_NE_lexicon_tags_2 },
    { 0xb10cea73, SUC_NE_lexicon_tags_2 },
    { 0x8ee4ea74, SUC_NE_lexicon_tags_7 },
    { 0x5078ea73, SUC_NE_lexicon_tags_7 },
    { 0xca12ea76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc814ea7a, SUC_NE_lexicon_tags_28 },
    { 0x4566ea7b, SUC_NE_lexicon_tags_4 },
    { 0x4384ea7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4816ea7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368ea83, SUC_NE_lexicon_tags_2 },
    { 0xa620ea83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59cea87, SUC_NE_lexicon_tags_2 },
    { 0x6758ea88, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd94ea8a, SUC_NE_lexicon_tags_2 },
    { 0x51faea8b, SUC_NE_lexicon_tags_2 },
    { 0xcac6ea8b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf06ea8f, SUC_NE_lexicon_tags_2 },
    { 0x21f2ea90, SUC_NE_lexicon_tags_2 },
    { 0xee94ea8f, SUC_NE_lexicon_tags_2 },
    { 0x6a2aea90, SUC_NE_lexicon_tags_2 },
    { 0xdac2ea90, SUC_NE_lexicon_tags_2 },
    { 0x8402ea94, SUC_NE_lexicon_tags_2 },
    { 0xc0a2ea95, SUC_NE_lexicon_tags_2 },
    { 0x81aea96, SUC_NE_lexicon_tags_2 },
    { 0x9b4ea91, SUC_NE_lexicon_tags_2 },
    { 0xd4aaea98, SUC_NE_lexicon_tags_2 },
    { 0xd2d0ea99, SUC_NE_lexicon_tags_2 },
    { 0xb04cea9a, SUC_NE_lexicon_tags_2 },
    { 0xb690ea9b, SUC_NE_lexicon_tags_2 },
    { 0x7f88ea9c, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682eaa1, SUC_NE_lexicon_tags_2 },
    { 0x77beeaa2, SUC_NE_lexicon_tags_2 },
    { 0xba60eaa2, SUC_NE_lexicon_tags_2 },
    { 0x808ceaa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb140eaa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x894eaa9, SUC_NE_lexicon_tags_2 },
    { 0x5f9ceaaa, SUC_NE_lexicon_tags_2 },
    { 0xd3cceaaa, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ceab0, SUC_NE_lexicon_tags_2 },
    { 0x66bceab1, SUC_NE_lexicon_tags_2 },
    { 0xe81eeab0, SUC_NE_lexicon_tags_2 },
    { 0xffd8eab3, SUC_NE_lexicon_tags_2 },
    { 0x5f90eab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef98eab9, SUC_NE_lexicon_tags_2 },
    { 0xf2a6eaba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5696eabe, SUC_NE_lexicon_tags_2 },
    { 0x58d8eabf, SUC_NE_lexicon_tags_2 },
    { 0x97a0eabf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d32eac3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1faeac6, SUC_NE_lexicon_tags_2 },
    { 0x43e0eac6, SUC_NE_lexicon_tags_2 },
    { 0xc816eac8, SUC_NE_lexicon_tags_2 },
    { 0x1eb2eac9, SUC_NE_lexicon_tags_2 },
    { 0x804aeac8, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x946aeacc, SUC_NE_lexicon_tags_2 },
    { 0x91e6eacd, SUC_NE_lexicon_tags_2 },
    { 0xf590eacd, SUC_NE_lexicon_tags_2 },
    { 0xa74eeace, SUC_NE_lexicon_tags_2 },
    { 0x74f2ead0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ceeadc, SUC_NE_lexicon_tags_2 },
    { 0xefb8eadc, SUC_NE_lexicon_tags_4 },
    { 0x1588eade, SUC_NE_lexicon_tags_2 },
    { 0x3918eade, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x54daeae1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb91ceae3, SUC_NE_lexicon_tags_4 },
    { 0x3efeeae4, SUC_NE_lexicon_tags_2 },
    { 0x8aeaeae5, SUC_NE_lexicon_tags_2 },
    { 0x45aeeae3, SUC_NE_lexicon_tags_2 },
    { 0x50bceae7, SUC_NE_lexicon_tags_2 },
    { 0x6c66eae4, SUC_NE_lexicon_tags_2 },
    { 0xc2a6eae9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x893ceaec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf984eaef, SUC_NE_lexicon_tags_4 },
    { 0xa4b6eaef, SUC_NE_lexicon_tags_2 },
    { 0x2ef0eaef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85eeeaf3, SUC_NE_lexicon_tags_7 },
    { 0x2040eaf4, SUC_NE_lexicon_tags_3 },
    { 0x9310eaf5, SUC_NE_lexicon_tags_2 },
    { 0xf6f8eaf6, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae10eafb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1338eafe, SUC_NE_lexicon_tags_2 },
    { 0xdc6eeaff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6eb0e, SUC_NE_lexicon_tags_2 },
    { 0x184eeb0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7eeb15, SUC_NE_lexicon_tags_2 },
    { 0xcfbeeb16, SUC_NE_lexicon_tags_2 },
    { 0xaa9eeb17, SUC_NE_lexicon_tags_2 },
    { 0xe22ceb18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d18eb1b, SUC_NE_lexicon_tags_2 },
    { 0xff24eb1c, SUC_NE_lexicon_tags_7 },
    { 0x817eeb1c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81feeb23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9eeb28, SUC_NE_lexicon_tags_45 },
    { 0xa81eeb28, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc332eb31, SUC_NE_lexicon_tags_2 },
    { 0x8594eb32, SUC_NE_lexicon_tags_1 },
    { 0x591ceb33, SUC_NE_lexicon_tags_2 },
    { 0x360ceb34, SUC_NE_lexicon_tags_2 },
    { 0x3ab8eb35, SUC_NE_lexicon_tags_2 },
    { 0x2d42eb35, SUC_NE_lexicon_tags_2 },
    { 0x5296eb36, SUC_NE_lexicon_tags_17 },
    { 0x14a6eb38, SUC_NE_lexicon_tags_25 },
    { 0x7ddceb39, SUC_NE_lexicon_tags_2 },
    { 0x144aeb3a, SUC_NE_lexicon_tags_2 },
    { 0xb69aeb3a, SUC_NE_lexicon_tags_3 },
    { 0x53f4eb3c, SUC_NE_lexicon_tags_2 },
    { 0x9dc2eb31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f94eb3f, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x85e6eb41, SUC_NE_lexicon_tags_2 },
    { 0x5b8eb41, SUC_NE_lexicon_tags_2 },
    { 0x2a8ceb41, SUC_NE_lexicon_tags_2 },
    { 0x1106eb42, SUC_NE_lexicon_tags_2 },
    { 0xc370eb45, SUC_NE_lexicon_tags_2 },
    { 0x7530eb46, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xe1daeb48, SUC_NE_lexicon_tags_2 },
    { 0xc576eb49, SUC_NE_lexicon_tags_2 },
    { 0x33f8eb4a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d84eb4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x856aeb51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b6eeb53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dceb5f, SUC_NE_lexicon_tags_24 },
    { 0x54f0eb5f, SUC_NE_lexicon_tags_2 },
    { 0x702aeb61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf322eb69, SUC_NE_lexicon_tags_4 },
    { 0xaa90eb69, SUC_NE_lexicon_tags_2 },
    { 0x243ceb6a, SUC_NE_lexicon_tags_2 },
    { 0xb030eb6c, SUC_NE_lexicon_tags_2 },
    { 0x95ceeb6a, SUC_NE_lexicon_tags_2 },
    { 0x7590eb6e, SUC_NE_lexicon_tags_10 },
    { 0x7634eb6f, SUC_NE_lexicon_tags_4 },
    { 0xcfa2eb6f, SUC_NE_lexicon_tags_2 },
    { 0x12d2eb71, SUC_NE_lexicon_tags_2 },
    { 0x4878eb72, SUC_NE_lexicon_tags_2 },
    { 0x8e40eb71, SUC_NE_lexicon_tags_2 },
    { 0x2556eb71, SUC_NE_lexicon_tags_2 },
    { 0xd6a6eb74, SUC_NE_lexicon_tags_2 },
    { 0x89e0eb76, SUC_NE_lexicon_tags_2 },
    { 0x7c12eb76, SUC_NE_lexicon_tags_2 },
    { 0x6e8aeb76, SUC_NE_lexicon_tags_2 },
    { 0x7d50eb76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f0eb7e, SUC_NE_lexicon_tags_3 },
    { 0xccceb7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf614eb85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf08eb87, SUC_NE_lexicon_tags_2 },
    { 0xe934eb88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fd8eb8a, SUC_NE_lexicon_tags_2 },
    { 0x9d4aeb8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29b6eb8d, SUC_NE_lexicon_tags_2 },
    { 0x1d76eb8d, SUC_NE_lexicon_tags_2 },
    { 0xd682eb8f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xe9c2eb91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed44eb93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb424eb96, SUC_NE_lexicon_tags_2 },
    { 0xa488eb96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d44eb99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd158eb9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8aeb9f, SUC_NE_lexicon_tags_2 },
    { 0x99deeba0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb06ceba2, SUC_NE_lexicon_tags_2 },
    { 0xbadaeba2, SUC_NE_lexicon_tags_3 },
    { 0x4386eba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee8eba7, SUC_NE_lexicon_tags_19 },
    { 0x5cceeba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf16ebaa, SUC_NE_lexicon_tags_2 },
    { 0x859cebaa, SUC_NE_lexicon_tags_2 },
    { 0xed9aebab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x328eebae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e68ebb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e6ebb7, SUC_NE_lexicon_tags_2 },
    { 0x85f8ebb7, SUC_NE_lexicon_tags_2 },
    { 0xe50eebb8, SUC_NE_lexicon_tags_2 },
    { 0xb258ebb8, SUC_NE_lexicon_tags_2 },
    { 0x9dd6ebb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce56ebc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5920ebc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeceebc5, SUC_NE_lexicon_tags_2 },
    { 0x30c6ebc6, SUC_NE_lexicon_tags_7 },
    { 0xd1b8ebc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc162ebcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x474aebd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf44cebd2, SUC_NE_lexicon_tags_2 },
    { 0xc89eebd2, SUC_NE_lexicon_tags_2 },
    { 0x8b5aebd2, SUC_NE_lexicon_tags_7 },
    { 0x73e0ebd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7556ebd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562ebdb, SUC_NE_lexicon_tags_2 },
    { 0x68b0ebdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d8ebdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d28ebe3, SUC_NE_lexicon_tags_3 },
    { 0x7284ebe4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6ebeb, SUC_NE_lexicon_tags_2 },
    { 0x532eebeb, SUC_NE_lexicon_tags_2 },
    { 0xf3a6ebeb, SUC_NE_lexicon_tags_2 },
    { 0xd80ebee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc4ebf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6d2ebf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b0ebf7, SUC_NE_lexicon_tags_2 },
    { 0xd166ebf8, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7eebfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x305aec01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc26aec03, SUC_NE_lexicon_tags_83 },
    { 0, NULL },
    { 0xd690ec05, SUC_NE_lexicon_tags_2 },
    { 0xabcaec05, SUC_NE_lexicon_tags_2 },
    { 0xa346ec07, SUC_NE_lexicon_tags_2 },
    { 0xd2a6ec07, SUC_NE_lexicon_tags_2 },
    { 0xe1fcec06, SUC_NE_lexicon_tags_2 },
    { 0x320ec08, SUC_NE_lexicon_tags_2 },
    { 0x9956ec0b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b82ec0e, SUC_NE_lexicon_tags_2 },
    { 0xdfa6ec0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ccec17, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85aec1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd130ec1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd260ec20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed84ec23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa6ec25, SUC_NE_lexicon_tags_2 },
    { 0x14ceec25, SUC_NE_lexicon_tags_2 },
    { 0x7012ec27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3efaec29, SUC_NE_lexicon_tags_19 },
    { 0xd5d8ec2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf19aec2c, SUC_NE_lexicon_tags_7 },
    { 0x4276ec2c, SUC_NE_lexicon_tags_2 },
    { 0xaeeec2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb076ec31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4eec34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x382eec36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38c0ec38, SUC_NE_lexicon_tags_2 },
    { 0xd626ec38, SUC_NE_lexicon_tags_2 },
    { 0xce92ec3a, SUC_NE_lexicon_tags_2 },
    { 0x6174ec3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbc8ec3d, SUC_NE_lexicon_tags_2 },
    { 0xa142ec3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1110ec42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7fcec44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x530eec47, SUC_NE_lexicon_tags_2 },
    { 0x7ac4ec48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e0cec4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f04ec4c, SUC_NE_lexicon_tags_2 },
    { 0xd36aec4d, SUC_NE_lexicon_tags_2 },
    { 0x1456ec4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ec52, SUC_NE_lexicon_tags_19 },
    { 0x8f2cec52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0eec58, SUC_NE_lexicon_tags_2 },
    { 0xbe94ec58, SUC_NE_lexicon_tags_2 },
    { 0xbc1cec5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf580ec5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8aec64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e86ec66, SUC_NE_lexicon_tags_2 },
    { 0xecf6ec67, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x5dd6ec69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x804eec6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x991eec6e, SUC_NE_lexicon_tags_2 },
    { 0x19a6ec6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8ec72, SUC_NE_lexicon_tags_5 },
    { 0xca8eec72, SUC_NE_lexicon_tags_2 },
    { 0x235aec74, SUC_NE_lexicon_tags_4 },
    { 0x3410ec75, SUC_NE_lexicon_tags_4 },
    { 0x2bf0ec76, SUC_NE_lexicon_tags_2 },
    { 0x3f1eec77, SUC_NE_lexicon_tags_2 },
    { 0x8da6ec76, SUC_NE_lexicon_tags_2 },
    { 0x1924ec76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3556ec7d, SUC_NE_lexicon_tags_4 },
    { 0x7228ec7e, SUC_NE_lexicon_tags_2 },
    { 0xa5c8ec7f, SUC_NE_lexicon_tags_2 },
    { 0x4548ec7d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778cec87, SUC_NE_lexicon_tags_2 },
    { 0x279eec87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dc6ec8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831cec90, SUC_NE_lexicon_tags_2 },
    { 0xc616ec91, SUC_NE_lexicon_tags_4 },
    { 0xceb8ec91, SUC_NE_lexicon_tags_2 },
    { 0x43beec93, SUC_NE_lexicon_tags_2 },
    { 0xd442ec93, SUC_NE_lexicon_tags_2 },
    { 0x16d6ec94, SUC_NE_lexicon_tags_2 },
    { 0x6d46ec96, SUC_NE_lexicon_tags_2 },
    { 0x9188ec90, SUC_NE_lexicon_tags_2 },
    { 0xf020ec91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333ceca2, SUC_NE_lexicon_tags_17 },
    { 0xae46eca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f6eca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd282eca9, SUC_NE_lexicon_tags_2 },
    { 0xcaacecaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca32ecac, SUC_NE_lexicon_tags_2 },
    { 0xa23aecac, SUC_NE_lexicon_tags_2 },
    { 0x6828ecad, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xcaf4ecb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7aecb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdba8ecb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09eecbb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec8ecc0, SUC_NE_lexicon_tags_2 },
    { 0x6062ecc1, SUC_NE_lexicon_tags_2 },
    { 0x9468ecc2, SUC_NE_lexicon_tags_2 },
    { 0x4534ecc0, SUC_NE_lexicon_tags_3 },
    { 0xc8a2ecc4, SUC_NE_lexicon_tags_2 },
    { 0x46d0ecc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34daecc8, SUC_NE_lexicon_tags_2 },
    { 0x3474ecc9, SUC_NE_lexicon_tags_2 },
    { 0x224aecca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4ecd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d6ecd7, SUC_NE_lexicon_tags_2 },
    { 0xba72ecd8, SUC_NE_lexicon_tags_2 },
    { 0x8c22ecd9, SUC_NE_lexicon_tags_3 },
    { 0xdd1eecda, SUC_NE_lexicon_tags_24 },
    { 0xa98eecd9, SUC_NE_lexicon_tags_2 },
    { 0xa1ceecdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6ece0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fcece3, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe08ece8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4992eceb, SUC_NE_lexicon_tags_2 },
    { 0x997ceceb, SUC_NE_lexicon_tags_2 },
    { 0xe290eceb, SUC_NE_lexicon_tags_2 },
    { 0xb46cecee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb4aecf0, SUC_NE_lexicon_tags_2 },
    { 0xd9f4ecf0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8f7aecf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb38ecf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbfcecf9, SUC_NE_lexicon_tags_3 },
    { 0xf162ecf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9116ecfc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23ced00, SUC_NE_lexicon_tags_2 },
    { 0x60a6ed00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2e4ed03, SUC_NE_lexicon_tags_2 },
    { 0xd862ed03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4192ed06, SUC_NE_lexicon_tags_30 },
    { 0x8264ed07, SUC_NE_lexicon_tags_2 },
    { 0x680aed08, SUC_NE_lexicon_tags_2 },
    { 0xb412ed06, SUC_NE_lexicon_tags_2 },
    { 0x694ed07, SUC_NE_lexicon_tags_2 },
    { 0xbe2aed0a, SUC_NE_lexicon_tags_2 },
    { 0x30d2ed0c, SUC_NE_lexicon_tags_2 },
    { 0x9f36ed0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99eed12, SUC_NE_lexicon_tags_2 },
    { 0x4ec4ed12, SUC_NE_lexicon_tags_2 },
    { 0x710eed14, SUC_NE_lexicon_tags_2 },
    { 0x7b74ed15, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5192ed17, SUC_NE_lexicon_tags_2 },
    { 0xf486ed18, SUC_NE_lexicon_tags_2 },
    { 0x6974ed17, SUC_NE_lexicon_tags_0 },
    { 0xa180ed1a, SUC_NE_lexicon_tags_2 },
    { 0x19baed1b, SUC_NE_lexicon_tags_2 },
    { 0x229aed1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d24ed1f, SUC_NE_lexicon_tags_2 },
    { 0x96aed20, SUC_NE_lexicon_tags_2 },
    { 0xee80ed20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x511ced23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaced29, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0ced2c, SUC_NE_lexicon_tags_2 },
    { 0xbcbced2d, SUC_NE_lexicon_tags_2 },
    { 0xcebaed2d, SUC_NE_lexicon_tags_2 },
    { 0x37c6ed2f, SUC_NE_lexicon_tags_2 },
    { 0x5520ed2f, SUC_NE_lexicon_tags_13 },
    { 0x7722ed2f, SUC_NE_lexicon_tags_2 },
    { 0x7fa8ed2c, SUC_NE_lexicon_tags_2 },
    { 0xe9bced33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe664ed36, SUC_NE_lexicon_tags_2 },
    { 0x4bd4ed37, SUC_NE_lexicon_tags_2 },
    { 0xbe84ed38, SUC_NE_lexicon_tags_2 },
    { 0x9c52ed39, SUC_NE_lexicon_tags_2 },
    { 0xf4ed3a, SUC_NE_lexicon_tags_2 },
    { 0x2a4ed3b, SUC_NE_lexicon_tags_2 },
    { 0x6180ed3c, SUC_NE_lexicon_tags_3 },
    { 0x86eed36, SUC_NE_lexicon_tags_2 },
    { 0x4f9aed3e, SUC_NE_lexicon_tags_2 },
    { 0x875ced3f, SUC_NE_lexicon_tags_2 },
    { 0xe3c8ed40, SUC_NE_lexicon_tags_2 },
    { 0xac5ced41, SUC_NE_lexicon_tags_2 },
    { 0x275aed41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3aaed44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a2ed47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f86ed4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6146ed4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x544aed4f, SUC_NE_lexicon_tags_2 },
    { 0x829eed50, SUC_NE_lexicon_tags_2 },
    { 0x210eed51, SUC_NE_lexicon_tags_98 },
    { 0xae3eed51, SUC_NE_lexicon_tags_2 },
    { 0x3d74ed50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b32ed55, SUC_NE_lexicon_tags_2 },
    { 0xbd16ed56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfe2ed58, SUC_NE_lexicon_tags_2 },
    { 0x423ced59, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf994ed5d, SUC_NE_lexicon_tags_129 },
    { 0, NULL },
    { 0x8abced5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf12ed61, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb12ed63, SUC_NE_lexicon_tags_2 },
    { 0xd01eed64, SUC_NE_lexicon_tags_2 },
    { 0x1430ed63, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce0ed69, SUC_NE_lexicon_tags_2 },
    { 0x6374ed6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6ced6d, SUC_NE_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa110ed7c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb68ed81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8baed83, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b80ed86, SUC_NE_lexicon_tags_2 },
    { 0xf9cced87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2656ed89, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb660ed8e, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe926ed91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8ed94, SUC_NE_lexicon_tags_2 },
    { 0xf736ed95, SUC_NE_lexicon_tags_3 },
    { 0x83b8ed96, SUC_NE_lexicon_tags_2 },
    { 0x2c0aed97, SUC_NE_lexicon_tags_2 },
    { 0xc0e8ed96, SUC_NE_lexicon_tags_2 },
    { 0x8180ed97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc6ed9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ceeda2, SUC_NE_lexicon_tags_2 },
    { 0x9fd6eda3, SUC_NE_lexicon_tags_2 },
    { 0x8668eda4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x1438eda6, SUC_NE_lexicon_tags_2 },
    { 0x676eda6, SUC_NE_lexicon_tags_2 },
    { 0x2670eda8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5804edaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4deedb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7e2edb3, SUC_NE_lexicon_tags_2 },
    { 0xe2d6edb4, SUC_NE_lexicon_tags_2 },
    { 0x2aceedb4, SUC_NE_lexicon_tags_18 },
    { 0x4e2edb6, SUC_NE_lexicon_tags_18 },
    { 0x175cedb5, SUC_NE_lexicon_tags_2 },
    { 0xa1f2edb6, SUC_NE_lexicon_tags_2 },
    { 0xa01aedb9, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94edbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6032edc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5724edc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7512edc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3592edce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59e2edd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb0eedd2, SUC_NE_lexicon_tags_2 },
    { 0xe17eedd3, SUC_NE_lexicon_tags_2 },
    { 0x7a02edd4, SUC_NE_lexicon_tags_2 },
    { 0xa3f4edd2, SUC_NE_lexicon_tags_2 },
    { 0xf56cedd6, SUC_NE_lexicon_tags_2 },
    { 0xe698edd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb856edd9, SUC_NE_lexicon_tags_2 },
    { 0xa6c8edda, SUC_NE_lexicon_tags_2 },
    { 0xdf4eeddb, SUC_NE_lexicon_tags_2 },
    { 0x2bdeeddc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38e4edde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe10ede0, SUC_NE_lexicon_tags_80 },
    { 0, NULL },
    { 0x5fb8ede2, SUC_NE_lexicon_tags_2 },
    { 0xfcf4ede3, SUC_NE_lexicon_tags_2 },
    { 0xf670ede4, SUC_NE_lexicon_tags_292 },
    { 0, NULL },
    { 0xca66ede6, SUC_NE_lexicon_tags_2 },
    { 0x490ede7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea6edee, SUC_NE_lexicon_tags_2 },
    { 0xc012edef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e9aedf1, SUC_NE_lexicon_tags_2 },
    { 0xb558edf2, SUC_NE_lexicon_tags_2 },
    { 0x3530edf1, SUC_NE_lexicon_tags_2 },
    { 0xe000edf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b8edf8, SUC_NE_lexicon_tags_2 },
    { 0xda2aedf9, SUC_NE_lexicon_tags_2 },
    { 0x7188edfa, SUC_NE_lexicon_tags_2 },
    { 0x864edfa, SUC_NE_lexicon_tags_2 },
    { 0xe09cedfb, SUC_NE_lexicon_tags_2 },
    { 0xd32edfd, SUC_NE_lexicon_tags_2 },
    { 0xe30eedfe, SUC_NE_lexicon_tags_13 },
    { 0xca54edfc, SUC_NE_lexicon_tags_2 },
    { 0x4cedfb, SUC_NE_lexicon_tags_2 },
    { 0xfdf2ee00, SUC_NE_lexicon_tags_7 },
    { 0x6ceaedfb, SUC_NE_lexicon_tags_4 },
    { 0x465cee03, SUC_NE_lexicon_tags_7 },
    { 0xd2bcee04, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x13a0ee06, SUC_NE_lexicon_tags_2 },
    { 0xd3e0ee07, SUC_NE_lexicon_tags_2 },
    { 0xeadcee07, SUC_NE_lexicon_tags_2 },
    { 0x9f06ee09, SUC_NE_lexicon_tags_32 },
    { 0xa61cee09, SUC_NE_lexicon_tags_2 },
    { 0xb126ee0b, SUC_NE_lexicon_tags_2 },
    { 0x757cee0c, SUC_NE_lexicon_tags_0 },
    { 0x9278ee0d, SUC_NE_lexicon_tags_2 },
    { 0x20eaee0e, SUC_NE_lexicon_tags_2 },
    { 0xac54ee0c, SUC_NE_lexicon_tags_2 },
    { 0x784eee10, SUC_NE_lexicon_tags_2 },
    { 0x29e6ee09, SUC_NE_lexicon_tags_2 },
    { 0xc934ee12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde5cee14, SUC_NE_lexicon_tags_2 },
    { 0x8b12ee14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x114cee17, SUC_NE_lexicon_tags_2 },
    { 0x442ee17, SUC_NE_lexicon_tags_2 },
    { 0x855aee19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a0ee22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb14ee28, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e24ee2b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828ee2f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d9aee31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa6eee33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b40ee37, SUC_NE_lexicon_tags_2 },
    { 0xdceaee37, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34eee3b, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71beee3f, SUC_NE_lexicon_tags_2 },
    { 0x5d5cee3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc39aee42, SUC_NE_lexicon_tags_2 },
    { 0xd1b4ee43, SUC_NE_lexicon_tags_2 },
    { 0xf8aaee43, SUC_NE_lexicon_tags_0 },
    { 0x8b4aee45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94cee49, SUC_NE_lexicon_tags_2 },
    { 0xd576ee4a, SUC_NE_lexicon_tags_2 },
    { 0x76d4ee4b, SUC_NE_lexicon_tags_2 },
    { 0x87dcee4b, SUC_NE_lexicon_tags_2 },
    { 0xac48ee4a, SUC_NE_lexicon_tags_2 },
    { 0x2358ee4e, SUC_NE_lexicon_tags_1 },
    { 0x3562ee4b, SUC_NE_lexicon_tags_2 },
    { 0xf9f6ee4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2ee53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6938ee55, SUC_NE_lexicon_tags_108 },
    { 0, NULL },
    { 0xac44ee57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5aee61, SUC_NE_lexicon_tags_2 },
    { 0x7798ee62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6baee66, SUC_NE_lexicon_tags_2 },
    { 0xf848ee66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf6eee69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2eee6c, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x566ee6e, SUC_NE_lexicon_tags_2 },
    { 0xcc52ee6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125cee73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8c8ee75, SUC_NE_lexicon_tags_2 },
    { 0x1b2ee76, SUC_NE_lexicon_tags_4 },
    { 0x9310ee75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb640ee7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6beee7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd78ee7f, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc12ee81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6aee84, SUC_NE_lexicon_tags_1 },
    { 0x794cee85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x878eee87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e2ee8a, SUC_NE_lexicon_tags_2 },
    { 0x37bcee8b, SUC_NE_lexicon_tags_2 },
    { 0x7278ee8c, SUC_NE_lexicon_tags_58 },
    { 0x328aee8d, SUC_NE_lexicon_tags_2 },
    { 0x720cee8b, SUC_NE_lexicon_tags_2 },
    { 0xcb98ee8e, SUC_NE_lexicon_tags_2 },
    { 0x8e24ee90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe214ee93, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb074ee99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d34ee9b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbe0ee9d, SUC_NE_lexicon_tags_7 },
    { 0x282cee9e, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24eea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eceea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaeeab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3ceeb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59dceeb3, SUC_NE_lexicon_tags_2 },
    { 0x1d6ceeb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d16eeb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aaeebd, SUC_NE_lexicon_tags_2 },
    { 0x1fb6eebe, SUC_NE_lexicon_tags_2 },
    { 0x4c16eebf, SUC_NE_lexicon_tags_72 },
    { 0x3fd0eebe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6918eec2, SUC_NE_lexicon_tags_2 },
    { 0x9b48eec3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1090eec7, SUC_NE_lexicon_tags_3 },
    { 0x40aceec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8aeecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcaeecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffeeed7, SUC_NE_lexicon_tags_2 },
    { 0x339aeed8, SUC_NE_lexicon_tags_2 },
    { 0x2f6ceed9, SUC_NE_lexicon_tags_2 },
    { 0x4558eeda, SUC_NE_lexicon_tags_2 },
    { 0x946eedb, SUC_NE_lexicon_tags_2 },
    { 0xbb54eedc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90daeee0, SUC_NE_lexicon_tags_2 },
    { 0x3100eee1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8d0eee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe23ceee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5eeeeb, SUC_NE_lexicon_tags_2 },
    { 0x8a3aeeeb, SUC_NE_lexicon_tags_2 },
    { 0x85f2eeed, SUC_NE_lexicon_tags_2 },
    { 0x6c3ceeee, SUC_NE_lexicon_tags_1 },
    { 0x8edaeeee, SUC_NE_lexicon_tags_2 },
    { 0x4baaeef0, SUC_NE_lexicon_tags_2 },
    { 0xf538eeee, SUC_NE_lexicon_tags_2 },
    { 0x65d2eeef, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb22eef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb806eef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e88eefb, SUC_NE_lexicon_tags_2 },
    { 0x4a0eeefc, SUC_NE_lexicon_tags_2 },
    { 0x19deeefd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eef05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7800ef09, SUC_NE_lexicon_tags_2 },
    { 0x8508ef0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf8ef0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ceef10, SUC_NE_lexicon_tags_7 },
    { 0xae68ef11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9a8ef13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bcef16, SUC_NE_lexicon_tags_2 },
    { 0x36fcef16, SUC_NE_lexicon_tags_2 },
    { 0x7dd2ef16, SUC_NE_lexicon_tags_2 },
    { 0xe2eaef17, SUC_NE_lexicon_tags_102 },
    { 0xaf4cef1a, SUC_NE_lexicon_tags_2 },
    { 0xd708ef1b, SUC_NE_lexicon_tags_2 },
    { 0xa942ef1b, SUC_NE_lexicon_tags_2 },
    { 0xab50ef1b, SUC_NE_lexicon_tags_2 },
    { 0xfee0ef1e, SUC_NE_lexicon_tags_2 },
    { 0x3f0ef1e, SUC_NE_lexicon_tags_2 },
    { 0x2acef20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b78ef23, SUC_NE_lexicon_tags_2 },
    { 0x65b4ef24, SUC_NE_lexicon_tags_2 },
    { 0x9b8aef23, SUC_NE_lexicon_tags_2 },
    { 0xd624ef23, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd272ef2a, SUC_NE_lexicon_tags_2 },
    { 0xc09cef2b, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf310ef2f, SUC_NE_lexicon_tags_2 },
    { 0x6766ef30, SUC_NE_lexicon_tags_2 },
    { 0xf456ef30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721eef35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6714ef39, SUC_NE_lexicon_tags_4 },
    { 0x9148ef3a, SUC_NE_lexicon_tags_2 },
    { 0x7116ef3b, SUC_NE_lexicon_tags_2 },
    { 0x75a6ef3a, SUC_NE_lexicon_tags_2 },
    { 0x318ef3a, SUC_NE_lexicon_tags_2 },
    { 0x28f2ef3e, SUC_NE_lexicon_tags_4 },
    { 0x3f94ef3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25caef41, SUC_NE_lexicon_tags_2 },
    { 0x2cf6ef42, SUC_NE_lexicon_tags_5 },
    { 0x5f7cef43, SUC_NE_lexicon_tags_2 },
    { 0x7666ef44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49feef49, SUC_NE_lexicon_tags_2 },
    { 0xa6aef49, SUC_NE_lexicon_tags_2 },
    { 0x33f4ef4b, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xbe7cef4d, SUC_NE_lexicon_tags_2 },
    { 0x7250ef4e, SUC_NE_lexicon_tags_2 },
    { 0x8566ef4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa146ef51, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd6eaef53, SUC_NE_lexicon_tags_2 },
    { 0x2d70ef54, SUC_NE_lexicon_tags_7 },
    { 0xac2aef55, SUC_NE_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4ef58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d0ef5d, SUC_NE_lexicon_tags_2 },
    { 0x77e0ef5e, SUC_NE_lexicon_tags_2 },
    { 0x1a5cef5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a2ef67, SUC_NE_lexicon_tags_2 },
    { 0xca2aef67, SUC_NE_lexicon_tags_18 },
    { 0x7caef69, SUC_NE_lexicon_tags_2 },
    { 0x9f2cef6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44cef6d, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132ef71, SUC_NE_lexicon_tags_2 },
    { 0x9638ef72, SUC_NE_lexicon_tags_2 },
    { 0x14b0ef73, SUC_NE_lexicon_tags_2 },
    { 0xc740ef72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d10ef79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf48aef7b, SUC_NE_lexicon_tags_2 },
    { 0xaee8ef7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5442ef80, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68ef83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa596ef85, SUC_NE_lexicon_tags_30 },
    { 0x8d00ef86, SUC_NE_lexicon_tags_3 },
    { 0xada0ef87, SUC_NE_lexicon_tags_2 },
    { 0xf974ef85, SUC_NE_lexicon_tags_2 },
    { 0x3172ef89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a6ef8f, SUC_NE_lexicon_tags_2 },
    { 0xa9ccef90, SUC_NE_lexicon_tags_2 },
    { 0x9d18ef90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb8ef94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1710ef96, SUC_NE_lexicon_tags_2 },
    { 0xa822ef96, SUC_NE_lexicon_tags_2 },
    { 0x4cd6ef98, SUC_NE_lexicon_tags_25 },
    { 0xdcceef98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1aef9e, SUC_NE_lexicon_tags_2 },
    { 0xb8b6ef9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54a4efa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbf0efa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdcefa6, SUC_NE_lexicon_tags_2 },
    { 0xa94cefa6, SUC_NE_lexicon_tags_4 },
    { 0x4e34efa8, SUC_NE_lexicon_tags_2 },
    { 0x6a18efa9, SUC_NE_lexicon_tags_2 },
    { 0x1af0efa9, SUC_NE_lexicon_tags_2 },
    { 0x3ccaefaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c0aefad, SUC_NE_lexicon_tags_2 },
    { 0xf120efae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ecefb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d4efb4, SUC_NE_lexicon_tags_2 },
    { 0x5e16efb5, SUC_NE_lexicon_tags_2 },
    { 0x61e8efb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2272efb9, SUC_NE_lexicon_tags_2 },
    { 0x693eefba, SUC_NE_lexicon_tags_2 },
    { 0x74a2efba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf05cefbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d74efc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fcefc3, SUC_NE_lexicon_tags_2 },
    { 0xfcbaefc3, SUC_NE_lexicon_tags_18 },
    { 0x3ab6efc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00eefcd, SUC_NE_lexicon_tags_1 },
    { 0x7dbaefcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2c8efd0, SUC_NE_lexicon_tags_2 },
    { 0xa800efd1, SUC_NE_lexicon_tags_2 },
    { 0x1494efd2, SUC_NE_lexicon_tags_2 },
    { 0x1d62efd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7616efd5, SUC_NE_lexicon_tags_2 },
    { 0x5db6efd5, SUC_NE_lexicon_tags_303 },
    { 0x9b7eefd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x373cefd9, SUC_NE_lexicon_tags_2 },
    { 0x1a4aefda, SUC_NE_lexicon_tags_2 },
    { 0xa5b0efdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8146efdd, SUC_NE_lexicon_tags_2 },
    { 0xc2eaefde, SUC_NE_lexicon_tags_2 },
    { 0x4104efde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ccefe2, SUC_NE_lexicon_tags_14 },
    { 0xc9aaefe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2270efe6, SUC_NE_lexicon_tags_2 },
    { 0xb408efe7, SUC_NE_lexicon_tags_34 },
    { 0x2cb8efe8, SUC_NE_lexicon_tags_2 },
    { 0x3738efe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x932aefec, SUC_NE_lexicon_tags_2 },
    { 0xd69cefed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97c2efef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b50eff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1808eff3, SUC_NE_lexicon_tags_2 },
    { 0x12d6eff3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb10eff6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd670eff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x950aeffa, SUC_NE_lexicon_tags_2 },
    { 0x7d32effb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364eefff, SUC_NE_lexicon_tags_2 },
    { 0x1b66efff, SUC_NE_lexicon_tags_2 },
    { 0xed0aefff, SUC_NE_lexicon_tags_2 },
    { 0x8a3cf002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8bcf004, SUC_NE_lexicon_tags_1 },
    { 0x79ecf005, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c9af007, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae4f00b, SUC_NE_lexicon_tags_2 },
    { 0x6fc8f00c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7086f00e, SUC_NE_lexicon_tags_2 },
    { 0xa25ef00f, SUC_NE_lexicon_tags_2 },
    { 0x2ea0f010, SUC_NE_lexicon_tags_2 },
    { 0xabf8f00f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe94cf013, SUC_NE_lexicon_tags_2 },
    { 0xfd22f014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18f017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5daf01d, SUC_NE_lexicon_tags_123 },
    { 0, NULL },
    { 0x7020f01f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec6f023, SUC_NE_lexicon_tags_0 },
    { 0x1054f024, SUC_NE_lexicon_tags_2 },
    { 0x3a26f025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea24f02a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb30f02c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x17e8f02e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00af035, SUC_NE_lexicon_tags_3 },
    { 0xb5baf036, SUC_NE_lexicon_tags_2 },
    { 0xeec8f035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3520f03b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8caaf03d, SUC_NE_lexicon_tags_2 },
    { 0x70eef03e, SUC_NE_lexicon_tags_2 },
    { 0xbd92f03f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e2f043, SUC_NE_lexicon_tags_2 },
    { 0x524ef044, SUC_NE_lexicon_tags_2 },
    { 0x1c26f045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f2f048, SUC_NE_lexicon_tags_2 },
    { 0xaaeaf049, SUC_NE_lexicon_tags_2 },
    { 0xfc54f049, SUC_NE_lexicon_tags_2 },
    { 0xc044f04b, SUC_NE_lexicon_tags_2 },
    { 0x9e76f04c, SUC_NE_lexicon_tags_1 },
    { 0x3778f04b, SUC_NE_lexicon_tags_2 },
    { 0x5bd6f04e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd526f050, SUC_NE_lexicon_tags_2 },
    { 0x23a6f050, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d4f055, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced0f059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37ef05c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc870f060, SUC_NE_lexicon_tags_2 },
    { 0x2d02f061, SUC_NE_lexicon_tags_2 },
    { 0x6c7ef062, SUC_NE_lexicon_tags_2 },
    { 0xb6b2f061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaef066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ae0f068, SUC_NE_lexicon_tags_2 },
    { 0x7904f069, SUC_NE_lexicon_tags_4 },
    { 0xd426f06a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f6f06d, SUC_NE_lexicon_tags_386 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db8f073, SUC_NE_lexicon_tags_2 },
    { 0x3d26f073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfac6f076, SUC_NE_lexicon_tags_2 },
    { 0xf976f077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd158f083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ab2f085, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x96baf088, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5acf08b, SUC_NE_lexicon_tags_2 },
    { 0xd764f08c, SUC_NE_lexicon_tags_3 },
    { 0xfd8cf08d, SUC_NE_lexicon_tags_35 },
    { 0, NULL },
    { 0x37f0f08f, SUC_NE_lexicon_tags_2 },
    { 0xc9d2f090, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1594f098, SUC_NE_lexicon_tags_40 },
    { 0x78d4f099, SUC_NE_lexicon_tags_2 },
    { 0x33cef09a, SUC_NE_lexicon_tags_2 },
    { 0x9684f09a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa768f0a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1cf0a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4248f0a6, SUC_NE_lexicon_tags_2 },
    { 0x2e52f0a7, SUC_NE_lexicon_tags_19 },
    { 0xeba0f0a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0def0aa, SUC_NE_lexicon_tags_2 },
    { 0x60eaf0ab, SUC_NE_lexicon_tags_2 },
    { 0x7562f0ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e88f0b0, SUC_NE_lexicon_tags_2 },
    { 0x1696f0b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59e2f0b3, SUC_NE_lexicon_tags_7 },
    { 0xa622f0b3, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c0f0b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5af0be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d4f0c0, SUC_NE_lexicon_tags_7 },
    { 0xc1aef0c1, SUC_NE_lexicon_tags_2 },
    { 0x1458f0c2, SUC_NE_lexicon_tags_2 },
    { 0xbbe8f0c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30af0ca, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff0f0cd, SUC_NE_lexicon_tags_2 },
    { 0xe6f0ce, SUC_NE_lexicon_tags_2 },
    { 0xa568f0cf, SUC_NE_lexicon_tags_2 },
    { 0x8982f0d0, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194cf0d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7696f0d6, SUC_NE_lexicon_tags_2 },
    { 0x32d4f0d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bcf0da, SUC_NE_lexicon_tags_2 },
    { 0xe988f0db, SUC_NE_lexicon_tags_2 },
    { 0x7e6cf0dc, SUC_NE_lexicon_tags_2 },
    { 0xdd6af0dc, SUC_NE_lexicon_tags_2 },
    { 0x7ba4f0da, SUC_NE_lexicon_tags_50 },
    { 0xeda8f0da, SUC_NE_lexicon_tags_2 },
    { 0xa692f0df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eecf0e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb294f0e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5324f0ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc2f0f1, SUC_NE_lexicon_tags_2 },
    { 0x45acf0f1, SUC_NE_lexicon_tags_2 },
    { 0xd4e8f0f3, SUC_NE_lexicon_tags_2 },
    { 0x89f0f0f1, SUC_NE_lexicon_tags_2 },
    { 0xba7cf0f2, SUC_NE_lexicon_tags_2 },
    { 0x1e64f0f6, SUC_NE_lexicon_tags_2 },
    { 0x872cf0f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5c0f0f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728af0fd, SUC_NE_lexicon_tags_2 },
    { 0x69ef0fe, SUC_NE_lexicon_tags_2 },
    { 0xa7a2f0ff, SUC_NE_lexicon_tags_2 },
    { 0xd294f100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8312f10c, SUC_NE_lexicon_tags_2 },
    { 0x5daaf10c, SUC_NE_lexicon_tags_2 },
    { 0x28fcf10d, SUC_NE_lexicon_tags_4 },
    { 0xdb0cf10f, SUC_NE_lexicon_tags_2 },
    { 0x4d8af10f, SUC_NE_lexicon_tags_2 },
    { 0xa278f10f, SUC_NE_lexicon_tags_7 },
    { 0x3e0ef112, SUC_NE_lexicon_tags_2 },
    { 0xcdd4f112, SUC_NE_lexicon_tags_2 },
    { 0x37a4f114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6f117, SUC_NE_lexicon_tags_2 },
    { 0x9e34f118, SUC_NE_lexicon_tags_2 },
    { 0x1e52f119, SUC_NE_lexicon_tags_2 },
    { 0x58aaf11a, SUC_NE_lexicon_tags_2 },
    { 0xb2caf11b, SUC_NE_lexicon_tags_7 },
    { 0x32c2f11b, SUC_NE_lexicon_tags_1 },
    { 0xbc8f117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10cf123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd886f127, SUC_NE_lexicon_tags_2 },
    { 0xe9fcf128, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f4f12c, SUC_NE_lexicon_tags_2 },
    { 0xef56f12d, SUC_NE_lexicon_tags_2 },
    { 0xaed0f12d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49cf131, SUC_NE_lexicon_tags_2 },
    { 0xe9b4f132, SUC_NE_lexicon_tags_18 },
    { 0x6506f133, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac8f139, SUC_NE_lexicon_tags_2 },
    { 0xa198f13a, SUC_NE_lexicon_tags_2 },
    { 0xe084f13b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4580f13e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1c6f140, SUC_NE_lexicon_tags_2 },
    { 0xe320f141, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b0f146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940af14a, SUC_NE_lexicon_tags_2 },
    { 0xd93cf14b, SUC_NE_lexicon_tags_4 },
    { 0x3396f14b, SUC_NE_lexicon_tags_2 },
    { 0xc550f14d, SUC_NE_lexicon_tags_8 },
    { 0xb8a8f14d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42af151, SUC_NE_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5752f159, SUC_NE_lexicon_tags_2 },
    { 0xc9cef15a, SUC_NE_lexicon_tags_2 },
    { 0x7468f15a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9ef164, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaef167, SUC_NE_lexicon_tags_0 },
    { 0x5658f167, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x9dc2f16a, SUC_NE_lexicon_tags_2 },
    { 0x607ef16b, SUC_NE_lexicon_tags_2 },
    { 0x71a4f16c, SUC_NE_lexicon_tags_2 },
    { 0xccbef16c, SUC_NE_lexicon_tags_2 },
    { 0x4a84f16c, SUC_NE_lexicon_tags_2 },
    { 0x7784f16e, SUC_NE_lexicon_tags_2 },
    { 0x57eef170, SUC_NE_lexicon_tags_4 },
    { 0xb338f170, SUC_NE_lexicon_tags_2 },
    { 0x5480f16e, SUC_NE_lexicon_tags_2 },
    { 0x36ef172, SUC_NE_lexicon_tags_11 },
    { 0xdb72f171, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1af2f176, SUC_NE_lexicon_tags_24 },
    { 0x8c6ef177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9c0f179, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1af17c, SUC_NE_lexicon_tags_2 },
    { 0xe346f17c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceaf181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e94f185, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56ef189, SUC_NE_lexicon_tags_2 },
    { 0xe882f18a, SUC_NE_lexicon_tags_2 },
    { 0xf030f18b, SUC_NE_lexicon_tags_2 },
    { 0xf12af18b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe604f18e, SUC_NE_lexicon_tags_3 },
    { 0x16d8f18f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffaf194, SUC_NE_lexicon_tags_2 },
    { 0xaaf8f195, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c0af197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49cf19e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf67cf1a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea88f1a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aef1a7, SUC_NE_lexicon_tags_2 },
    { 0xb0f0f1a7, SUC_NE_lexicon_tags_2 },
    { 0x69e4f1a7, SUC_NE_lexicon_tags_292 },
    { 0, NULL },
    { 0xe15cf1ab, SUC_NE_lexicon_tags_2 },
    { 0x594f1ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c40f1ae, SUC_NE_lexicon_tags_1 },
    { 0xfb9cf1ae, SUC_NE_lexicon_tags_2 },
    { 0x1e4ef1b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434ef1b6, SUC_NE_lexicon_tags_2 },
    { 0x3d50f1b7, SUC_NE_lexicon_tags_2 },
    { 0xbe5cf1b8, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cef1bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcaf1c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cdaf1c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf218f1c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x513af1c7, SUC_NE_lexicon_tags_2 },
    { 0x73b8f1c8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad38f1cd, SUC_NE_lexicon_tags_2 },
    { 0x938af1ce, SUC_NE_lexicon_tags_2 },
    { 0x960ef1ce, SUC_NE_lexicon_tags_2 },
    { 0x971cf1ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea2ef1d2, SUC_NE_lexicon_tags_2 },
    { 0xc714f1d3, SUC_NE_lexicon_tags_2 },
    { 0xce4f1d4, SUC_NE_lexicon_tags_2 },
    { 0xbc8ef1d4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf36f1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5126f1d9, SUC_NE_lexicon_tags_2 },
    { 0xe4b0f1da, SUC_NE_lexicon_tags_2 },
    { 0x127cf1d9, SUC_NE_lexicon_tags_2 },
    { 0x80b8f1dc, SUC_NE_lexicon_tags_2 },
    { 0xb4b2f1da, SUC_NE_lexicon_tags_25 },
    { 0x9a7ef1d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa838f1e2, SUC_NE_lexicon_tags_2 },
    { 0x2f08f1e3, SUC_NE_lexicon_tags_2 },
    { 0x3a12f1e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0af1e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7bef1ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70cf1ee, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f1f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6790f1f5, SUC_NE_lexicon_tags_2 },
    { 0x5c24f1f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8098f1fb, SUC_NE_lexicon_tags_2 },
    { 0x5b1cf1fc, SUC_NE_lexicon_tags_2 },
    { 0x81fcf1fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x772cf200, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfaf205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba6f208, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c46f210, SUC_NE_lexicon_tags_2 },
    { 0x8ab2f211, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a0f214, SUC_NE_lexicon_tags_2 },
    { 0x3c2af214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe38f217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c8f21c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36c0f21e, SUC_NE_lexicon_tags_2 },
    { 0x7c0ef21e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08ef222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb44f226, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x443ef229, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6e6f22b, SUC_NE_lexicon_tags_2 },
    { 0x590ef22c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7908f239, SUC_NE_lexicon_tags_2 },
    { 0x55faf23a, SUC_NE_lexicon_tags_2 },
    { 0x4944f23b, SUC_NE_lexicon_tags_17 },
    { 0xcd86f23c, SUC_NE_lexicon_tags_2 },
    { 0x35ef23d, SUC_NE_lexicon_tags_2 },
    { 0x1af4f23e, SUC_NE_lexicon_tags_3 },
    { 0xc618f23c, SUC_NE_lexicon_tags_2 },
    { 0xb908f240, SUC_NE_lexicon_tags_2 },
    { 0x8516f23a, SUC_NE_lexicon_tags_2 },
    { 0x41aaf23d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde90f245, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66b8f247, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed8f24a, SUC_NE_lexicon_tags_2 },
    { 0x930ef24a, SUC_NE_lexicon_tags_8 },
    { 0x5194f24c, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4f250, SUC_NE_lexicon_tags_2 },
    { 0x3b3cf251, SUC_NE_lexicon_tags_1 },
    { 0xe944f250, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372cf257, SUC_NE_lexicon_tags_2 },
    { 0x7ee8f258, SUC_NE_lexicon_tags_2 },
    { 0x43cef257, SUC_NE_lexicon_tags_7 },
    { 0xd548f25a, SUC_NE_lexicon_tags_2 },
    { 0xbd0ef259, SUC_NE_lexicon_tags_2 },
    { 0x52d0f25c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e0f266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7baf26c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x88e0f26e, SUC_NE_lexicon_tags_2 },
    { 0xeea4f26e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x706f271, SUC_NE_lexicon_tags_1 },
    { 0xb4dcf272, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cef277, SUC_NE_lexicon_tags_59 },
    { 0x61ccf277, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x704ef27b, SUC_NE_lexicon_tags_2 },
    { 0xa564f27b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3818f281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2264f284, SUC_NE_lexicon_tags_2 },
    { 0x97b0f285, SUC_NE_lexicon_tags_2 },
    { 0x5842f285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4208f288, SUC_NE_lexicon_tags_2 },
    { 0x95eaf289, SUC_NE_lexicon_tags_3 },
    { 0xee2f28a, SUC_NE_lexicon_tags_2 },
    { 0xd0f2f28a, SUC_NE_lexicon_tags_2 },
    { 0x5c0ef28a, SUC_NE_lexicon_tags_4 },
    { 0xc580f28d, SUC_NE_lexicon_tags_1 },
    { 0x15c4f28e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53fcf290, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf540f292, SUC_NE_lexicon_tags_3 },
    { 0x2932f292, SUC_NE_lexicon_tags_2 },
    { 0x19eaf294, SUC_NE_lexicon_tags_8 },
    { 0x6c68f295, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a44f29b, SUC_NE_lexicon_tags_2 },
    { 0xde6af29c, SUC_NE_lexicon_tags_1 },
    { 0x3994f29c, SUC_NE_lexicon_tags_2 },
    { 0x2a6ef29e, SUC_NE_lexicon_tags_2 },
    { 0xbe06f29e, SUC_NE_lexicon_tags_2 },
    { 0xa6ecf2a0, SUC_NE_lexicon_tags_2 },
    { 0xf8f2f2a1, SUC_NE_lexicon_tags_0 },
    { 0xaa76f2a0, SUC_NE_lexicon_tags_2 },
    { 0x6d2f2a1, SUC_NE_lexicon_tags_2 },
    { 0xdbaef2a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cf2a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f60f2aa, SUC_NE_lexicon_tags_5 },
    { 0x5904f2ab, SUC_NE_lexicon_tags_2 },
    { 0xf7a0f2ac, SUC_NE_lexicon_tags_2 },
    { 0x67aef2ab, SUC_NE_lexicon_tags_2 },
    { 0xe7acf2ad, SUC_NE_lexicon_tags_2 },
    { 0xbc26f2ad, SUC_NE_lexicon_tags_2 },
    { 0x9ef2b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0f2b4, SUC_NE_lexicon_tags_2 },
    { 0x5532f2b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x604f2b7, SUC_NE_lexicon_tags_2 },
    { 0x93c2f2b8, SUC_NE_lexicon_tags_2 },
    { 0x51a4f2b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b4f2bc, SUC_NE_lexicon_tags_2 },
    { 0xc34f2bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6ecf2bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8e0f2c1, SUC_NE_lexicon_tags_2 },
    { 0x7f08f2c2, SUC_NE_lexicon_tags_314 },
    { 0xa036f2c3, SUC_NE_lexicon_tags_7 },
    { 0xd4e2f2c1, SUC_NE_lexicon_tags_2 },
    { 0xf24ef2c5, SUC_NE_lexicon_tags_2 },
    { 0xfa74f2c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90eef2c8, SUC_NE_lexicon_tags_3 },
    { 0xd3e0f2c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fef2cd, SUC_NE_lexicon_tags_17 },
    { 0xeef8f2cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcf2d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa014f2d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f4f2d8, SUC_NE_lexicon_tags_2 },
    { 0x3a2ef2d9, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6def2dc, SUC_NE_lexicon_tags_2 },
    { 0xa268f2dd, SUC_NE_lexicon_tags_2 },
    { 0x4516f2de, SUC_NE_lexicon_tags_2 },
    { 0x8044f2dc, SUC_NE_lexicon_tags_2 },
    { 0xfbd8f2dc, SUC_NE_lexicon_tags_2 },
    { 0x7c6cf2e1, SUC_NE_lexicon_tags_18 },
    { 0xb78ef2df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6e6f2e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8426f2ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x451af2ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08ef2f2, SUC_NE_lexicon_tags_2 },
    { 0x33acf2f3, SUC_NE_lexicon_tags_2 },
    { 0x5f3ef2f4, SUC_NE_lexicon_tags_2 },
    { 0x59d0f2f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd270f2f7, SUC_NE_lexicon_tags_2 },
    { 0x68caf2f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c46f2fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f58f300, SUC_NE_lexicon_tags_17 },
    { 0x91b8f301, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854cf308, SUC_NE_lexicon_tags_2 },
    { 0x9f2ef309, SUC_NE_lexicon_tags_2 },
    { 0x5d4f308, SUC_NE_lexicon_tags_2 },
    { 0x29a4f30a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72a4f30d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a4f311, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xb792f314, SUC_NE_lexicon_tags_2 },
    { 0x22b6f315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4be6f317, SUC_NE_lexicon_tags_2 },
    { 0x9aa6f318, SUC_NE_lexicon_tags_0 },
    { 0x1106f319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c2f31d, SUC_NE_lexicon_tags_2 },
    { 0x283cf31e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29e2f320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27faf322, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x2a0cf324, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a6f329, SUC_NE_lexicon_tags_1 },
    { 0xbae4f32a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adcf32d, SUC_NE_lexicon_tags_2 },
    { 0x92eef32e, SUC_NE_lexicon_tags_2 },
    { 0xf334f32f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf310f332, SUC_NE_lexicon_tags_2 },
    { 0x232cf332, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fcf338, SUC_NE_lexicon_tags_2 },
    { 0x7304f339, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde48f33c, SUC_NE_lexicon_tags_2 },
    { 0x6f7cf33d, SUC_NE_lexicon_tags_2 },
    { 0x9f4cf33d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f38f340, SUC_NE_lexicon_tags_2 },
    { 0x6c5ef341, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dcf343, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d06f347, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xff84f349, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc984f34d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb366f357, SUC_NE_lexicon_tags_2 },
    { 0xfc76f357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b6f35c, SUC_NE_lexicon_tags_2 },
    { 0x504ef35d, SUC_NE_lexicon_tags_3 },
    { 0xfd0f35d, SUC_NE_lexicon_tags_11 },
    { 0x12f2f35f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa956f363, SUC_NE_lexicon_tags_2 },
    { 0x6c12f364, SUC_NE_lexicon_tags_2 },
    { 0x12a2f365, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93a8f367, SUC_NE_lexicon_tags_2 },
    { 0xbf6ef368, SUC_NE_lexicon_tags_2 },
    { 0x66f6f369, SUC_NE_lexicon_tags_18 },
    { 0x8b0f36a, SUC_NE_lexicon_tags_2 },
    { 0xb2a4f367, SUC_NE_lexicon_tags_2 },
    { 0xe19cf36c, SUC_NE_lexicon_tags_2 },
    { 0x94e4f36d, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa058f370, SUC_NE_lexicon_tags_3 },
    { 0x868cf371, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ccf377, SUC_NE_lexicon_tags_1 },
    { 0x7bfef378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb336f37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a8f383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe18f387, SUC_NE_lexicon_tags_32 },
    { 0x54f4f388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666f38c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6364f392, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x858cf394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb724f396, SUC_NE_lexicon_tags_2 },
    { 0x2c5ef396, SUC_NE_lexicon_tags_2 },
    { 0x9312f397, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53b0f39a, SUC_NE_lexicon_tags_2 },
    { 0x3bb8f39b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49af3a2, SUC_NE_lexicon_tags_2 },
    { 0xfbb2f3a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132f3a7, SUC_NE_lexicon_tags_2 },
    { 0xca78f3a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8964f3aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b56f3ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeaf3af, SUC_NE_lexicon_tags_2 },
    { 0x8df8f3af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeaf3b2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cf3b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35dcf3bd, SUC_NE_lexicon_tags_2 },
    { 0x7f28f3bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1722f3c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd48f3c2, SUC_NE_lexicon_tags_2 },
    { 0x2c2f3c2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e3cf3c5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8d8ef3c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6980f3cc, SUC_NE_lexicon_tags_4 },
    { 0xe046f3cd, SUC_NE_lexicon_tags_2 },
    { 0x7528f3cd, SUC_NE_lexicon_tags_2 },
    { 0x2edcf3cf, SUC_NE_lexicon_tags_2 },
    { 0xabf6f3cf, SUC_NE_lexicon_tags_2 },
    { 0x74f8f3d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcfcf3d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99b2f3d5, SUC_NE_lexicon_tags_2 },
    { 0x44dcf3d6, SUC_NE_lexicon_tags_1 },
    { 0x453cf3d7, SUC_NE_lexicon_tags_2 },
    { 0xeb82f3d7, SUC_NE_lexicon_tags_2 },
    { 0x5548f3d9, SUC_NE_lexicon_tags_2 },
    { 0x7f78f3d8, SUC_NE_lexicon_tags_2 },
    { 0xc450f3db, SUC_NE_lexicon_tags_2 },
    { 0xd68ef3d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa7cf3de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64f3e0, SUC_NE_lexicon_tags_2 },
    { 0x7f2f3e0, SUC_NE_lexicon_tags_2 },
    { 0x2064f3e1, SUC_NE_lexicon_tags_2 },
    { 0xcf6ef3e3, SUC_NE_lexicon_tags_2 },
    { 0xc50af3e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f3e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0af3ed, SUC_NE_lexicon_tags_2 },
    { 0xf854f3ed, SUC_NE_lexicon_tags_2 },
    { 0x74c4f3ef, SUC_NE_lexicon_tags_7 },
    { 0xc5b0f3f0, SUC_NE_lexicon_tags_1 },
    { 0x7956f3f1, SUC_NE_lexicon_tags_2 },
    { 0xcf70f3f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffaef3f4, SUC_NE_lexicon_tags_2 },
    { 0xd186f3f4, SUC_NE_lexicon_tags_2 },
    { 0xb8e6f3f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5644f3f8, SUC_NE_lexicon_tags_2 },
    { 0x4f74f3f8, SUC_NE_lexicon_tags_2 },
    { 0x22cf3fa, SUC_NE_lexicon_tags_2 },
    { 0xa61af3fb, SUC_NE_lexicon_tags_2 },
    { 0xfeef3fc, SUC_NE_lexicon_tags_2 },
    { 0xd742f3fd, SUC_NE_lexicon_tags_2 },
    { 0x2d32f3fe, SUC_NE_lexicon_tags_2 },
    { 0x15acf3fe, SUC_NE_lexicon_tags_2 },
    { 0x31a8f400, SUC_NE_lexicon_tags_2 },
    { 0x55a8f401, SUC_NE_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faaf408, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x1e86f40a, SUC_NE_lexicon_tags_2 },
    { 0x3048f40a, SUC_NE_lexicon_tags_2 },
    { 0x1118f40c, SUC_NE_lexicon_tags_2 },
    { 0x96f4f40d, SUC_NE_lexicon_tags_2 },
    { 0xa6a6f40d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x213ef410, SUC_NE_lexicon_tags_32 },
    { 0xbb50f410, SUC_NE_lexicon_tags_2 },
    { 0xfa5cf412, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0xac2f414, SUC_NE_lexicon_tags_2 },
    { 0x67e2f415, SUC_NE_lexicon_tags_7 },
    { 0x246f416, SUC_NE_lexicon_tags_2 },
    { 0xa398f416, SUC_NE_lexicon_tags_56 },
    { 0x6fe8f418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e98f41d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3f0f41f, SUC_NE_lexicon_tags_2 },
    { 0x88eef420, SUC_NE_lexicon_tags_2 },
    { 0xb146f420, SUC_NE_lexicon_tags_2 },
    { 0x72cef41f, SUC_NE_lexicon_tags_3 },
    { 0xccd0f421, SUC_NE_lexicon_tags_2 },
    { 0xb09af421, SUC_NE_lexicon_tags_2 },
    { 0x48b0f425, SUC_NE_lexicon_tags_2 },
    { 0x343cf426, SUC_NE_lexicon_tags_2 },
    { 0x47c4f427, SUC_NE_lexicon_tags_2 },
    { 0x68faf428, SUC_NE_lexicon_tags_3 },
    { 0x37d4f429, SUC_NE_lexicon_tags_2 },
    { 0xd6bcf426, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2caf43c, SUC_NE_lexicon_tags_2 },
    { 0x5f68f43d, SUC_NE_lexicon_tags_2 },
    { 0xf868f43e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9180f440, SUC_NE_lexicon_tags_2 },
    { 0x48c6f440, SUC_NE_lexicon_tags_2 },
    { 0x4318f441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928f446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c70f449, SUC_NE_lexicon_tags_2 },
    { 0xb2eef44a, SUC_NE_lexicon_tags_2 },
    { 0x3b1ef449, SUC_NE_lexicon_tags_2 },
    { 0x93a2f44b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd0f44f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbaf455, SUC_NE_lexicon_tags_2 },
    { 0xf93ef455, SUC_NE_lexicon_tags_263 },
    { 0x6874f457, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba16f45b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be6f45f, SUC_NE_lexicon_tags_2 },
    { 0x7c2cf45f, SUC_NE_lexicon_tags_7 },
    { 0x104cf461, SUC_NE_lexicon_tags_2 },
    { 0xeb46f462, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ef8f464, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe48af466, SUC_NE_lexicon_tags_4 },
    { 0xc0af467, SUC_NE_lexicon_tags_2 },
    { 0xb05af468, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68f46b, SUC_NE_lexicon_tags_2 },
    { 0x1b8f46c, SUC_NE_lexicon_tags_2 },
    { 0xe58cf46d, SUC_NE_lexicon_tags_2 },
    { 0x410af46e, SUC_NE_lexicon_tags_2 },
    { 0x7d30f46f, SUC_NE_lexicon_tags_7 },
    { 0x7d44f46e, SUC_NE_lexicon_tags_2 },
    { 0x77c0f471, SUC_NE_lexicon_tags_14 },
    { 0xbad2f46d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x300f475, SUC_NE_lexicon_tags_2 },
    { 0xe3c8f476, SUC_NE_lexicon_tags_2 },
    { 0x88f0f475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8c2f479, SUC_NE_lexicon_tags_2 },
    { 0xe2f2f47a, SUC_NE_lexicon_tags_2 },
    { 0xd802f47a, SUC_NE_lexicon_tags_2 },
    { 0x4f6f47c, SUC_NE_lexicon_tags_2 },
    { 0xe874f47d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348cf484, SUC_NE_lexicon_tags_2 },
    { 0xadb8f485, SUC_NE_lexicon_tags_2 },
    { 0x67d2f486, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3baef488, SUC_NE_lexicon_tags_2 },
    { 0xadecf489, SUC_NE_lexicon_tags_2 },
    { 0x7f70f48a, SUC_NE_lexicon_tags_2 },
    { 0xaf14f489, SUC_NE_lexicon_tags_2 },
    { 0x9c1af48c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b1cf48e, SUC_NE_lexicon_tags_2 },
    { 0x2b06f48f, SUC_NE_lexicon_tags_2 },
    { 0xc01af48f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b6f493, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x563ef495, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf942f497, SUC_NE_lexicon_tags_10 },
    { 0xb4fcf497, SUC_NE_lexicon_tags_2 },
    { 0xac78f497, SUC_NE_lexicon_tags_1 },
    { 0x674af49a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc276f49c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea2f4a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ef4a6, SUC_NE_lexicon_tags_0 },
    { 0xf1aaf4a6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cb2f4a9, SUC_NE_lexicon_tags_2 },
    { 0x62d6f4aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e4af4ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa968f4ae, SUC_NE_lexicon_tags_2 },
    { 0xbb6af4af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c0f4b5, SUC_NE_lexicon_tags_2 },
    { 0x863cf4b5, SUC_NE_lexicon_tags_2 },
    { 0x5648f4b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4888f4b9, SUC_NE_lexicon_tags_2 },
    { 0x1e42f4ba, SUC_NE_lexicon_tags_2 },
    { 0x1c88f4b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70f4be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9790f4c0, SUC_NE_lexicon_tags_4 },
    { 0x3df2f4c1, SUC_NE_lexicon_tags_2 },
    { 0x1c94f4c0, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe038f4ce, SUC_NE_lexicon_tags_2 },
    { 0xb28af4cf, SUC_NE_lexicon_tags_5 },
    { 0x1d44f4ce, SUC_NE_lexicon_tags_2 },
    { 0xe7f2f4ce, SUC_NE_lexicon_tags_1 },
    { 0xf96ef4d2, SUC_NE_lexicon_tags_2 },
    { 0xa2acf4d3, SUC_NE_lexicon_tags_3 },
    { 0x5b84f4d4, SUC_NE_lexicon_tags_109 },
    { 0x30c8f4d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef4f4d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec8f4de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x655ef4e0, SUC_NE_lexicon_tags_2 },
    { 0x94aaf4e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3cf4e4, SUC_NE_lexicon_tags_2 },
    { 0x22d6f4e5, SUC_NE_lexicon_tags_2 },
    { 0x4616f4e4, SUC_NE_lexicon_tags_7 },
    { 0x1682f4e7, SUC_NE_lexicon_tags_2 },
    { 0x4906f4e8, SUC_NE_lexicon_tags_2 },
    { 0x7b0f4e8, SUC_NE_lexicon_tags_2 },
    { 0xd564f4e9, SUC_NE_lexicon_tags_2 },
    { 0x1f3af4ea, SUC_NE_lexicon_tags_19 },
    { 0x90acf4ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x154cf4ef, SUC_NE_lexicon_tags_2 },
    { 0xe806f4ef, SUC_NE_lexicon_tags_2 },
    { 0x966af4f1, SUC_NE_lexicon_tags_2 },
    { 0x5478f4f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc56f4f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa94f4f7, SUC_NE_lexicon_tags_4 },
    { 0x7732f4f7, SUC_NE_lexicon_tags_2 },
    { 0xfbe8f4f7, SUC_NE_lexicon_tags_2 },
    { 0x6b36f4fa, SUC_NE_lexicon_tags_2 },
    { 0x13d4f4f7, SUC_NE_lexicon_tags_2 },
    { 0x4bd8f4f9, SUC_NE_lexicon_tags_2 },
    { 0xce8af4f7, SUC_NE_lexicon_tags_2 },
    { 0x544af4fe, SUC_NE_lexicon_tags_2 },
    { 0x5678f4ff, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c6f504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc68f507, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1644f50b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3cf50e, SUC_NE_lexicon_tags_2 },
    { 0xb16ef50f, SUC_NE_lexicon_tags_7 },
    { 0x8d7af510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45af518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40d2f51a, SUC_NE_lexicon_tags_2 },
    { 0x917cf51b, SUC_NE_lexicon_tags_2 },
    { 0x91cf51c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x440ef51f, SUC_NE_lexicon_tags_2 },
    { 0x8568f520, SUC_NE_lexicon_tags_2 },
    { 0x3382f520, SUC_NE_lexicon_tags_2 },
    { 0xd17ef521, SUC_NE_lexicon_tags_2 },
    { 0x881cf523, SUC_NE_lexicon_tags_2 },
    { 0xf8e2f524, SUC_NE_lexicon_tags_2 },
    { 0x8bdef523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8d2f527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4f52d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fe2f52f, SUC_NE_lexicon_tags_2 },
    { 0x828af52f, SUC_NE_lexicon_tags_2 },
    { 0xdab8f531, SUC_NE_lexicon_tags_2 },
    { 0x4802f532, SUC_NE_lexicon_tags_7 },
    { 0xac7ef532, SUC_NE_lexicon_tags_2 },
    { 0x95b8f532, SUC_NE_lexicon_tags_2 },
    { 0x51d2f535, SUC_NE_lexicon_tags_2 },
    { 0xec6ef536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86baf53b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1af53f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c4f544, SUC_NE_lexicon_tags_3 },
    { 0x242af545, SUC_NE_lexicon_tags_2 },
    { 0x3988f546, SUC_NE_lexicon_tags_2 },
    { 0xdb16f544, SUC_NE_lexicon_tags_25 },
    { 0xd3c0f548, SUC_NE_lexicon_tags_2 },
    { 0xff74f549, SUC_NE_lexicon_tags_2 },
    { 0xc5b8f54a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbef54f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8af552, SUC_NE_lexicon_tags_2 },
    { 0x2c8cf553, SUC_NE_lexicon_tags_28 },
    { 0xdae8f553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43cf558, SUC_NE_lexicon_tags_2 },
    { 0x6eb0f558, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacc0f55b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd8cf55d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c2af55f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2286f563, SUC_NE_lexicon_tags_1 },
    { 0xe36cf564, SUC_NE_lexicon_tags_2 },
    { 0xf392f563, SUC_NE_lexicon_tags_2 },
    { 0xec80f566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb96f569, SUC_NE_lexicon_tags_0 },
    { 0x5ce0f569, SUC_NE_lexicon_tags_2 },
    { 0xec94f56b, SUC_NE_lexicon_tags_2 },
    { 0x1780f56b, SUC_NE_lexicon_tags_2 },
    { 0xa09af56d, SUC_NE_lexicon_tags_2 },
    { 0xba0af56d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2cf572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f4f578, SUC_NE_lexicon_tags_2 },
    { 0x9ebcf579, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7912f57b, SUC_NE_lexicon_tags_2 },
    { 0xb610f57c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6798f57e, SUC_NE_lexicon_tags_2 },
    { 0xef48f57f, SUC_NE_lexicon_tags_2 },
    { 0xb196f580, SUC_NE_lexicon_tags_49 },
    { 0x7ed0f581, SUC_NE_lexicon_tags_7 },
    { 0xf926f57f, SUC_NE_lexicon_tags_2 },
    { 0x9844f580, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x685af585, SUC_NE_lexicon_tags_2 },
    { 0xbf04f586, SUC_NE_lexicon_tags_2 },
    { 0x815af585, SUC_NE_lexicon_tags_2 },
    { 0xb8d0f588, SUC_NE_lexicon_tags_2 },
    { 0x804ef586, SUC_NE_lexicon_tags_2 },
    { 0xcbc0f585, SUC_NE_lexicon_tags_2 },
    { 0x98d4f587, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8b6f58d, SUC_NE_lexicon_tags_2 },
    { 0x5960f58e, SUC_NE_lexicon_tags_4 },
    { 0x7254f58e, SUC_NE_lexicon_tags_2 },
    { 0x68acf590, SUC_NE_lexicon_tags_2 },
    { 0xd9faf590, SUC_NE_lexicon_tags_2 },
    { 0xba9af592, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xc370f594, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a14f596, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4af59c, SUC_NE_lexicon_tags_2 },
    { 0x451ef59c, SUC_NE_lexicon_tags_0 },
    { 0xee38f59d, SUC_NE_lexicon_tags_2 },
    { 0x585ef59f, SUC_NE_lexicon_tags_2 },
    { 0xd16af5a0, SUC_NE_lexicon_tags_2 },
    { 0xd84cf5a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6100f5a3, SUC_NE_lexicon_tags_2 },
    { 0x3734f5a4, SUC_NE_lexicon_tags_2 },
    { 0x4e12f5a5, SUC_NE_lexicon_tags_2 },
    { 0x4d26f5a6, SUC_NE_lexicon_tags_4 },
    { 0x65e0f5a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b2f5ac, SUC_NE_lexicon_tags_2 },
    { 0xffe8f5ad, SUC_NE_lexicon_tags_2 },
    { 0x379ef5ae, SUC_NE_lexicon_tags_2 },
    { 0x620ef5af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa260f5b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecf0f5b8, SUC_NE_lexicon_tags_4 },
    { 0x60a0f5b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa8f5bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885ef5c0, SUC_NE_lexicon_tags_2 },
    { 0x142f5c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f4f5c4, SUC_NE_lexicon_tags_2 },
    { 0xff3ef5c4, SUC_NE_lexicon_tags_2 },
    { 0xe13ef5c5, SUC_NE_lexicon_tags_2 },
    { 0xcc08f5c6, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c04f5cb, SUC_NE_lexicon_tags_2 },
    { 0x3092f5cc, SUC_NE_lexicon_tags_2 },
    { 0xa7eaf5cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6f5d2, SUC_NE_lexicon_tags_2 },
    { 0xd638f5d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c8f5d5, SUC_NE_lexicon_tags_5 },
    { 0xe286f5d6, SUC_NE_lexicon_tags_2 },
    { 0x81f6f5d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb36f5da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd8f5df, SUC_NE_lexicon_tags_7 },
    { 0xaa66f5e0, SUC_NE_lexicon_tags_2 },
    { 0xdfaaf5df, SUC_NE_lexicon_tags_2 },
    { 0x7ebef5df, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e98f5e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9a0f5e9, SUC_NE_lexicon_tags_4 },
    { 0xf15cf5e9, SUC_NE_lexicon_tags_12 },
    { 0x192cf5e9, SUC_NE_lexicon_tags_2 },
    { 0x4a5ef5ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2aaf5ee, SUC_NE_lexicon_tags_2 },
    { 0x54b4f5ee, SUC_NE_lexicon_tags_2 },
    { 0xc244f5f0, SUC_NE_lexicon_tags_2 },
    { 0xb32af5f0, SUC_NE_lexicon_tags_4 },
    { 0xa706f5f2, SUC_NE_lexicon_tags_2 },
    { 0x1e38f5f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe746f5f5, SUC_NE_lexicon_tags_2 },
    { 0xd634f5f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5980f5f9, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fef5fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc864f5ff, SUC_NE_lexicon_tags_2 },
    { 0x4ba0f600, SUC_NE_lexicon_tags_4 },
    { 0xfaa4f600, SUC_NE_lexicon_tags_17 },
    { 0x19b4f602, SUC_NE_lexicon_tags_2 },
    { 0xf9cef600, SUC_NE_lexicon_tags_2 },
    { 0xd76af604, SUC_NE_lexicon_tags_2 },
    { 0x4abaf605, SUC_NE_lexicon_tags_2 },
    { 0x4ac0f600, SUC_NE_lexicon_tags_2 },
    { 0x8de8f607, SUC_NE_lexicon_tags_2 },
    { 0xb990f608, SUC_NE_lexicon_tags_2 },
    { 0xca68f609, SUC_NE_lexicon_tags_2 },
    { 0x34ccf60a, SUC_NE_lexicon_tags_2 },
    { 0xd81cf60a, SUC_NE_lexicon_tags_2 },
    { 0x4ea8f608, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb18f60e, SUC_NE_lexicon_tags_2 },
    { 0xa68af60f, SUC_NE_lexicon_tags_2 },
    { 0x8344f610, SUC_NE_lexicon_tags_3 },
    { 0xfe98f610, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca16f613, SUC_NE_lexicon_tags_2 },
    { 0x2bd8f614, SUC_NE_lexicon_tags_2 },
    { 0xc374f615, SUC_NE_lexicon_tags_2 },
    { 0x6df4f615, SUC_NE_lexicon_tags_2 },
    { 0x773af617, SUC_NE_lexicon_tags_2 },
    { 0xd3d0f618, SUC_NE_lexicon_tags_4 },
    { 0x7142f619, SUC_NE_lexicon_tags_2 },
    { 0x544ef61a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2284f61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7acf61f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdaf623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2def62a, SUC_NE_lexicon_tags_2 },
    { 0x1d7af62b, SUC_NE_lexicon_tags_2 },
    { 0x391cf62b, SUC_NE_lexicon_tags_2 },
    { 0xafaf62a, SUC_NE_lexicon_tags_123 },
    { 0x4ca8f62e, SUC_NE_lexicon_tags_0 },
    { 0x1ef4f62f, SUC_NE_lexicon_tags_2 },
    { 0x8f4f630, SUC_NE_lexicon_tags_2 },
    { 0x45caf631, SUC_NE_lexicon_tags_2 },
    { 0xe01cf62f, SUC_NE_lexicon_tags_2 },
    { 0x8822f633, SUC_NE_lexicon_tags_2 },
    { 0x6c9af632, SUC_NE_lexicon_tags_2 },
    { 0x2b96f630, SUC_NE_lexicon_tags_2 },
    { 0xe9f8f630, SUC_NE_lexicon_tags_2 },
    { 0x38af631, SUC_NE_lexicon_tags_2 },
    { 0x3c1cf638, SUC_NE_lexicon_tags_2 },
    { 0xfef6f639, SUC_NE_lexicon_tags_2 },
    { 0x1360f630, SUC_NE_lexicon_tags_2 },
    { 0xe1d0f63b, SUC_NE_lexicon_tags_2 },
    { 0x292f63c, SUC_NE_lexicon_tags_4 },
    { 0xceb6f633, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb0f63f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada2f642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb34f648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aecf64b, SUC_NE_lexicon_tags_2 },
    { 0xbe42f64c, SUC_NE_lexicon_tags_2 },
    { 0x3840f64d, SUC_NE_lexicon_tags_2 },
    { 0xde64f64e, SUC_NE_lexicon_tags_2 },
    { 0x305cf64f, SUC_NE_lexicon_tags_2 },
    { 0xd27ef64c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb530f653, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf656, SUC_NE_lexicon_tags_2 },
    { 0xcfbef656, SUC_NE_lexicon_tags_2 },
    { 0x91b8f658, SUC_NE_lexicon_tags_17 },
    { 0xaff2f659, SUC_NE_lexicon_tags_2 },
    { 0xbc60f658, SUC_NE_lexicon_tags_2 },
    { 0xce5cf658, SUC_NE_lexicon_tags_2 },
    { 0xfb5af65c, SUC_NE_lexicon_tags_2 },
    { 0xe8ef65d, SUC_NE_lexicon_tags_2 },
    { 0xc418f65e, SUC_NE_lexicon_tags_2 },
    { 0x8a5af65f, SUC_NE_lexicon_tags_2 },
    { 0x2a88f660, SUC_NE_lexicon_tags_2 },
    { 0xf436f661, SUC_NE_lexicon_tags_14 },
    { 0x30e0f662, SUC_NE_lexicon_tags_240 },
    { 0xf90af663, SUC_NE_lexicon_tags_2 },
    { 0x7d5af662, SUC_NE_lexicon_tags_2 },
    { 0x80b6f665, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1af669, SUC_NE_lexicon_tags_2 },
    { 0xde30f66a, SUC_NE_lexicon_tags_2 },
    { 0xc040f66a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed44f66d, SUC_NE_lexicon_tags_49 },
    { 0xe486f66d, SUC_NE_lexicon_tags_2 },
    { 0xd650f66d, SUC_NE_lexicon_tags_2 },
    { 0x914cf670, SUC_NE_lexicon_tags_2 },
    { 0x64ccf66d, SUC_NE_lexicon_tags_4 },
    { 0x72d0f672, SUC_NE_lexicon_tags_2 },
    { 0x4692f673, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x642cf676, SUC_NE_lexicon_tags_2 },
    { 0xc6a8f677, SUC_NE_lexicon_tags_2 },
    { 0x5e58f678, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea90f67f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4e6f681, SUC_NE_lexicon_tags_3 },
    { 0x24d0f682, SUC_NE_lexicon_tags_2 },
    { 0x9cfcf681, SUC_NE_lexicon_tags_2 },
    { 0x1a2ef684, SUC_NE_lexicon_tags_2 },
    { 0x9338f685, SUC_NE_lexicon_tags_7 },
    { 0x6f62f686, SUC_NE_lexicon_tags_2 },
    { 0x12ef682, SUC_NE_lexicon_tags_109 },
    { 0x31c2f688, SUC_NE_lexicon_tags_2 },
    { 0x96c6f682, SUC_NE_lexicon_tags_2 },
    { 0x8d62f682, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xe9e8f68c, SUC_NE_lexicon_tags_2 },
    { 0x11ccf68d, SUC_NE_lexicon_tags_2 },
    { 0x75ecf68e, SUC_NE_lexicon_tags_2 },
    { 0xb638f68e, SUC_NE_lexicon_tags_2 },
    { 0x344ef68f, SUC_NE_lexicon_tags_2 },
    { 0x6e88f68d, SUC_NE_lexicon_tags_25 },
    { 0x4b4cf692, SUC_NE_lexicon_tags_2 },
    { 0xb310f692, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eaef695, SUC_NE_lexicon_tags_2 },
    { 0xe9af696, SUC_NE_lexicon_tags_3 },
    { 0xef64f697, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2122f69d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3af6a6, SUC_NE_lexicon_tags_1 },
    { 0x84d4f6a7, SUC_NE_lexicon_tags_49 },
    { 0x96ccf6a8, SUC_NE_lexicon_tags_4 },
    { 0xa112f6a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bcf6ad, SUC_NE_lexicon_tags_4 },
    { 0x2e72f6ad, SUC_NE_lexicon_tags_2 },
    { 0x6ff2f6ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1290f6b2, SUC_NE_lexicon_tags_2 },
    { 0x195af6b2, SUC_NE_lexicon_tags_2 },
    { 0xe6f2f6b4, SUC_NE_lexicon_tags_51 },
    { 0x2292f6b5, SUC_NE_lexicon_tags_2 },
    { 0xf76cf6b6, SUC_NE_lexicon_tags_2 },
    { 0xe5baf6b4, SUC_NE_lexicon_tags_3 },
    { 0xdde8f6b8, SUC_NE_lexicon_tags_2 },
    { 0x9178f6b8, SUC_NE_lexicon_tags_2 },
    { 0xcab0f6ba, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x5ab6f6bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x338f6bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd1ef6c1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfef6c4, SUC_NE_lexicon_tags_2 },
    { 0x27e0f6c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa256f6c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdef6cc, SUC_NE_lexicon_tags_2 },
    { 0x4908f6cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eef6d0, SUC_NE_lexicon_tags_2 },
    { 0x89daf6d1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada6f6d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac4f6d8, SUC_NE_lexicon_tags_2 },
    { 0x1a5ef6d9, SUC_NE_lexicon_tags_2 },
    { 0xdbdcf6d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d4f6df, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad6f6e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b2f6e8, SUC_NE_lexicon_tags_2 },
    { 0x1bb2f6e9, SUC_NE_lexicon_tags_2 },
    { 0x2dcf6ea, SUC_NE_lexicon_tags_2 },
    { 0x973ef6eb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3bef6ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a58f6f3, SUC_NE_lexicon_tags_7 },
    { 0x8572f6f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ae6f6f6, SUC_NE_lexicon_tags_2 },
    { 0xed00f6f7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfd06f6f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba08f6fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ccf702, SUC_NE_lexicon_tags_2 },
    { 0xb19cf702, SUC_NE_lexicon_tags_2 },
    { 0x6afef704, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d92f707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfebaf709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8894f70b, SUC_NE_lexicon_tags_2 },
    { 0x92e2f70c, SUC_NE_lexicon_tags_7 },
    { 0x81eef70b, SUC_NE_lexicon_tags_2 },
    { 0xb0af70b, SUC_NE_lexicon_tags_2 },
    { 0xcdacf70c, SUC_NE_lexicon_tags_2 },
    { 0x304f710, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b0f713, SUC_NE_lexicon_tags_2 },
    { 0x404cf714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1450f719, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258f71f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe638f723, SUC_NE_lexicon_tags_4 },
    { 0x7e84f724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1970f726, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5ef729, SUC_NE_lexicon_tags_2 },
    { 0x4bf0f72a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43d0f72c, SUC_NE_lexicon_tags_3 },
    { 0xbf72f72c, SUC_NE_lexicon_tags_32 },
    { 0x7d14f72c, SUC_NE_lexicon_tags_2 },
    { 0x374cf72d, SUC_NE_lexicon_tags_2 },
    { 0x71faf72c, SUC_NE_lexicon_tags_4 },
    { 0x1d32f72d, SUC_NE_lexicon_tags_3 },
    { 0x3918f732, SUC_NE_lexicon_tags_2 },
    { 0xe564f72d, SUC_NE_lexicon_tags_2 },
    { 0x24f734, SUC_NE_lexicon_tags_2 },
    { 0x6174f735, SUC_NE_lexicon_tags_2 },
    { 0x174ef735, SUC_NE_lexicon_tags_2 },
    { 0xb6c2f737, SUC_NE_lexicon_tags_2 },
    { 0x876af738, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20a8f73a, SUC_NE_lexicon_tags_2 },
    { 0xcfaef73a, SUC_NE_lexicon_tags_2 },
    { 0xf76f73c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf32f73e, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b4f741, SUC_NE_lexicon_tags_2 },
    { 0x6f76f742, SUC_NE_lexicon_tags_2 },
    { 0x1d54f743, SUC_NE_lexicon_tags_2 },
    { 0x6372f744, SUC_NE_lexicon_tags_2 },
    { 0xac08f745, SUC_NE_lexicon_tags_2 },
    { 0x3d54f741, SUC_NE_lexicon_tags_7 },
    { 0xdeb4f747, SUC_NE_lexicon_tags_2 },
    { 0xe22cf742, SUC_NE_lexicon_tags_24 },
    { 0xef0ef746, SUC_NE_lexicon_tags_2 },
    { 0xe5fcf74a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa384f74c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db8f753, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5630f75a, SUC_NE_lexicon_tags_2 },
    { 0xba02f75b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc494f75d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aef763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x535cf766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x364af768, SUC_NE_lexicon_tags_2 },
    { 0xefe4f769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd06f76c, SUC_NE_lexicon_tags_267 },
    { 0xd5bef76c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dcf771, SUC_NE_lexicon_tags_2 },
    { 0x1f8af772, SUC_NE_lexicon_tags_2 },
    { 0xdea8f771, SUC_NE_lexicon_tags_2 },
    { 0x2258f773, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a74f776, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xf342f778, SUC_NE_lexicon_tags_2 },
    { 0x9feef779, SUC_NE_lexicon_tags_3 },
    { 0x6f50f77a, SUC_NE_lexicon_tags_4 },
    { 0x11baf77b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5438f780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e7ef782, SUC_NE_lexicon_tags_2 },
    { 0x6ac0f783, SUC_NE_lexicon_tags_3 },
    { 0xc6fef784, SUC_NE_lexicon_tags_2 },
    { 0x43baf784, SUC_NE_lexicon_tags_2 },
    { 0x5b54f786, SUC_NE_lexicon_tags_2 },
    { 0x7a30f787, SUC_NE_lexicon_tags_2 },
    { 0x8f0cf786, SUC_NE_lexicon_tags_2 },
    { 0x4982f784, SUC_NE_lexicon_tags_2 },
    { 0x99fef786, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe884f78d, SUC_NE_lexicon_tags_17 },
    { 0x2b08f78e, SUC_NE_lexicon_tags_2 },
    { 0xab38f78d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc240f791, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1af798, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ecf79c, SUC_NE_lexicon_tags_2 },
    { 0x7938f79d, SUC_NE_lexicon_tags_2 },
    { 0xe836f79d, SUC_NE_lexicon_tags_2 },
    { 0x932cf79f, SUC_NE_lexicon_tags_2 },
    { 0x9c94f7a0, SUC_NE_lexicon_tags_2 },
    { 0x7f0cf7a1, SUC_NE_lexicon_tags_2 },
    { 0x7aa6f79c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf690f7a4, SUC_NE_lexicon_tags_2 },
    { 0x8c90f7a5, SUC_NE_lexicon_tags_2 },
    { 0x8018f7a6, SUC_NE_lexicon_tags_2 },
    { 0x665ef7a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7af7aa, SUC_NE_lexicon_tags_40 },
    { 0x143ef7aa, SUC_NE_lexicon_tags_2 },
    { 0xf5aef7ac, SUC_NE_lexicon_tags_2 },
    { 0xf538f7ad, SUC_NE_lexicon_tags_2 },
    { 0xebb4f7ac, SUC_NE_lexicon_tags_2 },
    { 0x2748f7af, SUC_NE_lexicon_tags_2 },
    { 0x9fcaf7b0, SUC_NE_lexicon_tags_2 },
    { 0x3024f7b1, SUC_NE_lexicon_tags_2 },
    { 0xbd8ef7b2, SUC_NE_lexicon_tags_64 },
    { 0xef4cf7b2, SUC_NE_lexicon_tags_2 },
    { 0xd446f7b4, SUC_NE_lexicon_tags_2 },
    { 0x4facf7b3, SUC_NE_lexicon_tags_17 },
    { 0x5fb0f7b6, SUC_NE_lexicon_tags_89 },
    { 0x394cf7b7, SUC_NE_lexicon_tags_1 },
    { 0x815ef7b4, SUC_NE_lexicon_tags_2 },
    { 0x860af7b9, SUC_NE_lexicon_tags_2 },
    { 0x66d0f7ba, SUC_NE_lexicon_tags_2 },
    { 0x18e8f7ba, SUC_NE_lexicon_tags_2 },
    { 0x448cf7bc, SUC_NE_lexicon_tags_2 },
    { 0x1820f7ba, SUC_NE_lexicon_tags_2 },
    { 0xb6bef7be, SUC_NE_lexicon_tags_2 },
    { 0x5564f7be, SUC_NE_lexicon_tags_2 },
    { 0x9268f7c0, SUC_NE_lexicon_tags_3 },
    { 0x6f70f7be, SUC_NE_lexicon_tags_3 },
    { 0xbaf2f7b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67ef7c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f42f7ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd88f7cd, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e12f7d2, SUC_NE_lexicon_tags_4 },
    { 0x6bcf7d3, SUC_NE_lexicon_tags_2 },
    { 0x6d1af7d4, SUC_NE_lexicon_tags_36 },
    { 0x9882f7d5, SUC_NE_lexicon_tags_2 },
    { 0x5336f7d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf62f7d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd512f7dc, SUC_NE_lexicon_tags_2 },
    { 0xa616f7dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2698f7e4, SUC_NE_lexicon_tags_2 },
    { 0xcb5ef7e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x713af7e7, SUC_NE_lexicon_tags_10 },
    { 0xd6ccf7e7, SUC_NE_lexicon_tags_2 },
    { 0x805ef7e8, SUC_NE_lexicon_tags_2 },
    { 0x6df2f7ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53af7f1, SUC_NE_lexicon_tags_24 },
    { 0x2ecaf7f1, SUC_NE_lexicon_tags_2 },
    { 0xdc52f7f3, SUC_NE_lexicon_tags_25 },
    { 0xed9ef7f1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df8f7f9, SUC_NE_lexicon_tags_3 },
    { 0x7b4af7fa, SUC_NE_lexicon_tags_2 },
    { 0xe88f7f9, SUC_NE_lexicon_tags_19 },
    { 0x9864f7fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d24f7ff, SUC_NE_lexicon_tags_2 },
    { 0xed08f800, SUC_NE_lexicon_tags_2 },
    { 0xbc0cf801, SUC_NE_lexicon_tags_2 },
    { 0x1c18f7ff, SUC_NE_lexicon_tags_2 },
    { 0x91e2f801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b28f805, SUC_NE_lexicon_tags_2 },
    { 0x279cf806, SUC_NE_lexicon_tags_2 },
    { 0xaf38f806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f80d, SUC_NE_lexicon_tags_2 },
    { 0x88d4f80d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf882f812, SUC_NE_lexicon_tags_2 },
    { 0x3d60f812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44f817, SUC_NE_lexicon_tags_2 },
    { 0xe7d8f817, SUC_NE_lexicon_tags_2 },
    { 0x9fbef817, SUC_NE_lexicon_tags_2 },
    { 0x879ef817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d60f81d, SUC_NE_lexicon_tags_2 },
    { 0xad2f81d, SUC_NE_lexicon_tags_109 },
    { 0x2366f81f, SUC_NE_lexicon_tags_2 },
    { 0x55ccf820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c4f823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc966f833, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc2f836, SUC_NE_lexicon_tags_18 },
    { 0x6daef836, SUC_NE_lexicon_tags_2 },
    { 0x8ea8f837, SUC_NE_lexicon_tags_2 },
    { 0xe18ef837, SUC_NE_lexicon_tags_2 },
    { 0x9508f839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a14f83d, SUC_NE_lexicon_tags_2 },
    { 0x3e32f83e, SUC_NE_lexicon_tags_17 },
    { 0x44fcf83f, SUC_NE_lexicon_tags_2 },
    { 0x4248f840, SUC_NE_lexicon_tags_2 },
    { 0x36d6f841, SUC_NE_lexicon_tags_2 },
    { 0xceaaf840, SUC_NE_lexicon_tags_2 },
    { 0x375cf843, SUC_NE_lexicon_tags_2 },
    { 0xd07cf83e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dc2f846, SUC_NE_lexicon_tags_2 },
    { 0xbe4ef847, SUC_NE_lexicon_tags_2 },
    { 0x902cf847, SUC_NE_lexicon_tags_2 },
    { 0x7f00f849, SUC_NE_lexicon_tags_2 },
    { 0x6bc2f84a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ef84d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x854ef84f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa624f851, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba8f85a, SUC_NE_lexicon_tags_2 },
    { 0x1ccaf85b, SUC_NE_lexicon_tags_2 },
    { 0xf942f85c, SUC_NE_lexicon_tags_2 },
    { 0xf1bcf85c, SUC_NE_lexicon_tags_2 },
    { 0x5d4ef85e, SUC_NE_lexicon_tags_2 },
    { 0x4c60f85d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff16f863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f0f865, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b0f868, SUC_NE_lexicon_tags_2 },
    { 0x8d9ef869, SUC_NE_lexicon_tags_2 },
    { 0x628cf868, SUC_NE_lexicon_tags_2 },
    { 0x7940f86b, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b74f86f, SUC_NE_lexicon_tags_285 },
    { 0x9334f86f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaf874, SUC_NE_lexicon_tags_2 },
    { 0x3456f874, SUC_NE_lexicon_tags_2 },
    { 0xb100f876, SUC_NE_lexicon_tags_2 },
    { 0x1522f877, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7586f87a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1acf87c, SUC_NE_lexicon_tags_2 },
    { 0x4484f87c, SUC_NE_lexicon_tags_2 },
    { 0x8496f87c, SUC_NE_lexicon_tags_2 },
    { 0x68acf87d, SUC_NE_lexicon_tags_2 },
    { 0xf7d4f880, SUC_NE_lexicon_tags_32 },
    { 0x2742f881, SUC_NE_lexicon_tags_2 },
    { 0xe496f882, SUC_NE_lexicon_tags_2 },
    { 0xdd90f883, SUC_NE_lexicon_tags_2 },
    { 0x59fcf883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b32f888, SUC_NE_lexicon_tags_18 },
    { 0x94b0f889, SUC_NE_lexicon_tags_2 },
    { 0x9870f88a, SUC_NE_lexicon_tags_49 },
    { 0x8b24f888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa73af88d, SUC_NE_lexicon_tags_2 },
    { 0x4ad8f88e, SUC_NE_lexicon_tags_2 },
    { 0xb33ef88f, SUC_NE_lexicon_tags_2 },
    { 0xb634f890, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1af892, SUC_NE_lexicon_tags_2 },
    { 0x6520f893, SUC_NE_lexicon_tags_2 },
    { 0x9334f894, SUC_NE_lexicon_tags_2 },
    { 0xb676f895, SUC_NE_lexicon_tags_17 },
    { 0xd556f896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb870f89a, SUC_NE_lexicon_tags_2 },
    { 0x2fd6f89b, SUC_NE_lexicon_tags_3 },
    { 0x35eef89b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5cf8a0, SUC_NE_lexicon_tags_4 },
    { 0x4810f8a1, SUC_NE_lexicon_tags_2 },
    { 0x48f4f8a2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3176f8a5, SUC_NE_lexicon_tags_2 },
    { 0x5272f8a5, SUC_NE_lexicon_tags_2 },
    { 0x306cf8a6, SUC_NE_lexicon_tags_2 },
    { 0x1382f8a5, SUC_NE_lexicon_tags_2 },
    { 0x5e5af8a9, SUC_NE_lexicon_tags_2 },
    { 0x6538f8aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e4cf8ac, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x74daf8ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa68f8b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9acf8b2, SUC_NE_lexicon_tags_2 },
    { 0x606af8b3, SUC_NE_lexicon_tags_2 },
    { 0x586ef8b2, SUC_NE_lexicon_tags_2 },
    { 0xb968f8b5, SUC_NE_lexicon_tags_2 },
    { 0xdc16f8b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa29af8b8, SUC_NE_lexicon_tags_2 },
    { 0xb22ef8b9, SUC_NE_lexicon_tags_2 },
    { 0x93dcf8b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58af8c1, SUC_NE_lexicon_tags_2 },
    { 0xe2cf8c2, SUC_NE_lexicon_tags_2 },
    { 0xea14f8c3, SUC_NE_lexicon_tags_2 },
    { 0x4d6ef8c4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2ef8cc, SUC_NE_lexicon_tags_2 },
    { 0xa316f8cd, SUC_NE_lexicon_tags_2 },
    { 0x5270f8ce, SUC_NE_lexicon_tags_2 },
    { 0x1c96f8cf, SUC_NE_lexicon_tags_2 },
    { 0xcbc8f8d0, SUC_NE_lexicon_tags_2 },
    { 0x92eaf8d1, SUC_NE_lexicon_tags_4 },
    { 0xbbf6f8cf, SUC_NE_lexicon_tags_3 },
    { 0x16e6f8cd, SUC_NE_lexicon_tags_2 },
    { 0x5b68f8d4, SUC_NE_lexicon_tags_2 },
    { 0x495af8ce, SUC_NE_lexicon_tags_2 },
    { 0x680ef8d6, SUC_NE_lexicon_tags_2 },
    { 0x99b8f8d7, SUC_NE_lexicon_tags_2 },
    { 0x333ef8cc, SUC_NE_lexicon_tags_2 },
    { 0xab4ef8d9, SUC_NE_lexicon_tags_2 },
    { 0x685cf8d4, SUC_NE_lexicon_tags_2 },
    { 0xbb5ef8db, SUC_NE_lexicon_tags_2 },
    { 0xf46f8dc, SUC_NE_lexicon_tags_2 },
    { 0xfccaf8dd, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735cf8e2, SUC_NE_lexicon_tags_2 },
    { 0xb3e8f8e3, SUC_NE_lexicon_tags_2 },
    { 0x55e8f8e4, SUC_NE_lexicon_tags_2 },
    { 0x20fef8e5, SUC_NE_lexicon_tags_2 },
    { 0xff26f8e3, SUC_NE_lexicon_tags_2 },
    { 0x9dc8f8e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6cf8e9, SUC_NE_lexicon_tags_2 },
    { 0x2090f8e9, SUC_NE_lexicon_tags_2 },
    { 0x8d22f8ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0af8ef, SUC_NE_lexicon_tags_2 },
    { 0xa240f8f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751ef8f5, SUC_NE_lexicon_tags_2 },
    { 0x3d4f8f6, SUC_NE_lexicon_tags_2 },
    { 0xfb32f8f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x746ef8f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5680f8fd, SUC_NE_lexicon_tags_23 },
    { 0xa210f8fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x465cf900, SUC_NE_lexicon_tags_2 },
    { 0xaf24f901, SUC_NE_lexicon_tags_2 },
    { 0x154af902, SUC_NE_lexicon_tags_2 },
    { 0x31d2f900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6ecf905, SUC_NE_lexicon_tags_2 },
    { 0x7008f906, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c4f909, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc36f90b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0f90e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2714f910, SUC_NE_lexicon_tags_2 },
    { 0xaf96f911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd6f91c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24cf91f, SUC_NE_lexicon_tags_36 },
    { 0x5faaf920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93b8f922, SUC_NE_lexicon_tags_2 },
    { 0xc800f923, SUC_NE_lexicon_tags_2 },
    { 0xb206f924, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf26f926, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e6f92b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf372f92d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa128f933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd8f93c, SUC_NE_lexicon_tags_2 },
    { 0x4c08f93d, SUC_NE_lexicon_tags_2 },
    { 0x3c70f93e, SUC_NE_lexicon_tags_2 },
    { 0xcdbcf93d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d8f942, SUC_NE_lexicon_tags_2 },
    { 0xa3a8f943, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a8f94a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33cf94c, SUC_NE_lexicon_tags_2 },
    { 0x6786f94d, SUC_NE_lexicon_tags_2 },
    { 0x75c4f94e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe16af950, SUC_NE_lexicon_tags_2 },
    { 0x35d8f951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e4f956, SUC_NE_lexicon_tags_2 },
    { 0x6c0ef957, SUC_NE_lexicon_tags_2 },
    { 0xe3d0f958, SUC_NE_lexicon_tags_2 },
    { 0x230ef956, SUC_NE_lexicon_tags_2 },
    { 0x3876f95a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fcf95d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9614f95f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefaf965, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8f969, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xcea4f96b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463af970, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x25faf972, SUC_NE_lexicon_tags_2 },
    { 0xb690f972, SUC_NE_lexicon_tags_2 },
    { 0xdef973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7156f976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9784f978, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ecf97b, SUC_NE_lexicon_tags_2 },
    { 0xb7a4f97b, SUC_NE_lexicon_tags_36 },
    { 0x6faaf97d, SUC_NE_lexicon_tags_2 },
    { 0x62c0f97d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92f989, SUC_NE_lexicon_tags_2 },
    { 0x3a46f98a, SUC_NE_lexicon_tags_2 },
    { 0xc6eef98b, SUC_NE_lexicon_tags_4 },
    { 0xa386f98b, SUC_NE_lexicon_tags_2 },
    { 0x2aecf98a, SUC_NE_lexicon_tags_59 },
    { 0x2ce2f98e, SUC_NE_lexicon_tags_2 },
    { 0xbb04f98e, SUC_NE_lexicon_tags_2 },
    { 0x2facf98e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4feaf997, SUC_NE_lexicon_tags_2 },
    { 0x1e2af997, SUC_NE_lexicon_tags_2 },
    { 0x4098f999, SUC_NE_lexicon_tags_117 },
    { 0x6ec8f997, SUC_NE_lexicon_tags_2 },
    { 0x409af99a, SUC_NE_lexicon_tags_17 },
    { 0x8de0f999, SUC_NE_lexicon_tags_18 },
    { 0xf2d8f99d, SUC_NE_lexicon_tags_2 },
    { 0x6e70f997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1ef9a1, SUC_NE_lexicon_tags_2 },
    { 0x677af9a1, SUC_NE_lexicon_tags_2 },
    { 0xaf40f9a3, SUC_NE_lexicon_tags_2 },
    { 0xd60ef9a4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a8f9a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f0f9ac, SUC_NE_lexicon_tags_2 },
    { 0x3ed4f9ad, SUC_NE_lexicon_tags_2 },
    { 0xf2a4f9ac, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d48f9b0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbb2ef9b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf82f9b6, SUC_NE_lexicon_tags_7 },
    { 0xc4acf9b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x243af9b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0f9be, SUC_NE_lexicon_tags_2 },
    { 0xda20f9be, SUC_NE_lexicon_tags_2 },
    { 0x3e7cf9bf, SUC_NE_lexicon_tags_2 },
    { 0x45bef9bf, SUC_NE_lexicon_tags_2 },
    { 0x4ac0f9c2, SUC_NE_lexicon_tags_2 },
    { 0x5ec6f9c2, SUC_NE_lexicon_tags_2 },
    { 0xfa34f9c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbe2f9c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0ef9cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8eaaf9cd, SUC_NE_lexicon_tags_2 },
    { 0x7ce0f9ce, SUC_NE_lexicon_tags_2 },
    { 0xb1ef9cd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5666f9d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0cf9dc, SUC_NE_lexicon_tags_2 },
    { 0x3976f9dd, SUC_NE_lexicon_tags_2 },
    { 0x61a6f9dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe350f9e1, SUC_NE_lexicon_tags_2 },
    { 0xb578f9e2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e4f9e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c0f9e8, SUC_NE_lexicon_tags_2 },
    { 0xf7eef9e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98d8f9eb, SUC_NE_lexicon_tags_2 },
    { 0xed06f9ec, SUC_NE_lexicon_tags_2 },
    { 0xd948f9eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5874f9f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eaf9f6, SUC_NE_lexicon_tags_2 },
    { 0x1430f9f6, SUC_NE_lexicon_tags_147 },
    { 0x73e6f9f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2ef9fe, SUC_NE_lexicon_tags_2 },
    { 0xbd46f9fe, SUC_NE_lexicon_tags_2 },
    { 0xb5e0fa00, SUC_NE_lexicon_tags_2 },
    { 0x43d8fa01, SUC_NE_lexicon_tags_2 },
    { 0x6840fa02, SUC_NE_lexicon_tags_2 },
    { 0x75f0f9fe, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x861cfa05, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xd0a4fa07, SUC_NE_lexicon_tags_403 },
    { 0xf09efa08, SUC_NE_lexicon_tags_2 },
    { 0x30aefa09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6778fa0d, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xe85afa0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54d6fa11, SUC_NE_lexicon_tags_2 },
    { 0xc242fa12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed6fa15, SUC_NE_lexicon_tags_2 },
    { 0xcbfcfa16, SUC_NE_lexicon_tags_2 },
    { 0x13e0fa17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x544cfa19, SUC_NE_lexicon_tags_2 },
    { 0xea8cfa1a, SUC_NE_lexicon_tags_2 },
    { 0x4464fa1b, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf4fa23, SUC_NE_lexicon_tags_1 },
    { 0xcf6fa24, SUC_NE_lexicon_tags_2 },
    { 0xfb76fa25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c02fa30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cd0fa32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f72fa34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e34fa39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44acfa3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a20fa3e, SUC_NE_lexicon_tags_2 },
    { 0x9aa8fa3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4cafa41, SUC_NE_lexicon_tags_2 },
    { 0xc75afa41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272afa46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x935cfa48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb8fa4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a36fa4e, SUC_NE_lexicon_tags_2 },
    { 0x7852fa4f, SUC_NE_lexicon_tags_2 },
    { 0xf77cfa50, SUC_NE_lexicon_tags_2 },
    { 0x3394fa51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f0fa54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f9afa56, SUC_NE_lexicon_tags_58 },
    { 0x3b8afa57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d28fa5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaefa65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7530fa69, SUC_NE_lexicon_tags_2 },
    { 0xdf30fa6a, SUC_NE_lexicon_tags_2 },
    { 0x4fe8fa6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa4fa71, SUC_NE_lexicon_tags_4 },
    { 0x111afa71, SUC_NE_lexicon_tags_2 },
    { 0x388afa73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc254fa77, SUC_NE_lexicon_tags_2 },
    { 0xe20efa78, SUC_NE_lexicon_tags_2 },
    { 0xddaefa79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf8fa7d, SUC_NE_lexicon_tags_2 },
    { 0x846afa7e, SUC_NE_lexicon_tags_2 },
    { 0x768fa7f, SUC_NE_lexicon_tags_2 },
    { 0xbdc8fa80, SUC_NE_lexicon_tags_2 },
    { 0xe0ecfa81, SUC_NE_lexicon_tags_2 },
    { 0xe172fa81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f30fa84, SUC_NE_lexicon_tags_2 },
    { 0xbce4fa85, SUC_NE_lexicon_tags_2 },
    { 0xb720fa86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a46fa88, SUC_NE_lexicon_tags_2 },
    { 0x9b6afa89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe40cfa8b, SUC_NE_lexicon_tags_2 },
    { 0x75b0fa8c, SUC_NE_lexicon_tags_34 },
    { 0x7322fa8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2fa92, SUC_NE_lexicon_tags_19 },
    { 0xa960fa92, SUC_NE_lexicon_tags_2 },
    { 0xf092fa94, SUC_NE_lexicon_tags_4 },
    { 0x590afa95, SUC_NE_lexicon_tags_2 },
    { 0xecc8fa96, SUC_NE_lexicon_tags_2 },
    { 0xb4b2fa97, SUC_NE_lexicon_tags_2 },
    { 0x706afa96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9460fa9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59b8fa9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe112faa4, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x4060faa6, SUC_NE_lexicon_tags_2 },
    { 0xd88cfaa7, SUC_NE_lexicon_tags_19 },
    { 0xeedcfaa8, SUC_NE_lexicon_tags_2 },
    { 0x84f4faa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe428faab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a2efaad, SUC_NE_lexicon_tags_1 },
    { 0xdcc0faae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc028fab2, SUC_NE_lexicon_tags_2 },
    { 0x2e20fab2, SUC_NE_lexicon_tags_2 },
    { 0xf436fab4, SUC_NE_lexicon_tags_2 },
    { 0x2fd0fab5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eafabe, SUC_NE_lexicon_tags_2 },
    { 0x40b0fabf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x445afac2, SUC_NE_lexicon_tags_2 },
    { 0xa6c6fac3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576fac9, SUC_NE_lexicon_tags_2 },
    { 0x697efaca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fe6facc, SUC_NE_lexicon_tags_2 },
    { 0xd45efacd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4332fad1, SUC_NE_lexicon_tags_2 },
    { 0x13cafad2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4efad4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeefad9, SUC_NE_lexicon_tags_14 },
    { 0x82fcfada, SUC_NE_lexicon_tags_2 },
    { 0xb406fada, SUC_NE_lexicon_tags_13 },
    { 0x7216fadc, SUC_NE_lexicon_tags_426 },
    { 0, NULL },
    { 0x2898fade, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x214efae1, SUC_NE_lexicon_tags_2 },
    { 0xb394fae2, SUC_NE_lexicon_tags_32 },
    { 0x8670fae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x911afae5, SUC_NE_lexicon_tags_2 },
    { 0x24f4fae6, SUC_NE_lexicon_tags_2 },
    { 0xd0acfae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cafaf0, SUC_NE_lexicon_tags_10 },
    { 0xbc52faf1, SUC_NE_lexicon_tags_2 },
    { 0xd4d6faf2, SUC_NE_lexicon_tags_2 },
    { 0x2170faf0, SUC_NE_lexicon_tags_3 },
    { 0xab7efaf2, SUC_NE_lexicon_tags_30 },
    { 0xf01cfaf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0cfaf8, SUC_NE_lexicon_tags_2 },
    { 0xf68efaf9, SUC_NE_lexicon_tags_64 },
    { 0x90f2fafa, SUC_NE_lexicon_tags_2 },
    { 0xb938fafa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fafe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41a2fb00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac02fb02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58cfb05, SUC_NE_lexicon_tags_2 },
    { 0xf662fb06, SUC_NE_lexicon_tags_2 },
    { 0x19cefb07, SUC_NE_lexicon_tags_7 },
    { 0x8fa8fb07, SUC_NE_lexicon_tags_195 },
    { 0x939afb09, SUC_NE_lexicon_tags_2 },
    { 0xb9acfb0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb72fb0e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afb13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c4fb17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa492fb19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab4fb1c, SUC_NE_lexicon_tags_2 },
    { 0x312afb1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cfb1f, SUC_NE_lexicon_tags_1 },
    { 0xb0aefb20, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593cfb28, SUC_NE_lexicon_tags_2 },
    { 0xc0a0fb28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8858fb2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x969cfb30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0cfb32, SUC_NE_lexicon_tags_7 },
    { 0xc9bcfb32, SUC_NE_lexicon_tags_2 },
    { 0xdeb8fb34, SUC_NE_lexicon_tags_2 },
    { 0xecfafb34, SUC_NE_lexicon_tags_2 },
    { 0x937afb36, SUC_NE_lexicon_tags_2 },
    { 0x3776fb37, SUC_NE_lexicon_tags_2 },
    { 0x395cfb38, SUC_NE_lexicon_tags_2 },
    { 0xf506fb39, SUC_NE_lexicon_tags_2 },
    { 0xd81efb3a, SUC_NE_lexicon_tags_2 },
    { 0x1232fb3b, SUC_NE_lexicon_tags_2 },
    { 0xe65afb39, SUC_NE_lexicon_tags_2 },
    { 0x930efb3d, SUC_NE_lexicon_tags_2 },
    { 0x2140fb3e, SUC_NE_lexicon_tags_2 },
    { 0xda94fb3e, SUC_NE_lexicon_tags_2 },
    { 0xd600fb38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cafb43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c4cfb45, SUC_NE_lexicon_tags_2 },
    { 0x3020fb46, SUC_NE_lexicon_tags_2 },
    { 0x8e84fb47, SUC_NE_lexicon_tags_2 },
    { 0xca8efb47, SUC_NE_lexicon_tags_2 },
    { 0xf25cfb49, SUC_NE_lexicon_tags_19 },
    { 0xbc04fb4a, SUC_NE_lexicon_tags_2 },
    { 0x5f34fb4a, SUC_NE_lexicon_tags_36 },
    { 0xd964fb4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6548fb4e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x56ccfb50, SUC_NE_lexicon_tags_2 },
    { 0x3ce8fb50, SUC_NE_lexicon_tags_2 },
    { 0x16dafb52, SUC_NE_lexicon_tags_2 },
    { 0x5446fb51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f6fb57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97bcfb59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa998fb5c, SUC_NE_lexicon_tags_2 },
    { 0x309cfb5c, SUC_NE_lexicon_tags_2 },
    { 0xa9aefb5e, SUC_NE_lexicon_tags_17 },
    { 0x7178fb5c, SUC_NE_lexicon_tags_2 },
    { 0xb934fb60, SUC_NE_lexicon_tags_1 },
    { 0xa5d4fb5f, SUC_NE_lexicon_tags_7 },
    { 0x5ed0fb5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8996fb64, SUC_NE_lexicon_tags_64 },
    { 0xcf6cfb65, SUC_NE_lexicon_tags_2 },
    { 0xabefb66, SUC_NE_lexicon_tags_2 },
    { 0xb6b2fb66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8efb6f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc6fb75, SUC_NE_lexicon_tags_2 },
    { 0x60c0fb76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fb4fb78, SUC_NE_lexicon_tags_2 },
    { 0xa874fb78, SUC_NE_lexicon_tags_2 },
    { 0x4782fb79, SUC_NE_lexicon_tags_2 },
    { 0xa812fb79, SUC_NE_lexicon_tags_2 },
    { 0x9c74fb7c, SUC_NE_lexicon_tags_2 },
    { 0xb764fb7a, SUC_NE_lexicon_tags_2 },
    { 0xf112fb7e, SUC_NE_lexicon_tags_2 },
    { 0xd488fb7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb64fb83, SUC_NE_lexicon_tags_2 },
    { 0xbd9efb83, SUC_NE_lexicon_tags_2 },
    { 0x561efb85, SUC_NE_lexicon_tags_2 },
    { 0xa738fb85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c28fb88, SUC_NE_lexicon_tags_2 },
    { 0x3ec0fb89, SUC_NE_lexicon_tags_2 },
    { 0xd686fb8a, SUC_NE_lexicon_tags_3 },
    { 0xe922fb89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4afb8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aafb92, SUC_NE_lexicon_tags_2 },
    { 0x93c6fb92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcafb97, SUC_NE_lexicon_tags_428 },
    { 0xe8f0fb98, SUC_NE_lexicon_tags_2 },
    { 0x6d5cfb98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77efb9c, SUC_NE_lexicon_tags_2 },
    { 0xed5cfb9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662efba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a94fba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8afbac, SUC_NE_lexicon_tags_2 },
    { 0x5a8afbac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20cfbb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85afbb7, SUC_NE_lexicon_tags_2 },
    { 0xc4c6fbb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd338fbba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75f2fbbc, SUC_NE_lexicon_tags_2 },
    { 0x6e44fbbd, SUC_NE_lexicon_tags_2 },
    { 0x9214fbbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adefbc7, SUC_NE_lexicon_tags_2 },
    { 0x57bafbc8, SUC_NE_lexicon_tags_2 },
    { 0xcddcfbc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x853cfbcb, SUC_NE_lexicon_tags_2 },
    { 0xbbb0fbcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83efbcf, SUC_NE_lexicon_tags_2 },
    { 0x6ab4fbd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6342fbd2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf230fbd6, SUC_NE_lexicon_tags_2 },
    { 0x4abefbd6, SUC_NE_lexicon_tags_2 },
    { 0xf0fafbd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc86fbda, SUC_NE_lexicon_tags_2 },
    { 0x7f8afbdb, SUC_NE_lexicon_tags_2 },
    { 0xefc0fbdb, SUC_NE_lexicon_tags_7 },
    { 0x2822fbdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68fbe4, SUC_NE_lexicon_tags_2 },
    { 0x4da8fbe5, SUC_NE_lexicon_tags_36 },
    { 0xfbf0fbe6, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x15eefbe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef4fbf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefcafbf5, SUC_NE_lexicon_tags_2 },
    { 0x2088fbf5, SUC_NE_lexicon_tags_2 },
    { 0xcef2fbf5, SUC_NE_lexicon_tags_2 },
    { 0x6cb2fbf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a12fbfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd674fbfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa352fc06, SUC_NE_lexicon_tags_2 },
    { 0xe482fc06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x627efc09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01efc0e, SUC_NE_lexicon_tags_2 },
    { 0x9ce8fc0f, SUC_NE_lexicon_tags_2 },
    { 0x8480fc0e, SUC_NE_lexicon_tags_2 },
    { 0x57aafc11, SUC_NE_lexicon_tags_3 },
    { 0x5380fc11, SUC_NE_lexicon_tags_2 },
    { 0x9542fc13, SUC_NE_lexicon_tags_2 },
    { 0xccc0fc14, SUC_NE_lexicon_tags_19 },
    { 0xa922fc14, SUC_NE_lexicon_tags_2 },
    { 0x44d6fc0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8efc19, SUC_NE_lexicon_tags_2 },
    { 0x268afc1a, SUC_NE_lexicon_tags_13 },
    { 0xfe3afc1b, SUC_NE_lexicon_tags_2 },
    { 0x24e6fc1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea7afc1e, SUC_NE_lexicon_tags_2 },
    { 0xe182fc1f, SUC_NE_lexicon_tags_188 },
    { 0xb936fc20, SUC_NE_lexicon_tags_2 },
    { 0x4738fc21, SUC_NE_lexicon_tags_2 },
    { 0x4c8efc22, SUC_NE_lexicon_tags_2 },
    { 0x94f2fc21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eefc27, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d2fc2c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xaaacfc2e, SUC_NE_lexicon_tags_2 },
    { 0xd24cfc2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3baafc31, SUC_NE_lexicon_tags_2 },
    { 0x4220fc32, SUC_NE_lexicon_tags_2 },
    { 0x6362fc33, SUC_NE_lexicon_tags_2 },
    { 0x6b68fc33, SUC_NE_lexicon_tags_2 },
    { 0x3dcafc34, SUC_NE_lexicon_tags_65 },
    { 0x54bcfc33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80fc3d, SUC_NE_lexicon_tags_7 },
    { 0x49defc3e, SUC_NE_lexicon_tags_2 },
    { 0xae7afc3e, SUC_NE_lexicon_tags_2 },
    { 0x621cfc40, SUC_NE_lexicon_tags_2 },
    { 0x8cf8fc41, SUC_NE_lexicon_tags_2 },
    { 0x8d7afc42, SUC_NE_lexicon_tags_5 },
    { 0x4282fc43, SUC_NE_lexicon_tags_2 },
    { 0xaae6fc40, SUC_NE_lexicon_tags_2 },
    { 0xb638fc45, SUC_NE_lexicon_tags_2 },
    { 0xc26fc46, SUC_NE_lexicon_tags_2 },
    { 0xd01cfc47, SUC_NE_lexicon_tags_18 },
    { 0x1b22fc3d, SUC_NE_lexicon_tags_2 },
    { 0xb644fc49, SUC_NE_lexicon_tags_2 },
    { 0x56eafc49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a42fc4e, SUC_NE_lexicon_tags_2 },
    { 0xfee6fc4f, SUC_NE_lexicon_tags_321 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fafc52, SUC_NE_lexicon_tags_2 },
    { 0xe044fc52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b56fc55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf74fc59, SUC_NE_lexicon_tags_2 },
    { 0x694fc5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baafc5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa914fc61, SUC_NE_lexicon_tags_2 },
    { 0x4c2afc62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3752fc65, SUC_NE_lexicon_tags_2 },
    { 0x5a7afc66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdd6fc68, SUC_NE_lexicon_tags_2 },
    { 0x10e0fc68, SUC_NE_lexicon_tags_2 },
    { 0x7edafc6a, SUC_NE_lexicon_tags_2 },
    { 0x46a4fc6b, SUC_NE_lexicon_tags_2 },
    { 0x6eaafc6c, SUC_NE_lexicon_tags_2 },
    { 0x4578fc6c, SUC_NE_lexicon_tags_2 },
    { 0xa6dafc6d, SUC_NE_lexicon_tags_2 },
    { 0xc4aefc68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e4fc73, SUC_NE_lexicon_tags_2 },
    { 0xea00fc74, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c4fc79, SUC_NE_lexicon_tags_2 },
    { 0x90f2fc79, SUC_NE_lexicon_tags_2 },
    { 0x315afc7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781efc7f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ecfc83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c08fc87, SUC_NE_lexicon_tags_0 },
    { 0x94d4fc88, SUC_NE_lexicon_tags_5 },
    { 0x8efefc87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9006fc8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x244efc8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfab0fc8f, SUC_NE_lexicon_tags_2 },
    { 0xfe42fc90, SUC_NE_lexicon_tags_2 },
    { 0xd5e2fc8f, SUC_NE_lexicon_tags_2 },
    { 0x8e9afc92, SUC_NE_lexicon_tags_2 },
    { 0xd2f8fc92, SUC_NE_lexicon_tags_22 },
    { 0xec6efc8f, SUC_NE_lexicon_tags_2 },
    { 0x6122fc95, SUC_NE_lexicon_tags_4 },
    { 0x9b52fc94, SUC_NE_lexicon_tags_2 },
    { 0x2206fc96, SUC_NE_lexicon_tags_2 },
    { 0x5182fc95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc346fc9a, SUC_NE_lexicon_tags_2 },
    { 0xad74fc9b, SUC_NE_lexicon_tags_36 },
    { 0xaca2fc9b, SUC_NE_lexicon_tags_0 },
    { 0x3e8efc9d, SUC_NE_lexicon_tags_2 },
    { 0x22dafc9d, SUC_NE_lexicon_tags_2 },
    { 0x4a7afc9e, SUC_NE_lexicon_tags_2 },
    { 0x6488fc9e, SUC_NE_lexicon_tags_2 },
    { 0xb01cfca1, SUC_NE_lexicon_tags_2 },
    { 0xe950fca2, SUC_NE_lexicon_tags_2 },
    { 0x316afca2, SUC_NE_lexicon_tags_2 },
    { 0x20eefca2, SUC_NE_lexicon_tags_11 },
    { 0x93befca5, SUC_NE_lexicon_tags_34 },
    { 0x26cfca5, SUC_NE_lexicon_tags_2 },
    { 0x236cfca3, SUC_NE_lexicon_tags_2 },
    { 0xbcb6fca1, SUC_NE_lexicon_tags_3 },
    { 0x7374fca2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe68fcac, SUC_NE_lexicon_tags_2 },
    { 0xbef4fcad, SUC_NE_lexicon_tags_2 },
    { 0x4a5cfcac, SUC_NE_lexicon_tags_2 },
    { 0xdfd6fcae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcfcb4, SUC_NE_lexicon_tags_2 },
    { 0x2f50fcb5, SUC_NE_lexicon_tags_2 },
    { 0x62fefcb5, SUC_NE_lexicon_tags_7 },
    { 0x42eefcb5, SUC_NE_lexicon_tags_2 },
    { 0xee4fcb6, SUC_NE_lexicon_tags_2 },
    { 0xccd4fcb9, SUC_NE_lexicon_tags_1 },
    { 0x5ff4fcb5, SUC_NE_lexicon_tags_2 },
    { 0x59fcfcbb, SUC_NE_lexicon_tags_25 },
    { 0x39f8fcbc, SUC_NE_lexicon_tags_2 },
    { 0x2eecfcbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5748fcbf, SUC_NE_lexicon_tags_2 },
    { 0x88efcc0, SUC_NE_lexicon_tags_2 },
    { 0x8fbcfcc1, SUC_NE_lexicon_tags_5 },
    { 0xd108fcc2, SUC_NE_lexicon_tags_2 },
    { 0x1eb2fcc2, SUC_NE_lexicon_tags_2 },
    { 0x5804fcc2, SUC_NE_lexicon_tags_2 },
    { 0xc2befcc5, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0x1b8cfcc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74f6fcc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf242fccc, SUC_NE_lexicon_tags_19 },
    { 0x40e8fccd, SUC_NE_lexicon_tags_2 },
    { 0xbc18fcce, SUC_NE_lexicon_tags_2 },
    { 0x9298fccf, SUC_NE_lexicon_tags_2 },
    { 0x973cfccf, SUC_NE_lexicon_tags_2 },
    { 0x116efcd1, SUC_NE_lexicon_tags_2 },
    { 0xf9cefcd2, SUC_NE_lexicon_tags_2 },
    { 0x2586fcd2, SUC_NE_lexicon_tags_2 },
    { 0x9846fcd4, SUC_NE_lexicon_tags_7 },
    { 0x3a4afcd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4fcd9, SUC_NE_lexicon_tags_2 },
    { 0xbc36fcda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e2fcde, SUC_NE_lexicon_tags_1 },
    { 0xcafcfcdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1566fce1, SUC_NE_lexicon_tags_0 },
    { 0x71eefce2, SUC_NE_lexicon_tags_2 },
    { 0x50d0fce3, SUC_NE_lexicon_tags_2 },
    { 0xeb0fce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x594cfce7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7a2fce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x108efcec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69aefcf3, SUC_NE_lexicon_tags_2 },
    { 0x3a74fcf4, SUC_NE_lexicon_tags_2 },
    { 0x2a98fcf5, SUC_NE_lexicon_tags_2 },
    { 0x1a76fcf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c8fcf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a20fcfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae2fd01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e08fd04, SUC_NE_lexicon_tags_2 },
    { 0xb62efd05, SUC_NE_lexicon_tags_2 },
    { 0x84c4fd05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74fd09, SUC_NE_lexicon_tags_2 },
    { 0x1818fd0a, SUC_NE_lexicon_tags_2 },
    { 0x2ecefd0b, SUC_NE_lexicon_tags_2 },
    { 0x486cfd0b, SUC_NE_lexicon_tags_2 },
    { 0x4b38fd0d, SUC_NE_lexicon_tags_2 },
    { 0xe7b0fd0c, SUC_NE_lexicon_tags_2 },
    { 0x30efd0f, SUC_NE_lexicon_tags_1 },
    { 0xb538fd0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e18fd13, SUC_NE_lexicon_tags_24 },
    { 0xa812fd14, SUC_NE_lexicon_tags_2 },
    { 0xe6d4fd15, SUC_NE_lexicon_tags_2 },
    { 0x9a06fd15, SUC_NE_lexicon_tags_2 },
    { 0x25a0fd15, SUC_NE_lexicon_tags_3 },
    { 0xa1f2fd18, SUC_NE_lexicon_tags_2 },
    { 0xf1dcfd18, SUC_NE_lexicon_tags_2 },
    { 0x18eafd1a, SUC_NE_lexicon_tags_2 },
    { 0x2e90fd19, SUC_NE_lexicon_tags_2 },
    { 0x48c8fd19, SUC_NE_lexicon_tags_2 },
    { 0x802cfd14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8efd2a, SUC_NE_lexicon_tags_2 },
    { 0x9718fd2b, SUC_NE_lexicon_tags_2 },
    { 0xa406fd2a, SUC_NE_lexicon_tags_2 },
    { 0xc2e8fd2d, SUC_NE_lexicon_tags_2 },
    { 0x162efd2d, SUC_NE_lexicon_tags_2 },
    { 0xed02fd2d, SUC_NE_lexicon_tags_2 },
    { 0x711efd30, SUC_NE_lexicon_tags_2 },
    { 0x29eefd30, SUC_NE_lexicon_tags_2 },
    { 0xd902fd32, SUC_NE_lexicon_tags_2 },
    { 0xd194fd2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7eafd38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86fd3d, SUC_NE_lexicon_tags_0 },
    { 0xdc48fd3d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xebd8fd40, SUC_NE_lexicon_tags_2 },
    { 0xb7d4fd41, SUC_NE_lexicon_tags_7 },
    { 0x8670fd42, SUC_NE_lexicon_tags_2 },
    { 0x1f0fd42, SUC_NE_lexicon_tags_2 },
    { 0xe696fd41, SUC_NE_lexicon_tags_2 },
    { 0x8666fd42, SUC_NE_lexicon_tags_2 },
    { 0x3e62fd46, SUC_NE_lexicon_tags_2 },
    { 0xfd7afd41, SUC_NE_lexicon_tags_2 },
    { 0x8346fd48, SUC_NE_lexicon_tags_2 },
    { 0xc346fd48, SUC_NE_lexicon_tags_2 },
    { 0x285efd48, SUC_NE_lexicon_tags_2 },
    { 0xd922fd4b, SUC_NE_lexicon_tags_2 },
    { 0x22aefd42, SUC_NE_lexicon_tags_2 },
    { 0x7976fd42, SUC_NE_lexicon_tags_2 },
    { 0xd852fd4e, SUC_NE_lexicon_tags_2 },
    { 0x3176fd47, SUC_NE_lexicon_tags_2 },
    { 0x372cfd50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55bcfd52, SUC_NE_lexicon_tags_2 },
    { 0x845cfd53, SUC_NE_lexicon_tags_2 },
    { 0x10aefd53, SUC_NE_lexicon_tags_1 },
    { 0x4856fd55, SUC_NE_lexicon_tags_2 },
    { 0x77d4fd56, SUC_NE_lexicon_tags_2 },
    { 0x596cfd57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1402fd59, SUC_NE_lexicon_tags_2 },
    { 0x2f8afd59, SUC_NE_lexicon_tags_2 },
    { 0x4e4fd5a, SUC_NE_lexicon_tags_2 },
    { 0x5062fd59, SUC_NE_lexicon_tags_2 },
    { 0xa3b0fd5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaecfd61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d6fd66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xade0fd68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d2efd6a, SUC_NE_lexicon_tags_2 },
    { 0xd3e2fd6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7efd6e, SUC_NE_lexicon_tags_7 },
    { 0x874efd6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3548fd73, SUC_NE_lexicon_tags_2 },
    { 0x10d4fd74, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb44fd77, SUC_NE_lexicon_tags_2 },
    { 0xdc52fd78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36c2fd7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54e8fd7c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3defd7e, SUC_NE_lexicon_tags_2 },
    { 0x379cfd7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f28fd82, SUC_NE_lexicon_tags_5 },
    { 0x8b18fd82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef4fd87, SUC_NE_lexicon_tags_2 },
    { 0x21defd88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9568fd8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f10fd91, SUC_NE_lexicon_tags_2 },
    { 0x6d6cfd92, SUC_NE_lexicon_tags_2 },
    { 0x4acfd93, SUC_NE_lexicon_tags_2 },
    { 0x19befd94, SUC_NE_lexicon_tags_2 },
    { 0x2860fd94, SUC_NE_lexicon_tags_1 },
    { 0xf10fd92, SUC_NE_lexicon_tags_2 },
    { 0xe976fd97, SUC_NE_lexicon_tags_18 },
    { 0xb64fd94, SUC_NE_lexicon_tags_2 },
    { 0xfd12fd92, SUC_NE_lexicon_tags_2 },
    { 0x192cfd9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b44fda5, SUC_NE_lexicon_tags_2 },
    { 0x6d8fda6, SUC_NE_lexicon_tags_2 },
    { 0xbcfefda7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5afdaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f0fdac, SUC_NE_lexicon_tags_2 },
    { 0x35aefdad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3788fdb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16defdb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc6fdb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe110fdba, SUC_NE_lexicon_tags_18 },
    { 0x7b2cfdba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6978fdbd, SUC_NE_lexicon_tags_2 },
    { 0x3342fdbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6fdc1, SUC_NE_lexicon_tags_2 },
    { 0xc75cfdc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfcfdca, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xcbc8fdcc, SUC_NE_lexicon_tags_2 },
    { 0x1892fdcd, SUC_NE_lexicon_tags_2 },
    { 0x772fdce, SUC_NE_lexicon_tags_2 },
    { 0xe988fdcf, SUC_NE_lexicon_tags_2 },
    { 0x21fefdce, SUC_NE_lexicon_tags_4 },
    { 0x9ad4fdd1, SUC_NE_lexicon_tags_2 },
    { 0x9c94fdd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93f4fdd4, SUC_NE_lexicon_tags_2 },
    { 0x1d2fdd5, SUC_NE_lexicon_tags_2 },
    { 0x3e54fdd5, SUC_NE_lexicon_tags_2 },
    { 0xa8e0fdd7, SUC_NE_lexicon_tags_1 },
    { 0x8c9efdd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e00fddb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4240fddd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x434fddf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7f2fde1, SUC_NE_lexicon_tags_2 },
    { 0x3088fde1, SUC_NE_lexicon_tags_5 },
    { 0xec20fde2, SUC_NE_lexicon_tags_2 },
    { 0x14f2fde4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc724fde6, SUC_NE_lexicon_tags_2 },
    { 0x43a0fde7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fefdea, SUC_NE_lexicon_tags_2 },
    { 0xe12cfdeb, SUC_NE_lexicon_tags_2 },
    { 0x73e4fdec, SUC_NE_lexicon_tags_2 },
    { 0xef92fdec, SUC_NE_lexicon_tags_2 },
    { 0xac6fdee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2fdf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e4afdf8, SUC_NE_lexicon_tags_2 },
    { 0xab32fdf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc59efdfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984efe03, SUC_NE_lexicon_tags_12 },
    { 0xf380fe03, SUC_NE_lexicon_tags_2 },
    { 0x25a0fe05, SUC_NE_lexicon_tags_3 },
    { 0x264afe06, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9014fe08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafe8fe0a, SUC_NE_lexicon_tags_2 },
    { 0x855efe0b, SUC_NE_lexicon_tags_2 },
    { 0x8eb6fe0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7672fe0f, SUC_NE_lexicon_tags_2 },
    { 0x28a8fe0f, SUC_NE_lexicon_tags_18 },
    { 0x37d0fe11, SUC_NE_lexicon_tags_2 },
    { 0x7b68fe11, SUC_NE_lexicon_tags_2 },
    { 0x3c3afe13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2afe17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433cfe1b, SUC_NE_lexicon_tags_2 },
    { 0xa0b8fe1c, SUC_NE_lexicon_tags_7 },
    { 0x5512fe1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x319afe20, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc0a8fe22, SUC_NE_lexicon_tags_2 },
    { 0x702fe22, SUC_NE_lexicon_tags_2 },
    { 0x2d28fe24, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a54fe27, SUC_NE_lexicon_tags_2 },
    { 0x3722fe27, SUC_NE_lexicon_tags_2 },
    { 0xfdb6fe28, SUC_NE_lexicon_tags_2 },
    { 0x5e3efe27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c2fe35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f4fe39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b2fe3e, SUC_NE_lexicon_tags_2 },
    { 0xea60fe3f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5608fe44, SUC_NE_lexicon_tags_2 },
    { 0x2c1cfe45, SUC_NE_lexicon_tags_2 },
    { 0x46f6fe46, SUC_NE_lexicon_tags_4 },
    { 0x3fdafe46, SUC_NE_lexicon_tags_2 },
    { 0x24a2fe48, SUC_NE_lexicon_tags_2 },
    { 0x84dcfe49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe2fe4c, SUC_NE_lexicon_tags_7 },
    { 0x435afe4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ecfe50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3564fe52, SUC_NE_lexicon_tags_2 },
    { 0xe932fe53, SUC_NE_lexicon_tags_2 },
    { 0x25efe54, SUC_NE_lexicon_tags_17 },
    { 0x3938fe55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eecfe57, SUC_NE_lexicon_tags_2 },
    { 0x9ca4fe58, SUC_NE_lexicon_tags_2 },
    { 0x6900fe59, SUC_NE_lexicon_tags_7 },
    { 0x621efe59, SUC_NE_lexicon_tags_19 },
    { 0xd83cfe5a, SUC_NE_lexicon_tags_2 },
    { 0x6d32fe59, SUC_NE_lexicon_tags_2 },
    { 0x64d6fe5c, SUC_NE_lexicon_tags_2 },
    { 0x1796fe5e, SUC_NE_lexicon_tags_2 },
    { 0xcb98fe5e, SUC_NE_lexicon_tags_2 },
    { 0xbd5afe5e, SUC_NE_lexicon_tags_2 },
    { 0xc930fe5e, SUC_NE_lexicon_tags_2 },
    { 0xdbc8fe62, SUC_NE_lexicon_tags_2 },
    { 0x6372fe63, SUC_NE_lexicon_tags_2 },
    { 0xed34fe5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa9afe66, SUC_NE_lexicon_tags_2 },
    { 0x97dafe67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaefe6b, SUC_NE_lexicon_tags_2 },
    { 0x202afe6b, SUC_NE_lexicon_tags_2 },
    { 0xd7eefe6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc86efe6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb3efe71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13a8fe73, SUC_NE_lexicon_tags_2 },
    { 0xac72fe74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84f8fe76, SUC_NE_lexicon_tags_2 },
    { 0x6c1cfe77, SUC_NE_lexicon_tags_2 },
    { 0x1bcfe77, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f20fe7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e6fe81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12cfe84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecafe8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd452fe8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3880fe90, SUC_NE_lexicon_tags_2 },
    { 0x87f2fe90, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xcdeafe93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ccfe97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8cfe9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5456fea6, SUC_NE_lexicon_tags_3 },
    { 0xe874fea7, SUC_NE_lexicon_tags_2 },
    { 0x3328fea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfcfeb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd86feb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabefeb7, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b8feba, SUC_NE_lexicon_tags_2 },
    { 0x37e4feba, SUC_NE_lexicon_tags_2 },
    { 0x6bccfeba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x250febf, SUC_NE_lexicon_tags_32 },
    { 0xb86afec0, SUC_NE_lexicon_tags_2 },
    { 0xe992febf, SUC_NE_lexicon_tags_3 },
    { 0xbf16fec2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbca8fec4, SUC_NE_lexicon_tags_2 },
    { 0xcc72fec5, SUC_NE_lexicon_tags_2 },
    { 0xd8fafec6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65afeca, SUC_NE_lexicon_tags_2 },
    { 0xc42fecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d80fecd, SUC_NE_lexicon_tags_2 },
    { 0xba14fecd, SUC_NE_lexicon_tags_2 },
    { 0xf96cfecf, SUC_NE_lexicon_tags_7 },
    { 0xcb3afed0, SUC_NE_lexicon_tags_2 },
    { 0x977afed1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e06fed5, SUC_NE_lexicon_tags_2 },
    { 0x31fefed5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f98fed8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16eafedb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8afedf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb42fee4, SUC_NE_lexicon_tags_2 },
    { 0xe694fee5, SUC_NE_lexicon_tags_2 },
    { 0x97f2fee6, SUC_NE_lexicon_tags_5 },
    { 0xa502fee6, SUC_NE_lexicon_tags_2 },
    { 0x9140fee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa560feea, SUC_NE_lexicon_tags_2 },
    { 0x2e98feeb, SUC_NE_lexicon_tags_2 },
    { 0xd404feea, SUC_NE_lexicon_tags_2 },
    { 0xdc60feed, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b8fef1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd95efef3, SUC_NE_lexicon_tags_2 },
    { 0xcf16fef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x764efef6, SUC_NE_lexicon_tags_2 },
    { 0x9d0cfef6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5d0fef9, SUC_NE_lexicon_tags_2 },
    { 0x7760fefa, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee4fefd, SUC_NE_lexicon_tags_2 },
    { 0xe400fefd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20cff01, SUC_NE_lexicon_tags_2 },
    { 0x2deaff02, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba6ff06, SUC_NE_lexicon_tags_2 },
    { 0x28ff07, SUC_NE_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x4898ff0a, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea6ff10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4cff13, SUC_NE_lexicon_tags_2 },
    { 0x6932ff13, SUC_NE_lexicon_tags_7 },
    { 0x8a70ff15, SUC_NE_lexicon_tags_2 },
    { 0x54b2ff14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83aff1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690aff1f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa562ff22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3da8ff24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e68ff2b, SUC_NE_lexicon_tags_2 },
    { 0xe17eff2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x237eff2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d14ff30, SUC_NE_lexicon_tags_7 },
    { 0x58eeff30, SUC_NE_lexicon_tags_2 },
    { 0xec26ff31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x760ff35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff24ff3c, SUC_NE_lexicon_tags_2 },
    { 0x83aaff3d, SUC_NE_lexicon_tags_3 },
    { 0x44ecff3d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf098ff43, SUC_NE_lexicon_tags_2 },
    { 0x6224ff43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf860ff46, SUC_NE_lexicon_tags_2 },
    { 0xa5ecff46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a2ff4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a46ff4e, SUC_NE_lexicon_tags_2 },
    { 0x8df0ff4f, SUC_NE_lexicon_tags_2 },
    { 0xa592ff4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f60ff53, SUC_NE_lexicon_tags_2 },
    { 0xfde6ff54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9b4ff56, SUC_NE_lexicon_tags_2 },
    { 0x20ecff56, SUC_NE_lexicon_tags_2 },
    { 0x396ff58, SUC_NE_lexicon_tags_3 },
    { 0xa37aff58, SUC_NE_lexicon_tags_2 },
    { 0xe25eff5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa83cff5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c6ff5f, SUC_NE_lexicon_tags_2 },
    { 0x3a7aff60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9014ff63, SUC_NE_lexicon_tags_2 },
    { 0xc7b0ff64, SUC_NE_lexicon_tags_2 },
    { 0x6d58ff65, SUC_NE_lexicon_tags_2 },
    { 0xba2aff65, SUC_NE_lexicon_tags_2 },
    { 0xc216ff64, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2b6ff69, SUC_NE_lexicon_tags_2 },
    { 0x73c2ff6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e50ff6f, SUC_NE_lexicon_tags_2 },
    { 0xbbeeff70, SUC_NE_lexicon_tags_2 },
    { 0xb5ccff71, SUC_NE_lexicon_tags_2 },
    { 0xa1a8ff70, SUC_NE_lexicon_tags_2 },
    { 0x12caff73, SUC_NE_lexicon_tags_2 },
    { 0x4bbeff73, SUC_NE_lexicon_tags_2 },
    { 0xddf8ff73, SUC_NE_lexicon_tags_4 },
    { 0x8196ff74, SUC_NE_lexicon_tags_2 },
    { 0xe8aaff70, SUC_NE_lexicon_tags_5 },
    { 0xf2d6ff76, SUC_NE_lexicon_tags_2 },
    { 0xcb8eff79, SUC_NE_lexicon_tags_19 },
    { 0xba9eff7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x768eff7d, SUC_NE_lexicon_tags_2 },
    { 0xc286ff7e, SUC_NE_lexicon_tags_2 },
    { 0x3c9aff7f, SUC_NE_lexicon_tags_2 },
    { 0x112ff7f, SUC_NE_lexicon_tags_19 },
    { 0xc852ff7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ac0ff83, SUC_NE_lexicon_tags_2 },
    { 0xf74ff84, SUC_NE_lexicon_tags_2 },
    { 0x22ecff85, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xda62ff87, SUC_NE_lexicon_tags_2 },
    { 0x2662ff88, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf110ff8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78cff8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe58aff90, SUC_NE_lexicon_tags_2 },
    { 0x4c9eff91, SUC_NE_lexicon_tags_2 },
    { 0x3dd0ff92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x664eff95, SUC_NE_lexicon_tags_2 },
    { 0xeb0cff96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa66ff9a, SUC_NE_lexicon_tags_2 },
    { 0x8064ff9a, SUC_NE_lexicon_tags_2 },
    { 0x44b2ff9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70effa4, SUC_NE_lexicon_tags_2 },
    { 0xe9eaffa4, SUC_NE_lexicon_tags_2 },
    { 0x90c4ffa6, SUC_NE_lexicon_tags_2 },
    { 0xc60ffa5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x722effaa, SUC_NE_lexicon_tags_4 },
    { 0x3afeffab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9cffb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337affb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d82ffb9, SUC_NE_lexicon_tags_17 },
    { 0x997cffba, SUC_NE_lexicon_tags_2 },
    { 0xb58affb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd988ffbe, SUC_NE_lexicon_tags_2 },
    { 0x935cffbe, SUC_NE_lexicon_tags_2 },
    { 0xe06cffc0, SUC_NE_lexicon_tags_2 },
    { 0x2ddaffc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3250ffc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4950ffc6, SUC_NE_lexicon_tags_2 },
    { 0x7af0ffc7, SUC_NE_lexicon_tags_2 },
    { 0xdfb2ffc8, SUC_NE_lexicon_tags_2 },
    { 0xceb2ffc6, SUC_NE_lexicon_tags_4 },
    { 0xd0eaffca, SUC_NE_lexicon_tags_107 },
    { 0x27feffcb, SUC_NE_lexicon_tags_2 },
    { 0x2844ffcc, SUC_NE_lexicon_tags_2 },
    { 0x7ab4ffcd, SUC_NE_lexicon_tags_2 },
    { 0x5878ffcb, SUC_NE_lexicon_tags_2 },
    { 0xd14ffcc, SUC_NE_lexicon_tags_2 },
    { 0x4276ffca, SUC_NE_lexicon_tags_2 },
    { 0xb8faffce, SUC_NE_lexicon_tags_2 },
    { 0xb6bcffd2, SUC_NE_lexicon_tags_2 },
    { 0xb4c8ffd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e98ffd9, SUC_NE_lexicon_tags_7 },
    { 0x78b0ffda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5538ffdc, SUC_NE_lexicon_tags_30 },
    { 0x918affdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8eaffe0, SUC_NE_lexicon_tags_5 },
    { 0xbcf4ffe0, SUC_NE_lexicon_tags_2 },
    { 0x2f78ffe0, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3effe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd4ffe7, SUC_NE_lexicon_tags_7 },
    { 0xd22cffe8, SUC_NE_lexicon_tags_7 },
    { 0x2174ffe9, SUC_NE_lexicon_tags_2 },
    { 0x62b0ffea, SUC_NE_lexicon_tags_144 },
    { 0x79bcffea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5e8ffed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c6fff0, SUC_NE_lexicon_tags_34 },
    { 0x5b3efff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b8fff4, SUC_NE_lexicon_tags_5 },
    { 0xb09cfff4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9b8fff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db0000, SUC_NE_lexicon_tags_2 },
    { 0xd96b0000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb930006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d90008, SUC_NE_lexicon_tags_2 },
    { 0xe0370008, SUC_NE_lexicon_tags_2 },
    { 0xd3570008, SUC_NE_lexicon_tags_2 },
    { 0x9f27000b, SUC_NE_lexicon_tags_1 },
    { 0xea6b000b, SUC_NE_lexicon_tags_2 },
    { 0x9a35000c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4bf000f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87130011, SUC_NE_lexicon_tags_2 },
    { 0xe4bd0012, SUC_NE_lexicon_tags_2 },
    { 0x463d0013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x394d0016, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca7001c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x410d001e, SUC_NE_lexicon_tags_1 },
    { 0x410f001f, SUC_NE_lexicon_tags_2 },
    { 0x5d50020, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7d0027, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39c90029, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42db002e, SUC_NE_lexicon_tags_2 },
    { 0x56d9002f, SUC_NE_lexicon_tags_2 },
    { 0xac630030, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b490034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f50036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42f0039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c5003d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x864d003f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x490041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78730045, SUC_NE_lexicon_tags_2 },
    { 0x1bd0045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d77004b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc245004d, SUC_NE_lexicon_tags_2 },
    { 0xa38f004d, SUC_NE_lexicon_tags_2 },
    { 0x6a1f004f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c70053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d390055, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28310057, SUC_NE_lexicon_tags_2 },
    { 0x90450057, SUC_NE_lexicon_tags_2 },
    { 0x5a2d0059, SUC_NE_lexicon_tags_2 },
    { 0x7a15005a, SUC_NE_lexicon_tags_2 },
    { 0x7b6f0059, SUC_NE_lexicon_tags_2 },
    { 0x98e7005c, SUC_NE_lexicon_tags_2 },
    { 0xb6d1005c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b90063, SUC_NE_lexicon_tags_2 },
    { 0x28a50063, SUC_NE_lexicon_tags_2 },
    { 0x59330064, SUC_NE_lexicon_tags_2 },
    { 0xcfc10064, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6550068, SUC_NE_lexicon_tags_2 },
    { 0xc83f0068, SUC_NE_lexicon_tags_2 },
    { 0xe399006a, SUC_NE_lexicon_tags_2 },
    { 0xe33d0068, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x173b006d, SUC_NE_lexicon_tags_2 },
    { 0x2785006e, SUC_NE_lexicon_tags_2 },
    { 0xc9bf006f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fab0071, SUC_NE_lexicon_tags_2 },
    { 0x7a2b0071, SUC_NE_lexicon_tags_4 },
    { 0x31d90073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c50076, SUC_NE_lexicon_tags_2 },
    { 0x317d0077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x790079, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce15007b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd51007d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8289007f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcac70081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7530086, SUC_NE_lexicon_tags_2 },
    { 0x2bb30086, SUC_NE_lexicon_tags_2 },
    { 0xb1390088, SUC_NE_lexicon_tags_2 },
    { 0xdaaf0089, SUC_NE_lexicon_tags_3 },
    { 0x9f250086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d5008d, SUC_NE_lexicon_tags_3 },
    { 0x8d7d008d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d070091, SUC_NE_lexicon_tags_7 },
    { 0x53d90092, SUC_NE_lexicon_tags_382 },
    { 0, NULL },
    { 0, NULL },
    { 0xddad0095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe35b0097, SUC_NE_lexicon_tags_2 },
    { 0xfb170097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a5009b, SUC_NE_lexicon_tags_2 },
    { 0xd8c5009c, SUC_NE_lexicon_tags_2 },
    { 0xb2a9009c, SUC_NE_lexicon_tags_2 },
    { 0xfcad009e, SUC_NE_lexicon_tags_2 },
    { 0xf28b009d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4700a2, SUC_NE_lexicon_tags_2 },
    { 0x414500a3, SUC_NE_lexicon_tags_2 },
    { 0x473300a4, SUC_NE_lexicon_tags_2 },
    { 0x547300a4, SUC_NE_lexicon_tags_0 },
    { 0x6ec700a5, SUC_NE_lexicon_tags_2 },
    { 0xe19b00a7, SUC_NE_lexicon_tags_381 },
    { 0xf2bb00a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d00ac, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23100b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ba900b3, SUC_NE_lexicon_tags_2 },
    { 0x3e5f00b4, SUC_NE_lexicon_tags_2 },
    { 0xd85b00b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79d00ba, SUC_NE_lexicon_tags_2 },
    { 0xc52900ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ed00c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dd00c3, SUC_NE_lexicon_tags_2 },
    { 0x4c3f00c3, SUC_NE_lexicon_tags_2 },
    { 0x153b00c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x813b00c7, SUC_NE_lexicon_tags_2 },
    { 0xde9500c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1c700ca, SUC_NE_lexicon_tags_2 },
    { 0x188300cb, SUC_NE_lexicon_tags_2 },
    { 0x583b00cb, SUC_NE_lexicon_tags_2 },
    { 0x35f300cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f2900cf, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xd1a500d1, SUC_NE_lexicon_tags_2 },
    { 0xd18d00d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd67300d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807d00dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b00e5, SUC_NE_lexicon_tags_2 },
    { 0xda7d00e5, SUC_NE_lexicon_tags_2 },
    { 0x9ed700e7, SUC_NE_lexicon_tags_17 },
    { 0x802100e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf18100ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d00ed, SUC_NE_lexicon_tags_2 },
    { 0xa90500ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbf00f2, SUC_NE_lexicon_tags_2 },
    { 0xfa2900f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7100fa, SUC_NE_lexicon_tags_2 },
    { 0x6b6500fb, SUC_NE_lexicon_tags_2 },
    { 0x5b8700fb, SUC_NE_lexicon_tags_2 },
    { 0xaf1500fd, SUC_NE_lexicon_tags_1 },
    { 0x22700fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7099010a, SUC_NE_lexicon_tags_2 },
    { 0x983f010a, SUC_NE_lexicon_tags_2 },
    { 0x5183010c, SUC_NE_lexicon_tags_4 },
    { 0x3ba5010b, SUC_NE_lexicon_tags_2 },
    { 0x1345010c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b10113, SUC_NE_lexicon_tags_2 },
    { 0xb57d0114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x341d0117, SUC_NE_lexicon_tags_2 },
    { 0xfa130117, SUC_NE_lexicon_tags_2 },
    { 0x3bc10119, SUC_NE_lexicon_tags_2 },
    { 0x10f70119, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa589011d, SUC_NE_lexicon_tags_2 },
    { 0xd617011e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97dd0121, SUC_NE_lexicon_tags_17 },
    { 0x8be50122, SUC_NE_lexicon_tags_2 },
    { 0xa1cf0123, SUC_NE_lexicon_tags_2 },
    { 0x8d6f0123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10fb0126, SUC_NE_lexicon_tags_18 },
    { 0x4f210127, SUC_NE_lexicon_tags_2 },
    { 0xe9990128, SUC_NE_lexicon_tags_2 },
    { 0x23730129, SUC_NE_lexicon_tags_2 },
    { 0xcbb9012a, SUC_NE_lexicon_tags_2 },
    { 0x9de5012b, SUC_NE_lexicon_tags_2 },
    { 0x11012a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6970130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf07b0132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6850134, SUC_NE_lexicon_tags_2 },
    { 0x983f0135, SUC_NE_lexicon_tags_2 },
    { 0x8f410136, SUC_NE_lexicon_tags_2 },
    { 0xffd10137, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43e70139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb267013c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf0143, SUC_NE_lexicon_tags_2 },
    { 0x694d0144, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47470147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59c30149, SUC_NE_lexicon_tags_2 },
    { 0xd3650149, SUC_NE_lexicon_tags_2 },
    { 0xe88d014a, SUC_NE_lexicon_tags_2 },
    { 0x703f014a, SUC_NE_lexicon_tags_2 },
    { 0x11cd014d, SUC_NE_lexicon_tags_2 },
    { 0x8c11014e, SUC_NE_lexicon_tags_4 },
    { 0x7fb1014f, SUC_NE_lexicon_tags_2 },
    { 0xed91014a, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x9b230152, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e030157, SUC_NE_lexicon_tags_2 },
    { 0x74e50158, SUC_NE_lexicon_tags_2 },
    { 0xee410159, SUC_NE_lexicon_tags_2 },
    { 0x7eb1015a, SUC_NE_lexicon_tags_40 },
    { 0x492f015b, SUC_NE_lexicon_tags_2 },
    { 0x8841015b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fb0161, SUC_NE_lexicon_tags_2 },
    { 0x8630162, SUC_NE_lexicon_tags_2 },
    { 0xeced0163, SUC_NE_lexicon_tags_11 },
    { 0x8a290164, SUC_NE_lexicon_tags_2 },
    { 0x41c50163, SUC_NE_lexicon_tags_2 },
    { 0xe8530163, SUC_NE_lexicon_tags_2 },
    { 0x97c70162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42c70169, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabb5016b, SUC_NE_lexicon_tags_2 },
    { 0xe723016b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x5f016e, SUC_NE_lexicon_tags_2 },
    { 0x6f6f016e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76590173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f810175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd0178, SUC_NE_lexicon_tags_2 },
    { 0x13ef0179, SUC_NE_lexicon_tags_2 },
    { 0x60bf0179, SUC_NE_lexicon_tags_2 },
    { 0x3243017b, SUC_NE_lexicon_tags_2 },
    { 0x690b017c, SUC_NE_lexicon_tags_2 },
    { 0xf28b0178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30180, SUC_NE_lexicon_tags_2 },
    { 0xf6bd0181, SUC_NE_lexicon_tags_2 },
    { 0x554d0182, SUC_NE_lexicon_tags_7 },
    { 0xd2290182, SUC_NE_lexicon_tags_4 },
    { 0x9a6f0181, SUC_NE_lexicon_tags_2 },
    { 0x92050184, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4870187, SUC_NE_lexicon_tags_2 },
    { 0xb63b0187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf539018b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed0190, SUC_NE_lexicon_tags_3 },
    { 0xf9c50191, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93610194, SUC_NE_lexicon_tags_2 },
    { 0xf2ef0195, SUC_NE_lexicon_tags_2 },
    { 0x1aa70194, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd3019a, SUC_NE_lexicon_tags_2 },
    { 0x2563019b, SUC_NE_lexicon_tags_2 },
    { 0xef3b019c, SUC_NE_lexicon_tags_2 },
    { 0x2b03019d, SUC_NE_lexicon_tags_2 },
    { 0xb2f5019e, SUC_NE_lexicon_tags_2 },
    { 0x5ddf019f, SUC_NE_lexicon_tags_7 },
    { 0x157901a0, SUC_NE_lexicon_tags_2 },
    { 0x796101a1, SUC_NE_lexicon_tags_2 },
    { 0x76cd019f, SUC_NE_lexicon_tags_2 },
    { 0x3df019d, SUC_NE_lexicon_tags_2 },
    { 0x143301a4, SUC_NE_lexicon_tags_2 },
    { 0x531f01a5, SUC_NE_lexicon_tags_2 },
    { 0xda4901a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfcf01a8, SUC_NE_lexicon_tags_2 },
    { 0x82301a8, SUC_NE_lexicon_tags_2 },
    { 0xd48301a8, SUC_NE_lexicon_tags_2 },
    { 0x7e0701ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867701af, SUC_NE_lexicon_tags_2 },
    { 0xd68d01af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9f01b7, SUC_NE_lexicon_tags_2 },
    { 0xcfc501b8, SUC_NE_lexicon_tags_2 },
    { 0x777b01b9, SUC_NE_lexicon_tags_2 },
    { 0x50d101ba, SUC_NE_lexicon_tags_2 },
    { 0xe5af01ba, SUC_NE_lexicon_tags_2 },
    { 0xfbfb01bb, SUC_NE_lexicon_tags_2 },
    { 0x8d1b01bd, SUC_NE_lexicon_tags_2 },
    { 0x9db01ba, SUC_NE_lexicon_tags_2 },
    { 0x70e101bf, SUC_NE_lexicon_tags_2 },
    { 0x294701be, SUC_NE_lexicon_tags_2 },
    { 0xfc901c1, SUC_NE_lexicon_tags_2 },
    { 0x49df01c2, SUC_NE_lexicon_tags_2 },
    { 0x1a6501c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x815f01c6, SUC_NE_lexicon_tags_2 },
    { 0xbe6b01c7, SUC_NE_lexicon_tags_2 },
    { 0x898f01c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac9f01ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58701d2, SUC_NE_lexicon_tags_2 },
    { 0x93d301d2, SUC_NE_lexicon_tags_34 },
    { 0xe9d901d2, SUC_NE_lexicon_tags_2 },
    { 0xe96501d5, SUC_NE_lexicon_tags_2 },
    { 0x5fa501d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e101d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc6f01db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70b501dd, SUC_NE_lexicon_tags_2 },
    { 0x9e9d01dd, SUC_NE_lexicon_tags_2 },
    { 0xd3db01df, SUC_NE_lexicon_tags_2 },
    { 0xcc3101e0, SUC_NE_lexicon_tags_2 },
    { 0x2d1b01e1, SUC_NE_lexicon_tags_2 },
    { 0xf30901df, SUC_NE_lexicon_tags_4 },
    { 0xa3d01e3, SUC_NE_lexicon_tags_2 },
    { 0xac4101df, SUC_NE_lexicon_tags_2 },
    { 0x658b01e5, SUC_NE_lexicon_tags_2 },
    { 0x7e9901e6, SUC_NE_lexicon_tags_2 },
    { 0xd75901e6, SUC_NE_lexicon_tags_2 },
    { 0x52b101e8, SUC_NE_lexicon_tags_2 },
    { 0x5fb101e9, SUC_NE_lexicon_tags_2 },
    { 0xcfbb01ea, SUC_NE_lexicon_tags_2 },
    { 0x345501e6, SUC_NE_lexicon_tags_7 },
    { 0x47cf01ea, SUC_NE_lexicon_tags_2 },
    { 0x48e301ed, SUC_NE_lexicon_tags_2 },
    { 0x3a8901ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4d01f1, SUC_NE_lexicon_tags_2 },
    { 0x619d01f2, SUC_NE_lexicon_tags_1 },
    { 0x10901f1, SUC_NE_lexicon_tags_2 },
    { 0xc78d01f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd78101f6, SUC_NE_lexicon_tags_2 },
    { 0x443501f7, SUC_NE_lexicon_tags_2 },
    { 0x843501f7, SUC_NE_lexicon_tags_2 },
    { 0x1e901f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c301fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d0203, SUC_NE_lexicon_tags_2 },
    { 0xf8650204, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4490207, SUC_NE_lexicon_tags_2 },
    { 0x1c70208, SUC_NE_lexicon_tags_2 },
    { 0xf8d10207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78a1020b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd409020e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f390212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f90215, SUC_NE_lexicon_tags_2 },
    { 0xea90215, SUC_NE_lexicon_tags_2 },
    { 0x56630215, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2ad0219, SUC_NE_lexicon_tags_2 },
    { 0x9a4f0219, SUC_NE_lexicon_tags_454 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff0220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad290226, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce1022c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1001022e, SUC_NE_lexicon_tags_3 },
    { 0xe61d022f, SUC_NE_lexicon_tags_2 },
    { 0x69890230, SUC_NE_lexicon_tags_2 },
    { 0x922b0230, SUC_NE_lexicon_tags_2 },
    { 0xe64d0232, SUC_NE_lexicon_tags_302 },
    { 0xd5290233, SUC_NE_lexicon_tags_2 },
    { 0x66f0231, SUC_NE_lexicon_tags_2 },
    { 0x5a2d0235, SUC_NE_lexicon_tags_7 },
    { 0x664d0235, SUC_NE_lexicon_tags_2 },
    { 0xc6d30235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8cd0239, SUC_NE_lexicon_tags_2 },
    { 0xb705023a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdf3023c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77f023f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60e70241, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2a8f0243, SUC_NE_lexicon_tags_2 },
    { 0xbb450244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ab024a, SUC_NE_lexicon_tags_2 },
    { 0x5ffb024a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3345024f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b0253, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x511f0255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a30258, SUC_NE_lexicon_tags_2 },
    { 0xbc650258, SUC_NE_lexicon_tags_17 },
    { 0x473025a, SUC_NE_lexicon_tags_2 },
    { 0x6481025a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd026a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6467026c, SUC_NE_lexicon_tags_2 },
    { 0xf451026c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae31026f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66430273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd765027c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ad7027e, SUC_NE_lexicon_tags_2 },
    { 0x4be1027f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x436b0282, SUC_NE_lexicon_tags_2 },
    { 0xf0430283, SUC_NE_lexicon_tags_2 },
    { 0xa1ab0284, SUC_NE_lexicon_tags_18 },
    { 0xe4e70285, SUC_NE_lexicon_tags_4 },
    { 0xb3ad0286, SUC_NE_lexicon_tags_2 },
    { 0xaba70287, SUC_NE_lexicon_tags_2 },
    { 0xee530288, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c15028f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd930291, SUC_NE_lexicon_tags_2 },
    { 0x2530292, SUC_NE_lexicon_tags_17 },
    { 0xa510293, SUC_NE_lexicon_tags_2 },
    { 0xfb0d0293, SUC_NE_lexicon_tags_2 },
    { 0x48990292, SUC_NE_lexicon_tags_2 },
    { 0x9910294, SUC_NE_lexicon_tags_4 },
    { 0xa1c10294, SUC_NE_lexicon_tags_2 },
    { 0xeedf0294, SUC_NE_lexicon_tags_2 },
    { 0x1d490298, SUC_NE_lexicon_tags_2 },
    { 0x110b029a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa389029c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8abd029e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32102a5, SUC_NE_lexicon_tags_2 },
    { 0x4c6b02a6, SUC_NE_lexicon_tags_7 },
    { 0xe0f902a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ff02aa, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9702ad, SUC_NE_lexicon_tags_2 },
    { 0xfd8302ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3d02b1, SUC_NE_lexicon_tags_2 },
    { 0xcf2702b2, SUC_NE_lexicon_tags_2 },
    { 0x6e8b02b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x808102b5, SUC_NE_lexicon_tags_2 },
    { 0x65f702b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2cb02b8, SUC_NE_lexicon_tags_2 },
    { 0x42c302b8, SUC_NE_lexicon_tags_245 },
    { 0x41db02b9, SUC_NE_lexicon_tags_3 },
    { 0x7dab02bb, SUC_NE_lexicon_tags_2 },
    { 0x476702ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6df02be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3bb02c0, SUC_NE_lexicon_tags_7 },
    { 0xebc502c1, SUC_NE_lexicon_tags_2 },
    { 0xc23702c2, SUC_NE_lexicon_tags_2 },
    { 0xfdc902c2, SUC_NE_lexicon_tags_2 },
    { 0x615702c3, SUC_NE_lexicon_tags_7 },
    { 0x262d02c5, SUC_NE_lexicon_tags_2 },
    { 0x419d02c3, SUC_NE_lexicon_tags_2 },
    { 0x11a302c2, SUC_NE_lexicon_tags_2 },
    { 0xb18902c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20bb02ca, SUC_NE_lexicon_tags_2 },
    { 0xd9ff02cb, SUC_NE_lexicon_tags_7 },
    { 0xdbad02ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x566702ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c902d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacfb02d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65102d9, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cb02dc, SUC_NE_lexicon_tags_2 },
    { 0xec5902dc, SUC_NE_lexicon_tags_2 },
    { 0xbb0902de, SUC_NE_lexicon_tags_2 },
    { 0x760302df, SUC_NE_lexicon_tags_2 },
    { 0x9bc302e0, SUC_NE_lexicon_tags_2 },
    { 0x7d6302e0, SUC_NE_lexicon_tags_2 },
    { 0x8a2f02e0, SUC_NE_lexicon_tags_2 },
    { 0x671d02e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398702e7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x17ed02e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7502eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acf02ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff302f2, SUC_NE_lexicon_tags_2 },
    { 0x127502f3, SUC_NE_lexicon_tags_2 },
    { 0x59c902f4, SUC_NE_lexicon_tags_3 },
    { 0xc4a902f5, SUC_NE_lexicon_tags_2 },
    { 0x69e302f6, SUC_NE_lexicon_tags_2 },
    { 0xf57f02f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x904f02f9, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90070301, SUC_NE_lexicon_tags_2 },
    { 0x46650301, SUC_NE_lexicon_tags_2 },
    { 0x70070303, SUC_NE_lexicon_tags_2 },
    { 0x85070302, SUC_NE_lexicon_tags_2 },
    { 0xd9150304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b0308, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ded030a, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf030d, SUC_NE_lexicon_tags_2 },
    { 0x60c3030e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ad10310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a890313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98790315, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15830319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32030320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1af0323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bf0326, SUC_NE_lexicon_tags_2 },
    { 0x482b0327, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611d032c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a70331, SUC_NE_lexicon_tags_2 },
    { 0x2c6d0331, SUC_NE_lexicon_tags_2 },
    { 0x34690332, SUC_NE_lexicon_tags_2 },
    { 0xfd1f0334, SUC_NE_lexicon_tags_2 },
    { 0x35790333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16b90337, SUC_NE_lexicon_tags_1 },
    { 0xea590337, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e63033b, SUC_NE_lexicon_tags_2 },
    { 0xd7f1033b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa470340, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xb0970342, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94b0344, SUC_NE_lexicon_tags_2 },
    { 0x8e730344, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f034c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x23e5034e, SUC_NE_lexicon_tags_2 },
    { 0xa6a9034f, SUC_NE_lexicon_tags_2 },
    { 0xa7630350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b70354, SUC_NE_lexicon_tags_2 },
    { 0xd5830355, SUC_NE_lexicon_tags_4 },
    { 0x6bef0356, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16310358, SUC_NE_lexicon_tags_18 },
    { 0x1d830359, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f37035b, SUC_NE_lexicon_tags_59 },
    { 0xa38f035c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc90360, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed190362, SUC_NE_lexicon_tags_4 },
    { 0xf5030363, SUC_NE_lexicon_tags_2 },
    { 0xbbfd0363, SUC_NE_lexicon_tags_4 },
    { 0x2cc50365, SUC_NE_lexicon_tags_2 },
    { 0xe9430364, SUC_NE_lexicon_tags_2 },
    { 0xa7710365, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d3036a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74b7036c, SUC_NE_lexicon_tags_2 },
    { 0x3a43036d, SUC_NE_lexicon_tags_2 },
    { 0x28c7036c, SUC_NE_lexicon_tags_2 },
    { 0x87e1036f, SUC_NE_lexicon_tags_2 },
    { 0xf903036c, SUC_NE_lexicon_tags_2 },
    { 0xda4f0371, SUC_NE_lexicon_tags_2 },
    { 0xb0a70372, SUC_NE_lexicon_tags_2 },
    { 0xa65b0373, SUC_NE_lexicon_tags_2 },
    { 0x5d290372, SUC_NE_lexicon_tags_4 },
    { 0x5c8f0373, SUC_NE_lexicon_tags_2 },
    { 0xa2670376, SUC_NE_lexicon_tags_17 },
    { 0xb9a30376, SUC_NE_lexicon_tags_2 },
    { 0x58290378, SUC_NE_lexicon_tags_18 },
    { 0xd1af0376, SUC_NE_lexicon_tags_2 },
    { 0xeab5037a, SUC_NE_lexicon_tags_0 },
    { 0x860b037b, SUC_NE_lexicon_tags_2 },
    { 0xd659037a, SUC_NE_lexicon_tags_2 },
    { 0x34ef0376, SUC_NE_lexicon_tags_2 },
    { 0x75390374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92f90380, SUC_NE_lexicon_tags_2 },
    { 0x58450381, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x55730383, SUC_NE_lexicon_tags_2 },
    { 0xb46f0383, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255038a, SUC_NE_lexicon_tags_2 },
    { 0xe131038a, SUC_NE_lexicon_tags_2 },
    { 0xccbd038c, SUC_NE_lexicon_tags_2 },
    { 0x5835038d, SUC_NE_lexicon_tags_40 },
    { 0x2e87038e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8950392, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3db10394, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e410397, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4010399, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc9039c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x531b039e, SUC_NE_lexicon_tags_2 },
    { 0xbe77039f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x324103a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2503a8, SUC_NE_lexicon_tags_2 },
    { 0x868b03a9, SUC_NE_lexicon_tags_2 },
    { 0x559303aa, SUC_NE_lexicon_tags_2 },
    { 0x793903ab, SUC_NE_lexicon_tags_2 },
    { 0xea4303ab, SUC_NE_lexicon_tags_3 },
    { 0xbb5f03aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b03b0, SUC_NE_lexicon_tags_2 },
    { 0x10cf03b0, SUC_NE_lexicon_tags_2 },
    { 0x62e503b0, SUC_NE_lexicon_tags_2 },
    { 0xb86103b3, SUC_NE_lexicon_tags_2 },
    { 0x39ff03b0, SUC_NE_lexicon_tags_2 },
    { 0x52ab03b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce703b9, SUC_NE_lexicon_tags_2 },
    { 0x140503b9, SUC_NE_lexicon_tags_2 },
    { 0xfbd503ba, SUC_NE_lexicon_tags_2 },
    { 0xba4903ba, SUC_NE_lexicon_tags_2 },
    { 0x131b03bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13bd03bf, SUC_NE_lexicon_tags_2 },
    { 0xacf703bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20af03c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adf03c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc903ca, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c9303cc, SUC_NE_lexicon_tags_2 },
    { 0xaf3703cd, SUC_NE_lexicon_tags_2 },
    { 0x65dd03ce, SUC_NE_lexicon_tags_2 },
    { 0x894903cf, SUC_NE_lexicon_tags_2 },
    { 0x9fe503cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16303d2, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30f03d6, SUC_NE_lexicon_tags_2 },
    { 0x6b1303d7, SUC_NE_lexicon_tags_1 },
    { 0xa91b03d8, SUC_NE_lexicon_tags_2 },
    { 0x829503d9, SUC_NE_lexicon_tags_19 },
    { 0x11c503d9, SUC_NE_lexicon_tags_2 },
    { 0x6a7f03d9, SUC_NE_lexicon_tags_4 },
    { 0x126903d8, SUC_NE_lexicon_tags_2 },
    { 0x71d903dc, SUC_NE_lexicon_tags_2 },
    { 0x817303de, SUC_NE_lexicon_tags_2 },
    { 0xc64b03de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x316b03e2, SUC_NE_lexicon_tags_2 },
    { 0x816f03e3, SUC_NE_lexicon_tags_2 },
    { 0xa55703e3, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x777703e6, SUC_NE_lexicon_tags_1 },
    { 0x8f0103e6, SUC_NE_lexicon_tags_2 },
    { 0x2cd03e7, SUC_NE_lexicon_tags_1 },
    { 0x24d103e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21f03f3, SUC_NE_lexicon_tags_2 },
    { 0xc12903f4, SUC_NE_lexicon_tags_2 },
    { 0xf0ed03f5, SUC_NE_lexicon_tags_2 },
    { 0x5d3903f6, SUC_NE_lexicon_tags_2 },
    { 0x4d2903f5, SUC_NE_lexicon_tags_2 },
    { 0x495303f3, SUC_NE_lexicon_tags_2 },
    { 0x9ed03f9, SUC_NE_lexicon_tags_2 },
    { 0x8f3303fa, SUC_NE_lexicon_tags_2 },
    { 0xe51903f5, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x4c5b03fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa17903ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68270401, SUC_NE_lexicon_tags_2 },
    { 0x429d0402, SUC_NE_lexicon_tags_3 },
    { 0x88e90403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab040b, SUC_NE_lexicon_tags_2 },
    { 0xf4ad040c, SUC_NE_lexicon_tags_2 },
    { 0x3da9040c, SUC_NE_lexicon_tags_2 },
    { 0x6039040d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2450410, SUC_NE_lexicon_tags_2 },
    { 0xe3b30411, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f950414, SUC_NE_lexicon_tags_2 },
    { 0x2ef90415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24550418, SUC_NE_lexicon_tags_2 },
    { 0x275d0419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1f1041b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1041f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f90422, SUC_NE_lexicon_tags_2 },
    { 0xd0af0423, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4850429, SUC_NE_lexicon_tags_2 },
    { 0x3d7042a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7211042d, SUC_NE_lexicon_tags_2 },
    { 0x7c77042e, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27470436, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61043f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78190443, SUC_NE_lexicon_tags_2 },
    { 0xbb610444, SUC_NE_lexicon_tags_2 },
    { 0x7ca70445, SUC_NE_lexicon_tags_2 },
    { 0xb2730446, SUC_NE_lexicon_tags_2 },
    { 0xc56b0444, SUC_NE_lexicon_tags_2 },
    { 0xe39f0448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5b044a, SUC_NE_lexicon_tags_2 },
    { 0xa843044a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ad044e, SUC_NE_lexicon_tags_2 },
    { 0xb671044f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb2f0451, SUC_NE_lexicon_tags_4 },
    { 0xd6df0452, SUC_NE_lexicon_tags_2 },
    { 0x83650453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5b0458, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5e1045a, SUC_NE_lexicon_tags_2 },
    { 0x5d9045b, SUC_NE_lexicon_tags_2 },
    { 0xc9ed045c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6de9045e, SUC_NE_lexicon_tags_40 },
    { 0xb141045f, SUC_NE_lexicon_tags_2 },
    { 0x78b30460, SUC_NE_lexicon_tags_2 },
    { 0xc2b10461, SUC_NE_lexicon_tags_7 },
    { 0xbfc90462, SUC_NE_lexicon_tags_11 },
    { 0xc7d90463, SUC_NE_lexicon_tags_2 },
    { 0xb0eb0464, SUC_NE_lexicon_tags_2 },
    { 0xb8f0460, SUC_NE_lexicon_tags_2 },
    { 0x718d0466, SUC_NE_lexicon_tags_2 },
    { 0x5c3d0467, SUC_NE_lexicon_tags_2 },
    { 0xf3010462, SUC_NE_lexicon_tags_2 },
    { 0xc5550469, SUC_NE_lexicon_tags_2 },
    { 0xaad90462, SUC_NE_lexicon_tags_2 },
    { 0xf95046b, SUC_NE_lexicon_tags_2 },
    { 0xd01b046b, SUC_NE_lexicon_tags_13 },
    { 0xd2cd046a, SUC_NE_lexicon_tags_2 },
    { 0x72d5046d, SUC_NE_lexicon_tags_2 },
    { 0xf6c5046f, SUC_NE_lexicon_tags_2 },
    { 0x7f410460, SUC_NE_lexicon_tags_2 },
    { 0xeeaf0471, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc3090473, SUC_NE_lexicon_tags_2 },
    { 0x8cf50474, SUC_NE_lexicon_tags_2 },
    { 0x8930473, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ff047b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6b047e, SUC_NE_lexicon_tags_2 },
    { 0x6ab5047f, SUC_NE_lexicon_tags_2 },
    { 0x3a950480, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b3048c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32410492, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe30495, SUC_NE_lexicon_tags_2 },
    { 0x62430496, SUC_NE_lexicon_tags_2 },
    { 0xd0cd0497, SUC_NE_lexicon_tags_2 },
    { 0xc3ad0495, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e99049a, SUC_NE_lexicon_tags_2 },
    { 0x5e4b049b, SUC_NE_lexicon_tags_2 },
    { 0x6f77049b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f04a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b04a4, SUC_NE_lexicon_tags_2 },
    { 0x44d04a5, SUC_NE_lexicon_tags_2 },
    { 0xa02d04a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc0b04a8, SUC_NE_lexicon_tags_2 },
    { 0x44f704a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ba704ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa71f04ad, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2eb104af, SUC_NE_lexicon_tags_2 },
    { 0xd22b04af, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494b04b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0304ba, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf52304bc, SUC_NE_lexicon_tags_2 },
    { 0x8e1704bc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb04c2, SUC_NE_lexicon_tags_2 },
    { 0x85ef04c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b04ca, SUC_NE_lexicon_tags_3 },
    { 0xbb704cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72b104cd, SUC_NE_lexicon_tags_2 },
    { 0xa40704ce, SUC_NE_lexicon_tags_2 },
    { 0xaaff04cd, SUC_NE_lexicon_tags_1 },
    { 0xf3e704d0, SUC_NE_lexicon_tags_2 },
    { 0xfc9d04d1, SUC_NE_lexicon_tags_7 },
    { 0x952704d1, SUC_NE_lexicon_tags_1 },
    { 0xd39f04d2, SUC_NE_lexicon_tags_2 },
    { 0x111704d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b0104d6, SUC_NE_lexicon_tags_2 },
    { 0x356304d7, SUC_NE_lexicon_tags_2 },
    { 0x861504d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e504da, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5704dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08704e8, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x41b304ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8304ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510904fc, SUC_NE_lexicon_tags_2 },
    { 0x5aed04fc, SUC_NE_lexicon_tags_2 },
    { 0xe8d704fe, SUC_NE_lexicon_tags_2 },
    { 0x4f7d04fd, SUC_NE_lexicon_tags_18 },
    { 0xdd1b0500, SUC_NE_lexicon_tags_2 },
    { 0xc87104fc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1450504, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d99050a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf131050c, SUC_NE_lexicon_tags_2 },
    { 0x11e5050c, SUC_NE_lexicon_tags_2 },
    { 0xfacd050e, SUC_NE_lexicon_tags_2 },
    { 0x4ad9050f, SUC_NE_lexicon_tags_2 },
    { 0x20b0510, SUC_NE_lexicon_tags_2 },
    { 0x133050f, SUC_NE_lexicon_tags_2 },
    { 0x547f050d, SUC_NE_lexicon_tags_2 },
    { 0xd8930513, SUC_NE_lexicon_tags_2 },
    { 0x532f0514, SUC_NE_lexicon_tags_89 },
    { 0xf2d0515, SUC_NE_lexicon_tags_2 },
    { 0x3eb30516, SUC_NE_lexicon_tags_2 },
    { 0xb2550512, SUC_NE_lexicon_tags_2 },
    { 0xacf0518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb145051a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1553051e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1b90520, SUC_NE_lexicon_tags_2 },
    { 0x9f1f0520, SUC_NE_lexicon_tags_7 },
    { 0x76610520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18390525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83b0528, SUC_NE_lexicon_tags_2 },
    { 0x47c30528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3130530, SUC_NE_lexicon_tags_34 },
    { 0x4b850531, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb70534, SUC_NE_lexicon_tags_2 },
    { 0x490b0535, SUC_NE_lexicon_tags_2 },
    { 0xec090536, SUC_NE_lexicon_tags_2 },
    { 0x66ff0537, SUC_NE_lexicon_tags_2 },
    { 0x3d2d0538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x407053a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f4d053c, SUC_NE_lexicon_tags_2 },
    { 0x1b51053d, SUC_NE_lexicon_tags_2 },
    { 0x1669053d, SUC_NE_lexicon_tags_2 },
    { 0x6cd7053f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62070544, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x1bd70546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51b30548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8def054c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eb0550, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47810554, SUC_NE_lexicon_tags_324 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196b0558, SUC_NE_lexicon_tags_172 },
    { 0, NULL },
    { 0x6fe3055a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd9055d, SUC_NE_lexicon_tags_2 },
    { 0x63e5055e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2690562, SUC_NE_lexicon_tags_7 },
    { 0x7f30563, SUC_NE_lexicon_tags_2 },
    { 0xb2e10564, SUC_NE_lexicon_tags_2 },
    { 0xf4990564, SUC_NE_lexicon_tags_2 },
    { 0x69f90566, SUC_NE_lexicon_tags_2 },
    { 0x50e70566, SUC_NE_lexicon_tags_2 },
    { 0xcc90568, SUC_NE_lexicon_tags_2 },
    { 0x1e5b0566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01056d, SUC_NE_lexicon_tags_3 },
    { 0xec7f056e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78890571, SUC_NE_lexicon_tags_2 },
    { 0x50990572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49710574, SUC_NE_lexicon_tags_2 },
    { 0xaa150575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe987057a, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0x84ed057c, SUC_NE_lexicon_tags_2 },
    { 0x488b057c, SUC_NE_lexicon_tags_2 },
    { 0x4013057e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8def0584, SUC_NE_lexicon_tags_2 },
    { 0x68ef0585, SUC_NE_lexicon_tags_3 },
    { 0xbf330586, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c70589, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb2f058b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5b058f, SUC_NE_lexicon_tags_2 },
    { 0x74030590, SUC_NE_lexicon_tags_4 },
    { 0x84b70590, SUC_NE_lexicon_tags_2 },
    { 0x6c0f0590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962d0598, SUC_NE_lexicon_tags_2 },
    { 0x94f0598, SUC_NE_lexicon_tags_2 },
    { 0xd25059a, SUC_NE_lexicon_tags_2 },
    { 0x2b97059b, SUC_NE_lexicon_tags_2 },
    { 0x2a49059c, SUC_NE_lexicon_tags_2 },
    { 0xe8e3059b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83e7059f, SUC_NE_lexicon_tags_2 },
    { 0x1f47059f, SUC_NE_lexicon_tags_4 },
    { 0x70505a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239305a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fb05aa, SUC_NE_lexicon_tags_2 },
    { 0x2bd105ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe905b4, SUC_NE_lexicon_tags_2 },
    { 0x70f305b4, SUC_NE_lexicon_tags_2 },
    { 0xf8c705b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48f05b9, SUC_NE_lexicon_tags_3 },
    { 0x7d9705b9, SUC_NE_lexicon_tags_2 },
    { 0xfdd05bb, SUC_NE_lexicon_tags_2 },
    { 0xc3d05bb, SUC_NE_lexicon_tags_2 },
    { 0x7f2d05b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4f05c0, SUC_NE_lexicon_tags_2 },
    { 0xdcc105c1, SUC_NE_lexicon_tags_2 },
    { 0xfb9705c2, SUC_NE_lexicon_tags_2 },
    { 0x578b05c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930305c8, SUC_NE_lexicon_tags_2 },
    { 0xaf5b05c9, SUC_NE_lexicon_tags_2 },
    { 0xa19305c8, SUC_NE_lexicon_tags_2 },
    { 0xbf905cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe6105cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887105d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7305d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9c505d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4105da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b305dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x904905e0, SUC_NE_lexicon_tags_2 },
    { 0x109905e1, SUC_NE_lexicon_tags_30 },
    { 0x9bcb05e2, SUC_NE_lexicon_tags_2 },
    { 0xf14905e3, SUC_NE_lexicon_tags_2 },
    { 0x8fab05e3, SUC_NE_lexicon_tags_2 },
    { 0xed6b05e5, SUC_NE_lexicon_tags_2 },
    { 0x832105e6, SUC_NE_lexicon_tags_2 },
    { 0xfb2705e4, SUC_NE_lexicon_tags_170 },
    { 0x447705e8, SUC_NE_lexicon_tags_59 },
    { 0x804105e9, SUC_NE_lexicon_tags_2 },
    { 0x1be105e3, SUC_NE_lexicon_tags_2 },
    { 0x4f05eb, SUC_NE_lexicon_tags_2 },
    { 0x5c9705ec, SUC_NE_lexicon_tags_238 },
    { 0xe59305ed, SUC_NE_lexicon_tags_17 },
    { 0x62e905eb, SUC_NE_lexicon_tags_2 },
    { 0x54d505ef, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbb705f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc28905f3, SUC_NE_lexicon_tags_2 },
    { 0xdccf05f4, SUC_NE_lexicon_tags_2 },
    { 0x109f05f5, SUC_NE_lexicon_tags_2 },
    { 0x9ac305f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a0305f8, SUC_NE_lexicon_tags_2 },
    { 0xae1d05f9, SUC_NE_lexicon_tags_3 },
    { 0x11f105f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e410600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08d0603, SUC_NE_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e210606, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3153060a, SUC_NE_lexicon_tags_2 },
    { 0x10fd060b, SUC_NE_lexicon_tags_2 },
    { 0xb0b060c, SUC_NE_lexicon_tags_2 },
    { 0xc201060c, SUC_NE_lexicon_tags_2 },
    { 0x89cd060e, SUC_NE_lexicon_tags_2 },
    { 0xab4b060c, SUC_NE_lexicon_tags_2 },
    { 0x62810610, SUC_NE_lexicon_tags_2 },
    { 0x9df3060d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c590614, SUC_NE_lexicon_tags_234 },
    { 0x5e5b0615, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702b061a, SUC_NE_lexicon_tags_2 },
    { 0x8763061b, SUC_NE_lexicon_tags_2 },
    { 0x2f7b061a, SUC_NE_lexicon_tags_2 },
    { 0xeb01061d, SUC_NE_lexicon_tags_2 },
    { 0xa62d061e, SUC_NE_lexicon_tags_7 },
    { 0x15b7061f, SUC_NE_lexicon_tags_2 },
    { 0xbe53061e, SUC_NE_lexicon_tags_5 },
    { 0x4fa1061b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa1062e, SUC_NE_lexicon_tags_2 },
    { 0x1189062f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x77250632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc89d0634, SUC_NE_lexicon_tags_2 },
    { 0xd6b0635, SUC_NE_lexicon_tags_2 },
    { 0xf21b0635, SUC_NE_lexicon_tags_2 },
    { 0xd28d0637, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2af063b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc45d063d, SUC_NE_lexicon_tags_11 },
    { 0xeea7063d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45130641, SUC_NE_lexicon_tags_2 },
    { 0x695b0641, SUC_NE_lexicon_tags_2 },
    { 0x41630643, SUC_NE_lexicon_tags_2 },
    { 0x33870644, SUC_NE_lexicon_tags_2 },
    { 0x7fa50645, SUC_NE_lexicon_tags_2 },
    { 0x22c30646, SUC_NE_lexicon_tags_2 },
    { 0x5950645, SUC_NE_lexicon_tags_2 },
    { 0xf2290648, SUC_NE_lexicon_tags_2 },
    { 0x46c30649, SUC_NE_lexicon_tags_2 },
    { 0xe7710648, SUC_NE_lexicon_tags_2 },
    { 0x93f70642, SUC_NE_lexicon_tags_2 },
    { 0x74b50643, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9055064e, SUC_NE_lexicon_tags_2 },
    { 0x7f1b064e, SUC_NE_lexicon_tags_2 },
    { 0x15c30650, SUC_NE_lexicon_tags_2 },
    { 0x65cd064f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94a90653, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x240d0656, SUC_NE_lexicon_tags_2 },
    { 0x54ab0657, SUC_NE_lexicon_tags_2 },
    { 0xe28b0658, SUC_NE_lexicon_tags_2 },
    { 0xb2ed0659, SUC_NE_lexicon_tags_2 },
    { 0xac3d0658, SUC_NE_lexicon_tags_2 },
    { 0x4d25065b, SUC_NE_lexicon_tags_1 },
    { 0xbfdf0659, SUC_NE_lexicon_tags_2 },
    { 0xbd95065d, SUC_NE_lexicon_tags_2 },
    { 0x35fb065e, SUC_NE_lexicon_tags_2 },
    { 0x9db7065d, SUC_NE_lexicon_tags_2 },
    { 0x3afd065f, SUC_NE_lexicon_tags_2 },
    { 0xd3bd065e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda90664, SUC_NE_lexicon_tags_2 },
    { 0x5ded0665, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01f066f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f50674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x953f0677, SUC_NE_lexicon_tags_2 },
    { 0x35410677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa273067a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fed067d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41030681, SUC_NE_lexicon_tags_2 },
    { 0x1da90682, SUC_NE_lexicon_tags_2 },
    { 0xf7830682, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d068b, SUC_NE_lexicon_tags_2 },
    { 0xbdf5068b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa519068f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ad3069a, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3add069e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8eb06a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc35106a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaef06ac, SUC_NE_lexicon_tags_2 },
    { 0xfd106ad, SUC_NE_lexicon_tags_2 },
    { 0xcfcb06ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdf06b2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xaced06b4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2fbb06b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e4f06b8, SUC_NE_lexicon_tags_2 },
    { 0x329106b8, SUC_NE_lexicon_tags_2 },
    { 0xea3106ba, SUC_NE_lexicon_tags_2 },
    { 0x1c9d06bb, SUC_NE_lexicon_tags_1 },
    { 0x8d06bc, SUC_NE_lexicon_tags_2 },
    { 0xc1e506bd, SUC_NE_lexicon_tags_2 },
    { 0x81ed06be, SUC_NE_lexicon_tags_2 },
    { 0xbbcd06bf, SUC_NE_lexicon_tags_19 },
    { 0x180706be, SUC_NE_lexicon_tags_2 },
    { 0xd2ad06b9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x434506c4, SUC_NE_lexicon_tags_3 },
    { 0x77ab06c4, SUC_NE_lexicon_tags_2 },
    { 0x926d06c6, SUC_NE_lexicon_tags_2 },
    { 0xaa8f06c7, SUC_NE_lexicon_tags_3 },
    { 0x1b1f06c8, SUC_NE_lexicon_tags_2 },
    { 0x8d8f06c7, SUC_NE_lexicon_tags_2 },
    { 0x6f5b06c9, SUC_NE_lexicon_tags_2 },
    { 0xae7906c9, SUC_NE_lexicon_tags_2 },
    { 0xf26706cc, SUC_NE_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24f06d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b506d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e706da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f6906dc, SUC_NE_lexicon_tags_2 },
    { 0xcf4f06dc, SUC_NE_lexicon_tags_2 },
    { 0x758b06de, SUC_NE_lexicon_tags_2 },
    { 0x738706de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fd06e1, SUC_NE_lexicon_tags_2 },
    { 0x17b306e1, SUC_NE_lexicon_tags_2 },
    { 0x3bd506e1, SUC_NE_lexicon_tags_2 },
    { 0x555b06e4, SUC_NE_lexicon_tags_2 },
    { 0x154106e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c8506e7, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x6e7906e9, SUC_NE_lexicon_tags_2 },
    { 0x6b4706e9, SUC_NE_lexicon_tags_2 },
    { 0xcb6d06ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70106f6, SUC_NE_lexicon_tags_2 },
    { 0xa8dd06f6, SUC_NE_lexicon_tags_2 },
    { 0x755706f6, SUC_NE_lexicon_tags_2 },
    { 0x64d306f9, SUC_NE_lexicon_tags_2 },
    { 0x985506f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178f06fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x749b0700, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75b0705, SUC_NE_lexicon_tags_2 },
    { 0x5af90706, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d330709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ac1070b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c69070d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31090711, SUC_NE_lexicon_tags_2 },
    { 0x4c190711, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0930718, SUC_NE_lexicon_tags_2 },
    { 0x5d710718, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xecc5071b, SUC_NE_lexicon_tags_3 },
    { 0xe2f3071c, SUC_NE_lexicon_tags_2 },
    { 0xe8ad071c, SUC_NE_lexicon_tags_2 },
    { 0x223071e, SUC_NE_lexicon_tags_2 },
    { 0x28b9071b, SUC_NE_lexicon_tags_2 },
    { 0x675071b, SUC_NE_lexicon_tags_2 },
    { 0x94fb0721, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb072b, SUC_NE_lexicon_tags_2 },
    { 0x3fa1072c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb99072e, SUC_NE_lexicon_tags_2 },
    { 0x7805072f, SUC_NE_lexicon_tags_2 },
    { 0x326f0730, SUC_NE_lexicon_tags_2 },
    { 0x1fd90731, SUC_NE_lexicon_tags_2 },
    { 0xc1a50730, SUC_NE_lexicon_tags_2 },
    { 0xff210733, SUC_NE_lexicon_tags_3 },
    { 0x9c670730, SUC_NE_lexicon_tags_2 },
    { 0xfe210735, SUC_NE_lexicon_tags_2 },
    { 0xc5ab0735, SUC_NE_lexicon_tags_2 },
    { 0x29c90735, SUC_NE_lexicon_tags_2 },
    { 0x8a90738, SUC_NE_lexicon_tags_2 },
    { 0xef50738, SUC_NE_lexicon_tags_2 },
    { 0x56c50739, SUC_NE_lexicon_tags_2 },
    { 0x3c39073b, SUC_NE_lexicon_tags_2 },
    { 0xaf4b072f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fff0741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae50747, SUC_NE_lexicon_tags_2 },
    { 0x70030748, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6a3074a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x957f074c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1901074f, SUC_NE_lexicon_tags_2 },
    { 0xca8b074f, SUC_NE_lexicon_tags_2 },
    { 0x38ef0750, SUC_NE_lexicon_tags_2 },
    { 0x5c5f0752, SUC_NE_lexicon_tags_113 },
    { 0xbb7f0752, SUC_NE_lexicon_tags_2 },
    { 0x421f0754, SUC_NE_lexicon_tags_2 },
    { 0x514d0755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ff075b, SUC_NE_lexicon_tags_14 },
    { 0x6835075c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a93075e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a4f0760, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x912d0762, SUC_NE_lexicon_tags_2 },
    { 0xe5590762, SUC_NE_lexicon_tags_25 },
    { 0x7d570764, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x147b0766, SUC_NE_lexicon_tags_2 },
    { 0x7ae70767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1151076a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bb076e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc8c70770, SUC_NE_lexicon_tags_2 },
    { 0x29010770, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f50774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde6d0776, SUC_NE_lexicon_tags_2 },
    { 0xdd970777, SUC_NE_lexicon_tags_2 },
    { 0x19390778, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8145077b, SUC_NE_lexicon_tags_2 },
    { 0x955f077b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8b077f, SUC_NE_lexicon_tags_2 },
    { 0x23630780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b550782, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55d10784, SUC_NE_lexicon_tags_2 },
    { 0xf190785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaef30787, SUC_NE_lexicon_tags_2 },
    { 0xef370788, SUC_NE_lexicon_tags_2 },
    { 0x1a0b0789, SUC_NE_lexicon_tags_2 },
    { 0xc33f0789, SUC_NE_lexicon_tags_2 },
    { 0xc843078b, SUC_NE_lexicon_tags_2 },
    { 0xc67b078c, SUC_NE_lexicon_tags_2 },
    { 0xedaf0789, SUC_NE_lexicon_tags_2 },
    { 0xe88f078a, SUC_NE_lexicon_tags_2 },
    { 0x47ed078f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a90794, SUC_NE_lexicon_tags_2 },
    { 0x2d830794, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4b50797, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49c50799, SUC_NE_lexicon_tags_2 },
    { 0xa579079a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9501079d, SUC_NE_lexicon_tags_1 },
    { 0x51df079e, SUC_NE_lexicon_tags_2 },
    { 0x9d09079e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb507a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5c107a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63307a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb36107a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ef07ad, SUC_NE_lexicon_tags_2 },
    { 0x77f07ae, SUC_NE_lexicon_tags_3 },
    { 0x341507af, SUC_NE_lexicon_tags_13 },
    { 0x61f307b0, SUC_NE_lexicon_tags_2 },
    { 0xcf8f07b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a707b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9707b9, SUC_NE_lexicon_tags_2 },
    { 0x646907ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc58b07bc, SUC_NE_lexicon_tags_7 },
    { 0x522b07bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f507c0, SUC_NE_lexicon_tags_2 },
    { 0x37907c1, SUC_NE_lexicon_tags_2 },
    { 0x62f107c2, SUC_NE_lexicon_tags_2 },
    { 0xd64d07c2, SUC_NE_lexicon_tags_2 },
    { 0xc88707c3, SUC_NE_lexicon_tags_2 },
    { 0x88af07c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f307c8, SUC_NE_lexicon_tags_2 },
    { 0xdaa907c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45907cc, SUC_NE_lexicon_tags_14 },
    { 0x820307cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a907d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b507d7, SUC_NE_lexicon_tags_2 },
    { 0xf3d307d8, SUC_NE_lexicon_tags_2 },
    { 0x47dd07d8, SUC_NE_lexicon_tags_2 },
    { 0x37f907da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e5307dc, SUC_NE_lexicon_tags_2 },
    { 0xe75f07dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32307e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6e907e2, SUC_NE_lexicon_tags_8 },
    { 0x84e707e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa2707e5, SUC_NE_lexicon_tags_1 },
    { 0x45d907e6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x786107e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cb07ec, SUC_NE_lexicon_tags_2 },
    { 0xa1f907ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad8d07ef, SUC_NE_lexicon_tags_2 },
    { 0x57ab07f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18707f6, SUC_NE_lexicon_tags_4 },
    { 0xf12107f7, SUC_NE_lexicon_tags_2 },
    { 0x848f07f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b07fb, SUC_NE_lexicon_tags_4 },
    { 0xfc3107fc, SUC_NE_lexicon_tags_2 },
    { 0x8a6d07fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8350801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e070803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96a50805, SUC_NE_lexicon_tags_2 },
    { 0xa8c70805, SUC_NE_lexicon_tags_2 },
    { 0xefdf0807, SUC_NE_lexicon_tags_2 },
    { 0xc78b0808, SUC_NE_lexicon_tags_2 },
    { 0xdaf50809, SUC_NE_lexicon_tags_2 },
    { 0x2ff70808, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c65080d, SUC_NE_lexicon_tags_3 },
    { 0xa9fd080e, SUC_NE_lexicon_tags_2 },
    { 0xc6ed080f, SUC_NE_lexicon_tags_2 },
    { 0x4b030810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x464f0812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea950814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e30818, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc9081b, SUC_NE_lexicon_tags_2 },
    { 0xbabd081c, SUC_NE_lexicon_tags_2 },
    { 0xd843081c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb381081f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d530821, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d090826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bbd0828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62eb082a, SUC_NE_lexicon_tags_2 },
    { 0x3ae7082a, SUC_NE_lexicon_tags_2 },
    { 0x66d1082a, SUC_NE_lexicon_tags_2 },
    { 0x8257082d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d70830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf0834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b250836, SUC_NE_lexicon_tags_1 },
    { 0x44a90837, SUC_NE_lexicon_tags_2 },
    { 0x80a30836, SUC_NE_lexicon_tags_2 },
    { 0xac6b0839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f3083f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf10841, SUC_NE_lexicon_tags_4 },
    { 0x64b90842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2a50844, SUC_NE_lexicon_tags_2 },
    { 0xd13b0844, SUC_NE_lexicon_tags_2 },
    { 0x17170846, SUC_NE_lexicon_tags_2 },
    { 0xc00d0846, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c3084c, SUC_NE_lexicon_tags_2 },
    { 0x87c7084c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11610852, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x11d10854, SUC_NE_lexicon_tags_2 },
    { 0x9f790855, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7f085b, SUC_NE_lexicon_tags_2 },
    { 0x18e3085c, SUC_NE_lexicon_tags_2 },
    { 0xb13b085b, SUC_NE_lexicon_tags_2 },
    { 0x80b1085c, SUC_NE_lexicon_tags_7 },
    { 0x3977085d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x637b0861, SUC_NE_lexicon_tags_1 },
    { 0xe5430861, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x65490865, SUC_NE_lexicon_tags_2 },
    { 0x296d0866, SUC_NE_lexicon_tags_3 },
    { 0xf39d0866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e5086c, SUC_NE_lexicon_tags_2 },
    { 0x7de5086d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc690873, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1f90875, SUC_NE_lexicon_tags_2 },
    { 0xdb210875, SUC_NE_lexicon_tags_2 },
    { 0x6fff0877, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b087d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ff0882, SUC_NE_lexicon_tags_2 },
    { 0x99b90883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1070885, SUC_NE_lexicon_tags_2 },
    { 0xc50886, SUC_NE_lexicon_tags_2 },
    { 0xea750887, SUC_NE_lexicon_tags_2 },
    { 0x141d0888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f088e, SUC_NE_lexicon_tags_0 },
    { 0xa73d088e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf30893, SUC_NE_lexicon_tags_7 },
    { 0x6db0894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f0897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a9d0899, SUC_NE_lexicon_tags_2 },
    { 0xcd230899, SUC_NE_lexicon_tags_2 },
    { 0x9dc3089a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9683089e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08d08a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b508a6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a708ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f908b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f08b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f508bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ed08c0, SUC_NE_lexicon_tags_8 },
    { 0xbfbb08c1, SUC_NE_lexicon_tags_2 },
    { 0x9cd708c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e508c5, SUC_NE_lexicon_tags_7 },
    { 0x46dd08c5, SUC_NE_lexicon_tags_2 },
    { 0x9d8908c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe39108c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58108cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x971b08cf, SUC_NE_lexicon_tags_2 },
    { 0xbb5908d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dd08d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230f08d9, SUC_NE_lexicon_tags_2 },
    { 0xc0908da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x614b08dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47ab08de, SUC_NE_lexicon_tags_2 },
    { 0x36e908df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x409308e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x749d08e4, SUC_NE_lexicon_tags_2 },
    { 0x95e108e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702708e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed308f5, SUC_NE_lexicon_tags_17 },
    { 0x915f08f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd9f08f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2308fa, SUC_NE_lexicon_tags_2 },
    { 0x1e8308fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389f08ff, SUC_NE_lexicon_tags_1 },
    { 0xbe5f0900, SUC_NE_lexicon_tags_2 },
    { 0x8bf0901, SUC_NE_lexicon_tags_25 },
    { 0xeab90900, SUC_NE_lexicon_tags_2 },
    { 0x9ea70903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbf0907, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c7090a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac50915, SUC_NE_lexicon_tags_17 },
    { 0x56590915, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9f091d, SUC_NE_lexicon_tags_2 },
    { 0xc851091d, SUC_NE_lexicon_tags_2 },
    { 0x4f59091f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35630923, SUC_NE_lexicon_tags_2 },
    { 0x6ae10923, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc539092a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec30934, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd0938, SUC_NE_lexicon_tags_2 },
    { 0x94090938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd52f093b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc813093d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d90941, SUC_NE_lexicon_tags_266 },
    { 0, NULL },
    { 0x6a7b0943, SUC_NE_lexicon_tags_2 },
    { 0x3f690944, SUC_NE_lexicon_tags_2 },
    { 0x2aeb0945, SUC_NE_lexicon_tags_5 },
    { 0xae170945, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8ff0948, SUC_NE_lexicon_tags_2 },
    { 0xb1790949, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b094d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7af094f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2f0952, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae410956, SUC_NE_lexicon_tags_2 },
    { 0xef0f0957, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe9610959, SUC_NE_lexicon_tags_2 },
    { 0x3b0d095a, SUC_NE_lexicon_tags_2 },
    { 0x3029095b, SUC_NE_lexicon_tags_2 },
    { 0x4ab1095c, SUC_NE_lexicon_tags_2 },
    { 0xc0ad095d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dd0964, SUC_NE_lexicon_tags_2 },
    { 0x72590964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7df096d, SUC_NE_lexicon_tags_2 },
    { 0xa725096e, SUC_NE_lexicon_tags_2 },
    { 0xf59f096f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb0975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0f0977, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf34b0979, SUC_NE_lexicon_tags_3 },
    { 0x5ba5097a, SUC_NE_lexicon_tags_2 },
    { 0x3a37097b, SUC_NE_lexicon_tags_2 },
    { 0x8e67097b, SUC_NE_lexicon_tags_2 },
    { 0x7521097d, SUC_NE_lexicon_tags_2 },
    { 0xa313097c, SUC_NE_lexicon_tags_2 },
    { 0xd31b097a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b30985, SUC_NE_lexicon_tags_49 },
    { 0x36930986, SUC_NE_lexicon_tags_2 },
    { 0xb0790986, SUC_NE_lexicon_tags_2 },
    { 0x905d0988, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f098b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0b098f, SUC_NE_lexicon_tags_2 },
    { 0x46ff0990, SUC_NE_lexicon_tags_2 },
    { 0x946f0991, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36690994, SUC_NE_lexicon_tags_2 },
    { 0x68010994, SUC_NE_lexicon_tags_2 },
    { 0x66f30995, SUC_NE_lexicon_tags_8 },
    { 0x9c5f0995, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1857099b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3337099f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86309a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbb09a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ed309a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8ed09ab, SUC_NE_lexicon_tags_3 },
    { 0x338109ac, SUC_NE_lexicon_tags_2 },
    { 0x2ff309ad, SUC_NE_lexicon_tags_2 },
    { 0x9a7509ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97b509b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25c509b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabf909b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29309b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x577109bb, SUC_NE_lexicon_tags_2 },
    { 0x935f09bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c6109be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731709c2, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781f09c8, SUC_NE_lexicon_tags_2 },
    { 0xd0c509c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43509cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11d09cf, SUC_NE_lexicon_tags_2 },
    { 0xd81509cf, SUC_NE_lexicon_tags_2 },
    { 0xb38309cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67509d7, SUC_NE_lexicon_tags_2 },
    { 0x18f309d7, SUC_NE_lexicon_tags_2 },
    { 0xe0ed09d9, SUC_NE_lexicon_tags_2 },
    { 0x172709d9, SUC_NE_lexicon_tags_2 },
    { 0x779b09db, SUC_NE_lexicon_tags_12 },
    { 0xfe6709dc, SUC_NE_lexicon_tags_2 },
    { 0x12709dd, SUC_NE_lexicon_tags_2 },
    { 0xa8c909dd, SUC_NE_lexicon_tags_2 },
    { 0x421f09dd, SUC_NE_lexicon_tags_2 },
    { 0xd01509da, SUC_NE_lexicon_tags_2 },
    { 0xaf9b09e1, SUC_NE_lexicon_tags_2 },
    { 0xae0509dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1d09eb, SUC_NE_lexicon_tags_1 },
    { 0x3f7509eb, SUC_NE_lexicon_tags_2 },
    { 0xb26f09eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acb09f0, SUC_NE_lexicon_tags_2 },
    { 0x271509f1, SUC_NE_lexicon_tags_7 },
    { 0x9c2109f2, SUC_NE_lexicon_tags_2 },
    { 0xa2e709f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77109f6, SUC_NE_lexicon_tags_2 },
    { 0x2f1f09f7, SUC_NE_lexicon_tags_2 },
    { 0x872309f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287909fe, SUC_NE_lexicon_tags_2 },
    { 0xda0709fe, SUC_NE_lexicon_tags_2 },
    { 0x22330a00, SUC_NE_lexicon_tags_2 },
    { 0x692f0a01, SUC_NE_lexicon_tags_3 },
    { 0xcdc50a01, SUC_NE_lexicon_tags_2 },
    { 0xb2910a03, SUC_NE_lexicon_tags_2 },
    { 0xdf5d0a03, SUC_NE_lexicon_tags_2 },
    { 0xb64d09ff, SUC_NE_lexicon_tags_2 },
    { 0x6b070a04, SUC_NE_lexicon_tags_2 },
    { 0x9e090a07, SUC_NE_lexicon_tags_2 },
    { 0xf4610a08, SUC_NE_lexicon_tags_2 },
    { 0xb9d90a09, SUC_NE_lexicon_tags_2 },
    { 0x9d4b0a0a, SUC_NE_lexicon_tags_2 },
    { 0x71ed0a0b, SUC_NE_lexicon_tags_2 },
    { 0x7b0509ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abd0a0f, SUC_NE_lexicon_tags_2 },
    { 0xedaf0a10, SUC_NE_lexicon_tags_2 },
    { 0xf7250a10, SUC_NE_lexicon_tags_2 },
    { 0xe80f0a12, SUC_NE_lexicon_tags_2 },
    { 0x5ab30a13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f90a18, SUC_NE_lexicon_tags_2 },
    { 0x79190a19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08b0a25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85eb0a29, SUC_NE_lexicon_tags_2 },
    { 0x41f0a2a, SUC_NE_lexicon_tags_2 },
    { 0x1e890a2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c230a2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9770a30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8cf0a32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63830a35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc90a38, SUC_NE_lexicon_tags_2 },
    { 0xa8170a39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4290a3e, SUC_NE_lexicon_tags_2 },
    { 0x73170a3e, SUC_NE_lexicon_tags_2 },
    { 0x57910a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x280b0a43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd50a46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347b0a4b, SUC_NE_lexicon_tags_2 },
    { 0x39b30a4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x776b0a4f, SUC_NE_lexicon_tags_2 },
    { 0x44f50a50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e4f0a52, SUC_NE_lexicon_tags_19 },
    { 0xf35d0a53, SUC_NE_lexicon_tags_210 },
    { 0x879d0a54, SUC_NE_lexicon_tags_2 },
    { 0x1650a55, SUC_NE_lexicon_tags_2 },
    { 0x5d4d0a56, SUC_NE_lexicon_tags_2 },
    { 0x36e30a57, SUC_NE_lexicon_tags_7 },
    { 0x7a8b0a56, SUC_NE_lexicon_tags_2 },
    { 0x15e30a59, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2be50a5b, SUC_NE_lexicon_tags_2 },
    { 0x9d510a5c, SUC_NE_lexicon_tags_2 },
    { 0x73ad0a5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf770a61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc330a63, SUC_NE_lexicon_tags_3 },
    { 0x19cf0a63, SUC_NE_lexicon_tags_2 },
    { 0x10530a65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deb0a69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1330a6b, SUC_NE_lexicon_tags_2 },
    { 0xad950a6c, SUC_NE_lexicon_tags_2 },
    { 0x34850a6d, SUC_NE_lexicon_tags_287 },
    { 0x759d0a6c, SUC_NE_lexicon_tags_2 },
    { 0x13a70a6e, SUC_NE_lexicon_tags_2 },
    { 0x7aed0a70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a7d0a72, SUC_NE_lexicon_tags_2 },
    { 0x46c30a73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4db0a77, SUC_NE_lexicon_tags_2 },
    { 0x72430a77, SUC_NE_lexicon_tags_2 },
    { 0x659f0a77, SUC_NE_lexicon_tags_2 },
    { 0xfd0b0a78, SUC_NE_lexicon_tags_2 },
    { 0x74350a77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92e70a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42610a7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73f0a82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d50a85, SUC_NE_lexicon_tags_2 },
    { 0xe5810a85, SUC_NE_lexicon_tags_2 },
    { 0x9c7f0a87, SUC_NE_lexicon_tags_2 },
    { 0xd2b0a85, SUC_NE_lexicon_tags_2 },
    { 0xc5cd0a87, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe250a8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87db0a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a0b0a91, SUC_NE_lexicon_tags_2 },
    { 0x9d710a91, SUC_NE_lexicon_tags_2 },
    { 0xe4030a92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebd0a97, SUC_NE_lexicon_tags_2 },
    { 0x78bf0a98, SUC_NE_lexicon_tags_2 },
    { 0xf67d0a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa330a9b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b310a9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d250aa1, SUC_NE_lexicon_tags_2 },
    { 0x63a30aa1, SUC_NE_lexicon_tags_2 },
    { 0xaf7f0aa3, SUC_NE_lexicon_tags_0 },
    { 0x8ccb0aa4, SUC_NE_lexicon_tags_2 },
    { 0xcd5f0aa4, SUC_NE_lexicon_tags_2 },
    { 0x2670aa4, SUC_NE_lexicon_tags_2 },
    { 0xd03f0aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2030aa9, SUC_NE_lexicon_tags_2 },
    { 0xd6030aa9, SUC_NE_lexicon_tags_2 },
    { 0xfc850aaa, SUC_NE_lexicon_tags_2 },
    { 0x6fed0aac, SUC_NE_lexicon_tags_2 },
    { 0xbb070aab, SUC_NE_lexicon_tags_2 },
    { 0x10f30aaa, SUC_NE_lexicon_tags_2 },
    { 0xd3c70aaf, SUC_NE_lexicon_tags_19 },
    { 0x93710ab0, SUC_NE_lexicon_tags_2 },
    { 0xab710aab, SUC_NE_lexicon_tags_2 },
    { 0x35cd0aab, SUC_NE_lexicon_tags_2 },
    { 0x1d330aab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35830ab5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c710ab8, SUC_NE_lexicon_tags_2 },
    { 0x9d30ab8, SUC_NE_lexicon_tags_2 },
    { 0x9c470aba, SUC_NE_lexicon_tags_2 },
    { 0x30790abb, SUC_NE_lexicon_tags_2 },
    { 0x22990abc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8410abf, SUC_NE_lexicon_tags_2 },
    { 0xf5d90ac0, SUC_NE_lexicon_tags_2 },
    { 0x1b950ac1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbb0ac6, SUC_NE_lexicon_tags_2 },
    { 0xdb230ac6, SUC_NE_lexicon_tags_2 },
    { 0xfc910ac8, SUC_NE_lexicon_tags_2 },
    { 0x42b0ac7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c90acd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e210acf, SUC_NE_lexicon_tags_2 },
    { 0x4b870acf, SUC_NE_lexicon_tags_2 },
    { 0x8d6d0ad1, SUC_NE_lexicon_tags_2 },
    { 0x79c50acf, SUC_NE_lexicon_tags_2 },
    { 0x3db90ad0, SUC_NE_lexicon_tags_2 },
    { 0xda690ad4, SUC_NE_lexicon_tags_7 },
    { 0x1330ad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef690ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8710ada, SUC_NE_lexicon_tags_2 },
    { 0xc10f0ada, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4690ae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec50ae4, SUC_NE_lexicon_tags_2 },
    { 0xbf730ae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93750ae7, SUC_NE_lexicon_tags_2 },
    { 0xbed90ae8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cf50aea, SUC_NE_lexicon_tags_2 },
    { 0xe06b0aeb, SUC_NE_lexicon_tags_2 },
    { 0x94490aeb, SUC_NE_lexicon_tags_2 },
    { 0x61a90aed, SUC_NE_lexicon_tags_3 },
    { 0x84910aee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b490af1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb970af4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd550afc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x832b0afe, SUC_NE_lexicon_tags_2 },
    { 0x651b0aff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x928b0b01, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba190b05, SUC_NE_lexicon_tags_2 },
    { 0x17290b06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f90b0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c30b0d, SUC_NE_lexicon_tags_2 },
    { 0x6b350b0e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1db0b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b30b13, SUC_NE_lexicon_tags_2 },
    { 0x29b30b14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6750b16, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x63510b19, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd890b1c, SUC_NE_lexicon_tags_2 },
    { 0x2e990b1d, SUC_NE_lexicon_tags_2 },
    { 0x5c810b1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1810b26, SUC_NE_lexicon_tags_2 },
    { 0xabed0b27, SUC_NE_lexicon_tags_2 },
    { 0xf5a90b26, SUC_NE_lexicon_tags_2 },
    { 0x70110b27, SUC_NE_lexicon_tags_2 },
    { 0xdd7f0b2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1d0b39, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a30b3c, SUC_NE_lexicon_tags_2 },
    { 0x6aaf0b3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49450b41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab790b4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2acd0b4d, SUC_NE_lexicon_tags_2 },
    { 0xc9830b4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b630b50, SUC_NE_lexicon_tags_2 },
    { 0x2d9f0b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430b0b58, SUC_NE_lexicon_tags_2 },
    { 0xfce90b59, SUC_NE_lexicon_tags_177 },
    { 0x56b30b5a, SUC_NE_lexicon_tags_2 },
    { 0xb5bb0b5a, SUC_NE_lexicon_tags_2 },
    { 0xd0d0b58, SUC_NE_lexicon_tags_30 },
    { 0xc1c90b59, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef0b63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6a50b65, SUC_NE_lexicon_tags_7 },
    { 0xd9f70b65, SUC_NE_lexicon_tags_2 },
    { 0x5e810b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3470b69, SUC_NE_lexicon_tags_355 },
    { 0x7dc30b69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1230b6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3f10b70, SUC_NE_lexicon_tags_2 },
    { 0xb3850b71, SUC_NE_lexicon_tags_2 },
    { 0x8f590b70, SUC_NE_lexicon_tags_7 },
    { 0x18910b73, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x759b0b76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b0b7b, SUC_NE_lexicon_tags_2 },
    { 0xba890b7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x755f0b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28310b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46af0b8b, SUC_NE_lexicon_tags_2 },
    { 0xb4490b8b, SUC_NE_lexicon_tags_2 },
    { 0xcc4f0b8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51350b8f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x60d50b91, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff30b94, SUC_NE_lexicon_tags_2 },
    { 0x16d30b94, SUC_NE_lexicon_tags_2 },
    { 0x25b10b94, SUC_NE_lexicon_tags_2 },
    { 0x62010b95, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0b0b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd70b9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x589f0ba0, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2f0ba4, SUC_NE_lexicon_tags_2 },
    { 0x28990ba5, SUC_NE_lexicon_tags_2 },
    { 0x360f0ba6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb0bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa10bb1, SUC_NE_lexicon_tags_2 },
    { 0x97fb0bb2, SUC_NE_lexicon_tags_2 },
    { 0x2d5b0bb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x443b0bb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76290bc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb0bc8, SUC_NE_lexicon_tags_2 },
    { 0xf71f0bc8, SUC_NE_lexicon_tags_2 },
    { 0xfcd70bca, SUC_NE_lexicon_tags_2 },
    { 0x29290bcb, SUC_NE_lexicon_tags_2 },
    { 0x28eb0bcb, SUC_NE_lexicon_tags_85 },
    { 0, NULL },
    { 0xabe10bce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38370bd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19910bd3, SUC_NE_lexicon_tags_2 },
    { 0xa1b0bd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e70bda, SUC_NE_lexicon_tags_2 },
    { 0xc5730bdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd130be0, SUC_NE_lexicon_tags_4 },
    { 0xcbe70be1, SUC_NE_lexicon_tags_2 },
    { 0x87930be2, SUC_NE_lexicon_tags_2 },
    { 0x7b230be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720d0be8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x653f0bea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53630bec, SUC_NE_lexicon_tags_2 },
    { 0x97430bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777b0bf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ff0bf5, SUC_NE_lexicon_tags_2 },
    { 0x77d90bf5, SUC_NE_lexicon_tags_2 },
    { 0x538d0bf7, SUC_NE_lexicon_tags_19 },
    { 0xc7ed0bf8, SUC_NE_lexicon_tags_2 },
    { 0x2db0bf7, SUC_NE_lexicon_tags_2 },
    { 0x73810bf6, SUC_NE_lexicon_tags_2 },
    { 0xc6a90bf9, SUC_NE_lexicon_tags_2 },
    { 0x68d90bfc, SUC_NE_lexicon_tags_2 },
    { 0x59790bfc, SUC_NE_lexicon_tags_2 },
    { 0xddf70bfe, SUC_NE_lexicon_tags_2 },
    { 0x80f90bf9, SUC_NE_lexicon_tags_2 },
    { 0x28b50bfb, SUC_NE_lexicon_tags_2 },
    { 0x2bdb0bfa, SUC_NE_lexicon_tags_2 },
    { 0x3aeb0c02, SUC_NE_lexicon_tags_2 },
    { 0x18f90c02, SUC_NE_lexicon_tags_1 },
    { 0x2d250bff, SUC_NE_lexicon_tags_2 },
    { 0xb7450c05, SUC_NE_lexicon_tags_2 },
    { 0x99510c06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e90c0d, SUC_NE_lexicon_tags_18 },
    { 0xcc2d0c0d, SUC_NE_lexicon_tags_3 },
    { 0x1e6b0c0f, SUC_NE_lexicon_tags_2 },
    { 0x1ec50c10, SUC_NE_lexicon_tags_2 },
    { 0x523b0c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa07d0c13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff450c15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2550c1c, SUC_NE_lexicon_tags_2 },
    { 0x9da10c1c, SUC_NE_lexicon_tags_2 },
    { 0x3f670c1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf110c21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5deb0c24, SUC_NE_lexicon_tags_2 },
    { 0x2fb0c25, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3750c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbe30c29, SUC_NE_lexicon_tags_2 },
    { 0xf1bf0c2a, SUC_NE_lexicon_tags_123 },
    { 0xbb0c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d050c2e, SUC_NE_lexicon_tags_2 },
    { 0x288f0c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdff70c31, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a630c34, SUC_NE_lexicon_tags_24 },
    { 0x9d390c35, SUC_NE_lexicon_tags_2 },
    { 0x71910c36, SUC_NE_lexicon_tags_2 },
    { 0x24ad0c34, SUC_NE_lexicon_tags_2 },
    { 0x63090c35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e90c3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e30c44, SUC_NE_lexicon_tags_2 },
    { 0x979b0c45, SUC_NE_lexicon_tags_50 },
    { 0x27930c44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb1d0c48, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ab0c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf95b0c4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aed0c4f, SUC_NE_lexicon_tags_2 },
    { 0x2f190c4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda50c56, SUC_NE_lexicon_tags_1 },
    { 0x35970c57, SUC_NE_lexicon_tags_32 },
    { 0xdcbf0c58, SUC_NE_lexicon_tags_2 },
    { 0x91ad0c58, SUC_NE_lexicon_tags_2 },
    { 0x451d0c5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc59f0c5c, SUC_NE_lexicon_tags_2 },
    { 0xee6b0c5d, SUC_NE_lexicon_tags_2 },
    { 0xcdb10c5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73630c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd6b0c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x252f0c6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec2f0c6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9be50c6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd81d0c70, SUC_NE_lexicon_tags_50 },
    { 0xffe70c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60530c73, SUC_NE_lexicon_tags_4 },
    { 0x3dcf0c74, SUC_NE_lexicon_tags_2 },
    { 0x77910c74, SUC_NE_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ff0c7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd0c81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7230c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6930c89, SUC_NE_lexicon_tags_2 },
    { 0x875f0c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b330c92, SUC_NE_lexicon_tags_2 },
    { 0x5cd30c93, SUC_NE_lexicon_tags_40 },
    { 0xd9f50c94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f2f0c96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe4b0c98, SUC_NE_lexicon_tags_2 },
    { 0xc1ad0c99, SUC_NE_lexicon_tags_2 },
    { 0x5a730c9a, SUC_NE_lexicon_tags_2 },
    { 0x547b0c9b, SUC_NE_lexicon_tags_2 },
    { 0xa2d70c9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d2f0c9e, SUC_NE_lexicon_tags_2 },
    { 0xa6790c9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f5b0ca1, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x88790ca4, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4890cb1, SUC_NE_lexicon_tags_2 },
    { 0x47410cb2, SUC_NE_lexicon_tags_2 },
    { 0x46e90cb3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e70cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706f0cbc, SUC_NE_lexicon_tags_2 },
    { 0x71270cbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3d0cc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f310cc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x531f0cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a10ccc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ed0ccf, SUC_NE_lexicon_tags_2 },
    { 0xb73d0ccf, SUC_NE_lexicon_tags_2 },
    { 0x529b0ccf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76690cd4, SUC_NE_lexicon_tags_2 },
    { 0x11b70cd5, SUC_NE_lexicon_tags_2 },
    { 0xa690cd5, SUC_NE_lexicon_tags_2 },
    { 0x42f10cd5, SUC_NE_lexicon_tags_2 },
    { 0x35030cd8, SUC_NE_lexicon_tags_2 },
    { 0xf3eb0cd9, SUC_NE_lexicon_tags_2 },
    { 0x36c90cda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b490ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61710cea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f0cee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd0cf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed1b0cf4, SUC_NE_lexicon_tags_2 },
    { 0x7a130cf4, SUC_NE_lexicon_tags_2 },
    { 0x1b390cf4, SUC_NE_lexicon_tags_2 },
    { 0xa4730cf7, SUC_NE_lexicon_tags_2 },
    { 0xe62b0cf4, SUC_NE_lexicon_tags_2 },
    { 0xeb850cf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd0cfe, SUC_NE_lexicon_tags_2 },
    { 0x7bff0cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x949f0d02, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb0d06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1770d08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cb0d0c, SUC_NE_lexicon_tags_2 },
    { 0xde890d0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77710d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75810d13, SUC_NE_lexicon_tags_2 },
    { 0x2cb0d13, SUC_NE_lexicon_tags_2 },
    { 0xdb830d15, SUC_NE_lexicon_tags_2 },
    { 0xbaf10d15, SUC_NE_lexicon_tags_2 },
    { 0x15bb0d17, SUC_NE_lexicon_tags_21 },
    { 0x2f30d18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd70d1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30b70d20, SUC_NE_lexicon_tags_2 },
    { 0xd8190d21, SUC_NE_lexicon_tags_2 },
    { 0x4b050d22, SUC_NE_lexicon_tags_17 },
    { 0x39f50d23, SUC_NE_lexicon_tags_2 },
    { 0x6f870d24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a850d26, SUC_NE_lexicon_tags_3 },
    { 0xa99f0d27, SUC_NE_lexicon_tags_2 },
    { 0x1750d27, SUC_NE_lexicon_tags_2 },
    { 0x7ac50d28, SUC_NE_lexicon_tags_2 },
    { 0x54cd0d27, SUC_NE_lexicon_tags_2 },
    { 0xc590d2b, SUC_NE_lexicon_tags_2 },
    { 0x72d70d28, SUC_NE_lexicon_tags_2 },
    { 0x2bbd0d2c, SUC_NE_lexicon_tags_2 },
    { 0xb9fd0d2e, SUC_NE_lexicon_tags_2 },
    { 0x336d0d2f, SUC_NE_lexicon_tags_2 },
    { 0xe0dd0d30, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xd290d33, SUC_NE_lexicon_tags_2 },
    { 0x5cfb0d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e90d39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda330d3b, SUC_NE_lexicon_tags_2 },
    { 0x73210d3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbc70d3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5590d40, SUC_NE_lexicon_tags_2 },
    { 0x1bb0d41, SUC_NE_lexicon_tags_2 },
    { 0x7c070d42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2b30d44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd550d46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a950d4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9610d4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b470d51, SUC_NE_lexicon_tags_2 },
    { 0x6a1b0d51, SUC_NE_lexicon_tags_2 },
    { 0x66850d53, SUC_NE_lexicon_tags_2 },
    { 0xda9f0d54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f50d58, SUC_NE_lexicon_tags_2 },
    { 0xa1970d59, SUC_NE_lexicon_tags_2 },
    { 0x15c10d5a, SUC_NE_lexicon_tags_2 },
    { 0xb3070d5b, SUC_NE_lexicon_tags_2 },
    { 0x1cf90d5c, SUC_NE_lexicon_tags_2 },
    { 0xa1090d5d, SUC_NE_lexicon_tags_2 },
    { 0xf6010d5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70750d63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7eb0d65, SUC_NE_lexicon_tags_2 },
    { 0xaf490d66, SUC_NE_lexicon_tags_2 },
    { 0x49f0d67, SUC_NE_lexicon_tags_2 },
    { 0x4a1b0d65, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58f0d6b, SUC_NE_lexicon_tags_2 },
    { 0xe4bb0d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a90d70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de30d73, SUC_NE_lexicon_tags_2 },
    { 0x135b0d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e30d78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fd10d7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x768d0d7c, SUC_NE_lexicon_tags_2 },
    { 0x75ed0d7d, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x870d7f, SUC_NE_lexicon_tags_2 },
    { 0xa6250d80, SUC_NE_lexicon_tags_2 },
    { 0x84250d7f, SUC_NE_lexicon_tags_2 },
    { 0x31470d82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf650d85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56df0d87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f0d8a, SUC_NE_lexicon_tags_2 },
    { 0x62090d8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f790d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc70d0d8f, SUC_NE_lexicon_tags_2 },
    { 0xe1890d90, SUC_NE_lexicon_tags_326 },
    { 0, NULL },
    { 0x15410d92, SUC_NE_lexicon_tags_2 },
    { 0x38090d92, SUC_NE_lexicon_tags_1 },
    { 0x58d30d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37af0d97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3b70d99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbd0d9f, SUC_NE_lexicon_tags_434 },
    { 0x51250da0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x727d0da3, SUC_NE_lexicon_tags_2 },
    { 0xe4a50da4, SUC_NE_lexicon_tags_2 },
    { 0x2b0b0da4, SUC_NE_lexicon_tags_2 },
    { 0x6f170da4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d0daa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df0db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d50db5, SUC_NE_lexicon_tags_2 },
    { 0x52d70db5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f250db8, SUC_NE_lexicon_tags_2 },
    { 0x4230db9, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xce770dbc, SUC_NE_lexicon_tags_34 },
    { 0x91450dbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3c30dbf, SUC_NE_lexicon_tags_2 },
    { 0xfeb0dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb8d0dc2, SUC_NE_lexicon_tags_5 },
    { 0x41590dc3, SUC_NE_lexicon_tags_32 },
    { 0xa39b0dc4, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x18790dc6, SUC_NE_lexicon_tags_2 },
    { 0x18a30dc7, SUC_NE_lexicon_tags_58 },
    { 0xe0970dc8, SUC_NE_lexicon_tags_2 },
    { 0x4bed0dc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f0dcc, SUC_NE_lexicon_tags_2 },
    { 0x79e30dcc, SUC_NE_lexicon_tags_2 },
    { 0x6da50dce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43bf0dd0, SUC_NE_lexicon_tags_2 },
    { 0x5b310dd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93f0dd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc230dd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6abb0dd9, SUC_NE_lexicon_tags_2 },
    { 0x55310dd9, SUC_NE_lexicon_tags_2 },
    { 0x7ab30ddb, SUC_NE_lexicon_tags_3 },
    { 0xd0150dd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x850b0dde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21510de2, SUC_NE_lexicon_tags_25 },
    { 0xbbaf0de3, SUC_NE_lexicon_tags_2 },
    { 0x76010de3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b70de7, SUC_NE_lexicon_tags_2 },
    { 0xc21b0de8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c30deb, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x412f0dee, SUC_NE_lexicon_tags_2 },
    { 0x71670dee, SUC_NE_lexicon_tags_2 },
    { 0x7a5d0df0, SUC_NE_lexicon_tags_2 },
    { 0x58a70df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe50df5, SUC_NE_lexicon_tags_2 },
    { 0x963b0df5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e70dfc, SUC_NE_lexicon_tags_1 },
    { 0x73e50dfd, SUC_NE_lexicon_tags_2 },
    { 0x50950dfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf530e02, SUC_NE_lexicon_tags_2 },
    { 0x2e0f0e03, SUC_NE_lexicon_tags_2 },
    { 0x63e10e04, SUC_NE_lexicon_tags_2 },
    { 0xdd9d0e03, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x997f0e08, SUC_NE_lexicon_tags_2 },
    { 0x59f30e09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66eb0e0c, SUC_NE_lexicon_tags_2 },
    { 0x96130e0d, SUC_NE_lexicon_tags_2 },
    { 0x2270e0d, SUC_NE_lexicon_tags_2 },
    { 0x45fb0e0f, SUC_NE_lexicon_tags_2 },
    { 0x1f130e10, SUC_NE_lexicon_tags_2 },
    { 0x7b9b0e11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95850e15, SUC_NE_lexicon_tags_2 },
    { 0x3d050e15, SUC_NE_lexicon_tags_2 },
    { 0x1a750e17, SUC_NE_lexicon_tags_2 },
    { 0x8f110e18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6550e1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3aab0e1c, SUC_NE_lexicon_tags_2 },
    { 0x7f930e1c, SUC_NE_lexicon_tags_98 },
    { 0xcef10e1e, SUC_NE_lexicon_tags_2 },
    { 0x7c810e1c, SUC_NE_lexicon_tags_2 },
    { 0x210b0e1f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf50e25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8430e29, SUC_NE_lexicon_tags_2 },
    { 0xdceb0e2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8f0e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf190e31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb570e34, SUC_NE_lexicon_tags_3 },
    { 0x862d0e35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x719f0e38, SUC_NE_lexicon_tags_2 },
    { 0xbfaf0e38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36bd0e3c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x71d10e3e, SUC_NE_lexicon_tags_2 },
    { 0xc4870e3f, SUC_NE_lexicon_tags_7 },
    { 0x973b0e3f, SUC_NE_lexicon_tags_24 },
    { 0xdf9d0e41, SUC_NE_lexicon_tags_2 },
    { 0x59c10e40, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x236d0e44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25210e46, SUC_NE_lexicon_tags_2 },
    { 0x96050e47, SUC_NE_lexicon_tags_2 },
    { 0xfcc90e47, SUC_NE_lexicon_tags_2 },
    { 0xcb2b0e47, SUC_NE_lexicon_tags_2 },
    { 0x8d8d0e49, SUC_NE_lexicon_tags_2 },
    { 0xe7ad0e48, SUC_NE_lexicon_tags_2 },
    { 0xe7b70e46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240f0e54, SUC_NE_lexicon_tags_2 },
    { 0x66770e55, SUC_NE_lexicon_tags_2 },
    { 0xc79f0e55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fb0e5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7d0e5f, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf0e66, SUC_NE_lexicon_tags_2 },
    { 0x4cb30e67, SUC_NE_lexicon_tags_2 },
    { 0x76ff0e68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84c10e6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985b0e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a10e73, SUC_NE_lexicon_tags_2 },
    { 0x21670e73, SUC_NE_lexicon_tags_2 },
    { 0x85490e73, SUC_NE_lexicon_tags_2 },
    { 0x276f0e74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1810e78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86370e7b, SUC_NE_lexicon_tags_2 },
    { 0x1d3f0e7b, SUC_NE_lexicon_tags_2 },
    { 0xe3a90e7d, SUC_NE_lexicon_tags_4 },
    { 0xf0ab0e7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62830e82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c90e8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45370e91, SUC_NE_lexicon_tags_2 },
    { 0xe3ad0e91, SUC_NE_lexicon_tags_2 },
    { 0x3ea70e93, SUC_NE_lexicon_tags_2 },
    { 0xf30b0e93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b0e9e, SUC_NE_lexicon_tags_2 },
    { 0xd1330e9f, SUC_NE_lexicon_tags_2 },
    { 0x3d4b0ea0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x940b0ea2, SUC_NE_lexicon_tags_2 },
    { 0xc7990ea2, SUC_NE_lexicon_tags_2 },
    { 0x86e90ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7410ea7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e6b0ea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ad0eac, SUC_NE_lexicon_tags_2 },
    { 0x8cb30eac, SUC_NE_lexicon_tags_4 },
    { 0x16c90eac, SUC_NE_lexicon_tags_2 },
    { 0xf8310eaf, SUC_NE_lexicon_tags_2 },
    { 0xa0910ead, SUC_NE_lexicon_tags_104 },
    { 0, NULL },
    { 0x62d70eb2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xf06f0eb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24d0eb7, SUC_NE_lexicon_tags_2 },
    { 0xe62f0eb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2fd0eba, SUC_NE_lexicon_tags_21 },
    { 0x44ef0ebb, SUC_NE_lexicon_tags_2 },
    { 0xacc50ebb, SUC_NE_lexicon_tags_2 },
    { 0xab690ebd, SUC_NE_lexicon_tags_2 },
    { 0xb7f10ebd, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcb0ec6, SUC_NE_lexicon_tags_2 },
    { 0x8df50ec7, SUC_NE_lexicon_tags_2 },
    { 0xf7970ec8, SUC_NE_lexicon_tags_56 },
    { 0x1b130ec9, SUC_NE_lexicon_tags_2 },
    { 0x323f0eca, SUC_NE_lexicon_tags_2 },
    { 0x12670ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89430ed2, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0f0ed6, SUC_NE_lexicon_tags_1 },
    { 0xd4f50ed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe290ed9, SUC_NE_lexicon_tags_2 },
    { 0x38d30eda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2510edd, SUC_NE_lexicon_tags_36 },
    { 0xdcab0edd, SUC_NE_lexicon_tags_2 },
    { 0x37310ede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75d0ee2, SUC_NE_lexicon_tags_2 },
    { 0x3bbb0ee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9910ee5, SUC_NE_lexicon_tags_2 },
    { 0xcd6b0ee6, SUC_NE_lexicon_tags_2 },
    { 0xd73f0ee7, SUC_NE_lexicon_tags_2 },
    { 0x59e50ee8, SUC_NE_lexicon_tags_2 },
    { 0x13f0ee9, SUC_NE_lexicon_tags_4 },
    { 0x9310ee9, SUC_NE_lexicon_tags_2 },
    { 0x251f0ee6, SUC_NE_lexicon_tags_2 },
    { 0xe3b30ee8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb98d0eee, SUC_NE_lexicon_tags_2 },
    { 0xdced0eef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6930ef2, SUC_NE_lexicon_tags_2 },
    { 0x6f8b0ef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9d0ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbd0efb, SUC_NE_lexicon_tags_2 },
    { 0xaf30efc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b0eff, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f04, SUC_NE_lexicon_tags_2 },
    { 0x4fb70f05, SUC_NE_lexicon_tags_2 },
    { 0x54670f05, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x953d0f08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd88b0f0a, SUC_NE_lexicon_tags_8 },
    { 0x74870f0a, SUC_NE_lexicon_tags_2 },
    { 0xf1fb0f0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbf0f13, SUC_NE_lexicon_tags_2 },
    { 0xb96f0f14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dfb0f16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f10f1a, SUC_NE_lexicon_tags_2 },
    { 0xb4510f1b, SUC_NE_lexicon_tags_2 },
    { 0xbeb70f1c, SUC_NE_lexicon_tags_2 },
    { 0xcfb70f1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f90f23, SUC_NE_lexicon_tags_2 },
    { 0x81490f23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e10f28, SUC_NE_lexicon_tags_2 },
    { 0xfed70f29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75890f2b, SUC_NE_lexicon_tags_2 },
    { 0x4e7b0f2c, SUC_NE_lexicon_tags_2 },
    { 0x17d10f2d, SUC_NE_lexicon_tags_2 },
    { 0xa7a70f2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7590f33, SUC_NE_lexicon_tags_2 },
    { 0x9ed30f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fb0f37, SUC_NE_lexicon_tags_2 },
    { 0x634d0f38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81290f3e, SUC_NE_lexicon_tags_2 },
    { 0x93e10f3f, SUC_NE_lexicon_tags_2 },
    { 0x16fd0f40, SUC_NE_lexicon_tags_2 },
    { 0xfecb0f41, SUC_NE_lexicon_tags_1 },
    { 0xfccf0f42, SUC_NE_lexicon_tags_2 },
    { 0x9af10f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b050f45, SUC_NE_lexicon_tags_2 },
    { 0x9bc90f46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb30f49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79d30f4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eb0f50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5050f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26850f5a, SUC_NE_lexicon_tags_2 },
    { 0x8f530f5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f990f5f, SUC_NE_lexicon_tags_2 },
    { 0xb9990f60, SUC_NE_lexicon_tags_3 },
    { 0x21950f61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc30f64, SUC_NE_lexicon_tags_2 },
    { 0xbc150f64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4990f67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f90f6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb54b0f6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19590f73, SUC_NE_lexicon_tags_2 },
    { 0xf1170f74, SUC_NE_lexicon_tags_2 },
    { 0x87c90f75, SUC_NE_lexicon_tags_2 },
    { 0x1cf70f75, SUC_NE_lexicon_tags_2 },
    { 0xf9850f77, SUC_NE_lexicon_tags_2 },
    { 0x269f0f76, SUC_NE_lexicon_tags_2 },
    { 0x59350f79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d910f7f, SUC_NE_lexicon_tags_2 },
    { 0x26b0f80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd650f85, SUC_NE_lexicon_tags_2 },
    { 0xa4030f85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff690f8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea710f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f750f90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf20b0f92, SUC_NE_lexicon_tags_2 },
    { 0x530f0f93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b790f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde770f9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c10fa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24610fab, SUC_NE_lexicon_tags_2 },
    { 0x8de90fac, SUC_NE_lexicon_tags_2 },
    { 0x208f0fac, SUC_NE_lexicon_tags_2 },
    { 0x1b490fae, SUC_NE_lexicon_tags_2 },
    { 0x3fbd0fae, SUC_NE_lexicon_tags_2 },
    { 0x20df0faf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b370fb5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc8b10fb7, SUC_NE_lexicon_tags_2 },
    { 0x62250fb8, SUC_NE_lexicon_tags_2 },
    { 0x6ccb0fb9, SUC_NE_lexicon_tags_3 },
    { 0x76290fb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdb0fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3b0fc4, SUC_NE_lexicon_tags_2 },
    { 0xe6d50fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9870fcb, SUC_NE_lexicon_tags_2 },
    { 0xa8ef0fcb, SUC_NE_lexicon_tags_2 },
    { 0x7f9f0fcd, SUC_NE_lexicon_tags_2 },
    { 0x10c10fce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd81f0fd0, SUC_NE_lexicon_tags_2 },
    { 0x2d430fd1, SUC_NE_lexicon_tags_2 },
    { 0x3ced0fd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa56b0fd4, SUC_NE_lexicon_tags_2 },
    { 0x95b90fd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70590fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8d0fde, SUC_NE_lexicon_tags_2 },
    { 0x5c4d0fdf, SUC_NE_lexicon_tags_88 },
    { 0x75ef0fde, SUC_NE_lexicon_tags_2 },
    { 0x91530fe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ff50fe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56c90fe5, SUC_NE_lexicon_tags_2 },
    { 0x6b3f0fe6, SUC_NE_lexicon_tags_2 },
    { 0x67b30fe7, SUC_NE_lexicon_tags_2 },
    { 0xb8b70fe7, SUC_NE_lexicon_tags_2 },
    { 0x4b550fe9, SUC_NE_lexicon_tags_2 },
    { 0x66bd0fea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a850fef, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0x12830ff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8730ff4, SUC_NE_lexicon_tags_2 },
    { 0x8190ff4, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae10ff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3130ffb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ed50ffd, SUC_NE_lexicon_tags_2 },
    { 0x9d910ffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0251000, SUC_NE_lexicon_tags_2 },
    { 0x627f1001, SUC_NE_lexicon_tags_2 },
    { 0x23af1001, SUC_NE_lexicon_tags_2 },
    { 0x1e6d1003, SUC_NE_lexicon_tags_2 },
    { 0x5ea11004, SUC_NE_lexicon_tags_2 },
    { 0xdb431002, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2d100b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb025100e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67371013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed4b1015, SUC_NE_lexicon_tags_2 },
    { 0xbc3d1015, SUC_NE_lexicon_tags_2 },
    { 0x2cc51017, SUC_NE_lexicon_tags_2 },
    { 0x696f1018, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b1101a, SUC_NE_lexicon_tags_7 },
    { 0xa8e9101a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94691023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51f102b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce5b102d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43f3102f, SUC_NE_lexicon_tags_2 },
    { 0x2dfb102f, SUC_NE_lexicon_tags_2 },
    { 0x9aa5102f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba91034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f951039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x377103b, SUC_NE_lexicon_tags_3 },
    { 0xe4a5103b, SUC_NE_lexicon_tags_2 },
    { 0x5025103d, SUC_NE_lexicon_tags_1 },
    { 0x3add103e, SUC_NE_lexicon_tags_2 },
    { 0xdc6f103f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e31044, SUC_NE_lexicon_tags_2 },
    { 0xaaab1045, SUC_NE_lexicon_tags_2 },
    { 0xd19d1046, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa925104b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55c7104d, SUC_NE_lexicon_tags_2 },
    { 0x1f27104d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b731051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34131053, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482d1058, SUC_NE_lexicon_tags_2 },
    { 0x29971058, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c99105d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf371061, SUC_NE_lexicon_tags_73 },
    { 0, NULL },
    { 0x40b11063, SUC_NE_lexicon_tags_2 },
    { 0x2eaf1063, SUC_NE_lexicon_tags_2 },
    { 0xda491065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec771068, SUC_NE_lexicon_tags_2 },
    { 0x72111069, SUC_NE_lexicon_tags_2 },
    { 0xc8dd106a, SUC_NE_lexicon_tags_14 },
    { 0xcda1106b, SUC_NE_lexicon_tags_2 },
    { 0x43b7106a, SUC_NE_lexicon_tags_2 },
    { 0x3ed106b, SUC_NE_lexicon_tags_2 },
    { 0xe705106d, SUC_NE_lexicon_tags_2 },
    { 0x4049106e, SUC_NE_lexicon_tags_2 },
    { 0x4e2f106f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21131072, SUC_NE_lexicon_tags_2 },
    { 0x32831073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x281b1075, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x857f1077, SUC_NE_lexicon_tags_4 },
    { 0xe7e71077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9107a, SUC_NE_lexicon_tags_7 },
    { 0x5bbf107b, SUC_NE_lexicon_tags_2 },
    { 0xf371107b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e0f107e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1111081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6991084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d11087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x396b1089, SUC_NE_lexicon_tags_2 },
    { 0x8ec7108a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750f108e, SUC_NE_lexicon_tags_2 },
    { 0x180b108e, SUC_NE_lexicon_tags_3 },
    { 0xdad3108e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695b1096, SUC_NE_lexicon_tags_1 },
    { 0x3911097, SUC_NE_lexicon_tags_2 },
    { 0xf7e91098, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992d10a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8110a8, SUC_NE_lexicon_tags_2 },
    { 0xd27310a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48ad10ab, SUC_NE_lexicon_tags_2 },
    { 0xff3f10ac, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d910ae, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0xf3eb10b0, SUC_NE_lexicon_tags_2 },
    { 0x24e910b1, SUC_NE_lexicon_tags_2 },
    { 0x197310b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17bd10b4, SUC_NE_lexicon_tags_2 },
    { 0xb35910b5, SUC_NE_lexicon_tags_30 },
    { 0x8e510b6, SUC_NE_lexicon_tags_2 },
    { 0x4ab10b7, SUC_NE_lexicon_tags_2 },
    { 0x6b2310b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560510be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x985d10c0, SUC_NE_lexicon_tags_2 },
    { 0x48b910c0, SUC_NE_lexicon_tags_2 },
    { 0xbabf10c2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7f10c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4b10cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde3b10cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb8910cf, SUC_NE_lexicon_tags_2 },
    { 0xe18910cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce310d3, SUC_NE_lexicon_tags_2 },
    { 0xa5f710d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fa710d6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6910da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3b10de, SUC_NE_lexicon_tags_39 },
    { 0xd07510df, SUC_NE_lexicon_tags_2 },
    { 0x81b710e0, SUC_NE_lexicon_tags_308 },
    { 0x297910e1, SUC_NE_lexicon_tags_2 },
    { 0xf70910e2, SUC_NE_lexicon_tags_2 },
    { 0xe4fb10e3, SUC_NE_lexicon_tags_2 },
    { 0xcc8b10de, SUC_NE_lexicon_tags_2 },
    { 0xb45d10e5, SUC_NE_lexicon_tags_2 },
    { 0xa66310e5, SUC_NE_lexicon_tags_2 },
    { 0xb42d10e7, SUC_NE_lexicon_tags_2 },
    { 0x29e710e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x239710eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d510ef, SUC_NE_lexicon_tags_12 },
    { 0xf40f10ef, SUC_NE_lexicon_tags_19 },
    { 0x416310f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5b10f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab1910f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x876310f9, SUC_NE_lexicon_tags_7 },
    { 0xf18b10fa, SUC_NE_lexicon_tags_2 },
    { 0x1aff10fa, SUC_NE_lexicon_tags_36 },
    { 0x6d3910fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d710ff, SUC_NE_lexicon_tags_2 },
    { 0x57191100, SUC_NE_lexicon_tags_2 },
    { 0x60111101, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x305b1103, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb67110b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5cd110d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778f1111, SUC_NE_lexicon_tags_2 },
    { 0xb2bd1111, SUC_NE_lexicon_tags_2 },
    { 0x4a511113, SUC_NE_lexicon_tags_7 },
    { 0xd4f1113, SUC_NE_lexicon_tags_2 },
    { 0xc7831115, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6191118, SUC_NE_lexicon_tags_2 },
    { 0x672d1119, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x1469111b, SUC_NE_lexicon_tags_2 },
    { 0x1fed111b, SUC_NE_lexicon_tags_2 },
    { 0x7e27111b, SUC_NE_lexicon_tags_2 },
    { 0x4e1f111e, SUC_NE_lexicon_tags_2 },
    { 0x90df111f, SUC_NE_lexicon_tags_2 },
    { 0x31b31120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab11124, SUC_NE_lexicon_tags_2 },
    { 0x50171124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2abd1127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a7112a, SUC_NE_lexicon_tags_2 },
    { 0x3175112b, SUC_NE_lexicon_tags_2 },
    { 0xf70d112c, SUC_NE_lexicon_tags_2 },
    { 0xc577112d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f31132, SUC_NE_lexicon_tags_2 },
    { 0x80bd1133, SUC_NE_lexicon_tags_2 },
    { 0x77231134, SUC_NE_lexicon_tags_7 },
    { 0xa7ff1133, SUC_NE_lexicon_tags_2 },
    { 0xe48f1136, SUC_NE_lexicon_tags_46 },
    { 0xd9af1134, SUC_NE_lexicon_tags_2 },
    { 0xa3691138, SUC_NE_lexicon_tags_2 },
    { 0x95431138, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xfcdb113b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcb113f, SUC_NE_lexicon_tags_7 },
    { 0x221f1140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145d1145, SUC_NE_lexicon_tags_2 },
    { 0x2b211146, SUC_NE_lexicon_tags_2 },
    { 0x57551147, SUC_NE_lexicon_tags_2 },
    { 0x38ff1147, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe969114a, SUC_NE_lexicon_tags_2 },
    { 0xe7eb114b, SUC_NE_lexicon_tags_1 },
    { 0xc3af114c, SUC_NE_lexicon_tags_2 },
    { 0x4289114d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d731152, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9351155, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38f1158, SUC_NE_lexicon_tags_2 },
    { 0x7e7d1159, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf11115e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0871164, SUC_NE_lexicon_tags_2 },
    { 0x67c11165, SUC_NE_lexicon_tags_2 },
    { 0xe091166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330b116a, SUC_NE_lexicon_tags_2 },
    { 0xebd1116b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43f1170, SUC_NE_lexicon_tags_4 },
    { 0x85d1171, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe251174, SUC_NE_lexicon_tags_2 },
    { 0x7191175, SUC_NE_lexicon_tags_2 },
    { 0x9c891176, SUC_NE_lexicon_tags_2 },
    { 0x34bd1177, SUC_NE_lexicon_tags_121 },
    { 0x488d1176, SUC_NE_lexicon_tags_2 },
    { 0x73eb1174, SUC_NE_lexicon_tags_2 },
    { 0xf7d71179, SUC_NE_lexicon_tags_2 },
    { 0x8dd3117b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x330f117d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xd159117f, SUC_NE_lexicon_tags_2 },
    { 0x35a71180, SUC_NE_lexicon_tags_2 },
    { 0x7be91180, SUC_NE_lexicon_tags_2 },
    { 0xad5f1180, SUC_NE_lexicon_tags_2 },
    { 0x3e771182, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd3d1185, SUC_NE_lexicon_tags_2 },
    { 0xab151185, SUC_NE_lexicon_tags_2 },
    { 0x1df71187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b3118c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2699118e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xded11190, SUC_NE_lexicon_tags_2 },
    { 0xf4151191, SUC_NE_lexicon_tags_0 },
    { 0xe8d91191, SUC_NE_lexicon_tags_2 },
    { 0xfa831193, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ff1196, SUC_NE_lexicon_tags_2 },
    { 0x20a71196, SUC_NE_lexicon_tags_2 },
    { 0x73931198, SUC_NE_lexicon_tags_2 },
    { 0x76591199, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7b9119b, SUC_NE_lexicon_tags_2 },
    { 0xcbdf119c, SUC_NE_lexicon_tags_2 },
    { 0x6c29119d, SUC_NE_lexicon_tags_2 },
    { 0x1755119e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38dd11a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce4d11a2, SUC_NE_lexicon_tags_4 },
    { 0xbb9511a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2311a6, SUC_NE_lexicon_tags_2 },
    { 0x233511a7, SUC_NE_lexicon_tags_2 },
    { 0x411111a8, SUC_NE_lexicon_tags_2 },
    { 0xcd2511a9, SUC_NE_lexicon_tags_2 },
    { 0xc1c911a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x843711ac, SUC_NE_lexicon_tags_2 },
    { 0x19f711ac, SUC_NE_lexicon_tags_2 },
    { 0xa34111ae, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1b11b1, SUC_NE_lexicon_tags_2 },
    { 0x321911b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59df11b6, SUC_NE_lexicon_tags_86 },
    { 0xbcc711b6, SUC_NE_lexicon_tags_2 },
    { 0x7e0d11b8, SUC_NE_lexicon_tags_2 },
    { 0x1c1911b7, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xce6911bb, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5111c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf511c6, SUC_NE_lexicon_tags_2 },
    { 0x58d111c7, SUC_NE_lexicon_tags_2 },
    { 0x66d11c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb711cd, SUC_NE_lexicon_tags_2 },
    { 0x634311ce, SUC_NE_lexicon_tags_2 },
    { 0xd28311cf, SUC_NE_lexicon_tags_2 },
    { 0xcb9d11cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340d11d5, SUC_NE_lexicon_tags_2 },
    { 0x9c1711d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77711d9, SUC_NE_lexicon_tags_2 },
    { 0x159d11da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b511e4, SUC_NE_lexicon_tags_2 },
    { 0x51bd11e5, SUC_NE_lexicon_tags_2 },
    { 0x9c1311e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x131711e9, SUC_NE_lexicon_tags_2 },
    { 0xa2e111ea, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x9d8511ec, SUC_NE_lexicon_tags_2 },
    { 0x583511ed, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9ba311ef, SUC_NE_lexicon_tags_2 },
    { 0xabc111ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x355911f3, SUC_NE_lexicon_tags_2 },
    { 0xa1b911f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa0d11f6, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7e911f8, SUC_NE_lexicon_tags_2 },
    { 0xaf4f11f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x915b11fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a331200, SUC_NE_lexicon_tags_2 },
    { 0x16711201, SUC_NE_lexicon_tags_2 },
    { 0xa6ab1200, SUC_NE_lexicon_tags_2 },
    { 0xd051201, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c71207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe51209, SUC_NE_lexicon_tags_2 },
    { 0x1d77120a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c11213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4191215, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x14891217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e5b1219, SUC_NE_lexicon_tags_2 },
    { 0xab41121a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd157121d, SUC_NE_lexicon_tags_2 },
    { 0x94cb121e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92791223, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9727122a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbe1122c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7215122e, SUC_NE_lexicon_tags_2 },
    { 0xcc17122e, SUC_NE_lexicon_tags_2 },
    { 0x6c2b122f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4b1232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f1235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d11239, SUC_NE_lexicon_tags_62 },
    { 0, NULL },
    { 0x38dd123b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9123e, SUC_NE_lexicon_tags_2 },
    { 0xd3df123f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f11241, SUC_NE_lexicon_tags_2 },
    { 0x2e391241, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xb1cf1244, SUC_NE_lexicon_tags_2 },
    { 0x627f1245, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf511247, SUC_NE_lexicon_tags_2 },
    { 0x8b5d1247, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x761124b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d124f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb84d1251, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb96d1253, SUC_NE_lexicon_tags_2 },
    { 0x6411254, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77291256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a211259, SUC_NE_lexicon_tags_2 },
    { 0x3cf125a, SUC_NE_lexicon_tags_2 },
    { 0xa5a5125a, SUC_NE_lexicon_tags_10 },
    { 0x2f41125a, SUC_NE_lexicon_tags_2 },
    { 0xbd05125c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x32a9125f, SUC_NE_lexicon_tags_2 },
    { 0x9891260, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafa71262, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fbb1264, SUC_NE_lexicon_tags_2 },
    { 0xd8b71264, SUC_NE_lexicon_tags_2 },
    { 0x3f4d1265, SUC_NE_lexicon_tags_2 },
    { 0xb12b1267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a07126c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c071270, SUC_NE_lexicon_tags_2 },
    { 0xc5ed1271, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b031273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed91277, SUC_NE_lexicon_tags_2 },
    { 0x14331278, SUC_NE_lexicon_tags_2 },
    { 0xc0991279, SUC_NE_lexicon_tags_5 },
    { 0xf03f127a, SUC_NE_lexicon_tags_2 },
    { 0xa5eb127b, SUC_NE_lexicon_tags_2 },
    { 0xc5d91279, SUC_NE_lexicon_tags_40 },
    { 0x59e3127d, SUC_NE_lexicon_tags_2 },
    { 0xdeb3127e, SUC_NE_lexicon_tags_2 },
    { 0x16f127f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8071282, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72331285, SUC_NE_lexicon_tags_2 },
    { 0x93831285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6f128b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba11128d, SUC_NE_lexicon_tags_167 },
    { 0xd1c9128d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54f1297, SUC_NE_lexicon_tags_2 },
    { 0x41271297, SUC_NE_lexicon_tags_4 },
    { 0x7cb71299, SUC_NE_lexicon_tags_2 },
    { 0xcf91129a, SUC_NE_lexicon_tags_2 },
    { 0x33691299, SUC_NE_lexicon_tags_0 },
    { 0xc3c9129c, SUC_NE_lexicon_tags_2 },
    { 0xf2151299, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x64e1129f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed12a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23cd12a6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b512a9, SUC_NE_lexicon_tags_2 },
    { 0xc30312aa, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x829112ac, SUC_NE_lexicon_tags_4 },
    { 0xb49712ac, SUC_NE_lexicon_tags_2 },
    { 0x251d12ad, SUC_NE_lexicon_tags_2 },
    { 0xa14112af, SUC_NE_lexicon_tags_2 },
    { 0xbd1312af, SUC_NE_lexicon_tags_2 },
    { 0x63d912b1, SUC_NE_lexicon_tags_0 },
    { 0xfbb712b2, SUC_NE_lexicon_tags_2 },
    { 0x5db312ad, SUC_NE_lexicon_tags_2 },
    { 0x14cd12b1, SUC_NE_lexicon_tags_2 },
    { 0xe312b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c112b8, SUC_NE_lexicon_tags_4 },
    { 0xeb1f12b9, SUC_NE_lexicon_tags_2 },
    { 0x8a5f12b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bb12bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f112cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d12ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x599b12d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7aa912d3, SUC_NE_lexicon_tags_2 },
    { 0x4be712d3, SUC_NE_lexicon_tags_2 },
    { 0x589312d4, SUC_NE_lexicon_tags_4 },
    { 0x22d912d6, SUC_NE_lexicon_tags_24 },
    { 0x954312d4, SUC_NE_lexicon_tags_2 },
    { 0x155312d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc27112da, SUC_NE_lexicon_tags_2 },
    { 0x1f4712da, SUC_NE_lexicon_tags_17 },
    { 0x5e7b12db, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x798112de, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0f12e7, SUC_NE_lexicon_tags_2 },
    { 0x50ad12e7, SUC_NE_lexicon_tags_2 },
    { 0x9e4112e7, SUC_NE_lexicon_tags_2 },
    { 0x60fd12ea, SUC_NE_lexicon_tags_2 },
    { 0xd6d312ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87eb12f0, SUC_NE_lexicon_tags_2 },
    { 0xd3ff12f1, SUC_NE_lexicon_tags_2 },
    { 0x3d3312f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbd12f7, SUC_NE_lexicon_tags_337 },
    { 0, NULL },
    { 0x15112f9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7caf12fc, SUC_NE_lexicon_tags_2 },
    { 0x84ef12fd, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x5b6d12ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6411304, SUC_NE_lexicon_tags_2 },
    { 0xde351304, SUC_NE_lexicon_tags_2 },
    { 0xa5791304, SUC_NE_lexicon_tags_7 },
    { 0xfb3b1307, SUC_NE_lexicon_tags_2 },
    { 0x2c811307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7d1310, SUC_NE_lexicon_tags_2 },
    { 0x6ab91310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc071316, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f51319, SUC_NE_lexicon_tags_2 },
    { 0xaa7d1319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f5131d, SUC_NE_lexicon_tags_2 },
    { 0xe7bb131d, SUC_NE_lexicon_tags_2 },
    { 0x1b75131f, SUC_NE_lexicon_tags_2 },
    { 0xbb31131e, SUC_NE_lexicon_tags_2 },
    { 0x36bd131f, SUC_NE_lexicon_tags_2 },
    { 0xc08b131e, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xd8971324, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad1328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41d132f, SUC_NE_lexicon_tags_2 },
    { 0xa0c9132f, SUC_NE_lexicon_tags_2 },
    { 0xa7871330, SUC_NE_lexicon_tags_1 },
    { 0x8d8f1330, SUC_NE_lexicon_tags_2 },
    { 0x8d7f1333, SUC_NE_lexicon_tags_3 },
    { 0xb0411330, SUC_NE_lexicon_tags_2 },
    { 0xe3e51335, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4f1338, SUC_NE_lexicon_tags_2 },
    { 0x62e71338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ff133f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1011341, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d11345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc255134d, SUC_NE_lexicon_tags_2 },
    { 0x91b3134e, SUC_NE_lexicon_tags_2 },
    { 0x446d134e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ed11351, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd22f1353, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93b31355, SUC_NE_lexicon_tags_2 },
    { 0x6b231355, SUC_NE_lexicon_tags_2 },
    { 0x276b1357, SUC_NE_lexicon_tags_2 },
    { 0xbd8b1358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f135a, SUC_NE_lexicon_tags_34 },
    { 0x1a67135b, SUC_NE_lexicon_tags_2 },
    { 0x7413135c, SUC_NE_lexicon_tags_2 },
    { 0xcb49135c, SUC_NE_lexicon_tags_2 },
    { 0xd3b5135e, SUC_NE_lexicon_tags_2 },
    { 0x8517135f, SUC_NE_lexicon_tags_2 },
    { 0x144b1360, SUC_NE_lexicon_tags_2 },
    { 0x16271361, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x428b1363, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa2fd1365, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1cd1367, SUC_NE_lexicon_tags_2 },
    { 0x39111368, SUC_NE_lexicon_tags_2 },
    { 0x7a791368, SUC_NE_lexicon_tags_2 },
    { 0xdb13136a, SUC_NE_lexicon_tags_1 },
    { 0x87331368, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88b1374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e871378, SUC_NE_lexicon_tags_2 },
    { 0x4c111378, SUC_NE_lexicon_tags_2 },
    { 0x61051378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7d137f, SUC_NE_lexicon_tags_2 },
    { 0x86c3137f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa511383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d31138c, SUC_NE_lexicon_tags_2 },
    { 0x5c37138c, SUC_NE_lexicon_tags_2 },
    { 0xf511138e, SUC_NE_lexicon_tags_2 },
    { 0xaf81138c, SUC_NE_lexicon_tags_2 },
    { 0xe0f9138c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b71396, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ff139b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23f139e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34cb13a0, SUC_NE_lexicon_tags_2 },
    { 0xb0ab13a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b6913a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3513a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484713ac, SUC_NE_lexicon_tags_2 },
    { 0xc26713ac, SUC_NE_lexicon_tags_2 },
    { 0x748713ae, SUC_NE_lexicon_tags_13 },
    { 0xf2b713ae, SUC_NE_lexicon_tags_2 },
    { 0xfaef13ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4913b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf70f13b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d13bb, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504d13bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f0313c1, SUC_NE_lexicon_tags_2 },
    { 0xd91313c1, SUC_NE_lexicon_tags_2 },
    { 0xd96b13c3, SUC_NE_lexicon_tags_2 },
    { 0x523113c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78513ca, SUC_NE_lexicon_tags_32 },
    { 0xe36713cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4e913cd, SUC_NE_lexicon_tags_2 },
    { 0x2c1b13ce, SUC_NE_lexicon_tags_2 },
    { 0xff9713cf, SUC_NE_lexicon_tags_2 },
    { 0x9d3b13ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe13513d2, SUC_NE_lexicon_tags_8 },
    { 0x777113d3, SUC_NE_lexicon_tags_2 },
    { 0x262113d2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9b13d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad313dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9b13e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ce913e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x606f13e6, SUC_NE_lexicon_tags_2 },
    { 0x274713e7, SUC_NE_lexicon_tags_2 },
    { 0x5e2d13e8, SUC_NE_lexicon_tags_2 },
    { 0x4f8b13e6, SUC_NE_lexicon_tags_3 },
    { 0xe07d13ea, SUC_NE_lexicon_tags_3 },
    { 0x593313e7, SUC_NE_lexicon_tags_2 },
    { 0xf1df13ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d713f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb13f7, SUC_NE_lexicon_tags_2 },
    { 0xa9313f8, SUC_NE_lexicon_tags_2 },
    { 0x2bbb13f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79713fb, SUC_NE_lexicon_tags_2 },
    { 0x21ff13fc, SUC_NE_lexicon_tags_4 },
    { 0xcc6d13fd, SUC_NE_lexicon_tags_2 },
    { 0xc7db13fc, SUC_NE_lexicon_tags_2 },
    { 0x994d13ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24b51401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754d1405, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe271408, SUC_NE_lexicon_tags_2 },
    { 0x69031409, SUC_NE_lexicon_tags_2 },
    { 0xc0bf140a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf089140d, SUC_NE_lexicon_tags_227 },
    { 0x7bdb140e, SUC_NE_lexicon_tags_2 },
    { 0x631d140e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44f1415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2269141a, SUC_NE_lexicon_tags_36 },
    { 0xe59141b, SUC_NE_lexicon_tags_2 },
    { 0x1ac1141a, SUC_NE_lexicon_tags_2 },
    { 0xe683141c, SUC_NE_lexicon_tags_4 },
    { 0xed95141e, SUC_NE_lexicon_tags_2 },
    { 0x949b141e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6971421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1b1425, SUC_NE_lexicon_tags_2 },
    { 0xa5871425, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d791428, SUC_NE_lexicon_tags_2 },
    { 0x9d311429, SUC_NE_lexicon_tags_2 },
    { 0xf74d142a, SUC_NE_lexicon_tags_2 },
    { 0xfa4f142a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x203f142e, SUC_NE_lexicon_tags_2 },
    { 0xc90f142f, SUC_NE_lexicon_tags_2 },
    { 0xb26b142e, SUC_NE_lexicon_tags_2 },
    { 0x555142f, SUC_NE_lexicon_tags_2 },
    { 0x90931432, SUC_NE_lexicon_tags_2 },
    { 0x1d271433, SUC_NE_lexicon_tags_2 },
    { 0x4d5b1432, SUC_NE_lexicon_tags_2 },
    { 0x7f0f1432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb351437, SUC_NE_lexicon_tags_2 },
    { 0xdeed1438, SUC_NE_lexicon_tags_2 },
    { 0x7bcf1438, SUC_NE_lexicon_tags_3 },
    { 0x407f143a, SUC_NE_lexicon_tags_2 },
    { 0xf6fb1439, SUC_NE_lexicon_tags_2 },
    { 0xf40f1438, SUC_NE_lexicon_tags_2 },
    { 0xcac7143d, SUC_NE_lexicon_tags_2 },
    { 0x396d143d, SUC_NE_lexicon_tags_2 },
    { 0x33bd143e, SUC_NE_lexicon_tags_2 },
    { 0x4b7f1440, SUC_NE_lexicon_tags_2 },
    { 0xf8411441, SUC_NE_lexicon_tags_2 },
    { 0x8133143e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b91445, SUC_NE_lexicon_tags_2 },
    { 0x32eb1446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda71448, SUC_NE_lexicon_tags_2 },
    { 0x90b71448, SUC_NE_lexicon_tags_2 },
    { 0x1809144a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31851450, SUC_NE_lexicon_tags_2 },
    { 0xba6d1451, SUC_NE_lexicon_tags_55 },
    { 0, NULL },
    { 0xc5931453, SUC_NE_lexicon_tags_2 },
    { 0x9a2b1454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaea91456, SUC_NE_lexicon_tags_2 },
    { 0xb9f71457, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x55a31459, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd739145c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bb1145e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b851462, SUC_NE_lexicon_tags_2 },
    { 0x74df1463, SUC_NE_lexicon_tags_7 },
    { 0x172d1464, SUC_NE_lexicon_tags_13 },
    { 0xe4cb1463, SUC_NE_lexicon_tags_7 },
    { 0xbf211466, SUC_NE_lexicon_tags_4 },
    { 0x4ce31464, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d146a, SUC_NE_lexicon_tags_2 },
    { 0xa3b5146a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43b7146d, SUC_NE_lexicon_tags_13 },
    { 0xf003146e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c51475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d691477, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b147a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9147d, SUC_NE_lexicon_tags_2 },
    { 0x9831147e, SUC_NE_lexicon_tags_2 },
    { 0x3261147e, SUC_NE_lexicon_tags_3 },
    { 0x24d3147e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eeb1482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a11488, SUC_NE_lexicon_tags_5 },
    { 0xd7891488, SUC_NE_lexicon_tags_2 },
    { 0xcd3b1489, SUC_NE_lexicon_tags_381 },
    { 0x8aa7148b, SUC_NE_lexicon_tags_2 },
    { 0x5c25148c, SUC_NE_lexicon_tags_40 },
    { 0x4657148c, SUC_NE_lexicon_tags_2 },
    { 0x273f148c, SUC_NE_lexicon_tags_2 },
    { 0xa64f148a, SUC_NE_lexicon_tags_2 },
    { 0x65f71490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984b1497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec7149a, SUC_NE_lexicon_tags_2 },
    { 0xfc9149b, SUC_NE_lexicon_tags_19 },
    { 0x729f149c, SUC_NE_lexicon_tags_0 },
    { 0x4ecd149d, SUC_NE_lexicon_tags_2 },
    { 0x2b2f149d, SUC_NE_lexicon_tags_12 },
    { 0xf3dd149d, SUC_NE_lexicon_tags_2 },
    { 0x7b3114a0, SUC_NE_lexicon_tags_7 },
    { 0xd72b149c, SUC_NE_lexicon_tags_2 },
    { 0xf91b14a2, SUC_NE_lexicon_tags_2 },
    { 0x7b7d14a3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcceb14a6, SUC_NE_lexicon_tags_4 },
    { 0xdf5514a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81914aa, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7114ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x830914af, SUC_NE_lexicon_tags_2 },
    { 0x324d14b0, SUC_NE_lexicon_tags_46 },
    { 0xaa0914af, SUC_NE_lexicon_tags_3 },
    { 0x4abd14b0, SUC_NE_lexicon_tags_2 },
    { 0xb09114b3, SUC_NE_lexicon_tags_1 },
    { 0x3e3b14b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd37b14b6, SUC_NE_lexicon_tags_2 },
    { 0xc2d714b7, SUC_NE_lexicon_tags_2 },
    { 0xef6d14b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a114bc, SUC_NE_lexicon_tags_2 },
    { 0x8ce114bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6b14c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd714c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771714ca, SUC_NE_lexicon_tags_2 },
    { 0x979514ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x509114ce, SUC_NE_lexicon_tags_87 },
    { 0, NULL },
    { 0xde1b14d0, SUC_NE_lexicon_tags_2 },
    { 0x826714d1, SUC_NE_lexicon_tags_2 },
    { 0x4dfd14d2, SUC_NE_lexicon_tags_2 },
    { 0x35f114d2, SUC_NE_lexicon_tags_2 },
    { 0x966114d2, SUC_NE_lexicon_tags_2 },
    { 0xcd4314d5, SUC_NE_lexicon_tags_2 },
    { 0xa89b14d5, SUC_NE_lexicon_tags_4 },
    { 0x746914d5, SUC_NE_lexicon_tags_2 },
    { 0x743b14d0, SUC_NE_lexicon_tags_2 },
    { 0x498314d4, SUC_NE_lexicon_tags_24 },
    { 0xd3f314d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x283d14dd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe914e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b6d14e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bef14e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285d14e8, SUC_NE_lexicon_tags_2 },
    { 0x9e2714e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb1514eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x777d14ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0314f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x276114f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc01d14f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4d14fd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb051501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d1504, SUC_NE_lexicon_tags_2 },
    { 0x667d1505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b71508, SUC_NE_lexicon_tags_2 },
    { 0x8d111509, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd150c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f150f, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xfbdf1511, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0891515, SUC_NE_lexicon_tags_2 },
    { 0xda6b1516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf371518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2d151f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f751523, SUC_NE_lexicon_tags_2 },
    { 0x36091524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4f1527, SUC_NE_lexicon_tags_2 },
    { 0x84e71528, SUC_NE_lexicon_tags_2 },
    { 0xa4e11529, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1ef152b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c9152e, SUC_NE_lexicon_tags_2 },
    { 0x6ebd152e, SUC_NE_lexicon_tags_2 },
    { 0x68231530, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb291533, SUC_NE_lexicon_tags_2 },
    { 0xa691534, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b79153a, SUC_NE_lexicon_tags_7 },
    { 0xc631153b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f153e, SUC_NE_lexicon_tags_2 },
    { 0x32f9153f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65d1544, SUC_NE_lexicon_tags_2 },
    { 0xdd7d1544, SUC_NE_lexicon_tags_2 },
    { 0xcb1f1546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe61154c, SUC_NE_lexicon_tags_2 },
    { 0xecb5154d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8597154f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411f1557, SUC_NE_lexicon_tags_2 },
    { 0xe091558, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c83155b, SUC_NE_lexicon_tags_2 },
    { 0x6e57155b, SUC_NE_lexicon_tags_2 },
    { 0x70e9155d, SUC_NE_lexicon_tags_2 },
    { 0xd1af155e, SUC_NE_lexicon_tags_2 },
    { 0x71c9155f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab451563, SUC_NE_lexicon_tags_2 },
    { 0xb1131564, SUC_NE_lexicon_tags_2 },
    { 0x18cd1563, SUC_NE_lexicon_tags_2 },
    { 0x18891566, SUC_NE_lexicon_tags_2 },
    { 0xc10d1567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53156a, SUC_NE_lexicon_tags_2 },
    { 0xb50b156b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f1571, SUC_NE_lexicon_tags_2 },
    { 0xbc811572, SUC_NE_lexicon_tags_2 },
    { 0x83151573, SUC_NE_lexicon_tags_2 },
    { 0x67111574, SUC_NE_lexicon_tags_3 },
    { 0x38b11575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f51577, SUC_NE_lexicon_tags_2 },
    { 0x629f1578, SUC_NE_lexicon_tags_2 },
    { 0xa7911579, SUC_NE_lexicon_tags_2 },
    { 0x460f157a, SUC_NE_lexicon_tags_2 },
    { 0xc081157b, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef7157e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6c2b1580, SUC_NE_lexicon_tags_2 },
    { 0x7d331581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3f1587, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0x84811589, SUC_NE_lexicon_tags_2 },
    { 0x6523158a, SUC_NE_lexicon_tags_2 },
    { 0x6b59158b, SUC_NE_lexicon_tags_282 },
    { 0x8da1158a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba791598, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0xeff7159a, SUC_NE_lexicon_tags_2 },
    { 0x9a7b159b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338b159f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd08915a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f915a4, SUC_NE_lexicon_tags_2 },
    { 0xf45315a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd79915a7, SUC_NE_lexicon_tags_2 },
    { 0x980f15a8, SUC_NE_lexicon_tags_2 },
    { 0x6be715a8, SUC_NE_lexicon_tags_2 },
    { 0xceb315aa, SUC_NE_lexicon_tags_2 },
    { 0xd18f15a7, SUC_NE_lexicon_tags_13 },
    { 0x2ee915a8, SUC_NE_lexicon_tags_2 },
    { 0x1b1315ab, SUC_NE_lexicon_tags_4 },
    { 0xceed15ae, SUC_NE_lexicon_tags_2 },
    { 0xae5515af, SUC_NE_lexicon_tags_1 },
    { 0x8fe515b0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40715b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5df15ba, SUC_NE_lexicon_tags_2 },
    { 0x71f715bb, SUC_NE_lexicon_tags_2 },
    { 0x390b15bc, SUC_NE_lexicon_tags_7 },
    { 0x544f15bc, SUC_NE_lexicon_tags_2 },
    { 0x5d6f15ba, SUC_NE_lexicon_tags_2 },
    { 0x900915bf, SUC_NE_lexicon_tags_2 },
    { 0xd54515bd, SUC_NE_lexicon_tags_2 },
    { 0xf56d15be, SUC_NE_lexicon_tags_2 },
    { 0x604915bc, SUC_NE_lexicon_tags_1 },
    { 0x10b715c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622315c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58515cb, SUC_NE_lexicon_tags_2 },
    { 0xebdd15cc, SUC_NE_lexicon_tags_0 },
    { 0x962b15cd, SUC_NE_lexicon_tags_2 },
    { 0x200315ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c515d0, SUC_NE_lexicon_tags_2 },
    { 0x20c715d1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7b0715d3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc315d7, SUC_NE_lexicon_tags_2 },
    { 0xebf15d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x202b15da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb90b15dc, SUC_NE_lexicon_tags_2 },
    { 0x970d15dd, SUC_NE_lexicon_tags_2 },
    { 0xdbd115dd, SUC_NE_lexicon_tags_2 },
    { 0x756b15df, SUC_NE_lexicon_tags_2 },
    { 0x11515dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09f15e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x492915ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254b15ee, SUC_NE_lexicon_tags_2 },
    { 0x228b15ef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510315f5, SUC_NE_lexicon_tags_2 },
    { 0xf3915f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efb15fb, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3d15fe, SUC_NE_lexicon_tags_17 },
    { 0xe78115ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cdb1601, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9151606, SUC_NE_lexicon_tags_4 },
    { 0x729f1607, SUC_NE_lexicon_tags_2 },
    { 0xbd931608, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f75160f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fb1613, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xb72b1615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ac51617, SUC_NE_lexicon_tags_19 },
    { 0x61011618, SUC_NE_lexicon_tags_2 },
    { 0x5f5f1618, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb415161b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355d161f, SUC_NE_lexicon_tags_2 },
    { 0xcd8b1620, SUC_NE_lexicon_tags_2 },
    { 0x746b1621, SUC_NE_lexicon_tags_2 },
    { 0xc8931622, SUC_NE_lexicon_tags_2 },
    { 0x17551623, SUC_NE_lexicon_tags_3 },
    { 0xfb1f1620, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x12ab1626, SUC_NE_lexicon_tags_2 },
    { 0x7c791627, SUC_NE_lexicon_tags_2 },
    { 0xa7611628, SUC_NE_lexicon_tags_2 },
    { 0x5a9f1626, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1162d, SUC_NE_lexicon_tags_2 },
    { 0x578d162e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfea91630, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531635, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab691638, SUC_NE_lexicon_tags_4 },
    { 0xd33d1638, SUC_NE_lexicon_tags_2 },
    { 0xc545163a, SUC_NE_lexicon_tags_2 },
    { 0xf3bf1638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b31642, SUC_NE_lexicon_tags_2 },
    { 0x12ed1642, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x95cf1645, SUC_NE_lexicon_tags_36 },
    { 0x62e51645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a41164b, SUC_NE_lexicon_tags_2 },
    { 0x4dbd164c, SUC_NE_lexicon_tags_2 },
    { 0xb769164c, SUC_NE_lexicon_tags_2 },
    { 0x97fb164e, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0751652, SUC_NE_lexicon_tags_4 },
    { 0x6a191653, SUC_NE_lexicon_tags_34 },
    { 0x788f1654, SUC_NE_lexicon_tags_5 },
    { 0x31c51655, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1791165d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f1661, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff91664, SUC_NE_lexicon_tags_2 },
    { 0xe5f31665, SUC_NE_lexicon_tags_2 },
    { 0x9b251666, SUC_NE_lexicon_tags_2 },
    { 0xc2731667, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea591669, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a9166d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3237166f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf91671, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdc91673, SUC_NE_lexicon_tags_2 },
    { 0x50f31674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d21167a, SUC_NE_lexicon_tags_2 },
    { 0x9235167b, SUC_NE_lexicon_tags_2 },
    { 0x153f167a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ff1682, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999b1687, SUC_NE_lexicon_tags_2 },
    { 0xa7e91688, SUC_NE_lexicon_tags_2 },
    { 0x1fb91687, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe581168b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df5168f, SUC_NE_lexicon_tags_2 },
    { 0xf571690, SUC_NE_lexicon_tags_7 },
    { 0xbe51690, SUC_NE_lexicon_tags_2 },
    { 0x6c5b1692, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcd31694, SUC_NE_lexicon_tags_2 },
    { 0xb6ed1694, SUC_NE_lexicon_tags_2 },
    { 0x2a3d1696, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1551698, SUC_NE_lexicon_tags_2 },
    { 0x28051699, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71f169c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x413d169e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f5916a0, SUC_NE_lexicon_tags_2 },
    { 0xda7916a1, SUC_NE_lexicon_tags_2 },
    { 0xc49d16a0, SUC_NE_lexicon_tags_2 },
    { 0xce9916a3, SUC_NE_lexicon_tags_2 },
    { 0xf8a316a3, SUC_NE_lexicon_tags_2 },
    { 0x265316a3, SUC_NE_lexicon_tags_2 },
    { 0x882d16a6, SUC_NE_lexicon_tags_2 },
    { 0x6a2716a6, SUC_NE_lexicon_tags_2 },
    { 0xc10116a6, SUC_NE_lexicon_tags_2 },
    { 0x393116a0, SUC_NE_lexicon_tags_13 },
    { 0xd12916aa, SUC_NE_lexicon_tags_2 },
    { 0x2b8116ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdd16ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x355716b0, SUC_NE_lexicon_tags_2 },
    { 0x15316b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcf16b4, SUC_NE_lexicon_tags_2 },
    { 0x390716b4, SUC_NE_lexicon_tags_19 },
    { 0x8dd116b6, SUC_NE_lexicon_tags_2 },
    { 0xee1b16b7, SUC_NE_lexicon_tags_0 },
    { 0xf34116b4, SUC_NE_lexicon_tags_2 },
    { 0x28d316b5, SUC_NE_lexicon_tags_2 },
    { 0x8b6716ba, SUC_NE_lexicon_tags_30 },
    { 0xf4df16ba, SUC_NE_lexicon_tags_2 },
    { 0x46bb16ba, SUC_NE_lexicon_tags_2 },
    { 0x15cd16b8, SUC_NE_lexicon_tags_2 },
    { 0x285916be, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x734516c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97b916c2, SUC_NE_lexicon_tags_2 },
    { 0xe59916c3, SUC_NE_lexicon_tags_2 },
    { 0x67a916c4, SUC_NE_lexicon_tags_2 },
    { 0x6a2316c5, SUC_NE_lexicon_tags_2 },
    { 0xd3b316c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb16cf, SUC_NE_lexicon_tags_2 },
    { 0xd7df16cf, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00916d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904916d7, SUC_NE_lexicon_tags_24 },
    { 0x6f8716d8, SUC_NE_lexicon_tags_2 },
    { 0x503f16d7, SUC_NE_lexicon_tags_2 },
    { 0xac9b16da, SUC_NE_lexicon_tags_2 },
    { 0x2cb916da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7516de, SUC_NE_lexicon_tags_2 },
    { 0xbbe316df, SUC_NE_lexicon_tags_2 },
    { 0x354716df, SUC_NE_lexicon_tags_2 },
    { 0x74b16de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x846916e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb93516e6, SUC_NE_lexicon_tags_2 },
    { 0x515316e6, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x763b16e9, SUC_NE_lexicon_tags_2 },
    { 0xa36916ea, SUC_NE_lexicon_tags_2 },
    { 0xbed316eb, SUC_NE_lexicon_tags_2 },
    { 0x432f16e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c316ef, SUC_NE_lexicon_tags_2 },
    { 0x98db16ef, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe516f3, SUC_NE_lexicon_tags_2 },
    { 0xedd316f4, SUC_NE_lexicon_tags_2 },
    { 0xbe5716f3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70316f9, SUC_NE_lexicon_tags_2 },
    { 0x565516f9, SUC_NE_lexicon_tags_2 },
    { 0xcc1d16fb, SUC_NE_lexicon_tags_2 },
    { 0xec1f16fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd316ff, SUC_NE_lexicon_tags_2 },
    { 0xe6116ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1851703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba11706, SUC_NE_lexicon_tags_2 },
    { 0xd73f1707, SUC_NE_lexicon_tags_2 },
    { 0x25bd1707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e15170a, SUC_NE_lexicon_tags_13 },
    { 0x4bd3170a, SUC_NE_lexicon_tags_2 },
    { 0x1955170c, SUC_NE_lexicon_tags_2 },
    { 0x2781170d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b31711, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cb1714, SUC_NE_lexicon_tags_2 },
    { 0x881b1715, SUC_NE_lexicon_tags_2 },
    { 0x300f1716, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a371718, SUC_NE_lexicon_tags_2 },
    { 0xa6791719, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc6f171b, SUC_NE_lexicon_tags_2 },
    { 0x708d171c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8d1720, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199b1726, SUC_NE_lexicon_tags_2 },
    { 0x77d11727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173f172d, SUC_NE_lexicon_tags_2 },
    { 0xd919172d, SUC_NE_lexicon_tags_2 },
    { 0x7607172e, SUC_NE_lexicon_tags_2 },
    { 0xd1711730, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22f71732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2f1736, SUC_NE_lexicon_tags_2 },
    { 0xa99d1736, SUC_NE_lexicon_tags_2 },
    { 0x52231737, SUC_NE_lexicon_tags_151 },
    { 0, NULL },
    { 0xa069173a, SUC_NE_lexicon_tags_2 },
    { 0xdfaf173a, SUC_NE_lexicon_tags_2 },
    { 0x994d173a, SUC_NE_lexicon_tags_2 },
    { 0x2a81173d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761b1742, SUC_NE_lexicon_tags_2 },
    { 0x76031743, SUC_NE_lexicon_tags_2 },
    { 0xf3ed1744, SUC_NE_lexicon_tags_3 },
    { 0xa3e11744, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2f1749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2975174b, SUC_NE_lexicon_tags_2 },
    { 0x5629174b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a771755, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e9175d, SUC_NE_lexicon_tags_2 },
    { 0xfb7d175e, SUC_NE_lexicon_tags_2 },
    { 0xc3f3175f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186d1763, SUC_NE_lexicon_tags_2 },
    { 0xd7ad1763, SUC_NE_lexicon_tags_2 },
    { 0x2c511765, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8e91767, SUC_NE_lexicon_tags_2 },
    { 0x59b11767, SUC_NE_lexicon_tags_2 },
    { 0x56971767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5749176c, SUC_NE_lexicon_tags_2 },
    { 0xfae7176d, SUC_NE_lexicon_tags_2 },
    { 0x42c9176e, SUC_NE_lexicon_tags_2 },
    { 0xd74f176f, SUC_NE_lexicon_tags_2 },
    { 0xd7e3176f, SUC_NE_lexicon_tags_2 },
    { 0xed33176d, SUC_NE_lexicon_tags_2 },
    { 0x9991176c, SUC_NE_lexicon_tags_2 },
    { 0xb46f176d, SUC_NE_lexicon_tags_2 },
    { 0xf8c31774, SUC_NE_lexicon_tags_2 },
    { 0x9b6b1774, SUC_NE_lexicon_tags_2 },
    { 0x77651776, SUC_NE_lexicon_tags_2 },
    { 0x328d1777, SUC_NE_lexicon_tags_19 },
    { 0x341b1775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd631177a, SUC_NE_lexicon_tags_2 },
    { 0xf38b177a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc517177d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e25177f, SUC_NE_lexicon_tags_2 },
    { 0x244f177f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x904d1783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d11786, SUC_NE_lexicon_tags_2 },
    { 0xd47b1786, SUC_NE_lexicon_tags_2 },
    { 0x14411787, SUC_NE_lexicon_tags_2 },
    { 0x6b611789, SUC_NE_lexicon_tags_28 },
    { 0xfbb178a, SUC_NE_lexicon_tags_2 },
    { 0xdceb178b, SUC_NE_lexicon_tags_2 },
    { 0x3a7178c, SUC_NE_lexicon_tags_2 },
    { 0x44bd1787, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xfd13178f, SUC_NE_lexicon_tags_2 },
    { 0x2c7b178f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f1793, SUC_NE_lexicon_tags_3 },
    { 0xfbdb1793, SUC_NE_lexicon_tags_3 },
    { 0xe6471793, SUC_NE_lexicon_tags_2 },
    { 0xa2b1794, SUC_NE_lexicon_tags_2 },
    { 0x98171797, SUC_NE_lexicon_tags_2 },
    { 0xc56f1798, SUC_NE_lexicon_tags_2 },
    { 0xb3f31798, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8ed179b, SUC_NE_lexicon_tags_2 },
    { 0xdb0f179c, SUC_NE_lexicon_tags_2 },
    { 0xe87179c, SUC_NE_lexicon_tags_2 },
    { 0xfb17179e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ed917a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4b17a3, SUC_NE_lexicon_tags_7 },
    { 0xe1717a3, SUC_NE_lexicon_tags_3 },
    { 0x918117a3, SUC_NE_lexicon_tags_2 },
    { 0xa9c717a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14f17a8, SUC_NE_lexicon_tags_2 },
    { 0xe3af17a9, SUC_NE_lexicon_tags_4 },
    { 0x84e917aa, SUC_NE_lexicon_tags_2 },
    { 0xc82517ab, SUC_NE_lexicon_tags_3 },
    { 0x824117ac, SUC_NE_lexicon_tags_2 },
    { 0x114517ac, SUC_NE_lexicon_tags_2 },
    { 0xb3b517ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5517b2, SUC_NE_lexicon_tags_2 },
    { 0x613717b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12317bd, SUC_NE_lexicon_tags_18 },
    { 0x7ec517bd, SUC_NE_lexicon_tags_2 },
    { 0x4ad517bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fd17c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x143917c7, SUC_NE_lexicon_tags_2 },
    { 0x286117c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a3517ca, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648517cf, SUC_NE_lexicon_tags_2 },
    { 0x165117d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401317d7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8bdf17d9, SUC_NE_lexicon_tags_2 },
    { 0x6b5717da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x827317dc, SUC_NE_lexicon_tags_2 },
    { 0xdbb517dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf917e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9b17e4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09317e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361317ed, SUC_NE_lexicon_tags_2 },
    { 0xd76317ed, SUC_NE_lexicon_tags_2 },
    { 0x502d17ee, SUC_NE_lexicon_tags_2 },
    { 0x472917ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b317f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb17fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9f17fd, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xa75717ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20791802, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f71809, SUC_NE_lexicon_tags_19 },
    { 0x2deb1809, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe55180c, SUC_NE_lexicon_tags_2 },
    { 0xe32d180c, SUC_NE_lexicon_tags_2 },
    { 0x8435180e, SUC_NE_lexicon_tags_2 },
    { 0x2e77180f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670d1813, SUC_NE_lexicon_tags_3 },
    { 0x207d1814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503f1819, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe947181d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0951821, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xad71823, SUC_NE_lexicon_tags_0 },
    { 0xf2131823, SUC_NE_lexicon_tags_1 },
    { 0x54dd1823, SUC_NE_lexicon_tags_2 },
    { 0xb6411826, SUC_NE_lexicon_tags_2 },
    { 0x68f11827, SUC_NE_lexicon_tags_2 },
    { 0x31531827, SUC_NE_lexicon_tags_2 },
    { 0x9e2d1829, SUC_NE_lexicon_tags_2 },
    { 0x31c3182a, SUC_NE_lexicon_tags_0 },
    { 0xcb31182b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb37182e, SUC_NE_lexicon_tags_2 },
    { 0x6da5182f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99fd1831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b831834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa50b1836, SUC_NE_lexicon_tags_2 },
    { 0x25071837, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dd183a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b1840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x747f1843, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28ad1845, SUC_NE_lexicon_tags_2 },
    { 0x1de91845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3631848, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b95184e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71854, SUC_NE_lexicon_tags_2 },
    { 0xfe411855, SUC_NE_lexicon_tags_2 },
    { 0xf58b1854, SUC_NE_lexicon_tags_2 },
    { 0x7931857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ab71859, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca75185d, SUC_NE_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc91860, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff71868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x117f186a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x218f186c, SUC_NE_lexicon_tags_2 },
    { 0xd9b7186c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb71873, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9b1876, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6071878, SUC_NE_lexicon_tags_24 },
    { 0x706b1878, SUC_NE_lexicon_tags_2 },
    { 0xc13d187a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97f187d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9951880, SUC_NE_lexicon_tags_24 },
    { 0x5a31881, SUC_NE_lexicon_tags_2 },
    { 0x80051881, SUC_NE_lexicon_tags_2 },
    { 0xc92f1883, SUC_NE_lexicon_tags_2 },
    { 0xfc211880, SUC_NE_lexicon_tags_2 },
    { 0xaa6b1884, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72af1887, SUC_NE_lexicon_tags_2 },
    { 0xa5d71887, SUC_NE_lexicon_tags_2 },
    { 0x88cb1889, SUC_NE_lexicon_tags_2 },
    { 0x2b59188a, SUC_NE_lexicon_tags_2 },
    { 0x5097188a, SUC_NE_lexicon_tags_2 },
    { 0x2107188c, SUC_NE_lexicon_tags_36 },
    { 0x7d031887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544b1892, SUC_NE_lexicon_tags_2 },
    { 0xc07f1893, SUC_NE_lexicon_tags_2 },
    { 0x7fe51894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c3189a, SUC_NE_lexicon_tags_2 },
    { 0xd873189a, SUC_NE_lexicon_tags_2 },
    { 0x5db3189c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8453189e, SUC_NE_lexicon_tags_36 },
    { 0x4735189e, SUC_NE_lexicon_tags_2 },
    { 0xb3ef189f, SUC_NE_lexicon_tags_2 },
    { 0x768b18a1, SUC_NE_lexicon_tags_167 },
    { 0x183518a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e8118a4, SUC_NE_lexicon_tags_2 },
    { 0x350b18a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x727d18a7, SUC_NE_lexicon_tags_2 },
    { 0x240918a8, SUC_NE_lexicon_tags_2 },
    { 0xdf9518a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02718ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd57718ae, SUC_NE_lexicon_tags_2 },
    { 0xe0e318af, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xcf7318b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94918b4, SUC_NE_lexicon_tags_2 },
    { 0xb5518b5, SUC_NE_lexicon_tags_2 },
    { 0xd62b18b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd18bb, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84518be, SUC_NE_lexicon_tags_34 },
    { 0xb2a918bf, SUC_NE_lexicon_tags_3 },
    { 0x605f18be, SUC_NE_lexicon_tags_2 },
    { 0xb17718bf, SUC_NE_lexicon_tags_2 },
    { 0x583718c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2918c7, SUC_NE_lexicon_tags_4 },
    { 0x588d18c8, SUC_NE_lexicon_tags_2 },
    { 0x78ed18c8, SUC_NE_lexicon_tags_2 },
    { 0x9fb518ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31718ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5f18d1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdf18d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c118d8, SUC_NE_lexicon_tags_2 },
    { 0xe90118d9, SUC_NE_lexicon_tags_2 },
    { 0xf60d18da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4d18e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557918e5, SUC_NE_lexicon_tags_19 },
    { 0x728518e6, SUC_NE_lexicon_tags_2 },
    { 0x99c518e5, SUC_NE_lexicon_tags_2 },
    { 0x48bf18e8, SUC_NE_lexicon_tags_2 },
    { 0x1b0518e8, SUC_NE_lexicon_tags_2 },
    { 0x71af18ea, SUC_NE_lexicon_tags_2 },
    { 0xd9f118e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0718f0, SUC_NE_lexicon_tags_2 },
    { 0x97318f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885718f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x836718fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfddd18fc, SUC_NE_lexicon_tags_2 },
    { 0xc7fb18fd, SUC_NE_lexicon_tags_2 },
    { 0x2ec518fe, SUC_NE_lexicon_tags_2 },
    { 0x99cb18fe, SUC_NE_lexicon_tags_2 },
    { 0x44af18fc, SUC_NE_lexicon_tags_2 },
    { 0xeef518ff, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0611904, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4551906, SUC_NE_lexicon_tags_2 },
    { 0xe5e71906, SUC_NE_lexicon_tags_2 },
    { 0x688b1906, SUC_NE_lexicon_tags_2 },
    { 0xf0091906, SUC_NE_lexicon_tags_2 },
    { 0x810d190a, SUC_NE_lexicon_tags_2 },
    { 0x33f3190a, SUC_NE_lexicon_tags_2 },
    { 0xdd9b190c, SUC_NE_lexicon_tags_0 },
    { 0x33e7190a, SUC_NE_lexicon_tags_4 },
    { 0xd165190b, SUC_NE_lexicon_tags_2 },
    { 0x7e3f190f, SUC_NE_lexicon_tags_7 },
    { 0xcc951910, SUC_NE_lexicon_tags_2 },
    { 0xedb51910, SUC_NE_lexicon_tags_2 },
    { 0xb0311912, SUC_NE_lexicon_tags_2 },
    { 0x13611911, SUC_NE_lexicon_tags_2 },
    { 0x81351914, SUC_NE_lexicon_tags_2 },
    { 0x26091914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bff1919, SUC_NE_lexicon_tags_2 },
    { 0x6eeb191a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8edb191c, SUC_NE_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c51922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc891924, SUC_NE_lexicon_tags_2 },
    { 0x20731925, SUC_NE_lexicon_tags_4 },
    { 0xa8731925, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46971929, SUC_NE_lexicon_tags_2 },
    { 0x75192a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f11931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79db1934, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04f1939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f5193d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x18c1193f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x51591941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c11944, SUC_NE_lexicon_tags_0 },
    { 0xeac91945, SUC_NE_lexicon_tags_2 },
    { 0xeb8b1946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c3194d, SUC_NE_lexicon_tags_77 },
    { 0, NULL },
    { 0x5c2b194f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6283195c, SUC_NE_lexicon_tags_2 },
    { 0xa909195d, SUC_NE_lexicon_tags_2 },
    { 0xb53f195e, SUC_NE_lexicon_tags_2 },
    { 0xf09f195c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a11961, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7b91963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2791967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbb11969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb115196c, SUC_NE_lexicon_tags_40 },
    { 0xa291196d, SUC_NE_lexicon_tags_2 },
    { 0x6cad196d, SUC_NE_lexicon_tags_2 },
    { 0xbb77196d, SUC_NE_lexicon_tags_2 },
    { 0x1a9f1970, SUC_NE_lexicon_tags_2 },
    { 0x61a11970, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bad1973, SUC_NE_lexicon_tags_2 },
    { 0xac8d1974, SUC_NE_lexicon_tags_2 },
    { 0x1dcf1975, SUC_NE_lexicon_tags_2 },
    { 0x6cd71975, SUC_NE_lexicon_tags_2 },
    { 0x36751977, SUC_NE_lexicon_tags_2 },
    { 0xce231975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19b197d, SUC_NE_lexicon_tags_2 },
    { 0xb4c5197e, SUC_NE_lexicon_tags_4 },
    { 0x5a6d197f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37bb1981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4ed1983, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd251989, SUC_NE_lexicon_tags_1 },
    { 0xee111989, SUC_NE_lexicon_tags_4 },
    { 0x20cd198b, SUC_NE_lexicon_tags_2 },
    { 0x41198c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a11990, SUC_NE_lexicon_tags_2 },
    { 0xa711990, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc031993, SUC_NE_lexicon_tags_2 },
    { 0xccf1993, SUC_NE_lexicon_tags_2 },
    { 0x7e071995, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82131999, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6093199e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df19a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568919aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x773119ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50319b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd19b6, SUC_NE_lexicon_tags_2 },
    { 0xfd8d19b6, SUC_NE_lexicon_tags_123 },
    { 0x9daf19b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d719be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff19c2, SUC_NE_lexicon_tags_2 },
    { 0x61a719c3, SUC_NE_lexicon_tags_2 },
    { 0xb93719c2, SUC_NE_lexicon_tags_2 },
    { 0x42a119c5, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xda3b19c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1119ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cf19d2, SUC_NE_lexicon_tags_2 },
    { 0x46ad19d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683919d8, SUC_NE_lexicon_tags_2 },
    { 0x2af19d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7d19dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d319e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bb19e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd19ed, SUC_NE_lexicon_tags_2 },
    { 0xf3a319ee, SUC_NE_lexicon_tags_2 },
    { 0x4e6919ee, SUC_NE_lexicon_tags_2 },
    { 0xbfd119f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd6119f2, SUC_NE_lexicon_tags_2 },
    { 0xd11d19f3, SUC_NE_lexicon_tags_2 },
    { 0x504319f3, SUC_NE_lexicon_tags_2 },
    { 0x621b19f5, SUC_NE_lexicon_tags_2 },
    { 0xfb7b19f3, SUC_NE_lexicon_tags_3 },
    { 0x395919f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b19fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44b319fc, SUC_NE_lexicon_tags_2 },
    { 0x435919fc, SUC_NE_lexicon_tags_2 },
    { 0x995d19fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf51a00, SUC_NE_lexicon_tags_13 },
    { 0xbdc91a01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8d91a03, SUC_NE_lexicon_tags_2 },
    { 0xd5891a04, SUC_NE_lexicon_tags_10 },
    { 0xd7551a05, SUC_NE_lexicon_tags_2 },
    { 0x1751a06, SUC_NE_lexicon_tags_2 },
    { 0x707f1a06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69bf1a09, SUC_NE_lexicon_tags_2 },
    { 0x91d31a0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3931a17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47b1a19, SUC_NE_lexicon_tags_2 },
    { 0x82611a1a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6f231a1c, SUC_NE_lexicon_tags_2 },
    { 0xfb451a1d, SUC_NE_lexicon_tags_2 },
    { 0x611a1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cd1a21, SUC_NE_lexicon_tags_2 },
    { 0x65a11a22, SUC_NE_lexicon_tags_2 },
    { 0xe1c91a23, SUC_NE_lexicon_tags_2 },
    { 0x63f91a24, SUC_NE_lexicon_tags_4 },
    { 0xa3a31a25, SUC_NE_lexicon_tags_2 },
    { 0xc7d31a26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb91a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb13d1a2b, SUC_NE_lexicon_tags_7 },
    { 0x29b31a2b, SUC_NE_lexicon_tags_4 },
    { 0x3c5b1a2d, SUC_NE_lexicon_tags_2 },
    { 0xf43b1a2e, SUC_NE_lexicon_tags_2 },
    { 0x688f1a2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36491a32, SUC_NE_lexicon_tags_13 },
    { 0x1a551a32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d931a3b, SUC_NE_lexicon_tags_2 },
    { 0xd3791a3c, SUC_NE_lexicon_tags_34 },
    { 0x7b211a3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd1a40, SUC_NE_lexicon_tags_2 },
    { 0xc9e71a41, SUC_NE_lexicon_tags_2 },
    { 0xa3b91a42, SUC_NE_lexicon_tags_7 },
    { 0x84451a40, SUC_NE_lexicon_tags_2 },
    { 0x21c71a40, SUC_NE_lexicon_tags_2 },
    { 0x6ec91a43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f1a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4b1a55, SUC_NE_lexicon_tags_4 },
    { 0x57cb1a56, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2651a59, SUC_NE_lexicon_tags_2 },
    { 0x898d1a5a, SUC_NE_lexicon_tags_2 },
    { 0x3f0f1a5a, SUC_NE_lexicon_tags_2 },
    { 0x133f1a5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd11a5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x800d1a61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71df1a6a, SUC_NE_lexicon_tags_2 },
    { 0x337b1a6a, SUC_NE_lexicon_tags_2 },
    { 0x2de51a6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c11a71, SUC_NE_lexicon_tags_2 },
    { 0x13091a72, SUC_NE_lexicon_tags_2 },
    { 0x790f1a73, SUC_NE_lexicon_tags_2 },
    { 0xc7a51a71, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x23711a77, SUC_NE_lexicon_tags_2 },
    { 0x84871a77, SUC_NE_lexicon_tags_188 },
    { 0xbe171a79, SUC_NE_lexicon_tags_2 },
    { 0xe74f1a77, SUC_NE_lexicon_tags_2 },
    { 0xff2f1a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88bd1a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf3f1a7f, SUC_NE_lexicon_tags_2 },
    { 0x173f1a80, SUC_NE_lexicon_tags_2 },
    { 0xcbb11a81, SUC_NE_lexicon_tags_2 },
    { 0x3e1b1a82, SUC_NE_lexicon_tags_3 },
    { 0x409f1a83, SUC_NE_lexicon_tags_1 },
    { 0x413f1a83, SUC_NE_lexicon_tags_2 },
    { 0x1e8d1a83, SUC_NE_lexicon_tags_2 },
    { 0x63991a81, SUC_NE_lexicon_tags_2 },
    { 0x7adf1a85, SUC_NE_lexicon_tags_2 },
    { 0xa4d51a83, SUC_NE_lexicon_tags_2 },
    { 0xba8f1a81, SUC_NE_lexicon_tags_2 },
    { 0xd8031a8a, SUC_NE_lexicon_tags_2 },
    { 0x2071a81, SUC_NE_lexicon_tags_2 },
    { 0x43611a85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed1a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b1a94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec91a9c, SUC_NE_lexicon_tags_2 },
    { 0x80d11a9c, SUC_NE_lexicon_tags_2 },
    { 0xf7411a9d, SUC_NE_lexicon_tags_2 },
    { 0xe6c31a9f, SUC_NE_lexicon_tags_4 },
    { 0x72471aa0, SUC_NE_lexicon_tags_18 },
    { 0xbbaf1a9f, SUC_NE_lexicon_tags_0 },
    { 0xdbbf1aa2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18051aa9, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d691aac, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f71ab0, SUC_NE_lexicon_tags_2 },
    { 0x82e31ab1, SUC_NE_lexicon_tags_2 },
    { 0x3e331ab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1531aba, SUC_NE_lexicon_tags_2 },
    { 0x571d1aba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7d1ac0, SUC_NE_lexicon_tags_2 },
    { 0xe0a11ac1, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758d1ac8, SUC_NE_lexicon_tags_2 },
    { 0x876f1ac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c91acc, SUC_NE_lexicon_tags_2 },
    { 0x3efb1acc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x601b1acf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfab31ad1, SUC_NE_lexicon_tags_2 },
    { 0xb1f1ad2, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1451ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe511ae0, SUC_NE_lexicon_tags_2 },
    { 0x6d71ae1, SUC_NE_lexicon_tags_2 },
    { 0xdc611ae2, SUC_NE_lexicon_tags_2 },
    { 0xe7b91ae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdb91ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf1aeb, SUC_NE_lexicon_tags_2 },
    { 0x43071aec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4d11aee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf1af1, SUC_NE_lexicon_tags_2 },
    { 0xdbf91af2, SUC_NE_lexicon_tags_2 },
    { 0x20cf1af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cd1af6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d1afa, SUC_NE_lexicon_tags_2 },
    { 0xae291afb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f1afe, SUC_NE_lexicon_tags_2 },
    { 0xb6c31afe, SUC_NE_lexicon_tags_2 },
    { 0x3ca71afe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b91b04, SUC_NE_lexicon_tags_2 },
    { 0xb5ab1b04, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b11b0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1b11, SUC_NE_lexicon_tags_2 },
    { 0xd0e51b12, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x12151b15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5731b1a, SUC_NE_lexicon_tags_2 },
    { 0x8a711b1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd08f1b1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x336d1b1f, SUC_NE_lexicon_tags_2 },
    { 0x7e771b20, SUC_NE_lexicon_tags_2 },
    { 0x67971b21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5271b25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa9d1b27, SUC_NE_lexicon_tags_2 },
    { 0xe2791b27, SUC_NE_lexicon_tags_2 },
    { 0x7e1d1b29, SUC_NE_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47851b2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc591b36, SUC_NE_lexicon_tags_5 },
    { 0x30fd1b37, SUC_NE_lexicon_tags_2 },
    { 0x5251b37, SUC_NE_lexicon_tags_2 },
    { 0x87f31b38, SUC_NE_lexicon_tags_2 },
    { 0x15231b37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e431b3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2f1b40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46291b43, SUC_NE_lexicon_tags_2 },
    { 0x5b91b44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91031b46, SUC_NE_lexicon_tags_2 },
    { 0x31651b47, SUC_NE_lexicon_tags_2 },
    { 0xc151b46, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x353b1b4a, SUC_NE_lexicon_tags_2 },
    { 0xc2ab1b4a, SUC_NE_lexicon_tags_2 },
    { 0x79f51b4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34371b51, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd71b54, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x3eb31b56, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5431b5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77671b60, SUC_NE_lexicon_tags_2 },
    { 0x381f1b61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe32f1b63, SUC_NE_lexicon_tags_1 },
    { 0x7cf51b64, SUC_NE_lexicon_tags_2 },
    { 0xf7e91b65, SUC_NE_lexicon_tags_2 },
    { 0x8c8d1b63, SUC_NE_lexicon_tags_2 },
    { 0x31971b64, SUC_NE_lexicon_tags_2 },
    { 0x766d1b68, SUC_NE_lexicon_tags_4 },
    { 0xd7ad1b68, SUC_NE_lexicon_tags_2 },
    { 0x3c31b6a, SUC_NE_lexicon_tags_2 },
    { 0xd3eb1b6b, SUC_NE_lexicon_tags_2 },
    { 0xf4211b66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7331b6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f91b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17f31b79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0251b7c, SUC_NE_lexicon_tags_4 },
    { 0x8ae31b7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6431b80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4891b84, SUC_NE_lexicon_tags_2 },
    { 0x160d1b85, SUC_NE_lexicon_tags_2 },
    { 0x7e4f1b86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b11b89, SUC_NE_lexicon_tags_2 },
    { 0x98c51b8a, SUC_NE_lexicon_tags_2 },
    { 0x1a9b1b8b, SUC_NE_lexicon_tags_2 },
    { 0x6fa31b8c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc251b8e, SUC_NE_lexicon_tags_2 },
    { 0xb1fb1b8e, SUC_NE_lexicon_tags_4 },
    { 0xdafb1b90, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c31b94, SUC_NE_lexicon_tags_2 },
    { 0x8851b95, SUC_NE_lexicon_tags_2 },
    { 0x434f1b96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd71b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d871b9e, SUC_NE_lexicon_tags_1 },
    { 0x3be51b9f, SUC_NE_lexicon_tags_2 },
    { 0xdd271ba0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eff1ba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f2f1ba5, SUC_NE_lexicon_tags_2 },
    { 0x66911ba5, SUC_NE_lexicon_tags_2 },
    { 0x23a51ba7, SUC_NE_lexicon_tags_151 },
    { 0, NULL },
    { 0xcd9b1ba9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52f91bab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f911bad, SUC_NE_lexicon_tags_2 },
    { 0xf1c51bae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f91bb1, SUC_NE_lexicon_tags_2 },
    { 0x52b91bb1, SUC_NE_lexicon_tags_23 },
    { 0x371f1bb3, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd31bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b351bb9, SUC_NE_lexicon_tags_2 },
    { 0x36a71bba, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x57bb1bbc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bd1bc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13131bc3, SUC_NE_lexicon_tags_2 },
    { 0x18df1bc4, SUC_NE_lexicon_tags_2 },
    { 0x87c51bc3, SUC_NE_lexicon_tags_2 },
    { 0x7e251bc6, SUC_NE_lexicon_tags_2 },
    { 0x7aab1bc3, SUC_NE_lexicon_tags_2 },
    { 0x7431bc8, SUC_NE_lexicon_tags_2 },
    { 0xca751bc3, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d851bce, SUC_NE_lexicon_tags_0 },
    { 0x11e91bcf, SUC_NE_lexicon_tags_2 },
    { 0xfb511bd0, SUC_NE_lexicon_tags_2 },
    { 0x69d51bd1, SUC_NE_lexicon_tags_2 },
    { 0xa6691bd2, SUC_NE_lexicon_tags_4 },
    { 0xe731bd2, SUC_NE_lexicon_tags_2 },
    { 0xb3751bd1, SUC_NE_lexicon_tags_2 },
    { 0x366b1bd4, SUC_NE_lexicon_tags_2 },
    { 0x757d1bd5, SUC_NE_lexicon_tags_2 },
    { 0xf8751bd7, SUC_NE_lexicon_tags_2 },
    { 0x62231bd3, SUC_NE_lexicon_tags_2 },
    { 0xd5391bd9, SUC_NE_lexicon_tags_2 },
    { 0xb4491bd4, SUC_NE_lexicon_tags_2 },
    { 0x819d1bd0, SUC_NE_lexicon_tags_2 },
    { 0xc2831bdc, SUC_NE_lexicon_tags_2 },
    { 0xb6291bdb, SUC_NE_lexicon_tags_32 },
    { 0xc2631bdd, SUC_NE_lexicon_tags_2 },
    { 0x92111bdf, SUC_NE_lexicon_tags_2 },
    { 0x2a111bd7, SUC_NE_lexicon_tags_2 },
    { 0x7f8f1bd1, SUC_NE_lexicon_tags_2 },
    { 0x2a9b1be2, SUC_NE_lexicon_tags_40 },
    { 0x76511be3, SUC_NE_lexicon_tags_1 },
    { 0x16331be2, SUC_NE_lexicon_tags_17 },
    { 0x8751be5, SUC_NE_lexicon_tags_2 },
    { 0x7a7b1be6, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x72111be8, SUC_NE_lexicon_tags_34 },
    { 0xf23b1be8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x722b1bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7771bf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7271bf2, SUC_NE_lexicon_tags_5 },
    { 0xb1791bf3, SUC_NE_lexicon_tags_2 },
    { 0x37911bf4, SUC_NE_lexicon_tags_2 },
    { 0x4a451bf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f7f1bf7, SUC_NE_lexicon_tags_2 },
    { 0x37f1bf8, SUC_NE_lexicon_tags_2 },
    { 0xacb31bf8, SUC_NE_lexicon_tags_304 },
    { 0x3e891bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4bb1bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ef1bff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x597b1c02, SUC_NE_lexicon_tags_2 },
    { 0x30c51c03, SUC_NE_lexicon_tags_2 },
    { 0x723b1c04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ff1c07, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb51c0c, SUC_NE_lexicon_tags_2 },
    { 0x60231c0c, SUC_NE_lexicon_tags_2 },
    { 0xdb751c0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12f1c18, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xde771c1a, SUC_NE_lexicon_tags_2 },
    { 0x8e5f1c1b, SUC_NE_lexicon_tags_2 },
    { 0xb11d1c1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x112d1c1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f51c22, SUC_NE_lexicon_tags_2 },
    { 0xb85b1c23, SUC_NE_lexicon_tags_2 },
    { 0xcedb1c24, SUC_NE_lexicon_tags_2 },
    { 0x55891c25, SUC_NE_lexicon_tags_2 },
    { 0xeeb71c26, SUC_NE_lexicon_tags_2 },
    { 0x3ae11c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x431f1c2a, SUC_NE_lexicon_tags_2 },
    { 0x93ab1c2b, SUC_NE_lexicon_tags_52 },
    { 0x2ca71c2a, SUC_NE_lexicon_tags_2 },
    { 0x1f331c2d, SUC_NE_lexicon_tags_2 },
    { 0x844b1c2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd65d1c30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf37f1c32, SUC_NE_lexicon_tags_2 },
    { 0x4dd71c32, SUC_NE_lexicon_tags_2 },
    { 0x882b1c34, SUC_NE_lexicon_tags_2 },
    { 0xdcf11c35, SUC_NE_lexicon_tags_2 },
    { 0x1d111c35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x728d1c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22151c3a, SUC_NE_lexicon_tags_2 },
    { 0xa97d1c3a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x524b1c3d, SUC_NE_lexicon_tags_2 },
    { 0xe2c71c3e, SUC_NE_lexicon_tags_2 },
    { 0x9d631c3f, SUC_NE_lexicon_tags_2 },
    { 0x657f1c40, SUC_NE_lexicon_tags_2 },
    { 0x9aa31c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb231c45, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89491c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ead1c4d, SUC_NE_lexicon_tags_2 },
    { 0x866b1c4d, SUC_NE_lexicon_tags_2 },
    { 0x6dbd1c4f, SUC_NE_lexicon_tags_2 },
    { 0xf6371c4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0831c52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5351c54, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3551c63, SUC_NE_lexicon_tags_2 },
    { 0xbc031c64, SUC_NE_lexicon_tags_2 },
    { 0xe0cf1c65, SUC_NE_lexicon_tags_2 },
    { 0x11411c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x144b1c69, SUC_NE_lexicon_tags_2 },
    { 0x1a671c6a, SUC_NE_lexicon_tags_2 },
    { 0xf0231c6b, SUC_NE_lexicon_tags_2 },
    { 0x288b1c6b, SUC_NE_lexicon_tags_2 },
    { 0xc1111c69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20751c70, SUC_NE_lexicon_tags_4 },
    { 0x40471c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecb91c73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb1c77, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5771c7c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d811c7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21f1c82, SUC_NE_lexicon_tags_8 },
    { 0x754f1c83, SUC_NE_lexicon_tags_167 },
    { 0x9cad1c83, SUC_NE_lexicon_tags_4 },
    { 0xd6d51c83, SUC_NE_lexicon_tags_2 },
    { 0x72451c86, SUC_NE_lexicon_tags_3 },
    { 0x7aa71c87, SUC_NE_lexicon_tags_2 },
    { 0x45ef1c88, SUC_NE_lexicon_tags_2 },
    { 0xcda31c89, SUC_NE_lexicon_tags_2 },
    { 0x4031c86, SUC_NE_lexicon_tags_2 },
    { 0x24f71c85, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869f1c92, SUC_NE_lexicon_tags_2 },
    { 0x9e231c92, SUC_NE_lexicon_tags_2 },
    { 0xc62f1c93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb591c99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfd1c9d, SUC_NE_lexicon_tags_2 },
    { 0x85951c9e, SUC_NE_lexicon_tags_2 },
    { 0x3c631c9f, SUC_NE_lexicon_tags_2 },
    { 0x6cc11ca0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19631ca3, SUC_NE_lexicon_tags_2 },
    { 0x13a91ca3, SUC_NE_lexicon_tags_2 },
    { 0xaa731ca5, SUC_NE_lexicon_tags_4 },
    { 0x918f1ca6, SUC_NE_lexicon_tags_0 },
    { 0x3acb1ca4, SUC_NE_lexicon_tags_2 },
    { 0x95fb1ca4, SUC_NE_lexicon_tags_2 },
    { 0x18931ca3, SUC_NE_lexicon_tags_2 },
    { 0x59c91ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78931cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46df1cb0, SUC_NE_lexicon_tags_2 },
    { 0x56331cb1, SUC_NE_lexicon_tags_2 },
    { 0x66991cb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87b71cb4, SUC_NE_lexicon_tags_150 },
    { 0x2a391cb5, SUC_NE_lexicon_tags_2 },
    { 0x1c351cb6, SUC_NE_lexicon_tags_2 },
    { 0xad291cb4, SUC_NE_lexicon_tags_0 },
    { 0x22271cb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d91cbb, SUC_NE_lexicon_tags_2 },
    { 0x78451cbc, SUC_NE_lexicon_tags_2 },
    { 0x57471cbd, SUC_NE_lexicon_tags_2 },
    { 0xc7b71cbe, SUC_NE_lexicon_tags_2 },
    { 0x53831cbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11f1cc2, SUC_NE_lexicon_tags_2 },
    { 0x9fef1cc2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x59271cc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ee11cc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26611cca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb671ccf, SUC_NE_lexicon_tags_2 },
    { 0xb93f1cd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6771cd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87951cd6, SUC_NE_lexicon_tags_0 },
    { 0xadcd1cd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30dd1cda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a811cdd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da11ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dd1ce6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4371ce8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6e31cea, SUC_NE_lexicon_tags_2 },
    { 0x8fb71ceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b971cf4, SUC_NE_lexicon_tags_2 },
    { 0xf2591cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7b1cfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90ab1cfc, SUC_NE_lexicon_tags_2 },
    { 0x9dbb1cfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8b11cff, SUC_NE_lexicon_tags_2 },
    { 0x653f1d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3991d02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7f1d06, SUC_NE_lexicon_tags_7 },
    { 0x837d1d06, SUC_NE_lexicon_tags_2 },
    { 0xf2fd1d08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e591d0a, SUC_NE_lexicon_tags_2 },
    { 0x3ff91d0b, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aeb1d0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7851d11, SUC_NE_lexicon_tags_2 },
    { 0xf0c51d12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53f1d16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65411d1a, SUC_NE_lexicon_tags_2 },
    { 0xbbe71d1a, SUC_NE_lexicon_tags_2 },
    { 0xa2011d1b, SUC_NE_lexicon_tags_2 },
    { 0x4a5d1d1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16411d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e11d22, SUC_NE_lexicon_tags_2 },
    { 0xdcbf1d23, SUC_NE_lexicon_tags_2 },
    { 0x9231d23, SUC_NE_lexicon_tags_2 },
    { 0xa3731d25, SUC_NE_lexicon_tags_288 },
    { 0x151d1d26, SUC_NE_lexicon_tags_2 },
    { 0x91d31d23, SUC_NE_lexicon_tags_7 },
    { 0x22151d25, SUC_NE_lexicon_tags_2 },
    { 0xbadb1d26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60071d2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab11d33, SUC_NE_lexicon_tags_2 },
    { 0xd3eb1d33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5531d39, SUC_NE_lexicon_tags_2 },
    { 0xf1891d39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c51d3e, SUC_NE_lexicon_tags_2 },
    { 0x125b1d3e, SUC_NE_lexicon_tags_2 },
    { 0xa9771d40, SUC_NE_lexicon_tags_2 },
    { 0x807b1d41, SUC_NE_lexicon_tags_3 },
    { 0x9ac91d42, SUC_NE_lexicon_tags_2 },
    { 0x9e11d41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc1f1d45, SUC_NE_lexicon_tags_2 },
    { 0xc6191d45, SUC_NE_lexicon_tags_2 },
    { 0x72bd1d47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65391d49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a91d4c, SUC_NE_lexicon_tags_2 },
    { 0x47591d4d, SUC_NE_lexicon_tags_2 },
    { 0xbd2b1d4d, SUC_NE_lexicon_tags_2 },
    { 0x46a11d4f, SUC_NE_lexicon_tags_2 },
    { 0x2d1d4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4611d52, SUC_NE_lexicon_tags_2 },
    { 0x11a51d53, SUC_NE_lexicon_tags_2 },
    { 0xae3b1d53, SUC_NE_lexicon_tags_2 },
    { 0x787b1d55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1b1d5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1b1d5d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2a731d5f, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeab1d62, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x913d1d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db71d67, SUC_NE_lexicon_tags_2 },
    { 0x6c3d1d68, SUC_NE_lexicon_tags_2 },
    { 0xe0411d68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15211d6b, SUC_NE_lexicon_tags_2 },
    { 0x8d791d6b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1871d6e, SUC_NE_lexicon_tags_2 },
    { 0x7dc71d6f, SUC_NE_lexicon_tags_2 },
    { 0xd39b1d70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfef11d72, SUC_NE_lexicon_tags_30 },
    { 0xcb371d72, SUC_NE_lexicon_tags_2 },
    { 0xe4a51d74, SUC_NE_lexicon_tags_2 },
    { 0x8e311d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7471d7a, SUC_NE_lexicon_tags_2 },
    { 0xb63d1d7b, SUC_NE_lexicon_tags_2 },
    { 0x94171d7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa611d7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d31d83, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc71d87, SUC_NE_lexicon_tags_2 },
    { 0xd2871d88, SUC_NE_lexicon_tags_34 },
    { 0xbc4d1d87, SUC_NE_lexicon_tags_2 },
    { 0x34171d87, SUC_NE_lexicon_tags_2 },
    { 0x60351d8b, SUC_NE_lexicon_tags_2 },
    { 0x4faf1d87, SUC_NE_lexicon_tags_2 },
    { 0x76691d8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f91d91, SUC_NE_lexicon_tags_2 },
    { 0x20951d92, SUC_NE_lexicon_tags_2 },
    { 0x60971d93, SUC_NE_lexicon_tags_2 },
    { 0xe7351d93, SUC_NE_lexicon_tags_2 },
    { 0x6b091d95, SUC_NE_lexicon_tags_2 },
    { 0x34fb1d95, SUC_NE_lexicon_tags_2 },
    { 0xe1851d95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe91d9b, SUC_NE_lexicon_tags_2 },
    { 0x704b1d9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde11da1, SUC_NE_lexicon_tags_2 },
    { 0x2a71da1, SUC_NE_lexicon_tags_2 },
    { 0x231da3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e791da7, SUC_NE_lexicon_tags_3 },
    { 0xa2bf1da8, SUC_NE_lexicon_tags_2 },
    { 0x3b551da9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46911dab, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa1db1dad, SUC_NE_lexicon_tags_7 },
    { 0x7e7d1dae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdd1db8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eb1dbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811f1dbf, SUC_NE_lexicon_tags_2 },
    { 0xda271dc0, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66551dc4, SUC_NE_lexicon_tags_2 },
    { 0xbb6b1dc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61d1dc9, SUC_NE_lexicon_tags_2 },
    { 0xc73d1dca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83331dcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9051dd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f71dd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acb1ddb, SUC_NE_lexicon_tags_2 },
    { 0xf4051ddb, SUC_NE_lexicon_tags_2 },
    { 0x8f171ddc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x70a91ddf, SUC_NE_lexicon_tags_2 },
    { 0xd5331de0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6d1de4, SUC_NE_lexicon_tags_2 },
    { 0xdbd1de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed1de9, SUC_NE_lexicon_tags_2 },
    { 0xa83b1dea, SUC_NE_lexicon_tags_2 },
    { 0xeed71deb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb2b1ded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74131df0, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13b1dfa, SUC_NE_lexicon_tags_2 },
    { 0x4c031dfb, SUC_NE_lexicon_tags_11 },
    { 0x13b11dfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f11dff, SUC_NE_lexicon_tags_2 },
    { 0xdefd1e00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317f1e05, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d51e0a, SUC_NE_lexicon_tags_2 },
    { 0xf80d1e0b, SUC_NE_lexicon_tags_2 },
    { 0x3cc11e0c, SUC_NE_lexicon_tags_2 },
    { 0xf2251e0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x706d1e0f, SUC_NE_lexicon_tags_2 },
    { 0x3f431e0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac91e13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11691e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fd51e18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b31e1d, SUC_NE_lexicon_tags_2 },
    { 0xaab11e1e, SUC_NE_lexicon_tags_1 },
    { 0x71731e1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x364b1e22, SUC_NE_lexicon_tags_2 },
    { 0xbddb1e22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa31e26, SUC_NE_lexicon_tags_4 },
    { 0xd7951e26, SUC_NE_lexicon_tags_2 },
    { 0x46bb1e28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc55d1e2a, SUC_NE_lexicon_tags_2 },
    { 0x54b51e2a, SUC_NE_lexicon_tags_18 },
    { 0xb4c71e2a, SUC_NE_lexicon_tags_2 },
    { 0x58491e2c, SUC_NE_lexicon_tags_2 },
    { 0xd1f91e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb851e31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e91e34, SUC_NE_lexicon_tags_2 },
    { 0x340f1e35, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c91e38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6031e3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad2b1e3d, SUC_NE_lexicon_tags_2 },
    { 0x3a351e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b391e40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1e45, SUC_NE_lexicon_tags_3 },
    { 0x4691e45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8db1e48, SUC_NE_lexicon_tags_2 },
    { 0x89971e48, SUC_NE_lexicon_tags_2 },
    { 0x64b1e49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce271e4f, SUC_NE_lexicon_tags_7 },
    { 0x96ed1e4f, SUC_NE_lexicon_tags_2 },
    { 0xdd951e51, SUC_NE_lexicon_tags_2 },
    { 0xcf291e52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11691e57, SUC_NE_lexicon_tags_2 },
    { 0x553b1e58, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712d1e5d, SUC_NE_lexicon_tags_3 },
    { 0xcc9b1e5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdc31e60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d4d1e62, SUC_NE_lexicon_tags_2 },
    { 0x5ead1e62, SUC_NE_lexicon_tags_2 },
    { 0x1a531e64, SUC_NE_lexicon_tags_2 },
    { 0xb4db1e62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a771e6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57f71e6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f31e70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb11e78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d51e7c, SUC_NE_lexicon_tags_2 },
    { 0x72c71e7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f11e86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd31e8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688b1e94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20f1e96, SUC_NE_lexicon_tags_2 },
    { 0x79d31e96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b0d1e99, SUC_NE_lexicon_tags_2 },
    { 0x58b91e99, SUC_NE_lexicon_tags_17 },
    { 0xe771e9a, SUC_NE_lexicon_tags_24 },
    { 0x35011e9c, SUC_NE_lexicon_tags_2 },
    { 0xacf71e99, SUC_NE_lexicon_tags_2 },
    { 0x5efd1e9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06b1ea3, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b271ea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00b1eab, SUC_NE_lexicon_tags_2 },
    { 0x68cd1eac, SUC_NE_lexicon_tags_2 },
    { 0x33b31eab, SUC_NE_lexicon_tags_2 },
    { 0xb9671eae, SUC_NE_lexicon_tags_2 },
    { 0xbcdd1eaf, SUC_NE_lexicon_tags_1 },
    { 0x49491eb0, SUC_NE_lexicon_tags_25 },
    { 0x312b1eb1, SUC_NE_lexicon_tags_2 },
    { 0x7e9f1eb2, SUC_NE_lexicon_tags_60 },
    { 0x13651eb3, SUC_NE_lexicon_tags_4 },
    { 0xb48f1eb3, SUC_NE_lexicon_tags_404 },
    { 0x6b811eac, SUC_NE_lexicon_tags_2 },
    { 0x6f291eb6, SUC_NE_lexicon_tags_2 },
    { 0xad7f1eb7, SUC_NE_lexicon_tags_2 },
    { 0xeba11eb7, SUC_NE_lexicon_tags_2 },
    { 0x9d5b1ead, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xf691ebb, SUC_NE_lexicon_tags_2 },
    { 0xeccd1ebc, SUC_NE_lexicon_tags_2 },
    { 0x14ad1ebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dd1ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd3f1ec6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0651ec9, SUC_NE_lexicon_tags_2 },
    { 0x226f1eca, SUC_NE_lexicon_tags_8 },
    { 0x931b1ecb, SUC_NE_lexicon_tags_2 },
    { 0xa0ef1eca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1d1ed0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b1ed9, SUC_NE_lexicon_tags_4 },
    { 0x31ed1eda, SUC_NE_lexicon_tags_2 },
    { 0xcf1eda, SUC_NE_lexicon_tags_2 },
    { 0xf2271edc, SUC_NE_lexicon_tags_2 },
    { 0x53db1edd, SUC_NE_lexicon_tags_2 },
    { 0x8eb11edd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x746f1ee1, SUC_NE_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d851ee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d31eed, SUC_NE_lexicon_tags_2 },
    { 0x94db1eee, SUC_NE_lexicon_tags_2 },
    { 0x71191eed, SUC_NE_lexicon_tags_2 },
    { 0xd8471ef0, SUC_NE_lexicon_tags_2 },
    { 0x50551ef0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d7d1ef3, SUC_NE_lexicon_tags_2 },
    { 0x19611ef4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35891ef8, SUC_NE_lexicon_tags_2 },
    { 0x68471ef9, SUC_NE_lexicon_tags_2 },
    { 0xdb2b1efa, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd11efd, SUC_NE_lexicon_tags_2 },
    { 0x45271efe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf31f02, SUC_NE_lexicon_tags_3 },
    { 0x2d271f02, SUC_NE_lexicon_tags_2 },
    { 0x39271f04, SUC_NE_lexicon_tags_2 },
    { 0xd1071f05, SUC_NE_lexicon_tags_2 },
    { 0xe1b1f06, SUC_NE_lexicon_tags_4 },
    { 0x48891f06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c51f0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81eb1f0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x923f1f11, SUC_NE_lexicon_tags_2 },
    { 0x5a911f11, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81b1f18, SUC_NE_lexicon_tags_2 },
    { 0x2d11f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14111f1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1f1f23, SUC_NE_lexicon_tags_2 },
    { 0xc30d1f24, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0x31b11f26, SUC_NE_lexicon_tags_0 },
    { 0x9dd51f27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x784f1f2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef051f2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f631f32, SUC_NE_lexicon_tags_2 },
    { 0xb61d1f33, SUC_NE_lexicon_tags_2 },
    { 0xca751f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d31f39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e51f3e, SUC_NE_lexicon_tags_2 },
    { 0x15df1f3f, SUC_NE_lexicon_tags_2 },
    { 0xdcef1f40, SUC_NE_lexicon_tags_2 },
    { 0xd4ff1f3f, SUC_NE_lexicon_tags_8 },
    { 0x2f4b1f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7af1f48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99711f4b, SUC_NE_lexicon_tags_2 },
    { 0xcdf1f4c, SUC_NE_lexicon_tags_2 },
    { 0x3e051f4d, SUC_NE_lexicon_tags_2 },
    { 0x37331f4e, SUC_NE_lexicon_tags_2 },
    { 0xd21b1f4e, SUC_NE_lexicon_tags_2 },
    { 0xb18b1f50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23c11f52, SUC_NE_lexicon_tags_2 },
    { 0x41a71f53, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3fb31f55, SUC_NE_lexicon_tags_2 },
    { 0xba631f55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d431f59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1351f5d, SUC_NE_lexicon_tags_2 },
    { 0xf5371f5e, SUC_NE_lexicon_tags_2 },
    { 0x3c411f5f, SUC_NE_lexicon_tags_2 },
    { 0x22cd1f60, SUC_NE_lexicon_tags_7 },
    { 0xe50f1f60, SUC_NE_lexicon_tags_2 },
    { 0xaeed1f60, SUC_NE_lexicon_tags_2 },
    { 0xcd3d1f63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68271f6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6c31f6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d951f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429b1f77, SUC_NE_lexicon_tags_2 },
    { 0x8d491f77, SUC_NE_lexicon_tags_2 },
    { 0xc6b91f79, SUC_NE_lexicon_tags_2 },
    { 0xf8671f79, SUC_NE_lexicon_tags_2 },
    { 0x8fd71f7b, SUC_NE_lexicon_tags_2 },
    { 0x4b4d1f7c, SUC_NE_lexicon_tags_2 },
    { 0x79e51f7d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c411f82, SUC_NE_lexicon_tags_2 },
    { 0x83791f83, SUC_NE_lexicon_tags_2 },
    { 0x78a91f84, SUC_NE_lexicon_tags_2 },
    { 0xc08f1f85, SUC_NE_lexicon_tags_2 },
    { 0xec131f86, SUC_NE_lexicon_tags_2 },
    { 0xf3431f87, SUC_NE_lexicon_tags_2 },
    { 0xe1e11f88, SUC_NE_lexicon_tags_2 },
    { 0x39231f89, SUC_NE_lexicon_tags_2 },
    { 0xc0cb1f8a, SUC_NE_lexicon_tags_2 },
    { 0x54711f82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db91f8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5db1f93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x451b1f96, SUC_NE_lexicon_tags_2 },
    { 0x634f1f97, SUC_NE_lexicon_tags_2 },
    { 0x2d0f1f97, SUC_NE_lexicon_tags_2 },
    { 0x72a31f98, SUC_NE_lexicon_tags_2 },
    { 0x76111f96, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x3aa51f9c, SUC_NE_lexicon_tags_2 },
    { 0x6f911f9c, SUC_NE_lexicon_tags_2 },
    { 0x92111f9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17a91fa0, SUC_NE_lexicon_tags_2 },
    { 0x56251fa0, SUC_NE_lexicon_tags_2 },
    { 0x8a091fa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad11fa5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x64e71fa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a751fa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x643b1fab, SUC_NE_lexicon_tags_49 },
    { 0x3851fab, SUC_NE_lexicon_tags_2 },
    { 0x62851fad, SUC_NE_lexicon_tags_104 },
    { 0x40b31fab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64071fb4, SUC_NE_lexicon_tags_2 },
    { 0x837b1fb5, SUC_NE_lexicon_tags_2 },
    { 0x9d931fb6, SUC_NE_lexicon_tags_2 },
    { 0xd0d31fb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x136b1fb9, SUC_NE_lexicon_tags_166 },
    { 0, NULL },
    { 0x782f1fbb, SUC_NE_lexicon_tags_2 },
    { 0x19651fbc, SUC_NE_lexicon_tags_2 },
    { 0x97331fbd, SUC_NE_lexicon_tags_2 },
    { 0x19831fbe, SUC_NE_lexicon_tags_2 },
    { 0x4eeb1fbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba091fc4, SUC_NE_lexicon_tags_13 },
    { 0x8a211fc4, SUC_NE_lexicon_tags_5 },
    { 0x4dc11fc5, SUC_NE_lexicon_tags_2 },
    { 0x1ad71fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b91fcb, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84f1fd1, SUC_NE_lexicon_tags_2 },
    { 0xe0ed1fd2, SUC_NE_lexicon_tags_13 },
    { 0xa32d1fd3, SUC_NE_lexicon_tags_2 },
    { 0x37cf1fd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x782f1fd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x578d1fd8, SUC_NE_lexicon_tags_2 },
    { 0x7b2f1fd9, SUC_NE_lexicon_tags_2 },
    { 0xbcd11fd8, SUC_NE_lexicon_tags_2 },
    { 0xed391fdb, SUC_NE_lexicon_tags_2 },
    { 0x5eab1fdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80f1fe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39791fe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f11fed, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e31ff5, SUC_NE_lexicon_tags_7 },
    { 0x65fb1ff6, SUC_NE_lexicon_tags_2 },
    { 0x56bb1ff5, SUC_NE_lexicon_tags_2 },
    { 0xeb71ff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc871ffa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e31ffd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e852001, SUC_NE_lexicon_tags_2 },
    { 0xc7f12002, SUC_NE_lexicon_tags_2 },
    { 0xf7292002, SUC_NE_lexicon_tags_2 },
    { 0x6ccf2003, SUC_NE_lexicon_tags_2 },
    { 0xdf012003, SUC_NE_lexicon_tags_2 },
    { 0xee2d2005, SUC_NE_lexicon_tags_2 },
    { 0x32a52007, SUC_NE_lexicon_tags_17 },
    { 0x5b152007, SUC_NE_lexicon_tags_2 },
    { 0xde0b2009, SUC_NE_lexicon_tags_2 },
    { 0x8b200a, SUC_NE_lexicon_tags_2 },
    { 0x98e1200b, SUC_NE_lexicon_tags_2 },
    { 0x73252008, SUC_NE_lexicon_tags_2 },
    { 0xc1df200d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f32011, SUC_NE_lexicon_tags_2 },
    { 0x87332012, SUC_NE_lexicon_tags_2 },
    { 0xc1472012, SUC_NE_lexicon_tags_2 },
    { 0xf70f2012, SUC_NE_lexicon_tags_2 },
    { 0xe54f2013, SUC_NE_lexicon_tags_155 },
    { 0x825d2016, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88472018, SUC_NE_lexicon_tags_2 },
    { 0x89112019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe14d201b, SUC_NE_lexicon_tags_2 },
    { 0xe5ff201b, SUC_NE_lexicon_tags_2 },
    { 0x85d201d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9232022, SUC_NE_lexicon_tags_7 },
    { 0x8d592023, SUC_NE_lexicon_tags_2 },
    { 0xca672023, SUC_NE_lexicon_tags_1 },
    { 0x59b2023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a052027, SUC_NE_lexicon_tags_2 },
    { 0xc4272028, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b202b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6161202d, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bd2032, SUC_NE_lexicon_tags_2 },
    { 0x96e32033, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9912037, SUC_NE_lexicon_tags_2 },
    { 0x1d32038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x995d203a, SUC_NE_lexicon_tags_7 },
    { 0x3681203a, SUC_NE_lexicon_tags_2 },
    { 0xe77d203b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ff2042, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x257b2045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d112047, SUC_NE_lexicon_tags_2 },
    { 0xeee52048, SUC_NE_lexicon_tags_2 },
    { 0x7df72047, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4b204d, SUC_NE_lexicon_tags_2 },
    { 0x714d204e, SUC_NE_lexicon_tags_2 },
    { 0x1b41204f, SUC_NE_lexicon_tags_2 },
    { 0xb4e92050, SUC_NE_lexicon_tags_2 },
    { 0x9872051, SUC_NE_lexicon_tags_2 },
    { 0x12d92051, SUC_NE_lexicon_tags_2 },
    { 0x977f2051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366f2059, SUC_NE_lexicon_tags_1 },
    { 0x2195205a, SUC_NE_lexicon_tags_2 },
    { 0xf82f205a, SUC_NE_lexicon_tags_4 },
    { 0x2cd92059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61f5205e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1d2061, SUC_NE_lexicon_tags_2 },
    { 0xba372062, SUC_NE_lexicon_tags_2 },
    { 0xedf72063, SUC_NE_lexicon_tags_2 },
    { 0x1cd72064, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bd2067, SUC_NE_lexicon_tags_11 },
    { 0x4d492067, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35dd206c, SUC_NE_lexicon_tags_2 },
    { 0xd17f206c, SUC_NE_lexicon_tags_2 },
    { 0xcde7206c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75d2070, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb92074, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9449207a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dc7207c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b32080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c512082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0872087, SUC_NE_lexicon_tags_2 },
    { 0x6e312087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6191208f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8632093, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f152096, SUC_NE_lexicon_tags_2 },
    { 0x7f12097, SUC_NE_lexicon_tags_2 },
    { 0x9efd2098, SUC_NE_lexicon_tags_2 },
    { 0x63952099, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93ef209b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51d20a0, SUC_NE_lexicon_tags_2 },
    { 0xc16520a1, SUC_NE_lexicon_tags_2 },
    { 0x673320a1, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x333f20a4, SUC_NE_lexicon_tags_2 },
    { 0xa99720a5, SUC_NE_lexicon_tags_2 },
    { 0x6f3120a6, SUC_NE_lexicon_tags_2 },
    { 0xa2b20a7, SUC_NE_lexicon_tags_2 },
    { 0x8a5520a7, SUC_NE_lexicon_tags_2 },
    { 0x8f720a9, SUC_NE_lexicon_tags_0 },
    { 0xc68520aa, SUC_NE_lexicon_tags_2 },
    { 0x80eb20ab, SUC_NE_lexicon_tags_3 },
    { 0x90320a6, SUC_NE_lexicon_tags_2 },
    { 0xeb9320ad, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c2320af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4f20b4, SUC_NE_lexicon_tags_4 },
    { 0x741120b4, SUC_NE_lexicon_tags_2 },
    { 0xf59120b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936120ba, SUC_NE_lexicon_tags_2 },
    { 0xe50120bb, SUC_NE_lexicon_tags_2 },
    { 0xb31520ba, SUC_NE_lexicon_tags_2 },
    { 0x582120bd, SUC_NE_lexicon_tags_2 },
    { 0xe5f120bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x296720c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83f720c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46d20d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda8920d3, SUC_NE_lexicon_tags_6 },
    { 0x6f0320d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ccf20d6, SUC_NE_lexicon_tags_2 },
    { 0xe81f20d7, SUC_NE_lexicon_tags_2 },
    { 0x860120d6, SUC_NE_lexicon_tags_2 },
    { 0x84f920d6, SUC_NE_lexicon_tags_2 },
    { 0x77f720da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ad20df, SUC_NE_lexicon_tags_2 },
    { 0x8c120e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64520e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e0520e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d920e9, SUC_NE_lexicon_tags_2 },
    { 0x18b720e9, SUC_NE_lexicon_tags_2 },
    { 0x95fb20ea, SUC_NE_lexicon_tags_2 },
    { 0x7e6720ec, SUC_NE_lexicon_tags_2 },
    { 0x7d1f20ed, SUC_NE_lexicon_tags_2 },
    { 0xf7df20ed, SUC_NE_lexicon_tags_2 },
    { 0x978d20ed, SUC_NE_lexicon_tags_2 },
    { 0x83f720ef, SUC_NE_lexicon_tags_3 },
    { 0x5d8b20f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5f20f6, SUC_NE_lexicon_tags_2 },
    { 0xfc6f20f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x727520f9, SUC_NE_lexicon_tags_2 },
    { 0xabdf20fa, SUC_NE_lexicon_tags_2 },
    { 0x210520fb, SUC_NE_lexicon_tags_2 },
    { 0x71bd20fc, SUC_NE_lexicon_tags_86 },
    { 0x4f1720fa, SUC_NE_lexicon_tags_2 },
    { 0x24f120fe, SUC_NE_lexicon_tags_2 },
    { 0xf21f20fe, SUC_NE_lexicon_tags_2 },
    { 0xc29f20fe, SUC_NE_lexicon_tags_3 },
    { 0xf5a52100, SUC_NE_lexicon_tags_2 },
    { 0xa7992100, SUC_NE_lexicon_tags_2 },
    { 0x70d32101, SUC_NE_lexicon_tags_2 },
    { 0xa2452104, SUC_NE_lexicon_tags_2 },
    { 0x97332105, SUC_NE_lexicon_tags_2 },
    { 0x69ad2106, SUC_NE_lexicon_tags_2 },
    { 0xc920fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7210a, SUC_NE_lexicon_tags_2 },
    { 0xd293210b, SUC_NE_lexicon_tags_0 },
    { 0xf833210c, SUC_NE_lexicon_tags_2 },
    { 0xcd1210c, SUC_NE_lexicon_tags_2 },
    { 0xff9b210d, SUC_NE_lexicon_tags_2 },
    { 0x832f210f, SUC_NE_lexicon_tags_2 },
    { 0xbadd2110, SUC_NE_lexicon_tags_4 },
    { 0xc733210f, SUC_NE_lexicon_tags_2 },
    { 0xfb41210b, SUC_NE_lexicon_tags_2 },
    { 0xde8f2113, SUC_NE_lexicon_tags_2 },
    { 0x3f772114, SUC_NE_lexicon_tags_2 },
    { 0xf32d2115, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25e52117, SUC_NE_lexicon_tags_2 },
    { 0xd4c72117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1695211a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0232121, SUC_NE_lexicon_tags_2 },
    { 0xac912122, SUC_NE_lexicon_tags_2 },
    { 0xdef72121, SUC_NE_lexicon_tags_2 },
    { 0x27fd2121, SUC_NE_lexicon_tags_2 },
    { 0x68a92121, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fd212a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2631212e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0232130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef2133, SUC_NE_lexicon_tags_1 },
    { 0x443f2134, SUC_NE_lexicon_tags_2 },
    { 0xff112135, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc92139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b3213c, SUC_NE_lexicon_tags_2 },
    { 0xe357213c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9b2140, SUC_NE_lexicon_tags_2 },
    { 0xbe7b2140, SUC_NE_lexicon_tags_2 },
    { 0xcbe12142, SUC_NE_lexicon_tags_2 },
    { 0xe0d12143, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b572145, SUC_NE_lexicon_tags_2 },
    { 0xafe32146, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d8f2148, SUC_NE_lexicon_tags_2 },
    { 0x82e32149, SUC_NE_lexicon_tags_2 },
    { 0x9caf2149, SUC_NE_lexicon_tags_2 },
    { 0x3ff7214b, SUC_NE_lexicon_tags_2 },
    { 0x19d214c, SUC_NE_lexicon_tags_4 },
    { 0x84e7214b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc72151, SUC_NE_lexicon_tags_2 },
    { 0x66752152, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf1215d, SUC_NE_lexicon_tags_2 },
    { 0x35d9215d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a92163, SUC_NE_lexicon_tags_2 },
    { 0x17c12163, SUC_NE_lexicon_tags_2 },
    { 0x4a732165, SUC_NE_lexicon_tags_2 },
    { 0x7b932166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43bb2168, SUC_NE_lexicon_tags_2 },
    { 0xe6e92168, SUC_NE_lexicon_tags_7 },
    { 0xfcdd216a, SUC_NE_lexicon_tags_2 },
    { 0x8d3f216b, SUC_NE_lexicon_tags_2 },
    { 0xb471216c, SUC_NE_lexicon_tags_2 },
    { 0x9bdb2168, SUC_NE_lexicon_tags_2 },
    { 0x4a0b216e, SUC_NE_lexicon_tags_2 },
    { 0xd9f5216f, SUC_NE_lexicon_tags_2 },
    { 0x401d216f, SUC_NE_lexicon_tags_4 },
    { 0xd0752168, SUC_NE_lexicon_tags_2 },
    { 0xe5872172, SUC_NE_lexicon_tags_2 },
    { 0x29c92173, SUC_NE_lexicon_tags_2 },
    { 0x83132174, SUC_NE_lexicon_tags_2 },
    { 0xf2812174, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x118f2177, SUC_NE_lexicon_tags_2 },
    { 0x65f2178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b9217c, SUC_NE_lexicon_tags_2 },
    { 0x69f1217c, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39092181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab2185, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a2d2187, SUC_NE_lexicon_tags_1 },
    { 0x37292188, SUC_NE_lexicon_tags_3 },
    { 0xa1d2189, SUC_NE_lexicon_tags_2 },
    { 0x3e6f2189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b3218e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e112192, SUC_NE_lexicon_tags_2 },
    { 0xb7c92193, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c432195, SUC_NE_lexicon_tags_2 },
    { 0xdf2f2196, SUC_NE_lexicon_tags_2 },
    { 0x91672197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89f219b, SUC_NE_lexicon_tags_2 },
    { 0x4fb1219b, SUC_NE_lexicon_tags_2 },
    { 0x5f0b219d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beb21a2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8c8d21a4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe71b21a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e521a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d121ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf0121ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c721af, SUC_NE_lexicon_tags_2 },
    { 0xd07121b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff21b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55521b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fc921ba, SUC_NE_lexicon_tags_2 },
    { 0x11dd21bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591521bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x682921c2, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df321c8, SUC_NE_lexicon_tags_2 },
    { 0x488321c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbf21cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5d21d3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe2721d5, SUC_NE_lexicon_tags_2 },
    { 0x4f8921d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe74f21d8, SUC_NE_lexicon_tags_13 },
    { 0x68c921d9, SUC_NE_lexicon_tags_2 },
    { 0xd35921da, SUC_NE_lexicon_tags_2 },
    { 0xe26921da, SUC_NE_lexicon_tags_2 },
    { 0x755521da, SUC_NE_lexicon_tags_2 },
    { 0x172d21dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ef21e1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6721e4, SUC_NE_lexicon_tags_2 },
    { 0x6b9721e5, SUC_NE_lexicon_tags_7 },
    { 0x476721e4, SUC_NE_lexicon_tags_2 },
    { 0x583921e7, SUC_NE_lexicon_tags_2 },
    { 0x2be321e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x350521ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x422d21ed, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x686521ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ad21f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d721f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28521f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadb21fd, SUC_NE_lexicon_tags_2 },
    { 0xc1af21fe, SUC_NE_lexicon_tags_2 },
    { 0x9f1321ff, SUC_NE_lexicon_tags_2 },
    { 0xfaef21fd, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x82912202, SUC_NE_lexicon_tags_2 },
    { 0x61652203, SUC_NE_lexicon_tags_2 },
    { 0x47df2204, SUC_NE_lexicon_tags_2 },
    { 0x81d12204, SUC_NE_lexicon_tags_2 },
    { 0x614b2205, SUC_NE_lexicon_tags_2 },
    { 0x3c6d2207, SUC_NE_lexicon_tags_2 },
    { 0x1f0b2208, SUC_NE_lexicon_tags_2 },
    { 0x9d3d2207, SUC_NE_lexicon_tags_2 },
    { 0x79212209, SUC_NE_lexicon_tags_2 },
    { 0x8eeb2208, SUC_NE_lexicon_tags_2 },
    { 0x3b3b220c, SUC_NE_lexicon_tags_1 },
    { 0x9949220c, SUC_NE_lexicon_tags_2 },
    { 0x6b05220c, SUC_NE_lexicon_tags_2 },
    { 0x5a7220f, SUC_NE_lexicon_tags_2 },
    { 0x2ed7220c, SUC_NE_lexicon_tags_430 },
    { 0xb2e52207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52372213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e9221a, SUC_NE_lexicon_tags_2 },
    { 0xf7b9221b, SUC_NE_lexicon_tags_2 },
    { 0x41b9221a, SUC_NE_lexicon_tags_2 },
    { 0x5c9f221d, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7f2220, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa852226, SUC_NE_lexicon_tags_2 },
    { 0x8c232227, SUC_NE_lexicon_tags_2 },
    { 0x70f72227, SUC_NE_lexicon_tags_19 },
    { 0xb59d2229, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92b222d, SUC_NE_lexicon_tags_2 },
    { 0xee89222e, SUC_NE_lexicon_tags_2 },
    { 0xda7b222f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf812231, SUC_NE_lexicon_tags_2 },
    { 0x9a732232, SUC_NE_lexicon_tags_3 },
    { 0x515f2233, SUC_NE_lexicon_tags_2 },
    { 0xba6f2234, SUC_NE_lexicon_tags_2 },
    { 0x2b252235, SUC_NE_lexicon_tags_2 },
    { 0x3ecb2233, SUC_NE_lexicon_tags_2 },
    { 0x14b52237, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297b223f, SUC_NE_lexicon_tags_2 },
    { 0xf603223f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92192246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10452249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e9224d, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb92257, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6472259, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbecb225b, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0x3113225d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x599b2260, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68f12262, SUC_NE_lexicon_tags_2 },
    { 0x3d832262, SUC_NE_lexicon_tags_102 },
    { 0x83af2262, SUC_NE_lexicon_tags_2 },
    { 0xa0cf2265, SUC_NE_lexicon_tags_2 },
    { 0x18892266, SUC_NE_lexicon_tags_2 },
    { 0xa7f12267, SUC_NE_lexicon_tags_2 },
    { 0x5152268, SUC_NE_lexicon_tags_1 },
    { 0x75f52269, SUC_NE_lexicon_tags_2 },
    { 0xaab226a, SUC_NE_lexicon_tags_2 },
    { 0xf7972269, SUC_NE_lexicon_tags_2 },
    { 0x733d2265, SUC_NE_lexicon_tags_2 },
    { 0x69212267, SUC_NE_lexicon_tags_2 },
    { 0xed97226e, SUC_NE_lexicon_tags_2 },
    { 0x6d95226e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e52274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca972277, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xbcdf2279, SUC_NE_lexicon_tags_2 },
    { 0x83012279, SUC_NE_lexicon_tags_4 },
    { 0xf9b5227a, SUC_NE_lexicon_tags_2 },
    { 0xac4b227c, SUC_NE_lexicon_tags_2 },
    { 0x56fb227b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4151227f, SUC_NE_lexicon_tags_2 },
    { 0x3f9b2280, SUC_NE_lexicon_tags_17 },
    { 0xd4c52280, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x837d2283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d32291, SUC_NE_lexicon_tags_2 },
    { 0xb2fb2292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71532294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79a92296, SUC_NE_lexicon_tags_2 },
    { 0x27a12297, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf29f2299, SUC_NE_lexicon_tags_2 },
    { 0xc94b229a, SUC_NE_lexicon_tags_2 },
    { 0x5179229b, SUC_NE_lexicon_tags_36 },
    { 0x7de5229c, SUC_NE_lexicon_tags_2 },
    { 0x7ef229c, SUC_NE_lexicon_tags_2 },
    { 0x67cf229e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e3522a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914522a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1ef22a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b122ab, SUC_NE_lexicon_tags_2 },
    { 0xf4b922ac, SUC_NE_lexicon_tags_2 },
    { 0xaeb922ad, SUC_NE_lexicon_tags_2 },
    { 0xdd6122ad, SUC_NE_lexicon_tags_1 },
    { 0x984322af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbd722b1, SUC_NE_lexicon_tags_2 },
    { 0x4c6322b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9c122b4, SUC_NE_lexicon_tags_2 },
    { 0xef7d22b5, SUC_NE_lexicon_tags_2 },
    { 0xc5cb22b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x718522b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd88722ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fe122bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62922c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fef22c3, SUC_NE_lexicon_tags_3 },
    { 0x8a8b22c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445122c8, SUC_NE_lexicon_tags_2 },
    { 0xbfcd22c9, SUC_NE_lexicon_tags_2 },
    { 0xcbd522ca, SUC_NE_lexicon_tags_2 },
    { 0xd7dd22c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457322d0, SUC_NE_lexicon_tags_2 },
    { 0x33722d0, SUC_NE_lexicon_tags_2 },
    { 0x281d22d2, SUC_NE_lexicon_tags_14 },
    { 0x391922d2, SUC_NE_lexicon_tags_13 },
    { 0x166922d2, SUC_NE_lexicon_tags_2 },
    { 0x6f7d22d5, SUC_NE_lexicon_tags_3 },
    { 0x3d1922d6, SUC_NE_lexicon_tags_2 },
    { 0x690b22d7, SUC_NE_lexicon_tags_2 },
    { 0x5e6922d4, SUC_NE_lexicon_tags_2 },
    { 0x74522d4, SUC_NE_lexicon_tags_3 },
    { 0xed3922d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88f522dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959322e0, SUC_NE_lexicon_tags_2 },
    { 0x57b322e1, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x201d22e4, SUC_NE_lexicon_tags_4 },
    { 0x4a8522e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x376922e7, SUC_NE_lexicon_tags_2 },
    { 0xd7a922e7, SUC_NE_lexicon_tags_3 },
    { 0x78dd22e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x348722ec, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0f522ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd84b22f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x568d22f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e522f6, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xfab22f8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x871922fb, SUC_NE_lexicon_tags_2 },
    { 0x24a322fc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa55b22fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a1d2300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x372d2302, SUC_NE_lexicon_tags_1 },
    { 0x8c3d2303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14372305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81b2309, SUC_NE_lexicon_tags_2 },
    { 0xb555230a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e09230c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f892313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a72316, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b192319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe623231c, SUC_NE_lexicon_tags_2 },
    { 0x473d231d, SUC_NE_lexicon_tags_2 },
    { 0xdebf231e, SUC_NE_lexicon_tags_2 },
    { 0xf607231f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb12323, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5ab2325, SUC_NE_lexicon_tags_17 },
    { 0x64c12326, SUC_NE_lexicon_tags_2 },
    { 0x8fa32325, SUC_NE_lexicon_tags_2 },
    { 0x6bd92328, SUC_NE_lexicon_tags_17 },
    { 0xb6ad2326, SUC_NE_lexicon_tags_2 },
    { 0x4127232a, SUC_NE_lexicon_tags_2 },
    { 0x47d1232a, SUC_NE_lexicon_tags_2 },
    { 0x4f55232b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x257b232f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48732331, SUC_NE_lexicon_tags_2 },
    { 0x51112331, SUC_NE_lexicon_tags_2 },
    { 0x93a92333, SUC_NE_lexicon_tags_2 },
    { 0x7d152333, SUC_NE_lexicon_tags_2 },
    { 0x15152335, SUC_NE_lexicon_tags_2 },
    { 0xa8452336, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x932f2339, SUC_NE_lexicon_tags_2 },
    { 0xf4d92339, SUC_NE_lexicon_tags_27 },
    { 0, NULL },
    { 0x753233c, SUC_NE_lexicon_tags_2 },
    { 0x2e9233d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf33233f, SUC_NE_lexicon_tags_2 },
    { 0x42ed2340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce132342, SUC_NE_lexicon_tags_2 },
    { 0xe9592343, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15a52345, SUC_NE_lexicon_tags_7 },
    { 0x8fb32346, SUC_NE_lexicon_tags_2 },
    { 0x2c492347, SUC_NE_lexicon_tags_2 },
    { 0x80472347, SUC_NE_lexicon_tags_2 },
    { 0x64e92347, SUC_NE_lexicon_tags_2 },
    { 0xe43d2345, SUC_NE_lexicon_tags_2 },
    { 0x90572346, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x168b234d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c592353, SUC_NE_lexicon_tags_2 },
    { 0x71412354, SUC_NE_lexicon_tags_2 },
    { 0xa3632354, SUC_NE_lexicon_tags_2 },
    { 0x1c3d2356, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ce92358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c79235a, SUC_NE_lexicon_tags_2 },
    { 0x642d235a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb17235d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa72360, SUC_NE_lexicon_tags_2 },
    { 0x34532361, SUC_NE_lexicon_tags_2 },
    { 0x2bf32362, SUC_NE_lexicon_tags_2 },
    { 0x5d912363, SUC_NE_lexicon_tags_2 },
    { 0xe2eb2360, SUC_NE_lexicon_tags_36 },
    { 0x19852364, SUC_NE_lexicon_tags_34 },
    { 0x9ff92360, SUC_NE_lexicon_tags_2 },
    { 0x5b4f2367, SUC_NE_lexicon_tags_2 },
    { 0x91232368, SUC_NE_lexicon_tags_18 },
    { 0x485b2365, SUC_NE_lexicon_tags_7 },
    { 0x8b31236a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x601b236d, SUC_NE_lexicon_tags_2 },
    { 0xf507236d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f32371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1092374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb152379, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45f237e, SUC_NE_lexicon_tags_2 },
    { 0xf165237f, SUC_NE_lexicon_tags_2 },
    { 0x59632380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea552382, SUC_NE_lexicon_tags_2 },
    { 0xf8d32383, SUC_NE_lexicon_tags_2 },
    { 0x15b32384, SUC_NE_lexicon_tags_8 },
    { 0x44cd2385, SUC_NE_lexicon_tags_2 },
    { 0x90bb2384, SUC_NE_lexicon_tags_2 },
    { 0x1f292384, SUC_NE_lexicon_tags_2 },
    { 0x8cd2388, SUC_NE_lexicon_tags_2 },
    { 0x139b2388, SUC_NE_lexicon_tags_2 },
    { 0x52cf2389, SUC_NE_lexicon_tags_2 },
    { 0xfef5238b, SUC_NE_lexicon_tags_2 },
    { 0x1f0f2389, SUC_NE_lexicon_tags_2 },
    { 0xba31238b, SUC_NE_lexicon_tags_2 },
    { 0x9997238e, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9912397, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd265239a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf577239e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60723a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947323ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20b23af, SUC_NE_lexicon_tags_2 },
    { 0xbb7323b0, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30323b3, SUC_NE_lexicon_tags_2 },
    { 0xcaa523b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5523b8, SUC_NE_lexicon_tags_4 },
    { 0x38a923b9, SUC_NE_lexicon_tags_2 },
    { 0xae7f23ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836723be, SUC_NE_lexicon_tags_34 },
    { 0x4a6323bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613923c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114f23ca, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9f23ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c323d0, SUC_NE_lexicon_tags_2 },
    { 0xbdd123d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4d23d6, SUC_NE_lexicon_tags_3 },
    { 0xf89b23d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6b23da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8723df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484d23e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e123e9, SUC_NE_lexicon_tags_2 },
    { 0x44d323e9, SUC_NE_lexicon_tags_2 },
    { 0x9a7723eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c4523ed, SUC_NE_lexicon_tags_2 },
    { 0x3a0523ed, SUC_NE_lexicon_tags_2 },
    { 0xcc6b23ee, SUC_NE_lexicon_tags_2 },
    { 0xd9ef23f0, SUC_NE_lexicon_tags_2 },
    { 0x1cfb23f1, SUC_NE_lexicon_tags_2 },
    { 0x8a3323f1, SUC_NE_lexicon_tags_49 },
    { 0xa71d23f1, SUC_NE_lexicon_tags_2 },
    { 0xd86123f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcb323f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c323f8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1b23fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35af23ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f12404, SUC_NE_lexicon_tags_19 },
    { 0x67f72405, SUC_NE_lexicon_tags_7 },
    { 0x7d1f2406, SUC_NE_lexicon_tags_2 },
    { 0x5bdf2406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23b92409, SUC_NE_lexicon_tags_2 },
    { 0x23e5240a, SUC_NE_lexicon_tags_2 },
    { 0xd269240b, SUC_NE_lexicon_tags_2 },
    { 0x2671240c, SUC_NE_lexicon_tags_2 },
    { 0x1209240c, SUC_NE_lexicon_tags_2 },
    { 0xdfcf240a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa252411, SUC_NE_lexicon_tags_2 },
    { 0x6b792412, SUC_NE_lexicon_tags_2 },
    { 0x87bf2413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91692415, SUC_NE_lexicon_tags_2 },
    { 0xc9e92416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x275b2418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f97241a, SUC_NE_lexicon_tags_2 },
    { 0xf0a3241a, SUC_NE_lexicon_tags_103 },
    { 0x6cc1241c, SUC_NE_lexicon_tags_257 },
    { 0x1799241b, SUC_NE_lexicon_tags_2 },
    { 0x1127241e, SUC_NE_lexicon_tags_2 },
    { 0x4a05241f, SUC_NE_lexicon_tags_2 },
    { 0xc09d2420, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0b2423, SUC_NE_lexicon_tags_2 },
    { 0xc3a52424, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cf2428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6469242a, SUC_NE_lexicon_tags_2 },
    { 0xdeef242b, SUC_NE_lexicon_tags_34 },
    { 0xc5e9242b, SUC_NE_lexicon_tags_2 },
    { 0x8dbb242c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915b2431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51492433, SUC_NE_lexicon_tags_2 },
    { 0x40012433, SUC_NE_lexicon_tags_18 },
    { 0x65cb2434, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ab2439, SUC_NE_lexicon_tags_2 },
    { 0x1ccb243a, SUC_NE_lexicon_tags_2 },
    { 0x4d7243b, SUC_NE_lexicon_tags_7 },
    { 0xec23243c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77752440, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73152446, SUC_NE_lexicon_tags_2 },
    { 0xfd9b2447, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3152452, SUC_NE_lexicon_tags_4 },
    { 0x51b2452, SUC_NE_lexicon_tags_2 },
    { 0x57cb2452, SUC_NE_lexicon_tags_2 },
    { 0x56452455, SUC_NE_lexicon_tags_36 },
    { 0xf5332455, SUC_NE_lexicon_tags_2 },
    { 0xa47f2455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5212459, SUC_NE_lexicon_tags_2 },
    { 0xde91245a, SUC_NE_lexicon_tags_2 },
    { 0x55112459, SUC_NE_lexicon_tags_2 },
    { 0xd2f1245c, SUC_NE_lexicon_tags_2 },
    { 0xd6d7245c, SUC_NE_lexicon_tags_2 },
    { 0xab8b245b, SUC_NE_lexicon_tags_2 },
    { 0xb87245b, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33b2462, SUC_NE_lexicon_tags_2 },
    { 0xd11f2463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75892465, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaef246a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b2f246c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb246f, SUC_NE_lexicon_tags_2 },
    { 0x8eb12470, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43112476, SUC_NE_lexicon_tags_40 },
    { 0x87352476, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146b247d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46c7247f, SUC_NE_lexicon_tags_2 },
    { 0xaa532480, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46272483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x552d2486, SUC_NE_lexicon_tags_2 },
    { 0xae1b2487, SUC_NE_lexicon_tags_0 },
    { 0x438f2486, SUC_NE_lexicon_tags_2 },
    { 0x49c92489, SUC_NE_lexicon_tags_2 },
    { 0xcddb2488, SUC_NE_lexicon_tags_3 },
    { 0x4a7248b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9512490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82d2495, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b7249b, SUC_NE_lexicon_tags_2 },
    { 0xe309249b, SUC_NE_lexicon_tags_2 },
    { 0xb93249d, SUC_NE_lexicon_tags_2 },
    { 0xdca5249b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365924a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x875b24a9, SUC_NE_lexicon_tags_1 },
    { 0xd3db24aa, SUC_NE_lexicon_tags_2 },
    { 0x758924ab, SUC_NE_lexicon_tags_2 },
    { 0xf98d24ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef924b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f324b4, SUC_NE_lexicon_tags_2 },
    { 0x472524b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b5b24b7, SUC_NE_lexicon_tags_2 },
    { 0x9f7324b7, SUC_NE_lexicon_tags_2 },
    { 0x358324b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0d24be, SUC_NE_lexicon_tags_2 },
    { 0x871924be, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b724c3, SUC_NE_lexicon_tags_2 },
    { 0x8d6924c4, SUC_NE_lexicon_tags_2 },
    { 0x2efb24c5, SUC_NE_lexicon_tags_2 },
    { 0xd50924c5, SUC_NE_lexicon_tags_7 },
    { 0x928724c6, SUC_NE_lexicon_tags_2 },
    { 0x394f24c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b24cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85cf24cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf12124d1, SUC_NE_lexicon_tags_2 },
    { 0xc1bf24d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6d24d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x517724d9, SUC_NE_lexicon_tags_0 },
    { 0x88e524d9, SUC_NE_lexicon_tags_2 },
    { 0x614524db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9d24df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d324e2, SUC_NE_lexicon_tags_2 },
    { 0x998524e3, SUC_NE_lexicon_tags_2 },
    { 0xcae924e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfee924e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e724ef, SUC_NE_lexicon_tags_2 },
    { 0x58cf24f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203f24f7, SUC_NE_lexicon_tags_2 },
    { 0x146f24f8, SUC_NE_lexicon_tags_2 },
    { 0xf24724f9, SUC_NE_lexicon_tags_2 },
    { 0xaaa924f9, SUC_NE_lexicon_tags_2 },
    { 0xbd8b24fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49324fd, SUC_NE_lexicon_tags_2 },
    { 0xeaa524fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13692502, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7250c, SUC_NE_lexicon_tags_2 },
    { 0xef97250d, SUC_NE_lexicon_tags_2 },
    { 0xaa95250e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44fd2510, SUC_NE_lexicon_tags_2 },
    { 0xa89b2511, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9992516, SUC_NE_lexicon_tags_2 },
    { 0x46f32517, SUC_NE_lexicon_tags_2 },
    { 0x725d2518, SUC_NE_lexicon_tags_2 },
    { 0xc0fd2519, SUC_NE_lexicon_tags_2 },
    { 0x840d251a, SUC_NE_lexicon_tags_2 },
    { 0x40952516, SUC_NE_lexicon_tags_2 },
    { 0x89b9251c, SUC_NE_lexicon_tags_2 },
    { 0xf765251d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f2520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23032524, SUC_NE_lexicon_tags_2 },
    { 0xb7252525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da3252a, SUC_NE_lexicon_tags_2 },
    { 0xc15252b, SUC_NE_lexicon_tags_4 },
    { 0x2f09252b, SUC_NE_lexicon_tags_2 },
    { 0xc571252b, SUC_NE_lexicon_tags_2 },
    { 0xa64d252c, SUC_NE_lexicon_tags_2 },
    { 0xe6cf252f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ed2532, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3b2535, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6373253b, SUC_NE_lexicon_tags_2 },
    { 0x2f8b253c, SUC_NE_lexicon_tags_2 },
    { 0x7701253d, SUC_NE_lexicon_tags_2 },
    { 0x90c9253e, SUC_NE_lexicon_tags_2 },
    { 0x1ce5253f, SUC_NE_lexicon_tags_2 },
    { 0xf35253c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71a52542, SUC_NE_lexicon_tags_2 },
    { 0x17af2543, SUC_NE_lexicon_tags_1 },
    { 0xf6252542, SUC_NE_lexicon_tags_2 },
    { 0xabe32545, SUC_NE_lexicon_tags_7 },
    { 0x536b2546, SUC_NE_lexicon_tags_2 },
    { 0x4cdf2545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4d254a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6931254e, SUC_NE_lexicon_tags_2 },
    { 0x7a1b254f, SUC_NE_lexicon_tags_109 },
    { 0xbc03254e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7512552, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c32555, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11092557, SUC_NE_lexicon_tags_2 },
    { 0xd4492558, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b1255c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7af255e, SUC_NE_lexicon_tags_2 },
    { 0x31bb255e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2792563, SUC_NE_lexicon_tags_4 },
    { 0x49ad2564, SUC_NE_lexicon_tags_2 },
    { 0x84332565, SUC_NE_lexicon_tags_2 },
    { 0xc2152566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0c32568, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3f2570, SUC_NE_lexicon_tags_2 },
    { 0xa1a72571, SUC_NE_lexicon_tags_1 },
    { 0x24a12572, SUC_NE_lexicon_tags_2 },
    { 0xea592571, SUC_NE_lexicon_tags_2 },
    { 0xab772574, SUC_NE_lexicon_tags_2 },
    { 0x6c7f2575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79912577, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb257b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8381257f, SUC_NE_lexicon_tags_2 },
    { 0xdb1d257f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31432582, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7392584, SUC_NE_lexicon_tags_2 },
    { 0x8b892585, SUC_NE_lexicon_tags_2 },
    { 0x16092584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de92589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd258f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40f2592, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d2598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44f3259a, SUC_NE_lexicon_tags_2 },
    { 0xfb3259a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b87259d, SUC_NE_lexicon_tags_2 },
    { 0xf51d259e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaf25a1, SUC_NE_lexicon_tags_2 },
    { 0x14bf25a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9df25a7, SUC_NE_lexicon_tags_2 },
    { 0x5d2125a7, SUC_NE_lexicon_tags_2 },
    { 0x398325a7, SUC_NE_lexicon_tags_2 },
    { 0x283d25aa, SUC_NE_lexicon_tags_2 },
    { 0x66a125ab, SUC_NE_lexicon_tags_2 },
    { 0x62e925ab, SUC_NE_lexicon_tags_2 },
    { 0x5bd725ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6325b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x835d25b4, SUC_NE_lexicon_tags_2 },
    { 0x8d5325b5, SUC_NE_lexicon_tags_2 },
    { 0xde125b6, SUC_NE_lexicon_tags_2 },
    { 0x6b3725b6, SUC_NE_lexicon_tags_3 },
    { 0xa84325b7, SUC_NE_lexicon_tags_2 },
    { 0xe21925b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcd25bc, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6925c1, SUC_NE_lexicon_tags_2 },
    { 0x254925c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x864925c5, SUC_NE_lexicon_tags_2 },
    { 0x340d25c5, SUC_NE_lexicon_tags_3 },
    { 0xb43325c7, SUC_NE_lexicon_tags_3 },
    { 0x70d325c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d125cc, SUC_NE_lexicon_tags_2 },
    { 0x636325cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15a325cf, SUC_NE_lexicon_tags_2 },
    { 0x1d5125d0, SUC_NE_lexicon_tags_2 },
    { 0xb84725d0, SUC_NE_lexicon_tags_2 },
    { 0x9be925d2, SUC_NE_lexicon_tags_2 },
    { 0x565b25d2, SUC_NE_lexicon_tags_2 },
    { 0xf87725d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8525d7, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe925da, SUC_NE_lexicon_tags_2 },
    { 0xfd8125da, SUC_NE_lexicon_tags_2 },
    { 0x38c325dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60125df, SUC_NE_lexicon_tags_2 },
    { 0x84f25e0, SUC_NE_lexicon_tags_2 },
    { 0xa09d25e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x481b25e3, SUC_NE_lexicon_tags_2 },
    { 0x1b5b25e4, SUC_NE_lexicon_tags_2 },
    { 0x655b25e5, SUC_NE_lexicon_tags_188 },
    { 0x8b725e6, SUC_NE_lexicon_tags_2 },
    { 0x907725e7, SUC_NE_lexicon_tags_1 },
    { 0x226725e7, SUC_NE_lexicon_tags_2 },
    { 0xab7f25e9, SUC_NE_lexicon_tags_4 },
    { 0xc6db25e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78525ed, SUC_NE_lexicon_tags_2 },
    { 0x544725ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a125f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1f25f3, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545925fc, SUC_NE_lexicon_tags_2 },
    { 0x155325fd, SUC_NE_lexicon_tags_2 },
    { 0xbb5325fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8da92600, SUC_NE_lexicon_tags_175 },
    { 0x80992601, SUC_NE_lexicon_tags_2 },
    { 0x38e72602, SUC_NE_lexicon_tags_2 },
    { 0xaadb2603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb1b2605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47352607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x158d2609, SUC_NE_lexicon_tags_2 },
    { 0xeca1260a, SUC_NE_lexicon_tags_2 },
    { 0xa8fd260a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f9260e, SUC_NE_lexicon_tags_3 },
    { 0xe3fd260f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf012613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54b2619, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3eb261b, SUC_NE_lexicon_tags_2 },
    { 0xd46b261c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b7261f, SUC_NE_lexicon_tags_2 },
    { 0x8f8d2620, SUC_NE_lexicon_tags_2 },
    { 0xf9e5261f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x726f2624, SUC_NE_lexicon_tags_2 },
    { 0xe2e32625, SUC_NE_lexicon_tags_58 },
    { 0x99972626, SUC_NE_lexicon_tags_3 },
    { 0x62ad2627, SUC_NE_lexicon_tags_2 },
    { 0xaa0b2628, SUC_NE_lexicon_tags_2 },
    { 0xde572628, SUC_NE_lexicon_tags_2 },
    { 0xb8c3262a, SUC_NE_lexicon_tags_2 },
    { 0x37fd262b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35b262f, SUC_NE_lexicon_tags_2 },
    { 0x8af7262f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc32633, SUC_NE_lexicon_tags_0 },
    { 0x5ef72633, SUC_NE_lexicon_tags_2 },
    { 0x5bf12635, SUC_NE_lexicon_tags_2 },
    { 0x8c752635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95cb2638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfc7263a, SUC_NE_lexicon_tags_2 },
    { 0x9457263b, SUC_NE_lexicon_tags_2 },
    { 0xed35263a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb12644, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320d264d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c72650, SUC_NE_lexicon_tags_2 },
    { 0x2efb2651, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ff2655, SUC_NE_lexicon_tags_2 },
    { 0xa12d2655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a3265a, SUC_NE_lexicon_tags_2 },
    { 0xf6f5265b, SUC_NE_lexicon_tags_2 },
    { 0xf139265a, SUC_NE_lexicon_tags_2 },
    { 0x3bbb265a, SUC_NE_lexicon_tags_2 },
    { 0xcb17265a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ed2663, SUC_NE_lexicon_tags_7 },
    { 0xcf7f2663, SUC_NE_lexicon_tags_2 },
    { 0xa7532664, SUC_NE_lexicon_tags_2 },
    { 0x13892665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x718d2668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f9266d, SUC_NE_lexicon_tags_19 },
    { 0x5357266e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95452676, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce92679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9351267b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f9267d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325b2682, SUC_NE_lexicon_tags_2 },
    { 0x4bcb2683, SUC_NE_lexicon_tags_2 },
    { 0xcee52684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59c92686, SUC_NE_lexicon_tags_2 },
    { 0xb8b32687, SUC_NE_lexicon_tags_3 },
    { 0xe10f2688, SUC_NE_lexicon_tags_24 },
    { 0x73812689, SUC_NE_lexicon_tags_2 },
    { 0x96e52687, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b1268e, SUC_NE_lexicon_tags_2 },
    { 0x83b1268f, SUC_NE_lexicon_tags_2 },
    { 0xf3e52690, SUC_NE_lexicon_tags_2 },
    { 0x5f512691, SUC_NE_lexicon_tags_2 },
    { 0x8c0f268f, SUC_NE_lexicon_tags_2 },
    { 0x89a52691, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x888f2696, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6207269b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7587269e, SUC_NE_lexicon_tags_449 },
    { 0x785b269f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d26a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a326ac, SUC_NE_lexicon_tags_24 },
    { 0x7d2926ac, SUC_NE_lexicon_tags_7 },
    { 0x3c9926ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d26b4, SUC_NE_lexicon_tags_2 },
    { 0x6ed926b4, SUC_NE_lexicon_tags_2 },
    { 0x9d4326b6, SUC_NE_lexicon_tags_2 },
    { 0x369326b7, SUC_NE_lexicon_tags_2 },
    { 0x1f9726b7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9ecb26ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b26bd, SUC_NE_lexicon_tags_2 },
    { 0x8f8b26be, SUC_NE_lexicon_tags_2 },
    { 0x534f26be, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x825126c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40f26c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdecd26c7, SUC_NE_lexicon_tags_2 },
    { 0x1d9f26c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4ef26ca, SUC_NE_lexicon_tags_2 },
    { 0x40c126ca, SUC_NE_lexicon_tags_4 },
    { 0x790726cc, SUC_NE_lexicon_tags_2 },
    { 0xc70726cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d26d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f726d3, SUC_NE_lexicon_tags_2 },
    { 0x39926d3, SUC_NE_lexicon_tags_2 },
    { 0xc25526d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe05326d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8d26da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6126dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed26e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf126ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecf26f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45f126f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7726f6, SUC_NE_lexicon_tags_2 },
    { 0x140126f6, SUC_NE_lexicon_tags_2 },
    { 0x33cb26f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb26fd, SUC_NE_lexicon_tags_2 },
    { 0x1e6326fe, SUC_NE_lexicon_tags_2 },
    { 0x94c126ff, SUC_NE_lexicon_tags_2 },
    { 0x491726ff, SUC_NE_lexicon_tags_2 },
    { 0xe1f2701, SUC_NE_lexicon_tags_19 },
    { 0x9bd926ff, SUC_NE_lexicon_tags_2 },
    { 0xe3dd26fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf772707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa583270b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320f2710, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xb27f2712, SUC_NE_lexicon_tags_2 },
    { 0x1f0d2713, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ad2716, SUC_NE_lexicon_tags_2 },
    { 0xf7732716, SUC_NE_lexicon_tags_2 },
    { 0xd0cf2718, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a59271a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3817271e, SUC_NE_lexicon_tags_7 },
    { 0x9781271e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6172721, SUC_NE_lexicon_tags_4 },
    { 0xe69b2722, SUC_NE_lexicon_tags_2 },
    { 0xac232723, SUC_NE_lexicon_tags_2 },
    { 0x93232724, SUC_NE_lexicon_tags_2 },
    { 0xb7e72725, SUC_NE_lexicon_tags_2 },
    { 0xbb792723, SUC_NE_lexicon_tags_2 },
    { 0xdb8b2727, SUC_NE_lexicon_tags_2 },
    { 0xdbdd2725, SUC_NE_lexicon_tags_2 },
    { 0x96752727, SUC_NE_lexicon_tags_4 },
    { 0x856d2723, SUC_NE_lexicon_tags_2 },
    { 0x3c55272b, SUC_NE_lexicon_tags_2 },
    { 0xd553272b, SUC_NE_lexicon_tags_17 },
    { 0x9d9272d, SUC_NE_lexicon_tags_2 },
    { 0x24d272e, SUC_NE_lexicon_tags_2 },
    { 0x9d75272f, SUC_NE_lexicon_tags_2 },
    { 0xaeb32730, SUC_NE_lexicon_tags_2 },
    { 0xd04b272c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74152734, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac15273c, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0x4243273e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bdf2740, SUC_NE_lexicon_tags_2 },
    { 0x906d2741, SUC_NE_lexicon_tags_2 },
    { 0x16f2742, SUC_NE_lexicon_tags_2 },
    { 0x605b2742, SUC_NE_lexicon_tags_2 },
    { 0xf6992742, SUC_NE_lexicon_tags_2 },
    { 0xd2772742, SUC_NE_lexicon_tags_2 },
    { 0xa0272746, SUC_NE_lexicon_tags_2 },
    { 0x918b2747, SUC_NE_lexicon_tags_4 },
    { 0x591b2748, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x682b274a, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x49f3274c, SUC_NE_lexicon_tags_2 },
    { 0x47f5274c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54492752, SUC_NE_lexicon_tags_2 },
    { 0x7c092753, SUC_NE_lexicon_tags_7 },
    { 0xd0392753, SUC_NE_lexicon_tags_2 },
    { 0x50d92754, SUC_NE_lexicon_tags_2 },
    { 0x517f2756, SUC_NE_lexicon_tags_3 },
    { 0x2bb52752, SUC_NE_lexicon_tags_2 },
    { 0x7b832755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4d275b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52532760, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50132764, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x513d2766, SUC_NE_lexicon_tags_2 },
    { 0x69d2766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5849276c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaf276f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b352771, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b1b2773, SUC_NE_lexicon_tags_2 },
    { 0x1df12774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf32779, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b23277c, SUC_NE_lexicon_tags_2 },
    { 0xde41277d, SUC_NE_lexicon_tags_2 },
    { 0xe7df277c, SUC_NE_lexicon_tags_2 },
    { 0xc071277f, SUC_NE_lexicon_tags_2 },
    { 0xbdfd2780, SUC_NE_lexicon_tags_2 },
    { 0xff692781, SUC_NE_lexicon_tags_2 },
    { 0xab172780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d32785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcea52787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa31f2789, SUC_NE_lexicon_tags_2 },
    { 0xaf2d2789, SUC_NE_lexicon_tags_2 },
    { 0x785b2789, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba7278f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87ff2791, SUC_NE_lexicon_tags_2 },
    { 0xab9b2792, SUC_NE_lexicon_tags_7 },
    { 0x21df2792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cbf2795, SUC_NE_lexicon_tags_2 },
    { 0x66c52796, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559f279e, SUC_NE_lexicon_tags_2 },
    { 0x281f279f, SUC_NE_lexicon_tags_2 },
    { 0xa42d27a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1bf27a2, SUC_NE_lexicon_tags_56 },
    { 0x872327a3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x941127a6, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42127af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6327b2, SUC_NE_lexicon_tags_2 },
    { 0xfbad27b2, SUC_NE_lexicon_tags_2 },
    { 0xa05d27b4, SUC_NE_lexicon_tags_2 },
    { 0xc8f327b5, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511d27ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ff27be, SUC_NE_lexicon_tags_2 },
    { 0xe0e727bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb327c3, SUC_NE_lexicon_tags_2 },
    { 0x16e527c4, SUC_NE_lexicon_tags_2 },
    { 0xdc3727c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84bf27c7, SUC_NE_lexicon_tags_2 },
    { 0x2d1f27c8, SUC_NE_lexicon_tags_2 },
    { 0x6e5b27c9, SUC_NE_lexicon_tags_2 },
    { 0x7f4f27c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4527cc, SUC_NE_lexicon_tags_2 },
    { 0x3c1727cc, SUC_NE_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0x180927d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88a127d2, SUC_NE_lexicon_tags_2 },
    { 0x5c3b27d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccd27d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x757b27d7, SUC_NE_lexicon_tags_2 },
    { 0xa05127d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e527df, SUC_NE_lexicon_tags_2 },
    { 0x8f4f27e0, SUC_NE_lexicon_tags_14 },
    { 0x4fdf27e0, SUC_NE_lexicon_tags_2 },
    { 0x37a527e2, SUC_NE_lexicon_tags_17 },
    { 0x331527e3, SUC_NE_lexicon_tags_2 },
    { 0x351527df, SUC_NE_lexicon_tags_2 },
    { 0xe04727e5, SUC_NE_lexicon_tags_2 },
    { 0x434127e6, SUC_NE_lexicon_tags_2 },
    { 0x8b2d27e3, SUC_NE_lexicon_tags_2 },
    { 0x9e8927e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x283727ea, SUC_NE_lexicon_tags_2 },
    { 0x2ba927eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb26327ed, SUC_NE_lexicon_tags_3 },
    { 0xad727ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191527f2, SUC_NE_lexicon_tags_2 },
    { 0x5c8127f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd72127f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb94927f7, SUC_NE_lexicon_tags_2 },
    { 0x104d27f8, SUC_NE_lexicon_tags_2 },
    { 0x461327f7, SUC_NE_lexicon_tags_2 },
    { 0x9d4527fa, SUC_NE_lexicon_tags_2 },
    { 0x85e127fb, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0xbb1127fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa727ff, SUC_NE_lexicon_tags_4 },
    { 0xc6792800, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ef2807, SUC_NE_lexicon_tags_2 },
    { 0x3afb2807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xead1280a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90092810, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xae6f2812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65f72814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617281a, SUC_NE_lexicon_tags_2 },
    { 0x3fd7281b, SUC_NE_lexicon_tags_2 },
    { 0xa5b7281c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15912820, SUC_NE_lexicon_tags_3 },
    { 0xe0d72821, SUC_NE_lexicon_tags_2 },
    { 0xa9992822, SUC_NE_lexicon_tags_2 },
    { 0x876b2823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d2827, SUC_NE_lexicon_tags_17 },
    { 0x3a472827, SUC_NE_lexicon_tags_2 },
    { 0x36bd2829, SUC_NE_lexicon_tags_2 },
    { 0x4b112827, SUC_NE_lexicon_tags_7 },
    { 0xe921282b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeb282f, SUC_NE_lexicon_tags_2 },
    { 0xbbed282f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45b2833, SUC_NE_lexicon_tags_2 },
    { 0x43072834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91d72836, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eab283b, SUC_NE_lexicon_tags_2 },
    { 0x16d283b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90752844, SUC_NE_lexicon_tags_2 },
    { 0x9c712845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24152848, SUC_NE_lexicon_tags_2 },
    { 0xe73d2848, SUC_NE_lexicon_tags_2 },
    { 0xc0f9284a, SUC_NE_lexicon_tags_5 },
    { 0x917284a, SUC_NE_lexicon_tags_2 },
    { 0x49ed284b, SUC_NE_lexicon_tags_2 },
    { 0xaa7b284d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2707284f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe32856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1285a, SUC_NE_lexicon_tags_2 },
    { 0x1e4f285a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d99285e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac52862, SUC_NE_lexicon_tags_2 },
    { 0x3e9d2862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552d2869, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec01286c, SUC_NE_lexicon_tags_2 },
    { 0x5c7b286d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bf2870, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb73f2872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b012874, SUC_NE_lexicon_tags_2 },
    { 0x7e8b2875, SUC_NE_lexicon_tags_5 },
    { 0xcfe52876, SUC_NE_lexicon_tags_2 },
    { 0x10572876, SUC_NE_lexicon_tags_2 },
    { 0xc0092878, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b9287e, SUC_NE_lexicon_tags_2 },
    { 0x409287f, SUC_NE_lexicon_tags_2 },
    { 0x69e52880, SUC_NE_lexicon_tags_19 },
    { 0x33792881, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92312885, SUC_NE_lexicon_tags_2 },
    { 0xb58b2886, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cd288c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa629288e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d12892, SUC_NE_lexicon_tags_7 },
    { 0x56092893, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4d2896, SUC_NE_lexicon_tags_2 },
    { 0xd7252897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf903289c, SUC_NE_lexicon_tags_2 },
    { 0xc75b289d, SUC_NE_lexicon_tags_2 },
    { 0xbc7f289d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa67d28a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbf28a3, SUC_NE_lexicon_tags_7 },
    { 0xa3728a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1928a7, SUC_NE_lexicon_tags_2 },
    { 0xf4cb28a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9d528aa, SUC_NE_lexicon_tags_2 },
    { 0x3cd328aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x508f28ad, SUC_NE_lexicon_tags_7 },
    { 0xeef28ae, SUC_NE_lexicon_tags_2 },
    { 0xeb6d28ad, SUC_NE_lexicon_tags_2 },
    { 0x6d5b28ad, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x742728b3, SUC_NE_lexicon_tags_2 },
    { 0x5afd28b3, SUC_NE_lexicon_tags_2 },
    { 0xba1728b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99128b8, SUC_NE_lexicon_tags_2 },
    { 0xf44128b9, SUC_NE_lexicon_tags_5 },
    { 0xd92928ba, SUC_NE_lexicon_tags_2 },
    { 0xf39f28b9, SUC_NE_lexicon_tags_1 },
    { 0x330b28bb, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x264928be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cd128c0, SUC_NE_lexicon_tags_2 },
    { 0x8cdf28c1, SUC_NE_lexicon_tags_11 },
    { 0xffcd28c2, SUC_NE_lexicon_tags_2 },
    { 0x4f3128c2, SUC_NE_lexicon_tags_2 },
    { 0x8e0f28c4, SUC_NE_lexicon_tags_2 },
    { 0xb92928c3, SUC_NE_lexicon_tags_2 },
    { 0xefe328c6, SUC_NE_lexicon_tags_2 },
    { 0xe0d528c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc52d28c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdb28cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8b28d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1f28d6, SUC_NE_lexicon_tags_2 },
    { 0x7ec128d6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xac1f28d9, SUC_NE_lexicon_tags_2 },
    { 0x8e3d28da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23af28dd, SUC_NE_lexicon_tags_2 },
    { 0xfe2128de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5928e1, SUC_NE_lexicon_tags_4 },
    { 0xac3328e1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x410d28e4, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663d28e8, SUC_NE_lexicon_tags_2 },
    { 0xea5928e9, SUC_NE_lexicon_tags_2 },
    { 0x342d28ea, SUC_NE_lexicon_tags_2 },
    { 0x46b928eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b328ef, SUC_NE_lexicon_tags_3 },
    { 0xeab528f0, SUC_NE_lexicon_tags_2 },
    { 0x7d4728f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e728f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f328f9, SUC_NE_lexicon_tags_2 },
    { 0x5a2328fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8128fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd2901, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf392903, SUC_NE_lexicon_tags_2 },
    { 0x1d12904, SUC_NE_lexicon_tags_2 },
    { 0x5ef32904, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f290c, SUC_NE_lexicon_tags_2 },
    { 0x53d5290d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc249290f, SUC_NE_lexicon_tags_2 },
    { 0x90752910, SUC_NE_lexicon_tags_2 },
    { 0x243f2911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18f52913, SUC_NE_lexicon_tags_2 },
    { 0x31992914, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x7f432916, SUC_NE_lexicon_tags_2 },
    { 0x96472916, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f8b2919, SUC_NE_lexicon_tags_2 },
    { 0x52dd2919, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3851291d, SUC_NE_lexicon_tags_2 },
    { 0x5f83291e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dab2925, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf92928, SUC_NE_lexicon_tags_2 },
    { 0xb87f2929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd9292c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4212933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e052937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e9293b, SUC_NE_lexicon_tags_2 },
    { 0x8a63293b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893b294c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa891294f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd692953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b3b2955, SUC_NE_lexicon_tags_2 },
    { 0xdd552956, SUC_NE_lexicon_tags_153 },
    { 0, NULL },
    { 0, NULL },
    { 0x59632959, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca3295e, SUC_NE_lexicon_tags_17 },
    { 0x4cd1295f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d72963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9672965, SUC_NE_lexicon_tags_4 },
    { 0x16232966, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b712968, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e89296a, SUC_NE_lexicon_tags_2 },
    { 0xb20b296b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa3296f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcd2972, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d52977, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x699d297a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4593297c, SUC_NE_lexicon_tags_2 },
    { 0xef5f297d, SUC_NE_lexicon_tags_2 },
    { 0x5e11297e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8272980, SUC_NE_lexicon_tags_2 },
    { 0x83792981, SUC_NE_lexicon_tags_2 },
    { 0xb3832981, SUC_NE_lexicon_tags_2 },
    { 0xd312983, SUC_NE_lexicon_tags_2 },
    { 0xa0b72982, SUC_NE_lexicon_tags_17 },
    { 0xc2572985, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51a32987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3852989, SUC_NE_lexicon_tags_2 },
    { 0xbc5d298a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35892991, SUC_NE_lexicon_tags_2 },
    { 0x634b2992, SUC_NE_lexicon_tags_2 },
    { 0xb29d2993, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e92999, SUC_NE_lexicon_tags_2 },
    { 0x73552999, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc483299c, SUC_NE_lexicon_tags_2 },
    { 0xadc9299c, SUC_NE_lexicon_tags_2 },
    { 0x107299e, SUC_NE_lexicon_tags_2 },
    { 0x2425299e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42729a2, SUC_NE_lexicon_tags_2 },
    { 0xbedd29a2, SUC_NE_lexicon_tags_2 },
    { 0x3c3d29a4, SUC_NE_lexicon_tags_2 },
    { 0x9e1b29a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f5f29a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9529ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538b29b0, SUC_NE_lexicon_tags_2 },
    { 0xfec929b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb25f29b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5329b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed4929b8, SUC_NE_lexicon_tags_2 },
    { 0xd01129b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x182729bc, SUC_NE_lexicon_tags_2 },
    { 0xcf0f29bd, SUC_NE_lexicon_tags_2 },
    { 0xaf7d29be, SUC_NE_lexicon_tags_17 },
    { 0x702729be, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe929c6, SUC_NE_lexicon_tags_2 },
    { 0x384729c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x669329c9, SUC_NE_lexicon_tags_4 },
    { 0x646329ca, SUC_NE_lexicon_tags_2 },
    { 0x83d29cb, SUC_NE_lexicon_tags_2 },
    { 0xe34b29c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f329d1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbf9129d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a529d6, SUC_NE_lexicon_tags_81 },
    { 0x8b8b29d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a129dc, SUC_NE_lexicon_tags_262 },
    { 0xe0a729dc, SUC_NE_lexicon_tags_2 },
    { 0x636129dd, SUC_NE_lexicon_tags_4 },
    { 0x696b29dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4d929e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff29e7, SUC_NE_lexicon_tags_2 },
    { 0x991729e8, SUC_NE_lexicon_tags_2 },
    { 0xb5b729e7, SUC_NE_lexicon_tags_2 },
    { 0xc2ad29e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99b529ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74529ef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716329f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ded29f7, SUC_NE_lexicon_tags_2 },
    { 0x9baf29f8, SUC_NE_lexicon_tags_2 },
    { 0x86e129f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x344b29fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x725329fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55172a01, SUC_NE_lexicon_tags_2 },
    { 0x6e372a02, SUC_NE_lexicon_tags_1 },
    { 0x3bf2a03, SUC_NE_lexicon_tags_0 },
    { 0x28612a01, SUC_NE_lexicon_tags_2 },
    { 0x472a05, SUC_NE_lexicon_tags_2 },
    { 0x48892a06, SUC_NE_lexicon_tags_2 },
    { 0x9c692a03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd3f2a09, SUC_NE_lexicon_tags_2 },
    { 0xc3672a09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36712a0f, SUC_NE_lexicon_tags_2 },
    { 0x30a72a0f, SUC_NE_lexicon_tags_2 },
    { 0xad8f2a11, SUC_NE_lexicon_tags_2 },
    { 0x712d2a12, SUC_NE_lexicon_tags_2 },
    { 0xd7512a13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea52a17, SUC_NE_lexicon_tags_2 },
    { 0xd59b2a18, SUC_NE_lexicon_tags_2 },
    { 0x47d72a18, SUC_NE_lexicon_tags_2 },
    { 0x1edd2a17, SUC_NE_lexicon_tags_18 },
    { 0xce6b2a18, SUC_NE_lexicon_tags_2 },
    { 0xb1b72a1c, SUC_NE_lexicon_tags_0 },
    { 0xaa8d2a1d, SUC_NE_lexicon_tags_2 },
    { 0x6d932a1b, SUC_NE_lexicon_tags_2 },
    { 0x58bd2a1f, SUC_NE_lexicon_tags_8 },
    { 0xb03b2a20, SUC_NE_lexicon_tags_2 },
    { 0xd5872a18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1432a23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d52a2a, SUC_NE_lexicon_tags_2 },
    { 0xa1392a2b, SUC_NE_lexicon_tags_2 },
    { 0xb2192a2c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e892a31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd992a35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf2a39, SUC_NE_lexicon_tags_2 },
    { 0x47ef2a39, SUC_NE_lexicon_tags_87 },
    { 0x5d7d2a39, SUC_NE_lexicon_tags_2 },
    { 0xf83b2a3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed2a40, SUC_NE_lexicon_tags_2 },
    { 0xe7112a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29992a44, SUC_NE_lexicon_tags_2 },
    { 0xd14d2a44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491d2a4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6912a4d, SUC_NE_lexicon_tags_2 },
    { 0x60512a4e, SUC_NE_lexicon_tags_2 },
    { 0x5f1b2a4f, SUC_NE_lexicon_tags_2 },
    { 0x3e512a50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55b2a52, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfed12a54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc12a57, SUC_NE_lexicon_tags_2 },
    { 0x37892a58, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x3ab12a5a, SUC_NE_lexicon_tags_2 },
    { 0x45832a5b, SUC_NE_lexicon_tags_18 },
    { 0x217d2a5b, SUC_NE_lexicon_tags_2 },
    { 0xd612a5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac92a62, SUC_NE_lexicon_tags_0 },
    { 0xfdf32a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64372a65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x199b2a68, SUC_NE_lexicon_tags_2 },
    { 0x9a0d2a68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb2a6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8492a71, SUC_NE_lexicon_tags_2 },
    { 0xd8232a71, SUC_NE_lexicon_tags_2 },
    { 0x1d8b2a71, SUC_NE_lexicon_tags_2 },
    { 0x73e12a72, SUC_NE_lexicon_tags_2 },
    { 0xf7e92a75, SUC_NE_lexicon_tags_2 },
    { 0x6ca32a75, SUC_NE_lexicon_tags_2 },
    { 0xd5032a76, SUC_NE_lexicon_tags_2 },
    { 0x3a632a71, SUC_NE_lexicon_tags_2 },
    { 0x4ee92a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75cd2a7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78ad2a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df2a80, SUC_NE_lexicon_tags_2 },
    { 0x6852a81, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x892b2a83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd392a87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9b2a8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dd2a93, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf09b2a95, SUC_NE_lexicon_tags_1 },
    { 0x107f2a96, SUC_NE_lexicon_tags_49 },
    { 0x7db2a95, SUC_NE_lexicon_tags_2 },
    { 0x2b952a96, SUC_NE_lexicon_tags_2 },
    { 0xe7ff2a96, SUC_NE_lexicon_tags_2 },
    { 0xe7d92a97, SUC_NE_lexicon_tags_2 },
    { 0x6b712a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4b2a9e, SUC_NE_lexicon_tags_2 },
    { 0x64d32a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65df2aa2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47372aa9, SUC_NE_lexicon_tags_7 },
    { 0xe7532aaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd052aad, SUC_NE_lexicon_tags_32 },
    { 0xae452aae, SUC_NE_lexicon_tags_2 },
    { 0x23032aae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1192ab5, SUC_NE_lexicon_tags_2 },
    { 0xf5692ab6, SUC_NE_lexicon_tags_2 },
    { 0xb6c12ab5, SUC_NE_lexicon_tags_2 },
    { 0x582f2ab5, SUC_NE_lexicon_tags_2 },
    { 0xa6b12ab9, SUC_NE_lexicon_tags_7 },
    { 0xbc672ab8, SUC_NE_lexicon_tags_2 },
    { 0x42712abb, SUC_NE_lexicon_tags_3 },
    { 0x4332abc, SUC_NE_lexicon_tags_2 },
    { 0x12892abc, SUC_NE_lexicon_tags_2 },
    { 0xc45d2abe, SUC_NE_lexicon_tags_2 },
    { 0x3db12abe, SUC_NE_lexicon_tags_2 },
    { 0xd20d2abf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4d2ac3, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7f2ac7, SUC_NE_lexicon_tags_2 },
    { 0x94132ac7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfd2acb, SUC_NE_lexicon_tags_2 },
    { 0x4fd72acb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ebd2ace, SUC_NE_lexicon_tags_2 },
    { 0xe6a72ace, SUC_NE_lexicon_tags_4 },
    { 0x2d1b2ad0, SUC_NE_lexicon_tags_1 },
    { 0x87fd2ad1, SUC_NE_lexicon_tags_2 },
    { 0xe32d2ad2, SUC_NE_lexicon_tags_2 },
    { 0x19112ad3, SUC_NE_lexicon_tags_2 },
    { 0xf2452ad4, SUC_NE_lexicon_tags_2 },
    { 0xa2872ad5, SUC_NE_lexicon_tags_3 },
    { 0xbcb72ad4, SUC_NE_lexicon_tags_2 },
    { 0xc3c92acf, SUC_NE_lexicon_tags_2 },
    { 0x3b452ad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc32adb, SUC_NE_lexicon_tags_2 },
    { 0x63ad2adc, SUC_NE_lexicon_tags_2 },
    { 0x60492add, SUC_NE_lexicon_tags_2 },
    { 0xe3cd2add, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fd2ae2, SUC_NE_lexicon_tags_3 },
    { 0x480f2ae3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbed2ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe32ae9, SUC_NE_lexicon_tags_2 },
    { 0xb2092aea, SUC_NE_lexicon_tags_2 },
    { 0xf9292aeb, SUC_NE_lexicon_tags_2 },
    { 0x30cb2aec, SUC_NE_lexicon_tags_2 },
    { 0x3812aeb, SUC_NE_lexicon_tags_2 },
    { 0xcc892aea, SUC_NE_lexicon_tags_2 },
    { 0x7e3d2aee, SUC_NE_lexicon_tags_19 },
    { 0x1a5b2af0, SUC_NE_lexicon_tags_4 },
    { 0xb2332af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36872af4, SUC_NE_lexicon_tags_2 },
    { 0x6cb32af5, SUC_NE_lexicon_tags_2 },
    { 0x5df92af6, SUC_NE_lexicon_tags_34 },
    { 0x6c52af6, SUC_NE_lexicon_tags_2 },
    { 0x5ebb2af8, SUC_NE_lexicon_tags_2 },
    { 0x82fb2af8, SUC_NE_lexicon_tags_2 },
    { 0x3572af5, SUC_NE_lexicon_tags_2 },
    { 0xc9c32afb, SUC_NE_lexicon_tags_2 },
    { 0x8cc12af4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x538b2aff, SUC_NE_lexicon_tags_2 },
    { 0x46ab2b00, SUC_NE_lexicon_tags_2 },
    { 0x468f2b01, SUC_NE_lexicon_tags_3 },
    { 0x210f2b01, SUC_NE_lexicon_tags_2 },
    { 0xe332b01, SUC_NE_lexicon_tags_2 },
    { 0x3feb2b04, SUC_NE_lexicon_tags_69 },
    { 0xa55f2b05, SUC_NE_lexicon_tags_2 },
    { 0x3cb52b05, SUC_NE_lexicon_tags_2 },
    { 0x878f2b05, SUC_NE_lexicon_tags_2 },
    { 0x4d312b08, SUC_NE_lexicon_tags_7 },
    { 0x7fad2b08, SUC_NE_lexicon_tags_2 },
    { 0xc8012b0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa32b0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba12b12, SUC_NE_lexicon_tags_7 },
    { 0x44732b13, SUC_NE_lexicon_tags_2 },
    { 0xa73b2b12, SUC_NE_lexicon_tags_2 },
    { 0xd5412b15, SUC_NE_lexicon_tags_1 },
    { 0x142d2b16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71312b18, SUC_NE_lexicon_tags_2 },
    { 0x1e7f2b19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d472b1b, SUC_NE_lexicon_tags_2 },
    { 0xe5352b1b, SUC_NE_lexicon_tags_2 },
    { 0x94192b1d, SUC_NE_lexicon_tags_2 },
    { 0xa3952b1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaf2b21, SUC_NE_lexicon_tags_2 },
    { 0x48d32b22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da72b29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2392b2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8232b32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c992b38, SUC_NE_lexicon_tags_2 },
    { 0x852d2b39, SUC_NE_lexicon_tags_2 },
    { 0xff392b3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10872b3c, SUC_NE_lexicon_tags_2 },
    { 0x3c952b3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7f2b42, SUC_NE_lexicon_tags_2 },
    { 0x1ff92b43, SUC_NE_lexicon_tags_2 },
    { 0xca732b43, SUC_NE_lexicon_tags_2 },
    { 0x5d312b45, SUC_NE_lexicon_tags_2 },
    { 0xcb452b46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9092b4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e72b4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f2b50, SUC_NE_lexicon_tags_2 },
    { 0x2b0f2b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec812b56, SUC_NE_lexicon_tags_2 },
    { 0x59772b57, SUC_NE_lexicon_tags_2 },
    { 0x23f92b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf2b5c, SUC_NE_lexicon_tags_2 },
    { 0xfb452b5c, SUC_NE_lexicon_tags_2 },
    { 0xcbf2b5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7b2b63, SUC_NE_lexicon_tags_7 },
    { 0x92b63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e712b67, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc12b6e, SUC_NE_lexicon_tags_2 },
    { 0xcecb2b6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e92b72, SUC_NE_lexicon_tags_19 },
    { 0xdb3d2b73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28292b75, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e12b7f, SUC_NE_lexicon_tags_1 },
    { 0xfcc72b80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ff2b82, SUC_NE_lexicon_tags_2 },
    { 0x6e032b83, SUC_NE_lexicon_tags_2 },
    { 0xe9ad2b84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367f2b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc32b8f, SUC_NE_lexicon_tags_2 },
    { 0x34e52b90, SUC_NE_lexicon_tags_2 },
    { 0x97f72b8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf38d2b93, SUC_NE_lexicon_tags_7 },
    { 0xee812b94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f32b97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe52b9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53ab2b9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b852ba0, SUC_NE_lexicon_tags_2 },
    { 0xff0d2ba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc832ba8, SUC_NE_lexicon_tags_2 },
    { 0x4b72ba8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xabe52bab, SUC_NE_lexicon_tags_2 },
    { 0x3b812bac, SUC_NE_lexicon_tags_4 },
    { 0xe6492bad, SUC_NE_lexicon_tags_2 },
    { 0x87512bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacc72bb0, SUC_NE_lexicon_tags_2 },
    { 0xcf2f2bb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9312bb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed952bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x684f2bb8, SUC_NE_lexicon_tags_2 },
    { 0xae2d2bb9, SUC_NE_lexicon_tags_2 },
    { 0x4e372bb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6f2bc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1a72bc2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x470b2bc4, SUC_NE_lexicon_tags_2 },
    { 0x73f52bc5, SUC_NE_lexicon_tags_2 },
    { 0x8052bc6, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a72bcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81852bcf, SUC_NE_lexicon_tags_2 },
    { 0x120d2bd0, SUC_NE_lexicon_tags_2 },
    { 0x4da72bd1, SUC_NE_lexicon_tags_2 },
    { 0x9d152bd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f52bd5, SUC_NE_lexicon_tags_2 },
    { 0x65992bd6, SUC_NE_lexicon_tags_2 },
    { 0x3a152bd6, SUC_NE_lexicon_tags_2 },
    { 0xdb3b2bd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9732bda, SUC_NE_lexicon_tags_7 },
    { 0x81872bda, SUC_NE_lexicon_tags_2 },
    { 0xcb272bdc, SUC_NE_lexicon_tags_2 },
    { 0x9b072bdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc972be1, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66352be4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88752be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba7b2be8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afb2bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f72bef, SUC_NE_lexicon_tags_2 },
    { 0xa30f2bef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2552bf2, SUC_NE_lexicon_tags_2 },
    { 0x95f12bf3, SUC_NE_lexicon_tags_2 },
    { 0x1032bf4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f72bf8, SUC_NE_lexicon_tags_2 },
    { 0xd25d2bf8, SUC_NE_lexicon_tags_92 },
    { 0x74872bfa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x3cf72bfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef12bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dd52c00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e232c04, SUC_NE_lexicon_tags_2 },
    { 0x4d252c04, SUC_NE_lexicon_tags_2 },
    { 0x4e3f2c06, SUC_NE_lexicon_tags_2 },
    { 0x884d2c07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef32c0a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x40072c0c, SUC_NE_lexicon_tags_2 },
    { 0xb0a32c0d, SUC_NE_lexicon_tags_0 },
    { 0xd1132c0e, SUC_NE_lexicon_tags_2 },
    { 0x95112c0d, SUC_NE_lexicon_tags_2 },
    { 0x3ad72c10, SUC_NE_lexicon_tags_2 },
    { 0x2012c11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12db2c15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74292c17, SUC_NE_lexicon_tags_2 },
    { 0xdd492c18, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e12c1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cd2c1e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4732c21, SUC_NE_lexicon_tags_2 },
    { 0xe4692c22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d92c26, SUC_NE_lexicon_tags_2 },
    { 0x29152c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7dd2c29, SUC_NE_lexicon_tags_2 },
    { 0x49d2c2a, SUC_NE_lexicon_tags_2 },
    { 0x3552c2b, SUC_NE_lexicon_tags_2 },
    { 0x3e912c2b, SUC_NE_lexicon_tags_18 },
    { 0x513f2c2d, SUC_NE_lexicon_tags_2 },
    { 0x71652c2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2c34, SUC_NE_lexicon_tags_2 },
    { 0x9e232c34, SUC_NE_lexicon_tags_2 },
    { 0x1f512c35, SUC_NE_lexicon_tags_87 },
    { 0xc06f2c35, SUC_NE_lexicon_tags_2 },
    { 0xf5812c38, SUC_NE_lexicon_tags_2 },
    { 0x9e392c39, SUC_NE_lexicon_tags_2 },
    { 0x1a752c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb772c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d92c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12112c53, SUC_NE_lexicon_tags_2 },
    { 0x1c292c54, SUC_NE_lexicon_tags_2 },
    { 0xa992c55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b72c59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0112c5d, SUC_NE_lexicon_tags_2 },
    { 0x66612c5e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e012c63, SUC_NE_lexicon_tags_2 },
    { 0x856f2c64, SUC_NE_lexicon_tags_7 },
    { 0xff4f2c65, SUC_NE_lexicon_tags_2 },
    { 0x56f12c64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86072c68, SUC_NE_lexicon_tags_2 },
    { 0xd5332c68, SUC_NE_lexicon_tags_2 },
    { 0x8c432c6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3092c6f, SUC_NE_lexicon_tags_2 },
    { 0x98432c70, SUC_NE_lexicon_tags_2 },
    { 0xba792c70, SUC_NE_lexicon_tags_2 },
    { 0xdbed2c72, SUC_NE_lexicon_tags_2 },
    { 0xbaf92c73, SUC_NE_lexicon_tags_2 },
    { 0x95f72c72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98052c76, SUC_NE_lexicon_tags_2 },
    { 0x9ead2c77, SUC_NE_lexicon_tags_2 },
    { 0x9cb32c77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c472c7a, SUC_NE_lexicon_tags_2 },
    { 0x948b2c7b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bab2c7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64b2c80, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x47bb2c82, SUC_NE_lexicon_tags_2 },
    { 0x39dd2c83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1b2c88, SUC_NE_lexicon_tags_1 },
    { 0xb2392c89, SUC_NE_lexicon_tags_2 },
    { 0x1b8d2c89, SUC_NE_lexicon_tags_11 },
    { 0x62b12c8b, SUC_NE_lexicon_tags_2 },
    { 0xfe332c8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6112c8e, SUC_NE_lexicon_tags_2 },
    { 0xa8432c8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b2c92, SUC_NE_lexicon_tags_2 },
    { 0x766b2c92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ff52c95, SUC_NE_lexicon_tags_56 },
    { 0xc8a92c95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x326d2c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbf2c9e, SUC_NE_lexicon_tags_2 },
    { 0x879d2c9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c072ca2, SUC_NE_lexicon_tags_49 },
    { 0x36c32ca2, SUC_NE_lexicon_tags_2 },
    { 0x99172ca4, SUC_NE_lexicon_tags_0 },
    { 0x73032ca2, SUC_NE_lexicon_tags_2 },
    { 0x93cd2ca6, SUC_NE_lexicon_tags_2 },
    { 0x52d12ca7, SUC_NE_lexicon_tags_13 },
    { 0x91fb2ca8, SUC_NE_lexicon_tags_2 },
    { 0x51ad2ca8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b92cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bd2cb6, SUC_NE_lexicon_tags_2 },
    { 0x8f832cb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fff2cbb, SUC_NE_lexicon_tags_4 },
    { 0xbd552cbc, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42572cc0, SUC_NE_lexicon_tags_2 },
    { 0x85cb2cc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f52cc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ea12cc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8612ccb, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0bd2ccd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda432cd0, SUC_NE_lexicon_tags_2 },
    { 0x3c512cd0, SUC_NE_lexicon_tags_2 },
    { 0x600f2cd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x972d2cd4, SUC_NE_lexicon_tags_34 },
    { 0x4e652cd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4312cd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d032cda, SUC_NE_lexicon_tags_2 },
    { 0xd8ef2cdb, SUC_NE_lexicon_tags_2 },
    { 0xd01d2cdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae72ce0, SUC_NE_lexicon_tags_2 },
    { 0x97492ce1, SUC_NE_lexicon_tags_2 },
    { 0xa7c32ce1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3672ce6, SUC_NE_lexicon_tags_2 },
    { 0x22632ce7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6372ce9, SUC_NE_lexicon_tags_2 },
    { 0x6a172cea, SUC_NE_lexicon_tags_2 },
    { 0x2ac32cea, SUC_NE_lexicon_tags_2 },
    { 0x2d012cec, SUC_NE_lexicon_tags_2 },
    { 0xd52cec, SUC_NE_lexicon_tags_2 },
    { 0xb8d52cee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43d2cf7, SUC_NE_lexicon_tags_2 },
    { 0xe3092cf8, SUC_NE_lexicon_tags_2 },
    { 0xa7f72cf8, SUC_NE_lexicon_tags_2 },
    { 0xc7f2cf7, SUC_NE_lexicon_tags_9 },
    { 0x324d2cfb, SUC_NE_lexicon_tags_128 },
    { 0x3b552cfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27252d02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5a92d04, SUC_NE_lexicon_tags_3 },
    { 0x1f712d04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e12d09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3492d0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac6b2d0e, SUC_NE_lexicon_tags_7 },
    { 0xcdd92d0f, SUC_NE_lexicon_tags_2 },
    { 0xfc0d2d10, SUC_NE_lexicon_tags_4 },
    { 0xc98d2d11, SUC_NE_lexicon_tags_2 },
    { 0xaf5b2d12, SUC_NE_lexicon_tags_2 },
    { 0xa74b2d11, SUC_NE_lexicon_tags_2 },
    { 0xb24f2d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53332d17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44f2d19, SUC_NE_lexicon_tags_2 },
    { 0x17bb2d1a, SUC_NE_lexicon_tags_2 },
    { 0x6fd52d19, SUC_NE_lexicon_tags_2 },
    { 0xb3612d19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab2d24, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b7f2d26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82d2d29, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f332d2c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d92d31, SUC_NE_lexicon_tags_2 },
    { 0xf3672d32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x572d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b952d36, SUC_NE_lexicon_tags_3 },
    { 0x86a92d37, SUC_NE_lexicon_tags_2 },
    { 0x42332d38, SUC_NE_lexicon_tags_11 },
    { 0x459b2d36, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb92d3c, SUC_NE_lexicon_tags_2 },
    { 0x2e152d3d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de12d40, SUC_NE_lexicon_tags_2 },
    { 0xe9852d40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b1b2d43, SUC_NE_lexicon_tags_2 },
    { 0x764d2d44, SUC_NE_lexicon_tags_2 },
    { 0x88a92d45, SUC_NE_lexicon_tags_2 },
    { 0x65972d44, SUC_NE_lexicon_tags_2 },
    { 0x64c32d45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad92d50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55452d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa52d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14fd2d5d, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcd2d60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bd2d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fd2d67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81bb2d69, SUC_NE_lexicon_tags_3 },
    { 0x11b72d69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e6f2d6c, SUC_NE_lexicon_tags_2 },
    { 0x45352d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b2d70, SUC_NE_lexicon_tags_2 },
    { 0x477f2d70, SUC_NE_lexicon_tags_2 },
    { 0x27b12d72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba652d75, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf12d7a, SUC_NE_lexicon_tags_18 },
    { 0x8a9b2d7b, SUC_NE_lexicon_tags_2 },
    { 0x15612d7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90272d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca072d87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9f2d8a, SUC_NE_lexicon_tags_2 },
    { 0x38432d8a, SUC_NE_lexicon_tags_2 },
    { 0x6a252d8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e112d8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b2d96, SUC_NE_lexicon_tags_0 },
    { 0x57c72d97, SUC_NE_lexicon_tags_2 },
    { 0xfc8d2d97, SUC_NE_lexicon_tags_2 },
    { 0x7b1b2d99, SUC_NE_lexicon_tags_2 },
    { 0x12952d99, SUC_NE_lexicon_tags_2 },
    { 0xa5452d9a, SUC_NE_lexicon_tags_2 },
    { 0xe9dd2d9c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0332da1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb32da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcee72da8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac652daa, SUC_NE_lexicon_tags_5 },
    { 0x76a12dab, SUC_NE_lexicon_tags_2 },
    { 0x16ef2dac, SUC_NE_lexicon_tags_2 },
    { 0x9c2f2dad, SUC_NE_lexicon_tags_2 },
    { 0x6df2dae, SUC_NE_lexicon_tags_2 },
    { 0x348d2daa, SUC_NE_lexicon_tags_0 },
    { 0x9d432db0, SUC_NE_lexicon_tags_3 },
    { 0x11cf2dab, SUC_NE_lexicon_tags_2 },
    { 0x31772db2, SUC_NE_lexicon_tags_2 },
    { 0x8aa12db3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f912db5, SUC_NE_lexicon_tags_2 },
    { 0xdc272db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa72dbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67332dbe, SUC_NE_lexicon_tags_2 },
    { 0x73b12dbe, SUC_NE_lexicon_tags_2 },
    { 0xbd172dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a3f2dc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177f2dc6, SUC_NE_lexicon_tags_2 },
    { 0x50472dc6, SUC_NE_lexicon_tags_2 },
    { 0x3872dc6, SUC_NE_lexicon_tags_2 },
    { 0x81b52dc9, SUC_NE_lexicon_tags_2 },
    { 0xda3f2dca, SUC_NE_lexicon_tags_2 },
    { 0x7772dcb, SUC_NE_lexicon_tags_13 },
    { 0xadc32dc9, SUC_NE_lexicon_tags_2 },
    { 0xc2e52dca, SUC_NE_lexicon_tags_2 },
    { 0xbe672dcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cab2dd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1292dd2, SUC_NE_lexicon_tags_2 },
    { 0x57a32dd2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd99f2dd5, SUC_NE_lexicon_tags_2 },
    { 0xf3b12dd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46912dda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e592de0, SUC_NE_lexicon_tags_2 },
    { 0x3e092de1, SUC_NE_lexicon_tags_2 },
    { 0xa552de2, SUC_NE_lexicon_tags_3 },
    { 0x723f2de3, SUC_NE_lexicon_tags_2 },
    { 0x5acb2de4, SUC_NE_lexicon_tags_2 },
    { 0x3ed92de1, SUC_NE_lexicon_tags_2 },
    { 0xca552de0, SUC_NE_lexicon_tags_2 },
    { 0x15572de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8b32de9, SUC_NE_lexicon_tags_5 },
    { 0x44432dea, SUC_NE_lexicon_tags_2 },
    { 0x4f9d2dea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58b72ded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4af2dfa, SUC_NE_lexicon_tags_2 },
    { 0x6f092dfa, SUC_NE_lexicon_tags_2 },
    { 0x38e52dfc, SUC_NE_lexicon_tags_18 },
    { 0xbd7d2dfc, SUC_NE_lexicon_tags_4 },
    { 0x31632dfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecb52e00, SUC_NE_lexicon_tags_2 },
    { 0x676b2e01, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0b2e04, SUC_NE_lexicon_tags_2 },
    { 0x7a452e04, SUC_NE_lexicon_tags_2 },
    { 0xc5632e06, SUC_NE_lexicon_tags_2 },
    { 0x26f92e06, SUC_NE_lexicon_tags_2 },
    { 0x24b92e08, SUC_NE_lexicon_tags_2 },
    { 0xf1c32e09, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x6dc92e0b, SUC_NE_lexicon_tags_2 },
    { 0x37192e0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b672e12, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61b2e16, SUC_NE_lexicon_tags_1 },
    { 0xf1e92e17, SUC_NE_lexicon_tags_2 },
    { 0xc9372e17, SUC_NE_lexicon_tags_2 },
    { 0x99f52e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c72e21, SUC_NE_lexicon_tags_2 },
    { 0x39dd2e21, SUC_NE_lexicon_tags_2 },
    { 0xd9c52e23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24352e25, SUC_NE_lexicon_tags_7 },
    { 0x67ab2e26, SUC_NE_lexicon_tags_2 },
    { 0x27092e27, SUC_NE_lexicon_tags_2 },
    { 0x5b472e28, SUC_NE_lexicon_tags_2 },
    { 0x209b2e29, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d52e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f2e35, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xd0612e37, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7712e3b, SUC_NE_lexicon_tags_2 },
    { 0xdc6f2e3c, SUC_NE_lexicon_tags_2 },
    { 0xf2d72e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3372e3f, SUC_NE_lexicon_tags_1 },
    { 0xa47b2e40, SUC_NE_lexicon_tags_2 },
    { 0xd8512e41, SUC_NE_lexicon_tags_2 },
    { 0x4b952e42, SUC_NE_lexicon_tags_2 },
    { 0xd3472e41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25c32e45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf252e4a, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x783d2e4d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b2e51, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c92e59, SUC_NE_lexicon_tags_2 },
    { 0x71b32e59, SUC_NE_lexicon_tags_1 },
    { 0x5d792e5b, SUC_NE_lexicon_tags_2 },
    { 0xc0712e5c, SUC_NE_lexicon_tags_2 },
    { 0x352d2e5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d2e60, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84912e63, SUC_NE_lexicon_tags_2 },
    { 0x443d2e64, SUC_NE_lexicon_tags_2 },
    { 0x26ad2e65, SUC_NE_lexicon_tags_2 },
    { 0xb7912e66, SUC_NE_lexicon_tags_2 },
    { 0xd9452e66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd952e6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf2e6f, SUC_NE_lexicon_tags_2 },
    { 0x789f2e70, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e72e73, SUC_NE_lexicon_tags_7 },
    { 0x91052e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edb2e81, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ad72e83, SUC_NE_lexicon_tags_2 },
    { 0x4a412e84, SUC_NE_lexicon_tags_2 },
    { 0xeff32e85, SUC_NE_lexicon_tags_2 },
    { 0xb4f12e86, SUC_NE_lexicon_tags_2 },
    { 0x7b4f2e87, SUC_NE_lexicon_tags_18 },
    { 0xcf632e86, SUC_NE_lexicon_tags_2 },
    { 0x41152e88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda52e8c, SUC_NE_lexicon_tags_2 },
    { 0x805d2e8d, SUC_NE_lexicon_tags_2 },
    { 0xe7992e8c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0212e91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b92e94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a72e9a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ae52e9c, SUC_NE_lexicon_tags_2 },
    { 0x32632e9d, SUC_NE_lexicon_tags_2 },
    { 0x2e152e9e, SUC_NE_lexicon_tags_7 },
    { 0x8abd2e9f, SUC_NE_lexicon_tags_2 },
    { 0xb01b2e9e, SUC_NE_lexicon_tags_2 },
    { 0xa0f52e9e, SUC_NE_lexicon_tags_2 },
    { 0x12a52ea2, SUC_NE_lexicon_tags_2 },
    { 0x8d2ea0, SUC_NE_lexicon_tags_2 },
    { 0xb2752e9f, SUC_NE_lexicon_tags_2 },
    { 0x52b72e9e, SUC_NE_lexicon_tags_2 },
    { 0x3bf72e9f, SUC_NE_lexicon_tags_2 },
    { 0x7c952e9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9752ea9, SUC_NE_lexicon_tags_2 },
    { 0xa1c12eaa, SUC_NE_lexicon_tags_2 },
    { 0x828d2eaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61712ead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d72eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc06b2eb3, SUC_NE_lexicon_tags_2 },
    { 0x15452eb4, SUC_NE_lexicon_tags_2 },
    { 0xb712eb4, SUC_NE_lexicon_tags_2 },
    { 0xade12eb4, SUC_NE_lexicon_tags_2 },
    { 0xc0772eb7, SUC_NE_lexicon_tags_2 },
    { 0x6f152eb8, SUC_NE_lexicon_tags_2 },
    { 0xae032eb9, SUC_NE_lexicon_tags_2 },
    { 0xa3a72eba, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed72ebd, SUC_NE_lexicon_tags_2 },
    { 0x26a72ebe, SUC_NE_lexicon_tags_2 },
    { 0xed212ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523d2ec6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb132ec8, SUC_NE_lexicon_tags_52 },
    { 0, NULL },
    { 0x2e712eca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5f2ecd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d2ed2, SUC_NE_lexicon_tags_2 },
    { 0x3a752ed3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x616f2ed5, SUC_NE_lexicon_tags_2 },
    { 0x42cb2ed6, SUC_NE_lexicon_tags_17 },
    { 0x466f2ed6, SUC_NE_lexicon_tags_2 },
    { 0xdcf92ed6, SUC_NE_lexicon_tags_2 },
    { 0xe3bf2ed6, SUC_NE_lexicon_tags_2 },
    { 0x36af2ed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb2ede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d932ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c52eec, SUC_NE_lexicon_tags_2 },
    { 0x36bb2eed, SUC_NE_lexicon_tags_2 },
    { 0xa1d72eee, SUC_NE_lexicon_tags_2 },
    { 0x350f2eef, SUC_NE_lexicon_tags_2 },
    { 0x6c32eee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5112ef2, SUC_NE_lexicon_tags_2 },
    { 0xcd2f2ef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x612f2ef6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8392ef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc52efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729b2f03, SUC_NE_lexicon_tags_2 },
    { 0x90472f04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf152f06, SUC_NE_lexicon_tags_2 },
    { 0x1d692f07, SUC_NE_lexicon_tags_2 },
    { 0x59cf2f06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe072f0a, SUC_NE_lexicon_tags_19 },
    { 0xc1132f0b, SUC_NE_lexicon_tags_2 },
    { 0xd3292f0c, SUC_NE_lexicon_tags_2 },
    { 0x5cb2f0d, SUC_NE_lexicon_tags_2 },
    { 0x2b6b2f0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63b2f12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12192f16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf2b2f18, SUC_NE_lexicon_tags_2 },
    { 0x3e092f19, SUC_NE_lexicon_tags_2 },
    { 0xcb132f1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f2f1d, SUC_NE_lexicon_tags_2 },
    { 0x805d2f1e, SUC_NE_lexicon_tags_2 },
    { 0x4c5d2f1e, SUC_NE_lexicon_tags_2 },
    { 0x17492f20, SUC_NE_lexicon_tags_2 },
    { 0x5ef92f20, SUC_NE_lexicon_tags_2 },
    { 0xf7f32f1d, SUC_NE_lexicon_tags_2 },
    { 0xf3192f23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae32f26, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89532f2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4d2f30, SUC_NE_lexicon_tags_2 },
    { 0x956f2f31, SUC_NE_lexicon_tags_2 },
    { 0x80452f32, SUC_NE_lexicon_tags_2 },
    { 0x40f12f33, SUC_NE_lexicon_tags_2 },
    { 0xbf432f33, SUC_NE_lexicon_tags_2 },
    { 0x8c012f31, SUC_NE_lexicon_tags_2 },
    { 0xa3532f36, SUC_NE_lexicon_tags_2 },
    { 0x66412f33, SUC_NE_lexicon_tags_2 },
    { 0x60192f38, SUC_NE_lexicon_tags_2 },
    { 0xafe52f39, SUC_NE_lexicon_tags_2 },
    { 0x42912f3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb212f3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6b72f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdb2f43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32f52f45, SUC_NE_lexicon_tags_3 },
    { 0x6f672f45, SUC_NE_lexicon_tags_4 },
    { 0x6ab92f45, SUC_NE_lexicon_tags_2 },
    { 0xcdff2f47, SUC_NE_lexicon_tags_2 },
    { 0x6a472f49, SUC_NE_lexicon_tags_2 },
    { 0xaae32f4a, SUC_NE_lexicon_tags_2 },
    { 0x7be12f4b, SUC_NE_lexicon_tags_2 },
    { 0xfef92f4c, SUC_NE_lexicon_tags_2 },
    { 0x88892f4c, SUC_NE_lexicon_tags_26 },
    { 0xa03f2f4e, SUC_NE_lexicon_tags_2 },
    { 0xa1a92f4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93012f53, SUC_NE_lexicon_tags_2 },
    { 0xff492f53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed512f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f52f5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a132f61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb12f69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65332f6f, SUC_NE_lexicon_tags_2 },
    { 0xd3052f70, SUC_NE_lexicon_tags_2 },
    { 0x26612f71, SUC_NE_lexicon_tags_2 },
    { 0x52c32f72, SUC_NE_lexicon_tags_2 },
    { 0xfdf72f73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9532f75, SUC_NE_lexicon_tags_2 },
    { 0x45652f76, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x97cb2f78, SUC_NE_lexicon_tags_2 },
    { 0x302f2f79, SUC_NE_lexicon_tags_2 },
    { 0xacc72f7a, SUC_NE_lexicon_tags_17 },
    { 0x61cd2f79, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf9092f7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xceaf2f7f, SUC_NE_lexicon_tags_2 },
    { 0x9a592f80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15d2f83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d2f89, SUC_NE_lexicon_tags_2 },
    { 0xc992f8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c692f8e, SUC_NE_lexicon_tags_49 },
    { 0x11852f8f, SUC_NE_lexicon_tags_2 },
    { 0x85752f90, SUC_NE_lexicon_tags_2 },
    { 0x28852f8f, SUC_NE_lexicon_tags_2 },
    { 0x69332f90, SUC_NE_lexicon_tags_2 },
    { 0x55b32f93, SUC_NE_lexicon_tags_2 },
    { 0x3cf92f94, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x952f96, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0352f9b, SUC_NE_lexicon_tags_2 },
    { 0xda352f9c, SUC_NE_lexicon_tags_2 },
    { 0xcff72f9d, SUC_NE_lexicon_tags_3 },
    { 0xff8d2f9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74152fa0, SUC_NE_lexicon_tags_2 },
    { 0x69672fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8972fa5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e32fac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cbd2fae, SUC_NE_lexicon_tags_2 },
    { 0x5a352faf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36232fb1, SUC_NE_lexicon_tags_4 },
    { 0xcc732fb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ed52fb4, SUC_NE_lexicon_tags_2 },
    { 0x4e8b2fb5, SUC_NE_lexicon_tags_2 },
    { 0xfbef2fb6, SUC_NE_lexicon_tags_2 },
    { 0x12472fb7, SUC_NE_lexicon_tags_3 },
    { 0xab4d2fb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43072fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf792fbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ab2fc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ff2fc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb2fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde032fd5, SUC_NE_lexicon_tags_3 },
    { 0xea3d2fd6, SUC_NE_lexicon_tags_2 },
    { 0x8d6b2fd7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9ef2fd9, SUC_NE_lexicon_tags_2 },
    { 0xb8e12fd9, SUC_NE_lexicon_tags_2 },
    { 0x3d512fda, SUC_NE_lexicon_tags_2 },
    { 0x13292fdc, SUC_NE_lexicon_tags_2 },
    { 0xfcd12fdd, SUC_NE_lexicon_tags_2 },
    { 0xab92fde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b52fe3, SUC_NE_lexicon_tags_2 },
    { 0x48512fe4, SUC_NE_lexicon_tags_2 },
    { 0xfee92fe4, SUC_NE_lexicon_tags_2 },
    { 0x10872fe5, SUC_NE_lexicon_tags_2 },
    { 0xbed92fe7, SUC_NE_lexicon_tags_2 },
    { 0x63cd2fe7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb212fea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a52ff1, SUC_NE_lexicon_tags_2 },
    { 0x52d2ff1, SUC_NE_lexicon_tags_2 },
    { 0xb3f52ff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba52ff6, SUC_NE_lexicon_tags_2 },
    { 0xad5b2ff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1d2ffa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40b2ffc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x62d12ffe, SUC_NE_lexicon_tags_2 },
    { 0xc172ffe, SUC_NE_lexicon_tags_2 },
    { 0x8a2b3000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33473003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc273005, SUC_NE_lexicon_tags_2 },
    { 0x44793006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9533008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4e9300a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae99300e, SUC_NE_lexicon_tags_2 },
    { 0xd0e7300f, SUC_NE_lexicon_tags_2 },
    { 0x99f53010, SUC_NE_lexicon_tags_2 },
    { 0xc3613010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a13018, SUC_NE_lexicon_tags_2 },
    { 0x5a1b3019, SUC_NE_lexicon_tags_7 },
    { 0x227b301a, SUC_NE_lexicon_tags_0 },
    { 0xad81301b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e39301d, SUC_NE_lexicon_tags_2 },
    { 0xa065301e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34073022, SUC_NE_lexicon_tags_5 },
    { 0xb7c33022, SUC_NE_lexicon_tags_2 },
    { 0x3e873024, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bb302b, SUC_NE_lexicon_tags_2 },
    { 0x7609302c, SUC_NE_lexicon_tags_2 },
    { 0x9427302d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a833030, SUC_NE_lexicon_tags_2 },
    { 0x3953031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd53034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66d3037, SUC_NE_lexicon_tags_2 },
    { 0xf4b3037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ca1303a, SUC_NE_lexicon_tags_2 },
    { 0x11fd303a, SUC_NE_lexicon_tags_2 },
    { 0x2f2b303c, SUC_NE_lexicon_tags_2 },
    { 0x9e67303a, SUC_NE_lexicon_tags_2 },
    { 0x18d1303e, SUC_NE_lexicon_tags_2 },
    { 0x1d63303d, SUC_NE_lexicon_tags_2 },
    { 0x52493040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c53044, SUC_NE_lexicon_tags_2 },
    { 0xbe33045, SUC_NE_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c13048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41b304a, SUC_NE_lexicon_tags_3 },
    { 0x7473304b, SUC_NE_lexicon_tags_2 },
    { 0x980d304c, SUC_NE_lexicon_tags_2 },
    { 0x56ed304d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8bd304f, SUC_NE_lexicon_tags_2 },
    { 0x5c9d3050, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9493053, SUC_NE_lexicon_tags_2 },
    { 0xa2873054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69373056, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55f3059, SUC_NE_lexicon_tags_2 },
    { 0xe7e73059, SUC_NE_lexicon_tags_2 },
    { 0x4ef5305a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x8d7305d, SUC_NE_lexicon_tags_2 },
    { 0x7ab1305d, SUC_NE_lexicon_tags_2 },
    { 0x6503305f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8573061, SUC_NE_lexicon_tags_2 },
    { 0xd5bf3062, SUC_NE_lexicon_tags_64 },
    { 0xffb33063, SUC_NE_lexicon_tags_2 },
    { 0x264d3061, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b613069, SUC_NE_lexicon_tags_2 },
    { 0x95c73069, SUC_NE_lexicon_tags_4 },
    { 0xeb9d306b, SUC_NE_lexicon_tags_2 },
    { 0xcedd306c, SUC_NE_lexicon_tags_2 },
    { 0x5fd5306d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa793070, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a93072, SUC_NE_lexicon_tags_2 },
    { 0x8d213073, SUC_NE_lexicon_tags_2 },
    { 0xcbd93074, SUC_NE_lexicon_tags_2 },
    { 0xb393072, SUC_NE_lexicon_tags_2 },
    { 0x185d3073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e233078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a913080, SUC_NE_lexicon_tags_2 },
    { 0x618b3081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58853085, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87c13087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374d3091, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389f3097, SUC_NE_lexicon_tags_7 },
    { 0x61853098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3309b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96130a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdce930a2, SUC_NE_lexicon_tags_1 },
    { 0xf33d30a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x512130a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f730a8, SUC_NE_lexicon_tags_2 },
    { 0x5d5730a9, SUC_NE_lexicon_tags_2 },
    { 0x4ebd30aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b30ae, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x64f30b0, SUC_NE_lexicon_tags_2 },
    { 0x3ae930b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66cd30b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef30b8, SUC_NE_lexicon_tags_2 },
    { 0xd59530b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850930be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d7930c0, SUC_NE_lexicon_tags_18 },
    { 0xeb4130c1, SUC_NE_lexicon_tags_2 },
    { 0xfa6730c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea330c6, SUC_NE_lexicon_tags_2 },
    { 0x347d30c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fc330c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab30ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaba930d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd46130d2, SUC_NE_lexicon_tags_2 },
    { 0x54a930d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8ed30d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ec730d7, SUC_NE_lexicon_tags_2 },
    { 0xe92b30d8, SUC_NE_lexicon_tags_2 },
    { 0x68d30d7, SUC_NE_lexicon_tags_2 },
    { 0xb72b30da, SUC_NE_lexicon_tags_2 },
    { 0xfa4330d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd530de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2f330e0, SUC_NE_lexicon_tags_7 },
    { 0xb69530e1, SUC_NE_lexicon_tags_2 },
    { 0xc0a330e1, SUC_NE_lexicon_tags_2 },
    { 0x47f730e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50530e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0b30eb, SUC_NE_lexicon_tags_2 },
    { 0x4e6130ec, SUC_NE_lexicon_tags_2 },
    { 0x7d0330ed, SUC_NE_lexicon_tags_2 },
    { 0x14e730ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8f30f1, SUC_NE_lexicon_tags_2 },
    { 0x478730f1, SUC_NE_lexicon_tags_2 },
    { 0x894530f1, SUC_NE_lexicon_tags_2 },
    { 0x695b30f3, SUC_NE_lexicon_tags_2 },
    { 0xb9a530f4, SUC_NE_lexicon_tags_2 },
    { 0x2b4530f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9dd30f8, SUC_NE_lexicon_tags_2 },
    { 0x44a330f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbb30fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e13101, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f53104, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af73107, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e073109, SUC_NE_lexicon_tags_2 },
    { 0xebd9310a, SUC_NE_lexicon_tags_2 },
    { 0x1dbb310b, SUC_NE_lexicon_tags_2 },
    { 0x76553109, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0xe681310e, SUC_NE_lexicon_tags_2 },
    { 0x6d4d310f, SUC_NE_lexicon_tags_2 },
    { 0x57953110, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40913114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35993117, SUC_NE_lexicon_tags_2 },
    { 0x30af3118, SUC_NE_lexicon_tags_2 },
    { 0x96853118, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e573120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ed3124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838b312a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0973131, SUC_NE_lexicon_tags_2 },
    { 0x1073131, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baf3136, SUC_NE_lexicon_tags_2 },
    { 0xab453137, SUC_NE_lexicon_tags_2 },
    { 0x4c6d3137, SUC_NE_lexicon_tags_2 },
    { 0xc7613139, SUC_NE_lexicon_tags_1 },
    { 0x5ca7313a, SUC_NE_lexicon_tags_2 },
    { 0x592d313b, SUC_NE_lexicon_tags_2 },
    { 0xf031313c, SUC_NE_lexicon_tags_2 },
    { 0x6747313d, SUC_NE_lexicon_tags_2 },
    { 0x983d313e, SUC_NE_lexicon_tags_2 },
    { 0x2797313f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9eff3141, SUC_NE_lexicon_tags_2 },
    { 0xf2453142, SUC_NE_lexicon_tags_2 },
    { 0x887f3143, SUC_NE_lexicon_tags_2 },
    { 0x3b33144, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6e33146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70913148, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e3314f, SUC_NE_lexicon_tags_19 },
    { 0xe58f314f, SUC_NE_lexicon_tags_2 },
    { 0x53c3314f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fb13153, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e3d3155, SUC_NE_lexicon_tags_2 },
    { 0x24313155, SUC_NE_lexicon_tags_2 },
    { 0xe8fd3157, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2373159, SUC_NE_lexicon_tags_2 },
    { 0xb8653159, SUC_NE_lexicon_tags_2 },
    { 0xac9315b, SUC_NE_lexicon_tags_3 },
    { 0x2b0b315b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf87315f, SUC_NE_lexicon_tags_2 },
    { 0x62d3160, SUC_NE_lexicon_tags_2 },
    { 0xa6f53161, SUC_NE_lexicon_tags_2 },
    { 0x5f233160, SUC_NE_lexicon_tags_2 },
    { 0x3d713163, SUC_NE_lexicon_tags_4 },
    { 0x90573164, SUC_NE_lexicon_tags_2 },
    { 0x1c053165, SUC_NE_lexicon_tags_2 },
    { 0xb8253165, SUC_NE_lexicon_tags_4 },
    { 0x67973167, SUC_NE_lexicon_tags_2 },
    { 0x853f3168, SUC_NE_lexicon_tags_2 },
    { 0xcea73168, SUC_NE_lexicon_tags_2 },
    { 0xd0b53168, SUC_NE_lexicon_tags_2 },
    { 0x6ed3316b, SUC_NE_lexicon_tags_2 },
    { 0xfc45316c, SUC_NE_lexicon_tags_2 },
    { 0x638f3169, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a73170, SUC_NE_lexicon_tags_2 },
    { 0x82d33171, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf25f3173, SUC_NE_lexicon_tags_2 },
    { 0x2db73174, SUC_NE_lexicon_tags_2 },
    { 0xcc4b3175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x867b3177, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e3317d, SUC_NE_lexicon_tags_2 },
    { 0xd97b317e, SUC_NE_lexicon_tags_2 },
    { 0x52e5317e, SUC_NE_lexicon_tags_14 },
    { 0xee4d3180, SUC_NE_lexicon_tags_19 },
    { 0xa42b317d, SUC_NE_lexicon_tags_2 },
    { 0x325f3180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19d53184, SUC_NE_lexicon_tags_2 },
    { 0xea7d3185, SUC_NE_lexicon_tags_19 },
    { 0xf4a33185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b57318a, SUC_NE_lexicon_tags_2 },
    { 0xf7cd318b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xede7318d, SUC_NE_lexicon_tags_2 },
    { 0x33e1318e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x681d3191, SUC_NE_lexicon_tags_2 },
    { 0xd29d3192, SUC_NE_lexicon_tags_2 },
    { 0x97d3191, SUC_NE_lexicon_tags_2 },
    { 0x1d153191, SUC_NE_lexicon_tags_2 },
    { 0x2fc33195, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6233199, SUC_NE_lexicon_tags_2 },
    { 0x1703319a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d3319e, SUC_NE_lexicon_tags_4 },
    { 0xb76b319e, SUC_NE_lexicon_tags_2 },
    { 0xc27331a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92931a4, SUC_NE_lexicon_tags_2 },
    { 0xce0b31a4, SUC_NE_lexicon_tags_7 },
    { 0x267531a6, SUC_NE_lexicon_tags_2 },
    { 0x457331a7, SUC_NE_lexicon_tags_2 },
    { 0xdd2331a8, SUC_NE_lexicon_tags_2 },
    { 0x6f0731a8, SUC_NE_lexicon_tags_2 },
    { 0xbee331a8, SUC_NE_lexicon_tags_2 },
    { 0xb0f131a6, SUC_NE_lexicon_tags_2 },
    { 0x184131ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ba131ae, SUC_NE_lexicon_tags_17 },
    { 0xf7af31af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69b931b1, SUC_NE_lexicon_tags_25 },
    { 0x7aa131b2, SUC_NE_lexicon_tags_2 },
    { 0x504531b1, SUC_NE_lexicon_tags_2 },
    { 0x7bf31b4, SUC_NE_lexicon_tags_2 },
    { 0x40d931b3, SUC_NE_lexicon_tags_2 },
    { 0xa1b331b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaed31b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f931bc, SUC_NE_lexicon_tags_2 },
    { 0x4ae531bc, SUC_NE_lexicon_tags_2 },
    { 0x6f6f31be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3d31c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b131c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27f31cb, SUC_NE_lexicon_tags_2 },
    { 0x4bf331cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770131d6, SUC_NE_lexicon_tags_3 },
    { 0x3d31d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789931db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7b31df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9d331e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc8131e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x484b31e6, SUC_NE_lexicon_tags_2 },
    { 0x4e0d31e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x527331e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b1731eb, SUC_NE_lexicon_tags_2 },
    { 0x45f31ec, SUC_NE_lexicon_tags_2 },
    { 0xdfc331ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972931f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49131f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f1331fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f0731fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb693201, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c53208, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2df320b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cbb320d, SUC_NE_lexicon_tags_7 },
    { 0xa749320d, SUC_NE_lexicon_tags_2 },
    { 0x745320e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeb3212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7273216, SUC_NE_lexicon_tags_2 },
    { 0xd10b3216, SUC_NE_lexicon_tags_2 },
    { 0x7ed73218, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c17321a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed5321d, SUC_NE_lexicon_tags_12 },
    { 0xbde3321e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f33222, SUC_NE_lexicon_tags_2 },
    { 0x46bf3222, SUC_NE_lexicon_tags_4 },
    { 0x46633224, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a83322a, SUC_NE_lexicon_tags_2 },
    { 0x3b3f322b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7f3230, SUC_NE_lexicon_tags_2 },
    { 0x9f5b3231, SUC_NE_lexicon_tags_2 },
    { 0x7c773231, SUC_NE_lexicon_tags_4 },
    { 0x913f3232, SUC_NE_lexicon_tags_2 },
    { 0x428b3234, SUC_NE_lexicon_tags_2 },
    { 0x353f3232, SUC_NE_lexicon_tags_2 },
    { 0x42613236, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8b323a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad91323f, SUC_NE_lexicon_tags_2 },
    { 0x63ab3240, SUC_NE_lexicon_tags_2 },
    { 0xbba53241, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf3244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3327324c, SUC_NE_lexicon_tags_2 },
    { 0x16c1324d, SUC_NE_lexicon_tags_2 },
    { 0x317324d, SUC_NE_lexicon_tags_2 },
    { 0x36c9324e, SUC_NE_lexicon_tags_2 },
    { 0x3c7f324f, SUC_NE_lexicon_tags_2 },
    { 0x8313324f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52113253, SUC_NE_lexicon_tags_2 },
    { 0x8ded3254, SUC_NE_lexicon_tags_2 },
    { 0x45ad3255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc093258, SUC_NE_lexicon_tags_2 },
    { 0xc96d3258, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbcf325b, SUC_NE_lexicon_tags_2 },
    { 0x3fa5325b, SUC_NE_lexicon_tags_59 },
    { 0x7ff325d, SUC_NE_lexicon_tags_233 },
    { 0xe2d325d, SUC_NE_lexicon_tags_2 },
    { 0xd337325f, SUC_NE_lexicon_tags_2 },
    { 0xf81f325d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b93267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd85326a, SUC_NE_lexicon_tags_2 },
    { 0x3a25326b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d73270, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70673274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb553276, SUC_NE_lexicon_tags_2 },
    { 0xc753276, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd677327c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa41327e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d3281, SUC_NE_lexicon_tags_2 },
    { 0xf1293281, SUC_NE_lexicon_tags_2 },
    { 0x98bd3283, SUC_NE_lexicon_tags_2 },
    { 0x44793284, SUC_NE_lexicon_tags_2 },
    { 0xdc9d3284, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35293289, SUC_NE_lexicon_tags_2 },
    { 0x4ca1328a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e3328d, SUC_NE_lexicon_tags_2 },
    { 0x1e49328e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc973293, SUC_NE_lexicon_tags_2 },
    { 0x5ded3294, SUC_NE_lexicon_tags_2 },
    { 0xd95f3295, SUC_NE_lexicon_tags_2 },
    { 0xa2ab3296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121329b, SUC_NE_lexicon_tags_2 },
    { 0x53c7329b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xfdf9329e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38d32a1, SUC_NE_lexicon_tags_2 },
    { 0x72b532a2, SUC_NE_lexicon_tags_2 },
    { 0x960932a2, SUC_NE_lexicon_tags_2 },
    { 0x66932a1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89532a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f6b32aa, SUC_NE_lexicon_tags_2 },
    { 0x2e0732aa, SUC_NE_lexicon_tags_2 },
    { 0xc2c132ac, SUC_NE_lexicon_tags_2 },
    { 0x793d32ad, SUC_NE_lexicon_tags_2 },
    { 0xeed532ad, SUC_NE_lexicon_tags_2 },
    { 0xf54732ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56f32b8, SUC_NE_lexicon_tags_2 },
    { 0xb7ef32b8, SUC_NE_lexicon_tags_2 },
    { 0xbe132b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca932be, SUC_NE_lexicon_tags_2 },
    { 0x37a132be, SUC_NE_lexicon_tags_2 },
    { 0x108932c0, SUC_NE_lexicon_tags_2 },
    { 0x3c3932c1, SUC_NE_lexicon_tags_2 },
    { 0xa6932c2, SUC_NE_lexicon_tags_13 },
    { 0xc7f132c2, SUC_NE_lexicon_tags_2 },
    { 0x864732c4, SUC_NE_lexicon_tags_2 },
    { 0x95f32c5, SUC_NE_lexicon_tags_2 },
    { 0x5cc132c0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fd32c9, SUC_NE_lexicon_tags_2 },
    { 0x1fbb32c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9932ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed32d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x587932d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8532dc, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x31c932de, SUC_NE_lexicon_tags_2 },
    { 0x130532df, SUC_NE_lexicon_tags_2 },
    { 0xe53332df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164932ea, SUC_NE_lexicon_tags_2 },
    { 0x7dc132eb, SUC_NE_lexicon_tags_2 },
    { 0x850932ec, SUC_NE_lexicon_tags_0 },
    { 0x7e6d32ed, SUC_NE_lexicon_tags_2 },
    { 0x4a6332ed, SUC_NE_lexicon_tags_2 },
    { 0x1d332ed, SUC_NE_lexicon_tags_2 },
    { 0x22132f0, SUC_NE_lexicon_tags_2 },
    { 0x87e732f0, SUC_NE_lexicon_tags_2 },
    { 0x38e732ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cf32f8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63bb32fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb132fe, SUC_NE_lexicon_tags_2 },
    { 0xa1cd32ff, SUC_NE_lexicon_tags_2 },
    { 0xbc553300, SUC_NE_lexicon_tags_2 },
    { 0x79d73301, SUC_NE_lexicon_tags_2 },
    { 0x309932fe, SUC_NE_lexicon_tags_2 },
    { 0x9613303, SUC_NE_lexicon_tags_2 },
    { 0x6c113300, SUC_NE_lexicon_tags_17 },
    { 0x88653305, SUC_NE_lexicon_tags_2 },
    { 0xb02d3305, SUC_NE_lexicon_tags_2 },
    { 0xa2313307, SUC_NE_lexicon_tags_2 },
    { 0xe5513303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f330c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa075330f, SUC_NE_lexicon_tags_2 },
    { 0xd1753310, SUC_NE_lexicon_tags_2 },
    { 0x71e73311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d73317, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe931331f, SUC_NE_lexicon_tags_2 },
    { 0xe4d7331f, SUC_NE_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354f3326, SUC_NE_lexicon_tags_2 },
    { 0x59213327, SUC_NE_lexicon_tags_2 },
    { 0xdcc13327, SUC_NE_lexicon_tags_2 },
    { 0xc1553329, SUC_NE_lexicon_tags_2 },
    { 0x7a653328, SUC_NE_lexicon_tags_2 },
    { 0x91b3328, SUC_NE_lexicon_tags_2 },
    { 0xf9f1332c, SUC_NE_lexicon_tags_2 },
    { 0xd381332c, SUC_NE_lexicon_tags_2 },
    { 0xf713332e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6efb3330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28dd3332, SUC_NE_lexicon_tags_2 },
    { 0x4ca13332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x842f3335, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18833338, SUC_NE_lexicon_tags_0 },
    { 0x54073339, SUC_NE_lexicon_tags_2 },
    { 0x18b1333a, SUC_NE_lexicon_tags_2 },
    { 0x9541333a, SUC_NE_lexicon_tags_2 },
    { 0xbdbb333c, SUC_NE_lexicon_tags_2 },
    { 0xabf7333a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f47333f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13d73341, SUC_NE_lexicon_tags_2 },
    { 0x77f13341, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd93347, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d334a, SUC_NE_lexicon_tags_2 },
    { 0xad1b334b, SUC_NE_lexicon_tags_2 },
    { 0x5893334b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee633351, SUC_NE_lexicon_tags_2 },
    { 0x59513352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77113355, SUC_NE_lexicon_tags_2 },
    { 0x1bcd3356, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68873359, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c5335c, SUC_NE_lexicon_tags_2 },
    { 0xc4f3335d, SUC_NE_lexicon_tags_2 },
    { 0x6a9b335e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96833360, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf593362, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f73369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5afb336b, SUC_NE_lexicon_tags_2 },
    { 0xb7e3336b, SUC_NE_lexicon_tags_2 },
    { 0xe1ef336d, SUC_NE_lexicon_tags_2 },
    { 0x33f3336b, SUC_NE_lexicon_tags_2 },
    { 0x6683336f, SUC_NE_lexicon_tags_2 },
    { 0x17b3370, SUC_NE_lexicon_tags_2 },
    { 0xd86f3370, SUC_NE_lexicon_tags_2 },
    { 0xfde53372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce2d3374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd633376, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ac73378, SUC_NE_lexicon_tags_2 },
    { 0x9d713378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf9337e, SUC_NE_lexicon_tags_2 },
    { 0xf853337f, SUC_NE_lexicon_tags_2 },
    { 0xdbf9337e, SUC_NE_lexicon_tags_2 },
    { 0xdb413381, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715b3383, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ddd3385, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30b3388, SUC_NE_lexicon_tags_2 },
    { 0x3fc33389, SUC_NE_lexicon_tags_13 },
    { 0xfb4b338a, SUC_NE_lexicon_tags_24 },
    { 0x52e5338b, SUC_NE_lexicon_tags_7 },
    { 0xbf37338c, SUC_NE_lexicon_tags_2 },
    { 0x9f29338d, SUC_NE_lexicon_tags_2 },
    { 0xe0e5338b, SUC_NE_lexicon_tags_2 },
    { 0x22a9338f, SUC_NE_lexicon_tags_2 },
    { 0xdbdd3388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2f3392, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb3399, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43f3339b, SUC_NE_lexicon_tags_2 },
    { 0x42a3339c, SUC_NE_lexicon_tags_2 },
    { 0x797d339d, SUC_NE_lexicon_tags_2 },
    { 0x5759339e, SUC_NE_lexicon_tags_2 },
    { 0x5f71339e, SUC_NE_lexicon_tags_2 },
    { 0x916f33a0, SUC_NE_lexicon_tags_2 },
    { 0x200133a0, SUC_NE_lexicon_tags_2 },
    { 0xc19933a2, SUC_NE_lexicon_tags_2 },
    { 0x3d3333a3, SUC_NE_lexicon_tags_2 },
    { 0x73b133a4, SUC_NE_lexicon_tags_2 },
    { 0xe79d33a3, SUC_NE_lexicon_tags_2 },
    { 0x842933a6, SUC_NE_lexicon_tags_2 },
    { 0x454533a6, SUC_NE_lexicon_tags_2 },
    { 0xdb4133a3, SUC_NE_lexicon_tags_7 },
    { 0x9ef533a9, SUC_NE_lexicon_tags_2 },
    { 0x673733aa, SUC_NE_lexicon_tags_2 },
    { 0x762333ab, SUC_NE_lexicon_tags_2 },
    { 0x464733a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x567133ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69533b3, SUC_NE_lexicon_tags_4 },
    { 0xd79333b3, SUC_NE_lexicon_tags_5 },
    { 0x6c3533b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7a133b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af133bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e733bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfcf33c1, SUC_NE_lexicon_tags_25 },
    { 0xaf2333c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40d33c5, SUC_NE_lexicon_tags_2 },
    { 0x28c333c5, SUC_NE_lexicon_tags_2 },
    { 0x92d33c7, SUC_NE_lexicon_tags_36 },
    { 0x18c333c7, SUC_NE_lexicon_tags_2 },
    { 0x20c533c9, SUC_NE_lexicon_tags_3 },
    { 0xbc2333ca, SUC_NE_lexicon_tags_2 },
    { 0xfe6333cb, SUC_NE_lexicon_tags_2 },
    { 0x7ee133cc, SUC_NE_lexicon_tags_2 },
    { 0xfba733c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f133d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80e533d3, SUC_NE_lexicon_tags_2 },
    { 0x250333d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75333d9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xddd133db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff33de, SUC_NE_lexicon_tags_420 },
    { 0x9ae533df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41f733e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8b33e5, SUC_NE_lexicon_tags_2 },
    { 0x391133e5, SUC_NE_lexicon_tags_2 },
    { 0xca933e7, SUC_NE_lexicon_tags_2 },
    { 0xe10b33e8, SUC_NE_lexicon_tags_2 },
    { 0xbf8133e9, SUC_NE_lexicon_tags_2 },
    { 0xbacf33e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0933ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6da133ee, SUC_NE_lexicon_tags_3 },
    { 0x8d9933ef, SUC_NE_lexicon_tags_2 },
    { 0xcda733f0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acb33f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3f33fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x439133fd, SUC_NE_lexicon_tags_2 },
    { 0xfd5733fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc93402, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde53405, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf75f3407, SUC_NE_lexicon_tags_2 },
    { 0x9e1f3408, SUC_NE_lexicon_tags_2 },
    { 0xc6513409, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57340e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90253411, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x398b3413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb72f3415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf3418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf9b341a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b3b341c, SUC_NE_lexicon_tags_2 },
    { 0x9acb341d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5b1341f, SUC_NE_lexicon_tags_2 },
    { 0xe4773420, SUC_NE_lexicon_tags_2 },
    { 0x4e693421, SUC_NE_lexicon_tags_2 },
    { 0x2d5d3421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b3427, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d342a, SUC_NE_lexicon_tags_2 },
    { 0x4d17342b, SUC_NE_lexicon_tags_2 },
    { 0x8ed7342b, SUC_NE_lexicon_tags_2 },
    { 0xcdc1342d, SUC_NE_lexicon_tags_2 },
    { 0x1c51342e, SUC_NE_lexicon_tags_7 },
    { 0x65e5342f, SUC_NE_lexicon_tags_1 },
    { 0xed013430, SUC_NE_lexicon_tags_2 },
    { 0xaa8f3431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccd3435, SUC_NE_lexicon_tags_8 },
    { 0x81033435, SUC_NE_lexicon_tags_2 },
    { 0xb76b3437, SUC_NE_lexicon_tags_7 },
    { 0x6e793437, SUC_NE_lexicon_tags_24 },
    { 0xffc33439, SUC_NE_lexicon_tags_2 },
    { 0x7a493436, SUC_NE_lexicon_tags_2 },
    { 0xb627343b, SUC_NE_lexicon_tags_2 },
    { 0xbe2b343c, SUC_NE_lexicon_tags_2 },
    { 0x5b2d343d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa70f343f, SUC_NE_lexicon_tags_2 },
    { 0x5ac53440, SUC_NE_lexicon_tags_2 },
    { 0x936b3440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24893446, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2313451, SUC_NE_lexicon_tags_2 },
    { 0xc1753452, SUC_NE_lexicon_tags_2 },
    { 0x6fa33453, SUC_NE_lexicon_tags_2 },
    { 0x86b13454, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe01345a, SUC_NE_lexicon_tags_2 },
    { 0x7daf345b, SUC_NE_lexicon_tags_2 },
    { 0x4819345c, SUC_NE_lexicon_tags_2 },
    { 0x731345c, SUC_NE_lexicon_tags_2 },
    { 0xff73345e, SUC_NE_lexicon_tags_2 },
    { 0x9a3d345c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c393461, SUC_NE_lexicon_tags_2 },
    { 0x74ef3462, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f33465, SUC_NE_lexicon_tags_2 },
    { 0x5aa13465, SUC_NE_lexicon_tags_3 },
    { 0xa4053467, SUC_NE_lexicon_tags_2 },
    { 0x242b3465, SUC_NE_lexicon_tags_2 },
    { 0x6c7d3467, SUC_NE_lexicon_tags_2 },
    { 0xb84d3469, SUC_NE_lexicon_tags_2 },
    { 0x67073466, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9031346e, SUC_NE_lexicon_tags_2 },
    { 0x7615346f, SUC_NE_lexicon_tags_2 },
    { 0x1737346e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52af3473, SUC_NE_lexicon_tags_2 },
    { 0x5f473474, SUC_NE_lexicon_tags_2 },
    { 0xb6693475, SUC_NE_lexicon_tags_2 },
    { 0x5bb53476, SUC_NE_lexicon_tags_2 },
    { 0x6a533475, SUC_NE_lexicon_tags_24 },
    { 0x3b473478, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfedd347a, SUC_NE_lexicon_tags_2 },
    { 0x8bb9347b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccb7347d, SUC_NE_lexicon_tags_2 },
    { 0xd611347e, SUC_NE_lexicon_tags_2 },
    { 0x2253347e, SUC_NE_lexicon_tags_2 },
    { 0x5557347f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea633482, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee73489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x351d348b, SUC_NE_lexicon_tags_5 },
    { 0x8da3348c, SUC_NE_lexicon_tags_2 },
    { 0x3bcb348c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x957f348f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed93495, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32913498, SUC_NE_lexicon_tags_5 },
    { 0xcbb3499, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb349e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0734a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d34a9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6d34ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff734b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd734b3, SUC_NE_lexicon_tags_2 },
    { 0xa82134b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc934b8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cd34c2, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dff34cb, SUC_NE_lexicon_tags_1 },
    { 0x8c1b34cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e334ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47c934d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05934d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937134d9, SUC_NE_lexicon_tags_2 },
    { 0x4b2334da, SUC_NE_lexicon_tags_2 },
    { 0xd90334db, SUC_NE_lexicon_tags_2 },
    { 0xf2a734d9, SUC_NE_lexicon_tags_2 },
    { 0x1ed534dd, SUC_NE_lexicon_tags_2 },
    { 0x209934da, SUC_NE_lexicon_tags_4 },
    { 0x31b34df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07134e2, SUC_NE_lexicon_tags_2 },
    { 0xcde534e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef534e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73934ef, SUC_NE_lexicon_tags_2 },
    { 0x2b9534ef, SUC_NE_lexicon_tags_2 },
    { 0xb13134ef, SUC_NE_lexicon_tags_28 },
    { 0x460134f2, SUC_NE_lexicon_tags_2 },
    { 0x7f0d34f3, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x684f34f5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab933500, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8b3504, SUC_NE_lexicon_tags_2 },
    { 0x4e973504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf32d3507, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd4c93509, SUC_NE_lexicon_tags_2 },
    { 0x68d73509, SUC_NE_lexicon_tags_2 },
    { 0xf825350b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe53511, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ed3513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x149f3515, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21a93517, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9793519, SUC_NE_lexicon_tags_7 },
    { 0xf96f3519, SUC_NE_lexicon_tags_2 },
    { 0xfa233519, SUC_NE_lexicon_tags_2 },
    { 0x2a7d3519, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb351f, SUC_NE_lexicon_tags_2 },
    { 0x45733520, SUC_NE_lexicon_tags_2 },
    { 0xb6473521, SUC_NE_lexicon_tags_19 },
    { 0x2c4f3522, SUC_NE_lexicon_tags_2 },
    { 0x62af3523, SUC_NE_lexicon_tags_2 },
    { 0xb7a33524, SUC_NE_lexicon_tags_2 },
    { 0xed933522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d09352a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d93530, SUC_NE_lexicon_tags_2 },
    { 0x595b3531, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f213533, SUC_NE_lexicon_tags_4 },
    { 0xf2e13534, SUC_NE_lexicon_tags_2 },
    { 0x80533535, SUC_NE_lexicon_tags_2 },
    { 0x888b3535, SUC_NE_lexicon_tags_2 },
    { 0xd99d3537, SUC_NE_lexicon_tags_2 },
    { 0x24c53538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7c1353a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd1ef353c, SUC_NE_lexicon_tags_2 },
    { 0xfad3353d, SUC_NE_lexicon_tags_2 },
    { 0x6051353c, SUC_NE_lexicon_tags_2 },
    { 0x4a5f353f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89293541, SUC_NE_lexicon_tags_2 },
    { 0x3fdb3542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5853544, SUC_NE_lexicon_tags_2 },
    { 0x333545, SUC_NE_lexicon_tags_2 },
    { 0x48373546, SUC_NE_lexicon_tags_2 },
    { 0x32653546, SUC_NE_lexicon_tags_2 },
    { 0x25ad3548, SUC_NE_lexicon_tags_2 },
    { 0xaf933549, SUC_NE_lexicon_tags_2 },
    { 0xa1a53549, SUC_NE_lexicon_tags_2 },
    { 0x86e9354b, SUC_NE_lexicon_tags_2 },
    { 0xe3af3546, SUC_NE_lexicon_tags_2 },
    { 0x54073549, SUC_NE_lexicon_tags_7 },
    { 0xf1a73545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24293550, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc14d3552, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2933554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27773558, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b05355b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5573561, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6633564, SUC_NE_lexicon_tags_2 },
    { 0xe1a53565, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafd13567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75b356a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc825356e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef93570, SUC_NE_lexicon_tags_2 },
    { 0x613f3570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd493574, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e73577, SUC_NE_lexicon_tags_4 },
    { 0xded3578, SUC_NE_lexicon_tags_7 },
    { 0x31433579, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x269b357b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16393580, SUC_NE_lexicon_tags_2 },
    { 0xc39b3581, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf153583, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f3588, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761f358c, SUC_NE_lexicon_tags_2 },
    { 0xe297358c, SUC_NE_lexicon_tags_2 },
    { 0xab2f358e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf3598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed2d359a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40735a4, SUC_NE_lexicon_tags_2 },
    { 0x2aa935a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe79735a7, SUC_NE_lexicon_tags_2 },
    { 0x525f35a8, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x699135aa, SUC_NE_lexicon_tags_2 },
    { 0xee2f35ab, SUC_NE_lexicon_tags_2 },
    { 0xa2c935ac, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1935b1, SUC_NE_lexicon_tags_2 },
    { 0xc73535b2, SUC_NE_lexicon_tags_2 },
    { 0x99cd35b3, SUC_NE_lexicon_tags_4 },
    { 0xa6a735b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c735b7, SUC_NE_lexicon_tags_384 },
    { 0, NULL },
    { 0xe2b335b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52ad35bb, SUC_NE_lexicon_tags_2 },
    { 0x903135bc, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64b35bf, SUC_NE_lexicon_tags_2 },
    { 0x8e6335c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0d35cb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x49db35ce, SUC_NE_lexicon_tags_2 },
    { 0xfced35ce, SUC_NE_lexicon_tags_2 },
    { 0x925b35d0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe7cf35d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abb35d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7e135da, SUC_NE_lexicon_tags_2 },
    { 0x608135da, SUC_NE_lexicon_tags_0 },
    { 0x52335dc, SUC_NE_lexicon_tags_2 },
    { 0xcf9735dc, SUC_NE_lexicon_tags_2 },
    { 0xc20335de, SUC_NE_lexicon_tags_2 },
    { 0xef7535de, SUC_NE_lexicon_tags_2 },
    { 0x8e6935df, SUC_NE_lexicon_tags_3 },
    { 0xb3bf35df, SUC_NE_lexicon_tags_2 },
    { 0x8e7735e2, SUC_NE_lexicon_tags_2 },
    { 0x4c2935e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109735e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6535ec, SUC_NE_lexicon_tags_2 },
    { 0x88ab35ed, SUC_NE_lexicon_tags_2 },
    { 0xc9fd35ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x772335f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa58735f2, SUC_NE_lexicon_tags_2 },
    { 0xc5e735f3, SUC_NE_lexicon_tags_2 },
    { 0x2cc735f2, SUC_NE_lexicon_tags_2 },
    { 0xaa6335f5, SUC_NE_lexicon_tags_2 },
    { 0x263335f6, SUC_NE_lexicon_tags_2 },
    { 0x7e1735f7, SUC_NE_lexicon_tags_2 },
    { 0xf75335f8, SUC_NE_lexicon_tags_2 },
    { 0x52c935f7, SUC_NE_lexicon_tags_4 },
    { 0x4f3f35fa, SUC_NE_lexicon_tags_3 },
    { 0x795335f3, SUC_NE_lexicon_tags_2 },
    { 0x972935fa, SUC_NE_lexicon_tags_2 },
    { 0xb30135f6, SUC_NE_lexicon_tags_24 },
    { 0xa00d35fe, SUC_NE_lexicon_tags_7 },
    { 0x7a9735f3, SUC_NE_lexicon_tags_2 },
    { 0xe2553600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f093602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85233604, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5360a, SUC_NE_lexicon_tags_2 },
    { 0xb475360b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a153612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd3617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f773619, SUC_NE_lexicon_tags_2 },
    { 0x218d361a, SUC_NE_lexicon_tags_2 },
    { 0x9435361a, SUC_NE_lexicon_tags_2 },
    { 0x280d361b, SUC_NE_lexicon_tags_2 },
    { 0x3e0f361d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4893620, SUC_NE_lexicon_tags_4 },
    { 0x7d493620, SUC_NE_lexicon_tags_2 },
    { 0x60c93622, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92673624, SUC_NE_lexicon_tags_17 },
    { 0xbf193625, SUC_NE_lexicon_tags_2 },
    { 0xf97b3625, SUC_NE_lexicon_tags_4 },
    { 0xb7833627, SUC_NE_lexicon_tags_2 },
    { 0x89f13626, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x523b362a, SUC_NE_lexicon_tags_2 },
    { 0x2b63362a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe499362e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d273632, SUC_NE_lexicon_tags_32 },
    { 0xbc793633, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f13636, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287d363d, SUC_NE_lexicon_tags_2 },
    { 0x7d3f363e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6be73640, SUC_NE_lexicon_tags_2 },
    { 0x35c73641, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87293645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d364e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2433650, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ff3653, SUC_NE_lexicon_tags_2 },
    { 0x50413653, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7333656, SUC_NE_lexicon_tags_2 },
    { 0xa73b3657, SUC_NE_lexicon_tags_2 },
    { 0xc4e73658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb127365a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b365d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x378f3660, SUC_NE_lexicon_tags_2 },
    { 0xb4253661, SUC_NE_lexicon_tags_2 },
    { 0xfc1f3662, SUC_NE_lexicon_tags_2 },
    { 0x7f413663, SUC_NE_lexicon_tags_3 },
    { 0xe8493664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6933666, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x612f3668, SUC_NE_lexicon_tags_2 },
    { 0x540f3668, SUC_NE_lexicon_tags_2 },
    { 0xc9513669, SUC_NE_lexicon_tags_2 },
    { 0xc5cb366b, SUC_NE_lexicon_tags_2 },
    { 0x5529366b, SUC_NE_lexicon_tags_2 },
    { 0x771366b, SUC_NE_lexicon_tags_2 },
    { 0x5a2d366e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d93677, SUC_NE_lexicon_tags_2 },
    { 0x4e7f3678, SUC_NE_lexicon_tags_2 },
    { 0x42133679, SUC_NE_lexicon_tags_2 },
    { 0x60fb3679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8acb367c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fd3680, SUC_NE_lexicon_tags_2 },
    { 0x5c2d3681, SUC_NE_lexicon_tags_2 },
    { 0x493b3682, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb3687, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0x1bf53689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf165368b, SUC_NE_lexicon_tags_2 },
    { 0x307b368c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b13693, SUC_NE_lexicon_tags_7 },
    { 0x53a73694, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81413696, SUC_NE_lexicon_tags_2 },
    { 0xe93697, SUC_NE_lexicon_tags_2 },
    { 0x8dc13696, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x451369a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa799369d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d936a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x144136a2, SUC_NE_lexicon_tags_2 },
    { 0x631136a3, SUC_NE_lexicon_tags_32 },
    { 0xb24936a4, SUC_NE_lexicon_tags_2 },
    { 0x123536a2, SUC_NE_lexicon_tags_2 },
    { 0xd95b36a4, SUC_NE_lexicon_tags_3 },
    { 0xf4d336a7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4336aa, SUC_NE_lexicon_tags_19 },
    { 0x3b2d36ab, SUC_NE_lexicon_tags_2 },
    { 0x129936aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9736b1, SUC_NE_lexicon_tags_2 },
    { 0x728d36b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f4f36b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e936b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6736bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad36c0, SUC_NE_lexicon_tags_2 },
    { 0xe9d336c1, SUC_NE_lexicon_tags_21 },
    { 0x6fc136c1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349736cd, SUC_NE_lexicon_tags_13 },
    { 0x7c8936cd, SUC_NE_lexicon_tags_8 },
    { 0x887b36cf, SUC_NE_lexicon_tags_2 },
    { 0xf54136d0, SUC_NE_lexicon_tags_4 },
    { 0x58db36cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31af36dc, SUC_NE_lexicon_tags_2 },
    { 0xd5e536dc, SUC_NE_lexicon_tags_2 },
    { 0x9a5136dd, SUC_NE_lexicon_tags_2 },
    { 0x78ab36df, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6b36e5, SUC_NE_lexicon_tags_2 },
    { 0x462336e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56fd36e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c1736ea, SUC_NE_lexicon_tags_2 },
    { 0x1b2f36eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53736ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bd36f4, SUC_NE_lexicon_tags_4 },
    { 0x333b36f5, SUC_NE_lexicon_tags_2 },
    { 0xfa2f36f4, SUC_NE_lexicon_tags_5 },
    { 0x6dd536f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb96536f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d936ff, SUC_NE_lexicon_tags_7 },
    { 0xa3f93700, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b53706, SUC_NE_lexicon_tags_13 },
    { 0x63913707, SUC_NE_lexicon_tags_2 },
    { 0x4fad3708, SUC_NE_lexicon_tags_2 },
    { 0x3b133708, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab5370d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd593712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77f73714, SUC_NE_lexicon_tags_2 },
    { 0x17dd3715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89393717, SUC_NE_lexicon_tags_2 },
    { 0xc2f73718, SUC_NE_lexicon_tags_2 },
    { 0x99913718, SUC_NE_lexicon_tags_2 },
    { 0xbbe1371a, SUC_NE_lexicon_tags_3 },
    { 0x87d371a, SUC_NE_lexicon_tags_2 },
    { 0x8ae1371c, SUC_NE_lexicon_tags_2 },
    { 0xf0c9371a, SUC_NE_lexicon_tags_1 },
    { 0x2f013719, SUC_NE_lexicon_tags_2 },
    { 0xb52d371c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66773726, SUC_NE_lexicon_tags_3 },
    { 0x90ff3727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ef372a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfd372d, SUC_NE_lexicon_tags_2 },
    { 0xdffb372e, SUC_NE_lexicon_tags_4 },
    { 0x62ab372d, SUC_NE_lexicon_tags_2 },
    { 0x2af93730, SUC_NE_lexicon_tags_2 },
    { 0x67ed3731, SUC_NE_lexicon_tags_2 },
    { 0xe0d13731, SUC_NE_lexicon_tags_2 },
    { 0x99973733, SUC_NE_lexicon_tags_2 },
    { 0x6faf3734, SUC_NE_lexicon_tags_2 },
    { 0xe8973735, SUC_NE_lexicon_tags_2 },
    { 0x99293736, SUC_NE_lexicon_tags_2 },
    { 0x70f33736, SUC_NE_lexicon_tags_17 },
    { 0xc35372d, SUC_NE_lexicon_tags_2 },
    { 0xb3773737, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f3373d, SUC_NE_lexicon_tags_1 },
    { 0x1577373e, SUC_NE_lexicon_tags_2 },
    { 0x65f5373f, SUC_NE_lexicon_tags_3 },
    { 0xf4d73740, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb3b3742, SUC_NE_lexicon_tags_24 },
    { 0x879d3743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcf374b, SUC_NE_lexicon_tags_2 },
    { 0x1019374c, SUC_NE_lexicon_tags_2 },
    { 0x9b99374d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62613751, SUC_NE_lexicon_tags_2 },
    { 0xc5813752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95713755, SUC_NE_lexicon_tags_2 },
    { 0xd0513755, SUC_NE_lexicon_tags_2 },
    { 0xc98f3756, SUC_NE_lexicon_tags_2 },
    { 0x60953756, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5277375a, SUC_NE_lexicon_tags_2 },
    { 0x3cc1375b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x627d375d, SUC_NE_lexicon_tags_2 },
    { 0xe2fb375d, SUC_NE_lexicon_tags_2 },
    { 0x818b375f, SUC_NE_lexicon_tags_2 },
    { 0x5d83375f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c53768, SUC_NE_lexicon_tags_2 },
    { 0x23053769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xddb5376b, SUC_NE_lexicon_tags_2 },
    { 0xf3e1376b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67f376f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10d3775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30653779, SUC_NE_lexicon_tags_2 },
    { 0xa1d7377a, SUC_NE_lexicon_tags_2 },
    { 0xb42d3779, SUC_NE_lexicon_tags_2 },
    { 0xccbb377b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b493786, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6553788, SUC_NE_lexicon_tags_2 },
    { 0x80853789, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444b378d, SUC_NE_lexicon_tags_2 },
    { 0x416d378e, SUC_NE_lexicon_tags_2 },
    { 0x9909378f, SUC_NE_lexicon_tags_2 },
    { 0x4b3d378d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb3794, SUC_NE_lexicon_tags_4 },
    { 0xf6433795, SUC_NE_lexicon_tags_2 },
    { 0x82733795, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76133799, SUC_NE_lexicon_tags_4 },
    { 0x1901379a, SUC_NE_lexicon_tags_2 },
    { 0x69ef379a, SUC_NE_lexicon_tags_2 },
    { 0x373b379a, SUC_NE_lexicon_tags_2 },
    { 0xc5d9379d, SUC_NE_lexicon_tags_24 },
    { 0x542b379e, SUC_NE_lexicon_tags_2 },
    { 0xb15b379f, SUC_NE_lexicon_tags_2 },
    { 0xd961379d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec4d37a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x330937a5, SUC_NE_lexicon_tags_3 },
    { 0xbaa937a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d737a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399d37ad, SUC_NE_lexicon_tags_2 },
    { 0x1d4d37ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbf37b2, SUC_NE_lexicon_tags_2 },
    { 0x710537b3, SUC_NE_lexicon_tags_2 },
    { 0xab6337b3, SUC_NE_lexicon_tags_2 },
    { 0xc09d37b5, SUC_NE_lexicon_tags_2 },
    { 0x75b737b4, SUC_NE_lexicon_tags_2 },
    { 0x8d5337b3, SUC_NE_lexicon_tags_2 },
    { 0xd7b337b8, SUC_NE_lexicon_tags_2 },
    { 0x30737b8, SUC_NE_lexicon_tags_2 },
    { 0xa97d37ba, SUC_NE_lexicon_tags_2 },
    { 0x5c737b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1f37c3, SUC_NE_lexicon_tags_2 },
    { 0xb5cb37c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x797f37c6, SUC_NE_lexicon_tags_2 },
    { 0x959937c7, SUC_NE_lexicon_tags_2 },
    { 0x7eaf37c8, SUC_NE_lexicon_tags_2 },
    { 0x346137c9, SUC_NE_lexicon_tags_19 },
    { 0x4db737c8, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cb37cd, SUC_NE_lexicon_tags_2 },
    { 0x33ef37ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9b937d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f537d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7b937d8, SUC_NE_lexicon_tags_2 },
    { 0xe26337d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x622f37dc, SUC_NE_lexicon_tags_2 },
    { 0x21db37dd, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6df37df, SUC_NE_lexicon_tags_2 },
    { 0xdd0937df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9c337e2, SUC_NE_lexicon_tags_2 },
    { 0xf68937e2, SUC_NE_lexicon_tags_2 },
    { 0xdc2937e4, SUC_NE_lexicon_tags_2 },
    { 0xf60937e5, SUC_NE_lexicon_tags_2 },
    { 0x362937e2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x10ab37e8, SUC_NE_lexicon_tags_3 },
    { 0x396537e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x550737eb, SUC_NE_lexicon_tags_2 },
    { 0x5c2137eb, SUC_NE_lexicon_tags_2 },
    { 0x1d9f37ec, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32137f0, SUC_NE_lexicon_tags_2 },
    { 0x741b37f0, SUC_NE_lexicon_tags_2 },
    { 0xfd5137f2, SUC_NE_lexicon_tags_2 },
    { 0xe2fd37f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ad37f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d337fc, SUC_NE_lexicon_tags_2 },
    { 0xafb137fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18f37ff, SUC_NE_lexicon_tags_4 },
    { 0x133f3800, SUC_NE_lexicon_tags_2 },
    { 0x9b2b37ff, SUC_NE_lexicon_tags_2 },
    { 0xddf93802, SUC_NE_lexicon_tags_2 },
    { 0xb72d3802, SUC_NE_lexicon_tags_2 },
    { 0x9a633800, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f380d, SUC_NE_lexicon_tags_2 },
    { 0xaaa9380e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ba93810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ad13812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4733817, SUC_NE_lexicon_tags_2 },
    { 0x19d73818, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74e5381a, SUC_NE_lexicon_tags_2 },
    { 0x927f381a, SUC_NE_lexicon_tags_2 },
    { 0x2db9381a, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5f3827, SUC_NE_lexicon_tags_2 },
    { 0xd333828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662d382d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bad3830, SUC_NE_lexicon_tags_2 },
    { 0x1c253831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970b3835, SUC_NE_lexicon_tags_2 },
    { 0x30633836, SUC_NE_lexicon_tags_2 },
    { 0xbe713837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18513839, SUC_NE_lexicon_tags_2 },
    { 0x1a7d383a, SUC_NE_lexicon_tags_2 },
    { 0xb8bb383a, SUC_NE_lexicon_tags_2 },
    { 0x56af383b, SUC_NE_lexicon_tags_2 },
    { 0x31f93839, SUC_NE_lexicon_tags_2 },
    { 0xed33383a, SUC_NE_lexicon_tags_3 },
    { 0x6713383f, SUC_NE_lexicon_tags_2 },
    { 0x6bf7383d, SUC_NE_lexicon_tags_1 },
    { 0xc29b383a, SUC_NE_lexicon_tags_2 },
    { 0xf98d3842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61853845, SUC_NE_lexicon_tags_2 },
    { 0xf42d3846, SUC_NE_lexicon_tags_2 },
    { 0xf54f3845, SUC_NE_lexicon_tags_2 },
    { 0x67b73846, SUC_NE_lexicon_tags_0 },
    { 0xaac73849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21a1384b, SUC_NE_lexicon_tags_2 },
    { 0xf21f384c, SUC_NE_lexicon_tags_2 },
    { 0xa631384d, SUC_NE_lexicon_tags_2 },
    { 0x73ff384e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x870f3850, SUC_NE_lexicon_tags_2 },
    { 0xaeed3851, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x1a6d3853, SUC_NE_lexicon_tags_2 },
    { 0x335b3854, SUC_NE_lexicon_tags_2 },
    { 0xce6b3854, SUC_NE_lexicon_tags_2 },
    { 0xf70b3856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0653859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc097385b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf637385e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe513861, SUC_NE_lexicon_tags_9 },
    { 0x59153861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3aeb3864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa06d3866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x784d3869, SUC_NE_lexicon_tags_2 },
    { 0x5499386a, SUC_NE_lexicon_tags_2 },
    { 0x213386b, SUC_NE_lexicon_tags_2 },
    { 0xda7386c, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x686f386f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd953871, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x21833873, SUC_NE_lexicon_tags_2 },
    { 0xc4833874, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc53d3876, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x842d3879, SUC_NE_lexicon_tags_2 },
    { 0x2d81387a, SUC_NE_lexicon_tags_5 },
    { 0x5147387a, SUC_NE_lexicon_tags_2 },
    { 0x26c9387c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf1387f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b673882, SUC_NE_lexicon_tags_2 },
    { 0xac5f3883, SUC_NE_lexicon_tags_2 },
    { 0x41433883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa73887, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa333889, SUC_NE_lexicon_tags_2 },
    { 0x45a73889, SUC_NE_lexicon_tags_2 },
    { 0x44eb388b, SUC_NE_lexicon_tags_2 },
    { 0xf5e33889, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x811388f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7491389e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da738a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd938a8, SUC_NE_lexicon_tags_2 },
    { 0x637538a8, SUC_NE_lexicon_tags_1 },
    { 0xf43d38aa, SUC_NE_lexicon_tags_2 },
    { 0x910d38ab, SUC_NE_lexicon_tags_94 },
    { 0xdceb38a8, SUC_NE_lexicon_tags_2 },
    { 0x1d1d38ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e2338af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310d38b3, SUC_NE_lexicon_tags_3 },
    { 0x314338b4, SUC_NE_lexicon_tags_2 },
    { 0x27ef38b5, SUC_NE_lexicon_tags_2 },
    { 0x622d38b6, SUC_NE_lexicon_tags_2 },
    { 0x542b38b7, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a738bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c2b38bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x912738c1, SUC_NE_lexicon_tags_2 },
    { 0xe40338c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcddd38c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66938c7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x792738ca, SUC_NE_lexicon_tags_2 },
    { 0x6ad738cb, SUC_NE_lexicon_tags_2 },
    { 0x655938cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x580138ce, SUC_NE_lexicon_tags_2 },
    { 0x774538cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x531538d1, SUC_NE_lexicon_tags_1 },
    { 0x869538d1, SUC_NE_lexicon_tags_2 },
    { 0x585338d3, SUC_NE_lexicon_tags_2 },
    { 0x617b38d1, SUC_NE_lexicon_tags_2 },
    { 0x501738d1, SUC_NE_lexicon_tags_2 },
    { 0xd51738d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f38d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84738e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31c138e3, SUC_NE_lexicon_tags_2 },
    { 0x7eff38e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622738ea, SUC_NE_lexicon_tags_2 },
    { 0xcf7d38eb, SUC_NE_lexicon_tags_2 },
    { 0xf21138ec, SUC_NE_lexicon_tags_2 },
    { 0x346738ed, SUC_NE_lexicon_tags_2 },
    { 0x21e938ee, SUC_NE_lexicon_tags_2 },
    { 0x918938ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c938f2, SUC_NE_lexicon_tags_19 },
    { 0xd7c538f2, SUC_NE_lexicon_tags_2 },
    { 0xffbd38f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0538f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce538fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe94538ff, SUC_NE_lexicon_tags_2 },
    { 0x972338ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1613906, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8d3911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f73915, SUC_NE_lexicon_tags_2 },
    { 0x635d3916, SUC_NE_lexicon_tags_2 },
    { 0x9b893917, SUC_NE_lexicon_tags_2 },
    { 0xf83b3918, SUC_NE_lexicon_tags_2 },
    { 0x56573915, SUC_NE_lexicon_tags_2 },
    { 0xc6d93918, SUC_NE_lexicon_tags_2 },
    { 0x9533919, SUC_NE_lexicon_tags_2 },
    { 0x19df391c, SUC_NE_lexicon_tags_2 },
    { 0xbe4b391a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b13924, SUC_NE_lexicon_tags_2 },
    { 0x72293924, SUC_NE_lexicon_tags_2 },
    { 0x868d3926, SUC_NE_lexicon_tags_2 },
    { 0x424d3927, SUC_NE_lexicon_tags_2 },
    { 0xbb533928, SUC_NE_lexicon_tags_2 },
    { 0x3f333929, SUC_NE_lexicon_tags_2 },
    { 0x474f3927, SUC_NE_lexicon_tags_2 },
    { 0x6e07392b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x604d392e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b893930, SUC_NE_lexicon_tags_2 },
    { 0x5b753931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53934, SUC_NE_lexicon_tags_2 },
    { 0xc6073935, SUC_NE_lexicon_tags_2 },
    { 0x37c53935, SUC_NE_lexicon_tags_2 },
    { 0x90c53937, SUC_NE_lexicon_tags_2 },
    { 0xbfa73937, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f393e, SUC_NE_lexicon_tags_2 },
    { 0x594f393e, SUC_NE_lexicon_tags_2 },
    { 0x7fb33940, SUC_NE_lexicon_tags_2 },
    { 0xbdc93941, SUC_NE_lexicon_tags_2 },
    { 0xfdc93942, SUC_NE_lexicon_tags_2 },
    { 0xbc01393f, SUC_NE_lexicon_tags_2 },
    { 0x752d3944, SUC_NE_lexicon_tags_2 },
    { 0x4f173941, SUC_NE_lexicon_tags_2 },
    { 0x98573946, SUC_NE_lexicon_tags_2 },
    { 0xe2ed3946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e03394a, SUC_NE_lexicon_tags_4 },
    { 0x5c67394b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f7b394d, SUC_NE_lexicon_tags_2 },
    { 0xc855394d, SUC_NE_lexicon_tags_2 },
    { 0x2be5394f, SUC_NE_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f3952, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e53954, SUC_NE_lexicon_tags_2 },
    { 0x89d3955, SUC_NE_lexicon_tags_7 },
    { 0x6af73955, SUC_NE_lexicon_tags_2 },
    { 0x3c133957, SUC_NE_lexicon_tags_2 },
    { 0x8f3f3958, SUC_NE_lexicon_tags_2 },
    { 0x70013959, SUC_NE_lexicon_tags_2 },
    { 0x262b3954, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0d3962, SUC_NE_lexicon_tags_18 },
    { 0x914b3963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d33966, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13493970, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d53974, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a593976, SUC_NE_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b13979, SUC_NE_lexicon_tags_2 },
    { 0x191397a, SUC_NE_lexicon_tags_58 },
    { 0x58b3979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa481397e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a8d3980, SUC_NE_lexicon_tags_2 },
    { 0xdbe53981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf693985, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac33988, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0c7398a, SUC_NE_lexicon_tags_2 },
    { 0xeded398b, SUC_NE_lexicon_tags_4 },
    { 0x188f398c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e13990, SUC_NE_lexicon_tags_3 },
    { 0xc65f3991, SUC_NE_lexicon_tags_2 },
    { 0x3e173990, SUC_NE_lexicon_tags_2 },
    { 0x780b3993, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a13997, SUC_NE_lexicon_tags_2 },
    { 0xafe13998, SUC_NE_lexicon_tags_2 },
    { 0xa45b3997, SUC_NE_lexicon_tags_17 },
    { 0xf9573997, SUC_NE_lexicon_tags_2 },
    { 0x9da73999, SUC_NE_lexicon_tags_2 },
    { 0x4851399c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23539a4, SUC_NE_lexicon_tags_3 },
    { 0xadad39a5, SUC_NE_lexicon_tags_2 },
    { 0x74a339a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49f539a8, SUC_NE_lexicon_tags_3 },
    { 0x994739a9, SUC_NE_lexicon_tags_2 },
    { 0xeb8d39a8, SUC_NE_lexicon_tags_2 },
    { 0x960f39a9, SUC_NE_lexicon_tags_2 },
    { 0x698539ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b339b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baf39b8, SUC_NE_lexicon_tags_2 },
    { 0xa5eb39b8, SUC_NE_lexicon_tags_2 },
    { 0xafad39ba, SUC_NE_lexicon_tags_2 },
    { 0x66f739ba, SUC_NE_lexicon_tags_4 },
    { 0xef5339bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c6939be, SUC_NE_lexicon_tags_2 },
    { 0xff0539be, SUC_NE_lexicon_tags_0 },
    { 0xe39f39c0, SUC_NE_lexicon_tags_2 },
    { 0x6e4739c1, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x995539c3, SUC_NE_lexicon_tags_2 },
    { 0xc16d39c4, SUC_NE_lexicon_tags_2 },
    { 0xed6939c5, SUC_NE_lexicon_tags_2 },
    { 0xa03739c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bf39c9, SUC_NE_lexicon_tags_2 },
    { 0xe75739ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9539cd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aff39d6, SUC_NE_lexicon_tags_2 },
    { 0x1eab39d7, SUC_NE_lexicon_tags_452 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db39dc, SUC_NE_lexicon_tags_2 },
    { 0x4f0139dd, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x926539df, SUC_NE_lexicon_tags_2 },
    { 0xa1f539e0, SUC_NE_lexicon_tags_2 },
    { 0xcdd339e1, SUC_NE_lexicon_tags_2 },
    { 0x53d39df, SUC_NE_lexicon_tags_2 },
    { 0xeb339e3, SUC_NE_lexicon_tags_2 },
    { 0x14ed39e4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87139ea, SUC_NE_lexicon_tags_2 },
    { 0x1da739eb, SUC_NE_lexicon_tags_2 },
    { 0xf3cb39ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26939ee, SUC_NE_lexicon_tags_2 },
    { 0x4f0739ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ef39f6, SUC_NE_lexicon_tags_0 },
    { 0x595939f7, SUC_NE_lexicon_tags_2 },
    { 0xd94939f8, SUC_NE_lexicon_tags_130 },
    { 0xef7139f7, SUC_NE_lexicon_tags_2 },
    { 0xbe5339fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ca139fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7539ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6673a02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28553a05, SUC_NE_lexicon_tags_2 },
    { 0x89f33a06, SUC_NE_lexicon_tags_2 },
    { 0xeea13a07, SUC_NE_lexicon_tags_2 },
    { 0x3b153a08, SUC_NE_lexicon_tags_2 },
    { 0x36bb3a08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60e53a0b, SUC_NE_lexicon_tags_3 },
    { 0xc0cb3a0c, SUC_NE_lexicon_tags_2 },
    { 0x65473a0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6f53a0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1253a11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cb33a13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e93a17, SUC_NE_lexicon_tags_3 },
    { 0x9f2f3a18, SUC_NE_lexicon_tags_2 },
    { 0x543f3a19, SUC_NE_lexicon_tags_3 },
    { 0xe82f3a1a, SUC_NE_lexicon_tags_2 },
    { 0x59ad3a19, SUC_NE_lexicon_tags_209 },
    { 0x1b5b3a1c, SUC_NE_lexicon_tags_2 },
    { 0xcd193a1d, SUC_NE_lexicon_tags_2 },
    { 0xd87f3a1d, SUC_NE_lexicon_tags_2 },
    { 0xf4213a1c, SUC_NE_lexicon_tags_2 },
    { 0x7533a20, SUC_NE_lexicon_tags_2 },
    { 0xf4333a1d, SUC_NE_lexicon_tags_2 },
    { 0xf70b3a1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33613a24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c7b3a26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b13a29, SUC_NE_lexicon_tags_2 },
    { 0x8a433a2a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c533a2f, SUC_NE_lexicon_tags_2 },
    { 0x22ab3a30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b53a34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4d3a3f, SUC_NE_lexicon_tags_2 },
    { 0xf5d3a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5473a42, SUC_NE_lexicon_tags_2 },
    { 0xa2c13a43, SUC_NE_lexicon_tags_2 },
    { 0x66fd3a43, SUC_NE_lexicon_tags_2 },
    { 0x7f7d3a42, SUC_NE_lexicon_tags_2 },
    { 0x9c633a45, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b33a4a, SUC_NE_lexicon_tags_3 },
    { 0x39113a4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0033a4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf3a52, SUC_NE_lexicon_tags_2 },
    { 0x52013a53, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x493d3a55, SUC_NE_lexicon_tags_2 },
    { 0x90b73a56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24fb3a58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7853a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52f93a65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb053a69, SUC_NE_lexicon_tags_2 },
    { 0x1af3a69, SUC_NE_lexicon_tags_2 },
    { 0x98313a6b, SUC_NE_lexicon_tags_2 },
    { 0x57bd3a6c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xad1d3a6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0493a70, SUC_NE_lexicon_tags_2 },
    { 0x91f53a71, SUC_NE_lexicon_tags_2 },
    { 0x12c53a70, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a1d3a74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaf3a77, SUC_NE_lexicon_tags_2 },
    { 0xddd3a78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93eb3a7a, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c7b3a7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10913a81, SUC_NE_lexicon_tags_188 },
    { 0xc5833a82, SUC_NE_lexicon_tags_2 },
    { 0xa04d3a83, SUC_NE_lexicon_tags_102 },
    { 0xb3293a81, SUC_NE_lexicon_tags_2 },
    { 0x68773a82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae73a87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3393a8c, SUC_NE_lexicon_tags_2 },
    { 0x55473a8d, SUC_NE_lexicon_tags_2 },
    { 0x97db3a8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20473a90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9033a92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3d3a96, SUC_NE_lexicon_tags_2 },
    { 0x153a97, SUC_NE_lexicon_tags_2 },
    { 0x40b13a98, SUC_NE_lexicon_tags_2 },
    { 0x19c13a99, SUC_NE_lexicon_tags_2 },
    { 0xa30d3a9a, SUC_NE_lexicon_tags_2 },
    { 0x93d93a9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cb13a9d, SUC_NE_lexicon_tags_2 },
    { 0x4b353a9e, SUC_NE_lexicon_tags_2 },
    { 0x694d3a9f, SUC_NE_lexicon_tags_2 },
    { 0xb1433aa0, SUC_NE_lexicon_tags_5 },
    { 0x1d313aa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7693aa5, SUC_NE_lexicon_tags_2 },
    { 0xd353aa6, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9573aa8, SUC_NE_lexicon_tags_2 },
    { 0x88153aa9, SUC_NE_lexicon_tags_2 },
    { 0x73b93aaa, SUC_NE_lexicon_tags_14 },
    { 0x1ab93aaa, SUC_NE_lexicon_tags_2 },
    { 0x62dd3aab, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f33ab2, SUC_NE_lexicon_tags_2 },
    { 0x4a2b3ab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc873ab7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69193ab9, SUC_NE_lexicon_tags_2 },
    { 0x16153aba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x375b3abc, SUC_NE_lexicon_tags_2 },
    { 0x353b3abd, SUC_NE_lexicon_tags_2 },
    { 0x99d93abc, SUC_NE_lexicon_tags_3 },
    { 0x10f13abf, SUC_NE_lexicon_tags_2 },
    { 0xc2fd3abf, SUC_NE_lexicon_tags_14 },
    { 0xa11f3abf, SUC_NE_lexicon_tags_2 },
    { 0x43513ac0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb213ac4, SUC_NE_lexicon_tags_2 },
    { 0xa5173ac5, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xb15b3ac7, SUC_NE_lexicon_tags_2 },
    { 0x86193ac8, SUC_NE_lexicon_tags_2 },
    { 0xf4d73ac9, SUC_NE_lexicon_tags_2 },
    { 0xa7b93ac9, SUC_NE_lexicon_tags_2 },
    { 0x670d3acb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x493ace, SUC_NE_lexicon_tags_2 },
    { 0xcdc13acf, SUC_NE_lexicon_tags_2 },
    { 0x3c593acf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdbd3ad2, SUC_NE_lexicon_tags_2 },
    { 0xa59d3ad3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a33ada, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bf3ae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99cd3ae3, SUC_NE_lexicon_tags_2 },
    { 0x12b33ae3, SUC_NE_lexicon_tags_5 },
    { 0xbdcf3ae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b033af1, SUC_NE_lexicon_tags_2 },
    { 0xd2393af2, SUC_NE_lexicon_tags_2 },
    { 0xd0593af3, SUC_NE_lexicon_tags_2 },
    { 0x47793af3, SUC_NE_lexicon_tags_2 },
    { 0xe6693af5, SUC_NE_lexicon_tags_3 },
    { 0x7a613af3, SUC_NE_lexicon_tags_2 },
    { 0x422f3af1, SUC_NE_lexicon_tags_2 },
    { 0x133b3af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a93afe, SUC_NE_lexicon_tags_2 },
    { 0x6f353aff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7d3b03, SUC_NE_lexicon_tags_2 },
    { 0x9f873b04, SUC_NE_lexicon_tags_2 },
    { 0xed533b03, SUC_NE_lexicon_tags_1 },
    { 0xb9013b06, SUC_NE_lexicon_tags_2 },
    { 0xc4b3b07, SUC_NE_lexicon_tags_2 },
    { 0xccf53b06, SUC_NE_lexicon_tags_2 },
    { 0xd8693b09, SUC_NE_lexicon_tags_2 },
    { 0xdd0b3b0a, SUC_NE_lexicon_tags_2 },
    { 0x64173b03, SUC_NE_lexicon_tags_2 },
    { 0xb46b3b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1db33b0e, SUC_NE_lexicon_tags_2 },
    { 0xe2ed3b0f, SUC_NE_lexicon_tags_3 },
    { 0x19353b10, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79633b17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146f3b1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89693b1e, SUC_NE_lexicon_tags_2 },
    { 0xb79f3b1f, SUC_NE_lexicon_tags_59 },
    { 0x3f6d3b1f, SUC_NE_lexicon_tags_2 },
    { 0xa4a73b20, SUC_NE_lexicon_tags_2 },
    { 0xbe833b21, SUC_NE_lexicon_tags_2 },
    { 0xf1ff3b20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39df3b26, SUC_NE_lexicon_tags_2 },
    { 0xaf153b27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314d3b30, SUC_NE_lexicon_tags_2 },
    { 0xd8213b31, SUC_NE_lexicon_tags_2 },
    { 0x14df3b30, SUC_NE_lexicon_tags_2 },
    { 0xfecb3b31, SUC_NE_lexicon_tags_2 },
    { 0x4c4d3b34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f073b3a, SUC_NE_lexicon_tags_2 },
    { 0xb4533b3a, SUC_NE_lexicon_tags_2 },
    { 0x92e13b3a, SUC_NE_lexicon_tags_2 },
    { 0x6a813b3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89573b41, SUC_NE_lexicon_tags_2 },
    { 0xba733b41, SUC_NE_lexicon_tags_75 },
    { 0xed793b43, SUC_NE_lexicon_tags_2 },
    { 0x23bf3b44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ef3b49, SUC_NE_lexicon_tags_2 },
    { 0x9f33b4a, SUC_NE_lexicon_tags_2 },
    { 0x43f33b49, SUC_NE_lexicon_tags_2 },
    { 0xd8033b4a, SUC_NE_lexicon_tags_2 },
    { 0x77873b49, SUC_NE_lexicon_tags_2 },
    { 0x7253b4d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfcf13b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fb3b54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7d3b58, SUC_NE_lexicon_tags_140 },
    { 0x9f8f3b58, SUC_NE_lexicon_tags_2 },
    { 0x99213b5a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea33b5e, SUC_NE_lexicon_tags_2 },
    { 0xa1793b5e, SUC_NE_lexicon_tags_2 },
    { 0xcf453b5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86433b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89a73b69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd8b3b6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76c13b6d, SUC_NE_lexicon_tags_7 },
    { 0xf5c73b6e, SUC_NE_lexicon_tags_2 },
    { 0x88613b6f, SUC_NE_lexicon_tags_32 },
    { 0x99ff3b6e, SUC_NE_lexicon_tags_2 },
    { 0xe8a93b6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ded3b74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1493b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd3b7a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9af3b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03f3b80, SUC_NE_lexicon_tags_2 },
    { 0xa413b80, SUC_NE_lexicon_tags_2 },
    { 0xefcf3b82, SUC_NE_lexicon_tags_2 },
    { 0x7ae73b80, SUC_NE_lexicon_tags_18 },
    { 0x83633b84, SUC_NE_lexicon_tags_2 },
    { 0x25b73b85, SUC_NE_lexicon_tags_2 },
    { 0x5f2f3b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41b93b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe92d3b8a, SUC_NE_lexicon_tags_2 },
    { 0x846b3b8a, SUC_NE_lexicon_tags_2 },
    { 0x3b513b8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb7b3b8e, SUC_NE_lexicon_tags_2 },
    { 0xbf73b8e, SUC_NE_lexicon_tags_2 },
    { 0x2fcb3b8e, SUC_NE_lexicon_tags_2 },
    { 0xfb113b91, SUC_NE_lexicon_tags_4 },
    { 0x28993b92, SUC_NE_lexicon_tags_2 },
    { 0x67673b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f693b97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17f13b99, SUC_NE_lexicon_tags_5 },
    { 0xd2d73b99, SUC_NE_lexicon_tags_2 },
    { 0xf3a53b99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb67f3b9d, SUC_NE_lexicon_tags_2 },
    { 0xa84d3b9d, SUC_NE_lexicon_tags_1 },
    { 0x6fcd3b9f, SUC_NE_lexicon_tags_2 },
    { 0x8fff3ba0, SUC_NE_lexicon_tags_2 },
    { 0x46d73ba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87553ba9, SUC_NE_lexicon_tags_2 },
    { 0x3d693ba9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3d3bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a753baf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaec33bb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41a53bb3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x235d3bb5, SUC_NE_lexicon_tags_2 },
    { 0xbf613bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaab3bb9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f33bbd, SUC_NE_lexicon_tags_2 },
    { 0xa9d73bbd, SUC_NE_lexicon_tags_4 },
    { 0xc36b3bbf, SUC_NE_lexicon_tags_7 },
    { 0x9cd73bc0, SUC_NE_lexicon_tags_23 },
    { 0x1d313bc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82733bc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf53bc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f3bca, SUC_NE_lexicon_tags_2 },
    { 0xca673bcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadf73bcd, SUC_NE_lexicon_tags_2 },
    { 0x289d3bcd, SUC_NE_lexicon_tags_370 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1b3bd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe153bdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dd3be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a8f3be5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68d3beb, SUC_NE_lexicon_tags_2 },
    { 0xd12d3bec, SUC_NE_lexicon_tags_8 },
    { 0xa6a13bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x277b3bf0, SUC_NE_lexicon_tags_32 },
    { 0xa193bf0, SUC_NE_lexicon_tags_19 },
    { 0x523b3bf1, SUC_NE_lexicon_tags_2 },
    { 0xf3013bf3, SUC_NE_lexicon_tags_2 },
    { 0xba853bf2, SUC_NE_lexicon_tags_2 },
    { 0x7eef3bf5, SUC_NE_lexicon_tags_2 },
    { 0x73e53bf6, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x4cfb3bf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aed3bfa, SUC_NE_lexicon_tags_2 },
    { 0xe5af3bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa93b3bfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f3c02, SUC_NE_lexicon_tags_2 },
    { 0x50f13c03, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x32153c05, SUC_NE_lexicon_tags_3 },
    { 0x75db3c05, SUC_NE_lexicon_tags_2 },
    { 0x96693c07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bf3c0b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e13c0d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a313c12, SUC_NE_lexicon_tags_2 },
    { 0x11e13c13, SUC_NE_lexicon_tags_2 },
    { 0xa4173c14, SUC_NE_lexicon_tags_2 },
    { 0xd54d3c15, SUC_NE_lexicon_tags_2 },
    { 0x32133c15, SUC_NE_lexicon_tags_40 },
    { 0x39153c16, SUC_NE_lexicon_tags_2 },
    { 0x1a2b3c16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b3c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99833c21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8d3c27, SUC_NE_lexicon_tags_2 },
    { 0x6d633c28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c33c2b, SUC_NE_lexicon_tags_4 },
    { 0x15853c2b, SUC_NE_lexicon_tags_1 },
    { 0x772d3c2b, SUC_NE_lexicon_tags_2 },
    { 0x5b1d3c2e, SUC_NE_lexicon_tags_2 },
    { 0x2a933c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4273c33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40113c36, SUC_NE_lexicon_tags_2 },
    { 0x15fb3c37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d1d3c39, SUC_NE_lexicon_tags_2 },
    { 0x574b3c3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12933c3c, SUC_NE_lexicon_tags_2 },
    { 0x58173c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe033c41, SUC_NE_lexicon_tags_2 },
    { 0x1fc33c42, SUC_NE_lexicon_tags_2 },
    { 0x195b3c43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc933c45, SUC_NE_lexicon_tags_2 },
    { 0xff7f3c45, SUC_NE_lexicon_tags_2 },
    { 0x60693c47, SUC_NE_lexicon_tags_18 },
    { 0xe8173c47, SUC_NE_lexicon_tags_2 },
    { 0xb88d3c47, SUC_NE_lexicon_tags_2 },
    { 0xceb3c4a, SUC_NE_lexicon_tags_2 },
    { 0x98093c4a, SUC_NE_lexicon_tags_2 },
    { 0xd73f3c4c, SUC_NE_lexicon_tags_2 },
    { 0x6d753c4d, SUC_NE_lexicon_tags_4 },
    { 0x78813c4d, SUC_NE_lexicon_tags_2 },
    { 0x89273c4f, SUC_NE_lexicon_tags_5 },
    { 0xd74d3c50, SUC_NE_lexicon_tags_3 },
    { 0x50533c50, SUC_NE_lexicon_tags_2 },
    { 0xe8133c51, SUC_NE_lexicon_tags_2 },
    { 0xfd753c53, SUC_NE_lexicon_tags_2 },
    { 0xb1a33c50, SUC_NE_lexicon_tags_2 },
    { 0x2a1b3c4b, SUC_NE_lexicon_tags_2 },
    { 0xfcfd3c56, SUC_NE_lexicon_tags_2 },
    { 0x3ca73c57, SUC_NE_lexicon_tags_2 },
    { 0xe1af3c58, SUC_NE_lexicon_tags_79 },
    { 0x138d3c59, SUC_NE_lexicon_tags_2 },
    { 0xdaf33c59, SUC_NE_lexicon_tags_2 },
    { 0x298f3c4c, SUC_NE_lexicon_tags_1 },
    { 0xfa533c5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc3d3c5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93633c61, SUC_NE_lexicon_tags_2 },
    { 0x65c53c61, SUC_NE_lexicon_tags_56 },
    { 0x9573c63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8cf3c65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fd3c6c, SUC_NE_lexicon_tags_2 },
    { 0x55493c6d, SUC_NE_lexicon_tags_3 },
    { 0xec013c6d, SUC_NE_lexicon_tags_7 },
    { 0xbac53c6e, SUC_NE_lexicon_tags_2 },
    { 0xb3973c70, SUC_NE_lexicon_tags_2 },
    { 0x81b33c70, SUC_NE_lexicon_tags_2 },
    { 0x6d5d3c72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x371f3c75, SUC_NE_lexicon_tags_19 },
    { 0x5b4f3c76, SUC_NE_lexicon_tags_2 },
    { 0xf3b33c77, SUC_NE_lexicon_tags_2 },
    { 0x2c833c77, SUC_NE_lexicon_tags_2 },
    { 0x28d33c77, SUC_NE_lexicon_tags_2 },
    { 0x81ff3c7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69e33c7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1af3c7f, SUC_NE_lexicon_tags_2 },
    { 0xa04b3c80, SUC_NE_lexicon_tags_2 },
    { 0xe2073c81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb13c85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc30d3c87, SUC_NE_lexicon_tags_17 },
    { 0x633c88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3173c8b, SUC_NE_lexicon_tags_1 },
    { 0x1e0f3c8c, SUC_NE_lexicon_tags_2 },
    { 0x57033c8d, SUC_NE_lexicon_tags_2 },
    { 0x84db3c8c, SUC_NE_lexicon_tags_91 },
    { 0x9dff3c8d, SUC_NE_lexicon_tags_2 },
    { 0x986b3c8d, SUC_NE_lexicon_tags_2 },
    { 0x98393c91, SUC_NE_lexicon_tags_17 },
    { 0xcdf3c92, SUC_NE_lexicon_tags_2 },
    { 0xbbc93c91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ec13c95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18813c99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff3c9e, SUC_NE_lexicon_tags_24 },
    { 0xe13f3c9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb13ca4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2833ca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8813cb6, SUC_NE_lexicon_tags_19 },
    { 0x9add3cb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fb3cba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b93cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45773cc2, SUC_NE_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98f3cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4813cc8, SUC_NE_lexicon_tags_2 },
    { 0x71373cc9, SUC_NE_lexicon_tags_2 },
    { 0x96a93cc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9693ccc, SUC_NE_lexicon_tags_2 },
    { 0x5833ccd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb153cd3, SUC_NE_lexicon_tags_2 },
    { 0xaa853cd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed8b3cd6, SUC_NE_lexicon_tags_2 },
    { 0xbaf33cd6, SUC_NE_lexicon_tags_2 },
    { 0x46d53cd8, SUC_NE_lexicon_tags_2 },
    { 0x917d3cd7, SUC_NE_lexicon_tags_2 },
    { 0x5c5f3cda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35093ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430d3ce5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea13ce8, SUC_NE_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b13ceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a93cf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d313cf4, SUC_NE_lexicon_tags_2 },
    { 0xcc293cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcf3cfa, SUC_NE_lexicon_tags_2 },
    { 0x82c13cfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72233cfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44a73cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98893d03, SUC_NE_lexicon_tags_2 },
    { 0x40273d04, SUC_NE_lexicon_tags_2 },
    { 0xbd833d05, SUC_NE_lexicon_tags_2 },
    { 0x17e33d03, SUC_NE_lexicon_tags_2 },
    { 0xdb233d06, SUC_NE_lexicon_tags_2 },
    { 0xebf3d08, SUC_NE_lexicon_tags_1 },
    { 0x9abd3d09, SUC_NE_lexicon_tags_2 },
    { 0x4b773d09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766f3d0e, SUC_NE_lexicon_tags_2 },
    { 0xf57d3d0f, SUC_NE_lexicon_tags_2 },
    { 0xd2133d10, SUC_NE_lexicon_tags_2 },
    { 0x48253d0f, SUC_NE_lexicon_tags_2 },
    { 0xcaf93d12, SUC_NE_lexicon_tags_2 },
    { 0x423f3d13, SUC_NE_lexicon_tags_2 },
    { 0x46753d14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f853d16, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x70933d18, SUC_NE_lexicon_tags_350 },
    { 0x67c33d19, SUC_NE_lexicon_tags_2 },
    { 0x37e73d19, SUC_NE_lexicon_tags_2 },
    { 0x4c153d1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a73d1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d3d22, SUC_NE_lexicon_tags_7 },
    { 0x606d3d23, SUC_NE_lexicon_tags_1 },
    { 0xec193d24, SUC_NE_lexicon_tags_2 },
    { 0xf64b3d25, SUC_NE_lexicon_tags_2 },
    { 0xe14f3d26, SUC_NE_lexicon_tags_2 },
    { 0x4a153d27, SUC_NE_lexicon_tags_2 },
    { 0xc6133d27, SUC_NE_lexicon_tags_3 },
    { 0x8f113d26, SUC_NE_lexicon_tags_2 },
    { 0x11913d2a, SUC_NE_lexicon_tags_77 },
    { 0xa3bf3d2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad853d2d, SUC_NE_lexicon_tags_2 },
    { 0x9ae33d2e, SUC_NE_lexicon_tags_2 },
    { 0x9e053d2d, SUC_NE_lexicon_tags_5 },
    { 0x5ba73d2d, SUC_NE_lexicon_tags_2 },
    { 0xc6e33d31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc33d36, SUC_NE_lexicon_tags_2 },
    { 0x6ff93d36, SUC_NE_lexicon_tags_2 },
    { 0xe1893d37, SUC_NE_lexicon_tags_2 },
    { 0x93793d36, SUC_NE_lexicon_tags_2 },
    { 0x2d1f3d3a, SUC_NE_lexicon_tags_2 },
    { 0x2ab3d3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681d3d42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97553d4a, SUC_NE_lexicon_tags_2 },
    { 0x6b3d4a, SUC_NE_lexicon_tags_2 },
    { 0x70a13d4c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d93d50, SUC_NE_lexicon_tags_2 },
    { 0xc5f33d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23633d55, SUC_NE_lexicon_tags_2 },
    { 0x42c33d55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d13d58, SUC_NE_lexicon_tags_2 },
    { 0xc0933d59, SUC_NE_lexicon_tags_2 },
    { 0xb38b3d5a, SUC_NE_lexicon_tags_113 },
    { 0xaaeb3d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95693d5e, SUC_NE_lexicon_tags_2 },
    { 0x34793d5f, SUC_NE_lexicon_tags_139 },
    { 0, NULL },
    { 0xf11f3d61, SUC_NE_lexicon_tags_2 },
    { 0xa0ad3d61, SUC_NE_lexicon_tags_2 },
    { 0x249f3d63, SUC_NE_lexicon_tags_2 },
    { 0x639d3d64, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adf3d68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x681f3d6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b3d6f, SUC_NE_lexicon_tags_7 },
    { 0x9713d70, SUC_NE_lexicon_tags_2 },
    { 0x175b3d70, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b3d75, SUC_NE_lexicon_tags_2 },
    { 0xe8e53d76, SUC_NE_lexicon_tags_2 },
    { 0xee0d3d77, SUC_NE_lexicon_tags_2 },
    { 0x75c53d77, SUC_NE_lexicon_tags_2 },
    { 0x56113d79, SUC_NE_lexicon_tags_2 },
    { 0xe21b3d77, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde73d7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf3d82, SUC_NE_lexicon_tags_8 },
    { 0x14373d83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6793d87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dff3d8a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd9dd3d8c, SUC_NE_lexicon_tags_2 },
    { 0xead53d8d, SUC_NE_lexicon_tags_2 },
    { 0x79053d8e, SUC_NE_lexicon_tags_2 },
    { 0xbf453d8f, SUC_NE_lexicon_tags_1 },
    { 0xfe1f3d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62993d92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad3d97, SUC_NE_lexicon_tags_18 },
    { 0xc9393d98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81613d9a, SUC_NE_lexicon_tags_2 },
    { 0x99b93d9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e253d9d, SUC_NE_lexicon_tags_7 },
    { 0x2cc53d9d, SUC_NE_lexicon_tags_2 },
    { 0x386d3d9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb613da5, SUC_NE_lexicon_tags_2 },
    { 0xfa993da6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3613dab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9df3dad, SUC_NE_lexicon_tags_59 },
    { 0x49c13dad, SUC_NE_lexicon_tags_51 },
    { 0xa3093daf, SUC_NE_lexicon_tags_2 },
    { 0x22073daf, SUC_NE_lexicon_tags_2 },
    { 0xa8153dad, SUC_NE_lexicon_tags_3 },
    { 0x63f93db2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29213db4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43cb3db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a0b3db8, SUC_NE_lexicon_tags_3 },
    { 0x49933db8, SUC_NE_lexicon_tags_2 },
    { 0xfc033db8, SUC_NE_lexicon_tags_2 },
    { 0xe5fb3dba, SUC_NE_lexicon_tags_2 },
    { 0xda513dba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb33dc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc473dc3, SUC_NE_lexicon_tags_2 },
    { 0xd44f3dc3, SUC_NE_lexicon_tags_3 },
    { 0x9953dc5, SUC_NE_lexicon_tags_4 },
    { 0x32dd3dc6, SUC_NE_lexicon_tags_2 },
    { 0xebb3dc5, SUC_NE_lexicon_tags_2 },
    { 0x41453dc7, SUC_NE_lexicon_tags_2 },
    { 0x32053dc9, SUC_NE_lexicon_tags_2 },
    { 0xeb213dca, SUC_NE_lexicon_tags_2 },
    { 0xa93d3dca, SUC_NE_lexicon_tags_2 },
    { 0x71fb3dcc, SUC_NE_lexicon_tags_2 },
    { 0xda3b3dc5, SUC_NE_lexicon_tags_5 },
    { 0xb6d13dce, SUC_NE_lexicon_tags_2 },
    { 0x195f3dce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c6b3dd1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9673dd5, SUC_NE_lexicon_tags_2 },
    { 0xd6cb3dd5, SUC_NE_lexicon_tags_2 },
    { 0x1d6f3dd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x495f3dd9, SUC_NE_lexicon_tags_2 },
    { 0x5bb33dda, SUC_NE_lexicon_tags_2 },
    { 0xed673ddb, SUC_NE_lexicon_tags_1 },
    { 0x871f3ddc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeab3de0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53773de5, SUC_NE_lexicon_tags_2 },
    { 0x39793de5, SUC_NE_lexicon_tags_30 },
    { 0xd6353de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f8b3de9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7a13deb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7333df0, SUC_NE_lexicon_tags_2 },
    { 0x47bf3df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a53df8, SUC_NE_lexicon_tags_2 },
    { 0x910b3df9, SUC_NE_lexicon_tags_2 },
    { 0x5aa53df9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5bb3dfc, SUC_NE_lexicon_tags_58 },
    { 0x70093dfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46f13dff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d553e02, SUC_NE_lexicon_tags_2 },
    { 0x86353e03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe73e08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7cf3e0a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x434f3e0d, SUC_NE_lexicon_tags_2 },
    { 0xef953e0d, SUC_NE_lexicon_tags_11 },
    { 0xfbcf3e0f, SUC_NE_lexicon_tags_2 },
    { 0xc3d3e0f, SUC_NE_lexicon_tags_2 },
    { 0x901f3e10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31713e16, SUC_NE_lexicon_tags_2 },
    { 0x4e773e17, SUC_NE_lexicon_tags_2 },
    { 0x55a33e18, SUC_NE_lexicon_tags_2 },
    { 0x59c73e18, SUC_NE_lexicon_tags_2 },
    { 0x43c53e19, SUC_NE_lexicon_tags_2 },
    { 0x3fd3e1b, SUC_NE_lexicon_tags_3 },
    { 0xd95d3e17, SUC_NE_lexicon_tags_4 },
    { 0xe9a33e1d, SUC_NE_lexicon_tags_2 },
    { 0x86513e1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31093e21, SUC_NE_lexicon_tags_2 },
    { 0x24473e21, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x91b53e24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2553e29, SUC_NE_lexicon_tags_2 },
    { 0xffd73e2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc61d3e2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d3e31, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x75573e33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ab3e38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48853e3a, SUC_NE_lexicon_tags_2 },
    { 0x28e93e3b, SUC_NE_lexicon_tags_2 },
    { 0x51253e3c, SUC_NE_lexicon_tags_2 },
    { 0x68573e3d, SUC_NE_lexicon_tags_2 },
    { 0xa3513e3d, SUC_NE_lexicon_tags_2 },
    { 0xab033e3c, SUC_NE_lexicon_tags_2 },
    { 0x437b3e3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9193e46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75673e4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x474f3e4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x567d3e50, SUC_NE_lexicon_tags_2 },
    { 0xbc733e50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14253e54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17713e56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28273e58, SUC_NE_lexicon_tags_2 },
    { 0x68133e59, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0653e5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e53e60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fb3e66, SUC_NE_lexicon_tags_2 },
    { 0xc3653e67, SUC_NE_lexicon_tags_2 },
    { 0xdb573e66, SUC_NE_lexicon_tags_2 },
    { 0xc6933e68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x307d3e6c, SUC_NE_lexicon_tags_2 },
    { 0x893d3e6d, SUC_NE_lexicon_tags_2 },
    { 0x68673e6e, SUC_NE_lexicon_tags_2 },
    { 0xb613e6f, SUC_NE_lexicon_tags_2 },
    { 0x5f0d3e70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x515b3e73, SUC_NE_lexicon_tags_2 },
    { 0x45bb3e74, SUC_NE_lexicon_tags_2 },
    { 0xee033e75, SUC_NE_lexicon_tags_2 },
    { 0xdf1d3e74, SUC_NE_lexicon_tags_2 },
    { 0xe2313e76, SUC_NE_lexicon_tags_2 },
    { 0xbb13e76, SUC_NE_lexicon_tags_2 },
    { 0x27333e73, SUC_NE_lexicon_tags_2 },
    { 0xc7a93e7a, SUC_NE_lexicon_tags_2 },
    { 0x8c953e7b, SUC_NE_lexicon_tags_2 },
    { 0x301f3e7a, SUC_NE_lexicon_tags_2 },
    { 0x3033e7b, SUC_NE_lexicon_tags_3 },
    { 0x4b213e7e, SUC_NE_lexicon_tags_2 },
    { 0xd0313e7f, SUC_NE_lexicon_tags_1 },
    { 0xa85b3e7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15df3e84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a53e88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58af3e8a, SUC_NE_lexicon_tags_2 },
    { 0x39db3e8b, SUC_NE_lexicon_tags_2 },
    { 0x9a013e8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x835b3e8e, SUC_NE_lexicon_tags_2 },
    { 0x86413e8f, SUC_NE_lexicon_tags_7 },
    { 0x43af3e90, SUC_NE_lexicon_tags_2 },
    { 0x65a13e91, SUC_NE_lexicon_tags_2 },
    { 0x67473e91, SUC_NE_lexicon_tags_2 },
    { 0x5f13e92, SUC_NE_lexicon_tags_351 },
    { 0x23ef3e8f, SUC_NE_lexicon_tags_2 },
    { 0x81a33e94, SUC_NE_lexicon_tags_24 },
    { 0x7eb73e96, SUC_NE_lexicon_tags_2 },
    { 0xc5f33e97, SUC_NE_lexicon_tags_4 },
    { 0xf9af3e97, SUC_NE_lexicon_tags_2 },
    { 0xe68f3e99, SUC_NE_lexicon_tags_2 },
    { 0x7f953e98, SUC_NE_lexicon_tags_2 },
    { 0x9b3f3e99, SUC_NE_lexicon_tags_2 },
    { 0xf293e9c, SUC_NE_lexicon_tags_2 },
    { 0xdb2b3e9d, SUC_NE_lexicon_tags_4 },
    { 0xc6f33e9e, SUC_NE_lexicon_tags_25 },
    { 0xa95f3e9f, SUC_NE_lexicon_tags_2 },
    { 0x36d13e9f, SUC_NE_lexicon_tags_2 },
    { 0x280b3ea1, SUC_NE_lexicon_tags_2 },
    { 0xa1c33e9e, SUC_NE_lexicon_tags_2 },
    { 0x12953ea2, SUC_NE_lexicon_tags_2 },
    { 0x763b3ea4, SUC_NE_lexicon_tags_25 },
    { 0xe4153e9b, SUC_NE_lexicon_tags_2 },
    { 0xbc113e9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609f3eaa, SUC_NE_lexicon_tags_3 },
    { 0x2adf3eab, SUC_NE_lexicon_tags_2 },
    { 0x5def3eac, SUC_NE_lexicon_tags_2 },
    { 0xa88f3ead, SUC_NE_lexicon_tags_2 },
    { 0x495b3eae, SUC_NE_lexicon_tags_2 },
    { 0xeacd3eae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec93eb2, SUC_NE_lexicon_tags_59 },
    { 0x9b5d3eb2, SUC_NE_lexicon_tags_2 },
    { 0x26713eb4, SUC_NE_lexicon_tags_2 },
    { 0xb68d3eb4, SUC_NE_lexicon_tags_2 },
    { 0xd6593eb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dd3eb9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fb3ebd, SUC_NE_lexicon_tags_2 },
    { 0xca653ebe, SUC_NE_lexicon_tags_19 },
    { 0xeee93ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e33ec2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db3ec7, SUC_NE_lexicon_tags_2 },
    { 0x6d8d3ec7, SUC_NE_lexicon_tags_2 },
    { 0x2caf3ec7, SUC_NE_lexicon_tags_2 },
    { 0xab613eca, SUC_NE_lexicon_tags_2 },
    { 0x10fb3ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3f3ece, SUC_NE_lexicon_tags_2 },
    { 0x778d3ecf, SUC_NE_lexicon_tags_2 },
    { 0x51b73ece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87473ed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fef3ed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e573ed8, SUC_NE_lexicon_tags_2 },
    { 0x453f3ed8, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fd3edc, SUC_NE_lexicon_tags_2 },
    { 0x95993edc, SUC_NE_lexicon_tags_270 },
    { 0x6cf3ede, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcab3ee1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0133ee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96b3ee6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a53eea, SUC_NE_lexicon_tags_2 },
    { 0x2ba73eeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f3eee, SUC_NE_lexicon_tags_2 },
    { 0x8f513eee, SUC_NE_lexicon_tags_2 },
    { 0x11b53ef0, SUC_NE_lexicon_tags_2 },
    { 0x22893ef1, SUC_NE_lexicon_tags_2 },
    { 0x8ad93eee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2ed3ef4, SUC_NE_lexicon_tags_2 },
    { 0x8aa13ef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a53ef8, SUC_NE_lexicon_tags_2 },
    { 0x3a93ef9, SUC_NE_lexicon_tags_4 },
    { 0x750d3efa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeed3efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a9d3eff, SUC_NE_lexicon_tags_2 },
    { 0x70173f00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81553f04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb133f06, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38773f09, SUC_NE_lexicon_tags_2 },
    { 0x353f3f0a, SUC_NE_lexicon_tags_2 },
    { 0xb9393f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9573f0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74b3f14, SUC_NE_lexicon_tags_2 },
    { 0xa8b3f14, SUC_NE_lexicon_tags_167 },
    { 0x10153f16, SUC_NE_lexicon_tags_2 },
    { 0x87033f17, SUC_NE_lexicon_tags_2 },
    { 0xc5093f18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf33f1d, SUC_NE_lexicon_tags_2 },
    { 0x20e13f1d, SUC_NE_lexicon_tags_2 },
    { 0xf2b53f1d, SUC_NE_lexicon_tags_2 },
    { 0x70593f20, SUC_NE_lexicon_tags_24 },
    { 0xde033f1f, SUC_NE_lexicon_tags_2 },
    { 0x6fb73f22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4f13f24, SUC_NE_lexicon_tags_2 },
    { 0xd5a13f25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18693f27, SUC_NE_lexicon_tags_2 },
    { 0x26f93f28, SUC_NE_lexicon_tags_7 },
    { 0xe9333f29, SUC_NE_lexicon_tags_2 },
    { 0x1f753f29, SUC_NE_lexicon_tags_2 },
    { 0x7fd93f2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eef3f2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadcb3f31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d0f3f33, SUC_NE_lexicon_tags_2 },
    { 0x3f6b3f33, SUC_NE_lexicon_tags_2 },
    { 0xb6093f35, SUC_NE_lexicon_tags_2 },
    { 0x92fd3f36, SUC_NE_lexicon_tags_2 },
    { 0x357b3f37, SUC_NE_lexicon_tags_2 },
    { 0x52dd3f38, SUC_NE_lexicon_tags_2 },
    { 0x7a6d3f39, SUC_NE_lexicon_tags_2 },
    { 0xd3673f3a, SUC_NE_lexicon_tags_2 },
    { 0xc7c53f39, SUC_NE_lexicon_tags_2 },
    { 0xc3e33f34, SUC_NE_lexicon_tags_2 },
    { 0x61933f38, SUC_NE_lexicon_tags_0 },
    { 0xb1eb3f3e, SUC_NE_lexicon_tags_2 },
    { 0x47c93f3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe113f46, SUC_NE_lexicon_tags_2 },
    { 0x6173f47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b6d3f49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbeb3f4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecd3f4e, SUC_NE_lexicon_tags_2 },
    { 0x1f6b3f4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d973f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83193f58, SUC_NE_lexicon_tags_2 },
    { 0xd9fd3f59, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0xbf7b3f5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d933f5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b093f61, SUC_NE_lexicon_tags_2 },
    { 0xc4e53f62, SUC_NE_lexicon_tags_2 },
    { 0xe3e73f63, SUC_NE_lexicon_tags_2 },
    { 0xf9433f63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20b33f66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8013f6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xded73f70, SUC_NE_lexicon_tags_2 },
    { 0xdf13f71, SUC_NE_lexicon_tags_2 },
    { 0x1faf3f72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58e33f74, SUC_NE_lexicon_tags_2 },
    { 0x2fd3f75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a33f78, SUC_NE_lexicon_tags_2 },
    { 0xdc853f79, SUC_NE_lexicon_tags_2 },
    { 0x35393f7a, SUC_NE_lexicon_tags_1 },
    { 0x91273f7a, SUC_NE_lexicon_tags_4 },
    { 0x73ad3f7c, SUC_NE_lexicon_tags_2 },
    { 0xe0c93f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2b3f81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf3f84, SUC_NE_lexicon_tags_127 },
    { 0, NULL },
    { 0x5df73f86, SUC_NE_lexicon_tags_2 },
    { 0xb3a13f87, SUC_NE_lexicon_tags_2 },
    { 0x61193f86, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf53f8b, SUC_NE_lexicon_tags_2 },
    { 0x58473f8c, SUC_NE_lexicon_tags_2 },
    { 0x2f433f8d, SUC_NE_lexicon_tags_2 },
    { 0x9c093f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83df3f92, SUC_NE_lexicon_tags_2 },
    { 0x91993f92, SUC_NE_lexicon_tags_2 },
    { 0x69213f92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59ab3f96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e793f98, SUC_NE_lexicon_tags_2 },
    { 0xdd393f99, SUC_NE_lexicon_tags_2 },
    { 0xaca33f99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68ff3f9c, SUC_NE_lexicon_tags_2 },
    { 0xcffd3f9c, SUC_NE_lexicon_tags_2 },
    { 0xebb73f9e, SUC_NE_lexicon_tags_40 },
    { 0x3fff3f9f, SUC_NE_lexicon_tags_2 },
    { 0xf9e53f9e, SUC_NE_lexicon_tags_2 },
    { 0x6a4f3f9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d793fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ef3fa9, SUC_NE_lexicon_tags_2 },
    { 0x67fb3faa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x552b3fad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134f3fb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x253d3fb4, SUC_NE_lexicon_tags_2 },
    { 0x1c1f3fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66373fb9, SUC_NE_lexicon_tags_2 },
    { 0xbe533fba, SUC_NE_lexicon_tags_2 },
    { 0x24313fbb, SUC_NE_lexicon_tags_2 },
    { 0x7033fbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e053fc0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03b3fc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea73fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb3fce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xba573fd0, SUC_NE_lexicon_tags_2 },
    { 0x89af3fd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2df3fd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x455d3fd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57453fd9, SUC_NE_lexicon_tags_2 },
    { 0xdfc73fda, SUC_NE_lexicon_tags_5 },
    { 0x98593fda, SUC_NE_lexicon_tags_2 },
    { 0x68013fdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e93fdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a33fe2, SUC_NE_lexicon_tags_30 },
    { 0xb9f53fe3, SUC_NE_lexicon_tags_2 },
    { 0xba1f3fe4, SUC_NE_lexicon_tags_2 },
    { 0x63693fe5, SUC_NE_lexicon_tags_2 },
    { 0x2c333fe4, SUC_NE_lexicon_tags_2 },
    { 0x12853fe7, SUC_NE_lexicon_tags_4 },
    { 0x7b753fe8, SUC_NE_lexicon_tags_2 },
    { 0x793f3fe3, SUC_NE_lexicon_tags_32 },
    { 0xae153fea, SUC_NE_lexicon_tags_2 },
    { 0x1ccb3fe3, SUC_NE_lexicon_tags_3 },
    { 0x10b3feb, SUC_NE_lexicon_tags_2 },
    { 0x34613fed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f93ff2, SUC_NE_lexicon_tags_2 },
    { 0x78d73ff3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e33ff6, SUC_NE_lexicon_tags_2 },
    { 0x31773ff7, SUC_NE_lexicon_tags_2 },
    { 0x189f3ff8, SUC_NE_lexicon_tags_2 },
    { 0xe3313ff9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9f3ffb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9633ffd, SUC_NE_lexicon_tags_0 },
    { 0x1d3f3ffe, SUC_NE_lexicon_tags_2 },
    { 0x10cd3fff, SUC_NE_lexicon_tags_2 },
    { 0x6d434000, SUC_NE_lexicon_tags_2 },
    { 0x6c6d4000, SUC_NE_lexicon_tags_2 },
    { 0x11f53fff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2af4008, SUC_NE_lexicon_tags_2 },
    { 0xb9df4009, SUC_NE_lexicon_tags_2 },
    { 0xaceb4009, SUC_NE_lexicon_tags_2 },
    { 0x43cb400b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2115400e, SUC_NE_lexicon_tags_2 },
    { 0x5ec3400f, SUC_NE_lexicon_tags_2 },
    { 0x3ec7400f, SUC_NE_lexicon_tags_2 },
    { 0xb81b4010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bb4014, SUC_NE_lexicon_tags_2 },
    { 0x3b754014, SUC_NE_lexicon_tags_4 },
    { 0xb41d4016, SUC_NE_lexicon_tags_7 },
    { 0x59f54015, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce95401b, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b95401e, SUC_NE_lexicon_tags_40 },
    { 0x645f401f, SUC_NE_lexicon_tags_2 },
    { 0xc5cb401e, SUC_NE_lexicon_tags_58 },
    { 0xfc69401e, SUC_NE_lexicon_tags_2 },
    { 0x61794022, SUC_NE_lexicon_tags_4 },
    { 0x6ad74023, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d914025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e654027, SUC_NE_lexicon_tags_7 },
    { 0xf1ff4028, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdba1402a, SUC_NE_lexicon_tags_2 },
    { 0x4e7f402a, SUC_NE_lexicon_tags_2 },
    { 0xb89402c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6534031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3a94033, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcc34035, SUC_NE_lexicon_tags_1 },
    { 0x18094035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0b403c, SUC_NE_lexicon_tags_2 },
    { 0xf4cb403c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe77b403f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60174042, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26f4047, SUC_NE_lexicon_tags_12 },
    { 0x187b4048, SUC_NE_lexicon_tags_2 },
    { 0x20a94049, SUC_NE_lexicon_tags_2 },
    { 0x18e34049, SUC_NE_lexicon_tags_2 },
    { 0x7ce1404b, SUC_NE_lexicon_tags_2 },
    { 0x139404b, SUC_NE_lexicon_tags_24 },
    { 0xf95b404d, SUC_NE_lexicon_tags_2 },
    { 0xdccf404e, SUC_NE_lexicon_tags_2 },
    { 0x97a7404f, SUC_NE_lexicon_tags_2 },
    { 0xf9d5404d, SUC_NE_lexicon_tags_4 },
    { 0xdea1404e, SUC_NE_lexicon_tags_2 },
    { 0xe38b4052, SUC_NE_lexicon_tags_2 },
    { 0x6de74048, SUC_NE_lexicon_tags_2 },
    { 0xf014052, SUC_NE_lexicon_tags_2 },
    { 0x6f394054, SUC_NE_lexicon_tags_3 },
    { 0xd0cb4049, SUC_NE_lexicon_tags_2 },
    { 0x49db4057, SUC_NE_lexicon_tags_2 },
    { 0xa8034052, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978b4062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03f4067, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8594069, SUC_NE_lexicon_tags_2 },
    { 0x16f9406a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb075406d, SUC_NE_lexicon_tags_2 },
    { 0x453d406d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19534071, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76094078, SUC_NE_lexicon_tags_4 },
    { 0xe1854078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2e9407b, SUC_NE_lexicon_tags_2 },
    { 0xa66b407c, SUC_NE_lexicon_tags_2 },
    { 0x3bb3407c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a1d407f, SUC_NE_lexicon_tags_2 },
    { 0xa117407f, SUC_NE_lexicon_tags_2 },
    { 0xdaa3407f, SUC_NE_lexicon_tags_2 },
    { 0x89f4082, SUC_NE_lexicon_tags_2 },
    { 0x6a1b4083, SUC_NE_lexicon_tags_2 },
    { 0xe6574084, SUC_NE_lexicon_tags_2 },
    { 0x8fcf4084, SUC_NE_lexicon_tags_3 },
    { 0x3fd34085, SUC_NE_lexicon_tags_7 },
    { 0xd9cf4087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c97408c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e73408e, SUC_NE_lexicon_tags_2 },
    { 0x5cb3408e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a514091, SUC_NE_lexicon_tags_7 },
    { 0x7bf54092, SUC_NE_lexicon_tags_2 },
    { 0x5edb4092, SUC_NE_lexicon_tags_5 },
    { 0x942d4092, SUC_NE_lexicon_tags_2 },
    { 0xeb074095, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b1d4097, SUC_NE_lexicon_tags_2 },
    { 0x54834098, SUC_NE_lexicon_tags_2 },
    { 0x3c714098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe063409d, SUC_NE_lexicon_tags_2 },
    { 0x6601409d, SUC_NE_lexicon_tags_2 },
    { 0xc1d9409d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f140a6, SUC_NE_lexicon_tags_2 },
    { 0xafdd40a7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eb40ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf0b40af, SUC_NE_lexicon_tags_2 },
    { 0x4a3b40af, SUC_NE_lexicon_tags_2 },
    { 0x644f40b0, SUC_NE_lexicon_tags_2 },
    { 0x2f8f40b2, SUC_NE_lexicon_tags_2 },
    { 0xb3e940b2, SUC_NE_lexicon_tags_4 },
    { 0xb26140b0, SUC_NE_lexicon_tags_2 },
    { 0x4aeb40b5, SUC_NE_lexicon_tags_2 },
    { 0x6b8140b5, SUC_NE_lexicon_tags_2 },
    { 0x295540b6, SUC_NE_lexicon_tags_2 },
    { 0xf140b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8e140ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0540bf, SUC_NE_lexicon_tags_2 },
    { 0xcbc540bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa740c5, SUC_NE_lexicon_tags_2 },
    { 0xb71d40c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b940c9, SUC_NE_lexicon_tags_2 },
    { 0xe0f40c9, SUC_NE_lexicon_tags_2 },
    { 0xc81340cb, SUC_NE_lexicon_tags_2 },
    { 0x71340cc, SUC_NE_lexicon_tags_2 },
    { 0x90b40cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5d40d0, SUC_NE_lexicon_tags_2 },
    { 0x35a540d0, SUC_NE_lexicon_tags_2 },
    { 0x3fc740d2, SUC_NE_lexicon_tags_7 },
    { 0x123740d3, SUC_NE_lexicon_tags_2 },
    { 0x714f40d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x644d40d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9db40d8, SUC_NE_lexicon_tags_275 },
    { 0xf95b40d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x563940dc, SUC_NE_lexicon_tags_3 },
    { 0xba4f40dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28d40e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdb40e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafbf40e6, SUC_NE_lexicon_tags_2 },
    { 0x39540e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ea740e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69740eb, SUC_NE_lexicon_tags_2 },
    { 0xbc9f40ec, SUC_NE_lexicon_tags_2 },
    { 0xf5bd40ec, SUC_NE_lexicon_tags_2 },
    { 0xb64d40ee, SUC_NE_lexicon_tags_2 },
    { 0x230940ef, SUC_NE_lexicon_tags_30 },
    { 0x5ee940ec, SUC_NE_lexicon_tags_2 },
    { 0x9a0140ec, SUC_NE_lexicon_tags_4 },
    { 0xaf8740f1, SUC_NE_lexicon_tags_2 },
    { 0xb8e940ee, SUC_NE_lexicon_tags_2 },
    { 0xa94f40f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5140fa, SUC_NE_lexicon_tags_4 },
    { 0x50c540fb, SUC_NE_lexicon_tags_2 },
    { 0x4c7940fc, SUC_NE_lexicon_tags_2 },
    { 0xa9f940fd, SUC_NE_lexicon_tags_2 },
    { 0x7acd40fe, SUC_NE_lexicon_tags_2 },
    { 0x158b40fe, SUC_NE_lexicon_tags_2 },
    { 0x930140fe, SUC_NE_lexicon_tags_2 },
    { 0x1c7540ff, SUC_NE_lexicon_tags_2 },
    { 0x3c7540ff, SUC_NE_lexicon_tags_2 },
    { 0x838940fe, SUC_NE_lexicon_tags_2 },
    { 0x4a9f4104, SUC_NE_lexicon_tags_2 },
    { 0x2f614100, SUC_NE_lexicon_tags_2 },
    { 0x3aa14106, SUC_NE_lexicon_tags_2 },
    { 0x9f3b40fc, SUC_NE_lexicon_tags_2 },
    { 0x7fe74107, SUC_NE_lexicon_tags_2 },
    { 0x4fc140fc, SUC_NE_lexicon_tags_7 },
    { 0x891b4107, SUC_NE_lexicon_tags_2 },
    { 0xed174107, SUC_NE_lexicon_tags_2 },
    { 0xd789410c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96114110, SUC_NE_lexicon_tags_2 },
    { 0x72534110, SUC_NE_lexicon_tags_2 },
    { 0xf09f4112, SUC_NE_lexicon_tags_2 },
    { 0xf60d4112, SUC_NE_lexicon_tags_2 },
    { 0xd07f4114, SUC_NE_lexicon_tags_4 },
    { 0xd3274111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0814119, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d411c, SUC_NE_lexicon_tags_2 },
    { 0xf9ad411c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd4120, SUC_NE_lexicon_tags_3 },
    { 0x63714121, SUC_NE_lexicon_tags_2 },
    { 0x2b0f4122, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1db4125, SUC_NE_lexicon_tags_2 },
    { 0xc3194126, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x134d4128, SUC_NE_lexicon_tags_2 },
    { 0x4974129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd3b412b, SUC_NE_lexicon_tags_2 },
    { 0x4085412b, SUC_NE_lexicon_tags_2 },
    { 0x83f3412d, SUC_NE_lexicon_tags_2 },
    { 0x2047412e, SUC_NE_lexicon_tags_2 },
    { 0x71cd412f, SUC_NE_lexicon_tags_2 },
    { 0x766d412f, SUC_NE_lexicon_tags_2 },
    { 0x1f894131, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12794134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c454136, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61414139, SUC_NE_lexicon_tags_3 },
    { 0xac49413a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e07413c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe37b413e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe22d4140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a794142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf34146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64c94148, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7f414a, SUC_NE_lexicon_tags_2 },
    { 0x7d9b414b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0294159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd389415e, SUC_NE_lexicon_tags_2 },
    { 0x560f415f, SUC_NE_lexicon_tags_2 },
    { 0x40fb415f, SUC_NE_lexicon_tags_4 },
    { 0x1ef1415f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd4165, SUC_NE_lexicon_tags_2 },
    { 0x7e4d4166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e374169, SUC_NE_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe54172, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76054176, SUC_NE_lexicon_tags_2 },
    { 0xf6b94177, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x81494179, SUC_NE_lexicon_tags_2 },
    { 0xd18d4179, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23af417c, SUC_NE_lexicon_tags_2 },
    { 0xb829417d, SUC_NE_lexicon_tags_3 },
    { 0x3021417d, SUC_NE_lexicon_tags_2 },
    { 0xb761417d, SUC_NE_lexicon_tags_2 },
    { 0x7bf4180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57d4183, SUC_NE_lexicon_tags_2 },
    { 0xb6694184, SUC_NE_lexicon_tags_3 },
    { 0xe0d54185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfda54187, SUC_NE_lexicon_tags_3 },
    { 0x6dbd4188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c7418d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba94194, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169b4199, SUC_NE_lexicon_tags_2 },
    { 0x2fb54199, SUC_NE_lexicon_tags_2 },
    { 0x3c0b4199, SUC_NE_lexicon_tags_2 },
    { 0x2bff419c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd677419e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24341a1, SUC_NE_lexicon_tags_2 },
    { 0x46b41a2, SUC_NE_lexicon_tags_7 },
    { 0x35f341a2, SUC_NE_lexicon_tags_2 },
    { 0xcfc141a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x584541a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8941aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dd141ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x490341af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d541b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0fb41b5, SUC_NE_lexicon_tags_2 },
    { 0x7ee741b6, SUC_NE_lexicon_tags_2 },
    { 0xdb1741b7, SUC_NE_lexicon_tags_2 },
    { 0x53cb41b8, SUC_NE_lexicon_tags_2 },
    { 0x1a6341b9, SUC_NE_lexicon_tags_2 },
    { 0x422f41ba, SUC_NE_lexicon_tags_2 },
    { 0x667741bb, SUC_NE_lexicon_tags_7 },
    { 0xc3b141ba, SUC_NE_lexicon_tags_2 },
    { 0xc7bf41bb, SUC_NE_lexicon_tags_89 },
    { 0x3d1541b9, SUC_NE_lexicon_tags_2 },
    { 0x2cd941bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57e741c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24141c6, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x736d41c8, SUC_NE_lexicon_tags_2 },
    { 0x30b141c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56d41cc, SUC_NE_lexicon_tags_2 },
    { 0x29ed41cd, SUC_NE_lexicon_tags_2 },
    { 0x1c541cd, SUC_NE_lexicon_tags_19 },
    { 0x2b2741ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60db41d9, SUC_NE_lexicon_tags_2 },
    { 0x56c941da, SUC_NE_lexicon_tags_2 },
    { 0x9df941d9, SUC_NE_lexicon_tags_2 },
    { 0xaab741d9, SUC_NE_lexicon_tags_17 },
    { 0xa7fd41da, SUC_NE_lexicon_tags_2 },
    { 0xabab41da, SUC_NE_lexicon_tags_2 },
    { 0x1b6f41da, SUC_NE_lexicon_tags_2 },
    { 0x2c2f41e0, SUC_NE_lexicon_tags_2 },
    { 0x106f41e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc541e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72ff41e9, SUC_NE_lexicon_tags_2 },
    { 0x59d541e9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcd41ed, SUC_NE_lexicon_tags_2 },
    { 0x7f0741ee, SUC_NE_lexicon_tags_2 },
    { 0x8f7741ef, SUC_NE_lexicon_tags_2 },
    { 0x183541f0, SUC_NE_lexicon_tags_23 },
    { 0x138141ed, SUC_NE_lexicon_tags_2 },
    { 0xbd9d41f2, SUC_NE_lexicon_tags_2 },
    { 0xec341f2, SUC_NE_lexicon_tags_2 },
    { 0x8bd941f4, SUC_NE_lexicon_tags_2 },
    { 0x535541f5, SUC_NE_lexicon_tags_2 },
    { 0xa9ad41ee, SUC_NE_lexicon_tags_2 },
    { 0xd29341f7, SUC_NE_lexicon_tags_2 },
    { 0x1cc741f8, SUC_NE_lexicon_tags_1 },
    { 0xb5c341f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aed41fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x306941fd, SUC_NE_lexicon_tags_13 },
    { 0x1c4b41fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x507d4200, SUC_NE_lexicon_tags_2 },
    { 0x418d4201, SUC_NE_lexicon_tags_2 },
    { 0xe7414200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d714205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84874209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c63420b, SUC_NE_lexicon_tags_2 },
    { 0xb823420c, SUC_NE_lexicon_tags_2 },
    { 0xeea9420d, SUC_NE_lexicon_tags_4 },
    { 0xa247420e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93f4211, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb74214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e374216, SUC_NE_lexicon_tags_7 },
    { 0x72ed4217, SUC_NE_lexicon_tags_2 },
    { 0xb094218, SUC_NE_lexicon_tags_2 },
    { 0x17f4216, SUC_NE_lexicon_tags_2 },
    { 0x3c9d421a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9275421d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ab4220, SUC_NE_lexicon_tags_2 },
    { 0xdc314220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24074227, SUC_NE_lexicon_tags_2 },
    { 0x99c74228, SUC_NE_lexicon_tags_7 },
    { 0x91214227, SUC_NE_lexicon_tags_2 },
    { 0xceb7422a, SUC_NE_lexicon_tags_2 },
    { 0x8093422a, SUC_NE_lexicon_tags_2 },
    { 0x6425422c, SUC_NE_lexicon_tags_7 },
    { 0x5a2d422d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61b4233, SUC_NE_lexicon_tags_2 },
    { 0xefb14234, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3469423a, SUC_NE_lexicon_tags_2 },
    { 0x2ae1423b, SUC_NE_lexicon_tags_2 },
    { 0x49b423b, SUC_NE_lexicon_tags_2 },
    { 0x926f423b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89df423f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa43d4241, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bb4245, SUC_NE_lexicon_tags_2 },
    { 0x4b874245, SUC_NE_lexicon_tags_2 },
    { 0xbb874245, SUC_NE_lexicon_tags_2 },
    { 0xea734248, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216b424c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x910f424e, SUC_NE_lexicon_tags_2 },
    { 0xb18b424f, SUC_NE_lexicon_tags_1 },
    { 0xaaed4250, SUC_NE_lexicon_tags_2 },
    { 0x45cd4251, SUC_NE_lexicon_tags_3 },
    { 0xb5754252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda13425a, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xad7f425c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a69425f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf314267, SUC_NE_lexicon_tags_2 },
    { 0xe60b4268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x409f426a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe153426c, SUC_NE_lexicon_tags_2 },
    { 0x9c59426d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a74270, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94b54272, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a34279, SUC_NE_lexicon_tags_2 },
    { 0xe8ad4279, SUC_NE_lexicon_tags_2 },
    { 0x5b427a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe14285, SUC_NE_lexicon_tags_2 },
    { 0x5bb4286, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55eb4289, SUC_NE_lexicon_tags_2 },
    { 0x5cdf4289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf51f428c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x562b428e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dff4292, SUC_NE_lexicon_tags_2 },
    { 0xb5f94292, SUC_NE_lexicon_tags_24 },
    { 0xf5e34293, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bcf4296, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cf429a, SUC_NE_lexicon_tags_2 },
    { 0x7b6b429a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdc7429d, SUC_NE_lexicon_tags_2 },
    { 0xcf7d429e, SUC_NE_lexicon_tags_2 },
    { 0x424d429f, SUC_NE_lexicon_tags_2 },
    { 0x7913429d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2742a4, SUC_NE_lexicon_tags_2 },
    { 0x40142a4, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274b42ab, SUC_NE_lexicon_tags_3 },
    { 0xe1a942ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x184342af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc942b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30942b5, SUC_NE_lexicon_tags_2 },
    { 0x26d342b5, SUC_NE_lexicon_tags_2 },
    { 0xf74b42b7, SUC_NE_lexicon_tags_2 },
    { 0x886142b8, SUC_NE_lexicon_tags_2 },
    { 0x744d42b6, SUC_NE_lexicon_tags_2 },
    { 0x15142ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d42bd, SUC_NE_lexicon_tags_2 },
    { 0xc55f42be, SUC_NE_lexicon_tags_2 },
    { 0xf33342bd, SUC_NE_lexicon_tags_98 },
    { 0x290342c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bb42c3, SUC_NE_lexicon_tags_2 },
    { 0xebf342c3, SUC_NE_lexicon_tags_2 },
    { 0xc2bf42c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f42d1, SUC_NE_lexicon_tags_2 },
    { 0xc86d42d2, SUC_NE_lexicon_tags_2 },
    { 0x587b42d3, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xbd6b42d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d2142d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b142e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7db42e4, SUC_NE_lexicon_tags_2 },
    { 0x3c942e5, SUC_NE_lexicon_tags_2 },
    { 0xe9db42e4, SUC_NE_lexicon_tags_2 },
    { 0x3c6b42e6, SUC_NE_lexicon_tags_2 },
    { 0xa8f342e8, SUC_NE_lexicon_tags_45 },
    { 0xea1742e9, SUC_NE_lexicon_tags_2 },
    { 0x717142e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673742f2, SUC_NE_lexicon_tags_2 },
    { 0x363742f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2cf42f5, SUC_NE_lexicon_tags_7 },
    { 0x6e1942f5, SUC_NE_lexicon_tags_2 },
    { 0xaa0142f7, SUC_NE_lexicon_tags_2 },
    { 0x856742f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd80f42fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x358542fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67b4300, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x68254302, SUC_NE_lexicon_tags_2 },
    { 0xe11d4302, SUC_NE_lexicon_tags_2 },
    { 0xdceb4304, SUC_NE_lexicon_tags_2 },
    { 0x54674303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69914307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9f430a, SUC_NE_lexicon_tags_2 },
    { 0x4ed430b, SUC_NE_lexicon_tags_2 },
    { 0xeb4b430b, SUC_NE_lexicon_tags_2 },
    { 0x58cf430b, SUC_NE_lexicon_tags_2 },
    { 0x5365430e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9614311, SUC_NE_lexicon_tags_2 },
    { 0x11814311, SUC_NE_lexicon_tags_2 },
    { 0x72f14313, SUC_NE_lexicon_tags_3 },
    { 0xb7f34314, SUC_NE_lexicon_tags_2 },
    { 0x12854315, SUC_NE_lexicon_tags_2 },
    { 0xd13b4316, SUC_NE_lexicon_tags_2 },
    { 0xfa234317, SUC_NE_lexicon_tags_2 },
    { 0xf93f4318, SUC_NE_lexicon_tags_7 },
    { 0xc3274319, SUC_NE_lexicon_tags_2 },
    { 0xc4114319, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x75a3431c, SUC_NE_lexicon_tags_2 },
    { 0xeab7431d, SUC_NE_lexicon_tags_1 },
    { 0xead431e, SUC_NE_lexicon_tags_13 },
    { 0x1547431c, SUC_NE_lexicon_tags_2 },
    { 0x1b94320, SUC_NE_lexicon_tags_2 },
    { 0x9054320, SUC_NE_lexicon_tags_1 },
    { 0x8bf4321, SUC_NE_lexicon_tags_4 },
    { 0x33eb4323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774328, SUC_NE_lexicon_tags_2 },
    { 0x2ef14328, SUC_NE_lexicon_tags_2 },
    { 0x711f432a, SUC_NE_lexicon_tags_2 },
    { 0xdc0f432b, SUC_NE_lexicon_tags_2 },
    { 0x6ccd432c, SUC_NE_lexicon_tags_2 },
    { 0x7cdd432d, SUC_NE_lexicon_tags_2 },
    { 0x3965432e, SUC_NE_lexicon_tags_2 },
    { 0xf601432f, SUC_NE_lexicon_tags_32 },
    { 0x3a454330, SUC_NE_lexicon_tags_2 },
    { 0x10b432c, SUC_NE_lexicon_tags_2 },
    { 0x3d7d4332, SUC_NE_lexicon_tags_4 },
    { 0xb7df4333, SUC_NE_lexicon_tags_3 },
    { 0xa1d94333, SUC_NE_lexicon_tags_2 },
    { 0xa91b4335, SUC_NE_lexicon_tags_2 },
    { 0x819d4336, SUC_NE_lexicon_tags_2 },
    { 0x9f054333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc777433b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4743433d, SUC_NE_lexicon_tags_2 },
    { 0x3d9b433d, SUC_NE_lexicon_tags_1 },
    { 0x850b433f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfc94341, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c934349, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1765434c, SUC_NE_lexicon_tags_2 },
    { 0x97a9434c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd4350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4754354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e854358, SUC_NE_lexicon_tags_2 },
    { 0xec554359, SUC_NE_lexicon_tags_2 },
    { 0xad0b435a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf591435c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc69435f, SUC_NE_lexicon_tags_2 },
    { 0xd2994360, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x547d4363, SUC_NE_lexicon_tags_2 },
    { 0x8ab4364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cd436a, SUC_NE_lexicon_tags_2 },
    { 0x9ddf436a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a13436f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x115f4372, SUC_NE_lexicon_tags_2 },
    { 0xdbb14373, SUC_NE_lexicon_tags_2 },
    { 0x80974373, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4f437d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2654381, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b934383, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f34386, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75db4389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffd438c, SUC_NE_lexicon_tags_2 },
    { 0xfaab438d, SUC_NE_lexicon_tags_7 },
    { 0xe42d438e, SUC_NE_lexicon_tags_3 },
    { 0x1115438f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f14399, SUC_NE_lexicon_tags_2 },
    { 0x83f54399, SUC_NE_lexicon_tags_2 },
    { 0x76734399, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b39439f, SUC_NE_lexicon_tags_157 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f43a2, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0xb8b43a4, SUC_NE_lexicon_tags_2 },
    { 0xbeb43a5, SUC_NE_lexicon_tags_2 },
    { 0x9d343a6, SUC_NE_lexicon_tags_2 },
    { 0x994d43a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77543ac, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43743b0, SUC_NE_lexicon_tags_8 },
    { 0xa7b943b1, SUC_NE_lexicon_tags_2 },
    { 0xb08743b2, SUC_NE_lexicon_tags_1 },
    { 0x4b6143b2, SUC_NE_lexicon_tags_2 },
    { 0xe0b343b4, SUC_NE_lexicon_tags_2 },
    { 0xb24943b4, SUC_NE_lexicon_tags_2 },
    { 0x6a4143b2, SUC_NE_lexicon_tags_1 },
    { 0xff2f43b1, SUC_NE_lexicon_tags_2 },
    { 0x6fdd43b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd54d43ba, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x65b543bc, SUC_NE_lexicon_tags_2 },
    { 0x389d43bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830143c9, SUC_NE_lexicon_tags_2 },
    { 0xa5bb43ca, SUC_NE_lexicon_tags_2 },
    { 0x531d43cb, SUC_NE_lexicon_tags_2 },
    { 0xe71943cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86743cf, SUC_NE_lexicon_tags_2 },
    { 0x22ed43d0, SUC_NE_lexicon_tags_2 },
    { 0x6b3543d0, SUC_NE_lexicon_tags_7 },
    { 0xccbd43d2, SUC_NE_lexicon_tags_2 },
    { 0x70143d3, SUC_NE_lexicon_tags_2 },
    { 0xda3b43cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846d43d9, SUC_NE_lexicon_tags_7 },
    { 0xfc8343d9, SUC_NE_lexicon_tags_3 },
    { 0xf66343db, SUC_NE_lexicon_tags_2 },
    { 0x9f1543da, SUC_NE_lexicon_tags_40 },
    { 0xe6bb43dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7343e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59543e5, SUC_NE_lexicon_tags_2 },
    { 0x1a9543e6, SUC_NE_lexicon_tags_2 },
    { 0xeac343e6, SUC_NE_lexicon_tags_2 },
    { 0x3fa543e7, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f43f0, SUC_NE_lexicon_tags_2 },
    { 0xd49943f1, SUC_NE_lexicon_tags_2 },
    { 0x875143f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefad43f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d743fc, SUC_NE_lexicon_tags_8 },
    { 0xebdd43fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40b543ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeab4404, SUC_NE_lexicon_tags_2 },
    { 0xfa5b4404, SUC_NE_lexicon_tags_2 },
    { 0xe2ff4406, SUC_NE_lexicon_tags_2 },
    { 0xc1934407, SUC_NE_lexicon_tags_2 },
    { 0x899d4407, SUC_NE_lexicon_tags_2 },
    { 0xcffb4409, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2d440c, SUC_NE_lexicon_tags_2 },
    { 0x983440d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0d4412, SUC_NE_lexicon_tags_14 },
    { 0x3d594412, SUC_NE_lexicon_tags_7 },
    { 0xf0814413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x196d4417, SUC_NE_lexicon_tags_1 },
    { 0x54894418, SUC_NE_lexicon_tags_2 },
    { 0x4de74419, SUC_NE_lexicon_tags_2 },
    { 0x67474419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x972b441c, SUC_NE_lexicon_tags_2 },
    { 0x2c55441d, SUC_NE_lexicon_tags_2 },
    { 0xe5a9441d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c34421, SUC_NE_lexicon_tags_2 },
    { 0xaa034422, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2d4424, SUC_NE_lexicon_tags_2 },
    { 0x9374425, SUC_NE_lexicon_tags_2 },
    { 0x50674425, SUC_NE_lexicon_tags_2 },
    { 0xc2c94425, SUC_NE_lexicon_tags_2 },
    { 0xb6f54428, SUC_NE_lexicon_tags_2 },
    { 0xda894428, SUC_NE_lexicon_tags_2 },
    { 0xddf9442a, SUC_NE_lexicon_tags_2 },
    { 0x807d442b, SUC_NE_lexicon_tags_2 },
    { 0x9d2d442c, SUC_NE_lexicon_tags_2 },
    { 0x8e8b442b, SUC_NE_lexicon_tags_2 },
    { 0xc05d442d, SUC_NE_lexicon_tags_2 },
    { 0xd041442f, SUC_NE_lexicon_tags_2 },
    { 0x3f6d442c, SUC_NE_lexicon_tags_14 },
    { 0xf8d14431, SUC_NE_lexicon_tags_1 },
    { 0x54374432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84554437, SUC_NE_lexicon_tags_2 },
    { 0x5cd4438, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55df443a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0d443d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8721443f, SUC_NE_lexicon_tags_2 },
    { 0x729f4440, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eab4442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d054444, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca054446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19444a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x965f444c, SUC_NE_lexicon_tags_2 },
    { 0x5871444d, SUC_NE_lexicon_tags_2 },
    { 0x2d9d444c, SUC_NE_lexicon_tags_7 },
    { 0x64a5444f, SUC_NE_lexicon_tags_4 },
    { 0x1e054450, SUC_NE_lexicon_tags_2 },
    { 0x51694451, SUC_NE_lexicon_tags_2 },
    { 0x5f154452, SUC_NE_lexicon_tags_2 },
    { 0xfd9d4453, SUC_NE_lexicon_tags_7 },
    { 0xe7ff444f, SUC_NE_lexicon_tags_2 },
    { 0x9f214455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fe54457, SUC_NE_lexicon_tags_2 },
    { 0x6b554458, SUC_NE_lexicon_tags_2 },
    { 0xb7614459, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xa973445b, SUC_NE_lexicon_tags_2 },
    { 0x991b445c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46d445e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9446a, SUC_NE_lexicon_tags_2 },
    { 0x78fd446b, SUC_NE_lexicon_tags_2 },
    { 0xbe2f446c, SUC_NE_lexicon_tags_2 },
    { 0x25f5446d, SUC_NE_lexicon_tags_2 },
    { 0x2895446d, SUC_NE_lexicon_tags_2 },
    { 0x5e2d446b, SUC_NE_lexicon_tags_2 },
    { 0xd2d3446f, SUC_NE_lexicon_tags_2 },
    { 0x1b914471, SUC_NE_lexicon_tags_17 },
    { 0x18694472, SUC_NE_lexicon_tags_4 },
    { 0x81914473, SUC_NE_lexicon_tags_2 },
    { 0x769f446d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0d4477, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd2d4479, SUC_NE_lexicon_tags_34 },
    { 0xe917447a, SUC_NE_lexicon_tags_2 },
    { 0x25bf4479, SUC_NE_lexicon_tags_2 },
    { 0x8a05447c, SUC_NE_lexicon_tags_13 },
    { 0x5bb3447d, SUC_NE_lexicon_tags_2 },
    { 0xebbd4479, SUC_NE_lexicon_tags_2 },
    { 0xc529447f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d14482, SUC_NE_lexicon_tags_3 },
    { 0x783f4483, SUC_NE_lexicon_tags_2 },
    { 0x425b4484, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd85448b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0eb4490, SUC_NE_lexicon_tags_2 },
    { 0x96874491, SUC_NE_lexicon_tags_2 },
    { 0xebf74491, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82874495, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9814497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6294499, SUC_NE_lexicon_tags_2 },
    { 0x1451449a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb699449d, SUC_NE_lexicon_tags_2 },
    { 0x34e3449e, SUC_NE_lexicon_tags_2 },
    { 0x1faf449f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x97a344a1, SUC_NE_lexicon_tags_2 },
    { 0x69eb44a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea544a6, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03944aa, SUC_NE_lexicon_tags_2 },
    { 0x690d44ab, SUC_NE_lexicon_tags_2 },
    { 0xd07b44ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb344af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ef44b2, SUC_NE_lexicon_tags_2 },
    { 0xbdbb44b2, SUC_NE_lexicon_tags_2 },
    { 0x81dd44b4, SUC_NE_lexicon_tags_2 },
    { 0x485544b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x735b44b7, SUC_NE_lexicon_tags_7 },
    { 0x4c3944b7, SUC_NE_lexicon_tags_2 },
    { 0x6e2544b9, SUC_NE_lexicon_tags_2 },
    { 0x890144b9, SUC_NE_lexicon_tags_2 },
    { 0x1a1744bb, SUC_NE_lexicon_tags_2 },
    { 0xea9f44b7, SUC_NE_lexicon_tags_2 },
    { 0x31c144bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad144c9, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0744ce, SUC_NE_lexicon_tags_2 },
    { 0x1a0344ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6544d2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb10544d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc144dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cf44df, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xd25b44e1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb3144e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab4f44e5, SUC_NE_lexicon_tags_2 },
    { 0x50d744e6, SUC_NE_lexicon_tags_2 },
    { 0xd7cb44e7, SUC_NE_lexicon_tags_2 },
    { 0xb4e744e7, SUC_NE_lexicon_tags_4 },
    { 0x580944e9, SUC_NE_lexicon_tags_3 },
    { 0x60e144ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc72944ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ab44ef, SUC_NE_lexicon_tags_2 },
    { 0x82bf44f0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x655544f2, SUC_NE_lexicon_tags_2 },
    { 0x9cbd44f3, SUC_NE_lexicon_tags_2 },
    { 0x68db44f4, SUC_NE_lexicon_tags_2 },
    { 0xde6744f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de744f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ff544fa, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0xc42744fc, SUC_NE_lexicon_tags_2 },
    { 0xebf544fd, SUC_NE_lexicon_tags_2 },
    { 0xbe8f44fe, SUC_NE_lexicon_tags_2 },
    { 0x3b4d44fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a34503, SUC_NE_lexicon_tags_19 },
    { 0x80014504, SUC_NE_lexicon_tags_2 },
    { 0xcd6f4505, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa001450b, SUC_NE_lexicon_tags_2 },
    { 0x84dd450b, SUC_NE_lexicon_tags_2 },
    { 0x3c7f450b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a154510, SUC_NE_lexicon_tags_2 },
    { 0xb8474511, SUC_NE_lexicon_tags_2 },
    { 0x8c634512, SUC_NE_lexicon_tags_2 },
    { 0x3fdb4513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46114515, SUC_NE_lexicon_tags_2 },
    { 0xf2714515, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121f451b, SUC_NE_lexicon_tags_8 },
    { 0xc067451c, SUC_NE_lexicon_tags_2 },
    { 0xfd81451c, SUC_NE_lexicon_tags_2 },
    { 0xad33451c, SUC_NE_lexicon_tags_19 },
    { 0xb6eb451d, SUC_NE_lexicon_tags_5 },
    { 0xfd41451f, SUC_NE_lexicon_tags_2 },
    { 0x5c13451b, SUC_NE_lexicon_tags_2 },
    { 0x27a74522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab314525, SUC_NE_lexicon_tags_2 },
    { 0x8daf4525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb1452a, SUC_NE_lexicon_tags_372 },
    { 0xeb39452b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115b452f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x78774531, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab4535, SUC_NE_lexicon_tags_2 },
    { 0xf74535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2729453a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3f453e, SUC_NE_lexicon_tags_2 },
    { 0xb37f453e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c74542, SUC_NE_lexicon_tags_2 },
    { 0xeeaf4542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b94545, SUC_NE_lexicon_tags_7 },
    { 0x24334546, SUC_NE_lexicon_tags_2 },
    { 0x45f34547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cd454c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63b5454e, SUC_NE_lexicon_tags_2 },
    { 0x365d454f, SUC_NE_lexicon_tags_7 },
    { 0xc5874550, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f54553, SUC_NE_lexicon_tags_32 },
    { 0xb55f4554, SUC_NE_lexicon_tags_2 },
    { 0xdd774553, SUC_NE_lexicon_tags_2 },
    { 0x150b4554, SUC_NE_lexicon_tags_2 },
    { 0x5db94557, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b3455b, SUC_NE_lexicon_tags_2 },
    { 0x442f455b, SUC_NE_lexicon_tags_2 },
    { 0x8065455b, SUC_NE_lexicon_tags_2 },
    { 0x650f455e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39934560, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8f4564, SUC_NE_lexicon_tags_2 },
    { 0xb31d4565, SUC_NE_lexicon_tags_3 },
    { 0xea8f4566, SUC_NE_lexicon_tags_2 },
    { 0x5cd34564, SUC_NE_lexicon_tags_2 },
    { 0x16ed4568, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51456d, SUC_NE_lexicon_tags_2 },
    { 0xafd1456e, SUC_NE_lexicon_tags_2 },
    { 0xf311456e, SUC_NE_lexicon_tags_2 },
    { 0x68834570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d74575, SUC_NE_lexicon_tags_2 },
    { 0xda234576, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe57457b, SUC_NE_lexicon_tags_2 },
    { 0xa163457c, SUC_NE_lexicon_tags_2 },
    { 0xbbd9457d, SUC_NE_lexicon_tags_2 },
    { 0x3251457e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d34582, SUC_NE_lexicon_tags_2 },
    { 0x6d934583, SUC_NE_lexicon_tags_2 },
    { 0xc6114584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f74588, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f458b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5bb458d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb4590, SUC_NE_lexicon_tags_2 },
    { 0xd6894591, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b474597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b3459b, SUC_NE_lexicon_tags_7 },
    { 0x7b27459c, SUC_NE_lexicon_tags_2 },
    { 0x34e7459d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd3f459f, SUC_NE_lexicon_tags_2 },
    { 0x1d6745a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x678745a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b545a5, SUC_NE_lexicon_tags_2 },
    { 0xa42d45a5, SUC_NE_lexicon_tags_2 },
    { 0x7e5545a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e545aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf45ae, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4f145b0, SUC_NE_lexicon_tags_2 },
    { 0x9ef145b1, SUC_NE_lexicon_tags_2 },
    { 0xf6c345b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x630945b4, SUC_NE_lexicon_tags_2 },
    { 0xe08545b5, SUC_NE_lexicon_tags_2 },
    { 0xb14b45b4, SUC_NE_lexicon_tags_2 },
    { 0xa74d45b4, SUC_NE_lexicon_tags_2 },
    { 0x68d45b8, SUC_NE_lexicon_tags_2 },
    { 0xd5cf45b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd45bc, SUC_NE_lexicon_tags_2 },
    { 0xfe0d45bc, SUC_NE_lexicon_tags_2 },
    { 0xf00745bc, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224b45c2, SUC_NE_lexicon_tags_2 },
    { 0xe39f45c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21eb45c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807d45cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab45ce, SUC_NE_lexicon_tags_2 },
    { 0xe10d45ce, SUC_NE_lexicon_tags_2 },
    { 0x12f945d0, SUC_NE_lexicon_tags_2 },
    { 0x7c4745d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c945d3, SUC_NE_lexicon_tags_2 },
    { 0xf77145d4, SUC_NE_lexicon_tags_2 },
    { 0xdebf45d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803145dc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5b45e0, SUC_NE_lexicon_tags_2 },
    { 0x560345e1, SUC_NE_lexicon_tags_2 },
    { 0x859f45e0, SUC_NE_lexicon_tags_2 },
    { 0xc53345e2, SUC_NE_lexicon_tags_4 },
    { 0x9ee145e3, SUC_NE_lexicon_tags_2 },
    { 0x5b8945e3, SUC_NE_lexicon_tags_2 },
    { 0xc17145e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b545e8, SUC_NE_lexicon_tags_2 },
    { 0x52ab45e9, SUC_NE_lexicon_tags_2 },
    { 0x756945e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x236145ec, SUC_NE_lexicon_tags_2 },
    { 0x79e545ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50a745ef, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x663745f1, SUC_NE_lexicon_tags_2 },
    { 0xa4bd45f1, SUC_NE_lexicon_tags_2 },
    { 0xe43545f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ec545f5, SUC_NE_lexicon_tags_2 },
    { 0xe56345f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e6945f8, SUC_NE_lexicon_tags_2 },
    { 0x6b9945f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdf45fc, SUC_NE_lexicon_tags_2 },
    { 0xc0ed45fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37134601, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d914608, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6313460a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1365460e, SUC_NE_lexicon_tags_2 },
    { 0xc1d9460f, SUC_NE_lexicon_tags_2 },
    { 0xf8f74610, SUC_NE_lexicon_tags_2 },
    { 0xc634610, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f4615, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb394617, SUC_NE_lexicon_tags_2 },
    { 0x38db4617, SUC_NE_lexicon_tags_7 },
    { 0xef294619, SUC_NE_lexicon_tags_2 },
    { 0x29f1461a, SUC_NE_lexicon_tags_4 },
    { 0x9e03461b, SUC_NE_lexicon_tags_4 },
    { 0x2967461a, SUC_NE_lexicon_tags_7 },
    { 0x8105461d, SUC_NE_lexicon_tags_2 },
    { 0x494b461c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92014622, SUC_NE_lexicon_tags_34 },
    { 0x7a1d4623, SUC_NE_lexicon_tags_2 },
    { 0xd24f4624, SUC_NE_lexicon_tags_2 },
    { 0x7d7b4624, SUC_NE_lexicon_tags_2 },
    { 0x219f4626, SUC_NE_lexicon_tags_2 },
    { 0x73034626, SUC_NE_lexicon_tags_2 },
    { 0x194d4624, SUC_NE_lexicon_tags_2 },
    { 0xae9d4629, SUC_NE_lexicon_tags_2 },
    { 0xd467462a, SUC_NE_lexicon_tags_2 },
    { 0x195b462b, SUC_NE_lexicon_tags_2 },
    { 0xa8e3462c, SUC_NE_lexicon_tags_2 },
    { 0xf8eb462d, SUC_NE_lexicon_tags_2 },
    { 0x6bff462e, SUC_NE_lexicon_tags_2 },
    { 0x46d1462f, SUC_NE_lexicon_tags_2 },
    { 0xb3d5462e, SUC_NE_lexicon_tags_2 },
    { 0x8871462b, SUC_NE_lexicon_tags_2 },
    { 0x6335462b, SUC_NE_lexicon_tags_3 },
    { 0xd214633, SUC_NE_lexicon_tags_2 },
    { 0x50454628, SUC_NE_lexicon_tags_2 },
    { 0x21ad4629, SUC_NE_lexicon_tags_2 },
    { 0x37b54636, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f463a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35db463d, SUC_NE_lexicon_tags_2 },
    { 0x9729463d, SUC_NE_lexicon_tags_2 },
    { 0xa923463d, SUC_NE_lexicon_tags_2 },
    { 0x639d463d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x236f4643, SUC_NE_lexicon_tags_0 },
    { 0x4dff4643, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb849464b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4652, SUC_NE_lexicon_tags_2 },
    { 0x58cf4652, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c11465c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d94660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cf4664, SUC_NE_lexicon_tags_3 },
    { 0x12734665, SUC_NE_lexicon_tags_2 },
    { 0x18a54665, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc14669, SUC_NE_lexicon_tags_3 },
    { 0x400b4669, SUC_NE_lexicon_tags_14 },
    { 0x840b466b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9dd466d, SUC_NE_lexicon_tags_345 },
    { 0x78ed466d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc97f4670, SUC_NE_lexicon_tags_2 },
    { 0x2a1d4671, SUC_NE_lexicon_tags_2 },
    { 0x41ed4672, SUC_NE_lexicon_tags_2 },
    { 0x42ef4672, SUC_NE_lexicon_tags_2 },
    { 0xcd034671, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe174677, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d467a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c79467d, SUC_NE_lexicon_tags_2 },
    { 0xf99d467e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfad94680, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b994683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2b4686, SUC_NE_lexicon_tags_19 },
    { 0x48054686, SUC_NE_lexicon_tags_2 },
    { 0x3de94687, SUC_NE_lexicon_tags_413 },
    { 0x984d4689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac5468c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xfe55468e, SUC_NE_lexicon_tags_2 },
    { 0xc535468f, SUC_NE_lexicon_tags_2 },
    { 0x7435468e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75614692, SUC_NE_lexicon_tags_2 },
    { 0xd3454693, SUC_NE_lexicon_tags_2 },
    { 0xc2994694, SUC_NE_lexicon_tags_2 },
    { 0xc1014693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefaf4697, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bab469b, SUC_NE_lexicon_tags_2 },
    { 0xab13469b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5059469e, SUC_NE_lexicon_tags_2 },
    { 0xf9cd469e, SUC_NE_lexicon_tags_2 },
    { 0xc3fd46a0, SUC_NE_lexicon_tags_0 },
    { 0x40dd46a1, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x83c346a3, SUC_NE_lexicon_tags_2 },
    { 0x2a9346a3, SUC_NE_lexicon_tags_176 },
    { 0x2bb146a3, SUC_NE_lexicon_tags_2 },
    { 0xca6146a5, SUC_NE_lexicon_tags_2 },
    { 0x3d2b46a6, SUC_NE_lexicon_tags_2 },
    { 0x420346a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f46ab, SUC_NE_lexicon_tags_2 },
    { 0xecbd46ac, SUC_NE_lexicon_tags_2 },
    { 0xbba146ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6c346af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3746b4, SUC_NE_lexicon_tags_2 },
    { 0xfe7346b4, SUC_NE_lexicon_tags_2 },
    { 0x99fb46b6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109746ba, SUC_NE_lexicon_tags_2 },
    { 0x67e746bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c946bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41b46c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577f46c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6f46ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x183546cc, SUC_NE_lexicon_tags_2 },
    { 0x802946cc, SUC_NE_lexicon_tags_2 },
    { 0xeb6f46ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19546d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb3f46d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e146d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4546dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a9b46de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2346e5, SUC_NE_lexicon_tags_2 },
    { 0xe00b46e6, SUC_NE_lexicon_tags_2 },
    { 0xe7b46e7, SUC_NE_lexicon_tags_2 },
    { 0x33af46e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61146ed, SUC_NE_lexicon_tags_2 },
    { 0x62cf46ed, SUC_NE_lexicon_tags_2 },
    { 0xd19b46ef, SUC_NE_lexicon_tags_2 },
    { 0x173746f0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7b46f3, SUC_NE_lexicon_tags_2 },
    { 0xe63f46f3, SUC_NE_lexicon_tags_2 },
    { 0x79f346f3, SUC_NE_lexicon_tags_2 },
    { 0x4d2d46f6, SUC_NE_lexicon_tags_2 },
    { 0x53e146f7, SUC_NE_lexicon_tags_2 },
    { 0x7a2946f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9e146fa, SUC_NE_lexicon_tags_4 },
    { 0x8aff46fa, SUC_NE_lexicon_tags_2 },
    { 0x566346fc, SUC_NE_lexicon_tags_2 },
    { 0x3c4b46fb, SUC_NE_lexicon_tags_7 },
    { 0xe99b46fe, SUC_NE_lexicon_tags_2 },
    { 0x700146ff, SUC_NE_lexicon_tags_2 },
    { 0x64734700, SUC_NE_lexicon_tags_7 },
    { 0xa8554701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24434703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53414705, SUC_NE_lexicon_tags_2 },
    { 0x64034705, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd014709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c07470b, SUC_NE_lexicon_tags_2 },
    { 0xbc07470c, SUC_NE_lexicon_tags_2 },
    { 0xce2b470b, SUC_NE_lexicon_tags_2 },
    { 0x96ef470c, SUC_NE_lexicon_tags_2 },
    { 0xd219470d, SUC_NE_lexicon_tags_4 },
    { 0x62db470c, SUC_NE_lexicon_tags_2 },
    { 0x7a494711, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11d4714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5114718, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17d3471a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d74722, SUC_NE_lexicon_tags_2 },
    { 0x60294723, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f34727, SUC_NE_lexicon_tags_2 },
    { 0x61834728, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bb472b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36ed472d, SUC_NE_lexicon_tags_2 },
    { 0x192f472d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc51d4730, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3634732, SUC_NE_lexicon_tags_2 },
    { 0x9bf74733, SUC_NE_lexicon_tags_17 },
    { 0x5e794734, SUC_NE_lexicon_tags_2 },
    { 0x72d54735, SUC_NE_lexicon_tags_2 },
    { 0xc0cf4736, SUC_NE_lexicon_tags_2 },
    { 0x342f4736, SUC_NE_lexicon_tags_3 },
    { 0x41854733, SUC_NE_lexicon_tags_17 },
    { 0x63bd4737, SUC_NE_lexicon_tags_2 },
    { 0x849d473a, SUC_NE_lexicon_tags_2 },
    { 0xc9ab473b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9473e, SUC_NE_lexicon_tags_37 },
    { 0, NULL },
    { 0x81494740, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48db4742, SUC_NE_lexicon_tags_2 },
    { 0xee254743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2474747, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x333d4749, SUC_NE_lexicon_tags_2 },
    { 0xd8e5474a, SUC_NE_lexicon_tags_1 },
    { 0x6ba5474b, SUC_NE_lexicon_tags_2 },
    { 0xa5f7474c, SUC_NE_lexicon_tags_11 },
    { 0xd64f474c, SUC_NE_lexicon_tags_3 },
    { 0x80b9474e, SUC_NE_lexicon_tags_51 },
    { 0x48df474f, SUC_NE_lexicon_tags_2 },
    { 0x63cd4750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc74753, SUC_NE_lexicon_tags_7 },
    { 0xbf734754, SUC_NE_lexicon_tags_2 },
    { 0x63414755, SUC_NE_lexicon_tags_3 },
    { 0x56414753, SUC_NE_lexicon_tags_2 },
    { 0xe7554757, SUC_NE_lexicon_tags_2 },
    { 0xdfa94753, SUC_NE_lexicon_tags_2 },
    { 0xa9914759, SUC_NE_lexicon_tags_2 },
    { 0x1715475a, SUC_NE_lexicon_tags_2 },
    { 0x9c914759, SUC_NE_lexicon_tags_2 },
    { 0x9453475b, SUC_NE_lexicon_tags_2 },
    { 0x7374755, SUC_NE_lexicon_tags_2 },
    { 0xa025475e, SUC_NE_lexicon_tags_2 },
    { 0x8dc1475e, SUC_NE_lexicon_tags_2 },
    { 0x705f475e, SUC_NE_lexicon_tags_3 },
    { 0x6efd475f, SUC_NE_lexicon_tags_1 },
    { 0xcf7b4760, SUC_NE_lexicon_tags_2 },
    { 0xf1234754, SUC_NE_lexicon_tags_2 },
    { 0x80914756, SUC_NE_lexicon_tags_2 },
    { 0x1e434764, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4514768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58894771, SUC_NE_lexicon_tags_2 },
    { 0xc70b4771, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2714777, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x8154779, SUC_NE_lexicon_tags_2 },
    { 0xa983477a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bf477e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1854784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bd4788, SUC_NE_lexicon_tags_2 },
    { 0x1e214789, SUC_NE_lexicon_tags_2 },
    { 0x82b478a, SUC_NE_lexicon_tags_2 },
    { 0xbe01478a, SUC_NE_lexicon_tags_1 },
    { 0x495478c, SUC_NE_lexicon_tags_5 },
    { 0xf2c7478a, SUC_NE_lexicon_tags_2 },
    { 0xede7478e, SUC_NE_lexicon_tags_2 },
    { 0x297d4788, SUC_NE_lexicon_tags_83 },
    { 0xc71b4790, SUC_NE_lexicon_tags_2 },
    { 0x87794790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb074793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb674795, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc914798, SUC_NE_lexicon_tags_2 },
    { 0x90034798, SUC_NE_lexicon_tags_2 },
    { 0xe1ef4798, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c99479d, SUC_NE_lexicon_tags_2 },
    { 0xcc71479d, SUC_NE_lexicon_tags_2 },
    { 0xb597479f, SUC_NE_lexicon_tags_2 },
    { 0xd86b479f, SUC_NE_lexicon_tags_4 },
    { 0x6b03479d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03147a5, SUC_NE_lexicon_tags_2 },
    { 0x105547a6, SUC_NE_lexicon_tags_2 },
    { 0xef5547a7, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x10947aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cf47ad, SUC_NE_lexicon_tags_2 },
    { 0x79fd47ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433547b3, SUC_NE_lexicon_tags_2 },
    { 0x5f2b47b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60547bb, SUC_NE_lexicon_tags_2 },
    { 0x6a6b47bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x702747be, SUC_NE_lexicon_tags_34 },
    { 0x674d47bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47fb47c1, SUC_NE_lexicon_tags_388 },
    { 0x8e0f47c2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26d47c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x864147c8, SUC_NE_lexicon_tags_2 },
    { 0x746147c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe46347cb, SUC_NE_lexicon_tags_2 },
    { 0xfbd947cc, SUC_NE_lexicon_tags_17 },
    { 0x644747cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f147d0, SUC_NE_lexicon_tags_2 },
    { 0xc95747d0, SUC_NE_lexicon_tags_8 },
    { 0x8be347d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43d347d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49a747d6, SUC_NE_lexicon_tags_2 },
    { 0x76c147d7, SUC_NE_lexicon_tags_2 },
    { 0xdcd947d8, SUC_NE_lexicon_tags_2 },
    { 0x67b147d9, SUC_NE_lexicon_tags_2 },
    { 0x652147da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf03747dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387147e3, SUC_NE_lexicon_tags_2 },
    { 0xa5ff47e3, SUC_NE_lexicon_tags_2 },
    { 0x5b2d47e5, SUC_NE_lexicon_tags_2 },
    { 0xcaf347e6, SUC_NE_lexicon_tags_2 },
    { 0xcc7f47e5, SUC_NE_lexicon_tags_1 },
    { 0x3d0347e8, SUC_NE_lexicon_tags_3 },
    { 0x614d47e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe77947eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d547ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0f47f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fef47f6, SUC_NE_lexicon_tags_115 },
    { 0xa70147f7, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55547fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb5f47ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3974802, SUC_NE_lexicon_tags_2 },
    { 0xcfed4803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed874807, SUC_NE_lexicon_tags_2 },
    { 0x90154808, SUC_NE_lexicon_tags_2 },
    { 0x6e3d4809, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x297d480b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x290d480e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7294810, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc14814, SUC_NE_lexicon_tags_7 },
    { 0x9f534814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e54823, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb4827, SUC_NE_lexicon_tags_2 },
    { 0x242f4828, SUC_NE_lexicon_tags_2 },
    { 0x17094829, SUC_NE_lexicon_tags_2 },
    { 0x594d482a, SUC_NE_lexicon_tags_2 },
    { 0xa51d482b, SUC_NE_lexicon_tags_91 },
    { 0x1fd1482b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96db482f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c3d4831, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eb4839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a47483b, SUC_NE_lexicon_tags_2 },
    { 0x69bb483c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a6b483e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab4841, SUC_NE_lexicon_tags_2 },
    { 0xc3e34842, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65e34844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d94848, SUC_NE_lexicon_tags_2 },
    { 0x85294848, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3ff484b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ff484e, SUC_NE_lexicon_tags_2 },
    { 0xb6df484e, SUC_NE_lexicon_tags_2 },
    { 0x8ad94850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a3b4852, SUC_NE_lexicon_tags_2 },
    { 0x70754853, SUC_NE_lexicon_tags_4 },
    { 0xb48f4853, SUC_NE_lexicon_tags_2 },
    { 0x7b6d4854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d5b4857, SUC_NE_lexicon_tags_2 },
    { 0x15ed4857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97485c, SUC_NE_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c194861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa34866, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb4869, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b486c, SUC_NE_lexicon_tags_2 },
    { 0x1117486d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f33486f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcadd4871, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24b74873, SUC_NE_lexicon_tags_3 },
    { 0x54114873, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfb94876, SUC_NE_lexicon_tags_18 },
    { 0x300b4877, SUC_NE_lexicon_tags_2 },
    { 0x9d914877, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a7487a, SUC_NE_lexicon_tags_2 },
    { 0x6a23487a, SUC_NE_lexicon_tags_7 },
    { 0xdcb5487c, SUC_NE_lexicon_tags_2 },
    { 0x17f1487d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7794882, SUC_NE_lexicon_tags_281 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0834885, SUC_NE_lexicon_tags_2 },
    { 0xea9b4886, SUC_NE_lexicon_tags_2 },
    { 0x74454886, SUC_NE_lexicon_tags_2 },
    { 0x8bf94888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92bd488a, SUC_NE_lexicon_tags_1 },
    { 0x90c7488b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11734892, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39674897, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba3489c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326d48a0, SUC_NE_lexicon_tags_2 },
    { 0x94a548a1, SUC_NE_lexicon_tags_2 },
    { 0x5f6348a2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615548a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe7748a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e948ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x357948b0, SUC_NE_lexicon_tags_7 },
    { 0x734748b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc72148b3, SUC_NE_lexicon_tags_2 },
    { 0x5bdf48b3, SUC_NE_lexicon_tags_2 },
    { 0xd08d48b5, SUC_NE_lexicon_tags_4 },
    { 0x59f348b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990b48bc, SUC_NE_lexicon_tags_2 },
    { 0x11a748bd, SUC_NE_lexicon_tags_32 },
    { 0xafe148be, SUC_NE_lexicon_tags_2 },
    { 0x11db48be, SUC_NE_lexicon_tags_7 },
    { 0x8b9348bd, SUC_NE_lexicon_tags_2 },
    { 0xc51f48c1, SUC_NE_lexicon_tags_36 },
    { 0x4fb948c2, SUC_NE_lexicon_tags_2 },
    { 0xc8d548c3, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0xb48948c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb9348c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ab48ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af748d2, SUC_NE_lexicon_tags_2 },
    { 0x60c748d2, SUC_NE_lexicon_tags_2 },
    { 0xe4148d4, SUC_NE_lexicon_tags_2 },
    { 0x4cd748d5, SUC_NE_lexicon_tags_2 },
    { 0xc4e548d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86d48da, SUC_NE_lexicon_tags_2 },
    { 0xdb0b48db, SUC_NE_lexicon_tags_2 },
    { 0x908748dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x347f48df, SUC_NE_lexicon_tags_2 },
    { 0x1db948e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33f48e2, SUC_NE_lexicon_tags_2 },
    { 0x45b48e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d148e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x509f48e9, SUC_NE_lexicon_tags_2 },
    { 0x10eb48ea, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0348f1, SUC_NE_lexicon_tags_2 },
    { 0x8add48f1, SUC_NE_lexicon_tags_7 },
    { 0x76e748f3, SUC_NE_lexicon_tags_1 },
    { 0xb35548f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x598d48f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd348fa, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0x18c948fc, SUC_NE_lexicon_tags_2 },
    { 0x23d348fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b1f48ff, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692d4903, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd4906, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e9f4908, SUC_NE_lexicon_tags_2 },
    { 0x1e434908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e53490b, SUC_NE_lexicon_tags_2 },
    { 0x31e1490c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea71490e, SUC_NE_lexicon_tags_2 },
    { 0x1e9b490f, SUC_NE_lexicon_tags_2 },
    { 0x47534910, SUC_NE_lexicon_tags_2 },
    { 0x7a6f490e, SUC_NE_lexicon_tags_2 },
    { 0x73314912, SUC_NE_lexicon_tags_312 },
    { 0x6a3f4913, SUC_NE_lexicon_tags_2 },
    { 0xd60b4914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x912f4916, SUC_NE_lexicon_tags_10 },
    { 0xa44d4916, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d3491a, SUC_NE_lexicon_tags_2 },
    { 0xe65d491b, SUC_NE_lexicon_tags_2 },
    { 0x87bf491c, SUC_NE_lexicon_tags_2 },
    { 0x4b03491b, SUC_NE_lexicon_tags_2 },
    { 0x4503491e, SUC_NE_lexicon_tags_2 },
    { 0x15d1491e, SUC_NE_lexicon_tags_2 },
    { 0x32394920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba534924, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd67492c, SUC_NE_lexicon_tags_13 },
    { 0x7b83492d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a94933, SUC_NE_lexicon_tags_2 },
    { 0x572f4933, SUC_NE_lexicon_tags_2 },
    { 0x512b4935, SUC_NE_lexicon_tags_145 },
    { 0xd4714936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbe34938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17b493b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa425493e, SUC_NE_lexicon_tags_2 },
    { 0xc4cd493e, SUC_NE_lexicon_tags_19 },
    { 0x74b94940, SUC_NE_lexicon_tags_2 },
    { 0x4351493e, SUC_NE_lexicon_tags_5 },
    { 0x4607493f, SUC_NE_lexicon_tags_2 },
    { 0xdcd54941, SUC_NE_lexicon_tags_4 },
    { 0x6d534941, SUC_NE_lexicon_tags_2 },
    { 0x33e34944, SUC_NE_lexicon_tags_201 },
    { 0xdb954944, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532f494a, SUC_NE_lexicon_tags_2 },
    { 0x784f494b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5041494d, SUC_NE_lexicon_tags_2 },
    { 0xf379494d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x41db4951, SUC_NE_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b13495c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab34961, SUC_NE_lexicon_tags_2 },
    { 0xfca14962, SUC_NE_lexicon_tags_2 },
    { 0x565f4963, SUC_NE_lexicon_tags_2 },
    { 0x80054964, SUC_NE_lexicon_tags_4 },
    { 0xdf334964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e54969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c37496d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4a7496f, SUC_NE_lexicon_tags_2 },
    { 0xdcb94970, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17674972, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfef4976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d497a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f1497d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6967497f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a74983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869b4987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde9498a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26cd498d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a274991, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9554999, SUC_NE_lexicon_tags_2 },
    { 0xf851499a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b23499d, SUC_NE_lexicon_tags_2 },
    { 0x588b499d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcaf149a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede349a6, SUC_NE_lexicon_tags_2 },
    { 0x4b5d49a6, SUC_NE_lexicon_tags_2 },
    { 0x403549a7, SUC_NE_lexicon_tags_2 },
    { 0x155b49a9, SUC_NE_lexicon_tags_2 },
    { 0xecd149aa, SUC_NE_lexicon_tags_2 },
    { 0xefcd49ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5ab49ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52949b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d149b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c949b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363b49bf, SUC_NE_lexicon_tags_2 },
    { 0x542549bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efd49c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccb49c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f149ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a4349d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x697f49d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec949db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96b49de, SUC_NE_lexicon_tags_2 },
    { 0xb24549de, SUC_NE_lexicon_tags_2 },
    { 0xc28f49de, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2f49e7, SUC_NE_lexicon_tags_2 },
    { 0xcae549e7, SUC_NE_lexicon_tags_2 },
    { 0xfcf49e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ef49ec, SUC_NE_lexicon_tags_2 },
    { 0x610549ed, SUC_NE_lexicon_tags_2 },
    { 0x1f8d49ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcd49f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c149f5, SUC_NE_lexicon_tags_2 },
    { 0x4c1d49f6, SUC_NE_lexicon_tags_2 },
    { 0x4a7d49f6, SUC_NE_lexicon_tags_2 },
    { 0xee0f49f8, SUC_NE_lexicon_tags_2 },
    { 0x197949f7, SUC_NE_lexicon_tags_2 },
    { 0xab3749fa, SUC_NE_lexicon_tags_2 },
    { 0x64ef49fb, SUC_NE_lexicon_tags_2 },
    { 0x20c349f9, SUC_NE_lexicon_tags_2 },
    { 0x573d49f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1f4a00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83934a09, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0xea414a0b, SUC_NE_lexicon_tags_2 },
    { 0x5194a0b, SUC_NE_lexicon_tags_2 },
    { 0x5a934a0c, SUC_NE_lexicon_tags_2 },
    { 0xd1254a0b, SUC_NE_lexicon_tags_2 },
    { 0x4bcb4a0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c354a13, SUC_NE_lexicon_tags_2 },
    { 0x288d4a13, SUC_NE_lexicon_tags_2 },
    { 0xdf574a14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f014a17, SUC_NE_lexicon_tags_2 },
    { 0x7f934a17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e894a1d, SUC_NE_lexicon_tags_2 },
    { 0x30254a1e, SUC_NE_lexicon_tags_2 },
    { 0x70db4a1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d14a28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21814a2f, SUC_NE_lexicon_tags_2 },
    { 0x2dcf4a30, SUC_NE_lexicon_tags_2 },
    { 0x836b4a31, SUC_NE_lexicon_tags_2 },
    { 0xc69d4a32, SUC_NE_lexicon_tags_2 },
    { 0xb5114a2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ead4a35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39f4a39, SUC_NE_lexicon_tags_2 },
    { 0x22574a3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9354a3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0f4a3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b4a42, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9074a45, SUC_NE_lexicon_tags_2 },
    { 0xe0c74a45, SUC_NE_lexicon_tags_2 },
    { 0x292b4a47, SUC_NE_lexicon_tags_2 },
    { 0x92854a48, SUC_NE_lexicon_tags_2 },
    { 0x599d4a48, SUC_NE_lexicon_tags_1 },
    { 0x12274a4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce014a4e, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xf92b4a50, SUC_NE_lexicon_tags_2 },
    { 0x17554a50, SUC_NE_lexicon_tags_2 },
    { 0xa254a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88d4a57, SUC_NE_lexicon_tags_2 },
    { 0x65694a58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe07b4a5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff0b4a5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb454a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafc94a62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2554a66, SUC_NE_lexicon_tags_2 },
    { 0x3cb4a66, SUC_NE_lexicon_tags_7 },
    { 0x75f94a68, SUC_NE_lexicon_tags_2 },
    { 0x56e14a68, SUC_NE_lexicon_tags_2 },
    { 0xaeeb4a6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14df4a73, SUC_NE_lexicon_tags_2 },
    { 0x83b74a74, SUC_NE_lexicon_tags_2 },
    { 0xdd274a75, SUC_NE_lexicon_tags_2 },
    { 0x96254a76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x601d4a79, SUC_NE_lexicon_tags_2 },
    { 0x94b14a7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce7f4a7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4b4a7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde614a82, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4a88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f314a8e, SUC_NE_lexicon_tags_2 },
    { 0x9acf4a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3cf4a91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d34a94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fd4a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f4a9d, SUC_NE_lexicon_tags_89 },
    { 0, NULL },
    { 0x1b794a9f, SUC_NE_lexicon_tags_8 },
    { 0xc7254aa0, SUC_NE_lexicon_tags_2 },
    { 0xb5394a9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90374aa3, SUC_NE_lexicon_tags_5 },
    { 0xa3354aa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1db4aa6, SUC_NE_lexicon_tags_2 },
    { 0xcca94aa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e854aa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5814aab, SUC_NE_lexicon_tags_2 },
    { 0x185f4aac, SUC_NE_lexicon_tags_2 },
    { 0x20634aab, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x30ef4aaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ef94ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43b14ab3, SUC_NE_lexicon_tags_8 },
    { 0x41df4ab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd12f4ab6, SUC_NE_lexicon_tags_2 },
    { 0xe8554ab7, SUC_NE_lexicon_tags_7 },
    { 0xf6f94ab7, SUC_NE_lexicon_tags_2 },
    { 0x79674ab9, SUC_NE_lexicon_tags_18 },
    { 0x12634ab9, SUC_NE_lexicon_tags_2 },
    { 0xed2b4ab8, SUC_NE_lexicon_tags_10 },
    { 0x6d714aba, SUC_NE_lexicon_tags_2 },
    { 0x397b4abd, SUC_NE_lexicon_tags_2 },
    { 0x164d4abb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2d4ac1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4974ac8, SUC_NE_lexicon_tags_2 },
    { 0x6794ac9, SUC_NE_lexicon_tags_2 },
    { 0x11cb4aca, SUC_NE_lexicon_tags_2 },
    { 0xb9db4acb, SUC_NE_lexicon_tags_2 },
    { 0xa8ed4acc, SUC_NE_lexicon_tags_17 },
    { 0xaacb4acd, SUC_NE_lexicon_tags_2 },
    { 0xb7f34acd, SUC_NE_lexicon_tags_2 },
    { 0x575d4acf, SUC_NE_lexicon_tags_2 },
    { 0x183f4acf, SUC_NE_lexicon_tags_18 },
    { 0x674d4acf, SUC_NE_lexicon_tags_1 },
    { 0x56af4ad0, SUC_NE_lexicon_tags_2 },
    { 0xc55b4acc, SUC_NE_lexicon_tags_2 },
    { 0xf8a14ad4, SUC_NE_lexicon_tags_2 },
    { 0xfd474ad1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b94ad8, SUC_NE_lexicon_tags_2 },
    { 0x199d4ad8, SUC_NE_lexicon_tags_2 },
    { 0x12d54ada, SUC_NE_lexicon_tags_3 },
    { 0x2d3f4ada, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x369b4add, SUC_NE_lexicon_tags_2 },
    { 0xeacf4ade, SUC_NE_lexicon_tags_2 },
    { 0x3f334adf, SUC_NE_lexicon_tags_2 },
    { 0x78614adf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69db4ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30c14ae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c74ae8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6854aed, SUC_NE_lexicon_tags_2 },
    { 0x936d4aed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ff4af1, SUC_NE_lexicon_tags_2 },
    { 0xf6c94af2, SUC_NE_lexicon_tags_2 },
    { 0x4d7f4af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a14af6, SUC_NE_lexicon_tags_2 },
    { 0x886f4af6, SUC_NE_lexicon_tags_2 },
    { 0xc6c94af7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4f4afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d154b02, SUC_NE_lexicon_tags_3 },
    { 0x791b4b03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c154b05, SUC_NE_lexicon_tags_2 },
    { 0x2a834b06, SUC_NE_lexicon_tags_2 },
    { 0xa32f4b07, SUC_NE_lexicon_tags_5 },
    { 0x15994b07, SUC_NE_lexicon_tags_2 },
    { 0x82654b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56dd4b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c014b10, SUC_NE_lexicon_tags_2 },
    { 0x2c74b11, SUC_NE_lexicon_tags_3 },
    { 0x70fb4b12, SUC_NE_lexicon_tags_2 },
    { 0x51df4b13, SUC_NE_lexicon_tags_2 },
    { 0x99d34b13, SUC_NE_lexicon_tags_2 },
    { 0x50614b14, SUC_NE_lexicon_tags_2 },
    { 0x6d054b16, SUC_NE_lexicon_tags_2 },
    { 0xae054b17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f4b1c, SUC_NE_lexicon_tags_2 },
    { 0x46bd4b1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde754b1f, SUC_NE_lexicon_tags_2 },
    { 0x8d394b1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x239f4b22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99294b24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7d4b29, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7b4b2d, SUC_NE_lexicon_tags_2 },
    { 0xd3b14b2d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xefb54b30, SUC_NE_lexicon_tags_2 },
    { 0x73334b31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c474b35, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x91634b38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0014b3a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc874b3e, SUC_NE_lexicon_tags_2 },
    { 0x34cb4b3e, SUC_NE_lexicon_tags_2 },
    { 0x822d4b3e, SUC_NE_lexicon_tags_2 },
    { 0xb9614b41, SUC_NE_lexicon_tags_19 },
    { 0xfce14b40, SUC_NE_lexicon_tags_2 },
    { 0x9da34b3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d74b46, SUC_NE_lexicon_tags_2 },
    { 0xdc1b4b47, SUC_NE_lexicon_tags_2 },
    { 0xd61f4b47, SUC_NE_lexicon_tags_2 },
    { 0x2f7d4b49, SUC_NE_lexicon_tags_2 },
    { 0xd10f4b49, SUC_NE_lexicon_tags_2 },
    { 0x9ce14b49, SUC_NE_lexicon_tags_2 },
    { 0xe86d4b4c, SUC_NE_lexicon_tags_2 },
    { 0x887b4b49, SUC_NE_lexicon_tags_2 },
    { 0x90c94b4c, SUC_NE_lexicon_tags_2 },
    { 0x64754b47, SUC_NE_lexicon_tags_2 },
    { 0xc2194b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c34b53, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb214b59, SUC_NE_lexicon_tags_2 },
    { 0xe3694b5a, SUC_NE_lexicon_tags_2 },
    { 0x45bf4b5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82714b60, SUC_NE_lexicon_tags_36 },
    { 0x62674b60, SUC_NE_lexicon_tags_2 },
    { 0xde14b62, SUC_NE_lexicon_tags_34 },
    { 0x2b514b63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75554b65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadff4b6b, SUC_NE_lexicon_tags_2 },
    { 0x88e14b6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc494b6e, SUC_NE_lexicon_tags_2 },
    { 0x116b4b6f, SUC_NE_lexicon_tags_2 },
    { 0x1bf74b70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1f4b74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf74b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7d4b7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae594b7e, SUC_NE_lexicon_tags_2 },
    { 0x299b4b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a94b82, SUC_NE_lexicon_tags_2 },
    { 0xb4e94b82, SUC_NE_lexicon_tags_2 },
    { 0xe634b83, SUC_NE_lexicon_tags_7 },
    { 0xfa114b85, SUC_NE_lexicon_tags_2 },
    { 0xf8c74b86, SUC_NE_lexicon_tags_1 },
    { 0x59df4b85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cc74b89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fd4b8d, SUC_NE_lexicon_tags_2 },
    { 0x4e1f4b8d, SUC_NE_lexicon_tags_2 },
    { 0x13a14b8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aaf4b91, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x32e54b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81b4b97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c94b9b, SUC_NE_lexicon_tags_2 },
    { 0x12894b9b, SUC_NE_lexicon_tags_2 },
    { 0x1094b9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db34ba0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f454ba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ead4ba7, SUC_NE_lexicon_tags_2 },
    { 0xb0b34ba8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f54bac, SUC_NE_lexicon_tags_2 },
    { 0xf7214bad, SUC_NE_lexicon_tags_2 },
    { 0xd0ed4bae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf414bb3, SUC_NE_lexicon_tags_2 },
    { 0x20854bb3, SUC_NE_lexicon_tags_30 },
    { 0x1a94bb5, SUC_NE_lexicon_tags_2 },
    { 0x354bb3, SUC_NE_lexicon_tags_2 },
    { 0xb6b14bb7, SUC_NE_lexicon_tags_4 },
    { 0xae2d4bb6, SUC_NE_lexicon_tags_19 },
    { 0xf1534bb3, SUC_NE_lexicon_tags_7 },
    { 0xb7294bba, SUC_NE_lexicon_tags_17 },
    { 0x67b94bbb, SUC_NE_lexicon_tags_2 },
    { 0xce0f4bbb, SUC_NE_lexicon_tags_2 },
    { 0xeaff4bbd, SUC_NE_lexicon_tags_2 },
    { 0xaa114bbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49874bc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb29d4bc5, SUC_NE_lexicon_tags_2 },
    { 0xdf234bc6, SUC_NE_lexicon_tags_7 },
    { 0xc7cd4bc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9d74bc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f054bcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb254bd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63d4bda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f594bdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84654be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585f4bea, SUC_NE_lexicon_tags_2 },
    { 0xe62b4beb, SUC_NE_lexicon_tags_2 },
    { 0x2654bec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faf4bf3, SUC_NE_lexicon_tags_2 },
    { 0x9b774bf4, SUC_NE_lexicon_tags_2 },
    { 0x98814bf5, SUC_NE_lexicon_tags_2 },
    { 0x7fcd4bf4, SUC_NE_lexicon_tags_7 },
    { 0x748d4bf7, SUC_NE_lexicon_tags_2 },
    { 0x759d4bf4, SUC_NE_lexicon_tags_2 },
    { 0x80bf4bf4, SUC_NE_lexicon_tags_2 },
    { 0x33d14bfa, SUC_NE_lexicon_tags_2 },
    { 0x73814bfb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2334c02, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c94c08, SUC_NE_lexicon_tags_4 },
    { 0xe8374c09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d4c0d, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fb4c12, SUC_NE_lexicon_tags_2 },
    { 0xe55b4c13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4554c16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c74c19, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d14c1d, SUC_NE_lexicon_tags_2 },
    { 0x88bb4c1d, SUC_NE_lexicon_tags_2 },
    { 0xfc614c1f, SUC_NE_lexicon_tags_2 },
    { 0xae594c20, SUC_NE_lexicon_tags_2 },
    { 0x741d4c1f, SUC_NE_lexicon_tags_2 },
    { 0x67614c22, SUC_NE_lexicon_tags_2 },
    { 0x472b4c1f, SUC_NE_lexicon_tags_2 },
    { 0xda1f4c1e, SUC_NE_lexicon_tags_2 },
    { 0x2a814c25, SUC_NE_lexicon_tags_2 },
    { 0x6d3d4c1e, SUC_NE_lexicon_tags_2 },
    { 0x22c14c23, SUC_NE_lexicon_tags_1 },
    { 0x24d14c28, SUC_NE_lexicon_tags_2 },
    { 0x6a014c29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c814c31, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x364b4c34, SUC_NE_lexicon_tags_2 },
    { 0x52db4c35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb4c3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ed34c40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600f4c47, SUC_NE_lexicon_tags_2 },
    { 0x69994c48, SUC_NE_lexicon_tags_2 },
    { 0x236f4c49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x707d4c4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cb4c51, SUC_NE_lexicon_tags_2 },
    { 0xe5f4c52, SUC_NE_lexicon_tags_2 },
    { 0x9bfb4c53, SUC_NE_lexicon_tags_2 },
    { 0x6c414c54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a4f4c56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c014c58, SUC_NE_lexicon_tags_10 },
    { 0xe0b74c59, SUC_NE_lexicon_tags_2 },
    { 0xb6934c58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ae74c5c, SUC_NE_lexicon_tags_2 },
    { 0x894f4c5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d954c5f, SUC_NE_lexicon_tags_2 },
    { 0x8e454c5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c54c63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6074c65, SUC_NE_lexicon_tags_2 },
    { 0xd5ed4c66, SUC_NE_lexicon_tags_2 },
    { 0x4fdf4c67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28ab4c69, SUC_NE_lexicon_tags_2 },
    { 0x182d4c6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24374c6c, SUC_NE_lexicon_tags_2 },
    { 0x2c2b4c6d, SUC_NE_lexicon_tags_368 },
    { 0xb6554c6e, SUC_NE_lexicon_tags_2 },
    { 0xb0774c6f, SUC_NE_lexicon_tags_4 },
    { 0xc1634c6f, SUC_NE_lexicon_tags_2 },
    { 0x194f4c6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b54c74, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36254c78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ed4c7b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932b4c80, SUC_NE_lexicon_tags_4 },
    { 0x4f094c80, SUC_NE_lexicon_tags_4 },
    { 0x33af4c82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56c34c84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a994c86, SUC_NE_lexicon_tags_2 },
    { 0xf6e94c87, SUC_NE_lexicon_tags_2 },
    { 0xaf7b4c86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7754c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5174c8d, SUC_NE_lexicon_tags_7 },
    { 0x6cf4c8e, SUC_NE_lexicon_tags_2 },
    { 0x73b34c8f, SUC_NE_lexicon_tags_2 },
    { 0x81ff4c8f, SUC_NE_lexicon_tags_2 },
    { 0x3fff4c91, SUC_NE_lexicon_tags_2 },
    { 0x9034c91, SUC_NE_lexicon_tags_2 },
    { 0x81814c93, SUC_NE_lexicon_tags_2 },
    { 0x1dd14c92, SUC_NE_lexicon_tags_2 },
    { 0xd84b4c95, SUC_NE_lexicon_tags_2 },
    { 0x386d4c96, SUC_NE_lexicon_tags_3 },
    { 0x4bbf4c96, SUC_NE_lexicon_tags_2 },
    { 0x4ef14c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44594c9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dc94ca0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a14ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991b4ca7, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7f4ca9, SUC_NE_lexicon_tags_13 },
    { 0x3a2b4caa, SUC_NE_lexicon_tags_2 },
    { 0xe42f4cab, SUC_NE_lexicon_tags_2 },
    { 0xc8b94cac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5294caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efb4cb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2314cb5, SUC_NE_lexicon_tags_2 },
    { 0xfaa94cb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5754cb9, SUC_NE_lexicon_tags_4 },
    { 0xa21d4cba, SUC_NE_lexicon_tags_2 },
    { 0x972d4cbb, SUC_NE_lexicon_tags_2 },
    { 0xad3f4cbc, SUC_NE_lexicon_tags_4 },
    { 0x90474cbd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe80f4cbf, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e654cc5, SUC_NE_lexicon_tags_2 },
    { 0xb0374cc6, SUC_NE_lexicon_tags_9 },
    { 0xb1634cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45234ccf, SUC_NE_lexicon_tags_2 },
    { 0xa41f4ccf, SUC_NE_lexicon_tags_2 },
    { 0xbceb4cd1, SUC_NE_lexicon_tags_2 },
    { 0x187f4cd1, SUC_NE_lexicon_tags_2 },
    { 0xdf794cd2, SUC_NE_lexicon_tags_2 },
    { 0x21e94cd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95494cdb, SUC_NE_lexicon_tags_7 },
    { 0x12034cdc, SUC_NE_lexicon_tags_2 },
    { 0x66674cdc, SUC_NE_lexicon_tags_2 },
    { 0x41b54cde, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x46594ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e34ce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29954ceb, SUC_NE_lexicon_tags_2 },
    { 0x6be14ceb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2acf4cee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60ef4cf0, SUC_NE_lexicon_tags_2 },
    { 0x8a1b4cf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9754cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed114cf6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e34cf9, SUC_NE_lexicon_tags_2 },
    { 0xbeb4cf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff4cfd, SUC_NE_lexicon_tags_2 },
    { 0x81414cfd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cd4d05, SUC_NE_lexicon_tags_2 },
    { 0x4c674d06, SUC_NE_lexicon_tags_2 },
    { 0x30e14d07, SUC_NE_lexicon_tags_2 },
    { 0xcbf14d07, SUC_NE_lexicon_tags_8 },
    { 0x7a634d09, SUC_NE_lexicon_tags_2 },
    { 0xddc14d06, SUC_NE_lexicon_tags_2 },
    { 0x4fd54d0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae14d0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2494d10, SUC_NE_lexicon_tags_2 },
    { 0x5ad94d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97954d13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa234d18, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42b4d1c, SUC_NE_lexicon_tags_2 },
    { 0x30834d1c, SUC_NE_lexicon_tags_2 },
    { 0xbd474d1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5b54d20, SUC_NE_lexicon_tags_2 },
    { 0x710d4d21, SUC_NE_lexicon_tags_2 },
    { 0x3e8d4d22, SUC_NE_lexicon_tags_2 },
    { 0x5fc94d23, SUC_NE_lexicon_tags_17 },
    { 0x91d74d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeba14d26, SUC_NE_lexicon_tags_2 },
    { 0x60cb4d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef854d29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141b4d2f, SUC_NE_lexicon_tags_2 },
    { 0xbc774d2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa94d33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87354d35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9534d38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dd74d3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c54d3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d394d41, SUC_NE_lexicon_tags_2 },
    { 0x7cf74d41, SUC_NE_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54f4d48, SUC_NE_lexicon_tags_3 },
    { 0x3acf4d48, SUC_NE_lexicon_tags_2 },
    { 0xaba74d4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51494d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5b94d4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85334d51, SUC_NE_lexicon_tags_2 },
    { 0x7ff54d51, SUC_NE_lexicon_tags_2 },
    { 0x81794d53, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d4d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f494d5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81d14d5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x572d4d62, SUC_NE_lexicon_tags_4 },
    { 0x973d4d63, SUC_NE_lexicon_tags_17 },
    { 0x23a74d64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef4d67, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xee14d69, SUC_NE_lexicon_tags_75 },
    { 0x54294d6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d354d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3e54d6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a54d74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ef4d78, SUC_NE_lexicon_tags_2 },
    { 0xe3c74d78, SUC_NE_lexicon_tags_2 },
    { 0xb1274d7a, SUC_NE_lexicon_tags_2 },
    { 0x5fd4d7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac94d7e, SUC_NE_lexicon_tags_341 },
    { 0xe1c14d7f, SUC_NE_lexicon_tags_2 },
    { 0xb59d4d7f, SUC_NE_lexicon_tags_7 },
    { 0xf2e34d80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4714d86, SUC_NE_lexicon_tags_2 },
    { 0x1a494d87, SUC_NE_lexicon_tags_2 },
    { 0xdb074d88, SUC_NE_lexicon_tags_2 },
    { 0x53874d87, SUC_NE_lexicon_tags_2 },
    { 0x64794d8a, SUC_NE_lexicon_tags_2 },
    { 0xa234d8a, SUC_NE_lexicon_tags_2 },
    { 0x8a2d4d8a, SUC_NE_lexicon_tags_2 },
    { 0x2eed4d8a, SUC_NE_lexicon_tags_2 },
    { 0x374b4d8e, SUC_NE_lexicon_tags_2 },
    { 0x1dcb4d8f, SUC_NE_lexicon_tags_2 },
    { 0x9eef4d90, SUC_NE_lexicon_tags_2 },
    { 0x67434d90, SUC_NE_lexicon_tags_2 },
    { 0x503d4d8f, SUC_NE_lexicon_tags_2 },
    { 0x77b54d93, SUC_NE_lexicon_tags_2 },
    { 0x28394d93, SUC_NE_lexicon_tags_2 },
    { 0x1694d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9d4d99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d94d9c, SUC_NE_lexicon_tags_2 },
    { 0xb30f4d9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac5d4d9f, SUC_NE_lexicon_tags_2 },
    { 0x3c654da0, SUC_NE_lexicon_tags_2 },
    { 0xa6d74da1, SUC_NE_lexicon_tags_2 },
    { 0xb9234da1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bef4da4, SUC_NE_lexicon_tags_2 },
    { 0xa4014da4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff4da9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d4db0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275f4db8, SUC_NE_lexicon_tags_2 },
    { 0x111d4db9, SUC_NE_lexicon_tags_34 },
    { 0xdd814db9, SUC_NE_lexicon_tags_1 },
    { 0xb7874dbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4774dbe, SUC_NE_lexicon_tags_2 },
    { 0x908f4dbf, SUC_NE_lexicon_tags_2 },
    { 0x68274dc0, SUC_NE_lexicon_tags_2 },
    { 0x46374dc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84934dc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ed74dc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14874dcc, SUC_NE_lexicon_tags_3 },
    { 0xc6254dcd, SUC_NE_lexicon_tags_2 },
    { 0xe69d4dce, SUC_NE_lexicon_tags_2 },
    { 0xade34dcf, SUC_NE_lexicon_tags_2 },
    { 0xcdbb4dce, SUC_NE_lexicon_tags_2 },
    { 0x66d4dcc, SUC_NE_lexicon_tags_2 },
    { 0xb354dd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49954dda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bd34ddc, SUC_NE_lexicon_tags_2 },
    { 0xcdbb4ddc, SUC_NE_lexicon_tags_2 },
    { 0x62014dde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbeb4de0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0314de2, SUC_NE_lexicon_tags_2 },
    { 0x92194de3, SUC_NE_lexicon_tags_2 },
    { 0x86d74de4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3434de6, SUC_NE_lexicon_tags_2 },
    { 0x324f4de6, SUC_NE_lexicon_tags_34 },
    { 0x5cd54de7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f994dec, SUC_NE_lexicon_tags_2 },
    { 0xc8754dec, SUC_NE_lexicon_tags_2 },
    { 0xbd734dee, SUC_NE_lexicon_tags_2 },
    { 0xbdef4def, SUC_NE_lexicon_tags_2 },
    { 0xd2894dee, SUC_NE_lexicon_tags_2 },
    { 0x61b94df1, SUC_NE_lexicon_tags_2 },
    { 0x460b4dec, SUC_NE_lexicon_tags_2 },
    { 0x39694df3, SUC_NE_lexicon_tags_2 },
    { 0x4d194df4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4674df8, SUC_NE_lexicon_tags_3 },
    { 0x58df4df8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439b4e00, SUC_NE_lexicon_tags_2 },
    { 0x5a6b4e00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d94e04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed674e08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde914e0b, SUC_NE_lexicon_tags_2 },
    { 0x4e754e0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc94e0f, SUC_NE_lexicon_tags_2 },
    { 0xaacb4e10, SUC_NE_lexicon_tags_2 },
    { 0xb16d4e11, SUC_NE_lexicon_tags_2 },
    { 0xf50b4e11, SUC_NE_lexicon_tags_2 },
    { 0x9cc94e11, SUC_NE_lexicon_tags_2 },
    { 0xe68f4e11, SUC_NE_lexicon_tags_2 },
    { 0x82554e15, SUC_NE_lexicon_tags_2 },
    { 0x3f894e15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x931b4e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdda94e1b, SUC_NE_lexicon_tags_2 },
    { 0x89434e1c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9f4e1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdff4e21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47334e23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd05b4e25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x215d4e28, SUC_NE_lexicon_tags_2 },
    { 0xb44b4e29, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc94e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x918d4e31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f974e34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5494e36, SUC_NE_lexicon_tags_2 },
    { 0x11514e37, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x98194e39, SUC_NE_lexicon_tags_2 },
    { 0x2e5f4e39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6854e3c, SUC_NE_lexicon_tags_2 },
    { 0xed34e3c, SUC_NE_lexicon_tags_2 },
    { 0xa8134e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17154e42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b74e45, SUC_NE_lexicon_tags_2 },
    { 0xa7d74e46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84f54e48, SUC_NE_lexicon_tags_17 },
    { 0x18894e49, SUC_NE_lexicon_tags_2 },
    { 0xcf034e4a, SUC_NE_lexicon_tags_2 },
    { 0xd2974e4b, SUC_NE_lexicon_tags_2 },
    { 0xfba34e4a, SUC_NE_lexicon_tags_2 },
    { 0xf1134e4d, SUC_NE_lexicon_tags_3 },
    { 0xb21d4e4e, SUC_NE_lexicon_tags_4 },
    { 0x16bf4e4c, SUC_NE_lexicon_tags_2 },
    { 0xb14f4e50, SUC_NE_lexicon_tags_2 },
    { 0xc67b4e4a, SUC_NE_lexicon_tags_2 },
    { 0xa7914e52, SUC_NE_lexicon_tags_2 },
    { 0xe04f4e4b, SUC_NE_lexicon_tags_2 },
    { 0x28374e54, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3b4e5a, SUC_NE_lexicon_tags_2 },
    { 0xd49b4e5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3f4e5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaed4e63, SUC_NE_lexicon_tags_2 },
    { 0xc4534e64, SUC_NE_lexicon_tags_2 },
    { 0xb1094e65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c74e6e, SUC_NE_lexicon_tags_0 },
    { 0x4cab4e6e, SUC_NE_lexicon_tags_2 },
    { 0xdf4e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd834e72, SUC_NE_lexicon_tags_2 },
    { 0xa3194e73, SUC_NE_lexicon_tags_2 },
    { 0x4a6b4e72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb4e79, SUC_NE_lexicon_tags_2 },
    { 0x32c74e7a, SUC_NE_lexicon_tags_2 },
    { 0x4c434e7b, SUC_NE_lexicon_tags_2 },
    { 0xed074e7b, SUC_NE_lexicon_tags_2 },
    { 0x6c3f4e7d, SUC_NE_lexicon_tags_2 },
    { 0xdab54e7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d14e80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60134e85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8714e88, SUC_NE_lexicon_tags_2 },
    { 0x54274e89, SUC_NE_lexicon_tags_2 },
    { 0x23134e89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa34e8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc27b4e91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb94e95, SUC_NE_lexicon_tags_2 },
    { 0x26fb4e96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a514e99, SUC_NE_lexicon_tags_1 },
    { 0xa3c14e9a, SUC_NE_lexicon_tags_2 },
    { 0xc18d4e9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba734ea0, SUC_NE_lexicon_tags_2 },
    { 0xfcfd4ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e354ea3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x76e34ea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe24d4ea7, SUC_NE_lexicon_tags_2 },
    { 0x24554ea8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971d4eac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc81b4eae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713b4eb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff794eb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8934ec0, SUC_NE_lexicon_tags_2 },
    { 0x718d4ec1, SUC_NE_lexicon_tags_2 },
    { 0xfc0f4ec2, SUC_NE_lexicon_tags_2 },
    { 0xca8d4ec3, SUC_NE_lexicon_tags_2 },
    { 0x6e914ec1, SUC_NE_lexicon_tags_2 },
    { 0x80c34ec4, SUC_NE_lexicon_tags_2 },
    { 0x5dc74ec6, SUC_NE_lexicon_tags_2 },
    { 0x4ce74ec7, SUC_NE_lexicon_tags_2 },
    { 0xf5734ec8, SUC_NE_lexicon_tags_2 },
    { 0x84694ec6, SUC_NE_lexicon_tags_2 },
    { 0x499d4ec4, SUC_NE_lexicon_tags_2 },
    { 0xf1d54ec6, SUC_NE_lexicon_tags_2 },
    { 0xad694ecc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3d4ed1, SUC_NE_lexicon_tags_2 },
    { 0xc3614ed1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16a14ed4, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x422d4ed6, SUC_NE_lexicon_tags_2 },
    { 0x657b4ed6, SUC_NE_lexicon_tags_2 },
    { 0x43574ed6, SUC_NE_lexicon_tags_2 },
    { 0xa674ed6, SUC_NE_lexicon_tags_2 },
    { 0xf1594ed9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14834ede, SUC_NE_lexicon_tags_2 },
    { 0x41014edf, SUC_NE_lexicon_tags_2 },
    { 0x97474ee0, SUC_NE_lexicon_tags_1 },
    { 0x8daf4ee1, SUC_NE_lexicon_tags_2 },
    { 0x3d834ee2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5854ee4, SUC_NE_lexicon_tags_2 },
    { 0xab394ee5, SUC_NE_lexicon_tags_2 },
    { 0xc9254ee5, SUC_NE_lexicon_tags_2 },
    { 0x44654ee7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80194eea, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e254eec, SUC_NE_lexicon_tags_1 },
    { 0x1f6f4eed, SUC_NE_lexicon_tags_2 },
    { 0xd0a74eed, SUC_NE_lexicon_tags_2 },
    { 0xe3c14eee, SUC_NE_lexicon_tags_2 },
    { 0x16214eee, SUC_NE_lexicon_tags_2 },
    { 0x4e914ef1, SUC_NE_lexicon_tags_2 },
    { 0xfcc34ef2, SUC_NE_lexicon_tags_2 },
    { 0xcb6f4ef3, SUC_NE_lexicon_tags_2 },
    { 0xbfcd4ef3, SUC_NE_lexicon_tags_2 },
    { 0x13f54ef5, SUC_NE_lexicon_tags_2 },
    { 0x86054ef6, SUC_NE_lexicon_tags_2 },
    { 0x82c34ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dd4efa, SUC_NE_lexicon_tags_2 },
    { 0x4a254efb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9ab4efd, SUC_NE_lexicon_tags_2 },
    { 0x34c14efe, SUC_NE_lexicon_tags_2 },
    { 0x9fa54eff, SUC_NE_lexicon_tags_2 },
    { 0x79a94eff, SUC_NE_lexicon_tags_77 },
    { 0xe3c54efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fe34f03, SUC_NE_lexicon_tags_5 },
    { 0x78bb4f03, SUC_NE_lexicon_tags_2 },
    { 0x91294f05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c14f09, SUC_NE_lexicon_tags_2 },
    { 0xe3914f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6b4f0e, SUC_NE_lexicon_tags_2 },
    { 0x36474f0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ec54f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604b4f18, SUC_NE_lexicon_tags_2 },
    { 0x6f474f19, SUC_NE_lexicon_tags_2 },
    { 0x571b4f18, SUC_NE_lexicon_tags_2 },
    { 0x87e34f1b, SUC_NE_lexicon_tags_2 },
    { 0x9f714f1c, SUC_NE_lexicon_tags_2 },
    { 0x4b514f1d, SUC_NE_lexicon_tags_2 },
    { 0x361f4f1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9daf4f21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50594f24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf74f27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5f4f2a, SUC_NE_lexicon_tags_2 },
    { 0x93a54f2b, SUC_NE_lexicon_tags_2 },
    { 0xa7ed4f2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb14f30, SUC_NE_lexicon_tags_4 },
    { 0x9a794f31, SUC_NE_lexicon_tags_2 },
    { 0xaa814f32, SUC_NE_lexicon_tags_2 },
    { 0x678d4f32, SUC_NE_lexicon_tags_2 },
    { 0x4cef4f34, SUC_NE_lexicon_tags_2 },
    { 0xe0934f35, SUC_NE_lexicon_tags_2 },
    { 0xa7054f34, SUC_NE_lexicon_tags_7 },
    { 0x83c34f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57154f3d, SUC_NE_lexicon_tags_2 },
    { 0x11654f3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7514f41, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba414f44, SUC_NE_lexicon_tags_2 },
    { 0x59e74f45, SUC_NE_lexicon_tags_2 },
    { 0x8dfd4f46, SUC_NE_lexicon_tags_2 },
    { 0x99e94f47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf4f4b, SUC_NE_lexicon_tags_4 },
    { 0x480d4f4c, SUC_NE_lexicon_tags_2 },
    { 0x61654f4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d4f51, SUC_NE_lexicon_tags_2 },
    { 0xff0d4f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1f4f55, SUC_NE_lexicon_tags_24 },
    { 0xb42d4f55, SUC_NE_lexicon_tags_2 },
    { 0x3c514f57, SUC_NE_lexicon_tags_2 },
    { 0xb6574f55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8554f5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f4f62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x104f4f64, SUC_NE_lexicon_tags_2 },
    { 0xf9d94f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a394f6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0fd4f6e, SUC_NE_lexicon_tags_59 },
    { 0x9d0b4f6f, SUC_NE_lexicon_tags_2 },
    { 0x56994f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca3d4f72, SUC_NE_lexicon_tags_2 },
    { 0xea934f72, SUC_NE_lexicon_tags_2 },
    { 0xe3374f72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef054f76, SUC_NE_lexicon_tags_2 },
    { 0x384f4f76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbea94f79, SUC_NE_lexicon_tags_2 },
    { 0x1cd34f7a, SUC_NE_lexicon_tags_2 },
    { 0x442b4f79, SUC_NE_lexicon_tags_2 },
    { 0xef894f7c, SUC_NE_lexicon_tags_2 },
    { 0xbe314f7a, SUC_NE_lexicon_tags_2 },
    { 0x2f774f7d, SUC_NE_lexicon_tags_19 },
    { 0x8b634f7f, SUC_NE_lexicon_tags_2 },
    { 0xe6d94f80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75314f83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e34f88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x704d4f8a, SUC_NE_lexicon_tags_2 },
    { 0xa3194f8b, SUC_NE_lexicon_tags_2 },
    { 0x77c14f8c, SUC_NE_lexicon_tags_2 },
    { 0xe01d4f8b, SUC_NE_lexicon_tags_2 },
    { 0x44614f8e, SUC_NE_lexicon_tags_2 },
    { 0x8b094f8d, SUC_NE_lexicon_tags_2 },
    { 0x57034f90, SUC_NE_lexicon_tags_2 },
    { 0x36274f90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b34f94, SUC_NE_lexicon_tags_2 },
    { 0x822f4f94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bc74f97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad14f9a, SUC_NE_lexicon_tags_2 },
    { 0x7d54f9a, SUC_NE_lexicon_tags_2 },
    { 0x1fbd4f9b, SUC_NE_lexicon_tags_2 },
    { 0xcdb4f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea74fa2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650d4fa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6f4fac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bf4fb0, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x42cb4fb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2d94fb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4434fb8, SUC_NE_lexicon_tags_17 },
    { 0xeac34fb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232f4fc2, SUC_NE_lexicon_tags_2 },
    { 0x74bb4fc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5854fc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x440f4fc7, SUC_NE_lexicon_tags_2 },
    { 0x4f0d4fc7, SUC_NE_lexicon_tags_2 },
    { 0x14494fc9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd94fd0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x29834fd2, SUC_NE_lexicon_tags_2 },
    { 0x15b54fd2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fd4fd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97594fdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37094fde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55e94fe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2beb4fe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1254feb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38d4fef, SUC_NE_lexicon_tags_2 },
    { 0xfbf94ff0, SUC_NE_lexicon_tags_2 },
    { 0xbb3d4ff1, SUC_NE_lexicon_tags_2 },
    { 0x195d4ff2, SUC_NE_lexicon_tags_2 },
    { 0xc7954ff3, SUC_NE_lexicon_tags_2 },
    { 0xf1c94fef, SUC_NE_lexicon_tags_2 },
    { 0x85b4ff5, SUC_NE_lexicon_tags_3 },
    { 0x331d4ff6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d4ff9, SUC_NE_lexicon_tags_2 },
    { 0x62e74ffa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabed4ffc, SUC_NE_lexicon_tags_2 },
    { 0x75294ffc, SUC_NE_lexicon_tags_4 },
    { 0xbba94ffe, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c55001, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf975003, SUC_NE_lexicon_tags_2 },
    { 0xc4395004, SUC_NE_lexicon_tags_1 },
    { 0x45fd5005, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb5008, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe475500a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e27500d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8755012, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffc55014, SUC_NE_lexicon_tags_2 },
    { 0x53995014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca35019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3047501b, SUC_NE_lexicon_tags_2 },
    { 0xfaab501b, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x3ded501e, SUC_NE_lexicon_tags_2 },
    { 0x398b501e, SUC_NE_lexicon_tags_2 },
    { 0xfff7501f, SUC_NE_lexicon_tags_2 },
    { 0x6587501f, SUC_NE_lexicon_tags_2 },
    { 0x1d51501e, SUC_NE_lexicon_tags_2 },
    { 0x31955022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b915027, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88f502b, SUC_NE_lexicon_tags_2 },
    { 0xd949502c, SUC_NE_lexicon_tags_5 },
    { 0x5453502d, SUC_NE_lexicon_tags_2 },
    { 0xf321502c, SUC_NE_lexicon_tags_2 },
    { 0x672b502f, SUC_NE_lexicon_tags_2 },
    { 0x7319502f, SUC_NE_lexicon_tags_40 },
    { 0xe5a95031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68415034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e95037, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc5e75039, SUC_NE_lexicon_tags_2 },
    { 0x2e1f5039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa003503c, SUC_NE_lexicon_tags_2 },
    { 0xb6a1503d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a255040, SUC_NE_lexicon_tags_2 },
    { 0x9cf75040, SUC_NE_lexicon_tags_2 },
    { 0xdb235042, SUC_NE_lexicon_tags_2 },
    { 0xcc5d5043, SUC_NE_lexicon_tags_2 },
    { 0x72375044, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dad5048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b7504c, SUC_NE_lexicon_tags_17 },
    { 0xdf7b504d, SUC_NE_lexicon_tags_2 },
    { 0xa2d5504d, SUC_NE_lexicon_tags_3 },
    { 0xbf1f504f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59915054, SUC_NE_lexicon_tags_2 },
    { 0xa15d5055, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75655058, SUC_NE_lexicon_tags_2 },
    { 0xc84f5059, SUC_NE_lexicon_tags_2 },
    { 0x2def5058, SUC_NE_lexicon_tags_2 },
    { 0x53c1505b, SUC_NE_lexicon_tags_2 },
    { 0x8d495059, SUC_NE_lexicon_tags_2 },
    { 0x1f5b5058, SUC_NE_lexicon_tags_2 },
    { 0x36d7505e, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xf66f5060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1175063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d75066, SUC_NE_lexicon_tags_2 },
    { 0x524b5067, SUC_NE_lexicon_tags_2 },
    { 0x47935068, SUC_NE_lexicon_tags_2 },
    { 0x29bf5069, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b9506f, SUC_NE_lexicon_tags_2 },
    { 0x93f15070, SUC_NE_lexicon_tags_2 },
    { 0xa2f506f, SUC_NE_lexicon_tags_5 },
    { 0xf195072, SUC_NE_lexicon_tags_2 },
    { 0x60035073, SUC_NE_lexicon_tags_2 },
    { 0xa3c95073, SUC_NE_lexicon_tags_2 },
    { 0x89155072, SUC_NE_lexicon_tags_2 },
    { 0xe1635076, SUC_NE_lexicon_tags_1 },
    { 0x90475077, SUC_NE_lexicon_tags_2 },
    { 0xabd15078, SUC_NE_lexicon_tags_2 },
    { 0x62e75078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c1d507b, SUC_NE_lexicon_tags_2 },
    { 0x10d3507c, SUC_NE_lexicon_tags_2 },
    { 0x711507d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c375080, SUC_NE_lexicon_tags_3 },
    { 0x57995081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99b55083, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd5086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9395088, SUC_NE_lexicon_tags_2 },
    { 0x6c0f5089, SUC_NE_lexicon_tags_7 },
    { 0x19595089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1d508d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560f5092, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66635097, SUC_NE_lexicon_tags_2 },
    { 0x6f535098, SUC_NE_lexicon_tags_6 },
    { 0xc1b15099, SUC_NE_lexicon_tags_2 },
    { 0x931f509a, SUC_NE_lexicon_tags_2 },
    { 0x1c13509a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8769509e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5d50a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40f50a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd2750a8, SUC_NE_lexicon_tags_2 },
    { 0x66b550a9, SUC_NE_lexicon_tags_2 },
    { 0xc72150aa, SUC_NE_lexicon_tags_2 },
    { 0xfe2350ab, SUC_NE_lexicon_tags_2 },
    { 0xcced50ac, SUC_NE_lexicon_tags_13 },
    { 0xf9f50ab, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e950b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34350b6, SUC_NE_lexicon_tags_0 },
    { 0xdeb750b7, SUC_NE_lexicon_tags_2 },
    { 0xcc2350b8, SUC_NE_lexicon_tags_2 },
    { 0x2d5b50b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3550bd, SUC_NE_lexicon_tags_2 },
    { 0xa8e150be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51950c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fc150c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe28f50c5, SUC_NE_lexicon_tags_2 },
    { 0x236b50c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x152d50c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadef50cb, SUC_NE_lexicon_tags_2 },
    { 0x7b7550cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0950d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ea950d2, SUC_NE_lexicon_tags_2 },
    { 0x12dd50d3, SUC_NE_lexicon_tags_87 },
    { 0, NULL },
    { 0xf43750d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x150950d8, SUC_NE_lexicon_tags_2 },
    { 0x67eb50d9, SUC_NE_lexicon_tags_2 },
    { 0x3e8b50d8, SUC_NE_lexicon_tags_2 },
    { 0xca2950db, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c350df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd350e4, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a950ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x700950ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d50f2, SUC_NE_lexicon_tags_2 },
    { 0xc16d50f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f50f7, SUC_NE_lexicon_tags_2 },
    { 0xe6af50f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b350fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8695100, SUC_NE_lexicon_tags_2 },
    { 0x81e15101, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c215104, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58e55106, SUC_NE_lexicon_tags_2 },
    { 0x79d15106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6941510c, SUC_NE_lexicon_tags_2 },
    { 0x896f510d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5f5110, SUC_NE_lexicon_tags_2 },
    { 0x853f5111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15116, SUC_NE_lexicon_tags_2 },
    { 0xd2eb5117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eeb5119, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bf511d, SUC_NE_lexicon_tags_2 },
    { 0xc023511e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad5126, SUC_NE_lexicon_tags_2 },
    { 0x691d5127, SUC_NE_lexicon_tags_2 },
    { 0xd0275128, SUC_NE_lexicon_tags_2 },
    { 0xfb355127, SUC_NE_lexicon_tags_2 },
    { 0x8ef5512a, SUC_NE_lexicon_tags_2 },
    { 0x80ed5128, SUC_NE_lexicon_tags_2 },
    { 0x5d57512c, SUC_NE_lexicon_tags_2 },
    { 0x24df5126, SUC_NE_lexicon_tags_2 },
    { 0x62b7512e, SUC_NE_lexicon_tags_2 },
    { 0xab5512f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27755132, SUC_NE_lexicon_tags_2 },
    { 0x37d95132, SUC_NE_lexicon_tags_34 },
    { 0x1b4f5132, SUC_NE_lexicon_tags_4 },
    { 0xa4b5135, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b5139, SUC_NE_lexicon_tags_2 },
    { 0x1da15139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x870b513c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b935143, SUC_NE_lexicon_tags_2 },
    { 0x8b3f5143, SUC_NE_lexicon_tags_1 },
    { 0x8be75145, SUC_NE_lexicon_tags_2 },
    { 0x7ba35146, SUC_NE_lexicon_tags_4 },
    { 0x9d775147, SUC_NE_lexicon_tags_2 },
    { 0x3135144, SUC_NE_lexicon_tags_2 },
    { 0x7dad5149, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0xa05f514b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab95152, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57755157, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71515c, SUC_NE_lexicon_tags_2 },
    { 0xa981515d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc75160, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b5166, SUC_NE_lexicon_tags_2 },
    { 0x30b35166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x774d516a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02d516d, SUC_NE_lexicon_tags_2 },
    { 0x3a71516e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0635172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96535174, SUC_NE_lexicon_tags_7 },
    { 0xba775175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97af5178, SUC_NE_lexicon_tags_24 },
    { 0xb81f5179, SUC_NE_lexicon_tags_1 },
    { 0xb581517a, SUC_NE_lexicon_tags_2 },
    { 0x88f7517b, SUC_NE_lexicon_tags_8 },
    { 0xaee7517b, SUC_NE_lexicon_tags_2 },
    { 0xa507517a, SUC_NE_lexicon_tags_2 },
    { 0xa5855179, SUC_NE_lexicon_tags_3 },
    { 0x82c5517f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b35183, SUC_NE_lexicon_tags_2 },
    { 0xdca75184, SUC_NE_lexicon_tags_2 },
    { 0x31255184, SUC_NE_lexicon_tags_2 },
    { 0x28115186, SUC_NE_lexicon_tags_2 },
    { 0xe3855187, SUC_NE_lexicon_tags_34 },
    { 0x92eb5188, SUC_NE_lexicon_tags_2 },
    { 0xe1f55185, SUC_NE_lexicon_tags_2 },
    { 0x7ce7518a, SUC_NE_lexicon_tags_2 },
    { 0x241f5185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe355190, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb7535192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268f5196, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f7b5198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dcf519a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bd519d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ab51a2, SUC_NE_lexicon_tags_2 },
    { 0xe2d151a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc5f51a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e151aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9d51ac, SUC_NE_lexicon_tags_2 },
    { 0x66d951ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1751b1, SUC_NE_lexicon_tags_2 },
    { 0x377351b1, SUC_NE_lexicon_tags_2 },
    { 0x12fd51b3, SUC_NE_lexicon_tags_2 },
    { 0x920f51b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x456d51b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb59951b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26c751ba, SUC_NE_lexicon_tags_2 },
    { 0xc65551bb, SUC_NE_lexicon_tags_2 },
    { 0x594b51bc, SUC_NE_lexicon_tags_394 },
    { 0xa90d51bb, SUC_NE_lexicon_tags_2 },
    { 0xdf6f51bb, SUC_NE_lexicon_tags_4 },
    { 0x6c5b51bf, SUC_NE_lexicon_tags_2 },
    { 0xc11b51c0, SUC_NE_lexicon_tags_2 },
    { 0xa37751c0, SUC_NE_lexicon_tags_2 },
    { 0x82551c1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf951c8, SUC_NE_lexicon_tags_5 },
    { 0x5db951c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x423f51cb, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff951d0, SUC_NE_lexicon_tags_2 },
    { 0xba5d51d1, SUC_NE_lexicon_tags_2 },
    { 0xc1fb51d2, SUC_NE_lexicon_tags_2 },
    { 0xd85b51d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ed51d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6a351db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x230b51dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883d51e3, SUC_NE_lexicon_tags_2 },
    { 0xc5b751e4, SUC_NE_lexicon_tags_2 },
    { 0xf8351e5, SUC_NE_lexicon_tags_2 },
    { 0x8d8951e4, SUC_NE_lexicon_tags_2 },
    { 0x652151e7, SUC_NE_lexicon_tags_2 },
    { 0xdd2351e8, SUC_NE_lexicon_tags_11 },
    { 0x9b1151e8, SUC_NE_lexicon_tags_2 },
    { 0xb97951ea, SUC_NE_lexicon_tags_2 },
    { 0x969d51eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d751ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fb51f1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x422f51f3, SUC_NE_lexicon_tags_2 },
    { 0xf0b551f4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x14351f6, SUC_NE_lexicon_tags_2 },
    { 0x6b2951f6, SUC_NE_lexicon_tags_2 },
    { 0x8a5351f8, SUC_NE_lexicon_tags_2 },
    { 0xd88351f8, SUC_NE_lexicon_tags_2 },
    { 0xdb9b51fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5b51ff, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2dd35201, SUC_NE_lexicon_tags_2 },
    { 0xe7695201, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb75207, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xb0135209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a95520c, SUC_NE_lexicon_tags_2 },
    { 0x2cfb520d, SUC_NE_lexicon_tags_2 },
    { 0x7251520d, SUC_NE_lexicon_tags_36 },
    { 0xaa9520e, SUC_NE_lexicon_tags_34 },
    { 0xa9a95210, SUC_NE_lexicon_tags_2 },
    { 0xca035211, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee295216, SUC_NE_lexicon_tags_2 },
    { 0xbb8b5217, SUC_NE_lexicon_tags_2 },
    { 0xe7795218, SUC_NE_lexicon_tags_4 },
    { 0x2df15216, SUC_NE_lexicon_tags_2 },
    { 0x4c1d5217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffd5223, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1115225, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc655227, SUC_NE_lexicon_tags_30 },
    { 0xfa435228, SUC_NE_lexicon_tags_2 },
    { 0x31135229, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a1522e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33935232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bb55234, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ad5236, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeba55238, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc9c3523a, SUC_NE_lexicon_tags_2 },
    { 0x5deb523a, SUC_NE_lexicon_tags_2 },
    { 0xa5ef523a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9523f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea415241, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d5246, SUC_NE_lexicon_tags_2 },
    { 0x6d835247, SUC_NE_lexicon_tags_2 },
    { 0x68955246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f1524b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f524e, SUC_NE_lexicon_tags_2 },
    { 0x78bd524e, SUC_NE_lexicon_tags_2 },
    { 0x9bef524f, SUC_NE_lexicon_tags_2 },
    { 0x65115251, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d95253, SUC_NE_lexicon_tags_2 },
    { 0x1a2d5254, SUC_NE_lexicon_tags_2 },
    { 0xc4a15255, SUC_NE_lexicon_tags_2 },
    { 0xee435255, SUC_NE_lexicon_tags_2 },
    { 0x4e055257, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b6f5259, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afb525c, SUC_NE_lexicon_tags_2 },
    { 0xf1d1525d, SUC_NE_lexicon_tags_2 },
    { 0x62d9525e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c55261, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x659d5263, SUC_NE_lexicon_tags_2 },
    { 0x5a455263, SUC_NE_lexicon_tags_2 },
    { 0x99a15265, SUC_NE_lexicon_tags_2 },
    { 0x5c475266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad895268, SUC_NE_lexicon_tags_2 },
    { 0x2c4d5268, SUC_NE_lexicon_tags_2 },
    { 0x3b8f5269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b1526d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd35274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f5277, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a31527c, SUC_NE_lexicon_tags_17 },
    { 0xcc4d527d, SUC_NE_lexicon_tags_2 },
    { 0xfba7527c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95555280, SUC_NE_lexicon_tags_36 },
    { 0x95ff5281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d35284, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c1f5286, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c01528b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19095290, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bf5296, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdf529b, SUC_NE_lexicon_tags_2 },
    { 0x6891529b, SUC_NE_lexicon_tags_2 },
    { 0x69df529d, SUC_NE_lexicon_tags_2 },
    { 0x1611529b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x563152a0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x98952a3, SUC_NE_lexicon_tags_2 },
    { 0xbd6552a4, SUC_NE_lexicon_tags_2 },
    { 0xe1f552a4, SUC_NE_lexicon_tags_4 },
    { 0x59352a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46f352a8, SUC_NE_lexicon_tags_2 },
    { 0xc68f52a8, SUC_NE_lexicon_tags_3 },
    { 0x504752a9, SUC_NE_lexicon_tags_2 },
    { 0xa6b52a8, SUC_NE_lexicon_tags_2 },
    { 0x852f52a9, SUC_NE_lexicon_tags_2 },
    { 0x6ff352a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ae552af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69a952b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x621b52b4, SUC_NE_lexicon_tags_4 },
    { 0xa30952b4, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x846352b7, SUC_NE_lexicon_tags_7 },
    { 0xb93152b8, SUC_NE_lexicon_tags_136 },
    { 0x588b52b8, SUC_NE_lexicon_tags_2 },
    { 0xeb7d52ba, SUC_NE_lexicon_tags_2 },
    { 0xac7b52bb, SUC_NE_lexicon_tags_2 },
    { 0x83352bb, SUC_NE_lexicon_tags_172 },
    { 0xfd0352bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f952bf, SUC_NE_lexicon_tags_2 },
    { 0x29df52c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28ad52c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64152c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e752c7, SUC_NE_lexicon_tags_2 },
    { 0x381f52c7, SUC_NE_lexicon_tags_7 },
    { 0x193152c7, SUC_NE_lexicon_tags_2 },
    { 0x3cbb52c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf59752cd, SUC_NE_lexicon_tags_2 },
    { 0x3a0f52ce, SUC_NE_lexicon_tags_2 },
    { 0x9b0b52cf, SUC_NE_lexicon_tags_2 },
    { 0x47f752d0, SUC_NE_lexicon_tags_2 },
    { 0xded152d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e952d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df552d8, SUC_NE_lexicon_tags_2 },
    { 0x24d952d8, SUC_NE_lexicon_tags_2 },
    { 0x964552da, SUC_NE_lexicon_tags_2 },
    { 0xcd8752db, SUC_NE_lexicon_tags_2 },
    { 0xead52dc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37752e4, SUC_NE_lexicon_tags_2 },
    { 0x705b52e5, SUC_NE_lexicon_tags_2 },
    { 0x896752e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde8352e8, SUC_NE_lexicon_tags_4 },
    { 0x62cb52e9, SUC_NE_lexicon_tags_2 },
    { 0x34b552e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72352ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127952f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d0752f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff152fc, SUC_NE_lexicon_tags_2 },
    { 0xf3752fd, SUC_NE_lexicon_tags_2 },
    { 0xffa552fd, SUC_NE_lexicon_tags_2 },
    { 0xdc1f52ff, SUC_NE_lexicon_tags_2 },
    { 0x828b5300, SUC_NE_lexicon_tags_2 },
    { 0xd0c752fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b195303, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5715305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x697b5307, SUC_NE_lexicon_tags_2 },
    { 0xa0075308, SUC_NE_lexicon_tags_2 },
    { 0xca055309, SUC_NE_lexicon_tags_2 },
    { 0x92eb5309, SUC_NE_lexicon_tags_2 },
    { 0x2471530b, SUC_NE_lexicon_tags_2 },
    { 0x1c635309, SUC_NE_lexicon_tags_2 },
    { 0x68d530d, SUC_NE_lexicon_tags_2 },
    { 0x9d67530d, SUC_NE_lexicon_tags_2 },
    { 0x19bb530d, SUC_NE_lexicon_tags_2 },
    { 0xfb41530f, SUC_NE_lexicon_tags_2 },
    { 0x410f530f, SUC_NE_lexicon_tags_2 },
    { 0xa0f75312, SUC_NE_lexicon_tags_2 },
    { 0x4a455313, SUC_NE_lexicon_tags_53 },
    { 0, NULL },
    { 0x959f5315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5d531a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2465531d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb77b531f, SUC_NE_lexicon_tags_2 },
    { 0xd0b75320, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x11635322, SUC_NE_lexicon_tags_2 },
    { 0x956b5323, SUC_NE_lexicon_tags_0 },
    { 0x863d5324, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0x90615326, SUC_NE_lexicon_tags_3 },
    { 0x19e55326, SUC_NE_lexicon_tags_2 },
    { 0x56815326, SUC_NE_lexicon_tags_3 },
    { 0x581f5327, SUC_NE_lexicon_tags_2 },
    { 0xb61d5327, SUC_NE_lexicon_tags_2 },
    { 0x9c37532b, SUC_NE_lexicon_tags_2 },
    { 0x9b4f532b, SUC_NE_lexicon_tags_2 },
    { 0x3e1b5327, SUC_NE_lexicon_tags_0 },
    { 0x1699532e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9775333, SUC_NE_lexicon_tags_19 },
    { 0xf6715333, SUC_NE_lexicon_tags_4 },
    { 0xeb295333, SUC_NE_lexicon_tags_2 },
    { 0x9a155333, SUC_NE_lexicon_tags_2 },
    { 0x59eb5336, SUC_NE_lexicon_tags_2 },
    { 0xfb35338, SUC_NE_lexicon_tags_2 },
    { 0x72015336, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44115342, SUC_NE_lexicon_tags_2 },
    { 0xf4635343, SUC_NE_lexicon_tags_2 },
    { 0xdf175343, SUC_NE_lexicon_tags_2 },
    { 0xc3a55344, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ef5349, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e59534b, SUC_NE_lexicon_tags_2 },
    { 0x8835534c, SUC_NE_lexicon_tags_2 },
    { 0xd275534b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83f5350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7415352, SUC_NE_lexicon_tags_2 },
    { 0xac835353, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa3535a, SUC_NE_lexicon_tags_17 },
    { 0xda8d535a, SUC_NE_lexicon_tags_2 },
    { 0x2ff9535c, SUC_NE_lexicon_tags_2 },
    { 0xeb2b535c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x976d5360, SUC_NE_lexicon_tags_4 },
    { 0x930d5361, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e35366, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbb15368, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb3536f, SUC_NE_lexicon_tags_2 },
    { 0xbcaf536f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcf5373, SUC_NE_lexicon_tags_2 },
    { 0xabcf5373, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95379, SUC_NE_lexicon_tags_2 },
    { 0x3edf537a, SUC_NE_lexicon_tags_2 },
    { 0x942b537a, SUC_NE_lexicon_tags_2 },
    { 0xe055537c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c9537f, SUC_NE_lexicon_tags_2 },
    { 0x63c75380, SUC_NE_lexicon_tags_2 },
    { 0x74e55381, SUC_NE_lexicon_tags_2 },
    { 0x7c835382, SUC_NE_lexicon_tags_2 },
    { 0xe0175381, SUC_NE_lexicon_tags_2 },
    { 0xc5f75384, SUC_NE_lexicon_tags_2 },
    { 0x5fb15380, SUC_NE_lexicon_tags_2 },
    { 0x31755386, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbf75388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55f5390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588f5394, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7f539a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69d539d, SUC_NE_lexicon_tags_2 },
    { 0xb227539d, SUC_NE_lexicon_tags_2 },
    { 0x8381539d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b6753a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76a553a3, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3953a6, SUC_NE_lexicon_tags_4 },
    { 0x706953a6, SUC_NE_lexicon_tags_4 },
    { 0x335d53a7, SUC_NE_lexicon_tags_0 },
    { 0xa70353a8, SUC_NE_lexicon_tags_2 },
    { 0xc70153aa, SUC_NE_lexicon_tags_2 },
    { 0x7ea753ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e153ae, SUC_NE_lexicon_tags_2 },
    { 0xe30553ae, SUC_NE_lexicon_tags_19 },
    { 0x7f7953b0, SUC_NE_lexicon_tags_2 },
    { 0xefb553b1, SUC_NE_lexicon_tags_2 },
    { 0xb08753b0, SUC_NE_lexicon_tags_184 },
    { 0, NULL },
    { 0xa28153b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf353bd, SUC_NE_lexicon_tags_13 },
    { 0xa82753bd, SUC_NE_lexicon_tags_2 },
    { 0x516753bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7153c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bf53c7, SUC_NE_lexicon_tags_50 },
    { 0x3cd553c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5a953ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e0953cc, SUC_NE_lexicon_tags_2 },
    { 0xa7ab53cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95753d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a153d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0d353d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe8553d7, SUC_NE_lexicon_tags_18 },
    { 0x28b53d8, SUC_NE_lexicon_tags_2 },
    { 0xcf2353d9, SUC_NE_lexicon_tags_399 },
    { 0, NULL },
    { 0, NULL },
    { 0x257953dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ad53df, SUC_NE_lexicon_tags_2 },
    { 0x571353e0, SUC_NE_lexicon_tags_2 },
    { 0xe12753e1, SUC_NE_lexicon_tags_2 },
    { 0x82b153e1, SUC_NE_lexicon_tags_30 },
    { 0xffe353e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73bd53e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d8553e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63553ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2b953ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x323753ee, SUC_NE_lexicon_tags_2 },
    { 0x446953ef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15cf53f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83153f7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cd53fa, SUC_NE_lexicon_tags_4 },
    { 0xc85b53fb, SUC_NE_lexicon_tags_2 },
    { 0x1bf353fb, SUC_NE_lexicon_tags_2 },
    { 0x917d53fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0953ff, SUC_NE_lexicon_tags_32 },
    { 0x153353ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22935403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x478f5406, SUC_NE_lexicon_tags_141 },
    { 0xbbdd5406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d540b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a5540e, SUC_NE_lexicon_tags_7 },
    { 0xaf09540e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b5413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b5d5415, SUC_NE_lexicon_tags_164 },
    { 0xb0f95416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ef5419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x700f541c, SUC_NE_lexicon_tags_2 },
    { 0xc2c7541d, SUC_NE_lexicon_tags_1 },
    { 0x9e4d541c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f935420, SUC_NE_lexicon_tags_2 },
    { 0xc12f5421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66135423, SUC_NE_lexicon_tags_3 },
    { 0x2f8f5424, SUC_NE_lexicon_tags_7 },
    { 0x54c35425, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ae55427, SUC_NE_lexicon_tags_2 },
    { 0x795d5428, SUC_NE_lexicon_tags_2 },
    { 0x3e3d5429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1355431, SUC_NE_lexicon_tags_2 },
    { 0x6b935432, SUC_NE_lexicon_tags_2 },
    { 0x59995433, SUC_NE_lexicon_tags_2 },
    { 0x35895434, SUC_NE_lexicon_tags_2 },
    { 0x78ad5433, SUC_NE_lexicon_tags_2 },
    { 0xcb8b5436, SUC_NE_lexicon_tags_2 },
    { 0x5cfb5437, SUC_NE_lexicon_tags_2 },
    { 0x25d75438, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9af7543a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd59d543c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98515443, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285d5446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d5448, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0xb7a9544a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1923544c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab5450, SUC_NE_lexicon_tags_2 },
    { 0x28fb5450, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96f5453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x383b5455, SUC_NE_lexicon_tags_2 },
    { 0xfa255456, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf1545a, SUC_NE_lexicon_tags_2 },
    { 0x9c6f545a, SUC_NE_lexicon_tags_2 },
    { 0x4ca3545c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad15460, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53775464, SUC_NE_lexicon_tags_2 },
    { 0x13fd5465, SUC_NE_lexicon_tags_2 },
    { 0xf1575464, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe9b35468, SUC_NE_lexicon_tags_2 },
    { 0xd1775469, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaff546b, SUC_NE_lexicon_tags_2 },
    { 0xa6fd546c, SUC_NE_lexicon_tags_2 },
    { 0xb601546d, SUC_NE_lexicon_tags_2 },
    { 0xe091546b, SUC_NE_lexicon_tags_2 },
    { 0x5d29546f, SUC_NE_lexicon_tags_2 },
    { 0x4c495470, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fd5472, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef0b5474, SUC_NE_lexicon_tags_2 },
    { 0x93195475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50715478, SUC_NE_lexicon_tags_2 },
    { 0x34c15479, SUC_NE_lexicon_tags_2 },
    { 0xed2b5479, SUC_NE_lexicon_tags_8 },
    { 0xc9af5478, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd363547d, SUC_NE_lexicon_tags_2 },
    { 0x8425547e, SUC_NE_lexicon_tags_1 },
    { 0x1d95547e, SUC_NE_lexicon_tags_2 },
    { 0xe401547e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d7b5482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dd75484, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19275489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c5548e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0935490, SUC_NE_lexicon_tags_2 },
    { 0xf5795491, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37615497, SUC_NE_lexicon_tags_17 },
    { 0x223d5497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e7549b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cd549f, SUC_NE_lexicon_tags_2 },
    { 0x499954a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55954a2, SUC_NE_lexicon_tags_1 },
    { 0x15a354a3, SUC_NE_lexicon_tags_2 },
    { 0xe77554a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e154a7, SUC_NE_lexicon_tags_3 },
    { 0x5aff54a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957754ac, SUC_NE_lexicon_tags_2 },
    { 0xd2e954ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5b54b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfb54b4, SUC_NE_lexicon_tags_2 },
    { 0x88d754b5, SUC_NE_lexicon_tags_91 },
    { 0x1dab54b6, SUC_NE_lexicon_tags_2 },
    { 0x38a154b7, SUC_NE_lexicon_tags_2 },
    { 0x999154b8, SUC_NE_lexicon_tags_2 },
    { 0xd1ab54b9, SUC_NE_lexicon_tags_2 },
    { 0xc77d54b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x665f54bd, SUC_NE_lexicon_tags_2 },
    { 0x183954be, SUC_NE_lexicon_tags_2 },
    { 0xf9ff54be, SUC_NE_lexicon_tags_2 },
    { 0x837554c0, SUC_NE_lexicon_tags_3 },
    { 0x215754c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f2b54c3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f54c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb54ca, SUC_NE_lexicon_tags_7 },
    { 0x11c754ca, SUC_NE_lexicon_tags_2 },
    { 0x91ad54cb, SUC_NE_lexicon_tags_2 },
    { 0xcc1754ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36f754cf, SUC_NE_lexicon_tags_17 },
    { 0x3ccf54cf, SUC_NE_lexicon_tags_4 },
    { 0x20bb54cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5154d5, SUC_NE_lexicon_tags_2 },
    { 0xe15154d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b954df, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a754e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74f54eb, SUC_NE_lexicon_tags_2 },
    { 0x53cd54eb, SUC_NE_lexicon_tags_2 },
    { 0x402f54ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3b54f0, SUC_NE_lexicon_tags_2 },
    { 0xb7ef54f0, SUC_NE_lexicon_tags_2 },
    { 0xae1754f2, SUC_NE_lexicon_tags_2 },
    { 0xd9bb54f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82d54f6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xca7554f8, SUC_NE_lexicon_tags_2 },
    { 0x571954f9, SUC_NE_lexicon_tags_2 },
    { 0x220154f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd54fd, SUC_NE_lexicon_tags_2 },
    { 0x6bef54fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d975500, SUC_NE_lexicon_tags_4 },
    { 0xb0295501, SUC_NE_lexicon_tags_2 },
    { 0xb55b5502, SUC_NE_lexicon_tags_2 },
    { 0x2d35503, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7955505, SUC_NE_lexicon_tags_2 },
    { 0xb9495506, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x79ff5508, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb081550a, SUC_NE_lexicon_tags_2 },
    { 0xda53550b, SUC_NE_lexicon_tags_2 },
    { 0x2f47550b, SUC_NE_lexicon_tags_2 },
    { 0xd22d550d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb495510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58b75512, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ff5515, SUC_NE_lexicon_tags_2 },
    { 0x29355516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c1551b, SUC_NE_lexicon_tags_2 },
    { 0x34c9551c, SUC_NE_lexicon_tags_163 },
    { 0x43b5551d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc35521, SUC_NE_lexicon_tags_2 },
    { 0xed395521, SUC_NE_lexicon_tags_2 },
    { 0x633b5523, SUC_NE_lexicon_tags_2 },
    { 0xd9ed5523, SUC_NE_lexicon_tags_2 },
    { 0xad215524, SUC_NE_lexicon_tags_2 },
    { 0xe9495526, SUC_NE_lexicon_tags_2 },
    { 0xd0195522, SUC_NE_lexicon_tags_0 },
    { 0xbc275525, SUC_NE_lexicon_tags_2 },
    { 0x62495529, SUC_NE_lexicon_tags_2 },
    { 0x9181552a, SUC_NE_lexicon_tags_2 },
    { 0x97675529, SUC_NE_lexicon_tags_2 },
    { 0xd7fb5523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8f5532, SUC_NE_lexicon_tags_2 },
    { 0xc2535533, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab5537, SUC_NE_lexicon_tags_2 },
    { 0x465b5538, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c7553d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2b9553f, SUC_NE_lexicon_tags_2 },
    { 0xfb25553f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8735542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66315544, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81e15546, SUC_NE_lexicon_tags_7 },
    { 0xae095546, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4d554a, SUC_NE_lexicon_tags_2 },
    { 0xfedd554b, SUC_NE_lexicon_tags_2 },
    { 0xff3f554c, SUC_NE_lexicon_tags_2 },
    { 0x1f5554d, SUC_NE_lexicon_tags_2 },
    { 0xc913554e, SUC_NE_lexicon_tags_2 },
    { 0xf555554e, SUC_NE_lexicon_tags_2 },
    { 0xcba1554f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37f35552, SUC_NE_lexicon_tags_2 },
    { 0xfc515553, SUC_NE_lexicon_tags_2 },
    { 0xb4e15553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30315557, SUC_NE_lexicon_tags_2 },
    { 0x1b915557, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853d555c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c1555f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d315562, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac95567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc69556a, SUC_NE_lexicon_tags_2 },
    { 0xb847556b, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf75556d, SUC_NE_lexicon_tags_2 },
    { 0x1345556e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4555570, SUC_NE_lexicon_tags_3 },
    { 0x980b5571, SUC_NE_lexicon_tags_2 },
    { 0x2b0b5572, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cd5578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22c1557a, SUC_NE_lexicon_tags_8 },
    { 0x2ec5557a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f557e, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x569b5581, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ed5584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad55588, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a3558c, SUC_NE_lexicon_tags_2 },
    { 0x28a5558c, SUC_NE_lexicon_tags_2 },
    { 0x70d3558e, SUC_NE_lexicon_tags_2 },
    { 0x48cd558f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf5593, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521f5597, SUC_NE_lexicon_tags_2 },
    { 0x2e5d5598, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x23b7559a, SUC_NE_lexicon_tags_46 },
    { 0x9689559b, SUC_NE_lexicon_tags_2 },
    { 0xace5559a, SUC_NE_lexicon_tags_2 },
    { 0xe823559a, SUC_NE_lexicon_tags_2 },
    { 0x34c1559b, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0xe72f55a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x643555a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f155a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356555aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x895755ac, SUC_NE_lexicon_tags_2 },
    { 0xd68755ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x186d55b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e6d55b2, SUC_NE_lexicon_tags_2 },
    { 0xa50b55b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd6755b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ae355b7, SUC_NE_lexicon_tags_2 },
    { 0x760955b7, SUC_NE_lexicon_tags_2 },
    { 0xe49355b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14f55bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1b955be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee155c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2b55c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x667f55c5, SUC_NE_lexicon_tags_2 },
    { 0x137b55c6, SUC_NE_lexicon_tags_4 },
    { 0x580155c7, SUC_NE_lexicon_tags_18 },
    { 0x527355c8, SUC_NE_lexicon_tags_2 },
    { 0xea1755c8, SUC_NE_lexicon_tags_2 },
    { 0x41d155c5, SUC_NE_lexicon_tags_2 },
    { 0x454555c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83fd55cd, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82155d0, SUC_NE_lexicon_tags_2 },
    { 0xe12d55d1, SUC_NE_lexicon_tags_2 },
    { 0x9cfd55d2, SUC_NE_lexicon_tags_2 },
    { 0xc02955d0, SUC_NE_lexicon_tags_2 },
    { 0x1c7955d4, SUC_NE_lexicon_tags_2 },
    { 0x4e6d55d5, SUC_NE_lexicon_tags_4 },
    { 0x21755d6, SUC_NE_lexicon_tags_2 },
    { 0x5d9955d6, SUC_NE_lexicon_tags_24 },
    { 0xbe4355d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc88b55da, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xada955dd, SUC_NE_lexicon_tags_2 },
    { 0x96e955de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb55e8, SUC_NE_lexicon_tags_2 },
    { 0x5dbd55e8, SUC_NE_lexicon_tags_2 },
    { 0x1f2155ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e1755ec, SUC_NE_lexicon_tags_2 },
    { 0x752755ec, SUC_NE_lexicon_tags_2 },
    { 0xfcd955ee, SUC_NE_lexicon_tags_2 },
    { 0xbfa755ee, SUC_NE_lexicon_tags_2 },
    { 0xa78d55ee, SUC_NE_lexicon_tags_2 },
    { 0xa88155ed, SUC_NE_lexicon_tags_2 },
    { 0x507d55f2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfb3f55f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82a355f6, SUC_NE_lexicon_tags_2 },
    { 0xeeb155f7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37555600, SUC_NE_lexicon_tags_2 },
    { 0x4c495600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa06b5603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9155605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbd560a, SUC_NE_lexicon_tags_2 },
    { 0x6c9f560a, SUC_NE_lexicon_tags_4 },
    { 0xf367560b, SUC_NE_lexicon_tags_2 },
    { 0xdc85560b, SUC_NE_lexicon_tags_2 },
    { 0x8bc1560e, SUC_NE_lexicon_tags_2 },
    { 0x538d560f, SUC_NE_lexicon_tags_2 },
    { 0xf6ed5610, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6395614, SUC_NE_lexicon_tags_2 },
    { 0xdb535614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x966f5617, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b55621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79b5624, SUC_NE_lexicon_tags_2 },
    { 0x99b55624, SUC_NE_lexicon_tags_4 },
    { 0xede15625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28455628, SUC_NE_lexicon_tags_2 },
    { 0xb22d5629, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03d5631, SUC_NE_lexicon_tags_2 },
    { 0x51755632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd915634, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x131b5636, SUC_NE_lexicon_tags_2 },
    { 0x5f095636, SUC_NE_lexicon_tags_8 },
    { 0x97155636, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa693563b, SUC_NE_lexicon_tags_2 },
    { 0x6023563b, SUC_NE_lexicon_tags_2 },
    { 0xa4a1563d, SUC_NE_lexicon_tags_2 },
    { 0xb25f563e, SUC_NE_lexicon_tags_2 },
    { 0xdec5563f, SUC_NE_lexicon_tags_2 },
    { 0xa337563f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbf5644, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41555646, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf15648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a7564b, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f9564f, SUC_NE_lexicon_tags_2 },
    { 0x5d475650, SUC_NE_lexicon_tags_2 },
    { 0x162d5650, SUC_NE_lexicon_tags_4 },
    { 0x41c9564f, SUC_NE_lexicon_tags_2 },
    { 0x1549564f, SUC_NE_lexicon_tags_2 },
    { 0xd53d5654, SUC_NE_lexicon_tags_2 },
    { 0x7ff9564f, SUC_NE_lexicon_tags_2 },
    { 0x894b5656, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1565a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48f5660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b35662, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b775664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff995677, SUC_NE_lexicon_tags_2 },
    { 0xb4a55678, SUC_NE_lexicon_tags_2 },
    { 0x81255679, SUC_NE_lexicon_tags_0 },
    { 0xb795567a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fb5682, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x36c15684, SUC_NE_lexicon_tags_2 },
    { 0xdd035685, SUC_NE_lexicon_tags_2 },
    { 0xd5a15686, SUC_NE_lexicon_tags_2 },
    { 0xda4d5687, SUC_NE_lexicon_tags_2 },
    { 0xab215688, SUC_NE_lexicon_tags_59 },
    { 0x59f55688, SUC_NE_lexicon_tags_2 },
    { 0xa8495687, SUC_NE_lexicon_tags_2 },
    { 0xeca55684, SUC_NE_lexicon_tags_4 },
    { 0xde37568c, SUC_NE_lexicon_tags_407 },
    { 0, NULL },
    { 0xa28b568e, SUC_NE_lexicon_tags_2 },
    { 0x1d83568e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200956a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a556a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbed56ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588d56af, SUC_NE_lexicon_tags_2 },
    { 0x832156af, SUC_NE_lexicon_tags_51 },
    { 0x119f56b1, SUC_NE_lexicon_tags_2 },
    { 0x6b6756b2, SUC_NE_lexicon_tags_2 },
    { 0x2ae356b3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad56b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4df56c0, SUC_NE_lexicon_tags_1 },
    { 0xe97f56c1, SUC_NE_lexicon_tags_2 },
    { 0x851556c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec6356c4, SUC_NE_lexicon_tags_2 },
    { 0xf1356c4, SUC_NE_lexicon_tags_0 },
    { 0x792756c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f956cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bc956cd, SUC_NE_lexicon_tags_17 },
    { 0xb4356ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81d56d0, SUC_NE_lexicon_tags_4 },
    { 0x955b56d0, SUC_NE_lexicon_tags_2 },
    { 0xa5f56d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc956d9, SUC_NE_lexicon_tags_0 },
    { 0x5ecf56da, SUC_NE_lexicon_tags_2 },
    { 0x9e4356d9, SUC_NE_lexicon_tags_2 },
    { 0x9c0f56dc, SUC_NE_lexicon_tags_2 },
    { 0xf50556db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563556e2, SUC_NE_lexicon_tags_2 },
    { 0x503556e2, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x841b56e5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294356eb, SUC_NE_lexicon_tags_2 },
    { 0x817b56ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d56f2, SUC_NE_lexicon_tags_2 },
    { 0xafe156f3, SUC_NE_lexicon_tags_2 },
    { 0x66f156f3, SUC_NE_lexicon_tags_2 },
    { 0xebd956f3, SUC_NE_lexicon_tags_2 },
    { 0xeb9f56f6, SUC_NE_lexicon_tags_40 },
    { 0x6ce556f5, SUC_NE_lexicon_tags_2 },
    { 0x385156f4, SUC_NE_lexicon_tags_2 },
    { 0x79b356f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa4556fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc84156fd, SUC_NE_lexicon_tags_2 },
    { 0xb34756fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c15701, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a7b5703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe15b5705, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2775707, SUC_NE_lexicon_tags_2 },
    { 0x72635707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3783570d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5f5710, SUC_NE_lexicon_tags_2 },
    { 0x94ad5710, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30535715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba5b5717, SUC_NE_lexicon_tags_58 },
    { 0x70fd5717, SUC_NE_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a93571d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d55721, SUC_NE_lexicon_tags_2 },
    { 0xceaf5722, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89695725, SUC_NE_lexicon_tags_4 },
    { 0x7fd35726, SUC_NE_lexicon_tags_2 },
    { 0xb955727, SUC_NE_lexicon_tags_2 },
    { 0xdb175727, SUC_NE_lexicon_tags_2 },
    { 0x16615729, SUC_NE_lexicon_tags_2 },
    { 0x3f5b572a, SUC_NE_lexicon_tags_2 },
    { 0x33a1572b, SUC_NE_lexicon_tags_2 },
    { 0x491f572c, SUC_NE_lexicon_tags_2 },
    { 0x5943572c, SUC_NE_lexicon_tags_2 },
    { 0x7031572a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a55732, SUC_NE_lexicon_tags_3 },
    { 0x4cc35732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fb55735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad1d5737, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d573b, SUC_NE_lexicon_tags_2 },
    { 0x8a7d573c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac3573f, SUC_NE_lexicon_tags_2 },
    { 0x2495740, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23315746, SUC_NE_lexicon_tags_2 },
    { 0x3c015747, SUC_NE_lexicon_tags_3 },
    { 0x740b5746, SUC_NE_lexicon_tags_2 },
    { 0xdeed5749, SUC_NE_lexicon_tags_2 },
    { 0xa25d574a, SUC_NE_lexicon_tags_4 },
    { 0xa623574b, SUC_NE_lexicon_tags_2 },
    { 0xe8bf5746, SUC_NE_lexicon_tags_164 },
    { 0x4ddb574d, SUC_NE_lexicon_tags_2 },
    { 0x7ab55748, SUC_NE_lexicon_tags_2 },
    { 0x88f5574f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1f5752, SUC_NE_lexicon_tags_2 },
    { 0xbb935753, SUC_NE_lexicon_tags_2 },
    { 0x84595754, SUC_NE_lexicon_tags_2 },
    { 0xe535755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ccb5757, SUC_NE_lexicon_tags_2 },
    { 0xe715758, SUC_NE_lexicon_tags_2 },
    { 0x38895757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d25575f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0d5765, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x441d5768, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dd576c, SUC_NE_lexicon_tags_2 },
    { 0x641d576c, SUC_NE_lexicon_tags_2 },
    { 0xa191576e, SUC_NE_lexicon_tags_2 },
    { 0x6e7b576e, SUC_NE_lexicon_tags_2 },
    { 0x899f5770, SUC_NE_lexicon_tags_2 },
    { 0x78415771, SUC_NE_lexicon_tags_2 },
    { 0x390d5771, SUC_NE_lexicon_tags_2 },
    { 0xa83d5773, SUC_NE_lexicon_tags_2 },
    { 0xbbf95773, SUC_NE_lexicon_tags_2 },
    { 0xf6c15775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7d5778, SUC_NE_lexicon_tags_2 },
    { 0xb5b5778, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806d5781, SUC_NE_lexicon_tags_2 },
    { 0x401f5781, SUC_NE_lexicon_tags_2 },
    { 0x2855783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f8b5785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf16d5787, SUC_NE_lexicon_tags_2 },
    { 0xdc2f5788, SUC_NE_lexicon_tags_2 },
    { 0x21fb5787, SUC_NE_lexicon_tags_2 },
    { 0x80415787, SUC_NE_lexicon_tags_2 },
    { 0xf135578b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cd578e, SUC_NE_lexicon_tags_2 },
    { 0xb42f578e, SUC_NE_lexicon_tags_2 },
    { 0x119f578f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1d95792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x591b5794, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0895796, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642157a1, SUC_NE_lexicon_tags_4 },
    { 0xd45b57a2, SUC_NE_lexicon_tags_2 },
    { 0x1d7757a3, SUC_NE_lexicon_tags_20 },
    { 0xd09757a4, SUC_NE_lexicon_tags_49 },
    { 0xe70757a5, SUC_NE_lexicon_tags_2 },
    { 0x70557a4, SUC_NE_lexicon_tags_2 },
    { 0x65db57a7, SUC_NE_lexicon_tags_2 },
    { 0x9ba757a8, SUC_NE_lexicon_tags_2 },
    { 0x846f57a8, SUC_NE_lexicon_tags_2 },
    { 0x811f57a8, SUC_NE_lexicon_tags_14 },
    { 0xddc757a9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131757b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43557b8, SUC_NE_lexicon_tags_2 },
    { 0x486b57b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0c157bb, SUC_NE_lexicon_tags_30 },
    { 0xb0c757bb, SUC_NE_lexicon_tags_2 },
    { 0x367b57bb, SUC_NE_lexicon_tags_2 },
    { 0xede357be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20557c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa9357c3, SUC_NE_lexicon_tags_2 },
    { 0x11a557c4, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x521d57c6, SUC_NE_lexicon_tags_2 },
    { 0xa0d57c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccf57c9, SUC_NE_lexicon_tags_2 },
    { 0xd34557ca, SUC_NE_lexicon_tags_2 },
    { 0x640157cb, SUC_NE_lexicon_tags_2 },
    { 0xb06b57cb, SUC_NE_lexicon_tags_2 },
    { 0xbdb557cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ad57d3, SUC_NE_lexicon_tags_2 },
    { 0xfb9357d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635b57d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c9357db, SUC_NE_lexicon_tags_2 },
    { 0xd3ff57dc, SUC_NE_lexicon_tags_2 },
    { 0xf1c957dd, SUC_NE_lexicon_tags_375 },
    { 0x397357de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966b57e3, SUC_NE_lexicon_tags_2 },
    { 0xf9a957e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715f57e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99557e9, SUC_NE_lexicon_tags_2 },
    { 0x638357e9, SUC_NE_lexicon_tags_2 },
    { 0xcbd757ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bfb57ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x620557f0, SUC_NE_lexicon_tags_2 },
    { 0x6d8557f1, SUC_NE_lexicon_tags_2 },
    { 0x718f57f0, SUC_NE_lexicon_tags_3 },
    { 0xd76357f3, SUC_NE_lexicon_tags_2 },
    { 0x58757f4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14957fb, SUC_NE_lexicon_tags_2 },
    { 0xb3e157fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4557ff, SUC_NE_lexicon_tags_3 },
    { 0xd7557ff, SUC_NE_lexicon_tags_2 },
    { 0x161b5800, SUC_NE_lexicon_tags_2 },
    { 0xd92b5802, SUC_NE_lexicon_tags_2 },
    { 0x56b35801, SUC_NE_lexicon_tags_2 },
    { 0x81955801, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4739580a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4d580d, SUC_NE_lexicon_tags_2 },
    { 0x72e9580e, SUC_NE_lexicon_tags_2 },
    { 0xb299580d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3f5812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2ff5814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3b581a, SUC_NE_lexicon_tags_2 },
    { 0x5b27581b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48c5581d, SUC_NE_lexicon_tags_2 },
    { 0x92e3581e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc15821, SUC_NE_lexicon_tags_2 },
    { 0xa3c95822, SUC_NE_lexicon_tags_2 },
    { 0x83b95823, SUC_NE_lexicon_tags_2 },
    { 0x507f5824, SUC_NE_lexicon_tags_2 },
    { 0x37255823, SUC_NE_lexicon_tags_2 },
    { 0xacdd5826, SUC_NE_lexicon_tags_2 },
    { 0x9ead5823, SUC_NE_lexicon_tags_2 },
    { 0x32135826, SUC_NE_lexicon_tags_2 },
    { 0xbe535829, SUC_NE_lexicon_tags_2 },
    { 0x1bdf5824, SUC_NE_lexicon_tags_32 },
    { 0x900d582b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf925582f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3a75831, SUC_NE_lexicon_tags_2 },
    { 0x32c35832, SUC_NE_lexicon_tags_7 },
    { 0x7e1f5831, SUC_NE_lexicon_tags_2 },
    { 0x50975834, SUC_NE_lexicon_tags_17 },
    { 0xe0b55835, SUC_NE_lexicon_tags_2 },
    { 0x32bb5836, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x26bd5838, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a1583c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4543583e, SUC_NE_lexicon_tags_5 },
    { 0xcf55583f, SUC_NE_lexicon_tags_2 },
    { 0x97f1583e, SUC_NE_lexicon_tags_5 },
    { 0x7ae55841, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d5844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e755847, SUC_NE_lexicon_tags_2 },
    { 0x57bd5847, SUC_NE_lexicon_tags_2 },
    { 0xe2b75849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b5584c, SUC_NE_lexicon_tags_2 },
    { 0xe033584c, SUC_NE_lexicon_tags_2 },
    { 0x4983584e, SUC_NE_lexicon_tags_2 },
    { 0xfc37584f, SUC_NE_lexicon_tags_2 },
    { 0x2bef5850, SUC_NE_lexicon_tags_2 },
    { 0x51d55851, SUC_NE_lexicon_tags_2 },
    { 0x93e1584c, SUC_NE_lexicon_tags_2 },
    { 0xdb7d5853, SUC_NE_lexicon_tags_2 },
    { 0xf5215854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddf5858, SUC_NE_lexicon_tags_2 },
    { 0xa3ed5859, SUC_NE_lexicon_tags_2 },
    { 0x6115585a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7f585d, SUC_NE_lexicon_tags_2 },
    { 0x7edf585d, SUC_NE_lexicon_tags_2 },
    { 0xb25585f, SUC_NE_lexicon_tags_2 },
    { 0xe305585d, SUC_NE_lexicon_tags_2 },
    { 0xdf0d5861, SUC_NE_lexicon_tags_7 },
    { 0xf6135862, SUC_NE_lexicon_tags_2 },
    { 0x53cf5863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bb15865, SUC_NE_lexicon_tags_2 },
    { 0xca915866, SUC_NE_lexicon_tags_2 },
    { 0x3c1f5867, SUC_NE_lexicon_tags_2 },
    { 0x2b7d5868, SUC_NE_lexicon_tags_2 },
    { 0x2af75869, SUC_NE_lexicon_tags_7 },
    { 0xfd475866, SUC_NE_lexicon_tags_2 },
    { 0xdc35867, SUC_NE_lexicon_tags_3 },
    { 0x64ad586a, SUC_NE_lexicon_tags_2 },
    { 0x2d795866, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd35870, SUC_NE_lexicon_tags_2 },
    { 0x32fd5870, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9d5875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37a15877, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x96a15879, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1597587b, SUC_NE_lexicon_tags_2 },
    { 0xd023587b, SUC_NE_lexicon_tags_2 },
    { 0xd6c5587d, SUC_NE_lexicon_tags_2 },
    { 0x11b1587e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x13835880, SUC_NE_lexicon_tags_5 },
    { 0x70715880, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3215884, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd5887, SUC_NE_lexicon_tags_2 },
    { 0x8f975888, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5588d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf311588f, SUC_NE_lexicon_tags_2 },
    { 0xd1d3588f, SUC_NE_lexicon_tags_2 },
    { 0xbdf75891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65795895, SUC_NE_lexicon_tags_7 },
    { 0x6c3d5896, SUC_NE_lexicon_tags_2 },
    { 0xdad35895, SUC_NE_lexicon_tags_2 },
    { 0x34435898, SUC_NE_lexicon_tags_2 },
    { 0xc16d5899, SUC_NE_lexicon_tags_2 },
    { 0x503d5895, SUC_NE_lexicon_tags_17 },
    { 0xc213589b, SUC_NE_lexicon_tags_2 },
    { 0xdf3f589c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x668d589e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473358a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda7158a4, SUC_NE_lexicon_tags_4 },
    { 0xd14b58a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba0958a7, SUC_NE_lexicon_tags_2 },
    { 0xcd7758a8, SUC_NE_lexicon_tags_2 },
    { 0xdc8f58a7, SUC_NE_lexicon_tags_2 },
    { 0x199f58aa, SUC_NE_lexicon_tags_2 },
    { 0x1f8958a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x123d58ae, SUC_NE_lexicon_tags_2 },
    { 0x1a2958af, SUC_NE_lexicon_tags_2 },
    { 0xa3c158af, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x860358b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x624b58b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3158b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70f58be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e758c2, SUC_NE_lexicon_tags_2 },
    { 0xa47758c3, SUC_NE_lexicon_tags_2 },
    { 0xca0558c4, SUC_NE_lexicon_tags_2 },
    { 0x651158c5, SUC_NE_lexicon_tags_2 },
    { 0x514958c6, SUC_NE_lexicon_tags_2 },
    { 0x91c958c7, SUC_NE_lexicon_tags_2 },
    { 0x1d7758c7, SUC_NE_lexicon_tags_2 },
    { 0x59f558c2, SUC_NE_lexicon_tags_2 },
    { 0x9bb158ca, SUC_NE_lexicon_tags_2 },
    { 0x9f3158cb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x956158ce, SUC_NE_lexicon_tags_2 },
    { 0x1ebd58cf, SUC_NE_lexicon_tags_11 },
    { 0x8d6158d0, SUC_NE_lexicon_tags_2 },
    { 0xe9558d1, SUC_NE_lexicon_tags_2 },
    { 0xa63f58d2, SUC_NE_lexicon_tags_2 },
    { 0x65cd58d3, SUC_NE_lexicon_tags_3 },
    { 0xde8f58ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff58d7, SUC_NE_lexicon_tags_2 },
    { 0x45b958d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4f58db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e958dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x318b58e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1958e2, SUC_NE_lexicon_tags_2 },
    { 0xc3bf58e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eb58e7, SUC_NE_lexicon_tags_2 },
    { 0x1f0358e8, SUC_NE_lexicon_tags_2 },
    { 0xfd1758e9, SUC_NE_lexicon_tags_2 },
    { 0x9a4958e9, SUC_NE_lexicon_tags_2 },
    { 0xfe3758eb, SUC_NE_lexicon_tags_4 },
    { 0x94d958ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04758ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x804d58f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e358f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d2f58f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa30758fb, SUC_NE_lexicon_tags_2 },
    { 0x330d58fc, SUC_NE_lexicon_tags_2 },
    { 0x6c9358fc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd19b58ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17835903, SUC_NE_lexicon_tags_1 },
    { 0x6bd15904, SUC_NE_lexicon_tags_2 },
    { 0x8d935905, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a35908, SUC_NE_lexicon_tags_2 },
    { 0xf8855908, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca69590c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda95910, SUC_NE_lexicon_tags_2 },
    { 0xd8cb5911, SUC_NE_lexicon_tags_2 },
    { 0x26775911, SUC_NE_lexicon_tags_2 },
    { 0x56db5913, SUC_NE_lexicon_tags_3 },
    { 0xe15d5914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ad95916, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c3591a, SUC_NE_lexicon_tags_2 },
    { 0xb1bd591b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6459591d, SUC_NE_lexicon_tags_2 },
    { 0x2a7b591e, SUC_NE_lexicon_tags_2 },
    { 0x7235591f, SUC_NE_lexicon_tags_2 },
    { 0x424b5920, SUC_NE_lexicon_tags_2 },
    { 0x2f8d5920, SUC_NE_lexicon_tags_2 },
    { 0x817f5922, SUC_NE_lexicon_tags_2 },
    { 0xe60b5923, SUC_NE_lexicon_tags_2 },
    { 0xea875920, SUC_NE_lexicon_tags_3 },
    { 0xbdfd591e, SUC_NE_lexicon_tags_2 },
    { 0x9975926, SUC_NE_lexicon_tags_2 },
    { 0x6de95926, SUC_NE_lexicon_tags_295 },
    { 0xf0015922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fb592d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58095932, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a95938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741f593c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d05593e, SUC_NE_lexicon_tags_2 },
    { 0x7767593e, SUC_NE_lexicon_tags_2 },
    { 0xa5af593e, SUC_NE_lexicon_tags_2 },
    { 0x51b55941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15735943, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99f594d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbda3594f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb235952, SUC_NE_lexicon_tags_2 },
    { 0x309d5953, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f35956, SUC_NE_lexicon_tags_2 },
    { 0x50775956, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8741595b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5103595d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc15962, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533f596c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8273596f, SUC_NE_lexicon_tags_2 },
    { 0x8e795970, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b9b5972, SUC_NE_lexicon_tags_3 },
    { 0x86d75972, SUC_NE_lexicon_tags_2 },
    { 0x152d5974, SUC_NE_lexicon_tags_2 },
    { 0xf0f35973, SUC_NE_lexicon_tags_2 },
    { 0x4155976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23595979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861b597d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x415d5980, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72555983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5b5986, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc955989, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6623598c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5615991, SUC_NE_lexicon_tags_2 },
    { 0xd7db5991, SUC_NE_lexicon_tags_2 },
    { 0xaa4b5991, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3215995, SUC_NE_lexicon_tags_2 },
    { 0x13475996, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8211599a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa637599f, SUC_NE_lexicon_tags_2 },
    { 0xb80159a0, SUC_NE_lexicon_tags_2 },
    { 0xf1f559a0, SUC_NE_lexicon_tags_2 },
    { 0x65d59a2, SUC_NE_lexicon_tags_2 },
    { 0x722359a3, SUC_NE_lexicon_tags_2 },
    { 0x266559a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c959a7, SUC_NE_lexicon_tags_19 },
    { 0x2e1159a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x180f59ab, SUC_NE_lexicon_tags_2 },
    { 0x4f7759ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x276d59ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a159b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca8759b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3559b7, SUC_NE_lexicon_tags_2 },
    { 0x14e959b7, SUC_NE_lexicon_tags_2 },
    { 0x5f7759b8, SUC_NE_lexicon_tags_2 },
    { 0x2c8159ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f3559bc, SUC_NE_lexicon_tags_2 },
    { 0xf65159bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858559c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d359c8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x76a359ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7b59cd, SUC_NE_lexicon_tags_17 },
    { 0x720159ce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93759d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45f959d9, SUC_NE_lexicon_tags_2 },
    { 0x40d59da, SUC_NE_lexicon_tags_2 },
    { 0x9e8759db, SUC_NE_lexicon_tags_2 },
    { 0xec159da, SUC_NE_lexicon_tags_2 },
    { 0xd94559dd, SUC_NE_lexicon_tags_19 },
    { 0xff0359db, SUC_NE_lexicon_tags_2 },
    { 0x55159da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24959e3, SUC_NE_lexicon_tags_2 },
    { 0xddbb59e4, SUC_NE_lexicon_tags_2 },
    { 0xf38559e4, SUC_NE_lexicon_tags_2 },
    { 0xa6a559e6, SUC_NE_lexicon_tags_2 },
    { 0x324f59e7, SUC_NE_lexicon_tags_2 },
    { 0xfbf159e8, SUC_NE_lexicon_tags_2 },
    { 0x203b59e7, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc1159eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1b59ee, SUC_NE_lexicon_tags_2 },
    { 0x17e759ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a359f4, SUC_NE_lexicon_tags_2 },
    { 0x132759f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8559fb, SUC_NE_lexicon_tags_2 },
    { 0x8e0159fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29f959fe, SUC_NE_lexicon_tags_2 },
    { 0x832b59ff, SUC_NE_lexicon_tags_2 },
    { 0x4959fe, SUC_NE_lexicon_tags_14 },
    { 0xfa055a01, SUC_NE_lexicon_tags_2 },
    { 0xb62b5a02, SUC_NE_lexicon_tags_2 },
    { 0x7d995a00, SUC_NE_lexicon_tags_176 },
    { 0xbbf359ff, SUC_NE_lexicon_tags_4 },
    { 0x6ab159fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf5a0a, SUC_NE_lexicon_tags_2 },
    { 0x1a9b5a0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34315a0f, SUC_NE_lexicon_tags_2 },
    { 0xdb175a10, SUC_NE_lexicon_tags_2 },
    { 0xb5ff5a11, SUC_NE_lexicon_tags_2 },
    { 0x3f695a12, SUC_NE_lexicon_tags_2 },
    { 0x81e55a13, SUC_NE_lexicon_tags_2 },
    { 0xddbb5a10, SUC_NE_lexicon_tags_2 },
    { 0xd7f15a11, SUC_NE_lexicon_tags_2 },
    { 0xcdf55a14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5d5a1a, SUC_NE_lexicon_tags_2 },
    { 0x38f95a1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d555a1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d655a25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5db5a2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ff5a2f, SUC_NE_lexicon_tags_2 },
    { 0x1ab55a30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e0f5a32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee635a35, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80215a3c, SUC_NE_lexicon_tags_2 },
    { 0x27ad5a3d, SUC_NE_lexicon_tags_2 },
    { 0xd2e95a3e, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196f5a42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28355a46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edb5a4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b15a52, SUC_NE_lexicon_tags_2 },
    { 0x53a75a53, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3555a56, SUC_NE_lexicon_tags_2 },
    { 0x4f315a57, SUC_NE_lexicon_tags_2 },
    { 0x46b75a58, SUC_NE_lexicon_tags_2 },
    { 0xb1015a57, SUC_NE_lexicon_tags_2 },
    { 0x77015a56, SUC_NE_lexicon_tags_2 },
    { 0x8bb35a56, SUC_NE_lexicon_tags_2 },
    { 0x3a915a5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a515a5f, SUC_NE_lexicon_tags_2 },
    { 0x4415a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbef5a62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d35a65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b5a69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaf5a71, SUC_NE_lexicon_tags_2 },
    { 0x3e275a71, SUC_NE_lexicon_tags_2 },
    { 0x9d795a73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22195a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d75a79, SUC_NE_lexicon_tags_2 },
    { 0xa255a7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1855a7f, SUC_NE_lexicon_tags_2 },
    { 0x5ad5a7f, SUC_NE_lexicon_tags_2 },
    { 0xe9f95a7f, SUC_NE_lexicon_tags_24 },
    { 0xea535a82, SUC_NE_lexicon_tags_2 },
    { 0xc2035a83, SUC_NE_lexicon_tags_2 },
    { 0x70015a81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcacf5a86, SUC_NE_lexicon_tags_2 },
    { 0x2c675a86, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x784d5a8a, SUC_NE_lexicon_tags_2 },
    { 0x4e035a8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbee95a8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e55a92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcde35a94, SUC_NE_lexicon_tags_2 },
    { 0xf7c35a95, SUC_NE_lexicon_tags_2 },
    { 0x99435a96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9e55a98, SUC_NE_lexicon_tags_2 },
    { 0x6ff75a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x306b5a9b, SUC_NE_lexicon_tags_2 },
    { 0x9e7d5a9c, SUC_NE_lexicon_tags_2 },
    { 0x4b175a9b, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dad5aa2, SUC_NE_lexicon_tags_2 },
    { 0xd1a55aa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa75aa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494f5aad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc15ab6, SUC_NE_lexicon_tags_2 },
    { 0x271d5ab7, SUC_NE_lexicon_tags_2 },
    { 0xbffd5ab8, SUC_NE_lexicon_tags_7 },
    { 0xe835ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3315ac1, SUC_NE_lexicon_tags_2 },
    { 0x935b5ac2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2715ac8, SUC_NE_lexicon_tags_2 },
    { 0xd9435ac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7ef5acb, SUC_NE_lexicon_tags_1 },
    { 0x57975acb, SUC_NE_lexicon_tags_2 },
    { 0xeadd5acc, SUC_NE_lexicon_tags_2 },
    { 0xb9095ace, SUC_NE_lexicon_tags_2 },
    { 0x50615acf, SUC_NE_lexicon_tags_2 },
    { 0x5e415ad0, SUC_NE_lexicon_tags_2 },
    { 0xf7d75ad1, SUC_NE_lexicon_tags_2 },
    { 0xac515ad1, SUC_NE_lexicon_tags_2 },
    { 0x97455ace, SUC_NE_lexicon_tags_2 },
    { 0xf9f55ad2, SUC_NE_lexicon_tags_50 },
    { 0x84275ad1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e5f5ad7, SUC_NE_lexicon_tags_2 },
    { 0xad515ad7, SUC_NE_lexicon_tags_2 },
    { 0xb8075ad9, SUC_NE_lexicon_tags_2 },
    { 0x64f95ad7, SUC_NE_lexicon_tags_2 },
    { 0x4ef35adb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe555ae0, SUC_NE_lexicon_tags_2 },
    { 0x4b3d5ae1, SUC_NE_lexicon_tags_2 },
    { 0xb2c95ae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb555ae9, SUC_NE_lexicon_tags_2 },
    { 0xe6b35ae9, SUC_NE_lexicon_tags_2 },
    { 0x72955aeb, SUC_NE_lexicon_tags_2 },
    { 0xead5aec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fd5af0, SUC_NE_lexicon_tags_2 },
    { 0x7d4d5af1, SUC_NE_lexicon_tags_2 },
    { 0xfa775af1, SUC_NE_lexicon_tags_2 },
    { 0x9b4d5af3, SUC_NE_lexicon_tags_2 },
    { 0x18fd5af4, SUC_NE_lexicon_tags_2 },
    { 0x28c95af5, SUC_NE_lexicon_tags_2 },
    { 0x43e35af0, SUC_NE_lexicon_tags_2 },
    { 0xce15af7, SUC_NE_lexicon_tags_2 },
    { 0x35795af3, SUC_NE_lexicon_tags_2 },
    { 0x1fa55af4, SUC_NE_lexicon_tags_2 },
    { 0xa6f95af0, SUC_NE_lexicon_tags_2 },
    { 0x87755afb, SUC_NE_lexicon_tags_2 },
    { 0x22435afc, SUC_NE_lexicon_tags_2 },
    { 0x40435af2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1db5b00, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4755b03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8295b07, SUC_NE_lexicon_tags_2 },
    { 0x56195b08, SUC_NE_lexicon_tags_3 },
    { 0xdb35b08, SUC_NE_lexicon_tags_2 },
    { 0x97335b09, SUC_NE_lexicon_tags_2 },
    { 0x3ea55b0b, SUC_NE_lexicon_tags_2 },
    { 0x15e15b08, SUC_NE_lexicon_tags_2 },
    { 0xe3b5b0d, SUC_NE_lexicon_tags_2 },
    { 0xd7bb5b0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb75b13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5195b15, SUC_NE_lexicon_tags_2 },
    { 0x53dd5b16, SUC_NE_lexicon_tags_2 },
    { 0x40775b16, SUC_NE_lexicon_tags_2 },
    { 0xc4315b18, SUC_NE_lexicon_tags_2 },
    { 0xb3db5b18, SUC_NE_lexicon_tags_2 },
    { 0xd35b5b1a, SUC_NE_lexicon_tags_2 },
    { 0x95855b1a, SUC_NE_lexicon_tags_2 },
    { 0x51e55b1c, SUC_NE_lexicon_tags_2 },
    { 0x79995b1d, SUC_NE_lexicon_tags_2 },
    { 0x76095b1d, SUC_NE_lexicon_tags_2 },
    { 0xb9e95b1f, SUC_NE_lexicon_tags_2 },
    { 0xadb15b1f, SUC_NE_lexicon_tags_2 },
    { 0x39695b21, SUC_NE_lexicon_tags_2 },
    { 0xd7d15b21, SUC_NE_lexicon_tags_2 },
    { 0x3b4d5b22, SUC_NE_lexicon_tags_2 },
    { 0xee8f5b24, SUC_NE_lexicon_tags_2 },
    { 0x58035b24, SUC_NE_lexicon_tags_2 },
    { 0x2be55b26, SUC_NE_lexicon_tags_101 },
    { 0x35695b27, SUC_NE_lexicon_tags_2 },
    { 0xb9d5b28, SUC_NE_lexicon_tags_2 },
    { 0x77d15b25, SUC_NE_lexicon_tags_2 },
    { 0xb60f5b2a, SUC_NE_lexicon_tags_4 },
    { 0xebed5b2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c275b2d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x497d5b30, SUC_NE_lexicon_tags_5 },
    { 0x74775b31, SUC_NE_lexicon_tags_4 },
    { 0x17095b30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd95b35, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d35b38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72375b3c, SUC_NE_lexicon_tags_3 },
    { 0xb4cf5b3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa275b44, SUC_NE_lexicon_tags_2 },
    { 0xafa55b45, SUC_NE_lexicon_tags_2 },
    { 0x84175b46, SUC_NE_lexicon_tags_2 },
    { 0xa8a75b44, SUC_NE_lexicon_tags_2 },
    { 0x609f5b46, SUC_NE_lexicon_tags_2 },
    { 0xca915b47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1575b4d, SUC_NE_lexicon_tags_5 },
    { 0xa335b4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3535b50, SUC_NE_lexicon_tags_2 },
    { 0x6fd35b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54b5b54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xecd5b56, SUC_NE_lexicon_tags_7 },
    { 0xb35d5b56, SUC_NE_lexicon_tags_2 },
    { 0x414b5b58, SUC_NE_lexicon_tags_2 },
    { 0xd1f35b58, SUC_NE_lexicon_tags_2 },
    { 0xc88d5b58, SUC_NE_lexicon_tags_2 },
    { 0x91df5b5b, SUC_NE_lexicon_tags_40 },
    { 0x23a75b58, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1095b60, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa2155b62, SUC_NE_lexicon_tags_2 },
    { 0x69f35b63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b5b66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b75b6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f95b6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11c55b70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59655b72, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x4d515b74, SUC_NE_lexicon_tags_3 },
    { 0x3bc95b74, SUC_NE_lexicon_tags_2 },
    { 0x86f5b76, SUC_NE_lexicon_tags_2 },
    { 0x93f75b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8855b79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1d5b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8595b81, SUC_NE_lexicon_tags_3 },
    { 0xadb15b82, SUC_NE_lexicon_tags_2 },
    { 0xe2055b83, SUC_NE_lexicon_tags_2 },
    { 0xdf595b84, SUC_NE_lexicon_tags_17 },
    { 0x18035b85, SUC_NE_lexicon_tags_2 },
    { 0x6be95b85, SUC_NE_lexicon_tags_4 },
    { 0x7cab5b87, SUC_NE_lexicon_tags_2 },
    { 0x70df5b87, SUC_NE_lexicon_tags_2 },
    { 0xfa15b89, SUC_NE_lexicon_tags_2 },
    { 0xfa7d5b86, SUC_NE_lexicon_tags_2 },
    { 0x87615b85, SUC_NE_lexicon_tags_2 },
    { 0x50c95b89, SUC_NE_lexicon_tags_2 },
    { 0xe54f5b8d, SUC_NE_lexicon_tags_2 },
    { 0x5d195b8d, SUC_NE_lexicon_tags_2 },
    { 0x2ef5b8f, SUC_NE_lexicon_tags_2 },
    { 0xa66d5b90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb015b92, SUC_NE_lexicon_tags_4 },
    { 0x842b5b93, SUC_NE_lexicon_tags_4 },
    { 0x62c15b94, SUC_NE_lexicon_tags_2 },
    { 0x5935b95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90b5b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd58b5b9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb15ba2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1475ba5, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e55ba8, SUC_NE_lexicon_tags_2 },
    { 0xee7b5ba8, SUC_NE_lexicon_tags_2 },
    { 0x16f75baa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x814b5bac, SUC_NE_lexicon_tags_2 },
    { 0x40255bac, SUC_NE_lexicon_tags_2 },
    { 0x6b875bae, SUC_NE_lexicon_tags_18 },
    { 0x461f5bac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a55bb3, SUC_NE_lexicon_tags_2 },
    { 0x75715bb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd87d5bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b555bbc, SUC_NE_lexicon_tags_2 },
    { 0x6c0f5bbd, SUC_NE_lexicon_tags_2 },
    { 0xe81f5bbe, SUC_NE_lexicon_tags_2 },
    { 0x75415bbf, SUC_NE_lexicon_tags_3 },
    { 0xef7b5bc0, SUC_NE_lexicon_tags_2 },
    { 0x12dd5bbd, SUC_NE_lexicon_tags_2 },
    { 0xf6915bbe, SUC_NE_lexicon_tags_2 },
    { 0x58195bc3, SUC_NE_lexicon_tags_2 },
    { 0xdbd55bbe, SUC_NE_lexicon_tags_2 },
    { 0x6bf75bbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5775bc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf5bcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ed5bce, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac55bd2, SUC_NE_lexicon_tags_7 },
    { 0xcba55bd2, SUC_NE_lexicon_tags_2 },
    { 0xa9db5bd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b175bd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3515bd9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb555bdb, SUC_NE_lexicon_tags_2 },
    { 0x1a9b5bdc, SUC_NE_lexicon_tags_2 },
    { 0x35915bdd, SUC_NE_lexicon_tags_2 },
    { 0x6e015bde, SUC_NE_lexicon_tags_2 },
    { 0xc3e15bdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda7f5be1, SUC_NE_lexicon_tags_2 },
    { 0x5aaf5be1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd75be6, SUC_NE_lexicon_tags_2 },
    { 0x590d5be6, SUC_NE_lexicon_tags_2 },
    { 0x98cd5be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27995bec, SUC_NE_lexicon_tags_2 },
    { 0x82ad5bed, SUC_NE_lexicon_tags_2 },
    { 0x17bd5bee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x382d5bf1, SUC_NE_lexicon_tags_2 },
    { 0x76fb5bf2, SUC_NE_lexicon_tags_2 },
    { 0x76df5bf1, SUC_NE_lexicon_tags_2 },
    { 0x72f55bf4, SUC_NE_lexicon_tags_2 },
    { 0xbe9d5bf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29855bf8, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c55bfc, SUC_NE_lexicon_tags_2 },
    { 0x9f055bfd, SUC_NE_lexicon_tags_2 },
    { 0x22c95bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4fb5c00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b135c02, SUC_NE_lexicon_tags_2 },
    { 0xeb9d5c02, SUC_NE_lexicon_tags_3 },
    { 0x5ce95c02, SUC_NE_lexicon_tags_2 },
    { 0x828b5c05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x397d5c08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605f5c0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0195c11, SUC_NE_lexicon_tags_2 },
    { 0x75875c12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55915c17, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc235c1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x580b5c1e, SUC_NE_lexicon_tags_2 },
    { 0xd1e55c1e, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304d5c23, SUC_NE_lexicon_tags_2 },
    { 0x9acf5c24, SUC_NE_lexicon_tags_2 },
    { 0xb05b5c25, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784b5c29, SUC_NE_lexicon_tags_2 },
    { 0x2ef15c29, SUC_NE_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3f5c2d, SUC_NE_lexicon_tags_2 },
    { 0x87f95c2d, SUC_NE_lexicon_tags_2 },
    { 0x6b1d5c2d, SUC_NE_lexicon_tags_2 },
    { 0xa3a75c2f, SUC_NE_lexicon_tags_2 },
    { 0xa6615c31, SUC_NE_lexicon_tags_2 },
    { 0x6ff75c31, SUC_NE_lexicon_tags_2 },
    { 0x1bef5c31, SUC_NE_lexicon_tags_2 },
    { 0x6d815c33, SUC_NE_lexicon_tags_2 },
    { 0xb935c33, SUC_NE_lexicon_tags_2 },
    { 0xaee35c33, SUC_NE_lexicon_tags_2 },
    { 0xdc6d5c32, SUC_NE_lexicon_tags_2 },
    { 0xc04b5c38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76b5c3b, SUC_NE_lexicon_tags_2 },
    { 0x12e35c3b, SUC_NE_lexicon_tags_2 },
    { 0xd83b5c3d, SUC_NE_lexicon_tags_2 },
    { 0xc2cf5c3d, SUC_NE_lexicon_tags_2 },
    { 0x97975c3f, SUC_NE_lexicon_tags_2 },
    { 0x9b135c40, SUC_NE_lexicon_tags_2 },
    { 0x856d5c3b, SUC_NE_lexicon_tags_17 },
    { 0x3ecf5c42, SUC_NE_lexicon_tags_2 },
    { 0xc3b5c3e, SUC_NE_lexicon_tags_4 },
    { 0x85795c3c, SUC_NE_lexicon_tags_2 },
    { 0xccdf5c45, SUC_NE_lexicon_tags_3 },
    { 0x71b35c45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad395c4a, SUC_NE_lexicon_tags_2 },
    { 0x380f5c4a, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdf5c50, SUC_NE_lexicon_tags_2 },
    { 0x52f75c50, SUC_NE_lexicon_tags_2 },
    { 0xca7d5c52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15055c55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60195c57, SUC_NE_lexicon_tags_2 },
    { 0x3de55c57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9675c5a, SUC_NE_lexicon_tags_2 },
    { 0xf98f5c5b, SUC_NE_lexicon_tags_2 },
    { 0x49af5c5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda555c5e, SUC_NE_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82315c63, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4f5c66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5715c6a, SUC_NE_lexicon_tags_30 },
    { 0xd4115c6b, SUC_NE_lexicon_tags_2 },
    { 0x41ff5c6b, SUC_NE_lexicon_tags_2 },
    { 0xb1015c6c, SUC_NE_lexicon_tags_2 },
    { 0x92b55c6c, SUC_NE_lexicon_tags_2 },
    { 0xa3fb5c6f, SUC_NE_lexicon_tags_2 },
    { 0xa5d95c6d, SUC_NE_lexicon_tags_2 },
    { 0x8ac35c6c, SUC_NE_lexicon_tags_2 },
    { 0xaf9d5c72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3f95c74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed95c77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c435c79, SUC_NE_lexicon_tags_2 },
    { 0x31275c7a, SUC_NE_lexicon_tags_2 },
    { 0xca8d5c79, SUC_NE_lexicon_tags_89 },
    { 0, NULL },
    { 0x79f15c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98155c7f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa4035c81, SUC_NE_lexicon_tags_2 },
    { 0x2f6b5c81, SUC_NE_lexicon_tags_2 },
    { 0x895c82, SUC_NE_lexicon_tags_2 },
    { 0x6b7f5c84, SUC_NE_lexicon_tags_2 },
    { 0x40d5c85, SUC_NE_lexicon_tags_2 },
    { 0x741d5c86, SUC_NE_lexicon_tags_4 },
    { 0x2e155c87, SUC_NE_lexicon_tags_2 },
    { 0x23295c85, SUC_NE_lexicon_tags_7 },
    { 0x5e55c89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3eed5c8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd35c8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x401f5c90, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2c935c92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x408d5c94, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a635c98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf655c9e, SUC_NE_lexicon_tags_2 },
    { 0x3dc15c9e, SUC_NE_lexicon_tags_2 },
    { 0xa5275ca0, SUC_NE_lexicon_tags_2 },
    { 0x64875ca0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b75ca4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf8f5ca6, SUC_NE_lexicon_tags_2 },
    { 0x246d5ca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b35cab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3cb5cad, SUC_NE_lexicon_tags_2 },
    { 0x4c075cad, SUC_NE_lexicon_tags_2 },
    { 0x35335cad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b75cb2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852f5cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa24b5cb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x839d5cbb, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7215cc7, SUC_NE_lexicon_tags_2 },
    { 0x251f5cc7, SUC_NE_lexicon_tags_2 },
    { 0x28695cc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2215ccb, SUC_NE_lexicon_tags_2 },
    { 0x61855ccc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bab5cd0, SUC_NE_lexicon_tags_7 },
    { 0x48b55cd1, SUC_NE_lexicon_tags_2 },
    { 0x73f55cd2, SUC_NE_lexicon_tags_19 },
    { 0xe9c75cd0, SUC_NE_lexicon_tags_2 },
    { 0x8f0d5cd4, SUC_NE_lexicon_tags_107 },
    { 0xa1255cd4, SUC_NE_lexicon_tags_2 },
    { 0xcd7b5cd4, SUC_NE_lexicon_tags_2 },
    { 0x66955cd0, SUC_NE_lexicon_tags_2 },
    { 0xeb275cd8, SUC_NE_lexicon_tags_2 },
    { 0xd7395cd8, SUC_NE_lexicon_tags_2 },
    { 0x2ca95cd7, SUC_NE_lexicon_tags_2 },
    { 0xf06d5cdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88655ce0, SUC_NE_lexicon_tags_2 },
    { 0x98cb5ce1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9d5ce4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5db5ce6, SUC_NE_lexicon_tags_2 },
    { 0x2d595ce7, SUC_NE_lexicon_tags_2 },
    { 0xd2bb5ce8, SUC_NE_lexicon_tags_18 },
    { 0x1cc55ce9, SUC_NE_lexicon_tags_2 },
    { 0x6375cea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeac95cec, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3d75cee, SUC_NE_lexicon_tags_2 },
    { 0x7e9d5cef, SUC_NE_lexicon_tags_2 },
    { 0x484b5cee, SUC_NE_lexicon_tags_2 },
    { 0x49df5cf1, SUC_NE_lexicon_tags_2 },
    { 0xad835cf2, SUC_NE_lexicon_tags_2 },
    { 0xe3715cf3, SUC_NE_lexicon_tags_2 },
    { 0x77735cf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33d5cf8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc55cfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dd5d07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde35d09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc455d0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b35d17, SUC_NE_lexicon_tags_2 },
    { 0xc3ad5d18, SUC_NE_lexicon_tags_7 },
    { 0x25255d19, SUC_NE_lexicon_tags_2 },
    { 0x59ef5d1a, SUC_NE_lexicon_tags_2 },
    { 0x64015d18, SUC_NE_lexicon_tags_24 },
    { 0x752d5d1c, SUC_NE_lexicon_tags_2 },
    { 0x2cf35d1c, SUC_NE_lexicon_tags_2 },
    { 0x84595d1e, SUC_NE_lexicon_tags_2 },
    { 0x1b635d1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d0b5d21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb915d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65915d2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55055d2f, SUC_NE_lexicon_tags_2 },
    { 0xf83f5d30, SUC_NE_lexicon_tags_2 },
    { 0xc9395d31, SUC_NE_lexicon_tags_2 },
    { 0xbbdd5d30, SUC_NE_lexicon_tags_2 },
    { 0x88155d30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a15d38, SUC_NE_lexicon_tags_2 },
    { 0x31b95d38, SUC_NE_lexicon_tags_2 },
    { 0x804f5d3a, SUC_NE_lexicon_tags_2 },
    { 0x4c275d3b, SUC_NE_lexicon_tags_2 },
    { 0x551f5d3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4095d40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e095d44, SUC_NE_lexicon_tags_2 },
    { 0x469d5d45, SUC_NE_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5b5d48, SUC_NE_lexicon_tags_2 },
    { 0x7f695d48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b1d5d4b, SUC_NE_lexicon_tags_2 },
    { 0x8c435d4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6455d4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc95d55, SUC_NE_lexicon_tags_30 },
    { 0x9e795d55, SUC_NE_lexicon_tags_2 },
    { 0x9fcb5d57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff5d5b, SUC_NE_lexicon_tags_2 },
    { 0x9fe75d5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d75d5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b175d60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841d5d65, SUC_NE_lexicon_tags_1 },
    { 0xd5655d66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e835d6d, SUC_NE_lexicon_tags_4 },
    { 0xa97f5d6e, SUC_NE_lexicon_tags_2 },
    { 0x51e15d6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29855d79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e615d7e, SUC_NE_lexicon_tags_2 },
    { 0xf1a35d7f, SUC_NE_lexicon_tags_2 },
    { 0xdb5b5d7f, SUC_NE_lexicon_tags_2 },
    { 0x59355d81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812d5d85, SUC_NE_lexicon_tags_2 },
    { 0x6f255d86, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x95195d88, SUC_NE_lexicon_tags_2 },
    { 0x443b5d89, SUC_NE_lexicon_tags_3 },
    { 0x83eb5d8a, SUC_NE_lexicon_tags_2 },
    { 0x58a35d89, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b655d8d, SUC_NE_lexicon_tags_2 },
    { 0xf8e55d8e, SUC_NE_lexicon_tags_2 },
    { 0xe5235d8f, SUC_NE_lexicon_tags_111 },
    { 0x46255d90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a55d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d15d95, SUC_NE_lexicon_tags_2 },
    { 0x44015d96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d3b5d98, SUC_NE_lexicon_tags_2 },
    { 0xab975d98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d935d9c, SUC_NE_lexicon_tags_2 },
    { 0xa3dd5d9c, SUC_NE_lexicon_tags_2 },
    { 0xd5415d9e, SUC_NE_lexicon_tags_2 },
    { 0xd47f5d9e, SUC_NE_lexicon_tags_2 },
    { 0x57375da0, SUC_NE_lexicon_tags_2 },
    { 0x8215da0, SUC_NE_lexicon_tags_89 },
    { 0x6db75d9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d15da5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3dd5da7, SUC_NE_lexicon_tags_2 },
    { 0x1c0b5da8, SUC_NE_lexicon_tags_2 },
    { 0xde235da9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f915dab, SUC_NE_lexicon_tags_2 },
    { 0xfafb5dac, SUC_NE_lexicon_tags_2 },
    { 0xa5e75dad, SUC_NE_lexicon_tags_2 },
    { 0xc0655dae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d215db1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad735db7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd04f5db9, SUC_NE_lexicon_tags_2 },
    { 0x4e255db9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e575dbe, SUC_NE_lexicon_tags_2 },
    { 0x4dc35dbf, SUC_NE_lexicon_tags_2 },
    { 0xfedf5dc0, SUC_NE_lexicon_tags_0 },
    { 0x17d75dbf, SUC_NE_lexicon_tags_2 },
    { 0xb0b75dc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a075dc8, SUC_NE_lexicon_tags_2 },
    { 0x26b55dc8, SUC_NE_lexicon_tags_2 },
    { 0xcd1b5dca, SUC_NE_lexicon_tags_2 },
    { 0xf2c75dc8, SUC_NE_lexicon_tags_2 },
    { 0x924f5dcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21695dce, SUC_NE_lexicon_tags_2 },
    { 0x362b5dce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f015dd3, SUC_NE_lexicon_tags_2 },
    { 0x8b595dd3, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xa3295dd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf57d5dd8, SUC_NE_lexicon_tags_2 },
    { 0x4fbb5dd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10275ddb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3335de8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45255deb, SUC_NE_lexicon_tags_2 },
    { 0x38815dec, SUC_NE_lexicon_tags_2 },
    { 0xb04f5ded, SUC_NE_lexicon_tags_4 },
    { 0x7395ded, SUC_NE_lexicon_tags_2 },
    { 0xa6175def, SUC_NE_lexicon_tags_2 },
    { 0x7a6b5dec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e675df4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf5df7, SUC_NE_lexicon_tags_2 },
    { 0x137b5df8, SUC_NE_lexicon_tags_6 },
    { 0x7f4f5df7, SUC_NE_lexicon_tags_2 },
    { 0x9a515dfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f5e00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7595e02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca235e04, SUC_NE_lexicon_tags_2 },
    { 0x1a755e05, SUC_NE_lexicon_tags_2 },
    { 0xa6d55e04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0435e09, SUC_NE_lexicon_tags_2 },
    { 0xbb7f5e0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef55e0d, SUC_NE_lexicon_tags_7 },
    { 0x8ad15e0d, SUC_NE_lexicon_tags_2 },
    { 0x89c75e0f, SUC_NE_lexicon_tags_2 },
    { 0x3b055e0d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x7cdf5e12, SUC_NE_lexicon_tags_2 },
    { 0x63f15e13, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e95e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967d5e1a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326b5e1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b795e20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a695e22, SUC_NE_lexicon_tags_5 },
    { 0x61dd5e23, SUC_NE_lexicon_tags_2 },
    { 0xc6415e24, SUC_NE_lexicon_tags_2 },
    { 0x3db15e25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e955e28, SUC_NE_lexicon_tags_2 },
    { 0xa1c95e28, SUC_NE_lexicon_tags_2 },
    { 0x8fc75e28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7435e2d, SUC_NE_lexicon_tags_2 },
    { 0xb8cf5e2d, SUC_NE_lexicon_tags_2 },
    { 0x6ae75e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a35e32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d15e35, SUC_NE_lexicon_tags_2 },
    { 0x62195e36, SUC_NE_lexicon_tags_2 },
    { 0x505f5e37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f875e3c, SUC_NE_lexicon_tags_9 },
    { 0xa25b5e3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6035e40, SUC_NE_lexicon_tags_2 },
    { 0xde955e41, SUC_NE_lexicon_tags_2 },
    { 0x1bb35e40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb75e46, SUC_NE_lexicon_tags_2 },
    { 0x1a475e47, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d55e4b, SUC_NE_lexicon_tags_2 },
    { 0xdd055e4b, SUC_NE_lexicon_tags_32 },
    { 0xed155e4b, SUC_NE_lexicon_tags_2 },
    { 0x89c55e4d, SUC_NE_lexicon_tags_2 },
    { 0x261b5e4b, SUC_NE_lexicon_tags_2 },
    { 0x6a515e50, SUC_NE_lexicon_tags_2 },
    { 0xdcdd5e51, SUC_NE_lexicon_tags_2 },
    { 0x97955e51, SUC_NE_lexicon_tags_2 },
    { 0x8c315e51, SUC_NE_lexicon_tags_2 },
    { 0xa5bf5e54, SUC_NE_lexicon_tags_2 },
    { 0x6b355e54, SUC_NE_lexicon_tags_4 },
    { 0xdb415e56, SUC_NE_lexicon_tags_2 },
    { 0xbdf5e57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e95e5e, SUC_NE_lexicon_tags_2 },
    { 0x68f95e5e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c95e63, SUC_NE_lexicon_tags_2 },
    { 0x44a75e64, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1d5e67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85b5e6a, SUC_NE_lexicon_tags_2 },
    { 0x8b235e6a, SUC_NE_lexicon_tags_2 },
    { 0xc4c55e6c, SUC_NE_lexicon_tags_7 },
    { 0x27fb5e6a, SUC_NE_lexicon_tags_4 },
    { 0xf1c95e6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56535e71, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38415e7c, SUC_NE_lexicon_tags_2 },
    { 0xba915e7d, SUC_NE_lexicon_tags_2 },
    { 0x6755e7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fb5e81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7115e83, SUC_NE_lexicon_tags_2 },
    { 0x26dd5e84, SUC_NE_lexicon_tags_7 },
    { 0x29d15e85, SUC_NE_lexicon_tags_2 },
    { 0xee875e84, SUC_NE_lexicon_tags_19 },
    { 0x73ed5e87, SUC_NE_lexicon_tags_2 },
    { 0xc88d5e87, SUC_NE_lexicon_tags_2 },
    { 0x16535e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b495e8b, SUC_NE_lexicon_tags_2 },
    { 0x8e275e8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f15e8f, SUC_NE_lexicon_tags_2 },
    { 0x61df5e90, SUC_NE_lexicon_tags_2 },
    { 0xf9f35e91, SUC_NE_lexicon_tags_30 },
    { 0x84615e92, SUC_NE_lexicon_tags_2 },
    { 0x21955e93, SUC_NE_lexicon_tags_2 },
    { 0x1a975e94, SUC_NE_lexicon_tags_2 },
    { 0x62195e93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f5e9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a75e9f, SUC_NE_lexicon_tags_2 },
    { 0x55535ea0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b8b5ea2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c755ea6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ff5eaa, SUC_NE_lexicon_tags_2 },
    { 0x55f15eab, SUC_NE_lexicon_tags_2 },
    { 0xc3b75eaa, SUC_NE_lexicon_tags_2 },
    { 0xa0df5eac, SUC_NE_lexicon_tags_2 },
    { 0x11f5ead, SUC_NE_lexicon_tags_2 },
    { 0x32095eaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dd5eb5, SUC_NE_lexicon_tags_2 },
    { 0xdb775eb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc95ebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb5ec1, SUC_NE_lexicon_tags_2 },
    { 0xca035ec2, SUC_NE_lexicon_tags_2 },
    { 0xb66f5ec2, SUC_NE_lexicon_tags_2 },
    { 0x2d5f5ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9895ec6, SUC_NE_lexicon_tags_2 },
    { 0x4d535ec7, SUC_NE_lexicon_tags_2 },
    { 0x53955ec6, SUC_NE_lexicon_tags_2 },
    { 0x2fe55ec9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c75ecd, SUC_NE_lexicon_tags_4 },
    { 0x89795ece, SUC_NE_lexicon_tags_2 },
    { 0x138f5ecf, SUC_NE_lexicon_tags_2 },
    { 0x4c75ed0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc335ed3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a555ed9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23935ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeceb5ee2, SUC_NE_lexicon_tags_4 },
    { 0x94d75ee3, SUC_NE_lexicon_tags_2 },
    { 0x6e975ee2, SUC_NE_lexicon_tags_2 },
    { 0xf9e75ee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31bd5ee7, SUC_NE_lexicon_tags_7 },
    { 0x8aa75ee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2115eea, SUC_NE_lexicon_tags_2 },
    { 0xa3795eea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294f5eef, SUC_NE_lexicon_tags_2 },
    { 0xc6355ef0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec95ef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8cb5ef7, SUC_NE_lexicon_tags_2 },
    { 0x1d735ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f5efb, SUC_NE_lexicon_tags_2 },
    { 0x89c75efc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ff5f00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe18f5f02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd35f09, SUC_NE_lexicon_tags_2 },
    { 0x7e735f0a, SUC_NE_lexicon_tags_2 },
    { 0xe8d95f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x845f5f0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79135f13, SUC_NE_lexicon_tags_4 },
    { 0xe8f55f13, SUC_NE_lexicon_tags_2 },
    { 0xc73f5f15, SUC_NE_lexicon_tags_2 },
    { 0x9ae75f13, SUC_NE_lexicon_tags_2 },
    { 0x79775f17, SUC_NE_lexicon_tags_2 },
    { 0xaa435f16, SUC_NE_lexicon_tags_2 },
    { 0x53f15f19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc55f1c, SUC_NE_lexicon_tags_2 },
    { 0x5c175f1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87955f21, SUC_NE_lexicon_tags_2 },
    { 0x557f5f22, SUC_NE_lexicon_tags_2 },
    { 0x311d5f23, SUC_NE_lexicon_tags_2 },
    { 0x15ff5f21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45f5f27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2935f2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ef5f2e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a5d5f30, SUC_NE_lexicon_tags_2 },
    { 0xf2855f31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8975f38, SUC_NE_lexicon_tags_2 },
    { 0x98275f38, SUC_NE_lexicon_tags_58 },
    { 0xe4655f3a, SUC_NE_lexicon_tags_7 },
    { 0x81855f3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80dd5f3d, SUC_NE_lexicon_tags_2 },
    { 0xd0c75f3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb535f4b, SUC_NE_lexicon_tags_2 },
    { 0xe1715f4b, SUC_NE_lexicon_tags_5 },
    { 0x424b5f4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30af5f4f, SUC_NE_lexicon_tags_2 },
    { 0x83735f50, SUC_NE_lexicon_tags_2 },
    { 0x89515f50, SUC_NE_lexicon_tags_3 },
    { 0xcf3d5f50, SUC_NE_lexicon_tags_2 },
    { 0x930d5f50, SUC_NE_lexicon_tags_1 },
    { 0x8a475f54, SUC_NE_lexicon_tags_2 },
    { 0x14e15f55, SUC_NE_lexicon_tags_2 },
    { 0x15635f56, SUC_NE_lexicon_tags_2 },
    { 0xeb535f57, SUC_NE_lexicon_tags_2 },
    { 0xed575f58, SUC_NE_lexicon_tags_135 },
    { 0, NULL },
    { 0x84255f5a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880f5f5f, SUC_NE_lexicon_tags_2 },
    { 0xa4ad5f5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf95f67, SUC_NE_lexicon_tags_2 },
    { 0xb2bd5f68, SUC_NE_lexicon_tags_34 },
    { 0xed1f5f69, SUC_NE_lexicon_tags_2 },
    { 0xf0c55f69, SUC_NE_lexicon_tags_2 },
    { 0x452f5f69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bc75f6d, SUC_NE_lexicon_tags_2 },
    { 0xc6115f6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b055f74, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48495f7b, SUC_NE_lexicon_tags_2 },
    { 0xadfb5f7b, SUC_NE_lexicon_tags_2 },
    { 0xe3735f7d, SUC_NE_lexicon_tags_2 },
    { 0xfdbb5f7e, SUC_NE_lexicon_tags_2 },
    { 0x1f535f7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc135f83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40c55f85, SUC_NE_lexicon_tags_2 },
    { 0x2d295f86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde235f89, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa195f8c, SUC_NE_lexicon_tags_2 },
    { 0xa46f5f8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92735f8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x771d5f92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b975f95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a635f99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40d5f9c, SUC_NE_lexicon_tags_2 },
    { 0xe3c15f9d, SUC_NE_lexicon_tags_2 },
    { 0xfb6b5f9e, SUC_NE_lexicon_tags_2 },
    { 0xa2495f9c, SUC_NE_lexicon_tags_2 },
    { 0x41195fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afd5fa6, SUC_NE_lexicon_tags_2 },
    { 0x8aff5fa7, SUC_NE_lexicon_tags_2 },
    { 0xf1db5fa8, SUC_NE_lexicon_tags_2 },
    { 0x2ecb5fa9, SUC_NE_lexicon_tags_2 },
    { 0x17a55faa, SUC_NE_lexicon_tags_2 },
    { 0x62735fab, SUC_NE_lexicon_tags_4 },
    { 0xe60d5fac, SUC_NE_lexicon_tags_7 },
    { 0x43b15faa, SUC_NE_lexicon_tags_2 },
    { 0x6a475fae, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x193f5fb1, SUC_NE_lexicon_tags_2 },
    { 0xf5855fb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74775fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a435fb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1715fb8, SUC_NE_lexicon_tags_2 },
    { 0x52575fb9, SUC_NE_lexicon_tags_2 },
    { 0x57115fba, SUC_NE_lexicon_tags_3 },
    { 0x62cb5fbb, SUC_NE_lexicon_tags_53 },
    { 0xea295fb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e95fc0, SUC_NE_lexicon_tags_2 },
    { 0xf695fc1, SUC_NE_lexicon_tags_2 },
    { 0xb3f95fc2, SUC_NE_lexicon_tags_2 },
    { 0x2eb95fc3, SUC_NE_lexicon_tags_2 },
    { 0x81295fc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebeb5fc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf15fcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc255fce, SUC_NE_lexicon_tags_2 },
    { 0x7c0f5fcf, SUC_NE_lexicon_tags_1 },
    { 0xfab55fcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f35fd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf75fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f815fda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981f5fdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb15fe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70d35fe4, SUC_NE_lexicon_tags_2 },
    { 0x26b15fe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27455fe7, SUC_NE_lexicon_tags_155 },
    { 0x28f95fe7, SUC_NE_lexicon_tags_2 },
    { 0x9c9b5fe9, SUC_NE_lexicon_tags_2 },
    { 0x40575fe9, SUC_NE_lexicon_tags_2 },
    { 0xdbfb5feb, SUC_NE_lexicon_tags_2 },
    { 0xc26b5fec, SUC_NE_lexicon_tags_2 },
    { 0xc7dd5fed, SUC_NE_lexicon_tags_17 },
    { 0x20d5feb, SUC_NE_lexicon_tags_2 },
    { 0xbebd5fed, SUC_NE_lexicon_tags_2 },
    { 0x145d5feb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7c35ff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d515ff7, SUC_NE_lexicon_tags_2 },
    { 0x971f5ff7, SUC_NE_lexicon_tags_2 },
    { 0x8fb95ff9, SUC_NE_lexicon_tags_2 },
    { 0xa04d5ff9, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x61535ffc, SUC_NE_lexicon_tags_2 },
    { 0xf2a95ffd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24af6002, SUC_NE_lexicon_tags_30 },
    { 0x602f6003, SUC_NE_lexicon_tags_2 },
    { 0xab216004, SUC_NE_lexicon_tags_2 },
    { 0xbf736004, SUC_NE_lexicon_tags_2 },
    { 0x9c0d6006, SUC_NE_lexicon_tags_2 },
    { 0x1a736007, SUC_NE_lexicon_tags_2 },
    { 0x4d696006, SUC_NE_lexicon_tags_2 },
    { 0x736b6009, SUC_NE_lexicon_tags_2 },
    { 0xe2ab6004, SUC_NE_lexicon_tags_2 },
    { 0xf309600b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2335600e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74196012, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b016014, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee636016, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f6019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3d6020, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10cf6022, SUC_NE_lexicon_tags_2 },
    { 0x71b16023, SUC_NE_lexicon_tags_2 },
    { 0x5036024, SUC_NE_lexicon_tags_4 },
    { 0x47996025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67576027, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38d602a, SUC_NE_lexicon_tags_2 },
    { 0xbb61602a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f5602e, SUC_NE_lexicon_tags_2 },
    { 0x2b67602f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b76034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41e96036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73ed6038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4103603c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9995603e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae16041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4076045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b85604a, SUC_NE_lexicon_tags_2 },
    { 0x9b13604b, SUC_NE_lexicon_tags_2 },
    { 0x7fe9604c, SUC_NE_lexicon_tags_2 },
    { 0xc7b604d, SUC_NE_lexicon_tags_2 },
    { 0x38bb604c, SUC_NE_lexicon_tags_2 },
    { 0x8401604b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x735f6051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b36054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a356057, SUC_NE_lexicon_tags_2 },
    { 0xd1fb6057, SUC_NE_lexicon_tags_2 },
    { 0x7ac56059, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0x2129605b, SUC_NE_lexicon_tags_2 },
    { 0xad3d605c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e56063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cd6068, SUC_NE_lexicon_tags_3 },
    { 0x89516068, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52b9606b, SUC_NE_lexicon_tags_2 },
    { 0x976b606c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe5f606e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35b96070, SUC_NE_lexicon_tags_2 },
    { 0xaa0f6070, SUC_NE_lexicon_tags_2 },
    { 0x3ef6072, SUC_NE_lexicon_tags_10 },
    { 0xeb376073, SUC_NE_lexicon_tags_2 },
    { 0x5a9f6071, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92396076, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1b607b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff607e, SUC_NE_lexicon_tags_2 },
    { 0xe6fb607f, SUC_NE_lexicon_tags_2 },
    { 0xb81f6080, SUC_NE_lexicon_tags_7 },
    { 0x2931607f, SUC_NE_lexicon_tags_7 },
    { 0xb3eb6082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d96086, SUC_NE_lexicon_tags_3 },
    { 0xd2e56086, SUC_NE_lexicon_tags_2 },
    { 0xc2d76088, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f7608c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9067608e, SUC_NE_lexicon_tags_3 },
    { 0xf255608e, SUC_NE_lexicon_tags_5 },
    { 0xe6f96090, SUC_NE_lexicon_tags_2 },
    { 0x7cb16091, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a56097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9cb6099, SUC_NE_lexicon_tags_254 },
    { 0, NULL },
    { 0x75ed609b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd11609e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67960a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b3960a8, SUC_NE_lexicon_tags_2 },
    { 0x91ff60a9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdd60ac, SUC_NE_lexicon_tags_2 },
    { 0xabeb60ac, SUC_NE_lexicon_tags_2 },
    { 0x3f9760ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a960b1, SUC_NE_lexicon_tags_30 },
    { 0xd93560b1, SUC_NE_lexicon_tags_2 },
    { 0x328b60b2, SUC_NE_lexicon_tags_2 },
    { 0xaa6d60b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc160b8, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df60bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fbd60be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adf60c3, SUC_NE_lexicon_tags_2 },
    { 0x7b3160c3, SUC_NE_lexicon_tags_2 },
    { 0x43c960c3, SUC_NE_lexicon_tags_2 },
    { 0x10f360c6, SUC_NE_lexicon_tags_2 },
    { 0xe63760c6, SUC_NE_lexicon_tags_2 },
    { 0xc90f60c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23760cb, SUC_NE_lexicon_tags_2 },
    { 0x4e5960cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddd60cf, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fff60d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f60d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9160d9, SUC_NE_lexicon_tags_2 },
    { 0xfdf560d9, SUC_NE_lexicon_tags_2 },
    { 0xab7f60da, SUC_NE_lexicon_tags_0 },
    { 0x495b60dc, SUC_NE_lexicon_tags_5 },
    { 0x4db960dc, SUC_NE_lexicon_tags_32 },
    { 0x765160de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8b60e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x352b60e4, SUC_NE_lexicon_tags_2 },
    { 0xfb1b60e5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf760eb, SUC_NE_lexicon_tags_2 },
    { 0xff360ec, SUC_NE_lexicon_tags_2 },
    { 0x345f60ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad760f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1760f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3960f8, SUC_NE_lexicon_tags_2 },
    { 0xbd7560f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa18160fb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe12760fd, SUC_NE_lexicon_tags_2 },
    { 0x6f2560fd, SUC_NE_lexicon_tags_7 },
    { 0xea9b60ff, SUC_NE_lexicon_tags_2 },
    { 0x91a76100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72736103, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf47d6105, SUC_NE_lexicon_tags_7 },
    { 0xd4fb6106, SUC_NE_lexicon_tags_3 },
    { 0xf1f76106, SUC_NE_lexicon_tags_2 },
    { 0xef6f6108, SUC_NE_lexicon_tags_2 },
    { 0xb5136106, SUC_NE_lexicon_tags_2 },
    { 0xc68d6109, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196f610e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f676113, SUC_NE_lexicon_tags_2 },
    { 0x2e0d6113, SUC_NE_lexicon_tags_2 },
    { 0xc32d6114, SUC_NE_lexicon_tags_2 },
    { 0x8eb36116, SUC_NE_lexicon_tags_2 },
    { 0xa7c96117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5219611a, SUC_NE_lexicon_tags_2 },
    { 0x3f2d611b, SUC_NE_lexicon_tags_2 },
    { 0xb151611c, SUC_NE_lexicon_tags_2 },
    { 0xf63d611d, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1b6123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88376128, SUC_NE_lexicon_tags_2 },
    { 0x3f316129, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f7612c, SUC_NE_lexicon_tags_2 },
    { 0x7c4f612d, SUC_NE_lexicon_tags_2 },
    { 0xae91612e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51f6130, SUC_NE_lexicon_tags_2 },
    { 0x6c9d6130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x279b6134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b6138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ed1613a, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb45613e, SUC_NE_lexicon_tags_2 },
    { 0x3783613e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e6b6141, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91f16143, SUC_NE_lexicon_tags_2 },
    { 0x4e7d6144, SUC_NE_lexicon_tags_7 },
    { 0x7ea76144, SUC_NE_lexicon_tags_2 },
    { 0xf60f6146, SUC_NE_lexicon_tags_2 },
    { 0x46336143, SUC_NE_lexicon_tags_2 },
    { 0x74fd6148, SUC_NE_lexicon_tags_2 },
    { 0xd5456149, SUC_NE_lexicon_tags_17 },
    { 0xad5b6145, SUC_NE_lexicon_tags_2 },
    { 0x9c81614b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f5614e, SUC_NE_lexicon_tags_2 },
    { 0x96b7614f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e756155, SUC_NE_lexicon_tags_2 },
    { 0x7dd76156, SUC_NE_lexicon_tags_2 },
    { 0x975f6156, SUC_NE_lexicon_tags_2 },
    { 0x13116156, SUC_NE_lexicon_tags_2 },
    { 0x3b336159, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d75615d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7099615f, SUC_NE_lexicon_tags_2 },
    { 0xa7296160, SUC_NE_lexicon_tags_2 },
    { 0x4b7f615f, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544d6165, SUC_NE_lexicon_tags_2 },
    { 0x35ab6166, SUC_NE_lexicon_tags_2 },
    { 0xab396167, SUC_NE_lexicon_tags_3 },
    { 0x96816168, SUC_NE_lexicon_tags_2 },
    { 0x3deb6165, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd291616b, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa43616e, SUC_NE_lexicon_tags_2 },
    { 0x32f1616f, SUC_NE_lexicon_tags_2 },
    { 0xccfd616e, SUC_NE_lexicon_tags_4 },
    { 0xdc4f616f, SUC_NE_lexicon_tags_0 },
    { 0xdfb9616e, SUC_NE_lexicon_tags_2 },
    { 0xce5616e, SUC_NE_lexicon_tags_2 },
    { 0x9a556174, SUC_NE_lexicon_tags_2 },
    { 0x152b6174, SUC_NE_lexicon_tags_2 },
    { 0xc0356175, SUC_NE_lexicon_tags_2 },
    { 0xd14b6174, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b1617f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd6183, SUC_NE_lexicon_tags_2 },
    { 0x3b7d6183, SUC_NE_lexicon_tags_2 },
    { 0xe6eb6183, SUC_NE_lexicon_tags_2 },
    { 0x6c156184, SUC_NE_lexicon_tags_7 },
    { 0x9fff6187, SUC_NE_lexicon_tags_2 },
    { 0xc4cd6188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ab7618a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x63eb618c, SUC_NE_lexicon_tags_220 },
    { 0, NULL },
    { 0xde87618e, SUC_NE_lexicon_tags_2 },
    { 0xbe91618f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xb88f6191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73e16193, SUC_NE_lexicon_tags_2 },
    { 0x12856193, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed556196, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cf619c, SUC_NE_lexicon_tags_2 },
    { 0x4a15619d, SUC_NE_lexicon_tags_2 },
    { 0x7d7d619e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x806761a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22e161a2, SUC_NE_lexicon_tags_2 },
    { 0xe8c561a3, SUC_NE_lexicon_tags_2 },
    { 0x828961a4, SUC_NE_lexicon_tags_2 },
    { 0xdd0161a4, SUC_NE_lexicon_tags_2 },
    { 0xdedd61a6, SUC_NE_lexicon_tags_2 },
    { 0x92e561a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf4761a9, SUC_NE_lexicon_tags_2 },
    { 0xf51b61a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248761ac, SUC_NE_lexicon_tags_2 },
    { 0xa9d61ad, SUC_NE_lexicon_tags_2 },
    { 0xd5dd61ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33ef61b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80d61b8, SUC_NE_lexicon_tags_2 },
    { 0xd7c161b9, SUC_NE_lexicon_tags_2 },
    { 0x5fed61ba, SUC_NE_lexicon_tags_2 },
    { 0x528961bb, SUC_NE_lexicon_tags_2 },
    { 0x694b61bc, SUC_NE_lexicon_tags_2 },
    { 0xa68161bc, SUC_NE_lexicon_tags_2 },
    { 0x6aa761be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc161c3, SUC_NE_lexicon_tags_2 },
    { 0x3c4561c4, SUC_NE_lexicon_tags_2 },
    { 0x36b761c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894161c9, SUC_NE_lexicon_tags_4 },
    { 0x65bb61c9, SUC_NE_lexicon_tags_40 },
    { 0xb54961cb, SUC_NE_lexicon_tags_2 },
    { 0x404561cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d9161ce, SUC_NE_lexicon_tags_2 },
    { 0xc2d161cf, SUC_NE_lexicon_tags_2 },
    { 0xca6561d0, SUC_NE_lexicon_tags_2 },
    { 0x7ee161d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe4961d3, SUC_NE_lexicon_tags_2 },
    { 0x12d161d4, SUC_NE_lexicon_tags_2 },
    { 0xe58361d5, SUC_NE_lexicon_tags_2 },
    { 0xf41961d4, SUC_NE_lexicon_tags_2 },
    { 0x273161d6, SUC_NE_lexicon_tags_2 },
    { 0x750361d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6d61db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78161df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47af61e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f61ec, SUC_NE_lexicon_tags_24 },
    { 0xa36761ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbb561ef, SUC_NE_lexicon_tags_2 },
    { 0x8f4961ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0dd61f2, SUC_NE_lexicon_tags_2 },
    { 0x9ef361f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29761f8, SUC_NE_lexicon_tags_2 },
    { 0x61a561f9, SUC_NE_lexicon_tags_2 },
    { 0xf3e561fa, SUC_NE_lexicon_tags_2 },
    { 0x7d1161fa, SUC_NE_lexicon_tags_2 },
    { 0xb72f61fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb2b61fe, SUC_NE_lexicon_tags_2 },
    { 0xa0a761fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee936202, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cd6205, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x73556207, SUC_NE_lexicon_tags_2 },
    { 0x9d116207, SUC_NE_lexicon_tags_2 },
    { 0x67636207, SUC_NE_lexicon_tags_7 },
    { 0x64e96209, SUC_NE_lexicon_tags_2 },
    { 0x8abd6207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44cf620d, SUC_NE_lexicon_tags_3 },
    { 0x7b8d620e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0916210, SUC_NE_lexicon_tags_2 },
    { 0x2d4b6211, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78436213, SUC_NE_lexicon_tags_2 },
    { 0x889b6213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf621d, SUC_NE_lexicon_tags_2 },
    { 0x4a5d621e, SUC_NE_lexicon_tags_2 },
    { 0xa959621d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc716222, SUC_NE_lexicon_tags_2 },
    { 0xacdd6223, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3536225, SUC_NE_lexicon_tags_2 },
    { 0x87cd6226, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6296229, SUC_NE_lexicon_tags_2 },
    { 0x24f36229, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd0b622c, SUC_NE_lexicon_tags_2 },
    { 0x8697622d, SUC_NE_lexicon_tags_2 },
    { 0xcc75622e, SUC_NE_lexicon_tags_2 },
    { 0x5325622e, SUC_NE_lexicon_tags_2 },
    { 0x183f622e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf896233, SUC_NE_lexicon_tags_5 },
    { 0x77d16234, SUC_NE_lexicon_tags_4 },
    { 0xa8c96235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fb623c, SUC_NE_lexicon_tags_2 },
    { 0x47ef623d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5916241, SUC_NE_lexicon_tags_2 },
    { 0x69556242, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4611624b, SUC_NE_lexicon_tags_2 },
    { 0x4f4f624c, SUC_NE_lexicon_tags_2 },
    { 0xf953624d, SUC_NE_lexicon_tags_2 },
    { 0xe8ef624d, SUC_NE_lexicon_tags_2 },
    { 0x5e41624f, SUC_NE_lexicon_tags_2 },
    { 0xa136250, SUC_NE_lexicon_tags_117 },
    { 0x2089624d, SUC_NE_lexicon_tags_2 },
    { 0x5c67624f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a5625d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67116261, SUC_NE_lexicon_tags_50 },
    { 0xe31d6262, SUC_NE_lexicon_tags_2 },
    { 0xe7bb6262, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb756266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6dd6268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce37626c, SUC_NE_lexicon_tags_2 },
    { 0x6131626c, SUC_NE_lexicon_tags_2 },
    { 0xb67f626e, SUC_NE_lexicon_tags_2 },
    { 0x2cf5626f, SUC_NE_lexicon_tags_2 },
    { 0x25576270, SUC_NE_lexicon_tags_13 },
    { 0xd9956271, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367b627b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75716280, SUC_NE_lexicon_tags_2 },
    { 0xb48f6281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2716285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b83628a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01628e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d596291, SUC_NE_lexicon_tags_2 },
    { 0x31056291, SUC_NE_lexicon_tags_2 },
    { 0xa40d6291, SUC_NE_lexicon_tags_2 },
    { 0xdb2f6294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d76297, SUC_NE_lexicon_tags_2 },
    { 0x91a36298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41d9629a, SUC_NE_lexicon_tags_2 },
    { 0x7eef629a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5b62a0, SUC_NE_lexicon_tags_2 },
    { 0xf38562a0, SUC_NE_lexicon_tags_2 },
    { 0x46df62a2, SUC_NE_lexicon_tags_2 },
    { 0x23f762a2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb40362a5, SUC_NE_lexicon_tags_2 },
    { 0xc7a162a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62762a9, SUC_NE_lexicon_tags_2 },
    { 0xc2f362aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9162b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99af62b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa362b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82762bc, SUC_NE_lexicon_tags_41 },
    { 0xa16562bd, SUC_NE_lexicon_tags_2 },
    { 0x1ad362bd, SUC_NE_lexicon_tags_2 },
    { 0xa46362bf, SUC_NE_lexicon_tags_2 },
    { 0x4d4d62c0, SUC_NE_lexicon_tags_2 },
    { 0xaa7d62c0, SUC_NE_lexicon_tags_87 },
    { 0x103d62c2, SUC_NE_lexicon_tags_2 },
    { 0x6ce962c3, SUC_NE_lexicon_tags_2 },
    { 0xcad562bf, SUC_NE_lexicon_tags_2 },
    { 0x26b962c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45df62c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x403562c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cf62cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf62d4, SUC_NE_lexicon_tags_2 },
    { 0x13f762d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cd62d9, SUC_NE_lexicon_tags_2 },
    { 0xae3362da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdefd62dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a962df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a762e9, SUC_NE_lexicon_tags_0 },
    { 0x975d62ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99962ec, SUC_NE_lexicon_tags_2 },
    { 0x71d762ed, SUC_NE_lexicon_tags_2 },
    { 0xaed562ee, SUC_NE_lexicon_tags_32 },
    { 0x607d62ee, SUC_NE_lexicon_tags_2 },
    { 0xcba362f0, SUC_NE_lexicon_tags_2 },
    { 0xeee362f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680562fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6436300, SUC_NE_lexicon_tags_2 },
    { 0x7c496301, SUC_NE_lexicon_tags_2 },
    { 0x90316301, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb176305, SUC_NE_lexicon_tags_2 },
    { 0xba236306, SUC_NE_lexicon_tags_2 },
    { 0x768f6307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5e56309, SUC_NE_lexicon_tags_2 },
    { 0x5941630a, SUC_NE_lexicon_tags_2 },
    { 0xdfed630b, SUC_NE_lexicon_tags_10 },
    { 0xe156309, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0b6312, SUC_NE_lexicon_tags_2 },
    { 0xd2d56313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba76317, SUC_NE_lexicon_tags_13 },
    { 0x8b7d6318, SUC_NE_lexicon_tags_2 },
    { 0x50176319, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a1631e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76ff6320, SUC_NE_lexicon_tags_2 },
    { 0xf596320, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58b6325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf632a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c7632e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30ab6330, SUC_NE_lexicon_tags_2 },
    { 0xc1416331, SUC_NE_lexicon_tags_17 },
    { 0x28f6332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053633a, SUC_NE_lexicon_tags_2 },
    { 0xa841633b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c47633d, SUC_NE_lexicon_tags_2 },
    { 0xbfdf633e, SUC_NE_lexicon_tags_2 },
    { 0xc3eb633f, SUC_NE_lexicon_tags_2 },
    { 0x20bb6340, SUC_NE_lexicon_tags_2 },
    { 0x2df76341, SUC_NE_lexicon_tags_2 },
    { 0xf2b56342, SUC_NE_lexicon_tags_2 },
    { 0x1d9f6342, SUC_NE_lexicon_tags_2 },
    { 0x4336344, SUC_NE_lexicon_tags_2 },
    { 0x57796345, SUC_NE_lexicon_tags_7 },
    { 0x7d536346, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9176348, SUC_NE_lexicon_tags_2 },
    { 0x9a9d6348, SUC_NE_lexicon_tags_2 },
    { 0xecd3634a, SUC_NE_lexicon_tags_2 },
    { 0xd259634b, SUC_NE_lexicon_tags_118 },
    { 0x43e9634c, SUC_NE_lexicon_tags_2 },
    { 0x79ab6348, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x461634f, SUC_NE_lexicon_tags_2 },
    { 0xff196350, SUC_NE_lexicon_tags_2 },
    { 0x6c75634f, SUC_NE_lexicon_tags_2 },
    { 0xd2616352, SUC_NE_lexicon_tags_2 },
    { 0x8ba56350, SUC_NE_lexicon_tags_2 },
    { 0xaa936352, SUC_NE_lexicon_tags_2 },
    { 0xe5fd6354, SUC_NE_lexicon_tags_2 },
    { 0x676f6356, SUC_NE_lexicon_tags_2 },
    { 0xec3b6355, SUC_NE_lexicon_tags_2 },
    { 0x591f6351, SUC_NE_lexicon_tags_2 },
    { 0x704d6359, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5475635b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a1635f, SUC_NE_lexicon_tags_17 },
    { 0x9779635f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23f6363, SUC_NE_lexicon_tags_2 },
    { 0xa6136363, SUC_NE_lexicon_tags_2 },
    { 0x5b3f6365, SUC_NE_lexicon_tags_2 },
    { 0x15bb6366, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19316368, SUC_NE_lexicon_tags_2 },
    { 0x28b56369, SUC_NE_lexicon_tags_2 },
    { 0x88556369, SUC_NE_lexicon_tags_2 },
    { 0xab5636b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e7636e, SUC_NE_lexicon_tags_2 },
    { 0x28ab636f, SUC_NE_lexicon_tags_2 },
    { 0x5a196370, SUC_NE_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78236374, SUC_NE_lexicon_tags_4 },
    { 0x68d16375, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3b637a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x410f637d, SUC_NE_lexicon_tags_2 },
    { 0xb1e1637e, SUC_NE_lexicon_tags_7 },
    { 0x37637d, SUC_NE_lexicon_tags_2 },
    { 0x80c5637d, SUC_NE_lexicon_tags_2 },
    { 0xbc5f6381, SUC_NE_lexicon_tags_7 },
    { 0x96056381, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e56385, SUC_NE_lexicon_tags_2 },
    { 0x2d236386, SUC_NE_lexicon_tags_22 },
    { 0xe95f6387, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd427638f, SUC_NE_lexicon_tags_7 },
    { 0x513d6390, SUC_NE_lexicon_tags_2 },
    { 0x38516391, SUC_NE_lexicon_tags_2 },
    { 0x20316391, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6b6395, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e396398, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe65639a, SUC_NE_lexicon_tags_2 },
    { 0x426b639b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7795639d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4445639f, SUC_NE_lexicon_tags_2 },
    { 0x281763a0, SUC_NE_lexicon_tags_2 },
    { 0xec4f63a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaddd63a3, SUC_NE_lexicon_tags_2 },
    { 0x104f63a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5363a7, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0x686763a9, SUC_NE_lexicon_tags_2 },
    { 0x867363a9, SUC_NE_lexicon_tags_4 },
    { 0xaf9b63ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958563b0, SUC_NE_lexicon_tags_2 },
    { 0x1ef963b1, SUC_NE_lexicon_tags_2 },
    { 0x1b5d63b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80b963b4, SUC_NE_lexicon_tags_2 },
    { 0x496563b4, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0x31a363b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb763bb, SUC_NE_lexicon_tags_3 },
    { 0x22a763bc, SUC_NE_lexicon_tags_2 },
    { 0x7c4f63bd, SUC_NE_lexicon_tags_2 },
    { 0x152363bd, SUC_NE_lexicon_tags_4 },
    { 0xf35d63be, SUC_NE_lexicon_tags_2 },
    { 0x10f363bf, SUC_NE_lexicon_tags_2 },
    { 0x2b1d63c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b63c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1fb63ca, SUC_NE_lexicon_tags_1 },
    { 0xe4dd63cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7963ce, SUC_NE_lexicon_tags_2 },
    { 0xe8763ce, SUC_NE_lexicon_tags_2 },
    { 0xbafb63d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e9d63d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf63d8, SUC_NE_lexicon_tags_11 },
    { 0xa50963d9, SUC_NE_lexicon_tags_2 },
    { 0x32af63d8, SUC_NE_lexicon_tags_2 },
    { 0x729363db, SUC_NE_lexicon_tags_2 },
    { 0xe75563da, SUC_NE_lexicon_tags_2 },
    { 0x232d63dd, SUC_NE_lexicon_tags_2 },
    { 0x539163de, SUC_NE_lexicon_tags_2 },
    { 0xac3963db, SUC_NE_lexicon_tags_419 },
    { 0x6b163e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c3d63e2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xcae363e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214563e8, SUC_NE_lexicon_tags_2 },
    { 0x357363e9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410f63ed, SUC_NE_lexicon_tags_2 },
    { 0xca7163ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca2763f0, SUC_NE_lexicon_tags_2 },
    { 0xb1dd63f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x428363f4, SUC_NE_lexicon_tags_2 },
    { 0x346363f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13f63f8, SUC_NE_lexicon_tags_1 },
    { 0x219f63f9, SUC_NE_lexicon_tags_2 },
    { 0xeacf63fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x930163fc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x295d63fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21976401, SUC_NE_lexicon_tags_4 },
    { 0xb1df6402, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31af640a, SUC_NE_lexicon_tags_2 },
    { 0xaf45640b, SUC_NE_lexicon_tags_2 },
    { 0x6b9640c, SUC_NE_lexicon_tags_2 },
    { 0x635f640d, SUC_NE_lexicon_tags_17 },
    { 0xd7a3640b, SUC_NE_lexicon_tags_2 },
    { 0x452d640f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x681d6411, SUC_NE_lexicon_tags_4 },
    { 0xc4ab6412, SUC_NE_lexicon_tags_2 },
    { 0x29676412, SUC_NE_lexicon_tags_2 },
    { 0x5716414, SUC_NE_lexicon_tags_2 },
    { 0x66df6414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab36418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17cb641b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3285641e, SUC_NE_lexicon_tags_2 },
    { 0x5157641f, SUC_NE_lexicon_tags_2 },
    { 0xfe4b6420, SUC_NE_lexicon_tags_1 },
    { 0xa383641e, SUC_NE_lexicon_tags_2 },
    { 0x39376422, SUC_NE_lexicon_tags_2 },
    { 0x95916422, SUC_NE_lexicon_tags_417 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffb6426, SUC_NE_lexicon_tags_2 },
    { 0xaeff6427, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4796429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9576434, SUC_NE_lexicon_tags_2 },
    { 0x23b76434, SUC_NE_lexicon_tags_2 },
    { 0xece56434, SUC_NE_lexicon_tags_2 },
    { 0xde4d6437, SUC_NE_lexicon_tags_2 },
    { 0x4c0f6438, SUC_NE_lexicon_tags_2 },
    { 0xc5c56439, SUC_NE_lexicon_tags_32 },
    { 0x5e716435, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x361643c, SUC_NE_lexicon_tags_2 },
    { 0x6a21643d, SUC_NE_lexicon_tags_2 },
    { 0xf687643e, SUC_NE_lexicon_tags_34 },
    { 0xfb0d643d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ad6444, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f644a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bef644d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf76450, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x450f6452, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3256457, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef7645a, SUC_NE_lexicon_tags_2 },
    { 0x8cb7645b, SUC_NE_lexicon_tags_2 },
    { 0x5aab645b, SUC_NE_lexicon_tags_2 },
    { 0x8d59645b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c96461, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cc76463, SUC_NE_lexicon_tags_2 },
    { 0xff7d6463, SUC_NE_lexicon_tags_2 },
    { 0xab7f6465, SUC_NE_lexicon_tags_2 },
    { 0xecb6465, SUC_NE_lexicon_tags_122 },
    { 0xa52f6467, SUC_NE_lexicon_tags_2 },
    { 0x30c16468, SUC_NE_lexicon_tags_2 },
    { 0xd4c16468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3291646c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3275646e, SUC_NE_lexicon_tags_1 },
    { 0x3b97646e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ad6477, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb68f6479, SUC_NE_lexicon_tags_18 },
    { 0x1f35647a, SUC_NE_lexicon_tags_2 },
    { 0xc971647b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x396f647e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c76482, SUC_NE_lexicon_tags_2 },
    { 0xfd276483, SUC_NE_lexicon_tags_2 },
    { 0xb78d6482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fe16486, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67136488, SUC_NE_lexicon_tags_3 },
    { 0xe0056489, SUC_NE_lexicon_tags_2 },
    { 0x505648a, SUC_NE_lexicon_tags_2 },
    { 0x54756488, SUC_NE_lexicon_tags_24 },
    { 0xd919648a, SUC_NE_lexicon_tags_2 },
    { 0x9e95648d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac36490, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2696492, SUC_NE_lexicon_tags_2 },
    { 0xfad36493, SUC_NE_lexicon_tags_2 },
    { 0xa7456494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaff6499, SUC_NE_lexicon_tags_2 },
    { 0x19e1649a, SUC_NE_lexicon_tags_2 },
    { 0x8f7f649a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1705649e, SUC_NE_lexicon_tags_4 },
    { 0x9b21649f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfaa364a1, SUC_NE_lexicon_tags_2 },
    { 0xbc9564a2, SUC_NE_lexicon_tags_2 },
    { 0xe31564a2, SUC_NE_lexicon_tags_2 },
    { 0x407564a4, SUC_NE_lexicon_tags_2 },
    { 0xcac364a5, SUC_NE_lexicon_tags_2 },
    { 0x669764a4, SUC_NE_lexicon_tags_1 },
    { 0xc8e164a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f964ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25fd64ae, SUC_NE_lexicon_tags_2 },
    { 0x3e6164ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb43364b1, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12364b4, SUC_NE_lexicon_tags_2 },
    { 0x460f64b5, SUC_NE_lexicon_tags_2 },
    { 0xef3364b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4f64ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d64bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b64c1, SUC_NE_lexicon_tags_2 },
    { 0x1fb364c2, SUC_NE_lexicon_tags_2 },
    { 0x500f64c3, SUC_NE_lexicon_tags_2 },
    { 0x688164c3, SUC_NE_lexicon_tags_2 },
    { 0x950164c5, SUC_NE_lexicon_tags_2 },
    { 0xb67d64c6, SUC_NE_lexicon_tags_2 },
    { 0x476764c6, SUC_NE_lexicon_tags_2 },
    { 0x848564c3, SUC_NE_lexicon_tags_2 },
    { 0xc64764c9, SUC_NE_lexicon_tags_2 },
    { 0x83f164ca, SUC_NE_lexicon_tags_40 },
    { 0x69ed64c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85c164cd, SUC_NE_lexicon_tags_2 },
    { 0x15ff64ce, SUC_NE_lexicon_tags_2 },
    { 0x9d7164ce, SUC_NE_lexicon_tags_7 },
    { 0xaa7764d0, SUC_NE_lexicon_tags_2 },
    { 0x75c164d1, SUC_NE_lexicon_tags_2 },
    { 0x332f64d0, SUC_NE_lexicon_tags_2 },
    { 0xee4564d3, SUC_NE_lexicon_tags_2 },
    { 0x60ff64d3, SUC_NE_lexicon_tags_2 },
    { 0xf02b64d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4f64d7, SUC_NE_lexicon_tags_2 },
    { 0xdd8f64d8, SUC_NE_lexicon_tags_2 },
    { 0x3bf164d8, SUC_NE_lexicon_tags_2 },
    { 0x78a564d7, SUC_NE_lexicon_tags_2 },
    { 0x36b364da, SUC_NE_lexicon_tags_2 },
    { 0x2b8f64dc, SUC_NE_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f764df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf83164e1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be364ee, SUC_NE_lexicon_tags_2 },
    { 0xcf64ef, SUC_NE_lexicon_tags_2 },
    { 0x8e1b64f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5364f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c764fc, SUC_NE_lexicon_tags_225 },
    { 0, NULL },
    { 0x9c5b64fe, SUC_NE_lexicon_tags_2 },
    { 0xa7df64ff, SUC_NE_lexicon_tags_2 },
    { 0xe8eb64fe, SUC_NE_lexicon_tags_2 },
    { 0xa8bb64fe, SUC_NE_lexicon_tags_2 },
    { 0x67bd6501, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fff6504, SUC_NE_lexicon_tags_2 },
    { 0x7d716505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3816507, SUC_NE_lexicon_tags_2 },
    { 0x11156507, SUC_NE_lexicon_tags_4 },
    { 0xb5ad6509, SUC_NE_lexicon_tags_2 },
    { 0xd15650a, SUC_NE_lexicon_tags_2 },
    { 0xe0ed6507, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d16515, SUC_NE_lexicon_tags_2 },
    { 0x7096516, SUC_NE_lexicon_tags_196 },
    { 0xd7956517, SUC_NE_lexicon_tags_1 },
    { 0xb0876515, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4989651c, SUC_NE_lexicon_tags_2 },
    { 0x61eb651d, SUC_NE_lexicon_tags_2 },
    { 0xc729651e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f1d6520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cf76522, SUC_NE_lexicon_tags_2 },
    { 0xbf976522, SUC_NE_lexicon_tags_2 },
    { 0xaa2b6522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c36527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d3652a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876b652f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d6535, SUC_NE_lexicon_tags_3 },
    { 0x90d36536, SUC_NE_lexicon_tags_2 },
    { 0xdf636537, SUC_NE_lexicon_tags_7 },
    { 0x93f6538, SUC_NE_lexicon_tags_2 },
    { 0xc6b6538, SUC_NE_lexicon_tags_2 },
    { 0xade1653a, SUC_NE_lexicon_tags_2 },
    { 0x3d69653a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf47653d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4063653f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d16542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c56546, SUC_NE_lexicon_tags_2 },
    { 0xd1776546, SUC_NE_lexicon_tags_2 },
    { 0x2a076548, SUC_NE_lexicon_tags_2 },
    { 0x52856548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf6552, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xad596554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf756556, SUC_NE_lexicon_tags_2 },
    { 0xb3b96556, SUC_NE_lexicon_tags_2 },
    { 0xe6676558, SUC_NE_lexicon_tags_2 },
    { 0xab76558, SUC_NE_lexicon_tags_2 },
    { 0x20416558, SUC_NE_lexicon_tags_2 },
    { 0x3655655b, SUC_NE_lexicon_tags_2 },
    { 0x224d655c, SUC_NE_lexicon_tags_2 },
    { 0xc537655d, SUC_NE_lexicon_tags_2 },
    { 0xfc53655e, SUC_NE_lexicon_tags_2 },
    { 0x790d655f, SUC_NE_lexicon_tags_2 },
    { 0x9049655f, SUC_NE_lexicon_tags_2 },
    { 0x1d16561, SUC_NE_lexicon_tags_2 },
    { 0xb9176562, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95656a, SUC_NE_lexicon_tags_2 },
    { 0xf847656a, SUC_NE_lexicon_tags_4 },
    { 0xe265656c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3d656f, SUC_NE_lexicon_tags_2 },
    { 0x9d31656f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc696573, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51cb6575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaf657c, SUC_NE_lexicon_tags_2 },
    { 0xbcbb657c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4c1657f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afd6583, SUC_NE_lexicon_tags_2 },
    { 0xa6256583, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5db6586, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb658d, SUC_NE_lexicon_tags_14 },
    { 0x9971658d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37796592, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28f6599, SUC_NE_lexicon_tags_2 },
    { 0xeed7659a, SUC_NE_lexicon_tags_2 },
    { 0x8d4f659a, SUC_NE_lexicon_tags_2 },
    { 0x48c5659c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8eed659e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67f65a4, SUC_NE_lexicon_tags_2 },
    { 0x41d65a5, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x604365a8, SUC_NE_lexicon_tags_2 },
    { 0x9d2f65a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de365ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5365b3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedb65ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x216d65bc, SUC_NE_lexicon_tags_2 },
    { 0x3f0365bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef9565bf, SUC_NE_lexicon_tags_2 },
    { 0x4faf65c0, SUC_NE_lexicon_tags_2 },
    { 0x2e3d65c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23cd65c3, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xeecb65c5, SUC_NE_lexicon_tags_2 },
    { 0x64b965c6, SUC_NE_lexicon_tags_2 },
    { 0xf6bb65c7, SUC_NE_lexicon_tags_2 },
    { 0x4d1b65c8, SUC_NE_lexicon_tags_2 },
    { 0x85b65c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x975565cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03b65ce, SUC_NE_lexicon_tags_2 },
    { 0x82b565cf, SUC_NE_lexicon_tags_2 },
    { 0x57e765cf, SUC_NE_lexicon_tags_2 },
    { 0x462365d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c6b65d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22365d6, SUC_NE_lexicon_tags_2 },
    { 0xb77165d6, SUC_NE_lexicon_tags_2 },
    { 0x6e765d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf42d65da, SUC_NE_lexicon_tags_2 },
    { 0x60b165db, SUC_NE_lexicon_tags_2 },
    { 0x5e6965dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4d65e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cd65e5, SUC_NE_lexicon_tags_2 },
    { 0x6e6765e6, SUC_NE_lexicon_tags_2 },
    { 0x3f965e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dab65e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x201365ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1565ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b1d65f1, SUC_NE_lexicon_tags_2 },
    { 0x1bbf65f1, SUC_NE_lexicon_tags_2 },
    { 0xddeb65f1, SUC_NE_lexicon_tags_19 },
    { 0x98c565f2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe24965f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1b6601, SUC_NE_lexicon_tags_2 },
    { 0xe4f76601, SUC_NE_lexicon_tags_2 },
    { 0x2796603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c256605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d56609, SUC_NE_lexicon_tags_2 },
    { 0xb15b6609, SUC_NE_lexicon_tags_2 },
    { 0xa2fd660b, SUC_NE_lexicon_tags_2 },
    { 0xeaef660a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdbeb660e, SUC_NE_lexicon_tags_2 },
    { 0x5edd660f, SUC_NE_lexicon_tags_2 },
    { 0x7371660f, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb56613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ad6615, SUC_NE_lexicon_tags_13 },
    { 0x1b996616, SUC_NE_lexicon_tags_2 },
    { 0xa5ef6617, SUC_NE_lexicon_tags_7 },
    { 0x16b76618, SUC_NE_lexicon_tags_2 },
    { 0x59e16618, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53661d, SUC_NE_lexicon_tags_2 },
    { 0xcdbb661e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e4d6620, SUC_NE_lexicon_tags_2 },
    { 0xc0f76621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10036625, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29896631, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bf6634, SUC_NE_lexicon_tags_2 },
    { 0xe9e76635, SUC_NE_lexicon_tags_102 },
    { 0xd9576636, SUC_NE_lexicon_tags_2 },
    { 0xc9e56637, SUC_NE_lexicon_tags_2 },
    { 0x65d6638, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc093663a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff39663f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f36643, SUC_NE_lexicon_tags_2 },
    { 0xad4f6644, SUC_NE_lexicon_tags_2 },
    { 0x95456644, SUC_NE_lexicon_tags_2 },
    { 0xcfa96645, SUC_NE_lexicon_tags_2 },
    { 0xc6716643, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ff664a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bf664d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe156650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4236654, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0f6658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a76660, SUC_NE_lexicon_tags_2 },
    { 0x29dd6660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99ab6663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ead6665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b96668, SUC_NE_lexicon_tags_2 },
    { 0x7cfd6669, SUC_NE_lexicon_tags_2 },
    { 0x83fd6669, SUC_NE_lexicon_tags_34 },
    { 0x1fb1666b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4267666e, SUC_NE_lexicon_tags_2 },
    { 0xaddb666f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc896673, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x952d6675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6afd6677, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xde9b6679, SUC_NE_lexicon_tags_3 },
    { 0x5a2d6679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d81667d, SUC_NE_lexicon_tags_2 },
    { 0x6157667d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9f6685, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8756688, SUC_NE_lexicon_tags_2 },
    { 0xcddf6689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc9668c, SUC_NE_lexicon_tags_7 },
    { 0xd709668d, SUC_NE_lexicon_tags_4 },
    { 0x3e87668e, SUC_NE_lexicon_tags_2 },
    { 0xed8f668d, SUC_NE_lexicon_tags_2 },
    { 0xfc616690, SUC_NE_lexicon_tags_2 },
    { 0x7dfb6691, SUC_NE_lexicon_tags_2 },
    { 0x1a01668d, SUC_NE_lexicon_tags_2 },
    { 0xce5f6692, SUC_NE_lexicon_tags_2 },
    { 0x126f6694, SUC_NE_lexicon_tags_2 },
    { 0x4d536695, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92b6698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59b669b, SUC_NE_lexicon_tags_2 },
    { 0xff4d669c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xaca9669e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe38366a0, SUC_NE_lexicon_tags_2 },
    { 0x8a8566a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x309b66a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b5366a6, SUC_NE_lexicon_tags_4 },
    { 0x911366a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a366a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda766ac, SUC_NE_lexicon_tags_7 },
    { 0x5aa566ad, SUC_NE_lexicon_tags_2 },
    { 0xdc0966ad, SUC_NE_lexicon_tags_2 },
    { 0x757566ac, SUC_NE_lexicon_tags_17 },
    { 0x7c7366ae, SUC_NE_lexicon_tags_2 },
    { 0xfc3b66b1, SUC_NE_lexicon_tags_227 },
    { 0, NULL },
    { 0x5b566b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477b66ba, SUC_NE_lexicon_tags_2 },
    { 0xe68f66bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc63b66bd, SUC_NE_lexicon_tags_2 },
    { 0x7af66bd, SUC_NE_lexicon_tags_2 },
    { 0x330166bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80166c3, SUC_NE_lexicon_tags_19 },
    { 0xa41166c4, SUC_NE_lexicon_tags_2 },
    { 0x743366c5, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76366ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0d66d2, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd66d5, SUC_NE_lexicon_tags_3 },
    { 0xfd1366d6, SUC_NE_lexicon_tags_2 },
    { 0x85c166d6, SUC_NE_lexicon_tags_2 },
    { 0xe0ab66d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32d66da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec3166dc, SUC_NE_lexicon_tags_2 },
    { 0xf366dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e0366df, SUC_NE_lexicon_tags_2 },
    { 0x80b166e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3766e4, SUC_NE_lexicon_tags_2 },
    { 0x307f66e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc52d66e7, SUC_NE_lexicon_tags_2 },
    { 0xa30f66e7, SUC_NE_lexicon_tags_2 },
    { 0x517766e8, SUC_NE_lexicon_tags_2 },
    { 0x385f66ea, SUC_NE_lexicon_tags_2 },
    { 0x26f966eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab766f2, SUC_NE_lexicon_tags_2 },
    { 0x3b9566f3, SUC_NE_lexicon_tags_58 },
    { 0xa62b66f4, SUC_NE_lexicon_tags_2 },
    { 0x9966f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f966fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bbf66fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd2966ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc156704, SUC_NE_lexicon_tags_2 },
    { 0x5e1b6704, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc22b6707, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf096709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c9670c, SUC_NE_lexicon_tags_2 },
    { 0x1e9f670d, SUC_NE_lexicon_tags_7 },
    { 0xb379670e, SUC_NE_lexicon_tags_2 },
    { 0x9e9b670f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72196712, SUC_NE_lexicon_tags_3 },
    { 0xd3dd6713, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37d671a, SUC_NE_lexicon_tags_2 },
    { 0x5573671b, SUC_NE_lexicon_tags_2 },
    { 0xcf79671c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89b671f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd16723, SUC_NE_lexicon_tags_2 },
    { 0xa50b6724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82e76726, SUC_NE_lexicon_tags_2 },
    { 0x49c96726, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e49672d, SUC_NE_lexicon_tags_2 },
    { 0x4ec7672e, SUC_NE_lexicon_tags_2 },
    { 0xf28f672e, SUC_NE_lexicon_tags_2 },
    { 0xafb96730, SUC_NE_lexicon_tags_4 },
    { 0x28c96731, SUC_NE_lexicon_tags_2 },
    { 0x5f9f6732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a996734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd96b6736, SUC_NE_lexicon_tags_2 },
    { 0x694b6737, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a3673b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3463673d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7477673f, SUC_NE_lexicon_tags_2 },
    { 0xc55d673f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x345d6742, SUC_NE_lexicon_tags_8 },
    { 0x96ef6742, SUC_NE_lexicon_tags_13 },
    { 0xcec96743, SUC_NE_lexicon_tags_2 },
    { 0x72856745, SUC_NE_lexicon_tags_2 },
    { 0x622d6743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x427f6748, SUC_NE_lexicon_tags_4 },
    { 0x16e16749, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x30b3674b, SUC_NE_lexicon_tags_2 },
    { 0x620f674b, SUC_NE_lexicon_tags_2 },
    { 0x56ad674d, SUC_NE_lexicon_tags_2 },
    { 0x70f5674e, SUC_NE_lexicon_tags_2 },
    { 0x87f7674c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27b6752, SUC_NE_lexicon_tags_1 },
    { 0x1f496752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87456756, SUC_NE_lexicon_tags_2 },
    { 0xadef6756, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb7675a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85f675f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70376763, SUC_NE_lexicon_tags_2 },
    { 0x8fc36764, SUC_NE_lexicon_tags_2 },
    { 0xcf556763, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd676c, SUC_NE_lexicon_tags_2 },
    { 0xd901676d, SUC_NE_lexicon_tags_2 },
    { 0x5ecb676d, SUC_NE_lexicon_tags_2 },
    { 0xec15676f, SUC_NE_lexicon_tags_2 },
    { 0xb52d676f, SUC_NE_lexicon_tags_2 },
    { 0x6bfb6771, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13856777, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38216779, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x48ad677b, SUC_NE_lexicon_tags_1 },
    { 0x16eb677b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79b9677e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a336781, SUC_NE_lexicon_tags_2 },
    { 0x2f776782, SUC_NE_lexicon_tags_3 },
    { 0x96d16783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9516787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b5678a, SUC_NE_lexicon_tags_2 },
    { 0xa035678a, SUC_NE_lexicon_tags_2 },
    { 0xbbb678c, SUC_NE_lexicon_tags_2 },
    { 0x69eb678c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bc9678f, SUC_NE_lexicon_tags_2 },
    { 0x4fe96790, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e156794, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445f679a, SUC_NE_lexicon_tags_2 },
    { 0xcc47679a, SUC_NE_lexicon_tags_2 },
    { 0x9a2f679c, SUC_NE_lexicon_tags_2 },
    { 0x4603679d, SUC_NE_lexicon_tags_2 },
    { 0x1b35679c, SUC_NE_lexicon_tags_2 },
    { 0xfd75679f, SUC_NE_lexicon_tags_2 },
    { 0xf0a967a0, SUC_NE_lexicon_tags_3 },
    { 0x445367a1, SUC_NE_lexicon_tags_2 },
    { 0xf9e767a1, SUC_NE_lexicon_tags_2 },
    { 0xcfb5679c, SUC_NE_lexicon_tags_2 },
    { 0xe111679d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d367a8, SUC_NE_lexicon_tags_2 },
    { 0xf4a767a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x491767ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b67ae, SUC_NE_lexicon_tags_2 },
    { 0xf42767ae, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4567b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd50d67ba, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x125d67bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907367c0, SUC_NE_lexicon_tags_1 },
    { 0x94167c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb13367c3, SUC_NE_lexicon_tags_2 },
    { 0xc5ff67c4, SUC_NE_lexicon_tags_2 },
    { 0x677d67c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ec167c7, SUC_NE_lexicon_tags_2 },
    { 0xe5d67c8, SUC_NE_lexicon_tags_2 },
    { 0x95cd67c7, SUC_NE_lexicon_tags_3 },
    { 0xf9a167ca, SUC_NE_lexicon_tags_2 },
    { 0xcd0367cb, SUC_NE_lexicon_tags_2 },
    { 0xbda967cb, SUC_NE_lexicon_tags_2 },
    { 0x67d67c8, SUC_NE_lexicon_tags_2 },
    { 0x174767cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e9167d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3167d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2d67da, SUC_NE_lexicon_tags_2 },
    { 0xb24367db, SUC_NE_lexicon_tags_2 },
    { 0xe5f567db, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26767e2, SUC_NE_lexicon_tags_2 },
    { 0xe21167e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4e567e5, SUC_NE_lexicon_tags_2 },
    { 0xc1bf67e6, SUC_NE_lexicon_tags_2 },
    { 0x3f0967e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef967eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b167f0, SUC_NE_lexicon_tags_2 },
    { 0x749d67f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862167f5, SUC_NE_lexicon_tags_2 },
    { 0x3bc767f6, SUC_NE_lexicon_tags_2 },
    { 0xe9b367f5, SUC_NE_lexicon_tags_7 },
    { 0x308d67f8, SUC_NE_lexicon_tags_2 },
    { 0x776567f8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1fb67fb, SUC_NE_lexicon_tags_2 },
    { 0xfc6d67fc, SUC_NE_lexicon_tags_2 },
    { 0x159f67fc, SUC_NE_lexicon_tags_2 },
    { 0x1d6567fc, SUC_NE_lexicon_tags_2 },
    { 0xf6e767fd, SUC_NE_lexicon_tags_2 },
    { 0x58d96800, SUC_NE_lexicon_tags_2 },
    { 0x55116800, SUC_NE_lexicon_tags_2 },
    { 0x4c6367fd, SUC_NE_lexicon_tags_2 },
    { 0xc9b36803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed36806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cb6810, SUC_NE_lexicon_tags_2 },
    { 0x4716811, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c6d6813, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9ff6815, SUC_NE_lexicon_tags_2 },
    { 0x31776816, SUC_NE_lexicon_tags_2 },
    { 0x37a56816, SUC_NE_lexicon_tags_2 },
    { 0x36f6818, SUC_NE_lexicon_tags_2 },
    { 0xc6af6817, SUC_NE_lexicon_tags_32 },
    { 0x146d681a, SUC_NE_lexicon_tags_1 },
    { 0x900b681a, SUC_NE_lexicon_tags_19 },
    { 0xa049681b, SUC_NE_lexicon_tags_2 },
    { 0xcddf681a, SUC_NE_lexicon_tags_2 },
    { 0xe685681e, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd716821, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe876825, SUC_NE_lexicon_tags_4 },
    { 0x11fb6826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf99682d, SUC_NE_lexicon_tags_2 },
    { 0x3a7d682d, SUC_NE_lexicon_tags_7 },
    { 0xa6ef682f, SUC_NE_lexicon_tags_2 },
    { 0x16d76830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3811683a, SUC_NE_lexicon_tags_7 },
    { 0xe743683b, SUC_NE_lexicon_tags_2 },
    { 0x83b683c, SUC_NE_lexicon_tags_2 },
    { 0xfd8d683d, SUC_NE_lexicon_tags_2 },
    { 0xd935683e, SUC_NE_lexicon_tags_7 },
    { 0x98f3683e, SUC_NE_lexicon_tags_4 },
    { 0x31ef683f, SUC_NE_lexicon_tags_2 },
    { 0xf703683e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1276843, SUC_NE_lexicon_tags_2 },
    { 0x15076843, SUC_NE_lexicon_tags_2 },
    { 0x7f696843, SUC_NE_lexicon_tags_2 },
    { 0xfb36846, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5684c, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xf55f684e, SUC_NE_lexicon_tags_2 },
    { 0x3c03684e, SUC_NE_lexicon_tags_2 },
    { 0x3c896850, SUC_NE_lexicon_tags_2 },
    { 0xd4476851, SUC_NE_lexicon_tags_2 },
    { 0xceb96852, SUC_NE_lexicon_tags_2 },
    { 0x9316851, SUC_NE_lexicon_tags_2 },
    { 0x88bf6853, SUC_NE_lexicon_tags_2 },
    { 0xcf216855, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bdd6857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f33685e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce916860, SUC_NE_lexicon_tags_18 },
    { 0xcc756861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20136863, SUC_NE_lexicon_tags_157 },
    { 0, NULL },
    { 0x98cb6865, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93af686a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25196874, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11596877, SUC_NE_lexicon_tags_2 },
    { 0x2e3d6878, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ef9687a, SUC_NE_lexicon_tags_2 },
    { 0xabf1687b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f3687f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda36882, SUC_NE_lexicon_tags_2 },
    { 0xaacf6883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50e76885, SUC_NE_lexicon_tags_2 },
    { 0xbc596886, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad196888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c3688e, SUC_NE_lexicon_tags_2 },
    { 0x98b1688f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb476894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15cb6896, SUC_NE_lexicon_tags_2 },
    { 0x4f356896, SUC_NE_lexicon_tags_2 },
    { 0x670f6898, SUC_NE_lexicon_tags_2 },
    { 0xb67d6897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb74f689b, SUC_NE_lexicon_tags_2 },
    { 0x673f689b, SUC_NE_lexicon_tags_2 },
    { 0xd9f3689d, SUC_NE_lexicon_tags_2 },
    { 0x3e31689c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x955568a1, SUC_NE_lexicon_tags_2 },
    { 0x411568a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c2168a4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce968a7, SUC_NE_lexicon_tags_0 },
    { 0xcccd68a8, SUC_NE_lexicon_tags_36 },
    { 0xa25368a8, SUC_NE_lexicon_tags_17 },
    { 0x8a2f68a7, SUC_NE_lexicon_tags_2 },
    { 0xfed568a7, SUC_NE_lexicon_tags_2 },
    { 0xf12568ac, SUC_NE_lexicon_tags_2 },
    { 0x536368aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec368b0, SUC_NE_lexicon_tags_3 },
    { 0x8be368b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbc568b3, SUC_NE_lexicon_tags_2 },
    { 0xab8168b4, SUC_NE_lexicon_tags_2 },
    { 0x615568b5, SUC_NE_lexicon_tags_2 },
    { 0x9ebb68b6, SUC_NE_lexicon_tags_2 },
    { 0xbadf68b4, SUC_NE_lexicon_tags_2 },
    { 0xc31f68b3, SUC_NE_lexicon_tags_2 },
    { 0x85bf68b9, SUC_NE_lexicon_tags_2 },
    { 0xc50d68ba, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acf68bd, SUC_NE_lexicon_tags_2 },
    { 0xc8cd68be, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x3f3168c0, SUC_NE_lexicon_tags_2 },
    { 0x5b6168c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d68c5, SUC_NE_lexicon_tags_2 },
    { 0x842368c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d68c9, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff168ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f768d1, SUC_NE_lexicon_tags_1 },
    { 0x2a7368d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4568d5, SUC_NE_lexicon_tags_2 },
    { 0x1e6168d6, SUC_NE_lexicon_tags_2 },
    { 0x9c4768d6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xea7368d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a4768db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x676768dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x416568df, SUC_NE_lexicon_tags_2 },
    { 0x220d68e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x876168e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f368e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18f568e9, SUC_NE_lexicon_tags_2 },
    { 0x694568ea, SUC_NE_lexicon_tags_2 },
    { 0x652368ea, SUC_NE_lexicon_tags_2 },
    { 0x54ed68ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51b68f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaf68f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27368f9, SUC_NE_lexicon_tags_2 },
    { 0x36f168f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec968fe, SUC_NE_lexicon_tags_2 },
    { 0xae7f68ff, SUC_NE_lexicon_tags_2 },
    { 0x98956900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f1b6902, SUC_NE_lexicon_tags_30 },
    { 0x22c56902, SUC_NE_lexicon_tags_2 },
    { 0x4ead6904, SUC_NE_lexicon_tags_2 },
    { 0x45f56905, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0f6908, SUC_NE_lexicon_tags_2 },
    { 0x42ff6909, SUC_NE_lexicon_tags_2 },
    { 0x32156909, SUC_NE_lexicon_tags_2 },
    { 0x4e3b690b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bbf690d, SUC_NE_lexicon_tags_2 },
    { 0xdcf5690d, SUC_NE_lexicon_tags_2 },
    { 0x5919690e, SUC_NE_lexicon_tags_2 },
    { 0x5929690d, SUC_NE_lexicon_tags_2 },
    { 0x1f876911, SUC_NE_lexicon_tags_2 },
    { 0x640d6912, SUC_NE_lexicon_tags_2 },
    { 0x4ffb6913, SUC_NE_lexicon_tags_2 },
    { 0xffd6912, SUC_NE_lexicon_tags_2 },
    { 0x7d196915, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55776918, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf691f, SUC_NE_lexicon_tags_2 },
    { 0xf1436920, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8b6923, SUC_NE_lexicon_tags_2 },
    { 0x92c36923, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x484f6926, SUC_NE_lexicon_tags_75 },
    { 0xed216927, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3692b, SUC_NE_lexicon_tags_2 },
    { 0x3fcf692c, SUC_NE_lexicon_tags_2 },
    { 0x2de7692d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe125692f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87416931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3b6934, SUC_NE_lexicon_tags_2 },
    { 0x10396934, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b1d6937, SUC_NE_lexicon_tags_2 },
    { 0x4c956937, SUC_NE_lexicon_tags_2 },
    { 0x8db56937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb277693b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542d693f, SUC_NE_lexicon_tags_2 },
    { 0x5149693f, SUC_NE_lexicon_tags_2 },
    { 0xbacb6941, SUC_NE_lexicon_tags_2 },
    { 0x5ea96941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2a76944, SUC_NE_lexicon_tags_5 },
    { 0xd9fd6945, SUC_NE_lexicon_tags_2 },
    { 0x8f7d6946, SUC_NE_lexicon_tags_2 },
    { 0xba5b6947, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xd0d56949, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a37694c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48b694f, SUC_NE_lexicon_tags_2 },
    { 0x10436950, SUC_NE_lexicon_tags_2 },
    { 0x3d4d694f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edd6956, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e936958, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3375695c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa56960, SUC_NE_lexicon_tags_4 },
    { 0xdc5b6961, SUC_NE_lexicon_tags_2 },
    { 0xe8f96961, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6736967, SUC_NE_lexicon_tags_2 },
    { 0x3e3d6967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4d3696a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51696d, SUC_NE_lexicon_tags_2 },
    { 0x9aab696e, SUC_NE_lexicon_tags_2 },
    { 0x8a73696f, SUC_NE_lexicon_tags_2 },
    { 0xe02b6970, SUC_NE_lexicon_tags_18 },
    { 0x8ddf696d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1656973, SUC_NE_lexicon_tags_2 },
    { 0xf65b6974, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46836979, SUC_NE_lexicon_tags_2 },
    { 0x976f697a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d9697e, SUC_NE_lexicon_tags_2 },
    { 0x7349697f, SUC_NE_lexicon_tags_2 },
    { 0x5ab5697e, SUC_NE_lexicon_tags_2 },
    { 0x6ea76981, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4df6983, SUC_NE_lexicon_tags_2 },
    { 0x9c4d6984, SUC_NE_lexicon_tags_2 },
    { 0xc62d6985, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa896987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7698c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd1698e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1f6998, SUC_NE_lexicon_tags_2 },
    { 0xad16999, SUC_NE_lexicon_tags_2 },
    { 0x970d699a, SUC_NE_lexicon_tags_3 },
    { 0x7ac76999, SUC_NE_lexicon_tags_2 },
    { 0x545f699c, SUC_NE_lexicon_tags_2 },
    { 0xff55699d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf69a0, SUC_NE_lexicon_tags_2 },
    { 0x608369a1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65169a7, SUC_NE_lexicon_tags_2 },
    { 0xcc0569a8, SUC_NE_lexicon_tags_2 },
    { 0x8f5969a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad9d69ab, SUC_NE_lexicon_tags_2 },
    { 0x665969ac, SUC_NE_lexicon_tags_2 },
    { 0xa3169ab, SUC_NE_lexicon_tags_34 },
    { 0xe06769ab, SUC_NE_lexicon_tags_2 },
    { 0x3c7969ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x486769b2, SUC_NE_lexicon_tags_3 },
    { 0x52eb69b3, SUC_NE_lexicon_tags_2 },
    { 0xfd3369b4, SUC_NE_lexicon_tags_2 },
    { 0xe8bb69b2, SUC_NE_lexicon_tags_2 },
    { 0xe15369b6, SUC_NE_lexicon_tags_11 },
    { 0x952b69b7, SUC_NE_lexicon_tags_2 },
    { 0xb11569b6, SUC_NE_lexicon_tags_2 },
    { 0x7b3769b9, SUC_NE_lexicon_tags_8 },
    { 0x86ad69ba, SUC_NE_lexicon_tags_2 },
    { 0x823369ba, SUC_NE_lexicon_tags_2 },
    { 0x3b8369bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb5769be, SUC_NE_lexicon_tags_2 },
    { 0xee8169bf, SUC_NE_lexicon_tags_2 },
    { 0x632b69c0, SUC_NE_lexicon_tags_34 },
    { 0x7ffd69c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b9569c3, SUC_NE_lexicon_tags_2 },
    { 0x3c3d69c3, SUC_NE_lexicon_tags_2 },
    { 0xbef69c4, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3969cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80d69d2, SUC_NE_lexicon_tags_2 },
    { 0x30569d2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7769d6, SUC_NE_lexicon_tags_2 },
    { 0x679369d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3169db, SUC_NE_lexicon_tags_2 },
    { 0x85b369db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2369df, SUC_NE_lexicon_tags_2 },
    { 0x682b69df, SUC_NE_lexicon_tags_2 },
    { 0x1d2769e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fe769e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x896f69e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f569ea, SUC_NE_lexicon_tags_2 },
    { 0xe36369ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b169ee, SUC_NE_lexicon_tags_2 },
    { 0x53a169ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17769f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33df69f5, SUC_NE_lexicon_tags_3 },
    { 0x606969f6, SUC_NE_lexicon_tags_17 },
    { 0x669369f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2969fa, SUC_NE_lexicon_tags_2 },
    { 0x43569fb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x876969fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb6a03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c676a05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70b96a07, SUC_NE_lexicon_tags_2 },
    { 0xe9c36a07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e16a0d, SUC_NE_lexicon_tags_2 },
    { 0x27936a0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca56a12, SUC_NE_lexicon_tags_2 },
    { 0x8fdb6a13, SUC_NE_lexicon_tags_2 },
    { 0x90f56a14, SUC_NE_lexicon_tags_2 },
    { 0xd74d6a15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4996a18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44996a25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39236a27, SUC_NE_lexicon_tags_2 },
    { 0x62cf6a27, SUC_NE_lexicon_tags_4 },
    { 0xa4d96a29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2256a2e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xce716a30, SUC_NE_lexicon_tags_2 },
    { 0x27676a31, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xba76a33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b656a36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e616a39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcad16a3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f816a3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x919b6a3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab5f6a41, SUC_NE_lexicon_tags_0 },
    { 0xaf016a41, SUC_NE_lexicon_tags_2 },
    { 0x9be16a41, SUC_NE_lexicon_tags_2 },
    { 0xe43b6a44, SUC_NE_lexicon_tags_2 },
    { 0x1dc16a45, SUC_NE_lexicon_tags_2 },
    { 0xed96a45, SUC_NE_lexicon_tags_2 },
    { 0x47536a45, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f16a4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5836a4c, SUC_NE_lexicon_tags_2 },
    { 0x93836a4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb6a50, SUC_NE_lexicon_tags_2 },
    { 0xff216a50, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xdd1b6a53, SUC_NE_lexicon_tags_2 },
    { 0x65516a54, SUC_NE_lexicon_tags_2 },
    { 0xab916a55, SUC_NE_lexicon_tags_3 },
    { 0x4aad6a56, SUC_NE_lexicon_tags_3 },
    { 0xfa056a57, SUC_NE_lexicon_tags_2 },
    { 0x52836a56, SUC_NE_lexicon_tags_2 },
    { 0xc6876a59, SUC_NE_lexicon_tags_2 },
    { 0x4e0d6a58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33d76a5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2f6a5f, SUC_NE_lexicon_tags_36 },
    { 0x636d6a5f, SUC_NE_lexicon_tags_2 },
    { 0xa61d6a5f, SUC_NE_lexicon_tags_7 },
    { 0xd4116a62, SUC_NE_lexicon_tags_2 },
    { 0xc5fb6a63, SUC_NE_lexicon_tags_7 },
    { 0x74af6a62, SUC_NE_lexicon_tags_17 },
    { 0xc9c76a65, SUC_NE_lexicon_tags_2 },
    { 0x1daf6a66, SUC_NE_lexicon_tags_2 },
    { 0xf17f6a67, SUC_NE_lexicon_tags_2 },
    { 0xf3676a68, SUC_NE_lexicon_tags_11 },
    { 0xf6476a63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf76a6f, SUC_NE_lexicon_tags_2 },
    { 0x93236a70, SUC_NE_lexicon_tags_2 },
    { 0x8e16a70, SUC_NE_lexicon_tags_2 },
    { 0xfc0d6a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb8f6a74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaf6a78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x396d6a7b, SUC_NE_lexicon_tags_2 },
    { 0x843d6a7b, SUC_NE_lexicon_tags_2 },
    { 0x44196a7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ad6a80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67f96a82, SUC_NE_lexicon_tags_2 },
    { 0xa98b6a83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ced6a85, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673b6a8d, SUC_NE_lexicon_tags_2 },
    { 0x59776a8d, SUC_NE_lexicon_tags_2 },
    { 0xc556a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e116a91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93376a93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15816a95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd6a98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60636a9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39696aa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84b6aa5, SUC_NE_lexicon_tags_2 },
    { 0x83dd6aa6, SUC_NE_lexicon_tags_2 },
    { 0xf5b56aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a376aab, SUC_NE_lexicon_tags_2 },
    { 0xd4996aab, SUC_NE_lexicon_tags_3 },
    { 0x53bd6aac, SUC_NE_lexicon_tags_2 },
    { 0xbfcd6aae, SUC_NE_lexicon_tags_2 },
    { 0x75596aae, SUC_NE_lexicon_tags_2 },
    { 0xd35b6aac, SUC_NE_lexicon_tags_36 },
    { 0x697d6ab1, SUC_NE_lexicon_tags_2 },
    { 0x7a6f6ab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe536ab6, SUC_NE_lexicon_tags_5 },
    { 0x75ff6ab7, SUC_NE_lexicon_tags_2 },
    { 0xa1bb6ab8, SUC_NE_lexicon_tags_2 },
    { 0x73d56ab9, SUC_NE_lexicon_tags_2 },
    { 0xff0d6ab9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98f6abd, SUC_NE_lexicon_tags_2 },
    { 0xcb7d6abe, SUC_NE_lexicon_tags_0 },
    { 0x4b8b6abf, SUC_NE_lexicon_tags_339 },
    { 0xa47f6abf, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6cdd6ac2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3d6ac5, SUC_NE_lexicon_tags_2 },
    { 0x68756ac6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b996acc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c536ad0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc276ad6, SUC_NE_lexicon_tags_2 },
    { 0xf6f6ad7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb9f6ad9, SUC_NE_lexicon_tags_2 },
    { 0xb0eb6ad9, SUC_NE_lexicon_tags_2 },
    { 0x515b6adb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1676adf, SUC_NE_lexicon_tags_2 },
    { 0x49d96ae0, SUC_NE_lexicon_tags_2 },
    { 0x171f6ae1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a56ae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb16ae9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4496af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93f6af6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48176afb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27b6afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ab16aff, SUC_NE_lexicon_tags_2 },
    { 0xbfd36aff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf816b04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68cb6b06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aff6b09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85b16b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a16b11, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f936b13, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x8e636b15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x527d6b17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19b6b1a, SUC_NE_lexicon_tags_2 },
    { 0x87f36b1a, SUC_NE_lexicon_tags_2 },
    { 0x66a56b1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8536b1f, SUC_NE_lexicon_tags_2 },
    { 0x9aed6b20, SUC_NE_lexicon_tags_2 },
    { 0xe43b6b21, SUC_NE_lexicon_tags_2 },
    { 0x966b6b21, SUC_NE_lexicon_tags_2 },
    { 0x6f9f6b23, SUC_NE_lexicon_tags_2 },
    { 0x25336b24, SUC_NE_lexicon_tags_7 },
    { 0x728d6b25, SUC_NE_lexicon_tags_23 },
    { 0xa4536b23, SUC_NE_lexicon_tags_2 },
    { 0xd2916b26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0b6b2d, SUC_NE_lexicon_tags_2 },
    { 0x3ffd6b2d, SUC_NE_lexicon_tags_2 },
    { 0xab836b2f, SUC_NE_lexicon_tags_2 },
    { 0x58956b30, SUC_NE_lexicon_tags_2 },
    { 0x8bb6b31, SUC_NE_lexicon_tags_2 },
    { 0xdc36b32, SUC_NE_lexicon_tags_2 },
    { 0xc76d6b33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7056b36, SUC_NE_lexicon_tags_3 },
    { 0x48b6b37, SUC_NE_lexicon_tags_0 },
    { 0x19676b36, SUC_NE_lexicon_tags_2 },
    { 0xf856b39, SUC_NE_lexicon_tags_2 },
    { 0xfaf36b38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9af56b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a16b41, SUC_NE_lexicon_tags_2 },
    { 0xb80b6b41, SUC_NE_lexicon_tags_2 },
    { 0x64a36b43, SUC_NE_lexicon_tags_2 },
    { 0xb2256b44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98236b47, SUC_NE_lexicon_tags_2 },
    { 0xb276b47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9d96b4a, SUC_NE_lexicon_tags_2 },
    { 0x97856b4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55b6b4e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e56b53, SUC_NE_lexicon_tags_2 },
    { 0x3f9d6b53, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x43596b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0096b59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21556b5c, SUC_NE_lexicon_tags_2 },
    { 0x8cc36b5d, SUC_NE_lexicon_tags_2 },
    { 0xf0796b5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8da76b60, SUC_NE_lexicon_tags_2 },
    { 0xc5fb6b61, SUC_NE_lexicon_tags_2 },
    { 0xbd576b62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d6b64, SUC_NE_lexicon_tags_2 },
    { 0x41bf6b64, SUC_NE_lexicon_tags_2 },
    { 0x4a7d6b66, SUC_NE_lexicon_tags_2 },
    { 0xc9556b67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97b56b69, SUC_NE_lexicon_tags_2 },
    { 0x286f6b6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d76b6d, SUC_NE_lexicon_tags_2 },
    { 0xb9ad6b6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf676b71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9f6b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12b6b7b, SUC_NE_lexicon_tags_2 },
    { 0xd32f6b7c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbd6b84, SUC_NE_lexicon_tags_2 },
    { 0xc876b84, SUC_NE_lexicon_tags_72 },
    { 0, NULL },
    { 0x919b6b87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fb6b89, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57856b8e, SUC_NE_lexicon_tags_5 },
    { 0x3eef6b8e, SUC_NE_lexicon_tags_4 },
    { 0x7b156b90, SUC_NE_lexicon_tags_2 },
    { 0x50056b8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b516b96, SUC_NE_lexicon_tags_2 },
    { 0x84bf6b97, SUC_NE_lexicon_tags_2 },
    { 0x71456b98, SUC_NE_lexicon_tags_2 },
    { 0x2dcd6b98, SUC_NE_lexicon_tags_2 },
    { 0x26b76b9a, SUC_NE_lexicon_tags_2 },
    { 0x43a36b99, SUC_NE_lexicon_tags_2 },
    { 0x72ef6b9c, SUC_NE_lexicon_tags_2 },
    { 0xf2316b99, SUC_NE_lexicon_tags_2 },
    { 0x50016b9e, SUC_NE_lexicon_tags_4 },
    { 0x95b16b9e, SUC_NE_lexicon_tags_2 },
    { 0x8e356ba0, SUC_NE_lexicon_tags_2 },
    { 0x3ed56ba0, SUC_NE_lexicon_tags_2 },
    { 0xfbdd6b99, SUC_NE_lexicon_tags_1 },
    { 0x25d16ba3, SUC_NE_lexicon_tags_3 },
    { 0x598f6b9a, SUC_NE_lexicon_tags_2 },
    { 0xaf676ba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b076ba7, SUC_NE_lexicon_tags_2 },
    { 0x5a3f6ba8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b6f6baa, SUC_NE_lexicon_tags_2 },
    { 0xe0d16bab, SUC_NE_lexicon_tags_2 },
    { 0x8f4f6bac, SUC_NE_lexicon_tags_3 },
    { 0xa0376bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10c76baf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c96bb3, SUC_NE_lexicon_tags_2 },
    { 0x812b6bb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63cb6bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58496bbb, SUC_NE_lexicon_tags_2 },
    { 0xb61b6bbb, SUC_NE_lexicon_tags_2 },
    { 0xc4796bbd, SUC_NE_lexicon_tags_32 },
    { 0x130f6bbd, SUC_NE_lexicon_tags_2 },
    { 0x679d6bbf, SUC_NE_lexicon_tags_2 },
    { 0x52ab6bbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fad6bc2, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae216bc7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f16bcb, SUC_NE_lexicon_tags_2 },
    { 0xa1036bcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dad6bcf, SUC_NE_lexicon_tags_2 },
    { 0xbed6bcf, SUC_NE_lexicon_tags_2 },
    { 0x6fe76bd1, SUC_NE_lexicon_tags_2 },
    { 0x27d16bd1, SUC_NE_lexicon_tags_2 },
    { 0xd4096bd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x319b6bd5, SUC_NE_lexicon_tags_2 },
    { 0x798f6bd6, SUC_NE_lexicon_tags_2 },
    { 0x51016bd7, SUC_NE_lexicon_tags_2 },
    { 0xeac76bd7, SUC_NE_lexicon_tags_2 },
    { 0x26816bd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd816bdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e56be0, SUC_NE_lexicon_tags_2 },
    { 0x95836be1, SUC_NE_lexicon_tags_2 },
    { 0x93e36be2, SUC_NE_lexicon_tags_2 },
    { 0x6c236be0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cb6bea, SUC_NE_lexicon_tags_2 },
    { 0xb56d6beb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f96bf4, SUC_NE_lexicon_tags_2 },
    { 0xc3e36bf5, SUC_NE_lexicon_tags_2 },
    { 0x8cb56bf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fc96bf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b6bfc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf0bb6bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56356c01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb28d6c03, SUC_NE_lexicon_tags_2 },
    { 0x67796c04, SUC_NE_lexicon_tags_2 },
    { 0x21176c05, SUC_NE_lexicon_tags_2 },
    { 0x3b216c06, SUC_NE_lexicon_tags_2 },
    { 0x651d6c07, SUC_NE_lexicon_tags_2 },
    { 0x6e2b6c07, SUC_NE_lexicon_tags_2 },
    { 0x840b6c05, SUC_NE_lexicon_tags_2 },
    { 0x14436c0a, SUC_NE_lexicon_tags_2 },
    { 0x3c76c05, SUC_NE_lexicon_tags_2 },
    { 0x8b076c0a, SUC_NE_lexicon_tags_19 },
    { 0x73836c04, SUC_NE_lexicon_tags_2 },
    { 0xa3416c09, SUC_NE_lexicon_tags_2 },
    { 0x18336c04, SUC_NE_lexicon_tags_7 },
    { 0xb5416c10, SUC_NE_lexicon_tags_2 },
    { 0xf92b6c11, SUC_NE_lexicon_tags_0 },
    { 0x1f5b6c12, SUC_NE_lexicon_tags_2 },
    { 0xf9bb6c13, SUC_NE_lexicon_tags_2 },
    { 0x24636c14, SUC_NE_lexicon_tags_2 },
    { 0x8ffb6c14, SUC_NE_lexicon_tags_2 },
    { 0xa35f6c16, SUC_NE_lexicon_tags_1 },
    { 0xb0b96c15, SUC_NE_lexicon_tags_4 },
    { 0xcf5b6c13, SUC_NE_lexicon_tags_2 },
    { 0x7eef6c15, SUC_NE_lexicon_tags_427 },
    { 0x13ab6c05, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeb6c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3056c25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b0f6c27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d56c2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbab16c31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3716c34, SUC_NE_lexicon_tags_2 },
    { 0x4d116c35, SUC_NE_lexicon_tags_2 },
    { 0x87a56c35, SUC_NE_lexicon_tags_2 },
    { 0x37fd6c36, SUC_NE_lexicon_tags_2 },
    { 0x82d6c38, SUC_NE_lexicon_tags_2 },
    { 0x4c916c39, SUC_NE_lexicon_tags_65 },
    { 0, NULL },
    { 0xc2096c3b, SUC_NE_lexicon_tags_2 },
    { 0xbb336c3c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x62dd6c3e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4176c41, SUC_NE_lexicon_tags_2 },
    { 0xd3936c42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x482b6c44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16976c4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d6c52, SUC_NE_lexicon_tags_2 },
    { 0xe5156c53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cb6c59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3f6c60, SUC_NE_lexicon_tags_2 },
    { 0x5a0b6c60, SUC_NE_lexicon_tags_2 },
    { 0x7a556c62, SUC_NE_lexicon_tags_2 },
    { 0x456f6c61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32816c65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5d6c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2496c6d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a5d6c6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadf16c71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c796c73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2536c77, SUC_NE_lexicon_tags_2 },
    { 0x214b6c78, SUC_NE_lexicon_tags_2 },
    { 0x58cb6c78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ec96c7b, SUC_NE_lexicon_tags_14 },
    { 0xd1dd6c7c, SUC_NE_lexicon_tags_2 },
    { 0x98096c7b, SUC_NE_lexicon_tags_2 },
    { 0xa6a16c7e, SUC_NE_lexicon_tags_425 },
    { 0x4c616c7e, SUC_NE_lexicon_tags_2 },
    { 0x2db16c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ed6c82, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5ef6c84, SUC_NE_lexicon_tags_2 },
    { 0x80696c84, SUC_NE_lexicon_tags_2 },
    { 0xde696c86, SUC_NE_lexicon_tags_4 },
    { 0xfd056c87, SUC_NE_lexicon_tags_2 },
    { 0x338b6c84, SUC_NE_lexicon_tags_2 },
    { 0x83c56c87, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x374b6c8c, SUC_NE_lexicon_tags_2 },
    { 0x7af6c8d, SUC_NE_lexicon_tags_2 },
    { 0xb5e16c8e, SUC_NE_lexicon_tags_3 },
    { 0xceed6c8f, SUC_NE_lexicon_tags_2 },
    { 0x24c76c8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eb6c93, SUC_NE_lexicon_tags_2 },
    { 0xcef16c94, SUC_NE_lexicon_tags_2 },
    { 0x3c136c95, SUC_NE_lexicon_tags_2 },
    { 0x48556c94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2d6c99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94bd6c9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f856c9d, SUC_NE_lexicon_tags_11 },
    { 0x85b6c9e, SUC_NE_lexicon_tags_2 },
    { 0x1aa76c9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb6ca8, SUC_NE_lexicon_tags_3 },
    { 0x4be16ca8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77916cab, SUC_NE_lexicon_tags_2 },
    { 0x9476cac, SUC_NE_lexicon_tags_2 },
    { 0x73796cab, SUC_NE_lexicon_tags_2 },
    { 0xdfbd6cae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9c36cb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6216cb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dbf6cb7, SUC_NE_lexicon_tags_2 },
    { 0x7dc96cb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ff6cbc, SUC_NE_lexicon_tags_7 },
    { 0xf27b6cbc, SUC_NE_lexicon_tags_2 },
    { 0x6b56cbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x459d6cc0, SUC_NE_lexicon_tags_1 },
    { 0x23dd6cc0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e16cc5, SUC_NE_lexicon_tags_2 },
    { 0x91616cc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f756cc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9356ccd, SUC_NE_lexicon_tags_2 },
    { 0x70396cce, SUC_NE_lexicon_tags_3 },
    { 0x70df6cce, SUC_NE_lexicon_tags_2 },
    { 0x40f36cd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba076cd2, SUC_NE_lexicon_tags_2 },
    { 0x33ff6cd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62936cd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed6cd9, SUC_NE_lexicon_tags_2 },
    { 0x6b936cda, SUC_NE_lexicon_tags_2 },
    { 0x40256cdb, SUC_NE_lexicon_tags_195 },
    { 0x913d6cd9, SUC_NE_lexicon_tags_1 },
    { 0xdc2d6cdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa7d6cdf, SUC_NE_lexicon_tags_2 },
    { 0xc2ef6ce0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21eb6ce5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47b6ce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36b36ceb, SUC_NE_lexicon_tags_2 },
    { 0x359d6ceb, SUC_NE_lexicon_tags_2 },
    { 0xd2f76ced, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499f6cf4, SUC_NE_lexicon_tags_2 },
    { 0x2e3d6cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0376cf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fd6cfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da16d00, SUC_NE_lexicon_tags_2 },
    { 0xedc16d01, SUC_NE_lexicon_tags_2 },
    { 0xdeb76d02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69156d05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4116d07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22ab6d09, SUC_NE_lexicon_tags_2 },
    { 0x19b16d0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f56d0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cb6d17, SUC_NE_lexicon_tags_2 },
    { 0xcc116d18, SUC_NE_lexicon_tags_2 },
    { 0x56f76d19, SUC_NE_lexicon_tags_1 },
    { 0x8d416d1a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f6d1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f296d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a8f6d25, SUC_NE_lexicon_tags_49 },
    { 0x6d6d6d26, SUC_NE_lexicon_tags_3 },
    { 0x7a716d26, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x214d6d29, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff6d2d, SUC_NE_lexicon_tags_2 },
    { 0x9eb56d2e, SUC_NE_lexicon_tags_2 },
    { 0x22bd6d2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82f6d32, SUC_NE_lexicon_tags_2 },
    { 0x38cd6d33, SUC_NE_lexicon_tags_2 },
    { 0x16576d34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f356d39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cf16d3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f96d3e, SUC_NE_lexicon_tags_2 },
    { 0xbb16d3f, SUC_NE_lexicon_tags_2 },
    { 0xeb336d40, SUC_NE_lexicon_tags_2 },
    { 0xa8db6d40, SUC_NE_lexicon_tags_2 },
    { 0x304d6d41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9b6d47, SUC_NE_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a56d4a, SUC_NE_lexicon_tags_2 },
    { 0xfd1f6d4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa33d6d4d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x35996d50, SUC_NE_lexicon_tags_2 },
    { 0x585d6d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc36d54, SUC_NE_lexicon_tags_2 },
    { 0x5e716d54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ff6d59, SUC_NE_lexicon_tags_2 },
    { 0xee96d5a, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0x9b136d5c, SUC_NE_lexicon_tags_2 },
    { 0xa8e36d5c, SUC_NE_lexicon_tags_19 },
    { 0xcc216d5e, SUC_NE_lexicon_tags_2 },
    { 0xa2af6d5f, SUC_NE_lexicon_tags_2 },
    { 0xe6416d5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4be96d62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99976d65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99116d68, SUC_NE_lexicon_tags_2 },
    { 0xa88b6d68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24016d6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84996d71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963b6d75, SUC_NE_lexicon_tags_51 },
    { 0x63436d75, SUC_NE_lexicon_tags_2 },
    { 0xc2e56d77, SUC_NE_lexicon_tags_2 },
    { 0xeaa16d78, SUC_NE_lexicon_tags_4 },
    { 0xf4c16d77, SUC_NE_lexicon_tags_2 },
    { 0x8b336d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c236d7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cf6d81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe176d84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46df6d89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18096d8b, SUC_NE_lexicon_tags_2 },
    { 0x1cbb6d8b, SUC_NE_lexicon_tags_2 },
    { 0x12196d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54b36d8f, SUC_NE_lexicon_tags_2 },
    { 0xac36d90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49816d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f356d95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe176d9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ef6da2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e16da5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d6da9, SUC_NE_lexicon_tags_7 },
    { 0xa6096daa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa96db0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6f6db3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc96db6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f16dc3, SUC_NE_lexicon_tags_2 },
    { 0x3a596dc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2f6dca, SUC_NE_lexicon_tags_2 },
    { 0x4bf36dca, SUC_NE_lexicon_tags_2 },
    { 0xe1296dcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6496dcf, SUC_NE_lexicon_tags_2 },
    { 0x5e976dcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b4f6dd2, SUC_NE_lexicon_tags_2 },
    { 0x7ff16dd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b6dd6, SUC_NE_lexicon_tags_2 },
    { 0xe29b6dd6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12af6ddb, SUC_NE_lexicon_tags_2 },
    { 0x6a436ddc, SUC_NE_lexicon_tags_2 },
    { 0xf08f6ddc, SUC_NE_lexicon_tags_2 },
    { 0xc7af6ddd, SUC_NE_lexicon_tags_2 },
    { 0x17dd6ddf, SUC_NE_lexicon_tags_7 },
    { 0x6c7f6de0, SUC_NE_lexicon_tags_2 },
    { 0x75376de1, SUC_NE_lexicon_tags_2 },
    { 0x81816de2, SUC_NE_lexicon_tags_2 },
    { 0xdd096de2, SUC_NE_lexicon_tags_2 },
    { 0x73316ddf, SUC_NE_lexicon_tags_2 },
    { 0xb4136de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd6df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7576df3, SUC_NE_lexicon_tags_4 },
    { 0x84e36df3, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b1f6df6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2c36df8, SUC_NE_lexicon_tags_2 },
    { 0xa7256df8, SUC_NE_lexicon_tags_17 },
    { 0xf9876df8, SUC_NE_lexicon_tags_2 },
    { 0x1fdb6dfa, SUC_NE_lexicon_tags_21 },
    { 0x2f0b6df8, SUC_NE_lexicon_tags_2 },
    { 0x73396dfd, SUC_NE_lexicon_tags_2 },
    { 0xb5256dfd, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10d6e01, SUC_NE_lexicon_tags_2 },
    { 0xa2c36e02, SUC_NE_lexicon_tags_2 },
    { 0x29f36e03, SUC_NE_lexicon_tags_2 },
    { 0xee596e04, SUC_NE_lexicon_tags_2 },
    { 0xa6f56e04, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2656e0d, SUC_NE_lexicon_tags_2 },
    { 0x3f0f6e0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c36e11, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bd6e14, SUC_NE_lexicon_tags_2 },
    { 0x5eb16e15, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e76e18, SUC_NE_lexicon_tags_2 },
    { 0xdf616e19, SUC_NE_lexicon_tags_7 },
    { 0x9c736e18, SUC_NE_lexicon_tags_4 },
    { 0x4dc16e19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c56e1e, SUC_NE_lexicon_tags_2 },
    { 0xe3076e1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33556e22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2136e29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d6e2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60756e31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39376e35, SUC_NE_lexicon_tags_86 },
    { 0x1ce56e36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12716e39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1b96e3b, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7d6e41, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xf0db6e43, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x423f6e45, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xab316e47, SUC_NE_lexicon_tags_2 },
    { 0xf9cb6e47, SUC_NE_lexicon_tags_2 },
    { 0xd13d6e49, SUC_NE_lexicon_tags_2 },
    { 0x303b6e4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da16e50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddb6e53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ef6e56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13116e59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38cf6e5c, SUC_NE_lexicon_tags_2 },
    { 0xa4b16e5c, SUC_NE_lexicon_tags_2 },
    { 0x703f6e5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494d6e65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80996e67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de56e6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x335b6e72, SUC_NE_lexicon_tags_2 },
    { 0x3c3f6e73, SUC_NE_lexicon_tags_2 },
    { 0x40676e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccb96e76, SUC_NE_lexicon_tags_2 },
    { 0x36c36e77, SUC_NE_lexicon_tags_2 },
    { 0x19416e78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf15f6e7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f16e7d, SUC_NE_lexicon_tags_2 },
    { 0xd3ad6e7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x221f6e80, SUC_NE_lexicon_tags_2 },
    { 0x87696e81, SUC_NE_lexicon_tags_19 },
    { 0x825f6e82, SUC_NE_lexicon_tags_2 },
    { 0xc1a56e83, SUC_NE_lexicon_tags_2 },
    { 0x58516e83, SUC_NE_lexicon_tags_2 },
    { 0x61d36e85, SUC_NE_lexicon_tags_2 },
    { 0xdd436e86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6356e88, SUC_NE_lexicon_tags_2 },
    { 0x47a76e89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a96e8c, SUC_NE_lexicon_tags_2 },
    { 0x3b996e8d, SUC_NE_lexicon_tags_2 },
    { 0x9a8b6e8e, SUC_NE_lexicon_tags_2 },
    { 0x9bbd6e8f, SUC_NE_lexicon_tags_2 },
    { 0x6a0f6e8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e16e95, SUC_NE_lexicon_tags_2 },
    { 0x59b96e96, SUC_NE_lexicon_tags_2 },
    { 0x53fb6e95, SUC_NE_lexicon_tags_2 },
    { 0xb9456e98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d616e9c, SUC_NE_lexicon_tags_2 },
    { 0x95d96e9c, SUC_NE_lexicon_tags_315 },
    { 0x2e496e9d, SUC_NE_lexicon_tags_2 },
    { 0xc8076e9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68976ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x342f6ea6, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x8f96ea8, SUC_NE_lexicon_tags_2 },
    { 0xfadf6ea8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8036eab, SUC_NE_lexicon_tags_2 },
    { 0xfd716eac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2256eb4, SUC_NE_lexicon_tags_2 },
    { 0x37bb6eb5, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d76eb8, SUC_NE_lexicon_tags_1 },
    { 0x79496eb8, SUC_NE_lexicon_tags_113 },
    { 0x55a76eb9, SUC_NE_lexicon_tags_3 },
    { 0xe8d36ebb, SUC_NE_lexicon_tags_2 },
    { 0xd516ebb, SUC_NE_lexicon_tags_2 },
    { 0x62676ebc, SUC_NE_lexicon_tags_4 },
    { 0xf1a16ebb, SUC_NE_lexicon_tags_2 },
    { 0x7b016ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c16ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a16ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabef6ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd6ece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55876ed0, SUC_NE_lexicon_tags_2 },
    { 0xc8056ed1, SUC_NE_lexicon_tags_19 },
    { 0x5f056ed0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48196ed7, SUC_NE_lexicon_tags_2 },
    { 0xd5db6ed8, SUC_NE_lexicon_tags_2 },
    { 0xb9496ed8, SUC_NE_lexicon_tags_2 },
    { 0xb9b96eda, SUC_NE_lexicon_tags_2 },
    { 0x72576edb, SUC_NE_lexicon_tags_2 },
    { 0x66056edc, SUC_NE_lexicon_tags_2 },
    { 0x8296edc, SUC_NE_lexicon_tags_2 },
    { 0xe6196eda, SUC_NE_lexicon_tags_2 },
    { 0xeef6ed9, SUC_NE_lexicon_tags_2 },
    { 0xe8936edb, SUC_NE_lexicon_tags_2 },
    { 0x84e16edd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x39c16ee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6616ee6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x64816ee8, SUC_NE_lexicon_tags_2 },
    { 0x8bd16ee8, SUC_NE_lexicon_tags_2 },
    { 0x5ce96ee9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92b6eed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c616ef0, SUC_NE_lexicon_tags_2 },
    { 0xa936ef1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb936ef4, SUC_NE_lexicon_tags_2 },
    { 0xc9716ef5, SUC_NE_lexicon_tags_2 },
    { 0x5576ef5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b16efb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c56efe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfedf6f00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f16f08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40ed6f0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64156f0e, SUC_NE_lexicon_tags_2 },
    { 0xc5596f0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b076f14, SUC_NE_lexicon_tags_1 },
    { 0x98596f15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f176f19, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe096f1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a56f27, SUC_NE_lexicon_tags_2 },
    { 0xfdcb6f28, SUC_NE_lexicon_tags_2 },
    { 0x71bb6f29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70a76f2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17116f2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72b6f32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a96f35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74976f37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac6d6f39, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bf6f3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x846d6f3f, SUC_NE_lexicon_tags_2 },
    { 0x44416f40, SUC_NE_lexicon_tags_2 },
    { 0xf9d76f41, SUC_NE_lexicon_tags_1 },
    { 0x5996f41, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c76f46, SUC_NE_lexicon_tags_2 },
    { 0xa7f96f47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac256f49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f596f4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806d6f53, SUC_NE_lexicon_tags_5 },
    { 0xd4676f53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad856f57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1576f59, SUC_NE_lexicon_tags_7 },
    { 0xcc2d6f5a, SUC_NE_lexicon_tags_2 },
    { 0x5f7d6f5b, SUC_NE_lexicon_tags_2 },
    { 0x30516f59, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb9a36f5e, SUC_NE_lexicon_tags_13 },
    { 0xe2376f5f, SUC_NE_lexicon_tags_49 },
    { 0xce96f60, SUC_NE_lexicon_tags_2 },
    { 0x16876f61, SUC_NE_lexicon_tags_2 },
    { 0x4ef76f62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29556f65, SUC_NE_lexicon_tags_2 },
    { 0xd5c76f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda936f6a, SUC_NE_lexicon_tags_2 },
    { 0xd1c36f6b, SUC_NE_lexicon_tags_2 },
    { 0x9e736f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3eff6f6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb92b6f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1e76f72, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5ab6f74, SUC_NE_lexicon_tags_2 },
    { 0xafc36f75, SUC_NE_lexicon_tags_2 },
    { 0xa82b6f74, SUC_NE_lexicon_tags_2 },
    { 0x19e16f74, SUC_NE_lexicon_tags_2 },
    { 0x74196f78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x116f7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e296f7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2f6f83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf216f85, SUC_NE_lexicon_tags_1 },
    { 0x7f156f86, SUC_NE_lexicon_tags_2 },
    { 0xd11d6f86, SUC_NE_lexicon_tags_102 },
    { 0x32796f86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d36f8c, SUC_NE_lexicon_tags_2 },
    { 0xacff6f8c, SUC_NE_lexicon_tags_2 },
    { 0xe2476f8e, SUC_NE_lexicon_tags_2 },
    { 0xbfcb6f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a96f92, SUC_NE_lexicon_tags_2 },
    { 0x3d16f92, SUC_NE_lexicon_tags_0 },
    { 0xb5916f94, SUC_NE_lexicon_tags_2 },
    { 0x109b6f94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcf6f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8b6fa2, SUC_NE_lexicon_tags_2 },
    { 0xd5556fa3, SUC_NE_lexicon_tags_2 },
    { 0x54ab6fa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c36fa8, SUC_NE_lexicon_tags_12 },
    { 0x6a236fa8, SUC_NE_lexicon_tags_2 },
    { 0x94b76faa, SUC_NE_lexicon_tags_17 },
    { 0xa32b6fab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8f6fb0, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xc4936fb2, SUC_NE_lexicon_tags_2 },
    { 0x93bb6fb3, SUC_NE_lexicon_tags_4 },
    { 0xc29f6fb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e96fb7, SUC_NE_lexicon_tags_2 },
    { 0x81536fb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b76fbc, SUC_NE_lexicon_tags_2 },
    { 0x4cc56fbc, SUC_NE_lexicon_tags_2 },
    { 0xf81d6fbe, SUC_NE_lexicon_tags_2 },
    { 0x80b76fbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f9b6fc1, SUC_NE_lexicon_tags_2 },
    { 0xaf6f6fc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db6fc6, SUC_NE_lexicon_tags_2 },
    { 0x78056fc7, SUC_NE_lexicon_tags_2 },
    { 0x8c996fc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbee36fca, SUC_NE_lexicon_tags_4 },
    { 0x83e16fcb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8696fd0, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6936fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86476fd9, SUC_NE_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56316fde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77d6fe4, SUC_NE_lexicon_tags_2 },
    { 0x714b6fe5, SUC_NE_lexicon_tags_2 },
    { 0xfdb76fe6, SUC_NE_lexicon_tags_2 },
    { 0xa82d6fe7, SUC_NE_lexicon_tags_2 },
    { 0xccf96fe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefc96fea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00f6fef, SUC_NE_lexicon_tags_2 },
    { 0xae476fef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de76ff5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395d6ffb, SUC_NE_lexicon_tags_239 },
    { 0x98bd6ffc, SUC_NE_lexicon_tags_2 },
    { 0xc1b76ffd, SUC_NE_lexicon_tags_2 },
    { 0xf2536ffe, SUC_NE_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d517001, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b137005, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f37008, SUC_NE_lexicon_tags_2 },
    { 0x9b877008, SUC_NE_lexicon_tags_2 },
    { 0x203b700a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe873700c, SUC_NE_lexicon_tags_2 },
    { 0x8a8d700c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x467b700f, SUC_NE_lexicon_tags_2 },
    { 0xaf137010, SUC_NE_lexicon_tags_2 },
    { 0xdc4f7010, SUC_NE_lexicon_tags_2 },
    { 0x9f0d7012, SUC_NE_lexicon_tags_2 },
    { 0x968d7011, SUC_NE_lexicon_tags_2 },
    { 0xe217014, SUC_NE_lexicon_tags_2 },
    { 0xae0f7010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80917019, SUC_NE_lexicon_tags_2 },
    { 0x6fb17019, SUC_NE_lexicon_tags_2 },
    { 0xcca7701b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa05701d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781d701f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b617022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0277025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361f7031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b617035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b57039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b1f703b, SUC_NE_lexicon_tags_2 },
    { 0x8979703c, SUC_NE_lexicon_tags_2 },
    { 0xf007703d, SUC_NE_lexicon_tags_2 },
    { 0xba09703e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6b7044, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x615f7047, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ea57049, SUC_NE_lexicon_tags_138 },
    { 0x1db37049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66f1704c, SUC_NE_lexicon_tags_2 },
    { 0x445b704c, SUC_NE_lexicon_tags_125 },
    { 0x37af704c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe337051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62897054, SUC_NE_lexicon_tags_2 },
    { 0x8de77055, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd917059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d6d705b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5869705e, SUC_NE_lexicon_tags_2 },
    { 0x6f7705e, SUC_NE_lexicon_tags_2 },
    { 0x2669705e, SUC_NE_lexicon_tags_2 },
    { 0xcbb37061, SUC_NE_lexicon_tags_2 },
    { 0xa6657062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1df7064, SUC_NE_lexicon_tags_2 },
    { 0xbb897065, SUC_NE_lexicon_tags_2 },
    { 0x1f157065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9d706c, SUC_NE_lexicon_tags_7 },
    { 0x9f09706d, SUC_NE_lexicon_tags_2 },
    { 0x813706e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa997070, SUC_NE_lexicon_tags_2 },
    { 0x57437071, SUC_NE_lexicon_tags_7 },
    { 0x8e8f7071, SUC_NE_lexicon_tags_2 },
    { 0xf4f97073, SUC_NE_lexicon_tags_2 },
    { 0x48cb7071, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16c77076, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1137078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3999707b, SUC_NE_lexicon_tags_2 },
    { 0xfff707b, SUC_NE_lexicon_tags_4 },
    { 0x8461707d, SUC_NE_lexicon_tags_2 },
    { 0x321f707b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891b7085, SUC_NE_lexicon_tags_2 },
    { 0xd1477086, SUC_NE_lexicon_tags_2 },
    { 0x800b7087, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb961708c, SUC_NE_lexicon_tags_32 },
    { 0x5a67708d, SUC_NE_lexicon_tags_2 },
    { 0x476f708c, SUC_NE_lexicon_tags_2 },
    { 0xe7ab708f, SUC_NE_lexicon_tags_3 },
    { 0x3c6b708f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d17093, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955d7098, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x24d9709a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd7709e, SUC_NE_lexicon_tags_2 },
    { 0x91cb709f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb73770a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5db70a8, SUC_NE_lexicon_tags_2 },
    { 0xe3e970a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb0f70ab, SUC_NE_lexicon_tags_4 },
    { 0xd0f770ac, SUC_NE_lexicon_tags_2 },
    { 0x3d8970ad, SUC_NE_lexicon_tags_2 },
    { 0xbd5b70ae, SUC_NE_lexicon_tags_2 },
    { 0x325f70af, SUC_NE_lexicon_tags_2 },
    { 0x9ee570b0, SUC_NE_lexicon_tags_2 },
    { 0xf38370b1, SUC_NE_lexicon_tags_2 },
    { 0x8db370b0, SUC_NE_lexicon_tags_2 },
    { 0x2cf570b1, SUC_NE_lexicon_tags_2 },
    { 0xe67970b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb19170b6, SUC_NE_lexicon_tags_2 },
    { 0x1cf770b7, SUC_NE_lexicon_tags_2 },
    { 0xb3f370b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x896370bb, SUC_NE_lexicon_tags_2 },
    { 0xc64f70bc, SUC_NE_lexicon_tags_2 },
    { 0xb77f70bc, SUC_NE_lexicon_tags_2 },
    { 0xf4a570be, SUC_NE_lexicon_tags_2 },
    { 0xfe6570bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x237570c1, SUC_NE_lexicon_tags_2 },
    { 0x3a7f70c2, SUC_NE_lexicon_tags_2 },
    { 0x57f170c3, SUC_NE_lexicon_tags_3 },
    { 0xccb770c3, SUC_NE_lexicon_tags_2 },
    { 0x50570c2, SUC_NE_lexicon_tags_2 },
    { 0xeb8170c2, SUC_NE_lexicon_tags_2 },
    { 0xa3cb70c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bdd70c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fb70cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80d70ce, SUC_NE_lexicon_tags_2 },
    { 0xe9dd70ce, SUC_NE_lexicon_tags_2 },
    { 0x618170cf, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xce570d3, SUC_NE_lexicon_tags_2 },
    { 0x3e0370d4, SUC_NE_lexicon_tags_2 },
    { 0x9fb770d5, SUC_NE_lexicon_tags_2 },
    { 0x686970d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb35570d8, SUC_NE_lexicon_tags_4 },
    { 0x119570d9, SUC_NE_lexicon_tags_2 },
    { 0x9a6770da, SUC_NE_lexicon_tags_2 },
    { 0x6a370da, SUC_NE_lexicon_tags_2 },
    { 0xab1370dc, SUC_NE_lexicon_tags_2 },
    { 0x65fd70dd, SUC_NE_lexicon_tags_2 },
    { 0xcff770de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x405d70e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d770e3, SUC_NE_lexicon_tags_30 },
    { 0x3bb770e4, SUC_NE_lexicon_tags_2 },
    { 0x145f70e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59c570e7, SUC_NE_lexicon_tags_2 },
    { 0x752970e7, SUC_NE_lexicon_tags_2 },
    { 0x3c4970e9, SUC_NE_lexicon_tags_2 },
    { 0x114170e9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf51170ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x617b70ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe26d70f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51a970f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acf70f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x431170f7, SUC_NE_lexicon_tags_2 },
    { 0xa5970f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b70fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad770ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34e77101, SUC_NE_lexicon_tags_2 },
    { 0x29437102, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bcf7104, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x851b7107, SUC_NE_lexicon_tags_17 },
    { 0x8b997107, SUC_NE_lexicon_tags_7 },
    { 0xe7a57109, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fb710d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9ab710f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4caf7111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15977113, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9117116, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db97119, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a7711c, SUC_NE_lexicon_tags_2 },
    { 0x1789711d, SUC_NE_lexicon_tags_2 },
    { 0x5265711d, SUC_NE_lexicon_tags_2 },
    { 0xef93711f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdf77121, SUC_NE_lexicon_tags_2 },
    { 0x49797122, SUC_NE_lexicon_tags_2 },
    { 0xe1b97123, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df7126, SUC_NE_lexicon_tags_30 },
    { 0xd0677127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd77b7129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fa9712b, SUC_NE_lexicon_tags_2 },
    { 0x3a83712b, SUC_NE_lexicon_tags_2 },
    { 0xc09f712d, SUC_NE_lexicon_tags_2 },
    { 0xfee7712c, SUC_NE_lexicon_tags_2 },
    { 0x3dfd712f, SUC_NE_lexicon_tags_2 },
    { 0x7647712d, SUC_NE_lexicon_tags_2 },
    { 0x8b1f7131, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98097136, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65817138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c1713b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d75713d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf809713f, SUC_NE_lexicon_tags_2 },
    { 0x8d337140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff257143, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb885714a, SUC_NE_lexicon_tags_2 },
    { 0x6c47714b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b57150, SUC_NE_lexicon_tags_2 },
    { 0xb8537151, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e17153, SUC_NE_lexicon_tags_3 },
    { 0x75b17153, SUC_NE_lexicon_tags_2 },
    { 0x1d377155, SUC_NE_lexicon_tags_7 },
    { 0xa497155, SUC_NE_lexicon_tags_2 },
    { 0x4d457155, SUC_NE_lexicon_tags_2 },
    { 0x197b7158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded715c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e17161, SUC_NE_lexicon_tags_2 },
    { 0x28c97162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd457164, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb27b7166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1f716c, SUC_NE_lexicon_tags_0 },
    { 0x6ac3716d, SUC_NE_lexicon_tags_130 },
    { 0, NULL },
    { 0x607716f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f297172, SUC_NE_lexicon_tags_2 },
    { 0x97af7172, SUC_NE_lexicon_tags_2 },
    { 0x88d37174, SUC_NE_lexicon_tags_2 },
    { 0xf3937174, SUC_NE_lexicon_tags_2 },
    { 0x957f7172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa48b7178, SUC_NE_lexicon_tags_310 },
    { 0, NULL },
    { 0x1c5d717a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9d3717c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d7181, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f77183, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33cb7185, SUC_NE_lexicon_tags_4 },
    { 0xb1df7186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0137188, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5479718c, SUC_NE_lexicon_tags_2 },
    { 0xa10f718d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88437191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2f7197, SUC_NE_lexicon_tags_2 },
    { 0x8ad97198, SUC_NE_lexicon_tags_2 },
    { 0x9a257199, SUC_NE_lexicon_tags_2 },
    { 0x68fb7197, SUC_NE_lexicon_tags_2 },
    { 0x8ff9719b, SUC_NE_lexicon_tags_2 },
    { 0x6b35719b, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857971a2, SUC_NE_lexicon_tags_2 },
    { 0xf92571a2, SUC_NE_lexicon_tags_2 },
    { 0x281571a4, SUC_NE_lexicon_tags_5 },
    { 0x74f171a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ebf71a7, SUC_NE_lexicon_tags_2 },
    { 0x98c371a7, SUC_NE_lexicon_tags_2 },
    { 0xed1971a7, SUC_NE_lexicon_tags_2 },
    { 0x5c2d71aa, SUC_NE_lexicon_tags_2 },
    { 0xb54371aa, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0x470371ad, SUC_NE_lexicon_tags_2 },
    { 0x948371ad, SUC_NE_lexicon_tags_2 },
    { 0x153771af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68171b4, SUC_NE_lexicon_tags_198 },
    { 0x9f6371b5, SUC_NE_lexicon_tags_0 },
    { 0x85ed71b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc8f71b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52971be, SUC_NE_lexicon_tags_58 },
    { 0xf5bd71bf, SUC_NE_lexicon_tags_2 },
    { 0xdecb71c0, SUC_NE_lexicon_tags_2 },
    { 0x1a6771c1, SUC_NE_lexicon_tags_2 },
    { 0x2b71c2, SUC_NE_lexicon_tags_2 },
    { 0x80fd71c3, SUC_NE_lexicon_tags_17 },
    { 0x14e571c2, SUC_NE_lexicon_tags_2 },
    { 0x56d171c5, SUC_NE_lexicon_tags_2 },
    { 0x952571c6, SUC_NE_lexicon_tags_2 },
    { 0x459b71c3, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d571c9, SUC_NE_lexicon_tags_2 },
    { 0xcfd71ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44a371cc, SUC_NE_lexicon_tags_178 },
    { 0, NULL },
    { 0xf77971ce, SUC_NE_lexicon_tags_2 },
    { 0x804f71cf, SUC_NE_lexicon_tags_2 },
    { 0xfbdf71d0, SUC_NE_lexicon_tags_2 },
    { 0xce9b71d1, SUC_NE_lexicon_tags_2 },
    { 0x2bfd71d1, SUC_NE_lexicon_tags_2 },
    { 0x69c571d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf18f71d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e971d9, SUC_NE_lexicon_tags_2 },
    { 0x9c4d71da, SUC_NE_lexicon_tags_322 },
    { 0xe02b71d9, SUC_NE_lexicon_tags_2 },
    { 0xf50371dc, SUC_NE_lexicon_tags_2 },
    { 0x7ab71da, SUC_NE_lexicon_tags_2 },
    { 0x1dcb71de, SUC_NE_lexicon_tags_2 },
    { 0xe7c371de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8c571e1, SUC_NE_lexicon_tags_2 },
    { 0x396f71e2, SUC_NE_lexicon_tags_2 },
    { 0xe79371e2, SUC_NE_lexicon_tags_2 },
    { 0x36cf71e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1971e9, SUC_NE_lexicon_tags_2 },
    { 0x134771ea, SUC_NE_lexicon_tags_2 },
    { 0xb2a771ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136771f0, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x453d71f2, SUC_NE_lexicon_tags_7 },
    { 0xb2bb71f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f71f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb71fc, SUC_NE_lexicon_tags_2 },
    { 0xde971fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92b371ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7117202, SUC_NE_lexicon_tags_4 },
    { 0x43197203, SUC_NE_lexicon_tags_2 },
    { 0x26077204, SUC_NE_lexicon_tags_7 },
    { 0xe0cb7204, SUC_NE_lexicon_tags_2 },
    { 0x4aa57206, SUC_NE_lexicon_tags_28 },
    { 0x73c57206, SUC_NE_lexicon_tags_2 },
    { 0x1e997207, SUC_NE_lexicon_tags_2 },
    { 0xa7cb7209, SUC_NE_lexicon_tags_2 },
    { 0xe7df720a, SUC_NE_lexicon_tags_2 },
    { 0x1ee9720b, SUC_NE_lexicon_tags_2 },
    { 0x8c8b7208, SUC_NE_lexicon_tags_367 },
    { 0x885720c, SUC_NE_lexicon_tags_2 },
    { 0xb67f720e, SUC_NE_lexicon_tags_2 },
    { 0xf1cf720f, SUC_NE_lexicon_tags_2 },
    { 0x4cd57203, SUC_NE_lexicon_tags_2 },
    { 0x780d7207, SUC_NE_lexicon_tags_2 },
    { 0xb4dd7212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x780f7214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69177217, SUC_NE_lexicon_tags_2 },
    { 0x64e17217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f7d721a, SUC_NE_lexicon_tags_2 },
    { 0xa525721b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd28b721d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff73721f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57497221, SUC_NE_lexicon_tags_2 },
    { 0x9e437222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0d37224, SUC_NE_lexicon_tags_2 },
    { 0xc0e37224, SUC_NE_lexicon_tags_2 },
    { 0x782d7226, SUC_NE_lexicon_tags_2 },
    { 0x53857227, SUC_NE_lexicon_tags_2 },
    { 0x8eab7228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14a9722a, SUC_NE_lexicon_tags_2 },
    { 0x8993722a, SUC_NE_lexicon_tags_2 },
    { 0x63f7722a, SUC_NE_lexicon_tags_2 },
    { 0x4bb722d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d9722f, SUC_NE_lexicon_tags_2 },
    { 0xdf03722f, SUC_NE_lexicon_tags_2 },
    { 0xe9ff7230, SUC_NE_lexicon_tags_2 },
    { 0x71637230, SUC_NE_lexicon_tags_2 },
    { 0x72b17233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb717237, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31d17239, SUC_NE_lexicon_tags_98 },
    { 0x81db723a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86db723c, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x796d723f, SUC_NE_lexicon_tags_2 },
    { 0xbf27723f, SUC_NE_lexicon_tags_2 },
    { 0x62c37241, SUC_NE_lexicon_tags_2 },
    { 0xbf677241, SUC_NE_lexicon_tags_2 },
    { 0x90c37240, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf437246, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b57249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768f724d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c57251, SUC_NE_lexicon_tags_2 },
    { 0x71f17251, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f1d7254, SUC_NE_lexicon_tags_2 },
    { 0x4fc37254, SUC_NE_lexicon_tags_5 },
    { 0x24c57256, SUC_NE_lexicon_tags_2 },
    { 0x64757257, SUC_NE_lexicon_tags_2 },
    { 0x8057258, SUC_NE_lexicon_tags_2 },
    { 0xb7f97259, SUC_NE_lexicon_tags_2 },
    { 0x90997255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a7725e, SUC_NE_lexicon_tags_2 },
    { 0x7e11725f, SUC_NE_lexicon_tags_2 },
    { 0x4cb5725f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eb7266, SUC_NE_lexicon_tags_2 },
    { 0x22c97266, SUC_NE_lexicon_tags_2 },
    { 0x285b7267, SUC_NE_lexicon_tags_2 },
    { 0xecad7269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc823726c, SUC_NE_lexicon_tags_2 },
    { 0x3e6b726d, SUC_NE_lexicon_tags_2 },
    { 0x9a1f726e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12977270, SUC_NE_lexicon_tags_2 },
    { 0xf0e57271, SUC_NE_lexicon_tags_2 },
    { 0x5fe97270, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ef7276, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2d7280, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd7285, SUC_NE_lexicon_tags_2 },
    { 0x2a2b7286, SUC_NE_lexicon_tags_2 },
    { 0x75fb7286, SUC_NE_lexicon_tags_2 },
    { 0x4e2d7287, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b7728b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd22d728d, SUC_NE_lexicon_tags_2 },
    { 0xc88b728e, SUC_NE_lexicon_tags_2 },
    { 0x1d91728f, SUC_NE_lexicon_tags_2 },
    { 0x5f857290, SUC_NE_lexicon_tags_2 },
    { 0xa6ab7291, SUC_NE_lexicon_tags_2 },
    { 0x722d7291, SUC_NE_lexicon_tags_2 },
    { 0x1d0d7290, SUC_NE_lexicon_tags_2 },
    { 0xa8ad7290, SUC_NE_lexicon_tags_2 },
    { 0x85117295, SUC_NE_lexicon_tags_2 },
    { 0xb1f37296, SUC_NE_lexicon_tags_2 },
    { 0x26797297, SUC_NE_lexicon_tags_14 },
    { 0x863b7296, SUC_NE_lexicon_tags_2 },
    { 0x5a397295, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8613729d, SUC_NE_lexicon_tags_2 },
    { 0x56d3729e, SUC_NE_lexicon_tags_5 },
    { 0x71f1729e, SUC_NE_lexicon_tags_2 },
    { 0xa709729f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x855d72a2, SUC_NE_lexicon_tags_86 },
    { 0xb65172a3, SUC_NE_lexicon_tags_2 },
    { 0x762b72a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757972a8, SUC_NE_lexicon_tags_2 },
    { 0x640f72a8, SUC_NE_lexicon_tags_2 },
    { 0xffdf72aa, SUC_NE_lexicon_tags_2 },
    { 0xf41972aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26b572ad, SUC_NE_lexicon_tags_2 },
    { 0xa83172ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f772b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef72b4, SUC_NE_lexicon_tags_2 },
    { 0x295372b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95ed72b7, SUC_NE_lexicon_tags_17 },
    { 0x396372b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466172bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x826772bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ad72c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a572c7, SUC_NE_lexicon_tags_2 },
    { 0x57c772c8, SUC_NE_lexicon_tags_2 },
    { 0xd30972c9, SUC_NE_lexicon_tags_2 },
    { 0x869d72c8, SUC_NE_lexicon_tags_2 },
    { 0xeb8572c9, SUC_NE_lexicon_tags_2 },
    { 0x1f9d72c9, SUC_NE_lexicon_tags_2 },
    { 0xf48b72c8, SUC_NE_lexicon_tags_2 },
    { 0xfab372cc, SUC_NE_lexicon_tags_2 },
    { 0x9cfb72cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x468372d1, SUC_NE_lexicon_tags_2 },
    { 0x807372d1, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x282572d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2372d8, SUC_NE_lexicon_tags_2 },
    { 0x184d72d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x547372db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1772df, SUC_NE_lexicon_tags_7 },
    { 0xd2e172df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd172ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155f72ee, SUC_NE_lexicon_tags_2 },
    { 0x8b4f72ef, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfeb172f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5f72f6, SUC_NE_lexicon_tags_4 },
    { 0x85bb72f6, SUC_NE_lexicon_tags_2 },
    { 0x40b172f6, SUC_NE_lexicon_tags_2 },
    { 0x6f4972f6, SUC_NE_lexicon_tags_4 },
    { 0xe5b772fa, SUC_NE_lexicon_tags_1 },
    { 0xd98f72fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e9d72fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f077304, SUC_NE_lexicon_tags_2 },
    { 0x7edb7305, SUC_NE_lexicon_tags_2 },
    { 0x39437304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b4f7308, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6715730e, SUC_NE_lexicon_tags_2 },
    { 0xa80f730f, SUC_NE_lexicon_tags_2 },
    { 0x5f637310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feb7318, SUC_NE_lexicon_tags_2 },
    { 0x1f3f7319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe1731c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1277323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x364f7325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0157328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d1732a, SUC_NE_lexicon_tags_2 },
    { 0x4d11732b, SUC_NE_lexicon_tags_2 },
    { 0xabad732c, SUC_NE_lexicon_tags_2 },
    { 0x3005732d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d37332, SUC_NE_lexicon_tags_2 },
    { 0x64497332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf57336, SUC_NE_lexicon_tags_2 },
    { 0x76697337, SUC_NE_lexicon_tags_2 },
    { 0xbb2d7338, SUC_NE_lexicon_tags_10 },
    { 0xe9157339, SUC_NE_lexicon_tags_2 },
    { 0x8469733a, SUC_NE_lexicon_tags_2 },
    { 0xc5c3733b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882b7341, SUC_NE_lexicon_tags_2 },
    { 0x26197341, SUC_NE_lexicon_tags_2 },
    { 0xdad7343, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cff734b, SUC_NE_lexicon_tags_2 },
    { 0x8509734c, SUC_NE_lexicon_tags_130 },
    { 0xabb734d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xcde9734f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38577352, SUC_NE_lexicon_tags_25 },
    { 0x2b257353, SUC_NE_lexicon_tags_2 },
    { 0x33857354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3af7356, SUC_NE_lexicon_tags_2 },
    { 0x4ee97356, SUC_NE_lexicon_tags_2 },
    { 0xb5617358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734d7360, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21717364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8057366, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e517369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9c9736b, SUC_NE_lexicon_tags_18 },
    { 0xdaf3736b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeed7371, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d237375, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x129d7377, SUC_NE_lexicon_tags_2 },
    { 0xd9557378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138b7381, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503d7385, SUC_NE_lexicon_tags_2 },
    { 0xd3417385, SUC_NE_lexicon_tags_7 },
    { 0x53eb7387, SUC_NE_lexicon_tags_2 },
    { 0x87d17387, SUC_NE_lexicon_tags_40 },
    { 0xdbb97387, SUC_NE_lexicon_tags_2 },
    { 0xb9ed7386, SUC_NE_lexicon_tags_2 },
    { 0xc6df7388, SUC_NE_lexicon_tags_2 },
    { 0x2a03738c, SUC_NE_lexicon_tags_2 },
    { 0xfded738d, SUC_NE_lexicon_tags_2 },
    { 0x63f17388, SUC_NE_lexicon_tags_2 },
    { 0x4fb37386, SUC_NE_lexicon_tags_2 },
    { 0x31197390, SUC_NE_lexicon_tags_2 },
    { 0xff577391, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebb17393, SUC_NE_lexicon_tags_2 },
    { 0xdab7394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4577396, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba7739a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c3739e, SUC_NE_lexicon_tags_2 },
    { 0x1ca1739f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb773a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a773a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7373ac, SUC_NE_lexicon_tags_2 },
    { 0x54d773ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bd773af, SUC_NE_lexicon_tags_2 },
    { 0xa5b173b0, SUC_NE_lexicon_tags_2 },
    { 0x6ba373b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45db73b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7d73b6, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c773b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x196d73bc, SUC_NE_lexicon_tags_2 },
    { 0x219173bd, SUC_NE_lexicon_tags_2 },
    { 0xc02173be, SUC_NE_lexicon_tags_2 },
    { 0xa25773bf, SUC_NE_lexicon_tags_2 },
    { 0x582d73c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6f73c6, SUC_NE_lexicon_tags_3 },
    { 0x6d9173c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb8773c9, SUC_NE_lexicon_tags_2 },
    { 0xb7b573ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x581173cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0773ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbb73d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22573d6, SUC_NE_lexicon_tags_2 },
    { 0x121373d7, SUC_NE_lexicon_tags_32 },
    { 0x7b5973d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11d573da, SUC_NE_lexicon_tags_2 },
    { 0x560373db, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xddb173dd, SUC_NE_lexicon_tags_2 },
    { 0x3dc773de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d773e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7b73e4, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a573e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x667b73e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd67973eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc60773ed, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x472173ef, SUC_NE_lexicon_tags_2 },
    { 0xd77973f0, SUC_NE_lexicon_tags_2 },
    { 0xd9a573f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55d73f3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbc5973f5, SUC_NE_lexicon_tags_2 },
    { 0x42373f6, SUC_NE_lexicon_tags_2 },
    { 0xb6f73f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d773fa, SUC_NE_lexicon_tags_2 },
    { 0xb86773fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d57400, SUC_NE_lexicon_tags_2 },
    { 0xfff77401, SUC_NE_lexicon_tags_2 },
    { 0xc1657401, SUC_NE_lexicon_tags_2 },
    { 0xfb557403, SUC_NE_lexicon_tags_2 },
    { 0x9e677403, SUC_NE_lexicon_tags_2 },
    { 0xa0597403, SUC_NE_lexicon_tags_4 },
    { 0x8ead7406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5673740b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc597740e, SUC_NE_lexicon_tags_2 },
    { 0xb7c9740f, SUC_NE_lexicon_tags_2 },
    { 0xa1217410, SUC_NE_lexicon_tags_2 },
    { 0xf1f5740f, SUC_NE_lexicon_tags_2 },
    { 0x73317412, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73517417, SUC_NE_lexicon_tags_261 },
    { 0xe0797418, SUC_NE_lexicon_tags_7 },
    { 0xdbb7419, SUC_NE_lexicon_tags_2 },
    { 0xd8f7741a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2393741d, SUC_NE_lexicon_tags_2 },
    { 0x84ad741d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccd7424, SUC_NE_lexicon_tags_2 },
    { 0xd4697425, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x9e357427, SUC_NE_lexicon_tags_0 },
    { 0x58b77428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d1742c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52d3742e, SUC_NE_lexicon_tags_21 },
    { 0x255d742e, SUC_NE_lexicon_tags_2 },
    { 0x9877430, SUC_NE_lexicon_tags_2 },
    { 0xfacf7431, SUC_NE_lexicon_tags_2 },
    { 0xd2ed7431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba97435, SUC_NE_lexicon_tags_19 },
    { 0x6eef7436, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3177439, SUC_NE_lexicon_tags_2 },
    { 0xa321743a, SUC_NE_lexicon_tags_2 },
    { 0x5517743a, SUC_NE_lexicon_tags_2 },
    { 0x6061743c, SUC_NE_lexicon_tags_2 },
    { 0xf6a1743d, SUC_NE_lexicon_tags_2 },
    { 0x78b743d, SUC_NE_lexicon_tags_2 },
    { 0x531b743d, SUC_NE_lexicon_tags_2 },
    { 0x78957440, SUC_NE_lexicon_tags_2 },
    { 0x55d7743e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba7744d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc34b744f, SUC_NE_lexicon_tags_2 },
    { 0x92b5744f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14617452, SUC_NE_lexicon_tags_4 },
    { 0x1e397452, SUC_NE_lexicon_tags_2 },
    { 0xb7ed7454, SUC_NE_lexicon_tags_2 },
    { 0xaaad7455, SUC_NE_lexicon_tags_2 },
    { 0x6a077456, SUC_NE_lexicon_tags_7 },
    { 0x5f957457, SUC_NE_lexicon_tags_2 },
    { 0x19d37458, SUC_NE_lexicon_tags_2 },
    { 0x71c37458, SUC_NE_lexicon_tags_2 },
    { 0x5c1d7458, SUC_NE_lexicon_tags_2 },
    { 0x93f7456, SUC_NE_lexicon_tags_2 },
    { 0x1189745c, SUC_NE_lexicon_tags_3 },
    { 0xaa01745c, SUC_NE_lexicon_tags_2 },
    { 0x129b745d, SUC_NE_lexicon_tags_4 },
    { 0x2831745f, SUC_NE_lexicon_tags_2 },
    { 0x90ab745f, SUC_NE_lexicon_tags_2 },
    { 0xe9a17461, SUC_NE_lexicon_tags_2 },
    { 0x510f7455, SUC_NE_lexicon_tags_2 },
    { 0x75857463, SUC_NE_lexicon_tags_2 },
    { 0xfda97464, SUC_NE_lexicon_tags_2 },
    { 0x9129745a, SUC_NE_lexicon_tags_2 },
    { 0x7349745f, SUC_NE_lexicon_tags_2 },
    { 0xf3c17467, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dd746d, SUC_NE_lexicon_tags_2 },
    { 0x5c73746e, SUC_NE_lexicon_tags_2 },
    { 0x7001746e, SUC_NE_lexicon_tags_198 },
    { 0, NULL },
    { 0x6c117471, SUC_NE_lexicon_tags_2 },
    { 0x7d57471, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bcd7474, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2a77476, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x61af7479, SUC_NE_lexicon_tags_2 },
    { 0xe2937479, SUC_NE_lexicon_tags_2 },
    { 0xd9377479, SUC_NE_lexicon_tags_2 },
    { 0x6d15747c, SUC_NE_lexicon_tags_215 },
    { 0xb07f747b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90177482, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33577484, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b17488, SUC_NE_lexicon_tags_2 },
    { 0x1a77489, SUC_NE_lexicon_tags_2 },
    { 0xb1d77489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e95748d, SUC_NE_lexicon_tags_2 },
    { 0xe9bb748e, SUC_NE_lexicon_tags_2 },
    { 0x8b2d748f, SUC_NE_lexicon_tags_2 },
    { 0xb8d5748f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec537493, SUC_NE_lexicon_tags_2 },
    { 0xe19f7493, SUC_NE_lexicon_tags_2 },
    { 0x1ebf7495, SUC_NE_lexicon_tags_2 },
    { 0x4b377493, SUC_NE_lexicon_tags_2 },
    { 0xd9117497, SUC_NE_lexicon_tags_2 },
    { 0x24eb7497, SUC_NE_lexicon_tags_2 },
    { 0x75377499, SUC_NE_lexicon_tags_2 },
    { 0x9ad37499, SUC_NE_lexicon_tags_2 },
    { 0x850f749a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c17749e, SUC_NE_lexicon_tags_2 },
    { 0xbc01749e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297774a5, SUC_NE_lexicon_tags_2 },
    { 0x878f74a5, SUC_NE_lexicon_tags_2 },
    { 0x8ca974a6, SUC_NE_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156d74ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362774b2, SUC_NE_lexicon_tags_95 },
    { 0x4e3374b3, SUC_NE_lexicon_tags_2 },
    { 0x800974b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704b74b8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf63174ba, SUC_NE_lexicon_tags_2 },
    { 0xd16774ba, SUC_NE_lexicon_tags_2 },
    { 0xc53774bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bd74c0, SUC_NE_lexicon_tags_2 },
    { 0xfc1174c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4774c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9574c7, SUC_NE_lexicon_tags_2 },
    { 0x38f74c8, SUC_NE_lexicon_tags_2 },
    { 0xf08574c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f2574cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x965f74cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e2f74cf, SUC_NE_lexicon_tags_2 },
    { 0x2ffb74cf, SUC_NE_lexicon_tags_2 },
    { 0x9c3574d1, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c774d4, SUC_NE_lexicon_tags_2 },
    { 0x533374d4, SUC_NE_lexicon_tags_2 },
    { 0xa2f974d6, SUC_NE_lexicon_tags_151 },
    { 0xe70174d7, SUC_NE_lexicon_tags_2 },
    { 0x7b8d74d5, SUC_NE_lexicon_tags_2 },
    { 0x6c0974d7, SUC_NE_lexicon_tags_2 },
    { 0xe79d74da, SUC_NE_lexicon_tags_2 },
    { 0x966374db, SUC_NE_lexicon_tags_2 },
    { 0x600574db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898574e4, SUC_NE_lexicon_tags_2 },
    { 0x312774e5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa27f74e7, SUC_NE_lexicon_tags_2 },
    { 0x72fd74e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540f74ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44df74f0, SUC_NE_lexicon_tags_2 },
    { 0x82bd74f1, SUC_NE_lexicon_tags_2 },
    { 0x9fa574f0, SUC_NE_lexicon_tags_2 },
    { 0x20af74f3, SUC_NE_lexicon_tags_2 },
    { 0x7dc374f4, SUC_NE_lexicon_tags_2 },
    { 0x552574f5, SUC_NE_lexicon_tags_2 },
    { 0xf94574f5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328374fb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e374ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b397503, SUC_NE_lexicon_tags_2 },
    { 0x3797504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad7507, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8b750a, SUC_NE_lexicon_tags_2 },
    { 0x229b750b, SUC_NE_lexicon_tags_2 },
    { 0x7b83750c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3219750f, SUC_NE_lexicon_tags_2 },
    { 0xc79f7510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x597f7512, SUC_NE_lexicon_tags_2 },
    { 0x16617513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9b7518, SUC_NE_lexicon_tags_2 },
    { 0xed217518, SUC_NE_lexicon_tags_347 },
    { 0x70d37518, SUC_NE_lexicon_tags_2 },
    { 0x5d857518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1b751f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7537524, SUC_NE_lexicon_tags_2 },
    { 0xcc5b7525, SUC_NE_lexicon_tags_2 },
    { 0xb097525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8d752b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2651752e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7957533, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917d7538, SUC_NE_lexicon_tags_2 },
    { 0x6cef7539, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb753f, SUC_NE_lexicon_tags_2 },
    { 0x6de3753f, SUC_NE_lexicon_tags_2 },
    { 0x72897541, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b97548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf21754a, SUC_NE_lexicon_tags_2 },
    { 0xb7d754a, SUC_NE_lexicon_tags_2 },
    { 0x311f754a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6b7552, SUC_NE_lexicon_tags_2 },
    { 0x36417553, SUC_NE_lexicon_tags_2 },
    { 0x9d8d7554, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c6f7556, SUC_NE_lexicon_tags_2 },
    { 0xfe0d7556, SUC_NE_lexicon_tags_2 },
    { 0x2dc17556, SUC_NE_lexicon_tags_7 },
    { 0xfa397559, SUC_NE_lexicon_tags_2 },
    { 0xc883755a, SUC_NE_lexicon_tags_2 },
    { 0x7c637557, SUC_NE_lexicon_tags_4 },
    { 0x18f5755c, SUC_NE_lexicon_tags_2 },
    { 0x70cf755c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b6b755f, SUC_NE_lexicon_tags_2 },
    { 0x20477560, SUC_NE_lexicon_tags_2 },
    { 0x22cb7560, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d7569, SUC_NE_lexicon_tags_2 },
    { 0x7fa57569, SUC_NE_lexicon_tags_2 },
    { 0x6541756a, SUC_NE_lexicon_tags_2 },
    { 0xbf3d756c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bab756e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1977570, SUC_NE_lexicon_tags_2 },
    { 0xfa6f7571, SUC_NE_lexicon_tags_2 },
    { 0x86477572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d0d7574, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0xdfcf7576, SUC_NE_lexicon_tags_7 },
    { 0x6ed37577, SUC_NE_lexicon_tags_2 },
    { 0x6f8f7577, SUC_NE_lexicon_tags_2 },
    { 0xaabd7579, SUC_NE_lexicon_tags_3 },
    { 0x74f1757a, SUC_NE_lexicon_tags_2 },
    { 0x801d7577, SUC_NE_lexicon_tags_2 },
    { 0xc0c7757c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fd7580, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bb7583, SUC_NE_lexicon_tags_7 },
    { 0x6a397583, SUC_NE_lexicon_tags_2 },
    { 0x628d7585, SUC_NE_lexicon_tags_2 },
    { 0xd43f7586, SUC_NE_lexicon_tags_4 },
    { 0x68477586, SUC_NE_lexicon_tags_2 },
    { 0x41937586, SUC_NE_lexicon_tags_2 },
    { 0x1f597588, SUC_NE_lexicon_tags_2 },
    { 0x80457583, SUC_NE_lexicon_tags_2 },
    { 0xfb4d758b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9c9758d, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0d7591, SUC_NE_lexicon_tags_2 },
    { 0xc0117591, SUC_NE_lexicon_tags_2 },
    { 0x6b4f7593, SUC_NE_lexicon_tags_2 },
    { 0x2ac17593, SUC_NE_lexicon_tags_2 },
    { 0x1d2b7594, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d57598, SUC_NE_lexicon_tags_2 },
    { 0x93637599, SUC_NE_lexicon_tags_19 },
    { 0xa93d759a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34b759d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82dd759f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0bb75a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0b975a3, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x294775a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8375ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa32175ad, SUC_NE_lexicon_tags_2 },
    { 0xc8db75ae, SUC_NE_lexicon_tags_2 },
    { 0x467975af, SUC_NE_lexicon_tags_2 },
    { 0x980775af, SUC_NE_lexicon_tags_2 },
    { 0x6d4975ae, SUC_NE_lexicon_tags_2 },
    { 0x54e575b2, SUC_NE_lexicon_tags_19 },
    { 0xcdf975af, SUC_NE_lexicon_tags_2 },
    { 0x49f775ae, SUC_NE_lexicon_tags_2 },
    { 0xa96775b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781d75b7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cd75ba, SUC_NE_lexicon_tags_2 },
    { 0x510175bb, SUC_NE_lexicon_tags_2 },
    { 0xeaf775bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b75c2, SUC_NE_lexicon_tags_2 },
    { 0x7f0d75c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bd75c6, SUC_NE_lexicon_tags_2 },
    { 0xac0175c6, SUC_NE_lexicon_tags_2 },
    { 0x65b375c8, SUC_NE_lexicon_tags_4 },
    { 0xacc575c9, SUC_NE_lexicon_tags_2 },
    { 0x5d2d75ca, SUC_NE_lexicon_tags_7 },
    { 0xb63d75cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeeb75cf, SUC_NE_lexicon_tags_2 },
    { 0xf38375d0, SUC_NE_lexicon_tags_2 },
    { 0x1d3575d1, SUC_NE_lexicon_tags_2 },
    { 0xefd575d2, SUC_NE_lexicon_tags_2 },
    { 0xe56f75d3, SUC_NE_lexicon_tags_2 },
    { 0xd4d975d4, SUC_NE_lexicon_tags_2 },
    { 0x355f75d3, SUC_NE_lexicon_tags_2 },
    { 0x8f2775d6, SUC_NE_lexicon_tags_2 },
    { 0xf4b375d7, SUC_NE_lexicon_tags_2 },
    { 0x132175d6, SUC_NE_lexicon_tags_2 },
    { 0xe5d975d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x492975dc, SUC_NE_lexicon_tags_2 },
    { 0x77d375dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb76375df, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28f75e2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4575e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71175e8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed75eb, SUC_NE_lexicon_tags_2 },
    { 0x365d75eb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29975f3, SUC_NE_lexicon_tags_2 },
    { 0x8efd75f4, SUC_NE_lexicon_tags_2 },
    { 0xcee375f5, SUC_NE_lexicon_tags_3 },
    { 0x8de575f6, SUC_NE_lexicon_tags_2 },
    { 0xa6e575f7, SUC_NE_lexicon_tags_2 },
    { 0x1e4375f8, SUC_NE_lexicon_tags_2 },
    { 0x7cb175f9, SUC_NE_lexicon_tags_2 },
    { 0x253f75f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0f75ff, SUC_NE_lexicon_tags_2 },
    { 0x5b557600, SUC_NE_lexicon_tags_2 },
    { 0x19a17600, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x6ae17603, SUC_NE_lexicon_tags_2 },
    { 0x62077603, SUC_NE_lexicon_tags_2 },
    { 0x9b057604, SUC_NE_lexicon_tags_2 },
    { 0x12e17606, SUC_NE_lexicon_tags_2 },
    { 0xee357603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba257609, SUC_NE_lexicon_tags_2 },
    { 0xbc79760a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dd7614, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5d37616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9edd7618, SUC_NE_lexicon_tags_2 },
    { 0xc7457619, SUC_NE_lexicon_tags_2 },
    { 0x2539761a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e41761f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d7629, SUC_NE_lexicon_tags_2 },
    { 0x734f762a, SUC_NE_lexicon_tags_2 },
    { 0x24f37629, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda7762e, SUC_NE_lexicon_tags_2 },
    { 0x6fab762f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabad7631, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa423763a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6605763d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe23763f, SUC_NE_lexicon_tags_2 },
    { 0xe2f1763f, SUC_NE_lexicon_tags_2 },
    { 0x15017641, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x253d7643, SUC_NE_lexicon_tags_13 },
    { 0x2baf7644, SUC_NE_lexicon_tags_2 },
    { 0xba5f7645, SUC_NE_lexicon_tags_2 },
    { 0x8d497646, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce437648, SUC_NE_lexicon_tags_2 },
    { 0xee777649, SUC_NE_lexicon_tags_2 },
    { 0x69017649, SUC_NE_lexicon_tags_2 },
    { 0x16217648, SUC_NE_lexicon_tags_2 },
    { 0x7ca5764c, SUC_NE_lexicon_tags_4 },
    { 0xf329764c, SUC_NE_lexicon_tags_2 },
    { 0x349d7648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad037653, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b6f7655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d17658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d765b, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf155765f, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0x211f7661, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa17667, SUC_NE_lexicon_tags_2 },
    { 0x1f5b7668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3766b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba657671, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28857673, SUC_NE_lexicon_tags_2 },
    { 0xe4b97674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6bb7676, SUC_NE_lexicon_tags_2 },
    { 0x23d37676, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x17137679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2ff767b, SUC_NE_lexicon_tags_2 },
    { 0xfc45767c, SUC_NE_lexicon_tags_1 },
    { 0x862d767d, SUC_NE_lexicon_tags_2 },
    { 0x8bd3767e, SUC_NE_lexicon_tags_2 },
    { 0x1ea5767b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c57683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fbb7685, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa37689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8711768c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x511768f, SUC_NE_lexicon_tags_2 },
    { 0x10897690, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68c77692, SUC_NE_lexicon_tags_2 },
    { 0xacf17693, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b77698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d7769f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95af76a2, SUC_NE_lexicon_tags_2 },
    { 0x863576a3, SUC_NE_lexicon_tags_4 },
    { 0x793176a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26af76a7, SUC_NE_lexicon_tags_2 },
    { 0x885f76a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x452d76ab, SUC_NE_lexicon_tags_2 },
    { 0xe39776ab, SUC_NE_lexicon_tags_2 },
    { 0x310376ad, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x941f76b0, SUC_NE_lexicon_tags_2 },
    { 0xb96176b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d7776b3, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead76b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x799f76b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fd76c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60ef76c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f176c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388d76d0, SUC_NE_lexicon_tags_2 },
    { 0xe01f76d1, SUC_NE_lexicon_tags_2 },
    { 0xa4f976d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed6176d4, SUC_NE_lexicon_tags_2 },
    { 0xac5176d5, SUC_NE_lexicon_tags_2 },
    { 0x86b776d6, SUC_NE_lexicon_tags_2 },
    { 0x550176d7, SUC_NE_lexicon_tags_2 },
    { 0x5acb76d8, SUC_NE_lexicon_tags_2 },
    { 0x22cb76d9, SUC_NE_lexicon_tags_2 },
    { 0x6ff976d9, SUC_NE_lexicon_tags_2 },
    { 0x50f976db, SUC_NE_lexicon_tags_2 },
    { 0xed5776d5, SUC_NE_lexicon_tags_2 },
    { 0x79c576dd, SUC_NE_lexicon_tags_2 },
    { 0xfb4776de, SUC_NE_lexicon_tags_2 },
    { 0x971576dc, SUC_NE_lexicon_tags_2 },
    { 0xc5dd76e0, SUC_NE_lexicon_tags_2 },
    { 0xb33576db, SUC_NE_lexicon_tags_2 },
    { 0xc98b76dd, SUC_NE_lexicon_tags_19 },
    { 0x33cb76e3, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0x1d9176e5, SUC_NE_lexicon_tags_2 },
    { 0x63ad76e5, SUC_NE_lexicon_tags_2 },
    { 0xc1cd76e7, SUC_NE_lexicon_tags_2 },
    { 0xba1b76e8, SUC_NE_lexicon_tags_0 },
    { 0x11af76e6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45976ed, SUC_NE_lexicon_tags_2 },
    { 0xe4cb76ee, SUC_NE_lexicon_tags_2 },
    { 0xcf8d76ef, SUC_NE_lexicon_tags_2 },
    { 0x8e976f0, SUC_NE_lexicon_tags_2 },
    { 0xe33776f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b76fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecd76fd, SUC_NE_lexicon_tags_2 },
    { 0x783f76fe, SUC_NE_lexicon_tags_2 },
    { 0x800976ff, SUC_NE_lexicon_tags_2 },
    { 0x630f76ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b770e, SUC_NE_lexicon_tags_2 },
    { 0x1d9770f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa037712, SUC_NE_lexicon_tags_2 },
    { 0x30c37713, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x626f7716, SUC_NE_lexicon_tags_2 },
    { 0x9a317717, SUC_NE_lexicon_tags_2 },
    { 0xb00d7717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf91f771a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa08b771c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36637722, SUC_NE_lexicon_tags_2 },
    { 0x51897722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efd7727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d5772d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8977731, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f0b7733, SUC_NE_lexicon_tags_2 },
    { 0xacaf7734, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e797736, SUC_NE_lexicon_tags_2 },
    { 0x59ff7736, SUC_NE_lexicon_tags_2 },
    { 0x18177738, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5773d, SUC_NE_lexicon_tags_2 },
    { 0x1d33773e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c1f7740, SUC_NE_lexicon_tags_2 },
    { 0x1dd7740, SUC_NE_lexicon_tags_34 },
    { 0xa88b7742, SUC_NE_lexicon_tags_2 },
    { 0xd4f7742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b17749, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee5774d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2677752, SUC_NE_lexicon_tags_7 },
    { 0xaaa57753, SUC_NE_lexicon_tags_2 },
    { 0x8e2d7754, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69d37756, SUC_NE_lexicon_tags_2 },
    { 0x3eff7757, SUC_NE_lexicon_tags_126 },
    { 0x694d7758, SUC_NE_lexicon_tags_7 },
    { 0x5c457758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc185775c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x382b775f, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xb72b7761, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab857763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5bb7765, SUC_NE_lexicon_tags_2 },
    { 0x9c1d7766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4357768, SUC_NE_lexicon_tags_2 },
    { 0x73c77768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ea3776b, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a1776e, SUC_NE_lexicon_tags_2 },
    { 0xfac5776e, SUC_NE_lexicon_tags_2 },
    { 0xd4a77770, SUC_NE_lexicon_tags_2 },
    { 0xb1997770, SUC_NE_lexicon_tags_2 },
    { 0x24837771, SUC_NE_lexicon_tags_2 },
    { 0xe8977773, SUC_NE_lexicon_tags_7 },
    { 0xe7757773, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f57778, SUC_NE_lexicon_tags_2 },
    { 0xdb57778, SUC_NE_lexicon_tags_2 },
    { 0x58f17778, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9777e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd77784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c17787, SUC_NE_lexicon_tags_2 },
    { 0x25cd7787, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76f778d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f95778f, SUC_NE_lexicon_tags_2 },
    { 0x20697790, SUC_NE_lexicon_tags_2 },
    { 0xcb3d7791, SUC_NE_lexicon_tags_2 },
    { 0xe0457792, SUC_NE_lexicon_tags_2 },
    { 0xf2a7778f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a5779a, SUC_NE_lexicon_tags_2 },
    { 0xbe93779b, SUC_NE_lexicon_tags_2 },
    { 0x99779c, SUC_NE_lexicon_tags_2 },
    { 0xb7af779b, SUC_NE_lexicon_tags_2 },
    { 0x3e95779c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863977a4, SUC_NE_lexicon_tags_2 },
    { 0x9ae377a5, SUC_NE_lexicon_tags_3 },
    { 0x3f8577a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa82b77a8, SUC_NE_lexicon_tags_2 },
    { 0x4c7777a8, SUC_NE_lexicon_tags_4 },
    { 0x91d977aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa377af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaa977b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1b77b4, SUC_NE_lexicon_tags_2 },
    { 0x7bb977b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb12177b7, SUC_NE_lexicon_tags_113 },
    { 0xea3777b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x992377ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb177bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e5f77bf, SUC_NE_lexicon_tags_2 },
    { 0xbb7f77c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2377c3, SUC_NE_lexicon_tags_2 },
    { 0x4a1f77c4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x733777c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41af77ca, SUC_NE_lexicon_tags_2 },
    { 0x372b77ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5d577cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38b77d3, SUC_NE_lexicon_tags_2 },
    { 0xec9977d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3177d6, SUC_NE_lexicon_tags_2 },
    { 0x9cd577d6, SUC_NE_lexicon_tags_2 },
    { 0xcf4177d7, SUC_NE_lexicon_tags_52 },
    { 0x993b77d7, SUC_NE_lexicon_tags_2 },
    { 0x4b1977d7, SUC_NE_lexicon_tags_2 },
    { 0xba5577db, SUC_NE_lexicon_tags_2 },
    { 0xfc5977dc, SUC_NE_lexicon_tags_2 },
    { 0x663777dd, SUC_NE_lexicon_tags_2 },
    { 0x902f77dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3177e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21c577e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16e977e5, SUC_NE_lexicon_tags_2 },
    { 0x6e9f77e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b977f2, SUC_NE_lexicon_tags_2 },
    { 0xa59777f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d777f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf74977fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x261f77fd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa70977ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e17802, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72317804, SUC_NE_lexicon_tags_11 },
    { 0xb4d7805, SUC_NE_lexicon_tags_2 },
    { 0x2b157806, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c1780c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472d7812, SUC_NE_lexicon_tags_2 },
    { 0x6ecd7813, SUC_NE_lexicon_tags_2 },
    { 0xedb37814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe79b7816, SUC_NE_lexicon_tags_2 },
    { 0x9e5b7816, SUC_NE_lexicon_tags_109 },
    { 0x31077818, SUC_NE_lexicon_tags_2 },
    { 0xa3237819, SUC_NE_lexicon_tags_2 },
    { 0x2967781a, SUC_NE_lexicon_tags_2 },
    { 0xd3781b, SUC_NE_lexicon_tags_3 },
    { 0x64f7781b, SUC_NE_lexicon_tags_2 },
    { 0x7ab3781d, SUC_NE_lexicon_tags_2 },
    { 0xb337781e, SUC_NE_lexicon_tags_4 },
    { 0xb489781e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb7822, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f57827, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcaa57829, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fd7831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11117836, SUC_NE_lexicon_tags_2 },
    { 0x242b7836, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b3783c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10b783f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd37841, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0097844, SUC_NE_lexicon_tags_18 },
    { 0x2fa17845, SUC_NE_lexicon_tags_2 },
    { 0x7b677846, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbf7849, SUC_NE_lexicon_tags_2 },
    { 0xea677849, SUC_NE_lexicon_tags_59 },
    { 0x95157849, SUC_NE_lexicon_tags_2 },
    { 0xa235784c, SUC_NE_lexicon_tags_2 },
    { 0xb9e3784d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2c7784f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96817858, SUC_NE_lexicon_tags_2 },
    { 0x71c17859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf933785b, SUC_NE_lexicon_tags_2 },
    { 0xbc8f785c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9785f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94d57861, SUC_NE_lexicon_tags_2 },
    { 0xda9f7862, SUC_NE_lexicon_tags_2 },
    { 0x54937863, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40eb7872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf5b7874, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2f7878, SUC_NE_lexicon_tags_2 },
    { 0xb70b7879, SUC_NE_lexicon_tags_2 },
    { 0xaeeb787a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcf787c, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed1787f, SUC_NE_lexicon_tags_2 },
    { 0xcb97880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecb7884, SUC_NE_lexicon_tags_65 },
    { 0, NULL },
    { 0x6a9d7886, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b697888, SUC_NE_lexicon_tags_2 },
    { 0xcb1d7889, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae37890, SUC_NE_lexicon_tags_210 },
    { 0x4fa17890, SUC_NE_lexicon_tags_2 },
    { 0x724d7892, SUC_NE_lexicon_tags_2 },
    { 0x7b6d7893, SUC_NE_lexicon_tags_2 },
    { 0xebbd7892, SUC_NE_lexicon_tags_2 },
    { 0xb6997894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa659789a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa127789e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64b78a0, SUC_NE_lexicon_tags_2 },
    { 0x528d78a1, SUC_NE_lexicon_tags_2 },
    { 0xc13378a2, SUC_NE_lexicon_tags_2 },
    { 0x449d78a3, SUC_NE_lexicon_tags_2 },
    { 0x678778a3, SUC_NE_lexicon_tags_2 },
    { 0x93cb78a3, SUC_NE_lexicon_tags_2 },
    { 0x5ff778a6, SUC_NE_lexicon_tags_2 },
    { 0x7cfb78a7, SUC_NE_lexicon_tags_2 },
    { 0x8b2378a8, SUC_NE_lexicon_tags_2 },
    { 0x5b5578a5, SUC_NE_lexicon_tags_4 },
    { 0xf94978aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x513778ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c9978ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27f578b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x699978b2, SUC_NE_lexicon_tags_2 },
    { 0xe49578b2, SUC_NE_lexicon_tags_2 },
    { 0xc82778b4, SUC_NE_lexicon_tags_2 },
    { 0x1e4f78b5, SUC_NE_lexicon_tags_17 },
    { 0xfdf378b4, SUC_NE_lexicon_tags_2 },
    { 0x751778b3, SUC_NE_lexicon_tags_2 },
    { 0x381178b8, SUC_NE_lexicon_tags_2 },
    { 0x66f978b8, SUC_NE_lexicon_tags_2 },
    { 0xf7f378b7, SUC_NE_lexicon_tags_2 },
    { 0xa8a178bb, SUC_NE_lexicon_tags_2 },
    { 0x360178bc, SUC_NE_lexicon_tags_2 },
    { 0x131d78bd, SUC_NE_lexicon_tags_2 },
    { 0xcc0f78bc, SUC_NE_lexicon_tags_2 },
    { 0x253978be, SUC_NE_lexicon_tags_2 },
    { 0x6df578c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x694378c3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xea7178c5, SUC_NE_lexicon_tags_2 },
    { 0x61f778c6, SUC_NE_lexicon_tags_2 },
    { 0xcd2d78c7, SUC_NE_lexicon_tags_2 },
    { 0x722778c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99778cb, SUC_NE_lexicon_tags_2 },
    { 0xb91178cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9778cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ef78d2, SUC_NE_lexicon_tags_2 },
    { 0x526778d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e378d7, SUC_NE_lexicon_tags_2 },
    { 0x6e0578d8, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0178dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x416d78df, SUC_NE_lexicon_tags_2 },
    { 0x9a9f78e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xade778e2, SUC_NE_lexicon_tags_2 },
    { 0xb2a378e3, SUC_NE_lexicon_tags_2 },
    { 0x183778e3, SUC_NE_lexicon_tags_2 },
    { 0x875b78e5, SUC_NE_lexicon_tags_2 },
    { 0x198978e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f78e9, SUC_NE_lexicon_tags_2 },
    { 0x8a4f78ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d978ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a0978ee, SUC_NE_lexicon_tags_2 },
    { 0x9f7178ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6378f4, SUC_NE_lexicon_tags_2 },
    { 0xaed978f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5378f9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b3f78fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52378fe, SUC_NE_lexicon_tags_11 },
    { 0x944978fe, SUC_NE_lexicon_tags_2 },
    { 0x1f7d7900, SUC_NE_lexicon_tags_2 },
    { 0x57697901, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bcf7903, SUC_NE_lexicon_tags_2 },
    { 0x94f77904, SUC_NE_lexicon_tags_2 },
    { 0xb99f7905, SUC_NE_lexicon_tags_2 },
    { 0x5237906, SUC_NE_lexicon_tags_7 },
    { 0xf7897905, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ed77909, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa299790d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6237917, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea2b7919, SUC_NE_lexicon_tags_2 },
    { 0xa3717919, SUC_NE_lexicon_tags_301 },
    { 0x970f791a, SUC_NE_lexicon_tags_50 },
    { 0xc7c57919, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd7922, SUC_NE_lexicon_tags_2 },
    { 0xdf2f7923, SUC_NE_lexicon_tags_2 },
    { 0x37e37924, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f357928, SUC_NE_lexicon_tags_17 },
    { 0xa3437929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccff792d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6457931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98357933, SUC_NE_lexicon_tags_2 },
    { 0xfb97934, SUC_NE_lexicon_tags_2 },
    { 0xb2557935, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f3793c, SUC_NE_lexicon_tags_2 },
    { 0x4867793d, SUC_NE_lexicon_tags_2 },
    { 0x5977793c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde497944, SUC_NE_lexicon_tags_2 },
    { 0x2b6b7945, SUC_NE_lexicon_tags_2 },
    { 0x80c77946, SUC_NE_lexicon_tags_2 },
    { 0x8a7d7947, SUC_NE_lexicon_tags_2 },
    { 0x747f7948, SUC_NE_lexicon_tags_34 },
    { 0x795f7949, SUC_NE_lexicon_tags_2 },
    { 0x4ca77946, SUC_NE_lexicon_tags_2 },
    { 0xf5f7945, SUC_NE_lexicon_tags_2 },
    { 0x1a63794c, SUC_NE_lexicon_tags_2 },
    { 0xf485794b, SUC_NE_lexicon_tags_2 },
    { 0xf693794e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cf7951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97617953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbf795d, SUC_NE_lexicon_tags_2 },
    { 0x7abb795e, SUC_NE_lexicon_tags_2 },
    { 0xed7795d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2d17961, SUC_NE_lexicon_tags_2 },
    { 0x8877961, SUC_NE_lexicon_tags_2 },
    { 0x1f357963, SUC_NE_lexicon_tags_2 },
    { 0x476d7964, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f7967, SUC_NE_lexicon_tags_2 },
    { 0x1feb7968, SUC_NE_lexicon_tags_2 },
    { 0xeffd7969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b3796d, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f37976, SUC_NE_lexicon_tags_6 },
    { 0xcfa17977, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x541d7979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d797d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x525b797f, SUC_NE_lexicon_tags_2 },
    { 0x72737980, SUC_NE_lexicon_tags_2 },
    { 0xf2317981, SUC_NE_lexicon_tags_1 },
    { 0xb52f7981, SUC_NE_lexicon_tags_1 },
    { 0xda37983, SUC_NE_lexicon_tags_2 },
    { 0x58877984, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43ef7986, SUC_NE_lexicon_tags_2 },
    { 0x25bd7986, SUC_NE_lexicon_tags_2 },
    { 0x961b7988, SUC_NE_lexicon_tags_2 },
    { 0x71b37988, SUC_NE_lexicon_tags_2 },
    { 0x24a9798a, SUC_NE_lexicon_tags_2 },
    { 0xd0eb798b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4137994, SUC_NE_lexicon_tags_2 },
    { 0xd8897995, SUC_NE_lexicon_tags_2 },
    { 0x31877994, SUC_NE_lexicon_tags_2 },
    { 0xc8e17996, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758b799b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d799e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9d79a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98779a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6979ad, SUC_NE_lexicon_tags_2 },
    { 0x6fb579ad, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0379b2, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7579b9, SUC_NE_lexicon_tags_2 },
    { 0x69f979ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75979bd, SUC_NE_lexicon_tags_30 },
    { 0x7e2f79bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf979c2, SUC_NE_lexicon_tags_2 },
    { 0xf2f79c3, SUC_NE_lexicon_tags_2 },
    { 0x244379c4, SUC_NE_lexicon_tags_2 },
    { 0x268179c4, SUC_NE_lexicon_tags_2 },
    { 0xb71979c5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc379d1, SUC_NE_lexicon_tags_2 },
    { 0x1c0b79d2, SUC_NE_lexicon_tags_2 },
    { 0x5e979d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a5179d5, SUC_NE_lexicon_tags_2 },
    { 0xb33379d6, SUC_NE_lexicon_tags_2 },
    { 0x3ef79d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9bb79d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x985d79db, SUC_NE_lexicon_tags_1 },
    { 0x545579dc, SUC_NE_lexicon_tags_2 },
    { 0xa76179dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x936979df, SUC_NE_lexicon_tags_2 },
    { 0x197979e0, SUC_NE_lexicon_tags_2 },
    { 0xb1a379e0, SUC_NE_lexicon_tags_2 },
    { 0xd20d79e0, SUC_NE_lexicon_tags_2 },
    { 0x329179e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a979ea, SUC_NE_lexicon_tags_2 },
    { 0xe2a979eb, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0xf6af79ed, SUC_NE_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4b79f0, SUC_NE_lexicon_tags_120 },
    { 0xd8779f1, SUC_NE_lexicon_tags_2 },
    { 0xabdf79f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb95179f4, SUC_NE_lexicon_tags_4 },
    { 0xa60b79f4, SUC_NE_lexicon_tags_2 },
    { 0x62bb79f4, SUC_NE_lexicon_tags_2 },
    { 0x335379f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96ff79f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa79379fb, SUC_NE_lexicon_tags_2 },
    { 0x786f79fb, SUC_NE_lexicon_tags_2 },
    { 0x297b79fb, SUC_NE_lexicon_tags_2 },
    { 0x5abf79fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x917d7a00, SUC_NE_lexicon_tags_270 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cd7a08, SUC_NE_lexicon_tags_2 },
    { 0x1df17a08, SUC_NE_lexicon_tags_2 },
    { 0x3e557a0a, SUC_NE_lexicon_tags_2 },
    { 0x6d977a0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279f7a0f, SUC_NE_lexicon_tags_2 },
    { 0x697d7a0f, SUC_NE_lexicon_tags_2 },
    { 0x8dad7a0f, SUC_NE_lexicon_tags_2 },
    { 0x20bb7a12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7257a16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33977a19, SUC_NE_lexicon_tags_2 },
    { 0x6a517a1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6b17a1c, SUC_NE_lexicon_tags_7 },
    { 0xcca37a1d, SUC_NE_lexicon_tags_3 },
    { 0xf37d7a1e, SUC_NE_lexicon_tags_2 },
    { 0xcd457a1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf38d7a21, SUC_NE_lexicon_tags_4 },
    { 0x49e77a21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f2b7a24, SUC_NE_lexicon_tags_2 },
    { 0x797b7a25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6b57a27, SUC_NE_lexicon_tags_2 },
    { 0xc7ff7a28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a577a2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f37a30, SUC_NE_lexicon_tags_2 },
    { 0x945b7a30, SUC_NE_lexicon_tags_2 },
    { 0x9c97a31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a77a37, SUC_NE_lexicon_tags_2 },
    { 0xc7777a38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37417a3a, SUC_NE_lexicon_tags_2 },
    { 0x3e537a3b, SUC_NE_lexicon_tags_2 },
    { 0xd87f7a3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54577a3f, SUC_NE_lexicon_tags_2 },
    { 0x56857a40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb397a45, SUC_NE_lexicon_tags_2 },
    { 0xca617a46, SUC_NE_lexicon_tags_4 },
    { 0xf2d57a47, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c37a4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da37a50, SUC_NE_lexicon_tags_2 },
    { 0x4ab17a50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31877a55, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe357a57, SUC_NE_lexicon_tags_30 },
    { 0xd1697a58, SUC_NE_lexicon_tags_2 },
    { 0x33017a59, SUC_NE_lexicon_tags_2 },
    { 0x59577a59, SUC_NE_lexicon_tags_2 },
    { 0x5c5f7a5b, SUC_NE_lexicon_tags_2 },
    { 0x7b297a59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce57a5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b57a66, SUC_NE_lexicon_tags_2 },
    { 0x3a037a67, SUC_NE_lexicon_tags_2 },
    { 0x3a7f7a68, SUC_NE_lexicon_tags_17 },
    { 0xcaaf7a67, SUC_NE_lexicon_tags_2 },
    { 0xe5877a6a, SUC_NE_lexicon_tags_2 },
    { 0xb9437a6b, SUC_NE_lexicon_tags_2 },
    { 0x28857a6c, SUC_NE_lexicon_tags_2 },
    { 0x68b37a69, SUC_NE_lexicon_tags_2 },
    { 0x816f7a69, SUC_NE_lexicon_tags_2 },
    { 0x35717a67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3537a71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d7a74, SUC_NE_lexicon_tags_2 },
    { 0xf0617a75, SUC_NE_lexicon_tags_2 },
    { 0x2f157a75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9b7a79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2297a7c, SUC_NE_lexicon_tags_2 },
    { 0xb8b7a7d, SUC_NE_lexicon_tags_2 },
    { 0x1d4f7a7d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa80f7a80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb997a82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x678b7a84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98697a87, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x83ed7a89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966b7a8e, SUC_NE_lexicon_tags_2 },
    { 0xbc117a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67c77a91, SUC_NE_lexicon_tags_2 },
    { 0xf3c17a92, SUC_NE_lexicon_tags_2 },
    { 0x10397a93, SUC_NE_lexicon_tags_7 },
    { 0x4bbd7a92, SUC_NE_lexicon_tags_2 },
    { 0x42897a95, SUC_NE_lexicon_tags_2 },
    { 0x70dd7a96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a99, SUC_NE_lexicon_tags_2 },
    { 0x1d977a99, SUC_NE_lexicon_tags_2 },
    { 0x4997a9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bb57a9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97d7aa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc617aab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2177aad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d357aaf, SUC_NE_lexicon_tags_2 },
    { 0x17a37ab0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cf7ab6, SUC_NE_lexicon_tags_2 },
    { 0x12f57ab7, SUC_NE_lexicon_tags_2 },
    { 0xf5b17ab7, SUC_NE_lexicon_tags_2 },
    { 0x39eb7ab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63a77abb, SUC_NE_lexicon_tags_2 },
    { 0x59937abc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86017abf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5b7ac2, SUC_NE_lexicon_tags_2 },
    { 0x61bd7ac3, SUC_NE_lexicon_tags_3 },
    { 0x89977ac4, SUC_NE_lexicon_tags_2 },
    { 0x29277ac5, SUC_NE_lexicon_tags_2 },
    { 0x55a97ac6, SUC_NE_lexicon_tags_4 },
    { 0xfddb7ac6, SUC_NE_lexicon_tags_2 },
    { 0xea9b7ac5, SUC_NE_lexicon_tags_2 },
    { 0xb6557ac9, SUC_NE_lexicon_tags_2 },
    { 0xa1737aca, SUC_NE_lexicon_tags_5 },
    { 0x17b17acb, SUC_NE_lexicon_tags_2 },
    { 0x937f7acb, SUC_NE_lexicon_tags_2 },
    { 0x85cf7ac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9177ad1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x76697ad3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xceef7ad5, SUC_NE_lexicon_tags_2 },
    { 0x466b7ad5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd677ad9, SUC_NE_lexicon_tags_2 },
    { 0xc5097ada, SUC_NE_lexicon_tags_2 },
    { 0x29b97ad9, SUC_NE_lexicon_tags_66 },
    { 0x2bc77adc, SUC_NE_lexicon_tags_2 },
    { 0x63b17add, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc717adf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd7ae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d437ae4, SUC_NE_lexicon_tags_2 },
    { 0xc1557ae4, SUC_NE_lexicon_tags_2 },
    { 0xd0a77ae4, SUC_NE_lexicon_tags_2 },
    { 0x39d97ae7, SUC_NE_lexicon_tags_2 },
    { 0xfe3f7ae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f677aea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91837aed, SUC_NE_lexicon_tags_2 },
    { 0x4fd57aee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417d7af2, SUC_NE_lexicon_tags_2 },
    { 0x5a97af2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b37af9, SUC_NE_lexicon_tags_2 },
    { 0x46237af9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31877afc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f717aff, SUC_NE_lexicon_tags_2 },
    { 0xdc057b00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10057b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79a17b09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ab7b0c, SUC_NE_lexicon_tags_2 },
    { 0xb63f7b0d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b7b10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f7b13, SUC_NE_lexicon_tags_2 },
    { 0x6c0d7b14, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0xb44b7b16, SUC_NE_lexicon_tags_4 },
    { 0x36957b17, SUC_NE_lexicon_tags_24 },
    { 0x60e37b17, SUC_NE_lexicon_tags_2 },
    { 0x2c0b7b17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dd7b1e, SUC_NE_lexicon_tags_2 },
    { 0x71217b1e, SUC_NE_lexicon_tags_2 },
    { 0x58077b20, SUC_NE_lexicon_tags_2 },
    { 0xf577b1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31117b24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a77b29, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeff7b2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa557b2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25077b35, SUC_NE_lexicon_tags_2 },
    { 0x777d7b35, SUC_NE_lexicon_tags_2 },
    { 0x31c17b37, SUC_NE_lexicon_tags_0 },
    { 0x11597b37, SUC_NE_lexicon_tags_2 },
    { 0xd5677b36, SUC_NE_lexicon_tags_2 },
    { 0x9637b35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47cf7b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f897b41, SUC_NE_lexicon_tags_2 },
    { 0xb87f7b42, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f57b45, SUC_NE_lexicon_tags_2 },
    { 0x30fd7b46, SUC_NE_lexicon_tags_2 },
    { 0xe7737b46, SUC_NE_lexicon_tags_2 },
    { 0x1a737b47, SUC_NE_lexicon_tags_2 },
    { 0xc08b7b46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98d7b4b, SUC_NE_lexicon_tags_2 },
    { 0x39677b4c, SUC_NE_lexicon_tags_2 },
    { 0x3f157b4b, SUC_NE_lexicon_tags_2 },
    { 0xc4d97b4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce37b50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea97b57, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf17b5a, SUC_NE_lexicon_tags_2 },
    { 0x5b457b5a, SUC_NE_lexicon_tags_2 },
    { 0xcd17b5c, SUC_NE_lexicon_tags_2 },
    { 0xdb57b5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46997b60, SUC_NE_lexicon_tags_2 },
    { 0x6ce17b61, SUC_NE_lexicon_tags_2 },
    { 0x40317b61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c537b64, SUC_NE_lexicon_tags_2 },
    { 0x8ea77b65, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d3f7b67, SUC_NE_lexicon_tags_4 },
    { 0xf8af7b68, SUC_NE_lexicon_tags_13 },
    { 0x2cd17b69, SUC_NE_lexicon_tags_2 },
    { 0x91957b68, SUC_NE_lexicon_tags_2 },
    { 0xe8b37b6b, SUC_NE_lexicon_tags_2 },
    { 0xfad37b68, SUC_NE_lexicon_tags_2 },
    { 0x49c17b6d, SUC_NE_lexicon_tags_2 },
    { 0xa4ed7b6d, SUC_NE_lexicon_tags_2 },
    { 0x17d7b6e, SUC_NE_lexicon_tags_2 },
    { 0x92797b70, SUC_NE_lexicon_tags_2 },
    { 0x51d97b71, SUC_NE_lexicon_tags_2 },
    { 0x81137b72, SUC_NE_lexicon_tags_2 },
    { 0xe1997b73, SUC_NE_lexicon_tags_2 },
    { 0xbdc37b70, SUC_NE_lexicon_tags_2 },
    { 0xff617b75, SUC_NE_lexicon_tags_2 },
    { 0x4fd57b6e, SUC_NE_lexicon_tags_2 },
    { 0x555f7b77, SUC_NE_lexicon_tags_2 },
    { 0xc1617b78, SUC_NE_lexicon_tags_65 },
    { 0x37ff7b79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bd7b7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ccd7b7f, SUC_NE_lexicon_tags_2 },
    { 0xc95b7b80, SUC_NE_lexicon_tags_2 },
    { 0xadb17b81, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdbd77b83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2837b85, SUC_NE_lexicon_tags_2 },
    { 0xe2897b86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13b7b8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7417b8e, SUC_NE_lexicon_tags_2 },
    { 0x32ab7b8f, SUC_NE_lexicon_tags_2 },
    { 0xc7a37b8e, SUC_NE_lexicon_tags_2 },
    { 0x548b7b90, SUC_NE_lexicon_tags_157 },
    { 0xab4b7b91, SUC_NE_lexicon_tags_2 },
    { 0x1ee77b93, SUC_NE_lexicon_tags_2 },
    { 0x71f57b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674b7b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a57b9d, SUC_NE_lexicon_tags_2 },
    { 0x712d7b9e, SUC_NE_lexicon_tags_0 },
    { 0x7dc77b9d, SUC_NE_lexicon_tags_2 },
    { 0xdf377ba0, SUC_NE_lexicon_tags_2 },
    { 0xb9737ba1, SUC_NE_lexicon_tags_2 },
    { 0xa7017b9e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x79357ba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be97ba7, SUC_NE_lexicon_tags_2 },
    { 0x6af57ba8, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xebe77baa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe817bad, SUC_NE_lexicon_tags_2 },
    { 0xb35d7bae, SUC_NE_lexicon_tags_2 },
    { 0xaf517baf, SUC_NE_lexicon_tags_2 },
    { 0xec9b7bb0, SUC_NE_lexicon_tags_2 },
    { 0xc03f7baf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca797bb5, SUC_NE_lexicon_tags_2 },
    { 0x124d7bb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52857bb8, SUC_NE_lexicon_tags_2 },
    { 0xe6737bb9, SUC_NE_lexicon_tags_2 },
    { 0xeb7f7bb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37bb7bbc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f77bc1, SUC_NE_lexicon_tags_2 },
    { 0xc7d37bc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6877bc4, SUC_NE_lexicon_tags_2 },
    { 0xbd177bc5, SUC_NE_lexicon_tags_7 },
    { 0xc77b7bc5, SUC_NE_lexicon_tags_1 },
    { 0x36377bc7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfbb7bc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe777bcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59db7bd5, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23897bd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d877bda, SUC_NE_lexicon_tags_2 },
    { 0xebc57bdb, SUC_NE_lexicon_tags_2 },
    { 0xd8ef7bdc, SUC_NE_lexicon_tags_7 },
    { 0xe6ad7bdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe37be2, SUC_NE_lexicon_tags_2 },
    { 0xda6b7be3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f277be5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccd7be9, SUC_NE_lexicon_tags_2 },
    { 0xe0c77bea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb217bed, SUC_NE_lexicon_tags_2 },
    { 0x2f1f7bed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1d7bf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b197bf5, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c337bf8, SUC_NE_lexicon_tags_2 },
    { 0x2a477bf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f97bfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93d7c00, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4277c02, SUC_NE_lexicon_tags_2 },
    { 0xe3897c02, SUC_NE_lexicon_tags_2 },
    { 0x23c77c04, SUC_NE_lexicon_tags_2 },
    { 0x9237c05, SUC_NE_lexicon_tags_2 },
    { 0x4cfd7c06, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0xcfb37c08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5b7c0b, SUC_NE_lexicon_tags_2 },
    { 0xc15d7c0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e77c0f, SUC_NE_lexicon_tags_2 },
    { 0x3cb37c10, SUC_NE_lexicon_tags_40 },
    { 0xb6717c11, SUC_NE_lexicon_tags_6 },
    { 0xb2f97c12, SUC_NE_lexicon_tags_3 },
    { 0x985f7c13, SUC_NE_lexicon_tags_2 },
    { 0x9f1b7c14, SUC_NE_lexicon_tags_2 },
    { 0xaa57c15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13b7c1d, SUC_NE_lexicon_tags_14 },
    { 0xd4ef7c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0977c24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68c17c26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d7c29, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x4cb7c2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35777c31, SUC_NE_lexicon_tags_2 },
    { 0x85d7c31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xead77c34, SUC_NE_lexicon_tags_2 },
    { 0xef077c35, SUC_NE_lexicon_tags_2 },
    { 0x7f0b7c36, SUC_NE_lexicon_tags_2 },
    { 0xb2cf7c37, SUC_NE_lexicon_tags_0 },
    { 0x3f717c36, SUC_NE_lexicon_tags_2 },
    { 0x793d7c38, SUC_NE_lexicon_tags_2 },
    { 0xd5f7c37, SUC_NE_lexicon_tags_2 },
    { 0x127f7c3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x457b7c3d, SUC_NE_lexicon_tags_2 },
    { 0xc977c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44c77c40, SUC_NE_lexicon_tags_2 },
    { 0x82b7c41, SUC_NE_lexicon_tags_2 },
    { 0x80f57c42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f97c46, SUC_NE_lexicon_tags_2 },
    { 0xe26b7c47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x918f7c49, SUC_NE_lexicon_tags_2 },
    { 0x3fc17c4a, SUC_NE_lexicon_tags_7 },
    { 0x138f7c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce457c4e, SUC_NE_lexicon_tags_4 },
    { 0xf3cd7c4f, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d7c53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fb7c56, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b7c59, SUC_NE_lexicon_tags_2 },
    { 0xc3a57c5a, SUC_NE_lexicon_tags_2 },
    { 0x23e37c5b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e77c5f, SUC_NE_lexicon_tags_2 },
    { 0x46917c60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebeb7c62, SUC_NE_lexicon_tags_4 },
    { 0xef937c63, SUC_NE_lexicon_tags_2 },
    { 0x20797c64, SUC_NE_lexicon_tags_2 },
    { 0x511b7c64, SUC_NE_lexicon_tags_2 },
    { 0x33857c65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c697c68, SUC_NE_lexicon_tags_2 },
    { 0x4e877c69, SUC_NE_lexicon_tags_2 },
    { 0xb3597c6a, SUC_NE_lexicon_tags_3 },
    { 0x2eff7c69, SUC_NE_lexicon_tags_2 },
    { 0xee077c68, SUC_NE_lexicon_tags_2 },
    { 0x520b7c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64977c6f, SUC_NE_lexicon_tags_57 },
    { 0xd5cd7c70, SUC_NE_lexicon_tags_2 },
    { 0x24577c70, SUC_NE_lexicon_tags_2 },
    { 0xf4877c71, SUC_NE_lexicon_tags_2 },
    { 0x5f5b7c72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4437c76, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7ca37c78, SUC_NE_lexicon_tags_2 },
    { 0x3da77c79, SUC_NE_lexicon_tags_2 },
    { 0xd92d7c78, SUC_NE_lexicon_tags_2 },
    { 0x1ed7c7b, SUC_NE_lexicon_tags_2 },
    { 0xc96d7c7b, SUC_NE_lexicon_tags_2 },
    { 0xccb97c7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e37c82, SUC_NE_lexicon_tags_2 },
    { 0x87397c82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x212b7c86, SUC_NE_lexicon_tags_2 },
    { 0xa4cd7c86, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e8d7c89, SUC_NE_lexicon_tags_2 },
    { 0x97777c89, SUC_NE_lexicon_tags_2 },
    { 0x44477c8b, SUC_NE_lexicon_tags_7 },
    { 0xb6177c8b, SUC_NE_lexicon_tags_2 },
    { 0xf7ff7c8d, SUC_NE_lexicon_tags_2 },
    { 0xa757c8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e097c92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9f7c97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde77c9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d4d7c9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d6b7ca0, SUC_NE_lexicon_tags_2 },
    { 0xedf17ca1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa317ca3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae77ca8, SUC_NE_lexicon_tags_2 },
    { 0x26e37ca9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3637cac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf77caf, SUC_NE_lexicon_tags_2 },
    { 0xbac17caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64397cb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61f97cb5, SUC_NE_lexicon_tags_3 },
    { 0xf38f7cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b97cb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85277cba, SUC_NE_lexicon_tags_2 },
    { 0xcdc57cba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb277cbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fb7cc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d417cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ccf7cc7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x85297cc9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf9cd7ccb, SUC_NE_lexicon_tags_2 },
    { 0x8ef17ccc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48297cd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf79d7cd2, SUC_NE_lexicon_tags_2 },
    { 0x870b7cd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c37cd6, SUC_NE_lexicon_tags_102 },
    { 0x13737cd7, SUC_NE_lexicon_tags_2 },
    { 0x27f57cd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4777cdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162f7ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16ef7ce3, SUC_NE_lexicon_tags_2 },
    { 0xf9877ce4, SUC_NE_lexicon_tags_2 },
    { 0x42c17ce5, SUC_NE_lexicon_tags_2 },
    { 0xee697ce4, SUC_NE_lexicon_tags_2 },
    { 0xa26d7ce7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87037ced, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f7cf1, SUC_NE_lexicon_tags_2 },
    { 0x4bab7cf2, SUC_NE_lexicon_tags_1 },
    { 0x52c57cf3, SUC_NE_lexicon_tags_2 },
    { 0x1aaf7cf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aed7cf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x292b7cf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65f7cfc, SUC_NE_lexicon_tags_2 },
    { 0xa7217cfd, SUC_NE_lexicon_tags_2 },
    { 0xc91d7cfc, SUC_NE_lexicon_tags_2 },
    { 0x97137cff, SUC_NE_lexicon_tags_1 },
    { 0x89017d00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501d7d04, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41d7d09, SUC_NE_lexicon_tags_5 },
    { 0x3b277d0a, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0x1a357d0c, SUC_NE_lexicon_tags_2 },
    { 0xd9d17d0c, SUC_NE_lexicon_tags_2 },
    { 0x81b97d0d, SUC_NE_lexicon_tags_2 },
    { 0xf20f7d0f, SUC_NE_lexicon_tags_2 },
    { 0xb8dd7d0e, SUC_NE_lexicon_tags_2 },
    { 0xa68d7d0c, SUC_NE_lexicon_tags_2 },
    { 0x23e77d11, SUC_NE_lexicon_tags_2 },
    { 0x45357d13, SUC_NE_lexicon_tags_2 },
    { 0xc8f37d14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc897d17, SUC_NE_lexicon_tags_2 },
    { 0x87ed7d18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fd7d1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97ab7d1c, SUC_NE_lexicon_tags_2 },
    { 0x76d77d1d, SUC_NE_lexicon_tags_2 },
    { 0x7d0f7d1c, SUC_NE_lexicon_tags_2 },
    { 0x41177d1f, SUC_NE_lexicon_tags_2 },
    { 0xc5c77d1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74977d29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1977d2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4cd7d2d, SUC_NE_lexicon_tags_2 },
    { 0x21ab7d2d, SUC_NE_lexicon_tags_2 },
    { 0xce8b7d2e, SUC_NE_lexicon_tags_2 },
    { 0x3377d30, SUC_NE_lexicon_tags_2 },
    { 0xfdbb7d31, SUC_NE_lexicon_tags_2 },
    { 0x51717d2f, SUC_NE_lexicon_tags_0 },
    { 0xf70b7d2d, SUC_NE_lexicon_tags_2 },
    { 0x5c357d2e, SUC_NE_lexicon_tags_17 },
    { 0x61df7d35, SUC_NE_lexicon_tags_2 },
    { 0x10577d36, SUC_NE_lexicon_tags_2 },
    { 0x68f77d2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8cb7d39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4657d3d, SUC_NE_lexicon_tags_2 },
    { 0x44bb7d3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61e57d40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80557d42, SUC_NE_lexicon_tags_2 },
    { 0x243b7d43, SUC_NE_lexicon_tags_2 },
    { 0xaf497d42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x272f7d46, SUC_NE_lexicon_tags_2 },
    { 0xe6037d46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69f7d4e, SUC_NE_lexicon_tags_3 },
    { 0x47a57d4e, SUC_NE_lexicon_tags_2 },
    { 0xdc557d50, SUC_NE_lexicon_tags_2 },
    { 0xdfe77d4e, SUC_NE_lexicon_tags_2 },
    { 0xf8ab7d52, SUC_NE_lexicon_tags_2 },
    { 0x498d7d52, SUC_NE_lexicon_tags_2 },
    { 0x6ec37d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7d5a, SUC_NE_lexicon_tags_2 },
    { 0x22037d5a, SUC_NE_lexicon_tags_13 },
    { 0x4ba77d5c, SUC_NE_lexicon_tags_2 },
    { 0x1c337d5d, SUC_NE_lexicon_tags_2 },
    { 0xcf3b7d5c, SUC_NE_lexicon_tags_2 },
    { 0x47657d5f, SUC_NE_lexicon_tags_2 },
    { 0x3e937d5e, SUC_NE_lexicon_tags_2 },
    { 0x83097d61, SUC_NE_lexicon_tags_2 },
    { 0xd41d7d61, SUC_NE_lexicon_tags_18 },
    { 0xc3737d63, SUC_NE_lexicon_tags_2 },
    { 0xbdf57d64, SUC_NE_lexicon_tags_2 },
    { 0x99bd7d61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25b7d68, SUC_NE_lexicon_tags_2 },
    { 0x7b117d69, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbc37d6b, SUC_NE_lexicon_tags_2 },
    { 0xa6a77d6c, SUC_NE_lexicon_tags_2 },
    { 0xa357d6d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d617d70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58af7d73, SUC_NE_lexicon_tags_2 },
    { 0x2d9d7d73, SUC_NE_lexicon_tags_4 },
    { 0x6b597d75, SUC_NE_lexicon_tags_2 },
    { 0x8a197d76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e97d7a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6257d7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bb37d80, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0f7d84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52617d87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80a97d89, SUC_NE_lexicon_tags_2 },
    { 0x279f7d8a, SUC_NE_lexicon_tags_2 },
    { 0x4b437d8a, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5c97d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69af7d8f, SUC_NE_lexicon_tags_2 },
    { 0xe8cb7d90, SUC_NE_lexicon_tags_4 },
    { 0xd117d91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafcb7d94, SUC_NE_lexicon_tags_2 },
    { 0x6ecb7d95, SUC_NE_lexicon_tags_2 },
    { 0x580d7d94, SUC_NE_lexicon_tags_2 },
    { 0xd1177d94, SUC_NE_lexicon_tags_2 },
    { 0xcaef7d95, SUC_NE_lexicon_tags_2 },
    { 0x1d317d99, SUC_NE_lexicon_tags_4 },
    { 0x1ef7d9a, SUC_NE_lexicon_tags_2 },
    { 0x77717d97, SUC_NE_lexicon_tags_2 },
    { 0x38857d97, SUC_NE_lexicon_tags_2 },
    { 0x483f7d97, SUC_NE_lexicon_tags_2 },
    { 0x23ab7d9e, SUC_NE_lexicon_tags_2 },
    { 0xcf837d96, SUC_NE_lexicon_tags_2 },
    { 0x773d7d9f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd2dd7da2, SUC_NE_lexicon_tags_2 },
    { 0x2b197da3, SUC_NE_lexicon_tags_3 },
    { 0xd5477da4, SUC_NE_lexicon_tags_2 },
    { 0xe8457da5, SUC_NE_lexicon_tags_2 },
    { 0xf7837da5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d57dad, SUC_NE_lexicon_tags_2 },
    { 0x5a997dad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79cd7db1, SUC_NE_lexicon_tags_19 },
    { 0xa8b97db2, SUC_NE_lexicon_tags_2 },
    { 0x338f7db3, SUC_NE_lexicon_tags_2 },
    { 0x47697db4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x78f97db6, SUC_NE_lexicon_tags_3 },
    { 0xeb9f7db7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97f7dbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6b7dc1, SUC_NE_lexicon_tags_2 },
    { 0x6b837dc2, SUC_NE_lexicon_tags_2 },
    { 0x119f7dc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83737dc6, SUC_NE_lexicon_tags_1 },
    { 0x8bc57dc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebb7dca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e097dcd, SUC_NE_lexicon_tags_2 },
    { 0xeff17dce, SUC_NE_lexicon_tags_2 },
    { 0x769b7dcf, SUC_NE_lexicon_tags_2 },
    { 0x21f37dce, SUC_NE_lexicon_tags_2 },
    { 0x23ad7dd1, SUC_NE_lexicon_tags_2 },
    { 0xa4eb7dd1, SUC_NE_lexicon_tags_2 },
    { 0x93b7dd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d77dd7, SUC_NE_lexicon_tags_2 },
    { 0xc7837dd7, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3f7ddb, SUC_NE_lexicon_tags_3 },
    { 0xf88b7ddc, SUC_NE_lexicon_tags_3 },
    { 0xb73d7ddc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb57ddf, SUC_NE_lexicon_tags_2 },
    { 0x23b7de0, SUC_NE_lexicon_tags_4 },
    { 0x1e6f7de1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e1f7de3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5537de8, SUC_NE_lexicon_tags_2 },
    { 0x95d57de8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f17dee, SUC_NE_lexicon_tags_2 },
    { 0xd4857dee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c657df1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21997df4, SUC_NE_lexicon_tags_3 },
    { 0x27217df5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc57b7df7, SUC_NE_lexicon_tags_49 },
    { 0x23b57df8, SUC_NE_lexicon_tags_2 },
    { 0x28ff7df8, SUC_NE_lexicon_tags_2 },
    { 0x91497dfa, SUC_NE_lexicon_tags_2 },
    { 0x68937df8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b7e04, SUC_NE_lexicon_tags_2 },
    { 0xe4a97e04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x933f7e07, SUC_NE_lexicon_tags_19 },
    { 0xcbb17e07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31597e0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fd7e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecf7e1e, SUC_NE_lexicon_tags_18 },
    { 0x85417e1f, SUC_NE_lexicon_tags_3 },
    { 0x6dcf7e20, SUC_NE_lexicon_tags_2 },
    { 0x53097e1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eb7e25, SUC_NE_lexicon_tags_2 },
    { 0xf3797e25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd257e2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9437e2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8fd7e2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f7e32, SUC_NE_lexicon_tags_2 },
    { 0x2f2f7e33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a97e36, SUC_NE_lexicon_tags_2 },
    { 0x78a17e37, SUC_NE_lexicon_tags_2 },
    { 0xc4d97e37, SUC_NE_lexicon_tags_2 },
    { 0xd1d17e37, SUC_NE_lexicon_tags_7 },
    { 0x2de77e3a, SUC_NE_lexicon_tags_2 },
    { 0x212f7e3b, SUC_NE_lexicon_tags_2 },
    { 0xf9417e38, SUC_NE_lexicon_tags_2 },
    { 0x33cd7e38, SUC_NE_lexicon_tags_2 },
    { 0x27cd7e3e, SUC_NE_lexicon_tags_2 },
    { 0x648b7e38, SUC_NE_lexicon_tags_2 },
    { 0xcb917e40, SUC_NE_lexicon_tags_2 },
    { 0xac2d7e41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c917e46, SUC_NE_lexicon_tags_2 },
    { 0xc2f17e46, SUC_NE_lexicon_tags_2 },
    { 0xed4f7e48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd397e4d, SUC_NE_lexicon_tags_2 },
    { 0xce1b7e4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3237e52, SUC_NE_lexicon_tags_2 },
    { 0x94997e52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed17e57, SUC_NE_lexicon_tags_2 },
    { 0xdad37e58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54557e5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b7e5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68437e66, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x78357e69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0617e6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb7e70, SUC_NE_lexicon_tags_437 },
    { 0x15357e71, SUC_NE_lexicon_tags_2 },
    { 0x5d3f7e72, SUC_NE_lexicon_tags_2 },
    { 0xbd857e72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2297e75, SUC_NE_lexicon_tags_2 },
    { 0xd3397e76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72917e78, SUC_NE_lexicon_tags_2 },
    { 0x63e57e79, SUC_NE_lexicon_tags_2 },
    { 0xb56f7e7a, SUC_NE_lexicon_tags_2 },
    { 0xc72f7e7b, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd377e7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccf7e85, SUC_NE_lexicon_tags_2 },
    { 0x218d7e86, SUC_NE_lexicon_tags_2 },
    { 0x64977e87, SUC_NE_lexicon_tags_2 },
    { 0x67e37e87, SUC_NE_lexicon_tags_2 },
    { 0x8a5f7e89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7977e8b, SUC_NE_lexicon_tags_2 },
    { 0x8c9b7e8b, SUC_NE_lexicon_tags_358 },
    { 0x5d617e8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56677e8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed7e93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2f7e99, SUC_NE_lexicon_tags_2 },
    { 0x66b37e99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11d7e9e, SUC_NE_lexicon_tags_2 },
    { 0x17697e9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b57ea3, SUC_NE_lexicon_tags_2 },
    { 0x873d7ea4, SUC_NE_lexicon_tags_2 },
    { 0xac697ea4, SUC_NE_lexicon_tags_2 },
    { 0x85137ea3, SUC_NE_lexicon_tags_2 },
    { 0xc1b37ea3, SUC_NE_lexicon_tags_4 },
    { 0x7abb7ea8, SUC_NE_lexicon_tags_2 },
    { 0xd237ea9, SUC_NE_lexicon_tags_2 },
    { 0xc3577eaa, SUC_NE_lexicon_tags_2 },
    { 0x7bcf7ea6, SUC_NE_lexicon_tags_4 },
    { 0x4db7eac, SUC_NE_lexicon_tags_2 },
    { 0x28437ead, SUC_NE_lexicon_tags_2 },
    { 0xba537eaa, SUC_NE_lexicon_tags_3 },
    { 0xac817eaf, SUC_NE_lexicon_tags_2 },
    { 0x12077eaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7eb4, SUC_NE_lexicon_tags_2 },
    { 0x19037eb5, SUC_NE_lexicon_tags_7 },
    { 0xac0f7eb6, SUC_NE_lexicon_tags_2 },
    { 0xe62d7eb5, SUC_NE_lexicon_tags_2 },
    { 0xe6ab7eb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4697ebb, SUC_NE_lexicon_tags_2 },
    { 0xda9b7ebc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1f7ebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31457ec4, SUC_NE_lexicon_tags_2 },
    { 0x8cb37ec4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x286b7ec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb37ecb, SUC_NE_lexicon_tags_2 },
    { 0x68537ecb, SUC_NE_lexicon_tags_3 },
    { 0x4d357ecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600b7ed2, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x9577ed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80b17ed6, SUC_NE_lexicon_tags_2 },
    { 0xc4cf7ed7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964b7edf, SUC_NE_lexicon_tags_2 },
    { 0xbe817ee0, SUC_NE_lexicon_tags_2 },
    { 0xd15f7ee1, SUC_NE_lexicon_tags_2 },
    { 0xe4c97ee0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56017ee7, SUC_NE_lexicon_tags_2 },
    { 0x57877ee8, SUC_NE_lexicon_tags_0 },
    { 0x7a397ee9, SUC_NE_lexicon_tags_2 },
    { 0xd2f37eea, SUC_NE_lexicon_tags_2 },
    { 0xddcf7eea, SUC_NE_lexicon_tags_2 },
    { 0x37eeb, SUC_NE_lexicon_tags_2 },
    { 0xfebd7ee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4377eef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba17ef2, SUC_NE_lexicon_tags_2 },
    { 0xabdd7ef3, SUC_NE_lexicon_tags_2 },
    { 0xbae77ef4, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba77ef7, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x936d7efa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f7efe, SUC_NE_lexicon_tags_2 },
    { 0x376b7eff, SUC_NE_lexicon_tags_2 },
    { 0x6c537f00, SUC_NE_lexicon_tags_2 },
    { 0x90577f00, SUC_NE_lexicon_tags_2 },
    { 0x75357f01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f7f06, SUC_NE_lexicon_tags_4 },
    { 0x999d7f07, SUC_NE_lexicon_tags_2 },
    { 0x960d7f08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d17f0c, SUC_NE_lexicon_tags_2 },
    { 0xcf137f0d, SUC_NE_lexicon_tags_1 },
    { 0xdf277f0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77357f10, SUC_NE_lexicon_tags_2 },
    { 0x20517f11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c417f16, SUC_NE_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca417f1b, SUC_NE_lexicon_tags_2 },
    { 0xca117f1b, SUC_NE_lexicon_tags_2 },
    { 0x91997f1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c837f21, SUC_NE_lexicon_tags_2 },
    { 0x13757f21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3817f26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29397f2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59017f2d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd957f31, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685d7f36, SUC_NE_lexicon_tags_2 },
    { 0xe9e37f37, SUC_NE_lexicon_tags_2 },
    { 0xf1ef7f37, SUC_NE_lexicon_tags_2 },
    { 0xdf437f37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59817f3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a577f41, SUC_NE_lexicon_tags_2 },
    { 0x10f7f42, SUC_NE_lexicon_tags_2 },
    { 0x180f7f41, SUC_NE_lexicon_tags_2 },
    { 0x152f7f44, SUC_NE_lexicon_tags_3 },
    { 0x8b457f45, SUC_NE_lexicon_tags_2 },
    { 0x42597f46, SUC_NE_lexicon_tags_2 },
    { 0x76537f45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42617f4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0077f4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49417f53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67997f55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc597f5d, SUC_NE_lexicon_tags_2 },
    { 0x1b0f7f5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41837f61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aed7f64, SUC_NE_lexicon_tags_4 },
    { 0x36197f65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x630b7f68, SUC_NE_lexicon_tags_2 },
    { 0x67497f69, SUC_NE_lexicon_tags_18 },
    { 0xb9c77f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42e37f6c, SUC_NE_lexicon_tags_2 },
    { 0xb4777f6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc10b7f6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x718d7f72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c157f75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60d7f79, SUC_NE_lexicon_tags_4 },
    { 0xdc4f7f7a, SUC_NE_lexicon_tags_2 },
    { 0x1f5f7f7a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34237f7f, SUC_NE_lexicon_tags_2 },
    { 0x504f7f7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76997f82, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d37f85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80957f8a, SUC_NE_lexicon_tags_2 },
    { 0x9a177f8a, SUC_NE_lexicon_tags_2 },
    { 0xb9877f8c, SUC_NE_lexicon_tags_2 },
    { 0x22fb7f8c, SUC_NE_lexicon_tags_2 },
    { 0xe04f7f8c, SUC_NE_lexicon_tags_2 },
    { 0x8c197f8c, SUC_NE_lexicon_tags_2 },
    { 0x91137f8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67117f97, SUC_NE_lexicon_tags_2 },
    { 0x9d497f98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47557f9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbb7f9d, SUC_NE_lexicon_tags_2 },
    { 0x19bf7f9e, SUC_NE_lexicon_tags_2 },
    { 0x313b7f9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5437fa9, SUC_NE_lexicon_tags_2 },
    { 0x97577fa9, SUC_NE_lexicon_tags_2 },
    { 0xc6c77fab, SUC_NE_lexicon_tags_2 },
    { 0x42d77fac, SUC_NE_lexicon_tags_98 },
    { 0x213f7fa9, SUC_NE_lexicon_tags_2 },
    { 0x12577fae, SUC_NE_lexicon_tags_2 },
    { 0xb3477faf, SUC_NE_lexicon_tags_4 },
    { 0xafdd7fb0, SUC_NE_lexicon_tags_2 },
    { 0x896f7faf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8b7fb6, SUC_NE_lexicon_tags_19 },
    { 0x7b5d7fb7, SUC_NE_lexicon_tags_2 },
    { 0xb9277fb8, SUC_NE_lexicon_tags_2 },
    { 0x874d7fb9, SUC_NE_lexicon_tags_14 },
    { 0x16917fba, SUC_NE_lexicon_tags_34 },
    { 0x323f7fba, SUC_NE_lexicon_tags_2 },
    { 0x11c57fbc, SUC_NE_lexicon_tags_4 },
    { 0xde5f7fba, SUC_NE_lexicon_tags_2 },
    { 0xa4157fb9, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2057fc2, SUC_NE_lexicon_tags_2 },
    { 0x29897fc2, SUC_NE_lexicon_tags_2 },
    { 0x9957fc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c497fc6, SUC_NE_lexicon_tags_2 },
    { 0x45177fc7, SUC_NE_lexicon_tags_2 },
    { 0x44037fc7, SUC_NE_lexicon_tags_1 },
    { 0xd8bb7fc8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x95a77fcb, SUC_NE_lexicon_tags_2 },
    { 0xef9d7fcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f377fce, SUC_NE_lexicon_tags_2 },
    { 0x9a77fcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba097fd1, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x22eb7fd3, SUC_NE_lexicon_tags_2 },
    { 0x94237fd4, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3717fd8, SUC_NE_lexicon_tags_1 },
    { 0x25097fd9, SUC_NE_lexicon_tags_2 },
    { 0xc20f7fd9, SUC_NE_lexicon_tags_2 },
    { 0x29757fdb, SUC_NE_lexicon_tags_7 },
    { 0xb3e17fdb, SUC_NE_lexicon_tags_2 },
    { 0xefc37fdd, SUC_NE_lexicon_tags_7 },
    { 0xb7ef7fda, SUC_NE_lexicon_tags_2 },
    { 0x50857fde, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b8d7fe1, SUC_NE_lexicon_tags_2 },
    { 0xb2217fe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d37fe8, SUC_NE_lexicon_tags_2 },
    { 0x9f457fe9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a137fec, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f97fef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae497ff1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x264b7ff3, SUC_NE_lexicon_tags_2 },
    { 0x36b97ff3, SUC_NE_lexicon_tags_2 },
    { 0xcceb7ff5, SUC_NE_lexicon_tags_2 },
    { 0xf0f37ff5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x491d7ff8, SUC_NE_lexicon_tags_2 },
    { 0x55497ff8, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x78dd7ffb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c978002, SUC_NE_lexicon_tags_111 },
    { 0x1f3b8003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cd8006, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3405800a, SUC_NE_lexicon_tags_18 },
    { 0xd683800a, SUC_NE_lexicon_tags_2 },
    { 0x4e7800c, SUC_NE_lexicon_tags_2 },
    { 0x2317800c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d8011, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3fd8013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47a78015, SUC_NE_lexicon_tags_2 },
    { 0x640f8015, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0b801a, SUC_NE_lexicon_tags_2 },
    { 0x491b801b, SUC_NE_lexicon_tags_2 },
    { 0xc117801b, SUC_NE_lexicon_tags_2 },
    { 0xc0d7801b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe558021, SUC_NE_lexicon_tags_2 },
    { 0xe8db8022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb278025, SUC_NE_lexicon_tags_2 },
    { 0xf4098025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cb802a, SUC_NE_lexicon_tags_2 },
    { 0x8fb7802a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x887802d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x746d802f, SUC_NE_lexicon_tags_19 },
    { 0x36fd8030, SUC_NE_lexicon_tags_2 },
    { 0xef658031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d8034, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe358036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a38038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4d803b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c67803f, SUC_NE_lexicon_tags_2 },
    { 0x44ab8040, SUC_NE_lexicon_tags_2 },
    { 0xa893803f, SUC_NE_lexicon_tags_2 },
    { 0xf15b8041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbff8044, SUC_NE_lexicon_tags_2 },
    { 0x30d18044, SUC_NE_lexicon_tags_2 },
    { 0x49d38046, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e578049, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b804c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85518050, SUC_NE_lexicon_tags_2 },
    { 0x85238051, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38438054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e9805a, SUC_NE_lexicon_tags_2 },
    { 0x3ce5805b, SUC_NE_lexicon_tags_2 },
    { 0x9d15805a, SUC_NE_lexicon_tags_2 },
    { 0x72d5805d, SUC_NE_lexicon_tags_2 },
    { 0x6c7805a, SUC_NE_lexicon_tags_2 },
    { 0x3eb7805e, SUC_NE_lexicon_tags_2 },
    { 0x279d8060, SUC_NE_lexicon_tags_2 },
    { 0x48398061, SUC_NE_lexicon_tags_2 },
    { 0x39c78062, SUC_NE_lexicon_tags_81 },
    { 0xb7118063, SUC_NE_lexicon_tags_2 },
    { 0x110b8064, SUC_NE_lexicon_tags_2 },
    { 0xf53b8063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5b8069, SUC_NE_lexicon_tags_2 },
    { 0xbf61806a, SUC_NE_lexicon_tags_2 },
    { 0x35af806b, SUC_NE_lexicon_tags_2 },
    { 0x9985806b, SUC_NE_lexicon_tags_19 },
    { 0xcf35806c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1535806f, SUC_NE_lexicon_tags_2 },
    { 0x68e58070, SUC_NE_lexicon_tags_2 },
    { 0x6c798071, SUC_NE_lexicon_tags_2 },
    { 0x27d98072, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de58078, SUC_NE_lexicon_tags_1 },
    { 0x8e4d8079, SUC_NE_lexicon_tags_2 },
    { 0xf541807a, SUC_NE_lexicon_tags_2 },
    { 0xc4b18079, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7ab807d, SUC_NE_lexicon_tags_2 },
    { 0x4af5807e, SUC_NE_lexicon_tags_2 },
    { 0x566b807d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa678081, SUC_NE_lexicon_tags_7 },
    { 0xaa0d8082, SUC_NE_lexicon_tags_5 },
    { 0x88e78083, SUC_NE_lexicon_tags_2 },
    { 0x64618084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1498088, SUC_NE_lexicon_tags_2 },
    { 0x35838088, SUC_NE_lexicon_tags_2 },
    { 0xc6a38089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cdd808c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdab9808e, SUC_NE_lexicon_tags_2 },
    { 0x8f23808f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc98093, SUC_NE_lexicon_tags_2 },
    { 0x17118094, SUC_NE_lexicon_tags_65 },
    { 0x408d8095, SUC_NE_lexicon_tags_2 },
    { 0x73718095, SUC_NE_lexicon_tags_2 },
    { 0xfcb8097, SUC_NE_lexicon_tags_2 },
    { 0x8c098098, SUC_NE_lexicon_tags_7 },
    { 0x9e1d8097, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x138b809c, SUC_NE_lexicon_tags_2 },
    { 0x1eb7809c, SUC_NE_lexicon_tags_2 },
    { 0xbd0d809c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe6f80a0, SUC_NE_lexicon_tags_2 },
    { 0x797b80a1, SUC_NE_lexicon_tags_2 },
    { 0xc47380a1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x676980a5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xcce980a7, SUC_NE_lexicon_tags_2 },
    { 0x4b8380a7, SUC_NE_lexicon_tags_2 },
    { 0xc7d80a8, SUC_NE_lexicon_tags_2 },
    { 0xa3c580aa, SUC_NE_lexicon_tags_2 },
    { 0x27b180aa, SUC_NE_lexicon_tags_2 },
    { 0x837b80a8, SUC_NE_lexicon_tags_2 },
    { 0x319d80ad, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649380b1, SUC_NE_lexicon_tags_2 },
    { 0x3e6b80b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x164b80b5, SUC_NE_lexicon_tags_7 },
    { 0x3e2580b6, SUC_NE_lexicon_tags_2 },
    { 0x390580b7, SUC_NE_lexicon_tags_2 },
    { 0x2c1d80b8, SUC_NE_lexicon_tags_2 },
    { 0xe22980b5, SUC_NE_lexicon_tags_2 },
    { 0x8b3b80ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbb80bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x993380bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86d80c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84f80ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x102580cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5380d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5d80db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144980df, SUC_NE_lexicon_tags_2 },
    { 0x9a0580e0, SUC_NE_lexicon_tags_7 },
    { 0x74e180e1, SUC_NE_lexicon_tags_2 },
    { 0x929d80e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1beb80e4, SUC_NE_lexicon_tags_2 },
    { 0xda9380e5, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e180eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d180f2, SUC_NE_lexicon_tags_34 },
    { 0x3acf80f3, SUC_NE_lexicon_tags_2 },
    { 0xcda580f3, SUC_NE_lexicon_tags_2 },
    { 0x2f0d80f5, SUC_NE_lexicon_tags_2 },
    { 0x1e0d80f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcb580f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0180fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcee980fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a558100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d8108, SUC_NE_lexicon_tags_2 },
    { 0x31578108, SUC_NE_lexicon_tags_2 },
    { 0xeda5810a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2325810c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15178113, SUC_NE_lexicon_tags_2 },
    { 0x42b38113, SUC_NE_lexicon_tags_113 },
    { 0x61eb8113, SUC_NE_lexicon_tags_2 },
    { 0x71d8114, SUC_NE_lexicon_tags_2 },
    { 0xd6798117, SUC_NE_lexicon_tags_2 },
    { 0x1f238118, SUC_NE_lexicon_tags_2 },
    { 0x51e78116, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c5811e, SUC_NE_lexicon_tags_2 },
    { 0xfbd3811f, SUC_NE_lexicon_tags_186 },
    { 0xab7b811f, SUC_NE_lexicon_tags_7 },
    { 0xbbe58120, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xec018124, SUC_NE_lexicon_tags_2 },
    { 0x218d8125, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5d8128, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc337812a, SUC_NE_lexicon_tags_2 },
    { 0x56a5812b, SUC_NE_lexicon_tags_2 },
    { 0x83c9812b, SUC_NE_lexicon_tags_17 },
    { 0xa07f812d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x288b812f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a118132, SUC_NE_lexicon_tags_2 },
    { 0xad8b8132, SUC_NE_lexicon_tags_24 },
    { 0x89438133, SUC_NE_lexicon_tags_2 },
    { 0x3e9d8135, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6395813d, SUC_NE_lexicon_tags_2 },
    { 0x1e29813e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfe58140, SUC_NE_lexicon_tags_2 },
    { 0x89f8140, SUC_NE_lexicon_tags_2 },
    { 0x9aa58142, SUC_NE_lexicon_tags_2 },
    { 0xe1c58143, SUC_NE_lexicon_tags_2 },
    { 0x9def8141, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a78149, SUC_NE_lexicon_tags_2 },
    { 0xcc118149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89d814d, SUC_NE_lexicon_tags_2 },
    { 0xff07814e, SUC_NE_lexicon_tags_4 },
    { 0x1b95814e, SUC_NE_lexicon_tags_2 },
    { 0x2841814d, SUC_NE_lexicon_tags_2 },
    { 0x900b814f, SUC_NE_lexicon_tags_2 },
    { 0xb8a78150, SUC_NE_lexicon_tags_3 },
    { 0x23038153, SUC_NE_lexicon_tags_2 },
    { 0x8469814f, SUC_NE_lexicon_tags_2 },
    { 0xde2b8155, SUC_NE_lexicon_tags_2 },
    { 0x56798156, SUC_NE_lexicon_tags_2 },
    { 0x4a358157, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4187815a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9098160, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8164, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50898166, SUC_NE_lexicon_tags_2 },
    { 0x96078167, SUC_NE_lexicon_tags_4 },
    { 0x2c578166, SUC_NE_lexicon_tags_4 },
    { 0xe8378169, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38178170, SUC_NE_lexicon_tags_2 },
    { 0xa0c98170, SUC_NE_lexicon_tags_2 },
    { 0xd8f98172, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65118177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc7817c, SUC_NE_lexicon_tags_2 },
    { 0x310d817d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5b8182, SUC_NE_lexicon_tags_2 },
    { 0x7c298183, SUC_NE_lexicon_tags_2 },
    { 0x6e578184, SUC_NE_lexicon_tags_2 },
    { 0x90238185, SUC_NE_lexicon_tags_2 },
    { 0xf85b8185, SUC_NE_lexicon_tags_2 },
    { 0x730d8187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c78191, SUC_NE_lexicon_tags_2 },
    { 0x35118192, SUC_NE_lexicon_tags_2 },
    { 0xe7458193, SUC_NE_lexicon_tags_3 },
    { 0x975d8193, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b8198, SUC_NE_lexicon_tags_2 },
    { 0x4e8f8199, SUC_NE_lexicon_tags_2 },
    { 0x6457819a, SUC_NE_lexicon_tags_2 },
    { 0x5fa9819a, SUC_NE_lexicon_tags_2 },
    { 0x1353819c, SUC_NE_lexicon_tags_2 },
    { 0xb423819c, SUC_NE_lexicon_tags_2 },
    { 0xbaf3819b, SUC_NE_lexicon_tags_2 },
    { 0xf315819f, SUC_NE_lexicon_tags_0 },
    { 0x89cb819f, SUC_NE_lexicon_tags_2 },
    { 0x14358198, SUC_NE_lexicon_tags_2 },
    { 0xe95581a2, SUC_NE_lexicon_tags_2 },
    { 0xb2b381a3, SUC_NE_lexicon_tags_2 },
    { 0x23b581a3, SUC_NE_lexicon_tags_2 },
    { 0xedc78198, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1381ab, SUC_NE_lexicon_tags_2 },
    { 0x65d381ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f781af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x264581b1, SUC_NE_lexicon_tags_2 },
    { 0xe6a981b2, SUC_NE_lexicon_tags_2 },
    { 0x777d81b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac9b81b5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27981b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bb81c2, SUC_NE_lexicon_tags_2 },
    { 0x5e9381c2, SUC_NE_lexicon_tags_2 },
    { 0xc94b81c2, SUC_NE_lexicon_tags_2 },
    { 0x431b81c4, SUC_NE_lexicon_tags_2 },
    { 0x3d3581c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef781ca, SUC_NE_lexicon_tags_2 },
    { 0x993381ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e381ce, SUC_NE_lexicon_tags_2 },
    { 0xb26781cf, SUC_NE_lexicon_tags_2 },
    { 0xb3f381cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b9b81d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be181d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x905181d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42581dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36f81df, SUC_NE_lexicon_tags_2 },
    { 0xc99981e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b381e2, SUC_NE_lexicon_tags_2 },
    { 0x551781e3, SUC_NE_lexicon_tags_2 },
    { 0x926f81e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x692381e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765981ea, SUC_NE_lexicon_tags_2 },
    { 0x341581eb, SUC_NE_lexicon_tags_2 },
    { 0xd0cf81eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a6581ee, SUC_NE_lexicon_tags_2 },
    { 0xf56581ef, SUC_NE_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2581f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55d381f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x873f81f9, SUC_NE_lexicon_tags_2 },
    { 0xd42d81f9, SUC_NE_lexicon_tags_2 },
    { 0x113181fb, SUC_NE_lexicon_tags_24 },
    { 0xafd981fb, SUC_NE_lexicon_tags_2 },
    { 0x126f81fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac18206, SUC_NE_lexicon_tags_2 },
    { 0xebb98207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe5820c, SUC_NE_lexicon_tags_2 },
    { 0x36d7820d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeb8213, SUC_NE_lexicon_tags_21 },
    { 0x686f8214, SUC_NE_lexicon_tags_2 },
    { 0x6df78214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97578219, SUC_NE_lexicon_tags_2 },
    { 0x593f821a, SUC_NE_lexicon_tags_2 },
    { 0x5b8d821a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e5821f, SUC_NE_lexicon_tags_2 },
    { 0xc4bd821f, SUC_NE_lexicon_tags_2 },
    { 0x7219821f, SUC_NE_lexicon_tags_19 },
    { 0xbae98220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b658227, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152b822b, SUC_NE_lexicon_tags_2 },
    { 0x4a5f822c, SUC_NE_lexicon_tags_2 },
    { 0x9f8d822d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8801822f, SUC_NE_lexicon_tags_7 },
    { 0x54498230, SUC_NE_lexicon_tags_2 },
    { 0x2c498230, SUC_NE_lexicon_tags_2 },
    { 0xed9b8232, SUC_NE_lexicon_tags_2 },
    { 0x54358232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896d8237, SUC_NE_lexicon_tags_2 },
    { 0x54e98238, SUC_NE_lexicon_tags_2 },
    { 0x615d8238, SUC_NE_lexicon_tags_2 },
    { 0x7f25823a, SUC_NE_lexicon_tags_2 },
    { 0x3659823b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c5823f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe458248, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef824d, SUC_NE_lexicon_tags_2 },
    { 0xe171824d, SUC_NE_lexicon_tags_2 },
    { 0xb701824f, SUC_NE_lexicon_tags_2 },
    { 0x3dff8250, SUC_NE_lexicon_tags_2 },
    { 0xb573824f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a558253, SUC_NE_lexicon_tags_2 },
    { 0x974d8253, SUC_NE_lexicon_tags_2 },
    { 0xf1cf8253, SUC_NE_lexicon_tags_2 },
    { 0xdfff8256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x510d8259, SUC_NE_lexicon_tags_2 },
    { 0xf035825a, SUC_NE_lexicon_tags_2 },
    { 0x4415825b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74ed825d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c58260, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8818263, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d8267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f1826e, SUC_NE_lexicon_tags_3 },
    { 0x1951826e, SUC_NE_lexicon_tags_4 },
    { 0x9987826e, SUC_NE_lexicon_tags_7 },
    { 0xa2a3826f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb578274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd8277, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd839827a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87cf827c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e598282, SUC_NE_lexicon_tags_7 },
    { 0x83218282, SUC_NE_lexicon_tags_0 },
    { 0x5e2b8284, SUC_NE_lexicon_tags_305 },
    { 0xc8958284, SUC_NE_lexicon_tags_2 },
    { 0x86e38285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc595828d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x914b828f, SUC_NE_lexicon_tags_2 },
    { 0xa01f8290, SUC_NE_lexicon_tags_2 },
    { 0xff65828f, SUC_NE_lexicon_tags_2 },
    { 0x9178292, SUC_NE_lexicon_tags_2 },
    { 0x2a458291, SUC_NE_lexicon_tags_2 },
    { 0xdea5828f, SUC_NE_lexicon_tags_2 },
    { 0x47678291, SUC_NE_lexicon_tags_2 },
    { 0xa0ab8295, SUC_NE_lexicon_tags_2 },
    { 0x77db8297, SUC_NE_lexicon_tags_2 },
    { 0x7dd8298, SUC_NE_lexicon_tags_103 },
    { 0x2b778291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff1829f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba2582a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1982a4, SUC_NE_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd982a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8582b0, SUC_NE_lexicon_tags_2 },
    { 0xb43d82b0, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xed1d82b3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570b82bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104b82c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe17582c4, SUC_NE_lexicon_tags_2 },
    { 0xb15f82c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c182c8, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283782cd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xee7582cf, SUC_NE_lexicon_tags_2 },
    { 0x4f6f82d0, SUC_NE_lexicon_tags_2 },
    { 0x1cc582d0, SUC_NE_lexicon_tags_2 },
    { 0x100982d0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5182d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4d82db, SUC_NE_lexicon_tags_2 },
    { 0xa4cd82dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ed82df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35582e5, SUC_NE_lexicon_tags_2 },
    { 0x30982e5, SUC_NE_lexicon_tags_19 },
    { 0x30a182e7, SUC_NE_lexicon_tags_4 },
    { 0xbde782e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb82eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20782ef, SUC_NE_lexicon_tags_2 },
    { 0x918382f0, SUC_NE_lexicon_tags_2 },
    { 0x278582ef, SUC_NE_lexicon_tags_40 },
    { 0x603182f2, SUC_NE_lexicon_tags_2 },
    { 0xd5f182ef, SUC_NE_lexicon_tags_2 },
    { 0x4c1d82f4, SUC_NE_lexicon_tags_2 },
    { 0x79a182f3, SUC_NE_lexicon_tags_2 },
    { 0x871f82f6, SUC_NE_lexicon_tags_2 },
    { 0xe38182f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5af382f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c582ff, SUC_NE_lexicon_tags_2 },
    { 0x6daf8300, SUC_NE_lexicon_tags_2 },
    { 0xb2b78300, SUC_NE_lexicon_tags_2 },
    { 0xc11b8302, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce18309, SUC_NE_lexicon_tags_2 },
    { 0x751830a, SUC_NE_lexicon_tags_2 },
    { 0xdc09830b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa501830f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9bd8311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6718314, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d58318, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6289831a, SUC_NE_lexicon_tags_2 },
    { 0xba31831b, SUC_NE_lexicon_tags_2 },
    { 0x1759831a, SUC_NE_lexicon_tags_2 },
    { 0x8ba9831d, SUC_NE_lexicon_tags_2 },
    { 0xabfb831d, SUC_NE_lexicon_tags_2 },
    { 0x4c69831e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312f8328, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3257832b, SUC_NE_lexicon_tags_2 },
    { 0x92bd832b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48d5832e, SUC_NE_lexicon_tags_2 },
    { 0x23f7832f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844f8333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde7f8335, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x67838337, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c9833a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e27833c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x996d833e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638b8342, SUC_NE_lexicon_tags_2 },
    { 0xe7258342, SUC_NE_lexicon_tags_2 },
    { 0x47738344, SUC_NE_lexicon_tags_2 },
    { 0x85e18344, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0d98347, SUC_NE_lexicon_tags_2 },
    { 0x96178348, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bb834b, SUC_NE_lexicon_tags_2 },
    { 0x8f67834c, SUC_NE_lexicon_tags_2 },
    { 0xb717834c, SUC_NE_lexicon_tags_3 },
    { 0x7d01834b, SUC_NE_lexicon_tags_2 },
    { 0x3f71834f, SUC_NE_lexicon_tags_2 },
    { 0xab738350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f0b8352, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b18356, SUC_NE_lexicon_tags_2 },
    { 0x4d1f8357, SUC_NE_lexicon_tags_2 },
    { 0x72d78358, SUC_NE_lexicon_tags_2 },
    { 0x51cb8359, SUC_NE_lexicon_tags_2 },
    { 0xd1d5835a, SUC_NE_lexicon_tags_2 },
    { 0x8b4b8357, SUC_NE_lexicon_tags_2 },
    { 0xb3e9835c, SUC_NE_lexicon_tags_2 },
    { 0x1cbb835c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa301835f, SUC_NE_lexicon_tags_2 },
    { 0x7bb18360, SUC_NE_lexicon_tags_2 },
    { 0xd82b8361, SUC_NE_lexicon_tags_2 },
    { 0xd7558362, SUC_NE_lexicon_tags_2 },
    { 0x61a38363, SUC_NE_lexicon_tags_2 },
    { 0xeb798363, SUC_NE_lexicon_tags_2 },
    { 0x3df18363, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ef836a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f7836d, SUC_NE_lexicon_tags_2 },
    { 0xde23836e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d3d8370, SUC_NE_lexicon_tags_2 },
    { 0xd8b98370, SUC_NE_lexicon_tags_2 },
    { 0xd798372, SUC_NE_lexicon_tags_2 },
    { 0x2a178373, SUC_NE_lexicon_tags_2 },
    { 0xdfd38374, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9918376, SUC_NE_lexicon_tags_2 },
    { 0x96358376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x827f8379, SUC_NE_lexicon_tags_2 },
    { 0x1a9d837a, SUC_NE_lexicon_tags_30 },
    { 0x45398379, SUC_NE_lexicon_tags_2 },
    { 0xf1b9837c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a118380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x54a18382, SUC_NE_lexicon_tags_7 },
    { 0xdab38382, SUC_NE_lexicon_tags_2 },
    { 0xe8af8383, SUC_NE_lexicon_tags_2 },
    { 0x7ae38385, SUC_NE_lexicon_tags_2 },
    { 0x5b538382, SUC_NE_lexicon_tags_1 },
    { 0x22418387, SUC_NE_lexicon_tags_2 },
    { 0xf0098382, SUC_NE_lexicon_tags_2 },
    { 0xdee98389, SUC_NE_lexicon_tags_2 },
    { 0x520d838a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbdd838c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87d8390, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99238394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb78398, SUC_NE_lexicon_tags_2 },
    { 0xbe78399, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58383a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e183a3, SUC_NE_lexicon_tags_2 },
    { 0x153383a4, SUC_NE_lexicon_tags_2 },
    { 0x37df83a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75b83a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x694983ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cdb83ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x926583b0, SUC_NE_lexicon_tags_2 },
    { 0x705783b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bd83b5, SUC_NE_lexicon_tags_2 },
    { 0xf5eb83b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x590783b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7b83be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4edd83c0, SUC_NE_lexicon_tags_2 },
    { 0x9c1583c1, SUC_NE_lexicon_tags_2 },
    { 0xc77f83c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60783c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33583ca, SUC_NE_lexicon_tags_2 },
    { 0xe5f983ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf83d2, SUC_NE_lexicon_tags_7 },
    { 0x515183d3, SUC_NE_lexicon_tags_4 },
    { 0x1c0983d4, SUC_NE_lexicon_tags_102 },
    { 0xed9d83d3, SUC_NE_lexicon_tags_2 },
    { 0x6be383d4, SUC_NE_lexicon_tags_2 },
    { 0x325783d4, SUC_NE_lexicon_tags_2 },
    { 0xcd9383d2, SUC_NE_lexicon_tags_49 },
    { 0x2def83d9, SUC_NE_lexicon_tags_17 },
    { 0xd74b83da, SUC_NE_lexicon_tags_2 },
    { 0xc1f183d6, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb183de, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222783e2, SUC_NE_lexicon_tags_2 },
    { 0xf83783e3, SUC_NE_lexicon_tags_30 },
    { 0x329983e3, SUC_NE_lexicon_tags_2 },
    { 0xb7b183e5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f183e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aab83ee, SUC_NE_lexicon_tags_1 },
    { 0xbfe383ef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x25c583f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x683f83f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x832f83f6, SUC_NE_lexicon_tags_2 },
    { 0xc75383f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc04383f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26783ff, SUC_NE_lexicon_tags_1 },
    { 0xfb478400, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fb8404, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7e58406, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50478409, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a01840f, SUC_NE_lexicon_tags_11 },
    { 0x9ca7840f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd55b8412, SUC_NE_lexicon_tags_2 },
    { 0x25598413, SUC_NE_lexicon_tags_2 },
    { 0x9df98414, SUC_NE_lexicon_tags_2 },
    { 0xef5d8414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6af78417, SUC_NE_lexicon_tags_2 },
    { 0xa0698417, SUC_NE_lexicon_tags_2 },
    { 0x26618419, SUC_NE_lexicon_tags_2 },
    { 0xf4d3841a, SUC_NE_lexicon_tags_3 },
    { 0x3fc38419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2525841d, SUC_NE_lexicon_tags_2 },
    { 0x20b7841e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x930b8420, SUC_NE_lexicon_tags_2 },
    { 0xca3d8421, SUC_NE_lexicon_tags_2 },
    { 0x51318422, SUC_NE_lexicon_tags_0 },
    { 0x9c538423, SUC_NE_lexicon_tags_0 },
    { 0x10b78424, SUC_NE_lexicon_tags_2 },
    { 0xa58b8425, SUC_NE_lexicon_tags_2 },
    { 0x50998420, SUC_NE_lexicon_tags_2 },
    { 0x49958420, SUC_NE_lexicon_tags_2 },
    { 0x46a78426, SUC_NE_lexicon_tags_2 },
    { 0x33b78429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36d842c, SUC_NE_lexicon_tags_2 },
    { 0xe85f842c, SUC_NE_lexicon_tags_2 },
    { 0xc48d842e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d358430, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f58433, SUC_NE_lexicon_tags_2 },
    { 0xde918434, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4b8437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd33b8439, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1057843b, SUC_NE_lexicon_tags_7 },
    { 0x1ae1843c, SUC_NE_lexicon_tags_2 },
    { 0x5e39843b, SUC_NE_lexicon_tags_2 },
    { 0x1e01843e, SUC_NE_lexicon_tags_2 },
    { 0xaafd843f, SUC_NE_lexicon_tags_2 },
    { 0xaad38440, SUC_NE_lexicon_tags_17 },
    { 0x91058441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x930f8443, SUC_NE_lexicon_tags_2 },
    { 0x36ed8443, SUC_NE_lexicon_tags_2 },
    { 0xca4b8444, SUC_NE_lexicon_tags_2 },
    { 0xadf8446, SUC_NE_lexicon_tags_2 },
    { 0xff038446, SUC_NE_lexicon_tags_2 },
    { 0xe6f98448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98f844a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10d844d, SUC_NE_lexicon_tags_2 },
    { 0x6e1b844e, SUC_NE_lexicon_tags_2 },
    { 0x65d3844d, SUC_NE_lexicon_tags_2 },
    { 0x6b61844d, SUC_NE_lexicon_tags_2 },
    { 0x35fb8451, SUC_NE_lexicon_tags_7 },
    { 0x37258452, SUC_NE_lexicon_tags_2 },
    { 0x75e38453, SUC_NE_lexicon_tags_7 },
    { 0x91378453, SUC_NE_lexicon_tags_2 },
    { 0x1b3b8455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4978459, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2765845c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ad845f, SUC_NE_lexicon_tags_2 },
    { 0xede1845f, SUC_NE_lexicon_tags_2 },
    { 0x67338461, SUC_NE_lexicon_tags_2 },
    { 0x8eb8462, SUC_NE_lexicon_tags_4 },
    { 0xfdab8463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7846b, SUC_NE_lexicon_tags_2 },
    { 0x82fd846c, SUC_NE_lexicon_tags_2 },
    { 0x64c7846d, SUC_NE_lexicon_tags_4 },
    { 0x6379846e, SUC_NE_lexicon_tags_2 },
    { 0xa483846d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da18471, SUC_NE_lexicon_tags_2 },
    { 0x832f8471, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da78475, SUC_NE_lexicon_tags_2 },
    { 0x58238476, SUC_NE_lexicon_tags_2 },
    { 0xf9178477, SUC_NE_lexicon_tags_2 },
    { 0xdceb8475, SUC_NE_lexicon_tags_2 },
    { 0xb4e18479, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3df847b, SUC_NE_lexicon_tags_17 },
    { 0x20eb847c, SUC_NE_lexicon_tags_2 },
    { 0x9fbd847d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec3847f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ecf8481, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2358483, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x62ff8485, SUC_NE_lexicon_tags_2 },
    { 0xa37b8486, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca418489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd705848b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165b848f, SUC_NE_lexicon_tags_2 },
    { 0xe105848f, SUC_NE_lexicon_tags_2 },
    { 0xff858491, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a98494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa698499, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf957849e, SUC_NE_lexicon_tags_2 },
    { 0xd183849e, SUC_NE_lexicon_tags_2 },
    { 0x4925849f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248d84a2, SUC_NE_lexicon_tags_2 },
    { 0x7fc984a2, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41384a7, SUC_NE_lexicon_tags_2 },
    { 0xb4f584a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d9384aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc784ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd784b1, SUC_NE_lexicon_tags_2 },
    { 0xdde384b2, SUC_NE_lexicon_tags_2 },
    { 0x6eff84b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x987984b5, SUC_NE_lexicon_tags_7 },
    { 0xc62784b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18cd84b9, SUC_NE_lexicon_tags_190 },
    { 0x756384ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68f184bc, SUC_NE_lexicon_tags_2 },
    { 0xbd6d84bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e184c0, SUC_NE_lexicon_tags_2 },
    { 0x2edf84c1, SUC_NE_lexicon_tags_2 },
    { 0xea2f84c1, SUC_NE_lexicon_tags_2 },
    { 0xd89d84c3, SUC_NE_lexicon_tags_4 },
    { 0x9fcd84c0, SUC_NE_lexicon_tags_2 },
    { 0xb8af84c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379184c9, SUC_NE_lexicon_tags_2 },
    { 0x391984ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2df84cc, SUC_NE_lexicon_tags_2 },
    { 0x776f84cd, SUC_NE_lexicon_tags_2 },
    { 0xa9f84cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x532d84d0, SUC_NE_lexicon_tags_19 },
    { 0xc42d84d1, SUC_NE_lexicon_tags_2 },
    { 0xed5384d1, SUC_NE_lexicon_tags_2 },
    { 0x77a784d3, SUC_NE_lexicon_tags_2 },
    { 0x74bf84d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f184d9, SUC_NE_lexicon_tags_49 },
    { 0x8ce584da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37d584dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf81584de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02184e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e784e4, SUC_NE_lexicon_tags_2 },
    { 0xb59584e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1d84ec, SUC_NE_lexicon_tags_2 },
    { 0x696b84ed, SUC_NE_lexicon_tags_131 },
    { 0xe6d384ec, SUC_NE_lexicon_tags_2 },
    { 0x950f84ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91384f2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd84984f4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345384fc, SUC_NE_lexicon_tags_2 },
    { 0x95f584fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b584ff, SUC_NE_lexicon_tags_2 },
    { 0xdb2584ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17918502, SUC_NE_lexicon_tags_2 },
    { 0x26098503, SUC_NE_lexicon_tags_2 },
    { 0x929f8504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb753850b, SUC_NE_lexicon_tags_2 },
    { 0x38d9850c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d398510, SUC_NE_lexicon_tags_2 },
    { 0x20a58510, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5d8518, SUC_NE_lexicon_tags_2 },
    { 0x978d8518, SUC_NE_lexicon_tags_415 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7938521, SUC_NE_lexicon_tags_2 },
    { 0xf118522, SUC_NE_lexicon_tags_2 },
    { 0xcee58521, SUC_NE_lexicon_tags_2 },
    { 0x1c358524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5add8526, SUC_NE_lexicon_tags_2 },
    { 0x74f18527, SUC_NE_lexicon_tags_2 },
    { 0xfbb8527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x600b852b, SUC_NE_lexicon_tags_2 },
    { 0xbe3d852b, SUC_NE_lexicon_tags_2 },
    { 0xc7d852d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58d8530, SUC_NE_lexicon_tags_2 },
    { 0xbfa58531, SUC_NE_lexicon_tags_2 },
    { 0xd2a58532, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89398536, SUC_NE_lexicon_tags_2 },
    { 0xb2fd8537, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd853c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf78541, SUC_NE_lexicon_tags_2 },
    { 0x4e458542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x971f8545, SUC_NE_lexicon_tags_7 },
    { 0x33558546, SUC_NE_lexicon_tags_2 },
    { 0x766f8545, SUC_NE_lexicon_tags_2 },
    { 0xd5e18546, SUC_NE_lexicon_tags_2 },
    { 0xbbd18549, SUC_NE_lexicon_tags_2 },
    { 0xe87f854a, SUC_NE_lexicon_tags_2 },
    { 0x7f6b854b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f854e, SUC_NE_lexicon_tags_2 },
    { 0x6783854f, SUC_NE_lexicon_tags_2 },
    { 0x76518550, SUC_NE_lexicon_tags_2 },
    { 0x66958551, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x744d8553, SUC_NE_lexicon_tags_4 },
    { 0x3a5d8553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88f58556, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25778559, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x64b7855b, SUC_NE_lexicon_tags_2 },
    { 0xc2d9855c, SUC_NE_lexicon_tags_2 },
    { 0x843855b, SUC_NE_lexicon_tags_337 },
    { 0xbea1855e, SUC_NE_lexicon_tags_2 },
    { 0xf375855f, SUC_NE_lexicon_tags_2 },
    { 0xdc97855f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x314f8563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6538566, SUC_NE_lexicon_tags_2 },
    { 0xc3df8566, SUC_NE_lexicon_tags_2 },
    { 0xc35d8568, SUC_NE_lexicon_tags_2 },
    { 0x9bb78569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34df856b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762b856f, SUC_NE_lexicon_tags_2 },
    { 0xa89f8570, SUC_NE_lexicon_tags_2 },
    { 0x3f698571, SUC_NE_lexicon_tags_2 },
    { 0xcc9d8570, SUC_NE_lexicon_tags_28 },
    { 0xc24b8573, SUC_NE_lexicon_tags_7 },
    { 0x48bf8572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d38577, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80f857c, SUC_NE_lexicon_tags_2 },
    { 0x611857c, SUC_NE_lexicon_tags_308 },
    { 0x2a9d857e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc358580, SUC_NE_lexicon_tags_4 },
    { 0x6d258581, SUC_NE_lexicon_tags_1 },
    { 0x5d7b8581, SUC_NE_lexicon_tags_2 },
    { 0x40ad8582, SUC_NE_lexicon_tags_2 },
    { 0xf6878581, SUC_NE_lexicon_tags_2 },
    { 0x23778585, SUC_NE_lexicon_tags_7 },
    { 0x4bd58584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317858a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7e1858c, SUC_NE_lexicon_tags_2 },
    { 0x44e1858d, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x12b9858f, SUC_NE_lexicon_tags_2 },
    { 0x68b38590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80918594, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b78597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2203859a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b85a6, SUC_NE_lexicon_tags_2 },
    { 0x6beb85a7, SUC_NE_lexicon_tags_2 },
    { 0x770b85a7, SUC_NE_lexicon_tags_2 },
    { 0x4ab985a9, SUC_NE_lexicon_tags_2 },
    { 0x7e5d85a8, SUC_NE_lexicon_tags_138 },
    { 0x2e4585ab, SUC_NE_lexicon_tags_2 },
    { 0xa1cf85a7, SUC_NE_lexicon_tags_4 },
    { 0x2d9185ad, SUC_NE_lexicon_tags_2 },
    { 0x89a785ad, SUC_NE_lexicon_tags_2 },
    { 0x923f85a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217585b4, SUC_NE_lexicon_tags_2 },
    { 0xf49f85b4, SUC_NE_lexicon_tags_2 },
    { 0x109b85b6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xde8585b8, SUC_NE_lexicon_tags_121 },
    { 0xd07185b9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7985bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b785c1, SUC_NE_lexicon_tags_24 },
    { 0x46bd85c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7585cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4b85ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4785d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa71d85d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaff85d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5585da, SUC_NE_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d85de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5c385e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7c985e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a5585e4, SUC_NE_lexicon_tags_50 },
    { 0x39fd85e4, SUC_NE_lexicon_tags_2 },
    { 0x875785e4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x979185e8, SUC_NE_lexicon_tags_2 },
    { 0xdacf85e9, SUC_NE_lexicon_tags_2 },
    { 0x83a185ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2185ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651b85f6, SUC_NE_lexicon_tags_2 },
    { 0x9f4785f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af985fa, SUC_NE_lexicon_tags_2 },
    { 0xdee185fb, SUC_NE_lexicon_tags_7 },
    { 0x8c7585fc, SUC_NE_lexicon_tags_2 },
    { 0x814d85fb, SUC_NE_lexicon_tags_2 },
    { 0xaef585fa, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bc98600, SUC_NE_lexicon_tags_2 },
    { 0xcd118601, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b78604, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefa98606, SUC_NE_lexicon_tags_2 },
    { 0x9ef8607, SUC_NE_lexicon_tags_2 },
    { 0x3dc98608, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf71b860a, SUC_NE_lexicon_tags_2 },
    { 0x9151860a, SUC_NE_lexicon_tags_2 },
    { 0x175b860c, SUC_NE_lexicon_tags_2 },
    { 0xe7ed860d, SUC_NE_lexicon_tags_2 },
    { 0xeb63860d, SUC_NE_lexicon_tags_2 },
    { 0x7eed860e, SUC_NE_lexicon_tags_2 },
    { 0xbe0b860e, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51918614, SUC_NE_lexicon_tags_2 },
    { 0x182f8614, SUC_NE_lexicon_tags_2 },
    { 0x6f2b8616, SUC_NE_lexicon_tags_2 },
    { 0xe8f98615, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x459f8619, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ef9861b, SUC_NE_lexicon_tags_218 },
    { 0, NULL },
    { 0x9237861d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae11861f, SUC_NE_lexicon_tags_2 },
    { 0xdf37861f, SUC_NE_lexicon_tags_2 },
    { 0x84378621, SUC_NE_lexicon_tags_2 },
    { 0xd76d8621, SUC_NE_lexicon_tags_18 },
    { 0x50278622, SUC_NE_lexicon_tags_2 },
    { 0xb5418624, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x132d8627, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9958629, SUC_NE_lexicon_tags_2 },
    { 0x44938629, SUC_NE_lexicon_tags_2 },
    { 0xab6f8629, SUC_NE_lexicon_tags_2 },
    { 0x5515862c, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51458633, SUC_NE_lexicon_tags_2 },
    { 0xeab8634, SUC_NE_lexicon_tags_2 },
    { 0xcdb58635, SUC_NE_lexicon_tags_2 },
    { 0x4b358633, SUC_NE_lexicon_tags_2 },
    { 0x44178634, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d018639, SUC_NE_lexicon_tags_34 },
    { 0x9f51863a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c18640, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8644, SUC_NE_lexicon_tags_2 },
    { 0xd41d8644, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a3864c, SUC_NE_lexicon_tags_1 },
    { 0xf403864d, SUC_NE_lexicon_tags_44 },
    { 0, NULL },
    { 0x48d1864f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cf865b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a41865f, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xb1d38661, SUC_NE_lexicon_tags_5 },
    { 0xab018661, SUC_NE_lexicon_tags_40 },
    { 0x9a3d8662, SUC_NE_lexicon_tags_5 },
    { 0xa4798661, SUC_NE_lexicon_tags_2 },
    { 0x4ca38665, SUC_NE_lexicon_tags_2 },
    { 0xc5538662, SUC_NE_lexicon_tags_2 },
    { 0x4d6d8664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b5866c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd9866f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f818675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cd8677, SUC_NE_lexicon_tags_7 },
    { 0x241d8678, SUC_NE_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a4f867a, SUC_NE_lexicon_tags_2 },
    { 0x202b867b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe21867d, SUC_NE_lexicon_tags_17 },
    { 0x72e3867d, SUC_NE_lexicon_tags_2 },
    { 0xc90f867f, SUC_NE_lexicon_tags_2 },
    { 0xfeb7867d, SUC_NE_lexicon_tags_4 },
    { 0xae8b8681, SUC_NE_lexicon_tags_2 },
    { 0x23af8682, SUC_NE_lexicon_tags_2 },
    { 0xeab78683, SUC_NE_lexicon_tags_18 },
    { 0x995b8684, SUC_NE_lexicon_tags_4 },
    { 0xe0f78685, SUC_NE_lexicon_tags_40 },
    { 0xcff38681, SUC_NE_lexicon_tags_2 },
    { 0x791f8685, SUC_NE_lexicon_tags_2 },
    { 0xf1eb8686, SUC_NE_lexicon_tags_2 },
    { 0x2c4f8689, SUC_NE_lexicon_tags_2 },
    { 0x7e7f8689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1f868d, SUC_NE_lexicon_tags_2 },
    { 0xa811868e, SUC_NE_lexicon_tags_2 },
    { 0x46f868e, SUC_NE_lexicon_tags_2 },
    { 0x6fd58690, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24bf8692, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb358697, SUC_NE_lexicon_tags_18 },
    { 0xac3d8698, SUC_NE_lexicon_tags_18 },
    { 0xbce58698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1619869d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34c9869f, SUC_NE_lexicon_tags_2 },
    { 0xa04b86a0, SUC_NE_lexicon_tags_40 },
    { 0x4e85869f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dd386a3, SUC_NE_lexicon_tags_2 },
    { 0x8a986a3, SUC_NE_lexicon_tags_2 },
    { 0xf7d986a5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5d8186a7, SUC_NE_lexicon_tags_4 },
    { 0x684586a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x419386aa, SUC_NE_lexicon_tags_8 },
    { 0xb5cb86ab, SUC_NE_lexicon_tags_2 },
    { 0x34b86aa, SUC_NE_lexicon_tags_2 },
    { 0xa85786ac, SUC_NE_lexicon_tags_2 },
    { 0xc3db86ae, SUC_NE_lexicon_tags_2 },
    { 0xca4f86ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x224786b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4186b4, SUC_NE_lexicon_tags_2 },
    { 0x18ef86b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf0786b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bdb86b9, SUC_NE_lexicon_tags_2 },
    { 0x539f86ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827386c1, SUC_NE_lexicon_tags_2 },
    { 0xf9f386c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e3186c4, SUC_NE_lexicon_tags_2 },
    { 0x1b2586c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1386c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x694b86ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa99d86cc, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc186cf, SUC_NE_lexicon_tags_2 },
    { 0xbc5386d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5386d7, SUC_NE_lexicon_tags_2 },
    { 0x24a986d8, SUC_NE_lexicon_tags_2 },
    { 0xabb986d9, SUC_NE_lexicon_tags_2 },
    { 0xf5cf86d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ff86dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61386e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x835986e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b9b86e5, SUC_NE_lexicon_tags_2 },
    { 0x2d9186e6, SUC_NE_lexicon_tags_2 },
    { 0x3ab386e7, SUC_NE_lexicon_tags_2 },
    { 0xe6f386e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27d86ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96786f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869186f5, SUC_NE_lexicon_tags_2 },
    { 0x6cb86f5, SUC_NE_lexicon_tags_2 },
    { 0x9bdf86f7, SUC_NE_lexicon_tags_2 },
    { 0x685f86f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5df86fc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f86fe, SUC_NE_lexicon_tags_2 },
    { 0xe0a386fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8498701, SUC_NE_lexicon_tags_0 },
    { 0xd5b18702, SUC_NE_lexicon_tags_2 },
    { 0x77978703, SUC_NE_lexicon_tags_18 },
    { 0x6b038704, SUC_NE_lexicon_tags_4 },
    { 0x5f998705, SUC_NE_lexicon_tags_2 },
    { 0x72578705, SUC_NE_lexicon_tags_2 },
    { 0xb8338702, SUC_NE_lexicon_tags_2 },
    { 0xde938708, SUC_NE_lexicon_tags_2 },
    { 0xc3138709, SUC_NE_lexicon_tags_4 },
    { 0x37598701, SUC_NE_lexicon_tags_2 },
    { 0x8265870a, SUC_NE_lexicon_tags_1 },
    { 0x2591870c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfd8711, SUC_NE_lexicon_tags_2 },
    { 0x61e78712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0f8715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49bb8717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb9871a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f3b871c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e33871e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3f8722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdb8725, SUC_NE_lexicon_tags_13 },
    { 0x31378726, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fbd8728, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b81872a, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfb8732, SUC_NE_lexicon_tags_2 },
    { 0x83218732, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3498738, SUC_NE_lexicon_tags_34 },
    { 0xa50f8739, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc061873d, SUC_NE_lexicon_tags_377 },
    { 0x5e97873e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0698742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0b8748, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1874d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f2f874f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d8753, SUC_NE_lexicon_tags_2 },
    { 0xe89d8754, SUC_NE_lexicon_tags_2 },
    { 0x1c0b8755, SUC_NE_lexicon_tags_2 },
    { 0x788f8756, SUC_NE_lexicon_tags_2 },
    { 0xd8c38754, SUC_NE_lexicon_tags_2 },
    { 0x7c9d8755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d3f875a, SUC_NE_lexicon_tags_2 },
    { 0x7563875b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5875e, SUC_NE_lexicon_tags_2 },
    { 0x1b17875f, SUC_NE_lexicon_tags_5 },
    { 0xe03f875f, SUC_NE_lexicon_tags_39 },
    { 0x56f78761, SUC_NE_lexicon_tags_2 },
    { 0xb4258761, SUC_NE_lexicon_tags_2 },
    { 0x9e3b875f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f21876a, SUC_NE_lexicon_tags_2 },
    { 0x66ad876b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ee7876d, SUC_NE_lexicon_tags_2 },
    { 0xe1c5876e, SUC_NE_lexicon_tags_2 },
    { 0x9901876f, SUC_NE_lexicon_tags_2 },
    { 0xd443876e, SUC_NE_lexicon_tags_2 },
    { 0x18e1876f, SUC_NE_lexicon_tags_2 },
    { 0x51c1876d, SUC_NE_lexicon_tags_3 },
    { 0x83198773, SUC_NE_lexicon_tags_2 },
    { 0x36818774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6338779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c1877c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3f5877e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d78783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3be78785, SUC_NE_lexicon_tags_2 },
    { 0x79a38785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4898789, SUC_NE_lexicon_tags_2 },
    { 0x6dcb878a, SUC_NE_lexicon_tags_2 },
    { 0xf8f9878b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf9f878d, SUC_NE_lexicon_tags_2 },
    { 0x2e9d878e, SUC_NE_lexicon_tags_2 },
    { 0x51a9878f, SUC_NE_lexicon_tags_2 },
    { 0x43558790, SUC_NE_lexicon_tags_2 },
    { 0x62ab8790, SUC_NE_lexicon_tags_3 },
    { 0x89bb878d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56498799, SUC_NE_lexicon_tags_2 },
    { 0xcbd879a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21f879e, SUC_NE_lexicon_tags_7 },
    { 0x518f879f, SUC_NE_lexicon_tags_2 },
    { 0x882d87a0, SUC_NE_lexicon_tags_2 },
    { 0x1b8787a1, SUC_NE_lexicon_tags_2 },
    { 0x9f1987a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2c587a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526587aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348f87af, SUC_NE_lexicon_tags_59 },
    { 0xc83f87b0, SUC_NE_lexicon_tags_4 },
    { 0x7ae787b1, SUC_NE_lexicon_tags_2 },
    { 0xbed187b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137d87b6, SUC_NE_lexicon_tags_2 },
    { 0x1b7187b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1d87bb, SUC_NE_lexicon_tags_2 },
    { 0x312787bb, SUC_NE_lexicon_tags_2 },
    { 0x66f187bd, SUC_NE_lexicon_tags_2 },
    { 0xdae187bb, SUC_NE_lexicon_tags_2 },
    { 0x59c987bc, SUC_NE_lexicon_tags_2 },
    { 0x9c0f87bc, SUC_NE_lexicon_tags_2 },
    { 0xc7c387c1, SUC_NE_lexicon_tags_7 },
    { 0x39d587c2, SUC_NE_lexicon_tags_2 },
    { 0xef3187c3, SUC_NE_lexicon_tags_2 },
    { 0x1bf787c2, SUC_NE_lexicon_tags_2 },
    { 0x46bf87c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8187c9, SUC_NE_lexicon_tags_1 },
    { 0xa7d987ca, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x8aed87cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38587cf, SUC_NE_lexicon_tags_2 },
    { 0xf01787d0, SUC_NE_lexicon_tags_17 },
    { 0x15b787d1, SUC_NE_lexicon_tags_2 },
    { 0xc53987d2, SUC_NE_lexicon_tags_2 },
    { 0xea1787d3, SUC_NE_lexicon_tags_2 },
    { 0x8b4987d4, SUC_NE_lexicon_tags_2 },
    { 0x323987d5, SUC_NE_lexicon_tags_7 },
    { 0x90a787d4, SUC_NE_lexicon_tags_234 },
    { 0xd76987d4, SUC_NE_lexicon_tags_2 },
    { 0xaab387d5, SUC_NE_lexicon_tags_2 },
    { 0x306987d9, SUC_NE_lexicon_tags_2 },
    { 0xd23b87d4, SUC_NE_lexicon_tags_2 },
    { 0xcaaf87d3, SUC_NE_lexicon_tags_2 },
    { 0xec9187dc, SUC_NE_lexicon_tags_42 },
    { 0xc43187da, SUC_NE_lexicon_tags_2 },
    { 0x7de387de, SUC_NE_lexicon_tags_2 },
    { 0x1ac787df, SUC_NE_lexicon_tags_1 },
    { 0x2b8787d9, SUC_NE_lexicon_tags_7 },
    { 0xf79387d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75eb87e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x623387e6, SUC_NE_lexicon_tags_2 },
    { 0xc8e987e7, SUC_NE_lexicon_tags_2 },
    { 0x304587e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x160587ea, SUC_NE_lexicon_tags_17 },
    { 0x498587ea, SUC_NE_lexicon_tags_1 },
    { 0xcf787ec, SUC_NE_lexicon_tags_2 },
    { 0x443f87ea, SUC_NE_lexicon_tags_3 },
    { 0xcfd987ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xab8387f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4187f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79787f7, SUC_NE_lexicon_tags_2 },
    { 0xf40d87f7, SUC_NE_lexicon_tags_2 },
    { 0xdbe587f9, SUC_NE_lexicon_tags_24 },
    { 0x8fef87f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x978b87fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc238800, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6f58802, SUC_NE_lexicon_tags_2 },
    { 0xa0e98803, SUC_NE_lexicon_tags_2 },
    { 0x2bed8804, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85698808, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd7880a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd629880c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c738810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5138812, SUC_NE_lexicon_tags_2 },
    { 0x2b418812, SUC_NE_lexicon_tags_2 },
    { 0x3c5b8813, SUC_NE_lexicon_tags_2 },
    { 0x34af8812, SUC_NE_lexicon_tags_2 },
    { 0x891b8816, SUC_NE_lexicon_tags_2 },
    { 0x61758817, SUC_NE_lexicon_tags_2 },
    { 0x9ff38817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2583881a, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399b8820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3178824, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df58827, SUC_NE_lexicon_tags_2 },
    { 0xd3bb8828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x979882b, SUC_NE_lexicon_tags_2 },
    { 0x9f6f882c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dd8833, SUC_NE_lexicon_tags_50 },
    { 0xa5bd8833, SUC_NE_lexicon_tags_2 },
    { 0xd81b8835, SUC_NE_lexicon_tags_2 },
    { 0xd5818834, SUC_NE_lexicon_tags_2 },
    { 0x6bd18834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa491883b, SUC_NE_lexicon_tags_2 },
    { 0x1735883c, SUC_NE_lexicon_tags_2 },
    { 0x856d883c, SUC_NE_lexicon_tags_2 },
    { 0x7fc3883e, SUC_NE_lexicon_tags_3 },
    { 0xe7cd883c, SUC_NE_lexicon_tags_2 },
    { 0x9777883f, SUC_NE_lexicon_tags_2 },
    { 0x518d883e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21b78843, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d8846, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7b8849, SUC_NE_lexicon_tags_13 },
    { 0x458b884a, SUC_NE_lexicon_tags_2 },
    { 0xccef884b, SUC_NE_lexicon_tags_1 },
    { 0xf21884b, SUC_NE_lexicon_tags_17 },
    { 0x9add884c, SUC_NE_lexicon_tags_18 },
    { 0xadb7884d, SUC_NE_lexicon_tags_2 },
    { 0xca9d884f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4f8853, SUC_NE_lexicon_tags_2 },
    { 0x28bb8853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2c18856, SUC_NE_lexicon_tags_2 },
    { 0x72578856, SUC_NE_lexicon_tags_2 },
    { 0xe8e38858, SUC_NE_lexicon_tags_2 },
    { 0xb7018859, SUC_NE_lexicon_tags_2 },
    { 0xe2438859, SUC_NE_lexicon_tags_19 },
    { 0x76b1885b, SUC_NE_lexicon_tags_2 },
    { 0xda9f8859, SUC_NE_lexicon_tags_2 },
    { 0x2b45885c, SUC_NE_lexicon_tags_2 },
    { 0xa6718859, SUC_NE_lexicon_tags_2 },
    { 0xf9f885f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9b8861, SUC_NE_lexicon_tags_2 },
    { 0x90658862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d18865, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734d886b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50b8870, SUC_NE_lexicon_tags_130 },
    { 0x59078871, SUC_NE_lexicon_tags_2 },
    { 0xab818872, SUC_NE_lexicon_tags_2 },
    { 0x892d8873, SUC_NE_lexicon_tags_2 },
    { 0xd3f38874, SUC_NE_lexicon_tags_90 },
    { 0xfb838871, SUC_NE_lexicon_tags_0 },
    { 0x26498874, SUC_NE_lexicon_tags_2 },
    { 0xdbff8873, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc3887b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecef887e, SUC_NE_lexicon_tags_2 },
    { 0x520f887f, SUC_NE_lexicon_tags_2 },
    { 0xe41f8880, SUC_NE_lexicon_tags_0 },
    { 0xa4ab8881, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e38884, SUC_NE_lexicon_tags_2 },
    { 0xcfb8884, SUC_NE_lexicon_tags_1 },
    { 0x3f6b8884, SUC_NE_lexicon_tags_2 },
    { 0xa0c78887, SUC_NE_lexicon_tags_2 },
    { 0xa6e18887, SUC_NE_lexicon_tags_2 },
    { 0x82e38889, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1773888e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d8891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1b8893, SUC_NE_lexicon_tags_1 },
    { 0xbc7d8894, SUC_NE_lexicon_tags_1 },
    { 0x896f8894, SUC_NE_lexicon_tags_2 },
    { 0x89e98896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec38899, SUC_NE_lexicon_tags_2 },
    { 0x6ca1889a, SUC_NE_lexicon_tags_2 },
    { 0x103d889a, SUC_NE_lexicon_tags_2 },
    { 0x9479889c, SUC_NE_lexicon_tags_2 },
    { 0x78c1889c, SUC_NE_lexicon_tags_2 },
    { 0xf6b3889e, SUC_NE_lexicon_tags_2 },
    { 0xe29d889e, SUC_NE_lexicon_tags_2 },
    { 0x15c5889e, SUC_NE_lexicon_tags_2 },
    { 0xecd188a1, SUC_NE_lexicon_tags_2 },
    { 0xa93988a2, SUC_NE_lexicon_tags_2 },
    { 0x943388a3, SUC_NE_lexicon_tags_2 },
    { 0x26f588a4, SUC_NE_lexicon_tags_7 },
    { 0x4e0388a4, SUC_NE_lexicon_tags_2 },
    { 0xb8a988a6, SUC_NE_lexicon_tags_2 },
    { 0x118d88a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf32588a9, SUC_NE_lexicon_tags_2 },
    { 0xd9fd88aa, SUC_NE_lexicon_tags_2 },
    { 0x1d3d88aa, SUC_NE_lexicon_tags_1 },
    { 0x1e0188a9, SUC_NE_lexicon_tags_2 },
    { 0xaca988ad, SUC_NE_lexicon_tags_258 },
    { 0xdfa588ae, SUC_NE_lexicon_tags_2 },
    { 0x4c4f88ad, SUC_NE_lexicon_tags_2 },
    { 0x94f88ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87d188b2, SUC_NE_lexicon_tags_2 },
    { 0x42e588b2, SUC_NE_lexicon_tags_2 },
    { 0x65b588b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dd88ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b988be, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0388c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566788c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997788cb, SUC_NE_lexicon_tags_2 },
    { 0x425788cb, SUC_NE_lexicon_tags_2 },
    { 0x2bc388cd, SUC_NE_lexicon_tags_2 },
    { 0x8c1d88ce, SUC_NE_lexicon_tags_2 },
    { 0x166988cf, SUC_NE_lexicon_tags_2 },
    { 0x6a4188cd, SUC_NE_lexicon_tags_2 },
    { 0x11b588cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd388d7, SUC_NE_lexicon_tags_2 },
    { 0xdb3b88d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3b88dd, SUC_NE_lexicon_tags_36 },
    { 0x818388de, SUC_NE_lexicon_tags_2 },
    { 0x6b1188de, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x79f88e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcd88e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83988ed, SUC_NE_lexicon_tags_2 },
    { 0xa3db88ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2588f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb79b88f6, SUC_NE_lexicon_tags_7 },
    { 0xddd88f7, SUC_NE_lexicon_tags_4 },
    { 0x42d188f8, SUC_NE_lexicon_tags_2 },
    { 0x756388f9, SUC_NE_lexicon_tags_2 },
    { 0xff8788fa, SUC_NE_lexicon_tags_2 },
    { 0x33f188fa, SUC_NE_lexicon_tags_2 },
    { 0xab9388fc, SUC_NE_lexicon_tags_2 },
    { 0x43f588f8, SUC_NE_lexicon_tags_2 },
    { 0x2b5b88fa, SUC_NE_lexicon_tags_2 },
    { 0x7fc988ff, SUC_NE_lexicon_tags_182 },
    { 0x97c588fd, SUC_NE_lexicon_tags_1 },
    { 0x26578901, SUC_NE_lexicon_tags_2 },
    { 0xff198902, SUC_NE_lexicon_tags_7 },
    { 0xf21988f8, SUC_NE_lexicon_tags_24 },
    { 0x249d88fc, SUC_NE_lexicon_tags_2 },
    { 0x35478905, SUC_NE_lexicon_tags_2 },
    { 0xea238906, SUC_NE_lexicon_tags_25 },
    { 0x35b18907, SUC_NE_lexicon_tags_2 },
    { 0xb0d98908, SUC_NE_lexicon_tags_2 },
    { 0x603b88f9, SUC_NE_lexicon_tags_4 },
    { 0xa0b3890a, SUC_NE_lexicon_tags_4 },
    { 0x28d890a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a5890e, SUC_NE_lexicon_tags_4 },
    { 0x7b4d890e, SUC_NE_lexicon_tags_2 },
    { 0xb29d890e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36798913, SUC_NE_lexicon_tags_2 },
    { 0x16a18914, SUC_NE_lexicon_tags_14 },
    { 0x88cb8915, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4158918, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844d891e, SUC_NE_lexicon_tags_49 },
    { 0x86fb891f, SUC_NE_lexicon_tags_3 },
    { 0x7ea1891e, SUC_NE_lexicon_tags_1 },
    { 0x711b8920, SUC_NE_lexicon_tags_2 },
    { 0x7f8922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369b8926, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9478929, SUC_NE_lexicon_tags_2 },
    { 0x66938929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42e3892c, SUC_NE_lexicon_tags_13 },
    { 0xe9b5892d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c578931, SUC_NE_lexicon_tags_2 },
    { 0xe39f8932, SUC_NE_lexicon_tags_2 },
    { 0xb1858932, SUC_NE_lexicon_tags_2 },
    { 0xf9818931, SUC_NE_lexicon_tags_14 },
    { 0x5f2f8933, SUC_NE_lexicon_tags_2 },
    { 0xc4bb8931, SUC_NE_lexicon_tags_2 },
    { 0x82f98933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbd893a, SUC_NE_lexicon_tags_2 },
    { 0xa37b893b, SUC_NE_lexicon_tags_3 },
    { 0x492b893c, SUC_NE_lexicon_tags_2 },
    { 0xadb7893d, SUC_NE_lexicon_tags_2 },
    { 0xfdcd893e, SUC_NE_lexicon_tags_4 },
    { 0x6f9893e, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x47b98941, SUC_NE_lexicon_tags_2 },
    { 0x61058942, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328f8947, SUC_NE_lexicon_tags_2 },
    { 0x22b78948, SUC_NE_lexicon_tags_2 },
    { 0x17a18949, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8b894c, SUC_NE_lexicon_tags_2 },
    { 0x1615894d, SUC_NE_lexicon_tags_2 },
    { 0x83c1894e, SUC_NE_lexicon_tags_2 },
    { 0xf723894e, SUC_NE_lexicon_tags_7 },
    { 0xbe178950, SUC_NE_lexicon_tags_2 },
    { 0x1cf98951, SUC_NE_lexicon_tags_2 },
    { 0x802f8952, SUC_NE_lexicon_tags_2 },
    { 0x4238952, SUC_NE_lexicon_tags_2 },
    { 0x9b1b8951, SUC_NE_lexicon_tags_2 },
    { 0x5eed894c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0498958, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad7895c, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x6417895e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf78962, SUC_NE_lexicon_tags_2 },
    { 0x8de58963, SUC_NE_lexicon_tags_2 },
    { 0xf7ef8963, SUC_NE_lexicon_tags_17 },
    { 0x102f8965, SUC_NE_lexicon_tags_2 },
    { 0x10e78962, SUC_NE_lexicon_tags_2 },
    { 0x502b8967, SUC_NE_lexicon_tags_4 },
    { 0x501f8964, SUC_NE_lexicon_tags_7 },
    { 0xac398962, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1a7896b, SUC_NE_lexicon_tags_3 },
    { 0x6169896c, SUC_NE_lexicon_tags_2 },
    { 0x398f896c, SUC_NE_lexicon_tags_2 },
    { 0x62bb896e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf718971, SUC_NE_lexicon_tags_1 },
    { 0xed518971, SUC_NE_lexicon_tags_7 },
    { 0x16df8973, SUC_NE_lexicon_tags_2 },
    { 0xf0a18973, SUC_NE_lexicon_tags_2 },
    { 0x50158975, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde458979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f3f897b, SUC_NE_lexicon_tags_34 },
    { 0x61b7897c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b9897f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d78986, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2faf8989, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3898d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6871898f, SUC_NE_lexicon_tags_2 },
    { 0xa03f8990, SUC_NE_lexicon_tags_2 },
    { 0x367d8991, SUC_NE_lexicon_tags_2 },
    { 0x71e9898f, SUC_NE_lexicon_tags_2 },
    { 0x7acb8993, SUC_NE_lexicon_tags_2 },
    { 0x4598994, SUC_NE_lexicon_tags_2 },
    { 0xb4998995, SUC_NE_lexicon_tags_2 },
    { 0xf7b38996, SUC_NE_lexicon_tags_0 },
    { 0x85e18997, SUC_NE_lexicon_tags_49 },
    { 0x4bcf8993, SUC_NE_lexicon_tags_2 },
    { 0xde418998, SUC_NE_lexicon_tags_2 },
    { 0xecf9899a, SUC_NE_lexicon_tags_2 },
    { 0x78fb899b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d2f899d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x617589a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fb89a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf12d89a5, SUC_NE_lexicon_tags_2 },
    { 0x452989a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9de389a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07789ab, SUC_NE_lexicon_tags_2 },
    { 0xc8b189ac, SUC_NE_lexicon_tags_2 },
    { 0xb11789ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec389b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1389b3, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784589b8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb89c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69189c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b8189c7, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441589cd, SUC_NE_lexicon_tags_0 },
    { 0xc28b89ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x93ab89d0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357989d5, SUC_NE_lexicon_tags_2 },
    { 0xe74b89d6, SUC_NE_lexicon_tags_14 },
    { 0xca8b89d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43589db, SUC_NE_lexicon_tags_2 },
    { 0x1a7389db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe31d89de, SUC_NE_lexicon_tags_4 },
    { 0x6c7589df, SUC_NE_lexicon_tags_2 },
    { 0xd45d89e0, SUC_NE_lexicon_tags_69 },
    { 0x38af89e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211f89ec, SUC_NE_lexicon_tags_7 },
    { 0x2b589ed, SUC_NE_lexicon_tags_5 },
    { 0x2d2589ee, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836189f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701189ff, SUC_NE_lexicon_tags_72 },
    { 0, NULL },
    { 0x333d8a01, SUC_NE_lexicon_tags_2 },
    { 0xb79d8a02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99818a04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3938a0a, SUC_NE_lexicon_tags_2 },
    { 0x6d418a0b, SUC_NE_lexicon_tags_2 },
    { 0x8e158a0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b1b8a0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fd8a13, SUC_NE_lexicon_tags_2 },
    { 0xe60f8a13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220d8a18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6918a1f, SUC_NE_lexicon_tags_2 },
    { 0x55058a1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f38a24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbae58a26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d78a29, SUC_NE_lexicon_tags_2 },
    { 0x314d8a2a, SUC_NE_lexicon_tags_2 },
    { 0x71bb8a2b, SUC_NE_lexicon_tags_2 },
    { 0xc6998a2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45b8a2f, SUC_NE_lexicon_tags_2 },
    { 0x40958a30, SUC_NE_lexicon_tags_2 },
    { 0xb7078a30, SUC_NE_lexicon_tags_2 },
    { 0x65a38a30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4538a35, SUC_NE_lexicon_tags_2 },
    { 0x3ad78a36, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf318a39, SUC_NE_lexicon_tags_4 },
    { 0xed138a39, SUC_NE_lexicon_tags_2 },
    { 0x4a18a3a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763d8a41, SUC_NE_lexicon_tags_0 },
    { 0x8c0b8a41, SUC_NE_lexicon_tags_2 },
    { 0x3b8a43, SUC_NE_lexicon_tags_2 },
    { 0x323d8a44, SUC_NE_lexicon_tags_2 },
    { 0xc6a98a45, SUC_NE_lexicon_tags_2 },
    { 0xcc8d8a46, SUC_NE_lexicon_tags_2 },
    { 0x9c958a47, SUC_NE_lexicon_tags_2 },
    { 0x95cd8a45, SUC_NE_lexicon_tags_2 },
    { 0x7df8a45, SUC_NE_lexicon_tags_2 },
    { 0xbca78a48, SUC_NE_lexicon_tags_40 },
    { 0x33118a4b, SUC_NE_lexicon_tags_2 },
    { 0xc8578a42, SUC_NE_lexicon_tags_2 },
    { 0xd1698a4c, SUC_NE_lexicon_tags_7 },
    { 0xf9d58a47, SUC_NE_lexicon_tags_2 },
    { 0xda158a4f, SUC_NE_lexicon_tags_2 },
    { 0xc3518a43, SUC_NE_lexicon_tags_2 },
    { 0xc3698a51, SUC_NE_lexicon_tags_2 },
    { 0xa7ff8a52, SUC_NE_lexicon_tags_2 },
    { 0x4f4f8a46, SUC_NE_lexicon_tags_2 },
    { 0x9e278a54, SUC_NE_lexicon_tags_2 },
    { 0x195b8a50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98f8a5a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29f8a5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9558a61, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f58a66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6838a68, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5d8a6b, SUC_NE_lexicon_tags_2 },
    { 0x91d58a6c, SUC_NE_lexicon_tags_2 },
    { 0xb4938a6b, SUC_NE_lexicon_tags_51 },
    { 0xbc8d8a6e, SUC_NE_lexicon_tags_2 },
    { 0x279b8a6b, SUC_NE_lexicon_tags_2 },
    { 0xfe278a6c, SUC_NE_lexicon_tags_2 },
    { 0xa3598a71, SUC_NE_lexicon_tags_2 },
    { 0x9a018a72, SUC_NE_lexicon_tags_2 },
    { 0x8ee98a6d, SUC_NE_lexicon_tags_2 },
    { 0x24618a74, SUC_NE_lexicon_tags_2 },
    { 0xbaff8a75, SUC_NE_lexicon_tags_2 },
    { 0xfae38a76, SUC_NE_lexicon_tags_4 },
    { 0x2bd38a77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215d8a7b, SUC_NE_lexicon_tags_2 },
    { 0x7ceb8a7c, SUC_NE_lexicon_tags_32 },
    { 0x4e578a7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5b8a82, SUC_NE_lexicon_tags_302 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ad8a85, SUC_NE_lexicon_tags_4 },
    { 0xe3d18a85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79a78a88, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87758a8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd18d8a8e, SUC_NE_lexicon_tags_2 },
    { 0xa0618a8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f98a91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69658a93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x814b8a95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beb8a99, SUC_NE_lexicon_tags_2 },
    { 0xab038a99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55238a9d, SUC_NE_lexicon_tags_2 },
    { 0xa258a9e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fb8aa1, SUC_NE_lexicon_tags_2 },
    { 0xf63f8aa1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da58aa4, SUC_NE_lexicon_tags_2 },
    { 0xc5098aa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431b8aa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a78aaf, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x24eb8ab1, SUC_NE_lexicon_tags_2 },
    { 0x2c858ab2, SUC_NE_lexicon_tags_2 },
    { 0x6b7d8ab3, SUC_NE_lexicon_tags_2 },
    { 0x39bb8ab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d8aba, SUC_NE_lexicon_tags_2 },
    { 0xbbcf8abb, SUC_NE_lexicon_tags_2 },
    { 0xcd8d8abc, SUC_NE_lexicon_tags_2 },
    { 0x123d8abc, SUC_NE_lexicon_tags_0 },
    { 0x7fdd8abd, SUC_NE_lexicon_tags_2 },
    { 0x26dd8abf, SUC_NE_lexicon_tags_2 },
    { 0x27378ac0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfe38ac2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d338ac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272b8ace, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x329f8ad1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b8ad5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a98ad9, SUC_NE_lexicon_tags_2 },
    { 0x88138ad9, SUC_NE_lexicon_tags_2 },
    { 0x76f8ada, SUC_NE_lexicon_tags_2 },
    { 0xa33f8adb, SUC_NE_lexicon_tags_2 },
    { 0xc2678ad9, SUC_NE_lexicon_tags_2 },
    { 0x499f8ade, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd58ae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b8ae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf98aea, SUC_NE_lexicon_tags_2 },
    { 0x6cf78aea, SUC_NE_lexicon_tags_2 },
    { 0x384b8aec, SUC_NE_lexicon_tags_2 },
    { 0x1bd18aea, SUC_NE_lexicon_tags_2 },
    { 0x47538aee, SUC_NE_lexicon_tags_4 },
    { 0xd478aea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe04f8af1, SUC_NE_lexicon_tags_2 },
    { 0xb7ad8af2, SUC_NE_lexicon_tags_2 },
    { 0xa5038af3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b138af5, SUC_NE_lexicon_tags_2 },
    { 0x4b18af6, SUC_NE_lexicon_tags_2 },
    { 0xed7f8af6, SUC_NE_lexicon_tags_30 },
    { 0xba918af8, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x14298afb, SUC_NE_lexicon_tags_30 },
    { 0xfaab8afb, SUC_NE_lexicon_tags_210 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c58b02, SUC_NE_lexicon_tags_2 },
    { 0xf91d8b03, SUC_NE_lexicon_tags_2 },
    { 0xc2878b04, SUC_NE_lexicon_tags_2 },
    { 0x5b178b02, SUC_NE_lexicon_tags_2 },
    { 0x9bd38b06, SUC_NE_lexicon_tags_2 },
    { 0x77d58b07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c78b0a, SUC_NE_lexicon_tags_2 },
    { 0x418d8b0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f938b0f, SUC_NE_lexicon_tags_2 },
    { 0x35d98b10, SUC_NE_lexicon_tags_7 },
    { 0xbd2f8b11, SUC_NE_lexicon_tags_2 },
    { 0x3a2d8b0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c98b18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62d8b1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60718b20, SUC_NE_lexicon_tags_2 },
    { 0x6ad38b21, SUC_NE_lexicon_tags_49 },
    { 0xa8bf8b22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf9d8b24, SUC_NE_lexicon_tags_2 },
    { 0x4ab38b25, SUC_NE_lexicon_tags_2 },
    { 0xca918b25, SUC_NE_lexicon_tags_2 },
    { 0xb1f18b27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3458b2f, SUC_NE_lexicon_tags_2 },
    { 0x2ddf8b30, SUC_NE_lexicon_tags_2 },
    { 0x169d8b2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x827d8b34, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff298b38, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x631b8b3b, SUC_NE_lexicon_tags_2 },
    { 0x56638b3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x511f8b3e, SUC_NE_lexicon_tags_2 },
    { 0xe238b3e, SUC_NE_lexicon_tags_2 },
    { 0xf4d18b40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe77b8b42, SUC_NE_lexicon_tags_2 },
    { 0xf7c58b43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0ed8b45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd578b47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dcb8b49, SUC_NE_lexicon_tags_3 },
    { 0x97eb8b49, SUC_NE_lexicon_tags_5 },
    { 0x40dd8b49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5a58b4d, SUC_NE_lexicon_tags_18 },
    { 0x3bf8b4d, SUC_NE_lexicon_tags_2 },
    { 0x6fc38b4f, SUC_NE_lexicon_tags_2 },
    { 0x7ee98b4e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bcd8b52, SUC_NE_lexicon_tags_2 },
    { 0xeb9b8b53, SUC_NE_lexicon_tags_2 },
    { 0x67218b54, SUC_NE_lexicon_tags_2 },
    { 0xed7b8b53, SUC_NE_lexicon_tags_2 },
    { 0x1aaf8b56, SUC_NE_lexicon_tags_2 },
    { 0xf4238b56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x119b8b59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef38b5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63638b60, SUC_NE_lexicon_tags_2 },
    { 0xb1ef8b61, SUC_NE_lexicon_tags_2 },
    { 0xc2c98b62, SUC_NE_lexicon_tags_2 },
    { 0x3acb8b60, SUC_NE_lexicon_tags_2 },
    { 0xac138b62, SUC_NE_lexicon_tags_2 },
    { 0x4a458b65, SUC_NE_lexicon_tags_2 },
    { 0x50438b66, SUC_NE_lexicon_tags_0 },
    { 0x17e98b67, SUC_NE_lexicon_tags_2 },
    { 0xbd58b62, SUC_NE_lexicon_tags_2 },
    { 0xfe78b69, SUC_NE_lexicon_tags_65 },
    { 0x1fe98b69, SUC_NE_lexicon_tags_2 },
    { 0xdb078b6b, SUC_NE_lexicon_tags_2 },
    { 0xc1c18b6c, SUC_NE_lexicon_tags_2 },
    { 0x38b78b61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2add8b70, SUC_NE_lexicon_tags_2 },
    { 0x33b8b70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca78b75, SUC_NE_lexicon_tags_2 },
    { 0xcb438b75, SUC_NE_lexicon_tags_2 },
    { 0x2b938b77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7f8b7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bdd8b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5efb8b80, SUC_NE_lexicon_tags_2 },
    { 0x39b8b81, SUC_NE_lexicon_tags_2 },
    { 0xb6898b82, SUC_NE_lexicon_tags_7 },
    { 0xa9c58b81, SUC_NE_lexicon_tags_2 },
    { 0x9a818b84, SUC_NE_lexicon_tags_2 },
    { 0x7e858b82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50cd8b87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb78b8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3a98b8e, SUC_NE_lexicon_tags_1 },
    { 0xa4ed8b8f, SUC_NE_lexicon_tags_2 },
    { 0x58df8b8f, SUC_NE_lexicon_tags_5 },
    { 0xb1d98b91, SUC_NE_lexicon_tags_2 },
    { 0xbec38b91, SUC_NE_lexicon_tags_2 },
    { 0x4e9b8b90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9058b95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10718b99, SUC_NE_lexicon_tags_3 },
    { 0x7b858b9a, SUC_NE_lexicon_tags_1 },
    { 0x7fe78b9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54118b9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6258ba1, SUC_NE_lexicon_tags_2 },
    { 0x4cb98ba1, SUC_NE_lexicon_tags_2 },
    { 0x4eb78ba3, SUC_NE_lexicon_tags_2 },
    { 0x33e38ba4, SUC_NE_lexicon_tags_2 },
    { 0x4b878ba3, SUC_NE_lexicon_tags_2 },
    { 0x61df8ba5, SUC_NE_lexicon_tags_2 },
    { 0x9a938ba7, SUC_NE_lexicon_tags_210 },
    { 0, NULL },
    { 0, NULL },
    { 0x77498baa, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4298bad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961f8bb1, SUC_NE_lexicon_tags_91 },
    { 0x1fa98bb2, SUC_NE_lexicon_tags_2 },
    { 0xf24f8bb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d8bbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31398bbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f938bc3, SUC_NE_lexicon_tags_2 },
    { 0xf8df8bc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bd38bc6, SUC_NE_lexicon_tags_7 },
    { 0x3dcb8bc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdf8bcc, SUC_NE_lexicon_tags_1 },
    { 0xfd838bcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36838bd5, SUC_NE_lexicon_tags_2 },
    { 0xce838bd6, SUC_NE_lexicon_tags_2 },
    { 0x96158bd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b78bda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce3f8bdc, SUC_NE_lexicon_tags_4 },
    { 0xb76f8bdd, SUC_NE_lexicon_tags_2 },
    { 0xffdd8bdd, SUC_NE_lexicon_tags_5 },
    { 0xfb698bdf, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d78be2, SUC_NE_lexicon_tags_2 },
    { 0xe3358be3, SUC_NE_lexicon_tags_2 },
    { 0xadcb8be3, SUC_NE_lexicon_tags_2 },
    { 0xc0f78be5, SUC_NE_lexicon_tags_2 },
    { 0xd81d8be4, SUC_NE_lexicon_tags_2 },
    { 0xb7cd8be6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f98bed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d498bef, SUC_NE_lexicon_tags_2 },
    { 0x42af8bf0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa87d8bf2, SUC_NE_lexicon_tags_2 },
    { 0x38e18bf3, SUC_NE_lexicon_tags_2 },
    { 0x66918bf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7b8bf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad098bfa, SUC_NE_lexicon_tags_1 },
    { 0x2af58bfa, SUC_NE_lexicon_tags_18 },
    { 0xea0f8bfc, SUC_NE_lexicon_tags_2 },
    { 0xb8c18bfa, SUC_NE_lexicon_tags_2 },
    { 0x5a8f8bfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x825f8c00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x549d8c02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dd8c05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd638c07, SUC_NE_lexicon_tags_2 },
    { 0x247b8c07, SUC_NE_lexicon_tags_14 },
    { 0xd6378c07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53958c11, SUC_NE_lexicon_tags_2 },
    { 0x738f8c12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x903b8c14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf38c19, SUC_NE_lexicon_tags_2 },
    { 0x7b578c19, SUC_NE_lexicon_tags_2 },
    { 0xe06d8c1b, SUC_NE_lexicon_tags_7 },
    { 0xdecd8c1c, SUC_NE_lexicon_tags_2 },
    { 0x8ad8c1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e58c21, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bd8c25, SUC_NE_lexicon_tags_2 },
    { 0x8b9b8c26, SUC_NE_lexicon_tags_2 },
    { 0xdb058c27, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d398c2d, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b8c32, SUC_NE_lexicon_tags_2 },
    { 0x9c158c32, SUC_NE_lexicon_tags_2 },
    { 0xc09d8c34, SUC_NE_lexicon_tags_3 },
    { 0x5c478c35, SUC_NE_lexicon_tags_4 },
    { 0x1fe38c32, SUC_NE_lexicon_tags_2 },
    { 0x9cdd8c33, SUC_NE_lexicon_tags_2 },
    { 0xb5d38c38, SUC_NE_lexicon_tags_3 },
    { 0x5ad78c39, SUC_NE_lexicon_tags_2 },
    { 0xa7e58c39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0f78c3c, SUC_NE_lexicon_tags_2 },
    { 0x172f8c3d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc58c45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0b8c49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96458c4d, SUC_NE_lexicon_tags_5 },
    { 0xd0138c4d, SUC_NE_lexicon_tags_2 },
    { 0x97d58c4f, SUC_NE_lexicon_tags_2 },
    { 0x9b758c50, SUC_NE_lexicon_tags_2 },
    { 0x8ec58c50, SUC_NE_lexicon_tags_2 },
    { 0x59478c4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c98c57, SUC_NE_lexicon_tags_2 },
    { 0x4b6f8c58, SUC_NE_lexicon_tags_2 },
    { 0x25158c58, SUC_NE_lexicon_tags_2 },
    { 0x7cbd8c5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87f8c5e, SUC_NE_lexicon_tags_2 },
    { 0x9d8c5f, SUC_NE_lexicon_tags_2 },
    { 0x2c0b8c60, SUC_NE_lexicon_tags_2 },
    { 0x9b978c61, SUC_NE_lexicon_tags_2 },
    { 0x675f8c62, SUC_NE_lexicon_tags_2 },
    { 0xab298c5e, SUC_NE_lexicon_tags_7 },
    { 0xc2c38c61, SUC_NE_lexicon_tags_2 },
    { 0x809b8c60, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d998c68, SUC_NE_lexicon_tags_2 },
    { 0x84b58c69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f058c6b, SUC_NE_lexicon_tags_2 },
    { 0x8e7f8c6c, SUC_NE_lexicon_tags_14 },
    { 0xbca78c6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf1d8c6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xded78c72, SUC_NE_lexicon_tags_2 },
    { 0x5e638c73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0b8c79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c78c7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d678c7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b8c85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b3f8c87, SUC_NE_lexicon_tags_7 },
    { 0xb5818c87, SUC_NE_lexicon_tags_2 },
    { 0xcfbf8c88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d8c8e, SUC_NE_lexicon_tags_2 },
    { 0x9a1d8c8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85b18c91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb738c97, SUC_NE_lexicon_tags_2 },
    { 0xfeab8c97, SUC_NE_lexicon_tags_2 },
    { 0xb93b8c99, SUC_NE_lexicon_tags_3 },
    { 0x5738c97, SUC_NE_lexicon_tags_2 },
    { 0x1c338c9b, SUC_NE_lexicon_tags_2 },
    { 0x4c858c9c, SUC_NE_lexicon_tags_2 },
    { 0x1e058c9d, SUC_NE_lexicon_tags_2 },
    { 0x339b8c9b, SUC_NE_lexicon_tags_7 },
    { 0xfd2d8c9f, SUC_NE_lexicon_tags_2 },
    { 0x80d58c9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e38ca2, SUC_NE_lexicon_tags_2 },
    { 0x7d318ca2, SUC_NE_lexicon_tags_2 },
    { 0x92778ca2, SUC_NE_lexicon_tags_2 },
    { 0xa3a38ca5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba38ca7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc078caa, SUC_NE_lexicon_tags_2 },
    { 0x1ad58cab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d18cb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cf58cb3, SUC_NE_lexicon_tags_2 },
    { 0x4b218cb4, SUC_NE_lexicon_tags_0 },
    { 0x33198cb4, SUC_NE_lexicon_tags_3 },
    { 0xfcc98cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc78cba, SUC_NE_lexicon_tags_2 },
    { 0x459f8cba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5418cbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9698cc3, SUC_NE_lexicon_tags_2 },
    { 0xf9738cc4, SUC_NE_lexicon_tags_2 },
    { 0xcbed8cc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cf8cc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8ccd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e78cd5, SUC_NE_lexicon_tags_2 },
    { 0x18978cd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672b8cda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1b8cde, SUC_NE_lexicon_tags_2 },
    { 0xb4ff8cde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x757f8ce1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f8ce4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x12b98ce6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bb8cea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x929d8ced, SUC_NE_lexicon_tags_2 },
    { 0x4ea98cee, SUC_NE_lexicon_tags_2 },
    { 0x2fb38cef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x63db8cf1, SUC_NE_lexicon_tags_2 },
    { 0x49378cf2, SUC_NE_lexicon_tags_2 },
    { 0x3bd58cf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x352d8cf5, SUC_NE_lexicon_tags_2 },
    { 0xf4738cf5, SUC_NE_lexicon_tags_2 },
    { 0x4f0b8cf7, SUC_NE_lexicon_tags_2 },
    { 0xe8d38cf8, SUC_NE_lexicon_tags_2 },
    { 0xc5bd8cf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c98cfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb38d01, SUC_NE_lexicon_tags_2 },
    { 0xa01f8d01, SUC_NE_lexicon_tags_2 },
    { 0xc76f8d03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e18d08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1be18d0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52398d0d, SUC_NE_lexicon_tags_2 },
    { 0x84a38d0e, SUC_NE_lexicon_tags_2 },
    { 0xc1058d0f, SUC_NE_lexicon_tags_3 },
    { 0x4ddd8d0f, SUC_NE_lexicon_tags_2 },
    { 0x9ba78d11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f5b8d13, SUC_NE_lexicon_tags_2 },
    { 0x678f8d14, SUC_NE_lexicon_tags_49 },
    { 0xf94b8d15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458f8d1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe758d1d, SUC_NE_lexicon_tags_2 },
    { 0x47af8d1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbd8d23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x477d8d25, SUC_NE_lexicon_tags_7 },
    { 0xe0778d26, SUC_NE_lexicon_tags_2 },
    { 0x73f58d27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19d8d2c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x352b8d2f, SUC_NE_lexicon_tags_2 },
    { 0xf0a18d30, SUC_NE_lexicon_tags_2 },
    { 0x89b58d31, SUC_NE_lexicon_tags_2 },
    { 0x15ad8d32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9278d35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b758d3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd8d43, SUC_NE_lexicon_tags_2 },
    { 0x7c958d44, SUC_NE_lexicon_tags_2 },
    { 0xfbdf8d45, SUC_NE_lexicon_tags_2 },
    { 0x1d4d8d46, SUC_NE_lexicon_tags_2 },
    { 0x7edb8d45, SUC_NE_lexicon_tags_2 },
    { 0x8fcd8d48, SUC_NE_lexicon_tags_2 },
    { 0x3cd58d49, SUC_NE_lexicon_tags_2 },
    { 0x4cd78d4a, SUC_NE_lexicon_tags_2 },
    { 0xabb18d49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc778d4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b6b8d51, SUC_NE_lexicon_tags_2 },
    { 0xe8ed8d51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f98d55, SUC_NE_lexicon_tags_2 },
    { 0x45178d56, SUC_NE_lexicon_tags_2 },
    { 0x3f898d57, SUC_NE_lexicon_tags_2 },
    { 0x16d38d58, SUC_NE_lexicon_tags_2 },
    { 0xfb0d8d59, SUC_NE_lexicon_tags_1 },
    { 0xdfff8d5a, SUC_NE_lexicon_tags_2 },
    { 0x26af8d57, SUC_NE_lexicon_tags_1 },
    { 0x35bf8d5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7338d5f, SUC_NE_lexicon_tags_2 },
    { 0x39f58d60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x402d8d63, SUC_NE_lexicon_tags_2 },
    { 0x610d8d63, SUC_NE_lexicon_tags_2 },
    { 0xba78d65, SUC_NE_lexicon_tags_2 },
    { 0x30358d66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d58d6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c698d6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1138d71, SUC_NE_lexicon_tags_2 },
    { 0xb8718d71, SUC_NE_lexicon_tags_2 },
    { 0x6e6f8d72, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbd8d76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ca38d78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcf8d7e, SUC_NE_lexicon_tags_2 },
    { 0x35078d7f, SUC_NE_lexicon_tags_2 },
    { 0xd24b8d80, SUC_NE_lexicon_tags_2 },
    { 0x65718d80, SUC_NE_lexicon_tags_2 },
    { 0xc40f8d80, SUC_NE_lexicon_tags_2 },
    { 0xab898d7f, SUC_NE_lexicon_tags_2 },
    { 0x1cd18d84, SUC_NE_lexicon_tags_19 },
    { 0x9a638d82, SUC_NE_lexicon_tags_2 },
    { 0x23458d86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3358d88, SUC_NE_lexicon_tags_2 },
    { 0xf4f78d89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3e58d8b, SUC_NE_lexicon_tags_2 },
    { 0x8b4f8d8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69418d8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26138d90, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0x4ab18d92, SUC_NE_lexicon_tags_2 },
    { 0x6f1b8d93, SUC_NE_lexicon_tags_2 },
    { 0x21118d92, SUC_NE_lexicon_tags_2 },
    { 0x78d8d93, SUC_NE_lexicon_tags_4 },
    { 0x9f3f8d96, SUC_NE_lexicon_tags_2 },
    { 0x807b8d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11738d9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1318da1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa84b8da3, SUC_NE_lexicon_tags_2 },
    { 0x3c298da4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78698da7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466d8dae, SUC_NE_lexicon_tags_8 },
    { 0x97d98dae, SUC_NE_lexicon_tags_2 },
    { 0x40c58db0, SUC_NE_lexicon_tags_2 },
    { 0x5f518daf, SUC_NE_lexicon_tags_2 },
    { 0x71498dae, SUC_NE_lexicon_tags_2 },
    { 0xa9a98db3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1f8dbc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b78dc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bb38dc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5f8dc6, SUC_NE_lexicon_tags_2 },
    { 0x9ed78dc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3118dca, SUC_NE_lexicon_tags_2 },
    { 0x5778dcb, SUC_NE_lexicon_tags_2 },
    { 0xe53b8dcc, SUC_NE_lexicon_tags_2 },
    { 0xf51d8dcc, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c58dd0, SUC_NE_lexicon_tags_2 },
    { 0xdc318dd0, SUC_NE_lexicon_tags_2 },
    { 0xbe5b8dd2, SUC_NE_lexicon_tags_3 },
    { 0x93318dd0, SUC_NE_lexicon_tags_1 },
    { 0x6c3b8dd4, SUC_NE_lexicon_tags_58 },
    { 0xcd978dd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa98dda, SUC_NE_lexicon_tags_2 },
    { 0x6f958dda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb83b8ddd, SUC_NE_lexicon_tags_2 },
    { 0x20698ddd, SUC_NE_lexicon_tags_2 },
    { 0x76b38ddd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d18de3, SUC_NE_lexicon_tags_2 },
    { 0x82498de4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404b8dea, SUC_NE_lexicon_tags_2 },
    { 0x41b18deb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64618dee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae098df0, SUC_NE_lexicon_tags_2 },
    { 0x4d338df1, SUC_NE_lexicon_tags_2 },
    { 0x15278df2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f8df9, SUC_NE_lexicon_tags_2 },
    { 0x9da98dfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7f8e03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee98e07, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38538e0e, SUC_NE_lexicon_tags_2 },
    { 0x41338e0e, SUC_NE_lexicon_tags_2 },
    { 0x83bd8e0f, SUC_NE_lexicon_tags_2 },
    { 0xfe418e0f, SUC_NE_lexicon_tags_2 },
    { 0xc4638e12, SUC_NE_lexicon_tags_8 },
    { 0x2b238e13, SUC_NE_lexicon_tags_2 },
    { 0x89ff8e14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf798e16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ff8e1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74b8e1d, SUC_NE_lexicon_tags_2 },
    { 0xb6458e1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d38e21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64d8e24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff58e27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7018e29, SUC_NE_lexicon_tags_4 },
    { 0xe57d8e2a, SUC_NE_lexicon_tags_2 },
    { 0xc12b8e2b, SUC_NE_lexicon_tags_2 },
    { 0xf5538e2c, SUC_NE_lexicon_tags_2 },
    { 0xba478e2d, SUC_NE_lexicon_tags_18 },
    { 0xe3a38e2e, SUC_NE_lexicon_tags_2 },
    { 0xf3898e2c, SUC_NE_lexicon_tags_2 },
    { 0xf7738e30, SUC_NE_lexicon_tags_2 },
    { 0xe9678e31, SUC_NE_lexicon_tags_2 },
    { 0xf44f8e32, SUC_NE_lexicon_tags_24 },
    { 0xf2c58e33, SUC_NE_lexicon_tags_2 },
    { 0x74e38e32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26ed8e36, SUC_NE_lexicon_tags_2 },
    { 0x66538e37, SUC_NE_lexicon_tags_2 },
    { 0xee958e37, SUC_NE_lexicon_tags_2 },
    { 0x9adb8e39, SUC_NE_lexicon_tags_4 },
    { 0x1b538e3a, SUC_NE_lexicon_tags_2 },
    { 0xd3178e3b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a58e3f, SUC_NE_lexicon_tags_2 },
    { 0xfe5d8e40, SUC_NE_lexicon_tags_2 },
    { 0x66e98e41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa7f8e43, SUC_NE_lexicon_tags_2 },
    { 0x50fd8e44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70438e46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de58e49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0d8e4f, SUC_NE_lexicon_tags_2 },
    { 0x564f8e50, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a758e53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c5f8e55, SUC_NE_lexicon_tags_2 },
    { 0x6efd8e55, SUC_NE_lexicon_tags_2 },
    { 0xbf1d8e57, SUC_NE_lexicon_tags_2 },
    { 0xeb118e57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e458e5b, SUC_NE_lexicon_tags_2 },
    { 0x68f38e5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde758e5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d8e64, SUC_NE_lexicon_tags_2 },
    { 0xf8878e65, SUC_NE_lexicon_tags_2 },
    { 0xee698e65, SUC_NE_lexicon_tags_2 },
    { 0x2cb8e67, SUC_NE_lexicon_tags_2 },
    { 0x8bc78e68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6638e6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8778e6d, SUC_NE_lexicon_tags_2 },
    { 0x698f8e6e, SUC_NE_lexicon_tags_8 },
    { 0x98eb8e6e, SUC_NE_lexicon_tags_2 },
    { 0x81018e6d, SUC_NE_lexicon_tags_2 },
    { 0x4dbd8e71, SUC_NE_lexicon_tags_2 },
    { 0xe3678e72, SUC_NE_lexicon_tags_2 },
    { 0x6a5b8e72, SUC_NE_lexicon_tags_7 },
    { 0x1c58e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56d8e77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec98e7f, SUC_NE_lexicon_tags_2 },
    { 0xfc178e7f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb378e87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d58e8a, SUC_NE_lexicon_tags_4 },
    { 0xc0718e8a, SUC_NE_lexicon_tags_30 },
    { 0x880b8e8a, SUC_NE_lexicon_tags_18 },
    { 0x52ef8e8d, SUC_NE_lexicon_tags_3 },
    { 0x68b78e8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0af8e94, SUC_NE_lexicon_tags_2 },
    { 0xa9918e95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d6b8e97, SUC_NE_lexicon_tags_3 },
    { 0x1c8b8e97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b8e9b, SUC_NE_lexicon_tags_2 },
    { 0x95738e9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beb8e9f, SUC_NE_lexicon_tags_2 },
    { 0xdef98e9f, SUC_NE_lexicon_tags_2 },
    { 0x35738ea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3678ea3, SUC_NE_lexicon_tags_2 },
    { 0x98198ea3, SUC_NE_lexicon_tags_2 },
    { 0xb0478ea3, SUC_NE_lexicon_tags_2 },
    { 0x2a898ea3, SUC_NE_lexicon_tags_2 },
    { 0x5b998ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x53ab8ea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48f8eb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88698eb4, SUC_NE_lexicon_tags_2 },
    { 0xeb658eb5, SUC_NE_lexicon_tags_2 },
    { 0x47c18eb5, SUC_NE_lexicon_tags_49 },
    { 0xad278eb6, SUC_NE_lexicon_tags_2 },
    { 0x2a0d8eb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe318ebe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68af8ec0, SUC_NE_lexicon_tags_2 },
    { 0x31138ec1, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0578ec6, SUC_NE_lexicon_tags_2 },
    { 0xd0c58ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382d8ecb, SUC_NE_lexicon_tags_2 },
    { 0x975b8ecc, SUC_NE_lexicon_tags_2 },
    { 0x68f98ecd, SUC_NE_lexicon_tags_2 },
    { 0x56878ecb, SUC_NE_lexicon_tags_0 },
    { 0xb8178ece, SUC_NE_lexicon_tags_2 },
    { 0x12d18ece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x625f8ed3, SUC_NE_lexicon_tags_49 },
    { 0x4538ed4, SUC_NE_lexicon_tags_2 },
    { 0x77578ed3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ef8ed9, SUC_NE_lexicon_tags_2 },
    { 0x67d78ed9, SUC_NE_lexicon_tags_2 },
    { 0xbfc38eda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f8edf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72b8ee2, SUC_NE_lexicon_tags_13 },
    { 0x696d8ee2, SUC_NE_lexicon_tags_2 },
    { 0xb8e98ee4, SUC_NE_lexicon_tags_2 },
    { 0xbbab8ee5, SUC_NE_lexicon_tags_2 },
    { 0xb73f8ee6, SUC_NE_lexicon_tags_2 },
    { 0x48ad8ee6, SUC_NE_lexicon_tags_2 },
    { 0xf4b98ee6, SUC_NE_lexicon_tags_32 },
    { 0xf858ee6, SUC_NE_lexicon_tags_2 },
    { 0xf0f58ee4, SUC_NE_lexicon_tags_4 },
    { 0x3b478eeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a838ef0, SUC_NE_lexicon_tags_2 },
    { 0x755b8ef1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d8ef5, SUC_NE_lexicon_tags_2 },
    { 0x7c58ef5, SUC_NE_lexicon_tags_2 },
    { 0x93b98ef7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17498ef9, SUC_NE_lexicon_tags_2 },
    { 0x93418efa, SUC_NE_lexicon_tags_2 },
    { 0xa7078efa, SUC_NE_lexicon_tags_2 },
    { 0xbc6f8efb, SUC_NE_lexicon_tags_1 },
    { 0xaf898efd, SUC_NE_lexicon_tags_2 },
    { 0xada58efe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3498f05, SUC_NE_lexicon_tags_2 },
    { 0xf9ad8f05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a438f08, SUC_NE_lexicon_tags_2 },
    { 0x40318f09, SUC_NE_lexicon_tags_2 },
    { 0xfb7b8f0a, SUC_NE_lexicon_tags_2 },
    { 0x1d6b8f0a, SUC_NE_lexicon_tags_81 },
    { 0x9a018f0c, SUC_NE_lexicon_tags_30 },
    { 0xc8bd8f0d, SUC_NE_lexicon_tags_18 },
    { 0x5c3b8f0d, SUC_NE_lexicon_tags_2 },
    { 0x9ac98f0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36258f1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x630f8f1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd8f20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b8f26, SUC_NE_lexicon_tags_2 },
    { 0x8e778f27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe01b8f29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4fb8f2b, SUC_NE_lexicon_tags_5 },
    { 0xd94f8f2b, SUC_NE_lexicon_tags_2 },
    { 0xd6df8f2b, SUC_NE_lexicon_tags_2 },
    { 0x30818f2e, SUC_NE_lexicon_tags_4 },
    { 0x35bf8f2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c58f32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf158f34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36838f39, SUC_NE_lexicon_tags_2 },
    { 0xca158f39, SUC_NE_lexicon_tags_2 },
    { 0xcd5f8f3b, SUC_NE_lexicon_tags_2 },
    { 0x7a698f3c, SUC_NE_lexicon_tags_7 },
    { 0xe8a98f3b, SUC_NE_lexicon_tags_2 },
    { 0x56358f3c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb8f43, SUC_NE_lexicon_tags_2 },
    { 0xe858f44, SUC_NE_lexicon_tags_2 },
    { 0x35638f44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25658f4b, SUC_NE_lexicon_tags_2 },
    { 0x96878f4c, SUC_NE_lexicon_tags_2 },
    { 0xb3518f4b, SUC_NE_lexicon_tags_2 },
    { 0x102f8f4e, SUC_NE_lexicon_tags_2 },
    { 0x91358f4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13e98f51, SUC_NE_lexicon_tags_17 },
    { 0x7338f51, SUC_NE_lexicon_tags_2 },
    { 0x69fb8f53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fb8f56, SUC_NE_lexicon_tags_2 },
    { 0xae658f57, SUC_NE_lexicon_tags_49 },
    { 0xb3818f58, SUC_NE_lexicon_tags_36 },
    { 0x54418f59, SUC_NE_lexicon_tags_2 },
    { 0xa4818f58, SUC_NE_lexicon_tags_1 },
    { 0xcb418f5b, SUC_NE_lexicon_tags_2 },
    { 0x57478f5c, SUC_NE_lexicon_tags_2 },
    { 0xc4f98f5c, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2838f60, SUC_NE_lexicon_tags_2 },
    { 0xa1778f60, SUC_NE_lexicon_tags_40 },
    { 0xf9fd8f62, SUC_NE_lexicon_tags_0 },
    { 0xea258f60, SUC_NE_lexicon_tags_2 },
    { 0x2d698f64, SUC_NE_lexicon_tags_2 },
    { 0xee618f62, SUC_NE_lexicon_tags_2 },
    { 0x31378f61, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x72278f68, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc7678f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0198f6e, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21b8f72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91238f75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3438f77, SUC_NE_lexicon_tags_2 },
    { 0xebf8f78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4878f7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9cb8f7c, SUC_NE_lexicon_tags_2 },
    { 0xe5858f7d, SUC_NE_lexicon_tags_2 },
    { 0xb8b8f7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd22b8f80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6138f82, SUC_NE_lexicon_tags_8 },
    { 0x3fbb8f82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a78f86, SUC_NE_lexicon_tags_2 },
    { 0x85598f87, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xee158f8a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c698f8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cd8f9f, SUC_NE_lexicon_tags_2 },
    { 0x9d8d8fa0, SUC_NE_lexicon_tags_2 },
    { 0x5e358fa1, SUC_NE_lexicon_tags_2 },
    { 0xbb418f9f, SUC_NE_lexicon_tags_2 },
    { 0x15d98fa2, SUC_NE_lexicon_tags_24 },
    { 0x92638fa4, SUC_NE_lexicon_tags_2 },
    { 0x3a9f8fa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b478fa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197d8faf, SUC_NE_lexicon_tags_2 },
    { 0x3da78fb0, SUC_NE_lexicon_tags_2 },
    { 0x73758fb1, SUC_NE_lexicon_tags_7 },
    { 0x3a798fb2, SUC_NE_lexicon_tags_2 },
    { 0x1e98fb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2f8fc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92c18fc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81878fc5, SUC_NE_lexicon_tags_2 },
    { 0x2c378fc6, SUC_NE_lexicon_tags_2 },
    { 0x10cd8fc5, SUC_NE_lexicon_tags_2 },
    { 0x657b8fc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92f8fcf, SUC_NE_lexicon_tags_2 },
    { 0x6d9b8fcf, SUC_NE_lexicon_tags_2 },
    { 0x861f8fd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c798fd4, SUC_NE_lexicon_tags_2 },
    { 0x76ad8fd5, SUC_NE_lexicon_tags_2 },
    { 0xd6d8fd6, SUC_NE_lexicon_tags_24 },
    { 0xb7538fd7, SUC_NE_lexicon_tags_2 },
    { 0x34038fd8, SUC_NE_lexicon_tags_2 },
    { 0x15018fd7, SUC_NE_lexicon_tags_2 },
    { 0xbb378fd9, SUC_NE_lexicon_tags_2 },
    { 0x63358fdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eab8fdd, SUC_NE_lexicon_tags_2 },
    { 0xc4498fdd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22098fe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd438fe8, SUC_NE_lexicon_tags_2 },
    { 0x13b38fe9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0598ff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f8ff5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b98ff7, SUC_NE_lexicon_tags_2 },
    { 0x2c938ff7, SUC_NE_lexicon_tags_2 },
    { 0x7a858ff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f38ffd, SUC_NE_lexicon_tags_2 },
    { 0x725f8ffe, SUC_NE_lexicon_tags_2 },
    { 0x37cd8ffe, SUC_NE_lexicon_tags_2 },
    { 0xe0bf8ffe, SUC_NE_lexicon_tags_2 },
    { 0x74338ffd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9d9004, SUC_NE_lexicon_tags_2 },
    { 0xbacf9005, SUC_NE_lexicon_tags_2 },
    { 0xe4499006, SUC_NE_lexicon_tags_2 },
    { 0xff7b9006, SUC_NE_lexicon_tags_2 },
    { 0xef479006, SUC_NE_lexicon_tags_2 },
    { 0x751f9009, SUC_NE_lexicon_tags_254 },
    { 0xfc7f900a, SUC_NE_lexicon_tags_7 },
    { 0x2999900a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62479010, SUC_NE_lexicon_tags_2 },
    { 0xeec19011, SUC_NE_lexicon_tags_2 },
    { 0x643d9011, SUC_NE_lexicon_tags_4 },
    { 0xe7fd9013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b39015, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a77901a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543901e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67f9021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32cf9023, SUC_NE_lexicon_tags_2 },
    { 0x6e019023, SUC_NE_lexicon_tags_2 },
    { 0x46b19025, SUC_NE_lexicon_tags_2 },
    { 0x1f179025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfd902a, SUC_NE_lexicon_tags_4 },
    { 0x6627902b, SUC_NE_lexicon_tags_2 },
    { 0x476d902c, SUC_NE_lexicon_tags_2 },
    { 0xe3a3902d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1117902f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d19034, SUC_NE_lexicon_tags_2 },
    { 0xe69f9035, SUC_NE_lexicon_tags_2 },
    { 0xdc219036, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f559039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd78f903b, SUC_NE_lexicon_tags_2 },
    { 0x48fb903b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed01903e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b9041, SUC_NE_lexicon_tags_2 },
    { 0xdbf39041, SUC_NE_lexicon_tags_2 },
    { 0xfc7b9042, SUC_NE_lexicon_tags_25 },
    { 0x593f9042, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132b904a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2507904c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b39904f, SUC_NE_lexicon_tags_2 },
    { 0x4deb9050, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab39054, SUC_NE_lexicon_tags_2 },
    { 0x8b299055, SUC_NE_lexicon_tags_2 },
    { 0x52e99056, SUC_NE_lexicon_tags_2 },
    { 0xec479057, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce3905a, SUC_NE_lexicon_tags_2 },
    { 0xc9eb905b, SUC_NE_lexicon_tags_2 },
    { 0x58b5905a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27d905f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9079061, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf5d9063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a399066, SUC_NE_lexicon_tags_4 },
    { 0xa6f39067, SUC_NE_lexicon_tags_2 },
    { 0x8a19068, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6927906a, SUC_NE_lexicon_tags_2 },
    { 0xae47906b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0d5906d, SUC_NE_lexicon_tags_2 },
    { 0x83bd906e, SUC_NE_lexicon_tags_2 },
    { 0xbc33906e, SUC_NE_lexicon_tags_2 },
    { 0x7f19906d, SUC_NE_lexicon_tags_2 },
    { 0x3e79906f, SUC_NE_lexicon_tags_2 },
    { 0xa5ef9072, SUC_NE_lexicon_tags_2 },
    { 0x53c79071, SUC_NE_lexicon_tags_2 },
    { 0xad299072, SUC_NE_lexicon_tags_2 },
    { 0xcc799075, SUC_NE_lexicon_tags_2 },
    { 0xfe439076, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb11d9078, SUC_NE_lexicon_tags_2 },
    { 0x7a099078, SUC_NE_lexicon_tags_2 },
    { 0x349907a, SUC_NE_lexicon_tags_98 },
    { 0x6d11907b, SUC_NE_lexicon_tags_2 },
    { 0x5c39907b, SUC_NE_lexicon_tags_2 },
    { 0xea05907d, SUC_NE_lexicon_tags_2 },
    { 0x52e9907e, SUC_NE_lexicon_tags_365 },
    { 0, NULL },
    { 0x526b9080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99659086, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4577908b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee7908e, SUC_NE_lexicon_tags_2 },
    { 0x4da3908f, SUC_NE_lexicon_tags_2 },
    { 0x686d9090, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b239093, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51419096, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x553b9099, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc7b909b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6d909e, SUC_NE_lexicon_tags_2 },
    { 0xff1909f, SUC_NE_lexicon_tags_2 },
    { 0x38df90a0, SUC_NE_lexicon_tags_7 },
    { 0xd761909e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cf990a3, SUC_NE_lexicon_tags_2 },
    { 0x565f90a4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xebe990a6, SUC_NE_lexicon_tags_2 },
    { 0xb38590a6, SUC_NE_lexicon_tags_19 },
    { 0x56e990a8, SUC_NE_lexicon_tags_2 },
    { 0x9fa190a6, SUC_NE_lexicon_tags_2 },
    { 0xff7b90a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb90af, SUC_NE_lexicon_tags_2 },
    { 0xa7f90af, SUC_NE_lexicon_tags_2 },
    { 0x93cb90b1, SUC_NE_lexicon_tags_2 },
    { 0x8d4190b2, SUC_NE_lexicon_tags_2 },
    { 0x6e2790b0, SUC_NE_lexicon_tags_2 },
    { 0x991b90b4, SUC_NE_lexicon_tags_2 },
    { 0x922590b5, SUC_NE_lexicon_tags_2 },
    { 0xf6f190b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78390bb, SUC_NE_lexicon_tags_2 },
    { 0x7fd390bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x324590be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc990c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb790c5, SUC_NE_lexicon_tags_2 },
    { 0x7a190c5, SUC_NE_lexicon_tags_2 },
    { 0x5cb590c7, SUC_NE_lexicon_tags_2 },
    { 0x92e590c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28db90ca, SUC_NE_lexicon_tags_2 },
    { 0x283b90cb, SUC_NE_lexicon_tags_2 },
    { 0xc0f390cb, SUC_NE_lexicon_tags_2 },
    { 0xe8e590cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0390d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bb90d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70b90db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x598b90de, SUC_NE_lexicon_tags_2 },
    { 0xee0b90df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d90e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cb90e6, SUC_NE_lexicon_tags_0 },
    { 0x848d90e7, SUC_NE_lexicon_tags_2 },
    { 0x416f90e8, SUC_NE_lexicon_tags_2 },
    { 0x76eb90e9, SUC_NE_lexicon_tags_2 },
    { 0xf7f990e9, SUC_NE_lexicon_tags_2 },
    { 0x7e8990eb, SUC_NE_lexicon_tags_2 },
    { 0xbfff90e9, SUC_NE_lexicon_tags_2 },
    { 0xbd9f90ed, SUC_NE_lexicon_tags_7 },
    { 0x1cd790ee, SUC_NE_lexicon_tags_2 },
    { 0x95c190ed, SUC_NE_lexicon_tags_2 },
    { 0xc7e790eb, SUC_NE_lexicon_tags_2 },
    { 0x4e0b90f1, SUC_NE_lexicon_tags_2 },
    { 0x438390f1, SUC_NE_lexicon_tags_2 },
    { 0x97b190f3, SUC_NE_lexicon_tags_2 },
    { 0xe00790f4, SUC_NE_lexicon_tags_2 },
    { 0x8b0f90f3, SUC_NE_lexicon_tags_2 },
    { 0x82cb90f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a190f8, SUC_NE_lexicon_tags_2 },
    { 0x442590f9, SUC_NE_lexicon_tags_2 },
    { 0x874f90fa, SUC_NE_lexicon_tags_2 },
    { 0xf78b90fb, SUC_NE_lexicon_tags_2 },
    { 0xa27390fa, SUC_NE_lexicon_tags_1 },
    { 0xbab590fd, SUC_NE_lexicon_tags_2 },
    { 0x906b90fd, SUC_NE_lexicon_tags_2 },
    { 0xdde590fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2901910a, SUC_NE_lexicon_tags_2 },
    { 0xa80d910b, SUC_NE_lexicon_tags_2 },
    { 0x1339910b, SUC_NE_lexicon_tags_2 },
    { 0x2ad5910b, SUC_NE_lexicon_tags_2 },
    { 0xd0b5910c, SUC_NE_lexicon_tags_2 },
    { 0x7223910f, SUC_NE_lexicon_tags_2 },
    { 0x9b43910f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa899113, SUC_NE_lexicon_tags_2 },
    { 0x9d339114, SUC_NE_lexicon_tags_2 },
    { 0x9fb19115, SUC_NE_lexicon_tags_2 },
    { 0xaf39116, SUC_NE_lexicon_tags_304 },
    { 0, NULL },
    { 0, NULL },
    { 0xc999119, SUC_NE_lexicon_tags_2 },
    { 0xe5239119, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42559120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7f9123, SUC_NE_lexicon_tags_2 },
    { 0x4c399123, SUC_NE_lexicon_tags_2 },
    { 0x8a2f9125, SUC_NE_lexicon_tags_51 },
    { 0xfb119125, SUC_NE_lexicon_tags_2 },
    { 0x58f39125, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef79129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ee7912b, SUC_NE_lexicon_tags_3 },
    { 0x5fab912c, SUC_NE_lexicon_tags_2 },
    { 0x7c91912d, SUC_NE_lexicon_tags_2 },
    { 0x8c45912b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66e79130, SUC_NE_lexicon_tags_7 },
    { 0x920f9131, SUC_NE_lexicon_tags_2 },
    { 0x91ef9132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26159138, SUC_NE_lexicon_tags_2 },
    { 0x22fb9139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b7913c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a99140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0b9143, SUC_NE_lexicon_tags_2 },
    { 0x9b579143, SUC_NE_lexicon_tags_2 },
    { 0x3abd9145, SUC_NE_lexicon_tags_2 },
    { 0xceb19146, SUC_NE_lexicon_tags_2 },
    { 0xaceb9146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c89914a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f7914e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8259156, SUC_NE_lexicon_tags_2 },
    { 0x6b5d9157, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd7915b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232d9162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc01916b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f39171, SUC_NE_lexicon_tags_2 },
    { 0xd2a99172, SUC_NE_lexicon_tags_4 },
    { 0x93779173, SUC_NE_lexicon_tags_2 },
    { 0x9fa79173, SUC_NE_lexicon_tags_0 },
    { 0x5cbd9172, SUC_NE_lexicon_tags_2 },
    { 0x158b9173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63679179, SUC_NE_lexicon_tags_2 },
    { 0x14d5917a, SUC_NE_lexicon_tags_2 },
    { 0xfad5917b, SUC_NE_lexicon_tags_2 },
    { 0x3bc5917a, SUC_NE_lexicon_tags_2 },
    { 0xdc2b917d, SUC_NE_lexicon_tags_2 },
    { 0x1747917b, SUC_NE_lexicon_tags_2 },
    { 0x63f7917d, SUC_NE_lexicon_tags_5 },
    { 0xc4b39180, SUC_NE_lexicon_tags_2 },
    { 0x46cd9180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88a39183, SUC_NE_lexicon_tags_2 },
    { 0x25159183, SUC_NE_lexicon_tags_2 },
    { 0x8f459185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9beb9187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbd918a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2365918c, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x5851918f, SUC_NE_lexicon_tags_2 },
    { 0x5e1f918f, SUC_NE_lexicon_tags_2 },
    { 0xf34f9191, SUC_NE_lexicon_tags_2 },
    { 0xc9a59192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f79196, SUC_NE_lexicon_tags_2 },
    { 0x18959197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683991a1, SUC_NE_lexicon_tags_4 },
    { 0xfc0991a2, SUC_NE_lexicon_tags_2 },
    { 0xff8791a3, SUC_NE_lexicon_tags_2 },
    { 0xcac591a4, SUC_NE_lexicon_tags_2 },
    { 0xd4f391a5, SUC_NE_lexicon_tags_2 },
    { 0xd0a191a6, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625b91aa, SUC_NE_lexicon_tags_2 },
    { 0xff8591ab, SUC_NE_lexicon_tags_4 },
    { 0x686191aa, SUC_NE_lexicon_tags_2 },
    { 0xc83791ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf91b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f391b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd79991b6, SUC_NE_lexicon_tags_3 },
    { 0x76c391b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebf91bb, SUC_NE_lexicon_tags_2 },
    { 0x549591bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3991bf, SUC_NE_lexicon_tags_2 },
    { 0x1d4f91c0, SUC_NE_lexicon_tags_2 },
    { 0x89f791c0, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759191c7, SUC_NE_lexicon_tags_2 },
    { 0x119391c8, SUC_NE_lexicon_tags_3 },
    { 0x522f91c9, SUC_NE_lexicon_tags_7 },
    { 0xe77f91c8, SUC_NE_lexicon_tags_7 },
    { 0x7ffb91cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311f91cf, SUC_NE_lexicon_tags_2 },
    { 0xa5ad91cf, SUC_NE_lexicon_tags_2 },
    { 0x395f91d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb33991d3, SUC_NE_lexicon_tags_2 },
    { 0xf85391d4, SUC_NE_lexicon_tags_2 },
    { 0x6ac591d5, SUC_NE_lexicon_tags_2 },
    { 0xbf3391d6, SUC_NE_lexicon_tags_2 },
    { 0x894791d7, SUC_NE_lexicon_tags_2 },
    { 0xd75d91d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eed91db, SUC_NE_lexicon_tags_2 },
    { 0x926991db, SUC_NE_lexicon_tags_2 },
    { 0x379391dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae191e2, SUC_NE_lexicon_tags_2 },
    { 0xfabb91e3, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63991e9, SUC_NE_lexicon_tags_40 },
    { 0x832591ea, SUC_NE_lexicon_tags_2 },
    { 0x79191ea, SUC_NE_lexicon_tags_2 },
    { 0x58a591ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad91f3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e7d91f5, SUC_NE_lexicon_tags_24 },
    { 0x36e791f5, SUC_NE_lexicon_tags_2 },
    { 0x17df91f7, SUC_NE_lexicon_tags_2 },
    { 0x41a391f8, SUC_NE_lexicon_tags_2 },
    { 0x4df991f9, SUC_NE_lexicon_tags_4 },
    { 0x392d91f8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0f91fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb39200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb59206, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe099209, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e59211, SUC_NE_lexicon_tags_2 },
    { 0x7c959211, SUC_NE_lexicon_tags_2 },
    { 0x7b879213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee999216, SUC_NE_lexicon_tags_16 },
    { 0xf219217, SUC_NE_lexicon_tags_2 },
    { 0x52bf9216, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ad921f, SUC_NE_lexicon_tags_2 },
    { 0x753d921f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x823d9222, SUC_NE_lexicon_tags_2 },
    { 0x60739222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f859226, SUC_NE_lexicon_tags_2 },
    { 0x9ec99227, SUC_NE_lexicon_tags_2 },
    { 0x99959228, SUC_NE_lexicon_tags_2 },
    { 0xbc599228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53999231, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ab9235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5b923c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b9240, SUC_NE_lexicon_tags_2 },
    { 0x1e7d9241, SUC_NE_lexicon_tags_19 },
    { 0x6c159242, SUC_NE_lexicon_tags_2 },
    { 0xf3b9242, SUC_NE_lexicon_tags_19 },
    { 0x94979244, SUC_NE_lexicon_tags_2 },
    { 0x835b9242, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf217924a, SUC_NE_lexicon_tags_2 },
    { 0xecdb924b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67924e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2579252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518b9256, SUC_NE_lexicon_tags_1 },
    { 0x50099256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb627925e, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86d9262, SUC_NE_lexicon_tags_2 },
    { 0x8ef19262, SUC_NE_lexicon_tags_2 },
    { 0x34039263, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b6f9266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239b926a, SUC_NE_lexicon_tags_2 },
    { 0x1ff926b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x280d926e, SUC_NE_lexicon_tags_2 },
    { 0xea33926f, SUC_NE_lexicon_tags_2 },
    { 0x7a5926f, SUC_NE_lexicon_tags_2 },
    { 0x9d1b9271, SUC_NE_lexicon_tags_2 },
    { 0xf8d59272, SUC_NE_lexicon_tags_2 },
    { 0x984f9271, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a39277, SUC_NE_lexicon_tags_5 },
    { 0xead39278, SUC_NE_lexicon_tags_2 },
    { 0x5e5f9279, SUC_NE_lexicon_tags_2 },
    { 0x31539279, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb927f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba799283, SUC_NE_lexicon_tags_2 },
    { 0xb4b19283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebef9286, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a19289, SUC_NE_lexicon_tags_2 },
    { 0x2b8b9289, SUC_NE_lexicon_tags_2 },
    { 0x5213928b, SUC_NE_lexicon_tags_2 },
    { 0x20039289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bbd928e, SUC_NE_lexicon_tags_2 },
    { 0x749b928f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddeb9294, SUC_NE_lexicon_tags_2 },
    { 0x97e79294, SUC_NE_lexicon_tags_2 },
    { 0x275b9294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d9299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617f92a2, SUC_NE_lexicon_tags_2 },
    { 0xcd392a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d592ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9f92b1, SUC_NE_lexicon_tags_2 },
    { 0x4acb92b2, SUC_NE_lexicon_tags_2 },
    { 0x55d992b3, SUC_NE_lexicon_tags_2 },
    { 0xc03992b4, SUC_NE_lexicon_tags_11 },
    { 0x66f392b4, SUC_NE_lexicon_tags_4 },
    { 0x54a392b3, SUC_NE_lexicon_tags_7 },
    { 0x4bc192b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8db92c4, SUC_NE_lexicon_tags_2 },
    { 0x3cdd92c5, SUC_NE_lexicon_tags_431 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216792c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x774392cb, SUC_NE_lexicon_tags_2 },
    { 0x35dd92cc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e5b92ce, SUC_NE_lexicon_tags_2 },
    { 0xc25792cf, SUC_NE_lexicon_tags_2 },
    { 0x758992cf, SUC_NE_lexicon_tags_2 },
    { 0x19b392d0, SUC_NE_lexicon_tags_3 },
    { 0x129192d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8abb92d4, SUC_NE_lexicon_tags_2 },
    { 0xa63792d4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469392e0, SUC_NE_lexicon_tags_2 },
    { 0x237192e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e392e4, SUC_NE_lexicon_tags_2 },
    { 0xb69792e5, SUC_NE_lexicon_tags_2 },
    { 0x9dc792e6, SUC_NE_lexicon_tags_2 },
    { 0x827792e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44592eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b2192ed, SUC_NE_lexicon_tags_2 },
    { 0xa83192ee, SUC_NE_lexicon_tags_2 },
    { 0xdd2392ef, SUC_NE_lexicon_tags_2 },
    { 0x460b92ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x874192f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356d92f7, SUC_NE_lexicon_tags_3 },
    { 0xc21d92f7, SUC_NE_lexicon_tags_10 },
    { 0xcaf392f9, SUC_NE_lexicon_tags_3 },
    { 0xf93f92f9, SUC_NE_lexicon_tags_2 },
    { 0x80fd92f8, SUC_NE_lexicon_tags_4 },
    { 0xbd9992fa, SUC_NE_lexicon_tags_2 },
    { 0x337d92fd, SUC_NE_lexicon_tags_2 },
    { 0xfa9792fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6419302, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba99306, SUC_NE_lexicon_tags_2 },
    { 0x69539307, SUC_NE_lexicon_tags_2 },
    { 0x66d9307, SUC_NE_lexicon_tags_2 },
    { 0xd3d79309, SUC_NE_lexicon_tags_2 },
    { 0x1569930a, SUC_NE_lexicon_tags_4 },
    { 0x804b9309, SUC_NE_lexicon_tags_2 },
    { 0x6a13930c, SUC_NE_lexicon_tags_4 },
    { 0xcc8f930d, SUC_NE_lexicon_tags_2 },
    { 0x6625930e, SUC_NE_lexicon_tags_5 },
    { 0x48f79307, SUC_NE_lexicon_tags_2 },
    { 0x2ba19310, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbaf9312, SUC_NE_lexicon_tags_2 },
    { 0xaaff9312, SUC_NE_lexicon_tags_2 },
    { 0xa0539314, SUC_NE_lexicon_tags_2 },
    { 0xeeb19315, SUC_NE_lexicon_tags_2 },
    { 0x49f19316, SUC_NE_lexicon_tags_2 },
    { 0xf019313, SUC_NE_lexicon_tags_2 },
    { 0x99559318, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x4335931a, SUC_NE_lexicon_tags_2 },
    { 0xb1ff931a, SUC_NE_lexicon_tags_2 },
    { 0xe1931c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacb1931e, SUC_NE_lexicon_tags_2 },
    { 0x45eb931f, SUC_NE_lexicon_tags_2 },
    { 0xe7ff931f, SUC_NE_lexicon_tags_3 },
    { 0xcfd5931e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9ad9323, SUC_NE_lexicon_tags_2 },
    { 0xd2799324, SUC_NE_lexicon_tags_2 },
    { 0xd6019325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617932d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16d9330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711b9335, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8f933b, SUC_NE_lexicon_tags_2 },
    { 0x5a51933c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fb933e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a319347, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d9934a, SUC_NE_lexicon_tags_34 },
    { 0x40c5934b, SUC_NE_lexicon_tags_4 },
    { 0x54b5934c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4273934f, SUC_NE_lexicon_tags_2 },
    { 0x31d3934f, SUC_NE_lexicon_tags_2 },
    { 0x62719351, SUC_NE_lexicon_tags_2 },
    { 0xfbf9352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cf9357, SUC_NE_lexicon_tags_2 },
    { 0xe46f9357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6b5935a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42f7935c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f79935f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43199361, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a459367, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb37936b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc251936d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xb3af936f, SUC_NE_lexicon_tags_2 },
    { 0x9535936f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0e39372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3999376, SUC_NE_lexicon_tags_2 },
    { 0x677b9377, SUC_NE_lexicon_tags_10 },
    { 0x34c39378, SUC_NE_lexicon_tags_2 },
    { 0x7f399379, SUC_NE_lexicon_tags_2 },
    { 0xb391937a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e7937f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xad799381, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11359384, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f999387, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xd5479389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd938c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x230f938e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd10b9390, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf59393, SUC_NE_lexicon_tags_2 },
    { 0xbdd79393, SUC_NE_lexicon_tags_2 },
    { 0xdf3d9395, SUC_NE_lexicon_tags_2 },
    { 0xcd7f9396, SUC_NE_lexicon_tags_2 },
    { 0x1c119397, SUC_NE_lexicon_tags_2 },
    { 0xb52b9394, SUC_NE_lexicon_tags_2 },
    { 0x48419399, SUC_NE_lexicon_tags_3 },
    { 0x1129939a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78f939f, SUC_NE_lexicon_tags_7 },
    { 0x9e8193a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827f93a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8d193a6, SUC_NE_lexicon_tags_3 },
    { 0xcd4b93a7, SUC_NE_lexicon_tags_3 },
    { 0x986993a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45193ac, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d793b0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d9393b2, SUC_NE_lexicon_tags_2 },
    { 0x154993b3, SUC_NE_lexicon_tags_7 },
    { 0x12db93b4, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60993b7, SUC_NE_lexicon_tags_2 },
    { 0x40db93b8, SUC_NE_lexicon_tags_2 },
    { 0xb0dd93b9, SUC_NE_lexicon_tags_133 },
    { 0xf51193b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aeb93bd, SUC_NE_lexicon_tags_2 },
    { 0x685593be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4f93c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc07393c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5ad93c5, SUC_NE_lexicon_tags_17 },
    { 0x4ddb93c6, SUC_NE_lexicon_tags_19 },
    { 0xce3193c6, SUC_NE_lexicon_tags_2 },
    { 0x558393c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f793cb, SUC_NE_lexicon_tags_2 },
    { 0x5d5793cc, SUC_NE_lexicon_tags_121 },
    { 0xf6cb93cd, SUC_NE_lexicon_tags_2 },
    { 0x1add93ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952b93d5, SUC_NE_lexicon_tags_19 },
    { 0x7a193d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x92e193d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe00b93da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16d93de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16393e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x516b93e3, SUC_NE_lexicon_tags_2 },
    { 0x463393e3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596993e8, SUC_NE_lexicon_tags_4 },
    { 0xac193e9, SUC_NE_lexicon_tags_2 },
    { 0x710d93e8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x95bf93ec, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4593ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3393f4, SUC_NE_lexicon_tags_17 },
    { 0xa81793f5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ae593f7, SUC_NE_lexicon_tags_3 },
    { 0x899793f8, SUC_NE_lexicon_tags_2 },
    { 0x7c993f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15f393fb, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfea793fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d239400, SUC_NE_lexicon_tags_2 },
    { 0xcb1d9401, SUC_NE_lexicon_tags_2 },
    { 0x292f9402, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b79406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb940c, SUC_NE_lexicon_tags_2 },
    { 0x837940d, SUC_NE_lexicon_tags_2 },
    { 0xafc7940e, SUC_NE_lexicon_tags_2 },
    { 0x2327940c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed459412, SUC_NE_lexicon_tags_5 },
    { 0xa4959413, SUC_NE_lexicon_tags_2 },
    { 0xbc099412, SUC_NE_lexicon_tags_2 },
    { 0x796f9415, SUC_NE_lexicon_tags_2 },
    { 0x673f9414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76519419, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f77941b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f941f, SUC_NE_lexicon_tags_2 },
    { 0xf8cb941f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d479423, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0xcce99425, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x51419428, SUC_NE_lexicon_tags_3 },
    { 0xe32f9429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f942f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e019437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x98919439, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd909943e, SUC_NE_lexicon_tags_2 },
    { 0x6b5f943f, SUC_NE_lexicon_tags_2 },
    { 0x439943f, SUC_NE_lexicon_tags_2 },
    { 0x148d9441, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b9445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79b9448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb715944d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x694f944f, SUC_NE_lexicon_tags_2 },
    { 0xf97f9450, SUC_NE_lexicon_tags_2 },
    { 0x3f059451, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd879455, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18d9458, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x459945a, SUC_NE_lexicon_tags_2 },
    { 0x8ee5945b, SUC_NE_lexicon_tags_2 },
    { 0xdb99945c, SUC_NE_lexicon_tags_2 },
    { 0x3d7d945d, SUC_NE_lexicon_tags_2 },
    { 0xb019945c, SUC_NE_lexicon_tags_2 },
    { 0xfb6b945f, SUC_NE_lexicon_tags_24 },
    { 0xed619460, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c19463, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb946c, SUC_NE_lexicon_tags_2 },
    { 0x4a07946c, SUC_NE_lexicon_tags_2 },
    { 0x65fb946e, SUC_NE_lexicon_tags_2 },
    { 0x7f53946f, SUC_NE_lexicon_tags_2 },
    { 0xb2139470, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea99472, SUC_NE_lexicon_tags_2 },
    { 0x4ce99473, SUC_NE_lexicon_tags_2 },
    { 0xf1599473, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473d947d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d099483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62c99485, SUC_NE_lexicon_tags_2 },
    { 0xa51d9485, SUC_NE_lexicon_tags_2 },
    { 0x59859485, SUC_NE_lexicon_tags_2 },
    { 0x7ecb9488, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd3948b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f948f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf65b9491, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26799496, SUC_NE_lexicon_tags_2 },
    { 0x6b59497, SUC_NE_lexicon_tags_2 },
    { 0xe3079497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d7949b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1394a0, SUC_NE_lexicon_tags_2 },
    { 0xa99f94a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc394ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9c794ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf1d94af, SUC_NE_lexicon_tags_2 },
    { 0xffcb94b0, SUC_NE_lexicon_tags_2 },
    { 0x38ab94b0, SUC_NE_lexicon_tags_2 },
    { 0x82d94b0, SUC_NE_lexicon_tags_2 },
    { 0x83cf94b0, SUC_NE_lexicon_tags_12 },
    { 0xf20794b1, SUC_NE_lexicon_tags_2 },
    { 0x8b6d94b4, SUC_NE_lexicon_tags_3 },
    { 0x8db794b4, SUC_NE_lexicon_tags_2 },
    { 0x813d94b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbe194b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78b394bb, SUC_NE_lexicon_tags_2 },
    { 0x962994bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57ab94be, SUC_NE_lexicon_tags_2 },
    { 0xd9ef94be, SUC_NE_lexicon_tags_2 },
    { 0x80f994c0, SUC_NE_lexicon_tags_2 },
    { 0x153594bf, SUC_NE_lexicon_tags_2 },
    { 0xf61594bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcdd94c4, SUC_NE_lexicon_tags_2 },
    { 0xa6af94c5, SUC_NE_lexicon_tags_24 },
    { 0x3aa194c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82df94c8, SUC_NE_lexicon_tags_2 },
    { 0x9e7994c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xced994cb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9cf94cd, SUC_NE_lexicon_tags_2 },
    { 0x10ed94ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd794d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c394d5, SUC_NE_lexicon_tags_1 },
    { 0x37f194d6, SUC_NE_lexicon_tags_2 },
    { 0x880b94d7, SUC_NE_lexicon_tags_2 },
    { 0x518394d5, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd994da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b594dd, SUC_NE_lexicon_tags_2 },
    { 0x8bd394dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86f594e0, SUC_NE_lexicon_tags_2 },
    { 0xd50394e1, SUC_NE_lexicon_tags_2 },
    { 0x5ad94e1, SUC_NE_lexicon_tags_2 },
    { 0x1b0f94e3, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x52f394e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425f94e9, SUC_NE_lexicon_tags_2 },
    { 0x77b194ea, SUC_NE_lexicon_tags_98 },
    { 0x294d94eb, SUC_NE_lexicon_tags_2 },
    { 0x673994ec, SUC_NE_lexicon_tags_2 },
    { 0xf6af94eb, SUC_NE_lexicon_tags_5 },
    { 0x37af94e9, SUC_NE_lexicon_tags_327 },
    { 0xd6bf94ee, SUC_NE_lexicon_tags_2 },
    { 0x3b994f0, SUC_NE_lexicon_tags_2 },
    { 0x9e0d94f1, SUC_NE_lexicon_tags_2 },
    { 0x6bc594f1, SUC_NE_lexicon_tags_4 },
    { 0x920b94f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6ad94f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd06394f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdcb94f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d594fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d4b9500, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0979506, SUC_NE_lexicon_tags_2 },
    { 0x10919507, SUC_NE_lexicon_tags_2 },
    { 0xca859508, SUC_NE_lexicon_tags_2 },
    { 0xb0cb9509, SUC_NE_lexicon_tags_2 },
    { 0xb65b950a, SUC_NE_lexicon_tags_2 },
    { 0xb5e1950b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9f9950d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65b9512, SUC_NE_lexicon_tags_28 },
    { 0xb6639513, SUC_NE_lexicon_tags_2 },
    { 0xaa4d9514, SUC_NE_lexicon_tags_2 },
    { 0x6e1d9515, SUC_NE_lexicon_tags_2 },
    { 0xe54d9515, SUC_NE_lexicon_tags_2 },
    { 0x95239517, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd15f9519, SUC_NE_lexicon_tags_2 },
    { 0x8d69951a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eb951d, SUC_NE_lexicon_tags_2 },
    { 0x5e45951d, SUC_NE_lexicon_tags_17 },
    { 0x7093951f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4659521, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a2f9523, SUC_NE_lexicon_tags_2 },
    { 0x15099524, SUC_NE_lexicon_tags_2 },
    { 0xd1479524, SUC_NE_lexicon_tags_28 },
    { 0xe4979524, SUC_NE_lexicon_tags_2 },
    { 0xfd2d9523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b952b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a19530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf459532, SUC_NE_lexicon_tags_2 },
    { 0x7a8f9533, SUC_NE_lexicon_tags_7 },
    { 0x41b9534, SUC_NE_lexicon_tags_2 },
    { 0x3e799534, SUC_NE_lexicon_tags_19 },
    { 0x6a119533, SUC_NE_lexicon_tags_2 },
    { 0xd7f79537, SUC_NE_lexicon_tags_2 },
    { 0x922d9537, SUC_NE_lexicon_tags_2 },
    { 0x43119539, SUC_NE_lexicon_tags_2 },
    { 0x35dd9539, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7b953c, SUC_NE_lexicon_tags_2 },
    { 0x4041953d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5b5953f, SUC_NE_lexicon_tags_2 },
    { 0x71ab9540, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd18b9542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d954a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5287954d, SUC_NE_lexicon_tags_2 },
    { 0xcd0b954e, SUC_NE_lexicon_tags_2 },
    { 0x7b87954e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81d79551, SUC_NE_lexicon_tags_2 },
    { 0xc9f59552, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47d99554, SUC_NE_lexicon_tags_1 },
    { 0x4de99555, SUC_NE_lexicon_tags_2 },
    { 0x57139554, SUC_NE_lexicon_tags_2 },
    { 0x6d919557, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e879559, SUC_NE_lexicon_tags_2 },
    { 0x36e7955a, SUC_NE_lexicon_tags_2 },
    { 0x5723955a, SUC_NE_lexicon_tags_2 },
    { 0x9591955c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc55955e, SUC_NE_lexicon_tags_2 },
    { 0x75e7955e, SUC_NE_lexicon_tags_2 },
    { 0xb959955e, SUC_NE_lexicon_tags_19 },
    { 0x3235955f, SUC_NE_lexicon_tags_2 },
    { 0xab81955e, SUC_NE_lexicon_tags_18 },
    { 0x7f7d955e, SUC_NE_lexicon_tags_2 },
    { 0x59c19561, SUC_NE_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c9d9566, SUC_NE_lexicon_tags_236 },
    { 0xd5a79567, SUC_NE_lexicon_tags_2 },
    { 0x7f9b9568, SUC_NE_lexicon_tags_2 },
    { 0x80bf9569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb5956e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x889d9570, SUC_NE_lexicon_tags_40 },
    { 0x2c659570, SUC_NE_lexicon_tags_2 },
    { 0xda8f9570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3d9576, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9899579, SUC_NE_lexicon_tags_0 },
    { 0xb3df9579, SUC_NE_lexicon_tags_2 },
    { 0xec93957b, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x3331957e, SUC_NE_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccf9584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c7f9586, SUC_NE_lexicon_tags_2 },
    { 0xd6239587, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f9589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadb958f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fcb9591, SUC_NE_lexicon_tags_5 },
    { 0xab279591, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e99595, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb44b9597, SUC_NE_lexicon_tags_2 },
    { 0x1ab19598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5b7959a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1971959d, SUC_NE_lexicon_tags_30 },
    { 0xb855959d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ff95a9, SUC_NE_lexicon_tags_2 },
    { 0x27a995aa, SUC_NE_lexicon_tags_2 },
    { 0x354d95ab, SUC_NE_lexicon_tags_3 },
    { 0x6b6395ac, SUC_NE_lexicon_tags_2 },
    { 0x2bc195ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb12b95af, SUC_NE_lexicon_tags_2 },
    { 0x840995b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf23195b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33395b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b595b8, SUC_NE_lexicon_tags_2 },
    { 0xb6a795b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4baf95bc, SUC_NE_lexicon_tags_2 },
    { 0x1a5d95bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1db95bf, SUC_NE_lexicon_tags_2 },
    { 0xdca195c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x274b95c3, SUC_NE_lexicon_tags_2 },
    { 0xc92595c4, SUC_NE_lexicon_tags_2 },
    { 0xa16d95c5, SUC_NE_lexicon_tags_2 },
    { 0x83b795c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2395cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x143f95ce, SUC_NE_lexicon_tags_3 },
    { 0x3fcf95cf, SUC_NE_lexicon_tags_2 },
    { 0x633795cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75dd95d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7595d5, SUC_NE_lexicon_tags_2 },
    { 0x691995d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6f95db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a195df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x636995e1, SUC_NE_lexicon_tags_2 },
    { 0xe54195e2, SUC_NE_lexicon_tags_30 },
    { 0x11df95e3, SUC_NE_lexicon_tags_2 },
    { 0xb55195e1, SUC_NE_lexicon_tags_2 },
    { 0x3edd95e5, SUC_NE_lexicon_tags_2 },
    { 0xcc3595e6, SUC_NE_lexicon_tags_2 },
    { 0x40795e7, SUC_NE_lexicon_tags_2 },
    { 0x87195e8, SUC_NE_lexicon_tags_2 },
    { 0x28ad95e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x468595eb, SUC_NE_lexicon_tags_2 },
    { 0x648995ec, SUC_NE_lexicon_tags_2 },
    { 0x3b0d95ed, SUC_NE_lexicon_tags_2 },
    { 0xa7b595ee, SUC_NE_lexicon_tags_2 },
    { 0x8e9795ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9195f4, SUC_NE_lexicon_tags_18 },
    { 0xe7c195f4, SUC_NE_lexicon_tags_164 },
    { 0, NULL },
    { 0xda4d95f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63079601, SUC_NE_lexicon_tags_2 },
    { 0x2dfd9602, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa99606, SUC_NE_lexicon_tags_34 },
    { 0xc21b9607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84c59609, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67559610, SUC_NE_lexicon_tags_2 },
    { 0x883f9611, SUC_NE_lexicon_tags_2 },
    { 0x845f9610, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x35f19614, SUC_NE_lexicon_tags_2 },
    { 0xe5e39614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2579617, SUC_NE_lexicon_tags_373 },
    { 0, NULL },
    { 0x27e59619, SUC_NE_lexicon_tags_2 },
    { 0xa6179619, SUC_NE_lexicon_tags_2 },
    { 0xef0d961b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f71961d, SUC_NE_lexicon_tags_2 },
    { 0xa613961e, SUC_NE_lexicon_tags_2 },
    { 0x82b7961d, SUC_NE_lexicon_tags_2 },
    { 0x494f961d, SUC_NE_lexicon_tags_30 },
    { 0x8bf9621, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x961d9623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1499626, SUC_NE_lexicon_tags_2 },
    { 0x18a59627, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3603962a, SUC_NE_lexicon_tags_2 },
    { 0x2b8b962b, SUC_NE_lexicon_tags_3 },
    { 0x20b5962a, SUC_NE_lexicon_tags_2 },
    { 0xbcfb962d, SUC_NE_lexicon_tags_2 },
    { 0x8659962e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550d9632, SUC_NE_lexicon_tags_2 },
    { 0xaad9632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18e39635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7639637, SUC_NE_lexicon_tags_2 },
    { 0xcf979638, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xa71f963a, SUC_NE_lexicon_tags_2 },
    { 0x54f3963b, SUC_NE_lexicon_tags_3 },
    { 0x2ec7963c, SUC_NE_lexicon_tags_2 },
    { 0x357b963d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1769963f, SUC_NE_lexicon_tags_2 },
    { 0xc2839640, SUC_NE_lexicon_tags_2 },
    { 0x70f1963f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x383d9644, SUC_NE_lexicon_tags_11 },
    { 0xb2b9645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578f964d, SUC_NE_lexicon_tags_2 },
    { 0x9fbd964e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82439650, SUC_NE_lexicon_tags_2 },
    { 0x7b6b9650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf22d9653, SUC_NE_lexicon_tags_2 },
    { 0x44e79654, SUC_NE_lexicon_tags_7 },
    { 0x6c9b9654, SUC_NE_lexicon_tags_2 },
    { 0xb8879653, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc947965a, SUC_NE_lexicon_tags_2 },
    { 0xe5d965a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59979660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2f9663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41739668, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c5966b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888d966f, SUC_NE_lexicon_tags_2 },
    { 0x1579670, SUC_NE_lexicon_tags_2 },
    { 0x72b99670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90219674, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x515b9677, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x52779679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53967d, SUC_NE_lexicon_tags_3 },
    { 0xc725967e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bf19680, SUC_NE_lexicon_tags_2 },
    { 0x6d8b9681, SUC_NE_lexicon_tags_2 },
    { 0x3edb9681, SUC_NE_lexicon_tags_2 },
    { 0xa7739683, SUC_NE_lexicon_tags_2 },
    { 0xce619683, SUC_NE_lexicon_tags_2 },
    { 0x91ef9683, SUC_NE_lexicon_tags_2 },
    { 0x46f59683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x131d9689, SUC_NE_lexicon_tags_2 },
    { 0xe1ad968a, SUC_NE_lexicon_tags_2 },
    { 0x4671968a, SUC_NE_lexicon_tags_2 },
    { 0xfd3f968c, SUC_NE_lexicon_tags_2 },
    { 0x1f9b968d, SUC_NE_lexicon_tags_2 },
    { 0x9457968e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a9969b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9521969d, SUC_NE_lexicon_tags_2 },
    { 0x37f1969d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d396a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe196a5, SUC_NE_lexicon_tags_2 },
    { 0xe26796a6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdde596a8, SUC_NE_lexicon_tags_13 },
    { 0x27e196a8, SUC_NE_lexicon_tags_2 },
    { 0x33e196aa, SUC_NE_lexicon_tags_2 },
    { 0x381d96ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x380796ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed796b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e996b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fef96bb, SUC_NE_lexicon_tags_2 },
    { 0xb8d396bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x918b96bf, SUC_NE_lexicon_tags_2 },
    { 0xc6b396bf, SUC_NE_lexicon_tags_2 },
    { 0x343596bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba6196c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x837196c5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b396c9, SUC_NE_lexicon_tags_2 },
    { 0x69d196c9, SUC_NE_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115196ce, SUC_NE_lexicon_tags_2 },
    { 0x4e1b96ce, SUC_NE_lexicon_tags_2 },
    { 0xc61596cf, SUC_NE_lexicon_tags_2 },
    { 0xf66596cf, SUC_NE_lexicon_tags_2 },
    { 0xb8b596d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e396d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x302f96d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaf96dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135996e3, SUC_NE_lexicon_tags_2 },
    { 0x24e796e4, SUC_NE_lexicon_tags_2 },
    { 0xd83396e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x72ad96e7, SUC_NE_lexicon_tags_2 },
    { 0xb1f996e8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa23d96ea, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10796ef, SUC_NE_lexicon_tags_2 },
    { 0x6c3996f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad9596f2, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2996f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x992b96f9, SUC_NE_lexicon_tags_2 },
    { 0xb6cb96f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32996fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9d9700, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94239704, SUC_NE_lexicon_tags_2 },
    { 0xcd419705, SUC_NE_lexicon_tags_2 },
    { 0xb5df9706, SUC_NE_lexicon_tags_2 },
    { 0x90059707, SUC_NE_lexicon_tags_2 },
    { 0xc4439707, SUC_NE_lexicon_tags_2 },
    { 0x53db9709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa249970f, SUC_NE_lexicon_tags_2 },
    { 0xde979710, SUC_NE_lexicon_tags_2 },
    { 0xe9db9710, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81199716, SUC_NE_lexicon_tags_18 },
    { 0xfe0b9716, SUC_NE_lexicon_tags_2 },
    { 0x50459716, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9971c, SUC_NE_lexicon_tags_2 },
    { 0x31fb971d, SUC_NE_lexicon_tags_2 },
    { 0x647d971e, SUC_NE_lexicon_tags_2 },
    { 0x6aef971f, SUC_NE_lexicon_tags_2 },
    { 0xf64b971e, SUC_NE_lexicon_tags_2 },
    { 0x5f1d9721, SUC_NE_lexicon_tags_2 },
    { 0x197b9722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cfb9724, SUC_NE_lexicon_tags_2 },
    { 0xd2c59724, SUC_NE_lexicon_tags_2 },
    { 0xbd059726, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ad19728, SUC_NE_lexicon_tags_2 },
    { 0xc7219729, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xed15972b, SUC_NE_lexicon_tags_2 },
    { 0x413d972c, SUC_NE_lexicon_tags_2 },
    { 0x485972d, SUC_NE_lexicon_tags_2 },
    { 0x1361972e, SUC_NE_lexicon_tags_2 },
    { 0x910f972f, SUC_NE_lexicon_tags_2 },
    { 0x7d2f972f, SUC_NE_lexicon_tags_2 },
    { 0x5f2b9730, SUC_NE_lexicon_tags_2 },
    { 0x47c99732, SUC_NE_lexicon_tags_2 },
    { 0x36b99733, SUC_NE_lexicon_tags_2 },
    { 0x828b9734, SUC_NE_lexicon_tags_2 },
    { 0x18dd9734, SUC_NE_lexicon_tags_2 },
    { 0x143b9736, SUC_NE_lexicon_tags_2 },
    { 0xfe959737, SUC_NE_lexicon_tags_2 },
    { 0x5dfd9738, SUC_NE_lexicon_tags_0 },
    { 0xcad79733, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b5973e, SUC_NE_lexicon_tags_4 },
    { 0x14b9973e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x391b9741, SUC_NE_lexicon_tags_2 },
    { 0x2afd9741, SUC_NE_lexicon_tags_5 },
    { 0x65919743, SUC_NE_lexicon_tags_2 },
    { 0x12059744, SUC_NE_lexicon_tags_2 },
    { 0x3d079745, SUC_NE_lexicon_tags_2 },
    { 0xe0eb9744, SUC_NE_lexicon_tags_2 },
    { 0xfd9746, SUC_NE_lexicon_tags_2 },
    { 0x2f7d9748, SUC_NE_lexicon_tags_3 },
    { 0x55799749, SUC_NE_lexicon_tags_3 },
    { 0x6ab99745, SUC_NE_lexicon_tags_14 },
    { 0x1ef7974b, SUC_NE_lexicon_tags_2 },
    { 0x8ecf9747, SUC_NE_lexicon_tags_1 },
    { 0x4a1b9747, SUC_NE_lexicon_tags_2 },
    { 0x905f9748, SUC_NE_lexicon_tags_7 },
    { 0x19cb974f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68a99751, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307b9755, SUC_NE_lexicon_tags_3 },
    { 0x6b139755, SUC_NE_lexicon_tags_17 },
    { 0x6aff9757, SUC_NE_lexicon_tags_2 },
    { 0xd02f9756, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef87975b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x340b975e, SUC_NE_lexicon_tags_2 },
    { 0xaebd975f, SUC_NE_lexicon_tags_2 },
    { 0x9f71975f, SUC_NE_lexicon_tags_2 },
    { 0x6c379761, SUC_NE_lexicon_tags_7 },
    { 0xadf39762, SUC_NE_lexicon_tags_4 },
    { 0x611f9763, SUC_NE_lexicon_tags_2 },
    { 0x27619764, SUC_NE_lexicon_tags_3 },
    { 0x16599763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3d9769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ff976c, SUC_NE_lexicon_tags_2 },
    { 0xaadd976d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5699770, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f9772, SUC_NE_lexicon_tags_1 },
    { 0x83c39773, SUC_NE_lexicon_tags_2 },
    { 0x91c99774, SUC_NE_lexicon_tags_2 },
    { 0x67c19773, SUC_NE_lexicon_tags_2 },
    { 0x60119774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443b977a, SUC_NE_lexicon_tags_2 },
    { 0x8425977a, SUC_NE_lexicon_tags_2 },
    { 0x9bf9977c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae75977e, SUC_NE_lexicon_tags_2 },
    { 0x5a55977f, SUC_NE_lexicon_tags_2 },
    { 0xd305977f, SUC_NE_lexicon_tags_2 },
    { 0xc30d9781, SUC_NE_lexicon_tags_2 },
    { 0xc9919781, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x753d9784, SUC_NE_lexicon_tags_7 },
    { 0x83e59785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x245f9787, SUC_NE_lexicon_tags_2 },
    { 0x9f059788, SUC_NE_lexicon_tags_2 },
    { 0x29459789, SUC_NE_lexicon_tags_2 },
    { 0x3e3d978a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc71978e, SUC_NE_lexicon_tags_2 },
    { 0x9423978e, SUC_NE_lexicon_tags_2 },
    { 0x5b3d9790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce579792, SUC_NE_lexicon_tags_2 },
    { 0x894b9793, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849b979b, SUC_NE_lexicon_tags_2 },
    { 0x4bf979c, SUC_NE_lexicon_tags_2 },
    { 0x5f79979d, SUC_NE_lexicon_tags_0 },
    { 0xc49979b, SUC_NE_lexicon_tags_2 },
    { 0x8fe3979b, SUC_NE_lexicon_tags_2 },
    { 0xaaa797a0, SUC_NE_lexicon_tags_2 },
    { 0xc31597a1, SUC_NE_lexicon_tags_2 },
    { 0xa5797a2, SUC_NE_lexicon_tags_2 },
    { 0x60a397a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff797a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x187997ab, SUC_NE_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d197ae, SUC_NE_lexicon_tags_2 },
    { 0x3e7b97ae, SUC_NE_lexicon_tags_2 },
    { 0xa47b97ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadb97b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05397b9, SUC_NE_lexicon_tags_2 },
    { 0xfe7597ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436397be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x168597c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2af597c3, SUC_NE_lexicon_tags_7 },
    { 0xf49597c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ad797c6, SUC_NE_lexicon_tags_332 },
    { 0x771797c7, SUC_NE_lexicon_tags_2 },
    { 0xae2597c7, SUC_NE_lexicon_tags_2 },
    { 0xf33797c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2797d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x672f97d4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d7b97d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x924997d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91997e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x241f97e3, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f97e6, SUC_NE_lexicon_tags_2 },
    { 0x100b97e7, SUC_NE_lexicon_tags_2 },
    { 0x862997e8, SUC_NE_lexicon_tags_4 },
    { 0x229d97e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ed197eb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xce1f97ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf00997ef, SUC_NE_lexicon_tags_2 },
    { 0xb86197ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce6397f2, SUC_NE_lexicon_tags_2 },
    { 0x30cd97f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x815d97f5, SUC_NE_lexicon_tags_2 },
    { 0x5b6197f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9397f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85b97fd, SUC_NE_lexicon_tags_32 },
    { 0xaf6b97fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cdb9800, SUC_NE_lexicon_tags_2 },
    { 0x8d059801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bd9804, SUC_NE_lexicon_tags_2 },
    { 0x97cd9804, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x386b9807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9345980a, SUC_NE_lexicon_tags_2 },
    { 0xb009980b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf783980d, SUC_NE_lexicon_tags_0 },
    { 0x386b980d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70bd9810, SUC_NE_lexicon_tags_2 },
    { 0xf039811, SUC_NE_lexicon_tags_2 },
    { 0x9b839810, SUC_NE_lexicon_tags_2 },
    { 0x48259813, SUC_NE_lexicon_tags_2 },
    { 0x46a99814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50859817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cd981c, SUC_NE_lexicon_tags_2 },
    { 0xe665981c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66f9820, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45099829, SUC_NE_lexicon_tags_2 },
    { 0x13eb982a, SUC_NE_lexicon_tags_2 },
    { 0xda759829, SUC_NE_lexicon_tags_4 },
    { 0x19f3982b, SUC_NE_lexicon_tags_2 },
    { 0x24fb9829, SUC_NE_lexicon_tags_2 },
    { 0xbf79982e, SUC_NE_lexicon_tags_13 },
    { 0x9dbb9829, SUC_NE_lexicon_tags_102 },
    { 0xc99f9830, SUC_NE_lexicon_tags_2 },
    { 0xd2c39830, SUC_NE_lexicon_tags_2 },
    { 0x55e7982d, SUC_NE_lexicon_tags_2 },
    { 0xd2a99833, SUC_NE_lexicon_tags_34 },
    { 0xcfd99834, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6801983b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1983e, SUC_NE_lexicon_tags_4 },
    { 0x11ef983f, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x87859841, SUC_NE_lexicon_tags_2 },
    { 0xcd619842, SUC_NE_lexicon_tags_2 },
    { 0x9c799841, SUC_NE_lexicon_tags_2 },
    { 0xd479844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34df9846, SUC_NE_lexicon_tags_4 },
    { 0x5c5f9846, SUC_NE_lexicon_tags_2 },
    { 0x57779848, SUC_NE_lexicon_tags_2 },
    { 0xed99848, SUC_NE_lexicon_tags_2 },
    { 0xd0fd984a, SUC_NE_lexicon_tags_2 },
    { 0x53ef984b, SUC_NE_lexicon_tags_2 },
    { 0x25984c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8971984e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a039854, SUC_NE_lexicon_tags_138 },
    { 0, NULL },
    { 0x2ba59856, SUC_NE_lexicon_tags_102 },
    { 0xdeed9857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68539859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a03985f, SUC_NE_lexicon_tags_2 },
    { 0x33c39860, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a99864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d1986a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59cf986c, SUC_NE_lexicon_tags_2 },
    { 0xebe9986d, SUC_NE_lexicon_tags_2 },
    { 0x5215986e, SUC_NE_lexicon_tags_2 },
    { 0xa55986f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29a19871, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a719874, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d39876, SUC_NE_lexicon_tags_36 },
    { 0x53139877, SUC_NE_lexicon_tags_2 },
    { 0xb5fd9877, SUC_NE_lexicon_tags_2 },
    { 0x3b1b9879, SUC_NE_lexicon_tags_2 },
    { 0x8699987a, SUC_NE_lexicon_tags_2 },
    { 0x7b13987b, SUC_NE_lexicon_tags_4 },
    { 0x1c59987b, SUC_NE_lexicon_tags_2 },
    { 0x249d987b, SUC_NE_lexicon_tags_4 },
    { 0x810b987e, SUC_NE_lexicon_tags_2 },
    { 0xa19f987f, SUC_NE_lexicon_tags_2 },
    { 0x8bb987a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa599883, SUC_NE_lexicon_tags_2 },
    { 0xaf199884, SUC_NE_lexicon_tags_2 },
    { 0x9f899883, SUC_NE_lexicon_tags_2 },
    { 0x62619886, SUC_NE_lexicon_tags_2 },
    { 0xfb219887, SUC_NE_lexicon_tags_2 },
    { 0xd6e59888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94af988a, SUC_NE_lexicon_tags_2 },
    { 0x16d5988b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6607988d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5e5988f, SUC_NE_lexicon_tags_2 },
    { 0x87af988f, SUC_NE_lexicon_tags_2 },
    { 0x81b7988f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x31219894, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ffb9896, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f59899, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb989c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ae9989e, SUC_NE_lexicon_tags_2 },
    { 0xfa7f989f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa01d98a1, SUC_NE_lexicon_tags_2 },
    { 0x635398a2, SUC_NE_lexicon_tags_2 },
    { 0xb39398a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624998a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1d98af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ab998b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95998b6, SUC_NE_lexicon_tags_2 },
    { 0x912998b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199f98bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cd98bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386d98c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb398c9, SUC_NE_lexicon_tags_2 },
    { 0xadf598c9, SUC_NE_lexicon_tags_2 },
    { 0x2f8b98cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd2598cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa98f98cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87b98d2, SUC_NE_lexicon_tags_2 },
    { 0xcd998d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x969598d5, SUC_NE_lexicon_tags_2 },
    { 0xaa5398d6, SUC_NE_lexicon_tags_2 },
    { 0x355998d7, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0x59ab98d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeae798db, SUC_NE_lexicon_tags_4 },
    { 0x5a3d98db, SUC_NE_lexicon_tags_2 },
    { 0xc34198db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238798e1, SUC_NE_lexicon_tags_2 },
    { 0x59f98e2, SUC_NE_lexicon_tags_2 },
    { 0x9fd998e3, SUC_NE_lexicon_tags_7 },
    { 0x28a998e4, SUC_NE_lexicon_tags_2 },
    { 0x1c4d98e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736398ea, SUC_NE_lexicon_tags_2 },
    { 0x641198eb, SUC_NE_lexicon_tags_2 },
    { 0xb7df98eb, SUC_NE_lexicon_tags_2 },
    { 0x7ba198eb, SUC_NE_lexicon_tags_92 },
    { 0x990198ec, SUC_NE_lexicon_tags_2 },
    { 0xcf98ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e3198f1, SUC_NE_lexicon_tags_1 },
    { 0x15fb98f2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d98f5, SUC_NE_lexicon_tags_3 },
    { 0xac198f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x575998f8, SUC_NE_lexicon_tags_2 },
    { 0x9d3998f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9398fe, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x82ad9900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd679905, SUC_NE_lexicon_tags_2 },
    { 0x22199906, SUC_NE_lexicon_tags_2 },
    { 0xe80d9907, SUC_NE_lexicon_tags_2 },
    { 0xfdf79907, SUC_NE_lexicon_tags_2 },
    { 0x43bf9907, SUC_NE_lexicon_tags_2 },
    { 0xc1dd9908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1407990c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b3990f, SUC_NE_lexicon_tags_2 },
    { 0x6d659910, SUC_NE_lexicon_tags_2 },
    { 0x3a89990f, SUC_NE_lexicon_tags_2 },
    { 0x72819910, SUC_NE_lexicon_tags_2 },
    { 0xaec79913, SUC_NE_lexicon_tags_18 },
    { 0xd9d99914, SUC_NE_lexicon_tags_3 },
    { 0x55359914, SUC_NE_lexicon_tags_2 },
    { 0xbc8d9914, SUC_NE_lexicon_tags_2 },
    { 0xfedd9917, SUC_NE_lexicon_tags_7 },
    { 0x7c37990f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53991e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8f9924, SUC_NE_lexicon_tags_2 },
    { 0xd7919925, SUC_NE_lexicon_tags_2 },
    { 0xddab9926, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d659928, SUC_NE_lexicon_tags_2 },
    { 0x7db19929, SUC_NE_lexicon_tags_2 },
    { 0xa2ef992a, SUC_NE_lexicon_tags_2 },
    { 0xb2b9992b, SUC_NE_lexicon_tags_2 },
    { 0x489f992b, SUC_NE_lexicon_tags_2 },
    { 0x6e13992d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f99931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd97b9933, SUC_NE_lexicon_tags_2 },
    { 0x921d9933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf999936, SUC_NE_lexicon_tags_2 },
    { 0x31319937, SUC_NE_lexicon_tags_2 },
    { 0x5e019936, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bb7993a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e19943, SUC_NE_lexicon_tags_18 },
    { 0xd1db9944, SUC_NE_lexicon_tags_2 },
    { 0xcb8b9943, SUC_NE_lexicon_tags_2 },
    { 0xdf099946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8339949, SUC_NE_lexicon_tags_2 },
    { 0x69c9994a, SUC_NE_lexicon_tags_7 },
    { 0xa00b994b, SUC_NE_lexicon_tags_2 },
    { 0x8bf1994b, SUC_NE_lexicon_tags_2 },
    { 0x679994c, SUC_NE_lexicon_tags_2 },
    { 0x7155994a, SUC_NE_lexicon_tags_3 },
    { 0xb9dd994f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40f9955, SUC_NE_lexicon_tags_11 },
    { 0x83f79956, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0999958, SUC_NE_lexicon_tags_2 },
    { 0x29b9959, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5b9995b, SUC_NE_lexicon_tags_2 },
    { 0xda61995c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb755995e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19964, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74959966, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x55ef9968, SUC_NE_lexicon_tags_2 },
    { 0x97c39969, SUC_NE_lexicon_tags_58 },
    { 0xfc11996a, SUC_NE_lexicon_tags_2 },
    { 0xd7af9969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd9996f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee139971, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1899973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeef9976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e57997b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe773997d, SUC_NE_lexicon_tags_2 },
    { 0x89af997e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7899980, SUC_NE_lexicon_tags_4 },
    { 0xbe779981, SUC_NE_lexicon_tags_2 },
    { 0xd1639981, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4419986, SUC_NE_lexicon_tags_5 },
    { 0x1ec79986, SUC_NE_lexicon_tags_2 },
    { 0x1ffb9987, SUC_NE_lexicon_tags_2 },
    { 0xfdd9988, SUC_NE_lexicon_tags_2 },
    { 0x317998a, SUC_NE_lexicon_tags_3 },
    { 0x134d9988, SUC_NE_lexicon_tags_2 },
    { 0xece19986, SUC_NE_lexicon_tags_2 },
    { 0x8273998b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d19990, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4bf9992, SUC_NE_lexicon_tags_21 },
    { 0x74319993, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d39997, SUC_NE_lexicon_tags_2 },
    { 0x5e3d9998, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb8d1999a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf565999e, SUC_NE_lexicon_tags_2 },
    { 0xdc59999f, SUC_NE_lexicon_tags_2 },
    { 0x2b399a0, SUC_NE_lexicon_tags_2 },
    { 0xc28f99a1, SUC_NE_lexicon_tags_4 },
    { 0xea8b99a0, SUC_NE_lexicon_tags_4 },
    { 0x701999e, SUC_NE_lexicon_tags_3 },
    { 0xf72599a2, SUC_NE_lexicon_tags_2 },
    { 0xc07f99a5, SUC_NE_lexicon_tags_81 },
    { 0x727b99a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a199a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ed99ac, SUC_NE_lexicon_tags_2 },
    { 0xfb799ad, SUC_NE_lexicon_tags_2 },
    { 0x68e399ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffd99b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8f99b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdee399b8, SUC_NE_lexicon_tags_2 },
    { 0x7cb799b8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x6eef99bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x383f99be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1db99c0, SUC_NE_lexicon_tags_2 },
    { 0x72e799c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ce399c3, SUC_NE_lexicon_tags_2 },
    { 0x6b799c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1399c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf799cb, SUC_NE_lexicon_tags_2 },
    { 0xc63f99cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x162799ce, SUC_NE_lexicon_tags_2 },
    { 0x55a599cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaff99d3, SUC_NE_lexicon_tags_2 },
    { 0x876599d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2799d8, SUC_NE_lexicon_tags_2 },
    { 0x87f399d9, SUC_NE_lexicon_tags_5 },
    { 0x436d99d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x177399dc, SUC_NE_lexicon_tags_2 },
    { 0x845f99dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17b99e0, SUC_NE_lexicon_tags_2 },
    { 0xf58599e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ab99e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458f99f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d99f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x612b99f6, SUC_NE_lexicon_tags_2 },
    { 0x98bb99f7, SUC_NE_lexicon_tags_1 },
    { 0x3c9f99f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2b99fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3719a03, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xaae39a05, SUC_NE_lexicon_tags_2 },
    { 0xd1cf9a06, SUC_NE_lexicon_tags_1 },
    { 0x7c399a06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3079a0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b199a0f, SUC_NE_lexicon_tags_2 },
    { 0x63eb9a10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51499a16, SUC_NE_lexicon_tags_2 },
    { 0xdc5f9a17, SUC_NE_lexicon_tags_2 },
    { 0x67ed9a17, SUC_NE_lexicon_tags_3 },
    { 0xe7939a19, SUC_NE_lexicon_tags_17 },
    { 0xc3b9a18, SUC_NE_lexicon_tags_50 },
    { 0x93219a1b, SUC_NE_lexicon_tags_2 },
    { 0xc8939a18, SUC_NE_lexicon_tags_159 },
    { 0x78639a1d, SUC_NE_lexicon_tags_2 },
    { 0x2b3b9a1c, SUC_NE_lexicon_tags_2 },
    { 0xbc6d9a1f, SUC_NE_lexicon_tags_2 },
    { 0x77fb9a1b, SUC_NE_lexicon_tags_2 },
    { 0x4b0b9a21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95f9a24, SUC_NE_lexicon_tags_2 },
    { 0x2fed9a25, SUC_NE_lexicon_tags_40 },
    { 0x77439a26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bb9a31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76559a33, SUC_NE_lexicon_tags_2 },
    { 0xdc3b9a34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd9a38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21459a3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfd9a3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21959a41, SUC_NE_lexicon_tags_14 },
    { 0xab6b9a42, SUC_NE_lexicon_tags_2 },
    { 0x3dd59a43, SUC_NE_lexicon_tags_3 },
    { 0xa1219a41, SUC_NE_lexicon_tags_3 },
    { 0x5fbd9a44, SUC_NE_lexicon_tags_2 },
    { 0x16a99a41, SUC_NE_lexicon_tags_2 },
    { 0x75a59a41, SUC_NE_lexicon_tags_2 },
    { 0xb0819a48, SUC_NE_lexicon_tags_2 },
    { 0xe2919a49, SUC_NE_lexicon_tags_2 },
    { 0xe13d9a47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925d9a4e, SUC_NE_lexicon_tags_2 },
    { 0xc69d9a4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d9a52, SUC_NE_lexicon_tags_2 },
    { 0x1bd9a52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5ff9a55, SUC_NE_lexicon_tags_2 },
    { 0xf9cf9a56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0b9a5a, SUC_NE_lexicon_tags_2 },
    { 0x97979a5b, SUC_NE_lexicon_tags_2 },
    { 0x40fb9a5c, SUC_NE_lexicon_tags_2 },
    { 0x6e9f9a5d, SUC_NE_lexicon_tags_7 },
    { 0xb1099a5c, SUC_NE_lexicon_tags_2 },
    { 0xfc7b9a5d, SUC_NE_lexicon_tags_2 },
    { 0x72cb9a60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa99a64, SUC_NE_lexicon_tags_2 },
    { 0x1ab59a64, SUC_NE_lexicon_tags_3 },
    { 0x9439a66, SUC_NE_lexicon_tags_2 },
    { 0xedb19a66, SUC_NE_lexicon_tags_2 },
    { 0xd79a68, SUC_NE_lexicon_tags_2 },
    { 0xe0e99a68, SUC_NE_lexicon_tags_2 },
    { 0x5d959a6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb9a6e, SUC_NE_lexicon_tags_2 },
    { 0x5c7b9a6f, SUC_NE_lexicon_tags_2 },
    { 0xe4579a70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b1d9a72, SUC_NE_lexicon_tags_2 },
    { 0xf6af9a73, SUC_NE_lexicon_tags_2 },
    { 0xbb959a74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9f9a76, SUC_NE_lexicon_tags_2 },
    { 0x31af9a77, SUC_NE_lexicon_tags_2 },
    { 0xb819a77, SUC_NE_lexicon_tags_247 },
    { 0x4c8d9a79, SUC_NE_lexicon_tags_2 },
    { 0x20599a7a, SUC_NE_lexicon_tags_2 },
    { 0x2e839a77, SUC_NE_lexicon_tags_2 },
    { 0x7d759a78, SUC_NE_lexicon_tags_2 },
    { 0x92c59a7d, SUC_NE_lexicon_tags_4 },
    { 0xaacb9a77, SUC_NE_lexicon_tags_2 },
    { 0x6e239a77, SUC_NE_lexicon_tags_2 },
    { 0xc73f9a80, SUC_NE_lexicon_tags_2 },
    { 0xc46f9a81, SUC_NE_lexicon_tags_2 },
    { 0x1e219a81, SUC_NE_lexicon_tags_2 },
    { 0x7f479a83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1f9a86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20079a88, SUC_NE_lexicon_tags_2 },
    { 0xc7339a89, SUC_NE_lexicon_tags_2 },
    { 0xf5cb9a89, SUC_NE_lexicon_tags_2 },
    { 0xeb179a89, SUC_NE_lexicon_tags_2 },
    { 0x2b379a8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x57fd9a8e, SUC_NE_lexicon_tags_2 },
    { 0x58bb9a8e, SUC_NE_lexicon_tags_2 },
    { 0x8dd19a8f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6639a95, SUC_NE_lexicon_tags_2 },
    { 0xf6579a96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb39a9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcb9a9f, SUC_NE_lexicon_tags_2 },
    { 0xf3259aa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb9aa5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3139aa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71eb9aa9, SUC_NE_lexicon_tags_2 },
    { 0xd24b9aaa, SUC_NE_lexicon_tags_2 },
    { 0x4ffd9aab, SUC_NE_lexicon_tags_2 },
    { 0x90279aac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84799ab1, SUC_NE_lexicon_tags_40 },
    { 0xa3ad9ab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55579ab4, SUC_NE_lexicon_tags_2 },
    { 0x7b199ab4, SUC_NE_lexicon_tags_49 },
    { 0xea459ab6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f19abd, SUC_NE_lexicon_tags_2 },
    { 0x10d39abe, SUC_NE_lexicon_tags_2 },
    { 0x883d9abf, SUC_NE_lexicon_tags_2 },
    { 0x22699ac0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2179ac2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78cf9ac5, SUC_NE_lexicon_tags_2 },
    { 0x6b999ac6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9b9ac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba799ace, SUC_NE_lexicon_tags_2 },
    { 0xcab59acf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd20f9ad1, SUC_NE_lexicon_tags_2 },
    { 0xf59f9ad2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f339ad4, SUC_NE_lexicon_tags_2 },
    { 0x472d9ad4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a799ad7, SUC_NE_lexicon_tags_2 },
    { 0xc9919ad8, SUC_NE_lexicon_tags_2 },
    { 0x27c99ad9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae99adc, SUC_NE_lexicon_tags_2 },
    { 0x548b9adc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f59ae0, SUC_NE_lexicon_tags_2 },
    { 0xa1d59ae1, SUC_NE_lexicon_tags_49 },
    { 0xbc539ae2, SUC_NE_lexicon_tags_2 },
    { 0xf2059ae2, SUC_NE_lexicon_tags_2 },
    { 0xd0099ae3, SUC_NE_lexicon_tags_2 },
    { 0x1c0b9ae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21c39ae7, SUC_NE_lexicon_tags_2 },
    { 0xf3b59ae7, SUC_NE_lexicon_tags_34 },
    { 0x34039ae9, SUC_NE_lexicon_tags_34 },
    { 0x7f779aea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d79aed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556f9af1, SUC_NE_lexicon_tags_2 },
    { 0x97bf9af1, SUC_NE_lexicon_tags_2 },
    { 0x86839af3, SUC_NE_lexicon_tags_2 },
    { 0xcdd9af3, SUC_NE_lexicon_tags_18 },
    { 0xea839af5, SUC_NE_lexicon_tags_2 },
    { 0xcf019af1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62699af8, SUC_NE_lexicon_tags_2 },
    { 0xa19b9af9, SUC_NE_lexicon_tags_2 },
    { 0x48fd9af9, SUC_NE_lexicon_tags_303 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f19afd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0299b05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b19b0a, SUC_NE_lexicon_tags_2 },
    { 0xaad9b0a, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xd9d79b0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce259b0f, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416b9b16, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x372d9b19, SUC_NE_lexicon_tags_2 },
    { 0xd3b19b19, SUC_NE_lexicon_tags_2 },
    { 0x60a39b1b, SUC_NE_lexicon_tags_91 },
    { 0x103f9b19, SUC_NE_lexicon_tags_2 },
    { 0xffd59b1d, SUC_NE_lexicon_tags_7 },
    { 0x2b259b19, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x67df9b20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f9b23, SUC_NE_lexicon_tags_2 },
    { 0x7e99b24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83019b27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1559b2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf9b2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9b39, SUC_NE_lexicon_tags_2 },
    { 0xe74b9b3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dfd9b3c, SUC_NE_lexicon_tags_2 },
    { 0xd139b3d, SUC_NE_lexicon_tags_2 },
    { 0x6cf99b3d, SUC_NE_lexicon_tags_2 },
    { 0x38bf9b3d, SUC_NE_lexicon_tags_2 },
    { 0x58ef9b3e, SUC_NE_lexicon_tags_2 },
    { 0xb9619b41, SUC_NE_lexicon_tags_2 },
    { 0xb31f9b42, SUC_NE_lexicon_tags_2 },
    { 0xd4cb9b43, SUC_NE_lexicon_tags_8 },
    { 0xbac39b42, SUC_NE_lexicon_tags_2 },
    { 0xa7539b45, SUC_NE_lexicon_tags_2 },
    { 0xd07b9b46, SUC_NE_lexicon_tags_19 },
    { 0x760b9b40, SUC_NE_lexicon_tags_2 },
    { 0xc2b59b48, SUC_NE_lexicon_tags_2 },
    { 0xe1379b49, SUC_NE_lexicon_tags_2 },
    { 0x611d9b3d, SUC_NE_lexicon_tags_2 },
    { 0xe4139b3d, SUC_NE_lexicon_tags_2 },
    { 0xb46d9b44, SUC_NE_lexicon_tags_2 },
    { 0x797b9b4a, SUC_NE_lexicon_tags_3 },
    { 0xb5659b48, SUC_NE_lexicon_tags_1 },
    { 0x93cf9b4f, SUC_NE_lexicon_tags_2 },
    { 0x453b9b46, SUC_NE_lexicon_tags_2 },
    { 0x56919b50, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb9b54, SUC_NE_lexicon_tags_2 },
    { 0xc8339b54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc79b58, SUC_NE_lexicon_tags_2 },
    { 0xf21d9b59, SUC_NE_lexicon_tags_2 },
    { 0xf5059b58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40499b61, SUC_NE_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bf9b64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d839b69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d59b6c, SUC_NE_lexicon_tags_2 },
    { 0xaf4b9b6c, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a9d9b6f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406f9b75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49919b78, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x5deb9b7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51279b7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6459b84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d859b88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60099b8a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd9b8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58779b8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf68d9b91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a079b93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4199b98, SUC_NE_lexicon_tags_2 },
    { 0x59db9b98, SUC_NE_lexicon_tags_2 },
    { 0x4ff99b98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0619ba1, SUC_NE_lexicon_tags_1 },
    { 0x38b39ba2, SUC_NE_lexicon_tags_2 },
    { 0xb2b99ba2, SUC_NE_lexicon_tags_2 },
    { 0xf6e79ba4, SUC_NE_lexicon_tags_2 },
    { 0x42b19ba5, SUC_NE_lexicon_tags_2 },
    { 0x86b59ba5, SUC_NE_lexicon_tags_34 },
    { 0x848f9ba7, SUC_NE_lexicon_tags_2 },
    { 0xe98b9ba6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c59bad, SUC_NE_lexicon_tags_2 },
    { 0x9b259bad, SUC_NE_lexicon_tags_2 },
    { 0x968d9bad, SUC_NE_lexicon_tags_4 },
    { 0x2f859bae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869f9bb4, SUC_NE_lexicon_tags_2 },
    { 0x56b79bb5, SUC_NE_lexicon_tags_2 },
    { 0xc4599bb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0839bba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95239bbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26b59bbe, SUC_NE_lexicon_tags_2 },
    { 0x56e79bbf, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd79bc1, SUC_NE_lexicon_tags_145 },
    { 0x25f59bc2, SUC_NE_lexicon_tags_2 },
    { 0xa5259bc2, SUC_NE_lexicon_tags_2 },
    { 0xd62b9bc4, SUC_NE_lexicon_tags_2 },
    { 0xc0c79bc1, SUC_NE_lexicon_tags_2 },
    { 0xc9a19bc6, SUC_NE_lexicon_tags_4 },
    { 0x85399bc7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b19bcb, SUC_NE_lexicon_tags_2 },
    { 0xc08d9bcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc539bce, SUC_NE_lexicon_tags_2 },
    { 0x2deb9bce, SUC_NE_lexicon_tags_2 },
    { 0x389b9bce, SUC_NE_lexicon_tags_2 },
    { 0xb5af9bd1, SUC_NE_lexicon_tags_2 },
    { 0xf1479bd2, SUC_NE_lexicon_tags_2 },
    { 0x67399bd2, SUC_NE_lexicon_tags_2 },
    { 0x73799bd3, SUC_NE_lexicon_tags_5 },
    { 0x9379bcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3799bda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85f9bdc, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xb67d9bde, SUC_NE_lexicon_tags_2 },
    { 0x2aa79bdf, SUC_NE_lexicon_tags_2 },
    { 0x62639bdf, SUC_NE_lexicon_tags_7 },
    { 0x6a8f9bde, SUC_NE_lexicon_tags_2 },
    { 0xe11b9bdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c79be4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f419be7, SUC_NE_lexicon_tags_2 },
    { 0x11259be7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fd9beb, SUC_NE_lexicon_tags_2 },
    { 0x38139bec, SUC_NE_lexicon_tags_2 },
    { 0x95359bed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46b9bf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bb9bf5, SUC_NE_lexicon_tags_2 },
    { 0x11859bf6, SUC_NE_lexicon_tags_2 },
    { 0xa5359bf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe09f9bf9, SUC_NE_lexicon_tags_2 },
    { 0xfb979bfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2b9c01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3b9c05, SUC_NE_lexicon_tags_2 },
    { 0x2d079c06, SUC_NE_lexicon_tags_2 },
    { 0x5b419c06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb07f9c09, SUC_NE_lexicon_tags_2 },
    { 0x2ad79c0a, SUC_NE_lexicon_tags_3 },
    { 0xc60b9c09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33479c0e, SUC_NE_lexicon_tags_2 },
    { 0xbb8b9c0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17799c11, SUC_NE_lexicon_tags_2 },
    { 0xe2d79c12, SUC_NE_lexicon_tags_2 },
    { 0xc26b9c11, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf139c1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa599c1d, SUC_NE_lexicon_tags_2 },
    { 0xcd7f9c1d, SUC_NE_lexicon_tags_2 },
    { 0x48439c1e, SUC_NE_lexicon_tags_7 },
    { 0x1f5d9c1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6599c22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66b9c24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b9c2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99179c31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cab9c36, SUC_NE_lexicon_tags_2 },
    { 0xb7b9c37, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x527d9c39, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0x61319c3b, SUC_NE_lexicon_tags_2 },
    { 0xa599c3b, SUC_NE_lexicon_tags_2 },
    { 0xbf839c3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffd9c41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2219c48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bd79c4a, SUC_NE_lexicon_tags_2 },
    { 0x161f9c4b, SUC_NE_lexicon_tags_2 },
    { 0xa0819c4b, SUC_NE_lexicon_tags_19 },
    { 0x88a59c4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f9c5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ff39c5c, SUC_NE_lexicon_tags_2 },
    { 0x94b79c5d, SUC_NE_lexicon_tags_2 },
    { 0x957f9c5d, SUC_NE_lexicon_tags_3 },
    { 0x96af9c5f, SUC_NE_lexicon_tags_2 },
    { 0x5379c5f, SUC_NE_lexicon_tags_2 },
    { 0xcd6b9c5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb059c65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8df9c68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d9c6e, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a19c71, SUC_NE_lexicon_tags_2 },
    { 0x677d9c72, SUC_NE_lexicon_tags_7 },
    { 0x55099c73, SUC_NE_lexicon_tags_2 },
    { 0x45719c72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c79c77, SUC_NE_lexicon_tags_56 },
    { 0x43c39c78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5d9c7b, SUC_NE_lexicon_tags_2 },
    { 0x83259c7c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb9c82, SUC_NE_lexicon_tags_2 },
    { 0x23ff9c82, SUC_NE_lexicon_tags_2 },
    { 0x36459c84, SUC_NE_lexicon_tags_2 },
    { 0x8c6d9c84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65ed9c87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2519c8a, SUC_NE_lexicon_tags_2 },
    { 0x77d59c8b, SUC_NE_lexicon_tags_2 },
    { 0x6fa19c8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b9c91, SUC_NE_lexicon_tags_2 },
    { 0x7d3d9c91, SUC_NE_lexicon_tags_2 },
    { 0xb35d9c91, SUC_NE_lexicon_tags_2 },
    { 0xee759c93, SUC_NE_lexicon_tags_2 },
    { 0x16599c95, SUC_NE_lexicon_tags_2 },
    { 0x6e419c95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf19c99, SUC_NE_lexicon_tags_2 },
    { 0xec7b9c9a, SUC_NE_lexicon_tags_2 },
    { 0x40bb9c9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbacd9c9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe699caa, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd059cad, SUC_NE_lexicon_tags_2 },
    { 0x2f939cad, SUC_NE_lexicon_tags_2 },
    { 0x52ad9caf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa639cb2, SUC_NE_lexicon_tags_2 },
    { 0x3c4b9cb3, SUC_NE_lexicon_tags_2 },
    { 0x6339cb4, SUC_NE_lexicon_tags_2 },
    { 0xc0779cb4, SUC_NE_lexicon_tags_2 },
    { 0xef2f9cb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1099cbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f7b9cc1, SUC_NE_lexicon_tags_2 },
    { 0xa0a39cc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf99cc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d9ccf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9b9cd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4739cdd, SUC_NE_lexicon_tags_2 },
    { 0x817d9cde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220f9ce8, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xc0d99cea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6179cef, SUC_NE_lexicon_tags_2 },
    { 0x44f9cef, SUC_NE_lexicon_tags_2 },
    { 0x427b9cf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e759cf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d199cfb, SUC_NE_lexicon_tags_2 },
    { 0xa01d9cfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95759cff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ff9d04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda279d06, SUC_NE_lexicon_tags_2 },
    { 0x15dd9d06, SUC_NE_lexicon_tags_3 },
    { 0x800f9d08, SUC_NE_lexicon_tags_2 },
    { 0x763d9d08, SUC_NE_lexicon_tags_2 },
    { 0x3d39d06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8839d0d, SUC_NE_lexicon_tags_2 },
    { 0xa8f19d0d, SUC_NE_lexicon_tags_2 },
    { 0xd7819d0f, SUC_NE_lexicon_tags_2 },
    { 0xd6ed9d0f, SUC_NE_lexicon_tags_2 },
    { 0x49b99d0d, SUC_NE_lexicon_tags_2 },
    { 0x26cf9d12, SUC_NE_lexicon_tags_2 },
    { 0xc6579d13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32e79d15, SUC_NE_lexicon_tags_2 },
    { 0x976d9d15, SUC_NE_lexicon_tags_2 },
    { 0x229b9d17, SUC_NE_lexicon_tags_2 },
    { 0x62279d18, SUC_NE_lexicon_tags_2 },
    { 0xb4859d18, SUC_NE_lexicon_tags_2 },
    { 0xb8399d18, SUC_NE_lexicon_tags_2 },
    { 0x7d19d19, SUC_NE_lexicon_tags_2 },
    { 0xe5619d1c, SUC_NE_lexicon_tags_17 },
    { 0x23c59d1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c59d27, SUC_NE_lexicon_tags_2 },
    { 0xf0539d28, SUC_NE_lexicon_tags_2 },
    { 0x5fb59d29, SUC_NE_lexicon_tags_252 },
    { 0x69f19d28, SUC_NE_lexicon_tags_2 },
    { 0xe9bb9d2b, SUC_NE_lexicon_tags_2 },
    { 0xd5859d2c, SUC_NE_lexicon_tags_2 },
    { 0x453f9d29, SUC_NE_lexicon_tags_7 },
    { 0x77959d2c, SUC_NE_lexicon_tags_7 },
    { 0xbdad9d2a, SUC_NE_lexicon_tags_2 },
    { 0xc8cb9d28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a19d34, SUC_NE_lexicon_tags_2 },
    { 0xedd39d35, SUC_NE_lexicon_tags_2 },
    { 0xfadf9d36, SUC_NE_lexicon_tags_2 },
    { 0xaef59d37, SUC_NE_lexicon_tags_4 },
    { 0x7119d37, SUC_NE_lexicon_tags_2 },
    { 0xaa039d39, SUC_NE_lexicon_tags_2 },
    { 0x3ded9d37, SUC_NE_lexicon_tags_2 },
    { 0x3e439d3b, SUC_NE_lexicon_tags_2 },
    { 0xfd259d3b, SUC_NE_lexicon_tags_2 },
    { 0xb279d3b, SUC_NE_lexicon_tags_2 },
    { 0x33ef9d3b, SUC_NE_lexicon_tags_2 },
    { 0x72819d3f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf79d44, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a79d49, SUC_NE_lexicon_tags_2 },
    { 0xd08f9d4a, SUC_NE_lexicon_tags_2 },
    { 0x732d9d4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a79d4d, SUC_NE_lexicon_tags_2 },
    { 0xeb7b9d4e, SUC_NE_lexicon_tags_2 },
    { 0x7019d4f, SUC_NE_lexicon_tags_13 },
    { 0x83db9d4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fa99d52, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76119d59, SUC_NE_lexicon_tags_2 },
    { 0xf0e79d5a, SUC_NE_lexicon_tags_2 },
    { 0xd8999d5a, SUC_NE_lexicon_tags_2 },
    { 0x40059d5b, SUC_NE_lexicon_tags_2 },
    { 0x4f4f9d5d, SUC_NE_lexicon_tags_5 },
    { 0x6a5f9d5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82779d62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb659d66, SUC_NE_lexicon_tags_2 },
    { 0x402b9d67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21ab9d69, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x75839d6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39d6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b7f9d71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9b9d74, SUC_NE_lexicon_tags_2 },
    { 0x4e3d9d75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a0f9d77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea19d7d, SUC_NE_lexicon_tags_2 },
    { 0xd4d79d7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a79d81, SUC_NE_lexicon_tags_2 },
    { 0x4df99d82, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a899d86, SUC_NE_lexicon_tags_2 },
    { 0x9ecb9d86, SUC_NE_lexicon_tags_2 },
    { 0x8959d88, SUC_NE_lexicon_tags_2 },
    { 0x8e739d89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd399d8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2ab9d8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad99d93, SUC_NE_lexicon_tags_2 },
    { 0x8de59d93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x873d9d97, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12539d9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x717f9d9d, SUC_NE_lexicon_tags_2 },
    { 0xeb199d9d, SUC_NE_lexicon_tags_2 },
    { 0x172f9d9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5b9da3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f039da6, SUC_NE_lexicon_tags_2 },
    { 0xb0f19da7, SUC_NE_lexicon_tags_13 },
    { 0x6bc79da8, SUC_NE_lexicon_tags_2 },
    { 0x33499da9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5db9dab, SUC_NE_lexicon_tags_2 },
    { 0x6b1f9dac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1979db0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d9db5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a39db8, SUC_NE_lexicon_tags_2 },
    { 0xfa619db9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5ed9dbb, SUC_NE_lexicon_tags_2 },
    { 0x7a679dbb, SUC_NE_lexicon_tags_2 },
    { 0xa65f9dbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c19dc4, SUC_NE_lexicon_tags_2 },
    { 0xfdab9dc4, SUC_NE_lexicon_tags_2 },
    { 0x1f1b9dc6, SUC_NE_lexicon_tags_2 },
    { 0x8ea19dc7, SUC_NE_lexicon_tags_2 },
    { 0x54e99dc8, SUC_NE_lexicon_tags_2 },
    { 0x4e219dc9, SUC_NE_lexicon_tags_2 },
    { 0x600b9dca, SUC_NE_lexicon_tags_2 },
    { 0x4b839dcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dd9dce, SUC_NE_lexicon_tags_2 },
    { 0x852f9dcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x29db9dd1, SUC_NE_lexicon_tags_2 },
    { 0x9b5b9dd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44439dd4, SUC_NE_lexicon_tags_2 },
    { 0xe82b9dd4, SUC_NE_lexicon_tags_2 },
    { 0x699d9dd6, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b59dda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd9b9ddc, SUC_NE_lexicon_tags_2 },
    { 0xa7b59ddd, SUC_NE_lexicon_tags_2 },
    { 0xd7b19dde, SUC_NE_lexicon_tags_7 },
    { 0x8d4b9ddd, SUC_NE_lexicon_tags_2 },
    { 0xf05f9de0, SUC_NE_lexicon_tags_2 },
    { 0xfe2d9de1, SUC_NE_lexicon_tags_2 },
    { 0x3d259de2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x15499de4, SUC_NE_lexicon_tags_2 },
    { 0x45ff9de5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aa79de7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853d9deb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd619def, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4b9df3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x269f9df5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99ff9df7, SUC_NE_lexicon_tags_2 },
    { 0x9c9b9df7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c519dfa, SUC_NE_lexicon_tags_1 },
    { 0xa5899dfb, SUC_NE_lexicon_tags_127 },
    { 0xdd219dfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7dd9dfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17159e01, SUC_NE_lexicon_tags_2 },
    { 0x47759e02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93159e0a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed9e10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d559e14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2d9e19, SUC_NE_lexicon_tags_2 },
    { 0xe7279e1a, SUC_NE_lexicon_tags_2 },
    { 0x1799e1a, SUC_NE_lexicon_tags_2 },
    { 0x9ae39e1b, SUC_NE_lexicon_tags_2 },
    { 0x1ae59e1d, SUC_NE_lexicon_tags_2 },
    { 0x8ca79e1e, SUC_NE_lexicon_tags_3 },
    { 0xa7d79e1f, SUC_NE_lexicon_tags_18 },
    { 0xfa2f9e1d, SUC_NE_lexicon_tags_2 },
    { 0xf1239e1f, SUC_NE_lexicon_tags_2 },
    { 0x618b9e21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5d9e30, SUC_NE_lexicon_tags_2 },
    { 0x17039e30, SUC_NE_lexicon_tags_2 },
    { 0x47fb9e32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b019e37, SUC_NE_lexicon_tags_2 },
    { 0x1d779e37, SUC_NE_lexicon_tags_30 },
    { 0xe1d39e39, SUC_NE_lexicon_tags_2 },
    { 0x13ed9e3a, SUC_NE_lexicon_tags_2 },
    { 0xa999e3b, SUC_NE_lexicon_tags_2 },
    { 0xdf5b9e3b, SUC_NE_lexicon_tags_2 },
    { 0xb59f9e3d, SUC_NE_lexicon_tags_2 },
    { 0xc5f59e3a, SUC_NE_lexicon_tags_2 },
    { 0xc2ef9e39, SUC_NE_lexicon_tags_2 },
    { 0x5e199e40, SUC_NE_lexicon_tags_2 },
    { 0xa0cb9e38, SUC_NE_lexicon_tags_2 },
    { 0x1a599e3d, SUC_NE_lexicon_tags_2 },
    { 0xbe19e43, SUC_NE_lexicon_tags_30 },
    { 0xcc019e44, SUC_NE_lexicon_tags_2 },
    { 0x19d79e37, SUC_NE_lexicon_tags_450 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec79e48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed659e4b, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xe8939e4d, SUC_NE_lexicon_tags_2 },
    { 0x25339e4e, SUC_NE_lexicon_tags_2 },
    { 0x9ded9e4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf9e52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b9d9e54, SUC_NE_lexicon_tags_4 },
    { 0x2d919e55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf44f9e57, SUC_NE_lexicon_tags_2 },
    { 0xf0499e57, SUC_NE_lexicon_tags_4 },
    { 0xe3959e59, SUC_NE_lexicon_tags_3 },
    { 0xcded9e5a, SUC_NE_lexicon_tags_2 },
    { 0x96339e59, SUC_NE_lexicon_tags_2 },
    { 0xb1f19e5c, SUC_NE_lexicon_tags_2 },
    { 0xd4639e5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f899e5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46199e62, SUC_NE_lexicon_tags_2 },
    { 0xc54f9e63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cb99e65, SUC_NE_lexicon_tags_2 },
    { 0xaa339e65, SUC_NE_lexicon_tags_1 },
    { 0x6afb9e66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e19e6a, SUC_NE_lexicon_tags_2 },
    { 0x7d659e6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba519e70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf39e73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe959e76, SUC_NE_lexicon_tags_7 },
    { 0x2c519e77, SUC_NE_lexicon_tags_2 },
    { 0x8fc99e78, SUC_NE_lexicon_tags_53 },
    { 0x61379e77, SUC_NE_lexicon_tags_2 },
    { 0x4da99e7a, SUC_NE_lexicon_tags_2 },
    { 0x1ced9e7b, SUC_NE_lexicon_tags_2 },
    { 0xa74f9e7c, SUC_NE_lexicon_tags_14 },
    { 0xf4119e77, SUC_NE_lexicon_tags_2 },
    { 0x5f139e7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b719e81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eef9e84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1999e86, SUC_NE_lexicon_tags_2 },
    { 0x7a7f9e87, SUC_NE_lexicon_tags_2 },
    { 0x85ff9e88, SUC_NE_lexicon_tags_2 },
    { 0x262d9e89, SUC_NE_lexicon_tags_2 },
    { 0x74519e8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e979e8d, SUC_NE_lexicon_tags_7 },
    { 0x396d9e8e, SUC_NE_lexicon_tags_2 },
    { 0x96c59e8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x843d9e92, SUC_NE_lexicon_tags_2 },
    { 0xcbed9e93, SUC_NE_lexicon_tags_4 },
    { 0xdc599e93, SUC_NE_lexicon_tags_2 },
    { 0x42cd9e95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e8f9e97, SUC_NE_lexicon_tags_2 },
    { 0x82359e98, SUC_NE_lexicon_tags_7 },
    { 0x2c999e98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72b9e9e, SUC_NE_lexicon_tags_2 },
    { 0x40539e9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ef99ea1, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc6ed9ea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5df9ea5, SUC_NE_lexicon_tags_40 },
    { 0x549d9ea6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x938f9ea8, SUC_NE_lexicon_tags_6 },
    { 0x388b9ea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd979eab, SUC_NE_lexicon_tags_263 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b9eb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f59eb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc5b9eb5, SUC_NE_lexicon_tags_2 },
    { 0xbe579eb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55239eb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28d59ebb, SUC_NE_lexicon_tags_2 },
    { 0xee2f9ebb, SUC_NE_lexicon_tags_2 },
    { 0x1359ebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7df9ec4, SUC_NE_lexicon_tags_2 },
    { 0x9e6f9ec4, SUC_NE_lexicon_tags_2 },
    { 0xbe919ec4, SUC_NE_lexicon_tags_2 },
    { 0xfb639ec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3f9ed2, SUC_NE_lexicon_tags_2 },
    { 0xcf299ed2, SUC_NE_lexicon_tags_2 },
    { 0xfe239ed4, SUC_NE_lexicon_tags_2 },
    { 0xe6699ed3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec459ed8, SUC_NE_lexicon_tags_2 },
    { 0x725d9ed9, SUC_NE_lexicon_tags_2 },
    { 0xf52d9eda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x792d9edc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3f79ede, SUC_NE_lexicon_tags_2 },
    { 0xf579edf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449d9ee5, SUC_NE_lexicon_tags_2 },
    { 0x420b9ee5, SUC_NE_lexicon_tags_19 },
    { 0x32159ee5, SUC_NE_lexicon_tags_2 },
    { 0x1edf9ee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeb9eed, SUC_NE_lexicon_tags_2 },
    { 0x10cd9eee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b0b9ef0, SUC_NE_lexicon_tags_2 },
    { 0xd53f9ef0, SUC_NE_lexicon_tags_2 },
    { 0x4f559ef2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b9ef7, SUC_NE_lexicon_tags_2 },
    { 0xf2839ef8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a99efb, SUC_NE_lexicon_tags_2 },
    { 0x821f9efb, SUC_NE_lexicon_tags_2 },
    { 0x279b9efd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xece39eff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40459f01, SUC_NE_lexicon_tags_2 },
    { 0x5f619f01, SUC_NE_lexicon_tags_2 },
    { 0x36d99f03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fd9f09, SUC_NE_lexicon_tags_2 },
    { 0xe12d9f09, SUC_NE_lexicon_tags_2 },
    { 0x76759f09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74f9f0f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e819f13, SUC_NE_lexicon_tags_2 },
    { 0x7d539f14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88c59f16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca19f19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcf9f1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff9f20, SUC_NE_lexicon_tags_2 },
    { 0xb6199f21, SUC_NE_lexicon_tags_2 },
    { 0xc3cb9f20, SUC_NE_lexicon_tags_2 },
    { 0x69ef9f23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c619f25, SUC_NE_lexicon_tags_3 },
    { 0xaf319f26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3319f2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3159f2c, SUC_NE_lexicon_tags_2 },
    { 0xcb939f2c, SUC_NE_lexicon_tags_2 },
    { 0x46ab9f2e, SUC_NE_lexicon_tags_2 },
    { 0xa8139f2e, SUC_NE_lexicon_tags_2 },
    { 0x6d019f30, SUC_NE_lexicon_tags_69 },
    { 0x87df9f31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42df9f34, SUC_NE_lexicon_tags_2 },
    { 0x30439f34, SUC_NE_lexicon_tags_174 },
    { 0x61c79f36, SUC_NE_lexicon_tags_2 },
    { 0xa9579f37, SUC_NE_lexicon_tags_2 },
    { 0x267d9f38, SUC_NE_lexicon_tags_4 },
    { 0x87f99f36, SUC_NE_lexicon_tags_2 },
    { 0xea259f39, SUC_NE_lexicon_tags_2 },
    { 0xd9d39f3b, SUC_NE_lexicon_tags_2 },
    { 0x8a5f9f3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3359f3e, SUC_NE_lexicon_tags_2 },
    { 0x85d9f3f, SUC_NE_lexicon_tags_2 },
    { 0xc14d9f3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d59f45, SUC_NE_lexicon_tags_2 },
    { 0x827f9f45, SUC_NE_lexicon_tags_2 },
    { 0xaa399f47, SUC_NE_lexicon_tags_2 },
    { 0x62bd9f47, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7f9f4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49e59f4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc759f51, SUC_NE_lexicon_tags_40 },
    { 0xd9899f51, SUC_NE_lexicon_tags_56 },
    { 0x80f19f52, SUC_NE_lexicon_tags_24 },
    { 0xe5e39f52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c19f58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d99f5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe459f62, SUC_NE_lexicon_tags_2 },
    { 0xe3cf9f62, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x256b9f66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad9f6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b9f6d, SUC_NE_lexicon_tags_2 },
    { 0x539b9f6d, SUC_NE_lexicon_tags_2 },
    { 0xfe839f6f, SUC_NE_lexicon_tags_2 },
    { 0x29419f70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83cd9f72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a79f74, SUC_NE_lexicon_tags_2 },
    { 0x844d9f74, SUC_NE_lexicon_tags_2 },
    { 0xac4d9f76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f9f7b, SUC_NE_lexicon_tags_2 },
    { 0x6c739f7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c59f7e, SUC_NE_lexicon_tags_2 },
    { 0xac8b9f7f, SUC_NE_lexicon_tags_2 },
    { 0x28a99f80, SUC_NE_lexicon_tags_2 },
    { 0xb6719f81, SUC_NE_lexicon_tags_2 },
    { 0x6f6d9f82, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0x2dcb9f84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32959f86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f19f89, SUC_NE_lexicon_tags_2 },
    { 0xf5079f8a, SUC_NE_lexicon_tags_2 },
    { 0x4d859f8b, SUC_NE_lexicon_tags_2 },
    { 0x729d9f8b, SUC_NE_lexicon_tags_2 },
    { 0xbbc99f8d, SUC_NE_lexicon_tags_2 },
    { 0x66919f8d, SUC_NE_lexicon_tags_2 },
    { 0x96fd9f8f, SUC_NE_lexicon_tags_2 },
    { 0xcfb9f90, SUC_NE_lexicon_tags_3 },
    { 0xc8b99f91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eab9f97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38b79f99, SUC_NE_lexicon_tags_2 },
    { 0x95ad9f9a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x72df9f9c, SUC_NE_lexicon_tags_2 },
    { 0x67eb9f9d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec19fa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47eb9fa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae79faa, SUC_NE_lexicon_tags_2 },
    { 0x4139fab, SUC_NE_lexicon_tags_2 },
    { 0x1ad59faa, SUC_NE_lexicon_tags_3 },
    { 0x8c4d9fad, SUC_NE_lexicon_tags_7 },
    { 0x4b019fae, SUC_NE_lexicon_tags_2 },
    { 0xafbb9faf, SUC_NE_lexicon_tags_2 },
    { 0x834f9fb0, SUC_NE_lexicon_tags_7 },
    { 0xfa4f9faf, SUC_NE_lexicon_tags_2 },
    { 0x77939faa, SUC_NE_lexicon_tags_0 },
    { 0x1f419fb3, SUC_NE_lexicon_tags_2 },
    { 0x936b9fb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf559fb6, SUC_NE_lexicon_tags_2 },
    { 0x2c879fb7, SUC_NE_lexicon_tags_17 },
    { 0x84119fb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a39fbb, SUC_NE_lexicon_tags_2 },
    { 0x6ab59fbc, SUC_NE_lexicon_tags_95 },
    { 0x24cf9fbd, SUC_NE_lexicon_tags_2 },
    { 0x8b7d9fbc, SUC_NE_lexicon_tags_2 },
    { 0x49439fbc, SUC_NE_lexicon_tags_4 },
    { 0x19739fc0, SUC_NE_lexicon_tags_2 },
    { 0x5e7b9fc1, SUC_NE_lexicon_tags_2 },
    { 0x80fb9fc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf959fc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x527b9fc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41579fce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f19fd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb19fd6, SUC_NE_lexicon_tags_2 },
    { 0x59f39fd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x693d9fd9, SUC_NE_lexicon_tags_2 },
    { 0x63f9fd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe39fde, SUC_NE_lexicon_tags_2 },
    { 0x68519fdf, SUC_NE_lexicon_tags_2 },
    { 0x3a19fe0, SUC_NE_lexicon_tags_3 },
    { 0xc4379fdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28739fe4, SUC_NE_lexicon_tags_2 },
    { 0x60299fe5, SUC_NE_lexicon_tags_2 },
    { 0x7e8b9fe6, SUC_NE_lexicon_tags_2 },
    { 0xb9e39fe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4219fee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c319ff0, SUC_NE_lexicon_tags_2 },
    { 0xb5a99ff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe699ff6, SUC_NE_lexicon_tags_2 },
    { 0x4a1f9ff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b79a003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9251a006, SUC_NE_lexicon_tags_2 },
    { 0x50c1a007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3da00d, SUC_NE_lexicon_tags_2 },
    { 0x5c65a00e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3539a011, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1883a016, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1a01a, SUC_NE_lexicon_tags_1 },
    { 0x7091a01b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77fa01d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32b7a01f, SUC_NE_lexicon_tags_2 },
    { 0x5547a020, SUC_NE_lexicon_tags_2 },
    { 0xc8ada020, SUC_NE_lexicon_tags_0 },
    { 0xc3fda022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f35a02b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe23ba02d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6811a032, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca39a035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf1a037, SUC_NE_lexicon_tags_7 },
    { 0x7ef9a037, SUC_NE_lexicon_tags_2 },
    { 0x9b15a037, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7479a040, SUC_NE_lexicon_tags_140 },
    { 0x859a040, SUC_NE_lexicon_tags_7 },
    { 0x7c9da040, SUC_NE_lexicon_tags_2 },
    { 0x92efa040, SUC_NE_lexicon_tags_4 },
    { 0x69bda042, SUC_NE_lexicon_tags_2 },
    { 0xf09fa045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7da048, SUC_NE_lexicon_tags_2 },
    { 0x16efa049, SUC_NE_lexicon_tags_2 },
    { 0xc21ba048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bffa04d, SUC_NE_lexicon_tags_7 },
    { 0x3a5a04e, SUC_NE_lexicon_tags_2 },
    { 0xd2c5a04f, SUC_NE_lexicon_tags_2 },
    { 0xd6a1a050, SUC_NE_lexicon_tags_1 },
    { 0x9c1fa04e, SUC_NE_lexicon_tags_7 },
    { 0x8f3a04e, SUC_NE_lexicon_tags_2 },
    { 0xb205a052, SUC_NE_lexicon_tags_2 },
    { 0xa1f1a054, SUC_NE_lexicon_tags_2 },
    { 0xf7cfa055, SUC_NE_lexicon_tags_2 },
    { 0x5139a056, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6423a058, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6133a05b, SUC_NE_lexicon_tags_2 },
    { 0xba7a05b, SUC_NE_lexicon_tags_7 },
    { 0x777ba05d, SUC_NE_lexicon_tags_2 },
    { 0x9cf9a05d, SUC_NE_lexicon_tags_2 },
    { 0x31a5a05c, SUC_NE_lexicon_tags_2 },
    { 0x81bba060, SUC_NE_lexicon_tags_2 },
    { 0x7133a061, SUC_NE_lexicon_tags_2 },
    { 0xe31ba062, SUC_NE_lexicon_tags_7 },
    { 0xf59fa063, SUC_NE_lexicon_tags_2 },
    { 0xcc51a064, SUC_NE_lexicon_tags_2 },
    { 0xa117a063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbee3a067, SUC_NE_lexicon_tags_13 },
    { 0x2e97a068, SUC_NE_lexicon_tags_2 },
    { 0xabba069, SUC_NE_lexicon_tags_2 },
    { 0x5993a068, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dba06e, SUC_NE_lexicon_tags_2 },
    { 0xcde3a06f, SUC_NE_lexicon_tags_23 },
    { 0xbd5fa06e, SUC_NE_lexicon_tags_2 },
    { 0x2b0da071, SUC_NE_lexicon_tags_2 },
    { 0xe0d3a072, SUC_NE_lexicon_tags_2 },
    { 0xace9a072, SUC_NE_lexicon_tags_34 },
    { 0xe6ada072, SUC_NE_lexicon_tags_3 },
    { 0xbd71a075, SUC_NE_lexicon_tags_2 },
    { 0xb7d7a073, SUC_NE_lexicon_tags_17 },
    { 0x72f7a073, SUC_NE_lexicon_tags_2 },
    { 0xe49ba078, SUC_NE_lexicon_tags_2 },
    { 0xbb5da072, SUC_NE_lexicon_tags_2 },
    { 0xc8bda07a, SUC_NE_lexicon_tags_2 },
    { 0x609ba07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc39a07d, SUC_NE_lexicon_tags_2 },
    { 0xce4ba07e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f5a081, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x808ba083, SUC_NE_lexicon_tags_4 },
    { 0x8075a084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1f9a086, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba089, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x419ba08b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa417a092, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40da099, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50da09e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba31a0a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75a0ab, SUC_NE_lexicon_tags_2 },
    { 0xa035a0ac, SUC_NE_lexicon_tags_2 },
    { 0x1a05a0ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9da0b1, SUC_NE_lexicon_tags_2 },
    { 0x4321a0b2, SUC_NE_lexicon_tags_2 },
    { 0x1d01a0b3, SUC_NE_lexicon_tags_2 },
    { 0x9b51a0b4, SUC_NE_lexicon_tags_2 },
    { 0xe947a0b4, SUC_NE_lexicon_tags_2 },
    { 0x6525a0b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8791a0b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x867a0ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf34da0bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd281a0c0, SUC_NE_lexicon_tags_2 },
    { 0x1145a0c0, SUC_NE_lexicon_tags_2 },
    { 0x7335a0c1, SUC_NE_lexicon_tags_51 },
    { 0x8889a0c3, SUC_NE_lexicon_tags_2 },
    { 0xd8bba0c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3a0c8, SUC_NE_lexicon_tags_2 },
    { 0xc1dda0c8, SUC_NE_lexicon_tags_2 },
    { 0x53c3a0ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1929a0ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccda0d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920ba0d7, SUC_NE_lexicon_tags_113 },
    { 0x557fa0d7, SUC_NE_lexicon_tags_2 },
    { 0xc8a7a0d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6749a0dc, SUC_NE_lexicon_tags_188 },
    { 0x8ae7a0dd, SUC_NE_lexicon_tags_2 },
    { 0xd599a0dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc31a0e1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9a0e5, SUC_NE_lexicon_tags_2 },
    { 0x431da0e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4fa0ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe901a0ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e13a0f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x716ba0f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e1a0fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf94fa0fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccba101, SUC_NE_lexicon_tags_2 },
    { 0x731ba101, SUC_NE_lexicon_tags_2 },
    { 0xca2ba103, SUC_NE_lexicon_tags_2 },
    { 0x2f83a104, SUC_NE_lexicon_tags_2 },
    { 0x8a75a105, SUC_NE_lexicon_tags_2 },
    { 0x62f3a106, SUC_NE_lexicon_tags_2 },
    { 0xf8d7a106, SUC_NE_lexicon_tags_2 },
    { 0x3f49a105, SUC_NE_lexicon_tags_2 },
    { 0x7d21a106, SUC_NE_lexicon_tags_2 },
    { 0xf641a10a, SUC_NE_lexicon_tags_2 },
    { 0x6cc5a10a, SUC_NE_lexicon_tags_2 },
    { 0xb3dfa10c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c3a10f, SUC_NE_lexicon_tags_2 },
    { 0x41e9a10f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe025a114, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57fa118, SUC_NE_lexicon_tags_2 },
    { 0x25fda119, SUC_NE_lexicon_tags_2 },
    { 0xd4cfa118, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aba11d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65a121, SUC_NE_lexicon_tags_4 },
    { 0x805fa122, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x864da124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9a129, SUC_NE_lexicon_tags_2 },
    { 0xbac3a12a, SUC_NE_lexicon_tags_2 },
    { 0xaaf9a12b, SUC_NE_lexicon_tags_2 },
    { 0x461a12a, SUC_NE_lexicon_tags_14 },
    { 0x5f3fa12c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6743a131, SUC_NE_lexicon_tags_2 },
    { 0xb4b9a132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x721a135, SUC_NE_lexicon_tags_2 },
    { 0x7495a135, SUC_NE_lexicon_tags_2 },
    { 0xf391a136, SUC_NE_lexicon_tags_2 },
    { 0x3f99a138, SUC_NE_lexicon_tags_2 },
    { 0x9161a139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7311a13b, SUC_NE_lexicon_tags_2 },
    { 0x1321a13b, SUC_NE_lexicon_tags_2 },
    { 0x4c4fa13b, SUC_NE_lexicon_tags_4 },
    { 0xa71fa13e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cfa142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a93a146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x279da148, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bba14e, SUC_NE_lexicon_tags_2 },
    { 0xe263a14f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7a153, SUC_NE_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e1a158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cda15b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ecfa15d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9c5a15f, SUC_NE_lexicon_tags_2 },
    { 0xc1eda160, SUC_NE_lexicon_tags_2 },
    { 0x5689a161, SUC_NE_lexicon_tags_2 },
    { 0xfb25a162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3813a164, SUC_NE_lexicon_tags_2 },
    { 0xfadba165, SUC_NE_lexicon_tags_2 },
    { 0xf92fa165, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfba16c, SUC_NE_lexicon_tags_2 },
    { 0xb92da16d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcfa171, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a95a176, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x291da178, SUC_NE_lexicon_tags_2 },
    { 0xba8ba178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7897a17e, SUC_NE_lexicon_tags_2 },
    { 0x91bfa17e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dba187, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3981a189, SUC_NE_lexicon_tags_2 },
    { 0x5befa18a, SUC_NE_lexicon_tags_2 },
    { 0x1ebfa18b, SUC_NE_lexicon_tags_2 },
    { 0xac7da18c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6627a18f, SUC_NE_lexicon_tags_3 },
    { 0x4829a190, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce9a194, SUC_NE_lexicon_tags_3 },
    { 0xb78fa195, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb9a199, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ae9a19b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd339a1a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e43a1a6, SUC_NE_lexicon_tags_2 },
    { 0x1f71a1a7, SUC_NE_lexicon_tags_30 },
    { 0x25cba1a6, SUC_NE_lexicon_tags_2 },
    { 0x5b49a1a9, SUC_NE_lexicon_tags_2 },
    { 0xec7da1aa, SUC_NE_lexicon_tags_14 },
    { 0x67cda1ab, SUC_NE_lexicon_tags_4 },
    { 0xcc3ba1ac, SUC_NE_lexicon_tags_2 },
    { 0xebc9a1aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb005a1af, SUC_NE_lexicon_tags_14 },
    { 0xf42fa1af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca6ba1b2, SUC_NE_lexicon_tags_416 },
    { 0, NULL },
    { 0x19cba1b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded3a1b9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe1a1bf, SUC_NE_lexicon_tags_2 },
    { 0x2bdfa1bf, SUC_NE_lexicon_tags_2 },
    { 0xf335a1c1, SUC_NE_lexicon_tags_2 },
    { 0xcb9da1c2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f5a1c5, SUC_NE_lexicon_tags_2 },
    { 0x92f3a1c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d85a1c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce9a1d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1bda1d3, SUC_NE_lexicon_tags_2 },
    { 0xd38ba1d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d75a1d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa77a1d8, SUC_NE_lexicon_tags_2 },
    { 0xfb93a1d9, SUC_NE_lexicon_tags_2 },
    { 0x135fa1da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa63ba1dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f0da1de, SUC_NE_lexicon_tags_2 },
    { 0x85f5a1de, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3297a1e2, SUC_NE_lexicon_tags_7 },
    { 0xe17fa1e3, SUC_NE_lexicon_tags_2 },
    { 0xc2cba1e4, SUC_NE_lexicon_tags_2 },
    { 0xfab1a1e4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xe921a1e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a35a1e9, SUC_NE_lexicon_tags_2 },
    { 0xf243a1ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7903a1ee, SUC_NE_lexicon_tags_2 },
    { 0xb777a1ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83b5a1f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc913a1f6, SUC_NE_lexicon_tags_2 },
    { 0xe3dda1f7, SUC_NE_lexicon_tags_2 },
    { 0xfb51a1f8, SUC_NE_lexicon_tags_2 },
    { 0x4c03a1f6, SUC_NE_lexicon_tags_2 },
    { 0x8225a1fa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08da1fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa395a204, SUC_NE_lexicon_tags_0 },
    { 0x9343a205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x455da207, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aba20b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf37a20d, SUC_NE_lexicon_tags_2 },
    { 0x599a20e, SUC_NE_lexicon_tags_2 },
    { 0xdadfa20f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11fa213, SUC_NE_lexicon_tags_2 },
    { 0x2a7fa214, SUC_NE_lexicon_tags_2 },
    { 0x3555a215, SUC_NE_lexicon_tags_2 },
    { 0x588ba214, SUC_NE_lexicon_tags_2 },
    { 0xf7cda217, SUC_NE_lexicon_tags_2 },
    { 0xf887a217, SUC_NE_lexicon_tags_2 },
    { 0xec09a218, SUC_NE_lexicon_tags_145 },
    { 0xb10ba216, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x631ba21c, SUC_NE_lexicon_tags_2 },
    { 0xae1da21c, SUC_NE_lexicon_tags_2 },
    { 0x973ba21d, SUC_NE_lexicon_tags_2 },
    { 0x53efa21f, SUC_NE_lexicon_tags_2 },
    { 0x63e9a220, SUC_NE_lexicon_tags_2 },
    { 0x9da5a21f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ba226, SUC_NE_lexicon_tags_2 },
    { 0xa7bfa226, SUC_NE_lexicon_tags_2 },
    { 0xd165a228, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5bba22a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ba22d, SUC_NE_lexicon_tags_2 },
    { 0x1729a22e, SUC_NE_lexicon_tags_2 },
    { 0x40dba22f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ba232, SUC_NE_lexicon_tags_4 },
    { 0x8127a233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd3a238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41a23b, SUC_NE_lexicon_tags_3 },
    { 0x5ae3a23c, SUC_NE_lexicon_tags_2 },
    { 0xab77a23d, SUC_NE_lexicon_tags_2 },
    { 0x32ba23e, SUC_NE_lexicon_tags_2 },
    { 0xd12da23c, SUC_NE_lexicon_tags_2 },
    { 0x466fa240, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6271a243, SUC_NE_lexicon_tags_13 },
    { 0x28dba244, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eda24b, SUC_NE_lexicon_tags_2 },
    { 0x8d97a24b, SUC_NE_lexicon_tags_17 },
    { 0xc753a24c, SUC_NE_lexicon_tags_2 },
    { 0xc053a24e, SUC_NE_lexicon_tags_2 },
    { 0xadcfa24e, SUC_NE_lexicon_tags_2 },
    { 0xd621a24e, SUC_NE_lexicon_tags_4 },
    { 0x22b9a250, SUC_NE_lexicon_tags_2 },
    { 0x1671a252, SUC_NE_lexicon_tags_2 },
    { 0x41da253, SUC_NE_lexicon_tags_2 },
    { 0x20a5a24d, SUC_NE_lexicon_tags_2 },
    { 0x3fa3a252, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x849da258, SUC_NE_lexicon_tags_2 },
    { 0xb0da259, SUC_NE_lexicon_tags_2 },
    { 0x8b07a259, SUC_NE_lexicon_tags_3 },
    { 0x4c25a258, SUC_NE_lexicon_tags_148 },
    { 0, NULL },
    { 0xccfba25d, SUC_NE_lexicon_tags_2 },
    { 0xb899a25d, SUC_NE_lexicon_tags_2 },
    { 0x355da25f, SUC_NE_lexicon_tags_2 },
    { 0xac59a260, SUC_NE_lexicon_tags_8 },
    { 0x8745a261, SUC_NE_lexicon_tags_2 },
    { 0xb4afa25d, SUC_NE_lexicon_tags_2 },
    { 0x9c55a25f, SUC_NE_lexicon_tags_18 },
    { 0xf3a7a264, SUC_NE_lexicon_tags_2 },
    { 0x53e1a265, SUC_NE_lexicon_tags_2 },
    { 0x8785a266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd17a268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc75a26a, SUC_NE_lexicon_tags_2 },
    { 0x4fdba26b, SUC_NE_lexicon_tags_7 },
    { 0x3c7da26c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe863a272, SUC_NE_lexicon_tags_7 },
    { 0x185a273, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f3ba275, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275a280, SUC_NE_lexicon_tags_34 },
    { 0xc06ba281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x151ba283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eba288, SUC_NE_lexicon_tags_2 },
    { 0x40e7a289, SUC_NE_lexicon_tags_2 },
    { 0x29b3a28a, SUC_NE_lexicon_tags_2 },
    { 0xff91a28b, SUC_NE_lexicon_tags_2 },
    { 0xa113a28c, SUC_NE_lexicon_tags_2 },
    { 0x7e67a288, SUC_NE_lexicon_tags_2 },
    { 0x7851a28c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd521a292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60c7a294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd9a298, SUC_NE_lexicon_tags_2 },
    { 0x3687a299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4fa29c, SUC_NE_lexicon_tags_2 },
    { 0xf741a29c, SUC_NE_lexicon_tags_2 },
    { 0x6e31a29e, SUC_NE_lexicon_tags_2 },
    { 0x93f3a29e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754fa2a3, SUC_NE_lexicon_tags_2 },
    { 0x8d9ba2a4, SUC_NE_lexicon_tags_2 },
    { 0x24b5a2a3, SUC_NE_lexicon_tags_89 },
    { 0x5d15a2a4, SUC_NE_lexicon_tags_2 },
    { 0xa39a2a6, SUC_NE_lexicon_tags_2 },
    { 0x7209a2a8, SUC_NE_lexicon_tags_7 },
    { 0x9549a2a9, SUC_NE_lexicon_tags_2 },
    { 0x234ba2aa, SUC_NE_lexicon_tags_2 },
    { 0x3e05a2aa, SUC_NE_lexicon_tags_2 },
    { 0xca69a2ac, SUC_NE_lexicon_tags_2 },
    { 0x8805a2aa, SUC_NE_lexicon_tags_2 },
    { 0xe955a2ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b9a2b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f5ba2b3, SUC_NE_lexicon_tags_2 },
    { 0x81d9a2b4, SUC_NE_lexicon_tags_2 },
    { 0x4f87a2b4, SUC_NE_lexicon_tags_2 },
    { 0xaac7a2b6, SUC_NE_lexicon_tags_2 },
    { 0x41cda2b5, SUC_NE_lexicon_tags_1 },
    { 0x7509a2b8, SUC_NE_lexicon_tags_2 },
    { 0xebb7a2b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbba2bd, SUC_NE_lexicon_tags_2 },
    { 0x1a8fa2be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7deba2c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129ba2c6, SUC_NE_lexicon_tags_2 },
    { 0x4af1a2c7, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc291a2cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8cfa2cd, SUC_NE_lexicon_tags_2 },
    { 0x5943a2ce, SUC_NE_lexicon_tags_2 },
    { 0x908ba2ce, SUC_NE_lexicon_tags_2 },
    { 0x1c67a2cf, SUC_NE_lexicon_tags_7 },
    { 0x4dd7a2d1, SUC_NE_lexicon_tags_2 },
    { 0x94fa2d0, SUC_NE_lexicon_tags_2 },
    { 0xa379a2d3, SUC_NE_lexicon_tags_2 },
    { 0xe18da2d4, SUC_NE_lexicon_tags_2 },
    { 0xfe55a2d3, SUC_NE_lexicon_tags_2 },
    { 0x4335a2d3, SUC_NE_lexicon_tags_2 },
    { 0x9995a2d7, SUC_NE_lexicon_tags_4 },
    { 0x26fda2d6, SUC_NE_lexicon_tags_2 },
    { 0x4227a2d9, SUC_NE_lexicon_tags_2 },
    { 0x2ff1a2d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc41a2dd, SUC_NE_lexicon_tags_2 },
    { 0x8dd1a2de, SUC_NE_lexicon_tags_2 },
    { 0xd5a3a2df, SUC_NE_lexicon_tags_2 },
    { 0x771fa2de, SUC_NE_lexicon_tags_2 },
    { 0x9eba2e0, SUC_NE_lexicon_tags_7 },
    { 0x3a23a2de, SUC_NE_lexicon_tags_14 },
    { 0xc037a2e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f45a2e8, SUC_NE_lexicon_tags_2 },
    { 0x539a2e9, SUC_NE_lexicon_tags_2 },
    { 0x68d5a2e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c11a2ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533fa2f0, SUC_NE_lexicon_tags_5 },
    { 0x2977a2f0, SUC_NE_lexicon_tags_18 },
    { 0x9cdfa2f2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8fa2f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f15a2f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb1a2fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3325a2fd, SUC_NE_lexicon_tags_2 },
    { 0x2015a2fe, SUC_NE_lexicon_tags_2 },
    { 0x778da2ff, SUC_NE_lexicon_tags_2 },
    { 0x2525a300, SUC_NE_lexicon_tags_2 },
    { 0xb86fa2fd, SUC_NE_lexicon_tags_2 },
    { 0x49b9a302, SUC_NE_lexicon_tags_2 },
    { 0xe1c3a301, SUC_NE_lexicon_tags_2 },
    { 0xbff1a300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86dfa306, SUC_NE_lexicon_tags_2 },
    { 0x607a306, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x396ba309, SUC_NE_lexicon_tags_2 },
    { 0x3c03a30a, SUC_NE_lexicon_tags_5 },
    { 0xc4da30b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb543a30e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57da311, SUC_NE_lexicon_tags_2 },
    { 0x3a09a311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9faba314, SUC_NE_lexicon_tags_2 },
    { 0xefb3a315, SUC_NE_lexicon_tags_2 },
    { 0xdb17a316, SUC_NE_lexicon_tags_5 },
    { 0x7023a317, SUC_NE_lexicon_tags_18 },
    { 0x3b9fa318, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7645a31b, SUC_NE_lexicon_tags_7 },
    { 0x61afa31c, SUC_NE_lexicon_tags_2 },
    { 0x1259a31b, SUC_NE_lexicon_tags_32 },
    { 0x15efa31e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde5a326, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f3fa328, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf821a32c, SUC_NE_lexicon_tags_2 },
    { 0xe4c1a32c, SUC_NE_lexicon_tags_49 },
    { 0xde45a32e, SUC_NE_lexicon_tags_2 },
    { 0x1fbfa32e, SUC_NE_lexicon_tags_2 },
    { 0x98fba32f, SUC_NE_lexicon_tags_3 },
    { 0xb331a331, SUC_NE_lexicon_tags_2 },
    { 0x29e7a331, SUC_NE_lexicon_tags_2 },
    { 0x2b03a333, SUC_NE_lexicon_tags_2 },
    { 0x4c8ba32e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2879a336, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xeb35a338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x971da33a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7ba33e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff67a345, SUC_NE_lexicon_tags_2 },
    { 0xe215a345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77c5a348, SUC_NE_lexicon_tags_2 },
    { 0xd05ba349, SUC_NE_lexicon_tags_2 },
    { 0x8825a348, SUC_NE_lexicon_tags_2 },
    { 0x5c87a34b, SUC_NE_lexicon_tags_2 },
    { 0xf6b7a349, SUC_NE_lexicon_tags_11 },
    { 0x1611a34d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e9a352, SUC_NE_lexicon_tags_2 },
    { 0xc5f1a352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x436fa356, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22da35c, SUC_NE_lexicon_tags_2 },
    { 0xb58da35d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4849a360, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1fa364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x325fa367, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x766ba369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d23a36d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fda372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5a376, SUC_NE_lexicon_tags_2 },
    { 0x7a11a377, SUC_NE_lexicon_tags_2 },
    { 0x34c7a378, SUC_NE_lexicon_tags_2 },
    { 0x6f69a379, SUC_NE_lexicon_tags_81 },
    { 0x6d6ba378, SUC_NE_lexicon_tags_2 },
    { 0x558da37b, SUC_NE_lexicon_tags_4 },
    { 0x7255a37a, SUC_NE_lexicon_tags_2 },
    { 0x8483a378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c9a380, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82afa38e, SUC_NE_lexicon_tags_2 },
    { 0xd647a38e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa651a394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37aba398, SUC_NE_lexicon_tags_2 },
    { 0x2e43a398, SUC_NE_lexicon_tags_2 },
    { 0x6553a39a, SUC_NE_lexicon_tags_2 },
    { 0x806ba39a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9da3a9, SUC_NE_lexicon_tags_2 },
    { 0x9a05a3aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4949a3ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25d1a3af, SUC_NE_lexicon_tags_2 },
    { 0xbeba3af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b9a3b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fa3b8, SUC_NE_lexicon_tags_2 },
    { 0x106fa3b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0a9a3bb, SUC_NE_lexicon_tags_2 },
    { 0x3acda3bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6051a3c2, SUC_NE_lexicon_tags_2 },
    { 0x66da3c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6db1a3c5, SUC_NE_lexicon_tags_2 },
    { 0xa2b1a3c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ba3cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82d5a3ce, SUC_NE_lexicon_tags_2 },
    { 0x2bbba3cf, SUC_NE_lexicon_tags_2 },
    { 0x72e5a3d0, SUC_NE_lexicon_tags_161 },
    { 0xe4f3a3d1, SUC_NE_lexicon_tags_2 },
    { 0xc8e3a3d1, SUC_NE_lexicon_tags_2 },
    { 0x1877a3d3, SUC_NE_lexicon_tags_2 },
    { 0xfd73a3d3, SUC_NE_lexicon_tags_2 },
    { 0x9435a3d0, SUC_NE_lexicon_tags_17 },
    { 0x152fa3ce, SUC_NE_lexicon_tags_2 },
    { 0xb6b1a3d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb61a3db, SUC_NE_lexicon_tags_2 },
    { 0xca01a3dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9a3e0, SUC_NE_lexicon_tags_2 },
    { 0x5ccda3e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfcda3e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc931a3e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeceba3eb, SUC_NE_lexicon_tags_2 },
    { 0xec9fa3ec, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xcf51a3ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fa3f4, SUC_NE_lexicon_tags_2 },
    { 0x9ef1a3f5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ada3fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4ba3fe, SUC_NE_lexicon_tags_2 },
    { 0xb083a3ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31da402, SUC_NE_lexicon_tags_2 },
    { 0xad6ba403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d5a406, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f9a408, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba40b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9709a414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c83a416, SUC_NE_lexicon_tags_30 },
    { 0xc487a416, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb09da419, SUC_NE_lexicon_tags_2 },
    { 0x31f5a419, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6741a420, SUC_NE_lexicon_tags_2 },
    { 0xfd65a421, SUC_NE_lexicon_tags_2 },
    { 0xc7dfa420, SUC_NE_lexicon_tags_2 },
    { 0xbc03a423, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf825a427, SUC_NE_lexicon_tags_2 },
    { 0xf3a5a428, SUC_NE_lexicon_tags_320 },
    { 0x68e1a427, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2519a436, SUC_NE_lexicon_tags_2 },
    { 0x38f7a437, SUC_NE_lexicon_tags_433 },
    { 0, NULL },
    { 0x6225a439, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf51a43b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cda43e, SUC_NE_lexicon_tags_2 },
    { 0x8e1fa43f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143a444, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1ba446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x255da448, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44a5a44a, SUC_NE_lexicon_tags_2 },
    { 0x59cfa44a, SUC_NE_lexicon_tags_2 },
    { 0x56aba44c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc35a44e, SUC_NE_lexicon_tags_2 },
    { 0x4d01a44e, SUC_NE_lexicon_tags_2 },
    { 0x1357a450, SUC_NE_lexicon_tags_3 },
    { 0x8271a44e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4763a453, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3fa457, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a1a45a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe061a45c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53fa461, SUC_NE_lexicon_tags_2 },
    { 0x5541a462, SUC_NE_lexicon_tags_2 },
    { 0x765ba463, SUC_NE_lexicon_tags_4 },
    { 0x6d89a462, SUC_NE_lexicon_tags_2 },
    { 0xfa8fa465, SUC_NE_lexicon_tags_8 },
    { 0x93a466, SUC_NE_lexicon_tags_4 },
    { 0x6d79a467, SUC_NE_lexicon_tags_4 },
    { 0xd2d3a466, SUC_NE_lexicon_tags_2 },
    { 0xb56da469, SUC_NE_lexicon_tags_2 },
    { 0x4f91a46a, SUC_NE_lexicon_tags_2 },
    { 0xd903a462, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d27a46f, SUC_NE_lexicon_tags_3 },
    { 0x4bb5a46f, SUC_NE_lexicon_tags_2 },
    { 0xa8bda471, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03a476, SUC_NE_lexicon_tags_2 },
    { 0xf1ba477, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cda47b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab3a47f, SUC_NE_lexicon_tags_5 },
    { 0xbae1a480, SUC_NE_lexicon_tags_2 },
    { 0xdaa7a47f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce09a485, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d35a487, SUC_NE_lexicon_tags_7 },
    { 0x2023a488, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1551a48c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf247a48e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a39a491, SUC_NE_lexicon_tags_2 },
    { 0x205fa492, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0f7a494, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4773a496, SUC_NE_lexicon_tags_2 },
    { 0xdd7ba496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaabda499, SUC_NE_lexicon_tags_2 },
    { 0x1b67a49a, SUC_NE_lexicon_tags_2 },
    { 0xb7a49b, SUC_NE_lexicon_tags_2 },
    { 0xf9b9a49c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8105a49e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fa4a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc855a4a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd181a4a6, SUC_NE_lexicon_tags_58 },
    { 0xb17da4a7, SUC_NE_lexicon_tags_2 },
    { 0xe841a4a7, SUC_NE_lexicon_tags_2 },
    { 0x2a85a4a9, SUC_NE_lexicon_tags_2 },
    { 0x6cf9a4a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5ba4ad, SUC_NE_lexicon_tags_2 },
    { 0x88bda4ad, SUC_NE_lexicon_tags_2 },
    { 0x1647a4af, SUC_NE_lexicon_tags_2 },
    { 0x46b1a4b0, SUC_NE_lexicon_tags_2 },
    { 0xd829a4ae, SUC_NE_lexicon_tags_2 },
    { 0x5441a4ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9ba4b5, SUC_NE_lexicon_tags_2 },
    { 0xacd9a4b5, SUC_NE_lexicon_tags_2 },
    { 0x1241a4b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x70d1a4b9, SUC_NE_lexicon_tags_7 },
    { 0x6ee1a4ba, SUC_NE_lexicon_tags_2 },
    { 0xe263a4bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597a4c0, SUC_NE_lexicon_tags_24 },
    { 0x3ea7a4c1, SUC_NE_lexicon_tags_2 },
    { 0x90b9a4c2, SUC_NE_lexicon_tags_3 },
    { 0x375a4c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32efa4c8, SUC_NE_lexicon_tags_2 },
    { 0x55bba4c8, SUC_NE_lexicon_tags_2 },
    { 0xc2a1a4ca, SUC_NE_lexicon_tags_2 },
    { 0xa11a4c9, SUC_NE_lexicon_tags_184 },
    { 0x3681a4cc, SUC_NE_lexicon_tags_7 },
    { 0x3d07a4c8, SUC_NE_lexicon_tags_7 },
    { 0x836fa4ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafdda4d0, SUC_NE_lexicon_tags_2 },
    { 0x77fba4d1, SUC_NE_lexicon_tags_2 },
    { 0x69ada4d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd603a4d7, SUC_NE_lexicon_tags_2 },
    { 0x4845a4d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09a4db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2ba4df, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c7a4e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9ba4eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecda4ee, SUC_NE_lexicon_tags_2 },
    { 0xa935a4ef, SUC_NE_lexicon_tags_2 },
    { 0x89bda4f0, SUC_NE_lexicon_tags_4 },
    { 0xb091a4f0, SUC_NE_lexicon_tags_2 },
    { 0x5491a4f0, SUC_NE_lexicon_tags_2 },
    { 0xc099a4f0, SUC_NE_lexicon_tags_2 },
    { 0xa241a4f4, SUC_NE_lexicon_tags_2 },
    { 0xde17a4f2, SUC_NE_lexicon_tags_2 },
    { 0x5445a4f4, SUC_NE_lexicon_tags_2 },
    { 0xdc95a4f0, SUC_NE_lexicon_tags_2 },
    { 0x3271a4f8, SUC_NE_lexicon_tags_3 },
    { 0x7c09a4f9, SUC_NE_lexicon_tags_2 },
    { 0x59b5a4fa, SUC_NE_lexicon_tags_2 },
    { 0xee5fa4fb, SUC_NE_lexicon_tags_2 },
    { 0xb79a4fb, SUC_NE_lexicon_tags_2 },
    { 0x5a71a4fd, SUC_NE_lexicon_tags_2 },
    { 0x8119a4fc, SUC_NE_lexicon_tags_2 },
    { 0xa1a7a4ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee0ba501, SUC_NE_lexicon_tags_2 },
    { 0xa503a502, SUC_NE_lexicon_tags_2 },
    { 0x889a502, SUC_NE_lexicon_tags_1 },
    { 0xae09a504, SUC_NE_lexicon_tags_353 },
    { 0, NULL },
    { 0, NULL },
    { 0x7727a507, SUC_NE_lexicon_tags_2 },
    { 0x9b1ba507, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x463ba50a, SUC_NE_lexicon_tags_2 },
    { 0xf16fa50b, SUC_NE_lexicon_tags_2 },
    { 0x9bcda50c, SUC_NE_lexicon_tags_2 },
    { 0xef3da50d, SUC_NE_lexicon_tags_2 },
    { 0x1831a50e, SUC_NE_lexicon_tags_2 },
    { 0xe44fa50e, SUC_NE_lexicon_tags_2 },
    { 0x4aafa50e, SUC_NE_lexicon_tags_2 },
    { 0xea5ba50f, SUC_NE_lexicon_tags_2 },
    { 0x83bfa50f, SUC_NE_lexicon_tags_2 },
    { 0x9f81a510, SUC_NE_lexicon_tags_56 },
    { 0, NULL },
    { 0xa6cda515, SUC_NE_lexicon_tags_7 },
    { 0x2119a516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd98fa518, SUC_NE_lexicon_tags_2 },
    { 0x4959a519, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef85a51d, SUC_NE_lexicon_tags_2 },
    { 0x98fda51e, SUC_NE_lexicon_tags_2 },
    { 0x2287a51f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ba522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73fda524, SUC_NE_lexicon_tags_2 },
    { 0x7bd7a525, SUC_NE_lexicon_tags_4 },
    { 0xbbc7a525, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d63a52c, SUC_NE_lexicon_tags_2 },
    { 0x404fa52d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1999a52f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5661a534, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b31a536, SUC_NE_lexicon_tags_2 },
    { 0x2a4ba537, SUC_NE_lexicon_tags_2 },
    { 0xec71a536, SUC_NE_lexicon_tags_2 },
    { 0x8755a539, SUC_NE_lexicon_tags_2 },
    { 0x5e5da53a, SUC_NE_lexicon_tags_2 },
    { 0xf6cba536, SUC_NE_lexicon_tags_2 },
    { 0x8abba53c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23fa53f, SUC_NE_lexicon_tags_2 },
    { 0xdb85a540, SUC_NE_lexicon_tags_2 },
    { 0x856ba53f, SUC_NE_lexicon_tags_2 },
    { 0xf03ba541, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9109a544, SUC_NE_lexicon_tags_2 },
    { 0xe769a545, SUC_NE_lexicon_tags_2 },
    { 0x51d9a546, SUC_NE_lexicon_tags_7 },
    { 0x83eba545, SUC_NE_lexicon_tags_4 },
    { 0xc02ba548, SUC_NE_lexicon_tags_2 },
    { 0xeb85a548, SUC_NE_lexicon_tags_2 },
    { 0x483da54a, SUC_NE_lexicon_tags_2 },
    { 0x3fdda546, SUC_NE_lexicon_tags_2 },
    { 0x1257a54c, SUC_NE_lexicon_tags_2 },
    { 0x7aafa54d, SUC_NE_lexicon_tags_2 },
    { 0xb68fa54e, SUC_NE_lexicon_tags_1 },
    { 0x1e9a54f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bba556, SUC_NE_lexicon_tags_2 },
    { 0x2c39a556, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe447a55a, SUC_NE_lexicon_tags_2 },
    { 0x7363a55b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb57a55d, SUC_NE_lexicon_tags_2 },
    { 0x9a69a55e, SUC_NE_lexicon_tags_2 },
    { 0x54bda55f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa677a565, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x191ba568, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9307a56d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fba571, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fa573, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d9a576, SUC_NE_lexicon_tags_7 },
    { 0x3deba577, SUC_NE_lexicon_tags_2 },
    { 0x4571a577, SUC_NE_lexicon_tags_2 },
    { 0x86c3a578, SUC_NE_lexicon_tags_2 },
    { 0x9e7da57a, SUC_NE_lexicon_tags_2 },
    { 0xb675a57a, SUC_NE_lexicon_tags_58 },
    { 0xd717a57c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31a582, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09da586, SUC_NE_lexicon_tags_2 },
    { 0xd469a587, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x51c5a589, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cfa58e, SUC_NE_lexicon_tags_2 },
    { 0xab8fa58f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x947a591, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa517a596, SUC_NE_lexicon_tags_2 },
    { 0xd0b9a597, SUC_NE_lexicon_tags_19 },
    { 0xc539a597, SUC_NE_lexicon_tags_2 },
    { 0x79b7a596, SUC_NE_lexicon_tags_2 },
    { 0x8177a59a, SUC_NE_lexicon_tags_2 },
    { 0x3fba59b, SUC_NE_lexicon_tags_2 },
    { 0xf5b9a59c, SUC_NE_lexicon_tags_2 },
    { 0x3301a59d, SUC_NE_lexicon_tags_2 },
    { 0xd99da59c, SUC_NE_lexicon_tags_2 },
    { 0x7f0da59e, SUC_NE_lexicon_tags_4 },
    { 0x5e35a59b, SUC_NE_lexicon_tags_2 },
    { 0x48aba5a0, SUC_NE_lexicon_tags_0 },
    { 0x2c97a59c, SUC_NE_lexicon_tags_2 },
    { 0x3a8fa596, SUC_NE_lexicon_tags_30 },
    { 0xd51ba5a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439ba5ab, SUC_NE_lexicon_tags_2 },
    { 0x9a3a5ac, SUC_NE_lexicon_tags_2 },
    { 0x4f71a5ad, SUC_NE_lexicon_tags_2 },
    { 0xefd7a5ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x648da5b1, SUC_NE_lexicon_tags_2 },
    { 0xe089a5b1, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb907a5b6, SUC_NE_lexicon_tags_2 },
    { 0x9b61a5b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x876fa5ba, SUC_NE_lexicon_tags_2 },
    { 0xfc63a5bb, SUC_NE_lexicon_tags_2 },
    { 0xdf83a5bb, SUC_NE_lexicon_tags_2 },
    { 0x2289a5bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafa5ca, SUC_NE_lexicon_tags_2 },
    { 0xd8c3a5cb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fba5cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7a5d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d3a5d6, SUC_NE_lexicon_tags_2 },
    { 0x2045a5d7, SUC_NE_lexicon_tags_2 },
    { 0x3e0fa5d8, SUC_NE_lexicon_tags_2 },
    { 0xaa63a5d8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5bf3a5db, SUC_NE_lexicon_tags_2 },
    { 0x8cd1a5dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aba5df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6afa5e4, SUC_NE_lexicon_tags_2 },
    { 0x75d5a5e4, SUC_NE_lexicon_tags_2 },
    { 0x157da5e6, SUC_NE_lexicon_tags_2 },
    { 0x95a1a5e6, SUC_NE_lexicon_tags_2 },
    { 0xa657a5e7, SUC_NE_lexicon_tags_2 },
    { 0x24afa5e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcba5ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c87a5ef, SUC_NE_lexicon_tags_2 },
    { 0xbe55a5ef, SUC_NE_lexicon_tags_2 },
    { 0x2c3da5f1, SUC_NE_lexicon_tags_2 },
    { 0xb311a5f2, SUC_NE_lexicon_tags_1 },
    { 0xe375a5f0, SUC_NE_lexicon_tags_2 },
    { 0xf0fda5f4, SUC_NE_lexicon_tags_2 },
    { 0xd107a5f4, SUC_NE_lexicon_tags_2 },
    { 0xf309a5f5, SUC_NE_lexicon_tags_2 },
    { 0xd457a5f7, SUC_NE_lexicon_tags_19 },
    { 0x762ba5f8, SUC_NE_lexicon_tags_3 },
    { 0xde19a5f9, SUC_NE_lexicon_tags_2 },
    { 0x4ad9a5f8, SUC_NE_lexicon_tags_2 },
    { 0xb23fa5fb, SUC_NE_lexicon_tags_2 },
    { 0xa7dda5f9, SUC_NE_lexicon_tags_1 },
    { 0x69e9a5fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9a602, SUC_NE_lexicon_tags_24 },
    { 0x28f3a603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf855a60a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc91a60f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x356ba612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7483a614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bba61a, SUC_NE_lexicon_tags_59 },
    { 0x168ba61a, SUC_NE_lexicon_tags_2 },
    { 0xb6e7a61a, SUC_NE_lexicon_tags_2 },
    { 0xf0e7a61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84eda61f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b9a623, SUC_NE_lexicon_tags_2 },
    { 0x3c09a623, SUC_NE_lexicon_tags_30 },
    { 0xc253a625, SUC_NE_lexicon_tags_2 },
    { 0x2c09a623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a13a629, SUC_NE_lexicon_tags_2 },
    { 0x4ca7a62a, SUC_NE_lexicon_tags_2 },
    { 0xcc4fa629, SUC_NE_lexicon_tags_2 },
    { 0xe5e3a62a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff5a632, SUC_NE_lexicon_tags_2 },
    { 0x5efa633, SUC_NE_lexicon_tags_17 },
    { 0x9a9da632, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5455a638, SUC_NE_lexicon_tags_2 },
    { 0xe8cda639, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x3941a63b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fa646, SUC_NE_lexicon_tags_2 },
    { 0x682ba647, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf67a649, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34ba656, SUC_NE_lexicon_tags_4 },
    { 0xfe5ba656, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdfa65b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x979da65d, SUC_NE_lexicon_tags_2 },
    { 0x81c3a65e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc23a660, SUC_NE_lexicon_tags_7 },
    { 0xba6fa660, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x999ba664, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x981a666, SUC_NE_lexicon_tags_2 },
    { 0x3b9ba667, SUC_NE_lexicon_tags_12 },
    { 0xad2fa668, SUC_NE_lexicon_tags_2 },
    { 0xe17a666, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ac7a66b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ada66e, SUC_NE_lexicon_tags_2 },
    { 0x2f43a66f, SUC_NE_lexicon_tags_4 },
    { 0xccd7a66f, SUC_NE_lexicon_tags_2 },
    { 0x65afa671, SUC_NE_lexicon_tags_2 },
    { 0x40f3a672, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03a675, SUC_NE_lexicon_tags_2 },
    { 0xb393a676, SUC_NE_lexicon_tags_2 },
    { 0x5283a677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7205a67a, SUC_NE_lexicon_tags_2 },
    { 0x7ceda67b, SUC_NE_lexicon_tags_2 },
    { 0xe9c1a67c, SUC_NE_lexicon_tags_32 },
    { 0xa28da67d, SUC_NE_lexicon_tags_2 },
    { 0x840fa67e, SUC_NE_lexicon_tags_2 },
    { 0xb62da67e, SUC_NE_lexicon_tags_2 },
    { 0xe583a680, SUC_NE_lexicon_tags_2 },
    { 0x195fa67e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8de1a683, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0da686, SUC_NE_lexicon_tags_0 },
    { 0xf209a686, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7dbda689, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x265ba68b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5243a68e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fa5a690, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x5cafa692, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cfa695, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b3a698, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf24fa69a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1ba69f, SUC_NE_lexicon_tags_2 },
    { 0x107ba6a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f7a6a3, SUC_NE_lexicon_tags_2 },
    { 0xc04fa6a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5d9a6a6, SUC_NE_lexicon_tags_2 },
    { 0xed75a6a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b77a6ad, SUC_NE_lexicon_tags_14 },
    { 0x2feba6ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3437a6b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bda6b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb99a6b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e5a6bb, SUC_NE_lexicon_tags_2 },
    { 0x5889a6bc, SUC_NE_lexicon_tags_2 },
    { 0xf755a6bd, SUC_NE_lexicon_tags_2 },
    { 0x67e9a6bd, SUC_NE_lexicon_tags_2 },
    { 0xfe79a6bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6fba6c1, SUC_NE_lexicon_tags_2 },
    { 0x1795a6c2, SUC_NE_lexicon_tags_2 },
    { 0x6d87a6c3, SUC_NE_lexicon_tags_2 },
    { 0xd933a6c4, SUC_NE_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0x320da6c7, SUC_NE_lexicon_tags_2 },
    { 0xcab1a6c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659a6cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dda6d0, SUC_NE_lexicon_tags_2 },
    { 0xd0dba6d1, SUC_NE_lexicon_tags_2 },
    { 0x1517a6d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253fa6d6, SUC_NE_lexicon_tags_2 },
    { 0xb5d3a6d7, SUC_NE_lexicon_tags_2 },
    { 0xdb77a6d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b7a6dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe999a6e6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf75da6e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14cba6ea, SUC_NE_lexicon_tags_2 },
    { 0x71a3a6eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e11a6ee, SUC_NE_lexicon_tags_2 },
    { 0xac8fa6ef, SUC_NE_lexicon_tags_2 },
    { 0x25cba6ee, SUC_NE_lexicon_tags_2 },
    { 0x6081a6f1, SUC_NE_lexicon_tags_2 },
    { 0xd0c5a6f1, SUC_NE_lexicon_tags_2 },
    { 0xd2dba6f3, SUC_NE_lexicon_tags_2 },
    { 0x1471a6f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf419a6f7, SUC_NE_lexicon_tags_2 },
    { 0x8107a6f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7635a6fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f45a6fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dfa705, SUC_NE_lexicon_tags_2 },
    { 0xa185a706, SUC_NE_lexicon_tags_2 },
    { 0x96afa707, SUC_NE_lexicon_tags_2 },
    { 0x6813a708, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cffa712, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe149a715, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9b5a717, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd3a71a, SUC_NE_lexicon_tags_2 },
    { 0x6341a71b, SUC_NE_lexicon_tags_2 },
    { 0x764ba71c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeeda71e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f1a724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bfa728, SUC_NE_lexicon_tags_2 },
    { 0xd305a728, SUC_NE_lexicon_tags_17 },
    { 0xcfb7a728, SUC_NE_lexicon_tags_2 },
    { 0xa83ba72b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ba731, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8ba735, SUC_NE_lexicon_tags_2 },
    { 0x30efa736, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d9a739, SUC_NE_lexicon_tags_1 },
    { 0x1f4ba739, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd305a73c, SUC_NE_lexicon_tags_17 },
    { 0x744ba73d, SUC_NE_lexicon_tags_3 },
    { 0x895ba73c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9207a742, SUC_NE_lexicon_tags_2 },
    { 0x54d1a742, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d5a746, SUC_NE_lexicon_tags_2 },
    { 0x3101a747, SUC_NE_lexicon_tags_2 },
    { 0xcbbda748, SUC_NE_lexicon_tags_2 },
    { 0xa2e5a748, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaafa750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc959a752, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dda758, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb21a75b, SUC_NE_lexicon_tags_2 },
    { 0x1afba75c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c3a75f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8471a764, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x895a766, SUC_NE_lexicon_tags_2 },
    { 0x16d7a767, SUC_NE_lexicon_tags_2 },
    { 0x97e3a768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6297a76a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3da76d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d13a771, SUC_NE_lexicon_tags_0 },
    { 0xad9da772, SUC_NE_lexicon_tags_2 },
    { 0x56efa773, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5513a776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967da77a, SUC_NE_lexicon_tags_2 },
    { 0x1d11a77b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a77e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe959a780, SUC_NE_lexicon_tags_2 },
    { 0xad7a781, SUC_NE_lexicon_tags_2 },
    { 0x6c9ba782, SUC_NE_lexicon_tags_2 },
    { 0xa5a7a783, SUC_NE_lexicon_tags_2 },
    { 0x2aeba782, SUC_NE_lexicon_tags_2 },
    { 0xe607a785, SUC_NE_lexicon_tags_2 },
    { 0x74a1a786, SUC_NE_lexicon_tags_7 },
    { 0x33e9a781, SUC_NE_lexicon_tags_2 },
    { 0xd611a788, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5729a78b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bffa78d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xea63a78f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb311a796, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x192ba799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x277fa79c, SUC_NE_lexicon_tags_2 },
    { 0x3987a79d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958fa7a1, SUC_NE_lexicon_tags_2 },
    { 0xe869a7a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedfa7a5, SUC_NE_lexicon_tags_2 },
    { 0x4bbba7a6, SUC_NE_lexicon_tags_2 },
    { 0xda1da7a5, SUC_NE_lexicon_tags_1 },
    { 0x7381a7a6, SUC_NE_lexicon_tags_2 },
    { 0x2733a7a5, SUC_NE_lexicon_tags_2 },
    { 0x95bba7a6, SUC_NE_lexicon_tags_2 },
    { 0x1717a7ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcffa7ad, SUC_NE_lexicon_tags_2 },
    { 0x12a5a7ad, SUC_NE_lexicon_tags_2 },
    { 0x99e3a7af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f7a7b1, SUC_NE_lexicon_tags_2 },
    { 0xdfd7a7b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85da7b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a5a7bc, SUC_NE_lexicon_tags_2 },
    { 0xa61a7bd, SUC_NE_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797a7c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a7a7c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eba7cc, SUC_NE_lexicon_tags_2 },
    { 0x3e9a7cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc767a7cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1a7d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab1a7d7, SUC_NE_lexicon_tags_4 },
    { 0xb9bda7d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb1fa7da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bda7de, SUC_NE_lexicon_tags_2 },
    { 0xcbdfa7df, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7a7e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d3a7e5, SUC_NE_lexicon_tags_2 },
    { 0x976fa7e6, SUC_NE_lexicon_tags_2 },
    { 0x8fdba7e7, SUC_NE_lexicon_tags_2 },
    { 0xb603a7e8, SUC_NE_lexicon_tags_2 },
    { 0x3721a7e5, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6da7ed, SUC_NE_lexicon_tags_2 },
    { 0x8e9fa7ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb33fa7f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x246fa7f2, SUC_NE_lexicon_tags_2 },
    { 0xf223a7f3, SUC_NE_lexicon_tags_2 },
    { 0xf8e5a7f4, SUC_NE_lexicon_tags_2 },
    { 0xd345a7f5, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab57a7f9, SUC_NE_lexicon_tags_2 },
    { 0x2fe1a7fa, SUC_NE_lexicon_tags_209 },
    { 0xfd99a7fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa349a7fe, SUC_NE_lexicon_tags_2 },
    { 0xc6c1a7fe, SUC_NE_lexicon_tags_2 },
    { 0x8dc1a800, SUC_NE_lexicon_tags_7 },
    { 0x5749a800, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5bfa803, SUC_NE_lexicon_tags_1 },
    { 0xf031a804, SUC_NE_lexicon_tags_5 },
    { 0xcd73a805, SUC_NE_lexicon_tags_2 },
    { 0xc569a806, SUC_NE_lexicon_tags_2 },
    { 0xffdfa805, SUC_NE_lexicon_tags_2 },
    { 0x65b1a808, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xc723a80a, SUC_NE_lexicon_tags_2 },
    { 0xfcc3a80b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7a811, SUC_NE_lexicon_tags_18 },
    { 0x9f3fa812, SUC_NE_lexicon_tags_2 },
    { 0xd21da812, SUC_NE_lexicon_tags_2 },
    { 0xffd9a814, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8209a816, SUC_NE_lexicon_tags_7 },
    { 0x1195a817, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196fa81c, SUC_NE_lexicon_tags_2 },
    { 0x1a3ba81d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c93a821, SUC_NE_lexicon_tags_17 },
    { 0xf073a821, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3ba827, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfa82d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x478ba82f, SUC_NE_lexicon_tags_13 },
    { 0x9e45a830, SUC_NE_lexicon_tags_2 },
    { 0x47b1a830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x995a833, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b3a837, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9621a83b, SUC_NE_lexicon_tags_7 },
    { 0x88ffa83b, SUC_NE_lexicon_tags_2 },
    { 0xc393a83d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b1a842, SUC_NE_lexicon_tags_2 },
    { 0x5b61a843, SUC_NE_lexicon_tags_2 },
    { 0xa79ba844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38eba846, SUC_NE_lexicon_tags_2 },
    { 0x356fa847, SUC_NE_lexicon_tags_2 },
    { 0xa919a848, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x57cda84a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc23a851, SUC_NE_lexicon_tags_2 },
    { 0xca65a852, SUC_NE_lexicon_tags_2 },
    { 0x44f5a853, SUC_NE_lexicon_tags_2 },
    { 0xff23a854, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x668fa856, SUC_NE_lexicon_tags_2 },
    { 0x3a41a857, SUC_NE_lexicon_tags_2 },
    { 0x6d65a858, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea89a85c, SUC_NE_lexicon_tags_5 },
    { 0x1283a85c, SUC_NE_lexicon_tags_2 },
    { 0x9913a85e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a1a862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa013a865, SUC_NE_lexicon_tags_2 },
    { 0xd4afa866, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xca25a869, SUC_NE_lexicon_tags_2 },
    { 0x2757a86a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x84fa86c, SUC_NE_lexicon_tags_2 },
    { 0x4d9da86d, SUC_NE_lexicon_tags_2 },
    { 0x5effa86c, SUC_NE_lexicon_tags_2 },
    { 0xdf03a86e, SUC_NE_lexicon_tags_2 },
    { 0xaf4ba870, SUC_NE_lexicon_tags_2 },
    { 0x5569a871, SUC_NE_lexicon_tags_13 },
    { 0x5b15a872, SUC_NE_lexicon_tags_2 },
    { 0xc361a872, SUC_NE_lexicon_tags_2 },
    { 0x5b5ba874, SUC_NE_lexicon_tags_2 },
    { 0xe425a875, SUC_NE_lexicon_tags_7 },
    { 0x6725a870, SUC_NE_lexicon_tags_2 },
    { 0x8023a872, SUC_NE_lexicon_tags_2 },
    { 0xf535a871, SUC_NE_lexicon_tags_2 },
    { 0x22f1a879, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af5a881, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a11a883, SUC_NE_lexicon_tags_2 },
    { 0x6219a883, SUC_NE_lexicon_tags_2 },
    { 0xac13a885, SUC_NE_lexicon_tags_2 },
    { 0xb069a886, SUC_NE_lexicon_tags_17 },
    { 0x8cefa887, SUC_NE_lexicon_tags_2 },
    { 0x272da887, SUC_NE_lexicon_tags_2 },
    { 0xc87da888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc3a88d, SUC_NE_lexicon_tags_2 },
    { 0x243ba88d, SUC_NE_lexicon_tags_2 },
    { 0xb3bba88f, SUC_NE_lexicon_tags_2 },
    { 0x8465a88d, SUC_NE_lexicon_tags_2 },
    { 0xe0afa891, SUC_NE_lexicon_tags_2 },
    { 0x967fa88f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0fa895, SUC_NE_lexicon_tags_2 },
    { 0x44e3a895, SUC_NE_lexicon_tags_223 },
    { 0, NULL },
    { 0x310da898, SUC_NE_lexicon_tags_2 },
    { 0x75b3a899, SUC_NE_lexicon_tags_2 },
    { 0x656fa89a, SUC_NE_lexicon_tags_2 },
    { 0xccb5a89b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe31da89d, SUC_NE_lexicon_tags_2 },
    { 0xbfdba89e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e1a8a3, SUC_NE_lexicon_tags_2 },
    { 0x6379a8a4, SUC_NE_lexicon_tags_1 },
    { 0xba2da8a5, SUC_NE_lexicon_tags_2 },
    { 0xde85a8a5, SUC_NE_lexicon_tags_2 },
    { 0x3321a8a7, SUC_NE_lexicon_tags_2 },
    { 0x2f7da8a4, SUC_NE_lexicon_tags_2 },
    { 0xd033a8a6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x37aba8ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c85a8ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc759a8b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a03a8b4, SUC_NE_lexicon_tags_2 },
    { 0x945ba8b5, SUC_NE_lexicon_tags_2 },
    { 0x8065a8b6, SUC_NE_lexicon_tags_2 },
    { 0xfd65a8b5, SUC_NE_lexicon_tags_2 },
    { 0xb607a8b8, SUC_NE_lexicon_tags_2 },
    { 0x2395a8b9, SUC_NE_lexicon_tags_2 },
    { 0x28e9a8ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe09a8bc, SUC_NE_lexicon_tags_2 },
    { 0xb411a8bd, SUC_NE_lexicon_tags_2 },
    { 0x9cb5a8bd, SUC_NE_lexicon_tags_2 },
    { 0xc3ffa8bd, SUC_NE_lexicon_tags_2 },
    { 0x1f2fa8c0, SUC_NE_lexicon_tags_2 },
    { 0x92cda8c1, SUC_NE_lexicon_tags_3 },
    { 0xed7ba8c1, SUC_NE_lexicon_tags_2 },
    { 0x87d1a8c1, SUC_NE_lexicon_tags_2 },
    { 0x85b9a8c4, SUC_NE_lexicon_tags_1 },
    { 0x1aba8c5, SUC_NE_lexicon_tags_2 },
    { 0x38a7a8c1, SUC_NE_lexicon_tags_2 },
    { 0xc221a8c1, SUC_NE_lexicon_tags_2 },
    { 0xa7f7a8c7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8dffa8ca, SUC_NE_lexicon_tags_2 },
    { 0x3b03a8cb, SUC_NE_lexicon_tags_2 },
    { 0x7a1ba8ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e9a8cf, SUC_NE_lexicon_tags_2 },
    { 0xca83a8d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cda8d4, SUC_NE_lexicon_tags_2 },
    { 0x1a15a8d4, SUC_NE_lexicon_tags_2 },
    { 0x4ff3a8d6, SUC_NE_lexicon_tags_2 },
    { 0x6cd7a8d7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c1a8dd, SUC_NE_lexicon_tags_14 },
    { 0x2929a8dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef53a8e5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb609a8e8, SUC_NE_lexicon_tags_2 },
    { 0x9565a8e9, SUC_NE_lexicon_tags_2 },
    { 0x32cfa8ea, SUC_NE_lexicon_tags_2 },
    { 0x90bba8eb, SUC_NE_lexicon_tags_2 },
    { 0x6aeba8ec, SUC_NE_lexicon_tags_2 },
    { 0x8b03a8ed, SUC_NE_lexicon_tags_2 },
    { 0xab35a8ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d15a8f1, SUC_NE_lexicon_tags_2 },
    { 0x4b19a8f2, SUC_NE_lexicon_tags_2 },
    { 0xf2e1a8f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09fa8f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1071a8ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da902, SUC_NE_lexicon_tags_2 },
    { 0x7033a902, SUC_NE_lexicon_tags_2 },
    { 0x69cba904, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2313a906, SUC_NE_lexicon_tags_2 },
    { 0xa2c9a907, SUC_NE_lexicon_tags_2 },
    { 0xf627a908, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xde57a90b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x26bba90d, SUC_NE_lexicon_tags_2 },
    { 0x8ea9a90d, SUC_NE_lexicon_tags_2 },
    { 0x945a90e, SUC_NE_lexicon_tags_2 },
    { 0x8e91a910, SUC_NE_lexicon_tags_2 },
    { 0xb63ba911, SUC_NE_lexicon_tags_2 },
    { 0xc763a910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a9a915, SUC_NE_lexicon_tags_2 },
    { 0x8db7a915, SUC_NE_lexicon_tags_4 },
    { 0x8801a917, SUC_NE_lexicon_tags_2 },
    { 0xebb9a915, SUC_NE_lexicon_tags_2 },
    { 0xdd27a919, SUC_NE_lexicon_tags_2 },
    { 0x68e1a91a, SUC_NE_lexicon_tags_2 },
    { 0xa647a91b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb61a91d, SUC_NE_lexicon_tags_2 },
    { 0xa87fa91d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723da922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aefa924, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82da928, SUC_NE_lexicon_tags_2 },
    { 0xff7ba928, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9215a92e, SUC_NE_lexicon_tags_2 },
    { 0x7497a92f, SUC_NE_lexicon_tags_4 },
    { 0x2375a92f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d41a933, SUC_NE_lexicon_tags_2 },
    { 0xb347a934, SUC_NE_lexicon_tags_2 },
    { 0xbdb9a935, SUC_NE_lexicon_tags_2 },
    { 0x5951a935, SUC_NE_lexicon_tags_2 },
    { 0xc47da937, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d9a939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b35a93b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa93e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb997a941, SUC_NE_lexicon_tags_25 },
    { 0x126da941, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x451fa945, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff3a948, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bda94d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd281a954, SUC_NE_lexicon_tags_2 },
    { 0xc75ba955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba2ba957, SUC_NE_lexicon_tags_2 },
    { 0xe469a958, SUC_NE_lexicon_tags_7 },
    { 0xe2c5a959, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2547a95e, SUC_NE_lexicon_tags_40 },
    { 0xf797a95f, SUC_NE_lexicon_tags_2 },
    { 0x40ada960, SUC_NE_lexicon_tags_2 },
    { 0x233ba960, SUC_NE_lexicon_tags_24 },
    { 0x5f2ba95f, SUC_NE_lexicon_tags_2 },
    { 0xb78ba960, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x4301a965, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2afa967, SUC_NE_lexicon_tags_2 },
    { 0x1b61a968, SUC_NE_lexicon_tags_2 },
    { 0xfb2ba968, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fa970, SUC_NE_lexicon_tags_147 },
    { 0, NULL },
    { 0xe9b5a972, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fba976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4fa979, SUC_NE_lexicon_tags_2 },
    { 0xfb03a979, SUC_NE_lexicon_tags_2 },
    { 0x6ca5a97b, SUC_NE_lexicon_tags_2 },
    { 0x6b11a97c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x281ba97f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3fa982, SUC_NE_lexicon_tags_2 },
    { 0x90f9a983, SUC_NE_lexicon_tags_2 },
    { 0xf853a984, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x255fa987, SUC_NE_lexicon_tags_2 },
    { 0x24f5a988, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf807a98a, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a01a98f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bfa994, SUC_NE_lexicon_tags_7 },
    { 0xd42fa994, SUC_NE_lexicon_tags_2 },
    { 0xc173a994, SUC_NE_lexicon_tags_2 },
    { 0xfc17a997, SUC_NE_lexicon_tags_2 },
    { 0xf045a998, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac3a99b, SUC_NE_lexicon_tags_2 },
    { 0xfd8fa99b, SUC_NE_lexicon_tags_2 },
    { 0x14e7a99d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x410ba99f, SUC_NE_lexicon_tags_2 },
    { 0xc0f9a99f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9da9a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5efa9ad, SUC_NE_lexicon_tags_22 },
    { 0x89fda9ad, SUC_NE_lexicon_tags_2 },
    { 0xd489a9ad, SUC_NE_lexicon_tags_7 },
    { 0x8545a9ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6fa9b5, SUC_NE_lexicon_tags_2 },
    { 0xf637a9b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb721a9b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeb7a9ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5195a9bc, SUC_NE_lexicon_tags_2 },
    { 0xd8f3a9bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e7a9c1, SUC_NE_lexicon_tags_2 },
    { 0x5a69a9c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6611a9c5, SUC_NE_lexicon_tags_2 },
    { 0x60b1a9c5, SUC_NE_lexicon_tags_2 },
    { 0x57d7a9c7, SUC_NE_lexicon_tags_40 },
    { 0x1703a9c8, SUC_NE_lexicon_tags_2 },
    { 0x1241a9c9, SUC_NE_lexicon_tags_2 },
    { 0x14b7a9ca, SUC_NE_lexicon_tags_2 },
    { 0x69a7a9cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba9d0, SUC_NE_lexicon_tags_5 },
    { 0xcef5a9d1, SUC_NE_lexicon_tags_2 },
    { 0xb951a9d2, SUC_NE_lexicon_tags_2 },
    { 0x27efa9d3, SUC_NE_lexicon_tags_2 },
    { 0xfcd1a9d4, SUC_NE_lexicon_tags_2 },
    { 0xa745a9d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6da9da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76bda9dc, SUC_NE_lexicon_tags_2 },
    { 0xdacda9dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5fa9e2, SUC_NE_lexicon_tags_2 },
    { 0xf4d9a9e3, SUC_NE_lexicon_tags_2 },
    { 0xb431a9e4, SUC_NE_lexicon_tags_2 },
    { 0x7619a9e5, SUC_NE_lexicon_tags_4 },
    { 0x2a75a9e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85cfa9e8, SUC_NE_lexicon_tags_2 },
    { 0xfd2ba9e9, SUC_NE_lexicon_tags_2 },
    { 0x7337a9e9, SUC_NE_lexicon_tags_2 },
    { 0x1e19a9eb, SUC_NE_lexicon_tags_7 },
    { 0xa861a9ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f65a9f2, SUC_NE_lexicon_tags_2 },
    { 0xfd13a9f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd11da9f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd941a9f7, SUC_NE_lexicon_tags_2 },
    { 0x7825a9f8, SUC_NE_lexicon_tags_2 },
    { 0x9c2da9f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb189a9fb, SUC_NE_lexicon_tags_1 },
    { 0x2745a9fb, SUC_NE_lexicon_tags_2 },
    { 0x5ba9a9fd, SUC_NE_lexicon_tags_2 },
    { 0xf721a9fe, SUC_NE_lexicon_tags_2 },
    { 0x5941a9ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9385aa02, SUC_NE_lexicon_tags_2 },
    { 0x1c3faa03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b2daa05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45e1aa07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6729aa0a, SUC_NE_lexicon_tags_2 },
    { 0x7b75aa0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9433aa0d, SUC_NE_lexicon_tags_2 },
    { 0x5565aa0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6faa11, SUC_NE_lexicon_tags_2 },
    { 0x5415aa12, SUC_NE_lexicon_tags_2 },
    { 0x6423aa13, SUC_NE_lexicon_tags_2 },
    { 0xac8daa13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f9aa17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7453aa19, SUC_NE_lexicon_tags_2 },
    { 0x4333aa1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d11aa1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb0faa1e, SUC_NE_lexicon_tags_4 },
    { 0x6945aa1e, SUC_NE_lexicon_tags_2 },
    { 0xa86baa20, SUC_NE_lexicon_tags_2 },
    { 0xbde7aa21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5915aa23, SUC_NE_lexicon_tags_2 },
    { 0xd93baa24, SUC_NE_lexicon_tags_2 },
    { 0xa001aa25, SUC_NE_lexicon_tags_2 },
    { 0x371daa25, SUC_NE_lexicon_tags_2 },
    { 0xb67daa27, SUC_NE_lexicon_tags_124 },
    { 0x2347aa25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479faa2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7aa30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb9aa35, SUC_NE_lexicon_tags_2 },
    { 0x9cddaa36, SUC_NE_lexicon_tags_2 },
    { 0xf3daa36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cedaa3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d6daa3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc391aa3f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1aa42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa04daa44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe3aa48, SUC_NE_lexicon_tags_2 },
    { 0xdbf9aa49, SUC_NE_lexicon_tags_7 },
    { 0xdebbaa4a, SUC_NE_lexicon_tags_2 },
    { 0xe003aa4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35fbaa4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce2daa4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53baa53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5241aa58, SUC_NE_lexicon_tags_2 },
    { 0x73ddaa59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7baa5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13aa61, SUC_NE_lexicon_tags_2 },
    { 0x9fb5aa62, SUC_NE_lexicon_tags_2 },
    { 0xb841aa63, SUC_NE_lexicon_tags_2 },
    { 0x7e83aa64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c29aa66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa799aa6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88adaa6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4625aa74, SUC_NE_lexicon_tags_2 },
    { 0x760faa75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7647aa77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f27aa79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fd7aa7b, SUC_NE_lexicon_tags_2 },
    { 0x2c2daa7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d5aa80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e1aa84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa581aa86, SUC_NE_lexicon_tags_2 },
    { 0xc9baa87, SUC_NE_lexicon_tags_3 },
    { 0x586daa86, SUC_NE_lexicon_tags_2 },
    { 0x6bd5aa89, SUC_NE_lexicon_tags_2 },
    { 0x53fbaa88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8365aa8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcbaa92, SUC_NE_lexicon_tags_7 },
    { 0x7fb5aa93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e09aa95, SUC_NE_lexicon_tags_65 },
    { 0, NULL },
    { 0x46bfaa97, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x6b79aa99, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00daa9f, SUC_NE_lexicon_tags_7 },
    { 0x5d5baa9f, SUC_NE_lexicon_tags_2 },
    { 0xc813aaa1, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb5aaa4, SUC_NE_lexicon_tags_2 },
    { 0x5753aaa4, SUC_NE_lexicon_tags_2 },
    { 0xa8f9aaa6, SUC_NE_lexicon_tags_2 },
    { 0x8a3baaa5, SUC_NE_lexicon_tags_2 },
    { 0xfe91aaa5, SUC_NE_lexicon_tags_2 },
    { 0x2a37aaa9, SUC_NE_lexicon_tags_2 },
    { 0x9065aaaa, SUC_NE_lexicon_tags_2 },
    { 0xebb9aaab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dedaaaf, SUC_NE_lexicon_tags_2 },
    { 0x40cdaab0, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a3aab3, SUC_NE_lexicon_tags_2 },
    { 0x4f3daab4, SUC_NE_lexicon_tags_7 },
    { 0xea29aab5, SUC_NE_lexicon_tags_2 },
    { 0x49f1aab6, SUC_NE_lexicon_tags_2 },
    { 0xf449aab7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc463aabb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edaabf, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a47aac2, SUC_NE_lexicon_tags_2 },
    { 0xc1fbaac3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbbaac7, SUC_NE_lexicon_tags_2 },
    { 0x4b19aac8, SUC_NE_lexicon_tags_2 },
    { 0x4385aac9, SUC_NE_lexicon_tags_2 },
    { 0x25a1aac9, SUC_NE_lexicon_tags_2 },
    { 0xcffdaacb, SUC_NE_lexicon_tags_2 },
    { 0xb2ebaacc, SUC_NE_lexicon_tags_2 },
    { 0x3f1faacd, SUC_NE_lexicon_tags_2 },
    { 0xd08daacd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3aad1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9faad4, SUC_NE_lexicon_tags_2 },
    { 0xb7aad5, SUC_NE_lexicon_tags_2 },
    { 0xb635aad6, SUC_NE_lexicon_tags_2 },
    { 0x962faad5, SUC_NE_lexicon_tags_1 },
    { 0xb11daad8, SUC_NE_lexicon_tags_2 },
    { 0xf3a5aad6, SUC_NE_lexicon_tags_2 },
    { 0xdc53aada, SUC_NE_lexicon_tags_2 },
    { 0x6173aadb, SUC_NE_lexicon_tags_2 },
    { 0xc95baad7, SUC_NE_lexicon_tags_2 },
    { 0x5b87aadd, SUC_NE_lexicon_tags_2 },
    { 0xb909aad5, SUC_NE_lexicon_tags_2 },
    { 0x19edaadf, SUC_NE_lexicon_tags_2 },
    { 0x3b19aae0, SUC_NE_lexicon_tags_2 },
    { 0x593faadf, SUC_NE_lexicon_tags_2 },
    { 0x8f41aae0, SUC_NE_lexicon_tags_2 },
    { 0xdb59aad5, SUC_NE_lexicon_tags_2 },
    { 0x9e5aae4, SUC_NE_lexicon_tags_2 },
    { 0x7bf5aae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2543aae9, SUC_NE_lexicon_tags_205 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44afaaf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412baaf5, SUC_NE_lexicon_tags_2 },
    { 0x1f85aaf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04daafa, SUC_NE_lexicon_tags_2 },
    { 0x3ee7aafb, SUC_NE_lexicon_tags_2 },
    { 0xc62baafa, SUC_NE_lexicon_tags_2 },
    { 0x2a5faafd, SUC_NE_lexicon_tags_2 },
    { 0xd95daafc, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0xcbe9ab00, SUC_NE_lexicon_tags_2 },
    { 0xf495ab01, SUC_NE_lexicon_tags_2 },
    { 0xa4b9ab01, SUC_NE_lexicon_tags_2 },
    { 0x7dd3ab02, SUC_NE_lexicon_tags_17 },
    { 0x8e3dab02, SUC_NE_lexicon_tags_2 },
    { 0x417fab02, SUC_NE_lexicon_tags_2 },
    { 0x923ab06, SUC_NE_lexicon_tags_2 },
    { 0x7fb7ab07, SUC_NE_lexicon_tags_2 },
    { 0x9e3dab07, SUC_NE_lexicon_tags_2 },
    { 0xbfa5ab04, SUC_NE_lexicon_tags_2 },
    { 0xb2edab0a, SUC_NE_lexicon_tags_2 },
    { 0x466fab0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f1ab0e, SUC_NE_lexicon_tags_2 },
    { 0x7c37ab0f, SUC_NE_lexicon_tags_2 },
    { 0xd0bfab10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d4bab12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e1ab15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67e3ab17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe815ab19, SUC_NE_lexicon_tags_2 },
    { 0xe68fab19, SUC_NE_lexicon_tags_2 },
    { 0xdeafab1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ad5ab1d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff31ab22, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cdab27, SUC_NE_lexicon_tags_2 },
    { 0xc565ab28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dddab2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x681bab2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dd1ab30, SUC_NE_lexicon_tags_2 },
    { 0x5195ab30, SUC_NE_lexicon_tags_2 },
    { 0xc445ab32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca23ab35, SUC_NE_lexicon_tags_2 },
    { 0x4fc1ab36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc361ab39, SUC_NE_lexicon_tags_2 },
    { 0xc505ab3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x487fab3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f7ab3e, SUC_NE_lexicon_tags_2 },
    { 0x329ab3f, SUC_NE_lexicon_tags_2 },
    { 0xffedab40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ddab43, SUC_NE_lexicon_tags_2 },
    { 0xf5e3ab44, SUC_NE_lexicon_tags_2 },
    { 0x7983ab45, SUC_NE_lexicon_tags_2 },
    { 0x9887ab46, SUC_NE_lexicon_tags_2 },
    { 0x4457ab47, SUC_NE_lexicon_tags_2 },
    { 0xf2c7ab48, SUC_NE_lexicon_tags_2 },
    { 0x355dab44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45ab4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x540dab4f, SUC_NE_lexicon_tags_2 },
    { 0xdfd9ab4f, SUC_NE_lexicon_tags_2 },
    { 0xd357ab51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5f7ab53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc61ab55, SUC_NE_lexicon_tags_2 },
    { 0xdaedab56, SUC_NE_lexicon_tags_2 },
    { 0x6309ab57, SUC_NE_lexicon_tags_2 },
    { 0xe80fab55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0dab5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabfab63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624fab67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa841ab6c, SUC_NE_lexicon_tags_32 },
    { 0xe943ab6d, SUC_NE_lexicon_tags_2 },
    { 0x9729ab6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215dab74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde59ab76, SUC_NE_lexicon_tags_2 },
    { 0xec7bab76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6a7ab79, SUC_NE_lexicon_tags_2 },
    { 0x943bab79, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9071ab7d, SUC_NE_lexicon_tags_2 },
    { 0x9a77ab7e, SUC_NE_lexicon_tags_2 },
    { 0xff1ab7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abfab82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d7ab85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f59ab87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x865dab8a, SUC_NE_lexicon_tags_187 },
    { 0, NULL },
    { 0xf165ab8c, SUC_NE_lexicon_tags_2 },
    { 0x4cc5ab8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c63ab91, SUC_NE_lexicon_tags_2 },
    { 0xb5d5ab91, SUC_NE_lexicon_tags_2 },
    { 0x42c5ab91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc7ab97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2d5ab99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e1ab9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb13aba0, SUC_NE_lexicon_tags_2 },
    { 0x481daba1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7aba5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24daba9, SUC_NE_lexicon_tags_4 },
    { 0x2a13abaa, SUC_NE_lexicon_tags_2 },
    { 0xc271abab, SUC_NE_lexicon_tags_7 },
    { 0x649dabac, SUC_NE_lexicon_tags_2 },
    { 0xb9c3abac, SUC_NE_lexicon_tags_2 },
    { 0xa6e5abab, SUC_NE_lexicon_tags_2 },
    { 0xf469abaa, SUC_NE_lexicon_tags_2 },
    { 0xb585abad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e3dabb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11d1abb4, SUC_NE_lexicon_tags_1 },
    { 0xfd71abb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b1fabb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493abbc, SUC_NE_lexicon_tags_7 },
    { 0x9137abbd, SUC_NE_lexicon_tags_2 },
    { 0xaebdabbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x615dabc1, SUC_NE_lexicon_tags_2 },
    { 0x2cf9abc2, SUC_NE_lexicon_tags_198 },
    { 0x7919abc3, SUC_NE_lexicon_tags_2 },
    { 0x709dabc4, SUC_NE_lexicon_tags_2 },
    { 0x4a5babc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ac3abc7, SUC_NE_lexicon_tags_2 },
    { 0xd0cfabc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f05abd1, SUC_NE_lexicon_tags_2 },
    { 0x336fabd2, SUC_NE_lexicon_tags_2 },
    { 0xadb1abd2, SUC_NE_lexicon_tags_2 },
    { 0xe7c7abd4, SUC_NE_lexicon_tags_3 },
    { 0xeca7abd5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e9abd8, SUC_NE_lexicon_tags_2 },
    { 0x27fbabd9, SUC_NE_lexicon_tags_2 },
    { 0x6b6fabd9, SUC_NE_lexicon_tags_2 },
    { 0xd14babdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6465abde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75babe1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c7abe6, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb7abe9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4127abeb, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa007abef, SUC_NE_lexicon_tags_2 },
    { 0x6e1dabf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4de7abf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dfabf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc211abfc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa661abfe, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x7ee1ac00, SUC_NE_lexicon_tags_2 },
    { 0xd969ac00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd05ac05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769bac0a, SUC_NE_lexicon_tags_2 },
    { 0xf0b1ac0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f49ac0d, SUC_NE_lexicon_tags_2 },
    { 0x9adac0e, SUC_NE_lexicon_tags_2 },
    { 0x94d9ac0f, SUC_NE_lexicon_tags_7 },
    { 0xb82dac0d, SUC_NE_lexicon_tags_2 },
    { 0xdc11ac11, SUC_NE_lexicon_tags_2 },
    { 0x5b13ac0e, SUC_NE_lexicon_tags_81 },
    { 0xd65fac13, SUC_NE_lexicon_tags_2 },
    { 0x6593ac13, SUC_NE_lexicon_tags_2 },
    { 0xcde1ac13, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ac18, SUC_NE_lexicon_tags_2 },
    { 0xb4ebac18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe897ac1c, SUC_NE_lexicon_tags_2 },
    { 0xfc39ac1c, SUC_NE_lexicon_tags_2 },
    { 0xbcf5ac1e, SUC_NE_lexicon_tags_2 },
    { 0xd4c1ac1f, SUC_NE_lexicon_tags_2 },
    { 0xb95bac1f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3195ac22, SUC_NE_lexicon_tags_5 },
    { 0xc715ac23, SUC_NE_lexicon_tags_2 },
    { 0xe797ac22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x824dac27, SUC_NE_lexicon_tags_2 },
    { 0x8d8fac27, SUC_NE_lexicon_tags_36 },
    { 0x2801ac29, SUC_NE_lexicon_tags_2 },
    { 0x1a47ac29, SUC_NE_lexicon_tags_2 },
    { 0x580dac2b, SUC_NE_lexicon_tags_2 },
    { 0x7a51ac2a, SUC_NE_lexicon_tags_2 },
    { 0xfe4fac27, SUC_NE_lexicon_tags_2 },
    { 0x8101ac2e, SUC_NE_lexicon_tags_2 },
    { 0x7e71ac2f, SUC_NE_lexicon_tags_2 },
    { 0xb40dac30, SUC_NE_lexicon_tags_2 },
    { 0x81abac30, SUC_NE_lexicon_tags_2 },
    { 0x3bd5ac2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x145dac35, SUC_NE_lexicon_tags_2 },
    { 0xfea5ac36, SUC_NE_lexicon_tags_2 },
    { 0x3c37ac35, SUC_NE_lexicon_tags_2 },
    { 0x6123ac36, SUC_NE_lexicon_tags_3 },
    { 0x529bac36, SUC_NE_lexicon_tags_2 },
    { 0x6dabac37, SUC_NE_lexicon_tags_2 },
    { 0x7833ac3b, SUC_NE_lexicon_tags_4 },
    { 0xcb4dac3b, SUC_NE_lexicon_tags_2 },
    { 0x76ffac3d, SUC_NE_lexicon_tags_2 },
    { 0x192dac36, SUC_NE_lexicon_tags_2 },
    { 0xe46dac3f, SUC_NE_lexicon_tags_2 },
    { 0x907bac40, SUC_NE_lexicon_tags_2 },
    { 0x54c1ac3a, SUC_NE_lexicon_tags_2 },
    { 0x6e59ac39, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea1ac45, SUC_NE_lexicon_tags_30 },
    { 0x6a07ac45, SUC_NE_lexicon_tags_2 },
    { 0x42e1ac47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f63ac49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77b7ac4b, SUC_NE_lexicon_tags_2 },
    { 0x4e35ac4c, SUC_NE_lexicon_tags_2 },
    { 0x9499ac4d, SUC_NE_lexicon_tags_2 },
    { 0xe3a3ac4d, SUC_NE_lexicon_tags_2 },
    { 0xf4e9ac4e, SUC_NE_lexicon_tags_2 },
    { 0xf7b1ac4c, SUC_NE_lexicon_tags_2 },
    { 0xcc93ac4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf1ac54, SUC_NE_lexicon_tags_2 },
    { 0x90fac55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcddbac57, SUC_NE_lexicon_tags_2 },
    { 0x25edac58, SUC_NE_lexicon_tags_2 },
    { 0x48e5ac59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b87ac5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86fac62, SUC_NE_lexicon_tags_2 },
    { 0x28c5ac62, SUC_NE_lexicon_tags_7 },
    { 0xd3efac64, SUC_NE_lexicon_tags_2 },
    { 0xca19ac62, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0xb94fac67, SUC_NE_lexicon_tags_109 },
    { 0xe577ac68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace3ac6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea5ac6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7409ac71, SUC_NE_lexicon_tags_2 },
    { 0xbb45ac72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe419ac78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9963ac7a, SUC_NE_lexicon_tags_2 },
    { 0x921ac7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5d1ac7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9997ac81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6405ac83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd30fac85, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed9ac89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddbac8e, SUC_NE_lexicon_tags_4 },
    { 0xb1d3ac8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c1ac92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2245ac94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4adac98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8dac9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6119ac9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb95aca0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b7acaa, SUC_NE_lexicon_tags_2 },
    { 0x55f9acab, SUC_NE_lexicon_tags_57 },
    { 0xbf43acac, SUC_NE_lexicon_tags_4 },
    { 0xa289acac, SUC_NE_lexicon_tags_2 },
    { 0x375bacad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4671acb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eefacb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1863acb4, SUC_NE_lexicon_tags_2 },
    { 0x6d9dacb5, SUC_NE_lexicon_tags_3 },
    { 0xdd3facb6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb23acb9, SUC_NE_lexicon_tags_2 },
    { 0x4c95acba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1acbd, SUC_NE_lexicon_tags_4 },
    { 0xdc9dacbe, SUC_NE_lexicon_tags_2 },
    { 0x6f99acbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a1acc9, SUC_NE_lexicon_tags_2 },
    { 0x3a9acca, SUC_NE_lexicon_tags_2 },
    { 0x6a11accb, SUC_NE_lexicon_tags_2 },
    { 0x90d3accb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4729acce, SUC_NE_lexicon_tags_7 },
    { 0x3255accf, SUC_NE_lexicon_tags_2 },
    { 0x6b41accf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a59acd3, SUC_NE_lexicon_tags_2 },
    { 0x911facd4, SUC_NE_lexicon_tags_19 },
    { 0xef13acd4, SUC_NE_lexicon_tags_2 },
    { 0xe8ddacd5, SUC_NE_lexicon_tags_2 },
    { 0x413facd7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c61acdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x708face0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x161dace3, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f91ace5, SUC_NE_lexicon_tags_2 },
    { 0xfaa7ace5, SUC_NE_lexicon_tags_2 },
    { 0x835dace5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f19acec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1acee, SUC_NE_lexicon_tags_2 },
    { 0x92e3acef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eefacf2, SUC_NE_lexicon_tags_2 },
    { 0xbd29acf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72facf7, SUC_NE_lexicon_tags_2 },
    { 0x94e7acf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6783acfb, SUC_NE_lexicon_tags_412 },
    { 0x3431acfc, SUC_NE_lexicon_tags_2 },
    { 0x20f3acfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6acfacff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe901ad01, SUC_NE_lexicon_tags_2 },
    { 0x21e3ad02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe575ad04, SUC_NE_lexicon_tags_2 },
    { 0x705bad05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c73ad09, SUC_NE_lexicon_tags_2 },
    { 0x71b5ad0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7ad0f, SUC_NE_lexicon_tags_2 },
    { 0x6dfbad0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2dad18, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1135ad1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f03ad20, SUC_NE_lexicon_tags_2 },
    { 0x20d7ad21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9759ad24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x287fad27, SUC_NE_lexicon_tags_2 },
    { 0x294fad27, SUC_NE_lexicon_tags_2 },
    { 0xfecfad29, SUC_NE_lexicon_tags_2 },
    { 0xc8dbad2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1adad2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9ad31, SUC_NE_lexicon_tags_2 },
    { 0x6ba5ad32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67c5ad34, SUC_NE_lexicon_tags_2 },
    { 0x5eb9ad34, SUC_NE_lexicon_tags_2 },
    { 0xedf7ad36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe3ad3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6943ad48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d9ad4d, SUC_NE_lexicon_tags_2 },
    { 0xbe21ad4e, SUC_NE_lexicon_tags_2 },
    { 0x497ad4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa039ad52, SUC_NE_lexicon_tags_2 },
    { 0x4a35ad53, SUC_NE_lexicon_tags_2 },
    { 0x27f9ad54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c7ad5a, SUC_NE_lexicon_tags_2 },
    { 0x68ddad5b, SUC_NE_lexicon_tags_2 },
    { 0x11cbad5b, SUC_NE_lexicon_tags_2 },
    { 0x7d19ad5d, SUC_NE_lexicon_tags_2 },
    { 0x1997ad5e, SUC_NE_lexicon_tags_2 },
    { 0x607bad5e, SUC_NE_lexicon_tags_2 },
    { 0x3ffad5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2033ad68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40dad6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x804bad70, SUC_NE_lexicon_tags_2 },
    { 0x3123ad71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2855ad73, SUC_NE_lexicon_tags_2 },
    { 0xc0a7ad74, SUC_NE_lexicon_tags_2 },
    { 0x88b1ad75, SUC_NE_lexicon_tags_2 },
    { 0x9321ad76, SUC_NE_lexicon_tags_2 },
    { 0x9663ad77, SUC_NE_lexicon_tags_2 },
    { 0x64e9ad78, SUC_NE_lexicon_tags_2 },
    { 0x260bad79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x118fad7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ad7e, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0x9725ad80, SUC_NE_lexicon_tags_2 },
    { 0x5587ad80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa34dad83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58dad86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x452bad88, SUC_NE_lexicon_tags_2 },
    { 0x9771ad88, SUC_NE_lexicon_tags_2 },
    { 0x8a2fad89, SUC_NE_lexicon_tags_2 },
    { 0xd0abad8b, SUC_NE_lexicon_tags_2 },
    { 0xdc6bad8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7543ad8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6817ad90, SUC_NE_lexicon_tags_2 },
    { 0x5d2bad90, SUC_NE_lexicon_tags_1 },
    { 0xb8cfad90, SUC_NE_lexicon_tags_2 },
    { 0x5b81ad93, SUC_NE_lexicon_tags_2 },
    { 0x351fad94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x969fad96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d3ad9b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x39bad9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x48ebad9f, SUC_NE_lexicon_tags_2 },
    { 0x55a7ada0, SUC_NE_lexicon_tags_2 },
    { 0x4821ada1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4273ada3, SUC_NE_lexicon_tags_2 },
    { 0x63e3ada4, SUC_NE_lexicon_tags_2 },
    { 0xb63bada5, SUC_NE_lexicon_tags_0 },
    { 0xdd7bada5, SUC_NE_lexicon_tags_2 },
    { 0x48b3ada4, SUC_NE_lexicon_tags_308 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4147adab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838badb1, SUC_NE_lexicon_tags_12 },
    { 0x4a1dadb1, SUC_NE_lexicon_tags_2 },
    { 0x7a7badb3, SUC_NE_lexicon_tags_2 },
    { 0x3735adb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb495adb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e7adba, SUC_NE_lexicon_tags_3 },
    { 0xbdf3adba, SUC_NE_lexicon_tags_2 },
    { 0x6e13adba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9117adbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79badc2, SUC_NE_lexicon_tags_2 },
    { 0xfbd5adc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf223adc5, SUC_NE_lexicon_tags_5 },
    { 0x7ba9adc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e8dadc8, SUC_NE_lexicon_tags_2 },
    { 0x4a73adc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c3adcf, SUC_NE_lexicon_tags_106 },
    { 0xaae7adcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf5add2, SUC_NE_lexicon_tags_10 },
    { 0x9749add2, SUC_NE_lexicon_tags_2 },
    { 0x4fdadd3, SUC_NE_lexicon_tags_2 },
    { 0xc0f5add3, SUC_NE_lexicon_tags_2 },
    { 0x743fadd5, SUC_NE_lexicon_tags_2 },
    { 0xd57dadd7, SUC_NE_lexicon_tags_2 },
    { 0x9abdadd4, SUC_NE_lexicon_tags_2 },
    { 0xaed9add9, SUC_NE_lexicon_tags_2 },
    { 0xbe4fadda, SUC_NE_lexicon_tags_2 },
    { 0xf2e9add9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49fade0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdb5ade2, SUC_NE_lexicon_tags_2 },
    { 0xd12fade3, SUC_NE_lexicon_tags_2 },
    { 0x4bbfade3, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x369bade7, SUC_NE_lexicon_tags_5 },
    { 0x8d4fade8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8323adec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd72dadee, SUC_NE_lexicon_tags_2 },
    { 0x8fc5adef, SUC_NE_lexicon_tags_2 },
    { 0xfb1adf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee93adf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c3adf9, SUC_NE_lexicon_tags_2 },
    { 0xc983adfa, SUC_NE_lexicon_tags_2 },
    { 0x45ffadfa, SUC_NE_lexicon_tags_2 },
    { 0x19a3adfc, SUC_NE_lexicon_tags_2 },
    { 0xaa4fadfc, SUC_NE_lexicon_tags_2 },
    { 0xe29fadfe, SUC_NE_lexicon_tags_2 },
    { 0x22bfadfa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b09ae05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c9ae0b, SUC_NE_lexicon_tags_2 },
    { 0xbda1ae0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e9ae10, SUC_NE_lexicon_tags_2 },
    { 0xa431ae10, SUC_NE_lexicon_tags_2 },
    { 0x24dbae11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5097ae14, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0x847ae16, SUC_NE_lexicon_tags_2 },
    { 0xeff3ae17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9965ae1b, SUC_NE_lexicon_tags_5 },
    { 0xc379ae1b, SUC_NE_lexicon_tags_2 },
    { 0x6ac9ae1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdec9ae1f, SUC_NE_lexicon_tags_3 },
    { 0xf4f3ae1f, SUC_NE_lexicon_tags_2 },
    { 0x5c31ae1f, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0x7257ae23, SUC_NE_lexicon_tags_7 },
    { 0xb4efae24, SUC_NE_lexicon_tags_2 },
    { 0xa427ae25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4afae29, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3d3dae2b, SUC_NE_lexicon_tags_113 },
    { 0xb51dae2c, SUC_NE_lexicon_tags_2 },
    { 0x81c9ae2b, SUC_NE_lexicon_tags_2 },
    { 0x3307ae2b, SUC_NE_lexicon_tags_2 },
    { 0x31e7ae2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4dae32, SUC_NE_lexicon_tags_2 },
    { 0x6077ae33, SUC_NE_lexicon_tags_2 },
    { 0xb489ae34, SUC_NE_lexicon_tags_2 },
    { 0xe503ae32, SUC_NE_lexicon_tags_2 },
    { 0x1369ae36, SUC_NE_lexicon_tags_2 },
    { 0xbe03ae37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf59ae3a, SUC_NE_lexicon_tags_2 },
    { 0xb461ae3b, SUC_NE_lexicon_tags_2 },
    { 0x9a4dae3c, SUC_NE_lexicon_tags_1 },
    { 0x3b8bae3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9043ae43, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe515ae4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x737dae4e, SUC_NE_lexicon_tags_2 },
    { 0xea51ae4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x286fae52, SUC_NE_lexicon_tags_2 },
    { 0x20dbae53, SUC_NE_lexicon_tags_2 },
    { 0x48b3ae52, SUC_NE_lexicon_tags_13 },
    { 0xae2dae55, SUC_NE_lexicon_tags_4 },
    { 0xcd1fae52, SUC_NE_lexicon_tags_2 },
    { 0xdfd1ae57, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe887ae5a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b3ae60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98fae64, SUC_NE_lexicon_tags_2 },
    { 0x2fd9ae64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f5ae6a, SUC_NE_lexicon_tags_2 },
    { 0xd02bae6b, SUC_NE_lexicon_tags_2 },
    { 0x721ae6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c7ae72, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc9ae75, SUC_NE_lexicon_tags_251 },
    { 0xfee7ae75, SUC_NE_lexicon_tags_272 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9643ae7b, SUC_NE_lexicon_tags_2 },
    { 0x7b05ae7b, SUC_NE_lexicon_tags_17 },
    { 0x8f8bae7d, SUC_NE_lexicon_tags_2 },
    { 0x6cedae7e, SUC_NE_lexicon_tags_2 },
    { 0x8445ae7f, SUC_NE_lexicon_tags_2 },
    { 0x1119ae7f, SUC_NE_lexicon_tags_2 },
    { 0x2011ae7d, SUC_NE_lexicon_tags_2 },
    { 0xa6dbae82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa447ae86, SUC_NE_lexicon_tags_2 },
    { 0xe377ae86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72dae8c, SUC_NE_lexicon_tags_7 },
    { 0x8953ae8d, SUC_NE_lexicon_tags_7 },
    { 0x82b9ae8c, SUC_NE_lexicon_tags_2 },
    { 0x5585ae8f, SUC_NE_lexicon_tags_0 },
    { 0xfdd9ae8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe96dae92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e5dae94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x157fae96, SUC_NE_lexicon_tags_2 },
    { 0x2bddae96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3983ae99, SUC_NE_lexicon_tags_2 },
    { 0x4ba7ae99, SUC_NE_lexicon_tags_2 },
    { 0x7253ae9b, SUC_NE_lexicon_tags_7 },
    { 0x12cdae9c, SUC_NE_lexicon_tags_2 },
    { 0x623ae9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc297ae9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dfaea1, SUC_NE_lexicon_tags_2 },
    { 0x73a3aea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec65aea6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6baeab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf363aead, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf887aeb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebbaeb6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x6cadaeb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa10baeba, SUC_NE_lexicon_tags_2 },
    { 0x30dfaeba, SUC_NE_lexicon_tags_4 },
    { 0x380faeba, SUC_NE_lexicon_tags_4 },
    { 0xbf1faebd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x581aebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1441aec4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cdaec9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x75efaecb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b65aece, SUC_NE_lexicon_tags_2 },
    { 0xbca9aecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5baed3, SUC_NE_lexicon_tags_2 },
    { 0xdab9aed4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb25aed7, SUC_NE_lexicon_tags_2 },
    { 0x5f2faed8, SUC_NE_lexicon_tags_2 },
    { 0xf5bfaed9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2651aedc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97daedf, SUC_NE_lexicon_tags_2 },
    { 0xa067aee0, SUC_NE_lexicon_tags_2 },
    { 0x77dfaee1, SUC_NE_lexicon_tags_25 },
    { 0x8723aee2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbfaeea, SUC_NE_lexicon_tags_2 },
    { 0xb5dbaeeb, SUC_NE_lexicon_tags_2 },
    { 0x18abaeea, SUC_NE_lexicon_tags_2 },
    { 0x9851aeed, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3daef4, SUC_NE_lexicon_tags_2 },
    { 0x1e77aef4, SUC_NE_lexicon_tags_2 },
    { 0x3ba3aef6, SUC_NE_lexicon_tags_2 },
    { 0xa7b9aef6, SUC_NE_lexicon_tags_2 },
    { 0xc1faef8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x541baefa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e5aefc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a37af03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb54faf05, SUC_NE_lexicon_tags_2 },
    { 0xd251af06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f25af08, SUC_NE_lexicon_tags_2 },
    { 0x535daf08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bbaf0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f5af15, SUC_NE_lexicon_tags_2 },
    { 0x1709af16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a5af1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2baf23, SUC_NE_lexicon_tags_2 },
    { 0xeeffaf24, SUC_NE_lexicon_tags_2 },
    { 0x3d73af25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7431af27, SUC_NE_lexicon_tags_2 },
    { 0xcfa1af28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee61af2a, SUC_NE_lexicon_tags_2 },
    { 0x63d1af2b, SUC_NE_lexicon_tags_2 },
    { 0x3291af2c, SUC_NE_lexicon_tags_2 },
    { 0xb195af2d, SUC_NE_lexicon_tags_2 },
    { 0xc067af2a, SUC_NE_lexicon_tags_2 },
    { 0x154faf2f, SUC_NE_lexicon_tags_2 },
    { 0x65a5af30, SUC_NE_lexicon_tags_2 },
    { 0x1307af31, SUC_NE_lexicon_tags_2 },
    { 0x7a33af31, SUC_NE_lexicon_tags_2 },
    { 0xdce3af2c, SUC_NE_lexicon_tags_2 },
    { 0xe273af34, SUC_NE_lexicon_tags_2 },
    { 0xe067af34, SUC_NE_lexicon_tags_2 },
    { 0x9d3daf36, SUC_NE_lexicon_tags_2 },
    { 0x7123af37, SUC_NE_lexicon_tags_2 },
    { 0x933baf38, SUC_NE_lexicon_tags_2 },
    { 0x8d1daf34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd9af3d, SUC_NE_lexicon_tags_2 },
    { 0xae3faf3d, SUC_NE_lexicon_tags_2 },
    { 0x6359af3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf701af41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e9daf43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5865af45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe535af4b, SUC_NE_lexicon_tags_2 },
    { 0x367af4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ce1af4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df1af5a, SUC_NE_lexicon_tags_2 },
    { 0x9babaf5b, SUC_NE_lexicon_tags_2 },
    { 0x5433af5c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xb941af5e, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0xf595af60, SUC_NE_lexicon_tags_2 },
    { 0xf4e1af61, SUC_NE_lexicon_tags_2 },
    { 0xeca7af62, SUC_NE_lexicon_tags_2 },
    { 0x3603af63, SUC_NE_lexicon_tags_5 },
    { 0x6537af62, SUC_NE_lexicon_tags_2 },
    { 0xd18daf65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e7af67, SUC_NE_lexicon_tags_2 },
    { 0xbd6faf68, SUC_NE_lexicon_tags_2 },
    { 0xa905af69, SUC_NE_lexicon_tags_2 },
    { 0x294faf6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd0faf6c, SUC_NE_lexicon_tags_2 },
    { 0xe6f1af6d, SUC_NE_lexicon_tags_2 },
    { 0x1ffaf6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4d5af70, SUC_NE_lexicon_tags_2 },
    { 0x7e9baf71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1229af73, SUC_NE_lexicon_tags_2 },
    { 0xa3f1af73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7daf77, SUC_NE_lexicon_tags_2 },
    { 0xfb4baf77, SUC_NE_lexicon_tags_2 },
    { 0xc539af79, SUC_NE_lexicon_tags_1 },
    { 0x724faf79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd13af7c, SUC_NE_lexicon_tags_2 },
    { 0xf935af7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7faf80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefedaf84, SUC_NE_lexicon_tags_2 },
    { 0x961daf84, SUC_NE_lexicon_tags_2 },
    { 0x881baf86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab3af89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce35af8c, SUC_NE_lexicon_tags_2 },
    { 0x9947af8d, SUC_NE_lexicon_tags_5 },
    { 0xe8a1af8c, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e7af94, SUC_NE_lexicon_tags_2 },
    { 0x4fd3af94, SUC_NE_lexicon_tags_2 },
    { 0x5c21af94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7daf99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f4daf9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8faf9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165dafa7, SUC_NE_lexicon_tags_4 },
    { 0xef9afa8, SUC_NE_lexicon_tags_2 },
    { 0x9153afa9, SUC_NE_lexicon_tags_2 },
    { 0x7253afaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb5afae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fbafb3, SUC_NE_lexicon_tags_2 },
    { 0x82adafb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2803afba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efbafc7, SUC_NE_lexicon_tags_57 },
    { 0x753bafc8, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96bafcc, SUC_NE_lexicon_tags_2 },
    { 0x5597afcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7729afcf, SUC_NE_lexicon_tags_2 },
    { 0x4bd1afd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7023afd4, SUC_NE_lexicon_tags_2 },
    { 0x5dcbafd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5221afd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b9afdb, SUC_NE_lexicon_tags_2 },
    { 0x580dafdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3787afde, SUC_NE_lexicon_tags_7 },
    { 0x3675afdf, SUC_NE_lexicon_tags_2 },
    { 0xc65afe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed9afe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e17afe7, SUC_NE_lexicon_tags_2 },
    { 0x5ad1afe8, SUC_NE_lexicon_tags_4 },
    { 0x94e7afe9, SUC_NE_lexicon_tags_2 },
    { 0xc1edafea, SUC_NE_lexicon_tags_2 },
    { 0x2069afeb, SUC_NE_lexicon_tags_2 },
    { 0x5df3afec, SUC_NE_lexicon_tags_2 },
    { 0xf323afed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x864baff0, SUC_NE_lexicon_tags_324 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51baff6, SUC_NE_lexicon_tags_2 },
    { 0x24abaff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7affa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218fb000, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcdb008, SUC_NE_lexicon_tags_102 },
    { 0xc981b009, SUC_NE_lexicon_tags_2 },
    { 0x7529b009, SUC_NE_lexicon_tags_2 },
    { 0xbf49b00b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5075b010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe7b013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9db017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe19b019, SUC_NE_lexicon_tags_7 },
    { 0x1bfb01a, SUC_NE_lexicon_tags_2 },
    { 0x8275b01b, SUC_NE_lexicon_tags_2 },
    { 0x21a1b01b, SUC_NE_lexicon_tags_2 },
    { 0xf4fbb019, SUC_NE_lexicon_tags_2 },
    { 0xf31fb01a, SUC_NE_lexicon_tags_2 },
    { 0x27c9b01f, SUC_NE_lexicon_tags_2 },
    { 0x8fdbb020, SUC_NE_lexicon_tags_3 },
    { 0xb2b9b01f, SUC_NE_lexicon_tags_4 },
    { 0x7d03b020, SUC_NE_lexicon_tags_2 },
    { 0xad55b021, SUC_NE_lexicon_tags_17 },
    { 0x4e39b023, SUC_NE_lexicon_tags_7 },
    { 0x21d1b025, SUC_NE_lexicon_tags_2 },
    { 0x5bafb020, SUC_NE_lexicon_tags_2 },
    { 0x353bb01a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x267b02a, SUC_NE_lexicon_tags_2 },
    { 0x4ff3b02a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x929fb02d, SUC_NE_lexicon_tags_2 },
    { 0x7e0bb02e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e27b031, SUC_NE_lexicon_tags_2 },
    { 0xd7abb032, SUC_NE_lexicon_tags_2 },
    { 0xf1b3b033, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcaadb035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed03b039, SUC_NE_lexicon_tags_2 },
    { 0xe0dfb03a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac3b03e, SUC_NE_lexicon_tags_2 },
    { 0xf48db03f, SUC_NE_lexicon_tags_2 },
    { 0x4b39b03f, SUC_NE_lexicon_tags_4 },
    { 0xffebb041, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcecfb043, SUC_NE_lexicon_tags_2 },
    { 0x7bf7b043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13f1b046, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe01b04f, SUC_NE_lexicon_tags_2 },
    { 0xd09db050, SUC_NE_lexicon_tags_2 },
    { 0xaa5fb051, SUC_NE_lexicon_tags_2 },
    { 0x490bb052, SUC_NE_lexicon_tags_2 },
    { 0xa65bb053, SUC_NE_lexicon_tags_0 },
    { 0x70f1b050, SUC_NE_lexicon_tags_2 },
    { 0x5f8db055, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b03b057, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1e5b059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1887b05c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56b7b05e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdbb062, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3db065, SUC_NE_lexicon_tags_2 },
    { 0xed65b066, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbefb069, SUC_NE_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fbb06c, SUC_NE_lexicon_tags_2 },
    { 0xda03b06c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7223b072, SUC_NE_lexicon_tags_2 },
    { 0xfccfb073, SUC_NE_lexicon_tags_2 },
    { 0x8bb7b074, SUC_NE_lexicon_tags_2 },
    { 0x3d3fb074, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccffb077, SUC_NE_lexicon_tags_2 },
    { 0x3ef9b078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cf1b07a, SUC_NE_lexicon_tags_2 },
    { 0xf62db07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8737b07e, SUC_NE_lexicon_tags_37 },
    { 0, NULL },
    { 0x683db080, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca81b086, SUC_NE_lexicon_tags_18 },
    { 0x3991b086, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26fb08c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873b090, SUC_NE_lexicon_tags_2 },
    { 0x3901b090, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61efb094, SUC_NE_lexicon_tags_2 },
    { 0x7097b095, SUC_NE_lexicon_tags_11 },
    { 0xadb096, SUC_NE_lexicon_tags_2 },
    { 0x5fcfb095, SUC_NE_lexicon_tags_2 },
    { 0xcae3b098, SUC_NE_lexicon_tags_2 },
    { 0x8507b099, SUC_NE_lexicon_tags_2 },
    { 0x526db09a, SUC_NE_lexicon_tags_2 },
    { 0xf9b3b09b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b7b09f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb09b0a6, SUC_NE_lexicon_tags_2 },
    { 0xb4e5b0a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501fb0b0, SUC_NE_lexicon_tags_2 },
    { 0xdd7bb0b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d01b0b3, SUC_NE_lexicon_tags_2 },
    { 0xe929b0b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a9b0b6, SUC_NE_lexicon_tags_3 },
    { 0x4e53b0b7, SUC_NE_lexicon_tags_2 },
    { 0xf565b0b8, SUC_NE_lexicon_tags_2 },
    { 0x88a1b0b6, SUC_NE_lexicon_tags_2 },
    { 0x58ebb0b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x801fb0bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe26bb0be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fb0c4, SUC_NE_lexicon_tags_2 },
    { 0x72b3b0c4, SUC_NE_lexicon_tags_2 },
    { 0x9375b0c4, SUC_NE_lexicon_tags_2 },
    { 0x17c7b0c7, SUC_NE_lexicon_tags_2 },
    { 0xa1d1b0c8, SUC_NE_lexicon_tags_3 },
    { 0x3771b0c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24c7b0cb, SUC_NE_lexicon_tags_2 },
    { 0xf053b0cc, SUC_NE_lexicon_tags_2 },
    { 0xec7db0cd, SUC_NE_lexicon_tags_2 },
    { 0x1a1bb0cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb817b0d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b7b0d4, SUC_NE_lexicon_tags_2 },
    { 0x32cdb0d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7121b0d9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x21ffb0db, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x55f9b0dd, SUC_NE_lexicon_tags_2 },
    { 0xe217b0de, SUC_NE_lexicon_tags_2 },
    { 0xfc07b0de, SUC_NE_lexicon_tags_2 },
    { 0x5029b0e0, SUC_NE_lexicon_tags_2 },
    { 0x16b1b0dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x11f3b0e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaabdb0e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb3fb0e7, SUC_NE_lexicon_tags_2 },
    { 0xb6abb0e8, SUC_NE_lexicon_tags_2 },
    { 0xebbb0e7, SUC_NE_lexicon_tags_1 },
    { 0x8d85b0ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x635b0ec, SUC_NE_lexicon_tags_2 },
    { 0x812fb0ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c3b0f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc42db0f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2045b0fb, SUC_NE_lexicon_tags_2 },
    { 0x7db3b0fc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577fb100, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bb105, SUC_NE_lexicon_tags_2 },
    { 0xac9bb106, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6fb109, SUC_NE_lexicon_tags_2 },
    { 0x24b3b10a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe9b111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e37b116, SUC_NE_lexicon_tags_7 },
    { 0x68c3b116, SUC_NE_lexicon_tags_3 },
    { 0xe6e1b118, SUC_NE_lexicon_tags_2 },
    { 0xcefbb116, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8fb11c, SUC_NE_lexicon_tags_2 },
    { 0x1341b11d, SUC_NE_lexicon_tags_2 },
    { 0x1a5bb11c, SUC_NE_lexicon_tags_2 },
    { 0x17b3b11d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f91b121, SUC_NE_lexicon_tags_2 },
    { 0xecbbb122, SUC_NE_lexicon_tags_1 },
    { 0xc641b121, SUC_NE_lexicon_tags_2 },
    { 0x7bdbb124, SUC_NE_lexicon_tags_2 },
    { 0xda21b124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8465b12a, SUC_NE_lexicon_tags_2 },
    { 0x31c1b12b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10db130, SUC_NE_lexicon_tags_2 },
    { 0x7b3db131, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fb136, SUC_NE_lexicon_tags_5 },
    { 0x9865b137, SUC_NE_lexicon_tags_2 },
    { 0xb469b138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4265b13a, SUC_NE_lexicon_tags_2 },
    { 0x63cdb13b, SUC_NE_lexicon_tags_225 },
    { 0x6c8fb13b, SUC_NE_lexicon_tags_2 },
    { 0x4405b13c, SUC_NE_lexicon_tags_17 },
    { 0x71d9b13e, SUC_NE_lexicon_tags_2 },
    { 0xab9db13c, SUC_NE_lexicon_tags_2 },
    { 0x70ddb140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc81b142, SUC_NE_lexicon_tags_2 },
    { 0xff0fb142, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x175db145, SUC_NE_lexicon_tags_2 },
    { 0x311b145, SUC_NE_lexicon_tags_7 },
    { 0xdc55b147, SUC_NE_lexicon_tags_2 },
    { 0xfe9db147, SUC_NE_lexicon_tags_2 },
    { 0xad49b148, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328bb150, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x1b7db152, SUC_NE_lexicon_tags_24 },
    { 0x7325b153, SUC_NE_lexicon_tags_2 },
    { 0x9673b154, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcfb158, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786bb15d, SUC_NE_lexicon_tags_2 },
    { 0xe959b15d, SUC_NE_lexicon_tags_2 },
    { 0x599db15f, SUC_NE_lexicon_tags_3 },
    { 0x7369b15f, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c81b162, SUC_NE_lexicon_tags_2 },
    { 0xa3ebb163, SUC_NE_lexicon_tags_2 },
    { 0x74a1b164, SUC_NE_lexicon_tags_2 },
    { 0x85dfb164, SUC_NE_lexicon_tags_2 },
    { 0xa5db166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3f5b168, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4401b16a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4579b16e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec73b170, SUC_NE_lexicon_tags_2 },
    { 0xe359b171, SUC_NE_lexicon_tags_2 },
    { 0xef0bb172, SUC_NE_lexicon_tags_2 },
    { 0xd891b173, SUC_NE_lexicon_tags_2 },
    { 0x5b63b173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50bb177, SUC_NE_lexicon_tags_1 },
    { 0x9b7fb178, SUC_NE_lexicon_tags_2 },
    { 0x47d5b179, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d07b17d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ddb181, SUC_NE_lexicon_tags_2 },
    { 0xe68db182, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983db186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4bb189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8455b18c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe077b18f, SUC_NE_lexicon_tags_2 },
    { 0x44d3b18f, SUC_NE_lexicon_tags_2 },
    { 0xab55b191, SUC_NE_lexicon_tags_2 },
    { 0xe437b191, SUC_NE_lexicon_tags_2 },
    { 0xb93db190, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb3b197, SUC_NE_lexicon_tags_2 },
    { 0x13d7b197, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f3b19c, SUC_NE_lexicon_tags_4 },
    { 0x7ec3b19d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe009b1a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xead5b1a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb069b1a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22b7b1a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc04bb1a9, SUC_NE_lexicon_tags_2 },
    { 0xabafb1aa, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5fb1ae, SUC_NE_lexicon_tags_2 },
    { 0x12d9b1af, SUC_NE_lexicon_tags_0 },
    { 0x375bb1ae, SUC_NE_lexicon_tags_2 },
    { 0xb50bb1b1, SUC_NE_lexicon_tags_2 },
    { 0x541fb1b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e9b1b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb175b1bb, SUC_NE_lexicon_tags_2 },
    { 0xfae1b1bc, SUC_NE_lexicon_tags_2 },
    { 0xf6d1b1bc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe33fb1bf, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8953b1c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f1b1c7, SUC_NE_lexicon_tags_2 },
    { 0x7415b1c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8997b1cc, SUC_NE_lexicon_tags_2 },
    { 0x445b1cd, SUC_NE_lexicon_tags_2 },
    { 0x2de3b1ce, SUC_NE_lexicon_tags_2 },
    { 0xed03b1cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xefefb1d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x725fb1d4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xadcbb1d6, SUC_NE_lexicon_tags_40 },
    { 0x6ab1b1d7, SUC_NE_lexicon_tags_1 },
    { 0x1607b1d8, SUC_NE_lexicon_tags_2 },
    { 0xf7dfb1d8, SUC_NE_lexicon_tags_2 },
    { 0x83bfb1d7, SUC_NE_lexicon_tags_2 },
    { 0x2073b1db, SUC_NE_lexicon_tags_2 },
    { 0x3375b1dc, SUC_NE_lexicon_tags_2 },
    { 0x1459b1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb22bb1df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2bb1e3, SUC_NE_lexicon_tags_2 },
    { 0xc5d5b1e3, SUC_NE_lexicon_tags_2 },
    { 0xb0b3b1e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75db1e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3277b1eb, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dbb1ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652db1f5, SUC_NE_lexicon_tags_2 },
    { 0x1c0bb1f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3813b1f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7db1fb, SUC_NE_lexicon_tags_2 },
    { 0x4443b1fc, SUC_NE_lexicon_tags_2 },
    { 0x5223b1fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe413b1ff, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfc0fb201, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xe4a7b203, SUC_NE_lexicon_tags_1 },
    { 0x2551b204, SUC_NE_lexicon_tags_2 },
    { 0x6cdb204, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x885fb208, SUC_NE_lexicon_tags_2 },
    { 0x769b208, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41e1b20b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0fb212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e6bb214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb321b217, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eebb21a, SUC_NE_lexicon_tags_2 },
    { 0xb6e7b21a, SUC_NE_lexicon_tags_2 },
    { 0xec57b21c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda3b220, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fc1b222, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c1b225, SUC_NE_lexicon_tags_2 },
    { 0x9501b226, SUC_NE_lexicon_tags_2 },
    { 0x31b5b226, SUC_NE_lexicon_tags_2 },
    { 0x94ddb228, SUC_NE_lexicon_tags_2 },
    { 0x6657b228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250db22f, SUC_NE_lexicon_tags_2 },
    { 0x8ab9b230, SUC_NE_lexicon_tags_2 },
    { 0xb47db231, SUC_NE_lexicon_tags_3 },
    { 0xe46db232, SUC_NE_lexicon_tags_2 },
    { 0x9379b231, SUC_NE_lexicon_tags_2 },
    { 0xd0efb231, SUC_NE_lexicon_tags_14 },
    { 0x1b55b234, SUC_NE_lexicon_tags_2 },
    { 0x517fb231, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a61b238, SUC_NE_lexicon_tags_2 },
    { 0x8b7b238, SUC_NE_lexicon_tags_2 },
    { 0xd169b23a, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf3b243, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf5b246, SUC_NE_lexicon_tags_2 },
    { 0xb315b246, SUC_NE_lexicon_tags_2 },
    { 0xf23b248, SUC_NE_lexicon_tags_2 },
    { 0x775fb249, SUC_NE_lexicon_tags_2 },
    { 0x785bb24a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc753b24c, SUC_NE_lexicon_tags_2 },
    { 0x2799b24c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade5b252, SUC_NE_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3b255, SUC_NE_lexicon_tags_2 },
    { 0x6f2fb255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada7b259, SUC_NE_lexicon_tags_2 },
    { 0x37a1b25a, SUC_NE_lexicon_tags_2 },
    { 0x2c4db25b, SUC_NE_lexicon_tags_2 },
    { 0x97c1b25b, SUC_NE_lexicon_tags_2 },
    { 0x45afb25c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ddb261, SUC_NE_lexicon_tags_2 },
    { 0x5f97b262, SUC_NE_lexicon_tags_3 },
    { 0xe209b262, SUC_NE_lexicon_tags_167 },
    { 0xf6fb263, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bdb268, SUC_NE_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f1b26d, SUC_NE_lexicon_tags_2 },
    { 0xc999b26d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb6bb270, SUC_NE_lexicon_tags_2 },
    { 0xf7fdb270, SUC_NE_lexicon_tags_2 },
    { 0x51b9b270, SUC_NE_lexicon_tags_2 },
    { 0x3359b273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e1b27d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb283, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bfb289, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5131b28b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x79ffb28d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95abb295, SUC_NE_lexicon_tags_2 },
    { 0xb527b296, SUC_NE_lexicon_tags_2 },
    { 0x1bc3b295, SUC_NE_lexicon_tags_4 },
    { 0x868fb298, SUC_NE_lexicon_tags_2 },
    { 0x35bbb299, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4977b29b, SUC_NE_lexicon_tags_2 },
    { 0x6bf9b29b, SUC_NE_lexicon_tags_2 },
    { 0x386bb29d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ee9b29f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3965b2a5, SUC_NE_lexicon_tags_2 },
    { 0xf3bfb2a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15fb2a9, SUC_NE_lexicon_tags_2 },
    { 0x1cfdb2aa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff5b2ad, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bbb2b1, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x253bb2b4, SUC_NE_lexicon_tags_2 },
    { 0x1a49b2b5, SUC_NE_lexicon_tags_2 },
    { 0xba1fb2b6, SUC_NE_lexicon_tags_2 },
    { 0x167b2b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795bb2bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d67b2be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b3b2c1, SUC_NE_lexicon_tags_2 },
    { 0xed5b2c2, SUC_NE_lexicon_tags_2 },
    { 0xac07b2c2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x79f1b2c5, SUC_NE_lexicon_tags_2 },
    { 0x85c7b2c5, SUC_NE_lexicon_tags_2 },
    { 0xebbdb2c7, SUC_NE_lexicon_tags_2 },
    { 0x4a27b2c8, SUC_NE_lexicon_tags_2 },
    { 0x5299b2c6, SUC_NE_lexicon_tags_2 },
    { 0x9c7fb2ca, SUC_NE_lexicon_tags_4 },
    { 0xeee3b2cb, SUC_NE_lexicon_tags_2 },
    { 0xa19b2c7, SUC_NE_lexicon_tags_2 },
    { 0xbe6bb2ca, SUC_NE_lexicon_tags_2 },
    { 0x8e79b2ce, SUC_NE_lexicon_tags_2 },
    { 0xd90bb2cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da9b2d2, SUC_NE_lexicon_tags_2 },
    { 0x7f83b2d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc085b2d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c1b2dd, SUC_NE_lexicon_tags_2 },
    { 0x7b6bb2de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6969b2e0, SUC_NE_lexicon_tags_5 },
    { 0xa419b2e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x675b2e3, SUC_NE_lexicon_tags_2 },
    { 0xb695b2e4, SUC_NE_lexicon_tags_2 },
    { 0xe6db2e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4e7b2e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8db2ed, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x541fb2ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3911b2f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbebb2f3, SUC_NE_lexicon_tags_2 },
    { 0x64dfb2f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5791b2f7, SUC_NE_lexicon_tags_2 },
    { 0xe409b2f7, SUC_NE_lexicon_tags_2 },
    { 0x4957b2f9, SUC_NE_lexicon_tags_2 },
    { 0x39ebb2fa, SUC_NE_lexicon_tags_2 },
    { 0x4403b2f8, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8457b2fe, SUC_NE_lexicon_tags_2 },
    { 0xed3b2ff, SUC_NE_lexicon_tags_2 },
    { 0xcf15b300, SUC_NE_lexicon_tags_2 },
    { 0x96a9b2ff, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb3db303, SUC_NE_lexicon_tags_4 },
    { 0x5895b304, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478bb308, SUC_NE_lexicon_tags_2 },
    { 0x94fb309, SUC_NE_lexicon_tags_2 },
    { 0xf7c1b30a, SUC_NE_lexicon_tags_2 },
    { 0xa357b30b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5d7b30d, SUC_NE_lexicon_tags_2 },
    { 0x2fb5b30e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4165b310, SUC_NE_lexicon_tags_2 },
    { 0xc381b311, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b3b315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x666db318, SUC_NE_lexicon_tags_2 },
    { 0xeb1bb319, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5b31d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aa3b31f, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1b326, SUC_NE_lexicon_tags_85 },
    { 0x273b326, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2447b32a, SUC_NE_lexicon_tags_2 },
    { 0xffcbb32a, SUC_NE_lexicon_tags_50 },
    { 0xa3d1b32c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7a9b32e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7d9b330, SUC_NE_lexicon_tags_2 },
    { 0x58b9b330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452bb335, SUC_NE_lexicon_tags_2 },
    { 0xc61bb336, SUC_NE_lexicon_tags_2 },
    { 0x45db336, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xc913b339, SUC_NE_lexicon_tags_2 },
    { 0xa9abb33a, SUC_NE_lexicon_tags_2 },
    { 0xc1fbb33b, SUC_NE_lexicon_tags_2 },
    { 0x3f41b33b, SUC_NE_lexicon_tags_2 },
    { 0xbe6db33d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe333b33f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5de1b341, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xebdbb343, SUC_NE_lexicon_tags_2 },
    { 0x549db344, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2d3b346, SUC_NE_lexicon_tags_2 },
    { 0xa57bb347, SUC_NE_lexicon_tags_18 },
    { 0x6449b348, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x298db34a, SUC_NE_lexicon_tags_2 },
    { 0x3321b34a, SUC_NE_lexicon_tags_2 },
    { 0xf9cdb34c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9db351, SUC_NE_lexicon_tags_2 },
    { 0x1e45b352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf5b359, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa111b35d, SUC_NE_lexicon_tags_2 },
    { 0x95cdb35d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f5b360, SUC_NE_lexicon_tags_2 },
    { 0xb1d9b361, SUC_NE_lexicon_tags_2 },
    { 0xd405b362, SUC_NE_lexicon_tags_2 },
    { 0xd507b363, SUC_NE_lexicon_tags_2 },
    { 0x871db364, SUC_NE_lexicon_tags_2 },
    { 0xb25bb365, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f9b368, SUC_NE_lexicon_tags_2 },
    { 0xd52db369, SUC_NE_lexicon_tags_10 },
    { 0xb8a1b36a, SUC_NE_lexicon_tags_2 },
    { 0xecfb36b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957bb370, SUC_NE_lexicon_tags_2 },
    { 0x8659b370, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b65b375, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1bb378, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d21b37a, SUC_NE_lexicon_tags_2 },
    { 0xf575b37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1213b37f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ffb382, SUC_NE_lexicon_tags_17 },
    { 0x228db383, SUC_NE_lexicon_tags_2 },
    { 0xdb9fb384, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa735b386, SUC_NE_lexicon_tags_2 },
    { 0x11b9b387, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b9b38a, SUC_NE_lexicon_tags_2 },
    { 0xbf09b38a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8a7b38d, SUC_NE_lexicon_tags_2 },
    { 0x279fb38e, SUC_NE_lexicon_tags_2 },
    { 0xc7d1b38f, SUC_NE_lexicon_tags_2 },
    { 0xfac7b38f, SUC_NE_lexicon_tags_2 },
    { 0xdabdb391, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7003b393, SUC_NE_lexicon_tags_2 },
    { 0x71e1b394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f8bb396, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fb3a3, SUC_NE_lexicon_tags_2 },
    { 0x5c07b3a4, SUC_NE_lexicon_tags_2 },
    { 0xd8e3b3a5, SUC_NE_lexicon_tags_2 },
    { 0x568db3a6, SUC_NE_lexicon_tags_1 },
    { 0x7cddb3a7, SUC_NE_lexicon_tags_3 },
    { 0xed7bb3a5, SUC_NE_lexicon_tags_2 },
    { 0x58ddb3a9, SUC_NE_lexicon_tags_2 },
    { 0x885db3aa, SUC_NE_lexicon_tags_2 },
    { 0x36f9b3ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964db3b2, SUC_NE_lexicon_tags_2 },
    { 0xa16bb3b3, SUC_NE_lexicon_tags_2 },
    { 0x8d63b3b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd5b3b8, SUC_NE_lexicon_tags_2 },
    { 0xe21b3b9, SUC_NE_lexicon_tags_2 },
    { 0x7a21b3b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a85b3bc, SUC_NE_lexicon_tags_2 },
    { 0xf619b3bd, SUC_NE_lexicon_tags_2 },
    { 0x636bb3bc, SUC_NE_lexicon_tags_30 },
    { 0xbfffb3bf, SUC_NE_lexicon_tags_2 },
    { 0x493db3bd, SUC_NE_lexicon_tags_5 },
    { 0x1ddfb3c1, SUC_NE_lexicon_tags_2 },
    { 0xfb79b3c1, SUC_NE_lexicon_tags_2 },
    { 0x4bf7b3c3, SUC_NE_lexicon_tags_2 },
    { 0x8a81b3c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2a7b3c6, SUC_NE_lexicon_tags_2 },
    { 0x2df7b3c6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac3b3cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2f9b3ce, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac5b3d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef11b3de, SUC_NE_lexicon_tags_2 },
    { 0xd915b3df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8307b3e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe057b3e5, SUC_NE_lexicon_tags_2 },
    { 0x469fb3e5, SUC_NE_lexicon_tags_2 },
    { 0x7203b3e7, SUC_NE_lexicon_tags_2 },
    { 0x34fb3e7, SUC_NE_lexicon_tags_2 },
    { 0x82dfb3e8, SUC_NE_lexicon_tags_2 },
    { 0xe345b3ea, SUC_NE_lexicon_tags_2 },
    { 0xfafb3eb, SUC_NE_lexicon_tags_2 },
    { 0x1e0db3e8, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c9b3f1, SUC_NE_lexicon_tags_2 },
    { 0xb6abb3f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7605b3f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8291b3f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49db3ff, SUC_NE_lexicon_tags_2 },
    { 0x69d3b400, SUC_NE_lexicon_tags_2 },
    { 0xfa1db401, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6997b408, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe461b40d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ddb414, SUC_NE_lexicon_tags_2 },
    { 0x3f55b415, SUC_NE_lexicon_tags_18 },
    { 0xe3d3b416, SUC_NE_lexicon_tags_1 },
    { 0x1b51b414, SUC_NE_lexicon_tags_2 },
    { 0xdcf9b418, SUC_NE_lexicon_tags_2 },
    { 0xe4b9b418, SUC_NE_lexicon_tags_2 },
    { 0xbb41b41a, SUC_NE_lexicon_tags_2 },
    { 0x7b03b41b, SUC_NE_lexicon_tags_4 },
    { 0x759b41b, SUC_NE_lexicon_tags_2 },
    { 0x6257b41c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeffb41f, SUC_NE_lexicon_tags_2 },
    { 0x445fb420, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x10edb422, SUC_NE_lexicon_tags_2 },
    { 0xc9e9b423, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8283b425, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc02db427, SUC_NE_lexicon_tags_2 },
    { 0x1477b428, SUC_NE_lexicon_tags_10 },
    { 0xa5f9b429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf245b42f, SUC_NE_lexicon_tags_2 },
    { 0xd04db430, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4395b432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b1b436, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc241b439, SUC_NE_lexicon_tags_2 },
    { 0x82c9b439, SUC_NE_lexicon_tags_2 },
    { 0x40c5b43a, SUC_NE_lexicon_tags_2 },
    { 0xc1c7b439, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d99b43e, SUC_NE_lexicon_tags_2 },
    { 0x8869b43f, SUC_NE_lexicon_tags_2 },
    { 0x81c3b43e, SUC_NE_lexicon_tags_2 },
    { 0x7ecfb441, SUC_NE_lexicon_tags_2 },
    { 0xb47bb442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9761b447, SUC_NE_lexicon_tags_2 },
    { 0x8b31b447, SUC_NE_lexicon_tags_2 },
    { 0x73bbb449, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf049b44b, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563b452, SUC_NE_lexicon_tags_2 },
    { 0xa9e7b453, SUC_NE_lexicon_tags_8 },
    { 0xf09db454, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11fb458, SUC_NE_lexicon_tags_2 },
    { 0x6c5bb459, SUC_NE_lexicon_tags_2 },
    { 0x61abb459, SUC_NE_lexicon_tags_2 },
    { 0xedc9b45a, SUC_NE_lexicon_tags_2 },
    { 0xf755b45b, SUC_NE_lexicon_tags_2 },
    { 0x2ebbb45d, SUC_NE_lexicon_tags_12 },
    { 0xb17fb45b, SUC_NE_lexicon_tags_2 },
    { 0x42a7b45c, SUC_NE_lexicon_tags_2 },
    { 0xed65b460, SUC_NE_lexicon_tags_2 },
    { 0x8b9fb461, SUC_NE_lexicon_tags_2 },
    { 0x4a65b45a, SUC_NE_lexicon_tags_2 },
    { 0x5a37b463, SUC_NE_lexicon_tags_59 },
    { 0xbc5b45d, SUC_NE_lexicon_tags_2 },
    { 0x70ddb45a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d39b46b, SUC_NE_lexicon_tags_2 },
    { 0xbf2bb46c, SUC_NE_lexicon_tags_2 },
    { 0xf12db46d, SUC_NE_lexicon_tags_4 },
    { 0x8b4fb46d, SUC_NE_lexicon_tags_2 },
    { 0x7e1b46f, SUC_NE_lexicon_tags_2 },
    { 0xe24fb470, SUC_NE_lexicon_tags_2 },
    { 0x8b1fb471, SUC_NE_lexicon_tags_2 },
    { 0x5771b472, SUC_NE_lexicon_tags_12 },
    { 0x987fb473, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56dbb475, SUC_NE_lexicon_tags_2 },
    { 0xcb23b475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9451b478, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x382fb47a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x188bb47d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf7b47f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91db482, SUC_NE_lexicon_tags_2 },
    { 0xa527b482, SUC_NE_lexicon_tags_2 },
    { 0x2707b484, SUC_NE_lexicon_tags_2 },
    { 0xc49db485, SUC_NE_lexicon_tags_3 },
    { 0x7bc1b483, SUC_NE_lexicon_tags_2 },
    { 0x449db487, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd1b48d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9093b490, SUC_NE_lexicon_tags_2 },
    { 0x3f55b490, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546bb495, SUC_NE_lexicon_tags_13 },
    { 0xb05db495, SUC_NE_lexicon_tags_2 },
    { 0x87cdb497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a57b4a0, SUC_NE_lexicon_tags_2 },
    { 0xb32db4a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe75b4a3, SUC_NE_lexicon_tags_2 },
    { 0x5da5b4a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce33b4a6, SUC_NE_lexicon_tags_3 },
    { 0x965fb4a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2639b4aa, SUC_NE_lexicon_tags_50 },
    { 0xaecdb4aa, SUC_NE_lexicon_tags_2 },
    { 0x2d09b4ab, SUC_NE_lexicon_tags_2 },
    { 0x41d9b4ac, SUC_NE_lexicon_tags_2 },
    { 0xc847b4aa, SUC_NE_lexicon_tags_2 },
    { 0xdfc5b4af, SUC_NE_lexicon_tags_2 },
    { 0xd02fb4b0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2f6db4b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359b4b6, SUC_NE_lexicon_tags_2 },
    { 0xa5ffb4b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6afbb4b9, SUC_NE_lexicon_tags_2 },
    { 0x61adb4b9, SUC_NE_lexicon_tags_7 },
    { 0x3841b4ba, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb7b4be, SUC_NE_lexicon_tags_2 },
    { 0x700db4bf, SUC_NE_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5db4c7, SUC_NE_lexicon_tags_2 },
    { 0xb9ddb4c7, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667bb4cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9db4cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a07b4d3, SUC_NE_lexicon_tags_2 },
    { 0x6e23b4d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91b4d7, SUC_NE_lexicon_tags_2 },
    { 0xfc31b4d8, SUC_NE_lexicon_tags_2 },
    { 0x170bb4d9, SUC_NE_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991b4dd, SUC_NE_lexicon_tags_2 },
    { 0x1e85b4de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f1b4e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8481b4e4, SUC_NE_lexicon_tags_4 },
    { 0x3639b4e5, SUC_NE_lexicon_tags_58 },
    { 0x1919b4e6, SUC_NE_lexicon_tags_2 },
    { 0x8bebb4e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce8db4e9, SUC_NE_lexicon_tags_2 },
    { 0xcddbb4ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5311b4ec, SUC_NE_lexicon_tags_2 },
    { 0x1d3b4ed, SUC_NE_lexicon_tags_11 },
    { 0x3ddb4ee, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467db4f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d3b4f8, SUC_NE_lexicon_tags_2 },
    { 0x53b3b4f8, SUC_NE_lexicon_tags_2 },
    { 0x335db4f8, SUC_NE_lexicon_tags_2 },
    { 0xbdc7b4fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22c3b4fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe799b502, SUC_NE_lexicon_tags_3 },
    { 0x4ac9b503, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1fb506, SUC_NE_lexicon_tags_7 },
    { 0xd321b507, SUC_NE_lexicon_tags_2 },
    { 0x6cafb508, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e5b50b, SUC_NE_lexicon_tags_2 },
    { 0x5efbb50c, SUC_NE_lexicon_tags_2 },
    { 0x4e69b50b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6767b516, SUC_NE_lexicon_tags_2 },
    { 0xc743b517, SUC_NE_lexicon_tags_2 },
    { 0x5dc1b518, SUC_NE_lexicon_tags_2 },
    { 0x885db519, SUC_NE_lexicon_tags_2 },
    { 0xb447b517, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac3b51d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d5db51f, SUC_NE_lexicon_tags_2 },
    { 0x7d9db51f, SUC_NE_lexicon_tags_2 },
    { 0xb12bb51f, SUC_NE_lexicon_tags_2 },
    { 0x77edb522, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc44fb524, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b29b529, SUC_NE_lexicon_tags_2 },
    { 0x4691b52a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4261b52c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1075b52e, SUC_NE_lexicon_tags_2 },
    { 0x613bb52f, SUC_NE_lexicon_tags_2 },
    { 0x9dfbb52e, SUC_NE_lexicon_tags_2 },
    { 0x9531b531, SUC_NE_lexicon_tags_3 },
    { 0xe9bfb532, SUC_NE_lexicon_tags_2 },
    { 0x57b533, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d3b536, SUC_NE_lexicon_tags_2 },
    { 0xb42bb536, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x510bb539, SUC_NE_lexicon_tags_2 },
    { 0x9e6db539, SUC_NE_lexicon_tags_2 },
    { 0xcd65b53a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8485b53e, SUC_NE_lexicon_tags_2 },
    { 0x5929b53f, SUC_NE_lexicon_tags_1 },
    { 0x5c8db540, SUC_NE_lexicon_tags_2 },
    { 0xdc9bb541, SUC_NE_lexicon_tags_2 },
    { 0x316bb540, SUC_NE_lexicon_tags_2 },
    { 0x5ed9b542, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1b9db545, SUC_NE_lexicon_tags_7 },
    { 0xb723b545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48db549, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xa9cfb54b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d5b54f, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5489b561, SUC_NE_lexicon_tags_5 },
    { 0x680bb562, SUC_NE_lexicon_tags_2 },
    { 0x9e21b562, SUC_NE_lexicon_tags_2 },
    { 0x539db562, SUC_NE_lexicon_tags_2 },
    { 0x49c3b565, SUC_NE_lexicon_tags_2 },
    { 0x2b59b566, SUC_NE_lexicon_tags_3 },
    { 0x310fb565, SUC_NE_lexicon_tags_1 },
    { 0xfe25b563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebb56b, SUC_NE_lexicon_tags_2 },
    { 0x67bdb56c, SUC_NE_lexicon_tags_18 },
    { 0x261db56d, SUC_NE_lexicon_tags_17 },
    { 0xffcfb56e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4adb570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1067b575, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe555b577, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f17b579, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7913b57b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a01b57e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc07b588, SUC_NE_lexicon_tags_2 },
    { 0xf4d1b589, SUC_NE_lexicon_tags_0 },
    { 0xe271b58a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ff9b58c, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9bb596, SUC_NE_lexicon_tags_2 },
    { 0xf58fb597, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x349fb599, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9721b59b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1d3b59d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x201db59f, SUC_NE_lexicon_tags_2 },
    { 0x7113b5a0, SUC_NE_lexicon_tags_2 },
    { 0x68d7b5a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe29b5a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e1b5a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x101fb5a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7b5ab, SUC_NE_lexicon_tags_2 },
    { 0xbfa5b5ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa633b5af, SUC_NE_lexicon_tags_2 },
    { 0x5453b5b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x648bb5b3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf825b5b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe635b5ba, SUC_NE_lexicon_tags_2 },
    { 0x24e9b5bb, SUC_NE_lexicon_tags_2 },
    { 0xf4bbb5bc, SUC_NE_lexicon_tags_2 },
    { 0xda6bb5bd, SUC_NE_lexicon_tags_2 },
    { 0x7eb1b5be, SUC_NE_lexicon_tags_2 },
    { 0xbfe5b5bf, SUC_NE_lexicon_tags_3 },
    { 0x9badb5c0, SUC_NE_lexicon_tags_4 },
    { 0x35cfb5c0, SUC_NE_lexicon_tags_5 },
    { 0x1dcbb5c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x957db5c5, SUC_NE_lexicon_tags_2 },
    { 0x301bb5c5, SUC_NE_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fb5c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x679b5cb, SUC_NE_lexicon_tags_2 },
    { 0x8f11b5cc, SUC_NE_lexicon_tags_2 },
    { 0x9065b5cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b7b5d2, SUC_NE_lexicon_tags_2 },
    { 0x7709b5d3, SUC_NE_lexicon_tags_2 },
    { 0xc6a3b5d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe073b5d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4169b5df, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xab83b5e1, SUC_NE_lexicon_tags_2 },
    { 0x75f1b5e2, SUC_NE_lexicon_tags_2 },
    { 0x2d2db5e2, SUC_NE_lexicon_tags_2 },
    { 0x2189b5e4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b95b5ec, SUC_NE_lexicon_tags_2 },
    { 0x5013b5ed, SUC_NE_lexicon_tags_4 },
    { 0xf269b5ec, SUC_NE_lexicon_tags_1 },
    { 0xb423b5ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500bb5f5, SUC_NE_lexicon_tags_2 },
    { 0xca27b5f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8897b5fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c7fb5fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a3b5ff, SUC_NE_lexicon_tags_5 },
    { 0xfcd7b600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa347b603, SUC_NE_lexicon_tags_2 },
    { 0xdbd5b604, SUC_NE_lexicon_tags_160 },
    { 0xa017b605, SUC_NE_lexicon_tags_2 },
    { 0xa0fbb605, SUC_NE_lexicon_tags_2 },
    { 0xf1efb607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70bb60c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54fb60f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa84db611, SUC_NE_lexicon_tags_2 },
    { 0xa6cfb611, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2db615, SUC_NE_lexicon_tags_249 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa761b61a, SUC_NE_lexicon_tags_2 },
    { 0x24b1b61b, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x214fb61e, SUC_NE_lexicon_tags_2 },
    { 0x633b61f, SUC_NE_lexicon_tags_2 },
    { 0x4107b61e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfedb622, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7dfb624, SUC_NE_lexicon_tags_2 },
    { 0x6723b625, SUC_NE_lexicon_tags_2 },
    { 0x4d1db624, SUC_NE_lexicon_tags_2 },
    { 0xaf17b627, SUC_NE_lexicon_tags_2 },
    { 0x9ed9b628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d37b62a, SUC_NE_lexicon_tags_2 },
    { 0x99fbb62a, SUC_NE_lexicon_tags_2 },
    { 0xd5c5b62c, SUC_NE_lexicon_tags_2 },
    { 0x15a5b62b, SUC_NE_lexicon_tags_2 },
    { 0xaa83b62c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bfb631, SUC_NE_lexicon_tags_30 },
    { 0x40f1b631, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb145b634, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4defb637, SUC_NE_lexicon_tags_2 },
    { 0xb55fb637, SUC_NE_lexicon_tags_2 },
    { 0xe137b639, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1031b63b, SUC_NE_lexicon_tags_2 },
    { 0x1223b63c, SUC_NE_lexicon_tags_2 },
    { 0xac6bb63d, SUC_NE_lexicon_tags_2 },
    { 0x283db63b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9db641, SUC_NE_lexicon_tags_2 },
    { 0x8e75b641, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a9b644, SUC_NE_lexicon_tags_2 },
    { 0x5ac1b645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3691b648, SUC_NE_lexicon_tags_7 },
    { 0x3575b649, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b64e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfa7b650, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e01b652, SUC_NE_lexicon_tags_2 },
    { 0xa0cdb652, SUC_NE_lexicon_tags_2 },
    { 0xea97b654, SUC_NE_lexicon_tags_2 },
    { 0x2ab7b655, SUC_NE_lexicon_tags_2 },
    { 0x51abb656, SUC_NE_lexicon_tags_2 },
    { 0x4f9fb657, SUC_NE_lexicon_tags_2 },
    { 0xc933b658, SUC_NE_lexicon_tags_2 },
    { 0x3d81b659, SUC_NE_lexicon_tags_30 },
    { 0x8d4fb65a, SUC_NE_lexicon_tags_5 },
    { 0x944db657, SUC_NE_lexicon_tags_17 },
    { 0x7af7b655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa855b65e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c45b664, SUC_NE_lexicon_tags_4 },
    { 0x84d5b665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1167b66c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44bb671, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7b675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x566fb677, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f91b67a, SUC_NE_lexicon_tags_2 },
    { 0x7667b67b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfb67e, SUC_NE_lexicon_tags_2 },
    { 0x1211b67e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed31b682, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6e3b684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5007b687, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5b68e, SUC_NE_lexicon_tags_3 },
    { 0xa1a3b68e, SUC_NE_lexicon_tags_2 },
    { 0xcd37b690, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c7b699, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a59b69b, SUC_NE_lexicon_tags_3 },
    { 0xf731b69b, SUC_NE_lexicon_tags_2 },
    { 0x1093b69d, SUC_NE_lexicon_tags_2 },
    { 0xc20fb69e, SUC_NE_lexicon_tags_2 },
    { 0xdff5b69e, SUC_NE_lexicon_tags_2 },
    { 0x7769b6a0, SUC_NE_lexicon_tags_18 },
    { 0x163db6a0, SUC_NE_lexicon_tags_34 },
    { 0xf5b7b6a0, SUC_NE_lexicon_tags_36 },
    { 0x6075b6a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e65b6a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb3b6ab, SUC_NE_lexicon_tags_2 },
    { 0x3d43b6ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea1b6b1, SUC_NE_lexicon_tags_2 },
    { 0x152bb6b2, SUC_NE_lexicon_tags_2 },
    { 0x925db6b3, SUC_NE_lexicon_tags_2 },
    { 0x74e5b6b4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf2f7b6b6, SUC_NE_lexicon_tags_2 },
    { 0xe6adb6b7, SUC_NE_lexicon_tags_2 },
    { 0x73ddb6b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8db6bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e55b6be, SUC_NE_lexicon_tags_2 },
    { 0xc9e1b6bf, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x62c3b6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc43fb6c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9933b6c6, SUC_NE_lexicon_tags_2 },
    { 0xf39db6c7, SUC_NE_lexicon_tags_7 },
    { 0xa061b6c8, SUC_NE_lexicon_tags_89 },
    { 0xfc3db6c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf151b6cb, SUC_NE_lexicon_tags_2 },
    { 0x84fdb6cc, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd885b6cf, SUC_NE_lexicon_tags_2 },
    { 0xc8b7b6d0, SUC_NE_lexicon_tags_1 },
    { 0xfbdbb6d1, SUC_NE_lexicon_tags_2 },
    { 0xaf29b6d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e01b6d4, SUC_NE_lexicon_tags_2 },
    { 0x1547b6d5, SUC_NE_lexicon_tags_2 },
    { 0x1d3b6d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff07b6d8, SUC_NE_lexicon_tags_2 },
    { 0x4277b6d9, SUC_NE_lexicon_tags_2 },
    { 0x5f15b6da, SUC_NE_lexicon_tags_2 },
    { 0xb71fb6d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb501b6dd, SUC_NE_lexicon_tags_0 },
    { 0x3371b6dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9263b6e1, SUC_NE_lexicon_tags_2 },
    { 0xa75b6e2, SUC_NE_lexicon_tags_2 },
    { 0x2043b6e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9db6e5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb3b6ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0db6ed, SUC_NE_lexicon_tags_36 },
    { 0xec77b6ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1bb6f3, SUC_NE_lexicon_tags_2 },
    { 0x6abbb6f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae7b6f7, SUC_NE_lexicon_tags_2 },
    { 0xab9bb6f8, SUC_NE_lexicon_tags_2 },
    { 0x3aa7b6f8, SUC_NE_lexicon_tags_2 },
    { 0xa3dbb6fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32fb703, SUC_NE_lexicon_tags_2 },
    { 0xd3a3b703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4edfb706, SUC_NE_lexicon_tags_4 },
    { 0x21f9b707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5393b70a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0d3b70c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7b70f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x67bdb711, SUC_NE_lexicon_tags_2 },
    { 0xe6f3b712, SUC_NE_lexicon_tags_2 },
    { 0xa1f9b713, SUC_NE_lexicon_tags_2 },
    { 0x360fb712, SUC_NE_lexicon_tags_2 },
    { 0xde11b715, SUC_NE_lexicon_tags_2 },
    { 0x5c15b716, SUC_NE_lexicon_tags_2 },
    { 0x19efb711, SUC_NE_lexicon_tags_2 },
    { 0x867b713, SUC_NE_lexicon_tags_2 },
    { 0xd9ddb719, SUC_NE_lexicon_tags_2 },
    { 0x8ab9b71a, SUC_NE_lexicon_tags_2 },
    { 0xb40bb71b, SUC_NE_lexicon_tags_2 },
    { 0x356fb713, SUC_NE_lexicon_tags_7 },
    { 0x6521b716, SUC_NE_lexicon_tags_102 },
    { 0x17efb71e, SUC_NE_lexicon_tags_2 },
    { 0xedb5b719, SUC_NE_lexicon_tags_2 },
    { 0x2c7db720, SUC_NE_lexicon_tags_2 },
    { 0x4aedb721, SUC_NE_lexicon_tags_4 },
    { 0x4b75b722, SUC_NE_lexicon_tags_2 },
    { 0xe623b722, SUC_NE_lexicon_tags_85 },
    { 0xe5cbb724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e3b72c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a71b72e, SUC_NE_lexicon_tags_2 },
    { 0xc13b72e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac83b732, SUC_NE_lexicon_tags_2 },
    { 0xbe19b733, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21b5b735, SUC_NE_lexicon_tags_2 },
    { 0x525fb736, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c1b73c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5313b740, SUC_NE_lexicon_tags_2 },
    { 0x9447b740, SUC_NE_lexicon_tags_2 },
    { 0xb2b3b742, SUC_NE_lexicon_tags_2 },
    { 0x44b7b743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e1b747, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49d3b749, SUC_NE_lexicon_tags_2 },
    { 0x75a1b74a, SUC_NE_lexicon_tags_2 },
    { 0x3753b74b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8989b74d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x87ffb74f, SUC_NE_lexicon_tags_2 },
    { 0x8731b750, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc7b753, SUC_NE_lexicon_tags_2 },
    { 0x85cfb754, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9db757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5411b75b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d1b762, SUC_NE_lexicon_tags_2 },
    { 0x3097b762, SUC_NE_lexicon_tags_2 },
    { 0xd979b763, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c1b768, SUC_NE_lexicon_tags_19 },
    { 0x76fbb768, SUC_NE_lexicon_tags_2 },
    { 0xe885b768, SUC_NE_lexicon_tags_2 },
    { 0x56a3b76b, SUC_NE_lexicon_tags_2 },
    { 0x5d6bb76c, SUC_NE_lexicon_tags_2 },
    { 0x98efb76d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00db773, SUC_NE_lexicon_tags_2 },
    { 0xa5bdb774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fdb777, SUC_NE_lexicon_tags_2 },
    { 0x9491b778, SUC_NE_lexicon_tags_2 },
    { 0x1ef1b779, SUC_NE_lexicon_tags_2 },
    { 0x1e87b779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7631b77d, SUC_NE_lexicon_tags_2 },
    { 0x5c5b77e, SUC_NE_lexicon_tags_2 },
    { 0x599fb77f, SUC_NE_lexicon_tags_5 },
    { 0x6d45b780, SUC_NE_lexicon_tags_2 },
    { 0xfc39b781, SUC_NE_lexicon_tags_14 },
    { 0x2315b780, SUC_NE_lexicon_tags_138 },
    { 0x5d1fb782, SUC_NE_lexicon_tags_2 },
    { 0xbfabb77f, SUC_NE_lexicon_tags_2 },
    { 0xebb1b784, SUC_NE_lexicon_tags_2 },
    { 0x2d6fb786, SUC_NE_lexicon_tags_8 },
    { 0xc4d7b786, SUC_NE_lexicon_tags_2 },
    { 0xb61b787, SUC_NE_lexicon_tags_2 },
    { 0x7c03b787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b9b78e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f71b790, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc191b797, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe267b79a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cbb79f, SUC_NE_lexicon_tags_2 },
    { 0x31b7b79f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cafb7a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f9b7ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7b7b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c49b7b5, SUC_NE_lexicon_tags_2 },
    { 0x8803b7b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e9b7c2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce7b7c5, SUC_NE_lexicon_tags_2 },
    { 0xcb01b7c6, SUC_NE_lexicon_tags_2 },
    { 0xbfe9b7c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2495b7c9, SUC_NE_lexicon_tags_2 },
    { 0x42d5b7ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194db7d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2435b7d4, SUC_NE_lexicon_tags_2 },
    { 0x4f2fb7d5, SUC_NE_lexicon_tags_2 },
    { 0xf14db7d6, SUC_NE_lexicon_tags_2 },
    { 0xbc5bb7d6, SUC_NE_lexicon_tags_2 },
    { 0xd9bbb7d8, SUC_NE_lexicon_tags_2 },
    { 0xadbdb7d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77c9b7db, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x60db7dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccadb7df, SUC_NE_lexicon_tags_2 },
    { 0x9f57b7df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e27b7e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2867b7ec, SUC_NE_lexicon_tags_18 },
    { 0x7d3b7ed, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c53b7ef, SUC_NE_lexicon_tags_2 },
    { 0x7079b7f0, SUC_NE_lexicon_tags_2 },
    { 0xa931b7f1, SUC_NE_lexicon_tags_2 },
    { 0xb991b7f2, SUC_NE_lexicon_tags_12 },
    { 0xc361b7f1, SUC_NE_lexicon_tags_2 },
    { 0xecd7b7f4, SUC_NE_lexicon_tags_1 },
    { 0xbbc9b7f0, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x74efb7f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef9b7f9, SUC_NE_lexicon_tags_36 },
    { 0xb6bfb7fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63b801, SUC_NE_lexicon_tags_2 },
    { 0xbf17b802, SUC_NE_lexicon_tags_2 },
    { 0x114db803, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cbb807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc3b80a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f89b80c, SUC_NE_lexicon_tags_2 },
    { 0xc00bb80c, SUC_NE_lexicon_tags_2 },
    { 0xc809b80e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff39b810, SUC_NE_lexicon_tags_2 },
    { 0xc6f9b811, SUC_NE_lexicon_tags_2 },
    { 0x3c6bb810, SUC_NE_lexicon_tags_2 },
    { 0xbc59b813, SUC_NE_lexicon_tags_4 },
    { 0xe01fb810, SUC_NE_lexicon_tags_2 },
    { 0xeaefb815, SUC_NE_lexicon_tags_2 },
    { 0x612bb811, SUC_NE_lexicon_tags_2 },
    { 0xad6db816, SUC_NE_lexicon_tags_2 },
    { 0x6fe1b818, SUC_NE_lexicon_tags_2 },
    { 0xbd47b819, SUC_NE_lexicon_tags_3 },
    { 0x467bb819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b7b81d, SUC_NE_lexicon_tags_7 },
    { 0xf9d5b81e, SUC_NE_lexicon_tags_2 },
    { 0xd159b81f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cfb822, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa281b824, SUC_NE_lexicon_tags_2 },
    { 0xbbdbb825, SUC_NE_lexicon_tags_2 },
    { 0x44a1b826, SUC_NE_lexicon_tags_2 },
    { 0x7fcbb827, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf78bb829, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd083b82c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa36bb82e, SUC_NE_lexicon_tags_3 },
    { 0x8ac1b82f, SUC_NE_lexicon_tags_2 },
    { 0x67c3b82f, SUC_NE_lexicon_tags_98 },
    { 0x4149b830, SUC_NE_lexicon_tags_2 },
    { 0x5be7b831, SUC_NE_lexicon_tags_2 },
    { 0xa9a3b833, SUC_NE_lexicon_tags_2 },
    { 0x3f8fb834, SUC_NE_lexicon_tags_2 },
    { 0x84bfb835, SUC_NE_lexicon_tags_2 },
    { 0x8021b833, SUC_NE_lexicon_tags_2 },
    { 0x4e43b834, SUC_NE_lexicon_tags_2 },
    { 0xa887b835, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053b83f, SUC_NE_lexicon_tags_2 },
    { 0x5b7db840, SUC_NE_lexicon_tags_2 },
    { 0xabdbb83f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25bbb843, SUC_NE_lexicon_tags_2 },
    { 0x81a9b844, SUC_NE_lexicon_tags_2 },
    { 0x2d81b845, SUC_NE_lexicon_tags_2 },
    { 0x76f9b846, SUC_NE_lexicon_tags_34 },
    { 0x9415b845, SUC_NE_lexicon_tags_2 },
    { 0xccddb848, SUC_NE_lexicon_tags_2 },
    { 0x95bb849, SUC_NE_lexicon_tags_2 },
    { 0x9b09b847, SUC_NE_lexicon_tags_2 },
    { 0x48d5b848, SUC_NE_lexicon_tags_2 },
    { 0x9505b844, SUC_NE_lexicon_tags_0 },
    { 0x1301b844, SUC_NE_lexicon_tags_2 },
    { 0x23b3b84e, SUC_NE_lexicon_tags_2 },
    { 0x665b847, SUC_NE_lexicon_tags_400 },
    { 0x168fb848, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2b1b852, SUC_NE_lexicon_tags_2 },
    { 0xe9e7b853, SUC_NE_lexicon_tags_3 },
    { 0xdddfb852, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7db859, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac69b85b, SUC_NE_lexicon_tags_2 },
    { 0x582bb85b, SUC_NE_lexicon_tags_2 },
    { 0x2fd7b85b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4abb861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d7b86b, SUC_NE_lexicon_tags_2 },
    { 0xe85db86c, SUC_NE_lexicon_tags_2 },
    { 0xcbd1b86d, SUC_NE_lexicon_tags_2 },
    { 0x5b69b86e, SUC_NE_lexicon_tags_135 },
    { 0, NULL },
    { 0x8cfbb870, SUC_NE_lexicon_tags_2 },
    { 0xb457b871, SUC_NE_lexicon_tags_2 },
    { 0x9219b871, SUC_NE_lexicon_tags_5 },
    { 0x9727b873, SUC_NE_lexicon_tags_7 },
    { 0x4f21b874, SUC_NE_lexicon_tags_2 },
    { 0x21bbb873, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x9951b877, SUC_NE_lexicon_tags_2 },
    { 0x3af7b878, SUC_NE_lexicon_tags_2 },
    { 0x1a43b879, SUC_NE_lexicon_tags_2 },
    { 0x8637b87a, SUC_NE_lexicon_tags_2 },
    { 0x240fb87b, SUC_NE_lexicon_tags_2 },
    { 0xec09b879, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e35b87e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad7b885, SUC_NE_lexicon_tags_30 },
    { 0x4875b886, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0db890, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33db892, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbabb895, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f85b897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe5b89e, SUC_NE_lexicon_tags_2 },
    { 0xa075b89f, SUC_NE_lexicon_tags_2 },
    { 0x5b63b89f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5dfb8a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3c5b8a4, SUC_NE_lexicon_tags_2 },
    { 0x4a4bb8a5, SUC_NE_lexicon_tags_3 },
    { 0x9607b8a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8db8a9, SUC_NE_lexicon_tags_2 },
    { 0xd16fb8a9, SUC_NE_lexicon_tags_2 },
    { 0x4797b8ab, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35db8ae, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xb0c5b8b0, SUC_NE_lexicon_tags_3 },
    { 0x12f3b8b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fbb8b4, SUC_NE_lexicon_tags_2 },
    { 0x7f5bb8b5, SUC_NE_lexicon_tags_7 },
    { 0x7b9bb8b4, SUC_NE_lexicon_tags_2 },
    { 0x548bb8b7, SUC_NE_lexicon_tags_2 },
    { 0xa039b8b6, SUC_NE_lexicon_tags_2 },
    { 0x2391b8b9, SUC_NE_lexicon_tags_2 },
    { 0xd3a1b8ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb53b8bc, SUC_NE_lexicon_tags_7 },
    { 0x2e0fb8bd, SUC_NE_lexicon_tags_7 },
    { 0xc083b8be, SUC_NE_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ebb8c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2c1b8c4, SUC_NE_lexicon_tags_2 },
    { 0xf2d5b8c5, SUC_NE_lexicon_tags_2 },
    { 0xe931b8c5, SUC_NE_lexicon_tags_2 },
    { 0x4313b8c4, SUC_NE_lexicon_tags_2 },
    { 0x81fb8c8, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b25b8ca, SUC_NE_lexicon_tags_2 },
    { 0x3de5b8cb, SUC_NE_lexicon_tags_2 },
    { 0xe289b8cc, SUC_NE_lexicon_tags_2 },
    { 0xe01db8cd, SUC_NE_lexicon_tags_2 },
    { 0xa911b8ce, SUC_NE_lexicon_tags_2 },
    { 0x2751b8ce, SUC_NE_lexicon_tags_2 },
    { 0xff15b8cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd657b8d2, SUC_NE_lexicon_tags_2 },
    { 0xd87db8d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1459b8d8, SUC_NE_lexicon_tags_2 },
    { 0x4d35b8d8, SUC_NE_lexicon_tags_2 },
    { 0xc8f3b8da, SUC_NE_lexicon_tags_4 },
    { 0xe071b8d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe183b8df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6513b8e1, SUC_NE_lexicon_tags_19 },
    { 0xeb71b8e2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf255b8e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e3b8e9, SUC_NE_lexicon_tags_8 },
    { 0x7a5fb8ea, SUC_NE_lexicon_tags_2 },
    { 0x38edb8e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf057b8ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x729bb8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1887b8f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6db8f4, SUC_NE_lexicon_tags_2 },
    { 0xfdc9b8f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd973b8f8, SUC_NE_lexicon_tags_2 },
    { 0x65a9b8f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87cbb8fb, SUC_NE_lexicon_tags_2 },
    { 0x769db8fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351bb901, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xd5d7b903, SUC_NE_lexicon_tags_2 },
    { 0x7ccbb904, SUC_NE_lexicon_tags_2 },
    { 0x7933b905, SUC_NE_lexicon_tags_2 },
    { 0xa37db903, SUC_NE_lexicon_tags_2 },
    { 0xdb41b907, SUC_NE_lexicon_tags_2 },
    { 0xb693b908, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffefb90b, SUC_NE_lexicon_tags_2 },
    { 0xfdc7b90b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x932bb90e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba79b911, SUC_NE_lexicon_tags_2 },
    { 0xba9db912, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x637bb914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5633b916, SUC_NE_lexicon_tags_2 },
    { 0x759db917, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d69b91d, SUC_NE_lexicon_tags_2 },
    { 0x62c3b91d, SUC_NE_lexicon_tags_5 },
    { 0xdeb9b91f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x1551b921, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b5b925, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1b92b, SUC_NE_lexicon_tags_1 },
    { 0xb8fbb92c, SUC_NE_lexicon_tags_2 },
    { 0x2253b92b, SUC_NE_lexicon_tags_13 },
    { 0x3027b92c, SUC_NE_lexicon_tags_7 },
    { 0xe5f3b92f, SUC_NE_lexicon_tags_2 },
    { 0x1911b930, SUC_NE_lexicon_tags_2 },
    { 0xdd33b931, SUC_NE_lexicon_tags_2 },
    { 0x1d0bb92e, SUC_NE_lexicon_tags_2 },
    { 0x10dbb933, SUC_NE_lexicon_tags_2 },
    { 0x890db92e, SUC_NE_lexicon_tags_2 },
    { 0xce61b932, SUC_NE_lexicon_tags_2 },
    { 0xc55b936, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2287b93a, SUC_NE_lexicon_tags_2 },
    { 0x6d77b93b, SUC_NE_lexicon_tags_2 },
    { 0xb22bb93c, SUC_NE_lexicon_tags_2 },
    { 0x5fa1b93d, SUC_NE_lexicon_tags_4 },
    { 0xe191b93b, SUC_NE_lexicon_tags_2 },
    { 0x928db93f, SUC_NE_lexicon_tags_2 },
    { 0x8e65b940, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a9b945, SUC_NE_lexicon_tags_2 },
    { 0x8439b946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62f3b948, SUC_NE_lexicon_tags_2 },
    { 0x9f93b948, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1b94c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b85b94e, SUC_NE_lexicon_tags_2 },
    { 0xdd2db94e, SUC_NE_lexicon_tags_2 },
    { 0xe701b950, SUC_NE_lexicon_tags_2 },
    { 0x7687b950, SUC_NE_lexicon_tags_2 },
    { 0x7da7b950, SUC_NE_lexicon_tags_0 },
    { 0x1fbbb953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4853b955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb95a, SUC_NE_lexicon_tags_2 },
    { 0x774fb95a, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xd03db95d, SUC_NE_lexicon_tags_2 },
    { 0xc85fb95e, SUC_NE_lexicon_tags_2 },
    { 0x5bb9b95f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60abb963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bbb965, SUC_NE_lexicon_tags_2 },
    { 0xcf9db965, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5377b968, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4331b96b, SUC_NE_lexicon_tags_4 },
    { 0xcf9fb96c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bbb973, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c3b979, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x10c5b97b, SUC_NE_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd737b97f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fb982, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x802bb985, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b69b988, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5e3b98a, SUC_NE_lexicon_tags_2 },
    { 0xafe7b98b, SUC_NE_lexicon_tags_2 },
    { 0x4c27b98c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c5b98f, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9027b992, SUC_NE_lexicon_tags_2 },
    { 0xc939b993, SUC_NE_lexicon_tags_2 },
    { 0xacbb994, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f5b997, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1041b999, SUC_NE_lexicon_tags_2 },
    { 0x813bb99a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xff39b99c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bdfb99e, SUC_NE_lexicon_tags_2 },
    { 0xd3d5b99f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc57b9a1, SUC_NE_lexicon_tags_2 },
    { 0xc765b9a2, SUC_NE_lexicon_tags_2 },
    { 0x1dc7b9a3, SUC_NE_lexicon_tags_2 },
    { 0xd9bbb9a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1bb9a9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xf60bb9ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7477b9ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1749b9af, SUC_NE_lexicon_tags_4 },
    { 0xcc39b9b0, SUC_NE_lexicon_tags_2 },
    { 0xfbabb9b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5123b9b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8fb9b7, SUC_NE_lexicon_tags_2 },
    { 0xc39db9b8, SUC_NE_lexicon_tags_2 },
    { 0x92dbb9b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb1db9bb, SUC_NE_lexicon_tags_2 },
    { 0x3cbbb9bc, SUC_NE_lexicon_tags_98 },
    { 0x6875b9bb, SUC_NE_lexicon_tags_2 },
    { 0xa685b9be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc777b9c0, SUC_NE_lexicon_tags_2 },
    { 0x4825b9c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb899b9c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781b9c6, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x243db9c9, SUC_NE_lexicon_tags_2 },
    { 0x2da3b9c9, SUC_NE_lexicon_tags_2 },
    { 0xabc7b9c9, SUC_NE_lexicon_tags_2 },
    { 0x321fb9cc, SUC_NE_lexicon_tags_24 },
    { 0x41f5b9cd, SUC_NE_lexicon_tags_2 },
    { 0xc16db9ce, SUC_NE_lexicon_tags_2 },
    { 0x8f19b9cf, SUC_NE_lexicon_tags_2 },
    { 0xdb5fb9ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2369b9d3, SUC_NE_lexicon_tags_4 },
    { 0xb36bb9d3, SUC_NE_lexicon_tags_2 },
    { 0x1d6bb9d4, SUC_NE_lexicon_tags_2 },
    { 0x341db9d3, SUC_NE_lexicon_tags_2 },
    { 0x2fd3b9d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15fb9dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e1b9e1, SUC_NE_lexicon_tags_2 },
    { 0x20ddb9e2, SUC_NE_lexicon_tags_2 },
    { 0x18a7b9e3, SUC_NE_lexicon_tags_2 },
    { 0x900bb9e4, SUC_NE_lexicon_tags_2 },
    { 0x18a3b9e3, SUC_NE_lexicon_tags_2 },
    { 0x7111b9e6, SUC_NE_lexicon_tags_2 },
    { 0xf9e9b9e4, SUC_NE_lexicon_tags_2 },
    { 0xd0bfb9e3, SUC_NE_lexicon_tags_2 },
    { 0xd627b9e1, SUC_NE_lexicon_tags_2 },
    { 0x7b49b9e5, SUC_NE_lexicon_tags_2 },
    { 0xfcdb9eb, SUC_NE_lexicon_tags_2 },
    { 0x3ba1b9eb, SUC_NE_lexicon_tags_2 },
    { 0x809bb9ed, SUC_NE_lexicon_tags_2 },
    { 0xeccfb9ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb70fb9f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3d7b9f2, SUC_NE_lexicon_tags_2 },
    { 0xdd5db9f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d09b9f5, SUC_NE_lexicon_tags_2 },
    { 0x7969b9f5, SUC_NE_lexicon_tags_2 },
    { 0xf745b9f7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xef97b9f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61adb9fb, SUC_NE_lexicon_tags_32 },
    { 0x23afb9fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fa5b9fe, SUC_NE_lexicon_tags_2 },
    { 0xafe5b9ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba3ba02, SUC_NE_lexicon_tags_2 },
    { 0x66bbba02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba87ba07, SUC_NE_lexicon_tags_2 },
    { 0x70bdba08, SUC_NE_lexicon_tags_2 },
    { 0x4647ba09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9dba0f, SUC_NE_lexicon_tags_7 },
    { 0x61b7ba10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186fba17, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dfba1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffbba24, SUC_NE_lexicon_tags_2 },
    { 0x1991ba24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1ba2b, SUC_NE_lexicon_tags_2 },
    { 0xaef5ba2c, SUC_NE_lexicon_tags_2 },
    { 0xf57fba2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8bba30, SUC_NE_lexicon_tags_2 },
    { 0x52adba31, SUC_NE_lexicon_tags_2 },
    { 0xd667ba32, SUC_NE_lexicon_tags_2 },
    { 0x6b41ba33, SUC_NE_lexicon_tags_2 },
    { 0x511ba31, SUC_NE_lexicon_tags_2 },
    { 0x80a5ba35, SUC_NE_lexicon_tags_4 },
    { 0xc091ba36, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xea43ba38, SUC_NE_lexicon_tags_2 },
    { 0xce0dba38, SUC_NE_lexicon_tags_2 },
    { 0x8565ba38, SUC_NE_lexicon_tags_2 },
    { 0xa97ba39, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f8dba3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55bba3f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1dba42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a6bba44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb9ba46, SUC_NE_lexicon_tags_2 },
    { 0x85ffba47, SUC_NE_lexicon_tags_2 },
    { 0x7fafba48, SUC_NE_lexicon_tags_2 },
    { 0x495ba48, SUC_NE_lexicon_tags_2 },
    { 0x919bba48, SUC_NE_lexicon_tags_2 },
    { 0xc7c1ba4b, SUC_NE_lexicon_tags_2 },
    { 0x2087ba49, SUC_NE_lexicon_tags_2 },
    { 0x5c95ba4d, SUC_NE_lexicon_tags_2 },
    { 0x7a27ba4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebddba50, SUC_NE_lexicon_tags_2 },
    { 0xdac1ba51, SUC_NE_lexicon_tags_17 },
    { 0xcafbba52, SUC_NE_lexicon_tags_2 },
    { 0x9e3bba50, SUC_NE_lexicon_tags_2 },
    { 0x7a89ba54, SUC_NE_lexicon_tags_2 },
    { 0xffcfba52, SUC_NE_lexicon_tags_5 },
    { 0x8ed3ba56, SUC_NE_lexicon_tags_2 },
    { 0x9bcdba55, SUC_NE_lexicon_tags_2 },
    { 0xe9ddba58, SUC_NE_lexicon_tags_2 },
    { 0x50f7ba52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe547ba5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x358fba5d, SUC_NE_lexicon_tags_2 },
    { 0x6471ba5e, SUC_NE_lexicon_tags_0 },
    { 0x1dadba5f, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb491ba63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fbba67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9adba6a, SUC_NE_lexicon_tags_2 },
    { 0x26a1ba6b, SUC_NE_lexicon_tags_34 },
    { 0xee21ba6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10b5ba6e, SUC_NE_lexicon_tags_2 },
    { 0xea93ba6e, SUC_NE_lexicon_tags_2 },
    { 0xa461ba70, SUC_NE_lexicon_tags_2 },
    { 0x2f07ba71, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x94ffba73, SUC_NE_lexicon_tags_2 },
    { 0xd69dba74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2783ba7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac55ba7d, SUC_NE_lexicon_tags_2 },
    { 0x51a3ba7d, SUC_NE_lexicon_tags_2 },
    { 0x10ebba7f, SUC_NE_lexicon_tags_2 },
    { 0x9995ba80, SUC_NE_lexicon_tags_2 },
    { 0x6749ba81, SUC_NE_lexicon_tags_2 },
    { 0x7aa1ba82, SUC_NE_lexicon_tags_2 },
    { 0xb6f1ba83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ddba86, SUC_NE_lexicon_tags_4 },
    { 0x66cdba86, SUC_NE_lexicon_tags_3 },
    { 0xf031ba88, SUC_NE_lexicon_tags_2 },
    { 0x580dba88, SUC_NE_lexicon_tags_2 },
    { 0x7f6dba86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb805ba8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd371ba8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433dba92, SUC_NE_lexicon_tags_4 },
    { 0x8abba93, SUC_NE_lexicon_tags_2 },
    { 0x1547ba93, SUC_NE_lexicon_tags_2 },
    { 0x4197ba92, SUC_NE_lexicon_tags_2 },
    { 0x80a9ba95, SUC_NE_lexicon_tags_17 },
    { 0xdbb5ba97, SUC_NE_lexicon_tags_2 },
    { 0x8c9dba98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5045ba9a, SUC_NE_lexicon_tags_75 },
    { 0xba9dba9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee0fba9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe705baa1, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45baad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2417bab1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1695bab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x589fbab6, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0xfb51bab8, SUC_NE_lexicon_tags_2 },
    { 0x80b5bab8, SUC_NE_lexicon_tags_2 },
    { 0x33baba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x485babd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe083babf, SUC_NE_lexicon_tags_2 },
    { 0x2dfdbabf, SUC_NE_lexicon_tags_2 },
    { 0xfa09bac1, SUC_NE_lexicon_tags_2 },
    { 0x7e4dbac0, SUC_NE_lexicon_tags_2 },
    { 0xb72bbac3, SUC_NE_lexicon_tags_17 },
    { 0xd0afbac1, SUC_NE_lexicon_tags_2 },
    { 0x43b3bac5, SUC_NE_lexicon_tags_2 },
    { 0x8a01bac6, SUC_NE_lexicon_tags_3 },
    { 0x9f9bac4, SUC_NE_lexicon_tags_2 },
    { 0xaa5bbac8, SUC_NE_lexicon_tags_2 },
    { 0xb7abbac7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x742dbacc, SUC_NE_lexicon_tags_2 },
    { 0x4b87bacd, SUC_NE_lexicon_tags_2 },
    { 0x26a9bacd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x754bbad1, SUC_NE_lexicon_tags_2 },
    { 0x6c85bad1, SUC_NE_lexicon_tags_109 },
    { 0x3ce5bad1, SUC_NE_lexicon_tags_2 },
    { 0x8977bad3, SUC_NE_lexicon_tags_2 },
    { 0x442dbad4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c1bad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc655badf, SUC_NE_lexicon_tags_2 },
    { 0xccbfbae0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9697bae4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc69bae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d1baeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2585baf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd105baf3, SUC_NE_lexicon_tags_2 },
    { 0xd591baf4, SUC_NE_lexicon_tags_51 },
    { 0x2a7bbaf5, SUC_NE_lexicon_tags_2 },
    { 0xa979baf6, SUC_NE_lexicon_tags_2 },
    { 0x93a3baf4, SUC_NE_lexicon_tags_2 },
    { 0x7d2bbaf8, SUC_NE_lexicon_tags_2 },
    { 0xc5f5baf9, SUC_NE_lexicon_tags_2 },
    { 0xc093bafa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e1bafd, SUC_NE_lexicon_tags_2 },
    { 0x2437bafe, SUC_NE_lexicon_tags_2 },
    { 0x3b2dbaff, SUC_NE_lexicon_tags_2 },
    { 0xc095bb00, SUC_NE_lexicon_tags_2 },
    { 0x7e3fbafd, SUC_NE_lexicon_tags_2 },
    { 0x4fcbbb02, SUC_NE_lexicon_tags_2 },
    { 0x1a1bb02, SUC_NE_lexicon_tags_2 },
    { 0x5013bb01, SUC_NE_lexicon_tags_2 },
    { 0x569fbb05, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xaadbb07, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa6b3bb09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe57bb0b, SUC_NE_lexicon_tags_2 },
    { 0x1a75bb0c, SUC_NE_lexicon_tags_2 },
    { 0x990bbb0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd655bb0f, SUC_NE_lexicon_tags_5 },
    { 0xd263bb10, SUC_NE_lexicon_tags_58 },
    { 0x63f7bb11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41afbb14, SUC_NE_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b3bb19, SUC_NE_lexicon_tags_2 },
    { 0x482dbb1a, SUC_NE_lexicon_tags_229 },
    { 0x220fbb1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7f9bb1d, SUC_NE_lexicon_tags_2 },
    { 0x7d07bb1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf22fbb20, SUC_NE_lexicon_tags_2 },
    { 0x8b8dbb21, SUC_NE_lexicon_tags_3 },
    { 0xb379bb22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ddbbb24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1597bb26, SUC_NE_lexicon_tags_2 },
    { 0xbe91bb27, SUC_NE_lexicon_tags_2 },
    { 0xad87bb26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf08dbb2a, SUC_NE_lexicon_tags_2 },
    { 0x9659bb2b, SUC_NE_lexicon_tags_2 },
    { 0x9a31bb2c, SUC_NE_lexicon_tags_2 },
    { 0xb50bbb2a, SUC_NE_lexicon_tags_2 },
    { 0xd413bb2e, SUC_NE_lexicon_tags_2 },
    { 0x1bc1bb2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc29bbb31, SUC_NE_lexicon_tags_2 },
    { 0xa807bb32, SUC_NE_lexicon_tags_443 },
    { 0, NULL },
    { 0x5bb7bb34, SUC_NE_lexicon_tags_2 },
    { 0x4b4fbb35, SUC_NE_lexicon_tags_2 },
    { 0xc085bb36, SUC_NE_lexicon_tags_2 },
    { 0x8f91bb37, SUC_NE_lexicon_tags_2 },
    { 0x3587bb37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bdbb3c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a85bb3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2725bb44, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd1bb48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9abbb4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f1bb4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x530fbb50, SUC_NE_lexicon_tags_3 },
    { 0xf89bb50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9bb54, SUC_NE_lexicon_tags_2 },
    { 0x6fbb55, SUC_NE_lexicon_tags_18 },
    { 0x936bbb55, SUC_NE_lexicon_tags_1 },
    { 0x237fbb57, SUC_NE_lexicon_tags_2 },
    { 0x815dbb57, SUC_NE_lexicon_tags_2 },
    { 0x6811bb59, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0xac47bb5b, SUC_NE_lexicon_tags_2 },
    { 0x9a2fbb5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4515bb5e, SUC_NE_lexicon_tags_2 },
    { 0xa319bb5e, SUC_NE_lexicon_tags_2 },
    { 0xd1bfbb5e, SUC_NE_lexicon_tags_2 },
    { 0x7ad7bb61, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b3bb64, SUC_NE_lexicon_tags_2 },
    { 0x30dbb64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb54fbb67, SUC_NE_lexicon_tags_2 },
    { 0xf849bb68, SUC_NE_lexicon_tags_2 },
    { 0x7addbb67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe041bb6b, SUC_NE_lexicon_tags_2 },
    { 0x3e47bb6c, SUC_NE_lexicon_tags_2 },
    { 0x2c1bbb6b, SUC_NE_lexicon_tags_2 },
    { 0x4959bb6b, SUC_NE_lexicon_tags_2 },
    { 0xd01fbb6f, SUC_NE_lexicon_tags_2 },
    { 0xc1f9bb70, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xd65dbb72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf757bb74, SUC_NE_lexicon_tags_2 },
    { 0xb75bb74, SUC_NE_lexicon_tags_2 },
    { 0x603dbb75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f91bb78, SUC_NE_lexicon_tags_1 },
    { 0xd9bb79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fbbb7d, SUC_NE_lexicon_tags_2 },
    { 0x8a59bb7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd95bb80, SUC_NE_lexicon_tags_2 },
    { 0xbadfbb81, SUC_NE_lexicon_tags_2 },
    { 0xd983bb82, SUC_NE_lexicon_tags_2 },
    { 0xfbbfbb83, SUC_NE_lexicon_tags_3 },
    { 0xb42fbb80, SUC_NE_lexicon_tags_2 },
    { 0x1411bb85, SUC_NE_lexicon_tags_2 },
    { 0xbcc5bb86, SUC_NE_lexicon_tags_2 },
    { 0x2e43bb87, SUC_NE_lexicon_tags_2 },
    { 0x8d61bb88, SUC_NE_lexicon_tags_2 },
    { 0xca35bb89, SUC_NE_lexicon_tags_2 },
    { 0x43e1bb81, SUC_NE_lexicon_tags_18 },
    { 0x7129bb8b, SUC_NE_lexicon_tags_2 },
    { 0x9c9bbb8b, SUC_NE_lexicon_tags_2 },
    { 0x2cc3bb8d, SUC_NE_lexicon_tags_2 },
    { 0xa9c3bb8e, SUC_NE_lexicon_tags_2 },
    { 0x9637bb8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dfbb92, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x59d7bb94, SUC_NE_lexicon_tags_2 },
    { 0xfcf1bb95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x44c7bb97, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f31bb9b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9eedbb9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a5bb9f, SUC_NE_lexicon_tags_188 },
    { 0x6781bba0, SUC_NE_lexicon_tags_2 },
    { 0xb981bba0, SUC_NE_lexicon_tags_2 },
    { 0x4047bba2, SUC_NE_lexicon_tags_2 },
    { 0x6953bba3, SUC_NE_lexicon_tags_2 },
    { 0x8005bb9f, SUC_NE_lexicon_tags_2 },
    { 0xc17dbba4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ab1bba7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa1bba9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe83bbab, SUC_NE_lexicon_tags_2 },
    { 0xabd5bbab, SUC_NE_lexicon_tags_2 },
    { 0xd311bbac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d7bbb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1dbbb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45dbbbc, SUC_NE_lexicon_tags_2 },
    { 0xabfbbbc, SUC_NE_lexicon_tags_3 },
    { 0x3a3dbbbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa649bbc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf57bbc6, SUC_NE_lexicon_tags_2 },
    { 0xafc3bbc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x225bbbca, SUC_NE_lexicon_tags_323 },
    { 0x7335bbcb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800bbbd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc997bbd2, SUC_NE_lexicon_tags_2 },
    { 0x43bfbbd3, SUC_NE_lexicon_tags_2 },
    { 0x378dbbd3, SUC_NE_lexicon_tags_3 },
    { 0xc467bbd2, SUC_NE_lexicon_tags_2 },
    { 0x56a5bbd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x900dbbd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49bbdd, SUC_NE_lexicon_tags_4 },
    { 0xbe5fbbde, SUC_NE_lexicon_tags_24 },
    { 0x61b5bbdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39abbbe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f1bbe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997dbbea, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a7bbee, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a97bbf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1afbbf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285dbbf7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xea63bbfa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb655bbfc, SUC_NE_lexicon_tags_2 },
    { 0x7a1fbbfd, SUC_NE_lexicon_tags_2 },
    { 0x7e39bbfe, SUC_NE_lexicon_tags_2 },
    { 0xa8a9bbff, SUC_NE_lexicon_tags_32 },
    { 0x3c07bc00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f1fbc02, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb5bc05, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dadbc0d, SUC_NE_lexicon_tags_2 },
    { 0x589bc0e, SUC_NE_lexicon_tags_7 },
    { 0xf977bc0e, SUC_NE_lexicon_tags_2 },
    { 0x9b69bc10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe283bc17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa29bc19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad77bc1e, SUC_NE_lexicon_tags_2 },
    { 0xccedbc1f, SUC_NE_lexicon_tags_2 },
    { 0xe49fbc20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d3bc27, SUC_NE_lexicon_tags_2 },
    { 0x93dbc28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181bc2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde57bc31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x880fbc34, SUC_NE_lexicon_tags_2 },
    { 0xc43bc34, SUC_NE_lexicon_tags_2 },
    { 0x6639bc36, SUC_NE_lexicon_tags_2 },
    { 0xd5c9bc37, SUC_NE_lexicon_tags_2 },
    { 0x6c9fbc36, SUC_NE_lexicon_tags_2 },
    { 0x6a6dbc39, SUC_NE_lexicon_tags_2 },
    { 0x8dbfbc3a, SUC_NE_lexicon_tags_2 },
    { 0x46fbc3b, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3fbc3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dfbc42, SUC_NE_lexicon_tags_2 },
    { 0x72d9bc43, SUC_NE_lexicon_tags_2 },
    { 0x4705bc44, SUC_NE_lexicon_tags_2 },
    { 0xba01bc42, SUC_NE_lexicon_tags_2 },
    { 0xc21bbc44, SUC_NE_lexicon_tags_2 },
    { 0x4affbc47, SUC_NE_lexicon_tags_2 },
    { 0xd983bc48, SUC_NE_lexicon_tags_2 },
    { 0xcc93bc48, SUC_NE_lexicon_tags_2 },
    { 0xe86bbc4a, SUC_NE_lexicon_tags_2 },
    { 0x4a69bc4b, SUC_NE_lexicon_tags_2 },
    { 0x9659bc4a, SUC_NE_lexicon_tags_2 },
    { 0xa661bc4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x952dbc50, SUC_NE_lexicon_tags_2 },
    { 0x7501bc51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46bbbc53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2fbc57, SUC_NE_lexicon_tags_2 },
    { 0x3c7dbc58, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496fbc5c, SUC_NE_lexicon_tags_2 },
    { 0x63ffbc5c, SUC_NE_lexicon_tags_2 },
    { 0xe00bbc5e, SUC_NE_lexicon_tags_4 },
    { 0xd3d1bc5f, SUC_NE_lexicon_tags_2 },
    { 0x2595bc5f, SUC_NE_lexicon_tags_2 },
    { 0x39bc61, SUC_NE_lexicon_tags_2 },
    { 0x9da7bc5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe163bc66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4729bc69, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5243bc6c, SUC_NE_lexicon_tags_2 },
    { 0x6b55bc6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1003bc6f, SUC_NE_lexicon_tags_2 },
    { 0x5e6bbc6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a15bc72, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xf74fbc74, SUC_NE_lexicon_tags_2 },
    { 0x7c5dbc75, SUC_NE_lexicon_tags_2 },
    { 0x54a5bc74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ddbc7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5123bc80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1dfbc82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a5bc84, SUC_NE_lexicon_tags_2 },
    { 0x6b11bc84, SUC_NE_lexicon_tags_2 },
    { 0xc519bc85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1ffbc88, SUC_NE_lexicon_tags_4 },
    { 0x79ddbc88, SUC_NE_lexicon_tags_2 },
    { 0x90cbbc8a, SUC_NE_lexicon_tags_2 },
    { 0x22e9bc8b, SUC_NE_lexicon_tags_2 },
    { 0x92fbc8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa465bc90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3bc94, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x29bbbc96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71bca1, SUC_NE_lexicon_tags_2 },
    { 0xf74bbca2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbca8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa553bcaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b75bcad, SUC_NE_lexicon_tags_2 },
    { 0x8acfbcae, SUC_NE_lexicon_tags_2 },
    { 0x7c57bcaf, SUC_NE_lexicon_tags_2 },
    { 0x5317bcaf, SUC_NE_lexicon_tags_2 },
    { 0x5b91bcb1, SUC_NE_lexicon_tags_2 },
    { 0x8165bcae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1347bcba, SUC_NE_lexicon_tags_2 },
    { 0xa827bcbb, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dfbcbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98afbccb, SUC_NE_lexicon_tags_2 },
    { 0x68dbccc, SUC_NE_lexicon_tags_2 },
    { 0xc3d9bccb, SUC_NE_lexicon_tags_2 },
    { 0xc54bbcce, SUC_NE_lexicon_tags_7 },
    { 0xc785bccb, SUC_NE_lexicon_tags_2 },
    { 0x4f33bccc, SUC_NE_lexicon_tags_1 },
    { 0x42f5bccf, SUC_NE_lexicon_tags_2 },
    { 0xa875bcd2, SUC_NE_lexicon_tags_2 },
    { 0x664fbcd3, SUC_NE_lexicon_tags_2 },
    { 0xb51bcd4, SUC_NE_lexicon_tags_4 },
    { 0xead9bcd5, SUC_NE_lexicon_tags_2 },
    { 0xf64dbcd6, SUC_NE_lexicon_tags_2 },
    { 0xf6bfbcd7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1a1bcd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde5bcdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x281bbcdd, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x27fbbcdf, SUC_NE_lexicon_tags_2 },
    { 0x9a61bce0, SUC_NE_lexicon_tags_2 },
    { 0x8b15bce1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e87bce3, SUC_NE_lexicon_tags_64 },
    { 0xcd9fbce4, SUC_NE_lexicon_tags_2 },
    { 0x9ce7bce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa087bce9, SUC_NE_lexicon_tags_2 },
    { 0xc9d5bce9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb47bcec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f4bbcee, SUC_NE_lexicon_tags_2 },
    { 0x91b7bcef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe247bcf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5009bcf7, SUC_NE_lexicon_tags_14 },
    { 0xc183bcf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a7bcfe, SUC_NE_lexicon_tags_2 },
    { 0x8ddbbcfe, SUC_NE_lexicon_tags_2 },
    { 0x5e59bcff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5593bd03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b7bd08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a2bbd0a, SUC_NE_lexicon_tags_2 },
    { 0x65abbd0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4643bd10, SUC_NE_lexicon_tags_2 },
    { 0xa529bd11, SUC_NE_lexicon_tags_2 },
    { 0xd413bd12, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd33bd14, SUC_NE_lexicon_tags_2 },
    { 0x3763bd15, SUC_NE_lexicon_tags_2 },
    { 0xbaa5bd15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a17bd1e, SUC_NE_lexicon_tags_2 },
    { 0x88cbbd1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1b9bd21, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x36afbd24, SUC_NE_lexicon_tags_2 },
    { 0x35cdbd25, SUC_NE_lexicon_tags_19 },
    { 0x992bbd26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcfbd2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7bfbd2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9bd2f, SUC_NE_lexicon_tags_4 },
    { 0xde5dbd2f, SUC_NE_lexicon_tags_7 },
    { 0x3479bd31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749bbd35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa341bd3a, SUC_NE_lexicon_tags_2 },
    { 0x89e3bd3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe705bd3d, SUC_NE_lexicon_tags_2 },
    { 0xab25bd3d, SUC_NE_lexicon_tags_2 },
    { 0x597bbd3e, SUC_NE_lexicon_tags_2 },
    { 0x63bfbd3d, SUC_NE_lexicon_tags_2 },
    { 0x220bbd3f, SUC_NE_lexicon_tags_2 },
    { 0x668fbd42, SUC_NE_lexicon_tags_219 },
    { 0, NULL },
    { 0xe963bd44, SUC_NE_lexicon_tags_2 },
    { 0xd88fbd44, SUC_NE_lexicon_tags_2 },
    { 0xcfbd46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9fbd4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff1bd4f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xda7bd51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd775bd53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d3bd55, SUC_NE_lexicon_tags_2 },
    { 0x4a63bd56, SUC_NE_lexicon_tags_2 },
    { 0xfc7bbd57, SUC_NE_lexicon_tags_2 },
    { 0x92b9bd57, SUC_NE_lexicon_tags_2 },
    { 0xfa5bd59, SUC_NE_lexicon_tags_2 },
    { 0xfcddbd5a, SUC_NE_lexicon_tags_2 },
    { 0x903fbd5b, SUC_NE_lexicon_tags_2 },
    { 0x37d9bd5c, SUC_NE_lexicon_tags_2 },
    { 0xc299bd5c, SUC_NE_lexicon_tags_2 },
    { 0xc4a7bd5c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dabbd61, SUC_NE_lexicon_tags_2 },
    { 0x4ae7bd61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe53bd67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a47bd69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x831dbd6b, SUC_NE_lexicon_tags_2 },
    { 0xe18fbd6c, SUC_NE_lexicon_tags_2 },
    { 0x7961bd6d, SUC_NE_lexicon_tags_2 },
    { 0x7103bd6d, SUC_NE_lexicon_tags_2 },
    { 0x2eb1bd6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcbbd72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4635bd74, SUC_NE_lexicon_tags_2 },
    { 0xa04dbd74, SUC_NE_lexicon_tags_2 },
    { 0x2e01bd75, SUC_NE_lexicon_tags_2 },
    { 0x5077bd77, SUC_NE_lexicon_tags_1 },
    { 0x6a59bd78, SUC_NE_lexicon_tags_2 },
    { 0x71d5bd79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb23bd7c, SUC_NE_lexicon_tags_2 },
    { 0xee07bd7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18fbd85, SUC_NE_lexicon_tags_2 },
    { 0x60edbd85, SUC_NE_lexicon_tags_2 },
    { 0xd8d9bd86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223bbd8c, SUC_NE_lexicon_tags_2 },
    { 0x6413bd8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9bd91, SUC_NE_lexicon_tags_2 },
    { 0xdc3bbd92, SUC_NE_lexicon_tags_2 },
    { 0x505dbd93, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xb207bd95, SUC_NE_lexicon_tags_2 },
    { 0x4813bd96, SUC_NE_lexicon_tags_2 },
    { 0xaabdbd97, SUC_NE_lexicon_tags_2 },
    { 0x360dbd96, SUC_NE_lexicon_tags_2 },
    { 0x72a9bd99, SUC_NE_lexicon_tags_107 },
    { 0x4f9fbd9a, SUC_NE_lexicon_tags_2 },
    { 0xb2ddbd9b, SUC_NE_lexicon_tags_2 },
    { 0x849fbd97, SUC_NE_lexicon_tags_2 },
    { 0x359bd9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9445bda0, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb927bdb0, SUC_NE_lexicon_tags_2 },
    { 0x621bdb1, SUC_NE_lexicon_tags_2 },
    { 0x4c43bdb2, SUC_NE_lexicon_tags_51 },
    { 0xc663bdb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9bdb6, SUC_NE_lexicon_tags_2 },
    { 0x4383bdb6, SUC_NE_lexicon_tags_7 },
    { 0xea8dbdb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce87bdba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b7bdc0, SUC_NE_lexicon_tags_2 },
    { 0x261dbdc1, SUC_NE_lexicon_tags_2 },
    { 0xe739bdc2, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c7bdc7, SUC_NE_lexicon_tags_2 },
    { 0xcc1dbdc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf3bdcb, SUC_NE_lexicon_tags_2 },
    { 0x1d4bbdcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbc3bdce, SUC_NE_lexicon_tags_2 },
    { 0x80c3bdcf, SUC_NE_lexicon_tags_2 },
    { 0x11f5bdd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5bdd4, SUC_NE_lexicon_tags_2 },
    { 0xc213bdd4, SUC_NE_lexicon_tags_2 },
    { 0x4699bdd6, SUC_NE_lexicon_tags_2 },
    { 0xcf2dbdd7, SUC_NE_lexicon_tags_7 },
    { 0x84cfbdd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf457bddd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x229bbde0, SUC_NE_lexicon_tags_2 },
    { 0xd7a5bde1, SUC_NE_lexicon_tags_2 },
    { 0x39efbde0, SUC_NE_lexicon_tags_2 },
    { 0x9069bde0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d91bde6, SUC_NE_lexicon_tags_2 },
    { 0xfd27bde7, SUC_NE_lexicon_tags_2 },
    { 0xd5fdbde8, SUC_NE_lexicon_tags_2 },
    { 0x848bbde8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259dbdf1, SUC_NE_lexicon_tags_2 },
    { 0x9d1bbdf2, SUC_NE_lexicon_tags_4 },
    { 0x241fbdf3, SUC_NE_lexicon_tags_2 },
    { 0x56d9bdf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x846bbdf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4dbdff, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf053be06, SUC_NE_lexicon_tags_0 },
    { 0x989bbe07, SUC_NE_lexicon_tags_2 },
    { 0x8435be07, SUC_NE_lexicon_tags_2 },
    { 0x3451be07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5581be0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed3be0e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f49be10, SUC_NE_lexicon_tags_2 },
    { 0x640dbe10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee3be15, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1181be1a, SUC_NE_lexicon_tags_2 },
    { 0xdd5dbe1a, SUC_NE_lexicon_tags_2 },
    { 0xb071be1c, SUC_NE_lexicon_tags_2 },
    { 0xf0f1be1d, SUC_NE_lexicon_tags_2 },
    { 0xdad1be1e, SUC_NE_lexicon_tags_2 },
    { 0x1031be1e, SUC_NE_lexicon_tags_2 },
    { 0xe1b7be1b, SUC_NE_lexicon_tags_2 },
    { 0x1bcbbe21, SUC_NE_lexicon_tags_2 },
    { 0xf3cfbe20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8badbe2c, SUC_NE_lexicon_tags_201 },
    { 0xe035be2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x184dbe30, SUC_NE_lexicon_tags_2 },
    { 0x27fdbe31, SUC_NE_lexicon_tags_7 },
    { 0x8103be30, SUC_NE_lexicon_tags_2 },
    { 0x6f3bbe33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d91be35, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc85be38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae4fbe3a, SUC_NE_lexicon_tags_2 },
    { 0xe6f7be3b, SUC_NE_lexicon_tags_2 },
    { 0x16f1be3b, SUC_NE_lexicon_tags_2 },
    { 0x1341be3a, SUC_NE_lexicon_tags_2 },
    { 0xc34fbe3e, SUC_NE_lexicon_tags_2 },
    { 0x594fbe3f, SUC_NE_lexicon_tags_2 },
    { 0x6423be3c, SUC_NE_lexicon_tags_13 },
    { 0x6757be41, SUC_NE_lexicon_tags_121 },
    { 0x69bbbe3a, SUC_NE_lexicon_tags_2 },
    { 0x1477be43, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x61f5be45, SUC_NE_lexicon_tags_2 },
    { 0xbc85be46, SUC_NE_lexicon_tags_2 },
    { 0x16d9be45, SUC_NE_lexicon_tags_2 },
    { 0x9c6bbe48, SUC_NE_lexicon_tags_2 },
    { 0x61b9be49, SUC_NE_lexicon_tags_2 },
    { 0xf18dbe49, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7be4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cefbe4f, SUC_NE_lexicon_tags_2 },
    { 0x6813be4f, SUC_NE_lexicon_tags_2 },
    { 0x8b43be51, SUC_NE_lexicon_tags_2 },
    { 0x4aa9be50, SUC_NE_lexicon_tags_2 },
    { 0xa35dbe53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe893be55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf05fbe57, SUC_NE_lexicon_tags_2 },
    { 0x941fbe58, SUC_NE_lexicon_tags_3 },
    { 0x5573be59, SUC_NE_lexicon_tags_2 },
    { 0x657bbe5a, SUC_NE_lexicon_tags_2 },
    { 0xb83fbe59, SUC_NE_lexicon_tags_2 },
    { 0x3389be5c, SUC_NE_lexicon_tags_2 },
    { 0xf2ffbe5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5187be5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea1be62, SUC_NE_lexicon_tags_2 },
    { 0x2cb3be63, SUC_NE_lexicon_tags_5 },
    { 0xa08bbe63, SUC_NE_lexicon_tags_2 },
    { 0xc673be65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957be69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731fbe6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb075be71, SUC_NE_lexicon_tags_2 },
    { 0x3f7fbe71, SUC_NE_lexicon_tags_2 },
    { 0x704fbe73, SUC_NE_lexicon_tags_2 },
    { 0xf8a7be74, SUC_NE_lexicon_tags_10 },
    { 0xa5be74, SUC_NE_lexicon_tags_2 },
    { 0xa4bbe72, SUC_NE_lexicon_tags_2 },
    { 0x5321be75, SUC_NE_lexicon_tags_2 },
    { 0x7903be78, SUC_NE_lexicon_tags_2 },
    { 0x89e1be74, SUC_NE_lexicon_tags_2 },
    { 0x6217be76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8041be7e, SUC_NE_lexicon_tags_2 },
    { 0x889bbe7f, SUC_NE_lexicon_tags_2 },
    { 0xb8f3be7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf3be83, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d7be87, SUC_NE_lexicon_tags_2 },
    { 0xe18dbe88, SUC_NE_lexicon_tags_1 },
    { 0xd75dbe89, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db3be8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184fbe93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d81be96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7be9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170dbea2, SUC_NE_lexicon_tags_2 },
    { 0x501fbea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23b7bea5, SUC_NE_lexicon_tags_2 },
    { 0x1edbbea6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe863bea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e6bbeab, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2f53bead, SUC_NE_lexicon_tags_2 },
    { 0x8adfbeae, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0xf419beb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c1beb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715fbeb5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa683bebc, SUC_NE_lexicon_tags_2 },
    { 0x424fbebd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc485bec0, SUC_NE_lexicon_tags_2 },
    { 0xc771bec1, SUC_NE_lexicon_tags_2 },
    { 0x848dbec1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b89bec7, SUC_NE_lexicon_tags_2 },
    { 0xf1d5bec8, SUC_NE_lexicon_tags_18 },
    { 0xb9d5bec9, SUC_NE_lexicon_tags_2 },
    { 0xcdfdbec9, SUC_NE_lexicon_tags_2 },
    { 0xc5f7becb, SUC_NE_lexicon_tags_2 },
    { 0x1acbbecc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x182dbecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561bed5, SUC_NE_lexicon_tags_4 },
    { 0x431dbed6, SUC_NE_lexicon_tags_2 },
    { 0x5c6fbed6, SUC_NE_lexicon_tags_4 },
    { 0x270bbed6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2cbbeda, SUC_NE_lexicon_tags_2 },
    { 0x75afbeda, SUC_NE_lexicon_tags_2 },
    { 0x6acbbedc, SUC_NE_lexicon_tags_2 },
    { 0xedcfbedd, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5dbee0, SUC_NE_lexicon_tags_2 },
    { 0x8c85bee0, SUC_NE_lexicon_tags_2 },
    { 0x98b9bee2, SUC_NE_lexicon_tags_2 },
    { 0xcb47bee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb04fbee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bfbee8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da5beee, SUC_NE_lexicon_tags_2 },
    { 0x6f63beee, SUC_NE_lexicon_tags_0 },
    { 0xb175beee, SUC_NE_lexicon_tags_2 },
    { 0x39a5bef1, SUC_NE_lexicon_tags_49 },
    { 0x8fd9bef2, SUC_NE_lexicon_tags_2 },
    { 0x99adbef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d1bef8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811bbefe, SUC_NE_lexicon_tags_2 },
    { 0xf303beff, SUC_NE_lexicon_tags_1 },
    { 0x4af3beff, SUC_NE_lexicon_tags_2 },
    { 0x366fbf00, SUC_NE_lexicon_tags_2 },
    { 0xce79beff, SUC_NE_lexicon_tags_2 },
    { 0x309fbf03, SUC_NE_lexicon_tags_2 },
    { 0x4ed7bf04, SUC_NE_lexicon_tags_2 },
    { 0x325bf05, SUC_NE_lexicon_tags_2 },
    { 0xfdddbf00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8dbf0c, SUC_NE_lexicon_tags_10 },
    { 0xec39bf0d, SUC_NE_lexicon_tags_2 },
    { 0x648fbf0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508dbf16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0bbf1a, SUC_NE_lexicon_tags_4 },
    { 0x64dbf1a, SUC_NE_lexicon_tags_2 },
    { 0x9219bf1c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xbf21bf1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb5bf21, SUC_NE_lexicon_tags_2 },
    { 0x5ea5bf22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3e5bf24, SUC_NE_lexicon_tags_2 },
    { 0xa399bf24, SUC_NE_lexicon_tags_7 },
    { 0xf53fbf25, SUC_NE_lexicon_tags_2 },
    { 0x7c3dbf27, SUC_NE_lexicon_tags_2 },
    { 0xdae3bf27, SUC_NE_lexicon_tags_2 },
    { 0x4863bf29, SUC_NE_lexicon_tags_17 },
    { 0x2d99bf29, SUC_NE_lexicon_tags_2 },
    { 0x6d07bf2b, SUC_NE_lexicon_tags_2 },
    { 0x79f7bf2c, SUC_NE_lexicon_tags_2 },
    { 0x5df3bf2d, SUC_NE_lexicon_tags_2 },
    { 0x9fafbf2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9675bf30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb61bbf32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66e1bf34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7cdbf36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3bf38, SUC_NE_lexicon_tags_2 },
    { 0xb089bf39, SUC_NE_lexicon_tags_2 },
    { 0x5bf3a, SUC_NE_lexicon_tags_21 },
    { 0xbdfbbf3a, SUC_NE_lexicon_tags_2 },
    { 0xcad3bf3c, SUC_NE_lexicon_tags_30 },
    { 0x49ddbf39, SUC_NE_lexicon_tags_186 },
    { 0, NULL },
    { 0xd7afbf3f, SUC_NE_lexicon_tags_2 },
    { 0xa5b5bf40, SUC_NE_lexicon_tags_2 },
    { 0xaef3bf3f, SUC_NE_lexicon_tags_2 },
    { 0x864dbf42, SUC_NE_lexicon_tags_2 },
    { 0x1d87bf43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6bbf48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcfbf4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e3bf4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29fbf55, SUC_NE_lexicon_tags_7 },
    { 0xdd43bf55, SUC_NE_lexicon_tags_2 },
    { 0xc2adbf57, SUC_NE_lexicon_tags_2 },
    { 0xa09fbf58, SUC_NE_lexicon_tags_2 },
    { 0x5fefbf56, SUC_NE_lexicon_tags_2 },
    { 0x3e7fbf58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc89bf5c, SUC_NE_lexicon_tags_2 },
    { 0xae0fbf5d, SUC_NE_lexicon_tags_2 },
    { 0x11adbf5d, SUC_NE_lexicon_tags_4 },
    { 0xbcadbf5d, SUC_NE_lexicon_tags_2 },
    { 0x37ffbf60, SUC_NE_lexicon_tags_2 },
    { 0xf0d7bf61, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0x3aafbf63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d07bf65, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb823bf68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70bbf6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6071bf6f, SUC_NE_lexicon_tags_2 },
    { 0x230bbf6f, SUC_NE_lexicon_tags_2 },
    { 0xf3d3bf71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1017bf74, SUC_NE_lexicon_tags_2 },
    { 0x3b1bbf74, SUC_NE_lexicon_tags_2 },
    { 0x6843bf76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1dbf78, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79fbf80, SUC_NE_lexicon_tags_2 },
    { 0xc0cbbf80, SUC_NE_lexicon_tags_2 },
    { 0xd0adbf80, SUC_NE_lexicon_tags_2 },
    { 0x5e7bbf82, SUC_NE_lexicon_tags_2 },
    { 0xd97fbf84, SUC_NE_lexicon_tags_7 },
    { 0x1957bf84, SUC_NE_lexicon_tags_2 },
    { 0x195bf86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71c1bf88, SUC_NE_lexicon_tags_2 },
    { 0x72dfbf89, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf055bf8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed5bf90, SUC_NE_lexicon_tags_2 },
    { 0xa8edbf91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1959bf97, SUC_NE_lexicon_tags_2 },
    { 0x20b1bf98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe387bf9a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6bbf9d, SUC_NE_lexicon_tags_7 },
    { 0x596fbf9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9adbfa1, SUC_NE_lexicon_tags_2 },
    { 0xd59dbfa1, SUC_NE_lexicon_tags_2 },
    { 0x926dbfa1, SUC_NE_lexicon_tags_2 },
    { 0x8a69bfa4, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15bfa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb3bfac, SUC_NE_lexicon_tags_2 },
    { 0xc269bfad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe349bfb2, SUC_NE_lexicon_tags_2 },
    { 0xdd8fbfb3, SUC_NE_lexicon_tags_2 },
    { 0x50cfbfb4, SUC_NE_lexicon_tags_2 },
    { 0xa5e5bfb4, SUC_NE_lexicon_tags_13 },
    { 0xa59bfb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8bbfb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6447bfbd, SUC_NE_lexicon_tags_13 },
    { 0x554bbfbe, SUC_NE_lexicon_tags_2 },
    { 0x34afbfbe, SUC_NE_lexicon_tags_2 },
    { 0xf47dbfbd, SUC_NE_lexicon_tags_2 },
    { 0xa3e1bfc0, SUC_NE_lexicon_tags_2 },
    { 0xef51bfc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a47bfc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x355fbfc7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4f37bfc9, SUC_NE_lexicon_tags_3 },
    { 0x728fbfca, SUC_NE_lexicon_tags_2 },
    { 0x45c5bfca, SUC_NE_lexicon_tags_2 },
    { 0x8dbbbfcc, SUC_NE_lexicon_tags_2 },
    { 0x2257bfc9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xedc1bfcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1835bfd2, SUC_NE_lexicon_tags_2 },
    { 0x4f9bfd3, SUC_NE_lexicon_tags_7 },
    { 0x2695bfd2, SUC_NE_lexicon_tags_2 },
    { 0x1331bfd5, SUC_NE_lexicon_tags_2 },
    { 0xad23bfd3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e5bfde, SUC_NE_lexicon_tags_2 },
    { 0x674dbfdf, SUC_NE_lexicon_tags_2 },
    { 0x65c5bfe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dbbfee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ca3bff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c1bff4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x801bbff6, SUC_NE_lexicon_tags_2 },
    { 0x2989bff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab9bffc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbb7bffe, SUC_NE_lexicon_tags_2 },
    { 0x6ba3bffe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786dc006, SUC_NE_lexicon_tags_2 },
    { 0xd0d1c007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b5bc009, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x3487c00b, SUC_NE_lexicon_tags_2 },
    { 0xc8f9c00c, SUC_NE_lexicon_tags_2 },
    { 0x853bc00c, SUC_NE_lexicon_tags_2 },
    { 0xd8fbc00e, SUC_NE_lexicon_tags_2 },
    { 0x4df1c00f, SUC_NE_lexicon_tags_2 },
    { 0xefb5c010, SUC_NE_lexicon_tags_18 },
    { 0x3bedc011, SUC_NE_lexicon_tags_2 },
    { 0xe913c00f, SUC_NE_lexicon_tags_2 },
    { 0xc4fbc00c, SUC_NE_lexicon_tags_2 },
    { 0xac1fc014, SUC_NE_lexicon_tags_23 },
    { 0x9aefc00f, SUC_NE_lexicon_tags_2 },
    { 0xb57bc012, SUC_NE_lexicon_tags_2 },
    { 0xef99c017, SUC_NE_lexicon_tags_2 },
    { 0xd731c018, SUC_NE_lexicon_tags_2 },
    { 0xca5fc010, SUC_NE_lexicon_tags_2 },
    { 0xa659c017, SUC_NE_lexicon_tags_2 },
    { 0xfd2dc01b, SUC_NE_lexicon_tags_2 },
    { 0xfb85c013, SUC_NE_lexicon_tags_2 },
    { 0x625c01d, SUC_NE_lexicon_tags_2 },
    { 0x833fc017, SUC_NE_lexicon_tags_2 },
    { 0x4881c01e, SUC_NE_lexicon_tags_2 },
    { 0x77d9c020, SUC_NE_lexicon_tags_2 },
    { 0xd15bc021, SUC_NE_lexicon_tags_25 },
    { 0x8953c021, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd9c027, SUC_NE_lexicon_tags_2 },
    { 0xec1bc028, SUC_NE_lexicon_tags_2 },
    { 0x6535c029, SUC_NE_lexicon_tags_2 },
    { 0xe281c028, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf573c031, SUC_NE_lexicon_tags_2 },
    { 0xac7bc031, SUC_NE_lexicon_tags_2 },
    { 0x96e5c033, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa13c035, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3051c038, SUC_NE_lexicon_tags_18 },
    { 0xdca1c038, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6fc042, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9c04c, SUC_NE_lexicon_tags_2 },
    { 0xaf1bc04d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xbec3c04f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3341c051, SUC_NE_lexicon_tags_2 },
    { 0x653dc051, SUC_NE_lexicon_tags_2 },
    { 0x89ffc053, SUC_NE_lexicon_tags_2 },
    { 0xf325c054, SUC_NE_lexicon_tags_2 },
    { 0x6bfc054, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5081c05a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf45c060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2f5c062, SUC_NE_lexicon_tags_2 },
    { 0xc2d1c063, SUC_NE_lexicon_tags_2 },
    { 0xe241c063, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x847bc066, SUC_NE_lexicon_tags_2 },
    { 0x26ebc067, SUC_NE_lexicon_tags_2 },
    { 0x3d09c068, SUC_NE_lexicon_tags_2 },
    { 0xa883c069, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41c06d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x368bc06f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa3c072, SUC_NE_lexicon_tags_2 },
    { 0xe7edc072, SUC_NE_lexicon_tags_2 },
    { 0xf74bc073, SUC_NE_lexicon_tags_0 },
    { 0x45ddc075, SUC_NE_lexicon_tags_2 },
    { 0x73a9c076, SUC_NE_lexicon_tags_2 },
    { 0xb27c073, SUC_NE_lexicon_tags_1 },
    { 0x61e7c078, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bbc07c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa711c082, SUC_NE_lexicon_tags_2 },
    { 0x5c53c083, SUC_NE_lexicon_tags_2 },
    { 0x16abc083, SUC_NE_lexicon_tags_2 },
    { 0xe789c085, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f99c088, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea47c08a, SUC_NE_lexicon_tags_2 },
    { 0xbdbc08a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01c091, SUC_NE_lexicon_tags_0 },
    { 0x8dc3c092, SUC_NE_lexicon_tags_2 },
    { 0x69b5c093, SUC_NE_lexicon_tags_2 },
    { 0x946dc093, SUC_NE_lexicon_tags_2 },
    { 0xa5f7c095, SUC_NE_lexicon_tags_18 },
    { 0x69b3c096, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabc099, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32bc09c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a3c0a1, SUC_NE_lexicon_tags_4 },
    { 0x78f7c0a1, SUC_NE_lexicon_tags_2 },
    { 0x4a43c0a3, SUC_NE_lexicon_tags_2 },
    { 0xed2bc0a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132fc0a9, SUC_NE_lexicon_tags_3 },
    { 0xce3bc0aa, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e3fc0ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb921c0ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x559c0b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cafc0b3, SUC_NE_lexicon_tags_2 },
    { 0xf7bfc0b3, SUC_NE_lexicon_tags_157 },
    { 0xdb7bc0b3, SUC_NE_lexicon_tags_2 },
    { 0x8a2dc0b5, SUC_NE_lexicon_tags_2 },
    { 0x6ff5c0b7, SUC_NE_lexicon_tags_2 },
    { 0x701c0b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3d9c0ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab9c0c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2131c0c6, SUC_NE_lexicon_tags_2 },
    { 0x4717c0c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd379c0cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90ddc0cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1dbc0d1, SUC_NE_lexicon_tags_2 },
    { 0x6743c0d1, SUC_NE_lexicon_tags_17 },
    { 0x400bc0d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cb7c0d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5c0da, SUC_NE_lexicon_tags_2 },
    { 0x1eb9c0db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2fc0df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b6dc0e1, SUC_NE_lexicon_tags_2 },
    { 0x3d75c0e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe731c0e5, SUC_NE_lexicon_tags_2 },
    { 0xd67fc0e5, SUC_NE_lexicon_tags_4 },
    { 0x751dc0e7, SUC_NE_lexicon_tags_2 },
    { 0xcdebc0e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94dc0ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6477c0f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb751c0f4, SUC_NE_lexicon_tags_2 },
    { 0xe363c0f4, SUC_NE_lexicon_tags_2 },
    { 0x3fa1c0f6, SUC_NE_lexicon_tags_2 },
    { 0x5061c0f7, SUC_NE_lexicon_tags_2 },
    { 0x767bc0f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85ffc0fa, SUC_NE_lexicon_tags_2 },
    { 0x4e79c0fb, SUC_NE_lexicon_tags_4 },
    { 0x6e4bc0fb, SUC_NE_lexicon_tags_2 },
    { 0x7313c0fa, SUC_NE_lexicon_tags_2 },
    { 0x3785c0fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3c108, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e93c10a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c10e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x253bc110, SUC_NE_lexicon_tags_2 },
    { 0xa8d9c111, SUC_NE_lexicon_tags_2 },
    { 0xd27fc112, SUC_NE_lexicon_tags_2 },
    { 0xef13c112, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca1c117, SUC_NE_lexicon_tags_2 },
    { 0x355fc118, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x90abc11a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee9c11d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1c120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf1c124, SUC_NE_lexicon_tags_2 },
    { 0xf3c7c125, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1799c12c, SUC_NE_lexicon_tags_2 },
    { 0xdf4bc12d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e7c132, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9c135, SUC_NE_lexicon_tags_2 },
    { 0xdfd9c136, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x473fc138, SUC_NE_lexicon_tags_2 },
    { 0x2415c139, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d5bc13b, SUC_NE_lexicon_tags_2 },
    { 0x332bc13b, SUC_NE_lexicon_tags_1 },
    { 0x9357c13d, SUC_NE_lexicon_tags_2 },
    { 0x4043c13d, SUC_NE_lexicon_tags_2 },
    { 0xde09c13d, SUC_NE_lexicon_tags_2 },
    { 0x8a21c140, SUC_NE_lexicon_tags_2 },
    { 0xe161c141, SUC_NE_lexicon_tags_2 },
    { 0x14e1c141, SUC_NE_lexicon_tags_2 },
    { 0x1ce5c141, SUC_NE_lexicon_tags_2 },
    { 0xd3f9c144, SUC_NE_lexicon_tags_19 },
    { 0xf3bc145, SUC_NE_lexicon_tags_2 },
    { 0x39cfc146, SUC_NE_lexicon_tags_2 },
    { 0x7407c140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00bc14b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f1c152, SUC_NE_lexicon_tags_24 },
    { 0xb197c153, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5e3c155, SUC_NE_lexicon_tags_1 },
    { 0x4137c155, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x347c159, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x347dc15b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x790dc15e, SUC_NE_lexicon_tags_2 },
    { 0x9897c15f, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0xf90bc161, SUC_NE_lexicon_tags_2 },
    { 0xaf7c161, SUC_NE_lexicon_tags_2 },
    { 0x4de3c163, SUC_NE_lexicon_tags_2 },
    { 0xf37bc164, SUC_NE_lexicon_tags_2 },
    { 0x650fc164, SUC_NE_lexicon_tags_2 },
    { 0xa989c161, SUC_NE_lexicon_tags_2 },
    { 0xb9afc167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb5c16c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x788dc16e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665bc173, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63efc175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c178, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fc17b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13bc180, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6b53c182, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5731c185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8dc18a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacd7c18c, SUC_NE_lexicon_tags_2 },
    { 0xda0fc18d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf41bc18f, SUC_NE_lexicon_tags_2 },
    { 0xcf6fc190, SUC_NE_lexicon_tags_2 },
    { 0xd4c3c191, SUC_NE_lexicon_tags_2 },
    { 0x665fc192, SUC_NE_lexicon_tags_2 },
    { 0xf721c193, SUC_NE_lexicon_tags_2 },
    { 0x3f83c18f, SUC_NE_lexicon_tags_2 },
    { 0xf84dc195, SUC_NE_lexicon_tags_2 },
    { 0x5041c191, SUC_NE_lexicon_tags_2 },
    { 0x673bc191, SUC_NE_lexicon_tags_1 },
    { 0x6b1bc191, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb963c19a, SUC_NE_lexicon_tags_2 },
    { 0x720dc19b, SUC_NE_lexicon_tags_2 },
    { 0x5007c19b, SUC_NE_lexicon_tags_2 },
    { 0xf07c19c, SUC_NE_lexicon_tags_3 },
    { 0x62e7c19a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3c1a1, SUC_NE_lexicon_tags_2 },
    { 0x2613c1a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aafc1ae, SUC_NE_lexicon_tags_2 },
    { 0xa191c1ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58edc1b5, SUC_NE_lexicon_tags_2 },
    { 0x4badc1b5, SUC_NE_lexicon_tags_1 },
    { 0xcc5fc1b7, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cbc1ba, SUC_NE_lexicon_tags_3 },
    { 0xcc1dc1bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e71c1c0, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7abc1c3, SUC_NE_lexicon_tags_2 },
    { 0xa8d5c1c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dc1c1c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf489c1c8, SUC_NE_lexicon_tags_2 },
    { 0x993dc1c8, SUC_NE_lexicon_tags_2 },
    { 0x793bc1ca, SUC_NE_lexicon_tags_2 },
    { 0x30ebc1ca, SUC_NE_lexicon_tags_2 },
    { 0xe67dc1cc, SUC_NE_lexicon_tags_7 },
    { 0xa5c9c1c9, SUC_NE_lexicon_tags_2 },
    { 0xae05c1cb, SUC_NE_lexicon_tags_2 },
    { 0xffebc1cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78fc1d2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xed35c1d4, SUC_NE_lexicon_tags_0 },
    { 0x5e9bc1d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf217c1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1fc1db, SUC_NE_lexicon_tags_2 },
    { 0xd5fdc1dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f37c1de, SUC_NE_lexicon_tags_2 },
    { 0x2937c1df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e47c1e2, SUC_NE_lexicon_tags_2 },
    { 0x276dc1e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9fc1e6, SUC_NE_lexicon_tags_2 },
    { 0x6a8fc1e7, SUC_NE_lexicon_tags_2 },
    { 0x5f5fc1e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b5c1ed, SUC_NE_lexicon_tags_2 },
    { 0x924fc1ee, SUC_NE_lexicon_tags_2 },
    { 0xaf09c1ef, SUC_NE_lexicon_tags_2 },
    { 0x4297c1ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb895c1f3, SUC_NE_lexicon_tags_2 },
    { 0x884dc1f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1da1c1f6, SUC_NE_lexicon_tags_0 },
    { 0x71ebc1f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d3c1fd, SUC_NE_lexicon_tags_2 },
    { 0xe6b1c1fe, SUC_NE_lexicon_tags_2 },
    { 0xc2efc1fd, SUC_NE_lexicon_tags_14 },
    { 0x2191c200, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb091c202, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8743c207, SUC_NE_lexicon_tags_2 },
    { 0xfc97c208, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3d5c20a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325fc20e, SUC_NE_lexicon_tags_2 },
    { 0xcf59c20f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b7c213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x753bc216, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe947c219, SUC_NE_lexicon_tags_1 },
    { 0x8e43c21a, SUC_NE_lexicon_tags_2 },
    { 0xb467c21b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x26edc21e, SUC_NE_lexicon_tags_2 },
    { 0xd399c21e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd023c221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x643dc223, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xcb85c225, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a1c227, SUC_NE_lexicon_tags_4 },
    { 0x3cb1c228, SUC_NE_lexicon_tags_2 },
    { 0xd1b9c229, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e1c22c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebbc232, SUC_NE_lexicon_tags_2 },
    { 0x2447c232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9483c237, SUC_NE_lexicon_tags_2 },
    { 0xa2afc238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3093c23a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc355c23c, SUC_NE_lexicon_tags_2 },
    { 0x83a1c23d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d9c245, SUC_NE_lexicon_tags_2 },
    { 0x3b7bc245, SUC_NE_lexicon_tags_2 },
    { 0xefedc247, SUC_NE_lexicon_tags_17 },
    { 0xea2fc248, SUC_NE_lexicon_tags_2 },
    { 0x9345c249, SUC_NE_lexicon_tags_2 },
    { 0x4577c24a, SUC_NE_lexicon_tags_2 },
    { 0x4113c248, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69dc253, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136bc25b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafadc25d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203bc263, SUC_NE_lexicon_tags_0 },
    { 0x2b27c264, SUC_NE_lexicon_tags_18 },
    { 0xe743c265, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad5c268, SUC_NE_lexicon_tags_2 },
    { 0xae3dc268, SUC_NE_lexicon_tags_2 },
    { 0x1059c26a, SUC_NE_lexicon_tags_2 },
    { 0xf999c26b, SUC_NE_lexicon_tags_2 },
    { 0x2dddc269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe631c271, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3543c274, SUC_NE_lexicon_tags_2 },
    { 0xf94bc275, SUC_NE_lexicon_tags_2 },
    { 0xdf9dc276, SUC_NE_lexicon_tags_2 },
    { 0xc8f5c277, SUC_NE_lexicon_tags_2 },
    { 0xa253c278, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5abc27b, SUC_NE_lexicon_tags_5 },
    { 0x168bc27c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffdc27f, SUC_NE_lexicon_tags_366 },
    { 0x9875c27f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b1c286, SUC_NE_lexicon_tags_2 },
    { 0x3255c287, SUC_NE_lexicon_tags_2 },
    { 0x739dc288, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7fc28c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x96f7c28e, SUC_NE_lexicon_tags_2 },
    { 0x5051c28f, SUC_NE_lexicon_tags_2 },
    { 0xb195c28f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7bc294, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d5c298, SUC_NE_lexicon_tags_2 },
    { 0xd9c1c299, SUC_NE_lexicon_tags_2 },
    { 0xc5abc298, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7811c29f, SUC_NE_lexicon_tags_2 },
    { 0x68a7c2a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb089c2a2, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x4a63c2a4, SUC_NE_lexicon_tags_3 },
    { 0xcb97c2a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481bc2a9, SUC_NE_lexicon_tags_2 },
    { 0x1e23c2a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9dfc2ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f45c2ae, SUC_NE_lexicon_tags_2 },
    { 0xbe0dc2ae, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xedadc2b2, SUC_NE_lexicon_tags_2 },
    { 0x1a29c2b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1f1c2b5, SUC_NE_lexicon_tags_2 },
    { 0xa663c2b5, SUC_NE_lexicon_tags_2 },
    { 0x921bc2b7, SUC_NE_lexicon_tags_2 },
    { 0x7c5dc2b8, SUC_NE_lexicon_tags_2 },
    { 0x698dc2b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee5c2be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4201c2c2, SUC_NE_lexicon_tags_2 },
    { 0xd35fc2c3, SUC_NE_lexicon_tags_2 },
    { 0x9d81c2c2, SUC_NE_lexicon_tags_2 },
    { 0x3181c2c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc05dc2c7, SUC_NE_lexicon_tags_2 },
    { 0x6d03c2c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78fc2ca, SUC_NE_lexicon_tags_2 },
    { 0x3977c2cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc2cf, SUC_NE_lexicon_tags_4 },
    { 0xaad3c2d0, SUC_NE_lexicon_tags_14 },
    { 0xa517c2d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330fc2d8, SUC_NE_lexicon_tags_0 },
    { 0x3c2dc2d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x19f7c2db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b1c2e0, SUC_NE_lexicon_tags_2 },
    { 0x20b7c2e1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7c2e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db1c2e8, SUC_NE_lexicon_tags_7 },
    { 0xf77dc2e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2149c2ed, SUC_NE_lexicon_tags_2 },
    { 0x3ec7c2ee, SUC_NE_lexicon_tags_2 },
    { 0x3921c2ef, SUC_NE_lexicon_tags_2 },
    { 0xab8bc2f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f83c2f2, SUC_NE_lexicon_tags_134 },
    { 0x7649c2f2, SUC_NE_lexicon_tags_2 },
    { 0xcf1dc2f4, SUC_NE_lexicon_tags_2 },
    { 0xc36bc2f5, SUC_NE_lexicon_tags_4 },
    { 0x824dc2f5, SUC_NE_lexicon_tags_2 },
    { 0xee09c2f7, SUC_NE_lexicon_tags_2 },
    { 0xd8a9c2f6, SUC_NE_lexicon_tags_2 },
    { 0xd12fc2f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17d7c2fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74d1c2fd, SUC_NE_lexicon_tags_4 },
    { 0x2f61c2fe, SUC_NE_lexicon_tags_2 },
    { 0xfdadc2fd, SUC_NE_lexicon_tags_2 },
    { 0x64cbc300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3581c304, SUC_NE_lexicon_tags_2 },
    { 0x21f1c305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd365c307, SUC_NE_lexicon_tags_2 },
    { 0xa03bc307, SUC_NE_lexicon_tags_2 },
    { 0x9a3dc308, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e1fc30b, SUC_NE_lexicon_tags_2 },
    { 0xd00fc30c, SUC_NE_lexicon_tags_2 },
    { 0x404bc30c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bbc310, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f5c315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e1c31b, SUC_NE_lexicon_tags_2 },
    { 0xcb73c31c, SUC_NE_lexicon_tags_0 },
    { 0xaa09c31b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba9c324, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c3c329, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dc32c, SUC_NE_lexicon_tags_2 },
    { 0x51c3c32c, SUC_NE_lexicon_tags_2 },
    { 0xb7efc32d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1c331, SUC_NE_lexicon_tags_2 },
    { 0x9b21c332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef5c337, SUC_NE_lexicon_tags_2 },
    { 0xddcbc338, SUC_NE_lexicon_tags_2 },
    { 0x1877c339, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee7bc33b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13d5c33d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9107c340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x129fc342, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x91adc344, SUC_NE_lexicon_tags_2 },
    { 0xe6dbc345, SUC_NE_lexicon_tags_19 },
    { 0xa3e3c345, SUC_NE_lexicon_tags_2 },
    { 0x6731c347, SUC_NE_lexicon_tags_2 },
    { 0x8897c345, SUC_NE_lexicon_tags_2 },
    { 0xa9a1c344, SUC_NE_lexicon_tags_2 },
    { 0x2e79c345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3711c34c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb1c34f, SUC_NE_lexicon_tags_2 },
    { 0x7c01c34f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dbc358, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9573c35a, SUC_NE_lexicon_tags_18 },
    { 0x4641c35b, SUC_NE_lexicon_tags_2 },
    { 0x16f1c35c, SUC_NE_lexicon_tags_2 },
    { 0xca85c35c, SUC_NE_lexicon_tags_3 },
    { 0xb815c35a, SUC_NE_lexicon_tags_4 },
    { 0x700fc35f, SUC_NE_lexicon_tags_3 },
    { 0x4f31c360, SUC_NE_lexicon_tags_2 },
    { 0xf7e5c35e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c5c364, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaefc36b, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0x3ff3c36d, SUC_NE_lexicon_tags_2 },
    { 0xbf2dc36d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafbfc371, SUC_NE_lexicon_tags_2 },
    { 0xf3b9c371, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc601c374, SUC_NE_lexicon_tags_2 },
    { 0x4cd1c375, SUC_NE_lexicon_tags_2 },
    { 0x98e5c376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad77c379, SUC_NE_lexicon_tags_215 },
    { 0x5e61c379, SUC_NE_lexicon_tags_2 },
    { 0x78fc37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe19fc37d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5c380, SUC_NE_lexicon_tags_2 },
    { 0xabb3c380, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f65c383, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78bc38b, SUC_NE_lexicon_tags_2 },
    { 0x8fdc38b, SUC_NE_lexicon_tags_3 },
    { 0x381fc38d, SUC_NE_lexicon_tags_2 },
    { 0x859bc38b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7bfc390, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7abc393, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ddc399, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x240fc39c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a7c3a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4617c3a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa521c3a6, SUC_NE_lexicon_tags_2 },
    { 0xb629c3a7, SUC_NE_lexicon_tags_2 },
    { 0xc8e5c3a8, SUC_NE_lexicon_tags_2 },
    { 0xbf9c3a9, SUC_NE_lexicon_tags_2 },
    { 0x4021c3aa, SUC_NE_lexicon_tags_2 },
    { 0x3721c3ab, SUC_NE_lexicon_tags_2 },
    { 0xd41c3ab, SUC_NE_lexicon_tags_11 },
    { 0x26f3c3a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f3c3b0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdfc3b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646fc3bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5bc3c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeebc3c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fffc3c4, SUC_NE_lexicon_tags_2 },
    { 0xcc1c3c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fc3ce, SUC_NE_lexicon_tags_2 },
    { 0xa5d5c3cf, SUC_NE_lexicon_tags_40 },
    { 0x1bf3c3ce, SUC_NE_lexicon_tags_2 },
    { 0x685c3cf, SUC_NE_lexicon_tags_2 },
    { 0x373dc3d2, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4795c3d7, SUC_NE_lexicon_tags_2 },
    { 0xf2c5c3d8, SUC_NE_lexicon_tags_2 },
    { 0x1469c3d8, SUC_NE_lexicon_tags_2 },
    { 0xa555c3da, SUC_NE_lexicon_tags_2 },
    { 0x48ebc3db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4edc3de, SUC_NE_lexicon_tags_3 },
    { 0xc567c3df, SUC_NE_lexicon_tags_2 },
    { 0x7f67c3e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d9c3e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f7bc3e6, SUC_NE_lexicon_tags_2 },
    { 0x8cbbc3e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59c3ea, SUC_NE_lexicon_tags_2 },
    { 0x834fc3ea, SUC_NE_lexicon_tags_3 },
    { 0x74e7c3ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59c3c3ee, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bd3c3f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e1c3f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fdc3fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fe1c400, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc69c402, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146dc40a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dc414, SUC_NE_lexicon_tags_2 },
    { 0xaac9c415, SUC_NE_lexicon_tags_2 },
    { 0xce23c416, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d3c41c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62dc420, SUC_NE_lexicon_tags_2 },
    { 0x9d21c420, SUC_NE_lexicon_tags_2 },
    { 0xdd37c422, SUC_NE_lexicon_tags_2 },
    { 0xf61bc423, SUC_NE_lexicon_tags_2 },
    { 0xe2fdc424, SUC_NE_lexicon_tags_2 },
    { 0xc28bc425, SUC_NE_lexicon_tags_2 },
    { 0x3775c426, SUC_NE_lexicon_tags_2 },
    { 0x71bc427, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fadc429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc1c42e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde99c430, SUC_NE_lexicon_tags_2 },
    { 0x8701c431, SUC_NE_lexicon_tags_2 },
    { 0x4d77c430, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bdc43a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8145c43e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f27c443, SUC_NE_lexicon_tags_2 },
    { 0xc6b9c443, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x735fc446, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2067c449, SUC_NE_lexicon_tags_3 },
    { 0x127bc449, SUC_NE_lexicon_tags_2 },
    { 0xac13c44b, SUC_NE_lexicon_tags_2 },
    { 0x166dc44c, SUC_NE_lexicon_tags_2 },
    { 0x4f09c449, SUC_NE_lexicon_tags_2 },
    { 0x291fc44e, SUC_NE_lexicon_tags_2 },
    { 0x3b4bc44e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x1731c451, SUC_NE_lexicon_tags_2 },
    { 0xba3c452, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2db5c454, SUC_NE_lexicon_tags_2 },
    { 0x1a4fc455, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc1fc457, SUC_NE_lexicon_tags_2 },
    { 0xfe2dc458, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8279c45c, SUC_NE_lexicon_tags_2 },
    { 0x9e17c45c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94dc461, SUC_NE_lexicon_tags_2 },
    { 0xa677c461, SUC_NE_lexicon_tags_2 },
    { 0x478bc463, SUC_NE_lexicon_tags_2 },
    { 0xaf9dc462, SUC_NE_lexicon_tags_4 },
    { 0xcbcbc462, SUC_NE_lexicon_tags_2 },
    { 0xefc3c466, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b53c468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd9c46b, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef9c46e, SUC_NE_lexicon_tags_2 },
    { 0x36f9c46f, SUC_NE_lexicon_tags_2 },
    { 0x4ea3c46e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad77c472, SUC_NE_lexicon_tags_2 },
    { 0x83f7c473, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc34fc475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65bc47a, SUC_NE_lexicon_tags_2 },
    { 0x7c8fc47a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3c480, SUC_NE_lexicon_tags_2 },
    { 0x9ccfc480, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9f5c483, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd687c487, SUC_NE_lexicon_tags_2 },
    { 0x32a3c487, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f7c48d, SUC_NE_lexicon_tags_2 },
    { 0x2879c48d, SUC_NE_lexicon_tags_2 },
    { 0xba75c48d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5115c493, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bc496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1019c498, SUC_NE_lexicon_tags_2 },
    { 0x79f5c499, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff5c49e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde1bc4a0, SUC_NE_lexicon_tags_17 },
    { 0xbdf1c4a1, SUC_NE_lexicon_tags_2 },
    { 0xc039c4a2, SUC_NE_lexicon_tags_2 },
    { 0x8941c4a0, SUC_NE_lexicon_tags_2 },
    { 0xe151c4a3, SUC_NE_lexicon_tags_2 },
    { 0xa2afc4a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680bc4a9, SUC_NE_lexicon_tags_25 },
    { 0x5417c4a9, SUC_NE_lexicon_tags_2 },
    { 0x8de7c4ab, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3bc4ae, SUC_NE_lexicon_tags_2 },
    { 0xa673c4af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc91c4b1, SUC_NE_lexicon_tags_4 },
    { 0x6b8fc4b2, SUC_NE_lexicon_tags_2 },
    { 0xa6d1c4b3, SUC_NE_lexicon_tags_30 },
    { 0xa57bc4b4, SUC_NE_lexicon_tags_2 },
    { 0xa72fc4b4, SUC_NE_lexicon_tags_2 },
    { 0x78e7c4b4, SUC_NE_lexicon_tags_1 },
    { 0xda57c4b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee1c4bc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9721c4c0, SUC_NE_lexicon_tags_2 },
    { 0xff21c4c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe651c4c4, SUC_NE_lexicon_tags_2 },
    { 0xea41c4c4, SUC_NE_lexicon_tags_3 },
    { 0x3a51c4c6, SUC_NE_lexicon_tags_4 },
    { 0xe9b7c4c6, SUC_NE_lexicon_tags_2 },
    { 0x695dc4c7, SUC_NE_lexicon_tags_2 },
    { 0x5f3c4c9, SUC_NE_lexicon_tags_2 },
    { 0xa511c4ca, SUC_NE_lexicon_tags_2 },
    { 0x2b3dc4c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x792dc4cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8993c4d5, SUC_NE_lexicon_tags_2 },
    { 0x8329c4d6, SUC_NE_lexicon_tags_2 },
    { 0x134bc4d7, SUC_NE_lexicon_tags_2 },
    { 0x5d29c4d8, SUC_NE_lexicon_tags_2 },
    { 0x831c4d9, SUC_NE_lexicon_tags_2 },
    { 0x206fc4d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0fc4dd, SUC_NE_lexicon_tags_2 },
    { 0x3aa9c4dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa565c4e0, SUC_NE_lexicon_tags_2 },
    { 0x7993c4e1, SUC_NE_lexicon_tags_2 },
    { 0x2a3fc4e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x69e1c4e4, SUC_NE_lexicon_tags_2 },
    { 0xdcf5c4e5, SUC_NE_lexicon_tags_2 },
    { 0x9dbdc4e6, SUC_NE_lexicon_tags_2 },
    { 0xeb47c4e6, SUC_NE_lexicon_tags_2 },
    { 0x9b9dc4e8, SUC_NE_lexicon_tags_2 },
    { 0xc09fc4e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdbc4ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a93c4f0, SUC_NE_lexicon_tags_32 },
    { 0x5197c4f1, SUC_NE_lexicon_tags_36 },
    { 0x9f3dc4f2, SUC_NE_lexicon_tags_2 },
    { 0x8fa3c4f2, SUC_NE_lexicon_tags_2 },
    { 0xeff3c4f0, SUC_NE_lexicon_tags_0 },
    { 0x8b77c4f5, SUC_NE_lexicon_tags_7 },
    { 0x6dcdc4f6, SUC_NE_lexicon_tags_148 },
    { 0, NULL },
    { 0xb239c4f8, SUC_NE_lexicon_tags_2 },
    { 0xd80bc4f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f77c4fb, SUC_NE_lexicon_tags_2 },
    { 0x2e85c4fb, SUC_NE_lexicon_tags_2 },
    { 0x2effc4fd, SUC_NE_lexicon_tags_2 },
    { 0x4f8bc4fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc677c500, SUC_NE_lexicon_tags_2 },
    { 0x51e1c500, SUC_NE_lexicon_tags_2 },
    { 0xca2fc502, SUC_NE_lexicon_tags_3 },
    { 0xff03c501, SUC_NE_lexicon_tags_2 },
    { 0x7a51c504, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce5c508, SUC_NE_lexicon_tags_2 },
    { 0x29e3c508, SUC_NE_lexicon_tags_7 },
    { 0x58afc50a, SUC_NE_lexicon_tags_2 },
    { 0x2dedc50b, SUC_NE_lexicon_tags_2 },
    { 0xc323c50c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba63c510, SUC_NE_lexicon_tags_2 },
    { 0x2c97c511, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56bc514, SUC_NE_lexicon_tags_2 },
    { 0xf963c515, SUC_NE_lexicon_tags_2 },
    { 0x17e3c516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a9c51a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbfc521, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd265c525, SUC_NE_lexicon_tags_2 },
    { 0xe541c526, SUC_NE_lexicon_tags_2 },
    { 0x7eb7c527, SUC_NE_lexicon_tags_2 },
    { 0x5af9c526, SUC_NE_lexicon_tags_2 },
    { 0x2191c529, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a83c52d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21ffc52f, SUC_NE_lexicon_tags_2 },
    { 0xee6dc530, SUC_NE_lexicon_tags_2 },
    { 0x65f9c530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd651c533, SUC_NE_lexicon_tags_168 },
    { 0x441bc534, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51c538, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf161c53d, SUC_NE_lexicon_tags_3 },
    { 0xdc49c53d, SUC_NE_lexicon_tags_2 },
    { 0x8bfbc53f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f51c542, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb47c546, SUC_NE_lexicon_tags_4 },
    { 0x57bc547, SUC_NE_lexicon_tags_2 },
    { 0x996dc547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e5c54d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90fc550, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bc553, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe415c55a, SUC_NE_lexicon_tags_2 },
    { 0xa89c55b, SUC_NE_lexicon_tags_2 },
    { 0x88dfc55c, SUC_NE_lexicon_tags_2 },
    { 0x2519c55a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985dc567, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf271c56a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8dc571, SUC_NE_lexicon_tags_2 },
    { 0x5fe5c571, SUC_NE_lexicon_tags_2 },
    { 0x997fc573, SUC_NE_lexicon_tags_2 },
    { 0x3507c573, SUC_NE_lexicon_tags_2 },
    { 0x2787c575, SUC_NE_lexicon_tags_7 },
    { 0x4af7c576, SUC_NE_lexicon_tags_2 },
    { 0x4ca3c576, SUC_NE_lexicon_tags_2 },
    { 0x5ac1c577, SUC_NE_lexicon_tags_2 },
    { 0x9ac3c579, SUC_NE_lexicon_tags_2 },
    { 0x486dc574, SUC_NE_lexicon_tags_2 },
    { 0x759c57b, SUC_NE_lexicon_tags_2 },
    { 0xab87c57c, SUC_NE_lexicon_tags_2 },
    { 0x78cbc57d, SUC_NE_lexicon_tags_7 },
    { 0xe7e7c57e, SUC_NE_lexicon_tags_34 },
    { 0x1071c57d, SUC_NE_lexicon_tags_2 },
    { 0xeaa3c57e, SUC_NE_lexicon_tags_2 },
    { 0xcbf1c57a, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5399c584, SUC_NE_lexicon_tags_2 },
    { 0xaee1c585, SUC_NE_lexicon_tags_2 },
    { 0x6303c586, SUC_NE_lexicon_tags_2 },
    { 0x3711c584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36dc58a, SUC_NE_lexicon_tags_2 },
    { 0xf69dc58b, SUC_NE_lexicon_tags_2 },
    { 0x3f65c58b, SUC_NE_lexicon_tags_2 },
    { 0x856fc58d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e1c593, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe835c59a, SUC_NE_lexicon_tags_337 },
    { 0x8bfc59b, SUC_NE_lexicon_tags_2 },
    { 0xff13c59b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e7c5a1, SUC_NE_lexicon_tags_2 },
    { 0x8c23c5a2, SUC_NE_lexicon_tags_2 },
    { 0x9e4fc5a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2517c5a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc341c5ab, SUC_NE_lexicon_tags_4 },
    { 0x9361c5ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e9c5af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cafc5b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e7c5b8, SUC_NE_lexicon_tags_2 },
    { 0x7225c5b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d91c5bb, SUC_NE_lexicon_tags_2 },
    { 0xcb85c5bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8581c5be, SUC_NE_lexicon_tags_2 },
    { 0xc325c5bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc89c5c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa881c5c5, SUC_NE_lexicon_tags_2 },
    { 0x7d4dc5c6, SUC_NE_lexicon_tags_98 },
    { 0x47dc5c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a9bc5c9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf1c5cd, SUC_NE_lexicon_tags_2 },
    { 0x268fc5ce, SUC_NE_lexicon_tags_4 },
    { 0x6b0dc5cf, SUC_NE_lexicon_tags_2 },
    { 0x490fc5cf, SUC_NE_lexicon_tags_3 },
    { 0x9d15c5ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa79c5d3, SUC_NE_lexicon_tags_2 },
    { 0x75e1c5d4, SUC_NE_lexicon_tags_2 },
    { 0x9ee9c5d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb11bc5d7, SUC_NE_lexicon_tags_2 },
    { 0xeb81c5d7, SUC_NE_lexicon_tags_2 },
    { 0xaba1c5d7, SUC_NE_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca3c5dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18dc5e7, SUC_NE_lexicon_tags_2 },
    { 0x278dc5e7, SUC_NE_lexicon_tags_2 },
    { 0x3d01c5e9, SUC_NE_lexicon_tags_2 },
    { 0xbaafc5ea, SUC_NE_lexicon_tags_2 },
    { 0x6397c5e9, SUC_NE_lexicon_tags_2 },
    { 0xbfe5c5ea, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df7c5f0, SUC_NE_lexicon_tags_2 },
    { 0x74b9c5f0, SUC_NE_lexicon_tags_2 },
    { 0xb3e7c5f2, SUC_NE_lexicon_tags_17 },
    { 0x270fc5f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c11c5f7, SUC_NE_lexicon_tags_2 },
    { 0x6b77c5f7, SUC_NE_lexicon_tags_36 },
    { 0x5c01c5f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee1c5fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8a1c5fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1325c600, SUC_NE_lexicon_tags_7 },
    { 0x8c37c601, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bdc604, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bf9c606, SUC_NE_lexicon_tags_2 },
    { 0x37e1c606, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fc60b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9c60e, SUC_NE_lexicon_tags_3 },
    { 0x56abc60f, SUC_NE_lexicon_tags_1 },
    { 0xfff5c610, SUC_NE_lexicon_tags_2 },
    { 0x7adfc60f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb047c614, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf879c616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8fc61b, SUC_NE_lexicon_tags_2 },
    { 0xaf4fc61b, SUC_NE_lexicon_tags_2 },
    { 0xf67c61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78bc61f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3bbc621, SUC_NE_lexicon_tags_2 },
    { 0x2089c621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc625, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e5c628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb915c62e, SUC_NE_lexicon_tags_2 },
    { 0xa3d5c62e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff7c633, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf9c636, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ce1c638, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x194fc63a, SUC_NE_lexicon_tags_2 },
    { 0x4f47c63a, SUC_NE_lexicon_tags_2 },
    { 0x1c65c63a, SUC_NE_lexicon_tags_2 },
    { 0xef8fc63b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd71c64a, SUC_NE_lexicon_tags_2 },
    { 0xfcfdc64b, SUC_NE_lexicon_tags_2 },
    { 0x8ca9c64c, SUC_NE_lexicon_tags_15 },
    { 0x917c64d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3343c64f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe277c651, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7663c653, SUC_NE_lexicon_tags_2 },
    { 0x1f1dc654, SUC_NE_lexicon_tags_64 },
    { 0x5e2bc655, SUC_NE_lexicon_tags_2 },
    { 0x4c59c656, SUC_NE_lexicon_tags_3 },
    { 0x27e7c657, SUC_NE_lexicon_tags_2 },
    { 0xba13c658, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf7c65b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee23c65d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a65c65f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb7c663, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbccfc665, SUC_NE_lexicon_tags_2 },
    { 0x8ac5c665, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa127c669, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9de7c66b, SUC_NE_lexicon_tags_2 },
    { 0x9523c66b, SUC_NE_lexicon_tags_2 },
    { 0x474dc66c, SUC_NE_lexicon_tags_2 },
    { 0xc419c66d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6467c670, SUC_NE_lexicon_tags_2 },
    { 0xcaafc670, SUC_NE_lexicon_tags_2 },
    { 0x6945c672, SUC_NE_lexicon_tags_4 },
    { 0x2381c670, SUC_NE_lexicon_tags_2 },
    { 0xa5bc674, SUC_NE_lexicon_tags_2 },
    { 0x85ddc671, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da5c678, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7bc67a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6795c680, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5173c682, SUC_NE_lexicon_tags_2 },
    { 0x6c35c682, SUC_NE_lexicon_tags_2 },
    { 0x5dc7c683, SUC_NE_lexicon_tags_2 },
    { 0x7ad5c684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb657c689, SUC_NE_lexicon_tags_2 },
    { 0x39dc68a, SUC_NE_lexicon_tags_2 },
    { 0xfecdc68b, SUC_NE_lexicon_tags_2 },
    { 0xe093c689, SUC_NE_lexicon_tags_7 },
    { 0x1275c68d, SUC_NE_lexicon_tags_2 },
    { 0x6d4bc68e, SUC_NE_lexicon_tags_2 },
    { 0xc14fc68e, SUC_NE_lexicon_tags_2 },
    { 0xe15c690, SUC_NE_lexicon_tags_2 },
    { 0xc219c691, SUC_NE_lexicon_tags_2 },
    { 0xdf4dc68f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa97c694, SUC_NE_lexicon_tags_2 },
    { 0x8153c695, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a61c698, SUC_NE_lexicon_tags_2 },
    { 0xdfa9c699, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc411c69b, SUC_NE_lexicon_tags_2 },
    { 0xc7c7c69c, SUC_NE_lexicon_tags_2 },
    { 0xeda3c69c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc63fc69f, SUC_NE_lexicon_tags_2 },
    { 0x163fc69f, SUC_NE_lexicon_tags_19 },
    { 0xcb19c6a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7fc6a4, SUC_NE_lexicon_tags_2 },
    { 0xb88dc6a4, SUC_NE_lexicon_tags_7 },
    { 0x120bc6a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4453c6a8, SUC_NE_lexicon_tags_2 },
    { 0xf09c6a9, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8713c6ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9225c6af, SUC_NE_lexicon_tags_2 },
    { 0x5341c6b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d35c6b3, SUC_NE_lexicon_tags_2 },
    { 0x5773c6b3, SUC_NE_lexicon_tags_2 },
    { 0x6359c6b5, SUC_NE_lexicon_tags_2 },
    { 0xdb15c6b3, SUC_NE_lexicon_tags_2 },
    { 0x2803c6b7, SUC_NE_lexicon_tags_2 },
    { 0x7287c6b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3083c6be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xce95c6c0, SUC_NE_lexicon_tags_2 },
    { 0xd7a1c6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1095c6c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb385c6c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1dc6cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7e7c6cd, SUC_NE_lexicon_tags_2 },
    { 0x76cdc6cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa18bc6d0, SUC_NE_lexicon_tags_31 },
    { 0x4f1dc6d1, SUC_NE_lexicon_tags_2 },
    { 0xdb43c6d2, SUC_NE_lexicon_tags_4 },
    { 0xff25c6d3, SUC_NE_lexicon_tags_2 },
    { 0x47a7c6d1, SUC_NE_lexicon_tags_2 },
    { 0x6e93c6d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb277c6d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd593c6e3, SUC_NE_lexicon_tags_2 },
    { 0x72a7c6e4, SUC_NE_lexicon_tags_2 },
    { 0x24dbc6e4, SUC_NE_lexicon_tags_2 },
    { 0x3041c6e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd1c6ec, SUC_NE_lexicon_tags_2 },
    { 0x6c0fc6ed, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x7bbbc6ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9bc6f3, SUC_NE_lexicon_tags_2 },
    { 0x8179c6f3, SUC_NE_lexicon_tags_2 },
    { 0x5dc6f4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76fc6fe, SUC_NE_lexicon_tags_2 },
    { 0xa8c7c6fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf433c703, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82bc707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc471c70d, SUC_NE_lexicon_tags_2 },
    { 0x85e1c70d, SUC_NE_lexicon_tags_2 },
    { 0xf82bc70d, SUC_NE_lexicon_tags_2 },
    { 0x37c5c70d, SUC_NE_lexicon_tags_2 },
    { 0x2721c711, SUC_NE_lexicon_tags_2 },
    { 0xd2e1c712, SUC_NE_lexicon_tags_2 },
    { 0xd9cdc70d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x6c17c715, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xfad1c717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91c71c, SUC_NE_lexicon_tags_2 },
    { 0x44f9c71d, SUC_NE_lexicon_tags_2 },
    { 0xdccbc71c, SUC_NE_lexicon_tags_2 },
    { 0xcb63c71f, SUC_NE_lexicon_tags_2 },
    { 0xfb5fc71c, SUC_NE_lexicon_tags_18 },
    { 0xf161c71c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8967c724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21c72d, SUC_NE_lexicon_tags_2 },
    { 0xf38fc72e, SUC_NE_lexicon_tags_2 },
    { 0x2d85c72f, SUC_NE_lexicon_tags_2 },
    { 0x514bc72e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0c3c732, SUC_NE_lexicon_tags_2 },
    { 0x7043c733, SUC_NE_lexicon_tags_2 },
    { 0x7e2dc734, SUC_NE_lexicon_tags_2 },
    { 0xac41c735, SUC_NE_lexicon_tags_2 },
    { 0x4935c736, SUC_NE_lexicon_tags_2 },
    { 0x8e25c737, SUC_NE_lexicon_tags_2 },
    { 0xa68fc738, SUC_NE_lexicon_tags_2 },
    { 0x67cdc734, SUC_NE_lexicon_tags_2 },
    { 0x4779c738, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a63c73c, SUC_NE_lexicon_tags_2 },
    { 0x670fc73c, SUC_NE_lexicon_tags_2 },
    { 0xb98dc73d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9c742, SUC_NE_lexicon_tags_3 },
    { 0x85e3c742, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5c749, SUC_NE_lexicon_tags_2 },
    { 0x3f69c74a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da7c74d, SUC_NE_lexicon_tags_2 },
    { 0x94b7c74e, SUC_NE_lexicon_tags_2 },
    { 0x1977c74f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900bc757, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc245c759, SUC_NE_lexicon_tags_17 },
    { 0x2ed9c75a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x673bc75d, SUC_NE_lexicon_tags_1 },
    { 0x2f75c75e, SUC_NE_lexicon_tags_2 },
    { 0xebf5c75e, SUC_NE_lexicon_tags_2 },
    { 0xb933c75d, SUC_NE_lexicon_tags_2 },
    { 0xf739c75e, SUC_NE_lexicon_tags_2 },
    { 0xc265c760, SUC_NE_lexicon_tags_2 },
    { 0x6431c763, SUC_NE_lexicon_tags_2 },
    { 0x83e7c764, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38a5c766, SUC_NE_lexicon_tags_2 },
    { 0x7095c767, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a65c769, SUC_NE_lexicon_tags_2 },
    { 0x981fc769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb91c76c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c29c76e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb95dc770, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f53c772, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f87c775, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xacf3c777, SUC_NE_lexicon_tags_2 },
    { 0xb133c778, SUC_NE_lexicon_tags_3 },
    { 0x79bfc777, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28cfc77b, SUC_NE_lexicon_tags_2 },
    { 0x37edc77c, SUC_NE_lexicon_tags_2 },
    { 0x29c5c77d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd763c780, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5fc783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4003c785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ae3c787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e7c792, SUC_NE_lexicon_tags_2 },
    { 0x10a3c792, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4fc796, SUC_NE_lexicon_tags_2 },
    { 0x8a6bc797, SUC_NE_lexicon_tags_2 },
    { 0xecb7c798, SUC_NE_lexicon_tags_2 },
    { 0x9f41c797, SUC_NE_lexicon_tags_2 },
    { 0x82bdc79a, SUC_NE_lexicon_tags_2 },
    { 0x1769c79b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7937c79d, SUC_NE_lexicon_tags_2 },
    { 0xa78bc79e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc905c7a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3953c7a3, SUC_NE_lexicon_tags_2 },
    { 0x9203c7a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49ebc7a6, SUC_NE_lexicon_tags_2 },
    { 0xb5d1c7a7, SUC_NE_lexicon_tags_2 },
    { 0x2ac1c7a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9433c7af, SUC_NE_lexicon_tags_2 },
    { 0x27f3c7af, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xd833c7b2, SUC_NE_lexicon_tags_2 },
    { 0x26dc7b3, SUC_NE_lexicon_tags_19 },
    { 0x1b0dc7b4, SUC_NE_lexicon_tags_2 },
    { 0xf67fc7b5, SUC_NE_lexicon_tags_2 },
    { 0x17e1c7b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c77c7bb, SUC_NE_lexicon_tags_13 },
    { 0xd9f7c7bb, SUC_NE_lexicon_tags_2 },
    { 0xd07fc7bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8565c7c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf777c7c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14dc7cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb413c7ce, SUC_NE_lexicon_tags_2 },
    { 0x1f7c7ce, SUC_NE_lexicon_tags_2 },
    { 0x2f01c7d0, SUC_NE_lexicon_tags_2 },
    { 0xa93c7d1, SUC_NE_lexicon_tags_2 },
    { 0x7253c7ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3907c7d6, SUC_NE_lexicon_tags_7 },
    { 0xe1a5c7d6, SUC_NE_lexicon_tags_7 },
    { 0xafabc7d8, SUC_NE_lexicon_tags_2 },
    { 0xe229c7d9, SUC_NE_lexicon_tags_2 },
    { 0xed11c7da, SUC_NE_lexicon_tags_2 },
    { 0xb603c7db, SUC_NE_lexicon_tags_2 },
    { 0xcb09c7dc, SUC_NE_lexicon_tags_2 },
    { 0xc5f5c7dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76e3c7df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb941c7e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9cdc7e7, SUC_NE_lexicon_tags_2 },
    { 0x3549c7e8, SUC_NE_lexicon_tags_2 },
    { 0xb4b3c7e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c7ef, SUC_NE_lexicon_tags_2 },
    { 0xe391c7ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bdc7f4, SUC_NE_lexicon_tags_2 },
    { 0x327c7f4, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff5c7fa, SUC_NE_lexicon_tags_2 },
    { 0x830dc7fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c7fe, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ffc802, SUC_NE_lexicon_tags_2 },
    { 0x4d65c803, SUC_NE_lexicon_tags_2 },
    { 0x74a5c803, SUC_NE_lexicon_tags_2 },
    { 0x52dbc805, SUC_NE_lexicon_tags_2 },
    { 0x182bc805, SUC_NE_lexicon_tags_6 },
    { 0, NULL },
    { 0x929dc808, SUC_NE_lexicon_tags_2 },
    { 0x5041c809, SUC_NE_lexicon_tags_252 },
    { 0xb6bc809, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0x52f7c80c, SUC_NE_lexicon_tags_2 },
    { 0x29d5c80d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61cbc80f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8001c814, SUC_NE_lexicon_tags_2 },
    { 0xaa39c815, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf585c818, SUC_NE_lexicon_tags_2 },
    { 0xbd9dc819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9789c81d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51efc81f, SUC_NE_lexicon_tags_2 },
    { 0xcf0dc820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fbc826, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dd5c828, SUC_NE_lexicon_tags_2 },
    { 0xb9cbc828, SUC_NE_lexicon_tags_2 },
    { 0x7653c829, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a57c834, SUC_NE_lexicon_tags_7 },
    { 0xd461c835, SUC_NE_lexicon_tags_2 },
    { 0x39dfc836, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb693c838, SUC_NE_lexicon_tags_189 },
    { 0x58d3c838, SUC_NE_lexicon_tags_2 },
    { 0x2f35c839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x397c83c, SUC_NE_lexicon_tags_2 },
    { 0x8969c83c, SUC_NE_lexicon_tags_2 },
    { 0x1797c83c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x56f7c840, SUC_NE_lexicon_tags_2 },
    { 0x7e51c841, SUC_NE_lexicon_tags_2 },
    { 0x2001c842, SUC_NE_lexicon_tags_2 },
    { 0x3c1fc841, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd57c845, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa953c84d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x74d1c84f, SUC_NE_lexicon_tags_2 },
    { 0xc02bc84f, SUC_NE_lexicon_tags_2 },
    { 0xae3c851, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d1c856, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665bc85a, SUC_NE_lexicon_tags_2 },
    { 0xc3f1c85b, SUC_NE_lexicon_tags_2 },
    { 0x79abc85c, SUC_NE_lexicon_tags_2 },
    { 0xc1c5c85a, SUC_NE_lexicon_tags_2 },
    { 0x115bc85d, SUC_NE_lexicon_tags_2 },
    { 0x7e27c85f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb797c864, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6793c870, SUC_NE_lexicon_tags_2 },
    { 0xf9b9c871, SUC_NE_lexicon_tags_2 },
    { 0xd083c872, SUC_NE_lexicon_tags_2 },
    { 0x3cb9c872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf79c875, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89dfc87a, SUC_NE_lexicon_tags_2 },
    { 0xda49c87a, SUC_NE_lexicon_tags_2 },
    { 0x9b1bc87c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1bc880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f39c884, SUC_NE_lexicon_tags_2 },
    { 0x9925c885, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x85f1c887, SUC_NE_lexicon_tags_24 },
    { 0x924fc888, SUC_NE_lexicon_tags_17 },
    { 0x3583c888, SUC_NE_lexicon_tags_2 },
    { 0x3fc7c88a, SUC_NE_lexicon_tags_2 },
    { 0x451bc888, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef5c891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe1c895, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d7c898, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3db5c89a, SUC_NE_lexicon_tags_2 },
    { 0xc44bc89b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2e1c89d, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcfc8a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccffc8a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x137fc8a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1c8ae, SUC_NE_lexicon_tags_0 },
    { 0xba65c8af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c9c8b1, SUC_NE_lexicon_tags_2 },
    { 0x9e8dc8b2, SUC_NE_lexicon_tags_2 },
    { 0x79bdc8b1, SUC_NE_lexicon_tags_2 },
    { 0xe2bbc8b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603fc8bc, SUC_NE_lexicon_tags_1 },
    { 0x12d1c8bd, SUC_NE_lexicon_tags_1 },
    { 0x9801c8be, SUC_NE_lexicon_tags_2 },
    { 0x91edc8bf, SUC_NE_lexicon_tags_19 },
    { 0xd82dc8c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6761c8c2, SUC_NE_lexicon_tags_169 },
    { 0xc8f9c8c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf1c8c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf757c8c9, SUC_NE_lexicon_tags_2 },
    { 0xd799c8ca, SUC_NE_lexicon_tags_2 },
    { 0x4af3c8ca, SUC_NE_lexicon_tags_17 },
    { 0x523bc8cc, SUC_NE_lexicon_tags_4 },
    { 0x2995c8cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13c1c8cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf367c8d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ffc8d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5c8d9, SUC_NE_lexicon_tags_2 },
    { 0x89efc8da, SUC_NE_lexicon_tags_2 },
    { 0x969c8da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224fc8e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb273c8e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f5c8e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb537c8eb, SUC_NE_lexicon_tags_2 },
    { 0xc475c8eb, SUC_NE_lexicon_tags_2 },
    { 0xdbbfc8ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80ddc8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6cbc8f1, SUC_NE_lexicon_tags_7 },
    { 0x5801c8f2, SUC_NE_lexicon_tags_2 },
    { 0x4fd5c8f3, SUC_NE_lexicon_tags_21 },
    { 0, NULL },
    { 0xe16fc8f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf69c8f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe739c8fa, SUC_NE_lexicon_tags_2 },
    { 0xa61dc8fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd91c8ff, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94fc908, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e65c90b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e9c90f, SUC_NE_lexicon_tags_2 },
    { 0xf0b1c910, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53c91d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbbc920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcad5c922, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe099c925, SUC_NE_lexicon_tags_2 },
    { 0xeb7c926, SUC_NE_lexicon_tags_2 },
    { 0x3b55c925, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a3c92b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ba9c92d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2239c930, SUC_NE_lexicon_tags_2 },
    { 0x8dddc931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62f3c933, SUC_NE_lexicon_tags_2 },
    { 0xb85bc934, SUC_NE_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96bc938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401fc93d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16dc93f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1043c941, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4fc944, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeddc946, SUC_NE_lexicon_tags_49 },
    { 0x4583c946, SUC_NE_lexicon_tags_2 },
    { 0x2fafc948, SUC_NE_lexicon_tags_2 },
    { 0x2131c949, SUC_NE_lexicon_tags_2 },
    { 0xa3d7c94a, SUC_NE_lexicon_tags_2 },
    { 0xd5cbc948, SUC_NE_lexicon_tags_2 },
    { 0xffddc947, SUC_NE_lexicon_tags_2 },
    { 0x2bddc94c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b9c950, SUC_NE_lexicon_tags_2 },
    { 0x3a73c951, SUC_NE_lexicon_tags_0 },
    { 0x3869c952, SUC_NE_lexicon_tags_2 },
    { 0xc10fc951, SUC_NE_lexicon_tags_2 },
    { 0xb8f3c954, SUC_NE_lexicon_tags_2 },
    { 0x3141c955, SUC_NE_lexicon_tags_2 },
    { 0x474fc956, SUC_NE_lexicon_tags_3 },
    { 0xfc53c957, SUC_NE_lexicon_tags_2 },
    { 0xccffc957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x61d1c95a, SUC_NE_lexicon_tags_2 },
    { 0xc467c95a, SUC_NE_lexicon_tags_2 },
    { 0xd93dc95b, SUC_NE_lexicon_tags_2 },
    { 0xab63c95c, SUC_NE_lexicon_tags_2 },
    { 0xf851c95e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603fc965, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0e1c967, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cdc96d, SUC_NE_lexicon_tags_2 },
    { 0xf281c96e, SUC_NE_lexicon_tags_2 },
    { 0xc9f7c96e, SUC_NE_lexicon_tags_2 },
    { 0xe5e9c970, SUC_NE_lexicon_tags_2 },
    { 0x211c971, SUC_NE_lexicon_tags_11 },
    { 0x7cc1c972, SUC_NE_lexicon_tags_2 },
    { 0xb543c96e, SUC_NE_lexicon_tags_2 },
    { 0x4717c974, SUC_NE_lexicon_tags_2 },
    { 0xb481c975, SUC_NE_lexicon_tags_2 },
    { 0x31e9c96e, SUC_NE_lexicon_tags_86 },
    { 0x35e1c977, SUC_NE_lexicon_tags_2 },
    { 0xc6ffc978, SUC_NE_lexicon_tags_2 },
    { 0xab3dc978, SUC_NE_lexicon_tags_2 },
    { 0xe13dc97a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa233c981, SUC_NE_lexicon_tags_7 },
    { 0x5df5c982, SUC_NE_lexicon_tags_2 },
    { 0x32adc983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ffc986, SUC_NE_lexicon_tags_2 },
    { 0xcc33c987, SUC_NE_lexicon_tags_2 },
    { 0xd033c988, SUC_NE_lexicon_tags_2 },
    { 0x416dc989, SUC_NE_lexicon_tags_17 },
    { 0x4d3c987, SUC_NE_lexicon_tags_2 },
    { 0xf74fc98b, SUC_NE_lexicon_tags_2 },
    { 0x8f2fc98c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5659c98e, SUC_NE_lexicon_tags_2 },
    { 0x4617c98f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab37c993, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13e5c995, SUC_NE_lexicon_tags_2 },
    { 0x6adc995, SUC_NE_lexicon_tags_2 },
    { 0x834fc996, SUC_NE_lexicon_tags_2 },
    { 0x2bb5c998, SUC_NE_lexicon_tags_2 },
    { 0xdffdc999, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d17c9a1, SUC_NE_lexicon_tags_2 },
    { 0xa1b3c9a2, SUC_NE_lexicon_tags_2 },
    { 0x6161c9a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c9c9a7, SUC_NE_lexicon_tags_2 },
    { 0x31c3c9a7, SUC_NE_lexicon_tags_2 },
    { 0x3d69c9a9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xa985c9ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd08fc9ad, SUC_NE_lexicon_tags_2 },
    { 0x9a1fc9ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd03bc9b0, SUC_NE_lexicon_tags_1 },
    { 0xbe2dc9b1, SUC_NE_lexicon_tags_2 },
    { 0x9fddc9b1, SUC_NE_lexicon_tags_2 },
    { 0xd35fc9b2, SUC_NE_lexicon_tags_2 },
    { 0x8763c9b4, SUC_NE_lexicon_tags_2 },
    { 0x861c9b1, SUC_NE_lexicon_tags_7 },
    { 0x243dc9b6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xe7e1c9b8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbafbc9ba, SUC_NE_lexicon_tags_2 },
    { 0xbbffc9bb, SUC_NE_lexicon_tags_2 },
    { 0x789fc9bc, SUC_NE_lexicon_tags_2 },
    { 0xeb93c9bc, SUC_NE_lexicon_tags_2 },
    { 0x9fedc9bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5003c9c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bbc9c4, SUC_NE_lexicon_tags_2 },
    { 0xcd7bc9c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe4fc9c7, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee3c9cb, SUC_NE_lexicon_tags_2 },
    { 0xf111c9cc, SUC_NE_lexicon_tags_2 },
    { 0xcfa3c9cd, SUC_NE_lexicon_tags_17 },
    { 0xcf4fc9cd, SUC_NE_lexicon_tags_2 },
    { 0xe365c9cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bc9d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c03c9d6, SUC_NE_lexicon_tags_2 },
    { 0x634bc9d6, SUC_NE_lexicon_tags_18 },
    { 0xb203c9d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4705c9db, SUC_NE_lexicon_tags_2 },
    { 0xd69bc9db, SUC_NE_lexicon_tags_2 },
    { 0xb7f9c9dd, SUC_NE_lexicon_tags_2 },
    { 0x71bc9dd, SUC_NE_lexicon_tags_2 },
    { 0x632bc9df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23afc9e3, SUC_NE_lexicon_tags_2 },
    { 0xa09dc9e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318dc9e9, SUC_NE_lexicon_tags_2 },
    { 0x8067c9ea, SUC_NE_lexicon_tags_2 },
    { 0xe15c9eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x211c9ed, SUC_NE_lexicon_tags_2 },
    { 0x8941c9ee, SUC_NE_lexicon_tags_3 },
    { 0x3247c9ef, SUC_NE_lexicon_tags_2 },
    { 0x5bf7c9f0, SUC_NE_lexicon_tags_2 },
    { 0x15fc9f0, SUC_NE_lexicon_tags_3 },
    { 0xaa21c9ed, SUC_NE_lexicon_tags_2 },
    { 0x3d7bc9f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c87c9f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc675c9fc, SUC_NE_lexicon_tags_2 },
    { 0xd953c9fc, SUC_NE_lexicon_tags_34 },
    { 0xed19c9fc, SUC_NE_lexicon_tags_2 },
    { 0xf843c9ff, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd9ca03, SUC_NE_lexicon_tags_2 },
    { 0x5937ca04, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xdf7ca06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4743ca09, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaebca0c, SUC_NE_lexicon_tags_2 },
    { 0x671bca0c, SUC_NE_lexicon_tags_2 },
    { 0x9d3bca0d, SUC_NE_lexicon_tags_2 },
    { 0xdbf5ca0f, SUC_NE_lexicon_tags_2 },
    { 0xffa5ca10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a5ca14, SUC_NE_lexicon_tags_2 },
    { 0x5f87ca14, SUC_NE_lexicon_tags_2 },
    { 0x8699ca16, SUC_NE_lexicon_tags_2 },
    { 0x8ea7ca16, SUC_NE_lexicon_tags_80 },
    { 0xef01ca14, SUC_NE_lexicon_tags_2 },
    { 0x828fca19, SUC_NE_lexicon_tags_4 },
    { 0x469bca1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1849ca1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1007ca23, SUC_NE_lexicon_tags_2 },
    { 0xad39ca24, SUC_NE_lexicon_tags_102 },
    { 0xbe7bca23, SUC_NE_lexicon_tags_2 },
    { 0xf19dca25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875bca2f, SUC_NE_lexicon_tags_2 },
    { 0x2c19ca30, SUC_NE_lexicon_tags_138 },
    { 0xe6b7ca31, SUC_NE_lexicon_tags_2 },
    { 0xcedca32, SUC_NE_lexicon_tags_18 },
    { 0xc4a9ca31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8657ca35, SUC_NE_lexicon_tags_2 },
    { 0x9aa9ca36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a7ca3a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb7ca3d, SUC_NE_lexicon_tags_2 },
    { 0xac59ca3e, SUC_NE_lexicon_tags_2 },
    { 0x6d2dca3d, SUC_NE_lexicon_tags_2 },
    { 0x3b09ca40, SUC_NE_lexicon_tags_2 },
    { 0x153ca40, SUC_NE_lexicon_tags_2 },
    { 0xb0c1ca42, SUC_NE_lexicon_tags_2 },
    { 0x4337ca42, SUC_NE_lexicon_tags_40 },
    { 0x2f53ca43, SUC_NE_lexicon_tags_2 },
    { 0xb19bca45, SUC_NE_lexicon_tags_2 },
    { 0xe6bca46, SUC_NE_lexicon_tags_2 },
    { 0x9a6dca47, SUC_NE_lexicon_tags_2 },
    { 0xdb23ca46, SUC_NE_lexicon_tags_0 },
    { 0x8141ca47, SUC_NE_lexicon_tags_2 },
    { 0x9025ca45, SUC_NE_lexicon_tags_5 },
    { 0x699fca42, SUC_NE_lexicon_tags_2 },
    { 0x5213ca3d, SUC_NE_lexicon_tags_2 },
    { 0x7b8dca4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf3dca4f, SUC_NE_lexicon_tags_2 },
    { 0x2a43ca50, SUC_NE_lexicon_tags_2 },
    { 0x6735ca4f, SUC_NE_lexicon_tags_2 },
    { 0xd5afca52, SUC_NE_lexicon_tags_2 },
    { 0xbe03ca50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b59ca58, SUC_NE_lexicon_tags_0 },
    { 0x928dca58, SUC_NE_lexicon_tags_2 },
    { 0xb69dca5a, SUC_NE_lexicon_tags_2 },
    { 0x2743ca5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1ca60, SUC_NE_lexicon_tags_2 },
    { 0x470bca61, SUC_NE_lexicon_tags_2 },
    { 0x98bbca61, SUC_NE_lexicon_tags_7 },
    { 0x3a13ca63, SUC_NE_lexicon_tags_2 },
    { 0x3665ca62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f79ca66, SUC_NE_lexicon_tags_2 },
    { 0x2b53ca67, SUC_NE_lexicon_tags_2 },
    { 0x8749ca66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd57ca6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc5ca70, SUC_NE_lexicon_tags_2 },
    { 0xc4d1ca70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda9ca75, SUC_NE_lexicon_tags_2 },
    { 0x6a5ca76, SUC_NE_lexicon_tags_2 },
    { 0x41d1ca77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x125dca7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b9ca80, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x500bca83, SUC_NE_lexicon_tags_7 },
    { 0xa7c5ca84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f17ca87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af3ca8a, SUC_NE_lexicon_tags_2 },
    { 0xc807ca8b, SUC_NE_lexicon_tags_2 },
    { 0x3395ca8c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3addca8f, SUC_NE_lexicon_tags_2 },
    { 0x1319ca90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf935ca92, SUC_NE_lexicon_tags_3 },
    { 0x5041ca93, SUC_NE_lexicon_tags_2 },
    { 0x981ca94, SUC_NE_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6329ca98, SUC_NE_lexicon_tags_4 },
    { 0x40fca99, SUC_NE_lexicon_tags_4 },
    { 0x9df7ca9a, SUC_NE_lexicon_tags_2 },
    { 0x798bca9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec37ca9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd23caa4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb787caa7, SUC_NE_lexicon_tags_1 },
    { 0x8de1caa7, SUC_NE_lexicon_tags_2 },
    { 0xc1e7caa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6221caac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf04fcaae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610fcab5, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e8fcab7, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcbcabd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfea3cabf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x174dcac2, SUC_NE_lexicon_tags_2 },
    { 0xdfc7cac2, SUC_NE_lexicon_tags_2 },
    { 0x5ca5cac2, SUC_NE_lexicon_tags_3 },
    { 0xb60bcac5, SUC_NE_lexicon_tags_2 },
    { 0x15cdcac5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5713cac8, SUC_NE_lexicon_tags_2 },
    { 0x21cbcac9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x444dcacb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6fcace, SUC_NE_lexicon_tags_2 },
    { 0x97fcacf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a87cad1, SUC_NE_lexicon_tags_40 },
    { 0x2815cad2, SUC_NE_lexicon_tags_0 },
    { 0x4e5dcad3, SUC_NE_lexicon_tags_2 },
    { 0x51f9cad4, SUC_NE_lexicon_tags_2 },
    { 0x175bcad5, SUC_NE_lexicon_tags_2 },
    { 0xc3cdcad6, SUC_NE_lexicon_tags_2 },
    { 0xa84fcad7, SUC_NE_lexicon_tags_2 },
    { 0x6859cad8, SUC_NE_lexicon_tags_2 },
    { 0xec9bcad2, SUC_NE_lexicon_tags_18 },
    { 0xe1dbcad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1325cadf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaffcae2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1499cae7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd337caec, SUC_NE_lexicon_tags_2 },
    { 0x339caed, SUC_NE_lexicon_tags_2 },
    { 0x3449caee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe951caf1, SUC_NE_lexicon_tags_157 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6151caf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fcafb, SUC_NE_lexicon_tags_2 },
    { 0x38bcafb, SUC_NE_lexicon_tags_2 },
    { 0xa8ddcafb, SUC_NE_lexicon_tags_2 },
    { 0x855bcafe, SUC_NE_lexicon_tags_2 },
    { 0x6805cafe, SUC_NE_lexicon_tags_7 },
    { 0x4121caff, SUC_NE_lexicon_tags_2 },
    { 0x8b8fcafe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25fcb04, SUC_NE_lexicon_tags_2 },
    { 0xee43cb05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f5cb08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc3cb0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad3fcb0d, SUC_NE_lexicon_tags_2 },
    { 0x7d3bcb0e, SUC_NE_lexicon_tags_3 },
    { 0x73afcb0f, SUC_NE_lexicon_tags_2 },
    { 0xf089cb10, SUC_NE_lexicon_tags_2 },
    { 0x9c91cb0d, SUC_NE_lexicon_tags_2 },
    { 0x4ac1cb0e, SUC_NE_lexicon_tags_2 },
    { 0xe137cb11, SUC_NE_lexicon_tags_2 },
    { 0xc879cb14, SUC_NE_lexicon_tags_2 },
    { 0xd3bfcb10, SUC_NE_lexicon_tags_2 },
    { 0x1cbfcb12, SUC_NE_lexicon_tags_2 },
    { 0xdd47cb17, SUC_NE_lexicon_tags_2 },
    { 0x610fcb18, SUC_NE_lexicon_tags_7 },
    { 0x33d9cb18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46fcb1c, SUC_NE_lexicon_tags_2 },
    { 0xfe9dcb1c, SUC_NE_lexicon_tags_17 },
    { 0xf8a1cb1e, SUC_NE_lexicon_tags_2 },
    { 0x2ccfcb1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f09cb21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db3cb24, SUC_NE_lexicon_tags_2 },
    { 0x7159cb25, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xa231cb27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5337cb2e, SUC_NE_lexicon_tags_2 },
    { 0xabbfcb2e, SUC_NE_lexicon_tags_2 },
    { 0x148fcb2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a1cb34, SUC_NE_lexicon_tags_2 },
    { 0xd041cb35, SUC_NE_lexicon_tags_2 },
    { 0xe0a3cb35, SUC_NE_lexicon_tags_2 },
    { 0xda47cb37, SUC_NE_lexicon_tags_2 },
    { 0x6255cb38, SUC_NE_lexicon_tags_2 },
    { 0x45a9cb38, SUC_NE_lexicon_tags_2 },
    { 0xa7b5cb3a, SUC_NE_lexicon_tags_2 },
    { 0xacebcb38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a3cb3e, SUC_NE_lexicon_tags_2 },
    { 0xa8d5cb3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904bcb44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7259cb4a, SUC_NE_lexicon_tags_2 },
    { 0x612dcb4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdbdcb4d, SUC_NE_lexicon_tags_2 },
    { 0x734bcb4d, SUC_NE_lexicon_tags_2 },
    { 0x3331cb4d, SUC_NE_lexicon_tags_2 },
    { 0x4009cb50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bcb54, SUC_NE_lexicon_tags_3 },
    { 0xf733cb55, SUC_NE_lexicon_tags_32 },
    { 0x8fe5cb56, SUC_NE_lexicon_tags_22 },
    { 0xd1fbcb57, SUC_NE_lexicon_tags_2 },
    { 0xfa1dcb57, SUC_NE_lexicon_tags_2 },
    { 0x542bcb59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9249cb5d, SUC_NE_lexicon_tags_2 },
    { 0xf5d5cb5d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd9cb61, SUC_NE_lexicon_tags_3 },
    { 0x216dcb62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xedffcb64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb31cb66, SUC_NE_lexicon_tags_2 },
    { 0x228fcb67, SUC_NE_lexicon_tags_4 },
    { 0x7931cb68, SUC_NE_lexicon_tags_2 },
    { 0xa9d3cb69, SUC_NE_lexicon_tags_2 },
    { 0x6a1bcb6a, SUC_NE_lexicon_tags_2 },
    { 0xf557cb6b, SUC_NE_lexicon_tags_2 },
    { 0x5d6fcb6c, SUC_NE_lexicon_tags_2 },
    { 0x521dcb69, SUC_NE_lexicon_tags_2 },
    { 0x608dcb6e, SUC_NE_lexicon_tags_2 },
    { 0xa681cb6f, SUC_NE_lexicon_tags_2 },
    { 0x3d35cb6e, SUC_NE_lexicon_tags_2 },
    { 0xe4ddcb71, SUC_NE_lexicon_tags_2 },
    { 0x6d1cb72, SUC_NE_lexicon_tags_2 },
    { 0xaaa3cb73, SUC_NE_lexicon_tags_4 },
    { 0x615dcb72, SUC_NE_lexicon_tags_2 },
    { 0x8b0bcb75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1581cb78, SUC_NE_lexicon_tags_2 },
    { 0x482bcb79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf9cb7d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9393cb80, SUC_NE_lexicon_tags_2 },
    { 0x74a1cb81, SUC_NE_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1bcb85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5645cb87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5bcb8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9dcb8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5161cb90, SUC_NE_lexicon_tags_2 },
    { 0x2875cb90, SUC_NE_lexicon_tags_2 },
    { 0x2c65cb92, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef13cb98, SUC_NE_lexicon_tags_5 },
    { 0xfcd7cb99, SUC_NE_lexicon_tags_2 },
    { 0x33bdcb98, SUC_NE_lexicon_tags_2 },
    { 0xd1fdcb99, SUC_NE_lexicon_tags_2 },
    { 0x6855cb9b, SUC_NE_lexicon_tags_2 },
    { 0x1ae1cb99, SUC_NE_lexicon_tags_2 },
    { 0xa09fcb9e, SUC_NE_lexicon_tags_24 },
    { 0xe3e9cb9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f9cba2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10dcba5, SUC_NE_lexicon_tags_2 },
    { 0xdb9cba6, SUC_NE_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8881cbaa, SUC_NE_lexicon_tags_24 },
    { 0xf1dfcbab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f3cbae, SUC_NE_lexicon_tags_0 },
    { 0xa437cbaf, SUC_NE_lexicon_tags_2 },
    { 0xd72fcbb0, SUC_NE_lexicon_tags_2 },
    { 0xce17cbb1, SUC_NE_lexicon_tags_17 },
    { 0xb717cbb2, SUC_NE_lexicon_tags_2 },
    { 0x8321cbb3, SUC_NE_lexicon_tags_2 },
    { 0x4b5dcbb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee85cbb6, SUC_NE_lexicon_tags_2 },
    { 0x75dfcbb6, SUC_NE_lexicon_tags_2 },
    { 0xd00fcbb7, SUC_NE_lexicon_tags_2 },
    { 0x1f8bcbb9, SUC_NE_lexicon_tags_2 },
    { 0x796fcbb9, SUC_NE_lexicon_tags_2 },
    { 0xaf27cbb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca5cbbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa279cbc1, SUC_NE_lexicon_tags_2 },
    { 0x66f1cbc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdfcbc5, SUC_NE_lexicon_tags_2 },
    { 0xc05dcbc5, SUC_NE_lexicon_tags_7 },
    { 0xbe8dcbc5, SUC_NE_lexicon_tags_7 },
    { 0x6677cbc7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x5655cbca, SUC_NE_lexicon_tags_2 },
    { 0x1c27cbcb, SUC_NE_lexicon_tags_2 },
    { 0x26e1cbca, SUC_NE_lexicon_tags_2 },
    { 0xbde5cbcc, SUC_NE_lexicon_tags_2 },
    { 0x6523cbce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a5cbd1, SUC_NE_lexicon_tags_4 },
    { 0x2a7cbd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e75cbd4, SUC_NE_lexicon_tags_2 },
    { 0x2687cbd4, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502bcbd9, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x770bcbdb, SUC_NE_lexicon_tags_2 },
    { 0x39b9cbdb, SUC_NE_lexicon_tags_2 },
    { 0xd95bcbdd, SUC_NE_lexicon_tags_2 },
    { 0xb4c5cbdd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9645cbe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aadcbe2, SUC_NE_lexicon_tags_2 },
    { 0x24a7cbe3, SUC_NE_lexicon_tags_4 },
    { 0xc837cbe4, SUC_NE_lexicon_tags_2 },
    { 0xe367cbe4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7691cbe8, SUC_NE_lexicon_tags_2 },
    { 0x23dfcbe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1897cbee, SUC_NE_lexicon_tags_2 },
    { 0xd265cbef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b45cbf3, SUC_NE_lexicon_tags_2 },
    { 0x791cbf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39abcbf9, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x789bcbfc, SUC_NE_lexicon_tags_2 },
    { 0xfa97cbfd, SUC_NE_lexicon_tags_2 },
    { 0xfc65cbfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x443cc01, SUC_NE_lexicon_tags_2 },
    { 0x5769cc02, SUC_NE_lexicon_tags_2 },
    { 0x1d41cc01, SUC_NE_lexicon_tags_401 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53cc09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35bfcc0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962bcc15, SUC_NE_lexicon_tags_7 },
    { 0xb035cc16, SUC_NE_lexicon_tags_8 },
    { 0x713dcc17, SUC_NE_lexicon_tags_2 },
    { 0x45bdcc18, SUC_NE_lexicon_tags_2 },
    { 0x9a5bcc19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9bcc1c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afcc1f, SUC_NE_lexicon_tags_2 },
    { 0xe65fcc20, SUC_NE_lexicon_tags_7 },
    { 0x4725cc21, SUC_NE_lexicon_tags_2 },
    { 0x6b25cc22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc89cc2e, SUC_NE_lexicon_tags_2 },
    { 0x9849cc2e, SUC_NE_lexicon_tags_2 },
    { 0xf413cc30, SUC_NE_lexicon_tags_2 },
    { 0xdc27cc30, SUC_NE_lexicon_tags_2 },
    { 0x6f03cc32, SUC_NE_lexicon_tags_2 },
    { 0x4b59cc33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b3cc38, SUC_NE_lexicon_tags_2 },
    { 0x2fa5cc38, SUC_NE_lexicon_tags_2 },
    { 0xbad5cc39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a3cc3c, SUC_NE_lexicon_tags_2 },
    { 0xf293cc3d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf335cc41, SUC_NE_lexicon_tags_2 },
    { 0xc19bcc41, SUC_NE_lexicon_tags_2 },
    { 0x9a61cc41, SUC_NE_lexicon_tags_2 },
    { 0x6be9cc44, SUC_NE_lexicon_tags_2 },
    { 0x7aa5cc44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6109cc47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5979cc4a, SUC_NE_lexicon_tags_2 },
    { 0xa217cc4b, SUC_NE_lexicon_tags_2 },
    { 0x238bcc4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c5cc4f, SUC_NE_lexicon_tags_2 },
    { 0xbcf5cc4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8445cc52, SUC_NE_lexicon_tags_2 },
    { 0xbfa7cc53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc5cc56, SUC_NE_lexicon_tags_2 },
    { 0xce35cc57, SUC_NE_lexicon_tags_8 },
    { 0x63ddcc57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26dcc5b, SUC_NE_lexicon_tags_2 },
    { 0x94a5cc5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71edcc5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e3cc62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd113cc64, SUC_NE_lexicon_tags_2 },
    { 0xd3c1cc65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a95cc67, SUC_NE_lexicon_tags_2 },
    { 0xcc83cc68, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x73b7cc6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97cc6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44bcc70, SUC_NE_lexicon_tags_2 },
    { 0x90e7cc70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f41cc75, SUC_NE_lexicon_tags_2 },
    { 0xca4bcc75, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f9cc7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedcc7e, SUC_NE_lexicon_tags_2 },
    { 0x3e7fcc7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7869cc82, SUC_NE_lexicon_tags_2 },
    { 0x6935cc83, SUC_NE_lexicon_tags_2 },
    { 0x5361cc84, SUC_NE_lexicon_tags_2 },
    { 0xd73cc83, SUC_NE_lexicon_tags_32 },
    { 0xf401cc86, SUC_NE_lexicon_tags_2 },
    { 0x27bdcc86, SUC_NE_lexicon_tags_2 },
    { 0x4e17cc88, SUC_NE_lexicon_tags_2 },
    { 0x6dc9cc89, SUC_NE_lexicon_tags_2 },
    { 0xe92dcc85, SUC_NE_lexicon_tags_2 },
    { 0x5361cc85, SUC_NE_lexicon_tags_2 },
    { 0xfa9bcc8a, SUC_NE_lexicon_tags_2 },
    { 0xba41cc8d, SUC_NE_lexicon_tags_2 },
    { 0x37c7cc8e, SUC_NE_lexicon_tags_2 },
    { 0x1f0dcc8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2955cc92, SUC_NE_lexicon_tags_3 },
    { 0x2e79cc93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6bcc9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b41cc9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddefcca2, SUC_NE_lexicon_tags_102 },
    { 0xe63fcca3, SUC_NE_lexicon_tags_2 },
    { 0xa6e5cca4, SUC_NE_lexicon_tags_2 },
    { 0x221cca5, SUC_NE_lexicon_tags_2 },
    { 0xff9bcca6, SUC_NE_lexicon_tags_2 },
    { 0x9cc5cca4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6689ccaa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fbccb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7705ccb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c5dccb7, SUC_NE_lexicon_tags_2 },
    { 0xb02dccb7, SUC_NE_lexicon_tags_2 },
    { 0xd93ccb9, SUC_NE_lexicon_tags_18 },
    { 0x268dccb9, SUC_NE_lexicon_tags_2 },
    { 0xc82bccbb, SUC_NE_lexicon_tags_2 },
    { 0xa3dccb7, SUC_NE_lexicon_tags_2 },
    { 0x4839ccba, SUC_NE_lexicon_tags_2 },
    { 0x2ef7ccbe, SUC_NE_lexicon_tags_2 },
    { 0x6975ccbe, SUC_NE_lexicon_tags_2 },
    { 0xb5b9ccbe, SUC_NE_lexicon_tags_2 },
    { 0x3799ccbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e3ccc7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f25ccc9, SUC_NE_lexicon_tags_2 },
    { 0x187ccca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x935fcccc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff65ccd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01ccd4, SUC_NE_lexicon_tags_19 },
    { 0xa2e5ccd4, SUC_NE_lexicon_tags_2 },
    { 0x1395ccd4, SUC_NE_lexicon_tags_4 },
    { 0x1f95ccd7, SUC_NE_lexicon_tags_2 },
    { 0xb0b1ccd8, SUC_NE_lexicon_tags_2 },
    { 0x24dfccd9, SUC_NE_lexicon_tags_2 },
    { 0xa717ccda, SUC_NE_lexicon_tags_2 },
    { 0xce17ccd9, SUC_NE_lexicon_tags_2 },
    { 0x9037ccdc, SUC_NE_lexicon_tags_2 },
    { 0xf6f1ccd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9585cce0, SUC_NE_lexicon_tags_2 },
    { 0x61bbcce0, SUC_NE_lexicon_tags_5 },
    { 0x6555cce2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9fcce5, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd1cce8, SUC_NE_lexicon_tags_2 },
    { 0x9b3cce8, SUC_NE_lexicon_tags_2 },
    { 0x675ccea, SUC_NE_lexicon_tags_2 },
    { 0x95f7cce8, SUC_NE_lexicon_tags_2 },
    { 0xe07dccec, SUC_NE_lexicon_tags_2 },
    { 0x6c79ccec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba29ccf1, SUC_NE_lexicon_tags_2 },
    { 0xb4b1ccf2, SUC_NE_lexicon_tags_2 },
    { 0x3553ccf3, SUC_NE_lexicon_tags_3 },
    { 0x9303ccf2, SUC_NE_lexicon_tags_2 },
    { 0x8dddccf5, SUC_NE_lexicon_tags_2 },
    { 0xa94fccf5, SUC_NE_lexicon_tags_2 },
    { 0xb387ccf7, SUC_NE_lexicon_tags_2 },
    { 0xae85ccf8, SUC_NE_lexicon_tags_3 },
    { 0x9593ccf9, SUC_NE_lexicon_tags_2 },
    { 0x7621ccfa, SUC_NE_lexicon_tags_2 },
    { 0xb1dfccf1, SUC_NE_lexicon_tags_24 },
    { 0x1433ccf2, SUC_NE_lexicon_tags_2 },
    { 0x2b43ccfc, SUC_NE_lexicon_tags_2 },
    { 0x79c9ccf9, SUC_NE_lexicon_tags_3 },
    { 0x12d5ccfd, SUC_NE_lexicon_tags_7 },
    { 0x9c4dcd00, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9cd07, SUC_NE_lexicon_tags_2 },
    { 0x7b73cd07, SUC_NE_lexicon_tags_11 },
    { 0x1d83cd09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72abcd0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e45cd12, SUC_NE_lexicon_tags_2 },
    { 0x3909cd13, SUC_NE_lexicon_tags_2 },
    { 0xeef1cd14, SUC_NE_lexicon_tags_2 },
    { 0xbab1cd14, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x2469cd17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a7cd1a, SUC_NE_lexicon_tags_2 },
    { 0x865bcd1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa433cd1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8f5cd20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc113cd23, SUC_NE_lexicon_tags_2 },
    { 0x2523cd23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb93cd26, SUC_NE_lexicon_tags_2 },
    { 0xd325cd27, SUC_NE_lexicon_tags_2 },
    { 0xe2bfcd26, SUC_NE_lexicon_tags_1 },
    { 0x8721cd29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78b7cd2b, SUC_NE_lexicon_tags_2 },
    { 0x2fd5cd2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x419dcd2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1479cd32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12fcd36, SUC_NE_lexicon_tags_2 },
    { 0xa7f7cd37, SUC_NE_lexicon_tags_2 },
    { 0xc079cd38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc763cd3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb415cd3f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x8f51cd41, SUC_NE_lexicon_tags_2 },
    { 0xdbcbcd41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8891cd44, SUC_NE_lexicon_tags_4 },
    { 0xac41cd45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdf9cd47, SUC_NE_lexicon_tags_2 },
    { 0xef7dcd48, SUC_NE_lexicon_tags_7 },
    { 0xbe4dcd47, SUC_NE_lexicon_tags_19 },
    { 0x187dcd4a, SUC_NE_lexicon_tags_2 },
    { 0xe6cbcd4b, SUC_NE_lexicon_tags_2 },
    { 0xd989cd48, SUC_NE_lexicon_tags_2 },
    { 0xc021cd48, SUC_NE_lexicon_tags_2 },
    { 0x3de3cd4c, SUC_NE_lexicon_tags_2 },
    { 0x714dcd4f, SUC_NE_lexicon_tags_7 },
    { 0xed03cd47, SUC_NE_lexicon_tags_2 },
    { 0x13edcd51, SUC_NE_lexicon_tags_14 },
    { 0xc529cd52, SUC_NE_lexicon_tags_2 },
    { 0x6a05cd51, SUC_NE_lexicon_tags_2 },
    { 0x1fdfcd54, SUC_NE_lexicon_tags_2 },
    { 0xc4a3cd4b, SUC_NE_lexicon_tags_17 },
    { 0x14e1cd56, SUC_NE_lexicon_tags_2 },
    { 0x3237cd4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc1cd5a, SUC_NE_lexicon_tags_2 },
    { 0x6cc1cd5b, SUC_NE_lexicon_tags_2 },
    { 0x87fcd5c, SUC_NE_lexicon_tags_2 },
    { 0x9fa7cd5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x60f3cd5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95cd62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x406bcd64, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8bcd67, SUC_NE_lexicon_tags_7 },
    { 0x1afdcd68, SUC_NE_lexicon_tags_2 },
    { 0x4357cd68, SUC_NE_lexicon_tags_2 },
    { 0x8377cd6a, SUC_NE_lexicon_tags_2 },
    { 0x2d23cd6b, SUC_NE_lexicon_tags_2 },
    { 0xaf1cd6b, SUC_NE_lexicon_tags_3 },
    { 0xc219cd68, SUC_NE_lexicon_tags_2 },
    { 0xec7cd6e, SUC_NE_lexicon_tags_2 },
    { 0xecdfcd6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc161cd72, SUC_NE_lexicon_tags_2 },
    { 0x393bcd73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f9cd75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360bcd7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff65cd80, SUC_NE_lexicon_tags_2 },
    { 0xb23fcd80, SUC_NE_lexicon_tags_2 },
    { 0x2e3cd82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c49cd87, SUC_NE_lexicon_tags_2 },
    { 0xdaedcd88, SUC_NE_lexicon_tags_2 },
    { 0x80a3cd89, SUC_NE_lexicon_tags_2 },
    { 0x1efcd87, SUC_NE_lexicon_tags_2 },
    { 0x5cd1cd8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x143bcd8e, SUC_NE_lexicon_tags_2 },
    { 0xff83cd8f, SUC_NE_lexicon_tags_2 },
    { 0x2361cd90, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb7cd95, SUC_NE_lexicon_tags_2 },
    { 0x7f1fcd95, SUC_NE_lexicon_tags_2 },
    { 0x4d5fcd97, SUC_NE_lexicon_tags_2 },
    { 0x8bf3cd98, SUC_NE_lexicon_tags_2 },
    { 0x787cd99, SUC_NE_lexicon_tags_2 },
    { 0xd9cbcd99, SUC_NE_lexicon_tags_2 },
    { 0x2b0fcd9b, SUC_NE_lexicon_tags_2 },
    { 0x1425cd96, SUC_NE_lexicon_tags_2 },
    { 0xdfd5cd9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59bcda3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb3cda8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c63cdad, SUC_NE_lexicon_tags_2 },
    { 0x76b5cdae, SUC_NE_lexicon_tags_2 },
    { 0xc251cdad, SUC_NE_lexicon_tags_3 },
    { 0xd3cbcdb0, SUC_NE_lexicon_tags_7 },
    { 0x2937cdb1, SUC_NE_lexicon_tags_2 },
    { 0x7fb9cdb2, SUC_NE_lexicon_tags_2 },
    { 0x3219cdaf, SUC_NE_lexicon_tags_2 },
    { 0x307fcdae, SUC_NE_lexicon_tags_2 },
    { 0xe941cdb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e97cdb8, SUC_NE_lexicon_tags_2 },
    { 0x7dcfcdb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2dcdbc, SUC_NE_lexicon_tags_2 },
    { 0x58a3cdbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x203cdbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6c9cdc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6389cdc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f5dcdc7, SUC_NE_lexicon_tags_2 },
    { 0x3823cdc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe985cdca, SUC_NE_lexicon_tags_2 },
    { 0xa283cdcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fbcdce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c7cdd8, SUC_NE_lexicon_tags_2 },
    { 0x7287cdd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1689cddb, SUC_NE_lexicon_tags_7 },
    { 0x9dffcddb, SUC_NE_lexicon_tags_2 },
    { 0xede7cddd, SUC_NE_lexicon_tags_2 },
    { 0x5509cddb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6531cde0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cde4, SUC_NE_lexicon_tags_3 },
    { 0x4d9bcde4, SUC_NE_lexicon_tags_2 },
    { 0xe649cde4, SUC_NE_lexicon_tags_7 },
    { 0x1965cde6, SUC_NE_lexicon_tags_2 },
    { 0x9181cde7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9787cded, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ddcdf1, SUC_NE_lexicon_tags_18 },
    { 0xc371cdf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c3cdf5, SUC_NE_lexicon_tags_2 },
    { 0xa177cdf5, SUC_NE_lexicon_tags_2 },
    { 0x983bcdf7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bcdfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96dce01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x571ce03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09fce0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f3ce0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5de5ce11, SUC_NE_lexicon_tags_2 },
    { 0x2733ce11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2131ce15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb173ce17, SUC_NE_lexicon_tags_2 },
    { 0xe281ce18, SUC_NE_lexicon_tags_2 },
    { 0xcfebce19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fce1c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xa73dce1e, SUC_NE_lexicon_tags_2 },
    { 0xd947ce1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a3ce23, SUC_NE_lexicon_tags_2 },
    { 0xa0c3ce24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x295dce26, SUC_NE_lexicon_tags_2 },
    { 0x57bce26, SUC_NE_lexicon_tags_2 },
    { 0xed87ce26, SUC_NE_lexicon_tags_2 },
    { 0xb8fce26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dbce2e, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8039ce31, SUC_NE_lexicon_tags_2 },
    { 0xbe47ce32, SUC_NE_lexicon_tags_329 },
    { 0, NULL },
    { 0, NULL },
    { 0xef01ce35, SUC_NE_lexicon_tags_2 },
    { 0x9f19ce36, SUC_NE_lexicon_tags_2 },
    { 0xa691ce37, SUC_NE_lexicon_tags_2 },
    { 0x8873ce38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20edce3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e63ce41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e15ce43, SUC_NE_lexicon_tags_2 },
    { 0x836bce44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15bce47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d3ce4a, SUC_NE_lexicon_tags_2 },
    { 0x9ab1ce4a, SUC_NE_lexicon_tags_2 },
    { 0x4bdbce4a, SUC_NE_lexicon_tags_2 },
    { 0x202dce4d, SUC_NE_lexicon_tags_2 },
    { 0x25f9ce4e, SUC_NE_lexicon_tags_2 },
    { 0x4757ce4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe481ce51, SUC_NE_lexicon_tags_2 },
    { 0x1449ce51, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x5d8bce54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fbce58, SUC_NE_lexicon_tags_25 },
    { 0x6ed1ce59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff3ce5d, SUC_NE_lexicon_tags_3 },
    { 0x29a5ce5d, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4dce64, SUC_NE_lexicon_tags_2 },
    { 0x23d3ce65, SUC_NE_lexicon_tags_2 },
    { 0x4be7ce66, SUC_NE_lexicon_tags_2 },
    { 0x34fce65, SUC_NE_lexicon_tags_2 },
    { 0x424bce68, SUC_NE_lexicon_tags_2 },
    { 0x86c7ce65, SUC_NE_lexicon_tags_2 },
    { 0xcc03ce66, SUC_NE_lexicon_tags_2 },
    { 0x199bce6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798bce70, SUC_NE_lexicon_tags_2 },
    { 0xbad1ce70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf5ce75, SUC_NE_lexicon_tags_342 },
    { 0xb3ebce76, SUC_NE_lexicon_tags_59 },
    { 0x3ff1ce75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3dce7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa725ce86, SUC_NE_lexicon_tags_2 },
    { 0x5337ce87, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5dce8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bffce92, SUC_NE_lexicon_tags_7 },
    { 0x33a7ce92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edbce96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4949ce9a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aedce9d, SUC_NE_lexicon_tags_2 },
    { 0xb0b9ce9e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6dcea2, SUC_NE_lexicon_tags_2 },
    { 0x61c1cea2, SUC_NE_lexicon_tags_2 },
    { 0x2c3cea4, SUC_NE_lexicon_tags_2 },
    { 0xe48dcea5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdd03cea7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5461cead, SUC_NE_lexicon_tags_2 },
    { 0xd921ceae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e7ceb1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4b3ceb3, SUC_NE_lexicon_tags_2 },
    { 0x5825ceb3, SUC_NE_lexicon_tags_2 },
    { 0x151bceb5, SUC_NE_lexicon_tags_2 },
    { 0x3d71ceb6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x83ceb8, SUC_NE_lexicon_tags_11 },
    { 0x6c3bceb9, SUC_NE_lexicon_tags_2 },
    { 0xef0dceba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab75cebf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523bcec4, SUC_NE_lexicon_tags_2 },
    { 0x496bcec4, SUC_NE_lexicon_tags_30 },
    { 0x2c3dcec5, SUC_NE_lexicon_tags_4 },
    { 0x5fe1cec7, SUC_NE_lexicon_tags_2 },
    { 0x4375cec8, SUC_NE_lexicon_tags_2 },
    { 0xf59dcec9, SUC_NE_lexicon_tags_2 },
    { 0xdb57cec5, SUC_NE_lexicon_tags_2 },
    { 0xf587cec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bcece, SUC_NE_lexicon_tags_1 },
    { 0x5569cece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdda7ced1, SUC_NE_lexicon_tags_2 },
    { 0x6bb3ced2, SUC_NE_lexicon_tags_2 },
    { 0x7fbced1, SUC_NE_lexicon_tags_2 },
    { 0xa351ced3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ca7ced6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffbceda, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x358dcedd, SUC_NE_lexicon_tags_7 },
    { 0xfae3cede, SUC_NE_lexicon_tags_2 },
    { 0x82a1cede, SUC_NE_lexicon_tags_2 },
    { 0xfd9cee0, SUC_NE_lexicon_tags_2 },
    { 0x81e3cee0, SUC_NE_lexicon_tags_102 },
    { 0xa955cee1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba49cee4, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fafcee7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b7ceed, SUC_NE_lexicon_tags_19 },
    { 0x3c3dceee, SUC_NE_lexicon_tags_2 },
    { 0x70f1ceee, SUC_NE_lexicon_tags_2 },
    { 0xcf03cef0, SUC_NE_lexicon_tags_2 },
    { 0x8531cef0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x184dcef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2887cef5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa033cef9, SUC_NE_lexicon_tags_2 },
    { 0xc8c3cefa, SUC_NE_lexicon_tags_2 },
    { 0x7ecfcefb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245fcf00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca3cf02, SUC_NE_lexicon_tags_2 },
    { 0x5c33cf03, SUC_NE_lexicon_tags_2 },
    { 0xdfe5cf02, SUC_NE_lexicon_tags_2 },
    { 0x3b23cf05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2493cf08, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429bcf0f, SUC_NE_lexicon_tags_2 },
    { 0xe2b5cf10, SUC_NE_lexicon_tags_2 },
    { 0x7e6fcf11, SUC_NE_lexicon_tags_2 },
    { 0xbd73cf12, SUC_NE_lexicon_tags_2 },
    { 0x8341cf12, SUC_NE_lexicon_tags_2 },
    { 0x27bcf10, SUC_NE_lexicon_tags_2 },
    { 0x8b03cf15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2fcf18, SUC_NE_lexicon_tags_2 },
    { 0x4403cf19, SUC_NE_lexicon_tags_2 },
    { 0xe6abcf19, SUC_NE_lexicon_tags_306 },
    { 0x406dcf18, SUC_NE_lexicon_tags_2 },
    { 0x45d7cf18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9539cf1e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b87cf21, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c55cf23, SUC_NE_lexicon_tags_2 },
    { 0x6ebbcf23, SUC_NE_lexicon_tags_40 },
    { 0xf3abcf25, SUC_NE_lexicon_tags_17 },
    { 0x8633cf26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4731cf29, SUC_NE_lexicon_tags_1 },
    { 0xd2e1cf29, SUC_NE_lexicon_tags_2 },
    { 0xa133cf2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d3cf2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa677cf31, SUC_NE_lexicon_tags_2 },
    { 0xf40dcf31, SUC_NE_lexicon_tags_2 },
    { 0x267fcf31, SUC_NE_lexicon_tags_2 },
    { 0xa795cf34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8fcf36, SUC_NE_lexicon_tags_2 },
    { 0xc91cf37, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67cf3d, SUC_NE_lexicon_tags_18 },
    { 0xa769cf3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3081cf40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe859cf43, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cdcf4a, SUC_NE_lexicon_tags_4 },
    { 0x8f85cf4b, SUC_NE_lexicon_tags_2 },
    { 0x3021cf4b, SUC_NE_lexicon_tags_2 },
    { 0xedabcf4c, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0x19f3cf4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8fbcf51, SUC_NE_lexicon_tags_2 },
    { 0x4f0bcf52, SUC_NE_lexicon_tags_2 },
    { 0x3c5fcf53, SUC_NE_lexicon_tags_2 },
    { 0xdc23cf51, SUC_NE_lexicon_tags_167 },
    { 0x4a57cf53, SUC_NE_lexicon_tags_2 },
    { 0x5699cf56, SUC_NE_lexicon_tags_5 },
    { 0x88bbcf55, SUC_NE_lexicon_tags_2 },
    { 0xf451cf58, SUC_NE_lexicon_tags_2 },
    { 0xc465cf51, SUC_NE_lexicon_tags_2 },
    { 0x352fcf5a, SUC_NE_lexicon_tags_3 },
    { 0x758dcf5b, SUC_NE_lexicon_tags_4 },
    { 0x2601cf5c, SUC_NE_lexicon_tags_2 },
    { 0xc989cf5d, SUC_NE_lexicon_tags_2 },
    { 0x138fcf5e, SUC_NE_lexicon_tags_2 },
    { 0x6f1dcf5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e3cf67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb75cf6b, SUC_NE_lexicon_tags_2 },
    { 0x4067cf6c, SUC_NE_lexicon_tags_2 },
    { 0x11f5cf6c, SUC_NE_lexicon_tags_2 },
    { 0xd981cf6d, SUC_NE_lexicon_tags_2 },
    { 0x7575cf6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed4bcf71, SUC_NE_lexicon_tags_2 },
    { 0x37c3cf71, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xb913cf74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8bcf77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678bcf7c, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x1f1cf7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbbcf82, SUC_NE_lexicon_tags_357 },
    { 0x57cdcf83, SUC_NE_lexicon_tags_2 },
    { 0xbc69cf84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba11cf89, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8dcf8d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe431cf91, SUC_NE_lexicon_tags_2 },
    { 0x747cf92, SUC_NE_lexicon_tags_2 },
    { 0x63a7cf93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bafcf95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeabcf9b, SUC_NE_lexicon_tags_2 },
    { 0x9153cf9c, SUC_NE_lexicon_tags_2 },
    { 0xeae7cf9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13bcfa2, SUC_NE_lexicon_tags_96 },
    { 0, NULL },
    { 0xa87cfa4, SUC_NE_lexicon_tags_2 },
    { 0x5c5cfa5, SUC_NE_lexicon_tags_2 },
    { 0x7ff3cfa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5bcfa9, SUC_NE_lexicon_tags_2 },
    { 0x80cfcfaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf57cfac, SUC_NE_lexicon_tags_2 },
    { 0x328fcfad, SUC_NE_lexicon_tags_2 },
    { 0x256dcfac, SUC_NE_lexicon_tags_2 },
    { 0x449fcfaf, SUC_NE_lexicon_tags_2 },
    { 0x2483cfac, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x6191cfb2, SUC_NE_lexicon_tags_2 },
    { 0xb145cfb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe987cfb6, SUC_NE_lexicon_tags_2 },
    { 0x52e5cfb7, SUC_NE_lexicon_tags_2 },
    { 0xfc01cfb7, SUC_NE_lexicon_tags_107 },
    { 0, NULL },
    { 0x4b3dcfba, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0x98efcfbc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x715fcfbe, SUC_NE_lexicon_tags_7 },
    { 0x5863cfbe, SUC_NE_lexicon_tags_7 },
    { 0x659fcfc0, SUC_NE_lexicon_tags_2 },
    { 0xbd95cfc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b1cfc5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc31cfc7, SUC_NE_lexicon_tags_2 },
    { 0x9d13cfc7, SUC_NE_lexicon_tags_2 },
    { 0xe823cfc8, SUC_NE_lexicon_tags_2 },
    { 0xb41cfca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3d7cfcc, SUC_NE_lexicon_tags_4 },
    { 0x3c6fcfcc, SUC_NE_lexicon_tags_2 },
    { 0xa9edcfce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20c5cfd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dbcfd4, SUC_NE_lexicon_tags_2 },
    { 0xb15cfd5, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3cfd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d75cfdc, SUC_NE_lexicon_tags_2 },
    { 0x8fbdcfdd, SUC_NE_lexicon_tags_2 },
    { 0xc097cfde, SUC_NE_lexicon_tags_2 },
    { 0xe5e9cfdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c53cfe2, SUC_NE_lexicon_tags_2 },
    { 0x9dd9cfe3, SUC_NE_lexicon_tags_2 },
    { 0x881cfe2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x43c3cfe6, SUC_NE_lexicon_tags_2 },
    { 0x8af9cfe6, SUC_NE_lexicon_tags_2 },
    { 0x1ae5cfe6, SUC_NE_lexicon_tags_4 },
    { 0xfc97cfe6, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6fcfed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdadcfef, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6503cff2, SUC_NE_lexicon_tags_2 },
    { 0x6645cff3, SUC_NE_lexicon_tags_2 },
    { 0xba3dcff4, SUC_NE_lexicon_tags_18 },
    { 0x2f6fcff5, SUC_NE_lexicon_tags_2 },
    { 0xc3f5cff6, SUC_NE_lexicon_tags_2 },
    { 0xcb9fcff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e5cffa, SUC_NE_lexicon_tags_2 },
    { 0xa94bcffb, SUC_NE_lexicon_tags_2 },
    { 0x4e71cffa, SUC_NE_lexicon_tags_2 },
    { 0xd44fcffd, SUC_NE_lexicon_tags_2 },
    { 0x30b7cffd, SUC_NE_lexicon_tags_2 },
    { 0xf241cfff, SUC_NE_lexicon_tags_2 },
    { 0xefd9cffb, SUC_NE_lexicon_tags_2 },
    { 0x48f1cfff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec25d004, SUC_NE_lexicon_tags_2 },
    { 0xd027d004, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9149d008, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc27d00a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f7d00d, SUC_NE_lexicon_tags_2 },
    { 0xcd9dd00e, SUC_NE_lexicon_tags_7 },
    { 0x7887d00d, SUC_NE_lexicon_tags_2 },
    { 0x4563d010, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79b3d012, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34dbd015, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d6dd017, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cdd01c, SUC_NE_lexicon_tags_2 },
    { 0x20c5d01d, SUC_NE_lexicon_tags_2 },
    { 0xa03bd01d, SUC_NE_lexicon_tags_2 },
    { 0x2739d01c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c41d022, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x62d5d024, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8825d026, SUC_NE_lexicon_tags_5 },
    { 0xf503d027, SUC_NE_lexicon_tags_2 },
    { 0x12d9d028, SUC_NE_lexicon_tags_2 },
    { 0x1581d026, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb37d02d, SUC_NE_lexicon_tags_2 },
    { 0x7d6fd02d, SUC_NE_lexicon_tags_2 },
    { 0x107fd02f, SUC_NE_lexicon_tags_2 },
    { 0x9215d030, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d3d03a, SUC_NE_lexicon_tags_2 },
    { 0x153d03b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55fd03d, SUC_NE_lexicon_tags_7 },
    { 0xeca3d03e, SUC_NE_lexicon_tags_2 },
    { 0x891bd03f, SUC_NE_lexicon_tags_2 },
    { 0x2475d040, SUC_NE_lexicon_tags_2 },
    { 0xcd2bd040, SUC_NE_lexicon_tags_2 },
    { 0xbaf3d03e, SUC_NE_lexicon_tags_2 },
    { 0xfea1d043, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5dd048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b1d050, SUC_NE_lexicon_tags_2 },
    { 0xba61d051, SUC_NE_lexicon_tags_2 },
    { 0xb83d052, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3bd057, SUC_NE_lexicon_tags_4 },
    { 0x9b2fd058, SUC_NE_lexicon_tags_2 },
    { 0xea79d058, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x245dd05c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf469d05f, SUC_NE_lexicon_tags_2 },
    { 0x2badd060, SUC_NE_lexicon_tags_7 },
    { 0x3533d060, SUC_NE_lexicon_tags_2 },
    { 0xe765d060, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3dd065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcddd067, SUC_NE_lexicon_tags_2 },
    { 0xf89bd068, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa09d06a, SUC_NE_lexicon_tags_2 },
    { 0xd2b7d06b, SUC_NE_lexicon_tags_2 },
    { 0x9e61d06b, SUC_NE_lexicon_tags_2 },
    { 0xebbbd06d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1135d070, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d3d076, SUC_NE_lexicon_tags_2 },
    { 0x7de1d077, SUC_NE_lexicon_tags_2 },
    { 0x897fd078, SUC_NE_lexicon_tags_142 },
    { 0, NULL },
    { 0x217bd07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78f5d07c, SUC_NE_lexicon_tags_2 },
    { 0xf221d07c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e39d07f, SUC_NE_lexicon_tags_2 },
    { 0xddcfd080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x17ebd082, SUC_NE_lexicon_tags_3 },
    { 0x9435d082, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe08bd085, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e05d087, SUC_NE_lexicon_tags_2 },
    { 0x6193d088, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d08b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d2fd08d, SUC_NE_lexicon_tags_2 },
    { 0xc18fd08d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2493d092, SUC_NE_lexicon_tags_2 },
    { 0xed9dd093, SUC_NE_lexicon_tags_2 },
    { 0xb8efd094, SUC_NE_lexicon_tags_2 },
    { 0x2071d093, SUC_NE_lexicon_tags_2 },
    { 0x7aebd096, SUC_NE_lexicon_tags_2 },
    { 0x42c5d097, SUC_NE_lexicon_tags_1 },
    { 0x7cbd098, SUC_NE_lexicon_tags_2 },
    { 0xdc6dd097, SUC_NE_lexicon_tags_14 },
    { 0x6d9bd097, SUC_NE_lexicon_tags_2 },
    { 0xfd91d09b, SUC_NE_lexicon_tags_2 },
    { 0x78abd096, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1bd0a0, SUC_NE_lexicon_tags_8 },
    { 0xd83bd0a1, SUC_NE_lexicon_tags_4 },
    { 0xc64dd0a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6dd0a7, SUC_NE_lexicon_tags_2 },
    { 0x3f3d0a7, SUC_NE_lexicon_tags_75 },
    { 0xa523d0a9, SUC_NE_lexicon_tags_54 },
    { 0xb5d0a9, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf435d0af, SUC_NE_lexicon_tags_2 },
    { 0x613bd0b0, SUC_NE_lexicon_tags_2 },
    { 0x719d0af, SUC_NE_lexicon_tags_2 },
    { 0x84add0b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc475d0b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc183d0b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb9d0bb, SUC_NE_lexicon_tags_2 },
    { 0x6271d0bc, SUC_NE_lexicon_tags_3 },
    { 0x727dd0bd, SUC_NE_lexicon_tags_2 },
    { 0x6b0bd0be, SUC_NE_lexicon_tags_2 },
    { 0x6ffd0bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1bd0c2, SUC_NE_lexicon_tags_2 },
    { 0xd31bd0c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ddd0c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a1d0c9, SUC_NE_lexicon_tags_2 },
    { 0x6d8dd0ca, SUC_NE_lexicon_tags_2 },
    { 0xdd51d0c9, SUC_NE_lexicon_tags_2 },
    { 0xf301d0cc, SUC_NE_lexicon_tags_19 },
    { 0x24f3d0ca, SUC_NE_lexicon_tags_2 },
    { 0x117d0ce, SUC_NE_lexicon_tags_2 },
    { 0x6021d0cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2a9d0d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x730dd0d3, SUC_NE_lexicon_tags_2 },
    { 0x38cfd0d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a9d0d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xafbbd0d9, SUC_NE_lexicon_tags_2 },
    { 0x7c7dd0da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb587d0e2, SUC_NE_lexicon_tags_2 },
    { 0xeeb5d0e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdecdd0e5, SUC_NE_lexicon_tags_2 },
    { 0x93b9d0e6, SUC_NE_lexicon_tags_52 },
    { 0x3b95d0e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727d0eb, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb609d0f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce43d102, SUC_NE_lexicon_tags_2 },
    { 0x7b99d103, SUC_NE_lexicon_tags_2 },
    { 0xcec1d104, SUC_NE_lexicon_tags_2 },
    { 0xb3d1d102, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x299fd107, SUC_NE_lexicon_tags_2 },
    { 0xaaf1d108, SUC_NE_lexicon_tags_3 },
    { 0x6e89d109, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x613d10c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5abd10e, SUC_NE_lexicon_tags_7 },
    { 0x724dd10f, SUC_NE_lexicon_tags_2 },
    { 0xac8dd10f, SUC_NE_lexicon_tags_326 },
    { 0x9a4bd111, SUC_NE_lexicon_tags_2 },
    { 0x8a6dd112, SUC_NE_lexicon_tags_0 },
    { 0x35e5d111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9941d119, SUC_NE_lexicon_tags_2 },
    { 0x23cfd11a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca5d11d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd183d124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fd127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f09d129, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c5d12e, SUC_NE_lexicon_tags_2 },
    { 0xe7b1d12f, SUC_NE_lexicon_tags_2 },
    { 0xd677d12e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd5d134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7d13a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43f3d13c, SUC_NE_lexicon_tags_2 },
    { 0x31ddd13d, SUC_NE_lexicon_tags_2 },
    { 0xdacfd13d, SUC_NE_lexicon_tags_4 },
    { 0x9875d13f, SUC_NE_lexicon_tags_2 },
    { 0x8e53d13f, SUC_NE_lexicon_tags_2 },
    { 0x70e3d141, SUC_NE_lexicon_tags_2 },
    { 0x7e9fd142, SUC_NE_lexicon_tags_2 },
    { 0x2801d141, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b3d146, SUC_NE_lexicon_tags_2 },
    { 0xb195d146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fefd14a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1bd14c, SUC_NE_lexicon_tags_2 },
    { 0xd3d5d14c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2bd14f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc5d155, SUC_NE_lexicon_tags_2 },
    { 0x5497d156, SUC_NE_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fdd15a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae1d15d, SUC_NE_lexicon_tags_2 },
    { 0x6d41d15e, SUC_NE_lexicon_tags_2 },
    { 0x7139d15e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb35d161, SUC_NE_lexicon_tags_2 },
    { 0x31c7d162, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3441d164, SUC_NE_lexicon_tags_2 },
    { 0x68d9d164, SUC_NE_lexicon_tags_2 },
    { 0xb1fbd166, SUC_NE_lexicon_tags_2 },
    { 0x4423d167, SUC_NE_lexicon_tags_2 },
    { 0x9361d166, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112dd16f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12dd174, SUC_NE_lexicon_tags_2 },
    { 0xbbb1d175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3435d17a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c71d17d, SUC_NE_lexicon_tags_2 },
    { 0xe777d17e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x77e9d180, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4dd183, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x347bd185, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafafd18a, SUC_NE_lexicon_tags_40 },
    { 0xacbfd18a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9051d18d, SUC_NE_lexicon_tags_2 },
    { 0x2707d18e, SUC_NE_lexicon_tags_2 },
    { 0x51ddd18f, SUC_NE_lexicon_tags_2 },
    { 0x197dd18e, SUC_NE_lexicon_tags_440 },
    { 0x79d18e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c0bd193, SUC_NE_lexicon_tags_2 },
    { 0x3319d193, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb9dd196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x861fd199, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76d9d19b, SUC_NE_lexicon_tags_2 },
    { 0x3029d19b, SUC_NE_lexicon_tags_2 },
    { 0x7e69d19d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3dd1a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4bd1ab, SUC_NE_lexicon_tags_2 },
    { 0x7b41d1ab, SUC_NE_lexicon_tags_2 },
    { 0x4a5bd1ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696dd1b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d13d1b5, SUC_NE_lexicon_tags_14 },
    { 0xcc59d1b5, SUC_NE_lexicon_tags_2 },
    { 0x8171d1b7, SUC_NE_lexicon_tags_2 },
    { 0x3403d1b6, SUC_NE_lexicon_tags_2 },
    { 0x9afdd1b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2345d1bb, SUC_NE_lexicon_tags_97 },
    { 0xc6a3d1bb, SUC_NE_lexicon_tags_2 },
    { 0x6fa9d1bd, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xa567d1bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32bd1c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77bd1c9, SUC_NE_lexicon_tags_2 },
    { 0x2c91d1ca, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c1d1cd, SUC_NE_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc33d1d1, SUC_NE_lexicon_tags_2 },
    { 0xf4b7d1d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebb5d1d4, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdd1d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8557d1da, SUC_NE_lexicon_tags_2 },
    { 0x603fd1da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6923d1de, SUC_NE_lexicon_tags_2 },
    { 0x9ef1d1df, SUC_NE_lexicon_tags_2 },
    { 0xf2d5d1de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9ddd1e2, SUC_NE_lexicon_tags_2 },
    { 0x14b5d1e3, SUC_NE_lexicon_tags_2 },
    { 0x64e1d1e4, SUC_NE_lexicon_tags_2 },
    { 0x2189d1e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6bd1e7, SUC_NE_lexicon_tags_2 },
    { 0xb943d1e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33a1d1ea, SUC_NE_lexicon_tags_24 },
    { 0x7013d1ea, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0xbf01d1ed, SUC_NE_lexicon_tags_49 },
    { 0xdbc5d1ee, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf391d1f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e9d1f5, SUC_NE_lexicon_tags_2 },
    { 0x97f9d1f6, SUC_NE_lexicon_tags_17 },
    { 0x6f93d1f7, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc911d1ff, SUC_NE_lexicon_tags_2 },
    { 0x971dd1ff, SUC_NE_lexicon_tags_4 },
    { 0x4467d1ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4dd204, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc3d206, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb493d20d, SUC_NE_lexicon_tags_2 },
    { 0x34e3d20e, SUC_NE_lexicon_tags_2 },
    { 0x49ffd20f, SUC_NE_lexicon_tags_3 },
    { 0xcd03d210, SUC_NE_lexicon_tags_2 },
    { 0xf357d20f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd967d213, SUC_NE_lexicon_tags_2 },
    { 0xf0f3d214, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebd217, SUC_NE_lexicon_tags_2 },
    { 0x818bd218, SUC_NE_lexicon_tags_7 },
    { 0x1f97d217, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf19d21b, SUC_NE_lexicon_tags_2 },
    { 0x792dd21c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x32e5d21e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5c3d220, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x48fdd222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de1d226, SUC_NE_lexicon_tags_2 },
    { 0x814fd227, SUC_NE_lexicon_tags_3 },
    { 0x66b7d228, SUC_NE_lexicon_tags_2 },
    { 0x485dd229, SUC_NE_lexicon_tags_2 },
    { 0xbe23d226, SUC_NE_lexicon_tags_1 },
    { 0xc8e7d22b, SUC_NE_lexicon_tags_2 },
    { 0xdedbd22c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x631bd22e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30bd233, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86dfd236, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8817d238, SUC_NE_lexicon_tags_2 },
    { 0x8c21d239, SUC_NE_lexicon_tags_4 },
    { 0xc8cdd23a, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0xd97fd23c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3dd23f, SUC_NE_lexicon_tags_4 },
    { 0x3217d240, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23bd243, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e1d24a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ddd24e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x48d7d250, SUC_NE_lexicon_tags_2 },
    { 0x5f29d251, SUC_NE_lexicon_tags_2 },
    { 0x46fbd250, SUC_NE_lexicon_tags_24 },
    { 0xda1d251, SUC_NE_lexicon_tags_2 },
    { 0x509fd254, SUC_NE_lexicon_tags_2 },
    { 0xd375d253, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc137d259, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c81d261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c93d264, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x9823d266, SUC_NE_lexicon_tags_2 },
    { 0x7d1fd267, SUC_NE_lexicon_tags_2 },
    { 0x5849d268, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f91d26b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc0dd26d, SUC_NE_lexicon_tags_2 },
    { 0x39f5d26e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7d271, SUC_NE_lexicon_tags_2 },
    { 0x889bd272, SUC_NE_lexicon_tags_2 },
    { 0x887bd273, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4609d275, SUC_NE_lexicon_tags_2 },
    { 0x582dd276, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243bd27b, SUC_NE_lexicon_tags_2 },
    { 0x3c75d27c, SUC_NE_lexicon_tags_2 },
    { 0xb9f7d27b, SUC_NE_lexicon_tags_3 },
    { 0x666fd27e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x525d280, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6887d284, SUC_NE_lexicon_tags_2 },
    { 0x7f49d285, SUC_NE_lexicon_tags_2 },
    { 0xf123d286, SUC_NE_lexicon_tags_2 },
    { 0x83dbd285, SUC_NE_lexicon_tags_2 },
    { 0xce81d288, SUC_NE_lexicon_tags_2 },
    { 0x9ba3d289, SUC_NE_lexicon_tags_2 },
    { 0x6075d28a, SUC_NE_lexicon_tags_2 },
    { 0xa68dd28b, SUC_NE_lexicon_tags_2 },
    { 0xcddfd28b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x791d28f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7dd292, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cb3d294, SUC_NE_lexicon_tags_2 },
    { 0x1ed3d294, SUC_NE_lexicon_tags_2 },
    { 0x285fd296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8dd29a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d7d29e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf535d2a0, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8673d2a4, SUC_NE_lexicon_tags_2 },
    { 0x3e01d2a5, SUC_NE_lexicon_tags_2 },
    { 0xce63d2a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed1bd2a8, SUC_NE_lexicon_tags_2 },
    { 0x5411d2a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea7d2af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a33d2b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fbd2b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a5d2bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d5fd2bd, SUC_NE_lexicon_tags_1 },
    { 0x21d3d2be, SUC_NE_lexicon_tags_5 },
    { 0x2e4fd2bd, SUC_NE_lexicon_tags_2 },
    { 0x8d7d2c0, SUC_NE_lexicon_tags_2 },
    { 0xdb65d2c1, SUC_NE_lexicon_tags_2 },
    { 0x328fd2c1, SUC_NE_lexicon_tags_2 },
    { 0x524fd2c3, SUC_NE_lexicon_tags_2 },
    { 0x98d5d2c4, SUC_NE_lexicon_tags_1 },
    { 0x5073d2bd, SUC_NE_lexicon_tags_2 },
    { 0x3c9d2c4, SUC_NE_lexicon_tags_2 },
    { 0x88cfd2c7, SUC_NE_lexicon_tags_2 },
    { 0x1037d2c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x831dd2ca, SUC_NE_lexicon_tags_0 },
    { 0x4acfd2cb, SUC_NE_lexicon_tags_19 },
    { 0x2415d2cc, SUC_NE_lexicon_tags_2 },
    { 0xf763d2cd, SUC_NE_lexicon_tags_2 },
    { 0x5e87d2ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x421bd2d0, SUC_NE_lexicon_tags_2 },
    { 0x89f7d2d1, SUC_NE_lexicon_tags_2 },
    { 0xd899d2d2, SUC_NE_lexicon_tags_2 },
    { 0x6977d2d3, SUC_NE_lexicon_tags_2 },
    { 0xe889d2d4, SUC_NE_lexicon_tags_2 },
    { 0x894dd2d5, SUC_NE_lexicon_tags_2 },
    { 0x29edd2d4, SUC_NE_lexicon_tags_2 },
    { 0x1e27d2d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f5d2da, SUC_NE_lexicon_tags_2 },
    { 0x3dfdd2db, SUC_NE_lexicon_tags_2 },
    { 0x7f8dd2dc, SUC_NE_lexicon_tags_2 },
    { 0xc3add2dd, SUC_NE_lexicon_tags_2 },
    { 0x77dd2dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c29d2e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d01d2e3, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x827fd2e6, SUC_NE_lexicon_tags_2 },
    { 0x8a65d2e7, SUC_NE_lexicon_tags_3 },
    { 0xf03dd2e7, SUC_NE_lexicon_tags_2 },
    { 0xa2dbd2e8, SUC_NE_lexicon_tags_0 },
    { 0xef1dd2e9, SUC_NE_lexicon_tags_2 },
    { 0x5735d2eb, SUC_NE_lexicon_tags_2 },
    { 0xb335d2ea, SUC_NE_lexicon_tags_0 },
    { 0x9851d2e7, SUC_NE_lexicon_tags_2 },
    { 0x4985d2ee, SUC_NE_lexicon_tags_2 },
    { 0x8a47d2ef, SUC_NE_lexicon_tags_2 },
    { 0xe7c7d2e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8be7d2f2, SUC_NE_lexicon_tags_2 },
    { 0x94d5d2f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dbd2f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c31d2f9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8dd300, SUC_NE_lexicon_tags_2 },
    { 0x57f3d301, SUC_NE_lexicon_tags_2 },
    { 0x60ffd300, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37d30a, SUC_NE_lexicon_tags_2 },
    { 0xcc81d30a, SUC_NE_lexicon_tags_2 },
    { 0x4587d30a, SUC_NE_lexicon_tags_2 },
    { 0x7ddd30b, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a3d310, SUC_NE_lexicon_tags_2 },
    { 0x619dd310, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xc7afd313, SUC_NE_lexicon_tags_2 },
    { 0x5dfdd313, SUC_NE_lexicon_tags_171 },
    { 0x933fd314, SUC_NE_lexicon_tags_25 },
    { 0x377dd313, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459bd31b, SUC_NE_lexicon_tags_2 },
    { 0x2535d31b, SUC_NE_lexicon_tags_2 },
    { 0x5f91d31b, SUC_NE_lexicon_tags_2 },
    { 0x8597d31e, SUC_NE_lexicon_tags_2 },
    { 0xc9d5d31b, SUC_NE_lexicon_tags_2 },
    { 0xced1d320, SUC_NE_lexicon_tags_2 },
    { 0xaa41d320, SUC_NE_lexicon_tags_2 },
    { 0xa9fd322, SUC_NE_lexicon_tags_2 },
    { 0x50f5d31b, SUC_NE_lexicon_tags_2 },
    { 0x8ff9d324, SUC_NE_lexicon_tags_2 },
    { 0x3bcbd323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d7d32c, SUC_NE_lexicon_tags_2 },
    { 0xb309d32d, SUC_NE_lexicon_tags_2 },
    { 0x6757d32d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x606dd330, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ac1d332, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe9d336, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1b3d338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0fd33d, SUC_NE_lexicon_tags_2 },
    { 0x821dd33e, SUC_NE_lexicon_tags_2 },
    { 0x69cfd33e, SUC_NE_lexicon_tags_18 },
    { 0x4f49d33d, SUC_NE_lexicon_tags_2 },
    { 0xdc65d340, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1767d345, SUC_NE_lexicon_tags_18 },
    { 0x97fd345, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230fd34b, SUC_NE_lexicon_tags_10 },
    { 0x4ad1d34b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c39d350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235d354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d1d357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d63d365, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd19d369, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98fd36c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b7d370, SUC_NE_lexicon_tags_2 },
    { 0xbc4fd371, SUC_NE_lexicon_tags_2 },
    { 0xdb37d370, SUC_NE_lexicon_tags_19 },
    { 0xf69bd373, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea5d379, SUC_NE_lexicon_tags_2 },
    { 0x16c7d37a, SUC_NE_lexicon_tags_2 },
    { 0xdd27d37b, SUC_NE_lexicon_tags_2 },
    { 0xdc6fd37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x537d37e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fdd382, SUC_NE_lexicon_tags_2 },
    { 0x4b57d383, SUC_NE_lexicon_tags_2 },
    { 0xd09dd382, SUC_NE_lexicon_tags_2 },
    { 0x5bc7d385, SUC_NE_lexicon_tags_2 },
    { 0xdb85d383, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0xb55d388, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f9fd38a, SUC_NE_lexicon_tags_2 },
    { 0x382fd38b, SUC_NE_lexicon_tags_2 },
    { 0xad9bd38c, SUC_NE_lexicon_tags_2 },
    { 0xb2fdd38d, SUC_NE_lexicon_tags_2 },
    { 0x5a81d38e, SUC_NE_lexicon_tags_2 },
    { 0xac83d38f, SUC_NE_lexicon_tags_2 },
    { 0x4a8fd38f, SUC_NE_lexicon_tags_58 },
    { 0x34bdd391, SUC_NE_lexicon_tags_2 },
    { 0xab15d38c, SUC_NE_lexicon_tags_2 },
    { 0x6767d38f, SUC_NE_lexicon_tags_5 },
    { 0x73d1d394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7109d398, SUC_NE_lexicon_tags_2 },
    { 0x5c3dd399, SUC_NE_lexicon_tags_2 },
    { 0xf021d39a, SUC_NE_lexicon_tags_2 },
    { 0x17dd398, SUC_NE_lexicon_tags_2 },
    { 0x51d7d39c, SUC_NE_lexicon_tags_2 },
    { 0x9cf5d398, SUC_NE_lexicon_tags_2 },
    { 0xc6a3d39a, SUC_NE_lexicon_tags_2 },
    { 0xfe8dd39f, SUC_NE_lexicon_tags_2 },
    { 0xfc31d3a0, SUC_NE_lexicon_tags_7 },
    { 0x1153d39d, SUC_NE_lexicon_tags_2 },
    { 0x89e1d3a2, SUC_NE_lexicon_tags_2 },
    { 0xdd9fd39b, SUC_NE_lexicon_tags_2 },
    { 0x9a17d3a4, SUC_NE_lexicon_tags_2 },
    { 0x8643d3a5, SUC_NE_lexicon_tags_5 },
    { 0x31d3d3a3, SUC_NE_lexicon_tags_2 },
    { 0x4599d3a1, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bf5d3a9, SUC_NE_lexicon_tags_2 },
    { 0x2e93d3a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe135d3b1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaffd3b3, SUC_NE_lexicon_tags_2 },
    { 0x62b3d3b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfdd3b6, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe691d3b9, SUC_NE_lexicon_tags_2 },
    { 0xfdffd3ba, SUC_NE_lexicon_tags_2 },
    { 0x3585d3bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3449d3bf, SUC_NE_lexicon_tags_2 },
    { 0x826fd3bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fd3c5, SUC_NE_lexicon_tags_2 },
    { 0x686fd3c6, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x694dd3c8, SUC_NE_lexicon_tags_2 },
    { 0x9c37d3c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb8fd3cb, SUC_NE_lexicon_tags_3 },
    { 0xb4ffd3cc, SUC_NE_lexicon_tags_2 },
    { 0xdb31d3cd, SUC_NE_lexicon_tags_2 },
    { 0xc6e5d3cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b1d3d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c3d3d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517d3df, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee39d3e7, SUC_NE_lexicon_tags_2 },
    { 0x4f13d3e8, SUC_NE_lexicon_tags_2 },
    { 0x4d23d3e9, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x8db1d3eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaee3d3ed, SUC_NE_lexicon_tags_2 },
    { 0x7595d3ee, SUC_NE_lexicon_tags_2 },
    { 0x3591d3ee, SUC_NE_lexicon_tags_12 },
    { 0x4c03d3f0, SUC_NE_lexicon_tags_2 },
    { 0x1891d3ef, SUC_NE_lexicon_tags_19 },
    { 0x953d3f1, SUC_NE_lexicon_tags_2 },
    { 0xa64dd3f3, SUC_NE_lexicon_tags_2 },
    { 0x5d13d3f4, SUC_NE_lexicon_tags_2 },
    { 0xd547d3ee, SUC_NE_lexicon_tags_2 },
    { 0x8ab7d3f6, SUC_NE_lexicon_tags_2 },
    { 0xb753d3f7, SUC_NE_lexicon_tags_2 },
    { 0x20cfd3f0, SUC_NE_lexicon_tags_2 },
    { 0xc725d3f8, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7add3fd, SUC_NE_lexicon_tags_2 },
    { 0x55fbd3fe, SUC_NE_lexicon_tags_2 },
    { 0x858dd3fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6dd402, SUC_NE_lexicon_tags_2 },
    { 0x9acfd403, SUC_NE_lexicon_tags_2 },
    { 0x9233d404, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x57d1d406, SUC_NE_lexicon_tags_2 },
    { 0xda43d407, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6033d40b, SUC_NE_lexicon_tags_59 },
    { 0x874bd40c, SUC_NE_lexicon_tags_2 },
    { 0xdb15d40c, SUC_NE_lexicon_tags_2 },
    { 0xbbdfd40e, SUC_NE_lexicon_tags_7 },
    { 0xee03d40b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b75d411, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d8fd413, SUC_NE_lexicon_tags_2 },
    { 0xcadd414, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d5d416, SUC_NE_lexicon_tags_2 },
    { 0xd1d9d416, SUC_NE_lexicon_tags_2 },
    { 0x863dd418, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd61d420, SUC_NE_lexicon_tags_2 },
    { 0x26d5d421, SUC_NE_lexicon_tags_50 },
    { 0x4369d421, SUC_NE_lexicon_tags_2 },
    { 0x68c1d423, SUC_NE_lexicon_tags_2 },
    { 0xdc1fd423, SUC_NE_lexicon_tags_4 },
    { 0x23e9d425, SUC_NE_lexicon_tags_2 },
    { 0x5e67d420, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79cdd428, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb399d42a, SUC_NE_lexicon_tags_2 },
    { 0xe9d7d42a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d97d42d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c81d430, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd91d432, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1867d434, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa585d438, SUC_NE_lexicon_tags_2 },
    { 0x7f07d439, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3d445, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x187bd447, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3903d44b, SUC_NE_lexicon_tags_2 },
    { 0x3b9dd44c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6fd44f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8dd457, SUC_NE_lexicon_tags_2 },
    { 0x65edd458, SUC_NE_lexicon_tags_2 },
    { 0xb467d458, SUC_NE_lexicon_tags_12 },
    { 0x70ffd45a, SUC_NE_lexicon_tags_7 },
    { 0x2bbbd45b, SUC_NE_lexicon_tags_2 },
    { 0xd433d45b, SUC_NE_lexicon_tags_2 },
    { 0xe363d45c, SUC_NE_lexicon_tags_2 },
    { 0x5ff3d45e, SUC_NE_lexicon_tags_2 },
    { 0x5085d45b, SUC_NE_lexicon_tags_2 },
    { 0x29ddd458, SUC_NE_lexicon_tags_7 },
    { 0x7c07d45d, SUC_NE_lexicon_tags_2 },
    { 0xcb31d462, SUC_NE_lexicon_tags_2 },
    { 0x3f1fd45b, SUC_NE_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc397d466, SUC_NE_lexicon_tags_2 },
    { 0xcedbd466, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1bd46c, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0x2865d46e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb69bd470, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1865d473, SUC_NE_lexicon_tags_2 },
    { 0x81a3d474, SUC_NE_lexicon_tags_2 },
    { 0x8065d475, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1f5d477, SUC_NE_lexicon_tags_7 },
    { 0x5d41d478, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96fd47c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b7d486, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936fd48b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edd491, SUC_NE_lexicon_tags_2 },
    { 0xe4e9d492, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ca7d494, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc649d49f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x1b03d4a1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x605dd4a3, SUC_NE_lexicon_tags_7 },
    { 0x7b7fd4a3, SUC_NE_lexicon_tags_30 },
    { 0x15ebd4a5, SUC_NE_lexicon_tags_2 },
    { 0x187fd4a3, SUC_NE_lexicon_tags_4 },
    { 0x9519d4a3, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4761d4aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bd4b2, SUC_NE_lexicon_tags_2 },
    { 0xaaa7d4b3, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fd4b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb20bd4bb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3f5bd4bd, SUC_NE_lexicon_tags_2 },
    { 0xe045d4be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde73d4c0, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a5d4c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3905d4c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcebd4cb, SUC_NE_lexicon_tags_387 },
    { 0x5501d4cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc155d4ce, SUC_NE_lexicon_tags_3 },
    { 0x3237d4cf, SUC_NE_lexicon_tags_2 },
    { 0xc10dd4d0, SUC_NE_lexicon_tags_30 },
    { 0x275bd4d1, SUC_NE_lexicon_tags_4 },
    { 0x96cdd4d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e1d4d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd46fd4d8, SUC_NE_lexicon_tags_2 },
    { 0xe69dd4d9, SUC_NE_lexicon_tags_2 },
    { 0x6795d4da, SUC_NE_lexicon_tags_2 },
    { 0xfd89d4da, SUC_NE_lexicon_tags_363 },
    { 0, NULL },
    { 0x15d5d4dd, SUC_NE_lexicon_tags_2 },
    { 0x5e8dd4de, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x447dd4e0, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21add4e4, SUC_NE_lexicon_tags_2 },
    { 0x4065d4e5, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c43d4e8, SUC_NE_lexicon_tags_4 },
    { 0x39dbd4e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80dd4eb, SUC_NE_lexicon_tags_2 },
    { 0x7b71d4ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x436fd4ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd849d4f3, SUC_NE_lexicon_tags_2 },
    { 0x270dd4f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd00bd4f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ebd4fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x589bd4fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc857d502, SUC_NE_lexicon_tags_2 },
    { 0x80efd503, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92bd506, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b49d508, SUC_NE_lexicon_tags_2 },
    { 0x5997d508, SUC_NE_lexicon_tags_2 },
    { 0xebfbd50a, SUC_NE_lexicon_tags_2 },
    { 0xeaefd50b, SUC_NE_lexicon_tags_2 },
    { 0xb2cfd50c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8e5d50e, SUC_NE_lexicon_tags_2 },
    { 0x296bd50e, SUC_NE_lexicon_tags_2 },
    { 0x33d3d510, SUC_NE_lexicon_tags_0 },
    { 0xfb39d50e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd1d514, SUC_NE_lexicon_tags_2 },
    { 0xb559d514, SUC_NE_lexicon_tags_1 },
    { 0xe231d516, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1bd51a, SUC_NE_lexicon_tags_2 },
    { 0xf707d51a, SUC_NE_lexicon_tags_2 },
    { 0x691dd51c, SUC_NE_lexicon_tags_2 },
    { 0xa3b9d51d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8149d51f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41d526, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4527d52a, SUC_NE_lexicon_tags_2 },
    { 0xeb5dd52b, SUC_NE_lexicon_tags_184 },
    { 0x5821d52c, SUC_NE_lexicon_tags_2 },
    { 0x77a7d52c, SUC_NE_lexicon_tags_2 },
    { 0x6549d52b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8089d530, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa3d535, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8279d537, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5dd53a, SUC_NE_lexicon_tags_2 },
    { 0x4715d53a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc203d53e, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a1d542, SUC_NE_lexicon_tags_1 },
    { 0xa13dd543, SUC_NE_lexicon_tags_2 },
    { 0xa62bd544, SUC_NE_lexicon_tags_2 },
    { 0x4481d545, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeafd547, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2fd54b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c1fd54d, SUC_NE_lexicon_tags_2 },
    { 0x6e85d54d, SUC_NE_lexicon_tags_18 },
    { 0x2d09d54f, SUC_NE_lexicon_tags_2 },
    { 0xdd27d54d, SUC_NE_lexicon_tags_2 },
    { 0xe4e9d550, SUC_NE_lexicon_tags_2 },
    { 0xd6bd550, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc28bd554, SUC_NE_lexicon_tags_2 },
    { 0x3e2bd555, SUC_NE_lexicon_tags_2 },
    { 0xb363d556, SUC_NE_lexicon_tags_2 },
    { 0xe973d556, SUC_NE_lexicon_tags_24 },
    { 0x78b9d556, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8277d55e, SUC_NE_lexicon_tags_2 },
    { 0x5d7d55f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a03d561, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffdd564, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b17d567, SUC_NE_lexicon_tags_164 },
    { 0xb675d568, SUC_NE_lexicon_tags_2 },
    { 0x15dd568, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x22bbd56b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xac05d56d, SUC_NE_lexicon_tags_2 },
    { 0xf559d56d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3bbd570, SUC_NE_lexicon_tags_2 },
    { 0xadc9d571, SUC_NE_lexicon_tags_2 },
    { 0x10f5d571, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf801d575, SUC_NE_lexicon_tags_2 },
    { 0xec5bd576, SUC_NE_lexicon_tags_2 },
    { 0xc3bd576, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfefd57a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5265d57f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13d582, SUC_NE_lexicon_tags_7 },
    { 0x3d0fd583, SUC_NE_lexicon_tags_2 },
    { 0x29add583, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x697bd587, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30afd58b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b7d58d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f43d590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ea5d592, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bbd595, SUC_NE_lexicon_tags_2 },
    { 0xf2e7d596, SUC_NE_lexicon_tags_2 },
    { 0xf585d596, SUC_NE_lexicon_tags_2 },
    { 0xc4d1d598, SUC_NE_lexicon_tags_2 },
    { 0x58fdd595, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e27d59b, SUC_NE_lexicon_tags_2 },
    { 0xb017d59c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1259d59e, SUC_NE_lexicon_tags_2 },
    { 0xf317d59e, SUC_NE_lexicon_tags_2 },
    { 0xab0bd5a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd25fd5a2, SUC_NE_lexicon_tags_2 },
    { 0x56dd5a3, SUC_NE_lexicon_tags_4 },
    { 0x9807d5a4, SUC_NE_lexicon_tags_4 },
    { 0x1adfd5a5, SUC_NE_lexicon_tags_2 },
    { 0xcbf5d5a5, SUC_NE_lexicon_tags_3 },
    { 0xabb1d5a7, SUC_NE_lexicon_tags_2 },
    { 0xaddbd5a6, SUC_NE_lexicon_tags_2 },
    { 0xbbb7d5a9, SUC_NE_lexicon_tags_2 },
    { 0xc1f5d5aa, SUC_NE_lexicon_tags_2 },
    { 0x5601d5aa, SUC_NE_lexicon_tags_2 },
    { 0xb4bd5ac, SUC_NE_lexicon_tags_2 },
    { 0xd917d5ac, SUC_NE_lexicon_tags_2 },
    { 0x13bdd5ad, SUC_NE_lexicon_tags_2 },
    { 0xe91fd5ae, SUC_NE_lexicon_tags_2 },
    { 0x7d6dd5a9, SUC_NE_lexicon_tags_2 },
    { 0xef21d5b1, SUC_NE_lexicon_tags_2 },
    { 0xa2bbd5a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76afd5b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x49d1d5b6, SUC_NE_lexicon_tags_2 },
    { 0xffffd5b7, SUC_NE_lexicon_tags_2 },
    { 0x3cd7d5b6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xaaddd5ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a33d5bc, SUC_NE_lexicon_tags_2 },
    { 0xcae3d5bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x59f3d5bf, SUC_NE_lexicon_tags_12 },
    { 0xf6b7d5c0, SUC_NE_lexicon_tags_2 },
    { 0xd581d5bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1831d5c3, SUC_NE_lexicon_tags_2 },
    { 0x7af7d5c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf101d5c6, SUC_NE_lexicon_tags_2 },
    { 0xd365d5c6, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0x4ef3d5c9, SUC_NE_lexicon_tags_2 },
    { 0xd795d5c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb23bd5cc, SUC_NE_lexicon_tags_2 },
    { 0x247dd5cd, SUC_NE_lexicon_tags_14 },
    { 0xdd3fd5ce, SUC_NE_lexicon_tags_2 },
    { 0x5fb5d5cc, SUC_NE_lexicon_tags_30 },
    { 0x3b7bd5d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebbd5d5, SUC_NE_lexicon_tags_2 },
    { 0xa3e9d5d5, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc757d5d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc343d5de, SUC_NE_lexicon_tags_2 },
    { 0xb405d5de, SUC_NE_lexicon_tags_2 },
    { 0xc3d1d5df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1abbd5e2, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de5d5e7, SUC_NE_lexicon_tags_2 },
    { 0x5277d5e7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1d5f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3d5f3, SUC_NE_lexicon_tags_2 },
    { 0x7457d5f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bdd5fb, SUC_NE_lexicon_tags_2 },
    { 0xe42bd5fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbafbd5fe, SUC_NE_lexicon_tags_2 },
    { 0x1fb5d5ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1e7d601, SUC_NE_lexicon_tags_2 },
    { 0x1c4bd602, SUC_NE_lexicon_tags_2 },
    { 0x68f1d603, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53d607, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa623d60a, SUC_NE_lexicon_tags_2 },
    { 0xbd55d60b, SUC_NE_lexicon_tags_2 },
    { 0x91f7d60b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2871d611, SUC_NE_lexicon_tags_2 },
    { 0x1df7d612, SUC_NE_lexicon_tags_2 },
    { 0x116fd613, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf1d618, SUC_NE_lexicon_tags_2 },
    { 0xa66bd618, SUC_NE_lexicon_tags_2 },
    { 0x7395d61a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2177d61c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6693d621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xadebd623, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc7dd625, SUC_NE_lexicon_tags_2 },
    { 0xc2f3d626, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4041d628, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc53d62c, SUC_NE_lexicon_tags_5 },
    { 0xe03fd62c, SUC_NE_lexicon_tags_0 },
    { 0x573d62e, SUC_NE_lexicon_tags_2 },
    { 0x5a69d62f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ed3d631, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa31d634, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x533d636, SUC_NE_lexicon_tags_2 },
    { 0x2ae1d637, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0xaabd639, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7707d63d, SUC_NE_lexicon_tags_2 },
    { 0x3a1d63e, SUC_NE_lexicon_tags_2 },
    { 0xaffdd63e, SUC_NE_lexicon_tags_2 },
    { 0xc7fd63f, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xfcffd642, SUC_NE_lexicon_tags_2 },
    { 0x665fd643, SUC_NE_lexicon_tags_2 },
    { 0x5f9dd643, SUC_NE_lexicon_tags_2 },
    { 0x65d5d645, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28bd648, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dfd64c, SUC_NE_lexicon_tags_2 },
    { 0x6605d64c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf897d651, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09bd654, SUC_NE_lexicon_tags_2 },
    { 0xcafdd654, SUC_NE_lexicon_tags_2 },
    { 0x38a3d656, SUC_NE_lexicon_tags_2 },
    { 0x502fd657, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a1d65a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b09d661, SUC_NE_lexicon_tags_2 },
    { 0xd347d662, SUC_NE_lexicon_tags_2 },
    { 0xc61fd663, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef19d666, SUC_NE_lexicon_tags_2 },
    { 0x4d59d667, SUC_NE_lexicon_tags_2 },
    { 0x42a9d666, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x90bbd66a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a3d66c, SUC_NE_lexicon_tags_2 },
    { 0x7e6fd66c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf693d671, SUC_NE_lexicon_tags_2 },
    { 0x5071d672, SUC_NE_lexicon_tags_18 },
    { 0xc95fd673, SUC_NE_lexicon_tags_1 },
    { 0x8b2fd672, SUC_NE_lexicon_tags_2 },
    { 0x3f21d675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33bd679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde3d67b, SUC_NE_lexicon_tags_2 },
    { 0x3683d67b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e2fd67e, SUC_NE_lexicon_tags_2 },
    { 0xb493d67f, SUC_NE_lexicon_tags_2 },
    { 0xa437d680, SUC_NE_lexicon_tags_2 },
    { 0x8655d681, SUC_NE_lexicon_tags_2 },
    { 0x5f71d681, SUC_NE_lexicon_tags_2 },
    { 0xd97d683, SUC_NE_lexicon_tags_2 },
    { 0x8cf9d683, SUC_NE_lexicon_tags_2 },
    { 0xb5e1d684, SUC_NE_lexicon_tags_4 },
    { 0x4569d680, SUC_NE_lexicon_tags_2 },
    { 0x923dd687, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f7d68a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec67d68c, SUC_NE_lexicon_tags_2 },
    { 0x5f91d68d, SUC_NE_lexicon_tags_2 },
    { 0xa487d68e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41efd696, SUC_NE_lexicon_tags_2 },
    { 0x1bcbd697, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dd69c, SUC_NE_lexicon_tags_2 },
    { 0x5087d69d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x710dd69f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x753bd6a1, SUC_NE_lexicon_tags_2 },
    { 0x764dd6a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7071d6a4, SUC_NE_lexicon_tags_2 },
    { 0xa8a1d6a5, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f3d6a8, SUC_NE_lexicon_tags_7 },
    { 0xe237d6a9, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7d6ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3007d6ae, SUC_NE_lexicon_tags_2 },
    { 0xa191d6ae, SUC_NE_lexicon_tags_2 },
    { 0xbe3dd6b0, SUC_NE_lexicon_tags_2 },
    { 0xe651d6af, SUC_NE_lexicon_tags_2 },
    { 0x9fdbd6b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9e1d6b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac35d6bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c3d6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd69d6c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3afd6cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d55d6d7, SUC_NE_lexicon_tags_2 },
    { 0x56e7d6d7, SUC_NE_lexicon_tags_2 },
    { 0x2327d6d9, SUC_NE_lexicon_tags_2 },
    { 0x107fd6d7, SUC_NE_lexicon_tags_2 },
    { 0xb3b3d6d7, SUC_NE_lexicon_tags_2 },
    { 0x85d6dc, SUC_NE_lexicon_tags_2 },
    { 0x84e5d6dd, SUC_NE_lexicon_tags_4 },
    { 0x6a67d6db, SUC_NE_lexicon_tags_2 },
    { 0x6e7dd6df, SUC_NE_lexicon_tags_2 },
    { 0x6ebbd6e0, SUC_NE_lexicon_tags_2 },
    { 0xbc21d6e1, SUC_NE_lexicon_tags_2 },
    { 0xe309d6df, SUC_NE_lexicon_tags_2 },
    { 0xd39bd6e3, SUC_NE_lexicon_tags_2 },
    { 0x8c85d6e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x141bd6e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf677d6e9, SUC_NE_lexicon_tags_2 },
    { 0x14a3d6e9, SUC_NE_lexicon_tags_2 },
    { 0x3c5fd6eb, SUC_NE_lexicon_tags_24 },
    { 0x4de1d6ec, SUC_NE_lexicon_tags_209 },
    { 0x6c81d6ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6113d6f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99d6f5, SUC_NE_lexicon_tags_28 },
    { 0, NULL },
    { 0xff0fd6f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4dd6f9, SUC_NE_lexicon_tags_104 },
    { 0x6ddd6f9, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188fd6fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab03d701, SUC_NE_lexicon_tags_2 },
    { 0xe05fd702, SUC_NE_lexicon_tags_2 },
    { 0x6df7d701, SUC_NE_lexicon_tags_2 },
    { 0xbd4bd704, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3311d706, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x378fd709, SUC_NE_lexicon_tags_2 },
    { 0x28c9d709, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ffd70d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8397d70f, SUC_NE_lexicon_tags_3 },
    { 0xb975d710, SUC_NE_lexicon_tags_2 },
    { 0xc101d70f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7add713, SUC_NE_lexicon_tags_2 },
    { 0xa7f1d714, SUC_NE_lexicon_tags_2 },
    { 0x5783d715, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa499d717, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8a5d719, SUC_NE_lexicon_tags_2 },
    { 0xe449d719, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x118fd71d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8217d720, SUC_NE_lexicon_tags_2 },
    { 0x61bd721, SUC_NE_lexicon_tags_2 },
    { 0x80fd722, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d9d727, SUC_NE_lexicon_tags_8 },
    { 0xdfafd728, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68add72a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50c7d72c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ddd731, SUC_NE_lexicon_tags_2 },
    { 0x3f0bd732, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad1d736, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fdd739, SUC_NE_lexicon_tags_2 },
    { 0x9c07d73a, SUC_NE_lexicon_tags_2 },
    { 0x6e63d73b, SUC_NE_lexicon_tags_2 },
    { 0xaa5fd73c, SUC_NE_lexicon_tags_1 },
    { 0xe32dd73b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c83d741, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db1d744, SUC_NE_lexicon_tags_2 },
    { 0x34efd745, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3751d749, SUC_NE_lexicon_tags_2 },
    { 0x12b7d74a, SUC_NE_lexicon_tags_2 },
    { 0x55ffd74b, SUC_NE_lexicon_tags_2 },
    { 0xbf25d74c, SUC_NE_lexicon_tags_2 },
    { 0xf929d74d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2341d74f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffcfd751, SUC_NE_lexicon_tags_2 },
    { 0x35d3d752, SUC_NE_lexicon_tags_2 },
    { 0xd7afd751, SUC_NE_lexicon_tags_2 },
    { 0xf6ffd754, SUC_NE_lexicon_tags_2 },
    { 0xb835d755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe083d75a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fdd75d, SUC_NE_lexicon_tags_2 },
    { 0xff4dd75e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf33d761, SUC_NE_lexicon_tags_2 },
    { 0x6f35d761, SUC_NE_lexicon_tags_2 },
    { 0x2675d763, SUC_NE_lexicon_tags_2 },
    { 0xb6c9d764, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2bd768, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x695bd76b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x1187d76d, SUC_NE_lexicon_tags_2 },
    { 0xc26dd76e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bd7d770, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00dd773, SUC_NE_lexicon_tags_2 },
    { 0x186dd774, SUC_NE_lexicon_tags_3 },
    { 0x3c7fd774, SUC_NE_lexicon_tags_2 },
    { 0x4f55d774, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76dd77a, SUC_NE_lexicon_tags_2 },
    { 0xd4ebd77b, SUC_NE_lexicon_tags_2 },
    { 0x16bdd77c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd7d77e, SUC_NE_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3d782, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x6fedd784, SUC_NE_lexicon_tags_17 },
    { 0xe9e3d785, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2755d789, SUC_NE_lexicon_tags_2 },
    { 0x8af5d78a, SUC_NE_lexicon_tags_1 },
    { 0x6c99d78a, SUC_NE_lexicon_tags_2 },
    { 0x1e83d78c, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x3c53d78e, SUC_NE_lexicon_tags_2 },
    { 0x6981d78f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaeddd791, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40cdd793, SUC_NE_lexicon_tags_2 },
    { 0xd775d794, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f9d79a, SUC_NE_lexicon_tags_2 },
    { 0x971fd79b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8837d79d, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x5469d79f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedd7a3, SUC_NE_lexicon_tags_1 },
    { 0xfeb3d7a4, SUC_NE_lexicon_tags_2 },
    { 0x893d7a5, SUC_NE_lexicon_tags_2 },
    { 0x4007d7a4, SUC_NE_lexicon_tags_2 },
    { 0x291fd7a5, SUC_NE_lexicon_tags_13 },
    { 0x7fcbd7a7, SUC_NE_lexicon_tags_2 },
    { 0x1a89d7a4, SUC_NE_lexicon_tags_2 },
    { 0x4ea5d7a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29bd7ae, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2abfd7b0, SUC_NE_lexicon_tags_2 },
    { 0x7055d7b0, SUC_NE_lexicon_tags_2 },
    { 0x6479d7b2, SUC_NE_lexicon_tags_2 },
    { 0xc7d9d7b3, SUC_NE_lexicon_tags_2 },
    { 0xf435d7b4, SUC_NE_lexicon_tags_4 },
    { 0x6533d7b5, SUC_NE_lexicon_tags_45 },
    { 0x80bd7b5, SUC_NE_lexicon_tags_2 },
    { 0xfe9d7b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4835d7b9, SUC_NE_lexicon_tags_2 },
    { 0x3d93d7b9, SUC_NE_lexicon_tags_2 },
    { 0x7b3dd7bb, SUC_NE_lexicon_tags_2 },
    { 0xbe7d7b9, SUC_NE_lexicon_tags_2 },
    { 0xe823d7b9, SUC_NE_lexicon_tags_2 },
    { 0xa007d7be, SUC_NE_lexicon_tags_7 },
    { 0xf799d7bf, SUC_NE_lexicon_tags_2 },
    { 0x8eafd7be, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd71d7c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe287d7c6, SUC_NE_lexicon_tags_2 },
    { 0x3647d7c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x415dd7c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x25dbd7cb, SUC_NE_lexicon_tags_2 },
    { 0x5e9fd7cc, SUC_NE_lexicon_tags_2 },
    { 0x726dd7cc, SUC_NE_lexicon_tags_352 },
    { 0x3d1d7cc, SUC_NE_lexicon_tags_2 },
    { 0x3963d7cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a1d7d2, SUC_NE_lexicon_tags_2 },
    { 0x9b7fd7d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e73d7d6, SUC_NE_lexicon_tags_2 },
    { 0x9dfbd7d7, SUC_NE_lexicon_tags_17 },
    { 0xd429d7d8, SUC_NE_lexicon_tags_2 },
    { 0xf319d7d6, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb053d7e1, SUC_NE_lexicon_tags_2 },
    { 0xb56bd7e2, SUC_NE_lexicon_tags_3 },
    { 0x4a35d7e1, SUC_NE_lexicon_tags_2 },
    { 0x4747d7e4, SUC_NE_lexicon_tags_2 },
    { 0xabc9d7e3, SUC_NE_lexicon_tags_2 },
    { 0xd09bd7e1, SUC_NE_lexicon_tags_2 },
    { 0xa5b5d7e7, SUC_NE_lexicon_tags_4 },
    { 0xa5f3d7e7, SUC_NE_lexicon_tags_2 },
    { 0x61a3d7e9, SUC_NE_lexicon_tags_2 },
    { 0x4c43d7e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdfd7ef, SUC_NE_lexicon_tags_2 },
    { 0xfa8dd7ef, SUC_NE_lexicon_tags_2 },
    { 0x3479d7f1, SUC_NE_lexicon_tags_2 },
    { 0xccc1d7f2, SUC_NE_lexicon_tags_2 },
    { 0x1817d7ef, SUC_NE_lexicon_tags_1 },
    { 0x9759d7ef, SUC_NE_lexicon_tags_2 },
    { 0x563d7f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c4fd7f7, SUC_NE_lexicon_tags_334 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe701d7fc, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfc59d7fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbfd801, SUC_NE_lexicon_tags_363 },
    { 0xaef3d801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x81c5d804, SUC_NE_lexicon_tags_3 },
    { 0x7d1fd805, SUC_NE_lexicon_tags_2 },
    { 0x681fd806, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565d80b, SUC_NE_lexicon_tags_11 },
    { 0x464dd80c, SUC_NE_lexicon_tags_2 },
    { 0x9af7d80b, SUC_NE_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107d812, SUC_NE_lexicon_tags_2 },
    { 0x38cfd813, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2481d815, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861dd819, SUC_NE_lexicon_tags_2 },
    { 0x38edd819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae1d81c, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0x451bd81e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a57d822, SUC_NE_lexicon_tags_2 },
    { 0x5f53d823, SUC_NE_lexicon_tags_1 },
    { 0xec27d823, SUC_NE_lexicon_tags_7 },
    { 0x70add823, SUC_NE_lexicon_tags_13 },
    { 0x82b7d826, SUC_NE_lexicon_tags_2 },
    { 0xa617d827, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x430dd829, SUC_NE_lexicon_tags_2 },
    { 0x5705d829, SUC_NE_lexicon_tags_2 },
    { 0x4e07d82b, SUC_NE_lexicon_tags_4 },
    { 0x6183d829, SUC_NE_lexicon_tags_2 },
    { 0x10d3d82d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6019d82f, SUC_NE_lexicon_tags_2 },
    { 0x7335d82f, SUC_NE_lexicon_tags_2 },
    { 0xe03fd830, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5441d833, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d99d836, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8b7d838, SUC_NE_lexicon_tags_2 },
    { 0x4f3bd839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56add83f, SUC_NE_lexicon_tags_2 },
    { 0x4d81d83f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dbd846, SUC_NE_lexicon_tags_2 },
    { 0x996dd847, SUC_NE_lexicon_tags_3 },
    { 0x22a3d848, SUC_NE_lexicon_tags_2 },
    { 0x2dbdd846, SUC_NE_lexicon_tags_2 },
    { 0x45c1d847, SUC_NE_lexicon_tags_2 },
    { 0xd11bd84b, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb5d853, SUC_NE_lexicon_tags_2 },
    { 0x4fdd853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcbd858, SUC_NE_lexicon_tags_4 },
    { 0x495d859, SUC_NE_lexicon_tags_2 },
    { 0x16f7d85a, SUC_NE_lexicon_tags_2 },
    { 0xee95d85b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e41d85e, SUC_NE_lexicon_tags_2 },
    { 0x1e4dd85f, SUC_NE_lexicon_tags_2 },
    { 0x2c5fd85f, SUC_NE_lexicon_tags_2 },
    { 0x6ac5d861, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da9d865, SUC_NE_lexicon_tags_7 },
    { 0xa079d866, SUC_NE_lexicon_tags_2 },
    { 0x184fd867, SUC_NE_lexicon_tags_0 },
    { 0xc37bd867, SUC_NE_lexicon_tags_2 },
    { 0x6035d865, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x710bd86c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccfd872, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f9d874, SUC_NE_lexicon_tags_2 },
    { 0xa8a5d874, SUC_NE_lexicon_tags_81 },
    { 0x3f37d874, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde39d87b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc79fd87d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a1d880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2675d883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec13d888, SUC_NE_lexicon_tags_2 },
    { 0xd08fd889, SUC_NE_lexicon_tags_1 },
    { 0xae7bd889, SUC_NE_lexicon_tags_2 },
    { 0x6993d889, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd543d88d, SUC_NE_lexicon_tags_2 },
    { 0xb17fd88d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5515d891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8445d893, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf521d897, SUC_NE_lexicon_tags_6 },
    { 0x2a19d898, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7fd89d, SUC_NE_lexicon_tags_17 },
    { 0xd90fd89d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4297d8a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3459d8a9, SUC_NE_lexicon_tags_3 },
    { 0x3dddd8aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a7d8ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed45d8b3, SUC_NE_lexicon_tags_4 },
    { 0x863dd8b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb3d8b7, SUC_NE_lexicon_tags_0 },
    { 0x4e41d8b7, SUC_NE_lexicon_tags_4 },
    { 0xb099d8b9, SUC_NE_lexicon_tags_2 },
    { 0x9ee5d8ba, SUC_NE_lexicon_tags_2 },
    { 0x6b27d8bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc7d8bf, SUC_NE_lexicon_tags_2 },
    { 0x3bfd8c0, SUC_NE_lexicon_tags_2 },
    { 0x23a7d8c1, SUC_NE_lexicon_tags_2 },
    { 0xeb67d8bf, SUC_NE_lexicon_tags_2 },
    { 0x664bd8c1, SUC_NE_lexicon_tags_2 },
    { 0x99f3d8c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed59d8c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1903d8cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3afbd8cd, SUC_NE_lexicon_tags_2 },
    { 0x4bcbd8ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217fd8d3, SUC_NE_lexicon_tags_2 },
    { 0xac9dd8d4, SUC_NE_lexicon_tags_4 },
    { 0x8d43d8d5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9c1d8d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca41d8db, SUC_NE_lexicon_tags_3 },
    { 0xcdbdd8dc, SUC_NE_lexicon_tags_2 },
    { 0x55b1d8db, SUC_NE_lexicon_tags_59 },
    { 0x4891d8dc, SUC_NE_lexicon_tags_2 },
    { 0x12f3d8dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e29d8e1, SUC_NE_lexicon_tags_2 },
    { 0x639fd8e2, SUC_NE_lexicon_tags_2 },
    { 0xc08fd8e3, SUC_NE_lexicon_tags_2 },
    { 0x54f9d8e2, SUC_NE_lexicon_tags_7 },
    { 0xfb8fd8e5, SUC_NE_lexicon_tags_2 },
    { 0xc89bd8e6, SUC_NE_lexicon_tags_2 },
    { 0x3163d8e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6b7d8e9, SUC_NE_lexicon_tags_2 },
    { 0x37f5d8ea, SUC_NE_lexicon_tags_7 },
    { 0xb2e9d8e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7119d8f0, SUC_NE_lexicon_tags_115 },
    { 0x5ea9d8f1, SUC_NE_lexicon_tags_2 },
    { 0xe061d8f1, SUC_NE_lexicon_tags_2 },
    { 0x986fd8f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec29d8f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c9d8fa, SUC_NE_lexicon_tags_2 },
    { 0xf003d8fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efd8ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce3d901, SUC_NE_lexicon_tags_2 },
    { 0x73d9d902, SUC_NE_lexicon_tags_2 },
    { 0x73d7d903, SUC_NE_lexicon_tags_2 },
    { 0xe175d903, SUC_NE_lexicon_tags_2 },
    { 0x1fedd905, SUC_NE_lexicon_tags_2 },
    { 0xaeefd905, SUC_NE_lexicon_tags_2 },
    { 0xd291d907, SUC_NE_lexicon_tags_2 },
    { 0xc48bd901, SUC_NE_lexicon_tags_2 },
    { 0x6d43d909, SUC_NE_lexicon_tags_2 },
    { 0x71b5d90a, SUC_NE_lexicon_tags_2 },
    { 0x9167d90b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66d3d90d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35d911, SUC_NE_lexicon_tags_2 },
    { 0x1dd9d912, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x881d914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4397d91a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7495d920, SUC_NE_lexicon_tags_4 },
    { 0xe6cdd920, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bd927, SUC_NE_lexicon_tags_66 },
    { 0x1201d928, SUC_NE_lexicon_tags_3 },
    { 0x82b7d927, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x24add92b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5339d92e, SUC_NE_lexicon_tags_2 },
    { 0x2341d92f, SUC_NE_lexicon_tags_11 },
    { 0x962bd92e, SUC_NE_lexicon_tags_2 },
    { 0x9ef1d931, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa351d936, SUC_NE_lexicon_tags_4 },
    { 0x466bd937, SUC_NE_lexicon_tags_2 },
    { 0x5e01d938, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf685d93c, SUC_NE_lexicon_tags_3 },
    { 0xebb5d93d, SUC_NE_lexicon_tags_2 },
    { 0x464fd93d, SUC_NE_lexicon_tags_2 },
    { 0xd685d93f, SUC_NE_lexicon_tags_7 },
    { 0x7297d93e, SUC_NE_lexicon_tags_15 },
    { 0, NULL },
    { 0x15a3d942, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x784bd944, SUC_NE_lexicon_tags_2 },
    { 0x53d9d944, SUC_NE_lexicon_tags_17 },
    { 0x8c99d946, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc32fd948, SUC_NE_lexicon_tags_2 },
    { 0x23dfd949, SUC_NE_lexicon_tags_2 },
    { 0xf155d94a, SUC_NE_lexicon_tags_2 },
    { 0xfabdd94b, SUC_NE_lexicon_tags_2 },
    { 0xcedd94b, SUC_NE_lexicon_tags_109 },
    { 0xfaf5d94d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2221d950, SUC_NE_lexicon_tags_2 },
    { 0x304fd951, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf223d955, SUC_NE_lexicon_tags_2 },
    { 0x4dd9d955, SUC_NE_lexicon_tags_2 },
    { 0x82c1d955, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8731d95b, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba7d960, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5893d963, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac27d968, SUC_NE_lexicon_tags_2 },
    { 0x87a3d968, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc35d96b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd39d96f, SUC_NE_lexicon_tags_1 },
    { 0xabcdd970, SUC_NE_lexicon_tags_2 },
    { 0x275bd971, SUC_NE_lexicon_tags_2 },
    { 0xf323d972, SUC_NE_lexicon_tags_2 },
    { 0x3469d972, SUC_NE_lexicon_tags_2 },
    { 0x6a75d974, SUC_NE_lexicon_tags_7 },
    { 0x7e67d973, SUC_NE_lexicon_tags_2 },
    { 0xb5c3d976, SUC_NE_lexicon_tags_2 },
    { 0xa4bfd973, SUC_NE_lexicon_tags_2 },
    { 0x36e7d978, SUC_NE_lexicon_tags_2 },
    { 0xdd8fd979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6331d97c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee1d985, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b67d987, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acfd991, SUC_NE_lexicon_tags_2 },
    { 0x7b47d992, SUC_NE_lexicon_tags_2 },
    { 0x123dd993, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d49d995, SUC_NE_lexicon_tags_2 },
    { 0x66c5d996, SUC_NE_lexicon_tags_2 },
    { 0xd01d997, SUC_NE_lexicon_tags_2 },
    { 0x2b4dd997, SUC_NE_lexicon_tags_2 },
    { 0x3f5fd998, SUC_NE_lexicon_tags_1 },
    { 0xef8dd99a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff13d99d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcddfd99f, SUC_NE_lexicon_tags_2 },
    { 0xf927d9a0, SUC_NE_lexicon_tags_2 },
    { 0xe1a1d9a1, SUC_NE_lexicon_tags_2 },
    { 0x7803d9a2, SUC_NE_lexicon_tags_2 },
    { 0x72d7d9a2, SUC_NE_lexicon_tags_2 },
    { 0xa71fd9a2, SUC_NE_lexicon_tags_2 },
    { 0xe73bd9a5, SUC_NE_lexicon_tags_2 },
    { 0x69a7d9a5, SUC_NE_lexicon_tags_49 },
    { 0x2087d9a0, SUC_NE_lexicon_tags_2 },
    { 0xdfadd9a8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x9209d9aa, SUC_NE_lexicon_tags_19 },
    { 0xebb9d9aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c5d9ad, SUC_NE_lexicon_tags_2 },
    { 0x216fd9ae, SUC_NE_lexicon_tags_2 },
    { 0x7871d9af, SUC_NE_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ffd9b3, SUC_NE_lexicon_tags_2 },
    { 0xc061d9b4, SUC_NE_lexicon_tags_2 },
    { 0xb845d9b5, SUC_NE_lexicon_tags_2 },
    { 0xca09d9b6, SUC_NE_lexicon_tags_2 },
    { 0x4a0dd9b4, SUC_NE_lexicon_tags_2 },
    { 0xa44bd9b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9d9bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32dd9c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd871d9cc, SUC_NE_lexicon_tags_2 },
    { 0x6135d9cc, SUC_NE_lexicon_tags_2 },
    { 0x6355d9cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x562bd9d1, SUC_NE_lexicon_tags_2 },
    { 0x31f3d9d1, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c1d9d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dd9d9, SUC_NE_lexicon_tags_2 },
    { 0x678bd9da, SUC_NE_lexicon_tags_2 },
    { 0xa0f7d9db, SUC_NE_lexicon_tags_2 },
    { 0xba1d9dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1b3d9de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc1d9e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x16abd9e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2bd9e9, SUC_NE_lexicon_tags_2 },
    { 0xbfedd9ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2439d9ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6453d9f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b77d9f3, SUC_NE_lexicon_tags_3 },
    { 0x55add9f4, SUC_NE_lexicon_tags_1 },
    { 0xe1d3d9f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bddd9f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x810fd9fa, SUC_NE_lexicon_tags_2 },
    { 0x5d2bd9fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x190dd9fe, SUC_NE_lexicon_tags_2 },
    { 0xdf31d9fe, SUC_NE_lexicon_tags_7 },
    { 0x5513da00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1095da02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26fda06, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989fda0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4409da10, SUC_NE_lexicon_tags_0 },
    { 0xcfe5da11, SUC_NE_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf27da16, SUC_NE_lexicon_tags_2 },
    { 0x63e1da16, SUC_NE_lexicon_tags_2 },
    { 0x1af3da17, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a61da1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b7da1e, SUC_NE_lexicon_tags_30 },
    { 0xf1cfda1f, SUC_NE_lexicon_tags_2 },
    { 0xdc45da1e, SUC_NE_lexicon_tags_2 },
    { 0xecafda1f, SUC_NE_lexicon_tags_2 },
    { 0xdb7bda1e, SUC_NE_lexicon_tags_2 },
    { 0xbe33da22, SUC_NE_lexicon_tags_2 },
    { 0x744fda23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8683da26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5bda29, SUC_NE_lexicon_tags_2 },
    { 0x6ec3da2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a1da2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb543da2f, SUC_NE_lexicon_tags_145 },
    { 0, NULL },
    { 0x6badda31, SUC_NE_lexicon_tags_2 },
    { 0x9807da31, SUC_NE_lexicon_tags_2 },
    { 0x3289da33, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d1da39, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1dda3c, SUC_NE_lexicon_tags_3 },
    { 0x9a65da3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa5da40, SUC_NE_lexicon_tags_18 },
    { 0xc02fda40, SUC_NE_lexicon_tags_2 },
    { 0xcd8bda42, SUC_NE_lexicon_tags_2 },
    { 0x355dda43, SUC_NE_lexicon_tags_2 },
    { 0xd1fda40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29dda49, SUC_NE_lexicon_tags_0 },
    { 0xac29da49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf751da4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31da56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc27da5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c0dda5d, SUC_NE_lexicon_tags_2 },
    { 0xefc7da5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf569da61, SUC_NE_lexicon_tags_2 },
    { 0x14b3da62, SUC_NE_lexicon_tags_2 },
    { 0xb397da63, SUC_NE_lexicon_tags_2 },
    { 0xdba7da64, SUC_NE_lexicon_tags_2 },
    { 0xaf63da65, SUC_NE_lexicon_tags_2 },
    { 0x1829da66, SUC_NE_lexicon_tags_2 },
    { 0xe5f3da67, SUC_NE_lexicon_tags_2 },
    { 0x50bfda61, SUC_NE_lexicon_tags_2 },
    { 0xfa79da69, SUC_NE_lexicon_tags_391 },
    { 0xe40fda6a, SUC_NE_lexicon_tags_2 },
    { 0x304fda6a, SUC_NE_lexicon_tags_2 },
    { 0x132dda6c, SUC_NE_lexicon_tags_2 },
    { 0xb239da6c, SUC_NE_lexicon_tags_3 },
    { 0x5a67da6c, SUC_NE_lexicon_tags_2 },
    { 0x7c99da6f, SUC_NE_lexicon_tags_2 },
    { 0x9193da6d, SUC_NE_lexicon_tags_19 },
    { 0xa65fda71, SUC_NE_lexicon_tags_2 },
    { 0x64b7da72, SUC_NE_lexicon_tags_2 },
    { 0x19b5da71, SUC_NE_lexicon_tags_2 },
    { 0xfa15da72, SUC_NE_lexicon_tags_2 },
    { 0x9fa5da72, SUC_NE_lexicon_tags_2 },
    { 0x9939da74, SUC_NE_lexicon_tags_3 },
    { 0xf1a1da75, SUC_NE_lexicon_tags_2 },
    { 0x2713da72, SUC_NE_lexicon_tags_2 },
    { 0xadc7da79, SUC_NE_lexicon_tags_3 },
    { 0xc81dda6f, SUC_NE_lexicon_tags_2 },
    { 0x60cfda6b, SUC_NE_lexicon_tags_2 },
    { 0x896fda6d, SUC_NE_lexicon_tags_2 },
    { 0x9fc1da7d, SUC_NE_lexicon_tags_7 },
    { 0xde8bda7e, SUC_NE_lexicon_tags_2 },
    { 0x39cbda7f, SUC_NE_lexicon_tags_2 },
    { 0x86adda7f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef0bda82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7027da86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c9da8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe5da94, SUC_NE_lexicon_tags_2 },
    { 0x8219da94, SUC_NE_lexicon_tags_3 },
    { 0xc137da96, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaddda99, SUC_NE_lexicon_tags_2 },
    { 0x2c89da99, SUC_NE_lexicon_tags_2 },
    { 0x7e81da9b, SUC_NE_lexicon_tags_2 },
    { 0xc61da9a, SUC_NE_lexicon_tags_2 },
    { 0x9055da9c, SUC_NE_lexicon_tags_2 },
    { 0xc871da99, SUC_NE_lexicon_tags_4 },
    { 0x33bfda9f, SUC_NE_lexicon_tags_2 },
    { 0xedebda9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1bdaa3, SUC_NE_lexicon_tags_2 },
    { 0x5b4fdaa4, SUC_NE_lexicon_tags_108 },
    { 0, NULL },
    { 0xe17daa6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5827daa8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316fdaac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa1dab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc23dab5, SUC_NE_lexicon_tags_2 },
    { 0xaee1dab5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bb7dab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd99ddaba, SUC_NE_lexicon_tags_2 },
    { 0x47a7daba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb91dabd, SUC_NE_lexicon_tags_2 },
    { 0x938ddabe, SUC_NE_lexicon_tags_2 },
    { 0x6c6fdabf, SUC_NE_lexicon_tags_2 },
    { 0x9ef5dabe, SUC_NE_lexicon_tags_7 },
    { 0xa879dac1, SUC_NE_lexicon_tags_2 },
    { 0x685ddabd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92fdac6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee9dac8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1997daca, SUC_NE_lexicon_tags_2 },
    { 0x7569daca, SUC_NE_lexicon_tags_2 },
    { 0x9641dacc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e29dad0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf9dad2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda97dad5, SUC_NE_lexicon_tags_2 },
    { 0x3237dad6, SUC_NE_lexicon_tags_379 },
    { 0x9841dad7, SUC_NE_lexicon_tags_2 },
    { 0x993dad8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c9dadb, SUC_NE_lexicon_tags_2 },
    { 0x987dadc, SUC_NE_lexicon_tags_2 },
    { 0xf77fdadd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2ddae0, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15fdae4, SUC_NE_lexicon_tags_7 },
    { 0xad19dae5, SUC_NE_lexicon_tags_2 },
    { 0xb48bdae5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a07daeb, SUC_NE_lexicon_tags_2 },
    { 0x7b29daec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x65a9daee, SUC_NE_lexicon_tags_130 },
    { 0x4dbfdaef, SUC_NE_lexicon_tags_2 },
    { 0x7407daee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8dddaf2, SUC_NE_lexicon_tags_0 },
    { 0x1627daf2, SUC_NE_lexicon_tags_2 },
    { 0x5497daf4, SUC_NE_lexicon_tags_2 },
    { 0x64addaf5, SUC_NE_lexicon_tags_2 },
    { 0x7415daf6, SUC_NE_lexicon_tags_2 },
    { 0xb0addaf7, SUC_NE_lexicon_tags_2 },
    { 0x59fdaf5, SUC_NE_lexicon_tags_2 },
    { 0xd603daf2, SUC_NE_lexicon_tags_2 },
    { 0xed09daf7, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x41cbdafc, SUC_NE_lexicon_tags_2 },
    { 0xbcc5dafc, SUC_NE_lexicon_tags_2 },
    { 0x5b5bdafe, SUC_NE_lexicon_tags_104 },
    { 0x633ddaff, SUC_NE_lexicon_tags_30 },
    { 0xc16bdafd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd059db03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9971db06, SUC_NE_lexicon_tags_2 },
    { 0xfb45db06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb45db14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x27c5db16, SUC_NE_lexicon_tags_17 },
    { 0x73a5db17, SUC_NE_lexicon_tags_2 },
    { 0xfb31db17, SUC_NE_lexicon_tags_2 },
    { 0x1331db19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c5db1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x483fdb21, SUC_NE_lexicon_tags_2 },
    { 0x8837db22, SUC_NE_lexicon_tags_2 },
    { 0xa659db23, SUC_NE_lexicon_tags_2 },
    { 0xf5bfdb22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7135db2c, SUC_NE_lexicon_tags_2 },
    { 0x3a79db2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9835db32, SUC_NE_lexicon_tags_3 },
    { 0x296fdb33, SUC_NE_lexicon_tags_2 },
    { 0x2575db34, SUC_NE_lexicon_tags_2 },
    { 0x4a5bdb35, SUC_NE_lexicon_tags_2 },
    { 0x2541db34, SUC_NE_lexicon_tags_7 },
    { 0x3c79db37, SUC_NE_lexicon_tags_2 },
    { 0x2815db37, SUC_NE_lexicon_tags_2 },
    { 0x4473db39, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4ddb40, SUC_NE_lexicon_tags_2 },
    { 0x35e9db40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8737db43, SUC_NE_lexicon_tags_0 },
    { 0x3b6ddb44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d5db47, SUC_NE_lexicon_tags_2 },
    { 0x2883db48, SUC_NE_lexicon_tags_2 },
    { 0xb193db48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x248bdb4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b5db4e, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8fdb58, SUC_NE_lexicon_tags_2 },
    { 0xbee1db59, SUC_NE_lexicon_tags_2 },
    { 0x3a7bdb58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dfdb5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c51db60, SUC_NE_lexicon_tags_2 },
    { 0xf86bdb61, SUC_NE_lexicon_tags_331 },
    { 0xd581db62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e5db65, SUC_NE_lexicon_tags_2 },
    { 0xfe7ddb65, SUC_NE_lexicon_tags_2 },
    { 0x2db9db67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4fdb6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9ddb6f, SUC_NE_lexicon_tags_2 },
    { 0x6375db70, SUC_NE_lexicon_tags_2 },
    { 0x2045db71, SUC_NE_lexicon_tags_5 },
    { 0xe6ebdb6f, SUC_NE_lexicon_tags_2 },
    { 0x101fdb73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe053db77, SUC_NE_lexicon_tags_2 },
    { 0x104fdb78, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cfdb7d, SUC_NE_lexicon_tags_17 },
    { 0x8353db7d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fbdb81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf05db83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ab1db85, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe59db87, SUC_NE_lexicon_tags_5 },
    { 0xbcd1db88, SUC_NE_lexicon_tags_2 },
    { 0xc621db88, SUC_NE_lexicon_tags_11 },
    { 0x62a1db8a, SUC_NE_lexicon_tags_40 },
    { 0x8fdb8b, SUC_NE_lexicon_tags_4 },
    { 0x8137db87, SUC_NE_lexicon_tags_2 },
    { 0xd029db8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd57db90, SUC_NE_lexicon_tags_4 },
    { 0x8085db91, SUC_NE_lexicon_tags_2 },
    { 0xb733db92, SUC_NE_lexicon_tags_2 },
    { 0x9b27db91, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a7db99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xba13db9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b3fdb9d, SUC_NE_lexicon_tags_1 },
    { 0x97fbdb9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d45dba5, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xd38bdba7, SUC_NE_lexicon_tags_69 },
    { 0x70a3dba8, SUC_NE_lexicon_tags_2 },
    { 0x8907dba8, SUC_NE_lexicon_tags_2 },
    { 0x614fdba9, SUC_NE_lexicon_tags_4 },
    { 0x3dddbab, SUC_NE_lexicon_tags_2 },
    { 0x72d5dba8, SUC_NE_lexicon_tags_2 },
    { 0x7c63dbad, SUC_NE_lexicon_tags_2 },
    { 0x9f2bdbae, SUC_NE_lexicon_tags_2 },
    { 0xb4bbdbaf, SUC_NE_lexicon_tags_2 },
    { 0xdc25dbb0, SUC_NE_lexicon_tags_2 },
    { 0x20f9dbb1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8fdbb5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7365dbb7, SUC_NE_lexicon_tags_2 },
    { 0x59b3dbb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x151ddbba, SUC_NE_lexicon_tags_2 },
    { 0x7fd1dbba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a5dbbe, SUC_NE_lexicon_tags_2 },
    { 0x50bfdbbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x80f9dbc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8889dbc3, SUC_NE_lexicon_tags_157 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e1dbc7, SUC_NE_lexicon_tags_2 },
    { 0x2147dbc8, SUC_NE_lexicon_tags_2 },
    { 0x5e7ddbc8, SUC_NE_lexicon_tags_2 },
    { 0x6b65dbca, SUC_NE_lexicon_tags_2 },
    { 0xe93dbc8, SUC_NE_lexicon_tags_2 },
    { 0xf46fdbc9, SUC_NE_lexicon_tags_1 },
    { 0x7083dbcd, SUC_NE_lexicon_tags_7 },
    { 0xfc3bdbcc, SUC_NE_lexicon_tags_2 },
    { 0x62ddbce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb179dbd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efbdbd8, SUC_NE_lexicon_tags_2 },
    { 0xe2d9dbd9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d81dbdd, SUC_NE_lexicon_tags_2 },
    { 0x29dbdd, SUC_NE_lexicon_tags_4 },
    { 0x855bdbdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdc7dbe1, SUC_NE_lexicon_tags_2 },
    { 0xdcf3dbe2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd129dbe8, SUC_NE_lexicon_tags_2 },
    { 0xb5b7dbe9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae5bdbeb, SUC_NE_lexicon_tags_2 },
    { 0x2eaddbec, SUC_NE_lexicon_tags_2 },
    { 0xf6fddbed, SUC_NE_lexicon_tags_2 },
    { 0xfca7dbee, SUC_NE_lexicon_tags_2 },
    { 0xb7d7dbef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd435dbf2, SUC_NE_lexicon_tags_2 },
    { 0xd44bdbf2, SUC_NE_lexicon_tags_2 },
    { 0x67d3dbf4, SUC_NE_lexicon_tags_2 },
    { 0xecf5dbf5, SUC_NE_lexicon_tags_7 },
    { 0x1eb1dbf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16fdbf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c83dbfd, SUC_NE_lexicon_tags_40 },
    { 0x3a4ddbfe, SUC_NE_lexicon_tags_4 },
    { 0xd549dbff, SUC_NE_lexicon_tags_2 },
    { 0xf2d3dc00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efbdc03, SUC_NE_lexicon_tags_2 },
    { 0xe12fdc03, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05dc0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4f1dc0c, SUC_NE_lexicon_tags_2 },
    { 0x3cc9dc0d, SUC_NE_lexicon_tags_2 },
    { 0xac9dc0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x45f1dc10, SUC_NE_lexicon_tags_2 },
    { 0x8969dc10, SUC_NE_lexicon_tags_2 },
    { 0xf42fdc12, SUC_NE_lexicon_tags_2 },
    { 0x8029dc13, SUC_NE_lexicon_tags_2 },
    { 0x18bfdc14, SUC_NE_lexicon_tags_2 },
    { 0xbffddc10, SUC_NE_lexicon_tags_2 },
    { 0xcddbdc16, SUC_NE_lexicon_tags_2 },
    { 0xd035dc17, SUC_NE_lexicon_tags_2 },
    { 0x799bdc17, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1dc1f, SUC_NE_lexicon_tags_2 },
    { 0x23dc20, SUC_NE_lexicon_tags_2 },
    { 0xad0bdc21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cddc26, SUC_NE_lexicon_tags_2 },
    { 0x27b5dc27, SUC_NE_lexicon_tags_2 },
    { 0x1a3dc27, SUC_NE_lexicon_tags_2 },
    { 0x7067dc29, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27bdc31, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247dc3a, SUC_NE_lexicon_tags_7 },
    { 0x3f3bdc3b, SUC_NE_lexicon_tags_2 },
    { 0x7129dc3b, SUC_NE_lexicon_tags_7 },
    { 0xbd19dc3d, SUC_NE_lexicon_tags_2 },
    { 0x1ee7dc3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe359dc40, SUC_NE_lexicon_tags_2 },
    { 0x3d37dc41, SUC_NE_lexicon_tags_2 },
    { 0x4b6ddc41, SUC_NE_lexicon_tags_2 },
    { 0xed69dc42, SUC_NE_lexicon_tags_2 },
    { 0x146bdc41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ddc47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc7dc4d, SUC_NE_lexicon_tags_2 },
    { 0x1563dc4e, SUC_NE_lexicon_tags_2 },
    { 0xe6c3dc4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x28c9dc51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b7dc55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2abdc57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc9bdc59, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x27bfdc5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edfdc5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fd7dc60, SUC_NE_lexicon_tags_2 },
    { 0x2887dc61, SUC_NE_lexicon_tags_2 },
    { 0xf1f3dc61, SUC_NE_lexicon_tags_2 },
    { 0xb89bdc63, SUC_NE_lexicon_tags_4 },
    { 0xc5b9dc63, SUC_NE_lexicon_tags_50 },
    { 0xd3bbdc65, SUC_NE_lexicon_tags_2 },
    { 0x3a35dc66, SUC_NE_lexicon_tags_4 },
    { 0xd8a3dc60, SUC_NE_lexicon_tags_4 },
    { 0x4cdddc68, SUC_NE_lexicon_tags_2 },
    { 0x50abdc63, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf5dc6c, SUC_NE_lexicon_tags_2 },
    { 0x4921dc6d, SUC_NE_lexicon_tags_2 },
    { 0x6f49dc6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0bdc72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e37dc74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7dc78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x684fdc7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41dfdc7d, SUC_NE_lexicon_tags_2 },
    { 0x89b7dc7e, SUC_NE_lexicon_tags_2 },
    { 0x7a6bdc7e, SUC_NE_lexicon_tags_2 },
    { 0x35b3dc80, SUC_NE_lexicon_tags_2 },
    { 0x1aa9dc81, SUC_NE_lexicon_tags_2 },
    { 0x4f65dc80, SUC_NE_lexicon_tags_2 },
    { 0x1bc5dc7d, SUC_NE_lexicon_tags_2 },
    { 0xacf3dc84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x35d9dc86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5221dc89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa245dc8e, SUC_NE_lexicon_tags_2 },
    { 0xcb87dc8f, SUC_NE_lexicon_tags_1 },
    { 0x774bdc90, SUC_NE_lexicon_tags_78 },
    { 0x24a5dc90, SUC_NE_lexicon_tags_2 },
    { 0x6a9fdc91, SUC_NE_lexicon_tags_2 },
    { 0x3913dc92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff55dc95, SUC_NE_lexicon_tags_2 },
    { 0xb9c3dc96, SUC_NE_lexicon_tags_2 },
    { 0xa961dc97, SUC_NE_lexicon_tags_2 },
    { 0x6d8fdc98, SUC_NE_lexicon_tags_2 },
    { 0x4e1dc99, SUC_NE_lexicon_tags_2 },
    { 0xb289dc9a, SUC_NE_lexicon_tags_2 },
    { 0xd1a9dc9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0b1dc9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x47b7dc9f, SUC_NE_lexicon_tags_4 },
    { 0x249bdca0, SUC_NE_lexicon_tags_2 },
    { 0xc291dca1, SUC_NE_lexicon_tags_2 },
    { 0x1495dca2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd119dca6, SUC_NE_lexicon_tags_2 },
    { 0x64ddca6, SUC_NE_lexicon_tags_2 },
    { 0xc9a5dca8, SUC_NE_lexicon_tags_2 },
    { 0x8141dca9, SUC_NE_lexicon_tags_2 },
    { 0xb959dcaa, SUC_NE_lexicon_tags_2 },
    { 0xaf8ddcab, SUC_NE_lexicon_tags_2 },
    { 0xfdd1dcac, SUC_NE_lexicon_tags_2 },
    { 0x5179dcac, SUC_NE_lexicon_tags_2 },
    { 0xf939dcae, SUC_NE_lexicon_tags_2 },
    { 0x44c5dcaf, SUC_NE_lexicon_tags_4 },
    { 0xf4dfdcb0, SUC_NE_lexicon_tags_1 },
    { 0xd385dcaf, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dbbdcb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4633dcb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1ddcba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb0bdcbc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1471dcc4, SUC_NE_lexicon_tags_408 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a7dcc7, SUC_NE_lexicon_tags_2 },
    { 0x9d8fdcc8, SUC_NE_lexicon_tags_2 },
    { 0xb62ddcc9, SUC_NE_lexicon_tags_2 },
    { 0x95fddcca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe46ddccc, SUC_NE_lexicon_tags_2 },
    { 0xa705dccd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6359dccf, SUC_NE_lexicon_tags_2 },
    { 0xb69dcd0, SUC_NE_lexicon_tags_2 },
    { 0x475fdcd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee3dcdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7b1dcdd, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4657dce3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f97dce5, SUC_NE_lexicon_tags_2 },
    { 0x3743dce6, SUC_NE_lexicon_tags_2 },
    { 0xc5b9dce7, SUC_NE_lexicon_tags_2 },
    { 0x4727dce8, SUC_NE_lexicon_tags_2 },
    { 0x1da3dce9, SUC_NE_lexicon_tags_2 },
    { 0x3cf3dcea, SUC_NE_lexicon_tags_2 },
    { 0xa61fdceb, SUC_NE_lexicon_tags_2 },
    { 0xdb6bdce6, SUC_NE_lexicon_tags_2 },
    { 0x1413dcea, SUC_NE_lexicon_tags_2 },
    { 0x7b09dce9, SUC_NE_lexicon_tags_2 },
    { 0x77c3dcee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeabdcf1, SUC_NE_lexicon_tags_2 },
    { 0x3d3bdcf1, SUC_NE_lexicon_tags_24 },
    { 0xca1fdcf1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1c3dcf5, SUC_NE_lexicon_tags_2 },
    { 0x20f9dcf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c39dcf9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67a9dcfb, SUC_NE_lexicon_tags_2 },
    { 0x4b07dcfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2145dcfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4789dd02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6e9dd04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8ddd07, SUC_NE_lexicon_tags_2 },
    { 0x4b01dd08, SUC_NE_lexicon_tags_2 },
    { 0x96a9dd08, SUC_NE_lexicon_tags_2 },
    { 0xe82ddd09, SUC_NE_lexicon_tags_2 },
    { 0xb3fdd0b, SUC_NE_lexicon_tags_2 },
    { 0x8825dd0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb79dd13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9ddd17, SUC_NE_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdddd1a, SUC_NE_lexicon_tags_2 },
    { 0x7c71dd1a, SUC_NE_lexicon_tags_2 },
    { 0xb46bdd1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7533dd21, SUC_NE_lexicon_tags_17 },
    { 0xbef7dd22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4be1dd24, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4107dd27, SUC_NE_lexicon_tags_2 },
    { 0x94f5dd27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x741fdd2a, SUC_NE_lexicon_tags_2 },
    { 0xc5d3dd2b, SUC_NE_lexicon_tags_2 },
    { 0x7063dd2c, SUC_NE_lexicon_tags_1 },
    { 0x8f33dd2a, SUC_NE_lexicon_tags_2 },
    { 0xa013dd2a, SUC_NE_lexicon_tags_2 },
    { 0x4e07dd2d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb3dd31, SUC_NE_lexicon_tags_2 },
    { 0xc9abdd32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a1dd34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcddd38, SUC_NE_lexicon_tags_2 },
    { 0x24fdd39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc17dd3b, SUC_NE_lexicon_tags_2 },
    { 0x6c2fdd3c, SUC_NE_lexicon_tags_2 },
    { 0x7df3dd3b, SUC_NE_lexicon_tags_7 },
    { 0xc289dd3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf29dd43, SUC_NE_lexicon_tags_4 },
    { 0xd431dd43, SUC_NE_lexicon_tags_2 },
    { 0x51d1dd45, SUC_NE_lexicon_tags_2 },
    { 0x5b1bdd46, SUC_NE_lexicon_tags_188 },
    { 0x577bdd46, SUC_NE_lexicon_tags_2 },
    { 0x70d3dd46, SUC_NE_lexicon_tags_2 },
    { 0xe243dd49, SUC_NE_lexicon_tags_2 },
    { 0x5b13dd4a, SUC_NE_lexicon_tags_2 },
    { 0x6795dd4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfd5dd4d, SUC_NE_lexicon_tags_2 },
    { 0x86f9dd4d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cbdd54, SUC_NE_lexicon_tags_2 },
    { 0x3f3ddd55, SUC_NE_lexicon_tags_2 },
    { 0x881ddd56, SUC_NE_lexicon_tags_2 },
    { 0xed79dd55, SUC_NE_lexicon_tags_18 },
    { 0xe891dd55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b0bdd5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561dd5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f63dd61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e7dd64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8491dd6d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x772fdd6f, SUC_NE_lexicon_tags_2 },
    { 0x592bdd70, SUC_NE_lexicon_tags_2 },
    { 0x2607dd70, SUC_NE_lexicon_tags_2 },
    { 0x82c7dd72, SUC_NE_lexicon_tags_2 },
    { 0xc037dd72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7dd7e, SUC_NE_lexicon_tags_2 },
    { 0x7b33dd7e, SUC_NE_lexicon_tags_2 },
    { 0x787bdd80, SUC_NE_lexicon_tags_2 },
    { 0x5a8bdd81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1fddd83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4401dd86, SUC_NE_lexicon_tags_2 },
    { 0x89fdd87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca81dd8a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca9dd8d, SUC_NE_lexicon_tags_2 },
    { 0xe681dd8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b3dd94, SUC_NE_lexicon_tags_2 },
    { 0xa791dd95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6761dd99, SUC_NE_lexicon_tags_2 },
    { 0xe07fdd99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac5dda0, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa645dda7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebcbdda9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd721ddab, SUC_NE_lexicon_tags_2 },
    { 0x27abddac, SUC_NE_lexicon_tags_2 },
    { 0x1187ddab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cd7ddaf, SUC_NE_lexicon_tags_50 },
    { 0x66c3ddaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34cdddb2, SUC_NE_lexicon_tags_2 },
    { 0x9c9ddb3, SUC_NE_lexicon_tags_2 },
    { 0xdde1ddb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ac1ddb6, SUC_NE_lexicon_tags_2 },
    { 0xe05ddb7, SUC_NE_lexicon_tags_2 },
    { 0x7e0fddb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbddbd, SUC_NE_lexicon_tags_2 },
    { 0xbfa3ddbe, SUC_NE_lexicon_tags_7 },
    { 0x9e21ddbf, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8fddc2, SUC_NE_lexicon_tags_2 },
    { 0x539bddc2, SUC_NE_lexicon_tags_2 },
    { 0xd09dddc2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d29ddcc, SUC_NE_lexicon_tags_2 },
    { 0x6479ddcd, SUC_NE_lexicon_tags_2 },
    { 0x3b77ddce, SUC_NE_lexicon_tags_375 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bfddd2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bddddd6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d9dddf, SUC_NE_lexicon_tags_2 },
    { 0x8d7bdde0, SUC_NE_lexicon_tags_4 },
    { 0xb267dde1, SUC_NE_lexicon_tags_2 },
    { 0x818ddde2, SUC_NE_lexicon_tags_2 },
    { 0xb8c1dde3, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf25dde5, SUC_NE_lexicon_tags_2 },
    { 0x6b8ddde5, SUC_NE_lexicon_tags_80 },
    { 0xd4fbdde6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x628fdde9, SUC_NE_lexicon_tags_2 },
    { 0x7127ddea, SUC_NE_lexicon_tags_69 },
    { 0x5edfddeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x365dded, SUC_NE_lexicon_tags_47 },
    { 0x6b9ddee, SUC_NE_lexicon_tags_2 },
    { 0x3559dded, SUC_NE_lexicon_tags_2 },
    { 0x375dded, SUC_NE_lexicon_tags_2 },
    { 0x9957ddee, SUC_NE_lexicon_tags_2 },
    { 0xf517ddf2, SUC_NE_lexicon_tags_2 },
    { 0x305fddf3, SUC_NE_lexicon_tags_2 },
    { 0xf86fddf4, SUC_NE_lexicon_tags_2 },
    { 0x231fddf3, SUC_NE_lexicon_tags_7 },
    { 0x94a1ddf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf13bddf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc661ddfe, SUC_NE_lexicon_tags_2 },
    { 0x97e5ddfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b7de04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xad35de06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8905de0a, SUC_NE_lexicon_tags_2 },
    { 0x4093de0a, SUC_NE_lexicon_tags_2 },
    { 0xb611de0c, SUC_NE_lexicon_tags_2 },
    { 0xdbbdde0d, SUC_NE_lexicon_tags_2 },
    { 0x8d35de0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe837de10, SUC_NE_lexicon_tags_4 },
    { 0x15cfde11, SUC_NE_lexicon_tags_2 },
    { 0x86a3de12, SUC_NE_lexicon_tags_2 },
    { 0xeedde13, SUC_NE_lexicon_tags_2 },
    { 0xb8a3de14, SUC_NE_lexicon_tags_2 },
    { 0xe607de12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71bde17, SUC_NE_lexicon_tags_2 },
    { 0xa8d9de17, SUC_NE_lexicon_tags_2 },
    { 0x6fcfde19, SUC_NE_lexicon_tags_2 },
    { 0xcc0dde1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942bde1e, SUC_NE_lexicon_tags_2 },
    { 0x445de1f, SUC_NE_lexicon_tags_2 },
    { 0x8b11de1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdbde23, SUC_NE_lexicon_tags_2 },
    { 0xe2f9de24, SUC_NE_lexicon_tags_2 },
    { 0xfda5de25, SUC_NE_lexicon_tags_2 },
    { 0x19adde26, SUC_NE_lexicon_tags_2 },
    { 0xd311de27, SUC_NE_lexicon_tags_2 },
    { 0x92e3de23, SUC_NE_lexicon_tags_2 },
    { 0xd5ffde29, SUC_NE_lexicon_tags_17 },
    { 0x89f3de2a, SUC_NE_lexicon_tags_2 },
    { 0xbc39de2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x33c1de2d, SUC_NE_lexicon_tags_2 },
    { 0xeee5de2e, SUC_NE_lexicon_tags_2 },
    { 0xe1b9de2f, SUC_NE_lexicon_tags_2 },
    { 0x63e5de2d, SUC_NE_lexicon_tags_2 },
    { 0x393fde2f, SUC_NE_lexicon_tags_2 },
    { 0x1e95de32, SUC_NE_lexicon_tags_2 },
    { 0x2fd7de33, SUC_NE_lexicon_tags_2 },
    { 0x268fde2e, SUC_NE_lexicon_tags_2 },
    { 0xd0c9de35, SUC_NE_lexicon_tags_2 },
    { 0x363fde36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd171de38, SUC_NE_lexicon_tags_2 },
    { 0xba33de38, SUC_NE_lexicon_tags_2 },
    { 0xa341de3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfbde3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1697de42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91dfde45, SUC_NE_lexicon_tags_2 },
    { 0x4fe1de45, SUC_NE_lexicon_tags_2 },
    { 0x9cf3de45, SUC_NE_lexicon_tags_2 },
    { 0x9149de45, SUC_NE_lexicon_tags_2 },
    { 0x4633de49, SUC_NE_lexicon_tags_0 },
    { 0x57ddde4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5401de4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24dde52, SUC_NE_lexicon_tags_2 },
    { 0x5af3de53, SUC_NE_lexicon_tags_2 },
    { 0x6dc1de54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d5de5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c5de63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x955de65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc497de67, SUC_NE_lexicon_tags_2 },
    { 0xaf7dde68, SUC_NE_lexicon_tags_57 },
    { 0x1433de69, SUC_NE_lexicon_tags_2 },
    { 0x1e31de6a, SUC_NE_lexicon_tags_7 },
    { 0x1cfdde6a, SUC_NE_lexicon_tags_51 },
    { 0x56c7de6c, SUC_NE_lexicon_tags_2 },
    { 0x6e4fde6d, SUC_NE_lexicon_tags_2 },
    { 0x859fde6c, SUC_NE_lexicon_tags_2 },
    { 0x8d2dde6f, SUC_NE_lexicon_tags_2 },
    { 0x3ff7de6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x848bde72, SUC_NE_lexicon_tags_2 },
    { 0x6349de73, SUC_NE_lexicon_tags_2 },
    { 0xb051de74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7adde7a, SUC_NE_lexicon_tags_2 },
    { 0x1a59de7a, SUC_NE_lexicon_tags_2 },
    { 0x95afde7c, SUC_NE_lexicon_tags_2 },
    { 0xe9ffde7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb05de81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfdde86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cbde89, SUC_NE_lexicon_tags_7 },
    { 0xe5adde89, SUC_NE_lexicon_tags_18 },
    { 0x2ed9de8b, SUC_NE_lexicon_tags_2 },
    { 0x34c7de8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c8bde8e, SUC_NE_lexicon_tags_2 },
    { 0xe4d1de8e, SUC_NE_lexicon_tags_7 },
    { 0x600fde90, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb9de93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdde97, SUC_NE_lexicon_tags_2 },
    { 0x63ddde97, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fdde9d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe9dea0, SUC_NE_lexicon_tags_2 },
    { 0x5ff1dea1, SUC_NE_lexicon_tags_2 },
    { 0x12b7dea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb053dea5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fd5dea7, SUC_NE_lexicon_tags_2 },
    { 0x3e65dea7, SUC_NE_lexicon_tags_2 },
    { 0x51b5dea9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3a7deab, SUC_NE_lexicon_tags_2 },
    { 0x9cfbdeab, SUC_NE_lexicon_tags_9 },
    { 0x6293dead, SUC_NE_lexicon_tags_49 },
    { 0xa63bdeac, SUC_NE_lexicon_tags_5 },
    { 0x4107deaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88c5deb1, SUC_NE_lexicon_tags_2 },
    { 0xf5a7deb2, SUC_NE_lexicon_tags_2 },
    { 0x1fa7deb3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x89c7deb5, SUC_NE_lexicon_tags_2 },
    { 0xa981deb6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd903deb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa749debe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe167dec2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeba9dec4, SUC_NE_lexicon_tags_2 },
    { 0x8b1fdec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7743dec7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cfdece, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb675ded0, SUC_NE_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0xae51ded3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c3dee1, SUC_NE_lexicon_tags_2 },
    { 0x9167dee2, SUC_NE_lexicon_tags_36 },
    { 0xcef9dee2, SUC_NE_lexicon_tags_4 },
    { 0xa18bdee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f79dee7, SUC_NE_lexicon_tags_2 },
    { 0x8b21dee7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2481def0, SUC_NE_lexicon_tags_2 },
    { 0x21b3def1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abfdef6, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8579def8, SUC_NE_lexicon_tags_2 },
    { 0xe7e3def8, SUC_NE_lexicon_tags_2 },
    { 0x73c5def8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5645df04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb5df09, SUC_NE_lexicon_tags_5 },
    { 0x1ea3df09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2055df0f, SUC_NE_lexicon_tags_2 },
    { 0x28e5df10, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1bdf14, SUC_NE_lexicon_tags_2 },
    { 0x9cf1df15, SUC_NE_lexicon_tags_3 },
    { 0xe961df15, SUC_NE_lexicon_tags_2 },
    { 0x5b67df17, SUC_NE_lexicon_tags_2 },
    { 0xc7dddf18, SUC_NE_lexicon_tags_2 },
    { 0x9095df17, SUC_NE_lexicon_tags_4 },
    { 0xcaa9df1a, SUC_NE_lexicon_tags_2 },
    { 0xf567df1a, SUC_NE_lexicon_tags_2 },
    { 0x4f3ddf1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab7df22, SUC_NE_lexicon_tags_2 },
    { 0x2be3df23, SUC_NE_lexicon_tags_2 },
    { 0xf10fdf24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8495df26, SUC_NE_lexicon_tags_2 },
    { 0x37addf26, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cbdf2b, SUC_NE_lexicon_tags_1 },
    { 0xb669df2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68fdf30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c11df36, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bbdf39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa91df3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa805df3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7701df40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d45df45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6ddf49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f93df4b, SUC_NE_lexicon_tags_2 },
    { 0x209fdf4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd5df50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x154ddf52, SUC_NE_lexicon_tags_8 },
    { 0x19f9df53, SUC_NE_lexicon_tags_2 },
    { 0xba89df54, SUC_NE_lexicon_tags_2 },
    { 0x936bdf54, SUC_NE_lexicon_tags_2 },
    { 0x92b1df56, SUC_NE_lexicon_tags_2 },
    { 0x1631df55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f1df5c, SUC_NE_lexicon_tags_2 },
    { 0xcda9df5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ddf60, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x4653df62, SUC_NE_lexicon_tags_3 },
    { 0x5ae7df63, SUC_NE_lexicon_tags_1 },
    { 0xafc9df64, SUC_NE_lexicon_tags_17 },
    { 0x958ddf64, SUC_NE_lexicon_tags_2 },
    { 0x868ddf63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb517df6b, SUC_NE_lexicon_tags_2 },
    { 0x53c3df6c, SUC_NE_lexicon_tags_2 },
    { 0x1987df6c, SUC_NE_lexicon_tags_2 },
    { 0x2cfbdf6b, SUC_NE_lexicon_tags_2 },
    { 0x6f31df6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50f5df71, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbb29df73, SUC_NE_lexicon_tags_4 },
    { 0x9d21df74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e45df78, SUC_NE_lexicon_tags_3 },
    { 0x9405df79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea3df7d, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91df82, SUC_NE_lexicon_tags_2 },
    { 0x39bddf83, SUC_NE_lexicon_tags_3 },
    { 0x4ec7df84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d5df87, SUC_NE_lexicon_tags_24 },
    { 0xf489df87, SUC_NE_lexicon_tags_2 },
    { 0xdd31df88, SUC_NE_lexicon_tags_2 },
    { 0x6faddf8a, SUC_NE_lexicon_tags_2 },
    { 0x9809df87, SUC_NE_lexicon_tags_2 },
    { 0x1d0fdf8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42a5df8e, SUC_NE_lexicon_tags_2 },
    { 0x100ddf8f, SUC_NE_lexicon_tags_2 },
    { 0xbbf1df8f, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xfbbfdf92, SUC_NE_lexicon_tags_371 },
    { 0xc875df92, SUC_NE_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9353df98, SUC_NE_lexicon_tags_7 },
    { 0x1109df99, SUC_NE_lexicon_tags_2 },
    { 0x2065df9a, SUC_NE_lexicon_tags_4 },
    { 0x9e59df99, SUC_NE_lexicon_tags_40 },
    { 0xe67bdf9c, SUC_NE_lexicon_tags_2 },
    { 0x2dafdf9c, SUC_NE_lexicon_tags_2 },
    { 0x882bdf98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f1dfa5, SUC_NE_lexicon_tags_2 },
    { 0xc09bdfa5, SUC_NE_lexicon_tags_2 },
    { 0x3ab1dfa7, SUC_NE_lexicon_tags_2 },
    { 0x74e1dfa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d7dfac, SUC_NE_lexicon_tags_7 },
    { 0x83dbdfac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43dbdfaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb51dfb1, SUC_NE_lexicon_tags_2 },
    { 0xe75dfb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc071dfb8, SUC_NE_lexicon_tags_2 },
    { 0x61b7dfb9, SUC_NE_lexicon_tags_1 },
    { 0x57efdfba, SUC_NE_lexicon_tags_2 },
    { 0x8af1dfba, SUC_NE_lexicon_tags_2 },
    { 0x4bfbdfbc, SUC_NE_lexicon_tags_2 },
    { 0xa64ddfbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f5dfbf, SUC_NE_lexicon_tags_2 },
    { 0xfc67dfbf, SUC_NE_lexicon_tags_2 },
    { 0x3b9bdfc1, SUC_NE_lexicon_tags_32 },
    { 0x3ec7dfc1, SUC_NE_lexicon_tags_0 },
    { 0x1749dfbf, SUC_NE_lexicon_tags_4 },
    { 0x5865dfc4, SUC_NE_lexicon_tags_2 },
    { 0xeb17dfc5, SUC_NE_lexicon_tags_2 },
    { 0x8d77dfc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5211dfcd, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe149dfd0, SUC_NE_lexicon_tags_2 },
    { 0xf13fdfd0, SUC_NE_lexicon_tags_1 },
    { 0xcf5fdfd2, SUC_NE_lexicon_tags_2 },
    { 0x5177dfd3, SUC_NE_lexicon_tags_2 },
    { 0xb975dfd4, SUC_NE_lexicon_tags_2 },
    { 0x8ec5dfd5, SUC_NE_lexicon_tags_2 },
    { 0xa5c5dfd4, SUC_NE_lexicon_tags_2 },
    { 0x94e9dfd2, SUC_NE_lexicon_tags_2 },
    { 0x206fdfd2, SUC_NE_lexicon_tags_2 },
    { 0x5c21dfd9, SUC_NE_lexicon_tags_40 },
    { 0xb381dfd1, SUC_NE_lexicon_tags_4 },
    { 0x7941dfd0, SUC_NE_lexicon_tags_104 },
    { 0x3071dfdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538fdfe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa92ddfe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc13dfe4, SUC_NE_lexicon_tags_2 },
    { 0x4411dfe5, SUC_NE_lexicon_tags_2 },
    { 0xa7fddfe6, SUC_NE_lexicon_tags_2 },
    { 0xfd7ddfe5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e5dfea, SUC_NE_lexicon_tags_2 },
    { 0x81c9dfeb, SUC_NE_lexicon_tags_7 },
    { 0x112ddfea, SUC_NE_lexicon_tags_2 },
    { 0x69dbdfed, SUC_NE_lexicon_tags_4 },
    { 0xbb43dfea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b7dff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ddff5, SUC_NE_lexicon_tags_2 },
    { 0xd663dff5, SUC_NE_lexicon_tags_2 },
    { 0x5a7bdff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ba3dff9, SUC_NE_lexicon_tags_2 },
    { 0xf71bdffa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8fdffc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bfdfff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeede003, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2d7e005, SUC_NE_lexicon_tags_2 },
    { 0x9525e006, SUC_NE_lexicon_tags_0 },
    { 0x5733e007, SUC_NE_lexicon_tags_2 },
    { 0xc25fe006, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0xd64fe00a, SUC_NE_lexicon_tags_2 },
    { 0xb79fe00a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dfe011, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb9e017, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0xce0de019, SUC_NE_lexicon_tags_2 },
    { 0x413e019, SUC_NE_lexicon_tags_2 },
    { 0xb77e01b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa507e01d, SUC_NE_lexicon_tags_2 },
    { 0xb287e01d, SUC_NE_lexicon_tags_2 },
    { 0x45d3e01f, SUC_NE_lexicon_tags_2 },
    { 0x1505e01f, SUC_NE_lexicon_tags_2 },
    { 0x7e03e021, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5de025, SUC_NE_lexicon_tags_2 },
    { 0x4b2be026, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f47e029, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccabe02e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7851e030, SUC_NE_lexicon_tags_2 },
    { 0x97fe031, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bde03e, SUC_NE_lexicon_tags_2 },
    { 0xff03e03f, SUC_NE_lexicon_tags_96 },
    { 0xd5fbe040, SUC_NE_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31fe043, SUC_NE_lexicon_tags_2 },
    { 0x27c7e044, SUC_NE_lexicon_tags_2 },
    { 0xff83e044, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2959e048, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1515e051, SUC_NE_lexicon_tags_2 },
    { 0x1965e051, SUC_NE_lexicon_tags_2 },
    { 0xc987e053, SUC_NE_lexicon_tags_2 },
    { 0xb83be053, SUC_NE_lexicon_tags_2 },
    { 0xb943e055, SUC_NE_lexicon_tags_2 },
    { 0x6bdbe056, SUC_NE_lexicon_tags_2 },
    { 0xe457e053, SUC_NE_lexicon_tags_2 },
    { 0x7f99e053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd903e05e, SUC_NE_lexicon_tags_2 },
    { 0xfbe05f, SUC_NE_lexicon_tags_2 },
    { 0xda7fe060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f7e063, SUC_NE_lexicon_tags_186 },
    { 0, NULL },
    { 0xdbe1e065, SUC_NE_lexicon_tags_2 },
    { 0xa9b1e066, SUC_NE_lexicon_tags_2 },
    { 0x998fe066, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e7e06a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddafe06d, SUC_NE_lexicon_tags_19 },
    { 0x3099e06e, SUC_NE_lexicon_tags_2 },
    { 0x888fe06f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x833be071, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a4be073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2909e077, SUC_NE_lexicon_tags_2 },
    { 0xe209e077, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2837e07c, SUC_NE_lexicon_tags_1 },
    { 0xb49e07d, SUC_NE_lexicon_tags_2 },
    { 0x217e07c, SUC_NE_lexicon_tags_2 },
    { 0x6bc1e07f, SUC_NE_lexicon_tags_2 },
    { 0x57ade07f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6635e084, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5141e08a, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a1e093, SUC_NE_lexicon_tags_2 },
    { 0x840de093, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf153e097, SUC_NE_lexicon_tags_201 },
    { 0x59fde098, SUC_NE_lexicon_tags_2 },
    { 0x688de099, SUC_NE_lexicon_tags_2 },
    { 0xe415e09a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd94be09c, SUC_NE_lexicon_tags_2 },
    { 0x921e09d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140be0a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e33e0a5, SUC_NE_lexicon_tags_2 },
    { 0x24f3e0a6, SUC_NE_lexicon_tags_2 },
    { 0x2437e0a6, SUC_NE_lexicon_tags_7 },
    { 0x5fe0a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x257de0ab, SUC_NE_lexicon_tags_7 },
    { 0xe01de0ac, SUC_NE_lexicon_tags_2 },
    { 0xbfe3e0ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a77e0b0, SUC_NE_lexicon_tags_2 },
    { 0x2d25e0b1, SUC_NE_lexicon_tags_2 },
    { 0xb853e0b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bcbe0b4, SUC_NE_lexicon_tags_4 },
    { 0xcc89e0b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82fe0b8, SUC_NE_lexicon_tags_2 },
    { 0xfb39e0b9, SUC_NE_lexicon_tags_2 },
    { 0xa34be0b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6293e0bc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb991e0bf, SUC_NE_lexicon_tags_2 },
    { 0x4c17e0bf, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7fe0c3, SUC_NE_lexicon_tags_2 },
    { 0xa24be0c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb3e0c9, SUC_NE_lexicon_tags_1 },
    { 0x338be0c9, SUC_NE_lexicon_tags_40 },
    { 0x85b9e0cb, SUC_NE_lexicon_tags_2 },
    { 0x9fa7e0cc, SUC_NE_lexicon_tags_2 },
    { 0xdfefe0cb, SUC_NE_lexicon_tags_2 },
    { 0xe135e0cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9237e0d0, SUC_NE_lexicon_tags_2 },
    { 0xa761e0d0, SUC_NE_lexicon_tags_0 },
    { 0xf599e0d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d3e0d8, SUC_NE_lexicon_tags_2 },
    { 0x5e79e0d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x765be0dc, SUC_NE_lexicon_tags_19 },
    { 0xf23e0dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe521e0df, SUC_NE_lexicon_tags_2 },
    { 0x7469e0df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x20f1e0e2, SUC_NE_lexicon_tags_2 },
    { 0x7d97e0e3, SUC_NE_lexicon_tags_2 },
    { 0xb4be0e4, SUC_NE_lexicon_tags_2 },
    { 0xa527e0e4, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2129e0e7, SUC_NE_lexicon_tags_2 },
    { 0xdf25e0e8, SUC_NE_lexicon_tags_2 },
    { 0x1d97e0e8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe99e0eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2011e0f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xccbfe0f5, SUC_NE_lexicon_tags_4 },
    { 0x4a07e0f6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbec5e0f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49fe0fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2b3e0fe, SUC_NE_lexicon_tags_2 },
    { 0x89de0ff, SUC_NE_lexicon_tags_2 },
    { 0x2243e100, SUC_NE_lexicon_tags_2 },
    { 0x3d61e101, SUC_NE_lexicon_tags_2 },
    { 0xc3cbe102, SUC_NE_lexicon_tags_2 },
    { 0x1007e101, SUC_NE_lexicon_tags_2 },
    { 0x6861e102, SUC_NE_lexicon_tags_3 },
    { 0x4849e104, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2fe109, SUC_NE_lexicon_tags_2 },
    { 0x9179e10a, SUC_NE_lexicon_tags_2 },
    { 0xb099e10b, SUC_NE_lexicon_tags_2 },
    { 0x645de10b, SUC_NE_lexicon_tags_2 },
    { 0xcc3fe10d, SUC_NE_lexicon_tags_2 },
    { 0xfb3de10c, SUC_NE_lexicon_tags_2 },
    { 0x419be10f, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0x9f11e111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8be114, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x346fe116, SUC_NE_lexicon_tags_2 },
    { 0x54a3e117, SUC_NE_lexicon_tags_2 },
    { 0x4035e118, SUC_NE_lexicon_tags_2 },
    { 0x66e5e119, SUC_NE_lexicon_tags_2 },
    { 0x748fe11a, SUC_NE_lexicon_tags_2 },
    { 0x8bebe11a, SUC_NE_lexicon_tags_2 },
    { 0xb14de11c, SUC_NE_lexicon_tags_2 },
    { 0x2a1be119, SUC_NE_lexicon_tags_2 },
    { 0xcf2fe11d, SUC_NE_lexicon_tags_2 },
    { 0xbffde119, SUC_NE_lexicon_tags_2 },
    { 0x3ac9e120, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f9e124, SUC_NE_lexicon_tags_2 },
    { 0xb0efe125, SUC_NE_lexicon_tags_2 },
    { 0xa8dfe124, SUC_NE_lexicon_tags_2 },
    { 0x5837e127, SUC_NE_lexicon_tags_2 },
    { 0xc0be128, SUC_NE_lexicon_tags_2 },
    { 0xe203e127, SUC_NE_lexicon_tags_2 },
    { 0xee99e12a, SUC_NE_lexicon_tags_2 },
    { 0x12b1e12b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f45e12d, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde7e130, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6de134, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5de138, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae49e141, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9369e144, SUC_NE_lexicon_tags_2 },
    { 0xa073e144, SUC_NE_lexicon_tags_4 },
    { 0xeddbe146, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe057e149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x64d7e14b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e5e14e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xcb5be150, SUC_NE_lexicon_tags_2 },
    { 0x71f3e151, SUC_NE_lexicon_tags_2 },
    { 0xfa27e151, SUC_NE_lexicon_tags_2 },
    { 0x4471e152, SUC_NE_lexicon_tags_2 },
    { 0x7547e154, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a01e158, SUC_NE_lexicon_tags_2 },
    { 0x7b17e158, SUC_NE_lexicon_tags_2 },
    { 0x4537e15a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf199e15e, SUC_NE_lexicon_tags_2 },
    { 0x33b9e15f, SUC_NE_lexicon_tags_2 },
    { 0xbad7e15f, SUC_NE_lexicon_tags_2 },
    { 0x4e49e161, SUC_NE_lexicon_tags_2 },
    { 0x7233e162, SUC_NE_lexicon_tags_2 },
    { 0x748fe163, SUC_NE_lexicon_tags_2 },
    { 0x82cbe163, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xded5e166, SUC_NE_lexicon_tags_2 },
    { 0xb3f3e167, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6be16a, SUC_NE_lexicon_tags_2 },
    { 0x4245e16a, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3de16e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d1be170, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0x599de172, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x961be175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x309be177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x67fe179, SUC_NE_lexicon_tags_2 },
    { 0x746fe17a, SUC_NE_lexicon_tags_2 },
    { 0x8605e17b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabede182, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x955e185, SUC_NE_lexicon_tags_2 },
    { 0x1483e186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e9e18a, SUC_NE_lexicon_tags_2 },
    { 0xb4c5e18a, SUC_NE_lexicon_tags_2 },
    { 0xe7d3e18c, SUC_NE_lexicon_tags_2 },
    { 0xfa0be18d, SUC_NE_lexicon_tags_2 },
    { 0xe483e18b, SUC_NE_lexicon_tags_369 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71e192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6de195, SUC_NE_lexicon_tags_2 },
    { 0xb605e196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cf1e198, SUC_NE_lexicon_tags_2 },
    { 0x5ab5e198, SUC_NE_lexicon_tags_2 },
    { 0xf69de199, SUC_NE_lexicon_tags_2 },
    { 0x29be198, SUC_NE_lexicon_tags_2 },
    { 0x58abe19a, SUC_NE_lexicon_tags_2 },
    { 0x28d7e19d, SUC_NE_lexicon_tags_4 },
    { 0x6313e19e, SUC_NE_lexicon_tags_2 },
    { 0xe6a7e19f, SUC_NE_lexicon_tags_2 },
    { 0xf043e1a0, SUC_NE_lexicon_tags_2 },
    { 0xe48be1a1, SUC_NE_lexicon_tags_19 },
    { 0x2543e1a2, SUC_NE_lexicon_tags_2 },
    { 0xe015e1a3, SUC_NE_lexicon_tags_2 },
    { 0x9f13e1a0, SUC_NE_lexicon_tags_2 },
    { 0xcb6fe19f, SUC_NE_lexicon_tags_2 },
    { 0x522de1a6, SUC_NE_lexicon_tags_2 },
    { 0x91c9e1a7, SUC_NE_lexicon_tags_2 },
    { 0x2901e1a8, SUC_NE_lexicon_tags_2 },
    { 0x855be1a8, SUC_NE_lexicon_tags_2 },
    { 0xc92fe1a8, SUC_NE_lexicon_tags_2 },
    { 0xac25e1a7, SUC_NE_lexicon_tags_2 },
    { 0xd75de1ac, SUC_NE_lexicon_tags_2 },
    { 0xf1d5e1ad, SUC_NE_lexicon_tags_2 },
    { 0xdd7de1ac, SUC_NE_lexicon_tags_0 },
    { 0x80efe1af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x416de1b2, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcbe1b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac9e1ba, SUC_NE_lexicon_tags_2 },
    { 0x218fe1ba, SUC_NE_lexicon_tags_2 },
    { 0xc211e1ba, SUC_NE_lexicon_tags_7 },
    { 0xfb8be1bd, SUC_NE_lexicon_tags_2 },
    { 0x7f97e1bd, SUC_NE_lexicon_tags_2 },
    { 0x794de1be, SUC_NE_lexicon_tags_2 },
    { 0xcd4be1bf, SUC_NE_lexicon_tags_2 },
    { 0xc647e1be, SUC_NE_lexicon_tags_2 },
    { 0xa79e1c0, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xbd57e1c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf745e1c7, SUC_NE_lexicon_tags_2 },
    { 0xe58fe1c8, SUC_NE_lexicon_tags_2 },
    { 0x7d01e1c9, SUC_NE_lexicon_tags_2 },
    { 0x13b7e1ca, SUC_NE_lexicon_tags_2 },
    { 0x6425e1cb, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x95fbe1cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dbe1d0, SUC_NE_lexicon_tags_2 },
    { 0x755be1d1, SUC_NE_lexicon_tags_2 },
    { 0xecdde1d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf635e1d6, SUC_NE_lexicon_tags_39 },
    { 0xc01de1d7, SUC_NE_lexicon_tags_2 },
    { 0xd9e1d8, SUC_NE_lexicon_tags_5 },
    { 0x9ebe1d9, SUC_NE_lexicon_tags_2 },
    { 0x10d1e1d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42fe1df, SUC_NE_lexicon_tags_2 },
    { 0x804de1e0, SUC_NE_lexicon_tags_2 },
    { 0xfa9be1e1, SUC_NE_lexicon_tags_2 },
    { 0x794fe1e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58e5e1e4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d3e1e9, SUC_NE_lexicon_tags_2 },
    { 0x5615e1e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e73e1ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf61e1ef, SUC_NE_lexicon_tags_3 },
    { 0x3f9be1f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7e1f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7989e1fb, SUC_NE_lexicon_tags_2 },
    { 0x5f67e1fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c5e1ff, SUC_NE_lexicon_tags_2 },
    { 0x5295e1ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec97e207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef61e20b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4277e20d, SUC_NE_lexicon_tags_14 },
    { 0x2e21e20d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da7e210, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1723e212, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3e215, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71e218, SUC_NE_lexicon_tags_2 },
    { 0xcd05e218, SUC_NE_lexicon_tags_286 },
    { 0x73e9e21a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd51e222, SUC_NE_lexicon_tags_2 },
    { 0xceb7e222, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c5e227, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54fe22a, SUC_NE_lexicon_tags_2 },
    { 0x2be3e22a, SUC_NE_lexicon_tags_2 },
    { 0x9639e22c, SUC_NE_lexicon_tags_2 },
    { 0x19d5e22d, SUC_NE_lexicon_tags_2 },
    { 0xe059e22b, SUC_NE_lexicon_tags_2 },
    { 0x2ff9e22f, SUC_NE_lexicon_tags_2 },
    { 0x924de22f, SUC_NE_lexicon_tags_2 },
    { 0x8e5fe231, SUC_NE_lexicon_tags_1 },
    { 0x9d91e232, SUC_NE_lexicon_tags_2 },
    { 0x1e65e22f, SUC_NE_lexicon_tags_2 },
    { 0x370be234, SUC_NE_lexicon_tags_2 },
    { 0x46fde22f, SUC_NE_lexicon_tags_2 },
    { 0x3cf7e231, SUC_NE_lexicon_tags_3 },
    { 0x9637e237, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b89e23a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe23e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8191e242, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xcba5e244, SUC_NE_lexicon_tags_2 },
    { 0x5fb9e244, SUC_NE_lexicon_tags_2 },
    { 0x1f6be246, SUC_NE_lexicon_tags_2 },
    { 0x56b7e246, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d11e24a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db1e24f, SUC_NE_lexicon_tags_2 },
    { 0x7f77e24f, SUC_NE_lexicon_tags_2 },
    { 0xb6c3e250, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b77e255, SUC_NE_lexicon_tags_2 },
    { 0xf1afe256, SUC_NE_lexicon_tags_2 },
    { 0x4fe5e257, SUC_NE_lexicon_tags_2 },
    { 0xeb0fe256, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x177e25a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69fe25f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fe267, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3265e26a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca73e26c, SUC_NE_lexicon_tags_2 },
    { 0x5c39e26d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c1e270, SUC_NE_lexicon_tags_2 },
    { 0x3419e271, SUC_NE_lexicon_tags_2 },
    { 0xff09e270, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x7ce3e274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1541e27c, SUC_NE_lexicon_tags_2 },
    { 0x53ffe27d, SUC_NE_lexicon_tags_2 },
    { 0x52c9e27d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1e282, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9e28b, SUC_NE_lexicon_tags_2 },
    { 0x2b17e28c, SUC_NE_lexicon_tags_2 },
    { 0x1be28b, SUC_NE_lexicon_tags_2 },
    { 0x21a3e28e, SUC_NE_lexicon_tags_2 },
    { 0xe4c1e28e, SUC_NE_lexicon_tags_25 },
    { 0xbb13e290, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e79e296, SUC_NE_lexicon_tags_2 },
    { 0xefafe296, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1471e29e, SUC_NE_lexicon_tags_2 },
    { 0xc05fe29f, SUC_NE_lexicon_tags_2 },
    { 0x5a05e2a0, SUC_NE_lexicon_tags_2 },
    { 0xd617e2a1, SUC_NE_lexicon_tags_2 },
    { 0xd749e2a1, SUC_NE_lexicon_tags_3 },
    { 0x40ffe2a3, SUC_NE_lexicon_tags_2 },
    { 0x3c35e2a1, SUC_NE_lexicon_tags_2 },
    { 0xba41e2a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa56be2a7, SUC_NE_lexicon_tags_2 },
    { 0xf073e2a8, SUC_NE_lexicon_tags_32 },
    { 0x3a45e2a9, SUC_NE_lexicon_tags_2 },
    { 0x862fe2a8, SUC_NE_lexicon_tags_2 },
    { 0xd4be2aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xee1fe2ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a9e2b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2769e2b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290fe2b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dbe2c0, SUC_NE_lexicon_tags_7 },
    { 0xf6d1e2c1, SUC_NE_lexicon_tags_2 },
    { 0xb497e2c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea27e2c9, SUC_NE_lexicon_tags_2 },
    { 0x543fe2c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc873e2cd, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fbe2d2, SUC_NE_lexicon_tags_2 },
    { 0x9ccbe2d3, SUC_NE_lexicon_tags_2 },
    { 0x9b11e2d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c5e2d7, SUC_NE_lexicon_tags_4 },
    { 0x6ae7e2d8, SUC_NE_lexicon_tags_2 },
    { 0xf823e2d9, SUC_NE_lexicon_tags_2 },
    { 0x34f9e2d7, SUC_NE_lexicon_tags_32 },
    { 0xcede2d7, SUC_NE_lexicon_tags_2 },
    { 0x708de2dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6de2df, SUC_NE_lexicon_tags_2 },
    { 0x7947e2e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c87e2e3, SUC_NE_lexicon_tags_2 },
    { 0xf4e1e2e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2753e2e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd44be2e8, SUC_NE_lexicon_tags_4 },
    { 0xc16fe2e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7821e2ef, SUC_NE_lexicon_tags_2 },
    { 0x36f3e2f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x967fe2f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58ffe2f4, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1115e2fb, SUC_NE_lexicon_tags_2 },
    { 0x6b2fe2fc, SUC_NE_lexicon_tags_2 },
    { 0xa8dbe2fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc71de2ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf341e302, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fbe305, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9fe30a, SUC_NE_lexicon_tags_1 },
    { 0x4b3de30a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xef59e30d, SUC_NE_lexicon_tags_2 },
    { 0x5cc7e30e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7de314, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e7e31f, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3fe325, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c37e32c, SUC_NE_lexicon_tags_2 },
    { 0xc325e32c, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a05e331, SUC_NE_lexicon_tags_2 },
    { 0x756be331, SUC_NE_lexicon_tags_7 },
    { 0x9c1be333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4afe335, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3e339, SUC_NE_lexicon_tags_18 },
    { 0x7b4fe33a, SUC_NE_lexicon_tags_2 },
    { 0xaaafe33b, SUC_NE_lexicon_tags_0 },
    { 0x127be33c, SUC_NE_lexicon_tags_2 },
    { 0x29f7e33c, SUC_NE_lexicon_tags_2 },
    { 0x6543e33c, SUC_NE_lexicon_tags_2 },
    { 0x6f5e33e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa149e348, SUC_NE_lexicon_tags_2 },
    { 0x135e349, SUC_NE_lexicon_tags_4 },
    { 0xc0a5e34a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3e34d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec15e34f, SUC_NE_lexicon_tags_270 },
    { 0x35efe350, SUC_NE_lexicon_tags_2 },
    { 0xe5e1e351, SUC_NE_lexicon_tags_2 },
    { 0x1695e351, SUC_NE_lexicon_tags_2 },
    { 0xccdfe353, SUC_NE_lexicon_tags_2 },
    { 0x4e8de353, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1181e357, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83efe359, SUC_NE_lexicon_tags_2 },
    { 0xec2de35a, SUC_NE_lexicon_tags_2 },
    { 0xf2de35b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa523e35f, SUC_NE_lexicon_tags_2 },
    { 0x4ff3e360, SUC_NE_lexicon_tags_2 },
    { 0xe07fe360, SUC_NE_lexicon_tags_2 },
    { 0x8d59e360, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ebe368, SUC_NE_lexicon_tags_2 },
    { 0x849be368, SUC_NE_lexicon_tags_2 },
    { 0x80ffe36a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd1e36e, SUC_NE_lexicon_tags_2 },
    { 0x4be7e36f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285de372, SUC_NE_lexicon_tags_7 },
    { 0x20bfe372, SUC_NE_lexicon_tags_2 },
    { 0x1c69e374, SUC_NE_lexicon_tags_2 },
    { 0x5f0be372, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e6be377, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd265e37b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2511e37f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65be382, SUC_NE_lexicon_tags_18 },
    { 0x574fe383, SUC_NE_lexicon_tags_7 },
    { 0x9e7e384, SUC_NE_lexicon_tags_2 },
    { 0x177e385, SUC_NE_lexicon_tags_2 },
    { 0xf47be386, SUC_NE_lexicon_tags_2 },
    { 0xbcf5e387, SUC_NE_lexicon_tags_1 },
    { 0x618be388, SUC_NE_lexicon_tags_2 },
    { 0x5cafe388, SUC_NE_lexicon_tags_2 },
    { 0x4cd5e38a, SUC_NE_lexicon_tags_2 },
    { 0x28bde389, SUC_NE_lexicon_tags_2 },
    { 0x3683e387, SUC_NE_lexicon_tags_2 },
    { 0x204fe387, SUC_NE_lexicon_tags_2 },
    { 0x3effe38e, SUC_NE_lexicon_tags_2 },
    { 0x4063e387, SUC_NE_lexicon_tags_2 },
    { 0xd685e386, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13d1e392, SUC_NE_lexicon_tags_2 },
    { 0xab63e393, SUC_NE_lexicon_tags_2 },
    { 0x6e51e394, SUC_NE_lexicon_tags_2 },
    { 0xc3e3e393, SUC_NE_lexicon_tags_2 },
    { 0xd3afe394, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x256be399, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x185e39b, SUC_NE_lexicon_tags_4 },
    { 0x39b9e39c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3501e39e, SUC_NE_lexicon_tags_50 },
    { 0, NULL },
    { 0x3e5e3a0, SUC_NE_lexicon_tags_7 },
    { 0xa7bfe3a1, SUC_NE_lexicon_tags_2 },
    { 0xce27e3a2, SUC_NE_lexicon_tags_2 },
    { 0xa43fe3a2, SUC_NE_lexicon_tags_2 },
    { 0x4521e3a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f11e3a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cfe3aa, SUC_NE_lexicon_tags_2 },
    { 0x88d9e3aa, SUC_NE_lexicon_tags_2 },
    { 0xee3be3aa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc7e9e3ae, SUC_NE_lexicon_tags_2 },
    { 0x79f9e3ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421e3b9, SUC_NE_lexicon_tags_2 },
    { 0x2411e3ba, SUC_NE_lexicon_tags_2 },
    { 0xcf2fe3bb, SUC_NE_lexicon_tags_2 },
    { 0x399fe3bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dfe3c3, SUC_NE_lexicon_tags_4 },
    { 0x536be3c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcaa1e3c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x143fe3c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d9e3cb, SUC_NE_lexicon_tags_2 },
    { 0x1fede3cc, SUC_NE_lexicon_tags_2 },
    { 0x7d83e3cd, SUC_NE_lexicon_tags_2 },
    { 0x158fe3ce, SUC_NE_lexicon_tags_2 },
    { 0x81f3e3ce, SUC_NE_lexicon_tags_2 },
    { 0x51cfe3cf, SUC_NE_lexicon_tags_2 },
    { 0x564de3d1, SUC_NE_lexicon_tags_2 },
    { 0x5b31e3d0, SUC_NE_lexicon_tags_2 },
    { 0xc5b3e3cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530be3d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b69e3dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee59e3df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3be3e2, SUC_NE_lexicon_tags_1 },
    { 0xf74fe3e3, SUC_NE_lexicon_tags_2 },
    { 0x903be3e4, SUC_NE_lexicon_tags_34 },
    { 0xb95e3e5, SUC_NE_lexicon_tags_2 },
    { 0xb9a1e3e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d73e3e8, SUC_NE_lexicon_tags_2 },
    { 0xcc6be3e9, SUC_NE_lexicon_tags_1 },
    { 0x88be3ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af9e3ef, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17de3f3, SUC_NE_lexicon_tags_2 },
    { 0x8d4de3f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b95e3f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2433e400, SUC_NE_lexicon_tags_7 },
    { 0xab57e401, SUC_NE_lexicon_tags_2 },
    { 0x60de401, SUC_NE_lexicon_tags_2 },
    { 0xb68de403, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f13e406, SUC_NE_lexicon_tags_2 },
    { 0x6603e406, SUC_NE_lexicon_tags_2 },
    { 0x3089e408, SUC_NE_lexicon_tags_2 },
    { 0x44d1e408, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3e40c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d5e40f, SUC_NE_lexicon_tags_3 },
    { 0xd6b1e410, SUC_NE_lexicon_tags_2 },
    { 0x44de411, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe841e413, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe953e415, SUC_NE_lexicon_tags_2 },
    { 0x55f3e416, SUC_NE_lexicon_tags_2 },
    { 0xedc1e417, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76be41f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10b7e421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb585e426, SUC_NE_lexicon_tags_2 },
    { 0xe0ebe427, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a3e430, SUC_NE_lexicon_tags_23 },
    { 0xee1fe431, SUC_NE_lexicon_tags_2 },
    { 0xd431e432, SUC_NE_lexicon_tags_2 },
    { 0xbee3e431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x219fe435, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa25e437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x207de439, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf885e43c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cfe43f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8531e441, SUC_NE_lexicon_tags_2 },
    { 0x5677e442, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad57e44a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7fe450, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b35e452, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb95e454, SUC_NE_lexicon_tags_2 },
    { 0xc421e455, SUC_NE_lexicon_tags_2 },
    { 0x833fe456, SUC_NE_lexicon_tags_2 },
    { 0x6e8fe455, SUC_NE_lexicon_tags_2 },
    { 0x1451e458, SUC_NE_lexicon_tags_2 },
    { 0xb01e456, SUC_NE_lexicon_tags_2 },
    { 0x3ed7e458, SUC_NE_lexicon_tags_2 },
    { 0x9a0be45b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71afe460, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b5e468, SUC_NE_lexicon_tags_2 },
    { 0xabcfe468, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8583e46c, SUC_NE_lexicon_tags_2 },
    { 0xd6b3e46d, SUC_NE_lexicon_tags_2 },
    { 0xb08be46e, SUC_NE_lexicon_tags_2 },
    { 0xda9de46f, SUC_NE_lexicon_tags_2 },
    { 0xab9e470, SUC_NE_lexicon_tags_2 },
    { 0x8f35e46f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03de476, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6993e478, SUC_NE_lexicon_tags_3 },
    { 0xbf3de478, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac7e47c, SUC_NE_lexicon_tags_2 },
    { 0x3cb1e47d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50e1e47f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0efe483, SUC_NE_lexicon_tags_2 },
    { 0x7f13e483, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51fe489, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5891e48e, SUC_NE_lexicon_tags_2 },
    { 0x4e47e48f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4fe494, SUC_NE_lexicon_tags_2 },
    { 0xa541e495, SUC_NE_lexicon_tags_2 },
    { 0x79ade496, SUC_NE_lexicon_tags_2 },
    { 0x314be497, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeea9e499, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9355e49f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8cde4a1, SUC_NE_lexicon_tags_7 },
    { 0x304be4a2, SUC_NE_lexicon_tags_7 },
    { 0xb009e4a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf815e4a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6995e4ab, SUC_NE_lexicon_tags_2 },
    { 0x1905e4ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbbe4af, SUC_NE_lexicon_tags_2 },
    { 0xc281e4b0, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x55dde4b3, SUC_NE_lexicon_tags_7 },
    { 0xb3ffe4b4, SUC_NE_lexicon_tags_2 },
    { 0x9297e4b5, SUC_NE_lexicon_tags_2 },
    { 0x1715e4b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc74fe4b8, SUC_NE_lexicon_tags_2 },
    { 0x618de4b9, SUC_NE_lexicon_tags_0 },
    { 0x1a1e4ba, SUC_NE_lexicon_tags_2 },
    { 0x8625e4bb, SUC_NE_lexicon_tags_2 },
    { 0x8f47e4bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9477e4c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2be4c5, SUC_NE_lexicon_tags_34 },
    { 0xe465e4c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb421e4cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca73e4d2, SUC_NE_lexicon_tags_19 },
    { 0x8059e4d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd579e4d5, SUC_NE_lexicon_tags_2 },
    { 0x5ba9e4d6, SUC_NE_lexicon_tags_2 },
    { 0xf9b7e4d7, SUC_NE_lexicon_tags_2 },
    { 0x6421e4d8, SUC_NE_lexicon_tags_1 },
    { 0x5367e4d9, SUC_NE_lexicon_tags_2 },
    { 0xea9de4da, SUC_NE_lexicon_tags_2 },
    { 0xfaffe4d9, SUC_NE_lexicon_tags_2 },
    { 0x22cde4db, SUC_NE_lexicon_tags_2 },
    { 0xcdbfe4d9, SUC_NE_lexicon_tags_2 },
    { 0x5a8fe4de, SUC_NE_lexicon_tags_2 },
    { 0x29d3e4de, SUC_NE_lexicon_tags_2 },
    { 0x6eafe4e0, SUC_NE_lexicon_tags_2 },
    { 0xd045e4e1, SUC_NE_lexicon_tags_2 },
    { 0x79bfe4dd, SUC_NE_lexicon_tags_2 },
    { 0x7263e4e3, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x40e3e4e5, SUC_NE_lexicon_tags_2 },
    { 0xee0de4e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c7e4f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xac5fe4f3, SUC_NE_lexicon_tags_2 },
    { 0x984be4f4, SUC_NE_lexicon_tags_2 },
    { 0x6549e4f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d73e4f7, SUC_NE_lexicon_tags_2 },
    { 0xa77e4f7, SUC_NE_lexicon_tags_2 },
    { 0x8539e4f8, SUC_NE_lexicon_tags_18 },
    { 0x7afbe4f9, SUC_NE_lexicon_tags_3 },
    { 0x5d81e4f9, SUC_NE_lexicon_tags_2 },
    { 0x1423e4fc, SUC_NE_lexicon_tags_37 },
    { 0, NULL },
    { 0xf667e4fe, SUC_NE_lexicon_tags_2 },
    { 0x52a3e4ff, SUC_NE_lexicon_tags_2 },
    { 0xf995e500, SUC_NE_lexicon_tags_113 },
    { 0xd9c3e500, SUC_NE_lexicon_tags_2 },
    { 0x4d3fe500, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67fe505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fafe508, SUC_NE_lexicon_tags_2 },
    { 0xdabfe508, SUC_NE_lexicon_tags_2 },
    { 0x44bfe50a, SUC_NE_lexicon_tags_2 },
    { 0xb81fe50a, SUC_NE_lexicon_tags_3 },
    { 0xa6c9e50c, SUC_NE_lexicon_tags_2 },
    { 0x9d9be50c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c27e511, SUC_NE_lexicon_tags_2 },
    { 0x42efe512, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5a1e514, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e7e518, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8cbde51a, SUC_NE_lexicon_tags_2 },
    { 0x7dd1e51b, SUC_NE_lexicon_tags_2 },
    { 0x1535e51c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fe520, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bd7e522, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a33e528, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb3e52c, SUC_NE_lexicon_tags_2 },
    { 0x1c93e52d, SUC_NE_lexicon_tags_2 },
    { 0xaa31e52e, SUC_NE_lexicon_tags_2 },
    { 0x180de52f, SUC_NE_lexicon_tags_2 },
    { 0x88afe52e, SUC_NE_lexicon_tags_2 },
    { 0x3a1e531, SUC_NE_lexicon_tags_2 },
    { 0xc23de530, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922de53a, SUC_NE_lexicon_tags_2 },
    { 0xd1c3e53b, SUC_NE_lexicon_tags_2 },
    { 0xfd3be53c, SUC_NE_lexicon_tags_2 },
    { 0x79cde53b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41be53f, SUC_NE_lexicon_tags_2 },
    { 0x9fbfe53f, SUC_NE_lexicon_tags_36 },
    { 0x6b73e541, SUC_NE_lexicon_tags_2 },
    { 0x9a23e542, SUC_NE_lexicon_tags_2 },
    { 0x5f19e543, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d5e546, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d0be548, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b21e54a, SUC_NE_lexicon_tags_2 },
    { 0xa561e54b, SUC_NE_lexicon_tags_2 },
    { 0xa75e54c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ede54f, SUC_NE_lexicon_tags_2 },
    { 0x5e93e550, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0de554, SUC_NE_lexicon_tags_2 },
    { 0x12cbe555, SUC_NE_lexicon_tags_2 },
    { 0x34b3e556, SUC_NE_lexicon_tags_3 },
    { 0x54dbe557, SUC_NE_lexicon_tags_2 },
    { 0xf505e558, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddfe55c, SUC_NE_lexicon_tags_2 },
    { 0x734fe55d, SUC_NE_lexicon_tags_2 },
    { 0x79a1e55e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d65e560, SUC_NE_lexicon_tags_2 },
    { 0x2735e561, SUC_NE_lexicon_tags_2 },
    { 0x5e19e561, SUC_NE_lexicon_tags_2 },
    { 0xbfb7e563, SUC_NE_lexicon_tags_2 },
    { 0x2c3e564, SUC_NE_lexicon_tags_2 },
    { 0xbb6be565, SUC_NE_lexicon_tags_2 },
    { 0x6abde566, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x79fde568, SUC_NE_lexicon_tags_2 },
    { 0xb61fe569, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1697e56b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7859e56d, SUC_NE_lexicon_tags_2 },
    { 0xbc33e56e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x527fe571, SUC_NE_lexicon_tags_2 },
    { 0x6d4be572, SUC_NE_lexicon_tags_13 },
    { 0x8a15e572, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe009e575, SUC_NE_lexicon_tags_4 },
    { 0x22cde576, SUC_NE_lexicon_tags_2 },
    { 0x58f3e577, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3befe57b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb87e57e, SUC_NE_lexicon_tags_2 },
    { 0x28efe57f, SUC_NE_lexicon_tags_81 },
    { 0x3a59e57f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf05e582, SUC_NE_lexicon_tags_2 },
    { 0x73d7e583, SUC_NE_lexicon_tags_2 },
    { 0xe2dde584, SUC_NE_lexicon_tags_2 },
    { 0x4eb5e584, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68fe589, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4691e58b, SUC_NE_lexicon_tags_2 },
    { 0xb957e58c, SUC_NE_lexicon_tags_2 },
    { 0x363e58d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x977be590, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1589e592, SUC_NE_lexicon_tags_2 },
    { 0x2ed7e593, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7213e595, SUC_NE_lexicon_tags_2 },
    { 0x1f21e596, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x956be598, SUC_NE_lexicon_tags_1 },
    { 0xfe07e599, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8971e59d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f17e5a0, SUC_NE_lexicon_tags_2 },
    { 0x6b2fe5a1, SUC_NE_lexicon_tags_17 },
    { 0x6a53e5a0, SUC_NE_lexicon_tags_2 },
    { 0x45e5e5a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb045e5a7, SUC_NE_lexicon_tags_2 },
    { 0x10d9e5a7, SUC_NE_lexicon_tags_2 },
    { 0x3857e5a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26be5ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6a3e5b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x201e5b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad9e5b5, SUC_NE_lexicon_tags_2 },
    { 0x9f27e5b6, SUC_NE_lexicon_tags_7 },
    { 0x5bf7e5b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8869e5ba, SUC_NE_lexicon_tags_1 },
    { 0xc795e5bb, SUC_NE_lexicon_tags_4 },
    { 0xb5a5e5bb, SUC_NE_lexicon_tags_2 },
    { 0xfe2fe5ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fe5c5, SUC_NE_lexicon_tags_15 },
    { 0xd7dbe5c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7637e5c8, SUC_NE_lexicon_tags_2 },
    { 0x5145e5c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbde5cc, SUC_NE_lexicon_tags_2 },
    { 0xbd31e5cd, SUC_NE_lexicon_tags_2 },
    { 0x9b9e5ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e7e5d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb939e5d5, SUC_NE_lexicon_tags_2 },
    { 0xb019e5d6, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bfe5dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x455be5df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b61e5e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9fe5e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7e5ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2657e5f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aebe5f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1e5fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fb9e600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x790fe603, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0xfa9fe605, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x721be608, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x781e60a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f73e611, SUC_NE_lexicon_tags_2 },
    { 0x393be612, SUC_NE_lexicon_tags_2 },
    { 0x5fabe612, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21cde615, SUC_NE_lexicon_tags_2 },
    { 0xb44be616, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab69e619, SUC_NE_lexicon_tags_2 },
    { 0x874be61a, SUC_NE_lexicon_tags_4 },
    { 0x2327e61b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6fe61e, SUC_NE_lexicon_tags_2 },
    { 0x8221e61e, SUC_NE_lexicon_tags_2 },
    { 0x4e7be61e, SUC_NE_lexicon_tags_2 },
    { 0x8d43e620, SUC_NE_lexicon_tags_2 },
    { 0xd52be620, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d7e629, SUC_NE_lexicon_tags_2 },
    { 0x16f7e629, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b67e62c, SUC_NE_lexicon_tags_2 },
    { 0xfbfbe62c, SUC_NE_lexicon_tags_2 },
    { 0x9d97e62d, SUC_NE_lexicon_tags_2 },
    { 0xaabde62e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5cfe631, SUC_NE_lexicon_tags_2 },
    { 0x744fe632, SUC_NE_lexicon_tags_24 },
    { 0xd7c5e632, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x436fe635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69fe639, SUC_NE_lexicon_tags_2 },
    { 0x32a3e63a, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xdf17e63c, SUC_NE_lexicon_tags_2 },
    { 0x7abe63d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf55e63f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc1e642, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57e646, SUC_NE_lexicon_tags_4 },
    { 0x2c73e647, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x8421e649, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x320be64b, SUC_NE_lexicon_tags_2 },
    { 0x817de64b, SUC_NE_lexicon_tags_2 },
    { 0x909be64d, SUC_NE_lexicon_tags_2 },
    { 0xa4e5e64d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f9e651, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc663e653, SUC_NE_lexicon_tags_2 },
    { 0x69f3e654, SUC_NE_lexicon_tags_2 },
    { 0xa07fe654, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9325e659, SUC_NE_lexicon_tags_2 },
    { 0x4423e65a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5d7e65c, SUC_NE_lexicon_tags_2 },
    { 0xfc63e65d, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2be662, SUC_NE_lexicon_tags_2 },
    { 0x93efe662, SUC_NE_lexicon_tags_2 },
    { 0x5bdfe662, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe22de666, SUC_NE_lexicon_tags_7 },
    { 0xfbbfe666, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf5e66b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fade670, SUC_NE_lexicon_tags_2 },
    { 0xd1d7e671, SUC_NE_lexicon_tags_2 },
    { 0x69efe672, SUC_NE_lexicon_tags_2 },
    { 0xeb3de673, SUC_NE_lexicon_tags_2 },
    { 0x5e3be671, SUC_NE_lexicon_tags_2 },
    { 0xf207e675, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x2147e677, SUC_NE_lexicon_tags_2 },
    { 0x8e31e678, SUC_NE_lexicon_tags_2 },
    { 0x831be678, SUC_NE_lexicon_tags_2 },
    { 0x7367e67a, SUC_NE_lexicon_tags_2 },
    { 0xa797e67b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585fe680, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dc9e682, SUC_NE_lexicon_tags_3 },
    { 0x2877e682, SUC_NE_lexicon_tags_2 },
    { 0x3411e684, SUC_NE_lexicon_tags_2 },
    { 0x1f1e682, SUC_NE_lexicon_tags_2 },
    { 0xfae9e686, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39be68a, SUC_NE_lexicon_tags_2 },
    { 0xdbdfe68b, SUC_NE_lexicon_tags_2 },
    { 0xb4dfe68c, SUC_NE_lexicon_tags_2 },
    { 0x2e03e68d, SUC_NE_lexicon_tags_2 },
    { 0xf95de68a, SUC_NE_lexicon_tags_272 },
    { 0x7d5e68c, SUC_NE_lexicon_tags_3 },
    { 0xece5e690, SUC_NE_lexicon_tags_2 },
    { 0xd4de68e, SUC_NE_lexicon_tags_2 },
    { 0x47a9e691, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28fe697, SUC_NE_lexicon_tags_2 },
    { 0x30ebe698, SUC_NE_lexicon_tags_24 },
    { 0x5f61e699, SUC_NE_lexicon_tags_2 },
    { 0x277e699, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c93e6a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe529e6a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8341e6a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9987e6ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5bfe6ae, SUC_NE_lexicon_tags_2 },
    { 0x1b85e6af, SUC_NE_lexicon_tags_2 },
    { 0x4465e6af, SUC_NE_lexicon_tags_2 },
    { 0x75abe6ae, SUC_NE_lexicon_tags_2 },
    { 0x39fde6b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f7e6bc, SUC_NE_lexicon_tags_2 },
    { 0x1d31e6bc, SUC_NE_lexicon_tags_2 },
    { 0x471de6be, SUC_NE_lexicon_tags_2 },
    { 0x2975e6bc, SUC_NE_lexicon_tags_2 },
    { 0x206fe6c0, SUC_NE_lexicon_tags_2 },
    { 0x2c7de6c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7fe6c8, SUC_NE_lexicon_tags_2 },
    { 0x3217e6c8, SUC_NE_lexicon_tags_2 },
    { 0xd67e6c9, SUC_NE_lexicon_tags_19 },
    { 0x66de6cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd351e6ce, SUC_NE_lexicon_tags_2 },
    { 0x62e9e6cf, SUC_NE_lexicon_tags_2 },
    { 0xf115e6d0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddde6d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99e9e6d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd399e6db, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4705e6dd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bbe6e0, SUC_NE_lexicon_tags_2 },
    { 0x791e6e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd3e6e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9be6ed, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x9367e6ef, SUC_NE_lexicon_tags_2 },
    { 0x7c2de6f0, SUC_NE_lexicon_tags_3 },
    { 0x4eafe6f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521be6f6, SUC_NE_lexicon_tags_2 },
    { 0xff49e6f7, SUC_NE_lexicon_tags_2 },
    { 0x7c07e6f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x42e3e6fa, SUC_NE_lexicon_tags_2 },
    { 0xc9d9e6fa, SUC_NE_lexicon_tags_2 },
    { 0x5d89e6fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63e6ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea5e702, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55e705, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf331e707, SUC_NE_lexicon_tags_14 },
    { 0xd9ffe707, SUC_NE_lexicon_tags_2 },
    { 0x8661e707, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74de70d, SUC_NE_lexicon_tags_2 },
    { 0x3145e70e, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0x610de710, SUC_NE_lexicon_tags_2 },
    { 0x1e85e710, SUC_NE_lexicon_tags_2 },
    { 0x833de711, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd3de714, SUC_NE_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019e719, SUC_NE_lexicon_tags_2 },
    { 0xfb53e71a, SUC_NE_lexicon_tags_2 },
    { 0x143be719, SUC_NE_lexicon_tags_2 },
    { 0x3a19e71c, SUC_NE_lexicon_tags_202 },
    { 0x3fd7e71d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e7e722, SUC_NE_lexicon_tags_2 },
    { 0x26b9e723, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x36a1e725, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa23e728, SUC_NE_lexicon_tags_2 },
    { 0xe055e729, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bfe72e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b7e733, SUC_NE_lexicon_tags_2 },
    { 0xf1ebe734, SUC_NE_lexicon_tags_2 },
    { 0x20f1e735, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587de73a, SUC_NE_lexicon_tags_49 },
    { 0x5cd9e73a, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0x59de73d, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e744, SUC_NE_lexicon_tags_2 },
    { 0xb003e745, SUC_NE_lexicon_tags_2 },
    { 0xc533e745, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1ede748, SUC_NE_lexicon_tags_2 },
    { 0x173be749, SUC_NE_lexicon_tags_2 },
    { 0x8209e74a, SUC_NE_lexicon_tags_7 },
    { 0x18cbe749, SUC_NE_lexicon_tags_2 },
    { 0xda19e74c, SUC_NE_lexicon_tags_2 },
    { 0x17bbe74d, SUC_NE_lexicon_tags_2 },
    { 0xd09fe74d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x943e750, SUC_NE_lexicon_tags_24 },
    { 0x9809e751, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb189e754, SUC_NE_lexicon_tags_2 },
    { 0x19e9e755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64de75d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38de762, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b1e766, SUC_NE_lexicon_tags_2 },
    { 0x2609e766, SUC_NE_lexicon_tags_2 },
    { 0x302fe768, SUC_NE_lexicon_tags_2 },
    { 0x79b1e769, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc423e76c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3807e76e, SUC_NE_lexicon_tags_2 },
    { 0xcc6fe76e, SUC_NE_lexicon_tags_2 },
    { 0x619be76f, SUC_NE_lexicon_tags_1 },
    { 0xcc03e771, SUC_NE_lexicon_tags_2 },
    { 0xa80be772, SUC_NE_lexicon_tags_2 },
    { 0xe9a1e773, SUC_NE_lexicon_tags_2 },
    { 0xaa71e774, SUC_NE_lexicon_tags_2 },
    { 0x25d5e774, SUC_NE_lexicon_tags_2 },
    { 0x320be776, SUC_NE_lexicon_tags_2 },
    { 0x84e7e777, SUC_NE_lexicon_tags_2 },
    { 0xe5bde778, SUC_NE_lexicon_tags_2 },
    { 0xc537e778, SUC_NE_lexicon_tags_2 },
    { 0x71a3e77a, SUC_NE_lexicon_tags_2 },
    { 0x6489e77b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40a3e77d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fb5e77f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e85e782, SUC_NE_lexicon_tags_2 },
    { 0x9791e783, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8fe786, SUC_NE_lexicon_tags_2 },
    { 0xe07fe787, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8277e789, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9945e78c, SUC_NE_lexicon_tags_324 },
    { 0, NULL },
    { 0, NULL },
    { 0x8459e78f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x68d3e791, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb885e798, SUC_NE_lexicon_tags_2 },
    { 0x8947e799, SUC_NE_lexicon_tags_2 },
    { 0xef8be79a, SUC_NE_lexicon_tags_2 },
    { 0x3383e79b, SUC_NE_lexicon_tags_2 },
    { 0x573be79b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe193e79e, SUC_NE_lexicon_tags_2 },
    { 0xcb73e79f, SUC_NE_lexicon_tags_2 },
    { 0x5583e7a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb27e7a4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b3e7a7, SUC_NE_lexicon_tags_2 },
    { 0x2a7be7a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e95e7aa, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6811e7ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc687e7af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x774fe7b2, SUC_NE_lexicon_tags_2 },
    { 0x852de7b3, SUC_NE_lexicon_tags_2 },
    { 0x109be7b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc3e7b7, SUC_NE_lexicon_tags_2 },
    { 0x5b01e7b7, SUC_NE_lexicon_tags_5 },
    { 0x53dfe7b7, SUC_NE_lexicon_tags_2 },
    { 0x863be7ba, SUC_NE_lexicon_tags_2 },
    { 0xaaade7ba, SUC_NE_lexicon_tags_2 },
    { 0x804fe7ba, SUC_NE_lexicon_tags_2 },
    { 0x7b41e7bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8371e7c1, SUC_NE_lexicon_tags_2 },
    { 0x129de7c1, SUC_NE_lexicon_tags_2 },
    { 0x1df9e7c2, SUC_NE_lexicon_tags_2 },
    { 0xcf6be7c2, SUC_NE_lexicon_tags_2 },
    { 0xf711e7c5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe735e7c8, SUC_NE_lexicon_tags_2 },
    { 0x382be7c8, SUC_NE_lexicon_tags_2 },
    { 0xe193e7c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc421e7cc, SUC_NE_lexicon_tags_2 },
    { 0x8725e7cd, SUC_NE_lexicon_tags_2 },
    { 0x5a53e7ce, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2fe7d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ebfe7d3, SUC_NE_lexicon_tags_2 },
    { 0x1025e7d3, SUC_NE_lexicon_tags_2 },
    { 0x67bfe7d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5849e7d7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4fe7da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bde7dd, SUC_NE_lexicon_tags_109 },
    { 0, NULL },
    { 0x1369e7df, SUC_NE_lexicon_tags_2 },
    { 0xf791e7e0, SUC_NE_lexicon_tags_2 },
    { 0x1455e7e1, SUC_NE_lexicon_tags_2 },
    { 0x5ad7e7e2, SUC_NE_lexicon_tags_2 },
    { 0x3391e7e3, SUC_NE_lexicon_tags_2 },
    { 0x3677e7e3, SUC_NE_lexicon_tags_2 },
    { 0x60f1e7e3, SUC_NE_lexicon_tags_2 },
    { 0x5fde7e3, SUC_NE_lexicon_tags_2 },
    { 0x9947e7e7, SUC_NE_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cebe7ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2561e7ec, SUC_NE_lexicon_tags_2 },
    { 0xdfa7e7ed, SUC_NE_lexicon_tags_2 },
    { 0x3777e7ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c9e7f5, SUC_NE_lexicon_tags_2 },
    { 0xf139e7f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dfe7fd, SUC_NE_lexicon_tags_2 },
    { 0x5451e7fe, SUC_NE_lexicon_tags_2 },
    { 0xeca3e7fd, SUC_NE_lexicon_tags_2 },
    { 0xac9be7fd, SUC_NE_lexicon_tags_2 },
    { 0xabd7e801, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6be804, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8effe807, SUC_NE_lexicon_tags_2 },
    { 0xec67e808, SUC_NE_lexicon_tags_2 },
    { 0x3b75e809, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35be80d, SUC_NE_lexicon_tags_281 },
    { 0, NULL },
    { 0x4b03e80f, SUC_NE_lexicon_tags_2 },
    { 0x890de810, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeede812, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe816, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495be81c, SUC_NE_lexicon_tags_11 },
    { 0x898de81c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e37e820, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x587fe823, SUC_NE_lexicon_tags_2 },
    { 0x1d13e824, SUC_NE_lexicon_tags_2 },
    { 0x502de823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e09e82e, SUC_NE_lexicon_tags_1 },
    { 0x12a7e82f, SUC_NE_lexicon_tags_2 },
    { 0xb9b5e830, SUC_NE_lexicon_tags_2 },
    { 0x4dbe830, SUC_NE_lexicon_tags_2 },
    { 0x4627e832, SUC_NE_lexicon_tags_2 },
    { 0x5521e82f, SUC_NE_lexicon_tags_2 },
    { 0x3a07e834, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5be839, SUC_NE_lexicon_tags_2 },
    { 0xfa1e839, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8699e840, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa623e844, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7785e846, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e69e848, SUC_NE_lexicon_tags_2 },
    { 0x686be849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc5e84d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8415e850, SUC_NE_lexicon_tags_2 },
    { 0x13c7e850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd08fe853, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x69b1e855, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4fe85c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c5e85f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x357e861, SUC_NE_lexicon_tags_2 },
    { 0x205be862, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e1e864, SUC_NE_lexicon_tags_0 },
    { 0x1fabe865, SUC_NE_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1de868, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x724de86b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecfe86f, SUC_NE_lexicon_tags_3 },
    { 0x21cde86f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf1e874, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xabafe876, SUC_NE_lexicon_tags_2 },
    { 0x225be877, SUC_NE_lexicon_tags_2 },
    { 0xa793e878, SUC_NE_lexicon_tags_2 },
    { 0x9159e876, SUC_NE_lexicon_tags_3 },
    { 0xe0a1e87a, SUC_NE_lexicon_tags_2 },
    { 0xf7f7e878, SUC_NE_lexicon_tags_2 },
    { 0x2d0de87c, SUC_NE_lexicon_tags_7 },
    { 0x5047e87d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b1e880, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb3e883, SUC_NE_lexicon_tags_2 },
    { 0x371e884, SUC_NE_lexicon_tags_2 },
    { 0xd35be885, SUC_NE_lexicon_tags_58 },
    { 0xef35e886, SUC_NE_lexicon_tags_34 },
    { 0x6541e885, SUC_NE_lexicon_tags_4 },
    { 0xe153e888, SUC_NE_lexicon_tags_2 },
    { 0xe7ade888, SUC_NE_lexicon_tags_2 },
    { 0xde89e886, SUC_NE_lexicon_tags_2 },
    { 0x70fe889, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23e88e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82bbe890, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859e897, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fbbe899, SUC_NE_lexicon_tags_2 },
    { 0x5435e899, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1311e89d, SUC_NE_lexicon_tags_2 },
    { 0x9909e89e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35e8a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf549e8a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4031e8a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6de8ac, SUC_NE_lexicon_tags_2 },
    { 0x1637e8ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7abe8af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ede8b2, SUC_NE_lexicon_tags_2 },
    { 0x4f69e8b3, SUC_NE_lexicon_tags_2 },
    { 0x6c69e8b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97e8ba, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec9e8bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6de8c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec29e8c9, SUC_NE_lexicon_tags_2 },
    { 0x3755e8ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a95e8cc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb1e8d2, SUC_NE_lexicon_tags_2 },
    { 0x45a7e8d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5e8d8, SUC_NE_lexicon_tags_2 },
    { 0xc113e8d9, SUC_NE_lexicon_tags_2 },
    { 0xfa25e8da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511fe8de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1e8e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5389e8e4, SUC_NE_lexicon_tags_19 },
    { 0x152be8e5, SUC_NE_lexicon_tags_2 },
    { 0x1f17e8e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9111e8ea, SUC_NE_lexicon_tags_2 },
    { 0x34fde8ea, SUC_NE_lexicon_tags_2 },
    { 0x701e8ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f1e8ef, SUC_NE_lexicon_tags_2 },
    { 0xa543e8f0, SUC_NE_lexicon_tags_2 },
    { 0xce09e8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xec81e8f3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9b9e8f5, SUC_NE_lexicon_tags_2 },
    { 0xf797e8f5, SUC_NE_lexicon_tags_2 },
    { 0x7833e8f7, SUC_NE_lexicon_tags_2 },
    { 0x570be8f7, SUC_NE_lexicon_tags_2 },
    { 0xd01fe8f5, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ebe8fc, SUC_NE_lexicon_tags_2 },
    { 0xcea3e8fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2025e8ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f65e901, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5791e903, SUC_NE_lexicon_tags_2 },
    { 0xe057e903, SUC_NE_lexicon_tags_2 },
    { 0x6b63e903, SUC_NE_lexicon_tags_2 },
    { 0xd215e906, SUC_NE_lexicon_tags_2 },
    { 0x9ca7e907, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xdadfe909, SUC_NE_lexicon_tags_1 },
    { 0x245be90a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ec3e90c, SUC_NE_lexicon_tags_2 },
    { 0xf347e90d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79be911, SUC_NE_lexicon_tags_2 },
    { 0x173fe911, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99e7e914, SUC_NE_lexicon_tags_2 },
    { 0xb8c1e915, SUC_NE_lexicon_tags_2 },
    { 0x9ca5e914, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa941e91e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a67e920, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29be923, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4285e925, SUC_NE_lexicon_tags_2 },
    { 0xf1f3e926, SUC_NE_lexicon_tags_2 },
    { 0x540de927, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e39e929, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd5e92d, SUC_NE_lexicon_tags_2 },
    { 0x82e1e92d, SUC_NE_lexicon_tags_11 },
    { 0xaf5e92e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff3e939, SUC_NE_lexicon_tags_2 },
    { 0x38ade93a, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x15be93d, SUC_NE_lexicon_tags_2 },
    { 0xd689e93e, SUC_NE_lexicon_tags_2 },
    { 0x1143e93f, SUC_NE_lexicon_tags_2 },
    { 0xd25fe93d, SUC_NE_lexicon_tags_2 },
    { 0xf369e941, SUC_NE_lexicon_tags_2 },
    { 0x1909e942, SUC_NE_lexicon_tags_2 },
    { 0xdfe5e943, SUC_NE_lexicon_tags_2 },
    { 0xdaf3e944, SUC_NE_lexicon_tags_59 },
    { 0x9a87e943, SUC_NE_lexicon_tags_2 },
    { 0x38e9e942, SUC_NE_lexicon_tags_2 },
    { 0x4c3fe947, SUC_NE_lexicon_tags_2 },
    { 0xa2ade948, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446de94c, SUC_NE_lexicon_tags_2 },
    { 0x735be94c, SUC_NE_lexicon_tags_83 },
    { 0, NULL },
    { 0x296be94f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1be952, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e7e957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca7e95e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf46be960, SUC_NE_lexicon_tags_2 },
    { 0x9e25e960, SUC_NE_lexicon_tags_2 },
    { 0x7ce7e962, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc61de964, SUC_NE_lexicon_tags_30 },
    { 0x3c9be964, SUC_NE_lexicon_tags_2 },
    { 0x20b7e966, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826fe96b, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xd629e96d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd033e970, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9fe976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1729e97a, SUC_NE_lexicon_tags_2 },
    { 0x5d45e97b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdcbe97d, SUC_NE_lexicon_tags_111 },
    { 0x76c3e97e, SUC_NE_lexicon_tags_2 },
    { 0x6467e97f, SUC_NE_lexicon_tags_2 },
    { 0xa557e97d, SUC_NE_lexicon_tags_2 },
    { 0x9657e97e, SUC_NE_lexicon_tags_7 },
    { 0x6405e982, SUC_NE_lexicon_tags_2 },
    { 0xafffe97f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bde987, SUC_NE_lexicon_tags_2 },
    { 0xef91e988, SUC_NE_lexicon_tags_2 },
    { 0x737de988, SUC_NE_lexicon_tags_2 },
    { 0xafb1e989, SUC_NE_lexicon_tags_2 },
    { 0x37dde989, SUC_NE_lexicon_tags_2 },
    { 0x7be7e98c, SUC_NE_lexicon_tags_2 },
    { 0x71a9e98d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8399e98f, SUC_NE_lexicon_tags_40 },
    { 0xa9d3e990, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddfe993, SUC_NE_lexicon_tags_2 },
    { 0x5f0fe993, SUC_NE_lexicon_tags_7 },
    { 0xb94fe995, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6abe998, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a6fe99a, SUC_NE_lexicon_tags_2 },
    { 0x7995e99a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bb5e99d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4be9a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d73e9a2, SUC_NE_lexicon_tags_2 },
    { 0x10be9a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaf3e9a5, SUC_NE_lexicon_tags_2 },
    { 0xfc9be9a5, SUC_NE_lexicon_tags_2 },
    { 0xda61e9a7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x1901e9a9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd77e9ab, SUC_NE_lexicon_tags_2 },
    { 0x464fe9ac, SUC_NE_lexicon_tags_2 },
    { 0xa069e9ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef8de9af, SUC_NE_lexicon_tags_2 },
    { 0x3225e9af, SUC_NE_lexicon_tags_2 },
    { 0xcb39e9b1, SUC_NE_lexicon_tags_2 },
    { 0x728de9b1, SUC_NE_lexicon_tags_7 },
    { 0xa229e9b2, SUC_NE_lexicon_tags_2 },
    { 0x1503e9af, SUC_NE_lexicon_tags_220 },
    { 0xfd3e9b5, SUC_NE_lexicon_tags_2 },
    { 0x8f3e9b4, SUC_NE_lexicon_tags_2 },
    { 0xb55e9b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a9e9bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76bfe9c1, SUC_NE_lexicon_tags_2 },
    { 0x3e1e9c2, SUC_NE_lexicon_tags_2 },
    { 0x9363e9c3, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5e9c8, SUC_NE_lexicon_tags_2 },
    { 0x911fe9c8, SUC_NE_lexicon_tags_2 },
    { 0xb1cfe9c9, SUC_NE_lexicon_tags_2 },
    { 0xfd1fe9cb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb2be9cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b49e9d0, SUC_NE_lexicon_tags_2 },
    { 0x3151e9d1, SUC_NE_lexicon_tags_2 },
    { 0x9d59e9d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8be9d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bbe9da, SUC_NE_lexicon_tags_2 },
    { 0xe91de9db, SUC_NE_lexicon_tags_2 },
    { 0xa7e1e9dc, SUC_NE_lexicon_tags_2 },
    { 0x4c8be9da, SUC_NE_lexicon_tags_2 },
    { 0xdbbe9dc, SUC_NE_lexicon_tags_2 },
    { 0x5275e9df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fc9e9e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa35fe9e3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb10de9e5, SUC_NE_lexicon_tags_2 },
    { 0x33f1e9e6, SUC_NE_lexicon_tags_2 },
    { 0xb151e9e7, SUC_NE_lexicon_tags_4 },
    { 0x2941e9e8, SUC_NE_lexicon_tags_2 },
    { 0xe207e9e7, SUC_NE_lexicon_tags_2 },
    { 0x21e9e9e9, SUC_NE_lexicon_tags_4 },
    { 0x2fede9eb, SUC_NE_lexicon_tags_2 },
    { 0x11c3e9ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ad3e9ee, SUC_NE_lexicon_tags_2 },
    { 0x4a5be9ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a03e9f1, SUC_NE_lexicon_tags_17 },
    { 0x74cfe9f2, SUC_NE_lexicon_tags_2 },
    { 0x9cd5e9f1, SUC_NE_lexicon_tags_2 },
    { 0xea0fe9f2, SUC_NE_lexicon_tags_2 },
    { 0xde53e9f4, SUC_NE_lexicon_tags_2 },
    { 0x6cb9e9f5, SUC_NE_lexicon_tags_2 },
    { 0xfa9e9f4, SUC_NE_lexicon_tags_2 },
    { 0x9a4be9f8, SUC_NE_lexicon_tags_2 },
    { 0x37ebe9f5, SUC_NE_lexicon_tags_2 },
    { 0xb32fe9fa, SUC_NE_lexicon_tags_5 },
    { 0x24f5e9fb, SUC_NE_lexicon_tags_2 },
    { 0x61f1e9fc, SUC_NE_lexicon_tags_2 },
    { 0x78c1e9fb, SUC_NE_lexicon_tags_3 },
    { 0x1213e9fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x400dea00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79ea04, SUC_NE_lexicon_tags_2 },
    { 0xe381ea04, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xf63dea07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7c1ea09, SUC_NE_lexicon_tags_2 },
    { 0x82adea0a, SUC_NE_lexicon_tags_34 },
    { 0x11b7ea0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3175ea0d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d39ea0f, SUC_NE_lexicon_tags_2 },
    { 0x46c5ea10, SUC_NE_lexicon_tags_2 },
    { 0x7555ea0f, SUC_NE_lexicon_tags_209 },
    { 0x6d4fea12, SUC_NE_lexicon_tags_2 },
    { 0x4be3ea10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd5ea16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3dea1c, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3bea1f, SUC_NE_lexicon_tags_2 },
    { 0xa6d3ea1f, SUC_NE_lexicon_tags_2 },
    { 0xc3ffea20, SUC_NE_lexicon_tags_30 },
    { 0x1945ea22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35dea26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabdbea29, SUC_NE_lexicon_tags_2 },
    { 0x43d9ea2a, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2e81ea2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd729ea2e, SUC_NE_lexicon_tags_13 },
    { 0x7043ea2f, SUC_NE_lexicon_tags_2 },
    { 0x573fea2e, SUC_NE_lexicon_tags_1 },
    { 0xf671ea31, SUC_NE_lexicon_tags_2 },
    { 0x119dea31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb635ea37, SUC_NE_lexicon_tags_30 },
    { 0x9763ea38, SUC_NE_lexicon_tags_2 },
    { 0x71dea38, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f9ea3e, SUC_NE_lexicon_tags_2 },
    { 0xabddea3f, SUC_NE_lexicon_tags_2 },
    { 0xedfdea40, SUC_NE_lexicon_tags_2 },
    { 0xa163ea40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf7ea44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4383ea47, SUC_NE_lexicon_tags_2 },
    { 0xb567ea48, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa659ea4a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b7ea4d, SUC_NE_lexicon_tags_2 },
    { 0x7af3ea4e, SUC_NE_lexicon_tags_2 },
    { 0x7251ea4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5785ea51, SUC_NE_lexicon_tags_2 },
    { 0x92fea52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9945ea55, SUC_NE_lexicon_tags_2 },
    { 0xc71fea55, SUC_NE_lexicon_tags_2 },
    { 0x91dbea56, SUC_NE_lexicon_tags_2 },
    { 0xaccfea58, SUC_NE_lexicon_tags_2 },
    { 0xdabea59, SUC_NE_lexicon_tags_2 },
    { 0xf075ea5a, SUC_NE_lexicon_tags_2 },
    { 0xfc71ea5a, SUC_NE_lexicon_tags_2 },
    { 0x6b51ea5c, SUC_NE_lexicon_tags_2 },
    { 0x2a89ea5c, SUC_NE_lexicon_tags_2 },
    { 0x4485ea5a, SUC_NE_lexicon_tags_4 },
    { 0xbe35ea5e, SUC_NE_lexicon_tags_2 },
    { 0x2173ea60, SUC_NE_lexicon_tags_2 },
    { 0xb9fdea5c, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bbea65, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda83ea67, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7abea6d, SUC_NE_lexicon_tags_2 },
    { 0xa4e3ea6e, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x1249ea70, SUC_NE_lexicon_tags_2 },
    { 0x6829ea71, SUC_NE_lexicon_tags_2 },
    { 0x141dea72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b7ea78, SUC_NE_lexicon_tags_2 },
    { 0xc0d3ea78, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbbea7e, SUC_NE_lexicon_tags_2 },
    { 0x248bea7f, SUC_NE_lexicon_tags_2 },
    { 0xd313ea7f, SUC_NE_lexicon_tags_2 },
    { 0x1f81ea80, SUC_NE_lexicon_tags_2 },
    { 0x9e5bea7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb63ea86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0cdea88, SUC_NE_lexicon_tags_2 },
    { 0xf27bea89, SUC_NE_lexicon_tags_4 },
    { 0x3d43ea89, SUC_NE_lexicon_tags_2 },
    { 0x833dea88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e07ea8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fdea91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc753ea94, SUC_NE_lexicon_tags_2 },
    { 0x60d7ea95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bfea98, SUC_NE_lexicon_tags_2 },
    { 0x8061ea98, SUC_NE_lexicon_tags_17 },
    { 0x1c95ea9a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e57ea9c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841eaa3, SUC_NE_lexicon_tags_2 },
    { 0x6a3feaa3, SUC_NE_lexicon_tags_2 },
    { 0x972beaa4, SUC_NE_lexicon_tags_2 },
    { 0x3085eaa5, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77cdeaaa, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9885eaad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e73eaaf, SUC_NE_lexicon_tags_2 },
    { 0x6b87eab0, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb111eab3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe701eab5, SUC_NE_lexicon_tags_2 },
    { 0x43cfeab6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c1deab8, SUC_NE_lexicon_tags_2 },
    { 0x3383eab8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bddeac3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa331eac8, SUC_NE_lexicon_tags_2 },
    { 0xde87eac9, SUC_NE_lexicon_tags_2 },
    { 0xf839eaca, SUC_NE_lexicon_tags_3 },
    { 0x7553eac8, SUC_NE_lexicon_tags_2 },
    { 0x36f9eacc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e7eace, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ead2, SUC_NE_lexicon_tags_2 },
    { 0xa10fead3, SUC_NE_lexicon_tags_2 },
    { 0xd98bead2, SUC_NE_lexicon_tags_2 },
    { 0x4593ead5, SUC_NE_lexicon_tags_2 },
    { 0x784dead4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c7eada, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b33eadc, SUC_NE_lexicon_tags_2 },
    { 0xd841eadc, SUC_NE_lexicon_tags_0 },
    { 0xbeb5eadc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb549eae0, SUC_NE_lexicon_tags_2 },
    { 0x54b3eae1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab63eae4, SUC_NE_lexicon_tags_2 },
    { 0x1f3beae5, SUC_NE_lexicon_tags_2 },
    { 0x73c7eae6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8affeaea, SUC_NE_lexicon_tags_2 },
    { 0xa18deaeb, SUC_NE_lexicon_tags_2 },
    { 0x6d75eaea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eaef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffeaf1, SUC_NE_lexicon_tags_2 },
    { 0x9061eaf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb69feaf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x37a5eaf6, SUC_NE_lexicon_tags_2 },
    { 0xa795eaf6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd451eafd, SUC_NE_lexicon_tags_98 },
    { 0xf23deafd, SUC_NE_lexicon_tags_10 },
    { 0x9741eaff, SUC_NE_lexicon_tags_2 },
    { 0xb957eb00, SUC_NE_lexicon_tags_2 },
    { 0x3dfdeafd, SUC_NE_lexicon_tags_2 },
    { 0x62b5eb01, SUC_NE_lexicon_tags_19 },
    { 0x4c95eb00, SUC_NE_lexicon_tags_2 },
    { 0x63f3eb03, SUC_NE_lexicon_tags_2 },
    { 0xb969eb05, SUC_NE_lexicon_tags_2 },
    { 0x26e7eb00, SUC_NE_lexicon_tags_222 },
    { 0x31a5eb03, SUC_NE_lexicon_tags_2 },
    { 0x7a03eafe, SUC_NE_lexicon_tags_2 },
    { 0x7fcbeb06, SUC_NE_lexicon_tags_2 },
    { 0x7c83eb0a, SUC_NE_lexicon_tags_2 },
    { 0xe0e1eb0b, SUC_NE_lexicon_tags_2 },
    { 0x257beb0a, SUC_NE_lexicon_tags_2 },
    { 0xe649eb0d, SUC_NE_lexicon_tags_2 },
    { 0xb373eb0e, SUC_NE_lexicon_tags_2 },
    { 0xbf9beb0f, SUC_NE_lexicon_tags_2 },
    { 0xd445eb10, SUC_NE_lexicon_tags_2 },
    { 0x8235eb10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9363eb16, SUC_NE_lexicon_tags_2 },
    { 0x9c71eb17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9a9eb19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4159eb1b, SUC_NE_lexicon_tags_2 },
    { 0xd229eb1c, SUC_NE_lexicon_tags_2 },
    { 0x471deb1d, SUC_NE_lexicon_tags_2 },
    { 0xc9ebeb1e, SUC_NE_lexicon_tags_2 },
    { 0xa683eb1e, SUC_NE_lexicon_tags_2 },
    { 0x9f6beb1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x842feb22, SUC_NE_lexicon_tags_2 },
    { 0x3e09eb23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb41eb28, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e5eb2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b9eb2f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xdb7beb31, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c1eb35, SUC_NE_lexicon_tags_17 },
    { 0xba99eb35, SUC_NE_lexicon_tags_19 },
    { 0x7259eb37, SUC_NE_lexicon_tags_2 },
    { 0x1015eb35, SUC_NE_lexicon_tags_2 },
    { 0xbc9deb39, SUC_NE_lexicon_tags_2 },
    { 0x36c1eb3a, SUC_NE_lexicon_tags_2 },
    { 0x9c91eb38, SUC_NE_lexicon_tags_2 },
    { 0xcde9eb39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f01eb3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cffeb47, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3f7eb49, SUC_NE_lexicon_tags_2 },
    { 0xb5bfeb4a, SUC_NE_lexicon_tags_2 },
    { 0x1ac3eb4b, SUC_NE_lexicon_tags_2 },
    { 0x3c83eb49, SUC_NE_lexicon_tags_2 },
    { 0x2985eb49, SUC_NE_lexicon_tags_34 },
    { 0x250feb4e, SUC_NE_lexicon_tags_3 },
    { 0x90f5eb4f, SUC_NE_lexicon_tags_7 },
    { 0x16c7eb4b, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0deb53, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf13eb55, SUC_NE_lexicon_tags_2 },
    { 0x52d3eb56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9595eb58, SUC_NE_lexicon_tags_2 },
    { 0x787deb59, SUC_NE_lexicon_tags_2 },
    { 0x2b6deb5a, SUC_NE_lexicon_tags_2 },
    { 0xa541eb5b, SUC_NE_lexicon_tags_2 },
    { 0xd811eb5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452deb61, SUC_NE_lexicon_tags_2 },
    { 0xfbdfeb62, SUC_NE_lexicon_tags_2 },
    { 0xdfc7eb62, SUC_NE_lexicon_tags_2 },
    { 0xa27eb61, SUC_NE_lexicon_tags_13 },
    { 0xa201eb65, SUC_NE_lexicon_tags_2 },
    { 0x4c17eb65, SUC_NE_lexicon_tags_2 },
    { 0xcf99eb62, SUC_NE_lexicon_tags_2 },
    { 0x5935eb68, SUC_NE_lexicon_tags_7 },
    { 0x3d1feb63, SUC_NE_lexicon_tags_2 },
    { 0x27f1eb64, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a09eb6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51eb72, SUC_NE_lexicon_tags_2 },
    { 0x574deb73, SUC_NE_lexicon_tags_2 },
    { 0x440deb73, SUC_NE_lexicon_tags_7 },
    { 0x55b1eb74, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b01eb7a, SUC_NE_lexicon_tags_2 },
    { 0xfd2deb7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd473eb7d, SUC_NE_lexicon_tags_2 },
    { 0x17a9eb7e, SUC_NE_lexicon_tags_2 },
    { 0xcd5eb7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9677eb81, SUC_NE_lexicon_tags_2 },
    { 0x9a0deb81, SUC_NE_lexicon_tags_2 },
    { 0xee67eb83, SUC_NE_lexicon_tags_2 },
    { 0x28c5eb84, SUC_NE_lexicon_tags_7 },
    { 0xc92deb85, SUC_NE_lexicon_tags_2 },
    { 0x2aadeb84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa3eb8d, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0xef89eb8f, SUC_NE_lexicon_tags_2 },
    { 0x7af9eb8f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c31eb94, SUC_NE_lexicon_tags_2 },
    { 0xe91beb94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6585eb97, SUC_NE_lexicon_tags_2 },
    { 0x5de3eb97, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x207deb9b, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x3ed7eb9d, SUC_NE_lexicon_tags_2 },
    { 0x2d57eb9d, SUC_NE_lexicon_tags_17 },
    { 0x6dcdeb9f, SUC_NE_lexicon_tags_2 },
    { 0x2379eb9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd299eba3, SUC_NE_lexicon_tags_2 },
    { 0x6039eba4, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f3eba7, SUC_NE_lexicon_tags_2 },
    { 0x4165eba8, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0febaf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86edebb2, SUC_NE_lexicon_tags_1 },
    { 0x290debb3, SUC_NE_lexicon_tags_2 },
    { 0x9f17ebb4, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487febba, SUC_NE_lexicon_tags_2 },
    { 0x33d9ebbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x726febbd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1bebc0, SUC_NE_lexicon_tags_2 },
    { 0x667ebc0, SUC_NE_lexicon_tags_24 },
    { 0xf1f9ebc0, SUC_NE_lexicon_tags_2 },
    { 0x13bdebc3, SUC_NE_lexicon_tags_2 },
    { 0xf81bebc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e1ebc8, SUC_NE_lexicon_tags_2 },
    { 0x57ebebc8, SUC_NE_lexicon_tags_2 },
    { 0x9d47ebc9, SUC_NE_lexicon_tags_2 },
    { 0xa3d9ebc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7bebcf, SUC_NE_lexicon_tags_2 },
    { 0x1751ebcf, SUC_NE_lexicon_tags_2 },
    { 0x9367ebcf, SUC_NE_lexicon_tags_7 },
    { 0xacb1ebd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa771ebd4, SUC_NE_lexicon_tags_2 },
    { 0x5181ebd5, SUC_NE_lexicon_tags_2 },
    { 0x6f11ebd6, SUC_NE_lexicon_tags_3 },
    { 0x59ddebd7, SUC_NE_lexicon_tags_2 },
    { 0xae2debd8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf973ebdd, SUC_NE_lexicon_tags_2 },
    { 0x8badebdd, SUC_NE_lexicon_tags_2 },
    { 0xe90bebdf, SUC_NE_lexicon_tags_4 },
    { 0x9683ebe0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x193febe3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb05ebe5, SUC_NE_lexicon_tags_2 },
    { 0x5995ebe6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59debea, SUC_NE_lexicon_tags_2 },
    { 0x3bcfebeb, SUC_NE_lexicon_tags_2 },
    { 0xec75ebec, SUC_NE_lexicon_tags_2 },
    { 0x64ebebed, SUC_NE_lexicon_tags_2 },
    { 0xb2a1ebea, SUC_NE_lexicon_tags_2 },
    { 0xeb8febef, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x329bebf1, SUC_NE_lexicon_tags_2 },
    { 0x8305ebf1, SUC_NE_lexicon_tags_25 },
    { 0xb167ebf3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a7debf5, SUC_NE_lexicon_tags_2 },
    { 0x267ebf6, SUC_NE_lexicon_tags_2 },
    { 0xd987ebf7, SUC_NE_lexicon_tags_2 },
    { 0xc5c7ebf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316debfe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcdec02, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7dec07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b3ec09, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0xeafbec0b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6949ec10, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b7ec14, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x2319ec16, SUC_NE_lexicon_tags_2 },
    { 0x76fec17, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b29ec1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3bec21, SUC_NE_lexicon_tags_2 },
    { 0x75f9ec22, SUC_NE_lexicon_tags_2 },
    { 0x7b7bec21, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ed3ec25, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xa305ec27, SUC_NE_lexicon_tags_7 },
    { 0xb9ffec27, SUC_NE_lexicon_tags_2 },
    { 0xcf49ec28, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f3ec2d, SUC_NE_lexicon_tags_163 },
    { 0, NULL },
    { 0x3be3ec2f, SUC_NE_lexicon_tags_2 },
    { 0x3f43ec30, SUC_NE_lexicon_tags_13 },
    { 0x4ab3ec30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x34a7ec33, SUC_NE_lexicon_tags_2 },
    { 0xf5f3ec33, SUC_NE_lexicon_tags_7 },
    { 0x517fec34, SUC_NE_lexicon_tags_2 },
    { 0x11bbec35, SUC_NE_lexicon_tags_81 },
    { 0xd3a1ec37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb23ec3b, SUC_NE_lexicon_tags_2 },
    { 0x6883ec3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf909ec3e, SUC_NE_lexicon_tags_0 },
    { 0x8869ec3f, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0x768dec41, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1ec46, SUC_NE_lexicon_tags_2 },
    { 0xdefdec46, SUC_NE_lexicon_tags_19 },
    { 0xe605ec46, SUC_NE_lexicon_tags_2 },
    { 0xeee3ec49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ffec51, SUC_NE_lexicon_tags_2 },
    { 0x4aa1ec52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9ec54, SUC_NE_lexicon_tags_2 },
    { 0xc9e1ec55, SUC_NE_lexicon_tags_2 },
    { 0xec57ec55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14fec58, SUC_NE_lexicon_tags_2 },
    { 0x8c39ec59, SUC_NE_lexicon_tags_2 },
    { 0x3e13ec5a, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5f9ec5c, SUC_NE_lexicon_tags_1 },
    { 0x44c5ec5c, SUC_NE_lexicon_tags_3 },
    { 0x5aa7ec5c, SUC_NE_lexicon_tags_34 },
    { 0x9fddec5c, SUC_NE_lexicon_tags_2 },
    { 0xf1e1ec5d, SUC_NE_lexicon_tags_2 },
    { 0x26c7ec61, SUC_NE_lexicon_tags_2 },
    { 0x2e15ec5c, SUC_NE_lexicon_tags_2 },
    { 0xef51ec5f, SUC_NE_lexicon_tags_2 },
    { 0x8d0bec5f, SUC_NE_lexicon_tags_2 },
    { 0xc33fec65, SUC_NE_lexicon_tags_2 },
    { 0xeb63ec66, SUC_NE_lexicon_tags_2 },
    { 0x8ab7ec5c, SUC_NE_lexicon_tags_2 },
    { 0x6e41ec68, SUC_NE_lexicon_tags_30 },
    { 0x86a5ec68, SUC_NE_lexicon_tags_2 },
    { 0x3f4bec6a, SUC_NE_lexicon_tags_2 },
    { 0x30e1ec69, SUC_NE_lexicon_tags_2 },
    { 0x5999ec6c, SUC_NE_lexicon_tags_4 },
    { 0x6addec6d, SUC_NE_lexicon_tags_2 },
    { 0xb045ec5f, SUC_NE_lexicon_tags_2 },
    { 0x16d7ec6f, SUC_NE_lexicon_tags_2 },
    { 0x2e73ec5e, SUC_NE_lexicon_tags_2 },
    { 0x20dfec71, SUC_NE_lexicon_tags_2 },
    { 0xd045ec71, SUC_NE_lexicon_tags_7 },
    { 0x29adec73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63fec76, SUC_NE_lexicon_tags_2 },
    { 0xd081ec77, SUC_NE_lexicon_tags_2 },
    { 0x59cdec78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc699ec7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa717ec80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x31a7ec82, SUC_NE_lexicon_tags_2 },
    { 0x5d75ec83, SUC_NE_lexicon_tags_4 },
    { 0xacf3ec84, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f19ec87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f35ec89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abec8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ec8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x849dec91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf849ec93, SUC_NE_lexicon_tags_2 },
    { 0xcd95ec94, SUC_NE_lexicon_tags_2 },
    { 0xd509ec94, SUC_NE_lexicon_tags_17 },
    { 0x1101ec95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x887fec98, SUC_NE_lexicon_tags_7 },
    { 0xbcb5ec99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66c7ec9b, SUC_NE_lexicon_tags_2 },
    { 0x4cc3ec9c, SUC_NE_lexicon_tags_2 },
    { 0xaa07ec9d, SUC_NE_lexicon_tags_2 },
    { 0xc217ec9e, SUC_NE_lexicon_tags_19 },
    { 0x65dfec9b, SUC_NE_lexicon_tags_2 },
    { 0xfc63eca0, SUC_NE_lexicon_tags_2 },
    { 0xcdc5eca1, SUC_NE_lexicon_tags_2 },
    { 0x29afeca2, SUC_NE_lexicon_tags_2 },
    { 0xa011eca0, SUC_NE_lexicon_tags_2 },
    { 0xf7e9eca4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e57eca7, SUC_NE_lexicon_tags_5 },
    { 0xf913eca8, SUC_NE_lexicon_tags_2 },
    { 0x63b9eca9, SUC_NE_lexicon_tags_7 },
    { 0x5a0feca8, SUC_NE_lexicon_tags_2 },
    { 0xf47feca9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xe01fecad, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3abecb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbe9ecb6, SUC_NE_lexicon_tags_17 },
    { 0xb3cdecb7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dc3ecb9, SUC_NE_lexicon_tags_2 },
    { 0x7119ecb9, SUC_NE_lexicon_tags_2 },
    { 0x30ffecb9, SUC_NE_lexicon_tags_2 },
    { 0x9d77ecbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc71becbe, SUC_NE_lexicon_tags_2 },
    { 0xda61ecbf, SUC_NE_lexicon_tags_10 },
    { 0x8f4fecbf, SUC_NE_lexicon_tags_2 },
    { 0xedc9ecbf, SUC_NE_lexicon_tags_2 },
    { 0x64d9ecc2, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c3ecc8, SUC_NE_lexicon_tags_2 },
    { 0x4d41ecc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9ecd3, SUC_NE_lexicon_tags_2 },
    { 0x2a0fecd3, SUC_NE_lexicon_tags_2 },
    { 0x9a39ecd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9decdd, SUC_NE_lexicon_tags_2 },
    { 0xdf4becdd, SUC_NE_lexicon_tags_2 },
    { 0xf841ecdd, SUC_NE_lexicon_tags_2 },
    { 0x9b7fecde, SUC_NE_lexicon_tags_2 },
    { 0x273decdf, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b9ece4, SUC_NE_lexicon_tags_69 },
    { 0x5c4bece4, SUC_NE_lexicon_tags_2 },
    { 0x4575ece5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f9eceb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd21ecf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb283ecf2, SUC_NE_lexicon_tags_2 },
    { 0xa4bbecf2, SUC_NE_lexicon_tags_2 },
    { 0x11b3ecf4, SUC_NE_lexicon_tags_2 },
    { 0xd38fecf5, SUC_NE_lexicon_tags_2 },
    { 0xdbffecf6, SUC_NE_lexicon_tags_30 },
    { 0xbce5ecf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a9ecfb, SUC_NE_lexicon_tags_7 },
    { 0xb1efecfc, SUC_NE_lexicon_tags_1 },
    { 0xf265ecfd, SUC_NE_lexicon_tags_2 },
    { 0xa985ecfe, SUC_NE_lexicon_tags_2 },
    { 0x3fb5ecff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47bed04, SUC_NE_lexicon_tags_2 },
    { 0xda4fed05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5ed09, SUC_NE_lexicon_tags_2 },
    { 0xca01ed0a, SUC_NE_lexicon_tags_2 },
    { 0x9fd1ed0a, SUC_NE_lexicon_tags_2 },
    { 0x4b93ed0c, SUC_NE_lexicon_tags_2 },
    { 0x7f9fed09, SUC_NE_lexicon_tags_2 },
    { 0x97e7ed0e, SUC_NE_lexicon_tags_2 },
    { 0xdfa5ed0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a5ed12, SUC_NE_lexicon_tags_2 },
    { 0x9d4fed13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3391ed16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cded18, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a97ed1c, SUC_NE_lexicon_tags_2 },
    { 0xb8a3ed1c, SUC_NE_lexicon_tags_2 },
    { 0x7f89ed1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6bed20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb47ed24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a59ed2c, SUC_NE_lexicon_tags_8 },
    { 0xd84fed2d, SUC_NE_lexicon_tags_2 },
    { 0x495bed2e, SUC_NE_lexicon_tags_2 },
    { 0x6fbfed2e, SUC_NE_lexicon_tags_2 },
    { 0xc0bded30, SUC_NE_lexicon_tags_2 },
    { 0x68e9ed2c, SUC_NE_lexicon_tags_2 },
    { 0xa9b3ed32, SUC_NE_lexicon_tags_2 },
    { 0xec7ded2e, SUC_NE_lexicon_tags_4 },
    { 0xadabed34, SUC_NE_lexicon_tags_2 },
    { 0xdd25ed34, SUC_NE_lexicon_tags_2 },
    { 0xdcd9ed34, SUC_NE_lexicon_tags_2 },
    { 0xdadbed2c, SUC_NE_lexicon_tags_2 },
    { 0x2dc9ed32, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x38c9ed3a, SUC_NE_lexicon_tags_2 },
    { 0x5801ed3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3943ed44, SUC_NE_lexicon_tags_2 },
    { 0xc4d3ed45, SUC_NE_lexicon_tags_49 },
    { 0xf7b5ed45, SUC_NE_lexicon_tags_2 },
    { 0x50d9ed47, SUC_NE_lexicon_tags_2 },
    { 0x71ed45, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd721ed4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7657ed4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe655ed57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x474ded5a, SUC_NE_lexicon_tags_2 },
    { 0x78dded5b, SUC_NE_lexicon_tags_2 },
    { 0xd531ed5c, SUC_NE_lexicon_tags_2 },
    { 0x5c51ed5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfde3ed5f, SUC_NE_lexicon_tags_2 },
    { 0xb425ed60, SUC_NE_lexicon_tags_2 },
    { 0xc3e9ed60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd71ed63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b3ed66, SUC_NE_lexicon_tags_2 },
    { 0x95fded67, SUC_NE_lexicon_tags_8 },
    { 0x8541ed68, SUC_NE_lexicon_tags_2 },
    { 0x706fed69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7739ed6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bfed6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4753ed71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e99ed75, SUC_NE_lexicon_tags_2 },
    { 0xcd09ed75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x24efed78, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddfed81, SUC_NE_lexicon_tags_2 },
    { 0xf107ed82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d5ed86, SUC_NE_lexicon_tags_7 },
    { 0xe0abed87, SUC_NE_lexicon_tags_2 },
    { 0x4b8bed88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea69ed8c, SUC_NE_lexicon_tags_28 },
    { 0x1f1fed8d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6ded91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc929ed94, SUC_NE_lexicon_tags_2 },
    { 0xf05ed95, SUC_NE_lexicon_tags_2 },
    { 0xc009ed95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x76f7ed98, SUC_NE_lexicon_tags_2 },
    { 0x9b4ded98, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aded9d, SUC_NE_lexicon_tags_2 },
    { 0x6c65ed9d, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x69fdeda0, SUC_NE_lexicon_tags_19 },
    { 0x32b3eda1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feda5, SUC_NE_lexicon_tags_2 },
    { 0x15efeda6, SUC_NE_lexicon_tags_2 },
    { 0x3007eda7, SUC_NE_lexicon_tags_2 },
    { 0x7799eda7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3b1edaa, SUC_NE_lexicon_tags_2 },
    { 0xa09bedab, SUC_NE_lexicon_tags_418 },
    { 0xd849edac, SUC_NE_lexicon_tags_2 },
    { 0xe6d3edac, SUC_NE_lexicon_tags_2 },
    { 0x3d43edae, SUC_NE_lexicon_tags_2 },
    { 0x7e4dedaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4191edb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1403edb6, SUC_NE_lexicon_tags_2 },
    { 0x22e3edb7, SUC_NE_lexicon_tags_2 },
    { 0xe621edb8, SUC_NE_lexicon_tags_2 },
    { 0xaac7edb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6d5edbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5991edc0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a5edc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e01edc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f49edc8, SUC_NE_lexicon_tags_2 },
    { 0xba7fedc9, SUC_NE_lexicon_tags_2 },
    { 0xa283edc8, SUC_NE_lexicon_tags_2 },
    { 0x447dedca, SUC_NE_lexicon_tags_2 },
    { 0x9e6bedcb, SUC_NE_lexicon_tags_2 },
    { 0x81d5edcd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x735dedcf, SUC_NE_lexicon_tags_2 },
    { 0x738bedd0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe60fedd2, SUC_NE_lexicon_tags_2 },
    { 0x92f5edd3, SUC_NE_lexicon_tags_2 },
    { 0x5181edd2, SUC_NE_lexicon_tags_2 },
    { 0xb19bedd5, SUC_NE_lexicon_tags_2 },
    { 0xe46dedd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a7bedd8, SUC_NE_lexicon_tags_2 },
    { 0xe6bedd9, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d13eddf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991ede3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea31edea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x87bbedec, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e1fedee, SUC_NE_lexicon_tags_25 },
    { 0x164fedee, SUC_NE_lexicon_tags_2 },
    { 0x6299edf0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbadedf3, SUC_NE_lexicon_tags_2 },
    { 0xa1cdedf4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b1edf7, SUC_NE_lexicon_tags_2 },
    { 0x4359edf8, SUC_NE_lexicon_tags_2 },
    { 0xd5b1edf9, SUC_NE_lexicon_tags_2 },
    { 0x414bedf9, SUC_NE_lexicon_tags_2 },
    { 0x96bedfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x995bedfd, SUC_NE_lexicon_tags_25 },
    { 0x51cfedfd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd275ee03, SUC_NE_lexicon_tags_2 },
    { 0x2aabee04, SUC_NE_lexicon_tags_2 },
    { 0x7a19ee05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4441ee09, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7165ee11, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b03ee13, SUC_NE_lexicon_tags_2 },
    { 0x54e7ee14, SUC_NE_lexicon_tags_2 },
    { 0xf12dee15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xde9ee17, SUC_NE_lexicon_tags_18 },
    { 0x8967ee17, SUC_NE_lexicon_tags_2 },
    { 0xd617ee19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3881ee1b, SUC_NE_lexicon_tags_2 },
    { 0x89c3ee1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8e3ee1e, SUC_NE_lexicon_tags_2 },
    { 0x65edee1e, SUC_NE_lexicon_tags_2 },
    { 0x3435ee1e, SUC_NE_lexicon_tags_2 },
    { 0x35bbee1f, SUC_NE_lexicon_tags_2 },
    { 0xdb91ee21, SUC_NE_lexicon_tags_2 },
    { 0xe8dfee23, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa393ee25, SUC_NE_lexicon_tags_2 },
    { 0xb09fee25, SUC_NE_lexicon_tags_2 },
    { 0xa841ee27, SUC_NE_lexicon_tags_2 },
    { 0x2b01ee25, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xff2fee2a, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x257bee2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925bee36, SUC_NE_lexicon_tags_3 },
    { 0xc5bee37, SUC_NE_lexicon_tags_2 },
    { 0x8981ee37, SUC_NE_lexicon_tags_2 },
    { 0x3317ee38, SUC_NE_lexicon_tags_4 },
    { 0xdf01ee37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc1ee3d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43ee4f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x423dee52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21cfee54, SUC_NE_lexicon_tags_2 },
    { 0x3c2dee55, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d77ee5e, SUC_NE_lexicon_tags_2 },
    { 0xcc39ee5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb147ee63, SUC_NE_lexicon_tags_2 },
    { 0xfe7dee64, SUC_NE_lexicon_tags_4 },
    { 0x9027ee65, SUC_NE_lexicon_tags_2 },
    { 0xa173ee63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e7ee6d, SUC_NE_lexicon_tags_2 },
    { 0x886dee6e, SUC_NE_lexicon_tags_2 },
    { 0x7bd5ee6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5ee71, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8449ee74, SUC_NE_lexicon_tags_2 },
    { 0x7291ee75, SUC_NE_lexicon_tags_2 },
    { 0x3b5ee76, SUC_NE_lexicon_tags_2 },
    { 0x418fee76, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e39ee7b, SUC_NE_lexicon_tags_0 },
    { 0xf68dee7c, SUC_NE_lexicon_tags_2 },
    { 0xfe47ee7d, SUC_NE_lexicon_tags_4 },
    { 0x3013ee7d, SUC_NE_lexicon_tags_3 },
    { 0x70fbee7f, SUC_NE_lexicon_tags_2 },
    { 0x39a1ee80, SUC_NE_lexicon_tags_2 },
    { 0xc379ee80, SUC_NE_lexicon_tags_17 },
    { 0xa1b3ee7f, SUC_NE_lexicon_tags_2 },
    { 0xac75ee7c, SUC_NE_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41dee86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b6bee88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb7ee8d, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0dee90, SUC_NE_lexicon_tags_2 },
    { 0x5a37ee91, SUC_NE_lexicon_tags_2 },
    { 0x2b77ee92, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b53ee9b, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3d9ee9d, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e39eea0, SUC_NE_lexicon_tags_4 },
    { 0x1aa1eea1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf599eea3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a1eeab, SUC_NE_lexicon_tags_2 },
    { 0x88ebeeac, SUC_NE_lexicon_tags_2 },
    { 0x7d35eeab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3537eeb0, SUC_NE_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaafeeb6, SUC_NE_lexicon_tags_2 },
    { 0x3917eeb7, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c3eebe, SUC_NE_lexicon_tags_330 },
    { 0, NULL },
    { 0, NULL },
    { 0x7debeec1, SUC_NE_lexicon_tags_2 },
    { 0x7cc9eec2, SUC_NE_lexicon_tags_2 },
    { 0xb789eec3, SUC_NE_lexicon_tags_40 },
    { 0x57bdeec4, SUC_NE_lexicon_tags_4 },
    { 0x3877eec5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfbeec8, SUC_NE_lexicon_tags_2 },
    { 0xde1eec9, SUC_NE_lexicon_tags_2 },
    { 0x75b7eeca, SUC_NE_lexicon_tags_2 },
    { 0x38beecb, SUC_NE_lexicon_tags_2 },
    { 0xfc01eeca, SUC_NE_lexicon_tags_2 },
    { 0xfcffeecd, SUC_NE_lexicon_tags_2 },
    { 0x6879eece, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe981eed5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bf3eed7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe75eed9, SUC_NE_lexicon_tags_2 },
    { 0xaf69eeda, SUC_NE_lexicon_tags_2 },
    { 0x1c77eeda, SUC_NE_lexicon_tags_2 },
    { 0xbb7beedc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f09eedf, SUC_NE_lexicon_tags_2 },
    { 0x5793eedf, SUC_NE_lexicon_tags_8 },
    { 0x86bfeedf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b93eee4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a1eee9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcdeeef, SUC_NE_lexicon_tags_2 },
    { 0xd94beef0, SUC_NE_lexicon_tags_2 },
    { 0x654beef1, SUC_NE_lexicon_tags_2 },
    { 0x8623eef0, SUC_NE_lexicon_tags_2 },
    { 0xa78deef2, SUC_NE_lexicon_tags_3 },
    { 0xe0bdeef3, SUC_NE_lexicon_tags_2 },
    { 0x56c3eef5, SUC_NE_lexicon_tags_3 },
    { 0x86c1eef6, SUC_NE_lexicon_tags_398 },
    { 0xc9fdeef7, SUC_NE_lexicon_tags_2 },
    { 0xc345eef8, SUC_NE_lexicon_tags_2 },
    { 0xd439eef1, SUC_NE_lexicon_tags_2 },
    { 0xee41eefa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00feefe, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3927ef03, SUC_NE_lexicon_tags_2 },
    { 0xaf9bef04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19ef07, SUC_NE_lexicon_tags_2 },
    { 0x5751ef07, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6a5ef0a, SUC_NE_lexicon_tags_40 },
    { 0xd58fef0a, SUC_NE_lexicon_tags_2 },
    { 0x3a69ef0c, SUC_NE_lexicon_tags_2 },
    { 0x3989ef0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5227ef13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769def1a, SUC_NE_lexicon_tags_2 },
    { 0xac37ef1a, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0x9b83ef1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef24, SUC_NE_lexicon_tags_2 },
    { 0x32b3ef25, SUC_NE_lexicon_tags_2 },
    { 0x9cd1ef26, SUC_NE_lexicon_tags_2 },
    { 0xa221ef26, SUC_NE_lexicon_tags_2 },
    { 0xb7bfef28, SUC_NE_lexicon_tags_2 },
    { 0x4845ef29, SUC_NE_lexicon_tags_2 },
    { 0xa721ef2a, SUC_NE_lexicon_tags_2 },
    { 0x5623ef2b, SUC_NE_lexicon_tags_2 },
    { 0x61ef2c, SUC_NE_lexicon_tags_14 },
    { 0x2e9fef2b, SUC_NE_lexicon_tags_2 },
    { 0xf8b5ef2a, SUC_NE_lexicon_tags_2 },
    { 0xe249ef2d, SUC_NE_lexicon_tags_2 },
    { 0xcd6fef24, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dfef35, SUC_NE_lexicon_tags_2 },
    { 0xf50fef36, SUC_NE_lexicon_tags_2 },
    { 0xd231ef36, SUC_NE_lexicon_tags_36 },
    { 0xf8e3ef38, SUC_NE_lexicon_tags_2 },
    { 0x4f9ef39, SUC_NE_lexicon_tags_40 },
    { 0x71c5ef39, SUC_NE_lexicon_tags_2 },
    { 0x26f3ef3a, SUC_NE_lexicon_tags_2 },
    { 0x557bef3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabafef42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a7ef47, SUC_NE_lexicon_tags_2 },
    { 0x7409ef47, SUC_NE_lexicon_tags_59 },
    { 0x3ec7ef47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2159ef4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6def52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3ef59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1691ef5b, SUC_NE_lexicon_tags_338 },
    { 0xc339ef5c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x23b5ef5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xffe7ef60, SUC_NE_lexicon_tags_4 },
    { 0xa27def60, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddffef64, SUC_NE_lexicon_tags_2 },
    { 0x572fef64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e47ef67, SUC_NE_lexicon_tags_2 },
    { 0x1e27ef68, SUC_NE_lexicon_tags_2 },
    { 0xb753ef69, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7ef6c, SUC_NE_lexicon_tags_2 },
    { 0x65ffef6d, SUC_NE_lexicon_tags_2 },
    { 0xadf1ef6e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd1ef72, SUC_NE_lexicon_tags_2 },
    { 0xd86def72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb88fef75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef3ef7a, SUC_NE_lexicon_tags_2 },
    { 0xc371ef7b, SUC_NE_lexicon_tags_2 },
    { 0x157bef7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2acfef7e, SUC_NE_lexicon_tags_2 },
    { 0xc279ef7e, SUC_NE_lexicon_tags_2 },
    { 0xa90def80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x418def82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bdef86, SUC_NE_lexicon_tags_2 },
    { 0x10fdef87, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a93ef8b, SUC_NE_lexicon_tags_2 },
    { 0x62e9ef8b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7979ef95, SUC_NE_lexicon_tags_2 },
    { 0x1ae1ef96, SUC_NE_lexicon_tags_2 },
    { 0xc27bef97, SUC_NE_lexicon_tags_2 },
    { 0x7edef96, SUC_NE_lexicon_tags_2 },
    { 0xe947ef99, SUC_NE_lexicon_tags_2 },
    { 0x6c69ef99, SUC_NE_lexicon_tags_2 },
    { 0xbe11ef9b, SUC_NE_lexicon_tags_2 },
    { 0xd495ef9c, SUC_NE_lexicon_tags_40 },
    { 0x4233ef9d, SUC_NE_lexicon_tags_0 },
    { 0x98b1ef9d, SUC_NE_lexicon_tags_2 },
    { 0xf42fef9f, SUC_NE_lexicon_tags_2 },
    { 0x4b53ef9e, SUC_NE_lexicon_tags_2 },
    { 0x7f83efa0, SUC_NE_lexicon_tags_2 },
    { 0x45b3efa2, SUC_NE_lexicon_tags_2 },
    { 0x1f9befa3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e7efa6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7efa9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8897efac, SUC_NE_lexicon_tags_2 },
    { 0xcf85efac, SUC_NE_lexicon_tags_2 },
    { 0x5c9defae, SUC_NE_lexicon_tags_2 },
    { 0x2955efad, SUC_NE_lexicon_tags_2 },
    { 0xedd7efb0, SUC_NE_lexicon_tags_2 },
    { 0xc497efb0, SUC_NE_lexicon_tags_2 },
    { 0x5c65efb2, SUC_NE_lexicon_tags_2 },
    { 0xf221efb1, SUC_NE_lexicon_tags_2 },
    { 0xd81defb1, SUC_NE_lexicon_tags_2 },
    { 0x8c5defb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbdefba, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b7efbe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd177efc0, SUC_NE_lexicon_tags_2 },
    { 0xc82defc1, SUC_NE_lexicon_tags_2 },
    { 0x45f1efc0, SUC_NE_lexicon_tags_2 },
    { 0x811fefc2, SUC_NE_lexicon_tags_2 },
    { 0xca6defc4, SUC_NE_lexicon_tags_2 },
    { 0x4d29efc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3757efc8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e7efcb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb61efcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eedefd3, SUC_NE_lexicon_tags_2 },
    { 0xa82defd4, SUC_NE_lexicon_tags_2 },
    { 0xa201efd4, SUC_NE_lexicon_tags_2 },
    { 0xa417efd6, SUC_NE_lexicon_tags_2 },
    { 0x9145efd4, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x17b1efd9, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0x961fefdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4189efe1, SUC_NE_lexicon_tags_2 },
    { 0x3dffefe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa599efe4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33fefef, SUC_NE_lexicon_tags_7 },
    { 0xab7beff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f5deff2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd479eff6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xd4adeff8, SUC_NE_lexicon_tags_2 },
    { 0xcfbeff8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb457effb, SUC_NE_lexicon_tags_4 },
    { 0xfa77effb, SUC_NE_lexicon_tags_2 },
    { 0x231feffd, SUC_NE_lexicon_tags_2 },
    { 0x4601effe, SUC_NE_lexicon_tags_2 },
    { 0xd3d7effb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f5f005, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeeebf007, SUC_NE_lexicon_tags_2 },
    { 0x4897f007, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x78dbf00a, SUC_NE_lexicon_tags_2 },
    { 0x7adf00b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae5f00f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caff013, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9ff016, SUC_NE_lexicon_tags_2 },
    { 0x8d03f017, SUC_NE_lexicon_tags_2 },
    { 0xb199f017, SUC_NE_lexicon_tags_2 },
    { 0xc015f019, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf437f023, SUC_NE_lexicon_tags_32 },
    { 0xf4f9f023, SUC_NE_lexicon_tags_2 },
    { 0x7e05f025, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca1f028, SUC_NE_lexicon_tags_2 },
    { 0x883f028, SUC_NE_lexicon_tags_2 },
    { 0x4bf3f029, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc37bf02c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d1f02f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf5f033, SUC_NE_lexicon_tags_2 },
    { 0x83b3f034, SUC_NE_lexicon_tags_2 },
    { 0xdb57f033, SUC_NE_lexicon_tags_2 },
    { 0x2a1f035, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf721f039, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1247f03c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee7f040, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a5f043, SUC_NE_lexicon_tags_2 },
    { 0x8f7ff044, SUC_NE_lexicon_tags_4 },
    { 0xd939f045, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bd5f047, SUC_NE_lexicon_tags_2 },
    { 0x3161f047, SUC_NE_lexicon_tags_2 },
    { 0x5ec3f047, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8693f04b, SUC_NE_lexicon_tags_2 },
    { 0xda29f04c, SUC_NE_lexicon_tags_2 },
    { 0xc327f04d, SUC_NE_lexicon_tags_2 },
    { 0x971bf04e, SUC_NE_lexicon_tags_2 },
    { 0x486ff04c, SUC_NE_lexicon_tags_2 },
    { 0xab41f050, SUC_NE_lexicon_tags_2 },
    { 0xc8fdf04d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x141f053, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95bf059, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf20df05b, SUC_NE_lexicon_tags_2 },
    { 0x3b8ff05c, SUC_NE_lexicon_tags_2 },
    { 0xe805f05d, SUC_NE_lexicon_tags_2 },
    { 0xce0df05d, SUC_NE_lexicon_tags_2 },
    { 0xd011f05f, SUC_NE_lexicon_tags_328 },
    { 0xab6bf060, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ff065, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cff067, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69abf06e, SUC_NE_lexicon_tags_2 },
    { 0x1fc1f06e, SUC_NE_lexicon_tags_2 },
    { 0xbb13f070, SUC_NE_lexicon_tags_2 },
    { 0xce7f070, SUC_NE_lexicon_tags_2 },
    { 0x86e1f072, SUC_NE_lexicon_tags_2 },
    { 0x897ff070, SUC_NE_lexicon_tags_2 },
    { 0xe19df074, SUC_NE_lexicon_tags_2 },
    { 0xc3b7f073, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x166ff078, SUC_NE_lexicon_tags_2 },
    { 0xe68bf078, SUC_NE_lexicon_tags_2 },
    { 0xf359f07a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x306df07c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e7f080, SUC_NE_lexicon_tags_2 },
    { 0xee89f080, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5bf085, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3afff087, SUC_NE_lexicon_tags_2 },
    { 0x2bb3f087, SUC_NE_lexicon_tags_2 },
    { 0xd56bf089, SUC_NE_lexicon_tags_2 },
    { 0x33f1f08a, SUC_NE_lexicon_tags_32 },
    { 0xeafff088, SUC_NE_lexicon_tags_2 },
    { 0x5e7bf08c, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x3e3f08e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c1f094, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e69f096, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x845df098, SUC_NE_lexicon_tags_2 },
    { 0xccbf098, SUC_NE_lexicon_tags_17 },
    { 0xe149f098, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb7f09e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x878df0a1, SUC_NE_lexicon_tags_2 },
    { 0x4f35f0a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x787df0a4, SUC_NE_lexicon_tags_2 },
    { 0x586df0a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf067f0a7, SUC_NE_lexicon_tags_2 },
    { 0x70c5f0a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3f0b3, SUC_NE_lexicon_tags_3 },
    { 0xf917f0b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f35f0ba, SUC_NE_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd7f0bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6bf0c0, SUC_NE_lexicon_tags_2 },
    { 0x65b7f0c1, SUC_NE_lexicon_tags_2 },
    { 0x54c5f0c2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e5f0c5, SUC_NE_lexicon_tags_2 },
    { 0x73c3f0c5, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bbf0cc, SUC_NE_lexicon_tags_51 },
    { 0, NULL },
    { 0x27c1f0ce, SUC_NE_lexicon_tags_2 },
    { 0x5c33f0ce, SUC_NE_lexicon_tags_4 },
    { 0x484bf0d0, SUC_NE_lexicon_tags_2 },
    { 0x7c23f0d1, SUC_NE_lexicon_tags_2 },
    { 0x8687f0d2, SUC_NE_lexicon_tags_3 },
    { 0x8f25f0d3, SUC_NE_lexicon_tags_2 },
    { 0x8af9f0d4, SUC_NE_lexicon_tags_4 },
    { 0x206bf0ce, SUC_NE_lexicon_tags_1 },
    { 0x7bd3f0d6, SUC_NE_lexicon_tags_2 },
    { 0xce79f0d4, SUC_NE_lexicon_tags_2 },
    { 0xc5b1f0d2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x563ff0db, SUC_NE_lexicon_tags_2 },
    { 0x29bff0dc, SUC_NE_lexicon_tags_2 },
    { 0xcad9f0dd, SUC_NE_lexicon_tags_2 },
    { 0xe16bf0de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x353ff0e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e95f0e5, SUC_NE_lexicon_tags_2 },
    { 0xc5fdf0e6, SUC_NE_lexicon_tags_2 },
    { 0xda33f0e7, SUC_NE_lexicon_tags_2 },
    { 0x2325f0e7, SUC_NE_lexicon_tags_2 },
    { 0x9697f0e8, SUC_NE_lexicon_tags_2 },
    { 0xc683f0e9, SUC_NE_lexicon_tags_2 },
    { 0xb8f3f0eb, SUC_NE_lexicon_tags_2 },
    { 0x542bf0ec, SUC_NE_lexicon_tags_2 },
    { 0xb8abf0ed, SUC_NE_lexicon_tags_2 },
    { 0xcda9f0ed, SUC_NE_lexicon_tags_2 },
    { 0x7dc9f0eb, SUC_NE_lexicon_tags_2 },
    { 0x998df0e7, SUC_NE_lexicon_tags_19 },
    { 0xdb7f0e6, SUC_NE_lexicon_tags_2 },
    { 0xeac9f0f1, SUC_NE_lexicon_tags_2 },
    { 0x1869f0f3, SUC_NE_lexicon_tags_2 },
    { 0x214bf0ee, SUC_NE_lexicon_tags_2 },
    { 0xaaf1f0f5, SUC_NE_lexicon_tags_2 },
    { 0x61b9f0f6, SUC_NE_lexicon_tags_2 },
    { 0xe30df0f7, SUC_NE_lexicon_tags_2 },
    { 0x225ff0f6, SUC_NE_lexicon_tags_2 },
    { 0x38d7f0f2, SUC_NE_lexicon_tags_2 },
    { 0xb60bf0fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd61df0fc, SUC_NE_lexicon_tags_2 },
    { 0x3c8df0fd, SUC_NE_lexicon_tags_2 },
    { 0x67ddf0fe, SUC_NE_lexicon_tags_2 },
    { 0x6695f0fd, SUC_NE_lexicon_tags_2 },
    { 0xb80bf100, SUC_NE_lexicon_tags_2 },
    { 0xd79f0fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e9f105, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e6df107, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d3bf109, SUC_NE_lexicon_tags_2 },
    { 0x1437f10a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1bf10d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709f111, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae31f115, SUC_NE_lexicon_tags_2 },
    { 0xa8bff116, SUC_NE_lexicon_tags_2 },
    { 0x9385f117, SUC_NE_lexicon_tags_2 },
    { 0x3b59f117, SUC_NE_lexicon_tags_2 },
    { 0x6cfdf117, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a95f11b, SUC_NE_lexicon_tags_2 },
    { 0x8fbbf11b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x52fff11e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bedf121, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x259bf123, SUC_NE_lexicon_tags_7 },
    { 0xb4e5f124, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b73f127, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebebf12f, SUC_NE_lexicon_tags_7 },
    { 0x8f5ff130, SUC_NE_lexicon_tags_17 },
    { 0x4773f131, SUC_NE_lexicon_tags_7 },
    { 0x5c01f132, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7369f137, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fff13f, SUC_NE_lexicon_tags_2 },
    { 0x7637f140, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb39f148, SUC_NE_lexicon_tags_17 },
    { 0x820bf149, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ff150, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb307f154, SUC_NE_lexicon_tags_19 },
    { 0xcbb9f155, SUC_NE_lexicon_tags_2 },
    { 0x5ca3f154, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452bf15b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd49f15f, SUC_NE_lexicon_tags_2 },
    { 0x473f15f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817df164, SUC_NE_lexicon_tags_444 },
    { 0x8edf165, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6525f168, SUC_NE_lexicon_tags_2 },
    { 0x6453f169, SUC_NE_lexicon_tags_2 },
    { 0xf2c9f168, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7acff16c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87f171, SUC_NE_lexicon_tags_2 },
    { 0x28b3f171, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7df175, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x189f177, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbbf17a, SUC_NE_lexicon_tags_2 },
    { 0xd393f17b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fff17e, SUC_NE_lexicon_tags_2 },
    { 0xb8a3f17f, SUC_NE_lexicon_tags_2 },
    { 0x338bf17f, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd117f185, SUC_NE_lexicon_tags_0 },
    { 0xd84ff186, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x913f188, SUC_NE_lexicon_tags_2 },
    { 0x5faff189, SUC_NE_lexicon_tags_2 },
    { 0x3827f18a, SUC_NE_lexicon_tags_4 },
    { 0x7e41f189, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5867f18f, SUC_NE_lexicon_tags_2 },
    { 0x11bff190, SUC_NE_lexicon_tags_2 },
    { 0x6c6df191, SUC_NE_lexicon_tags_3 },
    { 0x1bf9f192, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x119df195, SUC_NE_lexicon_tags_25 },
    { 0xc80ff196, SUC_NE_lexicon_tags_2 },
    { 0x4dc3f196, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2c5f199, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b91f19c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b65f1a0, SUC_NE_lexicon_tags_2 },
    { 0xd35df1a0, SUC_NE_lexicon_tags_2 },
    { 0x1e21f1a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e7f1a9, SUC_NE_lexicon_tags_0 },
    { 0x607bf1aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588bf1af, SUC_NE_lexicon_tags_2 },
    { 0xbdfff1b0, SUC_NE_lexicon_tags_2 },
    { 0x739bf1b0, SUC_NE_lexicon_tags_17 },
    { 0x3729f1b2, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x8901f1b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7485f1b6, SUC_NE_lexicon_tags_2 },
    { 0x3fc5f1b7, SUC_NE_lexicon_tags_2 },
    { 0x2ef1f1b6, SUC_NE_lexicon_tags_2 },
    { 0x717ff1b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b9f1bc, SUC_NE_lexicon_tags_2 },
    { 0x1edbf1bd, SUC_NE_lexicon_tags_2 },
    { 0xb12df1be, SUC_NE_lexicon_tags_2 },
    { 0xe181f1bd, SUC_NE_lexicon_tags_2 },
    { 0x6d15f1c0, SUC_NE_lexicon_tags_2 },
    { 0x9bbff1c0, SUC_NE_lexicon_tags_2 },
    { 0x4b7f1c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf9f1cd, SUC_NE_lexicon_tags_2 },
    { 0xcb21f1cd, SUC_NE_lexicon_tags_1 },
    { 0xef35f1ce, SUC_NE_lexicon_tags_2 },
    { 0xf70df1ce, SUC_NE_lexicon_tags_2 },
    { 0xc6bbf1d1, SUC_NE_lexicon_tags_0 },
    { 0xb7f3f1d1, SUC_NE_lexicon_tags_2 },
    { 0x6975f1d3, SUC_NE_lexicon_tags_2 },
    { 0xcbb7f1cf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3afff1d6, SUC_NE_lexicon_tags_2 },
    { 0x8673f1d6, SUC_NE_lexicon_tags_2 },
    { 0x9d9ff1d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc483f1da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56bf1df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf927f1e1, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fbf1e7, SUC_NE_lexicon_tags_2 },
    { 0x5a1ff1e8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1df1f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85bf1f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3531f1fa, SUC_NE_lexicon_tags_2 },
    { 0xa557f1fa, SUC_NE_lexicon_tags_2 },
    { 0xe455f1fb, SUC_NE_lexicon_tags_2 },
    { 0xb2ddf1fb, SUC_NE_lexicon_tags_2 },
    { 0x7a37f1fc, SUC_NE_lexicon_tags_3 },
    { 0xdd31f1ff, SUC_NE_lexicon_tags_2 },
    { 0xf9f1ff, SUC_NE_lexicon_tags_2 },
    { 0xb18bf201, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a59f203, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x9351f205, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55fff207, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1f20d, SUC_NE_lexicon_tags_2 },
    { 0x912df20e, SUC_NE_lexicon_tags_2 },
    { 0xf4f5f20f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1501f212, SUC_NE_lexicon_tags_7 },
    { 0x5057f213, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3925f215, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa3f218, SUC_NE_lexicon_tags_21 },
    { 0x1c2ff219, SUC_NE_lexicon_tags_2 },
    { 0x24cff21a, SUC_NE_lexicon_tags_2 },
    { 0x978df21a, SUC_NE_lexicon_tags_2 },
    { 0x63c9f21b, SUC_NE_lexicon_tags_2 },
    { 0x551bf21d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf603f220, SUC_NE_lexicon_tags_2 },
    { 0x66cff221, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f1df223, SUC_NE_lexicon_tags_85 },
    { 0x300df224, SUC_NE_lexicon_tags_2 },
    { 0x84bbf225, SUC_NE_lexicon_tags_2 },
    { 0x8973f226, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9acbf228, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4b3f22a, SUC_NE_lexicon_tags_2 },
    { 0xd2f9f22b, SUC_NE_lexicon_tags_2 },
    { 0x763bf22a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ca7f22e, SUC_NE_lexicon_tags_2 },
    { 0x8f99f22e, SUC_NE_lexicon_tags_2 },
    { 0xde1f22f, SUC_NE_lexicon_tags_2 },
    { 0x193df231, SUC_NE_lexicon_tags_2 },
    { 0x847bf232, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a2bf234, SUC_NE_lexicon_tags_2 },
    { 0x47b9f235, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x983ff238, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b51f240, SUC_NE_lexicon_tags_2 },
    { 0xd975f240, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483bf249, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9db5f24b, SUC_NE_lexicon_tags_2 },
    { 0x4a77f24b, SUC_NE_lexicon_tags_2 },
    { 0x52a5f24d, SUC_NE_lexicon_tags_17 },
    { 0x8ab7f24e, SUC_NE_lexicon_tags_2 },
    { 0x32b1f24f, SUC_NE_lexicon_tags_2 },
    { 0x66b7f250, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d1f255, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x937f257, SUC_NE_lexicon_tags_2 },
    { 0x7f7ff258, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x18bf25a, SUC_NE_lexicon_tags_3 },
    { 0xe3f25b, SUC_NE_lexicon_tags_2 },
    { 0xeb21f25c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1bf25f, SUC_NE_lexicon_tags_2 },
    { 0xbf17f25f, SUC_NE_lexicon_tags_2 },
    { 0x8a9f261, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb19f263, SUC_NE_lexicon_tags_2 },
    { 0xef99f263, SUC_NE_lexicon_tags_2 },
    { 0x3de1f265, SUC_NE_lexicon_tags_2 },
    { 0x6c23f266, SUC_NE_lexicon_tags_2 },
    { 0xb317f266, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46a5f269, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ff26c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a3f26f, SUC_NE_lexicon_tags_2 },
    { 0x5ff3f270, SUC_NE_lexicon_tags_2 },
    { 0xef71f271, SUC_NE_lexicon_tags_2 },
    { 0xe8f1f272, SUC_NE_lexicon_tags_2 },
    { 0x6d7ff273, SUC_NE_lexicon_tags_2 },
    { 0xf129f274, SUC_NE_lexicon_tags_2 },
    { 0xbefdf274, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeba5f277, SUC_NE_lexicon_tags_2 },
    { 0x6435f278, SUC_NE_lexicon_tags_2 },
    { 0xaecdf279, SUC_NE_lexicon_tags_2 },
    { 0xe35df27a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e4bf27c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0bf27e, SUC_NE_lexicon_tags_2 },
    { 0x5503f27f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa351f281, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2bf284, SUC_NE_lexicon_tags_2 },
    { 0xae9ff284, SUC_NE_lexicon_tags_2 },
    { 0x1f4df285, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa6ff288, SUC_NE_lexicon_tags_2 },
    { 0x1a57f289, SUC_NE_lexicon_tags_2 },
    { 0x8cff28a, SUC_NE_lexicon_tags_2 },
    { 0x883df28a, SUC_NE_lexicon_tags_1 },
    { 0x3f41f28b, SUC_NE_lexicon_tags_2 },
    { 0x6225f28d, SUC_NE_lexicon_tags_7 },
    { 0xb421f28e, SUC_NE_lexicon_tags_2 },
    { 0x479f28e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x46e5f291, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1057f294, SUC_NE_lexicon_tags_2 },
    { 0x4da5f295, SUC_NE_lexicon_tags_2 },
    { 0x401bf295, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d5f29b, SUC_NE_lexicon_tags_2 },
    { 0xb9dff29c, SUC_NE_lexicon_tags_2 },
    { 0x295df29d, SUC_NE_lexicon_tags_2 },
    { 0xfd53f29b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x535df2a0, SUC_NE_lexicon_tags_0 },
    { 0xf2b5f2a1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3623f2a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4edf2ab, SUC_NE_lexicon_tags_7 },
    { 0x2f17f2ab, SUC_NE_lexicon_tags_2 },
    { 0x87e5f2ad, SUC_NE_lexicon_tags_2 },
    { 0xb7e9f2ac, SUC_NE_lexicon_tags_2 },
    { 0xa6b1f2ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8671f2b5, SUC_NE_lexicon_tags_4 },
    { 0x6b23f2b6, SUC_NE_lexicon_tags_2 },
    { 0xb117f2b7, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9febf2ba, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9381f2bd, SUC_NE_lexicon_tags_2 },
    { 0x34fbf2bd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2df7f2c0, SUC_NE_lexicon_tags_2 },
    { 0xe10bf2c1, SUC_NE_lexicon_tags_2 },
    { 0xcc0df2c0, SUC_NE_lexicon_tags_72 },
    { 0xc047f2c3, SUC_NE_lexicon_tags_2 },
    { 0x36bf2c4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdccbf2c6, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0x99c5f2c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe963f2cd, SUC_NE_lexicon_tags_2 },
    { 0xd395f2cd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62abf2d3, SUC_NE_lexicon_tags_2 },
    { 0x76d1f2d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa251f2d6, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9ff2dd, SUC_NE_lexicon_tags_2 },
    { 0x262ff2dd, SUC_NE_lexicon_tags_2 },
    { 0x397bf2df, SUC_NE_lexicon_tags_2 },
    { 0x3b3ff2e0, SUC_NE_lexicon_tags_2 },
    { 0x5489f2e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x720df2e4, SUC_NE_lexicon_tags_2 },
    { 0x5269f2e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b33f2e9, SUC_NE_lexicon_tags_2 },
    { 0x1993f2e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ff2ed, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4423f2f1, SUC_NE_lexicon_tags_2 },
    { 0x25a5f2f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48ff2f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41ff2f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a45f2fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5665f2fc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e2df2fe, SUC_NE_lexicon_tags_2 },
    { 0x8d9ff2ff, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58d9f301, SUC_NE_lexicon_tags_2 },
    { 0x3229f301, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b23f307, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a5bf309, SUC_NE_lexicon_tags_2 },
    { 0xac01f30a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03f30d, SUC_NE_lexicon_tags_22 },
    { 0, NULL },
    { 0xe47ff30f, SUC_NE_lexicon_tags_2 },
    { 0x4799f310, SUC_NE_lexicon_tags_2 },
    { 0x93d9f311, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf9f315, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659f31d, SUC_NE_lexicon_tags_2 },
    { 0xe8ddf31d, SUC_NE_lexicon_tags_2 },
    { 0x7e8bf31f, SUC_NE_lexicon_tags_2 },
    { 0xde27f320, SUC_NE_lexicon_tags_2 },
    { 0x4d83f321, SUC_NE_lexicon_tags_2 },
    { 0xc2b3f321, SUC_NE_lexicon_tags_2 },
    { 0xdb4df321, SUC_NE_lexicon_tags_4 },
    { 0x5afff323, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf33ff326, SUC_NE_lexicon_tags_2 },
    { 0xc171f326, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b95f329, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6991f32c, SUC_NE_lexicon_tags_2 },
    { 0xd037f32d, SUC_NE_lexicon_tags_2 },
    { 0xf24bf32e, SUC_NE_lexicon_tags_51 },
    { 0xe5d9f32f, SUC_NE_lexicon_tags_2 },
    { 0x56d9f32e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf67f333, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1883f337, SUC_NE_lexicon_tags_2 },
    { 0x6773f338, SUC_NE_lexicon_tags_2 },
    { 0xf297f339, SUC_NE_lexicon_tags_2 },
    { 0xec2ff33a, SUC_NE_lexicon_tags_2 },
    { 0x2229f338, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e9bf33d, SUC_NE_lexicon_tags_4 },
    { 0x11bbf33d, SUC_NE_lexicon_tags_4 },
    { 0xb1a1f33f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7653f345, SUC_NE_lexicon_tags_40 },
    { 0xba81f346, SUC_NE_lexicon_tags_2 },
    { 0x2a93f346, SUC_NE_lexicon_tags_2 },
    { 0xbe69f348, SUC_NE_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5bf34b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2141f34d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa1f350, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5803f352, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe81f354, SUC_NE_lexicon_tags_2 },
    { 0xd2f1f354, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d9f357, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xad5df359, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a5f35c, SUC_NE_lexicon_tags_2 },
    { 0x23aff35d, SUC_NE_lexicon_tags_2 },
    { 0xb5b9f35c, SUC_NE_lexicon_tags_2 },
    { 0x6173f35f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6375f364, SUC_NE_lexicon_tags_2 },
    { 0x1361f364, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd3f368, SUC_NE_lexicon_tags_2 },
    { 0xae9ff369, SUC_NE_lexicon_tags_2 },
    { 0xb88ff36a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c7f370, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb331f376, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x58a3f378, SUC_NE_lexicon_tags_2 },
    { 0xa6cdf378, SUC_NE_lexicon_tags_2 },
    { 0x683f37a, SUC_NE_lexicon_tags_2 },
    { 0x4305f37b, SUC_NE_lexicon_tags_2 },
    { 0xd477f37a, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e4bf37e, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c1f381, SUC_NE_lexicon_tags_3 },
    { 0xba5bf382, SUC_NE_lexicon_tags_2 },
    { 0x1bbdf383, SUC_NE_lexicon_tags_2 },
    { 0xda33f381, SUC_NE_lexicon_tags_2 },
    { 0xacaff382, SUC_NE_lexicon_tags_356 },
    { 0xcc55f385, SUC_NE_lexicon_tags_107 },
    { 0xf399f387, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f99f389, SUC_NE_lexicon_tags_2 },
    { 0xfc3ff38a, SUC_NE_lexicon_tags_3 },
    { 0xb4cff389, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41df38e, SUC_NE_lexicon_tags_2 },
    { 0x18d1f38f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe1f393, SUC_NE_lexicon_tags_4 },
    { 0x3f21f394, SUC_NE_lexicon_tags_2 },
    { 0xaf73f395, SUC_NE_lexicon_tags_2 },
    { 0x9383f395, SUC_NE_lexicon_tags_2 },
    { 0x8ea3f393, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b3f39d, SUC_NE_lexicon_tags_2 },
    { 0x44e3f39e, SUC_NE_lexicon_tags_2 },
    { 0xeb43f39f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81ff3a3, SUC_NE_lexicon_tags_2 },
    { 0xaf2ff3a4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7065f3a6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40bf3ab, SUC_NE_lexicon_tags_2 },
    { 0x40f7f3ac, SUC_NE_lexicon_tags_32 },
    { 0x73ebf3ad, SUC_NE_lexicon_tags_2 },
    { 0x75e3f3ae, SUC_NE_lexicon_tags_2 },
    { 0x7501f3af, SUC_NE_lexicon_tags_2 },
    { 0x5cff3ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x32d3f3b2, SUC_NE_lexicon_tags_2 },
    { 0x917f3b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed1f3b6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd529f3ba, SUC_NE_lexicon_tags_2 },
    { 0x4347f3bb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282df3c0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6069f3c3, SUC_NE_lexicon_tags_2 },
    { 0xc17df3c4, SUC_NE_lexicon_tags_2 },
    { 0x6f09f3c5, SUC_NE_lexicon_tags_2 },
    { 0x4b91f3c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b17f3cd, SUC_NE_lexicon_tags_2 },
    { 0x2fe3f3ce, SUC_NE_lexicon_tags_2 },
    { 0x6095f3ce, SUC_NE_lexicon_tags_2 },
    { 0xba93f3cd, SUC_NE_lexicon_tags_2 },
    { 0x6169f3d1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f3d4, SUC_NE_lexicon_tags_2 },
    { 0x18ebf3d5, SUC_NE_lexicon_tags_0 },
    { 0x61cbf3d6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x8851f3d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6639f3df, SUC_NE_lexicon_tags_2 },
    { 0xb9aff3e0, SUC_NE_lexicon_tags_2 },
    { 0x9823f3e1, SUC_NE_lexicon_tags_2 },
    { 0x3673f3e2, SUC_NE_lexicon_tags_2 },
    { 0x7337f3e3, SUC_NE_lexicon_tags_4 },
    { 0x4f91f3e4, SUC_NE_lexicon_tags_19 },
    { 0xb3ff3e5, SUC_NE_lexicon_tags_2 },
    { 0xf31f3e0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a9f3ec, SUC_NE_lexicon_tags_2 },
    { 0xd05bf3ed, SUC_NE_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4671f3f1, SUC_NE_lexicon_tags_2 },
    { 0x46f3f3f2, SUC_NE_lexicon_tags_2 },
    { 0x7171f3f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8341f3f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6e5f3f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8055f3fd, SUC_NE_lexicon_tags_2 },
    { 0x9e3df3fe, SUC_NE_lexicon_tags_2 },
    { 0x3c75f3ff, SUC_NE_lexicon_tags_2 },
    { 0xf235f400, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71fdf402, SUC_NE_lexicon_tags_2 },
    { 0xf729f402, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0x7101f405, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6bf408, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb1f40b, SUC_NE_lexicon_tags_3 },
    { 0xdbf7f40c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873f411, SUC_NE_lexicon_tags_2 },
    { 0xff3df412, SUC_NE_lexicon_tags_2 },
    { 0x2a81f413, SUC_NE_lexicon_tags_2 },
    { 0x2ff5f414, SUC_NE_lexicon_tags_25 },
    { 0x21ff415, SUC_NE_lexicon_tags_75 },
    { 0xaa6df415, SUC_NE_lexicon_tags_2 },
    { 0xfee7f417, SUC_NE_lexicon_tags_2 },
    { 0x6dbdf415, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee5f41a, SUC_NE_lexicon_tags_2 },
    { 0xaa41f41b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x249df41e, SUC_NE_lexicon_tags_2 },
    { 0x704ff41e, SUC_NE_lexicon_tags_2 },
    { 0xb2d9f41f, SUC_NE_lexicon_tags_1 },
    { 0x90b5f421, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1103f424, SUC_NE_lexicon_tags_2 },
    { 0x4b1bf425, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x647ff428, SUC_NE_lexicon_tags_2 },
    { 0xd50ff429, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97f42d, SUC_NE_lexicon_tags_2 },
    { 0x6f53f42d, SUC_NE_lexicon_tags_2 },
    { 0xa03ff42f, SUC_NE_lexicon_tags_2 },
    { 0xace7f42f, SUC_NE_lexicon_tags_19 },
    { 0xc109f431, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149df435, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebf9f437, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d41f43b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5fbf43d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b5f441, SUC_NE_lexicon_tags_2 },
    { 0xd2eff442, SUC_NE_lexicon_tags_4 },
    { 0x3b97f442, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdbf447, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e6bf449, SUC_NE_lexicon_tags_2 },
    { 0x6fe3f449, SUC_NE_lexicon_tags_2 },
    { 0x961ff44a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb663f44e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed1df450, SUC_NE_lexicon_tags_87 },
    { 0xff39f451, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d55f456, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549f45a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61eff45d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0f1f45f, SUC_NE_lexicon_tags_18 },
    { 0x6c55f460, SUC_NE_lexicon_tags_7 },
    { 0xe2a3f460, SUC_NE_lexicon_tags_2 },
    { 0x1947f461, SUC_NE_lexicon_tags_2 },
    { 0x1003f462, SUC_NE_lexicon_tags_2 },
    { 0x74ff463, SUC_NE_lexicon_tags_2 },
    { 0x7935f462, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edf468, SUC_NE_lexicon_tags_2 },
    { 0x643df469, SUC_NE_lexicon_tags_2 },
    { 0x2657f468, SUC_NE_lexicon_tags_2 },
    { 0xd147f469, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61ff474, SUC_NE_lexicon_tags_2 },
    { 0x7039f475, SUC_NE_lexicon_tags_13 },
    { 0x27cff475, SUC_NE_lexicon_tags_18 },
    { 0x38cff474, SUC_NE_lexicon_tags_2 },
    { 0x1649f478, SUC_NE_lexicon_tags_2 },
    { 0xd4d9f479, SUC_NE_lexicon_tags_2 },
    { 0xb063f479, SUC_NE_lexicon_tags_2 },
    { 0x560bf47b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fdf47f, SUC_NE_lexicon_tags_2 },
    { 0xfaedf480, SUC_NE_lexicon_tags_2 },
    { 0x2cddf481, SUC_NE_lexicon_tags_2 },
    { 0x43fff482, SUC_NE_lexicon_tags_2 },
    { 0x571ff483, SUC_NE_lexicon_tags_2 },
    { 0xd3e9f483, SUC_NE_lexicon_tags_2 },
    { 0xb47bf485, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f65f492, SUC_NE_lexicon_tags_2 },
    { 0x9f37f493, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd13df495, SUC_NE_lexicon_tags_2 },
    { 0x8571f496, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb071f498, SUC_NE_lexicon_tags_2 },
    { 0x769ff498, SUC_NE_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9e3f49b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b09f49e, SUC_NE_lexicon_tags_3 },
    { 0xc1f7f49f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b77f4a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd031f4a7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xeb8bf4a9, SUC_NE_lexicon_tags_2 },
    { 0x8f6ff4aa, SUC_NE_lexicon_tags_2 },
    { 0x269bf4ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b5f4ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffdf4b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36df4b3, SUC_NE_lexicon_tags_2 },
    { 0xfe1f4b3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb351f4bf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a35f4c1, SUC_NE_lexicon_tags_2 },
    { 0x7dcdf4c1, SUC_NE_lexicon_tags_2 },
    { 0x303ff4c3, SUC_NE_lexicon_tags_2 },
    { 0xcaff4c3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b0df4c6, SUC_NE_lexicon_tags_2 },
    { 0xab1ff4c7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa19f4cb, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2df4cd, SUC_NE_lexicon_tags_2 },
    { 0x2051f4ce, SUC_NE_lexicon_tags_2 },
    { 0x1749f4cf, SUC_NE_lexicon_tags_2 },
    { 0x5743f4cf, SUC_NE_lexicon_tags_2 },
    { 0x639df4d1, SUC_NE_lexicon_tags_2 },
    { 0x547ff4d2, SUC_NE_lexicon_tags_2 },
    { 0x235f4d3, SUC_NE_lexicon_tags_2 },
    { 0x5ac5f4d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x66fdf4d6, SUC_NE_lexicon_tags_2 },
    { 0x7d55f4d7, SUC_NE_lexicon_tags_2 },
    { 0xe33bf4d8, SUC_NE_lexicon_tags_2 },
    { 0x2ccff4d9, SUC_NE_lexicon_tags_2 },
    { 0x5905f4d9, SUC_NE_lexicon_tags_30 },
    { 0x2ac9f4d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9db3f4dd, SUC_NE_lexicon_tags_2 },
    { 0x99b9f4de, SUC_NE_lexicon_tags_4 },
    { 0x458df4df, SUC_NE_lexicon_tags_2 },
    { 0xf6d5f4dd, SUC_NE_lexicon_tags_2 },
    { 0x72d1f4e1, SUC_NE_lexicon_tags_2 },
    { 0xadf4e1, SUC_NE_lexicon_tags_2 },
    { 0xb969f4dd, SUC_NE_lexicon_tags_4 },
    { 0xf985f4e4, SUC_NE_lexicon_tags_2 },
    { 0x7c85f4e4, SUC_NE_lexicon_tags_2 },
    { 0x4d83f4e6, SUC_NE_lexicon_tags_2 },
    { 0x371ff4e7, SUC_NE_lexicon_tags_2 },
    { 0xc9b9f4e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x625f4ea, SUC_NE_lexicon_tags_2 },
    { 0xc173f4eb, SUC_NE_lexicon_tags_2 },
    { 0x43ff4eb, SUC_NE_lexicon_tags_2 },
    { 0x5eb9f4ec, SUC_NE_lexicon_tags_2 },
    { 0x9319f4ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faff4f3, SUC_NE_lexicon_tags_4 },
    { 0x3409f4f3, SUC_NE_lexicon_tags_2 },
    { 0xdf8df4f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xddff4f7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dbf4fb, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7bf502, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x40fff504, SUC_NE_lexicon_tags_2 },
    { 0x395df504, SUC_NE_lexicon_tags_2 },
    { 0x5c17f505, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55bf509, SUC_NE_lexicon_tags_2 },
    { 0x3d3df50a, SUC_NE_lexicon_tags_2 },
    { 0xd4bff50a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb1f513, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a5f515, SUC_NE_lexicon_tags_2 },
    { 0x791df516, SUC_NE_lexicon_tags_2 },
    { 0xe8a7f515, SUC_NE_lexicon_tags_2 },
    { 0x49ddf516, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc563f523, SUC_NE_lexicon_tags_2 },
    { 0xe691f523, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0bf527, SUC_NE_lexicon_tags_1 },
    { 0x7ea3f527, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb885f52a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x265df52d, SUC_NE_lexicon_tags_2 },
    { 0xd11ff52e, SUC_NE_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfdf535, SUC_NE_lexicon_tags_2 },
    { 0x13a9f536, SUC_NE_lexicon_tags_2 },
    { 0x6a11f537, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e65f539, SUC_NE_lexicon_tags_2 },
    { 0xbd5ff53a, SUC_NE_lexicon_tags_5 },
    { 0x959bf539, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b8ff53d, SUC_NE_lexicon_tags_2 },
    { 0xb0aff53d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7943f540, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f57f542, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f59f546, SUC_NE_lexicon_tags_2 },
    { 0x31c9f547, SUC_NE_lexicon_tags_2 },
    { 0xb717f546, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce15f54e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf77f561, SUC_NE_lexicon_tags_2 },
    { 0xb25ff561, SUC_NE_lexicon_tags_2 },
    { 0x4d0ff563, SUC_NE_lexicon_tags_2 },
    { 0xc323f563, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa109f569, SUC_NE_lexicon_tags_2 },
    { 0xe103f569, SUC_NE_lexicon_tags_2 },
    { 0xbe0ff56b, SUC_NE_lexicon_tags_2 },
    { 0x3b81f56c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xca51f56e, SUC_NE_lexicon_tags_2 },
    { 0xa42df56e, SUC_NE_lexicon_tags_2 },
    { 0x6f57f56f, SUC_NE_lexicon_tags_2 },
    { 0x31eff570, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dbf578, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f89f57a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8789f57d, SUC_NE_lexicon_tags_2 },
    { 0x3df5f57e, SUC_NE_lexicon_tags_2 },
    { 0x2f9f57f, SUC_NE_lexicon_tags_2 },
    { 0xc42df57f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb7f584, SUC_NE_lexicon_tags_2 },
    { 0x953ff585, SUC_NE_lexicon_tags_14 },
    { 0xcc01f586, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5391f58a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x201bf58c, SUC_NE_lexicon_tags_2 },
    { 0xf6b3f58c, SUC_NE_lexicon_tags_2 },
    { 0x909bf58e, SUC_NE_lexicon_tags_2 },
    { 0x943df58f, SUC_NE_lexicon_tags_2 },
    { 0x7ba9f58c, SUC_NE_lexicon_tags_2 },
    { 0x5325f58f, SUC_NE_lexicon_tags_102 },
    { 0xa309f592, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f9f598, SUC_NE_lexicon_tags_2 },
    { 0x6153f598, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3f59d, SUC_NE_lexicon_tags_2 },
    { 0xfd51f59e, SUC_NE_lexicon_tags_3 },
    { 0x4651f59f, SUC_NE_lexicon_tags_4 },
    { 0x42fdf5a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x26b5f5a2, SUC_NE_lexicon_tags_2 },
    { 0xdc3f5a2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4877f5a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bdf5aa, SUC_NE_lexicon_tags_2 },
    { 0xba9bf5ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x259df5ad, SUC_NE_lexicon_tags_2 },
    { 0x58ebf5ae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82b3f5b0, SUC_NE_lexicon_tags_2 },
    { 0x7901f5b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f7f5b5, SUC_NE_lexicon_tags_2 },
    { 0x6103f5b5, SUC_NE_lexicon_tags_2 },
    { 0xd117f5b6, SUC_NE_lexicon_tags_2 },
    { 0x72cbf5b8, SUC_NE_lexicon_tags_2 },
    { 0x44ff5b9, SUC_NE_lexicon_tags_2 },
    { 0xdfa1f5b8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbd9f5bc, SUC_NE_lexicon_tags_2 },
    { 0x5f7f5bc, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e89f5bf, SUC_NE_lexicon_tags_17 },
    { 0xc359f5bf, SUC_NE_lexicon_tags_2 },
    { 0xe121f5c1, SUC_NE_lexicon_tags_2 },
    { 0x4a71f5c2, SUC_NE_lexicon_tags_2 },
    { 0x5df5f5c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x906ff5c6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x9063f5c8, SUC_NE_lexicon_tags_2 },
    { 0x1dc9f5c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe4ff5cb, SUC_NE_lexicon_tags_1 },
    { 0x68adf5cb, SUC_NE_lexicon_tags_8 },
    { 0x1af7f5cc, SUC_NE_lexicon_tags_2 },
    { 0xd9aff5ce, SUC_NE_lexicon_tags_2 },
    { 0x6117f5cf, SUC_NE_lexicon_tags_2 },
    { 0xc64ff5cf, SUC_NE_lexicon_tags_2 },
    { 0xef19f5d1, SUC_NE_lexicon_tags_2 },
    { 0xc975f5d1, SUC_NE_lexicon_tags_2 },
    { 0xf8bff5cb, SUC_NE_lexicon_tags_2 },
    { 0xcf5df5d4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5889f5d6, SUC_NE_lexicon_tags_2 },
    { 0x2c61f5d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cadf5d9, SUC_NE_lexicon_tags_2 },
    { 0x4927f5d9, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9569f5e0, SUC_NE_lexicon_tags_2 },
    { 0x1e73f5e1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x71d1f5e3, SUC_NE_lexicon_tags_3 },
    { 0x7885f5e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ba3f5e6, SUC_NE_lexicon_tags_2 },
    { 0x712bf5e6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf983f5eb, SUC_NE_lexicon_tags_2 },
    { 0xd345f5eb, SUC_NE_lexicon_tags_2 },
    { 0x1a9df5ed, SUC_NE_lexicon_tags_2 },
    { 0x379f5ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e87f5f0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fe7f5f2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8571f5f8, SUC_NE_lexicon_tags_2 },
    { 0x38d3f5f9, SUC_NE_lexicon_tags_2 },
    { 0x9afdf5f8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58abf5fd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1283f600, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x13edf602, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6ff607, SUC_NE_lexicon_tags_2 },
    { 0x3a35f608, SUC_NE_lexicon_tags_2 },
    { 0x4657f609, SUC_NE_lexicon_tags_2 },
    { 0x6c8df60a, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0xd349f60c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xef25f60e, SUC_NE_lexicon_tags_2 },
    { 0x4b79f60e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe69bf611, SUC_NE_lexicon_tags_2 },
    { 0xd5fff611, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x14e5f614, SUC_NE_lexicon_tags_2 },
    { 0x96f5f615, SUC_NE_lexicon_tags_2 },
    { 0x9aff614, SUC_NE_lexicon_tags_2 },
    { 0x786ff617, SUC_NE_lexicon_tags_1 },
    { 0x106df618, SUC_NE_lexicon_tags_40 },
    { 0x274ff619, SUC_NE_lexicon_tags_2 },
    { 0x18a7f61a, SUC_NE_lexicon_tags_7 },
    { 0xa05df61b, SUC_NE_lexicon_tags_2 },
    { 0xad1f61c, SUC_NE_lexicon_tags_2 },
    { 0x5f7f61d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437bf621, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe855f623, SUC_NE_lexicon_tags_2 },
    { 0x7ba3f624, SUC_NE_lexicon_tags_2 },
    { 0x9537f625, SUC_NE_lexicon_tags_2 },
    { 0xb0df626, SUC_NE_lexicon_tags_2 },
    { 0x4b25f626, SUC_NE_lexicon_tags_2 },
    { 0xc853f627, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fdf62f, SUC_NE_lexicon_tags_2 },
    { 0x798df630, SUC_NE_lexicon_tags_2 },
    { 0x5ce1f62f, SUC_NE_lexicon_tags_2 },
    { 0x8f73f632, SUC_NE_lexicon_tags_7 },
    { 0x34bf632, SUC_NE_lexicon_tags_2 },
    { 0xce51f634, SUC_NE_lexicon_tags_2 },
    { 0xbd99f635, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2291f638, SUC_NE_lexicon_tags_2 },
    { 0x102ff638, SUC_NE_lexicon_tags_159 },
    { 0x1601f639, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x9455f63d, SUC_NE_lexicon_tags_2 },
    { 0xe583f63d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x806bf640, SUC_NE_lexicon_tags_17 },
    { 0xaa3ff641, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d33f646, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbfdf648, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec7f650, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5f654, SUC_NE_lexicon_tags_2 },
    { 0xba7df655, SUC_NE_lexicon_tags_2 },
    { 0x478bf656, SUC_NE_lexicon_tags_34 },
    { 0xa2b5f655, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8859f65d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73bf668, SUC_NE_lexicon_tags_2 },
    { 0xc59bf668, SUC_NE_lexicon_tags_25 },
    { 0x97abf66a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xca3ff66c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd535f66e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dbbf670, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aff674, SUC_NE_lexicon_tags_2 },
    { 0xa905f675, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5aff677, SUC_NE_lexicon_tags_2 },
    { 0xcdc7f678, SUC_NE_lexicon_tags_2 },
    { 0x5db3f679, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5df67c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbf680, SUC_NE_lexicon_tags_2 },
    { 0x365df681, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x86f1f683, SUC_NE_lexicon_tags_2 },
    { 0x1b27f684, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefff68b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b5f691, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b9f694, SUC_NE_lexicon_tags_2 },
    { 0xa735f695, SUC_NE_lexicon_tags_2 },
    { 0x2757f694, SUC_NE_lexicon_tags_2 },
    { 0xd0c5f697, SUC_NE_lexicon_tags_1 },
    { 0xc3b3f695, SUC_NE_lexicon_tags_14 },
    { 0xc731f697, SUC_NE_lexicon_tags_2 },
    { 0x81a9f69a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9797f6a1, SUC_NE_lexicon_tags_3 },
    { 0x4bd3f6a1, SUC_NE_lexicon_tags_2 },
    { 0x6fedf6a3, SUC_NE_lexicon_tags_2 },
    { 0xc465f6a4, SUC_NE_lexicon_tags_2 },
    { 0x1175f6a5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e7f6aa, SUC_NE_lexicon_tags_7 },
    { 0x422df6ab, SUC_NE_lexicon_tags_2 },
    { 0xd1f5f6ab, SUC_NE_lexicon_tags_2 },
    { 0xbe59f6aa, SUC_NE_lexicon_tags_2 },
    { 0x1693f6ae, SUC_NE_lexicon_tags_2 },
    { 0x4653f6af, SUC_NE_lexicon_tags_2 },
    { 0xe583f6af, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a35f6b2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5891f6b7, SUC_NE_lexicon_tags_2 },
    { 0xa2cbf6b7, SUC_NE_lexicon_tags_2 },
    { 0xddb3f6b9, SUC_NE_lexicon_tags_2 },
    { 0xd9e7f6ba, SUC_NE_lexicon_tags_2 },
    { 0xc0dbf6bb, SUC_NE_lexicon_tags_2 },
    { 0x402bf6bc, SUC_NE_lexicon_tags_2 },
    { 0xe7dbf6bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b21f6c0, SUC_NE_lexicon_tags_2 },
    { 0x2791f6c1, SUC_NE_lexicon_tags_2 },
    { 0xd70bf6c0, SUC_NE_lexicon_tags_2 },
    { 0x3c7ff6c3, SUC_NE_lexicon_tags_3 },
    { 0xd699f6c2, SUC_NE_lexicon_tags_4 },
    { 0xe41ff6c5, SUC_NE_lexicon_tags_2 },
    { 0x85b9f6c2, SUC_NE_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xee61f6c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6085f6d0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5ddf6d2, SUC_NE_lexicon_tags_2 },
    { 0xe041f6d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2069f6d5, SUC_NE_lexicon_tags_2 },
    { 0x61d5f6d6, SUC_NE_lexicon_tags_2 },
    { 0xa1b7f6d5, SUC_NE_lexicon_tags_2 },
    { 0x739df6d6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ba7f6da, SUC_NE_lexicon_tags_2 },
    { 0x303ff6da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc13f6dd, SUC_NE_lexicon_tags_2 },
    { 0x16e9f6de, SUC_NE_lexicon_tags_3 },
    { 0xe129f6df, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4da1f6e1, SUC_NE_lexicon_tags_2 },
    { 0x970ff6e1, SUC_NE_lexicon_tags_2 },
    { 0x25f6e2, SUC_NE_lexicon_tags_2 },
    { 0x1f29f6e1, SUC_NE_lexicon_tags_2 },
    { 0xfb93f6e5, SUC_NE_lexicon_tags_30 },
    { 0x9471f6e3, SUC_NE_lexicon_tags_2 },
    { 0x1b11f6e7, SUC_NE_lexicon_tags_2 },
    { 0xa51f6e4, SUC_NE_lexicon_tags_2 },
    { 0xc9adf6e2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c9ff6eb, SUC_NE_lexicon_tags_2 },
    { 0x32a3f6ec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f85f6f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8bf6f4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e9f6fb, SUC_NE_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a65f6ff, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf747f708, SUC_NE_lexicon_tags_2 },
    { 0xbaeff709, SUC_NE_lexicon_tags_2 },
    { 0x9ae1f70a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7ff70d, SUC_NE_lexicon_tags_2 },
    { 0x62d5f70d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9359f714, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe221f717, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f87f719, SUC_NE_lexicon_tags_1 },
    { 0x9ed3f71a, SUC_NE_lexicon_tags_2 },
    { 0xd089f719, SUC_NE_lexicon_tags_2 },
    { 0x9129f71c, SUC_NE_lexicon_tags_2 },
    { 0x8063f71d, SUC_NE_lexicon_tags_2 },
    { 0xbc61f71e, SUC_NE_lexicon_tags_2 },
    { 0xf2b5f71d, SUC_NE_lexicon_tags_7 },
    { 0xebebf71c, SUC_NE_lexicon_tags_2 },
    { 0x143df721, SUC_NE_lexicon_tags_4 },
    { 0x6701f720, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bcbf724, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x361f727, SUC_NE_lexicon_tags_2 },
    { 0x7e77f727, SUC_NE_lexicon_tags_2 },
    { 0x6df9f727, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e65f72d, SUC_NE_lexicon_tags_14 },
    { 0xa84df72e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x498ff730, SUC_NE_lexicon_tags_2 },
    { 0x4a11f730, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79f735, SUC_NE_lexicon_tags_2 },
    { 0x145f736, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c63f739, SUC_NE_lexicon_tags_2 },
    { 0x5cfff739, SUC_NE_lexicon_tags_3 },
    { 0xb057f73b, SUC_NE_lexicon_tags_2 },
    { 0x7555f73c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05df73f, SUC_NE_lexicon_tags_2 },
    { 0xc3df73f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b1f742, SUC_NE_lexicon_tags_2 },
    { 0x7925f743, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4537f746, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x83ddf748, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e87f74d, SUC_NE_lexicon_tags_2 },
    { 0xfcadf74e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6275f750, SUC_NE_lexicon_tags_2 },
    { 0xcd7ff750, SUC_NE_lexicon_tags_2 },
    { 0x455ff752, SUC_NE_lexicon_tags_2 },
    { 0xb26bf753, SUC_NE_lexicon_tags_2 },
    { 0xeecbf754, SUC_NE_lexicon_tags_2 },
    { 0x1df5f752, SUC_NE_lexicon_tags_2 },
    { 0x1517f754, SUC_NE_lexicon_tags_2 },
    { 0x5453f754, SUC_NE_lexicon_tags_2 },
    { 0x17c3f758, SUC_NE_lexicon_tags_2 },
    { 0xafb3f755, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51aff75c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3df764, SUC_NE_lexicon_tags_2 },
    { 0xae3ff764, SUC_NE_lexicon_tags_2 },
    { 0xb533f766, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07bf76b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb9f770, SUC_NE_lexicon_tags_2 },
    { 0x546ff771, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2ff774, SUC_NE_lexicon_tags_2 },
    { 0x5c5bf775, SUC_NE_lexicon_tags_7 },
    { 0x5a33f776, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f3f779, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x51fbf77b, SUC_NE_lexicon_tags_2 },
    { 0xc00ff77c, SUC_NE_lexicon_tags_3 },
    { 0xb12df77c, SUC_NE_lexicon_tags_2 },
    { 0x6b4bf77e, SUC_NE_lexicon_tags_2 },
    { 0x26e7f77f, SUC_NE_lexicon_tags_2 },
    { 0x2e17f77e, SUC_NE_lexicon_tags_2 },
    { 0xa065f781, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9469f784, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1f787, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da7f78c, SUC_NE_lexicon_tags_2 },
    { 0xe65bf78c, SUC_NE_lexicon_tags_2 },
    { 0xa655f78c, SUC_NE_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731bf792, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa115f795, SUC_NE_lexicon_tags_3 },
    { 0x6dfbf796, SUC_NE_lexicon_tags_2 },
    { 0xc311f796, SUC_NE_lexicon_tags_3 },
    { 0x6453f798, SUC_NE_lexicon_tags_2 },
    { 0xf255f799, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8b1f79b, SUC_NE_lexicon_tags_2 },
    { 0xb55bf79c, SUC_NE_lexicon_tags_2 },
    { 0xf721f79d, SUC_NE_lexicon_tags_2 },
    { 0x5cd7f79e, SUC_NE_lexicon_tags_2 },
    { 0xf6d5f79f, SUC_NE_lexicon_tags_2 },
    { 0x82e7f79f, SUC_NE_lexicon_tags_2 },
    { 0xf4df79e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x55a5f7a3, SUC_NE_lexicon_tags_115 },
    { 0xc125f7a4, SUC_NE_lexicon_tags_2 },
    { 0xf795f7a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x551df7a8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x322df7aa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4689f7ac, SUC_NE_lexicon_tags_2 },
    { 0x4753f7ad, SUC_NE_lexicon_tags_2 },
    { 0xad1df7ac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c37f7b4, SUC_NE_lexicon_tags_2 },
    { 0x6337f7b5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1617f7b7, SUC_NE_lexicon_tags_2 },
    { 0x4473f7b7, SUC_NE_lexicon_tags_2 },
    { 0x132ff7b9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7df7bd, SUC_NE_lexicon_tags_2 },
    { 0x671df7be, SUC_NE_lexicon_tags_2 },
    { 0x3aebf7bf, SUC_NE_lexicon_tags_2 },
    { 0x5e19f7bf, SUC_NE_lexicon_tags_2 },
    { 0xecb3f7c1, SUC_NE_lexicon_tags_2 },
    { 0x9447f7c1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bbf7ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410bf7d3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f63f7d6, SUC_NE_lexicon_tags_2 },
    { 0x292df7d6, SUC_NE_lexicon_tags_2 },
    { 0xe515f7d8, SUC_NE_lexicon_tags_2 },
    { 0xe61ff7d9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7759f7db, SUC_NE_lexicon_tags_2 },
    { 0x5e87f7dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90ff7e3, SUC_NE_lexicon_tags_7 },
    { 0xf3d9f7e4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277bf7e9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x41fff7eb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e81f7ee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xed6ff7f0, SUC_NE_lexicon_tags_2 },
    { 0x93cdf7f1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93bf7f8, SUC_NE_lexicon_tags_2 },
    { 0xc33f7f9, SUC_NE_lexicon_tags_2 },
    { 0xd83bf7fa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fbf7fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf925f800, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c19f802, SUC_NE_lexicon_tags_2 },
    { 0xd27bf803, SUC_NE_lexicon_tags_2 },
    { 0xd2bff803, SUC_NE_lexicon_tags_18 },
    { 0xe8e9f805, SUC_NE_lexicon_tags_2 },
    { 0x1987f802, SUC_NE_lexicon_tags_2 },
    { 0xeb4bf807, SUC_NE_lexicon_tags_2 },
    { 0xbd65f807, SUC_NE_lexicon_tags_2 },
    { 0x654ff808, SUC_NE_lexicon_tags_2 },
    { 0xf969f80a, SUC_NE_lexicon_tags_2 },
    { 0xdae7f807, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fff812, SUC_NE_lexicon_tags_2 },
    { 0x6eeff813, SUC_NE_lexicon_tags_2 },
    { 0x9d2df814, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc553f816, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2197f819, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5923f81d, SUC_NE_lexicon_tags_2 },
    { 0xe64df81e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x254ff820, SUC_NE_lexicon_tags_2 },
    { 0x5aa7f821, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x36a7f823, SUC_NE_lexicon_tags_2 },
    { 0x7f9f823, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9b9f826, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xab1df828, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe10df82a, SUC_NE_lexicon_tags_2 },
    { 0xb567f82a, SUC_NE_lexicon_tags_8 },
    { 0x9d13f82c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x95a5f82e, SUC_NE_lexicon_tags_2 },
    { 0x24b3f82f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4563f831, SUC_NE_lexicon_tags_2 },
    { 0x811df831, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75f836, SUC_NE_lexicon_tags_2 },
    { 0x1ff836, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393bf83f, SUC_NE_lexicon_tags_2 },
    { 0xd0ddf83f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9303f845, SUC_NE_lexicon_tags_7 },
    { 0xe3adf846, SUC_NE_lexicon_tags_2 },
    { 0x719f847, SUC_NE_lexicon_tags_5 },
    { 0x38c1f848, SUC_NE_lexicon_tags_2 },
    { 0x7fbdf849, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf587f84c, SUC_NE_lexicon_tags_2 },
    { 0x3dc5f84c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2bf850, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbff853, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae19f855, SUC_NE_lexicon_tags_46 },
    { 0x9e9f855, SUC_NE_lexicon_tags_2 },
    { 0x7d31f857, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xcefbf859, SUC_NE_lexicon_tags_2 },
    { 0xd93df85a, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb9f85d, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5181f865, SUC_NE_lexicon_tags_2 },
    { 0x2c2bf866, SUC_NE_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7f86e, SUC_NE_lexicon_tags_2 },
    { 0x9983f86f, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175ff873, SUC_NE_lexicon_tags_2 },
    { 0x983bf874, SUC_NE_lexicon_tags_2 },
    { 0x99bdf873, SUC_NE_lexicon_tags_2 },
    { 0xc415f874, SUC_NE_lexicon_tags_2 },
    { 0x4709f874, SUC_NE_lexicon_tags_2 },
    { 0x2fadf878, SUC_NE_lexicon_tags_2 },
    { 0xd663f879, SUC_NE_lexicon_tags_2 },
    { 0x33c7f874, SUC_NE_lexicon_tags_2 },
    { 0x9a8df873, SUC_NE_lexicon_tags_2 },
    { 0x5951f87c, SUC_NE_lexicon_tags_2 },
    { 0x1f51f87b, SUC_NE_lexicon_tags_2 },
    { 0xe00ff87e, SUC_NE_lexicon_tags_2 },
    { 0xbaf1f87e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf51f883, SUC_NE_lexicon_tags_2 },
    { 0x9fa7f883, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa59f886, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa74ff888, SUC_NE_lexicon_tags_2 },
    { 0x4869f889, SUC_NE_lexicon_tags_2 },
    { 0xad93f88a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x899ff88c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8191f88e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e1f891, SUC_NE_lexicon_tags_24 },
    { 0xe289f891, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a1f895, SUC_NE_lexicon_tags_2 },
    { 0xfb57f895, SUC_NE_lexicon_tags_4 },
    { 0xf73ff896, SUC_NE_lexicon_tags_2 },
    { 0x295df895, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9ff89b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0x35a1f89d, SUC_NE_lexicon_tags_2 },
    { 0xc947f89e, SUC_NE_lexicon_tags_2 },
    { 0x66fdf89f, SUC_NE_lexicon_tags_2 },
    { 0x5771f8a0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7f7f8a2, SUC_NE_lexicon_tags_2 },
    { 0x3d9f8a3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb1f8a6, SUC_NE_lexicon_tags_2 },
    { 0xf691f8a7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890ff8ad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3323f8af, SUC_NE_lexicon_tags_51 },
    { 0x6f37f8b0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66df8b7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cff8bc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ff8c3, SUC_NE_lexicon_tags_2 },
    { 0xc559f8c4, SUC_NE_lexicon_tags_4 },
    { 0x9c0df8c5, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7f8c8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a33f8ca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x99b5f8cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3805f8ce, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x689f8d1, SUC_NE_lexicon_tags_2 },
    { 0xb055f8d1, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xe427f8d4, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26bf8d9, SUC_NE_lexicon_tags_30 },
    { 0xcad1f8da, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x88f3f8dc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1f8e0, SUC_NE_lexicon_tags_2 },
    { 0xda59f8e0, SUC_NE_lexicon_tags_2 },
    { 0x3d8ff8e2, SUC_NE_lexicon_tags_7 },
    { 0x310df8e3, SUC_NE_lexicon_tags_2 },
    { 0xe23f8e4, SUC_NE_lexicon_tags_2 },
    { 0x8d9bf8e2, SUC_NE_lexicon_tags_2 },
    { 0xc32ff8e5, SUC_NE_lexicon_tags_2 },
    { 0x228bf8e7, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae9f8ee, SUC_NE_lexicon_tags_2 },
    { 0xc475f8ef, SUC_NE_lexicon_tags_2 },
    { 0x7661f8ef, SUC_NE_lexicon_tags_2 },
    { 0xa301f8f1, SUC_NE_lexicon_tags_2 },
    { 0x77a5f8ef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349ff8f7, SUC_NE_lexicon_tags_2 },
    { 0xb527f8f8, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39ff8ff, SUC_NE_lexicon_tags_1 },
    { 0xbdebf900, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99f903, SUC_NE_lexicon_tags_3 },
    { 0xfcb3f904, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x868df906, SUC_NE_lexicon_tags_2 },
    { 0x9b6df906, SUC_NE_lexicon_tags_2 },
    { 0xd281f907, SUC_NE_lexicon_tags_2 },
    { 0xf685f909, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe953f90c, SUC_NE_lexicon_tags_2 },
    { 0xc879f90d, SUC_NE_lexicon_tags_2 },
    { 0xe53bf90e, SUC_NE_lexicon_tags_2 },
    { 0x556bf90e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf4df911, SUC_NE_lexicon_tags_2 },
    { 0x90abf912, SUC_NE_lexicon_tags_2 },
    { 0x9f99f913, SUC_NE_lexicon_tags_2 },
    { 0xca8df912, SUC_NE_lexicon_tags_2 },
    { 0xa64ff915, SUC_NE_lexicon_tags_2 },
    { 0xba61f915, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d3f91a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e5f91e, SUC_NE_lexicon_tags_2 },
    { 0x336df91f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4467f921, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51f925, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefff928, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb59df92a, SUC_NE_lexicon_tags_2 },
    { 0x52f7f92b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e7f932, SUC_NE_lexicon_tags_2 },
    { 0xa00df933, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0dbf935, SUC_NE_lexicon_tags_17 },
    { 0x228ff935, SUC_NE_lexicon_tags_2 },
    { 0xb943f937, SUC_NE_lexicon_tags_2 },
    { 0x55a9f938, SUC_NE_lexicon_tags_2 },
    { 0x3f37f939, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3393f93e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dc9f940, SUC_NE_lexicon_tags_2 },
    { 0x6b93f941, SUC_NE_lexicon_tags_2 },
    { 0x66cbf942, SUC_NE_lexicon_tags_4 },
    { 0xe5e5f942, SUC_NE_lexicon_tags_2 },
    { 0xd695f943, SUC_NE_lexicon_tags_2 },
    { 0x444ff945, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7cdf947, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4891f94a, SUC_NE_lexicon_tags_2 },
    { 0xefddf94a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659f94f, SUC_NE_lexicon_tags_2 },
    { 0x7fe3f950, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1f953, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3d9f955, SUC_NE_lexicon_tags_2 },
    { 0x53fbf955, SUC_NE_lexicon_tags_2 },
    { 0x56abf957, SUC_NE_lexicon_tags_2 },
    { 0x550ff957, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbdf95b, SUC_NE_lexicon_tags_2 },
    { 0x3301f95b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd3f95e, SUC_NE_lexicon_tags_2 },
    { 0xed5bf95f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9babf965, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1df969, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b07f971, SUC_NE_lexicon_tags_2 },
    { 0xdddbf972, SUC_NE_lexicon_tags_2 },
    { 0xabb7f973, SUC_NE_lexicon_tags_2 },
    { 0xff43f974, SUC_NE_lexicon_tags_2 },
    { 0x6085f975, SUC_NE_lexicon_tags_2 },
    { 0xdb67f976, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9147f979, SUC_NE_lexicon_tags_2 },
    { 0xdb03f979, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447bf97f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e3f983, SUC_NE_lexicon_tags_2 },
    { 0xcacdf984, SUC_NE_lexicon_tags_3 },
    { 0x9d09f983, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdbf988, SUC_NE_lexicon_tags_2 },
    { 0xb125f989, SUC_NE_lexicon_tags_2 },
    { 0xad17f989, SUC_NE_lexicon_tags_0 },
    { 0x8b23f98b, SUC_NE_lexicon_tags_81 },
    { 0, NULL },
    { 0xd315f98d, SUC_NE_lexicon_tags_2 },
    { 0x79fdf98e, SUC_NE_lexicon_tags_2 },
    { 0xac2ff98f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7f996, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf7f999, SUC_NE_lexicon_tags_2 },
    { 0xdbc3f99a, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f99e, SUC_NE_lexicon_tags_2 },
    { 0x50a9f99e, SUC_NE_lexicon_tags_2 },
    { 0x1bb7f99f, SUC_NE_lexicon_tags_4 },
    { 0xddff9a1, SUC_NE_lexicon_tags_2 },
    { 0x23fbf9a1, SUC_NE_lexicon_tags_2 },
    { 0x3a7df9a3, SUC_NE_lexicon_tags_2 },
    { 0x47ebf99e, SUC_NE_lexicon_tags_4 },
    { 0x4afbf9a4, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ff9a8, SUC_NE_lexicon_tags_2 },
    { 0x8685f9a9, SUC_NE_lexicon_tags_3 },
    { 0xfac1f9a9, SUC_NE_lexicon_tags_2 },
    { 0xb5cff9ab, SUC_NE_lexicon_tags_2 },
    { 0xea5f9ab, SUC_NE_lexicon_tags_2 },
    { 0xf8cdf9ab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb54bf9af, SUC_NE_lexicon_tags_7 },
    { 0x70b3f9b0, SUC_NE_lexicon_tags_2 },
    { 0xe1e1f9b1, SUC_NE_lexicon_tags_2 },
    { 0xc0e1f9af, SUC_NE_lexicon_tags_4 },
    { 0x7a9df9b1, SUC_NE_lexicon_tags_2 },
    { 0xf2c3f9b4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4889f9bd, SUC_NE_lexicon_tags_2 },
    { 0x8c4bf9be, SUC_NE_lexicon_tags_2 },
    { 0x8ff9bf, SUC_NE_lexicon_tags_2 },
    { 0x87a3f9c0, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bf9c6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c67f9c8, SUC_NE_lexicon_tags_2 },
    { 0x16cff9c9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e01f9cc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63f9cf, SUC_NE_lexicon_tags_167 },
    { 0x8cdf9d0, SUC_NE_lexicon_tags_2 },
    { 0x4febf9d1, SUC_NE_lexicon_tags_2 },
    { 0x1b97f9d2, SUC_NE_lexicon_tags_2 },
    { 0x12ff9d2, SUC_NE_lexicon_tags_2 },
    { 0xdc29f9d4, SUC_NE_lexicon_tags_1 },
    { 0xd2c5f9d5, SUC_NE_lexicon_tags_41 },
    { 0x99b5f9d5, SUC_NE_lexicon_tags_7 },
    { 0x803bf9d7, SUC_NE_lexicon_tags_2 },
    { 0x7a65f9d8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39f9db, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf71f9de, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c39f9e0, SUC_NE_lexicon_tags_2 },
    { 0x8091f9e0, SUC_NE_lexicon_tags_2 },
    { 0x4c29f9e2, SUC_NE_lexicon_tags_2 },
    { 0x7c1f9e3, SUC_NE_lexicon_tags_2 },
    { 0x20c9f9e2, SUC_NE_lexicon_tags_2 },
    { 0x8a55f9e5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa451f9e8, SUC_NE_lexicon_tags_2 },
    { 0x8c5bf9e9, SUC_NE_lexicon_tags_1 },
    { 0x9057f9ea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e51f9ec, SUC_NE_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe9f9f1, SUC_NE_lexicon_tags_2 },
    { 0xcf67f9f2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7397f9f4, SUC_NE_lexicon_tags_2 },
    { 0xa15ff9f5, SUC_NE_lexicon_tags_2 },
    { 0xd685f9f5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9f9fb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c87f9fe, SUC_NE_lexicon_tags_2 },
    { 0xe24ff9fe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb031fa03, SUC_NE_lexicon_tags_2 },
    { 0x433dfa04, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x171ffa06, SUC_NE_lexicon_tags_2 },
    { 0x1b9fa06, SUC_NE_lexicon_tags_2 },
    { 0xf417fa08, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x7513fa0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8821fa0c, SUC_NE_lexicon_tags_2 },
    { 0xa02dfa0d, SUC_NE_lexicon_tags_2 },
    { 0x7acbfa0e, SUC_NE_lexicon_tags_2 },
    { 0xc9c9fa0e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e79fa15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f3fa18, SUC_NE_lexicon_tags_2 },
    { 0xbe19fa19, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7edfa1d, SUC_NE_lexicon_tags_4 },
    { 0x7305fa1d, SUC_NE_lexicon_tags_2 },
    { 0x69ddfa1f, SUC_NE_lexicon_tags_2 },
    { 0x41ddfa20, SUC_NE_lexicon_tags_13 },
    { 0xbd61fa21, SUC_NE_lexicon_tags_19 },
    { 0x81bffa1e, SUC_NE_lexicon_tags_384 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf9fa26, SUC_NE_lexicon_tags_2 },
    { 0xfd93fa27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e9fa2b, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998dfa2f, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xc3a5fa31, SUC_NE_lexicon_tags_2 },
    { 0x9193fa32, SUC_NE_lexicon_tags_2 },
    { 0xcd8bfa32, SUC_NE_lexicon_tags_2 },
    { 0xda55fa33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf797fa36, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7953fa3a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd7fa3d, SUC_NE_lexicon_tags_2 },
    { 0x6b1dfa3e, SUC_NE_lexicon_tags_2 },
    { 0xf655fa3f, SUC_NE_lexicon_tags_2 },
    { 0xb70bfa3f, SUC_NE_lexicon_tags_2 },
    { 0x3119fa3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc049fa44, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e51fa49, SUC_NE_lexicon_tags_2 },
    { 0x368dfa49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x748bfa4d, SUC_NE_lexicon_tags_2 },
    { 0x98a7fa4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb7fa51, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5fa54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27fa59, SUC_NE_lexicon_tags_2 },
    { 0xec2dfa5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5f9fa5c, SUC_NE_lexicon_tags_2 },
    { 0x7285fa5d, SUC_NE_lexicon_tags_2 },
    { 0x41ffa5e, SUC_NE_lexicon_tags_2 },
    { 0x2285fa5f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x889dfa61, SUC_NE_lexicon_tags_2 },
    { 0xc637fa62, SUC_NE_lexicon_tags_2 },
    { 0x927bfa63, SUC_NE_lexicon_tags_2 },
    { 0x91e5fa64, SUC_NE_lexicon_tags_2 },
    { 0x3e11fa63, SUC_NE_lexicon_tags_2 },
    { 0x7c6bfa66, SUC_NE_lexicon_tags_2 },
    { 0xa7f9fa66, SUC_NE_lexicon_tags_2 },
    { 0xe075fa68, SUC_NE_lexicon_tags_2 },
    { 0xf5fdfa68, SUC_NE_lexicon_tags_58 },
    { 0, NULL },
    { 0xca7bfa6b, SUC_NE_lexicon_tags_2 },
    { 0x456ffa6b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d1fa73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f7fa78, SUC_NE_lexicon_tags_3 },
    { 0x38bbfa79, SUC_NE_lexicon_tags_2 },
    { 0x2567fa79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6349fa7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e9dfa7e, SUC_NE_lexicon_tags_2 },
    { 0xb5b1fa7e, SUC_NE_lexicon_tags_2 },
    { 0xd75bfa80, SUC_NE_lexicon_tags_7 },
    { 0x6277fa81, SUC_NE_lexicon_tags_2 },
    { 0xd751fa82, SUC_NE_lexicon_tags_2 },
    { 0x72dffa82, SUC_NE_lexicon_tags_34 },
    { 0x4ce9fa84, SUC_NE_lexicon_tags_2 },
    { 0xbce7fa82, SUC_NE_lexicon_tags_2 },
    { 0x9ce1fa85, SUC_NE_lexicon_tags_7 },
    { 0x6e25fa87, SUC_NE_lexicon_tags_1 },
    { 0x199bfa88, SUC_NE_lexicon_tags_2 },
    { 0xdec9fa84, SUC_NE_lexicon_tags_4 },
    { 0x33a5fa8a, SUC_NE_lexicon_tags_2 },
    { 0xbf55fa8b, SUC_NE_lexicon_tags_2 },
    { 0xee81fa82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cfdfa8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe379fa90, SUC_NE_lexicon_tags_2 },
    { 0x1d85fa90, SUC_NE_lexicon_tags_2 },
    { 0x14d1fa92, SUC_NE_lexicon_tags_2 },
    { 0x454bfa93, SUC_NE_lexicon_tags_2 },
    { 0xd39dfa93, SUC_NE_lexicon_tags_58 },
    { 0x72bfa95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73ffa9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x960dfa9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b43faa0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1d9faa2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a9faa6, SUC_NE_lexicon_tags_2 },
    { 0xfd15faa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ebfaaa, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdffaad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca9faaf, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba3fab2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7b3fab4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9663fab6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbffabd, SUC_NE_lexicon_tags_2 },
    { 0x6871fabe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19fac3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f47fac5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8bfac8, SUC_NE_lexicon_tags_2 },
    { 0x225ffac8, SUC_NE_lexicon_tags_2 },
    { 0xfff3faca, SUC_NE_lexicon_tags_2 },
    { 0x1e53facb, SUC_NE_lexicon_tags_2 },
    { 0x3187facc, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6ffacf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9dfad6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93dfadc, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87dfae1, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214dfaeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7d9faed, SUC_NE_lexicon_tags_2 },
    { 0x61d5faee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7faf2, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a9faf7, SUC_NE_lexicon_tags_75 },
    { 0xd8c5faf7, SUC_NE_lexicon_tags_2 },
    { 0x37a3faf7, SUC_NE_lexicon_tags_2 },
    { 0x90effafa, SUC_NE_lexicon_tags_2 },
    { 0x835fafb, SUC_NE_lexicon_tags_2 },
    { 0xca2ffafa, SUC_NE_lexicon_tags_2 },
    { 0x52b1fafd, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3665fb00, SUC_NE_lexicon_tags_2 },
    { 0xb435fb01, SUC_NE_lexicon_tags_2 },
    { 0x7dabfb02, SUC_NE_lexicon_tags_2 },
    { 0x47ebfb03, SUC_NE_lexicon_tags_2 },
    { 0x1471fb04, SUC_NE_lexicon_tags_2 },
    { 0x6863fb05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a7fb0a, SUC_NE_lexicon_tags_2 },
    { 0xdf99fb0a, SUC_NE_lexicon_tags_2 },
    { 0xfc61fb0a, SUC_NE_lexicon_tags_2 },
    { 0xbd37fb0a, SUC_NE_lexicon_tags_2 },
    { 0xe713fb0c, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105bfb13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd935fb18, SUC_NE_lexicon_tags_17 },
    { 0xf8a9fb19, SUC_NE_lexicon_tags_2 },
    { 0x40abfb19, SUC_NE_lexicon_tags_5 },
    { 0, NULL },
    { 0xe975fb1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a69fb1e, SUC_NE_lexicon_tags_2 },
    { 0x7649fb1f, SUC_NE_lexicon_tags_2 },
    { 0xee91fb1e, SUC_NE_lexicon_tags_2 },
    { 0x49a5fb1e, SUC_NE_lexicon_tags_2 },
    { 0xd695fb22, SUC_NE_lexicon_tags_2 },
    { 0x1645fb23, SUC_NE_lexicon_tags_2 },
    { 0xa321fb24, SUC_NE_lexicon_tags_2 },
    { 0x9f21fb1f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54dfb28, SUC_NE_lexicon_tags_2 },
    { 0x519dfb28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb923fb2c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xebebfb2e, SUC_NE_lexicon_tags_2 },
    { 0x8359fb2f, SUC_NE_lexicon_tags_2 },
    { 0x9e73fb30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa85dfb32, SUC_NE_lexicon_tags_2 },
    { 0x89a3fb32, SUC_NE_lexicon_tags_2 },
    { 0x7c19fb32, SUC_NE_lexicon_tags_2 },
    { 0x5dd5fb35, SUC_NE_lexicon_tags_2 },
    { 0x7a97fb36, SUC_NE_lexicon_tags_2 },
    { 0x74a9fb33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78effb3b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d83fb3e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfffb41, SUC_NE_lexicon_tags_2 },
    { 0x3bdbfb41, SUC_NE_lexicon_tags_2 },
    { 0xb537fb42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf031fb45, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ddfb48, SUC_NE_lexicon_tags_2 },
    { 0x84bfb49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada7fb4c, SUC_NE_lexicon_tags_2 },
    { 0xe28dfb4d, SUC_NE_lexicon_tags_2 },
    { 0x4851fb4e, SUC_NE_lexicon_tags_2 },
    { 0xbfb4e, SUC_NE_lexicon_tags_2 },
    { 0x804dfb4f, SUC_NE_lexicon_tags_2 },
    { 0x817ffb51, SUC_NE_lexicon_tags_2 },
    { 0xddaffb52, SUC_NE_lexicon_tags_2 },
    { 0x1ae1fb53, SUC_NE_lexicon_tags_4 },
    { 0x5063fb54, SUC_NE_lexicon_tags_2 },
    { 0x829dfb54, SUC_NE_lexicon_tags_2 },
    { 0x6593fb56, SUC_NE_lexicon_tags_3 },
    { 0x5fffb4c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9767fb5a, SUC_NE_lexicon_tags_2 },
    { 0x120dfb5b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb699fb5d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6dfb61, SUC_NE_lexicon_tags_2 },
    { 0xed33fb62, SUC_NE_lexicon_tags_2 },
    { 0x4f5ffb63, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf99fb65, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee1fb6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815dfb70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f1fb75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc5bfb77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe19fb7b, SUC_NE_lexicon_tags_2 },
    { 0xbd1fb7c, SUC_NE_lexicon_tags_2 },
    { 0xf4a3fb7d, SUC_NE_lexicon_tags_2 },
    { 0x6f53fb7e, SUC_NE_lexicon_tags_2 },
    { 0x10cdfb7e, SUC_NE_lexicon_tags_2 },
    { 0xaeb1fb80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5ffb83, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe7fb86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7659fb89, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5fb8b, SUC_NE_lexicon_tags_2 },
    { 0x8281fb8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x8277fb8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x142dfb90, SUC_NE_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf581fb93, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dadfb95, SUC_NE_lexicon_tags_77 },
    { 0x70b9fb96, SUC_NE_lexicon_tags_34 },
    { 0xfef9fb95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5f9fb99, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48effb9c, SUC_NE_lexicon_tags_2 },
    { 0x461dfb9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc1fba0, SUC_NE_lexicon_tags_2 },
    { 0x1d5fba1, SUC_NE_lexicon_tags_2 },
    { 0x61cffba0, SUC_NE_lexicon_tags_2 },
    { 0x86bbfba3, SUC_NE_lexicon_tags_2 },
    { 0x4499fba3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb77fba8, SUC_NE_lexicon_tags_2 },
    { 0xde7bfba8, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0xc277fbab, SUC_NE_lexicon_tags_2 },
    { 0x7f05fbac, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ef9fbae, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263ffbb9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de5fbbc, SUC_NE_lexicon_tags_2 },
    { 0xd24dfbbd, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ebfbc1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3311fbc3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03fbc8, SUC_NE_lexicon_tags_2 },
    { 0x1043fbc9, SUC_NE_lexicon_tags_2 },
    { 0x6cf1fbca, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da7fbce, SUC_NE_lexicon_tags_2 },
    { 0x607bfbcf, SUC_NE_lexicon_tags_2 },
    { 0x40bdfbce, SUC_NE_lexicon_tags_2 },
    { 0x360dfbd1, SUC_NE_lexicon_tags_34 },
    { 0, NULL },
    { 0x3899fbd3, SUC_NE_lexicon_tags_2 },
    { 0xaf47fbd4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcdfbdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x330dfbdf, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481ffbe4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce7fbe7, SUC_NE_lexicon_tags_2 },
    { 0x44d5fbe8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb919fbea, SUC_NE_lexicon_tags_2 },
    { 0x3059fbeb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xb55bfbed, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe7fbf0, SUC_NE_lexicon_tags_2 },
    { 0x9eddfbf1, SUC_NE_lexicon_tags_5 },
    { 0xacabfbf2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x43cdfbf4, SUC_NE_lexicon_tags_5 },
    { 0x14dffbf4, SUC_NE_lexicon_tags_7 },
    { 0x2feffbf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xda41fbf8, SUC_NE_lexicon_tags_2 },
    { 0x31fbfbf9, SUC_NE_lexicon_tags_2 },
    { 0xf73bfbf9, SUC_NE_lexicon_tags_4 },
    { 0x5985fbfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc01fbff, SUC_NE_lexicon_tags_2 },
    { 0xb135fc00, SUC_NE_lexicon_tags_2 },
    { 0x8397fc01, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d13fc07, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb09fc0a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632ffc0f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186ffc15, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9fc19, SUC_NE_lexicon_tags_2 },
    { 0x21ebfc1a, SUC_NE_lexicon_tags_2 },
    { 0x7df1fc1b, SUC_NE_lexicon_tags_2 },
    { 0x6323fc1b, SUC_NE_lexicon_tags_7 },
    { 0x2debfc1c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc9fc22, SUC_NE_lexicon_tags_2 },
    { 0xb741fc23, SUC_NE_lexicon_tags_2 },
    { 0x3f45fc24, SUC_NE_lexicon_tags_2 },
    { 0xd539fc22, SUC_NE_lexicon_tags_2 },
    { 0xdc47fc26, SUC_NE_lexicon_tags_2 },
    { 0xeaa3fc27, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88bfc2a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x10c3fc2c, SUC_NE_lexicon_tags_2 },
    { 0xdec1fc2c, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6955fc31, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddbfc38, SUC_NE_lexicon_tags_1 },
    { 0xac4bfc39, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb9fc3b, SUC_NE_lexicon_tags_2 },
    { 0xfa99fc3b, SUC_NE_lexicon_tags_3 },
    { 0x2c69fc3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dd7fc3f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e75fc41, SUC_NE_lexicon_tags_2 },
    { 0x7b9fc42, SUC_NE_lexicon_tags_2 },
    { 0xa1f1fc43, SUC_NE_lexicon_tags_2 },
    { 0x20d7fc43, SUC_NE_lexicon_tags_3 },
    { 0x2617fc43, SUC_NE_lexicon_tags_215 },
    { 0, NULL },
    { 0x976ffc47, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x21e5fc49, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53fc52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9683fc57, SUC_NE_lexicon_tags_2 },
    { 0xd721fc58, SUC_NE_lexicon_tags_2 },
    { 0xbf33fc58, SUC_NE_lexicon_tags_2 },
    { 0xfc93fc5a, SUC_NE_lexicon_tags_2 },
    { 0x9f8bfc59, SUC_NE_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe761fc5f, SUC_NE_lexicon_tags_4 },
    { 0x80e3fc5f, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b55fc64, SUC_NE_lexicon_tags_2 },
    { 0x5bf5fc65, SUC_NE_lexicon_tags_2 },
    { 0xd37bfc66, SUC_NE_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e5fc69, SUC_NE_lexicon_tags_2 },
    { 0xf7f9fc6a, SUC_NE_lexicon_tags_2 },
    { 0x1f29fc6a, SUC_NE_lexicon_tags_2 },
    { 0x6171fc6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3997fc6e, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0x201ffc71, SUC_NE_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1dfc76, SUC_NE_lexicon_tags_2 },
    { 0x35bfc77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x551fc79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd281fc7b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x226ffc7e, SUC_NE_lexicon_tags_2 },
    { 0x84e5fc7f, SUC_NE_lexicon_tags_2 },
    { 0x3b63fc80, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac31fc87, SUC_NE_lexicon_tags_2 },
    { 0x7c3bfc88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14affc8b, SUC_NE_lexicon_tags_34 },
    { 0xe823fc8c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa383fc8e, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66bfc91, SUC_NE_lexicon_tags_2 },
    { 0x8007fc91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3badfc94, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x601dfc96, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf953fc9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6b9fc9d, SUC_NE_lexicon_tags_4 },
    { 0xc8c9fc9e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9dfca1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9cdfca3, SUC_NE_lexicon_tags_28 },
    { 0x322ffca4, SUC_NE_lexicon_tags_2 },
    { 0x51e1fca5, SUC_NE_lexicon_tags_2 },
    { 0x4af5fca5, SUC_NE_lexicon_tags_7 },
    { 0x48dfca4, SUC_NE_lexicon_tags_2 },
    { 0x609dfca4, SUC_NE_lexicon_tags_25 },
    { 0x6da7fca9, SUC_NE_lexicon_tags_2 },
    { 0x405ffca9, SUC_NE_lexicon_tags_2 },
    { 0xf375fcab, SUC_NE_lexicon_tags_362 },
    { 0x3247fcac, SUC_NE_lexicon_tags_2 },
    { 0x8f63fcad, SUC_NE_lexicon_tags_2 },
    { 0xa685fcad, SUC_NE_lexicon_tags_2 },
    { 0xbad9fcaf, SUC_NE_lexicon_tags_2 },
    { 0x7ebdfcad, SUC_NE_lexicon_tags_4 },
    { 0x9fffcad, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3933fcb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6effcbf, SUC_NE_lexicon_tags_2 },
    { 0x5b9dfcc0, SUC_NE_lexicon_tags_7 },
    { 0xb14dfcc0, SUC_NE_lexicon_tags_2 },
    { 0x61f5fcc2, SUC_NE_lexicon_tags_2 },
    { 0xa47dfcc3, SUC_NE_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xac73fcc6, SUC_NE_lexicon_tags_30 },
    { 0x2ca5fcc7, SUC_NE_lexicon_tags_2 },
    { 0x2749fcc8, SUC_NE_lexicon_tags_2 },
    { 0xd8effcc6, SUC_NE_lexicon_tags_34 },
    { 0xbfd7fcc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9fcd1, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f3fcd4, SUC_NE_lexicon_tags_2 },
    { 0x5ba5fcd5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5bfcda, SUC_NE_lexicon_tags_2 },
    { 0x1003fcdb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f9fcdf, SUC_NE_lexicon_tags_2 },
    { 0xfc4ffcdf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2799fce2, SUC_NE_lexicon_tags_2 },
    { 0x93e5fce2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1789fcf3, SUC_NE_lexicon_tags_396 },
    { 0, NULL },
    { 0x2b6dfcf5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf969fcf7, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af9fcfa, SUC_NE_lexicon_tags_2 },
    { 0x949ffcfb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde97fd00, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe187fd03, SUC_NE_lexicon_tags_2 },
    { 0xf4edfd04, SUC_NE_lexicon_tags_2 },
    { 0xf61fd05, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x625dfd08, SUC_NE_lexicon_tags_2 },
    { 0x2f25fd08, SUC_NE_lexicon_tags_2 },
    { 0xac79fd09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e47fd11, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf255fd14, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2cbfd16, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c3fd1a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc883fd1f, SUC_NE_lexicon_tags_2 },
    { 0xdfd20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7dfd23, SUC_NE_lexicon_tags_2 },
    { 0x6dc9fd24, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x94e1fd26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f9fd2b, SUC_NE_lexicon_tags_2 },
    { 0x68fdfd2b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c7fd2f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6303fd32, SUC_NE_lexicon_tags_2 },
    { 0xb041fd33, SUC_NE_lexicon_tags_2 },
    { 0x9319fd32, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x40cdfd36, SUC_NE_lexicon_tags_2 },
    { 0xf5f3fd36, SUC_NE_lexicon_tags_2 },
    { 0xc83dfd38, SUC_NE_lexicon_tags_2 },
    { 0x9db5fd37, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe79fd3e, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d7ffd40, SUC_NE_lexicon_tags_2 },
    { 0x76b7fd41, SUC_NE_lexicon_tags_2 },
    { 0x54a7fd42, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc917fd46, SUC_NE_lexicon_tags_2 },
    { 0x9149fd47, SUC_NE_lexicon_tags_2 },
    { 0xf101fd48, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0xf8bffd4a, SUC_NE_lexicon_tags_2 },
    { 0x96d3fd4a, SUC_NE_lexicon_tags_2 },
    { 0x2ea3fd4c, SUC_NE_lexicon_tags_2 },
    { 0x5f47fd4d, SUC_NE_lexicon_tags_2 },
    { 0xbb35fd4b, SUC_NE_lexicon_tags_436 },
    { 0x34a1fd4f, SUC_NE_lexicon_tags_2 },
    { 0x6931fd50, SUC_NE_lexicon_tags_2 },
    { 0x9e45fd51, SUC_NE_lexicon_tags_2 },
    { 0x62d7fd52, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1099fd59, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922dfd61, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd5fd65, SUC_NE_lexicon_tags_3 },
    { 0xb4affd66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6145fd6a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde11fd6f, SUC_NE_lexicon_tags_2 },
    { 0xe825fd70, SUC_NE_lexicon_tags_2 },
    { 0xef69fd6f, SUC_NE_lexicon_tags_2 },
    { 0x2eebfd72, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a7fd75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bfd7a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82f1fd7c, SUC_NE_lexicon_tags_18 },
    { 0xd031fd7d, SUC_NE_lexicon_tags_2 },
    { 0x4a2bfd7c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afbfd81, SUC_NE_lexicon_tags_2 },
    { 0x607dfd81, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ffd86, SUC_NE_lexicon_tags_2 },
    { 0x78a9fd87, SUC_NE_lexicon_tags_104 },
    { 0x866ffd88, SUC_NE_lexicon_tags_4 },
    { 0x196dfd89, SUC_NE_lexicon_tags_2 },
    { 0xbe49fd89, SUC_NE_lexicon_tags_2 },
    { 0x83ddfd8a, SUC_NE_lexicon_tags_19 },
    { 0x1d05fd8c, SUC_NE_lexicon_tags_7 },
    { 0xb663fd8d, SUC_NE_lexicon_tags_2 },
    { 0x526ffd8c, SUC_NE_lexicon_tags_2 },
    { 0xd769fd8f, SUC_NE_lexicon_tags_2 },
    { 0x333fd8f, SUC_NE_lexicon_tags_2 },
    { 0x16ebfd8c, SUC_NE_lexicon_tags_2 },
    { 0xecf9fd92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cd3fd94, SUC_NE_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c7fd9b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x103bfd9d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c3fda1, SUC_NE_lexicon_tags_2 },
    { 0x341ffda2, SUC_NE_lexicon_tags_2 },
    { 0x30e1fda3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xa07bfda5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e53fda8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51abfdad, SUC_NE_lexicon_tags_2 },
    { 0x9aaffdad, SUC_NE_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7fdb1, SUC_NE_lexicon_tags_2 },
    { 0xc7fbfdb2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b69fdb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cbfdb8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a11fdbd, SUC_NE_lexicon_tags_3 },
    { 0xee1ffdbe, SUC_NE_lexicon_tags_2 },
    { 0xd497fdbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82dfdc3, SUC_NE_lexicon_tags_4 },
    { 0xd943fdc4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xea01fdc6, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe521fdc9, SUC_NE_lexicon_tags_1 },
    { 0x788dfdc9, SUC_NE_lexicon_tags_0 },
    { 0xbd6ffdcb, SUC_NE_lexicon_tags_2 },
    { 0x3ee3fdcc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba3fdcf, SUC_NE_lexicon_tags_11 },
    { 0, NULL },
    { 0x2579fdd1, SUC_NE_lexicon_tags_75 },
    { 0, NULL },
    { 0x6187fdd3, SUC_NE_lexicon_tags_2 },
    { 0x35c1fdd4, SUC_NE_lexicon_tags_2 },
    { 0xf1e7fdd5, SUC_NE_lexicon_tags_2 },
    { 0x1d77fdd5, SUC_NE_lexicon_tags_2 },
    { 0xabd1fdd6, SUC_NE_lexicon_tags_1 },
    { 0x9785fdd8, SUC_NE_lexicon_tags_2 },
    { 0x41c5fdd3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88bdfddc, SUC_NE_lexicon_tags_114 },
    { 0x560ffddc, SUC_NE_lexicon_tags_188 },
    { 0, NULL },
    { 0x5fc9fddf, SUC_NE_lexicon_tags_2 },
    { 0x253ffddf, SUC_NE_lexicon_tags_2 },
    { 0xfac9fddf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c71fde3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512ffdec, SUC_NE_lexicon_tags_19 },
    { 0, NULL },
    { 0x81c9fdee, SUC_NE_lexicon_tags_2 },
    { 0x8135fdef, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b61fdf2, SUC_NE_lexicon_tags_2 },
    { 0x44ddfdf3, SUC_NE_lexicon_tags_3 },
    { 0x4339fdf4, SUC_NE_lexicon_tags_2 },
    { 0x9787fdf3, SUC_NE_lexicon_tags_2 },
    { 0x3e31fdf6, SUC_NE_lexicon_tags_2 },
    { 0x18adfdf6, SUC_NE_lexicon_tags_2 },
    { 0x3a33fdf7, SUC_NE_lexicon_tags_3 },
    { 0x429dfdf8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x416ffdfc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d1fe0c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39dfe13, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xf91fe15, SUC_NE_lexicon_tags_2 },
    { 0xd2a5fe16, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0x51a5fe18, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa03fe1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145fe20, SUC_NE_lexicon_tags_2 },
    { 0xf31fe20, SUC_NE_lexicon_tags_2 },
    { 0x17a9fe22, SUC_NE_lexicon_tags_2 },
    { 0x71c5fe23, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0x8121fe25, SUC_NE_lexicon_tags_2 },
    { 0x4901fe26, SUC_NE_lexicon_tags_2 },
    { 0x4417fe27, SUC_NE_lexicon_tags_2 },
    { 0x3007fe28, SUC_NE_lexicon_tags_2 },
    { 0x1b8ffe28, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x50bffe2b, SUC_NE_lexicon_tags_32 },
    { 0, NULL },
    { 0xb4f9fe2d, SUC_NE_lexicon_tags_7 },
    { 0xc729fe2d, SUC_NE_lexicon_tags_4 },
    { 0xd849fe2e, SUC_NE_lexicon_tags_429 },
    { 0, NULL },
    { 0, NULL },
    { 0xca09fe32, SUC_NE_lexicon_tags_2 },
    { 0xce2dfe32, SUC_NE_lexicon_tags_2 },
    { 0x1f35fe33, SUC_NE_lexicon_tags_2 },
    { 0xa9abfe34, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe69fe37, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae81fe3b, SUC_NE_lexicon_tags_2 },
    { 0x13b1fe3c, SUC_NE_lexicon_tags_2 },
    { 0x943fe3d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8175fe40, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85fe43, SUC_NE_lexicon_tags_2 },
    { 0xf98dfe44, SUC_NE_lexicon_tags_2 },
    { 0x9efffe45, SUC_NE_lexicon_tags_2 },
    { 0xd1a3fe46, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f5fe4a, SUC_NE_lexicon_tags_2 },
    { 0x407fe4b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5bfe4d, SUC_NE_lexicon_tags_17 },
    { 0x672dfe4e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ec1fe50, SUC_NE_lexicon_tags_2 },
    { 0xb0cdfe50, SUC_NE_lexicon_tags_49 },
    { 0xccf5fe52, SUC_NE_lexicon_tags_2 },
    { 0x1899fe52, SUC_NE_lexicon_tags_2 },
    { 0x1c89fe54, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ae5fe56, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fb3fe58, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4687fe5a, SUC_NE_lexicon_tags_2 },
    { 0x471bfe5b, SUC_NE_lexicon_tags_2 },
    { 0xe747fe5b, SUC_NE_lexicon_tags_2 },
    { 0x11fe5a, SUC_NE_lexicon_tags_2 },
    { 0x51e9fe5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb49fe63, SUC_NE_lexicon_tags_2 },
    { 0x6fb1fe64, SUC_NE_lexicon_tags_2 },
    { 0xbc99fe64, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec5fe69, SUC_NE_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5533fe6c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e3fe70, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x7615fe72, SUC_NE_lexicon_tags_2 },
    { 0xca03fe73, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x262bfe75, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e97fe77, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fe7e, SUC_NE_lexicon_tags_2 },
    { 0xf7fffe7f, SUC_NE_lexicon_tags_2 },
    { 0x17e1fe7e, SUC_NE_lexicon_tags_2 },
    { 0xc2ddfe81, SUC_NE_lexicon_tags_2 },
    { 0x2f11fe81, SUC_NE_lexicon_tags_2 },
    { 0x5629fe7e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a9fe86, SUC_NE_lexicon_tags_51 },
    { 0x4bc5fe87, SUC_NE_lexicon_tags_2 },
    { 0x7c67fe88, SUC_NE_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4527fe8d, SUC_NE_lexicon_tags_2 },
    { 0x5fc5fe8e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0dfe91, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32bfe94, SUC_NE_lexicon_tags_2 },
    { 0xbb55fe95, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e1fea1, SUC_NE_lexicon_tags_2 },
    { 0xe2c1fea2, SUC_NE_lexicon_tags_2 },
    { 0x1277fea2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3fead, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fbfeb0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2709feb4, SUC_NE_lexicon_tags_2 },
    { 0xec7bfeb5, SUC_NE_lexicon_tags_2 },
    { 0xf04ffeb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xc579feb8, SUC_NE_lexicon_tags_2 },
    { 0x32bfeb9, SUC_NE_lexicon_tags_4 },
    { 0x9b31feb8, SUC_NE_lexicon_tags_2 },
    { 0xf6dbfebb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b5fec1, SUC_NE_lexicon_tags_2 },
    { 0x1fb5fec2, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432dfec8, SUC_NE_lexicon_tags_2 },
    { 0x6fb3fec9, SUC_NE_lexicon_tags_2 },
    { 0x9063fec8, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x701dfecc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ac3fece, SUC_NE_lexicon_tags_2 },
    { 0x639bfecf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ddffed1, SUC_NE_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2203fed5, SUC_NE_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd5fedc, SUC_NE_lexicon_tags_7 },
    { 0xe281fedd, SUC_NE_lexicon_tags_2 },
    { 0x1d5bfedd, SUC_NE_lexicon_tags_2 },
    { 0x8f51fede, SUC_NE_lexicon_tags_2 },
    { 0x807fedc, SUC_NE_lexicon_tags_2 },
    { 0x2555fee1, SUC_NE_lexicon_tags_2 },
    { 0xa059fee2, SUC_NE_lexicon_tags_2 },
    { 0xc2d3fee3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe75bfee5, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x655dfee7, SUC_NE_lexicon_tags_2 },
    { 0x6f09fee8, SUC_NE_lexicon_tags_2 },
    { 0xd2fbfee9, SUC_NE_lexicon_tags_2 },
    { 0xfafbfeea, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d31fef2, SUC_NE_lexicon_tags_40 },
    { 0xd84bfef3, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaffef9, SUC_NE_lexicon_tags_17 },
    { 0x33e7fef9, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe81fefc, SUC_NE_lexicon_tags_2 },
    { 0xba79fefc, SUC_NE_lexicon_tags_2 },
    { 0x33e7fefe, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25ff03, SUC_NE_lexicon_tags_2 },
    { 0xce47ff04, SUC_NE_lexicon_tags_4 },
    { 0xbea5ff04, SUC_NE_lexicon_tags_2 },
    { 0x254bff06, SUC_NE_lexicon_tags_2 },
    { 0xa75ff06, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6a3ff09, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb83ff10, SUC_NE_lexicon_tags_1 },
    { 0xb49bff11, SUC_NE_lexicon_tags_2 },
    { 0x84f7ff12, SUC_NE_lexicon_tags_2 },
    { 0xcf1fff12, SUC_NE_lexicon_tags_2 },
    { 0x308fff12, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc461ff1b, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bd3ff1d, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc61ff20, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0xae95ff22, SUC_NE_lexicon_tags_2 },
    { 0x6b33ff22, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f95ff26, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0dff29, SUC_NE_lexicon_tags_2 },
    { 0xbd05ff2a, SUC_NE_lexicon_tags_7 },
    { 0xde27ff29, SUC_NE_lexicon_tags_4 },
    { 0x2ee3ff2c, SUC_NE_lexicon_tags_2 },
    { 0x9537ff2a, SUC_NE_lexicon_tags_2 },
    { 0x4c6dff2e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4109ff30, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bff33, SUC_NE_lexicon_tags_25 },
    { 0xebddff33, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f87ff39, SUC_NE_lexicon_tags_2 },
    { 0x2705ff3a, SUC_NE_lexicon_tags_2 },
    { 0x1f0dff3a, SUC_NE_lexicon_tags_2 },
    { 0xc4bdff3b, SUC_NE_lexicon_tags_2 },
    { 0xa4fff3c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97ff43, SUC_NE_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1607ff4b, SUC_NE_lexicon_tags_2 },
    { 0xfc3fff4c, SUC_NE_lexicon_tags_2 },
    { 0x3fafff4d, SUC_NE_lexicon_tags_2 },
    { 0x984dff4b, SUC_NE_lexicon_tags_17 },
    { 0, NULL },
    { 0x49cfff50, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15ff53, SUC_NE_lexicon_tags_2 },
    { 0x8f67ff54, SUC_NE_lexicon_tags_2 },
    { 0x3949ff55, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0xb545ff57, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef1ff5a, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9391ff5e, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d45ff61, SUC_NE_lexicon_tags_2 },
    { 0x6029ff62, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5873ff65, SUC_NE_lexicon_tags_2 },
    { 0xf79ff66, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfff6c, SUC_NE_lexicon_tags_2 },
    { 0xbb71ff6d, SUC_NE_lexicon_tags_2 },
    { 0xb61fff6e, SUC_NE_lexicon_tags_2 },
    { 0x9429ff6f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e23ff73, SUC_NE_lexicon_tags_4 },
    { 0xbb17ff74, SUC_NE_lexicon_tags_2 },
    { 0x75b9ff75, SUC_NE_lexicon_tags_2 },
    { 0xcd5dff76, SUC_NE_lexicon_tags_7 },
    { 0x9cffff73, SUC_NE_lexicon_tags_2 },
    { 0x330dff78, SUC_NE_lexicon_tags_2 },
    { 0xca99ff79, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2753ff82, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4925ff84, SUC_NE_lexicon_tags_2 },
    { 0x5b31ff85, SUC_NE_lexicon_tags_2 },
    { 0x372dff86, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x670dff88, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8177ff8c, SUC_NE_lexicon_tags_294 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfff8f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5bff92, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e5ff98, SUC_NE_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c37ff9b, SUC_NE_lexicon_tags_14 },
    { 0xf061ff9c, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd915ff9f, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe71ffa7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a23ffab, SUC_NE_lexicon_tags_2 },
    { 0xe481ffab, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259ffb2, SUC_NE_lexicon_tags_2 },
    { 0xf61ffb2, SUC_NE_lexicon_tags_2 },
    { 0xfefdffb4, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x2de5ffb6, SUC_NE_lexicon_tags_2 },
    { 0x5d6fffb6, SUC_NE_lexicon_tags_2 },
    { 0xf2c7ffb8, SUC_NE_lexicon_tags_2 },
    { 0x3727ffb7, SUC_NE_lexicon_tags_69 },
    { 0, NULL },
    { 0x93a9ffbb, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dffbf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adbffc6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dfffca, SUC_NE_lexicon_tags_2 },
    { 0xc8d3ffca, SUC_NE_lexicon_tags_2 },
    { 0x14cbffcc, SUC_NE_lexicon_tags_2 },
    { 0xda8fffcc, SUC_NE_lexicon_tags_2 },
    { 0x60f9ffce, SUC_NE_lexicon_tags_2 },
    { 0x9a19ffcf, SUC_NE_lexicon_tags_2 },
    { 0x84f5ffcf, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec13ffd3, SUC_NE_lexicon_tags_2 },
    { 0x5227ffd4, SUC_NE_lexicon_tags_2 },
    { 0x2ff3ffd3, SUC_NE_lexicon_tags_2 },
    { 0x5d33ffd6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddfffd9, SUC_NE_lexicon_tags_23 },
    { 0, NULL },
    { 0x41a3ffdb, SUC_NE_lexicon_tags_2 },
    { 0x14dfffdc, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x82f7ffde, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c03ffe1, SUC_NE_lexicon_tags_2 },
    { 0xd1c5ffe2, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea3ffe6, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x4519ffe8, SUC_NE_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f5ffec, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b45ffee, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0x709ffff0, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc725fff6, SUC_NE_lexicon_tags_2 },
    { 0x39d1fff7, SUC_NE_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f7ffff, SUC_NE_lexicon_tags_2 }
};


static inline const label *SUC_NE_lexicon_get_tags(uint32_t key) {
    size_t i = key & 0x1ffff;
    for (;;) {
        if (SUC_NE_lexicon[i].hash == key) return (const label*)SUC_NE_lexicon[i].value;
        if (SUC_NE_lexicon[i].value == NULL) return SUC_NE_lexicon_tags_open;
        i = (i + 1) & 0x1ffff;
    }
}

#define get_tags SUC_NE_lexicon_get_tags
#define names_size 0x40000

static const hash32_kv_label names[names_size] = {
    { 0x14580000, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92240003, 3 },
    { 0x1b100004, 3 },
    { 0xbae40005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafc000a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe00010, 4 },
    { 0x614c0011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc0016, 4 },
    { 0, 0 },
    { 0xbc5c0018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08001b, 3 },
    { 0x4864001c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fc0022, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8500027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf858002a, 3 },
    { 0xa168002b, 3 },
    { 0xa4d0002c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fec002f, 3 },
    { 0xede40030, 4 },
    { 0x53340031, 4 },
    { 0x9700032, 3 },
    { 0, 0 },
    { 0xf8180034, 3 },
    { 0, 0 },
    { 0x92040036, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec40039, 3 },
    { 0, 0 },
    { 0xfab8003b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68840044, 3 },
    { 0x52640045, 3 },
    { 0, 0 },
    { 0x67480047, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e60004b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa028004e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83340054, 3 },
    { 0x50dc0055, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a0005c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d0005f, 3 },
    { 0x2888005f, 4 },
    { 0, 0 },
    { 0x464c0062, 4 },
    { 0x9cc00062, 3 },
    { 0xee600063, 3 },
    { 0xa9ec0062, 4 },
    { 0xa4fc0066, 3 },
    { 0xa1240067, 3 },
    { 0x5ef80062, 3 },
    { 0xde00069, 4 },
    { 0x20f4006a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec006d, 5 },
    { 0x9ecc006e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80071, 4 },
    { 0xef780071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d080078, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d00084, 3 },
    { 0x9ad40084, 3 },
    { 0x84c40086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb900008c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe768008f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9680094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b4009a, 3 },
    { 0, 0 },
    { 0xa29c009c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x540009f, 3 },
    { 0x734400a0, 3 },
    { 0xc0c00a0, 3 },
    { 0, 0 },
    { 0x972000a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb400a7, 4 },
    { 0x673800a8, 4 },
    { 0xad3800a7, 5 },
    { 0x738400aa, 3 },
    { 0x3d4c00a9, 3 },
    { 0x5c1000ac, 3 },
    { 0x30dc00ac, 3 },
    { 0x873c00ae, 3 },
    { 0x693400ae, 3 },
    { 0xdc6c00a8, 3 },
    { 0, 0 },
    { 0xf7b800b2, 3 },
    { 0xf80800b3, 4 },
    { 0x35fc00b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876c00be, 4 },
    { 0, 0 },
    { 0x5e0c00c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5c00c3, 3 },
    { 0x81f000c4, 3 },
    { 0x42f400c5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72000cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83400d3, 3 },
    { 0, 0 },
    { 0xbd1800d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58800dd, 4 },
    { 0xbb0c00dd, 3 },
    { 0x5f5000df, 3 },
    { 0x7a3000de, 4 },
    { 0x345000e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac400ee, 3 },
    { 0xf6ec00ee, 3 },
    { 0x8bc800f0, 3 },
    { 0xad4400ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c000f4, 3 },
    { 0x46800f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c00f8, 4 },
    { 0xd28000f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d400fe, 3 },
    { 0, 0 },
    { 0x2ac00100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4700103, 3 },
    { 0xca400104, 3 },
    { 0xbb80103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d00109, 3 },
    { 0, 0 },
    { 0xcf64010b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfac0112, 3 },
    { 0, 0 },
    { 0x4b840114, 4 },
    { 0x3a000115, 3 },
    { 0xdee40114, 3 },
    { 0xab100117, 3 },
    { 0xd3b40114, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58011b, 3 },
    { 0x8440011c, 3 },
    { 0xd080011d, 3 },
    { 0xf27c011e, 3 },
    { 0, 0 },
    { 0xa3e40120, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d80125, 3 },
    { 0, 0 },
    { 0x31500127, 3 },
    { 0x78b80128, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d8012c, 3 },
    { 0, 0 },
    { 0x62f4012e, 3 },
    { 0, 0 },
    { 0xdf380130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d80134, 3 },
    { 0xc9d40134, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac0139, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x964c013c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44013f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a700149, 3 },
    { 0, 0 },
    { 0x8eec014b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa8014e, 3 },
    { 0x1870014f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c0153, 4 },
    { 0xc8940154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b8015c, 3 },
    { 0xa774015d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c80160, 4 },
    { 0, 0 },
    { 0xb27c0162, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc0167, 3 },
    { 0, 0 },
    { 0xbe740169, 3 },
    { 0xdeac0169, 3 },
    { 0xf588016b, 3 },
    { 0x10e8016b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b0016f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5280172, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96600175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef00178, 3 },
    { 0xa3180178, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ec017e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4600185, 4 },
    { 0x89700185, 4 },
    { 0x61180186, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8018e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c400191, 3 },
    { 0, 0 },
    { 0x31cc0193, 3 },
    { 0, 0 },
    { 0xf0b00195, 3 },
    { 0, 0 },
    { 0xc27c0197, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5964019d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc801a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69001a6, 3 },
    { 0x6e1001a7, 3 },
    { 0x110c01a8, 4 },
    { 0x478c01a6, 4 },
    { 0x4b5401a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6401b2, 5 },
    { 0x5a7801b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486801b8, 3 },
    { 0xb77401b9, 3 },
    { 0, 0 },
    { 0x8a2001bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb801bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x933801c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0c01c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f801cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b801cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185001d5, 3 },
    { 0x319c01d5, 3 },
    { 0x90d801d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9801dc, 4 },
    { 0xf32801dd, 3 },
    { 0xc73401de, 3 },
    { 0x24f001df, 3 },
    { 0, 0 },
    { 0xf1f401e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26401e8, 3 },
    { 0x21bc01e9, 3 },
    { 0xf60001ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e801ee, 3 },
    { 0, 0 },
    { 0xd2d001f0, 4 },
    { 0xb26801f1, 3 },
    { 0, 0 },
    { 0xc3f001f3, 4 },
    { 0xea8401f4, 3 },
    { 0xbb4401f4, 4 },
    { 0x67401f4, 3 },
    { 0x8d6401f4, 3 },
    { 0x818801f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361801fd, 3 },
    { 0x113801fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cc0204, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30c020b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee4020f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46500214, 3 },
    { 0xb5c40214, 5 },
    { 0xd2680216, 3 },
    { 0xd7dc0216, 3 },
    { 0x2f840215, 4 },
    { 0xc6000219, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4084021c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a580220, 3 },
    { 0, 0 },
    { 0x56a40222, 3 },
    { 0, 0 },
    { 0xaf4c0224, 3 },
    { 0xbca80224, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed640229, 4 },
    { 0xd6f8022a, 3 },
    { 0x19a8022b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01c022e, 3 },
    { 0x884022f, 3 },
    { 0, 0 },
    { 0x9f700231, 4 },
    { 0, 0 },
    { 0x1f40233, 3 },
    { 0, 0 },
    { 0x7c4c0235, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xed940238, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca640240, 3 },
    { 0x1ce00240, 3 },
    { 0x9a5c0241, 3 },
    { 0x2d480243, 3 },
    { 0x5980244, 3 },
    { 0x4500245, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80248, 3 },
    { 0x6ec40248, 3 },
    { 0, 0 },
    { 0x7620024b, 3 },
    { 0, 0 },
    { 0x4ec8024d, 3 },
    { 0, 0 },
    { 0x738024f, 3 },
    { 0, 0 },
    { 0x980251, 3 },
    { 0x5b140252, 3 },
    { 0x17480252, 3 },
    { 0, 0 },
    { 0x39f80255, 3 },
    { 0xb00c0256, 4 },
    { 0x16700256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8025b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8025e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7c0265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775c0269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66c026c, 3 },
    { 0xb0b4026d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b080273, 3 },
    { 0x91e40274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e80279, 3 },
    { 0x8e580279, 3 },
    { 0x6204027b, 5 },
    { 0x11e8027b, 3 },
    { 0xd50027c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac40281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2c028b, 3 },
    { 0x322c028c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67700292, 3 },
    { 0, 0 },
    { 0x46e80294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61c029a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0029e, 3 },
    { 0x30b4029f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04c02a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca002a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361002ae, 4 },
    { 0x445002ae, 4 },
    { 0x50d802b0, 3 },
    { 0xe57002b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db402b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x755802b7, 3 },
    { 0, 0 },
    { 0x607802b9, 3 },
    { 0, 0 },
    { 0x69c402bb, 3 },
    { 0, 0 },
    { 0xbf1c02bd, 4 },
    { 0, 0 },
    { 0xf9b802bf, 4 },
    { 0xd5cc02c0, 3 },
    { 0xc48402c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca402c7, 3 },
    { 0x403402c7, 3 },
    { 0xf9a802c8, 3 },
    { 0x7ea802c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88402cd, 3 },
    { 0, 0 },
    { 0xba7802cf, 3 },
    { 0x4e5402d0, 3 },
    { 0xff9402cf, 3 },
    { 0x1f0802d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58402d9, 3 },
    { 0x857c02d9, 3 },
    { 0x9d002db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6802de, 3 },
    { 0x9d8002df, 4 },
    { 0x431002e0, 3 },
    { 0x524402df, 3 },
    { 0x4bd002e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c02e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c02f3, 3 },
    { 0x992802f4, 3 },
    { 0, 0 },
    { 0xf8cc02f6, 3 },
    { 0x304402f7, 3 },
    { 0x8fc02f8, 3 },
    { 0x960c02f9, 3 },
    { 0xe1ac02f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a600303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4c0308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6380030b, 3 },
    { 0xc0ac030b, 3 },
    { 0xd348030d, 3 },
    { 0xa0030e, 4 },
    { 0xe9c0030f, 3 },
    { 0x4094030e, 3 },
    { 0xfa880310, 3 },
    { 0xb4100312, 3 },
    { 0x9374030f, 3 },
    { 0xec80314, 4 },
    { 0x34a0030f, 3 },
    { 0xac030c, 3 },
    { 0x8c6c030e, 4 },
    { 0x479c0318, 3 },
    { 0xf7840319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d00321, 3 },
    { 0xc5680322, 3 },
    { 0, 0 },
    { 0x687c0324, 5 },
    { 0, 0 },
    { 0x8e8c0326, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6900329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14032c, 3 },
    { 0xe0fc032d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97c0332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c80336, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690033d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae80346, 3 },
    { 0x21f40347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda24034e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d40356, 3 },
    { 0xc9840357, 3 },
    { 0x5fb00358, 3 },
    { 0xd5f80358, 4 },
    { 0xe5f40358, 3 },
    { 0x57100358, 3 },
    { 0xd4ec0359, 3 },
    { 0x8a70035d, 3 },
    { 0xa704035e, 3 },
    { 0x9208035d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c0363, 3 },
    { 0x3e6c0363, 3 },
    { 0, 0 },
    { 0xf5700366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bc0369, 3 },
    { 0, 0 },
    { 0xf7d0036b, 4 },
    { 0x652c036b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b60036f, 3 },
    { 0xf3e8036f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6504037a, 4 },
    { 0, 0 },
    { 0x133c037c, 3 },
    { 0x23b4037d, 3 },
    { 0x44f8037c, 3 },
    { 0xd83c037f, 3 },
    { 0xab640380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777c0384, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840c0389, 3 },
    { 0xa120038a, 3 },
    { 0x5364038b, 3 },
    { 0, 0 },
    { 0xc8a8038d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16600392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4c0395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb380398, 3 },
    { 0, 0 },
    { 0xeb64039a, 4 },
    { 0, 0 },
    { 0x9478039c, 3 },
    { 0, 0 },
    { 0x593c039e, 3 },
    { 0x90c4039f, 3 },
    { 0x8b40039f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a403a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51403a7, 3 },
    { 0xed1403a8, 4 },
    { 0, 0 },
    { 0xaeac03aa, 3 },
    { 0x51a803ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726403bb, 3 },
    { 0, 0 },
    { 0xeab003bd, 3 },
    { 0, 0 },
    { 0x37003bf, 3 },
    { 0, 0 },
    { 0xa35c03c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b003c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x935403c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7003cc, 3 },
    { 0, 0 },
    { 0x139003ce, 3 },
    { 0x734003cf, 3 },
    { 0x825c03cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x106803d3, 3 },
    { 0x1aa003d3, 3 },
    { 0x76b003d5, 4 },
    { 0x8bb803d6, 3 },
    { 0x73f803d7, 3 },
    { 0x77403d6, 3 },
    { 0xcc5c03d5, 3 },
    { 0, 0 },
    { 0x30d803db, 4 },
    { 0, 0 },
    { 0x7ba003dd, 3 },
    { 0, 0 },
    { 0x7a5003df, 3 },
    { 0xe9dc03e0, 4 },
    { 0xda6c03e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b803e7, 3 },
    { 0x891803e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70003f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc003fb, 4 },
    { 0, 0 },
    { 0x63403fd, 3 },
    { 0xcd2803fe, 3 },
    { 0, 0 },
    { 0x10c0400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c0407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba880414, 3 },
    { 0x7580415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49180418, 3 },
    { 0xac00419, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb4041e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8980426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d940429, 3 },
    { 0, 0 },
    { 0x480042b, 4 },
    { 0xb090042b, 3 },
    { 0x3084042d, 3 },
    { 0x73c4042e, 3 },
    { 0, 0 },
    { 0x62240430, 3 },
    { 0x2b600431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e80435, 4 },
    { 0xda000436, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa724043a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08043d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f880440, 3 },
    { 0x7dcc0441, 3 },
    { 0x35140441, 3 },
    { 0xab880443, 3 },
    { 0x7e600441, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b80449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b18044d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae34045a, 3 },
    { 0, 0 },
    { 0x5278045c, 3 },
    { 0xade8045c, 3 },
    { 0, 0 },
    { 0xa3e4045f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e80463, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dc0469, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49c0473, 3 },
    { 0x981c0473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f700481, 3 },
    { 0, 0 },
    { 0x9e0c0483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f380487, 3 },
    { 0x3bf40488, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce84048d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4840490, 3 },
    { 0x2e900491, 4 },
    { 0, 0 },
    { 0xa7280493, 3 },
    { 0x9c700494, 3 },
    { 0, 0 },
    { 0x3a0c0496, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534049b, 3 },
    { 0, 0 },
    { 0xfb98049d, 4 },
    { 0, 0 },
    { 0x5f2c049f, 3 },
    { 0xef404a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0404a4, 3 },
    { 0x88b004a4, 3 },
    { 0xf0ac04a4, 5 },
    { 0x23e004a7, 3 },
    { 0x521c04a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc04ad, 3 },
    { 0xeb4004ae, 3 },
    { 0xe51004ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16804b3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73804b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e804bf, 3 },
    { 0x446004c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81404c3, 3 },
    { 0, 0 },
    { 0x87a804c5, 4 },
    { 0, 0 },
    { 0x77d004c7, 3 },
    { 0, 0 },
    { 0x20fc04c9, 3 },
    { 0x9a5804ca, 3 },
    { 0, 0 },
    { 0x14ac04cc, 3 },
    { 0xe28004cc, 3 },
    { 0x2ae404ce, 4 },
    { 0x107c04cf, 3 },
    { 0xd1f404cd, 3 },
    { 0, 0 },
    { 0x1b2c04d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0c04d7, 3 },
    { 0xf02804d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb404df, 3 },
    { 0x214404df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15004e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9804e9, 4 },
    { 0, 0 },
    { 0xcf2004eb, 3 },
    { 0x290c04eb, 3 },
    { 0x9c804eb, 4 },
    { 0, 0 },
    { 0x1fb804ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894404f7, 3 },
    { 0, 0 },
    { 0xfe9804f9, 3 },
    { 0xb26404fa, 3 },
    { 0x30bc04fb, 3 },
    { 0x4c0404fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597c0502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829c0506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b4050c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f280512, 4 },
    { 0x3b700513, 4 },
    { 0x5bc40514, 3 },
    { 0x8ab80512, 3 },
    { 0x2ea00516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f18051e, 3 },
    { 0x2d54051f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d00526, 3 },
    { 0, 0 },
    { 0xa8300528, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ec052b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c8052e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0053a, 4 },
    { 0xa3c8053b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d8053e, 3 },
    { 0, 0 },
    { 0x7900540, 3 },
    { 0x3b600541, 3 },
    { 0, 0 },
    { 0x45740543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b8054b, 3 },
    { 0x999c054c, 3 },
    { 0xe540054d, 3 },
    { 0x7160054e, 3 },
    { 0x8814054e, 4 },
    { 0x49d8054e, 3 },
    { 0xeea8054f, 3 },
    { 0x1d340552, 3 },
    { 0x97440553, 3 },
    { 0xd690054b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc055b, 3 },
    { 0, 0 },
    { 0xb97c055d, 4 },
    { 0x28c4055e, 3 },
    { 0x53d4055e, 4 },
    { 0, 0 },
    { 0x6f2c0561, 4 },
    { 0xb53c0562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c80565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc80569, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea0056d, 4 },
    { 0x48ac056e, 4 },
    { 0x7768056e, 3 },
    { 0x8e080570, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b040578, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6014057c, 3 },
    { 0xcedc057d, 4 },
    { 0x66bc057e, 3 },
    { 0x1fd8057f, 3 },
    { 0xf638057e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0180584, 3 },
    { 0xc3f40585, 3 },
    { 0xd32c0586, 3 },
    { 0xd79c0586, 3 },
    { 0x33a00588, 3 },
    { 0, 0 },
    { 0xe488058a, 4 },
    { 0x9b50058b, 4 },
    { 0xa1c8058c, 3 },
    { 0, 0 },
    { 0x9f0c058e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140595, 3 },
    { 0, 0 },
    { 0x95e40597, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7174059d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800005a4, 3 },
    { 0, 0 },
    { 0x5a3405a6, 4 },
    { 0xe08405a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620c05ae, 3 },
    { 0x7e9c05af, 3 },
    { 0xa1a405ae, 4 },
    { 0x57f405b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f405b6, 4 },
    { 0, 0 },
    { 0xae3c05b8, 3 },
    { 0x340405b9, 3 },
    { 0x4e9405b9, 3 },
    { 0, 0 },
    { 0x7edc05bc, 5 },
    { 0x9f3005bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701405c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313405c7, 3 },
    { 0, 0 },
    { 0x72b405c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f805cd, 3 },
    { 0x703805ce, 3 },
    { 0x6f0c05cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93c05d2, 3 },
    { 0x93a405d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207405d7, 4 },
    { 0x118c05d8, 3 },
    { 0, 0 },
    { 0x1d0805da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfc05dd, 3 },
    { 0, 0 },
    { 0x73cc05df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a005e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28405ee, 4 },
    { 0x13f005ee, 3 },
    { 0, 0 },
    { 0x1c6405f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8405ff, 3 },
    { 0x22fc05ff, 3 },
    { 0x70c40600, 3 },
    { 0x9d4c0602, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79580609, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8404060d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d100612, 3 },
    { 0x31f80613, 3 },
    { 0xd1400614, 3 },
    { 0x851c0612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b00618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec4061c, 3 },
    { 0xbce8061d, 5 },
    { 0x38b4061d, 3 },
    { 0xdf84061e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21900623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc00628, 3 },
    { 0x51d40628, 3 },
    { 0x128062a, 4 },
    { 0xfc14062b, 3 },
    { 0x70c0062c, 3 },
    { 0x498c0628, 3 },
    { 0xc33c062e, 3 },
    { 0x62f8062f, 3 },
    { 0xe5080630, 3 },
    { 0xce380631, 3 },
    { 0x114c0632, 4 },
    { 0x2fec0629, 3 },
    { 0x39a4062d, 3 },
    { 0x2e40635, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1140639, 3 },
    { 0x74880639, 3 },
    { 0xd8d8063a, 3 },
    { 0xcf30063c, 3 },
    { 0x60e8063b, 3 },
    { 0x72e0063b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a40642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bc0646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e50064b, 4 },
    { 0x6c48064c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf8064f, 3 },
    { 0x55a0064f, 4 },
    { 0x93380651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c40655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44065b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64065e, 3 },
    { 0x1338065e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9c0663, 3 },
    { 0, 0 },
    { 0xae8c0665, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa044066b, 3 },
    { 0xd2b8066c, 3 },
    { 0, 0 },
    { 0xc604066e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff00677, 3 },
    { 0, 0 },
    { 0x96140679, 3 },
    { 0, 0 },
    { 0x9ac8067b, 4 },
    { 0x9d50067c, 3 },
    { 0, 0 },
    { 0x9464067e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac0681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c0686, 4 },
    { 0x9fe00687, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c0690, 3 },
    { 0x3ec00691, 3 },
    { 0xff640692, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6700696, 5 },
    { 0x10500697, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b4069c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362006a3, 3 },
    { 0, 0 },
    { 0x9d2006a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90806a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781806ad, 4 },
    { 0, 0 },
    { 0xbd1c06af, 3 },
    { 0x232006b0, 4 },
    { 0xe79806af, 5 },
    { 0, 0 },
    { 0xd6ac06b3, 3 },
    { 0, 0 },
    { 0xfd5c06b5, 3 },
    { 0, 0 },
    { 0x265006b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5406bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61406be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x127c06c1, 3 },
    { 0xdb6406c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274c06ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0806ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76406d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41406d7, 4 },
    { 0xa36c06d8, 3 },
    { 0xfba806d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6406dc, 3 },
    { 0x739c06dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659406e1, 3 },
    { 0, 0 },
    { 0xc34c06e3, 3 },
    { 0x278406e3, 3 },
    { 0xa06c06e4, 3 },
    { 0xaec806e6, 3 },
    { 0x1ca406e7, 3 },
    { 0, 0 },
    { 0x4a8c06e9, 3 },
    { 0xec2406ea, 3 },
    { 0x9aec06eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8c06f0, 3 },
    { 0, 0 },
    { 0x315006f2, 3 },
    { 0x672806f2, 5 },
    { 0, 0 },
    { 0x155006f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d406f9, 3 },
    { 0x8c4006fa, 3 },
    { 0xc43806fa, 3 },
    { 0, 0 },
    { 0x985c06fd, 4 },
    { 0x11b406fe, 3 },
    { 0x8f9806fe, 3 },
    { 0x96406fd, 3 },
    { 0xd2880701, 4 },
    { 0, 0 },
    { 0x89c40703, 4 },
    { 0, 0 },
    { 0x511c0705, 3 },
    { 0xa6d00706, 3 },
    { 0x69cc0707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c070a, 3 },
    { 0xb65c070b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625c0713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d00717, 3 },
    { 0xc7540718, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0071e, 3 },
    { 0x8adc071f, 4 },
    { 0x3b4c0720, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e940725, 3 },
    { 0x4dd80726, 3 },
    { 0x51980727, 4 },
    { 0x40740727, 3 },
    { 0x4c380727, 3 },
    { 0xa9240727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c20072d, 4 },
    { 0x97ac072d, 3 },
    { 0x740072f, 4 },
    { 0x97580730, 4 },
    { 0xd5980731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b80734, 4 },
    { 0xab2c0735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b40739, 3 },
    { 0x6f98073a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9500745, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c0748, 3 },
    { 0xdb800749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c58074f, 3 },
    { 0, 0 },
    { 0x7e340751, 3 },
    { 0, 0 },
    { 0xb080753, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf100758, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa238075b, 3 },
    { 0xf258075b, 4 },
    { 0xa018075d, 3 },
    { 0xd0b8075b, 3 },
    { 0xc520075f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x161c0762, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ac0769, 3 },
    { 0, 0 },
    { 0xa4d4076b, 3 },
    { 0, 0 },
    { 0xfddc076d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da80770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50100774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0784, 3 },
    { 0x7ec40784, 3 },
    { 0x5ed00784, 3 },
    { 0x44440784, 3 },
    { 0xb0540785, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2800790, 3 },
    { 0xb7e80791, 3 },
    { 0x91f00792, 3 },
    { 0x7e700790, 3 },
    { 0x950c0794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc0798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e60079c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dec07a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef807a7, 4 },
    { 0xf03007a8, 3 },
    { 0, 0 },
    { 0xa0a807aa, 3 },
    { 0x8ebc07ab, 3 },
    { 0xc13007aa, 3 },
    { 0x786007ad, 3 },
    { 0x79ec07ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc807b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9007b7, 3 },
    { 0xf6807b8, 3 },
    { 0, 0 },
    { 0xf4f007ba, 3 },
    { 0, 0 },
    { 0x160007bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd407c0, 3 },
    { 0x561407c0, 3 },
    { 0, 0 },
    { 0x716007c3, 3 },
    { 0, 0 },
    { 0x3ee007c5, 3 },
    { 0, 0 },
    { 0xb5dc07c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c07d3, 3 },
    { 0x401007d3, 3 },
    { 0x709407d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9407d8, 4 },
    { 0x210807d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x640807dc, 3 },
    { 0x4a5407dd, 3 },
    { 0x147007de, 3 },
    { 0, 0 },
    { 0xf21807e0, 4 },
    { 0, 0 },
    { 0x4efc07e2, 3 },
    { 0, 0 },
    { 0xc1407e4, 3 },
    { 0x33a407e5, 3 },
    { 0, 0 },
    { 0xadac07e7, 3 },
    { 0x5bf807e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1407ef, 3 },
    { 0x385807ef, 3 },
    { 0xb6a807f0, 3 },
    { 0, 0 },
    { 0x534807f3, 4 },
    { 0x366807f4, 3 },
    { 0x7b2807f5, 3 },
    { 0, 0 },
    { 0x6e7407f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c807fa, 3 },
    { 0x810407fb, 4 },
    { 0xaf6007fc, 4 },
    { 0x3b7007fd, 3 },
    { 0, 0 },
    { 0x79f807ff, 4 },
    { 0xe2040800, 4 },
    { 0x6e840801, 3 },
    { 0, 0 },
    { 0x34c80803, 3 },
    { 0, 0 },
    { 0xda280805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7668080d, 3 },
    { 0, 0 },
    { 0x2f68080f, 3 },
    { 0xc2c80810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53380818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x700081b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0780820, 3 },
    { 0x73280821, 3 },
    { 0, 0 },
    { 0xd4380823, 3 },
    { 0x5d680823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92300827, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb214082c, 3 },
    { 0x8730082d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e00830, 3 },
    { 0x2400831, 3 },
    { 0xf7a80832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5340839, 3 },
    { 0x8a7c083a, 4 },
    { 0x9eb0083a, 3 },
    { 0x505c083c, 3 },
    { 0x18ac083b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d600841, 3 },
    { 0, 0 },
    { 0x8cc0843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fec0848, 3 },
    { 0xa3a80849, 3 },
    { 0xa220084a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea40852, 3 },
    { 0x36740853, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc194085a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa0085f, 4 },
    { 0, 0 },
    { 0x736c0861, 4 },
    { 0, 0 },
    { 0x4bfc0863, 3 },
    { 0xd9f80864, 4 },
    { 0xe4540865, 4 },
    { 0, 0 },
    { 0x2c480867, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b40875, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa100879, 3 },
    { 0xb994087a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dc087f, 5 },
    { 0, 0 },
    { 0x4e440881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c640884, 3 },
    { 0x7b80885, 3 },
    { 0, 0 },
    { 0xef900887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3128088c, 3 },
    { 0x42bc088d, 4 },
    { 0x37fc088e, 4 },
    { 0x7294088e, 3 },
    { 0x2cc088c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae40895, 4 },
    { 0x5f8c0895, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac800899, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8618089d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe808a3, 3 },
    { 0, 0 },
    { 0x20c408a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248408ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401808b4, 3 },
    { 0xd0a008b5, 3 },
    { 0, 0 },
    { 0xccac08b7, 4 },
    { 0, 0 },
    { 0xb7d008b9, 4 },
    { 0x630c08ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82808be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf408c1, 3 },
    { 0, 0 },
    { 0xd15008c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2c08c9, 3 },
    { 0xa72008ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae808d0, 3 },
    { 0, 0 },
    { 0x2f7808d2, 4 },
    { 0x6b6408d3, 3 },
    { 0xb62808d2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4008d8, 3 },
    { 0x587408d9, 3 },
    { 0xf44c08d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30c08de, 3 },
    { 0x316408de, 3 },
    { 0, 0 },
    { 0xa72008e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fc08e5, 3 },
    { 0x875808e6, 3 },
    { 0, 0 },
    { 0xe14808e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832808ee, 3 },
    { 0xd8e408ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb408f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c408f8, 4 },
    { 0x579408f8, 3 },
    { 0x59f008f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e40900, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9480903, 3 },
    { 0xfae80904, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7500909, 3 },
    { 0x57a80909, 3 },
    { 0xf7400909, 3 },
    { 0x38f8090c, 4 },
    { 0xb48090d, 3 },
    { 0x5640090e, 4 },
    { 0x4b64090c, 3 },
    { 0x16d40910, 3 },
    { 0x40480911, 3 },
    { 0x8f080911, 3 },
    { 0x3918090d, 3 },
    { 0xce80914, 3 },
    { 0x2a240915, 4 },
    { 0x3ebc0916, 4 },
    { 0xa35c0917, 3 },
    { 0x7d500918, 4 },
    { 0x74200919, 3 },
    { 0x6804091a, 3 },
    { 0xda34091b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc948091e, 3 },
    { 0xd954091f, 3 },
    { 0, 0 },
    { 0x652c0921, 3 },
    { 0x28740922, 3 },
    { 0x37940923, 3 },
    { 0, 0 },
    { 0xe4840925, 3 },
    { 0x62980926, 3 },
    { 0, 0 },
    { 0x29780928, 3 },
    { 0xe6900929, 3 },
    { 0xbd9c092a, 3 },
    { 0, 0 },
    { 0x1f8c092c, 5 },
    { 0, 0 },
    { 0xc25c092e, 3 },
    { 0xb4c092f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0700933, 3 },
    { 0, 0 },
    { 0xbec40935, 3 },
    { 0, 0 },
    { 0x26200937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d8093a, 3 },
    { 0, 0 },
    { 0xdc093c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34200945, 3 },
    { 0, 0 },
    { 0xd4080947, 3 },
    { 0, 0 },
    { 0x3db00949, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48094c, 4 },
    { 0x6b50094c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b240950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279c0954, 3 },
    { 0xa4600955, 3 },
    { 0, 0 },
    { 0xaa680957, 3 },
    { 0xbba80958, 3 },
    { 0, 0 },
    { 0x2c1c095a, 3 },
    { 0xe570095b, 3 },
    { 0xb44095c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed8095f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad00962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c0966, 3 },
    { 0, 0 },
    { 0xacb80968, 3 },
    { 0x33700969, 3 },
    { 0, 0 },
    { 0x904c096b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda84096f, 3 },
    { 0x24d40970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2480979, 3 },
    { 0x5c94097a, 3 },
    { 0x84ac097b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x792c097e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5600982, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffc0986, 3 },
    { 0xb1000986, 3 },
    { 0x603c0988, 3 },
    { 0, 0 },
    { 0xc360098a, 3 },
    { 0, 0 },
    { 0x3954098c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b0098f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc80996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5009a3, 3 },
    { 0x9d9809a3, 3 },
    { 0, 0 },
    { 0x529c09a6, 3 },
    { 0x103409a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa409ad, 4 },
    { 0x429009ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade809b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc09b8, 3 },
    { 0xb17c09b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48809be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71c09c1, 3 },
    { 0x365409c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f009c5, 3 },
    { 0xf5009c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5009c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac09cc, 4 },
    { 0x82409cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab409d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8009db, 3 },
    { 0x102c09db, 4 },
    { 0x6f4009dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d409e9, 3 },
    { 0xe9d009e9, 5 },
    { 0x6d3009e9, 3 },
    { 0x886409eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01409f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17009f9, 4 },
    { 0x4b4009fa, 3 },
    { 0, 0 },
    { 0xc70409fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1c09ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d40a02, 3 },
    { 0, 0 },
    { 0x57680a04, 3 },
    { 0, 0 },
    { 0x3e40a06, 3 },
    { 0xfd7c0a06, 3 },
    { 0xe7e80a08, 3 },
    { 0x24c80a09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad80a0d, 3 },
    { 0, 0 },
    { 0xd8580a0f, 3 },
    { 0x84540a0f, 3 },
    { 0, 0 },
    { 0x15a40a12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bc0a18, 3 },
    { 0x68d00a18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ac0a1c, 3 },
    { 0x101c0a1d, 3 },
    { 0x21cc0a1e, 3 },
    { 0xd7740a1f, 3 },
    { 0xd2780a20, 3 },
    { 0x63d00a1f, 3 },
    { 0x929c0a1c, 4 },
    { 0x62480a23, 4 },
    { 0xc5680a1f, 3 },
    { 0, 0 },
    { 0x27240a26, 3 },
    { 0xb4940a26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c0a2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f80a33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c00a3a, 3 },
    { 0xfe80a3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70300a46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c0a49, 4 },
    { 0, 0 },
    { 0x6b740a4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f00a58, 3 },
    { 0, 0 },
    { 0xc7200a5a, 3 },
    { 0x19940a5a, 3 },
    { 0xac840a5c, 3 },
    { 0x6ee80a5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f140a60, 3 },
    { 0x5cb80a61, 3 },
    { 0xa4080a61, 3 },
    { 0x50140a63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72900a68, 3 },
    { 0xf0380a69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe100a6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cec0a71, 3 },
    { 0, 0 },
    { 0x86bc0a73, 5 },
    { 0, 0 },
    { 0x629c0a75, 4 },
    { 0, 0 },
    { 0x444c0a77, 3 },
    { 0x77100a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x937c0a7b, 3 },
    { 0xf5480a7b, 3 },
    { 0xee480a7d, 3 },
    { 0xd280a7b, 3 },
    { 0xefd00a7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea00a82, 3 },
    { 0x4040a83, 3 },
    { 0x48c0a84, 3 },
    { 0x82740a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9500a89, 3 },
    { 0x6fa80a89, 3 },
    { 0x92240a8b, 4 },
    { 0x8ac80a8c, 3 },
    { 0xf64c0a8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x200c0a90, 3 },
    { 0x4a900a91, 4 },
    { 0xf7480a91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936c0a99, 3 },
    { 0x3e900a99, 3 },
    { 0x598c0a9b, 3 },
    { 0, 0 },
    { 0xff840a9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5280aa0, 3 },
    { 0, 0 },
    { 0x8b5c0aa2, 3 },
    { 0xb5a40aa2, 3 },
    { 0xae240aa4, 3 },
    { 0x4d3c0aa5, 3 },
    { 0x8bec0aa3, 3 },
    { 0xf2480aa7, 3 },
    { 0, 0 },
    { 0x15c0aa9, 3 },
    { 0, 0 },
    { 0xc8ac0aab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c00ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3740ab5, 4 },
    { 0x70ec0ab5, 3 },
    { 0xffc80ab7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1200abf, 3 },
    { 0, 0 },
    { 0xe0340ac1, 3 },
    { 0xf0ac0ac2, 3 },
    { 0, 0 },
    { 0x72f00ac4, 3 },
    { 0x71040ac4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dc0aca, 3 },
    { 0x6d280acb, 3 },
    { 0, 0 },
    { 0xd4a00acd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7400ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3140ad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e840adb, 3 },
    { 0, 0 },
    { 0x61fc0add, 4 },
    { 0, 0 },
    { 0xab480adf, 3 },
    { 0, 0 },
    { 0x7e780ae1, 4 },
    { 0, 0 },
    { 0x4bfc0ae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d040ae6, 3 },
    { 0xe9cc0ae7, 4 },
    { 0xf8300ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16740aee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f280af1, 4 },
    { 0xf86c0af2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d440af7, 3 },
    { 0xfa140af8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c40afe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d80b01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe700b0a, 3 },
    { 0, 0 },
    { 0x2f00b0c, 3 },
    { 0x40240b0d, 3 },
    { 0x75fc0b0d, 5 },
    { 0x57a80b0e, 4 },
    { 0xbe980b10, 3 },
    { 0xbc240b11, 3 },
    { 0x5c40b11, 5 },
    { 0x31980b12, 4 },
    { 0, 0 },
    { 0xf1280b15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12c0b18, 3 },
    { 0, 0 },
    { 0x8fc0b1a, 3 },
    { 0, 0 },
    { 0x7df40b1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf100b21, 3 },
    { 0x5c780b22, 3 },
    { 0xecdc0b23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe680b28, 4 },
    { 0xb44c0b28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17980b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c40b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53000b36, 4 },
    { 0, 0 },
    { 0x16c40b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x419c0b3b, 3 },
    { 0x8bbc0b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a40b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4480b4a, 3 },
    { 0, 0 },
    { 0x96c0b4c, 3 },
    { 0, 0 },
    { 0xb2a40b4e, 4 },
    { 0xbba80b4f, 3 },
    { 0xe15c0b4e, 4 },
    { 0, 0 },
    { 0x28600b52, 3 },
    { 0xd3e80b53, 3 },
    { 0, 0 },
    { 0x6a640b55, 3 },
    { 0x9480b56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c0b59, 3 },
    { 0xcf640b59, 3 },
    { 0, 0 },
    { 0x79e00b5c, 3 },
    { 0x42e00b5c, 3 },
    { 0x49400b5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc00b61, 3 },
    { 0x91080b62, 4 },
    { 0xe22c0b63, 4 },
    { 0x39c80b64, 3 },
    { 0x12140b65, 4 },
    { 0x7b580b65, 3 },
    { 0xf9e00b67, 3 },
    { 0x1d240b61, 3 },
    { 0x339c0b69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509c0b6d, 5 },
    { 0xb9b40b6e, 3 },
    { 0x83880b6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd40b72, 3 },
    { 0, 0 },
    { 0xa0240b74, 3 },
    { 0, 0 },
    { 0x642c0b76, 4 },
    { 0, 0 },
    { 0xbbb40b78, 3 },
    { 0, 0 },
    { 0xeb680b7a, 3 },
    { 0x9c540b7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e40b84, 4 },
    { 0, 0 },
    { 0x7b300b86, 3 },
    { 0, 0 },
    { 0x6d900b88, 5 },
    { 0x14b00b88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e00b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1640b93, 3 },
    { 0x222c0b93, 4 },
    { 0x2d680b95, 3 },
    { 0xd7540b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8c0b9b, 3 },
    { 0xea740b9b, 4 },
    { 0x5e400b9d, 3 },
    { 0x46580b9e, 3 },
    { 0x43b80b9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf880ba2, 3 },
    { 0, 0 },
    { 0x99b80ba4, 4 },
    { 0, 0 },
    { 0xf04c0ba6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b980bb3, 4 },
    { 0x75b40bb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc0bb9, 3 },
    { 0, 0 },
    { 0xab340bbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa180bbe, 4 },
    { 0, 0 },
    { 0x590c0bc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb40bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf440bc7, 5 },
    { 0xeb6c0bc8, 3 },
    { 0x91480bc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26740bcc, 3 },
    { 0x86180bcd, 4 },
    { 0xcd3c0bcc, 3 },
    { 0xb91c0bcd, 3 },
    { 0x6cf80bcd, 3 },
    { 0xd6bc0bcf, 3 },
    { 0xadf00bd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d000bd5, 4 },
    { 0x2ae80bd5, 3 },
    { 0x7c700bd7, 3 },
    { 0x9c200bd7, 4 },
    { 0xad680bd6, 3 },
    { 0xade00bda, 3 },
    { 0x5c40bd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c00bdf, 3 },
    { 0xc9580be0, 4 },
    { 0x76f40be1, 3 },
    { 0x34c80be1, 3 },
    { 0xf6e80be1, 3 },
    { 0xa82c0be1, 3 },
    { 0xc9b00be5, 3 },
    { 0x43800be6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7c0be9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78840bee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c40bf4, 3 },
    { 0xdbc40bf4, 4 },
    { 0xe6b00bf6, 3 },
    { 0x3cb40bf6, 3 },
    { 0x31c0bf8, 3 },
    { 0x3d400bf7, 3 },
    { 0x21ec0bfa, 4 },
    { 0xbda80bf7, 3 },
    { 0, 0 },
    { 0x91540bfd, 4 },
    { 0x30e40bfe, 3 },
    { 0, 0 },
    { 0x56a80c00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43640c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640c09, 3 },
    { 0xc6c40c0a, 4 },
    { 0x9c100c0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5c0c0e, 4 },
    { 0xb60c0c0e, 3 },
    { 0x701c0c10, 3 },
    { 0xd1300c11, 4 },
    { 0xdf500c12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c0c17, 4 },
    { 0xf8c40c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dc0c1f, 5 },
    { 0x57680c20, 3 },
    { 0x76ac0c21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a40c26, 3 },
    { 0xb7e80c26, 3 },
    { 0, 0 },
    { 0xa7040c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c40c33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2600c39, 3 },
    { 0xdeb00c3a, 3 },
    { 0xa6780c3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f740c3f, 3 },
    { 0x77900c40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd00c44, 3 },
    { 0x53680c45, 3 },
    { 0, 0 },
    { 0x37600c47, 3 },
    { 0x498c0c48, 3 },
    { 0, 0 },
    { 0xcde40c4a, 4 },
    { 0xeb0c0c4b, 4 },
    { 0x89100c4c, 3 },
    { 0x84680c4c, 3 },
    { 0x3d740c4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde600c54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea40c5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad40c64, 3 },
    { 0x157c0c65, 4 },
    { 0x6080c66, 3 },
    { 0xd7780c67, 3 },
    { 0xc8f80c65, 3 },
    { 0xf080c64, 4 },
    { 0xf4580c6a, 3 },
    { 0xe000c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b80c74, 3 },
    { 0x83380c75, 3 },
    { 0xa3ec0c75, 3 },
    { 0, 0 },
    { 0xb0080c78, 4 },
    { 0x26700c79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c80c7d, 3 },
    { 0xd1e80c7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcc0c87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8000c98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0340c9c, 3 },
    { 0, 0 },
    { 0xf6cc0c9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bc0ca3, 3 },
    { 0, 0 },
    { 0xdf200ca5, 4 },
    { 0, 0 },
    { 0xe56c0ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ec0caa, 3 },
    { 0xc3840cab, 3 },
    { 0, 0 },
    { 0x7b940cad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e100cb1, 4 },
    { 0x4c400cb1, 3 },
    { 0xec880cb3, 4 },
    { 0x25e40cb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f40cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa440cbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef00cc4, 3 },
    { 0x24d00cc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab380ccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82780cd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e40cd6, 3 },
    { 0, 0 },
    { 0xc99c0cd8, 4 },
    { 0, 0 },
    { 0x20140cda, 3 },
    { 0x7640cda, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43480ce2, 3 },
    { 0x6940ce3, 3 },
    { 0xc3080ce4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e140ce8, 4 },
    { 0x1a980ce8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a80cec, 3 },
    { 0x1dfc0ced, 3 },
    { 0x95b00ced, 3 },
    { 0x29280ced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8240cf2, 3 },
    { 0x6480cf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee80cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b680cfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c540cff, 3 },
    { 0xdff40d00, 4 },
    { 0xf2640d00, 3 },
    { 0x77f00d02, 3 },
    { 0, 0 },
    { 0x2b00d04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c0d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741c0d12, 4 },
    { 0xd33c0d13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7300d17, 4 },
    { 0, 0 },
    { 0x69680d19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b800d1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a00d26, 3 },
    { 0x8d700d27, 4 },
    { 0x41280d28, 4 },
    { 0x3b300d28, 3 },
    { 0xd8440d28, 3 },
    { 0xc6200d2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92440d2f, 3 },
    { 0xb8500d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f40d3e, 3 },
    { 0xc1040d3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20680d45, 4 },
    { 0x27640d46, 3 },
    { 0xd3640d46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a580d4a, 3 },
    { 0, 0 },
    { 0x3f240d4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a40d51, 3 },
    { 0x80e00d51, 3 },
    { 0xcbd00d51, 3 },
    { 0, 0 },
    { 0xecd40d55, 4 },
    { 0x83480d56, 3 },
    { 0x176c0d57, 3 },
    { 0x926c0d58, 3 },
    { 0x5d0c0d58, 3 },
    { 0x75fc0d5a, 3 },
    { 0x375c0d5b, 3 },
    { 0xad7c0d5a, 3 },
    { 0x10400d57, 5 },
    { 0x10640d5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b80d65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21940d68, 4 },
    { 0x71540d69, 3 },
    { 0x78440d6a, 3 },
    { 0xe140d6b, 4 },
    { 0xee400d6a, 3 },
    { 0x33c40d6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c80d73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380d78, 4 },
    { 0x95bc0d79, 3 },
    { 0x2740d79, 3 },
    { 0xe1000d7a, 3 },
    { 0x5d780d7c, 5 },
    { 0, 0 },
    { 0xdf640d7e, 4 },
    { 0xdf6c0d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x812c0d82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc0d8c, 4 },
    { 0xbb6c0d8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69000d93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda00d96, 3 },
    { 0, 0 },
    { 0xb1a40d98, 3 },
    { 0x70e80d98, 3 },
    { 0xd900d9a, 3 },
    { 0x1fa00d9b, 3 },
    { 0, 0 },
    { 0x46c00d9d, 3 },
    { 0, 0 },
    { 0xdac80d9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66580da7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70140dac, 4 },
    { 0x9bfc0dad, 3 },
    { 0, 0 },
    { 0xfa900daf, 3 },
    { 0x82580db0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda740db6, 4 },
    { 0xafc80db6, 3 },
    { 0x45380db6, 3 },
    { 0x188c0db9, 4 },
    { 0, 0 },
    { 0xdc540dbb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f200dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6000dc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d980dcb, 3 },
    { 0xbdd40dcc, 3 },
    { 0xbfa40dcc, 3 },
    { 0x53ac0dcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c100dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f80dda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38880ddd, 4 },
    { 0, 0 },
    { 0x48b80ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7c0de3, 3 },
    { 0x724c0de4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bc0de7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e380deb, 4 },
    { 0x67400dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17840df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07c0df4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40300dff, 3 },
    { 0x87800dff, 3 },
    { 0x8f700e01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eec0e06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee80e09, 3 },
    { 0, 0 },
    { 0x34b80e0b, 4 },
    { 0x75640e0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11800e0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc40e13, 3 },
    { 0, 0 },
    { 0x82c40e15, 4 },
    { 0x19980e16, 3 },
    { 0x16180e16, 3 },
    { 0x8fa40e18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6740e1c, 4 },
    { 0, 0 },
    { 0xce40e1e, 5 },
    { 0x30c00e1e, 3 },
    { 0x6ea40e20, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f00e25, 4 },
    { 0x1fcc0e26, 3 },
    { 0xc6800e26, 4 },
    { 0, 0 },
    { 0xb8100e29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c0e30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e00e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b00e3b, 3 },
    { 0xa9ac0e3b, 3 },
    { 0xb300e3d, 3 },
    { 0x4ffc0e3d, 3 },
    { 0xfd3c0e3f, 4 },
    { 0xe0a80e3f, 4 },
    { 0, 0 },
    { 0x30340e42, 4 },
    { 0, 0 },
    { 0x51e00e44, 4 },
    { 0x9b340e45, 3 },
    { 0x33380e44, 3 },
    { 0xc2440e45, 3 },
    { 0, 0 },
    { 0x26a00e49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d00e4c, 3 },
    { 0xacd80e4c, 3 },
    { 0x5a680e4d, 3 },
    { 0x54b00e4d, 3 },
    { 0, 0 },
    { 0x52000e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b540e54, 3 },
    { 0, 0 },
    { 0x7ab80e56, 3 },
    { 0x74cc0e57, 3 },
    { 0x82c40e57, 4 },
    { 0x52c80e59, 3 },
    { 0x59e80e5a, 3 },
    { 0, 0 },
    { 0x42e80e5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc00e60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bc0e66, 3 },
    { 0x36340e67, 4 },
    { 0x19c0e68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c40e6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3240e6f, 3 },
    { 0xb49c0e6f, 3 },
    { 0xf1dc0e71, 4 },
    { 0, 0 },
    { 0x2b3c0e73, 3 },
    { 0xd7400e74, 3 },
    { 0xa0cc0e75, 3 },
    { 0x13240e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18400e7a, 4 },
    { 0xbce00e7b, 3 },
    { 0, 0 },
    { 0x4db40e7d, 3 },
    { 0xfe000e7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997c0e85, 4 },
    { 0x57600e86, 3 },
    { 0, 0 },
    { 0x249c0e88, 3 },
    { 0, 0 },
    { 0xd9640e8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0c0e92, 3 },
    { 0, 0 },
    { 0x4c580e94, 3 },
    { 0x4acc0e95, 3 },
    { 0x7ba40e95, 3 },
    { 0xbdc80e97, 3 },
    { 0x40dc0e97, 4 },
    { 0x2680e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24040ea3, 3 },
    { 0xccb40ea4, 5 },
    { 0x91000ea5, 3 },
    { 0x20780ea5, 3 },
    { 0, 0 },
    { 0x5d200ea8, 4 },
    { 0xe47c0ea9, 3 },
    { 0xde980eaa, 3 },
    { 0xcef80eaa, 3 },
    { 0x631c0eac, 4 },
    { 0, 0 },
    { 0x5500eae, 3 },
    { 0x67240eaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5600eb2, 3 },
    { 0, 0 },
    { 0x9cd00eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d80eb8, 3 },
    { 0xad8c0eb8, 3 },
    { 0x63080eba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54c0ec1, 3 },
    { 0xe7a80ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac0ec9, 3 },
    { 0xedf80eca, 3 },
    { 0x4c280eca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2c0ed0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4140ed3, 3 },
    { 0xe81c0ed3, 3 },
    { 0, 0 },
    { 0x55d80ed6, 3 },
    { 0xaf3c0ed7, 3 },
    { 0x4dac0ed8, 5 },
    { 0x19880ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93100edc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d580edf, 4 },
    { 0, 0 },
    { 0x73fc0ee1, 3 },
    { 0xacf00ee2, 3 },
    { 0, 0 },
    { 0x7e6c0ee4, 3 },
    { 0x65f80ee4, 3 },
    { 0x7b400ee5, 3 },
    { 0xb9140ee5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dc0eea, 3 },
    { 0, 0 },
    { 0xd25c0eec, 4 },
    { 0x1d940eec, 4 },
    { 0xa1f40eee, 3 },
    { 0, 0 },
    { 0xf2440ef0, 4 },
    { 0x3d4c0ef1, 3 },
    { 0xedd80ef1, 3 },
    { 0xf1200ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58040ef6, 4 },
    { 0x2c640ef7, 3 },
    { 0, 0 },
    { 0xe7340ef9, 4 },
    { 0, 0 },
    { 0xa2c40efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34840f04, 4 },
    { 0xa9c00f05, 3 },
    { 0x31c00f06, 4 },
    { 0xde00f07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22680f0b, 3 },
    { 0, 0 },
    { 0x2b000f0d, 3 },
    { 0x61300f0e, 3 },
    { 0xeef40f0e, 3 },
    { 0, 0 },
    { 0xc8300f11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab00f14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6080f17, 4 },
    { 0x13ac0f18, 5 },
    { 0, 0 },
    { 0x57240f1a, 4 },
    { 0xc1180f1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f80f20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a840f24, 4 },
    { 0x2f00f24, 3 },
    { 0, 0 },
    { 0x4c5c0f27, 3 },
    { 0x8a600f28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e980f2c, 3 },
    { 0xa0800f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d40f30, 3 },
    { 0x830c0f31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d040f34, 4 },
    { 0x73d40f35, 3 },
    { 0x56800f36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa500f3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fc0f3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c380f45, 3 },
    { 0, 0 },
    { 0x34300f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a80f4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dc0f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8c0f57, 3 },
    { 0xbf580f57, 3 },
    { 0xd80c0f57, 3 },
    { 0xb2180f58, 3 },
    { 0x50400f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e280f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe200f62, 4 },
    { 0x6f400f63, 4 },
    { 0x7c080f64, 3 },
    { 0, 0 },
    { 0x241c0f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e40f6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d80f6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c0f72, 3 },
    { 0, 0 },
    { 0x26740f74, 4 },
    { 0x80e80f75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ec0f7b, 3 },
    { 0xc98c0f7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19980f83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9c0f86, 3 },
    { 0xe17c0f87, 4 },
    { 0, 0 },
    { 0x81a00f89, 3 },
    { 0xc00c0f8a, 3 },
    { 0, 0 },
    { 0xc3840f8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e100f90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5100f93, 3 },
    { 0, 0 },
    { 0x983c0f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea640f98, 3 },
    { 0xa6500f99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0180fa6, 4 },
    { 0xf3800fa6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd00fac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec840fb2, 3 },
    { 0xb7600fb3, 3 },
    { 0x23c40fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f80fb7, 4 },
    { 0xdc1c0fb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97440fc2, 3 },
    { 0, 0 },
    { 0xb7840fc4, 3 },
    { 0x19480fc4, 3 },
    { 0, 0 },
    { 0xd3380fc7, 4 },
    { 0x9e500fc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5840fcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29380fd0, 3 },
    { 0xf33c0fd1, 3 },
    { 0x95440fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b600fd5, 3 },
    { 0x43940fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8740fde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x816c0fe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98200fe5, 4 },
    { 0, 0 },
    { 0x22580fe7, 4 },
    { 0x2b980fe7, 5 },
    { 0x7f040fe9, 3 },
    { 0x449c0fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef40ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c80ff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd500ffc, 3 },
    { 0, 0 },
    { 0x5b800ffe, 3 },
    { 0x27a80fff, 3 },
    { 0x91441000, 4 },
    { 0x7e6c1000, 3 },
    { 0, 0 },
    { 0x3241003, 3 },
    { 0x1f381004, 3 },
    { 0xc9f01005, 3 },
    { 0, 0 },
    { 0x582c1007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094100c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570100f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab41014, 4 },
    { 0xe0081015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53481018, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf800101c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb401026, 3 },
    { 0x66c1026, 3 },
    { 0x8a3c1028, 3 },
    { 0x45681028, 4 },
    { 0x46a41029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb608102d, 3 },
    { 0xbda4102e, 3 },
    { 0, 0 },
    { 0x7c981030, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dc1034, 4 },
    { 0x31241035, 3 },
    { 0x86001034, 3 },
    { 0x199c1034, 3 },
    { 0x21541038, 3 },
    { 0x9eb01036, 3 },
    { 0xd0cc103a, 4 },
    { 0xf64c103b, 3 },
    { 0x7490103a, 4 },
    { 0x51301037, 3 },
    { 0x71c8103e, 4 },
    { 0x83b0103c, 4 },
    { 0, 0 },
    { 0x24b81041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11c1044, 3 },
    { 0xa981045, 3 },
    { 0xad881044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e98104c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a501052, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc1055, 3 },
    { 0, 0 },
    { 0x4d801057, 3 },
    { 0xe281057, 3 },
    { 0x79441059, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176c105d, 3 },
    { 0xdfc0105d, 3 },
    { 0xdbd4105f, 3 },
    { 0x3dd81060, 4 },
    { 0x5be01060, 3 },
    { 0, 0 },
    { 0x78181063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58481066, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c106c, 4 },
    { 0, 0 },
    { 0xd48c106e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b01071, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398107c, 4 },
    { 0xd518107d, 3 },
    { 0x4340107c, 3 },
    { 0xeafc107e, 3 },
    { 0x2598107c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c01083, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed01089, 3 },
    { 0x1784108a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e101091, 3 },
    { 0x812c1092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e4109d, 3 },
    { 0x68e8109e, 3 },
    { 0x7be8109e, 3 },
    { 0x6fe810a0, 3 },
    { 0, 0 },
    { 0xb0e010a2, 3 },
    { 0, 0 },
    { 0xe6010a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e810a8, 3 },
    { 0xf09410a8, 4 },
    { 0x8c8010aa, 3 },
    { 0x420c10ab, 4 },
    { 0, 0 },
    { 0x9f9010ad, 4 },
    { 0x89f010ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc10b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c10ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac10bd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00810c1, 3 },
    { 0x694410c1, 3 },
    { 0xf32c10c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2410c6, 4 },
    { 0, 0 },
    { 0xeb9010c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d410cb, 3 },
    { 0x331410cc, 3 },
    { 0x957010cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50410d4, 4 },
    { 0xa1c810d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476c10db, 4 },
    { 0x44c10dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1c10ea, 4 },
    { 0, 0 },
    { 0x299010ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296810f0, 4 },
    { 0x558410f0, 3 },
    { 0x687010f1, 3 },
    { 0xce0810f3, 4 },
    { 0x199410f4, 4 },
    { 0x47d810f4, 3 },
    { 0x20f410f6, 3 },
    { 0xe38810f7, 3 },
    { 0x580410f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e010fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa41100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc41104, 3 },
    { 0x5e2c1104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d8110a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c50110f, 3 },
    { 0xcea8110f, 3 },
    { 0, 0 },
    { 0x9f1c1112, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c41115, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb0111e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7981123, 3 },
    { 0x4dac1123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1241128, 4 },
    { 0x3281128, 3 },
    { 0x55d8112a, 3 },
    { 0, 0 },
    { 0xca5c112c, 3 },
    { 0, 0 },
    { 0x24f4112e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c1134, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c881137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d34113c, 3 },
    { 0x4214113c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e01144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c01148, 3 },
    { 0xd6e41148, 3 },
    { 0, 0 },
    { 0x116c114b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5934114f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66781154, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6350115c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3c1160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f001165, 3 },
    { 0x174c1166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50116b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19441177, 3 },
    { 0xc1541177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4041188, 3 },
    { 0xf63c1189, 3 },
    { 0, 0 },
    { 0xd188118b, 3 },
    { 0xda84118b, 4 },
    { 0, 0 },
    { 0x6fb4118e, 4 },
    { 0, 0 },
    { 0x3381190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99c1193, 3 },
    { 0, 0 },
    { 0xf5b81195, 3 },
    { 0xe5d81196, 4 },
    { 0xd1381196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0119f, 4 },
    { 0, 0 },
    { 0x4ebc11a1, 5 },
    { 0x2a7011a2, 3 },
    { 0xe64811a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc11a7, 4 },
    { 0xd6811a8, 3 },
    { 0xac6811a7, 3 },
    { 0x8c1811aa, 3 },
    { 0x830811a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b011af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf811b2, 4 },
    { 0xf04811b3, 3 },
    { 0x776c11b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c011b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2411bc, 3 },
    { 0x5c9011bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b811c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d011c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c11cf, 3 },
    { 0, 0 },
    { 0xb5b811d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f011d5, 4 },
    { 0x4d1011d6, 3 },
    { 0xdbe411d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c11db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9811e0, 3 },
    { 0x4fb011e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8011e4, 3 },
    { 0x821c11e4, 3 },
    { 0xd02011e6, 3 },
    { 0x491811e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19811ec, 3 },
    { 0xd8e811ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83411f0, 3 },
    { 0, 0 },
    { 0x74a811f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ac11fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f01200, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a441208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2618120b, 4 },
    { 0, 0 },
    { 0xa594120d, 4 },
    { 0x6268120e, 4 },
    { 0x3f88120e, 3 },
    { 0x3d281210, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6701217, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc121d, 3 },
    { 0x98ec121d, 5 },
    { 0, 0 },
    { 0xb6c81220, 3 },
    { 0xb1601221, 3 },
    { 0, 0 },
    { 0xa47c1223, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddc1229, 3 },
    { 0x9c58122a, 3 },
    { 0x3044122b, 3 },
    { 0x21b8122c, 4 },
    { 0, 0 },
    { 0x4fac122e, 3 },
    { 0, 0 },
    { 0x116c1230, 3 },
    { 0xef201231, 3 },
    { 0x7a701232, 3 },
    { 0, 0 },
    { 0x83f41234, 4 },
    { 0x9e4c1235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f8123c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb001241, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88901245, 3 },
    { 0x654c1246, 3 },
    { 0xd5441247, 3 },
    { 0x9e541247, 4 },
    { 0, 0 },
    { 0xd104124a, 3 },
    { 0x96ec124b, 5 },
    { 0xae34124c, 3 },
    { 0xf908124d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58141250, 3 },
    { 0x6f041251, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93c1254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a41260, 4 },
    { 0xbda81261, 3 },
    { 0xc8bc1262, 3 },
    { 0, 0 },
    { 0x35f01264, 4 },
    { 0x9fa01264, 3 },
    { 0xc1541266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b41269, 3 },
    { 0xbb3c126a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e81270, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb01273, 3 },
    { 0, 0 },
    { 0x48681275, 4 },
    { 0x68141275, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878127c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e01285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8190128b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24128f, 3 },
    { 0, 0 },
    { 0xf0b81291, 3 },
    { 0, 0 },
    { 0x60c81293, 4 },
    { 0, 0 },
    { 0xb401295, 3 },
    { 0x31e41296, 3 },
    { 0x85541295, 3 },
    { 0xcd501298, 3 },
    { 0, 0 },
    { 0xf0c129a, 3 },
    { 0, 0 },
    { 0xd898129c, 3 },
    { 0xb59c129d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c12a1, 4 },
    { 0x467012a2, 3 },
    { 0x62e412a3, 3 },
    { 0xe7e412a4, 3 },
    { 0xe6ec12a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfec12a8, 4 },
    { 0x5e012a9, 3 },
    { 0x87cc12aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50812b0, 3 },
    { 0x196c12b1, 3 },
    { 0x182012b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483012b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529c12be, 3 },
    { 0xa7812bf, 3 },
    { 0x69f412be, 3 },
    { 0xa73412be, 3 },
    { 0x703c12c2, 3 },
    { 0x967812c3, 3 },
    { 0xeb0c12c4, 3 },
    { 0x177812c5, 3 },
    { 0xa25812c6, 3 },
    { 0x5b7812c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53412cb, 3 },
    { 0x89dc12cc, 4 },
    { 0xcc8412cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6412d0, 3 },
    { 0x895412d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74812da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7412dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734012ef, 4 },
    { 0, 0 },
    { 0x3aec12f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef012f4, 3 },
    { 0x19d812f5, 3 },
    { 0x772c12f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff41302, 3 },
    { 0xd4881303, 3 },
    { 0, 0 },
    { 0x6e201305, 3 },
    { 0x56181306, 3 },
    { 0x6bec1307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324130c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b001311, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dc1316, 3 },
    { 0x7f081316, 3 },
    { 0x43281316, 3 },
    { 0x18201319, 3 },
    { 0x3aac1319, 3 },
    { 0xe688131a, 3 },
    { 0, 0 },
    { 0x8ae0131d, 4 },
    { 0x665c131d, 3 },
    { 0x6cc4131f, 3 },
    { 0, 0 },
    { 0x97d41321, 3 },
    { 0, 0 },
    { 0xda341323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73741329, 3 },
    { 0xacc132a, 4 },
    { 0x1830132b, 3 },
    { 0x88501329, 3 },
    { 0x6bcc132a, 4 },
    { 0xd14132b, 3 },
    { 0xef30132f, 3 },
    { 0xb90c132b, 3 },
    { 0x76f01331, 3 },
    { 0xea4c1332, 3 },
    { 0x65fc1333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d41337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb84133a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87381340, 4 },
    { 0x9c641340, 4 },
    { 0, 0 },
    { 0xfee81343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be01346, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa901349, 3 },
    { 0x9fb4134a, 4 },
    { 0x15fc134b, 3 },
    { 0, 0 },
    { 0xb638134d, 3 },
    { 0xab4134d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec1354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0c135a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc135d, 3 },
    { 0x1784135d, 3 },
    { 0, 0 },
    { 0xb6681360, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3481363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a81369, 3 },
    { 0xc968136a, 4 },
    { 0xdc90136b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54c136e, 3 },
    { 0xc910136e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af41372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c41376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c41379, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f98137c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5c1383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0541386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478138b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d04138e, 3 },
    { 0xcf8c138f, 4 },
    { 0xc1941390, 3 },
    { 0x5cac1390, 3 },
    { 0x6a6c1390, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc1398, 3 },
    { 0x8f01398, 3 },
    { 0xb984139a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ec139d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c13a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc13a5, 3 },
    { 0x738813a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571813ab, 5 },
    { 0xe3a413ac, 4 },
    { 0x1c2c13ad, 4 },
    { 0, 0 },
    { 0x785813af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962413b3, 3 },
    { 0x614813b4, 4 },
    { 0x5b0c13b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c13b9, 3 },
    { 0, 0 },
    { 0x860c13bb, 3 },
    { 0xbdc013bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb413c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c13c5, 4 },
    { 0x9da813c5, 3 },
    { 0x560c13c7, 3 },
    { 0x438413c5, 3 },
    { 0, 0 },
    { 0x541013ca, 3 },
    { 0x6f3013ca, 3 },
    { 0, 0 },
    { 0x385c13cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0c13d0, 3 },
    { 0x6dd813d1, 3 },
    { 0xdc5413d2, 3 },
    { 0x54013d1, 4 },
    { 0xa71413d4, 3 },
    { 0x48e813d5, 4 },
    { 0xcaa813d6, 3 },
    { 0, 0 },
    { 0x659013d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d413dc, 3 },
    { 0xefe013dd, 4 },
    { 0xc84013dd, 4 },
    { 0, 0 },
    { 0x498013e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccc13e9, 3 },
    { 0xef7413ea, 3 },
    { 0x86413e9, 3 },
    { 0xfb6013ec, 4 },
    { 0x950013ed, 3 },
    { 0x2c9813ea, 3 },
    { 0x2b7413ef, 3 },
    { 0xad0c13f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52813f9, 3 },
    { 0xb61813f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16013fd, 3 },
    { 0x203013fe, 3 },
    { 0, 0 },
    { 0x3dd81400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a481406, 4 },
    { 0x6241407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568140a, 3 },
    { 0x90f0140a, 4 },
    { 0xd308140c, 4 },
    { 0x1414140d, 3 },
    { 0xacd8140d, 3 },
    { 0x8210140f, 4 },
    { 0x73ec1410, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a941413, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edc141b, 3 },
    { 0x1ac4141c, 3 },
    { 0, 0 },
    { 0x56b8141e, 5 },
    { 0x9a9c141f, 3 },
    { 0x7a70141f, 4 },
    { 0, 0 },
    { 0x4fc81422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf01425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8481429, 3 },
    { 0x8830142a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e8142e, 3 },
    { 0x185c142f, 3 },
    { 0xa474142e, 3 },
    { 0x2c501431, 3 },
    { 0x62e81432, 3 },
    { 0xd90142e, 3 },
    { 0xf63c1434, 3 },
    { 0xe0881435, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42c1439, 5 },
    { 0, 0 },
    { 0x7648143b, 3 },
    { 0x9db0143c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd01441, 4 },
    { 0, 0 },
    { 0x8ea81443, 3 },
    { 0xb2981443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x124c1447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5820144f, 3 },
    { 0, 0 },
    { 0x69bc1451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a81454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe01459, 3 },
    { 0, 0 },
    { 0xcb30145b, 3 },
    { 0, 0 },
    { 0x7bbc145d, 3 },
    { 0x43f0145d, 3 },
    { 0xeb68145d, 4 },
    { 0x13701460, 4 },
    { 0xe6841461, 3 },
    { 0, 0 },
    { 0x58101463, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0c1466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2c1469, 3 },
    { 0, 0 },
    { 0x1aac146b, 3 },
    { 0xdadc146c, 5 },
    { 0xc4d4146b, 3 },
    { 0x9b1c146e, 3 },
    { 0, 0 },
    { 0x9a101470, 3 },
    { 0x53b01471, 4 },
    { 0x28581471, 5 },
    { 0, 0 },
    { 0x58841474, 3 },
    { 0xcf441475, 3 },
    { 0, 0 },
    { 0x206c1477, 3 },
    { 0x434c1477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a4147e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1981484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3401487, 5 },
    { 0x8b01488, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a70148b, 3 },
    { 0xa29c148c, 3 },
    { 0xc4d4148b, 4 },
    { 0, 0 },
    { 0x8e6c148f, 3 },
    { 0xf64148f, 4 },
    { 0x74d81491, 4 },
    { 0x5a681491, 3 },
    { 0, 0 },
    { 0x84941494, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a81498, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7754149c, 3 },
    { 0x4df4149d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c14a0, 3 },
    { 0x3a9414a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898014a6, 3 },
    { 0x8fe014a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236814b3, 3 },
    { 0xb8c814b3, 4 },
    { 0, 0 },
    { 0x463414b6, 4 },
    { 0x46d814b7, 3 },
    { 0x178814b8, 3 },
    { 0x321c14b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd014be, 3 },
    { 0x6b4814bf, 3 },
    { 0x1adc14be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7814c8, 3 },
    { 0, 0 },
    { 0xf1c014ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2814cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01414d0, 3 },
    { 0x75c414d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d014d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b014d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x865014db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38c14df, 3 },
    { 0xce3814e0, 3 },
    { 0xf42414df, 3 },
    { 0, 0 },
    { 0xcb0014e3, 4 },
    { 0x86b014e4, 4 },
    { 0x884814e5, 3 },
    { 0, 0 },
    { 0x69c014e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988c14f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e414f9, 3 },
    { 0, 0 },
    { 0x456814fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac14ff, 3 },
    { 0, 0 },
    { 0x3ed81501, 3 },
    { 0x94f01502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77c150d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61841510, 4 },
    { 0xfe41511, 3 },
    { 0xa8301511, 3 },
    { 0x6a241513, 3 },
    { 0x6b301514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ac1519, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0c151e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d41521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e70152a, 3 },
    { 0xa1a0152a, 3 },
    { 0x5fac152b, 3 },
    { 0xda00152d, 5 },
    { 0xa204152e, 4 },
    { 0x1d00152d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc01534, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56601538, 3 },
    { 0x35d41539, 4 },
    { 0x23fc153a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ac153d, 3 },
    { 0x25ac153d, 3 },
    { 0, 0 },
    { 0x69d41540, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa460154b, 3 },
    { 0xbcf4154c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740154f, 4 },
    { 0x47b0154f, 3 },
    { 0x16dc1551, 4 },
    { 0xffb81552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c1557, 3 },
    { 0, 0 },
    { 0x35bc1559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7964155d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f141561, 3 },
    { 0x663c1561, 3 },
    { 0xa8281563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25341566, 3 },
    { 0x430c1566, 3 },
    { 0x7ac01567, 3 },
    { 0x27741569, 3 },
    { 0x19e81567, 3 },
    { 0xcf7c156b, 3 },
    { 0xb6d0156c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1c1573, 4 },
    { 0, 0 },
    { 0x10381575, 4 },
    { 0x92c01576, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2448157e, 3 },
    { 0x1560157f, 3 },
    { 0xf7c157e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af01583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f01587, 3 },
    { 0x5c081588, 3 },
    { 0x51081589, 3 },
    { 0x9de0158a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457c1590, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c1593, 3 },
    { 0xc43c1593, 3 },
    { 0x89341595, 4 },
    { 0x1a3c1596, 3 },
    { 0x9ed01597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d0159a, 4 },
    { 0xe688159b, 3 },
    { 0x62d4159c, 3 },
    { 0x3f9c159d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe815a3, 3 },
    { 0x4c7c15a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f415a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0415ab, 4 },
    { 0xa69c15ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e815b2, 3 },
    { 0, 0 },
    { 0xba4c15b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677415bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80c15c5, 3 },
    { 0x4ec15c6, 3 },
    { 0x420415c6, 3 },
    { 0x1dac15c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878c15cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426815d1, 3 },
    { 0, 0 },
    { 0xa26c15d3, 3 },
    { 0, 0 },
    { 0x511c15d5, 3 },
    { 0, 0 },
    { 0x803415d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674415db, 4 },
    { 0, 0 },
    { 0x2d6815dd, 4 },
    { 0x6bcc15dd, 3 },
    { 0, 0 },
    { 0x49c815e0, 4 },
    { 0x116c15e0, 3 },
    { 0xd9c015e2, 3 },
    { 0x969415e0, 3 },
    { 0, 0 },
    { 0xb9dc15e5, 3 },
    { 0x1c3c15e6, 3 },
    { 0, 0 },
    { 0xe2bc15e8, 4 },
    { 0x1f4015e9, 5 },
    { 0x3ba815e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e415ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7015f3, 4 },
    { 0x2d415f3, 3 },
    { 0xfe5815f5, 3 },
    { 0, 0 },
    { 0xd73415f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900015fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x376015ff, 3 },
    { 0x62001600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a201608, 3 },
    { 0x4f1c1609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b01614, 3 },
    { 0x4d941615, 4 },
    { 0x42001616, 3 },
    { 0, 0 },
    { 0x5cc81618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fc161b, 3 },
    { 0x4550161c, 3 },
    { 0x9e58161d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47581621, 3 },
    { 0xa3cc1622, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a0162a, 4 },
    { 0xeb10162b, 3 },
    { 0, 0 },
    { 0xe274162d, 3 },
    { 0, 0 },
    { 0xcefc162f, 4 },
    { 0, 0 },
    { 0x4f41631, 4 },
    { 0xf9901632, 3 },
    { 0x42201632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c81636, 3 },
    { 0xe2001637, 3 },
    { 0xd4d81636, 3 },
    { 0xf4541638, 3 },
    { 0xf364163a, 3 },
    { 0, 0 },
    { 0xde28163c, 3 },
    { 0, 0 },
    { 0x3af8163e, 3 },
    { 0, 0 },
    { 0xf1601640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6141643, 3 },
    { 0x865c1644, 3 },
    { 0, 0 },
    { 0xbd501646, 3 },
    { 0, 0 },
    { 0xd8541648, 5 },
    { 0, 0 },
    { 0x7a1c164a, 3 },
    { 0xfa44164b, 3 },
    { 0xd198164a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5c1652, 3 },
    { 0, 0 },
    { 0xb0c41654, 4 },
    { 0x9b181655, 3 },
    { 0xdae01655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40901659, 3 },
    { 0x77c8165a, 3 },
    { 0x3204165b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1440165e, 3 },
    { 0, 0 },
    { 0x57901660, 3 },
    { 0xc2a01661, 3 },
    { 0x22f01662, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c81671, 3 },
    { 0x5a901671, 3 },
    { 0xfd4c1672, 4 },
    { 0x260c1674, 3 },
    { 0x44501675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4167b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec1688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e70168f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c01692, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c81696, 3 },
    { 0, 0 },
    { 0x91b41698, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8c169b, 4 },
    { 0xd858169c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c16a0, 3 },
    { 0x416816a0, 3 },
    { 0x2cc816a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c016a5, 4 },
    { 0x352416a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126c16ad, 3 },
    { 0x109816ae, 4 },
    { 0, 0 },
    { 0x92ec16b0, 3 },
    { 0, 0 },
    { 0x4b8016b2, 3 },
    { 0x62ac16b3, 3 },
    { 0x120016b4, 3 },
    { 0xa9c16b5, 4 },
    { 0xc8e816b6, 3 },
    { 0xbef416b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282016bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6416c2, 3 },
    { 0x2b2816c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efc16c6, 3 },
    { 0xcc2416c7, 3 },
    { 0x864816c6, 3 },
    { 0x7c1016c9, 3 },
    { 0xef7416c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739c16d1, 3 },
    { 0xf5fc16d2, 4 },
    { 0xd66c16d3, 3 },
    { 0x517016d4, 3 },
    { 0x526c16d3, 4 },
    { 0x2d7c16d6, 3 },
    { 0xc43416d6, 4 },
    { 0x99d816d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52416e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce016e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x391016e6, 3 },
    { 0x85c416e7, 3 },
    { 0xd9bc16e8, 3 },
    { 0xc05416e8, 3 },
    { 0x563016e9, 4 },
    { 0xc99c16eb, 3 },
    { 0xf39c16eb, 3 },
    { 0x396c16eb, 3 },
    { 0xe67816ee, 4 },
    { 0xe25016ed, 3 },
    { 0x9b3416f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1416f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1416f9, 3 },
    { 0xccb816fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d01702, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a941705, 3 },
    { 0xf84c1706, 4 },
    { 0, 0 },
    { 0xa5b41708, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafc1712, 3 },
    { 0x3041713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45381718, 3 },
    { 0, 0 },
    { 0x6a38171a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77981723, 3 },
    { 0xe3d41724, 3 },
    { 0xca801724, 3 },
    { 0, 0 },
    { 0x84181727, 3 },
    { 0, 0 },
    { 0x84001729, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2981731, 3 },
    { 0xd2341732, 3 },
    { 0xae501733, 3 },
    { 0x347c1733, 4 },
    { 0xd4d41734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e681739, 3 },
    { 0xb3681739, 3 },
    { 0xc434173b, 4 },
    { 0x79dc173c, 3 },
    { 0x28e4173d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6101744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e741747, 3 },
    { 0xc39c1747, 3 },
    { 0x1b101749, 3 },
    { 0, 0 },
    { 0x314c174b, 4 },
    { 0x5f24174c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c81756, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4175a, 3 },
    { 0xad28175b, 3 },
    { 0xd964175b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c175f, 3 },
    { 0x4350175f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd41769, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2220176c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8c1774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c1779, 4 },
    { 0x23bc177a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df8177f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf41782, 4 },
    { 0x5a801783, 3 },
    { 0x7c7c1784, 4 },
    { 0x448c1783, 3 },
    { 0x3f7c1786, 3 },
    { 0xdec41786, 3 },
    { 0x8b541788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547c178e, 4 },
    { 0, 0 },
    { 0xcf641790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c81795, 3 },
    { 0xfb81795, 3 },
    { 0x54201795, 3 },
    { 0x4c7c1798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f4179c, 4 },
    { 0xbde4179d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235817a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5017a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820c17ae, 3 },
    { 0, 0 },
    { 0x3ca817b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ac17bb, 3 },
    { 0x77ec17bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c17c3, 3 },
    { 0x1c2817c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992817cf, 3 },
    { 0, 0 },
    { 0xbfcc17d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x922417d4, 3 },
    { 0x790817d5, 4 },
    { 0x9b1017d6, 3 },
    { 0xaea017d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40c17dc, 3 },
    { 0, 0 },
    { 0x340c17de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9417e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x667417e5, 4 },
    { 0x4da817e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1417ed, 4 },
    { 0xc36c17ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c17f2, 3 },
    { 0xc99417f3, 4 },
    { 0x858017f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802c1800, 3 },
    { 0x1d981800, 3 },
    { 0x86441802, 3 },
    { 0x2d641802, 4 },
    { 0xf83c1804, 3 },
    { 0x27981805, 3 },
    { 0xbaf41806, 3 },
    { 0, 0 },
    { 0xc0781808, 3 },
    { 0xb84c1809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6c180c, 3 },
    { 0x210c180d, 3 },
    { 0xd608180e, 3 },
    { 0x7b2c180d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83041813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd501817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f4181b, 3 },
    { 0, 0 },
    { 0xcd80181d, 4 },
    { 0x951c181e, 4 },
    { 0xa43c181e, 3 },
    { 0, 0 },
    { 0xe5941821, 3 },
    { 0x75c01821, 3 },
    { 0, 0 },
    { 0x81e81824, 3 },
    { 0xc2741825, 3 },
    { 0x20781826, 4 },
    { 0, 0 },
    { 0x1c841828, 3 },
    { 0x32401829, 4 },
    { 0, 0 },
    { 0xb80182b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3741832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3c1836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ac1839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac183f, 4 },
    { 0xbf681840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a01843, 3 },
    { 0x5f681843, 3 },
    { 0x2f581845, 3 },
    { 0, 0 },
    { 0x47581847, 3 },
    { 0xee781848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9c184c, 3 },
    { 0xc5b4184d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850c1851, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b641854, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41841857, 3 },
    { 0x2f6c1858, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0185b, 3 },
    { 0xca58185c, 3 },
    { 0x68a8185c, 3 },
    { 0x49f4185e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32c1861, 3 },
    { 0x570c1862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce41869, 3 },
    { 0x71e81869, 4 },
    { 0xb0641869, 4 },
    { 0x58e8186c, 3 },
    { 0xb8a81869, 3 },
    { 0x7fd4186c, 3 },
    { 0xfc01869, 3 },
    { 0xd15c1869, 4 },
    { 0x33b41871, 3 },
    { 0xf1181871, 5 },
    { 0x72f01873, 3 },
    { 0x779c1874, 3 },
    { 0x4aec1874, 3 },
    { 0, 0 },
    { 0xe2e01877, 3 },
    { 0xd8001878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481c187e, 4 },
    { 0, 0 },
    { 0x38981880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86281884, 3 },
    { 0x56641885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc81889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec188c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1c188f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c1896, 3 },
    { 0xe4781897, 3 },
    { 0, 0 },
    { 0xd5f41899, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc38189d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72418a1, 3 },
    { 0x9fa418a2, 4 },
    { 0x842018a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0818a9, 4 },
    { 0, 0 },
    { 0x28b418ab, 3 },
    { 0, 0 },
    { 0x355818ad, 3 },
    { 0x69c18ad, 3 },
    { 0x20ac18ad, 3 },
    { 0xb7bc18ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9418b3, 3 },
    { 0x77fc18b3, 3 },
    { 0, 0 },
    { 0x7fc018b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a818ba, 3 },
    { 0, 0 },
    { 0x4e8818bc, 3 },
    { 0x92c018bc, 3 },
    { 0xe0f018bc, 3 },
    { 0x3fa018bf, 3 },
    { 0x23ec18c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b418c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c18c6, 3 },
    { 0, 0 },
    { 0x348018c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440418cc, 3 },
    { 0x133c18cd, 3 },
    { 0, 0 },
    { 0xa27018cf, 3 },
    { 0xf1418d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6418d4, 3 },
    { 0xcec818d5, 3 },
    { 0x255c18d6, 3 },
    { 0xd0c818d5, 3 },
    { 0xa44018d8, 3 },
    { 0x73b418d8, 3 },
    { 0, 0 },
    { 0x88418db, 3 },
    { 0xb1f818dc, 3 },
    { 0x1af818dc, 3 },
    { 0x65418db, 3 },
    { 0x67a018dd, 4 },
    { 0xf1c818e0, 3 },
    { 0xbd1418e1, 3 },
    { 0xebb818e2, 4 },
    { 0x2cd418db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b418e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37818ed, 3 },
    { 0, 0 },
    { 0xf60018ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705c18f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a418fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62018ff, 3 },
    { 0, 0 },
    { 0xf5781901, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c01904, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c01908, 3 },
    { 0xe74c1909, 3 },
    { 0, 0 },
    { 0x73a0190b, 4 },
    { 0xc5b4190c, 3 },
    { 0x2284190d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb81911, 3 },
    { 0x5ec81912, 3 },
    { 0, 0 },
    { 0x34e81914, 3 },
    { 0xc8401915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cc1919, 3 },
    { 0x6c0191a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04191e, 3 },
    { 0xb43c191f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f81929, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9724192f, 3 },
    { 0x4481930, 3 },
    { 0xd1341931, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1c1936, 5 },
    { 0xd0701936, 3 },
    { 0, 0 },
    { 0x26501939, 3 },
    { 0, 0 },
    { 0x9a3c193b, 4 },
    { 0xd598193c, 3 },
    { 0xd09c193c, 3 },
    { 0x80bc193b, 5 },
    { 0x52b4193c, 3 },
    { 0xbc701940, 3 },
    { 0xf53c1941, 4 },
    { 0, 0 },
    { 0x3d841943, 3 },
    { 0, 0 },
    { 0x9b781945, 3 },
    { 0x82ec1945, 3 },
    { 0, 0 },
    { 0x9d541948, 3 },
    { 0, 0 },
    { 0xfa0c194a, 3 },
    { 0xc4b0194a, 3 },
    { 0xd928194a, 3 },
    { 0x416c194d, 3 },
    { 0xe80c194d, 4 },
    { 0xf90c194f, 3 },
    { 0x3690194d, 4 },
    { 0, 0 },
    { 0xd5e41952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf660195f, 3 },
    { 0x1914195f, 4 },
    { 0x56001960, 3 },
    { 0xaf9c1961, 4 },
    { 0xf2901961, 4 },
    { 0xe2941962, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d8196a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c196d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12441970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbc1973, 3 },
    { 0x36381974, 3 },
    { 0xd7b81975, 3 },
    { 0x79901976, 4 },
    { 0x23b41977, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba24197e, 3 },
    { 0x77e8197f, 4 },
    { 0, 0 },
    { 0x39f01981, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba741984, 3 },
    { 0, 0 },
    { 0x8ab01986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd8198b, 3 },
    { 0x958c198c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5464198f, 3 },
    { 0, 0 },
    { 0x1d081991, 3 },
    { 0, 0 },
    { 0x14241993, 4 },
    { 0x17401994, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e8199b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a019a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6019a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba819ab, 4 },
    { 0x5b3019ab, 3 },
    { 0x421419ad, 3 },
    { 0xc06c19ae, 3 },
    { 0x81b819af, 4 },
    { 0x2f0819ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4819b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e419bb, 4 },
    { 0x826819bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x878819bf, 4 },
    { 0xa9f419c0, 3 },
    { 0x298419bf, 3 },
    { 0x394819c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7819c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef419d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c19d5, 3 },
    { 0, 0 },
    { 0x43ec19d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f819dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8c19df, 4 },
    { 0x59a419e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf419e5, 4 },
    { 0, 0 },
    { 0x77b019e7, 3 },
    { 0xe26c19e8, 4 },
    { 0x25bc19e8, 3 },
    { 0x2b0c19ea, 3 },
    { 0xb94419e8, 3 },
    { 0x16b419e8, 3 },
    { 0x121819ed, 3 },
    { 0xa76019eb, 3 },
    { 0, 0 },
    { 0xe78819f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265019f4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x934019f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0819fc, 3 },
    { 0, 0 },
    { 0xf74819fe, 4 },
    { 0x559419fe, 3 },
    { 0x97cc1a00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb741a07, 4 },
    { 0x57541a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46c1a0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bc1a0f, 3 },
    { 0xf4c01a10, 3 },
    { 0xd78c1a11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb01a18, 5 },
    { 0xd5441a19, 5 },
    { 0xbfb01a1a, 3 },
    { 0x19241a18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cc1a1f, 3 },
    { 0x66901a20, 3 },
    { 0x32c1a21, 3 },
    { 0x17641a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d41a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c01a2d, 4 },
    { 0x8b281a2d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0c1a34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6241a39, 3 },
    { 0x73801a3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62981a4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bc1a4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a01a52, 3 },
    { 0x3aa41a52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3141a56, 3 },
    { 0xd9e01a56, 3 },
    { 0x4a641a58, 3 },
    { 0xb9b01a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92901a5f, 3 },
    { 0, 0 },
    { 0xd51c1a61, 3 },
    { 0x502c1a61, 3 },
    { 0x4ab81a61, 3 },
    { 0, 0 },
    { 0x709c1a65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c081a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dc1a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9c1a6f, 3 },
    { 0x52c01a70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60101a75, 3 },
    { 0x8efc1a76, 3 },
    { 0xcb901a77, 3 },
    { 0xce601a76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ec1a7b, 3 },
    { 0x2d881a7c, 3 },
    { 0x8c001a7b, 3 },
    { 0xcadc1a7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd481a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb01a86, 3 },
    { 0x70f81a87, 3 },
    { 0x8c181a88, 3 },
    { 0x5a41a86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b941a8d, 4 },
    { 0, 0 },
    { 0x61441a8f, 3 },
    { 0xa81a8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca41a94, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c01a99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac41a9d, 3 },
    { 0, 0 },
    { 0x56141a9f, 3 },
    { 0, 0 },
    { 0x15941aa1, 3 },
    { 0xdd6c1aa2, 4 },
    { 0x65781aa3, 3 },
    { 0, 0 },
    { 0x197c1aa5, 4 },
    { 0, 0 },
    { 0xe8381aa7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b41aaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb781aad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63501ab0, 3 },
    { 0, 0 },
    { 0x26dc1ab2, 4 },
    { 0x1341ab3, 3 },
    { 0x86981ab3, 5 },
    { 0, 0 },
    { 0x47881ab6, 3 },
    { 0x78c41ab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0c1abc, 3 },
    { 0, 0 },
    { 0x14801abe, 3 },
    { 0xf2a41abf, 3 },
    { 0xd5d41ac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb41ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9801ac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b01acc, 3 },
    { 0, 0 },
    { 0x923c1ace, 3 },
    { 0, 0 },
    { 0xbc741ad0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e41ad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9c1adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1801ae1, 3 },
    { 0x81c41ae2, 3 },
    { 0x61bc1ae3, 3 },
    { 0x11901ae4, 3 },
    { 0x15441ae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83141ae8, 3 },
    { 0, 0 },
    { 0xa0041aea, 3 },
    { 0, 0 },
    { 0x60341aec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23c1af2, 3 },
    { 0x4fb81af3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca41af8, 3 },
    { 0xc1501af8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc1aff, 4 },
    { 0, 0 },
    { 0x97741b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f81b07, 3 },
    { 0x278c1b08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f981b10, 3 },
    { 0xc2881b11, 4 },
    { 0x496c1b12, 3 },
    { 0x35901b13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87981b16, 3 },
    { 0, 0 },
    { 0x31bc1b18, 3 },
    { 0xcd01b19, 3 },
    { 0, 0 },
    { 0x6901b1b, 3 },
    { 0, 0 },
    { 0xc6e01b1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dc1b21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccc1b27, 3 },
    { 0, 0 },
    { 0x63281b29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d01b2c, 3 },
    { 0x65081b2d, 3 },
    { 0xcaac1b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07c1b38, 3 },
    { 0xb3b41b39, 4 },
    { 0, 0 },
    { 0x45b41b3b, 3 },
    { 0x81981b3c, 3 },
    { 0x18f41b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60801b40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc81b43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd41b4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841b52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc01b59, 3 },
    { 0, 0 },
    { 0x6f441b5b, 4 },
    { 0, 0 },
    { 0x3b901b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c01b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab01b6d, 3 },
    { 0, 0 },
    { 0xd2e41b6f, 4 },
    { 0, 0 },
    { 0x6ad41b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a81b75, 3 },
    { 0x67b41b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86381b79, 3 },
    { 0xde9c1b7a, 3 },
    { 0, 0 },
    { 0xc1981b7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e701b7f, 3 },
    { 0, 0 },
    { 0xc2ec1b81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e901b8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5641b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913c1b93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cc1b98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e81b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4901ba1, 3 },
    { 0, 0 },
    { 0x83f41ba3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8c1bab, 3 },
    { 0x42b01bac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f041baf, 3 },
    { 0, 0 },
    { 0x45301bb1, 3 },
    { 0x2b741bb2, 3 },
    { 0xd2bc1bb3, 3 },
    { 0x429c1bb4, 3 },
    { 0, 0 },
    { 0x5fe81bb6, 4 },
    { 0, 0 },
    { 0xcaa41bb8, 3 },
    { 0x3b6c1bb8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107c1bbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7301bc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c1bc3, 3 },
    { 0, 0 },
    { 0x9f441bc5, 3 },
    { 0x14a01bc6, 4 },
    { 0xd1881bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc1bcb, 3 },
    { 0x54c81bcc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e41bcf, 3 },
    { 0, 0 },
    { 0xe0f41bd1, 3 },
    { 0x20a41bd1, 4 },
    { 0x4eb01bd1, 4 },
    { 0x3f101bd4, 3 },
    { 0xffb81bd5, 3 },
    { 0x87ac1bd2, 3 },
    { 0, 0 },
    { 0x17701bd8, 3 },
    { 0x77f41bd8, 4 },
    { 0xbab81bda, 3 },
    { 0xb21c1bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4081bde, 3 },
    { 0x20b81bde, 4 },
    { 0xd81c1be0, 3 },
    { 0x3c2c1bdf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcc1be6, 3 },
    { 0x7a3c1be7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba01bea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578c1bf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15741bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde01c00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c1c03, 3 },
    { 0x36bc1c03, 4 },
    { 0xc9f41c05, 3 },
    { 0x5bec1c05, 3 },
    { 0xb1a81c07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c781c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03c1c20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53501c26, 3 },
    { 0, 0 },
    { 0xfb7c1c28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a41c2c, 5 },
    { 0, 0 },
    { 0xb7f01c2e, 3 },
    { 0x2c8c1c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2381c34, 4 },
    { 0x31981c35, 5 },
    { 0xa10c1c36, 3 },
    { 0x2e981c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f01c3b, 3 },
    { 0x31fc1c3b, 4 },
    { 0x78ac1c3b, 3 },
    { 0, 0 },
    { 0xacec1c3f, 3 },
    { 0, 0 },
    { 0xdc801c41, 4 },
    { 0, 0 },
    { 0xbd041c43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7c1c47, 3 },
    { 0x63e01c48, 3 },
    { 0xbe5c1c48, 3 },
    { 0x72e41c49, 3 },
    { 0xc7d41c4a, 3 },
    { 0x5d001c49, 3 },
    { 0xb73c1c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa01c50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4781c5b, 4 },
    { 0x8ad81c5c, 4 },
    { 0xb2241c5c, 4 },
    { 0, 0 },
    { 0xe8a41c5f, 4 },
    { 0x4a741c5f, 3 },
    { 0x4c481c61, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c1c64, 4 },
    { 0xdb301c65, 3 },
    { 0x19c41c65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9581c69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c1c73, 4 },
    { 0, 0 },
    { 0x2cdc1c75, 3 },
    { 0, 0 },
    { 0xac581c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c41c7e, 3 },
    { 0xaa881c7f, 3 },
    { 0x7fdc1c80, 3 },
    { 0xc781c81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2c1c84, 3 },
    { 0x1e781c85, 3 },
    { 0x3bc81c86, 3 },
    { 0x5ec41c86, 3 },
    { 0xb58c1c88, 4 },
    { 0x577c1c88, 4 },
    { 0xa8581c89, 3 },
    { 0x9c101c8b, 3 },
    { 0x5af01c8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62441c95, 3 },
    { 0x74bc1c95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd81c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f81c9f, 3 },
    { 0, 0 },
    { 0xe3e01ca1, 4 },
    { 0, 0 },
    { 0xe3641ca3, 3 },
    { 0x30801ca3, 3 },
    { 0xf7f41ca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbc1ca8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ec1caf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c01cb2, 3 },
    { 0x40c1cb3, 4 },
    { 0xc36c1cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb01cb7, 4 },
    { 0, 0 },
    { 0x8741cb9, 4 },
    { 0xc1581cba, 3 },
    { 0xf841cba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b41cbe, 3 },
    { 0x55bc1cbf, 4 },
    { 0, 0 },
    { 0xd841cc1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd381ccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae441ccf, 4 },
    { 0, 0 },
    { 0xba481cd1, 3 },
    { 0x66f81cd1, 3 },
    { 0xd53c1cd2, 3 },
    { 0x7bc01cd3, 4 },
    { 0xedd41cd5, 3 },
    { 0xe6381cd3, 3 },
    { 0xe1a81cd7, 3 },
    { 0xeb981cd8, 4 },
    { 0, 0 },
    { 0x2341cda, 4 },
    { 0xe0401cdb, 3 },
    { 0x6141cdc, 4 },
    { 0x48c1cdd, 4 },
    { 0xb4281cdc, 5 },
    { 0x9be01cdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c1ce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8241ce7, 3 },
    { 0x9a8c1ce8, 4 },
    { 0x31d81ce9, 3 },
    { 0xd8401cea, 3 },
    { 0x45c81ceb, 3 },
    { 0x90741ceb, 3 },
    { 0, 0 },
    { 0x64481cee, 3 },
    { 0x91081cee, 3 },
    { 0x7a2c1cee, 3 },
    { 0xc1601cf1, 3 },
    { 0xf801cf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4c1cf5, 3 },
    { 0xcd701cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a01cf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc1cfc, 3 },
    { 0xcb081cfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53341d08, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f441d0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e01d10, 3 },
    { 0x134c1d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b01d18, 3 },
    { 0, 0 },
    { 0x90e41d1a, 3 },
    { 0x452c1d1b, 3 },
    { 0xd3081d1c, 3 },
    { 0x1f0c1d1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43601d2a, 3 },
    { 0x52601d2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36381d2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dc1d31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc801d3f, 3 },
    { 0x1cbc1d40, 3 },
    { 0xba981d40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35001d44, 3 },
    { 0xf4401d45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7701d49, 3 },
    { 0, 0 },
    { 0xf3141d4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22581d51, 3 },
    { 0xfcbc1d51, 3 },
    { 0xba7c1d53, 3 },
    { 0x11081d54, 3 },
    { 0xacac1d55, 4 },
    { 0x1f041d56, 3 },
    { 0x18c41d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b81d69, 3 },
    { 0xd43c1d6a, 3 },
    { 0xac0c1d6b, 3 },
    { 0, 0 },
    { 0x5fc81d6d, 4 },
    { 0xd3841d6e, 3 },
    { 0xcce41d6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d81d72, 3 },
    { 0x1ba01d73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de01d7a, 3 },
    { 0xdb981d7b, 3 },
    { 0x561c1d7c, 3 },
    { 0, 0 },
    { 0x76541d7e, 3 },
    { 0, 0 },
    { 0x98a81d80, 4 },
    { 0xae781d81, 3 },
    { 0x36981d80, 3 },
    { 0, 0 },
    { 0xf5001d84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c1d87, 4 },
    { 0xbf241d88, 3 },
    { 0xb49c1d89, 3 },
    { 0xc5501d87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45c1d8d, 4 },
    { 0x69bc1d8e, 3 },
    { 0x3cd01d8f, 3 },
    { 0xfd3c1d8e, 3 },
    { 0xc6901d91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c1d94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x76801d97, 3 },
    { 0x13941d98, 3 },
    { 0, 0 },
    { 0xd6941d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc1d9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d541da0, 3 },
    { 0, 0 },
    { 0x27e81da2, 3 },
    { 0x30b81da2, 3 },
    { 0x695c1da4, 3 },
    { 0x3c601da5, 3 },
    { 0x62b01da5, 3 },
    { 0, 0 },
    { 0x88241da8, 4 },
    { 0, 0 },
    { 0x400c1daa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c41daf, 3 },
    { 0x6e01db0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c01dbc, 3 },
    { 0xe4381dbd, 3 },
    { 0xeeec1dbc, 3 },
    { 0x84241dbf, 3 },
    { 0x3f5c1dc0, 4 },
    { 0, 0 },
    { 0x91441dc2, 4 },
    { 0x37241dc2, 4 },
    { 0x42c81dc4, 4 },
    { 0x6edc1dc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf41dcf, 3 },
    { 0x90781dcf, 4 },
    { 0x60341dd1, 3 },
    { 0xba3c1dd1, 3 },
    { 0x11401dd3, 3 },
    { 0x94fc1dd3, 4 },
    { 0xf5e81dd5, 4 },
    { 0xeab01dd2, 3 },
    { 0x70a01dd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd381dda, 3 },
    { 0, 0 },
    { 0xc07c1ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc801de0, 3 },
    { 0xbb01de0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7c1de4, 3 },
    { 0x59fc1de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f001dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7c1def, 3 },
    { 0x940c1def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee81df8, 3 },
    { 0, 0 },
    { 0xb5241dfa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1301dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7041e00, 3 },
    { 0x724c1e01, 3 },
    { 0x5e641e02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d401e06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0181e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3c1e0f, 3 },
    { 0x81941e10, 3 },
    { 0x35741e0f, 3 },
    { 0x75241e12, 3 },
    { 0, 0 },
    { 0x57801e14, 3 },
    { 0x52f01e15, 5 },
    { 0x90701e16, 3 },
    { 0, 0 },
    { 0x98d81e18, 3 },
    { 0xaac01e19, 3 },
    { 0, 0 },
    { 0x32c1e1b, 3 },
    { 0x2ee41e1c, 3 },
    { 0xc81e1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd81e21, 3 },
    { 0, 0 },
    { 0xb3dc1e23, 3 },
    { 0xeeb01e23, 4 },
    { 0, 0 },
    { 0x99501e26, 3 },
    { 0xb1201e26, 3 },
    { 0, 0 },
    { 0x39841e29, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1741e2c, 4 },
    { 0xa6401e2d, 3 },
    { 0x26041e2e, 3 },
    { 0, 0 },
    { 0x800c1e30, 3 },
    { 0, 0 },
    { 0x66501e32, 3 },
    { 0xdbdc1e33, 3 },
    { 0x2141e34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58c1e3a, 3 },
    { 0xc0d01e3b, 4 },
    { 0xed181e3c, 3 },
    { 0x532c1e3a, 3 },
    { 0x731c1e3e, 3 },
    { 0xfe4c1e3f, 4 },
    { 0xe7f01e3b, 3 },
    { 0, 0 },
    { 0x29c81e42, 4 },
    { 0, 0 },
    { 0xe7801e44, 5 },
    { 0, 0 },
    { 0x1ce01e46, 3 },
    { 0x30e01e47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e81e4e, 3 },
    { 0x561c1e4f, 3 },
    { 0, 0 },
    { 0x9cdc1e51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d681e56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1041e5b, 3 },
    { 0, 0 },
    { 0x53781e5d, 4 },
    { 0xf7341e5d, 3 },
    { 0xb54c1e5d, 3 },
    { 0x33e41e60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95701e65, 3 },
    { 0, 0 },
    { 0x3a041e67, 4 },
    { 0x65fc1e68, 4 },
    { 0x46581e68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197c1e6f, 3 },
    { 0x8ac41e70, 3 },
    { 0, 0 },
    { 0xa8181e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee81e78, 3 },
    { 0, 0 },
    { 0x4ea41e7a, 3 },
    { 0x16b81e7b, 3 },
    { 0x4ad81e7b, 3 },
    { 0xc3341e7b, 4 },
    { 0x202c1e7c, 3 },
    { 0x36841e7d, 3 },
    { 0, 0 },
    { 0x2f8c1e81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a281e84, 3 },
    { 0xe96c1e84, 3 },
    { 0xf4341e86, 3 },
    { 0, 0 },
    { 0xe6641e88, 3 },
    { 0x85cc1e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b081e8e, 3 },
    { 0x28941e8f, 5 },
    { 0x47481e8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x49341e93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20641e9a, 3 },
    { 0, 0 },
    { 0x51f41e9c, 3 },
    { 0xe77c1e9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e341ea1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88541eaa, 4 },
    { 0x33bc1eaa, 5 },
    { 0xf8ec1eac, 3 },
    { 0xf881ead, 3 },
    { 0x28981eae, 3 },
    { 0xeee41eab, 3 },
    { 0xed81eb0, 3 },
    { 0x26d01eb1, 4 },
    { 0xc4841eb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a01eb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab41eba, 4 },
    { 0xbfec1ebb, 3 },
    { 0xcecc1ebc, 3 },
    { 0xe1341eba, 4 },
    { 0, 0 },
    { 0xf21c1ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c41ec5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac1ed0, 3 },
    { 0, 0 },
    { 0xc01c1ed2, 4 },
    { 0xee101ed2, 3 },
    { 0xa5b01ed3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a01ed8, 3 },
    { 0xd4341ed9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e01ee0, 3 },
    { 0xd5b81ee0, 3 },
    { 0xe8441ee2, 3 },
    { 0xbb301ee3, 3 },
    { 0xc42c1ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c1eea, 5 },
    { 0xe3001eeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9101eee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1701ef2, 3 },
    { 0xa7141ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13541ef7, 3 },
    { 0x1a181ef7, 5 },
    { 0x9b2c1ef9, 4 },
    { 0x71141ef8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f81efd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ac1f00, 3 },
    { 0xece81f00, 3 },
    { 0, 0 },
    { 0xbf801f03, 4 },
    { 0, 0 },
    { 0xa8741f05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b81f0a, 4 },
    { 0, 0 },
    { 0xcf801f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b41f10, 3 },
    { 0xae101f11, 3 },
    { 0, 0 },
    { 0x2c3c1f13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69681f19, 3 },
    { 0xdb81f19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c1f1d, 3 },
    { 0xf2201f1e, 3 },
    { 0, 0 },
    { 0x82d01f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606c1f2b, 3 },
    { 0, 0 },
    { 0x5c781f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e81f30, 3 },
    { 0xc0601f30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf701f34, 4 },
    { 0xb7c81f35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e041f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd501f3d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d81f40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efc1f44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541f4b, 3 },
    { 0xd4581f4c, 4 },
    { 0xe1f81f4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d41f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1501f55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9581f59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93501f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3641f61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff141f67, 4 },
    { 0, 0 },
    { 0x8401f69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x633c1f6c, 3 },
    { 0x83001f6d, 3 },
    { 0x81e41f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57701f73, 3 },
    { 0, 0 },
    { 0x45701f75, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81141f78, 3 },
    { 0xc5c01f78, 3 },
    { 0x61bc1f7a, 3 },
    { 0xe46c1f7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccc1f7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2501f83, 3 },
    { 0xfcd81f84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b41f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64841f8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965c1f8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e601f92, 3 },
    { 0x8d941f92, 3 },
    { 0, 0 },
    { 0xb6d01f95, 3 },
    { 0, 0 },
    { 0x501c1f97, 4 },
    { 0x87f81f98, 3 },
    { 0xa4901f99, 3 },
    { 0xc9801f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f401f9f, 3 },
    { 0, 0 },
    { 0xa8b41fa1, 5 },
    { 0x8581fa2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99201fa5, 3 },
    { 0, 0 },
    { 0xebd81fa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x868c1faa, 3 },
    { 0x2741fab, 4 },
    { 0x7bcc1fab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7481faf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fc1fb3, 5 },
    { 0, 0 },
    { 0xbcf41fb5, 5 },
    { 0x70bc1fb5, 3 },
    { 0, 0 },
    { 0x9001fb8, 3 },
    { 0xd5241fb9, 3 },
    { 0xc71c1fb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63901fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c1fc1, 3 },
    { 0xe3c41fc2, 4 },
    { 0x1e141fc3, 3 },
    { 0xa2d81fc2, 3 },
    { 0x38001fc5, 3 },
    { 0, 0 },
    { 0x4b681fc7, 4 },
    { 0x972c1fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b81fcb, 3 },
    { 0xc6201fcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e541fd2, 3 },
    { 0x7a481fd3, 3 },
    { 0xf99c1fd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e781fd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d01fe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b41fe5, 3 },
    { 0x17601fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c901feb, 3 },
    { 0x37601fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa681ff3, 4 },
    { 0xd4801ff4, 3 },
    { 0, 0 },
    { 0xf0dc1ff6, 3 },
    { 0xaccc1ff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc541ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba41ffe, 4 },
    { 0xa2841ffe, 3 },
    { 0x81c02000, 3 },
    { 0x47c02000, 3 },
    { 0x391c2002, 3 },
    { 0x4b042002, 3 },
    { 0x685c2004, 3 },
    { 0x10ac2005, 3 },
    { 0x5dd42003, 4 },
    { 0x91982005, 3 },
    { 0x46d02007, 3 },
    { 0x61d82002, 3 },
    { 0xf6d82004, 3 },
    { 0, 0 },
    { 0x77c0200c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc2011, 3 },
    { 0x1ffc2012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a242017, 4 },
    { 0, 0 },
    { 0xfba02019, 3 },
    { 0x99f4201a, 3 },
    { 0xd928201b, 3 },
    { 0, 0 },
    { 0xacdc201d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8402020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb4202c, 3 },
    { 0x1054202d, 3 },
    { 0xb18c202c, 3 },
    { 0, 0 },
    { 0xd85c2030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ac2034, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc203d, 4 },
    { 0x4a24203e, 3 },
    { 0xe79c203e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4c2043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c02047, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f8204a, 3 },
    { 0, 0 },
    { 0x8ee0204c, 3 },
    { 0, 0 },
    { 0xfc204e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c2053, 4 },
    { 0xa8c42054, 3 },
    { 0x581c2055, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39942058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404205b, 3 },
    { 0xfa60205b, 3 },
    { 0xf65c205d, 4 },
    { 0xa044205d, 5 },
    { 0, 0 },
    { 0xeee82060, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1c2064, 3 },
    { 0xf342065, 3 },
    { 0, 0 },
    { 0x77102067, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84c206c, 3 },
    { 0xebf4206d, 3 },
    { 0x4100206d, 4 },
    { 0, 0 },
    { 0x23b82070, 3 },
    { 0x24b02071, 3 },
    { 0xec442071, 3 },
    { 0xde102072, 4 },
    { 0x4be82074, 3 },
    { 0xacd82075, 3 },
    { 0x99742076, 3 },
    { 0xdf182070, 3 },
    { 0xc582076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b04207d, 4 },
    { 0, 0 },
    { 0x853c207f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a42085, 4 },
    { 0, 0 },
    { 0x59d42087, 3 },
    { 0x4be82087, 4 },
    { 0, 0 },
    { 0x31d8208a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27782090, 3 },
    { 0xa102091, 4 },
    { 0x63a02091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec2095, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588020a5, 3 },
    { 0xc72c20a6, 3 },
    { 0, 0 },
    { 0x29d420a8, 3 },
    { 0xe3820a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9c20ad, 3 },
    { 0, 0 },
    { 0xf9dc20af, 3 },
    { 0x834820b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd020b3, 3 },
    { 0x6e4020b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679820ba, 3 },
    { 0x5d2c20bb, 3 },
    { 0x630c20bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185c20c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92020c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb820ca, 3 },
    { 0xb43820cb, 3 },
    { 0, 0 },
    { 0x481820cd, 3 },
    { 0x341c20ce, 4 },
    { 0, 0 },
    { 0xaff020d0, 5 },
    { 0x968820d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442c20d5, 4 },
    { 0x1f5420d5, 4 },
    { 0xfc7c20d7, 3 },
    { 0xcadc20d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ac20dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50020e0, 3 },
    { 0, 0 },
    { 0x963c20e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557020e8, 3 },
    { 0x9a3420e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4820ee, 3 },
    { 0xf05820ef, 4 },
    { 0xe89420ef, 4 },
    { 0xe8ac20f0, 3 },
    { 0x31a020f2, 4 },
    { 0xaed420f3, 3 },
    { 0x551420f4, 3 },
    { 0xb17c20f4, 3 },
    { 0xd84c20f6, 3 },
    { 0x54d420f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04420fa, 3 },
    { 0xef4020fb, 3 },
    { 0xd46820fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94702106, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e0210a, 3 },
    { 0x7cc210b, 3 },
    { 0x65fc210c, 3 },
    { 0x4c8c210d, 3 },
    { 0xa578210e, 4 },
    { 0x2538210c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3642114, 3 },
    { 0xc2982115, 3 },
    { 0, 0 },
    { 0xf042117, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7760211b, 3 },
    { 0xc690211c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc650211f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1642123, 3 },
    { 0x4cb02123, 3 },
    { 0xb9342124, 3 },
    { 0xaea42126, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba4212b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adc2130, 4 },
    { 0, 0 },
    { 0xc0942132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed402136, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c8213b, 4 },
    { 0x7a18213c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20482140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a82143, 4 },
    { 0x51e42144, 3 },
    { 0xa7782145, 3 },
    { 0, 0 },
    { 0x2b802147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6738214c, 4 },
    { 0xed28214d, 3 },
    { 0, 0 },
    { 0xf938214f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c882152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x835c2155, 4 },
    { 0, 0 },
    { 0xae802157, 3 },
    { 0x9f682157, 4 },
    { 0x81542159, 3 },
    { 0x6798215a, 3 },
    { 0x6580215b, 3 },
    { 0xb3a0215a, 4 },
    { 0x98c215a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03c2160, 4 },
    { 0x7f042161, 4 },
    { 0x1a6c2162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb542165, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4216a, 4 },
    { 0, 0 },
    { 0x7a50216c, 3 },
    { 0xeb1c216d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94c2175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb8217c, 3 },
    { 0x8e30217d, 4 },
    { 0xa188217c, 3 },
    { 0, 0 },
    { 0xabdc2180, 3 },
    { 0, 0 },
    { 0x4a502182, 3 },
    { 0xf0902183, 3 },
    { 0xb17c2184, 3 },
    { 0xb71c2185, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd402190, 3 },
    { 0x4fec2191, 3 },
    { 0x4acc2192, 4 },
    { 0xabac2193, 3 },
    { 0, 0 },
    { 0xc4002195, 3 },
    { 0x84782195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be0219c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x110219f, 4 },
    { 0x5ee0219f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb821a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12421b2, 3 },
    { 0x81f421b3, 3 },
    { 0xde6821b3, 4 },
    { 0xeb7c21b5, 4 },
    { 0xb33021b6, 3 },
    { 0, 0 },
    { 0xe6ec21b8, 4 },
    { 0x219c21b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8021bf, 3 },
    { 0xf66821c0, 3 },
    { 0x8b1021c1, 3 },
    { 0x715021c2, 3 },
    { 0x677c21c1, 4 },
    { 0xe9bc21c2, 4 },
    { 0, 0 },
    { 0x91a821c6, 3 },
    { 0x74d821c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946c21cc, 3 },
    { 0xcb0421cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1821d2, 3 },
    { 0, 0 },
    { 0xe82421d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30c21da, 3 },
    { 0, 0 },
    { 0x407821dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac21e1, 3 },
    { 0xad5c21e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae421e5, 3 },
    { 0xa3cc21e5, 5 },
    { 0x82dc21e5, 4 },
    { 0xaed421e7, 3 },
    { 0x44a021e5, 4 },
    { 0x379821ea, 4 },
    { 0x35cc21ea, 3 },
    { 0x2cc21ec, 4 },
    { 0x4b8c21ec, 3 },
    { 0x1c5c21e7, 3 },
    { 0xda0821ef, 3 },
    { 0xf2ec21ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4421f5, 3 },
    { 0, 0 },
    { 0x538c21f7, 3 },
    { 0x17c021f7, 3 },
    { 0xd88021f9, 3 },
    { 0xb46c21fa, 3 },
    { 0xfa8c21f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e42200, 3 },
    { 0, 0 },
    { 0xa55c2202, 3 },
    { 0x35982203, 3 },
    { 0, 0 },
    { 0xf7602205, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82208, 4 },
    { 0x4ef42209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c8220c, 4 },
    { 0xe07c220d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c02210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418c2215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc2218, 3 },
    { 0, 0 },
    { 0x9ccc221a, 3 },
    { 0x118c221a, 3 },
    { 0x1b14221c, 3 },
    { 0xce70221d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e742223, 3 },
    { 0x97f82223, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e42227, 3 },
    { 0, 0 },
    { 0x4d7c2229, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4002230, 4 },
    { 0xfd242231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fc2234, 3 },
    { 0x6f382235, 3 },
    { 0x3cf42234, 3 },
    { 0xd1ac2235, 3 },
    { 0x40b02238, 3 },
    { 0xa02237, 3 },
    { 0xe168223a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f0223f, 4 },
    { 0xa942240, 3 },
    { 0, 0 },
    { 0x17582242, 3 },
    { 0x4cc42243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a4224b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf98224e, 3 },
    { 0x963c224e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef82255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2482258, 5 },
    { 0, 0 },
    { 0x9994225a, 3 },
    { 0x9400225b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35042262, 3 },
    { 0, 0 },
    { 0x33dc2264, 4 },
    { 0xb0202264, 3 },
    { 0, 0 },
    { 0xac882267, 3 },
    { 0x5a602267, 3 },
    { 0x37002269, 3 },
    { 0xb8ac2267, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c08226d, 3 },
    { 0, 0 },
    { 0x272c226f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31342272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e8227a, 4 },
    { 0xecc0227b, 3 },
    { 0x9c10227c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf882286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4702289, 4 },
    { 0, 0 },
    { 0x124228b, 3 },
    { 0x9240228b, 4 },
    { 0x2e44228d, 3 },
    { 0xd11c228d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc682291, 3 },
    { 0xf1182292, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c2296, 3 },
    { 0xa0c82297, 3 },
    { 0, 0 },
    { 0x2c742299, 4 },
    { 0x9b28229a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e422a0, 3 },
    { 0, 0 },
    { 0x14c822a2, 4 },
    { 0xebc422a3, 4 },
    { 0xe9a422a3, 3 },
    { 0, 0 },
    { 0x3bc822a6, 3 },
    { 0x687022a6, 3 },
    { 0, 0 },
    { 0x44dc22a9, 3 },
    { 0, 0 },
    { 0x677422ab, 3 },
    { 0xda2022ac, 3 },
    { 0, 0 },
    { 0xc3c22ae, 3 },
    { 0, 0 },
    { 0x2c4022b0, 3 },
    { 0x874822b1, 3 },
    { 0, 0 },
    { 0xe15422b3, 3 },
    { 0x680022b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b022b7, 4 },
    { 0, 0 },
    { 0x1e9822b9, 3 },
    { 0xa92822ba, 3 },
    { 0x624022bb, 3 },
    { 0xe42822b9, 4 },
    { 0x49022bd, 3 },
    { 0x742c22ba, 3 },
    { 0x41b422bf, 3 },
    { 0x153822c0, 3 },
    { 0xa96c22bf, 3 },
    { 0, 0 },
    { 0xac9c22c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39822c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30822cc, 3 },
    { 0x30bc22cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b022d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62422d4, 3 },
    { 0xd1c22d5, 3 },
    { 0x6db422d4, 3 },
    { 0xc2c422d7, 3 },
    { 0xa4c22d8, 3 },
    { 0x6b8c22d9, 3 },
    { 0xcfc22da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b422de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b822e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22e4, 3 },
    { 0x4fc422e5, 3 },
    { 0x4a9022e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ec22ea, 3 },
    { 0, 0 },
    { 0xfd3c22ec, 3 },
    { 0x3bfc22ec, 3 },
    { 0x862022ee, 3 },
    { 0xa64422ec, 3 },
    { 0xd9422ed, 3 },
    { 0, 0 },
    { 0x591822f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444022f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974c2304, 4 },
    { 0x66802304, 3 },
    { 0xc7182304, 3 },
    { 0x9b782307, 3 },
    { 0x2df02308, 4 },
    { 0xc7602309, 3 },
    { 0x63082305, 3 },
    { 0xd280230b, 3 },
    { 0xeee8230c, 3 },
    { 0x730230b, 4 },
    { 0x70230e, 3 },
    { 0x3af8230e, 4 },
    { 0xf51c230f, 3 },
    { 0, 0 },
    { 0xa1542312, 4 },
    { 0xf51c2313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c2317, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4994231a, 4 },
    { 0x4ae4231b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978c2320, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28202327, 3 },
    { 0x8b3c2327, 3 },
    { 0, 0 },
    { 0x6a84232a, 3 },
    { 0xe874232b, 3 },
    { 0x44d8232b, 3 },
    { 0x852c232d, 5 },
    { 0xac80232e, 3 },
    { 0xae70232f, 3 },
    { 0x4f04232a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa02333, 4 },
    { 0xc7d82334, 3 },
    { 0, 0 },
    { 0x7d542336, 3 },
    { 0xe5682337, 3 },
    { 0x79f82338, 3 },
    { 0, 0 },
    { 0x54c4233a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c4233d, 3 },
    { 0x1bf4233e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5c2343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa02349, 3 },
    { 0xe5ac234a, 3 },
    { 0, 0 },
    { 0x1a7c234c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd8234f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d102354, 3 },
    { 0xce682354, 3 },
    { 0x9d942356, 4 },
    { 0xad202357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173c235d, 3 },
    { 0, 0 },
    { 0x99dc235f, 4 },
    { 0x7510235f, 3 },
    { 0x1e3c235f, 3 },
    { 0x715c2362, 4 },
    { 0x6dfc2363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77882366, 3 },
    { 0xe6c82366, 3 },
    { 0x2c82367, 3 },
    { 0x36bc2367, 3 },
    { 0, 0 },
    { 0x3b0236b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2242374, 3 },
    { 0, 0 },
    { 0xbd0c2376, 3 },
    { 0xe0ac2377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c237b, 5 },
    { 0xedd4237c, 3 },
    { 0xad70237b, 4 },
    { 0xb5e4237b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd502381, 3 },
    { 0x76f42381, 3 },
    { 0xed142381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ec2387, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd98238a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353c2391, 4 },
    { 0x94e82392, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac682399, 3 },
    { 0x5c182399, 4 },
    { 0xb8bc239b, 3 },
    { 0x108c239a, 3 },
    { 0xa41c239d, 4 },
    { 0x465c239e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920023a2, 4 },
    { 0x4ff423a2, 3 },
    { 0xe00423a4, 3 },
    { 0x748423a5, 3 },
    { 0, 0 },
    { 0xda6823a7, 3 },
    { 0x15dc23a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5423ac, 3 },
    { 0x2d9423ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfc23b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be823b8, 3 },
    { 0, 0 },
    { 0x677823ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee823bd, 4 },
    { 0, 0 },
    { 0xbe7423bf, 3 },
    { 0, 0 },
    { 0x4f4c23c1, 3 },
    { 0, 0 },
    { 0xefe423c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4023c6, 3 },
    { 0x766423c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab023ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf423d2, 4 },
    { 0xa3a423d3, 4 },
    { 0x5b0823d3, 4 },
    { 0, 0 },
    { 0xee7423d6, 3 },
    { 0xe8e423d6, 3 },
    { 0, 0 },
    { 0x2ad023d9, 4 },
    { 0xf3b023da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db423e6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf423e9, 3 },
    { 0, 0 },
    { 0xd18023eb, 3 },
    { 0, 0 },
    { 0xf09423ed, 4 },
    { 0x40e023ee, 4 },
    { 0x9f5423ed, 4 },
    { 0xa48023f0, 3 },
    { 0xbfa023f1, 3 },
    { 0, 0 },
    { 0x21823f3, 3 },
    { 0x8b7823f3, 3 },
    { 0xc5d423f5, 3 },
    { 0x1ef423f6, 3 },
    { 0x8e8823f7, 4 },
    { 0, 0 },
    { 0x357023f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65582405, 3 },
    { 0, 0 },
    { 0xb9082407, 4 },
    { 0x24442408, 4 },
    { 0x81502409, 3 },
    { 0xd71c240a, 3 },
    { 0x67702409, 4 },
    { 0xe4d8240a, 3 },
    { 0xff582407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86642414, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bc2417, 3 },
    { 0, 0 },
    { 0xf9882419, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae502423, 3 },
    { 0xde542424, 4 },
    { 0, 0 },
    { 0x76d82426, 3 },
    { 0xe8b82427, 4 },
    { 0x98a82427, 3 },
    { 0, 0 },
    { 0xedd4242a, 3 },
    { 0x3d5c242a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8c242e, 4 },
    { 0, 0 },
    { 0xac9c2430, 5 },
    { 0xff1c2431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd82439, 3 },
    { 0x7eb4243a, 3 },
    { 0xcb18243b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc2442, 4 },
    { 0x96042443, 3 },
    { 0x18982444, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7680244a, 4 },
    { 0x60ac244b, 3 },
    { 0x70d8244c, 4 },
    { 0x8b24244c, 3 },
    { 0xb254244d, 4 },
    { 0x4530244f, 3 },
    { 0xdf8c244e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c2453, 3 },
    { 0xf2f82454, 3 },
    { 0xef042454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce82458, 4 },
    { 0xe00c2459, 3 },
    { 0x8f6c2459, 4 },
    { 0x6d7c2459, 3 },
    { 0x3444245b, 3 },
    { 0, 0 },
    { 0x3fa0245e, 3 },
    { 0xa5ac245e, 3 },
    { 0xf7e02460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75582463, 3 },
    { 0, 0 },
    { 0xa8102465, 3 },
    { 0x56942466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8246d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe82470, 3 },
    { 0x1ea02471, 3 },
    { 0xc6882470, 3 },
    { 0xee382470, 3 },
    { 0x69542472, 3 },
    { 0xf44c2475, 4 },
    { 0xce82475, 3 },
    { 0, 0 },
    { 0xd2a42478, 3 },
    { 0xf2c42479, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c04247d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385c2484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd8248c, 4 },
    { 0xbd40248d, 3 },
    { 0x8238248e, 3 },
    { 0x6d00248f, 3 },
    { 0xc880248c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b402494, 3 },
    { 0x73902495, 3 },
    { 0xebac2496, 3 },
    { 0, 0 },
    { 0x4a042498, 3 },
    { 0, 0 },
    { 0x954249a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa18249d, 3 },
    { 0x7588249e, 3 },
    { 0, 0 },
    { 0x7cb024a0, 3 },
    { 0, 0 },
    { 0x825024a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c824a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac24a9, 4 },
    { 0x803024aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50c24b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca424b3, 3 },
    { 0, 0 },
    { 0x77b024b5, 3 },
    { 0, 0 },
    { 0xb1c24b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b024bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba424bf, 3 },
    { 0, 0 },
    { 0xfe6c24c1, 3 },
    { 0, 0 },
    { 0x21f024c3, 3 },
    { 0, 0 },
    { 0x7dd024c5, 3 },
    { 0x8bb824c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x391824c9, 3 },
    { 0, 0 },
    { 0x97e824cb, 3 },
    { 0x562424cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc24d0, 4 },
    { 0x740024d1, 3 },
    { 0x1b3824d1, 4 },
    { 0x647424d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6424d8, 4 },
    { 0xd86024d9, 4 },
    { 0x824424d8, 5 },
    { 0x16f024d8, 3 },
    { 0x26cc24da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab824e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be024e4, 4 },
    { 0, 0 },
    { 0x838c24e6, 4 },
    { 0x1ff424e6, 3 },
    { 0x7cd824e8, 5 },
    { 0, 0 },
    { 0xa49024ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x525424ed, 4 },
    { 0, 0 },
    { 0x8e9024ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e424f2, 3 },
    { 0x4fd424f2, 3 },
    { 0, 0 },
    { 0x2c0024f5, 4 },
    { 0, 0 },
    { 0x3e8424f7, 3 },
    { 0xa1c824f8, 3 },
    { 0xb2024f9, 3 },
    { 0x54ac24fa, 3 },
    { 0xbd9c24fa, 3 },
    { 0x2c3424fc, 3 },
    { 0xd83824fa, 3 },
    { 0, 0 },
    { 0x9afc24ff, 4 },
    { 0, 0 },
    { 0xb59c2501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28250d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41c2510, 3 },
    { 0xb6842511, 3 },
    { 0x214c2512, 4 },
    { 0x938c2513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bc251a, 3 },
    { 0xbf68251b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dac251f, 3 },
    { 0x22a0251f, 3 },
    { 0xdb782521, 4 },
    { 0xbb102522, 4 },
    { 0, 0 },
    { 0x56602524, 3 },
    { 0x3f142525, 3 },
    { 0x95582524, 3 },
    { 0xe0e42524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0252b, 5 },
    { 0xc8c8252c, 3 },
    { 0, 0 },
    { 0x509c252e, 3 },
    { 0xe724252f, 3 },
    { 0x5aa02530, 3 },
    { 0x78942530, 3 },
    { 0x510252e, 4 },
    { 0, 0 },
    { 0xe9342534, 3 },
    { 0x5e842535, 3 },
    { 0x102536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e382541, 3 },
    { 0, 0 },
    { 0x1cd82543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e02548, 3 },
    { 0, 0 },
    { 0x8524254a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x143c254d, 3 },
    { 0x1144254d, 4 },
    { 0xa84c254d, 3 },
    { 0x7114254d, 3 },
    { 0, 0 },
    { 0x5c582552, 3 },
    { 0xc3842552, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33902558, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7730255d, 3 },
    { 0x1260255d, 3 },
    { 0xe3c0255f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c02562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81982567, 3 },
    { 0, 0 },
    { 0x10b42569, 3 },
    { 0, 0 },
    { 0x47e8256b, 3 },
    { 0x4cac256c, 4 },
    { 0, 0 },
    { 0x7114256e, 3 },
    { 0x9314256f, 3 },
    { 0xe9e82570, 3 },
    { 0, 0 },
    { 0x43982572, 3 },
    { 0x45e82573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d302576, 3 },
    { 0xcfcc2577, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f8257a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd280257d, 3 },
    { 0, 0 },
    { 0x9348257f, 3 },
    { 0x9698257f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3c2583, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a0258a, 3 },
    { 0x790258a, 3 },
    { 0xe97c258c, 4 },
    { 0xa2e8258c, 3 },
    { 0xc60258e, 3 },
    { 0x3ab0258f, 3 },
    { 0, 0 },
    { 0x32402591, 4 },
    { 0x19342592, 3 },
    { 0x8d782593, 3 },
    { 0x7d242591, 3 },
    { 0, 0 },
    { 0xbd782596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0182599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc00259c, 4 },
    { 0, 0 },
    { 0xfe68259e, 3 },
    { 0xb600259e, 3 },
    { 0xbf5c25a0, 3 },
    { 0, 0 },
    { 0xd0a025a2, 3 },
    { 0x973425a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b825a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79025ae, 3 },
    { 0x25825af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x632825b2, 4 },
    { 0x8f425b3, 3 },
    { 0, 0 },
    { 0x2e825b5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f825b8, 3 },
    { 0xd87425b8, 3 },
    { 0x41dc25b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21c25be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67025c6, 4 },
    { 0, 0 },
    { 0xc10825c8, 3 },
    { 0, 0 },
    { 0x5c7425ca, 3 },
    { 0x9ecc25cb, 3 },
    { 0xd45c25cc, 4 },
    { 0x952025cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95425d0, 3 },
    { 0x1db825d0, 3 },
    { 0x74425d0, 3 },
    { 0, 0 },
    { 0x5f7825d4, 3 },
    { 0xdfdc25d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6825da, 3 },
    { 0x97825db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acc25de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2c25e5, 4 },
    { 0x1b6c25e5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736025eb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9425ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dc25f2, 4 },
    { 0xc09c25f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704425f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57782600, 4 },
    { 0x23d42601, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9342604, 3 },
    { 0, 0 },
    { 0x55dc2606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde0260e, 3 },
    { 0x6580260f, 4 },
    { 0xeca02610, 4 },
    { 0x57ac2611, 3 },
    { 0xe91c2611, 3 },
    { 0, 0 },
    { 0x4a342614, 3 },
    { 0, 0 },
    { 0xc8982616, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f82619, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a50261d, 3 },
    { 0xea64261e, 4 },
    { 0x9ae4261f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca182623, 3 },
    { 0x8aec2624, 3 },
    { 0x25e42625, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2990262c, 3 },
    { 0x7650262d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d42630, 3 },
    { 0xf4502631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5c2635, 3 },
    { 0x92442636, 3 },
    { 0, 0 },
    { 0xeafc2638, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f4263d, 3 },
    { 0, 0 },
    { 0x5394263f, 4 },
    { 0xb650263f, 4 },
    { 0x87a02641, 3 },
    { 0x4ec82640, 3 },
    { 0xb1c82643, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e42647, 4 },
    { 0xbb602647, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e78264c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c342650, 3 },
    { 0, 0 },
    { 0x177c2652, 3 },
    { 0, 0 },
    { 0x4d4c2654, 3 },
    { 0xda642655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6142658, 3 },
    { 0x8c4c2658, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8914265e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5882661, 3 },
    { 0x2b602661, 3 },
    { 0xd29c2661, 3 },
    { 0xb97c2664, 4 },
    { 0x27c2665, 3 },
    { 0, 0 },
    { 0x2a702667, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24266c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e02672, 3 },
    { 0x10f42673, 4 },
    { 0x2da82672, 3 },
    { 0x3dc82675, 3 },
    { 0, 0 },
    { 0x317c2677, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c4267c, 3 },
    { 0, 0 },
    { 0x9294267e, 4 },
    { 0, 0 },
    { 0x4e642680, 3 },
    { 0xd9042681, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x79602684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcc268a, 3 },
    { 0xcffc268a, 3 },
    { 0xe14268c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fc268f, 4 },
    { 0x2f4c268f, 4 },
    { 0x922c2691, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa302695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61c2698, 3 },
    { 0xa0042699, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00269f, 3 },
    { 0, 0 },
    { 0xfe026a1, 3 },
    { 0x661026a2, 4 },
    { 0x601c26a2, 3 },
    { 0xdaf026a4, 3 },
    { 0xf8b026a5, 3 },
    { 0, 0 },
    { 0xaf6826a7, 3 },
    { 0xd7d826a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4826b1, 3 },
    { 0xcf5026b1, 4 },
    { 0, 0 },
    { 0xe93426b4, 4 },
    { 0x3d7c26b5, 4 },
    { 0xb76826b4, 3 },
    { 0x5c1c26b7, 4 },
    { 0xe7d826b7, 3 },
    { 0, 0 },
    { 0xa73426ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c426bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4026c3, 3 },
    { 0x7c2026c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed826d5, 3 },
    { 0x993426d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x920426d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6826dc, 3 },
    { 0, 0 },
    { 0x699026de, 3 },
    { 0x6be026de, 3 },
    { 0x24426df, 4 },
    { 0, 0 },
    { 0xb45426e2, 5 },
    { 0, 0 },
    { 0x8b426e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x808826e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8426ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b026ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66c26f4, 4 },
    { 0xeee826f5, 3 },
    { 0x978826f5, 3 },
    { 0x5d426f6, 3 },
    { 0x5cb826f8, 3 },
    { 0xfee426f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5826fe, 5 },
    { 0x3da826fe, 3 },
    { 0, 0 },
    { 0xe4542701, 3 },
    { 0x2b542702, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3c2705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7302708, 3 },
    { 0, 0 },
    { 0x815c270a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa170270e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c2711, 3 },
    { 0xf282711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c30271b, 3 },
    { 0xcd58271c, 3 },
    { 0x449c271d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d82720, 3 },
    { 0x1fec2721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8002725, 3 },
    { 0x355c2726, 3 },
    { 0xc7e02727, 4 },
    { 0xf682728, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d8272e, 3 },
    { 0x490c272f, 4 },
    { 0xa29c272e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b82733, 3 },
    { 0x21c2734, 3 },
    { 0x55642735, 3 },
    { 0xe9d42735, 4 },
    { 0x7d3c2735, 3 },
    { 0x13ec2738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d0273b, 4 },
    { 0x74a4273c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616c2745, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff8274a, 3 },
    { 0xf664274a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c274e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf482751, 3 },
    { 0, 0 },
    { 0x1b182753, 4 },
    { 0, 0 },
    { 0x428c2755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ec275a, 3 },
    { 0xc398275b, 3 },
    { 0, 0 },
    { 0x450c275d, 3 },
    { 0xa7fc275e, 3 },
    { 0xc608275f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac02762, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8374276f, 3 },
    { 0xdf08276f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0742773, 3 },
    { 0, 0 },
    { 0xd67c2775, 3 },
    { 0, 0 },
    { 0x66f82777, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e82783, 3 },
    { 0x4ef02784, 3 },
    { 0x48642784, 4 },
    { 0xca942786, 3 },
    { 0xaa482784, 3 },
    { 0, 0 },
    { 0xffb82789, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fc278d, 4 },
    { 0x2d8278e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a02792, 4 },
    { 0, 0 },
    { 0x34082794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0279e, 4 },
    { 0, 0 },
    { 0x89027a0, 4 },
    { 0x4bd427a0, 3 },
    { 0, 0 },
    { 0x269c27a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f827ad, 4 },
    { 0x9b7c27ae, 4 },
    { 0xc83427af, 4 },
    { 0x898827b0, 3 },
    { 0xfd3427b0, 3 },
    { 0xfc2027ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eac27b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8827ba, 4 },
    { 0, 0 },
    { 0xa4d027bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc27bf, 3 },
    { 0xd38427c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70427c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3827c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be427cd, 3 },
    { 0x9c5c27ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15c27d9, 4 },
    { 0x297c27da, 3 },
    { 0, 0 },
    { 0x6a4427dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb027df, 3 },
    { 0, 0 },
    { 0xb2427e1, 3 },
    { 0x6a8c27e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5027e7, 3 },
    { 0, 0 },
    { 0xd5fc27e9, 3 },
    { 0xa46027ea, 3 },
    { 0, 0 },
    { 0x958827ec, 3 },
    { 0x8a4827ed, 3 },
    { 0, 0 },
    { 0xbf5427ef, 3 },
    { 0, 0 },
    { 0xa68c27f1, 3 },
    { 0x2a7427f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x159027f5, 3 },
    { 0, 0 },
    { 0x850827f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a827fc, 3 },
    { 0, 0 },
    { 0xdb3827fe, 3 },
    { 0xb4c827fe, 3 },
    { 0x369c27ff, 4 },
    { 0xa3dc27ff, 4 },
    { 0x624c2802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fc2805, 3 },
    { 0x5a482806, 3 },
    { 0xcd802807, 3 },
    { 0, 0 },
    { 0x98342809, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00280f, 3 },
    { 0xc0dc2810, 3 },
    { 0x55582811, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0c281b, 4 },
    { 0x4088281c, 3 },
    { 0x4968281c, 4 },
    { 0x2628281e, 3 },
    { 0xe100281b, 3 },
    { 0xc30c2820, 3 },
    { 0, 0 },
    { 0x9a02822, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2382826, 3 },
    { 0x77702827, 5 },
    { 0, 0 },
    { 0x56042829, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9c282e, 3 },
    { 0xdd3c282f, 4 },
    { 0x7cbc2830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2602833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b20283b, 3 },
    { 0x5f4283b, 5 },
    { 0x1dc0283d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c242843, 4 },
    { 0x1382843, 3 },
    { 0xeb42844, 4 },
    { 0xd1f82846, 3 },
    { 0xa9142847, 3 },
    { 0xad102847, 3 },
    { 0, 0 },
    { 0xf95c284a, 3 },
    { 0x131c284b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f02853, 3 },
    { 0, 0 },
    { 0x35c2855, 3 },
    { 0xf6182855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb02861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d342866, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8286a, 3 },
    { 0xb638286b, 4 },
    { 0xc3cc286c, 4 },
    { 0x1ab8286d, 3 },
    { 0xeff4286e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c602871, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2382877, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da42880, 3 },
    { 0x439c2880, 3 },
    { 0xc70c2882, 3 },
    { 0, 0 },
    { 0x541c2884, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ec2887, 3 },
    { 0xdb0c2888, 3 },
    { 0xf9e02887, 4 },
    { 0xa4dc288a, 3 },
    { 0xfe08288b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c08288f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1842895, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb082898, 4 },
    { 0x29ac2899, 3 },
    { 0xebe0289a, 4 },
    { 0x66e4289b, 3 },
    { 0xa424289c, 4 },
    { 0x4040289d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc28a1, 4 },
    { 0x36ac28a2, 4 },
    { 0, 0 },
    { 0x434028a4, 3 },
    { 0x449428a5, 3 },
    { 0xf38828a6, 3 },
    { 0, 0 },
    { 0xc7a428a8, 3 },
    { 0, 0 },
    { 0x879028aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc28ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6828b0, 4 },
    { 0x7c8428b0, 3 },
    { 0, 0 },
    { 0xbdc028b3, 3 },
    { 0, 0 },
    { 0x6a5828b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217428ba, 3 },
    { 0x1fe028bb, 3 },
    { 0, 0 },
    { 0xeedc28bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x268428c0, 3 },
    { 0x1d4028c1, 3 },
    { 0x9d9c28c1, 4 },
    { 0xdf6c28c3, 3 },
    { 0xde6428c4, 3 },
    { 0x679828c2, 3 },
    { 0x59a828c1, 3 },
    { 0x1f6028c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36428ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4028cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90828d3, 3 },
    { 0xbdf428d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01028d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x679028db, 3 },
    { 0xe14028dc, 3 },
    { 0xcf2c28dd, 3 },
    { 0xf17028dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f028e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c028e8, 4 },
    { 0x238c28e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2c28ef, 3 },
    { 0, 0 },
    { 0xebc28f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91028f4, 3 },
    { 0xe58428f4, 3 },
    { 0, 0 },
    { 0x6ab028f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ac28fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c28ff, 5 },
    { 0x88202900, 3 },
    { 0x54b428ff, 3 },
    { 0xc742901, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc8290b, 3 },
    { 0xf94290c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd020290f, 3 },
    { 0xadd82910, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd42913, 3 },
    { 0x36902913, 3 },
    { 0x43002915, 3 },
    { 0, 0 },
    { 0xe1182917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad982923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1098292a, 3 },
    { 0x3804292a, 4 },
    { 0x3158292b, 4 },
    { 0x3c8c292c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98242933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a82939, 3 },
    { 0xa6e8293a, 3 },
    { 0xfbd0293b, 3 },
    { 0, 0 },
    { 0xf9f4293d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26282940, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46582947, 4 },
    { 0, 0 },
    { 0x6ccc2949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb338294d, 3 },
    { 0xf28c294d, 4 },
    { 0, 0 },
    { 0xafcc2950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32c2954, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e82958, 3 },
    { 0x26782959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc824295d, 3 },
    { 0x4ae8295d, 4 },
    { 0, 0 },
    { 0xa4f82960, 3 },
    { 0xad302961, 3 },
    { 0x30ec2962, 3 },
    { 0xd1dc2963, 3 },
    { 0x9bb82964, 4 },
    { 0x37542964, 5 },
    { 0xaef42966, 3 },
    { 0x2d782965, 3 },
    { 0x9ec42966, 3 },
    { 0x499c2969, 3 },
    { 0xa6102969, 3 },
    { 0xcfc02969, 3 },
    { 0x9968296c, 3 },
    { 0x751c296c, 3 },
    { 0xeec0296e, 4 },
    { 0xb7b4296f, 3 },
    { 0, 0 },
    { 0xdbc42971, 4 },
    { 0xb6942972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x57902975, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2978, 4 },
    { 0, 0 },
    { 0x5154297a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb74297d, 3 },
    { 0x6094297e, 3 },
    { 0x6dec297e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7742982, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b42987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c298a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95982992, 4 },
    { 0x4c82993, 3 },
    { 0xc102992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d82998, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3000299b, 3 },
    { 0x2b84299c, 3 },
    { 0xdc1c299b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc829a5, 3 },
    { 0, 0 },
    { 0x5c6c29a7, 4 },
    { 0xee6c29a7, 3 },
    { 0, 0 },
    { 0xdc1429aa, 3 },
    { 0x62a029ab, 3 },
    { 0x477029ac, 3 },
    { 0xdda829ad, 4 },
    { 0, 0 },
    { 0xafa429af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4829b3, 4 },
    { 0, 0 },
    { 0xd44c29b5, 3 },
    { 0x196c29b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16029b9, 3 },
    { 0, 0 },
    { 0x1a7829bb, 3 },
    { 0, 0 },
    { 0x81fc29bd, 3 },
    { 0, 0 },
    { 0xed5029bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7029c6, 3 },
    { 0, 0 },
    { 0x2c2029c8, 3 },
    { 0, 0 },
    { 0x8f9829ca, 3 },
    { 0xb09029ca, 3 },
    { 0xdcc029ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3829d5, 3 },
    { 0x690029d6, 4 },
    { 0x135c29d7, 3 },
    { 0xbe3429d7, 4 },
    { 0xae9c29d9, 3 },
    { 0x72dc29d8, 3 },
    { 0x8e4829da, 3 },
    { 0x3d9c29dc, 3 },
    { 0xcfac29dd, 3 },
    { 0x235029dc, 3 },
    { 0xc4829df, 3 },
    { 0xe2e829d6, 3 },
    { 0xae429e1, 3 },
    { 0xf7f829e2, 3 },
    { 0x39b829e2, 3 },
    { 0x95429e2, 3 },
    { 0x61e029e5, 3 },
    { 0x5bbc29e2, 3 },
    { 0xc01429e7, 5 },
    { 0, 0 },
    { 0x418429e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0029ef, 3 },
    { 0x148829ef, 3 },
    { 0xa28029ef, 3 },
    { 0xb6a429f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407429fb, 3 },
    { 0xae0029fc, 3 },
    { 0, 0 },
    { 0x7ac029fe, 3 },
    { 0, 0 },
    { 0x2fb42a00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec342a06, 4 },
    { 0x9f4c2a07, 3 },
    { 0xafa42a06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cec2a15, 3 },
    { 0x7ef82a16, 3 },
    { 0xad002a17, 5 },
    { 0, 0 },
    { 0x2b7c2a19, 4 },
    { 0xfd702a1a, 4 },
    { 0x46742a1b, 3 },
    { 0, 0 },
    { 0xe0902a1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee82a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72282a29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bc2a2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f42a33, 3 },
    { 0xf3642a34, 3 },
    { 0x5b742a35, 3 },
    { 0xd7642a36, 3 },
    { 0x46c2a37, 4 },
    { 0x432c2a38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca42a3b, 3 },
    { 0xa58c2a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b002a40, 4 },
    { 0x70282a40, 3 },
    { 0xa0bc2a42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d82a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c02a4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b82a4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca702a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e942a54, 3 },
    { 0x87f82a55, 3 },
    { 0x64f82a54, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf182a5f, 3 },
    { 0, 0 },
    { 0x9242a61, 5 },
    { 0x74782a62, 3 },
    { 0x1cc82a63, 3 },
    { 0x92082a64, 3 },
    { 0xeb842a65, 3 },
    { 0xde882a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3382a6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f82a70, 3 },
    { 0x80e82a70, 5 },
    { 0xcad42a72, 3 },
    { 0x96542a73, 3 },
    { 0x55482a74, 3 },
    { 0x86942a73, 3 },
    { 0xd1e42a76, 3 },
    { 0x12202a75, 4 },
    { 0x8d8c2a78, 4 },
    { 0xe13c2a79, 3 },
    { 0xd21c2a7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f42a7d, 4 },
    { 0x7b82a7d, 3 },
    { 0, 0 },
    { 0x60102a80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afc2a8c, 3 },
    { 0x6c682a8c, 3 },
    { 0, 0 },
    { 0x2e4c2a8f, 4 },
    { 0x1f82a90, 4 },
    { 0x97d82a90, 3 },
    { 0x27782a91, 3 },
    { 0, 0 },
    { 0x9f282a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdc2a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1802aa1, 4 },
    { 0xcc102aa1, 3 },
    { 0xe5f02aa3, 3 },
    { 0xc6882aa1, 3 },
    { 0xee3c2aa1, 3 },
    { 0x3b902aa3, 3 },
    { 0xca282aa7, 3 },
    { 0x11982aa2, 3 },
    { 0xe5e82aa9, 3 },
    { 0x193c2aaa, 3 },
    { 0xf7082aa8, 3 },
    { 0xb2202aac, 4 },
    { 0x61b42aac, 5 },
    { 0, 0 },
    { 0xb3b02aaf, 3 },
    { 0, 0 },
    { 0x86e82ab1, 4 },
    { 0, 0 },
    { 0xed782ab3, 3 },
    { 0, 0 },
    { 0x956c2ab5, 3 },
    { 0, 0 },
    { 0x6bd82ab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8882aba, 3 },
    { 0, 0 },
    { 0x2f742abc, 4 },
    { 0, 0 },
    { 0x7e02abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fec2ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec42aca, 4 },
    { 0x7dc42aca, 3 },
    { 0, 0 },
    { 0xb21c2acd, 4 },
    { 0x11a82acd, 4 },
    { 0x8ab02ace, 3 },
    { 0x40382ace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0702ad6, 3 },
    { 0x16e02ad7, 4 },
    { 0xb8282ad8, 5 },
    { 0xaa042ad9, 3 },
    { 0, 0 },
    { 0xd89c2adb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12942ae1, 4 },
    { 0, 0 },
    { 0xddcc2ae3, 3 },
    { 0, 0 },
    { 0xdedc2ae5, 3 },
    { 0xbc142ae6, 3 },
    { 0, 0 },
    { 0x25b42ae8, 3 },
    { 0, 0 },
    { 0x9bf82aea, 3 },
    { 0xd6702aeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6882aee, 3 },
    { 0x3c542aef, 3 },
    { 0x73242af0, 3 },
    { 0x4aa02aef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad02af4, 3 },
    { 0xb00c2af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a82b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560c2b06, 3 },
    { 0, 0 },
    { 0xe81c2b08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c2b0b, 3 },
    { 0xf102b0b, 4 },
    { 0x84c2b0b, 4 },
    { 0, 0 },
    { 0x83bc2b0f, 4 },
    { 0x81002b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e42b13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67182b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8942b1c, 3 },
    { 0xdce02b1d, 4 },
    { 0, 0 },
    { 0xd4bc2b1f, 3 },
    { 0x70702b20, 3 },
    { 0, 0 },
    { 0x710c2b22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74102b33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b82b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8542b3b, 5 },
    { 0x2742b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8c2b3f, 3 },
    { 0, 0 },
    { 0xe8b82b41, 4 },
    { 0, 0 },
    { 0xc54c2b43, 3 },
    { 0xdac02b44, 3 },
    { 0x72dc2b44, 4 },
    { 0x6f7c2b45, 5 },
    { 0x32082b43, 3 },
    { 0xdd1c2b48, 3 },
    { 0x21802b49, 4 },
    { 0x7482b49, 4 },
    { 0x90d82b4b, 4 },
    { 0x11182b49, 3 },
    { 0x55c2b4d, 3 },
    { 0xff7c2b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf202b55, 3 },
    { 0x98582b55, 3 },
    { 0x8b702b55, 4 },
    { 0x5ecc2b58, 3 },
    { 0x6f902b56, 3 },
    { 0x3a782b5a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a02b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81442b60, 3 },
    { 0x21bc2b61, 3 },
    { 0x70f82b60, 4 },
    { 0xe24c2b63, 4 },
    { 0x2ec02b62, 3 },
    { 0x9bc2b65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe302b74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac42b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f82b7d, 3 },
    { 0xbf982b7e, 3 },
    { 0xb2c82b7e, 4 },
    { 0x93c82b7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb82b83, 3 },
    { 0, 0 },
    { 0x4b942b85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf842b89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cc2b8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc42b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a82b93, 3 },
    { 0, 0 },
    { 0xc3442b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24942b99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97282b9d, 3 },
    { 0x4ca42b9e, 4 },
    { 0xeef82b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62502ba2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6402bad, 3 },
    { 0xee5c2bad, 4 },
    { 0x24042bad, 3 },
    { 0x1f82bb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64182bb5, 3 },
    { 0x57e02bb6, 3 },
    { 0x2fa42bb7, 3 },
    { 0xc0f02bb5, 3 },
    { 0x77c2bb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab42bc5, 3 },
    { 0x85342bc5, 3 },
    { 0xe9ac2bc7, 3 },
    { 0x394c2bc7, 3 },
    { 0xcabc2bc6, 3 },
    { 0x39942bc7, 3 },
    { 0, 0 },
    { 0x5c542bcc, 4 },
    { 0xe27c2bcc, 3 },
    { 0xc0042bce, 3 },
    { 0xbca02bcf, 3 },
    { 0x98a82bcd, 3 },
    { 0, 0 },
    { 0x34002bd2, 3 },
    { 0x7c642bd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e42bd8, 3 },
    { 0x364c2bd8, 3 },
    { 0x6b502bda, 3 },
    { 0x5f4c2bda, 4 },
    { 0xed982bdb, 3 },
    { 0x69e02bd8, 3 },
    { 0x92b02bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d942bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623c2bee, 4 },
    { 0, 0 },
    { 0x92f42bf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c82bf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff42bf7, 3 },
    { 0x5a402bf8, 3 },
    { 0xecfc2bf8, 3 },
    { 0x620c2bfa, 3 },
    { 0x2002bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e82c09, 4 },
    { 0xd5382c0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f582c0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17202c10, 4 },
    { 0x5e802c10, 3 },
    { 0x48082c12, 3 },
    { 0x9f82c13, 3 },
    { 0xccc82c14, 4 },
    { 0x7d682c12, 3 },
    { 0x4b702c16, 3 },
    { 0xc6802c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb82c1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e82c20, 4 },
    { 0x25cc2c20, 4 },
    { 0xa6642c22, 3 },
    { 0x30c02c23, 3 },
    { 0x93bc2c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec882c27, 3 },
    { 0x73a82c28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd02c2b, 3 },
    { 0xea882c2c, 3 },
    { 0x91502c2d, 3 },
    { 0, 0 },
    { 0xaffc2c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af82c32, 3 },
    { 0, 0 },
    { 0x59c02c34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e02c3a, 3 },
    { 0x3bdc2c3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa82c42, 3 },
    { 0x57f02c42, 3 },
    { 0x59b02c44, 3 },
    { 0x531c2c44, 3 },
    { 0x64e82c45, 3 },
    { 0x99e42c46, 3 },
    { 0xdeac2c48, 4 },
    { 0x7642c48, 3 },
    { 0xa93c2c4a, 3 },
    { 0, 0 },
    { 0x39c02c4c, 4 },
    { 0xa2ec2c4d, 3 },
    { 0xdb842c4d, 3 },
    { 0x70482c4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4c2c52, 3 },
    { 0x25742c53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d02c59, 3 },
    { 0x63cc2c59, 3 },
    { 0x6a5c2c59, 4 },
    { 0xc1ec2c5c, 3 },
    { 0xe1c42c5d, 4 },
    { 0x26042c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78002c6a, 3 },
    { 0x54ec2c6b, 3 },
    { 0x6c6c2c6b, 3 },
    { 0x76a42c6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a42c72, 3 },
    { 0, 0 },
    { 0x146c2c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1c2c77, 3 },
    { 0x80902c78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22202c7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e02c85, 5 },
    { 0x2ce42c86, 3 },
    { 0x49642c87, 3 },
    { 0xed142c87, 3 },
    { 0xff582c89, 3 },
    { 0x98402c89, 3 },
    { 0, 0 },
    { 0xa582c8c, 4 },
    { 0x45a82c8d, 3 },
    { 0xcb342c8d, 3 },
    { 0x8f1c2c8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe42c96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26082c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39802ca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c02caa, 3 },
    { 0, 0 },
    { 0x45f82cac, 3 },
    { 0x3d1c2cad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7c2cb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c2cb4, 3 },
    { 0, 0 },
    { 0x3c8c2cb6, 4 },
    { 0, 0 },
    { 0xfe342cb8, 3 },
    { 0x16c82cb9, 3 },
    { 0, 0 },
    { 0x69382cbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7582cbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba042cc2, 3 },
    { 0xae482cc3, 3 },
    { 0xf7d02cc3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e82cc7, 3 },
    { 0, 0 },
    { 0xc1342cc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf682cd2, 4 },
    { 0x36f82cd2, 4 },
    { 0x8d282cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f42cd8, 4 },
    { 0xd102cd8, 3 },
    { 0xe6c02cda, 4 },
    { 0x90082cd9, 4 },
    { 0xfeac2cdc, 4 },
    { 0xd0dc2cdd, 3 },
    { 0xb102cde, 4 },
    { 0, 0 },
    { 0x64f82ce0, 3 },
    { 0, 0 },
    { 0x3202ce2, 3 },
    { 0, 0 },
    { 0xe6182ce4, 4 },
    { 0x15602ce5, 3 },
    { 0, 0 },
    { 0xf10c2ce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31442cec, 4 },
    { 0x25bc2ced, 3 },
    { 0xd1002cee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22602cf2, 3 },
    { 0, 0 },
    { 0xd9c82cf4, 4 },
    { 0xa51c2cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5882cfc, 3 },
    { 0, 0 },
    { 0x84a42cfe, 4 },
    { 0x89b82cff, 4 },
    { 0x7ba42cff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f42d03, 4 },
    { 0, 0 },
    { 0xf70c2d05, 3 },
    { 0xfb2c2d06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff82d0f, 5 },
    { 0x8c882d10, 3 },
    { 0x2b942d11, 5 },
    { 0x71b82d12, 3 },
    { 0, 0 },
    { 0xf3242d14, 3 },
    { 0, 0 },
    { 0x84a42d16, 4 },
    { 0xb15c2d17, 3 },
    { 0, 0 },
    { 0x8d002d19, 3 },
    { 0xbe502d19, 4 },
    { 0, 0 },
    { 0x6aa42d1c, 3 },
    { 0xf4d42d1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef82d22, 3 },
    { 0xc1ec2d23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b82d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60c2d30, 3 },
    { 0, 0 },
    { 0x4a942d32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c82d35, 3 },
    { 0xe4ec2d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6c2d3a, 3 },
    { 0x98742d3b, 3 },
    { 0x72b82d3a, 4 },
    { 0, 0 },
    { 0x41842d3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a82d43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3c2d47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e42d4a, 3 },
    { 0x31882d4b, 3 },
    { 0xb5e82d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4942d51, 3 },
    { 0x4d302d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb42d55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d42d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c2d63, 3 },
    { 0, 0 },
    { 0xd1202d65, 4 },
    { 0xd642d66, 4 },
    { 0xc6542d66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca042d6a, 5 },
    { 0, 0 },
    { 0x78e82d6c, 3 },
    { 0xb2502d6d, 3 },
    { 0x625c2d6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64642d75, 3 },
    { 0, 0 },
    { 0xad742d77, 3 },
    { 0, 0 },
    { 0xae8c2d79, 4 },
    { 0xf4c2d7a, 3 },
    { 0x502c2d7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e42d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab82d84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39542d87, 3 },
    { 0x88402d87, 3 },
    { 0, 0 },
    { 0xa4d42d8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d502d91, 3 },
    { 0x89e02d92, 3 },
    { 0x8d242d92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f742d99, 3 },
    { 0xc9ac2d9a, 3 },
    { 0xdc082d9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d402d9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c2da2, 4 },
    { 0x45f82da3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c2da6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d82da9, 4 },
    { 0xfdd42daa, 3 },
    { 0xd7402dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc02dae, 3 },
    { 0xf0a02daf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fc2db5, 3 },
    { 0x98202db5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x157c2db9, 3 },
    { 0x44902db9, 3 },
    { 0, 0 },
    { 0x3ef82dbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ec2dbf, 4 },
    { 0, 0 },
    { 0x73642dc1, 3 },
    { 0xd1fc2dc1, 3 },
    { 0x62882dc2, 4 },
    { 0xb3202dc2, 3 },
    { 0x12c42dc2, 3 },
    { 0, 0 },
    { 0xad082dc7, 3 },
    { 0xc3d42dc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc82dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c82dd3, 3 },
    { 0, 0 },
    { 0x47182dd5, 3 },
    { 0xe6b82dd6, 5 },
    { 0xa74c2dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25942dda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a82ddd, 3 },
    { 0x6af42dde, 3 },
    { 0x3e182ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63c2de6, 4 },
    { 0x40ec2de6, 3 },
    { 0x78142de8, 3 },
    { 0xfba82de9, 4 },
    { 0x41082de9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acc2ded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d42df1, 3 },
    { 0, 0 },
    { 0xffec2df3, 3 },
    { 0, 0 },
    { 0x74e82df5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb382df8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd02dfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f202dff, 3 },
    { 0, 0 },
    { 0xf23c2e01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d642e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43242e08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c142e0b, 4 },
    { 0x30082e0c, 4 },
    { 0x35a42e0d, 3 },
    { 0, 0 },
    { 0x7a582e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d82e12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a42e15, 4 },
    { 0x445c2e15, 3 },
    { 0x9f282e17, 4 },
    { 0x15942e18, 3 },
    { 0xa9982e16, 3 },
    { 0, 0 },
    { 0x44b02e1b, 3 },
    { 0xb70c2e1c, 3 },
    { 0xdc202e1c, 3 },
    { 0x3c642e1e, 4 },
    { 0xbca82e1b, 4 },
    { 0xf81c2e20, 4 },
    { 0xe3b42e20, 3 },
    { 0xce542e21, 3 },
    { 0xdfe42e1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d42e29, 5 },
    { 0x57882e2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2382e2f, 3 },
    { 0x55882e30, 3 },
    { 0xf7242e31, 3 },
    { 0xbcd02e32, 4 },
    { 0xffac2e32, 3 },
    { 0xb3242e34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73702e37, 3 },
    { 0, 0 },
    { 0xe6bc2e39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d202e45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec402e4a, 3 },
    { 0x6bbc2e4b, 3 },
    { 0, 0 },
    { 0xc28c2e4d, 3 },
    { 0x89c82e4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b102e51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f42e56, 3 },
    { 0, 0 },
    { 0x4c602e58, 3 },
    { 0x11242e58, 3 },
    { 0x113c2e58, 4 },
    { 0xcc702e5b, 4 },
    { 0x8fac2e58, 4 },
    { 0xbb802e5b, 4 },
    { 0xa1c82e5e, 3 },
    { 0x17dc2e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6742e67, 3 },
    { 0x99882e68, 3 },
    { 0x8edc2e69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e42e6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55242e72, 4 },
    { 0x6002e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a02e76, 3 },
    { 0xebfc2e76, 3 },
    { 0, 0 },
    { 0xfbc2e79, 3 },
    { 0xe61c2e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab02e7d, 4 },
    { 0x19f82e7d, 5 },
    { 0x8d3c2e7f, 4 },
    { 0x460c2e7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dc2e83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c82e87, 3 },
    { 0x94bc2e88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e382e8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18702e91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd902e99, 3 },
    { 0x59cc2e99, 3 },
    { 0x14042e9b, 3 },
    { 0xb1082e9b, 3 },
    { 0x8d002e9d, 3 },
    { 0x3c902e9e, 3 },
    { 0x1ba02e9f, 4 },
    { 0xce082e9f, 4 },
    { 0, 0 },
    { 0x5a102ea2, 3 },
    { 0xdfa02ea3, 3 },
    { 0x67642ea2, 3 },
    { 0, 0 },
    { 0x78442ea6, 3 },
    { 0x97942ea7, 3 },
    { 0xcf582ea8, 3 },
    { 0x48502ea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5c2eac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x16002eaf, 4 },
    { 0xa3c2eb0, 3 },
    { 0xc1702eb0, 3 },
    { 0x7bc42eb2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bc2eb7, 3 },
    { 0xb0642eb8, 4 },
    { 0x3442eb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df82ebe, 4 },
    { 0, 0 },
    { 0x6d6c2ec0, 3 },
    { 0, 0 },
    { 0x71d82ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc2ec9, 4 },
    { 0, 0 },
    { 0x54042ecb, 4 },
    { 0xd7c02ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefc2ecf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79942ed2, 4 },
    { 0xcdb02ed2, 4 },
    { 0, 0 },
    { 0x74d82ed5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a42ed8, 4 },
    { 0x8f242ed9, 3 },
    { 0x97742eda, 5 },
    { 0x92d82eda, 3 },
    { 0xee42edb, 3 },
    { 0x28202edb, 3 },
    { 0xbbfc2ede, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade82ee5, 3 },
    { 0xa9a02ee6, 3 },
    { 0x39102ee7, 3 },
    { 0x69c82ee7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5082eeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e82eee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402ef3, 3 },
    { 0x60082ef4, 3 },
    { 0xd8e42ef5, 3 },
    { 0, 0 },
    { 0x49382ef7, 3 },
    { 0xb87c2ef8, 3 },
    { 0x5c9c2ef9, 4 },
    { 0x58542efa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c2efe, 3 },
    { 0x50a82efe, 3 },
    { 0, 0 },
    { 0xfee02f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc2f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dc2f0d, 3 },
    { 0, 0 },
    { 0x96882f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58842f18, 3 },
    { 0x889c2f19, 3 },
    { 0x94202f19, 3 },
    { 0x91dc2f19, 4 },
    { 0x51d42f1c, 3 },
    { 0, 0 },
    { 0xa5882f1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57002f22, 3 },
    { 0x93702f23, 4 },
    { 0x40842f24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12582f2c, 4 },
    { 0x1c002f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05c2f32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd642f37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490c2f3e, 3 },
    { 0, 0 },
    { 0xe4542f40, 3 },
    { 0, 0 },
    { 0x66642f42, 3 },
    { 0, 0 },
    { 0x17c42f44, 4 },
    { 0, 0 },
    { 0x41602f46, 3 },
    { 0x44642f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26c2f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e42f4f, 3 },
    { 0xaa082f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204c2f54, 3 },
    { 0x106c2f55, 5 },
    { 0x94282f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c002f5a, 3 },
    { 0x95402f5b, 3 },
    { 0, 0 },
    { 0xcfe82f5d, 3 },
    { 0, 0 },
    { 0x8fdc2f5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a82f63, 3 },
    { 0x34e42f63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72842f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9082f6f, 5 },
    { 0x1d7c2f70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91082f7a, 3 },
    { 0x2ea02f7b, 3 },
    { 0x19d82f7c, 4 },
    { 0, 0 },
    { 0xb22c2f7e, 5 },
    { 0xbcbc2f7f, 3 },
    { 0xd6342f80, 3 },
    { 0x82fc2f80, 3 },
    { 0x72102f82, 3 },
    { 0x1402f7e, 3 },
    { 0xb1502f84, 3 },
    { 0, 0 },
    { 0x7e942f86, 3 },
    { 0x1fac2f86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6302f8c, 3 },
    { 0x11282f8d, 3 },
    { 0x1cec2f8e, 3 },
    { 0x7b9c2f8e, 3 },
    { 0x1cc82f8e, 3 },
    { 0, 0 },
    { 0x31f42f92, 3 },
    { 0xb3b82f93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7c2f97, 3 },
    { 0x6c682f98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08c2fa3, 3 },
    { 0x70002fa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf642fa9, 4 },
    { 0, 0 },
    { 0xa53c2fab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd42faf, 3 },
    { 0, 0 },
    { 0x9be02fb1, 3 },
    { 0x466c2fb1, 4 },
    { 0x6a6c2fb3, 3 },
    { 0, 0 },
    { 0x25ac2fb5, 4 },
    { 0x84502fb5, 3 },
    { 0x60742fb6, 3 },
    { 0x87742fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ec2fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac2fbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f442fc5, 4 },
    { 0, 0 },
    { 0x55282fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0542fca, 3 },
    { 0xe5502fcb, 3 },
    { 0, 0 },
    { 0xc5a02fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66182fd1, 4 },
    { 0, 0 },
    { 0xb3ec2fd3, 3 },
    { 0, 0 },
    { 0xb5402fd5, 3 },
    { 0xe1c02fd6, 3 },
    { 0x34b82fd7, 3 },
    { 0xe73c2fd8, 4 },
    { 0x44402fd9, 3 },
    { 0xbdc02fd9, 3 },
    { 0xb7182fdb, 3 },
    { 0x66942fda, 5 },
    { 0xfb002fdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54602fe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33c2fe3, 3 },
    { 0x627c2fe3, 3 },
    { 0xba482fe5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db02fef, 5 },
    { 0xf8bc2ff0, 3 },
    { 0xcc6c2ff1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc102ff6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e482ff9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dac2ffd, 4 },
    { 0x25b82ffe, 3 },
    { 0, 0 },
    { 0x6ab03000, 3 },
    { 0xd6983000, 4 },
    { 0x8c443002, 5 },
    { 0x1143002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fc3006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f83009, 3 },
    { 0xad64300a, 3 },
    { 0x5358300b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68583012, 4 },
    { 0x6e843012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39083017, 4 },
    { 0x49b83018, 3 },
    { 0x41243018, 3 },
    { 0, 0 },
    { 0x4a64301b, 3 },
    { 0xcc30301b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc83020, 4 },
    { 0x534c3021, 3 },
    { 0x4aa03022, 3 },
    { 0xcd283020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48c3026, 3 },
    { 0, 0 },
    { 0x4403028, 3 },
    { 0xbb703028, 3 },
    { 0, 0 },
    { 0x7d3c302b, 3 },
    { 0xefbc302b, 4 },
    { 0x484c302d, 4 },
    { 0xa44302c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bc3031, 3 },
    { 0xcbb83031, 3 },
    { 0, 0 },
    { 0x65203034, 4 },
    { 0x5f103034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93003039, 3 },
    { 0x5de83039, 3 },
    { 0x4d28303b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c3040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d03043, 3 },
    { 0xef443043, 3 },
    { 0x88483043, 4 },
    { 0x1a83046, 4 },
    { 0x42b43045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c4304a, 4 },
    { 0x7e48304b, 3 },
    { 0x1f80304a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f03050, 3 },
    { 0x48343051, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c43054, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe0305c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4008305f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4103063, 3 },
    { 0x2ae03064, 3 },
    { 0x7c483063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bc3068, 3 },
    { 0, 0 },
    { 0x9bf4306a, 3 },
    { 0x3578306b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa0306e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d03072, 4 },
    { 0x2e203072, 4 },
    { 0xf0c83074, 3 },
    { 0, 0 },
    { 0x6fd43076, 3 },
    { 0x1d043077, 4 },
    { 0x8f583078, 3 },
    { 0xda503079, 5 },
    { 0x2b483077, 3 },
    { 0, 0 },
    { 0x6968307c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e03082, 3 },
    { 0x4f903082, 3 },
    { 0xde43083, 4 },
    { 0x4fb43085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc903088, 3 },
    { 0xd0883089, 4 },
    { 0xd0e83088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91583090, 3 },
    { 0x6d183090, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3390309b, 3 },
    { 0xb138309b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc309f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71830a3, 3 },
    { 0, 0 },
    { 0x828830a5, 3 },
    { 0x54e830a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb830aa, 3 },
    { 0x172030ab, 4 },
    { 0x4bbc30ac, 5 },
    { 0xa52030ab, 3 },
    { 0x806030ae, 4 },
    { 0xb0c30ae, 4 },
    { 0x9ab430b0, 5 },
    { 0x2c2430b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de830ba, 3 },
    { 0xc19c30bb, 5 },
    { 0xfc5430bb, 3 },
    { 0x455030bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cc30c3, 3 },
    { 0, 0 },
    { 0xd17030c5, 4 },
    { 0x696c30c6, 3 },
    { 0x18b430c7, 3 },
    { 0xded030c8, 3 },
    { 0x511830c8, 4 },
    { 0x727030ca, 3 },
    { 0x1ed030c7, 3 },
    { 0x4a030cb, 3 },
    { 0xbd2430cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f430d4, 3 },
    { 0x424430d5, 3 },
    { 0xbb2430d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c30d9, 3 },
    { 0x630430da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c30de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709830e2, 3 },
    { 0x7a6c30e2, 4 },
    { 0x5b2430e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36430e7, 3 },
    { 0, 0 },
    { 0x19b830e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9830ed, 3 },
    { 0x909030ee, 4 },
    { 0x26e830ee, 3 },
    { 0, 0 },
    { 0x76c830f1, 3 },
    { 0x3fc430f1, 4 },
    { 0, 0 },
    { 0x429430f4, 4 },
    { 0x97b830f5, 4 },
    { 0x102030f6, 3 },
    { 0x4fc030f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83830fc, 3 },
    { 0, 0 },
    { 0x3c0830fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22983104, 4 },
    { 0xcd603105, 3 },
    { 0xcbb43104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6c3109, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2c310f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf943113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7c3117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e24311b, 4 },
    { 0, 0 },
    { 0x7f18311d, 3 },
    { 0x3220311d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8c3123, 3 },
    { 0x8f183124, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fc3127, 3 },
    { 0x67b83128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8312c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f03130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac3134, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bc313a, 4 },
    { 0xce4313b, 4 },
    { 0, 0 },
    { 0xb3a4313d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8314a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c314f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a983154, 4 },
    { 0, 0 },
    { 0x9083156, 3 },
    { 0, 0 },
    { 0xc4c43158, 4 },
    { 0x6afc3159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9040315f, 3 },
    { 0xaa503160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be83164, 3 },
    { 0xe2d83165, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c316d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc83170, 4 },
    { 0xc7ec3171, 4 },
    { 0xd38c3172, 3 },
    { 0, 0 },
    { 0x62d03174, 3 },
    { 0xa4603174, 3 },
    { 0, 0 },
    { 0xf43177, 3 },
    { 0, 0 },
    { 0xaefc3179, 4 },
    { 0, 0 },
    { 0x33b0317b, 3 },
    { 0x85dc317b, 3 },
    { 0, 0 },
    { 0xac6c317e, 4 },
    { 0xc498317e, 3 },
    { 0x43583180, 3 },
    { 0x80803181, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b083185, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea03189, 3 },
    { 0, 0 },
    { 0x9fe0318b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d8318e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958c3199, 4 },
    { 0x9478319a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219031a3, 3 },
    { 0xd11831a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a431a8, 3 },
    { 0x510831a9, 3 },
    { 0xd90031aa, 3 },
    { 0x4c6031a8, 3 },
    { 0x312431a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32c31b1, 3 },
    { 0x6f2431b2, 3 },
    { 0xecd431b3, 3 },
    { 0, 0 },
    { 0x9fdc31b5, 3 },
    { 0x98bc31b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691831bb, 3 },
    { 0xf11031bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x793c31bf, 3 },
    { 0x9a2031c0, 3 },
    { 0x16a431c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff031c4, 3 },
    { 0x2a9831c5, 3 },
    { 0x27031c5, 3 },
    { 0, 0 },
    { 0x1d2831c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a031cb, 3 },
    { 0x1d7c31cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x301431cf, 5 },
    { 0, 0 },
    { 0xa75031d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d431d4, 3 },
    { 0x28dc31d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742031d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f031e3, 3 },
    { 0x28b431e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x844431e7, 3 },
    { 0x8d3831e8, 3 },
    { 0xfc9831e8, 4 },
    { 0xc7c031ea, 3 },
    { 0x44831eb, 3 },
    { 0xfd5c31eb, 4 },
    { 0x5bc431ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1031f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c31f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f831fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519031ff, 4 },
    { 0x4e303200, 3 },
    { 0x514c3201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f03204, 3 },
    { 0x9cfc3205, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48143209, 3 },
    { 0x7924320a, 3 },
    { 0xa24c320b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af8320e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529c3215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7603219, 3 },
    { 0, 0 },
    { 0x977c321b, 3 },
    { 0xa650321c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd43226, 3 },
    { 0x1a783227, 3 },
    { 0x32e43228, 4 },
    { 0xaae83229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e8322f, 3 },
    { 0x61643230, 3 },
    { 0xe7e0322f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5843236, 3 },
    { 0x8a5c3237, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f44323a, 4 },
    { 0x6728323a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e4323f, 3 },
    { 0x84ac3240, 3 },
    { 0xb583241, 3 },
    { 0xea543241, 3 },
    { 0, 0 },
    { 0xa8343244, 3 },
    { 0x88983245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950c3249, 5 },
    { 0, 0 },
    { 0x489c324b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7990324e, 3 },
    { 0xe520324e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83143257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710325c, 3 },
    { 0xb43c325c, 3 },
    { 0x4508325e, 4 },
    { 0x4b50325f, 4 },
    { 0x8b1c3260, 3 },
    { 0xe48325f, 3 },
    { 0xcb78325f, 3 },
    { 0x28e83262, 3 },
    { 0x82c03262, 3 },
    { 0, 0 },
    { 0x5e703266, 3 },
    { 0x80843267, 3 },
    { 0xa6083268, 3 },
    { 0x35203269, 3 },
    { 0, 0 },
    { 0xbf74326b, 3 },
    { 0, 0 },
    { 0x21c4326d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d183278, 3 },
    { 0x1c8c3278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbac3282, 3 },
    { 0x1b143283, 3 },
    { 0xdc9c3284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a4328c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb203291, 3 },
    { 0, 0 },
    { 0x7d503293, 4 },
    { 0xd3e43294, 4 },
    { 0xcd883294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd383298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98329c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7c32a0, 3 },
    { 0x8db432a0, 3 },
    { 0xefc832a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441432a6, 3 },
    { 0xad7432a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31832ad, 4 },
    { 0xfbb432ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6032b2, 3 },
    { 0x44c032b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0832b6, 3 },
    { 0x60f832b7, 3 },
    { 0xf82032b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67032bc, 3 },
    { 0, 0 },
    { 0x234432be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42832c2, 4 },
    { 0xd98032c3, 5 },
    { 0x62e832c4, 3 },
    { 0x255832c4, 4 },
    { 0x40ac32c6, 3 },
    { 0x910832c2, 3 },
    { 0, 0 },
    { 0x143832c9, 3 },
    { 0x938832c9, 3 },
    { 0x32b832cb, 3 },
    { 0x49d432ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef832d0, 3 },
    { 0x94f432d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8832d5, 3 },
    { 0x683432d5, 4 },
    { 0x7c9832d7, 5 },
    { 0, 0 },
    { 0xb95032d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bc32e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495032ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee432ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bc32f9, 3 },
    { 0, 0 },
    { 0x664432fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09032ff, 4 },
    { 0xa0783300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb943304, 4 },
    { 0x75003305, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560330b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd60330e, 3 },
    { 0xd708330f, 3 },
    { 0xec943310, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd03317, 3 },
    { 0x38d43318, 3 },
    { 0x8a143318, 3 },
    { 0, 0 },
    { 0x24a0331b, 3 },
    { 0xca74331b, 3 },
    { 0, 0 },
    { 0xb8d8331e, 4 },
    { 0, 0 },
    { 0x46343320, 3 },
    { 0x411c3321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0743327, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97c332b, 3 },
    { 0x7684332b, 3 },
    { 0x5b10332b, 4 },
    { 0, 0 },
    { 0xd8c332f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16c3333, 3 },
    { 0x589c3333, 3 },
    { 0x7883334, 3 },
    { 0xac83334, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f28333d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e03343, 3 },
    { 0x83344, 3 },
    { 0x83503343, 3 },
    { 0, 0 },
    { 0x7f583347, 4 },
    { 0, 0 },
    { 0x1e343349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c334e, 4 },
    { 0x95a0334e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a83359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2c335d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c3362, 4 },
    { 0xf48c3363, 3 },
    { 0x2be03364, 3 },
    { 0xbe343365, 3 },
    { 0x46743366, 3 },
    { 0x132c3367, 3 },
    { 0x75f03368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9c336b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc3378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9c337b, 3 },
    { 0x94e4337c, 3 },
    { 0, 0 },
    { 0x2790337e, 3 },
    { 0, 0 },
    { 0x97943380, 3 },
    { 0x4cb03381, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e183386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c54338b, 3 },
    { 0x84ac338c, 3 },
    { 0x3e74338d, 3 },
    { 0xcd20338e, 3 },
    { 0x12c0338f, 3 },
    { 0x6ca8338f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8c3395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c339c, 3 },
    { 0x852c339d, 3 },
    { 0, 0 },
    { 0xafd8339f, 4 },
    { 0, 0 },
    { 0x5ce833a1, 3 },
    { 0xf9d033a2, 4 },
    { 0x25d833a1, 3 },
    { 0x9b5033a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c33a7, 3 },
    { 0xa2f833a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7033ab, 4 },
    { 0, 0 },
    { 0x3b3833ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e833b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29833b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c833bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c33c0, 4 },
    { 0x1da433c1, 3 },
    { 0xa15833c1, 4 },
    { 0x1e0433c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fac33ce, 3 },
    { 0, 0 },
    { 0x39e833d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423433d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c33dd, 3 },
    { 0, 0 },
    { 0x7d2833df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dc33e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a033ea, 3 },
    { 0x1a1433eb, 4 },
    { 0, 0 },
    { 0xe57033ed, 3 },
    { 0x8f1833ee, 3 },
    { 0x9b9833ef, 3 },
    { 0xef5833ef, 4 },
    { 0xc0433f1, 3 },
    { 0x5a6033f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7433fc, 4 },
    { 0x567c33fc, 3 },
    { 0x25e033fd, 3 },
    { 0x3dfc33fc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd43402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b83405, 4 },
    { 0xca283406, 3 },
    { 0, 0 },
    { 0x88e83408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798340d, 3 },
    { 0x9b4340e, 3 },
    { 0xc858340f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c83415, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdc341d, 4 },
    { 0, 0 },
    { 0x1354341f, 4 },
    { 0, 0 },
    { 0x2ca83421, 3 },
    { 0, 0 },
    { 0x1bb43423, 4 },
    { 0xe7943424, 3 },
    { 0, 0 },
    { 0x58603426, 4 },
    { 0x64083426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4342b, 3 },
    { 0xae5c342c, 3 },
    { 0, 0 },
    { 0x4004342e, 3 },
    { 0xc068342f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f03434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b8343c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732c3441, 3 },
    { 0x8c003442, 3 },
    { 0x24503441, 4 },
    { 0xbd203443, 3 },
    { 0x84c03445, 4 },
    { 0x9b303445, 3 },
    { 0x9bb43444, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4890344a, 3 },
    { 0xbc6c344a, 3 },
    { 0x258c344c, 3 },
    { 0, 0 },
    { 0x22a4344e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7983454, 3 },
    { 0x10403455, 3 },
    { 0xc43c3455, 3 },
    { 0x18683457, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd60345c, 3 },
    { 0xba30345d, 3 },
    { 0, 0 },
    { 0xbf48345f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68c3463, 4 },
    { 0, 0 },
    { 0x39603465, 4 },
    { 0x1db83465, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd83469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe324346f, 3 },
    { 0x77683470, 3 },
    { 0, 0 },
    { 0x59183472, 4 },
    { 0xdcfc3473, 3 },
    { 0x2f443473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad03478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d24347d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8783480, 3 },
    { 0, 0 },
    { 0xabe43482, 3 },
    { 0xb0e83482, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe44348a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7583490, 4 },
    { 0, 0 },
    { 0x64843492, 3 },
    { 0, 0 },
    { 0xd3c83494, 4 },
    { 0x4ad83494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220349d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca834ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42834af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10834b5, 3 },
    { 0x8bf434b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7034bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d834c2, 3 },
    { 0x307c34c2, 3 },
    { 0xf84034c2, 4 },
    { 0x3cd834c5, 3 },
    { 0xb95834c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f434ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x687834cd, 3 },
    { 0x75fc34cd, 4 },
    { 0x535c34cf, 3 },
    { 0, 0 },
    { 0xd61034d1, 4 },
    { 0x3e2034d2, 3 },
    { 0xc5d434d2, 3 },
    { 0, 0 },
    { 0xfb5834d5, 3 },
    { 0, 0 },
    { 0xaf434d7, 4 },
    { 0x82ec34d7, 3 },
    { 0, 0 },
    { 0x4e4834da, 3 },
    { 0, 0 },
    { 0x48834dc, 3 },
    { 0, 0 },
    { 0xb54c34de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234834e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dc34e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa034ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x216034f0, 4 },
    { 0xe58834f1, 3 },
    { 0x2a3034f2, 3 },
    { 0x57e434f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc34f7, 3 },
    { 0xa41034f8, 3 },
    { 0, 0 },
    { 0x38c434fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f434fe, 5 },
    { 0x51f434ff, 3 },
    { 0xfd703500, 3 },
    { 0, 0 },
    { 0xb9603502, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23c3505, 3 },
    { 0x4eec3506, 4 },
    { 0x591c3506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33c350c, 3 },
    { 0xe638350d, 4 },
    { 0xb688350c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda83512, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c3515, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb18351d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c03523, 3 },
    { 0x6e243524, 3 },
    { 0x5ae83525, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd518352a, 3 },
    { 0x5868352b, 3 },
    { 0x335c352c, 3 },
    { 0xf0e8352d, 4 },
    { 0x6ef8352d, 3 },
    { 0xc530352d, 3 },
    { 0x171c3530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28803534, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a83537, 3 },
    { 0x69d83537, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c0353b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a80353e, 3 },
    { 0x7ac0353e, 3 },
    { 0, 0 },
    { 0x57c83541, 3 },
    { 0x39943542, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a83545, 4 },
    { 0xf3403546, 3 },
    { 0x433c3546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2674354a, 3 },
    { 0xbc68354b, 3 },
    { 0, 0 },
    { 0x2bc8354d, 3 },
    { 0x1724354d, 3 },
    { 0x3a8c354e, 3 },
    { 0xdadc354e, 3 },
    { 0x93dc3551, 3 },
    { 0xe3f83552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a903559, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6c355c, 3 },
    { 0x4c7c355d, 3 },
    { 0xceac355d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1003564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8103567, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc8356a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cc3572, 3 },
    { 0x78b43573, 4 },
    { 0x7503574, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e83577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f0357e, 3 },
    { 0x131c357f, 4 },
    { 0xa73c357f, 4 },
    { 0x6bb03581, 4 },
    { 0, 0 },
    { 0x41b43583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab83586, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7950358b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c43590, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cc359c, 3 },
    { 0xdb48359c, 3 },
    { 0xede8359e, 4 },
    { 0x3530359e, 4 },
    { 0, 0 },
    { 0x692035a1, 4 },
    { 0, 0 },
    { 0xf05835a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80035a6, 4 },
    { 0, 0 },
    { 0x8c0035a8, 3 },
    { 0xc68035a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10835ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d835af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c035b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ac35b8, 3 },
    { 0x20f835b9, 3 },
    { 0x14e835ba, 3 },
    { 0x4dd435b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06435bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3035c2, 3 },
    { 0x5d5435c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e035ce, 3 },
    { 0xa27035ce, 3 },
    { 0x794035d0, 4 },
    { 0xc4a435d0, 3 },
    { 0x4f9835d2, 3 },
    { 0, 0 },
    { 0x74b435d4, 5 },
    { 0, 0 },
    { 0x204435d6, 4 },
    { 0x491035d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c35dc, 3 },
    { 0x70e435dd, 3 },
    { 0, 0 },
    { 0xb4c835df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x836435e2, 3 },
    { 0xd7a035e2, 3 },
    { 0xd9835e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c35e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca835eb, 3 },
    { 0, 0 },
    { 0x55b035ed, 3 },
    { 0, 0 },
    { 0x13cc35ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209835f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b435f6, 3 },
    { 0, 0 },
    { 0xdd8835f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d835fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b435fe, 4 },
    { 0x747035ff, 3 },
    { 0xc43c3600, 4 },
    { 0x75843601, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x683604, 3 },
    { 0x3e383604, 3 },
    { 0x28383604, 3 },
    { 0xbc603607, 3 },
    { 0xa1403607, 3 },
    { 0xa93c3608, 3 },
    { 0xc78c3609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f03612, 3 },
    { 0xd8c03612, 3 },
    { 0x93903614, 3 },
    { 0xc6e03613, 3 },
    { 0x38d03615, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e683619, 3 },
    { 0xa3b43619, 3 },
    { 0xd4503619, 3 },
    { 0x1d883619, 3 },
    { 0x7c143619, 3 },
    { 0xe3f8361c, 4 },
    { 0x347c361c, 3 },
    { 0xc9643620, 3 },
    { 0, 0 },
    { 0xc45c3622, 4 },
    { 0x9a0c3623, 3 },
    { 0xb8483624, 4 },
    { 0x43883625, 3 },
    { 0xaee03624, 3 },
    { 0, 0 },
    { 0x1bcc3628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a84362b, 3 },
    { 0x38ac362c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbc3633, 4 },
    { 0, 0 },
    { 0x160c3635, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d43639, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6143644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c364a, 4 },
    { 0xaf78364b, 3 },
    { 0x50c364b, 5 },
    { 0, 0 },
    { 0x3bac364e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d103651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb103656, 3 },
    { 0x31ac3657, 4 },
    { 0, 0 },
    { 0xc9f83659, 5 },
    { 0, 0 },
    { 0xd09c365b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc365f, 4 },
    { 0xd6143660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x543663, 3 },
    { 0xa34c3664, 3 },
    { 0xe4f43664, 3 },
    { 0xd2043664, 4 },
    { 0x96f03665, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc944366c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f943672, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583679, 4 },
    { 0x8914367a, 4 },
    { 0xa018367a, 3 },
    { 0, 0 },
    { 0x5848367d, 4 },
    { 0xbc84367e, 3 },
    { 0x41e0367e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8883686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd03690, 3 },
    { 0xb67c3690, 3 },
    { 0xa3903691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc03695, 3 },
    { 0xaaa83695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734c369c, 4 },
    { 0x2a8c369d, 3 },
    { 0xd94369d, 3 },
    { 0, 0 },
    { 0xb6fc36a0, 3 },
    { 0xee2836a1, 4 },
    { 0xe3d036a2, 3 },
    { 0x332436a3, 4 },
    { 0xe3a436a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d436af, 4 },
    { 0, 0 },
    { 0x70e036b1, 4 },
    { 0xab0036b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f036b8, 3 },
    { 0, 0 },
    { 0x5a836ba, 3 },
    { 0x65ac36bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907436c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8836c8, 3 },
    { 0x44c836c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a436d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435436d7, 3 },
    { 0x1ecc36d7, 4 },
    { 0x149c36d8, 4 },
    { 0, 0 },
    { 0xd2c836db, 3 },
    { 0xd7b036dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4836e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbc36e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dc36e6, 3 },
    { 0x79bc36e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cac36ea, 3 },
    { 0xe21c36eb, 3 },
    { 0x74f436eb, 3 },
    { 0, 0 },
    { 0x1fc036ee, 4 },
    { 0xcc7436ef, 3 },
    { 0xafb836ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b036f4, 3 },
    { 0x8d9836f5, 3 },
    { 0xf6dc36f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293836fa, 4 },
    { 0x38a836fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9443700, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b83703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1083706, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6c3709, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e4370d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fc3719, 3 },
    { 0x83f4371a, 3 },
    { 0x4328371b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed83720, 4 },
    { 0x4e183721, 3 },
    { 0xd6783720, 3 },
    { 0x7e3c3723, 3 },
    { 0xa5543724, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95543727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7283733, 4 },
    { 0, 0 },
    { 0x91943735, 3 },
    { 0x41103735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34373a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb260373f, 4 },
    { 0x127c3740, 3 },
    { 0, 0 },
    { 0x74b83742, 3 },
    { 0, 0 },
    { 0xfd943744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8c3748, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0c374d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ec375a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec375f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef43766, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc548376a, 3 },
    { 0, 0 },
    { 0xcecc376c, 3 },
    { 0xa77c376c, 4 },
    { 0xb658376d, 3 },
    { 0, 0 },
    { 0xd6703770, 5 },
    { 0x9f383771, 4 },
    { 0x1dd83770, 3 },
    { 0x10b03773, 3 },
    { 0x34543773, 3 },
    { 0xed503771, 4 },
    { 0x8a5c3776, 4 },
    { 0xe4483776, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac377e, 3 },
    { 0xe2c4377e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee43784, 3 },
    { 0xc8843784, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e483788, 3 },
    { 0, 0 },
    { 0x6cac378a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e64378d, 4 },
    { 0xd88378e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c83791, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e83794, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fc3797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450379b, 4 },
    { 0x4b08379c, 3 },
    { 0x15b0379d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab837a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e437a5, 3 },
    { 0xde1837a5, 3 },
    { 0xf03c37a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x581837aa, 5 },
    { 0x4d1437aa, 3 },
    { 0x1e3437ac, 3 },
    { 0, 0 },
    { 0x833037ae, 3 },
    { 0xfbd037af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc37b6, 3 },
    { 0xeb7437b7, 3 },
    { 0x649837b8, 4 },
    { 0xc6f437b6, 3 },
    { 0xa4c837ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d037c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33437c6, 3 },
    { 0x756037c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19c37cb, 3 },
    { 0x94c437cc, 3 },
    { 0x687037cd, 3 },
    { 0xdf4837ce, 3 },
    { 0x505c37cf, 5 },
    { 0xa9437d0, 3 },
    { 0x182c37d0, 3 },
    { 0x457c37d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99037d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fc37da, 3 },
    { 0, 0 },
    { 0x676837dc, 3 },
    { 0x503037dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294837e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50437ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802037ef, 4 },
    { 0x941837f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e837f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8837f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b703800, 3 },
    { 0x19b43800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50c3804, 3 },
    { 0x6f483805, 4 },
    { 0x228c3805, 4 },
    { 0, 0 },
    { 0x7f943808, 3 },
    { 0x6dac3808, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6908380d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5243811, 4 },
    { 0xcd383812, 3 },
    { 0, 0 },
    { 0x7c3c3814, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d03819, 3 },
    { 0xa1a0381a, 4 },
    { 0, 0 },
    { 0x3bf4381c, 3 },
    { 0xf814381c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d83824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15503829, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe068382c, 3 },
    { 0xf49c382d, 3 },
    { 0, 0 },
    { 0xafd0382f, 4 },
    { 0x85a83830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1983837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd00383b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c44383f, 3 },
    { 0xe790383f, 3 },
    { 0x5fbc3841, 3 },
    { 0xa2283842, 4 },
    { 0xba9c3841, 3 },
    { 0x21143844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae03847, 3 },
    { 0xfae83848, 3 },
    { 0x61203849, 4 },
    { 0x264c384a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8384d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96683851, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a583855, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa858385c, 3 },
    { 0x2748385c, 3 },
    { 0, 0 },
    { 0x3540385f, 4 },
    { 0x28b4385f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc43864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7803867, 3 },
    { 0xb0003868, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac386c, 4 },
    { 0x1f3c386d, 3 },
    { 0xc4a8386e, 3 },
    { 0x99b4386f, 3 },
    { 0x2184386c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7103874, 3 },
    { 0, 0 },
    { 0x2e6c3876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bc3879, 4 },
    { 0x84e0387a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c387d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f43882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ec388f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec03893, 3 },
    { 0, 0 },
    { 0xcd7c3895, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b83898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246c389f, 3 },
    { 0xb7b0389f, 3 },
    { 0x9a5838a0, 3 },
    { 0x29c038a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867c38a7, 3 },
    { 0xd63438a8, 3 },
    { 0, 0 },
    { 0x518438aa, 3 },
    { 0, 0 },
    { 0xf45838ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f838b5, 3 },
    { 0x69ec38b6, 3 },
    { 0, 0 },
    { 0xb47838b8, 3 },
    { 0xabd438b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9438bc, 3 },
    { 0x7b0438bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85038c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc038c3, 3 },
    { 0x47ac38c4, 4 },
    { 0x56f438c5, 3 },
    { 0x983038c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36438d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c38d9, 4 },
    { 0xdf7838da, 4 },
    { 0, 0 },
    { 0x702438dc, 3 },
    { 0x73ec38dc, 3 },
    { 0x278c38de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c38e1, 3 },
    { 0, 0 },
    { 0xafe438e3, 3 },
    { 0, 0 },
    { 0xff4438e5, 3 },
    { 0x52b038e6, 3 },
    { 0x9bbc38e7, 4 },
    { 0x262038e7, 3 },
    { 0xcee438e9, 4 },
    { 0, 0 },
    { 0x40038eb, 3 },
    { 0x308438ec, 3 },
    { 0x229038ec, 3 },
    { 0x3d5838eb, 3 },
    { 0xf28838ef, 5 },
    { 0xd5cc38ed, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x97438f3, 3 },
    { 0, 0 },
    { 0x9ec038f5, 3 },
    { 0xc84838f5, 3 },
    { 0x61a438f6, 3 },
    { 0xc87038f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44dc3908, 4 },
    { 0x53343908, 3 },
    { 0xd80390a, 4 },
    { 0x1484390b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08390e, 3 },
    { 0xb5f4390f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11283914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2240391d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d8392a, 3 },
    { 0, 0 },
    { 0xaac4392c, 3 },
    { 0x7d8392d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c3934, 4 },
    { 0x90a43935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c503938, 3 },
    { 0, 0 },
    { 0xca0c393a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe04393d, 5 },
    { 0x653c393d, 3 },
    { 0xc3bc393d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x417c3942, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34394c, 4 },
    { 0xbac0394d, 3 },
    { 0x8acc394e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ec3951, 4 },
    { 0, 0 },
    { 0x34b03953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62603958, 3 },
    { 0, 0 },
    { 0x352c395a, 3 },
    { 0, 0 },
    { 0x8888395c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c98395f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f803967, 3 },
    { 0, 0 },
    { 0x774c3969, 3 },
    { 0x85183969, 3 },
    { 0xe150396b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5043970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4183976, 3 },
    { 0, 0 },
    { 0xc2fc3978, 3 },
    { 0xa9343979, 4 },
    { 0, 0 },
    { 0x8630397b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa48397e, 3 },
    { 0x642c397f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad783983, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a903986, 3 },
    { 0xb48c3986, 3 },
    { 0xfc983988, 3 },
    { 0x2a83989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b8398c, 3 },
    { 0, 0 },
    { 0x3450398e, 3 },
    { 0, 0 },
    { 0xf2ac3990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86603995, 4 },
    { 0xbce83995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4c3999, 3 },
    { 0x72803999, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc039a0, 3 },
    { 0, 0 },
    { 0x7bc439a2, 4 },
    { 0x350439a2, 3 },
    { 0x935439a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368839a9, 3 },
    { 0x4b6c39aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a839ae, 3 },
    { 0xeae439af, 3 },
    { 0xec6c39ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19039b5, 3 },
    { 0x2bf439b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36439bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530839c0, 3 },
    { 0xcb5039c1, 3 },
    { 0x649839c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43839c7, 3 },
    { 0x52e439c7, 3 },
    { 0x81ac39c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d039cc, 3 },
    { 0xe4a039cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de039d3, 3 },
    { 0x3ea839d3, 3 },
    { 0x815439d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacac39db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de039de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2839e4, 3 },
    { 0, 0 },
    { 0x725839e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ec39ef, 3 },
    { 0xa6e039ef, 4 },
    { 0, 0 },
    { 0x808839f2, 3 },
    { 0xafb839f3, 3 },
    { 0x48c039f4, 4 },
    { 0, 0 },
    { 0xff8439f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60039fc, 4 },
    { 0x426439fd, 3 },
    { 0x26d439fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f743a01, 3 },
    { 0, 0 },
    { 0x32a03a03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d83a0a, 3 },
    { 0x2dfc3a0b, 4 },
    { 0x61683a0c, 4 },
    { 0xddf03a0d, 4 },
    { 0xb103a0e, 3 },
    { 0x246c3a0b, 3 },
    { 0xc6383a10, 4 },
    { 0x31f83a0e, 3 },
    { 0x119c3a0c, 3 },
    { 0, 0 },
    { 0xfcdc3a14, 3 },
    { 0x996c3a15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5c3a19, 3 },
    { 0, 0 },
    { 0x52483a1b, 3 },
    { 0x1f143a1c, 3 },
    { 0xc6f43a1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d83a21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b303a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5943a29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce783a2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5043a36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccc3a3d, 3 },
    { 0x56fc3a3d, 4 },
    { 0x8d203a3f, 3 },
    { 0xa4083a3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x70843a43, 3 },
    { 0xaf0c3a44, 3 },
    { 0xb73c3a44, 4 },
    { 0, 0 },
    { 0x94003a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71303a53, 4 },
    { 0, 0 },
    { 0xf73c3a55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed43a5b, 3 },
    { 0, 0 },
    { 0x6f743a5d, 3 },
    { 0x7f483a5e, 3 },
    { 0x14dc3a5f, 4 },
    { 0x26543a5f, 4 },
    { 0x22083a5f, 3 },
    { 0, 0 },
    { 0x2d043a63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaac3a6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74183a70, 3 },
    { 0x4ad83a71, 3 },
    { 0, 0 },
    { 0x71843a73, 3 },
    { 0xa6b83a74, 3 },
    { 0, 0 },
    { 0x8e703a76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac03a80, 3 },
    { 0x82f43a80, 3 },
    { 0x5b243a81, 3 },
    { 0, 0 },
    { 0xf95c3a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f983a87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce083a8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b983a8f, 3 },
    { 0xd2e43a8f, 3 },
    { 0x3e303a91, 3 },
    { 0x31a03a90, 3 },
    { 0xb1483a93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bc3a97, 3 },
    { 0x98b43a98, 4 },
    { 0x7e3c3a98, 3 },
    { 0xda583a9a, 3 },
    { 0xaac83a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xded83a9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe103aa1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadc3aa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f983aa7, 3 },
    { 0, 0 },
    { 0x2c3aa9, 3 },
    { 0x2cac3aa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56743ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c03ab8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde983abc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d743abf, 3 },
    { 0x41a83ac0, 3 },
    { 0x5bec3ac1, 3 },
    { 0x5d803ac2, 4 },
    { 0x49e03ac0, 5 },
    { 0x1d383abf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x160c3ac7, 3 },
    { 0x77803ac7, 4 },
    { 0x639c3ac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8c3ace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2203ad1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7803ad7, 3 },
    { 0xaec43ad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a703add, 5 },
    { 0, 0 },
    { 0xb6a03adf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3803ae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7c3ae8, 3 },
    { 0, 0 },
    { 0x522c3aea, 4 },
    { 0x53fc3aeb, 3 },
    { 0x56243aec, 3 },
    { 0, 0 },
    { 0x51e83aee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bc3af3, 3 },
    { 0xc8003af3, 5 },
    { 0xc1783af4, 3 },
    { 0x63c03af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5743af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8203afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30703b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57643b09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dc3b10, 4 },
    { 0x37a03b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec243b15, 3 },
    { 0x2d6c3b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc3b1a, 4 },
    { 0, 0 },
    { 0x213c3b1c, 3 },
    { 0, 0 },
    { 0x74983b1e, 3 },
    { 0x220c3b1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52003b22, 3 },
    { 0x26e03b23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0983b26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af03b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c3b30, 3 },
    { 0xb3143b31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f43b36, 3 },
    { 0xbb083b37, 3 },
    { 0xf9543b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa03b3b, 3 },
    { 0x5a1c3b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0483b3f, 3 },
    { 0, 0 },
    { 0x9643b41, 5 },
    { 0, 0 },
    { 0xf043b43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda483b46, 4 },
    { 0x1ffc3b47, 3 },
    { 0xc4dc3b47, 3 },
    { 0, 0 },
    { 0x6b283b4a, 3 },
    { 0xc5843b4b, 3 },
    { 0xa40c3b4c, 3 },
    { 0xfed43b4d, 3 },
    { 0xa4743b4e, 4 },
    { 0x1e1c3b4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231c3b54, 3 },
    { 0xba543b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de83b59, 4 },
    { 0x44c83b5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f83b5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b503b61, 3 },
    { 0x3ba03b62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb443b65, 4 },
    { 0xfc8c3b66, 3 },
    { 0x19c43b65, 3 },
    { 0, 0 },
    { 0x50c3b69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a03b6e, 4 },
    { 0x29083b6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1883b7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65483b7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b03b81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22243b85, 4 },
    { 0, 0 },
    { 0xc9703b87, 4 },
    { 0x20fc3b88, 3 },
    { 0x47bc3b88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefc3b8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfc3b95, 3 },
    { 0x3d643b95, 4 },
    { 0xdd583b97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1583b9f, 3 },
    { 0, 0 },
    { 0x848c3ba1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4c3ba4, 3 },
    { 0, 0 },
    { 0x36203ba6, 3 },
    { 0x23a03ba6, 3 },
    { 0xac543ba8, 3 },
    { 0xbc283ba8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac03bae, 3 },
    { 0x4b8c3baf, 3 },
    { 0xee003bae, 3 },
    { 0, 0 },
    { 0x2a583bb2, 5 },
    { 0x26043bb3, 3 },
    { 0x532c3bb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8c3bc1, 3 },
    { 0, 0 },
    { 0xbc443bc3, 4 },
    { 0xd33c3bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf43bcd, 4 },
    { 0x64483bce, 3 },
    { 0x32e83bce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x816c3bd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad243bd9, 3 },
    { 0, 0 },
    { 0xc1143bdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d43bdf, 3 },
    { 0, 0 },
    { 0x12303be1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa03be4, 3 },
    { 0, 0 },
    { 0x83b43be6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf883bea, 4 },
    { 0x72dc3beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x117c3bee, 3 },
    { 0x7ce03bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28543bf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149c3bf7, 3 },
    { 0x76c3bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9943bfb, 4 },
    { 0x83fc3bfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c3bff, 4 },
    { 0xd0943c00, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5983c04, 3 },
    { 0, 0 },
    { 0x40903c06, 3 },
    { 0, 0 },
    { 0xf59c3c08, 3 },
    { 0xe88c3c09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e043c0d, 4 },
    { 0xb57c3c0e, 3 },
    { 0x1ce83c0f, 4 },
    { 0x2d5c3c10, 3 },
    { 0xa4383c11, 3 },
    { 0x399c3c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c343c18, 3 },
    { 0xa2043c19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76743c21, 3 },
    { 0, 0 },
    { 0x9e143c23, 3 },
    { 0x37903c24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342c3c28, 3 },
    { 0x3e843c29, 3 },
    { 0x4283c2a, 3 },
    { 0x405c3c2b, 4 },
    { 0x5af03c2c, 3 },
    { 0x645c3c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3843c34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75083c38, 3 },
    { 0x5a9c3c38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5003c3c, 3 },
    { 0, 0 },
    { 0xde2c3c3e, 3 },
    { 0x2283c3f, 3 },
    { 0, 0 },
    { 0xa603c41, 3 },
    { 0xff483c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e83c46, 3 },
    { 0, 0 },
    { 0xfbb43c48, 3 },
    { 0x80143c49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0743c4d, 3 },
    { 0xfc103c4e, 3 },
    { 0x16843c4f, 3 },
    { 0x5d2c3c50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c03c58, 3 },
    { 0xaa683c59, 3 },
    { 0x853c3c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51483c60, 4 },
    { 0xc5343c60, 4 },
    { 0xb6e43c62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe43c65, 3 },
    { 0x42d03c66, 4 },
    { 0xaf243c65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c83c6a, 4 },
    { 0x43b83c6b, 3 },
    { 0x10c03c6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d803c70, 3 },
    { 0xccf83c71, 3 },
    { 0x60583c72, 3 },
    { 0xcac83c70, 5 },
    { 0, 0 },
    { 0xd2443c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8c3c79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb83c83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x46703c86, 3 },
    { 0, 0 },
    { 0x7ce43c88, 3 },
    { 0x3a783c89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d43c8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c3c8f, 3 },
    { 0x9b183c90, 3 },
    { 0x4543c91, 4 },
    { 0x410c3c8f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33143c9a, 3 },
    { 0xdff43c9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb03ca3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4143caa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b83cb0, 3 },
    { 0xd6543cb1, 3 },
    { 0x950c3cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb883cb5, 4 },
    { 0xf9a03cb5, 3 },
    { 0x68683cb7, 3 },
    { 0, 0 },
    { 0xff83cb9, 3 },
    { 0x2fe43cb9, 3 },
    { 0, 0 },
    { 0xc5383cbc, 5 },
    { 0x2fa03cbc, 3 },
    { 0xec683cbe, 5 },
    { 0, 0 },
    { 0xcdfc3cc0, 4 },
    { 0x16143cc1, 3 },
    { 0, 0 },
    { 0xec283cc3, 4 },
    { 0xb19c3cc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4143cc7, 4 },
    { 0, 0 },
    { 0x471c3cc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44903cd2, 3 },
    { 0, 0 },
    { 0xb3283cd4, 3 },
    { 0xa7003cd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b43cda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14383cde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fac3ce5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60103ceb, 4 },
    { 0, 0 },
    { 0x21083ced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc3cf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x470c3cf7, 4 },
    { 0xf2003cf8, 4 },
    { 0xd3a83cf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f43cfc, 5 },
    { 0x2a143cfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e643d00, 3 },
    { 0xefa83d00, 3 },
    { 0x92d43d02, 3 },
    { 0x7e283d03, 3 },
    { 0xc9ec3d02, 3 },
    { 0xd2643d05, 4 },
    { 0x20ac3d04, 3 },
    { 0, 0 },
    { 0xc33c3d08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52503d0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc3d0f, 3 },
    { 0x76583d10, 3 },
    { 0xf7443d10, 3 },
    { 0xbf003d12, 4 },
    { 0xf0b43d13, 3 },
    { 0, 0 },
    { 0xfff43d15, 3 },
    { 0, 0 },
    { 0xad183d17, 4 },
    { 0x58b43d17, 3 },
    { 0xf2a83d18, 3 },
    { 0x8be83d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7003d1d, 4 },
    { 0x67583d1e, 4 },
    { 0, 0 },
    { 0xc6883d20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d003d27, 3 },
    { 0xd5783d28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16343d2c, 3 },
    { 0, 0 },
    { 0x41403d2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fc3d34, 4 },
    { 0x48d83d34, 3 },
    { 0x6a43d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbc3d3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc643d50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad43d55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c3d5c, 3 },
    { 0, 0 },
    { 0x48303d5e, 3 },
    { 0x27603d5f, 3 },
    { 0x24ac3d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03d67, 4 },
    { 0x25343d68, 4 },
    { 0xa3c83d67, 3 },
    { 0, 0 },
    { 0x2f9c3d6b, 4 },
    { 0x447c3d6c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df43d6f, 3 },
    { 0x98fc3d6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0403d76, 3 },
    { 0xafd83d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc3d7c, 3 },
    { 0x49343d7d, 3 },
    { 0x52783d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c3d82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92383d88, 5 },
    { 0, 0 },
    { 0x45443d8a, 3 },
    { 0x5a2c3d8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c3d8e, 3 },
    { 0xc6e03d8e, 3 },
    { 0x8d2c3d90, 4 },
    { 0x8fc03d91, 3 },
    { 0xa2043d92, 3 },
    { 0x5b0c3d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3d97, 5 },
    { 0, 0 },
    { 0xe25c3d99, 3 },
    { 0x8f1c3d9a, 3 },
    { 0, 0 },
    { 0x15083d9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1003da1, 3 },
    { 0xf8543da2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc343dab, 3 },
    { 0x5bd83dac, 4 },
    { 0x59a43dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f03db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d83db7, 3 },
    { 0xfbd03db8, 3 },
    { 0xf2783db9, 4 },
    { 0, 0 },
    { 0x6e283dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46883dbf, 3 },
    { 0xd4303dbf, 3 },
    { 0x1b483dc0, 3 },
    { 0x482c3dc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc83dcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46183dd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73c3dd7, 3 },
    { 0x785c3dd8, 3 },
    { 0, 0 },
    { 0x31343dda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a583de0, 3 },
    { 0xe9383de1, 3 },
    { 0x9c783de2, 3 },
    { 0, 0 },
    { 0x876c3de4, 3 },
    { 0, 0 },
    { 0x680c3de6, 3 },
    { 0x4aac3de6, 4 },
    { 0x1d03de8, 4 },
    { 0x64143de8, 4 },
    { 0xddf43de7, 3 },
    { 0xa0f43de7, 3 },
    { 0xcb783dec, 4 },
    { 0, 0 },
    { 0xc66c3dee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10603df3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b03df7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e003dfa, 3 },
    { 0, 0 },
    { 0xbab43dfc, 4 },
    { 0x651c3dfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa703e02, 3 },
    { 0, 0 },
    { 0x1fc43e04, 3 },
    { 0, 0 },
    { 0x4c2c3e06, 3 },
    { 0x369c3e07, 4 },
    { 0, 0 },
    { 0x94043e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcc3e0e, 3 },
    { 0, 0 },
    { 0x4803e10, 3 },
    { 0, 0 },
    { 0x3bc3e12, 4 },
    { 0xd2883e13, 4 },
    { 0xda883e14, 4 },
    { 0, 0 },
    { 0xbfa43e16, 3 },
    { 0x60683e16, 3 },
    { 0xdef43e16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f83e1c, 3 },
    { 0xdca03e1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0403e21, 4 },
    { 0x56b83e21, 4 },
    { 0x96783e21, 3 },
    { 0xf71c3e23, 3 },
    { 0xf3ec3e25, 5 },
    { 0, 0 },
    { 0xa9143e27, 3 },
    { 0, 0 },
    { 0x47703e29, 3 },
    { 0, 0 },
    { 0xf0043e2b, 3 },
    { 0xa80c3e2b, 4 },
    { 0xc25c3e2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a43e31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d83e39, 3 },
    { 0x14783e39, 4 },
    { 0x40083e3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583e3e, 3 },
    { 0, 0 },
    { 0x112c3e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70983e43, 4 },
    { 0x317c3e44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c03e4c, 3 },
    { 0xec283e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf683e50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ac3e58, 3 },
    { 0x4e443e59, 3 },
    { 0x4a83e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21483e5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c43e63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfc3e67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a83e6a, 5 },
    { 0, 0 },
    { 0x6ddc3e6c, 3 },
    { 0x58cc3e6d, 3 },
    { 0, 0 },
    { 0xb9803e6f, 4 },
    { 0x4da43e6f, 3 },
    { 0, 0 },
    { 0xe5b03e72, 3 },
    { 0x93d03e73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23943e7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64083e81, 3 },
    { 0, 0 },
    { 0x75683e83, 4 },
    { 0x56783e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c903e8c, 4 },
    { 0xf0dc3e8c, 4 },
    { 0, 0 },
    { 0x715c3e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26243e92, 4 },
    { 0, 0 },
    { 0x6cc43e94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c743e99, 4 },
    { 0x549c3e99, 4 },
    { 0, 0 },
    { 0x78b43e9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb83e9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca83ea3, 3 },
    { 0, 0 },
    { 0x6ad43ea5, 4 },
    { 0x194c3ea5, 3 },
    { 0x82e43ea7, 3 },
    { 0, 0 },
    { 0xb5903ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e903eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc3eaf, 4 },
    { 0x54703eb0, 5 },
    { 0, 0 },
    { 0xb15c3eb2, 3 },
    { 0, 0 },
    { 0xc18c3eb4, 4 },
    { 0x4b8c3eb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6183ebc, 4 },
    { 0x57643ebc, 4 },
    { 0xc0003ebe, 4 },
    { 0x908c3ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13143ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64303ec5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27183eca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c3ed1, 3 },
    { 0x1bac3ed2, 3 },
    { 0xbeac3ed1, 3 },
    { 0x5ef03ed4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4243ed7, 3 },
    { 0xc7043ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b03edb, 4 },
    { 0, 0 },
    { 0x29fc3edd, 3 },
    { 0xc7703ede, 4 },
    { 0xe5343edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac83ee2, 3 },
    { 0, 0 },
    { 0xcf243ee4, 3 },
    { 0xf3b43ee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c483ee8, 3 },
    { 0x929c3ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52843eed, 4 },
    { 0, 0 },
    { 0x60f43eef, 3 },
    { 0x55e43ef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c3ef3, 4 },
    { 0x1a703ef4, 3 },
    { 0x27703ef3, 3 },
    { 0x70ac3ef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30c3ef9, 3 },
    { 0xc4103efa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51283f00, 4 },
    { 0, 0 },
    { 0x95f43f02, 4 },
    { 0x1df03f02, 3 },
    { 0x51383f04, 3 },
    { 0, 0 },
    { 0xcca03f06, 3 },
    { 0x11443f07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd503f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c3f0d, 3 },
    { 0x55e83f0e, 4 },
    { 0, 0 },
    { 0x4fdc3f10, 4 },
    { 0x76743f10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c443f16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19583f19, 3 },
    { 0xc1b83f1a, 3 },
    { 0x21183f1b, 3 },
    { 0x78403f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a683f21, 4 },
    { 0xe8a83f22, 3 },
    { 0, 0 },
    { 0x56743f24, 4 },
    { 0, 0 },
    { 0xbc3f26, 3 },
    { 0, 0 },
    { 0x5a743f28, 3 },
    { 0, 0 },
    { 0xa8943f2a, 3 },
    { 0, 0 },
    { 0x26d43f2c, 3 },
    { 0xff43f2d, 3 },
    { 0, 0 },
    { 0x1303f2f, 3 },
    { 0, 0 },
    { 0x161c3f31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c83f34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc3f37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e43f3a, 3 },
    { 0, 0 },
    { 0x9f203f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91083f40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2843f43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f543f46, 4 },
    { 0x11443f46, 3 },
    { 0, 0 },
    { 0xbae83f49, 3 },
    { 0xb7c03f4a, 4 },
    { 0xc3f83f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x580c3f4e, 3 },
    { 0x2e303f4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f83f53, 3 },
    { 0, 0 },
    { 0x35443f55, 3 },
    { 0xd8683f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4203f5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c903f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b43f6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc3f70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5643f73, 3 },
    { 0, 0 },
    { 0xb2903f75, 3 },
    { 0x1fe43f76, 4 },
    { 0x3db83f77, 4 },
    { 0xb3043f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3f7c, 3 },
    { 0x3fa03f7d, 3 },
    { 0xba6c3f7e, 3 },
    { 0, 0 },
    { 0x9bd43f80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca43f85, 3 },
    { 0, 0 },
    { 0x12683f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6803f8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23203f8f, 3 },
    { 0x7a583f90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b43f94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab83f97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5683f9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1803fa8, 3 },
    { 0x69343fa9, 4 },
    { 0xe30c3faa, 4 },
    { 0x89803fab, 3 },
    { 0xbe283fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec43fb2, 3 },
    { 0x44ec3fb3, 3 },
    { 0, 0 },
    { 0xd8a03fb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc03fbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e43fc0, 3 },
    { 0xae5c3fc1, 3 },
    { 0xb86c3fc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4083fc5, 4 },
    { 0xecc43fc6, 3 },
    { 0, 0 },
    { 0xf1483fc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b83fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e43fd0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e83fd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a483fdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea603fe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80183fe5, 3 },
    { 0x3a503fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdc3fec, 3 },
    { 0x215c3fed, 3 },
    { 0x98243fee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70083ff1, 4 },
    { 0x82b43ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f83ff8, 4 },
    { 0x7ff43ff9, 3 },
    { 0x40343ffa, 4 },
    { 0x99443ffb, 3 },
    { 0x27103ff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b84002, 3 },
    { 0x5b084002, 3 },
    { 0, 0 },
    { 0x76f44005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3504008, 3 },
    { 0xa6c4009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d20400f, 3 },
    { 0x1d34400f, 3 },
    { 0x80384011, 3 },
    { 0x54544012, 3 },
    { 0xa7444011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e4401c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c401f, 3 },
    { 0x940c4020, 3 },
    { 0x6ac04021, 3 },
    { 0x74d84022, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeec4025, 3 },
    { 0xaf644026, 3 },
    { 0x85444027, 3 },
    { 0xc9044028, 3 },
    { 0x50304029, 3 },
    { 0xff704027, 4 },
    { 0, 0 },
    { 0x666c402c, 3 },
    { 0, 0 },
    { 0x85c4402e, 3 },
    { 0x286c402f, 3 },
    { 0x750c402f, 3 },
    { 0xa730402e, 3 },
    { 0, 0 },
    { 0xf9544033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57304036, 4 },
    { 0x86204037, 3 },
    { 0x32284038, 3 },
    { 0xbcd04038, 4 },
    { 0x2b44403a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a8403f, 3 },
    { 0, 0 },
    { 0x1fc04041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83c4046, 3 },
    { 0x95404047, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699c404b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29c404e, 3 },
    { 0xe424404f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02c4057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed64405a, 5 },
    { 0xa0ec405a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be4405f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c4069, 3 },
    { 0xe740406a, 3 },
    { 0, 0 },
    { 0x7934406c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c80407b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba84080, 3 },
    { 0x17444081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6258408d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d84094, 3 },
    { 0xfbfc4095, 3 },
    { 0xd1e84094, 3 },
    { 0, 0 },
    { 0x5d84098, 4 },
    { 0x7df04098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12840a1, 3 },
    { 0xe9e440a1, 3 },
    { 0x47f840a3, 3 },
    { 0x912840a4, 3 },
    { 0x35440a2, 3 },
    { 0xe0b840a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7840a9, 3 },
    { 0x5f8040aa, 3 },
    { 0, 0 },
    { 0x6d6040ac, 3 },
    { 0xcd7c40ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdc40b7, 4 },
    { 0x308c40b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2440bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1440bf, 3 },
    { 0xe08c40bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db040c6, 3 },
    { 0xa39440c6, 3 },
    { 0xc39840c8, 4 },
    { 0x30f440c9, 3 },
    { 0x3fe040c7, 3 },
    { 0x2e4840c9, 3 },
    { 0xf00840cb, 3 },
    { 0xe9a040ca, 4 },
    { 0x558840c9, 4 },
    { 0, 0 },
    { 0xb87c40d0, 3 },
    { 0x88f440d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252040da, 3 },
    { 0x413040db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15040df, 3 },
    { 0x86b440e0, 4 },
    { 0xda3c40e0, 3 },
    { 0xd25440e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4440e5, 3 },
    { 0, 0 },
    { 0xe3c840e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b440ea, 3 },
    { 0, 0 },
    { 0x5c9040ec, 4 },
    { 0xe12840ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63440f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459840f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d840fb, 3 },
    { 0, 0 },
    { 0x663840fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x806c4100, 3 },
    { 0x29b04100, 3 },
    { 0, 0 },
    { 0x62ec4103, 3 },
    { 0x8fa04104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c410c, 3 },
    { 0, 0 },
    { 0x492c410e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f284113, 3 },
    { 0xd7e84114, 3 },
    { 0x8a644115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77144119, 3 },
    { 0, 0 },
    { 0xffe8411b, 4 },
    { 0x7e14411c, 3 },
    { 0x5500411d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04120, 3 },
    { 0, 0 },
    { 0x8c104122, 4 },
    { 0x7b044123, 3 },
    { 0, 0 },
    { 0x8af44125, 3 },
    { 0xa2144126, 3 },
    { 0x38fc4126, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd04130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3c4133, 3 },
    { 0xa7144133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc4138, 3 },
    { 0xf4c04139, 3 },
    { 0xcbe4413a, 4 },
    { 0xb590413b, 4 },
    { 0, 0 },
    { 0x40d0413d, 3 },
    { 0xd68c413e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7904142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb404145, 3 },
    { 0, 0 },
    { 0x9ca84147, 3 },
    { 0xbcd44147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef00414d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa44152, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2168415a, 4 },
    { 0x9a48415b, 5 },
    { 0, 0 },
    { 0xa96c415d, 4 },
    { 0xdcd0415e, 3 },
    { 0xc728415f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1984162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11144166, 3 },
    { 0, 0 },
    { 0x16044168, 4 },
    { 0, 0 },
    { 0xb7f0416a, 3 },
    { 0xb184416b, 3 },
    { 0, 0 },
    { 0x1ae8416d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d144174, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2690417a, 3 },
    { 0xbf7c417b, 3 },
    { 0x6f0c417b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fc417f, 3 },
    { 0x32fc417f, 3 },
    { 0xf3684180, 4 },
    { 0, 0 },
    { 0x3be44183, 3 },
    { 0, 0 },
    { 0xcba04185, 3 },
    { 0, 0 },
    { 0x2a904187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04418f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f844196, 3 },
    { 0, 0 },
    { 0x76744198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258441a1, 4 },
    { 0x654c41a1, 3 },
    { 0x36b041a3, 3 },
    { 0, 0 },
    { 0xa66841a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f841aa, 3 },
    { 0xe54441aa, 3 },
    { 0x153841ac, 3 },
    { 0xe7b841ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322c41b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c41ba, 4 },
    { 0, 0 },
    { 0x199441bc, 3 },
    { 0x552c41bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e841c1, 3 },
    { 0, 0 },
    { 0x17b841c3, 3 },
    { 0x283841c4, 3 },
    { 0x7f1841c4, 3 },
    { 0xed5c41c4, 3 },
    { 0x105c41c7, 3 },
    { 0x2c2841c8, 3 },
    { 0x3db441c8, 3 },
    { 0, 0 },
    { 0x9df841cb, 4 },
    { 0x90041cb, 3 },
    { 0xbbd441cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff441d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3c41d6, 3 },
    { 0, 0 },
    { 0x5da841d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c41dc, 3 },
    { 0xbf8041dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x576c41e0, 3 },
    { 0, 0 },
    { 0xa20841e2, 3 },
    { 0, 0 },
    { 0xb3d041e4, 3 },
    { 0x20f041e5, 3 },
    { 0x2f2841e6, 3 },
    { 0x8d1c41e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb841f0, 3 },
    { 0x985041f1, 3 },
    { 0, 0 },
    { 0x710c41f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc41fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55744202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7258420e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab84211, 3 },
    { 0xfbe44212, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f784216, 3 },
    { 0, 0 },
    { 0x13504218, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778421f, 3 },
    { 0x5e444220, 3 },
    { 0xd0c04221, 4 },
    { 0x8558421f, 3 },
    { 0x651c4223, 4 },
    { 0x354c4223, 3 },
    { 0x286c4224, 3 },
    { 0x8cb04225, 4 },
    { 0x80444227, 4 },
    { 0xd0ec4220, 3 },
    { 0xad544229, 3 },
    { 0x60e04229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa84231, 3 },
    { 0xfcbc4231, 4 },
    { 0, 0 },
    { 0xdb4c4234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588423c, 3 },
    { 0, 0 },
    { 0xd538423e, 4 },
    { 0x5df8423f, 3 },
    { 0xdf54423f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe744249, 3 },
    { 0xefec424a, 3 },
    { 0, 0 },
    { 0x7e74424c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75904250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ec4254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544257, 3 },
    { 0xc09c4258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454425c, 3 },
    { 0xbb10425c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b844260, 3 },
    { 0xd3b44261, 3 },
    { 0x126c4262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3604267, 3 },
    { 0x70a84267, 3 },
    { 0xa8dc4269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3360426c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d444270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf444274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2994427c, 3 },
    { 0, 0 },
    { 0x5bd0427e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a8428c, 3 },
    { 0x11c428d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68304290, 4 },
    { 0x99204291, 4 },
    { 0xcfa44292, 4 },
    { 0xbf8c4291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0284297, 5 },
    { 0x70a84298, 3 },
    { 0xcff84299, 3 },
    { 0x354429a, 4 },
    { 0x11c4298, 3 },
    { 0, 0 },
    { 0x1bdc429d, 3 },
    { 0x7d04429d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88442a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0c42a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755842b5, 4 },
    { 0x68ec42b6, 3 },
    { 0x29f042b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01442ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be042bd, 4 },
    { 0x765c42be, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc42c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74842c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x455442c8, 5 },
    { 0x54a442c9, 3 },
    { 0xd53442ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c42d2, 3 },
    { 0x1e2842d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c42d9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c42e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e842e7, 4 },
    { 0, 0 },
    { 0xe51842e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c442ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb842f2, 3 },
    { 0, 0 },
    { 0xfd1c42f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d842fc, 3 },
    { 0xd5fc42fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e84303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebac430e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89404311, 3 },
    { 0, 0 },
    { 0x8d9c4313, 3 },
    { 0x17084313, 3 },
    { 0x43dc4313, 4 },
    { 0xb5f04313, 3 },
    { 0x4df44314, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec90431a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fc4320, 4 },
    { 0x3d604321, 3 },
    { 0x7be44321, 4 },
    { 0x95444323, 4 },
    { 0x3d9c4320, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd44328, 3 },
    { 0xd0244329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a44330, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf944337, 3 },
    { 0x25204338, 3 },
    { 0x40044339, 3 },
    { 0x8fac433a, 3 },
    { 0, 0 },
    { 0x8cd4433c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f844340, 3 },
    { 0x266c4341, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51c4345, 3 },
    { 0x85484346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4094434a, 3 },
    { 0, 0 },
    { 0xd9e8434c, 3 },
    { 0, 0 },
    { 0x79a0434e, 3 },
    { 0x8e10434f, 3 },
    { 0x44504350, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83844355, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c30435a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1380435e, 3 },
    { 0x7580435f, 4 },
    { 0xb7584360, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941c4365, 4 },
    { 0xefc4365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3278436b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe150436f, 3 },
    { 0x3c2c4370, 4 },
    { 0x2d784370, 3 },
    { 0xba2c4372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd104375, 3 },
    { 0, 0 },
    { 0xf6884377, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7900437b, 3 },
    { 0x3ae4437c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48944388, 3 },
    { 0, 0 },
    { 0xf068438a, 3 },
    { 0x7dc438b, 3 },
    { 0x5ca0438c, 3 },
    { 0, 0 },
    { 0x5734438e, 5 },
    { 0xcf44438f, 4 },
    { 0, 0 },
    { 0x3af04391, 3 },
    { 0, 0 },
    { 0xaf6c4393, 3 },
    { 0, 0 },
    { 0x26dc4395, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x48504398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf74439c, 3 },
    { 0, 0 },
    { 0x6fac439e, 4 },
    { 0, 0 },
    { 0x37d843a0, 3 },
    { 0xd8843a0, 4 },
    { 0x2b6c43a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9843aa, 3 },
    { 0x7d9843ab, 3 },
    { 0, 0 },
    { 0xe98043ad, 3 },
    { 0x103043ae, 3 },
    { 0, 0 },
    { 0x15f043b0, 3 },
    { 0xf99843b1, 3 },
    { 0xacf843b2, 3 },
    { 0x8a6043b3, 3 },
    { 0xffbc43b2, 4 },
    { 0xb34443b5, 3 },
    { 0x835043b5, 3 },
    { 0xb1fc43b5, 3 },
    { 0x9b6843b5, 4 },
    { 0x91e443b3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc43bd, 3 },
    { 0, 0 },
    { 0xc77843bf, 3 },
    { 0, 0 },
    { 0xacb043c1, 3 },
    { 0xe20c43c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba843ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13c43ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf043d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1c43e1, 3 },
    { 0, 0 },
    { 0x6ae043e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95c43e6, 3 },
    { 0, 0 },
    { 0x572043e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea843eb, 3 },
    { 0, 0 },
    { 0x373443ed, 3 },
    { 0, 0 },
    { 0xf79c43ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109043f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c43fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc4404, 3 },
    { 0, 0 },
    { 0xb9b84406, 3 },
    { 0, 0 },
    { 0x5d944408, 3 },
    { 0, 0 },
    { 0x312c440a, 3 },
    { 0xe27c440b, 3 },
    { 0xbe90440c, 4 },
    { 0xff04440d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b04412, 3 },
    { 0, 0 },
    { 0xfdd44414, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bac441a, 3 },
    { 0x41b0441a, 3 },
    { 0xfe48441a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6244420, 3 },
    { 0x1e344421, 3 },
    { 0, 0 },
    { 0x50b84423, 4 },
    { 0xce1c4423, 3 },
    { 0x213c4425, 5 },
    { 0x494c4425, 4 },
    { 0, 0 },
    { 0x79ac4428, 3 },
    { 0xc9a84429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d384432, 4 },
    { 0x54244433, 3 },
    { 0x89244432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f84439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531c4440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c04446, 3 },
    { 0, 0 },
    { 0xa5844448, 5 },
    { 0x20884449, 3 },
    { 0x9f0c4449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27044453, 3 },
    { 0, 0 },
    { 0xc9584455, 4 },
    { 0xe9bc4456, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc4459, 3 },
    { 0x56f44459, 3 },
    { 0x1730445b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15304461, 3 },
    { 0, 0 },
    { 0x7284463, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77c4469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c446d, 3 },
    { 0x20d4446d, 4 },
    { 0xf150446d, 3 },
    { 0xf5a4446d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c4478, 3 },
    { 0, 0 },
    { 0xd5a8447a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a004482, 3 },
    { 0x5a3c4483, 4 },
    { 0x8e7c4484, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b44487, 3 },
    { 0, 0 },
    { 0xa3244489, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4448e, 3 },
    { 0xc4b4448f, 3 },
    { 0x7940448e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0449c, 3 },
    { 0, 0 },
    { 0x5e38449e, 3 },
    { 0x17e0449f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc844a3, 3 },
    { 0, 0 },
    { 0x5ff044a5, 3 },
    { 0x41a844a5, 4 },
    { 0x634444a7, 3 },
    { 0xb1d844a7, 3 },
    { 0x34b444a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164c44ad, 4 },
    { 0x708844ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de844b7, 5 },
    { 0, 0 },
    { 0x1cfc44b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38044bc, 3 },
    { 0x6ad844bd, 4 },
    { 0x691844be, 3 },
    { 0xf6f044be, 3 },
    { 0xde4c44c0, 3 },
    { 0xba9444c1, 4 },
    { 0x98a044c1, 3 },
    { 0, 0 },
    { 0x1d1044c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436c44ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c44cd, 3 },
    { 0x8cf844ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac44d2, 3 },
    { 0, 0 },
    { 0xd04844d4, 3 },
    { 0xb2e044d4, 4 },
    { 0x894044d5, 3 },
    { 0x649444d6, 3 },
    { 0x504c44d6, 3 },
    { 0x927844d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8044e1, 3 },
    { 0, 0 },
    { 0xfd1044e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fc44ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642844f0, 3 },
    { 0, 0 },
    { 0xa83844f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363444f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9c44f9, 3 },
    { 0, 0 },
    { 0xfd444fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7444fe, 3 },
    { 0xc4fc44ff, 3 },
    { 0, 0 },
    { 0x5dc44501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b84508, 5 },
    { 0, 0 },
    { 0x684c450a, 3 },
    { 0x96dc450b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d50450f, 3 },
    { 0, 0 },
    { 0x4b9c4511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d04516, 3 },
    { 0x47d04516, 3 },
    { 0x64304517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea8451d, 4 },
    { 0, 0 },
    { 0xbcbc451f, 3 },
    { 0x91844520, 3 },
    { 0xee7c4520, 3 },
    { 0, 0 },
    { 0xf5d04523, 3 },
    { 0x3db84524, 3 },
    { 0x75a84525, 3 },
    { 0x90144524, 4 },
    { 0, 0 },
    { 0xb5f04528, 3 },
    { 0x5ae44528, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c452d, 3 },
    { 0x726c452e, 3 },
    { 0x2b20452f, 3 },
    { 0x62e4452e, 3 },
    { 0xbf54452f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b704534, 3 },
    { 0, 0 },
    { 0xb5c4536, 3 },
    { 0, 0 },
    { 0x79d84538, 3 },
    { 0x4d044539, 4 },
    { 0, 0 },
    { 0x68fc453b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afc4545, 4 },
    { 0x39804545, 3 },
    { 0xac844545, 3 },
    { 0xba544548, 3 },
    { 0x77784549, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c0454e, 3 },
    { 0x39c0454f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec4552, 4 },
    { 0xb4c44553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f944556, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45144559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce0455d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x342c4560, 3 },
    { 0xbfcc4561, 4 },
    { 0x5e784561, 4 },
    { 0x840c4563, 4 },
    { 0, 0 },
    { 0x66ec4565, 4 },
    { 0xd7744566, 3 },
    { 0x727c4565, 5 },
    { 0, 0 },
    { 0xf4944569, 4 },
    { 0x223c4569, 3 },
    { 0, 0 },
    { 0x7370456c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f84570, 3 },
    { 0, 0 },
    { 0x547c4572, 3 },
    { 0, 0 },
    { 0x9f244574, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa384578, 3 },
    { 0x1b904579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77984584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a44588, 4 },
    { 0x3b2c4589, 3 },
    { 0xc154458a, 3 },
    { 0xfbdc458b, 4 },
    { 0x7d30458c, 4 },
    { 0xbb24458d, 3 },
    { 0, 0 },
    { 0x41ac458f, 3 },
    { 0xf0084590, 4 },
    { 0x316c4590, 3 },
    { 0x9e604592, 3 },
    { 0xf41c4593, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc4596, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7644459f, 3 },
    { 0x725045a0, 3 },
    { 0xa60045a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3445a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abc45a8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae045ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4445b7, 3 },
    { 0, 0 },
    { 0x99f845b9, 4 },
    { 0x14a045ba, 3 },
    { 0x837c45bb, 4 },
    { 0xa8b845bc, 3 },
    { 0, 0 },
    { 0xf9f845be, 3 },
    { 0, 0 },
    { 0x9ec045c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349c45c4, 4 },
    { 0x7af045c5, 3 },
    { 0x7a6045c6, 3 },
    { 0xe0e445c7, 3 },
    { 0x734845c7, 4 },
    { 0xbb045c9, 3 },
    { 0xfbbc45c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b845cf, 4 },
    { 0xc4845cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88045d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122c45d8, 3 },
    { 0, 0 },
    { 0xc2a045da, 4 },
    { 0xcfd045da, 3 },
    { 0, 0 },
    { 0x3c045dd, 3 },
    { 0x332c45de, 3 },
    { 0x16c045de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891845e4, 3 },
    { 0, 0 },
    { 0x7b1845e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e845ea, 4 },
    { 0x846045ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72045ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60045f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507445f7, 4 },
    { 0x32b045f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2445fb, 3 },
    { 0x47ec45fc, 3 },
    { 0x80445fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25344602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f44608, 3 },
    { 0, 0 },
    { 0x5184460a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3c460e, 3 },
    { 0, 0 },
    { 0x108c4610, 4 },
    { 0x5b84610, 5 },
    { 0xfb1c4612, 3 },
    { 0x95e04611, 3 },
    { 0x3f184610, 3 },
    { 0xe244611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964461b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64c461e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47c4622, 3 },
    { 0x32004623, 3 },
    { 0, 0 },
    { 0xfc04625, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf04629, 3 },
    { 0x9d68462a, 3 },
    { 0x3e1c462b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f00462f, 3 },
    { 0x44f84630, 3 },
    { 0xed68462f, 3 },
    { 0, 0 },
    { 0x1c7c4633, 3 },
    { 0x2f04633, 3 },
    { 0, 0 },
    { 0x22bc4636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b0463b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c463f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e04642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7784645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62044649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc464e, 4 },
    { 0, 0 },
    { 0x55804650, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x972c4653, 3 },
    { 0x22784654, 3 },
    { 0, 0 },
    { 0xb2344656, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6280465a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c465e, 4 },
    { 0, 0 },
    { 0x9d644660, 3 },
    { 0, 0 },
    { 0x35a84662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fc4669, 3 },
    { 0x4248466a, 3 },
    { 0, 0 },
    { 0x2184466c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3090466f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e04674, 3 },
    { 0, 0 },
    { 0xd95c4676, 3 },
    { 0, 0 },
    { 0x34584678, 3 },
    { 0, 0 },
    { 0x23ec467a, 3 },
    { 0x655c467b, 3 },
    { 0x9e50467c, 3 },
    { 0, 0 },
    { 0xa4c467e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c84681, 3 },
    { 0xd0804682, 3 },
    { 0xab644682, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c884686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50184689, 3 },
    { 0x5084689, 4 },
    { 0xe318468b, 3 },
    { 0x8d54468c, 3 },
    { 0, 0 },
    { 0x740468e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb04698, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8890469b, 4 },
    { 0x8160469c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7c46a1, 3 },
    { 0, 0 },
    { 0x63f846a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19046a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c46a9, 3 },
    { 0x80b446aa, 3 },
    { 0x613046aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4046b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b446b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582446be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e446c1, 4 },
    { 0xba3046c2, 3 },
    { 0x11846c2, 3 },
    { 0x358446c2, 4 },
    { 0xd5d446c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566046c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4846ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b846d5, 4 },
    { 0, 0 },
    { 0xbd5c46d7, 3 },
    { 0xdfa446d8, 3 },
    { 0, 0 },
    { 0xbc4046da, 3 },
    { 0xac046da, 3 },
    { 0x2c6446dc, 4 },
    { 0, 0 },
    { 0x227846de, 3 },
    { 0xeab046df, 4 },
    { 0, 0 },
    { 0x70bc46e1, 3 },
    { 0x9e446e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43846e5, 3 },
    { 0x8ad446e6, 4 },
    { 0, 0 },
    { 0xc6ec46e8, 3 },
    { 0x9cc046e9, 3 },
    { 0x23dc46e9, 3 },
    { 0, 0 },
    { 0x67a446ec, 3 },
    { 0, 0 },
    { 0xbd1c46ee, 3 },
    { 0, 0 },
    { 0xc80c46f0, 5 },
    { 0x181c46f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b046f7, 4 },
    { 0x902c46f7, 3 },
    { 0xe05446f7, 3 },
    { 0x3f6846fa, 4 },
    { 0xb3046f8, 4 },
    { 0x3bb446fc, 4 },
    { 0, 0 },
    { 0xa84446fe, 3 },
    { 0, 0 },
    { 0xac1c4700, 3 },
    { 0xa2584700, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe04704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c4709, 3 },
    { 0x86244709, 4 },
    { 0xeb74470b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b64470f, 4 },
    { 0, 0 },
    { 0xb9844711, 3 },
    { 0, 0 },
    { 0xff504713, 3 },
    { 0, 0 },
    { 0x1ce84715, 4 },
    { 0x5fbc4716, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfc4721, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304727, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bac472f, 3 },
    { 0x4bb4472f, 3 },
    { 0, 0 },
    { 0xb3d04732, 3 },
    { 0, 0 },
    { 0x6ea84734, 3 },
    { 0, 0 },
    { 0x15f44736, 3 },
    { 0x40a44737, 3 },
    { 0x18184738, 3 },
    { 0x76684739, 4 },
    { 0xcf80473a, 3 },
    { 0x25a84739, 3 },
    { 0x3878473c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c473f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec4747, 4 },
    { 0xf0944747, 3 },
    { 0xbd984749, 3 },
    { 0xa34c474a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1618474e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7444755, 3 },
    { 0, 0 },
    { 0x895c4757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccc475a, 3 },
    { 0x2184475b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b44761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb804764, 4 },
    { 0, 0 },
    { 0x47844766, 3 },
    { 0xeafc4766, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48476a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3084770, 3 },
    { 0, 0 },
    { 0xa4284772, 4 },
    { 0, 0 },
    { 0xc87c4774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710c4778, 3 },
    { 0, 0 },
    { 0xdc88477a, 4 },
    { 0, 0 },
    { 0xd8e0477c, 4 },
    { 0x3b90477d, 3 },
    { 0xbb68477e, 3 },
    { 0x937c477f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d84782, 3 },
    { 0x44d04783, 3 },
    { 0xb3c4784, 3 },
    { 0x10084782, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec4478e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83784791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd804794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b84797, 3 },
    { 0, 0 },
    { 0xff544799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe720479f, 3 },
    { 0x71c0479f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d047a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5047a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a447af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ec47b2, 4 },
    { 0x8de447b3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0047b7, 3 },
    { 0x46bc47b8, 3 },
    { 0xd13c47b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce847bf, 3 },
    { 0xd86847c0, 3 },
    { 0x520447c1, 3 },
    { 0x7d3c47c2, 3 },
    { 0xa96047c3, 3 },
    { 0x4b0047c4, 5 },
    { 0xf9f447c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393447cc, 3 },
    { 0, 0 },
    { 0xe8a447ce, 3 },
    { 0, 0 },
    { 0xb1ec47d0, 3 },
    { 0xfc9847d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64047d4, 3 },
    { 0, 0 },
    { 0xd38047d6, 3 },
    { 0, 0 },
    { 0xbaf047d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b847db, 3 },
    { 0, 0 },
    { 0xa1047dd, 5 },
    { 0xa39847de, 3 },
    { 0xafec47de, 3 },
    { 0xa28847de, 3 },
    { 0x80ac47e0, 3 },
    { 0, 0 },
    { 0xe24447e3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x656c47e6, 3 },
    { 0, 0 },
    { 0x2ac47e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7847ed, 4 },
    { 0, 0 },
    { 0xd89047ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456c47f4, 3 },
    { 0x159c47f5, 3 },
    { 0xc4d847f4, 3 },
    { 0xd56847f7, 3 },
    { 0x7ab047f7, 3 },
    { 0xc3b047f7, 3 },
    { 0x635447f5, 3 },
    { 0x47c847fb, 3 },
    { 0xe37847f8, 3 },
    { 0xdebc47fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944807, 3 },
    { 0x15ec4808, 3 },
    { 0xba304809, 3 },
    { 0x63ac480a, 3 },
    { 0x231c480b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693c4814, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87884817, 3 },
    { 0, 0 },
    { 0x64a44819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad84822, 3 },
    { 0x8fd04823, 3 },
    { 0x57084824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c4827, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2498482c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69744830, 3 },
    { 0, 0 },
    { 0x87e04832, 3 },
    { 0x8d244833, 4 },
    { 0, 0 },
    { 0x2bc04835, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1804839, 3 },
    { 0x28c483a, 3 },
    { 0xa34c483b, 3 },
    { 0xbef4483c, 3 },
    { 0xab64483d, 3 },
    { 0xe40c483e, 3 },
    { 0x4108483e, 3 },
    { 0xc82c4840, 4 },
    { 0xf1584839, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7704846, 3 },
    { 0, 0 },
    { 0x7b184848, 3 },
    { 0x1b644849, 4 },
    { 0xe2b4484a, 3 },
    { 0x950484b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8484f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0c4855, 4 },
    { 0, 0 },
    { 0x6b444857, 3 },
    { 0, 0 },
    { 0xf1d44859, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4485d, 3 },
    { 0x9b4485e, 3 },
    { 0, 0 },
    { 0x48304860, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c4865, 3 },
    { 0x967c4865, 3 },
    { 0x404c4867, 4 },
    { 0x98144868, 3 },
    { 0, 0 },
    { 0xba84486a, 3 },
    { 0, 0 },
    { 0x4b00486c, 4 },
    { 0xbe80486c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccc4874, 4 },
    { 0, 0 },
    { 0x406c4876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e0487a, 3 },
    { 0xd0bc487a, 3 },
    { 0x6bf0487c, 4 },
    { 0xe1b4487c, 3 },
    { 0xf2f0487d, 4 },
    { 0, 0 },
    { 0xa8104880, 4 },
    { 0x1ae44881, 4 },
    { 0xd8104882, 4 },
    { 0x69e44880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab44887, 3 },
    { 0, 0 },
    { 0x32744889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7124488e, 3 },
    { 0, 0 },
    { 0xd2744890, 3 },
    { 0x49244891, 3 },
    { 0x556c4891, 3 },
    { 0x81a04893, 3 },
    { 0x5b4c4891, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92984898, 3 },
    { 0xfa244898, 3 },
    { 0x5cb84899, 3 },
    { 0, 0 },
    { 0x81bc489c, 4 },
    { 0, 0 },
    { 0xb430489e, 4 },
    { 0xfcd4489e, 4 },
    { 0, 0 },
    { 0x95ac48a1, 3 },
    { 0xf38048a2, 3 },
    { 0xb8d848a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a848a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8448aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec448af, 3 },
    { 0xc0c848b0, 3 },
    { 0, 0 },
    { 0x85e448b2, 5 },
    { 0x3dcc48b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce048b6, 3 },
    { 0, 0 },
    { 0x292048b8, 3 },
    { 0xc93448b9, 3 },
    { 0xbe5848b9, 3 },
    { 0xe16c48bb, 4 },
    { 0x923848ba, 3 },
    { 0xfec448bd, 3 },
    { 0x928048bd, 3 },
    { 0xb45048be, 3 },
    { 0x35b448c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8c48c5, 3 },
    { 0xe6e448c6, 3 },
    { 0xc34448c7, 4 },
    { 0xca2c48c8, 3 },
    { 0x500048c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df448d0, 3 },
    { 0, 0 },
    { 0x7bb048d2, 4 },
    { 0x2e0048d2, 3 },
    { 0x20b848d4, 4 },
    { 0xb59848d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5048da, 3 },
    { 0x21b448db, 3 },
    { 0x6fc48db, 3 },
    { 0x1b5848dd, 5 },
    { 0xa4ac48dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc048e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553048e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7448ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde448ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6c48f4, 3 },
    { 0xa81048f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0448fb, 3 },
    { 0x6f6848fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa184903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec490a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4490e, 3 },
    { 0xd62c490f, 4 },
    { 0xef704910, 3 },
    { 0xf838490e, 3 },
    { 0xd91c4912, 4 },
    { 0, 0 },
    { 0xe23c4914, 4 },
    { 0x5bf84914, 4 },
    { 0, 0 },
    { 0x76b44917, 3 },
    { 0x259c4918, 3 },
    { 0x86c4917, 3 },
    { 0xf1944919, 3 },
    { 0xa128491b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58744925, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc4492c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca04930, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb04938, 4 },
    { 0, 0 },
    { 0x4bbc493a, 3 },
    { 0xc864493b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bc493e, 4 },
    { 0x603c493e, 3 },
    { 0x24dc4940, 4 },
    { 0, 0 },
    { 0x28e04942, 3 },
    { 0, 0 },
    { 0x7604944, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24494b, 3 },
    { 0, 0 },
    { 0x333c494d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5c4950, 3 },
    { 0xdea44950, 3 },
    { 0x3bc04951, 3 },
    { 0x89e44953, 3 },
    { 0x4b344954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e784959, 3 },
    { 0, 0 },
    { 0x46bc495b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa928495f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc4965, 3 },
    { 0xd84c4965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0c496a, 5 },
    { 0x8598496b, 3 },
    { 0xc3b8496b, 3 },
    { 0xb134496a, 3 },
    { 0, 0 },
    { 0xa464496f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7984978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda084988, 4 },
    { 0, 0 },
    { 0xcb24498a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14498f, 3 },
    { 0x925c4990, 4 },
    { 0xbd444991, 3 },
    { 0x85604992, 4 },
    { 0xda804993, 4 },
    { 0, 0 },
    { 0xf85c4995, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a44998, 3 },
    { 0, 0 },
    { 0x5b5c499a, 3 },
    { 0, 0 },
    { 0x14dc499c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e849a0, 3 },
    { 0xbdf449a1, 3 },
    { 0x77f049a0, 4 },
    { 0, 0 },
    { 0x881449a4, 3 },
    { 0x2fe049a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cc49ae, 3 },
    { 0xdffc49af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e449b9, 4 },
    { 0, 0 },
    { 0x3e8849bb, 3 },
    { 0, 0 },
    { 0xb2049bd, 4 },
    { 0x928c49be, 3 },
    { 0x1b3c49bf, 3 },
    { 0, 0 },
    { 0xb59049c1, 4 },
    { 0, 0 },
    { 0xd79c49c3, 3 },
    { 0xcac49c3, 3 },
    { 0xbb8c49c5, 3 },
    { 0x213049c5, 3 },
    { 0x72d049c5, 3 },
    { 0x1fb449c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ec49cb, 4 },
    { 0, 0 },
    { 0xc0bc49cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1449d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c449db, 3 },
    { 0x78dc49dc, 4 },
    { 0, 0 },
    { 0x7b2849de, 3 },
    { 0x4a9c49df, 3 },
    { 0xcf0c49df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c849e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ac49ea, 4 },
    { 0, 0 },
    { 0x3ce449ec, 3 },
    { 0xd6849ed, 3 },
    { 0x6a3c49ec, 4 },
    { 0, 0 },
    { 0x50f849f0, 4 },
    { 0xc2f849f1, 3 },
    { 0x573849f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba449fd, 3 },
    { 0xf44049fe, 3 },
    { 0, 0 },
    { 0x12744a00, 3 },
    { 0x95cc4a00, 3 },
    { 0, 0 },
    { 0x92dc4a03, 3 },
    { 0, 0 },
    { 0xa144a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b84a08, 4 },
    { 0x9ed84a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1304a0c, 3 },
    { 0, 0 },
    { 0x7b84a0e, 4 },
    { 0xbcc44a0e, 3 },
    { 0xa8184a0e, 3 },
    { 0x2d8c4a11, 4 },
    { 0, 0 },
    { 0x4c84a13, 4 },
    { 0xfaf44a14, 3 },
    { 0x49e44a15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fc4a18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bac4a20, 3 },
    { 0, 0 },
    { 0xff9c4a22, 3 },
    { 0, 0 },
    { 0x19104a24, 3 },
    { 0xf6ec4a25, 3 },
    { 0x2e984a24, 4 },
    { 0x30544a27, 3 },
    { 0xf1944a28, 3 },
    { 0x8c004a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76204a2c, 3 },
    { 0x9e44a2d, 4 },
    { 0, 0 },
    { 0xb3204a2f, 4 },
    { 0, 0 },
    { 0xc1ac4a31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c4a34, 3 },
    { 0, 0 },
    { 0xe0004a36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf644a3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4c4a43, 3 },
    { 0xde4c4a44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf04a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9444a4a, 4 },
    { 0, 0 },
    { 0x35944a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac4a4f, 3 },
    { 0x95e04a4f, 3 },
    { 0, 0 },
    { 0xa1044a52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ec4a5f, 3 },
    { 0x58ec4a60, 3 },
    { 0xc6c84a61, 3 },
    { 0x3e684a62, 3 },
    { 0x46084a63, 5 },
    { 0x78744a64, 3 },
    { 0x400c4a63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c4a6d, 4 },
    { 0xa2c84a6d, 3 },
    { 0xe8bc4a6e, 5 },
    { 0xc3a04a70, 3 },
    { 0x71144a70, 3 },
    { 0x5bb84a72, 3 },
    { 0x367c4a6d, 4 },
    { 0, 0 },
    { 0x7f144a75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba44a78, 3 },
    { 0x451c4a79, 3 },
    { 0xb7444a79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e04a7d, 3 },
    { 0, 0 },
    { 0xc8484a7f, 3 },
    { 0x1f084a7f, 3 },
    { 0x18e44a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c84a85, 4 },
    { 0x96484a86, 3 },
    { 0x63b84a86, 3 },
    { 0xc9b84a88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6104a8b, 4 },
    { 0xcee44a8c, 4 },
    { 0xad084a8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8584a91, 3 },
    { 0xe2404a92, 3 },
    { 0xaacc4a91, 4 },
    { 0, 0 },
    { 0xbb9c4a95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x58204a98, 3 },
    { 0x26904a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85144aa2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc4aac, 3 },
    { 0x29384aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62684ab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75984ab5, 3 },
    { 0x73fc4ab5, 4 },
    { 0x31c4ab7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8884abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c4ac5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc804ac8, 3 },
    { 0x1384ac8, 4 },
    { 0xeb344aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab44acd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d344ad1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c4ad6, 3 },
    { 0x13904ad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa44adc, 3 },
    { 0, 0 },
    { 0xd6784ade, 3 },
    { 0x56984adf, 3 },
    { 0x85504ade, 4 },
    { 0, 0 },
    { 0xd7b04ae2, 4 },
    { 0, 0 },
    { 0x7afc4ae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f844ae9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe84af0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d84afb, 3 },
    { 0xb5c4afb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc84b04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b44b09, 3 },
    { 0x6d9c4b0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad404b12, 3 },
    { 0xd2184b13, 4 },
    { 0x1e944b13, 3 },
    { 0x8cc44b15, 3 },
    { 0xa644b14, 3 },
    { 0x3e4c4b13, 3 },
    { 0x661c4b14, 3 },
    { 0x60604b19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99244b20, 4 },
    { 0x6a584b20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x758c4b24, 4 },
    { 0x99884b24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x77484b28, 3 },
    { 0xd2b84b29, 3 },
    { 0x35844b29, 3 },
    { 0xdcf84b2b, 3 },
    { 0x627c4b2c, 4 },
    { 0xb01c4b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65784b32, 4 },
    { 0, 0 },
    { 0xb4ec4b34, 4 },
    { 0x433c4b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53244b38, 3 },
    { 0x98544b39, 3 },
    { 0x56204b3a, 3 },
    { 0xdd904b38, 3 },
    { 0xcd044b38, 3 },
    { 0xd4f44b3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c4b44, 4 },
    { 0x5a504b45, 3 },
    { 0, 0 },
    { 0xbf044b47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d44b4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92144b4f, 3 },
    { 0x4d704b50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d84b58, 3 },
    { 0x28044b59, 3 },
    { 0, 0 },
    { 0x60cc4b5b, 3 },
    { 0x74544b5b, 4 },
    { 0xc58c4b5d, 3 },
    { 0x70bc4b5e, 4 },
    { 0x572c4b5d, 3 },
    { 0xb8384b60, 3 },
    { 0x93744b61, 3 },
    { 0x8f104b5e, 3 },
    { 0, 0 },
    { 0x304b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c04b67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61004b6c, 4 },
    { 0xe1c4b6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab04b78, 3 },
    { 0xb0844b79, 4 },
    { 0xa8584b79, 4 },
    { 0x5be84b7b, 3 },
    { 0x59844b7c, 3 },
    { 0xb0c84b7d, 4 },
    { 0xfa5c4b7e, 4 },
    { 0x48e04b7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf04b83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bc4b88, 4 },
    { 0x85084b88, 3 },
    { 0, 0 },
    { 0x2bc4b8b, 3 },
    { 0xa2d04b8c, 4 },
    { 0xe7404b8b, 4 },
    { 0x721c4b8c, 3 },
    { 0xeba44b8f, 3 },
    { 0xcb84b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd784b94, 3 },
    { 0x9ce44b95, 3 },
    { 0xf8384b95, 3 },
    { 0x41e44b97, 4 },
    { 0, 0 },
    { 0xb7e44b99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c4b9d, 3 },
    { 0, 0 },
    { 0x3f704b9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2584ba5, 3 },
    { 0, 0 },
    { 0xc404ba7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefc4bab, 3 },
    { 0xb1204bac, 3 },
    { 0xe9c04bab, 4 },
    { 0, 0 },
    { 0x5d144baf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce004bb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38704bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24104bbe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b44bc1, 3 },
    { 0xf89c4bc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b04bc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad644bc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b84bcd, 3 },
    { 0xf2744bce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c4bd2, 3 },
    { 0xbc304bd2, 3 },
    { 0xfa804bd4, 3 },
    { 0, 0 },
    { 0x1ef44bd6, 3 },
    { 0x42ac4bd6, 3 },
    { 0x8ba04bd6, 4 },
    { 0x95084bd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ac4bdc, 3 },
    { 0xad344bdd, 3 },
    { 0xa0784bde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6684be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc4be5, 3 },
    { 0x3d744be5, 3 },
    { 0, 0 },
    { 0x4e104be8, 3 },
    { 0xbec04be9, 3 },
    { 0x62144bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7784bed, 4 },
    { 0, 0 },
    { 0xfc544bef, 4 },
    { 0x73d04bef, 3 },
    { 0, 0 },
    { 0xc4904bf2, 3 },
    { 0, 0 },
    { 0xb6944bf4, 3 },
    { 0x184bf4, 4 },
    { 0x93d44bf6, 3 },
    { 0, 0 },
    { 0x24584bf8, 4 },
    { 0xa0284bf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40984bfe, 4 },
    { 0, 0 },
    { 0xd7d44c00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d44c0a, 3 },
    { 0x2f704c0b, 4 },
    { 0xd7a04c0b, 3 },
    { 0x5e184c0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be84c10, 3 },
    { 0x57844c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e44c16, 3 },
    { 0x8e904c16, 3 },
    { 0, 0 },
    { 0xccac4c19, 3 },
    { 0xcc684c1a, 3 },
    { 0x81f84c1b, 4 },
    { 0x45384c1c, 3 },
    { 0xe7804c1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88504c22, 3 },
    { 0xf0c04c23, 3 },
    { 0x36d44c24, 3 },
    { 0, 0 },
    { 0x1a684c26, 3 },
    { 0, 0 },
    { 0x87b84c28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc4c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67704c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b84c33, 3 },
    { 0x69604c34, 4 },
    { 0, 0 },
    { 0xf9884c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed44c3d, 4 },
    { 0x4e1c4c3e, 3 },
    { 0xc5044c3f, 3 },
    { 0x6c044c3f, 3 },
    { 0x1744c41, 4 },
    { 0x8d284c41, 3 },
    { 0x132c4c43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2c4c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f84c4c, 3 },
    { 0x64044c4c, 3 },
    { 0xfe904c4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecc4c54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d44c5c, 3 },
    { 0, 0 },
    { 0xd70c4c5e, 3 },
    { 0x52c84c5e, 3 },
    { 0x92484c5e, 3 },
    { 0xdd5c4c61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c84c66, 3 },
    { 0, 0 },
    { 0xd27c4c68, 3 },
    { 0x69704c69, 4 },
    { 0xf7a84c69, 3 },
    { 0x14d84c6b, 3 },
    { 0x7cbc4c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27884c78, 4 },
    { 0x2bdc4c79, 3 },
    { 0x74f04c7a, 3 },
    { 0, 0 },
    { 0x8ea04c7c, 3 },
    { 0x73184c7c, 4 },
    { 0, 0 },
    { 0x1ac84c7f, 4 },
    { 0x82544c7f, 4 },
    { 0x82404c80, 4 },
    { 0, 0 },
    { 0x48c44c83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f784c88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4584c8c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c04c90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e84c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36544c9f, 3 },
    { 0x2c004ca0, 3 },
    { 0x5f844ca1, 3 },
    { 0xc1d44ca1, 4 },
    { 0xdd184ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed844cad, 3 },
    { 0x451c4cae, 3 },
    { 0xbf984caf, 3 },
    { 0xc0284cb0, 3 },
    { 0xf0144caf, 3 },
    { 0xad884cae, 4 },
    { 0, 0 },
    { 0x6744cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed84cb8, 4 },
    { 0x4b044cb9, 3 },
    { 0, 0 },
    { 0x6184cbb, 3 },
    { 0xa7f04cbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef44cc4, 3 },
    { 0xad984cc5, 3 },
    { 0x27cc4cc6, 3 },
    { 0x26b84cc7, 3 },
    { 0, 0 },
    { 0xb2a04cc9, 3 },
    { 0x1e984cca, 3 },
    { 0, 0 },
    { 0x97d44ccc, 3 },
    { 0, 0 },
    { 0x9e884cce, 3 },
    { 0xc5584ccf, 3 },
    { 0xd73c4cce, 3 },
    { 0x471c4cd1, 3 },
    { 0x43c44cd1, 3 },
    { 0xd93c4ccf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca04cd7, 3 },
    { 0x9e604cd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54844cdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dc4cdf, 4 },
    { 0xf1f44cdf, 3 },
    { 0xaeb84ce1, 3 },
    { 0x2a544ce2, 3 },
    { 0, 0 },
    { 0x6a804ce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39884cea, 3 },
    { 0x95a04ceb, 3 },
    { 0x14484cec, 3 },
    { 0xbd804ced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4144cf1, 3 },
    { 0, 0 },
    { 0xbbec4cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f684cfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31144cff, 3 },
    { 0xa6c44d00, 3 },
    { 0x286c4d00, 3 },
    { 0xc4d84d01, 3 },
    { 0xb01c4d03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d44d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c4d0d, 3 },
    { 0, 0 },
    { 0x66404d0f, 3 },
    { 0, 0 },
    { 0x25f04d11, 4 },
    { 0x2db04d12, 4 },
    { 0x53fc4d12, 3 },
    { 0x91f44d14, 3 },
    { 0x55ac4d15, 4 },
    { 0, 0 },
    { 0xe3744d17, 3 },
    { 0, 0 },
    { 0x2bb44d19, 3 },
    { 0, 0 },
    { 0xcb184d1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b04d1e, 3 },
    { 0x6eb84d1e, 3 },
    { 0x31184d20, 5 },
    { 0x94ac4d21, 3 },
    { 0x1cbc4d22, 3 },
    { 0xe92c4d1e, 3 },
    { 0xad744d24, 3 },
    { 0x13e84d25, 5 },
    { 0x53e44d25, 4 },
    { 0xdc1c4d1f, 3 },
    { 0xb6fc4d28, 3 },
    { 0x439c4d29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872c4d2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad04d30, 4 },
    { 0x1e004d31, 4 },
    { 0x5f284d32, 3 },
    { 0xbafc4d33, 3 },
    { 0xdcb04d34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce04d38, 3 },
    { 0xa7ac4d39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2004d3c, 3 },
    { 0xc804d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d084d40, 5 },
    { 0x6c844d41, 3 },
    { 0, 0 },
    { 0x2c204d43, 4 },
    { 0xc64c4d43, 4 },
    { 0xfcb04d45, 3 },
    { 0xb9684d46, 3 },
    { 0, 0 },
    { 0x92c84d48, 3 },
    { 0xd9c4d49, 3 },
    { 0x1fc04d4a, 4 },
    { 0, 0 },
    { 0xf2984d4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab44d50, 3 },
    { 0xf6944d51, 3 },
    { 0x54944d51, 3 },
    { 0xe0484d50, 3 },
    { 0, 0 },
    { 0x8aa84d55, 3 },
    { 0, 0 },
    { 0xc2f84d57, 3 },
    { 0xc284d58, 3 },
    { 0x12944d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc4d5c, 4 },
    { 0x70444d5d, 4 },
    { 0xf9404d5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e04d62, 3 },
    { 0, 0 },
    { 0xe65c4d64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109c4d68, 3 },
    { 0, 0 },
    { 0xaa3c4d6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb644d71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd44d79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c344d7c, 3 },
    { 0xd4744d7d, 3 },
    { 0, 0 },
    { 0x73604d7f, 3 },
    { 0xf7c4d80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51c4d84, 3 },
    { 0x14c4d85, 3 },
    { 0, 0 },
    { 0xd6704d87, 3 },
    { 0x63504d87, 3 },
    { 0xa10c4d87, 4 },
    { 0xedb84d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821c4d8f, 3 },
    { 0, 0 },
    { 0xac3c4d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82144d97, 3 },
    { 0, 0 },
    { 0xa9404d99, 3 },
    { 0xbbe04d99, 4 },
    { 0x89e84d9b, 3 },
    { 0, 0 },
    { 0x589c4d9d, 3 },
    { 0xc6a84d9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a44da3, 3 },
    { 0, 0 },
    { 0x9f384da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df84da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c4dac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97384db4, 3 },
    { 0xf3504db5, 3 },
    { 0x15484db6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad904dbf, 3 },
    { 0x3e304dbf, 3 },
    { 0, 0 },
    { 0x6f644dc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c4dc6, 3 },
    { 0x49644dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a504dcb, 4 },
    { 0x7a004dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c84dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71704dd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d44dd7, 3 },
    { 0x74684dd7, 3 },
    { 0x79a84dd7, 3 },
    { 0, 0 },
    { 0x52e84ddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5604dde, 3 },
    { 0x54ac4dde, 3 },
    { 0, 0 },
    { 0xc144de1, 3 },
    { 0, 0 },
    { 0xf2144de3, 3 },
    { 0x51284de3, 3 },
    { 0xa2904de3, 4 },
    { 0xecc4de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c4deb, 3 },
    { 0, 0 },
    { 0xa4984ded, 4 },
    { 0x46944dee, 3 },
    { 0, 0 },
    { 0x92104df0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec4df6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7604df9, 3 },
    { 0, 0 },
    { 0x74844dfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff04dff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db04e02, 5 },
    { 0, 0 },
    { 0xe0e84e04, 3 },
    { 0, 0 },
    { 0xf8c4e06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8c4e11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x43284e14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b44e18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3c4e1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63944e21, 5 },
    { 0x1cd84e22, 3 },
    { 0x25d84e21, 3 },
    { 0x92004e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b84e2b, 4 },
    { 0x30a44e2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf184e30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c4e37, 3 },
    { 0x8ea44e37, 4 },
    { 0, 0 },
    { 0x722c4e3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51504e41, 3 },
    { 0, 0 },
    { 0x26444e43, 4 },
    { 0xa32c4e44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e104e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f84e50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84804e53, 4 },
    { 0xafd84e53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc844e59, 3 },
    { 0x7b4c4e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d04e5d, 4 },
    { 0x28984e5d, 4 },
    { 0xbd304e5e, 3 },
    { 0xf65c4e60, 4 },
    { 0, 0 },
    { 0x3bb84e62, 4 },
    { 0x67b04e62, 3 },
    { 0xd0144e63, 3 },
    { 0xf4e84e65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd984e68, 3 },
    { 0xb144e69, 3 },
    { 0, 0 },
    { 0xe8304e6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f04e72, 3 },
    { 0xc75c4e73, 4 },
    { 0x69c84e74, 3 },
    { 0x52284e74, 3 },
    { 0, 0 },
    { 0xa53c4e77, 3 },
    { 0x11e44e78, 3 },
    { 0xadec4e79, 3 },
    { 0x51e84e7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829c4e7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1684e84, 4 },
    { 0, 0 },
    { 0xb5104e86, 3 },
    { 0x357c4e86, 3 },
    { 0, 0 },
    { 0x33604e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7884e8d, 3 },
    { 0, 0 },
    { 0xde7c4e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dc4e92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb384e96, 3 },
    { 0, 0 },
    { 0xe4ec4e98, 3 },
    { 0xc9684e99, 3 },
    { 0x242c4e98, 3 },
    { 0x20e84e9b, 4 },
    { 0x12084e9c, 4 },
    { 0xeb604e9b, 4 },
    { 0, 0 },
    { 0xb9644e9f, 3 },
    { 0, 0 },
    { 0x1b0c4ea1, 3 },
    { 0x62304ea2, 3 },
    { 0, 0 },
    { 0xcf444ea4, 4 },
    { 0xaffc4ea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69284ea8, 3 },
    { 0xb304ea8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55304eac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c4eaf, 3 },
    { 0x2d284eb0, 3 },
    { 0x3db04eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e04eba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc44ec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52844ec9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5184ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c4ed3, 3 },
    { 0, 0 },
    { 0xad484ed5, 3 },
    { 0x1e844ed5, 3 },
    { 0xa57c4ed5, 4 },
    { 0, 0 },
    { 0x44084ed9, 3 },
    { 0, 0 },
    { 0xa8184edb, 3 },
    { 0, 0 },
    { 0x8c004edd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4684ee0, 3 },
    { 0, 0 },
    { 0x3f604ee2, 3 },
    { 0, 0 },
    { 0x32d04ee4, 3 },
    { 0xa55c4ee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6604ee9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22c4eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff504eef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4004ef7, 4 },
    { 0x63c04ef8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2804efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c4f00, 3 },
    { 0xc4a44f01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d84f04, 3 },
    { 0xc36c4f05, 4 },
    { 0xb7944f06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4044f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb04f0d, 3 },
    { 0, 0 },
    { 0x80f04f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99744f15, 3 },
    { 0, 0 },
    { 0xe4284f17, 3 },
    { 0x1c84f18, 3 },
    { 0x43a04f19, 3 },
    { 0xb044f19, 4 },
    { 0x7b884f19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b04f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b44f25, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda044f2c, 3 },
    { 0, 0 },
    { 0x41f04f2e, 4 },
    { 0xec884f2e, 3 },
    { 0x6dc84f30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f784f34, 3 },
    { 0, 0 },
    { 0xe04f36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad04f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56c4f42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeec4f47, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbc4f50, 4 },
    { 0xb3d84f50, 3 },
    { 0x5b804f52, 4 },
    { 0xf99c4f53, 3 },
    { 0xa6984f54, 3 },
    { 0xaef04f53, 4 },
    { 0x86844f53, 4 },
    { 0xb4104f55, 3 },
    { 0, 0 },
    { 0x878c4f59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde184f60, 3 },
    { 0x46e84f61, 5 },
    { 0x8fd04f60, 3 },
    { 0x9f284f63, 3 },
    { 0x57a44f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0704f67, 4 },
    { 0xd3784f68, 3 },
    { 0x83fc4f67, 3 },
    { 0x1e584f6a, 5 },
    { 0x20384f69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2044f70, 3 },
    { 0, 0 },
    { 0x21c04f72, 4 },
    { 0x9ad04f72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc44f7a, 4 },
    { 0xee4c4f7a, 3 },
    { 0xb9d04f7c, 4 },
    { 0, 0 },
    { 0x6a5c4f7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b484f82, 3 },
    { 0, 0 },
    { 0x3cb04f84, 3 },
    { 0x7f844f84, 3 },
    { 0xc93c4f85, 3 },
    { 0, 0 },
    { 0xfb04f88, 3 },
    { 0, 0 },
    { 0xcc284f8a, 3 },
    { 0x1e2c4f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5944f94, 3 },
    { 0x61704f94, 3 },
    { 0x4044f95, 4 },
    { 0xc05c4f97, 3 },
    { 0x36244f98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc4f9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f04fa0, 3 },
    { 0, 0 },
    { 0x4e5c4fa2, 3 },
    { 0, 0 },
    { 0x7ff44fa4, 3 },
    { 0x12744fa4, 3 },
    { 0x400c4fa5, 3 },
    { 0, 0 },
    { 0x4b4c4fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc444fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163c4fb2, 3 },
    { 0x67f84fb3, 5 },
    { 0xa2f44fb3, 3 },
    { 0, 0 },
    { 0x54dc4fb6, 3 },
    { 0, 0 },
    { 0x1d744fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80044fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34604fc1, 3 },
    { 0x82b84fc1, 4 },
    { 0x782c4fc3, 3 },
    { 0x66304fc4, 3 },
    { 0x82e04fc5, 3 },
    { 0, 0 },
    { 0x13544fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33444fcf, 3 },
    { 0x80784fd0, 3 },
    { 0xbedc4fd0, 3 },
    { 0xdec44fd2, 3 },
    { 0x77744fd2, 4 },
    { 0x7cd84fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03c4fd7, 3 },
    { 0, 0 },
    { 0xe7284fd9, 4 },
    { 0, 0 },
    { 0xee104fdb, 3 },
    { 0x43a84fdc, 3 },
    { 0x8e884fdd, 4 },
    { 0x5c744fdc, 3 },
    { 0x7fa04fdf, 3 },
    { 0xce884fe0, 3 },
    { 0xf9444fde, 5 },
    { 0, 0 },
    { 0x39544fe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc4fe9, 3 },
    { 0, 0 },
    { 0x20b84feb, 4 },
    { 0, 0 },
    { 0x75dc4fed, 3 },
    { 0, 0 },
    { 0x22004fef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25984ff6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b84ffa, 4 },
    { 0x24b44ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476c5003, 3 },
    { 0xa9505004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c5008, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18500b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22385010, 3 },
    { 0, 0 },
    { 0xa2485012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c45019, 3 },
    { 0xfea0501a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c501d, 3 },
    { 0x272c501e, 3 },
    { 0x3b14501f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf185024, 3 },
    { 0x762c5025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f685029, 3 },
    { 0x39ec5029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c38502d, 3 },
    { 0xb0b4502e, 3 },
    { 0x9414502f, 3 },
    { 0, 0 },
    { 0x2c005031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c545034, 3 },
    { 0, 0 },
    { 0xe0e45036, 4 },
    { 0, 0 },
    { 0x3e7c5038, 3 },
    { 0, 0 },
    { 0x5af4503a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb45041, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18905049, 3 },
    { 0, 0 },
    { 0x28c0504b, 3 },
    { 0, 0 },
    { 0x4008504d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33705052, 4 },
    { 0xe7445053, 3 },
    { 0x6b4c5053, 3 },
    { 0xeb505055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b45058, 3 },
    { 0, 0 },
    { 0x5a38505a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3758505f, 3 },
    { 0, 0 },
    { 0x5c9c5061, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2945065, 4 },
    { 0x14e05065, 3 },
    { 0x311c5067, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb8506a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84506d, 3 },
    { 0x3eb8506e, 3 },
    { 0, 0 },
    { 0xeac85070, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8705079, 4 },
    { 0x5d2c507a, 3 },
    { 0, 0 },
    { 0xbfd4507c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af05083, 3 },
    { 0xfb605084, 3 },
    { 0, 0 },
    { 0x9b445086, 3 },
    { 0x51d05087, 3 },
    { 0x358c5088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5090, 3 },
    { 0x2e2c5091, 3 },
    { 0xcaec5091, 3 },
    { 0x75ec5093, 3 },
    { 0xe9705092, 3 },
    { 0x7a385095, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de45099, 3 },
    { 0, 0 },
    { 0x282c509b, 3 },
    { 0x8bdc509c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3670509f, 3 },
    { 0xc28450a0, 3 },
    { 0xc830509f, 3 },
    { 0x263850a2, 3 },
    { 0xfce850a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84050a6, 3 },
    { 0xd55050a7, 3 },
    { 0xb2450a8, 3 },
    { 0x415850a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501850ad, 4 },
    { 0, 0 },
    { 0x8a7050af, 3 },
    { 0x5d2850b0, 3 },
    { 0x77450b0, 4 },
    { 0xc40850b0, 3 },
    { 0xb2a050af, 3 },
    { 0x34c050b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c50b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4c50c1, 3 },
    { 0xc9c850c2, 3 },
    { 0xe72c50c3, 4 },
    { 0, 0 },
    { 0x1a5450c5, 3 },
    { 0, 0 },
    { 0xaeb850c7, 3 },
    { 0x1ec050c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3850cb, 3 },
    { 0, 0 },
    { 0x551450cd, 4 },
    { 0x9af450ce, 3 },
    { 0xed3850cf, 3 },
    { 0x3a450d0, 3 },
    { 0xc86850d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c50d6, 3 },
    { 0xa2850d7, 3 },
    { 0xe88050d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433c50de, 3 },
    { 0x51a050df, 5 },
    { 0x95b050df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ac50e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5450eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x317850ee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1850f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7850f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a05101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74510a, 3 },
    { 0x81a8510a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5284510f, 3 },
    { 0xf5c45110, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df05116, 3 },
    { 0x488c5117, 3 },
    { 0xb2605116, 3 },
    { 0, 0 },
    { 0x5b9c511a, 3 },
    { 0xe390511b, 4 },
    { 0x84fc511a, 3 },
    { 0x8bb8511d, 3 },
    { 0, 0 },
    { 0xc034511f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadc5128, 3 },
    { 0xd9205129, 3 },
    { 0x61045129, 3 },
    { 0xc54512b, 3 },
    { 0xbd9c512c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af45134, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cec5137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f0513d, 3 },
    { 0x3048513e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f845142, 3 },
    { 0x70745142, 3 },
    { 0x301c5144, 3 },
    { 0xdf05143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a8514d, 4 },
    { 0xcc514e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc05152, 4 },
    { 0, 0 },
    { 0x8dbc5154, 3 },
    { 0xf7b85155, 5 },
    { 0x6a85155, 3 },
    { 0xcef05157, 3 },
    { 0xfd445158, 3 },
    { 0x5c5159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8108515c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb45161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec5168, 3 },
    { 0x180c5169, 4 },
    { 0x4b98516a, 4 },
    { 0x6cb0516b, 3 },
    { 0x6d2c516c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee545177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2304517c, 3 },
    { 0, 0 },
    { 0x1310517e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c45183, 3 },
    { 0xb3c85184, 3 },
    { 0, 0 },
    { 0x9d6c5186, 3 },
    { 0xa2405187, 3 },
    { 0x9b745187, 4 },
    { 0x3a945189, 3 },
    { 0x98845188, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ac5190, 3 },
    { 0, 0 },
    { 0xfa845192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c5196, 3 },
    { 0xc8b45197, 3 },
    { 0x670c5198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77451a1, 3 },
    { 0xf7f051a1, 3 },
    { 0x1f0051a3, 3 },
    { 0x373451a4, 5 },
    { 0x55b851a4, 4 },
    { 0, 0 },
    { 0x1c1051a7, 3 },
    { 0, 0 },
    { 0xac3c51a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c51ad, 4 },
    { 0xe37851ae, 3 },
    { 0xe11851ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf851b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90051bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752851c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c51cc, 3 },
    { 0x3ec051cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5451d6, 3 },
    { 0xe32851d6, 3 },
    { 0x566451d8, 3 },
    { 0x5d451d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ac51dc, 3 },
    { 0, 0 },
    { 0x9c4451de, 3 },
    { 0, 0 },
    { 0x7b8051e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2451e9, 4 },
    { 0xc9a051ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c51ed, 3 },
    { 0x517451ee, 3 },
    { 0x74b051ef, 3 },
    { 0x499451ef, 3 },
    { 0x517451f0, 3 },
    { 0xaf3051f2, 3 },
    { 0x9e6851f3, 3 },
    { 0x459c51f1, 3 },
    { 0xef7451f5, 4 },
    { 0xdb0051f6, 3 },
    { 0xffe051f7, 4 },
    { 0xfae851f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8451fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeac5200, 3 },
    { 0x29c05200, 4 },
    { 0x22c5202, 4 },
    { 0x69605202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82c5209, 3 },
    { 0, 0 },
    { 0x9444520b, 4 },
    { 0, 0 },
    { 0x9060520d, 3 },
    { 0, 0 },
    { 0x17a8520f, 4 },
    { 0xe8005210, 4 },
    { 0xae05211, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76c5214, 3 },
    { 0xad985215, 4 },
    { 0x382c5216, 3 },
    { 0x728c5214, 3 },
    { 0xde685215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30805220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a85225, 3 },
    { 0x2eb85225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d0522a, 3 },
    { 0, 0 },
    { 0xb380522c, 3 },
    { 0, 0 },
    { 0x6e78522e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf85234, 3 },
    { 0x89945234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d05239, 4 },
    { 0xd8f8523a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c5243, 3 },
    { 0x83f85244, 4 },
    { 0x90b45245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc045248, 4 },
    { 0xa0445249, 3 },
    { 0xc3c0524a, 3 },
    { 0x80c0524b, 4 },
    { 0xe934524c, 3 },
    { 0xb3fc524a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586c5254, 3 },
    { 0x53f85255, 3 },
    { 0x58e05256, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9c525d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a85260, 3 },
    { 0, 0 },
    { 0x14dc5262, 4 },
    { 0, 0 },
    { 0xf3405264, 4 },
    { 0x66085265, 3 },
    { 0xe6b05266, 4 },
    { 0, 0 },
    { 0x11005268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc840526f, 3 },
    { 0xea645270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5805275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf008527f, 4 },
    { 0x8458527f, 3 },
    { 0, 0 },
    { 0x300c5282, 3 },
    { 0, 0 },
    { 0xa37c5284, 4 },
    { 0x7f9c5285, 4 },
    { 0xd5605284, 3 },
    { 0, 0 },
    { 0x6845288, 3 },
    { 0x88405289, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce58528d, 3 },
    { 0xf904528d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5c5291, 3 },
    { 0, 0 },
    { 0xf99c5293, 3 },
    { 0x38c05294, 4 },
    { 0x9b545294, 3 },
    { 0xe1fc5296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf430529e, 4 },
    { 0x43c529f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2052a3, 3 },
    { 0x794052a4, 3 },
    { 0x1e8052a3, 4 },
    { 0x1c7452a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c52a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e052ac, 3 },
    { 0x138052ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105452b4, 3 },
    { 0x724452b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09052bf, 3 },
    { 0xd61052c0, 4 },
    { 0xa2e452bf, 4 },
    { 0x17c852c2, 3 },
    { 0xda1052c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b452c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496452cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457452d6, 3 },
    { 0, 0 },
    { 0xfc6452d8, 4 },
    { 0xe91852d9, 3 },
    { 0x62e852da, 3 },
    { 0, 0 },
    { 0xb17052dc, 3 },
    { 0, 0 },
    { 0x741852de, 3 },
    { 0xbbb452df, 4 },
    { 0x3fc052e0, 3 },
    { 0x1c7c52e1, 3 },
    { 0xa11c52e2, 3 },
    { 0x18a052e3, 3 },
    { 0xce7052e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc52ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x721852f0, 3 },
    { 0x932052f1, 3 },
    { 0x9fe452f2, 3 },
    { 0, 0 },
    { 0x117452f4, 3 },
    { 0x413c52f5, 3 },
    { 0, 0 },
    { 0xfb2852f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb305302, 3 },
    { 0x95b45302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3285306, 3 },
    { 0, 0 },
    { 0xf3205308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c04530b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb728530f, 3 },
    { 0, 0 },
    { 0x2eb45311, 3 },
    { 0x55105312, 4 },
    { 0xd4ec5312, 3 },
    { 0, 0 },
    { 0x9f205315, 3 },
    { 0, 0 },
    { 0x35145317, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe340531c, 3 },
    { 0xa54531d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19445329, 3 },
    { 0xcbf85329, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad4532f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73405332, 3 },
    { 0, 0 },
    { 0xfb705334, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce8533c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8185341, 3 },
    { 0xe8805342, 3 },
    { 0x86bc5343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00534c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dc5350, 3 },
    { 0xe3585351, 3 },
    { 0xf2885351, 4 },
    { 0x8a885353, 3 },
    { 0x76005352, 3 },
    { 0x6c405355, 3 },
    { 0x6e185356, 3 },
    { 0x58bc5357, 4 },
    { 0xc9f45358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f4535b, 3 },
    { 0x90ac535b, 3 },
    { 0, 0 },
    { 0x5f84535e, 3 },
    { 0, 0 },
    { 0xa5705360, 3 },
    { 0x8105360, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3505366, 3 },
    { 0xdb405366, 3 },
    { 0xa6445368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4536b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a45370, 3 },
    { 0x1df45371, 3 },
    { 0x20085372, 3 },
    { 0xea05373, 3 },
    { 0x4e885374, 3 },
    { 0x22d05372, 3 },
    { 0xd5d45376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4378537d, 3 },
    { 0x3a0537e, 3 },
    { 0x9360537f, 4 },
    { 0xd9bc537f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a05389, 3 },
    { 0xbd385389, 3 },
    { 0x7e7c538a, 3 },
    { 0xfc24538c, 3 },
    { 0, 0 },
    { 0x83ec538e, 3 },
    { 0, 0 },
    { 0xcee85390, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c5394, 3 },
    { 0x25785395, 4 },
    { 0xf16c5394, 4 },
    { 0xcc8c5397, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecc539a, 3 },
    { 0x3080539b, 3 },
    { 0, 0 },
    { 0xd5c539d, 3 },
    { 0x4928539e, 3 },
    { 0, 0 },
    { 0xc34453a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0453a4, 3 },
    { 0x462853a5, 4 },
    { 0x923c53a6, 4 },
    { 0x9a1053a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x993453aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa053ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c453b1, 3 },
    { 0x11fc53b1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd053bb, 3 },
    { 0, 0 },
    { 0x847853bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd853c6, 3 },
    { 0x6c9c53c7, 3 },
    { 0x438853c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82053cf, 4 },
    { 0, 0 },
    { 0x198453d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc53d7, 3 },
    { 0xabd053d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f453db, 3 },
    { 0x12c053dc, 5 },
    { 0xf31053dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9053e7, 3 },
    { 0, 0 },
    { 0xa93c53e9, 3 },
    { 0x9a2c53ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1c53ed, 4 },
    { 0, 0 },
    { 0xf49c53ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef453f3, 3 },
    { 0, 0 },
    { 0xb97053f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e453f8, 3 },
    { 0x2fec53f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57c5400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe45403, 3 },
    { 0, 0 },
    { 0xa6b05405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe968540f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208c5417, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a40541a, 3 },
    { 0xd1fc541b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928541f, 3 },
    { 0x99a05420, 3 },
    { 0, 0 },
    { 0x41645422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c005425, 4 },
    { 0x8a405426, 3 },
    { 0xd7245426, 4 },
    { 0xc23c5428, 3 },
    { 0xa2705429, 3 },
    { 0x2f985428, 3 },
    { 0x58e4542b, 3 },
    { 0x7c8c542b, 4 },
    { 0, 0 },
    { 0x1b0542e, 4 },
    { 0x1f84542f, 3 },
    { 0xf2c5430, 3 },
    { 0x98705431, 3 },
    { 0x17385431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c5435, 3 },
    { 0x407c5435, 4 },
    { 0xd2505436, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa280543a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde485447, 3 },
    { 0, 0 },
    { 0x7d785449, 3 },
    { 0xf9005449, 3 },
    { 0x52e8544b, 4 },
    { 0xf648544b, 3 },
    { 0x5e70544a, 3 },
    { 0x5fd0544e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac5451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb45455, 5 },
    { 0x3e905455, 3 },
    { 0, 0 },
    { 0xdc005458, 3 },
    { 0, 0 },
    { 0xb2a0545a, 4 },
    { 0xdccc545a, 3 },
    { 0x44ac545a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93c545f, 3 },
    { 0x3ce0545f, 3 },
    { 0x76745461, 3 },
    { 0xc105461, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00c5465, 3 },
    { 0x322c5466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec0546a, 3 },
    { 0, 0 },
    { 0x8cc4546c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23c546f, 3 },
    { 0xa485470, 4 },
    { 0xfd085471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa85475, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e45478, 3 },
    { 0x7d45479, 3 },
    { 0xa02c547a, 4 },
    { 0, 0 },
    { 0xf240547c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60085481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c885488, 4 },
    { 0, 0 },
    { 0xc538548a, 5 },
    { 0x1e9c548a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3605495, 3 },
    { 0, 0 },
    { 0x8c5497, 3 },
    { 0x51045497, 3 },
    { 0x1e2c5498, 3 },
    { 0, 0 },
    { 0x3c28549b, 4 },
    { 0x5f94549b, 4 },
    { 0x6d80549c, 3 },
    { 0x2c1c549e, 3 },
    { 0x1024549f, 3 },
    { 0x3f30549f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x869c54a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3c54a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0054ad, 3 },
    { 0xc8d454ae, 4 },
    { 0x87d454af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x205054b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9054b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x368854b8, 3 },
    { 0xfc8054b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595c54bd, 4 },
    { 0xa21454be, 3 },
    { 0x6d8c54bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257854c5, 4 },
    { 0x1e2454c6, 4 },
    { 0xd5ac54c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da454cb, 3 },
    { 0, 0 },
    { 0xbd0854cd, 3 },
    { 0xe19054ce, 3 },
    { 0x6e1c54cf, 3 },
    { 0, 0 },
    { 0xf7ec54d1, 3 },
    { 0x61e454d1, 4 },
    { 0xa86854d1, 3 },
    { 0, 0 },
    { 0xbfb854d5, 3 },
    { 0x80e054d5, 3 },
    { 0, 0 },
    { 0x74c454d8, 3 },
    { 0x94cc54d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c54dd, 3 },
    { 0, 0 },
    { 0xfafc54df, 4 },
    { 0x3e6854e0, 3 },
    { 0xade054e1, 4 },
    { 0xf26454e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ec54eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbc54f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb854f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c854f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b054fd, 4 },
    { 0x73a054fd, 3 },
    { 0x742054ff, 3 },
    { 0x17d85500, 3 },
    { 0x6d245501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c85506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd44550d, 3 },
    { 0x4148550d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbc5512, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc245517, 4 },
    { 0xc4c05518, 3 },
    { 0, 0 },
    { 0x345c551a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6838551e, 4 },
    { 0x3538551f, 3 },
    { 0, 0 },
    { 0xd19c5521, 3 },
    { 0x59b05522, 4 },
    { 0x46245523, 4 },
    { 0xd25c5524, 3 },
    { 0xe1045522, 5 },
    { 0x679c5521, 3 },
    { 0x4df45523, 3 },
    { 0x75545523, 3 },
    { 0x175c5529, 3 },
    { 0x8b685529, 3 },
    { 0x45e45523, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fac552e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d745536, 3 },
    { 0, 0 },
    { 0xcb4c5538, 4 },
    { 0, 0 },
    { 0xfffc553a, 3 },
    { 0, 0 },
    { 0xd40c553c, 3 },
    { 0, 0 },
    { 0x3234553e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b45543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c005548, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544554b, 4 },
    { 0x412c554c, 3 },
    { 0x1ee8554b, 4 },
    { 0x1078554e, 4 },
    { 0x3f7c554f, 3 },
    { 0x5488554e, 4 },
    { 0xf29c554f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54285557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d4555b, 4 },
    { 0xf848555b, 3 },
    { 0, 0 },
    { 0x62d0555e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd405562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c45567, 3 },
    { 0x3ea05568, 4 },
    { 0x5e2c5569, 3 },
    { 0xe30556a, 3 },
    { 0x5c74556a, 5 },
    { 0x63b4556a, 4 },
    { 0xa8805569, 3 },
    { 0, 0 },
    { 0x6dec556f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef05577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2634557c, 4 },
    { 0x8f94557d, 3 },
    { 0, 0 },
    { 0xe948557f, 3 },
    { 0, 0 },
    { 0xd5c05581, 3 },
    { 0x4cb05582, 3 },
    { 0, 0 },
    { 0xc6885584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e28558e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedc5593, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a85597, 4 },
    { 0xb4d05597, 3 },
    { 0x5d6c5598, 3 },
    { 0xe5ac5599, 3 },
    { 0x88505599, 3 },
    { 0x55945599, 4 },
    { 0x3034559c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee055a2, 3 },
    { 0x832855a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394c55ad, 3 },
    { 0x1e6855ae, 4 },
    { 0x753455ae, 3 },
    { 0, 0 },
    { 0xafe455b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21055b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4c55c2, 4 },
    { 0, 0 },
    { 0xdb7055c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101855cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38055d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195055d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771455db, 3 },
    { 0xe1e055db, 5 },
    { 0xc7d855db, 4 },
    { 0, 0 },
    { 0x480455df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ec55e2, 3 },
    { 0x4fc855e3, 4 },
    { 0, 0 },
    { 0x166c55e5, 4 },
    { 0, 0 },
    { 0xbac855e7, 3 },
    { 0xfd8455e8, 3 },
    { 0x5e6c55e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x302855ec, 4 },
    { 0x104055ed, 3 },
    { 0x726855ee, 3 },
    { 0xeca855ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e055f5, 4 },
    { 0x39e055f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7455fc, 3 },
    { 0, 0 },
    { 0x351055fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab105603, 3 },
    { 0, 0 },
    { 0xfb7c5605, 3 },
    { 0x7e845606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e0560f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48105614, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5405617, 3 },
    { 0, 0 },
    { 0xfe405619, 3 },
    { 0x7564561a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a045621, 3 },
    { 0x5ac45622, 3 },
    { 0x98205622, 3 },
    { 0xb5445624, 3 },
    { 0x14cc5624, 3 },
    { 0x34c45621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8805629, 4 },
    { 0xeda0562a, 5 },
    { 0xc4fc562b, 3 },
    { 0, 0 },
    { 0xe6e8562d, 3 },
    { 0, 0 },
    { 0x114562f, 3 },
    { 0x7a2c5630, 4 },
    { 0x22985630, 3 },
    { 0, 0 },
    { 0x38b45633, 3 },
    { 0xfbf85634, 4 },
    { 0xed385635, 4 },
    { 0x51505636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec563a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc05640, 3 },
    { 0x346c5641, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6485647, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb14564d, 4 },
    { 0x16c564d, 3 },
    { 0xc9b4564f, 3 },
    { 0xe98c5650, 4 },
    { 0, 0 },
    { 0x2a545652, 4 },
    { 0x85e85653, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b05656, 3 },
    { 0, 0 },
    { 0x81e45658, 3 },
    { 0xc4845659, 3 },
    { 0x3c94565a, 3 },
    { 0x7324565b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a44565f, 3 },
    { 0xfc145660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e05667, 4 },
    { 0xdfc85668, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb444566b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5334566e, 4 },
    { 0x7f9c566e, 3 },
    { 0x16845670, 3 },
    { 0xbd54566f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eec5674, 3 },
    { 0xc9185675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7305682, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7705686, 4 },
    { 0x82085687, 3 },
    { 0x27105687, 3 },
    { 0x4db85689, 5 },
    { 0xfa24568a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c84568d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c5691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47c5694, 4 },
    { 0, 0 },
    { 0xcc185696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534569a, 4 },
    { 0x509c569b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14569f, 3 },
    { 0x398856a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761856aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a056ae, 4 },
    { 0x830c56af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6456b3, 4 },
    { 0x297056b4, 5 },
    { 0xeffc56b5, 3 },
    { 0x2c1856b5, 3 },
    { 0x7a8856b7, 3 },
    { 0xd20c56b4, 3 },
    { 0xb43c56b9, 3 },
    { 0xcf7856ba, 3 },
    { 0xcba056b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8456be, 3 },
    { 0x24c056be, 3 },
    { 0x41856bf, 3 },
    { 0x9a1c56bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a056c6, 3 },
    { 0, 0 },
    { 0x9b2c56c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181456cc, 4 },
    { 0x856cd, 3 },
    { 0x70dc56cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacc56d3, 5 },
    { 0xe01456d4, 3 },
    { 0x226856d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7056d8, 4 },
    { 0xb2ec56d9, 3 },
    { 0, 0 },
    { 0xf09c56db, 4 },
    { 0, 0 },
    { 0x17e456dd, 3 },
    { 0xd67856de, 3 },
    { 0, 0 },
    { 0xc91c56e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dc56e3, 4 },
    { 0x70c56e3, 3 },
    { 0, 0 },
    { 0x498056e6, 3 },
    { 0x118c56e7, 3 },
    { 0xa5fc56e7, 3 },
    { 0, 0 },
    { 0x267856ea, 3 },
    { 0x572456ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b456f0, 4 },
    { 0xf7056f1, 4 },
    { 0x36c856f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5456f8, 3 },
    { 0x566456f9, 4 },
    { 0xe85856f9, 3 },
    { 0xbafc56fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x176856fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87405703, 3 },
    { 0x9bd45704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40485707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf468570a, 4 },
    { 0, 0 },
    { 0x6a28570c, 3 },
    { 0, 0 },
    { 0x8ad0570e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edc5711, 3 },
    { 0x42945711, 4 },
    { 0x1b05713, 3 },
    { 0, 0 },
    { 0x48f05715, 4 },
    { 0x84b85716, 3 },
    { 0xc8f05717, 3 },
    { 0xd5c45718, 4 },
    { 0xb5445716, 3 },
    { 0xf9e45718, 3 },
    { 0xf368571b, 5 },
    { 0x8c04571a, 3 },
    { 0x36c571d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d85726, 3 },
    { 0x10005727, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc572b, 3 },
    { 0xbfa4572c, 4 },
    { 0x2610572d, 3 },
    { 0x91ac572e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dc5739, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c573c, 3 },
    { 0, 0 },
    { 0x10e4573e, 3 },
    { 0xc494573f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bc5743, 3 },
    { 0, 0 },
    { 0x5ad05745, 3 },
    { 0x13d45746, 3 },
    { 0, 0 },
    { 0x4af45748, 3 },
    { 0x2a7c5749, 4 },
    { 0xb0dc5749, 4 },
    { 0x27bc574a, 3 },
    { 0x452c574c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613c5750, 4 },
    { 0, 0 },
    { 0xfaf45752, 3 },
    { 0, 0 },
    { 0xa46c5754, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69845758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0575f, 4 },
    { 0xf1245760, 3 },
    { 0, 0 },
    { 0x472c5762, 3 },
    { 0x5e85763, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb05769, 3 },
    { 0x15785769, 3 },
    { 0x34b4576b, 3 },
    { 0x3210576c, 4 },
    { 0xab24576d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc5773, 4 },
    { 0x97c05773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e45779, 3 },
    { 0x20e0577a, 3 },
    { 0x51f8577a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337c5783, 3 },
    { 0, 0 },
    { 0x5d4c5785, 3 },
    { 0x66a45786, 3 },
    { 0x5bac5787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff34578c, 3 },
    { 0x3c94578c, 3 },
    { 0, 0 },
    { 0xeed8578f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x690c5792, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f8579d, 3 },
    { 0x3348579e, 4 },
    { 0xdd84579e, 4 },
    { 0, 0 },
    { 0xee6457a1, 3 },
    { 0x6ed057a2, 4 },
    { 0xd14857a3, 4 },
    { 0x3b8c57a2, 4 },
    { 0x39d457a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf857a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4c57ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd457b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41857b8, 3 },
    { 0xf21c57b8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66057bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29457c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6857c6, 3 },
    { 0x6ea457c7, 3 },
    { 0, 0 },
    { 0x8c2857c9, 3 },
    { 0x7b6057ca, 3 },
    { 0xe2d857c9, 3 },
    { 0x8eac57cb, 3 },
    { 0, 0 },
    { 0xe04457ce, 4 },
    { 0xab9057ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970c57d3, 4 },
    { 0, 0 },
    { 0xbdd457d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483457d9, 3 },
    { 0, 0 },
    { 0xf2ac57db, 3 },
    { 0x4d8c57dc, 4 },
    { 0, 0 },
    { 0x297457de, 3 },
    { 0, 0 },
    { 0x23e857e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a057e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14057e9, 3 },
    { 0, 0 },
    { 0x12ec57eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13057ee, 3 },
    { 0xbde457ee, 4 },
    { 0x37b857f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a857f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d057f7, 3 },
    { 0x814857f7, 4 },
    { 0xdef057f9, 4 },
    { 0x734857f9, 3 },
    { 0, 0 },
    { 0x6a7457fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed45806, 3 },
    { 0xa2645807, 3 },
    { 0xba05808, 3 },
    { 0xe6f85809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f45815, 4 },
    { 0, 0 },
    { 0x32085817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc68581b, 3 },
    { 0xf7c0581b, 3 },
    { 0, 0 },
    { 0x1098581e, 3 },
    { 0xe74c581e, 5 },
    { 0x9c74581e, 3 },
    { 0x4ae45821, 3 },
    { 0xb2c05822, 4 },
    { 0, 0 },
    { 0x6f445824, 3 },
    { 0, 0 },
    { 0x92185826, 3 },
    { 0xda945827, 4 },
    { 0x6f9c5828, 3 },
    { 0xe505826, 4 },
    { 0xd75c582a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c582d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2545833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cc5836, 5 },
    { 0x1ad85837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c4583c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5228583f, 3 },
    { 0, 0 },
    { 0xf5005841, 3 },
    { 0xce105842, 4 },
    { 0xad545842, 3 },
    { 0x89e85843, 3 },
    { 0x87505844, 5 },
    { 0x48fc5845, 3 },
    { 0x1ec45846, 4 },
    { 0x88605841, 3 },
    { 0x2f945849, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60805852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7205857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d05860, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab85864, 3 },
    { 0, 0 },
    { 0xf7f45866, 4 },
    { 0x21d85867, 3 },
    { 0x3c7c5868, 3 },
    { 0x8b3c5869, 3 },
    { 0, 0 },
    { 0xb9b4586b, 3 },
    { 0x264586c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c05870, 3 },
    { 0x6a705870, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0c5874, 3 },
    { 0x3da45874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735c5879, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76c5882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d8588a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x470c588d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74c5891, 4 },
    { 0, 0 },
    { 0x7c745893, 3 },
    { 0x68a45894, 3 },
    { 0x93d85895, 3 },
    { 0xfecc5896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde8589d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68858a6, 3 },
    { 0, 0 },
    { 0xa8c058a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e458b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0c58b4, 3 },
    { 0x91f858b5, 3 },
    { 0x846c58b6, 4 },
    { 0xc17c58b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6c58bb, 3 },
    { 0x153858bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695858c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429c58cc, 4 },
    { 0, 0 },
    { 0x468c58ce, 3 },
    { 0, 0 },
    { 0x6b3c58d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ac58d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x498c58d8, 3 },
    { 0xdc0c58d9, 3 },
    { 0x9e8058da, 3 },
    { 0, 0 },
    { 0x60a858dc, 4 },
    { 0, 0 },
    { 0x823458de, 3 },
    { 0x66ec58de, 4 },
    { 0, 0 },
    { 0x684458e1, 3 },
    { 0x3f5058e2, 3 },
    { 0, 0 },
    { 0xc81c58e4, 3 },
    { 0x845458e5, 4 },
    { 0x757c58e5, 4 },
    { 0x913458e7, 3 },
    { 0x362058e8, 3 },
    { 0xe0f458e8, 4 },
    { 0x8b1858ea, 3 },
    { 0xd22858e4, 3 },
    { 0xab9058e6, 4 },
    { 0x154458ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a458f3, 3 },
    { 0x3c7c58f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65858f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc458fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1c5902, 3 },
    { 0xad85902, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae85906, 3 },
    { 0xf05c5907, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3320590e, 3 },
    { 0, 0 },
    { 0xf85c5910, 4 },
    { 0x81985910, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93045915, 3 },
    { 0x68fc5916, 4 },
    { 0, 0 },
    { 0x67485918, 4 },
    { 0xac9c5919, 5 },
    { 0, 0 },
    { 0x2580591b, 3 },
    { 0, 0 },
    { 0xae44591d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1405920, 3 },
    { 0xb2905921, 3 },
    { 0x41c5920, 4 },
    { 0x3f045922, 3 },
    { 0xfee85922, 3 },
    { 0xe5a85921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4592b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe280592f, 4 },
    { 0x32145930, 3 },
    { 0xc6945930, 3 },
    { 0x9645930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db45936, 4 },
    { 0xd5085937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd4593d, 3 },
    { 0, 0 },
    { 0xaac593f, 4 },
    { 0xa98c593f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8345949, 3 },
    { 0xbb50594a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ac594e, 3 },
    { 0x1658594e, 4 },
    { 0x730c594e, 3 },
    { 0x3c05951, 4 },
    { 0xa5445952, 3 },
    { 0, 0 },
    { 0xb8545954, 3 },
    { 0xf09c5955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f08595f, 3 },
    { 0xf0c8595f, 3 },
    { 0x5c105961, 4 },
    { 0x49305962, 3 },
    { 0xd1985963, 3 },
    { 0x1ba05964, 4 },
    { 0x6f3c5965, 3 },
    { 0, 0 },
    { 0xf9f85967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d0596b, 3 },
    { 0x8588596c, 3 },
    { 0x8b78596c, 3 },
    { 0xfacc596d, 3 },
    { 0x2b7c596c, 3 },
    { 0x1a1c5970, 3 },
    { 0x833c5970, 3 },
    { 0, 0 },
    { 0x36e05973, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca05977, 5 },
    { 0x60845978, 3 },
    { 0, 0 },
    { 0xd7a4597a, 3 },
    { 0x7360597b, 3 },
    { 0x15b8597c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c88597f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19c5988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafc5991, 3 },
    { 0, 0 },
    { 0x1b9c5993, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f05998, 3 },
    { 0x51505998, 3 },
    { 0x211c599a, 3 },
    { 0xd344599b, 3 },
    { 0xd3ac599a, 3 },
    { 0x9690599d, 4 },
    { 0xb020599e, 3 },
    { 0, 0 },
    { 0x974459a0, 3 },
    { 0xbf2859a1, 5 },
    { 0x214459a2, 4 },
    { 0xe65059a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387859ac, 3 },
    { 0xed3c59ad, 3 },
    { 0x89dc59ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8059b1, 3 },
    { 0xe83859b2, 4 },
    { 0xf7e059b3, 3 },
    { 0, 0 },
    { 0xcef459b5, 3 },
    { 0xbe5059b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6459be, 4 },
    { 0x328059bf, 3 },
    { 0xbbc459c0, 3 },
    { 0, 0 },
    { 0x26b859c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x256859c5, 4 },
    { 0xace859c6, 3 },
    { 0x340859c5, 5 },
    { 0xe50c59c8, 4 },
    { 0x347c59c9, 3 },
    { 0xf9c859ca, 4 },
    { 0xc13c59cb, 4 },
    { 0x988459cb, 3 },
    { 0x8df059cd, 3 },
    { 0xd61859c9, 3 },
    { 0x435859cf, 3 },
    { 0x6b2059cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89059d3, 3 },
    { 0xa2c059d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c59d7, 3 },
    { 0, 0 },
    { 0xd5059d9, 4 },
    { 0x15dc59d9, 3 },
    { 0x36d859db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1c59e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47459e8, 5 },
    { 0xd30059e9, 3 },
    { 0xf7c459e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84059f3, 3 },
    { 0xbc8859f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a459f8, 3 },
    { 0xbb059f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6059fc, 3 },
    { 0x852859fd, 3 },
    { 0xcec59fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4385a01, 4 },
    { 0, 0 },
    { 0x2d85a03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25405a0e, 3 },
    { 0xd6305a0f, 3 },
    { 0x86385a10, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a705a13, 4 },
    { 0xb8d45a14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d245a17, 4 },
    { 0, 0 },
    { 0x2cf85a19, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43105a1f, 4 },
    { 0, 0 },
    { 0xad385a21, 3 },
    { 0xa85a22, 3 },
    { 0xdcc05a23, 3 },
    { 0xc8405a24, 5 },
    { 0, 0 },
    { 0xe0085a26, 3 },
    { 0, 0 },
    { 0x19585a28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7605a34, 3 },
    { 0x22ac5a35, 5 },
    { 0, 0 },
    { 0xb405a37, 3 },
    { 0xaa905a38, 3 },
    { 0x7e845a39, 3 },
    { 0xaa9c5a3a, 3 },
    { 0xee805a3a, 4 },
    { 0xf02c5a38, 3 },
    { 0xc6d05a3d, 3 },
    { 0x17cc5a3e, 3 },
    { 0x44fc5a3f, 3 },
    { 0x6ea05a3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe945a43, 3 },
    { 0xb1145a43, 3 },
    { 0, 0 },
    { 0x7cc5a46, 3 },
    { 0, 0 },
    { 0xb6345a48, 3 },
    { 0xb0285a48, 4 },
    { 0x25345a49, 3 },
    { 0, 0 },
    { 0xc2205a4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5c5a50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67c5a53, 4 },
    { 0x80c5a54, 4 },
    { 0xaa845a53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c5a59, 3 },
    { 0, 0 },
    { 0xa99c5a5b, 4 },
    { 0x5a385a5b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea705a60, 3 },
    { 0xd045a60, 3 },
    { 0xae745a60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd285a67, 3 },
    { 0x5d585a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcc5a6d, 3 },
    { 0xd6805a6e, 3 },
    { 0x95a85a6e, 3 },
    { 0x5c205a70, 3 },
    { 0x6d885a6d, 3 },
    { 0xefa85a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be45a79, 3 },
    { 0, 0 },
    { 0xb2945a7b, 3 },
    { 0xb83c5a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e85a83, 4 },
    { 0x219c5a84, 3 },
    { 0, 0 },
    { 0xd0b85a86, 3 },
    { 0, 0 },
    { 0x2b405a88, 3 },
    { 0, 0 },
    { 0x69045a8a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c05a8d, 3 },
    { 0xc9545a8d, 4 },
    { 0x26d45a8f, 3 },
    { 0x6245a90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d685a93, 3 },
    { 0, 0 },
    { 0x43b05a95, 3 },
    { 0xd4e05a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5a99, 4 },
    { 0xf7a05a9a, 3 },
    { 0x61b85a9b, 3 },
    { 0x1edc5a9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c5aa1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88545aa6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc5aab, 3 },
    { 0x716c5aab, 4 },
    { 0x1705aad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe05ab3, 3 },
    { 0x32585ab4, 4 },
    { 0x19405ab5, 3 },
    { 0x4bf05ab3, 3 },
    { 0, 0 },
    { 0x9bf85ab8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39085abe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55705ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5ac8, 4 },
    { 0x9f245ac8, 4 },
    { 0xc3305aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35005acf, 3 },
    { 0x58c05ad0, 4 },
    { 0x2ce45ad1, 4 },
    { 0xfe005ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16285ad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c885adc, 3 },
    { 0x84605adc, 4 },
    { 0, 0 },
    { 0x95285adf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc5ae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05aec, 3 },
    { 0, 0 },
    { 0xf2b45aee, 3 },
    { 0, 0 },
    { 0xd1c85af0, 3 },
    { 0, 0 },
    { 0x75c5af2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8c5af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36c5afe, 3 },
    { 0x8bc5afe, 3 },
    { 0, 0 },
    { 0x671c5b01, 4 },
    { 0xbbe85b02, 3 },
    { 0x9ef05b03, 3 },
    { 0xcc205b04, 4 },
    { 0, 0 },
    { 0x125c5b06, 4 },
    { 0, 0 },
    { 0x7c0c5b08, 4 },
    { 0x732c5b08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8785b10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7c5b16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c05b1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8945b23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa145b26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc5b29, 3 },
    { 0x45f85b2a, 3 },
    { 0x9045b2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b45b2f, 3 },
    { 0, 0 },
    { 0x816c5b31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d85b36, 4 },
    { 0x1e705b36, 3 },
    { 0x33d45b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd05b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1045b3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c5b41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373c5b49, 3 },
    { 0x58b45b4a, 4 },
    { 0xe7045b4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42385b50, 3 },
    { 0, 0 },
    { 0xd6505b52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ec5b56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217c5b5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f45b65, 3 },
    { 0, 0 },
    { 0x5e685b67, 4 },
    { 0x94385b68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a05b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e45b76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c5b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d105b7c, 3 },
    { 0x8c9c5b7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ec5b83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f445b86, 3 },
    { 0xfd545b87, 3 },
    { 0xd3705b86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a05b8b, 3 },
    { 0xf5fc5b8b, 3 },
    { 0x64205b8d, 3 },
    { 0x6dd85b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c45b92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4385b9b, 4 },
    { 0, 0 },
    { 0xaa7c5b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x397c5ba0, 3 },
    { 0x3d145ba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47945ba4, 3 },
    { 0x7cbc5ba5, 3 },
    { 0, 0 },
    { 0x4fb05ba7, 3 },
    { 0x6205ba7, 3 },
    { 0x3da05ba9, 3 },
    { 0, 0 },
    { 0x674c5bab, 3 },
    { 0xfbb05bac, 3 },
    { 0x1cac5bad, 4 },
    { 0xa9385bad, 3 },
    { 0xc7045bab, 3 },
    { 0xa6885bb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7845bb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cc5bbc, 3 },
    { 0x69985bbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93c5bc5, 4 },
    { 0x22505bc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c5bca, 3 },
    { 0, 0 },
    { 0x2a785bcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab05bdc, 3 },
    { 0x5bd05bdd, 3 },
    { 0x604c5bde, 3 },
    { 0x9fd05bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf985be4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14345be8, 3 },
    { 0, 0 },
    { 0x3dd05bea, 3 },
    { 0, 0 },
    { 0x212c5bec, 3 },
    { 0xaaf05bec, 4 },
    { 0x34a45bee, 4 },
    { 0xc0305bee, 3 },
    { 0x2ed85bf0, 3 },
    { 0xd57c5bf1, 3 },
    { 0x1e045bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf145bfc, 3 },
    { 0xd95c5bfd, 3 },
    { 0xca3c5bfe, 3 },
    { 0xeb305bfd, 3 },
    { 0x5a445bff, 3 },
    { 0x4de05bfd, 3 },
    { 0xccd05c02, 3 },
    { 0x99b85c03, 3 },
    { 0x1f505c04, 3 },
    { 0, 0 },
    { 0x6bf45c06, 5 },
    { 0xbbd45c07, 4 },
    { 0x1c45c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48985c0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d105c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c5c12, 3 },
    { 0, 0 },
    { 0xc6cc5c14, 3 },
    { 0, 0 },
    { 0xd2d85c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7085c1a, 3 },
    { 0, 0 },
    { 0x12405c1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8205c21, 3 },
    { 0x6545c21, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88105c27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88705c2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cc5c35, 4 },
    { 0x72b85c36, 4 },
    { 0x92445c37, 3 },
    { 0xccec5c37, 3 },
    { 0xffdc5c37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec5c3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f45c42, 3 },
    { 0xfb5c5c43, 3 },
    { 0x405c43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d45c47, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54c5c4e, 3 },
    { 0x6bf85c4f, 4 },
    { 0, 0 },
    { 0xa29c5c51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f85c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0c5c61, 3 },
    { 0x67d45c62, 3 },
    { 0xf9585c63, 3 },
    { 0, 0 },
    { 0xc1d45c65, 3 },
    { 0x9aec5c66, 3 },
    { 0x5a485c65, 3 },
    { 0x28305c68, 3 },
    { 0x52d45c66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a85c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960c5c7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c805c82, 3 },
    { 0x78f85c83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffec5c89, 3 },
    { 0xfdcc5c8a, 3 },
    { 0xd0885c8a, 3 },
    { 0x3e7c5c8c, 3 },
    { 0x9a685c8b, 4 },
    { 0, 0 },
    { 0xe9745c8f, 3 },
    { 0, 0 },
    { 0xd33c5c91, 3 },
    { 0xe3685c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44105c98, 4 },
    { 0x75b45c99, 3 },
    { 0x49645c9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1945ca6, 4 },
    { 0, 0 },
    { 0xacac5ca8, 3 },
    { 0xc6845ca8, 4 },
    { 0xe5ec5ca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b05cad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94105cb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe45cbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e85cc2, 4 },
    { 0, 0 },
    { 0xbe6c5cc4, 4 },
    { 0, 0 },
    { 0xb3285cc6, 4 },
    { 0, 0 },
    { 0x1f1c5cc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e05cd3, 4 },
    { 0xf1a85cd3, 3 },
    { 0x62945cd5, 3 },
    { 0, 0 },
    { 0xb2b05cd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817c5cdb, 3 },
    { 0x8a645cdb, 4 },
    { 0, 0 },
    { 0x86105cde, 3 },
    { 0x1ed05cdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd45ce5, 3 },
    { 0, 0 },
    { 0xace05ce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5ceb, 3 },
    { 0x92345cec, 3 },
    { 0xaed45ced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb85cf0, 5 },
    { 0, 0 },
    { 0x3d445cf2, 3 },
    { 0xf3745cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305cf7, 4 },
    { 0, 0 },
    { 0x4b85cf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61245d04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05c5d07, 4 },
    { 0x30905d08, 4 },
    { 0, 0 },
    { 0x1fe85d0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e05d15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee05d18, 4 },
    { 0, 0 },
    { 0xa7ac5d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c5d1d, 3 },
    { 0x8145d1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d705d23, 3 },
    { 0x27d45d24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1805d27, 3 },
    { 0x9eb85d28, 3 },
    { 0xc7a85d28, 3 },
    { 0x69c05d2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12445d2f, 3 },
    { 0x2a6c5d2f, 3 },
    { 0xa2945d31, 4 },
    { 0x91fc5d32, 4 },
    { 0x4a005d33, 3 },
    { 0x1ab45d34, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10945d3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f85d3e, 3 },
    { 0x14505d3e, 3 },
    { 0, 0 },
    { 0x75645d41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2645d45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe585d48, 3 },
    { 0x55b45d48, 4 },
    { 0xa8e05d4a, 3 },
    { 0, 0 },
    { 0x9dc05d4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92945d51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c405d56, 4 },
    { 0, 0 },
    { 0xb8245d58, 3 },
    { 0x537c5d59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73c5d62, 3 },
    { 0x873c5d63, 3 },
    { 0, 0 },
    { 0x23245d65, 4 },
    { 0, 0 },
    { 0xb445d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68285d6a, 3 },
    { 0, 0 },
    { 0x1b585d6c, 3 },
    { 0xdf905d6d, 3 },
    { 0x31785d6e, 3 },
    { 0, 0 },
    { 0xf1d05d70, 3 },
    { 0x446c5d71, 4 },
    { 0, 0 },
    { 0x8b605d73, 5 },
    { 0xb25c5d74, 3 },
    { 0x37245d75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22545d7d, 3 },
    { 0x2d745d7e, 3 },
    { 0, 0 },
    { 0x2e485d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9945d83, 3 },
    { 0x73785d83, 3 },
    { 0x74c5d85, 3 },
    { 0, 0 },
    { 0x4bc85d87, 3 },
    { 0, 0 },
    { 0x2abc5d89, 3 },
    { 0x86945d8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5445d8d, 4 },
    { 0xe7d05d8e, 3 },
    { 0xb6b05d8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b705d9b, 3 },
    { 0, 0 },
    { 0x43805d9d, 4 },
    { 0x14a05d9e, 3 },
    { 0x98e05d9f, 4 },
    { 0x38c45da0, 3 },
    { 0x131c5d9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164c5da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06c5dac, 3 },
    { 0x41905dad, 3 },
    { 0x433c5dae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f85db9, 4 },
    { 0, 0 },
    { 0xdc385dbb, 5 },
    { 0xbe485dbb, 4 },
    { 0xf7505dbb, 3 },
    { 0x44345dbe, 3 },
    { 0x27e85dbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf05dc5, 3 },
    { 0, 0 },
    { 0x89905dc7, 3 },
    { 0x2ca85dc8, 3 },
    { 0xfa485dc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff605dce, 3 },
    { 0xe0885dcf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6285dd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b85ddc, 3 },
    { 0xbf8c5ddd, 3 },
    { 0xd8305ddd, 3 },
    { 0xe0cc5ddf, 3 },
    { 0, 0 },
    { 0x595c5de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c305de8, 3 },
    { 0x66b05de9, 4 },
    { 0x28585de8, 4 },
    { 0x4edc5dea, 3 },
    { 0x85fc5deb, 3 },
    { 0, 0 },
    { 0xe1e05dee, 3 },
    { 0, 0 },
    { 0x74b45df0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc085df4, 4 },
    { 0x58b05df4, 3 },
    { 0x1c245df6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f45e00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a85e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4c5e08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dc5e0c, 3 },
    { 0, 0 },
    { 0xa2d45e0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c985e11, 3 },
    { 0x49785e12, 3 },
    { 0x23ac5e11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb85e18, 4 },
    { 0xd0405e19, 4 },
    { 0x556c5e1a, 4 },
    { 0, 0 },
    { 0x40a45e1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7785e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8845e23, 4 },
    { 0, 0 },
    { 0x1fb85e25, 3 },
    { 0x125c5e25, 3 },
    { 0x8e545e27, 4 },
    { 0xdf1c5e28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d345e2c, 4 },
    { 0xfe605e2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb05e35, 4 },
    { 0, 0 },
    { 0x81f45e37, 4 },
    { 0x72805e38, 3 },
    { 0x8e785e39, 3 },
    { 0, 0 },
    { 0x66b45e3b, 3 },
    { 0xb9345e3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9685e3f, 3 },
    { 0x9c105e40, 4 },
    { 0xb4305e41, 3 },
    { 0, 0 },
    { 0xe0f85e43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d45e47, 3 },
    { 0xa6145e47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20305e54, 3 },
    { 0x61705e54, 4 },
    { 0x2ce85e55, 3 },
    { 0xf7745e56, 4 },
    { 0xbd205e58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61985e5b, 3 },
    { 0, 0 },
    { 0x14945e5d, 3 },
    { 0, 0 },
    { 0x38c85e5f, 3 },
    { 0, 0 },
    { 0x4c8c5e61, 3 },
    { 0xd8cc5e61, 3 },
    { 0x139c5e62, 4 },
    { 0xf2705e64, 3 },
    { 0x4ff05e65, 4 },
    { 0xf6d85e62, 4 },
    { 0x4e405e67, 3 },
    { 0x4ff05e62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5145e6b, 3 },
    { 0, 0 },
    { 0xdf205e6d, 3 },
    { 0, 0 },
    { 0x841c5e6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5c5e72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71705e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72045e7a, 3 },
    { 0xf645e7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d005e7f, 3 },
    { 0, 0 },
    { 0x92005e81, 3 },
    { 0x43885e82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65085e87, 3 },
    { 0, 0 },
    { 0x9fe05e89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c5e90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x261c5e93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7245e98, 3 },
    { 0x97005e99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1c5e9d, 3 },
    { 0, 0 },
    { 0xd2245e9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e45ea3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e605eaa, 3 },
    { 0x7e585eab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde605eae, 3 },
    { 0, 0 },
    { 0x66ec5eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x510c5eb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3085eb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec145ebb, 3 },
    { 0x804c5ebc, 3 },
    { 0xb0b05ebd, 3 },
    { 0, 0 },
    { 0xe6705ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca205ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf85ecc, 3 },
    { 0, 0 },
    { 0xc2545ece, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa005ed2, 3 },
    { 0x24885ed2, 3 },
    { 0x8bec5ed2, 3 },
    { 0, 0 },
    { 0x61a85ed6, 3 },
    { 0xcf005ed7, 4 },
    { 0, 0 },
    { 0x1d3c5ed9, 3 },
    { 0x3c605eda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd85ede, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d45ee5, 3 },
    { 0xad6c5ee5, 3 },
    { 0x89085ee7, 3 },
    { 0xe4705ee7, 3 },
    { 0x35285ee9, 3 },
    { 0, 0 },
    { 0x1a405eeb, 3 },
    { 0x14845eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8505ef1, 3 },
    { 0x50205ef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311c5ef6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905efe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8745f02, 4 },
    { 0xae105f03, 4 },
    { 0, 0 },
    { 0x6c85f05, 4 },
    { 0xe2d85f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef585f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf705f12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb05f18, 3 },
    { 0, 0 },
    { 0x4c4c5f1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35105f1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0045f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80c5f27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c5f2d, 3 },
    { 0xc1cc5f2e, 4 },
    { 0x69f85f2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be85f35, 3 },
    { 0x8b45f35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac5f3a, 4 },
    { 0, 0 },
    { 0x5c7c5f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f45f3f, 4 },
    { 0x90a85f40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf145f44, 3 },
    { 0x480c5f45, 4 },
    { 0x59505f45, 3 },
    { 0xf7545f47, 3 },
    { 0x48b05f48, 4 },
    { 0, 0 },
    { 0x73245f4a, 3 },
    { 0x33385f4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb05f4e, 4 },
    { 0xf8985f4f, 4 },
    { 0xb3785f4f, 3 },
    { 0xc7885f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae645f54, 4 },
    { 0xc98c5f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3245f5e, 3 },
    { 0, 0 },
    { 0x5be05f60, 3 },
    { 0xd0605f60, 3 },
    { 0x71cc5f62, 3 },
    { 0xce445f62, 3 },
    { 0xb9f05f60, 3 },
    { 0x91e05f65, 3 },
    { 0x7f0c5f60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc05f69, 3 },
    { 0x1d345f6a, 3 },
    { 0x18385f6b, 5 },
    { 0x40545f6c, 5 },
    { 0, 0 },
    { 0xda245f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c5f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b85f7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd305f81, 3 },
    { 0x8c345f82, 3 },
    { 0, 0 },
    { 0x8dac5f84, 3 },
    { 0, 0 },
    { 0xdcb05f86, 3 },
    { 0x461c5f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20285f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c5f8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a145f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5c5f9b, 3 },
    { 0xd2005f9c, 3 },
    { 0x8c745f9c, 5 },
    { 0x1d145f9e, 4 },
    { 0x86245f9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e85fa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6485fa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6c5faa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5945fad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d85fb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e45fb3, 4 },
    { 0x7b005fb4, 4 },
    { 0, 0 },
    { 0xcc605fb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b45fba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa145fbe, 4 },
    { 0x1f8c5fbf, 3 },
    { 0, 0 },
    { 0x14f85fc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef645fc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a85fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9c5fca, 3 },
    { 0, 0 },
    { 0xaf045fcc, 4 },
    { 0x7d985fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c5fd3, 3 },
    { 0xf3985fd3, 3 },
    { 0, 0 },
    { 0x6b3c5fd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a385fda, 3 },
    { 0, 0 },
    { 0x58e45fdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe885fe4, 3 },
    { 0x5ff85fe4, 3 },
    { 0, 0 },
    { 0xd5e85fe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec945ff3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4c5ff8, 3 },
    { 0xb74c5ff9, 3 },
    { 0xc5085ff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4c6001, 4 },
    { 0, 0 },
    { 0x5af86003, 3 },
    { 0x6ae06003, 3 },
    { 0xb986003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e86009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f906011, 3 },
    { 0, 0 },
    { 0x11b46013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cc601b, 3 },
    { 0x778c601c, 3 },
    { 0x3ec8601b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c086020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f46026, 3 },
    { 0xa0886027, 3 },
    { 0x31006028, 3 },
    { 0xd27c6028, 4 },
    { 0xbb48602a, 3 },
    { 0x56c86026, 4 },
    { 0x93c4602c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a786030, 3 },
    { 0xbffc6030, 3 },
    { 0x5e606032, 3 },
    { 0x3946033, 4 },
    { 0, 0 },
    { 0xfc06035, 3 },
    { 0x11706035, 3 },
    { 0xdf1c6037, 4 },
    { 0x8ffc6038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a48603b, 3 },
    { 0x936c603c, 3 },
    { 0x22d0603b, 3 },
    { 0x3090603e, 3 },
    { 0x8504603f, 3 },
    { 0, 0 },
    { 0x82506041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e86044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd06047, 3 },
    { 0x52406048, 3 },
    { 0xc9c06047, 4 },
    { 0x79f8604a, 3 },
    { 0x97e0604a, 3 },
    { 0x2008604c, 4 },
    { 0xffec604d, 3 },
    { 0, 0 },
    { 0xaa70604f, 3 },
    { 0x8d1c604f, 3 },
    { 0x5ebc604f, 3 },
    { 0x3530604f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4986057, 3 },
    { 0xdf586057, 5 },
    { 0x5b06058, 3 },
    { 0xa6e4605a, 3 },
    { 0x39386058, 4 },
    { 0xb47c605c, 4 },
    { 0x7e98605c, 3 },
    { 0x52f8605e, 3 },
    { 0x110605c, 4 },
    { 0, 0 },
    { 0xc6f06061, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb86065, 3 },
    { 0xfdc86066, 4 },
    { 0x22b06067, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe8606c, 3 },
    { 0x7534606d, 5 },
    { 0x20c606e, 5 },
    { 0xe080606f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e46074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c18607b, 3 },
    { 0x5a94607c, 4 },
    { 0x3f44607d, 4 },
    { 0x2894607c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60306082, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5c6087, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818608a, 4 },
    { 0xda8608b, 3 },
    { 0xd2a8608c, 4 },
    { 0x2b5c608c, 3 },
    { 0x7ac8608e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x29986091, 4 },
    { 0x31546092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c946095, 3 },
    { 0, 0 },
    { 0x4b86097, 4 },
    { 0xe2d46098, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a8609e, 3 },
    { 0, 0 },
    { 0xfab460a0, 4 },
    { 0x2cf060a1, 3 },
    { 0x9a860a1, 3 },
    { 0, 0 },
    { 0x93060a4, 3 },
    { 0x96c60a4, 4 },
    { 0xbc0860a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a860a9, 4 },
    { 0xa95c60aa, 3 },
    { 0xdeb060aa, 3 },
    { 0x9c3860aa, 3 },
    { 0, 0 },
    { 0x7c5c60ae, 3 },
    { 0, 0 },
    { 0x94d860b0, 3 },
    { 0xca5460b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac060b6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1860b9, 3 },
    { 0, 0 },
    { 0xdbd860bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7c60be, 3 },
    { 0, 0 },
    { 0x3b7460c0, 3 },
    { 0, 0 },
    { 0x3a5060c2, 3 },
    { 0xe91860c3, 4 },
    { 0, 0 },
    { 0x359060c5, 3 },
    { 0xe1c460c6, 5 },
    { 0x6e6460c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0860ce, 3 },
    { 0x21b860ce, 3 },
    { 0x64dc60ce, 3 },
    { 0x4e0060ce, 3 },
    { 0xd63060d2, 3 },
    { 0xbf6860cf, 3 },
    { 0x69fc60d3, 3 },
    { 0x379860d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2860d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8060de, 3 },
    { 0, 0 },
    { 0x62860e0, 3 },
    { 0, 0 },
    { 0x7bac60e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68c60e6, 3 },
    { 0, 0 },
    { 0x725c60e8, 3 },
    { 0x168460e9, 3 },
    { 0x52b060ea, 3 },
    { 0, 0 },
    { 0xf02860ec, 3 },
    { 0x573460ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e860f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff860f6, 4 },
    { 0x6b7860f6, 3 },
    { 0xff3460f8, 3 },
    { 0x41f060f6, 4 },
    { 0xb98c60f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d860fe, 4 },
    { 0xe81860fe, 3 },
    { 0, 0 },
    { 0xe7a06101, 3 },
    { 0, 0 },
    { 0xc2d46103, 4 },
    { 0x25746104, 3 },
    { 0x14306104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b0610b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae86112, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c46119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8611e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75886121, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab946124, 3 },
    { 0xbbf06125, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a586128, 3 },
    { 0, 0 },
    { 0x1c1c612a, 3 },
    { 0x1ee4612b, 4 },
    { 0, 0 },
    { 0xd1b8612d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc606130, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c6135, 3 },
    { 0x91946136, 4 },
    { 0x1ff06135, 3 },
    { 0x2ae86138, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab0613b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c613e, 3 },
    { 0x8c68613e, 3 },
    { 0xd8506140, 3 },
    { 0x4dcc613f, 4 },
    { 0xbb586141, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b86146, 3 },
    { 0xbd686147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46151, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17586156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fc6159, 3 },
    { 0, 0 },
    { 0xc4dc615b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a0615e, 3 },
    { 0, 0 },
    { 0x9f1c6160, 3 },
    { 0xd306161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b806168, 3 },
    { 0x5cf06169, 4 },
    { 0, 0 },
    { 0xfe24616b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef8616e, 3 },
    { 0x13e8616f, 3 },
    { 0, 0 },
    { 0xe9d86171, 3 },
    { 0x29486172, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b06175, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23686179, 3 },
    { 0xfe9c617a, 3 },
    { 0xc248617b, 3 },
    { 0x6ae0617c, 3 },
    { 0x5540617c, 3 },
    { 0xba8617e, 3 },
    { 0x8ad0617f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25986188, 4 },
    { 0x86e06189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21986192, 3 },
    { 0xa6486192, 3 },
    { 0xc0f06192, 3 },
    { 0x4e006195, 5 },
    { 0x6f186195, 3 },
    { 0x40306197, 3 },
    { 0xddb46196, 4 },
    { 0x34b46199, 4 },
    { 0xa21c6199, 3 },
    { 0xac6195, 4 },
    { 0x70ac6197, 3 },
    { 0xd4406194, 4 },
    { 0xffe0619e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83461a2, 3 },
    { 0, 0 },
    { 0x5fe461a4, 3 },
    { 0xd1e061a5, 3 },
    { 0x9a8c61a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2061aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81061b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdc61b4, 3 },
    { 0xe10861b4, 4 },
    { 0, 0 },
    { 0x73e461b7, 3 },
    { 0x936861b8, 3 },
    { 0xfa1061b8, 3 },
    { 0x3cb061ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c61c2, 4 },
    { 0, 0 },
    { 0x779861c4, 4 },
    { 0x7f5461c5, 4 },
    { 0, 0 },
    { 0xd4a861c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2861cb, 3 },
    { 0, 0 },
    { 0xb6e861cd, 3 },
    { 0, 0 },
    { 0x825061cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b461d3, 3 },
    { 0x378861d3, 3 },
    { 0x4fc861d5, 3 },
    { 0x7e2461d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313061e5, 3 },
    { 0, 0 },
    { 0x7af461e7, 4 },
    { 0x14861e8, 3 },
    { 0x148c61e9, 3 },
    { 0x633c61ea, 3 },
    { 0x7cc861ea, 3 },
    { 0x703461ea, 5 },
    { 0, 0 },
    { 0x47c861ee, 3 },
    { 0, 0 },
    { 0xe73461f0, 3 },
    { 0x689461f1, 3 },
    { 0x4f9061f0, 3 },
    { 0, 0 },
    { 0x5d2861f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd061f7, 4 },
    { 0xf49461f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602861fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52861ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c46206, 4 },
    { 0x79406206, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7184620b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d86211, 4 },
    { 0x31b06212, 4 },
    { 0x32546213, 3 },
    { 0xed486214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214621c, 4 },
    { 0x5770621d, 3 },
    { 0x302c621d, 3 },
    { 0, 0 },
    { 0x6de86220, 3 },
    { 0xb7b46221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4046226, 3 },
    { 0x246227, 3 },
    { 0x3de46228, 3 },
    { 0x9be86229, 3 },
    { 0, 0 },
    { 0x752c622b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26486232, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad06235, 3 },
    { 0xb4206235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c40623a, 3 },
    { 0x5600623a, 4 },
    { 0x5014623c, 3 },
    { 0x6eb0623d, 3 },
    { 0xf018623e, 3 },
    { 0x3084623f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb06242, 3 },
    { 0x65f06243, 3 },
    { 0x4bd46244, 3 },
    { 0x14d46243, 3 },
    { 0xd7886246, 3 },
    { 0x82a46242, 3 },
    { 0x749c6248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c624e, 3 },
    { 0xaf74624f, 4 },
    { 0xedc6250, 3 },
    { 0xe8846251, 4 },
    { 0x76106252, 4 },
    { 0x6b46253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c86258, 3 },
    { 0xb94c6258, 3 },
    { 0x2604625a, 4 },
    { 0xee98625a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b746262, 3 },
    { 0x68c86263, 3 },
    { 0x5c7c6264, 3 },
    { 0, 0 },
    { 0x8f06266, 3 },
    { 0x70446266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa8626a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c626d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a06274, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf786277, 3 },
    { 0x896c6278, 3 },
    { 0x947c6278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc0627c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffc627f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb486282, 3 },
    { 0x7ae46283, 3 },
    { 0, 0 },
    { 0xde4c6285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb84628a, 3 },
    { 0xba64628b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a8628e, 3 },
    { 0, 0 },
    { 0xfddc6290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39406295, 3 },
    { 0, 0 },
    { 0xf1f86297, 5 },
    { 0, 0 },
    { 0x7c886299, 4 },
    { 0x9ff06299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f84629f, 4 },
    { 0x678462a0, 4 },
    { 0x41c062a1, 3 },
    { 0x80f062a0, 3 },
    { 0xd73062a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11062a6, 3 },
    { 0, 0 },
    { 0xc29462a8, 3 },
    { 0x742862a9, 4 },
    { 0xa55462aa, 3 },
    { 0xff0462a9, 4 },
    { 0xd5b462aa, 4 },
    { 0, 0 },
    { 0x4d5c62ae, 3 },
    { 0, 0 },
    { 0x952c62b0, 3 },
    { 0, 0 },
    { 0xef9c62b2, 3 },
    { 0, 0 },
    { 0xd1fc62b4, 3 },
    { 0xf51c62b5, 4 },
    { 0xdce862b4, 5 },
    { 0xf5c462b5, 4 },
    { 0xf8f862b4, 3 },
    { 0xe46462b9, 4 },
    { 0xfa1462b9, 3 },
    { 0xe7d462ba, 3 },
    { 0xd0fc62bc, 5 },
    { 0x4ed862b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efc62c0, 3 },
    { 0x83d862c1, 3 },
    { 0, 0 },
    { 0xf35c62c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6062c7, 3 },
    { 0x5d062c8, 3 },
    { 0, 0 },
    { 0x618462ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x368062cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38062d0, 4 },
    { 0, 0 },
    { 0xe98462d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebc62d7, 3 },
    { 0, 0 },
    { 0x624862d9, 3 },
    { 0, 0 },
    { 0xa4ec62db, 3 },
    { 0, 0 },
    { 0x27862dd, 4 },
    { 0x340462de, 3 },
    { 0xd04062df, 4 },
    { 0xfd7862e0, 3 },
    { 0, 0 },
    { 0x318462e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c62e5, 4 },
    { 0, 0 },
    { 0x318c62e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec62ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f862f0, 4 },
    { 0, 0 },
    { 0xde1462f2, 3 },
    { 0x366c62f2, 4 },
    { 0x7ea462f2, 5 },
    { 0x9e1862f5, 3 },
    { 0xfab862f6, 3 },
    { 0xecac62f7, 3 },
    { 0xfa5062f8, 4 },
    { 0x22e062f6, 4 },
    { 0x91f062fa, 3 },
    { 0x86062fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba806306, 3 },
    { 0xcfc06307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1270630a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb0630e, 3 },
    { 0, 0 },
    { 0xfe86310, 3 },
    { 0, 0 },
    { 0xdcd46312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2c6315, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fc6319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8586320, 3 },
    { 0xcc4c6320, 3 },
    { 0, 0 },
    { 0xcd206323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a46329, 3 },
    { 0x58c0632a, 3 },
    { 0x3bac6329, 5 },
    { 0x65f4632a, 3 },
    { 0xcdd4632a, 3 },
    { 0x18f8632e, 3 },
    { 0xef08632f, 4 },
    { 0x4cd46330, 3 },
    { 0xfcac6330, 4 },
    { 0x630c6332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f606337, 4 },
    { 0xf0f46338, 4 },
    { 0x3e846338, 3 },
    { 0x72886337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d06341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43686346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f86349, 4 },
    { 0, 0 },
    { 0x680c634b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c6350, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee646353, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e86358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9868635b, 3 },
    { 0, 0 },
    { 0xb5a0635d, 4 },
    { 0x8548635e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9086366, 3 },
    { 0, 0 },
    { 0xbd446368, 3 },
    { 0, 0 },
    { 0x740c636a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a86371, 3 },
    { 0x63646372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944637a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92486385, 4 },
    { 0, 0 },
    { 0x860c6387, 3 },
    { 0xe6546388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf168638c, 3 },
    { 0xa6b0638d, 4 },
    { 0x8e04638d, 3 },
    { 0x7cf0638f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c86394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c46397, 4 },
    { 0xd686398, 3 },
    { 0x55a06399, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c0639f, 3 },
    { 0x5dec63a0, 3 },
    { 0x2e8063a0, 4 },
    { 0x8d2c63a0, 3 },
    { 0xf56863a2, 3 },
    { 0xe06063a4, 4 },
    { 0xac28639f, 3 },
    { 0x1ec863a6, 3 },
    { 0xaf863a6, 3 },
    { 0x6c4863a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4063af, 4 },
    { 0x775863af, 4 },
    { 0x946063b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed463b7, 3 },
    { 0xf70063b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c063cc, 3 },
    { 0xf17063cd, 3 },
    { 0x78b463ce, 3 },
    { 0xcc6c63cf, 4 },
    { 0x294c63cf, 3 },
    { 0xd90c63cf, 3 },
    { 0xc2bc63d2, 3 },
    { 0xdbec63d3, 3 },
    { 0xb7dc63d3, 3 },
    { 0xf9b463d3, 3 },
    { 0xf28063d4, 5 },
    { 0xd3f863d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34863da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1863de, 3 },
    { 0xb24863df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c63e3, 4 },
    { 0xf76063e4, 3 },
    { 0x33c63e3, 3 },
    { 0x4b4863e6, 3 },
    { 0, 0 },
    { 0xd09463e8, 3 },
    { 0x57e863e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f863ef, 3 },
    { 0xd93c63f0, 4 },
    { 0x47c63f1, 5 },
    { 0x15e063f1, 3 },
    { 0, 0 },
    { 0x19ac63f4, 3 },
    { 0x8e5063f5, 3 },
    { 0x602863f6, 3 },
    { 0x675063f7, 3 },
    { 0xf67063f8, 4 },
    { 0xbc063f9, 3 },
    { 0, 0 },
    { 0x3bc463fb, 3 },
    { 0xa73463fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c863ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46c6403, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ac6409, 3 },
    { 0xedb86409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33446410, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b746414, 3 },
    { 0xe03c6414, 3 },
    { 0, 0 },
    { 0xdeb86417, 3 },
    { 0x2fc06418, 3 },
    { 0x3e146419, 4 },
    { 0x3b446419, 3 },
    { 0xc1c06418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c6420, 4 },
    { 0, 0 },
    { 0x111c6422, 3 },
    { 0x15cc6422, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d46427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc294642a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfec6433, 3 },
    { 0x1db86433, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2c6437, 3 },
    { 0, 0 },
    { 0x4b786439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd68643d, 3 },
    { 0x3788643d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99006441, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43c6444, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36786448, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a8644b, 3 },
    { 0x17d4644c, 3 },
    { 0xfb48644d, 3 },
    { 0x97d4644d, 3 },
    { 0xd938644d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350c6459, 3 },
    { 0x1af4645a, 3 },
    { 0xc64c645a, 4 },
    { 0, 0 },
    { 0xefb4645d, 3 },
    { 0, 0 },
    { 0xdf8c645f, 3 },
    { 0, 0 },
    { 0x6fa46461, 3 },
    { 0xabbc6462, 3 },
    { 0x93046463, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b86467, 3 },
    { 0x9ce06468, 3 },
    { 0x2cf46467, 3 },
    { 0x7b46467, 4 },
    { 0x4564646b, 3 },
    { 0x18b8646b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c646f, 3 },
    { 0, 0 },
    { 0x242c6471, 3 },
    { 0x89206471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec46476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a0647a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c906480, 3 },
    { 0x24706481, 3 },
    { 0x5f2c6480, 3 },
    { 0x93086483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb730648a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6686490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1606493, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9c649c, 3 },
    { 0, 0 },
    { 0xd7d0649e, 4 },
    { 0x4a44649f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1464a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48064a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x721c64ac, 3 },
    { 0xe31064ad, 3 },
    { 0x4bf464ac, 5 },
    { 0x3e8864af, 3 },
    { 0, 0 },
    { 0xe1b864b1, 4 },
    { 0x32a464b2, 3 },
    { 0x12a464b3, 3 },
    { 0x10d864b1, 3 },
    { 0x74bc64b4, 3 },
    { 0xdba064b6, 3 },
    { 0, 0 },
    { 0x987864b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9864bd, 3 },
    { 0xc14c64be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee064c1, 3 },
    { 0x6bb464c2, 3 },
    { 0x79b864c2, 3 },
    { 0xa5ec64c2, 3 },
    { 0, 0 },
    { 0xbff464c6, 4 },
    { 0, 0 },
    { 0xf19c64c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1864cd, 3 },
    { 0, 0 },
    { 0xb34c64cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd064d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3464d8, 5 },
    { 0x4d2864d8, 3 },
    { 0x60d464da, 4 },
    { 0x519864db, 4 },
    { 0x99b864db, 3 },
    { 0x492864d8, 3 },
    { 0x1fa464de, 3 },
    { 0x3b9064dc, 3 },
    { 0xc27864e0, 4 },
    { 0xcdc864e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8064e4, 4 },
    { 0x56c064e5, 3 },
    { 0, 0 },
    { 0x565c64e7, 3 },
    { 0, 0 },
    { 0xeb464e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x824c64ec, 4 },
    { 0, 0 },
    { 0x153464ee, 3 },
    { 0x62c864ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb464f6, 3 },
    { 0xa73064f6, 3 },
    { 0x9f2064f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99864fb, 4 },
    { 0xaa9064fb, 3 },
    { 0x5e3864fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db46503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd846506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd88650d, 3 },
    { 0x4868650e, 3 },
    { 0, 0 },
    { 0x1c146510, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8986513, 3 },
    { 0, 0 },
    { 0x71246515, 3 },
    { 0, 0 },
    { 0x1bbc6517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2964651b, 4 },
    { 0xeb08651b, 3 },
    { 0, 0 },
    { 0x68c4651e, 3 },
    { 0, 0 },
    { 0xa386520, 3 },
    { 0x10f06521, 3 },
    { 0x2a106520, 3 },
    { 0xa1c46522, 3 },
    { 0, 0 },
    { 0x97886525, 5 },
    { 0, 0 },
    { 0x3f946527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af46535, 3 },
    { 0, 0 },
    { 0xcaa06537, 3 },
    { 0xec9c6537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3c653b, 3 },
    { 0x1350653c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0946542, 3 },
    { 0x2ed86543, 3 },
    { 0x6fe46544, 3 },
    { 0xdad06545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b46549, 3 },
    { 0x7ae8654a, 3 },
    { 0x23d4654b, 3 },
    { 0, 0 },
    { 0x2b44654d, 3 },
    { 0x722c654d, 4 },
    { 0xba3c654d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e46552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb606555, 3 },
    { 0x46606555, 4 },
    { 0x3e646557, 3 },
    { 0x3e246558, 3 },
    { 0x6c586559, 3 },
    { 0xea186559, 3 },
    { 0x608c655a, 4 },
    { 0, 0 },
    { 0xe170655d, 3 },
    { 0, 0 },
    { 0xec64655f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68046568, 3 },
    { 0, 0 },
    { 0x8940656a, 3 },
    { 0x3908656a, 5 },
    { 0, 0 },
    { 0x702c656d, 3 },
    { 0x3094656e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b06572, 5 },
    { 0xcd486573, 3 },
    { 0x7ae86574, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77c6578, 3 },
    { 0x3a686579, 3 },
    { 0x7fb8657a, 4 },
    { 0, 0 },
    { 0x4f34657c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b86583, 3 },
    { 0xad886583, 3 },
    { 0x7f406584, 3 },
    { 0x1f306585, 3 },
    { 0, 0 },
    { 0x67fc6588, 4 },
    { 0, 0 },
    { 0x4168658a, 3 },
    { 0, 0 },
    { 0xa1f0658c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c446591, 3 },
    { 0, 0 },
    { 0xa1b46593, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6886598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28065aa, 3 },
    { 0x348065ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38865ae, 3 },
    { 0, 0 },
    { 0xe19065b0, 3 },
    { 0x805465b1, 4 },
    { 0x121065b1, 4 },
    { 0x9b9c65b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1065ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e065be, 4 },
    { 0xcf4865be, 4 },
    { 0, 0 },
    { 0xf77465c1, 3 },
    { 0, 0 },
    { 0xfd3865c3, 3 },
    { 0x3a465c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43465ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b465ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df865d2, 3 },
    { 0x40065d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317865da, 4 },
    { 0, 0 },
    { 0xce3065dc, 3 },
    { 0x157065dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac065e0, 4 },
    { 0x2bc65e0, 4 },
    { 0x187c65e2, 3 },
    { 0x2bfc65e3, 3 },
    { 0xaa7065e3, 4 },
    { 0x36c865e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83465e9, 3 },
    { 0x788865e9, 3 },
    { 0xb29865eb, 4 },
    { 0xa15c65ea, 3 },
    { 0x203065ed, 4 },
    { 0x8e9865ee, 4 },
    { 0xe47065ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c465f2, 3 },
    { 0, 0 },
    { 0x3e6065f4, 4 },
    { 0xf38865f5, 3 },
    { 0x91f465f4, 4 },
    { 0x9a2465f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56065fb, 3 },
    { 0x101865fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e986602, 3 },
    { 0x945c6602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc660a, 4 },
    { 0x1db4660b, 3 },
    { 0x47a8660c, 3 },
    { 0x396c660d, 4 },
    { 0xb838660d, 3 },
    { 0x9108660f, 3 },
    { 0xb9dc6610, 3 },
    { 0x41f8660f, 3 },
    { 0x442c6612, 3 },
    { 0xa3b46613, 3 },
    { 0xe2a86612, 3 },
    { 0x93d86612, 3 },
    { 0x5ba86611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a46619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed8661c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0306621, 3 },
    { 0xef706621, 3 },
    { 0x18306623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a86626, 3 },
    { 0xd7e46627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044662c, 3 },
    { 0, 0 },
    { 0xab88662e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e886631, 3 },
    { 0x3e8c6631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfc6636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1610663b, 3 },
    { 0xbb90663b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40086643, 3 },
    { 0x9a046644, 3 },
    { 0, 0 },
    { 0x6b946646, 3 },
    { 0xb9846647, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8404664a, 3 },
    { 0x60f0664b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687c6653, 3 },
    { 0, 0 },
    { 0x1fb46655, 3 },
    { 0xb346655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd06659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c4665d, 3 },
    { 0xa1c665e, 4 },
    { 0x4050665f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd506662, 3 },
    { 0, 0 },
    { 0xda586664, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7086669, 3 },
    { 0xf58c666a, 3 },
    { 0x17f8666b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c46677, 3 },
    { 0x9a686678, 3 },
    { 0xae786677, 3 },
    { 0, 0 },
    { 0xb674667b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d46685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2506688, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1364668b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c6691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d086696, 3 },
    { 0xceb06696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2088669c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff4669f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba466a6, 4 },
    { 0xb95c66a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535466ab, 3 },
    { 0xba4066ac, 3 },
    { 0x861c66ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4c66b0, 4 },
    { 0, 0 },
    { 0xa4ac66b2, 3 },
    { 0x219066b3, 3 },
    { 0x6b5866b4, 3 },
    { 0x519866b5, 4 },
    { 0x98dc66b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfec66bd, 4 },
    { 0x4f8066be, 3 },
    { 0x44ec66be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a066c2, 3 },
    { 0, 0 },
    { 0x927466c4, 3 },
    { 0xbbfc66c4, 3 },
    { 0xe5bc66c6, 4 },
    { 0, 0 },
    { 0xb17c66c8, 3 },
    { 0x2c0066c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce866d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x490466d3, 3 },
    { 0xac0066d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x356c66d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638866dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d866e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dec66e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85866eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7866ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b066f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e466f6, 3 },
    { 0, 0 },
    { 0xa73c66f8, 4 },
    { 0, 0 },
    { 0x67c866fa, 4 },
    { 0xd0bc66fb, 3 },
    { 0, 0 },
    { 0xec466fd, 3 },
    { 0xe84866fe, 4 },
    { 0, 0 },
    { 0x28006700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb106707, 3 },
    { 0, 0 },
    { 0x2dcc6709, 3 },
    { 0, 0 },
    { 0x98bc670b, 3 },
    { 0, 0 },
    { 0xae98670d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42806710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f86713, 4 },
    { 0xb8006714, 4 },
    { 0, 0 },
    { 0xe8f06716, 3 },
    { 0, 0 },
    { 0x32f46718, 3 },
    { 0, 0 },
    { 0x640671a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc671e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e86723, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92846726, 3 },
    { 0xb1f86727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x438c672a, 3 },
    { 0xe4b4672b, 3 },
    { 0x82dc672c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742c6732, 3 },
    { 0x3eec6733, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee86739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0673f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6406742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59046747, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2738674b, 3 },
    { 0x7f68674c, 3 },
    { 0x3810674d, 3 },
    { 0x918674b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd46755, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e186759, 4 },
    { 0, 0 },
    { 0x9a84675b, 3 },
    { 0, 0 },
    { 0x631c675d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c6761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe206768, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9178676b, 4 },
    { 0x77a4676b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa86772, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae806777, 5 },
    { 0x2eb86778, 3 },
    { 0, 0 },
    { 0x80e4677a, 4 },
    { 0, 0 },
    { 0x8620677c, 3 },
    { 0xf750677d, 4 },
    { 0xab10677e, 3 },
    { 0x2a14677f, 3 },
    { 0xa1586780, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c8678b, 3 },
    { 0x36ec678b, 3 },
    { 0xdd2c678c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e46791, 5 },
    { 0, 0 },
    { 0x7d206793, 3 },
    { 0x39f86794, 3 },
    { 0, 0 },
    { 0x87606796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3180679e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c67a3, 3 },
    { 0, 0 },
    { 0xbf8867a5, 3 },
    { 0x96f467a6, 3 },
    { 0xf6ec67a6, 3 },
    { 0, 0 },
    { 0x77f067a9, 3 },
    { 0x46467aa, 3 },
    { 0x4067aa, 3 },
    { 0x10c067ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454067b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb467b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6467b7, 4 },
    { 0x809867b8, 4 },
    { 0, 0 },
    { 0x782467ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c067be, 4 },
    { 0x3e2067bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37067c2, 3 },
    { 0, 0 },
    { 0x23ec67c4, 3 },
    { 0xa61067c5, 3 },
    { 0, 0 },
    { 0xa6067c7, 3 },
    { 0, 0 },
    { 0x4ac067c9, 3 },
    { 0x6e6467ca, 4 },
    { 0xd84467cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0467ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96467d1, 3 },
    { 0x966867d2, 3 },
    { 0x634c67d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2c67d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0467e1, 3 },
    { 0, 0 },
    { 0x9a5c67e3, 3 },
    { 0xf3ec67e4, 3 },
    { 0, 0 },
    { 0x97bc67e6, 3 },
    { 0, 0 },
    { 0x564867e8, 4 },
    { 0xa89467e8, 3 },
    { 0x322867e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4c67ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c67f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b467fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0c6801, 3 },
    { 0, 0 },
    { 0xa7686803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8234680b, 4 },
    { 0x5ce4680c, 3 },
    { 0x64a0680d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2786811, 5 },
    { 0xffdc6812, 3 },
    { 0x482c6813, 3 },
    { 0x79306812, 4 },
    { 0, 0 },
    { 0x3a086816, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa46819, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5314681c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee86820, 3 },
    { 0, 0 },
    { 0xaf446822, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4346826, 4 },
    { 0, 0 },
    { 0x381c6828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ec682c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff06835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20786838, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd378683d, 4 },
    { 0xd5b8683e, 3 },
    { 0, 0 },
    { 0xb3446840, 3 },
    { 0x17e46840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4946848, 3 },
    { 0x36886849, 3 },
    { 0x22c86849, 3 },
    { 0xfbf0684b, 3 },
    { 0x6e6c684a, 3 },
    { 0x218684c, 3 },
    { 0xc2a8684e, 3 },
    { 0x89f4684f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f306852, 3 },
    { 0xd546852, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e4685b, 4 },
    { 0x79a0685c, 4 },
    { 0x551c685b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d386865, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35846869, 3 },
    { 0, 0 },
    { 0x9810686b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba86871, 3 },
    { 0, 0 },
    { 0xc2f46873, 3 },
    { 0x4fdc6874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d06878, 3 },
    { 0xda086879, 3 },
    { 0x74c4687a, 4 },
    { 0x3aac6879, 3 },
    { 0, 0 },
    { 0xfb74687d, 3 },
    { 0, 0 },
    { 0xe21c687f, 3 },
    { 0xe77c6880, 3 },
    { 0xf61c6880, 3 },
    { 0x42bc6882, 3 },
    { 0, 0 },
    { 0x18706884, 3 },
    { 0, 0 },
    { 0x99086886, 3 },
    { 0x2da86887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c688a, 4 },
    { 0x7f14688a, 3 },
    { 0xa328688a, 3 },
    { 0, 0 },
    { 0xbc688e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe706892, 3 },
    { 0x950c6893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f46898, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6580689d, 3 },
    { 0, 0 },
    { 0x9c14689f, 4 },
    { 0x38c8689f, 3 },
    { 0xc2d468a1, 3 },
    { 0xdecc68a1, 3 },
    { 0x30b868a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c468b0, 5 },
    { 0, 0 },
    { 0xb45868b2, 3 },
    { 0, 0 },
    { 0xc22468b4, 5 },
    { 0x8ca468b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dc68bc, 3 },
    { 0x5de868bd, 3 },
    { 0x31d068be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68068c4, 3 },
    { 0, 0 },
    { 0x867068c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf068ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c868d2, 3 },
    { 0x6d0068d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927468d9, 4 },
    { 0x227068da, 3 },
    { 0x249468d9, 3 },
    { 0x8cbc68dc, 4 },
    { 0x2ba468dc, 3 },
    { 0xf1d468dc, 3 },
    { 0x1c1068df, 4 },
    { 0, 0 },
    { 0x26fc68e1, 3 },
    { 0x6a8468e2, 3 },
    { 0xfbb868e3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca868e8, 3 },
    { 0xf5fc68e9, 3 },
    { 0xf9b068e9, 4 },
    { 0x688c68ea, 3 },
    { 0xf23068ec, 4 },
    { 0x5a6468ed, 3 },
    { 0x2cd868ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3868f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e868f6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac068f9, 3 },
    { 0x470c68f9, 3 },
    { 0xb63468fb, 3 },
    { 0xae068fc, 3 },
    { 0x15a068fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19546905, 4 },
    { 0x53c86905, 3 },
    { 0xb3ec6907, 3 },
    { 0xe0446908, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c690d, 3 },
    { 0x8a20690d, 3 },
    { 0xd6e8690f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b86913, 4 },
    { 0x8bdc6914, 3 },
    { 0x1b606915, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dc6918, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cc691c, 3 },
    { 0, 0 },
    { 0xfd40691e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3206921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ac6925, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b94692a, 3 },
    { 0, 0 },
    { 0xbc58692c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff86930, 4 },
    { 0xa5286930, 4 },
    { 0, 0 },
    { 0x550c6933, 3 },
    { 0x6346934, 3 },
    { 0x94686935, 3 },
    { 0xf3f46936, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b246939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad0693e, 3 },
    { 0x5928693e, 3 },
    { 0x90c693e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x672c6943, 3 },
    { 0xfc346943, 3 },
    { 0x6a006943, 4 },
    { 0x2ce86946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf10694a, 3 },
    { 0, 0 },
    { 0x9c40694c, 3 },
    { 0, 0 },
    { 0x4fdc694e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e06951, 3 },
    { 0, 0 },
    { 0xa78c6953, 3 },
    { 0, 0 },
    { 0xdec46955, 3 },
    { 0xa1d06956, 3 },
    { 0x19486956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb64695a, 3 },
    { 0x4df4695b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae4695e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c46964, 4 },
    { 0xcdb46965, 4 },
    { 0x6ba46964, 3 },
    { 0xbcd06967, 3 },
    { 0x2cec6968, 3 },
    { 0x35a86968, 4 },
    { 0x6f8c6969, 4 },
    { 0xe9046967, 3 },
    { 0, 0 },
    { 0xac74696d, 4 },
    { 0x39cc696d, 3 },
    { 0x8018696d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d46973, 3 },
    { 0xc6dc6974, 3 },
    { 0x5df46974, 3 },
    { 0x19606976, 3 },
    { 0x967c6977, 3 },
    { 0x1ed86978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74697f, 3 },
    { 0xa2946980, 3 },
    { 0x52e06981, 3 },
    { 0xcf34697f, 5 },
    { 0x58406983, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00c6987, 4 },
    { 0xedf86988, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x574c698b, 5 },
    { 0x33ec698c, 3 },
    { 0x5cc4698d, 4 },
    { 0x3150698e, 3 },
    { 0xceb0698b, 3 },
    { 0, 0 },
    { 0x4ac6991, 4 },
    { 0x8e4c6991, 3 },
    { 0x34e86993, 5 },
    { 0x685c6992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x358c6997, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce0699b, 3 },
    { 0x1dd8699c, 3 },
    { 0xb9d8699c, 3 },
    { 0, 0 },
    { 0xe204699f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b069a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767869a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca469ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c069b9, 4 },
    { 0x158c69ba, 3 },
    { 0, 0 },
    { 0x25e869bc, 3 },
    { 0xb45869bc, 3 },
    { 0, 0 },
    { 0xe52469bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75869c3, 3 },
    { 0x513469c3, 3 },
    { 0xd69469c5, 4 },
    { 0, 0 },
    { 0xa9a069c7, 3 },
    { 0x681469c8, 3 },
    { 0xe63469c7, 3 },
    { 0x3ba469ca, 5 },
    { 0xe96069cb, 4 },
    { 0xcb3869c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd469d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce869d4, 4 },
    { 0x319469d5, 4 },
    { 0, 0 },
    { 0x4c7469d7, 3 },
    { 0x81d069d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1c69db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x191069de, 4 },
    { 0, 0 },
    { 0x1c2069e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5069e7, 3 },
    { 0x37d069e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9069eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e869ee, 4 },
    { 0x25bc69ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc69f2, 3 },
    { 0xb63869f3, 4 },
    { 0x9bc69f4, 3 },
    { 0, 0 },
    { 0x1ebc69f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239069fa, 4 },
    { 0xcac69fb, 3 },
    { 0xaae069fb, 3 },
    { 0xd9c469fc, 3 },
    { 0xd43469fe, 3 },
    { 0xc9b869fe, 3 },
    { 0xb48c69fa, 5 },
    { 0x7e7c6a01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884c6a06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc6a0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c6a14, 4 },
    { 0x22786a15, 3 },
    { 0x97206a14, 4 },
    { 0x64f86a17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d06a1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4c6a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f646a25, 3 },
    { 0x89486a26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc986a29, 4 },
    { 0, 0 },
    { 0x37946a2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24586a31, 3 },
    { 0x63606a32, 4 },
    { 0xaa746a31, 3 },
    { 0x21046a34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e86a39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c6a3d, 4 },
    { 0xc846a3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46c6a44, 3 },
    { 0x43346a45, 3 },
    { 0xcd286a46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x445c6a49, 4 },
    { 0xe9146a49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c86a4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b86a50, 3 },
    { 0x57f46a51, 3 },
    { 0x31446a50, 3 },
    { 0x501c6a52, 3 },
    { 0xe006a54, 3 },
    { 0x13b86a55, 3 },
    { 0xea006a54, 3 },
    { 0xd7546a57, 4 },
    { 0x6c546a56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b186a5b, 4 },
    { 0, 0 },
    { 0xbe306a5d, 3 },
    { 0x40386a5e, 3 },
    { 0xb786a5d, 4 },
    { 0xaa346a5e, 5 },
    { 0x56c6a61, 3 },
    { 0x2e5c6a61, 3 },
    { 0x42f06a63, 4 },
    { 0x6d346a64, 4 },
    { 0xf86c6a65, 5 },
    { 0x61906a65, 4 },
    { 0xd8946a61, 4 },
    { 0x4f846a64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a46a6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61486a77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806a7d, 3 },
    { 0xffbc6a7d, 4 },
    { 0x15dc6a7f, 3 },
    { 0xe7946a80, 3 },
    { 0, 0 },
    { 0x1de86a82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8546a88, 3 },
    { 0xe3506a89, 3 },
    { 0x72786a8a, 4 },
    { 0x65e46a8b, 4 },
    { 0xfff86a8c, 3 },
    { 0x22706a8a, 3 },
    { 0, 0 },
    { 0x6f9c6a8f, 4 },
    { 0xdd146a8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62286a93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc6a97, 3 },
    { 0xcf3c6a97, 3 },
    { 0x21f46a99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e06a9c, 4 },
    { 0x46606a9c, 4 },
    { 0x94986a9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dc6aa4, 3 },
    { 0, 0 },
    { 0xce0c6aa6, 3 },
    { 0, 0 },
    { 0x7a946aa8, 4 },
    { 0x1c8c6aa9, 3 },
    { 0x99506aa9, 3 },
    { 0x4d06aab, 3 },
    { 0x47c06aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99306ab0, 3 },
    { 0x4f7c6ab0, 3 },
    { 0x79046ab0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f346abc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x502c6abf, 4 },
    { 0xdbec6ac0, 4 },
    { 0x9dc86ac1, 3 },
    { 0, 0 },
    { 0xa2286ac3, 4 },
    { 0x94686ac4, 3 },
    { 0xd646ac5, 3 },
    { 0x798c6ac6, 3 },
    { 0xe4186ac7, 3 },
    { 0x54a06ac8, 3 },
    { 0, 0 },
    { 0x45706aca, 3 },
    { 0, 0 },
    { 0xd18c6acc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd846ad0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b86ad4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f46ad7, 3 },
    { 0x99806ad8, 3 },
    { 0xced06ad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d06adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x388c6adf, 3 },
    { 0xa6646adf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27746aea, 3 },
    { 0, 0 },
    { 0x52606aec, 3 },
    { 0, 0 },
    { 0xf27c6aee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca06af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc06af9, 3 },
    { 0xea646af9, 3 },
    { 0, 0 },
    { 0xb5f06afc, 3 },
    { 0x49fc6afd, 4 },
    { 0x22206afd, 3 },
    { 0, 0 },
    { 0xec0c6b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0806b03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c86b06, 4 },
    { 0x5ec06b06, 4 },
    { 0xc7cc6b08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b86b0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209c6b10, 3 },
    { 0x4146b11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd06b14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b46b19, 3 },
    { 0xd1ec6b1a, 3 },
    { 0x79106b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00c6b1e, 4 },
    { 0xb6106b1f, 3 },
    { 0xd7d06b20, 4 },
    { 0x93786b21, 4 },
    { 0xd9c46b22, 3 },
    { 0x14bc6b22, 3 },
    { 0, 0 },
    { 0x53d46b25, 3 },
    { 0x94946b25, 3 },
    { 0xb5146b26, 3 },
    { 0x87f06b28, 4 },
    { 0xf4ac6b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c386b2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e46b30, 3 },
    { 0x45cc6b30, 3 },
    { 0x6f106b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3c6b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17086b42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a106b4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a906b4d, 5 },
    { 0xcef46b4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa86b51, 3 },
    { 0xf7d46b52, 4 },
    { 0x1f46b52, 3 },
    { 0x8306b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c06b58, 3 },
    { 0x8806b59, 3 },
    { 0xc90c6b5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9086b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae86b60, 3 },
    { 0xa1186b60, 4 },
    { 0, 0 },
    { 0x98206b63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd606b6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc6b6e, 3 },
    { 0x27c06b6e, 3 },
    { 0x6d06b6f, 5 },
    { 0x6b606b71, 3 },
    { 0x4b386b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae046b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c6b7d, 3 },
    { 0x3ff46b7e, 3 },
    { 0xfc486b7f, 4 },
    { 0x94586b80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4006b86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf06b8a, 4 },
    { 0x58806b8b, 3 },
    { 0, 0 },
    { 0xe606b8d, 3 },
    { 0xf7546b8e, 3 },
    { 0x61906b8e, 4 },
    { 0xd8f86b8d, 3 },
    { 0, 0 },
    { 0x3cf06b92, 3 },
    { 0x82f86b93, 3 },
    { 0, 0 },
    { 0x6f86b95, 4 },
    { 0, 0 },
    { 0x8506b97, 3 },
    { 0x41dc6b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcc6b9c, 3 },
    { 0x54e46b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f86ba3, 3 },
    { 0x13646ba3, 3 },
    { 0x1d586ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d06bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1546bb4, 4 },
    { 0, 0 },
    { 0xe64c6bb6, 3 },
    { 0, 0 },
    { 0x29d06bb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e906bbd, 3 },
    { 0x8046bbe, 4 },
    { 0, 0 },
    { 0x3e286bc0, 3 },
    { 0x20d06bc1, 3 },
    { 0, 0 },
    { 0x67a06bc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd86bc7, 4 },
    { 0x33846bc8, 4 },
    { 0, 0 },
    { 0x6a906bca, 4 },
    { 0xbfe06bcb, 3 },
    { 0x6cc06bcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0086bd3, 3 },
    { 0, 0 },
    { 0x5646bd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0c6bd9, 3 },
    { 0x5f246bda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1106bde, 4 },
    { 0x57346bdf, 3 },
    { 0x3c506be0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9486be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d406bec, 4 },
    { 0, 0 },
    { 0x6c906bee, 3 },
    { 0, 0 },
    { 0x16886bf0, 3 },
    { 0x51886bf1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c6bf4, 4 },
    { 0xe72c6bf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac946bf8, 3 },
    { 0xf2f46bf9, 3 },
    { 0xf1ac6bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e46c03, 3 },
    { 0x10946c04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac6c08, 3 },
    { 0, 0 },
    { 0x225c6c0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3686c11, 3 },
    { 0x576c6c11, 3 },
    { 0, 0 },
    { 0xba246c14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85386c1c, 3 },
    { 0, 0 },
    { 0xea446c1e, 3 },
    { 0, 0 },
    { 0xcb6c6c20, 3 },
    { 0xa2806c21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e06c25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd86c2a, 3 },
    { 0xb7c6c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd46c2e, 3 },
    { 0, 0 },
    { 0xaee86c30, 3 },
    { 0x10a46c30, 4 },
    { 0x5af06c31, 3 },
    { 0x4a646c33, 4 },
    { 0xaa1c6c34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e86c39, 3 },
    { 0, 0 },
    { 0x97bc6c3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d946c4c, 3 },
    { 0, 0 },
    { 0x6d5c6c4e, 3 },
    { 0x164c6c4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6c53, 4 },
    { 0, 0 },
    { 0x8e3c6c55, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x761c6c58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b206c5b, 3 },
    { 0xf39c6c5c, 3 },
    { 0x675c6c5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9946c63, 3 },
    { 0x6dd06c64, 4 },
    { 0xe84c6c64, 3 },
    { 0x40486c64, 3 },
    { 0x42ec6c67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c6c6f, 3 },
    { 0, 0 },
    { 0xf6e86c71, 4 },
    { 0, 0 },
    { 0xb8dc6c73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1c6c76, 4 },
    { 0xb3f86c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e786c7b, 3 },
    { 0x87e06c7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef246c83, 3 },
    { 0x37e46c84, 4 },
    { 0x3b586c85, 4 },
    { 0x7ad46c86, 4 },
    { 0x3d5c6c87, 4 },
    { 0xf4a86c88, 4 },
    { 0, 0 },
    { 0x44a86c8a, 3 },
    { 0, 0 },
    { 0xf1e46c8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c046c91, 3 },
    { 0xaccc6c92, 3 },
    { 0, 0 },
    { 0xa69c6c94, 3 },
    { 0xa0946c94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99446c98, 3 },
    { 0xd4dc6c98, 3 },
    { 0, 0 },
    { 0x4c06c9b, 3 },
    { 0x3d386c9c, 3 },
    { 0, 0 },
    { 0xe6ac6c9e, 3 },
    { 0x66bc6c9f, 3 },
    { 0xa3346c9f, 4 },
    { 0x81606ca1, 3 },
    { 0xc05c6ca2, 3 },
    { 0x80406ca2, 3 },
    { 0xdf3c6c9e, 3 },
    { 0, 0 },
    { 0x3b946ca6, 4 },
    { 0xeebc6ca7, 4 },
    { 0x95846ca7, 3 },
    { 0xd8806ca6, 4 },
    { 0x504c6caa, 3 },
    { 0x91b86caa, 4 },
    { 0xa3ec6cac, 3 },
    { 0xf7e86cab, 3 },
    { 0x57506cae, 3 },
    { 0x295c6cae, 3 },
    { 0xe9506cb0, 3 },
    { 0x29fc6cb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b06cb4, 4 },
    { 0, 0 },
    { 0xd2446cb6, 4 },
    { 0x39fc6cb7, 3 },
    { 0, 0 },
    { 0x66d46cb9, 3 },
    { 0xd5ac6cba, 3 },
    { 0, 0 },
    { 0x64246cbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed46cc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a806cc3, 3 },
    { 0, 0 },
    { 0xc7186cc5, 3 },
    { 0xfd8c6cc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c6cc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec386cce, 4 },
    { 0x88246ccf, 4 },
    { 0x8aec6cd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78606cd6, 3 },
    { 0x6806cd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e06cda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986cdd, 3 },
    { 0, 0 },
    { 0x1ac06cdf, 3 },
    { 0x40a46ce0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca86cec, 4 },
    { 0x37846cec, 4 },
    { 0, 0 },
    { 0x2a806cef, 4 },
    { 0x5bac6cf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda446cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f946cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f406cfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac86d01, 4 },
    { 0x3fa46d02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f86d05, 3 },
    { 0, 0 },
    { 0xe05c6d07, 3 },
    { 0xe7446d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309c6d0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b86d12, 4 },
    { 0xba286d12, 3 },
    { 0, 0 },
    { 0xf1606d15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35506d18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729c6d1c, 3 },
    { 0x35586d1c, 3 },
    { 0xbbc46d1c, 4 },
    { 0, 0 },
    { 0x2ddc6d20, 3 },
    { 0, 0 },
    { 0xacac6d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f06d27, 3 },
    { 0x37e46d27, 3 },
    { 0x74b06d28, 3 },
    { 0x5e86d28, 3 },
    { 0xafc46d2b, 4 },
    { 0x263c6d2c, 3 },
    { 0xae706d2c, 3 },
    { 0xba1c6d2b, 4 },
    { 0xebec6d2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5386d36, 5 },
    { 0x83346d37, 3 },
    { 0x4d286d38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37706d3c, 4 },
    { 0, 0 },
    { 0xa1206d3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f86d44, 4 },
    { 0x2a486d45, 3 },
    { 0xcb0c6d46, 3 },
    { 0xe3a06d45, 3 },
    { 0xc6e86d45, 3 },
    { 0x6b9c6d49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bc6d4c, 3 },
    { 0x58946d4c, 3 },
    { 0x56406d4c, 3 },
    { 0x84846d4f, 3 },
    { 0xd4cc6d4e, 3 },
    { 0x6e706d51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27c6d54, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3c6d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c6d5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c86d62, 3 },
    { 0, 0 },
    { 0xa0506d64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71286d69, 3 },
    { 0xc5146d69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf086d6e, 4 },
    { 0x20d06d6f, 3 },
    { 0xfe106d6e, 3 },
    { 0x10a06d71, 4 },
    { 0x53b06d71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x44406d75, 3 },
    { 0, 0 },
    { 0x38106d77, 3 },
    { 0, 0 },
    { 0x521c6d79, 3 },
    { 0x1ce06d7a, 3 },
    { 0x3be86d7b, 4 },
    { 0xd8e86d7a, 3 },
    { 0x38b86d7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c806d80, 3 },
    { 0x81206d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c06d8a, 4 },
    { 0, 0 },
    { 0x19f86d8c, 3 },
    { 0xa4146d8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0906d97, 3 },
    { 0xa0186d98, 3 },
    { 0x5e5c6d99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7146d9e, 3 },
    { 0xa2f86d9e, 4 },
    { 0, 0 },
    { 0xf2a86da1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f06da6, 3 },
    { 0, 0 },
    { 0xa8a06da8, 3 },
    { 0, 0 },
    { 0x49b46daa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec6db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdc6dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8946dbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8246dc2, 3 },
    { 0x80cc6dc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b206dcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d46dd0, 3 },
    { 0x76c46dd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cec6dd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c6ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d946de2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdc6de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5986de9, 3 },
    { 0x81786dea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7486df4, 3 },
    { 0x1b06df5, 3 },
    { 0, 0 },
    { 0x71e86df7, 3 },
    { 0x2c746df8, 3 },
    { 0xbf546df8, 3 },
    { 0x34346dfa, 3 },
    { 0xe8506dfb, 3 },
    { 0x1e1c6dfc, 3 },
    { 0xacec6dfb, 3 },
    { 0xdfe46df8, 4 },
    { 0x4d686dfa, 3 },
    { 0xbf306df9, 4 },
    { 0x21606e01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac06e04, 3 },
    { 0x5346e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a46e09, 3 },
    { 0x33546e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e86e0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c6e12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c6e1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x769c6e1e, 3 },
    { 0, 0 },
    { 0x2d46e20, 4 },
    { 0xe26c6e20, 3 },
    { 0xe3546e22, 4 },
    { 0x69206e21, 3 },
    { 0xcbdc6e24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a46e27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d786e2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x382c6e32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a386e37, 3 },
    { 0xac6c6e37, 4 },
    { 0x8e5c6e39, 3 },
    { 0, 0 },
    { 0x44106e3b, 3 },
    { 0, 0 },
    { 0xe95c6e3d, 4 },
    { 0x2b106e3e, 3 },
    { 0x49306e3f, 3 },
    { 0, 0 },
    { 0xba06e41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a86e44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8c6e47, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c6e4c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb206e50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf286e53, 3 },
    { 0x66f86e54, 3 },
    { 0x60746e55, 4 },
    { 0, 0 },
    { 0xa2e46e57, 3 },
    { 0, 0 },
    { 0x73446e59, 3 },
    { 0, 0 },
    { 0xa7686e5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb646e65, 3 },
    { 0x30a46e65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc146e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560c6e70, 3 },
    { 0, 0 },
    { 0xd9886e72, 3 },
    { 0x75146e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37386e78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5c6e7b, 3 },
    { 0x8b06e7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d146e7f, 3 },
    { 0, 0 },
    { 0x86946e81, 3 },
    { 0xc1606e82, 4 },
    { 0x3b046e83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd86e86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27786e89, 3 },
    { 0, 0 },
    { 0x92446e8b, 5 },
    { 0xbdcc6e8c, 4 },
    { 0x51146e8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f86e93, 3 },
    { 0xe8346e94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d606e97, 3 },
    { 0xbd606e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d86e9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c6e9f, 3 },
    { 0x7ec06e9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd186ea7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648c6eae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa286eb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8646eb5, 3 },
    { 0x49246eb6, 4 },
    { 0x57a86eb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff946ebc, 4 },
    { 0x86b06ebc, 3 },
    { 0x866c6ebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f446ec1, 3 },
    { 0x79846ec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce506ecc, 3 },
    { 0x88f46ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c6ed0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c6ed5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53c6ed9, 3 },
    { 0x5ba46eda, 3 },
    { 0x99746edb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd86ee3, 4 },
    { 0x7e406ee3, 4 },
    { 0xba246ee5, 3 },
    { 0xf9986ee4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56246eea, 3 },
    { 0, 0 },
    { 0x6fac6eec, 3 },
    { 0x6f086eed, 3 },
    { 0x892c6eee, 4 },
    { 0x2af86eef, 3 },
    { 0x1cf06eef, 4 },
    { 0xf3b06eee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b386ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d86efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70146eff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8c6f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3c6f05, 4 },
    { 0x42406f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acc6f09, 3 },
    { 0xbd346f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c46f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c6f17, 3 },
    { 0, 0 },
    { 0xcdf46f19, 3 },
    { 0, 0 },
    { 0xf2406f1b, 3 },
    { 0, 0 },
    { 0x47106f1d, 3 },
    { 0x43446f1e, 3 },
    { 0x5c206f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd06f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac6f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14446f30, 4 },
    { 0, 0 },
    { 0x98006f32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57806f37, 5 },
    { 0x5e506f38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a506f3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7806f3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c46f44, 4 },
    { 0xbcb06f45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda906f48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b86f4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b506f53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc6f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2746f59, 3 },
    { 0xb84c6f5a, 3 },
    { 0xbeac6f5b, 3 },
    { 0x544c6f5a, 4 },
    { 0x83946f5a, 4 },
    { 0xfb6c6f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c6f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17506f67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd46f6a, 3 },
    { 0x5f9c6f6b, 3 },
    { 0x65086f6c, 4 },
    { 0x23646f6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c46f75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c06f78, 4 },
    { 0x25cc6f78, 3 },
    { 0xdd346f7a, 3 },
    { 0, 0 },
    { 0xce906f7c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19c6f80, 3 },
    { 0x76506f81, 3 },
    { 0xaee46f82, 3 },
    { 0x646c6f80, 3 },
    { 0x6b4c6f80, 3 },
    { 0, 0 },
    { 0x99a06f86, 3 },
    { 0xb8006f86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c86f8b, 4 },
    { 0xbc886f8b, 3 },
    { 0xe5f86f8d, 3 },
    { 0x11246f8e, 3 },
    { 0x48506f8e, 3 },
    { 0xf2e46f90, 3 },
    { 0x57fc6f90, 5 },
    { 0xee046f92, 4 },
    { 0x9fb06f8d, 3 },
    { 0x10386f8e, 3 },
    { 0, 0 },
    { 0x54f06f96, 3 },
    { 0x30886f97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f786f9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a786fa5, 3 },
    { 0, 0 },
    { 0x159c6fa7, 4 },
    { 0, 0 },
    { 0x915c6fa9, 3 },
    { 0, 0 },
    { 0xda886fab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3306fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b86fb1, 3 },
    { 0x33e06fb1, 3 },
    { 0xba1c6fb3, 3 },
    { 0x3c2c6fb3, 4 },
    { 0x2b8c6fb3, 3 },
    { 0xf5586fb6, 3 },
    { 0xa9986fb7, 3 },
    { 0x22c46fb3, 4 },
    { 0x842c6fb9, 3 },
    { 0x4e146fb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e46fbd, 3 },
    { 0, 0 },
    { 0x88f46fbf, 3 },
    { 0, 0 },
    { 0xcc046fc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13186fc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d686fcb, 3 },
    { 0, 0 },
    { 0x67446fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3246fd0, 3 },
    { 0xe1f46fd1, 3 },
    { 0x7cc6fd1, 3 },
    { 0x91d86fd3, 4 },
    { 0x32806fd4, 3 },
    { 0x5f786fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9c6fd8, 3 },
    { 0x5c386fd9, 3 },
    { 0, 0 },
    { 0x5106fdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f806fe1, 3 },
    { 0xc9e06fe2, 3 },
    { 0x7506fe3, 4 },
    { 0, 0 },
    { 0xdabc6fe5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdac6fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e686ff0, 4 },
    { 0xefb06ff1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef86ff9, 3 },
    { 0x95a86ffa, 3 },
    { 0x4b786ff9, 3 },
    { 0xa9406ffc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88447000, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320700a, 4 },
    { 0, 0 },
    { 0xc9a4700c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf47011, 3 },
    { 0x68b87011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d07016, 4 },
    { 0x77bc7017, 4 },
    { 0xe7a07018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68701c, 3 },
    { 0x61b0701d, 4 },
    { 0, 0 },
    { 0xe394701f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xede87022, 3 },
    { 0x1487023, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be47026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768702b, 4 },
    { 0x806c702c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3178703b, 3 },
    { 0x4e68703c, 3 },
    { 0, 0 },
    { 0x86c0703e, 4 },
    { 0xd594703e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f07045, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad47048, 4 },
    { 0x9a107048, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe510704d, 3 },
    { 0xc2e8704e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468705c, 3 },
    { 0, 0 },
    { 0x459c705e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283c7065, 3 },
    { 0, 0 },
    { 0x3fcc7067, 4 },
    { 0x7047068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698706e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f87072, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147076, 4 },
    { 0x7cac7077, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc004707a, 3 },
    { 0, 0 },
    { 0xe708707c, 4 },
    { 0, 0 },
    { 0x812c707e, 3 },
    { 0, 0 },
    { 0x69f47080, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a47086, 4 },
    { 0, 0 },
    { 0xa30c7088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4708b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61787093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d07098, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a4709c, 3 },
    { 0x3c60709d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a870a2, 3 },
    { 0, 0 },
    { 0x910870a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc470a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37070ab, 3 },
    { 0x329c70ac, 3 },
    { 0, 0 },
    { 0xc49870ae, 5 },
    { 0, 0 },
    { 0x661070b0, 3 },
    { 0x4c4870b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80870b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbc70bd, 3 },
    { 0xc5e470bd, 4 },
    { 0, 0 },
    { 0xfcc70c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89070c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7070c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x500070c9, 3 },
    { 0, 0 },
    { 0x81b870cb, 3 },
    { 0xbe6c70cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4470d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bc70d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e470d9, 3 },
    { 0, 0 },
    { 0xc9b870db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3870e2, 3 },
    { 0x6df070e3, 3 },
    { 0x529070e2, 3 },
    { 0, 0 },
    { 0x8e2470e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee070ea, 3 },
    { 0xdca470eb, 3 },
    { 0xaa6870eb, 3 },
    { 0x85c70ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7870f0, 3 },
    { 0xf95870f0, 3 },
    { 0x5f0c70f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61070fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544070ff, 4 },
    { 0xaf87100, 3 },
    { 0x50f87101, 4 },
    { 0x57907102, 4 },
    { 0x14c87103, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86887108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c710d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3c7113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d687117, 3 },
    { 0, 0 },
    { 0xa8bc7119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc0711d, 3 },
    { 0x8cd4711e, 3 },
    { 0x7ec4711e, 4 },
    { 0x2a18711d, 3 },
    { 0xacbc7120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1807125, 4 },
    { 0x3f87126, 3 },
    { 0xfbf87126, 3 },
    { 0, 0 },
    { 0x85007129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5194712c, 3 },
    { 0, 0 },
    { 0xf010712e, 3 },
    { 0xd53c712f, 4 },
    { 0x49b07130, 4 },
    { 0x40f8712f, 5 },
    { 0x2f407132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb307135, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3784713e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a907141, 3 },
    { 0xdda07142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d087148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6c714c, 3 },
    { 0x1028714d, 3 },
    { 0x3414714d, 3 },
    { 0x9b58714f, 4 },
    { 0, 0 },
    { 0xd14c7151, 3 },
    { 0xccdc7151, 3 },
    { 0, 0 },
    { 0x54a07154, 3 },
    { 0x99a07155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba18715a, 3 },
    { 0xea7c715b, 3 },
    { 0x75ec715b, 3 },
    { 0xb868715d, 3 },
    { 0, 0 },
    { 0x272c715f, 3 },
    { 0xdfd87160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c47167, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d00716a, 3 },
    { 0, 0 },
    { 0xf738716c, 3 },
    { 0, 0 },
    { 0x3478716e, 3 },
    { 0xa164716f, 3 },
    { 0xb33c716e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c7173, 3 },
    { 0, 0 },
    { 0x56a47175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49c7178, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c717f, 3 },
    { 0, 0 },
    { 0x5d687181, 3 },
    { 0x60607182, 3 },
    { 0x19c47182, 3 },
    { 0xc0607184, 3 },
    { 0x9a187185, 3 },
    { 0x8d187186, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a07189, 3 },
    { 0xebb4718a, 4 },
    { 0xca30718b, 5 },
    { 0xdccc7189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x270c718f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb047196, 3 },
    { 0x18647197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553871a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c071ab, 3 },
    { 0x5d1071ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25871af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b871b2, 3 },
    { 0, 0 },
    { 0x658871b4, 4 },
    { 0xb1ac71b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d071b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e471bc, 4 },
    { 0xfb1c71bd, 3 },
    { 0x645c71bc, 3 },
    { 0xec7071bc, 3 },
    { 0xe04c71c0, 3 },
    { 0xad2071bc, 4 },
    { 0x87471c1, 3 },
    { 0x1d0871bd, 4 },
    { 0x809c71c4, 4 },
    { 0xd96c71c3, 3 },
    { 0x62871c6, 3 },
    { 0xc03071c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e471cd, 3 },
    { 0x40e871ce, 3 },
    { 0xc21471cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375871d3, 4 },
    { 0xc45071d3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c71d8, 3 },
    { 0x7e071d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4871e0, 4 },
    { 0x422471e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1871e4, 5 },
    { 0, 0 },
    { 0x639871e6, 3 },
    { 0x1fc471e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7071ef, 3 },
    { 0x465c71f0, 4 },
    { 0xef0071f1, 3 },
    { 0x8c6471f2, 3 },
    { 0x4e0c71f0, 3 },
    { 0xb78071f4, 3 },
    { 0x8d0471f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7c71f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3871fc, 4 },
    { 0x3c3071fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d07202, 3 },
    { 0xd9187203, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33c7209, 3 },
    { 0xa607209, 5 },
    { 0x538720b, 3 },
    { 0, 0 },
    { 0x9138720d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67607210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57207214, 3 },
    { 0xc0e07214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94721e, 3 },
    { 0x4db8721f, 3 },
    { 0xefa4721f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702c7227, 3 },
    { 0x79687227, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aec722c, 3 },
    { 0x3c3c722c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f87232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871c7238, 3 },
    { 0, 0 },
    { 0x9b40723a, 3 },
    { 0x59d8723b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287240, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214724a, 3 },
    { 0xa43c724a, 3 },
    { 0x256c724b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b907250, 3 },
    { 0xf7447251, 3 },
    { 0x488c7251, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30107258, 3 },
    { 0, 0 },
    { 0xcf30725a, 3 },
    { 0xf6f0725b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd018725f, 3 },
    { 0x186c7260, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe387264, 3 },
    { 0xd1147265, 3 },
    { 0xab987265, 3 },
    { 0, 0 },
    { 0xcc807268, 4 },
    { 0, 0 },
    { 0x820c726a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4726d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a47271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d07274, 3 },
    { 0x74107275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9447279, 3 },
    { 0xb904727a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd008727d, 3 },
    { 0, 0 },
    { 0xf4b4727f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51707284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbc728b, 3 },
    { 0, 0 },
    { 0x64fc728d, 3 },
    { 0x5ab0728d, 3 },
    { 0xd058728f, 3 },
    { 0x342c7290, 4 },
    { 0x5514728d, 3 },
    { 0xc87c7291, 3 },
    { 0xf2487293, 3 },
    { 0xf5947294, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c729a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b072a1, 3 },
    { 0, 0 },
    { 0xe5f872a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24872a8, 4 },
    { 0x4a472a8, 3 },
    { 0x47a072aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814872af, 3 },
    { 0xc6dc72b0, 3 },
    { 0x61c072b1, 4 },
    { 0x293872b1, 5 },
    { 0x585072b3, 4 },
    { 0x28b072b3, 3 },
    { 0, 0 },
    { 0x426472b6, 3 },
    { 0, 0 },
    { 0xe8a872b8, 3 },
    { 0xc35872b8, 4 },
    { 0xc6072b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15072bd, 4 },
    { 0x4e4072bd, 3 },
    { 0, 0 },
    { 0xc4f072c0, 3 },
    { 0xb88c72c1, 3 },
    { 0x782c72c2, 3 },
    { 0xf34072c3, 3 },
    { 0, 0 },
    { 0x6fdc72c5, 4 },
    { 0xcd4c72c6, 3 },
    { 0xc4c072c7, 3 },
    { 0, 0 },
    { 0x187072c9, 3 },
    { 0x14f472ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99872ce, 4 },
    { 0x379072ce, 3 },
    { 0x12872cf, 3 },
    { 0xfa2072d1, 4 },
    { 0xf3fc72ce, 4 },
    { 0, 0 },
    { 0x132872d4, 3 },
    { 0xa8e072d4, 3 },
    { 0x1c5472d5, 3 },
    { 0x110072d7, 4 },
    { 0x206c72d6, 4 },
    { 0x135072d9, 3 },
    { 0xc072d8, 3 },
    { 0x99c072db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaac72de, 4 },
    { 0x2da872df, 3 },
    { 0xe89872df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d072e4, 3 },
    { 0x912472e5, 4 },
    { 0, 0 },
    { 0xb61072e7, 3 },
    { 0xd74c72e8, 4 },
    { 0xc2e072e7, 3 },
    { 0, 0 },
    { 0xe6f472eb, 3 },
    { 0, 0 },
    { 0x992472ed, 3 },
    { 0, 0 },
    { 0x416072ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbc72f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b072fa, 3 },
    { 0, 0 },
    { 0xbdd072fc, 3 },
    { 0xaa6c72fd, 4 },
    { 0x380872fe, 3 },
    { 0, 0 },
    { 0x49147300, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494c7305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bc7308, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124730d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891c7313, 3 },
    { 0x85bc7313, 3 },
    { 0x8adc7315, 3 },
    { 0, 0 },
    { 0x27987317, 3 },
    { 0x4cdc7318, 3 },
    { 0x13387319, 3 },
    { 0x1be07319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d4731e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fec7322, 3 },
    { 0x10887323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c7327, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d4732a, 4 },
    { 0x8a10732b, 3 },
    { 0x9cc8732b, 3 },
    { 0x7f1c732a, 3 },
    { 0x4c38732b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f87333, 3 },
    { 0xbe987333, 3 },
    { 0x19b07333, 3 },
    { 0x90447333, 3 },
    { 0x31a07333, 3 },
    { 0xe6547338, 3 },
    { 0x5b87339, 5 },
    { 0, 0 },
    { 0xfd50733b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f07342, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54c7345, 3 },
    { 0, 0 },
    { 0xf2287347, 3 },
    { 0xd17c7347, 3 },
    { 0xd92c7349, 3 },
    { 0xd0e47349, 3 },
    { 0x8270734b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b4734e, 4 },
    { 0x99e8734e, 3 },
    { 0x16a07350, 3 },
    { 0x38707351, 3 },
    { 0xbd147352, 3 },
    { 0, 0 },
    { 0xbee87354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5590735b, 3 },
    { 0, 0 },
    { 0x2ac8735d, 3 },
    { 0x58c4735d, 3 },
    { 0x4144735d, 4 },
    { 0x8e10735e, 3 },
    { 0xa7bc735e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3307367, 3 },
    { 0x63d87368, 3 },
    { 0xc0787369, 3 },
    { 0, 0 },
    { 0x3978736b, 3 },
    { 0, 0 },
    { 0x298c736d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33247376, 3 },
    { 0xad647376, 3 },
    { 0x66947376, 3 },
    { 0x2cc07378, 4 },
    { 0, 0 },
    { 0x781c737b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98c737f, 3 },
    { 0, 0 },
    { 0x8b87381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea4738c, 3 },
    { 0xb8c738c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b087390, 3 },
    { 0, 0 },
    { 0xcb4c7392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccc7395, 3 },
    { 0x93c47396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f107399, 3 },
    { 0, 0 },
    { 0x71f4739b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c8739e, 3 },
    { 0xe67c739e, 4 },
    { 0x2f04739f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1873aa, 3 },
    { 0xc76c73ab, 4 },
    { 0xdb473ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61473af, 3 },
    { 0, 0 },
    { 0xd39873b1, 4 },
    { 0x939473b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599873b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x143873bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf873bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5073c4, 3 },
    { 0x3fb873c4, 3 },
    { 0x8adc73c4, 3 },
    { 0x4bf473c7, 3 },
    { 0x905c73c8, 3 },
    { 0x7b1c73c8, 4 },
    { 0xd57073c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4473d2, 4 },
    { 0, 0 },
    { 0xc58c73d4, 3 },
    { 0x8e3473d5, 4 },
    { 0xc95873d6, 3 },
    { 0xe8e473d7, 3 },
    { 0, 0 },
    { 0x2dc73d9, 4 },
    { 0x9a7873d9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826873df, 4 },
    { 0xe96873e0, 3 },
    { 0xa5f873e1, 3 },
    { 0, 0 },
    { 0xfe1073e3, 3 },
    { 0, 0 },
    { 0x522073e5, 3 },
    { 0x253873e6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b873e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7873f3, 3 },
    { 0x4a5473f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722073f8, 3 },
    { 0x8f073f9, 3 },
    { 0x428c73f8, 3 },
    { 0x9f2c73fb, 4 },
    { 0x63f873fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c473ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7c7404, 3 },
    { 0, 0 },
    { 0xc5647406, 3 },
    { 0x62747407, 3 },
    { 0x4e807408, 4 },
    { 0, 0 },
    { 0x2c2c740a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b44740f, 4 },
    { 0x3b04740f, 3 },
    { 0x6240740f, 3 },
    { 0x746c7411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cc7416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x783c7419, 4 },
    { 0x1790741a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb0741d, 3 },
    { 0x67d4741d, 3 },
    { 0xadb0741d, 3 },
    { 0x723c7420, 4 },
    { 0x23387420, 3 },
    { 0x323c741f, 3 },
    { 0, 0 },
    { 0x9a807424, 3 },
    { 0x37b87425, 3 },
    { 0x31347426, 3 },
    { 0xfa787427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fc742a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e8742f, 3 },
    { 0, 0 },
    { 0xbc807431, 3 },
    { 0x787432, 3 },
    { 0, 0 },
    { 0x7b747434, 3 },
    { 0x90e07435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb020743a, 3 },
    { 0xda6c743b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dc7441, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c87446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789c7451, 4 },
    { 0, 0 },
    { 0xc2f47453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18487458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03c745b, 3 },
    { 0xda3c745b, 3 },
    { 0x9a4c745d, 3 },
    { 0x1044745e, 3 },
    { 0xeaf8745d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc7462, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12847468, 3 },
    { 0x9d9c7468, 3 },
    { 0x36d0746a, 5 },
    { 0xb72c746b, 3 },
    { 0, 0 },
    { 0x4270746d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c387470, 3 },
    { 0x61a47471, 3 },
    { 0x3b47472, 3 },
    { 0, 0 },
    { 0xffd87474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904747d, 4 },
    { 0xdb54747e, 4 },
    { 0xde68747d, 3 },
    { 0x5da87480, 3 },
    { 0x796c7481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e47484, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae247488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb178748d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fc7490, 3 },
    { 0x8547491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c749e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27474a4, 3 },
    { 0x5dc874a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x446c74a8, 3 },
    { 0x3ab074a9, 3 },
    { 0xead074aa, 4 },
    { 0xdcac74aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba874b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9074b3, 4 },
    { 0, 0 },
    { 0xd1dc74b5, 4 },
    { 0x5c0074b6, 3 },
    { 0x404074b5, 4 },
    { 0x3f8874b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42074bc, 4 },
    { 0, 0 },
    { 0xccc74be, 3 },
    { 0x112474be, 3 },
    { 0x51fc74c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8874c4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1474c7, 3 },
    { 0xaa2c74c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4074cd, 3 },
    { 0x7f7874cd, 3 },
    { 0x93c474cd, 3 },
    { 0x81874d0, 3 },
    { 0, 0 },
    { 0xc2c074d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abc74dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a874e0, 3 },
    { 0xf6474e0, 3 },
    { 0x412074e2, 3 },
    { 0xce6474e3, 3 },
    { 0x465c74e2, 3 },
    { 0x177874e5, 3 },
    { 0xd5b474e5, 3 },
    { 0, 0 },
    { 0x4f3474e8, 3 },
    { 0x5e2c74e9, 4 },
    { 0x5e8074e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cc74ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e474ff, 3 },
    { 0x71ec74ff, 4 },
    { 0xf84874ff, 3 },
    { 0x6bd07502, 3 },
    { 0xc2cc7500, 4 },
    { 0, 0 },
    { 0xe9647505, 3 },
    { 0, 0 },
    { 0xcab07507, 3 },
    { 0xef3c7508, 3 },
    { 0x27587507, 3 },
    { 0, 0 },
    { 0x9da4750b, 3 },
    { 0xa80c750c, 3 },
    { 0x714c750d, 3 },
    { 0, 0 },
    { 0xfe0c750f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4747516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7258751a, 3 },
    { 0xc4c8751a, 3 },
    { 0xc0a8751c, 3 },
    { 0xb580751d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c7523, 3 },
    { 0, 0 },
    { 0x5bec7525, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc87528, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc470752e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7187531, 3 },
    { 0xf87532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf07535, 5 },
    { 0x218c7536, 3 },
    { 0, 0 },
    { 0xad287538, 3 },
    { 0xeb047539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e07541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5c7544, 3 },
    { 0xf2907545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b47548, 4 },
    { 0xf9187548, 3 },
    { 0, 0 },
    { 0xf160754b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a8754f, 3 },
    { 0, 0 },
    { 0x3ccc7551, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2c7556, 3 },
    { 0xa7bc7557, 3 },
    { 0xa8ec7558, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59c755b, 3 },
    { 0xb250755c, 3 },
    { 0xa71c755d, 4 },
    { 0, 0 },
    { 0x8e58755f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bc7564, 4 },
    { 0xd0d87565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c07568, 3 },
    { 0, 0 },
    { 0xda80756a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0756d, 3 },
    { 0, 0 },
    { 0xee8756f, 3 },
    { 0x35e87570, 3 },
    { 0x6e447570, 3 },
    { 0xf4ac756f, 3 },
    { 0x79887573, 3 },
    { 0x7f087574, 3 },
    { 0, 0 },
    { 0xe0b47576, 5 },
    { 0x383c7576, 3 },
    { 0x5b907578, 3 },
    { 0xcf647579, 3 },
    { 0x12707579, 3 },
    { 0x5280757b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcc757e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7c7581, 3 },
    { 0, 0 },
    { 0x329c7583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f47586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d98758b, 4 },
    { 0x5784758c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7347590, 3 },
    { 0xc9147591, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27607594, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc787597, 3 },
    { 0x84b87597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc759c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7075a0, 4 },
    { 0x77875a0, 3 },
    { 0x932075a2, 4 },
    { 0x700875a3, 3 },
    { 0xd56c75a4, 3 },
    { 0x612075a5, 3 },
    { 0xfd3875a4, 3 },
    { 0xba4475a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0c75aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5075af, 3 },
    { 0, 0 },
    { 0xc74875b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bec75b5, 3 },
    { 0, 0 },
    { 0x153475b7, 4 },
    { 0xe43475b8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cc75bd, 3 },
    { 0, 0 },
    { 0xfaec75bf, 4 },
    { 0x998075c0, 4 },
    { 0x306475c1, 3 },
    { 0x742875c2, 3 },
    { 0xaeb875c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x361475c6, 4 },
    { 0, 0 },
    { 0xcce475c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da075ce, 3 },
    { 0xfb875cf, 3 },
    { 0x6b4075d0, 3 },
    { 0x562075d0, 3 },
    { 0xaa2c75d2, 3 },
    { 0xe9c875d3, 3 },
    { 0x622c75d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c75d9, 3 },
    { 0, 0 },
    { 0x4dc875db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b475de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0c75e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2475ea, 3 },
    { 0x3c75eb, 4 },
    { 0xe15475eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x924075ef, 3 },
    { 0x6ab875f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b075f4, 3 },
    { 0xcd5875f5, 3 },
    { 0x3ac075f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95475fb, 3 },
    { 0xb39075fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce07601, 3 },
    { 0x76d47601, 4 },
    { 0x6ce07603, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa47607, 4 },
    { 0xa0087608, 4 },
    { 0xa7807609, 3 },
    { 0, 0 },
    { 0xc9c760b, 3 },
    { 0x1a44760c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b98760f, 3 },
    { 0, 0 },
    { 0xeb2c7611, 4 },
    { 0x19687611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a847617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df87628, 3 },
    { 0x5b0c7629, 3 },
    { 0xa2507628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf568762d, 3 },
    { 0x26b4762e, 3 },
    { 0xb2fc762f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12387634, 3 },
    { 0, 0 },
    { 0x16bc7636, 3 },
    { 0xf17c7637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a0763b, 3 },
    { 0, 0 },
    { 0xd070763d, 4 },
    { 0x8e78763e, 3 },
    { 0x9944763d, 3 },
    { 0xbfa07640, 4 },
    { 0xcee07641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa620764a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8764d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb107654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5174765c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85787666, 3 },
    { 0, 0 },
    { 0xe3787668, 3 },
    { 0, 0 },
    { 0x27fc766a, 4 },
    { 0, 0 },
    { 0x97bc766c, 4 },
    { 0, 0 },
    { 0x3c98766e, 4 },
    { 0xc4b8766f, 4 },
    { 0, 0 },
    { 0x16147671, 4 },
    { 0, 0 },
    { 0x97747673, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23607679, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7468767d, 3 },
    { 0, 0 },
    { 0xdb00767f, 3 },
    { 0x4d407680, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe587688, 3 },
    { 0xfee07689, 3 },
    { 0, 0 },
    { 0x3014768b, 3 },
    { 0x5cd4768c, 4 },
    { 0, 0 },
    { 0x7fe4768e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95387691, 3 },
    { 0, 0 },
    { 0x50247693, 3 },
    { 0, 0 },
    { 0x56a87695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc0769c, 3 },
    { 0x7a68769c, 3 },
    { 0, 0 },
    { 0x1714769f, 3 },
    { 0x34a8769f, 3 },
    { 0, 0 },
    { 0xea4876a2, 3 },
    { 0, 0 },
    { 0xdb9076a4, 4 },
    { 0x722076a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2076a9, 3 },
    { 0xd16c76aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76ae, 3 },
    { 0, 0 },
    { 0xcd5076b0, 3 },
    { 0x1d8c76b1, 3 },
    { 0xa52476b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0476b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa476b9, 3 },
    { 0x8bac76ba, 3 },
    { 0x177c76b9, 3 },
    { 0xbd4c76bb, 3 },
    { 0x968c76bd, 3 },
    { 0x4d1c76be, 3 },
    { 0xa93476b9, 3 },
    { 0x2bdc76bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d476c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee076c8, 3 },
    { 0, 0 },
    { 0xa85876ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9476ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82476d2, 3 },
    { 0x560076d3, 3 },
    { 0xeb5476d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87476d8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf076db, 3 },
    { 0x444876dc, 3 },
    { 0, 0 },
    { 0xb42c76de, 4 },
    { 0x981476de, 3 },
    { 0x2fa076e0, 4 },
    { 0x840076e1, 3 },
    { 0, 0 },
    { 0x86b476e3, 3 },
    { 0x651076e4, 3 },
    { 0x563876e5, 3 },
    { 0xdae876e6, 4 },
    { 0x51b876e7, 3 },
    { 0, 0 },
    { 0x819076e9, 4 },
    { 0x76f876ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808476f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635c76f5, 3 },
    { 0, 0 },
    { 0xdaec76f7, 4 },
    { 0x28f076f8, 3 },
    { 0x458876f9, 3 },
    { 0, 0 },
    { 0xf3b876fb, 3 },
    { 0x93876fc, 3 },
    { 0x9f9c76fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd047700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2047704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8270770d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91847712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe707715, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b847718, 4 },
    { 0, 0 },
    { 0x9a28771a, 3 },
    { 0xa150771b, 3 },
    { 0xa808771b, 4 },
    { 0, 0 },
    { 0x9b28771e, 3 },
    { 0xef88771f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46647725, 3 },
    { 0xf62c7726, 3 },
    { 0, 0 },
    { 0xf2347728, 3 },
    { 0xb2e07729, 3 },
    { 0x3728772a, 4 },
    { 0x294c772a, 3 },
    { 0xf340772c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe264772f, 4 },
    { 0x16f87730, 3 },
    { 0, 0 },
    { 0xa7ac7732, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7107736, 4 },
    { 0, 0 },
    { 0x2f9c7738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd34773e, 3 },
    { 0x870c773f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8907742, 3 },
    { 0x2907742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c547749, 3 },
    { 0x2658774a, 3 },
    { 0xbb24774b, 3 },
    { 0x19047749, 3 },
    { 0, 0 },
    { 0x80d4774e, 4 },
    { 0x609c774f, 4 },
    { 0x8cf07750, 3 },
    { 0xbfac774e, 3 },
    { 0x2487752, 3 },
    { 0x2fc47753, 3 },
    { 0, 0 },
    { 0xbfc07755, 3 },
    { 0x65d07755, 3 },
    { 0x66dc7756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f0775f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f47765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb07768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da8776c, 3 },
    { 0x50c4776d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32687771, 3 },
    { 0, 0 },
    { 0x7a207773, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba607778, 3 },
    { 0x3f147779, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90777c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc87782, 3 },
    { 0, 0 },
    { 0x13147784, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f87787, 4 },
    { 0x7ec07788, 3 },
    { 0xcd887789, 3 },
    { 0x95487789, 3 },
    { 0xc7247789, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb68778e, 3 },
    { 0x19cc778f, 3 },
    { 0xaa2c7790, 4 },
    { 0xb2507791, 4 },
    { 0x52b07791, 4 },
    { 0x9d907793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26047798, 3 },
    { 0xe4507799, 3 },
    { 0x7690779a, 3 },
    { 0x60ec779b, 3 },
    { 0xf908779c, 3 },
    { 0xe658779d, 3 },
    { 0xf9e8779e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea077a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258077ae, 3 },
    { 0, 0 },
    { 0x863077b0, 3 },
    { 0, 0 },
    { 0xe5e077b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c077b5, 3 },
    { 0, 0 },
    { 0xc0d877b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba077c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28477c5, 3 },
    { 0xb4fc77c6, 3 },
    { 0xa86077c7, 3 },
    { 0x29477c8, 4 },
    { 0x816877c9, 4 },
    { 0, 0 },
    { 0x53b477cb, 3 },
    { 0x3da077cb, 3 },
    { 0x6c8477cc, 3 },
    { 0x588077ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7077d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0c77d7, 3 },
    { 0xbe1077d8, 4 },
    { 0xced877d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4877dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7877e4, 4 },
    { 0x6af077e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913c77eb, 3 },
    { 0x6f0c77ec, 5 },
    { 0x32e877ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e877f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0477f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347877fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29247800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e87804, 3 },
    { 0, 0 },
    { 0x6fd47806, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50987809, 3 },
    { 0x23c4780a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572c780f, 3 },
    { 0x6dac780f, 3 },
    { 0, 0 },
    { 0x7c907812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf4781a, 5 },
    { 0x8d74781a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3887820, 3 },
    { 0x26bc7821, 3 },
    { 0x40147822, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b07825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc690782b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea307832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0783a, 4 },
    { 0xd28783b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0783e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e47841, 3 },
    { 0x74247842, 3 },
    { 0, 0 },
    { 0x1dfc7844, 4 },
    { 0x41f87845, 3 },
    { 0, 0 },
    { 0xfac87847, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x959c784a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c8784e, 3 },
    { 0x1cdc784f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaac7852, 4 },
    { 0xef387853, 3 },
    { 0xa3847854, 3 },
    { 0x92d47854, 3 },
    { 0x43587856, 3 },
    { 0x8a447857, 4 },
    { 0x8ed47858, 3 },
    { 0x6b347859, 4 },
    { 0xdd87852, 3 },
    { 0, 0 },
    { 0xf4bc785c, 3 },
    { 0xa7e8785d, 3 },
    { 0x3c34785d, 3 },
    { 0xdffc785f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b47864, 3 },
    { 0xa9547865, 3 },
    { 0x89947864, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c786c, 3 },
    { 0x7d84786c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6907870, 3 },
    { 0x22ec7871, 4 },
    { 0, 0 },
    { 0xd3007873, 3 },
    { 0xa2ec7873, 3 },
    { 0, 0 },
    { 0x5a87876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c787c, 4 },
    { 0xe30787c, 5 },
    { 0, 0 },
    { 0xc0cc787f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48147883, 4 },
    { 0x23507884, 3 },
    { 0x58947883, 3 },
    { 0xb3887886, 3 },
    { 0x365c7887, 3 },
    { 0xe0f87887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884c788d, 3 },
    { 0x7030788d, 3 },
    { 0xdb48788f, 4 },
    { 0x1a64788e, 4 },
    { 0, 0 },
    { 0xb4f87892, 4 },
    { 0, 0 },
    { 0xd37c7894, 4 },
    { 0xfee07895, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa4789d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4c78a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a478a6, 3 },
    { 0x96dc78a6, 4 },
    { 0xe88878a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139c78b4, 3 },
    { 0, 0 },
    { 0xbf4078b6, 3 },
    { 0x7bc78b7, 3 },
    { 0x191478b6, 3 },
    { 0xe40478b9, 3 },
    { 0x90ac78b9, 5 },
    { 0xe9a878ba, 4 },
    { 0x723c78bc, 4 },
    { 0x313c78bd, 3 },
    { 0x33f078bc, 3 },
    { 0xfb9478bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b478c3, 4 },
    { 0, 0 },
    { 0x5dc478c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1078c8, 4 },
    { 0x2c6c78c9, 3 },
    { 0xbda878c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d078d0, 4 },
    { 0xff0c78d0, 3 },
    { 0x748878d2, 4 },
    { 0x955078d1, 3 },
    { 0, 0 },
    { 0xed2878d5, 3 },
    { 0xa2dc78d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67478d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4878de, 4 },
    { 0x3e6478df, 3 },
    { 0, 0 },
    { 0x561478e1, 3 },
    { 0x3a478e1, 3 },
    { 0x6ff478e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac78e7, 3 },
    { 0x63478e8, 4 },
    { 0, 0 },
    { 0xe08478ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67478ed, 3 },
    { 0, 0 },
    { 0x697c78ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1878f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c078f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a078ff, 3 },
    { 0xdb3478ff, 3 },
    { 0x33947900, 3 },
    { 0xfa187902, 3 },
    { 0, 0 },
    { 0xfa787904, 3 },
    { 0x93147905, 3 },
    { 0x14cc7904, 3 },
    { 0x26f07906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640790d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8c7912, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc7915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a87921, 3 },
    { 0, 0 },
    { 0x66f07923, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c07926, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07931, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d07934, 3 },
    { 0xc8247934, 3 },
    { 0, 0 },
    { 0x1d1c7937, 3 },
    { 0xb6407937, 3 },
    { 0, 0 },
    { 0x3c1c793a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c793f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e07942, 4 },
    { 0x36047942, 3 },
    { 0x6a87944, 3 },
    { 0xc7e07942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78794c, 3 },
    { 0x36f8794c, 3 },
    { 0xa724794e, 3 },
    { 0xc88794d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x727c7952, 3 },
    { 0xc2f47952, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11507957, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d24795d, 3 },
    { 0x4794795e, 3 },
    { 0x3198795f, 3 },
    { 0x6574795f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87947963, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e287967, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc796c, 3 },
    { 0x4594796c, 3 },
    { 0, 0 },
    { 0x3f54796f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8847974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb87979, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c4797d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc7980, 4 },
    { 0xcecc7981, 3 },
    { 0, 0 },
    { 0x900c7983, 3 },
    { 0x3ba47984, 3 },
    { 0x3cf87984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c798b, 4 },
    { 0xcc0798c, 3 },
    { 0xc714798c, 3 },
    { 0, 0 },
    { 0x97ec798f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f47999, 3 },
    { 0, 0 },
    { 0x35d8799b, 4 },
    { 0x804c799c, 4 },
    { 0x73e4799d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x459479a0, 4 },
    { 0, 0 },
    { 0x495079a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x172479a5, 3 },
    { 0xcad479a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c79aa, 3 },
    { 0, 0 },
    { 0x361479ac, 3 },
    { 0, 0 },
    { 0x3d4079ae, 3 },
    { 0x6ec479af, 3 },
    { 0, 0 },
    { 0x599079b1, 3 },
    { 0, 0 },
    { 0x411079b3, 3 },
    { 0, 0 },
    { 0xbe7079b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd879bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc79c4, 3 },
    { 0x3a4c79c5, 3 },
    { 0xf1c79c6, 4 },
    { 0, 0 },
    { 0xd4d079c8, 3 },
    { 0xab6079c9, 3 },
    { 0xc64c79c9, 4 },
    { 0x405879c9, 3 },
    { 0x80a479cc, 3 },
    { 0, 0 },
    { 0x535079ce, 3 },
    { 0, 0 },
    { 0x274479d0, 3 },
    { 0xa36079d1, 4 },
    { 0x36a079d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dc79d6, 3 },
    { 0x9d5879d7, 4 },
    { 0, 0 },
    { 0x2ff879d9, 4 },
    { 0xdd9c79d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28079dd, 4 },
    { 0xcb3c79dd, 3 },
    { 0xd84479df, 3 },
    { 0xc4d079df, 3 },
    { 0x336079e1, 4 },
    { 0, 0 },
    { 0xd89479e3, 4 },
    { 0x69c79e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4479e8, 3 },
    { 0x5d4879e9, 3 },
    { 0xaabc79ea, 3 },
    { 0xb53079e9, 3 },
    { 0xf8cc79ea, 3 },
    { 0x6fac79ed, 3 },
    { 0, 0 },
    { 0x9b079ef, 3 },
    { 0x1e7079ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d879f3, 3 },
    { 0xaa0c79f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0879f7, 4 },
    { 0xb69479f8, 3 },
    { 0x83ac79f7, 3 },
    { 0xbbc879f9, 3 },
    { 0x81d879fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd47a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff587a12, 4 },
    { 0xdde87a13, 4 },
    { 0x94a07a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f07a1a, 3 },
    { 0xaf207a1a, 3 },
    { 0, 0 },
    { 0x6cc87a1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6907a21, 3 },
    { 0x7f647a21, 4 },
    { 0, 0 },
    { 0xaf007a24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b607a28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbc7a2b, 4 },
    { 0, 0 },
    { 0x9ce47a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e387a32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36087a39, 3 },
    { 0, 0 },
    { 0x20487a3b, 3 },
    { 0x3a387a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c7a45, 4 },
    { 0, 0 },
    { 0xa747a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be87a4e, 3 },
    { 0x25147a4f, 3 },
    { 0xcbac7a50, 4 },
    { 0, 0 },
    { 0x7a8c7a52, 3 },
    { 0xb02c7a53, 3 },
    { 0xd7f87a54, 4 },
    { 0x69587a55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91c7a59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57247a5d, 3 },
    { 0x5d187a5e, 3 },
    { 0xbbf07a5d, 3 },
    { 0x3c287a5e, 3 },
    { 0x9fdc7a5f, 4 },
    { 0xfc207a62, 4 },
    { 0, 0 },
    { 0xe1207a64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62407a68, 3 },
    { 0, 0 },
    { 0x2f387a6a, 3 },
    { 0x6d887a6a, 5 },
    { 0xca887a6c, 3 },
    { 0x906c7a6c, 3 },
    { 0xb9d87a6e, 3 },
    { 0xc0ec7a6c, 3 },
    { 0, 0 },
    { 0x51447a71, 4 },
    { 0x45387a72, 3 },
    { 0, 0 },
    { 0xa8887a74, 3 },
    { 0xbd4c7a75, 4 },
    { 0x1b07a76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x582c7a79, 3 },
    { 0, 0 },
    { 0xe4a87a7b, 3 },
    { 0x8ca47a7c, 3 },
    { 0xb3507a7d, 4 },
    { 0xac87a7c, 3 },
    { 0xc9687a7f, 4 },
    { 0x3f587a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c7a83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7c7a86, 3 },
    { 0x66547a86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e07a8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93907a8e, 3 },
    { 0x9ecc7a8e, 3 },
    { 0x26947a90, 3 },
    { 0x4e187a90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e47a94, 3 },
    { 0xb9c07a95, 3 },
    { 0x3887a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c7a9e, 4 },
    { 0xae5c7a9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77247aa4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5907aa7, 4 },
    { 0xe2c07aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e47aab, 4 },
    { 0xd2387aab, 3 },
    { 0x6f387aac, 3 },
    { 0x82347aae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4c7ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df87ab7, 4 },
    { 0, 0 },
    { 0xa8247ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c107abc, 4 },
    { 0x203c7abc, 3 },
    { 0xa5c47abd, 3 },
    { 0, 0 },
    { 0xd5447ac0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c7ac3, 3 },
    { 0, 0 },
    { 0x80987ac5, 3 },
    { 0, 0 },
    { 0x7a2c7ac7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80907aca, 3 },
    { 0xa63c7aca, 3 },
    { 0xf17c7acc, 4 },
    { 0xaa207acd, 3 },
    { 0x26a87acd, 5 },
    { 0xcec47acc, 3 },
    { 0x72447acc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffc7ad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6987ad6, 3 },
    { 0xad8c7ad7, 3 },
    { 0x9aa47ad8, 4 },
    { 0, 0 },
    { 0x77287ada, 4 },
    { 0xf57c7ada, 4 },
    { 0x41787adc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1747adf, 3 },
    { 0, 0 },
    { 0x3d947ae1, 3 },
    { 0, 0 },
    { 0x70747ae3, 4 },
    { 0x71147ae4, 3 },
    { 0x4a387ae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3087ae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d747aee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d87af3, 3 },
    { 0xd43c7af4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9307af8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82c7afb, 3 },
    { 0xcfb07afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccc7aff, 3 },
    { 0, 0 },
    { 0x6c647b01, 3 },
    { 0x65f07b01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c7b0c, 3 },
    { 0, 0 },
    { 0xfc007b0e, 3 },
    { 0, 0 },
    { 0xe4487b10, 3 },
    { 0, 0 },
    { 0xebbc7b12, 3 },
    { 0x6c3c7b12, 3 },
    { 0, 0 },
    { 0x3a487b15, 3 },
    { 0x81687b15, 3 },
    { 0x5b247b15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8407b1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c7b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3407b24, 3 },
    { 0xa3707b24, 4 },
    { 0x42507b26, 3 },
    { 0x79dc7b25, 3 },
    { 0, 0 },
    { 0x7cd47b29, 3 },
    { 0xdd847b2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb787b2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc7b32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d47b36, 4 },
    { 0xff787b36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cac7b3a, 4 },
    { 0x22a07b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c87b3f, 3 },
    { 0xa2187b3f, 3 },
    { 0, 0 },
    { 0x184c7b42, 3 },
    { 0x2ddc7b42, 3 },
    { 0xca4c7b44, 4 },
    { 0xd5e47b45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c07b48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747b4f, 3 },
    { 0xce447b4f, 4 },
    { 0xd1c7b50, 4 },
    { 0xfb347b52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d847b59, 4 },
    { 0, 0 },
    { 0xa0c07b5b, 4 },
    { 0xc4887b5c, 3 },
    { 0xb66c7b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f387b60, 3 },
    { 0xb0107b61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81c7b74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd607b77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708c7b7d, 3 },
    { 0x20a47b7e, 3 },
    { 0xc9387b7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a07b8b, 3 },
    { 0xa3707b8c, 3 },
    { 0x8f8c7b8b, 3 },
    { 0x763c7b8c, 3 },
    { 0xe3a47b8d, 3 },
    { 0x32287b8d, 3 },
    { 0x17687b91, 3 },
    { 0x7a7c7b92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007b95, 3 },
    { 0xf8307b96, 3 },
    { 0xcdf07b97, 3 },
    { 0xbaf07b97, 4 },
    { 0x49047b99, 3 },
    { 0x9aec7b99, 3 },
    { 0x3e8c7b9b, 3 },
    { 0x6b547b9a, 4 },
    { 0, 0 },
    { 0xf2f87b9e, 3 },
    { 0, 0 },
    { 0x19ec7ba0, 4 },
    { 0, 0 },
    { 0xa5247ba2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d47ba8, 4 },
    { 0xe37c7ba9, 3 },
    { 0x57d47baa, 3 },
    { 0x22747ba8, 3 },
    { 0, 0 },
    { 0xf1a47bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed587bb1, 4 },
    { 0x7ec87bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c7bba, 4 },
    { 0, 0 },
    { 0x727c7bbc, 3 },
    { 0x8fdc7bbc, 4 },
    { 0x93a07bbe, 3 },
    { 0x19587bbf, 4 },
    { 0xa0887bbc, 3 },
    { 0x900c7bc1, 3 },
    { 0x66e47bc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f07bc6, 3 },
    { 0x66147bc7, 3 },
    { 0xe26c7bc7, 3 },
    { 0xbce07bc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a47bcd, 3 },
    { 0x55cc7bce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f87bd1, 3 },
    { 0x17607bd1, 4 },
    { 0xc3dc7bd3, 5 },
    { 0x533c7bd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec087bd8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24247bdb, 3 },
    { 0x64a87bdc, 3 },
    { 0xbd447bdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7147be3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac07beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7607bf0, 3 },
    { 0, 0 },
    { 0xe8ac7bf2, 3 },
    { 0xb5807bf3, 4 },
    { 0, 0 },
    { 0x9c587bf5, 3 },
    { 0x39b87bf6, 3 },
    { 0x12107bf5, 3 },
    { 0xa2e87bf8, 4 },
    { 0, 0 },
    { 0x1ba87bfa, 3 },
    { 0x30547bfb, 3 },
    { 0xf2c47bfc, 5 },
    { 0x87907bfd, 3 },
    { 0xdde47bfe, 3 },
    { 0x90947bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78207c02, 3 },
    { 0, 0 },
    { 0xe4b07c04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf207c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae87c0c, 4 },
    { 0x8cf87c0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7947c11, 3 },
    { 0, 0 },
    { 0xf07c7c13, 4 },
    { 0x6a187c14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd07c1d, 3 },
    { 0x75647c1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc647c21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7c7c26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c07c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed807c2d, 3 },
    { 0x266c7c2d, 4 },
    { 0x11207c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4487c33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1c7c39, 3 },
    { 0, 0 },
    { 0xed187c3b, 3 },
    { 0x34607c3c, 3 },
    { 0xac8c7c3c, 4 },
    { 0xf0907c3e, 3 },
    { 0x47187c3e, 3 },
    { 0x584c7c3f, 3 },
    { 0, 0 },
    { 0x98a87c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae47c4b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c47c4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a87c56, 3 },
    { 0xc5c47c57, 3 },
    { 0xea1c7c57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc47c5b, 4 },
    { 0x33f07c5c, 3 },
    { 0, 0 },
    { 0xea487c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8207c61, 3 },
    { 0x999c7c62, 3 },
    { 0, 0 },
    { 0x11147c64, 3 },
    { 0x99c07c64, 3 },
    { 0x49247c66, 3 },
    { 0x33d07c66, 4 },
    { 0xd5f47c68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc87c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce507c6e, 3 },
    { 0, 0 },
    { 0xa9647c70, 3 },
    { 0xa5a07c71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127c7c78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc7c7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aec7c7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf07c82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64747c8d, 3 },
    { 0x45307c8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a187c96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a647ca0, 3 },
    { 0x3047ca0, 3 },
    { 0xe5707ca2, 3 },
    { 0, 0 },
    { 0xf1507ca4, 3 },
    { 0x281c7ca5, 3 },
    { 0x97647ca6, 3 },
    { 0xec947ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dac7cac, 3 },
    { 0xc8107cac, 3 },
    { 0x3f787cae, 4 },
    { 0xbd947caf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228c7cb5, 3 },
    { 0x98787cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f507cb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90007cbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e87cc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52987ccc, 3 },
    { 0, 0 },
    { 0xa4a07cce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb747cd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c07cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e547cd8, 4 },
    { 0, 0 },
    { 0x88c7cda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d87ce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab487ce4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3607ce7, 3 },
    { 0xf2647ce8, 4 },
    { 0xfba07ce9, 4 },
    { 0x64f47cea, 3 },
    { 0xa607cea, 3 },
    { 0xfe2c7cec, 3 },
    { 0x70c47cec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93207cf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e287cfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c7cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc7d03, 3 },
    { 0, 0 },
    { 0x5d287d05, 5 },
    { 0xa2e87d06, 4 },
    { 0xf46c7d06, 4 },
    { 0x61687d05, 4 },
    { 0x847c7d09, 3 },
    { 0xd6a07d0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8c7d0d, 3 },
    { 0x51c47d0e, 4 },
    { 0xd6f07d0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b07d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e87d16, 4 },
    { 0x46647d17, 5 },
    { 0, 0 },
    { 0xc2247d19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72747d21, 3 },
    { 0, 0 },
    { 0x3d887d23, 3 },
    { 0xb4787d24, 3 },
    { 0xdab47d25, 3 },
    { 0x7e987d25, 3 },
    { 0x44c47d25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f87d2c, 4 },
    { 0x3adc7d2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ac7d34, 3 },
    { 0x8f147d35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c7d3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc07d41, 3 },
    { 0xac907d42, 3 },
    { 0xa9847d43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a47d46, 4 },
    { 0, 0 },
    { 0x95047d48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a07d4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74607d52, 4 },
    { 0, 0 },
    { 0x7cdc7d54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59747d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c7d5e, 3 },
    { 0, 0 },
    { 0x3c7d60, 3 },
    { 0x29207d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x565c7d64, 5 },
    { 0, 0 },
    { 0xfa4c7d66, 3 },
    { 0, 0 },
    { 0x5a107d68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e47d6f, 4 },
    { 0x80987d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebec7d75, 3 },
    { 0x86007d76, 3 },
    { 0x9b547d77, 3 },
    { 0x5eb87d78, 3 },
    { 0, 0 },
    { 0x32107d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a87d83, 3 },
    { 0xb687d84, 3 },
    { 0x1f647d85, 4 },
    { 0xd9bc7d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f87d89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c7d8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe587d95, 4 },
    { 0, 0 },
    { 0x4af47d97, 3 },
    { 0xd6a87d97, 3 },
    { 0x81ec7d98, 3 },
    { 0xd0d07d9a, 3 },
    { 0, 0 },
    { 0x28ec7d9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f47d9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x69807da2, 3 },
    { 0x2347da3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868c7daa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc7db2, 3 },
    { 0x97647db2, 3 },
    { 0x28b87db4, 3 },
    { 0xfff47db5, 3 },
    { 0xc35c7db5, 5 },
    { 0xabcc7db3, 3 },
    { 0x44247db8, 4 },
    { 0x39547db8, 4 },
    { 0x6b507db9, 3 },
    { 0xc587db8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafc7dbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbc7dc3, 3 },
    { 0, 0 },
    { 0x5a2c7dc5, 3 },
    { 0xf0147dc5, 3 },
    { 0, 0 },
    { 0xe50c7dc8, 3 },
    { 0xb76c7dc9, 3 },
    { 0x39187dc9, 3 },
    { 0xed407dca, 3 },
    { 0x42f07dc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89347dd1, 3 },
    { 0x269c7dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b87dd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81247ddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac7de4, 3 },
    { 0xfe707de4, 5 },
    { 0x9c547de6, 4 },
    { 0, 0 },
    { 0x72f47de8, 3 },
    { 0, 0 },
    { 0x55807dea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b687dee, 3 },
    { 0xac547def, 4 },
    { 0x68687def, 3 },
    { 0x8b487dee, 4 },
    { 0x7a5c7df0, 4 },
    { 0, 0 },
    { 0xc99c7df4, 3 },
    { 0xaf807df4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc7dfa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8407dfd, 3 },
    { 0x78847dfe, 3 },
    { 0x1e2c7dff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d047e04, 4 },
    { 0x94787e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a47e0c, 3 },
    { 0, 0 },
    { 0xf5ac7e0e, 4 },
    { 0, 0 },
    { 0x7eb07e10, 3 },
    { 0x95e87e10, 4 },
    { 0x3ff47e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34207e16, 4 },
    { 0xe8247e17, 3 },
    { 0x1c407e18, 4 },
    { 0, 0 },
    { 0x3dd87e1a, 4 },
    { 0xbb647e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0c7e20, 4 },
    { 0x41047e21, 4 },
    { 0x73d07e22, 3 },
    { 0, 0 },
    { 0xe2a07e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37147e2d, 4 },
    { 0x3c287e2d, 3 },
    { 0x41387e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34447e32, 3 },
    { 0xfd547e33, 3 },
    { 0x562c7e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20c7e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff87e3e, 3 },
    { 0xce607e3f, 3 },
    { 0xde6c7e40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c7e45, 3 },
    { 0xf1c07e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab87e4a, 3 },
    { 0, 0 },
    { 0xb1a47e4c, 5 },
    { 0x3fe07e4d, 3 },
    { 0, 0 },
    { 0xe9607e4f, 3 },
    { 0, 0 },
    { 0x36fc7e51, 3 },
    { 0, 0 },
    { 0xbad47e53, 3 },
    { 0xba987e54, 3 },
    { 0x20347e55, 3 },
    { 0x53787e55, 3 },
    { 0x1d2c7e57, 3 },
    { 0, 0 },
    { 0x436c7e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea87e62, 4 },
    { 0x66ec7e63, 4 },
    { 0x3fa47e64, 4 },
    { 0, 0 },
    { 0xb51c7e66, 3 },
    { 0, 0 },
    { 0x32087e68, 3 },
    { 0x2cec7e69, 3 },
    { 0xd6147e69, 3 },
    { 0xa3bc7e6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ec7e71, 3 },
    { 0xf5f47e72, 3 },
    { 0xd4b87e73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b07e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff47e7f, 4 },
    { 0x26407e80, 3 },
    { 0x4dc7e81, 3 },
    { 0x80907e80, 3 },
    { 0xf60c7e83, 3 },
    { 0, 0 },
    { 0x3f547e85, 3 },
    { 0, 0 },
    { 0xb8007e87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41787e8a, 3 },
    { 0, 0 },
    { 0xdbb47e8c, 3 },
    { 0xd0ec7e8d, 4 },
    { 0x85b87e8e, 3 },
    { 0x8c047e8f, 3 },
    { 0xabd87e90, 4 },
    { 0xa3607e91, 3 },
    { 0x40e07e90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda987e9e, 3 },
    { 0, 0 },
    { 0x7e947ea0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b207ea7, 3 },
    { 0x8f387ea7, 3 },
    { 0x4b687ea9, 4 },
    { 0x86287eaa, 3 },
    { 0xa0b87eab, 3 },
    { 0x532c7eac, 4 },
    { 0x1ef47eac, 3 },
    { 0, 0 },
    { 0xeeb07eaf, 3 },
    { 0xf7647eaf, 3 },
    { 0xfdec7eb0, 3 },
    { 0, 0 },
    { 0xb0907eb3, 3 },
    { 0x19687eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fc7eb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb47ec0, 3 },
    { 0x41507ec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba07ec8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79787ecc, 3 },
    { 0x2cec7ecc, 3 },
    { 0xe907ece, 3 },
    { 0x67707ecf, 3 },
    { 0, 0 },
    { 0xa2987ed1, 3 },
    { 0xd9e07ed2, 3 },
    { 0xcc5c7ed3, 3 },
    { 0x36607ed1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2847ed8, 3 },
    { 0, 0 },
    { 0xdcc07eda, 4 },
    { 0x408c7edb, 3 },
    { 0xc68c7edc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb687ee2, 3 },
    { 0x15707ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf487ee6, 3 },
    { 0xd69c7ee6, 3 },
    { 0xdab07ee8, 3 },
    { 0x397c7ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7947ef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bc7ef5, 3 },
    { 0, 0 },
    { 0x803c7ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6987efa, 3 },
    { 0x4aa47efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccc7efe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64c7f05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f47f09, 3 },
    { 0, 0 },
    { 0x3b5c7f0b, 3 },
    { 0, 0 },
    { 0x65e87f0d, 3 },
    { 0xf6b87f0d, 3 },
    { 0xcd4c7f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b47f15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd887f1a, 3 },
    { 0, 0 },
    { 0x44087f1c, 3 },
    { 0, 0 },
    { 0x2b8c7f1e, 3 },
    { 0, 0 },
    { 0x4a007f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e087f23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7347f26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd447f32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23c7f35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd407f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b87f3b, 3 },
    { 0, 0 },
    { 0x7d4c7f3d, 5 },
    { 0, 0 },
    { 0xd7387f3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76747f42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2887f49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b47f4d, 3 },
    { 0x68407f4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14087f51, 3 },
    { 0x94b47f52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e87f58, 3 },
    { 0xb007f59, 4 },
    { 0x7107f58, 4 },
    { 0xff907f59, 4 },
    { 0x72607f5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5487f60, 4 },
    { 0xaa287f61, 3 },
    { 0x18807f61, 3 },
    { 0xdbe07f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d07f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c7f72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c7f78, 3 },
    { 0, 0 },
    { 0x42107f7a, 3 },
    { 0, 0 },
    { 0xcb6c7f7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6147f7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7487f83, 4 },
    { 0, 0 },
    { 0xf22c7f85, 3 },
    { 0x95747f85, 3 },
    { 0x2ae07f85, 3 },
    { 0xbdf07f86, 3 },
    { 0x4307f86, 3 },
    { 0xe9c7f8a, 3 },
    { 0, 0 },
    { 0xc6e47f8c, 3 },
    { 0x5db47f8d, 3 },
    { 0x290c7f8e, 3 },
    { 0xf987f8c, 4 },
    { 0x8347f90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18c7f94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf07f98, 4 },
    { 0xf9a07f98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc447f9c, 3 },
    { 0xf2107f9d, 3 },
    { 0xb4b87f9e, 3 },
    { 0, 0 },
    { 0x13d07fa0, 3 },
    { 0xf9807fa0, 3 },
    { 0xe8687fa2, 3 },
    { 0x10b07fa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c7fab, 3 },
    { 0xed3c7fac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83407fb1, 3 },
    { 0xf7907fb2, 3 },
    { 0xb0307fb3, 3 },
    { 0xd0647fb4, 4 },
    { 0x83747fb1, 3 },
    { 0, 0 },
    { 0xc75c7fb7, 3 },
    { 0xc5787fb8, 3 },
    { 0x7bf87fb8, 4 },
    { 0xd7287fba, 4 },
    { 0, 0 },
    { 0x29ac7fbc, 4 },
    { 0, 0 },
    { 0x10707fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df87fc1, 4 },
    { 0, 0 },
    { 0x5da47fc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f87fca, 3 },
    { 0, 0 },
    { 0x87c7fcc, 3 },
    { 0x8dd87fcd, 3 },
    { 0xfbbc7fcd, 3 },
    { 0, 0 },
    { 0xe4607fd0, 3 },
    { 0x213c7fd1, 3 },
    { 0xe1847fd2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadc7fd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c47fd9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a587fdc, 3 },
    { 0x21047fdd, 3 },
    { 0xe8907fde, 3 },
    { 0x404c7fde, 3 },
    { 0, 0 },
    { 0x88987fe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1807fe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac7fea, 5 },
    { 0, 0 },
    { 0x6ab47fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd07fef, 4 },
    { 0x2b787fef, 3 },
    { 0x63d87ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94007ff4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5007ff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c307ffb, 3 },
    { 0x25f87ffc, 4 },
    { 0xd8147ffd, 3 },
    { 0xf287ffd, 3 },
    { 0xb2587ffb, 3 },
    { 0x37408000, 3 },
    { 0xc1908001, 4 },
    { 0x5d748001, 3 },
    { 0x1b448001, 3 },
    { 0xecfc8004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9548007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f0800a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108800e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c708011, 4 },
    { 0x749c8011, 4 },
    { 0x4c388013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e08016, 3 },
    { 0, 0 },
    { 0xbd488018, 3 },
    { 0x354c8019, 3 },
    { 0xc500801a, 3 },
    { 0xd424801b, 4 },
    { 0xc70c801c, 3 },
    { 0xed58801b, 5 },
    { 0xf958801b, 3 },
    { 0, 0 },
    { 0x64088020, 3 },
    { 0xacb88021, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a248025, 3 },
    { 0x2dc08026, 3 },
    { 0, 0 },
    { 0x67dc8028, 3 },
    { 0x40448029, 4 },
    { 0x1adc802a, 3 },
    { 0, 0 },
    { 0xcb5c802c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4802f, 4 },
    { 0x3d248030, 3 },
    { 0xe41c802f, 3 },
    { 0, 0 },
    { 0x8b248033, 4 },
    { 0xbbc88033, 3 },
    { 0, 0 },
    { 0x99188036, 3 },
    { 0, 0 },
    { 0x50908038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4840803d, 3 },
    { 0xcdc803e, 3 },
    { 0x4460803f, 3 },
    { 0xb1ac8040, 3 },
    { 0x5d688040, 3 },
    { 0, 0 },
    { 0x3b208043, 3 },
    { 0, 0 },
    { 0xa4188045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2094804c, 4 },
    { 0x1b3c804d, 4 },
    { 0, 0 },
    { 0xbc7c804f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0805b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a58805e, 3 },
    { 0, 0 },
    { 0x3b0c8060, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0988066, 3 },
    { 0xa2ec8066, 3 },
    { 0x46808066, 4 },
    { 0x1c108066, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0806f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28807a, 4 },
    { 0x4e58807b, 4 },
    { 0xa1e8807c, 4 },
    { 0x51d8807a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfc8080, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2808083, 4 },
    { 0, 0 },
    { 0x80708085, 3 },
    { 0xf9cc8086, 4 },
    { 0x420c8087, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5908093, 3 },
    { 0xd3c48093, 4 },
    { 0xcd08095, 3 },
    { 0, 0 },
    { 0xd7608097, 3 },
    { 0xc948098, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6760809b, 3 },
    { 0, 0 },
    { 0x2798809d, 3 },
    { 0xd5b0809e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662480a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec080a8, 3 },
    { 0x14b480a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512080ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5c80b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1c80c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dac80c5, 3 },
    { 0x3fc480c5, 4 },
    { 0x8c3480c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa880cc, 3 },
    { 0x15a880cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46880d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9080e4, 3 },
    { 0x6f880e5, 4 },
    { 0xb5a880e6, 3 },
    { 0, 0 },
    { 0x804080e8, 4 },
    { 0x587880e9, 3 },
    { 0x7b0480ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5880f0, 3 },
    { 0, 0 },
    { 0x37dc80f2, 3 },
    { 0x637880f3, 3 },
    { 0x48f080f4, 3 },
    { 0xd83880f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c80f9, 4 },
    { 0x45b080fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc80fd, 4 },
    { 0, 0 },
    { 0xcd2480ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bc8103, 3 },
    { 0, 0 },
    { 0x96348105, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb08109, 4 },
    { 0, 0 },
    { 0xfe4810b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b88111, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cc8114, 3 },
    { 0xa1688115, 3 },
    { 0xb3708116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7348811c, 4 },
    { 0, 0 },
    { 0xe224811e, 3 },
    { 0xa49c811f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09c8122, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d88125, 4 },
    { 0x8e648125, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5584812a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4812d, 3 },
    { 0, 0 },
    { 0xa9e0812f, 3 },
    { 0x5dc0812f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee08134, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4813e, 3 },
    { 0x451c813f, 3 },
    { 0xc75c8140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6c8143, 4 },
    { 0x6c508144, 3 },
    { 0, 0 },
    { 0x91f48146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f88151, 4 },
    { 0x6d408152, 3 },
    { 0x1c6c8153, 3 },
    { 0xe2e88152, 3 },
    { 0x1b748155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b48158, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81848162, 4 },
    { 0x37c48162, 3 },
    { 0x6ed88164, 3 },
    { 0x798c8165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f64816d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d088173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a48177, 3 },
    { 0, 0 },
    { 0xd648179, 3 },
    { 0x324817a, 3 },
    { 0xb2b0817b, 3 },
    { 0x2f74817c, 3 },
    { 0xfedc817d, 4 },
    { 0x92a0817c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a88182, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e48187, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc818a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f50818e, 3 },
    { 0, 0 },
    { 0x4aa88190, 3 },
    { 0, 0 },
    { 0x38c08192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e10819b, 3 },
    { 0, 0 },
    { 0xa700819d, 3 },
    { 0x1640819e, 3 },
    { 0x9184819d, 4 },
    { 0x4da4819e, 3 },
    { 0xbe6c81a1, 3 },
    { 0x7df481a2, 4 },
    { 0x80e881a3, 3 },
    { 0x6a5881a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d881ad, 4 },
    { 0xfed881ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35081b3, 3 },
    { 0x61081b4, 4 },
    { 0x461081b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe081bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae081c5, 3 },
    { 0, 0 },
    { 0x695481c7, 3 },
    { 0xce1481c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0c81cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f881cf, 3 },
    { 0xd76c81d0, 3 },
    { 0x5dac81d0, 4 },
    { 0xbd0881d0, 3 },
    { 0xdb0081d3, 3 },
    { 0x1b5081d0, 3 },
    { 0, 0 },
    { 0xf9f081d6, 3 },
    { 0x2fc881d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822081dc, 3 },
    { 0xeb9881dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec881e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4481ec, 3 },
    { 0x813c81ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef881f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x852c81f4, 4 },
    { 0x9e9481f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17881f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5c81ff, 3 },
    { 0xa848200, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84188204, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9c8207, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8820a, 3 },
    { 0x1c88820b, 3 },
    { 0x1fbc820c, 3 },
    { 0, 0 },
    { 0x13e8820e, 3 },
    { 0x9c80820f, 4 },
    { 0x46248210, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4348214, 3 },
    { 0x45c48215, 3 },
    { 0, 0 },
    { 0xfbe48217, 3 },
    { 0x61c8218, 3 },
    { 0x901c8218, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a04821c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c8221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f08225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd88229, 4 },
    { 0x99ec8229, 3 },
    { 0, 0 },
    { 0xb624822c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda388232, 3 },
    { 0x26948233, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f0823a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee8823d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e08240, 4 },
    { 0xf2588241, 3 },
    { 0x9208242, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa4824a, 3 },
    { 0xd61c824a, 3 },
    { 0xfcd8824c, 3 },
    { 0x2594824d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69948254, 3 },
    { 0, 0 },
    { 0xaa108256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee0825a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f8825f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b088263, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793c826b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2628826e, 3 },
    { 0x431c826f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadc8275, 4 },
    { 0x38bc8275, 3 },
    { 0xc4488277, 3 },
    { 0x6aa08277, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be0827d, 3 },
    { 0, 0 },
    { 0x868c827f, 3 },
    { 0x66e08280, 3 },
    { 0, 0 },
    { 0x3dd48282, 3 },
    { 0xab248283, 3 },
    { 0, 0 },
    { 0x51788285, 3 },
    { 0x561c8286, 3 },
    { 0, 0 },
    { 0x3af88288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5374828c, 3 },
    { 0x7068828d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79c8293, 3 },
    { 0x86088293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f8829b, 4 },
    { 0x3088829c, 3 },
    { 0x52e0829d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76882a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f882a6, 3 },
    { 0xd7e882a7, 3 },
    { 0, 0 },
    { 0xd84482a9, 3 },
    { 0x633882aa, 3 },
    { 0xa82082aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ac82af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217482b4, 4 },
    { 0x20a482b5, 4 },
    { 0x8e1c82b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75c82bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9c82bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad082c7, 3 },
    { 0xcd1c82c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7082cd, 3 },
    { 0xc1f482ce, 3 },
    { 0, 0 },
    { 0xa5b082d0, 4 },
    { 0, 0 },
    { 0x356482d2, 3 },
    { 0x392082d2, 3 },
    { 0, 0 },
    { 0xccfc82d5, 4 },
    { 0x300c82d6, 3 },
    { 0x385482d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3082de, 4 },
    { 0x522082de, 3 },
    { 0x98a482de, 3 },
    { 0x973882e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fc82e4, 3 },
    { 0x68082e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba482ec, 4 },
    { 0x47ec82ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b882f1, 3 },
    { 0, 0 },
    { 0x9be482f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38082fd, 3 },
    { 0x95c082fd, 4 },
    { 0x9c2082fe, 3 },
    { 0, 0 },
    { 0xfb288301, 3 },
    { 0, 0 },
    { 0x5c488303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ec830c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd4830f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e44831a, 3 },
    { 0x7b5c831b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c8323, 4 },
    { 0xe0f88323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ec832f, 4 },
    { 0x2ff48330, 3 },
    { 0x48548330, 3 },
    { 0xbb448330, 4 },
    { 0x2a3c8333, 3 },
    { 0x45048333, 3 },
    { 0xe2548331, 3 },
    { 0xc4f0832f, 3 },
    { 0xfd44832f, 3 },
    { 0xde0c8338, 4 },
    { 0xd8dc8335, 3 },
    { 0x1d24833a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff308343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed048349, 3 },
    { 0xec24834a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4834e, 3 },
    { 0xda14834e, 4 },
    { 0x93e4834f, 3 },
    { 0x60c8351, 3 },
    { 0x44c48352, 4 },
    { 0x63388353, 4 },
    { 0x95288352, 3 },
    { 0, 0 },
    { 0x30748356, 3 },
    { 0xa7ec8357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1860835b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884c8364, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190c8369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc388374, 3 },
    { 0x1d2c8375, 4 },
    { 0, 0 },
    { 0xdca88377, 3 },
    { 0x99c8378, 3 },
    { 0xc6f08379, 4 },
    { 0, 0 },
    { 0x7f0c837b, 3 },
    { 0x38ac837c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa48380, 3 },
    { 0, 0 },
    { 0x4f988382, 3 },
    { 0x85e88383, 3 },
    { 0xf5d08382, 5 },
    { 0xd1608382, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52248389, 5 },
    { 0, 0 },
    { 0x2d90838b, 4 },
    { 0xe5d0838b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6608391, 3 },
    { 0x41c88392, 3 },
    { 0x1e788392, 3 },
    { 0xf0f48394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf288397, 4 },
    { 0, 0 },
    { 0xcba48399, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d54839c, 3 },
    { 0x4220839d, 3 },
    { 0xe844839e, 3 },
    { 0xcf28839e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60083a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ec83a6, 3 },
    { 0, 0 },
    { 0x8f9883a8, 3 },
    { 0, 0 },
    { 0x4ac883aa, 3 },
    { 0, 0 },
    { 0x2bf083ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e883b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14483b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c83b8, 3 },
    { 0, 0 },
    { 0x3e4883ba, 3 },
    { 0x461483ba, 5 },
    { 0x736083bc, 4 },
    { 0x181483bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f883cc, 3 },
    { 0xd06483cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79083d0, 3 },
    { 0xcccc83d1, 3 },
    { 0xc2ac83d1, 4 },
    { 0x878483d3, 3 },
    { 0xae6c83d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5483d7, 3 },
    { 0xecf083d8, 3 },
    { 0, 0 },
    { 0x397883da, 4 },
    { 0x8f8483da, 4 },
    { 0xa0a483da, 3 },
    { 0, 0 },
    { 0x195483de, 3 },
    { 0, 0 },
    { 0x98a883e0, 3 },
    { 0x206083e1, 3 },
    { 0, 0 },
    { 0xce2083e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8483e8, 3 },
    { 0xc9d083e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71c83fc, 4 },
    { 0x42dc83fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2788400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f48407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d8840b, 3 },
    { 0x4cf4840c, 4 },
    { 0, 0 },
    { 0xb98c840e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de88413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17c8419, 3 },
    { 0, 0 },
    { 0x1bc841b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d60841e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a08421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00c8427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa90842d, 3 },
    { 0, 0 },
    { 0x8c9c842f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb988432, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec688441, 3 },
    { 0, 0 },
    { 0xd0148443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7c8447, 3 },
    { 0xa2208447, 5 },
    { 0xd93c8448, 3 },
    { 0xeab08448, 5 },
    { 0x65a08447, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45948450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5088458, 4 },
    { 0, 0 },
    { 0x464c845a, 3 },
    { 0x9f10845b, 3 },
    { 0xdd58845c, 3 },
    { 0, 0 },
    { 0x80b4845e, 3 },
    { 0, 0 },
    { 0x24e48460, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13208468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c846c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd788470, 3 },
    { 0x55808471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08474, 3 },
    { 0xfad08475, 3 },
    { 0, 0 },
    { 0xbdc08477, 3 },
    { 0xcb948477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f4847b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a948483, 3 },
    { 0xc35c8484, 4 },
    { 0x42548485, 3 },
    { 0x12108485, 3 },
    { 0xdab08483, 3 },
    { 0xcfe08485, 3 },
    { 0x6ca88489, 3 },
    { 0x48c08487, 4 },
    { 0xbdfc848b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c848f, 3 },
    { 0x7484848f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54288495, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd708498, 3 },
    { 0x75208499, 3 },
    { 0, 0 },
    { 0xbe24849b, 4 },
    { 0x6c38849c, 3 },
    { 0xb218849d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9084a0, 4 },
    { 0x8aac84a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b484a7, 4 },
    { 0, 0 },
    { 0xf23084a9, 3 },
    { 0x8b6484aa, 3 },
    { 0, 0 },
    { 0xbb7484ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8084b1, 3 },
    { 0, 0 },
    { 0xb36084b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928484b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d084c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c84c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5084c8, 4 },
    { 0xcac884c9, 3 },
    { 0xb9a484c8, 3 },
    { 0xd7f484cb, 3 },
    { 0xb7f084cc, 4 },
    { 0x73c484cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11084d7, 3 },
    { 0xac3084d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91c84db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c484df, 3 },
    { 0, 0 },
    { 0xa44084e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf484e6, 3 },
    { 0xf82c84e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1c84eb, 4 },
    { 0x72f084ec, 3 },
    { 0x32ec84ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6884f7, 3 },
    { 0xf90084f7, 3 },
    { 0xd2a484f8, 3 },
    { 0, 0 },
    { 0x2f1884fb, 3 },
    { 0x2f484fc, 3 },
    { 0x136084fd, 3 },
    { 0x77bc84fe, 4 },
    { 0x3a3084ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e48505, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc88509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea08850e, 4 },
    { 0, 0 },
    { 0xf22c8510, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e08513, 4 },
    { 0x69bc8513, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe88517, 3 },
    { 0x84e88517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac0851b, 3 },
    { 0xac90851b, 3 },
    { 0x6904851d, 3 },
    { 0, 0 },
    { 0xc0851f, 3 },
    { 0x4340851f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87248524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e88529, 3 },
    { 0x9358852a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab48530, 4 },
    { 0, 0 },
    { 0xc8a48532, 3 },
    { 0, 0 },
    { 0x50ac8534, 4 },
    { 0, 0 },
    { 0xfa148536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb088853b, 4 },
    { 0, 0 },
    { 0xc5f4853d, 3 },
    { 0x9fc8853e, 3 },
    { 0, 0 },
    { 0xf2748540, 3 },
    { 0x2e8c8541, 3 },
    { 0x82008542, 4 },
    { 0x6acc8541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf48547, 3 },
    { 0x3ab08548, 3 },
    { 0x1e6c8549, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e94854c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb88552, 3 },
    { 0, 0 },
    { 0xa4388554, 3 },
    { 0x4a988555, 4 },
    { 0x5bb08556, 3 },
    { 0x99c8557, 4 },
    { 0, 0 },
    { 0x8d308559, 3 },
    { 0x6f0c8559, 3 },
    { 0xdc38855b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1320855f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f648567, 3 },
    { 0, 0 },
    { 0xd2888569, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34856e, 3 },
    { 0x2c68856f, 3 },
    { 0x40cc856f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872c8579, 4 },
    { 0xfa18857a, 3 },
    { 0x48d8857a, 3 },
    { 0x8440857c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d88580, 3 },
    { 0x5d648581, 4 },
    { 0x81348582, 3 },
    { 0x4f188583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f48586, 3 },
    { 0, 0 },
    { 0xf1488588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed8858b, 4 },
    { 0, 0 },
    { 0x2648858d, 3 },
    { 0xcc8c858e, 4 },
    { 0xc2a4858e, 3 },
    { 0x50f88590, 3 },
    { 0xd1548590, 3 },
    { 0xf3908591, 4 },
    { 0xb0c08593, 4 },
    { 0xeee48593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf88598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8228859b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94c859f, 3 },
    { 0xcf4c85a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ec85a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x985085a7, 4 },
    { 0xd77885a7, 3 },
    { 0, 0 },
    { 0xc25c85aa, 4 },
    { 0, 0 },
    { 0xe44885ac, 3 },
    { 0x469485ac, 4 },
    { 0xffa885ae, 4 },
    { 0xe91085ae, 4 },
    { 0x95d885b0, 3 },
    { 0xcfb485b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c85b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d885ba, 3 },
    { 0x371085ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57085c4, 3 },
    { 0x1fdc85c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb485ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d085ce, 3 },
    { 0, 0 },
    { 0x698c85d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52885d6, 3 },
    { 0, 0 },
    { 0xcb4885d8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d485e1, 4 },
    { 0, 0 },
    { 0x550485e3, 4 },
    { 0, 0 },
    { 0x417885e5, 3 },
    { 0, 0 },
    { 0xea0c85e7, 3 },
    { 0x82c85e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139885f0, 4 },
    { 0, 0 },
    { 0xd60485f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977885f8, 3 },
    { 0x367885f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95485fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34c8603, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac08606, 3 },
    { 0x76788607, 3 },
    { 0xbeec8608, 4 },
    { 0x2abc8607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb40860d, 4 },
    { 0xaef4860e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d88612, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7c8618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd584861b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c861e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa48621, 3 },
    { 0, 0 },
    { 0x80d88623, 4 },
    { 0xc7588624, 3 },
    { 0, 0 },
    { 0x571c8626, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b48629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf8862d, 3 },
    { 0, 0 },
    { 0x8b40862f, 3 },
    { 0xb9d08630, 4 },
    { 0x9da0862f, 4 },
    { 0xb7188631, 3 },
    { 0x69b08633, 4 },
    { 0xe1fc8633, 3 },
    { 0x22308635, 3 },
    { 0x200c8636, 4 },
    { 0, 0 },
    { 0x3e888638, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0208645, 3 },
    { 0x1af88646, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc188649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccc864e, 3 },
    { 0xffcc864f, 4 },
    { 0xa9848650, 3 },
    { 0x55108650, 3 },
    { 0, 0 },
    { 0xe83c8653, 3 },
    { 0, 0 },
    { 0x5c208655, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc48658, 3 },
    { 0x1df48658, 4 },
    { 0x67088658, 3 },
    { 0x3f0865b, 5 },
    { 0x498c8659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43c8662, 3 },
    { 0x13c08662, 4 },
    { 0xb2488664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96888667, 3 },
    { 0x115c8667, 3 },
    { 0x94308669, 3 },
    { 0xdf58866a, 3 },
    { 0xb79c866a, 3 },
    { 0xd2ec8668, 3 },
    { 0xb284866d, 4 },
    { 0x1c24866c, 3 },
    { 0xd57c866f, 3 },
    { 0x156c866f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb288675, 5 },
    { 0x25d48676, 3 },
    { 0xe4b88676, 3 },
    { 0xce8c8677, 3 },
    { 0x65408679, 3 },
    { 0x7140867a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3488684, 4 },
    { 0x17c48685, 3 },
    { 0x88c8685, 3 },
    { 0x48d48687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x514868a, 3 },
    { 0x22d8868b, 4 },
    { 0x93e4868a, 3 },
    { 0xe3f0868d, 4 },
    { 0x1f4868e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe608691, 4 },
    { 0, 0 },
    { 0x70708693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57148697, 3 },
    { 0x3d688698, 3 },
    { 0x5fac8699, 3 },
    { 0x9e848697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54886a2, 4 },
    { 0xffa486a2, 3 },
    { 0, 0 },
    { 0xb23086a5, 4 },
    { 0, 0 },
    { 0xd03c86a7, 3 },
    { 0xd05886a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d086af, 3 },
    { 0, 0 },
    { 0xcb9c86b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82086ba, 3 },
    { 0, 0 },
    { 0xa5bc86bc, 3 },
    { 0xd5b486bd, 3 },
    { 0x7d3086bd, 3 },
    { 0xd01c86bc, 3 },
    { 0xb2b486bd, 4 },
    { 0xfecc86c1, 3 },
    { 0, 0 },
    { 0x9cd486c3, 3 },
    { 0xa82486c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242086cd, 3 },
    { 0xca2486ce, 3 },
    { 0x5a6486cf, 3 },
    { 0x5b8c86d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4886d4, 3 },
    { 0xb2c486d5, 3 },
    { 0xf76c86d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c486d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cc86dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c486df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f086e2, 3 },
    { 0x3c4486e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820486e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7c86eb, 3 },
    { 0xd2c086ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929086f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9c86fa, 3 },
    { 0xebc86fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x215c86fe, 3 },
    { 0, 0 },
    { 0xfd48700, 4 },
    { 0xe308701, 3 },
    { 0x8c248701, 3 },
    { 0, 0 },
    { 0xae548704, 3 },
    { 0, 0 },
    { 0x48588706, 3 },
    { 0, 0 },
    { 0x65d48708, 3 },
    { 0x739c8708, 3 },
    { 0xe55c8709, 3 },
    { 0x7c188709, 3 },
    { 0xe640870c, 3 },
    { 0x6e608709, 3 },
    { 0, 0 },
    { 0xdd94870f, 4 },
    { 0x92cc8710, 4 },
    { 0x43688710, 4 },
    { 0x73f88712, 3 },
    { 0x6a988713, 4 },
    { 0xac988713, 3 },
    { 0xed08715, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78871c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8871f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e8872a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f64872f, 3 },
    { 0x41b8872f, 3 },
    { 0x69e0872f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f08734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc8737, 3 },
    { 0, 0 },
    { 0x99bc8739, 3 },
    { 0x1c508739, 4 },
    { 0xd88c8739, 3 },
    { 0xbbc873c, 3 },
    { 0x453c873d, 3 },
    { 0xe488739, 3 },
    { 0, 0 },
    { 0xd2bc8740, 3 },
    { 0x34688740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d08744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74808747, 5 },
    { 0x2dc88748, 3 },
    { 0xf11c8747, 3 },
    { 0x17f8874a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ec8750, 3 },
    { 0x59a88751, 3 },
    { 0x84148750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6708755, 3 },
    { 0xb6888755, 3 },
    { 0x86ac8757, 4 },
    { 0xd3648757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed448765, 4 },
    { 0x36248765, 3 },
    { 0xf2448767, 4 },
    { 0x7a908765, 4 },
    { 0xfdf08767, 3 },
    { 0x69f4876a, 4 },
    { 0x10bc8766, 3 },
    { 0xc00876c, 3 },
    { 0x9260876b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c48771, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd08775, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1770877d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70c8780, 3 },
    { 0x20dc8780, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9910878b, 3 },
    { 0x9558878c, 4 },
    { 0x90c8878c, 3 },
    { 0x5434878e, 5 },
    { 0x2bf4878e, 3 },
    { 0, 0 },
    { 0x8708791, 3 },
    { 0xc89c8792, 4 },
    { 0x47548793, 3 },
    { 0xfda48791, 3 },
    { 0x5b148791, 3 },
    { 0x42d48796, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x65088799, 3 },
    { 0x3e10879a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cc879e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23887ab, 3 },
    { 0xf3d087ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3487b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20087b3, 4 },
    { 0x9c87b4, 3 },
    { 0, 0 },
    { 0xb67c87b6, 3 },
    { 0xbac087b6, 3 },
    { 0, 0 },
    { 0x2f8087b9, 3 },
    { 0x40cc87b9, 3 },
    { 0x739887bb, 3 },
    { 0xad887bc, 3 },
    { 0, 0 },
    { 0x54a087be, 3 },
    { 0x891c87bf, 4 },
    { 0, 0 },
    { 0x9aa887c1, 3 },
    { 0x5a7887c2, 3 },
    { 0xb72087c2, 3 },
    { 0xabb487c4, 4 },
    { 0x7e2487c5, 3 },
    { 0xe51487c6, 3 },
    { 0x203487c6, 4 },
    { 0x7a6487c6, 3 },
    { 0x200487c5, 3 },
    { 0x52ec87ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb487cd, 3 },
    { 0x8a9487ce, 3 },
    { 0x546487cd, 3 },
    { 0, 0 },
    { 0xb70887d1, 3 },
    { 0xdf5c87d1, 3 },
    { 0, 0 },
    { 0xdb0c87d4, 4 },
    { 0x4eec87d5, 3 },
    { 0x69d087d5, 4 },
    { 0, 0 },
    { 0x3b7487d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e887dd, 3 },
    { 0x930c87de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7c87e1, 3 },
    { 0, 0 },
    { 0xd66887e3, 3 },
    { 0xda3887e4, 3 },
    { 0x8bbc87e5, 4 },
    { 0x83fc87e6, 3 },
    { 0x542887e6, 3 },
    { 0, 0 },
    { 0x45d887e9, 3 },
    { 0xe3a487ea, 3 },
    { 0x253887eb, 4 },
    { 0xec0887ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c87f2, 3 },
    { 0x877487f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904087f8, 4 },
    { 0, 0 },
    { 0x9af887fa, 3 },
    { 0, 0 },
    { 0x28c087fc, 3 },
    { 0x5efc87fd, 3 },
    { 0x40d087fe, 3 },
    { 0, 0 },
    { 0x884c8800, 4 },
    { 0xb45c8801, 3 },
    { 0xa6588802, 3 },
    { 0, 0 },
    { 0xbd88804, 4 },
    { 0x834c8805, 3 },
    { 0x304c8806, 3 },
    { 0x22ac8807, 3 },
    { 0x34548807, 3 },
    { 0xc6848807, 3 },
    { 0, 0 },
    { 0x1098880b, 4 },
    { 0x38d4880c, 3 },
    { 0x78a4880d, 3 },
    { 0, 0 },
    { 0xadec880f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91448812, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc08817, 3 },
    { 0, 0 },
    { 0x83588819, 4 },
    { 0x7ee4881a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc4881d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd008824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79288828, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2544882c, 3 },
    { 0x3880882c, 3 },
    { 0x510882d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2c8831, 4 },
    { 0, 0 },
    { 0xd2988833, 3 },
    { 0, 0 },
    { 0xa4888835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b48839, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd20883c, 3 },
    { 0x47f4883c, 4 },
    { 0x43fc883e, 3 },
    { 0, 0 },
    { 0xcc648840, 4 },
    { 0, 0 },
    { 0xaf488842, 4 },
    { 0x6d588843, 3 },
    { 0xc6bc8843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2988848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674c8850, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708855, 3 },
    { 0xe2488856, 4 },
    { 0xdcec8856, 4 },
    { 0x5508858, 3 },
    { 0xc3788859, 3 },
    { 0xa0f08856, 4 },
    { 0x1e80885b, 3 },
    { 0x7374885c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b84885f, 3 },
    { 0x340885f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa288863, 3 },
    { 0, 0 },
    { 0x28248865, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f48868, 3 },
    { 0x80048869, 4 },
    { 0xa7f48868, 4 },
    { 0, 0 },
    { 0x2538886c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43608872, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1988875, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda848878, 3 },
    { 0xa708878, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5648887e, 3 },
    { 0xd554887f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c588884, 3 },
    { 0, 0 },
    { 0x3fa48886, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c8889, 3 },
    { 0x6438888a, 3 },
    { 0x71e0888b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fc8893, 3 },
    { 0xab3c8893, 3 },
    { 0x37788895, 3 },
    { 0, 0 },
    { 0x24348897, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc338889b, 3 },
    { 0, 0 },
    { 0xa4d0889d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee088a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x588888a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c888a9, 3 },
    { 0, 0 },
    { 0x56c888ab, 3 },
    { 0, 0 },
    { 0xab5c88ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c088b3, 3 },
    { 0x2c8088b3, 3 },
    { 0xcba888b3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c088b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d888be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff088c1, 3 },
    { 0, 0 },
    { 0xb2f888c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7888c9, 3 },
    { 0x9bdc88ca, 3 },
    { 0, 0 },
    { 0x4a7c88cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06888d1, 3 },
    { 0xf6f488d2, 4 },
    { 0, 0 },
    { 0xc91888d4, 3 },
    { 0x30b888d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a488d8, 3 },
    { 0, 0 },
    { 0x5db088da, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13c88dd, 3 },
    { 0, 0 },
    { 0xe3d088df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccec88e5, 3 },
    { 0xea4c88e6, 3 },
    { 0xa2fc88e5, 3 },
    { 0xf8d088e5, 3 },
    { 0x7bc088e5, 3 },
    { 0x239c88e6, 3 },
    { 0xa17088eb, 4 },
    { 0xf9b488eb, 3 },
    { 0x46cc88ea, 4 },
    { 0x734488ed, 3 },
    { 0, 0 },
    { 0x93c088f0, 4 },
    { 0x81a888f1, 3 },
    { 0x901488f2, 3 },
    { 0xe5d088f3, 4 },
    { 0, 0 },
    { 0x154888f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7088f9, 4 },
    { 0x2efc88fa, 3 },
    { 0x20c88fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17308900, 3 },
    { 0xcc988901, 3 },
    { 0x2cb48902, 4 },
    { 0xd8f48903, 3 },
    { 0x6d408902, 3 },
    { 0x89cc8901, 3 },
    { 0xc1288906, 3 },
    { 0xac408907, 4 },
    { 0xd3488902, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40890b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cc8910, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2c8918, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd178891c, 3 },
    { 0xfec0891d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e48920, 3 },
    { 0xc9488921, 4 },
    { 0xbd3c8922, 5 },
    { 0x579c8923, 3 },
    { 0, 0 },
    { 0x450c8925, 3 },
    { 0xde808926, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed08930, 3 },
    { 0xd5008931, 3 },
    { 0x46c88931, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b88939, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ec893d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a88943, 4 },
    { 0, 0 },
    { 0x4dd88945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35c894b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b4894e, 4 },
    { 0x70b4894e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f348952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad4895a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d0895e, 3 },
    { 0x5d8c895f, 3 },
    { 0xda708960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7288968, 3 },
    { 0, 0 },
    { 0x44fc896a, 3 },
    { 0x2d04896b, 3 },
    { 0xd094896a, 3 },
    { 0xc07c896d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35248970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc48976, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c897a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f8897f, 4 },
    { 0, 0 },
    { 0xe3d08981, 3 },
    { 0, 0 },
    { 0x92208983, 3 },
    { 0xf71c8984, 3 },
    { 0, 0 },
    { 0x41588986, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e0898b, 3 },
    { 0x9e68898c, 3 },
    { 0, 0 },
    { 0x491c898e, 3 },
    { 0x9d00898f, 4 },
    { 0xeda48990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86588994, 3 },
    { 0x84f88994, 4 },
    { 0x1e288996, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc00899a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b0899d, 3 },
    { 0x66b0899e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x633489a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d889a4, 3 },
    { 0, 0 },
    { 0xa56089a6, 3 },
    { 0x362089a7, 4 },
    { 0xfef089a8, 3 },
    { 0x3e0489a9, 4 },
    { 0x25a089a8, 4 },
    { 0xfb2089a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48089ae, 3 },
    { 0, 0 },
    { 0xa0f089b0, 3 },
    { 0x475c89b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x425c89b4, 3 },
    { 0xc35889b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x972089b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8889bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e489c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150c89c5, 3 },
    { 0x7dd089c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1089d0, 3 },
    { 0xde5c89d1, 3 },
    { 0, 0 },
    { 0x5ab489d3, 4 },
    { 0xf13089d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c89d7, 5 },
    { 0x46089d8, 4 },
    { 0xebbc89d9, 3 },
    { 0xd7a489d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9c89dd, 3 },
    { 0xbec89de, 5 },
    { 0xa06489de, 4 },
    { 0x646889e0, 4 },
    { 0xea5089de, 3 },
    { 0x69e089e2, 4 },
    { 0x531489e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416489e8, 3 },
    { 0, 0 },
    { 0xcaac89ea, 3 },
    { 0xfc3889eb, 3 },
    { 0x31889ea, 3 },
    { 0, 0 },
    { 0xc89889ee, 4 },
    { 0x6c9089ef, 3 },
    { 0xe6c089ee, 3 },
    { 0xe85c89f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x872489f4, 4 },
    { 0xcb0089f5, 3 },
    { 0, 0 },
    { 0x437489f7, 3 },
    { 0x500889f7, 3 },
    { 0x203489f7, 3 },
    { 0xfdc489fa, 3 },
    { 0x9d1489f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8c89fe, 3 },
    { 0x48889ff, 3 },
    { 0x5cd88a00, 3 },
    { 0, 0 },
    { 0xbe2c8a02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51608a05, 3 },
    { 0xbb508a05, 4 },
    { 0, 0 },
    { 0xc48c8a08, 3 },
    { 0xfa1c8a09, 3 },
    { 0xac308a08, 3 },
    { 0x483c8a09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc8a0f, 3 },
    { 0x78648a0f, 4 },
    { 0xa7c48a0f, 3 },
    { 0xc588a0f, 4 },
    { 0x74fc8a13, 5 },
    { 0, 0 },
    { 0xebec8a15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f108a18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51088a1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c8a23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf08a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e908a29, 3 },
    { 0x53a48a29, 3 },
    { 0xff2c8a2b, 3 },
    { 0, 0 },
    { 0xb95c8a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c8a30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b208a36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3348a3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3488a3d, 4 },
    { 0x3a748a3e, 3 },
    { 0x97148a3f, 4 },
    { 0x633c8a40, 4 },
    { 0xc708a40, 3 },
    { 0x6f5c8a42, 3 },
    { 0xc6b48a43, 3 },
    { 0x114c8a42, 3 },
    { 0xc2d48a3f, 3 },
    { 0xd6f48a44, 3 },
    { 0x25188a3e, 3 },
    { 0x308c8a47, 4 },
    { 0, 0 },
    { 0x14e08a4a, 3 },
    { 0x9a788a4b, 3 },
    { 0x57b88a4b, 5 },
    { 0x66508a4b, 3 },
    { 0x82588a4d, 4 },
    { 0x9ec88a4f, 3 },
    { 0x49948a4c, 3 },
    { 0xc4a48a51, 4 },
    { 0x9cb88a51, 3 },
    { 0, 0 },
    { 0xb0e88a54, 4 },
    { 0xabb48a55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3688a59, 3 },
    { 0xe0188a5a, 4 },
    { 0x4588a59, 3 },
    { 0xa7688a5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc8a60, 3 },
    { 0, 0 },
    { 0x120c8a62, 3 },
    { 0, 0 },
    { 0x77f88a64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a88a68, 3 },
    { 0x83bc8a68, 5 },
    { 0xb4248a6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77648a70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e48a81, 3 },
    { 0, 0 },
    { 0x7bb08a83, 3 },
    { 0xe0588a84, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53808a90, 3 },
    { 0, 0 },
    { 0xf56c8a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f608a95, 4 },
    { 0, 0 },
    { 0x60b48a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce08aa0, 3 },
    { 0x82c8aa0, 3 },
    { 0x64648aa2, 3 },
    { 0, 0 },
    { 0x49808aa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dc8aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf08aaa, 4 },
    { 0xe71c8aab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f88ab6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14c8abb, 3 },
    { 0, 0 },
    { 0x54b88abd, 4 },
    { 0x2b2c8abe, 3 },
    { 0xe8888abe, 3 },
    { 0xa9948ac0, 3 },
    { 0xb9e08ac0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93788ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12848ac9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95888ad5, 3 },
    { 0xded88ad6, 3 },
    { 0xb7e88ad7, 3 },
    { 0x52f08ad8, 4 },
    { 0xcca08ad5, 3 },
    { 0x4a7c8ad8, 5 },
    { 0xdc888ada, 4 },
    { 0x83408adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1908ae2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed48ae5, 3 },
    { 0x7fb88ae5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd08ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688af0, 3 },
    { 0x35508af0, 3 },
    { 0, 0 },
    { 0x34a08af3, 3 },
    { 0x6b408af4, 3 },
    { 0xc4288af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3748afb, 3 },
    { 0, 0 },
    { 0xae4c8afd, 3 },
    { 0, 0 },
    { 0xb7e48aff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74748b02, 3 },
    { 0x65188b03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5888b09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f688b0e, 4 },
    { 0x16c48b0f, 3 },
    { 0, 0 },
    { 0x56b88b11, 3 },
    { 0, 0 },
    { 0x6a308b13, 4 },
    { 0x3db88b13, 3 },
    { 0x6f48b15, 3 },
    { 0xfa588b16, 3 },
    { 0, 0 },
    { 0x36588b18, 3 },
    { 0xf2f48b19, 4 },
    { 0, 0 },
    { 0x20c8b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33508b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9008b21, 4 },
    { 0x66a08b22, 4 },
    { 0, 0 },
    { 0x4de88b24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x656c8b27, 3 },
    { 0xca788b28, 3 },
    { 0x5b608b29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c8b2c, 3 },
    { 0x893c8b2c, 3 },
    { 0x588b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8208b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85788b35, 3 },
    { 0x4c5c8b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c508b3b, 4 },
    { 0x4e748b3c, 3 },
    { 0x4b508b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cc8b48, 4 },
    { 0xd51c8b48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80148b4d, 3 },
    { 0xda488b4e, 4 },
    { 0xf5988b4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd48b56, 3 },
    { 0x649c8b56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3c8b5c, 3 },
    { 0x92848b5c, 3 },
    { 0xb7b48b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a88b61, 3 },
    { 0x51208b62, 3 },
    { 0, 0 },
    { 0xaa1c8b64, 3 },
    { 0x2d108b65, 3 },
    { 0xf388b65, 5 },
    { 0x8cac8b67, 4 },
    { 0x3b748b68, 3 },
    { 0xd5f08b66, 3 },
    { 0xdb348b6a, 3 },
    { 0xb4308b6b, 3 },
    { 0, 0 },
    { 0xb2ac8b6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a88b70, 3 },
    { 0x68008b71, 4 },
    { 0xf2dc8b70, 3 },
    { 0x91988b71, 3 },
    { 0, 0 },
    { 0x5008b75, 3 },
    { 0, 0 },
    { 0x9f7c8b77, 4 },
    { 0, 0 },
    { 0xc4188b79, 4 },
    { 0, 0 },
    { 0x25c8b7b, 4 },
    { 0x35a08b7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72488b85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fc8b89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ec8b8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19508b90, 3 },
    { 0xd5a08b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x461c8b94, 3 },
    { 0xd8608b95, 5 },
    { 0, 0 },
    { 0xc8d48b97, 3 },
    { 0xbd648b98, 3 },
    { 0x6a008b99, 4 },
    { 0xb1d48b9a, 3 },
    { 0x4688b9b, 4 },
    { 0x9e388b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18688ba7, 4 },
    { 0, 0 },
    { 0x6e108ba9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fc8bae, 3 },
    { 0xf0748bae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0508bb4, 3 },
    { 0x39848bb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109c8bbd, 4 },
    { 0xfd848bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54688bc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf388bc6, 3 },
    { 0xcfe48bc6, 3 },
    { 0xc1a48bc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76408bd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4c8bd7, 4 },
    { 0xfe748bd8, 3 },
    { 0x6ff88bd9, 3 },
    { 0x6b908bd7, 3 },
    { 0xeaec8bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd308be1, 4 },
    { 0x60288be2, 3 },
    { 0, 0 },
    { 0xe34c8be4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddc8be8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82408beb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf08bf5, 3 },
    { 0x7f6c8bf6, 3 },
    { 0, 0 },
    { 0xf7a08bf8, 3 },
    { 0x3c08bf9, 4 },
    { 0, 0 },
    { 0x2ff08bfb, 3 },
    { 0xce4c8bfc, 5 },
    { 0x6f408bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0788c00, 4 },
    { 0x97788c00, 3 },
    { 0x40ac8c00, 3 },
    { 0, 0 },
    { 0xab48c04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3748c07, 3 },
    { 0x35408c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d08c0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e48c13, 4 },
    { 0, 0 },
    { 0x48f48c15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d48c1a, 3 },
    { 0, 0 },
    { 0x72288c1c, 3 },
    { 0x8ed88c1d, 4 },
    { 0, 0 },
    { 0x1f9c8c1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b08c26, 3 },
    { 0, 0 },
    { 0xbb688c28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ec8c2d, 3 },
    { 0xceac8c2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e88c35, 4 },
    { 0x55b88c35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c88c3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78508c3d, 3 },
    { 0x6c508c3d, 3 },
    { 0xb1488c3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd88c43, 3 },
    { 0x7c808c44, 3 },
    { 0xae488c43, 3 },
    { 0x94088c46, 4 },
    { 0xcce48c47, 3 },
    { 0, 0 },
    { 0xcb88c49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94948c4e, 4 },
    { 0, 0 },
    { 0x3ee48c50, 3 },
    { 0, 0 },
    { 0xe4008c52, 3 },
    { 0, 0 },
    { 0xfe688c54, 4 },
    { 0xaa688c55, 4 },
    { 0xeee88c54, 5 },
    { 0, 0 },
    { 0xbb188c58, 3 },
    { 0x84448c58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe48c5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa448c61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76308c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed788c68, 3 },
    { 0, 0 },
    { 0xa848c6a, 4 },
    { 0xae088c6b, 3 },
    { 0, 0 },
    { 0xdff08c6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab08c70, 4 },
    { 0x9eac8c70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9708c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61248c7b, 3 },
    { 0xf1c48c7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cc8c81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10448c85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a08c88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e288c8b, 3 },
    { 0x97388c8c, 3 },
    { 0xde248c8b, 3 },
    { 0xe0dc8c8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ec8c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b08c9a, 4 },
    { 0xb4c08c9b, 3 },
    { 0, 0 },
    { 0x66488c9d, 3 },
    { 0x633c8c9e, 3 },
    { 0xabf88c9e, 3 },
    { 0x6b08c9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d08ca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b48cab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8108caf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b508cb2, 3 },
    { 0xa9fc8cb2, 3 },
    { 0xda948cb4, 3 },
    { 0xc0948cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e88cb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0448cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29948cca, 3 },
    { 0xa8bc8ccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec88cd3, 3 },
    { 0x5b3c8cd4, 3 },
    { 0xbbc88cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e88cd9, 3 },
    { 0, 0 },
    { 0xf0248cdb, 3 },
    { 0, 0 },
    { 0xd3c08cdd, 3 },
    { 0xb6548cde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149c8ce2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1808ce7, 5 },
    { 0x430c8ce8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13688cec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fc8cf2, 4 },
    { 0x5be88cf2, 3 },
    { 0, 0 },
    { 0x8c888cf5, 3 },
    { 0, 0 },
    { 0x332c8cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa08cfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71948d00, 4 },
    { 0x17488d01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c8d04, 3 },
    { 0x56508d05, 4 },
    { 0x69888d05, 3 },
    { 0x75388d06, 3 },
    { 0xed588d08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d08d0c, 3 },
    { 0, 0 },
    { 0xf4208d0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c48d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee688d14, 5 },
    { 0, 0 },
    { 0x92048d16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc8d22, 4 },
    { 0x82b88d23, 4 },
    { 0x7c788d23, 4 },
    { 0x7e488d25, 3 },
    { 0x76bc8d22, 3 },
    { 0x83188d27, 5 },
    { 0xb0c08d28, 3 },
    { 0x2cd88d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35608d2d, 3 },
    { 0x8efc8d2e, 3 },
    { 0, 0 },
    { 0xdfe48d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf708d35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44c8d39, 5 },
    { 0, 0 },
    { 0x482c8d3b, 3 },
    { 0xa8a08d3c, 3 },
    { 0x36548d3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f48d43, 3 },
    { 0, 0 },
    { 0xe9a08d45, 3 },
    { 0x99148d46, 3 },
    { 0x7d3c8d46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d708d4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c08d56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec8d5c, 3 },
    { 0x9a148d5d, 3 },
    { 0x99848d5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54808d62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfc8d69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c08d6c, 4 },
    { 0, 0 },
    { 0xdd948d6e, 4 },
    { 0xb7348d6f, 3 },
    { 0x4e6c8d70, 3 },
    { 0, 0 },
    { 0xe2a08d72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd88d75, 3 },
    { 0xff648d76, 3 },
    { 0x91748d77, 3 },
    { 0x77108d78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce108d7e, 3 },
    { 0, 0 },
    { 0x2288d80, 4 },
    { 0xe5b08d80, 5 },
    { 0xe3708d81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1988d8d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da88d90, 3 },
    { 0xb6a08d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a688d9a, 3 },
    { 0x17548d9b, 3 },
    { 0x77788d9c, 3 },
    { 0xe10c8d9d, 3 },
    { 0xcebc8d9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27748da4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31088da8, 3 },
    { 0, 0 },
    { 0xb4088daa, 4 },
    { 0xc0588dab, 3 },
    { 0xc0548daa, 3 },
    { 0, 0 },
    { 0x3ee88dae, 5 },
    { 0xa17c8daf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x368c8db2, 3 },
    { 0xcc4c8db3, 3 },
    { 0, 0 },
    { 0x9d5c8db5, 4 },
    { 0, 0 },
    { 0xaa248db7, 3 },
    { 0, 0 },
    { 0x4d648db9, 3 },
    { 0xa248dba, 4 },
    { 0, 0 },
    { 0xb0fc8dbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afc8dbf, 3 },
    { 0x471c8dc0, 3 },
    { 0, 0 },
    { 0xe3b08dc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccc8dc7, 3 },
    { 0, 0 },
    { 0x974c8dc9, 4 },
    { 0xf48dca, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bc8dcf, 3 },
    { 0, 0 },
    { 0xb7c88dd1, 4 },
    { 0, 0 },
    { 0x805c8dd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5508dd6, 3 },
    { 0x36d48dd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df08ddc, 4 },
    { 0xb3b08ddd, 3 },
    { 0xb7908dde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c8de1, 3 },
    { 0xb92c8de2, 3 },
    { 0x84148de3, 4 },
    { 0xaf988de1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e08de7, 3 },
    { 0x7c6c8de8, 3 },
    { 0x8eb48de9, 5 },
    { 0x46288de9, 3 },
    { 0x6b208de7, 3 },
    { 0, 0 },
    { 0x5bac8ded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe848df1, 4 },
    { 0x1d3c8df2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3988df5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ec8df9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c048dfe, 3 },
    { 0, 0 },
    { 0x77088e00, 3 },
    { 0, 0 },
    { 0x9af08e02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68388e0d, 3 },
    { 0xc6888e0d, 3 },
    { 0x90188e0f, 3 },
    { 0, 0 },
    { 0x55508e11, 3 },
    { 0xab548e12, 4 },
    { 0xe69c8e13, 3 },
    { 0x983c8e14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5148e19, 4 },
    { 0xddcc8e1a, 3 },
    { 0xbeec8e1b, 4 },
    { 0x1d88e1b, 3 },
    { 0x44208e1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff308e23, 3 },
    { 0x9fb88e24, 3 },
    { 0xe0b08e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b308e29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c8e2d, 3 },
    { 0x6e888e2d, 3 },
    { 0x48b08e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11208e36, 4 },
    { 0xa8848e37, 3 },
    { 0xe6ac8e38, 3 },
    { 0, 0 },
    { 0xeb708e3a, 3 },
    { 0x886c8e3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67288e3e, 3 },
    { 0, 0 },
    { 0x55d48e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c188e45, 3 },
    { 0, 0 },
    { 0xac88e47, 3 },
    { 0xb9d88e48, 5 },
    { 0xeda08e47, 3 },
    { 0, 0 },
    { 0xb7a48e4b, 3 },
    { 0, 0 },
    { 0xe2808e4d, 3 },
    { 0x1be88e4e, 3 },
    { 0, 0 },
    { 0x2a848e50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30508e59, 3 },
    { 0x81748e59, 3 },
    { 0x51948e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c8e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb88e62, 3 },
    { 0xa8788e63, 3 },
    { 0x5cc88e64, 3 },
    { 0xc7ac8e65, 3 },
    { 0xbc348e66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b88e69, 3 },
    { 0, 0 },
    { 0x9ae88e6b, 3 },
    { 0xa3808e6b, 3 },
    { 0xde688e6d, 3 },
    { 0xf8c48e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f48e74, 3 },
    { 0, 0 },
    { 0x522c8e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb548e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482c8e7d, 3 },
    { 0, 0 },
    { 0x7e1c8e7f, 3 },
    { 0, 0 },
    { 0x19408e81, 3 },
    { 0x96188e82, 3 },
    { 0xf7a08e83, 3 },
    { 0xae4c8e83, 3 },
    { 0x6ae08e85, 3 },
    { 0x2b548e81, 4 },
    { 0x1a188e87, 4 },
    { 0x21908e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948e92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9008e9a, 3 },
    { 0, 0 },
    { 0xe6b48e9c, 3 },
    { 0xb62c8e9d, 4 },
    { 0xccec8e9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6c8ea4, 3 },
    { 0xcb588ea4, 3 },
    { 0x38788ea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x918c8ea9, 3 },
    { 0xc0d88ea9, 3 },
    { 0xac48eab, 3 },
    { 0x46088eac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d88eaf, 3 },
    { 0xcc248eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c548eb3, 3 },
    { 0x37048eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc8eb9, 4 },
    { 0, 0 },
    { 0x22148ebb, 3 },
    { 0xdad48ebc, 3 },
    { 0x2c4c8ebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a448ec3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8ec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60488ed0, 3 },
    { 0x90e88ed1, 3 },
    { 0xbb408ed0, 3 },
    { 0, 0 },
    { 0x3ec48ed4, 4 },
    { 0x3d688ed4, 3 },
    { 0x19348ed5, 3 },
    { 0xa1b48ed7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f048edd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d88ee5, 3 },
    { 0xd2788ee6, 3 },
    { 0x256c8ee7, 3 },
    { 0x99448ee8, 3 },
    { 0x59108ee7, 3 },
    { 0xf23c8eea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66c8ef2, 3 },
    { 0xbfec8ef2, 3 },
    { 0, 0 },
    { 0x4f9c8ef5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86388ef8, 5 },
    { 0x92548ef8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f788efc, 3 },
    { 0xd6bc8efd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab248f07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d88f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e88f13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae08f17, 3 },
    { 0xfc608f18, 4 },
    { 0x52308f19, 3 },
    { 0x13788f1a, 4 },
    { 0, 0 },
    { 0x95948f1c, 4 },
    { 0xb7a08f1d, 3 },
    { 0xbab08f1e, 3 },
    { 0, 0 },
    { 0xe5c88f20, 3 },
    { 0x9f088f20, 3 },
    { 0x75d48f22, 3 },
    { 0xe4e48f23, 3 },
    { 0xe54c8f24, 3 },
    { 0, 0 },
    { 0x12608f26, 4 },
    { 0x22fc8f27, 4 },
    { 0x51848f26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26c8f2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94048f2f, 3 },
    { 0x39c08f2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea08f37, 3 },
    { 0x9c788f38, 3 },
    { 0xc5e48f38, 3 },
    { 0, 0 },
    { 0x14288f3b, 3 },
    { 0xef848f3c, 3 },
    { 0x4bdc8f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50548f40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8688f43, 3 },
    { 0, 0 },
    { 0x8e448f45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f608f4a, 5 },
    { 0xc6808f4b, 5 },
    { 0x4b708f4c, 3 },
    { 0, 0 },
    { 0xc2e48f4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e388f52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b88f57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc8f65, 4 },
    { 0x1dc08f66, 3 },
    { 0xf5d08f67, 3 },
    { 0x400c8f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd88f6c, 5 },
    { 0, 0 },
    { 0xb6748f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df88f71, 3 },
    { 0xd9888f71, 4 },
    { 0x28b48f73, 3 },
    { 0x24ac8f73, 3 },
    { 0xb5348f75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc48f7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a48f80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e48f84, 3 },
    { 0x42308f85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc8f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab48f8d, 3 },
    { 0, 0 },
    { 0x8f248f8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fc8f92, 3 },
    { 0xc348f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd908f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10188fa3, 3 },
    { 0, 0 },
    { 0xa17c8fa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60748fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a08faf, 4 },
    { 0, 0 },
    { 0xf1e48fb1, 3 },
    { 0xd4a48fb2, 3 },
    { 0x99d48fb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80208fb7, 3 },
    { 0xd8b08fb8, 3 },
    { 0xd0648fb7, 5 },
    { 0x2f348fba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f08fbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5848fc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee08fca, 3 },
    { 0, 0 },
    { 0xbd288fcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55748fcf, 3 },
    { 0xb5fc8fd0, 4 },
    { 0xaa848fcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4c8fd6, 4 },
    { 0xbf288fd7, 3 },
    { 0x5d788fd6, 4 },
    { 0xbac08fd8, 3 },
    { 0xbf1c8fda, 4 },
    { 0, 0 },
    { 0x94108fdc, 3 },
    { 0xb3b08fdd, 4 },
    { 0x520c8fdc, 3 },
    { 0xa6b88fdc, 4 },
    { 0x67388fe0, 3 },
    { 0xf4008fe0, 3 },
    { 0, 0 },
    { 0x88908fe3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f508fe7, 3 },
    { 0x2f048fe8, 3 },
    { 0, 0 },
    { 0x32248fea, 3 },
    { 0x62548feb, 5 },
    { 0xea108fec, 3 },
    { 0x68d88fed, 3 },
    { 0x40988fed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56088ff1, 3 },
    { 0xca648ff2, 3 },
    { 0xcb448ff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26848ff6, 3 },
    { 0xc55c8ff6, 3 },
    { 0x2eec8ff7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87888ffc, 3 },
    { 0xe3448ffd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d149002, 3 },
    { 0x9d289003, 3 },
    { 0xffe89003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc154900b, 3 },
    { 0xffc4900c, 3 },
    { 0, 0 },
    { 0xccac900e, 3 },
    { 0x1b38900e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc9013, 4 },
    { 0, 0 },
    { 0x8a49015, 3 },
    { 0xc15c9016, 3 },
    { 0xa2c49016, 3 },
    { 0xa2789018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be09027, 3 },
    { 0x1b4c9028, 3 },
    { 0x13449029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4c902c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3849031, 3 },
    { 0x33ac9032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f48903f, 3 },
    { 0, 0 },
    { 0xe3389041, 4 },
    { 0xd65c9041, 4 },
    { 0, 0 },
    { 0x919c9044, 3 },
    { 0x52889045, 3 },
    { 0xd1749046, 3 },
    { 0xf0749047, 3 },
    { 0x7fc09048, 3 },
    { 0x96189049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4140904c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d709054, 4 },
    { 0xd4409055, 3 },
    { 0, 0 },
    { 0x4b809057, 3 },
    { 0x9f249057, 4 },
    { 0xa1a89059, 3 },
    { 0, 0 },
    { 0xc84c905b, 3 },
    { 0x4ab4905c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc189065, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1c906b, 3 },
    { 0x8488906c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4509071, 3 },
    { 0, 0 },
    { 0x62ac9073, 3 },
    { 0xed389074, 3 },
    { 0, 0 },
    { 0x51b49076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ac9079, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2c907c, 3 },
    { 0x1c80907d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc689085, 3 },
    { 0, 0 },
    { 0x679c9087, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6c908c, 3 },
    { 0x6e48908d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30c9092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64909a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e8909d, 3 },
    { 0x2538909e, 3 },
    { 0xe3b8909f, 3 },
    { 0x928c909d, 4 },
    { 0x2b890a0, 3 },
    { 0x458c90a1, 3 },
    { 0xfbac90a3, 4 },
    { 0xc9e890a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c490aa, 3 },
    { 0xc3c490aa, 3 },
    { 0, 0 },
    { 0xdd5c90ad, 3 },
    { 0, 0 },
    { 0xeb2890af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72890b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce490b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc90ba, 3 },
    { 0x807490ba, 3 },
    { 0, 0 },
    { 0xf48890bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270890c4, 3 },
    { 0, 0 },
    { 0xc0f090c6, 3 },
    { 0x39f090c7, 3 },
    { 0x1f2c90c8, 3 },
    { 0x725c90c8, 3 },
    { 0, 0 },
    { 0x709c90cb, 4 },
    { 0, 0 },
    { 0x113c90cd, 3 },
    { 0x9fd490cd, 4 },
    { 0xf60c90cf, 4 },
    { 0x8f3c90d0, 3 },
    { 0xa81490d0, 3 },
    { 0x757c90d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa490d9, 4 },
    { 0xf2b890da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121890de, 3 },
    { 0x457c90df, 3 },
    { 0, 0 },
    { 0xe2f090e1, 3 },
    { 0xda4c90e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce890e5, 3 },
    { 0x3ca490e5, 4 },
    { 0xc3b090e7, 3 },
    { 0x292c90e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c90eb, 4 },
    { 0, 0 },
    { 0x5e7c90ed, 3 },
    { 0xe56090ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b090f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572090f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62090fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cac90fe, 3 },
    { 0xbd7c90fe, 3 },
    { 0x33649100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f589103, 4 },
    { 0, 0 },
    { 0xa5809105, 3 },
    { 0, 0 },
    { 0xa51c9107, 3 },
    { 0x74389108, 4 },
    { 0, 0 },
    { 0x4d88910a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13c910d, 4 },
    { 0x6454910e, 3 },
    { 0xdb98910d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da09113, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c9118, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5548911c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c9121, 3 },
    { 0x54149122, 3 },
    { 0x70549123, 3 },
    { 0xbbd49124, 3 },
    { 0x2bf09125, 3 },
    { 0, 0 },
    { 0x2a789127, 4 },
    { 0x77849127, 4 },
    { 0, 0 },
    { 0x7b54912a, 3 },
    { 0x5cc8912b, 3 },
    { 0, 0 },
    { 0x558c912d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf49132, 3 },
    { 0x4bac9133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc09137, 3 },
    { 0xbad09138, 5 },
    { 0xeee89138, 4 },
    { 0xf58c913a, 4 },
    { 0x4658913b, 4 },
    { 0xabb0913b, 3 },
    { 0xafc913d, 4 },
    { 0xe9f0913d, 3 },
    { 0x7200913d, 3 },
    { 0xebf4913d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb49144, 3 },
    { 0x1b189145, 5 },
    { 0x56c9146, 3 },
    { 0x67d49145, 3 },
    { 0x8a0c9148, 3 },
    { 0x493c9149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a34914f, 3 },
    { 0xfc08914f, 3 },
    { 0, 0 },
    { 0xff49152, 4 },
    { 0, 0 },
    { 0xaa4c9154, 3 },
    { 0x12c09155, 3 },
    { 0x6cf49156, 3 },
    { 0xd7d49157, 3 },
    { 0xbb349155, 4 },
    { 0x88109159, 3 },
    { 0x79b8915a, 3 },
    { 0xcc3c915b, 4 },
    { 0x48915b, 4 },
    { 0x3e84915c, 3 },
    { 0xb080915e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64149163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8916d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61589170, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44c9173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba309176, 3 },
    { 0xbe849177, 3 },
    { 0xe8389178, 4 },
    { 0x6c4c9177, 4 },
    { 0, 0 },
    { 0xcdcc917b, 4 },
    { 0xa190917b, 3 },
    { 0x9a70917d, 4 },
    { 0x703c917e, 4 },
    { 0x611c917f, 3 },
    { 0, 0 },
    { 0xd73c9181, 3 },
    { 0x133c9182, 3 },
    { 0, 0 },
    { 0xbb49184, 3 },
    { 0x2fa89185, 3 },
    { 0xf2609184, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4918b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c68918e, 4 },
    { 0xd0d0918e, 3 },
    { 0x91c9190, 3 },
    { 0xdfc49191, 3 },
    { 0, 0 },
    { 0xe60c9193, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee74919a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70919d, 4 },
    { 0x3528919e, 4 },
    { 0xcab4919f, 3 },
    { 0x630919e, 3 },
    { 0x5038919f, 3 },
    { 0xe61091a2, 4 },
    { 0x75f891a2, 3 },
    { 0x8edc91a4, 4 },
    { 0x9b7891a2, 4 },
    { 0xab2491a6, 3 },
    { 0x4c7891a6, 4 },
    { 0xe26891a8, 3 },
    { 0x45e891a9, 3 },
    { 0x1eb491aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9c91ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba891b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01491bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a091bf, 3 },
    { 0x499491c0, 4 },
    { 0xc0c091c0, 3 },
    { 0xdac91c0, 4 },
    { 0xdc3c91c3, 3 },
    { 0xa00c91c3, 3 },
    { 0, 0 },
    { 0x884091c6, 4 },
    { 0, 0 },
    { 0x702891c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6c91cd, 4 },
    { 0x3e0c91cd, 3 },
    { 0x652891cf, 3 },
    { 0xb8c91d0, 3 },
    { 0x24b091d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312891df, 3 },
    { 0, 0 },
    { 0x681091e1, 3 },
    { 0x7ca091e2, 4 },
    { 0xb57091e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64c91e6, 3 },
    { 0x4acc91e6, 4 },
    { 0xe32891e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c891f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999091fa, 4 },
    { 0x6c7491fa, 3 },
    { 0x3b2491fc, 3 },
    { 0, 0 },
    { 0x113091fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44789205, 3 },
    { 0x68989205, 3 },
    { 0x5f449207, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbc920a, 3 },
    { 0x5a58920b, 4 },
    { 0x5cd0920b, 3 },
    { 0x949c920b, 3 },
    { 0, 0 },
    { 0xd014920f, 4 },
    { 0xd548920f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e49216, 3 },
    { 0x15589217, 4 },
    { 0, 0 },
    { 0x85749219, 3 },
    { 0xac24921a, 3 },
    { 0xc724921b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5789220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fc9227, 3 },
    { 0x7b4c9228, 4 },
    { 0xc40c9229, 3 },
    { 0xa6d89229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5894922e, 3 },
    { 0, 0 },
    { 0x8e749230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c9233, 3 },
    { 0, 0 },
    { 0x7ec89235, 3 },
    { 0x85949236, 3 },
    { 0xaa949237, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe468923b, 3 },
    { 0xee4c923b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59609242, 3 },
    { 0xeaa09243, 3 },
    { 0xf8789242, 4 },
    { 0x6cb89245, 3 },
    { 0x2f689246, 4 },
    { 0xf9749243, 3 },
    { 0xbcdc9247, 3 },
    { 0x63e49244, 4 },
    { 0x25b09249, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb0924e, 3 },
    { 0xb358924f, 3 },
    { 0x7ae8924f, 3 },
    { 0, 0 },
    { 0xd1709252, 3 },
    { 0x455c9253, 3 },
    { 0xda349252, 3 },
    { 0x63b09255, 3 },
    { 0xc0709256, 3 },
    { 0x40189255, 4 },
    { 0, 0 },
    { 0x31f89259, 3 },
    { 0x48589259, 4 },
    { 0xd5a4925b, 3 },
    { 0x7c5c925b, 3 },
    { 0, 0 },
    { 0x39dc925e, 4 },
    { 0x7ffc925f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c49264, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c9267, 3 },
    { 0, 0 },
    { 0xc5789269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x798926c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec89273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea0927b, 3 },
    { 0xd51c927c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb09280, 3 },
    { 0x403c9281, 3 },
    { 0xcec9282, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18149288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c0928c, 3 },
    { 0, 0 },
    { 0x3b04928e, 3 },
    { 0xb1dc928f, 3 },
    { 0x51cc928e, 3 },
    { 0, 0 },
    { 0x16e49292, 3 },
    { 0xc8309293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a89296, 3 },
    { 0x32209297, 3 },
    { 0xf4409296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094929b, 4 },
    { 0xe3a0929c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4892a1, 3 },
    { 0x1c6492a2, 3 },
    { 0, 0 },
    { 0x463c92a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5c92a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244092ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f092b1, 3 },
    { 0, 0 },
    { 0x35b092b3, 4 },
    { 0, 0 },
    { 0xba1892b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88c92b9, 4 },
    { 0, 0 },
    { 0xaa5c92bb, 3 },
    { 0, 0 },
    { 0xabcc92bd, 4 },
    { 0, 0 },
    { 0xc75c92bf, 3 },
    { 0xfaa892c0, 3 },
    { 0xd0bc92c1, 4 },
    { 0x304c92c0, 4 },
    { 0x1fc492c1, 3 },
    { 0x75c492c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4c92ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62492cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20492d8, 3 },
    { 0, 0 },
    { 0xf73492da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc92dd, 3 },
    { 0x2e7092de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1092e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5092e9, 3 },
    { 0xcd3892e9, 3 },
    { 0, 0 },
    { 0x5e3c92ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610c92f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c92fa, 3 },
    { 0x11f892fa, 3 },
    { 0xa80092fb, 3 },
    { 0x628c92fd, 3 },
    { 0x705492fd, 4 },
    { 0xddac92fd, 3 },
    { 0x4c892ff, 3 },
    { 0x2e549300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c9307, 3 },
    { 0, 0 },
    { 0x56809309, 3 },
    { 0x410c930a, 3 },
    { 0xe204930b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457c9310, 3 },
    { 0xa8409310, 3 },
    { 0, 0 },
    { 0xa6f89313, 4 },
    { 0x7f0c9314, 4 },
    { 0x6a889315, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa818931c, 3 },
    { 0xa530931c, 3 },
    { 0x36b0931d, 3 },
    { 0xf338931e, 3 },
    { 0xbf18931f, 3 },
    { 0x2a9c9321, 4 },
    { 0xcc0c9322, 4 },
    { 0x4e6c9322, 4 },
    { 0xa8e89323, 4 },
    { 0x3e8931c, 3 },
    { 0x77ac9326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c932a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5949332, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3449336, 3 },
    { 0, 0 },
    { 0x8e0c9338, 3 },
    { 0x86209338, 3 },
    { 0xe149338, 3 },
    { 0x9c14933b, 3 },
    { 0x2f2c933c, 3 },
    { 0x3980933b, 4 },
    { 0x545c933d, 4 },
    { 0xac34933c, 3 },
    { 0xd8989340, 4 },
    { 0xbdf09340, 3 },
    { 0xf9289341, 4 },
    { 0xc614933c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec9346, 3 },
    { 0, 0 },
    { 0x71189348, 3 },
    { 0, 0 },
    { 0x67b4934a, 3 },
    { 0x7208934b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c934f, 3 },
    { 0xb9889350, 3 },
    { 0, 0 },
    { 0x90249352, 4 },
    { 0x2fc89353, 3 },
    { 0x32049354, 3 },
    { 0x17909354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c9358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e8935c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c0935f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98689362, 3 },
    { 0x95209363, 3 },
    { 0x5f389364, 3 },
    { 0x95789365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33649369, 3 },
    { 0x44f89369, 4 },
    { 0, 0 },
    { 0x2ac8936c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa214936f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d49372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1c9378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c00937b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c89381, 3 },
    { 0x61389382, 3 },
    { 0xcec49383, 3 },
    { 0xb6e09383, 4 },
    { 0xe9749385, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680c938c, 4 },
    { 0, 0 },
    { 0xdcc938e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3149392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3209398, 3 },
    { 0, 0 },
    { 0xaf78939a, 3 },
    { 0, 0 },
    { 0x898c939c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2893a0, 3 },
    { 0x8b2893a1, 3 },
    { 0x503493a2, 5 },
    { 0xec9493a3, 3 },
    { 0xf27893a4, 3 },
    { 0x33ec93a4, 4 },
    { 0x127093a0, 4 },
    { 0xc91893a3, 4 },
    { 0, 0 },
    { 0x497093a9, 4 },
    { 0x1c4493aa, 4 },
    { 0x6bcc93ab, 3 },
    { 0xe7bc93ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192c93b0, 3 },
    { 0xde4c93b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206c93b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c93bb, 3 },
    { 0xfb7c93bc, 3 },
    { 0xbb8493bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e093c2, 3 },
    { 0, 0 },
    { 0x630c93c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c93c9, 3 },
    { 0xc20c93ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43093d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed493d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a093db, 3 },
    { 0x700893dc, 3 },
    { 0, 0 },
    { 0x35e893de, 4 },
    { 0xf9fc93de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288c93e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda093e7, 4 },
    { 0, 0 },
    { 0xe4bc93e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x702893ec, 4 },
    { 0x882093ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ac93f2, 3 },
    { 0xe9c093f3, 4 },
    { 0, 0 },
    { 0xf04493f5, 3 },
    { 0xf77093f6, 3 },
    { 0, 0 },
    { 0x947893f8, 5 },
    { 0x3b2093f9, 3 },
    { 0x98d493fa, 3 },
    { 0xcdb893fb, 3 },
    { 0xf39093fc, 3 },
    { 0xd0c493fd, 3 },
    { 0x85b093fe, 3 },
    { 0x8d8093ff, 3 },
    { 0xac3493fb, 4 },
    { 0xc94893fa, 4 },
    { 0x17289402, 3 },
    { 0x8ad49403, 4 },
    { 0x60309404, 3 },
    { 0xd03093ff, 3 },
    { 0x29b09406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad89409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf480940e, 3 },
    { 0x640c940f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee49415, 4 },
    { 0x12589416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a49419, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1498941c, 3 },
    { 0x3a60941d, 3 },
    { 0x9d20941e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab09424, 3 },
    { 0xf62c9424, 4 },
    { 0x52089424, 3 },
    { 0xba689427, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dc942a, 4 },
    { 0, 0 },
    { 0xb3d0942c, 3 },
    { 0, 0 },
    { 0xe194942e, 3 },
    { 0x4758942f, 5 },
    { 0x34f09430, 4 },
    { 0x141c9431, 3 },
    { 0x576c9431, 3 },
    { 0xc1f49433, 4 },
    { 0x3b489434, 4 },
    { 0xcdbc9430, 3 },
    { 0, 0 },
    { 0x8f7c9437, 3 },
    { 0, 0 },
    { 0xe4189439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e4943c, 3 },
    { 0xcacc943c, 3 },
    { 0x210943e, 3 },
    { 0x994c943e, 3 },
    { 0x2784943f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9449, 3 },
    { 0xe800944a, 3 },
    { 0x4ce4944a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d0944e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e09453, 3 },
    { 0x6a4c9453, 4 },
    { 0x67949453, 3 },
    { 0x9ae89456, 3 },
    { 0xa72c9454, 3 },
    { 0x6b5c9455, 3 },
    { 0xf5f49459, 3 },
    { 0x7a149457, 3 },
    { 0x10d89453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7789461, 3 },
    { 0xea5c9462, 3 },
    { 0, 0 },
    { 0x99289464, 4 },
    { 0x5ad49465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3209468, 3 },
    { 0xc0bc9469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c946c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c9478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25609482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c49487, 3 },
    { 0x1cb09488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc754948c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1434949c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3c94a0, 3 },
    { 0x191c94a1, 3 },
    { 0x132894a2, 3 },
    { 0x3c6894a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x791494a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732494aa, 3 },
    { 0, 0 },
    { 0x6b2494ac, 3 },
    { 0x726c94ad, 4 },
    { 0, 0 },
    { 0x20894af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01894b3, 3 },
    { 0xd52894b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x354c94b7, 3 },
    { 0, 0 },
    { 0x247c94b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bc94bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb894c2, 3 },
    { 0xe1d494c3, 3 },
    { 0x6de894c4, 3 },
    { 0x20a894c3, 3 },
    { 0, 0 },
    { 0x86c94c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71494d0, 3 },
    { 0, 0 },
    { 0x41b494d2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa494d6, 3 },
    { 0xdd5494d7, 3 },
    { 0xd81c94d6, 3 },
    { 0x2ab894d9, 4 },
    { 0x999894d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5894de, 3 },
    { 0x7494de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbc94e3, 3 },
    { 0x390c94e3, 4 },
    { 0x8c5894e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80c94e8, 3 },
    { 0x545494e9, 3 },
    { 0xadb894e9, 3 },
    { 0x3e8c94eb, 3 },
    { 0xcf8c94eb, 3 },
    { 0, 0 },
    { 0xa35c94ee, 3 },
    { 0x34a494ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48894f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e894f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c8950a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0109513, 3 },
    { 0, 0 },
    { 0xc3749515, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa189518, 4 },
    { 0, 0 },
    { 0xd894951a, 3 },
    { 0x3524951b, 3 },
    { 0xb344951c, 3 },
    { 0xfb40951c, 4 },
    { 0x6034951d, 3 },
    { 0x5bb4951d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c849524, 3 },
    { 0, 0 },
    { 0xaddc9526, 3 },
    { 0xff389527, 3 },
    { 0x53a89526, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9628952b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91c9530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57609533, 4 },
    { 0x22e49534, 3 },
    { 0x93a09535, 3 },
    { 0x7f289535, 3 },
    { 0x5a849537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e04953a, 3 },
    { 0, 0 },
    { 0xb89c953c, 3 },
    { 0xe008953c, 3 },
    { 0x7b64953d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9320954b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9789552, 3 },
    { 0xb1209553, 3 },
    { 0xd4209554, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe809558, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f49560, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0749563, 4 },
    { 0, 0 },
    { 0x3cc09565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d10956a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5874956e, 3 },
    { 0, 0 },
    { 0xdbc89570, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab389577, 4 },
    { 0, 0 },
    { 0xbe5c9579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7850957c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4289586, 4 },
    { 0x65909587, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x728c958a, 3 },
    { 0xc540958b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c9590, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca49593, 4 },
    { 0x5e649594, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92689599, 3 },
    { 0x5b60959a, 3 },
    { 0x5a24959b, 3 },
    { 0, 0 },
    { 0x272c959d, 4 },
    { 0x8268959e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974095a3, 4 },
    { 0xadc895a4, 4 },
    { 0, 0 },
    { 0xc37c95a6, 4 },
    { 0x40c95a6, 3 },
    { 0x1bb495a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec95ae, 3 },
    { 0, 0 },
    { 0x23e895b0, 3 },
    { 0x89c095b1, 4 },
    { 0, 0 },
    { 0xd07895b3, 3 },
    { 0xde8095b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e095b7, 3 },
    { 0x364095b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71095be, 3 },
    { 0x7d1495bf, 3 },
    { 0x8e3495c0, 3 },
    { 0xa59095c1, 3 },
    { 0x70f095c2, 3 },
    { 0x1e9895c1, 3 },
    { 0xbd1895c3, 3 },
    { 0, 0 },
    { 0xf8cc95c6, 4 },
    { 0x21c095c7, 3 },
    { 0xde8895c8, 3 },
    { 0x639c95c6, 3 },
    { 0, 0 },
    { 0x641095cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c495d0, 3 },
    { 0x28fc95d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb895d7, 3 },
    { 0xc3c895d8, 4 },
    { 0, 0 },
    { 0x1ef895da, 3 },
    { 0, 0 },
    { 0xcc8895dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc95e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c895e3, 4 },
    { 0x1d6495e4, 3 },
    { 0xb39095e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc95ee, 3 },
    { 0x557c95ef, 3 },
    { 0x46f495ee, 3 },
    { 0, 0 },
    { 0xa36495f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e495f6, 3 },
    { 0x515895f7, 4 },
    { 0xd1b495f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b895fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x749895ff, 3 },
    { 0x43609600, 3 },
    { 0, 0 },
    { 0xbf2c9602, 3 },
    { 0x6fc9603, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8c9606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b84960b, 3 },
    { 0, 0 },
    { 0xaff0960d, 4 },
    { 0x368960e, 3 },
    { 0x7c0c960e, 3 },
    { 0x9f909610, 3 },
    { 0xd2789611, 3 },
    { 0xd3d49612, 3 },
    { 0x995c9612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c9618, 3 },
    { 0x19409618, 4 },
    { 0xc704961a, 3 },
    { 0x801c9618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985c9621, 4 },
    { 0x3b049621, 4 },
    { 0, 0 },
    { 0x5709624, 3 },
    { 0, 0 },
    { 0xd189626, 4 },
    { 0xb3b09627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee09630, 3 },
    { 0xf9dc9631, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e4963b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b0963e, 4 },
    { 0xec68963f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96c9642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb809645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1149648, 4 },
    { 0x75c89649, 3 },
    { 0x52bc9648, 3 },
    { 0, 0 },
    { 0xe368964c, 5 },
    { 0x5cb0964d, 3 },
    { 0xc86c964e, 4 },
    { 0, 0 },
    { 0x52c89650, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd449653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c9657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x115c965a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78965d, 4 },
    { 0x94b4965e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3589661, 3 },
    { 0x4d849662, 4 },
    { 0xb3189661, 3 },
    { 0x1de89664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6489667, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e8966b, 3 },
    { 0xd538966c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc9670, 4 },
    { 0, 0 },
    { 0xc21c9672, 3 },
    { 0, 0 },
    { 0x22dc9674, 3 },
    { 0xf2449674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bc9678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd8967c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9681, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ac9684, 3 },
    { 0x33049685, 3 },
    { 0xa8349686, 4 },
    { 0x91509686, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794968c, 3 },
    { 0xf18c968c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92289693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f49697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b4969a, 3 },
    { 0x60e0969b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4338969f, 3 },
    { 0xce20969f, 4 },
    { 0xdfcc96a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ac96a4, 3 },
    { 0x28f096a5, 3 },
    { 0, 0 },
    { 0x782896a7, 3 },
    { 0, 0 },
    { 0x1a2c96a9, 4 },
    { 0x9abc96aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970896b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c096b4, 3 },
    { 0, 0 },
    { 0xde9c96b6, 4 },
    { 0x719896b7, 3 },
    { 0x290096b8, 3 },
    { 0xa00096b6, 3 },
    { 0x61e896ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf496bd, 3 },
    { 0xd51496be, 3 },
    { 0xb05496bd, 4 },
    { 0x9bd896bf, 3 },
    { 0xd84c96be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239496c6, 4 },
    { 0x771096c6, 3 },
    { 0xcbd896c6, 3 },
    { 0xbb5c96c9, 3 },
    { 0, 0 },
    { 0x863896cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52496ce, 4 },
    { 0x749896cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72496d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3896d5, 3 },
    { 0, 0 },
    { 0x83d096d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b096df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e896e3, 3 },
    { 0, 0 },
    { 0xf60496e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb496ea, 3 },
    { 0xcf2096eb, 3 },
    { 0x765096ec, 3 },
    { 0x96496ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815c96fb, 3 },
    { 0, 0 },
    { 0x68dc96fd, 3 },
    { 0, 0 },
    { 0x9fa896ff, 3 },
    { 0x7fc09700, 3 },
    { 0xdde49700, 3 },
    { 0xf05496ff, 4 },
    { 0x14509703, 3 },
    { 0xda589704, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad49707, 3 },
    { 0x81089708, 3 },
    { 0, 0 },
    { 0x6748970a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98989710, 3 },
    { 0x8609711, 4 },
    { 0xd8449711, 3 },
    { 0x75b89713, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72909717, 3 },
    { 0x36709717, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2c971f, 4 },
    { 0, 0 },
    { 0x4ea89721, 3 },
    { 0, 0 },
    { 0x2ee49723, 3 },
    { 0xad709723, 3 },
    { 0, 0 },
    { 0x30589726, 3 },
    { 0, 0 },
    { 0xa7cc9728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c972c, 3 },
    { 0xecf0972c, 4 },
    { 0, 0 },
    { 0xeeb8972f, 3 },
    { 0xe88c9730, 3 },
    { 0xe0c89731, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cec9735, 4 },
    { 0x85b09736, 5 },
    { 0x61e09737, 4 },
    { 0xb3789737, 3 },
    { 0x4909738, 3 },
    { 0x98fc973a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5470973e, 4 },
    { 0, 0 },
    { 0x7fb09740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4609744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a389747, 3 },
    { 0, 0 },
    { 0x94349749, 3 },
    { 0x96d8974a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd789750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70589756, 3 },
    { 0, 0 },
    { 0x2ca89758, 3 },
    { 0, 0 },
    { 0x9bfc975a, 3 },
    { 0x8a50975a, 3 },
    { 0xe844975c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b8975f, 3 },
    { 0x9964975f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a049764, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde049768, 3 },
    { 0, 0 },
    { 0xc4c976a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c9770, 3 },
    { 0xfc409770, 4 },
    { 0, 0 },
    { 0x9e309773, 3 },
    { 0xd4b89774, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e49777, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f8977d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52189780, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c249786, 3 },
    { 0, 0 },
    { 0xdcec9788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7228978c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed49790, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c9795, 3 },
    { 0xc0789796, 3 },
    { 0, 0 },
    { 0x54509798, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bc979c, 3 },
    { 0, 0 },
    { 0x828979e, 3 },
    { 0x6f8979e, 3 },
    { 0x81fc97a0, 3 },
    { 0x472c97a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338c97a5, 3 },
    { 0xa61897a6, 4 },
    { 0xcc8097a7, 3 },
    { 0xaed497a8, 3 },
    { 0x9d4c97a9, 3 },
    { 0xfe2c97aa, 3 },
    { 0x69897aa, 3 },
    { 0x59ac97ac, 4 },
    { 0x2e0c97ad, 4 },
    { 0xf5dc97ac, 3 },
    { 0xb30897af, 3 },
    { 0x87497a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564897b4, 4 },
    { 0x6bf897b4, 4 },
    { 0xc8497b4, 3 },
    { 0, 0 },
    { 0x153497b8, 3 },
    { 0x3d1c97b8, 3 },
    { 0xfa0497ba, 3 },
    { 0x73e897bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4497be, 3 },
    { 0x130897be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2097c7, 3 },
    { 0, 0 },
    { 0x9d6097c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c497cc, 4 },
    { 0, 0 },
    { 0x3e097ce, 3 },
    { 0xa85c97cf, 3 },
    { 0, 0 },
    { 0xf1a897d1, 3 },
    { 0x671097d1, 4 },
    { 0x3f8897d2, 3 },
    { 0xa88c97d2, 3 },
    { 0, 0 },
    { 0x832097d6, 3 },
    { 0, 0 },
    { 0x6ad497d8, 4 },
    { 0xb5cc97d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142c97df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b097e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5897e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf097ec, 3 },
    { 0, 0 },
    { 0xafc097ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c097f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e497f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bc97fc, 4 },
    { 0, 0 },
    { 0x3cc097fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2c9803, 4 },
    { 0, 0 },
    { 0x89b49805, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1749808, 3 },
    { 0x65d09809, 3 },
    { 0xcd849808, 3 },
    { 0x1560980b, 3 },
    { 0xf3e0980b, 3 },
    { 0x78749808, 4 },
    { 0x8768980b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76c9814, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc981c, 3 },
    { 0xd5fc981d, 4 },
    { 0xfee8981e, 3 },
    { 0xb954981f, 3 },
    { 0xb9549820, 4 },
    { 0, 0 },
    { 0x91b89822, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d89825, 4 },
    { 0, 0 },
    { 0xa4509827, 3 },
    { 0x4f909828, 3 },
    { 0x82c09829, 3 },
    { 0x7b1c9828, 4 },
    { 0x9ba0982b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x243c982e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3689833, 3 },
    { 0xcde09834, 4 },
    { 0xbbe09835, 3 },
    { 0xdbe49836, 4 },
    { 0xe29c9837, 4 },
    { 0x3ba09837, 3 },
    { 0x7eec9839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0c9844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c9848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c984b, 3 },
    { 0, 0 },
    { 0x84d4984d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f0985a, 3 },
    { 0xd2ec985b, 3 },
    { 0xe0d8985c, 4 },
    { 0x2560985d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc49866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9009869, 3 },
    { 0xaad0986a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce049878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734987b, 3 },
    { 0x6ccc987b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81249880, 3 },
    { 0xda149881, 5 },
    { 0, 0 },
    { 0xfdec9883, 3 },
    { 0x2d209884, 3 },
    { 0x68a09885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c9889, 3 },
    { 0xff64988a, 3 },
    { 0xc4e4988a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe8988e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf109893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa09896, 3 },
    { 0, 0 },
    { 0x6d09898, 3 },
    { 0, 0 },
    { 0x1d3c989a, 3 },
    { 0, 0 },
    { 0x3f20989c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc140989f, 3 },
    { 0xd4a498a0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49098a8, 3 },
    { 0x3e2c98a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275898ad, 4 },
    { 0x1ca498ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194c98b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda498b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1898bd, 3 },
    { 0x8f0c98bd, 3 },
    { 0x3fe498bf, 4 },
    { 0x820c98c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77498c4, 3 },
    { 0x72fc98c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83c98c8, 3 },
    { 0x66e498c9, 3 },
    { 0x4b3498c8, 3 },
    { 0xd79898cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7498ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354098d6, 3 },
    { 0xbd2498d7, 3 },
    { 0x92f098d7, 3 },
    { 0x28bc98d7, 3 },
    { 0x885498da, 3 },
    { 0x90a898d9, 3 },
    { 0x5c8c98d9, 5 },
    { 0x16498dd, 3 },
    { 0x40ac98de, 3 },
    { 0xc8a898df, 4 },
    { 0x238498e0, 4 },
    { 0x17c98dd, 3 },
    { 0x7de098e0, 3 },
    { 0x41c98e3, 3 },
    { 0x4e3c98e4, 3 },
    { 0xb91098dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2098ee, 3 },
    { 0x63f098ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69098f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cc98fa, 3 },
    { 0, 0 },
    { 0x8a5098fc, 3 },
    { 0xbca498fd, 3 },
    { 0x1b5098fe, 3 },
    { 0, 0 },
    { 0x35749900, 3 },
    { 0x63b09901, 3 },
    { 0x95549902, 4 },
    { 0xd0e09901, 3 },
    { 0x2cb09904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1c990b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c9910, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4289916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1389919, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3c991c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c4991f, 3 },
    { 0xd8d49920, 4 },
    { 0xe1a49920, 3 },
    { 0, 0 },
    { 0x2e909923, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d889927, 3 },
    { 0, 0 },
    { 0xff249929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb044992e, 4 },
    { 0, 0 },
    { 0x9b489930, 3 },
    { 0xd9f89930, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6789937, 3 },
    { 0, 0 },
    { 0xbee09939, 4 },
    { 0, 0 },
    { 0x3e08993b, 3 },
    { 0xaf00993b, 3 },
    { 0, 0 },
    { 0x8fc993e, 3 },
    { 0xd248993f, 3 },
    { 0, 0 },
    { 0xc85c9941, 3 },
    { 0x6689942, 3 },
    { 0x8bec9941, 4 },
    { 0x7ab49944, 3 },
    { 0xb05c9945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa40994a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fc994f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c209957, 3 },
    { 0x62149957, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3538995d, 3 },
    { 0, 0 },
    { 0xa2ec995f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223c9967, 4 },
    { 0xd9c9968, 3 },
    { 0xabb09967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f489971, 4 },
    { 0, 0 },
    { 0x54349973, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafec9977, 3 },
    { 0, 0 },
    { 0x35e49979, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc9981, 3 },
    { 0xa0289982, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3709985, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3449988, 4 },
    { 0xf5689989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f88998d, 3 },
    { 0xfd84998e, 3 },
    { 0x41d8998f, 3 },
    { 0xff7c9990, 3 },
    { 0x9b989991, 3 },
    { 0x9480998e, 3 },
    { 0x4f7c9991, 3 },
    { 0x88ac9994, 3 },
    { 0, 0 },
    { 0x88fc9996, 3 },
    { 0x1e2c9997, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff4999b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a28999f, 4 },
    { 0x5b4c999f, 3 },
    { 0, 0 },
    { 0x628099a2, 3 },
    { 0, 0 },
    { 0x3ef899a4, 4 },
    { 0, 0 },
    { 0x20a099a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c499ac, 3 },
    { 0, 0 },
    { 0x955c99ae, 3 },
    { 0x159099af, 3 },
    { 0, 0 },
    { 0x89e499b1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477c99b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bc99bb, 3 },
    { 0x724499bc, 3 },
    { 0x62099bb, 3 },
    { 0, 0 },
    { 0xc6c899bf, 4 },
    { 0, 0 },
    { 0xa0e899c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c499c5, 4 },
    { 0x921099c5, 4 },
    { 0x481099c7, 3 },
    { 0x7de899c7, 3 },
    { 0xf20c99c7, 4 },
    { 0x1f2899ca, 4 },
    { 0x37ec99c5, 4 },
    { 0, 0 },
    { 0x8aac99cd, 3 },
    { 0, 0 },
    { 0xbaf099cf, 3 },
    { 0xc90c99d0, 3 },
    { 0xaaf499d1, 4 },
    { 0x4fe099d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656c99d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b899db, 3 },
    { 0x227499db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542099e1, 4 },
    { 0x56e099e2, 3 },
    { 0x812899e2, 4 },
    { 0, 0 },
    { 0x15a099e5, 3 },
    { 0x62d899e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0499e9, 3 },
    { 0, 0 },
    { 0x639c99eb, 3 },
    { 0x196499ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1899f0, 4 },
    { 0, 0 },
    { 0xbaf099f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca899f5, 3 },
    { 0xf1a099f5, 4 },
    { 0, 0 },
    { 0xefd099f8, 3 },
    { 0xe8499f9, 3 },
    { 0, 0 },
    { 0xa2c899fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08499ff, 3 },
    { 0, 0 },
    { 0xe649a01, 4 },
    { 0x86c9a02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca49a05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea809a09, 3 },
    { 0x7c0c9a0a, 3 },
    { 0x75fc9a09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c49a0e, 4 },
    { 0x20749a0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c989a12, 4 },
    { 0x94989a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25649a1a, 3 },
    { 0x72b89a1a, 3 },
    { 0x6409a1a, 4 },
    { 0, 0 },
    { 0x64609a1e, 5 },
    { 0, 0 },
    { 0x3aec9a20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac9a26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac249a2b, 3 },
    { 0, 0 },
    { 0x48f09a2d, 3 },
    { 0, 0 },
    { 0x814c9a2f, 4 },
    { 0x98649a30, 4 },
    { 0, 0 },
    { 0xd4889a32, 3 },
    { 0xd6149a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e049a38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ec9a3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff49a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d849a4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1749a4e, 3 },
    { 0, 0 },
    { 0x91949a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6809a53, 3 },
    { 0x62ac9a54, 3 },
    { 0x4c289a53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec9a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c9a5e, 3 },
    { 0, 0 },
    { 0xc3889a60, 4 },
    { 0, 0 },
    { 0xdf449a62, 3 },
    { 0, 0 },
    { 0x1e089a64, 3 },
    { 0, 0 },
    { 0x10549a66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc009a6f, 3 },
    { 0xcbc09a70, 3 },
    { 0x118c9a6f, 3 },
    { 0xc189a71, 4 },
    { 0, 0 },
    { 0x16349a74, 3 },
    { 0, 0 },
    { 0x31789a76, 3 },
    { 0x267c9a77, 3 },
    { 0x7aa09a78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38509a7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59549a82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c89a85, 3 },
    { 0x58e09a86, 3 },
    { 0xe889a87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecc9a8b, 3 },
    { 0xad9c9a8c, 3 },
    { 0x83489a8c, 3 },
    { 0, 0 },
    { 0x75049a8f, 3 },
    { 0, 0 },
    { 0xe09c9a91, 3 },
    { 0x8b389a92, 3 },
    { 0x19009a92, 3 },
    { 0xa58c9a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x745c9a97, 4 },
    { 0, 0 },
    { 0xdf849a99, 4 },
    { 0x5a7c9a9a, 3 },
    { 0x6189a9b, 4 },
    { 0xb9289a9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7749aa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50149aa9, 5 },
    { 0x6a249aaa, 3 },
    { 0x18309aaa, 3 },
    { 0xab309aaa, 5 },
    { 0x2c909aad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e949ab0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41809ab3, 3 },
    { 0, 0 },
    { 0x334c9ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5489abc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ac9ac0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4c9ac3, 3 },
    { 0xcb589ac4, 3 },
    { 0, 0 },
    { 0x59949ac6, 4 },
    { 0, 0 },
    { 0x5e849ac8, 3 },
    { 0xba809ac9, 3 },
    { 0xe3c89aca, 4 },
    { 0x6da09acb, 3 },
    { 0xe9a89aca, 3 },
    { 0x91889acd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57809ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207c9ad6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46149ada, 4 },
    { 0, 0 },
    { 0xee749adc, 3 },
    { 0x4cd09add, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8889ae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47609aec, 4 },
    { 0, 0 },
    { 0xdc389aee, 3 },
    { 0xfa349aef, 3 },
    { 0, 0 },
    { 0x92549af1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16c9af4, 3 },
    { 0xc3709af4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74c9afd, 3 },
    { 0xf0949afe, 3 },
    { 0x23109afe, 3 },
    { 0xccac9afd, 3 },
    { 0xb6d09b01, 3 },
    { 0xd4909b02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63009b07, 4 },
    { 0x9aac9b08, 3 },
    { 0xef289b09, 4 },
    { 0x8e589b0a, 3 },
    { 0xac789b0b, 3 },
    { 0xe0349b0c, 3 },
    { 0x2d789b09, 3 },
    { 0x88189b0e, 3 },
    { 0xa0a89b0a, 3 },
    { 0x4fb49b0c, 3 },
    { 0x2ef49b11, 3 },
    { 0x9549b12, 3 },
    { 0xa7589b0d, 4 },
    { 0x8a649b14, 3 },
    { 0xd7b89b07, 3 },
    { 0x55bc9b0a, 4 },
    { 0xf06c9b17, 4 },
    { 0, 0 },
    { 0x6a109b19, 4 },
    { 0xd8d49b19, 3 },
    { 0, 0 },
    { 0x277c9b1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e249b1f, 3 },
    { 0x7fcc9b1f, 3 },
    { 0, 0 },
    { 0x44c89b22, 4 },
    { 0x5da49b23, 3 },
    { 0x48289b24, 4 },
    { 0x4d449b24, 4 },
    { 0xb2c9b26, 3 },
    { 0x80349b27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a49b2a, 3 },
    { 0, 0 },
    { 0x29a09b2c, 3 },
    { 0xb9e09b2d, 4 },
    { 0xc0f49b2c, 3 },
    { 0x3e589b2d, 3 },
    { 0xdb7c9b30, 4 },
    { 0xb07c9b30, 3 },
    { 0xb7c09b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffc9b35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe089b3a, 3 },
    { 0xd7ec9b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c9b3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870c9b42, 3 },
    { 0x7acc9b43, 3 },
    { 0x96a49b44, 4 },
    { 0, 0 },
    { 0x37a49b46, 4 },
    { 0x90809b47, 3 },
    { 0x5d2c9b48, 4 },
    { 0x90449b47, 3 },
    { 0xb5089b4a, 5 },
    { 0, 0 },
    { 0x37bc9b4c, 3 },
    { 0x11049b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f549b50, 3 },
    { 0x430c9b51, 5 },
    { 0x96a89b52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e649b56, 3 },
    { 0x9489b56, 3 },
    { 0xc7409b58, 3 },
    { 0x16409b59, 3 },
    { 0, 0 },
    { 0x17d89b5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc89b60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d89b68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a289b6b, 3 },
    { 0, 0 },
    { 0x57c9b6d, 4 },
    { 0x39689b6e, 3 },
    { 0xe8ec9b6e, 4 },
    { 0xee3c9b70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99009b75, 3 },
    { 0x60709b76, 3 },
    { 0, 0 },
    { 0xe20c9b78, 4 },
    { 0x9a589b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15949b87, 3 },
    { 0xfc689b88, 3 },
    { 0x6649b89, 3 },
    { 0xc8e49b8a, 4 },
    { 0x345c9b8b, 3 },
    { 0xe5849b89, 3 },
    { 0xa9c09b8a, 3 },
    { 0x9d949b8e, 5 },
    { 0, 0 },
    { 0x218c9b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85c9b99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1749b9d, 4 },
    { 0x8bf09b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c9ba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc9ba4, 3 },
    { 0x90d09ba4, 4 },
    { 0x40289ba5, 4 },
    { 0xd8c09ba7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec9bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d09bb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe89bb3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0009bc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135c9bcb, 5 },
    { 0x82b89bcc, 5 },
    { 0x7b89bcc, 3 },
    { 0, 0 },
    { 0x14a49bcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba949bd3, 3 },
    { 0, 0 },
    { 0xe5909bd5, 3 },
    { 0, 0 },
    { 0x44a49bd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d09bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e409be0, 3 },
    { 0x4fe09be1, 3 },
    { 0, 0 },
    { 0x2c849be3, 3 },
    { 0xeca89be3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec209be7, 3 },
    { 0xa6f49be8, 3 },
    { 0x594c9be8, 3 },
    { 0, 0 },
    { 0x5849beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23509bf0, 3 },
    { 0xfc549bf1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94c9bf7, 3 },
    { 0xe24c9bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c9bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49c02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53789c0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14809c10, 3 },
    { 0xf1289c11, 3 },
    { 0, 0 },
    { 0x3ba49c13, 3 },
    { 0x94149c13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb49c19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af49c21, 3 },
    { 0xded49c22, 3 },
    { 0x82089c22, 3 },
    { 0x73d09c23, 3 },
    { 0xc9a89c22, 3 },
    { 0x8a609c25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c649c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f149c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4949c3d, 3 },
    { 0xbda49c3e, 3 },
    { 0xd2cc9c3e, 4 },
    { 0, 0 },
    { 0x2309c41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dc9c44, 3 },
    { 0xb0e09c45, 3 },
    { 0x63749c44, 3 },
    { 0xb1649c47, 3 },
    { 0x7cf49c47, 3 },
    { 0x1a889c49, 3 },
    { 0x8f749c46, 4 },
    { 0x6bf89c45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c989c4e, 3 },
    { 0x99389c4f, 4 },
    { 0x12c09c4e, 3 },
    { 0xfe9c9c4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd989c55, 3 },
    { 0x9a309c56, 3 },
    { 0x4ca09c57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9749c5c, 3 },
    { 0, 0 },
    { 0xbac9c5e, 3 },
    { 0x7fa09c5e, 4 },
    { 0x45549c60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70709c64, 4 },
    { 0, 0 },
    { 0x1b189c66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f609c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd209c74, 4 },
    { 0x12049c75, 3 },
    { 0x775c9c75, 4 },
    { 0x9a589c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad249c7c, 3 },
    { 0xa3609c7c, 3 },
    { 0, 0 },
    { 0x9b349c7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c9c82, 3 },
    { 0, 0 },
    { 0x81b89c84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e49c87, 3 },
    { 0xffe09c88, 3 },
    { 0xac2c9c88, 3 },
    { 0, 0 },
    { 0x5e3c9c8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d09c8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd09c96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec9c9b, 3 },
    { 0xb5789c9c, 3 },
    { 0x22ac9c9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x578c9ca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68389caa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcc9cad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd89cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd089cb6, 4 },
    { 0x1f989cb6, 4 },
    { 0x2df49cb8, 3 },
    { 0x459c9cb9, 3 },
    { 0x511c9cb8, 3 },
    { 0x9a849cb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c9cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a409cc5, 3 },
    { 0x1fac9cc6, 4 },
    { 0, 0 },
    { 0x46449cc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9489cce, 3 },
    { 0xf6709ccf, 4 },
    { 0x92e89cd0, 4 },
    { 0xe389cce, 3 },
    { 0, 0 },
    { 0x215c9cd3, 3 },
    { 0, 0 },
    { 0x9b509cd5, 3 },
    { 0x84d09cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c549cdb, 3 },
    { 0xae849cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc009ce2, 3 },
    { 0x5a409ce2, 3 },
    { 0xb93c9ce4, 3 },
    { 0, 0 },
    { 0x23749ce6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3089cea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c9ced, 3 },
    { 0xd3ec9ced, 3 },
    { 0x2ef89cee, 3 },
    { 0x34ec9cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc89cf3, 4 },
    { 0, 0 },
    { 0x44809cf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3609cf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e049cfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c89d00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68049d05, 3 },
    { 0xe2cc9d05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2649d0b, 4 },
    { 0, 0 },
    { 0x775c9d0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc9d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac9d15, 3 },
    { 0x7d289d15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70709d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb09d23, 3 },
    { 0x9c809d24, 3 },
    { 0x2189d25, 3 },
    { 0, 0 },
    { 0xfa749d27, 3 },
    { 0xaa49d27, 4 },
    { 0xde349d29, 4 },
    { 0x3dc49d2a, 3 },
    { 0x66049d2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa89d2e, 3 },
    { 0x40689d2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x158c9d32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f549d36, 3 },
    { 0x40ec9d37, 3 },
    { 0x33c9d37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc09d3d, 3 },
    { 0x971c9d3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae09d42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a89d46, 3 },
    { 0x2e849d47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb289d4a, 3 },
    { 0, 0 },
    { 0xdd2c9d4c, 3 },
    { 0xdff89d4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68809d54, 3 },
    { 0, 0 },
    { 0x9e0c9d56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74389d5a, 3 },
    { 0, 0 },
    { 0x3dc09d5c, 3 },
    { 0, 0 },
    { 0x61389d5e, 3 },
    { 0x18389d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac949d64, 3 },
    { 0xd5449d65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d09d68, 3 },
    { 0x8d009d69, 3 },
    { 0x5b709d69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a09d70, 3 },
    { 0x61e49d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b849d74, 3 },
    { 0, 0 },
    { 0x9dac9d76, 3 },
    { 0xce5c9d77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73209d80, 3 },
    { 0, 0 },
    { 0xfc8c9d82, 5 },
    { 0xb6ac9d83, 3 },
    { 0xbc789d84, 3 },
    { 0x35549d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0509d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ac9d8f, 3 },
    { 0x1e349d8f, 3 },
    { 0x1d989d91, 3 },
    { 0x68bc9d92, 4 },
    { 0xd74c9d93, 4 },
    { 0x4ec9d92, 3 },
    { 0, 0 },
    { 0xe2609d96, 4 },
    { 0, 0 },
    { 0xd1c49d98, 3 },
    { 0, 0 },
    { 0x309d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a89da2, 3 },
    { 0x1ab89da3, 3 },
    { 0xb6a49da3, 3 },
    { 0x774c9da4, 3 },
    { 0xd80c9da3, 3 },
    { 0xdd089da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56509daa, 3 },
    { 0xd8749daa, 4 },
    { 0x9be89daa, 3 },
    { 0xa6d89daa, 4 },
    { 0xad2c9dad, 4 },
    { 0, 0 },
    { 0xbe449db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb89db3, 3 },
    { 0xc3789db3, 3 },
    { 0, 0 },
    { 0x2d9c9db6, 3 },
    { 0x45cc9db7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f509dba, 3 },
    { 0x4549dbb, 3 },
    { 0xee549dbb, 3 },
    { 0xa40c9dbd, 3 },
    { 0x5c549dbe, 3 },
    { 0x73849dbb, 4 },
    { 0xf4549dc0, 3 },
    { 0x91d49dc1, 3 },
    { 0x14889dbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f09dc7, 3 },
    { 0, 0 },
    { 0xaea09dc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acc9dce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56889dd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68489dd8, 3 },
    { 0xace49dd8, 3 },
    { 0x22ac9dd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9289ddd, 3 },
    { 0xf14c9dde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b09dec, 3 },
    { 0xe8f89dec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f449df2, 3 },
    { 0, 0 },
    { 0xccd09df4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f909dfb, 3 },
    { 0, 0 },
    { 0x9ed49dfd, 4 },
    { 0xaf09dfd, 3 },
    { 0x3e009dfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc9e04, 3 },
    { 0, 0 },
    { 0xb53c9e06, 3 },
    { 0x43009e06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd009e0b, 4 },
    { 0x14c09e0b, 3 },
    { 0, 0 },
    { 0xc78c9e0e, 3 },
    { 0x762c9e0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b09e16, 4 },
    { 0xc3589e16, 3 },
    { 0xe23c9e18, 4 },
    { 0x9b389e19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb09e1c, 4 },
    { 0x5809e1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe049e21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e49e26, 3 },
    { 0x82d89e27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c049e2a, 3 },
    { 0xc87c9e2a, 3 },
    { 0x3d49e2b, 4 },
    { 0xef909e2d, 3 },
    { 0x3a949e2d, 3 },
    { 0xbda09e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f89e36, 3 },
    { 0x63d89e36, 3 },
    { 0, 0 },
    { 0x1ec09e39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a09e3c, 3 },
    { 0x29c9e3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b09e43, 4 },
    { 0x6309e44, 3 },
    { 0xee9c9e45, 3 },
    { 0x440c9e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b49e4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2c9e4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7c9e52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa109e57, 4 },
    { 0x86ac9e57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac89e5b, 3 },
    { 0xa9409e5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24509e5f, 3 },
    { 0x56f09e5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e89e64, 3 },
    { 0x1a809e64, 3 },
    { 0, 0 },
    { 0xa85c9e67, 3 },
    { 0, 0 },
    { 0x95709e69, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6c9e6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a309e70, 4 },
    { 0, 0 },
    { 0xa2ec9e72, 4 },
    { 0xed509e73, 3 },
    { 0x8a109e74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc89e77, 5 },
    { 0, 0 },
    { 0x10c09e79, 4 },
    { 0x13289e7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d09e7d, 5 },
    { 0, 0 },
    { 0x492c9e7f, 3 },
    { 0x91789e80, 3 },
    { 0xa87c9e7f, 5 },
    { 0, 0 },
    { 0xe5e09e83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed89e8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6389e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3209e92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac49e9b, 3 },
    { 0, 0 },
    { 0x84689e9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d049ea0, 3 },
    { 0x71e49ea1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cc9ea5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73009eac, 4 },
    { 0x344c9ead, 4 },
    { 0xdc209ead, 3 },
    { 0x90589eaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9989eb9, 3 },
    { 0x62a09eb9, 3 },
    { 0xc2109ebb, 3 },
    { 0xafd09ebc, 3 },
    { 0xf6849ebd, 4 },
    { 0, 0 },
    { 0x9dc9ebf, 4 },
    { 0, 0 },
    { 0x62b49ec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89ec4, 3 },
    { 0xd8e89ec5, 3 },
    { 0, 0 },
    { 0xc0409ec7, 3 },
    { 0, 0 },
    { 0xe6789ec9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32709ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a349ecf, 4 },
    { 0x4cd49ed0, 3 },
    { 0x4ca89ed1, 4 },
    { 0x1ccc9ed1, 3 },
    { 0, 0 },
    { 0x18909ed4, 3 },
    { 0xf4e49ed4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16789eda, 3 },
    { 0, 0 },
    { 0xabd09edc, 4 },
    { 0x3709edd, 3 },
    { 0xc02c9ede, 3 },
    { 0, 0 },
    { 0x46609ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b689ee9, 3 },
    { 0x1db89ee9, 3 },
    { 0xb3289eeb, 4 },
    { 0x11ec9eec, 4 },
    { 0x89649eec, 3 },
    { 0x67509eee, 3 },
    { 0xb0d49ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa789ef2, 3 },
    { 0x8b549ef3, 3 },
    { 0x59dc9ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a89ef8, 3 },
    { 0x2f049ef9, 3 },
    { 0xb7149efa, 3 },
    { 0, 0 },
    { 0x2a1c9efc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x584c9eff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc689f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292c9f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a89f09, 3 },
    { 0, 0 },
    { 0xd9c9f0b, 4 },
    { 0x7a289f0b, 3 },
    { 0xc1e89f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a089f10, 3 },
    { 0x89f89f10, 3 },
    { 0xd3d89f11, 3 },
    { 0x7b389f11, 3 },
    { 0x5bc9f12, 3 },
    { 0, 0 },
    { 0x5a209f16, 5 },
    { 0x16289f16, 3 },
    { 0, 0 },
    { 0x74b49f19, 3 },
    { 0xff4c9f1a, 4 },
    { 0x36409f1b, 4 },
    { 0, 0 },
    { 0x7ac09f1d, 4 },
    { 0, 0 },
    { 0x34a09f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c9f25, 3 },
    { 0xa4309f25, 3 },
    { 0x41c49f25, 3 },
    { 0x19109f28, 3 },
    { 0x70009f29, 3 },
    { 0xa8b09f2a, 3 },
    { 0x6d849f2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90589f2f, 3 },
    { 0x28509f30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e309f34, 3 },
    { 0x5ad09f35, 3 },
    { 0xeae49f35, 4 },
    { 0x6dec9f35, 3 },
    { 0xd3909f35, 3 },
    { 0x78049f39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f089f3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f049f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a609f4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5649f4f, 3 },
    { 0xf3049f4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a09f56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19809f59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59c9f5e, 3 },
    { 0xff349f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42449f63, 3 },
    { 0x68489f64, 3 },
    { 0xdc4c9f64, 3 },
    { 0x7ea09f66, 4 },
    { 0x38bc9f67, 3 },
    { 0, 0 },
    { 0x56a89f69, 3 },
    { 0xd7d49f6a, 3 },
    { 0xd509f6a, 4 },
    { 0, 0 },
    { 0xb0bc9f6d, 3 },
    { 0x45cc9f6d, 3 },
    { 0, 0 },
    { 0x2d649f70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6649f78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64909f81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63889f86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35249f8a, 4 },
    { 0x72249f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d249f8e, 3 },
    { 0x81009f8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea89f93, 3 },
    { 0xedb89f94, 3 },
    { 0xc75c9f93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0449f98, 4 },
    { 0, 0 },
    { 0xe6d09f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8009fa0, 4 },
    { 0, 0 },
    { 0x41e49fa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38649fa8, 3 },
    { 0x90e89fa8, 3 },
    { 0, 0 },
    { 0x5e909fab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c9fae, 3 },
    { 0xa6b49fae, 3 },
    { 0x59589fb0, 3 },
    { 0, 0 },
    { 0xf2d49fb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95849fb6, 3 },
    { 0xb9b09fb7, 3 },
    { 0xd4f89fb8, 3 },
    { 0xf2309fb9, 3 },
    { 0x3ba89fb9, 4 },
    { 0x7c449fbb, 4 },
    { 0xf6889fbc, 4 },
    { 0xbb49fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc9fc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec9fc7, 3 },
    { 0xbb389fc8, 3 },
    { 0x70409fc8, 5 },
    { 0, 0 },
    { 0xe44c9fcb, 3 },
    { 0, 0 },
    { 0x27249fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce289fd4, 3 },
    { 0, 0 },
    { 0x8b049fd6, 5 },
    { 0xa7f89fd7, 3 },
    { 0x5309fd8, 3 },
    { 0x6c649fd8, 3 },
    { 0, 0 },
    { 0x92049fdb, 3 },
    { 0xae309fdb, 4 },
    { 0xb26c9fdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd49fe9, 3 },
    { 0xaa889fea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31089fed, 4 },
    { 0x9449fee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bc9ff2, 3 },
    { 0x4fbc9ff3, 3 },
    { 0x9889ff4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2c9ff9, 3 },
    { 0, 0 },
    { 0x876c9ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3310a000, 3 },
    { 0x398ca001, 3 },
    { 0xb748a002, 3 },
    { 0xb10a002, 3 },
    { 0xed98a003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9890a00c, 3 },
    { 0xf64a00d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca8a010, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260a014, 3 },
    { 0x4370a015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d38a019, 3 },
    { 0, 0 },
    { 0xf62ca01b, 4 },
    { 0, 0 },
    { 0xd5a0a01d, 3 },
    { 0x44e4a01e, 3 },
    { 0, 0 },
    { 0xf438a020, 4 },
    { 0, 0 },
    { 0x8c1ca022, 3 },
    { 0, 0 },
    { 0x96e4a024, 3 },
    { 0xfa80a025, 4 },
    { 0, 0 },
    { 0xb3f8a027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5320a02a, 3 },
    { 0xb590a02b, 5 },
    { 0xe924a02c, 4 },
    { 0x7c88a02d, 3 },
    { 0xa228a02e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cca03e, 3 },
    { 0, 0 },
    { 0xbdfca040, 3 },
    { 0x3d18a041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5ca04a, 4 },
    { 0, 0 },
    { 0xbe50a04c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbca050, 3 },
    { 0xecca050, 3 },
    { 0x47f0a052, 4 },
    { 0x9344a053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaca058, 3 },
    { 0x13dca058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa934a05f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74ca067, 3 },
    { 0, 0 },
    { 0x2a4ca069, 3 },
    { 0, 0 },
    { 0x18c4a06b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ca06f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6684a076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3658a07a, 3 },
    { 0, 0 },
    { 0x6064a07c, 3 },
    { 0, 0 },
    { 0x30b4a07e, 4 },
    { 0x5b68a07f, 3 },
    { 0, 0 },
    { 0xb338a081, 3 },
    { 0x6e14a081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2508a086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d98a089, 3 },
    { 0x568ca08a, 3 },
    { 0x532ca08a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14ca08e, 3 },
    { 0, 0 },
    { 0x9f50a090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccca095, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ca09a, 3 },
    { 0x62e0a09b, 4 },
    { 0x4a24a09a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff34a09f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79ca0a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a0a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a94a0aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e8a0b4, 3 },
    { 0x5a6ca0b5, 3 },
    { 0x8a60a0b4, 3 },
    { 0xaab4a0b4, 3 },
    { 0, 0 },
    { 0x4d08a0b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa598a0bc, 3 },
    { 0xcc38a0bc, 3 },
    { 0, 0 },
    { 0x9d24a0bf, 3 },
    { 0x8868a0c0, 3 },
    { 0x1aca0c0, 4 },
    { 0xfd30a0c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb408a0c5, 4 },
    { 0x1904a0c5, 3 },
    { 0xc100a0c5, 3 },
    { 0xcf7ca0c6, 3 },
    { 0, 0 },
    { 0x6c18a0ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d4a0d0, 4 },
    { 0, 0 },
    { 0xcd30a0d2, 3 },
    { 0x1b90a0d3, 4 },
    { 0x7e50a0d4, 3 },
    { 0x756ca0d5, 3 },
    { 0x67ca0d6, 4 },
    { 0x8b5ca0d6, 4 },
    { 0x8a58a0d5, 3 },
    { 0x93ca0d9, 3 },
    { 0xd850a0da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf740a0e4, 3 },
    { 0xc994a0e4, 3 },
    { 0xc94ca0e6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc234a0ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d28a0f0, 5 },
    { 0, 0 },
    { 0xc80a0f2, 3 },
    { 0xaef0a0f3, 4 },
    { 0xfa58a0f3, 3 },
    { 0xbd38a0f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e10a0f8, 3 },
    { 0x405ca0f9, 4 },
    { 0xac8a0f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a0a0fd, 5 },
    { 0xdf84a0fe, 3 },
    { 0xf588a0ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b70a105, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fca10a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9430a116, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebca11a, 4 },
    { 0x7884a11a, 3 },
    { 0, 0 },
    { 0x6f28a11d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd08a122, 5 },
    { 0, 0 },
    { 0xf1a0a124, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5824a127, 3 },
    { 0x1bfca127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794a12b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f70a12f, 3 },
    { 0xf258a130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58a135, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8a138, 3 },
    { 0x5c94a139, 4 },
    { 0x579ca138, 3 },
    { 0x1774a139, 3 },
    { 0xc03ca13b, 3 },
    { 0, 0 },
    { 0x21bca13e, 4 },
    { 0x621ca13f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4a143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4a146, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d4a149, 3 },
    { 0, 0 },
    { 0x8274a14b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa038a14e, 4 },
    { 0x9a34a14f, 3 },
    { 0x5e8ca150, 3 },
    { 0x6ec0a14f, 3 },
    { 0x70a8a150, 3 },
    { 0x9944a152, 3 },
    { 0xd020a154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba18a158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c8a15d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb4a160, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a74a163, 3 },
    { 0, 0 },
    { 0x7050a165, 4 },
    { 0x8cf8a166, 3 },
    { 0x1d54a167, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0a16a, 3 },
    { 0xad14a16a, 3 },
    { 0, 0 },
    { 0x3f9ca16d, 3 },
    { 0x65eca16e, 3 },
    { 0x5870a16f, 5 },
    { 0x8e50a16e, 3 },
    { 0, 0 },
    { 0xd864a172, 3 },
    { 0x3034a173, 3 },
    { 0, 0 },
    { 0xc004a175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba74a179, 3 },
    { 0, 0 },
    { 0xc6fca17b, 3 },
    { 0x15ca17c, 3 },
    { 0x1c80a17d, 3 },
    { 0xa2b4a17e, 3 },
    { 0x1dfca17f, 4 },
    { 0x7600a17e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724a186, 3 },
    { 0xd178a186, 3 },
    { 0, 0 },
    { 0xb07ca189, 3 },
    { 0xd308a189, 3 },
    { 0x1010a18a, 3 },
    { 0xbf08a18a, 3 },
    { 0xab80a189, 3 },
    { 0xdccca18e, 3 },
    { 0xba6ca18e, 4 },
    { 0x2be4a190, 3 },
    { 0, 0 },
    { 0x9a4a192, 3 },
    { 0xb918a193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0a197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f00a19a, 4 },
    { 0x4900a19a, 4 },
    { 0x1008a19c, 5 },
    { 0xa98ca19a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8230a1a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd0a1a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e48a1a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a0a1ac, 4 },
    { 0x6850a1ad, 3 },
    { 0xb3f0a1ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7798a1b1, 3 },
    { 0xe3a4a1b2, 3 },
    { 0x8d30a1b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5364a1b7, 3 },
    { 0x1570a1b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbca1be, 3 },
    { 0xc4d0a1bf, 4 },
    { 0x4124a1c0, 3 },
    { 0x5344a1c0, 3 },
    { 0x5e48a1c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6548a1c7, 4 },
    { 0xfbe4a1c8, 5 },
    { 0, 0 },
    { 0x3a00a1ca, 3 },
    { 0x1a00a1cb, 3 },
    { 0x78fca1cc, 3 },
    { 0x68ca1cc, 3 },
    { 0x4c30a1cc, 3 },
    { 0x23f4a1cf, 3 },
    { 0, 0 },
    { 0xeaf4a1d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a0a1d5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f4a1db, 3 },
    { 0, 0 },
    { 0x2ae0a1dd, 3 },
    { 0xc528a1de, 3 },
    { 0x4394a1dd, 4 },
    { 0x45b4a1de, 3 },
    { 0xc744a1e1, 3 },
    { 0xf530a1e2, 3 },
    { 0xe58ca1e1, 5 },
    { 0x5644a1df, 4 },
    { 0, 0 },
    { 0xf330a1e6, 3 },
    { 0x3854a1e7, 4 },
    { 0x9d80a1e8, 3 },
    { 0xfa98a1e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0a1ed, 3 },
    { 0xdd1ca1ee, 3 },
    { 0xbce0a1ed, 3 },
    { 0xb2f8a1ed, 3 },
    { 0xe6b0a1ee, 3 },
    { 0xf51ca1f2, 3 },
    { 0x9db0a1f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4188a200, 3 },
    { 0xd964a201, 4 },
    { 0, 0 },
    { 0x8268a203, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf8a208, 5 },
    { 0xb21ca209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa100a216, 4 },
    { 0x76d4a217, 4 },
    { 0x3a3ca218, 3 },
    { 0x8a38a219, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e4a220, 4 },
    { 0xf4dca221, 3 },
    { 0, 0 },
    { 0x5764a223, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605ca228, 3 },
    { 0xe7b8a229, 3 },
    { 0, 0 },
    { 0xcbe8a22b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ca230, 3 },
    { 0xd9cca230, 3 },
    { 0xeb04a230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554a235, 3 },
    { 0, 0 },
    { 0xf788a237, 3 },
    { 0, 0 },
    { 0x9c50a239, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dca23d, 3 },
    { 0, 0 },
    { 0x2108a23f, 3 },
    { 0x94a4a240, 3 },
    { 0x737ca241, 4 },
    { 0xc5bca242, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474a248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceeca24e, 4 },
    { 0xa1b4a24f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe74a25b, 3 },
    { 0, 0 },
    { 0x88a8a25d, 4 },
    { 0x2900a25e, 3 },
    { 0, 0 },
    { 0xa9b0a260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2840a26c, 3 },
    { 0x6da8a26c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edca271, 4 },
    { 0xe22ca272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3644a27b, 3 },
    { 0xbff4a27b, 3 },
    { 0x3c70a27c, 3 },
    { 0x1608a27e, 3 },
    { 0xc77ca27f, 3 },
    { 0, 0 },
    { 0x7baca281, 4 },
    { 0x3fb0a282, 3 },
    { 0x8a74a283, 3 },
    { 0x2134a281, 4 },
    { 0xd534a285, 3 },
    { 0, 0 },
    { 0x1c18a287, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d4a28a, 4 },
    { 0x7b9ca28b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f4a28f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7708a294, 3 },
    { 0, 0 },
    { 0x2088a296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbca299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdca29c, 4 },
    { 0xa6a8a29d, 3 },
    { 0x71fca29d, 4 },
    { 0, 0 },
    { 0x3094a2a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1768a2a6, 3 },
    { 0x9ec0a2a6, 4 },
    { 0x3efca2a8, 3 },
    { 0x6e30a2a9, 4 },
    { 0x3ce0a2aa, 4 },
    { 0xb160a2ab, 3 },
    { 0, 0 },
    { 0x1d4a2ad, 3 },
    { 0xdfe0a2ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2884a2b3, 3 },
    { 0x90a4a2b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa44a2b9, 5 },
    { 0, 0 },
    { 0x7df8a2bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8608a2bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26ca2c3, 3 },
    { 0x8deca2c4, 3 },
    { 0x7940a2c4, 3 },
    { 0, 0 },
    { 0xe8d8a2c7, 4 },
    { 0x76cca2c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20a2cb, 3 },
    { 0, 0 },
    { 0x7864a2cd, 4 },
    { 0xd884a2ce, 3 },
    { 0x338ca2cf, 3 },
    { 0x9124a2ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c0a2d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe940a2e2, 3 },
    { 0x5af4a2e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2144a2e7, 3 },
    { 0x714ca2e7, 3 },
    { 0x63d4a2e7, 3 },
    { 0xfee0a2ea, 3 },
    { 0x6ce4a2eb, 4 },
    { 0x878a2e7, 4 },
    { 0x444ca2ed, 3 },
    { 0x5da4a2ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9484a2f3, 3 },
    { 0, 0 },
    { 0x77f8a2f5, 3 },
    { 0, 0 },
    { 0x40fca2f7, 3 },
    { 0x65c4a2f8, 3 },
    { 0x9c64a2f9, 3 },
    { 0, 0 },
    { 0x90f8a2fb, 3 },
    { 0xa51ca2fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604a300, 3 },
    { 0xc188a300, 3 },
    { 0, 0 },
    { 0x3968a303, 3 },
    { 0xbef4a304, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8a307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe214a316, 4 },
    { 0x9c30a316, 3 },
    { 0xd034a318, 3 },
    { 0x2b4ca318, 4 },
    { 0xa224a31a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d4a321, 3 },
    { 0, 0 },
    { 0x99fca323, 4 },
    { 0x8e80a324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bca32c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcca330, 3 },
    { 0x5ed4a331, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f98a336, 3 },
    { 0, 0 },
    { 0xc418a338, 3 },
    { 0x2f08a339, 4 },
    { 0x7114a33a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b58a341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cca344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c80a34b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeca351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8a355, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f58a358, 4 },
    { 0x388a359, 3 },
    { 0x3f64a35a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbca35e, 3 },
    { 0, 0 },
    { 0x1ab4a360, 3 },
    { 0x236ca361, 4 },
    { 0x4688a360, 3 },
    { 0xd694a360, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e8a36c, 3 },
    { 0xf9d8a36d, 4 },
    { 0, 0 },
    { 0x2d9ca36f, 3 },
    { 0, 0 },
    { 0x7c04a371, 3 },
    { 0x1e9ca372, 3 },
    { 0x7a0ca372, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6658a37a, 3 },
    { 0, 0 },
    { 0x82e8a37c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aca380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eca383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a0a387, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034a38a, 3 },
    { 0, 0 },
    { 0x9368a38c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6038a390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405ca394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90ca39a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd4a39e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaca3a3, 4 },
    { 0, 0 },
    { 0xb4b8a3a5, 3 },
    { 0x2b14a3a5, 3 },
    { 0xf1a8a3a7, 3 },
    { 0xfeb0a3a5, 3 },
    { 0xbb70a3a9, 3 },
    { 0xef20a3aa, 3 },
    { 0, 0 },
    { 0x5ce0a3ac, 3 },
    { 0x6e20a3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8678a3b1, 3 },
    { 0x338ca3b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc338a3b6, 4 },
    { 0x5438a3b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f8a3be, 3 },
    { 0, 0 },
    { 0xb400a3c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7584a3c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x970a3c6, 4 },
    { 0, 0 },
    { 0xf9f0a3c8, 3 },
    { 0x6cdca3c8, 3 },
    { 0x79a0a3c8, 3 },
    { 0, 0 },
    { 0x3d5ca3cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dca3cf, 3 },
    { 0x2858a3d0, 3 },
    { 0, 0 },
    { 0x7660a3d2, 3 },
    { 0, 0 },
    { 0xd028a3d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f8a3dd, 3 },
    { 0xcd74a3de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c58a3e1, 3 },
    { 0x51a8a3e2, 3 },
    { 0xb2ca3e2, 3 },
    { 0x2480a3e4, 3 },
    { 0x942ca3e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd4a3eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d0a3ee, 3 },
    { 0x75fca3ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99eca3f7, 3 },
    { 0, 0 },
    { 0x3b4ca3f9, 3 },
    { 0x1320a3fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fca3fe, 3 },
    { 0xf6c0a3ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8060a402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c0a408, 3 },
    { 0, 0 },
    { 0x5e2ca40a, 3 },
    { 0x3638a40b, 3 },
    { 0x65e4a40b, 3 },
    { 0xd708a40c, 3 },
    { 0x1190a40a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x803ca411, 3 },
    { 0, 0 },
    { 0x7bb4a413, 4 },
    { 0, 0 },
    { 0xd014a415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd0a420, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc0a424, 3 },
    { 0xdd58a425, 4 },
    { 0xa904a425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a0a42c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3790a42f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc74a433, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740a437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce4a43c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad48a442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c4a448, 3 },
    { 0x6b98a449, 4 },
    { 0, 0 },
    { 0x9fcca44b, 3 },
    { 0xcaf4a44c, 3 },
    { 0xd478a44d, 3 },
    { 0x4b34a44d, 3 },
    { 0x33eca44e, 3 },
    { 0xa8b8a44c, 4 },
    { 0x1390a44c, 3 },
    { 0xa018a452, 3 },
    { 0x2f10a453, 3 },
    { 0x2368a453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ca459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a58a45c, 3 },
    { 0, 0 },
    { 0xcdf0a45e, 3 },
    { 0, 0 },
    { 0x2e14a460, 3 },
    { 0x3490a460, 3 },
    { 0, 0 },
    { 0xdf64a463, 3 },
    { 0x3eb4a464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4200a467, 3 },
    { 0xa63ca468, 3 },
    { 0, 0 },
    { 0x4320a46a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adca470, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6934a473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbca478, 3 },
    { 0xcc4ca478, 3 },
    { 0, 0 },
    { 0xbe20a47b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8a480, 3 },
    { 0xcad0a480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5798a489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd8a48d, 3 },
    { 0x7c94a48e, 3 },
    { 0x86fca48f, 4 },
    { 0x172ca490, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d70a494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034a49c, 4 },
    { 0x974ca49d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468a4a1, 4 },
    { 0x1dc4a4a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f4a4a5, 5 },
    { 0x6730a4a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8a4a9, 3 },
    { 0xdc08a4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2100a4ad, 3 },
    { 0xb8f4a4ae, 4 },
    { 0x8baca4af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dca4b3, 3 },
    { 0xdb08a4b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7004a4b7, 4 },
    { 0x61bca4b8, 3 },
    { 0, 0 },
    { 0x2ad4a4ba, 4 },
    { 0, 0 },
    { 0xc260a4bc, 5 },
    { 0, 0 },
    { 0xb850a4be, 3 },
    { 0x5068a4bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c4a4c6, 3 },
    { 0x5524a4c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab4a4cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2878a4d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcca4d7, 3 },
    { 0, 0 },
    { 0x7c0ca4d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f8a4df, 3 },
    { 0, 0 },
    { 0xb540a4e1, 3 },
    { 0xef3ca4e2, 4 },
    { 0, 0 },
    { 0x889ca4e4, 3 },
    { 0x6950a4e5, 3 },
    { 0x7fd4a4e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc360a4e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858ca4ed, 3 },
    { 0, 0 },
    { 0xebd4a4ef, 3 },
    { 0xc22ca4f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4a4f3, 3 },
    { 0, 0 },
    { 0xd3c8a4f5, 3 },
    { 0, 0 },
    { 0xe224a4f7, 3 },
    { 0x6420a4f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa8a4fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aca501, 3 },
    { 0x8740a502, 3 },
    { 0, 0 },
    { 0xe458a504, 3 },
    { 0x41cca504, 3 },
    { 0x7340a505, 3 },
    { 0xc034a507, 3 },
    { 0, 0 },
    { 0x5648a509, 3 },
    { 0x69eca509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd998a50d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf98a515, 3 },
    { 0x618ca516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee30a519, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1504a51c, 3 },
    { 0, 0 },
    { 0xd698a51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948a521, 3 },
    { 0, 0 },
    { 0x9604a523, 3 },
    { 0x88eca524, 3 },
    { 0xc4ca525, 3 },
    { 0xa150a524, 3 },
    { 0, 0 },
    { 0x847ca528, 4 },
    { 0xb000a529, 3 },
    { 0, 0 },
    { 0x1a1ca52b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3444a533, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de0a537, 3 },
    { 0x58b8a537, 3 },
    { 0x99b0a538, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8214a540, 3 },
    { 0xb050a541, 4 },
    { 0, 0 },
    { 0xf7b0a543, 3 },
    { 0x6f54a544, 3 },
    { 0xa470a545, 3 },
    { 0x7834a545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe290a54d, 4 },
    { 0xa660a54e, 4 },
    { 0, 0 },
    { 0x90a0a550, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b0a554, 4 },
    { 0, 0 },
    { 0x32a8a556, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290a559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112ca55d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030a562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b4a567, 4 },
    { 0x7bd4a568, 3 },
    { 0xdadca569, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cca56e, 3 },
    { 0x8be0a56e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aca575, 3 },
    { 0x4f94a576, 4 },
    { 0x5ad8a576, 3 },
    { 0xcb9ca576, 3 },
    { 0, 0 },
    { 0x75e0a57a, 4 },
    { 0, 0 },
    { 0x2f6ca57c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f8a580, 3 },
    { 0x1870a580, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7510a585, 4 },
    { 0, 0 },
    { 0xb674a587, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ca593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2930a59a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b8a59e, 3 },
    { 0xb694a59f, 3 },
    { 0x33eca59e, 3 },
    { 0x56f8a59f, 3 },
    { 0, 0 },
    { 0x9b04a5a3, 3 },
    { 0, 0 },
    { 0xf924a5a5, 4 },
    { 0, 0 },
    { 0xfe10a5a7, 3 },
    { 0x3ad8a5a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x501ca5ab, 4 },
    { 0xe164a5ac, 3 },
    { 0x3828a5ad, 3 },
    { 0xd8e0a5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bca5b3, 3 },
    { 0, 0 },
    { 0xbde8a5b5, 4 },
    { 0x5344a5b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb854a5bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bca5c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d8a5c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6524a5ca, 3 },
    { 0x5098a5cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e8a5cf, 3 },
    { 0x7488a5d0, 3 },
    { 0, 0 },
    { 0x9264a5d2, 4 },
    { 0xfecca5d3, 3 },
    { 0, 0 },
    { 0x7fa8a5d5, 3 },
    { 0, 0 },
    { 0xe558a5d7, 3 },
    { 0x1f5ca5d7, 4 },
    { 0x7078a5d9, 4 },
    { 0, 0 },
    { 0xd4a4a5db, 3 },
    { 0x1efca5dc, 3 },
    { 0x40f8a5db, 3 },
    { 0xa1dca5de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a5e2, 3 },
    { 0x1448a5e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8a5e8, 3 },
    { 0xc08ca5e9, 3 },
    { 0x732ca5e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418a5f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bca5f4, 4 },
    { 0, 0 },
    { 0xf89ca5f6, 3 },
    { 0x2808a5f7, 3 },
    { 0x63e8a5f7, 3 },
    { 0x751ca5f9, 3 },
    { 0x4048a5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae00a605, 3 },
    { 0x5cb8a606, 3 },
    { 0, 0 },
    { 0x553ca608, 4 },
    { 0, 0 },
    { 0x727ca60a, 4 },
    { 0x7994a60b, 3 },
    { 0, 0 },
    { 0x7ceca60d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc584a613, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568a617, 3 },
    { 0x6e2ca618, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a61b, 4 },
    { 0, 0 },
    { 0xb9f4a61d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c44a624, 3 },
    { 0x4e60a624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a0a628, 3 },
    { 0x8250a628, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa010a630, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a78a636, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ca639, 3 },
    { 0xd130a639, 3 },
    { 0, 0 },
    { 0x625ca63c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb40a642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd8a64c, 3 },
    { 0xd5c0a64c, 3 },
    { 0x48bca64d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3060a652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ca657, 3 },
    { 0, 0 },
    { 0xf948a659, 3 },
    { 0x6434a65a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf500a660, 3 },
    { 0, 0 },
    { 0xc380a662, 3 },
    { 0, 0 },
    { 0xb68a664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9ca668, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928a66e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030a673, 4 },
    { 0xc1c8a673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc270a677, 3 },
    { 0x1ff4a678, 3 },
    { 0, 0 },
    { 0x5ee0a67a, 3 },
    { 0x8afca67b, 3 },
    { 0, 0 },
    { 0x73aca67d, 3 },
    { 0, 0 },
    { 0xf7e8a67f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe438a682, 4 },
    { 0x4060a682, 3 },
    { 0, 0 },
    { 0x56cca685, 3 },
    { 0x94c0a686, 3 },
    { 0x8c10a687, 3 },
    { 0x6c00a686, 3 },
    { 0xc694a689, 3 },
    { 0x72e4a68a, 3 },
    { 0x520ca68b, 3 },
    { 0xc994a68b, 3 },
    { 0x6774a68b, 3 },
    { 0x94aca68e, 3 },
    { 0, 0 },
    { 0x474a690, 3 },
    { 0, 0 },
    { 0x4be8a692, 4 },
    { 0x3714a693, 4 },
    { 0x7b18a693, 3 },
    { 0x2a4a695, 3 },
    { 0, 0 },
    { 0x3360a697, 3 },
    { 0x5604a698, 4 },
    { 0x533ca699, 3 },
    { 0xa4f8a698, 4 },
    { 0x3960a69a, 3 },
    { 0xf94a69c, 3 },
    { 0x256ca69d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca6a0, 3 },
    { 0x6384a6a1, 3 },
    { 0x3e2ca6a0, 4 },
    { 0, 0 },
    { 0x5344a6a4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04ca6a8, 3 },
    { 0xf480a6a8, 3 },
    { 0x2e6ca6aa, 3 },
    { 0x820a6ab, 3 },
    { 0xf1f4a6ac, 3 },
    { 0x30a8a6ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6748a6b3, 4 },
    { 0, 0 },
    { 0x85cca6b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aeca6b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9510a6bc, 4 },
    { 0x5ec4a6bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2ca6c1, 3 },
    { 0x6f5ca6c1, 3 },
    { 0, 0 },
    { 0x851ca6c4, 4 },
    { 0, 0 },
    { 0xb504a6c6, 3 },
    { 0x2c80a6c7, 4 },
    { 0x3c00a6c7, 3 },
    { 0xf304a6c7, 3 },
    { 0x5828a6ca, 3 },
    { 0x3e58a6cb, 3 },
    { 0x29cca6cc, 5 },
    { 0x4678a6c8, 4 },
    { 0xf490a6ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b0a6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8a6dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8ca6e0, 3 },
    { 0xc34ca6e1, 4 },
    { 0x2768a6e2, 3 },
    { 0, 0 },
    { 0xe110a6e4, 3 },
    { 0, 0 },
    { 0x9044a6e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3074a6e9, 3 },
    { 0x9658a6ea, 3 },
    { 0x50e4a6e9, 4 },
    { 0x8974a6ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6130a6f1, 3 },
    { 0, 0 },
    { 0x2caca6f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8778a6f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaca6f9, 3 },
    { 0xf258a6fa, 3 },
    { 0x38a0a6fb, 3 },
    { 0x63f0a6fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9ca701, 4 },
    { 0xc50ca702, 3 },
    { 0xfbe4a703, 3 },
    { 0xd75ca704, 3 },
    { 0, 0 },
    { 0x1250a706, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330a70a, 3 },
    { 0x6b8ca70b, 3 },
    { 0x5c1ca70c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850a711, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde48a714, 3 },
    { 0x6bd8a715, 4 },
    { 0x3b08a714, 3 },
    { 0x8874a717, 3 },
    { 0x9010a717, 4 },
    { 0, 0 },
    { 0x5410a71a, 3 },
    { 0xf2b8a71b, 4 },
    { 0, 0 },
    { 0xe8dca71d, 3 },
    { 0xedc8a71e, 4 },
    { 0x91aca71e, 3 },
    { 0xc314a71f, 4 },
    { 0x3fd8a71f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd244a725, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970a729, 4 },
    { 0x5f64a72a, 3 },
    { 0x7674a72a, 3 },
    { 0, 0 },
    { 0x2cfca72d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00ca731, 3 },
    { 0xdd24a731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f68a736, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0a739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3334a73d, 4 },
    { 0xdb78a73e, 3 },
    { 0x6d28a73e, 3 },
    { 0, 0 },
    { 0x8faca741, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728a744, 3 },
    { 0xa5b4a745, 4 },
    { 0xbb68a745, 3 },
    { 0x1b90a747, 3 },
    { 0x8fbca747, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82ca74c, 3 },
    { 0x49d8a74d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1328a751, 3 },
    { 0xe538a751, 3 },
    { 0, 0 },
    { 0x5650a754, 3 },
    { 0x460a754, 3 },
    { 0xf288a756, 3 },
    { 0x5698a757, 3 },
    { 0x8d20a757, 4 },
    { 0x54bca759, 3 },
    { 0x46fca759, 4 },
    { 0x293ca75b, 3 },
    { 0x2e88a75c, 5 },
    { 0x1b34a756, 3 },
    { 0xe8f0a75e, 3 },
    { 0x7a1ca75f, 4 },
    { 0x31dca75f, 3 },
    { 0xe6b4a75e, 5 },
    { 0, 0 },
    { 0xf21ca763, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a0a766, 5 },
    { 0, 0 },
    { 0xa594a768, 4 },
    { 0xb37ca769, 3 },
    { 0x4f44a76a, 3 },
    { 0xba7ca76a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df4a772, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d8a776, 3 },
    { 0xcd1ca777, 3 },
    { 0x6970a778, 4 },
    { 0xefd8a776, 3 },
    { 0x46d0a778, 3 },
    { 0, 0 },
    { 0x7b08a77c, 3 },
    { 0xea20a77c, 3 },
    { 0x8b48a77e, 4 },
    { 0xde24a77f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa80a789, 3 },
    { 0, 0 },
    { 0x5c4ca78b, 4 },
    { 0, 0 },
    { 0x2868a78d, 3 },
    { 0xea4a78e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf494a793, 3 },
    { 0x9f98a794, 3 },
    { 0xeac0a795, 3 },
    { 0x7bbca794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1804a799, 3 },
    { 0x9de8a799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bca79f, 4 },
    { 0xe2d0a79f, 3 },
    { 0x51e8a7a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d50a7a5, 4 },
    { 0x7e98a7a6, 3 },
    { 0xd800a7a6, 3 },
    { 0xbe14a7a5, 3 },
    { 0x30a7a9, 3 },
    { 0x2318a7aa, 4 },
    { 0, 0 },
    { 0xc86ca7ac, 3 },
    { 0x9838a7ad, 3 },
    { 0x472ca7ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b28a7b1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc564a7b8, 4 },
    { 0x67c8a7b8, 4 },
    { 0x69e8a7b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3304a7bd, 3 },
    { 0xede8a7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d8a7c1, 3 },
    { 0x4bd0a7c1, 3 },
    { 0x2e8ca7c1, 3 },
    { 0x894ca7c2, 4 },
    { 0xaa84a7c5, 3 },
    { 0x5348a7c6, 3 },
    { 0xb784a7c7, 3 },
    { 0x3c1ca7c5, 3 },
    { 0x32f4a7c9, 3 },
    { 0xb62ca7c9, 4 },
    { 0x8cb8a7cb, 3 },
    { 0x722ca7c9, 3 },
    { 0x8c50a7c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128a7d1, 3 },
    { 0, 0 },
    { 0x46aca7d3, 3 },
    { 0, 0 },
    { 0xe91ca7d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4a7d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa058a7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3ca7df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1dca7e4, 4 },
    { 0xaeca7e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f64a7ed, 4 },
    { 0, 0 },
    { 0x6cd8a7ef, 4 },
    { 0x9600a7f0, 3 },
    { 0, 0 },
    { 0xcb0a7f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7780a7f8, 4 },
    { 0x3548a7f9, 3 },
    { 0, 0 },
    { 0xd89ca7fb, 4 },
    { 0, 0 },
    { 0x1bfca7fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8684a806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454a809, 4 },
    { 0xd27ca80a, 3 },
    { 0xb364a80b, 4 },
    { 0, 0 },
    { 0x8e08a80d, 3 },
    { 0x4338a80e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edca815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bca818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a8a81b, 3 },
    { 0x732ca81b, 3 },
    { 0x2b14a81d, 3 },
    { 0, 0 },
    { 0x1ebca81f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40a824, 4 },
    { 0xfbdca824, 3 },
    { 0, 0 },
    { 0x1e30a827, 3 },
    { 0xad3ca828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090a82b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e00a82e, 3 },
    { 0, 0 },
    { 0xeae0a830, 3 },
    { 0x894a830, 3 },
    { 0, 0 },
    { 0xf988a833, 3 },
    { 0xff14a834, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8a837, 3 },
    { 0xa0f8a837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc0a83c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe160a840, 4 },
    { 0xd0f4a841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fca847, 4 },
    { 0xb6b4a847, 3 },
    { 0xbef0a849, 3 },
    { 0x2608a84a, 4 },
    { 0xee0a849, 3 },
    { 0x3f0a84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4060a84f, 3 },
    { 0x8900a84f, 3 },
    { 0xe120a851, 3 },
    { 0x4cb0a852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff4a855, 3 },
    { 0, 0 },
    { 0xf8a857, 3 },
    { 0x7c04a858, 3 },
    { 0x1b80a858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x596ca85c, 3 },
    { 0x1474a85d, 3 },
    { 0xf254a85d, 3 },
    { 0, 0 },
    { 0x8cc8a860, 3 },
    { 0xc360a861, 3 },
    { 0xd758a861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d0a866, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d4a869, 4 },
    { 0x13b4a86a, 3 },
    { 0x6eb0a86a, 3 },
    { 0x7114a86c, 3 },
    { 0x2d2ca86c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe584a870, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6188a875, 3 },
    { 0, 0 },
    { 0xd034a877, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4ca884, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e58a888, 3 },
    { 0x884ca888, 3 },
    { 0x82f0a88a, 3 },
    { 0x52e0a88a, 3 },
    { 0x64a8a888, 4 },
    { 0xafaca88d, 4 },
    { 0xb0a0a88e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd670a893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60a898, 4 },
    { 0xed28a899, 3 },
    { 0x39b4a89a, 3 },
    { 0x7058a89b, 3 },
    { 0x78c0a89a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6190a89f, 3 },
    { 0x1564a8a0, 4 },
    { 0x2354a8a1, 3 },
    { 0, 0 },
    { 0x52b8a8a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9548a8a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd0a8ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7018a8b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8574a8b4, 5 },
    { 0xf3bca8b4, 3 },
    { 0x29d4a8b6, 4 },
    { 0x4630a8b7, 3 },
    { 0x7998a8b6, 3 },
    { 0, 0 },
    { 0xa8aca8ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e84a8c3, 4 },
    { 0x9b30a8c3, 3 },
    { 0xc67ca8c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbca8cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x909ca8cf, 4 },
    { 0xd9a4a8d0, 3 },
    { 0x40f0a8cf, 4 },
    { 0xa7cca8d1, 3 },
    { 0x61bca8d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb44a8d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a8e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x692ca8e4, 3 },
    { 0x2be8a8e5, 5 },
    { 0x19aca8e4, 3 },
    { 0x7198a8e6, 4 },
    { 0x77f4a8e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa318a8ec, 3 },
    { 0x43d0a8ec, 3 },
    { 0x37c0a8ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540a8f1, 3 },
    { 0, 0 },
    { 0x22a4a8f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28ca8fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8108a8fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4a900, 3 },
    { 0x9a0a901, 3 },
    { 0x6bb0a902, 4 },
    { 0, 0 },
    { 0x2038a904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4ca90a, 3 },
    { 0, 0 },
    { 0x4144a90c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4a911, 3 },
    { 0, 0 },
    { 0x915ca913, 3 },
    { 0xf034a914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71ca917, 3 },
    { 0x5950a917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3038a91d, 3 },
    { 0xde68a91e, 3 },
    { 0, 0 },
    { 0xdd94a920, 3 },
    { 0x93c4a920, 3 },
    { 0, 0 },
    { 0xacd4a923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815ca92c, 3 },
    { 0, 0 },
    { 0x5b58a92e, 3 },
    { 0, 0 },
    { 0xd638a930, 4 },
    { 0, 0 },
    { 0x15b8a932, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39aca937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f4a93e, 4 },
    { 0, 0 },
    { 0x3a48a940, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee40a945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc64a948, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ca94c, 3 },
    { 0xc66ca94c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534a952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ca956, 4 },
    { 0x3d50a957, 4 },
    { 0x4998a958, 4 },
    { 0, 0 },
    { 0xc748a95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8a95d, 3 },
    { 0x7960a95e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b0a964, 3 },
    { 0, 0 },
    { 0x4ef0a966, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2ca969, 3 },
    { 0, 0 },
    { 0x657ca96b, 3 },
    { 0x2074a96b, 3 },
    { 0x3bd8a96d, 4 },
    { 0xeca8a96e, 3 },
    { 0xd0d8a96f, 3 },
    { 0x8518a96f, 3 },
    { 0x3740a970, 3 },
    { 0xe04ca972, 4 },
    { 0xb104a973, 3 },
    { 0xc7c8a973, 4 },
    { 0x9518a974, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca978, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165ca97e, 4 },
    { 0x8710a97e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b50a985, 4 },
    { 0xaac4a985, 4 },
    { 0x2960a987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7ca98a, 3 },
    { 0x62e8a98b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ca991, 4 },
    { 0xa76ca991, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa08a995, 3 },
    { 0x9370a996, 4 },
    { 0xd764a996, 3 },
    { 0x9a28a996, 3 },
    { 0xc28a999, 3 },
    { 0x8f7ca999, 3 },
    { 0x1b00a999, 3 },
    { 0x6428a999, 3 },
    { 0, 0 },
    { 0x9168a99e, 4 },
    { 0, 0 },
    { 0x2264a9a0, 5 },
    { 0x3734a9a1, 4 },
    { 0xa640a9a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x387ca9a5, 3 },
    { 0, 0 },
    { 0xf66ca9a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fca9aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8a9af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4760a9b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a54a9b5, 3 },
    { 0xc3d0a9b5, 4 },
    { 0x6a08a9b7, 3 },
    { 0, 0 },
    { 0x10d4a9b9, 3 },
    { 0x4460a9ba, 5 },
    { 0, 0 },
    { 0x82bca9bc, 3 },
    { 0x2688a9bd, 3 },
    { 0xce54a9be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad0a9c4, 3 },
    { 0x4638a9c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5884a9cf, 3 },
    { 0, 0 },
    { 0x8448a9d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf204a9d7, 3 },
    { 0x7730a9d7, 3 },
    { 0xd3bca9d8, 3 },
    { 0x1170a9d8, 3 },
    { 0xc324a9da, 4 },
    { 0x5b48a9d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1564a9e0, 3 },
    { 0, 0 },
    { 0x1778a9e2, 3 },
    { 0x582ca9e2, 4 },
    { 0xe810a9e4, 4 },
    { 0x1d74a9e2, 3 },
    { 0xad88a9e6, 3 },
    { 0x3bdca9e7, 3 },
    { 0x1210a9e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207ca9ec, 3 },
    { 0x2970a9ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4aca9f1, 4 },
    { 0x7008a9f2, 3 },
    { 0, 0 },
    { 0xcc64a9f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4a9f7, 3 },
    { 0x294a9f8, 3 },
    { 0xc568a9f7, 3 },
    { 0xbdd4a9fa, 3 },
    { 0xdc34a9fa, 3 },
    { 0, 0 },
    { 0x7a10a9fd, 3 },
    { 0xa0c8a9fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8918aa01, 3 },
    { 0xb99caa02, 3 },
    { 0, 0 },
    { 0x7ca0aa04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x500aa07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1584aa0b, 3 },
    { 0xd85caa0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf638aa0f, 3 },
    { 0x37e0aa10, 3 },
    { 0x5340aa0f, 3 },
    { 0x9410aa12, 3 },
    { 0x69caa12, 3 },
    { 0x4f88aa14, 3 },
    { 0x8e9caa15, 4 },
    { 0x7a68aa14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0aa1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ecaa1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c4aa23, 3 },
    { 0x1ecaa24, 5 },
    { 0x68d8aa24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6caa2d, 3 },
    { 0, 0 },
    { 0xfe4aa2f, 5 },
    { 0xf74aa30, 3 },
    { 0x4264aa30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c0aa35, 3 },
    { 0, 0 },
    { 0x1420aa37, 3 },
    { 0xc8b4aa38, 3 },
    { 0x45caa39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe250aa3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462caa42, 3 },
    { 0x76d4aa43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bcaa46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f14aa49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fcaa4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190aa50, 3 },
    { 0x7cccaa51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6caa55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440aa59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe600aa5d, 3 },
    { 0xffc8aa5d, 3 },
    { 0x6e58aa5d, 3 },
    { 0, 0 },
    { 0x9b90aa61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe88aa66, 4 },
    { 0, 0 },
    { 0xa1d4aa68, 3 },
    { 0xa148aa69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa810aa6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8aa6f, 3 },
    { 0xc630aa70, 3 },
    { 0xde58aa70, 3 },
    { 0, 0 },
    { 0xca3caa73, 4 },
    { 0xdd14aa74, 3 },
    { 0xe478aa73, 3 },
    { 0xbc28aa76, 3 },
    { 0x5f94aa77, 3 },
    { 0x7864aa74, 4 },
    { 0x3dd8aa76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7caa7c, 3 },
    { 0, 0 },
    { 0x4e28aa7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e78aa89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf8aa8d, 3 },
    { 0, 0 },
    { 0x47fcaa8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3caa97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4caaa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34aaa3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0aaad, 4 },
    { 0x6400aaae, 3 },
    { 0, 0 },
    { 0xa370aab0, 3 },
    { 0, 0 },
    { 0xf750aab2, 4 },
    { 0x871caab3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640aab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e78aac2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f0aac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea0aacd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c8aad0, 3 },
    { 0xbd40aad1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b8aad5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb8aad9, 3 },
    { 0xb8fcaada, 3 },
    { 0xbe4aadb, 3 },
    { 0x1c1caadc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7314aae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6128aae8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924aaeb, 4 },
    { 0x99caaec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d84aaf1, 3 },
    { 0x75f4aaf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1caaf5, 3 },
    { 0xd1ccaaf5, 3 },
    { 0x5df0aaf6, 3 },
    { 0x83e4aaf8, 3 },
    { 0xce64aaf8, 3 },
    { 0xee28aaf9, 4 },
    { 0xf27caafb, 3 },
    { 0x4b70aafc, 3 },
    { 0xe098aafb, 3 },
    { 0x5c98aaf7, 3 },
    { 0x353caaff, 3 },
    { 0x7e28aafb, 3 },
    { 0xcfacaafe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13cab05, 4 },
    { 0xba30ab06, 4 },
    { 0xe074ab06, 4 },
    { 0xcd98ab06, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e74ab0e, 3 },
    { 0x4c4ab0e, 4 },
    { 0x7900ab0f, 3 },
    { 0x6a6cab11, 3 },
    { 0xde7cab0f, 4 },
    { 0x78fcab13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff0ab29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161cab2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf690ab31, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f8ab35, 4 },
    { 0xed00ab36, 3 },
    { 0, 0 },
    { 0xbb1cab38, 4 },
    { 0, 0 },
    { 0x80dcab3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc014ab3e, 3 },
    { 0x7b7cab3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608ab44, 3 },
    { 0x770ab44, 4 },
    { 0x8dfcab46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9cab4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd84ab4f, 3 },
    { 0xb4e4ab50, 3 },
    { 0x4a4cab50, 3 },
    { 0x5e50ab50, 3 },
    { 0x26d4ab4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ecab57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8cab5a, 3 },
    { 0, 0 },
    { 0xd504ab5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2194ab5f, 3 },
    { 0xd2e4ab60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4ab68, 4 },
    { 0x66fcab69, 3 },
    { 0, 0 },
    { 0x3d40ab6b, 3 },
    { 0x7ecab6b, 4 },
    { 0x15f4ab6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98ab70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cab74, 3 },
    { 0xc35cab75, 3 },
    { 0, 0 },
    { 0x6144ab77, 3 },
    { 0, 0 },
    { 0xd16cab79, 4 },
    { 0x1d14ab7a, 3 },
    { 0x44c0ab7b, 3 },
    { 0x11ecab7c, 5 },
    { 0x3de8ab7d, 3 },
    { 0x9dcab7e, 3 },
    { 0x5b48ab7f, 3 },
    { 0x508cab80, 3 },
    { 0x55c8ab7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fcab84, 3 },
    { 0x2830ab85, 4 },
    { 0x5910ab85, 3 },
    { 0, 0 },
    { 0xf768ab88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c4ab91, 4 },
    { 0xa818ab91, 3 },
    { 0xcad0ab93, 3 },
    { 0x45bcab94, 3 },
    { 0x4924ab94, 4 },
    { 0x7550ab92, 3 },
    { 0x3c90ab93, 3 },
    { 0xf38cab94, 3 },
    { 0xc0ccab99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb170aba2, 4 },
    { 0, 0 },
    { 0xbc74aba4, 3 },
    { 0, 0 },
    { 0x2848aba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa908abaa, 3 },
    { 0x36e8abab, 3 },
    { 0xf0dcabac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d64abb1, 3 },
    { 0x654cabb1, 3 },
    { 0x5640abb2, 3 },
    { 0xea8abb4, 3 },
    { 0xbf80abb1, 4 },
    { 0x4ce8abb6, 3 },
    { 0xca40abb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4abbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa604abc9, 3 },
    { 0xbca0abca, 5 },
    { 0x37f0abcb, 3 },
    { 0x6e64abca, 4 },
    { 0x868abcd, 3 },
    { 0x1108abcd, 3 },
    { 0xaed8abca, 3 },
    { 0x9b38abd0, 3 },
    { 0x8750abcf, 3 },
    { 0x8824abcc, 4 },
    { 0, 0 },
    { 0xde4abd4, 4 },
    { 0xdb60abd4, 4 },
    { 0xf14abd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a44abe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf0abe8, 3 },
    { 0x63f8abe8, 4 },
    { 0x5918abe8, 4 },
    { 0, 0 },
    { 0x7a20abec, 4 },
    { 0, 0 },
    { 0x78b4abee, 3 },
    { 0xef54abef, 3 },
    { 0x643cabee, 3 },
    { 0x62e4abef, 3 },
    { 0xa4d4abf2, 3 },
    { 0x7130abf3, 3 },
    { 0x7c6cabf4, 4 },
    { 0xe88abf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6068abfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf488abff, 3 },
    { 0, 0 },
    { 0xfc78ac01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb8ac09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63cac0c, 3 },
    { 0x6038ac0c, 4 },
    { 0xf7e8ac0c, 3 },
    { 0x998cac0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119cac17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1774ac1b, 3 },
    { 0xf370ac1c, 3 },
    { 0, 0 },
    { 0xabc0ac1e, 3 },
    { 0x6444ac1f, 4 },
    { 0x8e0ac1e, 3 },
    { 0xdaa8ac21, 3 },
    { 0x1d84ac21, 3 },
    { 0x4ef8ac21, 3 },
    { 0x31e0ac24, 3 },
    { 0xcae8ac20, 3 },
    { 0xcef8ac26, 3 },
    { 0xe85cac27, 3 },
    { 0x19bcac28, 4 },
    { 0x11f8ac28, 3 },
    { 0xd760ac2a, 3 },
    { 0, 0 },
    { 0x328ac2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2814ac2f, 3 },
    { 0xc498ac30, 4 },
    { 0x10b0ac30, 3 },
    { 0xcaecac32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaccac37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddcac3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74cac47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f8ac4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e18ac50, 3 },
    { 0, 0 },
    { 0x49b0ac52, 3 },
    { 0xdad0ac53, 3 },
    { 0, 0 },
    { 0xad08ac55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6cac5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3248ac64, 3 },
    { 0, 0 },
    { 0xb394ac66, 3 },
    { 0x9124ac67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dcac6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43cac6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf4ac7a, 3 },
    { 0x60b0ac7b, 3 },
    { 0x229cac7c, 3 },
    { 0xb390ac7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c8ac83, 3 },
    { 0xa524ac83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d8ac87, 3 },
    { 0, 0 },
    { 0x7834ac89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d0ac8f, 3 },
    { 0, 0 },
    { 0xbbf0ac91, 3 },
    { 0x5cb0ac92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1280ac95, 4 },
    { 0x4df8ac96, 3 },
    { 0x9ae0ac97, 3 },
    { 0x4f64ac98, 4 },
    { 0xd7e0ac98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a84ac9f, 3 },
    { 0xb034aca0, 3 },
    { 0, 0 },
    { 0x8364aca2, 3 },
    { 0x8bd8aca3, 4 },
    { 0x6ef4aca4, 3 },
    { 0xef84aca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a8aca8, 3 },
    { 0xd1e0aca9, 3 },
    { 0x2f78aca8, 4 },
    { 0, 0 },
    { 0xc4f4acac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443cacb0, 3 },
    { 0xfa08acb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2184acb7, 3 },
    { 0, 0 },
    { 0xc854acb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8acbd, 3 },
    { 0x8900acbe, 3 },
    { 0x2ef8acbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f0acc8, 3 },
    { 0x2768acc9, 3 },
    { 0xec44acca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7770acce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c8acd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda58acd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1cacdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea00acea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf60acf2, 3 },
    { 0x4c6cacf3, 3 },
    { 0x3e7cacf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bcacff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf834ad04, 5 },
    { 0xa694ad05, 3 },
    { 0, 0 },
    { 0xbf80ad07, 3 },
    { 0, 0 },
    { 0xbda0ad09, 4 },
    { 0x2afcad0a, 3 },
    { 0xb138ad09, 4 },
    { 0x3c40ad0b, 3 },
    { 0xe2ccad0b, 3 },
    { 0, 0 },
    { 0x2c04ad0f, 3 },
    { 0x5ca0ad10, 3 },
    { 0x5944ad0f, 4 },
    { 0xcc94ad12, 4 },
    { 0xfa88ad12, 3 },
    { 0xb17cad14, 3 },
    { 0xac68ad12, 4 },
    { 0xa7dcad16, 3 },
    { 0x4a64ad0f, 4 },
    { 0x4f9cad18, 3 },
    { 0xe7b4ad19, 4 },
    { 0xd4ccad17, 4 },
    { 0x2da0ad13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388ad24, 4 },
    { 0, 0 },
    { 0xbcb4ad26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed50ad2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1760ad2e, 3 },
    { 0, 0 },
    { 0xbe48ad30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ad35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2758ad39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6404ad3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd4ad3f, 3 },
    { 0x20d8ad3f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a4ad44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f4ad49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ad4d, 3 },
    { 0, 0 },
    { 0xd53cad4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabcad56, 3 },
    { 0, 0 },
    { 0x4afcad58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cad5f, 3 },
    { 0, 0 },
    { 0xd568ad61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8cad67, 4 },
    { 0x4480ad68, 3 },
    { 0, 0 },
    { 0xe1c8ad6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb0ad6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b0ad7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8ad7d, 3 },
    { 0x21acad7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a4ad82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d94ad86, 3 },
    { 0xa9bcad86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb598ad8b, 4 },
    { 0x981cad8c, 4 },
    { 0xca7cad8b, 3 },
    { 0x3e4ad8e, 4 },
    { 0, 0 },
    { 0xcbc4ad90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dacad95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b0ad99, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1648ad9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6cada4, 4 },
    { 0, 0 },
    { 0x9870ada6, 4 },
    { 0x917cada7, 4 },
    { 0, 0 },
    { 0x9530ada9, 3 },
    { 0x7008ada9, 4 },
    { 0, 0 },
    { 0x9640adac, 3 },
    { 0, 0 },
    { 0x3844adae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc810adb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4adb9, 3 },
    { 0x9c2cadba, 4 },
    { 0, 0 },
    { 0x49c8adbc, 5 },
    { 0, 0 },
    { 0xb1e8adbe, 3 },
    { 0, 0 },
    { 0x9218adc0, 3 },
    { 0xddcadc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef10adc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4364adce, 3 },
    { 0xd320adcf, 3 },
    { 0x6544adcf, 3 },
    { 0x78b8add1, 3 },
    { 0x91e0add2, 3 },
    { 0, 0 },
    { 0x48cadd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd684addb, 3 },
    { 0x618caddb, 3 },
    { 0x65f4addc, 3 },
    { 0x711cadde, 3 },
    { 0x10d8addf, 3 },
    { 0x25ecade0, 4 },
    { 0x6f2cade1, 3 },
    { 0x884ade2, 3 },
    { 0, 0 },
    { 0x591cade4, 4 },
    { 0xd318ade4, 3 },
    { 0x4354ade6, 3 },
    { 0xd034ade4, 5 },
    { 0, 0 },
    { 0xf59cade9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4aded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e4adf1, 4 },
    { 0, 0 },
    { 0x874adf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ecadf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120adf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa084adfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb568ae01, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1cae05, 3 },
    { 0x7f14ae06, 3 },
    { 0x3bccae05, 5 },
    { 0x8234ae08, 3 },
    { 0x3f30ae09, 3 },
    { 0xe010ae09, 4 },
    { 0x3c68ae0a, 3 },
    { 0xfa94ae0c, 4 },
    { 0, 0 },
    { 0x42ecae0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edcae15, 3 },
    { 0, 0 },
    { 0x3ce4ae17, 3 },
    { 0, 0 },
    { 0xebb0ae19, 4 },
    { 0x65ecae1a, 3 },
    { 0x8aa4ae1a, 4 },
    { 0xadccae19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99cae21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d60ae24, 3 },
    { 0x7fecae24, 3 },
    { 0, 0 },
    { 0x85b0ae27, 3 },
    { 0, 0 },
    { 0x4e08ae29, 4 },
    { 0xd208ae2a, 3 },
    { 0x70bcae29, 3 },
    { 0x8b0ae2a, 4 },
    { 0x22dcae2d, 3 },
    { 0x7854ae2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd0ae34, 4 },
    { 0x26a4ae35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cae39, 4 },
    { 0x762cae3a, 3 },
    { 0, 0 },
    { 0x90ae3c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3600ae40, 4 },
    { 0x1030ae40, 4 },
    { 0x13a8ae42, 3 },
    { 0x718ae42, 3 },
    { 0, 0 },
    { 0x1654ae45, 3 },
    { 0, 0 },
    { 0x4fc8ae47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04cae4d, 3 },
    { 0x79d8ae4e, 3 },
    { 0xbf54ae4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4734ae54, 3 },
    { 0x5400ae55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3cae58, 3 },
    { 0x3d98ae58, 4 },
    { 0xfa00ae5a, 3 },
    { 0, 0 },
    { 0xfb1cae5c, 3 },
    { 0, 0 },
    { 0xd430ae5e, 3 },
    { 0x4e2cae5f, 5 },
    { 0xf358ae60, 3 },
    { 0xf51cae61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0cae65, 3 },
    { 0x6414ae66, 3 },
    { 0, 0 },
    { 0xfea4ae68, 3 },
    { 0x5d74ae69, 3 },
    { 0x2da0ae6a, 3 },
    { 0xa14cae6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b70ae72, 4 },
    { 0x7f8cae73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd8ae7a, 4 },
    { 0x7da4ae7b, 3 },
    { 0x9474ae7c, 3 },
    { 0x620ae7d, 4 },
    { 0xaa18ae7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc8ae81, 4 },
    { 0x7058ae82, 3 },
    { 0, 0 },
    { 0x8eecae84, 3 },
    { 0x9ce8ae84, 3 },
    { 0, 0 },
    { 0x1780ae87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1480ae90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x829cae93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c98ae9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf8ae9e, 3 },
    { 0xc6ecae9f, 4 },
    { 0x5c98aea0, 3 },
    { 0, 0 },
    { 0x10e4aea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4aea6, 3 },
    { 0, 0 },
    { 0x8024aea8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480aeaf, 3 },
    { 0, 0 },
    { 0x1208aeb1, 3 },
    { 0x7380aeb2, 3 },
    { 0xd810aeb3, 3 },
    { 0x65d8aeb3, 3 },
    { 0x7ac4aeb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62caeba, 3 },
    { 0x6b60aeba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb578aec8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd8aecd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc374aed3, 3 },
    { 0x7d58aed3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1144aed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14aedb, 4 },
    { 0, 0 },
    { 0xbebcaedd, 4 },
    { 0x11c0aede, 3 },
    { 0x2488aedf, 3 },
    { 0xaaccaedf, 3 },
    { 0x2cf8aee1, 3 },
    { 0x2770aee0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e4aee6, 4 },
    { 0x1b4aee7, 4 },
    { 0xef7caee8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44aef1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0aef4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a8aef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df0aefd, 3 },
    { 0x7dc0aefe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc84af02, 3 },
    { 0x520af02, 3 },
    { 0, 0 },
    { 0x230caf05, 3 },
    { 0, 0 },
    { 0xab18af07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8af0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4caf10, 3 },
    { 0x2534af10, 3 },
    { 0x30c4af12, 4 },
    { 0xfba4af13, 3 },
    { 0x3388af13, 3 },
    { 0x7690af11, 3 },
    { 0x6890af11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e78af1f, 3 },
    { 0xcb1caf20, 4 },
    { 0x8614af21, 3 },
    { 0, 0 },
    { 0x8d14af23, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34af27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1caf2d, 3 },
    { 0x1f88af2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08af32, 3 },
    { 0x476caf33, 3 },
    { 0x5c0caf34, 3 },
    { 0xb5b8af35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbcaf38, 3 },
    { 0x25a4af39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c8af3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x272caf40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e0af44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4af48, 4 },
    { 0x9f28af48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5098af4c, 3 },
    { 0xf720af4d, 3 },
    { 0x5598af4c, 4 },
    { 0x9644af4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d0af56, 3 },
    { 0xbc68af57, 3 },
    { 0x11ccaf58, 4 },
    { 0xfcb4af59, 3 },
    { 0xe1ecaf58, 3 },
    { 0x2160af5b, 3 },
    { 0xb960af5c, 3 },
    { 0x4b68af5d, 3 },
    { 0xe20caf5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf844af61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca08af65, 3 },
    { 0x5040af65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb0af69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa150af6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bacaf7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1130af81, 4 },
    { 0x8928af81, 3 },
    { 0, 0 },
    { 0x6064af84, 3 },
    { 0x3484af85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebecaf89, 3 },
    { 0xa940af89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5804af8d, 3 },
    { 0, 0 },
    { 0x3410af8f, 3 },
    { 0xe5a4af90, 3 },
    { 0, 0 },
    { 0xf748af92, 3 },
    { 0, 0 },
    { 0xa86caf94, 3 },
    { 0, 0 },
    { 0xd9ccaf96, 3 },
    { 0xf998af97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc48af9b, 3 },
    { 0x52b8af9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6064af9f, 3 },
    { 0xf460afa0, 3 },
    { 0, 0 },
    { 0xb108afa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8420afac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c4afb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a0afb6, 3 },
    { 0, 0 },
    { 0x496cafb8, 3 },
    { 0x1a48afb9, 3 },
    { 0, 0 },
    { 0xe2bcafbb, 4 },
    { 0, 0 },
    { 0xc2f0afbd, 3 },
    { 0x5704afbe, 3 },
    { 0, 0 },
    { 0x3728afc0, 3 },
    { 0x4a04afc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e4afc6, 3 },
    { 0x3c90afc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eacafca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5630afce, 3 },
    { 0x49acafcf, 3 },
    { 0x1b28afd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a4afd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe710afda, 3 },
    { 0x6d28afdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9cafdf, 4 },
    { 0, 0 },
    { 0xa330afe1, 4 },
    { 0xdab8afe2, 4 },
    { 0xc1d4afe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9538afe7, 4 },
    { 0, 0 },
    { 0x35e8afe9, 3 },
    { 0, 0 },
    { 0xd33cafeb, 3 },
    { 0xabf8afec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4aff0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4aff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3caff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee0aff9, 3 },
    { 0x6c74affa, 4 },
    { 0x83c8affa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2770affe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770b004, 3 },
    { 0xf48cb004, 3 },
    { 0xf23cb004, 4 },
    { 0x13f4b006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa4b00c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd4b00f, 3 },
    { 0x8814b010, 4 },
    { 0, 0 },
    { 0xb698b012, 5 },
    { 0x86f8b012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8b016, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90b019, 3 },
    { 0x7c30b01a, 3 },
    { 0, 0 },
    { 0x6e14b01c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e64b020, 3 },
    { 0x7060b020, 3 },
    { 0, 0 },
    { 0x2830b023, 3 },
    { 0, 0 },
    { 0xae2cb025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0b028, 3 },
    { 0xcc34b029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ccb02e, 3 },
    { 0xb604b02f, 4 },
    { 0, 0 },
    { 0xb9f0b031, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7824b034, 3 },
    { 0x5810b035, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874b03d, 3 },
    { 0x6138b03d, 4 },
    { 0x1714b03d, 3 },
    { 0, 0 },
    { 0xc36cb041, 3 },
    { 0, 0 },
    { 0x8b043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134b04e, 3 },
    { 0, 0 },
    { 0x4728b050, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a0b054, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9844b05f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b063, 3 },
    { 0xd588b063, 3 },
    { 0x80c8b063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a0b06f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd0b072, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b8b07a, 3 },
    { 0xf3d4b07b, 3 },
    { 0, 0 },
    { 0x67cb07d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fcb085, 4 },
    { 0xfe50b086, 3 },
    { 0, 0 },
    { 0xec44b088, 3 },
    { 0xc6ccb089, 3 },
    { 0x1ac8b08a, 3 },
    { 0xc954b08b, 3 },
    { 0x1ec8b089, 4 },
    { 0x5148b08d, 3 },
    { 0, 0 },
    { 0x6ce4b08f, 4 },
    { 0, 0 },
    { 0x8800b091, 5 },
    { 0, 0 },
    { 0xd530b093, 3 },
    { 0xc7ccb094, 3 },
    { 0xd570b095, 3 },
    { 0x6b98b093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa60b0a3, 3 },
    { 0xe168b0a3, 4 },
    { 0xb17cb0a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c40b0a8, 3 },
    { 0, 0 },
    { 0xe794b0aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a8b0af, 3 },
    { 0x1868b0b0, 3 },
    { 0, 0 },
    { 0x960cb0b2, 3 },
    { 0x68fcb0b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4b0b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef8b0ba, 3 },
    { 0xb1b4b0bb, 3 },
    { 0xf7c0b0bc, 3 },
    { 0, 0 },
    { 0xe2e8b0be, 3 },
    { 0xfac0b0be, 4 },
    { 0xf5fcb0bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c4b0cd, 3 },
    { 0x5d78b0ce, 3 },
    { 0x62fcb0cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9928b0d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8b0dc, 3 },
    { 0xcd90b0dd, 3 },
    { 0xc1f8b0dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8b0e1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8b0e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb78b0ec, 3 },
    { 0xc70b0ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2268b0f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d24b0f8, 3 },
    { 0x1ae4b0f8, 3 },
    { 0xaa20b0f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d8b102, 3 },
    { 0xb0cb103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534b109, 3 },
    { 0xc0a4b109, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1238b10e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418b113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd280b120, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44b12a, 3 },
    { 0x8d34b12b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d24b12f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b132, 5 },
    { 0x5c50b133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd764b139, 4 },
    { 0xf0a8b13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558cb143, 3 },
    { 0xa564b143, 3 },
    { 0x5b18b145, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b0b148, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe58b14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd748b151, 3 },
    { 0x40c0b152, 3 },
    { 0x8718b152, 4 },
    { 0x1fc4b154, 3 },
    { 0xdaecb154, 3 },
    { 0, 0 },
    { 0x3e3cb157, 4 },
    { 0, 0 },
    { 0xeea8b159, 3 },
    { 0, 0 },
    { 0xca04b15b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1110b15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3754b161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef0b164, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b44b167, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e4b16e, 3 },
    { 0xf3e0b16f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9644b176, 4 },
    { 0x2ae0b176, 3 },
    { 0, 0 },
    { 0x229cb179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c14b17c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7cb182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b0b188, 5 },
    { 0, 0 },
    { 0xf774b18a, 4 },
    { 0xb2d8b18b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f4b18e, 3 },
    { 0x4730b18f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864b192, 4 },
    { 0x4e64b193, 3 },
    { 0x31b8b194, 3 },
    { 0x5f84b195, 3 },
    { 0x8504b193, 4 },
    { 0x4210b194, 3 },
    { 0x8f8cb198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e14b19b, 3 },
    { 0, 0 },
    { 0x39d0b19d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e4b1a0, 3 },
    { 0x8e8cb1a1, 4 },
    { 0xa014b1a0, 4 },
    { 0, 0 },
    { 0xeb0b1a4, 4 },
    { 0xed2cb1a5, 3 },
    { 0xb71cb1a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c88b1aa, 3 },
    { 0xec60b1ab, 3 },
    { 0x5848b1ab, 3 },
    { 0xcd54b1ac, 3 },
    { 0, 0 },
    { 0x3c5cb1af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4b1b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64cb1b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c4b1bc, 3 },
    { 0x8e00b1bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2cb1c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485cb1d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f78b1d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8b1de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4518b1e4, 3 },
    { 0x8190b1e5, 4 },
    { 0xa14cb1e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb24b1e9, 4 },
    { 0x3fecb1e9, 3 },
    { 0x4f50b1e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b1f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde0b1f5, 3 },
    { 0x4638b1f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98b1fd, 3 },
    { 0xbca4b1fe, 5 },
    { 0x65a8b1fd, 4 },
    { 0x8a6cb200, 3 },
    { 0x77acb201, 3 },
    { 0x4f34b202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc884b207, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe88b20b, 3 },
    { 0, 0 },
    { 0x4d28b20d, 3 },
    { 0xa79cb20e, 4 },
    { 0x5730b20e, 5 },
    { 0, 0 },
    { 0x4fe0b211, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2720b218, 3 },
    { 0, 0 },
    { 0x9184b21a, 4 },
    { 0, 0 },
    { 0x56b0b21c, 4 },
    { 0x1be8b21d, 4 },
    { 0x6d2cb21d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918cb222, 3 },
    { 0xe280b222, 3 },
    { 0, 0 },
    { 0x5678b225, 3 },
    { 0x2b60b226, 3 },
    { 0xc988b227, 3 },
    { 0x3af4b228, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba40b22d, 4 },
    { 0x4014b22d, 4 },
    { 0x462cb22d, 3 },
    { 0xf1ecb230, 3 },
    { 0x4828b22f, 3 },
    { 0, 0 },
    { 0x66c8b233, 4 },
    { 0xc240b234, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f8b237, 4 },
    { 0xc040b238, 3 },
    { 0xf330b239, 4 },
    { 0xc72cb239, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b23d, 3 },
    { 0x3208b23e, 3 },
    { 0x88fcb23d, 3 },
    { 0, 0 },
    { 0x26f8b241, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d88b24c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1264b251, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f4b255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a4b258, 4 },
    { 0x3170b259, 3 },
    { 0x550cb25a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f58b25e, 4 },
    { 0xa89cb25f, 3 },
    { 0, 0 },
    { 0xa808b261, 3 },
    { 0xca90b261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a4b265, 3 },
    { 0x3c98b265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c4b272, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d0b278, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35cb27b, 3 },
    { 0xac40b27c, 4 },
    { 0x2bf4b27b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1818b281, 3 },
    { 0xa168b281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf874b287, 3 },
    { 0x4540b288, 3 },
    { 0x516cb289, 3 },
    { 0xe0acb28a, 4 },
    { 0x1944b28b, 3 },
    { 0xd198b28a, 3 },
    { 0xec74b28c, 3 },
    { 0x3430b28a, 3 },
    { 0x4464b28d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7298b292, 3 },
    { 0, 0 },
    { 0x9348b294, 3 },
    { 0x9804b295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb58b298, 3 },
    { 0x8c88b299, 3 },
    { 0xa794b29a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb29d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6cb2a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d60b2a4, 3 },
    { 0xf75cb2a4, 4 },
    { 0x3498b2a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8858b2aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3cb2ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3320b2b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d4b2b5, 3 },
    { 0, 0 },
    { 0x88f0b2b7, 4 },
    { 0x5744b2b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856cb2bd, 3 },
    { 0x6534b2be, 4 },
    { 0, 0 },
    { 0xe1b4b2c0, 3 },
    { 0, 0 },
    { 0x30c8b2c2, 4 },
    { 0xf44cb2c3, 4 },
    { 0x77d0b2c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278b2c8, 3 },
    { 0x6d7cb2c9, 3 },
    { 0x85f4b2c8, 3 },
    { 0, 0 },
    { 0x8d70b2cc, 4 },
    { 0xcd58b2cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4b2d0, 3 },
    { 0x592cb2d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2cb2d6, 3 },
    { 0x2b50b2d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b94b2df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9324b2e2, 3 },
    { 0xdecb2e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c94b2f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69cb2f6, 3 },
    { 0x8684b2f7, 4 },
    { 0, 0 },
    { 0xe664b2f9, 3 },
    { 0x37f0b2fa, 3 },
    { 0x5508b2fb, 3 },
    { 0xc278b2fc, 3 },
    { 0x9a54b2fd, 3 },
    { 0, 0 },
    { 0xf9cb2ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e64b303, 3 },
    { 0x32a0b304, 3 },
    { 0x55c8b305, 3 },
    { 0x9c3cb306, 4 },
    { 0x383cb305, 5 },
    { 0xa7d0b304, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd274b312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf054b315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f8b31c, 3 },
    { 0x985cb31c, 3 },
    { 0x62f0b31c, 3 },
    { 0, 0 },
    { 0x6778b320, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4b324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0b327, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb610b331, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958b338, 3 },
    { 0x315cb339, 3 },
    { 0, 0 },
    { 0x9398b33b, 5 },
    { 0xacd4b33c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe4b341, 3 },
    { 0xe1b0b341, 5 },
    { 0, 0 },
    { 0x7d50b344, 4 },
    { 0x1960b344, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23acb348, 3 },
    { 0x5bbcb349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2cb34e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b8b351, 3 },
    { 0x283cb351, 3 },
    { 0, 0 },
    { 0x5b84b354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e4b358, 4 },
    { 0, 0 },
    { 0x2d14b35a, 3 },
    { 0x3960b35a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9118b35f, 3 },
    { 0xce98b360, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6870b364, 3 },
    { 0xe064b364, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe168b36d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf10b371, 3 },
    { 0x3674b372, 3 },
    { 0, 0 },
    { 0x9298b374, 3 },
    { 0xbc48b375, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6510b379, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b37d, 4 },
    { 0x80ccb37d, 3 },
    { 0xa544b37f, 3 },
    { 0xfb3cb380, 3 },
    { 0xa164b380, 3 },
    { 0x5c50b37f, 4 },
    { 0, 0 },
    { 0x9100b384, 3 },
    { 0, 0 },
    { 0x44a4b386, 3 },
    { 0, 0 },
    { 0x2ebcb388, 4 },
    { 0, 0 },
    { 0x8c54b38a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd4b38d, 3 },
    { 0, 0 },
    { 0x7e2cb38f, 4 },
    { 0, 0 },
    { 0xfbd0b391, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938b398, 3 },
    { 0xb594b399, 3 },
    { 0xf958b398, 3 },
    { 0x1690b39a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcb39f, 3 },
    { 0, 0 },
    { 0xd940b3a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80b3a4, 3 },
    { 0x4e54b3a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958b3a9, 3 },
    { 0xdaf8b3a9, 3 },
    { 0x1760b3ab, 4 },
    { 0x51acb3ac, 4 },
    { 0x14ccb3ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a0b3b2, 4 },
    { 0x7d78b3b2, 3 },
    { 0x3280b3b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff4b3b7, 3 },
    { 0, 0 },
    { 0xbdd0b3b9, 3 },
    { 0x1f74b3ba, 3 },
    { 0x7000b3bb, 3 },
    { 0x4878b3ba, 4 },
    { 0x99acb3bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf984b3c0, 4 },
    { 0xe42cb3c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917cb3c5, 3 },
    { 0, 0 },
    { 0x123cb3c7, 3 },
    { 0x4b0cb3c8, 3 },
    { 0x8624b3c9, 3 },
    { 0x31b8b3c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8cb3ce, 3 },
    { 0x8ab0b3cf, 3 },
    { 0x7278b3ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee4b3d8, 4 },
    { 0, 0 },
    { 0x1b80b3da, 3 },
    { 0x73d4b3db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412cb3df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9cb3e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c8b3ea, 3 },
    { 0x9d0b3ea, 3 },
    { 0x8b14b3eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f0b3ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac0b3f6, 3 },
    { 0xc0b0b3f7, 4 },
    { 0, 0 },
    { 0x804b3f9, 3 },
    { 0x9b3cb3fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb628b404, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eacb414, 3 },
    { 0xbd60b415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf398b41d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410b422, 4 },
    { 0x5778b422, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c68b429, 3 },
    { 0xee00b42a, 3 },
    { 0, 0 },
    { 0x83acb42c, 3 },
    { 0, 0 },
    { 0x6f60b42e, 3 },
    { 0xaa18b42e, 3 },
    { 0x9f04b42e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e4b434, 3 },
    { 0xb828b434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80cb438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bcb43c, 3 },
    { 0, 0 },
    { 0x3194b43e, 4 },
    { 0x5b4cb43e, 5 },
    { 0, 0 },
    { 0x7734b441, 4 },
    { 0, 0 },
    { 0x970b443, 3 },
    { 0x7ab0b444, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc4b447, 3 },
    { 0x9bcb448, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b0b44b, 3 },
    { 0x120cb44c, 3 },
    { 0, 0 },
    { 0xa13cb44e, 3 },
    { 0, 0 },
    { 0xc1f4b450, 4 },
    { 0x2c18b451, 3 },
    { 0x3300b452, 3 },
    { 0x15acb453, 3 },
    { 0xdedcb454, 4 },
    { 0xeafcb454, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e4b45c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3168b462, 3 },
    { 0x6e8cb463, 3 },
    { 0xb390b463, 3 },
    { 0, 0 },
    { 0xf590b466, 3 },
    { 0xbb24b466, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d8b46c, 3 },
    { 0, 0 },
    { 0x236cb46e, 3 },
    { 0xf6cb46e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814b472, 3 },
    { 0, 0 },
    { 0xb138b474, 3 },
    { 0x631cb474, 3 },
    { 0x7c20b476, 3 },
    { 0x956cb476, 3 },
    { 0x1568b476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4408b47c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2cb47f, 3 },
    { 0xc4a4b480, 3 },
    { 0x9c48b47f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba64b485, 3 },
    { 0xcb08b486, 4 },
    { 0xe8fcb487, 3 },
    { 0x9d20b488, 5 },
    { 0x5bb0b485, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfccb48c, 3 },
    { 0xe20b48d, 3 },
    { 0x4e58b48e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944b491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3cb494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c10b498, 3 },
    { 0x2b0b498, 3 },
    { 0x70fcb499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c34b49d, 3 },
    { 0x8e60b49d, 3 },
    { 0xe720b49e, 4 },
    { 0x3684b49f, 4 },
    { 0x18e4b4a1, 3 },
    { 0x4920b4a1, 3 },
    { 0xf648b4a1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f50b4a6, 3 },
    { 0, 0 },
    { 0x3ab8b4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b24b4ab, 4 },
    { 0x6adcb4ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cb4b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb740b4bb, 4 },
    { 0xe928b4bc, 3 },
    { 0x4a0b4bc, 3 },
    { 0xeef0b4be, 3 },
    { 0x66cb4bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c4b4c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5774b4c6, 4 },
    { 0x5188b4c7, 3 },
    { 0x226cb4c6, 3 },
    { 0x3058b4c6, 3 },
    { 0, 0 },
    { 0x9050b4cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0b4d6, 3 },
    { 0x8e74b4d7, 3 },
    { 0x9a64b4d8, 4 },
    { 0x5b44b4d9, 3 },
    { 0, 0 },
    { 0x76e4b4db, 3 },
    { 0, 0 },
    { 0xae54b4dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb8b4e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa80b4e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd160b4e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2cb4ed, 3 },
    { 0x9f50b4ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c4b4f4, 3 },
    { 0, 0 },
    { 0xb12cb4f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580b4f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fcb4fe, 3 },
    { 0x42bcb4fe, 3 },
    { 0x9960b4ff, 3 },
    { 0x9150b4fe, 3 },
    { 0x88e8b4fe, 4 },
    { 0, 0 },
    { 0xe420b504, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d8b507, 3 },
    { 0xc914b508, 4 },
    { 0x1acb507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe084b50c, 3 },
    { 0, 0 },
    { 0xbae0b50e, 3 },
    { 0xa18cb50e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350cb513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b0b516, 3 },
    { 0x15cb516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a08b51a, 3 },
    { 0, 0 },
    { 0x2654b51c, 3 },
    { 0x5768b51d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f4b525, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd08b529, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18b52c, 3 },
    { 0xb040b52d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb370b53c, 3 },
    { 0x75b4b53d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6018b541, 3 },
    { 0, 0 },
    { 0x7ce8b543, 3 },
    { 0xe3c8b543, 3 },
    { 0, 0 },
    { 0xfdd8b546, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d0b54a, 3 },
    { 0xf984b54a, 3 },
    { 0, 0 },
    { 0xe920b54d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334b551, 3 },
    { 0, 0 },
    { 0xab70b553, 3 },
    { 0xcc10b554, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7cb55d, 4 },
    { 0x62acb55e, 3 },
    { 0x1d80b55e, 3 },
    { 0x2ca8b560, 3 },
    { 0x5fd0b561, 4 },
    { 0xd2a8b560, 4 },
    { 0, 0 },
    { 0x8a38b564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb308b567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc558b570, 3 },
    { 0xd3c0b571, 4 },
    { 0x92dcb572, 3 },
    { 0xdd18b573, 3 },
    { 0xd48b570, 3 },
    { 0x353cb573, 4 },
    { 0x5528b571, 3 },
    { 0, 0 },
    { 0x62c4b578, 3 },
    { 0x802cb579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00cb57c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1190b580, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5cb583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8b58a, 3 },
    { 0x1338b58a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0b594, 3 },
    { 0x5ca8b595, 3 },
    { 0x8484b594, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48cb59c, 4 },
    { 0x14ccb59d, 3 },
    { 0x9658b59e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8cb5a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc98b5a5, 3 },
    { 0, 0 },
    { 0xd544b5a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169cb5b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f0b5b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc530b5b9, 4 },
    { 0x5944b5ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc088b5c0, 3 },
    { 0x1344b5c1, 3 },
    { 0xa020b5c2, 3 },
    { 0xcf9cb5c3, 3 },
    { 0x1a58b5c4, 3 },
    { 0xab24b5c5, 4 },
    { 0, 0 },
    { 0xad40b5c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc338b5cc, 3 },
    { 0, 0 },
    { 0xdd7cb5ce, 3 },
    { 0, 0 },
    { 0x5560b5d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b5d3, 3 },
    { 0x7d90b5d4, 3 },
    { 0xb10b5d5, 4 },
    { 0x4e48b5d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x259cb5d9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefcb5dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0cb5e0, 3 },
    { 0x5d30b5e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be0b5e7, 3 },
    { 0x1f60b5e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa78b5ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350b5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x427cb5f9, 3 },
    { 0, 0 },
    { 0xa6f0b5fb, 4 },
    { 0xb324b5fc, 4 },
    { 0xf6f0b5fd, 4 },
    { 0x29d4b5fb, 3 },
    { 0xb088b5ff, 4 },
    { 0xf838b600, 3 },
    { 0x908cb601, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89cb607, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0b60b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3280b60e, 3 },
    { 0xff34b60f, 3 },
    { 0x191cb610, 4 },
    { 0, 0 },
    { 0x3a9cb612, 3 },
    { 0xf78b613, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88cb618, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb540b61e, 4 },
    { 0xc160b61e, 3 },
    { 0xeff8b61e, 3 },
    { 0x17a0b61f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d4b62f, 3 },
    { 0xeda8b630, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d18b634, 3 },
    { 0, 0 },
    { 0xd74b636, 3 },
    { 0, 0 },
    { 0xe7d8b638, 4 },
    { 0, 0 },
    { 0xe34b63a, 3 },
    { 0xa82cb63b, 3 },
    { 0x7624b63c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4cb643, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413cb647, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9240b64a, 3 },
    { 0x7044b64b, 3 },
    { 0xe6f0b64b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50b64f, 3 },
    { 0xe6b4b650, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7334b655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cb659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a0b661, 4 },
    { 0, 0 },
    { 0xe48b663, 3 },
    { 0, 0 },
    { 0x8864b665, 3 },
    { 0, 0 },
    { 0xea50b667, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5718b66a, 3 },
    { 0x3958b66b, 3 },
    { 0x8fa0b66c, 3 },
    { 0x7234b66d, 3 },
    { 0x4470b66e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2614b672, 4 },
    { 0, 0 },
    { 0xd80b674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82cb677, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568b67a, 3 },
    { 0xfa78b67b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d4b680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcb684, 3 },
    { 0x6344b685, 3 },
    { 0, 0 },
    { 0x6c0b687, 3 },
    { 0xb678b687, 3 },
    { 0x5044b687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b4b68e, 3 },
    { 0x5000b68e, 4 },
    { 0xeca4b690, 3 },
    { 0xea14b690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc4b69a, 3 },
    { 0x10ecb69b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1534b6a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4acb6a6, 3 },
    { 0x356cb6a6, 4 },
    { 0xd890b6a8, 3 },
    { 0x2f44b6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe588b6b1, 3 },
    { 0x4608b6b2, 4 },
    { 0x84c8b6b1, 3 },
    { 0, 0 },
    { 0x51dcb6b5, 3 },
    { 0xa5f0b6b5, 3 },
    { 0, 0 },
    { 0x7decb6b8, 3 },
    { 0, 0 },
    { 0xb248b6ba, 3 },
    { 0x11bcb6ba, 3 },
    { 0xbb84b6bc, 4 },
    { 0, 0 },
    { 0xba40b6be, 3 },
    { 0xbb70b6bf, 3 },
    { 0x7434b6c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173cb6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a8b6ca, 3 },
    { 0, 0 },
    { 0x9428b6cc, 3 },
    { 0x14a4b6cd, 3 },
    { 0, 0 },
    { 0x31ecb6cf, 4 },
    { 0, 0 },
    { 0x24b6d1, 3 },
    { 0x9350b6d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb028b6df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a4b6e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4b6ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9654b6f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4734b6fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cb702, 3 },
    { 0xcf84b702, 3 },
    { 0x4de0b704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b0b709, 4 },
    { 0x6bf0b70a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d0b70d, 4 },
    { 0x641cb70d, 3 },
    { 0xaf54b70f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd080b712, 3 },
    { 0xea94b713, 3 },
    { 0x1f74b713, 3 },
    { 0x12c8b715, 3 },
    { 0x6798b716, 3 },
    { 0x8304b714, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0b71e, 3 },
    { 0xc1cb71e, 3 },
    { 0x9ed0b720, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b4b727, 3 },
    { 0xe334b728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6cb72c, 4 },
    { 0, 0 },
    { 0x3fc4b72e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4cb735, 4 },
    { 0, 0 },
    { 0x6aa8b737, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe0b73d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294b742, 3 },
    { 0xe58cb743, 3 },
    { 0x17c8b743, 4 },
    { 0x2298b745, 3 },
    { 0, 0 },
    { 0x69f8b747, 3 },
    { 0x27a0b748, 3 },
    { 0xe10cb749, 4 },
    { 0x3f8cb74a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b8b74d, 3 },
    { 0, 0 },
    { 0x28cb74f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e78b753, 4 },
    { 0x6048b754, 4 },
    { 0, 0 },
    { 0x2ac4b756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54b75d, 3 },
    { 0, 0 },
    { 0xb3a0b75f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f4b767, 3 },
    { 0xf244b768, 4 },
    { 0x824cb769, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f4b772, 3 },
    { 0x481cb772, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf350b776, 3 },
    { 0x1a48b777, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d4b77c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf8b781, 4 },
    { 0, 0 },
    { 0xf83cb783, 4 },
    { 0x87f4b784, 4 },
    { 0xd9ecb785, 4 },
    { 0xae8cb786, 3 },
    { 0x2378b787, 4 },
    { 0x1534b786, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c8b78e, 3 },
    { 0x57b0b78f, 3 },
    { 0, 0 },
    { 0x59b4b791, 3 },
    { 0xda5cb792, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8b798, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5080b7a0, 3 },
    { 0x8dcb7a1, 3 },
    { 0, 0 },
    { 0xfde8b7a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2570b7a9, 3 },
    { 0x4cb8b7aa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0b7af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa14b7b2, 3 },
    { 0x9808b7b2, 4 },
    { 0xf068b7b4, 5 },
    { 0x52e0b7b5, 3 },
    { 0xd04cb7b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14b7c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cb7c4, 3 },
    { 0, 0 },
    { 0x1c0cb7c6, 3 },
    { 0x6de0b7c7, 4 },
    { 0x7054b7c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5448b7cc, 4 },
    { 0xf56cb7cd, 4 },
    { 0xdec4b7cd, 3 },
    { 0xa9d0b7cd, 3 },
    { 0x4f60b7cc, 5 },
    { 0x1f30b7d0, 3 },
    { 0x993cb7d2, 3 },
    { 0xca54b7d2, 3 },
    { 0xd138b7d2, 3 },
    { 0xf7a4b7d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0b7db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a8b7df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9108b7e2, 4 },
    { 0x4908b7e2, 3 },
    { 0, 0 },
    { 0x7748b7e5, 3 },
    { 0, 0 },
    { 0x4218b7e7, 3 },
    { 0, 0 },
    { 0x86a8b7e9, 3 },
    { 0, 0 },
    { 0x27acb7eb, 3 },
    { 0, 0 },
    { 0x97b8b7ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x496cb7f0, 4 },
    { 0x4ca4b7f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a18b7f8, 3 },
    { 0x7628b7f9, 3 },
    { 0, 0 },
    { 0x4060b7fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ecb7ff, 3 },
    { 0xe9c0b7ff, 3 },
    { 0, 0 },
    { 0x22d4b802, 3 },
    { 0xc978b803, 4 },
    { 0, 0 },
    { 0x8f8cb805, 4 },
    { 0, 0 },
    { 0x5058b807, 4 },
    { 0, 0 },
    { 0xc73cb809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc40b80c, 3 },
    { 0xd9a0b80c, 3 },
    { 0x6f80b80e, 3 },
    { 0xb0a0b80f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2cb81a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c4b81d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67cb823, 4 },
    { 0xcfc0b824, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b8b828, 4 },
    { 0x9da4b828, 3 },
    { 0xe708b82a, 3 },
    { 0, 0 },
    { 0x43f4b82c, 3 },
    { 0x6108b82c, 3 },
    { 0x96b8b82d, 3 },
    { 0xfb38b82e, 4 },
    { 0, 0 },
    { 0xf08cb831, 3 },
    { 0xe240b832, 3 },
    { 0, 0 },
    { 0x4714b834, 3 },
    { 0x174cb835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacb838, 3 },
    { 0xb064b839, 4 },
    { 0xf600b839, 3 },
    { 0xaa10b838, 3 },
    { 0xb214b83b, 3 },
    { 0x6d40b83d, 4 },
    { 0x979cb83d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc78b841, 4 },
    { 0, 0 },
    { 0xce08b843, 3 },
    { 0xd4b843, 3 },
    { 0xa700b845, 3 },
    { 0x71d0b846, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4b849, 3 },
    { 0, 0 },
    { 0x9d04b84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8b850, 3 },
    { 0, 0 },
    { 0x347cb852, 3 },
    { 0x3080b853, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c78b858, 5 },
    { 0, 0 },
    { 0x863cb85a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e4b86c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8b86f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf380b875, 3 },
    { 0x42d8b876, 5 },
    { 0x2abcb877, 3 },
    { 0x4d50b878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace8b87c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x376cb87f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b0b885, 3 },
    { 0x806cb886, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a0b88d, 3 },
    { 0x2ff4b88e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e4b891, 3 },
    { 0, 0 },
    { 0x7ab8b893, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14b898, 4 },
    { 0xdb38b898, 4 },
    { 0x72fcb89a, 3 },
    { 0x4530b898, 5 },
    { 0x990b89a, 3 },
    { 0x42f8b89a, 3 },
    { 0x7dc0b89e, 3 },
    { 0x73b0b89e, 4 },
    { 0xbbe4b8a0, 4 },
    { 0, 0 },
    { 0x374cb8a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb8a7, 3 },
    { 0x7510b8a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e4b8ac, 3 },
    { 0x8c44b8ac, 3 },
    { 0x880b8ac, 4 },
    { 0x3598b8af, 4 },
    { 0x1e58b8ac, 4 },
    { 0, 0 },
    { 0xeb1cb8b2, 3 },
    { 0xa548b8b3, 3 },
    { 0x724b8b4, 3 },
    { 0, 0 },
    { 0xfb68b8b6, 3 },
    { 0x2b20b8b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a8b8ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b28b8c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1420b8c8, 3 },
    { 0xe148b8c8, 3 },
    { 0x3780b8c9, 3 },
    { 0x74d8b8ca, 3 },
    { 0x9fbcb8cc, 3 },
    { 0x1c3cb8cd, 4 },
    { 0xba00b8cd, 3 },
    { 0, 0 },
    { 0x725cb8d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cb8db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8cb8de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794b8e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e90b8f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838b8f5, 3 },
    { 0xd740b8f6, 3 },
    { 0xe554b8f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2184b8fa, 3 },
    { 0x2024b8fb, 3 },
    { 0xd2e0b8fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b901, 4 },
    { 0, 0 },
    { 0xd99cb903, 4 },
    { 0, 0 },
    { 0x974cb905, 3 },
    { 0xfad0b906, 4 },
    { 0, 0 },
    { 0xbd4cb908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8b90d, 4 },
    { 0xb8e4b90e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f8b911, 3 },
    { 0x6570b912, 4 },
    { 0x6d8cb913, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1684b917, 3 },
    { 0x5320b917, 3 },
    { 0xa378b919, 3 },
    { 0x3d28b91a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ecb922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1cb925, 3 },
    { 0, 0 },
    { 0xe068b927, 3 },
    { 0, 0 },
    { 0xa7b4b929, 4 },
    { 0xce10b92a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba8b934, 4 },
    { 0xe85cb935, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ccb93a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf750b93f, 3 },
    { 0x2038b940, 3 },
    { 0xe890b940, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00cb944, 3 },
    { 0xa01cb945, 3 },
    { 0x1afcb946, 3 },
    { 0x4e18b945, 4 },
    { 0xeaecb944, 4 },
    { 0, 0 },
    { 0x78e0b94a, 3 },
    { 0xf2ccb94b, 3 },
    { 0x19b4b94c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcb94f, 3 },
    { 0x33e8b950, 3 },
    { 0, 0 },
    { 0xc660b952, 3 },
    { 0xc2c8b953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe934b956, 3 },
    { 0xc4c0b957, 3 },
    { 0x610b957, 5 },
    { 0xa180b959, 3 },
    { 0xc210b958, 3 },
    { 0, 0 },
    { 0x21c0b95c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d24b964, 3 },
    { 0x3f50b965, 5 },
    { 0xb5b8b966, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f20b96d, 4 },
    { 0xfad0b96d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c10b971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc30b979, 3 },
    { 0, 0 },
    { 0x2598b97b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14b97e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150b985, 3 },
    { 0x6344b986, 4 },
    { 0xe3e0b987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14b98c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab8b98f, 4 },
    { 0x147cb990, 4 },
    { 0xffd8b991, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1038b994, 5 },
    { 0x185cb994, 3 },
    { 0x96f0b996, 3 },
    { 0x35d0b996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c4b99d, 3 },
    { 0x5cccb99e, 3 },
    { 0x1498b99f, 3 },
    { 0xa110b99f, 3 },
    { 0x3d70b9a0, 3 },
    { 0, 0 },
    { 0x7ca8b9a3, 3 },
    { 0xa720b9a4, 3 },
    { 0, 0 },
    { 0xd964b9a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a04b9aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5810b9ad, 3 },
    { 0, 0 },
    { 0xfa48b9af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47cb9b3, 3 },
    { 0, 0 },
    { 0x1bc0b9b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa170b9b8, 3 },
    { 0x5dbcb9b8, 4 },
    { 0x28a4b9b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe914b9bd, 3 },
    { 0xebfcb9bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacb9c2, 3 },
    { 0, 0 },
    { 0xc250b9c4, 5 },
    { 0x4f70b9c5, 3 },
    { 0xcd8b9c6, 3 },
    { 0x5310b9c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339cb9cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf800b9cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2cb9d3, 3 },
    { 0x1898b9d4, 3 },
    { 0x20b9d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18b9d9, 3 },
    { 0, 0 },
    { 0xdac0b9db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdcb9de, 3 },
    { 0xe854b9de, 3 },
    { 0x6bc8b9e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53cb9e6, 3 },
    { 0x5dc0b9e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc128b9ea, 3 },
    { 0, 0 },
    { 0x113cb9ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e8b9f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c48b9fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1160ba00, 3 },
    { 0x7eb0ba01, 3 },
    { 0xf3dcba00, 3 },
    { 0x3498ba01, 3 },
    { 0xdd5cba04, 4 },
    { 0xb3c0ba04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced8ba09, 3 },
    { 0x27d0ba0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960ba0f, 4 },
    { 0x5154ba10, 3 },
    { 0x4c58ba10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1490ba1a, 3 },
    { 0x55d8ba1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877cba20, 3 },
    { 0x1c8cba21, 3 },
    { 0, 0 },
    { 0x53c4ba23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x632cba26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8004ba2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78cba30, 3 },
    { 0, 0 },
    { 0x2da4ba32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f4ba38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8cba3b, 3 },
    { 0x9650ba3c, 4 },
    { 0, 0 },
    { 0x2938ba3e, 3 },
    { 0x6becba3f, 3 },
    { 0x7444ba3f, 4 },
    { 0, 0 },
    { 0x5874ba42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dcba4a, 4 },
    { 0xaafcba4a, 3 },
    { 0, 0 },
    { 0x12a8ba4d, 4 },
    { 0x49ecba4e, 4 },
    { 0, 0 },
    { 0xb54cba50, 3 },
    { 0x994cba50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cba57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c0ba5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb350ba5f, 3 },
    { 0xa3bcba5f, 3 },
    { 0x6584ba5f, 3 },
    { 0, 0 },
    { 0x8cb0ba63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb68ba66, 4 },
    { 0xb24ba67, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982cba6b, 3 },
    { 0x3b9cba6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf58ba6f, 5 },
    { 0x8c4cba70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b0ba73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b70ba76, 3 },
    { 0x46e8ba76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6234ba7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dcba7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3510ba86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e18ba8d, 3 },
    { 0x8698ba8e, 3 },
    { 0, 0 },
    { 0x9a54ba90, 4 },
    { 0, 0 },
    { 0x2a5cba92, 3 },
    { 0x72bcba93, 3 },
    { 0x9e0cba94, 3 },
    { 0xd290ba95, 3 },
    { 0xc928ba94, 3 },
    { 0x9294ba97, 4 },
    { 0, 0 },
    { 0x25acba99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca0ba9c, 3 },
    { 0, 0 },
    { 0xb904ba9e, 3 },
    { 0, 0 },
    { 0xcd84baa0, 3 },
    { 0xa6b0baa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8baa4, 3 },
    { 0x5848baa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4baac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716cbab0, 3 },
    { 0, 0 },
    { 0x666cbab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dcbab5, 3 },
    { 0xe390bab6, 3 },
    { 0, 0 },
    { 0xc3c8bab8, 5 },
    { 0x41a0bab9, 3 },
    { 0, 0 },
    { 0x3d58babb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfcbac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab4bac6, 3 },
    { 0, 0 },
    { 0x8330bac8, 3 },
    { 0, 0 },
    { 0x6244baca, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4bacf, 3 },
    { 0, 0 },
    { 0x3438bad1, 3 },
    { 0, 0 },
    { 0xdfb8bad3, 3 },
    { 0xbe18bad4, 3 },
    { 0x2568bad5, 3 },
    { 0x98bcbad4, 3 },
    { 0x9d8cbad7, 3 },
    { 0xbf20bad7, 4 },
    { 0x79f4bad7, 3 },
    { 0xaa58bad5, 3 },
    { 0x52e4badb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4badf, 3 },
    { 0x91a0badf, 3 },
    { 0, 0 },
    { 0x1ca8bae2, 3 },
    { 0, 0 },
    { 0x40b8bae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330bae9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdcbaef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11acbaf3, 3 },
    { 0, 0 },
    { 0xd6d8baf5, 3 },
    { 0, 0 },
    { 0xcc18baf7, 3 },
    { 0x3b90baf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf618bafc, 3 },
    { 0, 0 },
    { 0x9c4bafe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94bb08, 3 },
    { 0x1594bb09, 4 },
    { 0, 0 },
    { 0x8554bb0b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab70bb11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b0bb18, 3 },
    { 0, 0 },
    { 0xd354bb1a, 3 },
    { 0xbd9cbb1b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c64bb22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbcbb2b, 3 },
    { 0, 0 },
    { 0xf738bb2d, 3 },
    { 0xa85cbb2e, 3 },
    { 0x956cbb2f, 3 },
    { 0xc1b4bb30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc678bb36, 3 },
    { 0x7f64bb37, 3 },
    { 0xde34bb36, 3 },
    { 0x85c8bb38, 3 },
    { 0xb140bb3a, 3 },
    { 0x2450bb3b, 3 },
    { 0, 0 },
    { 0x48fcbb3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c8bb40, 3 },
    { 0x1814bb41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9964bb4c, 3 },
    { 0, 0 },
    { 0x888cbb4e, 3 },
    { 0, 0 },
    { 0xe05cbb50, 3 },
    { 0xa4f4bb51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x605cbb54, 3 },
    { 0xf6f0bb54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee0bb59, 3 },
    { 0x298cbb5a, 3 },
    { 0x9808bb5b, 4 },
    { 0xa598bb5b, 4 },
    { 0, 0 },
    { 0x57b0bb5e, 3 },
    { 0x2554bb5f, 4 },
    { 0x558cbb60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e4bb66, 3 },
    { 0, 0 },
    { 0x36e4bb68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e68bb6c, 3 },
    { 0x3f08bb6d, 3 },
    { 0, 0 },
    { 0x74d8bb6f, 3 },
    { 0, 0 },
    { 0xbcf0bb71, 3 },
    { 0xeaa8bb72, 3 },
    { 0, 0 },
    { 0x5df8bb74, 4 },
    { 0xce60bb74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3cbb7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734cbb7e, 3 },
    { 0x2e80bb7f, 3 },
    { 0xbd84bb80, 3 },
    { 0x7734bb81, 4 },
    { 0xe5ccbb82, 4 },
    { 0xc164bb83, 4 },
    { 0, 0 },
    { 0x1c50bb85, 3 },
    { 0xbbc8bb85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6614bb8c, 5 },
    { 0, 0 },
    { 0xd834bb8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7010bb92, 3 },
    { 0xe740bb93, 3 },
    { 0xcd50bb94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x860bb97, 3 },
    { 0xba28bb98, 3 },
    { 0x9e7cbb98, 3 },
    { 0, 0 },
    { 0x15e8bb9b, 3 },
    { 0, 0 },
    { 0x2dc8bb9d, 3 },
    { 0, 0 },
    { 0x4774bb9f, 4 },
    { 0xc130bba0, 3 },
    { 0, 0 },
    { 0xaf0bba2, 3 },
    { 0xfdcbba2, 4 },
    { 0xd8a0bba2, 4 },
    { 0xa580bba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8358bba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3220bbab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211cbbb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8bbb9, 3 },
    { 0xea84bbba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e0bbc4, 4 },
    { 0, 0 },
    { 0xbd8bbc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a8bbd0, 3 },
    { 0x9474bbd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e0bbd7, 4 },
    { 0x92b0bbd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897cbbdc, 3 },
    { 0xda18bbdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148bbe5, 3 },
    { 0, 0 },
    { 0x4b5cbbe7, 3 },
    { 0xcb54bbe7, 4 },
    { 0x7b20bbe9, 4 },
    { 0, 0 },
    { 0x74a8bbeb, 3 },
    { 0x82f0bbec, 3 },
    { 0xeb20bbeb, 4 },
    { 0, 0 },
    { 0x3e60bbef, 4 },
    { 0, 0 },
    { 0xfdfcbbf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7750bbf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6808bbfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffcbc08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214bc0b, 3 },
    { 0x82fcbc0c, 4 },
    { 0, 0 },
    { 0x68d0bc0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe278bc13, 3 },
    { 0xa08bc13, 3 },
    { 0x9404bc15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c74bc1b, 3 },
    { 0, 0 },
    { 0x4058bc1d, 3 },
    { 0x2584bc1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8bc21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4000bc2a, 3 },
    { 0x6878bc2b, 3 },
    { 0xe488bc2c, 3 },
    { 0x16b8bc2d, 3 },
    { 0xb7d8bc2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc4bc33, 3 },
    { 0x2ee8bc33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4920bc37, 4 },
    { 0xbecbc38, 4 },
    { 0x3300bc39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174bc3e, 3 },
    { 0xa444bc3e, 4 },
    { 0xd2a4bc40, 3 },
    { 0x6270bc41, 3 },
    { 0xc49cbc42, 3 },
    { 0xcdd0bc3f, 3 },
    { 0xaac0bc3e, 3 },
    { 0x402cbc45, 3 },
    { 0x27fcbc46, 3 },
    { 0x5940bc41, 3 },
    { 0x8f70bc41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2cbc4c, 3 },
    { 0, 0 },
    { 0x4734bc4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cbc53, 3 },
    { 0xf20bc54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8bc5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69cbc5d, 3 },
    { 0, 0 },
    { 0x9334bc5f, 3 },
    { 0, 0 },
    { 0x8d98bc61, 3 },
    { 0, 0 },
    { 0x77dcbc63, 3 },
    { 0, 0 },
    { 0x87f0bc65, 3 },
    { 0x79a4bc66, 3 },
    { 0x1e10bc67, 3 },
    { 0xbb94bc68, 3 },
    { 0x668bc68, 4 },
    { 0x279cbc6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fcbc6d, 3 },
    { 0, 0 },
    { 0x3e78bc6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe8bc72, 3 },
    { 0, 0 },
    { 0xc50cbc74, 3 },
    { 0xb94cbc75, 4 },
    { 0xc258bc76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x755cbc79, 3 },
    { 0x63a0bc7a, 3 },
    { 0x1a10bc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb408bc82, 3 },
    { 0x9508bc83, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375cbc8c, 4 },
    { 0, 0 },
    { 0x659cbc8e, 3 },
    { 0, 0 },
    { 0xbf7cbc90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x258cbc93, 3 },
    { 0x41bcbc93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb8bc98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ecbc9b, 3 },
    { 0xf01cbc9c, 3 },
    { 0xdaf4bc9b, 4 },
    { 0, 0 },
    { 0x2d64bc9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x534bca2, 5 },
    { 0x7830bca3, 4 },
    { 0xdbbcbca3, 3 },
    { 0xaef0bca5, 3 },
    { 0x5840bca2, 4 },
    { 0, 0 },
    { 0x75c0bca8, 4 },
    { 0, 0 },
    { 0xd6cbcaa, 3 },
    { 0xeb4cbcab, 3 },
    { 0x64c4bcab, 4 },
    { 0x774cbcad, 4 },
    { 0x93d4bcaa, 4 },
    { 0x2240bcaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1010bcb2, 3 },
    { 0, 0 },
    { 0x270cbcb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0cbcb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2108bcbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3964bcc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b8bcc3, 4 },
    { 0xbf84bcc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a8bcc7, 4 },
    { 0x8eecbcc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ecbccb, 4 },
    { 0xb9dcbccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0bcd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b0bcd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe620bcdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418bcdf, 4 },
    { 0x714cbce0, 3 },
    { 0xcc34bce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60bce5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd4bce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf64bcef, 3 },
    { 0x4d9cbcf0, 3 },
    { 0x1decbcf1, 4 },
    { 0x40cbcf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8bcf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706cbcfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e0bcff, 3 },
    { 0, 0 },
    { 0xe408bd01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1188bd09, 3 },
    { 0, 0 },
    { 0xf28cbd0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x520cbd0e, 3 },
    { 0xc58bd0f, 3 },
    { 0xa224bd0e, 3 },
    { 0, 0 },
    { 0x9ac8bd12, 4 },
    { 0xc5d4bd13, 4 },
    { 0x98b4bd14, 4 },
    { 0xec78bd14, 3 },
    { 0xf218bd16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210bd1b, 3 },
    { 0, 0 },
    { 0x2d80bd1d, 3 },
    { 0xacb8bd1e, 3 },
    { 0xb2b0bd1f, 3 },
    { 0x6e00bd1f, 4 },
    { 0x4260bd1f, 3 },
    { 0, 0 },
    { 0x9e8cbd23, 3 },
    { 0, 0 },
    { 0xcea0bd25, 3 },
    { 0x3428bd26, 5 },
    { 0xde50bd27, 3 },
    { 0x15dcbd28, 3 },
    { 0xff10bd29, 3 },
    { 0x5e78bd2a, 4 },
    { 0x21b4bd2b, 3 },
    { 0xaf08bd2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1608bd30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd688bd35, 5 },
    { 0, 0 },
    { 0x95f8bd37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a20bd3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ecbd3e, 3 },
    { 0x7bf4bd3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4814bd49, 3 },
    { 0, 0 },
    { 0xbb38bd4b, 3 },
    { 0x50cbd4b, 3 },
    { 0xdba8bd4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdecbd50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45cbd53, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894cbd5b, 3 },
    { 0, 0 },
    { 0xa654bd5d, 3 },
    { 0x3310bd5d, 3 },
    { 0, 0 },
    { 0xda60bd60, 3 },
    { 0xc37cbd61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8660bd6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6448bd70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c40bd75, 3 },
    { 0x7b64bd76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188bd7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b54bd82, 3 },
    { 0, 0 },
    { 0x9030bd84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa50bd8c, 3 },
    { 0xcba4bd8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b0bd97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24bd9d, 3 },
    { 0, 0 },
    { 0xf7b4bd9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf554bda3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20bda7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf778bdac, 4 },
    { 0x8f98bdad, 4 },
    { 0x4c70bdae, 3 },
    { 0x569cbdae, 4 },
    { 0xee30bdad, 3 },
    { 0x92fcbdac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b94bdb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf8bdbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728bdbf, 4 },
    { 0, 0 },
    { 0x7a8bdc1, 3 },
    { 0xa4d4bdc1, 3 },
    { 0xc1acbdc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cbdc7, 3 },
    { 0, 0 },
    { 0xa038bdc9, 3 },
    { 0xf7b0bdca, 3 },
    { 0x892cbdc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a4bdd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2094bdd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308bde0, 3 },
    { 0x8d30bde1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6650bde4, 3 },
    { 0, 0 },
    { 0xa97cbde6, 4 },
    { 0xa7c4bde7, 4 },
    { 0x44f8bde6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20bdec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1570bdf1, 4 },
    { 0x5b7cbdf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7cbdf6, 3 },
    { 0x1a6cbdf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41cbdfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cbdfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab64be03, 3 },
    { 0x9ac8be04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e8be09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1cbe0e, 3 },
    { 0, 0 },
    { 0x8ac0be10, 3 },
    { 0x3fd4be11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e8be15, 4 },
    { 0x1688be16, 3 },
    { 0x9e10be17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47cbe1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5898be1d, 3 },
    { 0x52cbe1d, 3 },
    { 0xb9d0be1d, 3 },
    { 0, 0 },
    { 0x4d5cbe21, 3 },
    { 0xb1ccbe21, 3 },
    { 0, 0 },
    { 0x1d00be24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1344be28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c8be2b, 4 },
    { 0xa6dcbe2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8be31, 3 },
    { 0x18acbe32, 3 },
    { 0x9bc8be31, 3 },
    { 0xfb94be34, 3 },
    { 0, 0 },
    { 0xa67cbe36, 4 },
    { 0xafd8be36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84acbe3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc8be40, 3 },
    { 0x8f9cbe41, 3 },
    { 0, 0 },
    { 0x7c28be43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce54be49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b8be4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c4be52, 3 },
    { 0x4514be52, 3 },
    { 0x8fa0be54, 5 },
    { 0, 0 },
    { 0xd8f0be56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408be59, 4 },
    { 0, 0 },
    { 0xb454be5b, 4 },
    { 0, 0 },
    { 0xcd30be5d, 3 },
    { 0, 0 },
    { 0x4290be5f, 4 },
    { 0xbf3cbe60, 3 },
    { 0xc5b0be60, 3 },
    { 0, 0 },
    { 0x2778be63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0be6a, 3 },
    { 0x39a0be6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf560be6f, 3 },
    { 0xf124be70, 3 },
    { 0, 0 },
    { 0xf888be72, 3 },
    { 0xddd8be73, 3 },
    { 0, 0 },
    { 0x2788be75, 3 },
    { 0, 0 },
    { 0x6cf4be77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dcbe86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4cbe8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb4be8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d8be91, 3 },
    { 0, 0 },
    { 0x2fcbe93, 3 },
    { 0, 0 },
    { 0x4308be95, 3 },
    { 0x5610be96, 4 },
    { 0, 0 },
    { 0xd5d4be98, 3 },
    { 0xa2e4be99, 3 },
    { 0x1598be99, 3 },
    { 0x70a8be9b, 3 },
    { 0xdf20be9c, 3 },
    { 0xa998be9b, 3 },
    { 0, 0 },
    { 0x5e28be9f, 4 },
    { 0x80f0be9f, 3 },
    { 0, 0 },
    { 0xc998bea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d40beb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c0beb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe8beb6, 3 },
    { 0x2884beb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b0bebb, 3 },
    { 0x3e88bebc, 3 },
    { 0x691cbebd, 3 },
    { 0, 0 },
    { 0xececbebf, 3 },
    { 0x1bfcbec0, 3 },
    { 0xb59cbec1, 3 },
    { 0xbbfcbebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf678bec6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf54bec9, 3 },
    { 0x4d4bec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266cbecf, 3 },
    { 0xaa24becf, 4 },
    { 0xec38becf, 3 },
    { 0xd238bed2, 3 },
    { 0, 0 },
    { 0xa094bed4, 4 },
    { 0x18d0bed5, 3 },
    { 0x3188bed6, 3 },
    { 0x110cbed7, 3 },
    { 0x2300bed8, 3 },
    { 0xdb60bed4, 3 },
    { 0x1628bed9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c00bedd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x196cbee0, 3 },
    { 0xef04bee1, 3 },
    { 0x5aa4bee1, 3 },
    { 0xcaf0bee0, 3 },
    { 0xf9cbee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb974bee7, 3 },
    { 0x5acbee7, 3 },
    { 0x4cb4bee9, 5 },
    { 0, 0 },
    { 0x4e5cbeeb, 3 },
    { 0, 0 },
    { 0xdb28beed, 3 },
    { 0, 0 },
    { 0x2fd0beef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4bef4, 4 },
    { 0xf3ccbef5, 3 },
    { 0x1f2cbef5, 4 },
    { 0xbe68bef4, 4 },
    { 0x84e8bef8, 3 },
    { 0, 0 },
    { 0x7308befa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34bf00, 4 },
    { 0, 0 },
    { 0x4bccbf02, 3 },
    { 0, 0 },
    { 0x184cbf04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa204bf08, 4 },
    { 0, 0 },
    { 0x8700bf0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d4bf0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac4bf13, 4 },
    { 0xe86cbf14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f8bf17, 3 },
    { 0, 0 },
    { 0xddf0bf19, 4 },
    { 0, 0 },
    { 0xd8dcbf1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c04bf2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0bf2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6540bf32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360bf3e, 4 },
    { 0xe28bf3f, 3 },
    { 0xdc40bf3e, 3 },
    { 0x4930bf41, 3 },
    { 0x37c4bf41, 3 },
    { 0, 0 },
    { 0xc924bf44, 3 },
    { 0xe4f8bf45, 3 },
    { 0xb06cbf46, 3 },
    { 0xd5a8bf47, 3 },
    { 0xda08bf48, 4 },
    { 0, 0 },
    { 0x267cbf4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b88bf4e, 3 },
    { 0, 0 },
    { 0x4e04bf50, 3 },
    { 0xe228bf50, 3 },
    { 0, 0 },
    { 0x2034bf53, 3 },
    { 0x5430bf53, 3 },
    { 0x77a0bf54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c80bf5b, 3 },
    { 0x2e6cbf5c, 3 },
    { 0xfd0cbf5c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6460bf61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1594bf64, 3 },
    { 0x3e08bf65, 4 },
    { 0x76acbf64, 3 },
    { 0xc7bcbf65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d44bf6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74acbf74, 3 },
    { 0x90c8bf75, 3 },
    { 0, 0 },
    { 0xd20cbf77, 4 },
    { 0x238bf78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7758bf7d, 3 },
    { 0, 0 },
    { 0xf04bf7f, 4 },
    { 0xedc8bf80, 4 },
    { 0xd974bf81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738cbf85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44bf8a, 3 },
    { 0x1f1cbf8b, 3 },
    { 0xbf8c, 3 },
    { 0x7858bf8a, 4 },
    { 0, 0 },
    { 0x1a58bf8f, 3 },
    { 0xdaa8bf90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d4bf93, 3 },
    { 0x37e0bf94, 4 },
    { 0, 0 },
    { 0xcd40bf96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee18bf9a, 3 },
    { 0x43f4bf9b, 3 },
    { 0x21a4bf9b, 4 },
    { 0x60acbf9d, 4 },
    { 0xf68bf9e, 4 },
    { 0x9f7cbf9c, 3 },
    { 0x5d38bfa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb4bfa7, 3 },
    { 0x1eb4bfa8, 3 },
    { 0x9c64bfa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9cbfac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae0bfb0, 3 },
    { 0xf180bfb1, 4 },
    { 0, 0 },
    { 0x6ebcbfb3, 3 },
    { 0, 0 },
    { 0xef0bfb5, 3 },
    { 0xdf5cbfb5, 3 },
    { 0xb98bfb7, 4 },
    { 0x8f68bfb7, 3 },
    { 0x1c40bfb7, 3 },
    { 0x8ed0bfb5, 3 },
    { 0x8ed4bfb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb210bfc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ecbfcc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e4bfcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8cbfd3, 4 },
    { 0, 0 },
    { 0x22dcbfd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ecbfdf, 3 },
    { 0, 0 },
    { 0xe1c0bfe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839cbfe5, 3 },
    { 0, 0 },
    { 0xefb8bfe7, 3 },
    { 0x3144bfe8, 3 },
    { 0x4d8cbfe8, 3 },
    { 0x96a8bfe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e40bff0, 3 },
    { 0xc5dcbff0, 3 },
    { 0x6f08bff0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c4bff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ecc002, 4 },
    { 0, 0 },
    { 0x1ba8c004, 4 },
    { 0x74e8c004, 3 },
    { 0x6d9cc005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa358c00a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda4c00e, 3 },
    { 0x79a0c00e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27cc013, 3 },
    { 0xf748c014, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca50c019, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfacc01d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb18c02b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9718c031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5948c036, 3 },
    { 0, 0 },
    { 0x110c038, 3 },
    { 0x9fa0c039, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c0c03c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e44c041, 4 },
    { 0x42d0c042, 4 },
    { 0x9cf0c043, 3 },
    { 0, 0 },
    { 0xfa4cc045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee8c04e, 4 },
    { 0xba2cc04e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766cc054, 3 },
    { 0x3a84c055, 4 },
    { 0xa2b8c056, 4 },
    { 0x8670c056, 5 },
    { 0x2b8cc055, 3 },
    { 0x85d4c059, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d4c05f, 3 },
    { 0x989cc060, 4 },
    { 0x2b34c061, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d0c065, 3 },
    { 0x290cc066, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08c06d, 3 },
    { 0x6c38c06d, 3 },
    { 0, 0 },
    { 0x7c58c070, 3 },
    { 0, 0 },
    { 0x67c4c072, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f8c077, 4 },
    { 0xf854c078, 3 },
    { 0xdecc077, 4 },
    { 0x70d8c077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc38c07e, 4 },
    { 0x5044c07f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7cc082, 3 },
    { 0, 0 },
    { 0xaa30c084, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cc089, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1130c08c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e4c091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7754c099, 3 },
    { 0, 0 },
    { 0x8becc09b, 3 },
    { 0x7ff8c09c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30c09f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c78c0a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37cc0a7, 3 },
    { 0xb45cc0a8, 4 },
    { 0x2484c0a7, 3 },
    { 0xc78cc0aa, 3 },
    { 0xd34cc0aa, 3 },
    { 0xc918c0ac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa794c0af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba0c0b6, 3 },
    { 0xfd74c0b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dcc0ba, 3 },
    { 0x8bd8c0ba, 4 },
    { 0x1cb8c0bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f4c0c0, 3 },
    { 0, 0 },
    { 0xe534c0c2, 4 },
    { 0xb2e0c0c3, 3 },
    { 0xc140c0c4, 3 },
    { 0xe490c0c5, 3 },
    { 0, 0 },
    { 0x4e8c0c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2808c0cf, 3 },
    { 0x2c7cc0cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad8c0d3, 3 },
    { 0xa24c0d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3cc0d7, 4 },
    { 0x4d64c0d7, 3 },
    { 0xdf0cc0d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd38c0dd, 3 },
    { 0, 0 },
    { 0x46f0c0df, 3 },
    { 0, 0 },
    { 0x3430c0e1, 4 },
    { 0, 0 },
    { 0xd724c0e3, 3 },
    { 0x6300c0e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3740c0e8, 3 },
    { 0x3964c0e9, 3 },
    { 0, 0 },
    { 0x201cc0eb, 3 },
    { 0, 0 },
    { 0x6b18c0ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b8c0f7, 4 },
    { 0xc998c0f8, 4 },
    { 0x9ea0c0f9, 3 },
    { 0x9278c0fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff8c100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc106, 4 },
    { 0x640cc107, 3 },
    { 0x4e88c107, 4 },
    { 0xbdcc107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cc10e, 3 },
    { 0xd73cc10f, 3 },
    { 0x138c10f, 3 },
    { 0x98a8c110, 4 },
    { 0x1b58c110, 5 },
    { 0x210c113, 3 },
    { 0x35c0c114, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8c117, 3 },
    { 0xeccc117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c8c11b, 4 },
    { 0x3678c11c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd4c11f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4c125, 3 },
    { 0, 0 },
    { 0xa0fcc127, 3 },
    { 0x73dcc128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45cc12f, 3 },
    { 0x13cc12f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc334c138, 3 },
    { 0, 0 },
    { 0x4814c13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c4c13e, 4 },
    { 0xab7cc13f, 3 },
    { 0x8090c13f, 4 },
    { 0x398c141, 3 },
    { 0x1730c142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c0c146, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494cc14e, 3 },
    { 0xb984c14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf4c152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f84c156, 3 },
    { 0x4e94c157, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8240c161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4254c164, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edcc168, 5 },
    { 0x237cc169, 3 },
    { 0, 0 },
    { 0xfadcc16b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf960c171, 3 },
    { 0x6da8c171, 3 },
    { 0x64b0c171, 3 },
    { 0x48d0c173, 3 },
    { 0xa3b8c175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34c179, 4 },
    { 0x28b8c179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ccc17d, 3 },
    { 0x5e6cc17d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98c181, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9404c186, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b4c190, 3 },
    { 0, 0 },
    { 0xc70c192, 3 },
    { 0xaaf8c193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb4c196, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c8c19c, 5 },
    { 0x2708c19c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929cc1a2, 3 },
    { 0, 0 },
    { 0x6170c1a4, 4 },
    { 0x374cc1a5, 3 },
    { 0xd6cc1a5, 3 },
    { 0x247cc1a7, 3 },
    { 0, 0 },
    { 0x23dcc1a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebcc1ac, 4 },
    { 0x3970c1ad, 3 },
    { 0xee6cc1ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab0c1b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d8c1b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa078c1bb, 4 },
    { 0, 0 },
    { 0xd86cc1bd, 3 },
    { 0x6174c1bd, 3 },
    { 0xd68cc1be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x436cc1c2, 3 },
    { 0x573cc1c3, 3 },
    { 0x14bcc1c3, 3 },
    { 0xf3dcc1c5, 3 },
    { 0x433cc1c6, 3 },
    { 0x7f2cc1c4, 3 },
    { 0xc46cc1c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608c1cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc18c1d0, 4 },
    { 0, 0 },
    { 0xa424c1d2, 4 },
    { 0xcc10c1d2, 3 },
    { 0xd4e0c1d4, 3 },
    { 0xe600c1d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc4c1da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4c1de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad90c1e1, 4 },
    { 0, 0 },
    { 0xd100c1e3, 3 },
    { 0, 0 },
    { 0x6680c1e5, 3 },
    { 0, 0 },
    { 0x54bcc1e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e8c1ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1368c1f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e8c1f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb570c204, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c80c208, 3 },
    { 0xe334c209, 4 },
    { 0, 0 },
    { 0x6cdcc20b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcc210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c4c219, 3 },
    { 0, 0 },
    { 0xe240c21b, 3 },
    { 0, 0 },
    { 0x4cd0c21d, 3 },
    { 0, 0 },
    { 0xbaccc21f, 3 },
    { 0x6564c220, 3 },
    { 0xe7b4c21f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76cc225, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104cc22a, 4 },
    { 0x4ed8c22a, 4 },
    { 0xcfd4c22a, 3 },
    { 0xfd08c22b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dcc237, 3 },
    { 0x23f0c238, 3 },
    { 0x65d8c239, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa818c23c, 3 },
    { 0xc1ecc23d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f40c241, 5 },
    { 0, 0 },
    { 0x1248c243, 3 },
    { 0x714c244, 3 },
    { 0, 0 },
    { 0xa894c246, 4 },
    { 0x9a20c246, 4 },
    { 0x4534c247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f68c24b, 4 },
    { 0, 0 },
    { 0xb30cc24d, 3 },
    { 0, 0 },
    { 0x7d08c24f, 3 },
    { 0x3e60c250, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7258c253, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea4c256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21cc25c, 4 },
    { 0xe544c25d, 3 },
    { 0xde4cc25d, 3 },
    { 0, 0 },
    { 0xc30cc260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abcc265, 4 },
    { 0xe17cc266, 3 },
    { 0x3f0cc266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2cc26d, 3 },
    { 0x3760c26d, 3 },
    { 0x2790c26f, 3 },
    { 0, 0 },
    { 0xa190c271, 3 },
    { 0xcb4cc271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f4c275, 3 },
    { 0xf640c276, 3 },
    { 0x6188c277, 4 },
    { 0xfeb8c277, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d0c281, 3 },
    { 0xb510c282, 3 },
    { 0x37c4c281, 3 },
    { 0x69d8c284, 4 },
    { 0x5cbcc284, 3 },
    { 0x85dcc286, 4 },
    { 0xc15cc281, 3 },
    { 0xa560c288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b0c294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ecc298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3260c29d, 4 },
    { 0xcbe0c29e, 3 },
    { 0x1b60c29f, 4 },
    { 0x40cc29e, 3 },
    { 0x49fcc2a1, 4 },
    { 0xbc88c2a2, 3 },
    { 0, 0 },
    { 0x6230c2a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de4c2ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd54c2af, 3 },
    { 0x3a70c2b0, 3 },
    { 0, 0 },
    { 0x6accc2b2, 3 },
    { 0x4054c2b2, 4 },
    { 0x730c2b2, 3 },
    { 0xb46cc2b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda0c2b9, 3 },
    { 0, 0 },
    { 0x5864c2bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b4c2c4, 3 },
    { 0xd628c2c4, 3 },
    { 0x7a5cc2c6, 3 },
    { 0xbf04c2c5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee8c2cb, 3 },
    { 0x998cc2cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a48c2cf, 3 },
    { 0x9424c2cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd954c2d3, 3 },
    { 0, 0 },
    { 0xd23cc2d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b4c2dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b4c2e0, 3 },
    { 0x8634c2e1, 4 },
    { 0x8f10c2e1, 3 },
    { 0, 0 },
    { 0x3794c2e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5414c2e8, 3 },
    { 0x6f30c2e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840cc2f0, 3 },
    { 0, 0 },
    { 0xb868c2f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa070c2f8, 3 },
    { 0, 0 },
    { 0x14b0c2fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126cc300, 3 },
    { 0xba98c301, 3 },
    { 0, 0 },
    { 0x71fcc303, 3 },
    { 0x8bb8c304, 3 },
    { 0x4760c305, 4 },
    { 0x9ddcc303, 4 },
    { 0xee58c307, 4 },
    { 0xbd50c308, 3 },
    { 0x5438c309, 3 },
    { 0x8090c30a, 3 },
    { 0x65c8c309, 3 },
    { 0x8ba0c30c, 4 },
    { 0x8f7cc30d, 3 },
    { 0x216cc304, 3 },
    { 0x16f8c30e, 3 },
    { 0, 0 },
    { 0x55e4c311, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b4c316, 3 },
    { 0, 0 },
    { 0xcb08c318, 3 },
    { 0xd170c319, 3 },
    { 0xfc90c31a, 5 },
    { 0x939cc31b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab4c31e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a4c323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917cc327, 3 },
    { 0x4d34c328, 3 },
    { 0xd8bcc327, 3 },
    { 0x1c84c329, 3 },
    { 0, 0 },
    { 0xe484c32c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be0c32f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3414c337, 3 },
    { 0xe130c338, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8634c341, 4 },
    { 0x4138c341, 4 },
    { 0xa8a4c342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c68c347, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e08c350, 5 },
    { 0x5b40c351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6198c357, 4 },
    { 0xbca0c357, 3 },
    { 0x8d8c359, 3 },
    { 0xb5cc359, 3 },
    { 0x468c35b, 3 },
    { 0x5014c358, 3 },
    { 0xfac0c35c, 3 },
    { 0xc568c35e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6790c362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc372, 4 },
    { 0x33bcc372, 3 },
    { 0x4bc0c373, 3 },
    { 0xc1ecc375, 3 },
    { 0xb7c0c376, 4 },
    { 0xc9ccc374, 3 },
    { 0x8624c378, 4 },
    { 0xc34cc379, 3 },
    { 0x1d10c37a, 3 },
    { 0xaa14c37b, 3 },
    { 0, 0 },
    { 0x3b48c37d, 3 },
    { 0x9bb8c37d, 4 },
    { 0, 0 },
    { 0x5220c380, 3 },
    { 0x89d8c381, 3 },
    { 0xb804c381, 3 },
    { 0x4c2cc382, 3 },
    { 0, 0 },
    { 0x6f3cc385, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8560c38a, 4 },
    { 0x1da0c38b, 3 },
    { 0x60ecc38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d48c38f, 3 },
    { 0, 0 },
    { 0x5100c391, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7cc394, 3 },
    { 0, 0 },
    { 0xaae0c396, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588c39d, 3 },
    { 0xdcecc39d, 4 },
    { 0x4ed0c39f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0c3a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4c3ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd128c3b7, 4 },
    { 0x6a48c3b8, 3 },
    { 0x26b4c3b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b24c3bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d0c3c0, 4 },
    { 0x1eb8c3c1, 3 },
    { 0xca38c3c2, 3 },
    { 0x40c4c3c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e34c3cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c4c3cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd4c3d3, 4 },
    { 0x70c8c3d3, 3 },
    { 0xc2f0c3d5, 3 },
    { 0x48a0c3d5, 4 },
    { 0, 0 },
    { 0x1b14c3d8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c60c3e0, 4 },
    { 0x9ec8c3e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56cc3e7, 3 },
    { 0x4928c3e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb578c3eb, 3 },
    { 0x7d10c3ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6cc3ef, 3 },
    { 0, 0 },
    { 0xc6d4c3f1, 4 },
    { 0xbd2cc3f1, 4 },
    { 0x620cc3f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710c3f6, 3 },
    { 0xd5d0c3f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe420c3fb, 3 },
    { 0xb28cc3fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549cc401, 3 },
    { 0xd08c401, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4c407, 3 },
    { 0x639cc408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270cc40c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0c40f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83cc414, 3 },
    { 0xeeb8c415, 4 },
    { 0xc58cc416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628cc41b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd164c41f, 3 },
    { 0x7e20c420, 4 },
    { 0x3180c41f, 3 },
    { 0x2e4c41f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc0c429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c42d, 4 },
    { 0x44c4c42e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544c431, 3 },
    { 0, 0 },
    { 0x454c433, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b4c438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee20c43d, 3 },
    { 0x900cc43e, 3 },
    { 0, 0 },
    { 0x88ecc440, 3 },
    { 0x3938c440, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe8c446, 3 },
    { 0x24a0c447, 3 },
    { 0xee08c447, 3 },
    { 0x1d70c446, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913cc44f, 4 },
    { 0, 0 },
    { 0x8f60c451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78c459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1cc45d, 3 },
    { 0x166cc45e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1728c465, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcc473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c8c476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74c47a, 4 },
    { 0x720cc47a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bcc47f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f4c482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10cc489, 3 },
    { 0, 0 },
    { 0x3210c48b, 3 },
    { 0x77c0c48c, 4 },
    { 0xc4d0c48d, 3 },
    { 0xa580c48c, 3 },
    { 0xd850c48e, 4 },
    { 0, 0 },
    { 0xa9a4c491, 3 },
    { 0x305cc491, 4 },
    { 0x4060c492, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7980c49b, 3 },
    { 0, 0 },
    { 0x9b7cc49d, 3 },
    { 0x1c4cc49d, 3 },
    { 0x5778c49f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cc4a3, 3 },
    { 0xf9f8c4a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acc4a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d8c4ae, 4 },
    { 0, 0 },
    { 0x896cc4b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8530c4b4, 3 },
    { 0x3848c4b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150c4ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd794c4c2, 3 },
    { 0, 0 },
    { 0xa4d0c4c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dacc4ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e4c4d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4c4d8, 3 },
    { 0, 0 },
    { 0xa240c4da, 4 },
    { 0xf330c4db, 4 },
    { 0xdaa8c4dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8c4e0, 3 },
    { 0x7c64c4e1, 3 },
    { 0x2020c4e1, 3 },
    { 0xad10c4e3, 3 },
    { 0x510c4e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798c4eb, 3 },
    { 0x5324c4eb, 3 },
    { 0xaf80c4ed, 3 },
    { 0xa218c4ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482cc4f3, 3 },
    { 0xe3d8c4f4, 3 },
    { 0xb8a4c4f4, 3 },
    { 0xfc60c4f5, 3 },
    { 0xa61cc4f7, 3 },
    { 0, 0 },
    { 0x8138c4f9, 5 },
    { 0, 0 },
    { 0xd44c4fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fcc4ff, 3 },
    { 0, 0 },
    { 0x514cc501, 3 },
    { 0xe9b0c501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf4c505, 4 },
    { 0x944c506, 3 },
    { 0x324c506, 4 },
    { 0x5954c508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa664c50b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e64c510, 3 },
    { 0x7820c511, 4 },
    { 0xe0ccc511, 3 },
    { 0x1cacc512, 3 },
    { 0x8740c514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e84c51e, 4 },
    { 0x3144c51f, 3 },
    { 0, 0 },
    { 0xd4f0c521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac24c524, 4 },
    { 0xa368c525, 3 },
    { 0, 0 },
    { 0xa364c527, 3 },
    { 0x684cc528, 3 },
    { 0x3490c529, 4 },
    { 0xb4c4c52a, 3 },
    { 0, 0 },
    { 0xc1ccc52c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a4c538, 3 },
    { 0x665cc539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf0c53e, 3 },
    { 0x8c00c53e, 3 },
    { 0x3504c540, 3 },
    { 0, 0 },
    { 0x38b4c542, 3 },
    { 0xc59cc543, 4 },
    { 0, 0 },
    { 0xaaf0c545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9038c548, 3 },
    { 0x25d4c549, 4 },
    { 0x59b4c548, 3 },
    { 0x8990c54b, 4 },
    { 0xaf34c549, 4 },
    { 0xcd34c54b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c80c550, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab84c555, 3 },
    { 0, 0 },
    { 0xe498c557, 4 },
    { 0x5df4c558, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38cc55b, 3 },
    { 0xecf0c55c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c90c560, 3 },
    { 0xecf4c560, 3 },
    { 0x738c562, 3 },
    { 0, 0 },
    { 0xfa34c564, 4 },
    { 0x3b8cc565, 3 },
    { 0x34ccc566, 3 },
    { 0xe0ecc567, 3 },
    { 0x5298c568, 3 },
    { 0xf5c8c567, 4 },
    { 0xbccc56a, 3 },
    { 0xc738c56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910c571, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb698c577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcc57a, 3 },
    { 0xb0cc57b, 3 },
    { 0, 0 },
    { 0xae64c57d, 3 },
    { 0, 0 },
    { 0x245cc57f, 3 },
    { 0, 0 },
    { 0xe428c581, 3 },
    { 0x7d2cc582, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a78c587, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc164c58b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb4c58e, 3 },
    { 0x4be8c58f, 3 },
    { 0, 0 },
    { 0xb890c591, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a8c597, 5 },
    { 0, 0 },
    { 0xa85cc599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154c59c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c8c5a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30c5a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2810c5aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0cc5ad, 4 },
    { 0xe844c5ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb74c5b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80cc5b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3440c5c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7cc5c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9014c5cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff4c5d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f4c5d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784c5da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a8c5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a4c5e8, 3 },
    { 0xdac8c5e8, 3 },
    { 0xe530c5ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e88c5ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa610c5f1, 3 },
    { 0x1f4cc5f2, 4 },
    { 0x2a4c5f3, 4 },
    { 0x8720c5f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a0c5f8, 3 },
    { 0, 0 },
    { 0xb818c5fa, 3 },
    { 0xfa60c5fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc601, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c0c606, 3 },
    { 0xcd9cc607, 3 },
    { 0x4194c608, 5 },
    { 0xd7e4c609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb0c60c, 3 },
    { 0x7d80c60c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58cc611, 3 },
    { 0, 0 },
    { 0xe310c613, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0c61a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528c61d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659cc624, 3 },
    { 0x540c625, 3 },
    { 0, 0 },
    { 0xd244c627, 3 },
    { 0, 0 },
    { 0x8680c629, 4 },
    { 0xd7b0c629, 4 },
    { 0x3dc4c62b, 3 },
    { 0, 0 },
    { 0x5960c62d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2114c630, 3 },
    { 0, 0 },
    { 0x4970c632, 3 },
    { 0x92ccc633, 3 },
    { 0x960cc633, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34c637, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8c644, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c64c, 3 },
    { 0xefa8c64c, 3 },
    { 0x7af4c64e, 4 },
    { 0, 0 },
    { 0xa5acc650, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50c654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8c657, 3 },
    { 0, 0 },
    { 0x2980c659, 3 },
    { 0x9814c65a, 3 },
    { 0x7f0c65b, 4 },
    { 0x7d14c65c, 3 },
    { 0xe400c65c, 3 },
    { 0x6878c65e, 3 },
    { 0, 0 },
    { 0x9af8c660, 3 },
    { 0x96e8c661, 3 },
    { 0x8938c662, 3 },
    { 0xf4a4c663, 3 },
    { 0x72b4c664, 3 },
    { 0x6b10c665, 3 },
    { 0x8fd0c660, 3 },
    { 0x8c88c665, 3 },
    { 0xce74c668, 3 },
    { 0x27a8c669, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5cc66c, 3 },
    { 0x8ab4c66d, 3 },
    { 0xedb4c66e, 3 },
    { 0x1078c66e, 3 },
    { 0xa37cc670, 3 },
    { 0, 0 },
    { 0x28d4c672, 3 },
    { 0xc648c673, 3 },
    { 0x9f0cc674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7acc679, 3 },
    { 0, 0 },
    { 0x64d4c67b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74c67e, 3 },
    { 0, 0 },
    { 0x1a94c680, 3 },
    { 0xaf5cc680, 4 },
    { 0x2ff8c682, 3 },
    { 0x4550c681, 5 },
    { 0xae18c683, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9dcc689, 4 },
    { 0, 0 },
    { 0x224c68b, 3 },
    { 0x8b8c68c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0cc68f, 3 },
    { 0x48cc690, 3 },
    { 0, 0 },
    { 0x9134c692, 3 },
    { 0x70d0c692, 3 },
    { 0, 0 },
    { 0x5a3cc695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c0c698, 3 },
    { 0, 0 },
    { 0x8228c69a, 4 },
    { 0, 0 },
    { 0xef78c69c, 3 },
    { 0, 0 },
    { 0x5ff8c69e, 3 },
    { 0x44d0c69f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f8c6a2, 4 },
    { 0, 0 },
    { 0xf408c6a4, 3 },
    { 0xc288c6a5, 4 },
    { 0x5fbcc6a5, 3 },
    { 0x25e4c6a6, 3 },
    { 0, 0 },
    { 0xc01cc6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db8c6af, 3 },
    { 0, 0 },
    { 0x2a84c6b1, 3 },
    { 0xe138c6b1, 4 },
    { 0, 0 },
    { 0x4290c6b4, 3 },
    { 0x9db0c6b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3550c6b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd968c6bd, 3 },
    { 0xb81cc6be, 4 },
    { 0xc60c6bf, 3 },
    { 0xec84c6bf, 4 },
    { 0x4f98c6bd, 3 },
    { 0, 0 },
    { 0x377cc6c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d8c6c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb190c6ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f0c6d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb080c6d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc130c6e0, 3 },
    { 0xbd54c6e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd770c6e7, 3 },
    { 0x4100c6e7, 3 },
    { 0x8f78c6e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c6ec, 3 },
    { 0xd6e4c6ed, 3 },
    { 0xf5b0c6ed, 5 },
    { 0x3e8cc6ec, 3 },
    { 0, 0 },
    { 0x2f3cc6f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a8c6f8, 3 },
    { 0, 0 },
    { 0x4b04c6fa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d28c6fd, 3 },
    { 0x1d8c6fd, 3 },
    { 0, 0 },
    { 0xc670c700, 3 },
    { 0, 0 },
    { 0xc368c702, 3 },
    { 0, 0 },
    { 0x9d88c704, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8784c70b, 3 },
    { 0, 0 },
    { 0x5e94c70d, 4 },
    { 0x36ecc70e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc711, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354c71e, 3 },
    { 0xdda4c71e, 3 },
    { 0xcc38c720, 3 },
    { 0xe7a4c71f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e4c724, 3 },
    { 0xe308c725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x906cc728, 3 },
    { 0xff50c729, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b8c733, 3 },
    { 0xe5e4c733, 3 },
    { 0x6dfcc735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x200c738, 3 },
    { 0xd2e0c739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968c73e, 3 },
    { 0xcaa0c73e, 3 },
    { 0xa750c740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410c743, 3 },
    { 0xfa2cc744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f8c74e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d10c752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cc759, 3 },
    { 0, 0 },
    { 0xde00c75b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc108c75e, 4 },
    { 0xfc7cc75e, 3 },
    { 0xeda8c760, 3 },
    { 0xc278c761, 4 },
    { 0x5ff0c761, 3 },
    { 0x4f10c762, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a14c76a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe48c76e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e48c773, 3 },
    { 0x9cc4c774, 3 },
    { 0x5dfcc775, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd520c778, 5 },
    { 0x1ef0c778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bcc780, 3 },
    { 0xfb38c781, 3 },
    { 0x2fa0c782, 3 },
    { 0x42a8c783, 3 },
    { 0x9df0c783, 3 },
    { 0x337cc781, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9238c788, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4c78b, 3 },
    { 0xd218c78c, 3 },
    { 0x3ee0c78b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b8c790, 3 },
    { 0xf4e4c791, 3 },
    { 0, 0 },
    { 0xb40c793, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068c796, 4 },
    { 0xa36cc796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4cc79c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2480c7a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0c7a9, 4 },
    { 0x3220c7aa, 3 },
    { 0, 0 },
    { 0xa4c4c7ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f4c7b2, 3 },
    { 0xe49cc7b3, 3 },
    { 0x8b94c7b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1cc7b7, 5 },
    { 0, 0 },
    { 0xfa04c7b9, 3 },
    { 0x350c7ba, 4 },
    { 0xa534c7bb, 4 },
    { 0xa50c7bc, 3 },
    { 0x492cc7b9, 4 },
    { 0x3a94c7b9, 4 },
    { 0xd9e0c7bc, 3 },
    { 0x1f6cc7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff84c7c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd8c7cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f40c7e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c58c7e9, 3 },
    { 0x8c38c7e9, 4 },
    { 0, 0 },
    { 0x254c7ec, 3 },
    { 0x8be4c7ed, 3 },
    { 0xa364c7ec, 3 },
    { 0x4accc7ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22cc7f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x521cc7f6, 4 },
    { 0x4dd4c7f7, 3 },
    { 0, 0 },
    { 0xbdacc7f9, 3 },
    { 0x71e0c7fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc168c802, 4 },
    { 0x2abcc803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ccc806, 4 },
    { 0, 0 },
    { 0x49d8c808, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6cc80d, 4 },
    { 0x58c0c80e, 4 },
    { 0x3774c80f, 3 },
    { 0xf328c810, 4 },
    { 0, 0 },
    { 0x4618c812, 3 },
    { 0xda0cc812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f8c819, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578cc81d, 3 },
    { 0x2340c81d, 3 },
    { 0, 0 },
    { 0x474cc820, 4 },
    { 0x7618c821, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d78c824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae68c828, 4 },
    { 0, 0 },
    { 0x8584c82a, 4 },
    { 0, 0 },
    { 0x3e88c82c, 3 },
    { 0x4e84c82c, 3 },
    { 0x3610c82e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ccc832, 3 },
    { 0xb8d0c832, 3 },
    { 0x58dcc833, 3 },
    { 0xb6f4c833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb154c838, 4 },
    { 0x1ab8c839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca0c83f, 3 },
    { 0x8098c83f, 3 },
    { 0, 0 },
    { 0x5c8cc842, 4 },
    { 0, 0 },
    { 0xaeacc844, 3 },
    { 0xd800c845, 3 },
    { 0, 0 },
    { 0xe374c847, 4 },
    { 0x7e90c847, 3 },
    { 0xeca4c849, 3 },
    { 0, 0 },
    { 0xbb10c84b, 3 },
    { 0, 0 },
    { 0xf8ccc84d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4c852, 3 },
    { 0xb614c853, 4 },
    { 0x3b9cc854, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc260c857, 3 },
    { 0x4f58c857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb8c860, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c0c863, 3 },
    { 0x12f8c864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe240c867, 3 },
    { 0, 0 },
    { 0xc3b4c869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728c86d, 3 },
    { 0, 0 },
    { 0xada8c86f, 3 },
    { 0x5698c86f, 3 },
    { 0x1514c871, 4 },
    { 0, 0 },
    { 0x346cc873, 3 },
    { 0x8ab8c874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148c87c, 3 },
    { 0x9d94c87d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744c886, 3 },
    { 0, 0 },
    { 0x1e20c888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1cc894, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690c89a, 3 },
    { 0xb214c89b, 3 },
    { 0xb670c89a, 3 },
    { 0x3f8c89d, 3 },
    { 0x3d00c89e, 3 },
    { 0x7b0c89f, 3 },
    { 0x6f30c8a0, 3 },
    { 0, 0 },
    { 0xee4c8a2, 5 },
    { 0x6b1cc8a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd8c8a6, 3 },
    { 0x1ed4c8a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb08c8ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00c8b0, 4 },
    { 0xb06cc8b1, 3 },
    { 0, 0 },
    { 0xd180c8b3, 4 },
    { 0xfc70c8b4, 3 },
    { 0, 0 },
    { 0xf130c8b6, 3 },
    { 0x24b4c8b6, 3 },
    { 0, 0 },
    { 0x8c8cc8b9, 4 },
    { 0x9dccc8ba, 4 },
    { 0x8bd0c8bb, 3 },
    { 0x6608c8ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a00c8c0, 4 },
    { 0x2580c8c1, 3 },
    { 0x93e4c8c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74c8c6, 3 },
    { 0x8db8c8c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50c8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f8c8d0, 3 },
    { 0x4e4c8d0, 3 },
    { 0, 0 },
    { 0x3a40c8d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f24c8d8, 3 },
    { 0xf04c8d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3640c8dc, 4 },
    { 0x9dcc8dc, 3 },
    { 0xcef0c8dd, 3 },
    { 0x871cc8de, 3 },
    { 0, 0 },
    { 0xc84c8e1, 3 },
    { 0xada8c8e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60c8e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbcc8ec, 3 },
    { 0, 0 },
    { 0x6a6cc8ee, 4 },
    { 0x5f00c8ef, 3 },
    { 0x5b2cc8ee, 3 },
    { 0x6c7cc8f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d20c8f6, 3 },
    { 0xf86cc8f6, 3 },
    { 0x6da4c8f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edcc8fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdcc902, 3 },
    { 0, 0 },
    { 0x8390c904, 4 },
    { 0x8c84c905, 3 },
    { 0, 0 },
    { 0xf390c907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4c90a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x838cc90d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994c914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608c917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc0c91a, 3 },
    { 0x9584c91b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7700c91f, 3 },
    { 0xac14c91f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edcc924, 4 },
    { 0xfe28c924, 3 },
    { 0, 0 },
    { 0x340c927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6cc92a, 3 },
    { 0xe634c92b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a8c92f, 4 },
    { 0, 0 },
    { 0x81f8c931, 3 },
    { 0, 0 },
    { 0x3bd8c933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd700c936, 3 },
    { 0xdcf4c937, 3 },
    { 0x2ea4c938, 3 },
    { 0x2774c939, 3 },
    { 0x4044c93a, 3 },
    { 0x66cc93b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5178c944, 3 },
    { 0, 0 },
    { 0xf750c946, 3 },
    { 0x4f1cc946, 3 },
    { 0x8eecc948, 3 },
    { 0x3318c949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618c951, 3 },
    { 0xb6acc952, 3 },
    { 0x1d8cc953, 3 },
    { 0x5becc952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d4c95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7cc963, 3 },
    { 0, 0 },
    { 0xfcf8c965, 3 },
    { 0, 0 },
    { 0xf0c0c967, 3 },
    { 0xe1d8c968, 3 },
    { 0xfd54c969, 4 },
    { 0, 0 },
    { 0xacc0c96b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf390c96e, 4 },
    { 0x892cc96f, 3 },
    { 0xac94c970, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd0c975, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dcc979, 3 },
    { 0x479cc97a, 3 },
    { 0x99f0c97b, 4 },
    { 0xac8c97c, 3 },
    { 0xa86cc97c, 3 },
    { 0x29bcc97e, 3 },
    { 0xb548c97f, 3 },
    { 0xfc90c979, 3 },
    { 0x2684c97c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b50c98b, 3 },
    { 0x3e54c98c, 3 },
    { 0x2d78c98b, 3 },
    { 0xdffcc98c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc8c991, 4 },
    { 0, 0 },
    { 0xe704c993, 3 },
    { 0, 0 },
    { 0x3660c995, 3 },
    { 0, 0 },
    { 0xce34c997, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982cc9a1, 3 },
    { 0x771cc9a2, 3 },
    { 0xe614c9a3, 3 },
    { 0xfa70c9a4, 4 },
    { 0x3f70c9a3, 4 },
    { 0xaa00c9a5, 3 },
    { 0x2864c9a3, 3 },
    { 0x8bb0c9a8, 4 },
    { 0x1428c9a8, 5 },
    { 0x277cc9aa, 3 },
    { 0x454cc9a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf918c9b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x898c9b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5650c9bb, 3 },
    { 0xaa28c9bb, 4 },
    { 0, 0 },
    { 0xe72cc9be, 3 },
    { 0, 0 },
    { 0x6810c9c0, 3 },
    { 0xc9acc9c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c4c9c4, 3 },
    { 0x59e4c9c5, 4 },
    { 0xe65cc9c4, 3 },
    { 0x5110c9c6, 4 },
    { 0x4decc9c8, 4 },
    { 0, 0 },
    { 0xd2bcc9ca, 3 },
    { 0xd6d8c9cb, 3 },
    { 0xb204c9cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664cc9d3, 3 },
    { 0x83c0c9d4, 3 },
    { 0x9a98c9d4, 3 },
    { 0xc100c9d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8984c9de, 3 },
    { 0xffa0c9df, 3 },
    { 0, 0 },
    { 0xa88c9e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae4c9e4, 4 },
    { 0, 0 },
    { 0x965cc9e6, 4 },
    { 0xaf8cc9e7, 3 },
    { 0x929cc9e7, 3 },
    { 0x22cc9e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c0c9ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb524c9f1, 3 },
    { 0xb53cc9f2, 5 },
    { 0x2990c9f3, 3 },
    { 0xd4acc9f3, 4 },
    { 0xe040c9f2, 3 },
    { 0x6274c9f6, 5 },
    { 0x2e8c9f3, 3 },
    { 0x20b4c9f3, 3 },
    { 0x9448c9f6, 3 },
    { 0x1978c9f9, 5 },
    { 0, 0 },
    { 0xf95cc9fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b0c9ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc888ca03, 3 },
    { 0xbecca04, 3 },
    { 0, 0 },
    { 0xcb4cca06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1220ca0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4facca0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cca12, 4 },
    { 0xf214ca12, 3 },
    { 0x9b5cca14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f4ca1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d8ca1f, 3 },
    { 0x98f4ca1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cccca2a, 4 },
    { 0, 0 },
    { 0xb9a4ca2c, 3 },
    { 0, 0 },
    { 0xa064ca2e, 3 },
    { 0x52e4ca2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c4ca33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa080ca37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf48ca3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8ca43, 3 },
    { 0x628cca43, 3 },
    { 0x4e84ca45, 3 },
    { 0xdcccca45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7cca4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac8ca4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8accca52, 4 },
    { 0x1a68ca53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350ca58, 3 },
    { 0x280cca59, 3 },
    { 0, 0 },
    { 0xe56cca5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c4ca5e, 4 },
    { 0xcd4cca5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e8ca62, 4 },
    { 0, 0 },
    { 0xf7a8ca64, 3 },
    { 0x3c1cca65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc0ca69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750ca74, 3 },
    { 0, 0 },
    { 0x8554ca76, 4 },
    { 0xbd2cca77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e8ca7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a54ca80, 4 },
    { 0xb9a8ca80, 4 },
    { 0x16acca82, 3 },
    { 0x76f0ca83, 3 },
    { 0, 0 },
    { 0x8f08ca85, 3 },
    { 0x9190ca85, 3 },
    { 0xbd08ca86, 3 },
    { 0xf550ca88, 3 },
    { 0, 0 },
    { 0x2650ca8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5cca8d, 3 },
    { 0x5f00ca8e, 3 },
    { 0x8328ca8e, 4 },
    { 0x2170ca8f, 3 },
    { 0x6cf0ca91, 3 },
    { 0xbf3cca92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf274ca97, 4 },
    { 0x1ed8ca98, 3 },
    { 0x54cca97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20ca9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45cca9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb4caa2, 3 },
    { 0, 0 },
    { 0xa3cccaa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb720caa7, 3 },
    { 0xa580caa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc938caac, 3 },
    { 0x254caad, 3 },
    { 0x4220caae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34cab4, 3 },
    { 0, 0 },
    { 0xbf98cab6, 4 },
    { 0x1b30cab6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa750cabf, 3 },
    { 0x1888cac0, 3 },
    { 0xe988cac0, 4 },
    { 0, 0 },
    { 0x2dbccac3, 4 },
    { 0x887ccac4, 4 },
    { 0, 0 },
    { 0x601ccac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724cad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e4cad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0cade, 3 },
    { 0xd304cade, 3 },
    { 0, 0 },
    { 0x69d0cae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c4cae5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103ccae9, 4 },
    { 0, 0 },
    { 0xd4fccaeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa324caee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3ccaf1, 4 },
    { 0x576ccaf2, 3 },
    { 0x14b0caf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce0caf7, 3 },
    { 0xc354caf7, 4 },
    { 0, 0 },
    { 0x3f84cafa, 3 },
    { 0xef88cafb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58caff, 3 },
    { 0x7a6ccb00, 4 },
    { 0x3720cb00, 3 },
    { 0x68c0cb02, 3 },
    { 0x6ecccb03, 3 },
    { 0, 0 },
    { 0x8fe0cb05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0cb0c, 4 },
    { 0xc658cb0d, 3 },
    { 0, 0 },
    { 0xd9f8cb0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b4cb12, 4 },
    { 0, 0 },
    { 0x2004cb14, 3 },
    { 0, 0 },
    { 0x55b4cb16, 3 },
    { 0xab58cb16, 3 },
    { 0x3d88cb18, 3 },
    { 0, 0 },
    { 0x88c8cb1a, 3 },
    { 0x5768cb1b, 3 },
    { 0x40f8cb1c, 4 },
    { 0xcedccb1d, 3 },
    { 0, 0 },
    { 0x6eeccb1f, 4 },
    { 0, 0 },
    { 0x9994cb21, 3 },
    { 0, 0 },
    { 0x8d28cb23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a8cb27, 3 },
    { 0, 0 },
    { 0xefdccb29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf8cb2f, 4 },
    { 0xf2accb30, 3 },
    { 0x2674cb31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84cb39, 3 },
    { 0x80dccb3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe84cb3d, 3 },
    { 0xc408cb3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace8cb43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440cb47, 3 },
    { 0x4998cb47, 3 },
    { 0x1d60cb49, 3 },
    { 0, 0 },
    { 0xeb80cb4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf840cb4e, 3 },
    { 0x3d24cb4f, 3 },
    { 0x838ccb50, 3 },
    { 0, 0 },
    { 0x7ee4cb52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a8cb57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4044cb5a, 3 },
    { 0x90c4cb5b, 3 },
    { 0x748cb5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa064cb60, 3 },
    { 0, 0 },
    { 0x5ac8cb62, 3 },
    { 0x8d08cb62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0cb66, 3 },
    { 0, 0 },
    { 0x7338cb68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e0cb6b, 3 },
    { 0, 0 },
    { 0x8f88cb6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcccb70, 4 },
    { 0, 0 },
    { 0x2508cb72, 3 },
    { 0x1094cb73, 3 },
    { 0, 0 },
    { 0x4e04cb75, 3 },
    { 0x5cc4cb75, 3 },
    { 0x8d58cb77, 4 },
    { 0xd38cb78, 4 },
    { 0xaafccb75, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26ccb7d, 4 },
    { 0x708ccb7e, 3 },
    { 0x858ccb7f, 3 },
    { 0, 0 },
    { 0x2d60cb81, 4 },
    { 0, 0 },
    { 0xe428cb83, 3 },
    { 0xc0d4cb84, 4 },
    { 0x3e34cb85, 3 },
    { 0, 0 },
    { 0x6b5ccb87, 3 },
    { 0, 0 },
    { 0x60b4cb89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a90cb93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x724cb96, 3 },
    { 0x81dccb97, 4 },
    { 0xb284cb97, 3 },
    { 0x5310cb99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e04cb9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc468cba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdccba3, 5 },
    { 0xf984cba4, 3 },
    { 0xba24cba5, 4 },
    { 0x35b0cba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd590cba9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b8cbb0, 3 },
    { 0xc7dccbb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875ccbb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8cbbd, 4 },
    { 0x9aeccbbd, 4 },
    { 0xc3dccbbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab18cbc3, 3 },
    { 0, 0 },
    { 0xdb28cbc5, 3 },
    { 0xe0bccbc6, 3 },
    { 0x36e0cbc5, 3 },
    { 0, 0 },
    { 0xff94cbc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554cbd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd848cbdc, 3 },
    { 0x70cccbdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1140cbe2, 3 },
    { 0x42dccbe3, 4 },
    { 0xe168cbe2, 3 },
    { 0, 0 },
    { 0xca38cbe6, 3 },
    { 0x4a94cbe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x508cbea, 3 },
    { 0x8d08cbeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904cbf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8cbf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408cbf6, 4 },
    { 0, 0 },
    { 0xcfc8cbf8, 4 },
    { 0x533ccbf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da4cc01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ccc04, 3 },
    { 0x2454cc05, 4 },
    { 0x64eccc05, 3 },
    { 0x92f4cc07, 3 },
    { 0xbe60cc08, 5 },
    { 0, 0 },
    { 0x3fc4cc0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a8cc0d, 3 },
    { 0x5568cc0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf784cc13, 4 },
    { 0x78d4cc13, 3 },
    { 0, 0 },
    { 0x6764cc16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80cc19, 3 },
    { 0, 0 },
    { 0x18f0cc1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde18cc21, 3 },
    { 0xfc8ccc22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9824cc27, 3 },
    { 0xcf54cc27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0cc2c, 3 },
    { 0xba78cc2d, 3 },
    { 0xcda8cc2e, 4 },
    { 0x8284cc2f, 3 },
    { 0x82a4cc2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ccc34, 3 },
    { 0x5844cc35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2004cc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc988cc3e, 3 },
    { 0, 0 },
    { 0xb768cc40, 3 },
    { 0x1168cc41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4cc45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8cc4b, 3 },
    { 0x8328cc4b, 3 },
    { 0, 0 },
    { 0x2a94cc4e, 3 },
    { 0, 0 },
    { 0x3c20cc50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a8cc53, 3 },
    { 0x91e4cc54, 3 },
    { 0, 0 },
    { 0x257ccc56, 4 },
    { 0x5e94cc56, 3 },
    { 0, 0 },
    { 0x5f00cc59, 5 },
    { 0xe650cc5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd030cc61, 4 },
    { 0xc30cc62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f0cc68, 3 },
    { 0xb8f4cc69, 3 },
    { 0x25ccc68, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfccc6d, 4 },
    { 0x340ccc6d, 3 },
    { 0xbb44cc6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca08cc7d, 3 },
    { 0x3e28cc7e, 4 },
    { 0x5d4ccc7e, 3 },
    { 0, 0 },
    { 0x88c8cc81, 4 },
    { 0x59d8cc82, 3 },
    { 0, 0 },
    { 0xd880cc84, 3 },
    { 0, 0 },
    { 0xc094cc86, 3 },
    { 0x1af8cc87, 4 },
    { 0x83d4cc88, 3 },
    { 0xdd2ccc89, 3 },
    { 0x2274cc8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2970cc8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c0cc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6204cca0, 3 },
    { 0xe308cca0, 4 },
    { 0xad80cca2, 3 },
    { 0xbbb0cca3, 4 },
    { 0, 0 },
    { 0x50b8cca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408ccad, 3 },
    { 0xf924ccad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd450ccb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b8ccb8, 3 },
    { 0, 0 },
    { 0x219cccba, 3 },
    { 0x7bd4ccbb, 4 },
    { 0x8de4ccbc, 4 },
    { 0x5a7cccbb, 3 },
    { 0x508ccbe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dacccc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f8ccc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18ccd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc0ccd6, 3 },
    { 0, 0 },
    { 0xdd04ccd8, 3 },
    { 0x5a60ccd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45ccce0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0eccce3, 3 },
    { 0x2a78cce3, 3 },
    { 0x86c4cce5, 3 },
    { 0xb7dccce4, 3 },
    { 0x96c4cce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb64cceb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4ccf0, 3 },
    { 0, 0 },
    { 0xf9d0ccf2, 3 },
    { 0, 0 },
    { 0x4100ccf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01cccfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8cccfd, 4 },
    { 0x5de4ccfd, 4 },
    { 0x9bdcccff, 4 },
    { 0xe77ccd00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab8cd08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b0cd0c, 3 },
    { 0xc830cd0c, 4 },
    { 0x4668cd0e, 3 },
    { 0xc94cd0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41ccd16, 4 },
    { 0xa588cd16, 3 },
    { 0x3810cd18, 3 },
    { 0x62f0cd18, 3 },
    { 0x585ccd1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c4cd1e, 3 },
    { 0xc40cd1f, 4 },
    { 0, 0 },
    { 0xec5ccd21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eccd26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7164cd2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80cd2e, 3 },
    { 0xdfa8cd2f, 3 },
    { 0, 0 },
    { 0x161ccd31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d30cd34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520ccd3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc484cd48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7724cd50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb20cd53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfccd58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcccd61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8cd65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7810cd68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3678cd73, 3 },
    { 0x340ccd74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f4cd7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6884cd83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb0cd8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240cd92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd638cd96, 3 },
    { 0x6e20cd97, 3 },
    { 0, 0 },
    { 0x1b70cd99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9740cd9c, 4 },
    { 0, 0 },
    { 0xd748cd9e, 4 },
    { 0x51b0cd9f, 3 },
    { 0, 0 },
    { 0x18a0cda1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbccda4, 4 },
    { 0xaafccda4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea8cdad, 4 },
    { 0x344cdad, 3 },
    { 0x9a44cdaf, 4 },
    { 0x7e44cdaf, 3 },
    { 0x9740cdb0, 3 },
    { 0x79c4cdb2, 3 },
    { 0xd33ccdaf, 3 },
    { 0xa1c4cdad, 4 },
    { 0, 0 },
    { 0x5874cdb6, 4 },
    { 0x6020cdb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x308ccdba, 3 },
    { 0xc094cdba, 3 },
    { 0x1124cdba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124ccdc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc8cdc4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb160cdc7, 3 },
    { 0x1778cdc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84cdcd, 4 },
    { 0, 0 },
    { 0x5200cdcf, 3 },
    { 0xd35ccdcf, 3 },
    { 0, 0 },
    { 0x3764cdd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d8cdd7, 4 },
    { 0xb2a8cdd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9408cddc, 3 },
    { 0xffe8cddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4420cde0, 3 },
    { 0x6ceccde0, 4 },
    { 0, 0 },
    { 0xcc20cde3, 4 },
    { 0x6c4cde4, 3 },
    { 0, 0 },
    { 0x2904cde6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e4cde9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d0cdee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d84cdf3, 4 },
    { 0x5baccdf4, 4 },
    { 0xbadccdf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ccdf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7accdfd, 3 },
    { 0xe200cdfd, 3 },
    { 0, 0 },
    { 0xdb64ce00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb4ce03, 3 },
    { 0xccccce04, 4 },
    { 0x8ff8ce05, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd0ce08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a90ce0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438ce0e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a4ce11, 3 },
    { 0x8fe8ce11, 4 },
    { 0, 0 },
    { 0xf770ce14, 3 },
    { 0, 0 },
    { 0x6bc8ce16, 3 },
    { 0xe898ce17, 3 },
    { 0x4f64ce16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6434ce1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fcce1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc8ce28, 4 },
    { 0x40b8ce29, 3 },
    { 0, 0 },
    { 0x7cf8ce2b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8210ce31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94ce35, 3 },
    { 0, 0 },
    { 0xab90ce37, 3 },
    { 0, 0 },
    { 0xc664ce39, 3 },
    { 0xa51cce3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668ce3e, 3 },
    { 0x512cce3e, 3 },
    { 0x6f1cce40, 4 },
    { 0x8a04ce41, 3 },
    { 0x7d2cce41, 4 },
    { 0, 0 },
    { 0x5258ce44, 3 },
    { 0, 0 },
    { 0xeb20ce46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e0ce4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c90ce50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe034ce53, 4 },
    { 0xf858ce54, 3 },
    { 0xe978ce55, 3 },
    { 0x2ad0ce56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8710ce5c, 4 },
    { 0x6c94ce5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcce62, 4 },
    { 0x8a38ce63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b88ce6c, 3 },
    { 0, 0 },
    { 0x2898ce6e, 3 },
    { 0x5f94ce6f, 3 },
    { 0x55a8ce6e, 3 },
    { 0xf2ccce71, 3 },
    { 0x7b84ce72, 3 },
    { 0xc9ecce72, 3 },
    { 0x9050ce74, 3 },
    { 0x2720ce75, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4ce82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5310ce88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a0ce97, 3 },
    { 0x5ba4ce98, 3 },
    { 0xc72cce99, 3 },
    { 0xb424ce9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28cce9e, 3 },
    { 0, 0 },
    { 0xc634cea0, 3 },
    { 0x3d58cea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x378ccea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d4ceab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a10ceb5, 4 },
    { 0, 0 },
    { 0x3f0ceb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2328cebb, 3 },
    { 0x9760cebb, 3 },
    { 0x9180cebd, 3 },
    { 0xe864cebe, 4 },
    { 0x20eccebc, 3 },
    { 0x414cec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a0cec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa480ceca, 4 },
    { 0x175cceca, 3 },
    { 0x8a70cecc, 3 },
    { 0xf094cecd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f4ced3, 3 },
    { 0, 0 },
    { 0x8938ced5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5180cedb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc64cede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6444cee3, 3 },
    { 0x32c4cee4, 5 },
    { 0, 0 },
    { 0xe8accee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfcceea, 3 },
    { 0xee18ceea, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dccefd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ccf02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8cf0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dccf10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1ccf14, 3 },
    { 0xb818cf15, 3 },
    { 0x7ac8cf16, 3 },
    { 0x9cf0cf17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e98cf1d, 3 },
    { 0xf670cf1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc0cf28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbccf2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3ccf2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8cf34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c8cf39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b8cf3c, 3 },
    { 0xce3ccf3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408cf42, 3 },
    { 0x7f44cf43, 4 },
    { 0x1198cf44, 3 },
    { 0xc080cf45, 3 },
    { 0xc4d4cf46, 4 },
    { 0x934cf42, 4 },
    { 0xa67ccf48, 5 },
    { 0x3cc0cf48, 3 },
    { 0xd84ccf4a, 3 },
    { 0x94ccf4a, 3 },
    { 0, 0 },
    { 0xcfbccf4d, 3 },
    { 0, 0 },
    { 0x8268cf4f, 3 },
    { 0xa6c0cf50, 3 },
    { 0xe3b4cf4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abccf54, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa98cf58, 4 },
    { 0xcb18cf58, 3 },
    { 0xde80cf5a, 3 },
    { 0xd804cf5a, 3 },
    { 0x168cf5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf064cf63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cccf6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5424cf72, 4 },
    { 0x3a14cf73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ccf78, 3 },
    { 0xa52ccf78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa774cf80, 3 },
    { 0, 0 },
    { 0x8cf0cf82, 3 },
    { 0x9faccf83, 3 },
    { 0x5eaccf83, 3 },
    { 0x1338cf82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5934cf8a, 3 },
    { 0x81f8cf8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e04cf97, 4 },
    { 0, 0 },
    { 0xd2accf99, 5 },
    { 0, 0 },
    { 0x1deccf9b, 3 },
    { 0, 0 },
    { 0x45e0cf9d, 3 },
    { 0xa48cf9e, 3 },
    { 0, 0 },
    { 0x5988cfa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c8cfa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4208cfab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf814cfb1, 4 },
    { 0x58c0cfb1, 3 },
    { 0x367ccfb1, 3 },
    { 0, 0 },
    { 0x5324cfb5, 3 },
    { 0, 0 },
    { 0x7b54cfb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f4cfbb, 3 },
    { 0xa968cfbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e74cfc0, 4 },
    { 0xec20cfc1, 3 },
    { 0x7018cfc2, 3 },
    { 0xdda8cfc3, 4 },
    { 0x27e8cfc1, 3 },
    { 0x37f0cfc4, 3 },
    { 0, 0 },
    { 0xb744cfc7, 4 },
    { 0, 0 },
    { 0xcabccfc9, 3 },
    { 0, 0 },
    { 0xc7a0cfcb, 3 },
    { 0xf7accfcc, 3 },
    { 0xff18cfcc, 4 },
    { 0, 0 },
    { 0x4b54cfcf, 3 },
    { 0xd2e4cfd0, 3 },
    { 0x5a7ccfcf, 3 },
    { 0x9f30cfd2, 3 },
    { 0xe818cfd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa394cfd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5420cfda, 3 },
    { 0x6974cfda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e8cfe5, 3 },
    { 0x71a8cfe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91ccfe9, 3 },
    { 0x4f80cfea, 3 },
    { 0x8094cfeb, 3 },
    { 0, 0 },
    { 0x51e0cfed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc0cff0, 4 },
    { 0, 0 },
    { 0xfe50cff2, 3 },
    { 0x90ccff3, 3 },
    { 0xee8ccff3, 3 },
    { 0xde88cff2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ccff8, 3 },
    { 0xa528cff9, 4 },
    { 0, 0 },
    { 0xc540cffb, 4 },
    { 0x5d00cffc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f28d000, 3 },
    { 0x8db0d001, 3 },
    { 0x5164d000, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45cd006, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa64d009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e0d00e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24d011, 3 },
    { 0x53dcd011, 3 },
    { 0x46ecd013, 3 },
    { 0xdef4d013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f0d017, 3 },
    { 0x559cd017, 3 },
    { 0xd310d017, 4 },
    { 0, 0 },
    { 0xa39cd01b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0d021, 3 },
    { 0x99ccd022, 3 },
    { 0xfa58d022, 3 },
    { 0x8284d021, 3 },
    { 0xb3dcd023, 3 },
    { 0xf4b4d026, 3 },
    { 0x72a8d027, 3 },
    { 0, 0 },
    { 0x1e34d029, 3 },
    { 0xb500d02a, 3 },
    { 0x1b34d02a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d032, 4 },
    { 0x9f4cd032, 4 },
    { 0x6d68d032, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dcd039, 4 },
    { 0, 0 },
    { 0x1e44d03b, 4 },
    { 0, 0 },
    { 0xb55cd03d, 3 },
    { 0xfb64d03e, 4 },
    { 0xe940d03f, 3 },
    { 0xcf5cd040, 3 },
    { 0x7b0cd040, 3 },
    { 0x9be4d03d, 3 },
    { 0x229cd043, 3 },
    { 0, 0 },
    { 0xb764d045, 3 },
    { 0x72c4d046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34cd04a, 4 },
    { 0, 0 },
    { 0x3440d04c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda48d058, 3 },
    { 0x664cd058, 5 },
    { 0, 0 },
    { 0xcdc0d05b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x951cd05e, 3 },
    { 0xca4cd05f, 3 },
    { 0x77f0d060, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4d063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d067, 3 },
    { 0x8b1cd067, 4 },
    { 0x9008d067, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b0d06c, 3 },
    { 0x6d8cd06d, 3 },
    { 0xa728d06e, 5 },
    { 0, 0 },
    { 0x63f0d070, 3 },
    { 0xa158d070, 4 },
    { 0xaee0d070, 3 },
    { 0x5a4cd073, 5 },
    { 0, 0 },
    { 0xa448d075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ecd081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe34d084, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5cd088, 3 },
    { 0x1180d089, 3 },
    { 0x1e34d08a, 3 },
    { 0xb8b8d08a, 4 },
    { 0x6be0d08c, 4 },
    { 0x1d00d08d, 4 },
    { 0x1dd0d08b, 3 },
    { 0x15d0d08f, 3 },
    { 0x9270d08f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d4d095, 3 },
    { 0x8534d095, 4 },
    { 0x5facd097, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1414d09a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901cd09e, 3 },
    { 0, 0 },
    { 0xde30d0a0, 4 },
    { 0x9ba4d0a0, 3 },
    { 0x45ecd0a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba08d0a6, 4 },
    { 0, 0 },
    { 0x6390d0a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196cd0ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb340d0b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8d0b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801cd0bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb34d0c0, 3 },
    { 0, 0 },
    { 0x56a8d0c2, 3 },
    { 0xf6ecd0c3, 4 },
    { 0x58e8d0c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b90d0c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b8d0cd, 4 },
    { 0xa47cd0ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1114d0d1, 3 },
    { 0x9a10d0d1, 3 },
    { 0, 0 },
    { 0xb06cd0d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f50d0dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf744d0df, 4 },
    { 0x1a6cd0e0, 5 },
    { 0xfce0d0e0, 3 },
    { 0x4cc4d0e2, 3 },
    { 0, 0 },
    { 0x71dcd0e4, 3 },
    { 0x6e7cd0e5, 4 },
    { 0xb6ecd0e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9cd0ef, 3 },
    { 0x7584d0f0, 3 },
    { 0x36cd0f0, 4 },
    { 0xb0b8d0f0, 3 },
    { 0x1b94d0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x218cd0f6, 3 },
    { 0, 0 },
    { 0xaf10d0f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf0d0fb, 3 },
    { 0xbb8cd0fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfcd101, 3 },
    { 0x4c14d102, 3 },
    { 0, 0 },
    { 0x8094d104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698d109, 3 },
    { 0xf9b0d10a, 4 },
    { 0x36fcd10b, 4 },
    { 0, 0 },
    { 0xffe8d10d, 4 },
    { 0x6554d10d, 3 },
    { 0, 0 },
    { 0x6600d110, 3 },
    { 0x3bb4d110, 3 },
    { 0x9a18d111, 3 },
    { 0x5e20d113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccd117, 4 },
    { 0x6e58d117, 3 },
    { 0xd21cd117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf448d11c, 3 },
    { 0xab0cd11d, 3 },
    { 0x512cd11e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7980d123, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1408d129, 4 },
    { 0x272cd129, 3 },
    { 0x354d129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fcd12e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf24d132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cecd138, 3 },
    { 0xd22cd139, 3 },
    { 0x8d14d13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650cd140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4d143, 4 },
    { 0, 0 },
    { 0xce0d145, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6694d149, 3 },
    { 0xf330d149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3cd14d, 3 },
    { 0x76d4d14e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078d153, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4d157, 3 },
    { 0xc014d157, 3 },
    { 0xe208d159, 3 },
    { 0, 0 },
    { 0x4730d15b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78d15e, 3 },
    { 0, 0 },
    { 0x2f0cd160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5600d163, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb4d16c, 3 },
    { 0xef7cd16c, 3 },
    { 0x9420d16e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634cd174, 3 },
    { 0, 0 },
    { 0x6df0d176, 4 },
    { 0x6e44d177, 3 },
    { 0, 0 },
    { 0x244cd179, 3 },
    { 0x2280d17a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4becd180, 4 },
    { 0x57f8d180, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704d184, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4cd187, 3 },
    { 0x7920d188, 3 },
    { 0, 0 },
    { 0xf73cd18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca0d18e, 3 },
    { 0, 0 },
    { 0x593cd190, 3 },
    { 0, 0 },
    { 0xee28d192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604d196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb860d19a, 3 },
    { 0x54d19b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354d19e, 3 },
    { 0, 0 },
    { 0x7c04d1a0, 3 },
    { 0xd76cd1a0, 3 },
    { 0x6a08d1a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a30d1a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38d1ad, 3 },
    { 0, 0 },
    { 0x24a0d1af, 3 },
    { 0x3350d1b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d4d1b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce54d1b8, 3 },
    { 0xbd34d1b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fcd1bd, 4 },
    { 0x86a0d1bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1424d1c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ccd1c5, 3 },
    { 0x6430d1c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6cd1cb, 3 },
    { 0, 0 },
    { 0xd684d1cd, 3 },
    { 0xffdcd1ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9cd1d2, 3 },
    { 0, 0 },
    { 0x56f8d1d4, 3 },
    { 0x95a4d1d5, 4 },
    { 0xd2bcd1d4, 4 },
    { 0x62c8d1d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1de, 3 },
    { 0, 0 },
    { 0x7654d1e0, 3 },
    { 0x80acd1e1, 3 },
    { 0xea6cd1e0, 3 },
    { 0xba0cd1e3, 3 },
    { 0xc328d1e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758d1e8, 3 },
    { 0x43c0d1e8, 3 },
    { 0, 0 },
    { 0x4b9cd1eb, 3 },
    { 0x56c0d1ec, 5 },
    { 0xbcecd1ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffcd1f0, 3 },
    { 0x4290d1f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c0d1f6, 3 },
    { 0, 0 },
    { 0x78d0d1f8, 3 },
    { 0, 0 },
    { 0xe3a8d1fa, 3 },
    { 0xf10cd1fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a4d200, 3 },
    { 0, 0 },
    { 0x7ad4d202, 3 },
    { 0xebd0d203, 3 },
    { 0x4890d204, 3 },
    { 0, 0 },
    { 0x7e48d206, 4 },
    { 0xa14cd206, 4 },
    { 0xd39cd208, 3 },
    { 0, 0 },
    { 0xcd74d20a, 3 },
    { 0, 0 },
    { 0xc688d20c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48d210, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf420d213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04cd216, 3 },
    { 0, 0 },
    { 0xf194d218, 3 },
    { 0xc268d219, 3 },
    { 0x8d28d21a, 3 },
    { 0, 0 },
    { 0x28f4d21c, 3 },
    { 0xe92cd21d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bcd222, 4 },
    { 0, 0 },
    { 0x1c1cd224, 3 },
    { 0x247cd225, 4 },
    { 0, 0 },
    { 0x3a10d227, 3 },
    { 0x8de8d228, 3 },
    { 0xcd0cd228, 3 },
    { 0xe584d228, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4424d231, 3 },
    { 0xdef4d232, 3 },
    { 0, 0 },
    { 0xd3ecd234, 3 },
    { 0x5ab8d235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6becd238, 3 },
    { 0x7558d239, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70cd241, 3 },
    { 0, 0 },
    { 0xe2c0d243, 4 },
    { 0xb43cd243, 3 },
    { 0xc914d245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b28d249, 3 },
    { 0xae14d24a, 3 },
    { 0x5e74d24b, 3 },
    { 0x730cd24c, 3 },
    { 0, 0 },
    { 0x78a4d24e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf8d251, 4 },
    { 0x6160d252, 3 },
    { 0xc580d252, 3 },
    { 0xa624d254, 3 },
    { 0, 0 },
    { 0x27c4d256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d4d261, 3 },
    { 0, 0 },
    { 0xee88d263, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ecd266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ecd26b, 4 },
    { 0x6a90d26c, 3 },
    { 0xa244d26d, 3 },
    { 0x580cd26e, 4 },
    { 0xf394d26d, 3 },
    { 0xfa4cd270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5cd277, 3 },
    { 0, 0 },
    { 0x1dd4d279, 3 },
    { 0x7300d27a, 4 },
    { 0x61d8d27b, 4 },
    { 0, 0 },
    { 0xb17cd27d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72acd283, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b08d286, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000d289, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1604d28c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x182cd28f, 3 },
    { 0, 0 },
    { 0xe400d291, 3 },
    { 0x5a68d291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40cd297, 4 },
    { 0x56f4d298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc008d29d, 3 },
    { 0, 0 },
    { 0xd3bcd29f, 3 },
    { 0x942cd29f, 3 },
    { 0x6994d29f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3650d2a4, 3 },
    { 0, 0 },
    { 0x97e0d2a6, 3 },
    { 0x3ca4d2a6, 3 },
    { 0, 0 },
    { 0x2c54d2a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2654d2ae, 3 },
    { 0, 0 },
    { 0xb90cd2b0, 3 },
    { 0x4e3cd2b1, 4 },
    { 0xc5c4d2b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5758d2b7, 3 },
    { 0x6568d2b8, 4 },
    { 0x5994d2b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d24d2bf, 3 },
    { 0xb0b0d2bf, 4 },
    { 0x55f0d2c0, 3 },
    { 0, 0 },
    { 0x47d8d2c3, 4 },
    { 0, 0 },
    { 0xd95cd2c5, 3 },
    { 0, 0 },
    { 0xde8cd2c7, 4 },
    { 0xae88d2c8, 4 },
    { 0x6c90d2c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130d2cd, 3 },
    { 0, 0 },
    { 0xcb34d2cf, 4 },
    { 0x949cd2d0, 3 },
    { 0x55d0d2cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b4d2d5, 3 },
    { 0, 0 },
    { 0x2eecd2d7, 3 },
    { 0x149cd2d7, 3 },
    { 0xa208d2d9, 3 },
    { 0xf0b0d2da, 4 },
    { 0, 0 },
    { 0x1e44d2dc, 3 },
    { 0x159cd2dd, 3 },
    { 0x4158d2de, 3 },
    { 0, 0 },
    { 0xce8cd2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f94d2e7, 3 },
    { 0x830cd2e8, 4 },
    { 0xe598d2e9, 3 },
    { 0, 0 },
    { 0x2490d2eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c88d2ee, 3 },
    { 0, 0 },
    { 0x2718d2f0, 3 },
    { 0xeeb4d2f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8d2ff, 4 },
    { 0, 0 },
    { 0x6238d301, 3 },
    { 0x4fc8d302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369cd306, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604d30d, 5 },
    { 0, 0 },
    { 0x7ffcd30f, 4 },
    { 0x71f4d310, 3 },
    { 0x4cecd311, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04d31f, 3 },
    { 0x1bcd31f, 3 },
    { 0, 0 },
    { 0xf89cd322, 3 },
    { 0, 0 },
    { 0xd20d324, 3 },
    { 0x3370d324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ecd328, 3 },
    { 0x7d6cd328, 3 },
    { 0, 0 },
    { 0x6e70d32b, 3 },
    { 0x6ee4d32c, 3 },
    { 0xc30d32d, 3 },
    { 0, 0 },
    { 0x30e8d32f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c4d333, 3 },
    { 0x41d0d334, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9cd343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd4d348, 4 },
    { 0x68c4d348, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd428d350, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf4d353, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x332cd356, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8cd35b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e4d362, 3 },
    { 0xbacd363, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a8d366, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8d36c, 5 },
    { 0, 0 },
    { 0x4030d36e, 4 },
    { 0, 0 },
    { 0xaaf8d370, 4 },
    { 0, 0 },
    { 0xdcd8d372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb648d375, 4 },
    { 0, 0 },
    { 0xd01cd377, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e4d37a, 4 },
    { 0x554cd37a, 3 },
    { 0, 0 },
    { 0x238d37d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceacd382, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa8d385, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f64d38a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1224d38e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac78d391, 4 },
    { 0xae8cd392, 3 },
    { 0x7120d392, 4 },
    { 0xd31cd393, 3 },
    { 0, 0 },
    { 0xf05cd396, 3 },
    { 0xa650d396, 3 },
    { 0x852cd398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc520d39b, 4 },
    { 0x1bd0d39c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018d3a2, 4 },
    { 0, 0 },
    { 0x32c4d3a4, 3 },
    { 0xdcb4d3a5, 4 },
    { 0xdb80d3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf40d3a9, 3 },
    { 0, 0 },
    { 0xbb68d3ab, 3 },
    { 0x2500d3ac, 3 },
    { 0xe3a0d3ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2274d3b3, 4 },
    { 0, 0 },
    { 0x1774d3b5, 3 },
    { 0xafe4d3b5, 3 },
    { 0x927cd3b7, 3 },
    { 0, 0 },
    { 0xc958d3b9, 3 },
    { 0x3a60d3ba, 3 },
    { 0x14a8d3bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f8d3c0, 3 },
    { 0x29a0d3c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18cd3c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1cd3c8, 3 },
    { 0, 0 },
    { 0xdaecd3ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0d3cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110cd3d4, 3 },
    { 0x761cd3d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bcd3d8, 3 },
    { 0x3ee8d3d9, 3 },
    { 0x578d3da, 3 },
    { 0x38f8d3db, 3 },
    { 0x2040d3dc, 3 },
    { 0x2950d3dd, 3 },
    { 0xed14d3d9, 3 },
    { 0, 0 },
    { 0xe844d3e0, 3 },
    { 0x22ccd3e1, 3 },
    { 0x9160d3e2, 4 },
    { 0xf9bcd3e1, 3 },
    { 0xec58d3e4, 3 },
    { 0, 0 },
    { 0xc54cd3e6, 3 },
    { 0, 0 },
    { 0x692cd3e8, 3 },
    { 0, 0 },
    { 0x1c34d3ea, 3 },
    { 0, 0 },
    { 0x66a8d3ec, 3 },
    { 0xfb44d3ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1cd3f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb930d3f4, 3 },
    { 0x6d60d3f4, 3 },
    { 0x8504d3f5, 4 },
    { 0xa178d3f7, 4 },
    { 0, 0 },
    { 0x2af0d3f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490d3fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaacd401, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda80d406, 3 },
    { 0, 0 },
    { 0x5608d408, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde8d40d, 3 },
    { 0xf2b8d40d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dcd411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a8d416, 3 },
    { 0, 0 },
    { 0x70a0d418, 3 },
    { 0, 0 },
    { 0xa3d8d41a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b98d41e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4784d424, 3 },
    { 0xced4d425, 4 },
    { 0xdef0d425, 3 },
    { 0xfe8d427, 5 },
    { 0xba20d425, 3 },
    { 0x3f4d428, 3 },
    { 0x7904d424, 3 },
    { 0xcd04d42a, 4 },
    { 0x9334d42c, 3 },
    { 0, 0 },
    { 0xbf80d42e, 3 },
    { 0xfa3cd42e, 3 },
    { 0x982cd430, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a58d434, 3 },
    { 0, 0 },
    { 0xda5cd436, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ecd43d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ccd440, 3 },
    { 0, 0 },
    { 0x8e60d442, 3 },
    { 0x7ed4d442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653cd44b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf04d450, 3 },
    { 0, 0 },
    { 0xb98d452, 3 },
    { 0x5290d452, 4 },
    { 0, 0 },
    { 0x3404d455, 3 },
    { 0x29dcd456, 3 },
    { 0xc30cd457, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b40d45c, 3 },
    { 0, 0 },
    { 0x18a0d45e, 3 },
    { 0x6f00d45f, 3 },
    { 0x2ed4d45e, 3 },
    { 0xa58d45f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55cd464, 3 },
    { 0x8d9cd465, 3 },
    { 0, 0 },
    { 0x5bc8d467, 3 },
    { 0xaec8d468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018d46c, 3 },
    { 0xec38d46d, 3 },
    { 0x41e4d46d, 3 },
    { 0x7e8cd46d, 3 },
    { 0, 0 },
    { 0x8588d471, 3 },
    { 0xa05cd472, 3 },
    { 0, 0 },
    { 0x3f64d474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d98d478, 3 },
    { 0, 0 },
    { 0xd510d47a, 4 },
    { 0x6cf8d47b, 4 },
    { 0, 0 },
    { 0x4b8cd47d, 4 },
    { 0xb41cd47e, 3 },
    { 0x5518d47f, 3 },
    { 0xd528d47f, 3 },
    { 0x7354d481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2984d486, 3 },
    { 0x90b8d486, 3 },
    { 0x48f4d488, 3 },
    { 0xfdf4d486, 3 },
    { 0, 0 },
    { 0xb210d48b, 3 },
    { 0xd6e4d48c, 3 },
    { 0x1d84d48b, 4 },
    { 0xe1acd48e, 4 },
    { 0xbcb0d48f, 3 },
    { 0x824d48f, 3 },
    { 0x145cd48f, 3 },
    { 0x20e8d492, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5048d495, 3 },
    { 0x9c50d495, 4 },
    { 0, 0 },
    { 0x304d498, 3 },
    { 0, 0 },
    { 0x2cbcd49a, 3 },
    { 0, 0 },
    { 0xe198d49c, 3 },
    { 0x96a8d49d, 5 },
    { 0x8bf0d49e, 3 },
    { 0x63acd49f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3cd4ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9668d4b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a4d4b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17cd4ba, 3 },
    { 0x128d4bb, 3 },
    { 0xb210d4ba, 4 },
    { 0xd670d4bd, 3 },
    { 0x6134d4bc, 4 },
    { 0, 0 },
    { 0xce60d4c0, 3 },
    { 0, 0 },
    { 0xb7cd4c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2cd4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fcd4c8, 3 },
    { 0, 0 },
    { 0x101cd4ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d4cd, 3 },
    { 0, 0 },
    { 0xefe4d4cf, 3 },
    { 0xae78d4d0, 3 },
    { 0xcd38d4d1, 4 },
    { 0xe7d8d4d0, 3 },
    { 0x960cd4d0, 3 },
    { 0, 0 },
    { 0xb9cd4d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7440d4d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c28d4dd, 3 },
    { 0x98dcd4dd, 4 },
    { 0xcf78d4df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94d4e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a8d4e6, 5 },
    { 0x594d4e6, 3 },
    { 0, 0 },
    { 0x4784d4e9, 4 },
    { 0x7488d4e9, 3 },
    { 0, 0 },
    { 0x1ae4d4ec, 3 },
    { 0, 0 },
    { 0xc948d4ee, 4 },
    { 0, 0 },
    { 0xd288d4f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb180d4f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc8d4f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9680d4fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4cd507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8828d50e, 3 },
    { 0xe50d50f, 5 },
    { 0x23ecd510, 3 },
    { 0x1f0d50e, 3 },
    { 0x16c8d512, 4 },
    { 0xf8fcd513, 5 },
    { 0x3d00d513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794d518, 3 },
    { 0, 0 },
    { 0xdd10d51a, 3 },
    { 0, 0 },
    { 0x8a14d51c, 3 },
    { 0, 0 },
    { 0xbc20d51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0d521, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc34d527, 4 },
    { 0, 0 },
    { 0x6654d529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe898d52c, 3 },
    { 0x2648d52c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ecd530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f90d53d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408d543, 5 },
    { 0, 0 },
    { 0x2ce0d545, 3 },
    { 0xb1e4d545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751cd551, 3 },
    { 0x9f58d552, 3 },
    { 0x9b64d552, 4 },
    { 0xfd0d552, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa110d55a, 3 },
    { 0, 0 },
    { 0xd4f8d55c, 3 },
    { 0xde18d55c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08d561, 4 },
    { 0, 0 },
    { 0x1004d563, 3 },
    { 0, 0 },
    { 0x819cd565, 3 },
    { 0xdfcd566, 4 },
    { 0xa42cd567, 3 },
    { 0xb2c4d566, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b60d56d, 3 },
    { 0xeebcd56d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab70d572, 4 },
    { 0xa7fcd573, 5 },
    { 0, 0 },
    { 0xea9cd575, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7cd578, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cd57c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2138d580, 4 },
    { 0, 0 },
    { 0xe9b8d582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb778d585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd238d58c, 3 },
    { 0x234d58c, 4 },
    { 0, 0 },
    { 0xc738d58f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba88d596, 3 },
    { 0x3d00d596, 4 },
    { 0, 0 },
    { 0xf18cd599, 3 },
    { 0xfda4d59a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b98d5a9, 3 },
    { 0x48a4d5aa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd554d5ad, 4 },
    { 0xad14d5ad, 3 },
    { 0x3964d5af, 4 },
    { 0x6998d5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474d5b9, 3 },
    { 0xb608d5ba, 3 },
    { 0x583cd5b9, 3 },
    { 0xf870d5bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb44d5c3, 4 },
    { 0xbb34d5c3, 3 },
    { 0x23f4d5c5, 4 },
    { 0, 0 },
    { 0x162cd5c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5cd5ca, 3 },
    { 0x7ed0d5ca, 3 },
    { 0x6380d5cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c4d5d2, 3 },
    { 0, 0 },
    { 0x35fcd5d4, 3 },
    { 0x274d5d5, 3 },
    { 0x314cd5d5, 4 },
    { 0x3520d5d7, 4 },
    { 0x538cd5d7, 3 },
    { 0, 0 },
    { 0xe0acd5da, 3 },
    { 0xd6e0d5da, 3 },
    { 0x3080d5dc, 3 },
    { 0x1db8d5dc, 3 },
    { 0x8f50d5de, 4 },
    { 0x3a90d5da, 3 },
    { 0x4064d5e0, 4 },
    { 0x37e8d5de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2850d5e4, 3 },
    { 0x7dccd5e4, 3 },
    { 0x86b8d5e6, 4 },
    { 0xa1d8d5e7, 3 },
    { 0x71d8d5e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fcd5ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a4d5f6, 4 },
    { 0, 0 },
    { 0xd318d5f8, 3 },
    { 0xccc8d5f9, 4 },
    { 0x8274d5fa, 3 },
    { 0xc9a8d5f8, 4 },
    { 0xd294d5fc, 3 },
    { 0x7780d5fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7130d600, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb798d603, 4 },
    { 0x5a0d604, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa088d608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf8d60b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb678d60e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f88d616, 3 },
    { 0xe5d0d617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a4d61a, 3 },
    { 0, 0 },
    { 0x4f84d61c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b4d620, 4 },
    { 0, 0 },
    { 0xcfd0d622, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dcd626, 3 },
    { 0xcbc4d627, 3 },
    { 0xd49cd627, 3 },
    { 0, 0 },
    { 0xfb28d62a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd814d635, 3 },
    { 0x7adcd636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e0d639, 3 },
    { 0xbb24d639, 3 },
    { 0, 0 },
    { 0xa3c8d63c, 3 },
    { 0, 0 },
    { 0x2dd0d63e, 4 },
    { 0xf590d63f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6cd644, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefcd647, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a0d64c, 3 },
    { 0x1f64d64d, 3 },
    { 0x1044d64d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0d652, 3 },
    { 0x288d653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c70d656, 4 },
    { 0x60e0d657, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfccd65e, 3 },
    { 0, 0 },
    { 0xc3a8d660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299cd665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306cd66a, 3 },
    { 0, 0 },
    { 0x738cd66c, 3 },
    { 0x8edcd66d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x831cd670, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x930d673, 3 },
    { 0x8c84d674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa338d678, 3 },
    { 0x3f48d678, 3 },
    { 0x850cd67a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554d681, 3 },
    { 0, 0 },
    { 0xe224d683, 4 },
    { 0xe8b8d683, 3 },
    { 0, 0 },
    { 0x16acd686, 3 },
    { 0, 0 },
    { 0xa868d688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c8d690, 4 },
    { 0, 0 },
    { 0x80d8d692, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3244d699, 3 },
    { 0xdb10d699, 3 },
    { 0x9c0d69b, 3 },
    { 0xe5e8d69c, 3 },
    { 0x3724d69d, 3 },
    { 0x4390d69d, 3 },
    { 0xece4d69f, 3 },
    { 0, 0 },
    { 0x95d8d6a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f24d6a5, 3 },
    { 0xe90cd6a5, 3 },
    { 0x4224d6a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fcd6ab, 3 },
    { 0, 0 },
    { 0xad64d6ad, 3 },
    { 0x838d6ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4d6b3, 3 },
    { 0x5934d6b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbcd6b7, 3 },
    { 0xc36cd6b8, 3 },
    { 0x48a8d6b9, 3 },
    { 0xa78d6ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a4d6bd, 3 },
    { 0xbb28d6be, 3 },
    { 0x9134d6bd, 3 },
    { 0x70b0d6c0, 3 },
    { 0xa18d6c1, 3 },
    { 0x94cd6c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194cd6c7, 4 },
    { 0x2cacd6c8, 4 },
    { 0x3288d6c9, 3 },
    { 0x4c00d6c7, 3 },
    { 0x14ccd6cb, 3 },
    { 0, 0 },
    { 0x2134d6cd, 3 },
    { 0, 0 },
    { 0xf54d6cf, 3 },
    { 0, 0 },
    { 0xc6d0d6d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc580d6d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb8d6d9, 3 },
    { 0x82cd6da, 4 },
    { 0x6e7cd6db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c0d6de, 3 },
    { 0xdc1cd6df, 4 },
    { 0x8f90d6e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0d6e5, 3 },
    { 0, 0 },
    { 0xcd8cd6e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd820d6eb, 4 },
    { 0x9770d6ec, 3 },
    { 0xcf98d6eb, 3 },
    { 0x33a0d6eb, 4 },
    { 0xddb0d6ed, 4 },
    { 0, 0 },
    { 0xae4d6f1, 3 },
    { 0xc1f4d6f1, 4 },
    { 0xfc4d6f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab8d6f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e8d6fb, 4 },
    { 0x5ec8d6fc, 4 },
    { 0x7464d6fc, 3 },
    { 0x3024d6fb, 3 },
    { 0, 0 },
    { 0x8368d700, 4 },
    { 0, 0 },
    { 0xa1c8d702, 3 },
    { 0, 0 },
    { 0xcecd704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a34d708, 3 },
    { 0, 0 },
    { 0x710cd70a, 3 },
    { 0x1e98d70b, 3 },
    { 0, 0 },
    { 0x1e4cd70d, 4 },
    { 0xc768d70e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc500d716, 3 },
    { 0, 0 },
    { 0xfdb0d718, 3 },
    { 0xe728d719, 4 },
    { 0x7090d71a, 3 },
    { 0xf800d71b, 5 },
    { 0xe5d0d719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb768d71f, 3 },
    { 0x5560d720, 4 },
    { 0x377cd721, 3 },
    { 0x96d4d722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4d729, 4 },
    { 0x1a3cd729, 3 },
    { 0xeba4d72b, 4 },
    { 0, 0 },
    { 0xbf98d72d, 4 },
    { 0, 0 },
    { 0x1948d72f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bcd736, 3 },
    { 0, 0 },
    { 0xd990d738, 3 },
    { 0xe144d738, 3 },
    { 0x5df4d73a, 5 },
    { 0xffc8d73b, 3 },
    { 0x565cd73b, 3 },
    { 0x3404d73d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c24d744, 3 },
    { 0x5458d745, 3 },
    { 0xc908d746, 3 },
    { 0, 0 },
    { 0xec24d748, 4 },
    { 0x4ce0d749, 3 },
    { 0x85cd749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb68d74e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492cd754, 4 },
    { 0, 0 },
    { 0xd138d756, 3 },
    { 0x4994d757, 3 },
    { 0, 0 },
    { 0xf5cd759, 4 },
    { 0x8cb4d75a, 4 },
    { 0xb03cd75b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a60d75e, 3 },
    { 0x7cdcd75f, 3 },
    { 0xa7dcd760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cecd764, 3 },
    { 0xfca8d765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3258d76e, 3 },
    { 0x65f4d76f, 4 },
    { 0xb1d8d770, 3 },
    { 0x5714d771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74d785, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d4d788, 3 },
    { 0x1dacd789, 3 },
    { 0xfe4d789, 4 },
    { 0, 0 },
    { 0xe580d78c, 3 },
    { 0xb5f0d78d, 4 },
    { 0, 0 },
    { 0xd650d78f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef40d799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881cd79f, 3 },
    { 0, 0 },
    { 0xc8c4d7a1, 3 },
    { 0, 0 },
    { 0x834d7a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bcd7a8, 4 },
    { 0, 0 },
    { 0x356cd7aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d7ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677cd7b3, 3 },
    { 0xaa48d7b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c28d7b9, 3 },
    { 0xc8e8d7ba, 3 },
    { 0x6480d7b9, 3 },
    { 0xb154d7ba, 3 },
    { 0xe92cd7bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc340d7c1, 3 },
    { 0, 0 },
    { 0x5a24d7c3, 3 },
    { 0, 0 },
    { 0x987cd7c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08d7c8, 3 },
    { 0xac98d7c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22cd7d0, 3 },
    { 0xc52cd7d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf878d7de, 3 },
    { 0x2ca8d7df, 4 },
    { 0, 0 },
    { 0x8984d7e1, 3 },
    { 0x3b58d7e2, 3 },
    { 0x8decd7e3, 4 },
    { 0, 0 },
    { 0xa58cd7e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358cd7e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f68d7ed, 3 },
    { 0x5394d7ee, 3 },
    { 0xddc0d7ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7530d7f3, 4 },
    { 0xa448d7f3, 4 },
    { 0xa1b8d7f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6164d7fa, 3 },
    { 0xee84d7fa, 3 },
    { 0x5168d7fc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cd7ff, 3 },
    { 0, 0 },
    { 0x755cd801, 4 },
    { 0, 0 },
    { 0xdd00d803, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9498d806, 4 },
    { 0xb484d806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8630d80c, 3 },
    { 0x78e8d80d, 4 },
    { 0x472cd80e, 3 },
    { 0x7d40d80f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef0d812, 3 },
    { 0, 0 },
    { 0x9f68d814, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab0d81a, 4 },
    { 0x85fcd81b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26cd821, 4 },
    { 0xacf8d821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aecd827, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af4d82a, 5 },
    { 0x634cd82a, 3 },
    { 0, 0 },
    { 0x51d8d82d, 3 },
    { 0x41d0d82e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec24d831, 3 },
    { 0x1b78d831, 3 },
    { 0xf8ccd832, 4 },
    { 0, 0 },
    { 0xbed0d835, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf0d83a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a8d83e, 3 },
    { 0x581cd83f, 3 },
    { 0xcfdcd83f, 3 },
    { 0xbbe4d840, 4 },
    { 0xfa54d842, 3 },
    { 0xa9bcd83e, 3 },
    { 0x29cd844, 4 },
    { 0x1a18d843, 3 },
    { 0, 0 },
    { 0xb3c0d847, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e4d857, 3 },
    { 0x447cd858, 4 },
    { 0x4be0d859, 4 },
    { 0, 0 },
    { 0xa0e4d85b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a40d85e, 3 },
    { 0xa7ecd85f, 3 },
    { 0, 0 },
    { 0x1990d861, 4 },
    { 0x3c40d861, 3 },
    { 0xf30cd861, 4 },
    { 0x5e98d864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe4d868, 4 },
    { 0, 0 },
    { 0xb7f8d86a, 3 },
    { 0x4290d86a, 3 },
    { 0, 0 },
    { 0x6ab0d86d, 3 },
    { 0x5840d86e, 3 },
    { 0x16b8d86f, 3 },
    { 0xfed8d86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c94d874, 3 },
    { 0x3690d875, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8cd87a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ccd87d, 3 },
    { 0x89c4d87e, 3 },
    { 0, 0 },
    { 0x3d18d880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4d885, 3 },
    { 0, 0 },
    { 0x4d2cd887, 4 },
    { 0xa6f0d888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x665cd88b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2660d899, 3 },
    { 0x6148d89a, 3 },
    { 0x498cd89b, 3 },
    { 0x6818d899, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d54d89f, 4 },
    { 0, 0 },
    { 0xbac8d8a1, 3 },
    { 0, 0 },
    { 0x3c70d8a3, 5 },
    { 0x3364d8a4, 3 },
    { 0x694d8a4, 5 },
    { 0, 0 },
    { 0x9becd8a7, 3 },
    { 0, 0 },
    { 0xd298d8a9, 3 },
    { 0, 0 },
    { 0x56c8d8ab, 3 },
    { 0, 0 },
    { 0x7938d8ad, 3 },
    { 0x7df4d8ae, 3 },
    { 0xa060d8af, 4 },
    { 0xbc34d8af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa310d8bc, 4 },
    { 0xf70cd8bd, 3 },
    { 0xd44d8be, 3 },
    { 0xded0d8bd, 5 },
    { 0xc188d8c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a0d8c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x442cd8c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928d8cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc0d8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d4d8dd, 4 },
    { 0xa738d8de, 3 },
    { 0, 0 },
    { 0x4e38d8e0, 3 },
    { 0, 0 },
    { 0x488d8e2, 3 },
    { 0, 0 },
    { 0x5eecd8e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3cd8e7, 3 },
    { 0, 0 },
    { 0x7a70d8e9, 4 },
    { 0xfff0d8e9, 3 },
    { 0x8ab4d8eb, 3 },
    { 0, 0 },
    { 0x4b9cd8ed, 3 },
    { 0x39f8d8ee, 4 },
    { 0x6e18d8ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924d8f2, 3 },
    { 0, 0 },
    { 0x6614d8f4, 3 },
    { 0xb474d8f5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37cd8fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0d901, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cd908, 4 },
    { 0x1a74d908, 3 },
    { 0, 0 },
    { 0xbc8cd90b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5544d90e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7874d911, 3 },
    { 0, 0 },
    { 0xc14cd913, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf988d91c, 4 },
    { 0x7e50d91c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e4d924, 3 },
    { 0x6f40d924, 4 },
    { 0x9b78d926, 3 },
    { 0x9340d927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770d92c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b8d92f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5024d932, 3 },
    { 0xa678d933, 4 },
    { 0xb518d934, 4 },
    { 0xb300d935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc170d938, 3 },
    { 0xf6d8d939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf480d93e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68d944, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b8d947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0d94a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc10d94e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800d951, 3 },
    { 0x5968d952, 4 },
    { 0xe2c0d952, 4 },
    { 0, 0 },
    { 0x8334d955, 3 },
    { 0, 0 },
    { 0xd46cd957, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb930d95b, 3 },
    { 0x759cd95c, 4 },
    { 0xb070d95d, 4 },
    { 0x25f8d95b, 3 },
    { 0xbb5cd95f, 3 },
    { 0xb488d95f, 3 },
    { 0xcc88d961, 4 },
    { 0x8ba0d95f, 3 },
    { 0xa1e0d963, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8064d967, 3 },
    { 0x305cd968, 4 },
    { 0x7014d968, 3 },
    { 0x2e9cd968, 4 },
    { 0x3730d96b, 4 },
    { 0x5cecd967, 3 },
    { 0, 0 },
    { 0x3ec8d96e, 5 },
    { 0, 0 },
    { 0x13a8d970, 3 },
    { 0xfc00d970, 4 },
    { 0x3130d972, 3 },
    { 0x909cd973, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb778d976, 4 },
    { 0x514d977, 4 },
    { 0x438cd976, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b4d97c, 3 },
    { 0x47f0d97c, 3 },
    { 0, 0 },
    { 0x73c8d97f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9030d982, 3 },
    { 0xb624d983, 5 },
    { 0xff08d984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb0d989, 3 },
    { 0x4a1cd98a, 3 },
    { 0x8958d98a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd658d991, 3 },
    { 0xd4f0d992, 3 },
    { 0x70a8d991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6934d99b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc0d9a3, 4 },
    { 0x766cd9a3, 3 },
    { 0x17f8d9a5, 3 },
    { 0x39e8d9a5, 5 },
    { 0x8c3cd9a5, 3 },
    { 0xa720d9a8, 3 },
    { 0xfd24d9a9, 4 },
    { 0xd048d9aa, 4 },
    { 0x3c6cd9aa, 3 },
    { 0, 0 },
    { 0x9374d9ad, 3 },
    { 0xfab4d9ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18d9b1, 3 },
    { 0xe344d9b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef0d9b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa88d9c0, 4 },
    { 0x8cd0d9c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd978d9c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dcd9c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b34d9d3, 3 },
    { 0xeb04d9d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebcd9da, 3 },
    { 0xbc04d9db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b8d9e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dcd9e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70d9fb, 3 },
    { 0x78f8d9fc, 3 },
    { 0x627cd9fd, 4 },
    { 0, 0 },
    { 0x7bccd9ff, 4 },
    { 0, 0 },
    { 0x85f4da01, 3 },
    { 0xe7c0da02, 4 },
    { 0xa344da02, 3 },
    { 0x76ccda03, 3 },
    { 0xc5cda04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606cda09, 3 },
    { 0xf700da0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d0da0f, 3 },
    { 0x2bd0da10, 3 },
    { 0xb140da10, 3 },
    { 0x8f08da12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb60da18, 3 },
    { 0, 0 },
    { 0x19c8da1a, 4 },
    { 0xf950da1b, 3 },
    { 0x7dc0da1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0cda23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4da26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5004da29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd4da2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa408da31, 3 },
    { 0, 0 },
    { 0xa328da33, 3 },
    { 0x4e88da34, 3 },
    { 0x3eacda35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb50da3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7594da40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b8da43, 3 },
    { 0x1480da44, 4 },
    { 0xd700da44, 3 },
    { 0xc2bcda46, 5 },
    { 0, 0 },
    { 0xe4a8da48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0da4d, 3 },
    { 0xd35cda4d, 4 },
    { 0xa594da4f, 3 },
    { 0xec3cda50, 3 },
    { 0xb744da50, 3 },
    { 0x6e0da4e, 3 },
    { 0x2a0cda53, 3 },
    { 0x67cda52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f0da57, 3 },
    { 0xc4c0da57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d0da5d, 4 },
    { 0x3988da5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2cda61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0da67, 3 },
    { 0x8934da67, 3 },
    { 0x4fa0da69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f54da6f, 3 },
    { 0x5dd0da6f, 3 },
    { 0, 0 },
    { 0x2f7cda72, 4 },
    { 0xa100da73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74cda76, 3 },
    { 0x64b8da77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d8da7f, 3 },
    { 0, 0 },
    { 0xa888da81, 4 },
    { 0x82dcda81, 3 },
    { 0x57c0da82, 3 },
    { 0x94d4da84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5808da87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cda8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394da92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570da95, 4 },
    { 0xa684da96, 3 },
    { 0, 0 },
    { 0xcb6cda98, 4 },
    { 0xdd6cda99, 5 },
    { 0x8d84da99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e0da9d, 5 },
    { 0, 0 },
    { 0xba00da9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbcdaa3, 3 },
    { 0xe35cdaa3, 3 },
    { 0, 0 },
    { 0x4e68daa6, 4 },
    { 0x5f9cdaa7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8daab, 3 },
    { 0x1834daab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd0dab3, 3 },
    { 0, 0 },
    { 0xdcacdab5, 3 },
    { 0x4134dab6, 3 },
    { 0x7728dab7, 3 },
    { 0xa54dab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd370dabb, 3 },
    { 0, 0 },
    { 0xf294dabd, 4 },
    { 0xb03cdabd, 3 },
    { 0xf080dabf, 3 },
    { 0x5e58dac0, 3 },
    { 0x678cdac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c20dac8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfcdace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f0dad1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd4dad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00dae3, 3 },
    { 0x8fa0dae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f0dae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e4daeb, 3 },
    { 0xdc34daec, 3 },
    { 0x9a48daed, 4 },
    { 0, 0 },
    { 0x27f0daef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9cdaf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e00dafc, 3 },
    { 0, 0 },
    { 0xd94cdafe, 3 },
    { 0x9d30dafe, 3 },
    { 0x9168dafe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b0db03, 3 },
    { 0xfaf0db03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4db0a, 4 },
    { 0x1bd8db0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdcdb10, 3 },
    { 0, 0 },
    { 0xd9e0db12, 4 },
    { 0xa174db13, 4 },
    { 0, 0 },
    { 0xe004db15, 3 },
    { 0xa0c4db16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd4db19, 3 },
    { 0x7b68db1a, 3 },
    { 0, 0 },
    { 0x9068db1c, 4 },
    { 0x5c90db1d, 3 },
    { 0x649cdb1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x716cdb21, 4 },
    { 0xc730db22, 3 },
    { 0xb794db23, 3 },
    { 0x86a0db24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d10db27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a30db2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x660db2e, 3 },
    { 0xd67cdb2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3630db33, 3 },
    { 0x1744db34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dcdb37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a4db3b, 4 },
    { 0xa88cdb3c, 4 },
    { 0, 0 },
    { 0xa408db3e, 4 },
    { 0x9214db3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa0db42, 4 },
    { 0x22dcdb43, 3 },
    { 0xc7ecdb44, 3 },
    { 0x77a0db42, 4 },
    { 0x88b4db44, 3 },
    { 0xdd2cdb47, 4 },
    { 0x7ee8db48, 3 },
    { 0, 0 },
    { 0xb6c0db4a, 3 },
    { 0x8858db4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa0db4e, 3 },
    { 0, 0 },
    { 0x68d0db50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d84db53, 3 },
    { 0x3600db54, 5 },
    { 0, 0 },
    { 0xd954db56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1acdb5b, 4 },
    { 0, 0 },
    { 0x84ecdb5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe908db60, 4 },
    { 0xa4acdb60, 3 },
    { 0x4420db62, 4 },
    { 0x8b08db62, 3 },
    { 0xfcc8db62, 3 },
    { 0xa694db65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d24db6c, 3 },
    { 0xb560db6d, 3 },
    { 0x9c58db6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x960db71, 4 },
    { 0x5028db71, 3 },
    { 0xa884db72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee8db7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c8db80, 3 },
    { 0xb800db81, 3 },
    { 0xf8b0db81, 3 },
    { 0x2020db81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5928db86, 3 },
    { 0, 0 },
    { 0xa770db88, 3 },
    { 0, 0 },
    { 0x5e28db8a, 3 },
    { 0x2978db8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff60db8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0acdb94, 4 },
    { 0xf70db94, 3 },
    { 0x4134db94, 4 },
    { 0xa01cdb97, 3 },
    { 0xe424db98, 3 },
    { 0x1400db94, 3 },
    { 0x9714db97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2798db9e, 3 },
    { 0x2ed4db9f, 3 },
    { 0x995cdba0, 5 },
    { 0xde74db9e, 3 },
    { 0x2fbcdba2, 4 },
    { 0xd2c8db9e, 4 },
    { 0x87d8dba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4dba7, 3 },
    { 0x34cdba8, 4 },
    { 0x36c0dba7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90dbac, 4 },
    { 0x2ef8dbac, 3 },
    { 0x49c4dbae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaacdbb6, 3 },
    { 0, 0 },
    { 0x25d8dbb8, 3 },
    { 0x74ccdbb8, 3 },
    { 0, 0 },
    { 0xa010dbbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd4dbc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6cdbc7, 4 },
    { 0xb544dbc8, 3 },
    { 0xaaf8dbc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde0dbcc, 3 },
    { 0x3fe4dbcd, 4 },
    { 0xff3cdbce, 3 },
    { 0x6c44dbcd, 4 },
    { 0x7eecdbd0, 4 },
    { 0x997cdbd1, 3 },
    { 0x78dbd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a0dbd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20dbde, 3 },
    { 0xad68dbde, 4 },
    { 0x7e00dbdf, 5 },
    { 0x832cdbe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf94dbe4, 3 },
    { 0x2e30dbe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b4dbe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa398dbee, 3 },
    { 0xb738dbee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69cdbf6, 3 },
    { 0, 0 },
    { 0x4840dbf8, 3 },
    { 0xc9f8dbf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad00dbfc, 3 },
    { 0x34dcdbfd, 4 },
    { 0xf464dbfd, 3 },
    { 0xf518dbfc, 3 },
    { 0x9b6cdc00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc34dc04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873cdc09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e4dc0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c70dc12, 3 },
    { 0xbb94dc13, 4 },
    { 0x4cf0dc14, 3 },
    { 0xee04dc15, 4 },
    { 0x9e5cdc14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2cdc1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dcdc23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd0dc26, 5 },
    { 0xb280dc27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8dc2b, 3 },
    { 0xd90cdc2c, 3 },
    { 0xadccdc2b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a8dc30, 3 },
    { 0, 0 },
    { 0x8218dc32, 3 },
    { 0, 0 },
    { 0x88c0dc34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42acdc37, 3 },
    { 0, 0 },
    { 0xbb8dc39, 3 },
    { 0xdd20dc39, 3 },
    { 0xaabcdc3b, 3 },
    { 0x18cdc3c, 3 },
    { 0xccc0dc3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cdc44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2984dc47, 3 },
    { 0x5cd4dc48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f4dc4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4004dc58, 4 },
    { 0, 0 },
    { 0x1dbcdc5a, 4 },
    { 0x47c8dc5a, 3 },
    { 0x7448dc5a, 4 },
    { 0x350cdc5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb50dc62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ecdc65, 3 },
    { 0xfb8cdc65, 3 },
    { 0xa30cdc67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edcdc71, 3 },
    { 0x431cdc71, 3 },
    { 0x5874dc72, 3 },
    { 0xe35cdc74, 3 },
    { 0x3ed0dc75, 4 },
    { 0xa300dc75, 3 },
    { 0, 0 },
    { 0xb6b0dc78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428cdc7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e0dc84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f00dc8b, 5 },
    { 0, 0 },
    { 0x3a8dc8d, 3 },
    { 0xf140dc8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff0dc91, 3 },
    { 0, 0 },
    { 0xaa10dc93, 3 },
    { 0, 0 },
    { 0x763cdc95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa580dc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17acdca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677cdca5, 3 },
    { 0xb55cdca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8008dca9, 3 },
    { 0, 0 },
    { 0x2018dcab, 3 },
    { 0, 0 },
    { 0x9724dcad, 4 },
    { 0x8514dcae, 3 },
    { 0xb7a4dcaf, 3 },
    { 0xc9e0dcae, 3 },
    { 0xe840dcae, 5 },
    { 0x3fd4dcb2, 4 },
    { 0x949cdcb2, 3 },
    { 0, 0 },
    { 0x341cdcb5, 3 },
    { 0xe714dcb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe54dcbb, 3 },
    { 0x4328dcbc, 3 },
    { 0xc054dcbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3628dcca, 3 },
    { 0x6588dccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f78dccf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7714dcd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad4dcd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15cdcd9, 3 },
    { 0x903cdcda, 3 },
    { 0, 0 },
    { 0x5600dcdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c4dce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7870dcea, 5 },
    { 0, 0 },
    { 0x1914dcec, 3 },
    { 0xd1c0dced, 3 },
    { 0x54bcdcee, 3 },
    { 0x5944dcee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d8dcf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950dcfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7cdd05, 3 },
    { 0xf36cdd06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x272cdd09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420cdd0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d4dd13, 3 },
    { 0xf5c4dd14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74dd17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5cdd1b, 3 },
    { 0x8b10dd1c, 3 },
    { 0xa7cdd1d, 3 },
    { 0, 0 },
    { 0xc8d8dd1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffcdd22, 3 },
    { 0xd220dd22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef4dd27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abcdd2a, 3 },
    { 0, 0 },
    { 0x8088dd2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb624dd2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce14dd34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe064dd37, 3 },
    { 0xa8ecdd38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed70dd3b, 3 },
    { 0xb70dd3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6404dd40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0dd45, 3 },
    { 0xb810dd46, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf990dd49, 3 },
    { 0xabb0dd4a, 3 },
    { 0x31ecdd4b, 3 },
    { 0x97b0dd4b, 3 },
    { 0x5ef0dd4d, 3 },
    { 0x71b4dd4b, 3 },
    { 0x1c74dd4f, 3 },
    { 0x88ccdd4d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b10dd54, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf8dd57, 4 },
    { 0, 0 },
    { 0x15a4dd59, 3 },
    { 0x5a50dd5a, 3 },
    { 0, 0 },
    { 0xc594dd5c, 3 },
    { 0xc054dd5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b8dd62, 4 },
    { 0, 0 },
    { 0x574cdd64, 3 },
    { 0x62b4dd64, 4 },
    { 0x540dd66, 4 },
    { 0x9a74dd67, 3 },
    { 0, 0 },
    { 0xf8c8dd69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30dd6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d4dd71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c8dd77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6cdd7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908dd85, 3 },
    { 0x6b98dd86, 3 },
    { 0x698dd85, 4 },
    { 0xff38dd88, 3 },
    { 0xb90cdd88, 4 },
    { 0x66ecdd8a, 3 },
    { 0xf9d0dd8b, 3 },
    { 0, 0 },
    { 0x6b58dd8d, 5 },
    { 0, 0 },
    { 0xffc0dd8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8dd93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace0dd98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5120dd9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df0dda2, 4 },
    { 0xb04cdda2, 3 },
    { 0, 0 },
    { 0x32c4dda5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a74dda9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc4ddaf, 5 },
    { 0x2114ddb0, 3 },
    { 0xee14ddb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8ddb6, 4 },
    { 0xabc0ddb6, 3 },
    { 0, 0 },
    { 0xf8e0ddb9, 3 },
    { 0x2418ddba, 3 },
    { 0xe4cddbb, 3 },
    { 0, 0 },
    { 0x99d4ddbd, 3 },
    { 0, 0 },
    { 0xc688ddbf, 4 },
    { 0xa444ddbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4ddc4, 3 },
    { 0xa8a0ddc5, 4 },
    { 0xfb70ddc4, 3 },
    { 0, 0 },
    { 0x5ca8ddc8, 3 },
    { 0xdd4cddc8, 3 },
    { 0x52a8ddc9, 3 },
    { 0xabf0ddcb, 3 },
    { 0xea18ddcc, 3 },
    { 0x9dc4ddcd, 3 },
    { 0, 0 },
    { 0x828cddcf, 3 },
    { 0xd45cddd0, 3 },
    { 0xf1d4ddd1, 3 },
    { 0xd8f4ddd0, 3 },
    { 0x1fc0ddcf, 3 },
    { 0x3530ddd2, 3 },
    { 0x7a3cddd5, 4 },
    { 0, 0 },
    { 0x4bfcddd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae8dde2, 3 },
    { 0x2768dde3, 4 },
    { 0, 0 },
    { 0xcce8dde5, 3 },
    { 0, 0 },
    { 0xb118dde7, 4 },
    { 0x28a8dde8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670ddf1, 4 },
    { 0x9dc4ddf1, 3 },
    { 0, 0 },
    { 0x7278ddf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80ddf7, 3 },
    { 0xc4e8ddf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a0ddff, 3 },
    { 0x86b8ddff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b0de04, 4 },
    { 0x5780de05, 4 },
    { 0x60a4de04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac64de09, 4 },
    { 0, 0 },
    { 0xf908de0b, 3 },
    { 0xfdf8de0b, 3 },
    { 0x8030de0c, 3 },
    { 0xd5b4de0e, 4 },
    { 0x64f4de0f, 3 },
    { 0, 0 },
    { 0x1f84de11, 4 },
    { 0xc020de12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecde15, 3 },
    { 0xab58de15, 3 },
    { 0x4ab8de17, 3 },
    { 0x7cfcde17, 3 },
    { 0x9198de19, 3 },
    { 0x8634de19, 4 },
    { 0xd690de19, 3 },
    { 0xa78cde1b, 3 },
    { 0x55c0de1c, 3 },
    { 0x4454de1e, 3 },
    { 0x3d1cde1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b0de24, 4 },
    { 0x9c28de25, 3 },
    { 0x95e8de25, 4 },
    { 0x34ecde25, 4 },
    { 0x9398de28, 3 },
    { 0, 0 },
    { 0x3670de2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98de2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8cde31, 3 },
    { 0xaa30de32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cde39, 3 },
    { 0x2ee4de3a, 3 },
    { 0, 0 },
    { 0xb83cde3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f0de40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1528de45, 3 },
    { 0xa9a0de45, 3 },
    { 0x341cde47, 4 },
    { 0xe474de48, 4 },
    { 0, 0 },
    { 0x985cde4a, 3 },
    { 0x6eacde4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd4de4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b90de53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bcde59, 4 },
    { 0xe674de59, 3 },
    { 0, 0 },
    { 0x5e8cde5c, 4 },
    { 0xa3cde5d, 3 },
    { 0xb79cde5d, 3 },
    { 0, 0 },
    { 0xf05cde60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cde63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8de6e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cde74, 3 },
    { 0x450de74, 3 },
    { 0, 0 },
    { 0x3b18de77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed60de7b, 3 },
    { 0, 0 },
    { 0x631cde7d, 3 },
    { 0, 0 },
    { 0xd26cde7f, 3 },
    { 0xbde4de7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238cde87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d8de8c, 3 },
    { 0x8998de8c, 3 },
    { 0, 0 },
    { 0x72c4de8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cde94, 5 },
    { 0, 0 },
    { 0xab4de96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541cdea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110cdea6, 4 },
    { 0, 0 },
    { 0x53ccdea8, 3 },
    { 0, 0 },
    { 0x7c94deaa, 4 },
    { 0, 0 },
    { 0xeae4deac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb8deaf, 4 },
    { 0, 0 },
    { 0xf940deb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x225cdeb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360deb8, 3 },
    { 0, 0 },
    { 0xc860deba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e54debd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5cdec3, 3 },
    { 0xd23cdec3, 3 },
    { 0x7564dec5, 3 },
    { 0, 0 },
    { 0xb05cdec7, 3 },
    { 0x43e8dec8, 4 },
    { 0xe220dec8, 3 },
    { 0, 0 },
    { 0xcd14decb, 3 },
    { 0xbe04decc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe754decf, 4 },
    { 0, 0 },
    { 0x7230ded1, 3 },
    { 0xf58cded2, 3 },
    { 0, 0 },
    { 0x82d8ded4, 3 },
    { 0xf278ded5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6cdeda, 4 },
    { 0x96e8deda, 4 },
    { 0x4f6cdedc, 3 },
    { 0x3394dedb, 3 },
    { 0, 0 },
    { 0x38d8dedf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a0dee2, 4 },
    { 0x7df0dee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924cdeec, 3 },
    { 0xa494deec, 3 },
    { 0x44a4deee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4def1, 3 },
    { 0x10f0def2, 4 },
    { 0x6434def3, 3 },
    { 0x9130def4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f4defc, 3 },
    { 0, 0 },
    { 0xe0a8defe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8df02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94df06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cdf0e, 4 },
    { 0xb420df0e, 3 },
    { 0xb4f4df10, 3 },
    { 0xf76cdf0e, 3 },
    { 0, 0 },
    { 0x3c8cdf13, 3 },
    { 0x488df14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3cdf17, 3 },
    { 0xf89cdf18, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8628df1b, 3 },
    { 0x73c8df1b, 3 },
    { 0, 0 },
    { 0xc9fcdf1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580df21, 3 },
    { 0x5078df22, 3 },
    { 0xc8b4df21, 3 },
    { 0x3660df21, 3 },
    { 0xaa4df25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5848df2a, 3 },
    { 0xecf4df2b, 3 },
    { 0x6a44df2b, 3 },
    { 0, 0 },
    { 0x5a8cdf2e, 3 },
    { 0x7cecdf2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8df33, 4 },
    { 0x2514df33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b34df37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddcdf3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1724df41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8df48, 3 },
    { 0x1400df48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ccdf50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32cdf54, 3 },
    { 0x6c74df55, 4 },
    { 0x65c8df56, 4 },
    { 0x9fd4df57, 4 },
    { 0x9640df58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d0df5d, 3 },
    { 0, 0 },
    { 0x4ab0df5f, 3 },
    { 0x358cdf60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e4df66, 3 },
    { 0xd490df66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e4df6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9850df71, 3 },
    { 0xcdc4df72, 3 },
    { 0x7f1cdf71, 3 },
    { 0, 0 },
    { 0x4168df75, 3 },
    { 0x36dcdf76, 3 },
    { 0x48f0df77, 3 },
    { 0, 0 },
    { 0x3fb4df79, 3 },
    { 0xf1ccdf7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe8df81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca0df89, 4 },
    { 0xe950df8a, 3 },
    { 0x8590df8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f0df91, 5 },
    { 0x797cdf92, 3 },
    { 0, 0 },
    { 0x860df94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b00df98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ccdf9b, 4 },
    { 0, 0 },
    { 0xcc80df9d, 3 },
    { 0x9e8cdf9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef80dfa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x530cdfa8, 3 },
    { 0x8dacdfa8, 3 },
    { 0x7c14dfa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2168dfad, 3 },
    { 0xb800dfae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad0dfb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588dfb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf04dfc2, 3 },
    { 0x1db0dfc3, 3 },
    { 0x3ef8dfc4, 4 },
    { 0x45c0dfc5, 4 },
    { 0x41f4dfc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1028dfcd, 4 },
    { 0xb5dcdfcd, 3 },
    { 0x548cdfcf, 3 },
    { 0, 0 },
    { 0x20c8dfd1, 4 },
    { 0xa520dfd2, 3 },
    { 0xf340dfd2, 3 },
    { 0x1afcdfd2, 4 },
    { 0xbc10dfd2, 3 },
    { 0x5c8cdfd6, 3 },
    { 0, 0 },
    { 0x3b0dfd8, 3 },
    { 0x2e10dfd9, 3 },
    { 0x6694dfda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e8dfe2, 4 },
    { 0x9084dfe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8dfe6, 3 },
    { 0x30acdfe7, 5 },
    { 0x5688dfe8, 3 },
    { 0x7170dfe8, 3 },
    { 0x1134dfe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x602cdfed, 3 },
    { 0x5f00dfee, 3 },
    { 0x6324dfef, 3 },
    { 0, 0 },
    { 0x1f70dff1, 3 },
    { 0x6b58dff2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9accdff5, 3 },
    { 0xa384dff6, 3 },
    { 0x4840dff6, 3 },
    { 0x5248dff8, 3 },
    { 0x8760dff8, 4 },
    { 0xb288dff8, 3 },
    { 0, 0 },
    { 0x73c4dffc, 4 },
    { 0x5340dffd, 3 },
    { 0x5eccdffd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8e002, 3 },
    { 0xe1dce003, 3 },
    { 0, 0 },
    { 0x4f88e005, 4 },
    { 0xe928e006, 3 },
    { 0, 0 },
    { 0x3130e008, 3 },
    { 0xd980e009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ece00c, 3 },
    { 0x90a0e00c, 4 },
    { 0x2fcce00e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf554e011, 4 },
    { 0x4c80e012, 4 },
    { 0, 0 },
    { 0x5e0e014, 4 },
    { 0xa984e014, 4 },
    { 0xd724e016, 5 },
    { 0x398e017, 3 },
    { 0, 0 },
    { 0xd154e019, 4 },
    { 0, 0 },
    { 0x33d0e01b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ce023, 3 },
    { 0x2264e024, 3 },
    { 0x9a74e023, 3 },
    { 0xcf9ce026, 4 },
    { 0x445ce027, 3 },
    { 0, 0 },
    { 0x614e029, 4 },
    { 0, 0 },
    { 0x8098e02b, 4 },
    { 0x16b0e02b, 3 },
    { 0xdcfce02c, 3 },
    { 0, 0 },
    { 0xaca8e02f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388ce033, 3 },
    { 0x1940e033, 3 },
    { 0xaf44e033, 3 },
    { 0xafbce036, 3 },
    { 0xba04e037, 4 },
    { 0xba80e038, 3 },
    { 0x2b78e036, 3 },
    { 0x4f98e039, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4e03d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03ce040, 4 },
    { 0x5904e040, 4 },
    { 0x28a8e042, 4 },
    { 0x7968e043, 3 },
    { 0, 0 },
    { 0xb03ce045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab8e049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d94e04f, 3 },
    { 0x164ce050, 3 },
    { 0x8158e051, 4 },
    { 0xb25ce050, 3 },
    { 0, 0 },
    { 0x3a48e054, 3 },
    { 0xbfb0e055, 3 },
    { 0xfe54e056, 4 },
    { 0xfe78e057, 4 },
    { 0x6bf8e058, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4e05b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e14e05f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c4e062, 3 },
    { 0xabace062, 4 },
    { 0x71fce064, 3 },
    { 0, 0 },
    { 0xc854e066, 4 },
    { 0, 0 },
    { 0x988e068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818e071, 3 },
    { 0x7404e071, 3 },
    { 0x4fe4e073, 3 },
    { 0x1380e074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b60e07d, 3 },
    { 0, 0 },
    { 0xc6b0e07f, 3 },
    { 0x55bce07f, 4 },
    { 0x1640e081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce98e085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559ce08c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5628e08f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1560e096, 3 },
    { 0, 0 },
    { 0xd540e098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8e09c, 3 },
    { 0, 0 },
    { 0x6f08e09e, 3 },
    { 0xf538e09f, 3 },
    { 0x9458e0a0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8ce0a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x374ce0aa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x590e0ad, 4 },
    { 0xc688e0ae, 4 },
    { 0x5bb8e0af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c20e0b2, 3 },
    { 0, 0 },
    { 0xc654e0b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707ce0bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea08e0c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfece0c4, 3 },
    { 0x3618e0c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf224e0cd, 3 },
    { 0x4610e0ce, 3 },
    { 0xbd38e0cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf774e0d6, 3 },
    { 0, 0 },
    { 0x49a4e0d8, 3 },
    { 0, 0 },
    { 0x7eace0da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6face0e0, 4 },
    { 0xedfce0e0, 3 },
    { 0, 0 },
    { 0x4a54e0e3, 4 },
    { 0, 0 },
    { 0x7668e0e5, 4 },
    { 0x8c04e0e6, 4 },
    { 0x6388e0e5, 4 },
    { 0, 0 },
    { 0xb8b8e0e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc920e0ec, 3 },
    { 0x9328e0ed, 3 },
    { 0, 0 },
    { 0x97ece0ef, 3 },
    { 0x5ef0e0f0, 3 },
    { 0, 0 },
    { 0x1118e0f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x597ce0f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca88e0fd, 3 },
    { 0xd458e0fd, 4 },
    { 0x9bf8e0ff, 4 },
    { 0x27d8e100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc4e108, 3 },
    { 0, 0 },
    { 0xd190e10a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef90e10d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb220e113, 3 },
    { 0xfcce113, 3 },
    { 0x41ace115, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe288e11a, 3 },
    { 0, 0 },
    { 0x1574e11c, 3 },
    { 0xcdc8e11c, 3 },
    { 0, 0 },
    { 0x5b84e11f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698e123, 3 },
    { 0xdb9ce124, 3 },
    { 0xb900e125, 3 },
    { 0x3cf0e126, 3 },
    { 0x87fce126, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4ce12b, 3 },
    { 0x9d14e12c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98e12f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6664e132, 3 },
    { 0x90f4e132, 3 },
    { 0x4ec4e133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc974e137, 3 },
    { 0x9f20e138, 3 },
    { 0x1448e139, 3 },
    { 0, 0 },
    { 0x3d48e13b, 3 },
    { 0x4a4e13c, 3 },
    { 0xf6a8e13c, 5 },
    { 0, 0 },
    { 0x4590e13f, 3 },
    { 0x6b44e13f, 3 },
    { 0, 0 },
    { 0x4fb0e142, 3 },
    { 0xe3c0e143, 3 },
    { 0x8d18e144, 4 },
    { 0, 0 },
    { 0x2e18e146, 3 },
    { 0, 0 },
    { 0x9758e148, 4 },
    { 0x13e8e148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe298e14c, 3 },
    { 0x9e28e14d, 4 },
    { 0xd0a4e14c, 3 },
    { 0, 0 },
    { 0xb6ce150, 3 },
    { 0x3b44e150, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e4e154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c8e157, 4 },
    { 0, 0 },
    { 0xf3b0e159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b4e15f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4e164, 3 },
    { 0, 0 },
    { 0xd4d8e166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ce16c, 4 },
    { 0xf1fce16d, 3 },
    { 0x9088e16e, 4 },
    { 0, 0 },
    { 0xbfe8e170, 4 },
    { 0, 0 },
    { 0x6d64e172, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4e177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335ce17b, 3 },
    { 0, 0 },
    { 0x96a4e17d, 4 },
    { 0, 0 },
    { 0xed5ce17f, 4 },
    { 0, 0 },
    { 0x7094e181, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb4e185, 3 },
    { 0x874ce185, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d4e18b, 4 },
    { 0, 0 },
    { 0x2170e18d, 4 },
    { 0, 0 },
    { 0xc6f0e18f, 4 },
    { 0, 0 },
    { 0xbebce191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5524e199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee64e19c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6830e19f, 3 },
    { 0xfd44e19f, 4 },
    { 0, 0 },
    { 0x2b58e1a2, 3 },
    { 0, 0 },
    { 0x917ce1a4, 3 },
    { 0xbbace1a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9398e1a8, 3 },
    { 0xc9a0e1a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816ce1ad, 3 },
    { 0, 0 },
    { 0x200e1af, 4 },
    { 0, 0 },
    { 0x6e9ce1b1, 3 },
    { 0, 0 },
    { 0xea40e1b3, 3 },
    { 0x44b8e1b4, 3 },
    { 0x96d4e1b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784e1bf, 3 },
    { 0x518e1c0, 5 },
    { 0xde54e1c1, 3 },
    { 0x38f4e1c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b0e1c6, 3 },
    { 0x24d4e1c6, 3 },
    { 0x83bce1c7, 3 },
    { 0x2620e1c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcce1cf, 3 },
    { 0x8e84e1cf, 3 },
    { 0xbb80e1d1, 3 },
    { 0xce18e1d2, 3 },
    { 0xb89ce1d3, 3 },
    { 0xf0a0e1d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8e1da, 4 },
    { 0x3940e1db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4e1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d8e1e2, 3 },
    { 0xa5a0e1e3, 3 },
    { 0, 0 },
    { 0xfc48e1e5, 3 },
    { 0xe688e1e5, 3 },
    { 0xe96ce1e7, 4 },
    { 0xbee8e1e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5354e1eb, 3 },
    { 0xdd60e1eb, 3 },
    { 0, 0 },
    { 0x1640e1ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30e1f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea8e1f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x622ce1fa, 4 },
    { 0xf8b4e1fa, 3 },
    { 0x6298e1fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c38e1ff, 5 },
    { 0, 0 },
    { 0x3ace201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc180e204, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ce207, 3 },
    { 0xf3a4e208, 3 },
    { 0x73b0e208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda8e222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4758e228, 3 },
    { 0xb274e229, 4 },
    { 0x2dd4e228, 3 },
    { 0, 0 },
    { 0x2964e22c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1794e22f, 3 },
    { 0, 0 },
    { 0x2d0ce231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d98e236, 4 },
    { 0xeece236, 3 },
    { 0, 0 },
    { 0x7664e239, 5 },
    { 0x8944e23a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4ce23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28ce245, 3 },
    { 0xe1f4e245, 4 },
    { 0x5b90e245, 3 },
    { 0x8404e248, 3 },
    { 0x68fce249, 3 },
    { 0xdc68e24a, 3 },
    { 0, 0 },
    { 0xfe88e24c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba38e252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2008e258, 3 },
    { 0xda48e258, 3 },
    { 0, 0 },
    { 0xca4e25b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddce25e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ce262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe790e265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f8e268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb48e26b, 3 },
    { 0, 0 },
    { 0x2f34e26d, 3 },
    { 0xcb78e26e, 3 },
    { 0, 0 },
    { 0x71b4e270, 3 },
    { 0xf564e271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9ce27e, 3 },
    { 0x2bc4e27e, 3 },
    { 0xe248e280, 3 },
    { 0, 0 },
    { 0xb8cce282, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090e287, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ce294, 4 },
    { 0xaa4ce295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff58e29a, 3 },
    { 0, 0 },
    { 0xbece29c, 3 },
    { 0x2e88e29d, 3 },
    { 0x8230e29e, 3 },
    { 0xe858e29f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a54e2a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8034e2a7, 4 },
    { 0, 0 },
    { 0xd800e2a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9110e2b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aace2b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850e2b9, 3 },
    { 0x2c18e2b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2034e2bf, 3 },
    { 0x3174e2bf, 4 },
    { 0, 0 },
    { 0xacd4e2c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b4e2c5, 3 },
    { 0, 0 },
    { 0xbc70e2c7, 3 },
    { 0xec64e2c7, 4 },
    { 0, 0 },
    { 0xeb20e2ca, 3 },
    { 0xd6f8e2ca, 3 },
    { 0, 0 },
    { 0xc614e2cd, 3 },
    { 0xaccce2cd, 4 },
    { 0xdf80e2cf, 3 },
    { 0x9c14e2ce, 3 },
    { 0x49f4e2d1, 3 },
    { 0xff8e2d1, 4 },
    { 0xb66ce2d3, 3 },
    { 0x8ecce2cf, 3 },
    { 0, 0 },
    { 0xb7d4e2d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66ce2d9, 3 },
    { 0x6708e2da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4ce2de, 3 },
    { 0x6640e2df, 3 },
    { 0xf6b0e2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ce2e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9ce2e6, 4 },
    { 0x3350e2e7, 3 },
    { 0, 0 },
    { 0x31d4e2e9, 3 },
    { 0x1264e2e9, 3 },
    { 0x1840e2eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a8e2f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c8e2f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8e2fd, 4 },
    { 0xff5ce2fe, 3 },
    { 0x676ce2fd, 3 },
    { 0xb1ace2fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b0e304, 3 },
    { 0xb2c4e305, 3 },
    { 0, 0 },
    { 0xe448e307, 3 },
    { 0x41b0e307, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe0e30c, 3 },
    { 0x64e4e30c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ce311, 3 },
    { 0, 0 },
    { 0x2e2ce313, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f78e316, 3 },
    { 0x93c0e316, 3 },
    { 0x1f7ce316, 3 },
    { 0, 0 },
    { 0xce50e31a, 3 },
    { 0x156ce31a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe808e323, 4 },
    { 0xa688e323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2734e329, 3 },
    { 0, 0 },
    { 0x291ce32b, 3 },
    { 0xb58e32c, 4 },
    { 0, 0 },
    { 0xef78e32e, 3 },
    { 0x4ad0e32e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x967ce332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f8e336, 3 },
    { 0xd644e337, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea98e340, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c8e34a, 3 },
    { 0, 0 },
    { 0x46d8e34c, 4 },
    { 0x6b0e34c, 5 },
    { 0x8ea8e34e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a14e352, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd0e35c, 3 },
    { 0xb860e35c, 4 },
    { 0xbdf0e35e, 3 },
    { 0xa628e35f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fce362, 3 },
    { 0xde40e363, 3 },
    { 0x1c68e364, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa998e36f, 3 },
    { 0x8a48e370, 3 },
    { 0x9d00e371, 3 },
    { 0xf6d4e370, 4 },
    { 0x53f0e373, 3 },
    { 0xcd7ce371, 3 },
    { 0xdf8ce375, 3 },
    { 0x578ce376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6340e379, 3 },
    { 0x4b90e37a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4e37d, 3 },
    { 0x6e98e37d, 3 },
    { 0x1464e37f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32ce385, 3 },
    { 0, 0 },
    { 0xa788e387, 4 },
    { 0x2128e388, 3 },
    { 0, 0 },
    { 0x8610e38a, 3 },
    { 0x7ed8e38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x239ce38e, 4 },
    { 0xe5ce38f, 3 },
    { 0, 0 },
    { 0xd0a4e391, 3 },
    { 0, 0 },
    { 0xa60ce393, 4 },
    { 0xc8f8e393, 3 },
    { 0xc4f8e393, 3 },
    { 0xacf0e395, 3 },
    { 0x35d8e397, 3 },
    { 0x22b8e394, 3 },
    { 0xd39ce399, 3 },
    { 0xe5ce397, 3 },
    { 0x5ab0e39b, 4 },
    { 0x6080e39c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaface39f, 3 },
    { 0, 0 },
    { 0x5c70e3a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b4e3a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9dce3ab, 3 },
    { 0xbece3ac, 3 },
    { 0xf100e3ac, 3 },
    { 0x1434e3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0e3b2, 3 },
    { 0x8ece3b3, 3 },
    { 0, 0 },
    { 0x89dce3b5, 4 },
    { 0, 0 },
    { 0xe358e3b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70e3ba, 3 },
    { 0x424e3ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb4e3be, 3 },
    { 0x61b8e3bf, 3 },
    { 0x5e70e3c0, 3 },
    { 0xb034e3c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c80e3c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc4e3c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690e3ce, 3 },
    { 0x6438e3ce, 4 },
    { 0x3ffce3d0, 3 },
    { 0x1140e3ce, 3 },
    { 0xf8b4e3d2, 3 },
    { 0x34b4e3d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0ce3d6, 4 },
    { 0x5a14e3d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce90e3db, 3 },
    { 0x9f4e3dc, 3 },
    { 0, 0 },
    { 0xd47ce3de, 3 },
    { 0, 0 },
    { 0x5594e3e0, 3 },
    { 0xcfce3e1, 3 },
    { 0, 0 },
    { 0x9b0e3e3, 3 },
    { 0, 0 },
    { 0xc3dce3e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a44e3ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb160e3f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4260e3f8, 4 },
    { 0, 0 },
    { 0x97b4e3fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98e3ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b08e40b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9710e411, 3 },
    { 0, 0 },
    { 0x3a9ce413, 3 },
    { 0xb3f0e414, 3 },
    { 0xf34e415, 3 },
    { 0x4dc0e415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1780e419, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d0e41c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f8e420, 3 },
    { 0xff54e421, 3 },
    { 0xc1e8e421, 3 },
    { 0x24f4e422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb178e427, 3 },
    { 0x7cfce427, 3 },
    { 0x9ae0e428, 3 },
    { 0, 0 },
    { 0x5c18e42b, 3 },
    { 0xecdce42c, 3 },
    { 0x9f30e42d, 5 },
    { 0, 0 },
    { 0x9ac0e42f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4e436, 3 },
    { 0x4a20e436, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c0e43d, 3 },
    { 0xf4e43d, 3 },
    { 0, 0 },
    { 0xf0a8e440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3298e444, 3 },
    { 0x66b0e445, 3 },
    { 0xc8a8e446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4970e44e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cece454, 3 },
    { 0, 0 },
    { 0x1fdce456, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe58e45a, 3 },
    { 0xd968e45b, 3 },
    { 0xbb7ce45c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af0e464, 3 },
    { 0xdface465, 3 },
    { 0x1198e464, 3 },
    { 0, 0 },
    { 0xe934e468, 3 },
    { 0xb088e468, 4 },
    { 0, 0 },
    { 0x6828e46b, 4 },
    { 0x1038e46c, 3 },
    { 0, 0 },
    { 0xec98e46e, 4 },
    { 0x7e48e46f, 4 },
    { 0x9c0e470, 3 },
    { 0x5b20e471, 3 },
    { 0x5fa8e472, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88e478, 4 },
    { 0, 0 },
    { 0x7d80e47a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0e481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f4e484, 3 },
    { 0xbb90e484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f34e493, 3 },
    { 0xd088e494, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7198e497, 4 },
    { 0, 0 },
    { 0x6ab4e499, 3 },
    { 0x9884e49a, 3 },
    { 0x9b38e49b, 3 },
    { 0, 0 },
    { 0x6a90e49d, 3 },
    { 0xbf80e49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ce4a1, 3 },
    { 0xb674e4a1, 3 },
    { 0x91d0e4a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d0e4a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc544e4ae, 3 },
    { 0xde0e4af, 4 },
    { 0x7b6ce4b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664e4b4, 3 },
    { 0x41ece4b5, 3 },
    { 0xb1d4e4b6, 3 },
    { 0x16d4e4b6, 3 },
    { 0x7c14e4b8, 3 },
    { 0x16c8e4b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ce4bc, 3 },
    { 0x51ce4bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88e4c1, 3 },
    { 0x3c0ce4c2, 3 },
    { 0x5664e4c3, 4 },
    { 0x4a94e4c2, 4 },
    { 0x936ce4c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060e4c8, 3 },
    { 0, 0 },
    { 0x9730e4ca, 3 },
    { 0xd848e4cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7884e4ce, 3 },
    { 0xb8e8e4cf, 3 },
    { 0xc2c0e4d0, 3 },
    { 0x3c34e4d1, 3 },
    { 0, 0 },
    { 0x3a1ce4d3, 4 },
    { 0, 0 },
    { 0x8924e4d5, 3 },
    { 0, 0 },
    { 0xe44e4d7, 4 },
    { 0x3568e4d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc274e4dc, 3 },
    { 0xd5b8e4dc, 3 },
    { 0x8c00e4dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e4e3, 5 },
    { 0x4c10e4e3, 4 },
    { 0x2154e4e5, 3 },
    { 0x4748e4e3, 4 },
    { 0, 0 },
    { 0x690ce4e8, 5 },
    { 0, 0 },
    { 0xadace4ea, 3 },
    { 0x7734e4ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ce4f0, 4 },
    { 0, 0 },
    { 0x1f78e4f2, 3 },
    { 0x162ce4f2, 3 },
    { 0x129ce4f4, 3 },
    { 0xe0f0e4f2, 3 },
    { 0xa748e4f6, 3 },
    { 0x6130e4f5, 3 },
    { 0x1fd0e4f6, 3 },
    { 0x6958e4f3, 3 },
    { 0xb7b0e4f7, 3 },
    { 0x3fb8e4fb, 3 },
    { 0x350e4fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc600e500, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904e503, 3 },
    { 0xa064e503, 3 },
    { 0x3848e505, 3 },
    { 0xfcb8e505, 3 },
    { 0, 0 },
    { 0xa4b0e508, 3 },
    { 0, 0 },
    { 0xd4d4e50a, 3 },
    { 0, 0 },
    { 0x511ce50c, 4 },
    { 0, 0 },
    { 0xffd8e50e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc60e513, 3 },
    { 0x47ece513, 4 },
    { 0x4ca0e515, 4 },
    { 0xaa68e514, 4 },
    { 0xb508e515, 5 },
    { 0xe010e518, 4 },
    { 0x40d0e517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8ce51c, 4 },
    { 0, 0 },
    { 0x3a50e51e, 3 },
    { 0, 0 },
    { 0x5d60e520, 4 },
    { 0x78fce521, 4 },
    { 0x1c94e522, 3 },
    { 0x44a4e520, 3 },
    { 0x15bce524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb0e527, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6818e52f, 3 },
    { 0x8508e530, 3 },
    { 0, 0 },
    { 0x12ece532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329ce537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e0e53a, 3 },
    { 0x4948e53b, 3 },
    { 0xa658e53c, 3 },
    { 0x9d38e53a, 3 },
    { 0, 0 },
    { 0x593ce53f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50e546, 4 },
    { 0x5250e547, 3 },
    { 0x667ce548, 3 },
    { 0x7b90e547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b4e54d, 4 },
    { 0, 0 },
    { 0x7970e54f, 4 },
    { 0xe74ce550, 3 },
    { 0xd120e551, 3 },
    { 0x9aa4e550, 3 },
    { 0x8b78e553, 3 },
    { 0xdc68e554, 3 },
    { 0x3a48e555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5ce55a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa468e55e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf814e561, 3 },
    { 0, 0 },
    { 0x6fc0e563, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc78e568, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b0e570, 3 },
    { 0x6688e571, 3 },
    { 0xad90e572, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x689ce575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffce579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf384e57e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d0e582, 3 },
    { 0x81e8e583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c4e586, 3 },
    { 0xd8e0e586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4e58c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2154e58f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa844e599, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53ce59d, 3 },
    { 0, 0 },
    { 0x9440e59f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2548e5a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c54e5a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130e5ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ece5b6, 4 },
    { 0xe6bce5b7, 3 },
    { 0xc534e5b8, 3 },
    { 0x48d4e5b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d4e5bc, 3 },
    { 0x3da4e5bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef60e5c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa030e5c8, 3 },
    { 0, 0 },
    { 0x16a0e5ca, 5 },
    { 0, 0 },
    { 0x46d4e5cc, 4 },
    { 0x8b8e5cc, 4 },
    { 0xe8cce5cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d38e5d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d00e5dd, 3 },
    { 0x8c5ce5de, 3 },
    { 0x1240e5dd, 3 },
    { 0, 0 },
    { 0x8888e5e1, 3 },
    { 0x487ce5e1, 3 },
    { 0, 0 },
    { 0xd2ace5e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832ce5e8, 3 },
    { 0, 0 },
    { 0xc140e5ea, 3 },
    { 0x93b0e5eb, 3 },
    { 0x1218e5ec, 4 },
    { 0x4140e5ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c88e5f3, 4 },
    { 0, 0 },
    { 0x3a14e5f5, 3 },
    { 0xa52ce5f6, 4 },
    { 0xa2b8e5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf0e5fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c4e600, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3240e607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a4e60c, 4 },
    { 0xeb74e60c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f84e616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb4e61a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df4e61d, 3 },
    { 0, 0 },
    { 0xd4d0e61f, 4 },
    { 0, 0 },
    { 0x7758e621, 3 },
    { 0, 0 },
    { 0x8cc8e623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b80e626, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94e62d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe358e630, 3 },
    { 0, 0 },
    { 0x467ce632, 3 },
    { 0x48b8e632, 3 },
    { 0x4884e632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a0e638, 3 },
    { 0x65b4e639, 3 },
    { 0x1fc8e63a, 4 },
    { 0, 0 },
    { 0x72e8e63c, 3 },
    { 0, 0 },
    { 0xd1c0e63e, 3 },
    { 0, 0 },
    { 0x91b8e640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdce643, 3 },
    { 0, 0 },
    { 0x96a4e645, 3 },
    { 0xfb04e646, 4 },
    { 0x3988e645, 3 },
    { 0xb8b4e648, 4 },
    { 0xd428e649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8e64c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d8e64f, 4 },
    { 0x87dce64f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18e653, 3 },
    { 0x2318e653, 4 },
    { 0x3f5ce655, 3 },
    { 0x5f9ce656, 3 },
    { 0x520ce657, 3 },
    { 0xcd98e655, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64e65b, 4 },
    { 0xea78e65c, 3 },
    { 0xcf5ce65d, 3 },
    { 0x478e65c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2310e661, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1460e665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e98e669, 4 },
    { 0xc1c8e66a, 3 },
    { 0x1b50e66b, 3 },
    { 0xd520e66b, 5 },
    { 0, 0 },
    { 0x2758e66e, 4 },
    { 0x8e64e66f, 4 },
    { 0xccbce66f, 3 },
    { 0x70d8e66f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcce675, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c80e67b, 4 },
    { 0x4d30e67b, 3 },
    { 0, 0 },
    { 0x5400e67e, 3 },
    { 0, 0 },
    { 0x9034e680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8e685, 3 },
    { 0x85ace686, 3 },
    { 0x4bb0e687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf498e68a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5ce68d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b54e690, 3 },
    { 0x804ce690, 3 },
    { 0xd7fce692, 5 },
    { 0x7958e693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a0e696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46ce69e, 3 },
    { 0xf7b0e69f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d8e6a6, 4 },
    { 0xe17ce6a6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f70e6ab, 3 },
    { 0x384ce6ac, 3 },
    { 0x1ed4e6ac, 3 },
    { 0x22c4e6ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e0e6b1, 3 },
    { 0x6174e6b1, 3 },
    { 0xf32ce6b3, 4 },
    { 0xfa64e6b2, 3 },
    { 0x7ce0e6b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd850e6bd, 4 },
    { 0xa78ce6be, 3 },
    { 0x2b68e6bf, 3 },
    { 0xa05ce6bf, 3 },
    { 0x880ce6c1, 3 },
    { 0, 0 },
    { 0xb508e6c3, 4 },
    { 0x5a38e6c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb4e6c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d70e6cb, 3 },
    { 0xf50ce6cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580e6d4, 4 },
    { 0x3214e6d4, 3 },
    { 0xbf1ce6d6, 3 },
    { 0xec9ce6d7, 3 },
    { 0x820ce6d8, 5 },
    { 0xb9ace6d6, 3 },
    { 0xb5e0e6da, 4 },
    { 0, 0 },
    { 0xd4a4e6dc, 4 },
    { 0xe5dce6dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f0e6e4, 4 },
    { 0x2480e6e4, 4 },
    { 0x808ce6e4, 3 },
    { 0xb334e6e7, 4 },
    { 0xdc84e6e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa840e6ee, 4 },
    { 0, 0 },
    { 0x7fb8e6f0, 3 },
    { 0x42f0e6f0, 3 },
    { 0, 0 },
    { 0x31e0e6f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d0e6f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24ce6fc, 4 },
    { 0xb8dce6fc, 3 },
    { 0x8a80e6fc, 3 },
    { 0x8e08e6ff, 3 },
    { 0xbb4e6fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce705, 3 },
    { 0xf10e706, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1ce70d, 3 },
    { 0, 0 },
    { 0x2248e70f, 3 },
    { 0x3094e710, 3 },
    { 0x8930e711, 3 },
    { 0xe520e70f, 4 },
    { 0xff8e713, 3 },
    { 0xfd04e714, 3 },
    { 0xeef4e714, 3 },
    { 0x78a4e713, 4 },
    { 0x6d20e717, 3 },
    { 0xba94e716, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f68e71d, 4 },
    { 0x6c90e71e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c70e721, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5460e727, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370e72b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838e735, 4 },
    { 0xd740e736, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b0e73d, 4 },
    { 0x6b50e73d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f78e745, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d0e749, 4 },
    { 0, 0 },
    { 0xe7ece74b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750e74f, 3 },
    { 0xe38ce750, 4 },
    { 0, 0 },
    { 0x26d8e752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2134e757, 3 },
    { 0xc258e757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e75c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe474e75f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc70e762, 3 },
    { 0, 0 },
    { 0x7c44e764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8e76b, 4 },
    { 0x2ece76b, 4 },
    { 0, 0 },
    { 0x1784e76e, 3 },
    { 0, 0 },
    { 0x9a74e770, 3 },
    { 0, 0 },
    { 0x81e4e772, 3 },
    { 0xf5f8e773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dce778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4e77d, 3 },
    { 0x13f0e77e, 3 },
    { 0, 0 },
    { 0x7348e780, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640e785, 5 },
    { 0xc554e786, 3 },
    { 0x7684e787, 3 },
    { 0x1668e788, 5 },
    { 0, 0 },
    { 0xbc80e78a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ce78d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf94e797, 4 },
    { 0, 0 },
    { 0xef2ce799, 3 },
    { 0, 0 },
    { 0x6668e79b, 3 },
    { 0xa444e79c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4914e7a0, 3 },
    { 0xc7c4e7a1, 4 },
    { 0, 0 },
    { 0x4654e7a3, 3 },
    { 0xf54e7a4, 4 },
    { 0, 0 },
    { 0x212ce7a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8544e7ab, 3 },
    { 0xb2fce7ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf520e7b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5024e7ba, 3 },
    { 0x2370e7ba, 3 },
    { 0xe98e7bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba54e7c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d80e7c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d74e7c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6794e7d3, 3 },
    { 0xfc50e7d4, 3 },
    { 0, 0 },
    { 0x6714e7d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc874e7db, 3 },
    { 0xdddce7dc, 3 },
    { 0, 0 },
    { 0x62cce7de, 4 },
    { 0xb210e7de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e7e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc608e7eb, 4 },
    { 0x9d48e7ec, 3 },
    { 0x76e4e7ec, 4 },
    { 0xb974e7ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7214e7f4, 3 },
    { 0x67ace7f5, 4 },
    { 0x33bce7f4, 3 },
    { 0x64f4e7f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4264e7fb, 3 },
    { 0x782ce7fc, 3 },
    { 0xb64ce7fd, 4 },
    { 0, 0 },
    { 0xfa28e7ff, 3 },
    { 0xc4c8e800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e4e803, 4 },
    { 0xbd58e804, 4 },
    { 0xc18e805, 3 },
    { 0xb898e806, 3 },
    { 0xe854e806, 3 },
    { 0x8ec4e808, 3 },
    { 0x6e78e807, 3 },
    { 0xac2ce80a, 3 },
    { 0xd5d8e80a, 3 },
    { 0xbd00e80a, 4 },
    { 0x2f28e805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf494e817, 3 },
    { 0x821ce817, 3 },
    { 0xb3e4e818, 3 },
    { 0xc94e817, 4 },
    { 0xd5d4e81b, 4 },
    { 0x8c8e81a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e28e81f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ce823, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce4e828, 3 },
    { 0x1c6ce829, 4 },
    { 0xd044e829, 4 },
    { 0x5d28e828, 4 },
    { 0xe8a8e82c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614e831, 4 },
    { 0xe9ece832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a58e836, 4 },
    { 0x33f4e837, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ce840, 3 },
    { 0xc054e841, 4 },
    { 0xc2ece842, 3 },
    { 0x4cb0e841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeece848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b0e84c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60e850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50e85a, 3 },
    { 0x9de0e85b, 4 },
    { 0, 0 },
    { 0xbf5ce85d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6640e862, 3 },
    { 0x48c8e863, 3 },
    { 0, 0 },
    { 0xac1ce865, 3 },
    { 0xbfb0e866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d84e86b, 3 },
    { 0x5eace86b, 3 },
    { 0, 0 },
    { 0x28cce86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe688e871, 3 },
    { 0x829ce872, 3 },
    { 0xc7f4e873, 3 },
    { 0, 0 },
    { 0x4ea8e875, 3 },
    { 0xd73ce875, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9ce87f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf0e882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db4e88b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d48e891, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e60e8a2, 4 },
    { 0, 0 },
    { 0x959ce8a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1778e8ae, 3 },
    { 0x1098e8af, 3 },
    { 0, 0 },
    { 0x329ce8b1, 3 },
    { 0x3fdce8b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd34e8b5, 3 },
    { 0x6b1ce8b5, 3 },
    { 0xc5a8e8b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa34e8bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6ce8c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfece8c4, 3 },
    { 0, 0 },
    { 0x6128e8c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2674e8ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28e8cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd80e8d4, 4 },
    { 0, 0 },
    { 0xab74e8d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0e8da, 3 },
    { 0xa2b4e8db, 4 },
    { 0x45d0e8db, 3 },
    { 0, 0 },
    { 0x8f68e8de, 4 },
    { 0xaaf4e8df, 4 },
    { 0x7c8ce8de, 3 },
    { 0x5bd8e8e1, 4 },
    { 0xc020e8e2, 3 },
    { 0x595ce8de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95ce8e8, 4 },
    { 0xd464e8e9, 4 },
    { 0xd028e8ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27ce8f3, 3 },
    { 0, 0 },
    { 0x7104e8f5, 3 },
    { 0x72e8e8f5, 3 },
    { 0x7cdce8f7, 3 },
    { 0x5448e8f8, 3 },
    { 0xfdd0e8f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c0e901, 3 },
    { 0xde08e902, 3 },
    { 0, 0 },
    { 0x5abce904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6ce907, 4 },
    { 0, 0 },
    { 0xed40e909, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e4e910, 3 },
    { 0xe094e911, 3 },
    { 0, 0 },
    { 0xacfce913, 4 },
    { 0, 0 },
    { 0xab8ce915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x428ce918, 3 },
    { 0xf3dce918, 4 },
    { 0x2d44e91a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a4e91f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f8e922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7694e92b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c54e92e, 3 },
    { 0x1970e92f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d0e938, 3 },
    { 0x17ece938, 3 },
    { 0xef04e93a, 3 },
    { 0x8110e939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b8e942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bce94b, 4 },
    { 0x3fa0e94c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e4e94f, 4 },
    { 0x9f5ce94f, 4 },
    { 0xe814e951, 4 },
    { 0xa958e951, 3 },
    { 0x1598e953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8e956, 3 },
    { 0x26f8e956, 3 },
    { 0xb528e958, 3 },
    { 0xd30ce958, 3 },
    { 0x34d8e95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf0e95e, 4 },
    { 0x6d5ce95f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f84e962, 3 },
    { 0, 0 },
    { 0xe8b0e964, 3 },
    { 0xe82ce964, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ce96c, 3 },
    { 0x674ce96d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0e971, 3 },
    { 0, 0 },
    { 0x8d04e973, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08e978, 3 },
    { 0, 0 },
    { 0x32e0e97a, 3 },
    { 0x1450e97b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e97e, 4 },
    { 0, 0 },
    { 0x7108e980, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbce985, 3 },
    { 0xd4e4e986, 3 },
    { 0x24a4e987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d8e98b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfce990, 4 },
    { 0x59fce991, 3 },
    { 0x10ece992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a0e995, 4 },
    { 0, 0 },
    { 0x84d8e997, 3 },
    { 0x4bdce997, 4 },
    { 0x6c68e997, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e0e99c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb490e99f, 3 },
    { 0xc308e9a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4598e9a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8e9a6, 3 },
    { 0x65b8e9a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c0e9b1, 4 },
    { 0x1378e9b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778e9b5, 3 },
    { 0x7648e9b5, 3 },
    { 0x840e9b5, 3 },
    { 0x21bce9b8, 3 },
    { 0, 0 },
    { 0xc7d8e9ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea4e9bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50ce9c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac08e9c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8ce9cc, 5 },
    { 0, 0 },
    { 0x3060e9ce, 4 },
    { 0xb730e9cf, 3 },
    { 0xfad4e9d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6050e9d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4694e9df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ce9e2, 3 },
    { 0x15a0e9e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960e9ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ece9f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8098e9f6, 3 },
    { 0x2fcce9f7, 3 },
    { 0, 0 },
    { 0xd364e9f9, 3 },
    { 0xb5d0e9fa, 3 },
    { 0, 0 },
    { 0x1730e9fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e8ea01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48cea04, 4 },
    { 0x4790ea05, 3 },
    { 0xde08ea06, 4 },
    { 0xab90ea07, 4 },
    { 0, 0 },
    { 0x76f0ea09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58ea0d, 3 },
    { 0xd3a8ea0d, 3 },
    { 0x5e78ea0f, 3 },
    { 0xea94ea0f, 3 },
    { 0x4850ea11, 3 },
    { 0xcce4ea12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000ea15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fcea1a, 5 },
    { 0x278ea1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4768ea1e, 3 },
    { 0x69d8ea1f, 4 },
    { 0x9a14ea20, 3 },
    { 0x981cea1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a38ea26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594ea29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78ea2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044ea30, 3 },
    { 0, 0 },
    { 0x9890ea32, 3 },
    { 0x9c2cea32, 3 },
    { 0x740ea34, 3 },
    { 0x3734ea35, 3 },
    { 0x1f88ea36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55cea39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa10ea42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a8ea47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1acea4e, 3 },
    { 0x8ab4ea4e, 3 },
    { 0x51f0ea50, 3 },
    { 0, 0 },
    { 0xf1e4ea52, 3 },
    { 0x5cfcea52, 4 },
    { 0x955cea54, 3 },
    { 0x9a04ea55, 4 },
    { 0xf504ea56, 3 },
    { 0x3b50ea56, 3 },
    { 0xf00ea56, 3 },
    { 0x3228ea53, 3 },
    { 0, 0 },
    { 0xfe5cea5b, 3 },
    { 0x903cea5c, 4 },
    { 0x1efcea5d, 3 },
    { 0x2328ea5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134cea63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90cea71, 4 },
    { 0xe6e0ea71, 3 },
    { 0xcca0ea73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e58ea76, 3 },
    { 0x7c3cea77, 3 },
    { 0xfbd4ea78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808ea7c, 3 },
    { 0xbd30ea7d, 3 },
    { 0, 0 },
    { 0xe070ea7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8670ea89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0cea90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x508ea93, 3 },
    { 0x71d0ea94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6950ea9e, 4 },
    { 0, 0 },
    { 0xea2ceaa0, 4 },
    { 0x9ba4eaa1, 5 },
    { 0, 0 },
    { 0x3310eaa3, 3 },
    { 0, 0 },
    { 0xf938eaa5, 3 },
    { 0xf588eaa6, 3 },
    { 0, 0 },
    { 0x51b8eaa8, 3 },
    { 0x42b0eaa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f4eaac, 4 },
    { 0x2990eaac, 3 },
    { 0xe42ceaae, 3 },
    { 0x2dceaaf, 3 },
    { 0, 0 },
    { 0x95dceab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9818eab5, 5 },
    { 0, 0 },
    { 0x27cceab7, 3 },
    { 0x1d48eab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eceabb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a0eabf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fceac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0eac8, 3 },
    { 0, 0 },
    { 0xf6d4eaca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd8ead1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0ead7, 3 },
    { 0x25dcead8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff04eadb, 3 },
    { 0, 0 },
    { 0x5724eadd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4eae1, 4 },
    { 0x15c8eae2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c0eae6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544eae9, 3 },
    { 0x5a4eaea, 4 },
    { 0, 0 },
    { 0x80e0eaec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb14eaf1, 3 },
    { 0xc9e0eaf2, 3 },
    { 0xc8e4eaf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08eaf9, 3 },
    { 0xb828eaf9, 3 },
    { 0x2028eaf9, 3 },
    { 0, 0 },
    { 0xa0f0eafd, 4 },
    { 0, 0 },
    { 0xb2eceaff, 4 },
    { 0, 0 },
    { 0x8204eb01, 3 },
    { 0xb120eb02, 3 },
    { 0, 0 },
    { 0xf84ceb04, 4 },
    { 0x1ef8eb04, 4 },
    { 0x9748eb06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570eb0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ceb11, 3 },
    { 0xc0f4eb11, 4 },
    { 0xeccceb12, 3 },
    { 0x6bb4eb12, 3 },
    { 0, 0 },
    { 0xd658eb16, 3 },
    { 0xcc0ceb17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0eb1a, 3 },
    { 0x9d90eb1b, 3 },
    { 0xbac4eb1c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e18eb20, 3 },
    { 0x4b98eb21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb8eb25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb4eb2a, 3 },
    { 0x2f8ceb2b, 3 },
    { 0xb030eb2a, 3 },
    { 0, 0 },
    { 0xfabceb2e, 3 },
    { 0x57c4eb2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a8eb33, 3 },
    { 0, 0 },
    { 0xd438eb35, 4 },
    { 0x7f94eb36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10eb3a, 4 },
    { 0xac40eb3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4030eb3e, 4 },
    { 0x59a8eb3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e8eb44, 4 },
    { 0x3194eb45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974eb4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48eb51, 3 },
    { 0, 0 },
    { 0xbf4eb53, 4 },
    { 0x81b8eb53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5ceb5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5758eb62, 4 },
    { 0, 0 },
    { 0x7eaceb64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d4eb67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8eb72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a4eb76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x947ceb79, 3 },
    { 0x6cfceb79, 3 },
    { 0x5084eb7b, 3 },
    { 0, 0 },
    { 0x2230eb7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eceb83, 4 },
    { 0xab50eb83, 4 },
    { 0xd3eceb85, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850eb89, 3 },
    { 0x6b6ceb8a, 3 },
    { 0xe7c4eb8b, 4 },
    { 0, 0 },
    { 0xfbbceb8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x110ceb90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532ceb97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab8eb9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc800eba1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb4eba4, 3 },
    { 0xca94eba5, 3 },
    { 0xcef8eba6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cccebab, 4 },
    { 0xf318ebac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fcebb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e38ebc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c8ebc3, 3 },
    { 0, 0 },
    { 0xc870ebc5, 3 },
    { 0xc57cebc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9cebca, 3 },
    { 0x163cebcb, 3 },
    { 0, 0 },
    { 0xb97cebcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ccebd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a8ebd7, 3 },
    { 0x9744ebd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d4ebdd, 3 },
    { 0x3148ebdd, 3 },
    { 0, 0 },
    { 0xb7c8ebe0, 4 },
    { 0xd00cebe1, 4 },
    { 0x694ebe2, 3 },
    { 0xa684ebe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f68ebe7, 3 },
    { 0x12c8ebe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8ebed, 3 },
    { 0xcf64ebee, 3 },
    { 0xd514ebee, 3 },
    { 0, 0 },
    { 0xaed4ebf1, 4 },
    { 0x3014ebf2, 3 },
    { 0xe4dcebf1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8ebf8, 3 },
    { 0xdb40ebf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dcebfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18ec01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d58ec04, 3 },
    { 0x78a8ec05, 3 },
    { 0, 0 },
    { 0x6b78ec07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4ec0a, 4 },
    { 0, 0 },
    { 0xca74ec0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ecec12, 3 },
    { 0xd1cec13, 3 },
    { 0xa73cec13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd0ec1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7cec21, 4 },
    { 0x2140ec21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774cec27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01cec2c, 4 },
    { 0x8f78ec2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1cec31, 3 },
    { 0xd2b4ec32, 3 },
    { 0, 0 },
    { 0xbf7cec34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad64ec37, 3 },
    { 0, 0 },
    { 0x5354ec39, 3 },
    { 0xa858ec39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf530ec43, 4 },
    { 0xee58ec44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3740ec48, 4 },
    { 0x80d8ec48, 3 },
    { 0xa334ec4a, 3 },
    { 0, 0 },
    { 0x597cec4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb80ec4f, 3 },
    { 0, 0 },
    { 0x925cec51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f4ec58, 5 },
    { 0x2928ec59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9cec62, 3 },
    { 0x8774ec63, 4 },
    { 0xbc38ec64, 3 },
    { 0xdbecec63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8560ec68, 3 },
    { 0x9f20ec69, 3 },
    { 0xe4ecec6a, 4 },
    { 0xe10cec6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8980ec6f, 4 },
    { 0, 0 },
    { 0x57c0ec71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae4ec76, 3 },
    { 0x3ca8ec76, 3 },
    { 0x8938ec78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6204ec7b, 4 },
    { 0xe2b8ec7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb274ec80, 3 },
    { 0x776cec80, 3 },
    { 0x48e8ec82, 3 },
    { 0, 0 },
    { 0xe2dcec84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cacec8a, 3 },
    { 0xaa2cec8b, 4 },
    { 0, 0 },
    { 0xc8acec8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5838ec90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cecec94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c0ec99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xece8ec9c, 3 },
    { 0xde74ec9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3228eca1, 3 },
    { 0, 0 },
    { 0x769ceca3, 3 },
    { 0xef6ceca4, 4 },
    { 0xb4cceca5, 3 },
    { 0, 0 },
    { 0xac18eca7, 3 },
    { 0x44e4eca7, 3 },
    { 0, 0 },
    { 0x44b8ecaa, 4 },
    { 0xec08ecaa, 3 },
    { 0, 0 },
    { 0x29cecad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d48ecb4, 3 },
    { 0xa7e8ecb5, 4 },
    { 0x7144ecb6, 4 },
    { 0x7068ecb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa74ecbb, 3 },
    { 0, 0 },
    { 0x82f0ecbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0ecc0, 3 },
    { 0xfb40ecc1, 4 },
    { 0xc824ecc0, 3 },
    { 0, 0 },
    { 0x5148ecc4, 3 },
    { 0xc4e4ecc4, 3 },
    { 0, 0 },
    { 0x4cd8ecc7, 3 },
    { 0x6a94ecc8, 3 },
    { 0xaa00ecc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3148ecd1, 3 },
    { 0xbbacecd2, 5 },
    { 0, 0 },
    { 0xb96cecd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4ecda, 3 },
    { 0x1f60ecdb, 3 },
    { 0xae6cecdc, 4 },
    { 0x2958ecdd, 4 },
    { 0, 0 },
    { 0xae20ecdf, 4 },
    { 0x71b0ece0, 4 },
    { 0xab98ece1, 3 },
    { 0x115cece0, 4 },
    { 0, 0 },
    { 0x66bcece4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d00ece7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669ceceb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc734ecee, 3 },
    { 0x47f4ecef, 3 },
    { 0x4a98ecef, 3 },
    { 0, 0 },
    { 0xe910ecf2, 4 },
    { 0, 0 },
    { 0xe7a4ecf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1564ecf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd14ecff, 3 },
    { 0, 0 },
    { 0x8920ed01, 4 },
    { 0x86c8ed02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca04ed05, 3 },
    { 0, 0 },
    { 0x71fced07, 3 },
    { 0x1718ed08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc130ed0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ced1b, 4 },
    { 0x68d4ed1b, 3 },
    { 0x24b0ed1d, 4 },
    { 0xc7e0ed1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4ed23, 3 },
    { 0x82a8ed24, 3 },
    { 0, 0 },
    { 0xf294ed26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ced2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d84ed2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800ed35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f4ed38, 3 },
    { 0x883ced39, 3 },
    { 0, 0 },
    { 0xfaa0ed3b, 3 },
    { 0xd034ed3b, 3 },
    { 0, 0 },
    { 0xfab0ed3e, 3 },
    { 0xff78ed3f, 4 },
    { 0x2b38ed40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc558ed45, 3 },
    { 0xbe0ced45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b28ed49, 3 },
    { 0, 0 },
    { 0xe3d8ed4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ced4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278ed55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb970ed58, 4 },
    { 0x4790ed58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c8ed61, 3 },
    { 0x3f04ed62, 3 },
    { 0x2b4ed62, 4 },
    { 0x6440ed64, 3 },
    { 0x9b08ed65, 4 },
    { 0x8570ed64, 3 },
    { 0x9204ed65, 5 },
    { 0x6530ed68, 3 },
    { 0x69c0ed69, 3 },
    { 0xea44ed69, 3 },
    { 0x3f18ed6b, 3 },
    { 0xf97ced69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47ced73, 3 },
    { 0x357ced73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x204ed77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068ed7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec0ed84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7350ed8a, 4 },
    { 0, 0 },
    { 0xfcf4ed8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe130ed91, 3 },
    { 0xc83ced92, 3 },
    { 0xda08ed91, 3 },
    { 0x3284ed94, 3 },
    { 0xe0a4ed92, 3 },
    { 0x5b50ed96, 3 },
    { 0xb97ced97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0ced9d, 4 },
    { 0x9d48ed9d, 3 },
    { 0x4a20ed9f, 4 },
    { 0, 0 },
    { 0x90bceda1, 4 },
    { 0xb2eceda2, 3 },
    { 0x8ca0eda2, 4 },
    { 0x87eceda4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e28edaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8edae, 3 },
    { 0xe210edaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb4edb2, 3 },
    { 0, 0 },
    { 0xbd0edb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0edb8, 3 },
    { 0x4fe8edb9, 3 },
    { 0x1038edb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde44edbd, 3 },
    { 0x5804edbd, 4 },
    { 0x9f08edbf, 3 },
    { 0, 0 },
    { 0x1640edc1, 4 },
    { 0xd250edc1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x494edc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cedca, 4 },
    { 0xf4ccedca, 3 },
    { 0x7280edcc, 4 },
    { 0x1a54edcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ecedd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8164edd6, 3 },
    { 0x5db0edd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5758edda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb0ede0, 3 },
    { 0x80a0ede0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4598edeb, 3 },
    { 0xdac0edec, 4 },
    { 0xf5ccedec, 4 },
    { 0xbfecedec, 3 },
    { 0x4fecedef, 3 },
    { 0x4868edef, 3 },
    { 0x88e4edf0, 4 },
    { 0xf714edf2, 3 },
    { 0xbdccedeb, 4 },
    { 0xff30edf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc4edf7, 3 },
    { 0, 0 },
    { 0x4a68edf9, 4 },
    { 0x634edfa, 4 },
    { 0, 0 },
    { 0x32ccedfc, 3 },
    { 0xe53cedfd, 3 },
    { 0x77b8edfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa884ee07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf4ee0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf964ee11, 3 },
    { 0, 0 },
    { 0xa2cee13, 3 },
    { 0, 0 },
    { 0xf810ee15, 3 },
    { 0x7418ee16, 4 },
    { 0x8780ee17, 3 },
    { 0x2cecee18, 3 },
    { 0x21f8ee19, 3 },
    { 0xe070ee18, 3 },
    { 0x6ffcee1a, 3 },
    { 0xe6c8ee1c, 4 },
    { 0x45b8ee17, 3 },
    { 0x3a5cee1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacee21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754ee2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd0ee2e, 4 },
    { 0, 0 },
    { 0x309cee30, 3 },
    { 0x1738ee31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1cee36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a90ee39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1360ee3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ccee45, 3 },
    { 0xad80ee46, 4 },
    { 0xc1b8ee46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b70ee4c, 4 },
    { 0, 0 },
    { 0x6138ee4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf130ee56, 4 },
    { 0x5ba8ee57, 3 },
    { 0x6604ee57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f20ee60, 3 },
    { 0x5eecee61, 3 },
    { 0x5f48ee60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d0ee65, 3 },
    { 0xc03cee66, 3 },
    { 0, 0 },
    { 0x95ecee68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8ee6c, 3 },
    { 0xac60ee6d, 4 },
    { 0x7168ee6e, 4 },
    { 0xdf98ee6d, 5 },
    { 0x2884ee70, 3 },
    { 0, 0 },
    { 0xe414ee72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe074ee7a, 4 },
    { 0xcb90ee7a, 5 },
    { 0, 0 },
    { 0x4f28ee7d, 3 },
    { 0x33dcee7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8518ee86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8184ee89, 3 },
    { 0xfd44ee8a, 3 },
    { 0, 0 },
    { 0x1e94ee8c, 4 },
    { 0x6190ee8c, 3 },
    { 0, 0 },
    { 0xd99cee8f, 3 },
    { 0x2484ee90, 3 },
    { 0x8ea4ee91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0cee9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb8eea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe238eea5, 4 },
    { 0, 0 },
    { 0xcf48eea7, 3 },
    { 0x8514eea8, 3 },
    { 0xe780eea9, 3 },
    { 0x8224eeaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fceead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f30eeb1, 3 },
    { 0x8670eeb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0eeb6, 3 },
    { 0x7238eeb7, 3 },
    { 0x95c4eeb8, 3 },
    { 0x2b1ceeb9, 3 },
    { 0x3278eeb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f8eebe, 4 },
    { 0, 0 },
    { 0xf4f8eec0, 5 },
    { 0x2974eec1, 3 },
    { 0x6b74eec2, 3 },
    { 0xbae4eec0, 3 },
    { 0x3ca0eec4, 4 },
    { 0x40b0eec4, 3 },
    { 0x23a4eec6, 3 },
    { 0x6134eec6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d4eecb, 3 },
    { 0x5fd4eecc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ceeda, 3 },
    { 0, 0 },
    { 0xac64eedc, 5 },
    { 0, 0 },
    { 0x2a0eede, 3 },
    { 0x2e68eedf, 3 },
    { 0, 0 },
    { 0x1064eee1, 3 },
    { 0x1f84eee2, 3 },
    { 0, 0 },
    { 0xfad0eee4, 4 },
    { 0x32ceee5, 3 },
    { 0xbe08eee6, 3 },
    { 0x5f94eee5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f0eef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf378eef4, 4 },
    { 0x8778eef5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba74eefb, 3 },
    { 0x1284eefc, 3 },
    { 0xe728eefc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f4ef09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8464ef0c, 4 },
    { 0x9e9cef0d, 4 },
    { 0, 0 },
    { 0xc558ef0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc444ef12, 3 },
    { 0xb378ef13, 3 },
    { 0xa980ef14, 3 },
    { 0, 0 },
    { 0xb760ef16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4434ef1a, 4 },
    { 0x18e4ef1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17cef21, 4 },
    { 0x1450ef21, 4 },
    { 0xb49cef22, 3 },
    { 0xdd54ef24, 3 },
    { 0xcfd0ef25, 4 },
    { 0xbe2cef26, 3 },
    { 0, 0 },
    { 0x7938ef28, 4 },
    { 0, 0 },
    { 0xcd5cef2a, 3 },
    { 0x8b48ef2b, 4 },
    { 0x3b60ef2a, 3 },
    { 0, 0 },
    { 0x160cef2e, 3 },
    { 0xb8ccef2f, 3 },
    { 0x8444ef30, 3 },
    { 0xef1cef31, 3 },
    { 0x2c0ef2e, 3 },
    { 0x7910ef31, 4 },
    { 0xce8cef2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2040ef38, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf394ef3b, 3 },
    { 0x9608ef3b, 3 },
    { 0, 0 },
    { 0x116cef3e, 3 },
    { 0, 0 },
    { 0x8200ef40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a38ef46, 4 },
    { 0xf5d4ef47, 5 },
    { 0x6868ef47, 4 },
    { 0xf334ef47, 3 },
    { 0x7cd8ef4a, 3 },
    { 0x3ac8ef4b, 3 },
    { 0x73ecef4c, 3 },
    { 0xb874ef49, 4 },
    { 0xbff8ef4e, 3 },
    { 0x492cef4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85cef56, 4 },
    { 0x253cef57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe324ef5a, 3 },
    { 0xd8bcef5a, 3 },
    { 0x97b8ef5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cef5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e94ef62, 4 },
    { 0x8d84ef62, 3 },
    { 0xd184ef63, 3 },
    { 0x5224ef65, 3 },
    { 0x3534ef66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c0ef6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb8ef71, 4 },
    { 0xaa64ef71, 3 },
    { 0x9054ef73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28ef76, 3 },
    { 0xc070ef77, 3 },
    { 0x32c8ef76, 3 },
    { 0x295cef79, 3 },
    { 0x31b8ef77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414ef83, 3 },
    { 0x27b8ef84, 3 },
    { 0xbc94ef83, 4 },
    { 0x46f8ef85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4ef8f, 4 },
    { 0, 0 },
    { 0x1f34ef91, 4 },
    { 0x2230ef92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fcef95, 4 },
    { 0, 0 },
    { 0x7f20ef97, 3 },
    { 0x108cef98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea0ef9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700efa2, 3 },
    { 0, 0 },
    { 0x1f08efa4, 4 },
    { 0, 0 },
    { 0x72b8efa6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b50efa9, 4 },
    { 0x5ac0efaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ccefad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eecefb0, 3 },
    { 0x5b70efb1, 3 },
    { 0x106cefb2, 3 },
    { 0xcd74efb1, 3 },
    { 0x7c38efb4, 3 },
    { 0x6bacefb4, 5 },
    { 0x4bb8efb6, 3 },
    { 0xcd90efb7, 3 },
    { 0x238efb6, 3 },
    { 0xeda0efb4, 3 },
    { 0, 0 },
    { 0x8a18efbb, 3 },
    { 0xa7e4efbb, 3 },
    { 0xb04cefbd, 5 },
    { 0, 0 },
    { 0xf2d4efbf, 3 },
    { 0x6104efc0, 3 },
    { 0x78b0efc1, 3 },
    { 0xcd34efc2, 3 },
    { 0xb3c4efc3, 3 },
    { 0x9c50efc4, 4 },
    { 0x7974efc5, 3 },
    { 0xc3acefc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x705cefc9, 3 },
    { 0xcf44efc9, 3 },
    { 0, 0 },
    { 0x45f8efcc, 3 },
    { 0xc300efcd, 4 },
    { 0, 0 },
    { 0xcd44efcf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a80efd8, 4 },
    { 0, 0 },
    { 0x6b64efda, 3 },
    { 0x978efdb, 3 },
    { 0x2bf0efdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb318efe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ecefe6, 3 },
    { 0xd6b4efe7, 3 },
    { 0x2e94efe6, 3 },
    { 0, 0 },
    { 0xdb64efea, 3 },
    { 0, 0 },
    { 0x7d98efec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc704effd, 3 },
    { 0x6870effe, 3 },
    { 0x4abcefff, 4 },
    { 0x1420effd, 4 },
    { 0, 0 },
    { 0x39a4f002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb0f006, 3 },
    { 0xa07cf006, 3 },
    { 0xdf3cf008, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d8f00b, 3 },
    { 0, 0 },
    { 0x7c28f00d, 3 },
    { 0x57c0f00e, 3 },
    { 0x37d0f00f, 4 },
    { 0xf760f010, 3 },
    { 0xdbe4f011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed4f018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bcf01b, 3 },
    { 0x9900f01b, 3 },
    { 0x9864f01d, 3 },
    { 0xca08f01d, 3 },
    { 0x8d78f01e, 3 },
    { 0xe828f020, 5 },
    { 0x8cd0f01f, 3 },
    { 0, 0 },
    { 0x7554f023, 3 },
    { 0xa228f024, 4 },
    { 0xa17cf024, 4 },
    { 0x5e04f024, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdcf02a, 4 },
    { 0xcacf02a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a8f032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec58f03a, 3 },
    { 0, 0 },
    { 0x3574f03c, 3 },
    { 0xf1d8f03d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a4f041, 3 },
    { 0, 0 },
    { 0x400f043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b18f046, 3 },
    { 0x2ca4f046, 3 },
    { 0x1cf8f048, 4 },
    { 0x2694f048, 3 },
    { 0x5d54f049, 3 },
    { 0, 0 },
    { 0x6ac8f04c, 3 },
    { 0, 0 },
    { 0xba04f04e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fcf054, 4 },
    { 0x7a8f055, 3 },
    { 0x8814f054, 3 },
    { 0xf680f055, 3 },
    { 0x5aecf057, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e78f05b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d0f060, 3 },
    { 0x2a3cf060, 3 },
    { 0x6cacf062, 5 },
    { 0, 0 },
    { 0xd494f064, 3 },
    { 0, 0 },
    { 0x14ccf066, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cf06a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2cf06d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7260f070, 3 },
    { 0, 0 },
    { 0x6794f072, 4 },
    { 0x9a38f072, 3 },
    { 0x51f0f072, 3 },
    { 0x1cdcf072, 3 },
    { 0xbe34f073, 3 },
    { 0x70dcf077, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6cf07d, 3 },
    { 0, 0 },
    { 0x2cbcf07f, 3 },
    { 0x41b4f080, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa980f085, 4 },
    { 0, 0 },
    { 0x8ab4f087, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e24f08a, 3 },
    { 0, 0 },
    { 0x79d4f08c, 3 },
    { 0x7058f08d, 3 },
    { 0xbd90f08e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3940f093, 3 },
    { 0, 0 },
    { 0xcc80f095, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac28f09a, 3 },
    { 0xa3e4f09a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ecf0a1, 3 },
    { 0xca88f0a2, 4 },
    { 0x295cf0a3, 3 },
    { 0xd3c0f0a4, 3 },
    { 0x6f20f0a5, 4 },
    { 0x17ecf0a3, 3 },
    { 0x7230f0a1, 3 },
    { 0, 0 },
    { 0xbd18f0a9, 3 },
    { 0x6850f0aa, 4 },
    { 0x55b0f0a9, 3 },
    { 0x791cf0ac, 3 },
    { 0, 0 },
    { 0x3774f0ae, 3 },
    { 0, 0 },
    { 0x6310f0b0, 3 },
    { 0x8c78f0b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4cf0b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd428f0b7, 3 },
    { 0, 0 },
    { 0xc088f0b9, 4 },
    { 0x2d24f0ba, 4 },
    { 0xd074f0ba, 4 },
    { 0xb1c4f0bc, 3 },
    { 0x6790f0ba, 3 },
    { 0x3ad8f0be, 3 },
    { 0x5118f0bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8f0c4, 3 },
    { 0x6aacf0c4, 3 },
    { 0xd0cf0c4, 3 },
    { 0x1828f0c7, 3 },
    { 0xede8f0c7, 3 },
    { 0x42e0f0c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x650cf0cc, 4 },
    { 0, 0 },
    { 0x39a0f0ce, 3 },
    { 0x37ccf0cf, 3 },
    { 0x4024f0cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3928f0d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e80f0dd, 3 },
    { 0, 0 },
    { 0x108cf0df, 3 },
    { 0x83ccf0df, 3 },
    { 0x6110f0df, 3 },
    { 0x5b70f0e2, 3 },
    { 0x4820f0e2, 3 },
    { 0xbfdcf0e4, 3 },
    { 0x3abcf0e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755cf0eb, 3 },
    { 0, 0 },
    { 0xa4cf0ed, 3 },
    { 0x7670f0ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b18f0f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x245cf0f7, 4 },
    { 0, 0 },
    { 0xa22cf0f9, 3 },
    { 0x7a6cf0fa, 3 },
    { 0x5bf4f0fb, 3 },
    { 0x668f0fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40f106, 4 },
    { 0x83a0f107, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd444f10a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b20f10d, 3 },
    { 0, 0 },
    { 0x1284f10f, 3 },
    { 0x98d8f110, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e4f114, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f0f117, 3 },
    { 0, 0 },
    { 0xe328f119, 3 },
    { 0xbd24f119, 3 },
    { 0x1ef0f119, 3 },
    { 0x228cf11c, 3 },
    { 0x90d0f11d, 3 },
    { 0x60f8f11d, 3 },
    { 0x26b0f11f, 4 },
    { 0xb800f120, 5 },
    { 0xb3f8f121, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8f124, 3 },
    { 0, 0 },
    { 0x4ae4f126, 3 },
    { 0x7104f127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdcf12a, 4 },
    { 0x1e80f12b, 3 },
    { 0x92acf12b, 3 },
    { 0xfbb8f12d, 3 },
    { 0xf400f12d, 3 },
    { 0x5c60f12f, 3 },
    { 0x4d68f130, 3 },
    { 0xa790f131, 4 },
    { 0x9760f132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd4f137, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654cf144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0f148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ccf14b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xee80f14e, 4 },
    { 0xaea8f14f, 3 },
    { 0, 0 },
    { 0x8398f151, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c8f156, 3 },
    { 0x23f8f156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f0f15a, 4 },
    { 0x712cf15b, 3 },
    { 0x4f80f15c, 3 },
    { 0x9874f15b, 3 },
    { 0x4ba8f15c, 3 },
    { 0x254f15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0f163, 3 },
    { 0xd140f164, 3 },
    { 0x8730f165, 3 },
    { 0x21e0f164, 3 },
    { 0, 0 },
    { 0xc3d8f168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa4f16d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5cf170, 3 },
    { 0x35e4f171, 3 },
    { 0, 0 },
    { 0xc4a8f173, 3 },
    { 0x1638f173, 3 },
    { 0, 0 },
    { 0x1c58f176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ecf17c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9158f185, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cf18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f18d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb8f1a0, 3 },
    { 0x337cf1a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3138f1a5, 3 },
    { 0, 0 },
    { 0xb830f1a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb8f1b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924f1b6, 3 },
    { 0x3054f1b6, 3 },
    { 0xabb4f1b8, 3 },
    { 0xf0c4f1b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360f1be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8cf1c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf754f1c7, 3 },
    { 0x2a84f1c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7384f1cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96cf1ce, 4 },
    { 0x4f0f1cf, 4 },
    { 0xe538f1d0, 5 },
    { 0, 0 },
    { 0xfae4f1d2, 3 },
    { 0x9d10f1d3, 3 },
    { 0x4520f1d3, 3 },
    { 0x5b74f1d4, 4 },
    { 0xa2c4f1d3, 3 },
    { 0, 0 },
    { 0xb84cf1d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b4f1db, 4 },
    { 0x2f20f1db, 4 },
    { 0x1f8f1dd, 4 },
    { 0x120f1de, 3 },
    { 0xaa8f1df, 3 },
    { 0x4294f1de, 3 },
    { 0x2384f1e1, 4 },
    { 0x2714f1dd, 3 },
    { 0xa220f1e1, 3 },
    { 0xcdb8f1e4, 4 },
    { 0xaa8cf1e4, 4 },
    { 0xc644f1e6, 3 },
    { 0x7b20f1e1, 4 },
    { 0xa0c8f1e6, 3 },
    { 0x3c7cf1e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b50f1ee, 5 },
    { 0x280f1ef, 4 },
    { 0, 0 },
    { 0x98f1f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c50f1f4, 3 },
    { 0x4a10f1f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9cf1f8, 4 },
    { 0, 0 },
    { 0x7c48f1fa, 3 },
    { 0xc188f1fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed4f1ff, 4 },
    { 0x9f40f1ff, 3 },
    { 0x6aa4f201, 3 },
    { 0xd654f202, 4 },
    { 0x8160f203, 3 },
    { 0x570cf203, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08f207, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf4f20f, 4 },
    { 0xe0fcf20f, 3 },
    { 0x49e4f210, 3 },
    { 0, 0 },
    { 0xb428f213, 3 },
    { 0, 0 },
    { 0xbcf215, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce8f219, 3 },
    { 0x14a4f21a, 4 },
    { 0x904cf21b, 3 },
    { 0x45e0f21c, 3 },
    { 0x48b4f219, 3 },
    { 0x4f14f21e, 4 },
    { 0xae84f21b, 4 },
    { 0x878cf220, 4 },
    { 0x9c38f221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf490f224, 4 },
    { 0xf1c8f225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e18f22c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xece0f22f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc0f232, 3 },
    { 0, 0 },
    { 0xb62cf234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1cf238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc538f23c, 3 },
    { 0xe014f23d, 3 },
    { 0, 0 },
    { 0x9070f23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c0f249, 3 },
    { 0, 0 },
    { 0x554f24b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8554f24e, 3 },
    { 0x91acf24f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8668f253, 4 },
    { 0xd0e8f253, 3 },
    { 0xe8e8f254, 3 },
    { 0x2774f256, 4 },
    { 0, 0 },
    { 0x8e04f258, 3 },
    { 0x26fcf258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e88f25e, 3 },
    { 0, 0 },
    { 0xa944f260, 3 },
    { 0xbdf4f261, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd4f264, 4 },
    { 0x4614f264, 3 },
    { 0xff24f266, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a0f26e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a68f273, 3 },
    { 0x9a8f273, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5cf277, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1148f27a, 3 },
    { 0, 0 },
    { 0xb0d4f27c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf768f282, 3 },
    { 0, 0 },
    { 0x8ab4f284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8f287, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5560f28c, 4 },
    { 0x3780f28c, 4 },
    { 0x1f40f28e, 3 },
    { 0, 0 },
    { 0x70ccf290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9078f29a, 4 },
    { 0x5140f29b, 3 },
    { 0xb2d8f29c, 3 },
    { 0x8c0f29b, 3 },
    { 0x412cf29e, 3 },
    { 0x7e6cf29f, 4 },
    { 0xe308f2a0, 3 },
    { 0xc6acf29e, 3 },
    { 0xa1a4f29f, 3 },
    { 0x91d0f29e, 3 },
    { 0x7d14f2a4, 3 },
    { 0xa054f29e, 3 },
    { 0x926cf2a6, 4 },
    { 0x76bcf2a7, 3 },
    { 0xd22cf2a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf908f2af, 3 },
    { 0x1a04f2b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18f2b3, 4 },
    { 0, 0 },
    { 0x4f70f2b5, 4 },
    { 0x5c48f2b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb78f2bb, 4 },
    { 0xaf60f2bb, 3 },
    { 0, 0 },
    { 0xa6bcf2be, 3 },
    { 0x2b3cf2be, 3 },
    { 0xa224f2c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b8f2c8, 4 },
    { 0, 0 },
    { 0x22acf2ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb4f2d3, 3 },
    { 0x7618f2d3, 3 },
    { 0x3b30f2d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed80f2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3404f2e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6cf2e6, 4 },
    { 0x4adcf2e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe194f2eb, 3 },
    { 0xafb4f2ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f8f2f1, 3 },
    { 0x5e8cf2f1, 3 },
    { 0xc004f2f3, 4 },
    { 0x3b00f2f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfccf2f7, 3 },
    { 0x6504f2f8, 3 },
    { 0xb778f2f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fcf2fc, 3 },
    { 0x6ea4f2fc, 3 },
    { 0x1c44f2fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8f301, 4 },
    { 0, 0 },
    { 0xf7ecf303, 3 },
    { 0, 0 },
    { 0xdbd4f305, 3 },
    { 0, 0 },
    { 0xc790f307, 3 },
    { 0x60f4f308, 3 },
    { 0xacc0f308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0f30c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81cf314, 3 },
    { 0x6504f315, 5 },
    { 0xaa70f314, 3 },
    { 0, 0 },
    { 0x85a8f318, 4 },
    { 0, 0 },
    { 0xfba8f31a, 5 },
    { 0x760cf31b, 3 },
    { 0xda98f31c, 3 },
    { 0x42fcf31d, 3 },
    { 0x23a0f31d, 3 },
    { 0xf244f31a, 4 },
    { 0, 0 },
    { 0x7ba0f321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd668f326, 3 },
    { 0xd18cf327, 3 },
    { 0xe988f326, 3 },
    { 0x8488f326, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3704f32c, 3 },
    { 0xafcf32d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba08f334, 3 },
    { 0x75f0f335, 3 },
    { 0x4964f335, 3 },
    { 0xb78cf335, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb40f33a, 3 },
    { 0x6454f33a, 3 },
    { 0xb5b8f33c, 3 },
    { 0xa21cf33a, 3 },
    { 0x9c48f33e, 3 },
    { 0x60bcf33e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5410f345, 3 },
    { 0xaeb4f346, 4 },
    { 0xb9f8f347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb678f34f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214f354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fcf359, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a90f361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e00f365, 3 },
    { 0x860f366, 3 },
    { 0, 0 },
    { 0x5e18f368, 4 },
    { 0, 0 },
    { 0x14c8f36a, 3 },
    { 0, 0 },
    { 0x518f36c, 3 },
    { 0x4f5cf36d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa4f375, 3 },
    { 0, 0 },
    { 0x68ccf377, 3 },
    { 0x9f5cf378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387cf37c, 3 },
    { 0x4cb8f37c, 3 },
    { 0x2a68f37e, 4 },
    { 0xb8e8f37f, 3 },
    { 0, 0 },
    { 0x1a2cf381, 3 },
    { 0xfd50f382, 3 },
    { 0xb424f383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8364f386, 3 },
    { 0xc750f387, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4220f38a, 3 },
    { 0x3b20f38b, 4 },
    { 0xb584f38c, 3 },
    { 0xf728f38a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4f390, 4 },
    { 0xa920f391, 3 },
    { 0xd8c0f391, 4 },
    { 0x646cf393, 4 },
    { 0xedb8f394, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9acf39a, 4 },
    { 0x1eb0f39b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6988f3a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc4f3aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f08f3af, 3 },
    { 0x5744f3af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb0f3b5, 3 },
    { 0x7fd0f3b6, 3 },
    { 0xdb80f3b6, 3 },
    { 0xc878f3b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7cf3bd, 5 },
    { 0x38c0f3bd, 4 },
    { 0, 0 },
    { 0x9bb0f3c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ccf3cd, 3 },
    { 0x67d4f3ce, 3 },
    { 0x1558f3cd, 3 },
    { 0xbe04f3d0, 3 },
    { 0xd2e0f3cd, 3 },
    { 0xe908f3d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c0f3d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2cf3da, 3 },
    { 0x8398f3db, 4 },
    { 0x188cf3da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa500f3df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6facf3e3, 3 },
    { 0xf104f3e4, 4 },
    { 0xc3ccf3e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4f3e8, 3 },
    { 0x8568f3e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7cf3ee, 4 },
    { 0x3ef0f3ee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f30f3f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd50f3f7, 3 },
    { 0x59d8f3f8, 3 },
    { 0xe4c4f3f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bcf3fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf610f400, 3 },
    { 0x3750f401, 3 },
    { 0x8510f401, 3 },
    { 0x1178f403, 3 },
    { 0, 0 },
    { 0x92f4f405, 4 },
    { 0, 0 },
    { 0x8728f407, 3 },
    { 0x62e0f408, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3cf40c, 4 },
    { 0xe18cf40d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fecf410, 3 },
    { 0x9c8f410, 3 },
    { 0x9f2cf412, 5 },
    { 0xb768f413, 3 },
    { 0, 0 },
    { 0x8250f415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb88f418, 4 },
    { 0xc8f418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b4f41d, 3 },
    { 0x9764f41d, 3 },
    { 0xa74f41e, 3 },
    { 0, 0 },
    { 0x19ccf421, 3 },
    { 0, 0 },
    { 0x7798f423, 3 },
    { 0x16e8f424, 3 },
    { 0x278f425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f64f429, 3 },
    { 0x4ed8f429, 4 },
    { 0x13f8f42b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b4f42e, 4 },
    { 0, 0 },
    { 0x77d8f430, 3 },
    { 0, 0 },
    { 0x4b88f432, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8f436, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3198f43a, 4 },
    { 0xf018f43a, 3 },
    { 0xa510f43b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x620f43f, 4 },
    { 0x1b60f440, 3 },
    { 0x516cf441, 3 },
    { 0, 0 },
    { 0x48cf443, 3 },
    { 0, 0 },
    { 0x76d8f445, 3 },
    { 0x1e2cf446, 4 },
    { 0xfcf4f447, 3 },
    { 0x7268f446, 3 },
    { 0x2a88f448, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f8f450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3200f455, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e0f459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1170f45d, 3 },
    { 0, 0 },
    { 0x6574f45f, 3 },
    { 0xed14f460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d20f464, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cf469, 4 },
    { 0xa804f469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae4f470, 4 },
    { 0xc440f470, 3 },
    { 0x8408f472, 3 },
    { 0xacf8f472, 3 },
    { 0xab90f474, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cf478, 3 },
    { 0x13f0f479, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753cf47e, 3 },
    { 0xbbf8f47f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4f486, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf8f489, 3 },
    { 0xb368f48a, 3 },
    { 0x69c8f48b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6cf491, 3 },
    { 0, 0 },
    { 0x7e20f493, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c78f496, 3 },
    { 0xb028f496, 5 },
    { 0, 0 },
    { 0x1f04f499, 3 },
    { 0xcb3cf49a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b38f49d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6210f4a1, 3 },
    { 0xf94cf4a1, 3 },
    { 0xcc00f4a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c8f4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4f4ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20f4b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f4b6, 4 },
    { 0, 0 },
    { 0x3508f4b8, 3 },
    { 0xf64cf4b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd784f4bf, 3 },
    { 0x2ae0f4c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4640f4c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dcf4c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c10f4ca, 3 },
    { 0x4c7cf4ca, 3 },
    { 0x40f8f4ca, 3 },
    { 0x7b48f4cd, 3 },
    { 0x3620f4cd, 3 },
    { 0x8fdcf4cf, 3 },
    { 0x7374f4cf, 3 },
    { 0xd7c4f4ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e90f4dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418f4df, 3 },
    { 0, 0 },
    { 0x664cf4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730f4e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734f4f2, 3 },
    { 0x9cc0f4f3, 4 },
    { 0xf7c8f4f4, 4 },
    { 0x1274f4f5, 3 },
    { 0x1b9cf4f6, 5 },
    { 0x953cf4f6, 3 },
    { 0x8f34f4f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4824f4fc, 3 },
    { 0xe098f4fc, 4 },
    { 0, 0 },
    { 0xff58f4ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc820f502, 5 },
    { 0x2968f502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b4f506, 4 },
    { 0x8960f507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5cf50b, 3 },
    { 0x4680f50b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d4f513, 3 },
    { 0x37dcf513, 3 },
    { 0x7d9cf514, 3 },
    { 0xb328f513, 3 },
    { 0, 0 },
    { 0x43e4f518, 3 },
    { 0xdad0f519, 3 },
    { 0xf0acf519, 5 },
    { 0xc7c8f51b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424f524, 3 },
    { 0x6d7cf524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d60f528, 3 },
    { 0x6810f529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cf52c, 3 },
    { 0x10f8f52d, 4 },
    { 0xc760f52d, 3 },
    { 0xad20f52d, 4 },
    { 0, 0 },
    { 0x4a94f531, 3 },
    { 0, 0 },
    { 0xf0a8f533, 4 },
    { 0xf998f533, 3 },
    { 0, 0 },
    { 0xbae0f536, 3 },
    { 0x36a0f537, 3 },
    { 0x297cf538, 3 },
    { 0x2704f539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db8f540, 3 },
    { 0, 0 },
    { 0xe3ecf542, 3 },
    { 0x4990f543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d0f549, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8cf557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215cf55b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f28f55f, 3 },
    { 0xa560f55f, 4 },
    { 0x5df8f561, 4 },
    { 0x14c4f562, 4 },
    { 0, 0 },
    { 0x4048f564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe8f56b, 3 },
    { 0x3c94f56c, 3 },
    { 0x4b84f56d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8f575, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd550f578, 3 },
    { 0, 0 },
    { 0x989cf57a, 3 },
    { 0, 0 },
    { 0x3010f57c, 3 },
    { 0, 0 },
    { 0x9f00f57e, 3 },
    { 0, 0 },
    { 0x3284f580, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d20f585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd490f589, 3 },
    { 0x71a4f58a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dcf58d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c8f591, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1518f596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813cf59a, 3 },
    { 0xe4cf59a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee0f59f, 3 },
    { 0xebd0f5a0, 3 },
    { 0x1b94f5a0, 3 },
    { 0x9264f5a2, 5 },
    { 0xa064f5a3, 3 },
    { 0x2798f5a3, 4 },
    { 0xc7acf5a4, 4 },
    { 0, 0 },
    { 0x98bcf5a7, 3 },
    { 0x74c4f5a8, 3 },
    { 0, 0 },
    { 0x983cf5aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae04f5ae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7608f5b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d0f5b7, 3 },
    { 0x3d38f5b8, 3 },
    { 0x6614f5b9, 3 },
    { 0x9e6cf5b9, 3 },
    { 0, 0 },
    { 0x6da8f5bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf268f5c2, 3 },
    { 0x6034f5c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7848f5d1, 3 },
    { 0xb718f5d1, 4 },
    { 0, 0 },
    { 0xc748f5d4, 3 },
    { 0x79acf5d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf5dd, 3 },
    { 0, 0 },
    { 0x48bcf5df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc24f5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3288f5e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f8f5ed, 4 },
    { 0xbc1cf5ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1588f5f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9cf5fb, 3 },
    { 0xdc6cf5fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1064f603, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b4f607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9550f614, 3 },
    { 0xbcb8f615, 3 },
    { 0x7e14f614, 3 },
    { 0xf77cf617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90f61b, 3 },
    { 0, 0 },
    { 0x798cf61d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ecf623, 3 },
    { 0xfc00f624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b8f628, 4 },
    { 0x3e34f629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05cf62d, 3 },
    { 0, 0 },
    { 0x8044f62f, 4 },
    { 0xf94cf630, 4 },
    { 0, 0 },
    { 0xf420f632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c0f636, 3 },
    { 0x5154f636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa748f63c, 3 },
    { 0xdcecf63d, 3 },
    { 0xf94f63d, 3 },
    { 0x9a20f63f, 3 },
    { 0x99c4f63e, 3 },
    { 0x3424f640, 3 },
    { 0x3d68f642, 3 },
    { 0xd344f63c, 3 },
    { 0, 0 },
    { 0x9184f645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ccf648, 3 },
    { 0x45a8f649, 3 },
    { 0, 0 },
    { 0x1994f64b, 3 },
    { 0, 0 },
    { 0xcfa0f64d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130f653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f8f659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b0f65c, 3 },
    { 0xea84f65d, 3 },
    { 0, 0 },
    { 0xcc68f65f, 3 },
    { 0, 0 },
    { 0xd710f661, 4 },
    { 0, 0 },
    { 0x2f78f663, 3 },
    { 0, 0 },
    { 0xa8f665, 3 },
    { 0x5248f665, 4 },
    { 0xd1ccf667, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b28f66a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc150f671, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc764f676, 3 },
    { 0, 0 },
    { 0x4d6cf678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4cf67c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2cf684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10f687, 3 },
    { 0xc24f687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54cf68d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe574f693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1314f697, 3 },
    { 0x35b8f698, 4 },
    { 0xc02cf699, 3 },
    { 0x3b18f698, 3 },
    { 0xc418f69b, 3 },
    { 0xa1d0f69b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4368f6a4, 3 },
    { 0, 0 },
    { 0xf614f6a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f14f6aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4998f6ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8690f6b1, 3 },
    { 0, 0 },
    { 0x60dcf6b3, 3 },
    { 0, 0 },
    { 0xdb3cf6b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354f6ba, 3 },
    { 0x7984f6ba, 3 },
    { 0x61e4f6bc, 3 },
    { 0x2f0f6bd, 4 },
    { 0xf188f6ba, 3 },
    { 0xdf30f6bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99cf6c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a70f6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x489cf6ca, 3 },
    { 0xe918f6cb, 3 },
    { 0x74fcf6cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f6dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e4f6e4, 3 },
    { 0, 0 },
    { 0x905cf6e6, 3 },
    { 0xd9e0f6e7, 3 },
    { 0x93c8f6e8, 3 },
    { 0xb260f6e9, 4 },
    { 0x4360f6ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409cf6ee, 3 },
    { 0, 0 },
    { 0x9898f6f0, 4 },
    { 0xb098f6f1, 3 },
    { 0x4acf6f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9788f6f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d8f6f8, 3 },
    { 0, 0 },
    { 0x9fbcf6fa, 4 },
    { 0xec0f6fb, 3 },
    { 0xd768f6fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf64f70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4250f70e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3cf713, 4 },
    { 0x9c8f713, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e4f717, 3 },
    { 0x4910f718, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadcf71b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6234f723, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332cf72b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38cf730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb0f735, 3 },
    { 0x3f90f735, 3 },
    { 0xb274f737, 5 },
    { 0, 0 },
    { 0xb0b4f739, 3 },
    { 0, 0 },
    { 0x575cf73b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df0f743, 4 },
    { 0x4708f744, 3 },
    { 0xc894f744, 4 },
    { 0xd050f746, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d80f749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588f74e, 3 },
    { 0, 0 },
    { 0xb1a4f750, 4 },
    { 0xd944f751, 4 },
    { 0x5340f752, 3 },
    { 0x603cf753, 3 },
    { 0x1110f752, 3 },
    { 0xc894f754, 3 },
    { 0x18ecf756, 5 },
    { 0xc278f752, 3 },
    { 0x16bcf758, 3 },
    { 0x9ee4f759, 5 },
    { 0x360f75a, 3 },
    { 0xbb6cf75b, 3 },
    { 0x9390f75b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf24f75f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e48f762, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0f766, 3 },
    { 0, 0 },
    { 0xed1cf768, 3 },
    { 0x3a88f769, 4 },
    { 0xfa40f76a, 3 },
    { 0xbdc8f76a, 3 },
    { 0, 0 },
    { 0xd63cf76d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac0f770, 3 },
    { 0xaf68f771, 3 },
    { 0, 0 },
    { 0x8f00f773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd48f77e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd964f784, 3 },
    { 0, 0 },
    { 0x3c10f786, 3 },
    { 0x3b40f787, 3 },
    { 0xacb8f788, 3 },
    { 0, 0 },
    { 0x3070f78a, 3 },
    { 0, 0 },
    { 0xf164f78c, 4 },
    { 0xe330f78c, 3 },
    { 0xce20f78c, 3 },
    { 0, 0 },
    { 0x68acf790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac0f797, 5 },
    { 0x7578f798, 3 },
    { 0, 0 },
    { 0x9e38f79a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88f7a4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2748f7aa, 3 },
    { 0, 0 },
    { 0x9c90f7ac, 3 },
    { 0x8df0f7ad, 3 },
    { 0x3af0f7ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fcf7b3, 3 },
    { 0xa83cf7b4, 4 },
    { 0, 0 },
    { 0xcb68f7b6, 3 },
    { 0xe9acf7b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf804f7ba, 3 },
    { 0x6d70f7bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18f7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd914f7c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b30f7c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5770f7c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f8f7cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f0f7d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5664f7d3, 4 },
    { 0, 0 },
    { 0x97b8f7d5, 3 },
    { 0x1fe0f7d5, 4 },
    { 0x73b4f7d5, 3 },
    { 0x5f50f7d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3790f7df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd530f7e2, 3 },
    { 0, 0 },
    { 0x22acf7e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164cf7e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4808f7ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f8f7f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9550f7f6, 4 },
    { 0x7900f7f7, 4 },
    { 0x1888f7f6, 3 },
    { 0x8618f7f7, 3 },
    { 0x4b84f7fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8664f802, 3 },
    { 0, 0 },
    { 0x63f0f804, 3 },
    { 0x8e34f805, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70f809, 3 },
    { 0x5474f80a, 3 },
    { 0xac1cf80a, 3 },
    { 0xfdbcf809, 4 },
    { 0xfa24f80d, 4 },
    { 0x6cf80c, 4 },
    { 0, 0 },
    { 0x42c0f810, 3 },
    { 0, 0 },
    { 0xb208f812, 4 },
    { 0x92c8f813, 3 },
    { 0x50b4f813, 4 },
    { 0xb78f813, 3 },
    { 0xd658f813, 3 },
    { 0x8654f817, 3 },
    { 0x3668f818, 4 },
    { 0xc964f819, 3 },
    { 0, 0 },
    { 0xb448f81b, 4 },
    { 0x7b90f81c, 4 },
    { 0xb9acf81d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6598f824, 3 },
    { 0, 0 },
    { 0x8860f826, 3 },
    { 0x73fcf827, 5 },
    { 0, 0 },
    { 0x2fb8f829, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98f831, 3 },
    { 0, 0 },
    { 0x2790f833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10f838, 4 },
    { 0x7e5cf839, 3 },
    { 0xa688f838, 3 },
    { 0xa640f83b, 3 },
    { 0x6924f83b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880cf840, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe684f844, 3 },
    { 0x3eecf845, 3 },
    { 0xc374f845, 3 },
    { 0, 0 },
    { 0x9064f848, 4 },
    { 0x4e48f848, 4 },
    { 0xb630f84a, 4 },
    { 0, 0 },
    { 0x2738f84c, 4 },
    { 0, 0 },
    { 0xf464f84e, 3 },
    { 0x935cf84e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3990f853, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a0f85b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afcf860, 3 },
    { 0x1b78f861, 3 },
    { 0, 0 },
    { 0x9d64f863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28f86a, 3 },
    { 0x8894f86b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf86f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x404cf872, 3 },
    { 0x2b64f872, 3 },
    { 0x31ccf874, 3 },
    { 0xb158f875, 4 },
    { 0x7f8cf874, 3 },
    { 0x21a0f876, 3 },
    { 0xec08f877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f880, 3 },
    { 0, 0 },
    { 0x6a9cf882, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84f887, 3 },
    { 0x4310f888, 3 },
    { 0xd304f889, 3 },
    { 0xd2f4f888, 5 },
    { 0x7a80f88b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ccf894, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1430f897, 4 },
    { 0x5d6cf898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987cf89f, 3 },
    { 0, 0 },
    { 0x7858f8a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd460f8a6, 3 },
    { 0, 0 },
    { 0x3678f8a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc754f8ab, 3 },
    { 0, 0 },
    { 0xa110f8ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd4f8b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd164f8bc, 5 },
    { 0x1244f8bc, 5 },
    { 0x5decf8be, 4 },
    { 0x53f0f8bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7560f8c6, 3 },
    { 0xe8e8f8c7, 3 },
    { 0x3d3cf8c7, 3 },
    { 0x9938f8c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7340f8d1, 3 },
    { 0xdcecf8d2, 4 },
    { 0, 0 },
    { 0xb6bcf8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acf8d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5538f8db, 3 },
    { 0x4e04f8dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90f8df, 3 },
    { 0xda94f8e0, 3 },
    { 0x9c68f8e1, 3 },
    { 0x7944f8e2, 3 },
    { 0x428cf8e3, 3 },
    { 0x3990f8e4, 3 },
    { 0x774f8e5, 3 },
    { 0xa338f8e1, 3 },
    { 0x2164f8e2, 3 },
    { 0x9c74f8e4, 3 },
    { 0x9894f8e9, 3 },
    { 0xe914f8ea, 3 },
    { 0x9a60f8eb, 3 },
    { 0x3b0f8e9, 3 },
    { 0, 0 },
    { 0x50dcf8ee, 4 },
    { 0, 0 },
    { 0x9c9cf8f0, 3 },
    { 0x8ef4f8f1, 3 },
    { 0x250cf8f0, 3 },
    { 0, 0 },
    { 0x670f8f4, 3 },
    { 0, 0 },
    { 0xf9f8f8f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920f8fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf4f903, 4 },
    { 0x1de4f904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddcf90c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf84f910, 3 },
    { 0, 0 },
    { 0x208f912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2020f918, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d8f91e, 4 },
    { 0x26a4f91f, 3 },
    { 0x8608f91f, 3 },
    { 0xce28f921, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6870f929, 3 },
    { 0x336cf92a, 3 },
    { 0x2698f929, 3 },
    { 0, 0 },
    { 0x5fd0f92d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedcf939, 3 },
    { 0xc708f93a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5158f93f, 3 },
    { 0x510f93f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45acf943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f98f94c, 4 },
    { 0x619cf94d, 3 },
    { 0x5e98f94d, 4 },
    { 0x21d0f94f, 4 },
    { 0x2c6cf94e, 3 },
    { 0x8010f94e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c68f956, 3 },
    { 0x35dcf957, 4 },
    { 0x7100f958, 3 },
    { 0x87cf959, 3 },
    { 0, 0 },
    { 0x85f8f95b, 5 },
    { 0, 0 },
    { 0xd388f95d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc160f961, 3 },
    { 0xad8cf961, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108cf966, 3 },
    { 0, 0 },
    { 0xb9a0f968, 3 },
    { 0x73ccf969, 3 },
    { 0x4c50f968, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c0f973, 3 },
    { 0xcf8f974, 3 },
    { 0x83ccf974, 4 },
    { 0xd840f976, 4 },
    { 0, 0 },
    { 0x8290f978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6900f97b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862cf987, 3 },
    { 0, 0 },
    { 0x774cf989, 3 },
    { 0, 0 },
    { 0x73f8f98b, 3 },
    { 0xdad4f98c, 3 },
    { 0xf0e8f98b, 3 },
    { 0xfcacf98e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f992, 3 },
    { 0x640cf993, 3 },
    { 0x5df0f993, 4 },
    { 0x815cf994, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7004f999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164f99f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77cf9a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddcf9ae, 4 },
    { 0x93c4f9af, 4 },
    { 0, 0 },
    { 0x37ecf9b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef60f9b4, 4 },
    { 0, 0 },
    { 0x37d0f9b6, 3 },
    { 0xc370f9b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ecf9ba, 4 },
    { 0xf298f9bb, 3 },
    { 0x474cf9bc, 3 },
    { 0xff54f9ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d4f9c0, 3 },
    { 0x99e4f9c1, 3 },
    { 0x5208f9c1, 3 },
    { 0, 0 },
    { 0x32c8f9c4, 4 },
    { 0x1e58f9c5, 4 },
    { 0xcce8f9c6, 3 },
    { 0xf68cf9c7, 3 },
    { 0, 0 },
    { 0xc65cf9c9, 3 },
    { 0xbe38f9ca, 3 },
    { 0, 0 },
    { 0x8e24f9cc, 3 },
    { 0x524f9cd, 3 },
    { 0x6688f9ce, 5 },
    { 0x62e8f9cf, 3 },
    { 0x264f9d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c8f9d3, 3 },
    { 0xd4b4f9d3, 3 },
    { 0x5da0f9d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20f9db, 3 },
    { 0xccbcf9dc, 4 },
    { 0x6d28f9dd, 3 },
    { 0, 0 },
    { 0x13a0f9df, 4 },
    { 0, 0 },
    { 0x93d0f9e1, 3 },
    { 0x60d4f9e1, 3 },
    { 0, 0 },
    { 0xa354f9e4, 4 },
    { 0xb580f9e5, 4 },
    { 0, 0 },
    { 0x5ff8f9e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c58f9ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9cf9f1, 3 },
    { 0x5a04f9f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbccf9f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fcfa02, 3 },
    { 0x33acfa03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cfa08, 4 },
    { 0x5d98fa09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc8fa0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae8fa11, 3 },
    { 0x5be8fa12, 3 },
    { 0x592cfa13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b4fa16, 3 },
    { 0, 0 },
    { 0x519cfa18, 3 },
    { 0xf56cfa19, 3 },
    { 0x598cfa18, 3 },
    { 0x197cfa1b, 3 },
    { 0, 0 },
    { 0xbb34fa1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd44fa21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c4fa25, 3 },
    { 0xe0d0fa26, 3 },
    { 0x29fcfa27, 4 },
    { 0x4b08fa25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9cfa30, 3 },
    { 0x8e80fa31, 4 },
    { 0x74d4fa32, 4 },
    { 0xfb74fa32, 3 },
    { 0xf9ccfa33, 3 },
    { 0x5a4cfa30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874fa39, 3 },
    { 0x89c4fa39, 4 },
    { 0x28c4fa3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30fa3f, 3 },
    { 0xdebcfa40, 3 },
    { 0x3f94fa3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4298fa4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a8fa4f, 5 },
    { 0xca1cfa50, 4 },
    { 0x8448fa51, 3 },
    { 0x520cfa52, 3 },
    { 0x8500fa4f, 4 },
    { 0xfe8cfa54, 4 },
    { 0x9078fa55, 3 },
    { 0xbb38fa56, 3 },
    { 0xc510fa57, 3 },
    { 0, 0 },
    { 0x6e94fa59, 4 },
    { 0xe5acfa5a, 3 },
    { 0xc4ccfa5b, 3 },
    { 0, 0 },
    { 0xf4c8fa5d, 4 },
    { 0x21d4fa5e, 3 },
    { 0xad4fa5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8754fa62, 3 },
    { 0x4774fa63, 3 },
    { 0, 0 },
    { 0xd644fa65, 3 },
    { 0xbb34fa66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa744fa69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9cfa6c, 3 },
    { 0x9b24fa6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f0fa72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db8fa78, 4 },
    { 0x70fcfa79, 4 },
    { 0xa87cfa7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4258fa7d, 3 },
    { 0, 0 },
    { 0xa60fa7f, 3 },
    { 0, 0 },
    { 0xfee8fa81, 3 },
    { 0xf144fa82, 3 },
    { 0, 0 },
    { 0xff1cfa84, 4 },
    { 0xf3d8fa84, 3 },
    { 0xbf1cfa84, 3 },
    { 0x73f4fa85, 3 },
    { 0, 0 },
    { 0xfe3cfa89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf258fa8c, 3 },
    { 0, 0 },
    { 0x2e54fa8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3cfa95, 4 },
    { 0xb728fa96, 3 },
    { 0x9f58fa97, 4 },
    { 0, 0 },
    { 0xad90fa99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2cfa9c, 3 },
    { 0xeea4fa9c, 3 },
    { 0x56dcfa9d, 3 },
    { 0xbd74fa9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80faa3, 3 },
    { 0x4280faa4, 3 },
    { 0x3af4faa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58faa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f0faae, 4 },
    { 0x8ac0faaf, 3 },
    { 0x3178fab0, 3 },
    { 0x907cfab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313cfaba, 3 },
    { 0, 0 },
    { 0x88bcfabc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f68fac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa34fac6, 4 },
    { 0x4e68fac7, 3 },
    { 0xb9e8fac8, 3 },
    { 0xb070fac6, 3 },
    { 0x1cc0faca, 4 },
    { 0x40acfac7, 4 },
    { 0xdaa4facc, 3 },
    { 0x4accfacd, 4 },
    { 0, 0 },
    { 0x2f20facf, 3 },
    { 0x1770facf, 4 },
    { 0x12f4fad1, 5 },
    { 0xbdecfad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8fad5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4fad9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7cfade, 3 },
    { 0xb9ecfadf, 3 },
    { 0, 0 },
    { 0x86b8fae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10fae5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcfae8, 3 },
    { 0x2ce8fae8, 3 },
    { 0, 0 },
    { 0xbc54faeb, 3 },
    { 0, 0 },
    { 0x334cfaed, 4 },
    { 0xdf44faee, 3 },
    { 0xa7fcfaee, 3 },
    { 0x7e0cfaf0, 3 },
    { 0, 0 },
    { 0x254cfaf2, 4 },
    { 0x14a0faf2, 4 },
    { 0, 0 },
    { 0x8900faf5, 3 },
    { 0x7904faf6, 3 },
    { 0x2668faf6, 3 },
    { 0, 0 },
    { 0x2290faf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x936cfafc, 3 },
    { 0x2f10fafd, 3 },
    { 0x62c0fafe, 4 },
    { 0, 0 },
    { 0x42fcfb00, 3 },
    { 0x96a4fb01, 3 },
    { 0x8c4cfb02, 4 },
    { 0, 0 },
    { 0x3b5cfb04, 3 },
    { 0xb4f0fb05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dcfb0c, 4 },
    { 0, 0 },
    { 0x88fcfb0e, 3 },
    { 0, 0 },
    { 0xd90cfb10, 3 },
    { 0x4c4fb11, 3 },
    { 0x2438fb12, 3 },
    { 0x19acfb13, 3 },
    { 0x99d0fb14, 3 },
    { 0x694cfb15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f80fb18, 3 },
    { 0, 0 },
    { 0xc188fb1a, 3 },
    { 0x91e4fb1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb00fb1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a60fb25, 3 },
    { 0x4c38fb25, 3 },
    { 0xe444fb26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a8fb2d, 3 },
    { 0x670fb2e, 4 },
    { 0x2700fb2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bacfb34, 3 },
    { 0xcde0fb34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x518cfb38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe134fb3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8fb3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x734cfb41, 3 },
    { 0, 0 },
    { 0xece0fb43, 5 },
    { 0x7310fb44, 4 },
    { 0x452cfb45, 3 },
    { 0xf6cfb46, 3 },
    { 0xffd0fb47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f40fb51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534fb5b, 3 },
    { 0xc4bcfb5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c4fb70, 3 },
    { 0xe400fb71, 3 },
    { 0x201cfb71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f24fb7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9870fb82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3684fb87, 4 },
    { 0xca0fb88, 3 },
    { 0, 0 },
    { 0xe1e0fb8a, 4 },
    { 0xa930fb8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db8fb95, 4 },
    { 0, 0 },
    { 0x9800fb97, 3 },
    { 0x8700fb97, 3 },
    { 0, 0 },
    { 0x78a0fb9a, 3 },
    { 0xeefcfb9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8970fba2, 4 },
    { 0x951cfba3, 3 },
    { 0, 0 },
    { 0xc438fba5, 4 },
    { 0xfe94fba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcfbaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6cfbae, 4 },
    { 0x5ee4fbae, 3 },
    { 0, 0 },
    { 0xd3ccfbb1, 4 },
    { 0, 0 },
    { 0x4e3cfbb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62cfbbd, 3 },
    { 0x23e0fbbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8fbc4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe998fbc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd8fbcb, 4 },
    { 0xf2a8fbcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5cfbd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fcfbd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d0fbdc, 3 },
    { 0xc10fbdc, 3 },
    { 0x6818fbde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8cfbe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b0fbe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bcfbe8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a8fbeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bcfbf0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x440cfbf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7458fbf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd918fbfc, 3 },
    { 0x92fcfbfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400fc01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fcfc08, 3 },
    { 0, 0 },
    { 0x837cfc0a, 3 },
    { 0, 0 },
    { 0x28fc0c, 5 },
    { 0, 0 },
    { 0x60b4fc0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1cfc11, 3 },
    { 0xf398fc12, 4 },
    { 0xa804fc13, 4 },
    { 0x6f1cfc14, 3 },
    { 0x9b10fc11, 3 },
    { 0x476cfc16, 3 },
    { 0x9860fc16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6cfc1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2470fc1e, 3 },
    { 0x9cd4fc1e, 3 },
    { 0, 0 },
    { 0xaafcfc21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d8fc27, 3 },
    { 0xcc70fc28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018fc2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935cfc30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb4fc34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b54fc37, 3 },
    { 0, 0 },
    { 0xe254fc39, 3 },
    { 0, 0 },
    { 0x97cfc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6cfc3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa624fc43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0fc47, 4 },
    { 0x95f0fc48, 4 },
    { 0x2014fc48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a8fc51, 3 },
    { 0, 0 },
    { 0x7640fc53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9694fc56, 3 },
    { 0x2328fc57, 3 },
    { 0, 0 },
    { 0xebcfc59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d4fc5d, 3 },
    { 0x5804fc5d, 3 },
    { 0x4b98fc5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc648fc66, 3 },
    { 0xa368fc67, 3 },
    { 0xc4fcfc68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x409cfc6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60fc6e, 3 },
    { 0, 0 },
    { 0xcb14fc70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db4fc76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fc7a, 3 },
    { 0x7b20fc7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0fc7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa110fc83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94cfc86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bcfc89, 3 },
    { 0xda74fc89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc80fc8d, 4 },
    { 0x61d0fc8e, 4 },
    { 0x5448fc8e, 3 },
    { 0x90d0fc90, 4 },
    { 0xedb0fc8d, 3 },
    { 0xfac8fc92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d4fc98, 3 },
    { 0x58f0fc99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8290fca3, 3 },
    { 0xf28fca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1944fca7, 3 },
    { 0x7abcfca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e90fcab, 5 },
    { 0xbd78fcac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b4fcb1, 3 },
    { 0xa924fcb2, 3 },
    { 0x3c70fcb3, 3 },
    { 0xd3e4fcb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a4fcb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f98fcba, 4 },
    { 0xe778fcba, 3 },
    { 0x27fcfcbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe010fcc2, 3 },
    { 0, 0 },
    { 0xd660fcc4, 3 },
    { 0xddd0fcc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50acfcc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8908fcce, 3 },
    { 0xed70fccf, 4 },
    { 0x33a0fcd0, 4 },
    { 0x5594fcd0, 3 },
    { 0x12e8fcd1, 3 },
    { 0, 0 },
    { 0x4040fcd4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8244fcdb, 3 },
    { 0xe1f0fcdb, 3 },
    { 0, 0 },
    { 0x2730fcde, 3 },
    { 0x18d8fcde, 3 },
    { 0x2fa4fcde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f0fce5, 5 },
    { 0x8e0fce6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc4fce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4fcec, 4 },
    { 0, 0 },
    { 0xafa8fcee, 3 },
    { 0xa44cfcef, 4 },
    { 0, 0 },
    { 0xffd4fcf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3cfcf4, 5 },
    { 0, 0 },
    { 0xe450fcf6, 3 },
    { 0, 0 },
    { 0xba5cfcf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcfcfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a14fd02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa548fd06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf0fd0b, 3 },
    { 0x31e0fd0c, 3 },
    { 0xf970fd0b, 3 },
    { 0xe090fd0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d8fd12, 3 },
    { 0x9f78fd13, 3 },
    { 0, 0 },
    { 0xbb3cfd15, 3 },
    { 0, 0 },
    { 0x3bb0fd17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee0fd1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2430fd1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6cfd27, 4 },
    { 0x6ec4fd28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4cfd2c, 3 },
    { 0x4738fd2c, 4 },
    { 0, 0 },
    { 0x975cfd2f, 3 },
    { 0, 0 },
    { 0x6c04fd31, 3 },
    { 0, 0 },
    { 0xb8f8fd33, 3 },
    { 0x8db4fd34, 4 },
    { 0x5970fd34, 3 },
    { 0, 0 },
    { 0xf6c0fd37, 3 },
    { 0x1198fd38, 3 },
    { 0, 0 },
    { 0x63d0fd3a, 3 },
    { 0xf348fd3b, 3 },
    { 0x83bcfd3c, 4 },
    { 0xd310fd3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3458fd42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ecfd46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687cfd4b, 3 },
    { 0x1a44fd4b, 4 },
    { 0, 0 },
    { 0xae1cfd4e, 3 },
    { 0, 0 },
    { 0x9a4fd50, 4 },
    { 0xd988fd50, 3 },
    { 0x51f4fd50, 3 },
    { 0x1cdcfd51, 3 },
    { 0xda7cfd52, 4 },
    { 0xcd18fd54, 3 },
    { 0xf2e4fd56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fcfd5a, 3 },
    { 0xbfb4fd5b, 3 },
    { 0, 0 },
    { 0x752cfd5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4fd62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40fd65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c60fd69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78fd6c, 4 },
    { 0x596cfd6d, 4 },
    { 0x5cfcfd6c, 3 },
    { 0, 0 },
    { 0xe614fd70, 3 },
    { 0xdfa4fd70, 3 },
    { 0x3eccfd70, 3 },
    { 0xb9fcfd73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d68fd79, 3 },
    { 0x5a2cfd7a, 3 },
    { 0, 0 },
    { 0xd668fd7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b40fd86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4558fd8a, 3 },
    { 0x36c4fd8b, 4 },
    { 0x8a74fd8b, 3 },
    { 0, 0 },
    { 0xee64fd8e, 4 },
    { 0, 0 },
    { 0xed48fd90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee8fd94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a8fd97, 3 },
    { 0, 0 },
    { 0x284fd99, 3 },
    { 0, 0 },
    { 0xe360fd9b, 3 },
    { 0, 0 },
    { 0x39ecfd9d, 4 },
    { 0xdf0cfd9e, 3 },
    { 0, 0 },
    { 0x9a1cfda0, 3 },
    { 0, 0 },
    { 0x3198fda2, 3 },
    { 0x5524fda3, 3 },
    { 0x7dc4fda4, 3 },
    { 0x3da8fda5, 3 },
    { 0x4884fda6, 3 },
    { 0, 0 },
    { 0x69b8fda8, 4 },
    { 0xdf44fda9, 4 },
    { 0xa9a0fda8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf74fdb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf574fdb9, 4 },
    { 0x1db8fdb9, 3 },
    { 0xc7ecfdb9, 3 },
    { 0x9a60fdb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dcfdbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcfdc2, 4 },
    { 0xbcb8fdc3, 3 },
    { 0xe154fdc4, 3 },
    { 0xc70fdc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd494fdc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fcfdd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44cfdd5, 3 },
    { 0x82bcfdd6, 4 },
    { 0x68f0fdd7, 3 },
    { 0, 0 },
    { 0xb6a0fdd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1918fddc, 3 },
    { 0xfd40fddc, 3 },
    { 0xb954fdde, 3 },
    { 0, 0 },
    { 0x6eccfde0, 3 },
    { 0xfa50fde1, 3 },
    { 0x7588fde1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2468fde5, 4 },
    { 0x1ec4fde6, 3 },
    { 0, 0 },
    { 0xa8e8fde8, 4 },
    { 0, 0 },
    { 0x21dcfdea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb550fded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed4fdf7, 3 },
    { 0x9580fdf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cfdfc, 3 },
    { 0, 0 },
    { 0x1580fdfe, 3 },
    { 0, 0 },
    { 0x7240fe00, 3 },
    { 0, 0 },
    { 0xe4d0fe02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10acfe08, 4 },
    { 0x1f4fe08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2414fe0c, 3 },
    { 0x974cfe0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a8fe14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf128fe1a, 3 },
    { 0, 0 },
    { 0x7dc8fe1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3cfe22, 3 },
    { 0, 0 },
    { 0x5e48fe24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c4fe2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x719cfe31, 3 },
    { 0, 0 },
    { 0xf554fe33, 3 },
    { 0, 0 },
    { 0xe158fe35, 3 },
    { 0, 0 },
    { 0xd384fe37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf208fe3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf968fe42, 4 },
    { 0x1e74fe42, 3 },
    { 0x7d0fe44, 3 },
    { 0x2800fe45, 3 },
    { 0x23e4fe46, 3 },
    { 0, 0 },
    { 0x87ecfe48, 3 },
    { 0x1fdcfe48, 4 },
    { 0xdd78fe4a, 4 },
    { 0x34d0fe4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4cfe4f, 3 },
    { 0x15f0fe50, 4 },
    { 0x857cfe51, 3 },
    { 0x8030fe52, 3 },
    { 0xa4fe53, 4 },
    { 0, 0 },
    { 0x65b0fe55, 3 },
    { 0x4cbcfe56, 4 },
    { 0xffb0fe55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fe5c, 4 },
    { 0x4664fe5d, 3 },
    { 0x8af0fe5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be4fe66, 4 },
    { 0x5cf8fe67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfcfe6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cacfe70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f4fe74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c8fe7f, 3 },
    { 0x8ff4fe80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8fe87, 3 },
    { 0x77e0fe88, 5 },
    { 0xd04cfe88, 3 },
    { 0xdafcfe89, 5 },
    { 0x141cfe89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90fe8e, 3 },
    { 0xa490fe8f, 3 },
    { 0x8eb0fe8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408fe9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4074fea2, 3 },
    { 0x8160fea3, 4 },
    { 0, 0 },
    { 0xe2bcfea5, 3 },
    { 0x2798fea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8fea9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe008fead, 3 },
    { 0, 0 },
    { 0x8368feaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e44feb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc0feba, 3 },
    { 0xc2e0febb, 3 },
    { 0xf6b0febc, 3 },
    { 0x95b8febd, 4 },
    { 0x3920febe, 3 },
    { 0, 0 },
    { 0xb958fec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d4fec7, 3 },
    { 0xb3d0fec8, 4 },
    { 0x703cfec8, 4 },
    { 0, 0 },
    { 0x7c78fecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820fecf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6730fed3, 3 },
    { 0, 0 },
    { 0x93a4fed5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802cfed9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb78fee4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9808fee8, 4 },
    { 0, 0 },
    { 0xf858feea, 4 },
    { 0, 0 },
    { 0xe74cfeec, 3 },
    { 0x3558feed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad8fef0, 4 },
    { 0, 0 },
    { 0x424cfef2, 3 },
    { 0, 0 },
    { 0xe2e0fef4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2274fef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac0fefb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda4fefe, 3 },
    { 0x6ef8fefe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654cff06, 3 },
    { 0, 0 },
    { 0x31f8ff08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa720ff0d, 3 },
    { 0x2e10ff0e, 3 },
    { 0x2398ff0f, 3 },
    { 0, 0 },
    { 0x7a1cff11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ff16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cff1a, 3 },
    { 0x8e8ff1b, 4 },
    { 0x37f8ff1c, 4 },
    { 0x78f4ff1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a4ff21, 3 },
    { 0, 0 },
    { 0xdb18ff23, 3 },
    { 0, 0 },
    { 0x1730ff25, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078ff2a, 3 },
    { 0x60b0ff2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa240ff2f, 4 },
    { 0xc1d0ff2f, 3 },
    { 0x9604ff2f, 3 },
    { 0x2af8ff2f, 3 },
    { 0xf558ff33, 3 },
    { 0xd7bcff33, 3 },
    { 0xdfc8ff35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e0ff3b, 3 },
    { 0xa974ff3b, 3 },
    { 0xaae8ff3d, 3 },
    { 0xff38ff3c, 3 },
    { 0xe330ff3d, 3 },
    { 0x55f0ff40, 4 },
    { 0, 0 },
    { 0x5e78ff42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ff46, 5 },
    { 0xc29cff47, 3 },
    { 0xb6cff47, 4 },
    { 0xd540ff48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8ff4c, 4 },
    { 0, 0 },
    { 0x3200ff4e, 3 },
    { 0x6c8cff4f, 3 },
    { 0, 0 },
    { 0xf2d0ff51, 3 },
    { 0xe568ff52, 3 },
    { 0, 0 },
    { 0xdd50ff54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888cff5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe548ff5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428ff64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec0ff68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cff6c, 3 },
    { 0, 0 },
    { 0x4b54ff6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4ff73, 3 },
    { 0x7454ff73, 3 },
    { 0xf0a4ff75, 3 },
    { 0x858cff73, 3 },
    { 0, 0 },
    { 0x5b9cff78, 3 },
    { 0x61e0ff79, 3 },
    { 0xcf6cff7a, 3 },
    { 0xdc14ff78, 3 },
    { 0x29f0ff7c, 5 },
    { 0xb4b8ff78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e4ff86, 3 },
    { 0xa0a4ff87, 4 },
    { 0x48f8ff88, 3 },
    { 0x7ee8ff88, 4 },
    { 0x8de4ff8a, 3 },
    { 0x44b4ff89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80ff93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ff96, 3 },
    { 0x340ff97, 3 },
    { 0x9188ff98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88ff9b, 3 },
    { 0, 0 },
    { 0x93f0ff9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be0ffa0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc38ffa9, 3 },
    { 0x3774ffa9, 3 },
    { 0, 0 },
    { 0x13acffac, 4 },
    { 0xb58ffac, 4 },
    { 0xb63cffac, 3 },
    { 0, 0 },
    { 0xbb70ffb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568ffb6, 3 },
    { 0, 0 },
    { 0xa700ffb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3180ffc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a84ffc8, 3 },
    { 0xaca0ffc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a0ffcc, 3 },
    { 0, 0 },
    { 0x1d88ffce, 3 },
    { 0, 0 },
    { 0x2144ffd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5584ffdd, 3 },
    { 0x7da8ffdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f88ffe1, 3 },
    { 0xf6c0ffe2, 4 },
    { 0, 0 },
    { 0x8cbcffe4, 3 },
    { 0, 0 },
    { 0x22c4ffe6, 3 },
    { 0xacf4ffe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e34ffef, 4 },
    { 0, 0 },
    { 0xb834fff1, 4 },
    { 0x71f8fff2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b0fffb, 3 },
    { 0x60f0fffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c150000, 3 },
    { 0x70490001, 4 },
    { 0xe5f50002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e50005, 4 },
    { 0, 0 },
    { 0x71190007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e9000b, 3 },
    { 0xc1f1000c, 3 },
    { 0xbbd5000c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf050011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f50017, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89001a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d1001e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e50021, 3 },
    { 0xa9010022, 3 },
    { 0x84290021, 3 },
    { 0xca690023, 3 },
    { 0x2edd0025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c50028, 3 },
    { 0, 0 },
    { 0x2f21002a, 3 },
    { 0xe5ad002b, 4 },
    { 0xdc51002b, 4 },
    { 0, 0 },
    { 0xf611002e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0610031, 4 },
    { 0x87450032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68550037, 4 },
    { 0x38310037, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d003e, 4 },
    { 0xaad1003e, 3 },
    { 0x96e10040, 4 },
    { 0xcaa50040, 4 },
    { 0, 0 },
    { 0x48e90043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5d0047, 3 },
    { 0x2a8d0047, 4 },
    { 0x96190049, 3 },
    { 0xc1550047, 4 },
    { 0x89e5004b, 3 },
    { 0x9fed0048, 4 },
    { 0, 0 },
    { 0xab41004e, 3 },
    { 0x2be1004f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6410053, 3 },
    { 0xe6910054, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1929005b, 3 },
    { 0, 0 },
    { 0xa32d005d, 3 },
    { 0, 0 },
    { 0x66e5005f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c610063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1635006a, 3 },
    { 0x1e45006b, 4 },
    { 0xa3dd006c, 4 },
    { 0xfb5d006b, 3 },
    { 0xb555006e, 3 },
    { 0xe3fd006d, 3 },
    { 0x84c10070, 4 },
    { 0x62f1006e, 3 },
    { 0, 0 },
    { 0x24b10073, 4 },
    { 0xea1d0074, 3 },
    { 0x8a2d0075, 3 },
    { 0x845d0074, 3 },
    { 0xe3f10074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d007a, 4 },
    { 0x720d007b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59450082, 3 },
    { 0xc1190083, 4 },
    { 0x28e50084, 3 },
    { 0xe4250082, 3 },
    { 0x766d0086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d10090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4d0093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0090098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf375009c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccd009f, 3 },
    { 0xd93d00a0, 3 },
    { 0x1c2d00a1, 3 },
    { 0x5a5500a2, 3 },
    { 0xf6bd00a0, 3 },
    { 0x2de100a3, 3 },
    { 0xde8900a5, 4 },
    { 0xfeb500a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48500b0, 3 },
    { 0xafb900b1, 3 },
    { 0xced100b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d00ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a500be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dd00c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784900c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1100ca, 4 },
    { 0x9f1900cb, 3 },
    { 0xbaf900cc, 3 },
    { 0x95c100cc, 4 },
    { 0x43500cd, 3 },
    { 0xb3100cd, 3 },
    { 0x71d500d0, 3 },
    { 0x121500d1, 3 },
    { 0xb0b900ce, 3 },
    { 0, 0 },
    { 0x381900d4, 3 },
    { 0xdfe500d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9d00d8, 4 },
    { 0xad7900d9, 3 },
    { 0x173900d9, 3 },
    { 0xac7100d8, 4 },
    { 0, 0 },
    { 0x7bfd00dd, 3 },
    { 0x2ec100dd, 3 },
    { 0, 0 },
    { 0xa96100e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f100e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52500e6, 3 },
    { 0xc7d100e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6500ec, 4 },
    { 0, 0 },
    { 0xb39900ee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ad00f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378d00f5, 4 },
    { 0x77c900f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86d00fb, 3 },
    { 0x6d8d00fc, 4 },
    { 0x43c500fc, 3 },
    { 0xd24900fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a10101, 3 },
    { 0x8ce90101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd210105, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ed0108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa951010b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d5010e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61490111, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c39011a, 3 },
    { 0xb3a9011b, 3 },
    { 0, 0 },
    { 0xbf21011d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d0123, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8d0129, 3 },
    { 0xe009012a, 3 },
    { 0xb955012b, 3 },
    { 0xaf750129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x923d012f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ed0133, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d0139, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x196d013c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56510142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9610145, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e10148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09014b, 4 },
    { 0x3c55014c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9014f, 3 },
    { 0xb8d90150, 4 },
    { 0xe2a50151, 3 },
    { 0xa871014f, 3 },
    { 0x130d0151, 3 },
    { 0x66d50154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f50157, 3 },
    { 0xda790158, 3 },
    { 0xecd90157, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b9015c, 3 },
    { 0xd1ad015d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee50164, 4 },
    { 0, 0 },
    { 0x5a410166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b1016a, 3 },
    { 0, 0 },
    { 0x5631016c, 3 },
    { 0, 0 },
    { 0xbac5016e, 3 },
    { 0xa931016f, 3 },
    { 0xb241016f, 3 },
    { 0x9a110171, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dd0176, 4 },
    { 0, 0 },
    { 0xb39d0178, 3 },
    { 0, 0 },
    { 0x9f51017a, 3 },
    { 0x854d017a, 3 },
    { 0, 0 },
    { 0xc881017d, 4 },
    { 0x6f29017e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90450182, 3 },
    { 0x1ed0183, 4 },
    { 0xeef50183, 3 },
    { 0x99990185, 3 },
    { 0x2e190183, 4 },
    { 0xbe150183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1d018a, 3 },
    { 0, 0 },
    { 0xe8a5018c, 3 },
    { 0xbf7d018c, 3 },
    { 0x3b95018e, 3 },
    { 0x8ee1018f, 3 },
    { 0, 0 },
    { 0x7af10191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd019b, 3 },
    { 0x6b49019c, 4 },
    { 0x19dd019d, 3 },
    { 0x1ccd019e, 3 },
    { 0, 0 },
    { 0x6f5501a0, 3 },
    { 0, 0 },
    { 0x500101a2, 3 },
    { 0xcaa901a2, 3 },
    { 0x59f901a2, 3 },
    { 0, 0 },
    { 0x694501a6, 4 },
    { 0, 0 },
    { 0x41a501a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5501ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xded101ae, 3 },
    { 0xb27501af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6d01b9, 3 },
    { 0, 0 },
    { 0x963d01bb, 3 },
    { 0, 0 },
    { 0xda6901bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46101c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69501c5, 4 },
    { 0xb01d01c5, 3 },
    { 0xfb5101c5, 4 },
    { 0x33ed01c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392d01ce, 3 },
    { 0, 0 },
    { 0xb3f101d0, 4 },
    { 0x774901d1, 3 },
    { 0xf6f501d2, 4 },
    { 0x718501d3, 3 },
    { 0, 0 },
    { 0x50e501d5, 3 },
    { 0xc4e901d5, 3 },
    { 0xfe2101d7, 3 },
    { 0, 0 },
    { 0x1c8901d9, 3 },
    { 0xd39901da, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x602901dd, 3 },
    { 0x18c101dd, 3 },
    { 0, 0 },
    { 0x5e7d01e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ad01e6, 4 },
    { 0xa50d01e7, 3 },
    { 0x3b5101e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d101eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x812501ee, 4 },
    { 0x673101ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39101f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895901f9, 4 },
    { 0xad7501f9, 3 },
    { 0x9ad901fa, 3 },
    { 0xe2dd01fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244d020b, 3 },
    { 0xe439020b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24650211, 3 },
    { 0x11850212, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8550218, 3 },
    { 0x55450218, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46d021d, 3 },
    { 0xdafd021e, 3 },
    { 0xa1bd021f, 3 },
    { 0, 0 },
    { 0x62450221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee10225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb010228, 3 },
    { 0xde010229, 3 },
    { 0, 0 },
    { 0xddb9022b, 3 },
    { 0, 0 },
    { 0x9231022d, 4 },
    { 0xddc9022e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d790231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd023d, 3 },
    { 0, 0 },
    { 0xfc2d023f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc10243, 3 },
    { 0, 0 },
    { 0x703d0245, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc939024a, 3 },
    { 0, 0 },
    { 0xe5d9024c, 4 },
    { 0, 0 },
    { 0x7121024e, 3 },
    { 0, 0 },
    { 0xd96d0250, 5 },
    { 0x14210250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c90254, 3 },
    { 0x7db90255, 3 },
    { 0, 0 },
    { 0xbf410257, 5 },
    { 0x8190257, 4 },
    { 0, 0 },
    { 0x6685025a, 3 },
    { 0, 0 },
    { 0xe79025c, 4 },
    { 0, 0 },
    { 0x4fd1025e, 3 },
    { 0x561025e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10450266, 3 },
    { 0x76850266, 3 },
    { 0xfd950266, 4 },
    { 0xfb7d0266, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35026c, 3 },
    { 0x11d026d, 4 },
    { 0x7a0d026e, 3 },
    { 0x9249026f, 4 },
    { 0xa489026e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1950274, 4 },
    { 0x24290275, 3 },
    { 0xa6d90275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc1027b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fd027e, 3 },
    { 0, 0 },
    { 0xb9fd0280, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3825028a, 3 },
    { 0xbcc9028b, 4 },
    { 0x9981028c, 3 },
    { 0, 0 },
    { 0x30d1028e, 3 },
    { 0xe6ad028f, 4 },
    { 0, 0 },
    { 0xb2610291, 3 },
    { 0x51450292, 3 },
    { 0x91790293, 4 },
    { 0x5b3d0293, 3 },
    { 0x60190295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d0298, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300d029c, 3 },
    { 0x4585029d, 3 },
    { 0xd695029c, 4 },
    { 0, 0 },
    { 0x1ff502a0, 3 },
    { 0xf2e902a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c102a7, 4 },
    { 0xe80902a8, 4 },
    { 0x8afd02a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd02ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2902b1, 3 },
    { 0xc11902b2, 4 },
    { 0x79c902b3, 3 },
    { 0x420102b3, 4 },
    { 0, 0 },
    { 0xa03d02b6, 4 },
    { 0xd65d02b7, 3 },
    { 0xca9902b6, 3 },
    { 0, 0 },
    { 0x627102ba, 3 },
    { 0, 0 },
    { 0x99102bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd502c1, 4 },
    { 0xb0102c1, 4 },
    { 0x7ff902c2, 3 },
    { 0x59a902c4, 3 },
    { 0x5f6d02c5, 3 },
    { 0x5df102c6, 3 },
    { 0x33102c5, 3 },
    { 0x7e3d02c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27102cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d02d4, 3 },
    { 0xab6502d5, 4 },
    { 0xa9c902d6, 4 },
    { 0xb7d02d5, 3 },
    { 0xc4d02d8, 3 },
    { 0xeb7902d9, 3 },
    { 0x966502d9, 3 },
    { 0x1b6502d5, 3 },
    { 0x845902dc, 5 },
    { 0xf9a902dd, 3 },
    { 0x348502db, 4 },
    { 0xae1902dd, 5 },
    { 0x468902e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3102e3, 4 },
    { 0xd9b502e4, 3 },
    { 0xbdf502e4, 3 },
    { 0, 0 },
    { 0x6ed902e7, 3 },
    { 0xfdf102e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c902ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0902ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6502f4, 4 },
    { 0x785502f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d502f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x127902fb, 3 },
    { 0x97dd02fc, 3 },
    { 0xfa9902fb, 3 },
    { 0, 0 },
    { 0xb9dd02ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749d0309, 3 },
    { 0xa5a50309, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc1030d, 4 },
    { 0, 0 },
    { 0x9421030f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73d0312, 3 },
    { 0x85fd0313, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7249031f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdd0328, 4 },
    { 0x7ca50329, 3 },
    { 0xc2190328, 4 },
    { 0, 0 },
    { 0x38cd032c, 3 },
    { 0xe5ed032d, 3 },
    { 0xd421032e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9d0331, 3 },
    { 0x89610332, 3 },
    { 0x76f90333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d0337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed31033c, 4 },
    { 0, 0 },
    { 0xdbd033e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e50343, 3 },
    { 0xbd590344, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b50348, 3 },
    { 0, 0 },
    { 0x88e1034a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab650351, 3 },
    { 0x3ed50352, 4 },
    { 0x67850352, 4 },
    { 0xb8d10354, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6910358, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86010364, 4 },
    { 0, 0 },
    { 0x18310366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0d0369, 3 },
    { 0xd121036a, 5 },
    { 0x25dd036b, 3 },
    { 0xee2d036c, 3 },
    { 0xbfa1036d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f50370, 3 },
    { 0x85fd0371, 3 },
    { 0, 0 },
    { 0xa80d0373, 4 },
    { 0xfd1d0373, 3 },
    { 0xbb6d0375, 3 },
    { 0x8a090376, 3 },
    { 0x84010373, 3 },
    { 0x952d0378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68990380, 3 },
    { 0xf5b50381, 3 },
    { 0x478d0381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad10385, 4 },
    { 0x21890386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef750389, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1038d, 3 },
    { 0, 0 },
    { 0x98ad038f, 3 },
    { 0xbed5038f, 3 },
    { 0x5c110391, 4 },
    { 0, 0 },
    { 0xd4090393, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5750398, 3 },
    { 0, 0 },
    { 0xfa01039a, 3 },
    { 0x6ecd039a, 3 },
    { 0, 0 },
    { 0xe331039d, 3 },
    { 0xa689039e, 4 },
    { 0xf945039e, 3 },
    { 0x8c95039f, 3 },
    { 0x3b7903a1, 3 },
    { 0x4ac903a2, 3 },
    { 0x559d03a2, 3 },
    { 0x975903a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4103a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x712503aa, 3 },
    { 0x709d03aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3103b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cd03b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b103ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08103bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bd03c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e903c6, 3 },
    { 0xd0dd03c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a903cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14503d0, 3 },
    { 0xe8d103d1, 3 },
    { 0xec5103d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92503d9, 3 },
    { 0, 0 },
    { 0x8e5103db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cd03de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2d03e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x329103e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448103eb, 3 },
    { 0x5e4503ec, 3 },
    { 0, 0 },
    { 0xaaed03ee, 3 },
    { 0, 0 },
    { 0xa79503f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5503fb, 3 },
    { 0x5ca503fb, 4 },
    { 0x22d903fc, 3 },
    { 0, 0 },
    { 0x3d9503ff, 4 },
    { 0, 0 },
    { 0x435d0401, 3 },
    { 0xdfd90402, 4 },
    { 0x7e8d0401, 5 },
    { 0, 0 },
    { 0x82390405, 3 },
    { 0, 0 },
    { 0x1f050407, 4 },
    { 0xebad0408, 3 },
    { 0x7ab90408, 4 },
    { 0x8ec50409, 3 },
    { 0x41c5040b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf961040f, 3 },
    { 0x7f65040f, 4 },
    { 0x389d040f, 4 },
    { 0x50610410, 4 },
    { 0x6bf50413, 4 },
    { 0, 0 },
    { 0x26590415, 3 },
    { 0, 0 },
    { 0x74310417, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a1041e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6350422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad10425, 3 },
    { 0x57710426, 3 },
    { 0xd3ed0425, 3 },
    { 0x52f10425, 4 },
    { 0, 0 },
    { 0x5f7d042a, 4 },
    { 0x7d3d042a, 3 },
    { 0x3799042a, 3 },
    { 0x7e39042d, 3 },
    { 0xf9a9042e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f90439, 3 },
    { 0, 0 },
    { 0x5b3d043b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce29043e, 3 },
    { 0xb571043f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d0442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91790449, 3 },
    { 0xf7b50449, 3 },
    { 0, 0 },
    { 0xb1f5044c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf19044f, 3 },
    { 0xdda50450, 3 },
    { 0xd56d0450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dd0454, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a590457, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f9045a, 4 },
    { 0x6365045a, 4 },
    { 0x4fd9045c, 4 },
    { 0xfb21045d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc90462, 3 },
    { 0x7a790462, 4 },
    { 0x43750464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2690468, 3 },
    { 0x2aa10469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65046d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a850477, 3 },
    { 0xe0e90478, 3 },
    { 0xd190478, 3 },
    { 0x2ab1047a, 4 },
    { 0, 0 },
    { 0x2329047c, 3 },
    { 0x19ed047c, 4 },
    { 0xb5f1047e, 3 },
    { 0x5369047e, 3 },
    { 0, 0 },
    { 0xa5dd0481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e50484, 3 },
    { 0x9d290484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fd0489, 3 },
    { 0, 0 },
    { 0x641048b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bd0494, 4 },
    { 0xb0e10494, 3 },
    { 0x4fe50496, 3 },
    { 0x1b5d0496, 3 },
    { 0, 0 },
    { 0x48310499, 3 },
    { 0x66a1049a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf104a2, 3 },
    { 0xb45104a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b504a6, 3 },
    { 0x4d504a7, 4 },
    { 0x135504a8, 3 },
    { 0x3bbd04a7, 3 },
    { 0, 0 },
    { 0xf1d904ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46104ae, 3 },
    { 0, 0 },
    { 0xa1c904b0, 3 },
    { 0, 0 },
    { 0xdc6504b2, 3 },
    { 0x2e1104b3, 3 },
    { 0xf75d04b4, 3 },
    { 0, 0 },
    { 0x3b0d04b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdded04bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34104be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32904c4, 3 },
    { 0xaec104c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99904c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x525104cc, 4 },
    { 0x34fd04cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dd04d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5504dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3d04e2, 3 },
    { 0xab5104e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d04eb, 3 },
    { 0xbbd504ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5904ef, 4 },
    { 0x2ec504ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c904f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110904f9, 3 },
    { 0x4f7d04fa, 3 },
    { 0xb9ad04fb, 3 },
    { 0xe07104fb, 3 },
    { 0xcd9104fb, 3 },
    { 0, 0 },
    { 0xe5e504ff, 3 },
    { 0x60fd0500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd50503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0310507, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed050e, 3 },
    { 0, 0 },
    { 0x22850510, 3 },
    { 0x5b0d0511, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88510516, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c5051b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd0521, 3 },
    { 0xafb50521, 3 },
    { 0xc2950523, 3 },
    { 0x26a90524, 3 },
    { 0x41f50521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6150529, 3 },
    { 0x3735052a, 3 },
    { 0xee99052b, 3 },
    { 0x7d25052c, 3 },
    { 0, 0 },
    { 0x4f5052e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c50534, 3 },
    { 0xf6710535, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb951053b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d25053f, 3 },
    { 0, 0 },
    { 0x5bbd0541, 3 },
    { 0xae210542, 3 },
    { 0x22dd0543, 3 },
    { 0x55dd0541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a1054c, 3 },
    { 0x548d054d, 3 },
    { 0x4d2d054e, 4 },
    { 0x17dd054e, 3 },
    { 0, 0 },
    { 0xccb50551, 3 },
    { 0x8bd90551, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b890558, 3 },
    { 0xf85d0558, 3 },
    { 0xc8f9055a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c90563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a290569, 3 },
    { 0x8b890569, 4 },
    { 0x23fd056b, 3 },
    { 0x52a50569, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c1056f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a950574, 3 },
    { 0x58390574, 3 },
    { 0x60290576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a10579, 3 },
    { 0xbe81057a, 3 },
    { 0x43a5057b, 3 },
    { 0x527d057a, 3 },
    { 0xee8d0579, 3 },
    { 0, 0 },
    { 0x7d09057f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512d0583, 3 },
    { 0x6a7d0584, 3 },
    { 0, 0 },
    { 0x97650586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a9058b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d058f, 3 },
    { 0x9d390590, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a50597, 3 },
    { 0, 0 },
    { 0x11990599, 3 },
    { 0x1995059a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4d059d, 3 },
    { 0, 0 },
    { 0x1051059f, 3 },
    { 0x6bdd059f, 3 },
    { 0, 0 },
    { 0xaf9905a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497d05a6, 4 },
    { 0x1f5105a7, 3 },
    { 0xd2a905a6, 3 },
    { 0x9d5905a9, 3 },
    { 0x5d905aa, 3 },
    { 0x268105ab, 3 },
    { 0xe41505ab, 3 },
    { 0, 0 },
    { 0x17ed05ae, 3 },
    { 0x97f505af, 3 },
    { 0x7a3105b0, 4 },
    { 0, 0 },
    { 0x339905b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c905b6, 3 },
    { 0x855105b7, 3 },
    { 0x44ad05b8, 3 },
    { 0x4d8905b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa505bc, 3 },
    { 0x58a905bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b105c1, 3 },
    { 0xb99905c2, 3 },
    { 0x351105c3, 4 },
    { 0xce8105c4, 3 },
    { 0, 0 },
    { 0x79a505c6, 4 },
    { 0, 0 },
    { 0x9a4505c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0505d6, 3 },
    { 0xd83105d7, 3 },
    { 0x774905d7, 3 },
    { 0, 0 },
    { 0x11d105da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdded05de, 3 },
    { 0, 0 },
    { 0xc2905e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76105e5, 3 },
    { 0x822d05e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x360905e9, 5 },
    { 0xee3505e9, 3 },
    { 0xf46905ea, 3 },
    { 0x4bb505ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809905f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95905fa, 3 },
    { 0x6e0d05fa, 3 },
    { 0x40a905fc, 3 },
    { 0x36ad05fd, 4 },
    { 0x6d4905fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17790604, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b90607, 3 },
    { 0, 0 },
    { 0xf7a90609, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949060f, 3 },
    { 0, 0 },
    { 0x618d0611, 4 },
    { 0x7f4d0611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d0623, 3 },
    { 0, 0 },
    { 0xc0c50625, 4 },
    { 0xf9790625, 4 },
    { 0, 0 },
    { 0x30850628, 3 },
    { 0, 0 },
    { 0xfb65062a, 3 },
    { 0, 0 },
    { 0x2471062c, 3 },
    { 0xc0ad062d, 3 },
    { 0x702d062e, 3 },
    { 0x2049062f, 3 },
    { 0x52fd0630, 3 },
    { 0x9f89062f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b10637, 3 },
    { 0, 0 },
    { 0x69850639, 3 },
    { 0, 0 },
    { 0xa70d063b, 3 },
    { 0xfdcd063c, 3 },
    { 0xb5c9063d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c290640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d0648, 3 },
    { 0, 0 },
    { 0x7479064a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9290652, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b90655, 3 },
    { 0xcb650656, 4 },
    { 0x6da90656, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780d065d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97910661, 3 },
    { 0xe4d10662, 3 },
    { 0x9f1d0661, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d0668, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a5066b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe9066f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcd0672, 3 },
    { 0xbcd90673, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a50676, 4 },
    { 0, 0 },
    { 0xe74d0678, 3 },
    { 0, 0 },
    { 0x8765067a, 4 },
    { 0xfb6d067b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3610680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30390684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde510687, 4 },
    { 0xd490688, 3 },
    { 0, 0 },
    { 0x23e9068a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3589068d, 3 },
    { 0x4b59068d, 3 },
    { 0x2cbd068f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bd0694, 3 },
    { 0x7eed0695, 3 },
    { 0x80350695, 3 },
    { 0x118d0697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d069a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d99069e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d106a4, 4 },
    { 0x955106a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x940506a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29506b0, 3 },
    { 0x981906b0, 3 },
    { 0xff106b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea506b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43506c2, 3 },
    { 0xa1ad06c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998906c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaed06d6, 3 },
    { 0xab3106d7, 3 },
    { 0x423d06d8, 4 },
    { 0xb27906d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603d06df, 4 },
    { 0, 0 },
    { 0x886d06e1, 3 },
    { 0xb29d06e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60506e7, 4 },
    { 0x483d06e8, 3 },
    { 0, 0 },
    { 0xb9a106ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9106ed, 3 },
    { 0x826506ee, 4 },
    { 0x2be106ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7106f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4506fc, 4 },
    { 0x612106fd, 4 },
    { 0x302d06fe, 3 },
    { 0xc0fd06fe, 3 },
    { 0x6e5106fd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188d070a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c5070e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3350712, 3 },
    { 0x69890713, 3 },
    { 0, 0 },
    { 0x483d0715, 3 },
    { 0xc2390716, 4 },
    { 0, 0 },
    { 0xe9c10718, 4 },
    { 0x61b50719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853d0721, 3 },
    { 0x6fb90722, 3 },
    { 0, 0 },
    { 0xbdc10724, 3 },
    { 0, 0 },
    { 0x11f50726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17310729, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1319072c, 3 },
    { 0xe7d072d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a690730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a390734, 3 },
    { 0x2c5d0734, 3 },
    { 0, 0 },
    { 0x40590737, 4 },
    { 0xe60d0738, 4 },
    { 0xc9b10737, 3 },
    { 0, 0 },
    { 0x1a95073b, 3 },
    { 0x3891073c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12090740, 3 },
    { 0, 0 },
    { 0x54ed0742, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16890745, 3 },
    { 0x94050746, 3 },
    { 0x22a10746, 3 },
    { 0, 0 },
    { 0xa3cd0749, 3 },
    { 0x8be9074a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e90750, 4 },
    { 0x50d90751, 5 },
    { 0x59f90752, 3 },
    { 0xd4d0752, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd650756, 3 },
    { 0, 0 },
    { 0x2a2d0758, 3 },
    { 0x43a50759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f71075e, 3 },
    { 0, 0 },
    { 0xf5c90760, 3 },
    { 0x13a50761, 3 },
    { 0, 0 },
    { 0x20910763, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfd0766, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68690769, 4 },
    { 0x7695076a, 3 },
    { 0x312d076b, 4 },
    { 0, 0 },
    { 0x2721076d, 3 },
    { 0xb809076e, 4 },
    { 0xf2bd076d, 3 },
    { 0xf1150770, 3 },
    { 0, 0 },
    { 0x842d0772, 3 },
    { 0, 0 },
    { 0xa8350774, 4 },
    { 0x4da10775, 3 },
    { 0x2fbd0775, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87d077a, 3 },
    { 0x8261077b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655d0784, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3849078a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf369078d, 3 },
    { 0, 0 },
    { 0x9c1d078f, 3 },
    { 0x16c10790, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ad0793, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dd0796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670d079a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfd07a1, 3 },
    { 0xe39d07a2, 3 },
    { 0xcd7907a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42907a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa507aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564507af, 3 },
    { 0xc32907b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd907b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0507b9, 3 },
    { 0x5ef907ba, 3 },
    { 0x2e8d07bb, 3 },
    { 0x1c1507bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68907c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3907c9, 4 },
    { 0x929907ca, 3 },
    { 0xaf9907c9, 3 },
    { 0x6a4907ca, 3 },
    { 0xe10d07cb, 3 },
    { 0xf17107ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0907d3, 3 },
    { 0xe46107d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da907d9, 4 },
    { 0x348107da, 3 },
    { 0x3a3507db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd107e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd07e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad07eb, 4 },
    { 0, 0 },
    { 0xa24907ed, 4 },
    { 0xf56507ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a10800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc50803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5dd0806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89080d, 4 },
    { 0x5259080d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37710811, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e90815, 4 },
    { 0x5ead0816, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9649081d, 4 },
    { 0, 0 },
    { 0x3cb1081f, 3 },
    { 0xf2ad081f, 3 },
    { 0x22690821, 3 },
    { 0x9b35081f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34d0827, 4 },
    { 0x8fdd0827, 4 },
    { 0, 0 },
    { 0xec51082a, 4 },
    { 0x975082a, 4 },
    { 0xc47d082c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x669082f, 4 },
    { 0xccf50830, 3 },
    { 0xc2d90830, 3 },
    { 0x8290830, 3 },
    { 0x79ad0833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a010836, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d0839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26890840, 4 },
    { 0, 0 },
    { 0x4b690842, 4 },
    { 0, 0 },
    { 0x359d0844, 3 },
    { 0xa4950845, 3 },
    { 0, 0 },
    { 0xd72d0847, 3 },
    { 0x63450847, 3 },
    { 0xf0b90847, 3 },
    { 0x4799084a, 3 },
    { 0, 0 },
    { 0x93ad084c, 3 },
    { 0, 0 },
    { 0x1cf9084e, 3 },
    { 0, 0 },
    { 0x23dd0850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3d0856, 3 },
    { 0x6f650856, 3 },
    { 0x87310857, 3 },
    { 0, 0 },
    { 0x5e11085a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c1085d, 4 },
    { 0, 0 },
    { 0xf41085f, 3 },
    { 0, 0 },
    { 0xd1310861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad510864, 3 },
    { 0, 0 },
    { 0x8e650866, 4 },
    { 0x1d390867, 3 },
    { 0x57a10867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad086b, 3 },
    { 0, 0 },
    { 0x42a9086d, 3 },
    { 0x49c5086d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26490871, 3 },
    { 0x4d310871, 3 },
    { 0xfaa50873, 3 },
    { 0, 0 },
    { 0xf010875, 3 },
    { 0, 0 },
    { 0xf15d0877, 3 },
    { 0xa8b90878, 4 },
    { 0x8010879, 4 },
    { 0xdbe9087a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6625087d, 3 },
    { 0xd8fd087d, 3 },
    { 0, 0 },
    { 0xf8490880, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60d0883, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d0886, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1210892, 3 },
    { 0xf2410892, 3 },
    { 0x183d0893, 3 },
    { 0xdb250895, 4 },
    { 0x55650896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db10899, 4 },
    { 0x2119089a, 4 },
    { 0, 0 },
    { 0xca81089c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e108a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218908a9, 3 },
    { 0xfacd08aa, 4 },
    { 0x983908ab, 3 },
    { 0xec9108ac, 3 },
    { 0xbe8908ac, 3 },
    { 0, 0 },
    { 0x1c0d08af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541908b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab108b9, 3 },
    { 0x434108ba, 5 },
    { 0xefad08ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e508c0, 3 },
    { 0x2b2108c1, 4 },
    { 0xa92508c0, 4 },
    { 0x4d3508c3, 3 },
    { 0x102d08c1, 3 },
    { 0x1bd908c5, 3 },
    { 0x99c908c6, 3 },
    { 0xb1f908c7, 3 },
    { 0x3b908c8, 3 },
    { 0x27b908c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958d08cd, 3 },
    { 0, 0 },
    { 0xb71108cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e908d2, 4 },
    { 0, 0 },
    { 0x4d9508d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04108d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f908dc, 4 },
    { 0, 0 },
    { 0x641508de, 3 },
    { 0, 0 },
    { 0xfba908e0, 3 },
    { 0x48ad08e1, 4 },
    { 0xe2cd08e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5108e5, 3 },
    { 0x5e2d08e6, 3 },
    { 0x5ad508e7, 4 },
    { 0x985d08e8, 3 },
    { 0xe13908e9, 3 },
    { 0x91a508e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3d08ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8108f0, 3 },
    { 0x77108f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a908f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a108f8, 3 },
    { 0x3e6908f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2508fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40990902, 3 },
    { 0xce650903, 3 },
    { 0xcfd10904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14410908, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4891090c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb399090f, 3 },
    { 0xcb90910, 3 },
    { 0xb5090911, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f790914, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e410917, 3 },
    { 0xa3c10917, 4 },
    { 0x5b5d0917, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656d0921, 3 },
    { 0x19f90921, 4 },
    { 0, 0 },
    { 0x67a10924, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x228d0927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59092d, 4 },
    { 0, 0 },
    { 0xfe11092f, 4 },
    { 0x5c690930, 3 },
    { 0x74f10930, 3 },
    { 0, 0 },
    { 0x94ed0933, 4 },
    { 0xbe7d0934, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd790937, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9093e, 4 },
    { 0, 0 },
    { 0xff190940, 4 },
    { 0, 0 },
    { 0x40890942, 3 },
    { 0x42e50943, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca190946, 3 },
    { 0x225d0947, 3 },
    { 0x77a10948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd094b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dd0950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a10956, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x72110959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cd095c, 4 },
    { 0xee21095c, 3 },
    { 0xa4a1095e, 3 },
    { 0x69fd095f, 3 },
    { 0x2289095f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41610965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68d096b, 3 },
    { 0xc271096b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x507d096f, 3 },
    { 0, 0 },
    { 0x49290971, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13150975, 4 },
    { 0, 0 },
    { 0xfb5d0977, 4 },
    { 0xb09d0977, 4 },
    { 0xcdcd0977, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb490982, 3 },
    { 0xfadd0983, 3 },
    { 0, 0 },
    { 0x5f690985, 3 },
    { 0xe0f50986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf79098d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd390991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2250995, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb911099b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282d09a0, 4 },
    { 0, 0 },
    { 0xf70d09a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57509a6, 4 },
    { 0x667109a7, 4 },
    { 0x7be509a8, 3 },
    { 0xcf6509a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba109b3, 3 },
    { 0xae6909b4, 3 },
    { 0x66f509b3, 3 },
    { 0xea4509b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7109ba, 3 },
    { 0x49d109bb, 4 },
    { 0xee1d09bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3109c0, 3 },
    { 0xecf509c1, 3 },
    { 0x383509c1, 3 },
    { 0, 0 },
    { 0x33c509c4, 4 },
    { 0xa92d09c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214909cb, 4 },
    { 0x20e109cc, 4 },
    { 0x5b3909cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbd09d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653909d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c909da, 3 },
    { 0xc0cd09db, 3 },
    { 0x3ead09db, 3 },
    { 0, 0 },
    { 0x263909de, 3 },
    { 0xe8c909de, 4 },
    { 0, 0 },
    { 0x8f8509e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1109e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d09e8, 3 },
    { 0x777909e9, 3 },
    { 0x261909ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd509f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x658909f4, 3 },
    { 0, 0 },
    { 0x28f909f6, 3 },
    { 0, 0 },
    { 0xbc7909f8, 3 },
    { 0, 0 },
    { 0xf7e909fa, 3 },
    { 0xcb2109fb, 4 },
    { 0x81d909fc, 3 },
    { 0, 0 },
    { 0x72f509fe, 3 },
    { 0, 0 },
    { 0x71250a00, 3 },
    { 0xc6c50a01, 3 },
    { 0x90090a02, 4 },
    { 0x78c10a03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1150a07, 3 },
    { 0, 0 },
    { 0x63590a09, 4 },
    { 0x5b690a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4750a12, 3 },
    { 0, 0 },
    { 0x51210a14, 3 },
    { 0x82a10a14, 3 },
    { 0xdb5d0a16, 4 },
    { 0x19890a14, 3 },
    { 0x8b4d0a17, 3 },
    { 0, 0 },
    { 0xffc90a1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa150a24, 3 },
    { 0xfa590a24, 3 },
    { 0x28510a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807d0a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74710a32, 3 },
    { 0x610d0a33, 3 },
    { 0, 0 },
    { 0x3db50a35, 5 },
    { 0xe2e90a36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250a3a, 5 },
    { 0xc8950a3b, 4 },
    { 0xa0410a3c, 4 },
    { 0x69bd0a3c, 3 },
    { 0xc1190a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c90a45, 3 },
    { 0, 0 },
    { 0xa4850a47, 3 },
    { 0x3cd0a48, 4 },
    { 0x1d010a48, 3 },
    { 0, 0 },
    { 0xad090a4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d0a50, 3 },
    { 0x96950a51, 3 },
    { 0x24e50a51, 4 },
    { 0xacf10a53, 4 },
    { 0x38f50a53, 3 },
    { 0xdb2d0a54, 3 },
    { 0xc8710a51, 4 },
    { 0xbe210a57, 3 },
    { 0xcc1d0a56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fd0a5f, 3 },
    { 0, 0 },
    { 0x5a310a61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce10a65, 4 },
    { 0x5e050a66, 3 },
    { 0xf96d0a66, 3 },
    { 0x76590a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d90a6b, 3 },
    { 0xe2350a6c, 3 },
    { 0, 0 },
    { 0x8ed50a6e, 3 },
    { 0, 0 },
    { 0xeb10a70, 3 },
    { 0xf2f50a71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d50a7a, 4 },
    { 0x416d0a7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e50a7e, 3 },
    { 0xc7090a7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f750a84, 4 },
    { 0x410a85, 4 },
    { 0xae150a85, 3 },
    { 0x72d90a86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dd0a8b, 3 },
    { 0x59910a8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd90a91, 3 },
    { 0x41a50a91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0a95, 3 },
    { 0xac410a96, 3 },
    { 0x56c90a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c10a9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50090aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac90aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb90aab, 3 },
    { 0xc990aac, 3 },
    { 0x54c10aad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb310ab0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d950ab3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff450abb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d0abf, 3 },
    { 0x7390ac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c10ac4, 3 },
    { 0x6ff50ac5, 3 },
    { 0x68e10ac6, 3 },
    { 0x44dd0ac6, 3 },
    { 0xaaf10ac6, 3 },
    { 0xe3590ac9, 4 },
    { 0x8e0d0ac7, 3 },
    { 0x96650ac8, 3 },
    { 0xb9390ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da90ad0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df50ad4, 3 },
    { 0xf96d0ad4, 3 },
    { 0, 0 },
    { 0xfb490ad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d50ada, 3 },
    { 0xf0c90ada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce250ae9, 4 },
    { 0x28a50ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb90af4, 3 },
    { 0, 0 },
    { 0x8e810af6, 4 },
    { 0, 0 },
    { 0x11fd0af8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda250afb, 4 },
    { 0xc550afb, 5 },
    { 0, 0 },
    { 0x6a610afe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d0b03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb490b07, 3 },
    { 0x41010b08, 3 },
    { 0xcf890b08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f90b0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c90b17, 3 },
    { 0, 0 },
    { 0xb5310b19, 3 },
    { 0x36cd0b1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c10b1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83510b24, 3 },
    { 0x6cd0b24, 3 },
    { 0xc91d0b26, 3 },
    { 0x9e610b27, 3 },
    { 0x2dcd0b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb10b2b, 4 },
    { 0x90d90b2b, 3 },
    { 0xce990b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e90b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2d0b38, 4 },
    { 0xf5810b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49790b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b990b41, 3 },
    { 0xd7e90b42, 4 },
    { 0, 0 },
    { 0x3e590b44, 3 },
    { 0x4ca90b44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68090b49, 3 },
    { 0xff6d0b4a, 4 },
    { 0, 0 },
    { 0xf7490b4c, 4 },
    { 0, 0 },
    { 0x3fa10b4e, 3 },
    { 0x6cb50b4f, 3 },
    { 0x5f1d0b50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd50b53, 4 },
    { 0x2b910b54, 3 },
    { 0x6e50b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4610b5d, 3 },
    { 0x12bd0b5d, 4 },
    { 0x19290b5d, 3 },
    { 0xda310b60, 4 },
    { 0xbb0d0b61, 3 },
    { 0xc9b90b60, 3 },
    { 0xc3c90b5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d50b6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f10b70, 3 },
    { 0, 0 },
    { 0xfb150b72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ad0b77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fd0b8a, 3 },
    { 0xa7310b8a, 4 },
    { 0, 0 },
    { 0xdb4d0b8d, 3 },
    { 0xe4a50b8e, 3 },
    { 0xb65d0b8e, 3 },
    { 0, 0 },
    { 0x5a8d0b91, 3 },
    { 0, 0 },
    { 0xca1d0b93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2550b96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee10b9d, 3 },
    { 0, 0 },
    { 0xd2210b9f, 3 },
    { 0xcbb50ba0, 3 },
    { 0x546d0ba1, 3 },
    { 0xab710ba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1990ba5, 4 },
    { 0x7e9d0ba6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce090ba9, 4 },
    { 0xf8cd0baa, 3 },
    { 0x28450baa, 3 },
    { 0x50b90baa, 5 },
    { 0x7f90bad, 3 },
    { 0xe09d0bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x826d0bb1, 3 },
    { 0x7efd0bb1, 3 },
    { 0xa0dd0bb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63090bb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b090bbd, 3 },
    { 0xa8bd0bbe, 4 },
    { 0xbe8d0bbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed510bc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f90bc8, 4 },
    { 0x44850bc9, 3 },
    { 0x90910bca, 4 },
    { 0x94b10bca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8650bce, 3 },
    { 0, 0 },
    { 0x3cc50bd0, 4 },
    { 0xbb690bd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ed0bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75850be3, 3 },
    { 0, 0 },
    { 0xb7dd0be5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f050be8, 4 },
    { 0x92dd0be8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cd0bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0950bf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f50bf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x727d0bfc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x562d0bff, 3 },
    { 0x1f510bff, 3 },
    { 0xa7ed0c00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5d0c04, 3 },
    { 0x55110c04, 3 },
    { 0, 0 },
    { 0xc3b50c07, 3 },
    { 0, 0 },
    { 0x199d0c09, 3 },
    { 0x1ce50c0a, 4 },
    { 0x4bcd0c09, 4 },
    { 0x66cd0c0c, 3 },
    { 0, 0 },
    { 0xdfa10c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x107d0c11, 3 },
    { 0xefd10c12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x844d0c15, 3 },
    { 0, 0 },
    { 0x6e1d0c17, 3 },
    { 0, 0 },
    { 0xb6490c19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff10c1d, 3 },
    { 0xe8cd0c1e, 3 },
    { 0xab5d0c1f, 3 },
    { 0x165d0c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364d0c27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa90c2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c50c2f, 3 },
    { 0, 0 },
    { 0x2c050c31, 3 },
    { 0x46790c32, 3 },
    { 0x13cd0c33, 3 },
    { 0x76e90c33, 3 },
    { 0, 0 },
    { 0x467d0c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd510c3a, 3 },
    { 0xa1810c3b, 3 },
    { 0xf5650c3c, 3 },
    { 0x98c10c3a, 3 },
    { 0x6d050c3e, 3 },
    { 0x16210c3c, 4 },
    { 0x33e90c3e, 3 },
    { 0, 0 },
    { 0x31d0c42, 3 },
    { 0, 0 },
    { 0xe0990c44, 3 },
    { 0, 0 },
    { 0xb3890c46, 4 },
    { 0, 0 },
    { 0x96950c48, 4 },
    { 0x79c90c48, 3 },
    { 0xdfc90c4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9090c4d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7790c50, 3 },
    { 0, 0 },
    { 0x45d10c52, 3 },
    { 0xd9d10c53, 3 },
    { 0xb3a10c54, 3 },
    { 0xf9ad0c55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6750c60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef290c64, 3 },
    { 0, 0 },
    { 0xcadd0c66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26190c69, 3 },
    { 0x39310c6a, 3 },
    { 0, 0 },
    { 0x484d0c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e50c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84450c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e90c7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a90c7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41550c83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de10c87, 3 },
    { 0, 0 },
    { 0xf22d0c89, 3 },
    { 0xea50c89, 3 },
    { 0, 0 },
    { 0x32550c8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4d0c8f, 3 },
    { 0, 0 },
    { 0xfe90c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa710c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec90ca2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f50ca6, 3 },
    { 0, 0 },
    { 0xf3490ca8, 3 },
    { 0xec010ca9, 3 },
    { 0x7be50caa, 3 },
    { 0, 0 },
    { 0x45450cac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a10cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee850cb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x59050cb7, 3 },
    { 0x2ea50cb8, 3 },
    { 0xa450cb7, 3 },
    { 0x7cc90cba, 3 },
    { 0, 0 },
    { 0xee710cbc, 3 },
    { 0xbcd50cbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da90cc0, 3 },
    { 0xeb650cc1, 4 },
    { 0x69a10cc1, 3 },
    { 0, 0 },
    { 0xa6dd0cc4, 3 },
    { 0, 0 },
    { 0x2c050cc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1d0ccc, 3 },
    { 0xcded0ccd, 3 },
    { 0x109d0cce, 4 },
    { 0, 0 },
    { 0xa2ed0cd0, 3 },
    { 0x2f010cd0, 4 },
    { 0x36390cd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefd0cd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0cda, 3 },
    { 0x3ffd0cda, 4 },
    { 0x996d0cdb, 3 },
    { 0, 0 },
    { 0xedcd0cde, 3 },
    { 0xefa90cdf, 4 },
    { 0xf2ad0cde, 4 },
    { 0, 0 },
    { 0x81b90ce2, 3 },
    { 0, 0 },
    { 0xfbd10ce4, 3 },
    { 0x76990ce5, 3 },
    { 0x19350ce6, 3 },
    { 0x66c50ce6, 3 },
    { 0xdddd0ce8, 3 },
    { 0x44390ce5, 3 },
    { 0x1e10ce5, 3 },
    { 0x14fd0ce5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be90cef, 3 },
    { 0x812d0cf0, 3 },
    { 0x50910cf1, 4 },
    { 0x9990cf1, 3 },
    { 0x125d0cf3, 3 },
    { 0xd7310cf3, 3 },
    { 0x72b10cf1, 3 },
    { 0x73a90cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd0cfa, 4 },
    { 0, 0 },
    { 0x55350cfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c90d00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250d0d06, 3 },
    { 0x7d250d06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b90d0c, 3 },
    { 0x36dd0d0c, 3 },
    { 0x7fe90d0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0890d13, 3 },
    { 0, 0 },
    { 0xfe6d0d15, 4 },
    { 0xaf910d16, 3 },
    { 0x1efd0d17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d50d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed410d23, 3 },
    { 0, 0 },
    { 0x82650d25, 3 },
    { 0x64d0d26, 3 },
    { 0xfc710d27, 3 },
    { 0, 0 },
    { 0xa0410d29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d290d32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310d35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac50d38, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1090d3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d90d41, 3 },
    { 0, 0 },
    { 0x12c10d43, 3 },
    { 0x30850d44, 3 },
    { 0x89f50d45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2d0d50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c50d56, 3 },
    { 0xc5710d56, 3 },
    { 0, 0 },
    { 0xe2910d59, 3 },
    { 0xd8850d59, 3 },
    { 0x30850d5a, 3 },
    { 0xc1210d5a, 3 },
    { 0x26850d5a, 3 },
    { 0xcaf50d59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b510d62, 3 },
    { 0x5ce90d63, 3 },
    { 0, 0 },
    { 0x78150d65, 3 },
    { 0, 0 },
    { 0xfdd10d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0150d6a, 3 },
    { 0xc3c10d6b, 3 },
    { 0xfa190d6a, 3 },
    { 0x45490d6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e10d70, 3 },
    { 0, 0 },
    { 0x5b10d72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d50d77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269d0d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c50d83, 3 },
    { 0xde3d0d84, 3 },
    { 0x2b990d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc10d8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1390d90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec10d93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5990d9a, 5 },
    { 0xeb950d9b, 3 },
    { 0, 0 },
    { 0x16b50d9d, 4 },
    { 0xa98d0d9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a90da2, 4 },
    { 0x87ad0da3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77050da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52d0dae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14150db2, 4 },
    { 0xac90db3, 3 },
    { 0x79850db3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a90db8, 3 },
    { 0, 0 },
    { 0xee850dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb90dc2, 3 },
    { 0x9be90dc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b50dc9, 4 },
    { 0, 0 },
    { 0x721d0dcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a90dce, 4 },
    { 0x30c90dce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e450dd3, 3 },
    { 0x4a610dd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d0ddb, 3 },
    { 0x3d2d0ddc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1310ddf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x52490de2, 3 },
    { 0x1dbd0de2, 3 },
    { 0xb8b50de4, 3 },
    { 0x73f90de5, 3 },
    { 0, 0 },
    { 0x7fc50de7, 3 },
    { 0, 0 },
    { 0x96dd0de9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9310dee, 3 },
    { 0, 0 },
    { 0x7c510df0, 3 },
    { 0x556d0df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9590df4, 3 },
    { 0, 0 },
    { 0xca090df6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67310dfb, 3 },
    { 0x95fd0dfc, 3 },
    { 0xc5ed0dfb, 3 },
    { 0, 0 },
    { 0x8d4d0dff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b90e07, 3 },
    { 0xc8f10e08, 3 },
    { 0x5e9d0e09, 4 },
    { 0, 0 },
    { 0xa46d0e0b, 3 },
    { 0, 0 },
    { 0xe450e0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83410e14, 3 },
    { 0xc8690e14, 3 },
    { 0, 0 },
    { 0xb0b90e17, 3 },
    { 0xb9bd0e18, 3 },
    { 0xcad0e18, 4 },
    { 0x890d0e1a, 3 },
    { 0x240d0e1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c690e1f, 3 },
    { 0, 0 },
    { 0x62b50e21, 3 },
    { 0, 0 },
    { 0x69750e23, 4 },
    { 0xe7150e24, 3 },
    { 0, 0 },
    { 0x80490e26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc90e29, 3 },
    { 0x6c50e2a, 4 },
    { 0x3f2d0e2b, 4 },
    { 0x360d0e2c, 3 },
    { 0xcd410e2d, 3 },
    { 0xa3d50e2b, 3 },
    { 0x49f10e2c, 4 },
    { 0x17410e2d, 3 },
    { 0x45890e2d, 3 },
    { 0x25d50e32, 3 },
    { 0xf9910e33, 4 },
    { 0xcab90e2d, 3 },
    { 0x19e90e35, 4 },
    { 0x49f50e36, 3 },
    { 0xb1650e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab910e3c, 3 },
    { 0x84d10e3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2110e41, 3 },
    { 0, 0 },
    { 0xf5f10e43, 4 },
    { 0x94d10e44, 3 },
    { 0, 0 },
    { 0xf5a10e46, 3 },
    { 0x73150e46, 3 },
    { 0x512d0e47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f90e56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc10e5a, 3 },
    { 0x92190e5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d90e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37050e62, 3 },
    { 0, 0 },
    { 0x27f10e64, 3 },
    { 0, 0 },
    { 0x7afd0e66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6750e69, 4 },
    { 0xd6c10e6a, 3 },
    { 0xe3b90e6b, 3 },
    { 0x1f610e6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15790e6f, 3 },
    { 0xaccd0e6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de10e73, 4 },
    { 0x34b50e73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef10e7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57050e7e, 3 },
    { 0xfc5d0e7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60850e82, 3 },
    { 0x35390e82, 3 },
    { 0, 0 },
    { 0xd0610e85, 3 },
    { 0x59fd0e85, 3 },
    { 0x7a050e87, 3 },
    { 0x2ec10e85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd0e8d, 3 },
    { 0x4fa90e8e, 3 },
    { 0x22a50e8e, 3 },
    { 0x7f710e90, 3 },
    { 0, 0 },
    { 0x14510e92, 3 },
    { 0x96f50e92, 3 },
    { 0x39ed0e94, 3 },
    { 0xb6fd0e95, 3 },
    { 0xe4910e94, 3 },
    { 0xf9d50e92, 3 },
    { 0xa0d10e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c290e9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec10ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95990ea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d0eab, 4 },
    { 0xcf550eac, 3 },
    { 0, 0 },
    { 0x43110eae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d10eb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90d0eb6, 3 },
    { 0xaefd0eb7, 3 },
    { 0xaf710eb6, 3 },
    { 0xef910eb9, 3 },
    { 0x39b90eb8, 3 },
    { 0x70a90eb6, 3 },
    { 0, 0 },
    { 0xe09d0ebd, 3 },
    { 0xd89d0ebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d90ec7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d90eca, 5 },
    { 0x18250ecb, 3 },
    { 0xa3350ecc, 3 },
    { 0, 0 },
    { 0xef790ece, 4 },
    { 0, 0 },
    { 0xc290ed0, 4 },
    { 0xac890ed0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a50ed5, 3 },
    { 0x1ae10ed5, 4 },
    { 0, 0 },
    { 0xc7150ed8, 4 },
    { 0xc7290ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d50ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb50ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3d0ee9, 4 },
    { 0, 0 },
    { 0x94610eeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f50ef3, 4 },
    { 0xfc250ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f190f00, 3 },
    { 0, 0 },
    { 0x1a010f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed50f07, 3 },
    { 0xe0810f08, 3 },
    { 0xdef50f09, 3 },
    { 0xbe310f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac10f11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d0f15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57850f19, 3 },
    { 0xd3a90f19, 3 },
    { 0x52b50f1b, 3 },
    { 0xdb910f1b, 3 },
    { 0xcbf90f1a, 3 },
    { 0xb7e10f1c, 3 },
    { 0x1c450f1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33850f25, 3 },
    { 0xfcc50f26, 3 },
    { 0xfd110f25, 3 },
    { 0xbbd0f25, 3 },
    { 0x21150f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96210f2d, 3 },
    { 0x107d0f2d, 3 },
    { 0, 0 },
    { 0x44d90f30, 3 },
    { 0, 0 },
    { 0x3f5d0f32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a710f36, 3 },
    { 0, 0 },
    { 0x6a690f38, 3 },
    { 0xe9650f39, 3 },
    { 0xfa010f38, 4 },
    { 0x8e5d0f38, 3 },
    { 0x117d0f3c, 3 },
    { 0xfd950f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc90f40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x342d0f43, 3 },
    { 0x37750f44, 4 },
    { 0x4c10f45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c90f49, 3 },
    { 0, 0 },
    { 0x37750f4b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde790f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e10f54, 3 },
    { 0xbd490f55, 3 },
    { 0, 0 },
    { 0xe4650f57, 3 },
    { 0, 0 },
    { 0x1fc50f59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d10f63, 3 },
    { 0x31390f64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e10f69, 4 },
    { 0x903d0f6a, 3 },
    { 0x8e490f6a, 3 },
    { 0x95fd0f6b, 3 },
    { 0x27310f6d, 4 },
    { 0x269d0f6e, 3 },
    { 0x17f90f6f, 3 },
    { 0x32590f70, 5 },
    { 0xd4bd0f6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb690f76, 3 },
    { 0xf99d0f77, 3 },
    { 0xe9e90f78, 4 },
    { 0x18190f78, 3 },
    { 0x5b210f7a, 5 },
    { 0x3a550f7a, 4 },
    { 0x9a8d0f7c, 3 },
    { 0x3ced0f79, 3 },
    { 0x8390f7e, 4 },
    { 0xcda90f76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b50f83, 4 },
    { 0x71a50f83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a090f87, 3 },
    { 0x1f4d0f88, 4 },
    { 0, 0 },
    { 0xb6350f8a, 3 },
    { 0x43f50f8b, 4 },
    { 0x6b110f8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18350f90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35850f97, 4 },
    { 0xde710f97, 3 },
    { 0x6a310f99, 3 },
    { 0, 0 },
    { 0xdb390f9b, 3 },
    { 0x25290f9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc090fa1, 3 },
    { 0x56090fa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd910fa5, 3 },
    { 0, 0 },
    { 0xfc50fa7, 3 },
    { 0, 0 },
    { 0x3e190fa9, 3 },
    { 0xacd90fa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d0fb4, 3 },
    { 0x32510fb5, 3 },
    { 0xf48d0fb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbd0fbb, 4 },
    { 0xde50fbc, 3 },
    { 0xfa910fbd, 3 },
    { 0xea710fbe, 4 },
    { 0xc7f50fbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5810fc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2850fca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0d0fd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe210fd4, 3 },
    { 0xc1f90fd5, 3 },
    { 0, 0 },
    { 0xb3150fd7, 3 },
    { 0xb5090fd7, 3 },
    { 0xa7010fd7, 3 },
    { 0xad710fda, 5 },
    { 0, 0 },
    { 0x62450fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80710fdf, 3 },
    { 0, 0 },
    { 0x3da90fe1, 3 },
    { 0xbfb10fe1, 3 },
    { 0x32190fe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c910fe6, 3 },
    { 0, 0 },
    { 0x362d0fe8, 3 },
    { 0, 0 },
    { 0xa0d10fea, 3 },
    { 0, 0 },
    { 0x33b50fec, 3 },
    { 0, 0 },
    { 0x7dd10fee, 3 },
    { 0xb4a90fef, 3 },
    { 0, 0 },
    { 0x39890ff1, 3 },
    { 0xf1a50ff1, 4 },
    { 0x33710ff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3110ff6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d0ffc, 4 },
    { 0, 0 },
    { 0xb53d0ffe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5d1006, 3 },
    { 0xd91d1007, 5 },
    { 0x99b91008, 4 },
    { 0, 0 },
    { 0x33a5100a, 3 },
    { 0xe15d100b, 3 },
    { 0x5091100b, 3 },
    { 0xf205100d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891010, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80091017, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d101a, 3 },
    { 0, 0 },
    { 0xcaf9101c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5101f, 3 },
    { 0x2d491020, 4 },
    { 0, 0 },
    { 0x80811022, 3 },
    { 0x49d1022, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2691028, 3 },
    { 0, 0 },
    { 0xaa9d102a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dd102e, 4 },
    { 0x274d102f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45103a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd209103d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20191041, 4 },
    { 0xc4611042, 3 },
    { 0x9dd1043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d51048, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf039104d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91056, 3 },
    { 0xcb0d1056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5585105b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5851060, 4 },
    { 0xcc311060, 3 },
    { 0xed0d1062, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d11066, 3 },
    { 0x60611066, 3 },
    { 0x6a1d1066, 4 },
    { 0xfc391068, 3 },
    { 0x1721106a, 3 },
    { 0x480d1069, 3 },
    { 0x1d106c, 3 },
    { 0x48e9106c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe51070, 3 },
    { 0xbee11071, 3 },
    { 0xf32d1070, 3 },
    { 0x6b311073, 4 },
    { 0, 0 },
    { 0x417d1075, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd249107e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3011081, 3 },
    { 0, 0 },
    { 0xd5c91083, 4 },
    { 0xc2151083, 3 },
    { 0x5ef91083, 3 },
    { 0x8dd91083, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375108b, 3 },
    { 0, 0 },
    { 0xf0f9108d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e11091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e991095, 3 },
    { 0, 0 },
    { 0xd0011097, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ad109b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9805109e, 3 },
    { 0x5051109e, 3 },
    { 0x1449109f, 3 },
    { 0x63b110a1, 3 },
    { 0xcb7110a2, 3 },
    { 0xd85d10a1, 3 },
    { 0xafe110a0, 3 },
    { 0, 0 },
    { 0x37fd10a6, 3 },
    { 0x98c510a6, 3 },
    { 0x97f510a8, 3 },
    { 0xa2b510a9, 3 },
    { 0x419d10a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98510ae, 3 },
    { 0, 0 },
    { 0x946510b0, 4 },
    { 0, 0 },
    { 0xc39110b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0910b5, 3 },
    { 0x96c910b5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3110b9, 3 },
    { 0xfcf110ba, 4 },
    { 0xb68110ba, 4 },
    { 0x835510ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x339d10bf, 3 },
    { 0x2b6910bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b910c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489d10c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325d10cd, 3 },
    { 0x618910ce, 4 },
    { 0x993110cf, 3 },
    { 0x5ced10d0, 3 },
    { 0x9b2910d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d110d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5d10d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811110dd, 3 },
    { 0, 0 },
    { 0xd60d10df, 3 },
    { 0, 0 },
    { 0x4d2510e1, 3 },
    { 0x4de510e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x267510e5, 3 },
    { 0xbbad10e6, 3 },
    { 0x8fcd10e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34110eb, 3 },
    { 0x191510ec, 3 },
    { 0x102910ed, 4 },
    { 0x26f110ee, 3 },
    { 0x387d10ef, 3 },
    { 0x874d10f0, 3 },
    { 0x8b9910f1, 3 },
    { 0x534910f2, 4 },
    { 0x1ae110f3, 3 },
    { 0x26e110f4, 3 },
    { 0xfe7910f3, 3 },
    { 0x6f6510f5, 3 },
    { 0xe9a910f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76510fb, 4 },
    { 0xe81510fc, 3 },
    { 0x28d910fd, 3 },
    { 0xd83d10fe, 3 },
    { 0xc51d10fd, 3 },
    { 0xaf1510fb, 3 },
    { 0x88891101, 3 },
    { 0xd3c11101, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb951105, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb395110b, 3 },
    { 0, 0 },
    { 0x149d110d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dd1114, 4 },
    { 0, 0 },
    { 0x1a151116, 3 },
    { 0x2bcd1116, 3 },
    { 0x3a451118, 3 },
    { 0x430d1119, 3 },
    { 0xfee9111a, 3 },
    { 0x633d1119, 4 },
    { 0xb599111b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d1122, 4 },
    { 0x7e551123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77851127, 3 },
    { 0x3a991128, 3 },
    { 0, 0 },
    { 0x2bed112a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf1112d, 3 },
    { 0x25a1112d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c511134, 4 },
    { 0x1fd1135, 3 },
    { 0x8ab51136, 3 },
    { 0, 0 },
    { 0x4e691138, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf375113e, 4 },
    { 0x9561113e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f11145, 3 },
    { 0xfd691146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35114d, 3 },
    { 0, 0 },
    { 0xf3d9114f, 3 },
    { 0xcf99114f, 3 },
    { 0x63511151, 4 },
    { 0x28011150, 4 },
    { 0, 0 },
    { 0xff251154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9791157, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b1115a, 3 },
    { 0x4e0d115b, 3 },
    { 0x4c91115c, 3 },
    { 0, 0 },
    { 0x7a91115e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a11162, 3 },
    { 0xdb291163, 3 },
    { 0, 0 },
    { 0xfacd1165, 3 },
    { 0x88a91166, 4 },
    { 0, 0 },
    { 0x69e11168, 4 },
    { 0x5d8d1169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe929116c, 3 },
    { 0x8bb1116c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef111173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84511176, 3 },
    { 0, 0 },
    { 0x4f111178, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c19117b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b1117e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d1183, 4 },
    { 0xd6991184, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffd118d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3151194, 4 },
    { 0, 0 },
    { 0xb1091196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630111a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4911aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x480111ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f111bc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8d11bf, 3 },
    { 0x78d511c0, 3 },
    { 0, 0 },
    { 0x4c2911c2, 3 },
    { 0x68d511c2, 3 },
    { 0xb71511c4, 3 },
    { 0xb1d11c5, 3 },
    { 0, 0 },
    { 0x750111c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d11ca, 3 },
    { 0xe38111cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf911cf, 4 },
    { 0xb63911d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aed11d5, 4 },
    { 0, 0 },
    { 0x91d111d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c111da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd111e0, 4 },
    { 0, 0 },
    { 0xb7911e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9111e5, 4 },
    { 0xa4f911e5, 3 },
    { 0xa32d11e7, 3 },
    { 0x86a511e5, 4 },
    { 0x10f911e9, 3 },
    { 0x7b2111e8, 3 },
    { 0xc59511eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7111ee, 3 },
    { 0x51ed11ef, 3 },
    { 0x47c511ef, 3 },
    { 0x76a511f1, 3 },
    { 0xd42911ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961111fb, 4 },
    { 0x797d11fb, 5 },
    { 0x48d511fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c11200, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef711206, 3 },
    { 0, 0 },
    { 0xe9151208, 3 },
    { 0xc4f11209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0d120c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26191210, 4 },
    { 0, 0 },
    { 0xca011212, 4 },
    { 0x6a891213, 4 },
    { 0xe2e11214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f991217, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0d122a, 5 },
    { 0x6255122b, 4 },
    { 0x4b5122c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac11230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e51235, 3 },
    { 0x2f591235, 4 },
    { 0xb351235, 3 },
    { 0x37211238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d9123c, 3 },
    { 0xfe5d123d, 4 },
    { 0xa371123e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da91243, 3 },
    { 0, 0 },
    { 0xfb151245, 4 },
    { 0x7ac51246, 3 },
    { 0x82c11246, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313d124f, 4 },
    { 0x59d11250, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e5125a, 4 },
    { 0xfeb1125a, 3 },
    { 0xc12d125c, 3 },
    { 0x44b9125d, 3 },
    { 0x1c95125c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51265, 4 },
    { 0x4c0d1266, 5 },
    { 0x70251267, 3 },
    { 0xab7d1268, 3 },
    { 0, 0 },
    { 0xe225126a, 3 },
    { 0xe2ed126b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc31126e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b11271, 3 },
    { 0x399d1271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa991276, 3 },
    { 0, 0 },
    { 0x11111278, 5 },
    { 0xb8e91279, 3 },
    { 0xdaad127a, 3 },
    { 0x77cd127a, 3 },
    { 0x88e1127c, 4 },
    { 0xf375127d, 4 },
    { 0, 0 },
    { 0xeef1127f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf611282, 3 },
    { 0, 0 },
    { 0x13651284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b1128e, 3 },
    { 0x6175128f, 3 },
    { 0xea951290, 3 },
    { 0x9c11128e, 3 },
    { 0xe7a1128e, 3 },
    { 0xacb51293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34551297, 3 },
    { 0x4a191298, 5 },
    { 0xc87d1298, 4 },
    { 0xe2151298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c9129e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d512a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b912a5, 3 },
    { 0x25f912a6, 3 },
    { 0xb75512a7, 3 },
    { 0xce4912a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b112b4, 3 },
    { 0, 0 },
    { 0x8e1512b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b112b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16d12bd, 3 },
    { 0, 0 },
    { 0xc64d12bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5912c2, 3 },
    { 0x18ad12c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4912c9, 4 },
    { 0x288512ca, 3 },
    { 0x43b912ca, 4 },
    { 0x92a912cc, 3 },
    { 0xbecd12cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9112d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43d12d8, 3 },
    { 0, 0 },
    { 0x78c912da, 3 },
    { 0, 0 },
    { 0xb2d112dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c512e0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371112e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85112ed, 3 },
    { 0xaef112ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e112f2, 5 },
    { 0x188d12f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307d12f7, 3 },
    { 0xfa7912f8, 4 },
    { 0, 0 },
    { 0x155512fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c512fe, 4 },
    { 0, 0 },
    { 0x1c6d1300, 4 },
    { 0, 0 },
    { 0x2f011302, 4 },
    { 0, 0 },
    { 0xe4391304, 3 },
    { 0, 0 },
    { 0x9ef91306, 4 },
    { 0x79091306, 3 },
    { 0, 0 },
    { 0xfc611309, 3 },
    { 0x9c591309, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fd130d, 3 },
    { 0x8189130e, 4 },
    { 0xf541130f, 3 },
    { 0, 0 },
    { 0x42d1311, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d1314, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dd1318, 3 },
    { 0, 0 },
    { 0x8699131a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d1323, 4 },
    { 0xcfd1324, 4 },
    { 0xe25d1324, 3 },
    { 0xa791325, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf839132d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d11333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42211337, 3 },
    { 0, 0 },
    { 0xd3291339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf211133c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3391340, 4 },
    { 0x48a51341, 4 },
    { 0x15811342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60d1348, 3 },
    { 0x4b591349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fd134c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1665134f, 5 },
    { 0, 0 },
    { 0xd8dd1351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d11358, 3 },
    { 0, 0 },
    { 0x3969135a, 3 },
    { 0xe625135a, 4 },
    { 0x8085135c, 4 },
    { 0, 0 },
    { 0x9ae5135e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4711362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd951136b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20891370, 3 },
    { 0x9891371, 3 },
    { 0xd7f51371, 3 },
    { 0xdf951373, 3 },
    { 0xdcb11373, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ed137b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c01137f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49138c, 3 },
    { 0, 0 },
    { 0x3db5138e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c11393, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdd139a, 3 },
    { 0xad59139b, 3 },
    { 0x3f1139b, 3 },
    { 0x135d139c, 5 },
    { 0xc4cd139c, 3 },
    { 0x28cd139f, 3 },
    { 0xc0fd139c, 3 },
    { 0xa125139b, 3 },
    { 0, 0 },
    { 0x570913a3, 3 },
    { 0, 0 },
    { 0x249913a5, 4 },
    { 0, 0 },
    { 0x714d13a7, 3 },
    { 0, 0 },
    { 0xe27513a9, 3 },
    { 0x170113a9, 3 },
    { 0x4d513aa, 4 },
    { 0, 0 },
    { 0x658113ad, 3 },
    { 0x34f113ae, 3 },
    { 0x441113ad, 3 },
    { 0xbc9513b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1913b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0113ba, 5 },
    { 0xefd913bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1513c0, 3 },
    { 0x881513c0, 4 },
    { 0, 0 },
    { 0x239913c3, 3 },
    { 0, 0 },
    { 0xbec913c5, 3 },
    { 0xfd7913c6, 3 },
    { 0x90d113c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8d13ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x402913cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d13d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5d13dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5513df, 4 },
    { 0xdca513e0, 4 },
    { 0x1fc113e1, 3 },
    { 0xdc7113e1, 4 },
    { 0x430113e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92513e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x33513e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ed13ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f513f4, 3 },
    { 0x558113f4, 3 },
    { 0x2d5113f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0913f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac651400, 3 },
    { 0, 0 },
    { 0xb0391402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd1406, 3 },
    { 0xdcc51407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474d140e, 3 },
    { 0x69b5140e, 3 },
    { 0, 0 },
    { 0x40b11411, 3 },
    { 0, 0 },
    { 0xa2491413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e691416, 4 },
    { 0, 0 },
    { 0x8a851418, 3 },
    { 0x426d1419, 3 },
    { 0, 0 },
    { 0xc701141b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fd1420, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a11424, 3 },
    { 0, 0 },
    { 0xc02d1426, 3 },
    { 0, 0 },
    { 0xdb491428, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5142d, 3 },
    { 0x2bb5142e, 3 },
    { 0, 0 },
    { 0x58111430, 4 },
    { 0x8fd11431, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x791434, 3 },
    { 0xec91435, 4 },
    { 0xe711434, 3 },
    { 0x109d1435, 3 },
    { 0xdb2d1436, 4 },
    { 0x374d1434, 3 },
    { 0, 0 },
    { 0x8391143b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd143f, 3 },
    { 0x2f0d143f, 3 },
    { 0xc005143f, 3 },
    { 0x1c711442, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37191447, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x785d144a, 3 },
    { 0xdbf1144b, 4 },
    { 0xe761144c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3051454, 4 },
    { 0xd1e51455, 3 },
    { 0, 0 },
    { 0xeb891457, 3 },
    { 0xe12d1457, 3 },
    { 0, 0 },
    { 0xa9c1145a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e29145d, 3 },
    { 0x3b51145d, 3 },
    { 0x8785145f, 3 },
    { 0xc06d1460, 3 },
    { 0x299d145f, 3 },
    { 0x2a111460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e91465, 4 },
    { 0xfca51465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c5146c, 3 },
    { 0x9985146c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ad1472, 4 },
    { 0, 0 },
    { 0xf5ad1474, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d1478, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6999147f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c51488, 4 },
    { 0x647d1489, 4 },
    { 0xc2411488, 4 },
    { 0x10f5148b, 3 },
    { 0xd2bd148c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3849148f, 3 },
    { 0, 0 },
    { 0x4bb91491, 3 },
    { 0xe2411491, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab51495, 4 },
    { 0x81211496, 3 },
    { 0x49451496, 4 },
    { 0x151d1496, 4 },
    { 0x4a691499, 3 },
    { 0xe2651499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf1149d, 4 },
    { 0x738d149e, 4 },
    { 0x21c5149d, 3 },
    { 0xff39149f, 3 },
    { 0x844914a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615114aa, 3 },
    { 0x71d914ab, 3 },
    { 0xdd6d14ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112514b3, 3 },
    { 0x220914b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da914b7, 4 },
    { 0xc8d114b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x273914bb, 3 },
    { 0x8f6d14bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x431514bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53114c2, 5 },
    { 0, 0 },
    { 0xe95114c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619914cc, 4 },
    { 0x4c9114cd, 3 },
    { 0, 0 },
    { 0x52bd14cf, 3 },
    { 0xc9d914cf, 3 },
    { 0x759d14cf, 5 },
    { 0x5d8d14d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4114d5, 3 },
    { 0x52e514d6, 3 },
    { 0xc9c514d6, 4 },
    { 0xdb8114d8, 3 },
    { 0xed9914d7, 5 },
    { 0x77a514da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9914e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89d14ef, 3 },
    { 0xa0dd14f0, 3 },
    { 0x1b2114f1, 3 },
    { 0x6f5114f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ead14f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ead14fb, 3 },
    { 0xc1b114fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d1500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f651507, 3 },
    { 0xabd1508, 4 },
    { 0x98491507, 4 },
    { 0xb9d1508, 3 },
    { 0xb3d150a, 4 },
    { 0x5651509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e11510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e51518, 3 },
    { 0xdba91519, 4 },
    { 0xca25151a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab91151d, 3 },
    { 0x6445151d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d251521, 3 },
    { 0x8f91521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x930d1525, 4 },
    { 0x56991525, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb91529, 3 },
    { 0x6c79152a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xee15152d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x712d1530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d1533, 3 },
    { 0x18111534, 3 },
    { 0xc59d1534, 3 },
    { 0xacb11536, 4 },
    { 0, 0 },
    { 0x8c511538, 3 },
    { 0, 0 },
    { 0xd799153a, 3 },
    { 0, 0 },
    { 0x559d153c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0d153f, 3 },
    { 0xdaa1153f, 3 },
    { 0, 0 },
    { 0xb64d1542, 3 },
    { 0, 0 },
    { 0xddd1544, 3 },
    { 0xb6011545, 3 },
    { 0x99711545, 4 },
    { 0xa58d1546, 4 },
    { 0, 0 },
    { 0x881d1549, 3 },
    { 0, 0 },
    { 0x1759154b, 4 },
    { 0x71c1154c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e11553, 4 },
    { 0x8f811553, 3 },
    { 0, 0 },
    { 0x62d11556, 4 },
    { 0x99991557, 3 },
    { 0xfec91558, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc845155c, 3 },
    { 0x2a65155d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd751563, 3 },
    { 0xefe91563, 3 },
    { 0x5f711565, 3 },
    { 0xa8b91565, 3 },
    { 0xaf0d1565, 3 },
    { 0x18351563, 3 },
    { 0, 0 },
    { 0xe4d156a, 4 },
    { 0xd61d156a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6331156e, 3 },
    { 0, 0 },
    { 0x31651570, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x586d1573, 3 },
    { 0, 0 },
    { 0xecad1575, 4 },
    { 0xb1d51575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f91579, 4 },
    { 0, 0 },
    { 0xe2a5157b, 3 },
    { 0, 0 },
    { 0x3629157d, 5 },
    { 0xb075157e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18191585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9158a, 3 },
    { 0, 0 },
    { 0xd749158c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4951158f, 4 },
    { 0x74dd158f, 3 },
    { 0xfa691591, 5 },
    { 0x3cad1592, 4 },
    { 0x8a491593, 4 },
    { 0xd9391593, 3 },
    { 0xf6d1590, 3 },
    { 0, 0 },
    { 0x46191597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab19159b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1159e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a515a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972515a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5515ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x271115af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353d15bc, 4 },
    { 0x1c5515bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26915c2, 3 },
    { 0xd22515c2, 3 },
    { 0x80c915c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7115c7, 3 },
    { 0x4d9115c8, 3 },
    { 0x718d15c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe115cc, 4 },
    { 0, 0 },
    { 0xac4515ce, 3 },
    { 0x1b8d15ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cd15d4, 4 },
    { 0xa69515d4, 4 },
    { 0x87dd15d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15515da, 4 },
    { 0x444d15db, 3 },
    { 0, 0 },
    { 0x221915dd, 3 },
    { 0x25a515dd, 3 },
    { 0x6f115de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39915e2, 3 },
    { 0xd0e515e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588115ef, 4 },
    { 0, 0 },
    { 0x33dd15f1, 3 },
    { 0x149d15f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d15f8, 3 },
    { 0, 0 },
    { 0x427515fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba915ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7291604, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e51607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6009160d, 3 },
    { 0, 0 },
    { 0xb801160f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f11612, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91d1615, 3 },
    { 0, 0 },
    { 0x3f611617, 3 },
    { 0x553d1618, 4 },
    { 0x36a11619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee9161d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f51622, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d162c, 3 },
    { 0xaa0d162d, 4 },
    { 0x3d4d162c, 3 },
    { 0, 0 },
    { 0x37b51630, 4 },
    { 0x93011631, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1163b, 4 },
    { 0x4181163b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e1163f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36d1642, 3 },
    { 0x625d1643, 3 },
    { 0x1cbd1644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226d1648, 3 },
    { 0, 0 },
    { 0xc1cd164a, 3 },
    { 0, 0 },
    { 0x164d164c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a91651, 3 },
    { 0xe27d1651, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb051655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44651658, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa09165b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791664, 3 },
    { 0x1a1d1664, 3 },
    { 0x20051666, 4 },
    { 0x172d1666, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5d1670, 3 },
    { 0, 0 },
    { 0x4f911672, 4 },
    { 0x98c51673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f691679, 3 },
    { 0xad35167a, 3 },
    { 0, 0 },
    { 0x5f2d167c, 3 },
    { 0, 0 },
    { 0xc289167e, 3 },
    { 0x3ad167f, 3 },
    { 0xe1111680, 3 },
    { 0xdb891681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c91685, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad1688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a79168b, 4 },
    { 0xa035168c, 3 },
    { 0x45e1168c, 3 },
    { 0, 0 },
    { 0xb799168f, 3 },
    { 0x86c9168f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8211693, 4 },
    { 0xfa691694, 3 },
    { 0xd8ed1695, 3 },
    { 0, 0 },
    { 0x5f251697, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452916a2, 4 },
    { 0x29c516a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479116aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1116ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52116b5, 4 },
    { 0xec6d16b5, 3 },
    { 0xb97d16b7, 3 },
    { 0, 0 },
    { 0xa4f916b9, 3 },
    { 0xc0ad16ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47116c2, 5 },
    { 0x4c4d16c2, 4 },
    { 0xec9516c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa116ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767516d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01916d7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609916dd, 4 },
    { 0xc02516de, 4 },
    { 0, 0 },
    { 0xdd7516e0, 3 },
    { 0, 0 },
    { 0x502516e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181916e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6116e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c516f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913516fa, 3 },
    { 0x735916fb, 3 },
    { 0xd48d16fb, 4 },
    { 0, 0 },
    { 0x646516fe, 4 },
    { 0xcaa116fe, 3 },
    { 0, 0 },
    { 0x56591701, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cd1705, 4 },
    { 0x2b291705, 4 },
    { 0, 0 },
    { 0xbdc11708, 4 },
    { 0x18a91709, 3 },
    { 0x41311709, 3 },
    { 0xe0911708, 3 },
    { 0xa9991708, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe931170f, 3 },
    { 0, 0 },
    { 0x96bd1711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31d1715, 4 },
    { 0, 0 },
    { 0x3e291717, 3 },
    { 0xf2f91718, 3 },
    { 0x25351718, 4 },
    { 0xb2f91718, 3 },
    { 0xf729171b, 3 },
    { 0x11b9171c, 3 },
    { 0x964d1717, 3 },
    { 0xbdf1171e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e991722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc91727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf01172b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a39172e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcd1734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4625173c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c211741, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1051745, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c11755, 3 },
    { 0, 0 },
    { 0x2ced1757, 4 },
    { 0xd2351757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4011765, 3 },
    { 0xe8951765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d176c, 4 },
    { 0x2e7d176d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2511770, 3 },
    { 0x47891771, 4 },
    { 0x42cd1770, 3 },
    { 0x7aa11773, 3 },
    { 0x929d1773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe5177c, 3 },
    { 0xf775177d, 3 },
    { 0, 0 },
    { 0xee19177f, 3 },
    { 0, 0 },
    { 0xe6b91781, 3 },
    { 0x35c91782, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3455178d, 4 },
    { 0x84e9178d, 3 },
    { 0xb031178e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf91793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3711179a, 4 },
    { 0x13b5179b, 3 },
    { 0xfb45179c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb61179f, 3 },
    { 0x12e917a0, 4 },
    { 0xddb517a1, 3 },
    { 0x378d179f, 3 },
    { 0, 0 },
    { 0x9d7d17a4, 3 },
    { 0x69b517a5, 3 },
    { 0xdf9d17a5, 3 },
    { 0xc11517a7, 3 },
    { 0, 0 },
    { 0x50cd17a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x613517ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf117b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x686d17b3, 4 },
    { 0x61f117b3, 3 },
    { 0, 0 },
    { 0x582117b6, 4 },
    { 0x913117b7, 3 },
    { 0x39117b8, 3 },
    { 0, 0 },
    { 0x3eb517ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468917be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc517c1, 4 },
    { 0xca7917c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55517c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb517cc, 5 },
    { 0, 0 },
    { 0x15c517ce, 3 },
    { 0, 0 },
    { 0xfaf117d0, 3 },
    { 0, 0 },
    { 0xb5c117d2, 3 },
    { 0xb91117d3, 3 },
    { 0x12bd17d2, 3 },
    { 0x507517d2, 3 },
    { 0x8bcd17d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a517de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecd17e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa517e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dd17eb, 4 },
    { 0x913d17ec, 4 },
    { 0xc9fd17ed, 3 },
    { 0x8f9917ec, 3 },
    { 0x300517ee, 3 },
    { 0xdb6917ef, 3 },
    { 0, 0 },
    { 0x6c5117f2, 4 },
    { 0x629917f2, 4 },
    { 0, 0 },
    { 0xfd8d17f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273d17fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffd1800, 3 },
    { 0xbd811800, 3 },
    { 0x2ad1802, 3 },
    { 0x79811803, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dd1806, 3 },
    { 0xd0911807, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8765180b, 3 },
    { 0x4a41180c, 3 },
    { 0xe521180c, 3 },
    { 0x48c9180e, 3 },
    { 0xbc8d180f, 3 },
    { 0xb81d180e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3391813, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2d1816, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391819, 3 },
    { 0, 0 },
    { 0xdc6d181b, 4 },
    { 0x92b5181c, 3 },
    { 0x8ca9181d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa211820, 3 },
    { 0x72651820, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7791824, 3 },
    { 0xb46d1825, 3 },
    { 0x8a7d1826, 4 },
    { 0x3d9d1827, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa5182e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce91831, 4 },
    { 0x49d11832, 3 },
    { 0x57911833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7d1836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca91843, 4 },
    { 0x7c411844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ced184a, 3 },
    { 0x4101184b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459d1856, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1185b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa601185f, 3 },
    { 0, 0 },
    { 0x91e51861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19451864, 3 },
    { 0xff11864, 3 },
    { 0x112d1866, 3 },
    { 0xcff11867, 4 },
    { 0x918d1868, 5 },
    { 0x2c91868, 3 },
    { 0x2c511865, 3 },
    { 0xe819186b, 3 },
    { 0xf62d186b, 3 },
    { 0, 0 },
    { 0x7115186e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be91871, 3 },
    { 0x61c51871, 3 },
    { 0xeac11872, 4 },
    { 0, 0 },
    { 0xa2611875, 3 },
    { 0x960d1876, 3 },
    { 0x20d1877, 4 },
    { 0x6ec91877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85751880, 4 },
    { 0xa9991881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e51884, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f611889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f5188d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cd1890, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2351898, 3 },
    { 0, 0 },
    { 0x71fd189a, 4 },
    { 0x9b3d189a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f89189e, 3 },
    { 0, 0 },
    { 0xac6d18a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fd18a5, 3 },
    { 0, 0 },
    { 0x94f118a7, 3 },
    { 0, 0 },
    { 0xc9518a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ad18ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5918bb, 3 },
    { 0x8ef918bc, 3 },
    { 0, 0 },
    { 0xceb118be, 3 },
    { 0, 0 },
    { 0xf61918c0, 4 },
    { 0xda0918c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c118c5, 3 },
    { 0x7cf118c5, 3 },
    { 0x8b7518c6, 3 },
    { 0x21518c5, 3 },
    { 0x571518c7, 3 },
    { 0x9b6518c9, 3 },
    { 0x7ffd18ca, 3 },
    { 0, 0 },
    { 0x743d18cd, 4 },
    { 0, 0 },
    { 0x223518cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ed18d2, 3 },
    { 0, 0 },
    { 0x56f918d4, 4 },
    { 0x255918d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20918d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b518db, 4 },
    { 0x50b918dc, 3 },
    { 0x462518dc, 3 },
    { 0x3b9918de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x234118e1, 3 },
    { 0xc58918e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6118e5, 3 },
    { 0, 0 },
    { 0xbb9918e7, 3 },
    { 0xcc6918e8, 4 },
    { 0x8c6518e9, 3 },
    { 0xac6518e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d18f5, 3 },
    { 0, 0 },
    { 0x720d18f7, 3 },
    { 0xa63118f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d518fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed91901, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26291906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9009190b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d451915, 4 },
    { 0, 0 },
    { 0xe6351917, 3 },
    { 0x22a91917, 4 },
    { 0, 0 },
    { 0x4ec5191a, 4 },
    { 0x89d1191b, 4 },
    { 0x9add191c, 3 },
    { 0xa195191d, 3 },
    { 0x7a5191e, 3 },
    { 0x5561191f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e51927, 3 },
    { 0xa0251928, 3 },
    { 0, 0 },
    { 0xa315192a, 3 },
    { 0x7745192b, 3 },
    { 0x9b11192c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd521192f, 3 },
    { 0x5b5d192f, 3 },
    { 0x4fcd1930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d193e, 3 },
    { 0xda05193f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0391942, 4 },
    { 0xd0811942, 3 },
    { 0xd0511942, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5311948, 3 },
    { 0x54b91949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519d194e, 3 },
    { 0, 0 },
    { 0x214d1950, 4 },
    { 0, 0 },
    { 0x92c91952, 4 },
    { 0xe2791953, 3 },
    { 0x71411953, 3 },
    { 0, 0 },
    { 0xb6391956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5195a, 3 },
    { 0x5a61195a, 3 },
    { 0, 0 },
    { 0x83f5195d, 3 },
    { 0, 0 },
    { 0xffa1195f, 4 },
    { 0xa4d11960, 3 },
    { 0xa8d1961, 3 },
    { 0x5b091962, 4 },
    { 0, 0 },
    { 0x20811964, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7851968, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd211196b, 4 },
    { 0x2369196c, 4 },
    { 0x2255196d, 5 },
    { 0, 0 },
    { 0xa085196f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f951972, 3 },
    { 0x591d1973, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949d1978, 3 },
    { 0, 0 },
    { 0x35c5197a, 3 },
    { 0, 0 },
    { 0xf905197c, 3 },
    { 0xfc39197c, 3 },
    { 0x63e5197c, 4 },
    { 0x3a59197d, 3 },
    { 0x86d1197d, 3 },
    { 0x27d11980, 3 },
    { 0, 0 },
    { 0x6d711983, 3 },
    { 0, 0 },
    { 0xaee91985, 3 },
    { 0xb1e91986, 3 },
    { 0x1f8d1986, 3 },
    { 0x51e91986, 3 },
    { 0x95791989, 4 },
    { 0xb4a11989, 4 },
    { 0x7d8d198b, 3 },
    { 0xbfb51986, 3 },
    { 0xd991198d, 3 },
    { 0x973d1987, 3 },
    { 0x55fd198f, 3 },
    { 0x6db11990, 3 },
    { 0x73bd1991, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d91994, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92d1997, 4 },
    { 0xd07d1998, 3 },
    { 0xbd1998, 3 },
    { 0x664d1997, 3 },
    { 0, 0 },
    { 0xa0b1199c, 4 },
    { 0, 0 },
    { 0xb53d199e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517919a3, 3 },
    { 0x690519a3, 3 },
    { 0, 0 },
    { 0x4a8119a6, 5 },
    { 0xf66119a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7d19b1, 3 },
    { 0xbf8519b1, 3 },
    { 0, 0 },
    { 0xb17119b4, 3 },
    { 0x696919b5, 3 },
    { 0x964519b4, 3 },
    { 0xbc1119b7, 4 },
    { 0xdd1119b6, 3 },
    { 0x33c119b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e519c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9519cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38519d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9919d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c519e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15119e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20119ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819d19ee, 3 },
    { 0xe8d519ef, 3 },
    { 0, 0 },
    { 0xf87919f1, 4 },
    { 0x5a519f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41d19f8, 3 },
    { 0, 0 },
    { 0x644119fa, 3 },
    { 0xb96919fa, 3 },
    { 0x2fa519fc, 3 },
    { 0x2d9919fb, 3 },
    { 0xbead19fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead1a02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b51a06, 3 },
    { 0x48151a07, 3 },
    { 0xf36d1a08, 3 },
    { 0, 0 },
    { 0xf7c91a0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d11a0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52891a10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e651a13, 3 },
    { 0x6a351a14, 3 },
    { 0, 0 },
    { 0x10d51a16, 3 },
    { 0, 0 },
    { 0x6ae11a18, 3 },
    { 0x8fd1a19, 5 },
    { 0xc4b11a18, 3 },
    { 0x14fd1a1b, 5 },
    { 0x1e0d1a1c, 5 },
    { 0, 0 },
    { 0x36a91a1e, 3 },
    { 0x76451a1e, 4 },
    { 0, 0 },
    { 0xada11a21, 3 },
    { 0xe09d1a22, 3 },
    { 0, 0 },
    { 0xf5d1a24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb11a27, 3 },
    { 0, 0 },
    { 0xf6711a29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c91a2f, 3 },
    { 0, 0 },
    { 0x3ae51a31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4d1a38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91011a3b, 4 },
    { 0, 0 },
    { 0x61a91a3d, 3 },
    { 0, 0 },
    { 0xed091a3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e851a42, 4 },
    { 0, 0 },
    { 0xea611a44, 3 },
    { 0x56f91a45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76651a52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d11a56, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2651a59, 3 },
    { 0x53d1a5a, 3 },
    { 0x7e91a5b, 3 },
    { 0xf7591a5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949d1a60, 3 },
    { 0x2fe11a61, 3 },
    { 0, 0 },
    { 0x58a51a63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7311a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd251a79, 3 },
    { 0, 0 },
    { 0xaf191a7b, 3 },
    { 0, 0 },
    { 0x69351a7d, 3 },
    { 0x7e4d1a7d, 3 },
    { 0x1e4d1a7f, 3 },
    { 0x42c11a7e, 5 },
    { 0x9de51a7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde111a8f, 4 },
    { 0xaf051a90, 3 },
    { 0xc4dd1a91, 3 },
    { 0x75611a90, 3 },
    { 0, 0 },
    { 0xe8fd1a94, 4 },
    { 0xedd51a95, 3 },
    { 0xe2191a96, 4 },
    { 0x8ab11a96, 4 },
    { 0xcf391a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2251aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e611aa5, 3 },
    { 0x39e91aa6, 3 },
    { 0x23791aa6, 5 },
    { 0x57d11aa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14551aad, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02d1ab0, 3 },
    { 0, 0 },
    { 0x44cd1ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd91ab5, 4 },
    { 0xed11ab6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92d1aba, 5 },
    { 0, 0 },
    { 0x4ac91abc, 3 },
    { 0x68111abd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8291ac0, 3 },
    { 0, 0 },
    { 0x53a51ac2, 3 },
    { 0, 0 },
    { 0xd5051ac4, 3 },
    { 0x5ab11ac5, 3 },
    { 0, 0 },
    { 0x34151ac7, 3 },
    { 0x27ed1ac8, 3 },
    { 0, 0 },
    { 0xb5791aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d1ace, 4 },
    { 0xfb091ace, 3 },
    { 0xffe11ad0, 3 },
    { 0, 0 },
    { 0xf01d1ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15611ad8, 3 },
    { 0x6e251ad9, 3 },
    { 0x12511ad9, 3 },
    { 0xb69d1adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bd1ae3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc51ae9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d51aec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d1aef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16791af9, 3 },
    { 0x9abd1afa, 3 },
    { 0x4b2d1afb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9911b02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990d1b09, 3 },
    { 0x2a7d1b0a, 4 },
    { 0x6dd11b0b, 3 },
    { 0, 0 },
    { 0x92891b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7751b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d1b1a, 4 },
    { 0xe2391b1b, 3 },
    { 0, 0 },
    { 0x957d1b1d, 3 },
    { 0x7d611b1d, 3 },
    { 0, 0 },
    { 0x18851b20, 4 },
    { 0xabe51b21, 3 },
    { 0xa1191b22, 3 },
    { 0xc9d1b22, 3 },
    { 0xab11b22, 3 },
    { 0xdddd1b25, 5 },
    { 0xccd51b21, 4 },
    { 0, 0 },
    { 0xea7d1b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d51b2b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a11b32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd51b38, 3 },
    { 0x60291b38, 4 },
    { 0x95c51b3a, 3 },
    { 0x20651b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46251b3e, 4 },
    { 0xa2e11b3f, 3 },
    { 0xeec51b40, 3 },
    { 0x50011b41, 3 },
    { 0, 0 },
    { 0xb1251b43, 3 },
    { 0, 0 },
    { 0xb4391b45, 3 },
    { 0x24411b46, 4 },
    { 0x552d1b47, 3 },
    { 0xeccd1b48, 3 },
    { 0xc4f51b48, 3 },
    { 0xa5b91b49, 3 },
    { 0x1f091b46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fd1b57, 3 },
    { 0x75291b58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53251b61, 3 },
    { 0x7d311b62, 3 },
    { 0xaaa51b63, 3 },
    { 0x661d1b62, 4 },
    { 0x45e11b65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc11b6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb51b6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde11b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d11b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e651b7c, 3 },
    { 0, 0 },
    { 0x7a611b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3551b82, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf291b85, 3 },
    { 0x8dc51b85, 4 },
    { 0, 0 },
    { 0xb32d1b88, 4 },
    { 0x64811b88, 3 },
    { 0, 0 },
    { 0x4ac11b8b, 3 },
    { 0xf491b8c, 3 },
    { 0, 0 },
    { 0x4f511b8e, 4 },
    { 0, 0 },
    { 0x9fed1b90, 3 },
    { 0x5fbd1b90, 4 },
    { 0, 0 },
    { 0x9251b93, 3 },
    { 0x11d11b94, 4 },
    { 0xa8511b95, 3 },
    { 0x83691b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad951b99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d1b9c, 4 },
    { 0x6bd91b9c, 4 },
    { 0, 0 },
    { 0xe5291b9f, 3 },
    { 0x4e891ba0, 3 },
    { 0xdb651ba1, 3 },
    { 0, 0 },
    { 0xd711ba3, 3 },
    { 0x7d1ba4, 3 },
    { 0x6d5d1ba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5211ba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa851bac, 3 },
    { 0x1e411bad, 4 },
    { 0, 0 },
    { 0x17e11baf, 3 },
    { 0xc4791bb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d91bb3, 3 },
    { 0xb1451bb4, 3 },
    { 0x80ad1bb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab711bbb, 3 },
    { 0x86191bbc, 3 },
    { 0, 0 },
    { 0x5f6d1bbe, 3 },
    { 0xbbd1bbf, 3 },
    { 0, 0 },
    { 0x8fe91bc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3d1bc5, 3 },
    { 0, 0 },
    { 0x665d1bc7, 4 },
    { 0x84a91bc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3811bcb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72991bd1, 3 },
    { 0x94a91bd2, 3 },
    { 0xdb811bd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c91bd8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12951bdc, 4 },
    { 0x8e91bdd, 3 },
    { 0xfda11bde, 4 },
    { 0xec9d1bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2991be4, 3 },
    { 0x9ac11be5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81691be9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0751bec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc51bef, 4 },
    { 0, 0 },
    { 0x2abd1bf1, 3 },
    { 0x79a51bf1, 3 },
    { 0x94851bf3, 5 },
    { 0, 0 },
    { 0xe0551bf5, 3 },
    { 0xe6d11bf5, 3 },
    { 0x59911bf6, 3 },
    { 0, 0 },
    { 0x551bf9, 3 },
    { 0x984d1bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6611bfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b91c01, 3 },
    { 0xa3c51c02, 3 },
    { 0xc8191c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75251c06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902d1c0b, 3 },
    { 0xe7911c0c, 3 },
    { 0x2e811c0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed191c14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0551c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51c25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb51c28, 3 },
    { 0xb1811c29, 3 },
    { 0, 0 },
    { 0x68c51c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d1c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89d1c37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f51c3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f91c42, 3 },
    { 0xff911c43, 3 },
    { 0x24c91c44, 3 },
    { 0xc7551c42, 3 },
    { 0x441d1c46, 3 },
    { 0x7a911c47, 3 },
    { 0xb43d1c46, 3 },
    { 0xbe091c44, 4 },
    { 0xf451c4a, 4 },
    { 0x79ed1c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3d1c50, 4 },
    { 0, 0 },
    { 0xcdc11c52, 4 },
    { 0x4fe11c53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae291c56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d411c5a, 3 },
    { 0, 0 },
    { 0xa2b91c5c, 4 },
    { 0x75011c5c, 4 },
    { 0xc0511c5e, 4 },
    { 0x46ad1c5e, 4 },
    { 0x9911c5c, 3 },
    { 0x3ba91c61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6611c71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc151c76, 3 },
    { 0xc4891c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb11c7b, 3 },
    { 0x84991c7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x688d1c7f, 3 },
    { 0xaf91c80, 3 },
    { 0xed1d1c80, 3 },
    { 0xdcf51c82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea51c86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb51c89, 4 },
    { 0x23e11c8a, 4 },
    { 0xce491c8a, 3 },
    { 0x91ad1c89, 3 },
    { 0x85191c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70491c95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad1c9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bd1ca6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d1cab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fd1cb0, 3 },
    { 0x73251cb1, 4 },
    { 0x45911cb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca91cb7, 3 },
    { 0x5151cb8, 3 },
    { 0, 0 },
    { 0x5eb11cba, 3 },
    { 0, 0 },
    { 0xe4691cbc, 3 },
    { 0x52ad1cbd, 3 },
    { 0x9d1d1cbe, 4 },
    { 0x10591cbe, 3 },
    { 0xf5591cbe, 3 },
    { 0x67791cbf, 5 },
    { 0x63891cc2, 3 },
    { 0xe6051cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21491cc9, 3 },
    { 0xfc211cc9, 4 },
    { 0x2cb51ccb, 3 },
    { 0, 0 },
    { 0xee711ccd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f11cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278d1cda, 4 },
    { 0, 0 },
    { 0xb4011cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de51ce1, 3 },
    { 0xf9d51ce2, 3 },
    { 0xd9e11ce3, 3 },
    { 0, 0 },
    { 0x58991ce5, 3 },
    { 0, 0 },
    { 0x7f851ce7, 3 },
    { 0, 0 },
    { 0xbe151ce9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9851cec, 3 },
    { 0, 0 },
    { 0xbf9d1cee, 5 },
    { 0x82891cef, 3 },
    { 0x846d1cf0, 3 },
    { 0xd4c91cf0, 3 },
    { 0x54451cf0, 3 },
    { 0x4c591cf3, 3 },
    { 0x253d1cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb091d04, 3 },
    { 0xaf551d05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f51d0d, 3 },
    { 0x10d51d0d, 3 },
    { 0x3bf11d0f, 4 },
    { 0x93b91d0f, 4 },
    { 0, 0 },
    { 0x8b351d12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36691d16, 3 },
    { 0, 0 },
    { 0x17d91d18, 3 },
    { 0xe0a11d19, 4 },
    { 0, 0 },
    { 0xb5e51d1b, 3 },
    { 0, 0 },
    { 0xcded1d1d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34751d21, 3 },
    { 0, 0 },
    { 0xc2bd1d23, 3 },
    { 0xbe651d24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05d1d29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98811d31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4511d34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387d1d38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9111d3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd1d45, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7191d4a, 4 },
    { 0xf5611d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2d1d51, 3 },
    { 0xb9e11d52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa91d58, 3 },
    { 0x1f891d59, 3 },
    { 0x21551d5a, 3 },
    { 0x7a011d5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed1d62, 3 },
    { 0x4b791d62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f11d6b, 4 },
    { 0xc5cd1d6b, 3 },
    { 0x9ea11d6d, 3 },
    { 0xa6a51d6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1d1d73, 3 },
    { 0x759d1d74, 3 },
    { 0x98011d75, 3 },
    { 0x64991d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691d7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6411d7e, 3 },
    { 0x82b51d7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc351d82, 3 },
    { 0x20551d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf191d88, 3 },
    { 0, 0 },
    { 0x76091d8a, 3 },
    { 0, 0 },
    { 0x96751d8c, 3 },
    { 0x6e551d8c, 4 },
    { 0xf1251d8e, 4 },
    { 0, 0 },
    { 0x8c411d90, 4 },
    { 0x195d1d91, 3 },
    { 0, 0 },
    { 0xe7e51d93, 3 },
    { 0x61d11d93, 4 },
    { 0x748d1d93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d1d99, 3 },
    { 0xd5f91d9a, 3 },
    { 0, 0 },
    { 0x82d11d9c, 4 },
    { 0x19291d9c, 3 },
    { 0x39cd1d9e, 3 },
    { 0x15cd1d9e, 3 },
    { 0, 0 },
    { 0xff391da1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7991da6, 3 },
    { 0x6ec91da7, 3 },
    { 0x50151da8, 3 },
    { 0xb7c51da8, 3 },
    { 0xf51d1daa, 3 },
    { 0x44b51dab, 3 },
    { 0xedd91daa, 3 },
    { 0xc8d91da9, 4 },
    { 0x22d11da8, 3 },
    { 0, 0 },
    { 0xceed1db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10511db4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33411db8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d911dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f11dc7, 3 },
    { 0x65511dc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd91dcb, 3 },
    { 0, 0 },
    { 0x228d1dcd, 3 },
    { 0xcec51dce, 3 },
    { 0x68cd1dcd, 3 },
    { 0, 0 },
    { 0x82a11dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa091dd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf91ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed1ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d1de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f11dea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d1def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8d1df4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e51df8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a11dfe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f351e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d11e0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a091e13, 3 },
    { 0xafcd1e14, 3 },
    { 0xb2391e15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1611e1d, 3 },
    { 0, 0 },
    { 0x22691e1f, 3 },
    { 0x16591e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36551e24, 3 },
    { 0x10c51e25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca11e2a, 3 },
    { 0xa6b11e2b, 3 },
    { 0, 0 },
    { 0x69991e2d, 3 },
    { 0x60fd1e2e, 4 },
    { 0, 0 },
    { 0x49f11e30, 3 },
    { 0, 0 },
    { 0x899d1e32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7751e37, 4 },
    { 0, 0 },
    { 0x437d1e39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51751e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b511e48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedd1e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56251e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fd1e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53d1e5c, 3 },
    { 0x9551e5d, 3 },
    { 0x6d9d1e5c, 3 },
    { 0, 0 },
    { 0xe6491e60, 3 },
    { 0xac611e60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5211e67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b91e6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c351e6e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ad1e71, 3 },
    { 0x893d1e72, 3 },
    { 0, 0 },
    { 0x27451e74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffd1e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a411e7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31651e7d, 3 },
    { 0xfe851e7e, 4 },
    { 0, 0 },
    { 0x8c851e80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ed1e84, 3 },
    { 0x1e5d1e84, 3 },
    { 0, 0 },
    { 0x19611e87, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74d1e8a, 3 },
    { 0x55a51e8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d1e8e, 3 },
    { 0xcc7d1e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1111e94, 4 },
    { 0, 0 },
    { 0x65791e96, 5 },
    { 0, 0 },
    { 0x34bd1e98, 3 },
    { 0, 0 },
    { 0x14991e9a, 3 },
    { 0, 0 },
    { 0x10211e9c, 3 },
    { 0, 0 },
    { 0x9911e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26611ea7, 3 },
    { 0x82091ea8, 4 },
    { 0, 0 },
    { 0x858d1eaa, 3 },
    { 0xd9011eab, 4 },
    { 0xe6f91eab, 4 },
    { 0xffb91eab, 3 },
    { 0xb98d1eae, 4 },
    { 0, 0 },
    { 0xbef11eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b11eb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d91eb8, 3 },
    { 0x91bd1eb8, 4 },
    { 0xf1311eba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18091ec0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851ec5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ed1ecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1791ed6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f91edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c711ee2, 3 },
    { 0x6bdd1ee2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e91ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29251ef1, 3 },
    { 0, 0 },
    { 0x5b811ef3, 3 },
    { 0xbcfd1ef4, 3 },
    { 0x14cd1ef5, 3 },
    { 0, 0 },
    { 0xe2d1ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b691efb, 3 },
    { 0xd9d51efc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfd1f08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e91f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34751f19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f91f1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb451f22, 4 },
    { 0, 0 },
    { 0xd1291f24, 3 },
    { 0, 0 },
    { 0x28191f26, 3 },
    { 0x508d1f27, 3 },
    { 0, 0 },
    { 0xb0d1f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd1f2f, 3 },
    { 0xf8c11f30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c191f37, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f451f3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf11f42, 4 },
    { 0, 0 },
    { 0x6dfd1f44, 3 },
    { 0, 0 },
    { 0xf8091f46, 5 },
    { 0x60c11f47, 3 },
    { 0xfedd1f48, 3 },
    { 0x61151f46, 4 },
    { 0x50551f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38291f4f, 3 },
    { 0, 0 },
    { 0x3b111f51, 3 },
    { 0x2ae51f52, 3 },
    { 0xda991f53, 4 },
    { 0, 0 },
    { 0x25211f55, 3 },
    { 0x52fd1f56, 4 },
    { 0, 0 },
    { 0x80311f58, 3 },
    { 0xf0051f58, 4 },
    { 0, 0 },
    { 0xa151f5b, 3 },
    { 0xf3dd1f5c, 4 },
    { 0xd8f91f5c, 3 },
    { 0xf0351f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e91f62, 3 },
    { 0x4d311f63, 3 },
    { 0xa15d1f64, 3 },
    { 0x45011f65, 3 },
    { 0xf6b11f62, 3 },
    { 0x4eb11f67, 3 },
    { 0x24c51f68, 3 },
    { 0x54891f68, 3 },
    { 0xb18d1f68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60391f6e, 3 },
    { 0, 0 },
    { 0xeaed1f70, 3 },
    { 0, 0 },
    { 0x6b891f72, 3 },
    { 0xb5a51f73, 3 },
    { 0xb3dd1f73, 3 },
    { 0, 0 },
    { 0x9211f76, 4 },
    { 0, 0 },
    { 0x60791f78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51f7b, 3 },
    { 0, 0 },
    { 0x63a51f7d, 3 },
    { 0x3b251f7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d791f82, 3 },
    { 0, 0 },
    { 0x27ad1f84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cd1f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5d1f8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefad1f91, 3 },
    { 0x5511f92, 4 },
    { 0x52911f93, 3 },
    { 0xb8c51f94, 3 },
    { 0x2f991f94, 5 },
    { 0x8bf91f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c851f99, 3 },
    { 0xae151f99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9051f9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79d1fa0, 3 },
    { 0xd22d1fa1, 3 },
    { 0x78811fa1, 3 },
    { 0xa8251fa3, 3 },
    { 0x9a7d1fa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e11fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191d1fb2, 3 },
    { 0xff551fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a51fb8, 3 },
    { 0xa0ed1fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc51fbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x288d1fc0, 4 },
    { 0x74991fc1, 3 },
    { 0, 0 },
    { 0xc2d11fc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c91fc7, 3 },
    { 0x86151fc8, 4 },
    { 0x34591fc9, 3 },
    { 0xb3451fc9, 3 },
    { 0xf11fc8, 4 },
    { 0x21dd1fc8, 4 },
    { 0x94111fca, 4 },
    { 0x6c151fce, 3 },
    { 0x5cfd1fce, 4 },
    { 0, 0 },
    { 0xc7791fd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2391fd8, 3 },
    { 0xd6a11fd9, 3 },
    { 0, 0 },
    { 0xe4d91fdb, 3 },
    { 0, 0 },
    { 0x35211fdd, 3 },
    { 0x528d1fdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a11fe3, 3 },
    { 0xd5d51fe3, 3 },
    { 0x8311fe5, 3 },
    { 0x86051fe6, 3 },
    { 0xbd751fe5, 4 },
    { 0x47991fe7, 3 },
    { 0x835d1fe3, 5 },
    { 0x4b211fe9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f491fed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26d1ff1, 3 },
    { 0xbe211ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd651ff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bd1ffc, 3 },
    { 0x1a0d1ffc, 3 },
    { 0, 0 },
    { 0x37391fff, 3 },
    { 0, 0 },
    { 0x961d2001, 3 },
    { 0xbd312002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8452006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bd200c, 3 },
    { 0xb181200d, 3 },
    { 0x85a1200c, 3 },
    { 0xd171200f, 4 },
    { 0xd069200f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d12013, 5 },
    { 0xefad2013, 4 },
    { 0, 0 },
    { 0x3a6d2016, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf52019, 3 },
    { 0, 0 },
    { 0xdd91201b, 3 },
    { 0x5c6d201c, 4 },
    { 0xcdf9201d, 3 },
    { 0xb339201c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14552021, 4 },
    { 0x61a92022, 3 },
    { 0xbdd52023, 4 },
    { 0x5eb12024, 3 },
    { 0x6cf92023, 3 },
    { 0xde992026, 4 },
    { 0, 0 },
    { 0xb7bd2028, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671d202d, 3 },
    { 0, 0 },
    { 0x1f1202f, 4 },
    { 0x4b512030, 3 },
    { 0xabd202f, 4 },
    { 0, 0 },
    { 0xbdc52033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f92036, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf845203b, 3 },
    { 0x5d29203b, 3 },
    { 0, 0 },
    { 0x4ec9203e, 3 },
    { 0x20c9203f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b812046, 3 },
    { 0xba312047, 3 },
    { 0x71492048, 3 },
    { 0x633d2048, 3 },
    { 0xb08d2046, 4 },
    { 0x6641204a, 3 },
    { 0xca3d204c, 3 },
    { 0x3489204d, 3 },
    { 0, 0 },
    { 0xc21204f, 3 },
    { 0x353d204f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x831d2053, 4 },
    { 0xc2a92054, 4 },
    { 0x8b0d2055, 4 },
    { 0, 0 },
    { 0xf85d2057, 3 },
    { 0xacc92058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f1205b, 4 },
    { 0, 0 },
    { 0xe449205d, 3 },
    { 0xd801205e, 4 },
    { 0, 0 },
    { 0x26012060, 3 },
    { 0xe5412061, 3 },
    { 0x4dad2062, 3 },
    { 0x12452063, 3 },
    { 0, 0 },
    { 0x8edd2065, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171206a, 3 },
    { 0, 0 },
    { 0x4eb9206c, 3 },
    { 0x187d206d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c212073, 3 },
    { 0x85bd2074, 3 },
    { 0xc7792075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d2079, 3 },
    { 0x6fa1207a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d2080, 3 },
    { 0, 0 },
    { 0xe4592082, 3 },
    { 0, 0 },
    { 0xd4452084, 3 },
    { 0x6c412085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35692089, 3 },
    { 0x20c92089, 3 },
    { 0x9d7d208b, 4 },
    { 0x9ddd208b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27352091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd2094, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ad209b, 4 },
    { 0, 0 },
    { 0x5371209d, 4 },
    { 0, 0 },
    { 0x10b1209f, 3 },
    { 0x3a11209f, 4 },
    { 0, 0 },
    { 0xfd9120a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16520a5, 3 },
    { 0xd77920a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411120ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c520ae, 4 },
    { 0, 0 },
    { 0x326520b0, 3 },
    { 0, 0 },
    { 0x9cc520b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad20bd, 3 },
    { 0x19b120be, 3 },
    { 0xcc0520be, 3 },
    { 0x972120bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d920c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d20ca, 4 },
    { 0x719520ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217120d1, 3 },
    { 0x42f120d2, 4 },
    { 0xe47120d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7520d9, 3 },
    { 0xb94120da, 3 },
    { 0xa25520d9, 3 },
    { 0x6be520dc, 3 },
    { 0x1e6120dd, 3 },
    { 0x595520dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d520e1, 4 },
    { 0x6c5520e1, 3 },
    { 0xe9d20e3, 4 },
    { 0x391120e4, 3 },
    { 0xf91520e4, 3 },
    { 0, 0 },
    { 0xa3a520e7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d20ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6520f0, 3 },
    { 0x85c520f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4920f9, 3 },
    { 0x753120fa, 4 },
    { 0x107520fb, 3 },
    { 0xfc7120fa, 3 },
    { 0, 0 },
    { 0xb63d20fe, 3 },
    { 0xc22520ff, 3 },
    { 0xc8512100, 3 },
    { 0x69212100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7d2107, 3 },
    { 0xac092108, 3 },
    { 0, 0 },
    { 0x5505210a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac552112, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999d2119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edd211e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc51212a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e5212e, 5 },
    { 0x8715212f, 3 },
    { 0x7865212f, 3 },
    { 0, 0 },
    { 0x92cd2132, 4 },
    { 0x90812132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd92137, 3 },
    { 0, 0 },
    { 0xd7812139, 3 },
    { 0x433d2139, 3 },
    { 0x12e9213b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9035213f, 3 },
    { 0x2d712140, 3 },
    { 0xbaa92141, 3 },
    { 0x85d52142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941d2146, 4 },
    { 0xb7f92147, 3 },
    { 0xe54d2148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd214c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f52154, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7652159, 3 },
    { 0x50fd2159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3291215d, 4 },
    { 0xcf4d215e, 3 },
    { 0x5289215d, 3 },
    { 0x73352160, 4 },
    { 0x29812161, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x410d2164, 3 },
    { 0, 0 },
    { 0xc9112166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d2169, 3 },
    { 0, 0 },
    { 0x981216b, 3 },
    { 0x7a75216c, 3 },
    { 0x6d9d216d, 3 },
    { 0, 0 },
    { 0xb985216f, 4 },
    { 0x6635216f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a52177, 4 },
    { 0, 0 },
    { 0x17a12179, 3 },
    { 0xa499217a, 5 },
    { 0xa029217b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe012181, 3 },
    { 0x5cc92182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370d2188, 3 },
    { 0x80652188, 3 },
    { 0x5d5d2189, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1218e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d2191, 4 },
    { 0, 0 },
    { 0xe6692193, 3 },
    { 0x648d2194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9412199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2d219d, 3 },
    { 0xadd219e, 3 },
    { 0x92c9219f, 3 },
    { 0x283121a0, 3 },
    { 0xc3c521a1, 4 },
    { 0x177921a1, 3 },
    { 0x5525219e, 4 },
    { 0x2b0121a4, 3 },
    { 0xf3521a5, 3 },
    { 0x138921a6, 3 },
    { 0x715921a3, 4 },
    { 0x57dd21a8, 4 },
    { 0x7e7521a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f121ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa121b1, 3 },
    { 0, 0 },
    { 0xa62921b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c121ba, 3 },
    { 0xd76521bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36121c0, 4 },
    { 0x7bbd21c0, 3 },
    { 0, 0 },
    { 0x865921c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d121c7, 3 },
    { 0x8f7d21c8, 3 },
    { 0xc92121c8, 3 },
    { 0x46f921ca, 3 },
    { 0x5fb921cb, 3 },
    { 0x149121cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18121d2, 4 },
    { 0xa08921d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9121de, 3 },
    { 0x641d21de, 3 },
    { 0x1afd21df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7521e5, 3 },
    { 0x6c1d21e5, 3 },
    { 0xc86d21e7, 3 },
    { 0xc4c921e7, 3 },
    { 0xc82521e7, 4 },
    { 0x2c0521e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d121ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d521f4, 4 },
    { 0xd39121f5, 3 },
    { 0, 0 },
    { 0xc77121f7, 3 },
    { 0x4fd121f7, 3 },
    { 0xc8d521f8, 4 },
    { 0xa0c521fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40921fe, 3 },
    { 0, 0 },
    { 0x44e52200, 3 },
    { 0x70352201, 3 },
    { 0xedb52202, 3 },
    { 0xa1ed2203, 3 },
    { 0x17452202, 3 },
    { 0, 0 },
    { 0x92892206, 4 },
    { 0, 0 },
    { 0x38392208, 3 },
    { 0x79952208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc935220e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d92213, 4 },
    { 0x2d592214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d221b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f45221e, 3 },
    { 0xd3e1221e, 3 },
    { 0xc6912220, 3 },
    { 0x3192221, 3 },
    { 0, 0 },
    { 0xf9112223, 4 },
    { 0xc1052224, 3 },
    { 0x6d452224, 3 },
    { 0x96ad2226, 4 },
    { 0, 0 },
    { 0x49f52228, 5 },
    { 0x24b12228, 4 },
    { 0x977d2229, 4 },
    { 0x8b7d222b, 3 },
    { 0x6299222c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91d2230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3952235, 3 },
    { 0xa2e52235, 3 },
    { 0x44a12237, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e41223b, 3 },
    { 0x4059223b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f352241, 3 },
    { 0xc6992242, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b612248, 4 },
    { 0x21552248, 3 },
    { 0, 0 },
    { 0xa095224b, 3 },
    { 0x8419224c, 4 },
    { 0x1ca5224b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1452252, 3 },
    { 0x7af92253, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b52256, 5 },
    { 0, 0 },
    { 0x183d2258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea52264, 3 },
    { 0x3b212264, 3 },
    { 0xde552266, 3 },
    { 0xd8b52265, 3 },
    { 0x2e12268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c52271, 3 },
    { 0xd4552272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d712275, 3 },
    { 0x2f5d2276, 3 },
    { 0, 0 },
    { 0x94592278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9292284, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0552288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2621228d, 3 },
    { 0xfe3d228d, 3 },
    { 0, 0 },
    { 0xa5012290, 4 },
    { 0xcda92291, 3 },
    { 0, 0 },
    { 0xba0d2293, 3 },
    { 0x7a492294, 4 },
    { 0x4bcd2293, 3 },
    { 0x40952293, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c49229c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c922a3, 3 },
    { 0x627d22a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75522a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e522ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a522af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bd22b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b122b7, 4 },
    { 0xdfe122b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04922c0, 3 },
    { 0, 0 },
    { 0x3e1922c2, 3 },
    { 0, 0 },
    { 0x420522c4, 3 },
    { 0x20a122c5, 3 },
    { 0x561d22c6, 3 },
    { 0xa11d22c6, 4 },
    { 0x749d22c8, 3 },
    { 0x806922c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0522cd, 4 },
    { 0x327d22cd, 4 },
    { 0x354522cf, 4 },
    { 0xad3522ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278522d5, 3 },
    { 0x760122d6, 3 },
    { 0, 0 },
    { 0x3e122d8, 4 },
    { 0xc13922d9, 3 },
    { 0xe99522da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd522e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e122ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f122f1, 4 },
    { 0xa59522f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4922fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43122fd, 3 },
    { 0x9c2522fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31d2301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a712304, 3 },
    { 0xfb152305, 3 },
    { 0, 0 },
    { 0xbd152307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e5230a, 3 },
    { 0x385d230a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5230f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45152315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c52319, 3 },
    { 0xfbad231a, 3 },
    { 0x4cc1231b, 3 },
    { 0xe881231c, 3 },
    { 0x29b5231a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fed2322, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62352326, 4 },
    { 0xd1f12326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e5232c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecd2330, 4 },
    { 0xfb112330, 5 },
    { 0xee452332, 3 },
    { 0x35e92331, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b92337, 4 },
    { 0, 0 },
    { 0x44612339, 3 },
    { 0x1e3d233a, 3 },
    { 0, 0 },
    { 0xb19233c, 3 },
    { 0x7dd5233d, 3 },
    { 0x6105233e, 3 },
    { 0x2635233e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4d2342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x340d2345, 3 },
    { 0x53692345, 3 },
    { 0xac952347, 3 },
    { 0xbc792348, 3 },
    { 0x21f52349, 3 },
    { 0xadd12348, 3 },
    { 0x6b352348, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4345234e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bd2357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6135235d, 4 },
    { 0x134d235e, 3 },
    { 0, 0 },
    { 0x72652360, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74592363, 3 },
    { 0xf6c92364, 3 },
    { 0xe0692365, 3 },
    { 0xe4452366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c52369, 3 },
    { 0, 0 },
    { 0x9d59236b, 3 },
    { 0xa615236c, 3 },
    { 0x945236c, 3 },
    { 0x619d236e, 4 },
    { 0x9875236e, 3 },
    { 0xc929236e, 3 },
    { 0x20c92371, 4 },
    { 0xda7d2370, 3 },
    { 0xedf9236d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3d2376, 3 },
    { 0x68e52377, 3 },
    { 0, 0 },
    { 0xa5652379, 3 },
    { 0x7fcd2379, 4 },
    { 0xfd9d237b, 3 },
    { 0xb799237c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca52380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed2385, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5695238a, 4 },
    { 0, 0 },
    { 0x6425238c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32692390, 4 },
    { 0x12412391, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a92397, 3 },
    { 0, 0 },
    { 0xba192399, 4 },
    { 0xea09239a, 3 },
    { 0, 0 },
    { 0x51b1239c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7923a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5923a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3123ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457123b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27923ba, 3 },
    { 0x166523bb, 3 },
    { 0, 0 },
    { 0x2fa123bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199d23c2, 3 },
    { 0x522d23c3, 4 },
    { 0xaa0523c4, 3 },
    { 0x56bd23c5, 4 },
    { 0x29bd23c6, 3 },
    { 0xcfcd23c4, 3 },
    { 0xe58d23c8, 3 },
    { 0x51f523c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279523d2, 4 },
    { 0x737523d3, 3 },
    { 0x538d23d4, 5 },
    { 0x470923d5, 3 },
    { 0xb95123d5, 3 },
    { 0x1ba123d4, 3 },
    { 0xef0923d2, 3 },
    { 0xd31523d9, 5 },
    { 0x296123da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40923dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb123e2, 4 },
    { 0xcaad23e2, 3 },
    { 0, 0 },
    { 0x341923e5, 3 },
    { 0, 0 },
    { 0xebb523e7, 4 },
    { 0xcac523e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c123eb, 4 },
    { 0x3b9923ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e523f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843523f5, 3 },
    { 0xf00d23f5, 3 },
    { 0x21e923f6, 3 },
    { 0x156523f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735d23fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf92403, 3 },
    { 0xfb592404, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13352409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e9240d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34492413, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dd2416, 3 },
    { 0, 0 },
    { 0x61292418, 4 },
    { 0xd2fd2419, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77d241d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef152423, 4 },
    { 0, 0 },
    { 0xf9192425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88952428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec1242d, 3 },
    { 0, 0 },
    { 0xcdf5242f, 4 },
    { 0, 0 },
    { 0xf1c92431, 3 },
    { 0x2c012432, 4 },
    { 0x8a852433, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd952436, 4 },
    { 0xdcb52437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5052445, 3 },
    { 0xdff92446, 3 },
    { 0, 0 },
    { 0xb8f12448, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b41244e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1412451, 3 },
    { 0, 0 },
    { 0xcdf52453, 3 },
    { 0xb2a12454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d245d, 3 },
    { 0, 0 },
    { 0xd091245f, 3 },
    { 0, 0 },
    { 0x15752461, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd2465, 4 },
    { 0x42692466, 3 },
    { 0, 0 },
    { 0x1fd92468, 3 },
    { 0, 0 },
    { 0x62a9246a, 4 },
    { 0, 0 },
    { 0x28b1246c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d2474, 3 },
    { 0, 0 },
    { 0x6ea92476, 3 },
    { 0xbd092477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db9247a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd5247e, 3 },
    { 0x967d247f, 3 },
    { 0xdf152480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fd2485, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x364d2488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c5248d, 4 },
    { 0xebd1248d, 4 },
    { 0, 0 },
    { 0xe9552490, 3 },
    { 0, 0 },
    { 0x303d2492, 3 },
    { 0x4ca12493, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d2496, 3 },
    { 0x7d512497, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae9249a, 3 },
    { 0x83ed249b, 3 },
    { 0xf075249c, 4 },
    { 0x7c41249d, 3 },
    { 0, 0 },
    { 0xae21249f, 3 },
    { 0x60524a0, 3 },
    { 0xbd9d24a0, 3 },
    { 0x324524a2, 4 },
    { 0x30c1249f, 4 },
    { 0x42f924a4, 4 },
    { 0xb8f124a3, 4 },
    { 0xffa524a6, 3 },
    { 0x3df124a1, 3 },
    { 0x4e9524a8, 3 },
    { 0x6ed24a2, 4 },
    { 0xa0b924aa, 3 },
    { 0x273d24a0, 4 },
    { 0xd59924ac, 4 },
    { 0x426124ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a524b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c124b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e924c1, 3 },
    { 0, 0 },
    { 0x3d0524c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd124c7, 3 },
    { 0xa7e924c8, 3 },
    { 0xe9fd24c9, 3 },
    { 0x3f1924ca, 3 },
    { 0xa53924cb, 3 },
    { 0xda8124cc, 3 },
    { 0xa1b124c8, 4 },
    { 0xc4f524ca, 3 },
    { 0x5bdd24ca, 4 },
    { 0x6f2524ca, 4 },
    { 0, 0 },
    { 0x486d24d2, 3 },
    { 0x91e524d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7924d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cd24dc, 4 },
    { 0xe58124dd, 4 },
    { 0, 0 },
    { 0x4eb124df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a124e2, 3 },
    { 0, 0 },
    { 0x849924e4, 4 },
    { 0, 0 },
    { 0x38d924e6, 3 },
    { 0x495924e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x898124ea, 3 },
    { 0, 0 },
    { 0x42dd24ec, 3 },
    { 0x553124ed, 3 },
    { 0x7e1d24ed, 3 },
    { 0x770124ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8d24f2, 3 },
    { 0x5c5924f3, 3 },
    { 0xe95d24f4, 3 },
    { 0xbbb924f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88524f9, 3 },
    { 0x1a5924fa, 3 },
    { 0x31e524fb, 3 },
    { 0xe98924fc, 3 },
    { 0x42fd24fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f52500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d2504, 4 },
    { 0, 0 },
    { 0xcc12506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6151250b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9412513, 3 },
    { 0, 0 },
    { 0x72f92515, 3 },
    { 0, 0 },
    { 0xf83d2517, 3 },
    { 0x17b12517, 3 },
    { 0xbda92519, 3 },
    { 0xd6d251a, 4 },
    { 0x8131251b, 4 },
    { 0xd7b5251c, 3 },
    { 0, 0 },
    { 0x2ff9251e, 4 },
    { 0xd425251f, 3 },
    { 0xe7652520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63492523, 3 },
    { 0, 0 },
    { 0x5c792525, 4 },
    { 0x8ba92525, 3 },
    { 0, 0 },
    { 0x11612528, 4 },
    { 0x45d52528, 3 },
    { 0, 0 },
    { 0x112d252b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x212d252e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f512532, 3 },
    { 0x62312532, 3 },
    { 0, 0 },
    { 0x60e52535, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a9253f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a52543, 3 },
    { 0x5c4d2544, 5 },
    { 0x96852544, 3 },
    { 0xaed92543, 3 },
    { 0xe72d2547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed254f, 5 },
    { 0xf092550, 5 },
    { 0x8bd92550, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efd2557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7425255d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c52560, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca92563, 3 },
    { 0x7e312563, 3 },
    { 0xb6092565, 3 },
    { 0, 0 },
    { 0xe9a12567, 4 },
    { 0xfaed2567, 3 },
    { 0xe3112569, 3 },
    { 0x751d2567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb21256d, 3 },
    { 0x9875256e, 4 },
    { 0, 0 },
    { 0x28812570, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78852573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8892576, 3 },
    { 0, 0 },
    { 0x31412578, 4 },
    { 0xe73d2579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29892583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6d2589, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c11258c, 3 },
    { 0x4ac9258d, 4 },
    { 0x99a9258d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e52592, 3 },
    { 0x37212593, 3 },
    { 0x68352594, 3 },
    { 0x646d2593, 3 },
    { 0xb4652595, 3 },
    { 0x81312592, 4 },
    { 0xea812593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469259c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9125a3, 3 },
    { 0, 0 },
    { 0x618d25a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490525aa, 3 },
    { 0, 0 },
    { 0x57d25ac, 3 },
    { 0x39b525ac, 4 },
    { 0, 0 },
    { 0xf68d25af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcd25b5, 3 },
    { 0xa62925b6, 3 },
    { 0x3db925b7, 3 },
    { 0x8f7925b8, 3 },
    { 0, 0 },
    { 0x1a9d25ba, 3 },
    { 0, 0 },
    { 0x135125bc, 3 },
    { 0x469925bd, 4 },
    { 0xf8c525be, 3 },
    { 0, 0 },
    { 0xafb925c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed925c5, 3 },
    { 0xade125c6, 3 },
    { 0x3ea125c5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289125cf, 3 },
    { 0x8d1925d0, 4 },
    { 0x9d8525d1, 3 },
    { 0x385525d1, 4 },
    { 0xe96925d1, 3 },
    { 0, 0 },
    { 0xc9525d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673d25dc, 3 },
    { 0x8ad25dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa525e0, 3 },
    { 0x82ad25e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7925eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x279925ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c925f4, 5 },
    { 0x42cd25f5, 3 },
    { 0x4a1125f5, 4 },
    { 0, 0 },
    { 0x96cd25f8, 3 },
    { 0x923d25f9, 4 },
    { 0x5ff925fa, 4 },
    { 0x180d25fb, 4 },
    { 0xae1925fa, 3 },
    { 0x879d25fd, 4 },
    { 0, 0 },
    { 0x995125ff, 3 },
    { 0x42892600, 4 },
    { 0, 0 },
    { 0xe852602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82412605, 3 },
    { 0x874d2606, 3 },
    { 0xa892607, 3 },
    { 0xc4812608, 3 },
    { 0xf19d2609, 3 },
    { 0x7521260a, 3 },
    { 0x61a1260a, 3 },
    { 0x8c19260a, 3 },
    { 0x763d260d, 3 },
    { 0xbe41260e, 3 },
    { 0x24a1260f, 3 },
    { 0x6705260a, 4 },
    { 0xef552608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b92617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c65261c, 3 },
    { 0, 0 },
    { 0xc601261e, 3 },
    { 0, 0 },
    { 0x6c592620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd2624, 3 },
    { 0xf0652624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a12629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d262d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2631, 4 },
    { 0xe23d2632, 3 },
    { 0x9a892633, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965263a, 3 },
    { 0x3c19263b, 4 },
    { 0, 0 },
    { 0x5d1263d, 3 },
    { 0xf18d263d, 3 },
    { 0, 0 },
    { 0x32552640, 3 },
    { 0x81c12641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0492645, 3 },
    { 0xa1c52646, 3 },
    { 0x6b92645, 3 },
    { 0, 0 },
    { 0x2ca12649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f29264d, 3 },
    { 0, 0 },
    { 0x59c9264f, 3 },
    { 0xa1d52650, 3 },
    { 0x63692651, 4 },
    { 0x89312651, 3 },
    { 0xe1912653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fd2656, 3 },
    { 0x1d012657, 3 },
    { 0x60d2658, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9265d, 3 },
    { 0x5e91265e, 3 },
    { 0x6c99265f, 3 },
    { 0x58a92660, 3 },
    { 0xab612660, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf92664, 3 },
    { 0, 0 },
    { 0x35412666, 3 },
    { 0xef552666, 3 },
    { 0, 0 },
    { 0x64ed2669, 4 },
    { 0xefdd2669, 3 },
    { 0x9315266b, 3 },
    { 0, 0 },
    { 0x5315266d, 4 },
    { 0xcec9266e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb252672, 4 },
    { 0xa59d2673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f75267f, 4 },
    { 0x9b1267f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1252685, 4 },
    { 0, 0 },
    { 0xe3a52687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05268b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3d2693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96d269a, 4 },
    { 0xb241269b, 3 },
    { 0xb521269b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd269f, 3 },
    { 0, 0 },
    { 0xaffd26a1, 3 },
    { 0xe88926a2, 4 },
    { 0x935926a2, 3 },
    { 0x2b5d26a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x104126a7, 4 },
    { 0x8c3526a8, 3 },
    { 0x447d26a9, 3 },
    { 0xf88526a9, 3 },
    { 0, 0 },
    { 0xae0126ac, 3 },
    { 0x916126ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44526b1, 3 },
    { 0, 0 },
    { 0xd18526b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a126b9, 4 },
    { 0, 0 },
    { 0x533126bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46d26be, 3 },
    { 0, 0 },
    { 0x3b8126c0, 3 },
    { 0x94f926c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bd26c5, 3 },
    { 0x844126c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b926c9, 4 },
    { 0xad2526c9, 3 },
    { 0x17c926c9, 4 },
    { 0x662526cb, 3 },
    { 0x504526cb, 4 },
    { 0x4a9926ce, 3 },
    { 0xe2ed26c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x247d26d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4126d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222926e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87d26e7, 3 },
    { 0, 0 },
    { 0x71c926e9, 4 },
    { 0x4d0926ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d126ed, 4 },
    { 0xbd2926ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d5270b, 3 },
    { 0x8dc9270c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40612710, 3 },
    { 0xd3992711, 3 },
    { 0x78352711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7152718, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821271b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0012721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186d2726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec712735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10692738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735273b, 3 },
    { 0, 0 },
    { 0x96bd273d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2740, 4 },
    { 0xa9712741, 3 },
    { 0x15b12742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a12745, 3 },
    { 0x64d12746, 4 },
    { 0, 0 },
    { 0x33c52748, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe045274d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71612758, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6329275b, 3 },
    { 0, 0 },
    { 0x5c9275d, 3 },
    { 0x90c5275e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd2765, 3 },
    { 0, 0 },
    { 0xbe852767, 3 },
    { 0x88b52768, 3 },
    { 0xc28d2769, 4 },
    { 0, 0 },
    { 0xc179276b, 4 },
    { 0xbd69276b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c5276f, 3 },
    { 0x4b412770, 3 },
    { 0x961d2771, 3 },
    { 0, 0 },
    { 0x22152773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd277c, 3 },
    { 0xa9d5277d, 4 },
    { 0xf819277e, 3 },
    { 0xe8b5277d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e592782, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f92786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf49278d, 3 },
    { 0, 0 },
    { 0x25f5278f, 3 },
    { 0x95912790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a12793, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16452797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9051279a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a49279e, 3 },
    { 0x295d279f, 4 },
    { 0, 0 },
    { 0x35e527a1, 3 },
    { 0xbaa127a2, 3 },
    { 0x499127a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475927aa, 4 },
    { 0xdae927ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fd27b1, 3 },
    { 0xc8c127b2, 5 },
    { 0xb44127b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c127ba, 4 },
    { 0x97b527bb, 3 },
    { 0x922527ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x137d27bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8927c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c127c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781527cc, 3 },
    { 0x6e5d27cd, 3 },
    { 0, 0 },
    { 0x356127cf, 4 },
    { 0xbfb527d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7127d8, 3 },
    { 0xf44527d8, 3 },
    { 0xe21927da, 3 },
    { 0x99a527da, 3 },
    { 0x463927da, 5 },
    { 0x785127dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d27e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85127ec, 3 },
    { 0xe97d27ed, 3 },
    { 0x3a4127ee, 3 },
    { 0x7ff927ee, 3 },
    { 0xa29527f0, 4 },
    { 0x9aa927f1, 4 },
    { 0x31b527f1, 4 },
    { 0x431d27f1, 4 },
    { 0xd79527f3, 4 },
    { 0x9b0127ec, 3 },
    { 0, 0 },
    { 0x407127f7, 4 },
    { 0x611527f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57527fb, 4 },
    { 0xc4127fb, 3 },
    { 0x149d27fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a92801, 3 },
    { 0xe6d2801, 3 },
    { 0x3fed2803, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744d2809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b9280d, 3 },
    { 0xa419280e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c712811, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd2817, 3 },
    { 0x8aa12818, 4 },
    { 0x6c6d2819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98692823, 3 },
    { 0x5f452823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6152827, 3 },
    { 0xaa352827, 4 },
    { 0x8e1d2829, 3 },
    { 0xa335282a, 3 },
    { 0, 0 },
    { 0x8959282c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129282f, 3 },
    { 0xf3e52830, 3 },
    { 0xec552831, 3 },
    { 0xe5992830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5652838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x471283b, 3 },
    { 0xfdb9283c, 3 },
    { 0, 0 },
    { 0x5bf5283e, 3 },
    { 0x9b2d283f, 4 },
    { 0x94c9283e, 3 },
    { 0x49a52841, 3 },
    { 0, 0 },
    { 0x81192843, 3 },
    { 0x65152844, 3 },
    { 0x91052845, 3 },
    { 0xce5d2846, 3 },
    { 0xb1652847, 3 },
    { 0x320d2846, 4 },
    { 0, 0 },
    { 0x368d284a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82112851, 3 },
    { 0xebc92852, 3 },
    { 0x1ce52852, 4 },
    { 0x67412854, 3 },
    { 0xebe52852, 3 },
    { 0xd452856, 4 },
    { 0, 0 },
    { 0x50092858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a9285e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78912861, 3 },
    { 0, 0 },
    { 0x19b52863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b452868, 3 },
    { 0xbf52869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae1286d, 3 },
    { 0xea45286d, 3 },
    { 0xe3fd286f, 4 },
    { 0x7c55286f, 4 },
    { 0, 0 },
    { 0xc52872, 4 },
    { 0xa00d2872, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f12876, 5 },
    { 0x52552877, 4 },
    { 0x66652877, 3 },
    { 0x96612876, 3 },
    { 0, 0 },
    { 0xb421287b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab39287e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2092881, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab12886, 3 },
    { 0, 0 },
    { 0x7dbd2888, 3 },
    { 0x5c452889, 4 },
    { 0x9dfd2889, 4 },
    { 0, 0 },
    { 0xea8d288c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d65288f, 4 },
    { 0x1eed288f, 3 },
    { 0x1ee92891, 4 },
    { 0x90cd2892, 3 },
    { 0x97792891, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51d2899, 4 },
    { 0x17bd2899, 3 },
    { 0x8be9289a, 3 },
    { 0x3221289c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882528a0, 4 },
    { 0x891128a1, 3 },
    { 0x41d928a2, 3 },
    { 0, 0 },
    { 0x634928a4, 3 },
    { 0xaa3d28a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4128ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe528b1, 3 },
    { 0x7928b2, 3 },
    { 0x61bd28b2, 3 },
    { 0x2a3928b4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6928bf, 4 },
    { 0xd49928bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x732928c3, 3 },
    { 0, 0 },
    { 0x969d28c5, 5 },
    { 0x1c528c6, 3 },
    { 0xbcad28c7, 3 },
    { 0x620d28c8, 3 },
    { 0, 0 },
    { 0x65c528ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccd28cd, 3 },
    { 0, 0 },
    { 0x1bc528cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea528d5, 4 },
    { 0x7cd128d5, 3 },
    { 0xb5e928d7, 3 },
    { 0, 0 },
    { 0x527928d9, 3 },
    { 0x90c528da, 3 },
    { 0x9df128db, 5 },
    { 0x69c128dc, 5 },
    { 0x519928dd, 3 },
    { 0x719928de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae928e2, 5 },
    { 0x841928e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25d28e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318528ee, 3 },
    { 0xf00928ef, 3 },
    { 0x65a928f0, 4 },
    { 0x61dd28f1, 3 },
    { 0x828d28f2, 4 },
    { 0xb47d28f3, 4 },
    { 0, 0 },
    { 0x44e128f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51892902, 3 },
    { 0x45512903, 3 },
    { 0x14b12904, 4 },
    { 0x3a812904, 4 },
    { 0xc0b12902, 4 },
    { 0, 0 },
    { 0xc0152908, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa405290e, 3 },
    { 0x56d9290f, 3 },
    { 0x73dd2910, 3 },
    { 0, 0 },
    { 0x89e52912, 4 },
    { 0x9d492912, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb12919, 3 },
    { 0xd265291a, 3 },
    { 0, 0 },
    { 0x3f61291c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435291f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fd2922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352926, 3 },
    { 0xbd152927, 3 },
    { 0, 0 },
    { 0x57652929, 3 },
    { 0x1375292a, 3 },
    { 0x38f5292b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7439292f, 3 },
    { 0x89fd2930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad92936, 3 },
    { 0x64952937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd649293a, 3 },
    { 0, 0 },
    { 0xaa05293c, 3 },
    { 0x1981293d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6d2942, 4 },
    { 0x33292942, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0d2947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d69294d, 4 },
    { 0, 0 },
    { 0x2ad5294f, 3 },
    { 0xa8912950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x587d2953, 3 },
    { 0x86e92954, 3 },
    { 0, 0 },
    { 0x20412956, 3 },
    { 0x94912957, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e51295e, 3 },
    { 0x67d295f, 3 },
    { 0xc0552960, 3 },
    { 0xd2c12961, 4 },
    { 0x6f9d2960, 3 },
    { 0x88f9295e, 3 },
    { 0x93b52964, 3 },
    { 0xfd212964, 3 },
    { 0x41852966, 3 },
    { 0xbc952967, 3 },
    { 0xe0212967, 4 },
    { 0xb1d5295e, 3 },
    { 0, 0 },
    { 0xd839296b, 3 },
    { 0, 0 },
    { 0x375296d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5d2971, 4 },
    { 0x3bed2971, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e092975, 3 },
    { 0xc4ed2976, 4 },
    { 0xa8412976, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9135297b, 4 },
    { 0, 0 },
    { 0xb969297d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb92982, 3 },
    { 0xe9752983, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a9298a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b9298d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d52991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf252996, 3 },
    { 0, 0 },
    { 0xeb5d2998, 3 },
    { 0x1fb52998, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd1299d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4529a5, 3 },
    { 0, 0 },
    { 0xc45529a7, 3 },
    { 0, 0 },
    { 0x6d7929a9, 4 },
    { 0x3a1d29aa, 3 },
    { 0, 0 },
    { 0xd16929ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c929af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8129b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b529b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769529ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378129bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdd29c3, 3 },
    { 0x124d29c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd29c7, 3 },
    { 0, 0 },
    { 0x20e529c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84d29cc, 3 },
    { 0, 0 },
    { 0x29cd29ce, 4 },
    { 0, 0 },
    { 0x879529d0, 3 },
    { 0, 0 },
    { 0x77f529d2, 4 },
    { 0x4c5129d2, 4 },
    { 0, 0 },
    { 0x1b4129d5, 4 },
    { 0x6c6929d6, 3 },
    { 0, 0 },
    { 0x8b129d8, 3 },
    { 0xdd7929d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14929df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb529e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd29e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f529f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59129f7, 3 },
    { 0xfc2d29f8, 3 },
    { 0x75929f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a529fe, 5 },
    { 0x4a3d29ff, 3 },
    { 0x71952a00, 3 },
    { 0x2b752a01, 4 },
    { 0xe99929ff, 3 },
    { 0, 0 },
    { 0x70f52a04, 3 },
    { 0x4f192a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18692a0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5d2a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed12a17, 3 },
    { 0, 0 },
    { 0x23f52a19, 3 },
    { 0, 0 },
    { 0xf392a1b, 4 },
    { 0, 0 },
    { 0x9b052a1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d2a22, 4 },
    { 0x5b492a23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4492a26, 3 },
    { 0x3fe12a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa12a31, 4 },
    { 0xb1d12a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364d2a39, 3 },
    { 0, 0 },
    { 0xfe492a3b, 4 },
    { 0xde5d2a3c, 3 },
    { 0xb592a3d, 4 },
    { 0, 0 },
    { 0xfa52a3f, 3 },
    { 0xf4152a3f, 3 },
    { 0, 0 },
    { 0x24992a42, 4 },
    { 0x4b352a43, 3 },
    { 0x1cc92a44, 5 },
    { 0x7b152a42, 3 },
    { 0x4a12a46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c912a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7d2a4f, 3 },
    { 0xb1252a50, 4 },
    { 0xf0b92a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc112a56, 3 },
    { 0, 0 },
    { 0x8dc12a58, 3 },
    { 0, 0 },
    { 0x6b552a5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c92a5d, 3 },
    { 0x94412a5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7d2a61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9452a65, 3 },
    { 0xd2d12a65, 3 },
    { 0x47a92a67, 4 },
    { 0x31492a67, 3 },
    { 0x95912a69, 4 },
    { 0, 0 },
    { 0x86752a6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f12a6f, 4 },
    { 0xf07d2a70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c192a7d, 3 },
    { 0xf9a12a7e, 3 },
    { 0x56492a7d, 3 },
    { 0x5b092a7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cd2a87, 3 },
    { 0x92452a87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4712a8e, 3 },
    { 0xa992a8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead92a94, 3 },
    { 0xa2b92a94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d2a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a92aa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0992aa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4552aac, 4 },
    { 0, 0 },
    { 0x75e52aae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49492ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f12ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8792ab8, 3 },
    { 0xd8552ab9, 4 },
    { 0x59fd2aba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x736d2abd, 3 },
    { 0, 0 },
    { 0x2d912abf, 4 },
    { 0x84192ac0, 3 },
    { 0x9a512ac1, 4 },
    { 0x69bd2ac2, 4 },
    { 0xa6d52ac2, 3 },
    { 0x78992ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76652acb, 3 },
    { 0xfce12acc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61652ad0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83d2ad4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27392ad7, 3 },
    { 0, 0 },
    { 0x3252ad9, 4 },
    { 0x3a3d2ad9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e12ae7, 4 },
    { 0x6852ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d2aec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e52aef, 4 },
    { 0xec712af0, 3 },
    { 0x86ed2af1, 4 },
    { 0, 0 },
    { 0x36512af3, 3 },
    { 0, 0 },
    { 0x599d2af5, 4 },
    { 0x72712af6, 3 },
    { 0x45792af5, 3 },
    { 0x44112af8, 3 },
    { 0xe2712af9, 3 },
    { 0x89052afa, 4 },
    { 0x39992af8, 3 },
    { 0x4ed12afc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88592b04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bd2b08, 4 },
    { 0xc2a52b09, 3 },
    { 0, 0 },
    { 0x69d2b0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82092b14, 3 },
    { 0, 0 },
    { 0x84512b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e92b1c, 4 },
    { 0xe0d92b1d, 3 },
    { 0xf16d2b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b52b21, 3 },
    { 0xac492b21, 4 },
    { 0xe012b23, 4 },
    { 0x59ed2b24, 3 },
    { 0, 0 },
    { 0xea92b26, 3 },
    { 0xdea12b27, 3 },
    { 0xecb52b28, 3 },
    { 0x569d2b29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9452b2d, 4 },
    { 0x4cdd2b2e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xac792b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x585d2b34, 3 },
    { 0xd24d2b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e992b39, 3 },
    { 0xbc452b39, 3 },
    { 0x77992b3b, 4 },
    { 0x5c992b3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7512b40, 3 },
    { 0, 0 },
    { 0x907d2b42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e352b47, 3 },
    { 0xf2252b48, 3 },
    { 0xa9092b49, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee52b4f, 3 },
    { 0, 0 },
    { 0xb652b51, 3 },
    { 0xe8bd2b52, 3 },
    { 0x70552b52, 3 },
    { 0x7fb52b54, 3 },
    { 0x84592b55, 3 },
    { 0, 0 },
    { 0x22a12b57, 3 },
    { 0x8ad52b58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de12b5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c92b63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe92b69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f12b6c, 4 },
    { 0xfafd2b6c, 4 },
    { 0x28f92b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a952b7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f92b7f, 3 },
    { 0, 0 },
    { 0xf1cd2b81, 4 },
    { 0xf1992b81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6692b85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a92b8a, 3 },
    { 0x8f612b8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a12b90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d2b97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d2b9a, 3 },
    { 0xde112b9b, 5 },
    { 0xd00d2b9a, 3 },
    { 0x13892b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49652ba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb692ba8, 5 },
    { 0x36452ba9, 3 },
    { 0, 0 },
    { 0xef912bab, 3 },
    { 0x3c0d2bac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3852bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37792bba, 3 },
    { 0, 0 },
    { 0x229d2bbc, 3 },
    { 0, 0 },
    { 0xa4b92bbe, 3 },
    { 0xbd552bbe, 3 },
    { 0, 0 },
    { 0xc8852bc1, 3 },
    { 0x19852bc2, 3 },
    { 0, 0 },
    { 0x57a12bc4, 4 },
    { 0xb7292bc4, 4 },
    { 0, 0 },
    { 0x37112bc7, 3 },
    { 0, 0 },
    { 0x42c12bc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54992bd2, 3 },
    { 0xa3cd2bd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d2bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3d2be5, 3 },
    { 0x5f792be5, 4 },
    { 0x96bd2be7, 3 },
    { 0xc4712be8, 3 },
    { 0, 0 },
    { 0xd7192bea, 4 },
    { 0, 0 },
    { 0xd9012bec, 3 },
    { 0xec692bec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab252bf2, 3 },
    { 0xa12bf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282d2bf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd292bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf252c02, 3 },
    { 0xa8ed2c02, 5 },
    { 0x6cd92c04, 4 },
    { 0x6e092c03, 4 },
    { 0x67812c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e92c09, 3 },
    { 0, 0 },
    { 0x12752c0b, 3 },
    { 0, 0 },
    { 0xbca12c0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a12c11, 3 },
    { 0xc3cd2c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cd2c15, 3 },
    { 0x4cb92c16, 3 },
    { 0, 0 },
    { 0xf3dd2c18, 5 },
    { 0x80192c19, 3 },
    { 0x542d2c1a, 3 },
    { 0x730d2c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ed2c1f, 3 },
    { 0, 0 },
    { 0x87bd2c21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9052c38, 3 },
    { 0x38c12c39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f92c44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fd2c47, 3 },
    { 0x11652c48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d12c4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb352c51, 3 },
    { 0x48892c52, 4 },
    { 0x607d2c52, 3 },
    { 0x1c592c53, 3 },
    { 0x9d792c54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3312c59, 3 },
    { 0, 0 },
    { 0xeb812c5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c12c5f, 3 },
    { 0xe2952c60, 3 },
    { 0x1d2d2c61, 3 },
    { 0x7ccd2c62, 3 },
    { 0xdbc52c63, 3 },
    { 0x3c2d2c62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d2c67, 3 },
    { 0x95c52c68, 3 },
    { 0x21212c69, 3 },
    { 0x20e92c69, 3 },
    { 0x86552c6b, 3 },
    { 0xf9792c69, 3 },
    { 0xd4f92c6a, 3 },
    { 0xfd452c6e, 4 },
    { 0, 0 },
    { 0xb6452c70, 3 },
    { 0, 0 },
    { 0x77d12c72, 5 },
    { 0, 0 },
    { 0x13612c74, 3 },
    { 0, 0 },
    { 0xdd6d2c76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80292c7c, 3 },
    { 0, 0 },
    { 0xcb392c7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63852c83, 4 },
    { 0x8f092c83, 3 },
    { 0x1d812c83, 3 },
    { 0xf9352c86, 3 },
    { 0, 0 },
    { 0x2fe92c88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x606d2c8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b52c8f, 3 },
    { 0, 0 },
    { 0xfca12c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d92c96, 4 },
    { 0xb4512c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a92c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c52ca5, 3 },
    { 0, 0 },
    { 0x11b92ca7, 3 },
    { 0, 0 },
    { 0xe3ed2ca9, 4 },
    { 0xcd9d2caa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf012cad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad52cb5, 4 },
    { 0, 0 },
    { 0xe64d2cb7, 4 },
    { 0x939d2cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab52cc0, 3 },
    { 0xaa412cc1, 3 },
    { 0, 0 },
    { 0x15ad2cc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fd2cc6, 3 },
    { 0x7aad2cc6, 3 },
    { 0x32452cc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a12ccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae92cd3, 3 },
    { 0x9992cd4, 3 },
    { 0xe9192cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2692cd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3312cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3192ce4, 3 },
    { 0x4c652ce4, 3 },
    { 0x61912ce4, 4 },
    { 0x36312ce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78052cea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50612cf0, 3 },
    { 0x37852cf1, 3 },
    { 0x78e92cf1, 4 },
    { 0xa7ed2cf3, 3 },
    { 0x5bad2cf3, 4 },
    { 0x59812cf5, 4 },
    { 0x4b2d2cf1, 3 },
    { 0, 0 },
    { 0x6e712cf8, 3 },
    { 0xff2d2cf9, 4 },
    { 0x3d6d2cfa, 3 },
    { 0x7f812cf8, 3 },
    { 0, 0 },
    { 0x27712cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff52d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181d2d0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57992d12, 3 },
    { 0xdd692d13, 3 },
    { 0x182d2d13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82d2d18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34092d1b, 4 },
    { 0xefd52d1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36412d20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced2d27, 3 },
    { 0xf1052d27, 3 },
    { 0, 0 },
    { 0xa3dd2d2a, 3 },
    { 0, 0 },
    { 0xf56d2d2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce52d33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de92d3b, 3 },
    { 0x5dd12d3c, 4 },
    { 0, 0 },
    { 0x828d2d3e, 4 },
    { 0x78492d3f, 3 },
    { 0x17912d40, 4 },
    { 0xc6cd2d41, 3 },
    { 0x39052d40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccd2d4a, 4 },
    { 0x59152d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22512d4f, 3 },
    { 0x92412d4f, 5 },
    { 0x98612d50, 3 },
    { 0, 0 },
    { 0xa2ed2d53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60992d56, 3 },
    { 0x1c9d2d57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d2d5e, 3 },
    { 0x7ba92d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73552d63, 3 },
    { 0xa252d63, 3 },
    { 0, 0 },
    { 0xf39d2d66, 3 },
    { 0xa7652d67, 3 },
    { 0xd8c12d66, 3 },
    { 0, 0 },
    { 0x90b52d6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd2d6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac712d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cad2d74, 3 },
    { 0x56012d75, 3 },
    { 0x58212d76, 3 },
    { 0x9b5d2d77, 4 },
    { 0, 0 },
    { 0x6a652d79, 3 },
    { 0x6b52d79, 4 },
    { 0, 0 },
    { 0x60412d7c, 3 },
    { 0xc012d7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c92d84, 3 },
    { 0, 0 },
    { 0x58852d86, 4 },
    { 0, 0 },
    { 0xe3f92d88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafd2d8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99192d8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10312d98, 3 },
    { 0x53a92d99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e412d9c, 3 },
    { 0, 0 },
    { 0x6d52d9e, 3 },
    { 0xd4492d9e, 3 },
    { 0xdb912da0, 3 },
    { 0, 0 },
    { 0x7e92da2, 3 },
    { 0xb3a52da3, 3 },
    { 0x83ed2da3, 3 },
    { 0x5a012da2, 3 },
    { 0x8d392da6, 3 },
    { 0x86dd2da3, 3 },
    { 0x9b3d2da4, 3 },
    { 0xc7e52da9, 3 },
    { 0xe8292daa, 3 },
    { 0x145d2dab, 4 },
    { 0x72d2dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17d2daf, 3 },
    { 0x6ed2db0, 3 },
    { 0x9bd2db1, 4 },
    { 0x79fd2db2, 3 },
    { 0, 0 },
    { 0xd1892db4, 3 },
    { 0xe0bd2db5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44952dba, 3 },
    { 0xd392dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dbe, 3 },
    { 0x77512dbf, 3 },
    { 0, 0 },
    { 0x47912dc1, 3 },
    { 0xd71d2dc2, 3 },
    { 0xca4d2dc2, 4 },
    { 0, 0 },
    { 0xd0b52dc5, 3 },
    { 0, 0 },
    { 0x9f052dc7, 4 },
    { 0x6dc92dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d92dcd, 4 },
    { 0, 0 },
    { 0xe712dcf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74192dd3, 3 },
    { 0, 0 },
    { 0x4fd92dd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25652de0, 3 },
    { 0x986d2de1, 4 },
    { 0xc6192de0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2de6, 3 },
    { 0xe6092de6, 3 },
    { 0, 0 },
    { 0x7df52de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79212dec, 3 },
    { 0, 0 },
    { 0xc2e52dee, 3 },
    { 0x8ead2def, 5 },
    { 0x571d2df0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0412df8, 3 },
    { 0, 0 },
    { 0xa0552dfa, 3 },
    { 0x9d512dfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd2dfe, 3 },
    { 0x5b112dfe, 3 },
    { 0, 0 },
    { 0x6a692e01, 3 },
    { 0, 0 },
    { 0xd8e12e03, 3 },
    { 0, 0 },
    { 0xd1bd2e05, 3 },
    { 0, 0 },
    { 0x92c52e07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff12e0b, 3 },
    { 0xfa952e0c, 3 },
    { 0x7fb52e0d, 4 },
    { 0x1ec92e0e, 5 },
    { 0, 0 },
    { 0xe8ad2e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e852e16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf412e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd92e1d, 4 },
    { 0x260d2e1e, 3 },
    { 0xde0d2e1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41192e22, 3 },
    { 0, 0 },
    { 0x54292e24, 3 },
    { 0xb4a52e25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad12e2c, 3 },
    { 0, 0 },
    { 0xf02d2e2e, 3 },
    { 0, 0 },
    { 0x26912e30, 3 },
    { 0x7292e31, 3 },
    { 0xfde92e32, 3 },
    { 0x59752e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4852e36, 3 },
    { 0xe9c92e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21152e3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6512e3e, 4 },
    { 0, 0 },
    { 0x4f312e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89d2e4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd992e51, 3 },
    { 0x40412e51, 4 },
    { 0x9ed52e53, 3 },
    { 0x95652e53, 3 },
    { 0, 0 },
    { 0xb20d2e56, 3 },
    { 0, 0 },
    { 0xb0392e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd2e5b, 3 },
    { 0x32d12e5c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea92e60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2e65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d2e69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd2e6c, 3 },
    { 0, 0 },
    { 0x50e52e6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f52e71, 3 },
    { 0xf9b92e71, 4 },
    { 0xb192e71, 3 },
    { 0x8ef12e74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef92e7c, 3 },
    { 0x23b52e7d, 3 },
    { 0x2a52e7e, 3 },
    { 0xc80d2e7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e12e86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0992e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95512e8c, 4 },
    { 0, 0 },
    { 0x66892e8e, 3 },
    { 0, 0 },
    { 0x62692e90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb52e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5d2e9d, 4 },
    { 0x78692e9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3852ea2, 3 },
    { 0, 0 },
    { 0xdae52ea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c992ea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f92eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad12eb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d2ebe, 3 },
    { 0x608d2ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c552ec2, 4 },
    { 0xc2892ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52852ec9, 3 },
    { 0, 0 },
    { 0xa1552ecb, 4 },
    { 0xcd752ecc, 3 },
    { 0xca052ecb, 3 },
    { 0xf8c92ecc, 3 },
    { 0, 0 },
    { 0x88892ed0, 3 },
    { 0xfc512ed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a12ed4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cd2ed8, 5 },
    { 0x526d2ed9, 3 },
    { 0, 0 },
    { 0xbf712edb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce952edf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d12ee3, 3 },
    { 0x2d212ee4, 3 },
    { 0, 0 },
    { 0x30712ee6, 3 },
    { 0x7cf92ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed52eee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fd2ef3, 4 },
    { 0xe02d2ef4, 3 },
    { 0x62c12ef5, 3 },
    { 0x79412ef6, 4 },
    { 0, 0 },
    { 0x5b52ef8, 3 },
    { 0, 0 },
    { 0x62bd2efa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d452f01, 4 },
    { 0, 0 },
    { 0xf24d2f03, 3 },
    { 0, 0 },
    { 0xd1bd2f05, 4 },
    { 0xb3cd2f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70052f09, 4 },
    { 0x6f492f0a, 4 },
    { 0x55992f0b, 3 },
    { 0x33552f0c, 3 },
    { 0x53692f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7d2f17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b92f1a, 3 },
    { 0x12912f1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97d2f25, 3 },
    { 0xd7912f25, 4 },
    { 0xe9d2f27, 3 },
    { 0x49012f28, 4 },
    { 0, 0 },
    { 0x8dad2f2a, 3 },
    { 0, 0 },
    { 0x74e52f2c, 3 },
    { 0x46d92f2d, 3 },
    { 0x79412f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e612f31, 4 },
    { 0x667d2f32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716d2f36, 3 },
    { 0, 0 },
    { 0xea492f38, 4 },
    { 0, 0 },
    { 0xcf852f3a, 3 },
    { 0xd6012f3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e052f42, 3 },
    { 0xfc112f42, 3 },
    { 0xacb12f42, 3 },
    { 0, 0 },
    { 0x6cad2f46, 3 },
    { 0x3aa12f47, 3 },
    { 0x77c12f47, 5 },
    { 0x8b992f49, 4 },
    { 0, 0 },
    { 0x192f4b, 4 },
    { 0x2d492f4c, 3 },
    { 0x9bb92f4b, 4 },
    { 0, 0 },
    { 0x51b12f4f, 3 },
    { 0x17552f50, 3 },
    { 0xad912f51, 3 },
    { 0xd7e92f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50512f55, 3 },
    { 0, 0 },
    { 0xea752f57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb92f5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2f64, 3 },
    { 0x8ddd2f65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e312f6c, 3 },
    { 0, 0 },
    { 0x38b12f6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85512f73, 3 },
    { 0, 0 },
    { 0x1fc52f75, 3 },
    { 0, 0 },
    { 0xf7bd2f77, 3 },
    { 0xf7512f77, 3 },
    { 0, 0 },
    { 0x1b392f7a, 3 },
    { 0x9a212f7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4252f7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843d2f88, 3 },
    { 0xd4c52f89, 4 },
    { 0x5a852f8a, 3 },
    { 0xd21d2f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98692f91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49392f97, 5 },
    { 0xdabd2f98, 3 },
    { 0, 0 },
    { 0xeb652f9a, 3 },
    { 0x10d2f9b, 3 },
    { 0x14f92f9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27d2fad, 4 },
    { 0x88312fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd092fb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31892fb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac692fbc, 3 },
    { 0x2bbd2fbc, 4 },
    { 0x99bd2fbe, 3 },
    { 0xab292fbf, 3 },
    { 0x7892fbe, 3 },
    { 0x7ad92fc1, 3 },
    { 0x67a92fc2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2652fc6, 3 },
    { 0x157d2fc7, 4 },
    { 0x21d12fc7, 4 },
    { 0x1e792fc6, 3 },
    { 0x47292fc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704d2fcf, 4 },
    { 0, 0 },
    { 0xb1352fd1, 3 },
    { 0, 0 },
    { 0x5b212fd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd752fd7, 3 },
    { 0x6e992fd8, 4 },
    { 0, 0 },
    { 0xae9d2fda, 4 },
    { 0, 0 },
    { 0xe9412fdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d2fe0, 3 },
    { 0, 0 },
    { 0x7f452fe2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8592fe7, 3 },
    { 0x99192fe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41852feb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c52fef, 3 },
    { 0, 0 },
    { 0xd0c12ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0d2ff4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd2ff9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd52ffd, 4 },
    { 0x19a12ffd, 3 },
    { 0, 0 },
    { 0xa8713000, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2013005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf679300a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab71300e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b453014, 3 },
    { 0, 0 },
    { 0xbaf53016, 4 },
    { 0x9c3d3016, 3 },
    { 0xbe093016, 4 },
    { 0xac653019, 3 },
    { 0xe0d5301a, 3 },
    { 0xd189301b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8553020, 3 },
    { 0, 0 },
    { 0x31713022, 3 },
    { 0x39d13023, 3 },
    { 0xdc7d3024, 3 },
    { 0x53913024, 3 },
    { 0x25493026, 3 },
    { 0, 0 },
    { 0xbcd3028, 3 },
    { 0x41a93029, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a31302d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a13031, 3 },
    { 0xec293032, 3 },
    { 0x9e813032, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8355303a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3971303d, 3 },
    { 0x6d69303d, 3 },
    { 0x2fdd303e, 3 },
    { 0xb9d1303e, 4 },
    { 0, 0 },
    { 0xe9853042, 3 },
    { 0x58753043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b53046, 3 },
    { 0x84bd3047, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d3053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59953056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e1305e, 3 },
    { 0xf829305f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20613062, 3 },
    { 0x8ca13063, 3 },
    { 0x43f53063, 3 },
    { 0, 0 },
    { 0x8893066, 3 },
    { 0xabf53067, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811306f, 3 },
    { 0x7c6d3070, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53393076, 3 },
    { 0xed593076, 3 },
    { 0, 0 },
    { 0x199d3079, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9765307f, 3 },
    { 0xb185307f, 3 },
    { 0x92fd3081, 3 },
    { 0x2fbd307f, 3 },
    { 0, 0 },
    { 0x20193084, 3 },
    { 0, 0 },
    { 0xfba13086, 3 },
    { 0xf1bd3086, 4 },
    { 0, 0 },
    { 0xb13d3089, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d293091, 3 },
    { 0xc48d3092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1d3097, 4 },
    { 0x2a5d3098, 3 },
    { 0x46fd3097, 4 },
    { 0, 0 },
    { 0x144d309b, 4 },
    { 0x6229309b, 4 },
    { 0xa59309c, 3 },
    { 0, 0 },
    { 0x6439309f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe130aa, 4 },
    { 0, 0 },
    { 0x38d530ac, 4 },
    { 0, 0 },
    { 0x430d30ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4d30b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c530b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccd30bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d30bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93930c2, 3 },
    { 0x457530c3, 4 },
    { 0x97f930c3, 3 },
    { 0xd27d30c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2130c9, 3 },
    { 0xf96d30ca, 4 },
    { 0x7a930ca, 3 },
    { 0x85e130cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d30cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e530d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bd30d6, 3 },
    { 0x156d30d7, 4 },
    { 0x5b9d30d8, 4 },
    { 0x89130d7, 3 },
    { 0xcdf530d8, 3 },
    { 0x801130db, 4 },
    { 0x63b930da, 3 },
    { 0xcd0d30dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e930e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3530e5, 3 },
    { 0xb63930e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1930e9, 3 },
    { 0x2be130ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa130ed, 4 },
    { 0xa06130ed, 3 },
    { 0x26b530ef, 4 },
    { 0xc17530f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629530f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x305530f8, 3 },
    { 0xf67530f9, 3 },
    { 0xb66d30f8, 3 },
    { 0xf6b130fb, 4 },
    { 0x260930fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e53108, 3 },
    { 0, 0 },
    { 0x438d310a, 3 },
    { 0x9929310a, 3 },
    { 0x16fd310c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d913112, 4 },
    { 0x46013113, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b013118, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5375311b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abd311e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff13121, 3 },
    { 0, 0 },
    { 0xce2d3123, 3 },
    { 0x63b53123, 3 },
    { 0x69853124, 3 },
    { 0x1a93123, 3 },
    { 0x17293127, 3 },
    { 0, 0 },
    { 0x6bf13129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebd312d, 3 },
    { 0x4e85312e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d3133, 3 },
    { 0, 0 },
    { 0xeae53135, 3 },
    { 0x6bed3136, 3 },
    { 0, 0 },
    { 0x114d3138, 4 },
    { 0xc9dd3139, 4 },
    { 0, 0 },
    { 0x7b5313b, 3 },
    { 0x7801313c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf93142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d3146, 4 },
    { 0xd3153146, 3 },
    { 0xa2013146, 4 },
    { 0, 0 },
    { 0xcb2d314a, 4 },
    { 0x6055314b, 4 },
    { 0x2b69314c, 4 },
    { 0xd4c9314c, 4 },
    { 0xb81d314c, 3 },
    { 0, 0 },
    { 0x39c53150, 3 },
    { 0x31cd3151, 3 },
    { 0x349d3152, 4 },
    { 0, 0 },
    { 0x95753154, 3 },
    { 0, 0 },
    { 0x24c53156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531315f, 3 },
    { 0x64453160, 4 },
    { 0, 0 },
    { 0x4e013162, 3 },
    { 0x2d893163, 3 },
    { 0x28053162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851316a, 4 },
    { 0x2449316b, 3 },
    { 0, 0 },
    { 0x5db9316d, 3 },
    { 0x5825316d, 3 },
    { 0x2df1316f, 3 },
    { 0xaf91316d, 4 },
    { 0x40653170, 5 },
    { 0x4b6d3172, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec613183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c5318a, 3 },
    { 0x18dd318a, 3 },
    { 0x9e69318c, 3 },
    { 0, 0 },
    { 0x76e9318e, 3 },
    { 0x5ca1318f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681319a, 3 },
    { 0x3fa9319b, 3 },
    { 0x98b1319b, 3 },
    { 0x1a51319a, 3 },
    { 0xb9d1319e, 3 },
    { 0, 0 },
    { 0x28931a0, 3 },
    { 0xd06931a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74131a5, 4 },
    { 0, 0 },
    { 0x5ef531a7, 3 },
    { 0xd59d31a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5131b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d31b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978d31ba, 3 },
    { 0xdedd31bb, 3 },
    { 0, 0 },
    { 0x3da931bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23931c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c931c8, 3 },
    { 0, 0 },
    { 0x78f931ca, 3 },
    { 0xba2d31cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39531d0, 3 },
    { 0xff0d31d1, 3 },
    { 0x16c531d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38531d5, 3 },
    { 0x230531d6, 3 },
    { 0x412131d7, 3 },
    { 0x98d531d7, 3 },
    { 0x426531d7, 3 },
    { 0x57e931da, 4 },
    { 0x3eed31db, 3 },
    { 0, 0 },
    { 0x946d31dd, 3 },
    { 0, 0 },
    { 0x629931df, 4 },
    { 0x753931e0, 3 },
    { 0x687d31e1, 3 },
    { 0x3be531e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4131e7, 4 },
    { 0, 0 },
    { 0xddfd31e9, 4 },
    { 0, 0 },
    { 0xa45131eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181131ef, 4 },
    { 0, 0 },
    { 0xd0e531f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5d31f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c131f9, 3 },
    { 0x49d931fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x407131fd, 3 },
    { 0xb8b931fe, 3 },
    { 0x5a6531ff, 3 },
    { 0x13493200, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39493204, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5213207, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e1320a, 3 },
    { 0xfcb9320b, 3 },
    { 0x9b5320c, 3 },
    { 0x4fd5320d, 4 },
    { 0xf469320d, 3 },
    { 0xcfad320f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa613215, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3395321e, 4 },
    { 0xc05d321f, 3 },
    { 0x8dad3220, 3 },
    { 0x6b69321e, 3 },
    { 0xa891321f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4913225, 3 },
    { 0xc6f13225, 3 },
    { 0xaa553227, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81322b, 3 },
    { 0x8c9322c, 4 },
    { 0x6b89322d, 3 },
    { 0x91d9322d, 4 },
    { 0xd7ad322c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f53235, 4 },
    { 0xe36d3236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a13239, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d323f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4d3245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a253249, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2245324d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5693251, 3 },
    { 0, 0 },
    { 0xbfe13253, 3 },
    { 0, 0 },
    { 0x16813255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60d3259, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c1325e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a753264, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb793268, 4 },
    { 0, 0 },
    { 0x736d326a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19813270, 3 },
    { 0x74913271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d3274, 4 },
    { 0x25ad3274, 4 },
    { 0, 0 },
    { 0x231d3277, 4 },
    { 0x490d3277, 3 },
    { 0xe9a53279, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd53283, 3 },
    { 0xf4f53283, 4 },
    { 0, 0 },
    { 0xfca13286, 5 },
    { 0x2e3d3287, 3 },
    { 0x95bd3288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf5328d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d053295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a853298, 4 },
    { 0xb113298, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb521329c, 3 },
    { 0x5711329c, 3 },
    { 0, 0 },
    { 0x3f55329f, 3 },
    { 0, 0 },
    { 0xe13132a1, 4 },
    { 0xc5c132a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x132132a5, 5 },
    { 0x1b3d32a6, 5 },
    { 0xff3d32a5, 3 },
    { 0xe77d32a8, 4 },
    { 0, 0 },
    { 0xabf932aa, 3 },
    { 0x66532ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec532ae, 3 },
    { 0, 0 },
    { 0x4a4932b0, 3 },
    { 0xa7132b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5132b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x739532b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x904532ba, 4 },
    { 0, 0 },
    { 0x5b3132bc, 4 },
    { 0, 0 },
    { 0x8a3d32be, 3 },
    { 0, 0 },
    { 0xfa8d32c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5132c3, 3 },
    { 0x17b132c4, 3 },
    { 0x495132c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f932cb, 3 },
    { 0x7d9932cb, 5 },
    { 0xa55132cc, 4 },
    { 0xa66132ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13932d4, 3 },
    { 0, 0 },
    { 0xb61932d6, 4 },
    { 0, 0 },
    { 0x449132d8, 4 },
    { 0x60fd32d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b132dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0d32e1, 3 },
    { 0x654d32e2, 3 },
    { 0, 0 },
    { 0xb86532e4, 3 },
    { 0x6f4d32e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8132e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d32f1, 3 },
    { 0x519532f2, 3 },
    { 0xba1532f3, 3 },
    { 0xb8cd32f2, 4 },
    { 0xab4132f5, 3 },
    { 0xca3932f6, 5 },
    { 0xe28d32f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40932fd, 3 },
    { 0x637932fe, 3 },
    { 0x67932ff, 4 },
    { 0, 0 },
    { 0xfb653301, 4 },
    { 0x29b53302, 3 },
    { 0xf0a53303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1330a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bd330e, 3 },
    { 0, 0 },
    { 0x29b13310, 3 },
    { 0xc813311, 4 },
    { 0x3f793312, 3 },
    { 0xc613313, 3 },
    { 0x5d753311, 4 },
    { 0xa4613312, 3 },
    { 0x69093312, 3 },
    { 0x28d13317, 4 },
    { 0xf8293318, 3 },
    { 0xd0053318, 3 },
    { 0x9cc5331a, 3 },
    { 0xe813315, 3 },
    { 0xb8a9331c, 3 },
    { 0x5831331c, 3 },
    { 0x75613314, 3 },
    { 0x994d331a, 3 },
    { 0, 0 },
    { 0x52d53321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee53325, 3 },
    { 0, 0 },
    { 0xa85d3327, 3 },
    { 0xb4713327, 3 },
    { 0x606d3329, 3 },
    { 0x75693329, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c13333, 4 },
    { 0, 0 },
    { 0x7093335, 4 },
    { 0x73a53335, 3 },
    { 0, 0 },
    { 0xeb3d3338, 3 },
    { 0x36153339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f01333d, 3 },
    { 0x2f01333d, 3 },
    { 0, 0 },
    { 0xb1ed3340, 4 },
    { 0x2d3d3341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd3344, 3 },
    { 0x53cd3345, 3 },
    { 0, 0 },
    { 0xa3ad3347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad334d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609d3351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab213358, 3 },
    { 0, 0 },
    { 0x88cd335a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5571335d, 3 },
    { 0x5ca5335e, 3 },
    { 0x426d335e, 5 },
    { 0xd6dd335e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d53364, 3 },
    { 0xc5993364, 3 },
    { 0x34113366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28353371, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf413375, 3 },
    { 0x68493376, 4 },
    { 0xcdd3377, 3 },
    { 0x3a993378, 3 },
    { 0x1d193378, 3 },
    { 0, 0 },
    { 0x155d337b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b93380, 3 },
    { 0xba53381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33353388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4079338e, 4 },
    { 0xa439338f, 4 },
    { 0x68753390, 3 },
    { 0xaa0d338f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2555339e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853933a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07133a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d33ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a133b3, 3 },
    { 0xe2d33b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x401933b7, 3 },
    { 0, 0 },
    { 0x2ead33b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x405533bc, 4 },
    { 0, 0 },
    { 0xa4f533be, 3 },
    { 0x6efd33bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6933c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958533c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133d0, 3 },
    { 0, 0 },
    { 0xea5533d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2533d6, 4 },
    { 0x8b6133d7, 3 },
    { 0xb0b533d7, 3 },
    { 0x370133d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f133dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf533e1, 3 },
    { 0x228933e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3533e5, 3 },
    { 0x12cd33e6, 3 },
    { 0x545933e5, 3 },
    { 0, 0 },
    { 0x63a933e9, 3 },
    { 0x905133e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x948533ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5533f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d933f9, 5 },
    { 0x76c133fa, 3 },
    { 0xdd6933fb, 3 },
    { 0, 0 },
    { 0x667933fd, 3 },
    { 0xcdf133fe, 4 },
    { 0x684133fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9753402, 4 },
    { 0xbf253403, 3 },
    { 0, 0 },
    { 0x12a53405, 3 },
    { 0, 0 },
    { 0xed3407, 3 },
    { 0xce53407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3595340b, 3 },
    { 0x32e9340b, 3 },
    { 0x4529340c, 3 },
    { 0xfd6d340b, 3 },
    { 0x3591340c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cd3412, 3 },
    { 0xbf853412, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21d3416, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d93419, 3 },
    { 0x9d55341a, 3 },
    { 0, 0 },
    { 0xabf1341c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd3421, 4 },
    { 0, 0 },
    { 0xa1913423, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f53426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01342b, 3 },
    { 0x1dad342b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a13433, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba61343a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd479343f, 3 },
    { 0x49493440, 3 },
    { 0x89313441, 3 },
    { 0x93ed3442, 3 },
    { 0x415d3443, 3 },
    { 0xe7453443, 4 },
    { 0x7a053440, 3 },
    { 0xccdd3444, 4 },
    { 0xd1413446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9709344a, 3 },
    { 0, 0 },
    { 0xb7a1344c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2693450, 4 },
    { 0x3f693451, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e793456, 3 },
    { 0, 0 },
    { 0x41b13458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de5345b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ed345e, 3 },
    { 0xa8dd345f, 3 },
    { 0x62b5345f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5353468, 3 },
    { 0, 0 },
    { 0xb19d346a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea71346e, 3 },
    { 0x62b5346f, 3 },
    { 0x1a35346e, 3 },
    { 0x7e61346f, 3 },
    { 0, 0 },
    { 0xef493473, 3 },
    { 0xeba93474, 3 },
    { 0xf6313475, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a693480, 3 },
    { 0xf5653481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd013489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71093493, 4 },
    { 0xb9553493, 4 },
    { 0x2fc93493, 3 },
    { 0x2a9d3496, 3 },
    { 0xddc93494, 3 },
    { 0x9ee13498, 3 },
    { 0xb4cd3499, 4 },
    { 0x35213493, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd134a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8934a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x881534a7, 3 },
    { 0, 0 },
    { 0x83d534a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x602534ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749d34b3, 3 },
    { 0, 0 },
    { 0xdf4134b5, 4 },
    { 0xd58134b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3934b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88534bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487534c1, 3 },
    { 0xe9134c1, 3 },
    { 0xeabd34c3, 3 },
    { 0, 0 },
    { 0x218934c5, 3 },
    { 0x9a9d34c5, 4 },
    { 0x4ed534c7, 3 },
    { 0xbd1d34c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d134cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab934d1, 4 },
    { 0x7e6d34d2, 3 },
    { 0x1ec134d2, 3 },
    { 0x5b3d34d1, 3 },
    { 0x22f934d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ad34dc, 3 },
    { 0x713134dd, 4 },
    { 0xac7934de, 3 },
    { 0x68ad34de, 3 },
    { 0, 0 },
    { 0xc18534e1, 3 },
    { 0x5c4d34e1, 4 },
    { 0xdec134e1, 3 },
    { 0xc97134e2, 3 },
    { 0x2f0534e4, 4 },
    { 0x590134e6, 3 },
    { 0x763534e2, 3 },
    { 0x8a134e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68934ed, 3 },
    { 0, 0 },
    { 0xcbd934ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35534f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74534fb, 3 },
    { 0, 0 },
    { 0x2c6534fd, 3 },
    { 0, 0 },
    { 0xba9d34ff, 4 },
    { 0, 0 },
    { 0x27e53501, 3 },
    { 0xc4413502, 3 },
    { 0xb68d3502, 4 },
    { 0x6b853504, 3 },
    { 0xbf5d3504, 3 },
    { 0x90453506, 3 },
    { 0, 0 },
    { 0x94953508, 3 },
    { 0, 0 },
    { 0xd39350a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd1350f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4293517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1351d, 4 },
    { 0x9589351d, 3 },
    { 0, 0 },
    { 0xdf653520, 3 },
    { 0, 0 },
    { 0x291d3522, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9d3526, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c93529, 3 },
    { 0, 0 },
    { 0xfbb9352b, 3 },
    { 0x5525352b, 3 },
    { 0, 0 },
    { 0xe0b9352e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e93532, 3 },
    { 0xb5f93532, 3 },
    { 0xa58d3534, 3 },
    { 0x4f193535, 4 },
    { 0, 0 },
    { 0x55713537, 3 },
    { 0xdfe13538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad353b, 4 },
    { 0, 0 },
    { 0xcc6d353d, 3 },
    { 0xcbd353e, 3 },
    { 0xb235353d, 4 },
    { 0, 0 },
    { 0xaa313541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5893544, 5 },
    { 0x16c13544, 3 },
    { 0xf9bd3546, 3 },
    { 0x8a553546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45d354b, 3 },
    { 0x1ad354b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a753550, 3 },
    { 0x84993550, 4 },
    { 0xcd7d3552, 4 },
    { 0, 0 },
    { 0xac153554, 3 },
    { 0x2ab53555, 3 },
    { 0x4f253556, 4 },
    { 0xe9f93557, 3 },
    { 0x2d8d3556, 4 },
    { 0x50393559, 3 },
    { 0xea59355a, 3 },
    { 0x9469355b, 3 },
    { 0x3b21355b, 3 },
    { 0xc225355d, 4 },
    { 0x94f1355d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf93568, 3 },
    { 0, 0 },
    { 0x75e9356a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd681356e, 3 },
    { 0xb41356f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36613573, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd3576, 4 },
    { 0, 0 },
    { 0x8753578, 3 },
    { 0x2c353579, 3 },
    { 0xc9853579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5357d, 3 },
    { 0, 0 },
    { 0xb201357f, 5 },
    { 0, 0 },
    { 0x8313581, 4 },
    { 0, 0 },
    { 0x9a13583, 3 },
    { 0xa6e93583, 4 },
    { 0x30113584, 4 },
    { 0, 0 },
    { 0x58553587, 4 },
    { 0, 0 },
    { 0x20d53589, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c5358c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8593599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7955359d, 4 },
    { 0x44c1359e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3d35a5, 3 },
    { 0xded135a6, 4 },
    { 0x66fd35a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601535ad, 3 },
    { 0xa1a935ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d35b9, 3 },
    { 0x14ed35ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x954d35bd, 3 },
    { 0xdbf535be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a935c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d35c5, 3 },
    { 0xc51135c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663935ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957d35cf, 3 },
    { 0x70dd35d0, 3 },
    { 0x332935d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429d35d5, 3 },
    { 0x109535d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d35d9, 3 },
    { 0, 0 },
    { 0x8e3135db, 4 },
    { 0, 0 },
    { 0x871935dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994d35e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185935ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb535f2, 3 },
    { 0, 0 },
    { 0x3c3935f4, 3 },
    { 0, 0 },
    { 0xf69d35f6, 3 },
    { 0xe16d35f7, 4 },
    { 0xbc0d35f6, 4 },
    { 0x9a4935f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20d35fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740d3603, 3 },
    { 0x674d3604, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b93607, 3 },
    { 0x45453608, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1d3611, 3 },
    { 0xb0093612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d93615, 4 },
    { 0, 0 },
    { 0xdcc53617, 3 },
    { 0, 0 },
    { 0x7bf13619, 3 },
    { 0x3149361a, 3 },
    { 0xd1b5361b, 3 },
    { 0xd191361b, 3 },
    { 0x86a1361d, 4 },
    { 0x3049361e, 3 },
    { 0x6935361e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11713623, 3 },
    { 0xac0d3624, 3 },
    { 0xc9713624, 4 },
    { 0xe6513626, 3 },
    { 0xe5a93627, 4 },
    { 0xcc693628, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d93632, 3 },
    { 0, 0 },
    { 0x65b53634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c5363a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd59363e, 3 },
    { 0, 0 },
    { 0x38dd3640, 3 },
    { 0x3e413641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ed364a, 3 },
    { 0x354d364b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5313651, 3 },
    { 0x66a13652, 3 },
    { 0x411d3651, 3 },
    { 0, 0 },
    { 0x18913655, 3 },
    { 0, 0 },
    { 0x70893657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb909365b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5671365e, 3 },
    { 0x4205365e, 3 },
    { 0x884d3660, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e813663, 3 },
    { 0x66513664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20213672, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ded3675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff95367a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d367e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1213683, 3 },
    { 0x727d3684, 3 },
    { 0x7cdd3683, 3 },
    { 0x387d3686, 4 },
    { 0x733d3684, 3 },
    { 0, 0 },
    { 0x89153689, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3629368d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x509d3690, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x64513693, 4 },
    { 0x85e93694, 3 },
    { 0, 0 },
    { 0xbbc53696, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe51369a, 3 },
    { 0x5945369b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf355369e, 3 },
    { 0x8fd5369e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6d36a5, 3 },
    { 0xccd136a6, 3 },
    { 0xe6cd36a7, 3 },
    { 0x646536a8, 3 },
    { 0x4a5536a9, 3 },
    { 0x16bd36a9, 3 },
    { 0xd51d36ab, 3 },
    { 0xd59936ac, 3 },
    { 0x1d8136ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8936b6, 4 },
    { 0x212d36b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d536ba, 3 },
    { 0x117136bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70d36c3, 3 },
    { 0x65f936c4, 3 },
    { 0x36ad36c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1536c8, 3 },
    { 0x9fc936c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d536cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180d36d0, 3 },
    { 0, 0 },
    { 0xb7136d2, 4 },
    { 0xff8536d3, 3 },
    { 0, 0 },
    { 0x339136d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba936da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d536dd, 3 },
    { 0xbed36dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b136e5, 3 },
    { 0x1b7936e5, 3 },
    { 0xa10536e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13936ed, 4 },
    { 0x7e9536ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b536f8, 3 },
    { 0x429936f8, 3 },
    { 0xb73936fa, 3 },
    { 0x504936fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835536ff, 3 },
    { 0xc2d13700, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b313703, 3 },
    { 0xbf7d3704, 3 },
    { 0xfeb53705, 3 },
    { 0x7a953705, 3 },
    { 0x99553705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d370e, 3 },
    { 0x9245370e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d753712, 3 },
    { 0x1a253712, 4 },
    { 0x71593713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da1371c, 3 },
    { 0, 0 },
    { 0x6a45371e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12913722, 4 },
    { 0x45c93722, 4 },
    { 0xd0213724, 3 },
    { 0, 0 },
    { 0x84453726, 3 },
    { 0xb6d3727, 3 },
    { 0xb9553726, 3 },
    { 0xb60d3729, 3 },
    { 0x7f9d3729, 3 },
    { 0xdffd3727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f93733, 4 },
    { 0, 0 },
    { 0x2b6d3735, 3 },
    { 0, 0 },
    { 0x8f0d3737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f293749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc79374c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b93750, 3 },
    { 0xded53751, 3 },
    { 0, 0 },
    { 0xb4753753, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac893759, 3 },
    { 0xe935375a, 3 },
    { 0x54e9375a, 3 },
    { 0xabe5375a, 5 },
    { 0x6a8d375d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f93767, 3 },
    { 0xc25d3768, 4 },
    { 0xd4993767, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d25376f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453772, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f53777, 4 },
    { 0xc4d3778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x349377b, 4 },
    { 0xa5e1377c, 3 },
    { 0, 0 },
    { 0xf571377e, 3 },
    { 0x9609377f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d378a, 3 },
    { 0, 0 },
    { 0xaa99378c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9645378f, 4 },
    { 0xc46d3790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b53794, 3 },
    { 0, 0 },
    { 0x57a13796, 3 },
    { 0, 0 },
    { 0xa1f13798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fd379b, 3 },
    { 0xbde1379c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8d379f, 3 },
    { 0, 0 },
    { 0xa37937a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c537a4, 3 },
    { 0, 0 },
    { 0x347537a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba937ae, 3 },
    { 0x944d37af, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f137b7, 4 },
    { 0x531d37b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32d37be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e137c1, 3 },
    { 0x8f3137c1, 3 },
    { 0x2fc937c2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a937c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01d37cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63137d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c937d5, 4 },
    { 0x2db137d6, 4 },
    { 0, 0 },
    { 0x9eb537d8, 3 },
    { 0x63a937d9, 3 },
    { 0xd5d537da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dad37de, 3 },
    { 0xe96137df, 3 },
    { 0xbc8537e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fd37e9, 3 },
    { 0x177d37ea, 3 },
    { 0xd7f937ea, 3 },
    { 0x389d37eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db937ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cd37fa, 3 },
    { 0x282537fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9937fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89693809, 3 },
    { 0, 0 },
    { 0x6f69380b, 3 },
    { 0x6739380c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6d3810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2193813, 4 },
    { 0x68413814, 3 },
    { 0x1ded3815, 3 },
    { 0, 0 },
    { 0xa1813817, 4 },
    { 0x28b13818, 3 },
    { 0xe37d3818, 3 },
    { 0, 0 },
    { 0xd1cd381b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3139381f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d3824, 3 },
    { 0, 0 },
    { 0xc8453826, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6365382f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0153836, 3 },
    { 0, 0 },
    { 0x2da53838, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7d383b, 3 },
    { 0xb94d383b, 5 },
    { 0x5b8d383d, 4 },
    { 0x54c5383b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f413842, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab53851, 4 },
    { 0x1cf53852, 3 },
    { 0xadfd3852, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f13859, 3 },
    { 0x8b3d385a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d3860, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a93864, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d386d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21013872, 3 },
    { 0, 0 },
    { 0x9b753874, 4 },
    { 0x27413875, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06d3878, 3 },
    { 0, 0 },
    { 0xe239387a, 4 },
    { 0xe719387a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6101387e, 3 },
    { 0x9fd5387f, 4 },
    { 0x4769387f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d3889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd388d, 4 },
    { 0xeeed388d, 3 },
    { 0xa85d388d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f13894, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c53898, 3 },
    { 0x20553898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d5389c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3938a2, 3 },
    { 0x1c6138a2, 3 },
    { 0xf44d38a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x557938a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee538aa, 3 },
    { 0xb0e538ab, 3 },
    { 0x220538ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7138b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x124138b5, 3 },
    { 0, 0 },
    { 0xdec538b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0d38bb, 3 },
    { 0x74a538bb, 4 },
    { 0x48e538bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34d38c3, 3 },
    { 0x779138c4, 4 },
    { 0xead538c5, 3 },
    { 0x15a938c6, 3 },
    { 0, 0 },
    { 0x4b6138c8, 4 },
    { 0x62f938c9, 3 },
    { 0x8fd38c8, 3 },
    { 0xa45938cb, 4 },
    { 0xeaed38cb, 3 },
    { 0, 0 },
    { 0x1f5138ce, 3 },
    { 0, 0 },
    { 0x6c6138d0, 3 },
    { 0xb1ad38d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6938d8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52538db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d38df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae538e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b138e8, 3 },
    { 0x6c8938e9, 4 },
    { 0xa52d38ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56138ed, 3 },
    { 0xd54938ee, 3 },
    { 0xb17938ef, 4 },
    { 0xe9ad38ee, 3 },
    { 0xc82938f1, 4 },
    { 0x212938f1, 3 },
    { 0xcffd38ed, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x884d38f6, 3 },
    { 0xb2e938f7, 3 },
    { 0xab6d38f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d38fc, 3 },
    { 0x82a938fd, 3 },
    { 0x19a938fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fd3903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb93907, 3 },
    { 0xcaf53907, 3 },
    { 0xcae93909, 3 },
    { 0, 0 },
    { 0xd20d390b, 3 },
    { 0xa8f1390c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d3911, 3 },
    { 0x74853911, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e13915, 3 },
    { 0, 0 },
    { 0x6a693917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x171391a, 4 },
    { 0x1ae9391b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc9391f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2369392c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b713930, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64d393a, 4 },
    { 0xc431393b, 3 },
    { 0xff09393c, 3 },
    { 0xe189393a, 3 },
    { 0x2285393e, 3 },
    { 0x6101393f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e93943, 3 },
    { 0, 0 },
    { 0x43053945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237d3949, 4 },
    { 0, 0 },
    { 0xe55394b, 3 },
    { 0x233d394c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fd3951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a53955, 3 },
    { 0xe4953955, 3 },
    { 0xbadd3955, 3 },
    { 0x8fc53956, 3 },
    { 0xdaed3959, 3 },
    { 0xc2d53958, 5 },
    { 0, 0 },
    { 0x803d395c, 3 },
    { 0, 0 },
    { 0x5d45395e, 3 },
    { 0x7869395f, 3 },
    { 0, 0 },
    { 0x4e193961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa493964, 3 },
    { 0x22c93964, 5 },
    { 0x1c753964, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2465396f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8153972, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b31397c, 4 },
    { 0, 0 },
    { 0xece5397e, 3 },
    { 0, 0 },
    { 0x28393980, 3 },
    { 0x3f6d3981, 3 },
    { 0x43513982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e453985, 4 },
    { 0xa21d3986, 3 },
    { 0, 0 },
    { 0xef693988, 4 },
    { 0x51693988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa691398d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4813991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b393994, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64693997, 3 },
    { 0xd5f93998, 3 },
    { 0x9e6d3997, 3 },
    { 0xd49399a, 3 },
    { 0xcdf9399a, 3 },
    { 0, 0 },
    { 0xe8d1399d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed139a0, 5 },
    { 0x2b7939a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528939a6, 3 },
    { 0x6ff939a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fd39af, 3 },
    { 0, 0 },
    { 0x115139b1, 3 },
    { 0, 0 },
    { 0xf9f539b3, 3 },
    { 0xeca939b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398539b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b139c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d39ca, 3 },
    { 0x455d39cb, 3 },
    { 0, 0 },
    { 0x23d939cd, 3 },
    { 0x330d39ce, 3 },
    { 0xf8fd39cf, 3 },
    { 0, 0 },
    { 0xeef139d1, 3 },
    { 0x24e539d2, 3 },
    { 0x91e539d3, 3 },
    { 0xcb3939d4, 3 },
    { 0x2c7539d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da139d8, 4 },
    { 0xbac139d9, 3 },
    { 0x2e4d39d8, 3 },
    { 0xc30539d9, 3 },
    { 0x632139dc, 3 },
    { 0x309139dd, 4 },
    { 0x237939de, 3 },
    { 0xdb0139d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0939e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0539ec, 3 },
    { 0x272139ed, 3 },
    { 0x13f539ec, 3 },
    { 0xa1b539ef, 3 },
    { 0xca8939ec, 3 },
    { 0x595139f1, 4 },
    { 0xf64539f0, 4 },
    { 0xf5ed39f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda939f6, 3 },
    { 0xd57539f6, 3 },
    { 0x2fb939f6, 3 },
    { 0xa6d539f8, 3 },
    { 0x691939fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x611139fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24753a00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde53a04, 3 },
    { 0, 0 },
    { 0xa8193a06, 3 },
    { 0, 0 },
    { 0x5d393a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2413a0c, 3 },
    { 0x87b53a0d, 3 },
    { 0xdc353a0e, 4 },
    { 0x38a93a0e, 4 },
    { 0xb853a10, 3 },
    { 0x313d3a10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bd3a15, 3 },
    { 0, 0 },
    { 0x30553a17, 3 },
    { 0xcb13a18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783d3a1c, 4 },
    { 0, 0 },
    { 0xe0153a1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c313a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9653a28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c453a2c, 3 },
    { 0, 0 },
    { 0xc1313a2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb613a32, 5 },
    { 0xc3413a33, 4 },
    { 0, 0 },
    { 0x57193a35, 3 },
    { 0xa4853a36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd3a39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f593a3c, 4 },
    { 0x64cd3a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ad3a40, 4 },
    { 0, 0 },
    { 0x30253a42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd3a45, 4 },
    { 0x18353a46, 3 },
    { 0xb6b53a47, 3 },
    { 0xf693a47, 3 },
    { 0x3dd3a49, 3 },
    { 0x35013a4a, 3 },
    { 0xe1513a4a, 3 },
    { 0xe0a13a49, 3 },
    { 0xfa293a4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6513a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd213a5a, 3 },
    { 0x223d3a5b, 3 },
    { 0x735d3a5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59113a60, 3 },
    { 0x87e93a61, 3 },
    { 0x15793a62, 3 },
    { 0xeb113a61, 3 },
    { 0x6d613a64, 3 },
    { 0xe2013a64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5d3a71, 3 },
    { 0, 0 },
    { 0x47953a73, 4 },
    { 0, 0 },
    { 0xfb653a75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f913a7d, 3 },
    { 0x7dd3a7e, 3 },
    { 0x38a53a7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd13a8a, 3 },
    { 0xe78d3a8b, 4 },
    { 0x4c653a8c, 3 },
    { 0x15a93a8d, 3 },
    { 0x6b13a8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23193a93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5653a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed393aa4, 4 },
    { 0x31393aa4, 4 },
    { 0, 0 },
    { 0x54bd3aa7, 3 },
    { 0, 0 },
    { 0xea253aa9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ad3aad, 3 },
    { 0xa4393aad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2713ab3, 4 },
    { 0, 0 },
    { 0xfd993ab5, 3 },
    { 0xb8a13ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89993abc, 3 },
    { 0x33393abd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22293ac1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e13ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53853ac8, 3 },
    { 0x803d3ac9, 3 },
    { 0, 0 },
    { 0x7b613acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f53acf, 4 },
    { 0xe2593acf, 3 },
    { 0xfa913ad1, 3 },
    { 0x73913ad1, 3 },
    { 0x65a53ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd293ad6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b13add, 4 },
    { 0x9a613add, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b293ae1, 3 },
    { 0xf7613ae2, 3 },
    { 0x5ba53ae3, 3 },
    { 0x1e913ae4, 5 },
    { 0xd0f93ae5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1d3ae8, 3 },
    { 0x4e253ae9, 3 },
    { 0x76dd3ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c553aed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67813af2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a53af7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe13afb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b53b0b, 3 },
    { 0xa0fd3b0c, 3 },
    { 0xd1813b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db93b11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3113b19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc013b1c, 4 },
    { 0x42893b1d, 3 },
    { 0x224d3b1d, 4 },
    { 0x978d3b1f, 3 },
    { 0xfc913b1d, 3 },
    { 0xe393b20, 5 },
    { 0, 0 },
    { 0xa2753b23, 3 },
    { 0xe4dd3b23, 3 },
    { 0, 0 },
    { 0x2be13b26, 3 },
    { 0x8d953b26, 4 },
    { 0x41e53b27, 4 },
    { 0xa8c93b29, 4 },
    { 0x5ded3b29, 3 },
    { 0x72353b2b, 4 },
    { 0x38b53b2c, 4 },
    { 0xe4fd3b29, 3 },
    { 0xfa9d3b2e, 3 },
    { 0x2eb53b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d3b32, 3 },
    { 0xf5dd3b33, 4 },
    { 0x89453b34, 4 },
    { 0, 0 },
    { 0x37e53b36, 4 },
    { 0x855d3b36, 4 },
    { 0x75dd3b38, 3 },
    { 0xc0c93b39, 3 },
    { 0x38b53b3a, 3 },
    { 0xc1f93b39, 3 },
    { 0x22253b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67393b3f, 3 },
    { 0x722d3b3f, 3 },
    { 0, 0 },
    { 0x85b13b42, 3 },
    { 0x3de93b43, 3 },
    { 0x80653b42, 4 },
    { 0xbf113b45, 3 },
    { 0x45853b46, 3 },
    { 0x83bd3b45, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a53b4e, 3 },
    { 0xb5493b4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda353b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b13b56, 3 },
    { 0x99a53b57, 3 },
    { 0x8a2d3b57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5053b5e, 3 },
    { 0, 0 },
    { 0x2f593b60, 3 },
    { 0x7c593b61, 4 },
    { 0x7cbd3b60, 3 },
    { 0x47553b63, 4 },
    { 0x2b53b64, 3 },
    { 0x5f8d3b64, 3 },
    { 0x61413b65, 4 },
    { 0x45bd3b67, 3 },
    { 0x95a53b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c93b6d, 3 },
    { 0, 0 },
    { 0xace93b6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e13b72, 3 },
    { 0xc8513b72, 3 },
    { 0xcf3d3b73, 3 },
    { 0x5bb53b75, 3 },
    { 0x53f13b76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7893b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e53b7c, 3 },
    { 0, 0 },
    { 0xc5913b7e, 3 },
    { 0x11893b7f, 3 },
    { 0xcded3b80, 3 },
    { 0xccc53b7e, 3 },
    { 0x4b0d3b82, 3 },
    { 0x32e53b82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59993b89, 3 },
    { 0x14213b8a, 3 },
    { 0x39893b8a, 3 },
    { 0x11e13b8a, 3 },
    { 0x1fcd3b8d, 4 },
    { 0xa0013b8c, 3 },
    { 0x52493b8f, 3 },
    { 0xb39d3b8c, 3 },
    { 0xe1253b91, 3 },
    { 0xfff93b92, 3 },
    { 0xb3b13b8f, 3 },
    { 0xfe193b94, 3 },
    { 0, 0 },
    { 0xbef13b96, 3 },
    { 0x86fd3b96, 3 },
    { 0xbfc13b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bd3b9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c53b9f, 3 },
    { 0xa2b53ba0, 3 },
    { 0xe28d3ba0, 3 },
    { 0xb4b13ba2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e53ba7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9113bb0, 4 },
    { 0, 0 },
    { 0x2e653bb2, 3 },
    { 0x9ea53bb2, 3 },
    { 0x79953bb4, 3 },
    { 0xb5c93bb4, 3 },
    { 0x65193bb4, 4 },
    { 0xa3293bb5, 3 },
    { 0xc7dd3bb8, 3 },
    { 0x71cd3bb9, 3 },
    { 0x11e53bb8, 3 },
    { 0x38f13bbb, 4 },
    { 0x96813bb2, 3 },
    { 0x1b693bbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x527d3bc0, 3 },
    { 0x78c13bc1, 3 },
    { 0, 0 },
    { 0x5d693bc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09d3bc6, 3 },
    { 0xd8413bc6, 3 },
    { 0xfe313bc8, 3 },
    { 0x56853bc9, 3 },
    { 0x816d3bc9, 3 },
    { 0x2fcd3bcb, 3 },
    { 0xba813bcc, 3 },
    { 0xd3bcb, 3 },
    { 0xad4d3bca, 3 },
    { 0x46153bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78553bd7, 3 },
    { 0x36553bd8, 4 },
    { 0xfa113bd7, 3 },
    { 0xc3f13bda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac13bde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1053be2, 3 },
    { 0x674d3be3, 4 },
    { 0x9a653be2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec793be9, 3 },
    { 0xd4313be9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4013bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd613bf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe93bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780d3bfd, 3 },
    { 0xc7f13bfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0413c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f393c07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71913c0e, 3 },
    { 0, 0 },
    { 0x74a13c10, 4 },
    { 0xf1fd3c11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d293c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa53c1b, 3 },
    { 0, 0 },
    { 0x80653c1d, 4 },
    { 0x45453c1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e93c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15693c25, 3 },
    { 0x41e53c26, 3 },
    { 0x55d53c26, 3 },
    { 0, 0 },
    { 0x30f93c29, 3 },
    { 0x68d13c29, 3 },
    { 0x2c313c2b, 3 },
    { 0x33953c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71113c33, 3 },
    { 0, 0 },
    { 0xdd13c35, 3 },
    { 0, 0 },
    { 0x68453c37, 4 },
    { 0x71693c37, 4 },
    { 0x57b53c39, 3 },
    { 0x93893c3a, 3 },
    { 0xe7713c3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d3c3f, 3 },
    { 0xebbd3c3f, 5 },
    { 0, 0 },
    { 0x11813c42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de13c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9193c4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe793c54, 4 },
    { 0xafd53c54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d3c59, 3 },
    { 0xb5d13c5a, 3 },
    { 0xa88d3c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34d3c64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce493c69, 4 },
    { 0x5ac93c69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ad3c6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce053c70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad93c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8313c7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda93c7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f13c80, 3 },
    { 0xc6153c80, 3 },
    { 0xfc413c82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb753c88, 4 },
    { 0x8fad3c88, 3 },
    { 0x52f13c89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed993c8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c653c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0d3c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b13c9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8053c9e, 3 },
    { 0x45013c9f, 3 },
    { 0x96ed3ca0, 3 },
    { 0x73193c9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ed3ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3d3cab, 3 },
    { 0xf6753cab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7453cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf893cb8, 4 },
    { 0x85693cb9, 3 },
    { 0, 0 },
    { 0x70253cbb, 4 },
    { 0, 0 },
    { 0x2c1d3cbd, 3 },
    { 0, 0 },
    { 0x4293cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1653cc4, 3 },
    { 0x3e293cc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35653cc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d13cd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36093cd5, 4 },
    { 0x69193cd6, 4 },
    { 0x3753cd7, 5 },
    { 0xabf13cd7, 3 },
    { 0, 0 },
    { 0x5a13cda, 4 },
    { 0xebf13cdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef53ce0, 3 },
    { 0xb2993ce1, 3 },
    { 0x5a953ce1, 3 },
    { 0x3b113ce2, 3 },
    { 0x4cb93ce4, 3 },
    { 0x5f053ce5, 3 },
    { 0x109d3ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdad3ceb, 3 },
    { 0x31a93cec, 3 },
    { 0x53f93ced, 3 },
    { 0xba253cec, 3 },
    { 0xed713cee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b493cf3, 5 },
    { 0xd41d3cf3, 4 },
    { 0x4a93cf3, 3 },
    { 0, 0 },
    { 0x62253cf7, 3 },
    { 0xbe13cf8, 4 },
    { 0x961d3cf9, 3 },
    { 0, 0 },
    { 0x3d113cfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d3cfe, 3 },
    { 0xe82d3cff, 4 },
    { 0x25413d00, 3 },
    { 0, 0 },
    { 0xd6d3d02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb93d07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f93d0a, 3 },
    { 0x66c53d0b, 4 },
    { 0x84293d0c, 4 },
    { 0xb8613d0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7613d13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fd3d16, 3 },
    { 0x82413d17, 3 },
    { 0xc1293d18, 3 },
    { 0x54bd3d19, 4 },
    { 0, 0 },
    { 0xb83d3d1b, 4 },
    { 0xf4b53d1b, 3 },
    { 0, 0 },
    { 0x5e853d1e, 3 },
    { 0xc6553d1f, 3 },
    { 0, 0 },
    { 0x7e313d21, 3 },
    { 0, 0 },
    { 0x9bed3d23, 5 },
    { 0, 0 },
    { 0x42593d25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be13d2a, 3 },
    { 0xd8313d2a, 3 },
    { 0xdb753d2b, 3 },
    { 0, 0 },
    { 0x7b113d2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25493d31, 3 },
    { 0xa0bd3d32, 3 },
    { 0, 0 },
    { 0xdbd53d34, 3 },
    { 0, 0 },
    { 0xbe653d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26693d3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39693d3e, 3 },
    { 0, 0 },
    { 0xcb253d40, 3 },
    { 0x6ec93d41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62353d48, 4 },
    { 0, 0 },
    { 0xf9f53d4a, 4 },
    { 0x931d3d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e93d57, 3 },
    { 0x8753d57, 3 },
    { 0x5ce13d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f53d60, 3 },
    { 0xf10d3d61, 3 },
    { 0xdf713d62, 3 },
    { 0x7b393d62, 3 },
    { 0xf9413d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab13d6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f893d6e, 3 },
    { 0, 0 },
    { 0xffb53d70, 3 },
    { 0, 0 },
    { 0x50493d72, 3 },
    { 0x8db93d73, 4 },
    { 0xabb13d74, 3 },
    { 0x72613d74, 4 },
    { 0x88d53d76, 3 },
    { 0x82893d75, 3 },
    { 0x26493d78, 3 },
    { 0xe9013d78, 3 },
    { 0x82f53d7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d3d81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6453d85, 4 },
    { 0xde393d86, 3 },
    { 0x36393d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12853d8a, 3 },
    { 0x71793d8b, 3 },
    { 0x3c293d8c, 3 },
    { 0x54e53d8b, 3 },
    { 0xc0793d8c, 3 },
    { 0x10713d8f, 3 },
    { 0, 0 },
    { 0xd0fd3d91, 3 },
    { 0xb23d3d92, 3 },
    { 0x22253d93, 3 },
    { 0, 0 },
    { 0x86d13d95, 4 },
    { 0xe4953d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4793d9c, 3 },
    { 0x8d693d9c, 3 },
    { 0, 0 },
    { 0x81f53d9f, 4 },
    { 0, 0 },
    { 0x7a4d3da1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d3daa, 3 },
    { 0, 0 },
    { 0xfeb53dac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd3daf, 4 },
    { 0xd3313daf, 3 },
    { 0x78a93db1, 3 },
    { 0xd9053db1, 3 },
    { 0x4ba93db3, 3 },
    { 0x34393db4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619d3db9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d13dbe, 3 },
    { 0x19553dbf, 4 },
    { 0, 0 },
    { 0x23b53dc1, 3 },
    { 0x33893dc2, 3 },
    { 0, 0 },
    { 0xe8d3dc4, 3 },
    { 0xf1d3dc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d3dc8, 3 },
    { 0x3be93dc9, 3 },
    { 0xf8e53dc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fd3dd1, 3 },
    { 0, 0 },
    { 0x7bf53dd3, 3 },
    { 0xcb953dd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c93ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a13de4, 3 },
    { 0x51d3de5, 4 },
    { 0, 0 },
    { 0x61dd3de7, 3 },
    { 0, 0 },
    { 0xe70d3de9, 3 },
    { 0xbdb93dea, 3 },
    { 0, 0 },
    { 0xf4713dec, 3 },
    { 0, 0 },
    { 0x13a13dee, 3 },
    { 0x36613dee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12d3df2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d53df6, 4 },
    { 0, 0 },
    { 0x3e293df8, 3 },
    { 0xd7453df8, 3 },
    { 0xa5a13df8, 3 },
    { 0x78253dfb, 3 },
    { 0xb0153df8, 3 },
    { 0xabe13dfd, 3 },
    { 0xfb3d3df8, 3 },
    { 0, 0 },
    { 0xf2553e00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83693e03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7d3e08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70413e0b, 3 },
    { 0x55253e0c, 3 },
    { 0, 0 },
    { 0x9abd3e0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb053e11, 3 },
    { 0x66dd3e11, 3 },
    { 0xcb753e13, 3 },
    { 0, 0 },
    { 0xdf613e15, 3 },
    { 0xdd193e16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a153e1a, 3 },
    { 0xf22d3e1b, 3 },
    { 0x3d713e1a, 3 },
    { 0xec053e1d, 4 },
    { 0, 0 },
    { 0x54d93e1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66013e22, 3 },
    { 0, 0 },
    { 0x6c513e24, 3 },
    { 0x81113e24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd3e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b13e2c, 3 },
    { 0, 0 },
    { 0x88553e2e, 3 },
    { 0, 0 },
    { 0x9b093e30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1213e33, 3 },
    { 0, 0 },
    { 0xd3ad3e35, 3 },
    { 0xc8813e36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbd3e3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45913e3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dad3e40, 3 },
    { 0, 0 },
    { 0x917d3e42, 3 },
    { 0, 0 },
    { 0x66093e44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a93e47, 3 },
    { 0x989d3e47, 3 },
    { 0, 0 },
    { 0xc3453e4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e13e50, 3 },
    { 0xd4a93e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d3e56, 3 },
    { 0, 0 },
    { 0xd9a93e58, 3 },
    { 0, 0 },
    { 0xf2d53e5a, 4 },
    { 0, 0 },
    { 0xac193e5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e13e61, 3 },
    { 0x4413e62, 4 },
    { 0xdfad3e61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab353e66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da93e6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a93e73, 3 },
    { 0x9ced3e74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef53e77, 3 },
    { 0xefe13e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45853e7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x528d3e7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9853e81, 3 },
    { 0x390d3e82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757d3e88, 4 },
    { 0x49753e89, 3 },
    { 0, 0 },
    { 0x699d3e8b, 3 },
    { 0xbbc13e8c, 4 },
    { 0, 0 },
    { 0xc9e93e8e, 3 },
    { 0x23f53e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad653e92, 3 },
    { 0xea313e93, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d93e9a, 4 },
    { 0, 0 },
    { 0xc5513e9c, 3 },
    { 0, 0 },
    { 0xd7bd3e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a353ea9, 3 },
    { 0xb5b53eaa, 3 },
    { 0x97c13eaa, 3 },
    { 0x76853ea9, 4 },
    { 0x8f653ea9, 3 },
    { 0xa2dd3eac, 4 },
    { 0x669d3ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef13eb3, 3 },
    { 0xbcb13eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d53eb9, 3 },
    { 0xf4fd3eb9, 3 },
    { 0, 0 },
    { 0x14153ebc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x73693ebf, 3 },
    { 0xc8e93ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13ec5, 3 },
    { 0xd5853ec6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bd3ec9, 3 },
    { 0x1a513ec9, 3 },
    { 0, 0 },
    { 0xc16d3ecc, 3 },
    { 0xa2153ecd, 4 },
    { 0xeb53ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64d3ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e353ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd3ede, 4 },
    { 0, 0 },
    { 0x395d3ee0, 4 },
    { 0x1b1d3ee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b53ee7, 3 },
    { 0, 0 },
    { 0x56a93ee9, 3 },
    { 0x24b13ee9, 3 },
    { 0x77513eea, 3 },
    { 0x28713eeb, 4 },
    { 0x39d13ee9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x76093ef0, 3 },
    { 0xcd713ef1, 4 },
    { 0x741d3ef2, 3 },
    { 0x165d3ef1, 3 },
    { 0, 0 },
    { 0x59153ef5, 3 },
    { 0x8a713ef6, 3 },
    { 0xce313ef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb913efa, 4 },
    { 0xdefd3efa, 3 },
    { 0xe3b53efc, 3 },
    { 0x433d3efc, 4 },
    { 0xd7713efb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80313f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52113f06, 3 },
    { 0xade53f07, 4 },
    { 0x62893f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a113f10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3893f14, 3 },
    { 0, 0 },
    { 0xaec13f16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd53f1b, 3 },
    { 0x49b53f1c, 3 },
    { 0xbe013f1c, 3 },
    { 0x5d813f1e, 4 },
    { 0xaa553f1d, 3 },
    { 0x5bc53f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09d3f23, 3 },
    { 0xd3793f24, 4 },
    { 0xd79d3f23, 3 },
    { 0xfe393f26, 3 },
    { 0x5c113f24, 3 },
    { 0x8ff93f28, 3 },
    { 0x230d3f25, 3 },
    { 0, 0 },
    { 0x9f53f2b, 3 },
    { 0x5d693f2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d53f2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c93f34, 3 },
    { 0, 0 },
    { 0xc7e13f36, 3 },
    { 0x629d3f37, 4 },
    { 0, 0 },
    { 0x51313f39, 4 },
    { 0, 0 },
    { 0x31093f3b, 3 },
    { 0xa0193f3c, 3 },
    { 0xeb193f3d, 4 },
    { 0xb4c93f3c, 3 },
    { 0x3ac93f3b, 3 },
    { 0x213d3f3f, 3 },
    { 0x5a053f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e53f46, 3 },
    { 0x55653f47, 4 },
    { 0x7e5d3f48, 3 },
    { 0x753d3f49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4013f4c, 3 },
    { 0, 0 },
    { 0x83c93f4e, 3 },
    { 0xe9113f4f, 3 },
    { 0x511d3f50, 4 },
    { 0x7f053f51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada53f5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e513f5e, 3 },
    { 0x9a9d3f5e, 3 },
    { 0xfdd93f60, 3 },
    { 0x95053f61, 4 },
    { 0x6b8d3f60, 3 },
    { 0, 0 },
    { 0xebcd3f64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5d3f67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2113f6a, 3 },
    { 0x7ffd3f6b, 3 },
    { 0x80193f6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9513f71, 3 },
    { 0x477d3f72, 3 },
    { 0x530d3f72, 3 },
    { 0x317d3f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575d3f7b, 4 },
    { 0x54513f7c, 3 },
    { 0, 0 },
    { 0xec313f7e, 3 },
    { 0x2ced3f7f, 3 },
    { 0xd8613f80, 4 },
    { 0x96e13f81, 3 },
    { 0xcd253f82, 4 },
    { 0xe3913f83, 3 },
    { 0x12553f81, 3 },
    { 0xa5893f85, 3 },
    { 0, 0 },
    { 0xa71d3f87, 4 },
    { 0xc3993f87, 3 },
    { 0, 0 },
    { 0x18813f8a, 3 },
    { 0, 0 },
    { 0x8e253f8c, 4 },
    { 0x2d313f8d, 3 },
    { 0, 0 },
    { 0xb4e53f8f, 3 },
    { 0, 0 },
    { 0x11ad3f91, 3 },
    { 0xd4fd3f92, 3 },
    { 0x94853f92, 4 },
    { 0x9e9d3f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80793f97, 3 },
    { 0x2a9d3f97, 4 },
    { 0xaf053f99, 3 },
    { 0xe0f13f9a, 3 },
    { 0xaa953f9b, 3 },
    { 0xa5493f9a, 3 },
    { 0xe2753f9d, 3 },
    { 0x36313f9e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8d3fa1, 3 },
    { 0xb3013fa1, 3 },
    { 0, 0 },
    { 0x813d3fa4, 4 },
    { 0x9b713fa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e493fb1, 3 },
    { 0xb9b93fb1, 4 },
    { 0xf1193fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732d3fb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6d3fba, 3 },
    { 0, 0 },
    { 0x3e213fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73d3fbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99493fc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a53fc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae693fcf, 4 },
    { 0x661d3fd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08d3fd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b13fe0, 4 },
    { 0xab093fe1, 3 },
    { 0x685d3fe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c353fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f113fea, 4 },
    { 0xd9d13fea, 4 },
    { 0xc12d3fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d53ff1, 3 },
    { 0, 0 },
    { 0xf0a93ff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47713ff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bd4003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd4007, 3 },
    { 0xdd4008, 3 },
    { 0x622d4009, 3 },
    { 0x90314009, 4 },
    { 0x3201400b, 3 },
    { 0, 0 },
    { 0xff59400d, 3 },
    { 0x9345400d, 3 },
    { 0x3f6d400f, 4 },
    { 0x4f754010, 3 },
    { 0xf531400d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c14015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c54018, 3 },
    { 0x5b114018, 3 },
    { 0, 0 },
    { 0xef09401b, 5 },
    { 0, 0 },
    { 0xbd81401d, 3 },
    { 0x3695401e, 4 },
    { 0xe10d401f, 3 },
    { 0x155d401f, 3 },
    { 0, 0 },
    { 0x66ed4022, 4 },
    { 0x7cbd4023, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x48614026, 3 },
    { 0x8f3d4027, 3 },
    { 0, 0 },
    { 0x405d4029, 3 },
    { 0x47d5402a, 4 },
    { 0xdc71402b, 3 },
    { 0x93d1402b, 3 },
    { 0x3cdd402c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd4031, 3 },
    { 0, 0 },
    { 0xeb2d4033, 3 },
    { 0xcdd4034, 3 },
    { 0x8a314035, 3 },
    { 0xa994035, 3 },
    { 0x33154037, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe751403a, 5 },
    { 0xd811403b, 3 },
    { 0xb131403c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d4041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57994044, 3 },
    { 0xd6294045, 4 },
    { 0x7c0d4044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62d404a, 3 },
    { 0x8f5d404a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2294052, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x98694055, 4 },
    { 0, 0 },
    { 0x36f54057, 3 },
    { 0, 0 },
    { 0x874d4059, 3 },
    { 0x35654059, 3 },
    { 0xb051405b, 3 },
    { 0, 0 },
    { 0xeeb9405d, 3 },
    { 0xf429405d, 3 },
    { 0, 0 },
    { 0xcbf94060, 3 },
    { 0, 0 },
    { 0x3e614062, 3 },
    { 0xb2994063, 3 },
    { 0xf6c14063, 3 },
    { 0x6eed4065, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e314068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd1406b, 3 },
    { 0x3945406b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c114072, 3 },
    { 0, 0 },
    { 0x1cf94074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec94077, 3 },
    { 0x174d4078, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c9407b, 3 },
    { 0x7175407c, 4 },
    { 0xc451407d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4d4080, 3 },
    { 0x99b54081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8d4084, 4 },
    { 0xdb514085, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d408b, 3 },
    { 0x2edd408b, 3 },
    { 0x4239408d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba714093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7775409a, 4 },
    { 0x5b7d409b, 3 },
    { 0xc3c9409c, 3 },
    { 0x5559409d, 3 },
    { 0xcf5d409c, 4 },
    { 0x69e9409f, 3 },
    { 0x9b25409e, 3 },
    { 0xd57140a1, 4 },
    { 0xfcad40a2, 3 },
    { 0x22e540a2, 3 },
    { 0x137d40a2, 3 },
    { 0x812140a5, 4 },
    { 0x732140a2, 3 },
    { 0x28c140a7, 3 },
    { 0x9ae540a8, 3 },
    { 0xa8a9409e, 3 },
    { 0xb09140aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496540ae, 4 },
    { 0xfdfd40ae, 3 },
    { 0xd40140b0, 3 },
    { 0, 0 },
    { 0x59fd40b2, 3 },
    { 0x5f9540b2, 3 },
    { 0x1c6540b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555540b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x985940bb, 3 },
    { 0, 0 },
    { 0x330540bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b140c4, 3 },
    { 0x4a1d40c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee140c8, 4 },
    { 0x35bd40c8, 3 },
    { 0xee4d40ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3d40ce, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd40d2, 3 },
    { 0x6aed40d2, 4 },
    { 0xbf7540d4, 4 },
    { 0xd09140d5, 3 },
    { 0x4f1540d5, 4 },
    { 0xf7540d6, 4 },
    { 0xe79940d5, 3 },
    { 0xf13540d5, 3 },
    { 0x5e6940da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5d40dd, 3 },
    { 0x671940de, 3 },
    { 0x1d4540df, 3 },
    { 0xded140df, 3 },
    { 0xcb140dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa140e4, 4 },
    { 0x7f0d40e5, 3 },
    { 0xb6ed40e6, 4 },
    { 0x214140e6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314540ec, 3 },
    { 0, 0 },
    { 0x250540ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a940f5, 4 },
    { 0x167140f5, 3 },
    { 0x892d40f5, 3 },
    { 0xe31940f6, 4 },
    { 0x89940f9, 3 },
    { 0x271d40fa, 3 },
    { 0, 0 },
    { 0xf2b940fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51540ff, 3 },
    { 0xb4654100, 3 },
    { 0x85fd40ff, 3 },
    { 0xbe314102, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b654105, 3 },
    { 0, 0 },
    { 0x5a214107, 3 },
    { 0x1f494108, 3 },
    { 0x31dd4109, 3 },
    { 0xaa594107, 3 },
    { 0xe5cd410b, 3 },
    { 0x660d410a, 3 },
    { 0, 0 },
    { 0xb181410e, 3 },
    { 0, 0 },
    { 0x916d4110, 3 },
    { 0x32d94111, 3 },
    { 0xa0914111, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f994116, 3 },
    { 0x8e114116, 3 },
    { 0x85054118, 3 },
    { 0xf6fd4118, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dd411c, 3 },
    { 0x56c1411d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e914120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70d4124, 3 },
    { 0, 0 },
    { 0x63dd4126, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca1412f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47854133, 3 },
    { 0x3dd4134, 3 },
    { 0xf394135, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5413a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109d4141, 4 },
    { 0, 0 },
    { 0x4b394143, 3 },
    { 0x18314144, 4 },
    { 0x1b294145, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14214149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5059414e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32414153, 4 },
    { 0x5e714154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604d4158, 3 },
    { 0, 0 },
    { 0x65c5415a, 3 },
    { 0xe01415a, 3 },
    { 0xf3e1415c, 3 },
    { 0x6259415b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d4161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd954167, 3 },
    { 0, 0 },
    { 0x152d4169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2549416e, 3 },
    { 0x8339416e, 4 },
    { 0x2d0d416f, 3 },
    { 0, 0 },
    { 0x19414172, 4 },
    { 0x37294172, 3 },
    { 0xfe14174, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd4177, 3 },
    { 0xae994178, 4 },
    { 0xacb54178, 3 },
    { 0x137d4178, 3 },
    { 0x7d29417b, 3 },
    { 0, 0 },
    { 0x3339417d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf054183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca354186, 3 },
    { 0x1d8d4187, 4 },
    { 0xc8394187, 3 },
    { 0, 0 },
    { 0x9779418a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbd418d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24594194, 3 },
    { 0x3bfd4195, 3 },
    { 0x2ddd4194, 4 },
    { 0x64e94194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc171419a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5541a1, 3 },
    { 0xd84541a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab941a6, 5 },
    { 0x59b541a6, 3 },
    { 0xb57d41a8, 4 },
    { 0x203941a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58d41ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2d41b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c141b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba141bc, 4 },
    { 0x742d41bc, 4 },
    { 0xad8d41bc, 4 },
    { 0x47fd41bc, 3 },
    { 0x7c8941c0, 3 },
    { 0x9e7541c1, 3 },
    { 0, 0 },
    { 0xffe541c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343541cd, 3 },
    { 0x1a9141cd, 3 },
    { 0x701d41cf, 3 },
    { 0, 0 },
    { 0x781941d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268941d8, 3 },
    { 0xdaad41d9, 3 },
    { 0x1ce941da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d41dd, 4 },
    { 0xee5941dd, 3 },
    { 0x1f8d41df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ad41e2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342541e6, 4 },
    { 0xc43541e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177141ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ed41f2, 3 },
    { 0, 0 },
    { 0x7b5541f4, 3 },
    { 0, 0 },
    { 0xce9d41f6, 3 },
    { 0x117141f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb694201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72094206, 3 },
    { 0xd91d4206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3151420c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb214211, 3 },
    { 0x8fb54212, 3 },
    { 0xd9d94213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715d4217, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df5421b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a614220, 4 },
    { 0xde454220, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d4225, 3 },
    { 0xb9294226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25422a, 4 },
    { 0x38e9422b, 4 },
    { 0, 0 },
    { 0xc231422d, 3 },
    { 0xb6d5422e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eed4232, 3 },
    { 0x97c14232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf721423b, 4 },
    { 0x341423c, 4 },
    { 0x75d9423d, 3 },
    { 0x694d423e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341d4242, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40414245, 3 },
    { 0x4f894246, 4 },
    { 0x5c714247, 4 },
    { 0xaed14248, 3 },
    { 0x3a414248, 3 },
    { 0xacf9424a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e1424d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2614250, 3 },
    { 0x33054251, 3 },
    { 0x55654251, 3 },
    { 0x7fd4251, 3 },
    { 0x46714251, 3 },
    { 0xe2214250, 3 },
    { 0, 0 },
    { 0x77394257, 3 },
    { 0, 0 },
    { 0xc95d4259, 3 },
    { 0x2594259, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411d4260, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8294266, 3 },
    { 0xffb54266, 3 },
    { 0xce514268, 4 },
    { 0x5ad54269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b94270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ed427b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5069427f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa021428b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab754290, 3 },
    { 0x4bf94290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f94296, 3 },
    { 0x952d4297, 3 },
    { 0, 0 },
    { 0x81214299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6201429c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1542aa, 4 },
    { 0x425142ab, 4 },
    { 0, 0 },
    { 0xd7ed42ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd42b5, 3 },
    { 0x6e9542b5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2542ba, 5 },
    { 0x9c6542bb, 3 },
    { 0xa96542bb, 3 },
    { 0, 0 },
    { 0xa43d42be, 3 },
    { 0, 0 },
    { 0xc88942c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xead942c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23542c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17142cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x366d42cf, 3 },
    { 0xba8d42d0, 5 },
    { 0xf5a942d1, 3 },
    { 0, 0 },
    { 0xa14942d3, 3 },
    { 0x6a4d42d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2d42d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07942da, 3 },
    { 0x828142da, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326942e5, 3 },
    { 0, 0 },
    { 0x214d42e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8942eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x141542ee, 3 },
    { 0xbda142ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dd42f7, 4 },
    { 0x9b0942f8, 3 },
    { 0xaff942f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41d42fc, 4 },
    { 0x827142fc, 3 },
    { 0, 0 },
    { 0x71b142ff, 3 },
    { 0xcc4d4300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec294307, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa9430a, 3 },
    { 0xac21430b, 3 },
    { 0, 0 },
    { 0x9f45430d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d814311, 3 },
    { 0x76314312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad894318, 3 },
    { 0x5a094318, 3 },
    { 0x7b55431a, 3 },
    { 0x62194318, 5 },
    { 0xf8c9431c, 4 },
    { 0, 0 },
    { 0x98c1431e, 3 },
    { 0, 0 },
    { 0x96d94320, 3 },
    { 0, 0 },
    { 0x4f94322, 3 },
    { 0xec1d4322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31194327, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd9432f, 3 },
    { 0x4ac9432f, 3 },
    { 0xd0394331, 3 },
    { 0xace54332, 3 },
    { 0x47fd4331, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24914337, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d433a, 3 },
    { 0, 0 },
    { 0xf649433c, 4 },
    { 0xc8d433d, 4 },
    { 0x53e1433d, 3 },
    { 0x7681433f, 3 },
    { 0xa5414340, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b54348, 4 },
    { 0x21c14349, 3 },
    { 0xd411434a, 3 },
    { 0x32ad434b, 4 },
    { 0xbc31434b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b5434f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c854355, 4 },
    { 0x4b614355, 3 },
    { 0x74514357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc9435a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc5435d, 3 },
    { 0x5909435e, 3 },
    { 0, 0 },
    { 0xa90d4360, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd4367, 3 },
    { 0xcc514367, 4 },
    { 0xf0454367, 3 },
    { 0, 0 },
    { 0x6dad436b, 3 },
    { 0x7025436c, 4 },
    { 0xbb2d436d, 3 },
    { 0xcb25436e, 3 },
    { 0x68a1436f, 3 },
    { 0, 0 },
    { 0x38a54371, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc94379, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d437c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a94381, 3 },
    { 0x718d4382, 4 },
    { 0x21714383, 3 },
    { 0xb93d4383, 4 },
    { 0x1bb14385, 3 },
    { 0x48814386, 3 },
    { 0, 0 },
    { 0xc77d4388, 3 },
    { 0x8da14388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4059438c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a71438f, 3 },
    { 0, 0 },
    { 0x5f854391, 3 },
    { 0x4b754391, 5 },
    { 0x22294392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8294399, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9439c, 3 },
    { 0, 0 },
    { 0x515d439e, 4 },
    { 0xc1439e, 3 },
    { 0x4aa9439f, 3 },
    { 0x32c543a1, 3 },
    { 0xf28d43a0, 3 },
    { 0, 0 },
    { 0xa47d43a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e143a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de543ae, 4 },
    { 0x7ae943af, 3 },
    { 0, 0 },
    { 0x1aed43b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139143b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7143be, 3 },
    { 0x3a4d43bf, 3 },
    { 0xe7e143c0, 4 },
    { 0xc1d943c1, 3 },
    { 0x9ad143c2, 3 },
    { 0x485943c3, 4 },
    { 0x9c9943bf, 3 },
    { 0x1ed943c1, 3 },
    { 0xff1d43c1, 3 },
    { 0xc46d43c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205d43ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1543d7, 3 },
    { 0x8a9d43d7, 4 },
    { 0xa97143d9, 3 },
    { 0x725143d9, 4 },
    { 0xab4d43db, 3 },
    { 0x157143dc, 4 },
    { 0x7e6d43dc, 3 },
    { 0xbad943de, 4 },
    { 0, 0 },
    { 0xfcd943e0, 4 },
    { 0, 0 },
    { 0x767543e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2543f2, 3 },
    { 0xba5943f3, 3 },
    { 0x6a1143f4, 3 },
    { 0, 0 },
    { 0xd47d43f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67d43fa, 3 },
    { 0, 0 },
    { 0xe99943fc, 4 },
    { 0xe60943fd, 3 },
    { 0xf17543fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa954402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ad4406, 3 },
    { 0x9d414406, 4 },
    { 0x11f14408, 3 },
    { 0xea354409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66d4412, 3 },
    { 0x2c4d4413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f694416, 3 },
    { 0x1e854416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe01441b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7769441f, 3 },
    { 0, 0 },
    { 0x7d454421, 4 },
    { 0, 0 },
    { 0xbb454423, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9154426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8794429, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec1442d, 4 },
    { 0, 0 },
    { 0x2115442f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69994432, 3 },
    { 0, 0 },
    { 0x538d4434, 3 },
    { 0xa5454434, 3 },
    { 0, 0 },
    { 0xf17d4437, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a39443b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75443f, 3 },
    { 0x9cf14440, 3 },
    { 0, 0 },
    { 0xd7914442, 3 },
    { 0x7fb94442, 3 },
    { 0xf014443, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3454448, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d444c, 3 },
    { 0x3749444d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa54450, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d14453, 3 },
    { 0xa9094453, 3 },
    { 0xdb94455, 4 },
    { 0, 0 },
    { 0xae514457, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1875445b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d14460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab14468, 3 },
    { 0x8994469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80394470, 3 },
    { 0x87c54471, 4 },
    { 0xb754471, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777d4478, 3 },
    { 0x11fd4479, 3 },
    { 0, 0 },
    { 0xf049447b, 3 },
    { 0, 0 },
    { 0xa3e9447d, 3 },
    { 0x22ad447d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed4481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74014485, 3 },
    { 0, 0 },
    { 0x2ef94487, 3 },
    { 0x47154488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2889448c, 3 },
    { 0, 0 },
    { 0xded1448e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32254492, 3 },
    { 0x3c394492, 4 },
    { 0, 0 },
    { 0xfc014495, 3 },
    { 0, 0 },
    { 0xda414497, 4 },
    { 0x46ed4498, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd9449c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0d44a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d44aa, 3 },
    { 0x653544ab, 4 },
    { 0xe2e144ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3d44b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46d44b5, 3 },
    { 0, 0 },
    { 0xedc944b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9544bd, 3 },
    { 0, 0 },
    { 0xd46d44bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c144ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1d44d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564544d8, 4 },
    { 0x751944d9, 4 },
    { 0, 0 },
    { 0x9b1144db, 3 },
    { 0x9ff944dc, 4 },
    { 0x3fdd44dc, 3 },
    { 0xe26144de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d44e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc544e9, 3 },
    { 0xbf3544ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd44ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506d44f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854d44f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb544fb, 3 },
    { 0, 0 },
    { 0x6a5144fd, 4 },
    { 0xc4b544fe, 3 },
    { 0xe0dd44ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc221450c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37714510, 3 },
    { 0x5914511, 3 },
    { 0xf4394511, 3 },
    { 0xc7fd4511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd4518, 4 },
    { 0xaded4519, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0d4528, 3 },
    { 0, 0 },
    { 0xb9b9452a, 3 },
    { 0x4551452a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1452e, 3 },
    { 0xe60d452f, 4 },
    { 0x214530, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d4536, 3 },
    { 0x2b614537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131453f, 4 },
    { 0xbc014540, 3 },
    { 0xbe19453f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49514544, 3 },
    { 0xb46d4545, 4 },
    { 0xeab54546, 4 },
    { 0, 0 },
    { 0xf4014548, 3 },
    { 0xfe214549, 3 },
    { 0, 0 },
    { 0xcaa1454b, 3 },
    { 0x39d5454c, 4 },
    { 0, 0 },
    { 0x45fd454e, 4 },
    { 0xb73d454f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b94555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6935455b, 3 },
    { 0xf621455c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39d4560, 3 },
    { 0, 0 },
    { 0x81cd4562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700d456c, 3 },
    { 0xc889456c, 3 },
    { 0, 0 },
    { 0x7e1456f, 5 },
    { 0, 0 },
    { 0xac7d4571, 3 },
    { 0, 0 },
    { 0x82ad4573, 3 },
    { 0, 0 },
    { 0x9e714575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e914578, 5 },
    { 0x49a14578, 3 },
    { 0x7b914579, 3 },
    { 0x8135457b, 3 },
    { 0x84bd457b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5d457f, 3 },
    { 0, 0 },
    { 0x7414581, 4 },
    { 0x52554582, 4 },
    { 0x54c54582, 3 },
    { 0x9fad4581, 3 },
    { 0xc1314585, 4 },
    { 0x53c14581, 3 },
    { 0, 0 },
    { 0xf0614588, 4 },
    { 0, 0 },
    { 0x8401458a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bd458e, 3 },
    { 0x441458f, 3 },
    { 0x51514590, 3 },
    { 0xa5a94591, 4 },
    { 0xb8d94592, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8715459c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32d45a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x711545a4, 5 },
    { 0xb3c145a5, 3 },
    { 0xe26d45a5, 4 },
    { 0x5a0d45a7, 3 },
    { 0x6f3945a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d45b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55145b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe145bb, 3 },
    { 0xe85945bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc945cc, 3 },
    { 0, 0 },
    { 0xd20545ce, 3 },
    { 0x84dd45ce, 3 },
    { 0x4f0d45cf, 3 },
    { 0xd86545d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8145d5, 4 },
    { 0xb2ad45d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243545da, 3 },
    { 0xd82145db, 4 },
    { 0x695d45db, 3 },
    { 0x21945dc, 4 },
    { 0xad8545de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51545e2, 3 },
    { 0x30545e3, 3 },
    { 0xbe1545e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad945e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5945eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05945f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2545f8, 3 },
    { 0x972945f9, 3 },
    { 0x15b945fa, 3 },
    { 0xfa6545fa, 4 },
    { 0x23e945fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x233145ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f54602, 3 },
    { 0x5d3d4603, 3 },
    { 0xf5854604, 4 },
    { 0x89714603, 4 },
    { 0xcac54606, 3 },
    { 0x59814603, 3 },
    { 0xd2d54608, 3 },
    { 0x96314609, 4 },
    { 0x26f1460a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b854610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c54615, 3 },
    { 0, 0 },
    { 0xa4e94617, 3 },
    { 0x3a594618, 5 },
    { 0x90fd4619, 3 },
    { 0xcfb14619, 4 },
    { 0x4e5d4619, 3 },
    { 0xedf9461a, 3 },
    { 0x8705461b, 3 },
    { 0x5d294619, 3 },
    { 0x5981461f, 3 },
    { 0xe7ed461b, 3 },
    { 0x3fe54621, 3 },
    { 0x13914617, 3 },
    { 0x33b9461a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3094627, 3 },
    { 0, 0 },
    { 0xdef14629, 3 },
    { 0xe7014629, 3 },
    { 0xe121462a, 4 },
    { 0x1e41462c, 3 },
    { 0x1351462d, 3 },
    { 0x8345462d, 3 },
    { 0x8d41462f, 4 },
    { 0x16194630, 4 },
    { 0xbe494631, 3 },
    { 0x4f49462f, 3 },
    { 0xcd314633, 3 },
    { 0x27214634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb154639, 4 },
    { 0x63e94639, 3 },
    { 0xba7d463a, 4 },
    { 0, 0 },
    { 0x18d5463d, 4 },
    { 0, 0 },
    { 0xffb5463f, 4 },
    { 0, 0 },
    { 0x53d94641, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf14646, 5 },
    { 0, 0 },
    { 0x30894648, 3 },
    { 0x90354649, 3 },
    { 0xc5d5464a, 4 },
    { 0x3cc1464b, 3 },
    { 0x6c394648, 3 },
    { 0x147d4649, 3 },
    { 0x48c5464e, 3 },
    { 0x97e5464e, 4 },
    { 0x4c94650, 4 },
    { 0x51bd464b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145d4658, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9d465b, 4 },
    { 0, 0 },
    { 0x5b35465d, 3 },
    { 0xad71465d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a94661, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fad4664, 3 },
    { 0xa4954665, 3 },
    { 0xeae54665, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75466a, 3 },
    { 0x1ec9466b, 4 },
    { 0xae59466c, 4 },
    { 0xb5c5466d, 3 },
    { 0x1655466c, 3 },
    { 0x241466f, 3 },
    { 0x589d466f, 3 },
    { 0x7de1466f, 4 },
    { 0xfaf5466f, 3 },
    { 0xabd14673, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe94678, 3 },
    { 0xb6294679, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d467c, 3 },
    { 0x7679467c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354680, 3 },
    { 0x4ab14680, 3 },
    { 0x30e14682, 3 },
    { 0x12414681, 4 },
    { 0x5a3d4684, 4 },
    { 0x885d4680, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1315468d, 4 },
    { 0xa2e5468e, 3 },
    { 0x2151468d, 4 },
    { 0x4cbd4690, 3 },
    { 0x5ccd4691, 3 },
    { 0x6bd4692, 4 },
    { 0x8b094693, 3 },
    { 0x81194692, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x831d4697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a1469d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x522146a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4146a3, 3 },
    { 0x6d9146a4, 3 },
    { 0, 0 },
    { 0x47c146a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f946a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f146af, 3 },
    { 0x303946af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864946b4, 3 },
    { 0, 0 },
    { 0xf5bd46b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a946b9, 3 },
    { 0, 0 },
    { 0x66c946bb, 3 },
    { 0, 0 },
    { 0x846146bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652146c2, 3 },
    { 0x97a946c3, 3 },
    { 0x2db146c4, 3 },
    { 0, 0 },
    { 0xdb8946c6, 3 },
    { 0x5f9146c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649546cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86146cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9146d2, 3 },
    { 0, 0 },
    { 0x529d46d4, 3 },
    { 0x216546d5, 3 },
    { 0xe73946d6, 3 },
    { 0x567946d7, 3 },
    { 0x4b6546d4, 4 },
    { 0x454946d9, 3 },
    { 0x465146d9, 4 },
    { 0xd0c946db, 3 },
    { 0x9ee946d9, 3 },
    { 0xd66d46db, 4 },
    { 0x2e5946de, 3 },
    { 0x581d46d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74946ed, 3 },
    { 0x827d46ee, 4 },
    { 0xfacd46ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0546f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd946fb, 3 },
    { 0xe62946fc, 3 },
    { 0, 0 },
    { 0xb7146fe, 3 },
    { 0xd20d46ff, 3 },
    { 0xbc54700, 4 },
    { 0xd1194701, 3 },
    { 0x95694702, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652d470a, 4 },
    { 0x709470b, 3 },
    { 0xe259470b, 3 },
    { 0, 0 },
    { 0xaa3d470e, 3 },
    { 0x99b9470e, 4 },
    { 0, 0 },
    { 0xbc214711, 4 },
    { 0xf9fd4712, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548d4720, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afd4725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e854729, 3 },
    { 0, 0 },
    { 0xfe0d472b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc249472f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f94732, 3 },
    { 0xd7494732, 3 },
    { 0, 0 },
    { 0xf5954735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27d4739, 5 },
    { 0xfeb5473a, 3 },
    { 0x6a4d4739, 3 },
    { 0, 0 },
    { 0x3e69473d, 3 },
    { 0x41d9473e, 4 },
    { 0, 0 },
    { 0x972d4740, 3 },
    { 0x2814741, 3 },
    { 0xacbd4740, 4 },
    { 0xdb2d4742, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65194748, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9d474c, 3 },
    { 0x5c2d474d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6854752, 4 },
    { 0xb9854752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16594756, 3 },
    { 0xaebd4757, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a1475c, 3 },
    { 0xcd29475d, 3 },
    { 0x1221475c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52254762, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f94766, 3 },
    { 0, 0 },
    { 0xa26d4768, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17d476c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d476f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b94775, 4 },
    { 0xc8314776, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5295477b, 3 },
    { 0x3ff1477c, 3 },
    { 0xc3a1477d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fd4780, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c194787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e94790, 3 },
    { 0xd8954791, 3 },
    { 0, 0 },
    { 0xb3f54793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1931479a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e45479f, 3 },
    { 0xc82d479f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb147a8, 4 },
    { 0x9b2147a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea147bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3d47c4, 4 },
    { 0x2aa547c5, 3 },
    { 0x5ea947c5, 3 },
    { 0xf50947c5, 3 },
    { 0x1d5947c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd547cd, 5 },
    { 0, 0 },
    { 0x351147cf, 3 },
    { 0x8b8d47d0, 3 },
    { 0xb18147d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649147e1, 3 },
    { 0x36c547e2, 4 },
    { 0xb24d47e3, 3 },
    { 0xde4947e2, 3 },
    { 0xd2e947e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f147e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ed47ed, 5 },
    { 0x9bf947ee, 3 },
    { 0x964547ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7d47f3, 3 },
    { 0, 0 },
    { 0xa3dd47f5, 3 },
    { 0x62c547f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6414803, 4 },
    { 0xdecd4804, 4 },
    { 0, 0 },
    { 0x7d894806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2301480b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e5480f, 3 },
    { 0xb7914810, 4 },
    { 0, 0 },
    { 0x7dad4812, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55214815, 3 },
    { 0xdc94816, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dd481d, 3 },
    { 0, 0 },
    { 0xce25481f, 3 },
    { 0x5cd4820, 4 },
    { 0x26b54821, 4 },
    { 0xe2254822, 4 },
    { 0xa9294823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae9482d, 3 },
    { 0x711d482e, 3 },
    { 0x4309482f, 4 },
    { 0x3ccd4830, 4 },
    { 0x549d482f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d4838, 3 },
    { 0xeae54838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e5483c, 4 },
    { 0x15e5483c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70154840, 3 },
    { 0x9b714841, 4 },
    { 0x49b54842, 3 },
    { 0xf50d4842, 3 },
    { 0x8614843, 3 },
    { 0x95314844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb21484a, 3 },
    { 0x7e25484a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fd484e, 3 },
    { 0x3ccd484f, 5 },
    { 0x50394850, 3 },
    { 0x8e25484e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abd4857, 3 },
    { 0xcf094858, 3 },
    { 0x96f54859, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382d485d, 4 },
    { 0xf641485e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f94865, 5 },
    { 0x8c054865, 3 },
    { 0x34a14867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabd486a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851486e, 3 },
    { 0, 0 },
    { 0x8edd4870, 4 },
    { 0xf694870, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d4876, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd754879, 3 },
    { 0, 0 },
    { 0xe851487b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ed487e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32d4881, 3 },
    { 0, 0 },
    { 0xf6f54883, 3 },
    { 0x4add4884, 3 },
    { 0, 0 },
    { 0xdc614886, 4 },
    { 0x27c94886, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3449488a, 3 },
    { 0xc5c1488a, 3 },
    { 0xec59488c, 3 },
    { 0x85a5488b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0d4892, 3 },
    { 0x6a894893, 3 },
    { 0x90ed4892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92454898, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a5489e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f548a3, 3 },
    { 0x7fd48a4, 3 },
    { 0x431548a5, 4 },
    { 0x5a1948a6, 4 },
    { 0, 0 },
    { 0x871548a8, 3 },
    { 0xdb3948a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785548b1, 3 },
    { 0, 0 },
    { 0x5c3d48b3, 3 },
    { 0x25ad48b3, 4 },
    { 0x3d6148b3, 3 },
    { 0x8d9d48b6, 3 },
    { 0x919148b6, 3 },
    { 0, 0 },
    { 0x523148b9, 3 },
    { 0x339548ba, 3 },
    { 0x262548bb, 3 },
    { 0x7f5d48b9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8548c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x537148c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4d48cb, 3 },
    { 0, 0 },
    { 0x7e4548cd, 3 },
    { 0x311d48ce, 4 },
    { 0xefe548cf, 3 },
    { 0xbb1948cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4948d3, 3 },
    { 0x58b948d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646948d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164548de, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f148f0, 3 },
    { 0, 0 },
    { 0x819148f2, 3 },
    { 0xba0148f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a148f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5148fc, 4 },
    { 0x87c948fd, 4 },
    { 0x61c548fd, 3 },
    { 0xf6ed48ff, 4 },
    { 0x41214900, 3 },
    { 0x7ae94901, 3 },
    { 0x99b14901, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25d4906, 4 },
    { 0xe2994907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e51490b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1425490e, 3 },
    { 0x740d490f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d94912, 4 },
    { 0x26ad4912, 3 },
    { 0, 0 },
    { 0x90494915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9254918, 3 },
    { 0, 0 },
    { 0xfe29491a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715491d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77854920, 3 },
    { 0xb33d4921, 3 },
    { 0xd2654922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89492d, 3 },
    { 0, 0 },
    { 0x6351492f, 3 },
    { 0xcbc1492f, 3 },
    { 0, 0 },
    { 0xcc554932, 3 },
    { 0x298d4932, 3 },
    { 0xa0814932, 3 },
    { 0x74394932, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ad493d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95114941, 4 },
    { 0x8ead4941, 3 },
    { 0, 0 },
    { 0x4cb14944, 3 },
    { 0, 0 },
    { 0x4f754946, 3 },
    { 0, 0 },
    { 0x40594948, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc45494b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221494f, 3 },
    { 0x4d254950, 3 },
    { 0x33ad4950, 3 },
    { 0x5b414952, 3 },
    { 0x59894953, 3 },
    { 0, 0 },
    { 0xc6a94955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18354959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39d495c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d495f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87514962, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e54965, 4 },
    { 0x41894966, 3 },
    { 0x17e94967, 3 },
    { 0x8ad54968, 3 },
    { 0x7c3d4968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3394970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21614973, 4 },
    { 0, 0 },
    { 0xd8094975, 5 },
    { 0x6b554975, 3 },
    { 0, 0 },
    { 0x706d4978, 3 },
    { 0, 0 },
    { 0x7c11497a, 3 },
    { 0xfa21497a, 3 },
    { 0xb735497c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a1497f, 3 },
    { 0, 0 },
    { 0xa6d94981, 3 },
    { 0xdd814982, 4 },
    { 0xaaf54982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x676d4986, 3 },
    { 0xf69d4987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f5498d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a554993, 3 },
    { 0xc1814994, 3 },
    { 0x3cd94993, 3 },
    { 0, 0 },
    { 0x8f6d4997, 3 },
    { 0xc1e14997, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef35499b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1465499e, 3 },
    { 0x5fa9499f, 4 },
    { 0x497949a0, 4 },
    { 0x831949a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683d49a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626549ae, 3 },
    { 0, 0 },
    { 0x1b4149b0, 3 },
    { 0xecf949b0, 3 },
    { 0xa17149b2, 3 },
    { 0x1bb949b0, 3 },
    { 0x4ed49b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0949ba, 4 },
    { 0xe1c549bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bed49bf, 3 },
    { 0x3f2d49bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dd49c3, 4 },
    { 0, 0 },
    { 0xb27949c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1549c8, 3 },
    { 0xade149c9, 3 },
    { 0, 0 },
    { 0x34bd49cb, 3 },
    { 0x4fc149cb, 3 },
    { 0x7b3d49cd, 3 },
    { 0x6d6d49ce, 3 },
    { 0xd84549cf, 3 },
    { 0, 0 },
    { 0x7a7949d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e949d4, 3 },
    { 0, 0 },
    { 0xb52d49d6, 3 },
    { 0xfb949d6, 3 },
    { 0x977149d8, 3 },
    { 0x582d49d9, 5 },
    { 0x37dd49d9, 3 },
    { 0x8d1549d9, 4 },
    { 0x242549db, 4 },
    { 0, 0 },
    { 0x4c7549de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9549e6, 3 },
    { 0x6d5149e7, 5 },
    { 0x9a3949e7, 4 },
    { 0x8fa549e6, 4 },
    { 0xb86d49ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92549ed, 3 },
    { 0x814d49ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72149f1, 3 },
    { 0x649549f2, 4 },
    { 0, 0 },
    { 0x143549f4, 4 },
    { 0, 0 },
    { 0xf08d49f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2149fd, 4 },
    { 0xe79949fe, 3 },
    { 0x7af949fe, 3 },
    { 0, 0 },
    { 0x80ad4a01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d14a05, 3 },
    { 0, 0 },
    { 0x7a094a07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a94a0b, 3 },
    { 0x9b254a0c, 3 },
    { 0xadf94a0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd4a11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc914a17, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b254a1a, 3 },
    { 0xde054a1b, 3 },
    { 0x25354a1b, 3 },
    { 0x84c54a1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58414a20, 3 },
    { 0x63214a20, 4 },
    { 0x6b314a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26154a26, 3 },
    { 0xde794a27, 3 },
    { 0x9694a28, 4 },
    { 0xef794a29, 4 },
    { 0, 0 },
    { 0x9dc54a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad294a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba54a35, 3 },
    { 0, 0 },
    { 0x3d014a37, 3 },
    { 0x8e994a38, 3 },
    { 0x84dd4a39, 3 },
    { 0x71794a39, 3 },
    { 0xffed4a39, 3 },
    { 0xf6854a3c, 3 },
    { 0x636d4a3d, 3 },
    { 0x2cb14a38, 4 },
    { 0x29554a38, 3 },
    { 0x52b54a40, 3 },
    { 0x81054a41, 3 },
    { 0xd03d4a42, 3 },
    { 0x33d14a40, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb494a48, 5 },
    { 0x4ee14a49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bed4a4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c194a52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b14a59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c54a5c, 4 },
    { 0xb5454a5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3d4a63, 4 },
    { 0x68094a63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d4a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77054a6c, 4 },
    { 0xe9c94a6c, 3 },
    { 0xc84d4a6e, 3 },
    { 0x3ec14a6e, 3 },
    { 0xeff94a6c, 3 },
    { 0, 0 },
    { 0xdf314a72, 4 },
    { 0x57b94a73, 3 },
    { 0, 0 },
    { 0x73194a75, 3 },
    { 0x910d4a75, 3 },
    { 0, 0 },
    { 0xe94a78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4094a7e, 3 },
    { 0, 0 },
    { 0x5fc54a80, 3 },
    { 0, 0 },
    { 0xd8394a82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cd4a8b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65d4a8f, 3 },
    { 0, 0 },
    { 0x5c214a91, 4 },
    { 0x55194a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51554a95, 3 },
    { 0, 0 },
    { 0x1d614a97, 3 },
    { 0xed514a98, 3 },
    { 0x87c54a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0694aa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64294aaa, 3 },
    { 0, 0 },
    { 0x28f14aac, 3 },
    { 0, 0 },
    { 0xe5754aae, 3 },
    { 0, 0 },
    { 0x16d4ab0, 3 },
    { 0xd0b94ab0, 5 },
    { 0x5b294ab2, 3 },
    { 0x59f94ab2, 3 },
    { 0x7514ab4, 3 },
    { 0xfbc14ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b54aba, 3 },
    { 0x2ef94abb, 4 },
    { 0, 0 },
    { 0x92d54abd, 3 },
    { 0, 0 },
    { 0x39b54abf, 3 },
    { 0x4d514abf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77d4aca, 3 },
    { 0x754d4acb, 3 },
    { 0x2e0d4acb, 3 },
    { 0x72e94acc, 3 },
    { 0x31454ace, 3 },
    { 0xb6654acc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c994ad9, 3 },
    { 0, 0 },
    { 0x7aed4adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a54adf, 4 },
    { 0xdb494ae0, 4 },
    { 0x239d4ae1, 3 },
    { 0x185d4ae1, 3 },
    { 0, 0 },
    { 0xf1f54ae4, 3 },
    { 0, 0 },
    { 0xdc814ae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b14ae9, 4 },
    { 0xd8614aea, 4 },
    { 0, 0 },
    { 0xc5514aec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a194af2, 3 },
    { 0x37694af2, 4 },
    { 0xfbd54af4, 4 },
    { 0xf7914af5, 4 },
    { 0xd3614af5, 4 },
    { 0xb0594af7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b94afb, 4 },
    { 0x5b8d4afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f294b01, 3 },
    { 0xf8594b01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14394b05, 4 },
    { 0, 0 },
    { 0x74c54b07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a54b0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74414b0d, 3 },
    { 0x9ee14b0d, 3 },
    { 0xd4594b0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffd4b20, 4 },
    { 0x4ca54b21, 4 },
    { 0xabb54b22, 3 },
    { 0x18854b23, 3 },
    { 0xf3bd4b20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd54b27, 3 },
    { 0x7de94b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce714b30, 3 },
    { 0, 0 },
    { 0x9e514b32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc594b36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb94b3c, 3 },
    { 0, 0 },
    { 0x41994b3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8754b41, 3 },
    { 0xefcd4b42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcd4b46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b54b49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58894b4d, 4 },
    { 0x927d4b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dd4b52, 3 },
    { 0x10f54b52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece94b5a, 3 },
    { 0, 0 },
    { 0xfcd14b5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71354b5f, 4 },
    { 0xdfbd4b60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31154b63, 3 },
    { 0xad814b63, 4 },
    { 0, 0 },
    { 0xe9c14b66, 4 },
    { 0xffb14b67, 3 },
    { 0, 0 },
    { 0x28994b69, 3 },
    { 0x30114b6a, 3 },
    { 0x4fe54b6b, 4 },
    { 0x8cf14b6b, 4 },
    { 0x212d4b6d, 3 },
    { 0x4af54b6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43314b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d54b79, 4 },
    { 0x956d4b7a, 4 },
    { 0, 0 },
    { 0x978d4b7c, 3 },
    { 0, 0 },
    { 0xace14b7e, 3 },
    { 0, 0 },
    { 0xdb9d4b80, 3 },
    { 0xa6c54b81, 4 },
    { 0x91054b82, 3 },
    { 0x40f54b83, 3 },
    { 0x93b94b83, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f254b8a, 3 },
    { 0, 0 },
    { 0xa9994b8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab14b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x421d4b93, 4 },
    { 0xfca54b94, 3 },
    { 0xd094b95, 3 },
    { 0x8df14b96, 3 },
    { 0x5f794b93, 3 },
    { 0x5fe14b98, 3 },
    { 0xc7c14b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd4ba1, 3 },
    { 0x89ad4ba2, 3 },
    { 0, 0 },
    { 0xbd54ba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74514ba9, 3 },
    { 0, 0 },
    { 0xe9854bab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abd4bb1, 3 },
    { 0x18d14bb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c14bb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd94bb9, 4 },
    { 0xe4d94bba, 3 },
    { 0, 0 },
    { 0x24d14bbc, 3 },
    { 0x57654bbd, 3 },
    { 0xce794bbe, 3 },
    { 0, 0 },
    { 0x61654bc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3354bc7, 3 },
    { 0, 0 },
    { 0xd2414bc9, 3 },
    { 0x4cbd4bca, 3 },
    { 0x29194bca, 3 },
    { 0xb0914bcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9914bd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6314bda, 3 },
    { 0, 0 },
    { 0x42314bdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd4be0, 3 },
    { 0, 0 },
    { 0x16514be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c94be5, 3 },
    { 0x9a314be5, 4 },
    { 0x514d4be5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70294bea, 3 },
    { 0x768d4bea, 3 },
    { 0xb14bec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb054bf3, 4 },
    { 0, 0 },
    { 0xaf994bf5, 3 },
    { 0x19a14bf6, 3 },
    { 0x34f54bf7, 3 },
    { 0x8b0d4bf8, 3 },
    { 0x9dbd4bf8, 3 },
    { 0x5314bfa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e014bfd, 3 },
    { 0x79114bfe, 3 },
    { 0x7dd4bff, 3 },
    { 0xb30d4bfd, 3 },
    { 0x44c54bfd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0714c05, 3 },
    { 0, 0 },
    { 0x9af94c07, 3 },
    { 0x9e014c08, 3 },
    { 0xfcdd4c08, 3 },
    { 0xb0d4c08, 3 },
    { 0x1cc94c0b, 3 },
    { 0, 0 },
    { 0x71054c0d, 3 },
    { 0x42b54c0d, 3 },
    { 0, 0 },
    { 0x457d4c10, 5 },
    { 0x1f9d4c11, 3 },
    { 0x9394c12, 4 },
    { 0x5e954c13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8854c16, 3 },
    { 0x6ab94c16, 3 },
    { 0, 0 },
    { 0x5bf94c19, 3 },
    { 0x6dd4c1a, 4 },
    { 0, 0 },
    { 0x6b894c1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40514c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9114c26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73514c2b, 3 },
    { 0, 0 },
    { 0x7db54c2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9894c30, 4 },
    { 0xfd694c31, 3 },
    { 0xa25d4c31, 3 },
    { 0xb1114c33, 3 },
    { 0, 0 },
    { 0xd9494c35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8d4c3a, 3 },
    { 0x107d4c3a, 5 },
    { 0xf4514c3c, 3 },
    { 0, 0 },
    { 0x3e754c3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68614c43, 3 },
    { 0x7d314c44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d4c48, 4 },
    { 0x4a9d4c48, 3 },
    { 0, 0 },
    { 0x41714c4b, 3 },
    { 0, 0 },
    { 0x904d4c4d, 3 },
    { 0, 0 },
    { 0x43d4c4f, 3 },
    { 0xb9914c50, 4 },
    { 0xbcad4c4f, 3 },
    { 0x14a14c52, 3 },
    { 0xdf54c53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb614c57, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a114c5a, 3 },
    { 0, 0 },
    { 0x101d4c5c, 3 },
    { 0x4494c5d, 3 },
    { 0xb794c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e514c62, 3 },
    { 0x24314c62, 3 },
    { 0xf1294c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e54c67, 3 },
    { 0x23554c68, 3 },
    { 0x9c9d4c69, 3 },
    { 0, 0 },
    { 0x305d4c6b, 3 },
    { 0, 0 },
    { 0xb8054c6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7414c75, 3 },
    { 0x2d054c75, 4 },
    { 0x27dd4c77, 3 },
    { 0x46c54c78, 4 },
    { 0x639d4c76, 3 },
    { 0x27754c7a, 3 },
    { 0xa4f14c7b, 3 },
    { 0xa1b94c7c, 3 },
    { 0x2a5d4c7c, 3 },
    { 0xd9214c7b, 4 },
    { 0, 0 },
    { 0x59254c80, 3 },
    { 0x99f54c81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e14c86, 4 },
    { 0x10914c87, 4 },
    { 0x85114c87, 5 },
    { 0, 0 },
    { 0xd5354c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e894c8e, 3 },
    { 0x52354c8f, 4 },
    { 0, 0 },
    { 0xddcd4c91, 3 },
    { 0x95414c92, 4 },
    { 0xdd0d4c92, 4 },
    { 0xf8e14c94, 3 },
    { 0x1f554c95, 4 },
    { 0, 0 },
    { 0x16cd4c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b94ca0, 3 },
    { 0x8a914ca0, 3 },
    { 0x9ba54ca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe254ca6, 3 },
    { 0, 0 },
    { 0x40c54ca8, 4 },
    { 0x79494ca9, 4 },
    { 0, 0 },
    { 0x61a94cab, 4 },
    { 0, 0 },
    { 0x17e54cad, 3 },
    { 0x18914cae, 3 },
    { 0, 0 },
    { 0x8c094cb0, 4 },
    { 0, 0 },
    { 0xd814cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f154cb8, 4 },
    { 0, 0 },
    { 0xe0d14cba, 3 },
    { 0, 0 },
    { 0x74994cbc, 3 },
    { 0, 0 },
    { 0x7f314cbe, 3 },
    { 0x20ed4cbf, 4 },
    { 0, 0 },
    { 0x12f94cc1, 3 },
    { 0, 0 },
    { 0xfcdd4cc3, 3 },
    { 0, 0 },
    { 0x80014cc5, 3 },
    { 0xd8e94cc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55854ccf, 3 },
    { 0x65cd4cd0, 3 },
    { 0x50354cd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x143d4cd4, 4 },
    { 0x7db14cd4, 3 },
    { 0x5f954cd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83794ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47694ce5, 4 },
    { 0x48b14ce6, 3 },
    { 0xfdd94ce5, 3 },
    { 0, 0 },
    { 0x48d4ce9, 3 },
    { 0, 0 },
    { 0x19554ceb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b14cee, 4 },
    { 0, 0 },
    { 0x79f94cf0, 3 },
    { 0xdadd4cf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f14cf4, 3 },
    { 0xd014cf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f54cfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea94d01, 3 },
    { 0x223d4d02, 3 },
    { 0x6e314d03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf14d06, 4 },
    { 0xef4d4d06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa54d0b, 5 },
    { 0xb61d4d0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b14d0f, 3 },
    { 0x4254d0f, 4 },
    { 0x12ad4d11, 3 },
    { 0xee654d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82814d19, 4 },
    { 0, 0 },
    { 0xb8354d1b, 3 },
    { 0xb6e54d1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca754d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab754d22, 3 },
    { 0, 0 },
    { 0xdeb14d24, 3 },
    { 0, 0 },
    { 0x3a294d26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b14d2b, 5 },
    { 0x30d4d2c, 3 },
    { 0, 0 },
    { 0x75194d2e, 4 },
    { 0x4ecd4d2e, 4 },
    { 0, 0 },
    { 0x4814d31, 3 },
    { 0, 0 },
    { 0xcb994d33, 3 },
    { 0xee754d34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594d3d, 3 },
    { 0xa2d54d3e, 3 },
    { 0xaccd4d3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b754d46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92714d49, 3 },
    { 0xe79d4d4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcd4d54, 3 },
    { 0x698d4d54, 4 },
    { 0x4d6d4d54, 3 },
    { 0x2e214d56, 3 },
    { 0x22794d54, 3 },
    { 0, 0 },
    { 0xab614d5a, 3 },
    { 0x1e94d5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47614d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46654d63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a114d67, 3 },
    { 0xe7954d67, 4 },
    { 0xc8c94d69, 3 },
    { 0x9c014d67, 3 },
    { 0xd87d4d68, 3 },
    { 0xf1314d6c, 3 },
    { 0xfb594d6d, 4 },
    { 0xbad4d6d, 4 },
    { 0x5e754d6d, 3 },
    { 0x663d4d70, 3 },
    { 0xfb7d4d6f, 4 },
    { 0xe6c94d6a, 3 },
    { 0x6f114d72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85d4d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1754d7a, 3 },
    { 0, 0 },
    { 0x16a14d7c, 3 },
    { 0xd4ed4d7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70694d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39314d8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84394d94, 3 },
    { 0xa6954d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74414d98, 3 },
    { 0xc06d4d99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9d4d9d, 3 },
    { 0, 0 },
    { 0xf2854d9f, 4 },
    { 0x7114da0, 4 },
    { 0, 0 },
    { 0x18354da2, 3 },
    { 0xbf14da3, 3 },
    { 0, 0 },
    { 0x28354da5, 3 },
    { 0xd8b54da6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d4da9, 3 },
    { 0xbe654da9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372d4dae, 3 },
    { 0xee5d4daf, 3 },
    { 0x46f54db0, 3 },
    { 0x576d4db1, 3 },
    { 0x17294db2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb954db5, 3 },
    { 0, 0 },
    { 0x524d4db7, 4 },
    { 0, 0 },
    { 0x159d4db9, 3 },
    { 0, 0 },
    { 0xa6754dbb, 3 },
    { 0, 0 },
    { 0x5e214dbd, 4 },
    { 0xb8514dbe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6694dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b14dc8, 3 },
    { 0, 0 },
    { 0xac1d4dca, 3 },
    { 0xa31d4dca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9414dd3, 4 },
    { 0, 0 },
    { 0x353d4dd5, 3 },
    { 0x50814dd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95554de0, 3 },
    { 0x9f54de0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554de7, 3 },
    { 0xf69d4de7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94854df1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720d4df5, 3 },
    { 0x62214df5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cd4dfa, 4 },
    { 0xc3094dfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a94dfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a114e04, 3 },
    { 0x1cb94e04, 3 },
    { 0x94f54e04, 3 },
    { 0x4a2d4e04, 5 },
    { 0x7ea94e08, 5 },
    { 0xf7494e09, 3 },
    { 0xc68d4e05, 3 },
    { 0x511d4e0b, 3 },
    { 0x4eed4e0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2854e11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f94e16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd54e1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80994e1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d4e24, 5 },
    { 0x73a94e25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e54e2d, 4 },
    { 0x12114e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f54e32, 3 },
    { 0xc1b14e33, 3 },
    { 0xaf1d4e34, 4 },
    { 0x6b414e35, 5 },
    { 0, 0 },
    { 0x56e54e37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcd4e3c, 3 },
    { 0xab514e3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62054e41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89794e44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c54e48, 3 },
    { 0x61454e49, 3 },
    { 0xec114e4a, 3 },
    { 0x50814e4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4294e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f14e55, 3 },
    { 0, 0 },
    { 0xe14e57, 3 },
    { 0x19c54e58, 4 },
    { 0, 0 },
    { 0x9c094e5a, 4 },
    { 0xba1d4e5b, 4 },
    { 0x2fbd4e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e94e5f, 3 },
    { 0, 0 },
    { 0x38294e61, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5214e64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425d4e6a, 3 },
    { 0x9ba94e6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c14e6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a14e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33514e7c, 3 },
    { 0x7e794e7d, 3 },
    { 0, 0 },
    { 0x9d494e7f, 3 },
    { 0x7aed4e7f, 3 },
    { 0x3bbd4e81, 3 },
    { 0, 0 },
    { 0x73994e83, 3 },
    { 0x72654e84, 3 },
    { 0xcc954e83, 3 },
    { 0x40d14e86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca694e90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d54e94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c054e9a, 3 },
    { 0x7ed94e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f414ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125d4ea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580d4eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab54eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20894eb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc054ebb, 3 },
    { 0xb9794ebb, 3 },
    { 0x65b54ebb, 3 },
    { 0x5c494ebc, 5 },
    { 0x6d5d4ebc, 4 },
    { 0x75714ec0, 3 },
    { 0x6cb94ec0, 3 },
    { 0, 0 },
    { 0xd1014ec3, 3 },
    { 0, 0 },
    { 0xd3614ec5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x49554ec8, 5 },
    { 0xbbad4ec8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4ecc, 3 },
    { 0x3f4d4ecd, 3 },
    { 0xf4494ecd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd4ed2, 3 },
    { 0xb6a14ed3, 3 },
    { 0x6af54ed4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d4ed8, 3 },
    { 0x21414ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x523d4edc, 3 },
    { 0, 0 },
    { 0x1dcd4ede, 3 },
    { 0, 0 },
    { 0x961d4ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf614ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95914ee9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9d4eef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd254ef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d4ef7, 3 },
    { 0xcd994ef7, 3 },
    { 0xf0714ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd4efc, 3 },
    { 0, 0 },
    { 0x48114efe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca54f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e794f11, 4 },
    { 0, 0 },
    { 0x5f854f13, 4 },
    { 0xd7994f13, 3 },
    { 0xc0b94f15, 3 },
    { 0x78894f16, 4 },
    { 0, 0 },
    { 0x3dd4f18, 3 },
    { 0xccc94f19, 3 },
    { 0x7b354f1a, 3 },
    { 0x5b7d4f19, 3 },
    { 0, 0 },
    { 0x94254f1d, 3 },
    { 0, 0 },
    { 0x13954f1f, 3 },
    { 0x9f494f20, 3 },
    { 0xa1d14f21, 3 },
    { 0x4aed4f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5494f25, 3 },
    { 0xc07d4f26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa614f2a, 3 },
    { 0, 0 },
    { 0x9f414f2c, 3 },
    { 0xc4854f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371d4f33, 4 },
    { 0, 0 },
    { 0x575d4f35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2714f39, 3 },
    { 0x27d14f3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f94f3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721d4f42, 3 },
    { 0, 0 },
    { 0x115d4f44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94714f47, 3 },
    { 0x10114f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8d4f53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5494f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e14f5b, 4 },
    { 0xf4894f5b, 3 },
    { 0x26f94f5c, 4 },
    { 0x96694f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77094f64, 4 },
    { 0, 0 },
    { 0x5fe14f66, 3 },
    { 0, 0 },
    { 0xf3754f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf94f6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87394f6f, 3 },
    { 0x1efd4f70, 3 },
    { 0xd5594f70, 3 },
    { 0x28214f72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd4f79, 4 },
    { 0x418d4f79, 4 },
    { 0, 0 },
    { 0xc5bd4f7c, 5 },
    { 0, 0 },
    { 0xa6694f7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b94f84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d4f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d54f8b, 3 },
    { 0, 0 },
    { 0xb2414f8d, 3 },
    { 0, 0 },
    { 0xf154f8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a714f98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5814f9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee94fa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7254fa6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf14faa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6d4fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e554fb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e54fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d4fc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373d4fc4, 4 },
    { 0xd3614fc5, 3 },
    { 0x9c554fc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89994fd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dd4fd4, 3 },
    { 0, 0 },
    { 0xab7d4fd6, 3 },
    { 0xfe714fd7, 3 },
    { 0x1c754fd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25854fdf, 3 },
    { 0xb2fd4fe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c54fe4, 3 },
    { 0xeb294fe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe14fe9, 3 },
    { 0x365d4fea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4fee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d654ff1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6814ff9, 3 },
    { 0xe2cd4ffa, 4 },
    { 0x36194ffb, 3 },
    { 0xb3b14ffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4814fff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cd5002, 4 },
    { 0, 0 },
    { 0xf4f15004, 4 },
    { 0, 0 },
    { 0x85195006, 3 },
    { 0xf8055006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd85500a, 3 },
    { 0, 0 },
    { 0x555d500c, 4 },
    { 0x6abd500d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee95010, 4 },
    { 0x39a95011, 3 },
    { 0xcb415012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74595016, 3 },
    { 0xc1595016, 3 },
    { 0xc2c55017, 3 },
    { 0x82095019, 4 },
    { 0xe961501a, 4 },
    { 0x7435501a, 4 },
    { 0xb87d501a, 3 },
    { 0x1b91501d, 3 },
    { 0x29d501e, 3 },
    { 0x887d501d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56195022, 4 },
    { 0x3e015023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28895028, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e9502e, 5 },
    { 0x6b1d502f, 4 },
    { 0xd8e15030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489d5035, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c515040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58395043, 3 },
    { 0, 0 },
    { 0xa755045, 3 },
    { 0x30555046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3709504c, 3 },
    { 0, 0 },
    { 0xbafd504e, 3 },
    { 0xaed9504f, 4 },
    { 0, 0 },
    { 0x75a55051, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d5054, 3 },
    { 0, 0 },
    { 0x6155056, 4 },
    { 0, 0 },
    { 0x63115058, 3 },
    { 0xc0795059, 3 },
    { 0xc0515058, 4 },
    { 0xc4915059, 3 },
    { 0, 0 },
    { 0x1dcd505d, 4 },
    { 0x226d505d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a15061, 4 },
    { 0x77ed5061, 3 },
    { 0, 0 },
    { 0x5a9d5064, 3 },
    { 0x69355065, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d055069, 3 },
    { 0xf909506a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f9506d, 3 },
    { 0x6705506e, 3 },
    { 0, 0 },
    { 0x8e495070, 4 },
    { 0x99c55071, 3 },
    { 0, 0 },
    { 0xcf395073, 3 },
    { 0xab115074, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fd507a, 3 },
    { 0x2815507a, 3 },
    { 0x37d507c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251507f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d95085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8751508c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db5508f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d5094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed55098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde5509e, 4 },
    { 0x92e5509e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e950a2, 3 },
    { 0xad5950a3, 4 },
    { 0, 0 },
    { 0x400550a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2550aa, 3 },
    { 0x8a7950ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad550b0, 3 },
    { 0x145d50b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7950ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6d50c0, 3 },
    { 0x40b150c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c550c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7d50cc, 3 },
    { 0x6c4550cd, 3 },
    { 0x9d3d50cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e950d1, 3 },
    { 0x5fd550d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f550db, 3 },
    { 0x209550db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ad50e3, 3 },
    { 0, 0 },
    { 0x324550e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d150ed, 3 },
    { 0x181950ee, 4 },
    { 0, 0 },
    { 0x63f150f0, 4 },
    { 0xe3f150f1, 4 },
    { 0, 0 },
    { 0xb2bd50f3, 3 },
    { 0x844d50f4, 3 },
    { 0, 0 },
    { 0xac8150f6, 4 },
    { 0xd8b150f6, 3 },
    { 0x9c2150f8, 4 },
    { 0x92f950f9, 3 },
    { 0x45a550f7, 3 },
    { 0xc2c950fb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52150fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10395101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab895104, 3 },
    { 0, 0 },
    { 0xe6415106, 4 },
    { 0x4c755107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6171510e, 3 },
    { 0x9389510f, 3 },
    { 0x55695110, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95015114, 3 },
    { 0xbf495114, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b15119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x174d511c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9415120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e55128, 3 },
    { 0xe5755129, 3 },
    { 0, 0 },
    { 0x858d512b, 3 },
    { 0xcb11512c, 3 },
    { 0, 0 },
    { 0x4569512e, 3 },
    { 0, 0 },
    { 0x34295130, 3 },
    { 0xba695131, 3 },
    { 0x23a95131, 3 },
    { 0xc2b95132, 3 },
    { 0, 0 },
    { 0x9195135, 3 },
    { 0xd5a15135, 4 },
    { 0x5d455137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d513a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d513d, 3 },
    { 0xc74d513d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6d5141, 3 },
    { 0x16015142, 3 },
    { 0x16015141, 3 },
    { 0xab615144, 3 },
    { 0x8cd55144, 3 },
    { 0xe0655144, 3 },
    { 0x24d15143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52d514d, 3 },
    { 0, 0 },
    { 0xb9d1514f, 3 },
    { 0, 0 },
    { 0x9c7d5151, 3 },
    { 0xc0c55152, 3 },
    { 0, 0 },
    { 0xfe55154, 3 },
    { 0xb1055155, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8495158, 3 },
    { 0xbbd15158, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cd5161, 3 },
    { 0x9cf95162, 3 },
    { 0x94955163, 4 },
    { 0xc1315164, 3 },
    { 0x91555163, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4039516e, 3 },
    { 0, 0 },
    { 0x937d5170, 3 },
    { 0xb0bd5171, 4 },
    { 0x12a95172, 3 },
    { 0x4a415171, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12555178, 3 },
    { 0xcb255178, 3 },
    { 0, 0 },
    { 0xe605517b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882d5181, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d95185, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7029518a, 3 },
    { 0x7a6d518a, 3 },
    { 0xc40d518c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d5191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8115194, 3 },
    { 0x71355194, 3 },
    { 0xa9715196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26d5199, 4 },
    { 0, 0 },
    { 0x4441519b, 3 },
    { 0x8119519c, 3 },
    { 0xa4dd519d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50151a3, 3 },
    { 0xd20551a4, 3 },
    { 0xd2d551a5, 3 },
    { 0, 0 },
    { 0x588951a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d51aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493d51b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46951b6, 3 },
    { 0xa06951b7, 4 },
    { 0xc71d51b8, 4 },
    { 0xedad51b9, 4 },
    { 0, 0 },
    { 0x141951bb, 3 },
    { 0, 0 },
    { 0xe30951bd, 3 },
    { 0x9b5d51be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39151c6, 3 },
    { 0x4cb151c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d51ca, 3 },
    { 0, 0 },
    { 0x917d51cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6551d0, 4 },
    { 0x96e951d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7951d5, 3 },
    { 0xeb7151d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984d51da, 4 },
    { 0x4e1151db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22951de, 3 },
    { 0xb0d551de, 4 },
    { 0xa1551e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x805151e3, 3 },
    { 0xcb4551e3, 3 },
    { 0xa40551e5, 3 },
    { 0x297151e6, 3 },
    { 0xf3e551e4, 3 },
    { 0x8b7151e7, 3 },
    { 0x223151e9, 3 },
    { 0, 0 },
    { 0x141d51eb, 3 },
    { 0x21d551ec, 4 },
    { 0x73c551ed, 3 },
    { 0x25a951ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc551f2, 3 },
    { 0xb40551f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238551f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ad51fd, 4 },
    { 0xdd0d51fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c55202, 3 },
    { 0xdf95202, 3 },
    { 0, 0 },
    { 0x48bd5205, 3 },
    { 0, 0 },
    { 0xb7b15207, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3139520b, 3 },
    { 0x78d9520c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae15212, 4 },
    { 0xa5f95213, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef95216, 3 },
    { 0xd1a95217, 3 },
    { 0x6fc95216, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37d521b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a915222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c995228, 4 },
    { 0x90dd5229, 3 },
    { 0xa6d9522a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9522e, 3 },
    { 0x8e7d522f, 3 },
    { 0, 0 },
    { 0xade55231, 3 },
    { 0xd1315232, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72795236, 3 },
    { 0xbe495237, 3 },
    { 0x70f55237, 3 },
    { 0x25215237, 3 },
    { 0xf0415238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5d523e, 3 },
    { 0x8159523e, 3 },
    { 0, 0 },
    { 0xf9a15241, 4 },
    { 0xeff15241, 3 },
    { 0x2355242, 3 },
    { 0xd42d5244, 3 },
    { 0xd2415245, 3 },
    { 0xfa015246, 3 },
    { 0x3fc15247, 3 },
    { 0x87b15242, 4 },
    { 0x68715249, 3 },
    { 0x50ad5249, 3 },
    { 0x1285524b, 4 },
    { 0x67bd5249, 3 },
    { 0xa5195241, 3 },
    { 0x4059524e, 3 },
    { 0x5cad524d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9295254, 3 },
    { 0, 0 },
    { 0xc6195256, 4 },
    { 0, 0 },
    { 0xb07d5258, 3 },
    { 0, 0 },
    { 0x5ad525a, 3 },
    { 0x6c01525b, 4 },
    { 0xfb7d525c, 4 },
    { 0xe011525d, 3 },
    { 0xd4c5525e, 4 },
    { 0, 0 },
    { 0x8b055260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e295264, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d5268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d15526b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb1526e, 4 },
    { 0, 0 },
    { 0xd60d5270, 3 },
    { 0, 0 },
    { 0x6ee15272, 3 },
    { 0xd0215273, 3 },
    { 0, 0 },
    { 0xc4e55275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28155279, 3 },
    { 0, 0 },
    { 0x8d61527b, 3 },
    { 0, 0 },
    { 0xe999527d, 3 },
    { 0x5ee5527e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d5283, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd55288, 3 },
    { 0, 0 },
    { 0x1175528a, 3 },
    { 0, 0 },
    { 0x3d2d528c, 3 },
    { 0x471528c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4d5291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda515294, 3 },
    { 0, 0 },
    { 0x3bb55296, 3 },
    { 0x92455297, 4 },
    { 0x7b2d5298, 3 },
    { 0xa1415297, 3 },
    { 0, 0 },
    { 0x17c5529b, 3 },
    { 0x18e1529b, 4 },
    { 0, 0 },
    { 0x39f9529e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7952a1, 3 },
    { 0xcc152a2, 3 },
    { 0xa66152a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b952b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dd52b6, 4 },
    { 0x473d52b7, 3 },
    { 0x616152b7, 3 },
    { 0xdb552b7, 3 },
    { 0x947152b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6952c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a552c5, 3 },
    { 0x391152c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1952c9, 4 },
    { 0x961152c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ed52d0, 3 },
    { 0x3e1552d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44552d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934d52dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c952df, 3 },
    { 0x9ec952e0, 3 },
    { 0, 0 },
    { 0x3f2952e2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418152e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e152ed, 3 },
    { 0x7ffd52ed, 3 },
    { 0xef0d52ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d952f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564952f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518952fa, 4 },
    { 0x554d52fa, 3 },
    { 0xfdd152fa, 3 },
    { 0xe1e952fd, 3 },
    { 0x81f552fe, 4 },
    { 0, 0 },
    { 0x61315300, 3 },
    { 0x49f15300, 3 },
    { 0xe76d5302, 3 },
    { 0x9a595303, 3 },
    { 0xe2315304, 3 },
    { 0, 0 },
    { 0x47855306, 3 },
    { 0xca715307, 3 },
    { 0, 0 },
    { 0x4e395309, 3 },
    { 0xb8655309, 3 },
    { 0x6109530b, 3 },
    { 0, 0 },
    { 0xf251530d, 3 },
    { 0x42f5530e, 3 },
    { 0xf3a9530f, 3 },
    { 0x8d815310, 3 },
    { 0x12f55310, 4 },
    { 0x18055312, 3 },
    { 0xe66d5313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63855318, 4 },
    { 0xb3a15319, 3 },
    { 0xb671531a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4711531e, 4 },
    { 0, 0 },
    { 0xf715320, 3 },
    { 0xac055320, 3 },
    { 0xe2b15322, 4 },
    { 0x24155321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d5326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795532e, 4 },
    { 0x70e5532f, 3 },
    { 0x5d65532e, 3 },
    { 0x9be55330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b915337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afd533c, 3 },
    { 0x5839533c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ad5340, 3 },
    { 0, 0 },
    { 0x9da55342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff55348, 3 },
    { 0, 0 },
    { 0xe205534a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e29534e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ad5351, 4 },
    { 0xa1855352, 3 },
    { 0xd895352, 3 },
    { 0x8fe95354, 3 },
    { 0x81815354, 3 },
    { 0xc2255351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7655359, 3 },
    { 0x92d55359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c5535d, 4 },
    { 0, 0 },
    { 0x27e9535f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d5362, 3 },
    { 0x32b95363, 3 },
    { 0xe46d5364, 4 },
    { 0x6f55363, 3 },
    { 0x76d5364, 3 },
    { 0xbf755367, 3 },
    { 0xb7a15368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x859d536b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345536e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d5373, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64295377, 3 },
    { 0, 0 },
    { 0xec995379, 3 },
    { 0, 0 },
    { 0x8461537b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c55384, 4 },
    { 0, 0 },
    { 0xf91d5386, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x46455389, 3 },
    { 0, 0 },
    { 0xfdfd538b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8289538f, 3 },
    { 0xfda95390, 3 },
    { 0x4a895390, 3 },
    { 0, 0 },
    { 0x11d5393, 3 },
    { 0, 0 },
    { 0x4ea15395, 3 },
    { 0x647d5396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcd539a, 3 },
    { 0x2bf1539b, 3 },
    { 0x6bb5539a, 3 },
    { 0x6479539d, 5 },
    { 0x2ecd539e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96153a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07553b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5553b5, 3 },
    { 0, 0 },
    { 0xaabd53b7, 3 },
    { 0xd29d53b8, 3 },
    { 0x84553b9, 3 },
    { 0xac4d53ba, 3 },
    { 0, 0 },
    { 0x570d53bc, 3 },
    { 0, 0 },
    { 0xb35153be, 3 },
    { 0x6b153bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c553c2, 4 },
    { 0x99d53c3, 3 },
    { 0xea6153c4, 3 },
    { 0x674d53c5, 3 },
    { 0xbd2953c6, 3 },
    { 0, 0 },
    { 0xd7a953c8, 3 },
    { 0x590d53c9, 3 },
    { 0xc9e553ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3553d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd53d9, 4 },
    { 0x378d53d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f553dd, 4 },
    { 0x8b5153de, 3 },
    { 0x72e953de, 3 },
    { 0xd3e553de, 3 },
    { 0x9c7d53e1, 3 },
    { 0, 0 },
    { 0xe1dd53e3, 3 },
    { 0x533553e4, 3 },
    { 0, 0 },
    { 0x15953e6, 3 },
    { 0xb55553e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92553ea, 4 },
    { 0xe7e553ea, 3 },
    { 0x23b553eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2953ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f153f2, 3 },
    { 0x223553f2, 3 },
    { 0xa97d53f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969153f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f153ff, 3 },
    { 0, 0 },
    { 0x53b15401, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd5405, 4 },
    { 0xd995405, 3 },
    { 0x23655405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf315540e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bd5413, 3 },
    { 0x79d55414, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x298d5417, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d75541d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c595420, 3 },
    { 0xc7815421, 4 },
    { 0x2d2d5422, 3 },
    { 0x94495421, 3 },
    { 0, 0 },
    { 0x2fc55425, 4 },
    { 0x4c315426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267d542a, 3 },
    { 0, 0 },
    { 0x7d41542c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd95434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d543b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5543e, 4 },
    { 0, 0 },
    { 0x77855440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d5443, 4 },
    { 0x6f715444, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b515448, 4 },
    { 0x3ddd5449, 3 },
    { 0xdf255448, 3 },
    { 0xc6c1544a, 3 },
    { 0x30dd544c, 3 },
    { 0xcc09544b, 4 },
    { 0, 0 },
    { 0xc21d544f, 3 },
    { 0x8521544f, 5 },
    { 0x88e1544f, 3 },
    { 0, 0 },
    { 0xd6c55453, 3 },
    { 0x17695454, 4 },
    { 0xf955455, 3 },
    { 0x73355456, 3 },
    { 0x24fd5456, 4 },
    { 0x473d5458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec9545b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2415460, 3 },
    { 0, 0 },
    { 0xa2355462, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9d5465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb521546c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9515471, 3 },
    { 0, 0 },
    { 0xb96d5473, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f15477, 3 },
    { 0x3a2d5478, 3 },
    { 0xe1255479, 3 },
    { 0x3a0d5479, 3 },
    { 0x50695479, 3 },
    { 0, 0 },
    { 0xc549547d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a855481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6115484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5487, 4 },
    { 0xcba95488, 4 },
    { 0x2b495488, 3 },
    { 0x9f955489, 4 },
    { 0x7fb1548b, 4 },
    { 0x2011548c, 4 },
    { 0x77fd548d, 3 },
    { 0x8eb55488, 3 },
    { 0x9f6d548f, 5 },
    { 0x57d5548d, 3 },
    { 0xe94d5490, 4 },
    { 0, 0 },
    { 0xa46d5493, 3 },
    { 0, 0 },
    { 0x373d5495, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86d5499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0154a0, 3 },
    { 0x887954a0, 4 },
    { 0x608554a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4554a7, 4 },
    { 0x277954a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348154ad, 4 },
    { 0xb8f554ad, 3 },
    { 0xee8954af, 3 },
    { 0xff9d54b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64154b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedd54b8, 3 },
    { 0x1d3554b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841954bf, 3 },
    { 0, 0 },
    { 0x7e1954c1, 4 },
    { 0x40ed54c2, 3 },
    { 0x9dbd54c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159554ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47954d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x351554d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc954de, 4 },
    { 0, 0 },
    { 0x4fc954e0, 3 },
    { 0xa27154e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fd54eb, 3 },
    { 0xeffd54ec, 3 },
    { 0, 0 },
    { 0xb29954ee, 3 },
    { 0x96c954ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x537954f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9d54f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b154fb, 3 },
    { 0x5b3154fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84554ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cd5502, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f55505, 3 },
    { 0x20c55506, 3 },
    { 0xd2f55507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f1550b, 3 },
    { 0x5dd9550c, 4 },
    { 0x1b6d550d, 3 },
    { 0xacb1550d, 4 },
    { 0x6535550c, 3 },
    { 0x476d550f, 3 },
    { 0xa0c1550c, 3 },
    { 0x7e595511, 3 },
    { 0xc4195512, 4 },
    { 0, 0 },
    { 0x2eed5515, 4 },
    { 0x3aad5516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f1551b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58715521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1415525, 3 },
    { 0xd5f15526, 3 },
    { 0, 0 },
    { 0x62915528, 3 },
    { 0, 0 },
    { 0xff65552a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d5530, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7395534, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9155538, 3 },
    { 0, 0 },
    { 0x5b2d553a, 3 },
    { 0xda59553a, 3 },
    { 0, 0 },
    { 0x3f81553d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a95541, 4 },
    { 0, 0 },
    { 0xe4dd5543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdd554d, 3 },
    { 0, 0 },
    { 0x4ec1554f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b95552, 3 },
    { 0x933d5553, 3 },
    { 0x25655554, 3 },
    { 0x78c15555, 3 },
    { 0, 0 },
    { 0xe96d5557, 3 },
    { 0x9dbd5558, 3 },
    { 0, 0 },
    { 0x50d1555a, 4 },
    { 0x5275555a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d555f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e655563, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d315566, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcd556a, 3 },
    { 0x8331556b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7539556e, 3 },
    { 0x7b09556f, 3 },
    { 0x52995570, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe095574, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x866d5577, 3 },
    { 0x4fa55578, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a115582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35255588, 4 },
    { 0, 0 },
    { 0x161558a, 3 },
    { 0x7bd1558b, 4 },
    { 0x4e5558c, 4 },
    { 0x94c9558d, 4 },
    { 0x13b9558c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c55591, 3 },
    { 0x9a455592, 3 },
    { 0, 0 },
    { 0x88955594, 3 },
    { 0xd195595, 3 },
    { 0xec415596, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9555a5, 3 },
    { 0xf54955a5, 3 },
    { 0, 0 },
    { 0x61c155a8, 4 },
    { 0x3f8555a9, 4 },
    { 0x3dc155aa, 4 },
    { 0xfa955aa, 3 },
    { 0xc24155a9, 3 },
    { 0xdd8555ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b955b3, 3 },
    { 0x79bd55b4, 4 },
    { 0x891d55b5, 3 },
    { 0xe06d55b6, 3 },
    { 0x8d3d55b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd55bb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14155bf, 3 },
    { 0x2e9155c0, 4 },
    { 0xa6e555c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12955c4, 3 },
    { 0xe7e555c5, 3 },
    { 0xb2d55c5, 3 },
    { 0xa70d55c7, 4 },
    { 0, 0 },
    { 0x3b0555c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ad55d0, 3 },
    { 0x78f955d1, 4 },
    { 0xba8155d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517d55d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d55d9, 3 },
    { 0x65a955da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9155e2, 3 },
    { 0, 0 },
    { 0xe5cd55e4, 3 },
    { 0, 0 },
    { 0xf9b155e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7555eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10d55ee, 3 },
    { 0xf1b955ef, 4 },
    { 0x847955f0, 3 },
    { 0x59d955ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d55f8, 3 },
    { 0, 0 },
    { 0x100155fa, 3 },
    { 0xec5555fb, 4 },
    { 0xb49155fa, 3 },
    { 0xd8e155fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59615602, 3 },
    { 0x6b55602, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e55607, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe311560e, 3 },
    { 0x5b09560f, 3 },
    { 0, 0 },
    { 0xe79d5611, 3 },
    { 0x6a055612, 3 },
    { 0xe7e15611, 5 },
    { 0x29a15614, 3 },
    { 0x4d3d5612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6755618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d5561b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a49561e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c95621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e495627, 3 },
    { 0x4a495628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a1562b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd825562f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e095636, 3 },
    { 0, 0 },
    { 0x96f95638, 3 },
    { 0, 0 },
    { 0xdd81563a, 3 },
    { 0, 0 },
    { 0xcf2d563c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1955640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d55644, 3 },
    { 0x14cd5645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfd5648, 4 },
    { 0, 0 },
    { 0x8f31564a, 4 },
    { 0xa065564b, 3 },
    { 0x82d564c, 3 },
    { 0xf205564d, 4 },
    { 0x2b9d564e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c95651, 3 },
    { 0x9af15652, 4 },
    { 0xa2555653, 3 },
    { 0x60655654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117d5659, 3 },
    { 0xeb25565a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8311565e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x895d5661, 3 },
    { 0xbf1d5662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a55665, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d95668, 3 },
    { 0x88995669, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c9566f, 3 },
    { 0xf20d5670, 3 },
    { 0, 0 },
    { 0x1a1d5672, 3 },
    { 0x92e55673, 3 },
    { 0xcb0d5674, 3 },
    { 0x6fad5675, 3 },
    { 0x60a55676, 5 },
    { 0x9d95677, 3 },
    { 0xf5d55676, 3 },
    { 0x6a595677, 3 },
    { 0x76755677, 3 },
    { 0xd82d567a, 3 },
    { 0, 0 },
    { 0x2bad567d, 4 },
    { 0x7d75567d, 3 },
    { 0xfc0d567e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d5684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93755687, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2069568a, 4 },
    { 0x6c69568b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe1568f, 3 },
    { 0x71255690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fad5698, 3 },
    { 0xda55699, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0156a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867556a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9d56ab, 5 },
    { 0, 0 },
    { 0x84ad56ad, 3 },
    { 0xf31956ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d156b5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668156bb, 4 },
    { 0xdcc556bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d56c3, 4 },
    { 0xef8156c4, 4 },
    { 0x4b4556c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf156c8, 3 },
    { 0xe56d56c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538556ce, 3 },
    { 0, 0 },
    { 0x1df556d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5956d5, 3 },
    { 0x2da556d6, 4 },
    { 0x50a556d6, 4 },
    { 0, 0 },
    { 0x235956d9, 3 },
    { 0, 0 },
    { 0x8acd56db, 3 },
    { 0x8db556dc, 3 },
    { 0xf5c556dc, 4 },
    { 0xec5d56de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x352556e1, 3 },
    { 0, 0 },
    { 0xb1b156e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a556e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374d56f1, 4 },
    { 0x72ad56f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246d56f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7556fb, 3 },
    { 0x1de156fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25156ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5195702, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47615707, 3 },
    { 0x3fdd5708, 3 },
    { 0x296d5707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf505570e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60055711, 3 },
    { 0, 0 },
    { 0xdd7d5713, 4 },
    { 0, 0 },
    { 0x1cfd5715, 3 },
    { 0, 0 },
    { 0xc5795717, 3 },
    { 0, 0 },
    { 0x30b15719, 3 },
    { 0xb82d5719, 3 },
    { 0x9139571a, 4 },
    { 0x9c85571c, 4 },
    { 0x318d571d, 3 },
    { 0, 0 },
    { 0x870d571f, 3 },
    { 0xe601571f, 4 },
    { 0, 0 },
    { 0xe7fd5722, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc55726, 3 },
    { 0, 0 },
    { 0x6b355728, 3 },
    { 0, 0 },
    { 0x553d572a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1572e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde295734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e95739, 3 },
    { 0xf8f1573a, 3 },
    { 0x6709573b, 4 },
    { 0x3e3d573a, 3 },
    { 0x7fc1573d, 5 },
    { 0x5055573e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c95741, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7415746, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3155574f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe095755, 3 },
    { 0xc4f95755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e1575a, 3 },
    { 0xad81575a, 3 },
    { 0, 0 },
    { 0xba09575d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded576a, 3 },
    { 0, 0 },
    { 0xd32d576c, 3 },
    { 0xc579576d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b95772, 3 },
    { 0, 0 },
    { 0x613d5774, 3 },
    { 0, 0 },
    { 0xfe895776, 3 },
    { 0, 0 },
    { 0x37595778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c9577b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x346d577e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a955782, 3 },
    { 0xda755783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b55786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2951578c, 3 },
    { 0x1d1578d, 3 },
    { 0xf4a5578e, 4 },
    { 0, 0 },
    { 0x73f55790, 3 },
    { 0x3b95791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10195796, 3 },
    { 0, 0 },
    { 0x12f95798, 3 },
    { 0, 0 },
    { 0xabc1579a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd655579d, 3 },
    { 0x5fb1579d, 3 },
    { 0x8a5579f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x457557a2, 4 },
    { 0x88d57a3, 4 },
    { 0x423557a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d57aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35957ad, 4 },
    { 0x8ec157ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a557b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0957b4, 4 },
    { 0, 0 },
    { 0xe3957b6, 3 },
    { 0x2dd57b7, 3 },
    { 0, 0 },
    { 0x487957b9, 3 },
    { 0x1cf957ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1d57bd, 3 },
    { 0x2dd157be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af157c3, 3 },
    { 0x40bd57c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8157c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57557ce, 4 },
    { 0x172957ce, 3 },
    { 0x95a957d0, 4 },
    { 0x968d57d1, 3 },
    { 0xd60957d0, 4 },
    { 0x903157d3, 3 },
    { 0x98d157d4, 3 },
    { 0, 0 },
    { 0xea2d57d6, 3 },
    { 0x2bf157d7, 3 },
    { 0xe63957d8, 3 },
    { 0x108157d9, 4 },
    { 0, 0 },
    { 0x745d57db, 3 },
    { 0xee5557dc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83157df, 3 },
    { 0x11c557df, 3 },
    { 0xc71157e1, 3 },
    { 0x147d57e2, 3 },
    { 0xffa157e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60157e7, 4 },
    { 0x502d57e8, 3 },
    { 0, 0 },
    { 0xfe957ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde157f6, 3 },
    { 0x45b957f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33557fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd5805, 4 },
    { 0, 0 },
    { 0x92855807, 3 },
    { 0x4ba55808, 3 },
    { 0x82315809, 4 },
    { 0xa3dd580a, 4 },
    { 0x33d9580a, 3 },
    { 0xf3c5580a, 4 },
    { 0x3929580d, 3 },
    { 0, 0 },
    { 0x9cd1580f, 3 },
    { 0xb6ad5810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9655815, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df15825, 4 },
    { 0xa9415826, 3 },
    { 0x6da15827, 4 },
    { 0xaacd5827, 3 },
    { 0x76355829, 3 },
    { 0, 0 },
    { 0x4351582b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbd582f, 3 },
    { 0xab115830, 4 },
    { 0x1ef9582f, 3 },
    { 0x6d015832, 3 },
    { 0x70455832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29155838, 3 },
    { 0xf6255838, 3 },
    { 0, 0 },
    { 0xb23d583b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce815841, 4 },
    { 0xe9c55842, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1055845, 3 },
    { 0xa1055845, 3 },
    { 0x10f55846, 3 },
    { 0x3ee95848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8045584d, 3 },
    { 0x6d39584e, 3 },
    { 0x28b1584f, 3 },
    { 0x3c95850, 4 },
    { 0x326d5851, 3 },
    { 0x74515850, 3 },
    { 0xf349584d, 3 },
    { 0xe2115854, 3 },
    { 0, 0 },
    { 0xe4315856, 3 },
    { 0x6d315857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c55860, 3 },
    { 0x64e95861, 3 },
    { 0x36c95860, 4 },
    { 0, 0 },
    { 0x48ed5864, 3 },
    { 0, 0 },
    { 0xc6495866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771586c, 3 },
    { 0, 0 },
    { 0x8db5586e, 3 },
    { 0xdd85586e, 3 },
    { 0x38455870, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdd5874, 3 },
    { 0xc8cd5875, 4 },
    { 0xe3555874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c15879, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c51587c, 4 },
    { 0x3aa9587c, 3 },
    { 0, 0 },
    { 0xc01587f, 3 },
    { 0, 0 },
    { 0xb31d5881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c15887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951588c, 3 },
    { 0xaf49588c, 3 },
    { 0x3b4d588d, 3 },
    { 0xce41588f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69d5893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c515896, 3 },
    { 0xd0595896, 3 },
    { 0x42215896, 3 },
    { 0, 0 },
    { 0x6a4d589a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd589f, 3 },
    { 0, 0 },
    { 0x661958a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884958ab, 3 },
    { 0x2c3958ac, 3 },
    { 0x9cb158ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x333158b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486958b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4158b7, 3 },
    { 0x639958b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2558bb, 3 },
    { 0, 0 },
    { 0x65a958bd, 3 },
    { 0x864958bd, 3 },
    { 0x70a558bf, 3 },
    { 0xef2d58c0, 3 },
    { 0x82f558c1, 3 },
    { 0xd5a958c0, 4 },
    { 0, 0 },
    { 0x712d58c4, 4 },
    { 0xf6c958c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6158cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5d58d1, 3 },
    { 0xb55d58d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dd58d9, 3 },
    { 0x78c958da, 3 },
    { 0x275558db, 3 },
    { 0x1d2158dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51958e1, 3 },
    { 0x86c158e2, 3 },
    { 0x24a958e1, 3 },
    { 0x8a3d58e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5158e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71958eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5d58f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd958f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba958fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33158fd, 3 },
    { 0x19fd58fe, 3 },
    { 0x6b9558fd, 3 },
    { 0, 0 },
    { 0x89955901, 3 },
    { 0xf4bd5902, 3 },
    { 0x4a615903, 3 },
    { 0, 0 },
    { 0x38215905, 3 },
    { 0, 0 },
    { 0x6bed5907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6049590a, 3 },
    { 0x8e1590a, 3 },
    { 0xb77d590c, 3 },
    { 0x4eb9590c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc395910, 4 },
    { 0x11055911, 3 },
    { 0, 0 },
    { 0xe1215913, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec95917, 3 },
    { 0xaab15918, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475591b, 5 },
    { 0x25e1591c, 5 },
    { 0xc8fd591b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a095923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfd5928, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675592b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886d592f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe15934, 3 },
    { 0, 0 },
    { 0x2bd55936, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1593c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b9593f, 3 },
    { 0, 0 },
    { 0x9b395941, 3 },
    { 0xb3355942, 4 },
    { 0x4fcd5943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10815948, 3 },
    { 0xf2295949, 3 },
    { 0, 0 },
    { 0x47fd594b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359594e, 4 },
    { 0xa4bd594f, 3 },
    { 0x1f655950, 4 },
    { 0xd5f55951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94d5955, 3 },
    { 0x49d15956, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3609595a, 3 },
    { 0xc8d595a, 3 },
    { 0, 0 },
    { 0xf2e1595d, 3 },
    { 0x5d5d595e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a15961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ed5964, 3 },
    { 0xfe755965, 3 },
    { 0xd6355965, 3 },
    { 0, 0 },
    { 0x393d5968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a5596d, 3 },
    { 0x55bd596e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d5975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab15978, 3 },
    { 0xdc195979, 3 },
    { 0x7441597a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631d597f, 4 },
    { 0, 0 },
    { 0x3dad5981, 4 },
    { 0x4b015982, 3 },
    { 0xfd6d5981, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf55986, 3 },
    { 0x9795986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6979598a, 4 },
    { 0xa67d598a, 4 },
    { 0x35f5598c, 3 },
    { 0x8cfd598c, 3 },
    { 0x5281598e, 3 },
    { 0xd369598e, 3 },
    { 0, 0 },
    { 0x36a15991, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b55996, 3 },
    { 0xc1bd5997, 3 },
    { 0xf04d5998, 3 },
    { 0xdd15999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198d59a1, 3 },
    { 0xa45159a2, 3 },
    { 0x2c7959a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de959a6, 4 },
    { 0xc4559a7, 3 },
    { 0x6c5d59a8, 4 },
    { 0, 0 },
    { 0x5f9d59aa, 4 },
    { 0x7af159aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbd59af, 3 },
    { 0xc66959af, 3 },
    { 0xd4e959af, 3 },
    { 0, 0 },
    { 0xf6559b3, 3 },
    { 0x82fd59b4, 3 },
    { 0xeab559b5, 4 },
    { 0x823959b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4959bc, 3 },
    { 0xd74159bd, 3 },
    { 0, 0 },
    { 0x9a2959bf, 3 },
    { 0x40f959bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cd59c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cd59cc, 4 },
    { 0x6a3959cd, 3 },
    { 0x6a6159cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ed59d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38159d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed59e6, 3 },
    { 0xd59d59e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2559ea, 3 },
    { 0x823159ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5559ee, 3 },
    { 0xd19d59ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b959f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16959fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2295a04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ead5a07, 3 },
    { 0xe2f95a08, 3 },
    { 0, 0 },
    { 0x24b95a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a15a0d, 3 },
    { 0x86a95a0e, 3 },
    { 0x41ed5a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e695a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d95a17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b95a1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79195a23, 3 },
    { 0x63e55a24, 3 },
    { 0, 0 },
    { 0x1a955a26, 3 },
    { 0x55695a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae15a31, 3 },
    { 0x5b8d5a31, 3 },
    { 0x97915a33, 3 },
    { 0xa7195a33, 3 },
    { 0, 0 },
    { 0x7b295a36, 4 },
    { 0xa9055a37, 4 },
    { 0x50f15a38, 4 },
    { 0x58c15a37, 3 },
    { 0x92115a3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd55a3d, 3 },
    { 0xa9255a3d, 3 },
    { 0xff015a3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b55a43, 3 },
    { 0xd4b15a43, 3 },
    { 0x39a95a45, 4 },
    { 0x6f655a43, 3 },
    { 0x8f2d5a47, 3 },
    { 0x2a115a47, 3 },
    { 0x83815a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9455a4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40555a51, 3 },
    { 0, 0 },
    { 0xb6b15a53, 3 },
    { 0, 0 },
    { 0xb6b15a55, 4 },
    { 0, 0 },
    { 0x14215a57, 3 },
    { 0x74615a58, 3 },
    { 0xaa1d5a59, 3 },
    { 0x71955a58, 3 },
    { 0xfdc95a5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf755a5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c15a63, 3 },
    { 0xfd6d5a63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fd5a67, 4 },
    { 0xc8195a68, 3 },
    { 0x47455a69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfd5a6d, 3 },
    { 0x4c595a6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80715a72, 3 },
    { 0xbe9d5a73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54655a78, 3 },
    { 0, 0 },
    { 0xebd55a7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff495a7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73895a81, 3 },
    { 0xac955a81, 3 },
    { 0x85a15a83, 3 },
    { 0xaf215a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61015a89, 3 },
    { 0x18895a8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d695a8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c55a91, 3 },
    { 0x2d955a92, 3 },
    { 0xe6295a91, 3 },
    { 0x79d55a94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ed5a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad5a9c, 3 },
    { 0xccdd5a9d, 4 },
    { 0, 0 },
    { 0x9d2d5a9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf55aa8, 3 },
    { 0xd4615aa9, 3 },
    { 0xcf1d5aa9, 3 },
    { 0, 0 },
    { 0x3fe95aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2355ab0, 3 },
    { 0, 0 },
    { 0x45dd5ab2, 3 },
    { 0, 0 },
    { 0x89a15ab4, 3 },
    { 0x5b315ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd5abb, 3 },
    { 0xc8595abb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df55ac0, 3 },
    { 0x8cad5ac0, 5 },
    { 0, 0 },
    { 0xe4ad5ac3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d5ac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c95acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126d5acf, 3 },
    { 0xc6d15acf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe55ad3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78d5add, 3 },
    { 0x9d855ade, 4 },
    { 0x28755adf, 3 },
    { 0, 0 },
    { 0x24495ae1, 3 },
    { 0x1f095ae2, 3 },
    { 0xf0c95ae3, 3 },
    { 0x2e315ae4, 4 },
    { 0xdfdd5ae3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffd5ae8, 3 },
    { 0, 0 },
    { 0xce0d5aea, 3 },
    { 0x61315aea, 3 },
    { 0x2cd95aec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0595aef, 3 },
    { 0x9ad55aef, 3 },
    { 0x2acd5af1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff695af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d15af8, 4 },
    { 0, 0 },
    { 0x289d5afa, 3 },
    { 0x26d95afb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0115b00, 3 },
    { 0, 0 },
    { 0x2ba55b02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf395b0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fd5b0e, 4 },
    { 0xd1695b0e, 4 },
    { 0xbf515b0f, 3 },
    { 0xfedd5b0f, 3 },
    { 0xf7f15b12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387d5b19, 4 },
    { 0x6c715b19, 3 },
    { 0x9fad5b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1d5b1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6395b27, 3 },
    { 0, 0 },
    { 0x712d5b29, 5 },
    { 0x15b95b29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad555b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87855b34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6995b38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6955b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3d5b48, 3 },
    { 0, 0 },
    { 0xe855b4a, 3 },
    { 0, 0 },
    { 0xb6795b4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a15b50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb15b54, 3 },
    { 0x12795b55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2355b5a, 4 },
    { 0, 0 },
    { 0x89815b5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b595b5f, 4 },
    { 0x952d5b60, 3 },
    { 0x59695b5f, 3 },
    { 0, 0 },
    { 0x58555b63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf55b66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fd5b6b, 4 },
    { 0x9d415b6c, 4 },
    { 0xd0895b6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7415b70, 4 },
    { 0, 0 },
    { 0x71355b72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a395b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d5b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755d5b82, 3 },
    { 0x48955b83, 4 },
    { 0x92815b83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf95b89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f95b8e, 3 },
    { 0x92855b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d5b94, 3 },
    { 0, 0 },
    { 0xd215b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb295b99, 3 },
    { 0xf3dd5b9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a55b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e55ba7, 3 },
    { 0xca915ba8, 3 },
    { 0xe9a15ba9, 4 },
    { 0x33e95baa, 3 },
    { 0x29195bab, 3 },
    { 0, 0 },
    { 0x6ee95bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b455bb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bd5bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3155bbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea15bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c055bcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64515bd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a95bd4, 3 },
    { 0, 0 },
    { 0xf3dd5bd6, 4 },
    { 0xb6755bd6, 4 },
    { 0, 0 },
    { 0xe4915bd9, 3 },
    { 0xbfcd5bda, 3 },
    { 0xb9555bd9, 3 },
    { 0xd7ed5bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ed5bdf, 3 },
    { 0x57a15be0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x404d5be3, 3 },
    { 0, 0 },
    { 0x56555be5, 4 },
    { 0, 0 },
    { 0x61955be7, 3 },
    { 0x78595be8, 3 },
    { 0x1f195be9, 3 },
    { 0xf4ad5bea, 3 },
    { 0xb5855beb, 3 },
    { 0, 0 },
    { 0x39cd5bed, 3 },
    { 0xd50d5bed, 3 },
    { 0x5a495bee, 3 },
    { 0x5aa15bef, 3 },
    { 0xb7255bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42615bf4, 3 },
    { 0x64855bf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb55bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd55bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38d5c00, 4 },
    { 0xd3415c00, 3 },
    { 0x66c95c02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ed5c0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a95c0e, 3 },
    { 0, 0 },
    { 0x13415c10, 3 },
    { 0, 0 },
    { 0x833d5c12, 3 },
    { 0xbc115c13, 3 },
    { 0xe1c55c14, 4 },
    { 0x7b95c15, 3 },
    { 0xacd5c16, 4 },
    { 0x9bb95c17, 3 },
    { 0, 0 },
    { 0xb155c19, 4 },
    { 0x63695c1a, 3 },
    { 0x52755c1b, 3 },
    { 0x40215c19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75615c20, 3 },
    { 0xc0a55c21, 3 },
    { 0x28955c21, 3 },
    { 0x7e455c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc15c26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0115c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2255c2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc015c30, 4 },
    { 0, 0 },
    { 0x97555c32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1d5c38, 3 },
    { 0, 0 },
    { 0xbd7d5c3a, 3 },
    { 0, 0 },
    { 0x46ad5c3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ad5c3f, 3 },
    { 0x3ca15c40, 3 },
    { 0x27995c41, 4 },
    { 0, 0 },
    { 0x8f995c43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a255c4f, 3 },
    { 0x3e3d5c4f, 3 },
    { 0xfc795c51, 3 },
    { 0, 0 },
    { 0xb2895c53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3415c58, 3 },
    { 0xa7f15c59, 3 },
    { 0x78095c5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d5c60, 3 },
    { 0, 0 },
    { 0x8b515c62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde495c69, 3 },
    { 0, 0 },
    { 0x4a395c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b55c6e, 3 },
    { 0x63615c6f, 3 },
    { 0, 0 },
    { 0xd4495c71, 3 },
    { 0xce915c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c415c77, 3 },
    { 0x61955c78, 3 },
    { 0x554d5c78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67555c7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31d5c80, 3 },
    { 0xb0015c81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71115c84, 3 },
    { 0, 0 },
    { 0x6e515c86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8d5c8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c15c97, 3 },
    { 0, 0 },
    { 0x80795c99, 3 },
    { 0, 0 },
    { 0xb57d5c9b, 3 },
    { 0, 0 },
    { 0xf3015c9d, 3 },
    { 0x93c55c9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415ca1, 3 },
    { 0xaa595ca1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef15ca5, 3 },
    { 0x6e155ca6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cd5cab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b95cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87055cb5, 3 },
    { 0, 0 },
    { 0xb7d15cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dd5cbe, 4 },
    { 0x20215cbf, 3 },
    { 0, 0 },
    { 0x6c6d5cc1, 3 },
    { 0xc5095cc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44d5cc5, 5 },
    { 0x54855cc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b15cca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f315ccd, 3 },
    { 0xc595cce, 3 },
    { 0x330d5ccf, 3 },
    { 0xe0215cd0, 3 },
    { 0x42ed5ccf, 3 },
    { 0x346d5ccf, 3 },
    { 0x1db55cd1, 3 },
    { 0x8a215cd4, 3 },
    { 0x65995cd5, 3 },
    { 0x5ac55cd4, 3 },
    { 0xe3d95cd5, 3 },
    { 0xf8d95cd7, 3 },
    { 0xf4655cd8, 3 },
    { 0x6f015cd9, 3 },
    { 0x20755cdb, 3 },
    { 0xb3895cdb, 3 },
    { 0x72395cdb, 5 },
    { 0x43215cd6, 3 },
    { 0, 0 },
    { 0xb1e55ce0, 3 },
    { 0xc6a95ce1, 3 },
    { 0xcc55ce0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69815ceb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77d5cf0, 3 },
    { 0, 0 },
    { 0xd2395cf2, 3 },
    { 0xa87d5cf2, 3 },
    { 0x12a95cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4915cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1315cfb, 4 },
    { 0x2f615cfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c155d02, 5 },
    { 0xc2dd5d03, 4 },
    { 0xa2b55d04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c715d07, 4 },
    { 0x8c3d5d08, 4 },
    { 0x54715d09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45555d0c, 4 },
    { 0x13dd5d0d, 3 },
    { 0, 0 },
    { 0xfa15d0f, 3 },
    { 0, 0 },
    { 0xbc6d5d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76095d14, 3 },
    { 0, 0 },
    { 0xadd55d16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3115d1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d15d1f, 3 },
    { 0x14295d20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x17315d23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38215d26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8915d35, 3 },
    { 0x2fc15d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82695d3b, 5 },
    { 0x254d5d3c, 3 },
    { 0, 0 },
    { 0x8e8d5d3e, 3 },
    { 0x7a555d3e, 3 },
    { 0, 0 },
    { 0x5f995d41, 3 },
    { 0, 0 },
    { 0x95fd5d43, 3 },
    { 0, 0 },
    { 0x30455d45, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7d5d55, 3 },
    { 0, 0 },
    { 0x21295d57, 3 },
    { 0, 0 },
    { 0x32cd5d59, 4 },
    { 0, 0 },
    { 0x3b015d5b, 3 },
    { 0xf0115d5c, 3 },
    { 0xa2515d5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cd5d62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e95d65, 3 },
    { 0, 0 },
    { 0xd1f55d67, 4 },
    { 0x83255d68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3855d6b, 4 },
    { 0xbe555d6c, 3 },
    { 0x603d5d6c, 4 },
    { 0x2d1d5d6e, 3 },
    { 0x28515d6c, 4 },
    { 0xe1fd5d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60755d73, 3 },
    { 0x99fd5d74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd5d77, 3 },
    { 0xd2f55d77, 3 },
    { 0x917d5d77, 3 },
    { 0xd515d7a, 4 },
    { 0, 0 },
    { 0x9ded5d7c, 4 },
    { 0x7aa15d7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b15d82, 3 },
    { 0, 0 },
    { 0x364d5d84, 3 },
    { 0, 0 },
    { 0x7e5d5d86, 3 },
    { 0, 0 },
    { 0x50b15d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3755d8f, 3 },
    { 0x6ed95d90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad5d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d5da0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d15da4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dd5da7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab415daf, 3 },
    { 0x2e255db0, 3 },
    { 0xb8055db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f95db4, 3 },
    { 0, 0 },
    { 0x21215db6, 3 },
    { 0xd7d95db6, 4 },
    { 0, 0 },
    { 0xcd995db9, 3 },
    { 0xc0e95dba, 5 },
    { 0xdc995dbb, 4 },
    { 0xa4615dbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a15dc2, 3 },
    { 0, 0 },
    { 0x12b55dc4, 3 },
    { 0xe6395dc4, 4 },
    { 0x5a195dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584d5dca, 3 },
    { 0x78dd5dcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e15dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2515dd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb915dda, 4 },
    { 0x10d5ddb, 3 },
    { 0xd2e55ddc, 3 },
    { 0x5095ddc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9795de0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa615de4, 3 },
    { 0xfd895de5, 3 },
    { 0xc5d15de6, 3 },
    { 0x1595de6, 3 },
    { 0xdcb55de8, 3 },
    { 0, 0 },
    { 0x7a8d5dea, 3 },
    { 0x69a55deb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b015dee, 3 },
    { 0x8e095dee, 4 },
    { 0x56ed5df0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bd5df4, 3 },
    { 0, 0 },
    { 0x29255df6, 4 },
    { 0x1515df7, 3 },
    { 0xabe95df8, 5 },
    { 0, 0 },
    { 0x56855dfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7d5dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718d5e01, 3 },
    { 0xe2d15e01, 3 },
    { 0, 0 },
    { 0x2b5d5e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392d5e09, 3 },
    { 0x8bf55e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff55e0d, 4 },
    { 0xa8915e0e, 4 },
    { 0xac055e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45095e13, 4 },
    { 0xe0995e13, 3 },
    { 0x8a1d5e15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d355e18, 3 },
    { 0x4b915e19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d5e1f, 3 },
    { 0, 0 },
    { 0x48d5e21, 4 },
    { 0xa3855e22, 3 },
    { 0x51d15e23, 3 },
    { 0x39bd5e23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a195e2b, 4 },
    { 0, 0 },
    { 0xb9f95e2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e095e30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f515e33, 3 },
    { 0x8ea95e34, 5 },
    { 0, 0 },
    { 0xd9715e36, 3 },
    { 0, 0 },
    { 0xb9995e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b395e3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672d5e42, 3 },
    { 0x16755e43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b815e46, 4 },
    { 0xcf5d5e46, 3 },
    { 0, 0 },
    { 0x355e49, 3 },
    { 0, 0 },
    { 0x3fb55e4b, 3 },
    { 0x570d5e4c, 3 },
    { 0x72a95e4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d5e51, 4 },
    { 0xd9e15e52, 3 },
    { 0x5a495e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d395e56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a15e59, 3 },
    { 0xc3d5e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f55e5f, 3 },
    { 0xd3355e60, 3 },
    { 0xd6e55e5f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b415e66, 3 },
    { 0, 0 },
    { 0x8b015e68, 4 },
    { 0x54fd5e69, 3 },
    { 0xd4855e69, 3 },
    { 0x367d5e6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1295e6e, 3 },
    { 0xefbd5e6f, 3 },
    { 0, 0 },
    { 0xb42d5e71, 3 },
    { 0x37355e71, 3 },
    { 0x15215e73, 3 },
    { 0x883d5e74, 5 },
    { 0xfde55e74, 4 },
    { 0xc0bd5e76, 3 },
    { 0x62d55e76, 3 },
    { 0xa7155e74, 3 },
    { 0x8a995e79, 3 },
    { 0xc8f15e7a, 3 },
    { 0xed195e7a, 3 },
    { 0x50795e7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86655e83, 3 },
    { 0, 0 },
    { 0xb0c55e85, 3 },
    { 0xe9815e86, 4 },
    { 0x79615e87, 3 },
    { 0, 0 },
    { 0x68495e89, 3 },
    { 0, 0 },
    { 0x9ce95e8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c915e91, 3 },
    { 0x49055e91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e95ea0, 3 },
    { 0x85ad5ea0, 4 },
    { 0xe0b55ea2, 3 },
    { 0xbbfd5ea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14215ea6, 3 },
    { 0, 0 },
    { 0x2b195ea8, 4 },
    { 0x3d595ea8, 4 },
    { 0x8515eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8355eae, 3 },
    { 0, 0 },
    { 0x69f95eb0, 3 },
    { 0x23815eb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d5eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07d5eb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb55ebf, 3 },
    { 0xa6915ebf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e95ec4, 3 },
    { 0x3c395ec5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea15ec8, 4 },
    { 0xbe6d5ec9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d5eda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d5ee2, 3 },
    { 0x76f55ee3, 3 },
    { 0, 0 },
    { 0x13755ee5, 3 },
    { 0x84915ee6, 3 },
    { 0, 0 },
    { 0x35255ee8, 3 },
    { 0xf7c15ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f615eec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80795ef0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2695ef9, 3 },
    { 0x13b95ef9, 4 },
    { 0x84895efb, 3 },
    { 0x3b055efb, 3 },
    { 0x8cc55efd, 3 },
    { 0x5c415efd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd795f01, 3 },
    { 0xb4e15f02, 3 },
    { 0, 0 },
    { 0x5ba15f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7955f0d, 3 },
    { 0, 0 },
    { 0xda655f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc95f14, 3 },
    { 0x9a2d5f15, 4 },
    { 0xfafd5f16, 3 },
    { 0x88ed5f17, 3 },
    { 0x50d5f18, 3 },
    { 0xcfa55f19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8d5f20, 3 },
    { 0xa3455f20, 4 },
    { 0x9815f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62d5f25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cd5f29, 3 },
    { 0xcf055f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9595f2f, 3 },
    { 0xbec95f30, 3 },
    { 0, 0 },
    { 0x16355f32, 3 },
    { 0xa4455f33, 3 },
    { 0, 0 },
    { 0x501d5f35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce155f38, 4 },
    { 0xa3815f39, 3 },
    { 0xb7355f3a, 3 },
    { 0x416d5f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f55f3e, 3 },
    { 0x1a195f3f, 3 },
    { 0xb3c15f3f, 3 },
    { 0xac655f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1515f44, 3 },
    { 0x12795f45, 3 },
    { 0, 0 },
    { 0x27c95f47, 3 },
    { 0, 0 },
    { 0x7fcd5f49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c95f4c, 3 },
    { 0xb615f4c, 4 },
    { 0x9cad5f4c, 3 },
    { 0, 0 },
    { 0xe4795f50, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f55f56, 3 },
    { 0, 0 },
    { 0xe4d15f58, 3 },
    { 0xc5995f58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab155f5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa095f60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7395f65, 3 },
    { 0x14fd5f65, 3 },
    { 0, 0 },
    { 0xad5d5f68, 4 },
    { 0x8eed5f68, 3 },
    { 0x73595f6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcd5f6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91d5f72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c15f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a55f7b, 3 },
    { 0xec295f7c, 3 },
    { 0xb95d5f7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20715f81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90d5f86, 3 },
    { 0xce415f87, 3 },
    { 0x607d5f88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cd5f8b, 3 },
    { 0x3c9d5f8c, 4 },
    { 0, 0 },
    { 0xad515f8e, 4 },
    { 0, 0 },
    { 0x8ee95f90, 3 },
    { 0, 0 },
    { 0x49355f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea95f96, 3 },
    { 0xaa315f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25595f9a, 3 },
    { 0xa8c95f9b, 3 },
    { 0xca695f9c, 3 },
    { 0xb4055f9c, 4 },
    { 0x4c515f9c, 4 },
    { 0x75f55f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41755fa2, 4 },
    { 0xac015fa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0255fa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9d5fa9, 3 },
    { 0xa6815fa9, 3 },
    { 0, 0 },
    { 0xe5cd5fac, 4 },
    { 0x3895fad, 4 },
    { 0, 0 },
    { 0xb6255faf, 4 },
    { 0x5ccd5faf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ad5fb8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21655fc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83455fc6, 3 },
    { 0xa5315fc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d5fca, 3 },
    { 0xe0095fcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa15fcf, 3 },
    { 0x7ef95fd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5d5fd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed5fdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10815fe3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb15fe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50055fed, 3 },
    { 0xd4b15fee, 3 },
    { 0, 0 },
    { 0x6bb55ff0, 3 },
    { 0xcd3d5ff0, 3 },
    { 0x8e0d5ff0, 3 },
    { 0xd9e95ff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cd5ff8, 3 },
    { 0, 0 },
    { 0xe9bd5ffa, 3 },
    { 0, 0 },
    { 0x2df95ffc, 3 },
    { 0x4c195ffd, 4 },
    { 0, 0 },
    { 0x50295fff, 3 },
    { 0, 0 },
    { 0x4756001, 4 },
    { 0x2356002, 3 },
    { 0x7edd6003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78816006, 3 },
    { 0x11d56007, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4d600b, 3 },
    { 0x47a5600b, 3 },
    { 0x68fd600b, 3 },
    { 0x2de9600e, 3 },
    { 0xaaad600c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5516012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61196017, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be9601b, 3 },
    { 0x1e91601c, 3 },
    { 0, 0 },
    { 0xa6d1601e, 3 },
    { 0, 0 },
    { 0x58496020, 3 },
    { 0, 0 },
    { 0xb3d56022, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b56028, 3 },
    { 0x26656029, 3 },
    { 0, 0 },
    { 0x7edd602b, 4 },
    { 0xe159602c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9789602f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89016034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7096037, 3 },
    { 0xda656038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd603c, 5 },
    { 0, 0 },
    { 0x1f21603e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d6041, 3 },
    { 0xc3956041, 4 },
    { 0x482d6043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25556046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955604a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe231604d, 3 },
    { 0, 0 },
    { 0x8f11604f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1896052, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab716056, 4 },
    { 0, 0 },
    { 0xee556058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e5605e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e96065, 3 },
    { 0, 0 },
    { 0xd8e96067, 4 },
    { 0, 0 },
    { 0x94156069, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0d606c, 5 },
    { 0x43fd606d, 3 },
    { 0x96b9606e, 3 },
    { 0xb599606d, 3 },
    { 0, 0 },
    { 0xe6356071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1196074, 3 },
    { 0x37356074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8016079, 3 },
    { 0x68dd6079, 4 },
    { 0x297d607b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x659d607e, 3 },
    { 0, 0 },
    { 0xcf696080, 4 },
    { 0x6c216080, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5956084, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfd608c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d9608f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f996095, 3 },
    { 0, 0 },
    { 0x3f916097, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b95609d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d160a0, 3 },
    { 0, 0 },
    { 0x53d60a2, 3 },
    { 0xa64560a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35560a6, 3 },
    { 0xbca960a7, 3 },
    { 0xcfbd60a6, 3 },
    { 0x677d60a9, 4 },
    { 0xd17960a9, 3 },
    { 0, 0 },
    { 0xb27560ac, 3 },
    { 0xa60d60ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d60b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93960b7, 4 },
    { 0, 0 },
    { 0xfbd960b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd60bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8560c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e960c4, 3 },
    { 0, 0 },
    { 0xce6960c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70560cb, 3 },
    { 0, 0 },
    { 0xba0960cd, 3 },
    { 0x4a2160cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b960da, 4 },
    { 0xa09160da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0560de, 4 },
    { 0, 0 },
    { 0x36dd60e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2960e7, 5 },
    { 0xb9c960e7, 5 },
    { 0x7c4d60e9, 4 },
    { 0xea9d60e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48960ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1560f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07d60fa, 3 },
    { 0xcc160fb, 3 },
    { 0x42c560fb, 4 },
    { 0x35160fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb656100, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57156109, 3 },
    { 0x8c556109, 3 },
    { 0xd4a9610b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd6110, 3 },
    { 0x45296111, 4 },
    { 0, 0 },
    { 0x6fc56113, 3 },
    { 0x9a0d6114, 3 },
    { 0x7b696115, 3 },
    { 0x28c96114, 5 },
    { 0x46c16117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9611a, 3 },
    { 0x9389611b, 4 },
    { 0x5d8d611c, 4 },
    { 0, 0 },
    { 0xb939611e, 4 },
    { 0, 0 },
    { 0xdd056120, 3 },
    { 0, 0 },
    { 0x571d6122, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf16127, 3 },
    { 0xf6d16127, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f59612c, 3 },
    { 0x3c4d612d, 3 },
    { 0, 0 },
    { 0x75c1612f, 4 },
    { 0xf8d9612f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e56133, 4 },
    { 0x60816134, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d656138, 3 },
    { 0x44516139, 3 },
    { 0, 0 },
    { 0x4211613b, 3 },
    { 0x980d613b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071613f, 4 },
    { 0, 0 },
    { 0x2c296141, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c65614c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40d6151, 3 },
    { 0, 0 },
    { 0x5eb96153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512d6158, 3 },
    { 0xdd656159, 3 },
    { 0xd439615a, 3 },
    { 0xde016159, 4 },
    { 0xe66d615a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8516160, 3 },
    { 0xe7616160, 4 },
    { 0, 0 },
    { 0x72296163, 3 },
    { 0, 0 },
    { 0x564d6165, 4 },
    { 0xad2d6166, 3 },
    { 0xbb696167, 4 },
    { 0xe7e56167, 4 },
    { 0x76416167, 3 },
    { 0, 0 },
    { 0xdbe1616b, 5 },
    { 0x952d616c, 4 },
    { 0x1481616d, 3 },
    { 0, 0 },
    { 0xef75616f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc96175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb856179, 4 },
    { 0x9d0d617a, 3 },
    { 0, 0 },
    { 0xb871617c, 5 },
    { 0, 0 },
    { 0xeedd617e, 3 },
    { 0, 0 },
    { 0xa2056180, 3 },
    { 0xb4ed6181, 4 },
    { 0x8a196181, 3 },
    { 0, 0 },
    { 0x9bf56184, 3 },
    { 0x43856185, 3 },
    { 0x77ad6186, 4 },
    { 0x9b316187, 3 },
    { 0xb8156187, 3 },
    { 0x2b796189, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b1618e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd16191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc996196, 4 },
    { 0, 0 },
    { 0x56d16198, 4 },
    { 0x4cb56199, 3 },
    { 0x1fe5619a, 5 },
    { 0x7105619a, 4 },
    { 0, 0 },
    { 0x7189619d, 3 },
    { 0xcf21619e, 3 },
    { 0x4125619d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c561a2, 3 },
    { 0x73d961a3, 3 },
    { 0, 0 },
    { 0x498161a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763d61a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6d61b0, 3 },
    { 0xe49961b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead61b4, 3 },
    { 0, 0 },
    { 0x859561b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906561ba, 3 },
    { 0x8d9161bb, 3 },
    { 0x267161bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135161c0, 4 },
    { 0, 0 },
    { 0x566561c2, 3 },
    { 0x809961c3, 3 },
    { 0xfd1961c4, 3 },
    { 0, 0 },
    { 0x52c561c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ad61cb, 3 },
    { 0x6a3161cc, 3 },
    { 0, 0 },
    { 0xede961ce, 3 },
    { 0x8d61cf, 4 },
    { 0x4b9161cf, 3 },
    { 0x3aa161d1, 3 },
    { 0xd48961cf, 4 },
    { 0, 0 },
    { 0xb6b161d4, 3 },
    { 0, 0 },
    { 0x5c3d61d6, 3 },
    { 0, 0 },
    { 0x593d61d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677961e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10161e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2561ec, 3 },
    { 0xb0b161ec, 3 },
    { 0, 0 },
    { 0x4da561ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726561f6, 3 },
    { 0x8f2961f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d961fa, 5 },
    { 0xf1b161fb, 4 },
    { 0x3f7161fb, 3 },
    { 0x93b561fc, 4 },
    { 0x78c161fe, 3 },
    { 0x866561fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d56202, 3 },
    { 0x12e16203, 3 },
    { 0, 0 },
    { 0x3f396205, 3 },
    { 0x9d916206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb95620a, 3 },
    { 0, 0 },
    { 0x7199620c, 3 },
    { 0, 0 },
    { 0x1899620e, 3 },
    { 0, 0 },
    { 0xd8f56210, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda856215, 3 },
    { 0xb5c16216, 3 },
    { 0x23bd6217, 3 },
    { 0, 0 },
    { 0xb8696219, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe769621e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e16222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d56225, 3 },
    { 0xb7a96225, 3 },
    { 0xc7d16227, 3 },
    { 0x28916225, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1585622d, 3 },
    { 0xdc85622e, 3 },
    { 0xfae9622e, 4 },
    { 0x9981622d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596235, 4 },
    { 0x42716235, 4 },
    { 0, 0 },
    { 0x3fa56238, 3 },
    { 0x7b456239, 3 },
    { 0xf91d6238, 4 },
    { 0x8881623b, 3 },
    { 0xf5816239, 4 },
    { 0, 0 },
    { 0x4f21623e, 4 },
    { 0x1029623f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d6242, 3 },
    { 0, 0 },
    { 0xa9156244, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf256247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65616250, 4 },
    { 0x4a7d6251, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b96255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a16258, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f81625d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c56262, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b356265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2956268, 3 },
    { 0x3e396269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544d626d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd6272, 3 },
    { 0, 0 },
    { 0x8e756274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e56278, 3 },
    { 0, 0 },
    { 0xe345627a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd69627d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262d6283, 3 },
    { 0x9ff96284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fd6293, 4 },
    { 0x8e2d6293, 4 },
    { 0xdfb16294, 3 },
    { 0, 0 },
    { 0x3a716297, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd629c, 3 },
    { 0x893d629d, 3 },
    { 0x9629d, 3 },
    { 0xcc79629f, 4 },
    { 0, 0 },
    { 0x3e7d62a1, 5 },
    { 0xad9d62a2, 4 },
    { 0xd96d62a3, 4 },
    { 0xf85962a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x379162a7, 3 },
    { 0xa54162a8, 3 },
    { 0, 0 },
    { 0x9b4562aa, 3 },
    { 0x5d1162ab, 3 },
    { 0x3ed562ac, 3 },
    { 0x5fbd62ad, 5 },
    { 0, 0 },
    { 0xae2162af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca162b4, 4 },
    { 0x88a962b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829962bd, 3 },
    { 0x1c162be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303962c2, 3 },
    { 0x97c162c3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82162c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a162d0, 3 },
    { 0x541d62d0, 3 },
    { 0, 0 },
    { 0x80962d3, 5 },
    { 0, 0 },
    { 0x3fed62d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2562da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89962dd, 3 },
    { 0, 0 },
    { 0x83b962df, 4 },
    { 0x8be962e0, 3 },
    { 0xb27562e1, 3 },
    { 0xf21562e2, 3 },
    { 0xda1d62e2, 4 },
    { 0xd8d62e4, 3 },
    { 0x95c562e5, 3 },
    { 0x220562e6, 3 },
    { 0x4e7162e4, 3 },
    { 0xcb4d62e2, 3 },
    { 0xd6e162e4, 3 },
    { 0xed4962e0, 3 },
    { 0x4f6162e1, 3 },
    { 0x797d62df, 3 },
    { 0x29a162e4, 3 },
    { 0, 0 },
    { 0x863562ef, 3 },
    { 0, 0 },
    { 0x944562f1, 3 },
    { 0, 0 },
    { 0xda9962f3, 4 },
    { 0, 0 },
    { 0x203d62f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48962fa, 4 },
    { 0x456962fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd62ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a16303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x401d6306, 3 },
    { 0x33f16307, 3 },
    { 0x2096308, 3 },
    { 0xfe56307, 3 },
    { 0xadb1630a, 3 },
    { 0xa241630a, 3 },
    { 0x7cfd6309, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c56313, 3 },
    { 0x9c96314, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce56317, 3 },
    { 0xe3296318, 3 },
    { 0xfc816317, 3 },
    { 0x9ac9631a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d631d, 3 },
    { 0x2889631e, 3 },
    { 0xc9b9631e, 3 },
    { 0x49896320, 3 },
    { 0xc13d6321, 4 },
    { 0x12656321, 3 },
    { 0xbc256322, 4 },
    { 0x149d6322, 3 },
    { 0xbb996321, 3 },
    { 0x61216324, 3 },
    { 0xb3e56323, 3 },
    { 0x373d6323, 4 },
    { 0x8a416329, 3 },
    { 0x4c8d6329, 3 },
    { 0x4251632b, 3 },
    { 0x4996329, 3 },
    { 0xe7096320, 3 },
    { 0xcc39632e, 3 },
    { 0xa10d632e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1396335, 3 },
    { 0x372d6336, 3 },
    { 0xb7496336, 3 },
    { 0, 0 },
    { 0xd5c56339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb09633c, 3 },
    { 0xf231633d, 3 },
    { 0x7d81633e, 3 },
    { 0, 0 },
    { 0xd2d6340, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec016344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41756349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a55634d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b96351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36356355, 3 },
    { 0x8cf16356, 3 },
    { 0x41d56356, 5 },
    { 0xb8e96358, 3 },
    { 0, 0 },
    { 0x8d0d635a, 4 },
    { 0, 0 },
    { 0xfc39635c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e896363, 3 },
    { 0, 0 },
    { 0xd5d96365, 3 },
    { 0, 0 },
    { 0x67856367, 3 },
    { 0x464d6368, 3 },
    { 0, 0 },
    { 0xdc5636a, 3 },
    { 0, 0 },
    { 0xf4f9636c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec96370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90156375, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb56381, 4 },
    { 0, 0 },
    { 0xfb396383, 3 },
    { 0, 0 },
    { 0x7d316385, 4 },
    { 0x57416385, 3 },
    { 0x7fe96387, 4 },
    { 0xa4596386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f1638d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecd6390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69d6395, 3 },
    { 0x694d6395, 4 },
    { 0x1b696397, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e5639a, 4 },
    { 0, 0 },
    { 0xecb5639c, 4 },
    { 0x54f1639d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0963a0, 3 },
    { 0xa0c163a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904963a7, 3 },
    { 0x776163a8, 3 },
    { 0xff8163a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f163ac, 3 },
    { 0xf79163ad, 5 },
    { 0xcb1163ae, 4 },
    { 0x457963ad, 5 },
    { 0x392563b0, 3 },
    { 0xfb0163b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48163b5, 4 },
    { 0, 0 },
    { 0x47d163b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628163c0, 3 },
    { 0xb91563c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1563c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14563c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5163ce, 3 },
    { 0, 0 },
    { 0x399163d0, 4 },
    { 0x7f0563d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc563d5, 3 },
    { 0x653563d5, 3 },
    { 0x243563d7, 3 },
    { 0x7e5563d8, 3 },
    { 0xc2163d5, 4 },
    { 0x8cbd63da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178d63e1, 4 },
    { 0x6cf963e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0563e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a963f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda963f6, 3 },
    { 0, 0 },
    { 0x54b963f8, 3 },
    { 0xb3ad63f8, 4 },
    { 0xc1c563fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5616400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd6403, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae56407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd885640b, 3 },
    { 0xb5b9640c, 3 },
    { 0x1951640d, 3 },
    { 0x27bd640b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf16411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd6415, 4 },
    { 0, 0 },
    { 0x1ff16417, 3 },
    { 0xea156418, 3 },
    { 0, 0 },
    { 0x4af5641a, 4 },
    { 0, 0 },
    { 0x9575641c, 4 },
    { 0, 0 },
    { 0xb4d641e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb496421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6916426, 3 },
    { 0xe8d56427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca1642a, 3 },
    { 0, 0 },
    { 0x551d642c, 3 },
    { 0x8789642d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acd6430, 4 },
    { 0, 0 },
    { 0x8bc56432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0d6435, 4 },
    { 0, 0 },
    { 0x9fe96437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7351643d, 3 },
    { 0x75ed643e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e796446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d9644a, 3 },
    { 0xf27d644b, 4 },
    { 0xb1b5644b, 3 },
    { 0x6859644d, 4 },
    { 0xf3b5644e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8d6454, 3 },
    { 0xdf156455, 3 },
    { 0x22596456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dd6459, 3 },
    { 0x125d6459, 4 },
    { 0xe4056459, 3 },
    { 0, 0 },
    { 0x2b91645d, 3 },
    { 0xa29d645e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d96465, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d96469, 3 },
    { 0, 0 },
    { 0x2595646b, 4 },
    { 0, 0 },
    { 0x3ed9646d, 3 },
    { 0, 0 },
    { 0xe041646f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea96472, 3 },
    { 0xe8796473, 3 },
    { 0, 0 },
    { 0x26956475, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb856479, 3 },
    { 0x64fd647a, 4 },
    { 0, 0 },
    { 0xe425647c, 3 },
    { 0xa011647c, 3 },
    { 0, 0 },
    { 0x4939647f, 4 },
    { 0, 0 },
    { 0xe8256481, 4 },
    { 0x94ad6482, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45856487, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8096490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6096494, 3 },
    { 0x8b9d6495, 3 },
    { 0, 0 },
    { 0xf2216497, 4 },
    { 0x7796498, 4 },
    { 0x59e56497, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fd649d, 3 },
    { 0x8469649e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x182164a1, 3 },
    { 0xffe564a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867564a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee564a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471964ad, 3 },
    { 0xcd0d64ae, 3 },
    { 0, 0 },
    { 0x53f564b0, 3 },
    { 0xfa8d64b1, 3 },
    { 0, 0 },
    { 0x131564b3, 3 },
    { 0xa7c964b4, 4 },
    { 0x1d0164b4, 4 },
    { 0xeb7964b4, 3 },
    { 0x297964b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78564bd, 3 },
    { 0x995964be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4164c3, 3 },
    { 0x471d64c4, 3 },
    { 0x569d64c5, 4 },
    { 0x7ee564c6, 3 },
    { 0xfc9d64c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8964cb, 4 },
    { 0x558964cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b164d0, 3 },
    { 0x7fc164d1, 3 },
    { 0x217964d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f164d8, 4 },
    { 0x91d564d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7164dc, 3 },
    { 0x9ec164dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x228964e0, 3 },
    { 0x601164e1, 4 },
    { 0xef0964e1, 3 },
    { 0xedb164e3, 3 },
    { 0x6ac164e4, 3 },
    { 0x4e5d64e3, 3 },
    { 0, 0 },
    { 0x635164e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4564f0, 3 },
    { 0x794564f1, 3 },
    { 0, 0 },
    { 0xe1d964f3, 3 },
    { 0xd1f164f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d164fa, 3 },
    { 0x449164fa, 3 },
    { 0x111d64fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73356502, 3 },
    { 0x19516502, 3 },
    { 0x32d6504, 3 },
    { 0xea896505, 3 },
    { 0x5b56506, 3 },
    { 0xac1d6506, 4 },
    { 0, 0 },
    { 0x8ed56509, 3 },
    { 0x9295650a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68516512, 3 },
    { 0x63196513, 3 },
    { 0x3cd56514, 3 },
    { 0x78256515, 3 },
    { 0x6756516, 3 },
    { 0, 0 },
    { 0xa356518, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe856520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc296525, 3 },
    { 0, 0 },
    { 0xa8516527, 3 },
    { 0x4a196527, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d652c, 3 },
    { 0, 0 },
    { 0x9d39652e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee56535, 3 },
    { 0x8d996536, 3 },
    { 0xc8d6537, 3 },
    { 0xccf16538, 3 },
    { 0, 0 },
    { 0xbc01653a, 3 },
    { 0x9f11653b, 4 },
    { 0, 0 },
    { 0xba6d653d, 3 },
    { 0, 0 },
    { 0xb415653f, 4 },
    { 0x51056540, 3 },
    { 0x9ef96540, 3 },
    { 0x3f196542, 4 },
    { 0x8fc16543, 3 },
    { 0x5ffd6542, 4 },
    { 0, 0 },
    { 0xe7196546, 3 },
    { 0xc4216547, 3 },
    { 0x3ac16547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5654d, 3 },
    { 0x9d75654e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e296552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea5655c, 3 },
    { 0, 0 },
    { 0x9b39655e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc96561, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3796568, 3 },
    { 0x96256569, 4 },
    { 0x2a99656a, 4 },
    { 0x4cb5656a, 3 },
    { 0x56e16569, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb9656f, 3 },
    { 0, 0 },
    { 0x47d6571, 4 },
    { 0, 0 },
    { 0xcb956573, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd5657c, 3 },
    { 0, 0 },
    { 0x5459657e, 3 },
    { 0xd911657e, 3 },
    { 0xeaf96580, 3 },
    { 0, 0 },
    { 0x9d2d6582, 3 },
    { 0x52096582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5496589, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca9658e, 3 },
    { 0x8765658f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab96595, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843d65a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1d65a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118965ab, 4 },
    { 0x238565ac, 3 },
    { 0, 0 },
    { 0xca7d65ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x30565b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a965b5, 5 },
    { 0, 0 },
    { 0x9f2965b7, 3 },
    { 0, 0 },
    { 0x876d65b9, 4 },
    { 0, 0 },
    { 0x306565bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bd65c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59165c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa565ca, 4 },
    { 0xfc6965cb, 3 },
    { 0x97bd65ca, 4 },
    { 0x218565cd, 3 },
    { 0x2da165ce, 3 },
    { 0x812965cf, 3 },
    { 0x459165d0, 3 },
    { 0x9f5165d1, 3 },
    { 0xb21565cf, 3 },
    { 0, 0 },
    { 0x354965d4, 3 },
    { 0, 0 },
    { 0x670565d6, 3 },
    { 0xc12565d6, 3 },
    { 0xc8d165d8, 3 },
    { 0x190965d6, 3 },
    { 0, 0 },
    { 0xf2ed65db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798d65e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3d65e4, 3 },
    { 0xca9165e5, 3 },
    { 0x5aad65e4, 3 },
    { 0, 0 },
    { 0x760165e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x399d65eb, 5 },
    { 0, 0 },
    { 0xbafd65ed, 3 },
    { 0xfcf165ee, 3 },
    { 0xd4b165ef, 3 },
    { 0, 0 },
    { 0xb8cd65f1, 3 },
    { 0, 0 },
    { 0x2c3165f3, 3 },
    { 0x966165f4, 4 },
    { 0, 0 },
    { 0x64d65f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x756565f9, 3 },
    { 0, 0 },
    { 0x663165fb, 3 },
    { 0xa56565fc, 3 },
    { 0x2b9965fc, 3 },
    { 0xddd65fd, 3 },
    { 0x473165ff, 3 },
    { 0xcce96600, 3 },
    { 0x842565ff, 3 },
    { 0, 0 },
    { 0xbbcd6603, 3 },
    { 0x9d756604, 3 },
    { 0, 0 },
    { 0xef816606, 3 },
    { 0x26b96607, 3 },
    { 0x505d6608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8299660c, 3 },
    { 0x12dd660d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d6611, 3 },
    { 0xe2ed6611, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393d6619, 3 },
    { 0, 0 },
    { 0x3b21661b, 3 },
    { 0x3711661b, 3 },
    { 0xa6fd661d, 3 },
    { 0x8349661b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9616622, 3 },
    { 0xdbe56622, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9256627, 4 },
    { 0xe5056627, 3 },
    { 0x5eed6629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d662e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82196633, 3 },
    { 0xfe356634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f96637, 4 },
    { 0x15b96638, 4 },
    { 0x54596639, 3 },
    { 0, 0 },
    { 0xf305663b, 3 },
    { 0x405d663c, 4 },
    { 0xf65d663b, 4 },
    { 0x982d663e, 3 },
    { 0, 0 },
    { 0x46316640, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337d6648, 3 },
    { 0xd3c56649, 3 },
    { 0, 0 },
    { 0x26b9664b, 4 },
    { 0x6bf9664c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a516651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45016656, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d16659, 4 },
    { 0xa3d1665a, 4 },
    { 0x84b16659, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e16661, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f5666a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e1666e, 3 },
    { 0x51a1666f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96656672, 4 },
    { 0xd5d6673, 3 },
    { 0x72456674, 3 },
    { 0xbe4d6675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe1667b, 3 },
    { 0x7bad667c, 3 },
    { 0x18d9667c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80d6682, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf096688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe89668b, 5 },
    { 0xa9a5668c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcd6691, 3 },
    { 0x38396692, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a16696, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b79669c, 3 },
    { 0x3fc9669d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f166a4, 3 },
    { 0x12d566a5, 3 },
    { 0x3e8966a6, 4 },
    { 0xadad66a4, 3 },
    { 0xb24d66a8, 3 },
    { 0, 0 },
    { 0xb72566aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x968566ad, 4 },
    { 0x6ae966ae, 3 },
    { 0xc26166af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4966bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce166c0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ad66c8, 3 },
    { 0, 0 },
    { 0x5f7966ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d66ce, 3 },
    { 0x71cd66cf, 4 },
    { 0x20bd66cf, 4 },
    { 0, 0 },
    { 0x520566d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x789566d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4d66d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63d66e4, 3 },
    { 0, 0 },
    { 0xd75166e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04166eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c966ef, 5 },
    { 0xe31166f0, 4 },
    { 0, 0 },
    { 0xdf2d66f2, 4 },
    { 0x95166f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ed66fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24896701, 3 },
    { 0xc33d6702, 3 },
    { 0, 0 },
    { 0xc2a96704, 3 },
    { 0x79256705, 3 },
    { 0xaae56706, 3 },
    { 0x2b796707, 3 },
    { 0x14316704, 3 },
    { 0x180d6704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1670e, 3 },
    { 0, 0 },
    { 0x9e4d6710, 3 },
    { 0, 0 },
    { 0xafd16712, 3 },
    { 0x84816713, 3 },
    { 0x954d6714, 4 },
    { 0xe0616713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0816720, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a156725, 3 },
    { 0xc50d6726, 3 },
    { 0x80896726, 3 },
    { 0, 0 },
    { 0x7156729, 3 },
    { 0x94f9672a, 3 },
    { 0xf391672a, 3 },
    { 0, 0 },
    { 0x5285672d, 3 },
    { 0x3775672d, 3 },
    { 0, 0 },
    { 0xe7756730, 3 },
    { 0x91496730, 4 },
    { 0x4a656731, 3 },
    { 0xdcfd6731, 3 },
    { 0, 0 },
    { 0xe4c56735, 4 },
    { 0, 0 },
    { 0xb6716737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5673a, 4 },
    { 0x5e69673b, 3 },
    { 0xe27d673c, 3 },
    { 0x6de9673d, 3 },
    { 0, 0 },
    { 0x7cc9673f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c496744, 3 },
    { 0x9d616744, 3 },
    { 0, 0 },
    { 0x2156747, 3 },
    { 0, 0 },
    { 0x1e656749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1785674d, 3 },
    { 0, 0 },
    { 0x8025674f, 3 },
    { 0x3b1d674f, 3 },
    { 0x9d16751, 3 },
    { 0x1121674f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf16758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca5675b, 3 },
    { 0xf15675b, 3 },
    { 0xef81675d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13716762, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c956767, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085676f, 3 },
    { 0x47216770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a96777, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd95677c, 4 },
    { 0x1bd5677d, 3 },
    { 0, 0 },
    { 0x4a11677f, 4 },
    { 0x7dfd6780, 3 },
    { 0x8e956781, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e816785, 3 },
    { 0, 0 },
    { 0xe9c16787, 3 },
    { 0xb1bd6788, 4 },
    { 0, 0 },
    { 0xd5bd678a, 3 },
    { 0xde81678b, 3 },
    { 0xecb9678c, 3 },
    { 0xb185678c, 3 },
    { 0x658d678e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27016791, 3 },
    { 0, 0 },
    { 0x86616793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b56797, 3 },
    { 0xea4d6798, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6869679f, 4 },
    { 0x199967a0, 3 },
    { 0, 0 },
    { 0x6c8167a2, 3 },
    { 0x9fa167a3, 4 },
    { 0x646567a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d67a8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172d67af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9967b4, 5 },
    { 0x3d9167b5, 3 },
    { 0, 0 },
    { 0xd11167b7, 3 },
    { 0xcd8167b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b567bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f167be, 3 },
    { 0xaa3967bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2567c2, 3 },
    { 0xc3ed67c3, 3 },
    { 0, 0 },
    { 0x38d67c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcd67c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89567d0, 3 },
    { 0, 0 },
    { 0x1a6d67d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c567db, 4 },
    { 0xa5b567dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6567df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42167e4, 3 },
    { 0, 0 },
    { 0xa6ad67e6, 3 },
    { 0x18c967e7, 3 },
    { 0x4ed167e7, 3 },
    { 0, 0 },
    { 0x34d167ea, 4 },
    { 0x52d67eb, 3 },
    { 0xdb9167ea, 3 },
    { 0x147d67ed, 3 },
    { 0x6f5d67ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690d67f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x264967f5, 3 },
    { 0xece167f6, 4 },
    { 0, 0 },
    { 0x864967f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c967fb, 3 },
    { 0, 0 },
    { 0x8fc167fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42956800, 3 },
    { 0x6d416801, 3 },
    { 0x32316800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27396808, 4 },
    { 0xa4256809, 3 },
    { 0, 0 },
    { 0xcded680b, 3 },
    { 0, 0 },
    { 0x7795680d, 3 },
    { 0xdbdd680d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d16812, 3 },
    { 0x3b96813, 3 },
    { 0x5b996814, 3 },
    { 0, 0 },
    { 0xd9456816, 4 },
    { 0, 0 },
    { 0x999d6818, 3 },
    { 0x364d6818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e1681c, 3 },
    { 0x6c6d681d, 4 },
    { 0x20e5681c, 3 },
    { 0, 0 },
    { 0x39016820, 3 },
    { 0xa6696821, 3 },
    { 0x60956822, 3 },
    { 0x959d6823, 3 },
    { 0x6c656824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b56827, 4 },
    { 0x58956828, 3 },
    { 0x2d516828, 5 },
    { 0x23816827, 3 },
    { 0xe661682b, 4 },
    { 0x125682a, 3 },
    { 0x7401682b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12396834, 3 },
    { 0x93716835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6d6838, 3 },
    { 0xddc56839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa45683d, 3 },
    { 0xef09683d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd956843, 4 },
    { 0xf1596844, 3 },
    { 0, 0 },
    { 0xe8b56846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585684d, 3 },
    { 0, 0 },
    { 0x12d684f, 3 },
    { 0xe3c5684f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63316853, 3 },
    { 0xb6ad6854, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3516858, 3 },
    { 0, 0 },
    { 0x356d685a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6716866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6056869, 4 },
    { 0, 0 },
    { 0x9245686b, 4 },
    { 0xe525686b, 3 },
    { 0xff89686d, 3 },
    { 0xfb69686c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d16872, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30256878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c9687c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b16883, 3 },
    { 0x1ca96884, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94616889, 4 },
    { 0x2b3d688a, 3 },
    { 0x3569688a, 4 },
    { 0xeee56889, 3 },
    { 0x14f9688d, 3 },
    { 0, 0 },
    { 0x5a69688f, 3 },
    { 0x30356890, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79496894, 3 },
    { 0xfb796895, 4 },
    { 0, 0 },
    { 0xe7796897, 3 },
    { 0x92b56898, 4 },
    { 0x2b7d6899, 3 },
    { 0x96296897, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36d68a1, 3 },
    { 0, 0 },
    { 0x65e968a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b568a6, 5 },
    { 0, 0 },
    { 0x3efd68a8, 3 },
    { 0, 0 },
    { 0x8b568aa, 4 },
    { 0xdd5568aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202168b2, 3 },
    { 0x631d68b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f568b7, 3 },
    { 0xbf2568b8, 3 },
    { 0x522d68b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c968c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270568cd, 3 },
    { 0, 0 },
    { 0x592168cf, 3 },
    { 0xe1968d0, 3 },
    { 0x8e4568d1, 3 },
    { 0x7ec968d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x570168d5, 3 },
    { 0, 0 },
    { 0xd98968d7, 4 },
    { 0xa85d68d8, 3 },
    { 0x2ec168d7, 4 },
    { 0x308d68da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bd68de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0568e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7968e4, 3 },
    { 0, 0 },
    { 0xd65568e6, 3 },
    { 0xda4168e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee968ee, 3 },
    { 0xda6168ee, 3 },
    { 0x882168ef, 3 },
    { 0xe70168ef, 3 },
    { 0, 0 },
    { 0xcf2168f3, 3 },
    { 0x1f8568f4, 3 },
    { 0xb0c568f5, 4 },
    { 0x349168f6, 3 },
    { 0x3a2168f5, 3 },
    { 0xf88d68f8, 3 },
    { 0x6ded68f9, 3 },
    { 0x449568f8, 4 },
    { 0, 0 },
    { 0x4fc568fc, 3 },
    { 0x8a5568fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d056901, 3 },
    { 0, 0 },
    { 0xe8696903, 3 },
    { 0x40856903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d9690a, 3 },
    { 0xe211690b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d99690f, 3 },
    { 0, 0 },
    { 0x6e896911, 3 },
    { 0x368d6912, 3 },
    { 0xe8dd6912, 3 },
    { 0x81f56914, 3 },
    { 0x2b256915, 3 },
    { 0xc0fd6915, 3 },
    { 0xd7156917, 3 },
    { 0x21d16916, 3 },
    { 0x50ed6919, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935691e, 4 },
    { 0, 0 },
    { 0x67756920, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136d6924, 3 },
    { 0xdb996925, 3 },
    { 0x12996926, 4 },
    { 0x34416927, 4 },
    { 0x40516928, 4 },
    { 0x30956929, 3 },
    { 0x9041692a, 3 },
    { 0x37ed692b, 3 },
    { 0x70c5692a, 3 },
    { 0, 0 },
    { 0x9be9692e, 3 },
    { 0xd791692f, 3 },
    { 0xf80d6930, 4 },
    { 0xeda56931, 3 },
    { 0x770d692e, 3 },
    { 0xec3d692e, 3 },
    { 0xc916934, 3 },
    { 0, 0 },
    { 0xfbe96936, 4 },
    { 0, 0 },
    { 0x6a1d6938, 4 },
    { 0x9bad6939, 3 },
    { 0xaaf16938, 3 },
    { 0xdf21693b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5425693e, 4 },
    { 0xd6ed693f, 3 },
    { 0xc0296940, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee16943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93616947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dd694e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b16951, 3 },
    { 0x8e216952, 4 },
    { 0xc1116951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37416958, 3 },
    { 0x45c96958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e9695f, 4 },
    { 0x70496960, 3 },
    { 0x8ef16961, 3 },
    { 0xb5496962, 3 },
    { 0x8469695f, 4 },
    { 0xb9416962, 4 },
    { 0xd7b1695f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58816969, 4 },
    { 0xd439696a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9696d, 3 },
    { 0, 0 },
    { 0x93d1696f, 3 },
    { 0xa8956970, 3 },
    { 0x4dad6970, 3 },
    { 0xe6556972, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd16977, 3 },
    { 0xda156977, 4 },
    { 0xa396979, 3 },
    { 0, 0 },
    { 0xe825697b, 3 },
    { 0x1ba1697c, 3 },
    { 0x2e1697b, 3 },
    { 0, 0 },
    { 0xdf65697f, 3 },
    { 0x52f96980, 3 },
    { 0x80bd6980, 3 },
    { 0x77a16982, 3 },
    { 0x2929697f, 3 },
    { 0, 0 },
    { 0x68d6985, 3 },
    { 0x422d6985, 3 },
    { 0xe39d6985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef5698d, 3 },
    { 0x6a09698d, 3 },
    { 0, 0 },
    { 0xa016990, 3 },
    { 0, 0 },
    { 0x44e16992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f16999, 4 },
    { 0xeb916999, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e99699e, 3 },
    { 0, 0 },
    { 0x89bd69a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06169a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0d69a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8969ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d69b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a969b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918969bc, 3 },
    { 0, 0 },
    { 0x43c969be, 3 },
    { 0xe63569be, 3 },
    { 0x544969c0, 3 },
    { 0, 0 },
    { 0x1d169c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823569c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccd69cc, 4 },
    { 0xb99969cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870969d5, 4 },
    { 0x816569d6, 3 },
    { 0, 0 },
    { 0x6e0969d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c969dc, 4 },
    { 0x232169dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x210d69e0, 4 },
    { 0x159569e0, 3 },
    { 0xa12169e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900569e6, 4 },
    { 0x9ad169e7, 3 },
    { 0, 0 },
    { 0x2cc169e9, 4 },
    { 0x8afd69ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51569f1, 3 },
    { 0x62b169f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798969f9, 3 },
    { 0x684d69fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418569fe, 3 },
    { 0xc3e169ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcd6a02, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x27796a05, 3 },
    { 0, 0 },
    { 0x3f0d6a07, 3 },
    { 0xc5996a07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5116a0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d6a0f, 3 },
    { 0, 0 },
    { 0xc8716a11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0d6a14, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8716a1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff16a1e, 3 },
    { 0xc3896a1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d6a23, 3 },
    { 0x957d6a23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac96a29, 4 },
    { 0x79616a2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe56a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c56a35, 3 },
    { 0, 0 },
    { 0x16816a37, 4 },
    { 0, 0 },
    { 0xcfad6a39, 3 },
    { 0x8a196a3a, 3 },
    { 0x3eb56a3b, 3 },
    { 0xc5316a3c, 3 },
    { 0xa256a3b, 4 },
    { 0xa6256a3e, 3 },
    { 0xd74d6a3e, 3 },
    { 0xf6256a3d, 3 },
    { 0x52a56a39, 3 },
    { 0x57956a42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555d6a49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x813d6a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d6a4f, 4 },
    { 0x9e916a4f, 3 },
    { 0x35956a51, 3 },
    { 0, 0 },
    { 0x72e96a53, 3 },
    { 0xa3556a54, 3 },
    { 0xfb5d6a55, 4 },
    { 0x382d6a56, 3 },
    { 0xd4d6a56, 3 },
    { 0, 0 },
    { 0x3b216a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca096a60, 3 },
    { 0x90396a61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c56a65, 4 },
    { 0, 0 },
    { 0x52896a67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d6a6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29756a75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4716a79, 3 },
    { 0x2e696a7a, 3 },
    { 0xc3256a79, 4 },
    { 0x5bf96a7c, 3 },
    { 0x85f56a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c56a87, 3 },
    { 0x30796a88, 4 },
    { 0xe9716a89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11056a8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0116a93, 4 },
    { 0xe0e56a94, 3 },
    { 0xf98d6a95, 3 },
    { 0x6bc16a93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d6a9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ad6aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ed6aa7, 4 },
    { 0, 0 },
    { 0x704d6aa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0156ab0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddd6ab8, 3 },
    { 0xaed56ab9, 3 },
    { 0x50d16ab8, 4 },
    { 0x56f16abb, 3 },
    { 0, 0 },
    { 0xd2c16abd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c356ac0, 3 },
    { 0, 0 },
    { 0x42856ac2, 3 },
    { 0x3556ac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd16ac8, 3 },
    { 0, 0 },
    { 0x2e8d6aca, 3 },
    { 0xa9b56acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea696ace, 3 },
    { 0xc5b56ace, 3 },
    { 0, 0 },
    { 0x9dd96ad1, 4 },
    { 0x3b256ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d56ada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13396ade, 3 },
    { 0x6bbd6ade, 5 },
    { 0xd09d6adf, 4 },
    { 0x62e56ae1, 4 },
    { 0xadf56ae2, 3 },
    { 0xa01d6ae3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72096ae7, 3 },
    { 0xfaf56ae8, 3 },
    { 0, 0 },
    { 0xf06d6aea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39d6aef, 3 },
    { 0, 0 },
    { 0x290d6af1, 3 },
    { 0x98196af2, 4 },
    { 0, 0 },
    { 0xc8596af4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e56af7, 3 },
    { 0x45fd6af7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d6afb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02d6aff, 3 },
    { 0x4a596b00, 3 },
    { 0, 0 },
    { 0xd3f96b02, 3 },
    { 0xb5416b02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911d6b0d, 3 },
    { 0x5dd96b0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c96b16, 4 },
    { 0, 0 },
    { 0x5f896b18, 3 },
    { 0x5616b19, 3 },
    { 0, 0 },
    { 0x1f556b1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dd6b1e, 3 },
    { 0, 0 },
    { 0x5d7d6b20, 4 },
    { 0x53cd6b21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d6b25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac656b32, 3 },
    { 0xab796b32, 4 },
    { 0x139d6b32, 4 },
    { 0x54896b33, 3 },
    { 0xe3516b36, 4 },
    { 0xfefd6b36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1d6b3b, 4 },
    { 0x6a6d6b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4d6b45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7356b4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96516b52, 4 },
    { 0, 0 },
    { 0x74916b54, 3 },
    { 0xdbbd6b55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b16b64, 3 },
    { 0xabe16b65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b616b69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c16b6f, 4 },
    { 0x95fd6b70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80556b76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354d6b7e, 4 },
    { 0x41d96b7e, 4 },
    { 0xfe7d6b7f, 5 },
    { 0, 0 },
    { 0x46516b82, 3 },
    { 0xd9f16b82, 3 },
    { 0x2aed6b82, 3 },
    { 0x30656b85, 3 },
    { 0, 0 },
    { 0x1bed6b87, 3 },
    { 0, 0 },
    { 0x2e1d6b89, 4 },
    { 0, 0 },
    { 0xa0956b8b, 3 },
    { 0x90496b8b, 3 },
    { 0xefd6b8d, 3 },
    { 0, 0 },
    { 0x3c6d6b8f, 4 },
    { 0xf4e56b90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b56b93, 3 },
    { 0x65a16b94, 3 },
    { 0x4456b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb56b9b, 3 },
    { 0xa7bd6b9c, 3 },
    { 0x535d6b9d, 4 },
    { 0x19dd6b9e, 4 },
    { 0, 0 },
    { 0x55d16ba0, 3 },
    { 0x43996ba0, 3 },
    { 0xdacd6ba2, 3 },
    { 0xe1116ba2, 4 },
    { 0xa3456ba4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ed6ba9, 3 },
    { 0xfcdd6baa, 3 },
    { 0x4556baa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda996baf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb96bb4, 3 },
    { 0x94b16bb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xae316bb8, 3 },
    { 0x8cc16bb8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d96bc2, 3 },
    { 0xc0456bc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b56bc6, 3 },
    { 0xad356bc7, 4 },
    { 0x1d016bc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b96bce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b756bd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x567d6bd4, 4 },
    { 0, 0 },
    { 0xa9cd6bd6, 3 },
    { 0xb0496bd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa296bdc, 3 },
    { 0xb74d6bdd, 3 },
    { 0xa0e56bde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a96be1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d56be5, 4 },
    { 0x55a96be5, 3 },
    { 0x1bad6be6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946d6bec, 3 },
    { 0x49716bed, 3 },
    { 0xa35d6bed, 3 },
    { 0x87416bee, 3 },
    { 0x673d6bf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43596bfa, 3 },
    { 0, 0 },
    { 0xc10d6bfc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c816bff, 3 },
    { 0x3e7d6bff, 3 },
    { 0xcbf16c01, 3 },
    { 0x86856c01, 3 },
    { 0, 0 },
    { 0x29c16c04, 3 },
    { 0xcf9d6c05, 3 },
    { 0x629d6c05, 3 },
    { 0, 0 },
    { 0xa1cd6c08, 3 },
    { 0xedad6c08, 3 },
    { 0x28556c09, 3 },
    { 0x11fd6c0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b56c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e496c19, 4 },
    { 0, 0 },
    { 0x28696c1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb16c1e, 4 },
    { 0xca616c1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d6c23, 4 },
    { 0x80f16c24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e16c27, 3 },
    { 0x8b096c28, 3 },
    { 0, 0 },
    { 0x65356c2a, 3 },
    { 0x7a1d6c2b, 3 },
    { 0xae296c2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffed6c37, 3 },
    { 0x72d6c38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77856c3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51216c49, 3 },
    { 0x3c516c49, 3 },
    { 0xdd596c49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89356c4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2216c53, 3 },
    { 0x66416c54, 4 },
    { 0xca216c53, 3 },
    { 0xf8496c56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec16c59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c16c61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c16c65, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db56c6b, 3 },
    { 0x1c196c6c, 3 },
    { 0, 0 },
    { 0xb27d6c6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef96c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d6c76, 3 },
    { 0x55d16c76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c56c84, 3 },
    { 0x9db56c85, 3 },
    { 0xf3356c85, 4 },
    { 0x82d56c87, 3 },
    { 0x4e656c87, 3 },
    { 0x76916c89, 3 },
    { 0xff296c8a, 3 },
    { 0, 0 },
    { 0xb5096c8c, 3 },
    { 0xe16d6c8d, 4 },
    { 0xdde56c8e, 3 },
    { 0xdff16c8f, 3 },
    { 0xa0716c90, 3 },
    { 0, 0 },
    { 0x8f316c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3416c97, 3 },
    { 0, 0 },
    { 0x65356c99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d6ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28416ca4, 3 },
    { 0, 0 },
    { 0x60c16ca6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa156ca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2896caf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31256cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d6cb9, 4 },
    { 0x8af56cba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf16cbd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca656cc2, 3 },
    { 0xfb56cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d16cc8, 3 },
    { 0xae556cc8, 3 },
    { 0x486d6cc8, 3 },
    { 0xc8516cc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24d6cce, 3 },
    { 0x19cd6cce, 3 },
    { 0, 0 },
    { 0xe8f96cd1, 3 },
    { 0, 0 },
    { 0xd2b16cd3, 3 },
    { 0, 0 },
    { 0x5d6d6cd5, 3 },
    { 0x157d6cd6, 4 },
    { 0xce7d6cd7, 3 },
    { 0x76f16cd6, 3 },
    { 0x59ed6cd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310d6ce2, 3 },
    { 0x556d6ce3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ed6cea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e16ced, 4 },
    { 0x6d9d6cee, 3 },
    { 0xb8396cef, 4 },
    { 0x1d6cef, 4 },
    { 0x9ad96cf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85556cf4, 3 },
    { 0xefc56cf5, 4 },
    { 0xfe356cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84716cfb, 3 },
    { 0xaae96cfc, 4 },
    { 0x1ff96cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8d6d02, 3 },
    { 0, 0 },
    { 0xec16d04, 3 },
    { 0, 0 },
    { 0x77196d06, 3 },
    { 0x4f456d06, 4 },
    { 0, 0 },
    { 0xf94d6d09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c956d0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f16d10, 3 },
    { 0x6f6d6d10, 3 },
    { 0xb5196d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d6d15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c56d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1416d20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5156d32, 3 },
    { 0x523d6d33, 3 },
    { 0x28896d34, 3 },
    { 0x27916d33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6d6d3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd6d40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47d6d52, 4 },
    { 0x5a56d53, 3 },
    { 0x3ea16d53, 4 },
    { 0x36f96d55, 3 },
    { 0x9dfd6d55, 3 },
    { 0x24bd6d57, 3 },
    { 0xdf916d58, 3 },
    { 0xb7c96d59, 3 },
    { 0xee056d56, 3 },
    { 0xbbf96d5b, 3 },
    { 0, 0 },
    { 0x3cf16d5d, 3 },
    { 0xc2156d5d, 3 },
    { 0xd13d6d5f, 3 },
    { 0x13496d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b616d65, 3 },
    { 0xdc516d65, 3 },
    { 0, 0 },
    { 0x68396d68, 3 },
    { 0, 0 },
    { 0x86a56d6a, 4 },
    { 0x4a956d6b, 4 },
    { 0xc8396d6c, 3 },
    { 0xbfad6d6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x179d6d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41516d73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d6d7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a96d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21096d84, 3 },
    { 0xa0b56d85, 3 },
    { 0x68156d85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c96d8e, 3 },
    { 0x4eed6d8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce496d93, 3 },
    { 0xe59d6d93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5656d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15656d9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea56da2, 3 },
    { 0, 0 },
    { 0x22ed6da4, 3 },
    { 0xad7d6da5, 3 },
    { 0x83956da5, 4 },
    { 0x37ed6da6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c16dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f56db1, 4 },
    { 0, 0 },
    { 0xddd56db3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dd6dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f96dc1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b96dc5, 4 },
    { 0, 0 },
    { 0x86596dc7, 5 },
    { 0, 0 },
    { 0x86656dc9, 4 },
    { 0x50e16dca, 3 },
    { 0x24616dc9, 3 },
    { 0x68916dcb, 3 },
    { 0x2a8d6dca, 4 },
    { 0xde256dcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd16dd1, 4 },
    { 0, 0 },
    { 0x461d6dd3, 3 },
    { 0xbc356dd4, 3 },
    { 0x63a96dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979d6dd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d16ddc, 3 },
    { 0x63716ddd, 3 },
    { 0xe656ddd, 3 },
    { 0xfded6dde, 3 },
    { 0x76d56dde, 5 },
    { 0x85b96de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f16de5, 3 },
    { 0x92956de6, 4 },
    { 0x6da56de5, 3 },
    { 0xe6856de8, 3 },
    { 0x359d6de9, 3 },
    { 0x24816de8, 3 },
    { 0x1c056deb, 4 },
    { 0xc2996deb, 4 },
    { 0xf09d6deb, 3 },
    { 0x9f496de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c816df2, 3 },
    { 0xcb296df3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab216df9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd6dfc, 3 },
    { 0x77716dfd, 3 },
    { 0, 0 },
    { 0xbcd16dff, 3 },
    { 0x6c16dff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9816e04, 3 },
    { 0x95d16e05, 4 },
    { 0x847d6e04, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad6e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd6e0f, 3 },
    { 0, 0 },
    { 0x76a16e11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82216e1d, 3 },
    { 0xd5c56e1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x116e21, 5 },
    { 0, 0 },
    { 0x5a616e23, 3 },
    { 0x5fb96e24, 3 },
    { 0x792d6e24, 3 },
    { 0xc06d6e26, 3 },
    { 0x4a996e27, 3 },
    { 0x99b96e27, 3 },
    { 0x45416e28, 5 },
    { 0x4f416e29, 3 },
    { 0, 0 },
    { 0xc2a56e2c, 3 },
    { 0, 0 },
    { 0x3fa16e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4796e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3456e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56e3e, 3 },
    { 0x3dd56e3e, 3 },
    { 0, 0 },
    { 0x22bd6e41, 5 },
    { 0x63bd6e42, 4 },
    { 0, 0 },
    { 0xc9816e44, 4 },
    { 0x6df16e45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad716e4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17d6e4e, 3 },
    { 0xe1556e4f, 3 },
    { 0xf56d6e4e, 3 },
    { 0x3b0d6e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96296e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbd6e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe156e5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f96e60, 3 },
    { 0, 0 },
    { 0x1ca96e62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81816e65, 4 },
    { 0xfd16e66, 3 },
    { 0x9ce16e66, 3 },
    { 0x88816e68, 3 },
    { 0, 0 },
    { 0x10bd6e6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58d6e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe916e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60856e79, 3 },
    { 0xd7e96e7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27056e84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba16e88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a96e91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c456e96, 3 },
    { 0x856e97, 3 },
    { 0, 0 },
    { 0x3fa16e99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626d6e9e, 3 },
    { 0, 0 },
    { 0x3e596ea0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6996ea7, 4 },
    { 0x43996ea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b16eaf, 3 },
    { 0x4d96eb0, 4 },
    { 0xbc956eb1, 3 },
    { 0, 0 },
    { 0xeef56eb3, 4 },
    { 0x95916eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9556ebb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9996ec0, 3 },
    { 0, 0 },
    { 0x911d6ec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd6ec5, 3 },
    { 0x42096ec5, 5 },
    { 0xc6e56ec7, 3 },
    { 0, 0 },
    { 0x61196ec9, 4 },
    { 0xb08d6eca, 3 },
    { 0xc0296ecb, 3 },
    { 0xb8f56ecc, 3 },
    { 0x5656ec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b16ed6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca396ed9, 3 },
    { 0xe6996eda, 3 },
    { 0x59d96edb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6916ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf916ee8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d6eee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43256ef2, 4 },
    { 0x11bd6ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2196ef8, 3 },
    { 0x63416ef8, 3 },
    { 0xb0716ef9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff16f00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01d6f03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f56f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74216f0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e296f14, 3 },
    { 0x72856f15, 3 },
    { 0x62716f15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8556f1a, 4 },
    { 0x10316f1a, 3 },
    { 0xc0856f1a, 3 },
    { 0xb6616f1b, 3 },
    { 0x9c556f1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8996f26, 3 },
    { 0xbe5d6f27, 3 },
    { 0x9aa56f28, 3 },
    { 0xe0e16f27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee696f2c, 3 },
    { 0x72196f2d, 3 },
    { 0x3e5d6f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b96f31, 4 },
    { 0x50996f31, 3 },
    { 0xce416f33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cd6f37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d6f3b, 4 },
    { 0x95396f3b, 3 },
    { 0xf5d16f3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde696f40, 3 },
    { 0x76856f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1596f44, 3 },
    { 0, 0 },
    { 0xc86d6f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a56f49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395d6f4e, 4 },
    { 0x4be96f4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6f52, 3 },
    { 0xc6c16f53, 4 },
    { 0, 0 },
    { 0xe9456f55, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da56f5b, 3 },
    { 0, 0 },
    { 0x66e56f5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x36116f60, 3 },
    { 0, 0 },
    { 0x3a96f62, 4 },
    { 0x4b56f63, 4 },
    { 0, 0 },
    { 0xda416f65, 4 },
    { 0x6b196f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a56f6c, 4 },
    { 0, 0 },
    { 0x2f8d6f6e, 3 },
    { 0xd3f96f6f, 3 },
    { 0x81896f70, 3 },
    { 0x61996f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90456f75, 3 },
    { 0, 0 },
    { 0x64c96f77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47d6f7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c56f80, 4 },
    { 0x80996f80, 3 },
    { 0, 0 },
    { 0xd2216f83, 4 },
    { 0xdc556f83, 4 },
    { 0xe4556f85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9956f8e, 3 },
    { 0xccb16f8f, 3 },
    { 0xb0c96f8f, 3 },
    { 0x9c756f8f, 3 },
    { 0xc45d6f91, 3 },
    { 0, 0 },
    { 0xb1256f94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc796f97, 3 },
    { 0x79e96f97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ed6f9b, 3 },
    { 0x13ed6f9c, 3 },
    { 0xea2d6f9c, 5 },
    { 0xa9e96f9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed6fa5, 3 },
    { 0, 0 },
    { 0xc87d6fa7, 4 },
    { 0, 0 },
    { 0x25c56fa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94456fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18896fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a96fb7, 3 },
    { 0xb6fd6fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba96fbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6d6fbe, 3 },
    { 0x3bc96fbe, 3 },
    { 0x27a96fc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e16fc5, 3 },
    { 0, 0 },
    { 0x6d196fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dd6fcf, 4 },
    { 0, 0 },
    { 0xf4d6fd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edd6fd7, 3 },
    { 0x4f656fd8, 3 },
    { 0x93756fd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13156fdc, 5 },
    { 0x3f416fdc, 3 },
    { 0, 0 },
    { 0xc8c16fdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6316fe6, 3 },
    { 0, 0 },
    { 0xb3e16fe8, 3 },
    { 0x1a156fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7396fed, 3 },
    { 0x5bbd6fee, 4 },
    { 0, 0 },
    { 0x9f056ff0, 3 },
    { 0x5f4d6ff1, 3 },
    { 0x45b56ff2, 3 },
    { 0x33c56ff3, 3 },
    { 0xa8016ff4, 3 },
    { 0xcfd56ff4, 3 },
    { 0x6a8d6ff6, 3 },
    { 0, 0 },
    { 0x1a996ff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2d6ffc, 4 },
    { 0xeba56ffd, 3 },
    { 0xb756ffe, 4 },
    { 0, 0 },
    { 0x45157000, 4 },
    { 0, 0 },
    { 0x1d417002, 3 },
    { 0, 0 },
    { 0xe1c17004, 3 },
    { 0xae217005, 3 },
    { 0xcd1d7004, 3 },
    { 0x68dd7007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aed700c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8d7012, 3 },
    { 0, 0 },
    { 0xda217014, 3 },
    { 0, 0 },
    { 0xe7657016, 3 },
    { 0, 0 },
    { 0x86417018, 4 },
    { 0xcddd7019, 3 },
    { 0, 0 },
    { 0xa499701b, 3 },
    { 0xb111701c, 3 },
    { 0xb3c5701d, 3 },
    { 0xa0f1701e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91717021, 3 },
    { 0xaf8d7022, 4 },
    { 0xff6d7022, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd357026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6d7032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42297035, 3 },
    { 0xb2b17035, 4 },
    { 0x60d7037, 3 },
    { 0xee1d7038, 3 },
    { 0x7ae57035, 4 },
    { 0x44b97036, 4 },
    { 0xc835703b, 3 },
    { 0xe389703c, 4 },
    { 0x3331703d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4497042, 3 },
    { 0xcb457042, 4 },
    { 0x3b5d7044, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ded7047, 3 },
    { 0x8e917048, 3 },
    { 0xc4757049, 4 },
    { 0xfe79704a, 5 },
    { 0xf2e1704a, 3 },
    { 0, 0 },
    { 0x2601704d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a57050, 3 },
    { 0x40a57051, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66897055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c57058, 4 },
    { 0, 0 },
    { 0xf4c5705a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb245705d, 3 },
    { 0x9545705d, 4 },
    { 0xb039705f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d17069, 3 },
    { 0x7ca5706a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9706e, 3 },
    { 0, 0 },
    { 0xacd97070, 4 },
    { 0x22fd7071, 4 },
    { 0x76357072, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd7077, 5 },
    { 0, 0 },
    { 0x21697079, 3 },
    { 0x80b5707a, 3 },
    { 0xd9ad707a, 3 },
    { 0x55217079, 4 },
    { 0xb589707d, 4 },
    { 0x4921707d, 3 },
    { 0xe645707e, 3 },
    { 0xbde1707f, 3 },
    { 0x322d707f, 3 },
    { 0x8a517081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba597089, 3 },
    { 0, 0 },
    { 0x8815708b, 3 },
    { 0, 0 },
    { 0x44c1708d, 3 },
    { 0, 0 },
    { 0x1e01708f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c17096, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b617099, 4 },
    { 0, 0 },
    { 0x5325709b, 4 },
    { 0xf3e1709c, 3 },
    { 0xcd01709d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93970a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d70a6, 3 },
    { 0, 0 },
    { 0x92c570a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2570ac, 3 },
    { 0x906570ac, 3 },
    { 0x49f570ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58170b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87970be, 3 },
    { 0xf52170bf, 3 },
    { 0, 0 },
    { 0xf31970c1, 3 },
    { 0x40f170c2, 4 },
    { 0xe78170c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d70c8, 4 },
    { 0xb7c970c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x115170cc, 3 },
    { 0x850d70cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713170d1, 3 },
    { 0xe18970d2, 3 },
    { 0, 0 },
    { 0x3f3970d4, 3 },
    { 0xc43570d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e970d8, 3 },
    { 0x617570d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d70dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee970e4, 3 },
    { 0x297170e4, 3 },
    { 0x302970e6, 3 },
    { 0xed2d70e7, 3 },
    { 0x7f6d70e8, 3 },
    { 0x72a970e7, 4 },
    { 0xb76170ea, 3 },
    { 0x6dad70eb, 3 },
    { 0x71b970ec, 3 },
    { 0x3e4970ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aad70f4, 3 },
    { 0x341970f5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d170fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x170970fe, 3 },
    { 0, 0 },
    { 0xcc1d7100, 3 },
    { 0, 0 },
    { 0xeeb57102, 3 },
    { 0, 0 },
    { 0x5ec57104, 4 },
    { 0xe5397104, 3 },
    { 0, 0 },
    { 0x1b797107, 3 },
    { 0xbc217108, 3 },
    { 0xc1417109, 3 },
    { 0xd0f97109, 3 },
    { 0x241710a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d7111, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd7116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea5711f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06d7122, 4 },
    { 0x28817123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d7128, 3 },
    { 0, 0 },
    { 0xd149712a, 3 },
    { 0xb929712b, 3 },
    { 0x6e3d712c, 3 },
    { 0, 0 },
    { 0xa5cd712e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f57133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7d713a, 3 },
    { 0, 0 },
    { 0xe6b1713c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88457140, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d7143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd969714f, 3 },
    { 0, 0 },
    { 0xa00d7151, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba57154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee5715d, 4 },
    { 0x80c1715d, 3 },
    { 0xfc4d715f, 3 },
    { 0, 0 },
    { 0xf9a97161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c57164, 3 },
    { 0, 0 },
    { 0xd4b97166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95197169, 3 },
    { 0, 0 },
    { 0x2661716b, 3 },
    { 0, 0 },
    { 0x906d716d, 4 },
    { 0, 0 },
    { 0x5935716f, 4 },
    { 0xcd897170, 3 },
    { 0, 0 },
    { 0xd3357172, 3 },
    { 0, 0 },
    { 0xf15d7174, 4 },
    { 0xaccd7175, 3 },
    { 0xfe157175, 3 },
    { 0x6f057177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5125717a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ad7182, 4 },
    { 0, 0 },
    { 0x77297184, 3 },
    { 0x8397185, 3 },
    { 0xc6117186, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32097190, 3 },
    { 0x96e97191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd7194, 3 },
    { 0xecbd7195, 3 },
    { 0, 0 },
    { 0xdde97197, 3 },
    { 0, 0 },
    { 0x8d417199, 3 },
    { 0, 0 },
    { 0x9581719b, 3 },
    { 0, 0 },
    { 0xa659719d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d71a3, 3 },
    { 0, 0 },
    { 0x65f571a5, 3 },
    { 0, 0 },
    { 0xad0171a7, 3 },
    { 0x8b5571a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd71ad, 3 },
    { 0x9a5571ae, 3 },
    { 0x72e571af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d71b2, 3 },
    { 0xe9171b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8d71b7, 4 },
    { 0xa85d71b8, 3 },
    { 0, 0 },
    { 0x36571ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f971bd, 3 },
    { 0xcc9971be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7171c1, 3 },
    { 0xb31571c1, 3 },
    { 0, 0 },
    { 0xf05171c4, 4 },
    { 0x248171c4, 3 },
    { 0xd19571c5, 3 },
    { 0x191971c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b571cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42171d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167971d4, 3 },
    { 0xadc571d5, 3 },
    { 0x29dd71d6, 4 },
    { 0xf98d71d7, 4 },
    { 0x500d71d5, 3 },
    { 0x54b171d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f571dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd71e1, 3 },
    { 0xa7b171e1, 4 },
    { 0x880571e2, 4 },
    { 0x948171e4, 3 },
    { 0x1a8971e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c571e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd571eb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813971f2, 3 },
    { 0, 0 },
    { 0xca0171f4, 3 },
    { 0, 0 },
    { 0xab2971f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9171f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860571fe, 3 },
    { 0xb3d171ff, 3 },
    { 0x75957200, 3 },
    { 0xa2c57201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf57207, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa671720a, 3 },
    { 0x587d720b, 3 },
    { 0xbe19720c, 4 },
    { 0, 0 },
    { 0xcce5720e, 4 },
    { 0, 0 },
    { 0x18c17210, 3 },
    { 0x17bd7210, 3 },
    { 0x2d9d7210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86497215, 3 },
    { 0x6bd17216, 3 },
    { 0x17357217, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d721a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x543d721d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd7226, 4 },
    { 0xe8d17226, 3 },
    { 0, 0 },
    { 0xda57229, 3 },
    { 0x77f9722a, 3 },
    { 0xd6797229, 3 },
    { 0x4bc5722c, 3 },
    { 0, 0 },
    { 0x270d722e, 3 },
    { 0x9a1d722e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6d7232, 3 },
    { 0, 0 },
    { 0x4ee97234, 3 },
    { 0, 0 },
    { 0xa0f17236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ad723d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b97243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74657249, 3 },
    { 0x8bd1724a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c1724f, 3 },
    { 0xf3e1724f, 4 },
    { 0xf4d7251, 3 },
    { 0xccd57252, 3 },
    { 0x57617253, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d7257, 3 },
    { 0, 0 },
    { 0x403d7259, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb659725c, 3 },
    { 0x6c9d725d, 3 },
    { 0xf435725e, 3 },
    { 0x6c05725e, 4 },
    { 0x4c7d725e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb657263, 3 },
    { 0, 0 },
    { 0x53317265, 3 },
    { 0x9e157266, 3 },
    { 0xebad7267, 3 },
    { 0x82897268, 3 },
    { 0x4bf97269, 3 },
    { 0x4431726a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d957276, 3 },
    { 0, 0 },
    { 0xe1d7278, 3 },
    { 0x6497278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf17286, 3 },
    { 0, 0 },
    { 0x7c757288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2271728c, 4 },
    { 0, 0 },
    { 0x6885728e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd97298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9729b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7565729e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd72a2, 3 },
    { 0xd5cd72a3, 3 },
    { 0xf69572a2, 3 },
    { 0x30e972a5, 3 },
    { 0xec5172a6, 3 },
    { 0x801972a5, 5 },
    { 0x505972a5, 3 },
    { 0x26d72a5, 4 },
    { 0xcfd72aa, 3 },
    { 0x2db972ab, 3 },
    { 0x64c972ac, 3 },
    { 0x287172ad, 4 },
    { 0x6d9d72ae, 3 },
    { 0xbe2572ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa972b5, 3 },
    { 0, 0 },
    { 0x5a0d72b7, 3 },
    { 0xd22972b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef972bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55972be, 3 },
    { 0xe09972be, 3 },
    { 0x138172c0, 4 },
    { 0, 0 },
    { 0x63a172c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69572c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf972d0, 3 },
    { 0, 0 },
    { 0x538572d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318972db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0572e0, 4 },
    { 0xaeed72e1, 3 },
    { 0xc49d72e2, 3 },
    { 0x573172e1, 3 },
    { 0x953972e4, 3 },
    { 0xe1dd72e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3d72eb, 3 },
    { 0xf41972ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0d72f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d972f8, 3 },
    { 0x42ad72f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3972fd, 3 },
    { 0, 0 },
    { 0x144d72ff, 3 },
    { 0, 0 },
    { 0xbdd7301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f97304, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad730a, 4 },
    { 0x78c1730b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa121730e, 4 },
    { 0x6bdd730f, 3 },
    { 0x2751730f, 3 },
    { 0, 0 },
    { 0xefd97312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659731a, 3 },
    { 0xf00d731b, 4 },
    { 0x19ed731c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b97320, 3 },
    { 0xaa1d7320, 3 },
    { 0x34f57322, 3 },
    { 0xe3ad7322, 5 },
    { 0x9b817324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71617327, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf869732b, 3 },
    { 0, 0 },
    { 0x35cd732d, 4 },
    { 0, 0 },
    { 0x276d732f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9d7333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed17337, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6217340, 4 },
    { 0xa2e97341, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81d7346, 4 },
    { 0xbe017346, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d01734b, 4 },
    { 0x1c65734c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58657350, 3 },
    { 0x9f0d7351, 5 },
    { 0x878d7352, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b417356, 3 },
    { 0xc8cd7356, 3 },
    { 0xfbb97357, 3 },
    { 0xad657359, 4 },
    { 0x14ad735a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x719735d, 3 },
    { 0x87b5735e, 3 },
    { 0xcbbd735e, 4 },
    { 0x79e5735e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d57366, 3 },
    { 0xd7d17367, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb1736c, 3 },
    { 0, 0 },
    { 0x129d736e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7597371, 3 },
    { 0, 0 },
    { 0x213d7373, 3 },
    { 0x98c17373, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefad7377, 3 },
    { 0xeb597378, 3 },
    { 0xfc57379, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8497381, 3 },
    { 0x83397382, 3 },
    { 0xa0557381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bd7389, 3 },
    { 0, 0 },
    { 0x1f29738b, 3 },
    { 0, 0 },
    { 0x8ae9738d, 3 },
    { 0, 0 },
    { 0x5645738f, 3 },
    { 0, 0 },
    { 0x8f3d7391, 4 },
    { 0x5b297392, 3 },
    { 0x984d7393, 3 },
    { 0xd3197393, 3 },
    { 0, 0 },
    { 0x9cad7396, 3 },
    { 0, 0 },
    { 0x1e217398, 4 },
    { 0, 0 },
    { 0x93f9739a, 3 },
    { 0x27a1739b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944173a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d73a4, 3 },
    { 0, 0 },
    { 0x4cc573a6, 4 },
    { 0x724973a6, 3 },
    { 0xeff973a8, 3 },
    { 0x51fd73a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78173ac, 3 },
    { 0xc10973ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7173b2, 3 },
    { 0x8cd573b3, 3 },
    { 0x597573b4, 4 },
    { 0x9d6573b5, 3 },
    { 0x29dd73b5, 5 },
    { 0x8cad73b4, 4 },
    { 0x86d73b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a573bc, 3 },
    { 0xc9f973bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513973c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e173cc, 3 },
    { 0x95a173cc, 3 },
    { 0x574573ce, 3 },
    { 0x62573cc, 3 },
    { 0x206173d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e173d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b573e0, 3 },
    { 0, 0 },
    { 0xd3c173e2, 3 },
    { 0, 0 },
    { 0x156173e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcd73ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9573f8, 3 },
    { 0xb16573f9, 3 },
    { 0, 0 },
    { 0xfbc173fb, 3 },
    { 0x1be173fc, 4 },
    { 0xd56173fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d617405, 3 },
    { 0x10ad7406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e97411, 3 },
    { 0x46857411, 4 },
    { 0x88597413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb97416, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27257419, 3 },
    { 0xc8c1741a, 3 },
    { 0, 0 },
    { 0xb941741c, 4 },
    { 0, 0 },
    { 0x257d741e, 3 },
    { 0x92f1741f, 3 },
    { 0x49f9741f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d817424, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad7428, 3 },
    { 0, 0 },
    { 0xa215742a, 3 },
    { 0xe1c1742b, 3 },
    { 0, 0 },
    { 0x58e9742d, 4 },
    { 0xd011742e, 3 },
    { 0, 0 },
    { 0x6cf57430, 3 },
    { 0x85197431, 3 },
    { 0, 0 },
    { 0xd797433, 4 },
    { 0x2a657433, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29617438, 4 },
    { 0xb7797439, 5 },
    { 0x9d29743a, 3 },
    { 0, 0 },
    { 0xbd59743c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f1743f, 3 },
    { 0x13f57440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc517446, 3 },
    { 0xc6397447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d744c, 3 },
    { 0x4239744d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30897451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c57455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d57458, 3 },
    { 0xc117459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec9745d, 3 },
    { 0xc795745e, 3 },
    { 0x1c11745d, 3 },
    { 0x36817460, 4 },
    { 0x14d97461, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d7464, 3 },
    { 0x13617464, 3 },
    { 0x87bd7466, 3 },
    { 0x92ed7465, 3 },
    { 0x671d7464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7d746c, 4 },
    { 0x7c09746c, 4 },
    { 0x1321746e, 3 },
    { 0, 0 },
    { 0x43c97470, 3 },
    { 0xe3097471, 3 },
    { 0, 0 },
    { 0x71197473, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7181747c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a57484, 3 },
    { 0x64117485, 3 },
    { 0, 0 },
    { 0xdb17487, 4 },
    { 0, 0 },
    { 0x42917489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b5748c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c17490, 3 },
    { 0xa0ed7491, 3 },
    { 0xca5d7491, 4 },
    { 0x22657490, 3 },
    { 0x68d97494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270d7499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7974a0, 3 },
    { 0x148d74a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc974a5, 4 },
    { 0, 0 },
    { 0xaac174a7, 3 },
    { 0x374574a8, 4 },
    { 0x7bd174a8, 3 },
    { 0x34a974a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cd74ad, 3 },
    { 0, 0 },
    { 0x72f974af, 3 },
    { 0x277d74af, 3 },
    { 0xdc574af, 3 },
    { 0xd6c974b2, 4 },
    { 0x19e174b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1174ba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1174bf, 3 },
    { 0x90dd74c0, 3 },
    { 0x7cd974c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e974c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f974c8, 3 },
    { 0x93f574c8, 4 },
    { 0, 0 },
    { 0x6f4d74cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53174d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b574d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21574d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e174db, 3 },
    { 0xe7c574dc, 4 },
    { 0x13d574dc, 3 },
    { 0x8c7d74db, 3 },
    { 0xc2b574df, 3 },
    { 0xe3f574e0, 3 },
    { 0, 0 },
    { 0x1b9d74e2, 3 },
    { 0x3e4574e2, 3 },
    { 0x944574e3, 3 },
    { 0x58d174e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b574eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00974f0, 3 },
    { 0x5a2d74f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a174fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7317504, 4 },
    { 0x18997505, 3 },
    { 0xa0c97506, 3 },
    { 0x4fa17507, 3 },
    { 0x6497508, 3 },
    { 0xd86d7509, 3 },
    { 0x91457506, 3 },
    { 0xab9750b, 3 },
    { 0xfb557506, 3 },
    { 0x93b57506, 4 },
    { 0x38c1750d, 4 },
    { 0xad0d750f, 3 },
    { 0xf61750f, 3 },
    { 0xf3e57511, 3 },
    { 0x27f97512, 3 },
    { 0xb9ad7513, 3 },
    { 0x38e57514, 3 },
    { 0x94fd7515, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4157518, 3 },
    { 0, 0 },
    { 0x7d91751a, 4 },
    { 0, 0 },
    { 0xf905751c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79751f, 3 },
    { 0, 0 },
    { 0x9b7d7521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b57528, 3 },
    { 0xbe197528, 3 },
    { 0xc9e97528, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9752f, 3 },
    { 0, 0 },
    { 0xd7697531, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14357534, 4 },
    { 0xf48d7534, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47357540, 3 },
    { 0x67b97541, 3 },
    { 0xc8197542, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4197545, 3 },
    { 0xcc0d7545, 3 },
    { 0x8ee97547, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41754a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85754d, 3 },
    { 0x7555754e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b57554, 3 },
    { 0x19017555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9755b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49157565, 3 },
    { 0, 0 },
    { 0x22a97567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc41756b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d7571, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf717578, 3 },
    { 0x87197579, 3 },
    { 0xbd417578, 3 },
    { 0x7665757b, 3 },
    { 0x3bd9757c, 4 },
    { 0x3e79757b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x48857580, 3 },
    { 0, 0 },
    { 0xa5057582, 4 },
    { 0x8add7582, 3 },
    { 0x5b3d7584, 3 },
    { 0, 0 },
    { 0x63697586, 3 },
    { 0x708d7587, 3 },
    { 0x89917588, 3 },
    { 0x798d7589, 3 },
    { 0, 0 },
    { 0x8465758b, 4 },
    { 0x7c65758b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe391758f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62097596, 3 },
    { 0x5ee97597, 3 },
    { 0x36e57598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05759b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3175a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac975a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866975ac, 3 },
    { 0xaec975ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371975b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954575b6, 4 },
    { 0x48dd75b7, 4 },
    { 0xce5575b7, 3 },
    { 0xeef975b7, 3 },
    { 0, 0 },
    { 0x1fcd75bb, 4 },
    { 0x7b6175bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd75c1, 3 },
    { 0, 0 },
    { 0x756975c3, 3 },
    { 0, 0 },
    { 0x289575c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcd75c8, 4 },
    { 0, 0 },
    { 0x771175ca, 3 },
    { 0, 0 },
    { 0x9e5975cc, 3 },
    { 0, 0 },
    { 0x541d75ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e975d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3175d6, 4 },
    { 0xd1dd75d7, 3 },
    { 0xef4975d8, 3 },
    { 0, 0 },
    { 0x26975da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352975df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ad75e2, 3 },
    { 0, 0 },
    { 0xa0d575e4, 4 },
    { 0x3edd75e5, 3 },
    { 0, 0 },
    { 0xb5f975e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d575ea, 3 },
    { 0xf7a175ea, 3 },
    { 0x4a1175ea, 3 },
    { 0x661975ed, 3 },
    { 0x3eb175ee, 3 },
    { 0x2fd975ef, 4 },
    { 0x42cd75f0, 3 },
    { 0xd575f1, 3 },
    { 0, 0 },
    { 0x85f175f3, 3 },
    { 0, 0 },
    { 0x4f7575f5, 3 },
    { 0x70e175f6, 3 },
    { 0x847575f7, 3 },
    { 0x449575f6, 3 },
    { 0, 0 },
    { 0x57d75fa, 3 },
    { 0x7d9575fb, 4 },
    { 0, 0 },
    { 0x518975fd, 4 },
    { 0xd0fd75fd, 3 },
    { 0x7e8975fe, 4 },
    { 0x1eb975fd, 3 },
    { 0x1b957600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9717607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891760b, 3 },
    { 0, 0 },
    { 0x3771760d, 5 },
    { 0xdda5760e, 3 },
    { 0x1ee9760d, 3 },
    { 0, 0 },
    { 0xbdf97611, 3 },
    { 0, 0 },
    { 0x175d7613, 3 },
    { 0x818d7614, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141761b, 4 },
    { 0x34cd761c, 3 },
    { 0xec9d761d, 3 },
    { 0xc3c9761d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c17621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8117625, 3 },
    { 0x9c517626, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e357629, 3 },
    { 0xbac9762a, 4 },
    { 0x3f5762a, 4 },
    { 0x8e557629, 3 },
    { 0x563d7629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7157635, 3 },
    { 0, 0 },
    { 0x5e517637, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37d763a, 3 },
    { 0x71bd763b, 3 },
    { 0x9285763b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7657645, 3 },
    { 0, 0 },
    { 0x95617647, 3 },
    { 0x99d57647, 4 },
    { 0xc9a57649, 3 },
    { 0x9d69764a, 5 },
    { 0xc6b1764b, 4 },
    { 0x3449764a, 3 },
    { 0xd73d764d, 3 },
    { 0x43c9764d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbd7652, 3 },
    { 0x99b57652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d7658, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566d765d, 3 },
    { 0, 0 },
    { 0x4489765f, 3 },
    { 0xb357660, 3 },
    { 0xf3d765f, 3 },
    { 0x3fbd7661, 3 },
    { 0, 0 },
    { 0xce657664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ad766b, 4 },
    { 0xe80d766b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc99766f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3897675, 4 },
    { 0xb7c57676, 3 },
    { 0x96097676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d1767a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ad767d, 3 },
    { 0x9d5d767d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81d7690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17694, 4 },
    { 0x85557695, 3 },
    { 0, 0 },
    { 0xf2757697, 4 },
    { 0x96757698, 3 },
    { 0xe6117699, 3 },
    { 0x6c957698, 3 },
    { 0x4f11769b, 3 },
    { 0x7717698, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c576a2, 3 },
    { 0x2db976a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x119976a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba176aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2576b0, 3 },
    { 0x7bd176b0, 3 },
    { 0, 0 },
    { 0x7cfd76b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x910d76b6, 3 },
    { 0x910576b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9976ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b976bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dd76c4, 3 },
    { 0x362d76c5, 3 },
    { 0xad4976c6, 3 },
    { 0x7fd76c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760176cd, 4 },
    { 0, 0 },
    { 0xd61576cf, 3 },
    { 0x1a2576d0, 4 },
    { 0x60ed76d0, 3 },
    { 0x60f976d2, 3 },
    { 0, 0 },
    { 0x8ec976d4, 3 },
    { 0xdced76d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dd76d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7576db, 4 },
    { 0x2dd576dc, 3 },
    { 0x98576dd, 4 },
    { 0xc53d76de, 4 },
    { 0x352976df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96976e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36d76e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6576ec, 4 },
    { 0x425d76ed, 3 },
    { 0xc6f176ee, 4 },
    { 0x9cd976ef, 3 },
    { 0xd31176ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce176f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13576fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a976ff, 3 },
    { 0xea617700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d97704, 4 },
    { 0x2d817705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf75770b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff97710, 3 },
    { 0xde897710, 3 },
    { 0x84397712, 4 },
    { 0xbbb17712, 4 },
    { 0x5aad7714, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae57718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x975d771b, 3 },
    { 0xdd1771c, 4 },
    { 0, 0 },
    { 0x4f29771e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d517724, 3 },
    { 0, 0 },
    { 0x703d7726, 4 },
    { 0xe4557726, 3 },
    { 0, 0 },
    { 0x47657729, 3 },
    { 0xee31772a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xada1772d, 4 },
    { 0x9261772e, 3 },
    { 0xa835772f, 3 },
    { 0xa99d7730, 3 },
    { 0xe0857730, 3 },
    { 0xf8297732, 3 },
    { 0x603d7733, 3 },
    { 0xa6497734, 4 },
    { 0x74997735, 3 },
    { 0xc7f97736, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75157739, 3 },
    { 0x2c397739, 3 },
    { 0x5a41773b, 3 },
    { 0x2fc5773b, 3 },
    { 0, 0 },
    { 0xc971773e, 3 },
    { 0, 0 },
    { 0xf957740, 3 },
    { 0, 0 },
    { 0x49757742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71357747, 4 },
    { 0x770d7747, 3 },
    { 0, 0 },
    { 0x4a01774a, 4 },
    { 0x89b5774b, 4 },
    { 0xbc5d774c, 5 },
    { 0xd31774d, 3 },
    { 0x9db5774e, 3 },
    { 0xa685774e, 3 },
    { 0xc37d774f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e57754, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9755775d, 4 },
    { 0x77ed775d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e57764, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff17767, 3 },
    { 0xbb957767, 3 },
    { 0xde717769, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc997770, 4 },
    { 0xec517771, 4 },
    { 0x9317771, 4 },
    { 0xe5cd7772, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e9777a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d9777d, 4 },
    { 0, 0 },
    { 0xbde5777f, 4 },
    { 0x102d7780, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca17784, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6497787, 4 },
    { 0x75857788, 3 },
    { 0x9c197787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd105778f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d7795, 3 },
    { 0x23957795, 3 },
    { 0xbd797797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce91779a, 4 },
    { 0x8a05779b, 3 },
    { 0xe18d779c, 3 },
    { 0x6345779d, 3 },
    { 0x760d779c, 3 },
    { 0x7c6d779f, 3 },
    { 0xfbad779f, 5 },
    { 0x400177a1, 3 },
    { 0, 0 },
    { 0x22a977a3, 4 },
    { 0, 0 },
    { 0xa5a977a5, 3 },
    { 0xbc0d77a6, 3 },
    { 0, 0 },
    { 0x3dd977a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3577ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b577ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4977b7, 3 },
    { 0xff4d77b8, 3 },
    { 0, 0 },
    { 0x7f3977ba, 4 },
    { 0, 0 },
    { 0x3c4177bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793977c1, 4 },
    { 0xb9b977c1, 3 },
    { 0, 0 },
    { 0xc2dd77c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9d77ca, 3 },
    { 0, 0 },
    { 0x4e0577cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427577d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd577dc, 3 },
    { 0xe92977dc, 3 },
    { 0x6da977dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28977e9, 3 },
    { 0, 0 },
    { 0xb68177eb, 3 },
    { 0, 0 },
    { 0x4dcd77ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c177f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cd77f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21577f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0577fb, 4 },
    { 0, 0 },
    { 0xf4e177fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2d7800, 3 },
    { 0, 0 },
    { 0xebf57802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b17806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e29780a, 4 },
    { 0x6e55780b, 3 },
    { 0x3345780c, 3 },
    { 0, 0 },
    { 0x88cd780e, 3 },
    { 0xd4d9780f, 3 },
    { 0x52117810, 4 },
    { 0, 0 },
    { 0xb4f97812, 4 },
    { 0, 0 },
    { 0xf7a17814, 3 },
    { 0, 0 },
    { 0x18bd7816, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf17819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b1781d, 3 },
    { 0xace5781e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15d7825, 4 },
    { 0x106d7826, 3 },
    { 0x1ee57827, 4 },
    { 0x17257828, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d31782b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97497831, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37457834, 4 },
    { 0xf4c97835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2417839, 3 },
    { 0x3f5d783a, 3 },
    { 0x8a997839, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b97841, 3 },
    { 0x37117842, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d17845, 3 },
    { 0xc1cd7846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf17849, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450d7850, 3 },
    { 0xf3f57850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eed7855, 4 },
    { 0x95ad7856, 3 },
    { 0, 0 },
    { 0x297d7858, 3 },
    { 0xdf817858, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d785c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3d7869, 4 },
    { 0x2935786a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d786e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb97872, 3 },
    { 0xf15d7872, 3 },
    { 0, 0 },
    { 0x29417875, 4 },
    { 0, 0 },
    { 0x45997877, 3 },
    { 0xd6c57878, 3 },
    { 0, 0 },
    { 0x88e9787a, 3 },
    { 0xcfbd787a, 3 },
    { 0x34c1787a, 3 },
    { 0, 0 },
    { 0xeb29787e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb57884, 4 },
    { 0x46b97885, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf17888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9788d, 3 },
    { 0xb03d788e, 3 },
    { 0, 0 },
    { 0x58017890, 3 },
    { 0, 0 },
    { 0x9e197892, 4 },
    { 0x61d97893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a17899, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3578a4, 3 },
    { 0x5de978a5, 3 },
    { 0x4dbd78a5, 3 },
    { 0, 0 },
    { 0x54dd78a8, 3 },
    { 0x8dfd78a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d978b4, 4 },
    { 0, 0 },
    { 0xcbf578b6, 3 },
    { 0x17d178b7, 3 },
    { 0xf46978b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132178bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee178bf, 3 },
    { 0xef6d78c0, 3 },
    { 0xaa7978c0, 3 },
    { 0xb46d78c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x965578c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6178c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d78cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd78cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95178d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02978d8, 4 },
    { 0x46d978d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab578e1, 3 },
    { 0xb02d78e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d78e8, 3 },
    { 0x5af578e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b578ef, 3 },
    { 0xc49178f0, 4 },
    { 0, 0 },
    { 0x9e2978f2, 3 },
    { 0x5ad978f2, 3 },
    { 0x73a178f3, 3 },
    { 0x1fd578f2, 4 },
    { 0xd3a178f6, 3 },
    { 0xc99978f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980978fb, 3 },
    { 0, 0 },
    { 0xfba578fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d617900, 4 },
    { 0x978d7901, 4 },
    { 0x29c57901, 3 },
    { 0x89dd7903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b97908, 3 },
    { 0x95257908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821790c, 3 },
    { 0xfa0d790d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8d7914, 3 },
    { 0xcf757915, 4 },
    { 0xe8e97916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d7922, 4 },
    { 0x7d1d7923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad31792d, 5 },
    { 0, 0 },
    { 0xd259792f, 3 },
    { 0xf22d7930, 4 },
    { 0x9f717930, 4 },
    { 0x88957932, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a97939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf5793d, 3 },
    { 0x4a41793e, 3 },
    { 0x7979793f, 4 },
    { 0xb411793e, 3 },
    { 0xe1857941, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73257949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b297951, 4 },
    { 0x4c017952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e497957, 4 },
    { 0x7df17958, 3 },
    { 0x2d117959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2395795d, 3 },
    { 0, 0 },
    { 0xeb5d795f, 3 },
    { 0xfe49795f, 3 },
    { 0, 0 },
    { 0x7a717962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3319796b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f1796e, 3 },
    { 0xb40d796f, 4 },
    { 0xb5897970, 3 },
    { 0, 0 },
    { 0x4e717972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a57977, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cd797c, 3 },
    { 0x83a5797d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c557982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f81798b, 3 },
    { 0x44c1798c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01798f, 4 },
    { 0xd76d7990, 4 },
    { 0, 0 },
    { 0x24cd7992, 3 },
    { 0xf9297993, 4 },
    { 0xd9457994, 3 },
    { 0x40d17992, 3 },
    { 0xffb17996, 4 },
    { 0x41a57997, 3 },
    { 0xffa97998, 3 },
    { 0x28b97999, 3 },
    { 0x43b97998, 3 },
    { 0xc4697996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a5799e, 3 },
    { 0x17ed799e, 4 },
    { 0x182979a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1d79a6, 3 },
    { 0x306979a7, 3 },
    { 0, 0 },
    { 0x84e179a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd979ad, 3 },
    { 0, 0 },
    { 0x557179af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30979b5, 5 },
    { 0x995979b6, 4 },
    { 0xa36179b6, 3 },
    { 0xa8b179b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949179ca, 4 },
    { 0x227179cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391d79d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f579db, 3 },
    { 0x2ed579dc, 4 },
    { 0xe979db, 3 },
    { 0xb80179de, 3 },
    { 0x540579df, 3 },
    { 0, 0 },
    { 0x2f6579e1, 4 },
    { 0, 0 },
    { 0x5ad179e3, 3 },
    { 0, 0 },
    { 0xa53979e5, 4 },
    { 0x4b2179e5, 3 },
    { 0x851179e7, 3 },
    { 0xa62979e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a179ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92579f2, 3 },
    { 0xa66579f3, 3 },
    { 0x81f579f4, 3 },
    { 0x56b579f5, 3 },
    { 0xcb4979f4, 3 },
    { 0x895179f5, 3 },
    { 0xe72579f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710579fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e997a00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d7a05, 4 },
    { 0x81c17a06, 3 },
    { 0x6ff17a07, 3 },
    { 0x21617a08, 4 },
    { 0xb0757a07, 3 },
    { 0x22417a09, 3 },
    { 0x91917a07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b57a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2257a14, 4 },
    { 0, 0 },
    { 0xd3a97a16, 3 },
    { 0xeca57a17, 3 },
    { 0x11097a17, 4 },
    { 0, 0 },
    { 0xf6997a1a, 3 },
    { 0, 0 },
    { 0x22257a1c, 3 },
    { 0xeb317a1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39497a28, 4 },
    { 0, 0 },
    { 0x3de17a2a, 3 },
    { 0x3f817a2b, 4 },
    { 0x46117a2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d97a30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeced7a3a, 4 },
    { 0, 0 },
    { 0x51817a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b97a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c517a4c, 3 },
    { 0x925d7a4d, 3 },
    { 0, 0 },
    { 0x294d7a4f, 3 },
    { 0x90917a50, 3 },
    { 0xc2357a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1217a55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf017a5b, 3 },
    { 0, 0 },
    { 0x50f57a5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c297a61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x47957a64, 4 },
    { 0x38317a65, 4 },
    { 0xae297a66, 3 },
    { 0, 0 },
    { 0xa29d7a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e397a6f, 3 },
    { 0x55617a70, 4 },
    { 0xbea97a71, 3 },
    { 0x75c17a72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d7a78, 3 },
    { 0x437d7a78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xee717a7c, 3 },
    { 0x75697a7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582d7a82, 5 },
    { 0xeb917a83, 3 },
    { 0, 0 },
    { 0xb8dd7a85, 3 },
    { 0x82297a86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdd7a89, 5 },
    { 0x1e297a8a, 3 },
    { 0, 0 },
    { 0xd65d7a8c, 3 },
    { 0xe0d17a8d, 4 },
    { 0x78c97a8d, 4 },
    { 0x97217a8f, 4 },
    { 0x4e5d7a8f, 3 },
    { 0x30397a8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdd7a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d97a9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c057aa1, 3 },
    { 0xe0397aa2, 4 },
    { 0xed117aa2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f17aad, 3 },
    { 0xbe117aae, 3 },
    { 0x71757aae, 3 },
    { 0x2a57ab0, 3 },
    { 0x53457ab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d7ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db57ab7, 4 },
    { 0xa2757ab8, 3 },
    { 0x4ca97ab9, 3 },
    { 0xec897aba, 3 },
    { 0xa8d97ab8, 3 },
    { 0x2f997abc, 4 },
    { 0x7117abc, 3 },
    { 0x808d7abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd17ac3, 3 },
    { 0x5c557ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b517acd, 3 },
    { 0x3c197ace, 4 },
    { 0x94f17acf, 4 },
    { 0x1c817ad0, 3 },
    { 0xe7ad7ad1, 4 },
    { 0x1e117ad0, 3 },
    { 0, 0 },
    { 0x8e0d7ad4, 3 },
    { 0x6e1d7ad5, 3 },
    { 0x6e517ad6, 3 },
    { 0x474d7ad6, 3 },
    { 0x397ad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d7adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce197ae2, 4 },
    { 0, 0 },
    { 0x50f97ae4, 3 },
    { 0x69b17ae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc57aeb, 3 },
    { 0x89d57aeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd7af3, 5 },
    { 0x377d7af3, 3 },
    { 0xea357af5, 3 },
    { 0x4dfd7af3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad7afa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dd7afd, 4 },
    { 0x3b0d7afd, 3 },
    { 0xcfe17afd, 3 },
    { 0x57497b00, 3 },
    { 0, 0 },
    { 0x41f17b02, 4 },
    { 0xed397b03, 3 },
    { 0, 0 },
    { 0x575d7b05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x722d7b08, 3 },
    { 0x63317b09, 3 },
    { 0x14bd7b0a, 3 },
    { 0xb0617b08, 4 },
    { 0x30017b0b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace57b11, 3 },
    { 0xc46d7b12, 3 },
    { 0, 0 },
    { 0x24297b14, 3 },
    { 0x38a97b14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37117b1b, 3 },
    { 0x2f917b1c, 3 },
    { 0x3be17b1d, 3 },
    { 0, 0 },
    { 0x7af57b1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98757b22, 4 },
    { 0x3dad7b23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c57b2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90057b2f, 3 },
    { 0xaa17b30, 3 },
    { 0xa5a17b2f, 3 },
    { 0, 0 },
    { 0xb17d7b33, 4 },
    { 0x1cd17b34, 3 },
    { 0x51517b35, 3 },
    { 0x86857b34, 3 },
    { 0x2e657b36, 3 },
    { 0xca3d7b38, 4 },
    { 0xb3517b35, 4 },
    { 0x4b117b36, 3 },
    { 0x80457b39, 3 },
    { 0x28e57b38, 3 },
    { 0x304d7b3d, 3 },
    { 0x82257b34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d7b41, 4 },
    { 0x5d497b42, 3 },
    { 0, 0 },
    { 0x1d3d7b44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc017b48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597b4d, 3 },
    { 0xf73d7b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f57b51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82457b56, 5 },
    { 0x7d797b57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f57b5c, 3 },
    { 0x337d7b5c, 3 },
    { 0xeb657b5c, 3 },
    { 0x2fdd7b5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa57b66, 3 },
    { 0x30197b67, 3 },
    { 0x717b68, 3 },
    { 0x1a817b69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bd7b6f, 3 },
    { 0, 0 },
    { 0xb8957b71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2497b76, 3 },
    { 0, 0 },
    { 0x4c097b78, 3 },
    { 0x9fd17b78, 3 },
    { 0xf24d7b78, 3 },
    { 0xdc097b7b, 3 },
    { 0x75f97b7b, 3 },
    { 0xbac57b7d, 3 },
    { 0x8c597b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e17b87, 3 },
    { 0xe0ed7b88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40717b8f, 4 },
    { 0x71317b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed7b93, 3 },
    { 0x58597b93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad7b98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb17b9d, 3 },
    { 0, 0 },
    { 0x9d197b9f, 4 },
    { 0x36597ba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d57ba3, 3 },
    { 0, 0 },
    { 0x53a97ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef317ba9, 3 },
    { 0xea2d7baa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e97bae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e97bb1, 4 },
    { 0xbdc57bb1, 3 },
    { 0xd09d7bb3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63857bb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad57bba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b17bbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38357bc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb017bc7, 4 },
    { 0x7ca97bc7, 3 },
    { 0x23d57bc9, 3 },
    { 0x59917bca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f97bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8757bd1, 4 },
    { 0x2b0d7bd2, 3 },
    { 0, 0 },
    { 0x3ad7bd4, 4 },
    { 0x449d7bd5, 3 },
    { 0x59657bd5, 3 },
    { 0xfdfd7bd7, 3 },
    { 0, 0 },
    { 0xbf7d7bd9, 3 },
    { 0x66d97bda, 3 },
    { 0, 0 },
    { 0x2b4d7bdc, 4 },
    { 0xf5d17bdc, 4 },
    { 0x5ed7bdc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b897be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4d7be7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3997bee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00d7bf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dd7bff, 3 },
    { 0x2ad57c00, 3 },
    { 0x67617c01, 3 },
    { 0xdc757c01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b97c0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53557c0f, 3 },
    { 0xe4097c0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51417c13, 4 },
    { 0x2b1d7c14, 4 },
    { 0xcc897c15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d7c20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd97c25, 3 },
    { 0xc4c97c25, 3 },
    { 0, 0 },
    { 0x402d7c28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e417c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12297c2f, 3 },
    { 0x76f57c30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b857c3b, 4 },
    { 0, 0 },
    { 0x17857c3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f17c40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf97c44, 3 },
    { 0, 0 },
    { 0xf5f57c46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf57c49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d97c4c, 4 },
    { 0x58c57c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e17c59, 3 },
    { 0x35497c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f57c5d, 5 },
    { 0, 0 },
    { 0x50fd7c5f, 4 },
    { 0x10557c60, 3 },
    { 0x48e17c60, 3 },
    { 0x138d7c62, 3 },
    { 0x16257c63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0317c6e, 4 },
    { 0xe30d7c6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb197c73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70697c78, 3 },
    { 0xa2457c78, 3 },
    { 0x4a757c7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525d7c80, 3 },
    { 0x8db97c81, 3 },
    { 0, 0 },
    { 0x5a497c83, 3 },
    { 0, 0 },
    { 0x24517c85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c17c8e, 4 },
    { 0x2717c8f, 3 },
    { 0xc6997c90, 3 },
    { 0x6c457c91, 4 },
    { 0xd3b17c8f, 3 },
    { 0x8a917c93, 3 },
    { 0, 0 },
    { 0x834d7c95, 5 },
    { 0x84017c96, 4 },
    { 0x25c57c97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4217ca1, 3 },
    { 0x3ae97ca2, 4 },
    { 0xc8217ca3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ed7ca6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e57cab, 3 },
    { 0x6c717cac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e57caf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3657cb3, 3 },
    { 0, 0 },
    { 0x58917cb5, 3 },
    { 0, 0 },
    { 0xa2197cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b97cba, 3 },
    { 0x32757cbb, 3 },
    { 0xebd97cbb, 4 },
    { 0xccc17cbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce97cc0, 3 },
    { 0x4b557cc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496d7cc6, 3 },
    { 0xd6797cc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c397cd0, 3 },
    { 0xae397cd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e117cd7, 3 },
    { 0x739d7cd8, 4 },
    { 0x8097cd9, 3 },
    { 0x1b997cd7, 3 },
    { 0, 0 },
    { 0xc5017cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d7ce0, 4 },
    { 0, 0 },
    { 0xc9917ce2, 3 },
    { 0xf81d7ce3, 3 },
    { 0xa5417ce4, 3 },
    { 0xe0217ce5, 4 },
    { 0x2de97ce5, 3 },
    { 0xbe397ce7, 4 },
    { 0x3f917ce5, 3 },
    { 0xe9b57ce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a97cee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec57cf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bd7cf6, 3 },
    { 0xf8c97cf6, 3 },
    { 0x12b57cf8, 3 },
    { 0, 0 },
    { 0x791d7cfa, 4 },
    { 0x1fa17cfa, 3 },
    { 0, 0 },
    { 0x6d357cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96d7d00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e97d03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5457d0b, 3 },
    { 0x68697d0c, 4 },
    { 0x54117d0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d017d10, 3 },
    { 0x93e17d11, 3 },
    { 0x87cd7d12, 3 },
    { 0x57797d13, 3 },
    { 0x83697d11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afd7d18, 3 },
    { 0x1bb97d18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a97d1f, 3 },
    { 0xdde97d20, 3 },
    { 0, 0 },
    { 0x7be17d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12157d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56297d2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b457d33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97697d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3697d39, 4 },
    { 0x479d7d3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96897d47, 4 },
    { 0x41d17d48, 4 },
    { 0x98ed7d49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c057d52, 3 },
    { 0, 0 },
    { 0xd7a17d54, 3 },
    { 0x81c17d54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd357d5c, 4 },
    { 0xd2cd7d5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce17d62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbd7d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dd7d6e, 3 },
    { 0x58f97d6e, 4 },
    { 0, 0 },
    { 0xb8957d71, 3 },
    { 0x8b497d71, 4 },
    { 0, 0 },
    { 0x9d717d74, 3 },
    { 0xa1c17d75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf17d7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35797d7d, 3 },
    { 0xf52d7d7e, 3 },
    { 0xffa97d7f, 3 },
    { 0x2c617d7f, 3 },
    { 0x99f17d81, 3 },
    { 0x15857d81, 4 },
    { 0x29dd7d82, 4 },
    { 0xe0957d84, 3 },
    { 0x1c417d84, 3 },
    { 0x5897d86, 3 },
    { 0, 0 },
    { 0x6857d88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb697d8c, 4 },
    { 0x8c857d8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad7d95, 5 },
    { 0x3f817d95, 3 },
    { 0xfcb57d97, 3 },
    { 0xddd7d98, 3 },
    { 0x95f97d99, 3 },
    { 0xbd417d99, 3 },
    { 0x18897d9a, 3 },
    { 0x591d7d9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbd7d9f, 3 },
    { 0x94117da0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a257da3, 3 },
    { 0x40917da4, 3 },
    { 0x85657da3, 3 },
    { 0xf7517da6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d7da9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d7daf, 3 },
    { 0x343d7daf, 3 },
    { 0xeb717db1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56757db6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f17dbb, 5 },
    { 0xc70d7dbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf297dc1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f17dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7dca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d7dd7, 3 },
    { 0, 0 },
    { 0x72417dd9, 3 },
    { 0, 0 },
    { 0x31957ddb, 3 },
    { 0, 0 },
    { 0x94e97ddd, 4 },
    { 0, 0 },
    { 0x953d7ddf, 3 },
    { 0x3b597de0, 3 },
    { 0x14917de0, 3 },
    { 0xc6dd7de2, 3 },
    { 0, 0 },
    { 0x65517de4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c17de7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc57dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0557df0, 3 },
    { 0xdc557df1, 4 },
    { 0, 0 },
    { 0xab697df3, 3 },
    { 0x277d7df3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb97dfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f97dfd, 4 },
    { 0x33897dfd, 3 },
    { 0x61517dff, 3 },
    { 0xe91d7e00, 4 },
    { 0xb55d7e00, 4 },
    { 0xadd57dff, 3 },
    { 0x95557e02, 4 },
    { 0xb97e00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b997e07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac17e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6d7e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b197e17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad857e1a, 3 },
    { 0, 0 },
    { 0xd1217e1c, 3 },
    { 0x42457e1c, 4 },
    { 0, 0 },
    { 0xfffd7e1f, 3 },
    { 0xcff17e20, 3 },
    { 0xfdf97e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53d7e26, 3 },
    { 0x3f17e26, 3 },
    { 0x7abd7e28, 5 },
    { 0x3a057e28, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf17e2c, 5 },
    { 0x1bc97e2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ed7e32, 3 },
    { 0x1ef57e33, 3 },
    { 0x6e497e32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63217e3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d357e3e, 3 },
    { 0x91057e3f, 3 },
    { 0xd4817e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a97e43, 3 },
    { 0x7e397e43, 4 },
    { 0xbc917e44, 3 },
    { 0, 0 },
    { 0x35617e47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9217e4d, 3 },
    { 0x6c597e4e, 3 },
    { 0x44b57e4f, 4 },
    { 0, 0 },
    { 0x72f97e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c17e56, 4 },
    { 0x87e17e57, 3 },
    { 0xff857e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf57e5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977d7e60, 4 },
    { 0xd7957e61, 3 },
    { 0x3597e61, 3 },
    { 0x690d7e61, 3 },
    { 0x58757e64, 3 },
    { 0xb37d7e65, 4 },
    { 0x5a217e65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ad7e6d, 3 },
    { 0, 0 },
    { 0x99cd7e6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ad7e72, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36757e77, 5 },
    { 0xa6917e77, 3 },
    { 0xd07d7e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b197e7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386d7e81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a17e84, 3 },
    { 0xb9f97e84, 3 },
    { 0x96e17e85, 3 },
    { 0xe2557e87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2057e8b, 3 },
    { 0x63397e8c, 4 },
    { 0xf1a57e8d, 3 },
    { 0x49d7e8e, 3 },
    { 0x8f757e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec957e92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183d7e9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4817e9f, 3 },
    { 0xb4057e9f, 3 },
    { 0, 0 },
    { 0x1cb97ea2, 3 },
    { 0x35e17ea3, 3 },
    { 0, 0 },
    { 0x15a97ea5, 3 },
    { 0, 0 },
    { 0xb2797ea7, 3 },
    { 0, 0 },
    { 0x79d7ea9, 3 },
    { 0, 0 },
    { 0xe397eab, 3 },
    { 0x78b17eac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54697eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95357eb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5797eb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf17ebe, 3 },
    { 0x27657ebf, 3 },
    { 0xb42d7ebf, 3 },
    { 0x36dd7ec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9917ec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e17ec8, 3 },
    { 0x9297ec8, 3 },
    { 0x73d57ec9, 3 },
    { 0, 0 },
    { 0xfdf57ecc, 3 },
    { 0x42ed7ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f357ed4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd7ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf397ede, 4 },
    { 0x48f97ede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf17ee7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90617ef9, 4 },
    { 0x6be97ef9, 4 },
    { 0xe3557efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5857f07, 4 },
    { 0xb6497f08, 3 },
    { 0xefb97f07, 3 },
    { 0x5bf57f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb297f0d, 3 },
    { 0x44f57f0d, 3 },
    { 0x626d7f0f, 3 },
    { 0x19c17f0f, 4 },
    { 0x78297f0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce797f14, 3 },
    { 0, 0 },
    { 0xd0597f16, 4 },
    { 0x9fd17f16, 4 },
    { 0, 0 },
    { 0x1e0d7f19, 3 },
    { 0x8a597f1a, 4 },
    { 0, 0 },
    { 0xb4797f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed7f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19597f22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b97f28, 4 },
    { 0, 0 },
    { 0x854d7f2a, 3 },
    { 0, 0 },
    { 0x3e7d7f2c, 3 },
    { 0x2fb17f2d, 4 },
    { 0xbd17f2e, 4 },
    { 0xf93d7f2f, 3 },
    { 0x9c497f30, 4 },
    { 0x9f0d7f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d7f34, 3 },
    { 0x74c57f35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80817f3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c57f43, 4 },
    { 0x98397f44, 4 },
    { 0x3aed7f44, 3 },
    { 0x2f317f43, 3 },
    { 0xd9b17f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fd7f51, 3 },
    { 0x31d7f52, 3 },
    { 0xeda57f52, 4 },
    { 0x40297f52, 3 },
    { 0xc2297f55, 3 },
    { 0x39f57f51, 3 },
    { 0xabd57f57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacd7f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab157f62, 3 },
    { 0x60e97f63, 3 },
    { 0x53a57f62, 3 },
    { 0xa15d7f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc057f73, 3 },
    { 0, 0 },
    { 0xf7ed7f75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8157f78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd7f7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5157f81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70d7f87, 3 },
    { 0x8fd7f88, 3 },
    { 0xfc997f89, 3 },
    { 0x4b3d7f8a, 4 },
    { 0x71d17f8b, 3 },
    { 0x5f3d7f87, 3 },
    { 0, 0 },
    { 0x93757f8e, 3 },
    { 0, 0 },
    { 0x87717f90, 4 },
    { 0, 0 },
    { 0xe2bd7f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd17f95, 3 },
    { 0x175d7f95, 3 },
    { 0x40c17f97, 4 },
    { 0xdb117f97, 5 },
    { 0, 0 },
    { 0x90c17f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b97f9d, 4 },
    { 0x13817f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83897fa4, 3 },
    { 0x89957fa5, 3 },
    { 0x2b217fa6, 4 },
    { 0, 0 },
    { 0x64dd7fa8, 3 },
    { 0, 0 },
    { 0x2d417faa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c17faf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeead7fb6, 4 },
    { 0, 0 },
    { 0x3e517fb8, 3 },
    { 0x4a017fb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f17fbe, 3 },
    { 0x87957fbf, 3 },
    { 0, 0 },
    { 0xd42d7fc1, 3 },
    { 0x48397fc1, 3 },
    { 0, 0 },
    { 0x94f97fc4, 3 },
    { 0x2fed7fc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f57fc8, 3 },
    { 0x53717fc8, 5 },
    { 0x38597fca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5857fd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47357fd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57d7fda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19717fe1, 5 },
    { 0xeb797fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f757fe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37557fe8, 3 },
    { 0, 0 },
    { 0xe8397fea, 3 },
    { 0, 0 },
    { 0x25957fec, 4 },
    { 0, 0 },
    { 0x8de17fee, 3 },
    { 0x61157fee, 4 },
    { 0, 0 },
    { 0x557ff1, 4 },
    { 0x2f817ff2, 4 },
    { 0x18157ff3, 3 },
    { 0x7cfd7ff4, 4 },
    { 0, 0 },
    { 0x57617ff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14617ffc, 4 },
    { 0xa597ffd, 4 },
    { 0xf8397ffd, 3 },
    { 0x38457ffe, 3 },
    { 0xa7398000, 3 },
    { 0x270d8000, 3 },
    { 0xaa4d8002, 3 },
    { 0x8cf18002, 3 },
    { 0xa7957ffd, 3 },
    { 0x7c0d8005, 3 },
    { 0x57198006, 3 },
    { 0x1ec98002, 5 },
    { 0, 0 },
    { 0x7f518009, 3 },
    { 0xb3a9800a, 3 },
    { 0xb0758009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x771d800e, 3 },
    { 0, 0 },
    { 0x15898010, 3 },
    { 0x25298010, 3 },
    { 0, 0 },
    { 0xa618013, 3 },
    { 0x72998014, 3 },
    { 0x85598015, 3 },
    { 0x15218013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa365801a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd39801e, 3 },
    { 0xef69801f, 3 },
    { 0xbbd98020, 4 },
    { 0x72498021, 3 },
    { 0, 0 },
    { 0x22518023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65018026, 3 },
    { 0, 0 },
    { 0xf3cd8028, 4 },
    { 0x3d7d8029, 3 },
    { 0x1305802a, 4 },
    { 0xffd98029, 4 },
    { 0xe4598029, 3 },
    { 0xe201802d, 3 },
    { 0x8861802a, 3 },
    { 0xa85d802d, 3 },
    { 0x54a1802b, 4 },
    { 0x1919802a, 3 },
    { 0xc2b18032, 3 },
    { 0x64618032, 3 },
    { 0x67018034, 4 },
    { 0xf2a18035, 3 },
    { 0xc4e58036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cd803e, 4 },
    { 0, 0 },
    { 0xd0058040, 4 },
    { 0, 0 },
    { 0xe0ed8042, 4 },
    { 0x35418043, 3 },
    { 0x66a18044, 3 },
    { 0, 0 },
    { 0xc6298046, 3 },
    { 0x27d8047, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b1804d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90798051, 3 },
    { 0x21298052, 3 },
    { 0xae798051, 3 },
    { 0x9a058053, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe98059, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4329805c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7458062, 3 },
    { 0xb7898063, 3 },
    { 0x77458064, 3 },
    { 0, 0 },
    { 0x8f7d8066, 4 },
    { 0xa1498067, 3 },
    { 0x819d8067, 4 },
    { 0, 0 },
    { 0x1fc1806a, 3 },
    { 0, 0 },
    { 0xe835806c, 3 },
    { 0x3289806d, 3 },
    { 0, 0 },
    { 0x689806f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77558077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2171807a, 3 },
    { 0xcd21807b, 4 },
    { 0x6271807b, 3 },
    { 0, 0 },
    { 0xa8d807e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd8081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10458086, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b58089, 4 },
    { 0xcf91808a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d1808d, 3 },
    { 0, 0 },
    { 0xeb61808f, 3 },
    { 0x3055808f, 3 },
    { 0xc36d8091, 3 },
    { 0xe8d8090, 3 },
    { 0xe2b58093, 3 },
    { 0xc8798094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d1809b, 4 },
    { 0x2935809c, 3 },
    { 0x59a1809d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727d80a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7180a6, 4 },
    { 0xb0580a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663180ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d980b2, 3 },
    { 0xe5c980b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937580b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb580bb, 4 },
    { 0x24f980bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65180c7, 3 },
    { 0, 0 },
    { 0x858d80c9, 3 },
    { 0xbc9580c9, 3 },
    { 0, 0 },
    { 0x4f2180cc, 3 },
    { 0, 0 },
    { 0x552180ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55180d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9d80d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bd80d8, 4 },
    { 0x3aad80d9, 4 },
    { 0xefa980da, 3 },
    { 0x6a180db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0180df, 3 },
    { 0x2bc580df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437580e5, 4 },
    { 0x31d980e6, 3 },
    { 0x7bf980e7, 3 },
    { 0xb30180e7, 3 },
    { 0x3ffd80e9, 4 },
    { 0xf7c180ea, 5 },
    { 0xa9d80eb, 4 },
    { 0x4a1980e8, 3 },
    { 0, 0 },
    { 0x9a4980ee, 3 },
    { 0, 0 },
    { 0xfa0580f0, 4 },
    { 0x449180f1, 3 },
    { 0xf35580f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8180f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52980f9, 3 },
    { 0, 0 },
    { 0xb5a980fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe158100, 3 },
    { 0, 0 },
    { 0x91758102, 3 },
    { 0xebb18103, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa58110, 4 },
    { 0, 0 },
    { 0xfb4d8112, 3 },
    { 0x35f98112, 4 },
    { 0, 0 },
    { 0xed2d8115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e318118, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252d811e, 4 },
    { 0, 0 },
    { 0xc4fd8120, 4 },
    { 0, 0 },
    { 0x72b98122, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90798128, 3 },
    { 0, 0 },
    { 0xeadd812a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242d812f, 3 },
    { 0xea69812f, 3 },
    { 0x9f658131, 3 },
    { 0xd158132, 3 },
    { 0x8bc18132, 4 },
    { 0x82058133, 3 },
    { 0xfe458135, 4 },
    { 0x62ed8133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7d8139, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b1813d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15098140, 4 },
    { 0x30498140, 3 },
    { 0x8ca98142, 4 },
    { 0xb08d8142, 3 },
    { 0x268d8144, 3 },
    { 0x19a58145, 3 },
    { 0x6c658146, 3 },
    { 0xe0158147, 4 },
    { 0x5a7d8147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x532d814b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2d814e, 3 },
    { 0x6849814e, 4 },
    { 0x7a4d814f, 5 },
    { 0xaa118150, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac598157, 3 },
    { 0xb8418158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab01815f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x454d8162, 3 },
    { 0, 0 },
    { 0x26ad8164, 4 },
    { 0xfbfd8164, 3 },
    { 0, 0 },
    { 0xfeb18167, 3 },
    { 0, 0 },
    { 0xe9298169, 4 },
    { 0x84f18169, 3 },
    { 0xcf298169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2d8170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef18175, 3 },
    { 0xcccd8176, 3 },
    { 0x56218176, 3 },
    { 0x99258178, 3 },
    { 0, 0 },
    { 0xf3a5817a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf559817e, 3 },
    { 0x8bb9817f, 3 },
    { 0x71858180, 3 },
    { 0x2159817e, 3 },
    { 0x67158182, 4 },
    { 0xcffd8182, 3 },
    { 0xfe7d8184, 3 },
    { 0x4ee58184, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec498188, 3 },
    { 0x48dd8188, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6765818d, 3 },
    { 0x3309818e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb58191, 4 },
    { 0xe3c58192, 3 },
    { 0, 0 },
    { 0x37958194, 3 },
    { 0, 0 },
    { 0xeb458196, 3 },
    { 0x6f3d8197, 3 },
    { 0x94e98196, 3 },
    { 0x7258199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ed81a7, 3 },
    { 0x779d81a7, 3 },
    { 0x105981a7, 3 },
    { 0x3ef181aa, 3 },
    { 0, 0 },
    { 0x65cd81ac, 3 },
    { 0xc43d81ad, 3 },
    { 0x7e3581ae, 3 },
    { 0x375981af, 3 },
    { 0x707d81b0, 3 },
    { 0x60b181af, 3 },
    { 0x482981b2, 3 },
    { 0x915d81b3, 3 },
    { 0xf76181b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41581b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca981bd, 3 },
    { 0xb61181be, 4 },
    { 0x917981bd, 4 },
    { 0xc6e581bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5981c5, 3 },
    { 0xfded81c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc581ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8d81d1, 4 },
    { 0, 0 },
    { 0x43d981d3, 4 },
    { 0x3be181d4, 3 },
    { 0, 0 },
    { 0x979581d6, 3 },
    { 0x78b181d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bd81db, 4 },
    { 0, 0 },
    { 0xa0c181dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab181e1, 4 },
    { 0, 0 },
    { 0xb55981e3, 3 },
    { 0xcff181e4, 3 },
    { 0, 0 },
    { 0x55d181e6, 3 },
    { 0x9e7d81e7, 3 },
    { 0, 0 },
    { 0x9c4181e9, 4 },
    { 0, 0 },
    { 0xb41981eb, 3 },
    { 0x137181eb, 3 },
    { 0xfb8581ed, 3 },
    { 0x26f181ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d81f3, 3 },
    { 0, 0 },
    { 0x751981f5, 3 },
    { 0xb19981f6, 4 },
    { 0xd39581f5, 3 },
    { 0x1aa181f8, 3 },
    { 0xb4a581f9, 3 },
    { 0, 0 },
    { 0xdac581fb, 3 },
    { 0x117581fb, 3 },
    { 0x6b7581fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7518206, 3 },
    { 0x921d8207, 3 },
    { 0x2a9d8207, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6801820f, 3 },
    { 0x32e98210, 4 },
    { 0, 0 },
    { 0x8ad98212, 4 },
    { 0x6b198213, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46518218, 4 },
    { 0xf3058218, 3 },
    { 0x6589821a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3615821f, 4 },
    { 0x9e118220, 3 },
    { 0x71d5821f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a58224, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc518228, 3 },
    { 0x70718229, 3 },
    { 0xfb8d822a, 3 },
    { 0x9bb5822b, 4 },
    { 0, 0 },
    { 0x8325822d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3398236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c58239, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd823c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85258245, 4 },
    { 0x68098246, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb519824a, 3 },
    { 0x7d09824a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac298250, 3 },
    { 0x72658251, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab598254, 4 },
    { 0, 0 },
    { 0xd0258256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d1825c, 4 },
    { 0, 0 },
    { 0x5dd1825e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc58265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a118268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b58270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb358274, 3 },
    { 0xc7458275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b18278, 3 },
    { 0x57358279, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a1827d, 3 },
    { 0, 0 },
    { 0xbec1827f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54d8286, 3 },
    { 0x60198286, 3 },
    { 0xf4b18286, 3 },
    { 0x4f418289, 3 },
    { 0x1669828a, 3 },
    { 0, 0 },
    { 0xc4e5828c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5598290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73518294, 3 },
    { 0, 0 },
    { 0x95a18296, 4 },
    { 0x69c18296, 3 },
    { 0, 0 },
    { 0x42f58299, 3 },
    { 0xe0258299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ed82a0, 3 },
    { 0x176d82a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b182aa, 3 },
    { 0, 0 },
    { 0x2b2982ac, 3 },
    { 0, 0 },
    { 0xddf982ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71582b2, 3 },
    { 0xb58d82b3, 3 },
    { 0x353582b4, 3 },
    { 0x8d4582b3, 3 },
    { 0x248982b3, 4 },
    { 0xd1e982b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7982ba, 3 },
    { 0xebdd82bb, 3 },
    { 0x2db582bb, 3 },
    { 0x177982bd, 3 },
    { 0, 0 },
    { 0xf1b982bf, 3 },
    { 0xb64982c0, 3 },
    { 0xa46982bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14182c5, 3 },
    { 0x759582c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925982cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55582ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0582d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100982da, 4 },
    { 0xa52d82da, 3 },
    { 0x6c8982da, 3 },
    { 0x56f982db, 3 },
    { 0xf09d82de, 3 },
    { 0x8f9d82df, 3 },
    { 0x903582e0, 3 },
    { 0x605d82e1, 3 },
    { 0xdf7182e2, 3 },
    { 0xd5b982e3, 3 },
    { 0x61b182df, 3 },
    { 0x627582e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dd82e9, 3 },
    { 0, 0 },
    { 0xd23d82eb, 3 },
    { 0, 0 },
    { 0x39c582ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63182f2, 3 },
    { 0xf86582f3, 3 },
    { 0x7ff182f4, 3 },
    { 0xed2982f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec182f8, 5 },
    { 0xd91d82f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f18301, 3 },
    { 0, 0 },
    { 0x85898303, 3 },
    { 0x48058304, 4 },
    { 0x8d58304, 3 },
    { 0x99158305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b1830a, 3 },
    { 0xe549830b, 3 },
    { 0x73f5830c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f358311, 3 },
    { 0x455d8312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e058316, 3 },
    { 0xcc958316, 3 },
    { 0x535d8318, 3 },
    { 0x1a758319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd831c, 3 },
    { 0xa6f1831d, 3 },
    { 0x2a79831e, 3 },
    { 0x89b9831f, 3 },
    { 0xc87d831f, 3 },
    { 0x3bb18320, 3 },
    { 0x95f9831d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19698326, 3 },
    { 0xaf998327, 3 },
    { 0xbdf18328, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8d832f, 3 },
    { 0xfeed8330, 3 },
    { 0x19858331, 5 },
    { 0xcc4d8331, 4 },
    { 0xaded8333, 3 },
    { 0x89798333, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb58341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f01834d, 3 },
    { 0xe8e5834d, 3 },
    { 0xc53d834f, 3 },
    { 0x649d8350, 3 },
    { 0x529834f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2d8356, 3 },
    { 0, 0 },
    { 0x86498358, 3 },
    { 0xf9d58359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96598360, 3 },
    { 0, 0 },
    { 0x7a8d8362, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea298367, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cd836a, 3 },
    { 0, 0 },
    { 0x7731836c, 3 },
    { 0, 0 },
    { 0x29d1836e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab61837a, 4 },
    { 0xf19837b, 3 },
    { 0xe2b9837c, 3 },
    { 0x6f21837c, 3 },
    { 0, 0 },
    { 0xdd41837f, 4 },
    { 0xe6e9837f, 4 },
    { 0xa6ed8381, 4 },
    { 0xf498382, 3 },
    { 0xb1f58383, 3 },
    { 0xca798383, 3 },
    { 0, 0 },
    { 0xafe58386, 3 },
    { 0, 0 },
    { 0x52e18388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b9838b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7269838e, 4 },
    { 0x3559838e, 5 },
    { 0xae59838f, 4 },
    { 0x49618391, 3 },
    { 0xf4a98392, 3 },
    { 0x49b18391, 3 },
    { 0x80e98391, 4 },
    { 0xd321838e, 3 },
    { 0xab098396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7389839d, 4 },
    { 0x76d1839e, 5 },
    { 0x2f9d839e, 4 },
    { 0x3ca983a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f583a8, 4 },
    { 0, 0 },
    { 0xf3b983aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a583ad, 3 },
    { 0x417d83ae, 3 },
    { 0xb5bd83af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55583b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7183b9, 4 },
    { 0, 0 },
    { 0xdd9183bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72583c5, 4 },
    { 0x339983c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e183ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb583d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da583db, 3 },
    { 0x69c183dc, 4 },
    { 0x71d183dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42d83e0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f983e9, 3 },
    { 0x9ef983ea, 4 },
    { 0x4e7183ea, 3 },
    { 0xd3e583ec, 4 },
    { 0x9dc583eb, 3 },
    { 0xfe0983ee, 3 },
    { 0x16bd83ef, 4 },
    { 0xd67983ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559983f6, 3 },
    { 0x916183f7, 3 },
    { 0xaac183f8, 3 },
    { 0x61fd83f6, 3 },
    { 0xb6d983f8, 3 },
    { 0x893183fb, 4 },
    { 0x49d583fc, 4 },
    { 0x134d83fd, 3 },
    { 0x619183fb, 3 },
    { 0x3cad83fe, 3 },
    { 0xe818400, 3 },
    { 0x27e58400, 3 },
    { 0x27258402, 3 },
    { 0x64b98403, 5 },
    { 0xe8758404, 3 },
    { 0xf73183ff, 3 },
    { 0xa5f183f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81840a, 3 },
    { 0, 0 },
    { 0xe2e9840c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb358411, 3 },
    { 0xa858412, 3 },
    { 0x21a18412, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125841b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13d8420, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af18425, 3 },
    { 0xe7418425, 3 },
    { 0x26298427, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa939842a, 3 },
    { 0x51c9842a, 4 },
    { 0, 0 },
    { 0x6215842d, 3 },
    { 0, 0 },
    { 0xa429842f, 4 },
    { 0x6a69842f, 4 },
    { 0x84cd8430, 3 },
    { 0x6c5d8432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2199843b, 3 },
    { 0x3035843b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d218440, 4 },
    { 0xfdbd8441, 4 },
    { 0x8fb58442, 3 },
    { 0xebd18442, 3 },
    { 0xb298441, 3 },
    { 0xd6c98445, 3 },
    { 0x4b8d8446, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc1844e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb58454, 3 },
    { 0xb4258455, 3 },
    { 0xccbd8455, 3 },
    { 0xd0618457, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20d845e, 4 },
    { 0x4349845e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a18464, 3 },
    { 0x4bf58464, 3 },
    { 0x9d898464, 3 },
    { 0x788d8467, 3 },
    { 0xa1c98465, 3 },
    { 0x41ad8469, 3 },
    { 0xfc118469, 3 },
    { 0xc4618468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39846e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd8474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea9847a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3089847e, 3 },
    { 0, 0 },
    { 0x24e58480, 3 },
    { 0x39258480, 3 },
    { 0xc6e58480, 3 },
    { 0x275d8483, 3 },
    { 0x7c718480, 3 },
    { 0, 0 },
    { 0xe4398486, 4 },
    { 0, 0 },
    { 0x72a98488, 3 },
    { 0xcde98489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d848f, 4 },
    { 0, 0 },
    { 0xd9c98491, 4 },
    { 0x60a18492, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c818496, 3 },
    { 0, 0 },
    { 0x14398498, 3 },
    { 0x72858499, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c89849f, 3 },
    { 0x1301849f, 4 },
    { 0x88dd84a1, 3 },
    { 0, 0 },
    { 0x8ee184a3, 3 },
    { 0xde9184a3, 4 },
    { 0xabd84a4, 3 },
    { 0x22e184a3, 3 },
    { 0xf2e184a7, 3 },
    { 0x845984a8, 3 },
    { 0x6b2184a8, 3 },
    { 0xab6584a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x153184ad, 5 },
    { 0, 0 },
    { 0x694984af, 3 },
    { 0xb59584b0, 3 },
    { 0x7ad184af, 3 },
    { 0, 0 },
    { 0xf32584b3, 3 },
    { 0, 0 },
    { 0x36f984b5, 3 },
    { 0, 0 },
    { 0x989184b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x702584ba, 3 },
    { 0, 0 },
    { 0xd2b584bc, 3 },
    { 0, 0 },
    { 0xc3c584be, 3 },
    { 0x33b584bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2584c3, 3 },
    { 0x5b9584c3, 4 },
    { 0x5ab184c5, 3 },
    { 0x80d84c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6984c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45984cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e184d4, 3 },
    { 0x4a4184d4, 3 },
    { 0, 0 },
    { 0x2da584d7, 3 },
    { 0xf8a584d7, 3 },
    { 0x3b1984d9, 3 },
    { 0x273584da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c984e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77984e3, 3 },
    { 0xf45984e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1d84e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b984ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c984f7, 3 },
    { 0x76ed84f8, 3 },
    { 0x874984f9, 3 },
    { 0xd2d984fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89184ff, 3 },
    { 0, 0 },
    { 0xced18501, 3 },
    { 0x556d8502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e818507, 3 },
    { 0x6258508, 3 },
    { 0x9fb58509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91850c, 3 },
    { 0, 0 },
    { 0x4e85850e, 4 },
    { 0xf20d850e, 3 },
    { 0x19f98510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad9851c, 3 },
    { 0xf72d851d, 3 },
    { 0, 0 },
    { 0x7155851f, 3 },
    { 0xda3d8520, 3 },
    { 0xd2398520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef818525, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea25852c, 5 },
    { 0x1395852d, 3 },
    { 0xf021852e, 3 },
    { 0, 0 },
    { 0x8ae98530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8539, 4 },
    { 0x3349853a, 4 },
    { 0xa439853b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5998545, 3 },
    { 0x919d8546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd95854c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10898552, 3 },
    { 0, 0 },
    { 0x87e58554, 3 },
    { 0xa7058555, 3 },
    { 0x62398556, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d9855a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23158560, 3 },
    { 0xe7758561, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16858565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c698568, 3 },
    { 0x71358568, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de18573, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x17098576, 3 },
    { 0xb4e98576, 3 },
    { 0x20098578, 3 },
    { 0x70f98579, 3 },
    { 0xe79d8578, 3 },
    { 0x8998578, 4 },
    { 0, 0 },
    { 0x58a9857d, 4 },
    { 0x9b59857d, 3 },
    { 0xe24d857f, 3 },
    { 0x9bdd857e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc18587, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a5858b, 3 },
    { 0x3e79858c, 4 },
    { 0x3491858d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a98592, 4 },
    { 0xad898593, 3 },
    { 0xa858594, 3 },
    { 0x7e918592, 3 },
    { 0xf0418596, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1859b, 3 },
    { 0xe93d859c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d859f, 4 },
    { 0x455585a0, 4 },
    { 0xab185a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cd85ad, 3 },
    { 0xfd8185ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ed85b1, 3 },
    { 0xf2dd85b2, 3 },
    { 0, 0 },
    { 0x61ad85b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524585b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba985bc, 3 },
    { 0xddd985bd, 5 },
    { 0x2f8185bc, 4 },
    { 0x359585bf, 5 },
    { 0xcb3585c0, 3 },
    { 0xeab985bf, 3 },
    { 0x919185c2, 4 },
    { 0x248985c3, 4 },
    { 0xfaf985c3, 3 },
    { 0xa85185c5, 4 },
    { 0xd89185c6, 3 },
    { 0x5e5585c7, 3 },
    { 0xf77185c2, 3 },
    { 0xe12d85c9, 3 },
    { 0xbdf585c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408185ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5185d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77585d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92185da, 3 },
    { 0xc43585db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd985de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x780185e1, 4 },
    { 0x133585e1, 3 },
    { 0xa75d85e3, 3 },
    { 0xe6e585e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x883985e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d185ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf185f2, 3 },
    { 0x9fdd85f3, 3 },
    { 0xae6d85f3, 3 },
    { 0xbd3985f3, 4 },
    { 0, 0 },
    { 0x64c985f7, 4 },
    { 0, 0 },
    { 0xdd5585f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bd85ff, 3 },
    { 0, 0 },
    { 0x7ca18601, 3 },
    { 0x1e98602, 3 },
    { 0xb198601, 3 },
    { 0x2f858604, 4 },
    { 0xfdf58605, 4 },
    { 0xbf258606, 3 },
    { 0xf5298606, 3 },
    { 0x8b258604, 3 },
    { 0xc3fd8607, 3 },
    { 0xc781860a, 3 },
    { 0x9415860a, 3 },
    { 0x3d5860c, 3 },
    { 0xfb9d860c, 4 },
    { 0xbb61860d, 4 },
    { 0xce2d860b, 3 },
    { 0x8b358610, 3 },
    { 0xff29860b, 3 },
    { 0x8dc18606, 3 },
    { 0x39c1860d, 3 },
    { 0x35798614, 3 },
    { 0x4a3d8615, 3 },
    { 0xf2098616, 3 },
    { 0x8ba58617, 3 },
    { 0x6ea58607, 4 },
    { 0x7bbd8619, 3 },
    { 0x99398609, 3 },
    { 0x1ecd861b, 3 },
    { 0, 0 },
    { 0x78a5861d, 4 },
    { 0x9b51861e, 3 },
    { 0x64d5861e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32258622, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71598629, 3 },
    { 0xd66d862a, 3 },
    { 0x69fd862a, 4 },
    { 0xfaa1862a, 4 },
    { 0x20c5862d, 3 },
    { 0x746d862e, 4 },
    { 0xe179862f, 3 },
    { 0x66958630, 3 },
    { 0xd1618630, 4 },
    { 0xab958632, 3 },
    { 0xd9798633, 3 },
    { 0xb1a5862f, 3 },
    { 0, 0 },
    { 0x10c18636, 3 },
    { 0x6f18637, 3 },
    { 0x988d8638, 3 },
    { 0xa7fd8639, 4 },
    { 0xfadd8637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd1863d, 3 },
    { 0x1aed863e, 3 },
    { 0x775863d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d58646, 3 },
    { 0xb0fd8646, 4 },
    { 0xa218647, 5 },
    { 0x60318647, 3 },
    { 0xcf2d864a, 3 },
    { 0x1409864b, 3 },
    { 0xac5864a, 3 },
    { 0x769d864d, 3 },
    { 0x5b31864e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7858651, 4 },
    { 0x7ab18651, 3 },
    { 0x897d8653, 4 },
    { 0, 0 },
    { 0xccf18655, 3 },
    { 0xbaa18656, 3 },
    { 0, 0 },
    { 0x1c698658, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1181865e, 3 },
    { 0xe809865f, 3 },
    { 0xf031865f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad58666, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19358669, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29866e, 3 },
    { 0x1d45866e, 3 },
    { 0, 0 },
    { 0xa9918671, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e518675, 4 },
    { 0, 0 },
    { 0x82318677, 3 },
    { 0x7b8d8678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7889867b, 3 },
    { 0x4ba5867b, 3 },
    { 0x7d79867d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62158683, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cd868b, 3 },
    { 0xd4e5868c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91258694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8d8698, 3 },
    { 0, 0 },
    { 0xcba9869a, 3 },
    { 0x9489869b, 4 },
    { 0, 0 },
    { 0x8475869d, 3 },
    { 0xd8b1869e, 4 },
    { 0x4e35869f, 3 },
    { 0xabf986a0, 3 },
    { 0x2ccd869f, 3 },
    { 0, 0 },
    { 0x99d986a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907186af, 3 },
    { 0x9df586b0, 3 },
    { 0xf54586b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9186b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4986ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb586c7, 3 },
    { 0x422986c7, 3 },
    { 0x19b586c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aed86cc, 5 },
    { 0xe56586cd, 4 },
    { 0xda7986cc, 3 },
    { 0xb72986ce, 3 },
    { 0, 0 },
    { 0x2bed86d1, 4 },
    { 0, 0 },
    { 0x8fad86d3, 3 },
    { 0xc96586d4, 3 },
    { 0x7e2d86d5, 4 },
    { 0x822986d5, 3 },
    { 0xe30986d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9586df, 3 },
    { 0xfc986df, 4 },
    { 0x61e186e1, 3 },
    { 0, 0 },
    { 0x196d86e3, 3 },
    { 0xfe1986e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6586e7, 3 },
    { 0xe84586e7, 3 },
    { 0, 0 },
    { 0x21d186ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62986ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac986f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1186f5, 3 },
    { 0x1986f6, 3 },
    { 0, 0 },
    { 0xd6f186f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73986fe, 3 },
    { 0x1c6586fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bd8703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cd8706, 4 },
    { 0x6d298707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e09870d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d8710, 4 },
    { 0, 0 },
    { 0x9098712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0d8716, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa165871e, 3 },
    { 0x4bdd871f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63298728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcd872b, 3 },
    { 0xd891872c, 4 },
    { 0x9629872c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f298730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8d8733, 3 },
    { 0x49ed8734, 3 },
    { 0, 0 },
    { 0xa1558736, 3 },
    { 0, 0 },
    { 0x5fa18738, 4 },
    { 0xeb098739, 3 },
    { 0xacf58739, 3 },
    { 0x4205873b, 4 },
    { 0xcfd5873b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3858741, 4 },
    { 0, 0 },
    { 0x92898743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94158747, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9211874b, 4 },
    { 0xf3e5874c, 3 },
    { 0x1461874d, 4 },
    { 0x3221874c, 3 },
    { 0, 0 },
    { 0x2b358750, 3 },
    { 0xb5898750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd8754, 3 },
    { 0xf2558755, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82998758, 4 },
    { 0x76a58758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd875d, 3 },
    { 0, 0 },
    { 0x2d9d875f, 3 },
    { 0xfc898760, 3 },
    { 0xb05d8761, 3 },
    { 0xf8798761, 3 },
    { 0xb595875f, 3 },
    { 0x5ec58762, 4 },
    { 0x65058765, 3 },
    { 0x4ce58766, 3 },
    { 0x6e4d8767, 4 },
    { 0, 0 },
    { 0xaed18769, 3 },
    { 0, 0 },
    { 0xc7a5876b, 4 },
    { 0, 0 },
    { 0x871d876d, 3 },
    { 0, 0 },
    { 0x82d876f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e58774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c18778, 3 },
    { 0, 0 },
    { 0xf5e1877a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec79877e, 4 },
    { 0x895877f, 4 },
    { 0x612d8780, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f18786, 3 },
    { 0, 0 },
    { 0xcea58788, 3 },
    { 0, 0 },
    { 0xebdd878a, 3 },
    { 0x8cdd878a, 4 },
    { 0x5c91878c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0918791, 3 },
    { 0x42c18792, 3 },
    { 0, 0 },
    { 0x88958794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d98797, 3 },
    { 0, 0 },
    { 0x9258799, 4 },
    { 0xc7018799, 3 },
    { 0x700d879b, 3 },
    { 0x5ad879b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8d87a1, 4 },
    { 0x751987a2, 3 },
    { 0xc1a987a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e987a7, 3 },
    { 0x2b187a8, 3 },
    { 0xf98187a9, 4 },
    { 0xa5d187aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767587af, 3 },
    { 0, 0 },
    { 0xa3b987b1, 5 },
    { 0, 0 },
    { 0xeabd87b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95d87b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7d87ba, 3 },
    { 0xbd5d87bb, 3 },
    { 0, 0 },
    { 0x807987bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b587c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0187c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2987cd, 3 },
    { 0x6acd87ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41987d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f587e2, 3 },
    { 0xebbd87e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86187e6, 3 },
    { 0x9c2587e7, 4 },
    { 0, 0 },
    { 0x502187e9, 4 },
    { 0xaa2d87ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfd87ed, 4 },
    { 0, 0 },
    { 0xc2d987ef, 4 },
    { 0xd5d187f0, 3 },
    { 0xa61587f1, 3 },
    { 0x7b6587f1, 3 },
    { 0, 0 },
    { 0xf73987f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dd87f8, 3 },
    { 0xe00d87f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f587fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x917d87ff, 4 },
    { 0x198800, 4 },
    { 0xc2358800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a58807, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd880a, 3 },
    { 0xd3b9880b, 3 },
    { 0x670d880a, 3 },
    { 0x27fd880c, 3 },
    { 0x3dc5880a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37458812, 5 },
    { 0xa6b58813, 3 },
    { 0xa3bd8813, 3 },
    { 0x5cc58814, 3 },
    { 0x8b258816, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba58819, 4 },
    { 0, 0 },
    { 0xff1881b, 3 },
    { 0xfad881b, 3 },
    { 0, 0 },
    { 0xc1a5881e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e98822, 4 },
    { 0x723d8823, 3 },
    { 0xccd8822, 3 },
    { 0x46a98825, 3 },
    { 0xed058826, 4 },
    { 0, 0 },
    { 0x84b58828, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f9882e, 4 },
    { 0xcfcd882f, 3 },
    { 0xa858830, 3 },
    { 0xdb018830, 4 },
    { 0x7ec18832, 4 },
    { 0xbba98832, 5 },
    { 0xc3fd8834, 3 },
    { 0x7ac18835, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe229883a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3629883e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e58841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d8844, 3 },
    { 0x3b5d8845, 3 },
    { 0xc7ed8846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bd884e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c58853, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12718858, 3 },
    { 0xe9458859, 3 },
    { 0x3d498859, 3 },
    { 0xbc29885b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed8861, 4 },
    { 0, 0 },
    { 0x24798863, 4 },
    { 0x3f258864, 3 },
    { 0x1d258865, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa5886a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8998870, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d8876, 3 },
    { 0x6a158877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef1887a, 4 },
    { 0xd531887b, 3 },
    { 0x586d887a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925887f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422d8884, 3 },
    { 0x25f98885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1888a, 3 },
    { 0xf119888a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1888f, 4 },
    { 0x48e18890, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8d8893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d8897, 3 },
    { 0, 0 },
    { 0xf7d8899, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67988a1, 3 },
    { 0x1d6588a2, 3 },
    { 0, 0 },
    { 0xa84d88a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7988ab, 3 },
    { 0, 0 },
    { 0xb1b588ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86988b5, 3 },
    { 0x401188b6, 3 },
    { 0xf40988b6, 3 },
    { 0x712d88b8, 3 },
    { 0x3b2988b6, 4 },
    { 0, 0 },
    { 0x833988bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e588be, 3 },
    { 0x2b9988bf, 3 },
    { 0, 0 },
    { 0x371188c1, 3 },
    { 0x742588c2, 3 },
    { 0xa27d88c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff988c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x204d88c9, 3 },
    { 0x66588ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4188ce, 3 },
    { 0xe83988cf, 3 },
    { 0x302588d0, 3 },
    { 0, 0 },
    { 0x7b4d88d2, 4 },
    { 0x89bd88d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeed88d6, 3 },
    { 0, 0 },
    { 0xead88d8, 3 },
    { 0, 0 },
    { 0xe66988da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850988de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d88e3, 4 },
    { 0xe9188e4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6988e9, 3 },
    { 0xc5d588ea, 3 },
    { 0, 0 },
    { 0x54e588ec, 3 },
    { 0, 0 },
    { 0x313588ee, 3 },
    { 0, 0 },
    { 0x29c588f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b588f6, 3 },
    { 0xd37588f7, 3 },
    { 0x872988f7, 4 },
    { 0xb3bd88f9, 3 },
    { 0, 0 },
    { 0x490d88fb, 3 },
    { 0, 0 },
    { 0x4d0588fd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1118901, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe258906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce58909, 3 },
    { 0x555890a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3149890e, 3 },
    { 0xeda9890e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecd8918, 3 },
    { 0xc8158919, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb58920, 4 },
    { 0, 0 },
    { 0xab6d8922, 3 },
    { 0xa5298922, 4 },
    { 0, 0 },
    { 0xc3b98925, 4 },
    { 0xe8918925, 3 },
    { 0x537d8927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5892b, 3 },
    { 0x1499892c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65718935, 5 },
    { 0x83458935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a18939, 3 },
    { 0xa911893a, 3 },
    { 0x2e79893a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda19893f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c58944, 3 },
    { 0xa5b58944, 3 },
    { 0x39b58946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5d8949, 3 },
    { 0, 0 },
    { 0x8f09894b, 3 },
    { 0, 0 },
    { 0xb031894d, 3 },
    { 0x660d894e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15958951, 4 },
    { 0x16e18951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce158955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a5895b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1895f, 3 },
    { 0, 0 },
    { 0x99358961, 3 },
    { 0x347d8962, 3 },
    { 0x4a318963, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d8967, 4 },
    { 0x97a58967, 3 },
    { 0x15f58969, 3 },
    { 0x5be5896a, 3 },
    { 0x5c51896b, 3 },
    { 0, 0 },
    { 0xaee1896d, 3 },
    { 0x375896e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63458973, 4 },
    { 0, 0 },
    { 0x12d8975, 4 },
    { 0x5d558975, 4 },
    { 0x54ad8975, 3 },
    { 0x47818978, 3 },
    { 0xf9418978, 3 },
    { 0x498d897a, 3 },
    { 0xd89d897b, 3 },
    { 0x1969897c, 3 },
    { 0x93a1897d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d18980, 3 },
    { 0x9ca18980, 4 },
    { 0xfae18982, 4 },
    { 0, 0 },
    { 0x62558984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d898a, 4 },
    { 0x5809898b, 3 },
    { 0x979898c, 3 },
    { 0xd511898d, 4 },
    { 0xb139898e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c698993, 4 },
    { 0, 0 },
    { 0xe3a18995, 5 },
    { 0x128d8996, 4 },
    { 0, 0 },
    { 0xbef18998, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd899d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad89a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7189a6, 3 },
    { 0xae0189a6, 4 },
    { 0xe29189a6, 4 },
    { 0, 0 },
    { 0x234589aa, 3 },
    { 0, 0 },
    { 0xef189ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306989b2, 4 },
    { 0xb5e189b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02189b9, 3 },
    { 0, 0 },
    { 0x3d6589bb, 4 },
    { 0xd4dd89bb, 3 },
    { 0, 0 },
    { 0xb36189be, 3 },
    { 0x9fe989bf, 4 },
    { 0, 0 },
    { 0xcf7589c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d989c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ed89cd, 3 },
    { 0x44fd89cd, 3 },
    { 0xc59d89cf, 3 },
    { 0x6bc989ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07189d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680d89d7, 3 },
    { 0xd7b589d7, 3 },
    { 0x5a0189d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d89dc, 4 },
    { 0, 0 },
    { 0x66dd89de, 4 },
    { 0, 0 },
    { 0x529d89e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed989e3, 4 },
    { 0, 0 },
    { 0xf0f589e5, 3 },
    { 0, 0 },
    { 0x724189e7, 3 },
    { 0, 0 },
    { 0x84fd89e9, 4 },
    { 0x413989ea, 3 },
    { 0, 0 },
    { 0x840589ec, 3 },
    { 0xc01d89ec, 4 },
    { 0x4e6989ec, 3 },
    { 0xf54989ef, 4 },
    { 0x6d5989f0, 3 },
    { 0xaa5189ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c589f7, 3 },
    { 0, 0 },
    { 0x481d89f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d589fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55318a00, 4 },
    { 0, 0 },
    { 0x146d8a02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40018a08, 4 },
    { 0x55418a09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71318a0c, 3 },
    { 0, 0 },
    { 0xbf998a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc98a17, 4 },
    { 0x8b198a18, 3 },
    { 0x5a858a19, 3 },
    { 0xdcc58a1a, 3 },
    { 0x44598a1a, 4 },
    { 0x529d8a1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1d8a1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d18a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e158a25, 3 },
    { 0xf6a98a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4318a2b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e98a31, 4 },
    { 0x7a698a31, 3 },
    { 0, 0 },
    { 0x491d8a34, 3 },
    { 0xad718a35, 3 },
    { 0x1a358a36, 3 },
    { 0x87598a35, 3 },
    { 0x7da18a34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a58a3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5058a3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a958a44, 4 },
    { 0x19bd8a45, 3 },
    { 0, 0 },
    { 0x42a18a47, 3 },
    { 0, 0 },
    { 0x79758a49, 3 },
    { 0, 0 },
    { 0x37458a4b, 3 },
    { 0x6f6d8a4c, 3 },
    { 0, 0 },
    { 0x65cd8a4e, 3 },
    { 0, 0 },
    { 0xd6398a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d498a56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1d8a59, 4 },
    { 0, 0 },
    { 0x23298a5b, 4 },
    { 0x2c518a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dd8a60, 3 },
    { 0, 0 },
    { 0x72fd8a62, 4 },
    { 0x4f058a63, 3 },
    { 0x2af98a63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e098a69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6c, 4 },
    { 0, 0 },
    { 0xbdcd8a6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b18a73, 4 },
    { 0x8e118a73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d518a78, 3 },
    { 0xa00d8a79, 4 },
    { 0xc8018a78, 4 },
    { 0x465d8a78, 4 },
    { 0xf6098a7c, 3 },
    { 0x3df98a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696d8a82, 3 },
    { 0, 0 },
    { 0x77b18a84, 3 },
    { 0x96358a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5d8a88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d198a8e, 3 },
    { 0, 0 },
    { 0x7ff58a90, 3 },
    { 0xcae58a91, 3 },
    { 0, 0 },
    { 0x1d298a93, 4 },
    { 0x28118a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8d8a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aed8a9b, 4 },
    { 0x1f518a9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33298a9f, 4 },
    { 0, 0 },
    { 0x58c18aa1, 3 },
    { 0xb4e58aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d8aa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc98aab, 3 },
    { 0x71698aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0518ab0, 3 },
    { 0x33758ab1, 3 },
    { 0, 0 },
    { 0x100d8ab3, 4 },
    { 0xbd6d8ab4, 4 },
    { 0x64658ab3, 3 },
    { 0xb9b18ab6, 3 },
    { 0x2a198ab7, 3 },
    { 0x1a218ab8, 3 },
    { 0xbf798ab4, 3 },
    { 0xde798ab6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ed8abe, 3 },
    { 0xec858abe, 4 },
    { 0xc7e98ac0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd798ac7, 3 },
    { 0x1a6d8ac8, 3 },
    { 0xf8b18ac9, 3 },
    { 0xf4bd8ac7, 3 },
    { 0x38018ac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9018acf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0918ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ad8ad7, 3 },
    { 0x272d8ad8, 4 },
    { 0x7cd8ad8, 4 },
    { 0, 0 },
    { 0xf4858adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7758ade, 3 },
    { 0xd98ade, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb058ae3, 3 },
    { 0, 0 },
    { 0xde3d8ae5, 4 },
    { 0x6da58ae6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af58aee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0d8af1, 3 },
    { 0xee298af2, 3 },
    { 0x71b18af1, 3 },
    { 0x31118af3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd8af9, 3 },
    { 0xaf98afa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b18aff, 3 },
    { 0x2be18aff, 3 },
    { 0x99e18b01, 3 },
    { 0x866d8b01, 3 },
    { 0x27218b03, 4 },
    { 0xe4058b04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df58b07, 3 },
    { 0, 0 },
    { 0xcff18b09, 4 },
    { 0x84398b09, 3 },
    { 0x45d8b0b, 3 },
    { 0, 0 },
    { 0x41718b0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29d8b11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fd8b15, 4 },
    { 0, 0 },
    { 0xe4998b17, 4 },
    { 0xdb0d8b18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d8b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e58b1f, 4 },
    { 0x80e18b20, 3 },
    { 0x7eb58b1f, 3 },
    { 0xf1998b20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca18b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d8b32, 4 },
    { 0x37e98b33, 3 },
    { 0xb14d8b34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb298b38, 4 },
    { 0x18b98b38, 3 },
    { 0x18e98b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15958b3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36058b48, 3 },
    { 0x4458b49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d8b4c, 3 },
    { 0xfec58b4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee58b52, 3 },
    { 0x2ee58b53, 3 },
    { 0xefad8b54, 4 },
    { 0xe32d8b55, 3 },
    { 0x25358b55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e98b5b, 3 },
    { 0x2fa18b5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98b60, 4 },
    { 0xf69d8b60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd8b66, 4 },
    { 0, 0 },
    { 0x1f918b68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48d8b6c, 3 },
    { 0xd8558b6d, 3 },
    { 0x74718b6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c718b71, 4 },
    { 0xb7a58b71, 3 },
    { 0xff98b72, 3 },
    { 0x6ddd8b74, 4 },
    { 0xe5cd8b73, 4 },
    { 0x75a58b76, 3 },
    { 0x8dc18b77, 4 },
    { 0x991d8b77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75798b7b, 3 },
    { 0xc4698b7c, 3 },
    { 0xdfa98b7c, 3 },
    { 0, 0 },
    { 0x638d8b7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x776d8b82, 3 },
    { 0x51bd8b83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59098b8f, 3 },
    { 0xf7e18b90, 3 },
    { 0x3b4d8b91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cd8b94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa18b98, 3 },
    { 0x92598b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d8b9c, 3 },
    { 0x47218b9d, 3 },
    { 0x7518b9e, 3 },
    { 0, 0 },
    { 0x22ad8ba0, 3 },
    { 0xbb918ba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5358ba4, 3 },
    { 0xc5a98ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a598bab, 3 },
    { 0x123d8bac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4398bb1, 4 },
    { 0, 0 },
    { 0xb0418bb3, 3 },
    { 0x19d58bb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0798bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d18bba, 3 },
    { 0, 0 },
    { 0x5d8d8bbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88718bbf, 3 },
    { 0x4a118bc0, 3 },
    { 0xb4398bc1, 3 },
    { 0xfd658bc2, 4 },
    { 0, 0 },
    { 0x3f558bc4, 3 },
    { 0x52418bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc18bca, 3 },
    { 0xf6e58bcb, 3 },
    { 0, 0 },
    { 0x9d518bcd, 3 },
    { 0xfe198bce, 3 },
    { 0x9cfd8bcf, 3 },
    { 0, 0 },
    { 0xbf598bd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3018bd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf118be4, 4 },
    { 0x75298be4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77418be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6498bf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f858bf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0698bf9, 4 },
    { 0x20498bfa, 3 },
    { 0xde898bfb, 3 },
    { 0xc7c58bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78098c00, 3 },
    { 0x1c798c01, 4 },
    { 0xe698c00, 3 },
    { 0x89d8c03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa198c10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dd8c14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90118c17, 3 },
    { 0x53cd8c18, 3 },
    { 0, 0 },
    { 0x533d8c1a, 3 },
    { 0x9b218c1b, 4 },
    { 0xc9218c1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e798c1f, 3 },
    { 0xd4298c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b58c24, 3 },
    { 0xd6258c24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e18c2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad8c30, 4 },
    { 0x27418c31, 3 },
    { 0x22e18c32, 3 },
    { 0xb2a98c33, 3 },
    { 0xc1618c33, 3 },
    { 0x42818c31, 5 },
    { 0x25498c30, 3 },
    { 0x7cf98c30, 3 },
    { 0x7f3d8c31, 3 },
    { 0x84418c39, 3 },
    { 0x595d8c3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bd8c48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c98c4b, 3 },
    { 0, 0 },
    { 0x32918c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f558c51, 4 },
    { 0x58498c52, 3 },
    { 0xd6b58c53, 3 },
    { 0x90318c54, 3 },
    { 0, 0 },
    { 0x9d758c56, 3 },
    { 0x52698c57, 4 },
    { 0xefd18c58, 3 },
    { 0xd3818c57, 3 },
    { 0x18158c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64718c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9118c62, 3 },
    { 0x8c458c63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86118c66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf18c69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7058c6e, 3 },
    { 0x4f2d8c6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44858c74, 3 },
    { 0, 0 },
    { 0x64218c76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe058c7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd58c82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce18c87, 4 },
    { 0, 0 },
    { 0x24f18c89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae18c8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f18c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1818c98, 3 },
    { 0x70618c99, 3 },
    { 0x4b258c9a, 4 },
    { 0xfa358c99, 4 },
    { 0xd9e18c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7098ca1, 3 },
    { 0x9ce18ca2, 4 },
    { 0x14518ca2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b58ca7, 4 },
    { 0, 0 },
    { 0x93e18ca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34d8cac, 4 },
    { 0x9fa58cac, 4 },
    { 0, 0 },
    { 0x3f3d8caf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65698cb5, 3 },
    { 0x9e998cb5, 3 },
    { 0, 0 },
    { 0x3f018cb8, 3 },
    { 0, 0 },
    { 0xb7e18cba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc98cbf, 4 },
    { 0, 0 },
    { 0x26458cc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665d8cca, 3 },
    { 0, 0 },
    { 0x47658ccc, 4 },
    { 0, 0 },
    { 0xd6018cce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c98cd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a58cd7, 3 },
    { 0, 0 },
    { 0x3d118cd9, 4 },
    { 0, 0 },
    { 0x6cc58cdb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a98ce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ed8ce6, 3 },
    { 0, 0 },
    { 0xb8398ce8, 3 },
    { 0, 0 },
    { 0x7f558cea, 4 },
    { 0x970d8ceb, 3 },
    { 0x86458ceb, 3 },
    { 0x77f98cec, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d8cf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea58cf4, 3 },
    { 0x69f98cf5, 5 },
    { 0x466d8cf6, 3 },
    { 0x6d258cf7, 3 },
    { 0x8fbd8cf8, 3 },
    { 0x2f4d8cf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7d8d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ed8d04, 4 },
    { 0x65498d04, 3 },
    { 0x2458d06, 3 },
    { 0x35f58d07, 4 },
    { 0x80658d08, 3 },
    { 0x47098d09, 3 },
    { 0, 0 },
    { 0x30b58d0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47218d0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0798d13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc918d17, 4 },
    { 0x86658d18, 4 },
    { 0xf0c18d19, 3 },
    { 0x9bc18d1a, 3 },
    { 0x577d8d1b, 4 },
    { 0x816d8d17, 3 },
    { 0xf6a58d1d, 3 },
    { 0xcd58d1d, 3 },
    { 0x4c118d1e, 3 },
    { 0x15798d20, 4 },
    { 0x67f98d21, 3 },
    { 0, 0 },
    { 0x23418d23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c98d29, 3 },
    { 0, 0 },
    { 0xba218d2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21858d2e, 3 },
    { 0xa4958d2f, 3 },
    { 0xdce58d2e, 4 },
    { 0x2b718d31, 3 },
    { 0xd9e18d32, 4 },
    { 0xf3598d33, 4 },
    { 0xa4fd8d34, 5 },
    { 0xfc858d2f, 3 },
    { 0x64d98d36, 4 },
    { 0x20dd8d37, 5 },
    { 0x63958d2f, 4 },
    { 0xf7798d2f, 5 },
    { 0xbcb58d3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51018d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbded8d43, 3 },
    { 0xbcf98d43, 4 },
    { 0x5f558d45, 3 },
    { 0xabbd8d46, 4 },
    { 0x1a318d47, 3 },
    { 0x705d8d47, 3 },
    { 0xa0e98d47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x607d8d4c, 3 },
    { 0x7a158d4d, 3 },
    { 0x87cd8d4e, 3 },
    { 0xb518d4f, 3 },
    { 0, 0 },
    { 0x746d8d51, 3 },
    { 0x785d8d52, 3 },
    { 0x14f18d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d98d56, 3 },
    { 0xfe058d56, 4 },
    { 0x5d918d58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a18d5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa998d5f, 3 },
    { 0, 0 },
    { 0x7df58d61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79998d66, 3 },
    { 0x7ed98d66, 3 },
    { 0x37798d68, 3 },
    { 0x1c4d8d69, 3 },
    { 0x4658d6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad18d80, 3 },
    { 0x33298d81, 4 },
    { 0xcdad8d80, 4 },
    { 0xeec98d82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec98d86, 3 },
    { 0, 0 },
    { 0x7e158d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe18d8c, 3 },
    { 0x30dd8d8d, 3 },
    { 0x20e98d8e, 3 },
    { 0, 0 },
    { 0x1e958d90, 4 },
    { 0, 0 },
    { 0xcd198d92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a18d98, 4 },
    { 0x7ae58d99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc118d9e, 4 },
    { 0x16418d9f, 4 },
    { 0xf0618da0, 3 },
    { 0, 0 },
    { 0xa0058da2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d8da9, 3 },
    { 0, 0 },
    { 0xbcc98dab, 4 },
    { 0xc17d8dab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc18daf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb898db9, 3 },
    { 0, 0 },
    { 0xebf98dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61598dc4, 3 },
    { 0, 0 },
    { 0x6bf58dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c18dcb, 4 },
    { 0, 0 },
    { 0xb3ed8dcd, 3 },
    { 0x5b4d8dce, 4 },
    { 0xc0118dce, 3 },
    { 0, 0 },
    { 0x825d8dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x256d8dd4, 3 },
    { 0xadf58dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad58dda, 4 },
    { 0xbe418ddb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20218de2, 4 },
    { 0x2f358de3, 3 },
    { 0, 0 },
    { 0x5d898de5, 3 },
    { 0, 0 },
    { 0x6dfd8de7, 3 },
    { 0, 0 },
    { 0x4bb58de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d218dee, 5 },
    { 0x9bd18def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1058df6, 3 },
    { 0x7a698df6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6698dfe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf718e03, 3 },
    { 0xaf1d8e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba658e0b, 3 },
    { 0, 0 },
    { 0x9d918e0d, 3 },
    { 0x32cd8e0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b458e12, 4 },
    { 0xa5558e13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51598e1b, 3 },
    { 0, 0 },
    { 0xdf058e1d, 3 },
    { 0x66158e1e, 3 },
    { 0, 0 },
    { 0x11518e20, 3 },
    { 0, 0 },
    { 0x93b18e22, 4 },
    { 0xd7ad8e22, 4 },
    { 0x82b98e22, 3 },
    { 0x78158e25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd8e28, 3 },
    { 0xfbad8e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e98e31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68958e34, 3 },
    { 0xabc98e34, 4 },
    { 0xd03d8e36, 3 },
    { 0x6e298e37, 3 },
    { 0x9b658e38, 3 },
    { 0xb1398e34, 4 },
    { 0xa65d8e3a, 4 },
    { 0xcf158e3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5558e3f, 3 },
    { 0x7ee58e40, 3 },
    { 0x1cf58e41, 3 },
    { 0xf5cd8e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac418e45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73798e48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898e4c, 4 },
    { 0x41698e4c, 4 },
    { 0, 0 },
    { 0x28998e4f, 3 },
    { 0x88498e4f, 3 },
    { 0x52a18e51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63518e54, 3 },
    { 0x6fdd8e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73d8e59, 3 },
    { 0, 0 },
    { 0x547d8e5b, 4 },
    { 0x68398e5c, 3 },
    { 0, 0 },
    { 0x9ed18e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16258e61, 3 },
    { 0xd6318e61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c218e65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85758e69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd58e6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa58e70, 3 },
    { 0, 0 },
    { 0x6bdd8e72, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3d8e78, 3 },
    { 0xec118e78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dd8e7d, 4 },
    { 0xc9058e7e, 4 },
    { 0x20898e7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb98e8b, 3 },
    { 0x458e8c, 3 },
    { 0x3e618e8b, 4 },
    { 0xd6f98e8e, 5 },
    { 0xc8b98e8b, 3 },
    { 0xc4418e8e, 5 },
    { 0x12dd8e91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea418e95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de18e98, 3 },
    { 0, 0 },
    { 0xb2c18e9a, 3 },
    { 0x2bf98e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea98e9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57418ea6, 3 },
    { 0xa7f18ea7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeed8eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe498eaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9818eb5, 3 },
    { 0, 0 },
    { 0xed018eb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31958eba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92d8ebe, 3 },
    { 0x2b5d8ebf, 3 },
    { 0x35718ec0, 3 },
    { 0xb4618ec0, 3 },
    { 0x71ed8ec2, 3 },
    { 0, 0 },
    { 0xfd5d8ec4, 3 },
    { 0x91f98ec5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0898ecd, 3 },
    { 0xb798ecd, 4 },
    { 0x6c418ece, 3 },
    { 0, 0 },
    { 0x26698ed1, 3 },
    { 0x14358ed2, 3 },
    { 0x33fd8ed3, 4 },
    { 0x6eed8ed4, 3 },
    { 0xb4918ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafed8ed8, 3 },
    { 0x179d8ed9, 3 },
    { 0x41258eda, 3 },
    { 0xde498eda, 3 },
    { 0, 0 },
    { 0x91218edd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58ee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5458ee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90458ee8, 3 },
    { 0x5f098ee9, 4 },
    { 0x1fc98ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4098eed, 3 },
    { 0x93358eee, 4 },
    { 0, 0 },
    { 0x7a918ef0, 3 },
    { 0x94e18ef1, 4 },
    { 0x4c898ef2, 3 },
    { 0x11318ef1, 4 },
    { 0xed098ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac58ef9, 3 },
    { 0x45a98ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1d8f00, 3 },
    { 0x3d918f01, 3 },
    { 0xd4558f00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1398f08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb858f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be98f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8d8f14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2718f1b, 4 },
    { 0x3c598f1c, 3 },
    { 0, 0 },
    { 0x46158f1e, 3 },
    { 0xf8a98f1e, 3 },
    { 0x5bcd8f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4d8f28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd98f2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25698f32, 3 },
    { 0xb8c98f33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9d8f37, 3 },
    { 0xd5d98f38, 3 },
    { 0, 0 },
    { 0x850d8f3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44718f40, 3 },
    { 0, 0 },
    { 0x8e198f42, 3 },
    { 0, 0 },
    { 0xe82d8f44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344d8f48, 3 },
    { 0, 0 },
    { 0x5a918f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96458f4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd718f57, 3 },
    { 0x2cb18f58, 4 },
    { 0x99b58f58, 4 },
    { 0, 0 },
    { 0xe0ad8f5b, 3 },
    { 0xee9d8f5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e98f5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10118f62, 3 },
    { 0x30b58f63, 4 },
    { 0x71998f64, 3 },
    { 0, 0 },
    { 0xafd98f66, 3 },
    { 0x54d98f67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d58f6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83718f6e, 3 },
    { 0x3b398f6e, 3 },
    { 0xf7b98f70, 3 },
    { 0x9dc98f71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5918f74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ad8f7f, 4 },
    { 0x11598f7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25d8f85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1458f88, 4 },
    { 0xe9b98f88, 3 },
    { 0, 0 },
    { 0x5c4d8f8b, 3 },
    { 0x37158f8b, 3 },
    { 0x199d8f8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed8f97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da58f9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22558f9f, 3 },
    { 0xb1ed8fa0, 3 },
    { 0x7a4d8f9f, 3 },
    { 0, 0 },
    { 0xc8d18fa3, 3 },
    { 0, 0 },
    { 0x29518fa5, 4 },
    { 0x4ecd8fa6, 3 },
    { 0, 0 },
    { 0x7bd98fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d8fab, 3 },
    { 0xc92d8fac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b98fb7, 3 },
    { 0x556d8fb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f98fbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8398fc2, 3 },
    { 0xf3958fc3, 4 },
    { 0xb2898fc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1018fc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23498fce, 4 },
    { 0xd0d58fce, 4 },
    { 0xbc4d8fcf, 4 },
    { 0x14d58fd1, 3 },
    { 0x957d8fd1, 3 },
    { 0xf2798fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14d8fdc, 3 },
    { 0, 0 },
    { 0x635d8fde, 3 },
    { 0x3eb98fdf, 4 },
    { 0x5b798fde, 4 },
    { 0, 0 },
    { 0x8f398fe2, 5 },
    { 0xb6f98fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fad8fe9, 4 },
    { 0, 0 },
    { 0x2fe58feb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a658ff1, 4 },
    { 0x99a98ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27218ff5, 3 },
    { 0, 0 },
    { 0x95e18ff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589d8fff, 4 },
    { 0x91a59000, 3 },
    { 0, 0 },
    { 0x231d9002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d9005, 3 },
    { 0, 0 },
    { 0x6bb19007, 3 },
    { 0x44799007, 3 },
    { 0, 0 },
    { 0x39a5900a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649900e, 3 },
    { 0, 0 },
    { 0x5ad99010, 3 },
    { 0, 0 },
    { 0x9a719012, 3 },
    { 0, 0 },
    { 0xf38d9014, 3 },
    { 0x2fd19015, 4 },
    { 0xafc19014, 3 },
    { 0, 0 },
    { 0x939d9018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3759020, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5d9026, 3 },
    { 0xc619027, 4 },
    { 0xa7b59027, 4 },
    { 0xc2bd9026, 3 },
    { 0x6d05902a, 4 },
    { 0xd6559026, 3 },
    { 0xe281902c, 3 },
    { 0, 0 },
    { 0x84c5902e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3259031, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e119034, 3 },
    { 0xd73d9035, 3 },
    { 0x45d9034, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba59039, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb1903c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd9042, 3 },
    { 0, 0 },
    { 0xa99044, 3 },
    { 0, 0 },
    { 0x48399046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8419049, 3 },
    { 0x40519049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x452d904d, 3 },
    { 0, 0 },
    { 0x4ae1904f, 3 },
    { 0x147d904f, 3 },
    { 0x56019050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d9055, 4 },
    { 0x10d59056, 3 },
    { 0xec599057, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf599905c, 3 },
    { 0xfb91905d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d59061, 3 },
    { 0xf8399061, 4 },
    { 0, 0 },
    { 0xc59064, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a9906b, 4 },
    { 0xaca1906c, 4 },
    { 0x6a41906c, 4 },
    { 0xd0b1906e, 3 },
    { 0xbc45906e, 3 },
    { 0x4ead906e, 3 },
    { 0x8999070, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf959076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd9079, 3 },
    { 0x3821907a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad15907e, 4 },
    { 0, 0 },
    { 0xffb19080, 5 },
    { 0x9c859081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd9088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa985908f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0d9094, 3 },
    { 0xd05d9095, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1519099, 3 },
    { 0xde41909a, 3 },
    { 0x3859909b, 3 },
    { 0xf0819099, 4 },
    { 0, 0 },
    { 0xe28d909e, 3 },
    { 0x6cad909e, 3 },
    { 0xbc1d90a0, 4 },
    { 0x976d90a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68990af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x435190b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701990b7, 4 },
    { 0xbbc590b8, 3 },
    { 0, 0 },
    { 0x865d90ba, 3 },
    { 0xd32190ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d90c3, 4 },
    { 0, 0 },
    { 0x7c8d90c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x170590c8, 3 },
    { 0x581190c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x442190cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707590d0, 3 },
    { 0x449190d0, 3 },
    { 0x355990d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6990d5, 3 },
    { 0x60f990d6, 4 },
    { 0xd1c590d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebd90e1, 3 },
    { 0x5c5590e1, 3 },
    { 0, 0 },
    { 0x3e3990e4, 3 },
    { 0xe8f990e4, 3 },
    { 0x392590e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e990e9, 4 },
    { 0xa8d590ea, 4 },
    { 0x674d90eb, 3 },
    { 0xb13590ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea590ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd90f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64590f5, 3 },
    { 0, 0 },
    { 0xaab590f7, 4 },
    { 0x549590f7, 3 },
    { 0x714590f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa859100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134d9104, 3 },
    { 0x4f459105, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b59108, 3 },
    { 0xdbcd9109, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b01910d, 3 },
    { 0, 0 },
    { 0x5459910f, 3 },
    { 0xa8e99110, 3 },
    { 0, 0 },
    { 0x6cd99112, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf59117, 3 },
    { 0, 0 },
    { 0x87399119, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669911f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d259122, 3 },
    { 0, 0 },
    { 0x82e19124, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde35912b, 4 },
    { 0x6f0d912c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac59131, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb59135, 3 },
    { 0, 0 },
    { 0x94419137, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a1913c, 3 },
    { 0x29a5913c, 3 },
    { 0xa99d913e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe1914b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c299152, 3 },
    { 0x47119153, 3 },
    { 0x85319153, 3 },
    { 0xf5619155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee319159, 3 },
    { 0x7f31915a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782d9161, 3 },
    { 0x70a59161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99165, 3 },
    { 0x6d0d9165, 3 },
    { 0xc6dd9167, 3 },
    { 0, 0 },
    { 0x74c19169, 3 },
    { 0x348d916a, 4 },
    { 0xb451916b, 3 },
    { 0x39819169, 3 },
    { 0x34ad916d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e19172, 3 },
    { 0xcfdd9173, 3 },
    { 0x16399173, 3 },
    { 0xbf759172, 4 },
    { 0xb7119173, 3 },
    { 0xa7f99177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bd9180, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb19183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8d918e, 3 },
    { 0, 0 },
    { 0xd2899190, 3 },
    { 0x3e8d9190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ad919c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715191a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15d91a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2991a9, 3 },
    { 0xe89d91aa, 3 },
    { 0x8e4591aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238591b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86591b4, 4 },
    { 0x5fe591b5, 3 },
    { 0x157d91b6, 3 },
    { 0, 0 },
    { 0x406991b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x203591bb, 3 },
    { 0x9ee591bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d91c5, 3 },
    { 0, 0 },
    { 0xeae591c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72d91cb, 3 },
    { 0xffd91cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1191d2, 3 },
    { 0, 0 },
    { 0xd59191d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1991da, 4 },
    { 0x3fa191db, 3 },
    { 0x5a3591dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x835191df, 3 },
    { 0xa9f591df, 3 },
    { 0xb03991df, 3 },
    { 0x469591e2, 3 },
    { 0xa30d91e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b591f1, 3 },
    { 0x8d2d91f2, 3 },
    { 0xf1b591f3, 4 },
    { 0, 0 },
    { 0x30d91f5, 3 },
    { 0x3e2991f6, 4 },
    { 0, 0 },
    { 0x325191f8, 3 },
    { 0x86cd91f9, 4 },
    { 0x713191fa, 4 },
    { 0xe78591fb, 3 },
    { 0xdca591fc, 3 },
    { 0xf2fd91fd, 3 },
    { 0x1f9191fb, 3 },
    { 0x989191fd, 3 },
    { 0x4c5591fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b99207, 3 },
    { 0, 0 },
    { 0xa3119209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071920d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18d9210, 3 },
    { 0, 0 },
    { 0xf5299212, 4 },
    { 0, 0 },
    { 0x38999214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce59217, 3 },
    { 0x1c219218, 3 },
    { 0x2b899219, 3 },
    { 0, 0 },
    { 0xf7b5921b, 3 },
    { 0x5fb5921b, 3 },
    { 0xd17d921d, 3 },
    { 0, 0 },
    { 0xf4ed921f, 3 },
    { 0x3f75921f, 3 },
    { 0x9ba5921f, 5 },
    { 0x71659222, 3 },
    { 0, 0 },
    { 0x2759224, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb19227, 3 },
    { 0xa7419228, 4 },
    { 0x52259229, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d59922c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d59230, 3 },
    { 0xa13d9231, 3 },
    { 0xeb99231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403d9239, 3 },
    { 0, 0 },
    { 0xd1d5923b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5923e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d699243, 3 },
    { 0x20dd9244, 3 },
    { 0xf3ad9245, 4 },
    { 0x74299245, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf71924a, 3 },
    { 0xc915924b, 3 },
    { 0xdc11924c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5889924f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf059257, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5925a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f499263, 4 },
    { 0, 0 },
    { 0x12bd9265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d9269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe355926f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec59276, 3 },
    { 0xb03d9276, 3 },
    { 0, 0 },
    { 0x8f459279, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc111927c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ad9280, 3 },
    { 0, 0 },
    { 0x31419282, 3 },
    { 0x40919282, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d9288, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429928b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ed9293, 3 },
    { 0, 0 },
    { 0xf8f59295, 3 },
    { 0, 0 },
    { 0xcf4d9297, 3 },
    { 0x56859298, 3 },
    { 0, 0 },
    { 0x692d929a, 3 },
    { 0xb5e1929b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe601929e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d92a7, 4 },
    { 0xd76d92a7, 3 },
    { 0xf80d92a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f192ac, 4 },
    { 0xed5192ac, 4 },
    { 0, 0 },
    { 0x2e7992af, 3 },
    { 0, 0 },
    { 0x15d592b1, 3 },
    { 0x163592b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6192b6, 4 },
    { 0, 0 },
    { 0x80c192b8, 3 },
    { 0, 0 },
    { 0xe98592ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd92c0, 3 },
    { 0x5d3d92c0, 3 },
    { 0x833192c2, 3 },
    { 0xa8992c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dd92c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34992cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x841592ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1d92d4, 3 },
    { 0xce3992d5, 3 },
    { 0x9fdd92d6, 3 },
    { 0xabf592d6, 3 },
    { 0, 0 },
    { 0xe3f592d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb592dc, 3 },
    { 0, 0 },
    { 0xc93992de, 3 },
    { 0, 0 },
    { 0x727992e0, 4 },
    { 0xd0992e1, 3 },
    { 0x7ee592e2, 3 },
    { 0, 0 },
    { 0xdd9192e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da192e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cad92ef, 3 },
    { 0, 0 },
    { 0xd42592f1, 4 },
    { 0, 0 },
    { 0x3d6992f3, 4 },
    { 0xb70992f3, 3 },
    { 0x8ccd92f3, 3 },
    { 0, 0 },
    { 0x90d92f7, 3 },
    { 0, 0 },
    { 0xf4e592f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63592fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9919304, 4 },
    { 0, 0 },
    { 0xdf3d9306, 4 },
    { 0x50499307, 3 },
    { 0, 0 },
    { 0x7459309, 3 },
    { 0, 0 },
    { 0x4d05930b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f15930f, 5 },
    { 0x3641930f, 3 },
    { 0x23519311, 4 },
    { 0xc4359311, 4 },
    { 0x3b59311, 3 },
    { 0x8a3d9311, 4 },
    { 0xf3a99310, 3 },
    { 0x53a19316, 3 },
    { 0x47119317, 4 },
    { 0xb42d9318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf9931e, 3 },
    { 0x6cd5931e, 4 },
    { 0, 0 },
    { 0x6d7d9321, 3 },
    { 0xd3299322, 3 },
    { 0x37ad9321, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9d9328, 3 },
    { 0xe7e59329, 3 },
    { 0x71759329, 3 },
    { 0xcb919328, 3 },
    { 0xd9dd932c, 4 },
    { 0xb9b19329, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef059339, 3 },
    { 0xd5d933a, 3 },
    { 0, 0 },
    { 0x6f95933c, 3 },
    { 0xb1cd933c, 4 },
    { 0x7381933e, 3 },
    { 0, 0 },
    { 0xb9059340, 3 },
    { 0xab799340, 3 },
    { 0x3e7d9342, 4 },
    { 0, 0 },
    { 0x133d9344, 3 },
    { 0, 0 },
    { 0x84c99346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759934a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822d9351, 3 },
    { 0x33499352, 3 },
    { 0x30ed9351, 3 },
    { 0xe16d9354, 4 },
    { 0xc6ad9354, 4 },
    { 0x51d59356, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde85935a, 5 },
    { 0, 0 },
    { 0x1a59935c, 3 },
    { 0xa27d935d, 4 },
    { 0x7299935d, 3 },
    { 0x7569935c, 3 },
    { 0xc2559360, 3 },
    { 0xdff1935f, 4 },
    { 0, 0 },
    { 0xfe919363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd99366, 3 },
    { 0, 0 },
    { 0x95e99368, 4 },
    { 0x497d9368, 3 },
    { 0xe2059368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x178d936d, 3 },
    { 0, 0 },
    { 0x3d4d936f, 4 },
    { 0x96e59370, 3 },
    { 0xeb9d9371, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb19378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cd937f, 3 },
    { 0x58ed937f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde559385, 3 },
    { 0x59559386, 5 },
    { 0xca859387, 3 },
    { 0xd8619387, 4 },
    { 0x57b99386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc545938c, 3 },
    { 0, 0 },
    { 0xe765938e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9d9392, 3 },
    { 0x56919393, 3 },
    { 0, 0 },
    { 0xba859395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5099399, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5051939d, 4 },
    { 0, 0 },
    { 0x7a21939f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d193a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf593a9, 3 },
    { 0x655593aa, 3 },
    { 0, 0 },
    { 0x186d93ac, 3 },
    { 0, 0 },
    { 0x27b193ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d93b1, 3 },
    { 0x6bc193b2, 4 },
    { 0x140593b3, 4 },
    { 0xf98593b3, 3 },
    { 0x644993b5, 3 },
    { 0xc6a993b5, 4 },
    { 0xb78993b6, 3 },
    { 0, 0 },
    { 0x288593b9, 3 },
    { 0, 0 },
    { 0xdffd93bb, 3 },
    { 0xd03193bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568993c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x353993cb, 3 },
    { 0x5ff593cc, 3 },
    { 0x4a9193cd, 3 },
    { 0xab4d93cb, 4 },
    { 0xfd5d93cf, 3 },
    { 0x609d93d0, 4 },
    { 0x9e5593d1, 3 },
    { 0xd6e193d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffd93d5, 3 },
    { 0xe9d593d6, 3 },
    { 0x318593d5, 3 },
    { 0x8bd593d5, 3 },
    { 0xe2fd93d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91993dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad593e1, 3 },
    { 0x9eb193e2, 3 },
    { 0xba8d93e3, 4 },
    { 0xe28193e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a193e8, 4 },
    { 0x341193e9, 4 },
    { 0, 0 },
    { 0xad7593eb, 3 },
    { 0, 0 },
    { 0x96b193ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483193f1, 3 },
    { 0x8c9993f1, 3 },
    { 0x4c1193f1, 3 },
    { 0, 0 },
    { 0x22a993f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc993fa, 4 },
    { 0, 0 },
    { 0xfae993fc, 3 },
    { 0, 0 },
    { 0xbfa593fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb599405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9099408, 4 },
    { 0x19919409, 3 },
    { 0xa5c9940a, 3 },
    { 0x20ad940b, 3 },
    { 0xcd81940b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c19414, 3 },
    { 0x1299414, 3 },
    { 0x17cd9415, 3 },
    { 0x1afd9414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce1941b, 3 },
    { 0xbd09941c, 3 },
    { 0x1429941c, 4 },
    { 0, 0 },
    { 0xcb3d941f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdd9422, 5 },
    { 0x47559422, 4 },
    { 0x942d9422, 3 },
    { 0, 0 },
    { 0x709d9426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ed9429, 4 },
    { 0, 0 },
    { 0xa061942b, 3 },
    { 0x4875942c, 3 },
    { 0xb5cd942d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd699431, 3 },
    { 0xa4699431, 4 },
    { 0x46ad9433, 4 },
    { 0x22619434, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd9943a, 3 },
    { 0x21e5943b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71019441, 4 },
    { 0x66619442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938d9447, 4 },
    { 0xebfd9447, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1725944b, 3 },
    { 0xea81944c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd9452, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf229945a, 3 },
    { 0, 0 },
    { 0x158d945c, 4 },
    { 0x7731945c, 3 },
    { 0, 0 },
    { 0x43b5945f, 4 },
    { 0x650d945f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fd9466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94d946c, 3 },
    { 0, 0 },
    { 0xfdf9946e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819d9472, 3 },
    { 0x46b19473, 3 },
    { 0xd2d99474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c5947d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39d9480, 3 },
    { 0x70819480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e19489, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a21948d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26299490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a759493, 3 },
    { 0x3e6d9494, 4 },
    { 0x9c199495, 4 },
    { 0x253d9495, 3 },
    { 0xce5d9497, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a11949b, 3 },
    { 0x3a1d949c, 3 },
    { 0xfb59949c, 3 },
    { 0x5691949e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c194a3, 3 },
    { 0x369d94a3, 3 },
    { 0, 0 },
    { 0xaa2994a6, 3 },
    { 0xd34594a6, 3 },
    { 0, 0 },
    { 0xdd7d94a9, 3 },
    { 0, 0 },
    { 0xa7d994ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd94b9, 3 },
    { 0xbda994ba, 3 },
    { 0, 0 },
    { 0xdfcd94bc, 3 },
    { 0x388994bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947594c3, 4 },
    { 0xfce994c3, 3 },
    { 0x8cb994c3, 3 },
    { 0x92c594c6, 3 },
    { 0xd2e194c7, 3 },
    { 0xaa3994c5, 3 },
    { 0x4b5194c7, 4 },
    { 0x41d994ca, 4 },
    { 0, 0 },
    { 0x6a9d94cc, 4 },
    { 0x1b0194cd, 3 },
    { 0xee8194ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdced94d1, 3 },
    { 0, 0 },
    { 0xb7994d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d994d6, 5 },
    { 0x9f6594d7, 4 },
    { 0, 0 },
    { 0xf0b594d9, 3 },
    { 0xee0d94d9, 4 },
    { 0, 0 },
    { 0xdf5194dc, 3 },
    { 0x3c7594dd, 3 },
    { 0x571594de, 3 },
    { 0x242594df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f994e2, 3 },
    { 0x229594e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83d94e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7994f0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970d94f4, 3 },
    { 0x642994f4, 3 },
    { 0, 0 },
    { 0xdadd94f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5994fa, 4 },
    { 0x1bd194fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e019503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5d9506, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab699509, 3 },
    { 0x9f41950a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf059517, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa011951e, 4 },
    { 0, 0 },
    { 0x2d3d9520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c319523, 3 },
    { 0x5de99524, 3 },
    { 0, 0 },
    { 0x5759526, 3 },
    { 0xc6b99527, 3 },
    { 0x948d9528, 3 },
    { 0, 0 },
    { 0x9f01952a, 3 },
    { 0xcfe1952a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3d9534, 4 },
    { 0, 0 },
    { 0xcb59536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa849953e, 3 },
    { 0x6455953e, 5 },
    { 0xb4d99540, 3 },
    { 0x995d9541, 3 },
    { 0x22459542, 4 },
    { 0, 0 },
    { 0x2dc99544, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace9954a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc45954f, 3 },
    { 0xefad9550, 4 },
    { 0, 0 },
    { 0x72119552, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da59556, 3 },
    { 0xfb119557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f9955b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d955f, 3 },
    { 0, 0 },
    { 0xdadd9561, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8599565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b419568, 4 },
    { 0xdb259569, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5956e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87d9571, 3 },
    { 0xb9f19572, 4 },
    { 0x6c19572, 3 },
    { 0x27ed9574, 4 },
    { 0xe7d59571, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c81957b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45957f, 3 },
    { 0x6cd957f, 4 },
    { 0xc6fd9580, 3 },
    { 0x8d7d9582, 3 },
    { 0x60bd9582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c61958d, 3 },
    { 0x4c09958e, 3 },
    { 0xef09958f, 3 },
    { 0xc8459590, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db19594, 3 },
    { 0, 0 },
    { 0x36c99596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7135959a, 3 },
    { 0, 0 },
    { 0xb44d959c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68995a3, 3 },
    { 0x544195a3, 3 },
    { 0, 0 },
    { 0x636d95a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce595ab, 3 },
    { 0, 0 },
    { 0xec6195ad, 3 },
    { 0x528195ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b995b3, 4 },
    { 0x166d95b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d595bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353995c2, 4 },
    { 0x653995c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d95c6, 3 },
    { 0x34c195c6, 3 },
    { 0xca3595c8, 4 },
    { 0xd11995c9, 3 },
    { 0xe11195c8, 4 },
    { 0x367195c9, 3 },
    { 0xc00995cc, 3 },
    { 0x2efd95c7, 3 },
    { 0x400995ce, 3 },
    { 0xdca595cf, 3 },
    { 0x2e2d95d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x574595d3, 3 },
    { 0xf29995d3, 3 },
    { 0xeaf195d5, 4 },
    { 0x5aa595d3, 3 },
    { 0x6af195d7, 3 },
    { 0xc4ad95d5, 4 },
    { 0x9c5995d9, 3 },
    { 0xb67595d9, 5 },
    { 0x7f6d95db, 3 },
    { 0xff3595d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276195e2, 3 },
    { 0, 0 },
    { 0xb68195e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43195e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10195ea, 3 },
    { 0x92a195eb, 4 },
    { 0x1b4995ec, 3 },
    { 0xd62195eb, 4 },
    { 0, 0 },
    { 0xb44595ef, 4 },
    { 0x96c195f0, 4 },
    { 0x4f5d95f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x453595f4, 4 },
    { 0xeffd95f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6d9602, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c19608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cd960b, 3 },
    { 0, 0 },
    { 0x569960d, 3 },
    { 0x7e65960d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5459612, 4 },
    { 0x5bf59613, 3 },
    { 0, 0 },
    { 0xed1d9615, 3 },
    { 0, 0 },
    { 0xc1219617, 3 },
    { 0, 0 },
    { 0x14ad9619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b79961d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4919624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b09962b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14659630, 3 },
    { 0x8d59631, 3 },
    { 0, 0 },
    { 0x36819633, 3 },
    { 0x7b899634, 3 },
    { 0, 0 },
    { 0x133d9636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3619639, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f5963d, 3 },
    { 0, 0 },
    { 0x20c1963f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b199645, 3 },
    { 0x5ded9646, 3 },
    { 0xaec19646, 4 },
    { 0, 0 },
    { 0x83e19649, 3 },
    { 0x601964a, 3 },
    { 0xdddd9649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20219655, 3 },
    { 0x2d719656, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x650d9659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a59965c, 4 },
    { 0, 0 },
    { 0x4005965e, 3 },
    { 0, 0 },
    { 0x3d699660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c19664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50619667, 3 },
    { 0x29919667, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e1966d, 4 },
    { 0xdfa9966d, 4 },
    { 0x8fbd966f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16719672, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a399677, 5 },
    { 0, 0 },
    { 0x2a459679, 3 },
    { 0x971967a, 3 },
    { 0xac71967a, 3 },
    { 0x3c45967b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8565967f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1159684, 3 },
    { 0, 0 },
    { 0x90699686, 3 },
    { 0x1db19687, 4 },
    { 0xc4759688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd89968f, 3 },
    { 0x3019968f, 4 },
    { 0x92e99691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8d9694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f19697, 3 },
    { 0, 0 },
    { 0x40819699, 4 },
    { 0x9a699699, 3 },
    { 0x2fa59699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a596a3, 4 },
    { 0x476d96a4, 5 },
    { 0, 0 },
    { 0x58cd96a6, 3 },
    { 0, 0 },
    { 0xea5d96a8, 3 },
    { 0, 0 },
    { 0xc8f596aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae596b0, 3 },
    { 0x9c2196b0, 4 },
    { 0xf26596b1, 3 },
    { 0xfad996b3, 3 },
    { 0x471596b4, 3 },
    { 0x668596b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1d96b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbded96bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x898596c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dd96c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4996c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9196ca, 4 },
    { 0xab9d96cb, 3 },
    { 0xf32d96cb, 3 },
    { 0x45c596cd, 3 },
    { 0x252596ce, 3 },
    { 0, 0 },
    { 0xa94d96d0, 3 },
    { 0, 0 },
    { 0x787596d2, 4 },
    { 0x4f6996d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de996d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf596dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbd96e5, 4 },
    { 0x399996e5, 3 },
    { 0, 0 },
    { 0xb22996e8, 3 },
    { 0x81d596e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603996f5, 3 },
    { 0, 0 },
    { 0xe51996f7, 3 },
    { 0x62e596f8, 4 },
    { 0xe75d96f7, 3 },
    { 0xb86596f8, 4 },
    { 0x17196fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa59707, 3 },
    { 0x6d059708, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299970f, 3 },
    { 0xf4e19710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05d9713, 3 },
    { 0, 0 },
    { 0x748d9715, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d719718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821971d, 4 },
    { 0xe209971e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f799722, 4 },
    { 0, 0 },
    { 0xcc899724, 3 },
    { 0xd6c99725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4541972a, 3 },
    { 0x43d5972a, 4 },
    { 0xc6e1972c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e59735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65d9739, 3 },
    { 0, 0 },
    { 0x41bd973b, 3 },
    { 0x2c6d973c, 3 },
    { 0x473d973d, 3 },
    { 0x5aed973e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacd9744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681974b, 4 },
    { 0xa201974b, 3 },
    { 0xaa8d974c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19752, 4 },
    { 0x65f19753, 3 },
    { 0xd2599754, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae299758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a71975b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71975f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219763, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f1976d, 3 },
    { 0x51a9976e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39819773, 3 },
    { 0x25b19773, 4 },
    { 0xd4d19774, 3 },
    { 0xb7099776, 3 },
    { 0x3d159777, 3 },
    { 0x44819778, 4 },
    { 0x57b99777, 3 },
    { 0x2525977a, 3 },
    { 0, 0 },
    { 0x3345977c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1d9780, 4 },
    { 0xd2e59781, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8599789, 3 },
    { 0x9d119789, 3 },
    { 0xd6a5978a, 3 },
    { 0x7f95978c, 3 },
    { 0xf1f5978d, 3 },
    { 0, 0 },
    { 0xbab1978f, 3 },
    { 0xe531978f, 3 },
    { 0xb6a99790, 4 },
    { 0x1a419792, 5 },
    { 0, 0 },
    { 0x52019794, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3519798, 3 },
    { 0xe1319799, 3 },
    { 0, 0 },
    { 0xec3d979b, 3 },
    { 0xf63d979c, 3 },
    { 0x5695979c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397597a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfd97a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8197a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb997b1, 3 },
    { 0, 0 },
    { 0x450597b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214d97b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30d97bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d97bf, 5 },
    { 0x8a2197c0, 3 },
    { 0xffa197c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58997c9, 4 },
    { 0, 0 },
    { 0x421d97cb, 3 },
    { 0x49b997cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8d97cf, 4 },
    { 0xa35997d0, 3 },
    { 0, 0 },
    { 0x611d97d2, 4 },
    { 0x915d97d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33997d7, 4 },
    { 0x8f2d97d8, 3 },
    { 0x439197d9, 3 },
    { 0, 0 },
    { 0x971597db, 3 },
    { 0, 0 },
    { 0xded97dd, 3 },
    { 0x4e5d97de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0997e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb197e4, 3 },
    { 0x6a2597e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114997e9, 3 },
    { 0x140197e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce997f4, 3 },
    { 0, 0 },
    { 0xe1d197f6, 4 },
    { 0xfa7197f7, 3 },
    { 0x459997f7, 4 },
    { 0xa78597f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68597fc, 3 },
    { 0, 0 },
    { 0xaa7d97fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab59801, 3 },
    { 0, 0 },
    { 0xf99803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e559806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed59809, 3 },
    { 0, 0 },
    { 0x7b29980b, 4 },
    { 0, 0 },
    { 0x6349980d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1619811, 3 },
    { 0, 0 },
    { 0x78ed9813, 3 },
    { 0x6c259814, 3 },
    { 0xc4b99815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe1981c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8d981f, 4 },
    { 0x9b7d9820, 4 },
    { 0x25919821, 3 },
    { 0x6b759820, 3 },
    { 0x6a899823, 4 },
    { 0x30cd981f, 3 },
    { 0, 0 },
    { 0xe0299826, 4 },
    { 0x75b99827, 3 },
    { 0x51719828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d5982c, 3 },
    { 0x19fd982c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d9830, 3 },
    { 0x9cf99830, 4 },
    { 0x5fad9832, 4 },
    { 0xafad9831, 4 },
    { 0x25d99834, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb99838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e99840, 3 },
    { 0, 0 },
    { 0xe6299842, 3 },
    { 0, 0 },
    { 0xe3b99844, 3 },
    { 0, 0 },
    { 0x29019846, 4 },
    { 0xe8e59846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba1984e, 3 },
    { 0xd8b1984f, 4 },
    { 0, 0 },
    { 0xe40d9851, 3 },
    { 0x8f319851, 3 },
    { 0, 0 },
    { 0xb3ad9854, 3 },
    { 0x60919855, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbd985a, 3 },
    { 0xe93d985b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd119860, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d19865, 3 },
    { 0xdc99866, 3 },
    { 0xad059867, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b5986a, 3 },
    { 0, 0 },
    { 0xebd1986c, 4 },
    { 0xb1d9986d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72319872, 5 },
    { 0x613d9873, 3 },
    { 0x4bbd9873, 4 },
    { 0x97ed9874, 3 },
    { 0x4699876, 3 },
    { 0, 0 },
    { 0x13199878, 3 },
    { 0xe0899879, 3 },
    { 0xc3a19878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee59880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f59883, 3 },
    { 0x46159883, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d99887, 3 },
    { 0, 0 },
    { 0x13399889, 3 },
    { 0xe015988a, 3 },
    { 0, 0 },
    { 0x61fd988c, 3 },
    { 0x815988c, 3 },
    { 0, 0 },
    { 0xd2e9988f, 3 },
    { 0x50bd988f, 3 },
    { 0xfa359891, 3 },
    { 0x34cd9890, 3 },
    { 0, 0 },
    { 0xc5219894, 4 },
    { 0xf4459895, 5 },
    { 0x35c99895, 3 },
    { 0x1a59897, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d989d, 3 },
    { 0xcc71989d, 4 },
    { 0x8b8d989f, 4 },
    { 0x7e6598a0, 3 },
    { 0xe59989f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x570198a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15198a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4d98aa, 3 },
    { 0, 0 },
    { 0xfe3198ac, 3 },
    { 0x2a4598ad, 3 },
    { 0x5ee198ac, 3 },
    { 0, 0 },
    { 0xd9598b0, 3 },
    { 0xa6f598b0, 3 },
    { 0xe77998b2, 3 },
    { 0xc07998b1, 4 },
    { 0xda0598b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67598b7, 3 },
    { 0xafcd98b7, 4 },
    { 0, 0 },
    { 0xb48198ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f198be, 3 },
    { 0x72dd98bf, 3 },
    { 0, 0 },
    { 0x416598c1, 3 },
    { 0x1c6998c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75598ca, 3 },
    { 0, 0 },
    { 0x96e198cc, 3 },
    { 0xfa1998cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c598d2, 4 },
    { 0, 0 },
    { 0x6f3598d4, 4 },
    { 0, 0 },
    { 0xc93d98d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217d98da, 3 },
    { 0x284198da, 4 },
    { 0, 0 },
    { 0x389598dd, 3 },
    { 0, 0 },
    { 0x671598df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27998e4, 3 },
    { 0xff4598e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05998eb, 4 },
    { 0x4e998ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2198fb, 3 },
    { 0xeb3598fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1959905, 3 },
    { 0x417d9906, 4 },
    { 0x3bad9907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125990c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfd990f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c559915, 3 },
    { 0x9cd9916, 3 },
    { 0xc2119916, 3 },
    { 0, 0 },
    { 0x3e59919, 3 },
    { 0, 0 },
    { 0x68cd991b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec159920, 4 },
    { 0x49ed9920, 3 },
    { 0x42859920, 3 },
    { 0, 0 },
    { 0x84459924, 4 },
    { 0xb2f99925, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda99929, 3 },
    { 0, 0 },
    { 0xf6e9992b, 3 },
    { 0x181992b, 3 },
    { 0xc09992d, 3 },
    { 0x10e5992c, 4 },
    { 0, 0 },
    { 0x2c259930, 4 },
    { 0x5899930, 3 },
    { 0x7ecd9931, 3 },
    { 0xa4a59931, 3 },
    { 0, 0 },
    { 0xb08d9935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18659939, 3 },
    { 0x499d993a, 3 },
    { 0xdef5993a, 4 },
    { 0xf931993b, 3 },
    { 0xee95993b, 5 },
    { 0x5a519939, 3 },
    { 0xd689993a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0499943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83959947, 3 },
    { 0, 0 },
    { 0x95b99949, 3 },
    { 0xa855994a, 3 },
    { 0x41b1994a, 3 },
    { 0x39d1994a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a19952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3d995c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca99964, 3 },
    { 0xb7599964, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c99968, 3 },
    { 0, 0 },
    { 0x5cd9996a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed996f, 4 },
    { 0, 0 },
    { 0x26f99971, 3 },
    { 0, 0 },
    { 0x13319973, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11d9977, 3 },
    { 0, 0 },
    { 0xcb9d9979, 3 },
    { 0xa759979, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6181997f, 3 },
    { 0xb2599980, 4 },
    { 0xe5b99980, 4 },
    { 0x6d4d997f, 3 },
    { 0xfcc19983, 3 },
    { 0, 0 },
    { 0x7b459985, 3 },
    { 0x29799985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c1998a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5865998f, 3 },
    { 0x9bb59990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94699994, 3 },
    { 0x59d59995, 4 },
    { 0x71319996, 3 },
    { 0x84a99996, 3 },
    { 0x97319994, 3 },
    { 0xb8e19999, 3 },
    { 0x12b59999, 3 },
    { 0xb819999a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454199a3, 3 },
    { 0x550d99a3, 3 },
    { 0, 0 },
    { 0x108d99a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7599a9, 3 },
    { 0, 0 },
    { 0x421999ab, 3 },
    { 0, 0 },
    { 0x678199ad, 3 },
    { 0xf6fd99ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b599b7, 5 },
    { 0x763599b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73d99bd, 4 },
    { 0xf56599be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9599c1, 3 },
    { 0xf8bd99c2, 3 },
    { 0x15cd99c3, 3 },
    { 0x1a2d99c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72999c7, 3 },
    { 0, 0 },
    { 0x6a8d99c9, 3 },
    { 0x6c6d99ca, 4 },
    { 0x9e3199cb, 3 },
    { 0xf0999ca, 3 },
    { 0xe86199cd, 3 },
    { 0x37dd99ce, 3 },
    { 0x19999cf, 3 },
    { 0xd3b199d0, 3 },
    { 0xe2b999d0, 3 },
    { 0xa59d99d2, 4 },
    { 0x688999c9, 5 },
    { 0x824d99d4, 4 },
    { 0x29999d5, 3 },
    { 0x3ac599d5, 3 },
    { 0xfcb199d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x899599da, 4 },
    { 0xfd8599db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac999e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d199e7, 3 },
    { 0xf35999e7, 4 },
    { 0x1c1599e8, 3 },
    { 0xbd8d99ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cd99ef, 4 },
    { 0, 0 },
    { 0x773199f1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e999f4, 5 },
    { 0, 0 },
    { 0x9d1199f6, 3 },
    { 0x134199f7, 3 },
    { 0x8c5999f6, 4 },
    { 0x7df199f6, 4 },
    { 0, 0 },
    { 0xe4a599fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1599fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b419a01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7919a06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6519a0a, 4 },
    { 0x97899a0a, 3 },
    { 0x4c619a0c, 3 },
    { 0x518d9a0d, 3 },
    { 0, 0 },
    { 0x36fd9a0f, 4 },
    { 0, 0 },
    { 0xf0819a11, 4 },
    { 0xe7ed9a12, 3 },
    { 0xe7019a12, 4 },
    { 0x29d59a14, 4 },
    { 0x4d399a15, 3 },
    { 0x98a19a11, 3 },
    { 0xd5b59a12, 3 },
    { 0xc3599a18, 3 },
    { 0x6e99a18, 4 },
    { 0x40699a18, 3 },
    { 0, 0 },
    { 0x9ea59a1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c59a20, 3 },
    { 0xa75d9a21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72919a26, 3 },
    { 0x5f699a26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dd9a2c, 4 },
    { 0x78bd9a2c, 3 },
    { 0xff6d9a2c, 3 },
    { 0x494d9a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c19a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06d9a37, 3 },
    { 0x31a99a38, 3 },
    { 0xb8cd9a39, 3 },
    { 0x6c399a39, 4 },
    { 0x16259a3b, 3 },
    { 0x8b499a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3919a41, 3 },
    { 0, 0 },
    { 0x488d9a43, 3 },
    { 0x64019a44, 4 },
    { 0, 0 },
    { 0x7d359a46, 3 },
    { 0x63f19a47, 3 },
    { 0xe7859a47, 3 },
    { 0xbff99a49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6859a4e, 3 },
    { 0x8bb19a4e, 4 },
    { 0x3a319a50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e19a56, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dd9a5a, 3 },
    { 0, 0 },
    { 0xac019a5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10419a62, 3 },
    { 0x82a99a63, 3 },
    { 0xb6a59a64, 4 },
    { 0x6a319a65, 4 },
    { 0x6a699a66, 3 },
    { 0xacad9a67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c99a6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3519a6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63599a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482d9a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21359a80, 4 },
    { 0xed999a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec59a86, 4 },
    { 0x25319a86, 4 },
    { 0x80c59a88, 3 },
    { 0x8cc59a89, 3 },
    { 0x4759a8a, 4 },
    { 0, 0 },
    { 0xf9e99a8c, 4 },
    { 0, 0 },
    { 0x72019a8e, 3 },
    { 0x93699a8f, 4 },
    { 0xda2d9a8f, 3 },
    { 0x82599a8e, 3 },
    { 0xeee19a92, 4 },
    { 0xba959a92, 3 },
    { 0x4e459a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ad9a97, 3 },
    { 0, 0 },
    { 0xb8ed9a99, 3 },
    { 0x74719a9a, 3 },
    { 0x63f19a9b, 4 },
    { 0xd8e59a9b, 4 },
    { 0xf0419a9d, 3 },
    { 0xcfa99a9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33559aa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c399aa8, 3 },
    { 0x79299aa9, 4 },
    { 0xd48d9aaa, 4 },
    { 0, 0 },
    { 0x56cd9aac, 3 },
    { 0x38cd9aad, 3 },
    { 0x38559aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe19ab1, 3 },
    { 0x68219ab2, 3 },
    { 0x61199ab1, 5 },
    { 0xda799ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bd9aba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x327d9abd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f319ac1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf419ac6, 3 },
    { 0xa3f19ac7, 4 },
    { 0xc7959ac7, 3 },
    { 0x69b99ac9, 4 },
    { 0x2bc59ac9, 4 },
    { 0x1b599ac7, 3 },
    { 0x57c59acc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7999acf, 3 },
    { 0, 0 },
    { 0x4f8d9ad1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675d9ad5, 4 },
    { 0xc8759ad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d99ada, 3 },
    { 0, 0 },
    { 0x5f119adc, 3 },
    { 0, 0 },
    { 0x5fb99ade, 5 },
    { 0, 0 },
    { 0xe93d9ae0, 3 },
    { 0x8b259ae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf59ae6, 4 },
    { 0, 0 },
    { 0x34219ae8, 4 },
    { 0xdb99ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2599aed, 3 },
    { 0x73519aee, 4 },
    { 0x71f19aee, 3 },
    { 0, 0 },
    { 0x68c99af1, 3 },
    { 0xe08d9af1, 5 },
    { 0x2d999af1, 3 },
    { 0xc5799af3, 3 },
    { 0xc2419af5, 3 },
    { 0x4cf19af3, 3 },
    { 0x3c699af7, 3 },
    { 0x49099af4, 4 },
    { 0x1c819af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6159afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b519b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dd9b06, 3 },
    { 0x5b959b07, 3 },
    { 0x85119b08, 4 },
    { 0x9ead9b09, 4 },
    { 0x72519b09, 3 },
    { 0xb3559b0b, 3 },
    { 0x1d019b09, 3 },
    { 0x95459b0d, 3 },
    { 0x52299b0e, 3 },
    { 0x9f1d9b0f, 3 },
    { 0x1c9d9b10, 4 },
    { 0x15c59b11, 4 },
    { 0x1ff99b12, 3 },
    { 0x3ee59b0c, 4 },
    { 0xafcd9b14, 3 },
    { 0xa9819b0b, 3 },
    { 0x76a99b16, 3 },
    { 0x37f59b0a, 3 },
    { 0, 0 },
    { 0x58999b19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6319b20, 4 },
    { 0xd7199b21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dad9b24, 3 },
    { 0x8dd99b24, 3 },
    { 0xcccd9b26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e59b2d, 3 },
    { 0, 0 },
    { 0xcd219b2f, 3 },
    { 0, 0 },
    { 0x20b59b31, 3 },
    { 0xdfe19b32, 3 },
    { 0x45f99b31, 4 },
    { 0x22b99b34, 3 },
    { 0x63b59b35, 4 },
    { 0xde699b36, 3 },
    { 0xc8999b37, 3 },
    { 0, 0 },
    { 0xddfd9b39, 3 },
    { 0x36c99b39, 3 },
    { 0, 0 },
    { 0x58ad9b3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7d9b46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8d9b4b, 3 },
    { 0, 0 },
    { 0xb3999b4d, 4 },
    { 0xb9399b4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44559b52, 3 },
    { 0xe7d9b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8319b58, 3 },
    { 0x869d9b59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b99b5c, 3 },
    { 0x6c919b5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52959b62, 3 },
    { 0, 0 },
    { 0x2c6d9b64, 3 },
    { 0xa1cd9b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc99b6a, 3 },
    { 0x3c699b6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6419b6e, 3 },
    { 0xb659b6f, 3 },
    { 0xebf59b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a19b76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb59b79, 4 },
    { 0, 0 },
    { 0xc5a19b7b, 4 },
    { 0x17859b7c, 3 },
    { 0xa4459b7d, 4 },
    { 0, 0 },
    { 0x3b319b7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c99b82, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d9b87, 4 },
    { 0xeb9d9b88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d9b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x374d9b91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96919b94, 3 },
    { 0xe7299b95, 3 },
    { 0xa6699b96, 3 },
    { 0xe2099b96, 3 },
    { 0x6cbd9b98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06d9ba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23059ba7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206d9bab, 3 },
    { 0, 0 },
    { 0xa0099bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c19bb0, 3 },
    { 0xd319bb1, 4 },
    { 0xaa699bb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b659bba, 3 },
    { 0x5f499bbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15059bc1, 4 },
    { 0x40499bc2, 3 },
    { 0xc2499bc3, 3 },
    { 0xd9e59bc3, 3 },
    { 0xf7c59bc5, 4 },
    { 0, 0 },
    { 0x42519bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc59bca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f59bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c59bd2, 3 },
    { 0xeb619bd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca99bd6, 3 },
    { 0x3b599bd6, 3 },
    { 0x9f159bd8, 4 },
    { 0xca0d9bd9, 4 },
    { 0, 0 },
    { 0x8f899bdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c659bde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d9be5, 4 },
    { 0xa8dd9be6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x556d9be9, 4 },
    { 0x3e999bea, 3 },
    { 0x988d9beb, 3 },
    { 0xdb699bec, 4 },
    { 0xdbcd9bec, 3 },
    { 0xa59d9bec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f19bf7, 3 },
    { 0x65959bf8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b19bfc, 3 },
    { 0x2a299bfd, 3 },
    { 0, 0 },
    { 0x30459bff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e19c03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e19c09, 4 },
    { 0xecf99c09, 4 },
    { 0x312d9c0b, 3 },
    { 0x2a759c0c, 3 },
    { 0x7ff99c0c, 3 },
    { 0x6c419c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b99c11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c99c14, 3 },
    { 0xe14d9c15, 3 },
    { 0xd63d9c15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf199c19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c99c23, 3 },
    { 0x790d9c23, 3 },
    { 0xf9619c25, 3 },
    { 0xaf659c25, 3 },
    { 0xf9319c27, 3 },
    { 0x5eb99c28, 3 },
    { 0xb6c59c29, 3 },
    { 0xb4819c28, 4 },
    { 0x7e819c28, 3 },
    { 0x218d9c2a, 3 },
    { 0x5cd9c2d, 3 },
    { 0x8bdd9c24, 3 },
    { 0xa9919c2f, 4 },
    { 0x3ced9c30, 3 },
    { 0xd0999c2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c99c34, 3 },
    { 0xe5cd9c35, 3 },
    { 0xe6159c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbd9c3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c859c49, 4 },
    { 0x5aa99c4a, 4 },
    { 0x41b19c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19c51, 3 },
    { 0x98599c51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf399c56, 3 },
    { 0x73a19c57, 4 },
    { 0x28899c58, 3 },
    { 0xbe0d9c59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16d9c61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70d9c65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d9c6c, 3 },
    { 0x7e399c6d, 3 },
    { 0x31c59c6e, 3 },
    { 0, 0 },
    { 0x7da99c70, 3 },
    { 0, 0 },
    { 0xe39d9c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab59c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db59c78, 3 },
    { 0xa0d59c79, 3 },
    { 0x4e859c7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a59c7d, 3 },
    { 0xde119c7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557d9c8e, 3 },
    { 0, 0 },
    { 0xb6cd9c90, 5 },
    { 0xc3219c91, 3 },
    { 0, 0 },
    { 0x1ead9c93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad9c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5d9c9c, 3 },
    { 0xa65d9c9d, 3 },
    { 0, 0 },
    { 0x5ab99c9f, 3 },
    { 0xa0519ca0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36d9ca5, 3 },
    { 0x6eed9ca6, 3 },
    { 0xb62d9ca6, 3 },
    { 0xee8d9ca5, 3 },
    { 0xc9c99ca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c19cac, 3 },
    { 0x236d9cac, 3 },
    { 0x5db59cae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed19cb2, 4 },
    { 0x9ecd9cb2, 3 },
    { 0xcc1d9cb4, 4 },
    { 0x703d9cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca19cb9, 3 },
    { 0x10219cb9, 3 },
    { 0xeb959cb9, 4 },
    { 0xeb3d9cbc, 3 },
    { 0, 0 },
    { 0xbda19cbe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d9cc6, 3 },
    { 0xba199cc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88619ccb, 4 },
    { 0x98359ccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8999cd0, 3 },
    { 0x10ad9cd1, 3 },
    { 0, 0 },
    { 0x22f99cd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b099cda, 3 },
    { 0x56a19cdb, 4 },
    { 0x8bcd9cdc, 3 },
    { 0xab2d9cdd, 4 },
    { 0xcd6d9cda, 3 },
    { 0x79a99cde, 3 },
    { 0x9ee59cdc, 3 },
    { 0x76a19cdc, 3 },
    { 0xa15d9ce2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5d9ce6, 3 },
    { 0x67d59ce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba359cea, 3 },
    { 0, 0 },
    { 0x97b99cec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c99cf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d9cf7, 3 },
    { 0, 0 },
    { 0x83e19cf9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4359cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6d9d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50699d05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81599d0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0519d0f, 3 },
    { 0xe3019d10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26259d13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a459d18, 3 },
    { 0xf7d19d19, 4 },
    { 0x65199d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1d9d1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a99d20, 3 },
    { 0xd1519d20, 3 },
    { 0, 0 },
    { 0xf9919d23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e59d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2819d2a, 3 },
    { 0x1bdd9d2b, 3 },
    { 0x597d9d2b, 3 },
    { 0xb5cd9d2b, 3 },
    { 0, 0 },
    { 0x19259d2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6459d32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdd9d35, 3 },
    { 0xd6a59d35, 4 },
    { 0x51699d35, 3 },
    { 0x4bfd9d35, 3 },
    { 0x54f59d39, 3 },
    { 0x3c059d39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d9d3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec99d41, 3 },
    { 0xdf59d41, 3 },
    { 0, 0 },
    { 0xea3d9d44, 3 },
    { 0, 0 },
    { 0x6bd19d46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a599d4b, 5 },
    { 0xa8599d4c, 3 },
    { 0x4e799d4d, 3 },
    { 0, 0 },
    { 0x8f119d4f, 3 },
    { 0x29719d50, 3 },
    { 0x331d9d51, 3 },
    { 0, 0 },
    { 0x28b99d53, 3 },
    { 0xdb59d54, 3 },
    { 0x73019d55, 3 },
    { 0xf15d9d56, 3 },
    { 0xbdc99d57, 3 },
    { 0xa3bd9d58, 3 },
    { 0x8db59d58, 3 },
    { 0x6e759d5a, 3 },
    { 0xa5a19d5b, 3 },
    { 0x98d59d5a, 3 },
    { 0x66a59d5b, 3 },
    { 0x6b2d9d5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240d9d68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bed9d6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efd9d73, 4 },
    { 0xc8c19d73, 3 },
    { 0x95dd9d75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41259d7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a19d82, 3 },
    { 0x10059d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85159d8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50019d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add9d99, 3 },
    { 0x96ed9d99, 3 },
    { 0x67a99d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ed9d9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe19da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8099dab, 3 },
    { 0x99a19dab, 4 },
    { 0, 0 },
    { 0xc8dd9dae, 3 },
    { 0xa7599daf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ded9db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x264d9db5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea59dba, 4 },
    { 0x5e3d9dbb, 4 },
    { 0xeeb59dba, 3 },
    { 0xa7059dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb99dc2, 4 },
    { 0xd4c19dc3, 3 },
    { 0x75819dc4, 3 },
    { 0, 0 },
    { 0xd7e59dc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4599dc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd9dcc, 3 },
    { 0x18219dcd, 4 },
    { 0x11259dcd, 3 },
    { 0x82fd9dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8d9dd6, 3 },
    { 0x8f8d9dd7, 3 },
    { 0, 0 },
    { 0x18919dd9, 3 },
    { 0xaac59dd9, 3 },
    { 0, 0 },
    { 0x25e59ddc, 4 },
    { 0x16a59ddd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a99de0, 3 },
    { 0, 0 },
    { 0x3b899de2, 3 },
    { 0, 0 },
    { 0x81d99de4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed19dea, 4 },
    { 0x53c19deb, 3 },
    { 0xae259deb, 3 },
    { 0x16c19deb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a259df3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8259df8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad9dfe, 3 },
    { 0, 0 },
    { 0x4ecd9e00, 4 },
    { 0x6b659e01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ed9e05, 3 },
    { 0, 0 },
    { 0x58899e07, 3 },
    { 0, 0 },
    { 0x55859e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36999e0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c19e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a19e13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cd9e16, 3 },
    { 0x58259e17, 3 },
    { 0, 0 },
    { 0x8f219e19, 3 },
    { 0xad059e1a, 3 },
    { 0, 0 },
    { 0x68ed9e1c, 3 },
    { 0xc9259e1c, 5 },
    { 0x6aad9e1e, 3 },
    { 0, 0 },
    { 0x32859e20, 3 },
    { 0, 0 },
    { 0x7df19e22, 3 },
    { 0x3aed9e23, 3 },
    { 0xb1ad9e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a659e27, 3 },
    { 0, 0 },
    { 0x7ed99e29, 3 },
    { 0, 0 },
    { 0xf7dd9e2b, 3 },
    { 0xf9559e2c, 3 },
    { 0x10dd9e2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66919e31, 3 },
    { 0x73459e31, 3 },
    { 0xb059e33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b99e39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d19e3e, 3 },
    { 0x12499e3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc119e44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d9e47, 3 },
    { 0xbdc59e47, 3 },
    { 0x6c359e49, 3 },
    { 0x3b5d9e49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c559e50, 4 },
    { 0, 0 },
    { 0xf7659e52, 3 },
    { 0, 0 },
    { 0xfa699e54, 3 },
    { 0x1bdd9e55, 3 },
    { 0, 0 },
    { 0xfbf19e57, 3 },
    { 0x86959e58, 3 },
    { 0, 0 },
    { 0xe39d9e5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31499e62, 3 },
    { 0xf8bd9e63, 4 },
    { 0x50599e63, 3 },
    { 0, 0 },
    { 0x647d9e66, 3 },
    { 0, 0 },
    { 0xded99e68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f59e6b, 3 },
    { 0, 0 },
    { 0x54f59e6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ad9e70, 5 },
    { 0xba419e71, 3 },
    { 0, 0 },
    { 0x8a999e73, 3 },
    { 0x24319e74, 3 },
    { 0, 0 },
    { 0x651d9e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e99e80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x79799e83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c799e88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x180d9e8b, 4 },
    { 0, 0 },
    { 0xa659e8d, 3 },
    { 0xe8119e8e, 4 },
    { 0xe1019e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad9e93, 3 },
    { 0x5a159e94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356d9e9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4d9ea2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b59ea5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20699ea8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3119eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68d9eb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dd9ebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66859eca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e419ecd, 3 },
    { 0, 0 },
    { 0x31fd9ecf, 3 },
    { 0, 0 },
    { 0xbcd59ed1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5d9ed6, 3 },
    { 0x7e1d9ed7, 3 },
    { 0x99d59ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae59ee7, 3 },
    { 0xfe219ee8, 3 },
    { 0xe8859ee9, 3 },
    { 0x57bd9eea, 3 },
    { 0, 0 },
    { 0xb1499eec, 4 },
    { 0x5f999eed, 3 },
    { 0xdd2d9eec, 3 },
    { 0x543d9eed, 3 },
    { 0x34099ef0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dd9ef3, 3 },
    { 0, 0 },
    { 0x6b5d9ef5, 3 },
    { 0, 0 },
    { 0xfcf99ef7, 3 },
    { 0x2a159ef8, 3 },
    { 0, 0 },
    { 0xf8c99efa, 4 },
    { 0x14c59efb, 4 },
    { 0xab699efb, 4 },
    { 0x45f99efb, 3 },
    { 0x76859efb, 3 },
    { 0x9c1d9efa, 3 },
    { 0xaa399f00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d99f03, 3 },
    { 0, 0 },
    { 0xaaf59f05, 3 },
    { 0, 0 },
    { 0xcd059f07, 3 },
    { 0x8bd9f07, 3 },
    { 0x18099f09, 3 },
    { 0x3be19f0a, 3 },
    { 0x81159f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2299f0f, 3 },
    { 0x5bd19f10, 4 },
    { 0x99dd9f11, 3 },
    { 0x91bd9f12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f599f18, 3 },
    { 0x8a8d9f19, 5 },
    { 0x20319f19, 3 },
    { 0xe4e99f18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bad9f1e, 3 },
    { 0x66219f1f, 3 },
    { 0, 0 },
    { 0x41f99f21, 4 },
    { 0x74159f21, 3 },
    { 0, 0 },
    { 0xac9d9f24, 3 },
    { 0, 0 },
    { 0x77119f26, 3 },
    { 0xd5059f27, 3 },
    { 0xb49d9f28, 4 },
    { 0xfb0d9f29, 3 },
    { 0, 0 },
    { 0x80999f2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785d9f31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89459f39, 3 },
    { 0, 0 },
    { 0x37dd9f3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d019f3f, 4 },
    { 0xce119f3f, 3 },
    { 0x5c659f41, 3 },
    { 0x9af59f42, 3 },
    { 0x37559f43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b419f46, 3 },
    { 0x34d9f47, 3 },
    { 0xbd459f48, 4 },
    { 0x29159f49, 5 },
    { 0x63e59f49, 4 },
    { 0xc9a19f4b, 3 },
    { 0xf74d9f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6919f51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a619f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d9f59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x677d9f5c, 3 },
    { 0x6a799f5c, 3 },
    { 0xd5859f5e, 4 },
    { 0x45519f5f, 3 },
    { 0x78299f60, 4 },
    { 0xb50d9f61, 4 },
    { 0x3dd19f5f, 3 },
    { 0xdef59f5e, 4 },
    { 0xf63d9f64, 3 },
    { 0x10099f64, 4 },
    { 0xfce99f66, 4 },
    { 0xed699f5e, 3 },
    { 0x11a99f68, 3 },
    { 0xd3e59f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d9f6e, 3 },
    { 0xa1b19f6f, 3 },
    { 0x495d9f6f, 3 },
    { 0xa6399f71, 3 },
    { 0x66f19f72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67219f7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec619f7d, 3 },
    { 0, 0 },
    { 0x44e19f7f, 3 },
    { 0x63f99f80, 3 },
    { 0xcef19f81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3d9f8a, 3 },
    { 0x31399f8b, 3 },
    { 0x6c299f8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fd9f8f, 3 },
    { 0xde419f90, 3 },
    { 0, 0 },
    { 0x66799f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8d9f95, 3 },
    { 0x69359f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824d9fa3, 5 },
    { 0x84119fa4, 3 },
    { 0, 0 },
    { 0xab619fa6, 4 },
    { 0xfd259fa7, 4 },
    { 0x12d59fa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90319fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fd9fb3, 3 },
    { 0x380d9fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60419fbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x98399fbe, 3 },
    { 0, 0 },
    { 0x81f59fc0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d19fc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1d9fcb, 3 },
    { 0, 0 },
    { 0x84459fcd, 5 },
    { 0xc9f59fce, 3 },
    { 0x9ed9fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec899fd2, 3 },
    { 0, 0 },
    { 0xa4ad9fd4, 3 },
    { 0x43f19fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da19fd8, 3 },
    { 0x8bf59fd9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670d9fdf, 3 },
    { 0xe4c99fe0, 3 },
    { 0x994d9fdf, 3 },
    { 0x1f659fdf, 4 },
    { 0x2ebd9fdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d9fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d99fef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec319ff6, 3 },
    { 0x99f99ff7, 3 },
    { 0xa83d9ff7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d99ffb, 3 },
    { 0xe5399ffb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc191a000, 4 },
    { 0x831da001, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd549a004, 4 },
    { 0xa851a005, 3 },
    { 0, 0 },
    { 0x321a007, 3 },
    { 0x7d01a007, 3 },
    { 0x877da007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c9a00d, 3 },
    { 0xcd7da00e, 4 },
    { 0x15c5a00f, 3 },
    { 0x9d9a00f, 3 },
    { 0x628da00e, 3 },
    { 0x750da011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b1a019, 3 },
    { 0xe7c9a019, 4 },
    { 0x66fda01b, 3 },
    { 0, 0 },
    { 0x1e7da01d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515a024, 4 },
    { 0, 0 },
    { 0x6745a026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba5a029, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26da03a, 3 },
    { 0, 0 },
    { 0x5201a03c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43da041, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a9a044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791a04b, 3 },
    { 0xc8f9a04b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4375a051, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236da056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a41a05a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac9a05e, 3 },
    { 0x84d5a05f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561da06a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f5a06d, 3 },
    { 0xcc31a06e, 3 },
    { 0x4005a06e, 3 },
    { 0x5561a06e, 3 },
    { 0x8b75a06e, 4 },
    { 0, 0 },
    { 0x6ff5a073, 3 },
    { 0x61eda073, 3 },
    { 0x14f5a074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc9a079, 3 },
    { 0x21bda07a, 4 },
    { 0xec1da079, 3 },
    { 0, 0 },
    { 0x55a07d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9da080, 3 },
    { 0x2f81a080, 4 },
    { 0x8639a082, 3 },
    { 0, 0 },
    { 0xfc6da084, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3869a087, 4 },
    { 0x34a1a088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f91a08d, 3 },
    { 0x464da08e, 3 },
    { 0x29ada08e, 3 },
    { 0x502da090, 3 },
    { 0, 0 },
    { 0xd289a092, 3 },
    { 0, 0 },
    { 0xbec5a094, 3 },
    { 0, 0 },
    { 0x9a39a096, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3825a09d, 3 },
    { 0xfd91a09e, 3 },
    { 0x67ada09f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b9a0a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eda0a5, 4 },
    { 0, 0 },
    { 0xaa5da0a7, 3 },
    { 0, 0 },
    { 0x8cc9a0a9, 4 },
    { 0, 0 },
    { 0xee95a0ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa01a0ae, 3 },
    { 0x9865a0ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae1a0b6, 3 },
    { 0x50f9a0b7, 3 },
    { 0x8a1a0b8, 3 },
    { 0x6beda0b9, 3 },
    { 0, 0 },
    { 0x54c1a0bb, 3 },
    { 0, 0 },
    { 0x82d5a0bd, 3 },
    { 0, 0 },
    { 0x16c5a0bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a1a0c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cda0c6, 3 },
    { 0x8d9a0c7, 3 },
    { 0, 0 },
    { 0x222da0c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed21a0cd, 3 },
    { 0x9669a0ce, 4 },
    { 0, 0 },
    { 0x55b9a0d0, 3 },
    { 0, 0 },
    { 0xec4da0d2, 4 },
    { 0x7da5a0d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e1a0dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d1a0e1, 3 },
    { 0xd6fda0e2, 4 },
    { 0x9f0da0e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1da0e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94da0e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c5a0ed, 4 },
    { 0, 0 },
    { 0x672da0ef, 3 },
    { 0x7c95a0f0, 4 },
    { 0, 0 },
    { 0xe901a0f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eda0f6, 4 },
    { 0, 0 },
    { 0x6561a0f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c5a0fb, 4 },
    { 0x78e1a0fc, 5 },
    { 0x6a9a0fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4da100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd255a104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9a107, 3 },
    { 0xff4da107, 4 },
    { 0x8d09a109, 4 },
    { 0x1aa1a107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e45a10d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3da110, 3 },
    { 0x7935a111, 4 },
    { 0x804da112, 4 },
    { 0x8bda113, 3 },
    { 0x561a113, 3 },
    { 0xd33da115, 4 },
    { 0x8e29a116, 3 },
    { 0x239a115, 3 },
    { 0x4715a113, 3 },
    { 0xb571a116, 3 },
    { 0x7a91a115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe275a125, 3 },
    { 0x8d25a125, 3 },
    { 0x1925a127, 5 },
    { 0x43da125, 3 },
    { 0x3f99a129, 3 },
    { 0xe1d1a125, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf559a12d, 3 },
    { 0, 0 },
    { 0x99c1a12f, 4 },
    { 0, 0 },
    { 0xfb59a131, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd531a134, 3 },
    { 0xe951a134, 3 },
    { 0xfb1a136, 3 },
    { 0xfd65a137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a1a13d, 3 },
    { 0x66a5a13e, 3 },
    { 0, 0 },
    { 0x7a29a140, 3 },
    { 0x9639a140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee1a144, 3 },
    { 0, 0 },
    { 0xd591a146, 3 },
    { 0xed99a147, 3 },
    { 0xabada147, 3 },
    { 0, 0 },
    { 0xfe2da14a, 3 },
    { 0, 0 },
    { 0xd6eda14c, 3 },
    { 0, 0 },
    { 0x68eda14e, 5 },
    { 0, 0 },
    { 0x6bb9a150, 3 },
    { 0xc659a150, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a154, 4 },
    { 0x6d4da154, 4 },
    { 0x7b65a156, 3 },
    { 0xcca5a156, 3 },
    { 0, 0 },
    { 0xae35a159, 3 },
    { 0x44d9a15a, 3 },
    { 0xfb81a15b, 4 },
    { 0xae71a15c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fda161, 3 },
    { 0x78d9a162, 3 },
    { 0x75ada163, 3 },
    { 0xbeada163, 4 },
    { 0x99d9a163, 3 },
    { 0, 0 },
    { 0x31bda167, 4 },
    { 0xdb8da167, 3 },
    { 0xb2bda169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0da16c, 3 },
    { 0x8091a16d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd1a172, 3 },
    { 0, 0 },
    { 0x4351a174, 3 },
    { 0x225a175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259a179, 3 },
    { 0x2061a17a, 3 },
    { 0x519a17a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5da182, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51a188, 3 },
    { 0, 0 },
    { 0x9de5a18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9a18f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad01a195, 4 },
    { 0x9a15a196, 3 },
    { 0x562da195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a85a19a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa891a1a0, 3 },
    { 0x1ceda1a1, 3 },
    { 0, 0 },
    { 0xf145a1a3, 3 },
    { 0, 0 },
    { 0xde41a1a5, 3 },
    { 0x30ada1a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0da1a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a51a1b2, 3 },
    { 0xaf19a1b2, 3 },
    { 0x2a49a1b4, 3 },
    { 0x4575a1b5, 3 },
    { 0xf115a1b5, 4 },
    { 0x5395a1b6, 4 },
    { 0x964da1b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a5a1c0, 3 },
    { 0x523da1c0, 3 },
    { 0xe7a1a1c2, 3 },
    { 0xe529a1c3, 3 },
    { 0, 0 },
    { 0xfa8da1c5, 3 },
    { 0, 0 },
    { 0xd629a1c7, 3 },
    { 0xf28da1c7, 4 },
    { 0xb765a1c9, 5 },
    { 0x4165a1ca, 3 },
    { 0xe119a1c9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2da1cf, 3 },
    { 0xba05a1d0, 3 },
    { 0x769a1d1, 4 },
    { 0x35c9a1d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa445a1d6, 3 },
    { 0x9ed1a1d6, 3 },
    { 0xb325a1d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1a1dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8da1e0, 3 },
    { 0xff5a1e1, 3 },
    { 0x4fc1a1e2, 3 },
    { 0xdca1a1e1, 4 },
    { 0, 0 },
    { 0xeeb5a1e5, 3 },
    { 0xb42da1e6, 3 },
    { 0x8bf1a1e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x600da1ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a1ed, 3 },
    { 0, 0 },
    { 0x9fc1a1ef, 4 },
    { 0xadc5a1ef, 3 },
    { 0x9189a1ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9a1f5, 3 },
    { 0, 0 },
    { 0x2c45a1f7, 3 },
    { 0xa509a1f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809a1fc, 3 },
    { 0, 0 },
    { 0x3ad5a1fe, 3 },
    { 0xb279a1ff, 3 },
    { 0, 0 },
    { 0xfae5a201, 3 },
    { 0x990da202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce9a208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f11a20d, 5 },
    { 0x7761a20e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44da211, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d5a216, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671a219, 3 },
    { 0xd3a9a21a, 3 },
    { 0, 0 },
    { 0x6d51a21c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501a223, 3 },
    { 0xe911a223, 3 },
    { 0x7eeda223, 3 },
    { 0x2bf1a226, 3 },
    { 0x9db5a226, 3 },
    { 0x4f61a225, 3 },
    { 0, 0 },
    { 0x753da22a, 3 },
    { 0xc1e9a22a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fda231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8da234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a1a23c, 3 },
    { 0xda5da23d, 3 },
    { 0x3e45a23e, 3 },
    { 0, 0 },
    { 0xd32da240, 3 },
    { 0xcb3da241, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a9a246, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5819a249, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695a253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb465a257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f85a25b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3491a263, 3 },
    { 0, 0 },
    { 0x4601a265, 3 },
    { 0, 0 },
    { 0xbf81a267, 3 },
    { 0, 0 },
    { 0xc029a269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8591a26d, 3 },
    { 0, 0 },
    { 0x9e21a26f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x311da272, 3 },
    { 0xecb9a273, 4 },
    { 0x6bb1a274, 5 },
    { 0x7449a275, 4 },
    { 0x218da276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca9a27d, 3 },
    { 0x3211a27e, 3 },
    { 0x9885a27f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469a286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eda28e, 4 },
    { 0, 0 },
    { 0xd639a290, 3 },
    { 0x80c5a290, 4 },
    { 0, 0 },
    { 0x8825a293, 3 },
    { 0, 0 },
    { 0xc8c1a295, 4 },
    { 0x6dc5a296, 3 },
    { 0, 0 },
    { 0x3a11a298, 3 },
    { 0x1cb1a298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfda29e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa5a2a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1da2a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacda2a7, 3 },
    { 0x9b01a2a7, 3 },
    { 0x3055a2a8, 3 },
    { 0x27f9a2aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fda2af, 3 },
    { 0, 0 },
    { 0xd6cda2b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9a2b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aada2ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bda2bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe139a2c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841a2cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5a2cf, 3 },
    { 0x6e1da2d0, 3 },
    { 0x7f8da2d1, 3 },
    { 0x6549a2d0, 3 },
    { 0x296da2d3, 3 },
    { 0xb45da2d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd765a2d8, 3 },
    { 0x587da2d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701a2dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb541a2e8, 3 },
    { 0x4b9a2e8, 3 },
    { 0xcf2da2ea, 3 },
    { 0, 0 },
    { 0x1b1a2ec, 4 },
    { 0, 0 },
    { 0xd3e9a2ee, 3 },
    { 0xbd99a2ee, 4 },
    { 0xafcda2f0, 3 },
    { 0, 0 },
    { 0x41d5a2f2, 3 },
    { 0, 0 },
    { 0xc611a2f4, 3 },
    { 0, 0 },
    { 0xc879a2f6, 4 },
    { 0x57d1a2f7, 3 },
    { 0xf775a2f8, 3 },
    { 0xdc29a2f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2261a2ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa759a302, 3 },
    { 0x6a1a302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a1a309, 4 },
    { 0x1dada30a, 3 },
    { 0x87c5a30b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c9a316, 4 },
    { 0x2ee9a317, 4 },
    { 0, 0 },
    { 0x5245a319, 3 },
    { 0xae51a31a, 3 },
    { 0x85d1a319, 3 },
    { 0xc325a31c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa219a320, 3 },
    { 0x62c1a320, 3 },
    { 0xf89da322, 3 },
    { 0x2c79a323, 3 },
    { 0x6911a321, 3 },
    { 0xd059a320, 5 },
    { 0xb211a326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4699a329, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b1a32c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0da335, 3 },
    { 0, 0 },
    { 0xe46da337, 3 },
    { 0, 0 },
    { 0x6205a339, 3 },
    { 0x4961a33a, 3 },
    { 0, 0 },
    { 0xce0da33c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3159a341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a346, 3 },
    { 0xc1f5a347, 3 },
    { 0x7625a347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfda34c, 3 },
    { 0x84f1a34c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b71a351, 3 },
    { 0x4d65a352, 3 },
    { 0xaf4da351, 3 },
    { 0xde65a353, 4 },
    { 0x67f5a355, 3 },
    { 0, 0 },
    { 0x41a1a357, 3 },
    { 0x392da358, 4 },
    { 0xdfc5a359, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc199a35e, 3 },
    { 0xe31a35f, 3 },
    { 0x4a29a360, 4 },
    { 0xd725a360, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b9a365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef45a369, 3 },
    { 0, 0 },
    { 0x1d4da36b, 3 },
    { 0xf2f5a36c, 4 },
    { 0x8ecda36d, 3 },
    { 0x785da36e, 3 },
    { 0x22d9a36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3405a374, 3 },
    { 0x2d9a374, 3 },
    { 0xf7a1a375, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee5a37b, 3 },
    { 0x5385a37c, 5 },
    { 0x3141a37d, 3 },
    { 0x37e9a37e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9a381, 4 },
    { 0xf725a382, 3 },
    { 0, 0 },
    { 0x3ee9a384, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329a388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc279a38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abda38f, 3 },
    { 0x13f5a390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde59a395, 3 },
    { 0xa555a396, 3 },
    { 0xc40da397, 3 },
    { 0xe2c1a397, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe661a3a1, 4 },
    { 0x212da3a1, 3 },
    { 0x7651a3a3, 3 },
    { 0xf53da3a4, 3 },
    { 0x6eada3a3, 3 },
    { 0xd261a3a5, 3 },
    { 0, 0 },
    { 0xde81a3a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f35a3ad, 3 },
    { 0xe631a3ad, 3 },
    { 0x9439a3af, 3 },
    { 0x9005a3ae, 3 },
    { 0xcd85a3b1, 3 },
    { 0xc7d5a3ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adda3b7, 3 },
    { 0x1cada3b8, 4 },
    { 0xaaf9a3b9, 3 },
    { 0x3299a3ba, 3 },
    { 0xc3c5a3b9, 3 },
    { 0xea7da3bc, 3 },
    { 0xdfc5a3bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eda3c4, 3 },
    { 0x3bd1a3c5, 3 },
    { 0x5121a3c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5701a3c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb261a3cc, 3 },
    { 0x8975a3cd, 3 },
    { 0x562da3ce, 3 },
    { 0x2bfda3cf, 4 },
    { 0, 0 },
    { 0x18f5a3d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3899a3d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1a3da, 3 },
    { 0xead5a3da, 3 },
    { 0, 0 },
    { 0xbf51a3dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa65a3e2, 3 },
    { 0xd7eda3e2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa9a3ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2641a3f0, 3 },
    { 0x10f5a3f0, 3 },
    { 0x6905a3f2, 3 },
    { 0, 0 },
    { 0xfeb1a3f4, 4 },
    { 0, 0 },
    { 0x78b1a3f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5301a3fc, 3 },
    { 0x8f5a3fd, 3 },
    { 0xd515a3fe, 3 },
    { 0x4599a3fd, 3 },
    { 0xd1c5a3fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c39a406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1189a409, 3 },
    { 0, 0 },
    { 0xc8a1a40b, 4 },
    { 0xa50da40c, 4 },
    { 0x706da40c, 3 },
    { 0xdae9a40e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb409a411, 3 },
    { 0x9dda412, 3 },
    { 0x9945a413, 3 },
    { 0x4185a411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309a41a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd771a41f, 5 },
    { 0x65cda420, 3 },
    { 0, 0 },
    { 0x70f1a422, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861a427, 4 },
    { 0, 0 },
    { 0x34c5a429, 3 },
    { 0x5ff1a42a, 4 },
    { 0xc0f9a42b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e81a431, 3 },
    { 0x82c5a431, 3 },
    { 0, 0 },
    { 0xb07da434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df1a438, 3 },
    { 0x7c91a439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9055a43d, 3 },
    { 0x7615a43e, 3 },
    { 0xbd61a43f, 3 },
    { 0, 0 },
    { 0x82e1a441, 4 },
    { 0, 0 },
    { 0x45a1a443, 3 },
    { 0x863da444, 3 },
    { 0x6909a445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc91a44c, 4 },
    { 0, 0 },
    { 0x327da44e, 3 },
    { 0, 0 },
    { 0x31bda450, 4 },
    { 0, 0 },
    { 0x2511a452, 3 },
    { 0x9a19a453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc95a457, 4 },
    { 0xfae1a457, 3 },
    { 0xb655a459, 3 },
    { 0xdf3da45a, 3 },
    { 0x9471a45b, 4 },
    { 0x97f9a45c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d21a467, 4 },
    { 0, 0 },
    { 0x7fa5a469, 3 },
    { 0x4329a46a, 3 },
    { 0x78b9a46b, 3 },
    { 0, 0 },
    { 0x5fdda46d, 4 },
    { 0x1e35a46e, 3 },
    { 0xfa1a46d, 3 },
    { 0xae9da470, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1965a475, 3 },
    { 0xe215a475, 4 },
    { 0x8789a477, 3 },
    { 0xd315a477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bda47d, 3 },
    { 0, 0 },
    { 0xbd7da47f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fda489, 4 },
    { 0x9235a48a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf549a48e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535a491, 3 },
    { 0x3f6da491, 4 },
    { 0x1075a492, 3 },
    { 0x384da494, 3 },
    { 0x7c89a494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b9a498, 3 },
    { 0x4151a498, 3 },
    { 0, 0 },
    { 0x8d55a49b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115da4a8, 3 },
    { 0x3a65a4a8, 4 },
    { 0xba19a4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e09a4ae, 3 },
    { 0, 0 },
    { 0xd999a4b0, 4 },
    { 0x3025a4b1, 3 },
    { 0x6a1da4b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5da4b5, 3 },
    { 0x1ae1a4b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a81a4bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c1a4bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff1a4c3, 3 },
    { 0, 0 },
    { 0xbc59a4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4805a4c8, 3 },
    { 0xd19da4c9, 3 },
    { 0xaf21a4ca, 3 },
    { 0x790da4cb, 4 },
    { 0xfc4da4ca, 3 },
    { 0xddc1a4cb, 3 },
    { 0xe5a1a4ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11a4d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6401a4d7, 3 },
    { 0xf34da4d8, 4 },
    { 0xab01a4d9, 4 },
    { 0xc351a4da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03da4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe689a4e1, 3 },
    { 0x5621a4e2, 3 },
    { 0x45e9a4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941a4e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2901a4ed, 3 },
    { 0, 0 },
    { 0xf515a4ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456da4f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e9a4f6, 3 },
    { 0, 0 },
    { 0xf0d5a4f8, 4 },
    { 0, 0 },
    { 0x94c5a4fa, 3 },
    { 0xaee1a4fb, 3 },
    { 0x4f59a4fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e59a505, 4 },
    { 0x20dda505, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3669a50b, 3 },
    { 0, 0 },
    { 0xa87da50d, 3 },
    { 0x92c9a50d, 4 },
    { 0x7d8da50e, 4 },
    { 0xfa39a50f, 3 },
    { 0x2dcda511, 4 },
    { 0xdd71a512, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68da51b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda5a51e, 3 },
    { 0xcb95a51f, 3 },
    { 0, 0 },
    { 0xddbda521, 4 },
    { 0xb2d9a522, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d45a526, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa121a52c, 3 },
    { 0xdaa9a52d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b19a531, 3 },
    { 0x4ebda532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb1a535, 3 },
    { 0x5b81a536, 4 },
    { 0, 0 },
    { 0x5b25a538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63da53c, 3 },
    { 0x36c5a53c, 3 },
    { 0x75c5a53e, 3 },
    { 0x37dda53f, 3 },
    { 0xf959a53d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c9a543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33da546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1a549, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe005a54c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a5a551, 3 },
    { 0xa4f5a552, 3 },
    { 0, 0 },
    { 0xdb51a554, 3 },
    { 0, 0 },
    { 0xcd69a556, 3 },
    { 0xa959a556, 3 },
    { 0, 0 },
    { 0xecb5a559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8845a55e, 4 },
    { 0, 0 },
    { 0x3741a560, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1da563, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25da566, 3 },
    { 0, 0 },
    { 0xb9e1a568, 3 },
    { 0xfff5a568, 3 },
    { 0x6a51a56a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dda56e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc949a57a, 3 },
    { 0x8dcda57b, 3 },
    { 0x38ada57b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4595a583, 3 },
    { 0x3c35a583, 3 },
    { 0, 0 },
    { 0x2759a586, 3 },
    { 0, 0 },
    { 0xbb91a588, 4 },
    { 0xcd59a588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40da591, 4 },
    { 0xb1c5a592, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f91a597, 4 },
    { 0, 0 },
    { 0x22eda599, 4 },
    { 0xb0d1a59a, 3 },
    { 0x3a25a59a, 3 },
    { 0, 0 },
    { 0x9901a59d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef1a5a7, 3 },
    { 0xfeeda5a8, 4 },
    { 0x347da5a9, 3 },
    { 0x7e65a5aa, 3 },
    { 0xd3e9a5aa, 3 },
    { 0, 0 },
    { 0x1fd9a5ad, 4 },
    { 0x1d69a5ae, 3 },
    { 0x6311a5af, 3 },
    { 0x9919a5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e85a5b3, 4 },
    { 0, 0 },
    { 0xc501a5b5, 4 },
    { 0xe961a5b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5921a5bb, 3 },
    { 0, 0 },
    { 0x9d05a5bd, 4 },
    { 0xb6f5a5bd, 3 },
    { 0xfee5a5bd, 3 },
    { 0xbc85a5c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c9a5c5, 3 },
    { 0x22b9a5c6, 3 },
    { 0x43f1a5c7, 3 },
    { 0x9129a5c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7959a5cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa835a5d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293da5d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdda5da, 3 },
    { 0x4f41a5da, 3 },
    { 0xbdd9a5dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6da5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9081a5e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dda5eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc815a5ef, 3 },
    { 0, 0 },
    { 0xdc3da5f1, 3 },
    { 0xbb85a5f2, 3 },
    { 0x9911a5f3, 3 },
    { 0xde35a5f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7325a5fc, 3 },
    { 0x1b1da5fd, 3 },
    { 0x7c11a5fd, 3 },
    { 0xd985a5fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0da605, 4 },
    { 0x64c5a605, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb1a609, 3 },
    { 0x1379a60a, 4 },
    { 0xbce9a60b, 3 },
    { 0xb8f9a60a, 3 },
    { 0x8111a60d, 3 },
    { 0, 0 },
    { 0x716da60f, 3 },
    { 0x4219a610, 4 },
    { 0xd83da610, 3 },
    { 0x2991a610, 5 },
    { 0xd59a613, 3 },
    { 0x4869a612, 3 },
    { 0xbed1a615, 3 },
    { 0x5b9da616, 3 },
    { 0, 0 },
    { 0xf48da618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f5a61f, 3 },
    { 0, 0 },
    { 0xffe5a621, 4 },
    { 0xbb5a622, 3 },
    { 0, 0 },
    { 0xdb31a624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65a627, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29a62b, 3 },
    { 0xef29a62c, 4 },
    { 0, 0 },
    { 0x135a62e, 4 },
    { 0xf3b5a62f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f55a635, 3 },
    { 0, 0 },
    { 0x4b15a637, 3 },
    { 0, 0 },
    { 0x3829a639, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9a63f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b1a646, 3 },
    { 0xee5da647, 3 },
    { 0x2509a648, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4349a64d, 3 },
    { 0x2cb9a64e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c5a653, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7909a65b, 3 },
    { 0x8765a65c, 3 },
    { 0xfbb9a65b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc279a660, 3 },
    { 0xe279a661, 3 },
    { 0, 0 },
    { 0x3cb1a663, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf1a668, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3da66c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a81a66f, 3 },
    { 0xb66da66f, 3 },
    { 0xc831a671, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22da674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375da678, 3 },
    { 0xfb29a679, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9a67d, 3 },
    { 0x4e9da67e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf1a682, 3 },
    { 0x896da683, 3 },
    { 0, 0 },
    { 0x9375a685, 3 },
    { 0xb115a686, 4 },
    { 0xc791a687, 4 },
    { 0x1bd9a685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205a68b, 4 },
    { 0x5b21a68c, 3 },
    { 0xaf25a68d, 3 },
    { 0x99f5a68e, 4 },
    { 0, 0 },
    { 0x9675a690, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e79a695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60da69a, 5 },
    { 0, 0 },
    { 0x5f85a69c, 4 },
    { 0x6731a69d, 3 },
    { 0x176da69c, 3 },
    { 0xe30da69f, 3 },
    { 0x9715a6a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427da6ac, 4 },
    { 0xad75a6ac, 4 },
    { 0xfa01a6ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd855a6b3, 3 },
    { 0xd325a6b3, 3 },
    { 0xa9b9a6b4, 3 },
    { 0x1ab1a6b4, 3 },
    { 0xc4b9a6b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cda6bd, 3 },
    { 0, 0 },
    { 0xbd0da6bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4085a6c2, 4 },
    { 0x963da6c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9a6cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e1a6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x348da6d6, 3 },
    { 0xff31a6d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe155a6dd, 3 },
    { 0x9651a6de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790da6e9, 3 },
    { 0x14e5a6ea, 3 },
    { 0x6629a6e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1129a6f0, 3 },
    { 0, 0 },
    { 0x44ada6f2, 3 },
    { 0x519da6f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434da6f8, 3 },
    { 0xf625a6f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7da702, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d75a705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2095a709, 4 },
    { 0x59bda70a, 3 },
    { 0x21c9a70b, 3 },
    { 0x8f55a709, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd9a715, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf905a718, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da5a71e, 3 },
    { 0xf0da71e, 3 },
    { 0xd979a720, 4 },
    { 0x2b61a721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305a727, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501a72c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e35a72f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9a732, 3 },
    { 0, 0 },
    { 0x889da734, 5 },
    { 0xec1a735, 4 },
    { 0x1855a736, 3 },
    { 0x2b4da737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6da73a, 3 },
    { 0, 0 },
    { 0xa9f9a73c, 3 },
    { 0xb325a73d, 3 },
    { 0, 0 },
    { 0x5871a73f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5a74b, 3 },
    { 0xd4a1a74c, 3 },
    { 0x4e9a74d, 3 },
    { 0x53f5a74e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb671a752, 4 },
    { 0x6935a753, 4 },
    { 0, 0 },
    { 0x5139a755, 4 },
    { 0x29e1a755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9a75c, 4 },
    { 0x6959a75c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5339a761, 3 },
    { 0x4f09a761, 3 },
    { 0, 0 },
    { 0x6a41a764, 3 },
    { 0x1bd9a765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bda768, 4 },
    { 0x60d5a769, 4 },
    { 0x567da769, 3 },
    { 0, 0 },
    { 0x5275a76c, 3 },
    { 0x76c5a76d, 4 },
    { 0x6f19a76e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b9a772, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69da77c, 3 },
    { 0, 0 },
    { 0xc129a77e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9111a781, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82da787, 3 },
    { 0xf389a788, 3 },
    { 0xb459a789, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acda78e, 3 },
    { 0x529da78e, 3 },
    { 0x1841a790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5da794, 4 },
    { 0, 0 },
    { 0x6e65a796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2595a799, 3 },
    { 0x1251a79a, 3 },
    { 0xa705a79b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d81a79e, 3 },
    { 0, 0 },
    { 0xa931a7a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ada7a4, 3 },
    { 0x70eda7a5, 3 },
    { 0, 0 },
    { 0xc869a7a7, 3 },
    { 0xe211a7a7, 3 },
    { 0xef1da7a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4771a7af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb21a7b2, 4 },
    { 0x2b59a7b3, 3 },
    { 0xac09a7b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd09a7b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ada7bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dda7be, 4 },
    { 0, 0 },
    { 0xa4a9a7c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9a7c6, 3 },
    { 0x90cda7c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa621a7ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda7cd, 3 },
    { 0, 0 },
    { 0x373da7cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934da7d6, 3 },
    { 0x15c1a7d6, 3 },
    { 0x5fa5a7d6, 3 },
    { 0, 0 },
    { 0x50e5a7da, 3 },
    { 0, 0 },
    { 0x3899a7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29a7df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129a7e6, 3 },
    { 0x6869a7e6, 3 },
    { 0x78f1a7e6, 3 },
    { 0x3965a7e9, 3 },
    { 0x4265a7ea, 3 },
    { 0xbd5da7ea, 4 },
    { 0, 0 },
    { 0x47cda7ed, 3 },
    { 0, 0 },
    { 0x4989a7ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe139a7f2, 3 },
    { 0x59d1a7f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7da7fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354da802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc895a809, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b9a80c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e61a819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6149a81f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438da823, 4 },
    { 0xae81a824, 3 },
    { 0x5b75a825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cda828, 4 },
    { 0, 0 },
    { 0x7911a82a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da5a830, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e9a833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b1a83b, 4 },
    { 0, 0 },
    { 0x1629a83d, 3 },
    { 0, 0 },
    { 0xd911a83f, 3 },
    { 0x9f9a840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe961a843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360da848, 3 },
    { 0, 0 },
    { 0xe655a84a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc575a84e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc025a856, 3 },
    { 0, 0 },
    { 0xee51a858, 3 },
    { 0xf451a858, 5 },
    { 0, 0 },
    { 0xb769a85b, 3 },
    { 0, 0 },
    { 0xade9a85d, 3 },
    { 0xfd09a85e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1a86b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149a871, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639a87a, 5 },
    { 0xde19a87a, 3 },
    { 0x9a05a87b, 3 },
    { 0xf579a87b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5479a881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1a888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9a88e, 3 },
    { 0, 0 },
    { 0xd1fda890, 3 },
    { 0, 0 },
    { 0xf581a892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6991a895, 3 },
    { 0x6f2da896, 4 },
    { 0xa21da897, 3 },
    { 0x8649a898, 3 },
    { 0x701da896, 3 },
    { 0x4bcda897, 3 },
    { 0, 0 },
    { 0x3ec1a89c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78da8a0, 3 },
    { 0, 0 },
    { 0xfc61a8a2, 3 },
    { 0x1ce5a8a3, 3 },
    { 0x30c1a8a4, 3 },
    { 0xd92da8a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7161a8a8, 3 },
    { 0, 0 },
    { 0x51ada8aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b89a8af, 3 },
    { 0, 0 },
    { 0xaaeda8b1, 3 },
    { 0xa1da8b2, 3 },
    { 0, 0 },
    { 0x17e9a8b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b5a8b9, 3 },
    { 0, 0 },
    { 0x8231a8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ada8c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02da8c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d9a8cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a8de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ada8e2, 3 },
    { 0, 0 },
    { 0x8489a8e4, 4 },
    { 0, 0 },
    { 0xc84da8e6, 4 },
    { 0xeec9a8e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd575a8ee, 3 },
    { 0, 0 },
    { 0xb0ada8f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x375da8f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a45a8f6, 3 },
    { 0, 0 },
    { 0xbd79a8f8, 3 },
    { 0xcc19a8f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d9a8fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7175a902, 3 },
    { 0, 0 },
    { 0xbaeda904, 4 },
    { 0, 0 },
    { 0xd5f9a906, 3 },
    { 0x7f61a906, 4 },
    { 0x303da908, 3 },
    { 0x7629a908, 3 },
    { 0x8451a908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe061a911, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0da917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e9a91a, 3 },
    { 0, 0 },
    { 0xc645a91c, 3 },
    { 0xf23da91d, 3 },
    { 0xf271a91e, 3 },
    { 0xc62da91e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xede1a922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab9a928, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8911a92f, 3 },
    { 0, 0 },
    { 0xaeb5a931, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dda934, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a5a938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc51a944, 5 },
    { 0, 0 },
    { 0x477da946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ada949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b1a950, 3 },
    { 0x8cb5a951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c69a956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061a959, 3 },
    { 0x5a45a95a, 4 },
    { 0x447da95a, 4 },
    { 0xe61a95c, 3 },
    { 0x9dd5a95b, 3 },
    { 0xca49a95b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e9a963, 3 },
    { 0x33da964, 3 },
    { 0, 0 },
    { 0x6a21a966, 3 },
    { 0x292da967, 3 },
    { 0x6edda968, 3 },
    { 0xa111a969, 3 },
    { 0x2945a969, 3 },
    { 0xdc19a96b, 3 },
    { 0x7d55a96c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa561a970, 4 },
    { 0xaa75a971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d1a974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901a977, 4 },
    { 0xd1d1a978, 4 },
    { 0x60a5a979, 4 },
    { 0x1915a979, 3 },
    { 0xc511a97b, 4 },
    { 0xa98da97c, 3 },
    { 0, 0 },
    { 0xe961a97e, 5 },
    { 0x376da97f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1381a984, 3 },
    { 0x4399a984, 3 },
    { 0x9db1a986, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3589a98a, 4 },
    { 0x9a8da98b, 3 },
    { 0x1809a98c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edda991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8909a995, 3 },
    { 0x4031a996, 3 },
    { 0x3331a996, 3 },
    { 0xb1e9a996, 3 },
    { 0x5e39a997, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6289a9a1, 3 },
    { 0x57c5a9a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4109a9a6, 3 },
    { 0x499a9a7, 4 },
    { 0xf415a9a7, 5 },
    { 0, 0 },
    { 0xdb3da9aa, 4 },
    { 0xac6da9ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd929a9b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b9a9b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d9a9bc, 3 },
    { 0, 0 },
    { 0x7451a9be, 3 },
    { 0, 0 },
    { 0x5709a9c0, 3 },
    { 0xc4cda9c0, 3 },
    { 0xad99a9c2, 3 },
    { 0, 0 },
    { 0xd665a9c4, 3 },
    { 0x8a21a9c4, 4 },
    { 0xff01a9c5, 3 },
    { 0xd69a9c6, 3 },
    { 0xd659a9c8, 3 },
    { 0x4cfda9c9, 3 },
    { 0x6af1a9ca, 4 },
    { 0x8ab5a9cb, 4 },
    { 0x8eb1a9c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9da9da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5da9df, 3 },
    { 0x24ada9e0, 3 },
    { 0xc9d9a9e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2061a9e4, 3 },
    { 0x790da9e5, 3 },
    { 0x1d85a9e4, 3 },
    { 0x7c2da9e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf79a9ee, 3 },
    { 0xcc7da9ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f71a9fc, 3 },
    { 0xe649a9fd, 4 },
    { 0xa315a9fe, 4 },
    { 0x7eb5a9fd, 4 },
    { 0x5269a9ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1aa03, 4 },
    { 0x10ddaa04, 4 },
    { 0xbab5aa05, 3 },
    { 0x56f5aa04, 3 },
    { 0xa939aa05, 3 },
    { 0x4825aa06, 3 },
    { 0x3f79aa09, 3 },
    { 0x23ddaa0a, 4 },
    { 0x6ff1aa03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe589aa12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e9aa15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86adaa19, 3 },
    { 0x2fc9aa1a, 3 },
    { 0, 0 },
    { 0x9835aa1c, 3 },
    { 0, 0 },
    { 0x2cedaa1e, 3 },
    { 0xbce1aa1f, 3 },
    { 0x241daa1e, 3 },
    { 0x2509aa20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1815aa28, 3 },
    { 0xbd99aa28, 4 },
    { 0x7f91aa2a, 3 },
    { 0xfc7daa2b, 3 },
    { 0xef19aa2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1aa30, 3 },
    { 0, 0 },
    { 0x7ac1aa32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925aa38, 3 },
    { 0x4199aa38, 3 },
    { 0xa745aa38, 3 },
    { 0xca25aa3b, 3 },
    { 0, 0 },
    { 0x7671aa3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14daa42, 3 },
    { 0xf611aa42, 3 },
    { 0, 0 },
    { 0x13e1aa45, 3 },
    { 0, 0 },
    { 0xdb29aa47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d75aa4a, 4 },
    { 0xe101aa4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530daa4f, 4 },
    { 0xf7bdaa50, 3 },
    { 0x539daa50, 3 },
    { 0xf6c5aa52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2daa56, 3 },
    { 0xa1b5aa57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e5aa5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2191aa64, 3 },
    { 0, 0 },
    { 0x27c9aa66, 4 },
    { 0x85edaa67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7551aa6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeddaa70, 3 },
    { 0xd2edaa71, 4 },
    { 0x8c61aa71, 3 },
    { 0, 0 },
    { 0x7b89aa74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81aa7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c5aa80, 3 },
    { 0xb789aa81, 3 },
    { 0, 0 },
    { 0x71e9aa83, 3 },
    { 0, 0 },
    { 0x8529aa85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce89aa89, 3 },
    { 0, 0 },
    { 0x9789aa8b, 3 },
    { 0xf2d5aa8c, 3 },
    { 0xcd3daa8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4daa90, 3 },
    { 0xbc2daa91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cdaa94, 5 },
    { 0, 0 },
    { 0x2fddaa96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe59aa9b, 4 },
    { 0x2205aa9b, 3 },
    { 0x6ffdaa9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7119aaa0, 4 },
    { 0x2f3daaa0, 3 },
    { 0, 0 },
    { 0x8e21aaa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe921aaa7, 3 },
    { 0, 0 },
    { 0x279daaa9, 3 },
    { 0xc351aaaa, 4 },
    { 0x94edaaaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f9aaae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d55aab6, 3 },
    { 0x51f9aab7, 3 },
    { 0, 0 },
    { 0x46b5aab9, 3 },
    { 0x4c15aaba, 3 },
    { 0xa55daab9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a55aabe, 4 },
    { 0, 0 },
    { 0xbb99aac0, 3 },
    { 0, 0 },
    { 0xeb31aac2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6521aac5, 3 },
    { 0x3a41aac6, 3 },
    { 0xb391aac7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145aaca, 4 },
    { 0x2a5daaca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49daace, 4 },
    { 0x7c45aacf, 4 },
    { 0xddd5aad0, 3 },
    { 0x2495aad1, 4 },
    { 0xbfedaad2, 3 },
    { 0xd85daad3, 4 },
    { 0x495daad4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e25aad9, 3 },
    { 0, 0 },
    { 0xb1e1aadb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08daade, 4 },
    { 0x61b9aade, 3 },
    { 0xa215aadf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45aae3, 4 },
    { 0x5619aae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7669aae7, 3 },
    { 0xc559aae8, 3 },
    { 0x3f69aae9, 3 },
    { 0, 0 },
    { 0xfab1aaeb, 4 },
    { 0x80f9aaec, 3 },
    { 0x3345aaec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2109aaf2, 4 },
    { 0xed91aaf3, 3 },
    { 0x50ddaaf4, 3 },
    { 0xaf4daaf5, 3 },
    { 0x4b5daaf6, 4 },
    { 0x6ef1aaf6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f9aafa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905aafe, 3 },
    { 0x5adaafe, 3 },
    { 0, 0 },
    { 0x8425ab01, 3 },
    { 0x1e55ab02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf89ab18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1ab1c, 3 },
    { 0x6011ab1c, 3 },
    { 0, 0 },
    { 0x853dab1f, 3 },
    { 0xd115ab20, 3 },
    { 0xd72dab20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa621ab24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9115ab28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5ab2b, 3 },
    { 0x679dab2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3409ab32, 3 },
    { 0, 0 },
    { 0x17d1ab34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb115ab38, 5 },
    { 0x7469ab39, 3 },
    { 0x2a25ab3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb631ab3e, 3 },
    { 0, 0 },
    { 0xbaf5ab40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1ab44, 3 },
    { 0, 0 },
    { 0xcffdab46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3391ab4e, 4 },
    { 0x95e1ab4f, 3 },
    { 0, 0 },
    { 0x8155ab51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6819ab54, 3 },
    { 0x92adab55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9039ab5a, 4 },
    { 0, 0 },
    { 0xc485ab5c, 3 },
    { 0xd3ddab5d, 3 },
    { 0x5d5ab5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f21ab61, 3 },
    { 0x111ab62, 3 },
    { 0x81c9ab63, 3 },
    { 0xc265ab62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3619ab69, 3 },
    { 0xf4bdab6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7205ab6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d9ab75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8edab78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0dab7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3369ab82, 3 },
    { 0x30dab83, 3 },
    { 0x9461ab84, 4 },
    { 0x5ac1ab82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1ab88, 4 },
    { 0x81cdab88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd45ab8c, 3 },
    { 0x36d9ab8d, 3 },
    { 0x3de9ab8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9699ab91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d1ab94, 3 },
    { 0, 0 },
    { 0xf9c1ab96, 3 },
    { 0x58bdab97, 3 },
    { 0x7a65ab98, 4 },
    { 0xd65ab99, 4 },
    { 0x9655ab9a, 4 },
    { 0xb825ab9b, 3 },
    { 0, 0 },
    { 0x1445ab9d, 3 },
    { 0x3841ab9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b11aba1, 3 },
    { 0x76cdaba1, 3 },
    { 0x385daba3, 3 },
    { 0xffddaba2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb289aba9, 4 },
    { 0x5e85abaa, 4 },
    { 0x3d6dabab, 3 },
    { 0, 0 },
    { 0xd1bdabad, 3 },
    { 0x53edabad, 3 },
    { 0, 0 },
    { 0x5081abb0, 4 },
    { 0xec19abb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2271abb5, 4 },
    { 0xeb81abb5, 3 },
    { 0x9dc5abb7, 4 },
    { 0, 0 },
    { 0xc1f5abb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc989abbc, 3 },
    { 0, 0 },
    { 0x3c49abbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5abc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c5abca, 3 },
    { 0x35adabcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce5abd0, 3 },
    { 0xaeb9abd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1abd4, 4 },
    { 0, 0 },
    { 0x8a75abd6, 4 },
    { 0, 0 },
    { 0x3b1dabd8, 3 },
    { 0, 0 },
    { 0xcee5abda, 3 },
    { 0x8bb5abda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3475abde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b99abe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94dabe8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b95abee, 3 },
    { 0x67f9abef, 3 },
    { 0xe79dabf0, 3 },
    { 0x53dabf0, 4 },
    { 0x7745abf0, 4 },
    { 0xa951abef, 4 },
    { 0xe29abef, 3 },
    { 0x67dabef, 3 },
    { 0x6079abf6, 4 },
    { 0xae71abf0, 4 },
    { 0x8131abf8, 3 },
    { 0x6969abf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21abfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a99ac05, 3 },
    { 0, 0 },
    { 0x77dac07, 3 },
    { 0x16f5ac08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa605ac0b, 3 },
    { 0xd099ac0c, 3 },
    { 0xb71dac0d, 3 },
    { 0, 0 },
    { 0xec95ac0f, 4 },
    { 0x570dac10, 3 },
    { 0xf729ac11, 3 },
    { 0xf805ac11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc9ac16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81ac1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9785ac20, 4 },
    { 0x70a9ac20, 4 },
    { 0xa2fdac22, 3 },
    { 0x9e01ac23, 3 },
    { 0x5749ac23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea5ac28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x773dac2b, 3 },
    { 0xd9e5ac2c, 3 },
    { 0, 0 },
    { 0xa1f5ac2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98edac31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435ac37, 4 },
    { 0x2269ac37, 3 },
    { 0x8125ac39, 3 },
    { 0xfd99ac3a, 3 },
    { 0x1e99ac3a, 3 },
    { 0x33d1ac3a, 4 },
    { 0xebd5ac3a, 3 },
    { 0xe9c9ac3b, 3 },
    { 0x2835ac3f, 3 },
    { 0x6bf1ac3a, 3 },
    { 0x4ea5ac3d, 4 },
    { 0, 0 },
    { 0x8f45ac43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c31ac46, 3 },
    { 0xda41ac47, 3 },
    { 0xac51ac47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e21ac4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321dac57, 3 },
    { 0, 0 },
    { 0x7215ac59, 3 },
    { 0xae35ac59, 4 },
    { 0xa7a1ac5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941ac5f, 4 },
    { 0x39ddac5f, 5 },
    { 0x3f4dac5f, 3 },
    { 0x73d5ac62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd5ac65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7dac68, 4 },
    { 0, 0 },
    { 0x8c7dac6a, 3 },
    { 0xd8d5ac6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x454dac6e, 3 },
    { 0xf999ac6e, 3 },
    { 0x21c1ac70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fedac75, 3 },
    { 0x873dac76, 3 },
    { 0x7765ac76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4965ac7c, 3 },
    { 0, 0 },
    { 0xaf45ac7e, 3 },
    { 0, 0 },
    { 0x77b1ac80, 4 },
    { 0x71cdac81, 5 },
    { 0xc269ac82, 4 },
    { 0x707dac83, 3 },
    { 0x36fdac84, 3 },
    { 0x4e69ac82, 4 },
    { 0xc601ac80, 3 },
    { 0xa0a5ac84, 3 },
    { 0, 0 },
    { 0x6e11ac89, 3 },
    { 0xbcadac8a, 3 },
    { 0xa931ac8b, 4 },
    { 0xb949ac8b, 3 },
    { 0x5bcdac8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8109ac92, 3 },
    { 0x409dac92, 3 },
    { 0x5539ac94, 4 },
    { 0x56f9ac94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c91ac98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b5ac9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25ac9e, 4 },
    { 0x5f39ac9e, 3 },
    { 0x78d9aca0, 3 },
    { 0x614dac9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3199acaa, 3 },
    { 0x5f91acab, 4 },
    { 0xc601acac, 3 },
    { 0xcbfdacad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fdacb4, 3 },
    { 0x8fa1acb5, 3 },
    { 0, 0 },
    { 0x1629acb7, 3 },
    { 0, 0 },
    { 0x1ef5acb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd5acbf, 3 },
    { 0xcd7dacbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde5acc5, 3 },
    { 0, 0 },
    { 0xb7e1acc7, 3 },
    { 0xa315acc8, 3 },
    { 0x1409acc9, 3 },
    { 0, 0 },
    { 0x97c1accb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5acd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5561acd7, 3 },
    { 0, 0 },
    { 0x8e15acd9, 3 },
    { 0x24b1acda, 3 },
    { 0xfc31acd9, 3 },
    { 0x2d9dacdb, 3 },
    { 0x390dacdb, 3 },
    { 0x74f5acde, 4 },
    { 0x57e1acd9, 3 },
    { 0xac91acdf, 4 },
    { 0, 0 },
    { 0x1a05ace2, 3 },
    { 0, 0 },
    { 0xc7c1ace4, 3 },
    { 0x9e2dace5, 3 },
    { 0, 0 },
    { 0x8109ace7, 3 },
    { 0x3bd9ace7, 3 },
    { 0, 0 },
    { 0xeddacea, 4 },
    { 0xb49dacea, 4 },
    { 0x87adacea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7039acef, 3 },
    { 0x29a5acf0, 3 },
    { 0x151acf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a5acf4, 3 },
    { 0xc759acf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f5acfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fddad03, 3 },
    { 0, 0 },
    { 0x4145ad05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ddad08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb9ad0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1051ad14, 3 },
    { 0x3d55ad15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c5ad1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a19ad24, 3 },
    { 0, 0 },
    { 0xebf5ad26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d51ad2b, 3 },
    { 0, 0 },
    { 0xd201ad2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x802dad30, 3 },
    { 0x3df9ad31, 4 },
    { 0xaab1ad30, 4 },
    { 0xfeb5ad33, 3 },
    { 0, 0 },
    { 0xf605ad35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e1ad38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055ad3b, 3 },
    { 0x8601ad3c, 4 },
    { 0xfe25ad3d, 3 },
    { 0x18b9ad3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x686dad41, 3 },
    { 0, 0 },
    { 0xb329ad43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c9ad49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afdad4d, 4 },
    { 0x38edad4d, 3 },
    { 0xfba9ad4e, 3 },
    { 0xd35dad50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b9ad57, 4 },
    { 0, 0 },
    { 0xdc5ad59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1ad5f, 3 },
    { 0x8b69ad60, 4 },
    { 0xed51ad5f, 4 },
    { 0x8251ad60, 3 },
    { 0x3339ad61, 3 },
    { 0x3511ad64, 3 },
    { 0, 0 },
    { 0x6f9ad66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d15ad71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf5ad74, 3 },
    { 0x667dad75, 3 },
    { 0x6aa1ad76, 5 },
    { 0, 0 },
    { 0xb2a1ad78, 3 },
    { 0x27bdad79, 3 },
    { 0x3cddad7a, 3 },
    { 0xcc79ad7b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e31ad7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a9ad83, 3 },
    { 0xc281ad83, 3 },
    { 0, 0 },
    { 0xcbfdad86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd1ad8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x696dad8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c45ad90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b5ad96, 3 },
    { 0xc055ad97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a1ad9b, 3 },
    { 0x66c1ad9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b69ada0, 4 },
    { 0xeff9ada0, 3 },
    { 0xdb75ada0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821dada8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b5adac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d1adb1, 3 },
    { 0x86a1adb2, 3 },
    { 0x4a05adb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409adb7, 3 },
    { 0, 0 },
    { 0xee81adb9, 3 },
    { 0x88d1adba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09adc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5dadc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4dadca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351adcd, 3 },
    { 0x1925adcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3895add6, 4 },
    { 0x2495add7, 4 },
    { 0xc051add8, 3 },
    { 0xaf0dadd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d5addf, 3 },
    { 0x89d1ade0, 3 },
    { 0, 0 },
    { 0x7619ade2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea1adea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x701aded, 3 },
    { 0x82bdadee, 3 },
    { 0x8bcdadef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261adf6, 3 },
    { 0xc1e5adf7, 4 },
    { 0, 0 },
    { 0x29bdadf9, 3 },
    { 0x2529adf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29adfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e19ae01, 3 },
    { 0, 0 },
    { 0x3aadae03, 3 },
    { 0x6dadae04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1dae07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672dae0e, 3 },
    { 0, 0 },
    { 0x90b1ae10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3295ae14, 3 },
    { 0x6ae9ae15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581ae1c, 3 },
    { 0x6d1ae1c, 3 },
    { 0xd459ae1e, 3 },
    { 0, 0 },
    { 0x3e45ae20, 3 },
    { 0, 0 },
    { 0xf215ae22, 3 },
    { 0x8b7dae23, 3 },
    { 0x9c65ae24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4edae29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d31ae2c, 3 },
    { 0x7295ae2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2835ae36, 3 },
    { 0xce45ae36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd61ae3f, 3 },
    { 0x5efdae40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05dae44, 4 },
    { 0x9469ae45, 3 },
    { 0xe101ae46, 3 },
    { 0xb1ddae45, 3 },
    { 0x3969ae48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ddae4b, 3 },
    { 0x74d5ae4c, 4 },
    { 0x6da1ae4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd75ae51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a91ae54, 3 },
    { 0x19a5ae55, 3 },
    { 0x7195ae56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf21ae5a, 3 },
    { 0x36adae5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad9ae5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb9ae61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740dae66, 3 },
    { 0x9839ae67, 4 },
    { 0, 0 },
    { 0x21c1ae69, 4 },
    { 0x5261ae69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee9ae74, 3 },
    { 0, 0 },
    { 0x2e6dae76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e5ae79, 3 },
    { 0, 0 },
    { 0xe869ae7b, 3 },
    { 0, 0 },
    { 0x6badae7d, 3 },
    { 0x5165ae7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf69ae83, 3 },
    { 0xec6dae84, 4 },
    { 0x8179ae85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd955ae8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc049ae96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5dae9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4479aea5, 3 },
    { 0xdd45aea5, 4 },
    { 0xb875aea5, 3 },
    { 0x6671aea8, 3 },
    { 0xb0bdaea8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362daeae, 4 },
    { 0xcbf1aeaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261aeb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2daeb6, 4 },
    { 0x9909aeb6, 4 },
    { 0x3351aeb6, 4 },
    { 0, 0 },
    { 0xd3daeba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa219aebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd315aec3, 4 },
    { 0x74ddaec4, 3 },
    { 0xe401aec4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e09aec8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb1aecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bdaecf, 4 },
    { 0, 0 },
    { 0xcc61aed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126daed8, 3 },
    { 0xdd35aed9, 3 },
    { 0x5ef5aeda, 3 },
    { 0xc8a5aedb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe55aee0, 3 },
    { 0x2ffdaee1, 3 },
    { 0x3fc1aee2, 3 },
    { 0xe0b9aee1, 3 },
    { 0xb705aee2, 4 },
    { 0xa2e1aee1, 3 },
    { 0xba1daee6, 3 },
    { 0xcc1aee7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c35aeea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688daeee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a1aef1, 3 },
    { 0xe795aef1, 3 },
    { 0, 0 },
    { 0xeb45aef4, 3 },
    { 0x4a51aef5, 3 },
    { 0, 0 },
    { 0x68bdaef7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1989aefe, 3 },
    { 0, 0 },
    { 0x4989af00, 4 },
    { 0xdf1af01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381af05, 4 },
    { 0xb175af05, 3 },
    { 0xc761af07, 4 },
    { 0, 0 },
    { 0x7659af09, 3 },
    { 0xaeb9af09, 3 },
    { 0x71e5af0b, 3 },
    { 0xb949af0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4daf17, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605af1a, 3 },
    { 0x7729af1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa009af1e, 4 },
    { 0xacb5af1e, 3 },
    { 0x4635af20, 4 },
    { 0, 0 },
    { 0xdf79af22, 4 },
    { 0, 0 },
    { 0x8935af24, 3 },
    { 0, 0 },
    { 0x53b1af26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3881af29, 3 },
    { 0x7ed9af2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65af2f, 4 },
    { 0xec61af30, 3 },
    { 0xe285af31, 3 },
    { 0x6101af2f, 3 },
    { 0x11edaf30, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d1af36, 4 },
    { 0xadf9af36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf571af3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9af40, 3 },
    { 0x5d81af41, 3 },
    { 0xb599af42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf021af47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcdaf50, 3 },
    { 0xff4daf51, 3 },
    { 0x6ac5af52, 5 },
    { 0, 0 },
    { 0x4861af54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe561af57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde25af5c, 3 },
    { 0x5741af5d, 4 },
    { 0, 0 },
    { 0x6915af5f, 3 },
    { 0x369af60, 3 },
    { 0xb3a9af60, 3 },
    { 0x9c41af62, 3 },
    { 0xeefdaf60, 4 },
    { 0x5ea5af62, 3 },
    { 0xd8c5af5f, 3 },
    { 0, 0 },
    { 0xaff9af67, 3 },
    { 0x8901af67, 3 },
    { 0xa4c9af68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59af6e, 3 },
    { 0, 0 },
    { 0xa845af70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae51af73, 4 },
    { 0x1ce9af73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8321af7c, 3 },
    { 0xae15af7c, 3 },
    { 0x1de9af7e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6889af82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd975af89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe471af91, 4 },
    { 0, 0 },
    { 0xdc41af93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61af96, 3 },
    { 0x78e9af97, 3 },
    { 0x1a35af97, 3 },
    { 0, 0 },
    { 0xc405af9a, 3 },
    { 0, 0 },
    { 0xe1d1af9c, 3 },
    { 0, 0 },
    { 0xa381af9e, 3 },
    { 0, 0 },
    { 0xd869afa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9afa3, 3 },
    { 0x2a11afa3, 3 },
    { 0x6e71afa5, 3 },
    { 0xe1f5afa3, 3 },
    { 0, 0 },
    { 0x2389afa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4155afb2, 3 },
    { 0x2b0dafb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc985afb6, 4 },
    { 0xa3c5afb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f71afba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ddafbf, 3 },
    { 0x8cb9afbf, 4 },
    { 0x61a5afc1, 4 },
    { 0x204dafc1, 3 },
    { 0x7535afc3, 3 },
    { 0x783dafc3, 3 },
    { 0x87c1afc5, 3 },
    { 0x2e21afc5, 3 },
    { 0x5241afc7, 3 },
    { 0x7325afc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x893dafcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b49afd1, 3 },
    { 0, 0 },
    { 0x15fdafd3, 3 },
    { 0x555dafd4, 4 },
    { 0, 0 },
    { 0x61d5afd6, 3 },
    { 0x2e15afd7, 3 },
    { 0, 0 },
    { 0x9c9afd9, 3 },
    { 0x2885afd9, 3 },
    { 0x9fddafdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ddafdf, 4 },
    { 0x5cedafe0, 3 },
    { 0, 0 },
    { 0x6f39afe2, 4 },
    { 0x29d9afe3, 4 },
    { 0x1545afe4, 3 },
    { 0xb4b9afe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cdafed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed15aff7, 4 },
    { 0x420daff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a09affd, 3 },
    { 0xc549affe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa011b001, 3 },
    { 0x6f0db001, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db9b00f, 5 },
    { 0x4ca5b010, 3 },
    { 0x5b91b011, 3 },
    { 0, 0 },
    { 0xb8c1b013, 3 },
    { 0x22c9b014, 3 },
    { 0xa921b015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd1b01a, 3 },
    { 0xf541b01a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bedb01e, 3 },
    { 0x2ce9b01e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f5b022, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9b02c, 3 },
    { 0x8e79b02d, 4 },
    { 0, 0 },
    { 0xb03db02f, 3 },
    { 0, 0 },
    { 0xbcd5b031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9711b037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925b03d, 4 },
    { 0, 0 },
    { 0x8619b03f, 3 },
    { 0x3a39b040, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f1b044, 3 },
    { 0xb169b045, 3 },
    { 0xb57db044, 3 },
    { 0xc60db047, 3 },
    { 0xc7a1b048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd5b050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38db057, 3 },
    { 0xd6d9b058, 3 },
    { 0x1669b059, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0db05d, 3 },
    { 0xb379b05e, 4 },
    { 0x1a55b05e, 3 },
    { 0xdb5b05e, 4 },
    { 0xffedb05e, 3 },
    { 0x5db5b062, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff5b067, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb59b06a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa441b06f, 4 },
    { 0xd669b070, 3 },
    { 0x3da1b071, 3 },
    { 0x2d29b06f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99db076, 3 },
    { 0x4959b077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de5b07c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1b083, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe925b086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2849b08c, 4 },
    { 0xb241b08d, 4 },
    { 0x3f49b08c, 3 },
    { 0x305b08f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb29b092, 3 },
    { 0, 0 },
    { 0x7e85b094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1029b09c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc31b0a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c1b0ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3595b0b0, 3 },
    { 0, 0 },
    { 0xadfdb0b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd999b0b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531db0be, 4 },
    { 0xe2c9b0be, 3 },
    { 0x3885b0bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d9b0c4, 4 },
    { 0x6e4db0c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0db0ca, 3 },
    { 0x1af5b0ca, 3 },
    { 0, 0 },
    { 0xf6d9b0cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f9b0d2, 4 },
    { 0xf7f5b0d3, 3 },
    { 0xfbc1b0d4, 3 },
    { 0x4a05b0d5, 3 },
    { 0x2bfdb0d6, 3 },
    { 0x335b0d7, 3 },
    { 0x3de1b0d2, 3 },
    { 0x9569b0d9, 4 },
    { 0, 0 },
    { 0x1a31b0db, 3 },
    { 0x60a5b0db, 3 },
    { 0xb451b0dc, 4 },
    { 0x6095b0de, 3 },
    { 0, 0 },
    { 0x2971b0e0, 4 },
    { 0, 0 },
    { 0xee1b0e2, 3 },
    { 0xf03db0e2, 3 },
    { 0x7f71b0e4, 3 },
    { 0xd085b0e5, 3 },
    { 0x9ea5b0e4, 4 },
    { 0xe2c9b0e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e9b0ec, 3 },
    { 0, 0 },
    { 0x3a89b0ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa1b0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b0f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5089b0fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b1b0ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa5b107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6db116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d5b119, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0db11c, 3 },
    { 0xfb35b11d, 4 },
    { 0xc321b11d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b5b122, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479b127, 3 },
    { 0x55e9b128, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb12b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a9b130, 3 },
    { 0xf435b130, 4 },
    { 0x77a5b132, 3 },
    { 0xbf69b130, 4 },
    { 0, 0 },
    { 0xafc1b135, 4 },
    { 0x4dadb136, 3 },
    { 0xabe1b137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72edb13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7db140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebdb146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867db14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d5b155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1279b15b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdb15f, 3 },
    { 0xb419b160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1045b163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc139b169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f1b16c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2db170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1b176, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291b179, 3 },
    { 0x70cdb17a, 4 },
    { 0x84e9b17b, 3 },
    { 0x8f6db17b, 3 },
    { 0x9c79b17d, 3 },
    { 0, 0 },
    { 0xc65b17f, 4 },
    { 0x73cdb180, 3 },
    { 0xf709b181, 3 },
    { 0x17d5b182, 3 },
    { 0x3115b180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7305b186, 3 },
    { 0x75c9b187, 5 },
    { 0x8e19b187, 3 },
    { 0xac29b189, 4 },
    { 0, 0 },
    { 0x28d1b18b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffdb18e, 3 },
    { 0xfa9db18f, 3 },
    { 0x5299b18f, 3 },
    { 0x5d5db191, 3 },
    { 0x6cbdb191, 3 },
    { 0x7cf9b193, 3 },
    { 0, 0 },
    { 0x8af5b195, 3 },
    { 0xd155b196, 4 },
    { 0x6d41b197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22db19c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089b1a0, 3 },
    { 0x9e99b1a1, 3 },
    { 0xd005b1a0, 3 },
    { 0, 0 },
    { 0xb001b1a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5b1a7, 4 },
    { 0xd609b1a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f1b1ad, 4 },
    { 0, 0 },
    { 0x33e1b1af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501b1b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d5b1b7, 3 },
    { 0, 0 },
    { 0x8365b1b9, 3 },
    { 0x55edb1ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf965b1c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb985b1c5, 4 },
    { 0x9915b1c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd135b1c9, 3 },
    { 0x56e1b1c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fdb1d4, 4 },
    { 0x6381b1d5, 4 },
    { 0xa28db1d6, 3 },
    { 0x8449b1d5, 3 },
    { 0x16a5b1d7, 4 },
    { 0x3d6db1d8, 3 },
    { 0x58f9b1da, 3 },
    { 0x19b5b1d8, 3 },
    { 0xd019b1dc, 5 },
    { 0, 0 },
    { 0x368db1de, 4 },
    { 0xab9db1df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28db1e2, 4 },
    { 0xc44db1e3, 3 },
    { 0xc0bdb1e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99b1ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9b1f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b9b1f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc095b1fa, 3 },
    { 0xa1b5b1fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01db1fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb719b208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c1b20e, 3 },
    { 0xe881b20e, 3 },
    { 0xe31b210, 4 },
    { 0xa109b20f, 3 },
    { 0x7f75b212, 3 },
    { 0xe7f9b213, 4 },
    { 0x2121b214, 4 },
    { 0xdcb1b214, 3 },
    { 0x6c81b214, 4 },
    { 0x57a9b215, 3 },
    { 0xeb49b218, 3 },
    { 0x7d05b219, 3 },
    { 0, 0 },
    { 0xec31b21b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca9b21f, 3 },
    { 0x5911b220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5355b228, 3 },
    { 0x942db228, 3 },
    { 0x9069b229, 3 },
    { 0, 0 },
    { 0x8d5db22c, 3 },
    { 0, 0 },
    { 0x9519b22e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973db233, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff9b236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db23a, 3 },
    { 0xef1db23b, 4 },
    { 0xdbf5b23b, 3 },
    { 0, 0 },
    { 0x9c75b23e, 4 },
    { 0, 0 },
    { 0x5819b240, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x29adb243, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b24f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9b253, 4 },
    { 0xf679b253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f81b25e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1b261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd341b264, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1edb269, 5 },
    { 0, 0 },
    { 0x7419b26b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd629b270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ddb273, 3 },
    { 0xb975b273, 3 },
    { 0x6935b275, 4 },
    { 0, 0 },
    { 0x43c5b277, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c1b283, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79db286, 4 },
    { 0x2f5db286, 4 },
    { 0x8a1db288, 4 },
    { 0xafb5b289, 4 },
    { 0xf7a5b28a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f75b291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa349b294, 3 },
    { 0, 0 },
    { 0x26b9b296, 5 },
    { 0x84b1b297, 3 },
    { 0xa82db298, 3 },
    { 0x4efdb298, 3 },
    { 0x1ac1b29a, 4 },
    { 0, 0 },
    { 0x27b1b29c, 3 },
    { 0x9f1db29d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1b2a0, 3 },
    { 0x7735b2a1, 3 },
    { 0, 0 },
    { 0x7d11b2a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504db2a9, 3 },
    { 0xaff1b2aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505b2ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a75b2b2, 3 },
    { 0x6e1db2b3, 4 },
    { 0x73bdb2b2, 4 },
    { 0, 0 },
    { 0x1cfdb2b6, 4 },
    { 0x9addb2b7, 3 },
    { 0xb031b2b8, 4 },
    { 0xdc39b2b9, 4 },
    { 0xadb1b2b7, 4 },
    { 0x9335b2bb, 3 },
    { 0x9f25b2bc, 4 },
    { 0x1549b2b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55db2c2, 3 },
    { 0xe575b2c3, 3 },
    { 0x7dddb2c4, 3 },
    { 0xe391b2c4, 3 },
    { 0xb4e5b2c6, 5 },
    { 0x37db2c5, 3 },
    { 0x2e71b2c5, 3 },
    { 0xc789b2c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65b2cc, 3 },
    { 0x388db2cd, 3 },
    { 0x692db2cc, 3 },
    { 0xeb15b2cf, 3 },
    { 0, 0 },
    { 0x6a41b2d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a1b2d7, 3 },
    { 0x2c71b2d7, 3 },
    { 0x2a39b2d8, 3 },
    { 0x6139b2da, 3 },
    { 0, 0 },
    { 0x1a6db2dc, 4 },
    { 0x5719b2dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x452db2e0, 3 },
    { 0x2705b2e1, 3 },
    { 0xa15db2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9441b2f2, 3 },
    { 0xeab9b2f2, 3 },
    { 0xc005b2f4, 3 },
    { 0x3b6db2f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f1b2f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8adb2fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98adb301, 3 },
    { 0x6badb301, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fedb315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc41b319, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe5b31d, 4 },
    { 0, 0 },
    { 0x7ea1b31f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae9b322, 3 },
    { 0, 0 },
    { 0xa205b324, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd01b327, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe89b32a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee5b338, 5 },
    { 0x21d5b338, 3 },
    { 0xa4a1b339, 3 },
    { 0x49c9b339, 4 },
    { 0xb6adb338, 3 },
    { 0x3901b33d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c9b343, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f5b346, 3 },
    { 0xda11b347, 4 },
    { 0xecd5b347, 3 },
    { 0x6c21b349, 3 },
    { 0xbe4db347, 3 },
    { 0, 0 },
    { 0x4b55b34c, 3 },
    { 0x1e99b34d, 3 },
    { 0x671b34e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a5b351, 3 },
    { 0x3fe9b352, 3 },
    { 0xa5b1b353, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea99b359, 3 },
    { 0x4029b35a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb615b364, 3 },
    { 0x4155b364, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb41b368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec1b36e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1db376, 3 },
    { 0xaf25b376, 3 },
    { 0x6b35b376, 3 },
    { 0xd5b9b379, 4 },
    { 0x4691b379, 3 },
    { 0x8585b376, 3 },
    { 0x7fa5b37b, 3 },
    { 0xafb9b37d, 3 },
    { 0xd8bdb377, 3 },
    { 0xe125b37c, 3 },
    { 0xeb69b380, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa5b384, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb181b388, 4 },
    { 0x8d75b389, 3 },
    { 0, 0 },
    { 0x9b51b38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4adb38e, 3 },
    { 0, 0 },
    { 0x505b390, 3 },
    { 0, 0 },
    { 0x168db392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb85b398, 5 },
    { 0x9fa5b398, 3 },
    { 0xad15b39a, 3 },
    { 0x3365b39b, 4 },
    { 0, 0 },
    { 0x885b39d, 3 },
    { 0, 0 },
    { 0xb235b39f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf49b3a3, 4 },
    { 0x7bbdb3a3, 4 },
    { 0xabb5b3a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f39b3a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f15b3af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce1b3b3, 3 },
    { 0x9659b3b3, 3 },
    { 0xf9cdb3b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb549b3b8, 3 },
    { 0, 0 },
    { 0xb6f9b3ba, 3 },
    { 0, 0 },
    { 0x62a9b3bc, 3 },
    { 0, 0 },
    { 0x7da5b3be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e9b3c2, 3 },
    { 0, 0 },
    { 0x25bdb3c4, 3 },
    { 0x5bedb3c5, 3 },
    { 0, 0 },
    { 0xecd5b3c7, 3 },
    { 0xe3c9b3c8, 3 },
    { 0x6d09b3c9, 4 },
    { 0x53a9b3ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x682db3cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61b3d3, 4 },
    { 0, 0 },
    { 0x6749b3d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fdb3d9, 3 },
    { 0xf5db3d9, 3 },
    { 0x871b3db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9b3e2, 3 },
    { 0, 0 },
    { 0x90ddb3e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7db3e7, 3 },
    { 0xd8edb3e7, 3 },
    { 0, 0 },
    { 0xf115b3ea, 3 },
    { 0x9c61b3eb, 3 },
    { 0, 0 },
    { 0xcfc9b3ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d35b3f1, 3 },
    { 0x4d69b3f1, 3 },
    { 0, 0 },
    { 0x3761b3f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb561b3fd, 3 },
    { 0xf121b3fe, 3 },
    { 0x1fbdb3fe, 3 },
    { 0, 0 },
    { 0xb0f9b401, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc609b405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd1b40a, 3 },
    { 0, 0 },
    { 0x1d85b40c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x974db40f, 3 },
    { 0x50d9b40f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3719b415, 3 },
    { 0xf159b415, 3 },
    { 0xbe1db415, 3 },
    { 0x295b418, 4 },
    { 0x76d9b415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d9b41d, 5 },
    { 0xd161b41e, 3 },
    { 0, 0 },
    { 0x6195b420, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48db424, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8edb429, 3 },
    { 0x8845b42a, 3 },
    { 0x2dc1b42b, 3 },
    { 0x8c2db42a, 4 },
    { 0x745b42b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad5b437, 3 },
    { 0, 0 },
    { 0x4e25b439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c71b441, 3 },
    { 0xe4b9b442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80db446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e99b44d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21b457, 4 },
    { 0x6b4db457, 3 },
    { 0, 0 },
    { 0xa975b45a, 3 },
    { 0xd7a1b45a, 3 },
    { 0x9345b45a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965b460, 3 },
    { 0, 0 },
    { 0x1ef9b462, 3 },
    { 0x4631b463, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c1b469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209b46d, 5 },
    { 0, 0 },
    { 0x4fe5b46f, 3 },
    { 0x353db470, 3 },
    { 0x7cc1b471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4549b477, 3 },
    { 0, 0 },
    { 0xecd5b479, 3 },
    { 0x13b1b47a, 4 },
    { 0x7579b47b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2035b480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9341b489, 3 },
    { 0xc521b48a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd59b48d, 4 },
    { 0, 0 },
    { 0x15d9b48f, 4 },
    { 0x6e91b490, 3 },
    { 0xde89b48f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54db497, 4 },
    { 0x319b498, 3 },
    { 0x8db1b499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd221b4a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc1b4ae, 3 },
    { 0x87b5b4ae, 3 },
    { 0x7ca9b4ae, 3 },
    { 0, 0 },
    { 0x3a59b4b2, 3 },
    { 0xccedb4b3, 3 },
    { 0, 0 },
    { 0x7b9db4b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb099b4b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275b4be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c9b4c2, 3 },
    { 0, 0 },
    { 0xad59b4c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84db4c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c39b4cb, 3 },
    { 0, 0 },
    { 0x52b1b4cd, 3 },
    { 0xfbfdb4ce, 4 },
    { 0x1f99b4cf, 3 },
    { 0, 0 },
    { 0xb7fdb4d1, 3 },
    { 0xc975b4d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769b4d5, 5 },
    { 0x245db4d6, 3 },
    { 0x8f81b4d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cedb4db, 3 },
    { 0xbc65b4dc, 4 },
    { 0, 0 },
    { 0x84f9b4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb5b4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f61b4e4, 3 },
    { 0, 0 },
    { 0x1f0db4e6, 3 },
    { 0xb4f9b4e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1415b4ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c5b4f1, 3 },
    { 0x9b35b4f2, 4 },
    { 0x33d9b4f3, 3 },
    { 0x9d9db4f4, 3 },
    { 0, 0 },
    { 0x206db4f6, 4 },
    { 0, 0 },
    { 0xfedb4f8, 3 },
    { 0x1229b4f9, 3 },
    { 0x2a51b4fa, 3 },
    { 0xe215b4fa, 3 },
    { 0x3aadb4fa, 3 },
    { 0x8eadb4fd, 3 },
    { 0xd2b5b4fa, 4 },
    { 0x99a5b4ff, 3 },
    { 0x8101b500, 4 },
    { 0x4f5db501, 4 },
    { 0x8bb5b502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77edb507, 3 },
    { 0xa3db508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25b50e, 3 },
    { 0, 0 },
    { 0x719b510, 3 },
    { 0, 0 },
    { 0x9c9b512, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e9b515, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625b51a, 3 },
    { 0, 0 },
    { 0x4addb51c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd5b51f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf565b525, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7009b529, 3 },
    { 0, 0 },
    { 0xddf1b52b, 3 },
    { 0x1ae9b52b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893db530, 3 },
    { 0, 0 },
    { 0x2665b532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba9b536, 4 },
    { 0, 0 },
    { 0x7da5b538, 3 },
    { 0, 0 },
    { 0x855b53a, 3 },
    { 0, 0 },
    { 0x7c7db53c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa99b541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219b546, 3 },
    { 0x6649b547, 3 },
    { 0x42adb548, 3 },
    { 0x5971b548, 3 },
    { 0xbd69b54a, 3 },
    { 0x7511b54b, 4 },
    { 0, 0 },
    { 0x29f5b54d, 3 },
    { 0x9339b54e, 3 },
    { 0x205b54e, 3 },
    { 0x7471b550, 3 },
    { 0x742db551, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5959b558, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89b55b, 3 },
    { 0x1635b55c, 3 },
    { 0x6f79b55b, 4 },
    { 0x4e0db55e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf61b566, 4 },
    { 0xf589b566, 3 },
    { 0x7281b566, 4 },
    { 0x19ddb567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1b56c, 5 },
    { 0, 0 },
    { 0x43f1b56e, 3 },
    { 0x56f9b56f, 3 },
    { 0x36ddb56f, 4 },
    { 0x4bc1b56e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0db57a, 4 },
    { 0xe279b57b, 3 },
    { 0x5529b57b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7191b583, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9775b588, 3 },
    { 0x45ddb588, 3 },
    { 0, 0 },
    { 0x5f8db58b, 3 },
    { 0x4331b58c, 3 },
    { 0, 0 },
    { 0x3f25b58e, 3 },
    { 0, 0 },
    { 0x137db590, 5 },
    { 0, 0 },
    { 0x251db592, 3 },
    { 0, 0 },
    { 0xb97db594, 4 },
    { 0x79b595, 3 },
    { 0x9191b596, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b5a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1b5af, 4 },
    { 0, 0 },
    { 0x5e65b5b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f1b5b9, 3 },
    { 0, 0 },
    { 0x7f15b5bb, 4 },
    { 0x58d5b5bc, 4 },
    { 0xec39b5bd, 3 },
    { 0xe05db5bc, 4 },
    { 0, 0 },
    { 0xc539b5c0, 4 },
    { 0xf021b5c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe9b5c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca41b5c7, 5 },
    { 0, 0 },
    { 0x8459b5c9, 3 },
    { 0xd6b1b5ca, 3 },
    { 0xf3cdb5cb, 3 },
    { 0xc529b5cc, 4 },
    { 0xc7ddb5cb, 3 },
    { 0, 0 },
    { 0x35d5b5cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa851b5d2, 4 },
    { 0, 0 },
    { 0xf80db5d4, 3 },
    { 0, 0 },
    { 0x30f5b5d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc809b5de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a9b5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d81b5e8, 3 },
    { 0x979db5e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76db5ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba95b5f2, 3 },
    { 0, 0 },
    { 0x37a1b5f4, 4 },
    { 0, 0 },
    { 0x5f21b5f6, 3 },
    { 0xbb0db5f6, 4 },
    { 0x4e35b5f8, 3 },
    { 0x7341b5f9, 3 },
    { 0x39d9b5fa, 5 },
    { 0x88b1b5f9, 4 },
    { 0x5e0db5fc, 3 },
    { 0x6035b5fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f9b600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5079b608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c71b60e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4761b611, 3 },
    { 0xfc31b611, 3 },
    { 0x88fdb612, 4 },
    { 0xe205b614, 3 },
    { 0xcc1b614, 3 },
    { 0xb29b616, 3 },
    { 0, 0 },
    { 0xd4db618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f9b621, 3 },
    { 0, 0 },
    { 0xebc5b623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef71b628, 3 },
    { 0xa03db629, 3 },
    { 0x3075b628, 3 },
    { 0x2f39b62b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ddb62e, 5 },
    { 0, 0 },
    { 0x6df9b630, 4 },
    { 0x4369b631, 3 },
    { 0xb12db632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf725b637, 3 },
    { 0xeb99b637, 4 },
    { 0x9025b639, 3 },
    { 0xba0db63a, 3 },
    { 0xfa95b63a, 3 },
    { 0xdc19b63c, 3 },
    { 0xa3d9b638, 3 },
    { 0xefadb63e, 4 },
    { 0x72edb63f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddb644, 3 },
    { 0, 0 },
    { 0x7b9b646, 3 },
    { 0x25e5b646, 3 },
    { 0xfc65b648, 3 },
    { 0xe5e9b648, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x825b64c, 5 },
    { 0x8029b64d, 3 },
    { 0, 0 },
    { 0x49e5b64f, 3 },
    { 0, 0 },
    { 0x27edb651, 4 },
    { 0xcefdb651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd5b655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6121b658, 3 },
    { 0, 0 },
    { 0xa12db65a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a1b65f, 3 },
    { 0, 0 },
    { 0x73d5b661, 4 },
    { 0x7fb9b662, 4 },
    { 0xb17db663, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f25b667, 3 },
    { 0x3895b668, 4 },
    { 0x52adb667, 3 },
    { 0xfad1b66a, 3 },
    { 0xf525b66a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa131b670, 3 },
    { 0x9d61b670, 4 },
    { 0, 0 },
    { 0x7ea9b673, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdb677, 3 },
    { 0x2741b678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1db67c, 3 },
    { 0x3131b67d, 3 },
    { 0, 0 },
    { 0x9045b67f, 3 },
    { 0xaf05b680, 3 },
    { 0xfbddb681, 3 },
    { 0xf1e1b681, 3 },
    { 0, 0 },
    { 0xb1edb684, 3 },
    { 0, 0 },
    { 0x80a9b686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f9b68a, 3 },
    { 0x3261b68b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a9b68f, 3 },
    { 0, 0 },
    { 0x4689b691, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d31b695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917db699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac29b6a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa25b6a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc645b6aa, 3 },
    { 0xc9edb6ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791b6ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7219b6b6, 3 },
    { 0xe40db6b7, 3 },
    { 0x8f5db6b7, 3 },
    { 0x60d1b6b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d5b6bc, 4 },
    { 0, 0 },
    { 0xbf5b6be, 3 },
    { 0, 0 },
    { 0x9cddb6c0, 3 },
    { 0xa235b6c1, 3 },
    { 0, 0 },
    { 0xc8a9b6c3, 4 },
    { 0xf89b6c4, 4 },
    { 0, 0 },
    { 0x9741b6c6, 4 },
    { 0xf259b6c7, 3 },
    { 0, 0 },
    { 0xd85db6c9, 4 },
    { 0xfac5b6ca, 3 },
    { 0, 0 },
    { 0x3b65b6cc, 3 },
    { 0x31d5b6cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef1b6d1, 4 },
    { 0, 0 },
    { 0xbd15b6d3, 3 },
    { 0x3b45b6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20edb6da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5db6de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3281b6e1, 4 },
    { 0x5eb1b6e1, 3 },
    { 0x5fd1b6e3, 3 },
    { 0xb931b6e4, 4 },
    { 0, 0 },
    { 0x6549b6e6, 3 },
    { 0x9ecdb6e7, 4 },
    { 0x3fc1b6e8, 3 },
    { 0x1f51b6e8, 3 },
    { 0xf11db6e6, 3 },
    { 0xce69b6e7, 4 },
    { 0x7479b6ec, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb59b6ef, 3 },
    { 0, 0 },
    { 0x255b6f1, 4 },
    { 0x42a5b6f2, 4 },
    { 0, 0 },
    { 0x2d75b6f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41b6f7, 3 },
    { 0x4a51b6f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a1b6fe, 3 },
    { 0x26b5b6fe, 3 },
    { 0, 0 },
    { 0x43adb701, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345b708, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x201db70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e9b70e, 3 },
    { 0xca1b70f, 3 },
    { 0x2f99b710, 3 },
    { 0, 0 },
    { 0xbd9b712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b9b716, 3 },
    { 0, 0 },
    { 0x10a5b718, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc275b71c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f5b71f, 3 },
    { 0, 0 },
    { 0x68d1b721, 3 },
    { 0x7709b721, 4 },
    { 0, 0 },
    { 0x20a9b724, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f5b728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a59b72c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x728db72f, 3 },
    { 0, 0 },
    { 0x8199b731, 3 },
    { 0x9279b732, 5 },
    { 0, 0 },
    { 0xc195b734, 3 },
    { 0x806db735, 4 },
    { 0x9439b734, 3 },
    { 0xca59b737, 3 },
    { 0xe49b737, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3811b73b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3159b73e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1db743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b75b74a, 3 },
    { 0x78b5b74b, 3 },
    { 0x1c39b74c, 3 },
    { 0, 0 },
    { 0x281db74e, 3 },
    { 0x859db74e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b25b753, 3 },
    { 0, 0 },
    { 0xe0db755, 3 },
    { 0x1ff1b756, 3 },
    { 0x8255b757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d5b75b, 4 },
    { 0xe341b75b, 3 },
    { 0x8ac9b75d, 4 },
    { 0xe325b75e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a95b761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6db768, 3 },
    { 0xe561b769, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b9b773, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7db777, 3 },
    { 0xe391b778, 3 },
    { 0x22a5b777, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab1b77c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c51b780, 4 },
    { 0xddd1b781, 3 },
    { 0xa3f9b781, 3 },
    { 0x5cd9b782, 3 },
    { 0xe8d5b782, 3 },
    { 0x289db781, 4 },
    { 0xdc1db783, 3 },
    { 0, 0 },
    { 0xe681b788, 3 },
    { 0xe375b789, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7621b78d, 4 },
    { 0, 0 },
    { 0xafadb78f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc869b793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7401b796, 3 },
    { 0, 0 },
    { 0xd69db798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c05b7a1, 4 },
    { 0x1039b7a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77adb7a5, 4 },
    { 0xb981b7a5, 3 },
    { 0, 0 },
    { 0xe369b7a8, 3 },
    { 0x5f95b7a9, 4 },
    { 0x9551b7a8, 3 },
    { 0x238db7ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a1b7af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761b7b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf709b7ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x491b7bd, 3 },
    { 0x1a2db7be, 3 },
    { 0x6089b7bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c5b7c2, 3 },
    { 0x6e81b7c3, 4 },
    { 0x9d7db7c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b1b7ca, 3 },
    { 0x3f41b7cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6871b7d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b1b7d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf969b7da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3305b7e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffdb7e5, 3 },
    { 0xd099b7e6, 4 },
    { 0xf8d9b7e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cadb7ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5185b7ed, 3 },
    { 0x9bc1b7ed, 4 },
    { 0x2c35b7ef, 3 },
    { 0x397db7ed, 3 },
    { 0, 0 },
    { 0x9709b7f2, 3 },
    { 0xeb59b7f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4db7fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28db800, 3 },
    { 0x5875b800, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5b808, 3 },
    { 0, 0 },
    { 0x2545b80a, 3 },
    { 0, 0 },
    { 0x87edb80c, 3 },
    { 0xfe79b80d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0db812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811b819, 3 },
    { 0x4765b81a, 3 },
    { 0, 0 },
    { 0xbad1b81c, 4 },
    { 0xdddb81d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de5b820, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe039b826, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb9b82b, 4 },
    { 0xb379b82c, 3 },
    { 0, 0 },
    { 0xe821b82e, 3 },
    { 0x6b6db82e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1b832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf985b837, 3 },
    { 0, 0 },
    { 0xc349b839, 3 },
    { 0, 0 },
    { 0xb34db83b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d11b83e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d65b843, 3 },
    { 0, 0 },
    { 0xd66db845, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf15b849, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf951b84c, 3 },
    { 0, 0 },
    { 0x4a41b84e, 3 },
    { 0x7d79b84f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3129b852, 3 },
    { 0, 0 },
    { 0xcabdb854, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf81b85a, 3 },
    { 0x1aadb85a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x519b85e, 3 },
    { 0x20cdb85f, 4 },
    { 0x411db85e, 3 },
    { 0x60fdb861, 3 },
    { 0xa5edb85e, 4 },
    { 0x3ff5b860, 3 },
    { 0x656db862, 4 },
    { 0x15f9b865, 4 },
    { 0x4b7db865, 3 },
    { 0, 0 },
    { 0x1aadb868, 3 },
    { 0xff55b869, 3 },
    { 0xe939b869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115b86d, 3 },
    { 0x1b75b86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7025b872, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac59b875, 4 },
    { 0xc9f9b876, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c75b87a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccdb87d, 3 },
    { 0xf889b87e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8929b889, 3 },
    { 0x5085b889, 3 },
    { 0x98e9b88b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8129b88f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a9b893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8edb897, 3 },
    { 0, 0 },
    { 0xb2b9b899, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa479b89c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543db8a0, 3 },
    { 0, 0 },
    { 0x83f5b8a2, 3 },
    { 0xcfadb8a3, 3 },
    { 0x7479b8a3, 3 },
    { 0, 0 },
    { 0x7205b8a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f1b8aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa55b8ae, 3 },
    { 0x20c1b8af, 3 },
    { 0, 0 },
    { 0xdde5b8b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9b8b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba5b8ba, 3 },
    { 0x2f31b8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bdb8be, 3 },
    { 0x11adb8bf, 3 },
    { 0x6729b8c0, 3 },
    { 0, 0 },
    { 0xfc1db8c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1b8ca, 3 },
    { 0x77c1b8ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f1b8d0, 4 },
    { 0, 0 },
    { 0xb05b8d2, 4 },
    { 0xc209b8d2, 3 },
    { 0x165b8d4, 3 },
    { 0x717db8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea1b8d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51adb8dd, 3 },
    { 0, 0 },
    { 0x8d09b8df, 3 },
    { 0xdf69b8e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3951b8e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a1b8e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ddb8ed, 4 },
    { 0xcd01b8ed, 5 },
    { 0x6135b8ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1699b8f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f89b8fa, 3 },
    { 0xece5b8fa, 3 },
    { 0xd7c1b8fc, 3 },
    { 0x8559b8fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ddb903, 3 },
    { 0, 0 },
    { 0x4701b905, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4db908, 3 },
    { 0x21db909, 3 },
    { 0x4401b909, 4 },
    { 0x3271b90b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355b90f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1db913, 3 },
    { 0xff91b913, 3 },
    { 0xdfd5b915, 3 },
    { 0x73d5b913, 4 },
    { 0xca91b917, 3 },
    { 0x9b25b918, 3 },
    { 0xb179b915, 3 },
    { 0, 0 },
    { 0xe151b91b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99b91e, 3 },
    { 0xd1f1b91f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21db922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1925b926, 3 },
    { 0, 0 },
    { 0x2da9b928, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bdb92b, 3 },
    { 0x5a1b92c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9075b934, 3 },
    { 0xd79b935, 4 },
    { 0, 0 },
    { 0x5c49b937, 4 },
    { 0, 0 },
    { 0xc395b939, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2181b93c, 3 },
    { 0, 0 },
    { 0x1f5db93e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f1b942, 4 },
    { 0x4781b943, 3 },
    { 0x8171b944, 3 },
    { 0x64a1b944, 3 },
    { 0x8e21b942, 3 },
    { 0xe075b947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edb94a, 3 },
    { 0xb269b94a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc171b950, 4 },
    { 0, 0 },
    { 0x6e45b952, 3 },
    { 0xe155b953, 3 },
    { 0xf8a1b952, 4 },
    { 0x1309b954, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84adb95b, 3 },
    { 0xadc9b95c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df1b963, 4 },
    { 0x65f5b964, 3 },
    { 0xafd9b963, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf9b971, 3 },
    { 0x9dbdb972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b9b976, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9b979, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3db97f, 3 },
    { 0x2011b97f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b98a, 3 },
    { 0, 0 },
    { 0xbb5db98c, 3 },
    { 0x1639b98d, 4 },
    { 0xa15b98e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd125b997, 3 },
    { 0x66adb998, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9adb9a1, 4 },
    { 0x179db9a2, 3 },
    { 0x7c21b9a2, 3 },
    { 0, 0 },
    { 0xc5a5b9a5, 3 },
    { 0, 0 },
    { 0x6069b9a7, 5 },
    { 0, 0 },
    { 0xb901b9a9, 3 },
    { 0x3859b9a9, 3 },
    { 0, 0 },
    { 0xa491b9ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa261b9b0, 4 },
    { 0, 0 },
    { 0x824db9b2, 3 },
    { 0, 0 },
    { 0x31b5b9b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d5b9b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7191b9bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75b9c2, 3 },
    { 0x5529b9c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b1b9cd, 3 },
    { 0xbf41b9ce, 3 },
    { 0, 0 },
    { 0x27b5b9d0, 4 },
    { 0xf5edb9d1, 4 },
    { 0x64db9d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf141b9da, 3 },
    { 0, 0 },
    { 0x7ab5b9dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6db9e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1b9e7, 4 },
    { 0x6be9b9e7, 3 },
    { 0xbff9b9e9, 3 },
    { 0xcde1b9ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b5b9ee, 3 },
    { 0x9941b9ef, 4 },
    { 0xf535b9f0, 3 },
    { 0x25cdb9ef, 3 },
    { 0x4f8db9ef, 3 },
    { 0xf335b9ef, 3 },
    { 0x8489b9f4, 3 },
    { 0xf3f1b9f4, 3 },
    { 0x1035b9f6, 4 },
    { 0, 0 },
    { 0xc71db9f8, 3 },
    { 0xe045b9f9, 3 },
    { 0xea59b9fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c5b9fe, 3 },
    { 0x2c9b9ff, 3 },
    { 0xfe0dba00, 4 },
    { 0x8b29ba01, 3 },
    { 0x2ac5ba02, 4 },
    { 0xc43dba02, 3 },
    { 0xb611ba02, 3 },
    { 0x5315b9ff, 4 },
    { 0x782dba00, 4 },
    { 0x77adba07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91ba0c, 3 },
    { 0x7cf5ba0d, 3 },
    { 0xd579ba0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46edba12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195ba15, 4 },
    { 0, 0 },
    { 0xbbf5ba17, 3 },
    { 0x223dba18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479ba24, 3 },
    { 0x9705ba25, 3 },
    { 0xb29dba26, 3 },
    { 0x53a1ba24, 3 },
    { 0x6581ba25, 4 },
    { 0, 0 },
    { 0xc4a5ba2a, 3 },
    { 0xe475ba2a, 3 },
    { 0x2505ba2c, 3 },
    { 0x450dba2d, 3 },
    { 0xa15dba2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8dba32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c69ba35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6dba39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a9ba40, 3 },
    { 0, 0 },
    { 0x57ddba42, 4 },
    { 0x6441ba43, 3 },
    { 0xc27dba43, 3 },
    { 0x3c31ba43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb69ba48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bdba4c, 3 },
    { 0xe6f9ba4c, 3 },
    { 0x60f9ba4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6dba54, 3 },
    { 0x7df9ba55, 4 },
    { 0, 0 },
    { 0xf19dba57, 3 },
    { 0, 0 },
    { 0x7a01ba59, 3 },
    { 0, 0 },
    { 0xcfe5ba5b, 4 },
    { 0, 0 },
    { 0x7835ba5d, 3 },
    { 0, 0 },
    { 0x6b91ba5f, 3 },
    { 0x7c21ba60, 3 },
    { 0x7199ba5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde05ba64, 3 },
    { 0xd455ba64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefadba68, 3 },
    { 0x3979ba68, 3 },
    { 0xc011ba6a, 3 },
    { 0x972dba6a, 3 },
    { 0xcfb9ba6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f1ba6f, 4 },
    { 0x862dba6f, 3 },
    { 0xadd9ba6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679dba75, 3 },
    { 0, 0 },
    { 0x5ff9ba77, 3 },
    { 0x5df9ba78, 3 },
    { 0x7b19ba77, 3 },
    { 0x5495ba7a, 3 },
    { 0x66cdba7b, 3 },
    { 0xbd11ba7c, 3 },
    { 0x971dba7d, 3 },
    { 0x4c19ba7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x819dba81, 3 },
    { 0, 0 },
    { 0xae19ba83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1ba86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c29ba8a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1325ba8e, 3 },
    { 0xea19ba8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1ba92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ba96, 3 },
    { 0x456dba97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5ba9a, 3 },
    { 0xd855ba9b, 3 },
    { 0xb8d9ba9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9baa1, 4 },
    { 0xc8b9baa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96dbaa7, 3 },
    { 0x8059baa8, 3 },
    { 0x23fdbaa7, 3 },
    { 0xee19baa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c1bab3, 3 },
    { 0x3fa1bab4, 4 },
    { 0xf06dbab4, 3 },
    { 0xabb5bab4, 3 },
    { 0x7375bab7, 3 },
    { 0, 0 },
    { 0x1569bab9, 3 },
    { 0xf5c5baba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706dbac1, 3 },
    { 0x137dbac2, 3 },
    { 0, 0 },
    { 0x88f9bac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b5bac8, 4 },
    { 0, 0 },
    { 0x26c5baca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44dbace, 3 },
    { 0xa27dbace, 3 },
    { 0x96bdbacf, 3 },
    { 0xeae1bacf, 3 },
    { 0xd199bace, 3 },
    { 0x5a4dbad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3dbad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bad9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf019badf, 3 },
    { 0xe42dbadf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f51bae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f1bae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31baea, 4 },
    { 0, 0 },
    { 0x4d09baec, 4 },
    { 0x8aa5baed, 3 },
    { 0xa6d1baee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45baf1, 3 },
    { 0, 0 },
    { 0x3c15baf3, 4 },
    { 0x8dc9baf4, 4 },
    { 0x1f01baf5, 3 },
    { 0xdd49baf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123dbb00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6dbb03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6dbb07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e9bb0d, 5 },
    { 0xf7bdbb0d, 3 },
    { 0xcdc5bb0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddbb12, 3 },
    { 0, 0 },
    { 0xeaf9bb14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aadbb17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe061bb20, 3 },
    { 0x9109bb20, 3 },
    { 0, 0 },
    { 0xf635bb23, 3 },
    { 0, 0 },
    { 0x1b4dbb25, 3 },
    { 0x4f81bb26, 3 },
    { 0xb18dbb25, 5 },
    { 0, 0 },
    { 0x6c21bb29, 3 },
    { 0x6085bb29, 3 },
    { 0xabb9bb2b, 3 },
    { 0x42f9bb2b, 4 },
    { 0xed51bb2c, 3 },
    { 0xb575bb2e, 3 },
    { 0xa579bb2e, 3 },
    { 0x4165bb2e, 4 },
    { 0xf235bb31, 4 },
    { 0xd049bb32, 3 },
    { 0x999bb2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3dbb36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2905bb39, 4 },
    { 0x5805bb3a, 4 },
    { 0xf481bb39, 3 },
    { 0xa621bb3c, 3 },
    { 0x3f1bb3d, 3 },
    { 0x93b5bb3c, 3 },
    { 0, 0 },
    { 0x9b49bb40, 3 },
    { 0, 0 },
    { 0xf545bb42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf605bb4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f49bb50, 4 },
    { 0x8ae9bb51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c19bb55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x414dbb58, 3 },
    { 0x90c5bb59, 5 },
    { 0, 0 },
    { 0xf045bb5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6dbb60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b1bb64, 3 },
    { 0, 0 },
    { 0x96d9bb66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac5bb6a, 4 },
    { 0x4475bb6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff1bb6e, 3 },
    { 0, 0 },
    { 0x59adbb70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8249bb75, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7dbb78, 3 },
    { 0x805bb78, 3 },
    { 0x1f2dbb7a, 3 },
    { 0x8751bb79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc415bb81, 3 },
    { 0, 0 },
    { 0x8c4dbb83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x621dbb86, 4 },
    { 0, 0 },
    { 0x85d5bb88, 3 },
    { 0, 0 },
    { 0x620dbb8a, 4 },
    { 0x7549bb8a, 4 },
    { 0, 0 },
    { 0x2e91bb8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8dbb91, 4 },
    { 0xeb89bb92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0dbb96, 3 },
    { 0x56c1bb97, 3 },
    { 0xe305bb98, 4 },
    { 0, 0 },
    { 0x969dbb9a, 3 },
    { 0x3fb5bb9b, 4 },
    { 0x7af9bb9a, 4 },
    { 0xa499bb9a, 3 },
    { 0xafa5bb9e, 4 },
    { 0x9a99bb9e, 3 },
    { 0xcf4dbba0, 3 },
    { 0x5b19bb9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb41bba5, 3 },
    { 0x9f69bba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad21bba9, 4 },
    { 0x299bbaa, 3 },
    { 0x9b89bbab, 3 },
    { 0x30e5bbac, 4 },
    { 0xafc1bbad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf255bbb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89adbbb3, 3 },
    { 0x1531bbb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa871bbb7, 3 },
    { 0, 0 },
    { 0x7ff5bbb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad01bbbc, 3 },
    { 0x8f5bbbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x499dbbc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9bbc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c1bbc9, 4 },
    { 0x1159bbc9, 3 },
    { 0, 0 },
    { 0x425dbbcc, 3 },
    { 0x81b9bbcd, 4 },
    { 0xf799bbce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dbbd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3655bbd9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa995bbdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc71bbe2, 3 },
    { 0x623dbbe3, 3 },
    { 0x9419bbe3, 3 },
    { 0, 0 },
    { 0x7f25bbe6, 4 },
    { 0x416dbbe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9bbea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a1bbee, 5 },
    { 0x7e9dbbef, 3 },
    { 0xe381bbef, 3 },
    { 0x38f1bbf1, 3 },
    { 0, 0 },
    { 0xe81bbf3, 3 },
    { 0x6fc5bbf4, 3 },
    { 0x2b5dbbf5, 4 },
    { 0x149bbf4, 3 },
    { 0x5701bbf7, 3 },
    { 0x87c9bbf6, 4 },
    { 0, 0 },
    { 0x9e25bbfa, 4 },
    { 0, 0 },
    { 0x1f25bbfc, 3 },
    { 0x4729bbfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961bc01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc831bc04, 3 },
    { 0x99f1bc04, 3 },
    { 0xb3c1bc04, 4 },
    { 0xbf25bc05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc31bc0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a9bc0d, 3 },
    { 0xefa1bc0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd735bc17, 3 },
    { 0x6e01bc18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6759bc21, 3 },
    { 0x4f59bc21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11bc28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0dbc30, 3 },
    { 0xbf81bc31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcdbc34, 3 },
    { 0x4eadbc35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a31bc38, 4 },
    { 0x184dbc39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62edbc43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e85bc49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1dbc4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5bc5b, 4 },
    { 0x9b11bc5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd911bc5f, 3 },
    { 0, 0 },
    { 0x5c5dbc61, 3 },
    { 0, 0 },
    { 0xbd55bc63, 3 },
    { 0, 0 },
    { 0xdaddbc65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176dbc6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa981bc6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x939bc70, 4 },
    { 0x371bc70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad51bc74, 3 },
    { 0x4d39bc75, 3 },
    { 0x2c31bc76, 4 },
    { 0, 0 },
    { 0x8e89bc78, 3 },
    { 0xdee5bc79, 3 },
    { 0, 0 },
    { 0x6685bc7b, 3 },
    { 0xc495bc7c, 3 },
    { 0xb359bc7d, 3 },
    { 0, 0 },
    { 0x69cdbc7f, 3 },
    { 0x4b7dbc80, 3 },
    { 0xba55bc81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828dbc8b, 4 },
    { 0x2eb9bc8c, 3 },
    { 0, 0 },
    { 0x6d01bc8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcdbc91, 4 },
    { 0x7691bc91, 3 },
    { 0x26f5bc92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9bc96, 3 },
    { 0xfe15bc96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b5bc9a, 3 },
    { 0x9aa9bc9b, 3 },
    { 0x5ef1bc9b, 3 },
    { 0, 0 },
    { 0xafddbc9e, 3 },
    { 0, 0 },
    { 0x16d5bca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf285bca8, 3 },
    { 0x2ae1bca8, 3 },
    { 0xc45dbca8, 5 },
    { 0, 0 },
    { 0x19a1bcac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdbcb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6435bcb8, 3 },
    { 0, 0 },
    { 0x8a29bcba, 3 },
    { 0xd325bcbb, 3 },
    { 0xc865bcbc, 3 },
    { 0x35cdbcbd, 3 },
    { 0x249bcbe, 3 },
    { 0xa565bcbf, 3 },
    { 0xcddbcc0, 5 },
    { 0x9279bcbf, 3 },
    { 0x85d5bcbf, 3 },
    { 0x9ab5bcc3, 4 },
    { 0xf559bcbe, 4 },
    { 0x8dc1bcc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1bcc8, 4 },
    { 0, 0 },
    { 0xf599bcca, 3 },
    { 0, 0 },
    { 0x8f6dbccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2015bcd7, 4 },
    { 0xc841bcd7, 3 },
    { 0xc391bcd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25dbce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1475bce6, 3 },
    { 0x3ef5bce7, 3 },
    { 0, 0 },
    { 0x6e91bce9, 3 },
    { 0, 0 },
    { 0xe9cdbceb, 4 },
    { 0xe39bcec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9bcef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9bcf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56edbcf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee29bcfb, 3 },
    { 0xb01dbcfc, 3 },
    { 0x5ed1bcfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b9bd07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5bd0a, 4 },
    { 0x3db9bd0b, 3 },
    { 0xe219bd0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bdbd10, 4 },
    { 0x3891bd11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9bd1b, 3 },
    { 0x5ac5bd1b, 3 },
    { 0x2785bd1d, 4 },
    { 0x929dbd1c, 3 },
    { 0x3e8dbd1f, 3 },
    { 0x58bdbd1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea1bd25, 3 },
    { 0, 0 },
    { 0xe915bd27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60adbd2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e1bd2d, 3 },
    { 0, 0 },
    { 0xce59bd2f, 3 },
    { 0x4ccdbd30, 3 },
    { 0xfeedbd30, 3 },
    { 0x4921bd31, 4 },
    { 0x1a71bd33, 3 },
    { 0xb9c9bd30, 5 },
    { 0x9ac9bd35, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349bd3b, 3 },
    { 0xe189bd3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf41bd44, 3 },
    { 0, 0 },
    { 0x5089bd46, 3 },
    { 0xeb85bd46, 4 },
    { 0x276dbd48, 3 },
    { 0x20a5bd49, 3 },
    { 0, 0 },
    { 0x5c61bd4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a59bd4f, 3 },
    { 0x3525bd4f, 3 },
    { 0x7519bd51, 3 },
    { 0x344dbd4f, 3 },
    { 0xb919bd50, 3 },
    { 0x3475bd54, 4 },
    { 0xbdfdbd4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b41bd60, 3 },
    { 0x62bdbd61, 4 },
    { 0xe51dbd62, 4 },
    { 0x2525bd61, 4 },
    { 0x1401bd64, 4 },
    { 0, 0 },
    { 0x25ddbd66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb479bd6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45bd76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acdbd7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc029bd7d, 3 },
    { 0xb0edbd7e, 3 },
    { 0xa05bd7d, 3 },
    { 0, 0 },
    { 0xfed1bd81, 3 },
    { 0xe3d1bd81, 3 },
    { 0x5261bd83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24adbd87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5bd8a, 3 },
    { 0, 0 },
    { 0x56a1bd8c, 3 },
    { 0x9c69bd8d, 3 },
    { 0x5a3dbd8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5bd91, 4 },
    { 0, 0 },
    { 0x38adbd93, 3 },
    { 0x2109bd94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215bd97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b1bd9c, 4 },
    { 0x3a9dbd9d, 4 },
    { 0, 0 },
    { 0xdc3dbd9f, 3 },
    { 0xb5c5bd9f, 3 },
    { 0x74c1bda1, 5 },
    { 0xcf45bda1, 3 },
    { 0xccb1bda0, 3 },
    { 0x7b9bda3, 3 },
    { 0x5955bd9f, 3 },
    { 0xfb8dbda6, 3 },
    { 0xce8dbda7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc5bdaa, 3 },
    { 0x905bdab, 3 },
    { 0xaab1bdac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669bdaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31bdb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61bdbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b9bdbf, 3 },
    { 0x6c15bdc0, 3 },
    { 0, 0 },
    { 0x19c1bdc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68dbdc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecadbdc9, 4 },
    { 0x3279bdc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc849bdcd, 3 },
    { 0, 0 },
    { 0x43edbdcf, 3 },
    { 0, 0 },
    { 0x9a69bdd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11bdd4, 3 },
    { 0xd689bdd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdbddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fdbddf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab5bde2, 3 },
    { 0x1bdbde2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f1bde8, 3 },
    { 0, 0 },
    { 0xca0dbdea, 3 },
    { 0xa5bdbdea, 4 },
    { 0x7315bdec, 3 },
    { 0, 0 },
    { 0xa2c1bdee, 4 },
    { 0, 0 },
    { 0x79adbdf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631bdfd, 4 },
    { 0, 0 },
    { 0x598dbdff, 4 },
    { 0xcd05bdff, 3 },
    { 0x65fdbdff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5dbe09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe055be12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc69be18, 3 },
    { 0xd2a9be19, 3 },
    { 0, 0 },
    { 0x4eedbe1b, 5 },
    { 0x326dbe1c, 3 },
    { 0, 0 },
    { 0x6731be1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27adbe2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9795be36, 3 },
    { 0xbe8dbe36, 3 },
    { 0xcf05be38, 3 },
    { 0xd545be37, 3 },
    { 0x63c5be3a, 4 },
    { 0x70bdbe3b, 3 },
    { 0x3b3dbe38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb009be40, 4 },
    { 0xdafdbe41, 3 },
    { 0x92b5be42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be1be46, 4 },
    { 0, 0 },
    { 0xd4e9be48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef5be53, 3 },
    { 0x9a6dbe54, 3 },
    { 0, 0 },
    { 0x858dbe56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb1be5a, 4 },
    { 0xba31be5a, 3 },
    { 0xccf5be5c, 4 },
    { 0x4bf5be5d, 3 },
    { 0x9959be5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe21be61, 3 },
    { 0, 0 },
    { 0xa359be63, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ddbe6c, 3 },
    { 0x7955be6d, 4 },
    { 0xf261be6c, 3 },
    { 0xa32dbe6f, 5 },
    { 0, 0 },
    { 0x6fd1be71, 5 },
    { 0, 0 },
    { 0x2da9be73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e1be76, 3 },
    { 0xd7bdbe77, 3 },
    { 0x9711be77, 3 },
    { 0x15dbe76, 5 },
    { 0, 0 },
    { 0x7b65be7b, 4 },
    { 0x98bdbe7c, 5 },
    { 0xab69be7c, 3 },
    { 0xe6f9be7e, 4 },
    { 0x7a61be7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1be82, 3 },
    { 0, 0 },
    { 0xac85be84, 3 },
    { 0, 0 },
    { 0xe395be86, 4 },
    { 0x34e5be86, 4 },
    { 0x901be88, 4 },
    { 0xd371be89, 3 },
    { 0xe429be8a, 3 },
    { 0, 0 },
    { 0x4dc9be8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee9be90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5dbe96, 3 },
    { 0xe899be97, 3 },
    { 0xe355be98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xded1be9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193dbea1, 3 },
    { 0, 0 },
    { 0xda51bea3, 3 },
    { 0xb165bea4, 3 },
    { 0xac09bea4, 4 },
    { 0, 0 },
    { 0xadb5bea7, 3 },
    { 0xc6f9bea7, 3 },
    { 0, 0 },
    { 0x50c5beaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dbeae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211beb1, 3 },
    { 0, 0 },
    { 0x85c1beb3, 3 },
    { 0, 0 },
    { 0x4439beb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5605beb8, 3 },
    { 0x185dbeb9, 3 },
    { 0, 0 },
    { 0x64f1bebb, 3 },
    { 0, 0 },
    { 0x35e5bebd, 3 },
    { 0xbdb5bebd, 3 },
    { 0x71b9bebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89bec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1955bec8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b5bed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345bed5, 4 },
    { 0x3195bed6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6dbed9, 3 },
    { 0, 0 },
    { 0x3e81bedb, 3 },
    { 0, 0 },
    { 0x3fd5bedd, 3 },
    { 0xef81bede, 3 },
    { 0x6d3dbedd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa55bee4, 3 },
    { 0x22c5bee5, 4 },
    { 0x524dbee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa909beec, 3 },
    { 0xe369beec, 3 },
    { 0xa651beee, 4 },
    { 0x9625beed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5bef2, 4 },
    { 0, 0 },
    { 0x7419bef4, 3 },
    { 0x2d75bef4, 3 },
    { 0x82e5bef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef95bef9, 4 },
    { 0x2cc5bef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7621bf00, 3 },
    { 0, 0 },
    { 0x4269bf02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd515bf05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c9bf0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6311bf11, 3 },
    { 0x5e81bf11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7769bf17, 3 },
    { 0, 0 },
    { 0x5c35bf19, 3 },
    { 0x395bf19, 3 },
    { 0x54d9bf1b, 4 },
    { 0x8921bf1a, 3 },
    { 0x97dbf1d, 3 },
    { 0x1731bf1d, 3 },
    { 0x2cd9bf1e, 4 },
    { 0x3681bf1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7655bf23, 3 },
    { 0xb3adbf23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7191bf27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb5bf2d, 3 },
    { 0x4c05bf2d, 3 },
    { 0, 0 },
    { 0x3849bf30, 3 },
    { 0, 0 },
    { 0x54d1bf32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d9bf35, 3 },
    { 0x9805bf35, 4 },
    { 0x1b21bf35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x126dbf3a, 3 },
    { 0x9761bf3b, 3 },
    { 0xf121bf3a, 4 },
    { 0x6895bf3b, 3 },
    { 0x17b1bf3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd5bf42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b31bf45, 4 },
    { 0xfc61bf46, 3 },
    { 0x70f1bf47, 4 },
    { 0, 0 },
    { 0xa949bf49, 4 },
    { 0x28ddbf49, 3 },
    { 0x16b5bf4a, 3 },
    { 0xe4d5bf49, 5 },
    { 0x8e75bf4d, 3 },
    { 0x55ddbf4d, 3 },
    { 0xc0e1bf4b, 3 },
    { 0x4a1bf4a, 4 },
    { 0x1855bf4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7dbf54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5bf57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815bf5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9bf5e, 3 },
    { 0, 0 },
    { 0x39e1bf60, 4 },
    { 0, 0 },
    { 0xe4a1bf62, 3 },
    { 0xaf81bf63, 3 },
    { 0xc771bf64, 3 },
    { 0, 0 },
    { 0xc115bf66, 3 },
    { 0, 0 },
    { 0xc28dbf68, 3 },
    { 0x455bf69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641bf6f, 3 },
    { 0x5699bf6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d9bf74, 4 },
    { 0xcd5bf75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1079bf79, 3 },
    { 0x2c09bf7a, 3 },
    { 0x1e81bf7a, 4 },
    { 0, 0 },
    { 0x3889bf7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a31bf80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd431bf83, 4 },
    { 0xce69bf84, 3 },
    { 0, 0 },
    { 0xf0b9bf86, 3 },
    { 0x489dbf87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fdbf8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc731bf91, 3 },
    { 0x7d39bf92, 3 },
    { 0x824dbf92, 3 },
    { 0x9085bf94, 3 },
    { 0, 0 },
    { 0x6239bf96, 3 },
    { 0, 0 },
    { 0xf1bf98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ddbf9c, 3 },
    { 0xe15dbf9c, 3 },
    { 0, 0 },
    { 0xbe2dbf9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8139bfa2, 3 },
    { 0, 0 },
    { 0xc659bfa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e9bfa8, 4 },
    { 0, 0 },
    { 0x8e1dbfaa, 4 },
    { 0x78a5bfab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x869bfae, 3 },
    { 0x4d49bfae, 3 },
    { 0, 0 },
    { 0xeae1bfb1, 3 },
    { 0x8cd9bfb2, 5 },
    { 0x14b9bfb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a9bfb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c5bfc2, 4 },
    { 0x70d5bfc2, 4 },
    { 0xd339bfc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f1bfc7, 3 },
    { 0x9559bfc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd579bfd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f9bfdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5bfe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1dbfe3, 3 },
    { 0xe61bfe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec9bfe9, 3 },
    { 0x197dbfea, 3 },
    { 0x2131bfeb, 3 },
    { 0x96d9bfea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e1bff2, 3 },
    { 0, 0 },
    { 0xdcadbff4, 3 },
    { 0x1c19bff5, 4 },
    { 0, 0 },
    { 0x470dbff7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e5bffc, 5 },
    { 0x10b5bffd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d1c007, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff39c00c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7085c010, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd945c013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9c016, 3 },
    { 0x7821c016, 3 },
    { 0, 0 },
    { 0xe1b1c019, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24dc01d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1dc020, 3 },
    { 0xf05c021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8229c026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9c029, 3 },
    { 0x4955c029, 3 },
    { 0xf7f9c02a, 3 },
    { 0xdb1c02c, 3 },
    { 0x10a1c02c, 3 },
    { 0xe911c02e, 4 },
    { 0x14c9c02e, 3 },
    { 0xb371c02e, 3 },
    { 0x1a35c030, 3 },
    { 0xb69c02a, 3 },
    { 0, 0 },
    { 0xcb91c034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e39c039, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05dc03c, 4 },
    { 0xb975c03d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d29c041, 3 },
    { 0x8a71c042, 4 },
    { 0, 0 },
    { 0x6209c044, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d9c048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c61c04f, 4 },
    { 0x97a5c04f, 3 },
    { 0x29f5c051, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6895c05c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9681c061, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d9c065, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2dc06f, 4 },
    { 0xb79dc06f, 3 },
    { 0xdd9dc06f, 3 },
    { 0, 0 },
    { 0xabc5c073, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc865c078, 5 },
    { 0, 0 },
    { 0xe20dc07a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb621c082, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d5c086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc08d, 3 },
    { 0x5095c08e, 3 },
    { 0x989dc08e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3105c093, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe35c096, 4 },
    { 0xd385c097, 3 },
    { 0x23e9c098, 3 },
    { 0xfc55c099, 4 },
    { 0xdad1c099, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefdc09f, 3 },
    { 0x4be1c09f, 3 },
    { 0x4729c09f, 3 },
    { 0xfd4dc0a0, 3 },
    { 0x5501c0a3, 3 },
    { 0x392dc0a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2155c0b3, 4 },
    { 0, 0 },
    { 0xc8cdc0b5, 3 },
    { 0x76d9c0b6, 3 },
    { 0xe815c0b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeedc0bc, 3 },
    { 0x9f1dc0bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe101c0c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc0c5, 4 },
    { 0x97d1c0c6, 3 },
    { 0x2571c0c7, 3 },
    { 0x25d9c0c8, 4 },
    { 0xc201c0c5, 4 },
    { 0x6755c0c5, 5 },
    { 0, 0 },
    { 0xd05dc0cc, 3 },
    { 0x3175c0cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5c0d1, 4 },
    { 0, 0 },
    { 0x5039c0d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35c0d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6621c0e0, 3 },
    { 0xb645c0e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd49c0e5, 4 },
    { 0xd7a5c0e6, 3 },
    { 0x3855c0e6, 4 },
    { 0, 0 },
    { 0x7f1dc0e9, 4 },
    { 0, 0 },
    { 0x4c45c0eb, 3 },
    { 0x99b1c0ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc069c0ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee5c0f2, 3 },
    { 0x6299c0f3, 3 },
    { 0xb9f9c0f4, 4 },
    { 0x4f1c0f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9261c0fb, 3 },
    { 0, 0 },
    { 0x9eddc0fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadc101, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe381c107, 4 },
    { 0xe811c108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb001c10c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01dc112, 3 },
    { 0, 0 },
    { 0x5f95c114, 3 },
    { 0, 0 },
    { 0x4779c116, 3 },
    { 0xb2f5c117, 3 },
    { 0, 0 },
    { 0x6575c119, 4 },
    { 0x2bbdc11a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da9c128, 3 },
    { 0xe9a5c129, 4 },
    { 0x1b29c12a, 3 },
    { 0x5485c12b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a5c130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae51c135, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9755c138, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181c13e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c35c145, 3 },
    { 0x3bd9c145, 3 },
    { 0xb95dc145, 3 },
    { 0x6f79c145, 4 },
    { 0xd491c149, 3 },
    { 0x515dc149, 4 },
    { 0x77fdc149, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf335c14f, 4 },
    { 0xee11c150, 3 },
    { 0x5b65c150, 3 },
    { 0, 0 },
    { 0x6da5c153, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b35c156, 4 },
    { 0x7e8dc157, 3 },
    { 0x7b39c158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8dc15b, 4 },
    { 0xea21c15b, 3 },
    { 0xfe21c15d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc1c165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8795c169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9c16d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31c170, 3 },
    { 0, 0 },
    { 0x9ae1c172, 3 },
    { 0xf1c9c173, 3 },
    { 0xf511c173, 4 },
    { 0x4f19c175, 4 },
    { 0x1559c174, 3 },
    { 0xf0d5c177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed9c17b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcdc17e, 3 },
    { 0x83edc17f, 4 },
    { 0, 0 },
    { 0x4af5c181, 3 },
    { 0, 0 },
    { 0xae8dc183, 3 },
    { 0xb135c184, 3 },
    { 0x90e5c185, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c9c189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb399c190, 4 },
    { 0xc111c190, 3 },
    { 0, 0 },
    { 0xb3c5c193, 3 },
    { 0xd6adc194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215c197, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc19b, 3 },
    { 0xb9adc19b, 3 },
    { 0, 0 },
    { 0x31f5c19e, 4 },
    { 0x7115c19f, 4 },
    { 0x1051c19e, 3 },
    { 0x80a9c19f, 3 },
    { 0xc3fdc1a2, 3 },
    { 0x43b9c19f, 3 },
    { 0xab05c1a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a95c1a7, 4 },
    { 0x6d05c1a8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8689c1b1, 3 },
    { 0, 0 },
    { 0x3fb9c1b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5c1b6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a01c1b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3095c1bf, 3 },
    { 0xfb1c1c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e75c1cc, 3 },
    { 0, 0 },
    { 0x79f9c1ce, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774dc1db, 3 },
    { 0x4d71c1dc, 3 },
    { 0xae79c1db, 3 },
    { 0, 0 },
    { 0x3175c1df, 3 },
    { 0x648dc1e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dc1eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d91c1ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe519c1f1, 4 },
    { 0, 0 },
    { 0x80e5c1f3, 3 },
    { 0xcf45c1f4, 3 },
    { 0x2bd5c1f5, 3 },
    { 0x40d1c1f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7dc1f9, 3 },
    { 0xb7d1c1fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5c201, 4 },
    { 0, 0 },
    { 0x5a79c203, 3 },
    { 0x14edc203, 4 },
    { 0x230dc204, 3 },
    { 0x9309c206, 5 },
    { 0xf655c207, 3 },
    { 0x2719c208, 4 },
    { 0x95e1c209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f45c20c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe9c20f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328dc21b, 3 },
    { 0x7215c21c, 3 },
    { 0x2a79c21b, 3 },
    { 0x9949c21d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x359dc221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe751c232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135c236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8931c23d, 3 },
    { 0x8529c23e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979c244, 3 },
    { 0, 0 },
    { 0x425dc246, 3 },
    { 0x1495c246, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1c24b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dc252, 3 },
    { 0x76b1c253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e1c258, 3 },
    { 0x86f1c259, 5 },
    { 0, 0 },
    { 0xd4a9c25b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675c25f, 4 },
    { 0x8555c25f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5061c263, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f5c267, 3 },
    { 0x23c9c268, 3 },
    { 0, 0 },
    { 0x4451c26a, 3 },
    { 0x38a5c26b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8189c26e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405c272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e5c275, 3 },
    { 0xa105c276, 4 },
    { 0x3521c277, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1011c27e, 5 },
    { 0xad19c27f, 3 },
    { 0xc13dc280, 4 },
    { 0x3c81c281, 3 },
    { 0, 0 },
    { 0xecd9c283, 4 },
    { 0x5809c284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd621c288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b1c28b, 3 },
    { 0, 0 },
    { 0x9b65c28d, 3 },
    { 0x601c28d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e9c295, 3 },
    { 0, 0 },
    { 0xdbf9c297, 3 },
    { 0x63b1c297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9641c2a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf595c2a5, 4 },
    { 0x48c1c2a5, 3 },
    { 0x1c0dc2a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b9c2ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f5c2ae, 3 },
    { 0, 0 },
    { 0xf71c2b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8dc2b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba15c2bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe125c2c1, 3 },
    { 0, 0 },
    { 0xd281c2c3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5c2ca, 3 },
    { 0, 0 },
    { 0x21ddc2cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852dc2d0, 3 },
    { 0x241c2d1, 4 },
    { 0xee89c2d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63dc2d5, 3 },
    { 0, 0 },
    { 0x68d9c2d7, 3 },
    { 0x9439c2d7, 3 },
    { 0, 0 },
    { 0x6d6dc2da, 3 },
    { 0, 0 },
    { 0x71b9c2dc, 3 },
    { 0x3d3dc2dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea29c2e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7785c2e9, 3 },
    { 0x8251c2e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86dc2ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9c2f7, 3 },
    { 0xb6f5c2f7, 3 },
    { 0, 0 },
    { 0xb8c9c2fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc5c301, 3 },
    { 0x1c5dc302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87edc306, 3 },
    { 0x1629c307, 4 },
    { 0xafd5c308, 3 },
    { 0, 0 },
    { 0xc8c5c30a, 3 },
    { 0x6475c30a, 3 },
    { 0, 0 },
    { 0x3fa5c30d, 4 },
    { 0xa751c30e, 3 },
    { 0x7271c30f, 4 },
    { 0xb16dc30e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acdc313, 3 },
    { 0, 0 },
    { 0x2ce1c315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2445c31b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fdc320, 3 },
    { 0, 0 },
    { 0x2f4dc322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd5c32b, 3 },
    { 0, 0 },
    { 0x9315c32d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40dc333, 4 },
    { 0x6badc333, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bdc337, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6dc33a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc421c33d, 3 },
    { 0x16a5c33d, 3 },
    { 0x6865c33e, 4 },
    { 0x98a9c340, 3 },
    { 0xc6a9c340, 3 },
    { 0x9075c342, 3 },
    { 0x7b01c343, 3 },
    { 0x1e9c344, 3 },
    { 0, 0 },
    { 0x5069c346, 3 },
    { 0, 0 },
    { 0x1d5c348, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ddc351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea5c359, 3 },
    { 0xe4d9c359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c5c35d, 3 },
    { 0xf709c35d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb575c361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2719c366, 3 },
    { 0xadbdc367, 3 },
    { 0xdb61c368, 3 },
    { 0, 0 },
    { 0x195c36a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15c36d, 3 },
    { 0, 0 },
    { 0x76e9c36f, 3 },
    { 0xf8fdc36f, 3 },
    { 0xbb19c36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc061c375, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184dc37b, 3 },
    { 0x3edc37c, 3 },
    { 0x51a9c37c, 3 },
    { 0x4ad5c37d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd9c381, 3 },
    { 0xb59c382, 3 },
    { 0xee29c383, 4 },
    { 0x66d1c384, 3 },
    { 0, 0 },
    { 0xe5bdc386, 3 },
    { 0xc691c386, 4 },
    { 0, 0 },
    { 0xeac5c389, 3 },
    { 0, 0 },
    { 0x533dc38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7dc38f, 3 },
    { 0, 0 },
    { 0x4f65c391, 3 },
    { 0x75a5c391, 3 },
    { 0x398dc392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863dc399, 4 },
    { 0, 0 },
    { 0x49f1c39b, 3 },
    { 0, 0 },
    { 0x221dc39d, 4 },
    { 0x3411c39e, 5 },
    { 0xbf61c39d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c1c3a5, 3 },
    { 0xac45c3a5, 3 },
    { 0xda59c3a7, 3 },
    { 0, 0 },
    { 0x4235c3a9, 3 },
    { 0x8145c3aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf1c3ae, 3 },
    { 0xbbe9c3ae, 3 },
    { 0, 0 },
    { 0x6a2dc3b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5c3b8, 3 },
    { 0xedc5c3b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5915c3bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf05c3c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4dc3c6, 5 },
    { 0xd5adc3c6, 3 },
    { 0, 0 },
    { 0x8749c3c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209c3cc, 4 },
    { 0xf981c3cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5c3d0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d05c3d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d35c3d6, 5 },
    { 0, 0 },
    { 0xa76dc3d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c69c3dc, 3 },
    { 0, 0 },
    { 0xd1adc3de, 3 },
    { 0x181c3df, 3 },
    { 0, 0 },
    { 0x50e5c3e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7709c3e4, 3 },
    { 0x2899c3e5, 4 },
    { 0xb4cdc3e6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ddc3ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01c3f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0dc3f9, 4 },
    { 0x1711c3fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1dc3fd, 3 },
    { 0x681c3fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149c401, 4 },
    { 0, 0 },
    { 0xfff1c403, 3 },
    { 0x5419c403, 4 },
    { 0x70b5c405, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5c40c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848dc411, 3 },
    { 0, 0 },
    { 0x8ea5c413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bdc418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c9c41b, 4 },
    { 0, 0 },
    { 0xab25c41d, 3 },
    { 0x4791c41e, 3 },
    { 0xb8fdc41d, 4 },
    { 0x1731c41e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42dc423, 3 },
    { 0, 0 },
    { 0xe86dc425, 3 },
    { 0x9905c426, 4 },
    { 0x8865c427, 3 },
    { 0xa96dc427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97dc42d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x743dc430, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc9c434, 4 },
    { 0xa899c434, 3 },
    { 0xa75dc436, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4011c43c, 3 },
    { 0xd5e1c43d, 3 },
    { 0xe2d9c43e, 5 },
    { 0x6701c43c, 3 },
    { 0x5c39c440, 3 },
    { 0, 0 },
    { 0xdc59c442, 4 },
    { 0xfd75c443, 3 },
    { 0xf171c444, 3 },
    { 0xe2e5c445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dc44b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a5c44f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359c454, 4 },
    { 0x95e1c454, 3 },
    { 0x8b4dc455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58dc459, 3 },
    { 0xacf5c45a, 3 },
    { 0x3ec9c459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5c45e, 4 },
    { 0xec1dc45e, 3 },
    { 0x17ddc460, 4 },
    { 0, 0 },
    { 0x58b9c462, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd1c468, 4 },
    { 0x8c5c469, 3 },
    { 0x8d89c469, 3 },
    { 0x21b5c46b, 3 },
    { 0x3869c46c, 4 },
    { 0xf8c9c468, 4 },
    { 0x23a5c46c, 3 },
    { 0x4ae9c46f, 3 },
    { 0xb5b9c470, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde1c473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265dc478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10dc481, 4 },
    { 0xb091c482, 4 },
    { 0xd981c483, 3 },
    { 0x3f5dc484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb1c488, 3 },
    { 0, 0 },
    { 0x5a61c48a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab39c48d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedc490, 3 },
    { 0xb389c491, 3 },
    { 0x91dc492, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd5c495, 3 },
    { 0xd6ddc496, 3 },
    { 0x5b15c495, 4 },
    { 0x726dc498, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc69c49d, 3 },
    { 0x5d49c49e, 3 },
    { 0x1935c49f, 5 },
    { 0x2cb1c4a0, 5 },
    { 0x9dd9c4a1, 3 },
    { 0x4085c4a1, 3 },
    { 0xc4a1c4a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4635c4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9661c4af, 3 },
    { 0x7891c4b0, 3 },
    { 0xa445c4b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0dc4b5, 3 },
    { 0x9045c4b6, 3 },
    { 0xa7b1c4b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c4bb, 4 },
    { 0xeae9c4bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a91c4c1, 3 },
    { 0xb095c4c2, 3 },
    { 0x8509c4c2, 3 },
    { 0x23f9c4c3, 3 },
    { 0xe4e9c4c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6631c4cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35c4d2, 3 },
    { 0x2a25c4d3, 3 },
    { 0x5c9c4d4, 3 },
    { 0x9005c4d5, 3 },
    { 0xfc49c4d6, 3 },
    { 0xc31c4d7, 3 },
    { 0x7455c4d3, 3 },
    { 0x5161c4d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce21c4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff9c4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790dc4e9, 3 },
    { 0x9f71c4ea, 3 },
    { 0x3899c4ea, 3 },
    { 0x1425c4ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa475c4f0, 3 },
    { 0x45a9c4f1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9c4f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a1c4fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49edc4ff, 3 },
    { 0, 0 },
    { 0xfa05c501, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4639c505, 4 },
    { 0xc879c506, 4 },
    { 0xd7b5c507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f61c50a, 3 },
    { 0xe8f1c50b, 3 },
    { 0xcf61c50c, 4 },
    { 0x3e39c50d, 3 },
    { 0xb729c50e, 4 },
    { 0xa929c50a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8099c512, 3 },
    { 0, 0 },
    { 0x8285c514, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff61c517, 3 },
    { 0, 0 },
    { 0xc049c519, 3 },
    { 0, 0 },
    { 0x4085c51b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f5c51e, 3 },
    { 0, 0 },
    { 0xedb5c520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5c523, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5681c52b, 4 },
    { 0x91fdc52c, 4 },
    { 0x93adc52d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc389c534, 4 },
    { 0x9f19c534, 3 },
    { 0x76a9c536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bedc53a, 3 },
    { 0x2ff5c53b, 3 },
    { 0x126dc53b, 3 },
    { 0x4075c53c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1881c543, 3 },
    { 0xb0b9c543, 3 },
    { 0x78d9c543, 4 },
    { 0, 0 },
    { 0xb431c547, 3 },
    { 0, 0 },
    { 0xf581c549, 3 },
    { 0x3d9c549, 3 },
    { 0x722dc54b, 3 },
    { 0, 0 },
    { 0x2e05c54d, 3 },
    { 0xc915c54d, 3 },
    { 0xc6a1c54f, 3 },
    { 0, 0 },
    { 0xf4a1c551, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf545c554, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a1c559, 4 },
    { 0xa779c55a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625dc55f, 4 },
    { 0xba5c560, 3 },
    { 0x971dc560, 3 },
    { 0x31d9c560, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd5c565, 3 },
    { 0xec01c566, 3 },
    { 0x8c29c567, 3 },
    { 0x1921c568, 3 },
    { 0x569dc569, 3 },
    { 0x4301c56a, 3 },
    { 0xc585c56a, 4 },
    { 0, 0 },
    { 0xab09c56d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd511c571, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3111c574, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e45c577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e1c57a, 4 },
    { 0x4569c57a, 3 },
    { 0x1e11c57c, 3 },
    { 0x38d1c57d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08dc582, 3 },
    { 0x7919c583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f5c586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015c58a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1045c592, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c5c595, 3 },
    { 0x8e6dc596, 4 },
    { 0, 0 },
    { 0x20b5c598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be5c59c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5c5a1, 3 },
    { 0x94c5c5a2, 3 },
    { 0x3459c5a3, 3 },
    { 0x7091c5a1, 3 },
    { 0xb529c5a1, 4 },
    { 0x3fc9c5a6, 3 },
    { 0, 0 },
    { 0x449dc5a8, 3 },
    { 0xb6adc5a9, 3 },
    { 0x87c5c5a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca5c5ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15c5b2, 3 },
    { 0, 0 },
    { 0x39e5c5b4, 3 },
    { 0, 0 },
    { 0x7789c5b6, 3 },
    { 0x1ef5c5b7, 4 },
    { 0xa8edc5b7, 3 },
    { 0xcd65c5b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1c5be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c85c5c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc149c5c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790dc5ca, 3 },
    { 0x73bdc5cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d89c5ce, 3 },
    { 0x1d59c5cf, 3 },
    { 0, 0 },
    { 0x6f5dc5d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31dc5d4, 3 },
    { 0x97b5c5d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389c5db, 3 },
    { 0xfdf1c5dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc069c5e0, 3 },
    { 0, 0 },
    { 0xd7a5c5e2, 4 },
    { 0, 0 },
    { 0x13a9c5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5c5ea, 3 },
    { 0x4b49c5eb, 3 },
    { 0xd275c5ea, 3 },
    { 0x4805c5ed, 4 },
    { 0x8dd9c5ee, 3 },
    { 0x82bdc5ef, 3 },
    { 0x9c81c5f0, 3 },
    { 0, 0 },
    { 0x500dc5f2, 3 },
    { 0, 0 },
    { 0x7ce5c5f4, 5 },
    { 0x8c31c5f5, 3 },
    { 0xce6dc5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211c5fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541dc5ff, 3 },
    { 0, 0 },
    { 0xf7adc601, 3 },
    { 0x6885c602, 3 },
    { 0, 0 },
    { 0xf975c604, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3965c609, 3 },
    { 0x6641c60a, 3 },
    { 0, 0 },
    { 0xe7f9c60c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc60f, 3 },
    { 0, 0 },
    { 0x9a4dc611, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb5c616, 3 },
    { 0, 0 },
    { 0xaa65c618, 3 },
    { 0, 0 },
    { 0x75c61a, 4 },
    { 0, 0 },
    { 0xd1a1c61c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba9c621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4dc624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b1c629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391c62f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf735c634, 4 },
    { 0xffcdc635, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945c638, 4 },
    { 0x64c9c639, 3 },
    { 0xc82dc638, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b1c63d, 3 },
    { 0x6125c63e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f1c641, 3 },
    { 0xe471c642, 3 },
    { 0, 0 },
    { 0xc755c644, 3 },
    { 0x6131c644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d85c64d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d99c655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5c658, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511c65d, 3 },
    { 0x4021c65d, 3 },
    { 0, 0 },
    { 0xd3bdc660, 3 },
    { 0xb245c660, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5201c66f, 4 },
    { 0, 0 },
    { 0x55d1c671, 4 },
    { 0x169dc672, 3 },
    { 0x4b25c671, 3 },
    { 0, 0 },
    { 0xdf65c675, 3 },
    { 0x67b1c676, 3 },
    { 0x6699c675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5865c67f, 3 },
    { 0, 0 },
    { 0xa865c681, 3 },
    { 0xc2e9c682, 3 },
    { 0xa725c681, 3 },
    { 0, 0 },
    { 0x1f31c685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5529c689, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1c690, 3 },
    { 0x8af1c691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b35c694, 3 },
    { 0xef41c695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9c698, 3 },
    { 0, 0 },
    { 0x392dc69a, 4 },
    { 0x452dc69b, 3 },
    { 0x3891c69c, 4 },
    { 0x5b95c69d, 3 },
    { 0xb8adc69b, 3 },
    { 0, 0 },
    { 0x2fa5c6a0, 3 },
    { 0x74c9c6a1, 4 },
    { 0, 0 },
    { 0x177dc6a3, 4 },
    { 0x32c9c6a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6205c6ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cdc6af, 4 },
    { 0x8ebdc6af, 3 },
    { 0x2779c6b1, 5 },
    { 0, 0 },
    { 0x2d59c6b3, 3 },
    { 0x4e9dc6b3, 3 },
    { 0, 0 },
    { 0x4b55c6b6, 3 },
    { 0xfbe5c6b7, 3 },
    { 0x1c35c6b8, 3 },
    { 0xe94dc6b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a9c6bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e35c6c0, 3 },
    { 0x25f5c6c0, 3 },
    { 0x3a01c6c0, 3 },
    { 0x8df1c6c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251c6c8, 3 },
    { 0, 0 },
    { 0xe139c6ca, 5 },
    { 0xfef5c6ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x488dc6ce, 3 },
    { 0, 0 },
    { 0xae01c6d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a25c6d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4659c6e3, 3 },
    { 0x2ea5c6e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71c6e9, 3 },
    { 0, 0 },
    { 0x2ad5c6eb, 3 },
    { 0, 0 },
    { 0x7785c6ed, 3 },
    { 0, 0 },
    { 0x35b5c6ef, 3 },
    { 0xf735c6ef, 3 },
    { 0x621c6ef, 3 },
    { 0, 0 },
    { 0xd74dc6f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb85c6fa, 3 },
    { 0x9f15c6fb, 4 },
    { 0x1c15c6fc, 4 },
    { 0, 0 },
    { 0x879dc6fe, 4 },
    { 0, 0 },
    { 0x1931c700, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189dc708, 3 },
    { 0, 0 },
    { 0x9775c70a, 3 },
    { 0x3e05c70a, 3 },
    { 0x5d91c70c, 3 },
    { 0x1a91c70b, 3 },
    { 0xe15dc70b, 3 },
    { 0x6e71c70f, 3 },
    { 0x5e05c70b, 5 },
    { 0x4385c710, 3 },
    { 0x99d1c712, 3 },
    { 0xfb79c712, 3 },
    { 0, 0 },
    { 0xf2d5c715, 3 },
    { 0xc82dc716, 3 },
    { 0, 0 },
    { 0xf11c718, 3 },
    { 0, 0 },
    { 0x6bc1c71a, 3 },
    { 0x2bf9c71a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36dc720, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b75c723, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb059c72d, 3 },
    { 0, 0 },
    { 0x3965c72f, 3 },
    { 0, 0 },
    { 0x719dc731, 3 },
    { 0xe105c732, 3 },
    { 0x19a1c732, 3 },
    { 0xfa49c734, 3 },
    { 0x8871c731, 3 },
    { 0xf451c736, 3 },
    { 0xc6e9c736, 3 },
    { 0xef71c738, 4 },
    { 0x3c5dc739, 3 },
    { 0xe1e5c738, 4 },
    { 0xbf1dc739, 3 },
    { 0x9bdc73c, 3 },
    { 0xce45c73d, 3 },
    { 0x4f2dc731, 4 },
    { 0, 0 },
    { 0xdfbdc740, 3 },
    { 0x11d1c741, 3 },
    { 0x7725c740, 3 },
    { 0xdff9c743, 4 },
    { 0xc809c743, 3 },
    { 0x3791c744, 3 },
    { 0x7441c746, 3 },
    { 0x619dc747, 3 },
    { 0xa9b5c742, 3 },
    { 0xd5b9c749, 3 },
    { 0x5b09c740, 3 },
    { 0x7f29c743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2865c74e, 4 },
    { 0, 0 },
    { 0xe719c750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fdc757, 3 },
    { 0x2cb5c757, 3 },
    { 0, 0 },
    { 0x9a09c75a, 3 },
    { 0x3165c75b, 3 },
    { 0x91f9c75b, 3 },
    { 0, 0 },
    { 0x7f39c75e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceadc762, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d51c765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a59c769, 3 },
    { 0, 0 },
    { 0xf0a5c76b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e9c771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade9c778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa821c782, 4 },
    { 0xd2edc783, 3 },
    { 0x7469c784, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe169c78a, 3 },
    { 0x6e15c78b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975c78e, 3 },
    { 0xfc05c78f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd061c793, 4 },
    { 0xf3e5c794, 4 },
    { 0x4fb9c795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe065c798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b15c7a1, 4 },
    { 0xb0edc7a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf061c7a6, 3 },
    { 0x4269c7a7, 3 },
    { 0x24e9c7a6, 4 },
    { 0xc6c1c7a9, 3 },
    { 0x4b81c7a9, 4 },
    { 0x9665c7aa, 3 },
    { 0xb8f5c7ac, 5 },
    { 0, 0 },
    { 0x157dc7ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc541c7b2, 4 },
    { 0xe209c7b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc1c7b6, 3 },
    { 0x5581c7b6, 3 },
    { 0x2425c7b8, 3 },
    { 0x2ac5c7b8, 3 },
    { 0xfc85c7ba, 4 },
    { 0xca89c7bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235c7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a55c7c2, 3 },
    { 0x951dc7c3, 4 },
    { 0x5449c7c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a1c7c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b1c7cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979c7ce, 4 },
    { 0x4789c7cf, 3 },
    { 0, 0 },
    { 0x8609c7d1, 3 },
    { 0x504dc7d2, 4 },
    { 0, 0 },
    { 0xbd81c7d4, 3 },
    { 0xb0a9c7d5, 3 },
    { 0x3779c7d5, 4 },
    { 0x673dc7d6, 3 },
    { 0x389c7d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5595c7de, 3 },
    { 0, 0 },
    { 0xac1c7e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb45c7e5, 4 },
    { 0xed35c7e6, 3 },
    { 0xfe39c7e7, 3 },
    { 0, 0 },
    { 0xa419c7e9, 4 },
    { 0x3a79c7ea, 4 },
    { 0xd8d5c7eb, 3 },
    { 0xb1c1c7ec, 3 },
    { 0x3c2dc7ed, 3 },
    { 0x2b49c7ed, 4 },
    { 0x9529c7ef, 3 },
    { 0xc441c7f0, 3 },
    { 0xc779c7ed, 3 },
    { 0, 0 },
    { 0x34f1c7f3, 4 },
    { 0xa281c7f4, 3 },
    { 0xd0edc7f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7269c7f9, 3 },
    { 0, 0 },
    { 0x9d7dc7fb, 3 },
    { 0xfb45c7fc, 3 },
    { 0xfba9c7fd, 4 },
    { 0xec11c7fb, 3 },
    { 0x1a71c7fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d9c804, 3 },
    { 0x6789c804, 5 },
    { 0, 0 },
    { 0xd049c807, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72dc80f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6489c813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2179c81b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc81e, 3 },
    { 0, 0 },
    { 0x39f1c820, 3 },
    { 0, 0 },
    { 0x1a2dc822, 3 },
    { 0x851dc823, 3 },
    { 0xe2adc824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c85c82e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a69c831, 3 },
    { 0x8e31c832, 3 },
    { 0x6db9c833, 3 },
    { 0x4a3dc834, 3 },
    { 0, 0 },
    { 0xc481c836, 5 },
    { 0, 0 },
    { 0xd935c838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f1c83e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60edc84d, 3 },
    { 0x54ddc84e, 3 },
    { 0, 0 },
    { 0x56e1c850, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f81c855, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1485c859, 4 },
    { 0x5be9c85a, 3 },
    { 0xddc5c85a, 3 },
    { 0xb62dc85a, 3 },
    { 0xb09c85d, 4 },
    { 0xb745c85e, 3 },
    { 0x3b1c85e, 4 },
    { 0xd791c85d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97dc863, 3 },
    { 0x6b35c864, 3 },
    { 0, 0 },
    { 0x65c1c866, 3 },
    { 0, 0 },
    { 0x1075c868, 3 },
    { 0x9abdc869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf529c86c, 3 },
    { 0xe2f9c86d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab5c872, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f49c878, 3 },
    { 0, 0 },
    { 0x9955c87a, 3 },
    { 0x7429c87b, 3 },
    { 0x9075c87a, 4 },
    { 0xfb79c87d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c9c880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e5c885, 4 },
    { 0xf139c885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9c889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8285c88e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe999c893, 3 },
    { 0, 0 },
    { 0x6ca1c895, 4 },
    { 0xf9cdc895, 3 },
    { 0, 0 },
    { 0x6ad1c898, 3 },
    { 0x5899c899, 3 },
    { 0x9c45c899, 4 },
    { 0x1d29c89b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd071c8a1, 3 },
    { 0xa319c8a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf915c8a5, 4 },
    { 0x7c85c8a6, 5 },
    { 0, 0 },
    { 0x6d01c8a8, 3 },
    { 0x50c1c8a9, 5 },
    { 0, 0 },
    { 0xc289c8ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff9c8ae, 3 },
    { 0x9cadc8ae, 4 },
    { 0x39e9c8af, 3 },
    { 0, 0 },
    { 0x50edc8b2, 3 },
    { 0xcc9dc8b2, 3 },
    { 0x2cc1c8b4, 3 },
    { 0x5839c8b5, 5 },
    { 0, 0 },
    { 0x617dc8b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd61c8c2, 4 },
    { 0xc5e1c8c3, 3 },
    { 0, 0 },
    { 0x7fddc8c5, 3 },
    { 0xdde9c8c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c59c8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0dc8ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71c8d4, 3 },
    { 0x4f4dc8d5, 4 },
    { 0x1e9dc8d5, 4 },
    { 0x5f8dc8d6, 3 },
    { 0xb421c8d8, 3 },
    { 0xee65c8d5, 4 },
    { 0, 0 },
    { 0xb285c8db, 3 },
    { 0x252dc8db, 3 },
    { 0x9611c8dd, 4 },
    { 0xd289c8dc, 3 },
    { 0xa955c8de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7079c8e2, 3 },
    { 0xb79dc8e3, 3 },
    { 0, 0 },
    { 0x2ab5c8e5, 4 },
    { 0x80d5c8e6, 4 },
    { 0xea59c8e7, 5 },
    { 0xc531c8e6, 3 },
    { 0xb23dc8e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f5c8ec, 3 },
    { 0x9c09c8ed, 3 },
    { 0xe659c8ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4015c8f1, 3 },
    { 0xc109c8f2, 3 },
    { 0x36e5c8f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df5c8f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5c8fa, 3 },
    { 0, 0 },
    { 0x29f5c8fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc879c8ff, 4 },
    { 0x2755c900, 3 },
    { 0x52e9c8ff, 3 },
    { 0xbcadc902, 3 },
    { 0xa4a9c903, 3 },
    { 0x8995c902, 4 },
    { 0xa5d9c905, 3 },
    { 0x6f6dc903, 3 },
    { 0x10b5c903, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e15c90a, 3 },
    { 0x7dfdc90b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x473dc90e, 3 },
    { 0, 0 },
    { 0x662dc910, 3 },
    { 0, 0 },
    { 0xaef5c912, 3 },
    { 0x347dc913, 3 },
    { 0, 0 },
    { 0xd381c915, 4 },
    { 0, 0 },
    { 0x9041c917, 3 },
    { 0xaf01c917, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b1c91d, 4 },
    { 0x939c91e, 4 },
    { 0xd2b9c91e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8255c922, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1c927, 3 },
    { 0x502dc928, 3 },
    { 0xceb5c927, 3 },
    { 0xbea1c92a, 3 },
    { 0x9a61c92a, 4 },
    { 0xe619c92c, 3 },
    { 0x55f9c927, 3 },
    { 0, 0 },
    { 0x34f1c92f, 3 },
    { 0xebb5c930, 3 },
    { 0x740dc931, 4 },
    { 0, 0 },
    { 0x6d01c933, 4 },
    { 0xf17dc933, 3 },
    { 0, 0 },
    { 0xe829c936, 4 },
    { 0, 0 },
    { 0xd189c938, 3 },
    { 0x7a79c939, 3 },
    { 0, 0 },
    { 0x747dc93b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7dc942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c947, 3 },
    { 0x231c948, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9901c94d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639c950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a69c954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb9c957, 3 },
    { 0xbeddc958, 3 },
    { 0x250dc959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ddc96a, 3 },
    { 0x1459c96b, 4 },
    { 0x1b85c96a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2dc970, 3 },
    { 0x79f9c970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x674dc974, 4 },
    { 0xca4dc974, 3 },
    { 0, 0 },
    { 0x1e9dc977, 3 },
    { 0x9a5c978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b1c97e, 3 },
    { 0, 0 },
    { 0xe3dc980, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e1c984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x771c987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541dc98d, 3 },
    { 0x5049c98e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6dc991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4525c994, 3 },
    { 0x2ca9c994, 3 },
    { 0xfaedc995, 4 },
    { 0x4009c994, 3 },
    { 0, 0 },
    { 0x66edc999, 4 },
    { 0x3cb5c99a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb1c99f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e1c9a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8439c9a6, 3 },
    { 0xf2f5c9a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d5c9ac, 4 },
    { 0x8b25c9ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbdc9b1, 3 },
    { 0xcd49c9b2, 3 },
    { 0xba7dc9b1, 3 },
    { 0, 0 },
    { 0xc03dc9b5, 3 },
    { 0, 0 },
    { 0xa88dc9b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2809c9ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2159c9be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1c9c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95dc9c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x705dc9ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d9c9d2, 3 },
    { 0x2349c9d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc5c9d9, 4 },
    { 0x73e5c9d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf251c9df, 3 },
    { 0, 0 },
    { 0x65dc9e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc791c9e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3dc9ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd11c9f0, 3 },
    { 0, 0 },
    { 0xced1c9f2, 3 },
    { 0x7e59c9f3, 3 },
    { 0x6b45c9f4, 4 },
    { 0x2eddc9f5, 3 },
    { 0, 0 },
    { 0xc6a5c9f7, 4 },
    { 0xb955c9f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc931c9fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f1ca04, 3 },
    { 0x4f59ca05, 3 },
    { 0, 0 },
    { 0xdf7dca07, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b09ca0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x792dca0d, 3 },
    { 0x3789ca0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7539ca11, 4 },
    { 0x93a1ca12, 3 },
    { 0x2575ca13, 3 },
    { 0, 0 },
    { 0x352dca15, 3 },
    { 0x4fddca16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa1ca1d, 3 },
    { 0xcc2dca1d, 4 },
    { 0x3281ca1d, 4 },
    { 0x8d5dca20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0dca29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f9ca2c, 3 },
    { 0, 0 },
    { 0xf33dca2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa151ca35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a9ca3a, 3 },
    { 0xa185ca3b, 4 },
    { 0xa8f9ca3c, 3 },
    { 0xb885ca3c, 4 },
    { 0x9e85ca3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdca43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35ca49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942dca51, 3 },
    { 0x76b5ca52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51ca55, 4 },
    { 0x8f99ca56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dca5a, 4 },
    { 0, 0 },
    { 0x2b81ca5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c51ca5f, 4 },
    { 0, 0 },
    { 0x686dca61, 4 },
    { 0, 0 },
    { 0x5a09ca63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b5ca66, 3 },
    { 0x31b5ca66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e1ca6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31ca70, 4 },
    { 0, 0 },
    { 0xcae9ca72, 3 },
    { 0x49dca73, 3 },
    { 0xb8d5ca74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b35ca77, 3 },
    { 0, 0 },
    { 0xd7a9ca79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fdca7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf9ca81, 3 },
    { 0, 0 },
    { 0x3419ca83, 3 },
    { 0, 0 },
    { 0xece1ca85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a5ca8c, 3 },
    { 0, 0 },
    { 0x2ffdca8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d9ca93, 3 },
    { 0, 0 },
    { 0xa611ca95, 3 },
    { 0x2801ca96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x981ca99, 3 },
    { 0xa9b5ca99, 4 },
    { 0xce41ca99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bdca9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb221caa5, 4 },
    { 0x2b61caa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135caaf, 3 },
    { 0xf8bdcaaf, 3 },
    { 0x4fa9cab1, 3 },
    { 0xc27dcab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349cab6, 4 },
    { 0x3f45cab6, 3 },
    { 0x97d5cab7, 3 },
    { 0x3469cab9, 4 },
    { 0x3ee9cab9, 4 },
    { 0xd01cab7, 4 },
    { 0xe999cabc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f19cac0, 3 },
    { 0x3cb9cac0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9cac4, 3 },
    { 0, 0 },
    { 0x51cdcac6, 3 },
    { 0, 0 },
    { 0x5099cac8, 4 },
    { 0xba2dcac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9545cacd, 3 },
    { 0, 0 },
    { 0x7a29cacf, 3 },
    { 0xc519cad0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa9cad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9cae1, 3 },
    { 0x5cb9cae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf9cae9, 4 },
    { 0x5a39caea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9929caed, 3 },
    { 0x5399caee, 3 },
    { 0xbec1caef, 4 },
    { 0, 0 },
    { 0x8839caf1, 4 },
    { 0, 0 },
    { 0x8831caf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc01caf9, 4 },
    { 0x5c9cafa, 3 },
    { 0x3381cafb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7009caff, 3 },
    { 0, 0 },
    { 0x2c29cb01, 3 },
    { 0, 0 },
    { 0xc241cb03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x339dcb06, 3 },
    { 0, 0 },
    { 0x3625cb08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf5cb0f, 4 },
    { 0x5a59cb10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x166dcb13, 4 },
    { 0xc325cb14, 3 },
    { 0xd371cb15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a9cb18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb731cb1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebdcb1f, 3 },
    { 0x5381cb20, 5 },
    { 0x893dcb21, 3 },
    { 0x6191cb1f, 3 },
    { 0xf45cb22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfdcb27, 3 },
    { 0x7c05cb28, 3 },
    { 0, 0 },
    { 0xacdcb2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a09cb2e, 4 },
    { 0x61adcb2f, 3 },
    { 0, 0 },
    { 0xe235cb31, 4 },
    { 0, 0 },
    { 0x82b1cb33, 3 },
    { 0xe651cb33, 3 },
    { 0xb5b5cb35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb649cb3b, 3 },
    { 0, 0 },
    { 0x32e5cb3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cddcb43, 3 },
    { 0, 0 },
    { 0x40d9cb45, 3 },
    { 0, 0 },
    { 0x7915cb47, 3 },
    { 0x88b5cb48, 3 },
    { 0x3fedcb48, 4 },
    { 0xa3f9cb4a, 4 },
    { 0xa53dcb49, 4 },
    { 0x9c39cb4b, 3 },
    { 0x5b01cb4d, 3 },
    { 0x1abdcb48, 3 },
    { 0xb099cb4f, 3 },
    { 0xfbd5cb48, 3 },
    { 0, 0 },
    { 0x9749cb52, 3 },
    { 0x4f05cb53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21cb5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2131cb63, 4 },
    { 0x4ed1cb64, 4 },
    { 0xe29cb65, 4 },
    { 0, 0 },
    { 0x47e1cb67, 3 },
    { 0x5845cb68, 3 },
    { 0xbeb1cb68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c5cb6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1cb71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319cb74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf609cb79, 3 },
    { 0x4825cb7a, 3 },
    { 0xae5dcb79, 4 },
    { 0, 0 },
    { 0x8199cb7d, 3 },
    { 0x5a11cb7e, 3 },
    { 0x344dcb7f, 3 },
    { 0x8c0dcb7f, 3 },
    { 0, 0 },
    { 0x564dcb82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd405cb85, 3 },
    { 0xf9d1cb85, 3 },
    { 0xa139cb87, 3 },
    { 0x9a69cb86, 3 },
    { 0x2629cb89, 3 },
    { 0x27d1cb89, 3 },
    { 0xb869cb89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb21cb92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b1cb98, 3 },
    { 0x4e01cb98, 3 },
    { 0xe255cb9a, 4 },
    { 0, 0 },
    { 0xd961cb9c, 3 },
    { 0x9579cb9d, 3 },
    { 0x1845cb9d, 3 },
    { 0x5fc5cb9f, 4 },
    { 0x66b1cb9d, 3 },
    { 0x8899cb9e, 3 },
    { 0x704dcba2, 3 },
    { 0xa085cba3, 3 },
    { 0x6dc5cba4, 3 },
    { 0, 0 },
    { 0xce79cba6, 3 },
    { 0, 0 },
    { 0x9a61cba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925cbac, 3 },
    { 0, 0 },
    { 0x743dcbae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1cbb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed5cbbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d5cbc1, 4 },
    { 0xeddcbc2, 3 },
    { 0x7b29cbc2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5cbc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7561cbcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c79cbd0, 3 },
    { 0, 0 },
    { 0xc17dcbd2, 3 },
    { 0x6035cbd3, 3 },
    { 0x1dbdcbd2, 4 },
    { 0x5e41cbd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb9cbdf, 3 },
    { 0xebc5cbe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b71cbe3, 3 },
    { 0x6271cbe3, 3 },
    { 0xde95cbe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef61cbe9, 3 },
    { 0, 0 },
    { 0x68edcbeb, 3 },
    { 0xd8c1cbec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b5cbef, 3 },
    { 0x8031cbf0, 3 },
    { 0, 0 },
    { 0x2571cbf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14edcbf8, 4 },
    { 0xd60dcbf9, 4 },
    { 0xd951cbfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e5cbfe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6841cc03, 3 },
    { 0, 0 },
    { 0x3639cc05, 3 },
    { 0x6311cc05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899cc09, 3 },
    { 0, 0 },
    { 0x4a25cc0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6dcc0e, 4 },
    { 0x1811cc0f, 3 },
    { 0x3ac1cc0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789cc1c, 4 },
    { 0x2f41cc1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589cc21, 3 },
    { 0x3c69cc21, 3 },
    { 0, 0 },
    { 0xd931cc24, 3 },
    { 0x5435cc25, 3 },
    { 0x7a19cc26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7dcc29, 3 },
    { 0x7cedcc29, 4 },
    { 0xc211cc2b, 3 },
    { 0x7da5cc2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdcc33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fdcc36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd229cc3a, 3 },
    { 0x11ddcc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc099cc3f, 3 },
    { 0x3e45cc40, 3 },
    { 0, 0 },
    { 0x4c95cc42, 4 },
    { 0x1335cc43, 3 },
    { 0xf719cc44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6dcc49, 4 },
    { 0xe769cc4a, 3 },
    { 0xa0adcc49, 4 },
    { 0x7631cc4c, 3 },
    { 0, 0 },
    { 0x106dcc4e, 3 },
    { 0, 0 },
    { 0x9c8dcc50, 4 },
    { 0xcaddcc51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c9cc54, 4 },
    { 0x151dcc55, 3 },
    { 0, 0 },
    { 0x6f9cc57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5809cc5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d1cc61, 4 },
    { 0, 0 },
    { 0x6529cc63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68dcc67, 3 },
    { 0xab11cc67, 3 },
    { 0xe285cc69, 3 },
    { 0xf5a1cc69, 4 },
    { 0x3609cc68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f31cc71, 3 },
    { 0x31cdcc71, 3 },
    { 0xb525cc73, 4 },
    { 0x1f19cc73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281cc7c, 4 },
    { 0x4ab9cc7c, 4 },
    { 0x8e79cc7c, 3 },
    { 0x28b5cc7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3dcc83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db9cc89, 3 },
    { 0, 0 },
    { 0x13f9cc8b, 3 },
    { 0x3705cc8b, 4 },
    { 0xf985cc8d, 3 },
    { 0, 0 },
    { 0xc49dcc8f, 3 },
    { 0, 0 },
    { 0x36c9cc91, 3 },
    { 0, 0 },
    { 0x534dcc93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2855cc96, 3 },
    { 0x9fa1cc97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd105cc9a, 4 },
    { 0x1c75cc9a, 3 },
    { 0xdf51cc9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785cc9f, 3 },
    { 0x6e91cca0, 4 },
    { 0x2ef9cca1, 3 },
    { 0xa4a9cca2, 3 },
    { 0xd9a5cca3, 4 },
    { 0xcdc5cca4, 3 },
    { 0xa0d9cca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9cca8, 4 },
    { 0, 0 },
    { 0x61b9ccaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce19ccb2, 4 },
    { 0x8559ccb2, 3 },
    { 0, 0 },
    { 0x4b65ccb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8221ccba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa105ccbf, 3 },
    { 0x6579ccbf, 3 },
    { 0xddd1ccbf, 3 },
    { 0x8fd9ccc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601ccc6, 3 },
    { 0x54c5ccc7, 3 },
    { 0xfca9ccc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f1ccd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9ccd4, 4 },
    { 0xbb65ccd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c5ccda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81ccde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e95cce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce5cce6, 5 },
    { 0, 0 },
    { 0x1785cce8, 3 },
    { 0x642dcce8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5ccef, 3 },
    { 0x6f69ccef, 3 },
    { 0x66a9ccef, 3 },
    { 0x22a1ccf0, 3 },
    { 0xd75dccf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc1ccf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb5ccf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bdccfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da5cd00, 3 },
    { 0, 0 },
    { 0xf9c9cd02, 4 },
    { 0xa1b9cd03, 3 },
    { 0xb6ddcd04, 3 },
    { 0xf44dcd05, 4 },
    { 0xaae9cd05, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f31cd0b, 3 },
    { 0xa9d5cd0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61dcd11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb5cd16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba09cd1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c1cd1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5edcd21, 4 },
    { 0x87dcd22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bdcd28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d1cd2c, 4 },
    { 0x7a51cd2d, 4 },
    { 0xf015cd2d, 5 },
    { 0x3669cd2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dcd32, 4 },
    { 0, 0 },
    { 0x32ddcd34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f1cd39, 3 },
    { 0, 0 },
    { 0x33b9cd3b, 3 },
    { 0, 0 },
    { 0x76a5cd3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21dcd44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb31cd48, 3 },
    { 0x2031cd49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb1cd4c, 3 },
    { 0, 0 },
    { 0x70f1cd4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97dcd55, 4 },
    { 0x70f1cd56, 3 },
    { 0xeb65cd56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e1cd5d, 3 },
    { 0, 0 },
    { 0x53edcd5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558dcd63, 3 },
    { 0, 0 },
    { 0x2e05cd65, 5 },
    { 0x9179cd66, 3 },
    { 0xb575cd67, 3 },
    { 0, 0 },
    { 0x421cd69, 3 },
    { 0x2ca5cd6a, 4 },
    { 0x6965cd69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa715cd6e, 4 },
    { 0x3441cd6f, 3 },
    { 0x2221cd6e, 3 },
    { 0x57adcd70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f31cd74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5275cd7a, 4 },
    { 0x111dcd7a, 3 },
    { 0x1d09cd7a, 3 },
    { 0xc11cd7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3dcd83, 3 },
    { 0xca49cd83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8879cd87, 5 },
    { 0x5349cd88, 4 },
    { 0x29b5cd89, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd79cd8e, 3 },
    { 0, 0 },
    { 0x8541cd90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x978dcd93, 3 },
    { 0, 0 },
    { 0x140dcd95, 3 },
    { 0x1865cd96, 4 },
    { 0xfc75cd97, 3 },
    { 0x70f1cd98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85cd9e, 4 },
    { 0x1739cd9f, 3 },
    { 0xde31cda0, 3 },
    { 0x8aa1cda1, 4 },
    { 0x3de1cda2, 3 },
    { 0x151dcda3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aadcdaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7dcdb3, 3 },
    { 0, 0 },
    { 0xa0d5cdb5, 4 },
    { 0x5131cdb6, 3 },
    { 0, 0 },
    { 0xa939cdb8, 3 },
    { 0x3b71cdb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430dcdbd, 3 },
    { 0x3625cdbe, 3 },
    { 0xd29cdbe, 3 },
    { 0xb7b9cdbd, 3 },
    { 0, 0 },
    { 0xcd81cdc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb845cdc5, 3 },
    { 0xc8b5cdc6, 4 },
    { 0xcf2dcdc7, 4 },
    { 0x5a85cdc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef9cdce, 4 },
    { 0xf3adcdcf, 3 },
    { 0, 0 },
    { 0x58c9cdd1, 4 },
    { 0, 0 },
    { 0xe951cdd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa435cdd8, 3 },
    { 0x7309cdd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0adcdde, 3 },
    { 0xdfc5cdde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9199cde2, 4 },
    { 0x6081cde3, 4 },
    { 0xbb91cde4, 3 },
    { 0xf821cde5, 3 },
    { 0x34a5cde6, 3 },
    { 0x8ab9cde7, 4 },
    { 0x3591cde8, 3 },
    { 0x3621cde9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae1cdf1, 3 },
    { 0, 0 },
    { 0x5dc9cdf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815cdf9, 3 },
    { 0x9b69cdfa, 3 },
    { 0x7e3dcdfa, 4 },
    { 0x74f5cdfc, 3 },
    { 0, 0 },
    { 0x7169cdfe, 3 },
    { 0, 0 },
    { 0xe969ce00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe19ce03, 4 },
    { 0x2905ce03, 3 },
    { 0x8ce5ce05, 3 },
    { 0xf2cdce05, 3 },
    { 0xfee9ce07, 5 },
    { 0, 0 },
    { 0x5ca1ce09, 4 },
    { 0xf1fdce0a, 3 },
    { 0xbe85ce0a, 3 },
    { 0, 0 },
    { 0x73a1ce0d, 3 },
    { 0x6f89ce0d, 3 },
    { 0xb129ce0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bdce19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301ce1e, 4 },
    { 0x219dce1f, 3 },
    { 0x9fe5ce1f, 3 },
    { 0x27f9ce21, 3 },
    { 0, 0 },
    { 0x8159ce23, 3 },
    { 0x9c29ce24, 3 },
    { 0, 0 },
    { 0xb409ce26, 4 },
    { 0x4161ce27, 3 },
    { 0x1371ce28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdce2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4379ce30, 3 },
    { 0xe021ce31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f5ce35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2785ce3a, 3 },
    { 0xb0bdce3b, 3 },
    { 0x1cc5ce3a, 3 },
    { 0xc8f5ce3a, 3 },
    { 0xde41ce3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e1ce41, 3 },
    { 0, 0 },
    { 0x5689ce43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8319ce48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f41ce4c, 3 },
    { 0, 0 },
    { 0xafe5ce4e, 3 },
    { 0, 0 },
    { 0x6bdce50, 3 },
    { 0, 0 },
    { 0x9e51ce52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629ce55, 4 },
    { 0x7705ce55, 3 },
    { 0xbb9dce57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1ce5a, 3 },
    { 0x898dce5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9dce5f, 3 },
    { 0xa349ce60, 3 },
    { 0x6fbdce5f, 3 },
    { 0xe7cdce62, 3 },
    { 0xf461ce5f, 3 },
    { 0x8449ce63, 3 },
    { 0x8c95ce65, 4 },
    { 0, 0 },
    { 0xba11ce67, 3 },
    { 0xe56dce68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfdce6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5ce71, 3 },
    { 0, 0 },
    { 0xa89ce73, 4 },
    { 0x6a0dce74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2dce77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911ce7a, 4 },
    { 0x2669ce7b, 3 },
    { 0x3d7dce7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78adce82, 4 },
    { 0, 0 },
    { 0x29d1ce84, 3 },
    { 0x1d01ce85, 5 },
    { 0xa6c5ce86, 4 },
    { 0x13d1ce86, 3 },
    { 0xb619ce86, 3 },
    { 0x4c89ce86, 3 },
    { 0xa379ce87, 4 },
    { 0xec01ce88, 4 },
    { 0x943dce84, 3 },
    { 0x5831ce8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809ce92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdce9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b5ce9f, 3 },
    { 0, 0 },
    { 0xc231cea1, 3 },
    { 0, 0 },
    { 0xe045cea3, 3 },
    { 0, 0 },
    { 0x4081cea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6dcea8, 3 },
    { 0x70b9cea9, 3 },
    { 0xada5ceaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa791ceaf, 3 },
    { 0, 0 },
    { 0x9649ceb1, 3 },
    { 0x574dceb1, 3 },
    { 0x7615ceb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9ceb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4949cebb, 3 },
    { 0x9e1dcebc, 3 },
    { 0x830dcebd, 3 },
    { 0, 0 },
    { 0x65a9cebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bdcec7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec95cecc, 3 },
    { 0xb8f1cecd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4791ced0, 3 },
    { 0x181ced1, 3 },
    { 0x7d9ced2, 3 },
    { 0, 0 },
    { 0x7dd5ced4, 3 },
    { 0xffe9ced4, 5 },
    { 0x4db1ced6, 3 },
    { 0x15f9ced7, 3 },
    { 0, 0 },
    { 0xf8b9ced9, 3 },
    { 0, 0 },
    { 0x652dcedb, 4 },
    { 0, 0 },
    { 0x881dcedd, 3 },
    { 0xc27dcede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef45cee4, 3 },
    { 0, 0 },
    { 0x690dcee6, 3 },
    { 0x9f01cee7, 3 },
    { 0, 0 },
    { 0xb6fdcee9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf535ceec, 3 },
    { 0xdeb9ceed, 4 },
    { 0x4071ceee, 3 },
    { 0x5a1dceef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d15cef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b9cefc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8525cf03, 3 },
    { 0x87cdcf04, 3 },
    { 0x8d45cf04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71cf09, 3 },
    { 0x3f61cf0a, 3 },
    { 0x425cf0b, 4 },
    { 0x5621cf0c, 3 },
    { 0x5c29cf0b, 3 },
    { 0x3f69cf0d, 3 },
    { 0x84c1cf0e, 3 },
    { 0xd77dcf0c, 4 },
    { 0xffddcf11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe349cf18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dcf1d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cdcf21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6529cf27, 3 },
    { 0x9a31cf27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7809cf2c, 4 },
    { 0xdf79cf2d, 3 },
    { 0, 0 },
    { 0xa84dcf2f, 3 },
    { 0, 0 },
    { 0x9e15cf31, 3 },
    { 0xd221cf32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5121cf35, 3 },
    { 0xbd15cf36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bdcf3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531cf40, 3 },
    { 0x9c95cf41, 3 },
    { 0xdc79cf40, 3 },
    { 0x37a5cf43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b09cf46, 3 },
    { 0x1aedcf46, 3 },
    { 0xeaf5cf48, 3 },
    { 0, 0 },
    { 0x9d55cf4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d39cf4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03dcf51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b1cf58, 3 },
    { 0xf9f1cf59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4171cf5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3035cf65, 4 },
    { 0, 0 },
    { 0xc615cf67, 3 },
    { 0xe615cf68, 3 },
    { 0xbb4dcf68, 3 },
    { 0x95d1cf68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6275cf6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839cf75, 3 },
    { 0xb41cf76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5721cf81, 3 },
    { 0x1921cf82, 3 },
    { 0x7af9cf83, 4 },
    { 0, 0 },
    { 0x21edcf85, 3 },
    { 0xc3a9cf85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f25cf8a, 3 },
    { 0xd3d1cf8b, 3 },
    { 0x5175cf8c, 4 },
    { 0x3b1dcf8b, 4 },
    { 0x3a5dcf8e, 3 },
    { 0x825cf8c, 3 },
    { 0, 0 },
    { 0x4badcf91, 3 },
    { 0x5a61cf92, 3 },
    { 0, 0 },
    { 0x20f5cf94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45cf9a, 4 },
    { 0, 0 },
    { 0x36a9cf9c, 3 },
    { 0, 0 },
    { 0xb0a5cf9e, 5 },
    { 0x79ddcf9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d5cfa3, 3 },
    { 0, 0 },
    { 0xf199cfa5, 3 },
    { 0, 0 },
    { 0xd211cfa7, 3 },
    { 0, 0 },
    { 0xeeedcfa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e1cfac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d49cfb3, 3 },
    { 0x563dcfb4, 3 },
    { 0x2f81cfb5, 3 },
    { 0, 0 },
    { 0xdaf1cfb7, 3 },
    { 0x39f9cfb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d41cfbe, 3 },
    { 0xabf9cfbf, 3 },
    { 0x84d5cfbf, 3 },
    { 0xda85cfbe, 4 },
    { 0x934dcfc0, 3 },
    { 0x4249cfc1, 3 },
    { 0x6a9cfc3, 3 },
    { 0x7405cfc5, 3 },
    { 0xa7a5cfc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fdcfcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515cfd1, 3 },
    { 0, 0 },
    { 0x6ad9cfd3, 3 },
    { 0x13b5cfd4, 5 },
    { 0, 0 },
    { 0xacddcfd6, 4 },
    { 0x82b5cfd7, 3 },
    { 0xea91cfd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecdcfdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcfde, 4 },
    { 0x3b7dcfdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x715dcfe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c11cfe8, 3 },
    { 0x163dcfe9, 3 },
    { 0xcd9dcfea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6245cfed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e9cff0, 3 },
    { 0xd331cff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33dcff5, 4 },
    { 0, 0 },
    { 0x1299cff7, 3 },
    { 0, 0 },
    { 0x9099cff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775cffc, 3 },
    { 0xad59cffd, 3 },
    { 0x7fadcffe, 3 },
    { 0xbd2dcfff, 3 },
    { 0xd9b9cfff, 3 },
    { 0x959d000, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1355d00c, 4 },
    { 0x62f1d00d, 3 },
    { 0, 0 },
    { 0xf5cdd00f, 3 },
    { 0, 0 },
    { 0x74d9d011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dd017, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d1d01a, 4 },
    { 0x7c4dd01b, 3 },
    { 0x2269d01c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1d022, 3 },
    { 0xbccdd023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8275d02b, 3 },
    { 0x9351d02c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541dd031, 3 },
    { 0xf8dd032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b9d03b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fdd040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef1d043, 3 },
    { 0x879dd044, 4 },
    { 0xdb4dd045, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa59d049, 3 },
    { 0xf8f1d04a, 3 },
    { 0x3bbdd04b, 3 },
    { 0x45bdd04b, 3 },
    { 0x53d9d049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e9d053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7879d058, 3 },
    { 0xf04dd058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b5d060, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05d064, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9d068, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1869d06b, 3 },
    { 0x7549d06c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a1d071, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1dd074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3129d077, 3 },
    { 0, 0 },
    { 0x915d079, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8add080, 3 },
    { 0xfe31d080, 3 },
    { 0x2a7dd081, 4 },
    { 0x22ddd083, 3 },
    { 0xa325d083, 3 },
    { 0x2555d085, 4 },
    { 0xfcc1d086, 3 },
    { 0x51e5d086, 3 },
    { 0xa06dd086, 3 },
    { 0, 0 },
    { 0xe8b1d08a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1841d090, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3525d094, 4 },
    { 0, 0 },
    { 0x1b95d096, 3 },
    { 0, 0 },
    { 0x826dd098, 3 },
    { 0x4f39d099, 4 },
    { 0x1211d099, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2809d0a4, 3 },
    { 0xa741d0a5, 3 },
    { 0x57a1d0a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f5d0ac, 4 },
    { 0xef85d0ac, 3 },
    { 0x6905d0ac, 3 },
    { 0x5e5dd0ae, 3 },
    { 0x589dd0b0, 3 },
    { 0, 0 },
    { 0x8879d0b2, 3 },
    { 0x6c49d0b2, 3 },
    { 0xbacdd0b2, 3 },
    { 0x1a51d0b2, 4 },
    { 0, 0 },
    { 0xf36dd0b7, 3 },
    { 0x89b1d0b7, 4 },
    { 0xdcf5d0b7, 3 },
    { 0x62edd0b8, 4 },
    { 0x88a9d0b8, 3 },
    { 0, 0 },
    { 0xb65dd0bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e65d0c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e41d0c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b85d0c6, 4 },
    { 0, 0 },
    { 0xc66dd0c8, 4 },
    { 0xb0ddd0c9, 3 },
    { 0, 0 },
    { 0xf761d0cb, 3 },
    { 0x75f5d0cc, 5 },
    { 0x7cb5d0cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4471d0d3, 3 },
    { 0x96b1d0d4, 3 },
    { 0x1a3dd0d5, 3 },
    { 0xd425d0d3, 4 },
    { 0xab31d0d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b29d0da, 3 },
    { 0x9b05d0db, 3 },
    { 0, 0 },
    { 0xee99d0dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949d0e3, 4 },
    { 0, 0 },
    { 0x8d31d0e5, 5 },
    { 0, 0 },
    { 0x70c1d0e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675d0ee, 3 },
    { 0x919dd0ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa089d0f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5dd105, 3 },
    { 0xe049d106, 4 },
    { 0x8fbdd107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31d10b, 4 },
    { 0x10f1d10c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d5d10f, 3 },
    { 0x5ac5d10f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b65d113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2781d116, 3 },
    { 0xcde1d117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dd122, 3 },
    { 0, 0 },
    { 0x1879d124, 3 },
    { 0xcef1d124, 4 },
    { 0x5c35d126, 3 },
    { 0x6d1d127, 4 },
    { 0x4a6dd127, 3 },
    { 0x3c25d128, 4 },
    { 0x3f7dd129, 4 },
    { 0x3a91d12b, 3 },
    { 0xeb3dd12c, 3 },
    { 0xd41dd12d, 3 },
    { 0xfd9d12e, 3 },
    { 0xc7b5d12d, 3 },
    { 0xbd01d12f, 3 },
    { 0x6439d131, 3 },
    { 0xe161d12f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d135, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bdd13b, 3 },
    { 0xe05d13c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d5d141, 3 },
    { 0x3055d142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d149, 3 },
    { 0, 0 },
    { 0x9679d14b, 4 },
    { 0x2bedd14b, 4 },
    { 0xcf9d14d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9dd152, 4 },
    { 0xa515d153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e1d158, 4 },
    { 0x4e1dd159, 3 },
    { 0xca05d15a, 3 },
    { 0xb831d15b, 3 },
    { 0xfa95d15c, 3 },
    { 0x901dd15d, 3 },
    { 0x4561d15d, 3 },
    { 0x510dd15f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16dd165, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4645d16f, 3 },
    { 0xf90dd170, 4 },
    { 0xd509d170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1585d174, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf501d17c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418dd182, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e55d185, 3 },
    { 0xa18dd186, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9075d18a, 3 },
    { 0x4101d18a, 3 },
    { 0, 0 },
    { 0x6d71d18d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf69d190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6869d194, 3 },
    { 0, 0 },
    { 0x4b1dd196, 4 },
    { 0xd231d196, 4 },
    { 0x3c05d198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fdd19f, 3 },
    { 0, 0 },
    { 0x73cdd1a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdd1a4, 3 },
    { 0, 0 },
    { 0x195dd1a6, 3 },
    { 0xed5d1a7, 3 },
    { 0, 0 },
    { 0x6a9dd1a9, 4 },
    { 0x2325d1a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2dd1ad, 4 },
    { 0x26b5d1ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3705d1b3, 3 },
    { 0x9451d1b4, 3 },
    { 0x6825d1b5, 3 },
    { 0x81c1d1b5, 3 },
    { 0xf691d1b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd45d1c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9d1cb, 3 },
    { 0, 0 },
    { 0xea25d1cd, 5 },
    { 0xcb89d1cd, 3 },
    { 0x9aa9d1cf, 3 },
    { 0xa65d1cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59add1d4, 3 },
    { 0, 0 },
    { 0xb7edd1d6, 3 },
    { 0x71e9d1d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed5d1da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8231d1e5, 3 },
    { 0, 0 },
    { 0xe795d1e7, 4 },
    { 0x9471d1e8, 3 },
    { 0xe345d1e9, 3 },
    { 0, 0 },
    { 0x2409d1eb, 4 },
    { 0, 0 },
    { 0x299d1ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d19d1f0, 3 },
    { 0, 0 },
    { 0x26fdd1f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef9d1f7, 5 },
    { 0x8ac1d1f8, 3 },
    { 0xfe19d1f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db5d202, 3 },
    { 0, 0 },
    { 0x2b75d204, 3 },
    { 0, 0 },
    { 0x8ab5d206, 4 },
    { 0, 0 },
    { 0xa3e1d208, 3 },
    { 0xece1d209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab55d214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500dd218, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa689d220, 3 },
    { 0x3fc9d220, 3 },
    { 0x5ddd222, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a1d226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5285d229, 4 },
    { 0, 0 },
    { 0x8309d22b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907dd22f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6835d232, 3 },
    { 0x7fcdd233, 4 },
    { 0, 0 },
    { 0x9d55d235, 3 },
    { 0xff5dd235, 3 },
    { 0xcc51d237, 3 },
    { 0x73d1d238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1d23d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e89d242, 3 },
    { 0xcbd9d243, 4 },
    { 0xc6edd244, 3 },
    { 0xf5d245, 3 },
    { 0x707dd244, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdd249, 3 },
    { 0, 0 },
    { 0xdb49d24b, 3 },
    { 0xad49d24c, 3 },
    { 0x8631d24d, 3 },
    { 0x5c89d24d, 3 },
    { 0x14dd24e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704dd253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd1d256, 4 },
    { 0, 0 },
    { 0x8631d258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f5d25b, 3 },
    { 0, 0 },
    { 0x5549d25d, 3 },
    { 0x11a5d25e, 3 },
    { 0, 0 },
    { 0xb071d260, 5 },
    { 0x4e8dd261, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3dd264, 3 },
    { 0xacf9d265, 4 },
    { 0x5aadd264, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91dd26c, 3 },
    { 0, 0 },
    { 0xda4dd26e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9209d276, 3 },
    { 0, 0 },
    { 0x6cedd278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3109d27e, 3 },
    { 0x9a05d27e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f9d288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef75d28b, 5 },
    { 0x48b9d28c, 3 },
    { 0x9e95d28c, 3 },
    { 0, 0 },
    { 0x9c01d28f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3251d292, 3 },
    { 0x74f9d293, 3 },
    { 0x9a15d292, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb871d297, 3 },
    { 0, 0 },
    { 0x6161d299, 3 },
    { 0, 0 },
    { 0xdc49d29b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05dd29f, 3 },
    { 0, 0 },
    { 0x415dd2a1, 3 },
    { 0x8ae9d2a1, 3 },
    { 0x653dd2a3, 4 },
    { 0, 0 },
    { 0x495d2a5, 3 },
    { 0, 0 },
    { 0xbf11d2a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809dd2b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa831d2b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85dd2b7, 4 },
    { 0x34a9d2b7, 3 },
    { 0x5fc1d2b7, 3 },
    { 0xe2c9d2b7, 4 },
    { 0x827dd2bb, 3 },
    { 0x2431d2bb, 3 },
    { 0, 0 },
    { 0xa141d2be, 3 },
    { 0x938dd2bf, 3 },
    { 0x4da9d2c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71dd2ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb349d2cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209d2d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc159d2d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bdd2db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f1d2e0, 3 },
    { 0xba1dd2e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa399d2e8, 3 },
    { 0x3a21d2e8, 3 },
    { 0xec19d2e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21dd2ee, 3 },
    { 0, 0 },
    { 0xd331d2f0, 3 },
    { 0xf961d2f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de5d2f4, 4 },
    { 0, 0 },
    { 0xf505d2f6, 3 },
    { 0x6329d2f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c9d2fc, 3 },
    { 0, 0 },
    { 0x4fd5d2fe, 3 },
    { 0, 0 },
    { 0x6a41d300, 3 },
    { 0, 0 },
    { 0xc129d302, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fdd306, 3 },
    { 0, 0 },
    { 0x6519d308, 3 },
    { 0, 0 },
    { 0xa80dd30a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5995d30d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dd312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc59d316, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9d320, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe345d324, 3 },
    { 0, 0 },
    { 0xffe9d326, 5 },
    { 0xcdadd327, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f5d32d, 3 },
    { 0xc9d9d32e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc1d338, 3 },
    { 0x1115d338, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9d33d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc461d340, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435d343, 3 },
    { 0x3555d343, 3 },
    { 0x9101d345, 3 },
    { 0x937dd346, 3 },
    { 0x9e5dd344, 3 },
    { 0xfdadd348, 3 },
    { 0x2cbdd349, 4 },
    { 0x151dd34a, 3 },
    { 0xb9d1d34a, 3 },
    { 0x432dd347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d9d353, 3 },
    { 0x5805d354, 3 },
    { 0x6721d355, 3 },
    { 0x59c1d356, 3 },
    { 0, 0 },
    { 0xe701d358, 3 },
    { 0x1365d358, 3 },
    { 0x1f75d359, 3 },
    { 0, 0 },
    { 0x315d35c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7659d363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d5d36a, 3 },
    { 0, 0 },
    { 0xbb91d36c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85dd36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9951d372, 4 },
    { 0x316dd373, 3 },
    { 0x75e1d373, 3 },
    { 0x265d375, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4075d379, 3 },
    { 0, 0 },
    { 0x1649d37b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8455d386, 4 },
    { 0x12add387, 3 },
    { 0, 0 },
    { 0x596dd389, 4 },
    { 0xf199d38a, 3 },
    { 0, 0 },
    { 0x1e19d38c, 3 },
    { 0x479d38d, 3 },
    { 0x7965d38d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd595d39a, 3 },
    { 0x1611d39b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c9d3a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe1d3a8, 3 },
    { 0, 0 },
    { 0xa711d3aa, 4 },
    { 0x3b3dd3ab, 4 },
    { 0, 0 },
    { 0x29f5d3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a5d3b0, 3 },
    { 0, 0 },
    { 0x7665d3b2, 4 },
    { 0x4f11d3b3, 4 },
    { 0xed11d3b4, 3 },
    { 0xad2dd3b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc379d3b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc69d3be, 4 },
    { 0xa6b1d3be, 3 },
    { 0, 0 },
    { 0x21d5d3c1, 3 },
    { 0x1e85d3c2, 3 },
    { 0xb635d3c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb51d3c7, 4 },
    { 0, 0 },
    { 0x6a01d3c9, 3 },
    { 0x7f79d3ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4509d3cf, 4 },
    { 0, 0 },
    { 0xd6d1d3d1, 3 },
    { 0, 0 },
    { 0xadd5d3d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd79d3d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d5d3df, 4 },
    { 0xc6a9d3e0, 3 },
    { 0xe4fdd3df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a9d3e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6089d3ec, 3 },
    { 0, 0 },
    { 0xb851d3ee, 3 },
    { 0, 0 },
    { 0xda75d3f0, 3 },
    { 0x2f7dd3f1, 3 },
    { 0x8b45d3f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c1d3f7, 3 },
    { 0xdb7dd3f8, 4 },
    { 0x8d65d3f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb1d3fd, 3 },
    { 0xdef1d3fe, 3 },
    { 0, 0 },
    { 0x4215d400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c1d404, 3 },
    { 0, 0 },
    { 0x22e9d406, 4 },
    { 0xef55d407, 4 },
    { 0xc65dd407, 3 },
    { 0x6eddd409, 4 },
    { 0x21e1d40a, 3 },
    { 0xd929d40b, 4 },
    { 0xf3d1d40c, 5 },
    { 0xaf15d40d, 4 },
    { 0, 0 },
    { 0xa4e1d40f, 3 },
    { 0xbbb9d40f, 4 },
    { 0x2595d411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e9d414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27add41d, 3 },
    { 0x44fdd41d, 4 },
    { 0x22c9d41f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dd426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5d429, 3 },
    { 0x7265d42a, 3 },
    { 0xfe99d42b, 3 },
    { 0xe919d42c, 3 },
    { 0, 0 },
    { 0xafd1d42e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8dd432, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f9d436, 4 },
    { 0, 0 },
    { 0xd511d438, 3 },
    { 0, 0 },
    { 0xc119d43a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1605d43f, 4 },
    { 0xd59d43f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3481d443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2721d447, 3 },
    { 0, 0 },
    { 0x7a25d449, 3 },
    { 0, 0 },
    { 0x1851d44b, 3 },
    { 0x8829d44c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b9d452, 3 },
    { 0x57c5d453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x133dd456, 3 },
    { 0x237dd456, 3 },
    { 0xa091d458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121dd45c, 3 },
    { 0xfb55d45d, 3 },
    { 0xc9ddd45e, 3 },
    { 0, 0 },
    { 0x5525d460, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8dd463, 3 },
    { 0, 0 },
    { 0xdd4dd465, 3 },
    { 0x58d5d466, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f9d46e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e71d472, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e5d478, 4 },
    { 0, 0 },
    { 0xcbb9d47a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c5d492, 3 },
    { 0xcd91d492, 3 },
    { 0x7a99d493, 4 },
    { 0x617dd492, 3 },
    { 0xea5dd493, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f25d499, 4 },
    { 0xf04dd499, 3 },
    { 0xf585d499, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7235d4a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1495d4a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d09d4a7, 3 },
    { 0, 0 },
    { 0x1ca9d4a9, 4 },
    { 0xe109d4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa391d4ae, 3 },
    { 0xed5d4ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2401d4b3, 4 },
    { 0, 0 },
    { 0xa7e9d4b5, 4 },
    { 0, 0 },
    { 0x3911d4b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fdd4bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386dd4c0, 3 },
    { 0, 0 },
    { 0x3079d4c2, 3 },
    { 0x60e9d4c3, 5 },
    { 0x88c1d4c4, 3 },
    { 0xaaf1d4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349d4c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab5d4cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639d4d2, 3 },
    { 0xf4c5d4d2, 3 },
    { 0x4589d4d4, 3 },
    { 0xfcb1d4d2, 4 },
    { 0xe9e9d4d6, 3 },
    { 0x2305d4d5, 3 },
    { 0xc64dd4d8, 5 },
    { 0x80f1d4d6, 4 },
    { 0x6cc1d4da, 3 },
    { 0, 0 },
    { 0x315d4dc, 4 },
    { 0x6e35d4dd, 4 },
    { 0x8ba1d4dc, 3 },
    { 0x52f9d4df, 3 },
    { 0x82a9d4e0, 3 },
    { 0x86e5d4df, 3 },
    { 0xe38dd4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd1d4e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebdd4ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6dd4f5, 4 },
    { 0xe98dd4f6, 4 },
    { 0x90b9d4f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd4fa, 3 },
    { 0x5839d4fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb919d503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52dd506, 3 },
    { 0xeda9d507, 3 },
    { 0xad21d507, 3 },
    { 0x5665d509, 3 },
    { 0x5c75d50a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaadd50e, 3 },
    { 0x94add50f, 4 },
    { 0xdb6dd50e, 3 },
    { 0xfef9d510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d514, 3 },
    { 0x7cadd515, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a1d51a, 4 },
    { 0xc211d51b, 3 },
    { 0x6805d51c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf631d51f, 3 },
    { 0x9ffdd520, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd1d524, 3 },
    { 0xaa61d525, 3 },
    { 0x313dd524, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9d529, 3 },
    { 0, 0 },
    { 0x6ff1d52b, 3 },
    { 0x5ad5d52b, 4 },
    { 0, 0 },
    { 0x90cdd52e, 3 },
    { 0x1ff1d52e, 4 },
    { 0x5edd530, 5 },
    { 0x45b9d52f, 3 },
    { 0, 0 },
    { 0x4f35d533, 4 },
    { 0, 0 },
    { 0x4f79d535, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad45d538, 3 },
    { 0x6fddd538, 3 },
    { 0x786dd538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b9d540, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd349d545, 3 },
    { 0xfde9d545, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7dd549, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15d551, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x273dd554, 3 },
    { 0, 0 },
    { 0x5a31d556, 3 },
    { 0xb381d557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb5d55a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b85d55d, 4 },
    { 0x33b9d55e, 4 },
    { 0x51b1d55d, 3 },
    { 0x23dd560, 3 },
    { 0x145dd55e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85d566, 3 },
    { 0x6ce5d567, 3 },
    { 0x6af9d567, 3 },
    { 0xbe39d566, 3 },
    { 0x1d25d56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e9d572, 4 },
    { 0x1885d572, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4dd576, 3 },
    { 0xadf1d577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3315d57b, 4 },
    { 0xd771d57c, 3 },
    { 0xce15d57c, 3 },
    { 0xe1b9d57e, 3 },
    { 0, 0 },
    { 0xa76dd580, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5d584, 4 },
    { 0x5ffdd585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79d58d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae1d595, 3 },
    { 0x1c59d595, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf15d599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79d5a0, 4 },
    { 0, 0 },
    { 0x9cedd5a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae15d5a5, 4 },
    { 0xda5d5a5, 5 },
    { 0x833dd5a7, 4 },
    { 0x3411d5a6, 3 },
    { 0x8199d5a7, 3 },
    { 0xe2c9d5aa, 4 },
    { 0x5181d5aa, 3 },
    { 0xa421d5aa, 3 },
    { 0, 0 },
    { 0x8e65d5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5d5b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dd5b4, 3 },
    { 0, 0 },
    { 0xd6a5d5b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x670dd5b9, 3 },
    { 0x2c75d5ba, 4 },
    { 0x52e5d5bb, 3 },
    { 0x84e9d5bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5739d5c3, 3 },
    { 0x3a21d5c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302dd5c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b55d5cc, 3 },
    { 0x336dd5cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe985d5d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d5d5d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe725d5d6, 3 },
    { 0x2d5d5d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9d5db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c79d5eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5d5f1, 3 },
    { 0, 0 },
    { 0x2be5d5f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f9d5ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9495d605, 3 },
    { 0, 0 },
    { 0x1c05d607, 3 },
    { 0x44add608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7691d60c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe161d612, 4 },
    { 0x5cf5d613, 3 },
    { 0x77a5d614, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435d61c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dd621, 3 },
    { 0xaff5d622, 3 },
    { 0xdb65d623, 3 },
    { 0x7ea9d624, 5 },
    { 0x8241d622, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd619d628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fdd62f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bdd634, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c5d639, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de5d63e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce9d642, 3 },
    { 0x8b9d642, 4 },
    { 0xea61d643, 3 },
    { 0x329dd645, 3 },
    { 0x1bfdd646, 4 },
    { 0xeed9d647, 3 },
    { 0, 0 },
    { 0x915d649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc309d64e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3dd653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd151d658, 3 },
    { 0xce19d659, 4 },
    { 0, 0 },
    { 0x5fc5d65b, 4 },
    { 0, 0 },
    { 0x97f5d65d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33dd662, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423dd669, 3 },
    { 0, 0 },
    { 0xddb9d66b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc01d674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11d677, 3 },
    { 0x2a25d678, 3 },
    { 0xf0a1d678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d55d681, 3 },
    { 0, 0 },
    { 0x7441d683, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9785d686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521d68a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf649d68d, 4 },
    { 0x5585d68e, 3 },
    { 0xb715d68f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa39d696, 3 },
    { 0x9e99d697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7dd69f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515d6a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c9d6a9, 3 },
    { 0xd419d6aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee9d6af, 3 },
    { 0xb3bdd6b0, 3 },
    { 0x825dd6b1, 4 },
    { 0x8971d6b1, 5 },
    { 0xc935d6b3, 3 },
    { 0x7c6dd6b3, 3 },
    { 0, 0 },
    { 0x773dd6b6, 3 },
    { 0xb6d5d6b7, 4 },
    { 0xd3f5d6b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dd6be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a25d6c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa15d6d1, 4 },
    { 0x48ddd6d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85dd6db, 3 },
    { 0x1abdd6dc, 3 },
    { 0xa849d6dd, 3 },
    { 0, 0 },
    { 0x545d6df, 3 },
    { 0x73c5d6e0, 3 },
    { 0, 0 },
    { 0xe0f1d6e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2dd6e7, 3 },
    { 0x9841d6e8, 3 },
    { 0, 0 },
    { 0x78d5d6ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a95d6ed, 3 },
    { 0x1e29d6ee, 4 },
    { 0x9e21d6ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861d6f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf271d6fb, 3 },
    { 0, 0 },
    { 0xa39dd6fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3765d70a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649d70d, 3 },
    { 0x8191d70e, 3 },
    { 0xabc9d70d, 4 },
    { 0x9789d70e, 3 },
    { 0, 0 },
    { 0xf751d712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c9d715, 3 },
    { 0x43d1d716, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f9d71d, 4 },
    { 0, 0 },
    { 0x82c1d71f, 3 },
    { 0x7d15d71f, 3 },
    { 0xe075d721, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3dd725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4dd728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed5d72c, 3 },
    { 0x524dd72c, 3 },
    { 0xd2cdd72d, 3 },
    { 0xa831d72f, 4 },
    { 0xfb89d730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5039d733, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5d736, 3 },
    { 0, 0 },
    { 0x7009d738, 4 },
    { 0xafe1d739, 3 },
    { 0, 0 },
    { 0x60e9d73b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaadd73f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a5d742, 3 },
    { 0xc831d742, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa029d746, 3 },
    { 0x4c1d746, 3 },
    { 0, 0 },
    { 0xbee1d749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7865d74d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91d759, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d5d75e, 5 },
    { 0xcb1d75f, 3 },
    { 0x4025d760, 3 },
    { 0xa131d760, 4 },
    { 0, 0 },
    { 0xa21dd763, 3 },
    { 0xabdd763, 3 },
    { 0x4a95d765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb639d768, 3 },
    { 0xbeb9d769, 3 },
    { 0, 0 },
    { 0x5561d76b, 3 },
    { 0xf575d76c, 3 },
    { 0, 0 },
    { 0x2549d76e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc211d771, 3 },
    { 0x33fdd771, 4 },
    { 0x6719d773, 4 },
    { 0xbfa5d771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a69d77d, 3 },
    { 0x61cdd77e, 3 },
    { 0xa651d77f, 4 },
    { 0xcb69d77e, 5 },
    { 0x26d1d781, 4 },
    { 0xae95d782, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e55d786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee9d78d, 4 },
    { 0x781d78d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe959d793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8635d799, 3 },
    { 0xe4d1d79a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe135d79d, 3 },
    { 0, 0 },
    { 0xeaa1d79f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf541d7ac, 4 },
    { 0xfb65d7ad, 4 },
    { 0xa0bdd7ae, 3 },
    { 0, 0 },
    { 0xdc35d7b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4381d7b9, 4 },
    { 0xa5f1d7ba, 4 },
    { 0, 0 },
    { 0x8561d7bc, 5 },
    { 0, 0 },
    { 0xb979d7be, 3 },
    { 0xa3f1d7be, 3 },
    { 0xcabdd7c0, 4 },
    { 0x3135d7c1, 4 },
    { 0xb381d7c2, 3 },
    { 0xc005d7c1, 4 },
    { 0x74b5d7c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe271d7ca, 3 },
    { 0x50edd7ca, 4 },
    { 0, 0 },
    { 0x8611d7cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fdd7d1, 3 },
    { 0xd799d7d2, 4 },
    { 0, 0 },
    { 0xcfc5d7d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b5d7d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39add7dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6175d7e0, 4 },
    { 0x8badd7e0, 4 },
    { 0xd335d7e0, 5 },
    { 0xfa99d7e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbdd7e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac5d7ec, 3 },
    { 0x9be1d7ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29d7f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b9d7fd, 3 },
    { 0x73f1d7fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fdd802, 3 },
    { 0x41d1d803, 3 },
    { 0x3eb9d804, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae1d807, 3 },
    { 0x49fdd808, 3 },
    { 0, 0 },
    { 0x8f31d80a, 3 },
    { 0, 0 },
    { 0x26a5d80c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831d812, 3 },
    { 0x1bedd813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea69d81a, 3 },
    { 0x27f1d81b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f1d81e, 3 },
    { 0, 0 },
    { 0x55e9d820, 4 },
    { 0x11c1d821, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a9d825, 3 },
    { 0xd271d825, 3 },
    { 0, 0 },
    { 0x8785d828, 3 },
    { 0x6d65d829, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cddd830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd225d833, 3 },
    { 0x6de1d833, 3 },
    { 0, 0 },
    { 0xc9e1d836, 3 },
    { 0, 0 },
    { 0x1919d838, 3 },
    { 0xa2e1d839, 3 },
    { 0, 0 },
    { 0x8c1d83b, 3 },
    { 0x67e9d83c, 3 },
    { 0x67e1d83d, 4 },
    { 0x35bdd83d, 4 },
    { 0x4be5d83b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc01d842, 3 },
    { 0x72fdd842, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1d846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745d850, 3 },
    { 0xa685d850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa179d854, 3 },
    { 0, 0 },
    { 0xdf3dd856, 3 },
    { 0xc5c9d857, 4 },
    { 0, 0 },
    { 0x315d859, 3 },
    { 0xdabdd859, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bdd860, 3 },
    { 0, 0 },
    { 0x6ea1d862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc9d866, 3 },
    { 0xb06dd867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2dd86c, 3 },
    { 0x19e1d86c, 3 },
    { 0xb7f5d86e, 3 },
    { 0x15e5d86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfddd872, 3 },
    { 0xfdfdd872, 3 },
    { 0, 0 },
    { 0x560dd875, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bdd883, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb695d887, 4 },
    { 0, 0 },
    { 0x396dd889, 3 },
    { 0xb59d88a, 3 },
    { 0x2a95d88b, 3 },
    { 0x69b9d88c, 3 },
    { 0x4439d889, 3 },
    { 0x6541d88e, 3 },
    { 0x372dd88a, 3 },
    { 0x4159d890, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe391d89c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75d8a2, 3 },
    { 0x73e9d8a3, 3 },
    { 0x70e1d8a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905d8a7, 3 },
    { 0x6d15d8a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2dd8b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dd8b6, 3 },
    { 0, 0 },
    { 0xa909d8b8, 3 },
    { 0xa989d8b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918dd8bd, 3 },
    { 0x24e9d8be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b5d8c3, 3 },
    { 0, 0 },
    { 0xd921d8c5, 3 },
    { 0x502dd8c6, 3 },
    { 0xede1d8c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bdd8cb, 3 },
    { 0xa685d8cc, 3 },
    { 0, 0 },
    { 0x6e05d8ce, 3 },
    { 0x2665d8cf, 3 },
    { 0x76f9d8d0, 4 },
    { 0x24a1d8d0, 5 },
    { 0x689dd8d0, 4 },
    { 0x7535d8d3, 4 },
    { 0x6619d8cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a1d8d9, 3 },
    { 0x4ee9d8da, 3 },
    { 0x1b4dd8db, 3 },
    { 0x1ebdd8dc, 3 },
    { 0x69e1d8dd, 4 },
    { 0, 0 },
    { 0x1721d8df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d8e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5575d8e5, 3 },
    { 0xedb5d8e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c5d8ea, 3 },
    { 0x9625d8eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb081d8ee, 3 },
    { 0xc759d8ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad41d8f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b31d8f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b5d902, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4835d908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1d90c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90dd914, 3 },
    { 0xbfe5d915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a9d918, 3 },
    { 0, 0 },
    { 0x5049d91a, 4 },
    { 0x2285d91a, 3 },
    { 0x12c9d91c, 3 },
    { 0, 0 },
    { 0xfc59d91e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7555d921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8add929, 3 },
    { 0xc1fdd929, 4 },
    { 0x6f1dd92a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc1d92e, 3 },
    { 0, 0 },
    { 0xededd930, 4 },
    { 0xa2bdd931, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d936, 5 },
    { 0xf1dd937, 3 },
    { 0xd479d938, 4 },
    { 0, 0 },
    { 0x8f75d93a, 3 },
    { 0x5bcdd93a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9d949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89d94c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee5d954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674dd95e, 4 },
    { 0x3d81d95f, 4 },
    { 0x4ef5d95e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949d965, 3 },
    { 0xe55d966, 3 },
    { 0, 0 },
    { 0x9fb5d968, 3 },
    { 0x33a1d968, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a65d974, 3 },
    { 0, 0 },
    { 0x69f9d976, 3 },
    { 0x8355d977, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a1d97a, 3 },
    { 0, 0 },
    { 0x2d15d97c, 3 },
    { 0, 0 },
    { 0x4325d97e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d1d983, 4 },
    { 0xa551d983, 3 },
    { 0, 0 },
    { 0x6911d986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5dd98a, 3 },
    { 0, 0 },
    { 0xfdb1d98c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3dd98f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1581d995, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71dd99c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1d9a0, 3 },
    { 0x541d9a0, 3 },
    { 0, 0 },
    { 0xb7e5d9a3, 3 },
    { 0, 0 },
    { 0x128dd9a5, 3 },
    { 0xed1d9a5, 3 },
    { 0, 0 },
    { 0xc41d9a8, 3 },
    { 0x347dd9a8, 4 },
    { 0xa0b9d9aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9165d9b3, 3 },
    { 0, 0 },
    { 0x72bdd9b5, 3 },
    { 0xa571d9b6, 3 },
    { 0, 0 },
    { 0x60cdd9b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1655d9bc, 3 },
    { 0x41b9d9bc, 3 },
    { 0, 0 },
    { 0x7cd5d9bf, 3 },
    { 0x8ba9d9c0, 3 },
    { 0xafc5d9c1, 5 },
    { 0, 0 },
    { 0x9ac5d9c3, 3 },
    { 0xa261d9c4, 4 },
    { 0x130dd9c4, 3 },
    { 0xd8e5d9c6, 4 },
    { 0x5e01d9c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa465d9cc, 3 },
    { 0x9379d9cc, 3 },
    { 0, 0 },
    { 0x8de9d9cf, 3 },
    { 0, 0 },
    { 0xe2d1d9d1, 3 },
    { 0xf025d9d2, 3 },
    { 0x913dd9d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bdd9d7, 4 },
    { 0, 0 },
    { 0x2521d9d9, 4 },
    { 0x298dd9da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb815d9df, 3 },
    { 0xf2dd9e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639d9e5, 3 },
    { 0x9445d9e5, 3 },
    { 0, 0 },
    { 0xe631d9e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc9d9eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cdd9ef, 3 },
    { 0, 0 },
    { 0xd3b9d9f1, 4 },
    { 0x5b9d9f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1889d9f5, 3 },
    { 0, 0 },
    { 0x9855d9f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x265d9fa, 3 },
    { 0xdf1dd9fb, 3 },
    { 0xf2d9d9fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdda03, 3 },
    { 0x351da04, 3 },
    { 0, 0 },
    { 0x3fedda06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a39da0b, 3 },
    { 0xbfcdda0c, 3 },
    { 0x759da0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdda14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7281da1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78dda1e, 3 },
    { 0, 0 },
    { 0x3dd1da20, 4 },
    { 0xd131da20, 4 },
    { 0, 0 },
    { 0xaa79da23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f41da26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29da33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ddda36, 3 },
    { 0x1325da37, 4 },
    { 0x1661da38, 3 },
    { 0x22bdda39, 3 },
    { 0x42b9da38, 3 },
    { 0x7ddda39, 3 },
    { 0x8245da37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81da3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bdda43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8559da46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dda4a, 3 },
    { 0xfb55da4b, 3 },
    { 0xb855da4b, 4 },
    { 0xd675da4d, 3 },
    { 0xc2a5da4e, 3 },
    { 0xcf0dda4f, 4 },
    { 0x531dda50, 3 },
    { 0xba89da4b, 3 },
    { 0xec91da4f, 3 },
    { 0x31c9da53, 3 },
    { 0xf115da54, 3 },
    { 0x7551da50, 3 },
    { 0x97b1da50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a9da59, 3 },
    { 0xa9f1da5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c5da5e, 3 },
    { 0x6921da5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8495da62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b19da66, 3 },
    { 0, 0 },
    { 0x7235da68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e1da70, 3 },
    { 0, 0 },
    { 0x5c11da72, 4 },
    { 0xbfadda73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac25da76, 3 },
    { 0x4b05da76, 3 },
    { 0x92cdda78, 3 },
    { 0, 0 },
    { 0xe79da7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15edda7d, 3 },
    { 0xaec1da7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca5da81, 3 },
    { 0x96d5da81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5125da85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bdda8a, 3 },
    { 0xf68dda8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7dda91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd1da99, 3 },
    { 0xffc9da9a, 4 },
    { 0xbfc9da9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dedda9e, 3 },
    { 0x7501da9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c5daa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491daad, 4 },
    { 0x5da5daae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567ddab2, 3 },
    { 0x30bddab3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c1dab7, 3 },
    { 0x5c61dab8, 3 },
    { 0x7951dab9, 3 },
    { 0, 0 },
    { 0x7501dabb, 3 },
    { 0xc4fddabc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9dac2, 3 },
    { 0xdb9ddac2, 3 },
    { 0x7cd5dac4, 4 },
    { 0, 0 },
    { 0x88ddac6, 3 },
    { 0x9955dac7, 3 },
    { 0x1f19dac8, 3 },
    { 0x5081dac8, 3 },
    { 0x5345dac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac5dace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015dad1, 3 },
    { 0x772ddad2, 3 },
    { 0, 0 },
    { 0xccc1dad4, 3 },
    { 0, 0 },
    { 0xb09dad6, 3 },
    { 0x1f49dad7, 4 },
    { 0xf435dad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c55dadb, 3 },
    { 0, 0 },
    { 0x8c71dadd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2129dae1, 3 },
    { 0, 0 },
    { 0x9579dae3, 3 },
    { 0xd389dae4, 3 },
    { 0, 0 },
    { 0x3891dae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ddae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1daed, 3 },
    { 0xb25daee, 3 },
    { 0x5aeddaef, 5 },
    { 0x69c9daef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf989daf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d9dafc, 3 },
    { 0, 0 },
    { 0x8831dafe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x892ddb01, 3 },
    { 0xb6e9db01, 3 },
    { 0xec9db02, 5 },
    { 0x4ed5db03, 3 },
    { 0xc11ddb02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc575db0d, 4 },
    { 0, 0 },
    { 0xd501db0f, 3 },
    { 0, 0 },
    { 0x1679db11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994ddb15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19db19, 3 },
    { 0xad35db19, 4 },
    { 0xa82ddb1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f59db20, 3 },
    { 0, 0 },
    { 0x31d9db22, 3 },
    { 0x1065db23, 4 },
    { 0xaa59db22, 3 },
    { 0x2b8ddb25, 3 },
    { 0x54f5db25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba9db29, 3 },
    { 0x4d49db2a, 3 },
    { 0x6465db2b, 3 },
    { 0, 0 },
    { 0x2fc1db2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7ddb30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3225db39, 4 },
    { 0x5f41db3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e5db3e, 3 },
    { 0xe4f9db3e, 3 },
    { 0xcab1db3e, 3 },
    { 0xc4e1db41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c01db44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a55db47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b1db4e, 3 },
    { 0xbbc9db4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b5db52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ddb5e, 3 },
    { 0x1ee1db5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ddb69, 3 },
    { 0, 0 },
    { 0x2c41db6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f91db6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ddb73, 3 },
    { 0xf525db74, 3 },
    { 0x535db75, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db81, 3 },
    { 0xdc41db81, 3 },
    { 0xc5a9db82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21db86, 4 },
    { 0xfdc1db87, 3 },
    { 0x52e9db86, 3 },
    { 0x4c35db86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217ddb8f, 4 },
    { 0xcefddb90, 3 },
    { 0, 0 },
    { 0x22e5db92, 3 },
    { 0xe461db93, 3 },
    { 0x6611db94, 3 },
    { 0x6119db95, 3 },
    { 0x872ddb96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7061db99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5131db9d, 3 },
    { 0xd311db9e, 3 },
    { 0xe825db9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0ddba9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21dbb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb021dbb4, 4 },
    { 0, 0 },
    { 0xafddbb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6625dbb9, 4 },
    { 0x19d9dbb9, 3 },
    { 0x2399dbbb, 4 },
    { 0xa95dbbb, 3 },
    { 0x876ddbbb, 3 },
    { 0x16f5dbbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbddbc1, 3 },
    { 0, 0 },
    { 0xf629dbc3, 3 },
    { 0xfb2ddbc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261dbc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a9dbcd, 3 },
    { 0xc369dbce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9dbd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b79dbd9, 3 },
    { 0xb91dbda, 3 },
    { 0, 0 },
    { 0xdcc5dbdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755dbdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa5dbe3, 3 },
    { 0xa7c5dbe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3959dbe8, 3 },
    { 0, 0 },
    { 0x4d5dbea, 3 },
    { 0x3af1dbea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c5dbf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b25dc01, 3 },
    { 0xe28ddc02, 3 },
    { 0, 0 },
    { 0x1045dc04, 3 },
    { 0x138ddc05, 3 },
    { 0xf53ddc05, 3 },
    { 0x69b9dc05, 5 },
    { 0, 0 },
    { 0xf04ddc09, 3 },
    { 0x6c39dc0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c9dc0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ddc16, 3 },
    { 0x90c1dc16, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eddc1b, 3 },
    { 0x5d55dc1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec79dc21, 3 },
    { 0, 0 },
    { 0xa981dc23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1dc28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c9dc2f, 3 },
    { 0x1dbddc30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b9dc34, 3 },
    { 0, 0 },
    { 0x2a6ddc36, 4 },
    { 0x1ff5dc37, 3 },
    { 0, 0 },
    { 0x8279dc39, 4 },
    { 0xea01dc3a, 3 },
    { 0x3ee5dc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789dc3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51dc41, 3 },
    { 0, 0 },
    { 0xb5f9dc43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1509dc47, 3 },
    { 0, 0 },
    { 0x795ddc49, 4 },
    { 0x4b89dc49, 5 },
    { 0, 0 },
    { 0x4f35dc4c, 3 },
    { 0x11cddc4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5dc51, 3 },
    { 0xb161dc52, 3 },
    { 0, 0 },
    { 0x8db5dc54, 3 },
    { 0, 0 },
    { 0x26ddc56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd919dc5a, 3 },
    { 0x2371dc5b, 4 },
    { 0, 0 },
    { 0xeac5dc5d, 3 },
    { 0x9239dc5e, 4 },
    { 0x80c5dc5e, 4 },
    { 0x7291dc60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ddc63, 4 },
    { 0x9b81dc63, 3 },
    { 0x305dc65, 4 },
    { 0xf791dc66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf171dc6d, 3 },
    { 0x1659dc6d, 3 },
    { 0, 0 },
    { 0x1301dc70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d05dc73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ddc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4899dc7f, 3 },
    { 0x28a1dc7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd41dc83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9dc88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cddc8d, 4 },
    { 0xec61dc8d, 3 },
    { 0x57d9dc8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe931dc92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e5dc96, 3 },
    { 0, 0 },
    { 0xccc1dc98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf059dc9d, 4 },
    { 0x6dd1dc9e, 3 },
    { 0xdf99dc9f, 3 },
    { 0x12a1dca0, 3 },
    { 0xac09dca1, 3 },
    { 0xdbd5dca2, 3 },
    { 0x9b59dca3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8681dca9, 3 },
    { 0, 0 },
    { 0xa3addcab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab9dcaf, 3 },
    { 0, 0 },
    { 0xe851dcb1, 3 },
    { 0x7101dcb1, 5 },
    { 0x5071dcb2, 3 },
    { 0x503ddcb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x131dcb7, 3 },
    { 0x9acddcb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bddcbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e49dcc2, 3 },
    { 0xb195dcc3, 4 },
    { 0x9469dcc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eddcc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eddcd0, 4 },
    { 0, 0 },
    { 0x5a41dcd2, 3 },
    { 0x4a95dcd2, 3 },
    { 0xa011dcd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f1dcd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bddcdb, 4 },
    { 0, 0 },
    { 0x3e8ddcdd, 3 },
    { 0x201ddcde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4345dce1, 3 },
    { 0x1051dce2, 3 },
    { 0, 0 },
    { 0x5501dce4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebddcea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b1dcf0, 4 },
    { 0, 0 },
    { 0x3aeddcf2, 3 },
    { 0x36f5dcf2, 4 },
    { 0x49addcf3, 4 },
    { 0xb1e1dcf5, 4 },
    { 0x77a1dcf6, 3 },
    { 0xbf85dcf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444ddcfb, 3 },
    { 0xe93ddcfc, 3 },
    { 0x18b9dcfd, 3 },
    { 0x2881dcfe, 3 },
    { 0x9e9ddcfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfddd03, 3 },
    { 0, 0 },
    { 0xdc49dd05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b39dd0f, 4 },
    { 0x4eb9dd10, 4 },
    { 0, 0 },
    { 0xc009dd12, 3 },
    { 0, 0 },
    { 0x8405dd14, 3 },
    { 0x95e1dd14, 4 },
    { 0xca8ddd16, 3 },
    { 0x7f3ddd16, 4 },
    { 0x8581dd18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5ddd1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd231dd1f, 5 },
    { 0xdbdddd20, 3 },
    { 0xd39dd21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5dd26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe145dd2c, 3 },
    { 0xd919dd2c, 3 },
    { 0xfbb1dd2c, 4 },
    { 0, 0 },
    { 0xd6a1dd30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309dd33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e99dd39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dddd3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf779dd44, 3 },
    { 0x768ddd45, 3 },
    { 0xc7a5dd44, 3 },
    { 0xaed1dd47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9dd4c, 3 },
    { 0, 0 },
    { 0x1715dd4e, 3 },
    { 0, 0 },
    { 0xef09dd50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1481dd57, 3 },
    { 0x10addd58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5885dd5b, 4 },
    { 0x659dd5b, 3 },
    { 0x5515dd5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8055dd64, 3 },
    { 0x7eb1dd65, 3 },
    { 0x455ddd66, 3 },
    { 0, 0 },
    { 0xfe5ddd68, 3 },
    { 0x3569dd69, 3 },
    { 0, 0 },
    { 0x2e6ddd6b, 4 },
    { 0x3935dd6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e5dd6f, 3 },
    { 0, 0 },
    { 0x93f1dd71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8345dd75, 3 },
    { 0, 0 },
    { 0xb281dd77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5dd7b, 3 },
    { 0x28f5dd7c, 3 },
    { 0x6425dd7d, 4 },
    { 0, 0 },
    { 0x66eddd7f, 3 },
    { 0xef41dd80, 4 },
    { 0x4519dd81, 3 },
    { 0x2b29dd82, 3 },
    { 0, 0 },
    { 0x6fb5dd84, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baddd89, 3 },
    { 0, 0 },
    { 0x3261dd8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb31dd90, 3 },
    { 0x3f65dd91, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431dd95, 3 },
    { 0, 0 },
    { 0xeec1dd97, 3 },
    { 0, 0 },
    { 0x96b1dd99, 3 },
    { 0x2f25dd9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4759dd9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5191dda3, 3 },
    { 0x208ddda4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdddb2, 3 },
    { 0x8485ddb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9501ddb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65ddba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581ddc9, 3 },
    { 0x9cc9ddca, 4 },
    { 0x766dddc9, 3 },
    { 0x1505ddcc, 3 },
    { 0x951ddcd, 3 },
    { 0x2d01ddce, 3 },
    { 0x97b9ddcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb829ddd2, 4 },
    { 0x9335ddd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1dddf, 3 },
    { 0x85f5dde0, 3 },
    { 0x9445dde1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9159dde7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc5dded, 3 },
    { 0, 0 },
    { 0x4cb5ddef, 4 },
    { 0, 0 },
    { 0x88b9ddf1, 3 },
    { 0x5001ddf2, 3 },
    { 0x91d1ddf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb1ddf6, 3 },
    { 0x5b29ddf7, 3 },
    { 0xa171ddf7, 3 },
    { 0, 0 },
    { 0x9e25ddfa, 3 },
    { 0, 0 },
    { 0x25f5ddfc, 4 },
    { 0, 0 },
    { 0xb7c1ddfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe599de01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65de04, 5 },
    { 0x35d1de05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fdde0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc91de0f, 3 },
    { 0x336dde10, 4 },
    { 0, 0 },
    { 0x39d9de12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa95de15, 3 },
    { 0, 0 },
    { 0xe8b5de17, 4 },
    { 0x1db9de18, 3 },
    { 0x8ce5de19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45de1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829de21, 3 },
    { 0x4099de22, 3 },
    { 0x7b49de22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f5de2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061de3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50dde3e, 4 },
    { 0x20bdde3e, 4 },
    { 0xa5de3f, 3 },
    { 0, 0 },
    { 0x1699de42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f9de45, 4 },
    { 0, 0 },
    { 0x1badde47, 3 },
    { 0, 0 },
    { 0xf679de49, 3 },
    { 0, 0 },
    { 0x911de4b, 3 },
    { 0x4ec9de4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1de54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cadde5b, 3 },
    { 0xdf31de5c, 3 },
    { 0xe869de5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d01de60, 4 },
    { 0, 0 },
    { 0x4fa5de62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc565de6a, 3 },
    { 0x206dde6a, 3 },
    { 0, 0 },
    { 0x6d31de6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccdde70, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40dde77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6885de7a, 5 },
    { 0, 0 },
    { 0x913dde7c, 4 },
    { 0, 0 },
    { 0x1f41de7e, 5 },
    { 0xd6b9de7e, 3 },
    { 0x8b2dde7f, 3 },
    { 0xb065de81, 3 },
    { 0x589dde81, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa875de85, 4 },
    { 0x67f1de86, 3 },
    { 0xad3dde87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb855de8a, 3 },
    { 0x6789de8b, 3 },
    { 0x9381de8c, 4 },
    { 0xf5d5de8d, 4 },
    { 0, 0 },
    { 0xb9f9de8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31de98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badde9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdddea0, 5 },
    { 0, 0 },
    { 0xc395dea2, 4 },
    { 0x1741dea3, 4 },
    { 0x3c39dea4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee75dea9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab39deac, 3 },
    { 0x841deac, 4 },
    { 0x8ba1deae, 3 },
    { 0x9aeddeae, 3 },
    { 0, 0 },
    { 0xa6b1deb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9deb4, 3 },
    { 0xf635deb5, 3 },
    { 0xfc35deb6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3391debf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81ddec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f9decb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d61ded0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d59ded3, 4 },
    { 0x5f59ded3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee5ded9, 3 },
    { 0xc7c9deda, 3 },
    { 0, 0 },
    { 0x57addedc, 3 },
    { 0x3469dedd, 3 },
    { 0, 0 },
    { 0x4a4ddedf, 3 },
    { 0, 0 },
    { 0x9731dee1, 5 },
    { 0x7001dee2, 3 },
    { 0xc4bddee3, 3 },
    { 0xe685dee2, 3 },
    { 0, 0 },
    { 0xb6c5dee6, 3 },
    { 0xeea5dee7, 3 },
    { 0xee5dee8, 3 },
    { 0x1d65dee6, 3 },
    { 0, 0 },
    { 0x1f9ddeeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de1deee, 3 },
    { 0x4071deee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b75def4, 3 },
    { 0, 0 },
    { 0x678ddef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc9def9, 3 },
    { 0x5701defa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff5defd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd5df01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa755df04, 3 },
    { 0xc0eddf04, 3 },
    { 0x6bc5df06, 4 },
    { 0xcc4ddf07, 4 },
    { 0x78b9df08, 3 },
    { 0, 0 },
    { 0x389ddf0a, 4 },
    { 0xb99df0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e1df0e, 3 },
    { 0, 0 },
    { 0x6f3ddf10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fddf14, 3 },
    { 0x3ebddf15, 3 },
    { 0x5b0ddf16, 3 },
    { 0, 0 },
    { 0x16bddf18, 4 },
    { 0x2de9df18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcddf1c, 4 },
    { 0x1d3ddf1c, 4 },
    { 0x71f1df1e, 3 },
    { 0, 0 },
    { 0x3509df20, 3 },
    { 0xa6b9df21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09df24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fddf2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f35df32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ddf36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c5df41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5311df47, 3 },
    { 0xe07ddf48, 3 },
    { 0, 0 },
    { 0x6469df4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa479df4f, 3 },
    { 0x4275df4f, 3 },
    { 0x3655df51, 3 },
    { 0, 0 },
    { 0xf0e9df53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d5df5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc5df63, 3 },
    { 0, 0 },
    { 0x20b1df65, 3 },
    { 0xfd85df65, 3 },
    { 0x94f5df66, 3 },
    { 0x8069df68, 3 },
    { 0, 0 },
    { 0xd839df6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621df6e, 3 },
    { 0xc59df6f, 3 },
    { 0xa5a1df70, 3 },
    { 0x2ca1df70, 4 },
    { 0x99d5df70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225df75, 3 },
    { 0x9baddf76, 3 },
    { 0xd371df77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb169df7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d69df82, 3 },
    { 0, 0 },
    { 0x5f6ddf84, 4 },
    { 0x8899df85, 3 },
    { 0, 0 },
    { 0x17dddf87, 4 },
    { 0x63d5df88, 3 },
    { 0x1071df89, 4 },
    { 0x4685df89, 3 },
    { 0x196ddf88, 3 },
    { 0x2c81df8c, 3 },
    { 0x7a55df8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d9df90, 4 },
    { 0xfa89df91, 3 },
    { 0x972ddf91, 3 },
    { 0xfde5df93, 4 },
    { 0x996ddf94, 4 },
    { 0x5a29df90, 3 },
    { 0, 0 },
    { 0x56cddf97, 4 },
    { 0xdb2ddf98, 4 },
    { 0xd9d1df99, 3 },
    { 0xdf81df9a, 3 },
    { 0x4e15df9a, 4 },
    { 0x231df9a, 3 },
    { 0x4641df9a, 3 },
    { 0x6d11df98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa641dfa6, 3 },
    { 0, 0 },
    { 0xd15dfa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc835dfb2, 3 },
    { 0, 0 },
    { 0x6765dfb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c01dfb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5ddfc4, 3 },
    { 0, 0 },
    { 0xb469dfc6, 3 },
    { 0xd691dfc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25dfcc, 3 },
    { 0, 0 },
    { 0x7f3ddfce, 4 },
    { 0x6cb1dfcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc1dfd4, 4 },
    { 0xec61dfd4, 3 },
    { 0x5b95dfd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8959dfdb, 4 },
    { 0x69dddfdc, 3 },
    { 0xb1ddfdd, 3 },
    { 0x4061dfde, 3 },
    { 0x121ddfdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad69dfe8, 3 },
    { 0xcb1ddfe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099dfec, 3 },
    { 0xf91ddfed, 3 },
    { 0x9529dfee, 3 },
    { 0, 0 },
    { 0xbc65dff0, 3 },
    { 0x81e9dff1, 5 },
    { 0x8d0ddff2, 3 },
    { 0x6cb9dff2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7ddff8, 3 },
    { 0, 0 },
    { 0x8b69dffa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02ddffe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccde001, 3 },
    { 0, 0 },
    { 0xf7c5e003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa71e00a, 3 },
    { 0x3b99e00b, 3 },
    { 0, 0 },
    { 0x1225e00d, 4 },
    { 0xe391e00e, 3 },
    { 0xb715e00f, 3 },
    { 0xc719e00f, 3 },
    { 0xf089e00e, 3 },
    { 0x61dde00e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e75e01a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb1e01d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91e021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef41e024, 3 },
    { 0xa0f5e025, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7141e028, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341e03d, 3 },
    { 0xbffde03e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7951e047, 3 },
    { 0xf68de048, 3 },
    { 0xa099e049, 3 },
    { 0, 0 },
    { 0x883de04b, 3 },
    { 0x6129e04c, 3 },
    { 0x846de04c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf149e051, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d5e058, 3 },
    { 0, 0 },
    { 0x2121e05a, 5 },
    { 0xfa21e05a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2de060, 3 },
    { 0x704de061, 3 },
    { 0xbb41e062, 3 },
    { 0xfb7de063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195e06b, 4 },
    { 0, 0 },
    { 0xf36de06d, 3 },
    { 0, 0 },
    { 0xc7a5e06f, 4 },
    { 0xf849e06f, 3 },
    { 0, 0 },
    { 0x46bde072, 4 },
    { 0, 0 },
    { 0x25e074, 3 },
    { 0x4139e074, 4 },
    { 0xc2d1e074, 3 },
    { 0, 0 },
    { 0x4dfde078, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe821e07b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c39e081, 3 },
    { 0xc979e082, 3 },
    { 0x6235e083, 3 },
    { 0x772de083, 4 },
    { 0x3181e085, 4 },
    { 0x67ade086, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x453de089, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b1e093, 3 },
    { 0x7609e094, 3 },
    { 0, 0 },
    { 0x2a2de096, 3 },
    { 0, 0 },
    { 0xade098, 3 },
    { 0x9e89e099, 3 },
    { 0x23b5e09a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdade09e, 3 },
    { 0xde95e09f, 4 },
    { 0, 0 },
    { 0x749de0a1, 3 },
    { 0xbcb5e0a2, 4 },
    { 0x16b5e0a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1e0a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48de0ab, 5 },
    { 0, 0 },
    { 0x83c5e0ad, 3 },
    { 0x3349e0ad, 4 },
    { 0xac55e0af, 3 },
    { 0x2619e0b0, 3 },
    { 0, 0 },
    { 0x1fe1e0b2, 3 },
    { 0, 0 },
    { 0x55bde0b4, 3 },
    { 0x5161e0b5, 3 },
    { 0xfd09e0b4, 3 },
    { 0x8801e0b4, 3 },
    { 0x16f5e0b7, 3 },
    { 0x1ee5e0b9, 3 },
    { 0xd705e0ba, 3 },
    { 0x3dc1e0bb, 3 },
    { 0x5e51e0b9, 4 },
    { 0xd929e0b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb5e0c5, 3 },
    { 0xd94de0c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fde0c9, 3 },
    { 0xd8d1e0c9, 3 },
    { 0x8af5e0cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b79e0ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3de0d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacde0d8, 4 },
    { 0, 0 },
    { 0x187de0da, 3 },
    { 0x56f9e0da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2de0de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21e0e6, 3 },
    { 0, 0 },
    { 0xb895e0e8, 3 },
    { 0xe825e0e9, 4 },
    { 0x910de0ea, 3 },
    { 0xcfd1e0e9, 4 },
    { 0x80d1e0e9, 3 },
    { 0xb4b1e0ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d9e0f2, 3 },
    { 0x4805e0f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1639e0f6, 3 },
    { 0, 0 },
    { 0x1125e0f8, 3 },
    { 0x4079e0f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9e0fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf5e102, 3 },
    { 0x817de103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe545e106, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d89e10a, 3 },
    { 0x55b9e10b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5e111, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8675e116, 3 },
    { 0x46f5e116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dade11a, 3 },
    { 0, 0 },
    { 0xad45e11c, 3 },
    { 0, 0 },
    { 0xb71de11e, 4 },
    { 0, 0 },
    { 0xc339e120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1e124, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7359e12a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da9e12d, 3 },
    { 0x865e12e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a15e131, 3 },
    { 0, 0 },
    { 0xb4ade133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdde139, 3 },
    { 0, 0 },
    { 0x5a49e13b, 4 },
    { 0xbc5de13b, 3 },
    { 0x33f9e13b, 3 },
    { 0xc4f5e13e, 3 },
    { 0xf035e13f, 3 },
    { 0, 0 },
    { 0xff71e141, 3 },
    { 0xe7f9e142, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c01e145, 4 },
    { 0xa465e146, 3 },
    { 0, 0 },
    { 0xfb6de148, 3 },
    { 0x829de148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc009e14d, 3 },
    { 0x3929e14d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6de159, 4 },
    { 0, 0 },
    { 0x75fde15b, 3 },
    { 0x1d0de15c, 3 },
    { 0xad95e15b, 3 },
    { 0x2b19e15e, 3 },
    { 0x91d1e15f, 3 },
    { 0xc299e15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2231e163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad9e167, 3 },
    { 0, 0 },
    { 0x15d5e169, 3 },
    { 0xf841e16a, 4 },
    { 0, 0 },
    { 0x3f8de16c, 3 },
    { 0, 0 },
    { 0xb115e16e, 4 },
    { 0x6759e16e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e75e175, 4 },
    { 0, 0 },
    { 0x293de177, 3 },
    { 0x40e9e178, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a1e17b, 3 },
    { 0x3381e17c, 3 },
    { 0x250de17d, 3 },
    { 0xfa81e17b, 3 },
    { 0, 0 },
    { 0xf551e180, 4 },
    { 0, 0 },
    { 0x42dde182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25e189, 3 },
    { 0xb4d9e18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dde18f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af9e194, 3 },
    { 0x38dde195, 3 },
    { 0x648de196, 3 },
    { 0, 0 },
    { 0x4aa9e198, 3 },
    { 0x5761e198, 3 },
    { 0x4831e198, 3 },
    { 0xa459e19b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7de1a5, 4 },
    { 0x9011e1a6, 3 },
    { 0x9f75e1a7, 3 },
    { 0, 0 },
    { 0x5cfde1a9, 3 },
    { 0, 0 },
    { 0xee51e1ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca9e1b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe155e1b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453de1bb, 4 },
    { 0x13d9e1bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f5e1c0, 3 },
    { 0xcb35e1c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f1e1c5, 4 },
    { 0xb169e1c5, 3 },
    { 0, 0 },
    { 0x2c9e1c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa429e1cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e71e1ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665de1d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241e1d8, 3 },
    { 0x2795e1d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1de1de, 3 },
    { 0x93a5e1de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414de1e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a5e1ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e1f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5e1fb, 4 },
    { 0xf495e1fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129e205, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d79e208, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd321e20c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169e211, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e65e215, 3 },
    { 0, 0 },
    { 0xe5fde217, 4 },
    { 0x95d5e217, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed15e21c, 3 },
    { 0x5af1e21d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff9e220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde81e225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9e229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce1e232, 3 },
    { 0, 0 },
    { 0x2be1e234, 4 },
    { 0x28bde234, 3 },
    { 0x3641e234, 5 },
    { 0xd985e234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e240, 3 },
    { 0xa3ede241, 3 },
    { 0xdc0de240, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9e245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d41e24b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb9e24f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b75e255, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7495e25c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d1e25f, 3 },
    { 0, 0 },
    { 0x3d51e261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54de264, 4 },
    { 0, 0 },
    { 0xcb75e266, 3 },
    { 0x81e9e267, 3 },
    { 0xb4a9e268, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9e26b, 3 },
    { 0x1f75e26c, 4 },
    { 0x3a79e26c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe295e273, 3 },
    { 0x7be5e273, 4 },
    { 0x6201e274, 3 },
    { 0xc051e276, 3 },
    { 0x5ff5e274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeade280, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9e283, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ede287, 3 },
    { 0x79bde287, 3 },
    { 0x75b1e289, 3 },
    { 0, 0 },
    { 0xd975e28b, 3 },
    { 0x23f9e28c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7585e290, 3 },
    { 0, 0 },
    { 0x28ede292, 3 },
    { 0, 0 },
    { 0xb5c5e294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa511e298, 3 },
    { 0x1b35e299, 3 },
    { 0x4249e29a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed1e2a1, 4 },
    { 0x8be5e2a2, 5 },
    { 0, 0 },
    { 0x435e2a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f1e2a8, 3 },
    { 0, 0 },
    { 0x2c09e2aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17de2ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5e2b2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eade2b5, 3 },
    { 0, 0 },
    { 0x9d7de2b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03de2bb, 3 },
    { 0, 0 },
    { 0xc91e2bd, 3 },
    { 0x1019e2be, 4 },
    { 0x3d69e2bd, 4 },
    { 0x13f5e2bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751e2c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec9e2c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b1e2cb, 3 },
    { 0x9ad9e2cb, 3 },
    { 0xe3e9e2cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea9e2d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dde2d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6045e2dd, 3 },
    { 0xb7f5e2de, 3 },
    { 0, 0 },
    { 0x428de2e0, 3 },
    { 0x76d1e2e0, 4 },
    { 0xaf89e2e2, 3 },
    { 0xccf9e2e3, 3 },
    { 0x68e9e2e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bde2e9, 3 },
    { 0xfe05e2e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c79e2f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cde2f7, 4 },
    { 0x8b81e2f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd1e2fb, 4 },
    { 0, 0 },
    { 0xbe25e2fd, 4 },
    { 0xc75e2fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e301, 3 },
    { 0x539de302, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f91e306, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f5e309, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bde313, 3 },
    { 0x697de314, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf431e31b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf029e329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ade32c, 4 },
    { 0xf9d5e32d, 3 },
    { 0, 0 },
    { 0x6c49e32f, 4 },
    { 0x8411e330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e334, 3 },
    { 0x7f95e335, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9481e33a, 3 },
    { 0xdd55e33a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9de33e, 3 },
    { 0xedc5e33f, 3 },
    { 0x2bf9e33e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd381e343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701e346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ede349, 3 },
    { 0x904de34a, 3 },
    { 0xb5b5e34b, 3 },
    { 0x72dde34c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa75e354, 3 },
    { 0x914de355, 3 },
    { 0x3e31e356, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1605e35f, 3 },
    { 0, 0 },
    { 0xc78de361, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5e365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d9e368, 3 },
    { 0x373de369, 3 },
    { 0x17a9e36a, 3 },
    { 0x93c1e36b, 3 },
    { 0x7509e369, 3 },
    { 0xfc81e36d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2815e370, 3 },
    { 0x6fc5e370, 4 },
    { 0, 0 },
    { 0x3331e373, 3 },
    { 0, 0 },
    { 0x3609e375, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dde378, 3 },
    { 0, 0 },
    { 0x8959e37a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c5e382, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781e385, 4 },
    { 0xc5e1e386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2959e38a, 3 },
    { 0x609de38b, 3 },
    { 0xc5cde38b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23de391, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcde39c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6025e3a3, 4 },
    { 0x9745e3a3, 3 },
    { 0, 0 },
    { 0x29d9e3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c51e3ac, 3 },
    { 0xb709e3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6871e3b4, 4 },
    { 0xd799e3b5, 4 },
    { 0xd295e3b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451de3bf, 3 },
    { 0, 0 },
    { 0x4771e3c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1545e3c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25e3cb, 5 },
    { 0x2735e3cc, 3 },
    { 0x1959e3cd, 3 },
    { 0, 0 },
    { 0x9c29e3cf, 3 },
    { 0x7b8de3cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b45e3d6, 3 },
    { 0, 0 },
    { 0x86dde3d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5e3dc, 3 },
    { 0x7665e3dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745e3e0, 3 },
    { 0xfc1e3e0, 3 },
    { 0xdc19e3e0, 3 },
    { 0xf0fde3e3, 3 },
    { 0x5a65e3e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc379e3e7, 3 },
    { 0xa9ede3e8, 4 },
    { 0x105e3e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4151e3ed, 3 },
    { 0x4adde3ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2791e3f1, 3 },
    { 0xade1e3f2, 4 },
    { 0xdb65e3f3, 3 },
    { 0xdcf1e3f1, 3 },
    { 0x1921e3f5, 3 },
    { 0x98ede3f1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4de3f9, 3 },
    { 0, 0 },
    { 0xd839e3fb, 3 },
    { 0, 0 },
    { 0xf16de3fd, 3 },
    { 0x5351e3fd, 3 },
    { 0xb109e3ff, 3 },
    { 0x30a1e400, 3 },
    { 0x15d9e400, 4 },
    { 0x6d89e402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd45e405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819e40b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3511e410, 3 },
    { 0xf185e410, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639e416, 3 },
    { 0xad8de417, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a9e426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf861e42a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979e42d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a59e431, 3 },
    { 0x1621e432, 3 },
    { 0, 0 },
    { 0x22f1e434, 4 },
    { 0xca9de434, 3 },
    { 0x44fde436, 4 },
    { 0, 0 },
    { 0xbfc9e438, 5 },
    { 0x6ff9e439, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf9e43c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7545e446, 3 },
    { 0xc5b5e446, 4 },
    { 0x371de447, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd1e450, 5 },
    { 0, 0 },
    { 0x9955e452, 4 },
    { 0x47a9e453, 4 },
    { 0xd2a1e454, 3 },
    { 0x1f81e455, 4 },
    { 0x38fde456, 3 },
    { 0x87d1e453, 4 },
    { 0x3c65e458, 4 },
    { 0x991de458, 3 },
    { 0x8d01e453, 4 },
    { 0xcc05e45b, 4 },
    { 0, 0 },
    { 0xf9c1e45d, 3 },
    { 0, 0 },
    { 0xb451e45f, 3 },
    { 0, 0 },
    { 0x30ade461, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5e464, 3 },
    { 0x4fc9e465, 4 },
    { 0x2aade466, 3 },
    { 0x7bf1e467, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8335e46c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f59e471, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859e475, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc351e478, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe369e47e, 3 },
    { 0xac3de47f, 3 },
    { 0, 0 },
    { 0xe445e481, 3 },
    { 0xa269e482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fde487, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf55e490, 3 },
    { 0, 0 },
    { 0xfbdde492, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad49e49b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f45e49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdde4a3, 4 },
    { 0x8985e4a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a85e4ae, 3 },
    { 0xbe9e4ae, 3 },
    { 0xa065e4ae, 3 },
    { 0x6641e4ae, 4 },
    { 0x3ba9e4af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2089e4b8, 4 },
    { 0xdd51e4b9, 3 },
    { 0, 0 },
    { 0xf3e9e4bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1e4be, 3 },
    { 0, 0 },
    { 0xecb5e4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665de4c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e35e4ce, 3 },
    { 0x207de4ce, 4 },
    { 0x4d65e4ce, 4 },
    { 0xc115e4d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x642de4d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b31e4da, 3 },
    { 0, 0 },
    { 0x20f1e4dc, 4 },
    { 0, 0 },
    { 0x49f9e4de, 3 },
    { 0xacc9e4de, 3 },
    { 0xc0fde4de, 3 },
    { 0xcff5e4e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x500de4e4, 3 },
    { 0x6a09e4e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618de4e9, 3 },
    { 0x5e11e4e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89e4ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed15e4f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa549e502, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5829e507, 3 },
    { 0xb451e508, 3 },
    { 0xb9c9e509, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a9e50c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9de50f, 4 },
    { 0x6271e510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf9e516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1e51c, 4 },
    { 0xb8f1e51d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x185de520, 4 },
    { 0, 0 },
    { 0x15b1e522, 4 },
    { 0xbd2de522, 3 },
    { 0, 0 },
    { 0xbae5e525, 3 },
    { 0xf2fde526, 4 },
    { 0, 0 },
    { 0xb30de528, 3 },
    { 0x6311e529, 3 },
    { 0x219e529, 5 },
    { 0, 0 },
    { 0xcbc1e52c, 4 },
    { 0, 0 },
    { 0x3251e52e, 3 },
    { 0, 0 },
    { 0x98fde530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1e534, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379e53a, 3 },
    { 0xf7d1e53b, 3 },
    { 0x8fa1e53c, 3 },
    { 0xe8ede53b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1e541, 5 },
    { 0xa0fde542, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669e546, 3 },
    { 0xaee5e547, 3 },
    { 0x4221e548, 4 },
    { 0, 0 },
    { 0x96c1e54a, 3 },
    { 0x42a5e54a, 4 },
    { 0xb59e54c, 5 },
    { 0x9ae5e54d, 3 },
    { 0xdc6de54d, 3 },
    { 0x8c85e54f, 3 },
    { 0x3b4de54f, 3 },
    { 0x20d9e551, 4 },
    { 0x93fde551, 3 },
    { 0x6dc9e550, 4 },
    { 0, 0 },
    { 0x1d71e555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ede55b, 3 },
    { 0xdc81e55c, 3 },
    { 0xcdfde55d, 3 },
    { 0x7359e55e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d79e563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab1e56b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5e573, 3 },
    { 0xfe1de573, 3 },
    { 0xd7b5e575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea1e57f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823de586, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c1e58a, 3 },
    { 0x7a49e58b, 3 },
    { 0xf97de58c, 3 },
    { 0, 0 },
    { 0xeb19e58e, 3 },
    { 0x8341e58f, 3 },
    { 0xd061e590, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x702de593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf1e597, 3 },
    { 0xc3ade597, 3 },
    { 0x6ba9e599, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f25e59d, 3 },
    { 0, 0 },
    { 0xf35de59f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25e5a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a9e5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5e5be, 3 },
    { 0xd98de5bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781e5c3, 4 },
    { 0x3fa1e5c4, 3 },
    { 0, 0 },
    { 0xe8a9e5c6, 4 },
    { 0, 0 },
    { 0xa1d9e5c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75e5cc, 3 },
    { 0x447de5cd, 3 },
    { 0xf795e5cd, 3 },
    { 0xc26de5cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f71e5d2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc729e5da, 3 },
    { 0, 0 },
    { 0x2149e5dc, 3 },
    { 0x9e11e5dc, 4 },
    { 0x35f5e5dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483de5e3, 3 },
    { 0x8a41e5e4, 3 },
    { 0, 0 },
    { 0x4e71e5e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c1e5e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e5e5ed, 3 },
    { 0x6099e5ee, 3 },
    { 0x9e29e5ee, 3 },
    { 0xdb85e5f0, 3 },
    { 0x625e5ef, 3 },
    { 0xba0de5f2, 3 },
    { 0x352de5f2, 3 },
    { 0x6a4de5f4, 3 },
    { 0xa99de5f3, 4 },
    { 0xfd75e5f3, 3 },
    { 0xba5e5f5, 4 },
    { 0, 0 },
    { 0xe141e5f9, 4 },
    { 0x2a9de5fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6075e5fd, 3 },
    { 0, 0 },
    { 0xcf29e5ff, 3 },
    { 0x502de600, 4 },
    { 0, 0 },
    { 0xd9f9e602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f49e60c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e41e611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4879e619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d41e61d, 4 },
    { 0x56fde61e, 3 },
    { 0, 0 },
    { 0xc9ade620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb9e624, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf1e627, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce9e62b, 4 },
    { 0x7029e62c, 3 },
    { 0xaf1de62d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5e632, 4 },
    { 0x2d51e632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1e636, 3 },
    { 0x9c61e637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc331e63c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c15e645, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a5e64a, 4 },
    { 0xd7e1e64b, 4 },
    { 0x51cde64b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb595e64f, 4 },
    { 0, 0 },
    { 0xb969e651, 3 },
    { 0x1151e652, 3 },
    { 0, 0 },
    { 0x4329e654, 3 },
    { 0, 0 },
    { 0xea41e656, 4 },
    { 0x7e79e657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3855e65a, 3 },
    { 0xa23de65b, 4 },
    { 0xcd21e65c, 3 },
    { 0x3dede65d, 3 },
    { 0, 0 },
    { 0x6bade65f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf589e664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7445e668, 4 },
    { 0, 0 },
    { 0x4441e66a, 3 },
    { 0, 0 },
    { 0xaaade66c, 3 },
    { 0x8ed5e66d, 3 },
    { 0x1a85e66e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a49e674, 3 },
    { 0x4905e675, 4 },
    { 0x1b4de676, 3 },
    { 0xc659e675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2041e680, 4 },
    { 0xbb0de681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519de686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c21e689, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ede68c, 4 },
    { 0x6555e68d, 3 },
    { 0xabe1e68e, 3 },
    { 0x4601e68f, 4 },
    { 0xb33de690, 3 },
    { 0xd889e690, 3 },
    { 0x9489e68d, 3 },
    { 0xbca1e68d, 3 },
    { 0xff65e694, 3 },
    { 0x3b55e695, 3 },
    { 0x8579e694, 3 },
    { 0xd08de696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb585e69a, 3 },
    { 0, 0 },
    { 0xd4e1e69c, 4 },
    { 0xb4f5e69d, 3 },
    { 0x65fde69c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba1e6a5, 3 },
    { 0x29e9e6a5, 4 },
    { 0x8c3de6a6, 3 },
    { 0xc53de6a8, 3 },
    { 0xa391e6a9, 3 },
    { 0x2811e6a5, 3 },
    { 0x55a5e6ab, 3 },
    { 0, 0 },
    { 0x1b25e6ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989e6b0, 3 },
    { 0, 0 },
    { 0x42d5e6b2, 3 },
    { 0x2eede6b3, 3 },
    { 0, 0 },
    { 0x94f1e6b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb071e6ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe845e6bd, 3 },
    { 0, 0 },
    { 0x1f5de6bf, 4 },
    { 0x1ca5e6bf, 3 },
    { 0xf665e6c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ade6c4, 3 },
    { 0xd8f1e6c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed59e6c8, 3 },
    { 0x1c55e6c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffede6d2, 3 },
    { 0x4b21e6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c29e6d7, 3 },
    { 0xcb5e6d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c71e6db, 3 },
    { 0x35b1e6dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6789e6e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf1e6e3, 4 },
    { 0, 0 },
    { 0xe351e6e5, 3 },
    { 0, 0 },
    { 0x2301e6e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ede6f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfde6f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3de6fd, 3 },
    { 0xbb6de6fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa039e707, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f9e70b, 3 },
    { 0xe3c5e70c, 4 },
    { 0xf60de70c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5729e714, 3 },
    { 0, 0 },
    { 0x53e1e716, 3 },
    { 0, 0 },
    { 0xadfde718, 3 },
    { 0, 0 },
    { 0x8561e71a, 5 },
    { 0x469de71b, 5 },
    { 0x5ff5e71a, 3 },
    { 0, 0 },
    { 0x77e9e71e, 4 },
    { 0x5bc1e71e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3291e724, 5 },
    { 0, 0 },
    { 0x728de726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc5e72b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619e733, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0de739, 3 },
    { 0xd895e73a, 4 },
    { 0xd86de73b, 3 },
    { 0xbfb9e73c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a55e743, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2785e746, 3 },
    { 0x5e15e747, 3 },
    { 0x8005e748, 5 },
    { 0xd39e746, 4 },
    { 0xd8d1e74a, 3 },
    { 0x8be9e74b, 4 },
    { 0x8a99e74c, 3 },
    { 0xd299e74c, 3 },
    { 0x574de74c, 3 },
    { 0x4251e74f, 3 },
    { 0x2c59e750, 4 },
    { 0xf699e74d, 3 },
    { 0xc311e74f, 4 },
    { 0x224de753, 3 },
    { 0xc601e754, 4 },
    { 0xcf45e755, 3 },
    { 0xe439e755, 3 },
    { 0xc349e754, 3 },
    { 0, 0 },
    { 0x3531e759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4001e75c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc79e760, 3 },
    { 0x4e55e761, 3 },
    { 0x7599e762, 3 },
    { 0x4dde763, 4 },
    { 0xc9c1e760, 3 },
    { 0xfdfde765, 3 },
    { 0x4795e766, 4 },
    { 0xb8a9e764, 3 },
    { 0x7b11e768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e76e, 3 },
    { 0x58a5e76f, 4 },
    { 0, 0 },
    { 0xc45e771, 3 },
    { 0x6a39e772, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafde778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a9e77b, 3 },
    { 0x2509e77c, 3 },
    { 0x78b1e77d, 3 },
    { 0x5789e77e, 4 },
    { 0xb19e77c, 3 },
    { 0xf3d9e780, 4 },
    { 0xd30de781, 3 },
    { 0x6979e782, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc279e785, 3 },
    { 0xd02de785, 3 },
    { 0x6171e787, 3 },
    { 0, 0 },
    { 0x155e789, 4 },
    { 0, 0 },
    { 0x3421e78b, 3 },
    { 0x333de78b, 3 },
    { 0x731e78d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35e791, 4 },
    { 0xaff9e791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911e795, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021e79c, 3 },
    { 0x6901e79d, 3 },
    { 0, 0 },
    { 0x4ad1e79f, 3 },
    { 0, 0 },
    { 0xaf35e7a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c5e7a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f5e7af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe271e7b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fde7b5, 3 },
    { 0x2479e7b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a9e7bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8171e7c3, 3 },
    { 0xfbc1e7c3, 3 },
    { 0x7335e7c4, 4 },
    { 0xedede7c3, 3 },
    { 0xe78de7c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x655de7ca, 3 },
    { 0xccfde7ca, 3 },
    { 0xe7f9e7cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3359e7cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b5e7d2, 3 },
    { 0, 0 },
    { 0x7bc1e7d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9955e7d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe1e7da, 3 },
    { 0x5081e7db, 3 },
    { 0xc245e7dc, 3 },
    { 0x9bd5e7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6019e7e1, 3 },
    { 0, 0 },
    { 0x4be5e7e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa059e7e9, 4 },
    { 0x8b2de7ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8989e7ed, 3 },
    { 0, 0 },
    { 0x5085e7ef, 3 },
    { 0xde5de7f0, 3 },
    { 0x3e9de7f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf101e7f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fde7fb, 4 },
    { 0xb0ede7fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf9e803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedde809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49e80c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4115e80f, 3 },
    { 0x3669e810, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239e814, 3 },
    { 0xc96de815, 3 },
    { 0x6545e816, 3 },
    { 0xda71e814, 3 },
    { 0xf5e815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dde81b, 3 },
    { 0x95b5e81c, 4 },
    { 0xc0e9e81b, 3 },
    { 0x619e81b, 4 },
    { 0x6091e81f, 3 },
    { 0xeb0de820, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf051e82d, 3 },
    { 0xd175e82d, 4 },
    { 0x8811e82f, 3 },
    { 0x9131e830, 3 },
    { 0x4e69e831, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789e837, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581e847, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf5e850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f5e856, 3 },
    { 0xaa75e856, 4 },
    { 0x8e05e858, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59e85b, 3 },
    { 0, 0 },
    { 0x4845e85d, 3 },
    { 0xe7a1e85d, 3 },
    { 0x2ea9e85f, 3 },
    { 0xa8ede860, 3 },
    { 0, 0 },
    { 0x44fde862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a9e86f, 3 },
    { 0xc201e86f, 3 },
    { 0, 0 },
    { 0x79a5e872, 4 },
    { 0x49d9e873, 4 },
    { 0xb621e874, 4 },
    { 0, 0 },
    { 0xefede876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1249e879, 4 },
    { 0, 0 },
    { 0x2e71e87b, 3 },
    { 0, 0 },
    { 0x6201e87d, 3 },
    { 0x962de87e, 3 },
    { 0, 0 },
    { 0x5005e880, 4 },
    { 0xa019e881, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3749e886, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd659e895, 3 },
    { 0x5dd1e896, 3 },
    { 0xaa8de897, 4 },
    { 0x8d51e896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419e89d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0de8a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4791e8a3, 3 },
    { 0, 0 },
    { 0x5e6de8a5, 3 },
    { 0x65ade8a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d41e8ac, 3 },
    { 0, 0 },
    { 0x3829e8ae, 3 },
    { 0x3b51e8ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x922de8b2, 3 },
    { 0xacd5e8b2, 3 },
    { 0, 0 },
    { 0xa5a9e8b5, 3 },
    { 0, 0 },
    { 0xf4d5e8b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf9e8bd, 3 },
    { 0x38a5e8be, 3 },
    { 0x6715e8bf, 3 },
    { 0x218de8c0, 4 },
    { 0xf98de8be, 3 },
    { 0x59ede8c2, 4 },
    { 0xbbede8c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcde8c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae1e8ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89e8d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf925e8d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f69e8de, 3 },
    { 0, 0 },
    { 0x925e8e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9e8e5, 3 },
    { 0xe76de8e6, 3 },
    { 0x75f1e8e7, 3 },
    { 0xafb5e8e8, 3 },
    { 0, 0 },
    { 0xfb51e8ea, 3 },
    { 0x38a9e8eb, 3 },
    { 0x6cf5e8ec, 3 },
    { 0x3209e8eb, 3 },
    { 0xe2bde8ec, 3 },
    { 0x64dde8eb, 3 },
    { 0x8a65e8ed, 3 },
    { 0, 0 },
    { 0x50de8f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff1e8fd, 3 },
    { 0xc025e8fe, 4 },
    { 0x3afde8ff, 3 },
    { 0x1195e8ff, 3 },
    { 0xd769e901, 3 },
    { 0xd289e8fe, 3 },
    { 0xa119e903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55e906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd505e90a, 4 },
    { 0x52f9e90b, 3 },
    { 0x4f0de90a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2789e911, 3 },
    { 0x98c1e912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x169de915, 3 },
    { 0xf2d9e916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d1e91d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d19e922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de9e927, 3 },
    { 0x527de927, 4 },
    { 0x8b75e929, 3 },
    { 0x6959e92a, 3 },
    { 0x44f1e92b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c1e92e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a39e933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a5e93b, 3 },
    { 0xb1d9e93c, 3 },
    { 0x274de93b, 3 },
    { 0x21de93b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1e945, 3 },
    { 0xfab5e945, 3 },
    { 0x84c9e945, 3 },
    { 0x4989e948, 3 },
    { 0xed11e946, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ede94c, 3 },
    { 0xb415e94d, 4 },
    { 0x5a39e94e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f89e951, 4 },
    { 0x5249e952, 3 },
    { 0x9e31e953, 3 },
    { 0x1511e954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bde957, 4 },
    { 0x4c81e958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16de965, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1e969, 4 },
    { 0, 0 },
    { 0x9b2de96b, 3 },
    { 0, 0 },
    { 0xf641e96d, 3 },
    { 0, 0 },
    { 0x889de96f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba1e979, 4 },
    { 0, 0 },
    { 0xe325e97b, 3 },
    { 0x1de5e97c, 3 },
    { 0, 0 },
    { 0x4c25e97e, 3 },
    { 0xaa0de97e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x256de982, 3 },
    { 0xb629e983, 3 },
    { 0x6f55e984, 4 },
    { 0, 0 },
    { 0xdb61e986, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf909e98d, 4 },
    { 0xe4e9e98e, 3 },
    { 0x1f29e98f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb821e99c, 3 },
    { 0xf271e99d, 3 },
    { 0x8b29e99e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e5e9a1, 3 },
    { 0x5e8de9a1, 4 },
    { 0, 0 },
    { 0x4f49e9a4, 3 },
    { 0, 0 },
    { 0x5a09e9a6, 4 },
    { 0, 0 },
    { 0xedd1e9a8, 3 },
    { 0xa9cde9a8, 3 },
    { 0xee61e9aa, 3 },
    { 0, 0 },
    { 0xd4e5e9ac, 4 },
    { 0x87b9e9ad, 3 },
    { 0x4289e9ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b39e9b2, 3 },
    { 0x9719e9b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x941de9b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499e9b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13de9be, 3 },
    { 0x2571e9bf, 3 },
    { 0, 0 },
    { 0x8779e9c1, 3 },
    { 0, 0 },
    { 0xcd05e9c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41e9c6, 4 },
    { 0xb559e9c6, 3 },
    { 0x91f1e9c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4de9cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad25e9d4, 3 },
    { 0, 0 },
    { 0xdd01e9d6, 3 },
    { 0xeb6de9d7, 3 },
    { 0xb705e9d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6389e9dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4811e9e1, 3 },
    { 0, 0 },
    { 0xd975e9e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3961e9e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9de9ec, 4 },
    { 0xa4e1e9ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e9f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9361e9f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1219e9fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb19ea01, 3 },
    { 0xf271ea02, 3 },
    { 0, 0 },
    { 0x8781ea04, 3 },
    { 0xeef9ea04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885ea09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49ea0c, 3 },
    { 0x82a5ea0d, 3 },
    { 0x8f99ea0e, 3 },
    { 0xeb89ea0f, 3 },
    { 0x3badea10, 3 },
    { 0xb661ea10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5ea14, 4 },
    { 0x735dea14, 3 },
    { 0xd83dea16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e39ea1b, 3 },
    { 0xf64dea1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb5ea21, 3 },
    { 0x53f1ea22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1dea28, 3 },
    { 0x1155ea29, 3 },
    { 0xc179ea29, 4 },
    { 0xb155ea2a, 3 },
    { 0xc0fdea2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f25ea31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d9ea35, 4 },
    { 0, 0 },
    { 0x8cf9ea37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e9ea3a, 4 },
    { 0x29d9ea3b, 3 },
    { 0x965dea3c, 3 },
    { 0, 0 },
    { 0x45b1ea3e, 4 },
    { 0x7459ea3f, 5 },
    { 0x912dea3f, 3 },
    { 0xb27dea41, 3 },
    { 0x75ddea42, 3 },
    { 0x2835ea41, 4 },
    { 0, 0 },
    { 0x1419ea45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c95ea48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a9ea51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15dea60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b75ea63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc5ea67, 3 },
    { 0xeed1ea68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4865ea6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7071ea6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d81ea75, 3 },
    { 0xd105ea76, 3 },
    { 0, 0 },
    { 0xe219ea78, 3 },
    { 0, 0 },
    { 0x5f39ea7a, 3 },
    { 0x7f1dea7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dadea7f, 3 },
    { 0, 0 },
    { 0x431dea81, 3 },
    { 0x72adea82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5ea8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9dea8f, 3 },
    { 0xf7b9ea90, 3 },
    { 0x33e1ea91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5ea98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259ea9b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4711ea9f, 3 },
    { 0xb385eaa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1381eaa5, 3 },
    { 0x71a9eaa6, 3 },
    { 0x4e21eaa7, 3 },
    { 0x4bb9eaa8, 3 },
    { 0x7afdeaa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1eaac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1eaaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94deab6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3adeab9, 3 },
    { 0, 0 },
    { 0x7bbdeabb, 4 },
    { 0, 0 },
    { 0xf1a9eabd, 3 },
    { 0x88e1eabe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3701eac3, 3 },
    { 0x7975eac4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb575eac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebdeace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26edead4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1deadb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305deae5, 3 },
    { 0xfec9eae6, 3 },
    { 0, 0 },
    { 0xb8deae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3181eaee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2161eaf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe281eafa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd1eb00, 4 },
    { 0x1b0deb01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575eb05, 4 },
    { 0, 0 },
    { 0xe5edeb07, 3 },
    { 0x5d5eb07, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545eb0d, 4 },
    { 0x6965eb0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc51eb11, 4 },
    { 0, 0 },
    { 0x1de5eb13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb935eb16, 3 },
    { 0x762deb16, 3 },
    { 0x182deb18, 3 },
    { 0x389deb19, 3 },
    { 0, 0 },
    { 0xd3b5eb1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519eb20, 4 },
    { 0x5dedeb21, 4 },
    { 0, 0 },
    { 0x2bedeb23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6409eb26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdadeb2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745eb31, 4 },
    { 0xcb79eb31, 4 },
    { 0xaf79eb32, 3 },
    { 0xbf9eb31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c9eb37, 3 },
    { 0x1d5deb38, 3 },
    { 0xde51eb38, 3 },
    { 0x5851eb38, 4 },
    { 0xa945eb3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cdeb3e, 3 },
    { 0xff85eb3f, 3 },
    { 0xfc95eb40, 3 },
    { 0, 0 },
    { 0x8541eb42, 3 },
    { 0x4189eb43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb61eb47, 3 },
    { 0, 0 },
    { 0x6701eb49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2721eb4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abdeb50, 3 },
    { 0xd41eb50, 3 },
    { 0xd951eb52, 3 },
    { 0x6785eb53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc1eb56, 3 },
    { 0, 0 },
    { 0x3b9deb58, 4 },
    { 0x111eb58, 3 },
    { 0x38a9eb5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4971eb62, 4 },
    { 0x1039eb63, 4 },
    { 0x9b3deb63, 4 },
    { 0xd5e9eb65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb68, 3 },
    { 0x4d9eb68, 3 },
    { 0x9509eb6a, 4 },
    { 0xd0b1eb6b, 4 },
    { 0x9e49eb6a, 4 },
    { 0xfc59eb6d, 3 },
    { 0x42b1eb6e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39eb74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1deb80, 3 },
    { 0xe2fdeb80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c35eb86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9755eb8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf81eb91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8091eb95, 3 },
    { 0x529deb96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb969eb9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f5eb9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f5eba0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd139eba3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2deba9, 3 },
    { 0, 0 },
    { 0xa7c5ebab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab9ebae, 3 },
    { 0x6a49ebaf, 3 },
    { 0x1c9debae, 4 },
    { 0xe21debb0, 3 },
    { 0, 0 },
    { 0x40f5ebb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1669ebb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f5ebbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656debc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa355ebca, 3 },
    { 0xad9ebcb, 4 },
    { 0, 0 },
    { 0xfae5ebcd, 3 },
    { 0xbe59ebcd, 3 },
    { 0x6665ebcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c31ebd2, 4 },
    { 0xa195ebd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6125ebd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5ebda, 3 },
    { 0, 0 },
    { 0x4da1ebdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35ebea, 3 },
    { 0xde5debeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9ebee, 3 },
    { 0xb23debef, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd9ebf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc75ebf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d55ebfb, 4 },
    { 0x4bedebfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3879ebff, 3 },
    { 0, 0 },
    { 0xb9f9ec01, 3 },
    { 0, 0 },
    { 0x9a35ec03, 3 },
    { 0xdd1dec04, 3 },
    { 0x6a01ec05, 3 },
    { 0, 0 },
    { 0x3009ec07, 4 },
    { 0xe441ec08, 3 },
    { 0xd889ec09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7679ec0c, 3 },
    { 0x6385ec0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d9ec12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d5ec16, 4 },
    { 0, 0 },
    { 0xfc89ec18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6541ec1c, 3 },
    { 0, 0 },
    { 0x16b9ec1e, 4 },
    { 0x52cdec1f, 3 },
    { 0xa9edec1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aedec25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade5ec2c, 5 },
    { 0xf78dec2c, 4 },
    { 0x7ef9ec2e, 4 },
    { 0, 0 },
    { 0x9dd9ec30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015ec35, 3 },
    { 0x7b49ec36, 3 },
    { 0xedb1ec37, 3 },
    { 0xe779ec37, 3 },
    { 0xfb3dec39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fdec42, 4 },
    { 0x8125ec43, 3 },
    { 0x2339ec44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dec48, 4 },
    { 0, 0 },
    { 0x3db1ec4a, 3 },
    { 0, 0 },
    { 0x6371ec4c, 3 },
    { 0, 0 },
    { 0xd9c1ec4e, 4 },
    { 0x4c0dec4e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e11ec55, 3 },
    { 0, 0 },
    { 0xf3e1ec57, 4 },
    { 0, 0 },
    { 0x1355ec59, 3 },
    { 0, 0 },
    { 0xa281ec5b, 3 },
    { 0x1b99ec5b, 3 },
    { 0xa755ec5d, 3 },
    { 0xf2e5ec5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x449ec61, 3 },
    { 0, 0 },
    { 0xc459ec63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x897dec66, 3 },
    { 0xe699ec66, 3 },
    { 0x3ee5ec68, 3 },
    { 0xa565ec68, 3 },
    { 0x2841ec6a, 3 },
    { 0x7da1ec6a, 4 },
    { 0x2571ec6c, 3 },
    { 0x8bcdec6d, 3 },
    { 0xd0e1ec6a, 4 },
    { 0x2da1ec6b, 3 },
    { 0, 0 },
    { 0x9dd9ec71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf219ec76, 4 },
    { 0x6b61ec76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ddec7b, 3 },
    { 0x371dec7c, 3 },
    { 0, 0 },
    { 0x7005ec7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe09ec81, 3 },
    { 0, 0 },
    { 0x1ad5ec83, 4 },
    { 0x7f21ec83, 3 },
    { 0xb895ec84, 3 },
    { 0xcbfdec85, 3 },
    { 0x1df9ec87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b5ec8b, 4 },
    { 0, 0 },
    { 0x4bbdec8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb1ec90, 3 },
    { 0xd26dec91, 3 },
    { 0x4fcdec92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c9ec96, 3 },
    { 0x477dec96, 3 },
    { 0xb85ec98, 3 },
    { 0x3865ec99, 4 },
    { 0xd179ec97, 3 },
    { 0, 0 },
    { 0xb119ec9c, 3 },
    { 0xc5a1ec9d, 3 },
    { 0xf65dec9e, 3 },
    { 0xbe61ec9f, 3 },
    { 0, 0 },
    { 0x6ac5eca1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc335eca6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b1eca9, 4 },
    { 0, 0 },
    { 0xaa3decab, 3 },
    { 0x5c09ecac, 3 },
    { 0xf4f5ecab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a1ecb3, 3 },
    { 0x2f6decb4, 3 },
    { 0x93e1ecb5, 4 },
    { 0x2c31ecb6, 3 },
    { 0xe7b5ecb6, 3 },
    { 0x5e41ecb6, 3 },
    { 0xa015ecb3, 3 },
    { 0x4b2decb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71ecbd, 3 },
    { 0x4839ecbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b89ecc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b9ecc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5099ecca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67deccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32decd1, 3 },
    { 0, 0 },
    { 0xb96decd3, 3 },
    { 0xe56decd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b5ecd9, 3 },
    { 0x669ecda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5ecdd, 3 },
    { 0x4201ecde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80dece4, 3 },
    { 0x3fe9ece5, 3 },
    { 0x684dece6, 4 },
    { 0, 0 },
    { 0x967dece8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269eced, 3 },
    { 0x18b1ecee, 3 },
    { 0xdf79ecef, 3 },
    { 0xcc31ecf0, 3 },
    { 0x1915ecf0, 3 },
    { 0x3e7decef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7111ecf7, 3 },
    { 0, 0 },
    { 0x9841ecf9, 4 },
    { 0xcbcdecf9, 3 },
    { 0xfe99ecfb, 3 },
    { 0xa969ecfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859ed01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842ded05, 3 },
    { 0, 0 },
    { 0x5c65ed07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff9ed0d, 3 },
    { 0xc4f1ed0e, 3 },
    { 0x8311ed0f, 3 },
    { 0x63fded0e, 4 },
    { 0x1ecded11, 5 },
    { 0xd91ded12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7705ed15, 3 },
    { 0xfb75ed15, 3 },
    { 0xc8aded16, 3 },
    { 0, 0 },
    { 0x661ed19, 3 },
    { 0x909ded1a, 3 },
    { 0x6e91ed1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5ed20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633ded28, 5 },
    { 0xffb1ed29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1695ed2c, 3 },
    { 0xd79ed2d, 3 },
    { 0, 0 },
    { 0x2b41ed2f, 3 },
    { 0xe31ded30, 3 },
    { 0, 0 },
    { 0xa7a1ed32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4035ed35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5ed39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebded3c, 3 },
    { 0x77f9ed3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe39ed41, 4 },
    { 0x9a7ded42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb249ed4e, 3 },
    { 0x10bded4e, 3 },
    { 0x2c49ed50, 3 },
    { 0xd171ed51, 3 },
    { 0x83b5ed52, 3 },
    { 0x9541ed53, 4 },
    { 0x2041ed54, 3 },
    { 0x6ef9ed54, 3 },
    { 0xb9e5ed53, 3 },
    { 0, 0 },
    { 0x2b69ed58, 3 },
    { 0x39fded59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11ded64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05ed68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3ded6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1ed6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9579ed74, 4 },
    { 0x9f59ed75, 3 },
    { 0x2075ed76, 3 },
    { 0, 0 },
    { 0xfb39ed78, 3 },
    { 0xa37ded79, 3 },
    { 0x7d29ed7a, 3 },
    { 0x9821ed7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19ded7e, 4 },
    { 0, 0 },
    { 0xb555ed80, 3 },
    { 0xdb51ed81, 3 },
    { 0xb679ed80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b31ed8c, 4 },
    { 0x9b91ed8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c1ed90, 3 },
    { 0, 0 },
    { 0x1205ed92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1549ed97, 4 },
    { 0xf5a9ed98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9741ed9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8841eda0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc531eda3, 3 },
    { 0x8a99eda3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd239eda8, 3 },
    { 0x9afdeda8, 3 },
    { 0x5475edaa, 4 },
    { 0xcd39edab, 3 },
    { 0, 0 },
    { 0xd22dedad, 3 },
    { 0, 0 },
    { 0x4569edaf, 3 },
    { 0xf2d1edb0, 4 },
    { 0x2d75edb0, 4 },
    { 0xf8bdedb2, 3 },
    { 0, 0 },
    { 0x4599edb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4939edba, 3 },
    { 0xe9d1edbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb529edc2, 3 },
    { 0x21a5edc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561edc8, 3 },
    { 0x6439edc9, 3 },
    { 0, 0 },
    { 0x884dedcb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1dedcf, 3 },
    { 0xd9c1edcf, 3 },
    { 0x94adedd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b9eddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7349eddf, 4 },
    { 0x5a1ede0, 4 },
    { 0x5d6dede0, 3 },
    { 0x2d35ede0, 3 },
    { 0x5e49ede3, 3 },
    { 0x8d59ede4, 3 },
    { 0xeef9ede5, 3 },
    { 0xf2ddede2, 3 },
    { 0x7c65ede7, 3 },
    { 0x3bc1ede8, 3 },
    { 0x8b41ede2, 4 },
    { 0x4ee1edea, 3 },
    { 0x19a5edeb, 4 },
    { 0x1951edeb, 4 },
    { 0x7299ede1, 3 },
    { 0x5581edee, 3 },
    { 0xee5dedee, 3 },
    { 0xcd79edef, 3 },
    { 0x4101edf1, 4 },
    { 0xf68dedf2, 4 },
    { 0x5c29edf2, 3 },
    { 0xb1adedf4, 3 },
    { 0x291edf5, 3 },
    { 0x71bdedf0, 3 },
    { 0x1645edf7, 3 },
    { 0xcf5deddf, 3 },
    { 0x6addedf9, 3 },
    { 0xb8d1eded, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5889edfe, 3 },
    { 0, 0 },
    { 0x6961ee00, 4 },
    { 0xce51ee00, 3 },
    { 0, 0 },
    { 0x9649ee03, 4 },
    { 0x18f1ee04, 3 },
    { 0x669ee05, 4 },
    { 0x47f5ee06, 3 },
    { 0xbb51ee07, 3 },
    { 0xaf2dee04, 4 },
    { 0xb059ee04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e75ee10, 3 },
    { 0, 0 },
    { 0x149ee12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d1ee1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c5ee21, 3 },
    { 0x782dee22, 4 },
    { 0xf0d1ee21, 3 },
    { 0xd8d1ee24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821ee27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc5ee2b, 3 },
    { 0x4a35ee2b, 4 },
    { 0xee1dee2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4edee30, 4 },
    { 0xf08dee30, 3 },
    { 0xcf1dee32, 3 },
    { 0x809ee32, 3 },
    { 0xf4b5ee34, 4 },
    { 0xb5d9ee35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9131ee3b, 3 },
    { 0x131ee3c, 4 },
    { 0x2855ee3c, 3 },
    { 0x50b9ee3e, 4 },
    { 0xe0e1ee3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedee42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35ee46, 3 },
    { 0xc2d9ee47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eddee4c, 4 },
    { 0xe779ee4d, 3 },
    { 0xbf6dee4e, 3 },
    { 0xdd19ee4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11ee54, 3 },
    { 0xfdd1ee55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de9ee61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ee65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55ee69, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8931ee6c, 3 },
    { 0xdfe1ee6c, 3 },
    { 0xa795ee6e, 4 },
    { 0x1db1ee6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55ee72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c5ee75, 3 },
    { 0, 0 },
    { 0x234dee77, 3 },
    { 0, 0 },
    { 0xcdf1ee79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f5ee7c, 3 },
    { 0x3fd5ee7c, 3 },
    { 0xdc81ee7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6341ee82, 4 },
    { 0, 0 },
    { 0x16ddee84, 3 },
    { 0xc7adee84, 3 },
    { 0x7f91ee86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984dee8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea29ee92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d1ee97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc989ee9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa481ee9e, 3 },
    { 0, 0 },
    { 0x7bfdeea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb9eea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d5eeae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465deeb5, 3 },
    { 0xe599eeb5, 3 },
    { 0, 0 },
    { 0x705eeb8, 3 },
    { 0, 0 },
    { 0xd6c5eeba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db5eebf, 3 },
    { 0x7b19eebf, 4 },
    { 0x6bf1eec1, 3 },
    { 0, 0 },
    { 0x588deec3, 3 },
    { 0x9cb5eec3, 3 },
    { 0xced5eec5, 3 },
    { 0xbc45eec3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef41eec9, 3 },
    { 0, 0 },
    { 0x31fdeecb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f5eecf, 3 },
    { 0x4d5deecf, 3 },
    { 0x4b45eed1, 4 },
    { 0, 0 },
    { 0x8091eed3, 4 },
    { 0x9e35eed4, 4 },
    { 0x4171eed3, 3 },
    { 0xd535eed6, 3 },
    { 0xf7a5eed7, 3 },
    { 0, 0 },
    { 0x1be1eed9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b9eede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc489eee1, 3 },
    { 0x1879eee2, 3 },
    { 0xb4a9eee3, 3 },
    { 0, 0 },
    { 0x418deee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a5eee8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf979eeec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9adeeef, 4 },
    { 0x19b1eef0, 3 },
    { 0xe715eef1, 5 },
    { 0x9e1eeef, 3 },
    { 0, 0 },
    { 0x5445eef4, 3 },
    { 0x21d1eef4, 5 },
    { 0x8d9deef6, 3 },
    { 0x50d9eef7, 4 },
    { 0xb059eef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe761eeff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad85ef02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf829ef05, 3 },
    { 0x2035ef06, 4 },
    { 0, 0 },
    { 0x5fa9ef08, 3 },
    { 0xd8fdef09, 3 },
    { 0, 0 },
    { 0x71a9ef0b, 5 },
    { 0xffe1ef0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcdef10, 3 },
    { 0x7241ef11, 3 },
    { 0, 0 },
    { 0x4271ef13, 3 },
    { 0, 0 },
    { 0xa0a9ef15, 3 },
    { 0xdbc9ef16, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d9ef19, 3 },
    { 0, 0 },
    { 0xf11def1b, 3 },
    { 0, 0 },
    { 0xe095ef1d, 4 },
    { 0, 0 },
    { 0x7a9ef1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7def25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb1ef29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df1ef2e, 4 },
    { 0x6219ef2f, 3 },
    { 0xbec5ef30, 3 },
    { 0, 0 },
    { 0x43c9ef32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8301ef38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b9ef3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf311ef43, 3 },
    { 0xd68def44, 3 },
    { 0x95c1ef44, 4 },
    { 0x29f1ef43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8def49, 3 },
    { 0x45d5ef4a, 3 },
    { 0x4459ef4a, 3 },
    { 0x5edef4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65ef4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28edef55, 3 },
    { 0xe959ef56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da1ef5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb249ef60, 3 },
    { 0x84e9ef61, 3 },
    { 0x72adef61, 3 },
    { 0, 0 },
    { 0x3405ef64, 3 },
    { 0xa415ef65, 3 },
    { 0, 0 },
    { 0x173def67, 4 },
    { 0xdf89ef68, 4 },
    { 0, 0 },
    { 0x3c3def6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86def72, 3 },
    { 0x7371ef72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605ef79, 3 },
    { 0xf3f1ef7a, 3 },
    { 0x2409ef7b, 3 },
    { 0x962def79, 3 },
    { 0xf071ef7d, 3 },
    { 0x19c1ef7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc5ef82, 4 },
    { 0xdd85ef83, 3 },
    { 0x33edef83, 3 },
    { 0x4235ef83, 4 },
    { 0, 0 },
    { 0xcb55ef87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1ef8d, 4 },
    { 0x7cb5ef8d, 4 },
    { 0x7005ef8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b35ef94, 3 },
    { 0xd949ef95, 3 },
    { 0xb9f9ef96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7371ef9f, 3 },
    { 0xce1efa0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01efa3, 3 },
    { 0x6ac1efa4, 3 },
    { 0xf3e5efa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x191efa8, 3 },
    { 0xeefdefa9, 4 },
    { 0x1f6defa8, 3 },
    { 0, 0 },
    { 0xac45efac, 3 },
    { 0x6e15efad, 3 },
    { 0x304defac, 3 },
    { 0x39e9efae, 3 },
    { 0x3a31efad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23defb3, 3 },
    { 0x91a1efb4, 3 },
    { 0xd235efb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a49efb8, 3 },
    { 0xa1d9efb9, 3 },
    { 0x1ed1efba, 4 },
    { 0xded9efbb, 4 },
    { 0xf4d5efbc, 3 },
    { 0, 0 },
    { 0x2e45efbe, 4 },
    { 0x476defbf, 3 },
    { 0, 0 },
    { 0x3d81efc1, 4 },
    { 0x945defc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1689efc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fdefcb, 4 },
    { 0xb38defcb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26defd1, 4 },
    { 0x2639efd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b89efd6, 5 },
    { 0xdbd5efd6, 4 },
    { 0x2e7defd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4defdd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a1efe2, 3 },
    { 0xce55efe3, 3 },
    { 0xb6a1efe4, 3 },
    { 0xf6b5efe4, 3 },
    { 0, 0 },
    { 0xda2defe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885efea, 3 },
    { 0, 0 },
    { 0x3cc1efec, 3 },
    { 0xb9c5efed, 3 },
    { 0x3005efec, 4 },
    { 0xfeb1efed, 3 },
    { 0, 0 },
    { 0xfdc1eff1, 3 },
    { 0xf9a5eff1, 3 },
    { 0x2491eff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79deff8, 3 },
    { 0xccb9eff9, 3 },
    { 0x543deffa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad1effe, 3 },
    { 0, 0 },
    { 0x57c5f000, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee1f003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9df007, 3 },
    { 0, 0 },
    { 0xfa2df009, 3 },
    { 0, 0 },
    { 0x5095f00b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bdf011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31f023, 3 },
    { 0x9f91f024, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c75f02b, 3 },
    { 0, 0 },
    { 0x2f21f02d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad5f031, 5 },
    { 0x8c09f032, 4 },
    { 0x20ddf033, 5 },
    { 0x8fa1f034, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51df038, 3 },
    { 0x2e31f038, 3 },
    { 0x4595f03a, 5 },
    { 0xdc91f03b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a5f03e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e19f041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6df045, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5f04c, 4 },
    { 0x4d85f04d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x642df050, 3 },
    { 0x1b99f050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad1f05b, 3 },
    { 0x8631f05b, 3 },
    { 0, 0 },
    { 0x3f99f05e, 3 },
    { 0x55fdf05f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cdf067, 3 },
    { 0, 0 },
    { 0x9fb1f069, 4 },
    { 0, 0 },
    { 0x2bd5f06b, 3 },
    { 0, 0 },
    { 0x1b61f06d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8df074, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1295f07c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01df07f, 3 },
    { 0xd161f080, 3 },
    { 0x20c9f080, 3 },
    { 0xd21df081, 3 },
    { 0x4911f081, 3 },
    { 0x638df084, 3 },
    { 0, 0 },
    { 0xb15f086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f1f089, 3 },
    { 0x411f089, 4 },
    { 0, 0 },
    { 0xb975f08c, 3 },
    { 0xdb71f08d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a9f092, 4 },
    { 0xf30df093, 3 },
    { 0xeef1f094, 3 },
    { 0x6c39f093, 3 },
    { 0x75adf094, 3 },
    { 0, 0 },
    { 0xa901f098, 3 },
    { 0x201df099, 3 },
    { 0xf981f099, 3 },
    { 0x8fe5f099, 3 },
    { 0xc321f09c, 3 },
    { 0x137df09c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2235f0a1, 3 },
    { 0x9601f0a2, 3 },
    { 0x7215f0a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a5f0a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a9f0ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3161f0b0, 3 },
    { 0, 0 },
    { 0xa0c5f0b2, 3 },
    { 0, 0 },
    { 0x4d99f0b4, 3 },
    { 0x88e5f0b4, 4 },
    { 0xbcadf0b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea9f0b9, 3 },
    { 0xd29df0b9, 4 },
    { 0, 0 },
    { 0xca89f0bc, 3 },
    { 0xb131f0bd, 3 },
    { 0xd0bdf0be, 3 },
    { 0xadd1f0bd, 4 },
    { 0xa8d9f0bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6edf0c3, 4 },
    { 0x1a41f0c4, 5 },
    { 0xac65f0c5, 3 },
    { 0xdba1f0c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe365f0c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbddf0da, 4 },
    { 0xf365f0da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c9f0e2, 3 },
    { 0x9445f0e2, 3 },
    { 0x405df0e2, 3 },
    { 0xb635f0e3, 3 },
    { 0x8ba9f0e6, 3 },
    { 0xdf29f0e7, 3 },
    { 0xbd19f0e6, 3 },
    { 0xfd11f0e9, 4 },
    { 0x88e1f0e9, 3 },
    { 0x2255f0e6, 3 },
    { 0x2e55f0e9, 5 },
    { 0xe009f0e5, 3 },
    { 0x4319f0e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e9f0f4, 3 },
    { 0xe39f0f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91f0f9, 3 },
    { 0x3d71f0fa, 4 },
    { 0xfe31f0fb, 3 },
    { 0x5dc9f0f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf135f109, 3 },
    { 0xffa5f109, 4 },
    { 0x1f65f10b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a9f10f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a15f113, 3 },
    { 0, 0 },
    { 0x9135f115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375f11f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9f128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6049f12b, 3 },
    { 0, 0 },
    { 0x2e9f12d, 3 },
    { 0x7f19f12d, 4 },
    { 0xe1e1f12f, 4 },
    { 0x1999f12e, 3 },
    { 0x9c2df131, 3 },
    { 0xc841f130, 3 },
    { 0xf2b5f133, 3 },
    { 0x29ddf12d, 3 },
    { 0x31f1f135, 4 },
    { 0x1da9f135, 3 },
    { 0x1f61f137, 3 },
    { 0x7659f138, 3 },
    { 0xc87df136, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e9f13c, 3 },
    { 0x226df13c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94edf140, 4 },
    { 0x10fdf141, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed9f145, 4 },
    { 0xa4d5f146, 3 },
    { 0x7c89f146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf21f14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x715f151, 3 },
    { 0, 0 },
    { 0xc491f153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb89f159, 3 },
    { 0xfa9f15a, 3 },
    { 0, 0 },
    { 0x4d6df15c, 3 },
    { 0, 0 },
    { 0x95e1f15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc5f161, 3 },
    { 0, 0 },
    { 0x4b41f163, 3 },
    { 0, 0 },
    { 0xffb1f165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1099f169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f5f16e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fdf179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5f17c, 3 },
    { 0x3f61f17c, 3 },
    { 0x4bd1f17d, 3 },
    { 0xa229f17e, 3 },
    { 0, 0 },
    { 0x11fdf181, 3 },
    { 0x4105f182, 3 },
    { 0xfd65f183, 4 },
    { 0x3ddf184, 4 },
    { 0xca2df185, 3 },
    { 0xbe01f181, 3 },
    { 0x2535f187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf279f190, 3 },
    { 0x4a9df191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x368df194, 3 },
    { 0xd899f194, 3 },
    { 0x3c65f196, 3 },
    { 0xedf195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f19c, 3 },
    { 0x2111f19d, 3 },
    { 0xc8a9f19e, 3 },
    { 0x7f01f19e, 4 },
    { 0xcfbdf19c, 3 },
    { 0x24bdf1a1, 3 },
    { 0xc035f1a2, 3 },
    { 0x25fdf19c, 3 },
    { 0xe759f1a4, 3 },
    { 0xa64df19f, 3 },
    { 0x159f19e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5265f1a9, 5 },
    { 0x315f1aa, 3 },
    { 0xeb75f1aa, 3 },
    { 0x1165f1ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd105f1af, 3 },
    { 0x899df1b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82df1b6, 4 },
    { 0x7391f1b7, 3 },
    { 0x1319f1b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed55f1c1, 3 },
    { 0x8c85f1c2, 3 },
    { 0x2f75f1c2, 3 },
    { 0xbdf1f1c4, 3 },
    { 0, 0 },
    { 0x27a5f1c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7369f1c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd941f1cc, 3 },
    { 0x6309f1cc, 3 },
    { 0, 0 },
    { 0x619f1cf, 3 },
    { 0xb175f1d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc39f1d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed1f1d8, 3 },
    { 0, 0 },
    { 0x2a75f1da, 3 },
    { 0x68bdf1da, 3 },
    { 0, 0 },
    { 0xa039f1dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a5f1e2, 3 },
    { 0x665df1e3, 4 },
    { 0x86b9f1e2, 3 },
    { 0, 0 },
    { 0x1391f1e6, 4 },
    { 0, 0 },
    { 0x1f0df1e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a9f1ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f1f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x341df1f4, 3 },
    { 0xfaadf1f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b1f1f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac5f203, 4 },
    { 0x8241f204, 3 },
    { 0x229f205, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b01f208, 3 },
    { 0xf395f208, 3 },
    { 0x1b81f20a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafdf20d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f1f216, 4 },
    { 0xaeddf217, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad5f21e, 4 },
    { 0x87d5f21f, 3 },
    { 0xbb01f21f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc755f228, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741f22b, 3 },
    { 0xfcddf22b, 3 },
    { 0x6af1f22b, 3 },
    { 0xe7f9f22e, 4 },
    { 0xdcc9f22f, 3 },
    { 0xde01f22f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3201f233, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea39f237, 4 },
    { 0xcbf5f238, 4 },
    { 0xc34df239, 3 },
    { 0x803df23a, 4 },
    { 0x5f5f23b, 3 },
    { 0x67d9f23c, 3 },
    { 0x731f23d, 4 },
    { 0, 0 },
    { 0xb30df23f, 3 },
    { 0x7fd9f240, 4 },
    { 0, 0 },
    { 0x9285f242, 3 },
    { 0x804df243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce1f246, 3 },
    { 0, 0 },
    { 0xb765f248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb069f24c, 3 },
    { 0x67d5f24d, 3 },
    { 0x76adf24e, 4 },
    { 0x12a9f24f, 3 },
    { 0xe32df24d, 3 },
    { 0x9ac9f251, 4 },
    { 0, 0 },
    { 0x9f19f253, 3 },
    { 0x53d1f253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24df25c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9f261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edf264, 3 },
    { 0xc38df264, 4 },
    { 0xb615f266, 5 },
    { 0x81c1f267, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1df26b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911f26e, 3 },
    { 0x82f1f26f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3251f272, 4 },
    { 0x25a9f273, 3 },
    { 0x63df274, 3 },
    { 0xa545f272, 4 },
    { 0x3ce9f274, 3 },
    { 0, 0 },
    { 0x142df278, 3 },
    { 0x87a5f279, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d21f284, 3 },
    { 0xdbadf285, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb929f28b, 3 },
    { 0, 0 },
    { 0x702df28d, 3 },
    { 0, 0 },
    { 0xd691f28f, 4 },
    { 0x5d05f290, 3 },
    { 0x2aa9f291, 3 },
    { 0x1f51f292, 3 },
    { 0x809df293, 4 },
    { 0x5851f291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bdf298, 3 },
    { 0, 0 },
    { 0x8365f29a, 3 },
    { 0xf51f29b, 4 },
    { 0, 0 },
    { 0x3699f29d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b69f2a1, 3 },
    { 0, 0 },
    { 0xe551f2a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd21f2a7, 3 },
    { 0xac59f2a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9f2ab, 3 },
    { 0x1f11f2ab, 4 },
    { 0xaf85f2ac, 3 },
    { 0xb7e9f2ab, 4 },
    { 0x3491f2ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df1f2b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df2b7, 4 },
    { 0x2055f2b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bdf2bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf75f2c7, 3 },
    { 0x8351f2c8, 3 },
    { 0xd675f2c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a5f2cf, 4 },
    { 0x55c1f2cf, 3 },
    { 0, 0 },
    { 0x2ce1f2d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952df2d7, 4 },
    { 0x4ee5f2d8, 4 },
    { 0, 0 },
    { 0x7d81f2da, 3 },
    { 0, 0 },
    { 0x4e31f2dc, 3 },
    { 0xff5f2dd, 3 },
    { 0, 0 },
    { 0xdfa9f2df, 3 },
    { 0xa0ddf2e0, 4 },
    { 0x92e5f2df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf401f2ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33df2ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf9f2f2, 4 },
    { 0x74ddf2f3, 4 },
    { 0xa895f2f3, 3 },
    { 0xfaa1f2f4, 3 },
    { 0xa4c9f2f6, 4 },
    { 0xe199f2f6, 3 },
    { 0x3729f2f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa929f2fb, 3 },
    { 0xff55f2fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd831f2ff, 3 },
    { 0, 0 },
    { 0xf165f301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca5f305, 3 },
    { 0x303df305, 4 },
    { 0x4029f305, 3 },
    { 0x2485f308, 3 },
    { 0xdf91f309, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb479f310, 3 },
    { 0, 0 },
    { 0xba31f312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd925f318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca5f31b, 3 },
    { 0, 0 },
    { 0x5609f31d, 4 },
    { 0x878df31e, 4 },
    { 0, 0 },
    { 0x6ab1f320, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f5f32b, 3 },
    { 0xa6e1f32b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53df331, 3 },
    { 0x67a5f331, 3 },
    { 0x6d7df332, 3 },
    { 0x2829f332, 3 },
    { 0x8be9f335, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1065f33a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1f33d, 3 },
    { 0x6e39f33e, 3 },
    { 0, 0 },
    { 0x109f340, 4 },
    { 0, 0 },
    { 0x79c9f342, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c5f346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3df34c, 3 },
    { 0, 0 },
    { 0x2035f34e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc945f351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc095f358, 3 },
    { 0x1631f359, 3 },
    { 0x50a1f359, 4 },
    { 0x65c1f358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145f35e, 3 },
    { 0xd9edf35f, 3 },
    { 0xd815f35f, 3 },
    { 0xa641f361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c5f364, 3 },
    { 0, 0 },
    { 0xdb0df366, 4 },
    { 0x19c9f367, 3 },
    { 0x799f366, 4 },
    { 0x1ef1f366, 3 },
    { 0x9d61f36a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67df36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f1f375, 3 },
    { 0xe515f376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89f37e, 4 },
    { 0, 0 },
    { 0x1b41f380, 4 },
    { 0, 0 },
    { 0x2d75f382, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b1f38a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469f392, 3 },
    { 0x5c5f392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b9f396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3379f39a, 3 },
    { 0x4a2df39b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8df3a1, 4 },
    { 0xe921f3a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa79f3a5, 3 },
    { 0xfdf5f3a6, 3 },
    { 0x8849f3a7, 3 },
    { 0x8199f3a7, 3 },
    { 0x494df3a9, 3 },
    { 0x1c05f3a6, 3 },
    { 0xbcc9f3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269f3b1, 3 },
    { 0, 0 },
    { 0xf021f3b3, 5 },
    { 0, 0 },
    { 0x5a95f3b5, 4 },
    { 0x8881f3b5, 3 },
    { 0xc6e9f3b7, 3 },
    { 0x2271f3b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d9f3bc, 4 },
    { 0xafedf3bd, 3 },
    { 0xb585f3bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde61f3c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6365f3cb, 3 },
    { 0, 0 },
    { 0x8509f3cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891f3d4, 3 },
    { 0xd515f3d5, 3 },
    { 0xc931f3d6, 4 },
    { 0xa8edf3d7, 4 },
    { 0, 0 },
    { 0xb99f3d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8151f3e5, 3 },
    { 0, 0 },
    { 0x8999f3e7, 3 },
    { 0x4b9f3e8, 3 },
    { 0x16d9f3e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1adf3ef, 3 },
    { 0, 0 },
    { 0xdc9df3f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e29f3f7, 4 },
    { 0x6111f3f7, 3 },
    { 0xe81df3f9, 3 },
    { 0, 0 },
    { 0xf9fdf3fb, 4 },
    { 0xecd5f3fc, 5 },
    { 0x6f65f3fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cdf400, 3 },
    { 0, 0 },
    { 0x3055f402, 4 },
    { 0, 0 },
    { 0x3981f404, 3 },
    { 0xfe1df405, 4 },
    { 0x8e41f406, 3 },
    { 0xa195f407, 4 },
    { 0, 0 },
    { 0x709df409, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71f40d, 5 },
    { 0x7da1f40e, 4 },
    { 0xdf1f40f, 3 },
    { 0, 0 },
    { 0x139f411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3669f415, 3 },
    { 0x61a1f416, 3 },
    { 0xcb5df415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d9f41b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba91f420, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0df426, 3 },
    { 0xf82df427, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5145f42a, 3 },
    { 0xd421f42b, 3 },
    { 0xda15f42a, 3 },
    { 0, 0 },
    { 0xa51f42e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bdf432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fdf435, 3 },
    { 0, 0 },
    { 0xabc5f437, 3 },
    { 0, 0 },
    { 0xae85f439, 4 },
    { 0xc9bdf43a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x436df43d, 3 },
    { 0, 0 },
    { 0x93c9f43f, 3 },
    { 0x3381f440, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1df446, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef9f44a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc39f452, 4 },
    { 0x91c1f453, 3 },
    { 0x97a1f454, 4 },
    { 0x87a5f455, 4 },
    { 0xfcddf456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b5f45a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f91f460, 4 },
    { 0xc205f461, 3 },
    { 0xb395f461, 3 },
    { 0x731f460, 4 },
    { 0, 0 },
    { 0xff91f465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdedf468, 4 },
    { 0x8a71f468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa071f46c, 4 },
    { 0, 0 },
    { 0xc51df46e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x886df471, 3 },
    { 0, 0 },
    { 0x1b99f473, 3 },
    { 0x12e5f474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29f477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85edf47d, 3 },
    { 0x59b9f47e, 4 },
    { 0xf19f47f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x75df482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd749f486, 4 },
    { 0xb5adf487, 3 },
    { 0xb21df487, 3 },
    { 0xe55df486, 4 },
    { 0xae1df48a, 3 },
    { 0x4a41f48b, 3 },
    { 0xd17df48b, 3 },
    { 0x7ef1f488, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c9f494, 4 },
    { 0, 0 },
    { 0xb9c5f496, 3 },
    { 0x5795f497, 3 },
    { 0xd191f497, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e85f4a7, 3 },
    { 0xec35f4a8, 3 },
    { 0, 0 },
    { 0x4a41f4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7241f4ae, 3 },
    { 0x20d5f4ae, 3 },
    { 0x19f4b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc409f4b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c1f4b9, 3 },
    { 0xc109f4b9, 4 },
    { 0x5cb5f4bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749f4c0, 4 },
    { 0xe205f4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5f4c6, 3 },
    { 0, 0 },
    { 0x396df4c8, 3 },
    { 0xc2f9f4c9, 3 },
    { 0xa919f4c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fdf4ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652df4d5, 3 },
    { 0x171df4d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x578df4d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99df4e1, 3 },
    { 0x1f21f4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x456df4e5, 3 },
    { 0x1249f4e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d9f4ee, 3 },
    { 0x9c19f4ef, 3 },
    { 0xb569f4ee, 3 },
    { 0, 0 },
    { 0xcad5f4f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cdf4f7, 3 },
    { 0xcb95f4f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf235f4fb, 3 },
    { 0x6df1f4fc, 3 },
    { 0xa9c9f4fb, 3 },
    { 0x55c5f4fc, 3 },
    { 0xa8b5f4fe, 4 },
    { 0xeb7df4ff, 4 },
    { 0xbf69f501, 3 },
    { 0x38c9f502, 5 },
    { 0xa35df503, 4 },
    { 0x2231f504, 4 },
    { 0x4031f504, 4 },
    { 0x93e9f506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedf50b, 3 },
    { 0xa585f50c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2305f510, 3 },
    { 0xd4a5f511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa861f515, 3 },
    { 0x13df516, 3 },
    { 0xd49df516, 4 },
    { 0x7439f515, 3 },
    { 0, 0 },
    { 0x33e1f51a, 3 },
    { 0xc0d1f51b, 4 },
    { 0x23a5f51c, 3 },
    { 0, 0 },
    { 0x27cdf51e, 4 },
    { 0x53b5f51e, 3 },
    { 0x7ee5f520, 3 },
    { 0xde51f521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a45f524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1f529, 3 },
    { 0x60f1f52a, 4 },
    { 0, 0 },
    { 0x605df52c, 3 },
    { 0, 0 },
    { 0x77cdf52e, 4 },
    { 0xeb1f52f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799f533, 3 },
    { 0, 0 },
    { 0x2895f535, 3 },
    { 0xa55df536, 3 },
    { 0xd049f537, 3 },
    { 0x98a9f537, 3 },
    { 0x338df539, 4 },
    { 0x71a1f539, 3 },
    { 0x4b1f53b, 4 },
    { 0xfbd9f53c, 4 },
    { 0xbaf5f53d, 3 },
    { 0x8339f53d, 4 },
    { 0xa8b1f53d, 3 },
    { 0x2a11f535, 3 },
    { 0, 0 },
    { 0x9971f542, 3 },
    { 0x15b1f543, 3 },
    { 0x154df543, 5 },
    { 0xcedf545, 3 },
    { 0x4bc5f546, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d85f54a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651f552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd1f557, 4 },
    { 0xe9f557, 4 },
    { 0, 0 },
    { 0xe4f9f55a, 3 },
    { 0xada5f55b, 3 },
    { 0x28f5f55a, 3 },
    { 0x8bfdf55c, 3 },
    { 0x8e11f55b, 5 },
    { 0, 0 },
    { 0xf2a9f560, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3019f563, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7361f568, 3 },
    { 0xb42df569, 3 },
    { 0, 0 },
    { 0xee91f56b, 3 },
    { 0x79bdf56c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745f56f, 3 },
    { 0xdbddf570, 3 },
    { 0xc141f571, 3 },
    { 0x27ddf56f, 3 },
    { 0, 0 },
    { 0xc481f574, 3 },
    { 0xd011f575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e9f57e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7749f581, 4 },
    { 0, 0 },
    { 0xa4fdf583, 3 },
    { 0, 0 },
    { 0x5e95f585, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47df58b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be9f591, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121f595, 3 },
    { 0x1731f596, 3 },
    { 0x645f597, 3 },
    { 0x54fdf597, 4 },
    { 0, 0 },
    { 0x7a4df59a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda9f59d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a75f5a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb9f5a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae11f5a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2df5ac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8699f5af, 3 },
    { 0x5595f5b0, 3 },
    { 0x3c8df5af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca9f5b4, 3 },
    { 0x4665f5b4, 3 },
    { 0, 0 },
    { 0x8159f5b7, 3 },
    { 0x708df5b8, 3 },
    { 0x68adf5b8, 3 },
    { 0x62cdf5ba, 4 },
    { 0x1b75f5b7, 4 },
    { 0xb981f5b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a5f5bf, 3 },
    { 0x1f55f5c0, 3 },
    { 0x7de5f5c0, 3 },
    { 0x4df5c1, 4 },
    { 0, 0 },
    { 0x3ef9f5c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc959f5cd, 4 },
    { 0, 0 },
    { 0xa50df5cf, 3 },
    { 0xc3b1f5d0, 4 },
    { 0, 0 },
    { 0x5595f5d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c1f5d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8649f5dc, 3 },
    { 0xdfb5f5dc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd9f5e0, 3 },
    { 0x7dadf5e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5921f5e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3605f5eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e9f5ee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9df5f3, 4 },
    { 0x8cedf5f3, 4 },
    { 0x90f1f5f5, 3 },
    { 0x39f5f6, 4 },
    { 0x821df5f6, 3 },
    { 0xa679f5f8, 3 },
    { 0xf4cdf5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781f5fc, 4 },
    { 0x1f25f5fd, 4 },
    { 0xeda1f5fd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5f602, 3 },
    { 0, 0 },
    { 0x901df604, 3 },
    { 0, 0 },
    { 0xec5f606, 4 },
    { 0xf089f607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd9f60b, 3 },
    { 0xc2e1f60c, 4 },
    { 0x359f60d, 3 },
    { 0x312df60e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb1f613, 3 },
    { 0x4ec5f613, 3 },
    { 0, 0 },
    { 0xaf9f616, 3 },
    { 0x2f75f617, 3 },
    { 0xd3f9f617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e81f61c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8df622, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd1f625, 3 },
    { 0xa195f626, 3 },
    { 0x6ebdf627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19f62a, 3 },
    { 0x9469f62a, 3 },
    { 0, 0 },
    { 0xf79df62d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc5f634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c05f63c, 3 },
    { 0x5509f63c, 3 },
    { 0x62f9f63e, 3 },
    { 0x3619f63e, 5 },
    { 0x44e9f640, 4 },
    { 0x1295f640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f1f645, 3 },
    { 0xb4e9f646, 3 },
    { 0x12a5f647, 4 },
    { 0x818df648, 4 },
    { 0xd11f649, 4 },
    { 0, 0 },
    { 0xc5a5f64b, 3 },
    { 0x8169f64b, 4 },
    { 0xd9a1f64c, 3 },
    { 0, 0 },
    { 0xfcddf64f, 3 },
    { 0, 0 },
    { 0xa345f651, 3 },
    { 0, 0 },
    { 0xee79f653, 3 },
    { 0xad2df654, 3 },
    { 0x8cfdf654, 3 },
    { 0xab11f653, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5f65b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf01f664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3991f66e, 3 },
    { 0x8fadf66f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115f676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd211f67b, 3 },
    { 0, 0 },
    { 0xfd31f67d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a19f680, 4 },
    { 0, 0 },
    { 0x2b25f682, 3 },
    { 0, 0 },
    { 0x6455f684, 3 },
    { 0, 0 },
    { 0xf115f686, 4 },
    { 0xb809f687, 3 },
    { 0x7b45f686, 3 },
    { 0, 0 },
    { 0xa045f68a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd9f690, 3 },
    { 0, 0 },
    { 0x9ee9f692, 3 },
    { 0xcea1f693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1389f696, 3 },
    { 0, 0 },
    { 0x1481f698, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36df69d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1f6a1, 3 },
    { 0x40c5f6a2, 4 },
    { 0x9755f6a3, 3 },
    { 0xc971f6a3, 4 },
    { 0x92df6a4, 4 },
    { 0xffe9f6a6, 3 },
    { 0xa659f6a1, 3 },
    { 0x3c31f6a8, 3 },
    { 0x30a1f6a9, 3 },
    { 0, 0 },
    { 0x315f6ab, 3 },
    { 0x6e6df6ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e5f6af, 3 },
    { 0, 0 },
    { 0x543df6b1, 4 },
    { 0, 0 },
    { 0x61f6b3, 3 },
    { 0xffe1f6b4, 4 },
    { 0x8de9f6b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a75f6b8, 4 },
    { 0, 0 },
    { 0x7295f6ba, 4 },
    { 0xff39f6ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc985f6c9, 3 },
    { 0x9f6ca, 4 },
    { 0, 0 },
    { 0x9c99f6cc, 3 },
    { 0xb5e9f6cd, 4 },
    { 0x9085f6ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8119f6d1, 3 },
    { 0xeb29f6d2, 4 },
    { 0x718df6d1, 3 },
    { 0x44c5f6d3, 3 },
    { 0x8d21f6d5, 3 },
    { 0xae75f6d5, 3 },
    { 0xa03df6d2, 3 },
    { 0x7959f6d6, 3 },
    { 0x1d05f6d3, 4 },
    { 0x6d19f6da, 3 },
    { 0x9a81f6db, 4 },
    { 0xe709f6dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab85f6e7, 3 },
    { 0, 0 },
    { 0xb775f6e9, 3 },
    { 0xf3f5f6e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf559f6f2, 4 },
    { 0, 0 },
    { 0xca79f6f4, 3 },
    { 0, 0 },
    { 0xec7df6f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ddf6f9, 3 },
    { 0x7e5df6fa, 3 },
    { 0x7fd9f6fb, 4 },
    { 0x9e69f6fb, 4 },
    { 0x125f6f9, 3 },
    { 0, 0 },
    { 0x91b1f6ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d5f706, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae9f70a, 3 },
    { 0, 0 },
    { 0x855df70c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12df70f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1f714, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a19f71a, 4 },
    { 0xc1bdf71b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa9f720, 3 },
    { 0x16b5f721, 4 },
    { 0xc08df722, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3df725, 4 },
    { 0, 0 },
    { 0x46c1f727, 3 },
    { 0, 0 },
    { 0xf631f729, 3 },
    { 0x49c5f72a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9445f72d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdf730, 3 },
    { 0x4e99f730, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa1f734, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9365f739, 3 },
    { 0x4005f73a, 3 },
    { 0xf6df739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b79f73e, 4 },
    { 0, 0 },
    { 0x17bdf740, 3 },
    { 0x5ad9f741, 4 },
    { 0x2cb1f742, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8df747, 3 },
    { 0x3259f748, 4 },
    { 0x3f15f749, 3 },
    { 0xa5df74a, 4 },
    { 0x6da9f74a, 3 },
    { 0, 0 },
    { 0xd765f74d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc699f753, 3 },
    { 0, 0 },
    { 0xc091f755, 3 },
    { 0x9165f756, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c69f75f, 3 },
    { 0, 0 },
    { 0xac89f761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6df764, 4 },
    { 0x651f765, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605f768, 3 },
    { 0, 0 },
    { 0x5115f76a, 3 },
    { 0x6719f76b, 3 },
    { 0x41d5f76c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819f770, 3 },
    { 0, 0 },
    { 0x76edf772, 4 },
    { 0, 0 },
    { 0x1e85f774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc5f778, 4 },
    { 0, 0 },
    { 0xf949f77a, 4 },
    { 0, 0 },
    { 0xba81f77c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f49f780, 3 },
    { 0, 0 },
    { 0xa3df782, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a1f785, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5df789, 3 },
    { 0, 0 },
    { 0x6271f78b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef75f791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df794, 3 },
    { 0, 0 },
    { 0x7bf1f796, 3 },
    { 0, 0 },
    { 0xa705f798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1f79b, 3 },
    { 0x668df79c, 3 },
    { 0xb431f79b, 3 },
    { 0xde4df79e, 3 },
    { 0x4c5df79f, 3 },
    { 0x7925f79c, 3 },
    { 0, 0 },
    { 0x17f9f7a2, 3 },
    { 0xe315f7a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33df7a9, 3 },
    { 0, 0 },
    { 0xd8d9f7ab, 3 },
    { 0x17c5f7ac, 4 },
    { 0, 0 },
    { 0xf3d5f7ae, 3 },
    { 0xa6ddf7ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea19f7b5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6469f7b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9f7bd, 5 },
    { 0xc58df7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd69f7c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb789f7c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccdf7cc, 3 },
    { 0, 0 },
    { 0x9465f7ce, 3 },
    { 0x1519f7cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fdf7d2, 4 },
    { 0, 0 },
    { 0x56c9f7d4, 3 },
    { 0xf2e5f7d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9df7d8, 4 },
    { 0x6b9f7d8, 3 },
    { 0xb1e5f7da, 3 },
    { 0, 0 },
    { 0xf6e9f7dc, 4 },
    { 0x9f35f7dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd661f7e1, 4 },
    { 0xb1edf7e2, 3 },
    { 0xe0f1f7e3, 3 },
    { 0x3a99f7e1, 3 },
    { 0x2df1f7e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99f7eb, 3 },
    { 0, 0 },
    { 0x5a51f7ed, 3 },
    { 0, 0 },
    { 0xbc9df7ef, 3 },
    { 0, 0 },
    { 0xdeb1f7f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d5f7fa, 3 },
    { 0, 0 },
    { 0xb289f7fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed1f803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cdf807, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775f80b, 4 },
    { 0xd195f80c, 3 },
    { 0, 0 },
    { 0x48e5f80e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f811, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25df816, 4 },
    { 0xa169f817, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829f81c, 4 },
    { 0xa1f5f81c, 3 },
    { 0, 0 },
    { 0xede9f81f, 4 },
    { 0xe3edf820, 4 },
    { 0x2ce9f821, 3 },
    { 0x8a01f820, 3 },
    { 0xff7df821, 3 },
    { 0x1785f824, 3 },
    { 0xea19f821, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67adf82c, 3 },
    { 0x9709f82d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb355f833, 3 },
    { 0xda69f834, 4 },
    { 0x1819f834, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1adf83b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf5f83e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b5f843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac01f846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5f854, 3 },
    { 0x5cedf855, 3 },
    { 0xa1f5f856, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba1f85a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475f85d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f5f860, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131df864, 3 },
    { 0x2f21f865, 3 },
    { 0xa4e5f866, 4 },
    { 0, 0 },
    { 0xd42df868, 4 },
    { 0x11cdf869, 3 },
    { 0x33ddf86a, 3 },
    { 0xbbbdf86b, 3 },
    { 0xc92df86c, 4 },
    { 0x4e01f86b, 3 },
    { 0xd4fdf86a, 3 },
    { 0xbb45f86f, 3 },
    { 0xb5ddf870, 3 },
    { 0x1cedf86b, 3 },
    { 0x37c1f872, 4 },
    { 0xfd8df872, 4 },
    { 0xcbcdf872, 3 },
    { 0xd99df873, 4 },
    { 0x7085f876, 3 },
    { 0x826df877, 3 },
    { 0x395f878, 4 },
    { 0x905f879, 4 },
    { 0x3a15f879, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1f87d, 3 },
    { 0x8551f87e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf883, 4 },
    { 0, 0 },
    { 0xff79f885, 3 },
    { 0x5369f886, 4 },
    { 0xa831f886, 3 },
    { 0, 0 },
    { 0x4241f889, 3 },
    { 0x1031f889, 3 },
    { 0x3041f88a, 3 },
    { 0x4715f889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9f890, 3 },
    { 0x981f890, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6629f894, 3 },
    { 0x83cdf894, 3 },
    { 0x8971f895, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f899, 4 },
    { 0x4d65f899, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a65f89d, 3 },
    { 0xd2b9f89e, 4 },
    { 0x339f89f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75f8a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3401f8af, 3 },
    { 0, 0 },
    { 0x6361f8b1, 3 },
    { 0x7cdf8b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35f8b9, 5 },
    { 0xd951f8ba, 3 },
    { 0x7279f8bb, 3 },
    { 0xf6adf8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d9f8bf, 4 },
    { 0xed75f8c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde71f8c3, 3 },
    { 0, 0 },
    { 0xeba5f8c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae1f8ca, 3 },
    { 0x4d31f8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea05f8d0, 3 },
    { 0x9035f8d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5735f8d5, 3 },
    { 0xb05f8d5, 3 },
    { 0, 0 },
    { 0x32d5f8d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd15f8e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4155f8ea, 4 },
    { 0x1b0df8eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe661f8f0, 3 },
    { 0, 0 },
    { 0x8491f8f2, 4 },
    { 0x6139f8f3, 3 },
    { 0xc7a5f8f4, 3 },
    { 0xf2f5f8f5, 3 },
    { 0xd0cdf8f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9f8f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafdf8fc, 4 },
    { 0xa8d9f8fd, 3 },
    { 0x352df8fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51f907, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb1f90b, 3 },
    { 0x4b75f90c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb655f90f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e75f917, 3 },
    { 0x86df918, 3 },
    { 0, 0 },
    { 0x943df91a, 3 },
    { 0x4099f91b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535f924, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4611f927, 4 },
    { 0x5d9df928, 4 },
    { 0xe4a9f929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc785f92d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cdf932, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcdf935, 4 },
    { 0, 0 },
    { 0xabdf937, 3 },
    { 0xa489f938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acdf93c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf365f93f, 3 },
    { 0x74ddf93f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94df944, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf39f948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e61f94b, 3 },
    { 0, 0 },
    { 0xd7cdf94d, 3 },
    { 0, 0 },
    { 0xea19f94f, 3 },
    { 0, 0 },
    { 0xe5c1f951, 3 },
    { 0, 0 },
    { 0x6039f953, 3 },
    { 0xedd9f954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8df958, 3 },
    { 0, 0 },
    { 0xc521f95a, 4 },
    { 0x1361f95a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ddf95e, 3 },
    { 0, 0 },
    { 0x1d95f960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7049f96f, 3 },
    { 0, 0 },
    { 0x6431f971, 3 },
    { 0xd251f972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a5f977, 3 },
    { 0, 0 },
    { 0xe6cdf979, 3 },
    { 0xf445f97a, 4 },
    { 0, 0 },
    { 0xa915f97c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd1f981, 3 },
    { 0, 0 },
    { 0xec61f983, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf419f987, 3 },
    { 0xe721f988, 4 },
    { 0, 0 },
    { 0xc539f98a, 3 },
    { 0xe269f98b, 3 },
    { 0x9d25f98b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7df98f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a05f993, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf815f998, 3 },
    { 0xbbb1f999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c41f9aa, 4 },
    { 0xe51df9ab, 3 },
    { 0, 0 },
    { 0x1a59f9ad, 3 },
    { 0x146df9ae, 3 },
    { 0x5729f9af, 4 },
    { 0, 0 },
    { 0xe521f9b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33df9be, 4 },
    { 0, 0 },
    { 0x7165f9c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf209f9c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7df9c9, 3 },
    { 0xe291f9ca, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608df9d2, 3 },
    { 0xfa7df9d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e1f9d8, 3 },
    { 0, 0 },
    { 0x3a51f9da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d05f9e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9469f9e6, 3 },
    { 0xac1df9e7, 3 },
    { 0x1dd1f9e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f9f9eb, 3 },
    { 0xcef5f9ec, 3 },
    { 0xbcb1f9ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb49f9f1, 3 },
    { 0xcfdf9f2, 3 },
    { 0xab85f9f3, 4 },
    { 0x29df9f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4999f9f8, 3 },
    { 0, 0 },
    { 0x515df9fa, 3 },
    { 0, 0 },
    { 0xeefdf9fc, 3 },
    { 0x9c35f9fc, 3 },
    { 0xe3d5f9fc, 3 },
    { 0x9ec5f9fd, 3 },
    { 0x7439f9ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1fa08, 3 },
    { 0x718dfa09, 3 },
    { 0x2e49fa0a, 3 },
    { 0x8139fa0a, 4 },
    { 0x3185fa0c, 4 },
    { 0x24bdfa08, 3 },
    { 0xf88dfa09, 3 },
    { 0x4be1fa0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69adfa12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae1fa18, 4 },
    { 0, 0 },
    { 0x1d85fa1a, 3 },
    { 0, 0 },
    { 0x65d9fa1c, 4 },
    { 0x97dfa1d, 4 },
    { 0, 0 },
    { 0xad61fa1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7dfa26, 3 },
    { 0x7d65fa27, 3 },
    { 0x199dfa28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdfa2c, 3 },
    { 0x62b1fa2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c9fa31, 3 },
    { 0xb995fa32, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5dfa35, 4 },
    { 0x5519fa36, 3 },
    { 0x1829fa36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf215fa3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e9fa3f, 4 },
    { 0x59c9fa40, 4 },
    { 0xafe5fa41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9791fa48, 3 },
    { 0xa071fa49, 3 },
    { 0xfe95fa4a, 4 },
    { 0xc2a5fa49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d5fa52, 3 },
    { 0x7dc9fa52, 3 },
    { 0x8e59fa52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641fa59, 3 },
    { 0xe219fa5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a1fa5f, 4 },
    { 0, 0 },
    { 0xb5c5fa61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee9fa65, 3 },
    { 0xd7d5fa66, 3 },
    { 0x31d9fa67, 3 },
    { 0, 0 },
    { 0x939fa69, 5 },
    { 0, 0 },
    { 0x775dfa6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9fa71, 3 },
    { 0, 0 },
    { 0xd7e1fa73, 3 },
    { 0x6635fa73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144dfa79, 3 },
    { 0, 0 },
    { 0x94c1fa7b, 3 },
    { 0x860dfa7b, 3 },
    { 0xed59fa7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140dfa82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee91fa88, 3 },
    { 0x16c9fa89, 3 },
    { 0x74d1fa8a, 3 },
    { 0x84c1fa8b, 3 },
    { 0x2a1dfa8c, 4 },
    { 0x8175fa8d, 3 },
    { 0, 0 },
    { 0x37b1fa8f, 3 },
    { 0xb7a9fa90, 3 },
    { 0x8499fa91, 3 },
    { 0x9ac5fa91, 3 },
    { 0x7cadfa92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fdfa98, 4 },
    { 0x5be1fa99, 3 },
    { 0, 0 },
    { 0x6725fa9b, 3 },
    { 0x221dfa9c, 4 },
    { 0xa6b5fa9b, 3 },
    { 0, 0 },
    { 0xc01dfa9f, 3 },
    { 0xa761faa0, 3 },
    { 0x4ebdfa9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc195faa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec9faac, 3 },
    { 0, 0 },
    { 0xf159faae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc435fab2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d1fab6, 3 },
    { 0x3891fab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed5fabc, 4 },
    { 0xe0f5fabd, 4 },
    { 0x6d2dfabd, 3 },
    { 0x233dfabf, 3 },
    { 0, 0 },
    { 0x22cdfac1, 3 },
    { 0xe2fdfac2, 3 },
    { 0xa3b1fac2, 3 },
    { 0x54d1fac4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4051fac7, 5 },
    { 0, 0 },
    { 0x9761fac9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9fad1, 3 },
    { 0xe389fad1, 3 },
    { 0x1eedfad3, 3 },
    { 0, 0 },
    { 0x669dfad5, 3 },
    { 0, 0 },
    { 0x4019fad7, 3 },
    { 0, 0 },
    { 0x6b8dfad9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdfade, 3 },
    { 0xb1f1fade, 3 },
    { 0xb469fae0, 3 },
    { 0x6aadfadf, 3 },
    { 0x3d75fade, 3 },
    { 0x8865fae3, 3 },
    { 0x9b51fae2, 3 },
    { 0, 0 },
    { 0x412dfae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8915fae9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601faf8, 3 },
    { 0x7fd9faf9, 3 },
    { 0xa41fafa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb1fb01, 3 },
    { 0x87c9fb02, 4 },
    { 0xd5fb03, 4 },
    { 0, 0 },
    { 0xd3f9fb05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbdfb14, 3 },
    { 0xbac5fb15, 4 },
    { 0xed9dfb16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb619fb1a, 3 },
    { 0xc4fdfb1b, 3 },
    { 0xd6b9fb1c, 4 },
    { 0xcd99fb1d, 3 },
    { 0x3671fb1e, 3 },
    { 0xd0a1fb1e, 4 },
    { 0xfd5dfb1e, 3 },
    { 0x8399fb21, 4 },
    { 0x15f9fb1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc389fb26, 3 },
    { 0x8e19fb27, 4 },
    { 0x70f5fb27, 3 },
    { 0x5169fb29, 3 },
    { 0x8251fb2a, 3 },
    { 0x7589fb29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c1fb37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2dfb3a, 4 },
    { 0, 0 },
    { 0x2b19fb3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a9fb40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29dfb44, 3 },
    { 0, 0 },
    { 0x8585fb46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975fb49, 4 },
    { 0xa3f9fb4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2dfb50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd199fb53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5fb58, 3 },
    { 0, 0 },
    { 0x4bf5fb5a, 3 },
    { 0xd981fb5a, 3 },
    { 0xb1ddfb5c, 3 },
    { 0, 0 },
    { 0x5bf1fb5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851fb63, 4 },
    { 0, 0 },
    { 0x9f59fb65, 3 },
    { 0, 0 },
    { 0xe37dfb67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf85fb70, 3 },
    { 0x7371fb71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fdfb74, 3 },
    { 0xd275fb74, 3 },
    { 0x6f95fb76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb701fb7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfb84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5fb88, 3 },
    { 0x39a9fb89, 4 },
    { 0x91b1fb88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055fb94, 3 },
    { 0xe091fb95, 5 },
    { 0x5fb96, 3 },
    { 0x8215fb95, 3 },
    { 0xc409fb98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2725fb9c, 5 },
    { 0x4705fb9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa5fba6, 3 },
    { 0, 0 },
    { 0xd725fba8, 3 },
    { 0x5ee9fba9, 4 },
    { 0, 0 },
    { 0x79a1fbab, 3 },
    { 0xe131fbac, 4 },
    { 0xed65fbab, 3 },
    { 0xd075fbae, 4 },
    { 0xb105fbaf, 3 },
    { 0x58d9fbb0, 3 },
    { 0x1339fbae, 3 },
    { 0x4b4dfbab, 3 },
    { 0, 0 },
    { 0x46cdfbb4, 3 },
    { 0xc879fbb5, 3 },
    { 0, 0 },
    { 0x1761fbb7, 4 },
    { 0x569fbb8, 5 },
    { 0xc211fbb8, 3 },
    { 0, 0 },
    { 0x9665fbbb, 3 },
    { 0, 0 },
    { 0x4561fbbd, 4 },
    { 0xd671fbbe, 3 },
    { 0x8891fbbf, 3 },
    { 0xaa31fbbf, 3 },
    { 0x25bdfbbe, 3 },
    { 0x93d5fbc2, 3 },
    { 0, 0 },
    { 0x5aa1fbc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7675fbcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fbd0, 3 },
    { 0, 0 },
    { 0x8599fbd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa999fbd6, 3 },
    { 0xd97dfbd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e89fbdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd991fbdf, 4 },
    { 0xf2d5fbe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e5fbe4, 3 },
    { 0x9995fbe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f1fbed, 3 },
    { 0xc269fbee, 3 },
    { 0, 0 },
    { 0xab89fbf0, 3 },
    { 0xdd59fbf1, 3 },
    { 0xd6b5fbf1, 3 },
    { 0x5569fbf3, 3 },
    { 0xd2a9fbf3, 3 },
    { 0x8339fbf5, 3 },
    { 0x32edfbf1, 3 },
    { 0x72c5fbf4, 3 },
    { 0x85f1fbf8, 3 },
    { 0x22fdfbf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ddfbff, 3 },
    { 0x2ddfc00, 4 },
    { 0x2e99fc01, 3 },
    { 0x4495fc01, 3 },
    { 0x4825fc01, 3 },
    { 0, 0 },
    { 0xa3a5fc05, 3 },
    { 0, 0 },
    { 0x7699fc07, 3 },
    { 0xa5dfc08, 3 },
    { 0xe149fc09, 4 },
    { 0x26b9fc09, 3 },
    { 0xbc4dfc0b, 3 },
    { 0, 0 },
    { 0xb355fc0d, 3 },
    { 0xfb7dfc0d, 4 },
    { 0x7951fc0d, 3 },
    { 0x5e89fc0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d9fc17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439fc1a, 3 },
    { 0x157dfc1b, 3 },
    { 0x1f99fc1c, 3 },
    { 0xca65fc1c, 5 },
    { 0x6d8dfc1e, 3 },
    { 0xc711fc1f, 3 },
    { 0, 0 },
    { 0xbd89fc21, 3 },
    { 0x471fc22, 3 },
    { 0xc9c9fc23, 3 },
    { 0x323dfc24, 4 },
    { 0, 0 },
    { 0xed25fc26, 3 },
    { 0x9d5dfc27, 3 },
    { 0x3ac1fc27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15fc2b, 3 },
    { 0xbcb1fc2c, 3 },
    { 0xbb79fc2d, 3 },
    { 0, 0 },
    { 0x3b99fc2f, 4 },
    { 0, 0 },
    { 0xb83dfc31, 3 },
    { 0x4861fc32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276dfc36, 3 },
    { 0xbdd1fc37, 4 },
    { 0, 0 },
    { 0x4879fc39, 4 },
    { 0xce2dfc3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb1fc40, 4 },
    { 0x608dfc41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7529fc44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad5fc4a, 3 },
    { 0x4f9fc4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565fc4f, 3 },
    { 0xb981fc4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24adfc59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadfc60, 4 },
    { 0, 0 },
    { 0x1d75fc62, 3 },
    { 0x89c1fc63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8585fc68, 3 },
    { 0x8625fc69, 3 },
    { 0x5201fc68, 4 },
    { 0x5fe9fc6b, 4 },
    { 0xde1fc6c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199fc71, 4 },
    { 0x2501fc71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3dfc75, 4 },
    { 0x8535fc76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180dfc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fc7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551fc83, 3 },
    { 0x1b51fc83, 3 },
    { 0x9bcdfc85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf041fc8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c9fc91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b1fc94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2559fc99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dfc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254dfca5, 3 },
    { 0x22e5fca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d31fcac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd79fcaf, 3 },
    { 0xbd3dfcb0, 5 },
    { 0xb0a5fcb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf45fcb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d5fcbb, 3 },
    { 0, 0 },
    { 0x8d61fcbd, 3 },
    { 0x494dfcbe, 3 },
    { 0x811fcbe, 5 },
    { 0xeb11fcc0, 3 },
    { 0xdf19fcbf, 3 },
    { 0xbe49fcc1, 4 },
    { 0, 0 },
    { 0xc129fcc4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcdfcc8, 3 },
    { 0, 0 },
    { 0x482dfcca, 3 },
    { 0, 0 },
    { 0x9b35fccc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb79fccf, 3 },
    { 0, 0 },
    { 0x1171fcd1, 4 },
    { 0, 0 },
    { 0x8335fcd3, 3 },
    { 0x98ddfcd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f35fcd9, 3 },
    { 0x40f9fcda, 3 },
    { 0x3a29fcdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab1fce4, 3 },
    { 0, 0 },
    { 0x4491fce6, 4 },
    { 0, 0 },
    { 0x7eedfce8, 3 },
    { 0xc771fce9, 3 },
    { 0xd041fcea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa455fcf1, 4 },
    { 0xc2edfcf1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71fcf7, 5 },
    { 0x7f7dfcf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddfcfc, 3 },
    { 0xd89fcfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba1fd00, 3 },
    { 0x4e09fd01, 3 },
    { 0, 0 },
    { 0xebc9fd03, 3 },
    { 0, 0 },
    { 0x43fdfd05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f85fd08, 3 },
    { 0x9ebdfd09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a05fd0e, 3 },
    { 0x3199fd0f, 3 },
    { 0xfc5dfd10, 3 },
    { 0xb439fd0e, 5 },
    { 0xad6dfd12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x632dfd15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9495fd1b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe441fd1f, 3 },
    { 0xa7cdfd20, 5 },
    { 0x706dfd1f, 3 },
    { 0x8b1fd22, 3 },
    { 0xccc5fd22, 3 },
    { 0x8a95fd23, 3 },
    { 0x72d5fd22, 3 },
    { 0xbaadfd26, 3 },
    { 0xe87dfd27, 3 },
    { 0xe85dfd28, 3 },
    { 0, 0 },
    { 0x9d25fd2a, 3 },
    { 0xeb51fd2b, 3 },
    { 0x649dfd2c, 3 },
    { 0x405fd2b, 3 },
    { 0xfe95fd2e, 3 },
    { 0, 0 },
    { 0x1ee5fd30, 5 },
    { 0, 0 },
    { 0x8885fd32, 3 },
    { 0x21ddfd32, 3 },
    { 0, 0 },
    { 0x46c9fd35, 3 },
    { 0x5fdfd36, 3 },
    { 0, 0 },
    { 0x6605fd38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935fd3d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fddfd44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb29fd47, 3 },
    { 0xffe5fd48, 3 },
    { 0x87f9fd49, 3 },
    { 0, 0 },
    { 0xeb41fd4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81fd4f, 3 },
    { 0x8d0dfd50, 3 },
    { 0xed35fd51, 3 },
    { 0x33e5fd52, 3 },
    { 0xa25fd52, 4 },
    { 0xd7b9fd54, 3 },
    { 0x4cddfd55, 3 },
    { 0xca31fd55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8835fd59, 3 },
    { 0x52c1fd5a, 4 },
    { 0xae5dfd5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486dfd65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f41fd69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85fd6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299fd70, 3 },
    { 0x3cc9fd71, 3 },
    { 0xa749fd70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b5fd76, 3 },
    { 0x88d9fd77, 3 },
    { 0, 0 },
    { 0xa32dfd79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2dfd7c, 3 },
    { 0, 0 },
    { 0x6889fd7e, 3 },
    { 0, 0 },
    { 0xc4c5fd80, 3 },
    { 0x68a1fd81, 3 },
    { 0, 0 },
    { 0x3045fd83, 3 },
    { 0x6015fd84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a69fd89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe9fd8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdfd92, 3 },
    { 0x3e3dfd93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e55fd99, 3 },
    { 0x40c9fd9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bdfda0, 3 },
    { 0x182dfda1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdfda5, 3 },
    { 0xada9fda5, 3 },
    { 0, 0 },
    { 0xcd35fda8, 3 },
    { 0x24e5fda9, 3 },
    { 0x1951fdaa, 3 },
    { 0, 0 },
    { 0x52a9fdac, 3 },
    { 0xf0a9fdac, 3 },
    { 0, 0 },
    { 0x6f81fdaf, 4 },
    { 0x2eb5fdb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fedfdb6, 4 },
    { 0, 0 },
    { 0xe019fdb8, 4 },
    { 0, 0 },
    { 0x6ba1fdba, 4 },
    { 0x17a1fdbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8509fdc2, 4 },
    { 0x5ef5fdc3, 3 },
    { 0x8721fdc3, 3 },
    { 0xa065fdc5, 4 },
    { 0x9095fdc2, 3 },
    { 0xf619fdc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab1fdca, 4 },
    { 0x9dddfdcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b5fdce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9fdd1, 3 },
    { 0, 0 },
    { 0xdf35fdd3, 3 },
    { 0x9e79fdd4, 3 },
    { 0, 0 },
    { 0x23adfdd6, 4 },
    { 0xba9fdd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1fddb, 4 },
    { 0, 0 },
    { 0x5095fddd, 3 },
    { 0, 0 },
    { 0x4fd9fddf, 3 },
    { 0xf715fddf, 3 },
    { 0x5c79fde1, 3 },
    { 0xded1fde2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b1fdf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f1fdfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e1fe01, 3 },
    { 0x1c31fe01, 3 },
    { 0x40b9fe01, 3 },
    { 0x248dfe02, 3 },
    { 0xe211fe01, 3 },
    { 0x5875fe03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fe0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9fe0f, 3 },
    { 0, 0 },
    { 0x3a09fe11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27dfe16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7431fe1e, 3 },
    { 0x3f0dfe1e, 3 },
    { 0x72fdfe20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c69fe23, 3 },
    { 0, 0 },
    { 0xfb35fe25, 3 },
    { 0x640dfe25, 4 },
    { 0, 0 },
    { 0x1a5dfe28, 3 },
    { 0x4f1fe29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690dfe32, 3 },
    { 0, 0 },
    { 0xa8d5fe34, 3 },
    { 0x5c0dfe35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6115fe3b, 4 },
    { 0x9119fe3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf95fe40, 3 },
    { 0x8d39fe41, 3 },
    { 0x4d21fe42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c95fe47, 3 },
    { 0x2e09fe47, 3 },
    { 0x8c49fe49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d5fe4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf75fe53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5dfe56, 3 },
    { 0x4ffdfe56, 4 },
    { 0xc4d5fe58, 5 },
    { 0x9e1dfe58, 3 },
    { 0x7de9fe5a, 4 },
    { 0x66ddfe5a, 3 },
    { 0xd71dfe5c, 4 },
    { 0xc66dfe5b, 3 },
    { 0xd229fe5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6dfe62, 3 },
    { 0xc7adfe63, 3 },
    { 0x46ddfe64, 4 },
    { 0x6f41fe65, 4 },
    { 0x8761fe65, 3 },
    { 0xe449fe67, 3 },
    { 0xfd25fe68, 3 },
    { 0x28ddfe69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d61fe6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d9fe71, 3 },
    { 0xa961fe72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb5fe78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8739fe85, 3 },
    { 0xb66dfe86, 4 },
    { 0x2a01fe86, 3 },
    { 0xbb19fe87, 3 },
    { 0xf5f5fe88, 3 },
    { 0x1479fe89, 3 },
    { 0xd4a5fe8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad5fe91, 4 },
    { 0xaf15fe92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6dfea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5dfeb0, 3 },
    { 0, 0 },
    { 0x1e91feb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7dfeb6, 3 },
    { 0xd2c1feb6, 4 },
    { 0, 0 },
    { 0xb74dfeb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa361fec0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecdfec3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e71fecb, 4 },
    { 0x22b9fecc, 3 },
    { 0xe989fecc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7dfed2, 4 },
    { 0x483dfed3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4555fed6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9fed9, 3 },
    { 0x4c1fed9, 3 },
    { 0, 0 },
    { 0xe419fedc, 3 },
    { 0x4951fedd, 3 },
    { 0, 0 },
    { 0x5b45fedf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd539fee2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5fee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7dfeeb, 3 },
    { 0xb4dfeec, 3 },
    { 0x894dfeed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bdfef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a75fef3, 3 },
    { 0x3e49fef4, 3 },
    { 0x8c5dfef5, 3 },
    { 0x9085fef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb561fefe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a81ff01, 4 },
    { 0x235ff02, 3 },
    { 0xa925ff03, 3 },
    { 0x43ddff04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f5ff07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b1ff0f, 4 },
    { 0x3ee5ff0f, 3 },
    { 0x2419ff11, 3 },
    { 0xb8a1ff12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145ff15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859ff18, 3 },
    { 0x9079ff19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a9ff1d, 3 },
    { 0x9605ff1e, 3 },
    { 0xf90dff1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ddff26, 3 },
    { 0xff55ff26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f9ff2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f59ff33, 4 },
    { 0xf3adff34, 3 },
    { 0x1ad5ff35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f9ff39, 3 },
    { 0xbf9dff39, 3 },
    { 0x7e4dff3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd115ff3f, 3 },
    { 0, 0 },
    { 0x4d85ff41, 3 },
    { 0x1179ff42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745ff49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89ff4c, 4 },
    { 0x3209ff4d, 4 },
    { 0, 0 },
    { 0x8f81ff4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8409ff56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3671ff5b, 3 },
    { 0x80c5ff5c, 3 },
    { 0x1935ff5b, 3 },
    { 0xc48dff5d, 3 },
    { 0x73f5ff5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c09ff62, 4 },
    { 0xc065ff62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5ff6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf79ff76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3985ff7a, 4 },
    { 0xd639ff7b, 4 },
    { 0x87b1ff7b, 4 },
    { 0x369ff7a, 4 },
    { 0, 0 },
    { 0x789dff7f, 3 },
    { 0x8fd9ff80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62dff83, 3 },
    { 0, 0 },
    { 0x7535ff85, 3 },
    { 0x8495ff85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e9ff8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd791ff8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e9ff90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8791ff93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb639ffa1, 3 },
    { 0x5d31ffa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4395ffa8, 5 },
    { 0x8335ffa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bdffae, 3 },
    { 0x1c81ffaf, 3 },
    { 0xe7cdffb0, 3 },
    { 0xa5fdffb0, 3 },
    { 0x5dfdffb1, 3 },
    { 0, 0 },
    { 0x8b39ffb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5351ffb8, 3 },
    { 0x7291ffb9, 3 },
    { 0xc4a9ffb8, 5 },
    { 0, 0 },
    { 0xee6dffbc, 3 },
    { 0x3fcdffbd, 3 },
    { 0x5671ffbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d25ffc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x582dffc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11ffc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc51ffcd, 3 },
    { 0x3821ffce, 3 },
    { 0, 0 },
    { 0x615dffd0, 3 },
    { 0x78adffd0, 4 },
    { 0x40e9ffd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fdffd7, 3 },
    { 0xff51ffd7, 3 },
    { 0, 0 },
    { 0x71a5ffda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31ffde, 4 },
    { 0xe80dffdf, 3 },
    { 0xcb9ffe0, 3 },
    { 0, 0 },
    { 0x5281ffe2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5ffe6, 3 },
    { 0xf9b9ffe7, 3 },
    { 0x1069ffe8, 3 },
    { 0x8d21ffe9, 3 },
    { 0, 0 },
    { 0x405dffeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ffee, 3 },
    { 0x7261ffef, 3 },
    { 0x2879ffee, 3 },
    { 0, 0 },
    { 0xd6d1fff2, 4 },
    { 0x1661fff3, 4 },
    { 0x4019fff4, 4 },
    { 0, 0 },
    { 0xb381fff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6dfff9, 3 },
    { 0x894dfff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca60001, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a860005, 3 },
    { 0xd2c20005, 3 },
    { 0xd4460007, 4 },
    { 0x7eaa0008, 3 },
    { 0xabf60007, 3 },
    { 0xad06000a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce000d, 3 },
    { 0x103e000e, 3 },
    { 0, 0 },
    { 0x10aa0010, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea2001b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37a001e, 3 },
    { 0, 0 },
    { 0x3a7e0020, 3 },
    { 0xfb860021, 3 },
    { 0, 0 },
    { 0x3b6a0023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceae0027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94a002b, 3 },
    { 0x8252002c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142002f, 4 },
    { 0x4cda0030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a0036, 3 },
    { 0xc7220036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa66003c, 4 },
    { 0, 0 },
    { 0xe7da003e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b60043, 3 },
    { 0xb7720043, 3 },
    { 0x6a660045, 5 },
    { 0x4afa0045, 3 },
    { 0xf78a0046, 4 },
    { 0xbd760046, 3 },
    { 0x9ce20049, 4 },
    { 0x85260048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc6004d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db60050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792a0055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x214a0058, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10be005d, 3 },
    { 0x206a005d, 3 },
    { 0x4a4a005e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20a0062, 3 },
    { 0xe6b60063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489a0069, 3 },
    { 0xcfc2006a, 3 },
    { 0x559e006b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432e006f, 3 },
    { 0, 0 },
    { 0x211a0071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e0074, 3 },
    { 0x6d4e0075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7e0078, 3 },
    { 0, 0 },
    { 0x91b6007a, 3 },
    { 0xc102007a, 3 },
    { 0x32fa007c, 4 },
    { 0x929a007d, 3 },
    { 0x1d52007d, 3 },
    { 0xf4a007d, 4 },
    { 0xcba007d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a0085, 3 },
    { 0x7d220086, 4 },
    { 0xe55e0087, 4 },
    { 0, 0 },
    { 0x2dca0089, 3 },
    { 0x337e008a, 3 },
    { 0xd3ba008b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70960093, 3 },
    { 0, 0 },
    { 0x35de0095, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35e009c, 4 },
    { 0x5b22009c, 3 },
    { 0x8076009e, 4 },
    { 0, 0 },
    { 0x230a00a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed200a4, 3 },
    { 0xaa6a00a4, 3 },
    { 0x8ebe00a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d200a9, 4 },
    { 0xc5ea00a9, 3 },
    { 0xc1c200ab, 3 },
    { 0, 0 },
    { 0x610600ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a600b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea600b9, 3 },
    { 0xa4be00b9, 4 },
    { 0xd1f600bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828e00bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1600c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539600cd, 3 },
    { 0xa78e00ce, 3 },
    { 0x63d200cd, 3 },
    { 0x416600d0, 3 },
    { 0x510a00d0, 3 },
    { 0x54c600d2, 4 },
    { 0x31f200d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840200d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e00dc, 3 },
    { 0x6e7600dc, 3 },
    { 0x9c1600dc, 3 },
    { 0xb4de00df, 4 },
    { 0x354a00e0, 4 },
    { 0x731200e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b600e6, 3 },
    { 0, 0 },
    { 0x151200e8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c200eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fee00f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88200f6, 3 },
    { 0, 0 },
    { 0xc94200f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e00fb, 3 },
    { 0x30d200fc, 3 },
    { 0xb7e00fb, 3 },
    { 0, 0 },
    { 0x712e00ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c960102, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e0106, 3 },
    { 0x269e0107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeae010d, 3 },
    { 0x94f6010e, 3 },
    { 0, 0 },
    { 0x66b20110, 4 },
    { 0xcd1a0110, 3 },
    { 0, 0 },
    { 0x32aa0113, 3 },
    { 0, 0 },
    { 0xe0aa0115, 3 },
    { 0xd5960116, 3 },
    { 0x12a60116, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a2011d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5a0120, 3 },
    { 0x43ee0121, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be012a, 3 },
    { 0x289e012a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8002012e, 3 },
    { 0, 0 },
    { 0xa8360130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f20133, 3 },
    { 0, 0 },
    { 0x9aba0135, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aa0139, 3 },
    { 0xbc42013a, 3 },
    { 0xbae013b, 3 },
    { 0xed3a013a, 3 },
    { 0, 0 },
    { 0xce3e013e, 4 },
    { 0, 0 },
    { 0x460e0140, 4 },
    { 0x959e0140, 3 },
    { 0x40ca0140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83a0146, 4 },
    { 0, 0 },
    { 0x2a4e0148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x630a014b, 3 },
    { 0x8402014c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6560153, 3 },
    { 0xbdde0154, 3 },
    { 0x781a0155, 3 },
    { 0xad060153, 3 },
    { 0xaa120154, 3 },
    { 0x86420153, 3 },
    { 0x47ae0159, 4 },
    { 0xea6e0156, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad20162, 3 },
    { 0, 0 },
    { 0xf2920164, 3 },
    { 0x90360165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ce016f, 3 },
    { 0x4d160170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee0174, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46060177, 5 },
    { 0xa65e0178, 3 },
    { 0xacba0179, 4 },
    { 0xc74e0177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a0187, 3 },
    { 0x86c20188, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a2018b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ee0191, 3 },
    { 0xa30e0192, 4 },
    { 0x7df20191, 4 },
    { 0, 0 },
    { 0xab060195, 3 },
    { 0, 0 },
    { 0xb6460197, 3 },
    { 0, 0 },
    { 0x945e0199, 4 },
    { 0, 0 },
    { 0x1462019b, 3 },
    { 0xffa2019b, 3 },
    { 0x7e52019c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd201a2, 3 },
    { 0x267e01a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647201a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x432601ab, 4 },
    { 0xa65e01ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a01af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899601b5, 3 },
    { 0x34c601b6, 3 },
    { 0, 0 },
    { 0xeec201b8, 3 },
    { 0xf03e01b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x306e01bc, 3 },
    { 0x450601bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290e01c1, 4 },
    { 0, 0 },
    { 0xcb5e01c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3a01ca, 4 },
    { 0, 0 },
    { 0x34aa01cc, 4 },
    { 0, 0 },
    { 0x958201ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7e01d1, 5 },
    { 0xfa3a01d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x671a01d5, 3 },
    { 0, 0 },
    { 0x33f601d7, 4 },
    { 0x754e01d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea01db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc601de, 3 },
    { 0x50aa01df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x614601e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de01e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117201ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x449a01ed, 3 },
    { 0xd87e01ee, 3 },
    { 0xffca01ef, 3 },
    { 0x9e201f0, 3 },
    { 0, 0 },
    { 0x17ce01f2, 3 },
    { 0x26d601f3, 3 },
    { 0xac7a01f2, 3 },
    { 0x9dd601f2, 3 },
    { 0xcf1a01f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e01fa, 4 },
    { 0xd0aa01fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8260200, 4 },
    { 0xccb20200, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61120208, 3 },
    { 0x79820209, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9a020c, 5 },
    { 0x694a020d, 3 },
    { 0, 0 },
    { 0x3686020f, 5 },
    { 0xffea0210, 3 },
    { 0x42660211, 3 },
    { 0x20960210, 3 },
    { 0xf9760213, 3 },
    { 0xb9a20213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ba0217, 3 },
    { 0, 0 },
    { 0x734a0219, 3 },
    { 0xa37e0219, 3 },
    { 0x80020219, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ce021e, 3 },
    { 0xa2fe021f, 3 },
    { 0, 0 },
    { 0x59c20221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f020224, 3 },
    { 0x84a0224, 3 },
    { 0x79fa0226, 3 },
    { 0xf5e60227, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa432022b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ca0230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83a0234, 3 },
    { 0x4f620235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa286023c, 3 },
    { 0, 0 },
    { 0x75f2023e, 3 },
    { 0x9826023f, 4 },
    { 0x14820240, 3 },
    { 0x8f4e0241, 4 },
    { 0x63c20242, 3 },
    { 0xc09e0243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea0249, 4 },
    { 0x106a0249, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebda024d, 4 },
    { 0x5c3e024e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2a0251, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99360254, 3 },
    { 0, 0 },
    { 0xf6c20256, 3 },
    { 0xcf720257, 4 },
    { 0x54ae0257, 4 },
    { 0, 0 },
    { 0xdffa025a, 3 },
    { 0x1546025b, 3 },
    { 0, 0 },
    { 0x74de025d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16a0260, 3 },
    { 0x3520261, 3 },
    { 0, 0 },
    { 0xab6e0263, 3 },
    { 0x8fea0263, 3 },
    { 0x14aa0265, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916a026a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a026d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e0271, 3 },
    { 0, 0 },
    { 0x64ba0273, 4 },
    { 0, 0 },
    { 0xf50a0275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb6027a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6027e, 3 },
    { 0x7ae6027f, 3 },
    { 0x88e2027f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f60283, 3 },
    { 0xf00e0284, 3 },
    { 0xc91a0283, 3 },
    { 0xcd620286, 3 },
    { 0xa8c20287, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a60293, 3 },
    { 0x94620294, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd246029b, 3 },
    { 0x35ca029c, 3 },
    { 0x98ca029d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb602a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3202ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1202b0, 4 },
    { 0x739e02b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53202b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e02b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca602bb, 5 },
    { 0, 0 },
    { 0xe3ba02bd, 4 },
    { 0x52602bd, 3 },
    { 0x767e02bf, 4 },
    { 0x837e02be, 3 },
    { 0, 0 },
    { 0xf40a02c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcaa02c8, 3 },
    { 0x76fa02c9, 3 },
    { 0, 0 },
    { 0xeeba02cb, 4 },
    { 0x947a02cc, 3 },
    { 0, 0 },
    { 0x2a6202ce, 4 },
    { 0x82b602cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789e02d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20de02d9, 3 },
    { 0, 0 },
    { 0x21aa02db, 3 },
    { 0x144202db, 3 },
    { 0, 0 },
    { 0xf92602de, 3 },
    { 0xede202df, 3 },
    { 0x2eee02df, 3 },
    { 0xcb0a02e1, 3 },
    { 0x531602e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ee02e8, 3 },
    { 0xf8aa02e9, 3 },
    { 0x86a602ea, 3 },
    { 0x58ae02ea, 3 },
    { 0xe05202ea, 3 },
    { 0x252e02e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592202f2, 3 },
    { 0, 0 },
    { 0x18aa02f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce02f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf602fe, 3 },
    { 0, 0 },
    { 0x18ea0300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa20309, 3 },
    { 0, 0 },
    { 0xf842030b, 3 },
    { 0xa64a030c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee20314, 3 },
    { 0x3fe20315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8352031a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea2031f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8860322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b2032e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cca0333, 3 },
    { 0x1d2a0334, 3 },
    { 0xb9e60335, 4 },
    { 0xd0920335, 3 },
    { 0x21e20336, 4 },
    { 0x40060337, 3 },
    { 0x1960336, 3 },
    { 0x52e033a, 3 },
    { 0, 0 },
    { 0x4d3e033c, 4 },
    { 0xc8aa033d, 4 },
    { 0x7de033d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4a0345, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1a0348, 4 },
    { 0x14960348, 3 },
    { 0xa8ba034a, 4 },
    { 0xe8e2034b, 3 },
    { 0x5ce2034c, 3 },
    { 0x816034d, 5 },
    { 0xa106034a, 3 },
    { 0x5356034f, 3 },
    { 0xdf3e034f, 4 },
    { 0xb060351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5860354, 4 },
    { 0xb7ae0354, 3 },
    { 0x30820356, 4 },
    { 0, 0 },
    { 0x12060358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14a035f, 3 },
    { 0, 0 },
    { 0x2e2a0361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce0368, 3 },
    { 0x94120369, 3 },
    { 0x94220368, 3 },
    { 0, 0 },
    { 0x6656036c, 3 },
    { 0, 0 },
    { 0xb8c6036e, 4 },
    { 0x619e036f, 3 },
    { 0x8d4e0370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49960373, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39da0378, 3 },
    { 0, 0 },
    { 0x9cf2037a, 3 },
    { 0, 0 },
    { 0xab22037c, 4 },
    { 0, 0 },
    { 0xcb2037e, 4 },
    { 0xa262037f, 3 },
    { 0x2c3e037e, 4 },
    { 0, 0 },
    { 0x6ee0382, 3 },
    { 0x4fb60382, 3 },
    { 0x4aa0384, 3 },
    { 0x18260385, 4 },
    { 0x85660386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b2038b, 5 },
    { 0, 0 },
    { 0x42ee038d, 3 },
    { 0x328a038e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca0394, 4 },
    { 0, 0 },
    { 0xec960396, 3 },
    { 0, 0 },
    { 0xec4a0398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee039d, 3 },
    { 0, 0 },
    { 0x47f6039f, 3 },
    { 0x2bca03a0, 3 },
    { 0x58ba03a0, 4 },
    { 0x168e039f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x306a03a5, 3 },
    { 0xb03203a6, 4 },
    { 0x1c8a03a7, 3 },
    { 0x4dfe03a8, 3 },
    { 0, 0 },
    { 0xd5ee03aa, 3 },
    { 0x903603aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24e03ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370a03b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe03b7, 3 },
    { 0x1f7603b8, 4 },
    { 0, 0 },
    { 0x9cca03ba, 5 },
    { 0xf65e03bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x219203be, 3 },
    { 0x1dc603bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3a03c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b603cc, 3 },
    { 0x723603cd, 3 },
    { 0xfdfe03cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aa03d2, 3 },
    { 0x188203d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee03d6, 3 },
    { 0xc2ee03d6, 3 },
    { 0x5a5a03d8, 3 },
    { 0x56fe03d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x488203dc, 4 },
    { 0x109a03dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297a03e1, 3 },
    { 0x72ca03e2, 3 },
    { 0x5bc603e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebca03f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75603f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efe0401, 3 },
    { 0, 0 },
    { 0xdf460403, 4 },
    { 0xa10a0403, 4 },
    { 0, 0 },
    { 0x10ba0406, 3 },
    { 0x74e60407, 3 },
    { 0xce220408, 3 },
    { 0x27760408, 3 },
    { 0x9916040a, 4 },
    { 0xe322040b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e040e, 3 },
    { 0x9de6040f, 3 },
    { 0, 0 },
    { 0x25560411, 5 },
    { 0x3f60412, 4 },
    { 0x5f820413, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x120416, 3 },
    { 0x72560417, 3 },
    { 0x1a520418, 3 },
    { 0x82be0419, 3 },
    { 0xb476041a, 4 },
    { 0x5d5e041b, 3 },
    { 0, 0 },
    { 0x194e041d, 3 },
    { 0x9ffe041e, 3 },
    { 0xeec6041e, 4 },
    { 0x83560420, 4 },
    { 0xdfee0421, 3 },
    { 0x2b1a0422, 3 },
    { 0x37520420, 3 },
    { 0xe2ea0424, 3 },
    { 0x29a20425, 3 },
    { 0x1c6a0424, 3 },
    { 0x898e0427, 5 },
    { 0xb83a0428, 3 },
    { 0x92760429, 3 },
    { 0x758a0429, 4 },
    { 0, 0 },
    { 0xff1e042c, 3 },
    { 0xd532042d, 3 },
    { 0, 0 },
    { 0xd9ea042f, 4 },
    { 0xaae60430, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fe0433, 4 },
    { 0xeb7a0433, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9a043a, 3 },
    { 0xfc9a043b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac6043e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce20445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dae0449, 3 },
    { 0xe2e60449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc22044d, 3 },
    { 0x99be044e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22560453, 3 },
    { 0xec5e0453, 4 },
    { 0, 0 },
    { 0x6d2e0456, 4 },
    { 0xb98e0457, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7882045a, 3 },
    { 0x8d9a045b, 3 },
    { 0x5b8a045c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d2045f, 3 },
    { 0xa5a20460, 4 },
    { 0, 0 },
    { 0x26b20462, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07a046b, 3 },
    { 0xe6ae046c, 3 },
    { 0x83ae046b, 3 },
    { 0x22aa046e, 3 },
    { 0xad02046c, 3 },
    { 0x525a0470, 4 },
    { 0xc2e20471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef20476, 3 },
    { 0x636a0476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e047b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d520481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504a048a, 3 },
    { 0xb52e048b, 5 },
    { 0x1fb2048c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e048f, 3 },
    { 0, 0 },
    { 0xcdd60491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cee0497, 3 },
    { 0, 0 },
    { 0xe3960499, 4 },
    { 0xc5de049a, 3 },
    { 0xba3e0499, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc2049e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16204a7, 3 },
    { 0x55ca04a8, 3 },
    { 0, 0 },
    { 0x8bc604aa, 3 },
    { 0, 0 },
    { 0xa23604ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6e04b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486204b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6604bd, 3 },
    { 0, 0 },
    { 0xf1ce04bf, 3 },
    { 0x6d4a04c0, 4 },
    { 0x5b2204bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e204cb, 3 },
    { 0x3fce04cb, 3 },
    { 0xdf9e04cb, 3 },
    { 0xd82604ce, 3 },
    { 0xcc0204cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65204d5, 3 },
    { 0x6d4204d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfe04dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4204e2, 3 },
    { 0xa9ae04e2, 3 },
    { 0xcbb204e2, 3 },
    { 0x85b604e5, 3 },
    { 0x986a04e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e604ea, 3 },
    { 0x3c9604ea, 3 },
    { 0x695e04ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20204f8, 4 },
    { 0xf1b204f8, 4 },
    { 0xa68e04fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a204fd, 3 },
    { 0x858204fe, 4 },
    { 0x449204fe, 3 },
    { 0, 0 },
    { 0xc9f20501, 3 },
    { 0xca7a0501, 3 },
    { 0xdd7e0501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda0509, 3 },
    { 0xaf6a050a, 4 },
    { 0x4b96050b, 3 },
    { 0x6876050c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa920515, 4 },
    { 0xd760516, 3 },
    { 0xef9e0517, 3 },
    { 0x21f60517, 3 },
    { 0x71d20516, 3 },
    { 0x5dc20519, 3 },
    { 0x2966051b, 4 },
    { 0x2ee2051c, 3 },
    { 0, 0 },
    { 0xb72e051e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a0523, 3 },
    { 0xd6ce0523, 4 },
    { 0x202a0525, 5 },
    { 0x6d7e0526, 3 },
    { 0x70fa0526, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c6052e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa960531, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe0538, 3 },
    { 0, 0 },
    { 0x7026053a, 5 },
    { 0, 0 },
    { 0x5022053c, 3 },
    { 0x4702053d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa0540, 4 },
    { 0xe7260540, 3 },
    { 0x9f160541, 3 },
    { 0x9bda0540, 4 },
    { 0xe46e0543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a520547, 4 },
    { 0x426a0547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea054b, 4 },
    { 0, 0 },
    { 0x6ba6054d, 4 },
    { 0, 0 },
    { 0x5622054f, 3 },
    { 0x73ba0550, 3 },
    { 0x318e054f, 4 },
    { 0x10260552, 4 },
    { 0xa46a0553, 3 },
    { 0xc10a0554, 3 },
    { 0x3cae0553, 3 },
    { 0x68c60552, 3 },
    { 0xdca20556, 3 },
    { 0xfbfe0551, 4 },
    { 0xae60556, 3 },
    { 0xce8e0553, 3 },
    { 0xf95a055b, 3 },
    { 0xb1c60550, 4 },
    { 0x36920555, 4 },
    { 0x384e055e, 3 },
    { 0x5242055e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3720567, 3 },
    { 0x9f660568, 4 },
    { 0x24a0567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd216056d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9e0571, 3 },
    { 0, 0 },
    { 0x82b60573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5160576, 4 },
    { 0x53760577, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ea057d, 3 },
    { 0, 0 },
    { 0xc292057f, 3 },
    { 0, 0 },
    { 0xedea0581, 4 },
    { 0x47960581, 3 },
    { 0x40c20581, 3 },
    { 0x12aa0584, 3 },
    { 0xa9a0584, 4 },
    { 0xe55a0586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca0589, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ae058c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc20592, 3 },
    { 0x397e0592, 4 },
    { 0x426a0594, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8560599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d2059c, 4 },
    { 0x46ea059c, 3 },
    { 0, 0 },
    { 0xfa22059f, 3 },
    { 0xfcde059f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e205a3, 4 },
    { 0xbeca05a4, 3 },
    { 0x40be05a5, 3 },
    { 0xa0e05a5, 3 },
    { 0x4db605a7, 3 },
    { 0x86d605a4, 4 },
    { 0x5b3e05a9, 3 },
    { 0x2cd205aa, 4 },
    { 0x338a05ab, 3 },
    { 0x8c5a05aa, 3 },
    { 0, 0 },
    { 0x7d9a05ae, 3 },
    { 0, 0 },
    { 0x854a05b0, 4 },
    { 0x1c9a05b1, 4 },
    { 0xa24605b1, 3 },
    { 0, 0 },
    { 0x259a05b4, 3 },
    { 0x2c7e05b4, 3 },
    { 0xe28605b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7605bc, 4 },
    { 0x7abe05bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09e05c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad605c6, 3 },
    { 0, 0 },
    { 0x555605c8, 4 },
    { 0xa09a05c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b205d1, 3 },
    { 0xebde05d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a05d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75205dc, 3 },
    { 0x5aae05dc, 4 },
    { 0x9fb205de, 3 },
    { 0xee8e05df, 3 },
    { 0, 0 },
    { 0x61fa05e1, 4 },
    { 0x2e6205e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a205e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x363605ec, 3 },
    { 0x6205ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75da05f1, 4 },
    { 0xc59e05f2, 4 },
    { 0x5c6a05f2, 3 },
    { 0, 0 },
    { 0xc6c605f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475205fa, 4 },
    { 0, 0 },
    { 0x7a2a05fc, 3 },
    { 0, 0 },
    { 0x933e05fe, 3 },
    { 0xa3ce05ff, 3 },
    { 0x7f2205fe, 3 },
    { 0x9e020600, 4 },
    { 0x1fca05fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f60606, 3 },
    { 0x26be0607, 4 },
    { 0xe78e0608, 3 },
    { 0x3fea0606, 3 },
    { 0xe3fa060a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8e060d, 3 },
    { 0x4422060d, 4 },
    { 0xfdd2060f, 4 },
    { 0xceb20610, 3 },
    { 0xb32a0610, 4 },
    { 0, 0 },
    { 0x38a0613, 4 },
    { 0x6c9e0614, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cea061b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da0621, 3 },
    { 0x70c60621, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbede0625, 4 },
    { 0xab360626, 3 },
    { 0, 0 },
    { 0xcb2e0628, 4 },
    { 0xebc60629, 4 },
    { 0x8d7a0628, 3 },
    { 0, 0 },
    { 0xd96a062c, 3 },
    { 0x4d26062d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52e0630, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc20633, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78620637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb756063a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6a063e, 3 },
    { 0xb4a2063e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40820649, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38760650, 3 },
    { 0x8520651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3560656, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacda0659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a065d, 3 },
    { 0x3d76065d, 4 },
    { 0x5aea065f, 3 },
    { 0x5416065e, 3 },
    { 0xa61e065f, 3 },
    { 0xc62e0661, 3 },
    { 0xdc2a065e, 4 },
    { 0x87a20664, 3 },
    { 0x4ba60665, 4 },
    { 0x66260666, 3 },
    { 0x51120667, 3 },
    { 0, 0 },
    { 0x9c460669, 3 },
    { 0xb772066a, 4 },
    { 0xc442066a, 3 },
    { 0xf7e2066c, 3 },
    { 0x246a0669, 3 },
    { 0x2c26066e, 3 },
    { 0, 0 },
    { 0x798e0670, 3 },
    { 0xdd520671, 4 },
    { 0x16420671, 3 },
    { 0xa9460671, 3 },
    { 0, 0 },
    { 0x9cba0675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea0678, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ba0681, 4 },
    { 0xb0f60682, 3 },
    { 0x7b020683, 5 },
    { 0, 0 },
    { 0x31760685, 3 },
    { 0x88f60686, 3 },
    { 0x936a0687, 3 },
    { 0xf9ee0688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04a068d, 3 },
    { 0x8062068e, 4 },
    { 0x487a068f, 3 },
    { 0x33ea068d, 3 },
    { 0xcff60691, 3 },
    { 0x3daa0691, 4 },
    { 0x5cb20691, 3 },
    { 0x268e0694, 3 },
    { 0xbb760695, 3 },
    { 0x8ba20694, 3 },
    { 0xe2e60697, 3 },
    { 0x8dda068e, 3 },
    { 0, 0 },
    { 0x4fee069a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x553a069d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a606a0, 3 },
    { 0xb5606a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc606a4, 4 },
    { 0xb6a206a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1206a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a606ab, 3 },
    { 0, 0 },
    { 0xff3e06ad, 3 },
    { 0xb58a06ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea06b3, 3 },
    { 0, 0 },
    { 0x16c606b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de06bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x870206be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6606cc, 3 },
    { 0x90aa06cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7606d1, 4 },
    { 0, 0 },
    { 0x366206d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbea06d9, 3 },
    { 0xe9b606d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822606e3, 3 },
    { 0x9be06e3, 4 },
    { 0x44f606e5, 3 },
    { 0xc3d206e5, 3 },
    { 0xdbb606e7, 3 },
    { 0xf74a06e7, 3 },
    { 0xbf606e9, 3 },
    { 0x16e206e6, 3 },
    { 0x809a06e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71206ee, 3 },
    { 0, 0 },
    { 0xe08e06f0, 3 },
    { 0x21a206f1, 3 },
    { 0x88ba06f0, 3 },
    { 0x133206f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6a06f6, 3 },
    { 0, 0 },
    { 0xc73e06f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be0703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a220706, 3 },
    { 0xd8260707, 3 },
    { 0, 0 },
    { 0x9dae0709, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa070c, 3 },
    { 0xe68a070d, 3 },
    { 0x6642070d, 4 },
    { 0xa102070d, 4 },
    { 0x9a60710, 3 },
    { 0x782a0711, 3 },
    { 0x18260712, 3 },
    { 0x7c26070e, 3 },
    { 0x13c60713, 3 },
    { 0x6f9a0715, 4 },
    { 0x8cfe0710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4022071d, 3 },
    { 0x1106071e, 3 },
    { 0x4dc6071f, 3 },
    { 0xd276071d, 3 },
    { 0x11fa0721, 3 },
    { 0x319a0720, 4 },
    { 0, 0 },
    { 0x32960724, 3 },
    { 0xef7e0724, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac20736, 3 },
    { 0xc2da0737, 5 },
    { 0xbcd60737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b6073b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594a073f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5e0744, 3 },
    { 0, 0 },
    { 0x54a0746, 3 },
    { 0x9e9e0747, 4 },
    { 0x87060746, 4 },
    { 0xc81e0749, 3 },
    { 0x5272074a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0a0751, 3 },
    { 0, 0 },
    { 0x43fa0753, 3 },
    { 0xed520754, 4 },
    { 0, 0 },
    { 0xcd9e0756, 3 },
    { 0, 0 },
    { 0x415a0758, 3 },
    { 0x581a0759, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8a075e, 4 },
    { 0x2a02075e, 5 },
    { 0, 0 },
    { 0xce860761, 3 },
    { 0, 0 },
    { 0x5e7e0763, 3 },
    { 0x6b6e0764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333e0768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e2076b, 3 },
    { 0xa29e076b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca20771, 3 },
    { 0x7b7e0772, 3 },
    { 0, 0 },
    { 0xeab60774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb860778, 3 },
    { 0xfaee0779, 4 },
    { 0x93820778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ba077d, 3 },
    { 0xbfa077e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f160781, 3 },
    { 0x18da0782, 4 },
    { 0x768a0782, 3 },
    { 0x41de0784, 3 },
    { 0x73e0785, 3 },
    { 0x31b60782, 3 },
    { 0, 0 },
    { 0xf55e0788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda52078c, 4 },
    { 0xc6ba078d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104e0793, 4 },
    { 0xfa0793, 3 },
    { 0x55aa0794, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601a079a, 3 },
    { 0, 0 },
    { 0x3f52079c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff607a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e07a7, 3 },
    { 0x3d1a07a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ba07ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477607b0, 3 },
    { 0xf3fa07b0, 3 },
    { 0x8d607b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a07b5, 3 },
    { 0, 0 },
    { 0xd63607b7, 3 },
    { 0x895e07b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4e07bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0a07ce, 5 },
    { 0xbc2a07cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x479207d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f607d5, 4 },
    { 0x8aae07d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a607d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc607df, 3 },
    { 0x136207e0, 3 },
    { 0, 0 },
    { 0x9c0607e2, 3 },
    { 0x8cee07e3, 3 },
    { 0x70ea07e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111207ea, 4 },
    { 0xba1e07eb, 3 },
    { 0xe79a07ec, 3 },
    { 0xc67207eb, 3 },
    { 0x4afe07ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883607f5, 3 },
    { 0x55a607f6, 3 },
    { 0, 0 },
    { 0xd74a07f8, 3 },
    { 0x259a07f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87a07ff, 3 },
    { 0, 0 },
    { 0x53060801, 4 },
    { 0xff320801, 3 },
    { 0xcb920803, 3 },
    { 0xa35e0802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa260809, 3 },
    { 0xc9a080a, 3 },
    { 0xd36080b, 4 },
    { 0xe482080b, 3 },
    { 0x440e080d, 4 },
    { 0x11ca0809, 5 },
    { 0, 0 },
    { 0x34760810, 3 },
    { 0, 0 },
    { 0x8a960812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ca0815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ba0818, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb2081f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0827, 3 },
    { 0, 0 },
    { 0x4f8a0829, 3 },
    { 0xd572082a, 3 },
    { 0, 0 },
    { 0x1212082c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d20831, 4 },
    { 0xeee20832, 4 },
    { 0, 0 },
    { 0xfd260834, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3a0837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c2083b, 3 },
    { 0x7bbe083c, 3 },
    { 0x200a083c, 3 },
    { 0, 0 },
    { 0x1dca083f, 3 },
    { 0x7ab60840, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b220850, 3 },
    { 0xae2e0851, 3 },
    { 0, 0 },
    { 0xe3120853, 3 },
    { 0xccf60853, 3 },
    { 0xf6ca0855, 3 },
    { 0, 0 },
    { 0x86d60857, 4 },
    { 0x6ba60858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe085d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87360863, 3 },
    { 0x94de0864, 3 },
    { 0x141a0864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5e086a, 3 },
    { 0, 0 },
    { 0x7552086c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b20871, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45220874, 3 },
    { 0xaab60875, 4 },
    { 0, 0 },
    { 0x26a0877, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5e087a, 3 },
    { 0xd266087b, 3 },
    { 0x151e087b, 4 },
    { 0xcf72087d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17a0881, 3 },
    { 0xf8e20881, 3 },
    { 0x94b20883, 4 },
    { 0xf9420883, 3 },
    { 0xa23e0885, 4 },
    { 0xc2e0884, 3 },
    { 0, 0 },
    { 0x33920888, 4 },
    { 0x6bf60889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b62088c, 3 },
    { 0, 0 },
    { 0xb0b6088e, 3 },
    { 0xb5a088f, 3 },
    { 0x3f3e088f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e20893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fa089d, 3 },
    { 0, 0 },
    { 0xcbe089f, 4 },
    { 0xb27a089f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde208a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f608ac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4e08af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec208b4, 3 },
    { 0x3fa208b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862608c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba608d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x620608d7, 3 },
    { 0xa3c208d8, 3 },
    { 0, 0 },
    { 0xe13208da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874208e3, 3 },
    { 0, 0 },
    { 0x87ea08e5, 3 },
    { 0xbf1208e6, 4 },
    { 0x373208e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9608eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513208f1, 3 },
    { 0x6f9e08f2, 3 },
    { 0xf35e08f3, 3 },
    { 0, 0 },
    { 0x753e08f5, 3 },
    { 0xc07e08f6, 3 },
    { 0x2bf208f7, 3 },
    { 0xb9b208f6, 3 },
    { 0xa74208f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d960905, 4 },
    { 0xa92e0906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9e090e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace0918, 4 },
    { 0xb9ee0919, 3 },
    { 0xc04e0918, 3 },
    { 0xb476091b, 3 },
    { 0x95ea091c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49460921, 4 },
    { 0x624a0922, 3 },
    { 0x1b4a0921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x352a0926, 3 },
    { 0xb5420927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134a092c, 3 },
    { 0x18d2092d, 3 },
    { 0x435e092d, 4 },
    { 0xf61a092c, 3 },
    { 0xeed6092e, 3 },
    { 0, 0 },
    { 0x5120932, 3 },
    { 0x87c60933, 5 },
    { 0x63460933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f20939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9612093d, 4 },
    { 0, 0 },
    { 0x3102093f, 4 },
    { 0, 0 },
    { 0x9a8a0941, 3 },
    { 0, 0 },
    { 0xb79a0943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fa094d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c60955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6620958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93a095d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88760960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e0967, 3 },
    { 0, 0 },
    { 0x4cde0969, 4 },
    { 0, 0 },
    { 0x36a2096b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf2096f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11e0974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee097a, 3 },
    { 0xd462097b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c6097e, 4 },
    { 0, 0 },
    { 0x7d960980, 4 },
    { 0x10f20981, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7660985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bea0988, 3 },
    { 0, 0 },
    { 0xc13e098a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec60992, 3 },
    { 0, 0 },
    { 0x371a0994, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896a099a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ee099e, 3 },
    { 0x44b6099f, 3 },
    { 0xacce09a0, 3 },
    { 0x3ac209a1, 3 },
    { 0xa1da09a1, 3 },
    { 0x8cba09a0, 3 },
    { 0x8a7209a2, 3 },
    { 0x2ade09a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e09a9, 3 },
    { 0x449e09a9, 3 },
    { 0x83c609ab, 3 },
    { 0, 0 },
    { 0x646609ad, 3 },
    { 0, 0 },
    { 0x5e6a09af, 4 },
    { 0x336209af, 3 },
    { 0x84e09b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0609b5, 3 },
    { 0, 0 },
    { 0x963609b7, 4 },
    { 0, 0 },
    { 0xa40209b9, 3 },
    { 0x7fee09b9, 3 },
    { 0x52fa09bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0609be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc609c2, 3 },
    { 0x92d609c3, 3 },
    { 0, 0 },
    { 0x3e0609c5, 3 },
    { 0xa6ba09c6, 3 },
    { 0x104609c6, 4 },
    { 0x879609c8, 4 },
    { 0x20fe09c9, 3 },
    { 0xe7a209c5, 3 },
    { 0xfeba09cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e609d0, 3 },
    { 0x981609d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e09d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8209da, 3 },
    { 0, 0 },
    { 0x51fa09dc, 4 },
    { 0x1aa09dd, 5 },
    { 0x566e09de, 3 },
    { 0x58e609de, 3 },
    { 0xc3d209e0, 3 },
    { 0, 0 },
    { 0x31de09e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ba09e8, 3 },
    { 0xe2aa09e8, 3 },
    { 0x7d6609ea, 3 },
    { 0x296609e9, 5 },
    { 0xa12e09ec, 3 },
    { 0x28e609ed, 3 },
    { 0xcb4609ec, 4 },
    { 0x5e7a09ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6209f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93609fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee20a01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea0a06, 3 },
    { 0, 0 },
    { 0x29fe0a08, 4 },
    { 0xcbe60a08, 4 },
    { 0x33fa0a0a, 4 },
    { 0, 0 },
    { 0xf6b20a0c, 3 },
    { 0x341a0a0d, 4 },
    { 0x96a20a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4860a18, 3 },
    { 0xb1ea0a19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a20a1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85860a1f, 4 },
    { 0, 0 },
    { 0x2b6e0a21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b60a24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26560a27, 5 },
    { 0x23ce0a28, 3 },
    { 0x9b3a0a29, 3 },
    { 0xf8020a29, 4 },
    { 0xe7060a29, 3 },
    { 0x89560a29, 3 },
    { 0xf4a20a2d, 3 },
    { 0xc08a0a2e, 3 },
    { 0x84320a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a20a37, 3 },
    { 0x537a0a38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72160a41, 3 },
    { 0x5d720a42, 3 },
    { 0, 0 },
    { 0x6cda0a44, 3 },
    { 0, 0 },
    { 0x533a0a46, 3 },
    { 0x66ba0a46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbee0a4b, 3 },
    { 0xa7960a4b, 3 },
    { 0xa90e0a4c, 3 },
    { 0x177e0a4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49020a55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5e0a5a, 4 },
    { 0x36ae0a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5de0a60, 3 },
    { 0, 0 },
    { 0x28520a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae0a65, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66860a69, 3 },
    { 0x27320a6a, 4 },
    { 0, 0 },
    { 0x5a260a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d20a72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee0a75, 3 },
    { 0xd4e0a76, 3 },
    { 0x73aa0a77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf260a7a, 3 },
    { 0, 0 },
    { 0x2dda0a7c, 3 },
    { 0x341a0a7d, 3 },
    { 0x5cb60a7d, 3 },
    { 0, 0 },
    { 0x5b9e0a80, 3 },
    { 0xee4e0a81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaee0a8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a0a8f, 5 },
    { 0, 0 },
    { 0x3dba0a91, 3 },
    { 0x9a1e0a92, 3 },
    { 0xbe6a0a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0a9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d60aa4, 3 },
    { 0xe1860aa5, 4 },
    { 0x4e960aa6, 4 },
    { 0x1dbe0aa6, 3 },
    { 0xdd8a0aa8, 4 },
    { 0xa3b60aa9, 3 },
    { 0xfd820aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5a0aad, 3 },
    { 0xd76a0aae, 3 },
    { 0xaf320aaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22420ab5, 3 },
    { 0, 0 },
    { 0x160a0ab7, 4 },
    { 0xb9520ab7, 4 },
    { 0x26fe0ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fe0abc, 3 },
    { 0xe5920abd, 3 },
    { 0x58160abe, 3 },
    { 0, 0 },
    { 0xf9b20ac0, 3 },
    { 0x82860ac0, 3 },
    { 0x83d20ac1, 3 },
    { 0xe3960ac0, 3 },
    { 0xec020ac4, 4 },
    { 0x39a60ac5, 3 },
    { 0xbc520ac6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a0ac9, 4 },
    { 0xe6120ac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa120ad6, 3 },
    { 0xede60ad7, 3 },
    { 0, 0 },
    { 0x9e9a0ad9, 3 },
    { 0x2bc60ada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedca0add, 3 },
    { 0xaec60ade, 3 },
    { 0, 0 },
    { 0x8cf60ae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e0ae4, 3 },
    { 0x22860ae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46460ae8, 3 },
    { 0, 0 },
    { 0x2afa0aea, 3 },
    { 0x69a20aeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ea0af7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x68660afa, 3 },
    { 0x68160afb, 4 },
    { 0x10760afc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x960a0aff, 3 },
    { 0, 0 },
    { 0x90760b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe860b06, 3 },
    { 0x7b8e0b06, 3 },
    { 0x963a0b06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8760b0d, 4 },
    { 0x8ebe0b0d, 3 },
    { 0x8d820b0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5860b13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1a0b17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee0b22, 3 },
    { 0xfc360b22, 4 },
    { 0, 0 },
    { 0x3e160b25, 4 },
    { 0xfc9e0b26, 3 },
    { 0xbe360b25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29520b2a, 3 },
    { 0, 0 },
    { 0xa40e0b2c, 3 },
    { 0xe0020b2d, 4 },
    { 0x85a60b2e, 3 },
    { 0x544e0b2f, 4 },
    { 0xfcaa0b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a0b39, 3 },
    { 0xba4e0b3a, 4 },
    { 0x29ba0b39, 3 },
    { 0x857e0b3b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x83020b3f, 3 },
    { 0, 0 },
    { 0xb43a0b41, 3 },
    { 0x22760b41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41120b4c, 3 },
    { 0xa6820b4c, 3 },
    { 0xd13e0b4d, 4 },
    { 0x835e0b4f, 4 },
    { 0xc460b4f, 3 },
    { 0xe1220b4d, 3 },
    { 0, 0 },
    { 0xdaf20b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eda0b56, 3 },
    { 0xc1120b56, 4 },
    { 0, 0 },
    { 0x312e0b59, 3 },
    { 0xca8e0b5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebe0b62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86420b6e, 3 },
    { 0x64d20b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62920b7b, 4 },
    { 0, 0 },
    { 0xda220b7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a0b81, 3 },
    { 0, 0 },
    { 0x6f820b83, 3 },
    { 0xb7ee0b84, 4 },
    { 0, 0 },
    { 0xa8a60b86, 3 },
    { 0x83aa0b87, 4 },
    { 0, 0 },
    { 0x92f60b89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8e0b8d, 3 },
    { 0, 0 },
    { 0x833a0b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249e0b93, 3 },
    { 0x385e0b93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e0b98, 3 },
    { 0, 0 },
    { 0x85ba0b9a, 3 },
    { 0x7dee0b9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36760b9f, 3 },
    { 0, 0 },
    { 0x88ba0ba1, 3 },
    { 0, 0 },
    { 0xc91e0ba3, 4 },
    { 0x56f20ba4, 3 },
    { 0xebde0ba5, 3 },
    { 0x3b1a0ba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369a0bac, 4 },
    { 0x79e60bad, 4 },
    { 0x60be0bae, 4 },
    { 0x808e0baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ba0bb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cde0bb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73720bb9, 3 },
    { 0xa5a20bba, 3 },
    { 0x7a9a0bbb, 3 },
    { 0xf50a0bb9, 3 },
    { 0, 0 },
    { 0x102a0bbe, 4 },
    { 0xc9be0bbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ba0bd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b60bd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6560bd6, 4 },
    { 0x90c20bd7, 3 },
    { 0xd6ea0bd7, 3 },
    { 0x324e0bd8, 3 },
    { 0x16520bda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3760be0, 4 },
    { 0xda3a0be1, 3 },
    { 0x4ef20be2, 3 },
    { 0x409e0be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e360be6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0be0be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27320bec, 3 },
    { 0xd2d20bed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f220bf0, 3 },
    { 0, 0 },
    { 0xc2fe0bf2, 3 },
    { 0xe2160bf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336e0bfa, 4 },
    { 0, 0 },
    { 0xd9220bfc, 3 },
    { 0xda820bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c20c00, 3 },
    { 0xa0860c00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcda0c05, 4 },
    { 0x93820c06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd420c0a, 4 },
    { 0, 0 },
    { 0xa5120c0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba920c0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165e0c18, 3 },
    { 0xd660c19, 4 },
    { 0x20ce0c1a, 3 },
    { 0, 0 },
    { 0xad0e0c1c, 3 },
    { 0x3af20c1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69a0c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1e0c2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1960c31, 3 },
    { 0, 0 },
    { 0x90160c33, 3 },
    { 0, 0 },
    { 0x8eea0c35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1660c39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3920c3c, 3 },
    { 0x11420c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dee0c43, 4 },
    { 0, 0 },
    { 0x8c3e0c45, 3 },
    { 0x5ec20c46, 4 },
    { 0, 0 },
    { 0xa1620c48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b60c4b, 3 },
    { 0x202e0c4c, 3 },
    { 0, 0 },
    { 0x461a0c4e, 3 },
    { 0xa90e0c4f, 4 },
    { 0x55b20c50, 3 },
    { 0x4fe60c50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba0c5a, 5 },
    { 0x91160c5b, 5 },
    { 0x2ca60c5a, 3 },
    { 0x44c20c5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa0c62, 3 },
    { 0x70520c62, 3 },
    { 0, 0 },
    { 0xee8e0c65, 3 },
    { 0xb8260c66, 3 },
    { 0x3d560c66, 3 },
    { 0xc75a0c68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2920c6b, 4 },
    { 0xdfd60c6c, 3 },
    { 0xeee60c6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60820c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde0c7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b20c82, 3 },
    { 0x5b9e0c83, 4 },
    { 0x7cae0c84, 5 },
    { 0x97de0c82, 4 },
    { 0x86b60c85, 3 },
    { 0x3560c82, 3 },
    { 0x19620c88, 3 },
    { 0x6ba20c89, 3 },
    { 0, 0 },
    { 0x7a620c8b, 4 },
    { 0, 0 },
    { 0x6efe0c8d, 3 },
    { 0x88a20c8e, 3 },
    { 0x9c520c8e, 4 },
    { 0, 0 },
    { 0xd0160c91, 3 },
    { 0xde4a0c92, 3 },
    { 0x22e60c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0860c96, 3 },
    { 0, 0 },
    { 0x1bde0c98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50360c9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38a0ca5, 3 },
    { 0, 0 },
    { 0x91ce0ca7, 4 },
    { 0xa5560ca7, 3 },
    { 0xdf660ca8, 4 },
    { 0xf4fe0caa, 3 },
    { 0x2dea0cab, 4 },
    { 0xec4a0caa, 4 },
    { 0x87020cac, 5 },
    { 0, 0 },
    { 0x5a660caf, 3 },
    { 0x72ea0cb0, 3 },
    { 0, 0 },
    { 0x4ff60cb2, 3 },
    { 0xff820cb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd60cb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ae0cbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127e0cc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d60cca, 3 },
    { 0, 0 },
    { 0x44120ccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611e0cd0, 3 },
    { 0, 0 },
    { 0xa15a0cd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a0cd7, 3 },
    { 0, 0 },
    { 0xcfc20cd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab20cdc, 3 },
    { 0, 0 },
    { 0xd00a0cde, 3 },
    { 0x8f9a0cdf, 5 },
    { 0, 0 },
    { 0x7aca0ce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bda0ce6, 3 },
    { 0, 0 },
    { 0x29320ce8, 3 },
    { 0, 0 },
    { 0x92ee0cea, 4 },
    { 0xee6a0cea, 4 },
    { 0, 0 },
    { 0xa2e20ced, 3 },
    { 0, 0 },
    { 0xfb4e0cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44060cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696a0cf7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1760cfd, 3 },
    { 0xa0160cfd, 4 },
    { 0x92c20cff, 4 },
    { 0x2e160d00, 3 },
    { 0x633e0d01, 4 },
    { 0x819e0d02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9e0d07, 3 },
    { 0, 0 },
    { 0x376a0d09, 3 },
    { 0, 0 },
    { 0xa9660d0b, 3 },
    { 0, 0 },
    { 0xa5ea0d0d, 4 },
    { 0xa4360d0d, 3 },
    { 0x11f60d0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc20d12, 3 },
    { 0xf60a0d13, 3 },
    { 0xccde0d13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a0d1a, 4 },
    { 0, 0 },
    { 0x5ac60d1c, 3 },
    { 0xaf3e0d1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fe0d22, 3 },
    { 0, 0 },
    { 0x1e420d24, 3 },
    { 0xbade0d25, 3 },
    { 0xe1060d26, 3 },
    { 0x6af60d27, 3 },
    { 0x1b860d27, 3 },
    { 0x1ae0d26, 3 },
    { 0x96520d2a, 3 },
    { 0x5a2a0d25, 3 },
    { 0, 0 },
    { 0xe9720d2d, 3 },
    { 0xfea0d2d, 3 },
    { 0, 0 },
    { 0xe05a0d30, 3 },
    { 0x885a0d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a60d35, 4 },
    { 0x88e0d36, 3 },
    { 0x341e0d37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0520d3c, 3 },
    { 0, 0 },
    { 0x98260d3e, 4 },
    { 0xb9760d3f, 3 },
    { 0x32260d3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1a0d4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41760d55, 3 },
    { 0x35220d55, 4 },
    { 0x606a0d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fce0d5f, 3 },
    { 0xb120d60, 3 },
    { 0x574e0d5f, 4 },
    { 0x1dfe0d5f, 4 },
    { 0, 0 },
    { 0xa2f60d64, 3 },
    { 0xa46a0d65, 3 },
    { 0x90520d65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a0d6a, 3 },
    { 0x77460d6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3e0d74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57660d77, 3 },
    { 0, 0 },
    { 0x536a0d79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94a0d7c, 3 },
    { 0xab5e0d7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0d80, 4 },
    { 0xf1760d81, 3 },
    { 0x75ea0d81, 3 },
    { 0x27a20d83, 3 },
    { 0x8a8e0d80, 3 },
    { 0xcbba0d85, 4 },
    { 0x256a0d86, 3 },
    { 0xc5020d83, 4 },
    { 0x9c0e0d87, 3 },
    { 0x58820d80, 3 },
    { 0, 0 },
    { 0xf7a0d8b, 3 },
    { 0xb6c60d8b, 3 },
    { 0xc1760d8d, 4 },
    { 0x463a0d8c, 3 },
    { 0x8dd60d8f, 3 },
    { 0, 0 },
    { 0x53260d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186e0d97, 3 },
    { 0, 0 },
    { 0xa4ca0d99, 4 },
    { 0xdb460d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x235a0d9d, 3 },
    { 0xdbb20d9e, 3 },
    { 0x34aa0d9f, 3 },
    { 0xd3e60da0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5820da7, 3 },
    { 0xf9c60da8, 5 },
    { 0x57b20da7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aba0dac, 3 },
    { 0, 0 },
    { 0x6a220dae, 3 },
    { 0xdea60daf, 3 },
    { 0x56b20db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7e0db5, 3 },
    { 0x8ef60db6, 3 },
    { 0x63e20db7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254e0dbb, 4 },
    { 0x426e0dbb, 3 },
    { 0, 0 },
    { 0xccde0dbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba0dc4, 4 },
    { 0xd8860dc4, 3 },
    { 0xaf1a0dc6, 3 },
    { 0xaa0a0dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea460dcb, 4 },
    { 0x78520dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b60dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c20dd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b20dd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54a0ddd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aa0de0, 3 },
    { 0, 0 },
    { 0xec060de2, 4 },
    { 0xc2d20de2, 4 },
    { 0x49c20de4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca0de9, 4 },
    { 0x163a0dea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78460def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7260df6, 3 },
    { 0x721a0df6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d20dfc, 5 },
    { 0x53760dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ba0e02, 3 },
    { 0, 0 },
    { 0x18fa0e04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baa0e0b, 3 },
    { 0x6ab20e0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6920e0f, 3 },
    { 0xf0de0e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a0e14, 4 },
    { 0, 0 },
    { 0x665a0e16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fca0e19, 5 },
    { 0x15ae0e19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe60e1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1a0e23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x518a0e26, 3 },
    { 0x975a0e26, 3 },
    { 0xa32a0e26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25060e2b, 3 },
    { 0xad1e0e2c, 3 },
    { 0x391a0e2d, 3 },
    { 0x8aee0e2d, 3 },
    { 0xc8d20e2f, 3 },
    { 0, 0 },
    { 0xeaca0e31, 3 },
    { 0, 0 },
    { 0x380a0e33, 4 },
    { 0x60d60e34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ca0e39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216a0e3d, 4 },
    { 0, 0 },
    { 0xc12a0e3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8e0e4b, 3 },
    { 0xaa0e0e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26560e51, 4 },
    { 0x76120e52, 4 },
    { 0, 0 },
    { 0xa3960e54, 3 },
    { 0x66ce0e55, 4 },
    { 0xcf660e56, 3 },
    { 0x27ae0e57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff20e5a, 3 },
    { 0x1a7a0e5b, 3 },
    { 0x74fe0e5b, 3 },
    { 0x3a620e5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x562a0e60, 3 },
    { 0xd4a0e61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a20e64, 4 },
    { 0x54b60e65, 4 },
    { 0xd7fa0e66, 3 },
    { 0x74e0e66, 4 },
    { 0xf18a0e67, 4 },
    { 0x429e0e69, 3 },
    { 0x9db60e6a, 3 },
    { 0xb7fe0e67, 3 },
    { 0, 0 },
    { 0x88d20e6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4a0e70, 3 },
    { 0x63420e70, 4 },
    { 0x5f220e72, 3 },
    { 0x7ab60e73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a0e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68020e7b, 4 },
    { 0xd9fa0e7b, 3 },
    { 0, 0 },
    { 0xae060e7e, 3 },
    { 0, 0 },
    { 0x634e0e80, 3 },
    { 0, 0 },
    { 0x9fbe0e82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464e0e89, 3 },
    { 0xb83a0e89, 3 },
    { 0xfe6a0e8b, 4 },
    { 0x5fd60e8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec20e90, 3 },
    { 0x7dd20e90, 4 },
    { 0, 0 },
    { 0x1cf60e93, 3 },
    { 0xdb9a0e93, 4 },
    { 0x1fa60e95, 3 },
    { 0x5e2a0e95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6e0e9a, 4 },
    { 0x5e720e9b, 4 },
    { 0x3e120e9c, 3 },
    { 0, 0 },
    { 0x7dda0e9e, 3 },
    { 0, 0 },
    { 0xe45a0ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23a0ea3, 3 },
    { 0x2a20ea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f20ea9, 3 },
    { 0xa82a0ea9, 3 },
    { 0, 0 },
    { 0xfd3e0eac, 3 },
    { 0x64a0ead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb20eb6, 3 },
    { 0, 0 },
    { 0xbc860eb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de0ebd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd60ec0, 4 },
    { 0xac2a0ec0, 3 },
    { 0, 0 },
    { 0x76460ec3, 3 },
    { 0, 0 },
    { 0xadd20ec5, 3 },
    { 0xed8a0ec6, 3 },
    { 0x356e0ec7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f60eca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307a0ed0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a0ed5, 3 },
    { 0x6420ed6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5920edd, 3 },
    { 0x3c060edd, 3 },
    { 0xdbde0edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4a0ee3, 3 },
    { 0xf69a0ee3, 4 },
    { 0xca60ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e120ee8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de0eeb, 3 },
    { 0x1e160eec, 3 },
    { 0, 0 },
    { 0xbf160eee, 3 },
    { 0xa60eef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86de0ef2, 3 },
    { 0x952a0ef3, 4 },
    { 0x943e0ef4, 3 },
    { 0, 0 },
    { 0x75a0ef6, 3 },
    { 0x67ca0ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe0efc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e0f01, 3 },
    { 0x77b60f02, 3 },
    { 0x26b20f03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ea0f07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ba0f0b, 3 },
    { 0xd5be0f0c, 3 },
    { 0x10aa0f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63be0f13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20760f17, 3 },
    { 0xb20e0f18, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a920f1b, 5 },
    { 0x11260f1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc160f20, 4 },
    { 0x59420f20, 4 },
    { 0, 0 },
    { 0x975a0f23, 3 },
    { 0x8320f24, 4 },
    { 0x570e0f25, 3 },
    { 0xd6a0f25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7a0f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfede0f37, 3 },
    { 0, 0 },
    { 0x88f20f39, 3 },
    { 0x6dd60f3a, 3 },
    { 0xf0b20f3b, 3 },
    { 0, 0 },
    { 0x3820f3d, 3 },
    { 0xbbda0f3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e0f48, 3 },
    { 0x9b9a0f49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b060f4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x863a0f51, 3 },
    { 0x91ca0f52, 3 },
    { 0xaea20f52, 3 },
    { 0xbd520f54, 4 },
    { 0xb6ca0f53, 3 },
    { 0x244a0f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea0f59, 4 },
    { 0xfbbe0f5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e20f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a0f61, 4 },
    { 0xeaea0f61, 3 },
    { 0x80a20f63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba0f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ee0f6c, 4 },
    { 0, 0 },
    { 0xe44a0f6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e60f73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce0f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0920f7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25e0f86, 3 },
    { 0xd3420f87, 3 },
    { 0x991a0f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2a0f8b, 3 },
    { 0x11ce0f8c, 3 },
    { 0xe7ca0f8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b60f91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1a0f94, 3 },
    { 0xfada0f94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89960f9e, 3 },
    { 0x2fe20f9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b60fb0, 5 },
    { 0, 0 },
    { 0xc9660fb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc60fb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b60fba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f60fbd, 4 },
    { 0, 0 },
    { 0xe36e0fbf, 4 },
    { 0, 0 },
    { 0x3f760fc1, 3 },
    { 0x8e3e0fc2, 3 },
    { 0xcea20fc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35620fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68420fd0, 3 },
    { 0x43a60fd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367a0fd5, 3 },
    { 0x177e0fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e0fe1, 3 },
    { 0x2c9a0fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a60fe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34e0fec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86420ff2, 3 },
    { 0x2c820ff2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaea0ff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0de0ffc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a1000, 3 },
    { 0xc2361001, 3 },
    { 0x3c9a1001, 4 },
    { 0x229a1002, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x94021006, 3 },
    { 0x6b921007, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02100c, 4 },
    { 0xea26100c, 3 },
    { 0x8b5e100e, 4 },
    { 0x1ed6100f, 3 },
    { 0xb96a1010, 3 },
    { 0x7f3a1010, 3 },
    { 0, 0 },
    { 0x87661013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2101d, 5 },
    { 0, 0 },
    { 0x1f9a101f, 3 },
    { 0x83ba1020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc61023, 4 },
    { 0x9d0a1024, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce61027, 4 },
    { 0xb19a1028, 4 },
    { 0xfb5a1029, 3 },
    { 0xdc021029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36102e, 3 },
    { 0x2232102e, 3 },
    { 0xb61e1030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd361034, 3 },
    { 0xef0e1035, 3 },
    { 0x2ae61034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd21039, 3 },
    { 0xe9f2103a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce6103e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b21042, 4 },
    { 0, 0 },
    { 0x12121044, 3 },
    { 0xa0061045, 3 },
    { 0x8ebe1046, 4 },
    { 0x63921047, 3 },
    { 0xdf8a1046, 3 },
    { 0xb3921049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b2104e, 4 },
    { 0, 0 },
    { 0x4f8a1050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1053, 4 },
    { 0xa02e1053, 5 },
    { 0x864e1055, 3 },
    { 0x53821056, 3 },
    { 0x30061057, 3 },
    { 0xa23a1057, 4 },
    { 0xc6e1059, 3 },
    { 0xb22a105a, 3 },
    { 0xaf4e1057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23361062, 3 },
    { 0x33a61063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fae1069, 5 },
    { 0, 0 },
    { 0x1892106b, 3 },
    { 0x35be106c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e1070, 3 },
    { 0xc9f61070, 3 },
    { 0xa9ce1072, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452e1076, 3 },
    { 0x93e61076, 4 },
    { 0x4a321077, 3 },
    { 0xba361079, 3 },
    { 0x59c61079, 3 },
    { 0xc10a107b, 3 },
    { 0, 0 },
    { 0x3fb6107d, 4 },
    { 0, 0 },
    { 0xf826107f, 3 },
    { 0, 0 },
    { 0xd46e1081, 3 },
    { 0xfb2e1082, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baa1088, 3 },
    { 0x739e1088, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe046108d, 3 },
    { 0, 0 },
    { 0x1fba108f, 4 },
    { 0x7a0e1090, 4 },
    { 0, 0 },
    { 0x14b61092, 3 },
    { 0xcc821093, 3 },
    { 0x20261093, 3 },
    { 0xcbde1095, 3 },
    { 0x9e021094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89be1099, 3 },
    { 0, 0 },
    { 0xec6a109b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a10a1, 3 },
    { 0xae4a10a2, 4 },
    { 0x54e10a3, 4 },
    { 0xdcee10a4, 4 },
    { 0x734210a5, 3 },
    { 0xc0f610a5, 4 },
    { 0x595a10a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ae10aa, 4 },
    { 0x3bd610ab, 4 },
    { 0, 0 },
    { 0xf9a10ad, 3 },
    { 0, 0 },
    { 0x501610af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a10b4, 4 },
    { 0xab0a10b4, 3 },
    { 0xd9fa10b6, 4 },
    { 0x972e10b7, 3 },
    { 0xbb5e10b8, 3 },
    { 0, 0 },
    { 0xc15e10ba, 3 },
    { 0, 0 },
    { 0xc78210bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d210bf, 3 },
    { 0x873210c0, 3 },
    { 0xc77e10c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a10c5, 3 },
    { 0x94fa10c5, 3 },
    { 0x381210c6, 4 },
    { 0xd86610c6, 5 },
    { 0xf15210c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d610cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a610d4, 3 },
    { 0xbade10d5, 3 },
    { 0, 0 },
    { 0x66210d7, 5 },
    { 0, 0 },
    { 0x8aca10d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d610dc, 4 },
    { 0xbb2210dd, 3 },
    { 0x993a10dd, 4 },
    { 0, 0 },
    { 0xc6e210e0, 3 },
    { 0, 0 },
    { 0xa84610e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b610e5, 3 },
    { 0x2f0e10e6, 3 },
    { 0x8a3a10e7, 3 },
    { 0xa93a10e8, 4 },
    { 0xf99610e7, 3 },
    { 0x835610ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdce10ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x798210f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb610f4, 3 },
    { 0, 0 },
    { 0x29ae10f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1e10f9, 5 },
    { 0, 0 },
    { 0x739a10fb, 4 },
    { 0xad9210fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc610ff, 3 },
    { 0x9a4a1100, 4 },
    { 0x8fae10ff, 3 },
    { 0, 0 },
    { 0x83221103, 5 },
    { 0x5d2e1103, 4 },
    { 0x2c721103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880e1109, 3 },
    { 0xd582110a, 3 },
    { 0x9da2110b, 3 },
    { 0xf921109, 3 },
    { 0xd156110d, 3 },
    { 0x89e6110e, 3 },
    { 0xc9aa110d, 3 },
    { 0x488a110d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725e1114, 3 },
    { 0, 0 },
    { 0x838e1116, 4 },
    { 0xeb521117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8892111d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc61120, 3 },
    { 0xd98e1121, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ae1126, 3 },
    { 0, 0 },
    { 0x34961128, 4 },
    { 0x58ae1129, 3 },
    { 0xa166112a, 4 },
    { 0, 0 },
    { 0xc0e112c, 3 },
    { 0x24b6112d, 4 },
    { 0xb98a112e, 3 },
    { 0xb61a112e, 4 },
    { 0x892e1130, 3 },
    { 0x894a112f, 3 },
    { 0xf6f6112c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a1135, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a76113a, 3 },
    { 0xbc0e113b, 3 },
    { 0x5412113c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c461140, 3 },
    { 0xc5e61141, 3 },
    { 0x3661140, 3 },
    { 0x28da1142, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96a1146, 3 },
    { 0x178a1147, 3 },
    { 0x3d521146, 3 },
    { 0xb6761149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e2114d, 3 },
    { 0xe7d6114e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d821152, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38a1158, 3 },
    { 0xad561158, 3 },
    { 0x47ce115a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76da115d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca421161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16661166, 4 },
    { 0xe8e21166, 3 },
    { 0x11ca1168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d6116d, 4 },
    { 0xbdb6116d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ee1171, 3 },
    { 0x174e1171, 5 },
    { 0x9b7e1173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e721178, 3 },
    { 0, 0 },
    { 0x402a117a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8e117d, 3 },
    { 0x316a117e, 3 },
    { 0x6a92117f, 3 },
    { 0x9b861180, 3 },
    { 0x9e121180, 3 },
    { 0xf3b21182, 3 },
    { 0x8fe2117f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fe1189, 3 },
    { 0x885a1189, 3 },
    { 0, 0 },
    { 0xd2118c, 3 },
    { 0xaa9a118d, 3 },
    { 0x5d0a118e, 3 },
    { 0x4026118f, 4 },
    { 0x8b121190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d21195, 3 },
    { 0x11721195, 3 },
    { 0x47a61197, 3 },
    { 0, 0 },
    { 0x900a1199, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52119f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91a11a8, 3 },
    { 0, 0 },
    { 0x1b8e11aa, 3 },
    { 0x238a11aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad611ae, 4 },
    { 0xa0b211af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da11b4, 4 },
    { 0x9e9a11b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f611b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x423211bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28611bf, 3 },
    { 0x803a11bf, 3 },
    { 0xff4e11c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7611c9, 3 },
    { 0, 0 },
    { 0xff5611cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3211ce, 3 },
    { 0x376211cf, 3 },
    { 0x77e611d0, 3 },
    { 0x9e5611d1, 3 },
    { 0, 0 },
    { 0x9ff211d3, 3 },
    { 0, 0 },
    { 0x411a11d5, 3 },
    { 0x615211d6, 4 },
    { 0xe6d211d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f211dd, 5 },
    { 0x116e11de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35e11e4, 3 },
    { 0x158e11e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98be11e8, 3 },
    { 0, 0 },
    { 0x7a1a11ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba11f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e11f7, 4 },
    { 0x1f1611f8, 4 },
    { 0, 0 },
    { 0x551e11fa, 3 },
    { 0xa00e11fb, 3 },
    { 0x12ba11fc, 5 },
    { 0, 0 },
    { 0x479211fe, 3 },
    { 0xb62611ff, 4 },
    { 0, 0 },
    { 0xeeb61201, 3 },
    { 0xf5b61202, 3 },
    { 0xe021203, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7a1209, 4 },
    { 0xce421209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b6120f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca1212, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ca1215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c46121e, 3 },
    { 0, 0 },
    { 0x46e1220, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43e1224, 4 },
    { 0x47ba1224, 3 },
    { 0x54461224, 4 },
    { 0x19ba1227, 3 },
    { 0x2ac61227, 3 },
    { 0x9d561225, 3 },
    { 0x96021226, 3 },
    { 0x46e122b, 4 },
    { 0x547a122c, 3 },
    { 0x45ae122d, 3 },
    { 0, 0 },
    { 0xc87a122f, 3 },
    { 0, 0 },
    { 0xa0ca1231, 3 },
    { 0xaa3e1231, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda1235, 4 },
    { 0, 0 },
    { 0x3bf21237, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb2123b, 3 },
    { 0, 0 },
    { 0x13ca123d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a961241, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8121248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2124b, 4 },
    { 0x2b2a124b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e21250, 3 },
    { 0, 0 },
    { 0xa521252, 3 },
    { 0xb54e1252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ce1256, 4 },
    { 0xf67a1256, 3 },
    { 0x94ea1258, 3 },
    { 0x98f21259, 4 },
    { 0xb2a2125a, 3 },
    { 0, 0 },
    { 0x682e125c, 3 },
    { 0, 0 },
    { 0x3eda125e, 4 },
    { 0x97e6125f, 3 },
    { 0, 0 },
    { 0x73ee1261, 3 },
    { 0x928a1262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda1265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73e1268, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2a126b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf666126f, 3 },
    { 0xbd561270, 4 },
    { 0xbd521271, 3 },
    { 0xd33e1272, 3 },
    { 0xbdc61273, 3 },
    { 0x2fde1272, 4 },
    { 0x754e1275, 3 },
    { 0x70921275, 4 },
    { 0x6f6e1277, 3 },
    { 0xc8ee1275, 4 },
    { 0xa786126f, 4 },
    { 0, 0 },
    { 0x9646127b, 3 },
    { 0xa96a127b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411e1280, 3 },
    { 0, 0 },
    { 0x92d61282, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1a1285, 3 },
    { 0x55fa1285, 3 },
    { 0xd3b61285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88a128b, 3 },
    { 0, 0 },
    { 0x372a128d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a21290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07a1296, 3 },
    { 0, 0 },
    { 0x3dde1298, 3 },
    { 0xd5721299, 3 },
    { 0xf0ca1299, 3 },
    { 0xf65e129b, 3 },
    { 0xb50e129c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f612a1, 4 },
    { 0x1aa12a2, 3 },
    { 0x418612a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589612a7, 4 },
    { 0, 0 },
    { 0x3efa12a9, 3 },
    { 0xf2be12aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0e12ad, 3 },
    { 0xcfc612ae, 4 },
    { 0x2d212ad, 3 },
    { 0, 0 },
    { 0xb3f612b1, 3 },
    { 0xdd6e12b2, 4 },
    { 0xb7a212b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9212b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3212c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x133212c7, 3 },
    { 0x702e12c8, 4 },
    { 0xaa0e12c9, 3 },
    { 0, 0 },
    { 0xd24e12cb, 4 },
    { 0x4c6a12cc, 4 },
    { 0, 0 },
    { 0x43ee12ce, 3 },
    { 0, 0 },
    { 0xa4ce12d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9612d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65da12d9, 3 },
    { 0, 0 },
    { 0xca6e12db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26212e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fba12e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32aa12e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba12ee, 3 },
    { 0, 0 },
    { 0x6db612f0, 3 },
    { 0x4a8612f0, 3 },
    { 0x19ca12f2, 3 },
    { 0xad4a12f3, 3 },
    { 0xa83612f4, 4 },
    { 0x644e12f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514612fe, 3 },
    { 0, 0 },
    { 0xe34e1300, 3 },
    { 0x5e421300, 3 },
    { 0, 0 },
    { 0x54621303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfede130a, 3 },
    { 0, 0 },
    { 0x5706130c, 4 },
    { 0x341e130c, 3 },
    { 0, 0 },
    { 0x210a130f, 3 },
    { 0, 0 },
    { 0x19561311, 3 },
    { 0x922e1312, 3 },
    { 0x1dd61312, 3 },
    { 0x6d961312, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799e1319, 3 },
    { 0x85de131a, 3 },
    { 0xa8721319, 3 },
    { 0x1836131c, 4 },
    { 0xc91a1319, 3 },
    { 0, 0 },
    { 0x518a131f, 3 },
    { 0x15a1320, 3 },
    { 0x79ee1321, 3 },
    { 0xca3a1322, 3 },
    { 0, 0 },
    { 0xcb321324, 4 },
    { 0x44c21325, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7416132b, 5 },
    { 0xa986132c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6132f, 3 },
    { 0, 0 },
    { 0x545a1331, 3 },
    { 0, 0 },
    { 0x4b961333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a1336, 3 },
    { 0, 0 },
    { 0x54c61338, 4 },
    { 0, 0 },
    { 0x8db6133a, 3 },
    { 0, 0 },
    { 0xd5b2133c, 3 },
    { 0xc096133c, 3 },
    { 0x3976133e, 3 },
    { 0xb3a6133e, 3 },
    { 0xb1221340, 3 },
    { 0x22c21340, 3 },
    { 0x9886133e, 3 },
    { 0xefe2133d, 3 },
    { 0, 0 },
    { 0xbeb61345, 3 },
    { 0x35461345, 3 },
    { 0, 0 },
    { 0xcbba1348, 4 },
    { 0xbd561349, 3 },
    { 0x3626134a, 4 },
    { 0x1e66134b, 3 },
    { 0x57de134b, 3 },
    { 0x78ae134c, 4 },
    { 0x5521349, 3 },
    { 0xa51e134f, 4 },
    { 0xe482134d, 4 },
    { 0x775e1348, 3 },
    { 0x69c61352, 3 },
    { 0xa3f2134c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd21356, 3 },
    { 0xebea1357, 3 },
    { 0xb2661358, 4 },
    { 0xee721358, 3 },
    { 0xd69e135a, 3 },
    { 0x6a8a135b, 3 },
    { 0xdc12135b, 3 },
    { 0x466135d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde52136c, 4 },
    { 0x9c6a136c, 3 },
    { 0xb946136c, 4 },
    { 0xd96e136f, 3 },
    { 0x3e21370, 3 },
    { 0x2456136f, 3 },
    { 0xab7e1372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae137c, 3 },
    { 0x75ee137c, 4 },
    { 0xc39e137e, 3 },
    { 0x8a6e137f, 3 },
    { 0x85261380, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdba1389, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba138c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e1392, 4 },
    { 0xec421393, 3 },
    { 0x75e1394, 3 },
    { 0xbaa61395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef6139f, 3 },
    { 0x104213a0, 4 },
    { 0xe48613a1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383a13a9, 4 },
    { 0, 0 },
    { 0xae4213ab, 3 },
    { 0xc95e13ab, 3 },
    { 0, 0 },
    { 0xa56a13ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12a13b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7e13b7, 4 },
    { 0xf65213b8, 3 },
    { 0xe3613b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c613c1, 4 },
    { 0x659e13c2, 3 },
    { 0x8f7e13c2, 3 },
    { 0xab6a13c4, 3 },
    { 0, 0 },
    { 0x2fa13c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e13ca, 3 },
    { 0x4ae213cb, 4 },
    { 0xa6d213cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a13d6, 4 },
    { 0, 0 },
    { 0x9bd213d8, 3 },
    { 0x5c7a13d9, 4 },
    { 0xee4e13da, 3 },
    { 0x2f1a13da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x174613de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d613e2, 3 },
    { 0x641613e3, 4 },
    { 0x4a5213e4, 4 },
    { 0xb87613e4, 3 },
    { 0xb6f613e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe213e9, 5 },
    { 0x64b213ea, 3 },
    { 0xf91e13ea, 3 },
    { 0x284213eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8a13f0, 3 },
    { 0x1e4e13f1, 3 },
    { 0, 0 },
    { 0xd26213f3, 3 },
    { 0x203e13f4, 3 },
    { 0x441613f5, 3 },
    { 0x140213f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f561401, 3 },
    { 0, 0 },
    { 0xb80a1403, 3 },
    { 0x94321404, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d61407, 3 },
    { 0xa2021408, 3 },
    { 0, 0 },
    { 0xd606140a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ea140f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18a1416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e6141c, 4 },
    { 0x63f2141d, 3 },
    { 0x6792141e, 3 },
    { 0, 0 },
    { 0xb41a1420, 4 },
    { 0x19961421, 3 },
    { 0, 0 },
    { 0x64d21423, 3 },
    { 0x9fea1423, 3 },
    { 0, 0 },
    { 0xc6821426, 3 },
    { 0xd7621426, 3 },
    { 0xc521426, 3 },
    { 0x601e1429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2a142e, 3 },
    { 0, 0 },
    { 0xec0a1430, 3 },
    { 0, 0 },
    { 0xc1e21432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9a1435, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x669e1438, 3 },
    { 0, 0 },
    { 0xb162143a, 3 },
    { 0, 0 },
    { 0x4eea143c, 3 },
    { 0x71f6143d, 3 },
    { 0, 0 },
    { 0x60c2143f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94321443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8a1447, 3 },
    { 0, 0 },
    { 0xc16e1449, 3 },
    { 0, 0 },
    { 0x57da144b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a144e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760a1452, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a1458, 3 },
    { 0x45b21459, 3 },
    { 0x56c2145a, 3 },
    { 0x757a1458, 3 },
    { 0x9a2145b, 4 },
    { 0, 0 },
    { 0xba52145e, 3 },
    { 0xd79a145e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4a1462, 4 },
    { 0x8de61462, 4 },
    { 0xc8261464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25a146b, 3 },
    { 0xcfea146c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ae1473, 3 },
    { 0x16f21473, 3 },
    { 0x61a21475, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeba1478, 4 },
    { 0x4c761479, 4 },
    { 0xe022147a, 3 },
    { 0x6a6147b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf21484, 3 },
    { 0x970a1485, 4 },
    { 0x8c8e1486, 3 },
    { 0x61561484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ba148b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27061492, 3 },
    { 0x15ca1493, 3 },
    { 0, 0 },
    { 0x63061495, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187e149a, 3 },
    { 0x4c02149b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572e14a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38614a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfe14a8, 4 },
    { 0, 0 },
    { 0x6da614aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfa14b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4e14b6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x872a14b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93a14bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f614bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ce14c3, 4 },
    { 0xae1614c4, 3 },
    { 0x1ee14c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b614cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405a14d6, 4 },
    { 0x64ca14d6, 3 },
    { 0xe59a14d8, 3 },
    { 0x8ca214d9, 3 },
    { 0x47d614da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e14df, 4 },
    { 0xa99614e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802a14e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2214ee, 3 },
    { 0, 0 },
    { 0x218a14f0, 3 },
    { 0x287214f1, 3 },
    { 0, 0 },
    { 0x536214f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3e14f7, 4 },
    { 0x42ea14f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e1500, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4921509, 3 },
    { 0xca8a1509, 4 },
    { 0x21161509, 3 },
    { 0xfb42150c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69e150f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca1514, 3 },
    { 0, 0 },
    { 0x9a4e1516, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfa1519, 3 },
    { 0x44ca151a, 4 },
    { 0x8c4e151b, 3 },
    { 0xf6c2151b, 3 },
    { 0xb7ae151c, 4 },
    { 0xd456151e, 4 },
    { 0xd88e151f, 3 },
    { 0xd63e151f, 3 },
    { 0x9f3a1521, 3 },
    { 0x3c4a151b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47e1526, 3 },
    { 0x63321527, 3 },
    { 0x50061528, 3 },
    { 0x523a1529, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e152c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2172152f, 3 },
    { 0, 0 },
    { 0xd2561531, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd21538, 3 },
    { 0, 0 },
    { 0xbbb2153a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f421540, 3 },
    { 0x66d21541, 3 },
    { 0x1a1e1542, 3 },
    { 0, 0 },
    { 0xe5021544, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a1548, 3 },
    { 0, 0 },
    { 0xdd0a154a, 3 },
    { 0, 0 },
    { 0x1a16154c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da1554, 3 },
    { 0x84161554, 3 },
    { 0xcbbe1556, 3 },
    { 0xfad21557, 3 },
    { 0x5b261558, 3 },
    { 0x56c61559, 3 },
    { 0, 0 },
    { 0x7022155b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e1563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85a1566, 3 },
    { 0x2b721567, 3 },
    { 0x7fb21567, 3 },
    { 0x3e21567, 3 },
    { 0xbec21566, 3 },
    { 0xd00a156b, 3 },
    { 0xa76a156c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13e156f, 3 },
    { 0x2a1e1570, 4 },
    { 0xe9b21571, 3 },
    { 0x69c21571, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ede1575, 3 },
    { 0x66fa1576, 3 },
    { 0, 0 },
    { 0x3aca1578, 4 },
    { 0, 0 },
    { 0xc8a157a, 3 },
    { 0x22a6157b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61581, 3 },
    { 0, 0 },
    { 0x9e2a1583, 3 },
    { 0xd4fa1584, 4 },
    { 0, 0 },
    { 0x27021586, 3 },
    { 0, 0 },
    { 0x65aa1588, 3 },
    { 0x631a1589, 4 },
    { 0x8da61589, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb6158d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x492e1590, 3 },
    { 0, 0 },
    { 0x312a1592, 3 },
    { 0xf4e21592, 3 },
    { 0x3d5a1594, 3 },
    { 0, 0 },
    { 0x3af61596, 5 },
    { 0x2e461596, 3 },
    { 0x4d861596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc6159b, 3 },
    { 0xce6a159c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5215a4, 3 },
    { 0, 0 },
    { 0x640215a6, 3 },
    { 0xcc5615a7, 3 },
    { 0x764615a7, 3 },
    { 0xd38a15a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x598e15ac, 3 },
    { 0x2a7a15ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f215b0, 3 },
    { 0xba8e15b1, 4 },
    { 0x411615b2, 3 },
    { 0x747615b1, 4 },
    { 0x195a15b4, 4 },
    { 0xc02215b5, 3 },
    { 0x969a15b2, 3 },
    { 0, 0 },
    { 0x36d615b8, 3 },
    { 0x9a3e15b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fe15c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833615c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa215cb, 3 },
    { 0xdbd215cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0615d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9615d4, 3 },
    { 0xf41615d5, 3 },
    { 0, 0 },
    { 0x616a15d7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab215db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac615df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4615e2, 4 },
    { 0, 0 },
    { 0xbf3a15e4, 3 },
    { 0, 0 },
    { 0x630215e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e15ea, 3 },
    { 0x823e15ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13615ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2615f2, 3 },
    { 0x555615f3, 4 },
    { 0xeda615f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a15f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3615fb, 4 },
    { 0xb15e15fc, 4 },
    { 0, 0 },
    { 0x9d5215fe, 3 },
    { 0x5eae15ff, 3 },
    { 0xea4a15fe, 3 },
    { 0x4be61600, 3 },
    { 0x837e15fe, 3 },
    { 0xaca21603, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e1608, 3 },
    { 0xb1261609, 4 },
    { 0x598e160a, 3 },
    { 0xacb61608, 4 },
    { 0, 0 },
    { 0x558e160d, 3 },
    { 0x30a160e, 4 },
    { 0x9156160e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eae1613, 3 },
    { 0, 0 },
    { 0xfb721615, 3 },
    { 0, 0 },
    { 0x838e1617, 3 },
    { 0, 0 },
    { 0x1bca1619, 3 },
    { 0x9416161a, 3 },
    { 0x2cfe161a, 4 },
    { 0, 0 },
    { 0x2226161d, 3 },
    { 0x840a161e, 3 },
    { 0x1b6161d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae1622, 3 },
    { 0, 0 },
    { 0x10b21624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886a1629, 3 },
    { 0x1062162a, 4 },
    { 0x8396162b, 3 },
    { 0xd1f2162b, 3 },
    { 0xacb6162c, 3 },
    { 0x24fa162e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1521638, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a163b, 4 },
    { 0x5cea163c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a21647, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2202164a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8a164f, 3 },
    { 0, 0 },
    { 0x3b161651, 3 },
    { 0x265a1652, 3 },
    { 0x2f461652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c6165d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa21661, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ee1666, 3 },
    { 0, 0 },
    { 0x8f661668, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x887a166b, 4 },
    { 0, 0 },
    { 0xb4e2166d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d21674, 3 },
    { 0x3b521675, 3 },
    { 0x5d61676, 3 },
    { 0xa4f21676, 3 },
    { 0, 0 },
    { 0xb4021679, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca167e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba1681, 3 },
    { 0x27e61681, 3 },
    { 0x6bf61683, 3 },
    { 0, 0 },
    { 0x49b21685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c6168c, 3 },
    { 0x9bd2168d, 4 },
    { 0x2f6a168e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d61691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b21695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f2169d, 3 },
    { 0x8d0e169e, 4 },
    { 0x6642169d, 3 },
    { 0x5dee16a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde16a3, 3 },
    { 0, 0 },
    { 0x419216a5, 4 },
    { 0, 0 },
    { 0x9ce616a7, 3 },
    { 0xde7a16a8, 3 },
    { 0x6fd216a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x760e16ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x539e16af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc616b2, 3 },
    { 0x9cde16b3, 4 },
    { 0, 0 },
    { 0x29a216b5, 3 },
    { 0xfcae16b6, 3 },
    { 0, 0 },
    { 0x7cd216b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacae16bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e16c1, 3 },
    { 0x217216c1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd216c9, 4 },
    { 0xe49616c9, 3 },
    { 0xf39a16ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275e16cf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c616d9, 5 },
    { 0, 0 },
    { 0x97fa16db, 3 },
    { 0x7afe16db, 3 },
    { 0, 0 },
    { 0xff6216de, 3 },
    { 0xbe1a16de, 3 },
    { 0x4c3216df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ca16e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639e16f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9a16f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51616f9, 3 },
    { 0, 0 },
    { 0x51a16fb, 3 },
    { 0xa96616fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f216ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca1702, 5 },
    { 0, 0 },
    { 0x71961704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa21707, 3 },
    { 0xaa0e1708, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6192170d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2e1710, 3 },
    { 0xd2fa1710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e421720, 3 },
    { 0x730a1721, 4 },
    { 0x526e1721, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a21729, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea56172c, 4 },
    { 0x88ee172c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x843e1730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c921733, 3 },
    { 0xa5ce1734, 3 },
    { 0x503a1735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e173a, 4 },
    { 0x111a173a, 3 },
    { 0xb58a173c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0621745, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe1749, 3 },
    { 0x84be174a, 3 },
    { 0x145a174b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a174e, 4 },
    { 0xbaa174f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61861753, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eca1758, 4 },
    { 0, 0 },
    { 0xa906175a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9e175e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba1762, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a661765, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fee176b, 5 },
    { 0xb516176b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ea1770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5061775, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93be1778, 4 },
    { 0xbe6a1778, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7e177c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7232177f, 4 },
    { 0x32e21780, 3 },
    { 0x63021781, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26178b, 3 },
    { 0x5f62178b, 3 },
    { 0, 0 },
    { 0xa17a178e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e1793, 3 },
    { 0xda261794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a1797, 3 },
    { 0x8521797, 3 },
    { 0x14161797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfe17a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895a17a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18617ac, 3 },
    { 0x11ee17ad, 3 },
    { 0x1dae17ae, 3 },
    { 0xfde17af, 3 },
    { 0xbdae17b0, 3 },
    { 0x477617b0, 3 },
    { 0xd6a17b1, 3 },
    { 0xf0a217ac, 3 },
    { 0xec9e17ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6617b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a17bb, 3 },
    { 0xa96217bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a17c5, 3 },
    { 0xc8e17c6, 3 },
    { 0xc2f217c7, 3 },
    { 0xc65617c6, 3 },
    { 0xc39e17c9, 3 },
    { 0, 0 },
    { 0x926617cb, 3 },
    { 0x826617cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aa17d0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e217d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9a17da, 4 },
    { 0, 0 },
    { 0xe39217dc, 4 },
    { 0xab7a17dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aee17e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ea17e3, 3 },
    { 0x8f5617e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85217e9, 3 },
    { 0x9c6a17ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a17ed, 4 },
    { 0x122a17ee, 3 },
    { 0x588617ed, 3 },
    { 0, 0 },
    { 0xdd217f1, 3 },
    { 0x888a17f2, 4 },
    { 0xa63617f3, 4 },
    { 0x869617f4, 3 },
    { 0x19ba17f2, 4 },
    { 0xa8ee17f2, 3 },
    { 0xbcd617f7, 3 },
    { 0xe8be17f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x120617fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e17fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x395a1801, 3 },
    { 0x777a1802, 3 },
    { 0xab421802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83661808, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600a180e, 3 },
    { 0xc822180f, 3 },
    { 0xfcb21810, 3 },
    { 0x93961811, 3 },
    { 0x80261811, 3 },
    { 0, 0 },
    { 0xa4121814, 3 },
    { 0, 0 },
    { 0x4a1816, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ea1819, 4 },
    { 0, 0 },
    { 0xf356181b, 3 },
    { 0x9c6181c, 3 },
    { 0x51b6181d, 5 },
    { 0xcd8a181e, 3 },
    { 0x6c42181f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5e1825, 3 },
    { 0x71a61826, 3 },
    { 0x82021827, 4 },
    { 0x12da1828, 3 },
    { 0xd7061829, 3 },
    { 0x273e182a, 3 },
    { 0x4a66182a, 3 },
    { 0xad0e182a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a1835, 4 },
    { 0, 0 },
    { 0x288e1837, 4 },
    { 0x5eae1838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f61843, 3 },
    { 0, 0 },
    { 0x72e1845, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a461849, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x156184c, 4 },
    { 0, 0 },
    { 0x65c6184e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a1852, 3 },
    { 0x641a1853, 4 },
    { 0xdb0a1854, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1862185f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b361869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6186d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd161876, 3 },
    { 0x956e1877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6187d, 3 },
    { 0, 0 },
    { 0x23e187f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47461882, 3 },
    { 0, 0 },
    { 0x12961884, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bce1887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88e188a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3661891, 3 },
    { 0, 0 },
    { 0xd50a1893, 3 },
    { 0xc9ea1893, 4 },
    { 0x8b1a1895, 3 },
    { 0x29d21895, 3 },
    { 0x34fa1896, 4 },
    { 0x5f3a1898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9592189b, 3 },
    { 0xb572189c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1218a7, 4 },
    { 0x8b8218a8, 3 },
    { 0, 0 },
    { 0xc1c218aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a218ae, 3 },
    { 0xd3218af, 3 },
    { 0x65ea18b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a18b3, 3 },
    { 0xf63618b4, 3 },
    { 0, 0 },
    { 0x714a18b6, 3 },
    { 0x588e18b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8618ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b618c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5618cb, 4 },
    { 0, 0 },
    { 0x967a18cd, 3 },
    { 0xbd7a18ce, 3 },
    { 0x44ba18cf, 4 },
    { 0, 0 },
    { 0xf2a218d1, 3 },
    { 0, 0 },
    { 0x907a18d3, 3 },
    { 0, 0 },
    { 0xf73218d5, 3 },
    { 0, 0 },
    { 0x926a18d7, 4 },
    { 0, 0 },
    { 0x34a18d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce218dd, 3 },
    { 0, 0 },
    { 0x56ae18df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e18e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d618e9, 3 },
    { 0, 0 },
    { 0xf2d218eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f618ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34218f2, 3 },
    { 0x424218f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5e18f8, 4 },
    { 0x45f618f9, 3 },
    { 0xa73a18fa, 3 },
    { 0, 0 },
    { 0x207e18fc, 3 },
    { 0x6f6618fc, 5 },
    { 0x153218fe, 4 },
    { 0x376618fe, 3 },
    { 0xdae218ff, 3 },
    { 0, 0 },
    { 0xe7de1902, 3 },
    { 0x189e1903, 3 },
    { 0xd7ee1904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab861909, 3 },
    { 0x3c06190a, 3 },
    { 0xfdf2190b, 3 },
    { 0xe63a190a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcba190f, 3 },
    { 0x33d2190f, 4 },
    { 0xd5321910, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d61916, 3 },
    { 0x9c9a1917, 3 },
    { 0, 0 },
    { 0x400a1919, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff02191c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad61922, 3 },
    { 0x3dc61923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6661926, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544a192c, 4 },
    { 0xe686192d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3721934, 3 },
    { 0x9aea1934, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9a1939, 4 },
    { 0x6a2a1939, 3 },
    { 0x1ede193b, 3 },
    { 0x950e193b, 3 },
    { 0, 0 },
    { 0xb82e193e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a1944, 3 },
    { 0, 0 },
    { 0xe7d61946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c2194e, 3 },
    { 0x41ea194f, 3 },
    { 0x7756194f, 3 },
    { 0xe5f61951, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8061957, 3 },
    { 0, 0 },
    { 0x8c361959, 4 },
    { 0, 0 },
    { 0xdbfa195b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7522195f, 3 },
    { 0, 0 },
    { 0x692e1961, 4 },
    { 0x162a1962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b421965, 3 },
    { 0, 0 },
    { 0x43821967, 3 },
    { 0x8cb21968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc6196d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2a1972, 3 },
    { 0, 0 },
    { 0x21f21974, 3 },
    { 0, 0 },
    { 0x681e1976, 4 },
    { 0x15321977, 4 },
    { 0xbfe1978, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6e197d, 3 },
    { 0x296e197e, 3 },
    { 0, 0 },
    { 0x904a1980, 3 },
    { 0xe1ba1980, 3 },
    { 0x44fa1982, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xac161985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a198b, 3 },
    { 0xd3be198b, 4 },
    { 0, 0 },
    { 0x8b2198e, 4 },
    { 0, 0 },
    { 0x7d5a1990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ba1993, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51e1998, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a619a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77a19ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e219b2, 3 },
    { 0x74aa19b3, 3 },
    { 0x524619b3, 3 },
    { 0x86ae19b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ca19ba, 3 },
    { 0x47b619ba, 3 },
    { 0x355619bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864e19c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a619c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb619cc, 3 },
    { 0x3e1219cd, 3 },
    { 0xb57a19cd, 3 },
    { 0x625219ce, 3 },
    { 0x3dc219cc, 3 },
    { 0xb3c219d1, 3 },
    { 0x982a19ce, 5 },
    { 0xd6ea19ce, 3 },
    { 0xb08a19ce, 3 },
    { 0x585219d5, 4 },
    { 0xd27219d6, 3 },
    { 0, 0 },
    { 0x11d219d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e19db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88a19de, 4 },
    { 0x9f7619de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c219e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0219e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0e19ea, 3 },
    { 0x1c2e19eb, 5 },
    { 0, 0 },
    { 0xa15619ed, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb219f0, 4 },
    { 0xe10619f0, 3 },
    { 0x119219f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e219f7, 3 },
    { 0, 0 },
    { 0xa3a19f9, 3 },
    { 0, 0 },
    { 0x6c3e19fb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa61a03, 4 },
    { 0x77821a03, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x98561a07, 3 },
    { 0x74261a07, 3 },
    { 0xef2a1a07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e1a0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa121a13, 3 },
    { 0x8561a13, 3 },
    { 0xdbfe1a15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9a1a19, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8321a1f, 3 },
    { 0xf9be1a1f, 4 },
    { 0x484e1a21, 3 },
    { 0x37061a22, 3 },
    { 0x9dae1a23, 3 },
    { 0, 0 },
    { 0x2621a25, 3 },
    { 0x2c21a26, 3 },
    { 0xce1e1a26, 4 },
    { 0x98ba1a26, 3 },
    { 0x549e1a29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba61a2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224e1a31, 3 },
    { 0x3b1a1a32, 3 },
    { 0x5fe61a33, 3 },
    { 0x511e1a33, 3 },
    { 0xa31a1a35, 3 },
    { 0x29221a36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e61a3a, 3 },
    { 0, 0 },
    { 0x492a1a3c, 3 },
    { 0x33de1a3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c61a41, 3 },
    { 0, 0 },
    { 0xa0121a43, 3 },
    { 0, 0 },
    { 0x399e1a45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea121a57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ee1a5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69a1a61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f61a66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37521a6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef521a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e1a73, 4 },
    { 0x6d961a74, 3 },
    { 0x80321a73, 3 },
    { 0x895e1a76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb21a79, 3 },
    { 0xe45e1a79, 3 },
    { 0, 0 },
    { 0xf7ba1a7c, 3 },
    { 0, 0 },
    { 0xd2161a7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aa1a81, 3 },
    { 0, 0 },
    { 0x1b61a83, 3 },
    { 0x71e1a83, 3 },
    { 0xe3ba1a85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3a1a8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e1a91, 3 },
    { 0x562e1a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0e1a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821a9c, 3 },
    { 0, 0 },
    { 0xe41a1a9e, 3 },
    { 0x33be1a9e, 3 },
    { 0xcb661aa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac21aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842a1aab, 3 },
    { 0xfed21aab, 3 },
    { 0, 0 },
    { 0x354e1aae, 3 },
    { 0, 0 },
    { 0xc97a1ab0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a1ab4, 3 },
    { 0x75921ab4, 3 },
    { 0x64e1ab6, 3 },
    { 0xf7521ab5, 3 },
    { 0, 0 },
    { 0x80da1ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461e1ac0, 3 },
    { 0x83b61ac0, 4 },
    { 0x13c21ac1, 3 },
    { 0xcb0a1ac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fa1ac6, 3 },
    { 0xffce1ac6, 5 },
    { 0xb2961ac8, 3 },
    { 0x8bd21ac7, 3 },
    { 0xe9961aca, 4 },
    { 0x3fb21acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561ace, 3 },
    { 0x68a61ace, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f61ad5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3a1ad9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a1ade, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691e1ae2, 4 },
    { 0, 0 },
    { 0x6c561ae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a1ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb61aef, 3 },
    { 0x60ea1af0, 3 },
    { 0xa02e1af1, 4 },
    { 0, 0 },
    { 0xf61a1af3, 3 },
    { 0xa7e61af3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c61af7, 3 },
    { 0x66ae1af8, 4 },
    { 0xc6a21af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6a1afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be21b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee921b04, 3 },
    { 0xb3ce1b04, 4 },
    { 0x7d661b06, 3 },
    { 0xf5c61b07, 4 },
    { 0, 0 },
    { 0x695a1b09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ba1b0c, 4 },
    { 0x4c8a1b0d, 3 },
    { 0xcebe1b0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55061b13, 4 },
    { 0x7a221b13, 3 },
    { 0x42261b15, 3 },
    { 0xa3161b15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2621b19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b61b20, 4 },
    { 0xd1761b21, 3 },
    { 0xf21b22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9e1b25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aa1b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f21b31, 3 },
    { 0x149a1b32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3921b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda461b38, 3 },
    { 0xe3761b38, 3 },
    { 0x105e1b3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf321b3d, 3 },
    { 0xaa461b3d, 3 },
    { 0x69061b3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7a1b42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7e1b45, 3 },
    { 0x929e1b45, 4 },
    { 0xc6261b47, 3 },
    { 0, 0 },
    { 0x5a2a1b49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46221b4c, 3 },
    { 0xc5ce1b4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8a1b50, 4 },
    { 0xddca1b50, 3 },
    { 0x4e1b52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a1b5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c761b65, 3 },
    { 0x4a61b66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799e1b70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6e1b73, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2a1b79, 3 },
    { 0xd6e1b79, 4 },
    { 0, 0 },
    { 0xcdc21b7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233e1b83, 3 },
    { 0x36721b84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eea1b8a, 3 },
    { 0, 0 },
    { 0x1e8a1b8c, 4 },
    { 0, 0 },
    { 0x37d61b8e, 3 },
    { 0x26fe1b8f, 3 },
    { 0x947e1b8f, 3 },
    { 0x82ea1b91, 3 },
    { 0x5dfa1b91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ea1b96, 4 },
    { 0x2d3e1b97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ae1b9a, 4 },
    { 0x308a1b9b, 3 },
    { 0x3d721b9a, 3 },
    { 0xa6f61b9d, 3 },
    { 0xe97e1b9a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e1ba8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154a1bac, 3 },
    { 0x84aa1bad, 4 },
    { 0, 0 },
    { 0x14121baf, 3 },
    { 0x11961baf, 3 },
    { 0x863a1baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e1bb4, 4 },
    { 0xb56e1bb5, 3 },
    { 0xdc4e1bb5, 3 },
    { 0xfd0e1bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9621bba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee61bbd, 3 },
    { 0x62a1bbe, 3 },
    { 0, 0 },
    { 0x46d21bc0, 3 },
    { 0, 0 },
    { 0x4eea1bc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a1bca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c21bd5, 3 },
    { 0, 0 },
    { 0xc1621bd7, 4 },
    { 0, 0 },
    { 0xdc861bd9, 3 },
    { 0xb5ee1bda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc61bde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ba1be3, 4 },
    { 0xc3821be4, 3 },
    { 0x3b721be3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe1bea, 4 },
    { 0x98421bea, 3 },
    { 0x877a1bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e761bf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ae1bfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f21c00, 3 },
    { 0x8b661c01, 4 },
    { 0x31ae1c01, 5 },
    { 0x8a0a1c03, 3 },
    { 0x21161c03, 3 },
    { 0xe2261c05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee21c0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51da1c10, 4 },
    { 0xc3fe1c10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a661c18, 3 },
    { 0x6b9e1c19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63461c20, 3 },
    { 0xc36a1c21, 3 },
    { 0, 0 },
    { 0x36fe1c23, 3 },
    { 0, 0 },
    { 0x23761c25, 3 },
    { 0x78121c26, 3 },
    { 0x40821c25, 3 },
    { 0, 0 },
    { 0xf3821c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28421c2c, 4 },
    { 0x91fa1c2d, 4 },
    { 0xf7e1c2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f61c33, 3 },
    { 0xcde1c34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd61c3d, 4 },
    { 0xb9ea1c3e, 3 },
    { 0, 0 },
    { 0x7c4a1c40, 4 },
    { 0xb43e1c40, 3 },
    { 0xe5b21c41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a021c46, 5 },
    { 0x2c261c46, 3 },
    { 0x157a1c47, 3 },
    { 0xef261c49, 3 },
    { 0, 0 },
    { 0x19de1c4b, 3 },
    { 0, 0 },
    { 0xfe461c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2621c50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a1c57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe061c65, 3 },
    { 0xb8a61c66, 3 },
    { 0x4dae1c67, 4 },
    { 0, 0 },
    { 0xfe3a1c69, 3 },
    { 0xb9ba1c6a, 4 },
    { 0, 0 },
    { 0xb6e21c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde821c71, 3 },
    { 0xe2ee1c71, 3 },
    { 0, 0 },
    { 0x4a4a1c74, 4 },
    { 0xa7fa1c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39aa1c7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de1c80, 3 },
    { 0xdbca1c81, 3 },
    { 0x546a1c82, 5 },
    { 0x18861c83, 3 },
    { 0xc3a1c82, 3 },
    { 0x4c021c83, 3 },
    { 0xa6ce1c86, 3 },
    { 0xf6ea1c86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1c8a, 4 },
    { 0xa80a1c8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a161c8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f521c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb361c97, 4 },
    { 0x79261c97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83661c9b, 3 },
    { 0xa75a1c9c, 3 },
    { 0xdbba1c9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317e1ca3, 3 },
    { 0, 0 },
    { 0x2d161ca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a21cac, 3 },
    { 0, 0 },
    { 0x2a0a1cae, 4 },
    { 0xc7121caf, 3 },
    { 0, 0 },
    { 0xd1ae1cb1, 3 },
    { 0x85a21cb2, 4 },
    { 0xc6621cb1, 4 },
    { 0x506a1cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f721cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea1cc3, 4 },
    { 0xae361cc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93e1cc9, 4 },
    { 0xdd7a1cca, 3 },
    { 0x1e9e1cca, 3 },
    { 0xbf4a1ccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57e1cd1, 3 },
    { 0xc41e1cd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe1cd7, 3 },
    { 0xb0a1cd8, 3 },
    { 0x152e1cd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x88de1cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d421ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426e1ce8, 3 },
    { 0x204a1ce8, 4 },
    { 0xd4861ce8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d21cf1, 3 },
    { 0x1dca1cf2, 3 },
    { 0x607e1cf2, 3 },
    { 0x90d21cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d661cf9, 3 },
    { 0xffa21cfa, 4 },
    { 0x77661cf9, 3 },
    { 0xd77e1cfb, 3 },
    { 0, 0 },
    { 0xf7ea1cfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf61d02, 4 },
    { 0xcdaa1d03, 3 },
    { 0x11661d04, 4 },
    { 0x8e2e1d05, 4 },
    { 0x69b21d06, 3 },
    { 0xa7921d04, 4 },
    { 0xd23a1d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f661d0c, 3 },
    { 0, 0 },
    { 0xf2061d0e, 5 },
    { 0x21161d0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0a1d16, 3 },
    { 0x685e1d17, 3 },
    { 0x2d061d16, 3 },
    { 0x37c21d19, 4 },
    { 0xed021d17, 3 },
    { 0x2ed21d1b, 3 },
    { 0, 0 },
    { 0x97d61d1d, 4 },
    { 0, 0 },
    { 0xae061d1f, 4 },
    { 0x22d21d20, 3 },
    { 0, 0 },
    { 0xe7fa1d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fe1d26, 3 },
    { 0xec5a1d26, 3 },
    { 0xf99e1d28, 3 },
    { 0x9cb61d29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8321d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff61d35, 3 },
    { 0xbbca1d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3121d3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39361d47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63221d4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93561d51, 3 },
    { 0xfd21d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef21d55, 3 },
    { 0xe0a1d56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44e1d5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4a1d60, 3 },
    { 0xb7661d61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22461d65, 4 },
    { 0, 0 },
    { 0x363a1d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe1d6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b61d70, 3 },
    { 0xe7061d71, 3 },
    { 0xa2c61d72, 4 },
    { 0, 0 },
    { 0xed2a1d74, 3 },
    { 0, 0 },
    { 0x2cae1d76, 3 },
    { 0xb0b61d77, 3 },
    { 0x57421d78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2e1d7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e21d80, 3 },
    { 0xeede1d81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67e1d8a, 3 },
    { 0x7e3e1d8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2e1d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ea1d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b21d9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b361d9e, 3 },
    { 0x7f61d9f, 3 },
    { 0xe1021d9e, 3 },
    { 0x51921da1, 3 },
    { 0x49e61da2, 3 },
    { 0xc6d61da3, 3 },
    { 0x9dce1da4, 3 },
    { 0x2f4a1da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a21dae, 3 },
    { 0x5caa1daf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47e1db2, 3 },
    { 0x57be1db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8e1db7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0a1dbe, 3 },
    { 0, 0 },
    { 0x3e3e1dc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e21dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60561dca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737e1dcf, 3 },
    { 0xb2f61dcf, 3 },
    { 0x67921dd1, 4 },
    { 0x29121dd2, 4 },
    { 0x6dda1dd2, 3 },
    { 0x74961dd0, 3 },
    { 0x2e61dd5, 3 },
    { 0xd9ba1dd6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea1dde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc61de5, 3 },
    { 0x5d321de6, 3 },
    { 0x19621de6, 3 },
    { 0x9e4e1de6, 3 },
    { 0x267e1de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbe1dec, 3 },
    { 0xb1621ded, 3 },
    { 0x7ab61dec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9361df3, 3 },
    { 0x4da21df4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3a1dfa, 4 },
    { 0x69121dfb, 3 },
    { 0x62e21dfc, 3 },
    { 0, 0 },
    { 0xbf4e1dfe, 3 },
    { 0, 0 },
    { 0x66861e00, 4 },
    { 0x2d661e00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a61e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbee1e0b, 3 },
    { 0x2eb21e0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ce1e0f, 4 },
    { 0, 0 },
    { 0x89c61e11, 3 },
    { 0xca5e1e12, 3 },
    { 0, 0 },
    { 0xe0861e14, 4 },
    { 0, 0 },
    { 0x95da1e16, 3 },
    { 0x44121e17, 3 },
    { 0x62621e18, 3 },
    { 0x38821e19, 3 },
    { 0xd8b21e1a, 3 },
    { 0xfe0a1e1b, 3 },
    { 0, 0 },
    { 0xb5521e1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3421e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2461e25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f721e2a, 3 },
    { 0, 0 },
    { 0xf3aa1e2c, 3 },
    { 0xeae61e2d, 3 },
    { 0, 0 },
    { 0xe6921e2f, 3 },
    { 0xed921e30, 3 },
    { 0xe3021e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48161e3c, 3 },
    { 0x511e1e3d, 4 },
    { 0x9ac61e3d, 3 },
    { 0xf8ae1e3f, 3 },
    { 0, 0 },
    { 0xaf9e1e41, 3 },
    { 0xaeb21e42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe1e4a, 4 },
    { 0, 0 },
    { 0x8b161e4c, 4 },
    { 0xfd4a1e4d, 4 },
    { 0xc7aa1e4e, 3 },
    { 0x9f7a1e4f, 3 },
    { 0x2e561e4d, 3 },
    { 0x4a4a1e50, 3 },
    { 0x367a1e52, 3 },
    { 0xb7e61e52, 5 },
    { 0x12261e4c, 3 },
    { 0xb661e55, 3 },
    { 0x87261e56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8e1e60, 3 },
    { 0, 0 },
    { 0xf6f21e62, 4 },
    { 0xf12a1e62, 3 },
    { 0x99be1e64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494e1e6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea1e70, 3 },
    { 0, 0 },
    { 0x2d721e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19de1e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1321e7c, 3 },
    { 0x2fd21e7d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e21e80, 3 },
    { 0xc361e81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d161e86, 3 },
    { 0x9b3e1e87, 3 },
    { 0xd4e61e88, 3 },
    { 0, 0 },
    { 0x19d61e8a, 3 },
    { 0xdf061e8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437e1e8f, 5 },
    { 0x31161e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0661e93, 3 },
    { 0xea8a1e93, 3 },
    { 0x2d721e95, 3 },
    { 0, 0 },
    { 0x7fde1e97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ba1e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c21e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5de1ea1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11421ea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a321ea9, 3 },
    { 0x815a1eaa, 3 },
    { 0x7821eab, 3 },
    { 0, 0 },
    { 0x1fea1ead, 4 },
    { 0, 0 },
    { 0xafea1eaf, 3 },
    { 0x1da61eb0, 4 },
    { 0x11821eaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e1eba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa21ebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a1ec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e61ec5, 3 },
    { 0xfc261ec5, 4 },
    { 0x2f961ec7, 3 },
    { 0x509e1ec8, 3 },
    { 0x73721ec8, 3 },
    { 0xaa9a1ec8, 3 },
    { 0x7e221ecb, 3 },
    { 0xed921ecb, 5 },
    { 0x5d361ec6, 3 },
    { 0, 0 },
    { 0xe7ee1ecf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e1ed5, 3 },
    { 0xe3621ed5, 3 },
    { 0x79961ed7, 4 },
    { 0xdb7e1ed8, 4 },
    { 0xc6e21ed9, 3 },
    { 0xa4b61ed9, 3 },
    { 0x3b1a1edb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e1edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae761ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fda1ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a61ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7a1eec, 3 },
    { 0, 0 },
    { 0x69521eee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c61ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a1efa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e1f01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x110a1f04, 4 },
    { 0x26561f05, 4 },
    { 0xd74a1f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf61f09, 3 },
    { 0xdade1f0a, 3 },
    { 0, 0 },
    { 0xf6fa1f0c, 4 },
    { 0xe6ce1f0d, 3 },
    { 0x5f461f0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf721f14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665e1f1b, 3 },
    { 0xc1261f1c, 3 },
    { 0x5cf61f1d, 3 },
    { 0x21761f1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec21f27, 3 },
    { 0, 0 },
    { 0x438e1f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84361f2d, 4 },
    { 0x82361f2e, 3 },
    { 0x7ba61f2e, 3 },
    { 0, 0 },
    { 0x41561f31, 3 },
    { 0, 0 },
    { 0x72d61f33, 3 },
    { 0x23461f33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c21f37, 3 },
    { 0x8a261f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e1f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9461f3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea721f44, 3 },
    { 0, 0 },
    { 0xf5821f46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9221f5d, 3 },
    { 0, 0 },
    { 0x32e61f5f, 4 },
    { 0x5d3e1f60, 3 },
    { 0x45961f61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c261f64, 4 },
    { 0x7c9e1f65, 3 },
    { 0x53261f65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93421f69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a1f6e, 3 },
    { 0xce5e1f6f, 3 },
    { 0, 0 },
    { 0x60aa1f71, 4 },
    { 0xa7a1f72, 3 },
    { 0x6fa61f73, 3 },
    { 0xf9561f73, 3 },
    { 0xf37a1f75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaa1f79, 3 },
    { 0x18f21f7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b21f7d, 3 },
    { 0xd5a1f7d, 3 },
    { 0, 0 },
    { 0xff0e1f80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17761f85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2e1f88, 3 },
    { 0xa9ca1f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde1f8c, 4 },
    { 0xa9521f8c, 4 },
    { 0x80fe1f8e, 3 },
    { 0, 0 },
    { 0xbf061f90, 3 },
    { 0x35821f91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bce1f94, 3 },
    { 0, 0 },
    { 0x21ba1f96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c61f99, 4 },
    { 0, 0 },
    { 0xe65e1f9b, 3 },
    { 0x5daa1f9c, 3 },
    { 0, 0 },
    { 0xb2261f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319e1fa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3561fab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47261fae, 3 },
    { 0x5a1faf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108e1fb3, 3 },
    { 0xeb721fb3, 3 },
    { 0, 0 },
    { 0x463e1fb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f361fbd, 3 },
    { 0x653e1fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bce1fc3, 3 },
    { 0, 0 },
    { 0x1ec61fc5, 3 },
    { 0x340e1fc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622a1fcd, 3 },
    { 0xcb9a1fcd, 4 },
    { 0x14621fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1121fd2, 3 },
    { 0, 0 },
    { 0xefc21fd4, 4 },
    { 0x8ad61fd5, 3 },
    { 0, 0 },
    { 0x62ce1fd7, 4 },
    { 0, 0 },
    { 0xd7aa1fd9, 3 },
    { 0x526e1fda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aa1fe2, 4 },
    { 0xb5a1fe3, 3 },
    { 0x42d21fe2, 3 },
    { 0x62161fe2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a1fe9, 3 },
    { 0xf6c61fea, 3 },
    { 0xf04e1feb, 3 },
    { 0x8f661fec, 5 },
    { 0, 0 },
    { 0x595a1fee, 3 },
    { 0x95ba1fef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fe1ffa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a21ffd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e2003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aae200a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6816200e, 4 },
    { 0x1e0a200f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a2014, 3 },
    { 0x1ba62015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b122018, 3 },
    { 0, 0 },
    { 0x4a72201a, 3 },
    { 0, 0 },
    { 0x9e0a201c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e22023, 3 },
    { 0x4a62024, 3 },
    { 0x91ba2024, 4 },
    { 0x4b962026, 3 },
    { 0x55ae2027, 3 },
    { 0x703e2027, 3 },
    { 0x3d122027, 3 },
    { 0x39c2202a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a2031, 4 },
    { 0xdb262032, 3 },
    { 0xb53e2033, 4 },
    { 0xe47a2033, 3 },
    { 0, 0 },
    { 0xc1b62036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fa203f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91022043, 4 },
    { 0x37162044, 4 },
    { 0x11862045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea204b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a204e, 3 },
    { 0xcb56204f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b222052, 4 },
    { 0, 0 },
    { 0xb1fa2054, 3 },
    { 0x983e2055, 3 },
    { 0xb8e22054, 5 },
    { 0, 0 },
    { 0x33222058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e2205c, 3 },
    { 0, 0 },
    { 0x48a2205e, 4 },
    { 0xabea205f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ba2062, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98de2066, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a22069, 3 },
    { 0xd99e206a, 3 },
    { 0x47ba206b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fe206f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49362074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7e2077, 3 },
    { 0xb1862078, 3 },
    { 0, 0 },
    { 0x600a207a, 4 },
    { 0x8fc6207b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c462081, 3 },
    { 0, 0 },
    { 0x904a2083, 4 },
    { 0, 0 },
    { 0x9cfe2085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfe2088, 3 },
    { 0xbfe62089, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa208c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x447e208f, 4 },
    { 0x42fe2090, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf562097, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd072209a, 3 },
    { 0x52e209b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56209e, 4 },
    { 0xb596209f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981e20a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72620a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca20a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc220ad, 3 },
    { 0, 0 },
    { 0x166620af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe20b4, 4 },
    { 0x156620b4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296220b9, 4 },
    { 0xb33220ba, 4 },
    { 0x634e20bb, 3 },
    { 0x6a8220b9, 3 },
    { 0, 0 },
    { 0xd1fa20be, 3 },
    { 0x1ca220bf, 3 },
    { 0x1f5e20be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873a20c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1620cd, 3 },
    { 0x322a20ce, 3 },
    { 0, 0 },
    { 0x966a20d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5a20d4, 3 },
    { 0, 0 },
    { 0x5e3a20d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a620dc, 3 },
    { 0xddca20dd, 4 },
    { 0x9c6220dd, 4 },
    { 0xf67e20dc, 4 },
    { 0xe2f220dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe220e6, 3 },
    { 0xfabe20e6, 3 },
    { 0x933e20e7, 4 },
    { 0x931a20e9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4220ec, 3 },
    { 0x540a20ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aee20f2, 4 },
    { 0x9a1e20f3, 4 },
    { 0, 0 },
    { 0xbd2220f5, 3 },
    { 0x73620f6, 3 },
    { 0, 0 },
    { 0x808a20f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248220fc, 3 },
    { 0x7f1620fc, 3 },
    { 0, 0 },
    { 0x1a8a20ff, 4 },
    { 0xc3e2100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ba2103, 3 },
    { 0xe3a2104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73462107, 4 },
    { 0x30e22108, 3 },
    { 0xf662108, 3 },
    { 0x2a2210a, 3 },
    { 0x48da2109, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed36210e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7362119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef72211c, 3 },
    { 0x6d86211d, 3 },
    { 0x252e211e, 3 },
    { 0xd69e211f, 3 },
    { 0xf7a22120, 3 },
    { 0x6f26211f, 4 },
    { 0xc0822121, 4 },
    { 0x6c1e2123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef22127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e212e, 3 },
    { 0xed76212e, 3 },
    { 0xe1722130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7562133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4a213a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e213f, 4 },
    { 0xd90e2140, 3 },
    { 0x772e2141, 4 },
    { 0x4c0a2142, 3 },
    { 0x98c62143, 4 },
    { 0xadd22142, 4 },
    { 0x56522145, 3 },
    { 0x18822146, 4 },
    { 0x6f422147, 3 },
    { 0x5d62145, 4 },
    { 0xbd8a2142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a214c, 4 },
    { 0x2b8e214d, 3 },
    { 0, 0 },
    { 0x97de214f, 3 },
    { 0x2d962150, 3 },
    { 0x588a2151, 3 },
    { 0x318e214f, 3 },
    { 0xb0d62153, 3 },
    { 0xaac62153, 3 },
    { 0x1faa2154, 3 },
    { 0x6482214f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c62159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808e215d, 5 },
    { 0xc87a215d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc622169, 3 },
    { 0xef1a216a, 4 },
    { 0xa1c2216b, 4 },
    { 0x8a1e2169, 3 },
    { 0xde422169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48a2171, 3 },
    { 0x242a2172, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb462176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf72217c, 3 },
    { 0xae2a217d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6a2186, 3 },
    { 0, 0 },
    { 0x80ea2188, 4 },
    { 0xc00a2188, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb846218e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00e2191, 3 },
    { 0, 0 },
    { 0xea222193, 3 },
    { 0x509e2194, 5 },
    { 0x8f7a2194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87e2198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a6219d, 4 },
    { 0x2f56219e, 3 },
    { 0x2872219f, 5 },
    { 0xbcfa219f, 3 },
    { 0x450621a1, 4 },
    { 0x648a21a2, 3 },
    { 0xbbca21a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec621a9, 3 },
    { 0, 0 },
    { 0x51b621ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c621af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58e21b9, 4 },
    { 0x8a4e21b9, 5 },
    { 0x407621bb, 3 },
    { 0x986e21bc, 4 },
    { 0x3bd221ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef221c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0621c7, 4 },
    { 0xfd6a21c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d621ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1e21d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a21d4, 5 },
    { 0xba8a21d4, 4 },
    { 0, 0 },
    { 0x27621d7, 3 },
    { 0x6b3621d8, 3 },
    { 0x309e21d8, 4 },
    { 0xf47a21da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fca21e8, 5 },
    { 0, 0 },
    { 0x4a21ea, 4 },
    { 0x5b7621eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59621f3, 3 },
    { 0, 0 },
    { 0xb72621f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b221f8, 3 },
    { 0, 0 },
    { 0xfd3221fa, 4 },
    { 0x6e7621fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a621fe, 3 },
    { 0x30a621fe, 3 },
    { 0xb7122200, 3 },
    { 0xfb221ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae2206, 3 },
    { 0x61222207, 3 },
    { 0xe8b22207, 3 },
    { 0x200a2209, 3 },
    { 0x225a220a, 5 },
    { 0x67722208, 3 },
    { 0x74e6220a, 4 },
    { 0x495a220d, 4 },
    { 0xed0e220e, 4 },
    { 0x1aea2206, 3 },
    { 0x677e2210, 3 },
    { 0x75062211, 3 },
    { 0, 0 },
    { 0xce0a2213, 3 },
    { 0, 0 },
    { 0xc7262215, 4 },
    { 0x27f22215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be221a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f2221d, 4 },
    { 0xae221d, 3 },
    { 0xe36221f, 3 },
    { 0xa25a221f, 3 },
    { 0x418a221e, 3 },
    { 0x69da221f, 3 },
    { 0x2d8a2223, 3 },
    { 0x34d62224, 3 },
    { 0x2ffa221d, 3 },
    { 0, 0 },
    { 0x61822227, 3 },
    { 0x684e2228, 4 },
    { 0, 0 },
    { 0x820e222a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b6222d, 3 },
    { 0xeda2222d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a2232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce2237, 3 },
    { 0, 0 },
    { 0x1b922239, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2a223e, 3 },
    { 0xa76223f, 4 },
    { 0xab6e2240, 3 },
    { 0x7d562240, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3322246, 3 },
    { 0x85d62247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00e224b, 3 },
    { 0xc6e2224b, 4 },
    { 0x2966224d, 3 },
    { 0xbeca224e, 3 },
    { 0xb376224f, 3 },
    { 0xc8be2250, 4 },
    { 0x50c22250, 4 },
    { 0, 0 },
    { 0x79f22253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c2225a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca2261, 4 },
    { 0, 0 },
    { 0x89422263, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63562266, 3 },
    { 0x4a662267, 3 },
    { 0x86e2268, 3 },
    { 0xc1ae2269, 3 },
    { 0, 0 },
    { 0x1376226b, 3 },
    { 0, 0 },
    { 0x66c6226d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee2270, 3 },
    { 0, 0 },
    { 0x17ea2272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffe2275, 3 },
    { 0x533a2276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ca227e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4422283, 4 },
    { 0, 0 },
    { 0xaa162285, 3 },
    { 0xcea22286, 3 },
    { 0x80be2286, 3 },
    { 0xff22288, 3 },
    { 0x24a2289, 4 },
    { 0, 0 },
    { 0xcfea228b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8322290, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df62293, 3 },
    { 0, 0 },
    { 0x18a2295, 4 },
    { 0, 0 },
    { 0xfe1a2297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e229f, 3 },
    { 0x8f9e22a0, 3 },
    { 0xd6d2229f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x873622a4, 4 },
    { 0x805622a5, 3 },
    { 0xdfce22a6, 4 },
    { 0x9b7a22a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d622aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997e22af, 3 },
    { 0xf56a22af, 4 },
    { 0x600a22b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2e22b4, 4 },
    { 0x3c6622b4, 3 },
    { 0xc1c622b6, 4 },
    { 0x2ea222b7, 3 },
    { 0, 0 },
    { 0x519a22b9, 3 },
    { 0x71ce22ba, 3 },
    { 0, 0 },
    { 0xb42a22bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5222bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x123a22c2, 4 },
    { 0x8bb222c3, 3 },
    { 0, 0 },
    { 0xaee222c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71e22cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffa22d1, 3 },
    { 0, 0 },
    { 0xbda222d3, 3 },
    { 0x96d622d4, 4 },
    { 0, 0 },
    { 0xc83e22d6, 3 },
    { 0x24222d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fee22da, 3 },
    { 0x5f1a22da, 3 },
    { 0x2ace22dc, 3 },
    { 0, 0 },
    { 0x59ee22de, 3 },
    { 0x424a22df, 3 },
    { 0x22622de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b622e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad622eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52622f1, 3 },
    { 0xdce222f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128222f7, 4 },
    { 0x3a3a22f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee22fd, 3 },
    { 0, 0 },
    { 0x3ee622ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18922303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda962307, 3 },
    { 0x95b62308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880a230c, 3 },
    { 0x9376230c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a62310, 3 },
    { 0x28aa2310, 4 },
    { 0x75ea2312, 4 },
    { 0x26c22313, 4 },
    { 0x540e2314, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc62319, 3 },
    { 0xcd4e231a, 3 },
    { 0xa2f6231a, 3 },
    { 0x10ce231c, 3 },
    { 0x4aa22319, 3 },
    { 0xfd722319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69062322, 3 },
    { 0x347a2323, 3 },
    { 0x1c3a2324, 4 },
    { 0x914a2323, 3 },
    { 0xe7622323, 3 },
    { 0x77362327, 4 },
    { 0x16ba2322, 3 },
    { 0xc1ea2324, 3 },
    { 0xdbda2325, 3 },
    { 0, 0 },
    { 0x678e232c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e62330, 4 },
    { 0x7a1e2331, 3 },
    { 0x46ea2332, 3 },
    { 0, 0 },
    { 0x9e722334, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d22337, 3 },
    { 0x12262337, 4 },
    { 0x3b7e2339, 3 },
    { 0xdefe233a, 4 },
    { 0xcf56233b, 3 },
    { 0, 0 },
    { 0xbda6233d, 4 },
    { 0, 0 },
    { 0xac8e233f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6e2342, 3 },
    { 0, 0 },
    { 0x7b562344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96562349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd6234f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10962357, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafa2361, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae2366, 3 },
    { 0, 0 },
    { 0xdd562368, 4 },
    { 0, 0 },
    { 0x6242236a, 4 },
    { 0x14f2236b, 3 },
    { 0x4402236b, 3 },
    { 0x6572236c, 3 },
    { 0x864e236e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ade2379, 3 },
    { 0xe8f6237a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20162381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6a2384, 4 },
    { 0x717a2385, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8e2388, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ba238e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a22398, 3 },
    { 0xf2e2399, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3a239d, 4 },
    { 0x36da239e, 3 },
    { 0xd612239e, 4 },
    { 0xc91e239e, 3 },
    { 0xe36e239e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x331623a4, 3 },
    { 0x25a623a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a23a8, 3 },
    { 0x416223a9, 3 },
    { 0x5a5e23aa, 3 },
    { 0xf06a23ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686e23b0, 3 },
    { 0xde4623b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5623b4, 3 },
    { 0x472e23b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ba23b8, 3 },
    { 0xa2d623b9, 3 },
    { 0xf5ea23ba, 4 },
    { 0, 0 },
    { 0x360223bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206223c1, 3 },
    { 0, 0 },
    { 0x4e4223c3, 4 },
    { 0x416a23c4, 3 },
    { 0, 0 },
    { 0x3c4623c6, 3 },
    { 0x556223c7, 3 },
    { 0, 0 },
    { 0x8db623c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9e23cc, 3 },
    { 0x5cf623cc, 4 },
    { 0xc2fe23ce, 4 },
    { 0, 0 },
    { 0xd1c623d0, 3 },
    { 0x562223d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711623d6, 3 },
    { 0x466a23d7, 4 },
    { 0, 0 },
    { 0xd9b623d9, 3 },
    { 0x19a623da, 4 },
    { 0xd8be23db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a23df, 3 },
    { 0, 0 },
    { 0x891a23e1, 4 },
    { 0, 0 },
    { 0x9b1623e3, 3 },
    { 0xce2623e3, 3 },
    { 0x727e23e4, 3 },
    { 0, 0 },
    { 0xeb7623e7, 3 },
    { 0xd5fe23e8, 3 },
    { 0x5a2623e7, 3 },
    { 0x7fa223ea, 3 },
    { 0x76be23ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7e23f2, 4 },
    { 0x4be623f2, 3 },
    { 0, 0 },
    { 0x48ea23f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea623f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7623ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8a2405, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d62408, 3 },
    { 0xa70e2409, 3 },
    { 0x680a240a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ba240e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x681e2411, 3 },
    { 0, 0 },
    { 0x9d3e2413, 3 },
    { 0x998e2413, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a2417, 3 },
    { 0xbf222418, 3 },
    { 0xbbfa2417, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676e241d, 3 },
    { 0x74f6241e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82522421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a2425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d62429, 3 },
    { 0xa172242a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa2430, 4 },
    { 0x787a2430, 4 },
    { 0, 0 },
    { 0xb52a2433, 3 },
    { 0, 0 },
    { 0x6d3e2435, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0e2438, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c462440, 4 },
    { 0xd5662441, 3 },
    { 0xb0522442, 5 },
    { 0, 0 },
    { 0x5ea62444, 3 },
    { 0x17e62444, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346244a, 3 },
    { 0x5022244b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfe2450, 3 },
    { 0xd2c22451, 3 },
    { 0x8f762452, 3 },
    { 0xf4a2453, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb862458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6462460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca2463, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16246a, 4 },
    { 0x4b2a246b, 3 },
    { 0, 0 },
    { 0x763e246d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ee2471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db22476, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa306247c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a247f, 3 },
    { 0xf57a2480, 4 },
    { 0, 0 },
    { 0xc0522482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c62487, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc056248d, 4 },
    { 0x2726248e, 3 },
    { 0x51ba248f, 3 },
    { 0x11ca248f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a2494, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80e2498, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6249b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be24a1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6224a6, 3 },
    { 0, 0 },
    { 0x43aa24a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83224ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1624b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be24b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4224bc, 4 },
    { 0x588624bd, 3 },
    { 0x3c3a24be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e24c3, 3 },
    { 0x84aa24c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fa24ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e24cf, 3 },
    { 0x39d224d0, 3 },
    { 0, 0 },
    { 0xf54a24d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a24d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78da24da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01224de, 3 },
    { 0x96f624de, 3 },
    { 0, 0 },
    { 0x207e24e1, 3 },
    { 0x1b6a24e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a624ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff224f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16224f4, 4 },
    { 0xf94e24f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e24fc, 5 },
    { 0x1cb624fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x873a2500, 3 },
    { 0x42422500, 3 },
    { 0xb2e22500, 3 },
    { 0, 0 },
    { 0xe09e2504, 3 },
    { 0, 0 },
    { 0x6b7e2506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e16250e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xece2511, 4 },
    { 0x9c662512, 3 },
    { 0x5aae2513, 4 },
    { 0, 0 },
    { 0xddc62515, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52322520, 3 },
    { 0xfec62521, 5 },
    { 0x5dd22521, 4 },
    { 0, 0 },
    { 0x7622524, 3 },
    { 0, 0 },
    { 0x6bc62526, 3 },
    { 0x71022527, 3 },
    { 0x479e2526, 5 },
    { 0, 0 },
    { 0x83f2252a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde252e, 3 },
    { 0xc1fa252f, 3 },
    { 0, 0 },
    { 0x13ca2531, 3 },
    { 0xfb062532, 3 },
    { 0xe3ba2532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca2538, 3 },
    { 0, 0 },
    { 0xacea253a, 4 },
    { 0xf0b2253b, 3 },
    { 0xd5d2253c, 3 },
    { 0x2eb6253d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c22540, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fe254b, 3 },
    { 0xee62254c, 4 },
    { 0xbd2a254d, 3 },
    { 0x108a254b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d22554, 3 },
    { 0x692a2555, 3 },
    { 0x5f6a2555, 3 },
    { 0xc2562557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6255a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1e255e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39722564, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6da2567, 3 },
    { 0xb3b62568, 3 },
    { 0xa6f22568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442256e, 3 },
    { 0xa2fe256e, 4 },
    { 0x9bc6256e, 4 },
    { 0, 0 },
    { 0x6a962572, 4 },
    { 0x59762572, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x561a2576, 3 },
    { 0x940a2576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb56257a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6257f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22a2582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb62585, 3 },
    { 0, 0 },
    { 0x4ec22587, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86a258e, 3 },
    { 0xd5a258f, 4 },
    { 0x2eae2590, 3 },
    { 0x4126258f, 3 },
    { 0xfe062592, 4 },
    { 0x7aa62592, 3 },
    { 0xbdf2258f, 4 },
    { 0xedfe258f, 3 },
    { 0, 0 },
    { 0x87fe2597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbe25a1, 3 },
    { 0xc98225a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a25a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97625af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fea25b3, 3 },
    { 0, 0 },
    { 0xcf9625b5, 3 },
    { 0xfe4225b5, 3 },
    { 0, 0 },
    { 0xd2e225b8, 3 },
    { 0xcd3225b8, 4 },
    { 0, 0 },
    { 0xbc5225bb, 4 },
    { 0xc56e25bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a25c8, 3 },
    { 0x670625c9, 3 },
    { 0x839225ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4e25d1, 3 },
    { 0x14c225d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa25d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84625d9, 3 },
    { 0, 0 },
    { 0x954e25db, 3 },
    { 0xf4ea25db, 4 },
    { 0xcd2625dd, 3 },
    { 0xe4e25dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9e25e2, 3 },
    { 0, 0 },
    { 0xb7ca25e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a25ef, 3 },
    { 0x24ea25f0, 4 },
    { 0xd30225f1, 3 },
    { 0xec7e25f2, 3 },
    { 0x783225f1, 4 },
    { 0xf39225f4, 3 },
    { 0xdea625f2, 4 },
    { 0xf2f625f3, 3 },
    { 0xc57625f7, 3 },
    { 0x5a9a25f8, 3 },
    { 0x26be25f9, 3 },
    { 0xeff225fa, 3 },
    { 0x914e25fb, 3 },
    { 0x89e25fb, 4 },
    { 0x62ba25f3, 3 },
    { 0, 0 },
    { 0x8dae25ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f22603, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7a2606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a2609, 4 },
    { 0x7cfe2609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65be260d, 3 },
    { 0, 0 },
    { 0xf8d2260f, 3 },
    { 0x7986260f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37a2613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892a261c, 3 },
    { 0, 0 },
    { 0xc3be261e, 3 },
    { 0x19b2261f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ba2624, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ea2629, 3 },
    { 0x1aba262a, 3 },
    { 0x435a2629, 3 },
    { 0, 0 },
    { 0x1daa262d, 4 },
    { 0x6e9a262e, 3 },
    { 0xcf76262e, 3 },
    { 0xd9862630, 4 },
    { 0, 0 },
    { 0xf2b22632, 4 },
    { 0x6e762633, 3 },
    { 0, 0 },
    { 0xed622635, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aae2638, 3 },
    { 0, 0 },
    { 0x3c6263a, 3 },
    { 0x45e6263a, 3 },
    { 0x2b5e263c, 3 },
    { 0x7772263d, 3 },
    { 0x303a263e, 3 },
    { 0x620e263f, 3 },
    { 0xaf36263f, 3 },
    { 0xa4462640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca022645, 3 },
    { 0xf6be2646, 4 },
    { 0x8c1e2646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357a264c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2046264f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a022655, 3 },
    { 0x3db22656, 4 },
    { 0xc7622656, 3 },
    { 0x2f822658, 4 },
    { 0x9b922658, 3 },
    { 0x198a2657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b46265d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9a2660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e2664, 3 },
    { 0xa4762664, 3 },
    { 0x5e8a2665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30e266b, 3 },
    { 0x8d46266b, 3 },
    { 0xf196266b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fa2671, 4 },
    { 0x5f7a2672, 3 },
    { 0, 0 },
    { 0xc3862674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4262267a, 3 },
    { 0, 0 },
    { 0x388e267c, 3 },
    { 0xc1aa267d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7de2681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63922689, 3 },
    { 0x4b1e268a, 3 },
    { 0xdc56268b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba66268f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11162697, 4 },
    { 0, 0 },
    { 0x8f2a2699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e269d, 3 },
    { 0, 0 },
    { 0x57e2269f, 3 },
    { 0, 0 },
    { 0xcf3626a1, 3 },
    { 0xd9b626a1, 4 },
    { 0x90e26a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee26a7, 4 },
    { 0, 0 },
    { 0x923226a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ce26b0, 3 },
    { 0x14a226b0, 3 },
    { 0x305626b2, 3 },
    { 0xcb1226b0, 4 },
    { 0x351626b4, 4 },
    { 0x4dfa26b4, 4 },
    { 0x29226b6, 3 },
    { 0xc0e626b7, 3 },
    { 0xd0e626b8, 3 },
    { 0xdcce26b6, 3 },
    { 0x334e26b7, 4 },
    { 0x21626b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168a26c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965226cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baa26d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce26d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fa26db, 3 },
    { 0, 0 },
    { 0x706226dd, 4 },
    { 0xfcda26de, 3 },
    { 0, 0 },
    { 0x3ed226e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ea26e4, 3 },
    { 0x63da26e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1226e8, 3 },
    { 0xc19a26e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x223226ec, 3 },
    { 0, 0 },
    { 0x952e26ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a226f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495e26f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0226fb, 3 },
    { 0, 0 },
    { 0xc81e26fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f22706, 3 },
    { 0x21e62706, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06270c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8a2714, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefa2718, 3 },
    { 0xc84a2719, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f6271e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1e2723, 3 },
    { 0, 0 },
    { 0x550e2725, 3 },
    { 0x52262726, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b6272d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61662731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc2273a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f2273e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d62744, 3 },
    { 0, 0 },
    { 0xd54e2746, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e22749, 5 },
    { 0xf1022749, 4 },
    { 0, 0 },
    { 0xfe8a274c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78a274f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefa275b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3a275e, 3 },
    { 0x9de6275e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e2767, 3 },
    { 0x354e2768, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd2276b, 4 },
    { 0xbfa276c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52522772, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a62777, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34122783, 4 },
    { 0, 0 },
    { 0x264e2785, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21422789, 3 },
    { 0x5db2278a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a522794, 3 },
    { 0x685e2795, 5 },
    { 0x2ece2796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x561e2799, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be6279c, 3 },
    { 0, 0 },
    { 0x490e279e, 3 },
    { 0x3e16279f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69627a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63da27a9, 3 },
    { 0x991627aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x830227ad, 4 },
    { 0x172227ad, 3 },
    { 0x69ea27ad, 3 },
    { 0x26e27af, 3 },
    { 0x707e27ae, 4 },
    { 0xfec227b1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33e27ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8227c1, 3 },
    { 0, 0 },
    { 0x418e27c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e227c8, 4 },
    { 0xb13627c8, 4 },
    { 0xa5ae27c8, 3 },
    { 0x771627ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eee27d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x828627da, 3 },
    { 0, 0 },
    { 0xc94a27dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e27e3, 4 },
    { 0xa4e227e4, 3 },
    { 0x75a227e5, 3 },
    { 0, 0 },
    { 0x1bc227e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9e27eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fe27ee, 3 },
    { 0xfb5227ee, 3 },
    { 0x407a27f0, 3 },
    { 0, 0 },
    { 0x60da27f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa627f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14227fb, 3 },
    { 0, 0 },
    { 0xc8ce27fd, 3 },
    { 0xfc4627fd, 3 },
    { 0xd43a27ff, 4 },
    { 0xee062800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b122807, 4 },
    { 0, 0 },
    { 0x4d062809, 4 },
    { 0x539e280a, 4 },
    { 0x6eaa280a, 4 },
    { 0, 0 },
    { 0xa10a280d, 3 },
    { 0, 0 },
    { 0x82c2280f, 3 },
    { 0, 0 },
    { 0x9b2a2811, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a22815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2562819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb2281c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac6281f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aba2823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7e2826, 3 },
    { 0xe3d62827, 3 },
    { 0xf4ce2828, 3 },
    { 0x70022826, 4 },
    { 0x1022282a, 3 },
    { 0x9e7a282b, 4 },
    { 0xf5ca2828, 3 },
    { 0, 0 },
    { 0x1092282e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5a2833, 3 },
    { 0x4e0e2834, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86122837, 3 },
    { 0x36e22838, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa376283e, 4 },
    { 0x60fa283e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed662847, 4 },
    { 0, 0 },
    { 0xa80e2849, 3 },
    { 0x2d1e2849, 3 },
    { 0x48ee2849, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0e2851, 3 },
    { 0x30f22852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58822856, 3 },
    { 0xf1ae2856, 4 },
    { 0xddce2857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a2860, 3 },
    { 0, 0 },
    { 0x3c522862, 3 },
    { 0x7bca2862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e2867, 3 },
    { 0xa9aa2868, 4 },
    { 0x70b62867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f62871, 4 },
    { 0, 0 },
    { 0x7e2e2873, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e162878, 3 },
    { 0, 0 },
    { 0xd80a287a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72287f, 3 },
    { 0x1a7e2880, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb62884, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b22887, 4 },
    { 0, 0 },
    { 0xc022889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd532288c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a72288f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b462892, 3 },
    { 0x230e2892, 4 },
    { 0xe4a22894, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23a289a, 4 },
    { 0xcd9a289a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ae28a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e28ad, 3 },
    { 0xe5ca28ae, 3 },
    { 0xd93228af, 3 },
    { 0x699628af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4228b5, 3 },
    { 0x1f3e28b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfe28bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60da28c5, 4 },
    { 0xe2aa28c6, 4 },
    { 0x6fe628c6, 3 },
    { 0x9b5228c8, 3 },
    { 0x7c9628c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925e28d0, 3 },
    { 0x804628d1, 3 },
    { 0, 0 },
    { 0xb1a228d3, 4 },
    { 0, 0 },
    { 0x727628d5, 3 },
    { 0, 0 },
    { 0x69ea28d7, 3 },
    { 0x3ade28d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf628db, 3 },
    { 0x5cf228dc, 3 },
    { 0xc2ce28db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc228e0, 3 },
    { 0, 0 },
    { 0xf2aa28e2, 3 },
    { 0x1b3e28e3, 4 },
    { 0xa57e28e2, 3 },
    { 0xb9d228e5, 4 },
    { 0x836a28e6, 3 },
    { 0x311628e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559e28f5, 3 },
    { 0xfc8228f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716628fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85a28ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab862902, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac62905, 4 },
    { 0, 0 },
    { 0xb34a2907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04a290a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab6290f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab62913, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a62917, 3 },
    { 0x54e22918, 3 },
    { 0xb65e2919, 3 },
    { 0, 0 },
    { 0x4d7e291b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62291e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2a2924, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cae2929, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a292e, 3 },
    { 0x8d1a292e, 3 },
    { 0x13fe292e, 4 },
    { 0x54ba2931, 3 },
    { 0x27f62932, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd62935, 5 },
    { 0, 0 },
    { 0xb7c22937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e293b, 4 },
    { 0xa58a293b, 3 },
    { 0x5652293d, 3 },
    { 0x8bc2293e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9862942, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11a2948, 3 },
    { 0, 0 },
    { 0x3472294a, 3 },
    { 0x836a294b, 4 },
    { 0x2932294b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6e2954, 3 },
    { 0x6bd62955, 4 },
    { 0x7b262956, 3 },
    { 0x54562957, 4 },
    { 0, 0 },
    { 0x8c762959, 3 },
    { 0xf676295a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ae295e, 4 },
    { 0x4462295f, 3 },
    { 0x9482295f, 3 },
    { 0x40e295f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b62965, 3 },
    { 0x25ee2965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2296a, 4 },
    { 0, 0 },
    { 0x96f2296c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d62970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17be2973, 4 },
    { 0xe5862974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238a2978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb862980, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57e2985, 5 },
    { 0xe88a2986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6a298d, 3 },
    { 0x5e3a298e, 3 },
    { 0x4cd6298f, 3 },
    { 0x89a62990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722998, 4 },
    { 0x32622998, 3 },
    { 0, 0 },
    { 0xb18e299b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893629a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a29b1, 3 },
    { 0xb1c629b1, 3 },
    { 0x6e0e29b3, 3 },
    { 0x7f229b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aa29b7, 3 },
    { 0x22a29b7, 3 },
    { 0x599629b9, 4 },
    { 0x7a5a29ba, 3 },
    { 0, 0 },
    { 0xff1629bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879a29c0, 3 },
    { 0x1ee229c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77229c4, 3 },
    { 0x294e29c4, 3 },
    { 0, 0 },
    { 0xf07229c7, 3 },
    { 0, 0 },
    { 0x9bd229c9, 3 },
    { 0x3f9229c9, 4 },
    { 0x191a29cb, 3 },
    { 0x4bb629cb, 3 },
    { 0xff8629cd, 3 },
    { 0x52a29cd, 3 },
    { 0x1f9229cf, 3 },
    { 0x69229d0, 3 },
    { 0x59d229cd, 5 },
    { 0x37fa29d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1a29d5, 3 },
    { 0x60c629d6, 4 },
    { 0xec6e29d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf229db, 3 },
    { 0x7b7e29dc, 4 },
    { 0x684229dc, 5 },
    { 0xc8d629de, 3 },
    { 0x520a29de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae29e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f229e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6229ec, 3 },
    { 0x862e29ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0a29f4, 3 },
    { 0x9b4e29f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb229f8, 3 },
    { 0, 0 },
    { 0x564a29fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc629ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277a2a04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2a07, 3 },
    { 0xacf22a08, 3 },
    { 0, 0 },
    { 0x408a2a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5262a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a2a1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8522a1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1922a22, 4 },
    { 0x3ad62a23, 3 },
    { 0x54422a23, 3 },
    { 0, 0 },
    { 0xe2b22a26, 3 },
    { 0x3e8a2a27, 4 },
    { 0, 0 },
    { 0x87ba2a29, 3 },
    { 0x9bee2a2a, 3 },
    { 0x30722a2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ee2a30, 3 },
    { 0, 0 },
    { 0xc99e2a32, 4 },
    { 0xaaee2a32, 3 },
    { 0xb22a2a34, 4 },
    { 0x92622a32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e2a3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfce2a3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ca2a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e2a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f322a48, 3 },
    { 0, 0 },
    { 0x7e922a4a, 3 },
    { 0x287a2a4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ee2a50, 3 },
    { 0xc48e2a51, 3 },
    { 0x3ee2a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7e2a57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610a2a5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70a2a5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x599e2a61, 3 },
    { 0x535a2a61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b62a67, 3 },
    { 0x3c8a2a67, 3 },
    { 0x40e2a69, 3 },
    { 0x15922a6a, 3 },
    { 0x6d3e2a6b, 3 },
    { 0, 0 },
    { 0x28822a6d, 4 },
    { 0, 0 },
    { 0x521e2a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c62a77, 3 },
    { 0x13962a78, 4 },
    { 0x500a2a79, 3 },
    { 0xb31e2a77, 3 },
    { 0x74122a7b, 3 },
    { 0, 0 },
    { 0x3fae2a7d, 3 },
    { 0x58262a7d, 3 },
    { 0x4b7a2a7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317a2a88, 3 },
    { 0x15d22a88, 3 },
    { 0xaf722a88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a62a8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaa2a96, 5 },
    { 0, 0 },
    { 0x23e2a98, 3 },
    { 0xfed22a98, 4 },
    { 0x1f422a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f22a9d, 4 },
    { 0, 0 },
    { 0x4ef22a9f, 3 },
    { 0x9c8a2aa0, 4 },
    { 0xea062aa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4222aa4, 3 },
    { 0x9e162aa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f62aa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2aaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5522ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58822ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda2ab9, 4 },
    { 0x9f0a2aba, 3 },
    { 0x187e2abb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7362abf, 4 },
    { 0x481a2ac0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce62ac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6462acb, 3 },
    { 0, 0 },
    { 0xb60a2acd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f222ad0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2662ae0, 3 },
    { 0x434a2ae0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b22ae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9e2ae9, 3 },
    { 0xaaba2aea, 4 },
    { 0xd1a2aea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e2af0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d722af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e62af8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b462afb, 3 },
    { 0, 0 },
    { 0x44522afd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a2b00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37122b04, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e2b07, 3 },
    { 0x539e2b08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f62b0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba2b11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0362b16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1e2b19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40522b1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4262b25, 3 },
    { 0xddca2b25, 3 },
    { 0xf5fa2b27, 3 },
    { 0x364a2b28, 3 },
    { 0x429a2b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3e2b2c, 4 },
    { 0xa15a2b2c, 4 },
    { 0x76da2b2e, 3 },
    { 0x4e622b2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e2b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1722b3e, 3 },
    { 0xc80a2b3e, 4 },
    { 0xbbee2b40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38162b44, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea2b4a, 4 },
    { 0xcab22b4b, 3 },
    { 0xbf0e2b4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a22b4f, 3 },
    { 0, 0 },
    { 0xc1e2b51, 3 },
    { 0, 0 },
    { 0x726a2b53, 3 },
    { 0xc47e2b53, 3 },
    { 0x16022b55, 3 },
    { 0, 0 },
    { 0x74d62b57, 4 },
    { 0x4c9a2b58, 4 },
    { 0x84662b57, 3 },
    { 0xea7a2b5a, 3 },
    { 0x93222b5b, 3 },
    { 0xccca2b5c, 3 },
    { 0xfa9e2b5a, 3 },
    { 0xd722b5e, 3 },
    { 0x81a62b59, 3 },
    { 0x46362b60, 3 },
    { 0x9ea62b5f, 3 },
    { 0x293a2b57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43322b66, 3 },
    { 0xa9222b67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa2b6b, 3 },
    { 0x91522b6b, 3 },
    { 0, 0 },
    { 0x93022b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aa2b73, 3 },
    { 0, 0 },
    { 0x93262b75, 3 },
    { 0xa0c62b76, 3 },
    { 0xbe422b75, 3 },
    { 0, 0 },
    { 0x752e2b79, 3 },
    { 0x6dfe2b79, 4 },
    { 0xc3622b7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e2b7e, 5 },
    { 0x14322b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7962b84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbe2b89, 4 },
    { 0, 0 },
    { 0xa0422b8b, 3 },
    { 0xecd22b8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de2b96, 3 },
    { 0, 0 },
    { 0x4e8a2b98, 3 },
    { 0x79c22b99, 4 },
    { 0, 0 },
    { 0x41e2b9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea2ba0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97622ba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad562ba7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa2baa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17a2bae, 4 },
    { 0x75c22baf, 3 },
    { 0xa09a2baf, 3 },
    { 0xd7a62bb1, 3 },
    { 0xe20e2bb0, 3 },
    { 0x86fa2bb1, 3 },
    { 0xa06a2bb4, 3 },
    { 0x89de2bb4, 3 },
    { 0xb1022bb6, 3 },
    { 0xfd962bb4, 4 },
    { 0x8cb62bb8, 3 },
    { 0x4f722bb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6a2bbd, 3 },
    { 0xa0522bbe, 3 },
    { 0x13522bbf, 3 },
    { 0xe7be2bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927e2bc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27522bcc, 3 },
    { 0, 0 },
    { 0xa4f62bce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abe2bd3, 3 },
    { 0xec122bd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7862bd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e962bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0be2be6, 4 },
    { 0xcf1e2be7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45362bec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb62bef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05a2bf7, 3 },
    { 0xba822bf8, 3 },
    { 0, 0 },
    { 0x419e2bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e22c00, 3 },
    { 0x76762c00, 4 },
    { 0, 0 },
    { 0x4cba2c03, 4 },
    { 0xbf9a2c03, 3 },
    { 0xc7a22c04, 3 },
    { 0x94e22c06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc22c0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6722c0f, 4 },
    { 0x869e2c0f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9a2c17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e2c1a, 3 },
    { 0x34a62c1a, 3 },
    { 0x71fa2c1a, 3 },
    { 0x689a2c1c, 3 },
    { 0x16122c1e, 3 },
    { 0x26fa2c1f, 3 },
    { 0x96f22c1b, 3 },
    { 0x67522c21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49762c26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfe2c29, 4 },
    { 0x51262c2a, 3 },
    { 0x9dca2c2b, 4 },
    { 0x5f362c2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a2c31, 3 },
    { 0x62fe2c31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67562c3f, 3 },
    { 0x562a2c40, 3 },
    { 0, 0 },
    { 0x967e2c42, 3 },
    { 0xd42a2c43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2322c48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1762c57, 3 },
    { 0xec822c57, 3 },
    { 0x57be2c59, 3 },
    { 0x4ed62c5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c322c5f, 3 },
    { 0x33d62c60, 3 },
    { 0x32922c61, 3 },
    { 0, 0 },
    { 0x9f2e2c63, 4 },
    { 0, 0 },
    { 0x76a62c65, 4 },
    { 0x1ea62c66, 3 },
    { 0x5d662c67, 3 },
    { 0x1c622c68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22c6e, 3 },
    { 0xbdb22c6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e22c73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee2c78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe962c7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349a2c80, 4 },
    { 0xadbe2c81, 3 },
    { 0, 0 },
    { 0xd4c62c83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12022c87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f322c8c, 3 },
    { 0x116a2c8d, 4 },
    { 0xcc522c8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33522c93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba2c96, 3 },
    { 0x3e862c97, 4 },
    { 0x8e5a2c98, 3 },
    { 0x294e2c99, 3 },
    { 0x35ae2c99, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f62c9e, 3 },
    { 0x29922c9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1a2ca4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d962ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88922caa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254a2cae, 3 },
    { 0x4da2caf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e2cb3, 3 },
    { 0x56062cb3, 4 },
    { 0xe9122cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ea2cbb, 3 },
    { 0xdcba2cbb, 3 },
    { 0xadce2cbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31822cc0, 3 },
    { 0, 0 },
    { 0x27aa2cc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b62cc6, 3 },
    { 0, 0 },
    { 0xd8fe2cc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22cce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1be2cd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589e2cda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fa2cdd, 3 },
    { 0x711e2cde, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8e2ce2, 4 },
    { 0, 0 },
    { 0xd2162ce4, 3 },
    { 0xdc322ce5, 3 },
    { 0x62c22ce6, 3 },
    { 0x87b62ce6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7122ced, 3 },
    { 0x875e2cee, 4 },
    { 0, 0 },
    { 0x178e2cf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40762cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f162cf7, 5 },
    { 0x58462cf8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a62cfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1022cfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a2d03, 3 },
    { 0x43b22d03, 4 },
    { 0x13962d05, 4 },
    { 0xbfae2d05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf862d0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca2d0f, 5 },
    { 0, 0 },
    { 0x536a2d11, 3 },
    { 0x682a2d12, 5 },
    { 0x119a2d12, 3 },
    { 0xe6d22d14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28da2d19, 3 },
    { 0x1d062d1a, 3 },
    { 0, 0 },
    { 0xf7262d1c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43922d20, 3 },
    { 0x7ed22d21, 4 },
    { 0, 0 },
    { 0xa05e2d23, 4 },
    { 0x1b3e2d24, 4 },
    { 0, 0 },
    { 0x5d922d26, 3 },
    { 0x82c62d26, 3 },
    { 0x5bfe2d28, 3 },
    { 0xaf462d26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75e2d2d, 3 },
    { 0xd7f22d2e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42922d32, 3 },
    { 0x38fa2d33, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xae862d36, 4 },
    { 0x243e2d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9162d44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d62d47, 4 },
    { 0x9e9a2d48, 3 },
    { 0, 0 },
    { 0x6cca2d4a, 3 },
    { 0x6d322d4b, 3 },
    { 0xc0e2d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf22d52, 3 },
    { 0x2e2e2d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a2d58, 3 },
    { 0x18ba2d59, 3 },
    { 0x4f1e2d58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32222d67, 3 },
    { 0xff4e2d68, 3 },
    { 0x29de2d69, 3 },
    { 0xce22d69, 4 },
    { 0xf1fe2d6b, 4 },
    { 0x69ea2d6b, 3 },
    { 0xce822d6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f622d73, 4 },
    { 0x52a22d74, 4 },
    { 0xffa2d75, 3 },
    { 0x2c622d76, 3 },
    { 0xc77e2d74, 3 },
    { 0x15422d78, 4 },
    { 0x6dba2d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d22d7f, 5 },
    { 0x8b5e2d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa62d86, 4 },
    { 0x1b262d86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656a2d8d, 4 },
    { 0x3f7e2d8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225e2d95, 4 },
    { 0xc3ce2d96, 4 },
    { 0xd95a2d97, 3 },
    { 0xeb4e2d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a2d9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39862da3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de2da7, 3 },
    { 0x77822da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3262dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8462db1, 3 },
    { 0xed22db2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fda2db6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a2db9, 3 },
    { 0, 0 },
    { 0xe9a62dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e22dc0, 3 },
    { 0x780a2dc1, 3 },
    { 0xe5c62dc2, 3 },
    { 0xda262dc3, 3 },
    { 0x16a2dc4, 3 },
    { 0xabd22dc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2a2dc8, 3 },
    { 0x7b722dc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9e2dd2, 3 },
    { 0, 0 },
    { 0x662dd4, 3 },
    { 0x30e2dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a2ddb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f62dde, 3 },
    { 0x473e2ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4e2de6, 3 },
    { 0xdd322de6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaee2dea, 3 },
    { 0, 0 },
    { 0xad322dec, 3 },
    { 0x5f522ded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c922df0, 3 },
    { 0xef62df0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36162df5, 3 },
    { 0, 0 },
    { 0x45062df7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ea2dfd, 3 },
    { 0, 0 },
    { 0xa1e62dff, 4 },
    { 0x10ca2e00, 5 },
    { 0x899a2e01, 4 },
    { 0xb5b62e02, 3 },
    { 0x7f622e03, 3 },
    { 0xce122e04, 4 },
    { 0x23262e04, 3 },
    { 0xed862e06, 4 },
    { 0x5e762e07, 4 },
    { 0x26ce2e06, 3 },
    { 0x6f562e07, 5 },
    { 0x63f22e0a, 3 },
    { 0xada22e0a, 3 },
    { 0x3fd22e07, 3 },
    { 0x64f62e07, 4 },
    { 0, 0 },
    { 0xabde2e0f, 3 },
    { 0xc8ce2e10, 3 },
    { 0x37e2e10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ea2e1b, 3 },
    { 0x674a2e1b, 3 },
    { 0xc47a2e1d, 4 },
    { 0xf9f62e1e, 3 },
    { 0, 0 },
    { 0x1f7a2e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b962e25, 3 },
    { 0x5f862e26, 3 },
    { 0, 0 },
    { 0xabee2e28, 4 },
    { 0x6f062e29, 3 },
    { 0x4f0a2e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4822e2d, 3 },
    { 0, 0 },
    { 0x8bb22e2f, 3 },
    { 0, 0 },
    { 0xd1322e31, 4 },
    { 0xa0e22e32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b22e35, 3 },
    { 0x7e22e36, 3 },
    { 0, 0 },
    { 0xa1ea2e38, 3 },
    { 0xc43a2e38, 3 },
    { 0xbae22e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba462e3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b22e42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x681e2e45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36a2e49, 3 },
    { 0, 0 },
    { 0x5f422e4b, 3 },
    { 0x5dd62e4c, 3 },
    { 0x8da62e4b, 5 },
    { 0x14062e4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0362e51, 3 },
    { 0xf8562e52, 3 },
    { 0x60522e51, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87a2e59, 3 },
    { 0xeb22e5a, 4 },
    { 0x627e2e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c262e5e, 3 },
    { 0x67962e5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75062e66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a2e6a, 3 },
    { 0x25962e6b, 4 },
    { 0, 0 },
    { 0x90d22e6d, 3 },
    { 0, 0 },
    { 0xa75a2e6f, 4 },
    { 0x2e462e6f, 3 },
    { 0x751a2e71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be2e75, 4 },
    { 0x24b62e76, 4 },
    { 0xd5fe2e77, 4 },
    { 0x46862e75, 3 },
    { 0xf3e22e79, 4 },
    { 0xe81a2e7a, 3 },
    { 0xa39e2e7b, 3 },
    { 0x46d22e7b, 3 },
    { 0, 0 },
    { 0x333a2e7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e2e82, 4 },
    { 0, 0 },
    { 0x65be2e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b22e87, 5 },
    { 0, 0 },
    { 0x484a2e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740e2e8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d22e93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36a2e96, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ea2e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4a2ea1, 3 },
    { 0x7c822ea1, 3 },
    { 0xba5a2ea3, 3 },
    { 0xd5e62ea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a2ea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8122eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e2eb5, 3 },
    { 0x80ca2eb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf862ebb, 4 },
    { 0x14c22ebc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37de2ec7, 4 },
    { 0xba262ec8, 3 },
    { 0x7a822ec9, 4 },
    { 0, 0 },
    { 0x52922ecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77822ecf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ed3, 3 },
    { 0x69ce2ed3, 3 },
    { 0x86de2ed3, 3 },
    { 0x8ee22ed6, 3 },
    { 0x79f62ed7, 3 },
    { 0xf9762ed7, 3 },
    { 0xa12e2ed9, 3 },
    { 0xee2ed8, 3 },
    { 0x750a2edb, 3 },
    { 0xff9a2ed6, 3 },
    { 0, 0 },
    { 0x2e6a2ede, 3 },
    { 0x72022edf, 4 },
    { 0xa41e2ee0, 5 },
    { 0xd19a2edf, 3 },
    { 0x1ee22edf, 3 },
    { 0x77022ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb22ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1e2ee9, 3 },
    { 0x5ede2ee9, 3 },
    { 0xd96a2eeb, 3 },
    { 0x4cc22eec, 3 },
    { 0xadee2eed, 3 },
    { 0, 0 },
    { 0x95322eef, 3 },
    { 0x33ea2eef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5e2ef6, 3 },
    { 0xd8222ef6, 3 },
    { 0xa0822ef7, 3 },
    { 0xc2262ef9, 3 },
    { 0x49ba2ef6, 3 },
    { 0x1ffa2efb, 4 },
    { 0xccee2ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3e2f00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a62f03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x774a2f06, 3 },
    { 0x8f522f07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5e2f0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b62f23, 3 },
    { 0xa4da2f24, 4 },
    { 0xfdda2f25, 3 },
    { 0x59aa2f26, 3 },
    { 0xe26e2f27, 3 },
    { 0x989a2f28, 3 },
    { 0xff0a2f24, 3 },
    { 0x8c4e2f2a, 3 },
    { 0x6d7e2f2a, 3 },
    { 0, 0 },
    { 0xd0462f2d, 3 },
    { 0xcabe2f2d, 3 },
    { 0, 0 },
    { 0x39e22f30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b62f33, 3 },
    { 0xa5262f34, 3 },
    { 0, 0 },
    { 0xa1b22f36, 3 },
    { 0, 0 },
    { 0x8e7e2f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aca2f40, 3 },
    { 0x48522f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbe2f44, 4 },
    { 0, 0 },
    { 0xb8c22f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2e2f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34a2f4e, 3 },
    { 0, 0 },
    { 0xa1ca2f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ce2f53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c22f5a, 3 },
    { 0x55022f5b, 3 },
    { 0xf5c22f5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ca2f62, 3 },
    { 0, 0 },
    { 0xe362f64, 3 },
    { 0xdd022f65, 3 },
    { 0, 0 },
    { 0x23162f67, 3 },
    { 0, 0 },
    { 0x5e6e2f69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfa2f6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68062f71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f62f74, 3 },
    { 0xfca22f74, 4 },
    { 0x122f76, 3 },
    { 0xc9362f77, 3 },
    { 0x1ca22f78, 3 },
    { 0, 0 },
    { 0x73422f7a, 3 },
    { 0x7fe2f7b, 3 },
    { 0, 0 },
    { 0x3ed62f7d, 3 },
    { 0, 0 },
    { 0x5a7e2f7f, 3 },
    { 0x191e2f80, 3 },
    { 0, 0 },
    { 0x53662f82, 3 },
    { 0x89d22f83, 3 },
    { 0, 0 },
    { 0x422e2f85, 4 },
    { 0, 0 },
    { 0x9c162f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e2f8b, 3 },
    { 0xbb3a2f8c, 4 },
    { 0x628e2f8d, 3 },
    { 0xd06e2f8e, 5 },
    { 0x5d5a2f8d, 3 },
    { 0, 0 },
    { 0xc3662f91, 3 },
    { 0xa41a2f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e2f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22222f99, 3 },
    { 0xa6ce2f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9e2fa8, 3 },
    { 0xb4c22fa9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea2fac, 3 },
    { 0x15f62fad, 3 },
    { 0x1a8a2fae, 4 },
    { 0xbd2e2fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be2fb3, 4 },
    { 0x15b62fb3, 4 },
    { 0xbd7a2fb5, 4 },
    { 0xb2922fb4, 4 },
    { 0, 0 },
    { 0xce1e2fb8, 3 },
    { 0x68262fb9, 3 },
    { 0xa5222fba, 3 },
    { 0x63922fb9, 3 },
    { 0x17162fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ba2fc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b22fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e2fcc, 3 },
    { 0xd7322fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e2fd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff62fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ce2fd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293e2fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6a2fe5, 4 },
    { 0x59f62fe5, 4 },
    { 0x324e2fe7, 3 },
    { 0x9ca2fe7, 3 },
    { 0xd7662fe7, 4 },
    { 0xa21a2fea, 4 },
    { 0xfc0e2feb, 3 },
    { 0xccc62fea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a2fef, 3 },
    { 0xb6de2ff0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330e2ff5, 4 },
    { 0, 0 },
    { 0xd54e2ff7, 4 },
    { 0x2d562ff8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2a2ffb, 3 },
    { 0, 0 },
    { 0xc9de2ffd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0a3000, 3 },
    { 0, 0 },
    { 0xf5223002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685e3006, 3 },
    { 0xa4ee3006, 3 },
    { 0x8b623006, 3 },
    { 0, 0 },
    { 0xc01a300a, 4 },
    { 0x74a6300a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2300f, 3 },
    { 0x4dfa300f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5056301a, 3 },
    { 0x7ae6301b, 3 },
    { 0xf38a301a, 4 },
    { 0x6c2a301d, 3 },
    { 0x150a301d, 3 },
    { 0xe77a301d, 4 },
    { 0xfdfa301d, 4 },
    { 0x64723021, 3 },
    { 0x9d563022, 3 },
    { 0, 0 },
    { 0xb05e3024, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8012302d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a3031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda763034, 3 },
    { 0, 0 },
    { 0xe4323036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1816303e, 4 },
    { 0, 0 },
    { 0xcd163040, 3 },
    { 0, 0 },
    { 0x1dde3042, 3 },
    { 0, 0 },
    { 0xddc63044, 3 },
    { 0, 0 },
    { 0x621a3046, 3 },
    { 0, 0 },
    { 0x8fc63048, 3 },
    { 0, 0 },
    { 0x3466304a, 3 },
    { 0x5fde304a, 3 },
    { 0x250e304c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x173e304f, 3 },
    { 0x56a3050, 4 },
    { 0x940e3051, 3 },
    { 0xca0e304f, 4 },
    { 0xe3263053, 4 },
    { 0x876a3054, 3 },
    { 0, 0 },
    { 0xb2fe3056, 5 },
    { 0, 0 },
    { 0x71a23058, 4 },
    { 0x48aa3059, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076305d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8723061, 3 },
    { 0, 0 },
    { 0xeb523063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7363067, 3 },
    { 0x9eee3068, 3 },
    { 0x61e63069, 3 },
    { 0x1b6a3068, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046306e, 3 },
    { 0x1a12306e, 3 },
    { 0x3cde306e, 3 },
    { 0x3c62306e, 3 },
    { 0x627e3072, 3 },
    { 0x87a63073, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179a307b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9a307e, 3 },
    { 0x3bfa307f, 3 },
    { 0x357e3080, 4 },
    { 0xc1823080, 3 },
    { 0x71da3082, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a3085, 3 },
    { 0xf3fe3086, 4 },
    { 0x44a23087, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3662308c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55e308f, 3 },
    { 0xe1de308f, 3 },
    { 0xd9b2308f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba3095, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60e309a, 3 },
    { 0x1e6309a, 4 },
    { 0x8156309a, 3 },
    { 0, 0 },
    { 0xf56a309e, 3 },
    { 0, 0 },
    { 0x770a30a0, 3 },
    { 0xee2630a1, 4 },
    { 0x44d630a2, 3 },
    { 0x21be30a2, 3 },
    { 0, 0 },
    { 0x62d230a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46a30aa, 5 },
    { 0x252a30ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a30af, 3 },
    { 0x658630b0, 3 },
    { 0x24be30af, 3 },
    { 0x2f8e30b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a630b5, 3 },
    { 0x6bfa30b5, 3 },
    { 0x48b230b6, 3 },
    { 0x420630b8, 3 },
    { 0xeb1a30b9, 3 },
    { 0x965230b5, 4 },
    { 0x837e30b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f630be, 4 },
    { 0x1630bf, 3 },
    { 0x844630c0, 3 },
    { 0xce5230be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da30c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d630c7, 3 },
    { 0xb48a30c7, 3 },
    { 0xab0e30c8, 3 },
    { 0, 0 },
    { 0x7cce30cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xade630ce, 3 },
    { 0xe78630cf, 5 },
    { 0xd47230d0, 3 },
    { 0x998e30d1, 5 },
    { 0x8a0e30d2, 3 },
    { 0x763e30d3, 3 },
    { 0x8a5e30d2, 4 },
    { 0x80a630d4, 3 },
    { 0x888a30d2, 4 },
    { 0x20ea30d6, 3 },
    { 0x9fa30d1, 3 },
    { 0x99b630d9, 3 },
    { 0xb3ca30d3, 3 },
    { 0x9fa230d8, 4 },
    { 0xbd8630dc, 3 },
    { 0xf40e30dc, 3 },
    { 0x55b630de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8230e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74230ea, 4 },
    { 0x75ce30ea, 3 },
    { 0xbfa30ea, 4 },
    { 0xbf7230eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538e30f2, 3 },
    { 0x97ee30f2, 3 },
    { 0x3a4230f4, 3 },
    { 0x8a8230f5, 4 },
    { 0xbfc230f5, 4 },
    { 0xfa230f4, 3 },
    { 0xc6ee30f8, 3 },
    { 0, 0 },
    { 0xbf4230fa, 4 },
    { 0x809e30fb, 3 },
    { 0x3b5630fb, 3 },
    { 0x946e30fb, 4 },
    { 0xbbfe30fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a63101, 5 },
    { 0, 0 },
    { 0x7a7a3103, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca63106, 5 },
    { 0, 0 },
    { 0x4e1a3108, 4 },
    { 0xe91e3109, 3 },
    { 0, 0 },
    { 0x790a310b, 3 },
    { 0, 0 },
    { 0xca46310d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43a3115, 3 },
    { 0xa5f63116, 3 },
    { 0x10563115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf32311f, 3 },
    { 0, 0 },
    { 0xabaa3121, 3 },
    { 0, 0 },
    { 0x889a3123, 3 },
    { 0xf9163124, 3 },
    { 0x182e3125, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72a312a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fa312d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee63133, 3 },
    { 0x6d4e3134, 5 },
    { 0, 0 },
    { 0xc5b23136, 4 },
    { 0xe11a3137, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa313a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50e3140, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x599a3143, 4 },
    { 0x9c663143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f723148, 3 },
    { 0x3fa23148, 3 },
    { 0, 0 },
    { 0xcea2314b, 3 },
    { 0xcd0a314c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e3155, 3 },
    { 0x39c23156, 3 },
    { 0, 0 },
    { 0x77ca3158, 5 },
    { 0x88ca3159, 3 },
    { 0, 0 },
    { 0x4e46315b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e315e, 4 },
    { 0, 0 },
    { 0x39e23160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x278a3163, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0e3166, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd323169, 3 },
    { 0, 0 },
    { 0x178e316b, 3 },
    { 0x9876316c, 3 },
    { 0x48ba316d, 4 },
    { 0xe6d6316d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf63173, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c023176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea3179, 3 },
    { 0xbeb6317a, 3 },
    { 0, 0 },
    { 0x260e317c, 3 },
    { 0x7aa6317d, 3 },
    { 0x7ac6317e, 3 },
    { 0xc19a317d, 3 },
    { 0xd47a3180, 3 },
    { 0x9eaa317f, 3 },
    { 0x994e3182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92de3185, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94e318c, 3 },
    { 0, 0 },
    { 0x2662318e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c23191, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca3194, 3 },
    { 0x60463195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716a3199, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf02319d, 3 },
    { 0xbd5e319e, 3 },
    { 0x25ba319f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973231a4, 4 },
    { 0x7a5231a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667a31aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fe31b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec631c1, 3 },
    { 0, 0 },
    { 0x55ae31c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ae31c7, 3 },
    { 0, 0 },
    { 0x3d8631c9, 3 },
    { 0x1dae31ca, 3 },
    { 0x9b8a31ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9631d0, 3 },
    { 0, 0 },
    { 0xbf6231d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fa31d5, 3 },
    { 0xe35231d6, 3 },
    { 0x45f631d7, 3 },
    { 0xd79631d7, 3 },
    { 0xc1ae31d9, 3 },
    { 0x8c2231d6, 4 },
    { 0x61be31db, 3 },
    { 0, 0 },
    { 0xe6b231dd, 3 },
    { 0x5d0a31de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x621231e1, 4 },
    { 0x9fa31e1, 3 },
    { 0xb16231e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b231e8, 3 },
    { 0xd33231e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73e31ec, 3 },
    { 0x802231ec, 3 },
    { 0x986a31ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487e31f4, 3 },
    { 0xb231f5, 3 },
    { 0x176e31f6, 4 },
    { 0xbfe631f7, 3 },
    { 0x5a7631f8, 3 },
    { 0, 0 },
    { 0xd28631fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cca31ff, 4 },
    { 0x4e6e31ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33123205, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e320a, 3 },
    { 0xdfae320b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d223214, 4 },
    { 0x1e323214, 4 },
    { 0x17de3214, 3 },
    { 0xaca23215, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8902321b, 4 },
    { 0x7dfe321c, 3 },
    { 0x5002321d, 3 },
    { 0xf5ca321e, 4 },
    { 0, 0 },
    { 0x15623220, 4 },
    { 0x8c4a3221, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a63226, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee6322e, 3 },
    { 0, 0 },
    { 0x2cbe3230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67863237, 3 },
    { 0, 0 },
    { 0x96a3239, 3 },
    { 0, 0 },
    { 0x81f6323b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cae323f, 4 },
    { 0x39223240, 3 },
    { 0, 0 },
    { 0xc37a3242, 4 },
    { 0xedde3243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c63247, 3 },
    { 0x57d23247, 3 },
    { 0x90c23249, 3 },
    { 0, 0 },
    { 0x180a324b, 3 },
    { 0x173e324c, 3 },
    { 0xf1d6324c, 3 },
    { 0xe64e324d, 4 },
    { 0, 0 },
    { 0xde4a3250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544a3255, 3 },
    { 0xf22e3256, 3 },
    { 0x84a3257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e325b, 3 },
    { 0, 0 },
    { 0x4e92325d, 3 },
    { 0x1aba325d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef23263, 4 },
    { 0xe5563263, 3 },
    { 0xd80e3264, 3 },
    { 0xc2d63266, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0a326c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d463271, 3 },
    { 0, 0 },
    { 0x124a3273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95e3276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a3279, 3 },
    { 0, 0 },
    { 0x8662327b, 4 },
    { 0xbe26327b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9e3284, 3 },
    { 0xe61e3284, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b6328a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a2328e, 3 },
    { 0, 0 },
    { 0x481e3290, 3 },
    { 0x98063291, 3 },
    { 0xfe7e3292, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12be3297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2a329e, 4 },
    { 0, 0 },
    { 0xb18a32a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8a32a4, 5 },
    { 0, 0 },
    { 0x54fe32a6, 3 },
    { 0xd3ae32a7, 3 },
    { 0x445e32a7, 3 },
    { 0x9ac632a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93a32ae, 3 },
    { 0x71e632ae, 3 },
    { 0x4b8e32b0, 4 },
    { 0, 0 },
    { 0x41b232b2, 4 },
    { 0x65ca32b3, 3 },
    { 0, 0 },
    { 0xe35232b5, 3 },
    { 0x76a632b5, 3 },
    { 0, 0 },
    { 0x19de32b8, 3 },
    { 0xc45232b9, 3 },
    { 0x367232b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4232c0, 4 },
    { 0, 0 },
    { 0x62d232c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90232c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae32ca, 3 },
    { 0x782e32cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208a32cf, 5 },
    { 0x6bf232d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977a32d6, 4 },
    { 0, 0 },
    { 0x1f2232d8, 3 },
    { 0xe85e32d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6a32df, 3 },
    { 0xf3d632e0, 3 },
    { 0x9b9632e1, 3 },
    { 0x102232df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efa32e6, 3 },
    { 0x568e32e6, 4 },
    { 0, 0 },
    { 0xf7b632e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232632ed, 3 },
    { 0xe04632ee, 4 },
    { 0x202e32ef, 3 },
    { 0, 0 },
    { 0x86ce32f1, 3 },
    { 0x813232f1, 3 },
    { 0, 0 },
    { 0xef632f4, 4 },
    { 0xf5de32f5, 3 },
    { 0x7bf632f6, 4 },
    { 0xa6b632f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9e32ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3a3304, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8336330a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e330e, 4 },
    { 0, 0 },
    { 0x4db63310, 3 },
    { 0, 0 },
    { 0xf91a3312, 3 },
    { 0x92923312, 4 },
    { 0xd5063314, 3 },
    { 0x8f6a3314, 4 },
    { 0xe3e63316, 5 },
    { 0x68be3312, 4 },
    { 0x4d163318, 3 },
    { 0x2fd23316, 3 },
    { 0xcb23313, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ce331f, 4 },
    { 0, 0 },
    { 0x54b63321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd323328, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e332f, 3 },
    { 0x4a3a3330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28423336, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f63339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3463348, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ee334c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e3352, 3 },
    { 0x47c63353, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2463357, 3 },
    { 0x987a3358, 3 },
    { 0x17663359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c2335f, 3 },
    { 0x884a335f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a336b, 3 },
    { 0, 0 },
    { 0x229e336d, 3 },
    { 0xc0c2336d, 4 },
    { 0x750a336d, 3 },
    { 0xb36a3370, 3 },
    { 0xfbee3371, 3 },
    { 0x4bee336e, 3 },
    { 0, 0 },
    { 0x23ae3374, 3 },
    { 0, 0 },
    { 0x4a763376, 3 },
    { 0x70a63377, 3 },
    { 0xd8763376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e337b, 3 },
    { 0x7046337b, 3 },
    { 0xe62a337b, 3 },
    { 0xb0e2337e, 3 },
    { 0x4f1e337f, 3 },
    { 0xddb6337b, 3 },
    { 0x256337b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b23388, 3 },
    { 0, 0 },
    { 0x37ce338a, 4 },
    { 0x7ffa338a, 4 },
    { 0xc56338c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88de3391, 3 },
    { 0xb72e3392, 4 },
    { 0xa23e3392, 3 },
    { 0x53323394, 4 },
    { 0x14ae3393, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eee339d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61233a1, 3 },
    { 0, 0 },
    { 0xdba633a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bce33a6, 3 },
    { 0, 0 },
    { 0xd23e33a8, 3 },
    { 0x413633a9, 4 },
    { 0x99aa33aa, 3 },
    { 0x509e33a9, 3 },
    { 0xef233ab, 5 },
    { 0, 0 },
    { 0x39d233ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a33b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad233b6, 4 },
    { 0x280633b7, 4 },
    { 0x65f633b8, 4 },
    { 0, 0 },
    { 0xaf7633ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x596e33bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea33c0, 3 },
    { 0, 0 },
    { 0x643e33c2, 4 },
    { 0x309a33c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a233c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4633cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2633d2, 3 },
    { 0, 0 },
    { 0x6d9e33d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1e33d8, 4 },
    { 0x6e5633d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87be33dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05633e3, 3 },
    { 0x788233e4, 3 },
    { 0, 0 },
    { 0x726633e6, 3 },
    { 0xd7ce33e7, 3 },
    { 0x171233e8, 3 },
    { 0, 0 },
    { 0x498633ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef633ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5e33f2, 3 },
    { 0x9d0a33f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e33f9, 3 },
    { 0xe81633fa, 3 },
    { 0x9a7233fa, 3 },
    { 0x544e33fc, 3 },
    { 0, 0 },
    { 0x579a33fe, 4 },
    { 0xc19e33ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a823405, 4 },
    { 0xeefe3406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b63413, 3 },
    { 0xe6ba3414, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aa3418, 3 },
    { 0x5e0e3419, 3 },
    { 0x4b5e341a, 3 },
    { 0, 0 },
    { 0x769e341c, 4 },
    { 0x76f2341d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7823422, 4 },
    { 0x89ee3423, 3 },
    { 0x71523424, 3 },
    { 0x9cba3424, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06e3428, 3 },
    { 0xd18a3428, 4 },
    { 0x4036342a, 4 },
    { 0, 0 },
    { 0xb736342c, 3 },
    { 0x2ed2342d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846a3436, 5 },
    { 0x97823436, 3 },
    { 0x592e3438, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0e343c, 4 },
    { 0xcd3e343c, 3 },
    { 0, 0 },
    { 0x28be343f, 3 },
    { 0x6b9a3440, 3 },
    { 0xc79e3440, 3 },
    { 0x1b2e3442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacea3445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa332344b, 3 },
    { 0, 0 },
    { 0x711e344d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9de3451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaee3454, 3 },
    { 0x1d6e3455, 3 },
    { 0xe0fe3455, 3 },
    { 0, 0 },
    { 0x2ebe3458, 4 },
    { 0x3b723458, 3 },
    { 0x612a3458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabe345d, 3 },
    { 0, 0 },
    { 0xd76a345f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7963466, 3 },
    { 0x5f223467, 4 },
    { 0, 0 },
    { 0xead63469, 4 },
    { 0xfae23469, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1096346d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7963471, 3 },
    { 0x5a823472, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e6347a, 4 },
    { 0, 0 },
    { 0xc81e347c, 3 },
    { 0xabb6347c, 3 },
    { 0xad66347c, 4 },
    { 0xc6c6347e, 3 },
    { 0x6d923480, 3 },
    { 0, 0 },
    { 0x4c5e3482, 4 },
    { 0x26763483, 4 },
    { 0xb9aa3484, 3 },
    { 0, 0 },
    { 0x536e3486, 3 },
    { 0x118a3487, 3 },
    { 0x82863488, 3 },
    { 0xd9663489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1502348d, 3 },
    { 0, 0 },
    { 0x66e2348f, 4 },
    { 0xc8d6348f, 3 },
    { 0xeaba348f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fea3496, 3 },
    { 0x96163496, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92e349a, 3 },
    { 0x912e349b, 3 },
    { 0x6e2e349c, 3 },
    { 0x20fa349d, 4 },
    { 0, 0 },
    { 0xd38e349f, 3 },
    { 0x60da34a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74234a5, 3 },
    { 0x8bea34a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630a34ae, 4 },
    { 0xe16234ae, 5 },
    { 0, 0 },
    { 0x5ae634b1, 5 },
    { 0, 0 },
    { 0x8e7e34b3, 4 },
    { 0, 0 },
    { 0x25be34b5, 3 },
    { 0xbe5a34b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa634b9, 3 },
    { 0x730634b9, 3 },
    { 0xfed234bb, 3 },
    { 0x317634bb, 3 },
    { 0, 0 },
    { 0x3ce34be, 3 },
    { 0xa81a34bf, 4 },
    { 0x108a34be, 3 },
    { 0xdb8234c1, 3 },
    { 0x934234c2, 3 },
    { 0, 0 },
    { 0x451e34c4, 3 },
    { 0x27634c5, 4 },
    { 0xa53634c6, 3 },
    { 0x137a34c7, 4 },
    { 0xab3634c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ae34cb, 3 },
    { 0xed9234cb, 3 },
    { 0x4fe234cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53a34d3, 3 },
    { 0x595e34d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65e34e4, 3 },
    { 0x1a3e34e5, 3 },
    { 0x265234e4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x801234e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x642234ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93e34f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4234f6, 3 },
    { 0xf4ba34f7, 3 },
    { 0x5d2634f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a34fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54663507, 4 },
    { 0x45ce3508, 3 },
    { 0x273e3509, 3 },
    { 0xe26a350a, 3 },
    { 0x6b5e350a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a3511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4723514, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30da351e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8e3522, 4 },
    { 0x18423522, 3 },
    { 0x66063523, 3 },
    { 0x29ea3522, 5 },
    { 0xd3123523, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185e352a, 3 },
    { 0, 0 },
    { 0x1d5a352c, 3 },
    { 0, 0 },
    { 0xe752352e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea3531, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91663538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5262353c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd22353f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38a3542, 3 },
    { 0, 0 },
    { 0xbfae3544, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcca3547, 3 },
    { 0xfb663547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d42354e, 3 },
    { 0x2b36354f, 3 },
    { 0xfda6354e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fde3554, 3 },
    { 0x896a3555, 3 },
    { 0xd5563555, 4 },
    { 0, 0 },
    { 0xfd123558, 3 },
    { 0xf35e3559, 3 },
    { 0x9c0e355a, 3 },
    { 0x4ca6355a, 3 },
    { 0x2de355c, 3 },
    { 0x162e355c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fa3561, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3296356b, 4 },
    { 0xd60a356c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73e3572, 3 },
    { 0x683a3572, 4 },
    { 0x7b0a3573, 3 },
    { 0x4c763575, 3 },
    { 0, 0 },
    { 0x47a3577, 4 },
    { 0, 0 },
    { 0xb65a3579, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146357e, 3 },
    { 0xe372357e, 3 },
    { 0x2672357e, 3 },
    { 0xd3763581, 4 },
    { 0x795e3582, 3 },
    { 0x90aa3581, 4 },
    { 0xc36a3584, 4 },
    { 0x93063582, 4 },
    { 0xd33a3580, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e3589, 4 },
    { 0x31a358a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf52358f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc663595, 3 },
    { 0xb6b23596, 3 },
    { 0x8a723597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d6359b, 3 },
    { 0xf00a359b, 3 },
    { 0x6c62359c, 4 },
    { 0x5eda359e, 4 },
    { 0x962e359f, 4 },
    { 0, 0 },
    { 0x357235a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0a35a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c635ac, 3 },
    { 0xc99235ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602635b4, 3 },
    { 0, 0 },
    { 0xf46a35b6, 3 },
    { 0x8eb635b7, 3 },
    { 0x2de635b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35635bc, 3 },
    { 0x9a5235bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62635c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x406a35c7, 3 },
    { 0x66ae35c7, 4 },
    { 0xadde35c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fa35cd, 3 },
    { 0xb8fe35ce, 3 },
    { 0x7c8235cf, 3 },
    { 0xe08235cf, 3 },
    { 0xefae35cf, 3 },
    { 0, 0 },
    { 0x2ce235d3, 3 },
    { 0, 0 },
    { 0xb1a635d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de635e1, 3 },
    { 0x74b635e2, 3 },
    { 0x320e35e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52635e8, 3 },
    { 0x32a635e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e35ec, 4 },
    { 0, 0 },
    { 0x2e7e35ee, 3 },
    { 0xf57e35ef, 3 },
    { 0x9a1235f0, 3 },
    { 0xee0e35f0, 4 },
    { 0x9cd235f2, 4 },
    { 0xec5e35f3, 3 },
    { 0x620e35f0, 3 },
    { 0xa41a35f2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32235f8, 3 },
    { 0xd3ee35f9, 3 },
    { 0xc91235f9, 4 },
    { 0, 0 },
    { 0x5c8635fc, 3 },
    { 0xc5d235fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d423600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29a3606, 3 },
    { 0xc6a3606, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41a360a, 3 },
    { 0, 0 },
    { 0x4ab6360c, 3 },
    { 0, 0 },
    { 0xfa42360e, 3 },
    { 0, 0 },
    { 0x632e3610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b63617, 3 },
    { 0, 0 },
    { 0x229a3619, 3 },
    { 0x3bee361a, 3 },
    { 0, 0 },
    { 0x98a2361c, 3 },
    { 0xf4f6361d, 4 },
    { 0x501a361e, 3 },
    { 0xcd56361e, 3 },
    { 0, 0 },
    { 0x62b23621, 3 },
    { 0x319e3621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66463627, 4 },
    { 0, 0 },
    { 0x8cf23629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e63630, 3 },
    { 0xd59a3630, 3 },
    { 0x296e3631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51e3636, 3 },
    { 0x34763637, 3 },
    { 0xd6ea3638, 4 },
    { 0x2ab23639, 3 },
    { 0xb162363a, 3 },
    { 0xeba23639, 4 },
    { 0xb52a363c, 4 },
    { 0, 0 },
    { 0xf9b6363e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f663648, 3 },
    { 0x138e3648, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0a364d, 3 },
    { 0x81de364e, 4 },
    { 0x4fde364d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e3652, 3 },
    { 0x56863653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d663656, 4 },
    { 0, 0 },
    { 0x26ca3658, 4 },
    { 0xea423659, 3 },
    { 0x9e1a3659, 3 },
    { 0x1eb2365a, 5 },
    { 0x157a3659, 4 },
    { 0x24d63659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488e3661, 3 },
    { 0x8cfa3662, 3 },
    { 0, 0 },
    { 0x1e9a3664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587e366b, 3 },
    { 0xe8c6366b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5a3670, 3 },
    { 0x54163671, 3 },
    { 0, 0 },
    { 0xc1663673, 3 },
    { 0, 0 },
    { 0x3c123675, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e367c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2367f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a3682, 4 },
    { 0, 0 },
    { 0x4cea3684, 4 },
    { 0x6bf23685, 4 },
    { 0x19f23686, 3 },
    { 0xdc923685, 3 },
    { 0, 0 },
    { 0x124a3689, 4 },
    { 0x45e2368a, 3 },
    { 0, 0 },
    { 0x3b82368c, 3 },
    { 0, 0 },
    { 0xd706368e, 4 },
    { 0, 0 },
    { 0x1d4a3690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e3694, 3 },
    { 0xbf2e3695, 4 },
    { 0xe6763694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c236a3, 3 },
    { 0, 0 },
    { 0xf2be36a5, 4 },
    { 0, 0 },
    { 0xdf4a36a7, 4 },
    { 0x2de236a8, 3 },
    { 0xec6636a9, 3 },
    { 0, 0 },
    { 0x1eca36ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c636af, 3 },
    { 0, 0 },
    { 0xd17e36b1, 3 },
    { 0xe23e36b2, 3 },
    { 0, 0 },
    { 0xfdf236b4, 3 },
    { 0, 0 },
    { 0x262a36b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2a36b9, 3 },
    { 0, 0 },
    { 0x57a36bb, 4 },
    { 0x875636bc, 5 },
    { 0x588236bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1a36c5, 3 },
    { 0x9efa36c6, 3 },
    { 0x4f0e36c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41de36cd, 3 },
    { 0x3d9236ce, 3 },
    { 0, 0 },
    { 0xcd2a36d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9636d7, 4 },
    { 0, 0 },
    { 0x3b5a36d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aa36dc, 4 },
    { 0x60c636dd, 3 },
    { 0x15ca36de, 3 },
    { 0, 0 },
    { 0x8eee36e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed236e5, 3 },
    { 0x323236e6, 3 },
    { 0xbf9236e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508e36eb, 4 },
    { 0, 0 },
    { 0x5a0a36ed, 3 },
    { 0x13f636ee, 4 },
    { 0x37fe36ef, 4 },
    { 0xfad636f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a36f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c636fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fe3704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbe3707, 3 },
    { 0, 0 },
    { 0x30ca3709, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee370e, 4 },
    { 0, 0 },
    { 0x88e63710, 3 },
    { 0x320a3711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2e3714, 4 },
    { 0, 0 },
    { 0xba5a3716, 4 },
    { 0xc1f63717, 3 },
    { 0x24463718, 4 },
    { 0x9cea3717, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbe371c, 3 },
    { 0, 0 },
    { 0xa76e371e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e3721, 3 },
    { 0x9b123722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd63726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb6372c, 3 },
    { 0, 0 },
    { 0xe5aa372e, 3 },
    { 0x3f6372f, 4 },
    { 0, 0 },
    { 0xe0763731, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b23739, 3 },
    { 0x819e373a, 3 },
    { 0x355a3739, 4 },
    { 0, 0 },
    { 0xb556373d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da3741, 4 },
    { 0, 0 },
    { 0x98b63743, 3 },
    { 0x7c923744, 3 },
    { 0, 0 },
    { 0x96623746, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e374a, 3 },
    { 0xf42374b, 4 },
    { 0x1f7e374c, 3 },
    { 0xa29a374d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x788a3750, 3 },
    { 0, 0 },
    { 0xb76e3752, 3 },
    { 0xa2963753, 4 },
    { 0x685a3753, 3 },
    { 0x257a3755, 4 },
    { 0, 0 },
    { 0x8cb23757, 3 },
    { 0x48d63757, 3 },
    { 0, 0 },
    { 0x8f3a375a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226375e, 4 },
    { 0xda92375f, 3 },
    { 0, 0 },
    { 0x86863761, 3 },
    { 0xf3223761, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47e3769, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f23771, 3 },
    { 0x8d7a3771, 3 },
    { 0x6b663771, 3 },
    { 0x5d163771, 4 },
    { 0, 0 },
    { 0xf9f23776, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e377f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95563783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20163787, 3 },
    { 0x35de3788, 4 },
    { 0x56da3789, 4 },
    { 0, 0 },
    { 0x9bb6378b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a02378e, 3 },
    { 0x9266378f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e3792, 3 },
    { 0xf30a3793, 3 },
    { 0x5bc63793, 3 },
    { 0xd6463795, 3 },
    { 0x8baa3796, 4 },
    { 0x7fba3797, 3 },
    { 0x8ed23798, 4 },
    { 0x2a363799, 3 },
    { 0, 0 },
    { 0xf8fe379b, 3 },
    { 0xba82379c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6237a6, 3 },
    { 0x505237a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e237aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b637ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35237b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd637bd, 4 },
    { 0, 0 },
    { 0x650a37bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156237c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae37c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4237d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6237df, 4 },
    { 0, 0 },
    { 0x1e3237e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee37e6, 3 },
    { 0xdc9e37e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed237f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40423801, 3 },
    { 0x80ba3801, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c263805, 3 },
    { 0xb76a3806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a380b, 3 },
    { 0xc85a380b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94a380f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0e3812, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9a3816, 3 },
    { 0xbd2e3816, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6381b, 3 },
    { 0, 0 },
    { 0x372a381d, 3 },
    { 0x959a381e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a63821, 4 },
    { 0, 0 },
    { 0x617a3823, 3 },
    { 0x1c5a3823, 3 },
    { 0xa0ae3825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x687e3828, 4 },
    { 0xf7463829, 3 },
    { 0, 0 },
    { 0x9362382b, 3 },
    { 0x37c6382b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e223833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee3838, 4 },
    { 0x6e2e3839, 3 },
    { 0, 0 },
    { 0x9f6e383b, 3 },
    { 0, 0 },
    { 0x4fbe383d, 3 },
    { 0x34ca383d, 3 },
    { 0x8c06383e, 4 },
    { 0x7b86383f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefe3845, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x48063848, 3 },
    { 0x3d663849, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6382384d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5e3853, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9623865, 3 },
    { 0xbf123866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f66386c, 3 },
    { 0xf416386d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63872, 3 },
    { 0x59aa3872, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ae3878, 3 },
    { 0, 0 },
    { 0xc822387a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7a387d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0a3882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e3889, 4 },
    { 0x2d76388a, 3 },
    { 0x3d963889, 3 },
    { 0x1926388c, 3 },
    { 0x9bbe388c, 3 },
    { 0x16aa388e, 3 },
    { 0, 0 },
    { 0x78a23890, 3 },
    { 0x5e4a3891, 4 },
    { 0, 0 },
    { 0xa5123893, 3 },
    { 0, 0 },
    { 0x6d523895, 4 },
    { 0x47523895, 3 },
    { 0x2f9e3897, 3 },
    { 0x3ab63897, 5 },
    { 0, 0 },
    { 0x41ce389a, 3 },
    { 0, 0 },
    { 0xf29a389c, 3 },
    { 0x7c6389c, 4 },
    { 0x6e3a389e, 3 },
    { 0xf572389f, 3 },
    { 0x937638a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96638a3, 3 },
    { 0x8a6e38a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8638ab, 4 },
    { 0, 0 },
    { 0xee4238ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ce38b0, 5 },
    { 0, 0 },
    { 0x4fee38b2, 4 },
    { 0, 0 },
    { 0x8b1638b4, 4 },
    { 0x689638b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa38b8, 3 },
    { 0x6b8638b9, 5 },
    { 0xc81238ba, 3 },
    { 0, 0 },
    { 0xcc1e38bc, 4 },
    { 0x2dae38bc, 3 },
    { 0xa6fa38be, 3 },
    { 0x8c9238bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a38c6, 3 },
    { 0, 0 },
    { 0x4b3638c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x151638cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aa38d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x217638d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c238da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fe38e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaa38ef, 3 },
    { 0, 0 },
    { 0x8c3a38f1, 4 },
    { 0x396a38f1, 3 },
    { 0xd78a38f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b238fb, 4 },
    { 0, 0 },
    { 0xc0ba38fd, 3 },
    { 0x82d638fe, 3 },
    { 0x39e238ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe123902, 3 },
    { 0x3563903, 4 },
    { 0x149a3903, 3 },
    { 0x28b63905, 3 },
    { 0x2afa3903, 3 },
    { 0x38b63906, 3 },
    { 0x9f2e3906, 5 },
    { 0, 0 },
    { 0xf35a390a, 3 },
    { 0, 0 },
    { 0xdd86390c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7a3914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27de391b, 3 },
    { 0x982a391b, 3 },
    { 0x1322391c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ce3925, 3 },
    { 0xb91a3925, 3 },
    { 0, 0 },
    { 0xcf0a3928, 3 },
    { 0x25e23928, 3 },
    { 0xceb63929, 3 },
    { 0xc2a3928, 3 },
    { 0xd456392c, 3 },
    { 0x1416392d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6393a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8a393e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ce3942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99963946, 3 },
    { 0xba2e3947, 3 },
    { 0x2d0a3946, 3 },
    { 0xec923948, 4 },
    { 0x4af6394a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244e3951, 3 },
    { 0, 0 },
    { 0xea2e3953, 3 },
    { 0xb3023954, 3 },
    { 0x1b6e3955, 3 },
    { 0xb0ce3955, 3 },
    { 0x516e3957, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88e395b, 3 },
    { 0, 0 },
    { 0xeb1a395d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5263961, 3 },
    { 0x602a3962, 3 },
    { 0xd8fa3962, 3 },
    { 0x5eca3964, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42323968, 3 },
    { 0x99f23968, 3 },
    { 0x30ea3969, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a396d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d863973, 4 },
    { 0xb623974, 4 },
    { 0x15aa3974, 3 },
    { 0xac623974, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc923980, 3 },
    { 0, 0 },
    { 0xaa163982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e63986, 3 },
    { 0x7b563986, 3 },
    { 0x3e1e3988, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa398f, 3 },
    { 0, 0 },
    { 0x3d1a3991, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffa3995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c2399c, 3 },
    { 0, 0 },
    { 0x8666399e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ace39a3, 3 },
    { 0xfbc639a4, 4 },
    { 0x1e2639a5, 3 },
    { 0x4be639a6, 4 },
    { 0xa48e39a7, 4 },
    { 0xa33239a6, 3 },
    { 0x9c0639a6, 4 },
    { 0x4f0e39aa, 3 },
    { 0x285e39a8, 3 },
    { 0, 0 },
    { 0x6d0a39ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45239b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f239b5, 3 },
    { 0, 0 },
    { 0x191e39b7, 3 },
    { 0, 0 },
    { 0xf91a39b9, 4 },
    { 0xa74639ba, 3 },
    { 0, 0 },
    { 0x361639bc, 3 },
    { 0, 0 },
    { 0xb3239be, 3 },
    { 0x666639bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303239c4, 3 },
    { 0xae0a39c5, 3 },
    { 0x12d639c6, 3 },
    { 0x996a39c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63239ca, 3 },
    { 0xdea239ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252639d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde39d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d639dc, 3 },
    { 0x13ba39dc, 3 },
    { 0xfc2639de, 3 },
    { 0x668639dc, 3 },
    { 0xee5639dc, 3 },
    { 0xd75a39dd, 3 },
    { 0x558a39e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a39e5, 3 },
    { 0x5e5639e6, 3 },
    { 0x187239e6, 4 },
    { 0xa17639e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e39ee, 4 },
    { 0x5e6239ee, 3 },
    { 0, 0 },
    { 0x147a39f1, 3 },
    { 0xe09639f2, 3 },
    { 0, 0 },
    { 0xc38239f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00a39f7, 3 },
    { 0x3fa239f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de39fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e63a04, 3 },
    { 0xe4623a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffe3a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f23a0e, 4 },
    { 0x66e3a0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a23a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a3a1d, 3 },
    { 0x66123a1e, 3 },
    { 0, 0 },
    { 0xbfa63a20, 4 },
    { 0, 0 },
    { 0x98ee3a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1a3a29, 3 },
    { 0xaf5a3a2a, 3 },
    { 0x3e3a3a2a, 3 },
    { 0, 0 },
    { 0x1afe3a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13e3a33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d23a3e, 3 },
    { 0, 0 },
    { 0xcb4e3a40, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fa3a46, 4 },
    { 0xc8023a47, 3 },
    { 0x5e363a46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbea3a4b, 3 },
    { 0x1ca63a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8463a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde3a53, 3 },
    { 0x89363a53, 3 },
    { 0x582a3a55, 3 },
    { 0x16963a54, 3 },
    { 0x218a3a54, 4 },
    { 0x68be3a56, 3 },
    { 0x64923a54, 3 },
    { 0, 0 },
    { 0xd8ca3a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aa3a6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd723a70, 3 },
    { 0x5a1e3a71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee3a77, 3 },
    { 0, 0 },
    { 0x30163a79, 5 },
    { 0, 0 },
    { 0x38ca3a7b, 3 },
    { 0x79ee3a7c, 4 },
    { 0, 0 },
    { 0xcb3a3a7e, 4 },
    { 0xf19e3a7f, 3 },
    { 0, 0 },
    { 0x6b0a3a81, 3 },
    { 0xa7b63a82, 3 },
    { 0, 0 },
    { 0xbf923a84, 3 },
    { 0x261e3a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36023a89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e3a8c, 3 },
    { 0x1263a8d, 4 },
    { 0x130e3a8d, 3 },
    { 0, 0 },
    { 0xdb823a90, 3 },
    { 0x70ba3a91, 3 },
    { 0xb5723a91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4e3a97, 3 },
    { 0, 0 },
    { 0x5e723a99, 3 },
    { 0x66d23a99, 3 },
    { 0xaca23a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e3aa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ce3aa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc23aa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a23aaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfe3ab4, 3 },
    { 0, 0 },
    { 0xca3a3ab6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x92623ab9, 3 },
    { 0xed723aba, 3 },
    { 0x15a3ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbce3ac1, 3 },
    { 0, 0 },
    { 0x8fa63ac3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e23ac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1523aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23123ad0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48223ad4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x359a3ad7, 3 },
    { 0, 0 },
    { 0x7c4a3ad9, 3 },
    { 0x49da3ada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fa3ade, 4 },
    { 0x44863ade, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e63ae2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2a3ae5, 3 },
    { 0xab9a3ae6, 3 },
    { 0x7fca3ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d323aea, 3 },
    { 0x665a3aeb, 3 },
    { 0x5ea63aec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac763aef, 3 },
    { 0, 0 },
    { 0xa80e3af1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3a3af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aa3af8, 3 },
    { 0x2f8a3af9, 4 },
    { 0, 0 },
    { 0x2d3e3afb, 3 },
    { 0xa5263afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44a3aff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12323b04, 4 },
    { 0, 0 },
    { 0x4a7e3b06, 3 },
    { 0xc8fa3b06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ae3b0e, 3 },
    { 0x29763b0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51263b13, 4 },
    { 0x198e3b14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fe3b17, 3 },
    { 0x924a3b18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fa3b1e, 3 },
    { 0xd58a3b1f, 3 },
    { 0x34a23b20, 4 },
    { 0xa5423b21, 3 },
    { 0x260a3b22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8e3b25, 3 },
    { 0x72963b26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabde3b2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4e3b2f, 3 },
    { 0x8f723b30, 3 },
    { 0xe0f63b2f, 3 },
    { 0xb7863b32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1963b37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b63b3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x745a3b40, 3 },
    { 0, 0 },
    { 0x99763b42, 3 },
    { 0x1d1e3b43, 3 },
    { 0xd4623b44, 3 },
    { 0x510a3b44, 4 },
    { 0x5c1e3b46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c23b49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35963b4e, 3 },
    { 0x6efa3b4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f023b57, 3 },
    { 0, 0 },
    { 0x71063b59, 3 },
    { 0, 0 },
    { 0xdf4e3b5b, 3 },
    { 0x60663b5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ae3b5f, 3 },
    { 0xb2123b5f, 3 },
    { 0xc7a63b61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd63b69, 3 },
    { 0xaece3b6a, 4 },
    { 0x2e2e3b69, 3 },
    { 0xb9e3b6c, 3 },
    { 0xaef63b6c, 3 },
    { 0x3bda3b6c, 4 },
    { 0x4ee3b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac23b72, 3 },
    { 0x71c63b72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4463b77, 3 },
    { 0, 0 },
    { 0x5bea3b79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14463b7c, 3 },
    { 0x2a5e3b7d, 3 },
    { 0x4a863b7d, 5 },
    { 0xbf563b7f, 3 },
    { 0xd8223b7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e63b83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9e3b87, 4 },
    { 0xac863b87, 3 },
    { 0, 0 },
    { 0xfcf63b8a, 3 },
    { 0xcb223b8b, 3 },
    { 0, 0 },
    { 0xe4aa3b8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21063b94, 3 },
    { 0x3cd23b94, 3 },
    { 0xf5263b95, 3 },
    { 0xca7a3b97, 3 },
    { 0x8ec63b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6a3b9c, 4 },
    { 0x93763b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3a3ba0, 4 },
    { 0x960e3ba0, 3 },
    { 0x6ade3ba2, 5 },
    { 0x74fa3ba2, 3 },
    { 0x81223ba1, 3 },
    { 0xefa3ba2, 3 },
    { 0x786e3ba2, 3 },
    { 0xe7763ba7, 3 },
    { 0xca4e3ba8, 3 },
    { 0x6d223ba9, 3 },
    { 0x78aa3ba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe63bae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f63bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78da3bb5, 3 },
    { 0xe723bb5, 3 },
    { 0xddc63bb5, 3 },
    { 0, 0 },
    { 0xe96a3bb9, 3 },
    { 0xe5ae3bba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18163bc1, 4 },
    { 0x28de3bc2, 4 },
    { 0xc68a3bc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906a3bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ca3bca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c23bce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a23bd2, 3 },
    { 0xc66e3bd3, 3 },
    { 0xefa23bd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aa3bd7, 3 },
    { 0x356a3bd8, 4 },
    { 0, 0 },
    { 0xbc263bda, 4 },
    { 0xd93a3bda, 3 },
    { 0x23c23bdc, 4 },
    { 0xc863bdc, 4 },
    { 0x9e4e3bdc, 3 },
    { 0xb9223bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a23be7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ae3bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6523bed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66763bf3, 3 },
    { 0, 0 },
    { 0x8ad23bf5, 3 },
    { 0, 0 },
    { 0xad323bf7, 3 },
    { 0xd9da3bf8, 4 },
    { 0, 0 },
    { 0xb9063bfa, 4 },
    { 0, 0 },
    { 0xad1e3bfc, 3 },
    { 0, 0 },
    { 0x48e3bfe, 5 },
    { 0, 0 },
    { 0xcf8e3c00, 3 },
    { 0x104e3c01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae3c04, 3 },
    { 0x976a3c05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ae3c08, 3 },
    { 0xc37a3c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe63c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5023c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37123c20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88e3c23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59123c29, 3 },
    { 0, 0 },
    { 0xe74e3c2b, 3 },
    { 0xf7e63c2c, 3 },
    { 0x52723c2d, 3 },
    { 0x8b663c2e, 3 },
    { 0x39023c2c, 3 },
    { 0xcc23c30, 3 },
    { 0, 0 },
    { 0xf8e3c32, 4 },
    { 0x16ee3c33, 3 },
    { 0xd05a3c33, 4 },
    { 0xc3323c33, 3 },
    { 0, 0 },
    { 0xcdbe3c37, 4 },
    { 0x3be63c38, 3 },
    { 0x5f9a3c39, 3 },
    { 0x22fa3c3a, 4 },
    { 0x3e7e3c39, 3 },
    { 0xc9663c3c, 3 },
    { 0x57c63c3d, 4 },
    { 0x64623c3e, 3 },
    { 0xc1823c3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f823c48, 3 },
    { 0x38823c49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e3c4d, 4 },
    { 0xe06e3c4d, 4 },
    { 0xb51e3c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a623c5c, 3 },
    { 0xece63c5d, 3 },
    { 0x57f23c5c, 3 },
    { 0, 0 },
    { 0x76723c60, 4 },
    { 0xf4fe3c61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29963c65, 3 },
    { 0, 0 },
    { 0x22d63c67, 5 },
    { 0, 0 },
    { 0xde9a3c69, 3 },
    { 0xbf8e3c6a, 3 },
    { 0xb2863c6b, 4 },
    { 0xe61e3c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8e3c6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631a3c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba923c7b, 3 },
    { 0x1ae63c7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c663c82, 3 },
    { 0x34da3c82, 3 },
    { 0, 0 },
    { 0x8b663c85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37623c88, 3 },
    { 0x97963c89, 3 },
    { 0xd7ee3c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e3c90, 3 },
    { 0x92c23c90, 3 },
    { 0xcdca3c92, 3 },
    { 0x23c23c93, 4 },
    { 0x8dfe3c94, 3 },
    { 0x33823c94, 4 },
    { 0x9eda3c96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa3c9c, 3 },
    { 0x92ca3c9d, 4 },
    { 0x182e3c9c, 3 },
    { 0x940a3c9f, 3 },
    { 0xd77a3ca0, 4 },
    { 0x61e23ca0, 3 },
    { 0xbb623ca2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a3ca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca3ca8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906a3cb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2e3cb5, 3 },
    { 0x825e3cb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a3cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16323cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b23cc8, 4 },
    { 0x97823cc8, 3 },
    { 0x2d6e3cca, 3 },
    { 0x688a3ccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77863ccf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a23cd2, 3 },
    { 0x8bba3cd2, 4 },
    { 0xdcee3cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ae3cd7, 3 },
    { 0x5ca23cd8, 3 },
    { 0, 0 },
    { 0xfcba3cda, 3 },
    { 0x69fe3cda, 3 },
    { 0xef563cdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2463cdf, 4 },
    { 0x54c23ce0, 3 },
    { 0x25163ce1, 3 },
    { 0, 0 },
    { 0x80823ce3, 4 },
    { 0x6d963ce4, 3 },
    { 0xa8e3ce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dce3cec, 3 },
    { 0x90ce3ced, 5 },
    { 0x92563cee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59563cf1, 3 },
    { 0x5dce3cf2, 3 },
    { 0x119e3cf3, 4 },
    { 0, 0 },
    { 0xa96a3cf5, 3 },
    { 0, 0 },
    { 0x9fbe3cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f23cfa, 3 },
    { 0xd8f63cfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c023d00, 3 },
    { 0, 0 },
    { 0xeb923d02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb63d0a, 3 },
    { 0x7c023d0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d63d11, 3 },
    { 0xb7ae3d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae63d1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07a3d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a3d27, 3 },
    { 0x477e3d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7223d2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe3d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e23d33, 3 },
    { 0, 0 },
    { 0x28563d35, 3 },
    { 0x56423d36, 4 },
    { 0xb663d37, 4 },
    { 0, 0 },
    { 0x5123d39, 3 },
    { 0, 0 },
    { 0x37863d3b, 3 },
    { 0x6c063d3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38a3d3f, 3 },
    { 0xe28a3d40, 4 },
    { 0, 0 },
    { 0x6d5a3d42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8623d46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68363d4a, 3 },
    { 0x983e3d4b, 3 },
    { 0xd86a3d4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63d4f, 3 },
    { 0xded63d50, 4 },
    { 0x79a23d51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x645e3d54, 3 },
    { 0x1e3e3d54, 3 },
    { 0x5b063d56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9763d5a, 4 },
    { 0xaeca3d5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4a3d64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x394e3d67, 4 },
    { 0x7e9e3d68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x891e3d6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e3d6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a63d72, 3 },
    { 0x2d123d73, 4 },
    { 0x68ba3d74, 3 },
    { 0, 0 },
    { 0xbba63d76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419a3d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae63d7e, 4 },
    { 0x250e3d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b563d85, 3 },
    { 0x2b5a3d86, 4 },
    { 0xccba3d87, 3 },
    { 0x5c4a3d88, 3 },
    { 0x63623d88, 4 },
    { 0xb7023d89, 4 },
    { 0x28f63d8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be3d8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29263d93, 3 },
    { 0xec2a3d93, 3 },
    { 0x78f63d93, 3 },
    { 0x6ec63d96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad023d9f, 3 },
    { 0x12ca3da0, 3 },
    { 0xe3163da1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17223da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffe3da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14a3dac, 5 },
    { 0, 0 },
    { 0x334a3dae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae3db3, 3 },
    { 0xd0163db4, 3 },
    { 0x63b63db5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aba3db8, 3 },
    { 0x13523db9, 3 },
    { 0x4a223dba, 3 },
    { 0xeb8e3dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29863dc1, 3 },
    { 0xf44a3dc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e3dc5, 3 },
    { 0x30363dc6, 4 },
    { 0xbf9e3dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67163dcc, 3 },
    { 0x66a3dcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db63dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a3dd6, 3 },
    { 0, 0 },
    { 0xd0a3dd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0463ddb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afa3ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd263de2, 3 },
    { 0xe1863de3, 3 },
    { 0x79ba3de2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768e3dea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82a3df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8463df6, 3 },
    { 0x654a3df7, 4 },
    { 0x3c3a3df7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f763e02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ce3e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55823e0e, 3 },
    { 0x86ea3e0f, 3 },
    { 0x55463e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x675a3e13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a363e18, 3 },
    { 0, 0 },
    { 0xd4a3e1a, 3 },
    { 0x7a3e3e1b, 4 },
    { 0x84323e1c, 3 },
    { 0x9b7e3e1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9e3e21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f63e27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648e3e2c, 3 },
    { 0, 0 },
    { 0x97c63e2e, 3 },
    { 0, 0 },
    { 0x1fd63e30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a3e38, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d63e3b, 3 },
    { 0, 0 },
    { 0xc82a3e3d, 3 },
    { 0xcd0e3e3e, 3 },
    { 0, 0 },
    { 0xd49e3e40, 3 },
    { 0, 0 },
    { 0x9a323e42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daa3e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a63e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef963e5f, 3 },
    { 0x79863e60, 3 },
    { 0xc7e3e61, 3 },
    { 0xba323e62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e3e67, 5 },
    { 0x8fb63e67, 3 },
    { 0x9c923e67, 3 },
    { 0, 0 },
    { 0x34963e6b, 3 },
    { 0xcfbe3e6c, 3 },
    { 0x55ba3e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4e3e71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e3e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf863e7b, 3 },
    { 0x274e3e7b, 3 },
    { 0xb69a3e7d, 4 },
    { 0xe9923e7e, 3 },
    { 0xdf1a3e7e, 4 },
    { 0x9d623e7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f63e85, 3 },
    { 0, 0 },
    { 0xde323e87, 3 },
    { 0, 0 },
    { 0x64da3e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3e3e8e, 3 },
    { 0x2b9a3e8f, 5 },
    { 0x1b6a3e8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b23e93, 3 },
    { 0x5f23e94, 3 },
    { 0x211a3e95, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de3e99, 3 },
    { 0xdeda3e99, 3 },
    { 0x5b2e3e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144e3ea0, 3 },
    { 0x6cde3ea1, 3 },
    { 0, 0 },
    { 0xa9ca3ea3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d23ea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3263eab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71a3eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ee3eb7, 3 },
    { 0x21d63eb8, 3 },
    { 0, 0 },
    { 0xc0863eba, 3 },
    { 0x57c63ebb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e3ebf, 3 },
    { 0xe69e3ebf, 3 },
    { 0, 0 },
    { 0x7d9e3ec2, 3 },
    { 0, 0 },
    { 0x606a3ec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e3ec7, 3 },
    { 0x99c23ec8, 3 },
    { 0x2a463ec7, 3 },
    { 0x13b23eca, 3 },
    { 0xea3e3ecb, 4 },
    { 0xe7223ecc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8263ed1, 3 },
    { 0x7b523ed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c23ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21a3ede, 3 },
    { 0, 0 },
    { 0xc1363ee0, 3 },
    { 0x63da3ee1, 3 },
    { 0x7d623ee2, 3 },
    { 0xc0f63ee2, 3 },
    { 0x61563ee4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8e3eed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae3ef1, 3 },
    { 0, 0 },
    { 0x410e3ef3, 3 },
    { 0x6d7a3ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea3ef7, 3 },
    { 0, 0 },
    { 0x21e63ef9, 3 },
    { 0x746a3ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3963efd, 3 },
    { 0, 0 },
    { 0x102e3eff, 3 },
    { 0, 0 },
    { 0x459a3f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba23f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65363f0b, 4 },
    { 0x2f8a3f0b, 4 },
    { 0x914a3f0d, 3 },
    { 0x1ec23f0d, 3 },
    { 0x71763f0e, 3 },
    { 0x40ae3f0d, 3 },
    { 0, 0 },
    { 0x73d63f12, 3 },
    { 0x81723f13, 3 },
    { 0x626e3f14, 3 },
    { 0x95b23f14, 5 },
    { 0xb71e3f16, 3 },
    { 0xb15a3f17, 3 },
    { 0x43463f18, 3 },
    { 0xb9323f17, 5 },
    { 0x27363f1a, 3 },
    { 0x1bd23f1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe3f1f, 3 },
    { 0x12be3f20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25723f23, 4 },
    { 0, 0 },
    { 0xaada3f25, 4 },
    { 0x88a23f26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6e3f2d, 4 },
    { 0, 0 },
    { 0xa5ca3f2f, 3 },
    { 0x5c023f30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacae3f34, 4 },
    { 0, 0 },
    { 0xe9963f36, 3 },
    { 0, 0 },
    { 0xa44a3f38, 3 },
    { 0x19623f38, 3 },
    { 0x5e4a3f3a, 3 },
    { 0xa6e3f3b, 4 },
    { 0x85823f3a, 4 },
    { 0x7cca3f3a, 3 },
    { 0xf48e3f3e, 3 },
    { 0, 0 },
    { 0x31da3f40, 4 },
    { 0xab2e3f41, 3 },
    { 0xd9763f42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dca3f4a, 3 },
    { 0, 0 },
    { 0xb7b63f4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53363f50, 3 },
    { 0x7f423f51, 3 },
    { 0, 0 },
    { 0xb7623f53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b63f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ca3f5a, 4 },
    { 0xe1d63f5a, 3 },
    { 0xbce23f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa3f60, 4 },
    { 0x764e3f61, 3 },
    { 0x7fd23f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71223f68, 3 },
    { 0x2b123f68, 3 },
    { 0, 0 },
    { 0xe8723f6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b63f71, 3 },
    { 0, 0 },
    { 0x56163f73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a63f78, 3 },
    { 0x99563f78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e23f83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fe3f8e, 3 },
    { 0, 0 },
    { 0x99de3f90, 3 },
    { 0x908a3f91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ece3f96, 4 },
    { 0xc61a3f97, 3 },
    { 0xaf0e3f98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fa3f9c, 3 },
    { 0xf463f9c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76e3fa0, 3 },
    { 0xe48e3fa1, 3 },
    { 0x35923fa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338a3fa6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2a3faa, 3 },
    { 0x749a3fab, 3 },
    { 0xb0023fab, 3 },
    { 0x3d963fad, 4 },
    { 0xc3b23fae, 4 },
    { 0xafb23fae, 3 },
    { 0x8ce23faf, 3 },
    { 0x2e163fb1, 4 },
    { 0xe4a3fac, 3 },
    { 0x415a3fb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a3fbb, 4 },
    { 0xdda3fbb, 3 },
    { 0x3e823fbc, 3 },
    { 0xe51e3fbe, 3 },
    { 0xbd8e3fbe, 3 },
    { 0x55ea3fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1a3fc4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca3fc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a3fcc, 3 },
    { 0x18fa3fcd, 3 },
    { 0, 0 },
    { 0xfc9e3fcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e23fd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x345a3fd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164a3fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9a3fdf, 4 },
    { 0, 0 },
    { 0xd8fa3fe1, 3 },
    { 0, 0 },
    { 0x85323fe3, 4 },
    { 0xf12e3fe4, 3 },
    { 0xa3b63fe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eca3feb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a863ff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29463ff9, 4 },
    { 0, 0 },
    { 0x77723ffb, 4 },
    { 0x30e63ffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4263fff, 3 },
    { 0x43124000, 4 },
    { 0x7fae3fff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ca4005, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c124009, 4 },
    { 0, 0 },
    { 0x8e8e400b, 4 },
    { 0, 0 },
    { 0xfafe400d, 3 },
    { 0x8b8e400d, 3 },
    { 0x60aa400f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ba4013, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3424016, 4 },
    { 0x9564016, 3 },
    { 0x6db64016, 3 },
    { 0xd4024018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a401d, 3 },
    { 0, 0 },
    { 0xc2ae401f, 4 },
    { 0, 0 },
    { 0xf7924021, 3 },
    { 0, 0 },
    { 0xe8864023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1924026, 3 },
    { 0x5ae24027, 3 },
    { 0x8d7a4028, 5 },
    { 0x128e4029, 4 },
    { 0, 0 },
    { 0x5b26402b, 4 },
    { 0x298a402c, 3 },
    { 0x3632402d, 4 },
    { 0x5302402e, 3 },
    { 0x5886402d, 3 },
    { 0, 0 },
    { 0xf8ea4031, 4 },
    { 0, 0 },
    { 0x3064033, 3 },
    { 0x41ee4034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8e4038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fa403c, 3 },
    { 0x503a403d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f24042, 3 },
    { 0x2824043, 3 },
    { 0, 0 },
    { 0xccc64045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4e4049, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af6404c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f24054, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ee4057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6405a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ca405d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f624061, 4 },
    { 0x502a4062, 3 },
    { 0x2a9a4061, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf964067, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae406f, 3 },
    { 0, 0 },
    { 0x46864071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b24075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ee4079, 3 },
    { 0x242407a, 3 },
    { 0x7866407b, 3 },
    { 0, 0 },
    { 0xdb42407d, 3 },
    { 0x6b56407d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fe4083, 3 },
    { 0, 0 },
    { 0x28ca4085, 3 },
    { 0, 0 },
    { 0xb7c24087, 4 },
    { 0, 0 },
    { 0x874e4089, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385e408f, 4 },
    { 0x73fa4090, 4 },
    { 0xd4424091, 3 },
    { 0xe3264092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a4095, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce4099, 3 },
    { 0x28b2409a, 3 },
    { 0x797e4099, 3 },
    { 0, 0 },
    { 0x95ae409d, 4 },
    { 0xe92e409e, 3 },
    { 0x7b62409f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda40a3, 3 },
    { 0x69f640a3, 3 },
    { 0x147a40a3, 3 },
    { 0, 0 },
    { 0x36240a7, 3 },
    { 0, 0 },
    { 0xf7ea40a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1240ac, 4 },
    { 0x78a640ad, 3 },
    { 0x776240ae, 3 },
    { 0, 0 },
    { 0xf45e40b0, 4 },
    { 0, 0 },
    { 0x1fce40b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x278e40b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477640bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x831a40bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24240c9, 3 },
    { 0, 0 },
    { 0x334e40cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a40ce, 3 },
    { 0x841a40cf, 4 },
    { 0, 0 },
    { 0xe19e40d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6e40d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5240dc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b240e4, 4 },
    { 0, 0 },
    { 0x113e40e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e40ea, 3 },
    { 0x49e40ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06640f3, 3 },
    { 0xbe3240f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a640f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652a40fd, 3 },
    { 0xe1b240fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ce4104, 4 },
    { 0xf3ca4105, 3 },
    { 0x1e024105, 4 },
    { 0xd74a4107, 3 },
    { 0x7dba4108, 3 },
    { 0x56ee4109, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f26410c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc24110, 3 },
    { 0x329e4110, 3 },
    { 0x51464110, 5 },
    { 0, 0 },
    { 0x6bda4114, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6124119, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf882411f, 4 },
    { 0x475e4120, 3 },
    { 0xc04a411f, 3 },
    { 0x897a4122, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0264128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ee412b, 3 },
    { 0, 0 },
    { 0xeaca412d, 3 },
    { 0xcfae412d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43da4133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6a413a, 3 },
    { 0, 0 },
    { 0x6422413c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31724140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ce4143, 4 },
    { 0x209a4144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8a4147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fde414e, 4 },
    { 0x9b42414e, 4 },
    { 0x21424150, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x932e4153, 4 },
    { 0, 0 },
    { 0x607e4155, 3 },
    { 0x2d9a4155, 3 },
    { 0x277e4155, 3 },
    { 0, 0 },
    { 0xc8b24159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846e415f, 3 },
    { 0, 0 },
    { 0x1e824161, 3 },
    { 0x9b264162, 3 },
    { 0x66124162, 3 },
    { 0xbf464163, 3 },
    { 0x23064165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16416c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a4170, 4 },
    { 0, 0 },
    { 0x51124172, 3 },
    { 0x9a0e4173, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf24179, 4 },
    { 0, 0 },
    { 0xcb16417b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4764180, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff64183, 3 },
    { 0xaa5e4184, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d24189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fa418d, 3 },
    { 0x896a418e, 4 },
    { 0xad2a418d, 3 },
    { 0, 0 },
    { 0x13ee4191, 4 },
    { 0x314a4192, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x468a4195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5824199, 4 },
    { 0, 0 },
    { 0xfdde419b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536641a8, 3 },
    { 0, 0 },
    { 0xd72a41aa, 3 },
    { 0xfb4e41ab, 3 },
    { 0, 0 },
    { 0xeb0e41ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655a41b1, 3 },
    { 0x100641b1, 3 },
    { 0x50ee41b3, 4 },
    { 0xba2e41b4, 3 },
    { 0, 0 },
    { 0x23ea41b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73241bb, 4 },
    { 0x808e41bb, 3 },
    { 0x72da41bc, 4 },
    { 0x3eda41be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41a41c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d241c5, 4 },
    { 0, 0 },
    { 0x404641c7, 4 },
    { 0, 0 },
    { 0x6e5641c9, 4 },
    { 0x60a241c9, 3 },
    { 0xf9aa41cb, 3 },
    { 0x44f241ca, 4 },
    { 0xd2de41cc, 3 },
    { 0x403a41cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e241d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x193e41d4, 4 },
    { 0, 0 },
    { 0xc73241d6, 3 },
    { 0, 0 },
    { 0xc0ea41d8, 5 },
    { 0xbf3e41d9, 3 },
    { 0x6ca641da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563e41df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61641e3, 4 },
    { 0x20fe41e3, 3 },
    { 0xb3e241e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab641ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2a41f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665a41f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241241f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe41ff, 3 },
    { 0x798241ff, 3 },
    { 0x703241ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad24206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6420b, 3 },
    { 0x2d3a420c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96024212, 3 },
    { 0x3db24212, 3 },
    { 0x2f964213, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce4219, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1be421c, 3 },
    { 0xade2421c, 4 },
    { 0x3f7e421e, 3 },
    { 0xc096421d, 3 },
    { 0x40fa4220, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6424224, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2422a, 3 },
    { 0x4dde422a, 3 },
    { 0x6b6a422c, 3 },
    { 0x79de422d, 3 },
    { 0xab56422e, 3 },
    { 0xd9d2422e, 3 },
    { 0x360a4230, 3 },
    { 0x283e4231, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18e4235, 3 },
    { 0x4e464235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876423b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60524247, 4 },
    { 0x80024248, 3 },
    { 0xa22e4249, 3 },
    { 0xdd2e4248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b2424d, 4 },
    { 0x61d6424e, 3 },
    { 0x7a82424e, 3 },
    { 0, 0 },
    { 0x7f524251, 3 },
    { 0xa7864251, 3 },
    { 0, 0 },
    { 0xaa6a4254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa4258, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd782425d, 3 },
    { 0, 0 },
    { 0x388a425f, 3 },
    { 0xdf0e425f, 4 },
    { 0x25764261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a24264, 3 },
    { 0xc45e4264, 5 },
    { 0x6e4266, 3 },
    { 0xdd64265, 3 },
    { 0xb3924268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e426c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec24271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93e4274, 3 },
    { 0xc90a4275, 3 },
    { 0x9dc24276, 3 },
    { 0, 0 },
    { 0xec9a4278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a62427c, 4 },
    { 0x16427d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1324282, 3 },
    { 0x728a4283, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efe428d, 4 },
    { 0, 0 },
    { 0x2c42428f, 3 },
    { 0x54aa428f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36324295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf32429a, 3 },
    { 0, 0 },
    { 0x958a429c, 4 },
    { 0x9aae429d, 3 },
    { 0xf01e429d, 5 },
    { 0xc5f2429f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76a42a2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63a42a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1e42a9, 3 },
    { 0x39fe42a9, 4 },
    { 0x930a42aa, 3 },
    { 0x7de242a9, 3 },
    { 0xd46a42ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6642b1, 4 },
    { 0xab0e42b2, 4 },
    { 0xe6aa42b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afa42b7, 3 },
    { 0xfa7a42b8, 3 },
    { 0x88a42b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6e42bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135e42c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ce42ca, 3 },
    { 0xa21a42ca, 4 },
    { 0xfda242cb, 3 },
    { 0xcb3e42cc, 3 },
    { 0x6aa242ce, 3 },
    { 0x8cee42cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657642d4, 3 },
    { 0x7efe42d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x832e42d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa42df, 3 },
    { 0, 0 },
    { 0xcb6242e1, 3 },
    { 0xd3b242e1, 3 },
    { 0x268a42e1, 3 },
    { 0x586642e3, 3 },
    { 0x2d5e42e1, 3 },
    { 0, 0 },
    { 0xcc7642e7, 3 },
    { 0, 0 },
    { 0xcbea42e9, 3 },
    { 0xf42e42ea, 3 },
    { 0, 0 },
    { 0x8b7642ec, 3 },
    { 0xee0242ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f242f0, 4 },
    { 0, 0 },
    { 0x967e42f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73642f6, 3 },
    { 0x51ae42f7, 4 },
    { 0x282242f8, 3 },
    { 0xc49e42f8, 3 },
    { 0xf61642f8, 3 },
    { 0x67ca42f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa4300, 4 },
    { 0xba564301, 3 },
    { 0x334a4300, 4 },
    { 0, 0 },
    { 0xf12a4304, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a430a, 3 },
    { 0x428a430b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefa4311, 3 },
    { 0, 0 },
    { 0x4c524313, 4 },
    { 0, 0 },
    { 0xbfca4315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2a4318, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf286431e, 3 },
    { 0x7d12431f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83324324, 3 },
    { 0, 0 },
    { 0xffb64326, 3 },
    { 0x458a4327, 3 },
    { 0x625a4328, 3 },
    { 0xb93e4328, 4 },
    { 0, 0 },
    { 0x619e432b, 4 },
    { 0, 0 },
    { 0xcca2432d, 3 },
    { 0x586e432e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a4331, 3 },
    { 0x1be24332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbe4339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a433e, 3 },
    { 0xb96a433f, 3 },
    { 0xe03e4340, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x70264343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68e4346, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ee434b, 4 },
    { 0xdbae434c, 3 },
    { 0x838e434c, 3 },
    { 0x8c2434e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27664351, 5 },
    { 0, 0 },
    { 0x12c64353, 3 },
    { 0, 0 },
    { 0x4064355, 3 },
    { 0x2eae4355, 3 },
    { 0xb3764355, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ca435e, 3 },
    { 0x59a6435f, 3 },
    { 0x93b2435e, 3 },
    { 0xdbb64360, 3 },
    { 0xe8e4360, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b324366, 4 },
    { 0xf5ba4367, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57be436e, 3 },
    { 0xf76436e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a4373, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4964378, 3 },
    { 0x8f964379, 3 },
    { 0x70564379, 3 },
    { 0xa0ba437b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a4384, 3 },
    { 0x712a4384, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ae438a, 4 },
    { 0, 0 },
    { 0x1dc6438c, 3 },
    { 0x4bf2438d, 4 },
    { 0x5476438e, 4 },
    { 0x4b9a438e, 4 },
    { 0x726e4390, 3 },
    { 0x4c264391, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ea4394, 4 },
    { 0xf7ea4395, 3 },
    { 0xa3424396, 3 },
    { 0x64024394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fda439a, 3 },
    { 0xb836439a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a439f, 3 },
    { 0, 0 },
    { 0xdf1243a1, 3 },
    { 0xd5d243a1, 3 },
    { 0x59d643a3, 3 },
    { 0, 0 },
    { 0x8ade43a5, 3 },
    { 0xdd9243a5, 4 },
    { 0, 0 },
    { 0xa2ca43a8, 3 },
    { 0x1f8243a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af243b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906e43b7, 3 },
    { 0x795643b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dba43bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e43be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5643c1, 3 },
    { 0xdcda43c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5e43c9, 3 },
    { 0, 0 },
    { 0xfbfe43cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x316643ce, 3 },
    { 0, 0 },
    { 0xfbaa43d0, 3 },
    { 0, 0 },
    { 0x23ee43d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5243d5, 4 },
    { 0x50a643d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380643db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5e43df, 3 },
    { 0xe9c643e0, 3 },
    { 0xbe7643df, 3 },
    { 0, 0 },
    { 0xa41643e3, 3 },
    { 0x70fa43e4, 4 },
    { 0x1ea43e5, 4 },
    { 0x18ba43e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a43e9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fe43ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c243ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce43f7, 4 },
    { 0, 0 },
    { 0xc3643f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17e43ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c64402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e524405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6a4409, 4 },
    { 0x6b76440a, 3 },
    { 0x6b86440a, 3 },
    { 0, 0 },
    { 0x471a440d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaa4410, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b64415, 3 },
    { 0xb5f24416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bca441c, 3 },
    { 0x3fe441c, 3 },
    { 0, 0 },
    { 0xf4de441f, 3 },
    { 0, 0 },
    { 0x38fe4421, 4 },
    { 0x5764422, 4 },
    { 0x63d24423, 3 },
    { 0xd71a4421, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa4428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f2442e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73564438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc102443c, 3 },
    { 0x6d86443d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ae4440, 3 },
    { 0x39264440, 5 },
    { 0xfe924442, 3 },
    { 0x8664443, 3 },
    { 0x1faa4443, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643a444b, 4 },
    { 0xacee444b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ea444f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b324452, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaea445a, 3 },
    { 0xecae445b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85264466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcca4472, 3 },
    { 0, 0 },
    { 0xa24474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea6447c, 3 },
    { 0, 0 },
    { 0x6c92447e, 3 },
    { 0x4396447f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fe4483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b24488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e448c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc64491, 3 },
    { 0x3af64491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3064497, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb92449e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa44a7, 3 },
    { 0x7f6244a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4244ad, 3 },
    { 0xafca44ae, 4 },
    { 0, 0 },
    { 0xc73a44b0, 3 },
    { 0x982644b1, 5 },
    { 0xf27a44b1, 3 },
    { 0x8c4644b3, 3 },
    { 0x8d6e44b4, 4 },
    { 0x30ca44b4, 4 },
    { 0xfaa44b5, 3 },
    { 0x490e44b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731e44c0, 3 },
    { 0xc1d644c1, 3 },
    { 0, 0 },
    { 0x162a44c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3244cb, 4 },
    { 0, 0 },
    { 0x599e44cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca44d1, 3 },
    { 0x5c7a44d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee44da, 3 },
    { 0, 0 },
    { 0xbb1244dc, 3 },
    { 0xec0e44dd, 3 },
    { 0x489644dc, 3 },
    { 0xb1644df, 4 },
    { 0x2e0244e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231244e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173a44ee, 3 },
    { 0x616a44ee, 3 },
    { 0, 0 },
    { 0xa23644f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16e44f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977244fa, 3 },
    { 0xa3be44fb, 3 },
    { 0x5b4a44fc, 3 },
    { 0xa82244fd, 3 },
    { 0x67ee44fb, 4 },
    { 0, 0 },
    { 0x50e4500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a24503, 4 },
    { 0xddf64504, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a4508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa492450b, 3 },
    { 0x1ace450c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe4511, 5 },
    { 0xbeca4512, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa286451a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f6451e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0e4522, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ca452f, 3 },
    { 0x52a6452f, 3 },
    { 0, 0 },
    { 0x88ba4532, 3 },
    { 0x9fa64533, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e24536, 4 },
    { 0x381e4537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee453c, 3 },
    { 0x5fea453c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44864542, 3 },
    { 0x2f0a4543, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca24550, 3 },
    { 0x4ad64551, 3 },
    { 0, 0 },
    { 0x2aae4553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a455a, 3 },
    { 0xabb2455b, 3 },
    { 0xdffe455a, 3 },
    { 0, 0 },
    { 0x5eba455e, 3 },
    { 0, 0 },
    { 0xd1ee4560, 3 },
    { 0xba3e4561, 3 },
    { 0x94a4562, 3 },
    { 0x3ec24562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72456a, 4 },
    { 0, 0 },
    { 0x519a456c, 3 },
    { 0, 0 },
    { 0x6366456e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9e4572, 4 },
    { 0x9a024572, 3 },
    { 0xec124574, 3 },
    { 0xecda4575, 3 },
    { 0xdb6a4576, 4 },
    { 0x38ca4576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a457f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456e4586, 3 },
    { 0, 0 },
    { 0x533a4588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c36458b, 3 },
    { 0xbcd6458c, 3 },
    { 0x4dfe458d, 5 },
    { 0x8c8e458c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf364591, 3 },
    { 0, 0 },
    { 0xfd8a4593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8424599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa45a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6645a5, 3 },
    { 0x8a3245a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1a45ab, 4 },
    { 0, 0 },
    { 0x8fba45ad, 3 },
    { 0x94b645ae, 4 },
    { 0xd5f245af, 4 },
    { 0xb18a45ae, 3 },
    { 0x854245ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48e45bc, 4 },
    { 0x84da45bc, 3 },
    { 0xcba645be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71245c1, 3 },
    { 0x327645c2, 3 },
    { 0xac8e45c3, 3 },
    { 0x7f8a45c4, 3 },
    { 0xad0e45c5, 3 },
    { 0, 0 },
    { 0x305a45c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a45cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x49be45d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3a45d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0245d7, 3 },
    { 0x39ba45d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5245df, 4 },
    { 0x129e45e0, 3 },
    { 0, 0 },
    { 0x6d9e45e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15a45e5, 3 },
    { 0x968a45e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a45eb, 4 },
    { 0x518645ec, 3 },
    { 0x390e45ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44e45f0, 3 },
    { 0xb38245f1, 3 },
    { 0x3cc245f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78245f7, 3 },
    { 0, 0 },
    { 0x328245f9, 3 },
    { 0x734e45fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e4600, 3 },
    { 0x4b0e4600, 3 },
    { 0xc7d64600, 3 },
    { 0, 0 },
    { 0x35464604, 3 },
    { 0xe2524605, 3 },
    { 0, 0 },
    { 0x53864607, 3 },
    { 0x8d724608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a64610, 3 },
    { 0xb6464611, 3 },
    { 0xfede4611, 3 },
    { 0xeb624611, 3 },
    { 0x80324614, 3 },
    { 0x8d924612, 3 },
    { 0xff1e4615, 3 },
    { 0x6124617, 4 },
    { 0xde7a4613, 4 },
    { 0x329a4615, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea461d, 4 },
    { 0x63fe461e, 4 },
    { 0xf22e461e, 3 },
    { 0xe3d64620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fe4623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fba4627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca462b, 3 },
    { 0x522e462c, 3 },
    { 0x2a0a462c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4a4631, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78164634, 3 },
    { 0, 0 },
    { 0xd564636, 4 },
    { 0x3d2a4637, 5 },
    { 0x198a4638, 3 },
    { 0xec124639, 3 },
    { 0xafda463a, 4 },
    { 0xdada463b, 3 },
    { 0x55a463c, 3 },
    { 0x5186463c, 3 },
    { 0x197e463b, 3 },
    { 0x5376463d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29e4642, 3 },
    { 0xdaf64642, 3 },
    { 0x53324644, 3 },
    { 0x64724645, 3 },
    { 0x73664643, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e464a, 3 },
    { 0, 0 },
    { 0xfa9a464c, 3 },
    { 0, 0 },
    { 0x40e6464e, 3 },
    { 0x778e464e, 3 },
    { 0x5ad6464f, 3 },
    { 0, 0 },
    { 0xb06a4652, 3 },
    { 0x8964653, 3 },
    { 0x9f2e4653, 3 },
    { 0xdc3a4653, 3 },
    { 0x221a4656, 3 },
    { 0, 0 },
    { 0xfbca4658, 3 },
    { 0xc5aa4659, 3 },
    { 0xc77e465a, 3 },
    { 0, 0 },
    { 0xe4b2465c, 3 },
    { 0x7ad6465d, 4 },
    { 0xe39a465e, 3 },
    { 0, 0 },
    { 0xe1e24660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6a466e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24672, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9964676, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274e467c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294e4682, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3524687, 3 },
    { 0xe7264688, 4 },
    { 0xf8ba4688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efe468e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd24694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf024698, 3 },
    { 0xbede4699, 3 },
    { 0, 0 },
    { 0x6fe469b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffe469f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe246a5, 3 },
    { 0, 0 },
    { 0x9bb646a7, 3 },
    { 0x29a246a8, 3 },
    { 0, 0 },
    { 0x32d646aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec246ae, 3 },
    { 0x9d0646af, 3 },
    { 0, 0 },
    { 0x730246b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cce46b6, 4 },
    { 0, 0 },
    { 0xec8646b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ca46bd, 3 },
    { 0, 0 },
    { 0x3dc646bf, 3 },
    { 0x11b646c0, 3 },
    { 0xb5c246bf, 3 },
    { 0x345e46c1, 5 },
    { 0xc05e46c1, 3 },
    { 0x3a4a46c4, 3 },
    { 0xd75e46c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x817246c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5e46cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4246d3, 3 },
    { 0, 0 },
    { 0xba4646d5, 4 },
    { 0x78b246d6, 4 },
    { 0x8b5e46d7, 3 },
    { 0xf12e46d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1a46ea, 3 },
    { 0x269246ea, 3 },
    { 0, 0 },
    { 0xe16e46ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x406a46f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff246f3, 3 },
    { 0x130646f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31646f9, 4 },
    { 0x396a46f9, 3 },
    { 0xb71e46fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x385246fe, 4 },
    { 0, 0 },
    { 0x20f24700, 3 },
    { 0, 0 },
    { 0x9f324702, 3 },
    { 0x4bf64703, 3 },
    { 0xe81e4702, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba764708, 3 },
    { 0xa2164709, 3 },
    { 0x9db64708, 3 },
    { 0x12a470b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7076470e, 3 },
    { 0x712e470f, 3 },
    { 0xe5de470f, 3 },
    { 0xb8724711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa4714, 3 },
    { 0xceee4715, 3 },
    { 0x6d124716, 3 },
    { 0x2daa4716, 3 },
    { 0xc17e4717, 3 },
    { 0xb9624716, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8a471d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab264723, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe64726, 3 },
    { 0xe6764727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7722472a, 4 },
    { 0x267e472b, 3 },
    { 0xacae472c, 3 },
    { 0xa8d2472d, 3 },
    { 0x386a472a, 4 },
    { 0xdfc2472d, 3 },
    { 0x3c8e472c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda24733, 3 },
    { 0x559e4734, 3 },
    { 0, 0 },
    { 0x92c64736, 3 },
    { 0x62a4737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2473b, 3 },
    { 0, 0 },
    { 0x3c4e473d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c364741, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a24746, 3 },
    { 0, 0 },
    { 0x6c064748, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd502474b, 3 },
    { 0xef36474c, 3 },
    { 0x3272474c, 4 },
    { 0, 0 },
    { 0xc82474f, 3 },
    { 0, 0 },
    { 0x9f964751, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9264754, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a4760, 3 },
    { 0xc8b24761, 3 },
    { 0, 0 },
    { 0xeaee4763, 4 },
    { 0x577a4763, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca476a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb324772, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b564775, 4 },
    { 0, 0 },
    { 0x5b3a4777, 4 },
    { 0, 0 },
    { 0xd84a4779, 3 },
    { 0x653e477a, 3 },
    { 0x7a3e477b, 3 },
    { 0x4bb2477b, 3 },
    { 0xf3f6477c, 4 },
    { 0xc666477e, 3 },
    { 0xc976477b, 3 },
    { 0x237a477a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1324783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b6478b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f2478f, 3 },
    { 0x3e4e4790, 3 },
    { 0x94c64790, 3 },
    { 0xdc2a4790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76064795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e479b, 4 },
    { 0, 0 },
    { 0xef22479d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9e47a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e647aa, 3 },
    { 0, 0 },
    { 0xef5e47ac, 3 },
    { 0xe89a47ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74647b2, 3 },
    { 0, 0 },
    { 0x642247b4, 4 },
    { 0x581e47b5, 4 },
    { 0, 0 },
    { 0x7ec247b7, 3 },
    { 0, 0 },
    { 0xad0e47b9, 3 },
    { 0, 0 },
    { 0x571e47bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x688e47be, 3 },
    { 0, 0 },
    { 0xd73247c0, 3 },
    { 0x684a47c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3247c5, 3 },
    { 0x14b647c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x906a47c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00a47cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a47d0, 3 },
    { 0xce2647d1, 3 },
    { 0, 0 },
    { 0xbf6e47d3, 3 },
    { 0xf0c247d3, 3 },
    { 0x40f247d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e47d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a47e2, 4 },
    { 0x52da47e2, 3 },
    { 0x389647e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1e47ea, 5 },
    { 0xefe647eb, 3 },
    { 0x783a47ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x793a47ef, 3 },
    { 0x733247ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04247f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef247f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8e47fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e4800, 4 },
    { 0, 0 },
    { 0x9eae4802, 3 },
    { 0x62ca4803, 4 },
    { 0, 0 },
    { 0x20824805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4a4808, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6480b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43724811, 3 },
    { 0xc0424812, 3 },
    { 0x32ae4812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924816, 4 },
    { 0x5eb24816, 3 },
    { 0, 0 },
    { 0xdf024819, 3 },
    { 0x7c66481a, 3 },
    { 0x8aaa481b, 3 },
    { 0x23d2481c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6466481f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76364824, 4 },
    { 0, 0 },
    { 0xce664826, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7892482a, 3 },
    { 0x13ce482b, 3 },
    { 0xb386482a, 3 },
    { 0x8eca482d, 3 },
    { 0xa37a482d, 3 },
    { 0, 0 },
    { 0xb8f24830, 4 },
    { 0x23624831, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c24834, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e64837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf406483b, 3 },
    { 0x7c6a483b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a4842, 3 },
    { 0xb59e4843, 3 },
    { 0x317e4844, 3 },
    { 0x1ef64843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x950a4848, 3 },
    { 0xc4364848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fe484c, 3 },
    { 0xa18a484d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52364856, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f2485a, 3 },
    { 0x6dbe485b, 3 },
    { 0x6206485c, 3 },
    { 0x920a485d, 3 },
    { 0xd956485e, 3 },
    { 0, 0 },
    { 0x20d24860, 4 },
    { 0x1b864860, 3 },
    { 0, 0 },
    { 0x3d7a4863, 3 },
    { 0, 0 },
    { 0xcf624865, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2be4868, 3 },
    { 0, 0 },
    { 0xc5ca486a, 3 },
    { 0xf922486b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d66486e, 3 },
    { 0x3472486f, 3 },
    { 0x5fe2486f, 3 },
    { 0x1e264871, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8464875, 3 },
    { 0, 0 },
    { 0x32d24877, 3 },
    { 0xbe264878, 4 },
    { 0xcdb24879, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5a487e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e4881, 3 },
    { 0xd78a4882, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aa4886, 3 },
    { 0x66164886, 3 },
    { 0xd11a4888, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22e488c, 4 },
    { 0x6132488d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50324891, 3 },
    { 0, 0 },
    { 0x16b64893, 4 },
    { 0xdd424894, 4 },
    { 0x11224895, 3 },
    { 0xb224896, 3 },
    { 0, 0 },
    { 0x55464898, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86489c, 3 },
    { 0xf68a489c, 3 },
    { 0x6a0a489d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a648a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102648a5, 3 },
    { 0, 0 },
    { 0xebca48a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ba48aa, 3 },
    { 0x32d648aa, 3 },
    { 0x2b0e48ac, 3 },
    { 0x9eae48ad, 4 },
    { 0xe73648ae, 3 },
    { 0x759248af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa48b9, 4 },
    { 0xc16e48b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44648c2, 4 },
    { 0x4e1248c3, 3 },
    { 0, 0 },
    { 0x168e48c5, 3 },
    { 0xb83248c6, 4 },
    { 0x493248c7, 3 },
    { 0xc5e248c6, 5 },
    { 0x3a3a48c5, 3 },
    { 0xabee48c8, 3 },
    { 0x3f5248cb, 4 },
    { 0xe97a48c9, 3 },
    { 0xfdea48cd, 4 },
    { 0x52da48ce, 3 },
    { 0x5d9a48cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b648d3, 3 },
    { 0x644248d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e248d9, 3 },
    { 0, 0 },
    { 0xaf8a48db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce48e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539e48e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ce48ea, 3 },
    { 0xcd8a48eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x448648ee, 3 },
    { 0xe0f248ee, 3 },
    { 0xa45e48f0, 3 },
    { 0xae9648f1, 3 },
    { 0x61da48ee, 3 },
    { 0x2b1248f0, 4 },
    { 0x52f648f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda48f8, 4 },
    { 0x7fca48f8, 3 },
    { 0x7cde48fa, 3 },
    { 0xde5248fb, 3 },
    { 0xed5648fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea24901, 4 },
    { 0, 0 },
    { 0xa28a4903, 3 },
    { 0x6364904, 3 },
    { 0x347e4904, 3 },
    { 0xa80a4906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01a490d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4724911, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x263e4914, 4 },
    { 0x9bfe4915, 3 },
    { 0x5f5e4914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf66491a, 4 },
    { 0xc9ee491b, 3 },
    { 0, 0 },
    { 0x860e491d, 3 },
    { 0x793a491d, 3 },
    { 0xbb36491d, 3 },
    { 0xd2b24920, 4 },
    { 0x18be4921, 3 },
    { 0xf1624921, 3 },
    { 0xcc2a4923, 3 },
    { 0xb43e4923, 4 },
    { 0xc224925, 3 },
    { 0x4b7a4924, 3 },
    { 0x6ace4927, 3 },
    { 0x9a3e4928, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb66492b, 3 },
    { 0x2516492c, 3 },
    { 0, 0 },
    { 0x865a492e, 4 },
    { 0x5e56492e, 3 },
    { 0, 0 },
    { 0x3a824931, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ea4934, 3 },
    { 0xcf3a4935, 3 },
    { 0x3b964936, 3 },
    { 0, 0 },
    { 0xd67a4938, 3 },
    { 0xc38a4939, 3 },
    { 0xac724939, 3 },
    { 0x459a493b, 3 },
    { 0xf32e493c, 3 },
    { 0xda86493c, 4 },
    { 0x4006493b, 3 },
    { 0x3d52493c, 3 },
    { 0, 0 },
    { 0xe9da4941, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6424946, 3 },
    { 0, 0 },
    { 0xeca64948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a24950, 3 },
    { 0, 0 },
    { 0x97b64952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4a4957, 3 },
    { 0, 0 },
    { 0x16ee4959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb66495c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1566495f, 4 },
    { 0x5b2495f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be496a, 3 },
    { 0, 0 },
    { 0x8dae496c, 4 },
    { 0, 0 },
    { 0x31c6496e, 3 },
    { 0x46d6496f, 3 },
    { 0x7102496f, 3 },
    { 0xc51e4971, 3 },
    { 0, 0 },
    { 0x5a064973, 4 },
    { 0x37064974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d66497f, 3 },
    { 0xe296497f, 3 },
    { 0xe364981, 3 },
    { 0xc95e4982, 3 },
    { 0x46824983, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc524988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d6498b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5224999, 3 },
    { 0, 0 },
    { 0x4b9e499b, 3 },
    { 0xfec2499c, 3 },
    { 0x3d56499c, 4 },
    { 0x66a6499c, 3 },
    { 0x9bde499f, 3 },
    { 0x2ede499f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9a49ad, 3 },
    { 0, 0 },
    { 0x328e49af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352249b3, 4 },
    { 0xb0ce49b4, 4 },
    { 0x70f249b5, 3 },
    { 0x1c5a49b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9249ba, 3 },
    { 0xab4649ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f649bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4e49c2, 3 },
    { 0xa88a49c2, 3 },
    { 0x5a8649c4, 5 },
    { 0, 0 },
    { 0x274a49c6, 3 },
    { 0x18ea49c6, 3 },
    { 0x810e49c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37a49cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce249d3, 3 },
    { 0, 0 },
    { 0xf7da49d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e49d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15e49dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233249e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ee49e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6249ed, 3 },
    { 0xc649ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e49f2, 3 },
    { 0x149a49f3, 4 },
    { 0x755249f4, 3 },
    { 0x3e1e49f5, 3 },
    { 0xfdca49f6, 3 },
    { 0x202a49f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0364a04, 4 },
    { 0, 0 },
    { 0x9ace4a06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41364a09, 3 },
    { 0, 0 },
    { 0xa6d64a0b, 3 },
    { 0x6fc24a0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a4a0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387a4a17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca4a1c, 3 },
    { 0x1cde4a1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7824a24, 4 },
    { 0x49524a25, 3 },
    { 0, 0 },
    { 0x9bb64a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae464a2a, 3 },
    { 0, 0 },
    { 0x82e4a2c, 4 },
    { 0x10a4a2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a4a32, 4 },
    { 0xa33a4a33, 3 },
    { 0xa5da4a33, 3 },
    { 0x581a4a33, 4 },
    { 0x4e864a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2624a39, 3 },
    { 0, 0 },
    { 0x87de4a3b, 3 },
    { 0, 0 },
    { 0x938e4a3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74864a41, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x31564a44, 3 },
    { 0x14fe4a44, 3 },
    { 0xd4ea4a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160e4a4a, 3 },
    { 0x357a4a4b, 3 },
    { 0xed5a4a4b, 3 },
    { 0, 0 },
    { 0xd07e4a4e, 3 },
    { 0x1c4e4a4f, 3 },
    { 0x98a4a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a4a57, 3 },
    { 0xade64a58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502e4a63, 4 },
    { 0, 0 },
    { 0xf82e4a65, 3 },
    { 0x8ef64a66, 3 },
    { 0x94764a67, 3 },
    { 0x31e64a65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523e4a74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd24a78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ca4a7b, 3 },
    { 0xa35e4a7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58064a7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a64a85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de24a88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00a4a8b, 4 },
    { 0x74c64a8c, 3 },
    { 0, 0 },
    { 0x1aca4a8e, 3 },
    { 0x62de4a8f, 3 },
    { 0x7e664a8f, 3 },
    { 0x26be4a91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e64a95, 3 },
    { 0x379a4a96, 3 },
    { 0x8f0e4a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf64a9a, 3 },
    { 0x804a4a9a, 4 },
    { 0xdf124a9c, 4 },
    { 0, 0 },
    { 0x370e4a9e, 3 },
    { 0xa7be4a9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f724aa3, 3 },
    { 0, 0 },
    { 0xd6e64aa5, 3 },
    { 0x95364aa5, 3 },
    { 0x6a4a4aa7, 3 },
    { 0xd1ba4aa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9a4aac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ce4aaf, 3 },
    { 0x66ea4ab0, 3 },
    { 0x38564aaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x757a4ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48da4ab8, 4 },
    { 0x41124ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987e4ac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50be4ac9, 3 },
    { 0x627a4aca, 4 },
    { 0x45da4aca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640a4acf, 4 },
    { 0xe5564ad0, 3 },
    { 0xc4ea4acf, 3 },
    { 0x603a4ad1, 3 },
    { 0xf4ca4ad3, 4 },
    { 0x6d064ad4, 4 },
    { 0xdb2a4ad2, 4 },
    { 0xa7de4ad6, 3 },
    { 0x9fa4ad6, 3 },
    { 0x36464ad8, 3 },
    { 0xb20e4ad8, 3 },
    { 0, 0 },
    { 0x29ee4adb, 3 },
    { 0xa14e4adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04a4adf, 3 },
    { 0x28624ae0, 4 },
    { 0x26b24ae1, 3 },
    { 0xa2c24ae2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4a4aec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42164aef, 3 },
    { 0, 0 },
    { 0x6d0e4af1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e324af4, 3 },
    { 0x428a4af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9a4af8, 4 },
    { 0xd6b64af9, 4 },
    { 0, 0 },
    { 0x17464afb, 3 },
    { 0xdd8a4afc, 3 },
    { 0, 0 },
    { 0xe40e4afe, 3 },
    { 0, 0 },
    { 0x79864b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb424b04, 3 },
    { 0x65e64b05, 3 },
    { 0x56524b06, 4 },
    { 0x80ba4b07, 4 },
    { 0x16264b08, 3 },
    { 0x31be4b09, 3 },
    { 0x8aba4b09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e4b13, 4 },
    { 0, 0 },
    { 0x38524b15, 3 },
    { 0x1d064b16, 4 },
    { 0x47964b15, 3 },
    { 0x908e4b18, 4 },
    { 0xa70e4b16, 5 },
    { 0, 0 },
    { 0x9d64b1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5e4b1e, 3 },
    { 0x96724b1f, 3 },
    { 0xbcde4b1f, 3 },
    { 0, 0 },
    { 0x52be4b22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fa4b27, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aa4b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5e4b31, 3 },
    { 0, 0 },
    { 0x14724b33, 5 },
    { 0x748a4b33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4624b3c, 3 },
    { 0x6f624b3c, 4 },
    { 0, 0 },
    { 0x3d764b3f, 3 },
    { 0x843e4b40, 3 },
    { 0, 0 },
    { 0x967a4b42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afa4b45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f64b4c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1024b56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfe4b62, 3 },
    { 0x5d9e4b63, 5 },
    { 0x8dfe4b64, 3 },
    { 0, 0 },
    { 0x3ace4b66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32be4b6a, 3 },
    { 0, 0 },
    { 0x96564b6c, 4 },
    { 0, 0 },
    { 0x9b864b6e, 3 },
    { 0xb2ca4b6f, 3 },
    { 0xdc624b70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76a4b7c, 5 },
    { 0, 0 },
    { 0xfeda4b7e, 3 },
    { 0x7064b7f, 4 },
    { 0, 0 },
    { 0x722e4b81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588a4b85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a4b89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x962a4b8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95864b90, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2a4b93, 3 },
    { 0x73e24b94, 3 },
    { 0xb85a4b94, 3 },
    { 0x712e4b94, 3 },
    { 0xfea4b97, 3 },
    { 0xb36a4b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97e4b9c, 3 },
    { 0xa0e64b9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x693a4ba0, 3 },
    { 0x4d964ba0, 4 },
    { 0x33ba4ba2, 4 },
    { 0xbe564ba2, 3 },
    { 0xaf8a4ba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70464ba7, 3 },
    { 0xbdae4ba7, 3 },
    { 0xa7ce4ba9, 3 },
    { 0xf1ea4baa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e4bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da24bb1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a4bb4, 3 },
    { 0, 0 },
    { 0x8124bb6, 3 },
    { 0xb6d24bb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e24bbe, 3 },
    { 0x7224bbf, 3 },
    { 0xb49a4bc0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd64bc4, 3 },
    { 0x627e4bc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11824bc9, 3 },
    { 0xe0fa4bca, 3 },
    { 0xb7564bcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f24bcf, 4 },
    { 0x66fe4bcf, 4 },
    { 0xb72a4bcf, 3 },
    { 0, 0 },
    { 0x687e4bd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8724bdd, 3 },
    { 0x4ae24bdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e4be1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c24be6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x407e4be9, 3 },
    { 0, 0 },
    { 0x4c324beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x364a4bee, 3 },
    { 0, 0 },
    { 0xc48e4bf0, 3 },
    { 0, 0 },
    { 0x24fa4bf2, 3 },
    { 0x1124bf3, 3 },
    { 0, 0 },
    { 0x76f64bf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84064bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37264c04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83224c07, 3 },
    { 0, 0 },
    { 0x34424c09, 3 },
    { 0x87e64c0a, 3 },
    { 0x3df64c0b, 4 },
    { 0, 0 },
    { 0x9de4c0d, 3 },
    { 0x642a4c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c64c11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25a4c18, 3 },
    { 0x701a4c18, 3 },
    { 0x3ea64c1a, 3 },
    { 0x81224c1a, 3 },
    { 0x86f24c1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc364c21, 3 },
    { 0x782e4c21, 3 },
    { 0x4a624c23, 3 },
    { 0xe1024c21, 4 },
    { 0, 0 },
    { 0x27264c26, 4 },
    { 0, 0 },
    { 0x95ae4c28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4c2f, 3 },
    { 0, 0 },
    { 0x17624c31, 3 },
    { 0xed9e4c32, 4 },
    { 0, 0 },
    { 0x66324c34, 5 },
    { 0x248a4c35, 3 },
    { 0x506a4c36, 3 },
    { 0x14024c35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x84164c3a, 3 },
    { 0x4a9e4c3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a664c40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29364c43, 3 },
    { 0, 0 },
    { 0x9b924c45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4a4c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c24c4e, 4 },
    { 0x8bf24c4f, 4 },
    { 0x11324c50, 4 },
    { 0x4cb64c50, 3 },
    { 0x22864c4e, 4 },
    { 0, 0 },
    { 0xb2124c54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e4c5a, 3 },
    { 0xc6fa4c5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ae4c5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aa4c65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba4c69, 4 },
    { 0, 0 },
    { 0xae924c6b, 4 },
    { 0, 0 },
    { 0x5aba4c6d, 3 },
    { 0, 0 },
    { 0xd67e4c6f, 3 },
    { 0xc15e4c70, 3 },
    { 0, 0 },
    { 0x2fd64c72, 3 },
    { 0x17be4c72, 4 },
    { 0x52e64c73, 3 },
    { 0x9cca4c75, 4 },
    { 0x3dd24c76, 4 },
    { 0xd6be4c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd64c7a, 4 },
    { 0xf85e4c7b, 4 },
    { 0x25c24c7c, 4 },
    { 0xe28a4c7c, 3 },
    { 0x1fea4c7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbe4c84, 4 },
    { 0, 0 },
    { 0x15ea4c86, 3 },
    { 0xf3624c87, 3 },
    { 0xbcda4c88, 3 },
    { 0x9ab64c89, 3 },
    { 0xabda4c86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69364c92, 3 },
    { 0, 0 },
    { 0x35e24c94, 3 },
    { 0, 0 },
    { 0x57824c96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810a4c9a, 3 },
    { 0, 0 },
    { 0xe39e4c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5924c9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4764ca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56824caa, 3 },
    { 0, 0 },
    { 0x76224cac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f24cb0, 3 },
    { 0x700e4cb1, 3 },
    { 0x11d24cb2, 3 },
    { 0, 0 },
    { 0xb2264cb4, 4 },
    { 0x4a1e4cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1e4cb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f64cbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198e4cc0, 3 },
    { 0x81864cc1, 3 },
    { 0xeb1a4cc0, 3 },
    { 0xc5c24cc3, 3 },
    { 0xc1864cc3, 3 },
    { 0, 0 },
    { 0xdb4a4cc6, 3 },
    { 0x2f1a4cc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f64ccb, 4 },
    { 0, 0 },
    { 0xca8e4ccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e64cd2, 3 },
    { 0xc5a64cd3, 3 },
    { 0, 0 },
    { 0x4664cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28064cdc, 3 },
    { 0xacc64cdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb564ce1, 3 },
    { 0xdc724ce2, 4 },
    { 0, 0 },
    { 0x8d4a4ce4, 3 },
    { 0xe0aa4ce4, 3 },
    { 0xe8ea4ce5, 3 },
    { 0x1a4a4ce7, 3 },
    { 0x54124ce7, 4 },
    { 0x2e8a4ce8, 3 },
    { 0x9b24ce9, 3 },
    { 0xd42e4ce7, 3 },
    { 0xe8524ce9, 3 },
    { 0, 0 },
    { 0x7dea4cee, 3 },
    { 0xb82e4cef, 4 },
    { 0xc5a4cee, 3 },
    { 0x31924cf1, 3 },
    { 0x74c64cf1, 5 },
    { 0x94da4cf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b64cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a24cfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1964cff, 3 },
    { 0xe3fa4d00, 3 },
    { 0, 0 },
    { 0xdd0e4d02, 3 },
    { 0x52ba4d03, 3 },
    { 0x148a4d04, 3 },
    { 0x28f24d05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a4d0a, 4 },
    { 0xdd924d0a, 3 },
    { 0, 0 },
    { 0x16864d0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d464d1c, 3 },
    { 0, 0 },
    { 0xac524d1e, 3 },
    { 0x30164d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ace4d24, 4 },
    { 0xd2524d25, 4 },
    { 0, 0 },
    { 0x2d324d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a24d2b, 3 },
    { 0xbffe4d2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f64d2f, 3 },
    { 0x36f24d30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8e4d34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e4d38, 3 },
    { 0xf45a4d38, 3 },
    { 0, 0 },
    { 0x30624d3b, 4 },
    { 0, 0 },
    { 0xdace4d3d, 3 },
    { 0, 0 },
    { 0x34964d3f, 4 },
    { 0x56ae4d3f, 3 },
    { 0x723a4d3f, 5 },
    { 0x35f64d42, 3 },
    { 0x82c24d41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17524d46, 4 },
    { 0x8d124d46, 3 },
    { 0, 0 },
    { 0x1a5e4d49, 3 },
    { 0x24be4d4a, 3 },
    { 0xeb324d4b, 3 },
    { 0x29e4d4c, 3 },
    { 0xeeda4d4d, 3 },
    { 0xa34e4d4a, 3 },
    { 0, 0 },
    { 0x5eca4d50, 4 },
    { 0, 0 },
    { 0x86e24d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x502a4d55, 5 },
    { 0, 0 },
    { 0x23164d57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa264d5a, 4 },
    { 0xbe6e4d5b, 3 },
    { 0, 0 },
    { 0xdfa4d5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d64d60, 3 },
    { 0xf8ee4d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x988a4d64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0724d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150a4d6d, 5 },
    { 0, 0 },
    { 0xf3da4d6f, 4 },
    { 0, 0 },
    { 0xf8f64d71, 3 },
    { 0, 0 },
    { 0x403a4d73, 4 },
    { 0x45564d74, 3 },
    { 0, 0 },
    { 0xb8aa4d76, 4 },
    { 0x633a4d77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd24d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94924d7f, 3 },
    { 0, 0 },
    { 0x4ca64d81, 4 },
    { 0x66a64d82, 3 },
    { 0xb4b24d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e24d87, 3 },
    { 0x6a624d88, 3 },
    { 0x2f924d89, 4 },
    { 0, 0 },
    { 0xd4164d8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ca4d8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38064d99, 3 },
    { 0, 0 },
    { 0x7464d9b, 3 },
    { 0x189e4d9c, 3 },
    { 0x40c24d9c, 3 },
    { 0xf43a4d9e, 3 },
    { 0xa3da4d9e, 4 },
    { 0x9a2a4d9e, 4 },
    { 0x467a4da1, 4 },
    { 0x81824da2, 4 },
    { 0xd8a64da0, 3 },
    { 0xebe24da1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aa4dab, 3 },
    { 0x38464dac, 3 },
    { 0x69ca4dad, 3 },
    { 0xb2724dad, 4 },
    { 0x76fe4dad, 3 },
    { 0xebfe4db0, 3 },
    { 0x50624dae, 3 },
    { 0xd13e4db0, 3 },
    { 0xd4324db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a4db7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62964dba, 3 },
    { 0xc21a4dbb, 5 },
    { 0x6be64dbc, 3 },
    { 0x564dbd, 3 },
    { 0x922a4dbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ee4dc1, 3 },
    { 0x7eee4dc1, 3 },
    { 0, 0 },
    { 0x54664dc4, 3 },
    { 0, 0 },
    { 0x5cf64dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aae4dc9, 3 },
    { 0xe7424dca, 3 },
    { 0, 0 },
    { 0x43be4dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ee4dcf, 3 },
    { 0xcaaa4dd0, 3 },
    { 0, 0 },
    { 0x2f324dd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f724dd6, 3 },
    { 0x75264dd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83824ddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26564de0, 3 },
    { 0x537a4de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f064dea, 4 },
    { 0x7c5e4deb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50a4dee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55024df1, 3 },
    { 0, 0 },
    { 0xb1c64df3, 3 },
    { 0xcf2e4df3, 3 },
    { 0xc85a4df5, 3 },
    { 0x96f24df3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a4df9, 3 },
    { 0, 0 },
    { 0x72fa4dfb, 3 },
    { 0, 0 },
    { 0x7f2a4dfd, 3 },
    { 0x51b64dfd, 3 },
    { 0, 0 },
    { 0xf71a4e00, 3 },
    { 0, 0 },
    { 0x63ee4e02, 3 },
    { 0x3b3e4e03, 4 },
    { 0, 0 },
    { 0x3ffa4e05, 3 },
    { 0x356e4e06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80024e09, 3 },
    { 0xe60e4e0a, 3 },
    { 0x445e4e0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40a4e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4624e12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x932e4e15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a64e19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc64e1d, 4 },
    { 0, 0 },
    { 0xb8ba4e1f, 3 },
    { 0xb08a4e1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea4e25, 3 },
    { 0, 0 },
    { 0xcc64e27, 3 },
    { 0xb75a4e27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314e4e2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea4e3b, 3 },
    { 0xa5464e3b, 5 },
    { 0x1a9a4e3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2e4e42, 3 },
    { 0xcbca4e43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6724e47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e4e4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaee4e51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7324e55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb64e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aa4e61, 3 },
    { 0x8bba4e62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ba4e67, 3 },
    { 0x206a4e68, 3 },
    { 0xb7aa4e69, 3 },
    { 0x15224e6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe24e6e, 4 },
    { 0, 0 },
    { 0x7c164e70, 4 },
    { 0x75aa4e71, 4 },
    { 0, 0 },
    { 0x99924e73, 3 },
    { 0xfb224e73, 4 },
    { 0xefa24e73, 3 },
    { 0, 0 },
    { 0xe77a4e77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e324e7a, 3 },
    { 0xda764e7b, 5 },
    { 0xa5ba4e7c, 3 },
    { 0xe9064e7a, 3 },
    { 0, 0 },
    { 0xa7da4e7f, 4 },
    { 0x37fa4e80, 3 },
    { 0x3f964e80, 3 },
    { 0x7a124e80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e824e86, 3 },
    { 0x60b24e87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce4e8c, 5 },
    { 0xff124e8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a24e90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e24e93, 3 },
    { 0x64ea4e93, 3 },
    { 0, 0 },
    { 0x99324e96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1864e9a, 3 },
    { 0, 0 },
    { 0x6d624e9c, 4 },
    { 0, 0 },
    { 0x69e64e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e24ea4, 5 },
    { 0x92624ea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60424ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ce4eb3, 3 },
    { 0, 0 },
    { 0x7b564eb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca24eb8, 4 },
    { 0x918e4eb9, 4 },
    { 0x12ca4eb8, 4 },
    { 0, 0 },
    { 0x436e4ebc, 3 },
    { 0x34ba4ebc, 3 },
    { 0xbe6a4ebd, 3 },
    { 0xe8ae4ebd, 4 },
    { 0xcc964ec0, 3 },
    { 0, 0 },
    { 0x5d724ec2, 3 },
    { 0x2e9e4ec2, 3 },
    { 0x87624ec4, 3 },
    { 0xc8724ec4, 3 },
    { 0x8c224ec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca524ecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x487e4ece, 3 },
    { 0x1b664ecf, 3 },
    { 0x6f0e4ecf, 3 },
    { 0x82e4ed0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x475a4ed4, 3 },
    { 0xfdd64ed5, 3 },
    { 0xeac64ed6, 5 },
    { 0x4e2a4ed7, 3 },
    { 0xd3924ed6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a4edb, 3 },
    { 0x99fa4edc, 4 },
    { 0x6ada4edb, 3 },
    { 0xffd24ede, 3 },
    { 0, 0 },
    { 0x9fbe4ee0, 4 },
    { 0x39d64ee1, 5 },
    { 0x10864ee2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b24ee7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b764eef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38e4efa, 4 },
    { 0x8fa64efb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e24efe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0424f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e564f04, 3 },
    { 0, 0 },
    { 0x8d924f06, 4 },
    { 0x9bb64f07, 3 },
    { 0, 0 },
    { 0x6fda4f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fa4f0c, 4 },
    { 0xcee4f0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e4f14, 4 },
    { 0x919a4f15, 3 },
    { 0, 0 },
    { 0x89224f17, 4 },
    { 0x5f224f18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d724f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5be4f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562a4f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eea4f31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafba4f38, 3 },
    { 0xb6ae4f39, 3 },
    { 0xb8824f38, 3 },
    { 0, 0 },
    { 0x27024f3c, 3 },
    { 0xc1224f3c, 3 },
    { 0xd35e4f3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc264f44, 3 },
    { 0, 0 },
    { 0x7bc24f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b24f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696a4f54, 3 },
    { 0, 0 },
    { 0x99c24f56, 3 },
    { 0x5d8e4f57, 3 },
    { 0x58be4f58, 3 },
    { 0, 0 },
    { 0x737a4f5a, 4 },
    { 0x7a7a4f5b, 3 },
    { 0x852e4f5c, 4 },
    { 0x5a4f5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19e4f62, 3 },
    { 0, 0 },
    { 0x40964f64, 4 },
    { 0xd2624f65, 3 },
    { 0x78de4f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a4f6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827e4f6f, 3 },
    { 0, 0 },
    { 0xa73e4f71, 3 },
    { 0, 0 },
    { 0xd6ee4f73, 4 },
    { 0x2d0a4f74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3564f79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ba4f7d, 3 },
    { 0, 0 },
    { 0xb9064f7f, 4 },
    { 0xb8be4f80, 4 },
    { 0xa94e4f80, 4 },
    { 0x41fa4f82, 3 },
    { 0xf31e4f81, 3 },
    { 0x73aa4f84, 4 },
    { 0xa4be4f82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce64f8b, 3 },
    { 0, 0 },
    { 0xadee4f8d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d24f91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb64f96, 3 },
    { 0x5dca4f97, 4 },
    { 0x86a24f98, 3 },
    { 0x8f364f99, 4 },
    { 0, 0 },
    { 0x79ae4f9b, 3 },
    { 0, 0 },
    { 0xad064f9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aa4fa0, 3 },
    { 0, 0 },
    { 0x2d564fa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4fa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x117a4faa, 3 },
    { 0x62564faa, 3 },
    { 0x39624fab, 3 },
    { 0xc97a4fad, 3 },
    { 0, 0 },
    { 0x18ca4faf, 3 },
    { 0xece4fb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a4fb3, 3 },
    { 0xdbfe4fb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51624fb9, 3 },
    { 0x84fe4fba, 3 },
    { 0x2d524fbb, 3 },
    { 0x2a24fbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ea4fbf, 4 },
    { 0x70164fc0, 4 },
    { 0x89f24fc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb64fc5, 4 },
    { 0x583a4fc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e4fc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03e4fd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ca4fd4, 3 },
    { 0x5eee4fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b24fda, 3 },
    { 0, 0 },
    { 0x56be4fdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc64fdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb24fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f924fea, 3 },
    { 0x5e224feb, 4 },
    { 0xa3224fec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ff7, 3 },
    { 0x9e2e4ff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e4ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x747a4ffe, 5 },
    { 0x4b8a4fff, 3 },
    { 0x88e25000, 3 },
    { 0x464e4ffe, 3 },
    { 0xe1d65002, 4 },
    { 0xbf9a5003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46500c, 3 },
    { 0x5e6a500c, 3 },
    { 0xb07a500e, 3 },
    { 0x2e92500f, 4 },
    { 0, 0 },
    { 0xa91a5011, 4 },
    { 0x50fe5012, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e525015, 3 },
    { 0x1d0e5016, 3 },
    { 0x5e265017, 4 },
    { 0x365016, 3 },
    { 0xf3465015, 4 },
    { 0xfd76501a, 3 },
    { 0x4362501b, 3 },
    { 0x697e501c, 3 },
    { 0x1d46501b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac25022, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bda5025, 3 },
    { 0x6fa25025, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdae5029, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da502c, 3 },
    { 0xe0ca502d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b25030, 3 },
    { 0x68ba5030, 3 },
    { 0, 0 },
    { 0x59aa5033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc25038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefe503c, 3 },
    { 0xb0e2503d, 3 },
    { 0xdfc2503e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0e5041, 4 },
    { 0x881e5042, 4 },
    { 0x7aa65042, 3 },
    { 0x3cee5044, 3 },
    { 0xc42e5044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda5048, 3 },
    { 0x8b1e5048, 3 },
    { 0, 0 },
    { 0x6f4e504b, 4 },
    { 0x3d22504c, 3 },
    { 0xcdd6504b, 3 },
    { 0xe14a504e, 3 },
    { 0xee06504f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87365057, 3 },
    { 0x42da5058, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba505d, 4 },
    { 0x3ace505e, 3 },
    { 0xede2505d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f365062, 4 },
    { 0x74ae5063, 3 },
    { 0xa76e5064, 3 },
    { 0x9f165063, 3 },
    { 0xe9f65066, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab265069, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4125070, 4 },
    { 0xea6a5071, 3 },
    { 0, 0 },
    { 0x72f25073, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c2507c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd292507f, 4 },
    { 0xa88e5080, 3 },
    { 0, 0 },
    { 0x9d625082, 3 },
    { 0x8d8a5083, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b465086, 3 },
    { 0, 0 },
    { 0x6e2a5088, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e508e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a925093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae65099, 3 },
    { 0, 0 },
    { 0x4cee509b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x497a509e, 3 },
    { 0x78fa509f, 3 },
    { 0x5c6e50a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e650a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5a50ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b250b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ce50b5, 3 },
    { 0xb86650b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276e50ba, 4 },
    { 0, 0 },
    { 0x994e50bc, 3 },
    { 0, 0 },
    { 0x303a50be, 3 },
    { 0x46ca50bf, 3 },
    { 0x14d650bf, 3 },
    { 0x2fce50be, 3 },
    { 0xb82650c2, 4 },
    { 0x4dae50c3, 3 },
    { 0x3a5e50be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5250ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60250cf, 4 },
    { 0, 0 },
    { 0x424e50d1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b250d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05a50d9, 3 },
    { 0x4a8650da, 3 },
    { 0x973e50db, 3 },
    { 0xabba50db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e50e2, 3 },
    { 0xb54e50e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4a50e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96650ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f250ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18a50f5, 3 },
    { 0xce650f6, 4 },
    { 0, 0 },
    { 0x1a3250f8, 3 },
    { 0x41d250f9, 3 },
    { 0x448e50f8, 3 },
    { 0, 0 },
    { 0xc19650fc, 3 },
    { 0xa5da50fd, 3 },
    { 0, 0 },
    { 0xf7ea50ff, 3 },
    { 0x3bfa50ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcae5106, 3 },
    { 0, 0 },
    { 0x864e5108, 3 },
    { 0xb97e5109, 3 },
    { 0x1ec65108, 3 },
    { 0x569e510a, 3 },
    { 0x301a5108, 3 },
    { 0x443a510d, 3 },
    { 0xc03e510d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f25117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aa511a, 3 },
    { 0x1582511b, 3 },
    { 0, 0 },
    { 0x563e511d, 3 },
    { 0, 0 },
    { 0xef1e511f, 3 },
    { 0x1ab65120, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce25125, 3 },
    { 0xfbc25126, 3 },
    { 0xbdc25127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2512c, 3 },
    { 0x4a96512d, 3 },
    { 0, 0 },
    { 0x1f12512f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00a5133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8865136, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62513a, 3 },
    { 0x49be513b, 3 },
    { 0xea52513b, 3 },
    { 0xd002513d, 4 },
    { 0xee1a513e, 3 },
    { 0x8c32513f, 3 },
    { 0x5f42513c, 3 },
    { 0, 0 },
    { 0xd4c65142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77e5146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61e5149, 3 },
    { 0xb52e514a, 4 },
    { 0xc87a5149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4906514e, 3 },
    { 0x89de514f, 4 },
    { 0, 0 },
    { 0xdb3a5151, 4 },
    { 0x65965152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ba515b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fe5160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64825166, 3 },
    { 0x7e225167, 3 },
    { 0x714e5167, 4 },
    { 0, 0 },
    { 0x68ea516a, 3 },
    { 0, 0 },
    { 0x248a516c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca5176, 4 },
    { 0x88aa5176, 3 },
    { 0, 0 },
    { 0x7ea5179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f65181, 4 },
    { 0x8f665182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ade5186, 3 },
    { 0xe6a5187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e518a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15a5191, 3 },
    { 0x6aa25191, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e5199, 3 },
    { 0x396e519a, 3 },
    { 0xb6d2519b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89e519f, 4 },
    { 0x316651a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d651a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7a51ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b651b2, 3 },
    { 0x3cce51b3, 3 },
    { 0xedc251b4, 3 },
    { 0x82fe51b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfa51b8, 3 },
    { 0xaaee51b9, 4 },
    { 0x8ea251ba, 3 },
    { 0xb1d251ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba51c0, 4 },
    { 0xe9f651c0, 3 },
    { 0xdf2e51c2, 3 },
    { 0x544251c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26e51c7, 3 },
    { 0x85e51c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b651d0, 4 },
    { 0xb87e51d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8a51d8, 3 },
    { 0, 0 },
    { 0xc0b651da, 4 },
    { 0, 0 },
    { 0xca5651dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7251e1, 3 },
    { 0xf65e51e1, 3 },
    { 0xa1be51e3, 3 },
    { 0, 0 },
    { 0x813651e5, 4 },
    { 0x6d7251e6, 3 },
    { 0, 0 },
    { 0xc54251e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97a51ec, 3 },
    { 0, 0 },
    { 0x203a51ee, 4 },
    { 0xfa5a51ef, 3 },
    { 0x585651f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec251fa, 3 },
    { 0x76a251fb, 4 },
    { 0x8cc251fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x528a51ff, 3 },
    { 0, 0 },
    { 0x25225201, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1125208, 3 },
    { 0, 0 },
    { 0xfcd6520a, 4 },
    { 0, 0 },
    { 0xfc56520c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda5211, 4 },
    { 0x5e5a5211, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41a5219, 4 },
    { 0x281a5219, 3 },
    { 0x8822521a, 3 },
    { 0x90f6521c, 3 },
    { 0, 0 },
    { 0x736521e, 3 },
    { 0xef0e521f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fe5223, 3 },
    { 0, 0 },
    { 0xab25225, 3 },
    { 0, 0 },
    { 0x659e5227, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4242522a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ea522e, 3 },
    { 0x6c6a522f, 4 },
    { 0, 0 },
    { 0x224a5231, 3 },
    { 0xe9d65232, 3 },
    { 0xdd6e5232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46be5237, 3 },
    { 0x11ba5238, 4 },
    { 0xdd9a5238, 3 },
    { 0x14725238, 3 },
    { 0x41d6523a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b76523e, 3 },
    { 0xb3c6523f, 3 },
    { 0xa15e523f, 3 },
    { 0x40a25241, 3 },
    { 0, 0 },
    { 0xc6465243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e65247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce5250, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x485a5253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28e5257, 3 },
    { 0xe0525257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ca525b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43e525e, 3 },
    { 0x458a525f, 3 },
    { 0xa1365260, 3 },
    { 0, 0 },
    { 0xf0a25262, 4 },
    { 0xe6e5262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd25266, 4 },
    { 0x5b465267, 3 },
    { 0, 0 },
    { 0x318e5269, 3 },
    { 0x2626526a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ba5271, 3 },
    { 0x53c25272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f16527f, 3 },
    { 0x7cf65280, 3 },
    { 0xf99e5280, 3 },
    { 0, 0 },
    { 0x5ff25283, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686a5288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f2528e, 4 },
    { 0x15ca528f, 3 },
    { 0x932528e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd25299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc252529f, 3 },
    { 0x31f252a0, 3 },
    { 0x429652a1, 3 },
    { 0xe68652a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5a52a5, 3 },
    { 0x7ce652a6, 3 },
    { 0, 0 },
    { 0x954a52a8, 3 },
    { 0, 0 },
    { 0x349252aa, 5 },
    { 0xfc9a52ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0652af, 4 },
    { 0xab1a52b0, 3 },
    { 0, 0 },
    { 0xb56e52b2, 3 },
    { 0x947a52b2, 3 },
    { 0x921a52b3, 4 },
    { 0x723652b5, 3 },
    { 0x378e52b6, 4 },
    { 0, 0 },
    { 0xd68652b8, 3 },
    { 0xb54e52b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5a52bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd652c2, 3 },
    { 0x228652c2, 3 },
    { 0xd0e252c4, 3 },
    { 0x8bbe52c2, 3 },
    { 0xdbfa52c6, 4 },
    { 0xc24a52c7, 3 },
    { 0xda1652c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c252cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2a52d3, 3 },
    { 0x3e9652d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3652d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca52de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a52e2, 3 },
    { 0xf10a52e3, 3 },
    { 0xaae252e4, 3 },
    { 0, 0 },
    { 0x944e52e6, 3 },
    { 0xcb7e52e7, 3 },
    { 0xbf2652e8, 3 },
    { 0xbd0252e6, 3 },
    { 0xfee652ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a252ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a252ff, 4 },
    { 0x1b665300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fe5308, 3 },
    { 0x7aee5308, 3 },
    { 0xc6f6530a, 3 },
    { 0xa796530a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a530f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be65314, 3 },
    { 0xb96a5315, 4 },
    { 0, 0 },
    { 0x80925317, 3 },
    { 0x365a5318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e5321, 4 },
    { 0x26ae5322, 3 },
    { 0xaa565323, 3 },
    { 0, 0 },
    { 0x1dd65325, 3 },
    { 0x7f0a5326, 3 },
    { 0x515a5326, 3 },
    { 0x53f65326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dae532d, 3 },
    { 0x226532e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39e5333, 3 },
    { 0x25665334, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2e5344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53465347, 3 },
    { 0xc3565347, 3 },
    { 0x5c765349, 3 },
    { 0x6eee5349, 3 },
    { 0x77ea534b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948e5350, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ee5359, 3 },
    { 0xaa6535a, 4 },
    { 0, 0 },
    { 0x6dae535c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ca535f, 4 },
    { 0xefa2535f, 3 },
    { 0, 0 },
    { 0x665a5362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ea5366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54525370, 3 },
    { 0x4b225371, 3 },
    { 0x2ee25371, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8e5380, 4 },
    { 0xdb1a5381, 3 },
    { 0x1cde5382, 3 },
    { 0x7f4e5383, 3 },
    { 0, 0 },
    { 0xaade5385, 3 },
    { 0xd73e5386, 3 },
    { 0x5c725387, 3 },
    { 0xbf425387, 3 },
    { 0xc3fa5389, 4 },
    { 0xd552538a, 3 },
    { 0x3a925387, 4 },
    { 0x4c465388, 3 },
    { 0, 0 },
    { 0x666a538e, 4 },
    { 0, 0 },
    { 0x313a5390, 3 },
    { 0x89ee5391, 3 },
    { 0xa32e5391, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcade5398, 3 },
    { 0xb8c25399, 4 },
    { 0x1b6539a, 3 },
    { 0x1b425398, 3 },
    { 0x3f22539c, 3 },
    { 0x626e539d, 3 },
    { 0, 0 },
    { 0xac82539f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3a53a2, 4 },
    { 0, 0 },
    { 0x425e53a4, 4 },
    { 0, 0 },
    { 0x7cba53a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a53a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff653b0, 3 },
    { 0, 0 },
    { 0x8ea53b2, 3 },
    { 0x88ce53b2, 3 },
    { 0x8df253b4, 4 },
    { 0xff6253b5, 3 },
    { 0, 0 },
    { 0xb95e53b7, 3 },
    { 0, 0 },
    { 0x59b253b9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829253c0, 3 },
    { 0x6a0653c1, 3 },
    { 0xd8ca53c2, 3 },
    { 0xe2a253c3, 3 },
    { 0, 0 },
    { 0x3d6253c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0e53c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b253cd, 4 },
    { 0, 0 },
    { 0x404253cf, 3 },
    { 0xfc9a53cf, 3 },
    { 0x689653d0, 3 },
    { 0x611253d2, 3 },
    { 0xa6ee53d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a53d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671653de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfe53e1, 3 },
    { 0xbcca53e2, 4 },
    { 0x402a53e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1253e6, 3 },
    { 0xb90e53e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ea53eb, 3 },
    { 0x6bce53eb, 3 },
    { 0x5f4653ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2253f3, 3 },
    { 0x9e1e53f4, 3 },
    { 0, 0 },
    { 0xb34a53f6, 5 },
    { 0, 0 },
    { 0xad2653f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x776653fb, 4 },
    { 0xd70a53fc, 3 },
    { 0xbdb653fb, 4 },
    { 0, 0 },
    { 0x6ca253ff, 3 },
    { 0x679e5400, 5 },
    { 0, 0 },
    { 0x37825402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d665406, 3 },
    { 0, 0 },
    { 0xc8f65408, 4 },
    { 0x894e5409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d65411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29565414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a2541e, 3 },
    { 0x7e82541e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x845a5422, 3 },
    { 0xea1e5422, 3 },
    { 0x4c1e5424, 3 },
    { 0, 0 },
    { 0x138a5426, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b25429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2a542d, 5 },
    { 0x117e542e, 3 },
    { 0x44e2542f, 4 },
    { 0xa7be542f, 3 },
    { 0x6c5e5431, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede65435, 3 },
    { 0, 0 },
    { 0xbb865437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf265442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee5447, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a2544b, 4 },
    { 0x11a2544c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25a544f, 4 },
    { 0, 0 },
    { 0xf4ca5451, 4 },
    { 0, 0 },
    { 0x88be5453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6545a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e5460, 4 },
    { 0x56ba5461, 3 },
    { 0, 0 },
    { 0xde925463, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fa5468, 3 },
    { 0x824e5469, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52546d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacca5473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3552547a, 3 },
    { 0, 0 },
    { 0xb09a547c, 3 },
    { 0x1cee547d, 3 },
    { 0xf4f6547c, 3 },
    { 0x7a9e547f, 3 },
    { 0xd5925480, 3 },
    { 0, 0 },
    { 0x5e8e5482, 3 },
    { 0xe3b65483, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6da5486, 4 },
    { 0, 0 },
    { 0x9c9e5488, 5 },
    { 0, 0 },
    { 0xf466548a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbce548d, 4 },
    { 0, 0 },
    { 0xa2ca548f, 3 },
    { 0x4a125490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfe5493, 3 },
    { 0x72525494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891a549c, 3 },
    { 0x2462549d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4254a2, 3 },
    { 0xabca54a3, 3 },
    { 0xb2de54a4, 4 },
    { 0x51ce54a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4da54a8, 3 },
    { 0, 0 },
    { 0xa83e54aa, 4 },
    { 0x7b4e54aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb654ae, 3 },
    { 0xd0de54ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0da54b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde254b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0654bd, 3 },
    { 0xa71254be, 4 },
    { 0, 0 },
    { 0x989a54c0, 3 },
    { 0, 0 },
    { 0xe91254c2, 3 },
    { 0, 0 },
    { 0xbe4654c4, 3 },
    { 0, 0 },
    { 0x579e54c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffae54cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db654d8, 4 },
    { 0x44d254d9, 3 },
    { 0x317a54d9, 3 },
    { 0x537a54db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5254e1, 3 },
    { 0x47d254e2, 3 },
    { 0x656254e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124654e7, 3 },
    { 0x179e54e7, 3 },
    { 0xedf654e9, 3 },
    { 0, 0 },
    { 0xa86a54eb, 3 },
    { 0x20ae54ec, 3 },
    { 0x29f254ed, 3 },
    { 0x1bf254ee, 3 },
    { 0xe3a654ef, 3 },
    { 0x35ca54f0, 3 },
    { 0x9f1654ee, 4 },
    { 0, 0 },
    { 0x9dce54f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96654fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73da54fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64165501, 3 },
    { 0, 0 },
    { 0xade25503, 3 },
    { 0, 0 },
    { 0xcce65505, 4 },
    { 0x9e9e5506, 3 },
    { 0, 0 },
    { 0x71765508, 3 },
    { 0, 0 },
    { 0x320e550a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9656550d, 3 },
    { 0, 0 },
    { 0x24de550f, 4 },
    { 0, 0 },
    { 0xefde5511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1a5516, 3 },
    { 0, 0 },
    { 0x52aa5518, 3 },
    { 0xefae5519, 3 },
    { 0, 0 },
    { 0xb6ce551b, 3 },
    { 0x186a551b, 3 },
    { 0, 0 },
    { 0xa522551e, 3 },
    { 0xdc82551f, 3 },
    { 0, 0 },
    { 0xe1c65521, 3 },
    { 0, 0 },
    { 0xfc2a5523, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3725529, 3 },
    { 0, 0 },
    { 0xc2c6552b, 3 },
    { 0x900a552c, 3 },
    { 0xde02552d, 5 },
    { 0xe122552e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51e5536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed25539, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4e553e, 5 },
    { 0x2d36553e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7e554f, 4 },
    { 0xd3a25550, 3 },
    { 0xc26e5550, 3 },
    { 0x6dde5552, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce555a, 3 },
    { 0xde12555a, 3 },
    { 0xbea2555c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb2555f, 3 },
    { 0x814e555f, 3 },
    { 0x7c8e5561, 3 },
    { 0xe6025562, 3 },
    { 0x12365561, 3 },
    { 0, 0 },
    { 0xdf025565, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8425568, 3 },
    { 0x368e5569, 3 },
    { 0x4f0a5569, 3 },
    { 0, 0 },
    { 0x8186556c, 4 },
    { 0x909a556d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2e5571, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2557f, 3 },
    { 0x8982557f, 4 },
    { 0x7ce65580, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaa5589, 3 },
    { 0x613e558a, 5 },
    { 0xc9e6558b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e5594, 3 },
    { 0x38865594, 3 },
    { 0, 0 },
    { 0xf9065597, 3 },
    { 0x2fde5598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ee559f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09255a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319655af, 4 },
    { 0x380655b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0a55b4, 3 },
    { 0x3ef655b4, 3 },
    { 0x201e55b5, 3 },
    { 0x4e1255b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86a55ba, 4 },
    { 0x699255ba, 3 },
    { 0x16a55bb, 3 },
    { 0x828255bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955e55c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2655c8, 3 },
    { 0x5ffa55c8, 5 },
    { 0x24a255ca, 3 },
    { 0xa6fe55cb, 3 },
    { 0x2d0a55c8, 3 },
    { 0x495e55cb, 4 },
    { 0, 0 },
    { 0x258255cf, 3 },
    { 0xb32a55cf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x944a55d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b255d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91255dd, 3 },
    { 0x19ce55dd, 3 },
    { 0xd2e655dd, 3 },
    { 0xb8a55e0, 5 },
    { 0xa24655e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde55e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa255f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298255f7, 4 },
    { 0x35b655f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358e55fd, 4 },
    { 0xa0e655fd, 3 },
    { 0, 0 },
    { 0x140a5600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28025603, 4 },
    { 0x24625604, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70a560d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59965610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ba5613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42561c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41e5620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce5627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad65630, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5a5636, 4 },
    { 0xb76a5637, 3 },
    { 0x9ebe5638, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee6563f, 3 },
    { 0x4ba6563f, 3 },
    { 0x81e5641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402564e, 3 },
    { 0x5706564e, 3 },
    { 0, 0 },
    { 0x314e5651, 3 },
    { 0xbd725652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e5657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365666, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee2566d, 3 },
    { 0xd04e566e, 3 },
    { 0xbd72566f, 3 },
    { 0xbca65670, 3 },
    { 0xa52a5671, 3 },
    { 0x6b625672, 3 },
    { 0xc83e5672, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x62865676, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aa5680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9965695, 3 },
    { 0x5f925696, 3 },
    { 0, 0 },
    { 0x4c065698, 3 },
    { 0xf48e5698, 3 },
    { 0x55fe5698, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9886569d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1656a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d256a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fda56ac, 3 },
    { 0x288656ac, 3 },
    { 0x892e56ae, 3 },
    { 0x2f2a56af, 3 },
    { 0, 0 },
    { 0xd39a56b1, 3 },
    { 0xf1256b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2e56b6, 4 },
    { 0x6a7656b7, 3 },
    { 0x23be56b7, 3 },
    { 0x829e56b8, 4 },
    { 0x7a0256ba, 3 },
    { 0, 0 },
    { 0x96e656bc, 4 },
    { 0x7d4e56bd, 3 },
    { 0x615a56bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7256c2, 4 },
    { 0x14a56c2, 4 },
    { 0x1efa56c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4a56cb, 3 },
    { 0x897a56cb, 3 },
    { 0x4e2a56cd, 3 },
    { 0x18c656ce, 3 },
    { 0x5b2a56cb, 3 },
    { 0x229e56d0, 3 },
    { 0, 0 },
    { 0xac0a56d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55e56d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b656dc, 3 },
    { 0x16a56dc, 3 },
    { 0, 0 },
    { 0x50aa56df, 3 },
    { 0xab0656e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca56e3, 4 },
    { 0xccf256e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a256ee, 3 },
    { 0x7bc656ef, 3 },
    { 0xdd0a56f0, 3 },
    { 0x5f4e56f1, 4 },
    { 0x100656f0, 3 },
    { 0xcd4e56f2, 3 },
    { 0x39c656f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3a56f9, 4 },
    { 0xa4ea56fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4e56ff, 3 },
    { 0x9ca25700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b2570a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x443e570d, 3 },
    { 0x4146570e, 4 },
    { 0x1a76570f, 3 },
    { 0x1362570f, 4 },
    { 0xa80e570d, 3 },
    { 0x89fa5712, 3 },
    { 0, 0 },
    { 0x43365714, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2265717, 3 },
    { 0xb3965718, 3 },
    { 0xaee5719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c36571e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc25726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f6572a, 3 },
    { 0xbac6572a, 3 },
    { 0, 0 },
    { 0x3a22572d, 3 },
    { 0x7166572d, 3 },
    { 0xf0ae572f, 3 },
    { 0, 0 },
    { 0xe7d25731, 3 },
    { 0x43765732, 3 },
    { 0x3e7e5731, 3 },
    { 0xdae5732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e625739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04e5743, 3 },
    { 0x430a5744, 3 },
    { 0x410a5744, 3 },
    { 0x89025746, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafa574a, 4 },
    { 0x41ce574b, 4 },
    { 0x9e32574c, 3 },
    { 0xed6574c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e5752, 4 },
    { 0x3e165752, 3 },
    { 0x6d365754, 4 },
    { 0xb0b25753, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e5759, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd526575c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda5760, 3 },
    { 0x83e5761, 3 },
    { 0x93da5762, 3 },
    { 0x4f725763, 3 },
    { 0xdc165764, 4 },
    { 0xfa165765, 3 },
    { 0xd32a5766, 4 },
    { 0xb2d25763, 3 },
    { 0x9afe5764, 3 },
    { 0xb1fe5766, 3 },
    { 0x122a576a, 4 },
    { 0, 0 },
    { 0x313a576c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36225770, 3 },
    { 0xb8ea5771, 3 },
    { 0xad965770, 3 },
    { 0xab925773, 3 },
    { 0xaf0a5773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60025777, 3 },
    { 0x37525777, 3 },
    { 0x15e5779, 3 },
    { 0, 0 },
    { 0xddce577b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa577f, 4 },
    { 0xb5f25780, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e25787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa578c, 3 },
    { 0xff9a578d, 3 },
    { 0, 0 },
    { 0xd932578f, 3 },
    { 0, 0 },
    { 0xc95a5791, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e65794, 3 },
    { 0, 0 },
    { 0xf4f25796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2a5799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e579c, 3 },
    { 0x22ce579c, 4 },
    { 0x7b22579d, 3 },
    { 0xe8ba579e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fe57a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7a57a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139e57ae, 4 },
    { 0xee8a57af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ca57b2, 3 },
    { 0xff8a57b3, 4 },
    { 0xf27e57b4, 4 },
    { 0xc94e57b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3657b8, 3 },
    { 0, 0 },
    { 0xe9e257ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ece57c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279e57c6, 3 },
    { 0x91aa57c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593a57ce, 3 },
    { 0x749a57ce, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39257d6, 3 },
    { 0, 0 },
    { 0x402257d8, 3 },
    { 0xce9e57d9, 3 },
    { 0x8bfa57da, 4 },
    { 0xf6b257d9, 4 },
    { 0x53657dc, 4 },
    { 0x1f5657dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26257e1, 3 },
    { 0x355257e2, 4 },
    { 0x867e57e2, 3 },
    { 0, 0 },
    { 0xda2a57e5, 3 },
    { 0xfe7657e6, 4 },
    { 0xb7d257e6, 3 },
    { 0x48c657e8, 3 },
    { 0, 0 },
    { 0xd08257ea, 3 },
    { 0xe53e57ea, 4 },
    { 0xe6a257ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b257f0, 4 },
    { 0, 0 },
    { 0xb11e57f2, 3 },
    { 0, 0 },
    { 0xa40657f4, 3 },
    { 0x68e657f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde580c, 4 },
    { 0xc886580d, 3 },
    { 0xb84e580e, 3 },
    { 0xaa92580f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e5814, 3 },
    { 0, 0 },
    { 0x89d25816, 3 },
    { 0xe1865816, 3 },
    { 0xd1ae5817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0465820, 3 },
    { 0, 0 },
    { 0x1eda5822, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae765829, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaa582f, 5 },
    { 0, 0 },
    { 0x861e5831, 4 },
    { 0x1bd25832, 3 },
    { 0x55325833, 3 },
    { 0x39925834, 3 },
    { 0x667a5835, 3 },
    { 0xcc965836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe6583a, 3 },
    { 0xb3ae583b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa965840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be25844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fa584c, 3 },
    { 0, 0 },
    { 0xc962584e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d65851, 4 },
    { 0xeb25851, 3 },
    { 0xbcf65851, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c6585d, 3 },
    { 0xd64a585e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e5869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2586d, 4 },
    { 0, 0 },
    { 0x859e586f, 3 },
    { 0x8a46586f, 3 },
    { 0x273e5870, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad65874, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef25879, 4 },
    { 0x4a6587a, 3 },
    { 0xb8be587b, 3 },
    { 0xe76a587b, 4 },
    { 0xc9d6587c, 3 },
    { 0xdc5e587d, 4 },
    { 0xf07e587b, 3 },
    { 0x70165880, 3 },
    { 0x1ed25881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x460e5884, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a22588b, 3 },
    { 0x11da588b, 4 },
    { 0xc72588b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411a5892, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa06589d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8e58a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9e58a3, 3 },
    { 0xcfb658a3, 3 },
    { 0, 0 },
    { 0xb39658a6, 3 },
    { 0x4d2a58a6, 3 },
    { 0, 0 },
    { 0x35de58a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7658ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e58af, 3 },
    { 0x9b1e58af, 3 },
    { 0x345e58b1, 3 },
    { 0x7f7a58af, 3 },
    { 0xe22258b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb658b6, 3 },
    { 0xb84a58b7, 3 },
    { 0x19d658b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x171258bb, 3 },
    { 0x98b258bb, 3 },
    { 0x581e58bd, 3 },
    { 0xb88258be, 3 },
    { 0xa35658bf, 3 },
    { 0xf43258bf, 3 },
    { 0x1c1a58c1, 3 },
    { 0xf30658c2, 3 },
    { 0x197258bc, 3 },
    { 0x91d258c0, 4 },
    { 0x1eca58bd, 3 },
    { 0, 0 },
    { 0xc69e58c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97258cb, 3 },
    { 0xa258cb, 3 },
    { 0x623258cd, 3 },
    { 0x3ea258ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6a58d8, 4 },
    { 0, 0 },
    { 0x8db258da, 3 },
    { 0, 0 },
    { 0x9f4a58dc, 3 },
    { 0xae8658dd, 3 },
    { 0x3cea58de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e58e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x121e58e4, 3 },
    { 0x44e258e5, 4 },
    { 0xaaca58e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8258e9, 3 },
    { 0x6f7e58ea, 3 },
    { 0x929e58ea, 3 },
    { 0xe9a58e9, 4 },
    { 0, 0 },
    { 0x21ee58ee, 3 },
    { 0xd45a58ef, 3 },
    { 0x8bba58f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fca58f5, 3 },
    { 0, 0 },
    { 0x1efe58f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa58fd, 4 },
    { 0, 0 },
    { 0x7a8658ff, 3 },
    { 0x1b225900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9e5905, 3 },
    { 0xf8565905, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a16590b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e2590e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71765912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87025918, 4 },
    { 0x4b7e5919, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9a591d, 3 },
    { 0x681e591d, 3 },
    { 0x2616591f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10365926, 3 },
    { 0, 0 },
    { 0xdce5928, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e2592c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea5931, 4 },
    { 0x81565932, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636e5936, 3 },
    { 0xf2f65937, 5 },
    { 0xa39e5938, 3 },
    { 0xe08a5939, 4 },
    { 0xd76e593a, 3 },
    { 0xa8365936, 3 },
    { 0xcf1e5938, 3 },
    { 0, 0 },
    { 0xd0be593e, 5 },
    { 0, 0 },
    { 0x788a5940, 3 },
    { 0x13225941, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd25945, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c225948, 3 },
    { 0x15165948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ca594e, 3 },
    { 0xe8ca594f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9e595e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0665961, 3 },
    { 0x5daa5962, 3 },
    { 0xcf365962, 4 },
    { 0x40625961, 3 },
    { 0xed4a5965, 4 },
    { 0x3a4e5965, 3 },
    { 0x24e65967, 3 },
    { 0x5b425967, 4 },
    { 0xcdaa5967, 3 },
    { 0xb7b6596a, 4 },
    { 0x2402596a, 3 },
    { 0xe076596c, 3 },
    { 0, 0 },
    { 0x194a596e, 3 },
    { 0xccba596f, 3 },
    { 0xf09e5970, 3 },
    { 0, 0 },
    { 0x4a125972, 4 },
    { 0, 0 },
    { 0x6865974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeda597d, 3 },
    { 0x9e22597d, 3 },
    { 0xbbce597e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e765982, 3 },
    { 0x8c465982, 3 },
    { 0, 0 },
    { 0x3f325985, 3 },
    { 0x74965986, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253a598b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298a598f, 3 },
    { 0x6666598f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca5995, 3 },
    { 0xdfb65996, 3 },
    { 0, 0 },
    { 0x103a5998, 3 },
    { 0x5c3a5999, 3 },
    { 0, 0 },
    { 0x297e599b, 3 },
    { 0, 0 },
    { 0x362a599d, 3 },
    { 0, 0 },
    { 0x4cba599f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b659ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e59b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228e59b5, 4 },
    { 0x6aa659b6, 4 },
    { 0x715e59b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871259be, 3 },
    { 0x52f659be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85659c3, 3 },
    { 0x670659c3, 3 },
    { 0x6a6259c5, 3 },
    { 0xf29659c6, 3 },
    { 0, 0 },
    { 0x83a259c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9a59cb, 4 },
    { 0, 0 },
    { 0x2b6a59cd, 3 },
    { 0x569e59cd, 4 },
    { 0xeee259cf, 3 },
    { 0xb6fe59d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fda59d3, 3 },
    { 0, 0 },
    { 0x7e5259d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x321259d8, 3 },
    { 0x316a59d8, 3 },
    { 0xcaa659da, 3 },
    { 0x90d259db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x192259de, 3 },
    { 0xf27e59df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ea59e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd659e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fa59e9, 3 },
    { 0xf0ea59ea, 3 },
    { 0x8cc259e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76a59ee, 4 },
    { 0xfe4259ef, 3 },
    { 0, 0 },
    { 0xad7659f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116259f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290259ff, 3 },
    { 0x966e59ff, 4 },
    { 0xb20e5a01, 3 },
    { 0x2cda59ff, 3 },
    { 0xcdc25a00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6e5a0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xffee5a0e, 3 },
    { 0x1c825a0e, 4 },
    { 0x3d5a5a10, 3 },
    { 0, 0 },
    { 0x4f565a12, 4 },
    { 0x72f25a13, 3 },
    { 0x69165a14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61525a19, 4 },
    { 0x627e5a19, 3 },
    { 0x98265a1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ae5a20, 3 },
    { 0x89ba5a21, 3 },
    { 0, 0 },
    { 0xe44e5a23, 4 },
    { 0, 0 },
    { 0xf0225a25, 3 },
    { 0, 0 },
    { 0x33825a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c65a2c, 3 },
    { 0, 0 },
    { 0x72ce5a2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda165a31, 4 },
    { 0x43ba5a32, 3 },
    { 0, 0 },
    { 0x782a5a34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53a5a37, 3 },
    { 0x15d25a38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b65a3e, 4 },
    { 0, 0 },
    { 0x2e8e5a40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5765a45, 3 },
    { 0, 0 },
    { 0x9ac65a47, 3 },
    { 0x796a5a48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c65a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1de5a54, 3 },
    { 0, 0 },
    { 0x447e5a56, 4 },
    { 0xa8c65a56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fa5a5a, 3 },
    { 0x48525a5b, 3 },
    { 0, 0 },
    { 0xd17e5a5d, 4 },
    { 0, 0 },
    { 0x84aa5a5f, 4 },
    { 0x59565a60, 4 },
    { 0xdc365a61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe5a65, 5 },
    { 0x29d65a66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e065a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6e5a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae5a77, 4 },
    { 0xf8265a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee625a7b, 4 },
    { 0x86f25a7c, 3 },
    { 0xec5e5a7c, 3 },
    { 0x99625a7e, 4 },
    { 0, 0 },
    { 0x69265a80, 4 },
    { 0x146a5a80, 4 },
    { 0xf2ee5a82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e965a90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e65a93, 3 },
    { 0, 0 },
    { 0x74a65a95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e65a9a, 5 },
    { 0, 0 },
    { 0xa20e5a9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea5aa0, 4 },
    { 0xe2e5aa0, 3 },
    { 0, 0 },
    { 0xbd3a5aa3, 3 },
    { 0, 0 },
    { 0x65da5aa5, 3 },
    { 0x76225aa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12165aa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8de5aaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99a5ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x953e5ab8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b25abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf25ac9, 3 },
    { 0x2a5a5aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x837a5acd, 3 },
    { 0xc03a5acd, 3 },
    { 0, 0 },
    { 0x6e9e5ad0, 3 },
    { 0, 0 },
    { 0x36125ad2, 5 },
    { 0xb4ce5ad2, 3 },
    { 0xea7a5ad4, 3 },
    { 0x999a5ad4, 3 },
    { 0xa77e5ad2, 4 },
    { 0x90da5ad7, 3 },
    { 0x4ea5ad5, 3 },
    { 0x18325ad9, 4 },
    { 0x9cae5ada, 3 },
    { 0xcf625ad9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa5ae0, 3 },
    { 0x416a5ae1, 3 },
    { 0, 0 },
    { 0x8dd65ae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849e5af0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e5af3, 4 },
    { 0x7a65af3, 3 },
    { 0x65fa5af5, 3 },
    { 0x9af25af6, 3 },
    { 0xd52a5af3, 3 },
    { 0xc5665af8, 3 },
    { 0x92e5af9, 3 },
    { 0x3c365af8, 4 },
    { 0, 0 },
    { 0x11d25afc, 4 },
    { 0xbd365afd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x193a5b00, 4 },
    { 0, 0 },
    { 0x56e25b02, 4 },
    { 0x78565b03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523e5b07, 3 },
    { 0, 0 },
    { 0xa9a25b09, 3 },
    { 0x58c65b0a, 4 },
    { 0x766e5b0b, 4 },
    { 0x73a25b0c, 4 },
    { 0x340a5b0c, 3 },
    { 0xbde25b0e, 4 },
    { 0x38025b09, 3 },
    { 0, 0 },
    { 0xc38e5b11, 3 },
    { 0, 0 },
    { 0x775a5b13, 4 },
    { 0xbb025b13, 3 },
    { 0x6e365b14, 3 },
    { 0, 0 },
    { 0x24e25b17, 4 },
    { 0x27625b18, 4 },
    { 0x94ba5b19, 3 },
    { 0x490a5b1a, 3 },
    { 0xf50a5b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987a5b1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408e5b24, 4 },
    { 0, 0 },
    { 0xdef25b26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac965b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a5b38, 3 },
    { 0xbb3a5b39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcba5b40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d125b44, 3 },
    { 0xfd365b45, 3 },
    { 0xf51e5b45, 3 },
    { 0xdd1e5b47, 3 },
    { 0, 0 },
    { 0x58c25b49, 3 },
    { 0x735e5b4a, 3 },
    { 0, 0 },
    { 0x29ee5b4c, 5 },
    { 0xcc225b4d, 3 },
    { 0x788a5b4e, 3 },
    { 0xc44a5b4e, 4 },
    { 0xbc8e5b4f, 3 },
    { 0, 0 },
    { 0x394e5b52, 3 },
    { 0x9c8a5b53, 3 },
    { 0xd5825b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73525b5a, 3 },
    { 0, 0 },
    { 0xcc925b5c, 4 },
    { 0xf7da5b5d, 3 },
    { 0x996a5b5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8825b67, 4 },
    { 0x2f1e5b68, 3 },
    { 0, 0 },
    { 0x7d525b6a, 3 },
    { 0, 0 },
    { 0x8a8a5b6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d65b71, 3 },
    { 0x3bee5b72, 4 },
    { 0x3e325b71, 3 },
    { 0x78a65b74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed25b7c, 4 },
    { 0x10125b7c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae5b80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49925b85, 3 },
    { 0xaea25b86, 4 },
    { 0x59ae5b86, 3 },
    { 0, 0 },
    { 0x33325b89, 4 },
    { 0x1ee5b8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x376e5b8d, 3 },
    { 0x74c25b8e, 4 },
    { 0x576a5b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21165b9a, 3 },
    { 0x752e5b9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf265b9e, 3 },
    { 0, 0 },
    { 0x83b25ba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa5ba5, 3 },
    { 0x6fbe5ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a5baa, 3 },
    { 0x57e5bab, 3 },
    { 0x10ea5bab, 5 },
    { 0xfe525bad, 3 },
    { 0x46525bad, 3 },
    { 0x7a865baf, 3 },
    { 0xfb8a5baf, 3 },
    { 0, 0 },
    { 0x61e25bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a5bb7, 4 },
    { 0, 0 },
    { 0x6e365bb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x925bc1, 4 },
    { 0x6cc65bc2, 4 },
    { 0xae425bc3, 4 },
    { 0xaa2a5bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db25bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50e5bca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8825bce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3325bd2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb65bd6, 3 },
    { 0, 0 },
    { 0x8a8e5bd8, 4 },
    { 0xa2125bd8, 3 },
    { 0xf22e5bda, 3 },
    { 0x20565bdb, 3 },
    { 0, 0 },
    { 0x5fce5bdd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b65be1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcede5be5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0e5be8, 5 },
    { 0x89565be9, 5 },
    { 0xfc125bea, 4 },
    { 0x682e5be9, 3 },
    { 0xb5325bec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff65bf1, 3 },
    { 0xf71e5bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e25bf7, 4 },
    { 0x2da5bf7, 3 },
    { 0x3f765bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91565c02, 3 },
    { 0xa0625c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e25c0c, 5 },
    { 0xd43a5c0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e25c11, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca5c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9e5c1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba65c21, 3 },
    { 0x7a1e5c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b65c26, 4 },
    { 0xf26a5c27, 3 },
    { 0xef525c28, 3 },
    { 0x8e565c27, 3 },
    { 0x255a5c27, 4 },
    { 0x2d265c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabe5c2f, 3 },
    { 0, 0 },
    { 0xc27a5c31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a65c38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62365c3d, 3 },
    { 0, 0 },
    { 0xaa025c3f, 4 },
    { 0x511a5c40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82965c43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a5c4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1e5c4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a665c53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaada5c57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a25c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x102a5c5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36e5c69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a5c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f325c73, 4 },
    { 0xe1025c74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42725c79, 3 },
    { 0xc8465c7a, 3 },
    { 0xde225c7b, 3 },
    { 0, 0 },
    { 0x20225c7d, 4 },
    { 0, 0 },
    { 0x4dda5c7f, 3 },
    { 0x2eba5c80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc65c8b, 3 },
    { 0, 0 },
    { 0x2be25c8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780a5c91, 3 },
    { 0, 0 },
    { 0x27925c93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc65c96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a5c9c, 3 },
    { 0xc1e25c9d, 3 },
    { 0x51de5c9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5725ca1, 4 },
    { 0x918e5ca2, 3 },
    { 0x9e765ca3, 3 },
    { 0xe7b65ca2, 3 },
    { 0x74165ca5, 3 },
    { 0x15a65ca2, 3 },
    { 0x62b65ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde5cac, 3 },
    { 0, 0 },
    { 0xd21a5cae, 3 },
    { 0, 0 },
    { 0xd00e5cb0, 3 },
    { 0x11da5cb0, 3 },
    { 0, 0 },
    { 0x4e7a5cb3, 3 },
    { 0x70165cb4, 3 },
    { 0x363e5cb4, 3 },
    { 0x21a65cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d65cbb, 4 },
    { 0xa35a5cbc, 3 },
    { 0x4cca5cbb, 3 },
    { 0, 0 },
    { 0x1bfa5cbf, 3 },
    { 0x36e5cbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c65cc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb625ccd, 4 },
    { 0x95ee5ccd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f765cd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x884a5cd4, 3 },
    { 0x46ea5cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde25cda, 3 },
    { 0x21165cda, 3 },
    { 0xccce5cdc, 3 },
    { 0x344a5cdc, 3 },
    { 0, 0 },
    { 0x86b65cdf, 4 },
    { 0x16925ce0, 4 },
    { 0x66ca5ce1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e5ce6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x449a5ce9, 3 },
    { 0xba8e5ce9, 4 },
    { 0x6a725ceb, 3 },
    { 0xafe25cec, 3 },
    { 0x36025ceb, 3 },
    { 0x98e5cee, 3 },
    { 0x74325cee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a5cf2, 4 },
    { 0x5c6a5cf3, 5 },
    { 0xd44a5cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebe5cf7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2265cfa, 3 },
    { 0x81865cfa, 4 },
    { 0x7f3a5cfb, 3 },
    { 0x2eda5cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3e5d08, 3 },
    { 0xc3325d09, 3 },
    { 0xd2625d0a, 3 },
    { 0, 0 },
    { 0x150e5d0c, 3 },
    { 0, 0 },
    { 0x63e5d0e, 5 },
    { 0xf44e5d0f, 3 },
    { 0x49665d10, 3 },
    { 0, 0 },
    { 0x801e5d12, 3 },
    { 0x9ea5d13, 3 },
    { 0, 0 },
    { 0x291e5d15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40425d18, 3 },
    { 0x46ea5d19, 3 },
    { 0x6a8a5d1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6a5d1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8325d21, 3 },
    { 0xda1a5d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3125d27, 4 },
    { 0x1d425d28, 3 },
    { 0x1a525d29, 3 },
    { 0x9b525d2a, 4 },
    { 0xd62a5d2b, 3 },
    { 0, 0 },
    { 0x78ee5d2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd065d30, 3 },
    { 0xbc965d30, 3 },
    { 0x865d32, 3 },
    { 0xc0a5d33, 4 },
    { 0x38a65d32, 3 },
    { 0x10fe5d34, 4 },
    { 0xf8825d36, 3 },
    { 0, 0 },
    { 0x648a5d38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3465d3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78865d3e, 3 },
    { 0x341a5d3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79865d42, 3 },
    { 0, 0 },
    { 0x741a5d44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f25d48, 3 },
    { 0x7bf65d49, 3 },
    { 0x825a5d4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a5d4d, 3 },
    { 0x6a0e5d4e, 4 },
    { 0xe5865d4f, 3 },
    { 0, 0 },
    { 0x69725d51, 3 },
    { 0, 0 },
    { 0x70da5d53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80a5d57, 3 },
    { 0xc29e5d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d25d5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a5d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd265d62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625d67, 3 },
    { 0x3a525d68, 3 },
    { 0xb0065d69, 3 },
    { 0x4b725d69, 3 },
    { 0x5825d6b, 4 },
    { 0x62465d67, 3 },
    { 0x98865d69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408a5d76, 3 },
    { 0, 0 },
    { 0xd525d78, 3 },
    { 0x66fa5d78, 4 },
    { 0x7b525d7a, 4 },
    { 0, 0 },
    { 0x162a5d7c, 3 },
    { 0, 0 },
    { 0xc59a5d7e, 3 },
    { 0x8de25d7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d125d83, 3 },
    { 0x44265d83, 3 },
    { 0x11025d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e5d89, 3 },
    { 0x614e5d8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4e5d99, 3 },
    { 0x40165d9a, 3 },
    { 0x8625d99, 3 },
    { 0x59f65d99, 3 },
    { 0x77d25d9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac665da3, 3 },
    { 0, 0 },
    { 0xc8265da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a5dad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce5db1, 3 },
    { 0x59e65db1, 4 },
    { 0, 0 },
    { 0x8bce5db4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98a5db7, 3 },
    { 0xb0065db8, 3 },
    { 0x507e5db8, 4 },
    { 0x3a25dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e65dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede5dc4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56965dc9, 3 },
    { 0x66fa5dca, 3 },
    { 0x5eea5dcb, 4 },
    { 0, 0 },
    { 0xd8e25dcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde5dd0, 3 },
    { 0, 0 },
    { 0x7fa5dd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb925dd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea5dde, 3 },
    { 0xf5c25ddf, 3 },
    { 0x13525de0, 3 },
    { 0, 0 },
    { 0xd9a65de2, 3 },
    { 0xa98e5de2, 4 },
    { 0x95ae5de2, 4 },
    { 0, 0 },
    { 0x3f765de6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1e5df0, 3 },
    { 0, 0 },
    { 0xfb425df2, 4 },
    { 0xb725df2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaa5df6, 4 },
    { 0x783a5df7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4e5dfa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd65dfe, 4 },
    { 0x31d25dfe, 3 },
    { 0, 0 },
    { 0xc99e5e01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4a5e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58365e07, 3 },
    { 0x9d425e08, 3 },
    { 0, 0 },
    { 0xa2365e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d65e12, 3 },
    { 0xb78e5e13, 3 },
    { 0xd52a5e13, 5 },
    { 0xc6fa5e15, 5 },
    { 0xcd825e14, 3 },
    { 0x60425e17, 3 },
    { 0, 0 },
    { 0x795e5e19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2e5e1d, 3 },
    { 0xd0a65e1e, 3 },
    { 0x8cbe5e1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c725e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ce5e2a, 4 },
    { 0, 0 },
    { 0xd68a5e2c, 3 },
    { 0, 0 },
    { 0xf6e65e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b25e31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b65e3a, 4 },
    { 0xae5e5e3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4165e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e5e44, 3 },
    { 0, 0 },
    { 0xb2fe5e46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a5e49, 4 },
    { 0, 0 },
    { 0xb65e4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77de5e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaa5e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3a5e59, 3 },
    { 0xe3125e5a, 3 },
    { 0x3c3a5e5b, 3 },
    { 0, 0 },
    { 0xda565e5d, 3 },
    { 0x95525e5e, 3 },
    { 0xa2225e5f, 3 },
    { 0x634e5e60, 3 },
    { 0x8de65e5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9225e65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f25e69, 4 },
    { 0, 0 },
    { 0xd6ea5e6b, 4 },
    { 0x39b25e6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac865e70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0e5e74, 3 },
    { 0, 0 },
    { 0xb94a5e76, 3 },
    { 0x1f0e5e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4565e7d, 3 },
    { 0, 0 },
    { 0x83ee5e7f, 3 },
    { 0x678e5e80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb025e83, 3 },
    { 0, 0 },
    { 0xef9e5e85, 4 },
    { 0, 0 },
    { 0xc0aa5e87, 3 },
    { 0x33ae5e88, 3 },
    { 0x23725e87, 3 },
    { 0x8ffa5e89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e5e94, 4 },
    { 0, 0 },
    { 0x38865e96, 3 },
    { 0, 0 },
    { 0x24125e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c25ea0, 4 },
    { 0xf76a5ea0, 3 },
    { 0x821e5ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3da5ea7, 3 },
    { 0x5fde5ea8, 3 },
    { 0xa17e5ea9, 3 },
    { 0xd3a25ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ba5eb3, 4 },
    { 0x44365eb3, 3 },
    { 0, 0 },
    { 0xe53e5eb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5eb9, 3 },
    { 0x1ada5eba, 3 },
    { 0x1a65eb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ea5ec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c25ec4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a25ed0, 3 },
    { 0x1ca25ed1, 3 },
    { 0x20de5ed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a5ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea5edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163a5ee4, 3 },
    { 0x792e5ee4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a5eea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b25ef1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79be5ef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5625efa, 4 },
    { 0x48ee5efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e5efe, 5 },
    { 0x96de5eff, 4 },
    { 0, 0 },
    { 0x33f65f01, 3 },
    { 0xa0365f01, 3 },
    { 0, 0 },
    { 0x2d325f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7da5f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4a5f0d, 4 },
    { 0xb1fe5f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8be5f14, 3 },
    { 0, 0 },
    { 0xeace5f16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221e5f1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f365f1d, 4 },
    { 0xd96e5f1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346a5f24, 3 },
    { 0x2bb25f25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba5f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60525f30, 3 },
    { 0x56565f30, 4 },
    { 0x9c265f32, 4 },
    { 0, 0 },
    { 0xb84a5f34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a5f3a, 3 },
    { 0xdc6a5f3a, 3 },
    { 0x43325f3c, 3 },
    { 0, 0 },
    { 0x8f365f3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51665f41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8e5f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82825f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e5f53, 3 },
    { 0x54025f54, 3 },
    { 0, 0 },
    { 0x31025f56, 4 },
    { 0, 0 },
    { 0x52525f58, 3 },
    { 0, 0 },
    { 0xf8c65f5a, 3 },
    { 0x53a25f5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae5f61, 4 },
    { 0x57ca5f61, 3 },
    { 0x4c9e5f63, 4 },
    { 0x2aae5f64, 3 },
    { 0x16125f64, 3 },
    { 0xf1d65f66, 4 },
    { 0xe5225f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f65f6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a5f6f, 4 },
    { 0, 0 },
    { 0xbf1e5f71, 4 },
    { 0x210a5f72, 3 },
    { 0xb12a5f72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7425f79, 3 },
    { 0, 0 },
    { 0xc0ee5f7b, 3 },
    { 0xa3f65f7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77de5f80, 3 },
    { 0x2e325f81, 3 },
    { 0, 0 },
    { 0xa0c25f83, 4 },
    { 0xa0625f83, 3 },
    { 0xf1e25f85, 3 },
    { 0, 0 },
    { 0x3b865f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd65f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa5f8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ca5f93, 3 },
    { 0xccd25f94, 3 },
    { 0, 0 },
    { 0xd73e5f96, 3 },
    { 0x286a5f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27a5f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e5f9d, 3 },
    { 0x2b3e5f9e, 3 },
    { 0, 0 },
    { 0xc0a5fa0, 3 },
    { 0xb5065fa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e5fa4, 3 },
    { 0, 0 },
    { 0xf8425fa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ce5fac, 3 },
    { 0x7da65fad, 3 },
    { 0xcbc25fad, 3 },
    { 0xf0525fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefde5fb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa65fb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x125e5fb8, 3 },
    { 0xf7ae5fb9, 4 },
    { 0x71b65fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcce5fbd, 3 },
    { 0, 0 },
    { 0xbab65fbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11a5fc5, 4 },
    { 0x4c2e5fc5, 3 },
    { 0xa0e65fc7, 3 },
    { 0xe9765fc5, 3 },
    { 0xbde65fc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa765fcd, 4 },
    { 0xf9665fce, 4 },
    { 0xfd925fce, 3 },
    { 0xacaa5fce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ba5fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x870e5fd8, 3 },
    { 0x22a5fd9, 3 },
    { 0, 0 },
    { 0xc1ea5fdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c65fde, 4 },
    { 0, 0 },
    { 0x65265fe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8a5fe4, 3 },
    { 0xebfe5fe5, 3 },
    { 0, 0 },
    { 0xf84e5fe7, 3 },
    { 0x60665fe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ba5feb, 3 },
    { 0x6c6e5fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a5ff3, 3 },
    { 0x5f465ff4, 3 },
    { 0x997e5ff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23e5ff9, 3 },
    { 0xe6c25ff9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5e5ffe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307a6007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26600b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x415a600e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f666012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa6017, 3 },
    { 0, 0 },
    { 0x71126019, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7a601c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b66021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7de6025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b26029, 3 },
    { 0x839a602a, 3 },
    { 0, 0 },
    { 0x74e602c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2602f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eae6033, 4 },
    { 0x3f966034, 3 },
    { 0x434a6035, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7926038, 3 },
    { 0x251e6039, 3 },
    { 0xf916603a, 3 },
    { 0x1a3e603b, 3 },
    { 0xf20e603c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40826040, 3 },
    { 0x4c0a6040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8e6049, 5 },
    { 0x3a06604a, 4 },
    { 0x104e6049, 5 },
    { 0x316604c, 3 },
    { 0xc2da604d, 4 },
    { 0x1d9a604d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e166054, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8a6063, 3 },
    { 0x2126063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b72606b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8a606e, 3 },
    { 0xfe72606f, 4 },
    { 0, 0 },
    { 0x91c66071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c66074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de6607a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef26082, 3 },
    { 0x8ba6083, 3 },
    { 0xf3b26083, 3 },
    { 0xa23a6083, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc452608b, 3 },
    { 0xe4b6608b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc26096, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf36609b, 4 },
    { 0x3e2e609c, 3 },
    { 0x7652609b, 3 },
    { 0, 0 },
    { 0x116609f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba60a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed660a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2660b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914a60b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae60b9, 3 },
    { 0x73de60ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157660be, 3 },
    { 0, 0 },
    { 0x8e9e60c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe60c4, 3 },
    { 0x7bd660c5, 3 },
    { 0x187660c6, 3 },
    { 0x568660c5, 4 },
    { 0x614a60c8, 3 },
    { 0xf2de60c9, 4 },
    { 0x2f1260c7, 3 },
    { 0x857260ca, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a660ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7e60d5, 3 },
    { 0x5b5e60d6, 3 },
    { 0x6c0a60d7, 3 },
    { 0, 0 },
    { 0x646260d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf260e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70260e8, 4 },
    { 0x4ada60e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd660ed, 4 },
    { 0, 0 },
    { 0x76da60ef, 3 },
    { 0x6f4a60ef, 4 },
    { 0x869660f1, 3 },
    { 0x4fba60f0, 3 },
    { 0xc86660ef, 4 },
    { 0xfc3260ef, 5 },
    { 0x327a60f5, 4 },
    { 0x99e660f6, 3 },
    { 0x5d5a60ef, 3 },
    { 0xd3ba60f8, 3 },
    { 0x424a60f9, 3 },
    { 0x2ba260fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331a60fe, 3 },
    { 0xf25e60ff, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d66102, 3 },
    { 0xbd5e6103, 3 },
    { 0, 0 },
    { 0xffa26105, 3 },
    { 0x37a66106, 3 },
    { 0, 0 },
    { 0xe1f26108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef2610f, 3 },
    { 0x7e02610f, 3 },
    { 0, 0 },
    { 0x7e426112, 4 },
    { 0x11a6112, 3 },
    { 0, 0 },
    { 0x54266115, 3 },
    { 0x38366115, 3 },
    { 0xa39a6117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac2611a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309a611f, 3 },
    { 0x6bb66120, 3 },
    { 0x1a566121, 3 },
    { 0xa8666122, 3 },
    { 0x24ae6123, 3 },
    { 0, 0 },
    { 0xae7e6125, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c6612b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fe6135, 3 },
    { 0x77ee6135, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4de6139, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de613c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6e613f, 4 },
    { 0x3b866140, 3 },
    { 0xefb6613f, 3 },
    { 0, 0 },
    { 0xda826143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a6146, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8666614b, 3 },
    { 0x1d5e614c, 4 },
    { 0xd2e6614d, 3 },
    { 0xd32e614d, 3 },
    { 0xd652614e, 3 },
    { 0x34fa6150, 4 },
    { 0x82a2614d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e26157, 3 },
    { 0x11626158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b6615f, 3 },
    { 0x21fa6160, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d26164, 5 },
    { 0x1a2e6165, 3 },
    { 0, 0 },
    { 0x2a0a6167, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae616a, 4 },
    { 0, 0 },
    { 0x235a616c, 3 },
    { 0, 0 },
    { 0xce8e616e, 4 },
    { 0xbdc2616e, 3 },
    { 0x95e616f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fe6175, 3 },
    { 0x995a6176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c26179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da617e, 4 },
    { 0x8fa617f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d326186, 3 },
    { 0, 0 },
    { 0xe226188, 3 },
    { 0xce826189, 3 },
    { 0, 0 },
    { 0xf792618b, 3 },
    { 0xeeda618b, 3 },
    { 0x68d6618b, 3 },
    { 0, 0 },
    { 0xa06a618f, 4 },
    { 0, 0 },
    { 0x65466191, 4 },
    { 0, 0 },
    { 0x6f526193, 4 },
    { 0, 0 },
    { 0x1e026195, 3 },
    { 0, 0 },
    { 0xce866197, 4 },
    { 0, 0 },
    { 0x72926199, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb752619d, 4 },
    { 0, 0 },
    { 0x522619f, 3 },
    { 0x676619f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692e61a6, 3 },
    { 0, 0 },
    { 0xdbae61a8, 3 },
    { 0x1aaa61a9, 3 },
    { 0xf0d661a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506661af, 4 },
    { 0x130261af, 3 },
    { 0x9b5261b1, 4 },
    { 0xa3c661b1, 4 },
    { 0x577261b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670a61b9, 3 },
    { 0x9b5e61ba, 5 },
    { 0x320661bb, 3 },
    { 0xf2ae61b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95261c4, 5 },
    { 0, 0 },
    { 0xb7c661c6, 3 },
    { 0, 0 },
    { 0xa7c261c8, 4 },
    { 0, 0 },
    { 0xc34e61ca, 3 },
    { 0x955a61cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785661d8, 4 },
    { 0x372a61d9, 3 },
    { 0x602e61da, 3 },
    { 0, 0 },
    { 0xe98661dc, 3 },
    { 0, 0 },
    { 0xc17261de, 3 },
    { 0x989a61df, 3 },
    { 0, 0 },
    { 0xe1de61e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05a61ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31661ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734e61f5, 3 },
    { 0xbb4661f5, 3 },
    { 0x3d3a61f7, 3 },
    { 0x17da61f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aa61fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0e61ff, 3 },
    { 0xfa966200, 3 },
    { 0xb17a6200, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b66209, 3 },
    { 0, 0 },
    { 0x74de620b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48e6212, 3 },
    { 0x51ce6212, 3 },
    { 0, 0 },
    { 0x947a6215, 3 },
    { 0x8326216, 4 },
    { 0, 0 },
    { 0xe8866218, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e621c, 3 },
    { 0x8016621c, 3 },
    { 0x25aa621e, 3 },
    { 0x8446621e, 3 },
    { 0xdeee6220, 3 },
    { 0x326a6221, 4 },
    { 0x5c2a6222, 3 },
    { 0x6c126223, 3 },
    { 0x98a6223, 3 },
    { 0x1726225, 4 },
    { 0x94f26222, 5 },
    { 0xd3b26225, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3246622d, 3 },
    { 0, 0 },
    { 0x74e622f, 3 },
    { 0x1e466230, 3 },
    { 0xdb26231, 3 },
    { 0xbc266232, 3 },
    { 0xc0626233, 5 },
    { 0x6efe6230, 4 },
    { 0x335a6235, 3 },
    { 0xa8f26233, 3 },
    { 0, 0 },
    { 0x82a6238, 3 },
    { 0, 0 },
    { 0x2b6e623a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaa623e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226241, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ca6246, 5 },
    { 0, 0 },
    { 0xd9b26248, 3 },
    { 0, 0 },
    { 0xad86624a, 3 },
    { 0x841e624b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59e624f, 3 },
    { 0x8df26250, 3 },
    { 0x81366250, 3 },
    { 0x8c566251, 4 },
    { 0, 0 },
    { 0xb8426254, 4 },
    { 0x5a9e6254, 3 },
    { 0xe6b66256, 3 },
    { 0x95f26256, 3 },
    { 0x4d9a6257, 4 },
    { 0x9fba6259, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7606625d, 3 },
    { 0, 0 },
    { 0xf16a625f, 3 },
    { 0x8db6625f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe6265, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x242a6268, 3 },
    { 0, 0 },
    { 0x2112626a, 3 },
    { 0xb29e626b, 3 },
    { 0, 0 },
    { 0xb9da626d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19866270, 3 },
    { 0, 0 },
    { 0xdc7e6272, 3 },
    { 0x439e6272, 3 },
    { 0, 0 },
    { 0xd2f26275, 3 },
    { 0x98ba6276, 3 },
    { 0, 0 },
    { 0xc84a6278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ae627b, 4 },
    { 0xc572627c, 3 },
    { 0, 0 },
    { 0xe856627e, 4 },
    { 0x125a627f, 3 },
    { 0, 0 },
    { 0xa95e6281, 5 },
    { 0xb5a66281, 5 },
    { 0, 0 },
    { 0xc1126284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e526288, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7966293, 3 },
    { 0, 0 },
    { 0xda1e6295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7126298, 3 },
    { 0xe1466298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd446629f, 3 },
    { 0, 0 },
    { 0x783662a1, 3 },
    { 0xd0e262a2, 3 },
    { 0x122e62a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931e62aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4662b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81662b4, 4 },
    { 0x9a2662b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c262b8, 4 },
    { 0x9e9a62b9, 3 },
    { 0xe2262ba, 4 },
    { 0x1a8e62ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66262bf, 3 },
    { 0x6a6a62c0, 4 },
    { 0x74a662c1, 3 },
    { 0x6f1e62c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3262c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20662cb, 3 },
    { 0xa28262cc, 5 },
    { 0, 0 },
    { 0x123262ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d262d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55262db, 3 },
    { 0xeafe62dc, 4 },
    { 0x110262dd, 3 },
    { 0xc03662dd, 3 },
    { 0x64d662df, 4 },
    { 0x200e62db, 3 },
    { 0xaa5e62df, 3 },
    { 0, 0 },
    { 0x4d7e62e3, 4 },
    { 0x1b8662e4, 3 },
    { 0x92ba62e5, 4 },
    { 0xfa8a62e6, 4 },
    { 0, 0 },
    { 0x392662e8, 3 },
    { 0x15be62e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc262ee, 3 },
    { 0xb17e62ef, 4 },
    { 0xa29a62ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a262f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90a62f9, 4 },
    { 0x1f6262fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ee62fe, 4 },
    { 0xbe262fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a826303, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c66312, 5 },
    { 0x2f726313, 3 },
    { 0xe05e6314, 3 },
    { 0x240e6315, 3 },
    { 0, 0 },
    { 0x75766317, 3 },
    { 0, 0 },
    { 0xae966319, 5 },
    { 0x3ca2631a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x848e631d, 4 },
    { 0x56fe631d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ae6321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfea6324, 3 },
    { 0, 0 },
    { 0xe6d26326, 3 },
    { 0x4f66326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ca632f, 3 },
    { 0xcd3e6330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ae6334, 3 },
    { 0, 0 },
    { 0x46266336, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a166339, 3 },
    { 0, 0 },
    { 0x746a633b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a633e, 4 },
    { 0, 0 },
    { 0x130a6340, 3 },
    { 0x62626341, 3 },
    { 0x73e6340, 3 },
    { 0x4afa6341, 3 },
    { 0xe2566344, 4 },
    { 0xdcde6345, 4 },
    { 0x3c1e6341, 3 },
    { 0x5aee6345, 3 },
    { 0x9d26341, 3 },
    { 0xec426349, 3 },
    { 0, 0 },
    { 0xbf7a634b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fe634f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193e6354, 3 },
    { 0x30e6355, 3 },
    { 0, 0 },
    { 0x50366357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaa635c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c26361, 4 },
    { 0x71f66361, 4 },
    { 0, 0 },
    { 0x10466364, 3 },
    { 0x105a6365, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62966369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1056636e, 3 },
    { 0, 0 },
    { 0x7d7e6370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc526377, 3 },
    { 0, 0 },
    { 0x2c266379, 3 },
    { 0x767e637a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6472637e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a6388, 3 },
    { 0, 0 },
    { 0xe406638a, 3 },
    { 0, 0 },
    { 0x9ffe638c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e638f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fa6394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57a6398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d26639d, 3 },
    { 0x698a639e, 3 },
    { 0xe9ca639f, 4 },
    { 0x42e663a0, 3 },
    { 0x84d263a0, 3 },
    { 0, 0 },
    { 0xe84663a3, 3 },
    { 0, 0 },
    { 0xe14e63a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3e63a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746e63af, 4 },
    { 0xc43663b0, 4 },
    { 0, 0 },
    { 0xf56263b2, 4 },
    { 0xb8f263b2, 3 },
    { 0x950663b4, 3 },
    { 0, 0 },
    { 0x39e263b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907663ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ee63bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77663c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5be63cb, 3 },
    { 0xaf663cc, 3 },
    { 0xc43a63cd, 3 },
    { 0, 0 },
    { 0xbe6263cf, 3 },
    { 0x8d6e63d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35263d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1da63d6, 3 },
    { 0xd9da63d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefe63dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9a63e2, 4 },
    { 0x8e0263e3, 3 },
    { 0x715663e3, 3 },
    { 0x40e263e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6263ea, 3 },
    { 0x43d263eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1663f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda63f5, 3 },
    { 0x222263f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50766405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9252640a, 3 },
    { 0x54d2640b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e6414, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a66421, 3 },
    { 0x14026421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a426428, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e642e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f66432, 3 },
    { 0x26a66433, 4 },
    { 0x45ba6434, 3 },
    { 0xa6ae6434, 3 },
    { 0, 0 },
    { 0x7b4a6437, 3 },
    { 0x810e6438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5402643d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ce6447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafe644b, 3 },
    { 0x8346644b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41de644f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2726455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79366459, 3 },
    { 0x8e06645a, 3 },
    { 0x40fe6459, 3 },
    { 0, 0 },
    { 0x7f5a645d, 4 },
    { 0, 0 },
    { 0x980e645f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6066463, 4 },
    { 0x24766464, 4 },
    { 0xe20e6465, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a526468, 3 },
    { 0x53426469, 3 },
    { 0x8832646a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d6646d, 3 },
    { 0x1636646e, 3 },
    { 0xb8fa646f, 3 },
    { 0x1fde6470, 4 },
    { 0, 0 },
    { 0x260e6472, 3 },
    { 0x50826472, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963e6479, 5 },
    { 0x319a6479, 3 },
    { 0xd98a647b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bea647f, 3 },
    { 0, 0 },
    { 0x3fb26481, 3 },
    { 0, 0 },
    { 0xcdd66483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a6491, 3 },
    { 0, 0 },
    { 0xffba6493, 3 },
    { 0x2aa6493, 3 },
    { 0, 0 },
    { 0x6fba6496, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c66499, 3 },
    { 0xa81a6499, 3 },
    { 0x509a6499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8236649f, 5 },
    { 0x64ce64a0, 3 },
    { 0xbafe64a0, 4 },
    { 0x696264a0, 3 },
    { 0, 0 },
    { 0xf9d664a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e64a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5264ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b264b3, 3 },
    { 0x705264b4, 3 },
    { 0x4b264b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ee64ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x964e64bd, 3 },
    { 0xa53264bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b264c5, 4 },
    { 0x12aa64c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e64d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20a64d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e64d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438664da, 3 },
    { 0x93d664db, 3 },
    { 0x754a64dc, 3 },
    { 0x9ce264dd, 4 },
    { 0, 0 },
    { 0xde5a64df, 3 },
    { 0, 0 },
    { 0x700264e1, 3 },
    { 0xf0be64e1, 3 },
    { 0xf5be64e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caa64ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b264ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a264f2, 3 },
    { 0x89d664f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79664f6, 3 },
    { 0x6c1a64f7, 3 },
    { 0x430e64f8, 4 },
    { 0, 0 },
    { 0x659664fa, 3 },
    { 0x58ca64fb, 4 },
    { 0x409a64fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6500, 4 },
    { 0x91f26501, 4 },
    { 0, 0 },
    { 0x73126503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1426506, 3 },
    { 0xccd66506, 3 },
    { 0xae526508, 3 },
    { 0xea4a6508, 3 },
    { 0x2b3a650a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5166513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea651e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca6521, 3 },
    { 0xbd2e6521, 3 },
    { 0x12126523, 3 },
    { 0xeaae6524, 3 },
    { 0xd5166525, 3 },
    { 0xd35e6525, 4 },
    { 0, 0 },
    { 0xac826528, 3 },
    { 0x35626529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865e6530, 3 },
    { 0x2f226530, 3 },
    { 0x99466532, 3 },
    { 0, 0 },
    { 0x98066534, 4 },
    { 0x6b0e6535, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e226539, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ee653f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9e6546, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x69866549, 3 },
    { 0x5d126549, 3 },
    { 0x3802654b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82654f, 3 },
    { 0xbee26550, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba566554, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe655a, 3 },
    { 0x69f6655a, 4 },
    { 0xe6ae655a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad26561, 4 },
    { 0x4dfe6562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e26566, 4 },
    { 0x970a6567, 3 },
    { 0x9a5e6567, 3 },
    { 0x1ae26568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0a656c, 3 },
    { 0x542656d, 4 },
    { 0x9732656e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9066572, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31866576, 3 },
    { 0xb58e6577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe906657c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a6581, 3 },
    { 0x27726582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7e658a, 3 },
    { 0x9202658a, 3 },
    { 0, 0 },
    { 0x1c9a658d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d66593, 4 },
    { 0xed526594, 3 },
    { 0, 0 },
    { 0x7de26596, 4 },
    { 0, 0 },
    { 0x5b3e6598, 3 },
    { 0xd0566599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a659c, 3 },
    { 0x6326659d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8265a3, 3 },
    { 0x133665a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f665aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e65af, 4 },
    { 0xd0de65b0, 5 },
    { 0xe8ae65b1, 3 },
    { 0xcf0265b0, 3 },
    { 0x271265b2, 3 },
    { 0, 0 },
    { 0xeb0665b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0665ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9265be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357a65c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1e65c5, 4 },
    { 0x954e65c6, 4 },
    { 0xf9ca65c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2e65cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa65ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad665d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4a65d9, 3 },
    { 0x7eda65da, 3 },
    { 0xf8ee65da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e665e2, 3 },
    { 0x98e665e2, 4 },
    { 0xe77665e2, 3 },
    { 0, 0 },
    { 0xcc4665e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3265ec, 3 },
    { 0x701a65ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce65f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x666665f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe65fb, 3 },
    { 0, 0 },
    { 0x7ece65fd, 3 },
    { 0x1a1265fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32be660a, 3 },
    { 0x121a660b, 3 },
    { 0x9c2e660b, 3 },
    { 0xf252660d, 3 },
    { 0x9722660d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92426614, 3 },
    { 0x9a066615, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68a661a, 3 },
    { 0x633e661a, 3 },
    { 0xb68a661b, 3 },
    { 0, 0 },
    { 0x8b16661e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26621, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a662d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba6633, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66e6636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba663a, 4 },
    { 0x49e2663b, 3 },
    { 0x8d42663a, 3 },
    { 0, 0 },
    { 0x4312663e, 3 },
    { 0xb4d6663e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae6643, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901a6647, 3 },
    { 0x680a6648, 3 },
    { 0, 0 },
    { 0x958a664a, 3 },
    { 0xf3a2664b, 3 },
    { 0x1f72664c, 4 },
    { 0, 0 },
    { 0xce3e664e, 3 },
    { 0, 0 },
    { 0xc8fe6650, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62da6654, 3 },
    { 0x10066655, 3 },
    { 0xd10a6655, 3 },
    { 0, 0 },
    { 0xa28a6658, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e665e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc26662, 3 },
    { 0, 0 },
    { 0xb14a6664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4666668, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ae666b, 3 },
    { 0, 0 },
    { 0xe746666d, 5 },
    { 0, 0 },
    { 0x35d6666f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27be6672, 4 },
    { 0, 0 },
    { 0xf30a6674, 3 },
    { 0x54466675, 3 },
    { 0xe4026676, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182667b, 3 },
    { 0, 0 },
    { 0x8d56667d, 4 },
    { 0x7fda667d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaa6681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54466687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eee668a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf966692, 4 },
    { 0xbde66693, 3 },
    { 0xe9a26692, 3 },
    { 0x71526693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed66698, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a669e, 3 },
    { 0x44b6669f, 3 },
    { 0xc8ee669f, 3 },
    { 0x6f6666a1, 3 },
    { 0xe3ce66a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429e66a6, 3 },
    { 0xf71666a7, 4 },
    { 0xad7e66a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24266ab, 3 },
    { 0, 0 },
    { 0xf32266ad, 3 },
    { 0xbdaa66ad, 3 },
    { 0xa54666ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0666b3, 3 },
    { 0, 0 },
    { 0x950266b5, 3 },
    { 0x3d4266b6, 3 },
    { 0xbd8e66b7, 3 },
    { 0x597266b8, 3 },
    { 0xac6266b9, 3 },
    { 0xc27a66b5, 3 },
    { 0x58d266b5, 3 },
    { 0, 0 },
    { 0xd21a66bd, 4 },
    { 0xa8d266be, 4 },
    { 0xa41666bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae66c2, 3 },
    { 0x8bb266c3, 3 },
    { 0xceae66c4, 5 },
    { 0x1ac666c3, 3 },
    { 0, 0 },
    { 0x60de66c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1a66d1, 3 },
    { 0xdaa266d1, 4 },
    { 0x252266d3, 3 },
    { 0x7fae66d4, 4 },
    { 0x975e66d5, 3 },
    { 0xdba666d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b666db, 4 },
    { 0, 0 },
    { 0x960266dd, 4 },
    { 0x453266de, 3 },
    { 0xac5a66df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x871266e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a66e5, 3 },
    { 0x14a266e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f666eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3266ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x111666f1, 3 },
    { 0x7bae66f2, 3 },
    { 0, 0 },
    { 0x3f4266f4, 5 },
    { 0xee2a66f4, 3 },
    { 0xde4e66f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14e66fa, 3 },
    { 0xcd7266fa, 3 },
    { 0x2a6266fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef66702, 3 },
    { 0xbbb26703, 3 },
    { 0x26326704, 3 },
    { 0xeae6705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a670b, 4 },
    { 0xd682670c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8a6718, 3 },
    { 0, 0 },
    { 0x3332671a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965a6721, 3 },
    { 0x73ae6722, 3 },
    { 0, 0 },
    { 0x55466724, 3 },
    { 0, 0 },
    { 0x6e7a6726, 4 },
    { 0, 0 },
    { 0x6c926728, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca672d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c66731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a26735, 3 },
    { 0xf1ba6735, 3 },
    { 0x45d66737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777a673e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf226741, 4 },
    { 0x846a6742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407a6747, 3 },
    { 0, 0 },
    { 0x1d166749, 4 },
    { 0x3b92674a, 3 },
    { 0, 0 },
    { 0x935e674c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36926750, 3 },
    { 0, 0 },
    { 0xff326752, 3 },
    { 0, 0 },
    { 0x36926754, 3 },
    { 0, 0 },
    { 0xecb26756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c26759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe2675d, 4 },
    { 0, 0 },
    { 0x4e2675f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827e676b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32de676e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce126776, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6352677d, 3 },
    { 0xaa42677e, 4 },
    { 0xb78e677d, 3 },
    { 0, 0 },
    { 0xba926781, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a6787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678a6790, 3 },
    { 0xab8e6791, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ce6796, 4 },
    { 0x62066796, 4 },
    { 0xe0a6798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a2679f, 4 },
    { 0xede67a0, 4 },
    { 0x5d8667a1, 3 },
    { 0x430a67a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218a67a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647267ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba67b5, 3 },
    { 0, 0 },
    { 0xa17a67b7, 3 },
    { 0, 0 },
    { 0x3f1e67b9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c667c0, 3 },
    { 0x12ce67c1, 3 },
    { 0xf58a67c2, 3 },
    { 0xc99267c3, 4 },
    { 0x6ef267c4, 3 },
    { 0x4a0e67c5, 3 },
    { 0x4d4267c4, 3 },
    { 0xf2f267c1, 4 },
    { 0xcfc667c5, 3 },
    { 0, 0 },
    { 0xe3a267ca, 3 },
    { 0x79e667cb, 3 },
    { 0, 0 },
    { 0x8d5e67cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6a67d0, 3 },
    { 0x869a67d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ae67d4, 3 },
    { 0, 0 },
    { 0x801e67d6, 3 },
    { 0x23ea67d7, 4 },
    { 0, 0 },
    { 0x4e9667d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fa67dc, 4 },
    { 0x71ee67dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31de67e0, 3 },
    { 0x986a67e0, 3 },
    { 0, 0 },
    { 0x9f5667e3, 3 },
    { 0, 0 },
    { 0x71ea67e5, 3 },
    { 0x803667e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aa67e9, 4 },
    { 0x8c3267e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8e67f4, 4 },
    { 0x2efa67f4, 3 },
    { 0x8bee67f6, 3 },
    { 0xd3e67f6, 3 },
    { 0xffce67f6, 3 },
    { 0xbac267f9, 3 },
    { 0xf49e67f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb426803, 5 },
    { 0x6d3e6803, 3 },
    { 0xdebe6804, 3 },
    { 0, 0 },
    { 0x4b026807, 3 },
    { 0x9a226808, 3 },
    { 0x32ee6809, 3 },
    { 0xfcee6809, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b26680d, 4 },
    { 0, 0 },
    { 0x2c7a680f, 3 },
    { 0x747a6810, 4 },
    { 0x203e6811, 3 },
    { 0x8e666810, 3 },
    { 0xcf5a680f, 3 },
    { 0x157a6813, 3 },
    { 0x38826812, 3 },
    { 0, 0 },
    { 0x18266817, 4 },
    { 0, 0 },
    { 0xc66a6819, 3 },
    { 0xf7426819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a681d, 3 },
    { 0xa05a681d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547a682a, 3 },
    { 0x62ea682a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aa682f, 3 },
    { 0xc50e682f, 4 },
    { 0x6ee66831, 3 },
    { 0xb54a6831, 3 },
    { 0xc52e6831, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceea683e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff626841, 3 },
    { 0xa43e6842, 3 },
    { 0xb4926842, 3 },
    { 0xe226844, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6426848, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efa684b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a96684f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce6853, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fa685e, 4 },
    { 0x36f2685f, 3 },
    { 0x938a685e, 4 },
    { 0x29ce6861, 3 },
    { 0x34b66862, 3 },
    { 0, 0 },
    { 0xebd26864, 3 },
    { 0x1ca26865, 3 },
    { 0xd4266865, 3 },
    { 0x886a6867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6686d, 3 },
    { 0, 0 },
    { 0x8816686f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2766874, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3a6878, 3 },
    { 0, 0 },
    { 0x6a5e687a, 4 },
    { 0, 0 },
    { 0xb08a687c, 3 },
    { 0x4ea2687d, 3 },
    { 0x1e72687e, 4 },
    { 0, 0 },
    { 0x46666880, 5 },
    { 0xb366881, 3 },
    { 0, 0 },
    { 0x922e6883, 3 },
    { 0x294a6883, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea6888, 3 },
    { 0x20be6888, 3 },
    { 0, 0 },
    { 0x36ee688b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa912688e, 3 },
    { 0xfeee688f, 3 },
    { 0x66e26890, 3 },
    { 0xc38a6891, 4 },
    { 0x42726892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d66898, 3 },
    { 0x47f26898, 3 },
    { 0, 0 },
    { 0x2742689b, 4 },
    { 0x8932689b, 5 },
    { 0xceae689d, 3 },
    { 0, 0 },
    { 0xe2ba689f, 4 },
    { 0xfd52689f, 3 },
    { 0x16668a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5e68ab, 3 },
    { 0, 0 },
    { 0xb8fe68ad, 4 },
    { 0x51d268ae, 3 },
    { 0x601a68af, 3 },
    { 0x26f668b0, 3 },
    { 0x676a68b0, 4 },
    { 0, 0 },
    { 0x1d5668b3, 4 },
    { 0, 0 },
    { 0xad2e68b5, 3 },
    { 0xe7d268b6, 4 },
    { 0xdc8e68b5, 3 },
    { 0x606668b5, 3 },
    { 0x356268b9, 3 },
    { 0xcc0a68ba, 3 },
    { 0x998668b7, 3 },
    { 0xae268bc, 3 },
    { 0xd4f268b9, 4 },
    { 0xf1fa68be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x266e68c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07268c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c268cb, 4 },
    { 0x107a68cc, 3 },
    { 0xa6fe68cc, 3 },
    { 0xbf7a68ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174668d2, 4 },
    { 0, 0 },
    { 0x115e68d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1668d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afa68de, 3 },
    { 0x445e68de, 4 },
    { 0xb80668e0, 3 },
    { 0xfdfa68e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176668e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57a68eb, 3 },
    { 0x280268ec, 3 },
    { 0, 0 },
    { 0x223e68ee, 4 },
    { 0xc33e68ef, 3 },
    { 0x8e2668f0, 3 },
    { 0xfb6268ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696268f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34268fa, 3 },
    { 0x67b268fb, 3 },
    { 0x2e4668fc, 4 },
    { 0, 0 },
    { 0x583268fe, 4 },
    { 0xa71268ff, 3 },
    { 0xe28268fe, 3 },
    { 0x9f2a6901, 3 },
    { 0, 0 },
    { 0xb23e6903, 3 },
    { 0xa4a6903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aa6907, 3 },
    { 0, 0 },
    { 0xfc926909, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaa690e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce766913, 3 },
    { 0xbdaa6913, 4 },
    { 0xdc6e6915, 3 },
    { 0x3e866916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd2691a, 3 },
    { 0, 0 },
    { 0x7e72691c, 3 },
    { 0, 0 },
    { 0x8f86691e, 4 },
    { 0xc5a2691f, 3 },
    { 0x88366920, 4 },
    { 0, 0 },
    { 0xe0da6922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a26925, 3 },
    { 0x32be6926, 4 },
    { 0xfd9a6926, 3 },
    { 0x3e0a6927, 3 },
    { 0x9bba6929, 3 },
    { 0xded6692a, 3 },
    { 0, 0 },
    { 0x35b2692c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ba6931, 3 },
    { 0xcf526932, 3 },
    { 0xc66a6933, 3 },
    { 0x2e466931, 3 },
    { 0x157e6934, 3 },
    { 0xbef66936, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9586693a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3326693f, 3 },
    { 0x1d86693f, 4 },
    { 0, 0 },
    { 0x57ce6942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5266946, 3 },
    { 0, 0 },
    { 0x4b2e6948, 3 },
    { 0x81de6948, 5 },
    { 0xd07e6949, 4 },
    { 0x1f2a694b, 3 },
    { 0xad4e694b, 4 },
    { 0, 0 },
    { 0x3332694e, 3 },
    { 0x792694f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a6955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a6960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e026966, 3 },
    { 0, 0 },
    { 0xfba6968, 4 },
    { 0x4ff66968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe66971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae26974, 3 },
    { 0x20a26974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36697b, 3 },
    { 0, 0 },
    { 0x888e697d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59a6980, 4 },
    { 0xd0166981, 3 },
    { 0x51a66981, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce6985, 3 },
    { 0, 0 },
    { 0xf80a6987, 3 },
    { 0x2b2e6988, 3 },
    { 0xd1fe6988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce6990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ea6993, 3 },
    { 0, 0 },
    { 0xae766995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18566999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bea699e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5e69a2, 3 },
    { 0, 0 },
    { 0xb15e69a4, 3 },
    { 0xf4fa69a4, 3 },
    { 0x65c669a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d269ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea69ae, 3 },
    { 0x9f8669af, 3 },
    { 0xc0c269b0, 4 },
    { 0x18c269b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c669b5, 3 },
    { 0x985269b6, 3 },
    { 0, 0 },
    { 0xbe3269b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215e69bd, 4 },
    { 0, 0 },
    { 0x332269bf, 4 },
    { 0, 0 },
    { 0x26ee69c1, 3 },
    { 0xc5b269c1, 4 },
    { 0x8ea269c1, 3 },
    { 0xf1a69c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00a69c7, 3 },
    { 0x34f269c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ae69cd, 3 },
    { 0, 0 },
    { 0x45c669cf, 3 },
    { 0x90fa69d0, 3 },
    { 0x28ba69d0, 3 },
    { 0x18e669cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7e69d5, 4 },
    { 0, 0 },
    { 0xcaee69d7, 3 },
    { 0, 0 },
    { 0x529669d9, 3 },
    { 0, 0 },
    { 0x987669db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5269df, 4 },
    { 0, 0 },
    { 0xb49269e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54e69e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d669f1, 3 },
    { 0xc7da69f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa69f7, 3 },
    { 0xefe269f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6669fe, 4 },
    { 0xba1669ff, 3 },
    { 0xfcba6a00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a6a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ce6a0c, 3 },
    { 0xef9a6a0d, 3 },
    { 0xd1de6a0d, 3 },
    { 0xc02e6a0e, 3 },
    { 0x334e6a10, 4 },
    { 0, 0 },
    { 0x2f926a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fda6a15, 3 },
    { 0xb826a16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31666a1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01e6a20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e66a23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9426a26, 3 },
    { 0, 0 },
    { 0xdd1e6a28, 3 },
    { 0xad666a28, 3 },
    { 0, 0 },
    { 0xbc066a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80e6a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc966a35, 3 },
    { 0x5ebe6a35, 3 },
    { 0xc58a6a37, 4 },
    { 0xd1c26a36, 3 },
    { 0, 0 },
    { 0x32066a3a, 4 },
    { 0, 0 },
    { 0x13b66a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7a6a44, 4 },
    { 0x1c3a6a44, 3 },
    { 0x48966a46, 3 },
    { 0x483a6a47, 3 },
    { 0x591e6a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5766a4e, 3 },
    { 0, 0 },
    { 0x7b0e6a50, 3 },
    { 0xbc9a6a51, 4 },
    { 0xaf4a6a52, 3 },
    { 0x7cb66a52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d726a5c, 3 },
    { 0, 0 },
    { 0x606a6a5e, 3 },
    { 0xfe9e6a5e, 4 },
    { 0, 0 },
    { 0x9a926a61, 3 },
    { 0xd5ce6a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c26a65, 4 },
    { 0xfdf66a65, 5 },
    { 0x83566a67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91be6a6c, 3 },
    { 0, 0 },
    { 0x5b4a6a6e, 3 },
    { 0x530a6a6e, 4 },
    { 0, 0 },
    { 0x84866a71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fa6a77, 3 },
    { 0x30726a77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0326a7c, 3 },
    { 0x8daa6a7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67a6a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7e6a86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde6a8a, 3 },
    { 0, 0 },
    { 0x834e6a8c, 3 },
    { 0x399e6a8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593e6a91, 3 },
    { 0, 0 },
    { 0xeee26a93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fe6a96, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x42826a99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade6aa3, 3 },
    { 0x48166aa4, 3 },
    { 0x36e66aa4, 3 },
    { 0x78be6aa4, 3 },
    { 0, 0 },
    { 0xf4f66aa8, 3 },
    { 0, 0 },
    { 0x49066aaa, 3 },
    { 0, 0 },
    { 0xf2be6aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e6ab3, 3 },
    { 0x9ce66ab4, 3 },
    { 0, 0 },
    { 0x7f4e6ab6, 3 },
    { 0xb9e66ab7, 3 },
    { 0, 0 },
    { 0x110e6ab9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x83266abc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3e6ac0, 3 },
    { 0x4e226ac0, 3 },
    { 0x15ea6ac1, 4 },
    { 0x8f26ac3, 4 },
    { 0x78ba6ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45626ac7, 3 },
    { 0xb17a6ac8, 3 },
    { 0, 0 },
    { 0xb2e6aca, 3 },
    { 0, 0 },
    { 0x5c26acc, 3 },
    { 0x42e66acc, 3 },
    { 0, 0 },
    { 0x63b66acf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa6ad5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a6ad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a6ae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48266ae6, 3 },
    { 0x7b666ae6, 4 },
    { 0xfe66ae8, 3 },
    { 0xad3a6ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a6aef, 3 },
    { 0x53fa6af0, 3 },
    { 0, 0 },
    { 0xdafe6af2, 3 },
    { 0xc28e6af2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed26af8, 3 },
    { 0x54566af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d66afc, 3 },
    { 0x413a6afc, 3 },
    { 0, 0 },
    { 0xfcd66aff, 4 },
    { 0x49d26b00, 4 },
    { 0x110e6b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82226b04, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ca6b0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c26b0e, 3 },
    { 0x595a6b0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f326b1a, 3 },
    { 0x9e3a6b1a, 3 },
    { 0x76126b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab26b22, 3 },
    { 0xbf566b22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a826b2a, 3 },
    { 0x77e66b2b, 3 },
    { 0x775a6b2a, 3 },
    { 0, 0 },
    { 0x501e6b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97566b32, 4 },
    { 0, 0 },
    { 0xd3a6b34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a26b39, 3 },
    { 0x972e6b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5a6b3e, 3 },
    { 0, 0 },
    { 0x7a666b40, 3 },
    { 0, 0 },
    { 0x404a6b42, 4 },
    { 0x5d966b43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d926b46, 3 },
    { 0xf2b66b47, 3 },
    { 0, 0 },
    { 0xadfe6b49, 3 },
    { 0x6b26b4a, 3 },
    { 0x74366b4a, 4 },
    { 0xbee6b4b, 3 },
    { 0x4e1e6b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778a6b52, 3 },
    { 0xbe6e6b53, 3 },
    { 0x3faa6b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef66b57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5666b5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7526b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e26b65, 3 },
    { 0x6a3e6b66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e6b6d, 3 },
    { 0x40be6b6e, 4 },
    { 0x771a6b6e, 4 },
    { 0x5e126b70, 4 },
    { 0, 0 },
    { 0xeb4e6b72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7666b76, 3 },
    { 0xa41e6b77, 3 },
    { 0xd8fa6b78, 3 },
    { 0x242e6b79, 3 },
    { 0, 0 },
    { 0xd52a6b7b, 3 },
    { 0xbf226b7c, 4 },
    { 0, 0 },
    { 0x76726b7e, 3 },
    { 0x5d9a6b7f, 3 },
    { 0xdd626b80, 3 },
    { 0x9ae6b7f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cda6b85, 3 },
    { 0x37d66b86, 5 },
    { 0xcd926b87, 3 },
    { 0xb14e6b88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa6b8b, 3 },
    { 0, 0 },
    { 0x86026b8d, 3 },
    { 0xa6b66b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94726b93, 3 },
    { 0xe2066b94, 3 },
    { 0xa92a6b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68426b9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f366ba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a866ba3, 3 },
    { 0xea666ba3, 3 },
    { 0xee26ba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab026ba9, 4 },
    { 0xfd926baa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa766baf, 5 },
    { 0, 0 },
    { 0x291e6bb1, 3 },
    { 0, 0 },
    { 0xad8e6bb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f366bb6, 3 },
    { 0x3ede6bb6, 3 },
    { 0x6b8e6bb6, 3 },
    { 0x9cc66bb8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66da6bbc, 3 },
    { 0, 0 },
    { 0xd6866bbe, 4 },
    { 0x80326bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf266bc2, 3 },
    { 0x99da6bc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16626bc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b966bcc, 3 },
    { 0x90c66bcd, 3 },
    { 0, 0 },
    { 0x9f4a6bcf, 3 },
    { 0xb58e6bd0, 3 },
    { 0, 0 },
    { 0x21966bd2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db26bd6, 3 },
    { 0x80526bd6, 3 },
    { 0x5f526bd6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x92726bdb, 3 },
    { 0x8a166bdc, 4 },
    { 0x264a6bdd, 3 },
    { 0, 0 },
    { 0x31526bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabee6be5, 3 },
    { 0, 0 },
    { 0xc4e66be7, 3 },
    { 0, 0 },
    { 0x745a6be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6526bf0, 3 },
    { 0x92a26bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e866bfc, 3 },
    { 0x94466bfd, 3 },
    { 0xa84a6bfd, 3 },
    { 0, 0 },
    { 0x49226c00, 3 },
    { 0x823e6c00, 3 },
    { 0xf7da6c01, 3 },
    { 0x1c666c01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85926c07, 3 },
    { 0x1ab66c08, 4 },
    { 0xd7166c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1666c12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71a6c1a, 3 },
    { 0x1aee6c1a, 4 },
    { 0xa0166c1c, 3 },
    { 0x2c9a6c1d, 3 },
    { 0xda5a6c1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6626c21, 3 },
    { 0, 0 },
    { 0x2c5e6c23, 3 },
    { 0xbe326c24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a6c28, 3 },
    { 0x50066c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82a6c2c, 3 },
    { 0x671e6c2d, 4 },
    { 0x679e6c2d, 3 },
    { 0xb83e6c2f, 3 },
    { 0xec166c30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53626c33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabea6c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58526c40, 3 },
    { 0x1b5a6c40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba6c46, 5 },
    { 0xe9f26c47, 3 },
    { 0, 0 },
    { 0xa0a26c49, 3 },
    { 0, 0 },
    { 0x31126c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa6c4e, 3 },
    { 0, 0 },
    { 0xc0326c50, 3 },
    { 0x90c26c51, 4 },
    { 0xcf926c51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caa6c59, 4 },
    { 0xa86a6c59, 3 },
    { 0xfb1a6c5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a6c63, 3 },
    { 0x900e6c63, 3 },
    { 0x433e6c64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e626c69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2e6c6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e6c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9e6c75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fe6c7c, 5 },
    { 0x84b66c7c, 4 },
    { 0xbbda6c7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24026c82, 3 },
    { 0, 0 },
    { 0xd28a6c84, 3 },
    { 0xdaa66c84, 4 },
    { 0xa80a6c84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb66c8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x37766c8e, 3 },
    { 0x8a3a6c8f, 5 },
    { 0x3a0e6c90, 3 },
    { 0x333e6c91, 4 },
    { 0xa8926c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6a6c95, 3 },
    { 0, 0 },
    { 0x4b3a6c97, 3 },
    { 0, 0 },
    { 0x301a6c99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c66c9d, 3 },
    { 0, 0 },
    { 0x3dd26c9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2e6ca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3e6caa, 3 },
    { 0xdd0a6cab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dba6cb0, 3 },
    { 0x384a6cb1, 4 },
    { 0, 0 },
    { 0x73de6cb3, 4 },
    { 0x40aa6cb3, 3 },
    { 0xea5a6cb5, 3 },
    { 0xe8266cb4, 3 },
    { 0xe7366cb7, 3 },
    { 0, 0 },
    { 0xcee26cb9, 4 },
    { 0, 0 },
    { 0x807a6cbb, 3 },
    { 0x20426cbc, 3 },
    { 0x87866cbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f026cc1, 3 },
    { 0x7cfa6cc2, 3 },
    { 0xb4626cc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835e6cc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551a6ccc, 4 },
    { 0x17326ccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b926cd6, 3 },
    { 0x7d26cd6, 3 },
    { 0xc1a66cd8, 3 },
    { 0, 0 },
    { 0xbef26cda, 4 },
    { 0, 0 },
    { 0x4966cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f66cdf, 3 },
    { 0x14266cdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457e6ce4, 3 },
    { 0x8eea6ce4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4166ce8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbe6cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37966cf3, 3 },
    { 0x5a526cf3, 4 },
    { 0xa0626cf5, 3 },
    { 0xfaa66cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae6cff, 3 },
    { 0, 0 },
    { 0x400e6d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7a6d09, 3 },
    { 0, 0 },
    { 0xc8fa6d0b, 4 },
    { 0x2ffe6d0c, 3 },
    { 0x30f26d0d, 3 },
    { 0x5d0e6d0c, 3 },
    { 0x27c66d0f, 3 },
    { 0x445a6d10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5226d14, 3 },
    { 0, 0 },
    { 0x2b7a6d16, 4 },
    { 0xd8266d16, 3 },
    { 0x93ea6d16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de6d1c, 3 },
    { 0, 0 },
    { 0x9baa6d1e, 4 },
    { 0x4de26d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba6d22, 3 },
    { 0xde7a6d23, 4 },
    { 0x96ba6d24, 3 },
    { 0xbfaa6d25, 3 },
    { 0x60e26d26, 3 },
    { 0xa366d26, 3 },
    { 0, 0 },
    { 0x33fa6d29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc26d2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff666d2f, 3 },
    { 0x5e666d30, 3 },
    { 0xfc326d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e6d34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef26d38, 3 },
    { 0x1f66d39, 4 },
    { 0x10ca6d3a, 4 },
    { 0xc17e6d38, 5 },
    { 0, 0 },
    { 0x3aaa6d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666d41, 3 },
    { 0x7ae26d42, 4 },
    { 0x92026d43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x855e6d46, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3966d4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f366d4d, 3 },
    { 0, 0 },
    { 0xef366d4f, 3 },
    { 0x412a6d50, 3 },
    { 0, 0 },
    { 0x9e426d52, 3 },
    { 0, 0 },
    { 0xf0a66d54, 4 },
    { 0xb29a6d55, 4 },
    { 0x333a6d54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2666d5f, 3 },
    { 0x29ee6d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60826d65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8a6d6d, 3 },
    { 0x440e6d6e, 3 },
    { 0x49826d6e, 3 },
    { 0xf6466d6d, 5 },
    { 0x6afa6d6f, 3 },
    { 0, 0 },
    { 0xb326d73, 3 },
    { 0, 0 },
    { 0x8b6a6d75, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a6d7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25e6d7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401e6d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc826d8b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35226d93, 3 },
    { 0xf3a66d93, 4 },
    { 0xb1e66d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d26d99, 3 },
    { 0xe6ea6d9a, 3 },
    { 0, 0 },
    { 0x582e6d9c, 3 },
    { 0x37726d9d, 3 },
    { 0x904a6d9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af66da1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3e6da6, 3 },
    { 0x3aa26da6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9126dab, 4 },
    { 0x9f066dac, 3 },
    { 0xa72a6dad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a6db1, 3 },
    { 0x67fa6db2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a6dba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa6dbd, 4 },
    { 0, 0 },
    { 0xd0666dbf, 3 },
    { 0xd84a6dc0, 4 },
    { 0x4f2a6dc1, 4 },
    { 0, 0 },
    { 0x590e6dc3, 4 },
    { 0xc6e26dc4, 3 },
    { 0, 0 },
    { 0x3b826dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fa6dca, 4 },
    { 0x9ae6dcb, 3 },
    { 0xb4d66dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b26dcf, 3 },
    { 0, 0 },
    { 0x9b366dd1, 5 },
    { 0, 0 },
    { 0x44a6dd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af26dd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b26ddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16a6ddf, 3 },
    { 0, 0 },
    { 0x4ae6de1, 4 },
    { 0, 0 },
    { 0x99ce6de3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6a6de8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb66deb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7226def, 3 },
    { 0xdbb26def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f826df6, 3 },
    { 0, 0 },
    { 0xd7266df8, 3 },
    { 0xf78e6df8, 4 },
    { 0x227a6df9, 3 },
    { 0x612e6df9, 3 },
    { 0x9d666dfc, 4 },
    { 0x788a6df8, 3 },
    { 0x36826dfe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f166e01, 3 },
    { 0x8b126e02, 4 },
    { 0x3dce6e03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a926e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b926e10, 3 },
    { 0x1cc26e10, 3 },
    { 0x500a6e12, 4 },
    { 0x6d326e13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09e6e16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5a6e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ce6e23, 3 },
    { 0, 0 },
    { 0x8c8e6e25, 4 },
    { 0xf7f26e26, 4 },
    { 0, 0 },
    { 0x91ce6e28, 3 },
    { 0xc6ca6e29, 3 },
    { 0x69a26e2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8a6e2d, 3 },
    { 0x5c26e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfa6e34, 3 },
    { 0xe9ee6e35, 3 },
    { 0, 0 },
    { 0xc5166e37, 3 },
    { 0x385e6e37, 4 },
    { 0x253a6e39, 3 },
    { 0, 0 },
    { 0xd2fe6e3b, 3 },
    { 0x2efa6e3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725e6e46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6726e49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f66e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ce6e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccee6e54, 3 },
    { 0x14f26e54, 4 },
    { 0xa7a66e54, 4 },
    { 0x26926e57, 3 },
    { 0, 0 },
    { 0xc66a6e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a6e5d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ea6e60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b126e64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0e6e6a, 3 },
    { 0xb8e6e6b, 3 },
    { 0x43466e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99da6e70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc66e74, 3 },
    { 0, 0 },
    { 0xc9926e76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61926e7f, 3 },
    { 0, 0 },
    { 0x55fa6e81, 5 },
    { 0x56ba6e82, 4 },
    { 0xd60a6e83, 3 },
    { 0, 0 },
    { 0x4b66e85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67266e91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47366e96, 3 },
    { 0xca726e97, 3 },
    { 0x12166e98, 3 },
    { 0xb15e6e99, 3 },
    { 0x606a6e97, 3 },
    { 0x11a26e97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32a6ea0, 3 },
    { 0x3826ea1, 3 },
    { 0xdce6ea1, 3 },
    { 0xffee6ea3, 4 },
    { 0xf1c66ea4, 3 },
    { 0x114a6ea0, 3 },
    { 0, 0 },
    { 0x89126ea7, 3 },
    { 0x8ff26ea8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a926eae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ee6ebe, 4 },
    { 0, 0 },
    { 0x11c66ec0, 3 },
    { 0xf85e6ec0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb26ec4, 3 },
    { 0, 0 },
    { 0x39c26ec6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37826ec9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee6ecc, 3 },
    { 0x60ee6ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae6ed5, 5 },
    { 0xb0c66ed5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b26ed9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eba6ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6ee8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ee6eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a26ef0, 3 },
    { 0x41126ef1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b266ef7, 3 },
    { 0, 0 },
    { 0xb91a6ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ba6efc, 4 },
    { 0, 0 },
    { 0xaf566efe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754e6f07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc26f12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75326f15, 3 },
    { 0x87366f16, 3 },
    { 0xa2be6f15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74026f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fe6f22, 3 },
    { 0x6b2e6f22, 3 },
    { 0, 0 },
    { 0xa566f25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x696a6f28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4a6f2d, 3 },
    { 0, 0 },
    { 0x86566f2f, 3 },
    { 0x3e3e6f30, 4 },
    { 0, 0 },
    { 0x968e6f32, 4 },
    { 0x39166f33, 3 },
    { 0x99d66f34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa866f37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed66f3b, 3 },
    { 0x73ba6f3b, 3 },
    { 0xd5c66f3b, 3 },
    { 0x39726f3d, 4 },
    { 0xd8226f3c, 3 },
    { 0x4a5a6f40, 3 },
    { 0x83be6f3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f46, 3 },
    { 0x10ca6f47, 4 },
    { 0xa6ee6f47, 3 },
    { 0, 0 },
    { 0x71e26f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba6f4e, 4 },
    { 0xf79e6f4f, 4 },
    { 0x657a6f4f, 4 },
    { 0x2cb66f51, 3 },
    { 0x258a6f52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb766f57, 3 },
    { 0x461a6f58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7a6f5e, 4 },
    { 0, 0 },
    { 0xc65a6f60, 3 },
    { 0x63ca6f60, 3 },
    { 0xd0266f62, 3 },
    { 0x4e066f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1a6f67, 4 },
    { 0x31026f68, 3 },
    { 0xa4466f67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x856a6f6c, 3 },
    { 0xe48a6f6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc266f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87e6f7f, 3 },
    { 0xa58e6f80, 4 },
    { 0xfcbe6f81, 3 },
    { 0xd5326f80, 3 },
    { 0x365e6f81, 4 },
    { 0x2afe6f7f, 3 },
    { 0xa81a6f81, 4 },
    { 0x36fa6f85, 3 },
    { 0xf66e6f87, 3 },
    { 0xf89a6f88, 4 },
    { 0x9d3e6f88, 3 },
    { 0xe88e6f89, 4 },
    { 0, 0 },
    { 0x48aa6f8c, 4 },
    { 0xcd9a6f8c, 3 },
    { 0x59166f8d, 4 },
    { 0xf8066f8d, 3 },
    { 0, 0 },
    { 0x465a6f91, 3 },
    { 0, 0 },
    { 0xf6ee6f93, 3 },
    { 0, 0 },
    { 0x39266f95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de6f98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a6f9b, 4 },
    { 0, 0 },
    { 0xa39a6f9d, 3 },
    { 0xaf8a6f9e, 3 },
    { 0x5c2a6f9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x78266fa2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ea6fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee6fb3, 4 },
    { 0xd3c26fb4, 3 },
    { 0x39466fb5, 3 },
    { 0x20ea6fb6, 4 },
    { 0x76426fb6, 3 },
    { 0, 0 },
    { 0x67c66fb9, 3 },
    { 0xbbc26fba, 3 },
    { 0x6466fba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db66fc6, 3 },
    { 0x523e6fc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f66fd0, 3 },
    { 0x19a26fd1, 3 },
    { 0xa9d26fd1, 3 },
    { 0xa9a6fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78666fd7, 3 },
    { 0, 0 },
    { 0xaeda6fd9, 3 },
    { 0x40be6fda, 3 },
    { 0x925a6fda, 3 },
    { 0x74166fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea6fe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65666fe3, 4 },
    { 0, 0 },
    { 0x42b26fe5, 3 },
    { 0, 0 },
    { 0x2bb66fe7, 3 },
    { 0x57a26fe8, 3 },
    { 0x58426fe9, 3 },
    { 0x5aea6fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6226fef, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa26ff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a6ffb, 4 },
    { 0, 0 },
    { 0xeb166ffd, 3 },
    { 0x69a66ffe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c667003, 3 },
    { 0x87367003, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc42700a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x557a700d, 3 },
    { 0x5a6e700e, 3 },
    { 0xfd3e700f, 3 },
    { 0, 0 },
    { 0x183a7011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637e7015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91a7018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdee701b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64e701f, 4 },
    { 0, 0 },
    { 0xc5ea7021, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba7029, 3 },
    { 0x923a7029, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e7030, 3 },
    { 0xfb927031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd67034, 4 },
    { 0, 0 },
    { 0xd43a7036, 4 },
    { 0xdb67037, 3 },
    { 0xb1c27036, 3 },
    { 0x9d1e7039, 3 },
    { 0xcd6703a, 3 },
    { 0x1026703b, 3 },
    { 0xe5ce703b, 3 },
    { 0x63ce703d, 3 },
    { 0x67f2703d, 4 },
    { 0, 0 },
    { 0x4f3e7040, 3 },
    { 0x15967040, 3 },
    { 0xa5d27041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14a7045, 3 },
    { 0, 0 },
    { 0x57d27047, 3 },
    { 0, 0 },
    { 0xc80a7049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad2704e, 3 },
    { 0x7a62704f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c67054, 3 },
    { 0xd1467055, 4 },
    { 0, 0 },
    { 0x5d127057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac6705a, 3 },
    { 0xdcea705b, 3 },
    { 0xf982705c, 3 },
    { 0x8ffa705c, 3 },
    { 0xb94e705c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be7061, 3 },
    { 0xffa67062, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77a706d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5a7074, 3 },
    { 0, 0 },
    { 0x8b5e7076, 3 },
    { 0xf8f67077, 3 },
    { 0x18da7078, 3 },
    { 0, 0 },
    { 0xdc56707a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4526707e, 4 },
    { 0, 0 },
    { 0x520a7080, 3 },
    { 0xb9c27080, 3 },
    { 0x65327082, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0267085, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a7088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae667090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca127093, 4 },
    { 0xc6027094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95e7098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2709f, 3 },
    { 0xe53e709f, 3 },
    { 0, 0 },
    { 0x829270a2, 3 },
    { 0xbda670a3, 3 },
    { 0xbb0a70a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7e70a9, 3 },
    { 0, 0 },
    { 0xa87e70ab, 3 },
    { 0, 0 },
    { 0xcc7270ad, 3 },
    { 0xf74e70ae, 3 },
    { 0x5fa670af, 3 },
    { 0, 0 },
    { 0x161a70b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ba70b5, 3 },
    { 0, 0 },
    { 0xc10a70b7, 3 },
    { 0xfe8670b8, 3 },
    { 0, 0 },
    { 0x5ce670ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e70c1, 3 },
    { 0, 0 },
    { 0x33c670c3, 3 },
    { 0xdfa270c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x439e70c7, 4 },
    { 0x37ae70c8, 3 },
    { 0x7d8270c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27be70cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74670d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57be70d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3e70d7, 4 },
    { 0x2bea70d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1670e7, 3 },
    { 0xf73e70e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48a70f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8270f5, 3 },
    { 0x164270f6, 3 },
    { 0xeede70f7, 3 },
    { 0xb91670f8, 3 },
    { 0x786670f8, 3 },
    { 0x8cc270f9, 3 },
    { 0x7ffe70f6, 3 },
    { 0, 0 },
    { 0x55f270fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5667102, 3 },
    { 0xac6a7103, 4 },
    { 0xcb067103, 3 },
    { 0xb8327105, 4 },
    { 0x81e7106, 4 },
    { 0xa18e7106, 3 },
    { 0xd3267105, 3 },
    { 0xcb6a7109, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a710d, 3 },
    { 0xd9ca710d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d327113, 3 },
    { 0, 0 },
    { 0xe6767115, 4 },
    { 0xc9967116, 3 },
    { 0x758a7117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad67128, 3 },
    { 0, 0 },
    { 0x7376712a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8a712f, 3 },
    { 0x95e67130, 3 },
    { 0x266a7131, 3 },
    { 0xdb3e7132, 5 },
    { 0x7c3a7133, 4 },
    { 0x651e7134, 3 },
    { 0x34267135, 3 },
    { 0x621e7136, 4 },
    { 0, 0 },
    { 0x62da7138, 3 },
    { 0x64267139, 3 },
    { 0x4fa6713a, 5 },
    { 0x38e6713b, 4 },
    { 0xa42e713b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c76713f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc67142, 3 },
    { 0, 0 },
    { 0x90de7144, 3 },
    { 0x40a7144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e27148, 3 },
    { 0xae467149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a2714d, 3 },
    { 0xadfe714e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abe7151, 4 },
    { 0, 0 },
    { 0xd4ce7153, 4 },
    { 0x22a7153, 3 },
    { 0, 0 },
    { 0x304e7156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedae7159, 3 },
    { 0xfaf6715a, 3 },
    { 0xaaf6715b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d27160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd67163, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7a7168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7056716e, 3 },
    { 0, 0 },
    { 0xd2fe7170, 3 },
    { 0xd3c67171, 3 },
    { 0x81267172, 4 },
    { 0, 0 },
    { 0xf7727174, 3 },
    { 0x7a3a7175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa6717a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb867185, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862a7190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231a7196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb6719b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6e719f, 3 },
    { 0x90a271a0, 3 },
    { 0x6ae271a1, 4 },
    { 0, 0 },
    { 0xed7a71a3, 3 },
    { 0, 0 },
    { 0xaab671a5, 4 },
    { 0x213271a5, 3 },
    { 0x255e71a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x928271aa, 3 },
    { 0, 0 },
    { 0x512a71ac, 4 },
    { 0xf2e271ad, 3 },
    { 0x7b271ae, 3 },
    { 0x671af, 4 },
    { 0x950a71ac, 3 },
    { 0xaf3e71ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f271b4, 3 },
    { 0, 0 },
    { 0x86671b6, 3 },
    { 0, 0 },
    { 0xc6ba71b8, 4 },
    { 0xa78671b9, 3 },
    { 0xa59a71b8, 3 },
    { 0xc7b671ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194a71c3, 3 },
    { 0x822671c4, 3 },
    { 0x34f271c4, 3 },
    { 0x641a71c6, 3 },
    { 0xfcde71c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4e71cb, 4 },
    { 0xa5c271cc, 4 },
    { 0, 0 },
    { 0x18d671ce, 3 },
    { 0, 0 },
    { 0x2ee271d0, 3 },
    { 0xfa5e71d1, 3 },
    { 0x53ca71d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b271d6, 3 },
    { 0xc0d671d6, 3 },
    { 0x60fa71d8, 3 },
    { 0x79f271d8, 3 },
    { 0xb88e71d6, 3 },
    { 0x962a71db, 3 },
    { 0x80f271dc, 3 },
    { 0x9cf271dc, 3 },
    { 0x768271dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388671e2, 3 },
    { 0, 0 },
    { 0x160a71e4, 3 },
    { 0, 0 },
    { 0xa81a71e6, 3 },
    { 0xe17671e7, 3 },
    { 0xed1a71e7, 4 },
    { 0xcbaa71e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebea71ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58271ef, 3 },
    { 0, 0 },
    { 0xec9a71f1, 3 },
    { 0xe67671f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d671f6, 3 },
    { 0xa39271f6, 3 },
    { 0x83e671f6, 3 },
    { 0x8f9671f9, 4 },
    { 0x21a671f6, 3 },
    { 0xc8be71f9, 3 },
    { 0, 0 },
    { 0x3aa71fd, 3 },
    { 0xbeba71fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3de7201, 3 },
    { 0xc9da7202, 3 },
    { 0x1aea7202, 3 },
    { 0xddfe7204, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac67208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757e720d, 3 },
    { 0xd4da720d, 3 },
    { 0x7c8a720d, 3 },
    { 0x13c2720e, 3 },
    { 0x9e4a720f, 3 },
    { 0, 0 },
    { 0xea3e7213, 4 },
    { 0, 0 },
    { 0xf7227215, 3 },
    { 0, 0 },
    { 0xefe7217, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f46721b, 3 },
    { 0xc42721c, 4 },
    { 0xdaee721d, 3 },
    { 0xe9be721e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a7221, 4 },
    { 0x14ba7221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a7227, 3 },
    { 0x4efa7227, 4 },
    { 0xd8927228, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aca723c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fe7247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e724d, 3 },
    { 0, 0 },
    { 0x4c12724f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7467256, 3 },
    { 0xb8ee7257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e725a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a82725d, 5 },
    { 0xb696725e, 4 },
    { 0x7bde725e, 3 },
    { 0xf5e27260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37de7263, 3 },
    { 0xd3b27264, 3 },
    { 0xb24a7265, 3 },
    { 0x409a7263, 3 },
    { 0, 0 },
    { 0x82827268, 3 },
    { 0, 0 },
    { 0x60ae726a, 3 },
    { 0, 0 },
    { 0x4aee726c, 4 },
    { 0xcf66726d, 3 },
    { 0, 0 },
    { 0x173e726f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e7272, 3 },
    { 0, 0 },
    { 0xb6067274, 5 },
    { 0x6bfa7274, 3 },
    { 0xcf167276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74927279, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x594e727c, 3 },
    { 0, 0 },
    { 0x663e727e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31027281, 5 },
    { 0x406e7282, 3 },
    { 0xf6267283, 3 },
    { 0x8dae7284, 3 },
    { 0, 0 },
    { 0x29d27286, 3 },
    { 0xb6127287, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42728a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0a7290, 3 },
    { 0x1b3e7291, 3 },
    { 0xd8727292, 3 },
    { 0, 0 },
    { 0x55be7294, 4 },
    { 0, 0 },
    { 0xb26e7296, 3 },
    { 0x20427297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fa729a, 4 },
    { 0xade2729a, 3 },
    { 0x3afe729c, 3 },
    { 0xf1e2729d, 4 },
    { 0x1f32729e, 3 },
    { 0x825e729f, 3 },
    { 0x80ba729e, 3 },
    { 0xf0c272a1, 3 },
    { 0x7e1672a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58272ab, 3 },
    { 0x85a272ab, 4 },
    { 0x787672ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a672b1, 4 },
    { 0x3b5272b2, 3 },
    { 0, 0 },
    { 0x4b8672b4, 3 },
    { 0x4e6672b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x977a72b8, 3 },
    { 0x7e7a72b9, 4 },
    { 0xc80272b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528272be, 3 },
    { 0x958e72be, 3 },
    { 0xe57e72c0, 3 },
    { 0xa99a72c1, 3 },
    { 0, 0 },
    { 0x359272c3, 4 },
    { 0x35ca72c3, 5 },
    { 0x2e4672c5, 4 },
    { 0, 0 },
    { 0x8e1672c7, 3 },
    { 0x70aa72c7, 3 },
    { 0xe3c672c9, 3 },
    { 0, 0 },
    { 0xb20e72cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e72d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524272d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x911e72d8, 4 },
    { 0xe95e72d8, 4 },
    { 0xc4272d9, 4 },
    { 0x6cf672d8, 3 },
    { 0x3672d9, 3 },
    { 0x2c3672dd, 3 },
    { 0xc4da72de, 4 },
    { 0xfdb272db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a72e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c272e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6272ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2272fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66272ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5e7304, 3 },
    { 0, 0 },
    { 0x62fe7306, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98967309, 3 },
    { 0x58267309, 3 },
    { 0x1a1e730a, 3 },
    { 0xe862730c, 3 },
    { 0x9bd6730a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a7311, 4 },
    { 0x98367312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca7315, 3 },
    { 0xddf27316, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3a7319, 3 },
    { 0x6a4a7319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9362731d, 3 },
    { 0xd956731e, 4 },
    { 0x4efe731f, 3 },
    { 0, 0 },
    { 0xde2a7321, 3 },
    { 0, 0 },
    { 0x2e8a7323, 3 },
    { 0x77467324, 3 },
    { 0, 0 },
    { 0x3f527326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce27329, 4 },
    { 0, 0 },
    { 0x828a732b, 4 },
    { 0xdc6e732c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75667331, 4 },
    { 0, 0 },
    { 0xe0867333, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca27337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e2733e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19a7344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e667347, 3 },
    { 0, 0 },
    { 0x3b27349, 3 },
    { 0x41867349, 3 },
    { 0x112734b, 4 },
    { 0xf7f2734c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3527353, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79a7357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e735a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2e7362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf467366, 3 },
    { 0x72867367, 4 },
    { 0x48667368, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2836736d, 3 },
    { 0x753e736e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f67374, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e6737a, 4 },
    { 0x470a737b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x114e737e, 3 },
    { 0x2c76737e, 3 },
    { 0xb24a7380, 3 },
    { 0xb6b2737f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a12738a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77567391, 3 },
    { 0x6ff27391, 4 },
    { 0, 0 },
    { 0xcdfe7394, 3 },
    { 0x4d427394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60a739a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a56739d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c673a1, 4 },
    { 0, 0 },
    { 0x132a73a3, 4 },
    { 0, 0 },
    { 0x65673a5, 3 },
    { 0x11be73a5, 3 },
    { 0xa40a73a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x624673aa, 3 },
    { 0x1eda73aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aa73b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e73b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3de73bc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315273c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37673c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287273c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678a73cf, 4 },
    { 0xb4b273d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15da73d3, 4 },
    { 0xd7a73d4, 3 },
    { 0xdcce73d5, 3 },
    { 0xd70673d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587e73da, 3 },
    { 0xb4ca73da, 3 },
    { 0x993a73dc, 3 },
    { 0x719673dd, 3 },
    { 0x745673de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x398a73e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aa73e6, 3 },
    { 0x91fa73e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a73f0, 3 },
    { 0, 0 },
    { 0xd06e73f2, 3 },
    { 0xc3ae73f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9273f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566e73fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e7402, 3 },
    { 0xe9ea7402, 4 },
    { 0x619e7402, 3 },
    { 0xe6ee7405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2227408, 4 },
    { 0x69667409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e740c, 3 },
    { 0xeb0a740d, 3 },
    { 0x1b0e740d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e7421, 3 },
    { 0x2d2e7422, 3 },
    { 0x33aa7422, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77e7426, 3 },
    { 0xaeca7426, 3 },
    { 0xabb67428, 3 },
    { 0xa5527427, 3 },
    { 0x69c6742a, 3 },
    { 0x587e742b, 3 },
    { 0x7d76742c, 4 },
    { 0xb996742d, 3 },
    { 0xe6a6742e, 3 },
    { 0x5ef27428, 3 },
    { 0, 0 },
    { 0x78e27431, 3 },
    { 0x1367432, 4 },
    { 0x11667431, 3 },
    { 0xd4a67434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7437, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a96743c, 3 },
    { 0xa022743d, 3 },
    { 0x7ab6743e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05a7442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e27445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8da7449, 3 },
    { 0xb14e744a, 3 },
    { 0x3b56744b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a7450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82927453, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5167456, 3 },
    { 0xbc8a7457, 4 },
    { 0, 0 },
    { 0xfc267459, 3 },
    { 0, 0 },
    { 0xeeee745b, 3 },
    { 0x211a745b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf666745f, 3 },
    { 0, 0 },
    { 0xbc7e7461, 3 },
    { 0, 0 },
    { 0x77f67463, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c267468, 3 },
    { 0xd6927469, 3 },
    { 0x6eca7468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d2746d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488e7473, 3 },
    { 0, 0 },
    { 0x87c27475, 3 },
    { 0x93027476, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a747b, 3 },
    { 0xf356747c, 3 },
    { 0x398e747c, 3 },
    { 0x60f2747e, 3 },
    { 0x9f12747d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e7482, 4 },
    { 0xbe8a7483, 4 },
    { 0x21a7484, 3 },
    { 0xb9ca7484, 3 },
    { 0x63567482, 3 },
    { 0x78da7482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186748c, 3 },
    { 0xf7e748d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291a7495, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca7499, 4 },
    { 0x13767499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba74a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e74a5, 3 },
    { 0x8a7a74a5, 3 },
    { 0xb75e74a5, 3 },
    { 0xff674a8, 3 },
    { 0, 0 },
    { 0x6e3a74aa, 5 },
    { 0xdeaa74aa, 3 },
    { 0xa2ca74aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3de74af, 3 },
    { 0xf5b674b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1274b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x923a74b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca74bf, 4 },
    { 0, 0 },
    { 0x533674c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96e74c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27be74cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf274cf, 3 },
    { 0, 0 },
    { 0xbcda74d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5674d5, 3 },
    { 0xcbfa74d5, 3 },
    { 0x17a74d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2de74dd, 3 },
    { 0xae3674de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a74e4, 3 },
    { 0, 0 },
    { 0x4cbe74e6, 4 },
    { 0xf90a74e7, 3 },
    { 0, 0 },
    { 0xff2674e9, 3 },
    { 0x740a74ea, 4 },
    { 0x973a74eb, 4 },
    { 0x452e74ec, 3 },
    { 0xb80674eb, 3 },
    { 0xfb1274e9, 3 },
    { 0, 0 },
    { 0x50fa74f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a74f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ea74fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa7500, 3 },
    { 0x270e7500, 4 },
    { 0xaec27502, 3 },
    { 0x88ee7502, 4 },
    { 0xb1ba7504, 3 },
    { 0x21b67501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961a7509, 4 },
    { 0x9562750a, 3 },
    { 0x5a06750a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2a7513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db67517, 3 },
    { 0x24527518, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468e751f, 3 },
    { 0x68627520, 3 },
    { 0x3ae27520, 3 },
    { 0, 0 },
    { 0x7a927523, 3 },
    { 0x668e7524, 5 },
    { 0x322e7524, 3 },
    { 0xbc167524, 3 },
    { 0, 0 },
    { 0x553e7528, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd846752d, 4 },
    { 0xf9de752e, 5 },
    { 0xa696752f, 3 },
    { 0xe5ce7530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74267534, 3 },
    { 0, 0 },
    { 0x1c3e7536, 3 },
    { 0xa87a7536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6753a, 3 },
    { 0, 0 },
    { 0xc14a753c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a7541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eda7548, 3 },
    { 0x38867549, 3 },
    { 0x7a167548, 3 },
    { 0, 0 },
    { 0xd64a754c, 3 },
    { 0xc292754c, 3 },
    { 0, 0 },
    { 0x8cba754f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96027553, 4 },
    { 0, 0 },
    { 0xd75a7555, 3 },
    { 0x51367556, 4 },
    { 0, 0 },
    { 0x7a9e7558, 3 },
    { 0x7fd27559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c62755c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce755f, 3 },
    { 0, 0 },
    { 0x477a7561, 3 },
    { 0x1dc27561, 4 },
    { 0, 0 },
    { 0xc08a7564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a367569, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d2756c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33167570, 3 },
    { 0x5f227571, 3 },
    { 0x83227572, 3 },
    { 0x265a7572, 3 },
    { 0, 0 },
    { 0x71b27575, 3 },
    { 0xf10e7576, 5 },
    { 0x287e7577, 4 },
    { 0, 0 },
    { 0x77527579, 3 },
    { 0xdb0e757a, 3 },
    { 0x88d2757b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee757f, 4 },
    { 0xfb167580, 3 },
    { 0x508e757f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a7585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f67589, 3 },
    { 0, 0 },
    { 0x72ce758b, 3 },
    { 0x3ca2758c, 3 },
    { 0x2e8a758d, 5 },
    { 0, 0 },
    { 0x77da758f, 5 },
    { 0xc2e27590, 3 },
    { 0, 0 },
    { 0xf39e7592, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec67595, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34927598, 3 },
    { 0, 0 },
    { 0xf896759a, 4 },
    { 0x1c5e759b, 3 },
    { 0, 0 },
    { 0xbbd2759d, 3 },
    { 0x1bb6759d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aa75a1, 3 },
    { 0, 0 },
    { 0x657275a3, 3 },
    { 0, 0 },
    { 0x1675a5, 3 },
    { 0xc41675a6, 3 },
    { 0x7d9675a5, 3 },
    { 0xcba275a8, 3 },
    { 0x494275a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375275ad, 3 },
    { 0xc6ca75ae, 3 },
    { 0x498275af, 3 },
    { 0xe31675af, 3 },
    { 0x4c6275b1, 4 },
    { 0xf62e75b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2a75b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83675b8, 4 },
    { 0x874a75b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea75bc, 3 },
    { 0x95275bc, 4 },
    { 0x905a75be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78a75c1, 3 },
    { 0x57a75c2, 3 },
    { 0xc0a275c3, 3 },
    { 0x1c9e75c1, 3 },
    { 0x9f1a75c3, 3 },
    { 0, 0 },
    { 0x23ca75c7, 3 },
    { 0xb2d275c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a75cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a675d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a75da, 3 },
    { 0x1cd675da, 3 },
    { 0, 0 },
    { 0x754a75dd, 3 },
    { 0xccee75dd, 3 },
    { 0xaace75df, 3 },
    { 0x559a75de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1675e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b275e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x141675ea, 4 },
    { 0xa5fa75eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3275ef, 3 },
    { 0x3f3275ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01675f4, 3 },
    { 0x268e75f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x766a75f8, 3 },
    { 0, 0 },
    { 0x10ea75fa, 3 },
    { 0xace75fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6275fe, 3 },
    { 0x410675fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10e7602, 3 },
    { 0x19aa7602, 3 },
    { 0x209e7603, 4 },
    { 0x7d927605, 4 },
    { 0xe97e7605, 3 },
    { 0x9eca7606, 4 },
    { 0x9e9e7608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763a760d, 3 },
    { 0, 0 },
    { 0xe68a760f, 3 },
    { 0x5a4e760f, 3 },
    { 0xf767610, 3 },
    { 0xcdd6760f, 3 },
    { 0x8cee7610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474a7619, 3 },
    { 0x7cb67619, 3 },
    { 0x4cb67619, 4 },
    { 0x75527619, 4 },
    { 0xd32e761d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2967623, 4 },
    { 0x289a7623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066762f, 5 },
    { 0, 0 },
    { 0x12fe7631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e467634, 5 },
    { 0, 0 },
    { 0xc8627636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6be7639, 3 },
    { 0x951a7639, 3 },
    { 0x9127639, 4 },
    { 0x478a763b, 4 },
    { 0xcfae763d, 5 },
    { 0xc0a2763e, 4 },
    { 0xb45a763e, 4 },
    { 0xfd0a7640, 3 },
    { 0x35ee763a, 3 },
    { 0, 0 },
    { 0x27c67643, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54027646, 3 },
    { 0x60627646, 3 },
    { 0x82ca7646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd2764b, 3 },
    { 0x85ae764b, 3 },
    { 0xdb86764d, 4 },
    { 0xe87e764e, 3 },
    { 0xd556764b, 3 },
    { 0x111e764b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a27654, 3 },
    { 0xc4ee7655, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe006765b, 4 },
    { 0xa136765c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8a7662, 3 },
    { 0xf8ba7663, 4 },
    { 0xe7da7664, 3 },
    { 0, 0 },
    { 0xf4527666, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a766b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39327672, 3 },
    { 0x82ee7673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04e7677, 4 },
    { 0x9cca7678, 3 },
    { 0x4b927678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066767c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e67680, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aa7685, 3 },
    { 0, 0 },
    { 0x792e7687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c32768a, 3 },
    { 0x211a768a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50627690, 4 },
    { 0x61ca7691, 4 },
    { 0x36b27690, 4 },
    { 0x6ef67691, 3 },
    { 0xbb667693, 3 },
    { 0xa5fa7695, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd667698, 3 },
    { 0x9c467698, 3 },
    { 0, 0 },
    { 0x82b6769b, 3 },
    { 0x2eee769c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74276a0, 3 },
    { 0x821676a0, 3 },
    { 0xdaca76a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aa76a6, 3 },
    { 0x1d1e76a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f676ae, 3 },
    { 0xf94a76af, 3 },
    { 0xea5e76ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9276b4, 3 },
    { 0, 0 },
    { 0x82f276b6, 3 },
    { 0x69da76b6, 3 },
    { 0x213676b8, 3 },
    { 0x39ae76b9, 3 },
    { 0, 0 },
    { 0xdb2a76bb, 3 },
    { 0x3a0276bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97a76cd, 3 },
    { 0x6c7a76ce, 3 },
    { 0, 0 },
    { 0xc1de76d0, 4 },
    { 0x450276d1, 4 },
    { 0, 0 },
    { 0xe83676d3, 3 },
    { 0x583e76d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f276d9, 3 },
    { 0x46aa76da, 4 },
    { 0x9a0676db, 3 },
    { 0x8de276d9, 4 },
    { 0xac7a76dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0676e4, 4 },
    { 0x7ec276e4, 3 },
    { 0xa2aa76e5, 3 },
    { 0, 0 },
    { 0x49fa76e8, 4 },
    { 0x726276e8, 4 },
    { 0, 0 },
    { 0x25e76eb, 4 },
    { 0xdbb676eb, 4 },
    { 0, 0 },
    { 0x687e76ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e76f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a76f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2e76fb, 3 },
    { 0xa5fe76fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50727701, 3 },
    { 0, 0 },
    { 0x9dce7703, 3 },
    { 0xc3a7704, 4 },
    { 0xaafa7705, 3 },
    { 0, 0 },
    { 0xd3267707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e2770c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba7711, 3 },
    { 0, 0 },
    { 0x94327713, 3 },
    { 0, 0 },
    { 0x460a7715, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ce771b, 3 },
    { 0xec96771c, 3 },
    { 0xb13a771d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf567721, 4 },
    { 0x71f27721, 3 },
    { 0x412e7723, 4 },
    { 0xc4f67724, 3 },
    { 0xfe7a7724, 3 },
    { 0x3d3e7726, 3 },
    { 0x4f827727, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e772c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c2772f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01e7739, 3 },
    { 0x9982773a, 3 },
    { 0xeafe773b, 3 },
    { 0x7c7e773c, 3 },
    { 0, 0 },
    { 0x6eea773e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda67746, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f267749, 3 },
    { 0x63a67749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x669e774d, 3 },
    { 0, 0 },
    { 0x2742774f, 3 },
    { 0xbd7e7750, 3 },
    { 0x5e36774f, 3 },
    { 0xc5267752, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f27759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac96775d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x993e7760, 3 },
    { 0x6baa7760, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e7769, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a2776c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd267770, 3 },
    { 0, 0 },
    { 0x1b1e7772, 4 },
    { 0, 0 },
    { 0xfdfe7774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7e7778, 3 },
    { 0xfdaa7778, 3 },
    { 0x97ea777a, 3 },
    { 0, 0 },
    { 0xbb72777c, 3 },
    { 0x887e777c, 3 },
    { 0, 0 },
    { 0xcfca777f, 4 },
    { 0xb006777f, 3 },
    { 0xc4d27781, 3 },
    { 0x9b727782, 3 },
    { 0, 0 },
    { 0xb8ba7784, 3 },
    { 0xeba7785, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efa778c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf462778f, 4 },
    { 0xf52e7790, 4 },
    { 0xeb2e778f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fe779b, 4 },
    { 0, 0 },
    { 0x3f56779d, 4 },
    { 0, 0 },
    { 0x9a52779f, 3 },
    { 0x1c7677a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fa77a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32277a7, 4 },
    { 0, 0 },
    { 0xe22e77a9, 4 },
    { 0, 0 },
    { 0x6c7277ab, 3 },
    { 0x603a77ac, 3 },
    { 0, 0 },
    { 0x5cf677ae, 5 },
    { 0x393a77af, 3 },
    { 0xbc3e77b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50277b4, 3 },
    { 0x4fd277b4, 3 },
    { 0x514277b5, 4 },
    { 0xe78a77b4, 3 },
    { 0xf24e77b8, 4 },
    { 0x9eaa77b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51e77bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3e77bf, 4 },
    { 0, 0 },
    { 0xc8ae77c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ba77c5, 3 },
    { 0x9a6e77c6, 3 },
    { 0, 0 },
    { 0x44ae77c8, 4 },
    { 0x107a77c9, 3 },
    { 0x789a77c9, 3 },
    { 0x993e77c9, 3 },
    { 0x344a77cc, 3 },
    { 0, 0 },
    { 0x3fda77ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3e77d3, 3 },
    { 0xa40a77d4, 3 },
    { 0x7b4677d5, 3 },
    { 0x55ba77d4, 4 },
    { 0, 0 },
    { 0x3a7a77d8, 3 },
    { 0x614277d9, 3 },
    { 0, 0 },
    { 0xad5a77db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23677df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820277e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x407a77ea, 3 },
    { 0xfd2a77eb, 3 },
    { 0x16677ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776677f7, 3 },
    { 0x6a7677f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a7803, 4 },
    { 0xb8867804, 3 },
    { 0, 0 },
    { 0x821a7806, 4 },
    { 0x296e7807, 4 },
    { 0xae3e7807, 3 },
    { 0x3d6e7809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2de7811, 3 },
    { 0x9127812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5927817, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884e7822, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ca7826, 3 },
    { 0, 0 },
    { 0x3e5a7828, 3 },
    { 0xdada7829, 3 },
    { 0xa23e782a, 3 },
    { 0x57d27828, 3 },
    { 0xe7ba782c, 3 },
    { 0x8efe782c, 4 },
    { 0xcbd6782e, 4 },
    { 0x176a782f, 3 },
    { 0xa00e782e, 5 },
    { 0x4e5e782e, 3 },
    { 0, 0 },
    { 0xf9267833, 3 },
    { 0, 0 },
    { 0xdeda7835, 3 },
    { 0x6e7836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffde783a, 3 },
    { 0x3b2a783a, 4 },
    { 0xf75e783b, 3 },
    { 0x2c6e783c, 3 },
    { 0xf402783c, 3 },
    { 0x9f52783f, 3 },
    { 0xccf6783f, 3 },
    { 0x10e6783f, 4 },
    { 0x96167840, 4 },
    { 0x1ab2783c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a767848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aa784b, 3 },
    { 0, 0 },
    { 0x6782784d, 3 },
    { 0, 0 },
    { 0xe29e784f, 4 },
    { 0x8ec6784f, 5 },
    { 0xb6127850, 3 },
    { 0x36a7852, 3 },
    { 0xd0567852, 3 },
    { 0xec3e7852, 3 },
    { 0x37627855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca667859, 3 },
    { 0, 0 },
    { 0x67c6785b, 4 },
    { 0x7cfe785c, 3 },
    { 0, 0 },
    { 0x41be785e, 3 },
    { 0x32da785f, 5 },
    { 0, 0 },
    { 0x186a7861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5a786d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c67871, 3 },
    { 0x68627871, 3 },
    { 0xaffe7873, 3 },
    { 0x3bda7874, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x88427877, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312a787c, 3 },
    { 0x6cee787c, 3 },
    { 0x4056787d, 3 },
    { 0xb6b2787c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74a7883, 3 },
    { 0, 0 },
    { 0x391e7885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d67888, 3 },
    { 0x73c67889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ce788c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd6788f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb027892, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe867895, 3 },
    { 0xfada7896, 3 },
    { 0xa8e7896, 3 },
    { 0x3bd67895, 4 },
    { 0, 0 },
    { 0xacfa789a, 4 },
    { 0xe9c2789b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ea789e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba78a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40da78aa, 3 },
    { 0x62ce78aa, 4 },
    { 0xf13e78ac, 4 },
    { 0xda4278aa, 3 },
    { 0x929e78ae, 3 },
    { 0x988a78af, 4 },
    { 0x15ae78b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x147678b3, 3 },
    { 0, 0 },
    { 0xbe2e78b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766a78b9, 3 },
    { 0x4a8678ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266a78be, 3 },
    { 0, 0 },
    { 0x93c678c0, 3 },
    { 0x403a78c1, 3 },
    { 0, 0 },
    { 0x875e78c3, 4 },
    { 0x334e78c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e78c7, 3 },
    { 0x6b0e78c8, 3 },
    { 0x56c678c8, 4 },
    { 0x2e9678ca, 3 },
    { 0xe2d278c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5a78d2, 4 },
    { 0, 0 },
    { 0xd53278d4, 4 },
    { 0x397a78d4, 3 },
    { 0xdee678d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fce78db, 3 },
    { 0, 0 },
    { 0xf5de78dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9278e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe78e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685678ec, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b678ef, 3 },
    { 0x45d678f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x837678f3, 3 },
    { 0, 0 },
    { 0xfbee78f5, 3 },
    { 0, 0 },
    { 0x1cf678f7, 4 },
    { 0x1c2678f8, 3 },
    { 0xa71e78f7, 3 },
    { 0x8fbe78fa, 3 },
    { 0, 0 },
    { 0x378278fc, 3 },
    { 0, 0 },
    { 0xcf3a78fe, 3 },
    { 0xaf4e78ff, 4 },
    { 0xa7e278ff, 3 },
    { 0x408e78fe, 3 },
    { 0, 0 },
    { 0x7c167903, 3 },
    { 0xb5ca7903, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ce790a, 4 },
    { 0, 0 },
    { 0x821e790c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf27910, 4 },
    { 0, 0 },
    { 0xf6527912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30e7916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa6791f, 3 },
    { 0xb88e7920, 5 },
    { 0x6e227921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ea7924, 3 },
    { 0xdc7a7925, 3 },
    { 0x3f327926, 3 },
    { 0xb3fe7927, 3 },
    { 0x77127926, 3 },
    { 0x8c2e7929, 3 },
    { 0xd2b67929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a7930, 3 },
    { 0x78d67931, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa7939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95a793c, 4 },
    { 0x4112793c, 3 },
    { 0x9b12793d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fa7946, 3 },
    { 0, 0 },
    { 0x6bfe7948, 3 },
    { 0, 0 },
    { 0xf752794a, 3 },
    { 0, 0 },
    { 0xfb62794c, 3 },
    { 0xc46794d, 3 },
    { 0xd342794e, 3 },
    { 0x542794f, 3 },
    { 0xbc167950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x836e7953, 4 },
    { 0x155a7954, 3 },
    { 0x32227953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1a7959, 4 },
    { 0x3456795a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e82795f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e7965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e7968, 3 },
    { 0x9067969, 4 },
    { 0x405e7969, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8da796f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e7973, 4 },
    { 0, 0 },
    { 0x212e7975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a7978, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a797e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838e7982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e27985, 3 },
    { 0x5ae7985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec6798a, 3 },
    { 0, 0 },
    { 0x57f2798c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cde7991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1127995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef2799a, 4 },
    { 0x1f16799b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aa79aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947e79af, 3 },
    { 0, 0 },
    { 0xbb0a79b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa679b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce79b7, 4 },
    { 0x7a6279b8, 3 },
    { 0x7cba79b9, 4 },
    { 0x29f279ba, 4 },
    { 0x755a79bb, 3 },
    { 0x3e4279b9, 3 },
    { 0xeb8279b8, 3 },
    { 0xdf0279be, 4 },
    { 0x77a279b8, 3 },
    { 0x6c279b9, 3 },
    { 0x3f3e79b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7679c6, 3 },
    { 0x8d1a79c7, 3 },
    { 0x6bc279c7, 3 },
    { 0x137279c9, 3 },
    { 0x461279ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x298279cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99e79d0, 3 },
    { 0x2b7a79d1, 3 },
    { 0, 0 },
    { 0xf82279d3, 3 },
    { 0x7b7679d4, 4 },
    { 0x1a79d5, 4 },
    { 0xa39e79d6, 3 },
    { 0x3df679d5, 3 },
    { 0x88a279d8, 3 },
    { 0xf76679d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec679de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1679e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a79e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372a79ea, 3 },
    { 0, 0 },
    { 0xff9279ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae279f0, 3 },
    { 0xe43e79f1, 3 },
    { 0x8e8e79f2, 3 },
    { 0, 0 },
    { 0xb45e79f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x229279f7, 3 },
    { 0, 0 },
    { 0x8b3679f9, 5 },
    { 0x65ba79fa, 3 },
    { 0xc33e79fa, 3 },
    { 0xa40279fc, 3 },
    { 0xae2a79fd, 3 },
    { 0x6a2279fd, 3 },
    { 0x9f8679f9, 5 },
    { 0x98227a00, 4 },
    { 0x31f67a01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59a7a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30767a0a, 4 },
    { 0xe2aa7a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c27a16, 3 },
    { 0x558a7a16, 3 },
    { 0x99ae7a18, 3 },
    { 0x82f67a19, 3 },
    { 0, 0 },
    { 0x777e7a1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e7a25, 3 },
    { 0x7227a26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x510e7a29, 4 },
    { 0, 0 },
    { 0x31e7a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x978a7a2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38327a34, 4 },
    { 0x20de7a34, 3 },
    { 0x69ba7a34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a67a39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5a7a3e, 3 },
    { 0xc15a7a3e, 3 },
    { 0, 0 },
    { 0x6cf27a41, 4 },
    { 0x795e7a41, 3 },
    { 0xc40a7a43, 4 },
    { 0x46a67a43, 3 },
    { 0xc5a7a41, 4 },
    { 0xe2167a44, 3 },
    { 0x4ff67a47, 4 },
    { 0, 0 },
    { 0xd44e7a49, 4 },
    { 0x49a67a49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e7a4f, 3 },
    { 0x7b227a50, 3 },
    { 0xf2ea7a51, 3 },
    { 0x85fe7a52, 3 },
    { 0xb8ae7a4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e7a56, 3 },
    { 0x311a7a56, 4 },
    { 0x1d0a7a58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdca7a5c, 3 },
    { 0, 0 },
    { 0xa27a5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e7a63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b267a6a, 3 },
    { 0x373e7a6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8467a6e, 3 },
    { 0xd8027a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e627a72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e27a76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16967a7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x687e7a82, 3 },
    { 0x425e7a83, 4 },
    { 0xfdc27a83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30567a88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f27a8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17927a8e, 3 },
    { 0x40ce7a8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc27a93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58a7a96, 3 },
    { 0, 0 },
    { 0xd0de7a98, 3 },
    { 0x18da7a99, 4 },
    { 0xb6fe7a99, 4 },
    { 0xeaa67a98, 3 },
    { 0xe6667a9c, 3 },
    { 0x36227a9d, 3 },
    { 0x3d67a9d, 3 },
    { 0, 0 },
    { 0x23667aa0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca67aa8, 3 },
    { 0x48227aa9, 3 },
    { 0, 0 },
    { 0x49d27aab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2a7ab0, 4 },
    { 0x46467ab0, 4 },
    { 0, 0 },
    { 0xca067ab3, 4 },
    { 0x8df67ab3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67abe, 3 },
    { 0x198a7abf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18e7ac5, 3 },
    { 0xae0a7ac5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x226e7ac9, 3 },
    { 0x95927aca, 5 },
    { 0xff27ac9, 3 },
    { 0xb9fe7acc, 4 },
    { 0x8a1e7acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c467ad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1827ade, 3 },
    { 0x739e7adf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82e7ae4, 4 },
    { 0x46627ae5, 3 },
    { 0xd2ca7ae6, 3 },
    { 0xf9827ae6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7567aea, 4 },
    { 0xf25e7aeb, 3 },
    { 0, 0 },
    { 0xce0a7aed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3a7af0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978e7af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d67afe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a7b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321a7b05, 4 },
    { 0, 0 },
    { 0x812a7b07, 3 },
    { 0xfea67b08, 4 },
    { 0, 0 },
    { 0x409a7b0a, 3 },
    { 0xf47a7b0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f27b0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e7b12, 5 },
    { 0x6c667b12, 3 },
    { 0, 0 },
    { 0x9f067b15, 3 },
    { 0x9b8a7b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee7b1c, 4 },
    { 0x8eca7b1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc67b27, 3 },
    { 0x60367b28, 3 },
    { 0, 0 },
    { 0xda227b2a, 4 },
    { 0x349e7b2a, 4 },
    { 0, 0 },
    { 0xb6ba7b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7a7b36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be7b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a7b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e67b42, 3 },
    { 0, 0 },
    { 0x9f127b44, 3 },
    { 0x7cf27b45, 3 },
    { 0x709e7b44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeba7b4c, 3 },
    { 0x4e567b4c, 3 },
    { 0xfa4a7b4c, 3 },
    { 0xc32a7b4f, 4 },
    { 0xfa67b50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb27b54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48e7b5b, 3 },
    { 0, 0 },
    { 0x72ca7b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29227b60, 3 },
    { 0, 0 },
    { 0xbe5e7b62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c27b66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6767b71, 3 },
    { 0x8faa7b72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd67b75, 5 },
    { 0x49127b76, 3 },
    { 0xa7867b77, 4 },
    { 0x973a7b77, 3 },
    { 0xaa0a7b76, 3 },
    { 0x8ace7b77, 5 },
    { 0xfa127b7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3667b7e, 4 },
    { 0x359a7b7f, 3 },
    { 0x1ee67b7e, 3 },
    { 0xd7a7b80, 3 },
    { 0x459e7b82, 3 },
    { 0x74127b7e, 3 },
    { 0, 0 },
    { 0x59427b85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e67b8a, 4 },
    { 0x299a7b8b, 3 },
    { 0, 0 },
    { 0x9d7a7b8d, 3 },
    { 0xd35a7b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0e7b91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb67b94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a7b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b67ba1, 4 },
    { 0x82367ba1, 3 },
    { 0x88627ba3, 3 },
    { 0x51e7ba3, 3 },
    { 0, 0 },
    { 0xb8127ba6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de7ba9, 3 },
    { 0x97ae7ba9, 3 },
    { 0x2fbe7bab, 4 },
    { 0x52de7bac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ee7bb0, 4 },
    { 0xd5e7bb1, 3 },
    { 0xdde7bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60267bbc, 3 },
    { 0, 0 },
    { 0x10b27bbe, 4 },
    { 0xa78a7bbf, 3 },
    { 0, 0 },
    { 0xf3067bc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a667bc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a27bcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af67bd5, 3 },
    { 0xea7a7bd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b927bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae7be3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96927be7, 3 },
    { 0x9f5e7be8, 3 },
    { 0xa19e7be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e7bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c167bf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc67bf9, 3 },
    { 0, 0 },
    { 0x7aee7bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60e7c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea7c0a, 3 },
    { 0xe1f27c0b, 3 },
    { 0x90127c0c, 3 },
    { 0, 0 },
    { 0x316a7c0e, 3 },
    { 0, 0 },
    { 0xcfd67c10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360e7c14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f67c18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84e7c1c, 3 },
    { 0x415e7c1c, 3 },
    { 0x6f367c1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bce7c22, 3 },
    { 0, 0 },
    { 0x5bb27c24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fe7c27, 3 },
    { 0x2c727c28, 3 },
    { 0x26b27c29, 3 },
    { 0xd2d27c27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565e7c2e, 3 },
    { 0x2d7e7c2f, 3 },
    { 0xb4aa7c2f, 3 },
    { 0, 0 },
    { 0x269e7c32, 5 },
    { 0, 0 },
    { 0xce4a7c34, 4 },
    { 0xf5c67c35, 3 },
    { 0x4d5a7c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41e7c39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e7c40, 3 },
    { 0x97ce7c41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25867c44, 3 },
    { 0x268a7c45, 4 },
    { 0, 0 },
    { 0x512a7c47, 4 },
    { 0x5dba7c48, 3 },
    { 0xf39a7c49, 3 },
    { 0, 0 },
    { 0x37de7c4b, 3 },
    { 0x688a7c4c, 3 },
    { 0x69067c4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x88de7c50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e7c57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea27c5a, 3 },
    { 0xb6d27c5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b67c5e, 4 },
    { 0xa40a7c5e, 4 },
    { 0x351a7c60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28867c63, 3 },
    { 0xa8fa7c64, 3 },
    { 0x32ba7c64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24a7c68, 3 },
    { 0x8267c69, 4 },
    { 0, 0 },
    { 0xd4a27c6b, 4 },
    { 0x72727c6c, 3 },
    { 0xeffe7c6c, 3 },
    { 0xfe627c6b, 3 },
    { 0, 0 },
    { 0x4c2a7c70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3e7c78, 3 },
    { 0, 0 },
    { 0xc7327c7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25127c7f, 4 },
    { 0xdd5a7c7f, 3 },
    { 0xd3467c81, 3 },
    { 0, 0 },
    { 0x88167c83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de7c8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fda7c8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b27c94, 3 },
    { 0, 0 },
    { 0x83de7c96, 3 },
    { 0xa6ca7c97, 4 },
    { 0xf9be7c98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6127c9b, 3 },
    { 0xa1167c9c, 3 },
    { 0xb6167c9d, 3 },
    { 0x38067c9d, 3 },
    { 0, 0 },
    { 0xa99a7ca0, 3 },
    { 0xb1a67ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ae7ca4, 3 },
    { 0x89167ca5, 3 },
    { 0, 0 },
    { 0x4a027ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e7caa, 4 },
    { 0xca5e7cab, 3 },
    { 0x719a7caa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92767cb0, 3 },
    { 0xbe6a7cb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea7cb5, 4 },
    { 0xfc067cb5, 4 },
    { 0xd6fe7cb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfe7cba, 3 },
    { 0xd5f27cba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64de7cbe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c27cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31827cc7, 3 },
    { 0x87467cc7, 3 },
    { 0x63f67cc9, 3 },
    { 0x8bb27cca, 4 },
    { 0, 0 },
    { 0xcc4e7ccc, 3 },
    { 0x53ba7ccd, 5 },
    { 0xdd067cce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d67cd1, 3 },
    { 0xa1e67cd2, 3 },
    { 0x1c9e7cd3, 3 },
    { 0xe2e27cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaa7cd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe7cda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe7cdf, 3 },
    { 0xa82e7cdf, 4 },
    { 0x8a667ce0, 3 },
    { 0x55d67ce2, 3 },
    { 0x9d6a7cdf, 3 },
    { 0x4f927ce4, 4 },
    { 0xf1867ce3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6327ce8, 3 },
    { 0x98ee7ce9, 4 },
    { 0xd9ca7cea, 3 },
    { 0xdade7ceb, 4 },
    { 0xe8fa7cec, 4 },
    { 0xcf467ceb, 3 },
    { 0x126a7ce9, 3 },
    { 0, 0 },
    { 0x2f667cf0, 3 },
    { 0xca6a7cf1, 3 },
    { 0xe9ba7cf2, 3 },
    { 0x217e7cf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0a7cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9da7cfd, 3 },
    { 0x87d27cfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae27d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f27d07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fe7d0a, 4 },
    { 0x6dda7d0b, 3 },
    { 0x8927d0a, 3 },
    { 0xc66e7d0b, 4 },
    { 0xee167d0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efa7d15, 3 },
    { 0xed4a7d16, 4 },
    { 0, 0 },
    { 0x91ca7d18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d27d1f, 3 },
    { 0x1f027d20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16567d25, 4 },
    { 0, 0 },
    { 0x276e7d27, 3 },
    { 0, 0 },
    { 0x3dc27d29, 4 },
    { 0x93167d29, 4 },
    { 0x9ed67d2b, 3 },
    { 0x53327d2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85267d36, 3 },
    { 0x609e7d36, 4 },
    { 0xa5e27d38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c27d3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c67d3f, 3 },
    { 0xb4aa7d3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d867d43, 3 },
    { 0x430e7d43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f67d49, 3 },
    { 0, 0 },
    { 0x4eb27d4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97427d4e, 3 },
    { 0xdd6e7d4f, 4 },
    { 0, 0 },
    { 0x4a6e7d51, 3 },
    { 0x190a7d52, 4 },
    { 0xcb927d53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee7d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3e7d64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb967d69, 3 },
    { 0xa71a7d69, 3 },
    { 0x56aa7d69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e567d6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a27d72, 3 },
    { 0, 0 },
    { 0x8fb67d74, 3 },
    { 0x20ea7d75, 3 },
    { 0xf00e7d76, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fca7d7d, 3 },
    { 0, 0 },
    { 0x2f1e7d7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f027d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x953e7d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x184a7d8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda7d8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be7d95, 3 },
    { 0xdec27d95, 3 },
    { 0x90de7d97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba7d9b, 4 },
    { 0, 0 },
    { 0xcbf27d9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae7da0, 3 },
    { 0x1f1a7da1, 3 },
    { 0x91767da2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de7da5, 3 },
    { 0xed467da5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba7da9, 3 },
    { 0, 0 },
    { 0xd967dab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9527db1, 3 },
    { 0xb87a7db2, 4 },
    { 0x9a67db3, 3 },
    { 0xeb4a7db4, 4 },
    { 0xfba67db4, 3 },
    { 0x7fe67db6, 5 },
    { 0, 0 },
    { 0xde4e7db8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259a7dbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e7dc2, 3 },
    { 0x33e27dc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fe7dc9, 3 },
    { 0x1d7a7dca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e367dcf, 4 },
    { 0xef2e7dd0, 3 },
    { 0x7a827dd1, 3 },
    { 0, 0 },
    { 0x294e7dd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ba7dd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc27ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1da7ddf, 4 },
    { 0, 0 },
    { 0x46667de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432e7de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a267dee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e7df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd267df4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1e7df7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a367e03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ae7e07, 3 },
    { 0xff67e08, 3 },
    { 0x72a7e07, 3 },
    { 0x5c0a7e0a, 4 },
    { 0x1f527e0b, 3 },
    { 0x92a67e0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ba7e12, 3 },
    { 0x2b327e12, 3 },
    { 0x8ee7e13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbde7e19, 4 },
    { 0x2f467e1a, 4 },
    { 0x4467e19, 3 },
    { 0xfa267e1c, 4 },
    { 0xd68e7e1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fe7e22, 3 },
    { 0, 0 },
    { 0xe50a7e24, 4 },
    { 0, 0 },
    { 0xed67e26, 3 },
    { 0x6dee7e26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e7e2c, 4 },
    { 0, 0 },
    { 0xd5b67e2e, 4 },
    { 0, 0 },
    { 0x9ef67e30, 3 },
    { 0xca1a7e31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a7e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b567e42, 3 },
    { 0xbca67e42, 3 },
    { 0xafee7e44, 3 },
    { 0x99527e45, 4 },
    { 0x86b27e45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ae7e4a, 4 },
    { 0, 0 },
    { 0x297a7e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ae7e4f, 3 },
    { 0x53fa7e50, 3 },
    { 0, 0 },
    { 0x60be7e52, 3 },
    { 0x624e7e53, 3 },
    { 0x17267e54, 3 },
    { 0x3ece7e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac67e58, 3 },
    { 0x58ca7e59, 3 },
    { 0x97467e59, 3 },
    { 0xbcde7e58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd67e5f, 3 },
    { 0, 0 },
    { 0xe8d67e61, 3 },
    { 0x5d5e7e62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9527e67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c27e6c, 3 },
    { 0x499a7e6d, 3 },
    { 0xfafe7e6c, 4 },
    { 0xad9a7e6e, 3 },
    { 0, 0 },
    { 0x46367e71, 4 },
    { 0xbb2a7e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d67e76, 3 },
    { 0x1fe7e77, 4 },
    { 0, 0 },
    { 0x1e2a7e79, 3 },
    { 0xc8da7e7a, 3 },
    { 0x4aaa7e79, 4 },
    { 0xdfa27e7b, 3 },
    { 0xb2267e7d, 3 },
    { 0xadfe7e7e, 3 },
    { 0x8f9e7e7c, 3 },
    { 0x56fa7e80, 3 },
    { 0x169a7e79, 4 },
    { 0xf7a7e7c, 3 },
    { 0x99267e80, 4 },
    { 0xe85e7e7c, 3 },
    { 0xc9367e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3a7e88, 3 },
    { 0, 0 },
    { 0x3a167e8a, 3 },
    { 0xd9a7e8a, 3 },
    { 0, 0 },
    { 0x2e527e8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30da7e94, 3 },
    { 0x4bbe7e95, 3 },
    { 0, 0 },
    { 0x2c5e7e97, 3 },
    { 0, 0 },
    { 0xc71e7e99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84a7ea4, 3 },
    { 0, 0 },
    { 0x67527ea6, 3 },
    { 0, 0 },
    { 0x6ce7ea8, 3 },
    { 0x61227ea9, 3 },
    { 0x88ce7ea9, 3 },
    { 0, 0 },
    { 0x8e2a7eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4467eb5, 4 },
    { 0x8a67eb5, 3 },
    { 0xd83a7eb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33127eba, 4 },
    { 0x38067ebb, 4 },
    { 0x2eb67ebb, 4 },
    { 0xf04a7ebb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x776e7ec0, 4 },
    { 0x95c67ec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109a7ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e7ed1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ba7ed6, 4 },
    { 0xa7ea7ed7, 3 },
    { 0x6c167ed7, 3 },
    { 0, 0 },
    { 0xd1967eda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6a7edd, 4 },
    { 0, 0 },
    { 0x39727edf, 3 },
    { 0xae6a7ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667a7ee5, 4 },
    { 0x9bb67ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84a7eeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97e7eee, 3 },
    { 0, 0 },
    { 0x60ee7ef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c27ef9, 5 },
    { 0x2f067efa, 5 },
    { 0x1f967efa, 3 },
    { 0x85667efc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142a7f00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbe7f04, 3 },
    { 0xd0ae7f04, 3 },
    { 0x12ca7f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9e7f0d, 3 },
    { 0, 0 },
    { 0x74627f0f, 4 },
    { 0x79ba7f10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7e7f13, 3 },
    { 0, 0 },
    { 0x34167f15, 3 },
    { 0x84d27f16, 3 },
    { 0, 0 },
    { 0x1bea7f18, 3 },
    { 0x11267f19, 5 },
    { 0x179e7f18, 3 },
    { 0x3d9e7f18, 4 },
    { 0, 0 },
    { 0xcc5a7f1d, 4 },
    { 0x6a0a7f1e, 3 },
    { 0, 0 },
    { 0x4fba7f20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a27f25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec867f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e627f31, 4 },
    { 0, 0 },
    { 0xf9467f33, 3 },
    { 0x8227f33, 3 },
    { 0x54fe7f34, 3 },
    { 0xe627f35, 3 },
    { 0x352a7f36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0a7f3a, 3 },
    { 0x63a27f3b, 4 },
    { 0x505e7f3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a7f4b, 4 },
    { 0xd2da7f4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3167f50, 3 },
    { 0xa76a7f51, 3 },
    { 0x444e7f51, 3 },
    { 0, 0 },
    { 0x16c27f54, 3 },
    { 0x55aa7f54, 3 },
    { 0xe0027f56, 3 },
    { 0, 0 },
    { 0x5fde7f58, 4 },
    { 0x4b327f59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e7f64, 3 },
    { 0, 0 },
    { 0x24ee7f66, 3 },
    { 0x2e927f66, 3 },
    { 0xd46a7f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17a7f6c, 3 },
    { 0, 0 },
    { 0xc5aa7f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c27f73, 4 },
    { 0, 0 },
    { 0xcad27f75, 4 },
    { 0, 0 },
    { 0xe5767f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67127f7a, 4 },
    { 0, 0 },
    { 0x3827f7c, 3 },
    { 0x771a7f7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0be7f82, 3 },
    { 0x707a7f83, 5 },
    { 0x92667f83, 3 },
    { 0x5cf27f85, 4 },
    { 0x45267f83, 3 },
    { 0x8d927f87, 3 },
    { 0xda4e7f88, 4 },
    { 0xa78a7f89, 4 },
    { 0, 0 },
    { 0xe8727f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46227f90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667f9d, 4 },
    { 0xc73e7f9e, 4 },
    { 0x219e7f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25167fa3, 3 },
    { 0x95227fa4, 4 },
    { 0x2bb67fa4, 3 },
    { 0x6f4a7fa6, 3 },
    { 0x4f427fa6, 3 },
    { 0x5ef67fa6, 3 },
    { 0xb7ba7fa9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf27faf, 3 },
    { 0, 0 },
    { 0x20ea7fb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e67fb5, 3 },
    { 0x3427fb6, 3 },
    { 0x8bf27fb7, 4 },
    { 0x9c167fb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ea7fc2, 3 },
    { 0x7f527fc2, 4 },
    { 0x90b27fc4, 4 },
    { 0x47b27fc4, 3 },
    { 0, 0 },
    { 0xf9e27fc7, 3 },
    { 0xaae27fc7, 3 },
    { 0, 0 },
    { 0x1eee7fca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ca7fd2, 4 },
    { 0xafe67fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8a7fd8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf27fdb, 3 },
    { 0x73da7fdc, 4 },
    { 0x77d27fdc, 3 },
    { 0xccf27fde, 3 },
    { 0, 0 },
    { 0x79ae7fe0, 3 },
    { 0x67e7fe1, 3 },
    { 0x480e7fe2, 4 },
    { 0xf3427fe3, 3 },
    { 0, 0 },
    { 0xb7f27fe5, 3 },
    { 0x8e1a7fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf67fea, 3 },
    { 0x1e3a7feb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89867ff0, 3 },
    { 0x950e7ff0, 3 },
    { 0, 0 },
    { 0xcd667ff3, 4 },
    { 0, 0 },
    { 0x145e7ff5, 3 },
    { 0x6e4e7ff5, 4 },
    { 0xcbae7ff7, 3 },
    { 0xb0067ff8, 4 },
    { 0xb8d67ff9, 3 },
    { 0x96a7ff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda767ffd, 4 },
    { 0x73667ffd, 3 },
    { 0xa68a7fff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381e8003, 3 },
    { 0, 0 },
    { 0x1cea8005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ae800a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2800d, 4 },
    { 0x189a800d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354a8012, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95928015, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf828020, 3 },
    { 0xecda8021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a68026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4a8036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b6803b, 4 },
    { 0x9f46803b, 3 },
    { 0x7eb2803d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce8040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b428043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66428047, 3 },
    { 0x3e4e8048, 3 },
    { 0x8ab28048, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb26804d, 3 },
    { 0, 0 },
    { 0xcdc2804f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c28054, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512805d, 3 },
    { 0x3c5e805e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bae8061, 4 },
    { 0xf00a8062, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4a8066, 3 },
    { 0x79be8066, 3 },
    { 0x411a8068, 3 },
    { 0xc5da8069, 4 },
    { 0xd91e8066, 3 },
    { 0, 0 },
    { 0x43ba806c, 4 },
    { 0x1dfa806d, 3 },
    { 0x5ede806d, 3 },
    { 0xac1a806f, 3 },
    { 0x2bde806f, 3 },
    { 0x8ede806e, 4 },
    { 0xf15e806c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa8075, 3 },
    { 0x124a8076, 4 },
    { 0x1e68077, 3 },
    { 0x12228076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe807b, 3 },
    { 0x6e22807c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22807f, 3 },
    { 0, 0 },
    { 0x8b168081, 3 },
    { 0, 0 },
    { 0x319e8083, 4 },
    { 0xf0868084, 4 },
    { 0x17668085, 3 },
    { 0x255a8083, 4 },
    { 0xdcde8084, 3 },
    { 0x247a8088, 4 },
    { 0x2b068089, 3 },
    { 0xf0ae808a, 3 },
    { 0x7e5a808a, 3 },
    { 0x59e6808a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ee809f, 3 },
    { 0x137280a0, 3 },
    { 0x5eb280a1, 4 },
    { 0x1d1a80a0, 3 },
    { 0x3f9a80a3, 3 },
    { 0x64e80a3, 3 },
    { 0x538280a0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9a80ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0680b8, 3 },
    { 0x7dce80b9, 4 },
    { 0xda7e80ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x556280bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0280c8, 3 },
    { 0x59ba80c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fe80cc, 3 },
    { 0, 0 },
    { 0xb6a280ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da80d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9280d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183a80da, 3 },
    { 0x68fe80da, 3 },
    { 0x81b280dc, 4 },
    { 0x146280da, 3 },
    { 0x205e80db, 3 },
    { 0xe30a80df, 3 },
    { 0xade680e0, 3 },
    { 0x6cc680e1, 3 },
    { 0x22e80e2, 4 },
    { 0x224a80de, 3 },
    { 0x782680e0, 4 },
    { 0x2e2e80e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3680e8, 4 },
    { 0xb6a680e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9280ee, 4 },
    { 0, 0 },
    { 0x363a80f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972a80fb, 3 },
    { 0, 0 },
    { 0xd09680fd, 3 },
    { 0, 0 },
    { 0x724a80ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e8103, 3 },
    { 0x56228103, 3 },
    { 0xb8128103, 5 },
    { 0x835e8103, 4 },
    { 0xe5e8105, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea2810a, 4 },
    { 0x7222810a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8111, 3 },
    { 0x8b2a8112, 4 },
    { 0xdaf28112, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a2811f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4068122, 3 },
    { 0x1f8a8123, 3 },
    { 0, 0 },
    { 0x4c728125, 3 },
    { 0, 0 },
    { 0xccaa8127, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4a812d, 3 },
    { 0, 0 },
    { 0x3362812f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e8132, 4 },
    { 0x44168132, 3 },
    { 0x313e8133, 3 },
    { 0xe0e8135, 3 },
    { 0x64168132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c28139, 3 },
    { 0xa936813a, 4 },
    { 0, 0 },
    { 0x4206813c, 3 },
    { 0xd212813c, 4 },
    { 0xf02a813e, 3 },
    { 0x2afe813e, 3 },
    { 0x19aa813d, 3 },
    { 0xb766813f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c68147, 3 },
    { 0xf328147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56a8150, 3 },
    { 0xe7368150, 4 },
    { 0, 0 },
    { 0x4f3e8153, 4 },
    { 0xf9b68154, 3 },
    { 0xe7228153, 3 },
    { 0xa0828156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6828163, 3 },
    { 0x97528163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38a8168, 3 },
    { 0xabd68169, 4 },
    { 0, 0 },
    { 0xd73e816b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242e816f, 3 },
    { 0, 0 },
    { 0xba1a8171, 3 },
    { 0, 0 },
    { 0xb3f28173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e8179, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98be8180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45668186, 3 },
    { 0x34ee8187, 3 },
    { 0xa1528188, 3 },
    { 0, 0 },
    { 0x427a818a, 4 },
    { 0x467a818b, 3 },
    { 0, 0 },
    { 0xa2ce818d, 3 },
    { 0xce3a818e, 3 },
    { 0xd056818f, 3 },
    { 0xea6a8190, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683e8194, 4 },
    { 0x3d728194, 3 },
    { 0xf21a8195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ca8199, 3 },
    { 0x53ae8199, 3 },
    { 0, 0 },
    { 0x235a819c, 4 },
    { 0, 0 },
    { 0x3aa819e, 3 },
    { 0x177e819f, 3 },
    { 0xa30e819e, 3 },
    { 0x6f66819f, 3 },
    { 0, 0 },
    { 0xebfe81a3, 3 },
    { 0x9dd281a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x618e81a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67a81b0, 4 },
    { 0xe62681b1, 3 },
    { 0x498681b1, 4 },
    { 0xad4a81b3, 4 },
    { 0, 0 },
    { 0x810681b5, 3 },
    { 0x367681b6, 3 },
    { 0, 0 },
    { 0x7dea81b8, 3 },
    { 0x701e81b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7681bc, 3 },
    { 0, 0 },
    { 0x25ca81be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa81c1, 3 },
    { 0xb5e681c2, 3 },
    { 0xff2a81c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459e81ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915681cf, 4 },
    { 0, 0 },
    { 0x115681d1, 3 },
    { 0, 0 },
    { 0x3db281d3, 3 },
    { 0x182a81d3, 3 },
    { 0x2ffe81d5, 3 },
    { 0xdf0681d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3681d9, 3 },
    { 0x8e0a81da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fe81dd, 4 },
    { 0x29a81de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e81e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da81e6, 3 },
    { 0x897e81e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cae81ea, 3 },
    { 0, 0 },
    { 0xd08681ec, 3 },
    { 0, 0 },
    { 0xc89a81ee, 4 },
    { 0x25ae81ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecba81f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec681fc, 3 },
    { 0xe8e81fd, 3 },
    { 0xbd5281fe, 4 },
    { 0x4da681fe, 3 },
    { 0xf39e81fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf428211, 3 },
    { 0xdf828212, 3 },
    { 0, 0 },
    { 0x615a8214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5e8219, 3 },
    { 0x8e368219, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a8220, 4 },
    { 0x543e8220, 4 },
    { 0xadc28222, 4 },
    { 0xd28a8222, 3 },
    { 0xb6f28224, 3 },
    { 0xede68224, 3 },
    { 0x30b68222, 4 },
    { 0x6ab68227, 3 },
    { 0, 0 },
    { 0x40b28229, 3 },
    { 0, 0 },
    { 0xc3f6822b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a8230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06a8235, 3 },
    { 0, 0 },
    { 0xe8de8237, 3 },
    { 0xd3928238, 4 },
    { 0, 0 },
    { 0x2eba823a, 3 },
    { 0, 0 },
    { 0x9d4e823c, 3 },
    { 0, 0 },
    { 0x6ee823e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cba8244, 3 },
    { 0x744a8244, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5e8250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x397a8253, 3 },
    { 0, 0 },
    { 0xf2228255, 3 },
    { 0xae3e8255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8992825d, 3 },
    { 0xc2b6825d, 3 },
    { 0xfb02825d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29868262, 3 },
    { 0xfca68262, 3 },
    { 0x89e68264, 3 },
    { 0x4328264, 3 },
    { 0x33028266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd136826d, 3 },
    { 0, 0 },
    { 0xfc3e826f, 4 },
    { 0xf33a8270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7be8277, 3 },
    { 0, 0 },
    { 0x7ac28279, 3 },
    { 0x73f68279, 3 },
    { 0xfb0a827b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152827e, 4 },
    { 0xef46827f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e8283, 3 },
    { 0x5bf68284, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9756828d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72028292, 3 },
    { 0x371a8293, 4 },
    { 0xb89e8292, 3 },
    { 0xc0b28292, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e8299, 3 },
    { 0x112e8299, 3 },
    { 0, 0 },
    { 0x40ae829c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f86829f, 3 },
    { 0xd9ae82a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dca82a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fa82a7, 3 },
    { 0xdb4282a7, 3 },
    { 0x327682a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x555282ac, 3 },
    { 0, 0 },
    { 0xe0e682ae, 3 },
    { 0x69f682af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fa82b4, 4 },
    { 0x9e6e82b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7a82ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155a82c4, 4 },
    { 0, 0 },
    { 0x7e7282c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e682c9, 3 },
    { 0x5ffa82ca, 4 },
    { 0xf65282cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422a82cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x625282d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7e82d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05282dc, 3 },
    { 0x900e82dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48e82e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x743282e5, 3 },
    { 0xb99e82e5, 3 },
    { 0x5e4682e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x79682ea, 3 },
    { 0xbe8e82eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9282ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153a82f5, 3 },
    { 0, 0 },
    { 0x358682f7, 3 },
    { 0xdee82f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539282fd, 4 },
    { 0xc2a282fd, 3 },
    { 0x510282ff, 3 },
    { 0, 0 },
    { 0xfcea8301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63568305, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6a8308, 3 },
    { 0, 0 },
    { 0x17d2830a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ae830d, 3 },
    { 0xe1aa830d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508e8312, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af68317, 3 },
    { 0x6e5e8317, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce628326, 3 },
    { 0, 0 },
    { 0x18de8328, 3 },
    { 0xebe68329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13e832d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ee8332, 4 },
    { 0xbeb28333, 4 },
    { 0x81868332, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4826833c, 5 },
    { 0x746e833d, 4 },
    { 0, 0 },
    { 0xe66a833f, 3 },
    { 0, 0 },
    { 0x42e8341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0a8346, 3 },
    { 0, 0 },
    { 0xe0ae8348, 3 },
    { 0x7a228349, 4 },
    { 0x1ea8348, 4 },
    { 0xea834b, 3 },
    { 0x9fae834c, 3 },
    { 0xf156834d, 3 },
    { 0x9ae2834c, 3 },
    { 0xec7e834e, 4 },
    { 0xcd4e8350, 3 },
    { 0x1c3a834d, 3 },
    { 0xa27e834d, 3 },
    { 0xf3368350, 3 },
    { 0x60ce8351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350e8359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0a835d, 3 },
    { 0, 0 },
    { 0x8a32835f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802a8363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e468368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaa8370, 4 },
    { 0x1bf28371, 3 },
    { 0xeff28370, 5 },
    { 0, 0 },
    { 0xffe8374, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb28377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd732837a, 3 },
    { 0x1fea837b, 3 },
    { 0x9716837a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7228380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f28383, 4 },
    { 0x628384, 3 },
    { 0x6e9a8385, 4 },
    { 0x3aa68386, 3 },
    { 0x2d3e8387, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066838b, 5 },
    { 0, 0 },
    { 0xf07a838d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7668392, 3 },
    { 0x25b28392, 3 },
    { 0x73928394, 3 },
    { 0x59768393, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3a8399, 3 },
    { 0xae12839a, 4 },
    { 0, 0 },
    { 0x1a4e839c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08283a2, 5 },
    { 0x40683a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae83a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4683ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58683b7, 4 },
    { 0x54683b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77e83bd, 3 },
    { 0x275e83bd, 4 },
    { 0xfefe83be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e683c4, 3 },
    { 0, 0 },
    { 0xfe8a83c6, 3 },
    { 0x917283c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad283cd, 3 },
    { 0x7c1e83ce, 3 },
    { 0x9e0e83ce, 3 },
    { 0x45d283cf, 3 },
    { 0x44f683d1, 3 },
    { 0x526a83ce, 3 },
    { 0x695683d3, 4 },
    { 0xe4ee83d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711e83d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da283db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7683de, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fe83e1, 3 },
    { 0xb0ce83e2, 4 },
    { 0x27f283e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7283e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2683ea, 3 },
    { 0x806e83eb, 4 },
    { 0, 0 },
    { 0xe8a83ed, 3 },
    { 0x2a2283ee, 3 },
    { 0x48b283ed, 3 },
    { 0x3ece83f0, 3 },
    { 0xa17a83f0, 3 },
    { 0x486283f2, 3 },
    { 0x32fa83f3, 4 },
    { 0xb75a83f0, 3 },
    { 0x2fe83f3, 3 },
    { 0xc8a283f6, 5 },
    { 0xed3a83f7, 3 },
    { 0x6aca83f8, 3 },
    { 0xd48283f8, 3 },
    { 0x911a83ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ae83fd, 3 },
    { 0x1eba83fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11e8401, 3 },
    { 0x9aae8402, 5 },
    { 0x476a8403, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44e8407, 4 },
    { 0, 0 },
    { 0x3be28409, 3 },
    { 0x205a840a, 4 },
    { 0, 0 },
    { 0x7bea840c, 3 },
    { 0xa192840c, 3 },
    { 0, 0 },
    { 0x54a2840f, 4 },
    { 0, 0 },
    { 0x1d268411, 3 },
    { 0, 0 },
    { 0xf50e8413, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae8416, 3 },
    { 0x2b168417, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a841c, 3 },
    { 0, 0 },
    { 0xd2b2841e, 3 },
    { 0, 0 },
    { 0x11a28420, 4 },
    { 0xb4828421, 3 },
    { 0x4cb68422, 3 },
    { 0xe8728423, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2468428, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0a842c, 3 },
    { 0x111a842c, 4 },
    { 0, 0 },
    { 0xc06e842f, 3 },
    { 0, 0 },
    { 0x731a8431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54428434, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a68437, 3 },
    { 0x127e8438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce843f, 3 },
    { 0xc9868440, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf928444, 5 },
    { 0x1b428445, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4a8449, 3 },
    { 0xf05e844a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2428450, 4 },
    { 0xc23a8451, 4 },
    { 0, 0 },
    { 0x962a8453, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e68456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e6845c, 3 },
    { 0x6fd6845c, 3 },
    { 0, 0 },
    { 0x660a845f, 3 },
    { 0, 0 },
    { 0xad7a8461, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71768464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68467, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42846c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e28470, 5 },
    { 0xf0328471, 3 },
    { 0x1c468472, 3 },
    { 0x2528473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5528479, 4 },
    { 0x95c6847a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ee847f, 3 },
    { 0xe76e847f, 3 },
    { 0x40868481, 4 },
    { 0x650e8481, 3 },
    { 0x75068483, 3 },
    { 0x9f368482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35e8489, 4 },
    { 0, 0 },
    { 0x2122848b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a848f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3428495, 3 },
    { 0, 0 },
    { 0xbf3e8497, 3 },
    { 0, 0 },
    { 0xd8ca8499, 4 },
    { 0x152849a, 3 },
    { 0x5db6849a, 4 },
    { 0, 0 },
    { 0xc502849d, 3 },
    { 0xa556849d, 3 },
    { 0xf5ca849e, 3 },
    { 0x262684a0, 3 },
    { 0xcd16849e, 4 },
    { 0xfd8684a2, 4 },
    { 0x77f284a0, 3 },
    { 0xe8ca84a4, 4 },
    { 0, 0 },
    { 0xe72a84a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b684a9, 3 },
    { 0x726684a9, 4 },
    { 0, 0 },
    { 0x732284ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ba84b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426a84b6, 3 },
    { 0x608a84b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fa84bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93284c2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a84c9, 3 },
    { 0, 0 },
    { 0x534e84cb, 3 },
    { 0xe3f284cc, 5 },
    { 0xcc0a84cd, 4 },
    { 0xca3684cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1684d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa84d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6e84d7, 3 },
    { 0x385684d8, 3 },
    { 0x9ebe84d9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ea84dd, 4 },
    { 0, 0 },
    { 0xd18a84df, 4 },
    { 0xa02e84df, 3 },
    { 0, 0 },
    { 0xc02284e2, 3 },
    { 0xf52684e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689a84ea, 4 },
    { 0xa22e84ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e84ee, 3 },
    { 0, 0 },
    { 0x5a6684f0, 3 },
    { 0x499e84f1, 3 },
    { 0x15be84f2, 3 },
    { 0xe7e684f3, 4 },
    { 0x19d684f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e84f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0684fc, 3 },
    { 0x4a3a84fd, 3 },
    { 0x860a84fc, 3 },
    { 0x278e84ff, 4 },
    { 0xca84ff, 3 },
    { 0x94228500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ba8506, 3 },
    { 0, 0 },
    { 0xe2028508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647a850e, 3 },
    { 0x1e12850f, 3 },
    { 0x1fe28510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d968514, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc12851a, 3 },
    { 0x4c4a851b, 3 },
    { 0, 0 },
    { 0x9692851d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a68520, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa328523, 3 },
    { 0xdfe28524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b6852b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x839e852e, 3 },
    { 0x3266852e, 3 },
    { 0xdafa8530, 4 },
    { 0xbfb68531, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51e8534, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x777e8537, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27da853b, 3 },
    { 0x14c2853b, 3 },
    { 0, 0 },
    { 0x5f92853e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f68541, 3 },
    { 0xec5a8541, 3 },
    { 0x4f7a8542, 3 },
    { 0x266a8544, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe68549, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa854d, 3 },
    { 0xaea6854e, 4 },
    { 0x5176854f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a8553, 3 },
    { 0x56d68553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a028557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a855b, 4 },
    { 0xacca855c, 3 },
    { 0, 0 },
    { 0x83d6855e, 4 },
    { 0, 0 },
    { 0xb3ee8560, 3 },
    { 0xde568561, 3 },
    { 0x768e8562, 3 },
    { 0xce028561, 3 },
    { 0xc9e28561, 4 },
    { 0x2ba68565, 3 },
    { 0, 0 },
    { 0x83628567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd16856b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f2856e, 3 },
    { 0x1d5e856e, 3 },
    { 0, 0 },
    { 0xe7da8571, 4 },
    { 0, 0 },
    { 0x38928573, 3 },
    { 0x61128573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb68578, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1a857b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26568582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19128585, 3 },
    { 0x37428585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e858a, 3 },
    { 0, 0 },
    { 0x3ac6858c, 3 },
    { 0xbdd2858d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e8590, 4 },
    { 0x8e1e8591, 4 },
    { 0xb47e8592, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6859a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3a859e, 3 },
    { 0x2da2859f, 3 },
    { 0xc7fe85a0, 3 },
    { 0x1f3285a1, 3 },
    { 0x8376859f, 3 },
    { 0, 0 },
    { 0xbcd685a4, 5 },
    { 0x4de285a5, 3 },
    { 0xea2e85a6, 3 },
    { 0xec4a85a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a85ab, 4 },
    { 0xeaba85ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318e85b2, 3 },
    { 0xe8be85b3, 3 },
    { 0, 0 },
    { 0x2fd685b5, 3 },
    { 0xa6ce85b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db685bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e85c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3685c6, 3 },
    { 0x4d0685c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa85cc, 4 },
    { 0, 0 },
    { 0x33ea85ce, 4 },
    { 0, 0 },
    { 0xf2ca85d0, 3 },
    { 0x918685d1, 4 },
    { 0xa3a285d1, 3 },
    { 0x213a85d0, 4 },
    { 0xb34285d0, 3 },
    { 0, 0 },
    { 0x3fce85d6, 3 },
    { 0x6ce285d7, 3 },
    { 0x3b7e85d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915a85e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ce85e6, 3 },
    { 0x921285e7, 3 },
    { 0, 0 },
    { 0x213e85e9, 3 },
    { 0xd43e85e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7a85ed, 3 },
    { 0xe8b685ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e685f5, 3 },
    { 0x566a85f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d685f9, 3 },
    { 0, 0 },
    { 0xb63e85fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14a85ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91868605, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a8610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34128613, 3 },
    { 0x135a8614, 4 },
    { 0, 0 },
    { 0x59e28616, 3 },
    { 0x4b728616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fa861a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe506861e, 4 },
    { 0xfd0a861f, 3 },
    { 0xf768620, 3 },
    { 0x82528620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b68626, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66328629, 4 },
    { 0, 0 },
    { 0x2b76862b, 3 },
    { 0x850e862c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f668630, 3 },
    { 0x60468630, 3 },
    { 0, 0 },
    { 0x1b268633, 3 },
    { 0xce8634, 3 },
    { 0x72ca8634, 3 },
    { 0x58828636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8a863f, 3 },
    { 0xa1f2863f, 3 },
    { 0x329a8641, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aba8645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235a864a, 3 },
    { 0xa912864b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39be8651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef28659, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23028661, 3 },
    { 0xc58e8662, 3 },
    { 0xc84a8662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7128667, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f2866b, 3 },
    { 0, 0 },
    { 0xedce866d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a8677, 3 },
    { 0x4d628677, 3 },
    { 0xc1328677, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cba867d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7a8681, 3 },
    { 0x79068682, 3 },
    { 0x56528683, 4 },
    { 0, 0 },
    { 0xeb468685, 4 },
    { 0x66d68686, 3 },
    { 0, 0 },
    { 0x7e368688, 3 },
    { 0x8f728689, 3 },
    { 0x83b2868a, 3 },
    { 0, 0 },
    { 0x5d4a868c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ea868f, 3 },
    { 0x986e8690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8a8694, 3 },
    { 0x21328695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac76869b, 4 },
    { 0, 0 },
    { 0xef82869d, 3 },
    { 0x9aa2869d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8686a3, 5 },
    { 0, 0 },
    { 0xfce286a5, 3 },
    { 0xe37686a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9286aa, 3 },
    { 0x550686ab, 4 },
    { 0x4f1286ac, 3 },
    { 0x122686ad, 3 },
    { 0, 0 },
    { 0xb31a86af, 3 },
    { 0xc53686af, 3 },
    { 0x463e86b1, 4 },
    { 0, 0 },
    { 0x3b9286b3, 3 },
    { 0, 0 },
    { 0x9ffa86b5, 3 },
    { 0, 0 },
    { 0x500a86b7, 4 },
    { 0x768286b7, 3 },
    { 0xb08a86b9, 3 },
    { 0xf5b686b9, 4 },
    { 0x808286ba, 3 },
    { 0, 0 },
    { 0x59ca86bd, 3 },
    { 0xe0be86be, 3 },
    { 0xac9a86bf, 3 },
    { 0x11e86c0, 3 },
    { 0xd52286c0, 3 },
    { 0, 0 },
    { 0x70a286c3, 3 },
    { 0x152a86c4, 3 },
    { 0x4ec686c5, 3 },
    { 0, 0 },
    { 0xda2686c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38286cb, 4 },
    { 0, 0 },
    { 0x7e5e86cd, 3 },
    { 0x772a86ce, 4 },
    { 0x409e86cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b286d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c286d5, 4 },
    { 0x328e86d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e86dd, 3 },
    { 0xc36686de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d686e3, 3 },
    { 0x71a686e3, 3 },
    { 0x39f286e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea686e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeba86ed, 3 },
    { 0, 0 },
    { 0xc00686ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a86f3, 4 },
    { 0x567a86f4, 4 },
    { 0xa3686f5, 3 },
    { 0xc36a86f5, 3 },
    { 0x608286f7, 3 },
    { 0x53e286f4, 3 },
    { 0x96a686f9, 4 },
    { 0x93d286fa, 3 },
    { 0x5bfa86f9, 3 },
    { 0, 0 },
    { 0x192286fd, 3 },
    { 0xf6f686fe, 3 },
    { 0x758a86ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x848a8702, 3 },
    { 0xc75e8703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb028706, 3 },
    { 0xee528707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ea870f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x460a8712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ba8715, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e8718, 3 },
    { 0xd90a8719, 3 },
    { 0, 0 },
    { 0x8fb6871b, 3 },
    { 0, 0 },
    { 0x8fc2871d, 5 },
    { 0, 0 },
    { 0x75d2871f, 4 },
    { 0xfb968720, 3 },
    { 0x616a8721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879a8729, 3 },
    { 0, 0 },
    { 0x2166872b, 3 },
    { 0x796e872c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabba8732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4068735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a873a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ce8743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e874a, 3 },
    { 0, 0 },
    { 0x2fce874c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31928750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ea8757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210a875b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9556875e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfba8763, 4 },
    { 0xbca68764, 4 },
    { 0, 0 },
    { 0xc0ee8766, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddba876b, 3 },
    { 0x6762876c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a876f, 3 },
    { 0xb4928770, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca8773, 3 },
    { 0x5cc68774, 3 },
    { 0x3f568774, 3 },
    { 0x932e8776, 4 },
    { 0x647e8773, 4 },
    { 0x5cc68778, 3 },
    { 0, 0 },
    { 0x9b96877a, 3 },
    { 0x34e877a, 3 },
    { 0xb6fa877c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a8780, 4 },
    { 0xb0b68780, 4 },
    { 0, 0 },
    { 0xb9068783, 3 },
    { 0, 0 },
    { 0x893a8785, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87a8788, 3 },
    { 0, 0 },
    { 0x6906878a, 3 },
    { 0x3116878b, 4 },
    { 0xa82a878c, 3 },
    { 0, 0 },
    { 0x8d56878e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486a8794, 4 },
    { 0x3e928795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe872879a, 4 },
    { 0xec16879b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa87a5, 3 },
    { 0x547287a6, 3 },
    { 0x890a87a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af287aa, 3 },
    { 0x53d687ab, 3 },
    { 0x9ac687ac, 3 },
    { 0x40a287ad, 3 },
    { 0x673687ab, 3 },
    { 0x6a4a87aa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa687b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97287b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da287ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647287be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760a87c2, 4 },
    { 0x29d287c2, 3 },
    { 0x888a87c4, 3 },
    { 0x251a87c4, 3 },
    { 0, 0 },
    { 0xceda87c7, 3 },
    { 0, 0 },
    { 0xb1fa87c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54287ce, 4 },
    { 0, 0 },
    { 0xe90287d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233a87d6, 3 },
    { 0xbf6e87d7, 4 },
    { 0, 0 },
    { 0xb10a87d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1e87de, 3 },
    { 0x2dc287df, 3 },
    { 0, 0 },
    { 0xf82687e1, 4 },
    { 0x469a87e2, 3 },
    { 0x86ce87e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ea87e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a287ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2e87ed, 3 },
    { 0, 0 },
    { 0xdeea87ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a87f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbe87f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3687fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e8802, 3 },
    { 0xbe5e8803, 4 },
    { 0, 0 },
    { 0xf9928805, 3 },
    { 0x8db28806, 3 },
    { 0xbd828805, 3 },
    { 0xdb128808, 3 },
    { 0, 0 },
    { 0x7dde880a, 3 },
    { 0x7772880b, 4 },
    { 0x8496880b, 3 },
    { 0x2f52880d, 3 },
    { 0, 0 },
    { 0xb392880f, 3 },
    { 0x49768810, 3 },
    { 0x38ee8811, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76068814, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35a8821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ce8829, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f32882c, 3 },
    { 0xd09a882c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa28830, 3 },
    { 0x6028830, 3 },
    { 0x2df68832, 4 },
    { 0xcc7a8832, 3 },
    { 0xd20e8830, 3 },
    { 0x87268834, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d928841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7e8847, 4 },
    { 0xc43e8847, 3 },
    { 0, 0 },
    { 0x3bc6884a, 4 },
    { 0xa83a884a, 4 },
    { 0x1c86884a, 3 },
    { 0x6e1a884d, 3 },
    { 0, 0 },
    { 0xbc26884f, 3 },
    { 0, 0 },
    { 0x35128851, 3 },
    { 0x99528851, 3 },
    { 0, 0 },
    { 0x34028854, 3 },
    { 0xc6528855, 3 },
    { 0, 0 },
    { 0xe50e8857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476885f, 3 },
    { 0x9066885f, 3 },
    { 0x3a1e8861, 3 },
    { 0xe58e8862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aa8868, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a886b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc316886e, 4 },
    { 0x9cae886f, 3 },
    { 0, 0 },
    { 0xba268871, 4 },
    { 0x8a68872, 3 },
    { 0x158e8873, 3 },
    { 0x556e8874, 3 },
    { 0, 0 },
    { 0xb6ee8876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe68879, 4 },
    { 0x95568879, 4 },
    { 0x3ea887a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd32887e, 3 },
    { 0, 0 },
    { 0x37428880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b68883, 3 },
    { 0, 0 },
    { 0x2c828885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678a8889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a6888e, 3 },
    { 0x3636888e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3806889e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59688a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680a88a8, 3 },
    { 0x404288a9, 3 },
    { 0x1bce88a8, 3 },
    { 0, 0 },
    { 0x278688ac, 4 },
    { 0xa7ca88ad, 3 },
    { 0, 0 },
    { 0x169e88af, 4 },
    { 0x825a88b0, 4 },
    { 0xb10e88b1, 3 },
    { 0xc8ce88b2, 3 },
    { 0, 0 },
    { 0x550288b4, 4 },
    { 0x249688b5, 3 },
    { 0x105288b6, 3 },
    { 0x531288b7, 3 },
    { 0x4d688b8, 3 },
    { 0x342a88b9, 3 },
    { 0x206e88ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5a88bd, 3 },
    { 0, 0 },
    { 0x5e9288bf, 3 },
    { 0x7c5a88bf, 3 },
    { 0xdc6e88c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fba88c6, 4 },
    { 0xee2288c7, 3 },
    { 0x9bd688c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11e88d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9e88d8, 3 },
    { 0x91be88d9, 3 },
    { 0x98be88da, 4 },
    { 0, 0 },
    { 0x274288dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385288e4, 3 },
    { 0x274e88e5, 5 },
    { 0xe0ba88e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a288ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66688f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684688f5, 3 },
    { 0xbcc688f6, 3 },
    { 0x29fe88f7, 3 },
    { 0xfa2288f8, 3 },
    { 0xf9a688f9, 3 },
    { 0, 0 },
    { 0x50ca88fb, 3 },
    { 0x81ba88fc, 4 },
    { 0xacca88fb, 3 },
    { 0xb11a88fd, 3 },
    { 0, 0 },
    { 0x90228900, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x923e8903, 3 },
    { 0x3a768904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406a890a, 3 },
    { 0xd9da890b, 3 },
    { 0, 0 },
    { 0x32d2890d, 3 },
    { 0, 0 },
    { 0x63d6890f, 3 },
    { 0x4ada8910, 4 },
    { 0x983e890f, 3 },
    { 0xd41e8911, 3 },
    { 0x300e8910, 3 },
    { 0xca328912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2568920, 4 },
    { 0x36d28921, 4 },
    { 0, 0 },
    { 0x797a8923, 3 },
    { 0x805e8924, 3 },
    { 0x93ce8925, 4 },
    { 0, 0 },
    { 0x5468927, 3 },
    { 0x2afe8928, 3 },
    { 0x9b428928, 3 },
    { 0, 0 },
    { 0x898e892b, 3 },
    { 0xaaca892c, 4 },
    { 0xa422892c, 3 },
    { 0x2532892e, 3 },
    { 0xb95e892c, 4 },
    { 0, 0 },
    { 0x7cee8931, 3 },
    { 0xad468931, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09e8937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dda893c, 3 },
    { 0, 0 },
    { 0xf15e893e, 4 },
    { 0x799a893f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e8944, 4 },
    { 0xced28944, 4 },
    { 0, 0 },
    { 0xf8228947, 3 },
    { 0xb4028947, 3 },
    { 0xac1a8947, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a894c, 4 },
    { 0x3e76894c, 3 },
    { 0x8c46894e, 4 },
    { 0, 0 },
    { 0xee8e8950, 3 },
    { 0x849a8950, 3 },
    { 0x48b28952, 3 },
    { 0, 0 },
    { 0x56828954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71e8957, 3 },
    { 0, 0 },
    { 0xa81e8959, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc136895d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x16168960, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e8966, 3 },
    { 0x5eb28966, 3 },
    { 0x541e8968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e42896d, 3 },
    { 0, 0 },
    { 0xbdb6896f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a8973, 4 },
    { 0, 0 },
    { 0xdeb68975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2897c, 3 },
    { 0, 0 },
    { 0xaa72897e, 3 },
    { 0xae82897f, 3 },
    { 0xce1a8980, 3 },
    { 0x3db68981, 3 },
    { 0xb4128982, 3 },
    { 0x9eba8983, 3 },
    { 0xae628982, 3 },
    { 0xdc7e8985, 3 },
    { 0x885e8986, 3 },
    { 0x7e82897e, 3 },
    { 0x12168983, 3 },
    { 0x935e8989, 3 },
    { 0x962e8984, 3 },
    { 0x4f7e8987, 3 },
    { 0x94c28982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca28991, 3 },
    { 0xa21a8992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e8999, 4 },
    { 0, 0 },
    { 0xa6de899b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8a89a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a89a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e89af, 4 },
    { 0xabc689af, 4 },
    { 0xafaa89af, 3 },
    { 0, 0 },
    { 0x5a9e89b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90289b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735689be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8a89c2, 4 },
    { 0x4efa89c3, 3 },
    { 0xf16289c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197289ce, 3 },
    { 0, 0 },
    { 0x57a89d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1289d4, 3 },
    { 0xd47a89d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0e89d8, 4 },
    { 0x4ffa89d9, 3 },
    { 0xf2b289da, 4 },
    { 0xaf2a89da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40689e8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262e89ee, 3 },
    { 0xadf289ef, 4 },
    { 0x2dd289f0, 4 },
    { 0xb07289f0, 4 },
    { 0x3cd689f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a289f7, 4 },
    { 0xfc289f8, 3 },
    { 0xed4a89f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6be89fe, 3 },
    { 0x669689fe, 4 },
    { 0x93f689fe, 3 },
    { 0, 0 },
    { 0x249a8a02, 3 },
    { 0, 0 },
    { 0xede8a04, 4 },
    { 0, 0 },
    { 0x15b68a06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615e8a0d, 4 },
    { 0, 0 },
    { 0x439e8a0f, 3 },
    { 0xfe6a8a0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36768a13, 3 },
    { 0x2ce68a14, 3 },
    { 0x44a68a15, 3 },
    { 0x39ae8a16, 3 },
    { 0x97c28a13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537a8a1b, 3 },
    { 0xba1a8a1c, 3 },
    { 0x4c868a1b, 3 },
    { 0xdece8a1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d628a23, 5 },
    { 0, 0 },
    { 0x7c3e8a25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a8a28, 5 },
    { 0x912a8a29, 3 },
    { 0, 0 },
    { 0xe9968a2b, 4 },
    { 0x9d2a8a2b, 3 },
    { 0, 0 },
    { 0x6d428a2e, 3 },
    { 0xb4a28a2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de28a37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fee8a3a, 3 },
    { 0, 0 },
    { 0xf96e8a3c, 4 },
    { 0x156a8a3d, 3 },
    { 0, 0 },
    { 0x853a8a3f, 3 },
    { 0x9c228a40, 3 },
    { 0x4e5a8a41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2a8a48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d128a4b, 3 },
    { 0, 0 },
    { 0xac728a4d, 3 },
    { 0x87ba8a4e, 4 },
    { 0xda268a4f, 3 },
    { 0, 0 },
    { 0x3cf28a51, 5 },
    { 0x3f6a8a52, 3 },
    { 0xe7528a53, 4 },
    { 0xa4468a51, 3 },
    { 0x1ed28a53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9268a58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ae8a5f, 3 },
    { 0xb4ea8a60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a428a65, 4 },
    { 0x5a68a66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a8a6d, 3 },
    { 0, 0 },
    { 0xe4a28a6f, 3 },
    { 0, 0 },
    { 0x3a668a71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822a8a77, 4 },
    { 0x3168a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4168a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4768a80, 3 },
    { 0, 0 },
    { 0xd8128a82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8a8a85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae8a8a, 4 },
    { 0xd52e8a8b, 3 },
    { 0x26f68a8c, 4 },
    { 0xf4a8a8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa8a91, 3 },
    { 0, 0 },
    { 0x662e8a93, 5 },
    { 0x7fde8a94, 4 },
    { 0x2b728a95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ba8a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7268a9f, 3 },
    { 0x9a368a9f, 3 },
    { 0x4b2e8aa1, 3 },
    { 0xba68aa2, 3 },
    { 0, 0 },
    { 0x49be8aa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b928aad, 3 },
    { 0x8b068aae, 3 },
    { 0xbca8aaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0528ab2, 4 },
    { 0, 0 },
    { 0xa0628ab4, 3 },
    { 0x78e68ab4, 3 },
    { 0xe3da8ab6, 3 },
    { 0x6e1a8ab7, 3 },
    { 0xf3568ab4, 3 },
    { 0, 0 },
    { 0x24368aba, 3 },
    { 0x17528abb, 3 },
    { 0xbfda8abb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2828abf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71528ac7, 3 },
    { 0, 0 },
    { 0xf5da8ac9, 4 },
    { 0, 0 },
    { 0x85d28acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9768ace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ca8ad1, 3 },
    { 0x8caa8ad2, 3 },
    { 0x45d68ad3, 3 },
    { 0xd50e8ad2, 3 },
    { 0xb8ae8ad5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744e8ad9, 3 },
    { 0, 0 },
    { 0xdcde8adb, 3 },
    { 0xdc2a8adb, 3 },
    { 0xb80a8adb, 4 },
    { 0xb17a8adb, 3 },
    { 0xd1e28adf, 3 },
    { 0x6ad68ae0, 3 },
    { 0x8528ae1, 4 },
    { 0x657e8ae1, 4 },
    { 0x562e8ae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ca8aed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b028af1, 3 },
    { 0x39a8af2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e28af5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a8af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a368afe, 4 },
    { 0, 0 },
    { 0x2afe8b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f368b03, 4 },
    { 0xd22e8b03, 4 },
    { 0xba928b03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4968b08, 3 },
    { 0xe768b09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6928b0c, 4 },
    { 0x62368b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da68b10, 4 },
    { 0xc55a8b10, 3 },
    { 0x4a2a8b12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x634a8b15, 3 },
    { 0xb4e8b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9728b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e8b23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68e8b27, 3 },
    { 0x26768b28, 3 },
    { 0x18868b28, 3 },
    { 0xc09a8b27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f128b2d, 4 },
    { 0x29da8b2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8b32, 3 },
    { 0xf1828b32, 3 },
    { 0xe8128b32, 4 },
    { 0xf9468b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd268b43, 3 },
    { 0xd2068b44, 3 },
    { 0x50868b45, 3 },
    { 0, 0 },
    { 0x35fe8b47, 3 },
    { 0, 0 },
    { 0x52d68b49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba8b4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290a8b54, 3 },
    { 0xf92e8b55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e8b59, 4 },
    { 0, 0 },
    { 0xa36e8b5b, 3 },
    { 0, 0 },
    { 0x10728b5d, 3 },
    { 0, 0 },
    { 0xa4b28b5f, 3 },
    { 0x89f28b60, 4 },
    { 0xaf3e8b61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b68b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7a8b69, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf368b6c, 4 },
    { 0xfc5a8b6c, 3 },
    { 0x42128b6e, 4 },
    { 0, 0 },
    { 0x66d68b70, 3 },
    { 0, 0 },
    { 0x6de28b72, 3 },
    { 0x6e2e8b73, 4 },
    { 0xbda8b74, 5 },
    { 0x82ca8b73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49068b78, 4 },
    { 0x82c68b78, 4 },
    { 0, 0 },
    { 0x41fa8b7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3a8b7e, 4 },
    { 0, 0 },
    { 0x3d28b80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d928b83, 4 },
    { 0xe0a68b84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8b8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151e8b97, 3 },
    { 0xc80a8b98, 3 },
    { 0xfe868b97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff68b9d, 3 },
    { 0x2e768b9e, 3 },
    { 0, 0 },
    { 0xc2d68ba0, 3 },
    { 0, 0 },
    { 0x48668ba2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f68ba5, 3 },
    { 0xe9ce8ba6, 3 },
    { 0xdee28ba7, 3 },
    { 0x91968ba7, 3 },
    { 0xd0828ba6, 3 },
    { 0x2d128baa, 3 },
    { 0, 0 },
    { 0xf1be8bac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fa8baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e8bb5, 3 },
    { 0x73228bb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58828bba, 3 },
    { 0, 0 },
    { 0xafc68bbc, 3 },
    { 0x43b68bbc, 3 },
    { 0x81828bbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29028bc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaae8bc6, 3 },
    { 0x72668bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf68bca, 3 },
    { 0, 0 },
    { 0xa6da8bcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee28bcf, 3 },
    { 0xa1e28bd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ba8bd3, 4 },
    { 0x50be8bd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e8bd8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e68bde, 4 },
    { 0, 0 },
    { 0x1aca8be0, 4 },
    { 0x236e8be0, 3 },
    { 0x60da8be1, 4 },
    { 0, 0 },
    { 0x488a8be4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27468be9, 4 },
    { 0, 0 },
    { 0x96f68beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cba8bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877e8bf3, 3 },
    { 0x7b968bf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc28bf7, 3 },
    { 0, 0 },
    { 0x19368bf9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa8bfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddba8c05, 3 },
    { 0x2e9e8c06, 3 },
    { 0, 0 },
    { 0x23ea8c08, 3 },
    { 0xec4e8c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc28c0d, 3 },
    { 0x5abe8c0e, 3 },
    { 0x169e8c0f, 3 },
    { 0, 0 },
    { 0x70d68c11, 3 },
    { 0x48be8c12, 4 },
    { 0x4ede8c11, 3 },
    { 0, 0 },
    { 0x5b968c15, 5 },
    { 0, 0 },
    { 0xa17a8c17, 3 },
    { 0x96728c18, 5 },
    { 0, 0 },
    { 0x59828c1a, 3 },
    { 0xb51e8c1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1568c1e, 4 },
    { 0, 0 },
    { 0x21a8c20, 4 },
    { 0xea768c21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e8c26, 3 },
    { 0x9eae8c26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d668c2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fea8c31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e8c34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcece8c37, 3 },
    { 0, 0 },
    { 0x7d4a8c39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a8c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e8c42, 3 },
    { 0xcdb28c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa8c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e8c4c, 3 },
    { 0xe8a68c4d, 3 },
    { 0x95c28c4c, 4 },
    { 0x99a8c4f, 5 },
    { 0xa7268c4f, 3 },
    { 0xb6668c51, 3 },
    { 0x48628c52, 4 },
    { 0xa12a8c53, 3 },
    { 0x432a8c54, 3 },
    { 0x16568c54, 3 },
    { 0x72fe8c56, 3 },
    { 0x2128c4d, 3 },
    { 0x456e8c58, 3 },
    { 0, 0 },
    { 0x78968c5a, 3 },
    { 0xd06e8c5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efa8c5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbe8c62, 3 },
    { 0, 0 },
    { 0xa62a8c64, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5068c6b, 3 },
    { 0x78ae8c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ae8c70, 3 },
    { 0, 0 },
    { 0xb7d28c72, 3 },
    { 0x2be8c72, 3 },
    { 0x20968c73, 4 },
    { 0, 0 },
    { 0xeb68c76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca68c79, 4 },
    { 0xda8c7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f68c7e, 3 },
    { 0xf9a68c7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136a8c86, 4 },
    { 0xa1c68c86, 4 },
    { 0xd3a28c88, 3 },
    { 0x5c0e8c88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd628c8c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e8c8f, 3 },
    { 0x64428c90, 4 },
    { 0x24328c91, 4 },
    { 0x63da8c8f, 3 },
    { 0x9e2e8c93, 5 },
    { 0x3668c94, 4 },
    { 0, 0 },
    { 0x1bb68c96, 3 },
    { 0xa26a8c96, 3 },
    { 0xb3568c98, 3 },
    { 0x6ace8c96, 4 },
    { 0xa1ce8c9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f868ca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4e8ca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9a8ca8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x505a8cab, 3 },
    { 0xff0e8cab, 4 },
    { 0x970e8cab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfa8cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeda8cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9a8cba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8868cbe, 3 },
    { 0, 0 },
    { 0xd33e8cc0, 3 },
    { 0x7e028cc0, 4 },
    { 0xcaaa8cc1, 5 },
    { 0xc5368cc3, 3 },
    { 0xfc1a8cc3, 3 },
    { 0x93ee8cc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c28cca, 3 },
    { 0xc8ea8cca, 3 },
    { 0x27968ccc, 3 },
    { 0xee668ccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da68cd1, 5 },
    { 0x5b828cd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543a8cd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60028cdc, 3 },
    { 0x360a8cdc, 3 },
    { 0xbbe28cde, 3 },
    { 0xfb9a8cdf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde8ce4, 3 },
    { 0xa9728ce5, 4 },
    { 0x25068ce6, 3 },
    { 0xb0a28ce4, 3 },
    { 0, 0 },
    { 0x4eda8ce9, 3 },
    { 0, 0 },
    { 0xc38e8ceb, 4 },
    { 0xf41e8ceb, 4 },
    { 0, 0 },
    { 0x84728cee, 3 },
    { 0xf5be8cee, 3 },
    { 0, 0 },
    { 0xe3b68cf1, 3 },
    { 0x52f28cf2, 3 },
    { 0, 0 },
    { 0x65868cf4, 3 },
    { 0x19ce8cf5, 3 },
    { 0, 0 },
    { 0xd93e8cf7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83e8cfb, 4 },
    { 0x255e8cfc, 3 },
    { 0x2c9a8cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c668d00, 4 },
    { 0, 0 },
    { 0xe0fa8d02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f228d05, 3 },
    { 0x8a828d06, 4 },
    { 0x3dbe8d06, 4 },
    { 0x37aa8d05, 3 },
    { 0x60d28d08, 4 },
    { 0xa2328d0a, 4 },
    { 0x3c128d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa8d11, 3 },
    { 0, 0 },
    { 0x8e4e8d13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a828d18, 3 },
    { 0, 0 },
    { 0xee568d1a, 3 },
    { 0, 0 },
    { 0xf05e8d1c, 3 },
    { 0, 0 },
    { 0x83d68d1e, 3 },
    { 0xb70e8d1f, 3 },
    { 0xd24a8d20, 3 },
    { 0x415e8d21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea8d26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f168d29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7e8d2f, 3 },
    { 0x36928d30, 3 },
    { 0xb2ca8d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658a8d3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbe8d3d, 4 },
    { 0x24768d3e, 3 },
    { 0xaf3a8d3f, 3 },
    { 0, 0 },
    { 0x30228d41, 3 },
    { 0xbb1e8d42, 4 },
    { 0, 0 },
    { 0x23528d44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdba8d47, 3 },
    { 0x85e28d48, 4 },
    { 0x9d468d48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5a8d4c, 4 },
    { 0x62868d4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd68d52, 3 },
    { 0x485e8d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a768d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc28d5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f28d5f, 3 },
    { 0, 0 },
    { 0x11ee8d61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8d6a, 4 },
    { 0xd6968d6a, 3 },
    { 0x78a8d6c, 4 },
    { 0, 0 },
    { 0xbcc28d6e, 5 },
    { 0x93568d6f, 3 },
    { 0x725e8d70, 3 },
    { 0xaa328d71, 4 },
    { 0xe9d28d72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86928d79, 4 },
    { 0x90428d79, 4 },
    { 0, 0 },
    { 0x88628d7c, 3 },
    { 0x83b28d7d, 3 },
    { 0x1c868d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26328d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x965e8d86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x711e8d89, 4 },
    { 0x80fa8d8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de8d91, 3 },
    { 0x46828d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60328d97, 4 },
    { 0, 0 },
    { 0xc74e8d99, 3 },
    { 0x11e28d9a, 3 },
    { 0xcc2e8d9b, 3 },
    { 0x4eca8d9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94868d9f, 3 },
    { 0xff0e8da0, 3 },
    { 0x62128da1, 3 },
    { 0x7a9e8da0, 4 },
    { 0x1fee8da3, 4 },
    { 0x4c3e8da3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f28da9, 3 },
    { 0, 0 },
    { 0x4968dab, 4 },
    { 0xa8e8dac, 4 },
    { 0x78628dac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae8db5, 3 },
    { 0xb47a8db5, 4 },
    { 0, 0 },
    { 0x64f28db8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a28dbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73128dc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f928dcc, 3 },
    { 0x2d1a8dcd, 3 },
    { 0, 0 },
    { 0x2cfe8dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c68dd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1e8dd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d528dda, 3 },
    { 0x98668dda, 3 },
    { 0x10e28ddc, 3 },
    { 0xabfe8ddb, 3 },
    { 0x1e4a8dde, 4 },
    { 0, 0 },
    { 0x67928de0, 3 },
    { 0xc7b68de1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75428de8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3e8def, 3 },
    { 0xb6428df0, 3 },
    { 0xf6468df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce28dfa, 3 },
    { 0x221a8dfb, 3 },
    { 0x3c0a8dfc, 4 },
    { 0xad6a8dfd, 3 },
    { 0x2d868dfc, 3 },
    { 0xc168dfd, 3 },
    { 0x18c28dfe, 3 },
    { 0, 0 },
    { 0x11de8e02, 5 },
    { 0x4d328e03, 3 },
    { 0, 0 },
    { 0x52528e05, 3 },
    { 0x3bce8e06, 3 },
    { 0xbd868e07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28e8e0b, 3 },
    { 0x80728e0c, 3 },
    { 0, 0 },
    { 0x8a968e0e, 4 },
    { 0, 0 },
    { 0x347a8e10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884a8e17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c168e1b, 4 },
    { 0x224a8e1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c68e1f, 4 },
    { 0x9a828e20, 4 },
    { 0x38ea8e21, 3 },
    { 0x4bfa8e22, 3 },
    { 0x76c28e23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca8e26, 3 },
    { 0x96228e26, 3 },
    { 0x4a9a8e26, 3 },
    { 0xf6168e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ca8e2d, 3 },
    { 0, 0 },
    { 0x87d68e2f, 3 },
    { 0x41e8e30, 3 },
    { 0xc45e8e2f, 3 },
    { 0, 0 },
    { 0x44028e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5728e36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d068e3a, 3 },
    { 0x64aa8e3b, 3 },
    { 0xc2668e3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96e8e3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e068e43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9a8e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de8e4a, 3 },
    { 0xcd268e4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a68e51, 3 },
    { 0xa39e8e52, 3 },
    { 0xfe6e8e53, 4 },
    { 0x70528e52, 4 },
    { 0xbc068e55, 4 },
    { 0x44928e56, 3 },
    { 0xaa1e8e57, 3 },
    { 0xb0ea8e53, 4 },
    { 0x21d68e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eca8e5c, 3 },
    { 0x7b9a8e5c, 3 },
    { 0, 0 },
    { 0x46de8e5f, 4 },
    { 0x43fe8e60, 3 },
    { 0xa9c68e61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae8e6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8e71, 3 },
    { 0xddba8e72, 3 },
    { 0x286a8e73, 3 },
    { 0xaffe8e74, 3 },
    { 0x452a8e71, 3 },
    { 0, 0 },
    { 0xc28e8e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f28e7c, 3 },
    { 0, 0 },
    { 0x14768e7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3e8e81, 3 },
    { 0, 0 },
    { 0x2b8e8e83, 3 },
    { 0xdbae8e84, 3 },
    { 0x957a8e83, 3 },
    { 0xb8528e83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58628e8c, 3 },
    { 0x1aa28e8d, 3 },
    { 0xf2528e8c, 4 },
    { 0xfea8e8f, 3 },
    { 0x9afe8e8f, 3 },
    { 0, 0 },
    { 0xee8e8e92, 3 },
    { 0xaa0a8e92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e8e96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a8e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2368ea5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e8ea9, 3 },
    { 0, 0 },
    { 0x31668eab, 4 },
    { 0x8d128eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3228eb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fe8ebd, 3 },
    { 0xcb928ebd, 3 },
    { 0x27a68ebf, 3 },
    { 0, 0 },
    { 0x92de8ec1, 3 },
    { 0, 0 },
    { 0x52628ec3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba168ec8, 3 },
    { 0x4ed68ec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d528ecc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee28ecf, 3 },
    { 0xf1ae8ecf, 4 },
    { 0, 0 },
    { 0xc2628ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8868ed7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6e8edb, 3 },
    { 0x7fd68edc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34428ee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3e8ee6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb628ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a28eec, 5 },
    { 0, 0 },
    { 0xccfa8eee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e68ef2, 3 },
    { 0xbe3a8ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0a8ef6, 3 },
    { 0x18fa8ef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e8efc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4868f01, 3 },
    { 0xe1de8f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef28f09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffa8f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e8f11, 3 },
    { 0xaf328f12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299a8f17, 3 },
    { 0x10168f18, 4 },
    { 0xb3c28f19, 3 },
    { 0xd79e8f18, 3 },
    { 0x5e568f1b, 3 },
    { 0x245e8f17, 4 },
    { 0xa81e8f1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5e8f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e668f27, 3 },
    { 0, 0 },
    { 0x788a8f29, 3 },
    { 0xb8e8f2a, 3 },
    { 0x71128f2b, 4 },
    { 0, 0 },
    { 0x89768f2d, 3 },
    { 0x152a8f2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd068f33, 3 },
    { 0, 0 },
    { 0x7b0a8f35, 4 },
    { 0xc1a8f36, 4 },
    { 0, 0 },
    { 0xbb968f38, 3 },
    { 0x74468f38, 3 },
    { 0x6e568f38, 3 },
    { 0, 0 },
    { 0x62728f3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aa8f41, 3 },
    { 0xf47a8f42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a8f4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba8f52, 3 },
    { 0x51428f52, 3 },
    { 0x67a68f54, 4 },
    { 0xc2168f53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c28f5d, 3 },
    { 0xf96e8f5d, 3 },
    { 0x1dd28f5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6368f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03e8f66, 3 },
    { 0xb9268f67, 3 },
    { 0xd00a8f68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7228f6c, 3 },
    { 0x3f028f6d, 4 },
    { 0x44228f6e, 3 },
    { 0x56028f6c, 3 },
    { 0xb6e8f70, 3 },
    { 0, 0 },
    { 0x9dde8f72, 4 },
    { 0, 0 },
    { 0x86fa8f74, 3 },
    { 0xda9e8f74, 3 },
    { 0, 0 },
    { 0x89e28f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a8f7a, 4 },
    { 0xb31e8f7b, 3 },
    { 0, 0 },
    { 0x3e728f7d, 3 },
    { 0x2b628f7d, 3 },
    { 0, 0 },
    { 0x98e8f80, 3 },
    { 0xf4568f81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98e8f84, 4 },
    { 0xb4e68f84, 3 },
    { 0, 0 },
    { 0x1c7e8f87, 4 },
    { 0, 0 },
    { 0xf9668f89, 3 },
    { 0, 0 },
    { 0x10fe8f8b, 3 },
    { 0x1eb68f8c, 3 },
    { 0xbbb68f8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92728f93, 3 },
    { 0, 0 },
    { 0xa8028f95, 3 },
    { 0x86228f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca8f9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x430a8f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d68fa3, 3 },
    { 0x15528fa3, 3 },
    { 0xf8768fa3, 4 },
    { 0x75ee8fa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffe8fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ba8fb1, 4 },
    { 0x7bfe8fb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd28fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0de8fbb, 3 },
    { 0xf6528fbb, 3 },
    { 0, 0 },
    { 0x1468fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8a8fc1, 3 },
    { 0x2cd68fc1, 5 },
    { 0x9d028fc3, 3 },
    { 0x5aa28fc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f28fca, 3 },
    { 0xebb28fcb, 4 },
    { 0, 0 },
    { 0x28628fcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aca8fd5, 3 },
    { 0, 0 },
    { 0x404e8fd7, 4 },
    { 0x94b28fd8, 4 },
    { 0xc61a8fd7, 3 },
    { 0x42428fda, 4 },
    { 0, 0 },
    { 0x56928fdc, 4 },
    { 0, 0 },
    { 0x9efe8fde, 5 },
    { 0xda8a8fdf, 4 },
    { 0, 0 },
    { 0xc7ee8fe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c628fe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fe9006, 3 },
    { 0xc20e9006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x239a900a, 4 },
    { 0xe556900b, 3 },
    { 0x65d2900a, 3 },
    { 0x6dd6900a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7629012, 3 },
    { 0xa0be9013, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca901a, 3 },
    { 0xd36a901a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe901e, 3 },
    { 0x7a2e901f, 3 },
    { 0x22969020, 3 },
    { 0x187e901e, 3 },
    { 0xf39e9021, 4 },
    { 0x9ca29020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef769029, 4 },
    { 0, 0 },
    { 0x52fe902b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9e902e, 3 },
    { 0xa60e902f, 4 },
    { 0x14b69030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5e9033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a9037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3a903a, 3 },
    { 0xac9e903a, 3 },
    { 0xb886903c, 3 },
    { 0x8c0a903c, 3 },
    { 0x4122903c, 3 },
    { 0x483a903e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a9043, 4 },
    { 0x1fb29043, 4 },
    { 0x47569045, 3 },
    { 0xc3ee9046, 3 },
    { 0xaec29043, 3 },
    { 0xa0fa9046, 5 },
    { 0, 0 },
    { 0x2456904a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6786904e, 3 },
    { 0xd8ca904f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d6905e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff529064, 4 },
    { 0, 0 },
    { 0x62429066, 4 },
    { 0xf85e9066, 3 },
    { 0xa96e9068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a906c, 4 },
    { 0xe6aa906d, 4 },
    { 0xe33e906e, 3 },
    { 0x5aea906e, 3 },
    { 0xb1de9070, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x154e9073, 4 },
    { 0, 0 },
    { 0xe1329075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60e907c, 4 },
    { 0x52a907d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fe9084, 3 },
    { 0, 0 },
    { 0x66aa9086, 3 },
    { 0xc2969086, 3 },
    { 0xd9a9088, 4 },
    { 0x94529086, 3 },
    { 0x1ca2908a, 3 },
    { 0x9bee908b, 3 },
    { 0x14f2908c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ae908f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b69092, 4 },
    { 0xc9be9093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192909c, 3 },
    { 0xd19e909c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a90a1, 4 },
    { 0x2b1690a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x356690a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1290a8, 4 },
    { 0, 0 },
    { 0x165e90aa, 3 },
    { 0x94ba90ab, 3 },
    { 0x312a90ac, 3 },
    { 0xba9e90ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11e90b0, 3 },
    { 0x8ab290b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc290bc, 3 },
    { 0, 0 },
    { 0xc2ba90be, 3 },
    { 0x77e290bf, 3 },
    { 0xa290c0, 3 },
    { 0x7b7290be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689e90c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef290cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78290cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0290d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3a90d6, 3 },
    { 0xbb6e90d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6e90dc, 4 },
    { 0, 0 },
    { 0xe11290de, 3 },
    { 0x93a90de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37690e3, 3 },
    { 0, 0 },
    { 0xa9be90e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x237690e8, 3 },
    { 0x322e90e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e90f0, 4 },
    { 0x6b1e90f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3290fb, 3 },
    { 0x15690fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225a9100, 3 },
    { 0xdbfa9101, 4 },
    { 0xfd8a9101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4e9106, 4 },
    { 0, 0 },
    { 0xd1369108, 3 },
    { 0, 0 },
    { 0x8032910a, 3 },
    { 0, 0 },
    { 0xcd1a910c, 3 },
    { 0xc14a910d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ce9117, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab6911b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364a9120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d2912a, 3 },
    { 0x9106912a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4856912f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9a9132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8e9136, 5 },
    { 0xbe629137, 4 },
    { 0x42669137, 4 },
    { 0x94269137, 3 },
    { 0x1a36913a, 3 },
    { 0xccee913a, 3 },
    { 0x6f82913c, 3 },
    { 0x4b329139, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaa9141, 4 },
    { 0x213a9142, 3 },
    { 0x62fa9143, 3 },
    { 0xc77e9144, 3 },
    { 0x70e69142, 3 },
    { 0, 0 },
    { 0xe9329147, 3 },
    { 0x78569148, 3 },
    { 0x732a9149, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e6914c, 3 },
    { 0x5bfe914d, 4 },
    { 0xa2e2914e, 3 },
    { 0, 0 },
    { 0x39b69150, 3 },
    { 0, 0 },
    { 0xeab69152, 3 },
    { 0, 0 },
    { 0x383e9154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a9159, 3 },
    { 0x53e6915a, 3 },
    { 0x52b69159, 4 },
    { 0x2a02915c, 3 },
    { 0xbd9e915a, 3 },
    { 0x607e9159, 3 },
    { 0x8936915e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2029164, 3 },
    { 0x9e869165, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc69168, 3 },
    { 0, 0 },
    { 0xb596916a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d46916e, 3 },
    { 0xb4fe916f, 3 },
    { 0x14ca916f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58269173, 3 },
    { 0, 0 },
    { 0x887a9175, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa342917a, 3 },
    { 0xd8da917b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d69180, 3 },
    { 0x9b929180, 4 },
    { 0xb8129182, 3 },
    { 0x4c9e9183, 3 },
    { 0x23b29184, 3 },
    { 0x9a69182, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae918a, 3 },
    { 0xb07e918a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4a918f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaa9196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08a91a3, 3 },
    { 0, 0 },
    { 0x32da91a5, 3 },
    { 0xacaa91a6, 4 },
    { 0xb29691a7, 3 },
    { 0x3f4691a8, 4 },
    { 0, 0 },
    { 0x2a4291aa, 3 },
    { 0x8e5a91ab, 4 },
    { 0xb2fa91ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816e91bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c291c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867691c4, 4 },
    { 0, 0 },
    { 0x265a91c6, 3 },
    { 0xe2291c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe91cb, 3 },
    { 0x37d691cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b691d0, 3 },
    { 0xbee291d0, 4 },
    { 0, 0 },
    { 0x70f291d3, 3 },
    { 0, 0 },
    { 0x4f6291d5, 3 },
    { 0x462291d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507691db, 3 },
    { 0, 0 },
    { 0x24f691dd, 4 },
    { 0x5cfa91de, 3 },
    { 0x115a91df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd291e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfa91e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756291ec, 3 },
    { 0xdb5291ed, 4 },
    { 0, 0 },
    { 0x26b691ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e91f2, 3 },
    { 0, 0 },
    { 0x738e91f4, 4 },
    { 0x722a91f5, 3 },
    { 0x6be291f4, 3 },
    { 0x221a91f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8a91fa, 3 },
    { 0x83ce91fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f69201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecee9204, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10469209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9e920c, 3 },
    { 0xa1ea920d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5e9210, 4 },
    { 0, 0 },
    { 0x1c5e9212, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b829215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93329219, 4 },
    { 0x1e921a, 4 },
    { 0x6ec2921b, 3 },
    { 0x6bb6921b, 3 },
    { 0xc50e921d, 3 },
    { 0xc46e9219, 3 },
    { 0x6836921f, 3 },
    { 0xb25a9220, 3 },
    { 0x3ffa9220, 4 },
    { 0x4be29222, 3 },
    { 0x1fc69223, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f29228, 3 },
    { 0, 0 },
    { 0x7b1e922a, 3 },
    { 0x617e922b, 4 },
    { 0, 0 },
    { 0xd262922d, 3 },
    { 0, 0 },
    { 0x37b2922f, 3 },
    { 0xdb429230, 3 },
    { 0x5706922f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62e9237, 4 },
    { 0x62ba9238, 3 },
    { 0xa5a29238, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aa923c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c29242, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa69245, 4 },
    { 0, 0 },
    { 0xf0b69247, 3 },
    { 0, 0 },
    { 0x876a9249, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd762924c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x298a924f, 3 },
    { 0x7c36924f, 3 },
    { 0x5929251, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e069257, 3 },
    { 0x97a29258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e925b, 3 },
    { 0x6166925c, 5 },
    { 0xfb0a925d, 3 },
    { 0x344e925e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff29262, 5 },
    { 0x17829263, 4 },
    { 0xc8769262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4e9268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912e926d, 5 },
    { 0x5fb6926e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7a9271, 3 },
    { 0x1d429271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220e9276, 4 },
    { 0, 0 },
    { 0x41de9278, 3 },
    { 0x95569278, 3 },
    { 0xedbe927a, 5 },
    { 0xef06927b, 3 },
    { 0xe102927c, 3 },
    { 0xd31e927d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9e9282, 3 },
    { 0x40669283, 4 },
    { 0xb7329283, 3 },
    { 0, 0 },
    { 0x3229286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b2928c, 3 },
    { 0x978a928c, 3 },
    { 0xae6928e, 4 },
    { 0x695a928e, 3 },
    { 0xf9fa9290, 3 },
    { 0xc8ba9291, 4 },
    { 0xddb69292, 3 },
    { 0x6542928d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b629296, 3 },
    { 0x561a9297, 3 },
    { 0, 0 },
    { 0xa9069299, 4 },
    { 0x6fea929a, 4 },
    { 0x2966929a, 3 },
    { 0, 0 },
    { 0xd706929d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e92a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x326a92a4, 3 },
    { 0xa2b292a5, 5 },
    { 0x872e92a5, 3 },
    { 0, 0 },
    { 0x933a92a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba292ae, 4 },
    { 0, 0 },
    { 0xa43a92b0, 4 },
    { 0xf1ca92b1, 3 },
    { 0, 0 },
    { 0xf41a92b3, 3 },
    { 0, 0 },
    { 0xefaa92b5, 3 },
    { 0x32a692b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb692ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a92c5, 4 },
    { 0x109a92c6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2a92d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74292d5, 3 },
    { 0, 0 },
    { 0x258a92d7, 3 },
    { 0, 0 },
    { 0xcf8692d9, 4 },
    { 0xe70a92da, 3 },
    { 0xacd292db, 3 },
    { 0x8db292da, 3 },
    { 0xfb8a92dd, 3 },
    { 0xfc1692db, 4 },
    { 0x7f1292df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x907a92e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738292e7, 4 },
    { 0, 0 },
    { 0x924e92e9, 3 },
    { 0x31f292ea, 3 },
    { 0x3eba92eb, 3 },
    { 0x2a8a92ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a92ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7a92f2, 4 },
    { 0, 0 },
    { 0xf77692f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f692f9, 4 },
    { 0, 0 },
    { 0xd4292fb, 4 },
    { 0x27da92fc, 3 },
    { 0xdce92fd, 4 },
    { 0, 0 },
    { 0x43692ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fa9302, 5 },
    { 0xf78e9303, 4 },
    { 0, 0 },
    { 0x8d3a9305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990e930a, 4 },
    { 0xd2ca930b, 4 },
    { 0x9c2e930b, 3 },
    { 0x651a930a, 3 },
    { 0, 0 },
    { 0x6ba6930f, 4 },
    { 0x77029310, 5 },
    { 0x34ce9311, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7569315, 4 },
    { 0, 0 },
    { 0x11ce9317, 3 },
    { 0xa3a29318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa216931f, 3 },
    { 0xed569320, 4 },
    { 0x8b22931f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d69324, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x97729327, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f2932a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a932f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450e9337, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d6933b, 3 },
    { 0x5a02933c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x327a933f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f29343, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202e934b, 3 },
    { 0x284e934b, 4 },
    { 0xafe934d, 4 },
    { 0x49c2934e, 4 },
    { 0, 0 },
    { 0x895e9350, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a935a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e935d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f69361, 3 },
    { 0x960a9362, 3 },
    { 0xbd569363, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b29366, 4 },
    { 0x57169367, 3 },
    { 0x56a9368, 3 },
    { 0x365e9369, 3 },
    { 0xd52e9369, 5 },
    { 0x5b12936b, 3 },
    { 0xd76e9369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ce9375, 3 },
    { 0, 0 },
    { 0xa7269377, 3 },
    { 0x90469378, 3 },
    { 0x1a229379, 3 },
    { 0, 0 },
    { 0xd94e937b, 3 },
    { 0xc2a6937c, 3 },
    { 0xc432937d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac29381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53669384, 3 },
    { 0, 0 },
    { 0x24829386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb56938a, 3 },
    { 0x377a938b, 3 },
    { 0xe38e938c, 3 },
    { 0, 0 },
    { 0xac52938e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b69394, 3 },
    { 0x6fc69395, 3 },
    { 0xf7229396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2939c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e93a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a93b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a93ba, 4 },
    { 0, 0 },
    { 0x3b5a93bc, 3 },
    { 0xcf5a93bd, 3 },
    { 0x430a93be, 3 },
    { 0xa9f293bf, 4 },
    { 0x189a93c0, 3 },
    { 0xaf7293bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5293c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e293c8, 4 },
    { 0x9a0a93c9, 3 },
    { 0x638693ca, 4 },
    { 0, 0 },
    { 0xb48293cc, 3 },
    { 0, 0 },
    { 0xd9d693ce, 3 },
    { 0xe01e93cf, 3 },
    { 0, 0 },
    { 0xc90693d1, 3 },
    { 0xa6ae93d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded693d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160693da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998693e0, 3 },
    { 0xe6c693e0, 3 },
    { 0xb32693e1, 4 },
    { 0x803693e2, 3 },
    { 0, 0 },
    { 0xdcbe93e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x563293e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab693eb, 3 },
    { 0x6d0293eb, 3 },
    { 0x86ee93eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf693f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aa93f4, 3 },
    { 0x30a293f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x980e93f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680293fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad293ff, 3 },
    { 0, 0 },
    { 0xec429401, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3a9408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4016940d, 3 },
    { 0, 0 },
    { 0x30ca940f, 3 },
    { 0x26d6940f, 3 },
    { 0x734a940f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba9416, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x976a9419, 3 },
    { 0xd29a941a, 4 },
    { 0x645a941b, 3 },
    { 0xed9e941a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb2942c, 3 },
    { 0x5dd2942c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7229430, 3 },
    { 0xa2b29430, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf629438, 3 },
    { 0x9dc29438, 5 },
    { 0, 0 },
    { 0x262a943b, 3 },
    { 0, 0 },
    { 0xb17a943d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe9445, 3 },
    { 0x6f469446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb029449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73a9455, 4 },
    { 0xfe629456, 4 },
    { 0x9e069455, 4 },
    { 0, 0 },
    { 0x1dba9459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fe945d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8e9460, 3 },
    { 0x6c929460, 5 },
    { 0, 0 },
    { 0x59529463, 3 },
    { 0xdb9e9464, 3 },
    { 0x80969465, 4 },
    { 0x5ab29465, 3 },
    { 0xb26a9467, 4 },
    { 0, 0 },
    { 0x34569469, 3 },
    { 0x964e9469, 3 },
    { 0xdfb6946b, 3 },
    { 0x15c6946c, 3 },
    { 0, 0 },
    { 0xfe3e946e, 3 },
    { 0, 0 },
    { 0x455a9470, 3 },
    { 0, 0 },
    { 0x11029472, 3 },
    { 0xf3069473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11269476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fe947b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e947e, 3 },
    { 0xe9e2947f, 3 },
    { 0, 0 },
    { 0xbb8a9481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a29486, 3 },
    { 0, 0 },
    { 0xe2469488, 3 },
    { 0x902a9488, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e948d, 3 },
    { 0xbf02948e, 3 },
    { 0x45a2948f, 3 },
    { 0xfe069490, 3 },
    { 0, 0 },
    { 0x45e9492, 3 },
    { 0, 0 },
    { 0x59269494, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941e9498, 4 },
    { 0x553a9499, 3 },
    { 0xf9d29499, 3 },
    { 0x4bee949a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10294a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a694a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x132e94a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273a94ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80694b0, 3 },
    { 0x512694b1, 4 },
    { 0, 0 },
    { 0xabba94b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x509a94b6, 4 },
    { 0x7f5694b7, 3 },
    { 0x718e94b8, 3 },
    { 0x425a94b9, 3 },
    { 0xbece94b8, 3 },
    { 0xc6fe94bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c694bf, 4 },
    { 0, 0 },
    { 0xc8f294c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32694c6, 3 },
    { 0xf8aa94c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a694ca, 3 },
    { 0xd84694ca, 3 },
    { 0x860694cc, 4 },
    { 0x12ea94cd, 3 },
    { 0xdc4294cb, 3 },
    { 0, 0 },
    { 0xf72694d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97be94d4, 3 },
    { 0, 0 },
    { 0xf3694d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111694dd, 4 },
    { 0x5ab694de, 3 },
    { 0xfab694df, 3 },
    { 0x460294de, 3 },
    { 0xa77e94e1, 3 },
    { 0x94ee94e2, 3 },
    { 0x291a94e3, 3 },
    { 0xb91294de, 3 },
    { 0xd6c694e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ce94e8, 4 },
    { 0, 0 },
    { 0x126694ea, 4 },
    { 0, 0 },
    { 0x994a94ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce94ef, 3 },
    { 0xbefe94f0, 3 },
    { 0xa53294f0, 3 },
    { 0, 0 },
    { 0x4f8e94f3, 3 },
    { 0, 0 },
    { 0x342a94f5, 4 },
    { 0x769294f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce94f9, 3 },
    { 0x7f6e94fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x692a94fd, 3 },
    { 0x949694fe, 4 },
    { 0x88e94fe, 3 },
    { 0x6fae94fd, 3 },
    { 0xd27694ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba9505, 3 },
    { 0x7cb69505, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f86950b, 4 },
    { 0x76d2950b, 3 },
    { 0xf69a950b, 4 },
    { 0xc24a950d, 3 },
    { 0xf16e950f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1da9512, 3 },
    { 0x25ae9513, 3 },
    { 0, 0 },
    { 0x392a9515, 3 },
    { 0xe8c29516, 4 },
    { 0xf0929517, 3 },
    { 0xc9a69517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca66951e, 3 },
    { 0x8f9a951f, 3 },
    { 0, 0 },
    { 0x951a9521, 3 },
    { 0xcd369522, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a9527, 3 },
    { 0x11f29528, 3 },
    { 0xa44e9527, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51e952e, 3 },
    { 0, 0 },
    { 0x286a9530, 3 },
    { 0x80be9531, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff29534, 4 },
    { 0x81929535, 4 },
    { 0x8f469535, 3 },
    { 0, 0 },
    { 0x78f29538, 4 },
    { 0x96869539, 3 },
    { 0x8a8e953a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa576953e, 3 },
    { 0, 0 },
    { 0x2c0e9540, 3 },
    { 0xb1869540, 4 },
    { 0xd1329542, 3 },
    { 0x9cc69543, 3 },
    { 0x655a9544, 3 },
    { 0xb90a9544, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de9548, 4 },
    { 0xc81a9549, 4 },
    { 0x2f92954a, 4 },
    { 0x87729548, 3 },
    { 0x723e954c, 3 },
    { 0x13da954d, 3 },
    { 0x928e954e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af29552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7452955b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935a9561, 3 },
    { 0, 0 },
    { 0x16869563, 3 },
    { 0x92e69564, 3 },
    { 0, 0 },
    { 0xd8de9566, 3 },
    { 0, 0 },
    { 0x669568, 3 },
    { 0xa8729568, 3 },
    { 0, 0 },
    { 0xb9ee956b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d36956e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4729573, 3 },
    { 0x17d29573, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19e9578, 4 },
    { 0xb3829578, 3 },
    { 0x8fe29578, 3 },
    { 0x33fe957b, 4 },
    { 0x8206957c, 3 },
    { 0x8342957d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570e9581, 4 },
    { 0xec6e9582, 4 },
    { 0x51429582, 3 },
    { 0, 0 },
    { 0xaffe9585, 3 },
    { 0x895a9585, 3 },
    { 0x3fb29587, 3 },
    { 0x4869588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998a958c, 3 },
    { 0x4da6958d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36669591, 4 },
    { 0x841e9592, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42169595, 3 },
    { 0, 0 },
    { 0x906e9597, 3 },
    { 0, 0 },
    { 0xb84e9599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc82959c, 4 },
    { 0xa03a959d, 3 },
    { 0x1ee6959e, 3 },
    { 0x441e959e, 3 },
    { 0, 0 },
    { 0xe38295a1, 3 },
    { 0x985295a2, 4 },
    { 0, 0 },
    { 0xb72295a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b695a9, 3 },
    { 0, 0 },
    { 0x702e95ab, 3 },
    { 0, 0 },
    { 0xfd3695ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebe95b1, 3 },
    { 0, 0 },
    { 0xf65a95b3, 3 },
    { 0x7dda95b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b295b9, 3 },
    { 0, 0 },
    { 0xeb2295bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6e95c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f295c6, 4 },
    { 0x846a95c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f695cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1295d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45a95d7, 4 },
    { 0xfff295d8, 3 },
    { 0, 0 },
    { 0xcd8295da, 3 },
    { 0x689695db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3295df, 4 },
    { 0, 0 },
    { 0x830295e1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3695e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108695eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e295ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa695f2, 3 },
    { 0x331695f3, 3 },
    { 0x3c4e95f3, 3 },
    { 0xb08295f5, 3 },
    { 0x92a295f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81695f9, 3 },
    { 0, 0 },
    { 0x991a95fb, 3 },
    { 0, 0 },
    { 0xc71a95fd, 4 },
    { 0xbf6695fe, 3 },
    { 0, 0 },
    { 0x1f369600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ee9604, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74869608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aa960c, 3 },
    { 0, 0 },
    { 0x639e960e, 3 },
    { 0x8286960f, 4 },
    { 0x8f569610, 3 },
    { 0x71729610, 3 },
    { 0x9ac29611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fe9615, 3 },
    { 0, 0 },
    { 0x3a169617, 3 },
    { 0x52969618, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408e961d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9929621, 3 },
    { 0x10a69622, 3 },
    { 0x659e9623, 4 },
    { 0x68729622, 3 },
    { 0, 0 },
    { 0xd9029626, 3 },
    { 0, 0 },
    { 0xcdfa9628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8612962b, 3 },
    { 0x3a46962c, 3 },
    { 0xd8b6962d, 4 },
    { 0x66aa962e, 3 },
    { 0xe472962d, 3 },
    { 0, 0 },
    { 0x23c69631, 4 },
    { 0, 0 },
    { 0x994a9633, 3 },
    { 0x727a9633, 3 },
    { 0x6a929635, 4 },
    { 0xf3f29636, 3 },
    { 0x9b8a9636, 3 },
    { 0, 0 },
    { 0x9f7e9639, 3 },
    { 0x41de9639, 4 },
    { 0x61f2963a, 3 },
    { 0x3f5e963b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e29643, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13229646, 3 },
    { 0x4d869646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc2964e, 3 },
    { 0x22a6964f, 3 },
    { 0x352964e, 4 },
    { 0xa7ee964f, 4 },
    { 0, 0 },
    { 0x39e9653, 3 },
    { 0, 0 },
    { 0x4ea29655, 3 },
    { 0x1e429656, 4 },
    { 0, 0 },
    { 0x6aa29658, 4 },
    { 0xa3be9659, 5 },
    { 0x4e1a9658, 4 },
    { 0xc962965b, 3 },
    { 0, 0 },
    { 0xc43a965d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516966d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b29671, 4 },
    { 0xe86e9671, 3 },
    { 0, 0 },
    { 0x566e9674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e29677, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31be967a, 3 },
    { 0, 0 },
    { 0xd376967c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de9682, 3 },
    { 0x6bba9683, 3 },
    { 0xd6a9683, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2372968a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf192968e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04e9693, 3 },
    { 0x3b029694, 3 },
    { 0, 0 },
    { 0x1d469696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926a969d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57a96a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4e96a6, 4 },
    { 0x436696a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa96aa, 3 },
    { 0x6d9296aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90296b1, 3 },
    { 0x23ea96b2, 3 },
    { 0xcb2e96b1, 3 },
    { 0, 0 },
    { 0x7cfa96b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95296bc, 3 },
    { 0xf66696bc, 5 },
    { 0x213296be, 4 },
    { 0, 0 },
    { 0xb49696c0, 3 },
    { 0, 0 },
    { 0xf3a296c2, 3 },
    { 0, 0 },
    { 0xec4296c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94e96c7, 3 },
    { 0xace96c7, 4 },
    { 0x2b1696c9, 3 },
    { 0, 0 },
    { 0x9e8e96cb, 3 },
    { 0x9f4a96cc, 3 },
    { 0xea2a96cb, 3 },
    { 0xf63696ce, 5 },
    { 0x79de96cc, 3 },
    { 0, 0 },
    { 0x9e7296d1, 3 },
    { 0x937a96d2, 3 },
    { 0x1f2296d3, 3 },
    { 0xfaea96d4, 4 },
    { 0, 0 },
    { 0x528696d6, 3 },
    { 0x49aa96d7, 3 },
    { 0x6d5696d8, 3 },
    { 0x25da96d9, 3 },
    { 0x751696d9, 3 },
    { 0x1dc696d9, 3 },
    { 0xbf6296d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa696df, 4 },
    { 0xb5d696e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd696eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fba96ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba96f1, 5 },
    { 0x963296f2, 3 },
    { 0, 0 },
    { 0x2f6e96f4, 3 },
    { 0xb75696f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e296f9, 3 },
    { 0, 0 },
    { 0x29ba96fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81de9700, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca69704, 3 },
    { 0x57e69705, 5 },
    { 0xbc769706, 3 },
    { 0, 0 },
    { 0xfab29708, 3 },
    { 0xdc869708, 3 },
    { 0xfe26970a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a9711, 3 },
    { 0xf25a9711, 3 },
    { 0, 0 },
    { 0xc6729714, 3 },
    { 0, 0 },
    { 0xf4e29716, 3 },
    { 0xf89a9717, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517e971e, 3 },
    { 0xcc26971f, 4 },
    { 0x11ca9720, 3 },
    { 0xc069721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9725, 3 },
    { 0, 0 },
    { 0x97769727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8a972a, 3 },
    { 0xc9d2972a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x681a972e, 4 },
    { 0xbc8e972f, 3 },
    { 0xa8d2972e, 3 },
    { 0x6fda9730, 3 },
    { 0x207a9732, 3 },
    { 0x964a9732, 3 },
    { 0xcb7e9734, 4 },
    { 0x46fa9735, 4 },
    { 0x17729735, 3 },
    { 0, 0 },
    { 0xdeca9738, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c6973c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e973f, 3 },
    { 0, 0 },
    { 0xacfa9741, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34a9747, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24629750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374a9756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8729759, 4 },
    { 0, 0 },
    { 0xbf82975b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc366975e, 3 },
    { 0x86e2975f, 4 },
    { 0, 0 },
    { 0x996e9761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22269765, 3 },
    { 0xcc429766, 3 },
    { 0x1f469767, 4 },
    { 0x9fe9765, 4 },
    { 0xf18a9769, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456976c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404e9771, 3 },
    { 0x252a9771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde977a, 3 },
    { 0xdf5e977b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8a977f, 3 },
    { 0, 0 },
    { 0x6cb29781, 4 },
    { 0, 0 },
    { 0xee9a9783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c729786, 3 },
    { 0xb3869787, 3 },
    { 0xfe829787, 4 },
    { 0xc5229788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe456978e, 3 },
    { 0xb90a978f, 3 },
    { 0xc97a9790, 4 },
    { 0, 0 },
    { 0x34929792, 3 },
    { 0x4ef29793, 3 },
    { 0, 0 },
    { 0xbd769795, 3 },
    { 0, 0 },
    { 0xbd229797, 3 },
    { 0, 0 },
    { 0xaee69799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e97a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec297a9, 4 },
    { 0xcc4697aa, 3 },
    { 0x8d8697ab, 3 },
    { 0x162e97ab, 3 },
    { 0x547297a9, 4 },
    { 0x6f4297ae, 4 },
    { 0xadfa97ad, 3 },
    { 0xe8e697b0, 3 },
    { 0xaeee97a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5e97bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de97bf, 4 },
    { 0xd66e97c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01697c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550297c9, 3 },
    { 0, 0 },
    { 0x565297cb, 3 },
    { 0, 0 },
    { 0x616a97cd, 4 },
    { 0x130297cd, 3 },
    { 0xd26697cf, 3 },
    { 0xfebe97cd, 3 },
    { 0, 0 },
    { 0x368297d2, 3 },
    { 0x39e297d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a97d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43e97da, 3 },
    { 0x8fc697db, 3 },
    { 0, 0 },
    { 0x553297dd, 4 },
    { 0xc44a97de, 3 },
    { 0x7f0e97dd, 3 },
    { 0xf5d297dd, 3 },
    { 0x330a97e1, 3 },
    { 0xfb8a97df, 3 },
    { 0x294697e3, 3 },
    { 0xaeca97e1, 3 },
    { 0xa44a97e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3697e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea97ee, 3 },
    { 0xf44a97ef, 4 },
    { 0xfe9697f0, 3 },
    { 0x18a97f1, 4 },
    { 0x18e697f2, 3 },
    { 0x52c297f2, 3 },
    { 0x25ba97f4, 4 },
    { 0xb8ca97f5, 3 },
    { 0x8dc297f2, 3 },
    { 0x912697f1, 3 },
    { 0xd4697f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x268297fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x965e97fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afa9805, 3 },
    { 0, 0 },
    { 0x8069807, 4 },
    { 0xb1ba9808, 4 },
    { 0, 0 },
    { 0x93ee980a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115e980e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa69812, 4 },
    { 0, 0 },
    { 0xf09e9814, 3 },
    { 0xa1f29814, 3 },
    { 0x106a9814, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0829819, 4 },
    { 0xa9ca9819, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d2981d, 3 },
    { 0xdfb6981e, 4 },
    { 0xb056981e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f69823, 3 },
    { 0x18769823, 3 },
    { 0, 0 },
    { 0x1c8a9826, 3 },
    { 0, 0 },
    { 0x7ede9828, 3 },
    { 0x2a429829, 3 },
    { 0x35de982a, 3 },
    { 0x2e529828, 4 },
    { 0x1a6982c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a982f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab69833, 3 },
    { 0, 0 },
    { 0xbf4a9835, 3 },
    { 0xcb029836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe476983a, 4 },
    { 0x60b6983b, 4 },
    { 0, 0 },
    { 0x9282983d, 4 },
    { 0x34da983d, 3 },
    { 0, 0 },
    { 0x6fd69840, 4 },
    { 0x6629840, 3 },
    { 0xd07e9841, 3 },
    { 0, 0 },
    { 0x101e9844, 4 },
    { 0, 0 },
    { 0x24ba9846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e984a, 3 },
    { 0, 0 },
    { 0x7f2e984c, 3 },
    { 0xac5a984d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43369851, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7869857, 3 },
    { 0xe2ae9858, 3 },
    { 0xa8769858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1e985c, 3 },
    { 0, 0 },
    { 0xf07e985e, 3 },
    { 0x5d3a985f, 3 },
    { 0xc09a9860, 3 },
    { 0xf61a9861, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde986a, 3 },
    { 0, 0 },
    { 0xa2aa986c, 3 },
    { 0, 0 },
    { 0x246a986e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d29874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb966987d, 4 },
    { 0, 0 },
    { 0xd6ae987f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce988b, 3 },
    { 0xc5c2988b, 3 },
    { 0x156988c, 3 },
    { 0x7cbe988e, 4 },
    { 0x9066988f, 3 },
    { 0x5ae2988f, 4 },
    { 0x1922988f, 4 },
    { 0, 0 },
    { 0x1a069893, 4 },
    { 0x984e9894, 4 },
    { 0xcf469895, 3 },
    { 0x5e4e9894, 3 },
    { 0, 0 },
    { 0x8c369898, 4 },
    { 0x96aa9898, 4 },
    { 0xd126989a, 3 },
    { 0xbfe9898, 4 },
    { 0, 0 },
    { 0xc162989d, 3 },
    { 0, 0 },
    { 0x73b6989f, 3 },
    { 0, 0 },
    { 0xf9c298a1, 3 },
    { 0x7c6a98a2, 3 },
    { 0xce7e98a2, 3 },
    { 0, 0 },
    { 0x54be98a5, 4 },
    { 0xac6698a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e98ac, 3 },
    { 0x62ca98ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792298b2, 3 },
    { 0xf1a298b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af298b6, 3 },
    { 0xa35698b6, 3 },
    { 0x76e98b8, 5 },
    { 0x60f298b9, 3 },
    { 0x986698b9, 3 },
    { 0xd6ba98b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0698be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x793298c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11698c4, 3 },
    { 0x9d9698c5, 4 },
    { 0, 0 },
    { 0x7da698c7, 4 },
    { 0xd23298c8, 4 },
    { 0, 0 },
    { 0x872a98ca, 3 },
    { 0x16f298ca, 4 },
    { 0, 0 },
    { 0xd4ea98cd, 3 },
    { 0x981298cd, 3 },
    { 0xc47298ce, 3 },
    { 0x96e98d0, 3 },
    { 0xf6a698ce, 3 },
    { 0x43ae98d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a98d6, 3 },
    { 0, 0 },
    { 0x5cf298d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e298db, 3 },
    { 0xe45a98db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ba98e6, 3 },
    { 0x69a98e6, 3 },
    { 0, 0 },
    { 0xd9da98e9, 3 },
    { 0x5afe98e9, 4 },
    { 0xeaf698eb, 3 },
    { 0x9ee298ec, 3 },
    { 0x4d1e98ec, 3 },
    { 0x73de98eb, 3 },
    { 0xf83698ec, 3 },
    { 0x12a98f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d298f5, 3 },
    { 0xd3c298f6, 4 },
    { 0, 0 },
    { 0x36a98f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a98fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aae9902, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9896990a, 4 },
    { 0x819e990b, 3 },
    { 0x1caa990c, 3 },
    { 0x545e990d, 3 },
    { 0x6702990c, 4 },
    { 0x2346990b, 4 },
    { 0x4e429910, 3 },
    { 0, 0 },
    { 0x3ce29912, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a9916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f729919, 4 },
    { 0xbbde9919, 4 },
    { 0xb416991a, 3 },
    { 0x153e991c, 3 },
    { 0x6f12991c, 4 },
    { 0x4aa991e, 4 },
    { 0xc522991e, 3 },
    { 0x150a9920, 3 },
    { 0x4172991c, 3 },
    { 0x5522991e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe992b, 4 },
    { 0, 0 },
    { 0x7ab2992d, 3 },
    { 0x314e992d, 4 },
    { 0x1266992f, 3 },
    { 0x8076992e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa9937, 4 },
    { 0, 0 },
    { 0x3f9a9939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e9940, 3 },
    { 0, 0 },
    { 0x63f69942, 4 },
    { 0x85f69943, 3 },
    { 0x52529943, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b6994b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf29952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4369958, 3 },
    { 0xedb29958, 3 },
    { 0x87269958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc2995d, 3 },
    { 0xc1a6995e, 3 },
    { 0x64ce995f, 5 },
    { 0xad8e995f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e969963, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad69966, 3 },
    { 0x57169966, 5 },
    { 0, 0 },
    { 0xc4be9969, 3 },
    { 0xcc969969, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b6996f, 4 },
    { 0xf9c29970, 5 },
    { 0xf7929971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41a9977, 3 },
    { 0xf1e9978, 3 },
    { 0x86ce9977, 4 },
    { 0, 0 },
    { 0x7746997b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ee9981, 3 },
    { 0x10b69982, 3 },
    { 0x5d1a9983, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcde998b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd69997, 4 },
    { 0x113e9998, 3 },
    { 0xce0e9998, 4 },
    { 0xad4a999a, 3 },
    { 0x47ea999b, 3 },
    { 0xdb12999c, 4 },
    { 0x8a52999d, 5 },
    { 0, 0 },
    { 0x61e6999f, 3 },
    { 0x28c2999f, 3 },
    { 0x5aa699a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aa99a6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840699aa, 4 },
    { 0x69a299ab, 3 },
    { 0x71699ac, 3 },
    { 0x628a99ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a99b0, 4 },
    { 0, 0 },
    { 0x24ee99b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164299b9, 3 },
    { 0, 0 },
    { 0xe04299bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x563a99be, 3 },
    { 0, 0 },
    { 0xf88699c0, 4 },
    { 0x3afe99c1, 4 },
    { 0x5be299c1, 3 },
    { 0x22ee99c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x160a99c6, 4 },
    { 0, 0 },
    { 0x441699c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05a99d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4a99d5, 3 },
    { 0x90ba99d5, 3 },
    { 0xe9ea99d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28699db, 4 },
    { 0, 0 },
    { 0x291299dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c299e1, 3 },
    { 0xd1ca99e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe699e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x370299ec, 3 },
    { 0xaa1699ed, 3 },
    { 0x8da299ed, 4 },
    { 0x604699ef, 4 },
    { 0xefde99ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3e99f7, 4 },
    { 0xf85699f7, 3 },
    { 0, 0 },
    { 0x2b5699fa, 3 },
    { 0x2bf299fa, 4 },
    { 0xe63e99fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269a05, 3 },
    { 0x878a9a06, 3 },
    { 0xd6329a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec29a0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38e9a13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d229a16, 3 },
    { 0, 0 },
    { 0xb8aa9a18, 3 },
    { 0x58ae9a18, 3 },
    { 0, 0 },
    { 0xa60a9a1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ba9a1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86e9a25, 3 },
    { 0, 0 },
    { 0xe9ba9a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396e9a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54729a32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16da9a38, 3 },
    { 0x341a9a39, 5 },
    { 0xfff29a3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea9a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a9a44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e969a49, 3 },
    { 0xe51a9a4a, 3 },
    { 0xbd9e9a4b, 4 },
    { 0, 0 },
    { 0x1129a4d, 3 },
    { 0xdc529a4e, 3 },
    { 0xf8329a4e, 3 },
    { 0xc4b29a50, 3 },
    { 0x69169a51, 3 },
    { 0xa6d29a52, 3 },
    { 0x6f2a9a52, 3 },
    { 0xa3969a54, 3 },
    { 0x21429a54, 3 },
    { 0xa8e9a54, 4 },
    { 0, 0 },
    { 0x7a369a58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe29a5b, 5 },
    { 0, 0 },
    { 0xdcea9a5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327e9a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfe9a65, 3 },
    { 0x24029a66, 3 },
    { 0x79ba9a67, 3 },
    { 0, 0 },
    { 0x9b4a9a69, 3 },
    { 0x597e9a69, 4 },
    { 0x9b569a6b, 3 },
    { 0x38c69a6c, 3 },
    { 0, 0 },
    { 0x32a29a6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a29a77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf369a7a, 3 },
    { 0x7b729a7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f29a83, 4 },
    { 0, 0 },
    { 0x9cc69a85, 4 },
    { 0, 0 },
    { 0x72b69a87, 3 },
    { 0x84169a88, 3 },
    { 0, 0 },
    { 0xfda69a8a, 3 },
    { 0, 0 },
    { 0x1a7e9a8c, 3 },
    { 0x28be9a8c, 3 },
    { 0x5b269a8e, 3 },
    { 0x6b669a8f, 3 },
    { 0, 0 },
    { 0x2bfe9a91, 3 },
    { 0x418e9a92, 3 },
    { 0xa4029a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4029a96, 3 },
    { 0x40f69a97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fe9a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c29a9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b69aa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef69aaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89a9ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a029ab7, 3 },
    { 0x52729ab8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41369abe, 3 },
    { 0x5f0a9abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f829ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f929aca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1329ad1, 3 },
    { 0, 0 },
    { 0xaf469ad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499e9adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7529adf, 4 },
    { 0x8eca9ae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19769ae5, 3 },
    { 0, 0 },
    { 0x765a9ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce9aea, 3 },
    { 0xc4ca9aeb, 3 },
    { 0x2c129aeb, 3 },
    { 0x5e629aed, 3 },
    { 0x60329aec, 3 },
    { 0xd6d69aef, 3 },
    { 0, 0 },
    { 0x4ea9af1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d69af6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa29af9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x682a9afc, 3 },
    { 0xd0da9afc, 3 },
    { 0, 0 },
    { 0xf2829aff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79e9b05, 5 },
    { 0x12ce9b05, 3 },
    { 0xe7169b05, 3 },
    { 0x9bea9b06, 3 },
    { 0, 0 },
    { 0x2d1a9b0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f329b0d, 3 },
    { 0, 0 },
    { 0x3d069b0f, 3 },
    { 0xddba9b0f, 4 },
    { 0x211e9b11, 3 },
    { 0x1469b0f, 3 },
    { 0, 0 },
    { 0xec2a9b14, 3 },
    { 0x26de9b14, 3 },
    { 0, 0 },
    { 0xa4369b17, 3 },
    { 0, 0 },
    { 0x7b829b19, 3 },
    { 0x7eba9b1a, 4 },
    { 0x91129b1b, 3 },
    { 0x5df29b1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ace9b20, 5 },
    { 0, 0 },
    { 0x87129b22, 4 },
    { 0, 0 },
    { 0x853a9b24, 3 },
    { 0xa5ca9b24, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13069b2c, 4 },
    { 0x338e9b2d, 4 },
    { 0xb67a9b2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81369b32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a29b36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ee9b3a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9629b3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56e9b42, 3 },
    { 0x80f29b42, 3 },
    { 0, 0 },
    { 0x6cfe9b45, 3 },
    { 0, 0 },
    { 0x39c69b47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1769b4a, 3 },
    { 0, 0 },
    { 0xdabe9b4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x423e9b4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa729b52, 3 },
    { 0xa1869b53, 4 },
    { 0x7e529b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9a9b58, 3 },
    { 0x8f4a9b59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f29b5c, 4 },
    { 0xe4ba9b5c, 3 },
    { 0x1d469b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcea9b62, 4 },
    { 0x2e929b63, 4 },
    { 0xd91e9b63, 3 },
    { 0x249e9b65, 3 },
    { 0xd1d29b65, 4 },
    { 0xbb4e9b67, 3 },
    { 0x3a369b68, 3 },
    { 0x26fa9b69, 3 },
    { 0x42d69b6a, 4 },
    { 0x25529b68, 3 },
    { 0xee4e9b63, 3 },
    { 0xb6429b69, 4 },
    { 0x20ae9b6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e29b74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae9b78, 3 },
    { 0, 0 },
    { 0x8da9b7a, 3 },
    { 0x4f1e9b7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26229b80, 3 },
    { 0x5ed29b81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32469b89, 3 },
    { 0xb8769b8a, 3 },
    { 0xb8769b8b, 3 },
    { 0x523a9b8c, 3 },
    { 0x1f969b89, 3 },
    { 0x95169b8c, 4 },
    { 0x4f229b8f, 4 },
    { 0, 0 },
    { 0xa4be9b91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e569b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d69b99, 3 },
    { 0x8faa9b9a, 4 },
    { 0x75ba9b9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627e9b9f, 3 },
    { 0x28e9b9f, 3 },
    { 0x7fba9ba1, 3 },
    { 0x15969ba2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe9ba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f69bb0, 3 },
    { 0, 0 },
    { 0x6d069bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98229bb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2e9bc2, 3 },
    { 0xaa869bc3, 3 },
    { 0x602a9bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a29bc7, 3 },
    { 0, 0 },
    { 0x6ca29bc9, 4 },
    { 0x5d669bca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e29bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6729be1, 3 },
    { 0xa2069be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f29be7, 3 },
    { 0, 0 },
    { 0x67369be9, 3 },
    { 0xc1ee9bea, 3 },
    { 0x8c629beb, 3 },
    { 0, 0 },
    { 0x82729bed, 3 },
    { 0x30fe9bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94629bf2, 5 },
    { 0x6629bf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b469bf8, 3 },
    { 0xf7ee9bf9, 4 },
    { 0xf52a9bf8, 3 },
    { 0, 0 },
    { 0xc6ba9bfc, 4 },
    { 0x975a9bfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fda9c00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea929c04, 3 },
    { 0xdad69c05, 3 },
    { 0x49d69c06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a9c14, 4 },
    { 0x231e9c14, 3 },
    { 0xf9369c15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ea9c1f, 3 },
    { 0x15da9c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ee9c23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ae9c2a, 3 },
    { 0xc3e29c2b, 4 },
    { 0xeaa9c2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd29c2f, 3 },
    { 0, 0 },
    { 0x22029c31, 3 },
    { 0x7b7a9c31, 3 },
    { 0, 0 },
    { 0xf7d29c34, 3 },
    { 0x57769c35, 3 },
    { 0, 0 },
    { 0x26f69c37, 3 },
    { 0xcf2a9c38, 3 },
    { 0, 0 },
    { 0xc1be9c3a, 4 },
    { 0xa02a9c3b, 3 },
    { 0, 0 },
    { 0x5e029c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408e9c41, 3 },
    { 0xf84e9c42, 3 },
    { 0x70c29c43, 3 },
    { 0x96669c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfa9c4f, 3 },
    { 0xf3da9c50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8769c56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc229c60, 4 },
    { 0, 0 },
    { 0x2e529c62, 3 },
    { 0x3d169c62, 3 },
    { 0x23a29c62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e269c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e9c75, 3 },
    { 0, 0 },
    { 0x360e9c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee929c7b, 3 },
    { 0x7f229c7c, 3 },
    { 0xa9129c7d, 3 },
    { 0x406e9c7d, 4 },
    { 0xc1d29c7d, 3 },
    { 0xdfde9c80, 3 },
    { 0, 0 },
    { 0xfde29c82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a69c8a, 3 },
    { 0xd00a9c8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e9c8e, 3 },
    { 0xecbe9c8f, 3 },
    { 0xf3fa9c8e, 3 },
    { 0, 0 },
    { 0xb42e9c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69e9c95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf69c98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a9ca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6029ca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x911a9ca7, 3 },
    { 0xca669ca8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x780a9cab, 3 },
    { 0x935a9cab, 3 },
    { 0, 0 },
    { 0x42ba9cae, 3 },
    { 0x4f3e9cae, 4 },
    { 0xe0929cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9729cb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40869cb6, 4 },
    { 0x13869cb6, 4 },
    { 0x4bf69cb7, 4 },
    { 0x224e9cb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709e9cbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e9cc0, 3 },
    { 0xf4fe9cc1, 3 },
    { 0x2aea9cc2, 3 },
    { 0x99c29cc3, 3 },
    { 0xd1669cc4, 3 },
    { 0x24429cc5, 3 },
    { 0, 0 },
    { 0xec529cc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1869ccc, 3 },
    { 0x163a9ccd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4469cd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ae9cd4, 3 },
    { 0x3bee9cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87a9cdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418e9ce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fe9ceb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12529cef, 3 },
    { 0x531a9cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46a9cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d29cfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4029cff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83e9d04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a229d0e, 3 },
    { 0x51429d0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x736a9d12, 3 },
    { 0x7e569d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e9d21, 4 },
    { 0xe66e9d21, 3 },
    { 0x2eba9d23, 3 },
    { 0xaaf69d24, 3 },
    { 0xdd269d25, 3 },
    { 0xce7e9d26, 3 },
    { 0x98b29d24, 3 },
    { 0x38669d27, 5 },
    { 0, 0 },
    { 0x34ea9d2a, 3 },
    { 0, 0 },
    { 0xae3e9d2c, 3 },
    { 0xabfe9d2d, 3 },
    { 0x2bfe9d2e, 3 },
    { 0, 0 },
    { 0xe4029d30, 4 },
    { 0, 0 },
    { 0xc3a69d32, 3 },
    { 0, 0 },
    { 0x2d629d34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e429d38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e29d3e, 4 },
    { 0xb7529d3f, 3 },
    { 0xd0969d40, 3 },
    { 0x96e9d41, 3 },
    { 0x67e9d42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e9d45, 3 },
    { 0xb3ca9d46, 4 },
    { 0x67c29d47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7e9d4c, 4 },
    { 0x8d69d4c, 3 },
    { 0x9cd69d4d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55169d54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4a9d59, 4 },
    { 0xaa429d5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d369d5e, 3 },
    { 0xff929d5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9d62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab29d65, 3 },
    { 0x264a9d66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145e9d71, 3 },
    { 0x684a9d71, 3 },
    { 0, 0 },
    { 0x5ce9d74, 3 },
    { 0x74f69d74, 3 },
    { 0xcef69d76, 3 },
    { 0xce569d75, 3 },
    { 0xcf9e9d78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x88829d7b, 5 },
    { 0xf2d29d7c, 4 },
    { 0x88829d7c, 3 },
    { 0x13f29d7e, 3 },
    { 0xa60a9d7f, 4 },
    { 0x61829d80, 3 },
    { 0xc40e9d80, 4 },
    { 0x71369d81, 3 },
    { 0x5c3a9d83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62e9d89, 3 },
    { 0, 0 },
    { 0xa0fe9d8b, 3 },
    { 0, 0 },
    { 0xc9c69d8d, 3 },
    { 0x84e69d8e, 3 },
    { 0, 0 },
    { 0xddc69d90, 3 },
    { 0x91ca9d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a9d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90de9d99, 3 },
    { 0x73829d9a, 3 },
    { 0xc59e9d9b, 3 },
    { 0x615e9d9a, 4 },
    { 0x8bf29d99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7a9da0, 3 },
    { 0x54a69da1, 3 },
    { 0x5a4a9da2, 3 },
    { 0x1ff29da2, 4 },
    { 0x836e9da0, 3 },
    { 0x16e9da5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca9dad, 3 },
    { 0x74a69dad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d269db2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96769db5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b29dbc, 3 },
    { 0x4daa9dbc, 3 },
    { 0x6b969dbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e29dc3, 3 },
    { 0x4dca9dc4, 3 },
    { 0x17929dc5, 3 },
    { 0xa5be9dc5, 5 },
    { 0xb62e9dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a29dd1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2a9dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47a9dd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86229ddc, 3 },
    { 0x24f69ddd, 5 },
    { 0xf429dde, 3 },
    { 0xf1269ddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40da9de2, 3 },
    { 0x9d7a9de2, 3 },
    { 0x4c0a9de3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe9de7, 5 },
    { 0, 0 },
    { 0x2e629de9, 3 },
    { 0, 0 },
    { 0x3f4a9deb, 3 },
    { 0x46aa9dec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c69df1, 3 },
    { 0xa5469df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf29df7, 3 },
    { 0x1dee9df7, 3 },
    { 0xeb029df9, 3 },
    { 0x582e9df7, 3 },
    { 0xa7729dfb, 3 },
    { 0, 0 },
    { 0xbdd69dfd, 3 },
    { 0xf10a9dfe, 3 },
    { 0x88369dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a9e03, 3 },
    { 0x79869e04, 3 },
    { 0x40929e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b69e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51e9e0c, 3 },
    { 0, 0 },
    { 0x37f69e0e, 3 },
    { 0xa5a69e0f, 3 },
    { 0x2e429e0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f329e16, 4 },
    { 0, 0 },
    { 0x9eba9e18, 4 },
    { 0xd2ea9e18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8a9e1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470e9e28, 3 },
    { 0, 0 },
    { 0xff4e9e2a, 3 },
    { 0x692e9e2b, 3 },
    { 0, 0 },
    { 0x291e9e2d, 3 },
    { 0, 0 },
    { 0x42f29e2f, 5 },
    { 0, 0 },
    { 0x676a9e31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa9e34, 3 },
    { 0, 0 },
    { 0x9eba9e36, 4 },
    { 0, 0 },
    { 0x37da9e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580a9e3c, 3 },
    { 0x59c29e3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda9e42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e9e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x657e9e49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efe9e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9e53, 3 },
    { 0xa569e53, 3 },
    { 0x22f69e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324e9e59, 3 },
    { 0x73669e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71769e6b, 3 },
    { 0x93da9e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a69e6f, 5 },
    { 0, 0 },
    { 0x153e9e71, 3 },
    { 0xa64e9e71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa9e75, 3 },
    { 0x3ff69e76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90be9e79, 3 },
    { 0xfd3e9e7a, 4 },
    { 0x6d69e7b, 3 },
    { 0x1c6a9e7a, 3 },
    { 0x5fe29e7d, 3 },
    { 0xc8069e7e, 3 },
    { 0x27ca9e7f, 3 },
    { 0xc2ce9e80, 3 },
    { 0, 0 },
    { 0xc79a9e82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e869e8a, 3 },
    { 0x61329e8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3569e90, 4 },
    { 0x3aca9e91, 4 },
    { 0, 0 },
    { 0x863a9e93, 3 },
    { 0x285e9e94, 3 },
    { 0x36aa9e95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2e9e9a, 3 },
    { 0, 0 },
    { 0xbfea9e9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62e9ea0, 3 },
    { 0xdec29ea1, 4 },
    { 0x8f6e9ea2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af69ea9, 3 },
    { 0x4b69eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c29eb1, 3 },
    { 0xd34a9eb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6369eb8, 3 },
    { 0xed829eb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70329ebd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb69ec2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dee9ec7, 3 },
    { 0xf00e9ec8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc69ecb, 3 },
    { 0x8cba9ecc, 3 },
    { 0xb5669ecd, 3 },
    { 0x25329ecb, 5 },
    { 0xb2969ecf, 3 },
    { 0, 0 },
    { 0xb82a9ed1, 4 },
    { 0xe81e9ed1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13e9ed5, 3 },
    { 0x808e9ed6, 3 },
    { 0, 0 },
    { 0xb7e9ed8, 4 },
    { 0xe2ae9ed9, 3 },
    { 0xf8329ed9, 4 },
    { 0x339e9edb, 3 },
    { 0xc2929edb, 3 },
    { 0x14be9ed9, 3 },
    { 0xa6869edc, 3 },
    { 0x72129edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0929ee3, 5 },
    { 0xa02a9ee4, 3 },
    { 0xc68a9ee5, 3 },
    { 0x1a969ee5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba9ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee29eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61a9ef1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e9ef5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4a9ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2029efc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a9f0c, 3 },
    { 0, 0 },
    { 0x8d929f0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e69f17, 3 },
    { 0, 0 },
    { 0x854a9f19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b69f20, 4 },
    { 0xb7729f21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e9f26, 3 },
    { 0xe81a9f27, 3 },
    { 0, 0 },
    { 0x25329f29, 3 },
    { 0, 0 },
    { 0xa5f29f2b, 3 },
    { 0x945a9f2c, 4 },
    { 0xd1ae9f2d, 4 },
    { 0xf14e9f2e, 3 },
    { 0, 0 },
    { 0xbe929f30, 3 },
    { 0x59929f31, 3 },
    { 0x64069f32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274a9f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9129f40, 4 },
    { 0x4cea9f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39669f47, 4 },
    { 0, 0 },
    { 0xf6a69f49, 4 },
    { 0x334e9f4a, 4 },
    { 0x4be29f4a, 3 },
    { 0xfc269f4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a9f4f, 3 },
    { 0, 0 },
    { 0xa7529f51, 3 },
    { 0, 0 },
    { 0xd9d29f53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74769f57, 3 },
    { 0x285e9f58, 3 },
    { 0, 0 },
    { 0xf57a9f5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd769f65, 3 },
    { 0x3fe29f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40229f69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586a9f6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b829f70, 4 },
    { 0, 0 },
    { 0xce8e9f72, 4 },
    { 0x3e8e9f73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5a9f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4429f7d, 4 },
    { 0x97b69f7e, 4 },
    { 0, 0 },
    { 0xc96a9f80, 4 },
    { 0xa69e9f81, 3 },
    { 0xc5c69f81, 3 },
    { 0x5cba9f83, 3 },
    { 0x91c29f84, 3 },
    { 0, 0 },
    { 0x71de9f86, 3 },
    { 0x916e9f87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1e9f8f, 3 },
    { 0, 0 },
    { 0x252e9f91, 3 },
    { 0x75c69f92, 3 },
    { 0x43d29f93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d69f97, 3 },
    { 0, 0 },
    { 0x90d29f99, 3 },
    { 0xf10a9f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37469f9f, 4 },
    { 0, 0 },
    { 0x3c929fa1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe729fa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa9fa8, 3 },
    { 0, 0 },
    { 0x4ca29faa, 3 },
    { 0x412a9faa, 3 },
    { 0x29769fac, 4 },
    { 0, 0 },
    { 0xbe129fae, 3 },
    { 0x935e9fae, 4 },
    { 0x7df29fb0, 3 },
    { 0xe3f69fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81e9fb5, 4 },
    { 0x272a9fb5, 3 },
    { 0x76869fb6, 3 },
    { 0x71ae9fb8, 3 },
    { 0xeaa69fb9, 3 },
    { 0, 0 },
    { 0x5aae9fbb, 4 },
    { 0xd9a29fbc, 3 },
    { 0, 0 },
    { 0x4f2e9fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542a9fc5, 4 },
    { 0, 0 },
    { 0xb54e9fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba29fcc, 4 },
    { 0x449e9fcd, 3 },
    { 0x15169fce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a9fd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aa9fd9, 4 },
    { 0x5bce9fd9, 3 },
    { 0x976a9fd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a9fde, 4 },
    { 0, 0 },
    { 0xbb4a9fe0, 4 },
    { 0, 0 },
    { 0x92a9fe2, 4 },
    { 0x12e69fe3, 3 },
    { 0x245a9fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6829fee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55a9ff2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89de9ff6, 3 },
    { 0, 0 },
    { 0xd49e9ff8, 3 },
    { 0, 0 },
    { 0x10aa9ffa, 4 },
    { 0x82d69ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e629ffe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6a001, 3 },
    { 0x406ea002, 3 },
    { 0, 0 },
    { 0x2feea004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aea010, 3 },
    { 0xe96a011, 3 },
    { 0x20aea012, 3 },
    { 0, 0 },
    { 0xb2aea014, 4 },
    { 0xde7aa014, 3 },
    { 0x4016a016, 3 },
    { 0x3a8ea017, 3 },
    { 0xd55aa018, 3 },
    { 0x4de2a018, 3 },
    { 0x39eaa016, 3 },
    { 0x3af6a01b, 4 },
    { 0x5c32a01c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aa025, 3 },
    { 0xee52a025, 3 },
    { 0x72aa027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae6a02a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fea02d, 4 },
    { 0x9106a02e, 3 },
    { 0xebcea02e, 3 },
    { 0xef9aa030, 4 },
    { 0, 0 },
    { 0x1ad6a032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e36a037, 3 },
    { 0xb152a038, 3 },
    { 0xf8dea038, 3 },
    { 0xbd26a03a, 4 },
    { 0, 0 },
    { 0x6edaa03c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a6a044, 4 },
    { 0, 0 },
    { 0x762ea046, 4 },
    { 0, 0 },
    { 0x529ea048, 3 },
    { 0x56f2a048, 3 },
    { 0x28eaa04a, 3 },
    { 0x1966a04b, 3 },
    { 0, 0 },
    { 0xd69ea04d, 3 },
    { 0xfa56a04e, 3 },
    { 0xe76a04f, 3 },
    { 0xdf9ea04e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3aa054, 3 },
    { 0x7202a055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefaa05a, 3 },
    { 0xcc6ea05b, 3 },
    { 0xc226a05c, 3 },
    { 0xbc86a05a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdaa061, 3 },
    { 0x1aa2a062, 3 },
    { 0x4ce2a063, 3 },
    { 0xb602a062, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726a070, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9632a074, 4 },
    { 0x4d2aa074, 3 },
    { 0xd9f2a076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adaa07a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a086, 3 },
    { 0, 0 },
    { 0x782ea088, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x679aa08b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6aa08e, 3 },
    { 0xebbea08f, 3 },
    { 0, 0 },
    { 0x710ea091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a6a094, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6696a098, 3 },
    { 0x8496a099, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51ea09d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa402a0a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86a0a5, 3 },
    { 0xf30ea0a6, 4 },
    { 0x172ea0a5, 3 },
    { 0x2fa6a0a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6392a0af, 3 },
    { 0, 0 },
    { 0x1182a0b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1716a0b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfaa0bd, 3 },
    { 0xd426a0be, 3 },
    { 0x1356a0bf, 3 },
    { 0xd2eaa0bd, 3 },
    { 0, 0 },
    { 0x58caa0c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516a0c8, 3 },
    { 0x9a0aa0c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04aa0cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ea0d3, 3 },
    { 0xf16ea0d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77aa0d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453aa0de, 4 },
    { 0, 0 },
    { 0xd3baa0e0, 3 },
    { 0, 0 },
    { 0x8b96a0e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c2a0e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd622a0e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439ea0f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88faa0f9, 4 },
    { 0xa57aa0fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c6a0fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ea100, 3 },
    { 0xf49ea100, 4 },
    { 0x796a102, 3 },
    { 0xeebaa100, 3 },
    { 0x30b6a103, 3 },
    { 0, 0 },
    { 0x621ea106, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a66a10a, 3 },
    { 0x9d62a10b, 4 },
    { 0x96aea10a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa632a110, 3 },
    { 0x95a2a111, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb32a115, 3 },
    { 0xb1bea115, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e2a11e, 3 },
    { 0x49aa11f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70ea127, 3 },
    { 0xea52a128, 3 },
    { 0xc906a128, 3 },
    { 0x70b6a12a, 3 },
    { 0x8c42a128, 3 },
    { 0xacb2a127, 3 },
    { 0x983ea12d, 3 },
    { 0x3602a12e, 3 },
    { 0xaf5ea12f, 4 },
    { 0xd6baa130, 3 },
    { 0xa25ea130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196a136, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad36a13d, 3 },
    { 0x7966a13e, 3 },
    { 0xd692a13f, 3 },
    { 0x4c26a13f, 3 },
    { 0x190ea141, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7aa144, 3 },
    { 0xea86a145, 4 },
    { 0, 0 },
    { 0x1f8aa147, 3 },
    { 0, 0 },
    { 0xcdd2a149, 3 },
    { 0, 0 },
    { 0x5b2a14b, 3 },
    { 0, 0 },
    { 0xfcd6a14d, 4 },
    { 0x67d2a14e, 3 },
    { 0, 0 },
    { 0xf0d2a150, 3 },
    { 0x8be2a150, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267aa155, 3 },
    { 0xe0d2a156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc946a15b, 3 },
    { 0x311aa15b, 3 },
    { 0, 0 },
    { 0xdcdaa15e, 3 },
    { 0xe1e6a15f, 4 },
    { 0, 0 },
    { 0x7e9aa161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26a165, 4 },
    { 0xd66a166, 3 },
    { 0, 0 },
    { 0x88aa168, 4 },
    { 0x4686a169, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ea16c, 3 },
    { 0x4976a16d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14ea171, 3 },
    { 0, 0 },
    { 0xa7a6a173, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1196a180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d16a183, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ea186, 3 },
    { 0x7946a186, 3 },
    { 0xfbdea188, 5 },
    { 0xdb0ea189, 3 },
    { 0x36aea188, 4 },
    { 0, 0 },
    { 0x786aa18c, 3 },
    { 0x4e62a18c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e2a190, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b92a196, 3 },
    { 0x3036a196, 3 },
    { 0, 0 },
    { 0xa1e6a199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456a19d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2a1a2, 3 },
    { 0xd57aa1a3, 3 },
    { 0x3ce2a1a4, 3 },
    { 0x42b2a1a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a2a1a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677aa1ae, 4 },
    { 0x99a2a1af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc462a1b2, 3 },
    { 0x60f6a1b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aaa1b6, 3 },
    { 0, 0 },
    { 0xef1ea1b8, 3 },
    { 0x2ceea1b9, 4 },
    { 0x282aa1b9, 3 },
    { 0xfe16a1bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792aa1c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aa1c6, 3 },
    { 0x9076a1c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e6a1cd, 4 },
    { 0xcaaaa1ce, 3 },
    { 0x83a2a1ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5336a1d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb596a1d7, 3 },
    { 0, 0 },
    { 0xe042a1d9, 3 },
    { 0, 0 },
    { 0x1582a1db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x820ea1de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88ea1e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ea1e6, 3 },
    { 0xbb96a1e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead6a1ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e6a1f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9886a1fa, 3 },
    { 0xb782a1fa, 3 },
    { 0x934aa1fb, 3 },
    { 0x1ac2a1fd, 3 },
    { 0xb69aa1fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3ea208, 3 },
    { 0x7832a209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x981ea20c, 3 },
    { 0x70eaa20d, 3 },
    { 0xf7d2a20e, 3 },
    { 0xbdb2a20e, 3 },
    { 0x3d32a210, 3 },
    { 0, 0 },
    { 0xab8aa212, 3 },
    { 0, 0 },
    { 0x40d6a214, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294aa219, 3 },
    { 0xa98aa21a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fea223, 3 },
    { 0xc17aa224, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c32a229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74aa22c, 3 },
    { 0xae52a22d, 3 },
    { 0x36daa22e, 3 },
    { 0, 0 },
    { 0xe7a2a230, 3 },
    { 0x160ea231, 4 },
    { 0, 0 },
    { 0x17caa233, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbea23d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532aa241, 4 },
    { 0, 0 },
    { 0xe63aa243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef86a248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc532a24b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x187aa24e, 3 },
    { 0, 0 },
    { 0x4baaa250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ea256, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2922a25a, 3 },
    { 0, 0 },
    { 0xe83ea25c, 3 },
    { 0x73f6a25d, 3 },
    { 0x11faa25c, 3 },
    { 0, 0 },
    { 0xfb36a260, 4 },
    { 0, 0 },
    { 0x73dea262, 4 },
    { 0xd496a262, 3 },
    { 0xeaa264, 3 },
    { 0xf5aaa265, 3 },
    { 0, 0 },
    { 0x5e36a267, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198aa26c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5baa270, 3 },
    { 0xe736a271, 3 },
    { 0x7882a272, 4 },
    { 0xf06aa273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627aa27b, 3 },
    { 0x355ea27c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c6a27f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc826a282, 4 },
    { 0x232ea283, 4 },
    { 0xa4eea284, 3 },
    { 0x4c0aa285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x838aa288, 4 },
    { 0x4a5ea289, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x390ea28c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eaa296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6aa299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dea29e, 4 },
    { 0xcaf6a29e, 4 },
    { 0, 0 },
    { 0x528ea2a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5aa2a4, 3 },
    { 0x2b6ea2a5, 3 },
    { 0xc75aa2a5, 4 },
    { 0xd086a2a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4472a2aa, 3 },
    { 0, 0 },
    { 0x5632a2ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33daa2af, 3 },
    { 0, 0 },
    { 0x1eb2a2b1, 3 },
    { 0, 0 },
    { 0x6aaea2b3, 3 },
    { 0, 0 },
    { 0x222aa2b5, 4 },
    { 0, 0 },
    { 0x8f6aa2b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f6a2c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6aa2c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cea2cf, 3 },
    { 0xc496a2d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5ea2d3, 3 },
    { 0x1a16a2d3, 3 },
    { 0xfb2a2d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe66a2d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb42a2de, 3 },
    { 0x9e2ea2df, 3 },
    { 0, 0 },
    { 0x9bf6a2e1, 4 },
    { 0xfc0ea2e2, 3 },
    { 0, 0 },
    { 0xcc8aa2e4, 3 },
    { 0x2842a2e5, 3 },
    { 0x7816a2e6, 3 },
    { 0x5a8aa2e7, 3 },
    { 0x87bea2e8, 3 },
    { 0x2426a2e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304ea2ed, 3 },
    { 0, 0 },
    { 0x7a7aa2ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x463aa2f2, 3 },
    { 0x2942a2f2, 3 },
    { 0, 0 },
    { 0x1c7aa2f5, 3 },
    { 0, 0 },
    { 0x11ea2f7, 3 },
    { 0x732aa2f8, 3 },
    { 0x6f72a2f9, 3 },
    { 0xa7dea2fa, 4 },
    { 0xc7bea2fa, 4 },
    { 0xd85ea2fc, 3 },
    { 0xa222a2f9, 3 },
    { 0x7206a2f9, 4 },
    { 0xd612a2f8, 3 },
    { 0xb856a2f8, 3 },
    { 0xf592a301, 3 },
    { 0x4052a302, 4 },
    { 0x5f06a302, 3 },
    { 0x688aa302, 3 },
    { 0x51faa305, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe022a308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6a312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7aa315, 3 },
    { 0, 0 },
    { 0xcab6a317, 3 },
    { 0, 0 },
    { 0xfd6aa319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792aa31d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4446a323, 3 },
    { 0x52f2a324, 3 },
    { 0, 0 },
    { 0xbb1aa326, 3 },
    { 0x6066a326, 3 },
    { 0xa602a328, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d6a32d, 4 },
    { 0, 0 },
    { 0x6846a32f, 3 },
    { 0x78d6a330, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aa333, 4 },
    { 0, 0 },
    { 0xf816a335, 3 },
    { 0, 0 },
    { 0x560ea337, 3 },
    { 0xcea338, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ea341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16aa345, 3 },
    { 0x416aa345, 3 },
    { 0xc13aa346, 3 },
    { 0x67faa345, 3 },
    { 0x3962a349, 4 },
    { 0x1182a349, 4 },
    { 0x2b5ea34b, 3 },
    { 0xa6d2a348, 3 },
    { 0x32d2a346, 3 },
    { 0xa4a2a34d, 3 },
    { 0, 0 },
    { 0x3c22a350, 3 },
    { 0, 0 },
    { 0xcdbaa352, 4 },
    { 0xb2e2a352, 4 },
    { 0, 0 },
    { 0x716ea355, 3 },
    { 0x88e2a355, 3 },
    { 0x8ae2a355, 4 },
    { 0xf70aa356, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29caa35c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082a362, 4 },
    { 0xce1ea362, 3 },
    { 0, 0 },
    { 0xc516a365, 3 },
    { 0xf112a366, 3 },
    { 0xbf6a367, 3 },
    { 0x16baa366, 3 },
    { 0xdd02a368, 3 },
    { 0, 0 },
    { 0x58baa36b, 3 },
    { 0, 0 },
    { 0x4e56a36d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25ea371, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daaa376, 4 },
    { 0xf9dea377, 3 },
    { 0x6a7aa376, 3 },
    { 0x9112a378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c6a380, 5 },
    { 0xcedaa381, 3 },
    { 0, 0 },
    { 0x3e36a383, 3 },
    { 0xafe6a384, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2a388, 3 },
    { 0xdc0ea389, 3 },
    { 0, 0 },
    { 0xec0ea38b, 3 },
    { 0, 0 },
    { 0x5d32a38d, 4 },
    { 0x6b36a38e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c2a392, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64eaa396, 3 },
    { 0xa3d2a396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefaa39b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6952a39f, 4 },
    { 0xd6eea3a0, 4 },
    { 0xd8aaa3a1, 3 },
    { 0xb716a39f, 3 },
    { 0x55bea3a3, 4 },
    { 0xcd4aa3a3, 3 },
    { 0x18caa3a5, 3 },
    { 0x168aa3a6, 3 },
    { 0, 0 },
    { 0x287ea3a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9ea3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8ea3b1, 3 },
    { 0, 0 },
    { 0x3492a3b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613aa3b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d6a3c3, 4 },
    { 0, 0 },
    { 0xe4bea3c5, 3 },
    { 0xad82a3c6, 3 },
    { 0x666ea3c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acaa3ce, 4 },
    { 0xb642a3cf, 3 },
    { 0x174aa3ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8782a3d7, 3 },
    { 0x642ea3d7, 3 },
    { 0, 0 },
    { 0x4b9ea3da, 4 },
    { 0xabc6a3db, 4 },
    { 0x5d56a3db, 3 },
    { 0, 0 },
    { 0xecc2a3de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f2a3e2, 3 },
    { 0, 0 },
    { 0xc60aa3e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f6a3e9, 3 },
    { 0, 0 },
    { 0x2696a3eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaa3ef, 4 },
    { 0xf766a3ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12ea3f9, 3 },
    { 0x5f8ea3f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e2a400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ea403, 4 },
    { 0, 0 },
    { 0xfbeea405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943aa40c, 4 },
    { 0x2fd2a40d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c56a410, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb6a414, 3 },
    { 0xb72aa415, 3 },
    { 0xd76a415, 3 },
    { 0, 0 },
    { 0xfe9aa418, 3 },
    { 0xfa7ea418, 3 },
    { 0, 0 },
    { 0xa76a41b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84daa41f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd892a42c, 4 },
    { 0x9aa6a42d, 4 },
    { 0x4456a42d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac96a431, 4 },
    { 0, 0 },
    { 0xae9aa433, 3 },
    { 0, 0 },
    { 0xacd2a435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ea443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686a44b, 3 },
    { 0x811ea44c, 3 },
    { 0x5fd2a44d, 3 },
    { 0xad6a44d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9122a451, 3 },
    { 0, 0 },
    { 0xcfbea453, 3 },
    { 0x829aa454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e06a45d, 3 },
    { 0, 0 },
    { 0xb21aa45f, 3 },
    { 0xada2a45f, 5 },
    { 0x373aa461, 3 },
    { 0x7ccea461, 3 },
    { 0x29ea463, 3 },
    { 0x1f9aa45f, 3 },
    { 0xaa56a465, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aaa468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x618ea46b, 3 },
    { 0x3aaea46b, 3 },
    { 0x5d5aa46c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb776a473, 3 },
    { 0x97caa474, 3 },
    { 0x778aa474, 3 },
    { 0x6d8ea476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2a47a, 4 },
    { 0xbab6a47a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636ea47f, 3 },
    { 0xbecea480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874aa484, 4 },
    { 0x5c7ea485, 5 },
    { 0x4feaa485, 3 },
    { 0xf956a486, 3 },
    { 0xa81aa488, 3 },
    { 0xe6aa489, 3 },
    { 0x88cea488, 3 },
    { 0xfceea487, 4 },
    { 0, 0 },
    { 0x5ee6a48d, 3 },
    { 0x8042a48e, 4 },
    { 0xc1faa48f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3642a492, 3 },
    { 0x8352a493, 4 },
    { 0x2452a494, 3 },
    { 0, 0 },
    { 0x8e66a496, 4 },
    { 0x2a36a496, 4 },
    { 0x1116a498, 3 },
    { 0x792a499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf266a49c, 4 },
    { 0x8d62a49c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feea4a3, 3 },
    { 0, 0 },
    { 0xfc4aa4a5, 3 },
    { 0x5822a4a6, 3 },
    { 0, 0 },
    { 0xe62ea4a8, 3 },
    { 0x84b2a4a8, 3 },
    { 0xc6b6a4aa, 3 },
    { 0xa9baa4ab, 3 },
    { 0xd24aa4a9, 4 },
    { 0x55caa4ac, 3 },
    { 0x9486a4ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360aa4b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4aa4b5, 3 },
    { 0xaab6a4b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a36a4bc, 4 },
    { 0xc2fea4bc, 3 },
    { 0x8a56a4be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22a4c1, 3 },
    { 0x3a96a4c2, 3 },
    { 0x3046a4c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9aa4ca, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ea4cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1ea4d7, 3 },
    { 0, 0 },
    { 0x7e7ea4d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d2a4df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98aa4e3, 3 },
    { 0, 0 },
    { 0x216ea4e5, 3 },
    { 0xe202a4e6, 3 },
    { 0, 0 },
    { 0x8346a4e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ea4ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ea4f3, 3 },
    { 0x5a6aa4f3, 3 },
    { 0, 0 },
    { 0x5506a4f6, 3 },
    { 0xf0baa4f7, 3 },
    { 0xc832a4f8, 3 },
    { 0x5186a4f9, 4 },
    { 0x2b02a4f7, 3 },
    { 0x3d56a4fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6ea500, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e2a505, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eaa508, 3 },
    { 0x92caa509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cea50d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d42a512, 3 },
    { 0, 0 },
    { 0xc24aa514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ea518, 3 },
    { 0xf116a519, 3 },
    { 0x7602a518, 3 },
    { 0, 0 },
    { 0xc6cea51c, 3 },
    { 0x38d6a51c, 4 },
    { 0x61eea51d, 3 },
    { 0, 0 },
    { 0x19bea520, 3 },
    { 0xdb9ea521, 3 },
    { 0x4e0ea522, 3 },
    { 0x85c2a521, 3 },
    { 0x7c76a524, 4 },
    { 0xe206a520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13caa529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5936a52d, 3 },
    { 0xb2a6a52e, 3 },
    { 0x1cc6a52f, 3 },
    { 0x2386a530, 3 },
    { 0x10fea52f, 3 },
    { 0x8296a52e, 4 },
    { 0x479aa52f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x382aa536, 3 },
    { 0x7dfea536, 3 },
    { 0x87e6a538, 3 },
    { 0, 0 },
    { 0xa62a53a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cea540, 3 },
    { 0x47aea541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea66a544, 5 },
    { 0x80faa545, 4 },
    { 0, 0 },
    { 0xc29ea547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4aa54a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08aa54e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46aa552, 3 },
    { 0x59f6a553, 5 },
    { 0x5c2a554, 4 },
    { 0, 0 },
    { 0x96f2a556, 4 },
    { 0xbbd6a557, 3 },
    { 0, 0 },
    { 0x5caaa559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0ea55c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7292a55f, 3 },
    { 0, 0 },
    { 0x4176a561, 4 },
    { 0xfe2a562, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c6a565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aea569, 3 },
    { 0x3bb2a56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e2a56f, 3 },
    { 0x3c4ea570, 3 },
    { 0xe06a571, 3 },
    { 0x770ea570, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0aa57e, 4 },
    { 0, 0 },
    { 0x1432a580, 3 },
    { 0x2bfaa581, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32a58c, 4 },
    { 0x193aa58c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a6a599, 3 },
    { 0, 0 },
    { 0xb5e2a59b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd6a59f, 3 },
    { 0, 0 },
    { 0x3b6ea5a1, 4 },
    { 0, 0 },
    { 0x8beaa5a3, 3 },
    { 0x898ea5a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aaa5a7, 4 },
    { 0xa1fea5a7, 3 },
    { 0, 0 },
    { 0x39b2a5aa, 4 },
    { 0x509ea5ab, 4 },
    { 0, 0 },
    { 0x594aa5ad, 4 },
    { 0x2636a5ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b6a5b2, 3 },
    { 0xba42a5b3, 4 },
    { 0x6972a5b4, 3 },
    { 0, 0 },
    { 0x5d5ea5b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29aa5bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e46a5be, 4 },
    { 0xd166a5bf, 3 },
    { 0, 0 },
    { 0xe812a5c1, 4 },
    { 0, 0 },
    { 0xa946a5c3, 4 },
    { 0xf9faa5c3, 3 },
    { 0, 0 },
    { 0xe66a5c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcea5c9, 3 },
    { 0x29e6a5c9, 3 },
    { 0x4962a5cb, 3 },
    { 0xa93aa5cc, 3 },
    { 0x2d2a5ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83aea5de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ea5e3, 3 },
    { 0x7696a5e4, 3 },
    { 0x247ea5e5, 3 },
    { 0xce0ea5e5, 3 },
    { 0, 0 },
    { 0xe29ea5e8, 3 },
    { 0x2f82a5e9, 4 },
    { 0x7296a5ea, 5 },
    { 0xfe86a5eb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1836a5ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6aa5f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acea5f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ea5fa, 3 },
    { 0x8722a5fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45aa601, 3 },
    { 0x82c6a602, 3 },
    { 0xcb02a603, 4 },
    { 0x3786a604, 3 },
    { 0x6b5ea604, 4 },
    { 0x914ea602, 3 },
    { 0xdb8aa604, 3 },
    { 0, 0 },
    { 0x86fea609, 4 },
    { 0xe68ea60a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dea611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb242a61b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88aa625, 3 },
    { 0xfe1ea626, 3 },
    { 0x2fe2a627, 3 },
    { 0xe6aea627, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa596a62c, 4 },
    { 0xb4c6a62d, 4 },
    { 0x1b06a62c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa472a631, 3 },
    { 0x196a631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4ea636, 4 },
    { 0xe8a6a637, 4 },
    { 0x9bc6a637, 3 },
    { 0x5f42a639, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb722a640, 3 },
    { 0xb88aa640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb2a644, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x758aa647, 3 },
    { 0, 0 },
    { 0xca1ea649, 3 },
    { 0xa42ea649, 3 },
    { 0xe94aa649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf2a64e, 3 },
    { 0x6f22a64f, 3 },
    { 0x984aa650, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ea658, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc72a65b, 3 },
    { 0xb3fea65c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a665, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ea669, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeaa66c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324ea672, 3 },
    { 0x53eaa672, 3 },
    { 0, 0 },
    { 0x5e06a675, 4 },
    { 0x3eeaa676, 3 },
    { 0, 0 },
    { 0xb276a678, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2a67b, 3 },
    { 0x9596a67c, 3 },
    { 0xdfc2a67c, 3 },
    { 0x839aa67e, 4 },
    { 0x7792a67f, 4 },
    { 0xe68aa67e, 3 },
    { 0x736a681, 3 },
    { 0, 0 },
    { 0x46e6a683, 3 },
    { 0x6052a684, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a22a689, 3 },
    { 0x18aaa68a, 5 },
    { 0x7e0ea689, 3 },
    { 0xfe66a689, 4 },
    { 0xef6a68d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb2a692, 3 },
    { 0x5582a693, 3 },
    { 0x234ea692, 3 },
    { 0xf366a694, 4 },
    { 0x3b56a693, 3 },
    { 0xf27aa697, 3 },
    { 0x15fea695, 4 },
    { 0xfe7ea699, 3 },
    { 0, 0 },
    { 0xe04aa69b, 3 },
    { 0x19fea69c, 3 },
    { 0xd3e2a69b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd6a6a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adaa6a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c2a6a6, 3 },
    { 0x517aa6a7, 3 },
    { 0x9aa6a6a7, 3 },
    { 0x34eea6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c06a6b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc2a6b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39daa6b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c82a6be, 3 },
    { 0x6952a6bf, 4 },
    { 0x1b06a6c0, 3 },
    { 0, 0 },
    { 0x85aa6c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52ea6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04ea6cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ea6d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x750aa6d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cea6db, 4 },
    { 0, 0 },
    { 0x159aa6dd, 3 },
    { 0x660ea6de, 4 },
    { 0x8822a6de, 4 },
    { 0x6a3ea6e0, 3 },
    { 0x1672a6e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb6a6ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf712a6ef, 3 },
    { 0x3a6aa6f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c6a6f3, 3 },
    { 0x97c6a6f4, 3 },
    { 0x61ea6f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe096a6f8, 3 },
    { 0x81faa6f8, 3 },
    { 0x333ea6f9, 3 },
    { 0, 0 },
    { 0x867aa6fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa6a6ff, 4 },
    { 0x1b72a6ff, 5 },
    { 0x266a701, 3 },
    { 0x33b6a701, 4 },
    { 0xa9daa6ff, 3 },
    { 0xd85ea704, 3 },
    { 0xf12aa704, 4 },
    { 0x8d1aa706, 4 },
    { 0x12bea707, 3 },
    { 0x84eea708, 3 },
    { 0x6bb2a707, 4 },
    { 0, 0 },
    { 0xda1ea70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d52a70e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430aa716, 4 },
    { 0x742aa717, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9246a71a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0baa71d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aea723, 3 },
    { 0x71cea723, 3 },
    { 0x759ea724, 3 },
    { 0x8ee2a725, 4 },
    { 0x37c2a725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473ea72b, 3 },
    { 0x2662a72b, 4 },
    { 0, 0 },
    { 0x7416a72e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc2a735, 3 },
    { 0x2faea735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a6a739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0aa73d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eea745, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106aa74c, 3 },
    { 0xb23aa74d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16a751, 3 },
    { 0x1a4aa752, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92a756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab2a760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9046a763, 3 },
    { 0xcfe6a764, 3 },
    { 0x8eaa764, 3 },
    { 0x5326a766, 3 },
    { 0xefeea765, 3 },
    { 0x3c12a767, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88aa76b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aaa76f, 3 },
    { 0, 0 },
    { 0x9e92a771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176aa778, 3 },
    { 0x6166a779, 3 },
    { 0x6552a778, 3 },
    { 0xca6aa77b, 3 },
    { 0x7d12a77c, 3 },
    { 0x6eeea77c, 3 },
    { 0, 0 },
    { 0x7bc6a77f, 3 },
    { 0xef96a77f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c2a784, 3 },
    { 0xec7aa784, 3 },
    { 0x4c7ea786, 3 },
    { 0xd0daa786, 4 },
    { 0xd356a788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f2a78b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47ea791, 3 },
    { 0, 0 },
    { 0x7deaa793, 3 },
    { 0, 0 },
    { 0x6e36a795, 3 },
    { 0, 0 },
    { 0xe52aa797, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb436a79b, 3 },
    { 0x89baa79c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc2a7a2, 3 },
    { 0x448aa7a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb406a7a7, 3 },
    { 0x9272a7a7, 3 },
    { 0, 0 },
    { 0x96eea7aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd486a7ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6aa7b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf6a7b8, 4 },
    { 0xbe26a7b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea7bd, 3 },
    { 0xb232a7be, 3 },
    { 0xf4d6a7bf, 3 },
    { 0x57d2a7be, 3 },
    { 0x849ea7bf, 3 },
    { 0, 0 },
    { 0x200aa7c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3256a7c7, 3 },
    { 0xded2a7c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbea7cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c26a7d0, 3 },
    { 0, 0 },
    { 0x5172a7d2, 3 },
    { 0x3fbaa7d3, 3 },
    { 0x1a7aa7d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1aa7d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3aa7e1, 3 },
    { 0, 0 },
    { 0xd4baa7e3, 4 },
    { 0, 0 },
    { 0xdea2a7e5, 4 },
    { 0x5aaaa7e5, 4 },
    { 0xee5ea7e7, 3 },
    { 0, 0 },
    { 0x22bea7e9, 3 },
    { 0x76faa7ea, 3 },
    { 0, 0 },
    { 0x3056a7ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff2a7f6, 3 },
    { 0xf10aa7f7, 3 },
    { 0x91faa7f6, 3 },
    { 0xa7e2a7f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe582a800, 3 },
    { 0xec4aa800, 3 },
    { 0xd5baa801, 3 },
    { 0xcfaa802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e6a806, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaea809, 4 },
    { 0xbd3aa809, 3 },
    { 0x7282a80b, 3 },
    { 0x734aa80c, 4 },
    { 0, 0 },
    { 0x29a2a80e, 4 },
    { 0, 0 },
    { 0xc7d2a810, 3 },
    { 0x2d62a810, 3 },
    { 0xd87ea812, 4 },
    { 0xd786a813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3512a816, 3 },
    { 0xdd02a816, 3 },
    { 0xce4aa818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edea81e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f2a821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096a826, 4 },
    { 0xdc4ea827, 3 },
    { 0x79f6a828, 3 },
    { 0x150ea826, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2aa832, 3 },
    { 0, 0 },
    { 0x9c7ea834, 3 },
    { 0x5536a835, 3 },
    { 0x84f2a836, 3 },
    { 0, 0 },
    { 0xd92a838, 3 },
    { 0x466a839, 3 },
    { 0xb68ea83a, 4 },
    { 0xdcf6a83a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae46a83e, 3 },
    { 0, 0 },
    { 0xc8aa840, 4 },
    { 0x9d56a841, 3 },
    { 0x1aaa840, 4 },
    { 0xf036a843, 4 },
    { 0, 0 },
    { 0x5432a845, 3 },
    { 0x426aa846, 4 },
    { 0x3d02a847, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x857ea84a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ea851, 3 },
    { 0x3306a852, 3 },
    { 0, 0 },
    { 0xc7eea854, 4 },
    { 0, 0 },
    { 0x629aa856, 5 },
    { 0xda42a857, 3 },
    { 0xff56a857, 4 },
    { 0xf16aa859, 3 },
    { 0x8336a859, 3 },
    { 0x1732a857, 3 },
    { 0, 0 },
    { 0x2a85d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e2a860, 3 },
    { 0xb87aa860, 3 },
    { 0, 0 },
    { 0xaaf6a863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aa866, 3 },
    { 0, 0 },
    { 0x4b56a868, 4 },
    { 0, 0 },
    { 0x63fea86a, 3 },
    { 0x9982a86b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3366a86e, 3 },
    { 0xcecaa86f, 4 },
    { 0x5d02a86f, 3 },
    { 0, 0 },
    { 0x2532a872, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8aa876, 3 },
    { 0xb1eaa877, 4 },
    { 0xd98ea878, 3 },
    { 0x941ea879, 3 },
    { 0, 0 },
    { 0xb6faa87b, 3 },
    { 0, 0 },
    { 0x3202a87d, 4 },
    { 0x3536a87e, 3 },
    { 0x97f2a87f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb772a884, 3 },
    { 0x11e6a885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18aa889, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d2a88e, 4 },
    { 0, 0 },
    { 0xb97ea890, 4 },
    { 0xf8bea891, 3 },
    { 0, 0 },
    { 0xcc3aa893, 3 },
    { 0x7a82a894, 3 },
    { 0x1966a894, 3 },
    { 0xa772a894, 3 },
    { 0, 0 },
    { 0xb09aa898, 3 },
    { 0, 0 },
    { 0xee5ea89a, 3 },
    { 0xa9e2a89b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e6a8a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6a8ac, 3 },
    { 0xe1e2a8ad, 3 },
    { 0, 0 },
    { 0x3aaa8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965ea8b7, 4 },
    { 0x7bcea8b7, 4 },
    { 0, 0 },
    { 0x8a6aa8ba, 3 },
    { 0x21f2a8ba, 4 },
    { 0, 0 },
    { 0x61a6a8bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fea8c1, 3 },
    { 0, 0 },
    { 0xfe02a8c3, 3 },
    { 0xe03ea8c4, 3 },
    { 0x342aa8c5, 3 },
    { 0xbcea8c6, 3 },
    { 0x8306a8c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c6a8ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e72a8ce, 3 },
    { 0x17eea8ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd872a8d4, 4 },
    { 0xe392a8d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7aa8da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ea8e3, 4 },
    { 0xc926a8e3, 3 },
    { 0x130aa8e5, 3 },
    { 0xaa8ea8e5, 3 },
    { 0x242ea8e7, 3 },
    { 0xfc12a8e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6a8ee, 3 },
    { 0xbfeea8ee, 3 },
    { 0x652a8ee, 3 },
    { 0x6542a8f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd682a8f7, 3 },
    { 0, 0 },
    { 0xf656a8f9, 4 },
    { 0x8d7ea8fa, 3 },
    { 0x344ea8fb, 4 },
    { 0x3cb6a8fc, 3 },
    { 0, 0 },
    { 0x1b3ea8fe, 3 },
    { 0x63ea8fe, 3 },
    { 0xb706a900, 3 },
    { 0x5916a901, 3 },
    { 0x77c6a8ff, 3 },
    { 0x59dea903, 4 },
    { 0x7dbaa904, 3 },
    { 0xea82a905, 3 },
    { 0xa716a906, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eea909, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe2a90e, 3 },
    { 0, 0 },
    { 0x9326a910, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d26a914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22a917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabea91d, 4 },
    { 0, 0 },
    { 0x776ea91f, 3 },
    { 0x2256a920, 4 },
    { 0xf21aa91f, 4 },
    { 0xda2aa922, 3 },
    { 0xdccaa922, 3 },
    { 0x3092a924, 3 },
    { 0x646aa925, 4 },
    { 0xe25ea926, 3 },
    { 0x3e9aa927, 3 },
    { 0xca82a928, 3 },
    { 0x20bea926, 3 },
    { 0xfd22a92a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42ea936, 3 },
    { 0x855ea937, 3 },
    { 0xd452a938, 3 },
    { 0, 0 },
    { 0xfd0aa93a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ea93d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5aa941, 4 },
    { 0xe262a942, 3 },
    { 0xffdaa943, 4 },
    { 0xdc06a943, 4 },
    { 0, 0 },
    { 0xe89aa946, 4 },
    { 0x19a6a947, 4 },
    { 0xfeb6a948, 3 },
    { 0x2e36a949, 3 },
    { 0x52b6a948, 3 },
    { 0xbaaea94b, 3 },
    { 0xd542a94c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f6a94f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c76a952, 3 },
    { 0, 0 },
    { 0x7de2a954, 3 },
    { 0xb1b6a955, 3 },
    { 0x15a6a954, 3 },
    { 0x72f2a955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eea95b, 3 },
    { 0x24eea95b, 3 },
    { 0xe712a95c, 4 },
    { 0x79f2a95e, 4 },
    { 0, 0 },
    { 0xad6aa960, 3 },
    { 0, 0 },
    { 0xa1d2a962, 3 },
    { 0x19eea962, 4 },
    { 0x34c6a963, 3 },
    { 0x1cd6a965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9546a968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef2a96f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51ea975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d2a979, 3 },
    { 0, 0 },
    { 0xb0d6a97b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaa97e, 3 },
    { 0x5682a97f, 4 },
    { 0x7446a980, 3 },
    { 0x8416a981, 3 },
    { 0xd6c6a982, 3 },
    { 0x40e6a983, 4 },
    { 0x31aa97f, 3 },
    { 0, 0 },
    { 0x3b76a986, 3 },
    { 0x6132a987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0aea98a, 4 },
    { 0x1b92a98b, 3 },
    { 0, 0 },
    { 0xc406a98d, 3 },
    { 0xb526a98e, 3 },
    { 0, 0 },
    { 0x31eaa990, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2a99b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1aa99e, 3 },
    { 0, 0 },
    { 0xf966a9a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516a9a3, 4 },
    { 0, 0 },
    { 0x9a1aa9a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853aa9ad, 5 },
    { 0xb22aa9ae, 4 },
    { 0x9cf6a9ad, 3 },
    { 0, 0 },
    { 0xc2e6a9b1, 3 },
    { 0x4e2aa9b2, 3 },
    { 0x555aa9b1, 4 },
    { 0x1a96a9b4, 3 },
    { 0x4fd6a9b4, 4 },
    { 0x1c06a9b2, 3 },
    { 0xe5daa9b7, 3 },
    { 0xe63aa9b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf6a9be, 3 },
    { 0x564aa9be, 3 },
    { 0, 0 },
    { 0xf0dea9c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2a9c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f96a9c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c6a9cd, 3 },
    { 0x5046a9ce, 3 },
    { 0xdf9ea9cf, 3 },
    { 0xb786a9cf, 4 },
    { 0xcb5ea9cf, 3 },
    { 0x8332a9d1, 3 },
    { 0x5206a9d3, 3 },
    { 0xeac2a9d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aaa9dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ea9df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e6a9e2, 3 },
    { 0, 0 },
    { 0x9756a9e4, 3 },
    { 0x3f56a9e5, 3 },
    { 0xd2aa9e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64daa9f0, 3 },
    { 0x32faa9f1, 5 },
    { 0x7e72a9f2, 3 },
    { 0xfe92a9f2, 3 },
    { 0x39a6a9f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391aa9fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cceaa02, 4 },
    { 0x4dc2aa03, 3 },
    { 0x59beaa04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x974aaa07, 3 },
    { 0, 0 },
    { 0x98feaa09, 3 },
    { 0xb33aaa0a, 3 },
    { 0, 0 },
    { 0x9126aa0c, 3 },
    { 0, 0 },
    { 0x1852aa0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aaaa14, 4 },
    { 0xb78aaa15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9962aa18, 3 },
    { 0, 0 },
    { 0x45c2aa1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50aaa1e, 5 },
    { 0x433eaa1f, 3 },
    { 0x60aaa20, 3 },
    { 0xa5c6aa20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16aa28, 3 },
    { 0x70b6aa28, 3 },
    { 0xf9baaa2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6faaa2e, 3 },
    { 0xfdbaaa2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a2aa32, 3 },
    { 0xa91eaa33, 3 },
    { 0, 0 },
    { 0x251aaa35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec76aa38, 3 },
    { 0, 0 },
    { 0x62f6aa3a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x416aa3d, 3 },
    { 0x8c72aa3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x739aaa41, 3 },
    { 0xeaeaaa42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfaaa46, 3 },
    { 0x47eeaa47, 3 },
    { 0, 0 },
    { 0x8c0eaa49, 5 },
    { 0xba1eaa4a, 3 },
    { 0x2c1eaa4a, 3 },
    { 0x71eeaa4c, 3 },
    { 0x469eaa4d, 3 },
    { 0xccaaaa4e, 3 },
    { 0xa546aa4f, 3 },
    { 0x2ec6aa4c, 3 },
    { 0x341eaa51, 3 },
    { 0x6406aa50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6eaa56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc02aa5f, 3 },
    { 0xf192aa60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a6aa63, 3 },
    { 0x5512aa64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbeaa69, 3 },
    { 0x111aaa69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caaa6d, 3 },
    { 0x6ee6aa6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbaaa71, 3 },
    { 0, 0 },
    { 0xfd1aaa73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c6aa77, 4 },
    { 0x7a52aa77, 3 },
    { 0x6886aa79, 4 },
    { 0, 0 },
    { 0xebd2aa7b, 3 },
    { 0x863eaa7b, 4 },
    { 0xccc6aa7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06aaa81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b6aa89, 3 },
    { 0x1e72aa8a, 3 },
    { 0x8bbeaa8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3622aa93, 3 },
    { 0, 0 },
    { 0x8b9aaa95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee82aa98, 3 },
    { 0x8c72aa98, 3 },
    { 0x5116aa98, 3 },
    { 0x1192aa9a, 4 },
    { 0xaa32aa9c, 3 },
    { 0x1776aa9c, 3 },
    { 0x1996aa9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7aaaa5, 3 },
    { 0xe7a6aaa6, 3 },
    { 0x40caaaa7, 4 },
    { 0xba32aaa8, 3 },
    { 0x36daaaa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3116aaad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eaab1, 3 },
    { 0x22d2aab2, 3 },
    { 0, 0 },
    { 0xc3caaab4, 3 },
    { 0x52c2aab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a82aaba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312aaabe, 3 },
    { 0x6e7eaabf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f42aac2, 3 },
    { 0x79daaac3, 3 },
    { 0x561eaac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc06aac7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adaaaca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27aaacf, 3 },
    { 0xfceaaad0, 3 },
    { 0, 0 },
    { 0x927aaad2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaaad8, 3 },
    { 0x58eaaad9, 3 },
    { 0xfd7aaad9, 4 },
    { 0xa17aaad9, 4 },
    { 0, 0 },
    { 0xc9d2aadd, 5 },
    { 0, 0 },
    { 0xa7e6aadf, 3 },
    { 0, 0 },
    { 0x683aaae1, 3 },
    { 0xe92eaae2, 3 },
    { 0, 0 },
    { 0x5726aae4, 4 },
    { 0x8deaaae4, 3 },
    { 0, 0 },
    { 0xedc6aae7, 3 },
    { 0xb07aaae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3aaaee, 3 },
    { 0, 0 },
    { 0xdc6eaaf0, 3 },
    { 0x36caaaf1, 3 },
    { 0x1f1aaaf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd152aaf9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf252aafd, 3 },
    { 0, 0 },
    { 0x1b1aaaff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c2ab03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726ab07, 4 },
    { 0xe22ab08, 4 },
    { 0xa5c6ab07, 4 },
    { 0x17c2ab0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79aeab0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44eab10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab22ab14, 3 },
    { 0xb81aab14, 4 },
    { 0, 0 },
    { 0x7dc2ab17, 3 },
    { 0xe136ab17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1eab1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6002ab20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc6ab24, 3 },
    { 0x6f22ab25, 3 },
    { 0x4c1aab26, 3 },
    { 0, 0 },
    { 0x1f46ab28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aceab2b, 3 },
    { 0x6a32ab2c, 3 },
    { 0x3792ab2d, 3 },
    { 0, 0 },
    { 0xd9eab2f, 3 },
    { 0x7d9eab30, 3 },
    { 0xd51aab30, 4 },
    { 0xb9eab30, 3 },
    { 0xd5feab32, 4 },
    { 0x4f6ab34, 3 },
    { 0, 0 },
    { 0xba4eab36, 3 },
    { 0x1d2ab37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692ab3a, 3 },
    { 0x222aab3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ab3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x267aab42, 3 },
    { 0, 0 },
    { 0xc28aab44, 3 },
    { 0x2346ab45, 3 },
    { 0xb16aab45, 3 },
    { 0, 0 },
    { 0x737aab48, 3 },
    { 0x4ebeab48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a6ab4c, 3 },
    { 0, 0 },
    { 0xeaab4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beeab51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aab55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88deab59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f2ab5e, 3 },
    { 0x563eab5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6476ab62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586ab66, 3 },
    { 0xe74eab67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e2ab6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9eab6d, 3 },
    { 0x8276ab6e, 4 },
    { 0, 0 },
    { 0x230aab70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0eab76, 3 },
    { 0xa586ab76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdeab7b, 3 },
    { 0x20eaab7b, 4 },
    { 0, 0 },
    { 0x6a22ab7e, 3 },
    { 0, 0 },
    { 0xf466ab80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80aab84, 3 },
    { 0x32e2ab84, 4 },
    { 0, 0 },
    { 0xdde2ab87, 3 },
    { 0xd31eab87, 3 },
    { 0x1f26ab89, 3 },
    { 0x24eab8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f6ab8f, 3 },
    { 0x2beab90, 3 },
    { 0x153aab91, 3 },
    { 0xfca6ab90, 4 },
    { 0, 0 },
    { 0xc93aab94, 4 },
    { 0xcf1aab95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb296ab9a, 3 },
    { 0, 0 },
    { 0x66aab9c, 3 },
    { 0x749eab9d, 4 },
    { 0x2ff6ab9e, 3 },
    { 0x524aab9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232aaba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a42aba8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4146abab, 3 },
    { 0xd6beabab, 3 },
    { 0, 0 },
    { 0x28c2abae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e62abb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b96abb5, 3 },
    { 0, 0 },
    { 0x103aabb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa206abc3, 3 },
    { 0x2bdaabc3, 4 },
    { 0xeabaabc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679aabc9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4252abcd, 3 },
    { 0x66e2abce, 3 },
    { 0xde76abcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2abd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372abdc, 3 },
    { 0x154eabdd, 3 },
    { 0xa552abdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6abe1, 3 },
    { 0x4512abe2, 3 },
    { 0xdbdeabe3, 4 },
    { 0x810eabe4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf6abeb, 4 },
    { 0x57eeabec, 4 },
    { 0xeb56abed, 3 },
    { 0x50c6abeb, 3 },
    { 0, 0 },
    { 0xce0aabf0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5eabf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485aabf8, 3 },
    { 0x4872abf8, 3 },
    { 0, 0 },
    { 0xcc76abfb, 4 },
    { 0x302aabfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724eac06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdaac09, 3 },
    { 0x76b2ac0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e72ac0d, 3 },
    { 0x2ec2ac0e, 4 },
    { 0, 0 },
    { 0x3cdaac10, 3 },
    { 0, 0 },
    { 0x6d26ac12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1806ac1b, 4 },
    { 0, 0 },
    { 0xc996ac1d, 3 },
    { 0x8202ac1e, 4 },
    { 0x8e36ac1e, 3 },
    { 0xfeb6ac1e, 3 },
    { 0xa172ac21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeeac27, 3 },
    { 0, 0 },
    { 0x6b5eac29, 3 },
    { 0xa31aac2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a6ac30, 3 },
    { 0, 0 },
    { 0xeb0eac32, 4 },
    { 0xa0aac33, 3 },
    { 0xa352ac34, 3 },
    { 0x1b32ac35, 3 },
    { 0xab46ac36, 3 },
    { 0, 0 },
    { 0xc2f2ac38, 3 },
    { 0xb336ac39, 3 },
    { 0xb102ac3a, 3 },
    { 0xae9aac3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa372ac3f, 3 },
    { 0x3316ac40, 3 },
    { 0x175eac41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa536ac47, 4 },
    { 0x851aac47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c2ac4d, 3 },
    { 0xc382ac4e, 3 },
    { 0xfc2aac4f, 3 },
    { 0xbd86ac4f, 4 },
    { 0xf8e6ac50, 3 },
    { 0x1542ac50, 3 },
    { 0xa3f6ac53, 4 },
    { 0x33f2ac54, 3 },
    { 0x2b62ac54, 4 },
    { 0x3f9eac56, 3 },
    { 0x5562ac57, 3 },
    { 0x775aac54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464eac5d, 3 },
    { 0xd1d2ac5e, 3 },
    { 0xd142ac5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aeac70, 3 },
    { 0x64c2ac71, 3 },
    { 0xef76ac72, 3 },
    { 0xf68eac71, 3 },
    { 0, 0 },
    { 0x984aac75, 4 },
    { 0x19a2ac75, 3 },
    { 0x353eac77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e2ac7a, 4 },
    { 0x585aac7b, 3 },
    { 0x677eac7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0eac7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2ac86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9aac89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842ac8c, 3 },
    { 0x8326ac8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaac92, 3 },
    { 0x4fdaac93, 3 },
    { 0, 0 },
    { 0x4e2eac95, 3 },
    { 0, 0 },
    { 0xa12ac97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ac9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddaaca2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deaca6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x568aaca9, 3 },
    { 0, 0 },
    { 0x5aaaacab, 3 },
    { 0, 0 },
    { 0x8ee2acad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42caacb3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a16acb6, 4 },
    { 0xa81aacb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbceacbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6daacc4, 3 },
    { 0x2e32acc5, 3 },
    { 0xfdeaacc6, 3 },
    { 0x82acc5, 3 },
    { 0x1ddeacc8, 3 },
    { 0xaa36acc9, 3 },
    { 0x41e6acca, 3 },
    { 0x7f96accb, 5 },
    { 0xbd16acca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1eacd0, 3 },
    { 0xd8c2acd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a2acd4, 3 },
    { 0, 0 },
    { 0xb562acd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eeacd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3762acde, 3 },
    { 0xa382acde, 3 },
    { 0x8aeace0, 4 },
    { 0x8562ace0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aaace4, 3 },
    { 0x9c0eace5, 3 },
    { 0xa19eace6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffeaceb, 3 },
    { 0, 0 },
    { 0xe08eaced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4feacf0, 3 },
    { 0x3b6acf1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64eacfd, 3 },
    { 0xb17aacfe, 3 },
    { 0xb53eacff, 4 },
    { 0x5932ad00, 3 },
    { 0x5646acfe, 3 },
    { 0xd6b6ad02, 3 },
    { 0xb586ad02, 3 },
    { 0x2756ad04, 3 },
    { 0, 0 },
    { 0x4556ad06, 3 },
    { 0, 0 },
    { 0xe3c6ad08, 4 },
    { 0x1f2ead08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf16ad0c, 3 },
    { 0x8226ad0d, 4 },
    { 0x827aad0d, 3 },
    { 0x53e6ad0e, 3 },
    { 0xd4a6ad0f, 4 },
    { 0x8de2ad0f, 3 },
    { 0x24fead12, 3 },
    { 0xabb6ad11, 4 },
    { 0xa2e2ad14, 4 },
    { 0x5e32ad15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d42ad18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x469aad1b, 3 },
    { 0, 0 },
    { 0xd37ead1d, 3 },
    { 0, 0 },
    { 0xb39ead1f, 3 },
    { 0, 0 },
    { 0x78f2ad21, 3 },
    { 0, 0 },
    { 0xbd22ad23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7732ad26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1892ad2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7aad31, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bead38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x366ad3b, 3 },
    { 0x862ead3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db2ad3f, 5 },
    { 0x512ad40, 5 },
    { 0x8e8aad3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d82ad44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef66ad49, 3 },
    { 0x5ce2ad4a, 3 },
    { 0xd48aad4b, 4 },
    { 0xb16ead4c, 5 },
    { 0x4fbead49, 3 },
    { 0x554ead49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2642ad51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef6ad57, 4 },
    { 0xddaad57, 3 },
    { 0x7ad6ad59, 4 },
    { 0x4b52ad5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ead5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66aad61, 3 },
    { 0, 0 },
    { 0x3d86ad63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa926ad67, 3 },
    { 0x3baaad67, 3 },
    { 0xd06ad69, 3 },
    { 0x6b22ad67, 4 },
    { 0xa9ead68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x92daad6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fead71, 4 },
    { 0x2d86ad72, 3 },
    { 0, 0 },
    { 0xfd3aad74, 3 },
    { 0x3476ad74, 3 },
    { 0, 0 },
    { 0x1dcead77, 3 },
    { 0, 0 },
    { 0x59aad79, 4 },
    { 0xd57ead7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26ad7d, 3 },
    { 0x10faad7e, 3 },
    { 0, 0 },
    { 0x4b56ad80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba12ad84, 3 },
    { 0, 0 },
    { 0x656ead86, 3 },
    { 0xc25aad87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aead8c, 3 },
    { 0x1aeaad8d, 3 },
    { 0x4276ad8e, 4 },
    { 0xf5fead8e, 3 },
    { 0xd83aad90, 3 },
    { 0xf90aad8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac22ad94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3aad98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faad9b, 3 },
    { 0x6a6ead9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82ada5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1aada9, 3 },
    { 0x9f6eadaa, 3 },
    { 0, 0 },
    { 0x9956adac, 3 },
    { 0x10c6adad, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8daadb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b52adb6, 3 },
    { 0xf96eadb7, 3 },
    { 0, 0 },
    { 0x4642adb9, 3 },
    { 0x96eadba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a6adbe, 3 },
    { 0xaefaadbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b2adc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45baadc8, 4 },
    { 0x3a12adc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x662aadcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe266add3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eadd9, 5 },
    { 0x27e2adda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87eaddd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x239eade0, 3 },
    { 0x9286ade1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a2ade9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0eadf0, 3 },
    { 0x1ae2adf1, 4 },
    { 0, 0 },
    { 0xf602adf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f2adf6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7846adfa, 4 },
    { 0x7996adfa, 3 },
    { 0x55a6adfc, 4 },
    { 0x8f56adfc, 3 },
    { 0, 0 },
    { 0xc0aeadff, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04eae02, 3 },
    { 0x7562ae03, 4 },
    { 0xf052ae02, 3 },
    { 0xd3eae05, 4 },
    { 0x782aae06, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7676ae09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4aae0e, 3 },
    { 0x7a92ae0f, 3 },
    { 0x5f2eae0f, 3 },
    { 0xc25eae11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1776ae14, 3 },
    { 0xfcb6ae14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac2ae1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb612ae1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e2ae23, 3 },
    { 0xa526ae24, 4 },
    { 0x9892ae25, 3 },
    { 0x6f3aae25, 3 },
    { 0x36ae25, 3 },
    { 0, 0 },
    { 0x83c2ae29, 4 },
    { 0, 0 },
    { 0x49beae2b, 4 },
    { 0, 0 },
    { 0x206ae2d, 3 },
    { 0, 0 },
    { 0xc3aae2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1242ae32, 3 },
    { 0, 0 },
    { 0xb6e2ae34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdeae37, 4 },
    { 0x96eae37, 3 },
    { 0x64e2ae39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c22ae3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c6ae42, 4 },
    { 0x21b2ae43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff2ae4d, 3 },
    { 0x805eae4e, 3 },
    { 0, 0 },
    { 0xdaeaae50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf2ae57, 3 },
    { 0, 0 },
    { 0x4fd2ae59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72eae5e, 3 },
    { 0x7252ae5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc2ae63, 4 },
    { 0, 0 },
    { 0xc26aae65, 3 },
    { 0, 0 },
    { 0xb512ae67, 3 },
    { 0x16eeae68, 3 },
    { 0xd752ae69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1eae72, 3 },
    { 0x98e2ae72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ae78, 5 },
    { 0, 0 },
    { 0x9baae7a, 3 },
    { 0xfaa6ae7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ae80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a2ae83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x157eae86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc122ae89, 3 },
    { 0x312eae8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0caae8e, 3 },
    { 0, 0 },
    { 0x53c6ae90, 4 },
    { 0, 0 },
    { 0x89c6ae92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd532ae97, 3 },
    { 0xe6d6ae97, 3 },
    { 0x3d4aae99, 3 },
    { 0xe2caae99, 3 },
    { 0x2116ae97, 3 },
    { 0x393eae9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa2aea1, 3 },
    { 0x66baaea2, 3 },
    { 0xab96aea3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456aea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8372aead, 4 },
    { 0x745aaeae, 3 },
    { 0x6dc2aeaf, 3 },
    { 0x6a9eaeb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6eaeb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aaaeb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12aeba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afaaed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2aaed9, 5 },
    { 0x1ddaaed9, 3 },
    { 0, 0 },
    { 0x419eaedc, 3 },
    { 0, 0 },
    { 0xbb62aede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86aee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6aee7, 3 },
    { 0x24eaee8, 3 },
    { 0xbadeaee9, 3 },
    { 0x7deaaee7, 5 },
    { 0x85f2aeeb, 4 },
    { 0xcd96aee9, 3 },
    { 0x5f9eaeed, 3 },
    { 0xbadaaee7, 3 },
    { 0, 0 },
    { 0x6b5aaef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2aef4, 4 },
    { 0, 0 },
    { 0xa022aef6, 3 },
    { 0, 0 },
    { 0x87b6aef8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaeaefe, 4 },
    { 0x9812aeff, 3 },
    { 0x214eaf00, 3 },
    { 0x556af01, 5 },
    { 0xf5deaefe, 3 },
    { 0, 0 },
    { 0xbb2aaf04, 3 },
    { 0xdbf6af05, 3 },
    { 0, 0 },
    { 0xf532af07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46af0b, 3 },
    { 0xe9eaaf0c, 5 },
    { 0x5dceaf0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5deaf10, 4 },
    { 0x2f1aaf11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953aaf15, 3 },
    { 0x7bcaaf16, 3 },
    { 0, 0 },
    { 0x40ceaf18, 3 },
    { 0x3a3eaf19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e96af22, 4 },
    { 0xe606af23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453eaf27, 4 },
    { 0xdf76af27, 3 },
    { 0, 0 },
    { 0x9132af2a, 3 },
    { 0x8d6eaf2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0eaf36, 3 },
    { 0x8faeaf37, 3 },
    { 0xe172af37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aaf3e, 3 },
    { 0xade6af3f, 3 },
    { 0, 0 },
    { 0xd072af41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2186af44, 4 },
    { 0xff8eaf44, 3 },
    { 0x4966af46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2af49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6062af4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec2af52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622af55, 4 },
    { 0xa02aaf56, 3 },
    { 0xc82eaf57, 3 },
    { 0, 0 },
    { 0xdc32af59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc82af61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beaf65, 3 },
    { 0xd24eaf66, 3 },
    { 0xfcb6af66, 3 },
    { 0, 0 },
    { 0xf07eaf69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddaaf6c, 4 },
    { 0xf07eaf6c, 3 },
    { 0, 0 },
    { 0xa62af6f, 3 },
    { 0, 0 },
    { 0x188aaf71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37aaf77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22af7d, 3 },
    { 0x8cd2af7e, 3 },
    { 0xfeaf7f, 3 },
    { 0x3fdaaf80, 3 },
    { 0xa2aaf7d, 3 },
    { 0x8efaaf82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6af89, 3 },
    { 0x380aaf89, 4 },
    { 0x117eaf89, 3 },
    { 0xdcf2af89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185eaf90, 3 },
    { 0xa0deaf91, 3 },
    { 0x39f2af91, 4 },
    { 0x49aaaf93, 3 },
    { 0xf0d6af90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb446af97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa2af9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93deafa0, 3 },
    { 0x624aafa0, 3 },
    { 0x582eafa0, 3 },
    { 0x5d3aafa1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfaafa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9aafab, 3 },
    { 0x3e1aafab, 3 },
    { 0x4662afad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d2afb2, 5 },
    { 0xef0eafb2, 3 },
    { 0x83d2afb4, 5 },
    { 0, 0 },
    { 0x16e2afb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5eafba, 3 },
    { 0xf9ceafbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x128eafbe, 3 },
    { 0, 0 },
    { 0x84eaafc0, 4 },
    { 0, 0 },
    { 0x33aafc2, 3 },
    { 0xa412afc3, 3 },
    { 0, 0 },
    { 0x9ea6afc5, 3 },
    { 0, 0 },
    { 0xef12afc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101eafce, 5 },
    { 0x28aaafcf, 4 },
    { 0x5962afcf, 3 },
    { 0x42d2afce, 3 },
    { 0, 0 },
    { 0xa696afd3, 3 },
    { 0xa992afd3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89aafda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b2afdd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff12afe5, 3 },
    { 0x57b6afe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4872afea, 3 },
    { 0x5386afea, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eaafef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae2aff5, 3 },
    { 0xf93aaff6, 3 },
    { 0xfafeaff7, 3 },
    { 0, 0 },
    { 0x8c6eaff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056b002, 3 },
    { 0xd1ceb002, 4 },
    { 0x39aeb004, 3 },
    { 0x5e16b005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc12b008, 4 },
    { 0x14aab009, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa622b00d, 3 },
    { 0xb7bab00e, 3 },
    { 0x6046b00f, 3 },
    { 0xce06b00f, 4 },
    { 0x291ab00e, 3 },
    { 0xe622b012, 3 },
    { 0x4b2eb011, 3 },
    { 0xc2c6b00f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x274ab017, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b2b01e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21eb021, 3 },
    { 0, 0 },
    { 0x3e2ab023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed6b029, 3 },
    { 0xf096b029, 3 },
    { 0, 0 },
    { 0x383eb02c, 4 },
    { 0, 0 },
    { 0xe04eb02e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35ab031, 3 },
    { 0, 0 },
    { 0xd1eb033, 3 },
    { 0, 0 },
    { 0x9b0eb035, 4 },
    { 0x5c7ab036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7eb039, 3 },
    { 0x853ab03a, 5 },
    { 0x3aeeb03b, 3 },
    { 0x88aab03c, 3 },
    { 0xa6e2b03d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefab040, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeab043, 4 },
    { 0x8512b044, 3 },
    { 0x55b6b045, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf76b048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ceb04b, 3 },
    { 0x78ab04c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3506b04f, 3 },
    { 0x1b4ab050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a2b055, 3 },
    { 0x732eb056, 3 },
    { 0x3732b057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc32b05a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37eb05e, 3 },
    { 0x78feb05f, 4 },
    { 0, 0 },
    { 0x88d2b061, 3 },
    { 0x78d6b062, 3 },
    { 0xf416b063, 3 },
    { 0x7d9eb062, 3 },
    { 0xf122b065, 3 },
    { 0x5e86b066, 3 },
    { 0, 0 },
    { 0xfcfeb068, 3 },
    { 0xfcc2b068, 3 },
    { 0x363ab06a, 3 },
    { 0x7456b06b, 3 },
    { 0xadcab06b, 4 },
    { 0, 0 },
    { 0x5d46b06e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1eb073, 3 },
    { 0xfd96b074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bceb079, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6b07c, 3 },
    { 0x6a9eb07d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda6b080, 3 },
    { 0, 0 },
    { 0x3f6ab082, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbeb086, 5 },
    { 0xa68ab087, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9436b08a, 5 },
    { 0x2936b08a, 3 },
    { 0x4bab08b, 3 },
    { 0xd75eb08d, 3 },
    { 0x2762b08e, 3 },
    { 0, 0 },
    { 0xadeab090, 3 },
    { 0x1cb2b091, 3 },
    { 0x780eb091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294eb098, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026b0a7, 3 },
    { 0, 0 },
    { 0x91e2b0a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53feb0ac, 3 },
    { 0x6322b0ad, 3 },
    { 0xef52b0ae, 3 },
    { 0x16eeb0ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb6b0b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26beb0ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x206eb0bd, 4 },
    { 0xf63eb0be, 4 },
    { 0, 0 },
    { 0x8a62b0c0, 3 },
    { 0x597ab0c1, 3 },
    { 0x8012b0c2, 3 },
    { 0x842ab0c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc6b0c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5eb0cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1ab0d1, 3 },
    { 0x2592b0d2, 4 },
    { 0x129ab0d3, 4 },
    { 0, 0 },
    { 0x30b2b0d5, 3 },
    { 0, 0 },
    { 0x2056b0d7, 5 },
    { 0x450eb0d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aab0dd, 3 },
    { 0x9efab0de, 3 },
    { 0x5c1ab0de, 4 },
    { 0x82c6b0e0, 4 },
    { 0x1ee6b0e1, 3 },
    { 0xbedeb0df, 3 },
    { 0, 0 },
    { 0xe01ab0e4, 3 },
    { 0x92d6b0e5, 3 },
    { 0xbc4eb0e6, 3 },
    { 0x519eb0e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6ab0ea, 3 },
    { 0xff6b0eb, 3 },
    { 0x94eb0eb, 4 },
    { 0x1232b0eb, 3 },
    { 0x5c6b0ee, 3 },
    { 0xa28eb0ef, 3 },
    { 0xb57eb0ef, 3 },
    { 0x9c52b0ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a6b0f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1222b0fe, 3 },
    { 0x84b6b0fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca6b102, 3 },
    { 0x655eb103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798eb10a, 3 },
    { 0x380eb10b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea6b10e, 3 },
    { 0, 0 },
    { 0x4fe2b110, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b119, 3 },
    { 0xe732b119, 4 },
    { 0xcb86b11b, 3 },
    { 0xda72b11b, 3 },
    { 0xc26eb11d, 4 },
    { 0x4252b11e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2786b121, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc756b125, 3 },
    { 0, 0 },
    { 0x7106b127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eb12c, 4 },
    { 0, 0 },
    { 0x5312b12e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x145eb131, 3 },
    { 0, 0 },
    { 0xc892b133, 3 },
    { 0xbd96b133, 5 },
    { 0, 0 },
    { 0xa112b136, 3 },
    { 0xf312b136, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd2b13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f02b13e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fab142, 3 },
    { 0xfc32b142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f2b146, 4 },
    { 0x176ab147, 3 },
    { 0xc022b148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb366b14d, 4 },
    { 0xfadab14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa286b153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7842b159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ab15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b162, 3 },
    { 0xa5eb163, 3 },
    { 0, 0 },
    { 0x626ab165, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2256b168, 4 },
    { 0, 0 },
    { 0xb502b16a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7352b171, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc636b175, 3 },
    { 0x1362b176, 3 },
    { 0x79fab176, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1ab17a, 3 },
    { 0, 0 },
    { 0x2e82b17c, 3 },
    { 0, 0 },
    { 0x663ab17e, 3 },
    { 0x5956b17e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98eb183, 3 },
    { 0x9ef6b183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2b187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3156b18a, 3 },
    { 0x1876b18b, 3 },
    { 0xfbeb18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfab18f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5ab193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ab198, 4 },
    { 0x6bd6b199, 3 },
    { 0x3b22b19a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fab19e, 3 },
    { 0x1e3ab19f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50eb1a3, 3 },
    { 0xd512b1a4, 3 },
    { 0xe76b1a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122ab1aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bab1ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3eb1b5, 3 },
    { 0, 0 },
    { 0xd836b1b7, 4 },
    { 0xda36b1b8, 3 },
    { 0, 0 },
    { 0x15feb1ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1eb1be, 3 },
    { 0, 0 },
    { 0x50fab1c0, 3 },
    { 0x442eb1c1, 3 },
    { 0, 0 },
    { 0xd70eb1c3, 4 },
    { 0x84b6b1c4, 3 },
    { 0, 0 },
    { 0x934eb1c6, 4 },
    { 0xc35eb1c6, 3 },
    { 0xd5d6b1c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eeb1cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6ab1ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ab1d5, 3 },
    { 0, 0 },
    { 0xcf06b1d7, 5 },
    { 0x7862b1d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecab1e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13ab1e6, 3 },
    { 0x201ab1e7, 4 },
    { 0xceeb1e8, 4 },
    { 0x1d26b1e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0ab1f0, 4 },
    { 0, 0 },
    { 0xcddeb1f2, 3 },
    { 0, 0 },
    { 0xfdceb1f4, 4 },
    { 0x1586b1f4, 3 },
    { 0xdf0eb1f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3072b201, 3 },
    { 0xe6eab201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5686b206, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ab20a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb20d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d2b210, 4 },
    { 0x62ab211, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb502b214, 3 },
    { 0, 0 },
    { 0xaf0ab216, 3 },
    { 0x489eb216, 3 },
    { 0, 0 },
    { 0x5ca2b219, 3 },
    { 0x1cceb21a, 3 },
    { 0x212eb219, 4 },
    { 0xcde2b21b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeab220, 3 },
    { 0, 0 },
    { 0x8f9ab222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bab225, 4 },
    { 0x7ceab226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b86b229, 4 },
    { 0x164ab229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aeb22d, 3 },
    { 0, 0 },
    { 0xa82eb22f, 3 },
    { 0, 0 },
    { 0x18eab231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86beb238, 4 },
    { 0xa74ab239, 3 },
    { 0xda52b239, 3 },
    { 0x6bcab23a, 3 },
    { 0x624eb239, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xec96b23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6156b243, 5 },
    { 0xc7deb244, 3 },
    { 0x2762b245, 3 },
    { 0xb9dab245, 3 },
    { 0x78b6b247, 3 },
    { 0, 0 },
    { 0xeafeb249, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eab24c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba6b253, 4 },
    { 0x6092b253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1ab258, 3 },
    { 0xc056b259, 4 },
    { 0, 0 },
    { 0x53eeb25b, 4 },
    { 0x757ab25c, 3 },
    { 0, 0 },
    { 0x27aeb25e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe2b261, 4 },
    { 0x886eb261, 3 },
    { 0xedb6b262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eeb267, 5 },
    { 0, 0 },
    { 0x5e56b269, 3 },
    { 0x542eb26a, 3 },
    { 0xeb56b26b, 3 },
    { 0xa62eb26b, 3 },
    { 0xfe42b26b, 3 },
    { 0xfbab26e, 5 },
    { 0x428eb26d, 4 },
    { 0xe97ab270, 3 },
    { 0, 0 },
    { 0x579ab272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606b277, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f6b27b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3ab27e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a6b282, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x848ab285, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a6b288, 4 },
    { 0, 0 },
    { 0xcdfab28a, 3 },
    { 0x9286b28b, 3 },
    { 0, 0 },
    { 0x5a4ab28d, 3 },
    { 0x1c06b28e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e62b293, 3 },
    { 0, 0 },
    { 0x4046b295, 3 },
    { 0x850ab296, 3 },
    { 0x93ab297, 3 },
    { 0x37a2b298, 3 },
    { 0x8eeb297, 3 },
    { 0x5fb2b29a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b02b29d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edab2a5, 4 },
    { 0xa77ab2a6, 4 },
    { 0, 0 },
    { 0x626b2a8, 3 },
    { 0x7f9eb2a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646b2b3, 3 },
    { 0x9e3ab2b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaab2be, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2356b2c3, 3 },
    { 0xcceeb2c3, 4 },
    { 0x7f7eb2c5, 4 },
    { 0xc872b2c3, 3 },
    { 0, 0 },
    { 0xfd16b2c8, 3 },
    { 0xc192b2c8, 3 },
    { 0x849eb2c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf286b2d1, 4 },
    { 0, 0 },
    { 0xdb4eb2d3, 3 },
    { 0x4f6b2d4, 3 },
    { 0, 0 },
    { 0xba36b2d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a2b2db, 3 },
    { 0, 0 },
    { 0xb49eb2dd, 3 },
    { 0x7e0ab2de, 3 },
    { 0x8826b2de, 3 },
    { 0x6eceb2de, 3 },
    { 0xc9fab2e0, 3 },
    { 0, 0 },
    { 0xb272b2e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ceb2e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5882b2ef, 3 },
    { 0x7b02b2f0, 4 },
    { 0xeb96b2f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59ab2f4, 4 },
    { 0x27eab2f5, 4 },
    { 0x13eeb2f5, 5 },
    { 0, 0 },
    { 0xa942b2f8, 3 },
    { 0xb212b2f8, 3 },
    { 0xaca2b2fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af6b2fd, 4 },
    { 0x44aeb2fe, 3 },
    { 0x4b7ab2ff, 4 },
    { 0x38ab2fd, 3 },
    { 0x318eb301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8beeb308, 3 },
    { 0, 0 },
    { 0x73eeb30a, 3 },
    { 0xc566b30b, 4 },
    { 0xffaeb30c, 3 },
    { 0xa416b30d, 4 },
    { 0xd9f6b30e, 3 },
    { 0x36bab30f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7486b312, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc856b31a, 4 },
    { 0, 0 },
    { 0xa8e6b31c, 3 },
    { 0xe29ab31d, 4 },
    { 0xf782b31d, 3 },
    { 0, 0 },
    { 0x30aeb320, 3 },
    { 0xf682b321, 4 },
    { 0xa95ab320, 4 },
    { 0x3ee6b320, 3 },
    { 0xa79ab322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fab327, 4 },
    { 0, 0 },
    { 0x2e7eb329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fab32f, 3 },
    { 0xdd2ab32f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ab335, 3 },
    { 0, 0 },
    { 0x4daeb337, 3 },
    { 0xb066b337, 3 },
    { 0x91e2b337, 4 },
    { 0x1cc2b338, 3 },
    { 0xd3feb338, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784ab340, 3 },
    { 0x856ab340, 3 },
    { 0xce06b341, 3 },
    { 0x4a12b343, 3 },
    { 0x78aeb344, 3 },
    { 0xecb6b344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff26b348, 3 },
    { 0xb586b349, 4 },
    { 0x562b34a, 3 },
    { 0xcb7eb34a, 4 },
    { 0xdff2b34c, 3 },
    { 0x7e96b34d, 3 },
    { 0xeb3eb34a, 4 },
    { 0x86e2b34f, 4 },
    { 0x1e02b34f, 3 },
    { 0, 0 },
    { 0x59fab352, 3 },
    { 0xcaa2b353, 3 },
    { 0x68feb354, 4 },
    { 0x9ee2b355, 3 },
    { 0x16bab352, 3 },
    { 0x636ab357, 3 },
    { 0x34bab358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deab360, 4 },
    { 0xdbeeb361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x100ab364, 3 },
    { 0x9a0ab365, 3 },
    { 0x17e2b364, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f06b369, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3092b36d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ab372, 4 },
    { 0, 0 },
    { 0x9152b374, 4 },
    { 0, 0 },
    { 0xdd62b376, 3 },
    { 0x5b42b377, 3 },
    { 0xa66eb376, 3 },
    { 0xacf2b379, 3 },
    { 0x882eb37a, 4 },
    { 0x8e2ab37b, 4 },
    { 0xed32b37a, 3 },
    { 0x4eb2b37d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44eb382, 3 },
    { 0x13eeb383, 3 },
    { 0x8e9ab384, 3 },
    { 0xd82ab384, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffeb388, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d2b38c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82beb38f, 3 },
    { 0xff86b390, 3 },
    { 0x764eb38f, 4 },
    { 0xa692b38f, 3 },
    { 0x9bbab393, 3 },
    { 0x3382b391, 3 },
    { 0, 0 },
    { 0x9e3eb396, 4 },
    { 0x7322b397, 3 },
    { 0xb836b398, 3 },
    { 0xf62ab399, 3 },
    { 0, 0 },
    { 0xd5eeb39b, 4 },
    { 0x9ff2b39c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecab3a2, 3 },
    { 0x1d42b3a3, 4 },
    { 0, 0 },
    { 0xdd5eb3a5, 3 },
    { 0xf502b3a5, 3 },
    { 0x636eb3a7, 3 },
    { 0x97aab3a8, 3 },
    { 0, 0 },
    { 0x4906b3aa, 3 },
    { 0xea8ab3ab, 3 },
    { 0xcbab3ab, 4 },
    { 0xa37eb3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e72b3b2, 3 },
    { 0x9986b3b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b2b3b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6616b3be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8806b3c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ab3c7, 3 },
    { 0x3462b3c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4986b3cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x527ab3d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79ab3d6, 3 },
    { 0, 0 },
    { 0x12fab3d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5032b3df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86b3e7, 3 },
    { 0xd5e2b3e8, 3 },
    { 0xc7b6b3e9, 3 },
    { 0x1332b3ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422ab3ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceeb3f2, 3 },
    { 0x6ad6b3f2, 3 },
    { 0xff0eb3f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aab3f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b3fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a6b400, 3 },
    { 0, 0 },
    { 0x4b72b402, 4 },
    { 0xfe82b403, 3 },
    { 0x1d1ab402, 3 },
    { 0, 0 },
    { 0x6cf6b406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bab409, 4 },
    { 0x640eb409, 3 },
    { 0x93aeb40b, 3 },
    { 0xf2c6b40c, 4 },
    { 0x87eab40c, 3 },
    { 0, 0 },
    { 0x2306b40f, 4 },
    { 0x852b40f, 3 },
    { 0, 0 },
    { 0xde42b412, 3 },
    { 0xcc76b413, 4 },
    { 0x13a6b414, 3 },
    { 0x8b32b412, 4 },
    { 0x3dceb415, 3 },
    { 0, 0 },
    { 0xafaeb418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458eb41f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceab425, 3 },
    { 0, 0 },
    { 0x4612b427, 3 },
    { 0xc872b427, 3 },
    { 0x5d12b429, 4 },
    { 0x5aa6b42a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a6b432, 3 },
    { 0xa946b432, 3 },
    { 0x6226b434, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1ab437, 3 },
    { 0xc226b438, 4 },
    { 0, 0 },
    { 0x439eb43a, 4 },
    { 0xa82ab43b, 4 },
    { 0x482ab43a, 4 },
    { 0x8506b43d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6b440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b444, 3 },
    { 0, 0 },
    { 0xc8fab446, 3 },
    { 0x757ab446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9252b44e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb646b451, 4 },
    { 0x18e2b452, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96eeb456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d2b45b, 5 },
    { 0, 0 },
    { 0x3216b45d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaeb46c, 3 },
    { 0xf95eb46d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32aeb470, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eeb473, 3 },
    { 0xbc1ab474, 3 },
    { 0x178eb475, 4 },
    { 0xde46b476, 4 },
    { 0xdf4eb477, 3 },
    { 0xffe6b477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2132b47b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6b480, 3 },
    { 0, 0 },
    { 0x457eb482, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4282b485, 3 },
    { 0xce02b485, 3 },
    { 0x1c62b485, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf776b48d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2536b490, 3 },
    { 0, 0 },
    { 0xf956b492, 3 },
    { 0, 0 },
    { 0xd44ab494, 3 },
    { 0xa41eb494, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xba12b498, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cab49b, 3 },
    { 0x770eb49c, 3 },
    { 0x9c2b49c, 3 },
    { 0, 0 },
    { 0xdf6eb49f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf792b4a2, 3 },
    { 0xc566b4a3, 4 },
    { 0, 0 },
    { 0x7042b4a5, 3 },
    { 0x8492b4a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9002b4a9, 3 },
    { 0xcf0ab4aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5eb4ae, 3 },
    { 0, 0 },
    { 0x510ab4b0, 5 },
    { 0xe3c6b4b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b36b4b5, 3 },
    { 0x5beb4b5, 3 },
    { 0x47eeb4b7, 3 },
    { 0x9c72b4b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f2b4bc, 3 },
    { 0, 0 },
    { 0x636b4be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2b4c1, 4 },
    { 0x3c9ab4c2, 3 },
    { 0, 0 },
    { 0x771ab4c4, 3 },
    { 0x73b2b4c5, 3 },
    { 0xdbeb4c5, 3 },
    { 0xecfeb4c6, 5 },
    { 0x1d16b4c8, 3 },
    { 0x5552b4c9, 3 },
    { 0x945ab4ca, 3 },
    { 0x325eb4c6, 3 },
    { 0x2216b4c7, 3 },
    { 0, 0 },
    { 0xcf92b4ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf46b4d1, 4 },
    { 0x1e9eb4d2, 3 },
    { 0, 0 },
    { 0xd972b4d4, 3 },
    { 0x286ab4d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631ab4d9, 3 },
    { 0xce92b4da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2502b4df, 3 },
    { 0x8d06b4e0, 3 },
    { 0, 0 },
    { 0xbb1eb4e2, 3 },
    { 0x7d0eb4e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a66b4e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ab4ec, 3 },
    { 0xfe8eb4ed, 3 },
    { 0x550ab4ec, 3 },
    { 0xb04ab4ee, 3 },
    { 0x4c7ab4f0, 4 },
    { 0, 0 },
    { 0x6466b4f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe382b4f7, 3 },
    { 0xab52b4f8, 4 },
    { 0x8f6eb4f7, 4 },
    { 0x4386b4fa, 3 },
    { 0xf3d6b4fa, 3 },
    { 0x16c6b4fc, 3 },
    { 0xb98eb4fa, 4 },
    { 0x93b6b4fe, 4 },
    { 0xacc6b4fa, 4 },
    { 0x33f6b500, 3 },
    { 0x3252b501, 3 },
    { 0x762b500, 3 },
    { 0xc836b503, 3 },
    { 0, 0 },
    { 0xca42b505, 3 },
    { 0, 0 },
    { 0xcbfeb507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5612b50b, 3 },
    { 0xb43eb50b, 3 },
    { 0, 0 },
    { 0x60e6b50e, 3 },
    { 0, 0 },
    { 0xd9deb510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6b513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d2b519, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x175ab51c, 3 },
    { 0x4a22b51c, 5 },
    { 0xac56b51e, 3 },
    { 0x2d22b51f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ab522, 3 },
    { 0xc3aeb523, 4 },
    { 0xc5b2b524, 3 },
    { 0, 0 },
    { 0x2cab526, 3 },
    { 0x5f7ab527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476eb52e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94eb533, 3 },
    { 0, 0 },
    { 0x54bab535, 3 },
    { 0, 0 },
    { 0xff6b537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2492b53a, 3 },
    { 0xf436b53b, 3 },
    { 0, 0 },
    { 0xdffab53d, 3 },
    { 0x45a2b53e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d6b542, 3 },
    { 0, 0 },
    { 0x59fab544, 4 },
    { 0x64b2b545, 4 },
    { 0x8a0ab546, 3 },
    { 0x879ab544, 4 },
    { 0xe07ab548, 3 },
    { 0x280eb548, 3 },
    { 0xd3b2b549, 5 },
    { 0x9a4ab54b, 4 },
    { 0, 0 },
    { 0xc3b2b54d, 3 },
    { 0xda12b54e, 3 },
    { 0xb28ab54e, 4 },
    { 0xee2b550, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b556, 3 },
    { 0xfac6b557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2b55a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc6b55d, 3 },
    { 0, 0 },
    { 0x2f22b55f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f06b565, 3 },
    { 0x79a2b566, 3 },
    { 0xf26ab567, 4 },
    { 0x4d92b567, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ab56b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b56f, 3 },
    { 0xe7d2b570, 3 },
    { 0xada6b571, 5 },
    { 0x5226b572, 5 },
    { 0xa9f6b573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a16b578, 3 },
    { 0x734ab579, 4 },
    { 0, 0 },
    { 0x4b2eb57b, 3 },
    { 0, 0 },
    { 0x132eb57d, 3 },
    { 0xbf7ab57d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf706b582, 4 },
    { 0, 0 },
    { 0x976b584, 3 },
    { 0xd672b584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf2b588, 3 },
    { 0, 0 },
    { 0x4cab58a, 3 },
    { 0, 0 },
    { 0x3622b58c, 3 },
    { 0x8b2ab58d, 3 },
    { 0xb032b58e, 3 },
    { 0x190ab58e, 3 },
    { 0x7ad6b58e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38ab594, 4 },
    { 0x934eb595, 3 },
    { 0, 0 },
    { 0x20cab597, 4 },
    { 0, 0 },
    { 0x76b2b599, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x556b59c, 4 },
    { 0x5cd2b59d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeab5a0, 3 },
    { 0x9836b5a1, 4 },
    { 0, 0 },
    { 0xd09ab5a3, 3 },
    { 0x7b9ab5a4, 3 },
    { 0x3b5ab5a5, 5 },
    { 0x43eeb5a6, 3 },
    { 0x1f96b5a7, 3 },
    { 0, 0 },
    { 0x9b92b5a9, 4 },
    { 0x49ceb5aa, 4 },
    { 0, 0 },
    { 0xb566b5ac, 3 },
    { 0x9146b5ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b12b5b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e62b5b4, 3 },
    { 0x4c06b5b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7eb5b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2b5bd, 4 },
    { 0, 0 },
    { 0x55b2b5bf, 4 },
    { 0xfd76b5c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbeb5c3, 3 },
    { 0x1462b5c4, 3 },
    { 0, 0 },
    { 0x4172b5c6, 4 },
    { 0xa2e2b5c7, 3 },
    { 0x7262b5c6, 5 },
    { 0, 0 },
    { 0xe96eb5ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7342b5d2, 3 },
    { 0x7a92b5d3, 3 },
    { 0xd752b5d4, 3 },
    { 0xdedab5d5, 4 },
    { 0xe38eb5d6, 3 },
    { 0x3bf6b5d3, 3 },
    { 0xceeeb5d8, 3 },
    { 0x698eb5d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfeb5dc, 3 },
    { 0x4b3ab5dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85eb5e3, 4 },
    { 0xf606b5e4, 3 },
    { 0xfe92b5e5, 3 },
    { 0, 0 },
    { 0xde36b5e7, 3 },
    { 0xf062b5e8, 3 },
    { 0xdd26b5e9, 3 },
    { 0xcb66b5ea, 3 },
    { 0x2c92b5eb, 3 },
    { 0xc44eb5eb, 5 },
    { 0x7e86b5eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bab5f4, 3 },
    { 0, 0 },
    { 0xe7d6b5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef2b5fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eab606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d2b609, 4 },
    { 0x65cab60a, 3 },
    { 0xbc7eb60b, 3 },
    { 0xf562b609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9076b60f, 3 },
    { 0x6e0ab60f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e62b613, 3 },
    { 0, 0 },
    { 0x3deb615, 3 },
    { 0xbd32b616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9206b61c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6b620, 4 },
    { 0, 0 },
    { 0x4c2ab622, 3 },
    { 0x6e42b623, 3 },
    { 0x6902b623, 3 },
    { 0x49bab625, 3 },
    { 0xf4aab625, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe996b62c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9eb631, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3eb63b, 3 },
    { 0xd8dab63b, 3 },
    { 0xd072b63b, 5 },
    { 0x7572b63d, 4 },
    { 0xa2bab63f, 3 },
    { 0, 0 },
    { 0x6e1ab641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb422b648, 4 },
    { 0, 0 },
    { 0x633ab64a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce2b64d, 3 },
    { 0xb936b64e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26eb652, 3 },
    { 0x948eb653, 4 },
    { 0x9a1ab654, 4 },
    { 0xeedab653, 3 },
    { 0x7b5ab656, 3 },
    { 0xb0b6b657, 3 },
    { 0x2892b656, 3 },
    { 0x9556b659, 3 },
    { 0, 0 },
    { 0xc98eb65b, 3 },
    { 0xf602b65c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edeb65f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beab662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225eb66a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6b671, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eb676, 4 },
    { 0, 0 },
    { 0x3df2b678, 4 },
    { 0x580ab678, 5 },
    { 0xaf2eb67a, 4 },
    { 0x66ab67a, 4 },
    { 0x796ab67b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7ab67f, 3 },
    { 0xd6f6b680, 3 },
    { 0, 0 },
    { 0xacb6b682, 4 },
    { 0, 0 },
    { 0x412ab684, 5 },
    { 0xc3eeb685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc2b68a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd346b68e, 3 },
    { 0x665eb68e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fab694, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x894ab697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34beb69a, 3 },
    { 0x5872b69b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432b6a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa212b6a9, 4 },
    { 0xf2b6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa76b6ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5eb6b1, 3 },
    { 0, 0 },
    { 0x482b6b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65eb6b7, 3 },
    { 0, 0 },
    { 0x80eb6b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3362b6bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdab6c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfab6c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e2b6c8, 3 },
    { 0, 0 },
    { 0xbfb6b6ca, 3 },
    { 0x8986b6cb, 3 },
    { 0xb746b6cc, 3 },
    { 0, 0 },
    { 0x4a82b6ce, 3 },
    { 0xec5eb6cf, 4 },
    { 0x1d8ab6d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf2b6d3, 4 },
    { 0xc552b6d4, 4 },
    { 0x158eb6d3, 3 },
    { 0x3fab6d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e6b6e2, 3 },
    { 0x851eb6e2, 3 },
    { 0, 0 },
    { 0xc892b6e5, 3 },
    { 0, 0 },
    { 0x332ab6e7, 3 },
    { 0, 0 },
    { 0x509eb6e9, 3 },
    { 0x84dab6ea, 3 },
    { 0xfe9eb6ea, 3 },
    { 0x2612b6ec, 4 },
    { 0x823ab6ed, 3 },
    { 0x54ab6ee, 3 },
    { 0x27aab6ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd2b6f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eb6f6, 3 },
    { 0xad76b6f7, 3 },
    { 0, 0 },
    { 0xa7e6b6f9, 4 },
    { 0xd7ceb6f9, 3 },
    { 0x8506b6fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b6ff, 4 },
    { 0x4d06b6ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dab704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd6b719, 4 },
    { 0x702b71a, 3 },
    { 0, 0 },
    { 0x404eb71c, 3 },
    { 0x236ab71d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fab722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8deb725, 5 },
    { 0, 0 },
    { 0x43ceb727, 3 },
    { 0x69beb728, 4 },
    { 0xff4ab729, 3 },
    { 0x6a82b729, 4 },
    { 0xe8aab72a, 5 },
    { 0x479ab727, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6b737, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecab73a, 3 },
    { 0xe62ab73b, 3 },
    { 0x34eab73b, 3 },
    { 0x1afeb73d, 4 },
    { 0xf34eb73d, 3 },
    { 0x2cdeb73f, 5 },
    { 0x68dab73f, 3 },
    { 0xdd5ab73f, 4 },
    { 0xeaa2b741, 3 },
    { 0xd66ab73a, 3 },
    { 0x5f16b744, 3 },
    { 0, 0 },
    { 0xc046b746, 3 },
    { 0xc1d2b746, 4 },
    { 0, 0 },
    { 0xe6a6b749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3eb74e, 3 },
    { 0x854eb74f, 3 },
    { 0, 0 },
    { 0x116eb751, 3 },
    { 0xfddeb752, 3 },
    { 0, 0 },
    { 0x919ab754, 4 },
    { 0x1dcab755, 3 },
    { 0x96d6b756, 3 },
    { 0xefa6b754, 3 },
    { 0x934ab754, 3 },
    { 0x7426b759, 3 },
    { 0xc846b754, 3 },
    { 0, 0 },
    { 0x836b75c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56eb760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976b763, 3 },
    { 0x11feb764, 3 },
    { 0, 0 },
    { 0x41ab766, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b76a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ab76d, 3 },
    { 0x16a2b76e, 3 },
    { 0xa322b76f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7126b777, 3 },
    { 0xf6ab778, 3 },
    { 0x336eb779, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55ab77c, 4 },
    { 0x18aeb77d, 3 },
    { 0xbd56b77e, 3 },
    { 0, 0 },
    { 0x27f6b780, 3 },
    { 0x8fceb781, 3 },
    { 0x56b2b781, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cab78b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7416b791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bab795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652b79a, 4 },
    { 0xca96b79b, 3 },
    { 0xa392b79b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6ab79f, 3 },
    { 0, 0 },
    { 0x70cab7a1, 3 },
    { 0, 0 },
    { 0xfb8ab7a3, 4 },
    { 0xa5cab7a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f6b7a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d12b7ac, 3 },
    { 0, 0 },
    { 0x2b1eb7ae, 3 },
    { 0, 0 },
    { 0xb6b2b7b0, 3 },
    { 0, 0 },
    { 0x2fb2b7b2, 3 },
    { 0, 0 },
    { 0xbb66b7b4, 3 },
    { 0, 0 },
    { 0x649ab7b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92eb7ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cab7c0, 3 },
    { 0, 0 },
    { 0x624ab7c2, 3 },
    { 0x4f26b7c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb846b7cc, 3 },
    { 0x21f6b7cd, 3 },
    { 0, 0 },
    { 0x4bc2b7cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d6b7d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe656b7d8, 4 },
    { 0xf2a6b7d9, 3 },
    { 0xb97ab7d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed2b7e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceab7e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abab7ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c16b7ef, 3 },
    { 0, 0 },
    { 0x1a16b7f1, 3 },
    { 0, 0 },
    { 0xecb2b7f3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3eb7f7, 3 },
    { 0x6cc2b7f8, 3 },
    { 0x328eb7f9, 4 },
    { 0x88a6b7fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3eb801, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaeb804, 3 },
    { 0, 0 },
    { 0x5ea2b806, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b809, 4 },
    { 0x82f2b809, 5 },
    { 0, 0 },
    { 0x3fceb80c, 5 },
    { 0xe88eb80d, 4 },
    { 0xefdab80d, 3 },
    { 0xaff2b80e, 5 },
    { 0x2deeb80d, 3 },
    { 0x5f32b811, 3 },
    { 0xbd52b811, 4 },
    { 0x63c6b811, 4 },
    { 0x70d2b80d, 4 },
    { 0x718eb815, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0eb818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6eb81c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef2b822, 3 },
    { 0x2866b823, 3 },
    { 0, 0 },
    { 0x360ab825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2b828, 3 },
    { 0x7716b829, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec6b82e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f2b835, 3 },
    { 0x3a6ab836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6eb839, 3 },
    { 0x9ce6b83a, 3 },
    { 0x756b83b, 3 },
    { 0x307ab83b, 3 },
    { 0, 0 },
    { 0x2dbeb83e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6b843, 3 },
    { 0, 0 },
    { 0x699ab845, 3 },
    { 0x53deb845, 3 },
    { 0x38f6b846, 4 },
    { 0x6aeeb848, 4 },
    { 0x481eb848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21ab84c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc6b853, 3 },
    { 0x5356b854, 4 },
    { 0xf4a6b854, 3 },
    { 0x99d2b856, 3 },
    { 0x6aaab856, 3 },
    { 0, 0 },
    { 0x5c2eb859, 3 },
    { 0, 0 },
    { 0xbdd2b85b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5752b861, 3 },
    { 0xe1f2b862, 3 },
    { 0xc972b861, 4 },
    { 0x72beb863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423ab86d, 3 },
    { 0x481eb86e, 4 },
    { 0x600ab86e, 3 },
    { 0x2ceab870, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282ab875, 3 },
    { 0, 0 },
    { 0x9b6b877, 3 },
    { 0xc3eeb878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959ab87c, 4 },
    { 0x6f6b87d, 3 },
    { 0x4196b87d, 3 },
    { 0xcab2b87c, 3 },
    { 0, 0 },
    { 0x8a7eb881, 3 },
    { 0xe4c6b882, 4 },
    { 0x8282b883, 3 },
    { 0xa1fab882, 3 },
    { 0x7402b882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6316b88c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24ab893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8eb896, 4 },
    { 0xd562b896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfeb89b, 3 },
    { 0x22a6b89c, 3 },
    { 0xb912b89d, 3 },
    { 0x18e2b89e, 4 },
    { 0xc972b89f, 3 },
    { 0xfe4ab89e, 4 },
    { 0, 0 },
    { 0x21feb8a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5132b8ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5302b8b1, 3 },
    { 0xfbd6b8b2, 3 },
    { 0x18deb8b1, 4 },
    { 0x2beab8b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x667eb8b7, 3 },
    { 0xc346b8b7, 3 },
    { 0, 0 },
    { 0x8abeb8ba, 3 },
    { 0x8942b8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8872b8c3, 4 },
    { 0x6d32b8c4, 3 },
    { 0x6ffeb8c3, 3 },
    { 0x389ab8c3, 5 },
    { 0, 0 },
    { 0x698ab8c8, 4 },
    { 0x6c16b8c9, 3 },
    { 0, 0 },
    { 0xb89eb8cb, 3 },
    { 0x95aab8cc, 3 },
    { 0x5cbab8cb, 3 },
    { 0xe622b8ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ab8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe266b8db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb576b8e2, 3 },
    { 0, 0 },
    { 0x2ac6b8e4, 4 },
    { 0, 0 },
    { 0x7192b8e6, 4 },
    { 0xa0fab8e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbeb8ea, 3 },
    { 0xebfab8ea, 3 },
    { 0xab06b8eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33ab8fa, 3 },
    { 0, 0 },
    { 0x971eb8fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716ab905, 3 },
    { 0x819ab905, 3 },
    { 0, 0 },
    { 0x9a46b908, 3 },
    { 0, 0 },
    { 0x9922b90a, 4 },
    { 0, 0 },
    { 0x3752b90c, 3 },
    { 0, 0 },
    { 0x89aeb90e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f6b913, 3 },
    { 0x9622b914, 3 },
    { 0, 0 },
    { 0x2c82b916, 3 },
    { 0x701eb917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbab91a, 3 },
    { 0x7672b91b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2ab91f, 3 },
    { 0x2a7ab920, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5496b927, 4 },
    { 0xcddeb928, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x656eb92b, 3 },
    { 0xf1ceb92b, 4 },
    { 0x5126b92d, 3 },
    { 0xdc5eb92d, 3 },
    { 0x980ab92c, 4 },
    { 0x7a7eb930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470eb934, 3 },
    { 0x6c2ab935, 3 },
    { 0, 0 },
    { 0x58d6b937, 3 },
    { 0, 0 },
    { 0xa752b939, 3 },
    { 0xc6d6b93a, 3 },
    { 0, 0 },
    { 0x53feb93c, 4 },
    { 0x7eb93d, 3 },
    { 0x5ca6b93d, 3 },
    { 0x261eb93f, 3 },
    { 0xbf1ab940, 3 },
    { 0xc71ab941, 4 },
    { 0xa046b942, 4 },
    { 0xd322b943, 3 },
    { 0x60b2b944, 3 },
    { 0x504ab943, 3 },
    { 0xbf36b943, 3 },
    { 0xb5b6b941, 3 },
    { 0x997ab948, 3 },
    { 0, 0 },
    { 0x8f0eb94a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d2b950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b2b953, 3 },
    { 0xa40eb954, 3 },
    { 0x6492b953, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ceb95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e2b95d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e16b961, 3 },
    { 0xbb16b962, 4 },
    { 0xd1f2b963, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce6b96b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x508ab96e, 3 },
    { 0x33c6b96e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eb973, 3 },
    { 0x5feeb974, 3 },
    { 0xe026b974, 4 },
    { 0, 0 },
    { 0xcb7eb977, 4 },
    { 0xa06eb978, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557eb97e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c32b982, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eb987, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2b98a, 3 },
    { 0, 0 },
    { 0xad42b98c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ab98f, 4 },
    { 0xaeaab990, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aab993, 3 },
    { 0x7b82b994, 3 },
    { 0, 0 },
    { 0x6622b996, 4 },
    { 0x9a56b997, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512b99b, 3 },
    { 0xcc4ab99c, 4 },
    { 0, 0 },
    { 0xf472b99e, 3 },
    { 0x862b99f, 3 },
    { 0x80d2b99f, 3 },
    { 0xc05eb99f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de2b9a8, 3 },
    { 0, 0 },
    { 0x51aeb9aa, 3 },
    { 0xb47ab9ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6ab9ae, 3 },
    { 0xfe16b9af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6662b9b2, 3 },
    { 0x6576b9b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eab9b6, 4 },
    { 0xbf3eb9b7, 4 },
    { 0, 0 },
    { 0xc532b9b9, 3 },
    { 0, 0 },
    { 0x6e72b9bb, 3 },
    { 0, 0 },
    { 0x2cd2b9bd, 5 },
    { 0, 0 },
    { 0x73bab9bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38eb9c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe432b9c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4ab9cd, 3 },
    { 0x60f6b9ce, 4 },
    { 0xdc02b9cf, 4 },
    { 0, 0 },
    { 0x547ab9d1, 3 },
    { 0x5ba6b9d1, 3 },
    { 0xbbf6b9d1, 4 },
    { 0x9eeab9d4, 3 },
    { 0x4d02b9d5, 3 },
    { 0x473ab9d6, 3 },
    { 0x6e16b9d6, 5 },
    { 0xa4b6b9d4, 4 },
    { 0x31d6b9d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5732b9e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eab9e5, 4 },
    { 0, 0 },
    { 0x1fc2b9e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993ab9eb, 5 },
    { 0, 0 },
    { 0xd566b9ed, 4 },
    { 0x177ab9ed, 4 },
    { 0, 0 },
    { 0x54a2b9f0, 3 },
    { 0x9886b9f1, 4 },
    { 0x8b16b9f0, 3 },
    { 0, 0 },
    { 0xde2ab9f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde76b9fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x621eba00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9606ba08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8816ba0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4852ba0f, 4 },
    { 0x15e2ba0f, 3 },
    { 0x13b2ba11, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a96ba15, 3 },
    { 0, 0 },
    { 0xabb2ba17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5956ba1b, 3 },
    { 0x6316ba1c, 3 },
    { 0x1b76ba1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7112ba26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0aba2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0eba30, 3 },
    { 0xa972ba30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ba34, 4 },
    { 0x84deba34, 4 },
    { 0, 0 },
    { 0x71eeba37, 3 },
    { 0x14baba38, 4 },
    { 0x66b2ba39, 3 },
    { 0x108eba38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe302ba41, 3 },
    { 0, 0 },
    { 0xe652ba43, 4 },
    { 0x986ba44, 3 },
    { 0xdc32ba45, 3 },
    { 0x5fe2ba46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcaba49, 3 },
    { 0xd216ba4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afaba4e, 3 },
    { 0, 0 },
    { 0xddceba50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53eba53, 3 },
    { 0xc1aeba54, 3 },
    { 0xc742ba55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8aba59, 3 },
    { 0xa7e2ba59, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6ba5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b76ba60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75eba68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e96ba6e, 3 },
    { 0xa4c2ba6e, 3 },
    { 0xcf6eba70, 3 },
    { 0xa01aba71, 4 },
    { 0x47deba72, 3 },
    { 0x9126ba73, 3 },
    { 0x7b26ba73, 3 },
    { 0x1302ba73, 4 },
    { 0xa9ceba76, 3 },
    { 0, 0 },
    { 0x4c4aba78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeba7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a62ba80, 3 },
    { 0, 0 },
    { 0x8b2ba82, 3 },
    { 0x4a0aba83, 3 },
    { 0x9f62ba84, 3 },
    { 0x513aba85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e92ba88, 3 },
    { 0x97feba89, 4 },
    { 0xcaba8a, 3 },
    { 0x6ca2ba8a, 4 },
    { 0x31eaba8c, 3 },
    { 0x1236ba8c, 3 },
    { 0, 0 },
    { 0x253eba8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1606ba94, 3 },
    { 0xc7aaba95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebaba98, 3 },
    { 0x1d82ba99, 4 },
    { 0x4bceba9a, 4 },
    { 0xdd42ba9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76aba9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abaa3, 4 },
    { 0, 0 },
    { 0x140abaa5, 3 },
    { 0, 0 },
    { 0x8a0ebaa7, 3 },
    { 0xa436baa7, 3 },
    { 0xfcaabaa8, 4 },
    { 0, 0 },
    { 0xf346baab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b2baae, 3 },
    { 0x1516baae, 4 },
    { 0, 0 },
    { 0x6516bab1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53abab4, 3 },
    { 0x7cc6bab5, 3 },
    { 0, 0 },
    { 0xbd92bab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ebaba, 3 },
    { 0, 0 },
    { 0xca66babc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29eababf, 3 },
    { 0x86aebabf, 3 },
    { 0x1a7abac1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2bac6, 3 },
    { 0x33febac6, 3 },
    { 0, 0 },
    { 0x262abac9, 3 },
    { 0xe1b2bac9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc896bad1, 3 },
    { 0xb1aebad2, 3 },
    { 0x893ebad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60febadf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e2bae2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ebae7, 3 },
    { 0, 0 },
    { 0xbb1abae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x396abaec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8512baf5, 4 },
    { 0x630ebaf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0ebafa, 3 },
    { 0, 0 },
    { 0x9f8ebafc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220ebb01, 3 },
    { 0x6f9ebb02, 4 },
    { 0x3586bb02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f2bb06, 3 },
    { 0x28f2bb07, 3 },
    { 0, 0 },
    { 0xf746bb09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9312bb13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6bb16, 3 },
    { 0, 0 },
    { 0xd20abb18, 3 },
    { 0, 0 },
    { 0x46dabb1a, 4 },
    { 0, 0 },
    { 0x33debb1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca26bb20, 3 },
    { 0x236bb20, 3 },
    { 0xebe6bb22, 3 },
    { 0xe0eabb21, 4 },
    { 0x5616bb22, 3 },
    { 0x36d2bb21, 3 },
    { 0x3c16bb26, 3 },
    { 0xc41ebb26, 3 },
    { 0, 0 },
    { 0x5c42bb29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeebb2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9306bb34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09ebb37, 5 },
    { 0xd662bb38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa72bb3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89abb40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667abb49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bebb4d, 4 },
    { 0, 0 },
    { 0x4cb6bb4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a22bb56, 4 },
    { 0xdb1ebb57, 5 },
    { 0xede2bb58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fabb5c, 3 },
    { 0, 0 },
    { 0x3832bb5e, 3 },
    { 0x76c6bb5e, 3 },
    { 0xa412bb5f, 5 },
    { 0xa68abb61, 4 },
    { 0xdeb6bb62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb572bb67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86abb6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cabb6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867abb72, 3 },
    { 0, 0 },
    { 0x254ebb74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1abb77, 4 },
    { 0, 0 },
    { 0x9ceabb79, 3 },
    { 0, 0 },
    { 0x3b66bb7b, 3 },
    { 0x61f6bb7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ebb7f, 3 },
    { 0x3712bb80, 3 },
    { 0xb896bb81, 3 },
    { 0x1226bb82, 3 },
    { 0x2c1ebb82, 3 },
    { 0x9afebb82, 4 },
    { 0x56dabb84, 3 },
    { 0x4a52bb86, 3 },
    { 0xbcdabb85, 3 },
    { 0x115ebb86, 3 },
    { 0x79c2bb83, 3 },
    { 0xc75abb84, 3 },
    { 0, 0 },
    { 0x16e6bb8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c32bb93, 3 },
    { 0, 0 },
    { 0xb7b6bb95, 4 },
    { 0xb9e6bb96, 3 },
    { 0xe46abb96, 4 },
    { 0x62c6bb97, 4 },
    { 0, 0 },
    { 0x579abb9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ebba3, 4 },
    { 0, 0 },
    { 0x4c4ebba5, 4 },
    { 0xf8e6bba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aebbaf, 3 },
    { 0x435ebbaf, 3 },
    { 0x5de6bbb1, 5 },
    { 0x1722bbb2, 4 },
    { 0x45d2bbb2, 3 },
    { 0x8f2ebbb2, 3 },
    { 0x6ea6bbb5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d2bbba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa712bbc3, 3 },
    { 0x8192bbc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96bbc7, 3 },
    { 0xd1d2bbc8, 4 },
    { 0, 0 },
    { 0x59a2bbca, 3 },
    { 0x13dabbcb, 3 },
    { 0xb7aebbcc, 4 },
    { 0, 0 },
    { 0x3f2ebbce, 3 },
    { 0x42d6bbcf, 4 },
    { 0x2f6bbce, 3 },
    { 0xf40abbd1, 3 },
    { 0, 0 },
    { 0x527ebbd3, 4 },
    { 0x8286bbd4, 3 },
    { 0x5146bbd5, 3 },
    { 0xf0e6bbd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862bbdb, 4 },
    { 0x6e86bbdc, 3 },
    { 0x61f2bbdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84abbe0, 3 },
    { 0xd816bbe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3276bbe4, 3 },
    { 0x694ebbe5, 4 },
    { 0xf5a2bbe5, 3 },
    { 0xf502bbe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9abbee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773abbfd, 3 },
    { 0xe216bbfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3daabc03, 4 },
    { 0, 0 },
    { 0xe2fabc05, 3 },
    { 0, 0 },
    { 0x7ccebc07, 3 },
    { 0, 0 },
    { 0x3b42bc09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62bc11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eebc16, 3 },
    { 0xdd5abc16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc562bc1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1ebc1f, 3 },
    { 0x49aabc20, 3 },
    { 0x9c76bc21, 4 },
    { 0x5d5ebc22, 3 },
    { 0x3abebc22, 3 },
    { 0xcfaebc23, 4 },
    { 0x336ebc20, 3 },
    { 0x65aebc23, 3 },
    { 0x9ab6bc25, 4 },
    { 0, 0 },
    { 0x5c2ebc29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5102bc2d, 3 },
    { 0xe6bebc2d, 3 },
    { 0, 0 },
    { 0xa7bebc30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d86bc35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x998abc38, 4 },
    { 0, 0 },
    { 0x36eabc3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaebc40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ebc43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe76bc47, 3 },
    { 0x1802bc48, 4 },
    { 0x7452bc47, 4 },
    { 0xaa36bc4a, 3 },
    { 0xf812bc48, 3 },
    { 0x30f6bc4b, 3 },
    { 0x740ebc47, 4 },
    { 0x3f5ebc4e, 3 },
    { 0, 0 },
    { 0xcc1abc50, 3 },
    { 0xbaeebc50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7abc54, 3 },
    { 0xaa92bc55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe646bc60, 3 },
    { 0, 0 },
    { 0x6786bc62, 3 },
    { 0xca52bc63, 3 },
    { 0xdc42bc62, 3 },
    { 0, 0 },
    { 0xd7e2bc66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d26bc6a, 3 },
    { 0xb3eebc6a, 3 },
    { 0, 0 },
    { 0x344abc6d, 3 },
    { 0xfd26bc6e, 4 },
    { 0xbdc2bc6f, 3 },
    { 0x4646bc70, 5 },
    { 0, 0 },
    { 0x212abc72, 3 },
    { 0, 0 },
    { 0x9d4abc74, 4 },
    { 0x796ebc74, 3 },
    { 0x9c76bc76, 3 },
    { 0xc77abc77, 3 },
    { 0, 0 },
    { 0x427abc79, 4 },
    { 0, 0 },
    { 0xc936bc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a32bc7f, 3 },
    { 0, 0 },
    { 0xf7d2bc81, 3 },
    { 0, 0 },
    { 0x5e16bc83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cebc86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbabc8b, 3 },
    { 0xff7abc8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491ebc92, 3 },
    { 0x61b6bc93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9486bc9e, 3 },
    { 0xb2c2bc9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a2bca3, 4 },
    { 0xdbb6bca4, 3 },
    { 0xbb8ebca3, 4 },
    { 0xf41abca6, 3 },
    { 0x74cabca7, 4 },
    { 0xcf3abca6, 3 },
    { 0xdf32bca9, 3 },
    { 0x5132bca9, 4 },
    { 0, 0 },
    { 0xbc72bcac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2bcb3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf66bcb6, 3 },
    { 0xb97abcb7, 3 },
    { 0, 0 },
    { 0x4922bcb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726bcbe, 3 },
    { 0xc5cabcbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4796bcc7, 3 },
    { 0xbd5abcc7, 3 },
    { 0x53c6bcc9, 3 },
    { 0x9cd2bcc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6bcce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eebcd6, 4 },
    { 0xd9d2bcd7, 4 },
    { 0, 0 },
    { 0xe4cabcd9, 5 },
    { 0xf2bcda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0abcdd, 3 },
    { 0x18b6bcde, 3 },
    { 0x18aebcdf, 3 },
    { 0x3b3abce0, 4 },
    { 0, 0 },
    { 0xb60abce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc2bce5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fabce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11abcf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c52bcfa, 3 },
    { 0xf112bcfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1846bcff, 3 },
    { 0, 0 },
    { 0x80debd01, 4 },
    { 0x31d6bd02, 5 },
    { 0x5552bd02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905ebd09, 4 },
    { 0x6986bd09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a2bd0e, 3 },
    { 0xe982bd0f, 3 },
    { 0xaeaabd0e, 3 },
    { 0x81b2bd11, 3 },
    { 0, 0 },
    { 0x957abd13, 4 },
    { 0, 0 },
    { 0xd7aabd15, 3 },
    { 0xb986bd16, 3 },
    { 0xb052bd16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3abd22, 3 },
    { 0x71dabd22, 3 },
    { 0x7bfebd24, 3 },
    { 0xc8cabd24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x997ebd28, 3 },
    { 0xb1babd29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181ebd31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f2bd35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdabd39, 3 },
    { 0x2a52bd3a, 4 },
    { 0, 0 },
    { 0xe792bd3c, 4 },
    { 0, 0 },
    { 0x298abd3e, 4 },
    { 0, 0 },
    { 0x3ebabd40, 3 },
    { 0, 0 },
    { 0xb2abd42, 4 },
    { 0x85a2bd42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935abd48, 4 },
    { 0x3052bd49, 4 },
    { 0x647abd48, 3 },
    { 0xaf36bd48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c82bd4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f6bd54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1abd57, 3 },
    { 0xdceabd57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfabd5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda86bd63, 3 },
    { 0, 0 },
    { 0x143ebd65, 3 },
    { 0x2bc2bd66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2ebd6f, 3 },
    { 0xd362bd70, 3 },
    { 0, 0 },
    { 0xe782bd72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dabd77, 4 },
    { 0xefaabd77, 3 },
    { 0x87febd77, 3 },
    { 0xb0febd79, 4 },
    { 0xabeabd7b, 3 },
    { 0x74ebd7b, 4 },
    { 0x9a9abd7d, 3 },
    { 0, 0 },
    { 0x134abd7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5abd83, 3 },
    { 0x8f2bd84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39abd8d, 3 },
    { 0xe2e6bd8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12abd93, 3 },
    { 0x337ebd94, 3 },
    { 0, 0 },
    { 0xea7ebd96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42abd9d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723abda1, 3 },
    { 0xe142bda1, 4 },
    { 0xad5abda3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68abda6, 3 },
    { 0xd286bda7, 3 },
    { 0, 0 },
    { 0x87cabda9, 3 },
    { 0x6262bdaa, 3 },
    { 0x1ebdab, 3 },
    { 0, 0 },
    { 0xb83abdad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812abdb3, 3 },
    { 0xc0fabdb4, 3 },
    { 0, 0 },
    { 0xc616bdb6, 4 },
    { 0xbd32bdb6, 3 },
    { 0x30aebdb7, 4 },
    { 0, 0 },
    { 0xb466bdba, 4 },
    { 0x2e3ebdbb, 4 },
    { 0, 0 },
    { 0xf12ebdbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6ebdc0, 3 },
    { 0x73cabdc1, 3 },
    { 0xae06bdc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb6bdca, 3 },
    { 0x82d6bdcb, 3 },
    { 0, 0 },
    { 0x974abdcd, 3 },
    { 0x1e36bdce, 3 },
    { 0xa0fabdce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ebdd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c56bddb, 3 },
    { 0, 0 },
    { 0x2c86bddd, 4 },
    { 0, 0 },
    { 0x227abddf, 3 },
    { 0, 0 },
    { 0x9deebde1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daebde4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cebdeb, 3 },
    { 0x3896bdeb, 3 },
    { 0xb10ebded, 3 },
    { 0, 0 },
    { 0xa8babdef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795ebdf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588abdfa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02ebdfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172be02, 3 },
    { 0xa1a6be03, 3 },
    { 0x25c6be03, 5 },
    { 0x3e66be04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273abe0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864abe10, 3 },
    { 0xe96be10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b16be15, 4 },
    { 0x7a66be15, 3 },
    { 0, 0 },
    { 0x1c52be18, 3 },
    { 0xfbd6be19, 3 },
    { 0xc20ebe1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549ebe1e, 4 },
    { 0x4306be1e, 4 },
    { 0x51b2be1e, 5 },
    { 0xf156be21, 4 },
    { 0xee46be21, 3 },
    { 0x404ebe23, 3 },
    { 0x5ebe24, 3 },
    { 0x23cebe1e, 4 },
    { 0, 0 },
    { 0x36f2be27, 3 },
    { 0x927ebe28, 4 },
    { 0x43b2be29, 4 },
    { 0, 0 },
    { 0x3432be2b, 4 },
    { 0x4ffabe2c, 3 },
    { 0xe6b2be2c, 4 },
    { 0xac66be2c, 4 },
    { 0xc046be2d, 3 },
    { 0x571ebe2e, 3 },
    { 0x17dabe2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747ebe37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bebe40, 4 },
    { 0x18f2be40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e6be47, 3 },
    { 0x9c6abe47, 4 },
    { 0x305abe49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21febe4e, 3 },
    { 0x2272be4f, 3 },
    { 0, 0 },
    { 0xc2cabe51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566be54, 4 },
    { 0x4116be55, 3 },
    { 0, 0 },
    { 0x40cebe57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542ebe5b, 3 },
    { 0xe752be5c, 3 },
    { 0, 0 },
    { 0xa2cabe5e, 4 },
    { 0x1f42be5e, 5 },
    { 0x7d66be5e, 4 },
    { 0, 0 },
    { 0x7ecebe62, 4 },
    { 0x17e2be62, 4 },
    { 0x246be64, 3 },
    { 0x7b16be64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3006be70, 4 },
    { 0x7b72be70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe886be74, 3 },
    { 0xf43abe75, 3 },
    { 0, 0 },
    { 0xde8abe77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2be7e, 4 },
    { 0, 0 },
    { 0x3a76be80, 4 },
    { 0, 0 },
    { 0xbf12be82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1112be8a, 3 },
    { 0x5e0abe8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6262be90, 3 },
    { 0xf346be90, 3 },
    { 0, 0 },
    { 0x7a8abe93, 4 },
    { 0x5292be94, 3 },
    { 0x8d62be94, 3 },
    { 0xab86be96, 3 },
    { 0x7a7ebe97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69debea0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x892ebea3, 3 },
    { 0, 0 },
    { 0xa9eebea5, 3 },
    { 0, 0 },
    { 0x4c86bea7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a66beaa, 3 },
    { 0xd90abeab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cabeae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95abeb2, 3 },
    { 0x29e2beb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7726beb6, 3 },
    { 0, 0 },
    { 0x3206beb8, 3 },
    { 0xe9e6beb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb96bebc, 3 },
    { 0xe15abebd, 3 },
    { 0xc53abebc, 3 },
    { 0xc3e2bebf, 3 },
    { 0xe376bebf, 3 },
    { 0x12ebec1, 3 },
    { 0x5ffabec2, 4 },
    { 0, 0 },
    { 0x3886bec4, 3 },
    { 0x423abec4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd2beca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846becf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6336bed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9abedb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400abedf, 4 },
    { 0, 0 },
    { 0xbe62bee1, 3 },
    { 0xc462bee2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabee5, 4 },
    { 0x8856bee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56beea, 3 },
    { 0, 0 },
    { 0xd71abeec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feabef2, 3 },
    { 0xf14abef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aabef7, 3 },
    { 0xf24ebef8, 4 },
    { 0xfe7ebef9, 3 },
    { 0x837abefa, 3 },
    { 0xe66befb, 3 },
    { 0x7862befa, 3 },
    { 0xb992befc, 4 },
    { 0x483ebef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e6bf02, 5 },
    { 0x7236bf03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfebf07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ebf0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb416bf11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ebf16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2bf1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127abf22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x828abf25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b6bf2c, 3 },
    { 0x8a46bf2d, 3 },
    { 0x77febf2c, 3 },
    { 0xd792bf2e, 4 },
    { 0xaccabf30, 3 },
    { 0, 0 },
    { 0x3d5abf32, 3 },
    { 0, 0 },
    { 0x256ebf34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eabf37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cebf3f, 3 },
    { 0x3aaabf40, 3 },
    { 0xe066bf40, 4 },
    { 0xf632bf40, 3 },
    { 0x222ebf42, 3 },
    { 0x4c06bf44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceebf49, 3 },
    { 0x3ebebf4a, 3 },
    { 0x1eb6bf4b, 4 },
    { 0, 0 },
    { 0x9b66bf4d, 4 },
    { 0x5c2ebf4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0debf53, 3 },
    { 0xbf92bf54, 3 },
    { 0, 0 },
    { 0xa24abf56, 3 },
    { 0xbeeabf57, 3 },
    { 0xe0fabf58, 3 },
    { 0x9e8ebf59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cebf62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd6bf66, 4 },
    { 0x6416bf67, 4 },
    { 0, 0 },
    { 0xccbebf69, 3 },
    { 0x24cabf69, 3 },
    { 0x99d2bf6b, 3 },
    { 0xef8ebf6b, 3 },
    { 0x6352bf6b, 3 },
    { 0xaf6ebf6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6012bf71, 3 },
    { 0x7d46bf72, 3 },
    { 0xd7f2bf72, 4 },
    { 0xeb3ebf73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9186bf7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8ebf81, 3 },
    { 0x5716bf81, 3 },
    { 0, 0 },
    { 0xfd6ebf84, 4 },
    { 0, 0 },
    { 0xf33ebf86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf6bf89, 3 },
    { 0xcd3ebf8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e06bf8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2462bf91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769ebf97, 4 },
    { 0x77febf98, 4 },
    { 0, 0 },
    { 0x34ebf9a, 4 },
    { 0xd0cebf9a, 5 },
    { 0x3b7ebf9b, 3 },
    { 0, 0 },
    { 0x8cbabf9e, 3 },
    { 0x57d2bf9f, 3 },
    { 0xb1eabf9e, 3 },
    { 0x792bfa1, 4 },
    { 0xf5c2bfa1, 3 },
    { 0x5cdabfa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b6bfaa, 3 },
    { 0, 0 },
    { 0x22b6bfac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494abfb1, 3 },
    { 0, 0 },
    { 0x5e6abfb3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x466abfb6, 3 },
    { 0x255abfb7, 3 },
    { 0xa4aabfb8, 3 },
    { 0x41cabfb9, 3 },
    { 0x5ce2bfba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32bfbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b66bfc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c56bfc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa2bfcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf302bfd0, 3 },
    { 0, 0 },
    { 0xf7abfd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26bfda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82abfdd, 4 },
    { 0, 0 },
    { 0x85fabfdf, 3 },
    { 0x2502bfdf, 3 },
    { 0x1b52bfe1, 3 },
    { 0x5212bfdf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b6bfe7, 3 },
    { 0xfb9abfe7, 4 },
    { 0x302ebfe9, 3 },
    { 0x23e6bfea, 3 },
    { 0, 0 },
    { 0x3f2ebfec, 4 },
    { 0xe49ebfec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43ebff0, 3 },
    { 0, 0 },
    { 0xbb56bff2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc806bff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aabffe, 4 },
    { 0x4466bfff, 4 },
    { 0xfe16bfff, 3 },
    { 0x4ac2c001, 3 },
    { 0, 0 },
    { 0xcdfec003, 3 },
    { 0xf42ec004, 4 },
    { 0xade2c005, 4 },
    { 0x80ec006, 3 },
    { 0, 0 },
    { 0x9176c008, 3 },
    { 0, 0 },
    { 0xdc32c00a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac00e, 4 },
    { 0x51c6c00e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8112c023, 3 },
    { 0xa3cac023, 4 },
    { 0, 0 },
    { 0x5b3ec026, 3 },
    { 0xf796c026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bec02c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ec030, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b2c035, 3 },
    { 0x1a26c036, 4 },
    { 0xa392c036, 3 },
    { 0x77aac038, 4 },
    { 0x5206c039, 3 },
    { 0, 0 },
    { 0xfe9ac03b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2c03f, 4 },
    { 0x84a6c040, 4 },
    { 0x7332c041, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x704ac044, 3 },
    { 0x8edec044, 3 },
    { 0xbaa6c044, 3 },
    { 0, 0 },
    { 0x3136c048, 3 },
    { 0x8c66c049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeac04d, 3 },
    { 0, 0 },
    { 0x3c3ec04f, 4 },
    { 0, 0 },
    { 0x2a52c051, 4 },
    { 0, 0 },
    { 0x5e56c053, 3 },
    { 0x1a96c054, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d6c05a, 3 },
    { 0, 0 },
    { 0xeeb2c05c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc6c05f, 3 },
    { 0xde12c060, 3 },
    { 0, 0 },
    { 0x2ac6c062, 3 },
    { 0x4cc2c063, 3 },
    { 0xbae2c064, 4 },
    { 0x156c062, 3 },
    { 0xff0ac064, 3 },
    { 0xe97ec064, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60ac06c, 3 },
    { 0xc4b2c06d, 3 },
    { 0xd86ac06d, 3 },
    { 0x579ac06f, 3 },
    { 0x9422c06f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109ac074, 3 },
    { 0xb0ec074, 3 },
    { 0x6922c075, 3 },
    { 0x2c1ec077, 3 },
    { 0xf6aec078, 3 },
    { 0, 0 },
    { 0x47c6c07a, 4 },
    { 0xf2fac07b, 3 },
    { 0x102c07b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5ec080, 4 },
    { 0x13aac081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec6c089, 3 },
    { 0, 0 },
    { 0xbc66c08b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5ec091, 4 },
    { 0, 0 },
    { 0xa7d6c093, 3 },
    { 0xa8aac093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88ec09b, 3 },
    { 0xf872c09c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb6c0a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8866c0a5, 4 },
    { 0, 0 },
    { 0x1626c0a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed2c0b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52ac0b7, 3 },
    { 0x3362c0b8, 3 },
    { 0x344ac0b9, 3 },
    { 0xc76c0ba, 3 },
    { 0x49bac0ba, 4 },
    { 0xab9ac0bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7ac0bf, 3 },
    { 0x5c6ec0c0, 4 },
    { 0x14aac0c1, 3 },
    { 0, 0 },
    { 0x52aec0c3, 3 },
    { 0xe32ec0c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f22c0ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2862c0cf, 3 },
    { 0, 0 },
    { 0xc086c0d1, 4 },
    { 0, 0 },
    { 0x48aec0d3, 3 },
    { 0x99f6c0d4, 3 },
    { 0x859ec0d5, 3 },
    { 0xfd6c0d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8ac0db, 3 },
    { 0, 0 },
    { 0x253ac0dd, 3 },
    { 0xc92ac0dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ec0e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7662c0e4, 3 },
    { 0x9b1ec0e5, 4 },
    { 0xf9cec0e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebac0e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daec0ec, 4 },
    { 0x4dfec0ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bec0f0, 3 },
    { 0x547ec0f0, 3 },
    { 0xebf2c0f2, 3 },
    { 0x1f36c0f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646c0fd, 3 },
    { 0, 0 },
    { 0xe96ac0ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd6c103, 4 },
    { 0, 0 },
    { 0xf722c105, 3 },
    { 0, 0 },
    { 0x15ac107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b2c10c, 3 },
    { 0x566c10d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fac111, 3 },
    { 0x6a32c112, 3 },
    { 0xd6f2c113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4032c11c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f2c122, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a76c128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc212c12b, 3 },
    { 0x4176c12c, 4 },
    { 0xaffac12d, 3 },
    { 0x80aac12e, 3 },
    { 0x46e6c12e, 3 },
    { 0xfecec12c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066c134, 3 },
    { 0x36b2c135, 3 },
    { 0xf5b6c134, 3 },
    { 0x3006c135, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d62c13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2c142, 3 },
    { 0xaca6c143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a2c14a, 3 },
    { 0, 0 },
    { 0x2212c14c, 3 },
    { 0xd062c14c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9962c152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abec155, 3 },
    { 0, 0 },
    { 0x67c6c157, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3ec15a, 4 },
    { 0x2a9ec15a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d6c15f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd852c163, 3 },
    { 0x8d6ec164, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8ec169, 3 },
    { 0xf5eec16a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1202c171, 3 },
    { 0, 0 },
    { 0x9d22c173, 3 },
    { 0x4a8ac173, 4 },
    { 0x747ec173, 3 },
    { 0x8a0ac176, 4 },
    { 0xcdfac176, 3 },
    { 0xcefec176, 3 },
    { 0x6f62c176, 4 },
    { 0x8f0ec173, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13ac17d, 4 },
    { 0, 0 },
    { 0x246ac17f, 3 },
    { 0, 0 },
    { 0xe6c6c181, 3 },
    { 0x9482c182, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac186, 3 },
    { 0x500ec186, 3 },
    { 0x63c6c188, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0ac18b, 4 },
    { 0xabcac18c, 3 },
    { 0x30a6c18c, 3 },
    { 0x7bfec18e, 3 },
    { 0x7b3ec18b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba82c193, 3 },
    { 0x2b26c193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aac198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5316c19c, 3 },
    { 0x83fec19d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eac1a7, 3 },
    { 0xeba2c1a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ec1ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9742c1b0, 4 },
    { 0, 0 },
    { 0x279ec1b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bec1b6, 3 },
    { 0xcce2c1b6, 3 },
    { 0x21f2c1b8, 3 },
    { 0xab62c1b9, 3 },
    { 0xfa5ac1b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6496c1c0, 3 },
    { 0xde8ac1c1, 3 },
    { 0x7942c1c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ac1c9, 3 },
    { 0x1e76c1ca, 3 },
    { 0, 0 },
    { 0x5306c1cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b66c1cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592c1d2, 3 },
    { 0xf432c1d2, 3 },
    { 0x40f2c1d4, 3 },
    { 0x8586c1d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f6c1dd, 4 },
    { 0xb98ec1de, 3 },
    { 0, 0 },
    { 0xe57ac1e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7792c1e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafec1e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a6c1ed, 3 },
    { 0, 0 },
    { 0xe762c1ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdac1f3, 3 },
    { 0x681ec1f4, 5 },
    { 0xea6c1f5, 4 },
    { 0x2ec1f5, 3 },
    { 0x8cbac1f6, 3 },
    { 0x600ac1f8, 4 },
    { 0x80e2c1f4, 4 },
    { 0xfa62c1fa, 3 },
    { 0x3186c1f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c02c201, 3 },
    { 0x692ec201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399ec20c, 4 },
    { 0x7f6c20d, 3 },
    { 0xc592c20d, 3 },
    { 0xd54ec20f, 3 },
    { 0x6a1ac210, 5 },
    { 0x1cc2c211, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5ac218, 4 },
    { 0, 0 },
    { 0x8d46c21a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293ec21e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x964ac221, 5 },
    { 0, 0 },
    { 0x2aeac223, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb456c229, 4 },
    { 0xcb6ac22a, 3 },
    { 0xdf7ec22b, 3 },
    { 0, 0 },
    { 0xb666c22d, 3 },
    { 0xddf2c22e, 3 },
    { 0, 0 },
    { 0x45aac230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eec236, 3 },
    { 0xfb6c236, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6476c23e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcac241, 3 },
    { 0x8e22c241, 3 },
    { 0x7f7ec242, 3 },
    { 0xb466c244, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146c248, 3 },
    { 0, 0 },
    { 0xe58ec24a, 3 },
    { 0x6adec24a, 4 },
    { 0x6c6c24c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062c24f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdac252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ac25b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc2c25f, 4 },
    { 0xde2c260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ac266, 3 },
    { 0xfc6c266, 3 },
    { 0, 0 },
    { 0x2ccec269, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eac26c, 4 },
    { 0, 0 },
    { 0x4042c26e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd2c272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8ec276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c6c27a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bec27e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d6c289, 3 },
    { 0x8e22c289, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x822ac28d, 3 },
    { 0, 0 },
    { 0x687ac28f, 3 },
    { 0xb86ec290, 5 },
    { 0xc31ec291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af2c296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b32c29b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eec29e, 3 },
    { 0, 0 },
    { 0x522c2a0, 4 },
    { 0xaec2a1, 4 },
    { 0xe69ac2a1, 3 },
    { 0xbb9ec2a3, 3 },
    { 0x9a16c2a3, 4 },
    { 0xac72c2a3, 4 },
    { 0x952ac2a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b6c2ab, 3 },
    { 0xfa5ac2ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e2c2b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ec2b6, 4 },
    { 0x4342c2b6, 4 },
    { 0xf462c2b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ac2bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa992c2c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56c2c6, 3 },
    { 0x79cec2c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eac2cd, 3 },
    { 0xaa62c2ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bec2d2, 3 },
    { 0, 0 },
    { 0xa12ec2d4, 3 },
    { 0xd81ec2d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fec2d8, 3 },
    { 0x966ac2d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa2c2de, 3 },
    { 0, 0 },
    { 0x737ec2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb2c2e3, 3 },
    { 0, 0 },
    { 0x29f2c2e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52c2ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9ac2f8, 3 },
    { 0, 0 },
    { 0x5312c2fa, 3 },
    { 0xd43ac2fb, 3 },
    { 0xcb9ec2fc, 3 },
    { 0x2f52c2fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbac300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ec307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fac30c, 3 },
    { 0xa3eac30c, 3 },
    { 0x9d06c30e, 3 },
    { 0xead2c30f, 3 },
    { 0xb5c6c310, 3 },
    { 0x5f6ec311, 5 },
    { 0x6ae2c30d, 3 },
    { 0xd662c313, 3 },
    { 0x5f12c314, 3 },
    { 0xfd6ac313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb782c31a, 4 },
    { 0x3e82c31a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9122c320, 3 },
    { 0x6f1ac321, 4 },
    { 0x8c1ec321, 4 },
    { 0x7896c321, 4 },
    { 0xd4ac324, 4 },
    { 0x6a6ac325, 4 },
    { 0x9b86c324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f2c329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274ec333, 4 },
    { 0, 0 },
    { 0xdbeec335, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f72c33a, 3 },
    { 0x74eac33b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48ac33e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e2c349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d6c34c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5ac353, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eac35c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb16c35f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf546c365, 3 },
    { 0xf6d6c366, 3 },
    { 0xbb42c365, 3 },
    { 0x640ac368, 3 },
    { 0x74cec369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64ec373, 3 },
    { 0x7ec6c374, 4 },
    { 0, 0 },
    { 0x2ffac376, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6eac37b, 3 },
    { 0x8d0ec37b, 3 },
    { 0, 0 },
    { 0x42d2c37e, 3 },
    { 0xbbf2c37f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe6c382, 3 },
    { 0, 0 },
    { 0xae2ac384, 4 },
    { 0, 0 },
    { 0x62e2c386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf766c389, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ec38e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3ac395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2ec398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa516c39c, 3 },
    { 0xf2c39d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9912c3a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966c3a5, 3 },
    { 0, 0 },
    { 0x77a2c3a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f36c3ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbac3b6, 3 },
    { 0x4d86c3b6, 5 },
    { 0xc9f2c3b8, 4 },
    { 0x6082c3b9, 5 },
    { 0x1726c3b6, 3 },
    { 0xe7b2c3b8, 3 },
    { 0, 0 },
    { 0x8042c3bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dec3c0, 4 },
    { 0x698ac3c1, 4 },
    { 0xbf0ac3c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afec3c5, 3 },
    { 0x2912c3c6, 3 },
    { 0xc792c3c6, 4 },
    { 0xb66ac3c8, 4 },
    { 0, 0 },
    { 0x9ddec3ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c3cd, 4 },
    { 0, 0 },
    { 0xefaec3cf, 3 },
    { 0x2e86c3d0, 3 },
    { 0x5012c3d1, 3 },
    { 0xde46c3d2, 3 },
    { 0x1ecac3d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0ec3d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457ac3df, 5 },
    { 0xc36ac3df, 3 },
    { 0x2f26c3df, 3 },
    { 0, 0 },
    { 0x8b56c3e3, 4 },
    { 0xb35ec3e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fac3e7, 3 },
    { 0x857ac3e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6ac3f1, 3 },
    { 0x421ac3f1, 3 },
    { 0x6672c3f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eec3f8, 4 },
    { 0, 0 },
    { 0xd606c3fa, 3 },
    { 0x6676c3fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a6c400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9ec407, 3 },
    { 0x126ec408, 4 },
    { 0x778ec409, 4 },
    { 0x1d2ec409, 3 },
    { 0xb0a6c40a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c2c413, 3 },
    { 0x331ec413, 3 },
    { 0x4e22c414, 5 },
    { 0, 0 },
    { 0xf16c417, 3 },
    { 0xfd9ac418, 3 },
    { 0x6e82c419, 3 },
    { 0x2d0ec41a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7766c41d, 3 },
    { 0x91f6c41d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaec425, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2c429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeec42e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645ec438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcec43d, 3 },
    { 0, 0 },
    { 0xf81ec43f, 3 },
    { 0xb28ec440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512c446, 3 },
    { 0x8eaac446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4546c44d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc632c451, 3 },
    { 0x99cec451, 4 },
    { 0, 0 },
    { 0x7192c454, 4 },
    { 0x2a76c454, 3 },
    { 0xfeaec456, 3 },
    { 0x3382c457, 3 },
    { 0x2932c457, 3 },
    { 0x8766c455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ac45e, 3 },
    { 0xfb1ec45e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8ac462, 3 },
    { 0x7356c463, 3 },
    { 0x22eec462, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa6c46b, 3 },
    { 0, 0 },
    { 0x6fd2c46d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de2c471, 3 },
    { 0x717ec472, 4 },
    { 0x926ac473, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccac480, 3 },
    { 0xa24ec481, 3 },
    { 0, 0 },
    { 0x6d06c483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92ec48a, 3 },
    { 0xa216c48b, 4 },
    { 0x9816c48a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ac492, 4 },
    { 0, 0 },
    { 0x8eb2c494, 3 },
    { 0xab3ec494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb566c498, 4 },
    { 0xbff6c498, 3 },
    { 0xf16c49a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1832c49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abac4a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cec4a4, 4 },
    { 0xd26ec4a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91ec4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x820ec4ab, 4 },
    { 0x81e6c4ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8aac4b1, 4 },
    { 0xb1fac4b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efac4bd, 3 },
    { 0x957ac4be, 4 },
    { 0x7822c4bf, 3 },
    { 0x3072c4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbec4c3, 3 },
    { 0xfba2c4c4, 3 },
    { 0x4566c4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb942c4c8, 3 },
    { 0x8f62c4c9, 3 },
    { 0xb70ac4ca, 3 },
    { 0x4ccec4cb, 3 },
    { 0xc23ec4cc, 3 },
    { 0x3c0ac4c9, 3 },
    { 0x9a6ec4c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722c4db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152ac4df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba82c4e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a2c4e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dac4ea, 4 },
    { 0, 0 },
    { 0x14c6c4ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f16c4f0, 4 },
    { 0xcc66c4f1, 3 },
    { 0x55bec4f2, 4 },
    { 0, 0 },
    { 0x5b3ac4f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c82c4f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e02c501, 4 },
    { 0x40e6c501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ac506, 3 },
    { 0, 0 },
    { 0x54eac508, 3 },
    { 0x5092c508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec50d, 3 },
    { 0xa67ac50e, 4 },
    { 0x2162c50e, 3 },
    { 0, 0 },
    { 0x2786c511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99ec514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcec517, 3 },
    { 0xe8bec517, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b02c51c, 3 },
    { 0xba9ec51d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32c520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eec523, 3 },
    { 0xc32ac524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dec52b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a6c534, 3 },
    { 0x9d1ac535, 4 },
    { 0, 0 },
    { 0x6036c537, 3 },
    { 0, 0 },
    { 0xdce2c539, 3 },
    { 0x32ec53a, 3 },
    { 0xe93ec53b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d26c542, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276c547, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ec54f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcec557, 5 },
    { 0xb996c558, 4 },
    { 0x15e6c559, 3 },
    { 0xe09ac559, 3 },
    { 0x906ac55b, 3 },
    { 0, 0 },
    { 0xe052c55d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f82c563, 3 },
    { 0xbca6c564, 3 },
    { 0xaa92c563, 3 },
    { 0xcc82c563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb2c56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eec56d, 3 },
    { 0, 0 },
    { 0x48aac56f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774ac573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436c578, 4 },
    { 0, 0 },
    { 0xfa2ec57a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a2c589, 3 },
    { 0, 0 },
    { 0x2956c58b, 3 },
    { 0, 0 },
    { 0xfe82c58d, 3 },
    { 0x6e82c58e, 3 },
    { 0x3352c58f, 3 },
    { 0, 0 },
    { 0x48c6c591, 3 },
    { 0xbbcec592, 4 },
    { 0, 0 },
    { 0xdbf2c594, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bec598, 4 },
    { 0xa016c598, 4 },
    { 0, 0 },
    { 0x7936c59b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefec5a3, 5 },
    { 0x230ec5a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3ac5a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7352c5ab, 3 },
    { 0, 0 },
    { 0x8f7ac5ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83ec5b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fac5b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acec5bc, 3 },
    { 0, 0 },
    { 0xf442c5be, 3 },
    { 0xb0b6c5bf, 4 },
    { 0xaa76c5bf, 3 },
    { 0x4b2ec5c1, 3 },
    { 0, 0 },
    { 0x98cec5c3, 4 },
    { 0, 0 },
    { 0xd8d6c5c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a6c5c8, 3 },
    { 0x7216c5c8, 3 },
    { 0x1672c5ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c2c5cf, 4 },
    { 0xbf52c5d0, 3 },
    { 0, 0 },
    { 0x866ec5d2, 4 },
    { 0xa886c5d3, 3 },
    { 0x6182c5d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fac5d7, 3 },
    { 0xb666c5d7, 3 },
    { 0, 0 },
    { 0xdccec5da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802c5de, 4 },
    { 0, 0 },
    { 0x3fec5e0, 4 },
    { 0, 0 },
    { 0x1672c5e2, 3 },
    { 0, 0 },
    { 0xecb6c5e4, 3 },
    { 0x653ac5e5, 4 },
    { 0x3332c5e6, 4 },
    { 0x19e2c5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583ec5ec, 3 },
    { 0xe3a2c5ec, 3 },
    { 0, 0 },
    { 0x5722c5ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acec5f3, 3 },
    { 0, 0 },
    { 0xb522c5f5, 3 },
    { 0x83c2c5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2ec5fa, 3 },
    { 0xcbfac5fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d22c601, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57ac605, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdac608, 4 },
    { 0x908ac609, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fec60c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa662c610, 3 },
    { 0x18dac611, 3 },
    { 0xb9dac612, 3 },
    { 0, 0 },
    { 0xb692c614, 3 },
    { 0x5ed6c614, 3 },
    { 0x3eb2c615, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b2c61d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7ac620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad56c624, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48eec62c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40ec632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18ec638, 3 },
    { 0x3f92c639, 3 },
    { 0xca8ac639, 3 },
    { 0x2dac638, 3 },
    { 0x678ec638, 3 },
    { 0x516ac63d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf676c642, 3 },
    { 0x1a8ec642, 3 },
    { 0, 0 },
    { 0xe52ec645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac2c649, 3 },
    { 0, 0 },
    { 0x4c1ec64b, 4 },
    { 0, 0 },
    { 0xd06ec64d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40ec652, 3 },
    { 0, 0 },
    { 0x7edec654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282c65b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03ec65f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a02c666, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b2c669, 3 },
    { 0, 0 },
    { 0xf41ec66b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb6c670, 3 },
    { 0xe702c670, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafec674, 3 },
    { 0xae02c675, 3 },
    { 0x7656c675, 3 },
    { 0xf462c675, 3 },
    { 0, 0 },
    { 0x70bec679, 3 },
    { 0, 0 },
    { 0xf922c67b, 3 },
    { 0x180ec67c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2c681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8aac687, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16c68c, 3 },
    { 0xc3dec68d, 3 },
    { 0x2b3ec68e, 3 },
    { 0xf00ac68f, 5 },
    { 0xea62c68f, 3 },
    { 0xb112c691, 3 },
    { 0xca3ac690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201ac697, 3 },
    { 0, 0 },
    { 0xd74ac699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ac69e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ac6a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd996c6a6, 3 },
    { 0, 0 },
    { 0xec22c6a8, 3 },
    { 0xb70ec6a9, 4 },
    { 0x431ac6a8, 4 },
    { 0x8dd2c6ab, 3 },
    { 0xfec6ac, 3 },
    { 0x42a2c6ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6ac6b1, 5 },
    { 0x8a06c6b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8ec6b6, 5 },
    { 0, 0 },
    { 0x550ec6b8, 3 },
    { 0x774ac6b9, 3 },
    { 0x79a2c6b9, 3 },
    { 0, 0 },
    { 0xd44ac6bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd426c6bf, 3 },
    { 0, 0 },
    { 0x913ec6c1, 3 },
    { 0, 0 },
    { 0x4912c6c3, 3 },
    { 0, 0 },
    { 0xbd16c6c5, 4 },
    { 0x9caac6c6, 4 },
    { 0xd96ac6c6, 3 },
    { 0, 0 },
    { 0x61dac6c9, 3 },
    { 0x3662c6ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec6d1, 3 },
    { 0xfc62c6d1, 3 },
    { 0xb9aac6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb2c6de, 3 },
    { 0, 0 },
    { 0xe262c6e0, 3 },
    { 0x575ac6e0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2c6e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5436c6e8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e2c6ed, 4 },
    { 0x91d6c6ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6ec6f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d6c6f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7ac6f7, 4 },
    { 0x2ba6c6f8, 3 },
    { 0x930ac6f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72ec6fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974ac703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9ac709, 3 },
    { 0xe2aac709, 3 },
    { 0, 0 },
    { 0x61cec70c, 3 },
    { 0xdf02c70d, 3 },
    { 0, 0 },
    { 0x138ec70f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1456c712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de6c71d, 4 },
    { 0xb416c71e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b56c722, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d6c72c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdec730, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e52c733, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc66c738, 3 },
    { 0x9dc2c738, 3 },
    { 0xf74ac73a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aec73d, 3 },
    { 0x5f62c73e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396c745, 4 },
    { 0, 0 },
    { 0xde2ec747, 3 },
    { 0x233ac748, 3 },
    { 0x9be6c749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ac74f, 4 },
    { 0x6ceac750, 3 },
    { 0x96fac751, 3 },
    { 0x960ac752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x359ac755, 4 },
    { 0xea3ac755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adec75c, 4 },
    { 0xd886c75d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ec764, 3 },
    { 0x1972c765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaec769, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6c770, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a2c776, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccec77a, 4 },
    { 0xd4f2c77b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ac784, 3 },
    { 0, 0 },
    { 0xa292c786, 3 },
    { 0x889ec787, 3 },
    { 0, 0 },
    { 0xd882c789, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7ac78c, 3 },
    { 0x70b2c78d, 3 },
    { 0x3ec2c78d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666c791, 3 },
    { 0x18c6c791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86c797, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x726ac79a, 3 },
    { 0, 0 },
    { 0x6d5ac79c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1082c7a0, 5 },
    { 0x46b2c7a1, 3 },
    { 0, 0 },
    { 0xd336c7a3, 4 },
    { 0x7996c7a4, 4 },
    { 0xf842c7a5, 4 },
    { 0xe19ac7a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b26c7a9, 3 },
    { 0x873ac7aa, 4 },
    { 0xc1a2c7aa, 3 },
    { 0, 0 },
    { 0x8926c7ad, 3 },
    { 0x1f6ac7ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0ac7b2, 3 },
    { 0x9caec7b3, 3 },
    { 0, 0 },
    { 0x5d2c7b5, 3 },
    { 0, 0 },
    { 0x23d6c7b7, 3 },
    { 0xa6aec7b8, 4 },
    { 0x6732c7b8, 3 },
    { 0x6c6ec7ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fac7bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372c7c4, 3 },
    { 0x6376c7c5, 4 },
    { 0xb57ac7c5, 3 },
    { 0xb52ec7c5, 3 },
    { 0, 0 },
    { 0x286c7c9, 3 },
    { 0x9d5ac7ca, 5 },
    { 0x9fc6c7ca, 3 },
    { 0, 0 },
    { 0xbfa2c7cd, 4 },
    { 0xdaaac7ce, 3 },
    { 0xaa9ac7cf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ac7d3, 3 },
    { 0x9676c7d4, 4 },
    { 0x46c2c7d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb62c7de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc2c7e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ac7ec, 3 },
    { 0x2966c7ed, 3 },
    { 0x38dac7ee, 3 },
    { 0x40a2c7ee, 3 },
    { 0x17f2c7f0, 3 },
    { 0, 0 },
    { 0x8cec7f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ec7f5, 4 },
    { 0, 0 },
    { 0x83c2c7f7, 3 },
    { 0x7d0ec7f8, 4 },
    { 0xe26ec7f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb972c7fc, 3 },
    { 0x5dc2c7fc, 3 },
    { 0xff6ac7fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da6c801, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfec804, 3 },
    { 0, 0 },
    { 0x6acec806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6ec80c, 3 },
    { 0xda3ec80c, 3 },
    { 0x55b2c80c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefac811, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfac818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ec820, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda2c827, 3 },
    { 0, 0 },
    { 0x194ec829, 3 },
    { 0xf33ec829, 3 },
    { 0xa37ec82b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc576c82e, 4 },
    { 0, 0 },
    { 0x72c6c830, 3 },
    { 0x3016c831, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8526c835, 4 },
    { 0x6d36c835, 4 },
    { 0, 0 },
    { 0x4692c838, 4 },
    { 0, 0 },
    { 0xa072c83a, 3 },
    { 0, 0 },
    { 0x2266c83c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d6c840, 3 },
    { 0x80dac840, 3 },
    { 0xfe76c842, 3 },
    { 0x62f6c840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7582c846, 3 },
    { 0xa482c846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3ec84a, 3 },
    { 0, 0 },
    { 0x678ac84c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968ac850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf26c855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb6c85a, 4 },
    { 0x623ac85b, 3 },
    { 0xc74ac85c, 3 },
    { 0xbf9ac85b, 3 },
    { 0x746c85c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c2c861, 3 },
    { 0x2f6ac862, 3 },
    { 0x8a9ec863, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46c868, 3 },
    { 0xfeec869, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abac86d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33ec872, 3 },
    { 0, 0 },
    { 0xd506c874, 3 },
    { 0xe4d2c874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1ec879, 3 },
    { 0xb06ec87a, 5 },
    { 0x33e6c87b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x167ac87e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af6c886, 3 },
    { 0x9a32c887, 3 },
    { 0, 0 },
    { 0xb8cac889, 3 },
    { 0xbc6c88a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dec893, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa566c896, 3 },
    { 0x8cf6c897, 4 },
    { 0x9cd2c898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfec89d, 3 },
    { 0xca1ac89e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dec8a1, 4 },
    { 0xc47ec8a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef6c8a5, 4 },
    { 0x8866c8a5, 3 },
    { 0x51c2c8a7, 3 },
    { 0x1cfec8a8, 3 },
    { 0, 0 },
    { 0xa6ac8aa, 3 },
    { 0xd59ec8ab, 3 },
    { 0x771ac8ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fec8af, 3 },
    { 0, 0 },
    { 0xc86c8b1, 3 },
    { 0, 0 },
    { 0x483ac8b3, 4 },
    { 0x352ec8b3, 3 },
    { 0x4302c8b3, 3 },
    { 0x3c22c8b6, 3 },
    { 0xdddec8b6, 3 },
    { 0xd726c8b8, 3 },
    { 0, 0 },
    { 0xa6f6c8ba, 3 },
    { 0x4112c8ba, 3 },
    { 0xab26c8bc, 4 },
    { 0xc3ac8ba, 3 },
    { 0, 0 },
    { 0x6bbac8bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce66c8ce, 3 },
    { 0, 0 },
    { 0xe8eec8d0, 4 },
    { 0x3a4ec8d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c6c8d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa72c8dc, 3 },
    { 0, 0 },
    { 0xc13ec8de, 4 },
    { 0xae26c8df, 3 },
    { 0x722ec8df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaec8e3, 3 },
    { 0xdb2ac8e3, 3 },
    { 0x58ac8e4, 3 },
    { 0x5feec8e6, 3 },
    { 0x5076c8e7, 3 },
    { 0x72ac8e3, 3 },
    { 0, 0 },
    { 0x54e2c8ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bec8ed, 4 },
    { 0x9c32c8ee, 3 },
    { 0xea82c8ef, 3 },
    { 0, 0 },
    { 0x90f6c8f1, 3 },
    { 0x6c1ac8f2, 3 },
    { 0, 0 },
    { 0xab9ec8f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2c8f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ec8fc, 4 },
    { 0, 0 },
    { 0xb8dac8fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acec903, 3 },
    { 0x3d76c904, 4 },
    { 0x1bcac903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfec913, 3 },
    { 0xf712c913, 4 },
    { 0xf84ec913, 3 },
    { 0xc556c913, 4 },
    { 0xa6b6c916, 3 },
    { 0x55e2c917, 3 },
    { 0x8b6ac913, 3 },
    { 0, 0 },
    { 0x5b5ec91b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa542c91f, 3 },
    { 0x383ec920, 3 },
    { 0, 0 },
    { 0xd072c922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2ac927, 3 },
    { 0, 0 },
    { 0xa312c929, 3 },
    { 0xe9cac92a, 3 },
    { 0xdec92b, 3 },
    { 0xa5ec92c, 3 },
    { 0xcce2c92d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6c930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa932c935, 3 },
    { 0xc896c936, 3 },
    { 0x289ac937, 4 },
    { 0xb8b6c938, 5 },
    { 0, 0 },
    { 0x96e6c93a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x226ac93d, 3 },
    { 0, 0 },
    { 0xd8a2c93f, 3 },
    { 0xcd22c93f, 3 },
    { 0, 0 },
    { 0x70a6c942, 3 },
    { 0xa396c943, 3 },
    { 0xf9cec944, 3 },
    { 0xc402c944, 3 },
    { 0x6d3ec945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd116c94a, 3 },
    { 0, 0 },
    { 0x6782c94c, 3 },
    { 0xb412c94c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dac953, 4 },
    { 0x3acec954, 3 },
    { 0xcbbec955, 3 },
    { 0x1786c956, 3 },
    { 0xadd6c957, 4 },
    { 0x8ca2c958, 3 },
    { 0xaff6c959, 3 },
    { 0, 0 },
    { 0x6afec95b, 3 },
    { 0x57c2c95c, 4 },
    { 0x7166c95c, 3 },
    { 0x6d2ec95e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10ec967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fec96b, 4 },
    { 0x624ac96b, 3 },
    { 0xb066c96c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ec971, 4 },
    { 0xa3eac972, 3 },
    { 0x1f32c971, 3 },
    { 0x5d5ac974, 3 },
    { 0x76fec973, 3 },
    { 0xbee6c975, 4 },
    { 0x3ca6c974, 4 },
    { 0xa77ac978, 3 },
    { 0x78dec979, 3 },
    { 0, 0 },
    { 0xb9aac97b, 4 },
    { 0x571ec97c, 3 },
    { 0xc67ec97b, 3 },
    { 0, 0 },
    { 0x3a36c97f, 3 },
    { 0, 0 },
    { 0xdfb2c981, 4 },
    { 0, 0 },
    { 0x7726c983, 3 },
    { 0, 0 },
    { 0x5f7ac985, 3 },
    { 0, 0 },
    { 0xe4e6c987, 3 },
    { 0x3a1ec987, 4 },
    { 0x9a1ac987, 4 },
    { 0, 0 },
    { 0x9c02c98b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315ec98f, 3 },
    { 0xfb4ac990, 3 },
    { 0x553ac991, 3 },
    { 0, 0 },
    { 0xf12c993, 5 },
    { 0x8976c994, 4 },
    { 0, 0 },
    { 0x6976c996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ec99a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ac99e, 3 },
    { 0, 0 },
    { 0x21dac9a0, 4 },
    { 0x382ac9a1, 3 },
    { 0x4dc2c9a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x224ec9a5, 3 },
    { 0x50fec9a5, 3 },
    { 0xc9f2c9a7, 3 },
    { 0xdba6c9a8, 3 },
    { 0xc43ec9a9, 3 },
    { 0xf8a2c9a5, 3 },
    { 0x5f76c9a9, 3 },
    { 0, 0 },
    { 0x1362c9ad, 4 },
    { 0x62eac9ae, 3 },
    { 0xe872c9af, 5 },
    { 0xbe86c9af, 3 },
    { 0, 0 },
    { 0xa356c9b2, 5 },
    { 0x2c2c9b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989ec9b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d72c9c2, 3 },
    { 0xd982c9c3, 3 },
    { 0xaef6c9c2, 3 },
    { 0x907ac9c4, 3 },
    { 0x5d26c9c6, 3 },
    { 0xd3eec9c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a32c9cc, 3 },
    { 0x792ec9cd, 4 },
    { 0xa57ac9ce, 3 },
    { 0xcbd6c9ce, 4 },
    { 0xbfa2c9ce, 3 },
    { 0xb1b2c9d1, 3 },
    { 0xe002c9d2, 3 },
    { 0xd522c9cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d2c9d6, 3 },
    { 0, 0 },
    { 0x26ac9d8, 3 },
    { 0x82d6c9d9, 3 },
    { 0, 0 },
    { 0x724ec9db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972c9de, 3 },
    { 0xca5ac9df, 3 },
    { 0xc7bac9e0, 4 },
    { 0x8c4ac9de, 3 },
    { 0x714ac9e0, 3 },
    { 0, 0 },
    { 0x206ac9e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffac9e7, 3 },
    { 0x586ac9e8, 4 },
    { 0xc1d2c9e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c9ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab82c9f5, 3 },
    { 0x23bac9f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ac9f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e46ca03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7802ca07, 4 },
    { 0xf3aca08, 4 },
    { 0x898aca07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538eca0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2722ca10, 3 },
    { 0xe9c6ca10, 4 },
    { 0x426ca12, 4 },
    { 0x49c6ca13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5eca16, 3 },
    { 0x8152ca17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b6ca23, 3 },
    { 0, 0 },
    { 0xbdb2ca25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d06ca36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba2ca3b, 3 },
    { 0x58d2ca3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761aca49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcaca4c, 3 },
    { 0, 0 },
    { 0x9376ca4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3eca52, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xce12ca55, 3 },
    { 0x724eca55, 3 },
    { 0x2146ca57, 4 },
    { 0x19beca58, 3 },
    { 0x1f8aca57, 3 },
    { 0xf81eca5a, 3 },
    { 0xe8a2ca59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803aca5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e2ca6f, 3 },
    { 0x2d1aca6f, 3 },
    { 0, 0 },
    { 0xf982ca72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219eca7e, 3 },
    { 0xf946ca7f, 3 },
    { 0x4e2eca7f, 3 },
    { 0xca0eca7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852ca85, 3 },
    { 0xec6eca86, 3 },
    { 0x679aca87, 3 },
    { 0x419eca88, 3 },
    { 0, 0 },
    { 0x78aeca8a, 3 },
    { 0xb5aaca8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6ca90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71aca94, 3 },
    { 0, 0 },
    { 0x8e46ca96, 3 },
    { 0x1442ca97, 4 },
    { 0x676ca96, 4 },
    { 0xdae2ca97, 4 },
    { 0, 0 },
    { 0xe6deca9b, 3 },
    { 0, 0 },
    { 0x923eca9d, 4 },
    { 0, 0 },
    { 0x397eca9f, 3 },
    { 0, 0 },
    { 0x1f92caa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ecaa9, 3 },
    { 0xc87acaaa, 3 },
    { 0, 0 },
    { 0xc012caac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6caaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1296cab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141ecab6, 3 },
    { 0, 0 },
    { 0xf536cab8, 3 },
    { 0, 0 },
    { 0xa876caba, 3 },
    { 0x70decabb, 3 },
    { 0xe10acabb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9502cabf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743acac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea6cac7, 3 },
    { 0xdc9acac8, 4 },
    { 0x1ceacac7, 4 },
    { 0, 0 },
    { 0xcc5ecacb, 5 },
    { 0, 0 },
    { 0xad9ecacd, 3 },
    { 0, 0 },
    { 0x9fa2cacf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8ecad7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f2cada, 3 },
    { 0xa656cadb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2ecae1, 3 },
    { 0x7a0acae2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8acae5, 3 },
    { 0x9aa2cae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38ecaea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc6caee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca6caf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c42cafc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06acb02, 3 },
    { 0xa74ecb03, 4 },
    { 0xcf82cb04, 3 },
    { 0x7452cb04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2acb08, 3 },
    { 0xa60ecb09, 4 },
    { 0x802acb0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767ecb11, 3 },
    { 0x8516cb12, 3 },
    { 0x9e16cb13, 4 },
    { 0x7b22cb13, 3 },
    { 0xd626cb15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6cb18, 3 },
    { 0x10bacb19, 4 },
    { 0xd622cb19, 4 },
    { 0xe6dacb1b, 3 },
    { 0, 0 },
    { 0x854acb1d, 3 },
    { 0xbe1ecb1d, 3 },
    { 0xb52ecb1d, 3 },
    { 0xb8bacb1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ecb27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbacb2a, 3 },
    { 0xcceecb2b, 4 },
    { 0x5a3ecb2b, 4 },
    { 0xf196cb2c, 3 },
    { 0x2aaacb2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1666cb31, 3 },
    { 0xea96cb32, 3 },
    { 0x765acb33, 3 },
    { 0x6716cb32, 3 },
    { 0, 0 },
    { 0x63acb36, 4 },
    { 0, 0 },
    { 0xa6aacb38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2acb40, 3 },
    { 0, 0 },
    { 0x6916cb42, 4 },
    { 0, 0 },
    { 0xba4acb44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8ecb56, 3 },
    { 0x4ff6cb57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e76cb5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bacb5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98acb62, 4 },
    { 0xc5eacb63, 4 },
    { 0xc70acb64, 3 },
    { 0x5ab2cb62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00acb6d, 3 },
    { 0x930ecb6e, 4 },
    { 0x7dc6cb6f, 3 },
    { 0x2af2cb6e, 3 },
    { 0, 0 },
    { 0xdc22cb72, 4 },
    { 0, 0 },
    { 0xf7decb74, 4 },
    { 0x847acb75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9acb79, 3 },
    { 0x15dacb79, 3 },
    { 0x77aacb7b, 4 },
    { 0, 0 },
    { 0x38acb7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2872cb81, 3 },
    { 0x22dacb82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2cb8f, 3 },
    { 0xce1acb90, 3 },
    { 0x5f4acb8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9facb99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19acb9c, 3 },
    { 0x1f96cb9d, 3 },
    { 0xb4d6cb9d, 3 },
    { 0x8b86cb9f, 3 },
    { 0x6bd2cb9e, 3 },
    { 0xd7aecba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f2cba7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x359acbaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8302cbad, 3 },
    { 0xb37ecbae, 3 },
    { 0, 0 },
    { 0x7d7ecbb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x722ecbb3, 3 },
    { 0xe5aacbb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd62cbba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825acbbe, 3 },
    { 0x5c52cbbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b2cbc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0acbc7, 4 },
    { 0x92bacbc8, 3 },
    { 0, 0 },
    { 0x612ecbca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20ecbce, 3 },
    { 0xd692cbcf, 3 },
    { 0xc692cbd0, 3 },
    { 0, 0 },
    { 0xfc26cbd2, 3 },
    { 0, 0 },
    { 0x4d06cbd4, 3 },
    { 0x4332cbd4, 3 },
    { 0x8f0ecbd4, 3 },
    { 0x8316cbd7, 5 },
    { 0xc92ecbd8, 3 },
    { 0xeb5ecbd8, 3 },
    { 0, 0 },
    { 0xd256cbdb, 4 },
    { 0x5de2cbdc, 3 },
    { 0x6392cbdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04ecbe5, 3 },
    { 0x299acbe6, 4 },
    { 0x418acbe6, 3 },
    { 0x507acbe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62cbef, 3 },
    { 0x84facbf0, 3 },
    { 0, 0 },
    { 0x48aecbf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6cbf5, 3 },
    { 0, 0 },
    { 0xd6b6cbf7, 4 },
    { 0, 0 },
    { 0xab3ecbf9, 3 },
    { 0x304acbf9, 3 },
    { 0, 0 },
    { 0x652cbfc, 3 },
    { 0, 0 },
    { 0xec0ecbfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b12cc05, 4 },
    { 0, 0 },
    { 0x88d2cc07, 3 },
    { 0, 0 },
    { 0xd1a6cc09, 3 },
    { 0x198acc0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697acc10, 4 },
    { 0x2f9ecc11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c52cc17, 3 },
    { 0x6566cc18, 4 },
    { 0xe2c2cc19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abecc1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294acc22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62cc27, 3 },
    { 0x16b2cc27, 4 },
    { 0x37c2cc29, 4 },
    { 0xbddecc29, 3 },
    { 0xc9cacc2b, 3 },
    { 0x500ecc29, 4 },
    { 0x5e66cc29, 4 },
    { 0, 0 },
    { 0x8fe6cc2f, 5 },
    { 0, 0 },
    { 0x2f1acc31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae2cc35, 3 },
    { 0xb75ecc36, 4 },
    { 0, 0 },
    { 0x286acc38, 3 },
    { 0xe462cc38, 3 },
    { 0, 0 },
    { 0x35a6cc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2cc3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfecc43, 4 },
    { 0xa262cc44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292cc48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06cc4b, 3 },
    { 0x2386cc4c, 4 },
    { 0x1e66cc4c, 3 },
    { 0, 0 },
    { 0xdab2cc4f, 3 },
    { 0xc4c6cc50, 3 },
    { 0, 0 },
    { 0x6e4acc52, 3 },
    { 0, 0 },
    { 0x3a56cc54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6facc5b, 3 },
    { 0x6efacc5c, 3 },
    { 0xf65acc5d, 4 },
    { 0xb84acc5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1acc61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b2cc64, 3 },
    { 0xcfc6cc65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcecc68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69acc6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc016cc73, 4 },
    { 0, 0 },
    { 0xb87ecc75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aecc79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2ecc7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62decc85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ecc89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf322cc8c, 3 },
    { 0xde66cc8d, 4 },
    { 0xc80ecc8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e6cc91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2902cc94, 3 },
    { 0x2482cc95, 4 },
    { 0x1202cc94, 3 },
    { 0xdbf6cc94, 4 },
    { 0x3532cc96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x183acc9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ecca1, 3 },
    { 0xaa82cca2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7acca6, 3 },
    { 0x188acca6, 4 },
    { 0x4ccecca8, 3 },
    { 0x67decca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x562ccac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e6ccb2, 3 },
    { 0x9f5eccb3, 3 },
    { 0, 0 },
    { 0xcc9eccb5, 3 },
    { 0xeff2ccb6, 5 },
    { 0xdc8accb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9accbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0eccc8, 3 },
    { 0, 0 },
    { 0x2d82ccca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6cccd, 3 },
    { 0, 0 },
    { 0x5066cccf, 3 },
    { 0x90eeccd0, 4 },
    { 0x13daccd1, 3 },
    { 0x831eccd1, 3 },
    { 0xd99eccd2, 3 },
    { 0x1f5accd2, 3 },
    { 0x2e5accd5, 3 },
    { 0xdc0eccd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5feccd9, 3 },
    { 0x4f2eccda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12acce5, 3 },
    { 0x9edecce5, 3 },
    { 0x9732cce7, 3 },
    { 0x58a6cce8, 3 },
    { 0xdb32cce8, 3 },
    { 0x9866cce7, 3 },
    { 0x10acce9, 5 },
    { 0x90f6cce7, 3 },
    { 0xea2ecced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c2ccf7, 4 },
    { 0, 0 },
    { 0x7f4eccf9, 3 },
    { 0x9d56ccfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55aeccfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeacd03, 3 },
    { 0xdb76cd04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ecd07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed6cd0d, 4 },
    { 0x4792cd0e, 3 },
    { 0x8c72cd0f, 3 },
    { 0x7ef6cd10, 5 },
    { 0, 0 },
    { 0x959ecd12, 3 },
    { 0, 0 },
    { 0x783acd14, 3 },
    { 0x476ecd14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38ecd1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbecd23, 3 },
    { 0, 0 },
    { 0xe9b6cd25, 3 },
    { 0x2e32cd26, 3 },
    { 0, 0 },
    { 0xe866cd28, 4 },
    { 0, 0 },
    { 0xd35ecd2a, 3 },
    { 0, 0 },
    { 0xd2e2cd2c, 5 },
    { 0x430acd2d, 5 },
    { 0xbdeacd2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x357acd31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62cd38, 3 },
    { 0x7af2cd38, 3 },
    { 0xed3ecd3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cd41, 3 },
    { 0, 0 },
    { 0x9862cd43, 3 },
    { 0xd8f6cd44, 3 },
    { 0, 0 },
    { 0x487acd46, 3 },
    { 0xd55ecd47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5012cd4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc72cd56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807ecd5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22acd61, 3 },
    { 0xd7f6cd62, 3 },
    { 0xbb2ecd63, 3 },
    { 0, 0 },
    { 0xa8decd65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796acd6b, 4 },
    { 0x595ecd6c, 4 },
    { 0x5806cd6d, 3 },
    { 0xb166cd6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eecd71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e6cd7b, 3 },
    { 0x669acd7c, 3 },
    { 0x4fc2cd7d, 3 },
    { 0, 0 },
    { 0xac6cd7f, 3 },
    { 0xd552cd80, 4 },
    { 0xed6ecd7f, 3 },
    { 0xc22acd7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49acd85, 3 },
    { 0x2cd6cd86, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6acd8e, 3 },
    { 0, 0 },
    { 0xb33acd90, 3 },
    { 0x87becd91, 3 },
    { 0x30cacd92, 3 },
    { 0xb382cd93, 4 },
    { 0x8dd2cd94, 3 },
    { 0xe31ecd94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8862cd99, 3 },
    { 0, 0 },
    { 0xa7b6cd9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb2cd9f, 3 },
    { 0x368ecda0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe456cda5, 3 },
    { 0x1bd2cda5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58eacda9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c32cdae, 3 },
    { 0, 0 },
    { 0x6f16cdb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618ecdb5, 3 },
    { 0, 0 },
    { 0x4222cdb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf846cdbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86facdbf, 3 },
    { 0x9c6acdbf, 4 },
    { 0, 0 },
    { 0x8baecdc2, 3 },
    { 0x813ecdc3, 4 },
    { 0x48e6cdc2, 4 },
    { 0, 0 },
    { 0x2886cdc6, 3 },
    { 0x3a3ecdc7, 3 },
    { 0x79c2cdc8, 4 },
    { 0xb6e2cdc8, 4 },
    { 0, 0 },
    { 0x1a7acdcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed2cdce, 3 },
    { 0, 0 },
    { 0xd2a6cdd0, 3 },
    { 0, 0 },
    { 0xf0fecdd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882cdd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70acde4, 3 },
    { 0xad7acde5, 3 },
    { 0x39c6cde6, 3 },
    { 0, 0 },
    { 0x6d92cde8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3372cdeb, 3 },
    { 0x484acdeb, 3 },
    { 0x621ecded, 4 },
    { 0x13f6cdeb, 3 },
    { 0x957ecdef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01acdf5, 3 },
    { 0xd696cdf6, 3 },
    { 0xc7eecdf7, 3 },
    { 0x531ecdf6, 3 },
    { 0x51e6cdf8, 3 },
    { 0, 0 },
    { 0xf1acdfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c2ce02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d6ce08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16ce13, 3 },
    { 0xb576ce13, 3 },
    { 0xcf1ece15, 3 },
    { 0xb7cece15, 3 },
    { 0x4f6ce17, 3 },
    { 0xc2a6ce18, 3 },
    { 0, 0 },
    { 0xe8a2ce1a, 3 },
    { 0x30b2ce1a, 3 },
    { 0x1f52ce1c, 4 },
    { 0x516ece1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eece24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafece2a, 4 },
    { 0xe252ce2a, 3 },
    { 0xb31ece2c, 4 },
    { 0x25c2ce2d, 3 },
    { 0x43f2ce2c, 3 },
    { 0, 0 },
    { 0x46b6ce30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaece37, 5 },
    { 0x3826ce38, 3 },
    { 0xa0a2ce39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ace3d, 3 },
    { 0xf2aace3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afece45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1ace48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7ece52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952ce58, 3 },
    { 0x588ece59, 3 },
    { 0xd422ce5a, 3 },
    { 0x2906ce58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6ce5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06ece65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ece69, 3 },
    { 0, 0 },
    { 0xf04ace6b, 4 },
    { 0xb50ece6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61ece70, 3 },
    { 0xdb72ce71, 3 },
    { 0xe8d2ce71, 3 },
    { 0x1d2ce70, 3 },
    { 0, 0 },
    { 0x30cece75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a6ce79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2752ce7c, 3 },
    { 0x70f2ce7c, 4 },
    { 0x146ace7c, 3 },
    { 0, 0 },
    { 0x6f4ece80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ece86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x683ace89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3ece8d, 3 },
    { 0xebface8d, 3 },
    { 0x8126ce8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932ace93, 3 },
    { 0x3f82ce94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2ce98, 4 },
    { 0xaaa2ce99, 3 },
    { 0, 0 },
    { 0xbfa2ce9b, 3 },
    { 0x6666ce9c, 3 },
    { 0x70a2ce9b, 3 },
    { 0xea26ce9c, 4 },
    { 0x4e7ece9d, 3 },
    { 0x7c9ecea0, 3 },
    { 0x308ece9f, 3 },
    { 0x92becea1, 3 },
    { 0, 0 },
    { 0x85c2cea4, 3 },
    { 0x54e2cea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf392ceab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7662ceaf, 3 },
    { 0x36a2ceb0, 3 },
    { 0x296ceb1, 3 },
    { 0x5982ceb2, 3 },
    { 0, 0 },
    { 0x7dbeceb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb136ceba, 4 },
    { 0x116aceba, 4 },
    { 0xcc22cebc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd16cec0, 3 },
    { 0x66d2cec1, 3 },
    { 0x9de6cec1, 3 },
    { 0xc14acec3, 3 },
    { 0x79facec4, 3 },
    { 0, 0 },
    { 0x568ecec6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3ecec9, 3 },
    { 0x2cbaceca, 3 },
    { 0x79b6ceca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb986cece, 3 },
    { 0xc4decece, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ced4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d6cedc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cacee0, 3 },
    { 0x8376cee1, 4 },
    { 0x29f2cee1, 3 },
    { 0x7c2cee3, 3 },
    { 0x4ecacee0, 3 },
    { 0x70bacee5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2ceea, 3 },
    { 0, 0 },
    { 0x2786ceec, 3 },
    { 0x7692ceed, 3 },
    { 0, 0 },
    { 0x1766ceef, 3 },
    { 0xaf92cef0, 3 },
    { 0x2dc2cef1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bacf02, 3 },
    { 0x3d6ecf02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8452cf07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcacf0c, 4 },
    { 0x4cdecf0c, 3 },
    { 0xd92acf0d, 3 },
    { 0, 0 },
    { 0x5086cf10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d2cf13, 3 },
    { 0, 0 },
    { 0xd12cf15, 3 },
    { 0xfdfecf15, 3 },
    { 0xe0e6cf17, 3 },
    { 0xc9dacf17, 3 },
    { 0x868ecf19, 3 },
    { 0, 0 },
    { 0x9eeecf1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0acf21, 3 },
    { 0x2e0ecf21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab6cf25, 3 },
    { 0, 0 },
    { 0xe2c6cf27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd246cf2b, 3 },
    { 0, 0 },
    { 0x386acf2d, 3 },
    { 0xcfaecf2e, 3 },
    { 0xfa8acf2e, 5 },
    { 0, 0 },
    { 0xa632cf31, 3 },
    { 0, 0 },
    { 0x5a12cf33, 3 },
    { 0, 0 },
    { 0x9a66cf35, 3 },
    { 0x5242cf35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe142cf3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572cf3d, 4 },
    { 0x82bacf3d, 3 },
    { 0, 0 },
    { 0x251acf40, 3 },
    { 0, 0 },
    { 0x8282cf42, 3 },
    { 0x140ecf43, 3 },
    { 0, 0 },
    { 0x4b3acf45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17acf49, 4 },
    { 0xd96acf49, 3 },
    { 0xbf9acf4b, 3 },
    { 0xf5a2cf4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aacf53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd306cf5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdecf5e, 4 },
    { 0, 0 },
    { 0xaa2ecf60, 3 },
    { 0, 0 },
    { 0x7c52cf62, 3 },
    { 0xbbc2cf63, 3 },
    { 0, 0 },
    { 0x5f0ecf65, 3 },
    { 0xaf62cf65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93ecf71, 3 },
    { 0xd6a6cf71, 4 },
    { 0, 0 },
    { 0x72f2cf74, 3 },
    { 0xb4aecf75, 3 },
    { 0xde4ecf76, 3 },
    { 0, 0 },
    { 0x2c12cf78, 3 },
    { 0x8646cf79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3facf7c, 3 },
    { 0x2592cf7d, 3 },
    { 0xcec6cf7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7166cf83, 4 },
    { 0x45bacf83, 4 },
    { 0x1606cf83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dacf8a, 3 },
    { 0xd17ecf8a, 4 },
    { 0, 0 },
    { 0xfff2cf8d, 3 },
    { 0, 0 },
    { 0xe2f2cf8f, 3 },
    { 0xd03ecf8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0acf94, 3 },
    { 0xb342cf94, 4 },
    { 0x706cf96, 3 },
    { 0, 0 },
    { 0x43fecf98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ecf9b, 3 },
    { 0x380acf9c, 3 },
    { 0xa92cf9d, 4 },
    { 0x1cf6cf9e, 3 },
    { 0x7896cf9d, 3 },
    { 0xd9f6cfa0, 5 },
    { 0x9f2cfa1, 3 },
    { 0x3a9acf9c, 4 },
    { 0xc212cfa3, 5 },
    { 0x5eb2cfa4, 3 },
    { 0xea8acfa3, 3 },
    { 0xa2a6cfa5, 3 },
    { 0xaee2cfa5, 4 },
    { 0x5a52cfa8, 3 },
    { 0x81f2cfa7, 4 },
    { 0xec06cfa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03acfae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c6cfb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fecfb7, 3 },
    { 0, 0 },
    { 0xa87acfb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eacfbd, 3 },
    { 0x8cf6cfbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde2cfc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e2cfc4, 3 },
    { 0xc6bacfc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4996cfc8, 3 },
    { 0, 0 },
    { 0xad7ecfca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1ecfce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9acfd4, 3 },
    { 0, 0 },
    { 0x897ecfd6, 3 },
    { 0x9b2cfd7, 3 },
    { 0xa706cfd8, 3 },
    { 0x29a2cfd9, 3 },
    { 0x543acfda, 4 },
    { 0xc492cfdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf626cfdf, 3 },
    { 0x202ecfe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666cfe3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee6cfe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc2cfee, 3 },
    { 0x6576cfef, 3 },
    { 0x21ecfee, 4 },
    { 0x8fbacfef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ecff5, 3 },
    { 0x3d2acff5, 3 },
    { 0xe56ecff7, 3 },
    { 0xfc4ecff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d6d005, 4 },
    { 0x53ad005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47ad00b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ad014, 4 },
    { 0x42e2d015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x625ad018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaad01e, 4 },
    { 0x7ebad01f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead022, 3 },
    { 0, 0 },
    { 0x369ed024, 3 },
    { 0x6146d025, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6d028, 3 },
    { 0x4626d029, 3 },
    { 0x6b0ad02a, 4 },
    { 0x41dad029, 3 },
    { 0xeb66d02c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e72d02f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bed032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592d036, 3 },
    { 0xf2aed037, 3 },
    { 0xdc72d037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5996d03c, 3 },
    { 0x7706d03c, 4 },
    { 0x356ad03e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c2d042, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd6d04b, 3 },
    { 0, 0 },
    { 0x8646d04d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8742d050, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66ed056, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2d064, 4 },
    { 0x33f2d064, 4 },
    { 0xc7f6d065, 4 },
    { 0xe842d067, 3 },
    { 0x2ddad068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4ad06b, 5 },
    { 0x4aaad06c, 3 },
    { 0x1b8ad06d, 4 },
    { 0x7612d06e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2d074, 3 },
    { 0xd40ed074, 3 },
    { 0x9686d076, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe56d079, 3 },
    { 0xdee2d07a, 3 },
    { 0x8bf2d07b, 3 },
    { 0, 0 },
    { 0xb236d07d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ed084, 3 },
    { 0xa6e2d085, 3 },
    { 0, 0 },
    { 0xdcead087, 3 },
    { 0x7b2d087, 3 },
    { 0x815ad089, 3 },
    { 0, 0 },
    { 0x80b2d08b, 4 },
    { 0, 0 },
    { 0x346ed08d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4ad090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fed096, 3 },
    { 0xa126d096, 3 },
    { 0x1b4ed098, 3 },
    { 0x6fed099, 3 },
    { 0x8b62d09a, 3 },
    { 0x9c8ad099, 3 },
    { 0xd12ed097, 3 },
    { 0x676d097, 3 },
    { 0xb29ed09c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826ed0a3, 3 },
    { 0, 0 },
    { 0xcd9ed0a5, 4 },
    { 0, 0 },
    { 0xc60ed0a7, 3 },
    { 0, 0 },
    { 0x66cad0a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6336d0ac, 3 },
    { 0x9902d0ad, 4 },
    { 0xd8ced0ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10ad0b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad0b8, 4 },
    { 0x69c6d0b9, 3 },
    { 0x5252d0ba, 3 },
    { 0x8c82d0b8, 3 },
    { 0, 0 },
    { 0x75d2d0bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb162d0c3, 3 },
    { 0x7896d0c4, 3 },
    { 0xc6a6d0c5, 3 },
    { 0xdd3ad0c6, 4 },
    { 0xc3bed0c7, 3 },
    { 0, 0 },
    { 0x89b2d0c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2d0ce, 3 },
    { 0xc10ed0cf, 3 },
    { 0x843ed0cf, 4 },
    { 0, 0 },
    { 0x3ded0d2, 4 },
    { 0x2b2ed0d2, 3 },
    { 0x1fa6d0d4, 5 },
    { 0xa21ad0d5, 3 },
    { 0x93a2d0d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28ed0dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cad0e2, 4 },
    { 0x292ad0e2, 3 },
    { 0xbcad0e4, 3 },
    { 0, 0 },
    { 0x7402d0e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ad0e9, 4 },
    { 0xffced0ea, 5 },
    { 0xa92d0e9, 4 },
    { 0, 0 },
    { 0x6236d0ed, 4 },
    { 0x1aa6d0ed, 4 },
    { 0xb936d0ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f2d0f6, 3 },
    { 0x9f2ed0f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5ed0fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa316d100, 3 },
    { 0xd1c6d100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2d111, 3 },
    { 0xe32d112, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eced115, 4 },
    { 0x17dad116, 3 },
    { 0x6f4ad116, 3 },
    { 0x5e52d118, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a2d11c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e2d11f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17ed128, 4 },
    { 0, 0 },
    { 0x6ff2d12a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9136d12d, 3 },
    { 0xe0aad12e, 4 },
    { 0xc84ed12f, 3 },
    { 0x438ed12e, 3 },
    { 0xaf9ad12f, 3 },
    { 0x382d132, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ad138, 3 },
    { 0x348ed139, 4 },
    { 0, 0 },
    { 0x2fa2d13b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3ed148, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe16d14c, 3 },
    { 0, 0 },
    { 0xf6bed14e, 3 },
    { 0x2a06d14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292d15a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6662d15d, 4 },
    { 0x9556d15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef46d164, 3 },
    { 0x65eed164, 3 },
    { 0x8f0ad166, 4 },
    { 0x468ed164, 3 },
    { 0x6dc2d168, 5 },
    { 0xc492d169, 5 },
    { 0x63b6d168, 4 },
    { 0, 0 },
    { 0x6a7ed16c, 3 },
    { 0xbc76d16c, 3 },
    { 0x2c82d16e, 4 },
    { 0x76d6d16c, 3 },
    { 0x56ded16f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d2d173, 4 },
    { 0xc0b2d173, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bed17b, 5 },
    { 0, 0 },
    { 0x7b0ad17d, 4 },
    { 0, 0 },
    { 0xd63ad17f, 3 },
    { 0x91eed180, 3 },
    { 0x7f1ed17f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79ed189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fed195, 3 },
    { 0xc32ad195, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ed19b, 4 },
    { 0xd0a6d19c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec6d1a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e26d1aa, 3 },
    { 0x478ed1aa, 3 },
    { 0x17ed1aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aad1b2, 4 },
    { 0, 0 },
    { 0xabc2d1b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98ed1b8, 4 },
    { 0x6242d1b8, 3 },
    { 0xc76ed1ba, 3 },
    { 0, 0 },
    { 0x41ded1bc, 4 },
    { 0xf016d1bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5186d1c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f2d1c6, 3 },
    { 0xdd12d1c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd666d1ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeed1cf, 4 },
    { 0xc6e2d1d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee6d1d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66d1d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8ed1de, 3 },
    { 0x796ad1df, 3 },
    { 0xa72ad1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9ad1ed, 4 },
    { 0, 0 },
    { 0xbcb6d1ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x522ad1f2, 3 },
    { 0xbb86d1f3, 3 },
    { 0x93f6d1f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf92d1f7, 3 },
    { 0, 0 },
    { 0x682d1f9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ed206, 3 },
    { 0x30ead207, 4 },
    { 0xf846d208, 3 },
    { 0x5d52d209, 4 },
    { 0xa022d209, 3 },
    { 0x544ad20b, 3 },
    { 0, 0 },
    { 0xbc9ad20d, 3 },
    { 0xebcad20d, 3 },
    { 0xbcdad20d, 3 },
    { 0x54d2d210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c6d214, 3 },
    { 0x3562d215, 3 },
    { 0, 0 },
    { 0x263ad217, 3 },
    { 0x8902d217, 4 },
    { 0xc5e6d219, 3 },
    { 0xeeeed218, 3 },
    { 0x811ed21b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x218ed21e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2536d221, 3 },
    { 0, 0 },
    { 0x362d223, 3 },
    { 0xd2d224, 3 },
    { 0xc33ed223, 3 },
    { 0xa92ed226, 3 },
    { 0xbd5ad226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd982d22c, 4 },
    { 0x7d2ed22c, 3 },
    { 0x757ad22e, 4 },
    { 0x2d9ed22f, 3 },
    { 0xefe2d22e, 5 },
    { 0x4506d231, 3 },
    { 0xeaaed232, 4 },
    { 0x7aaed233, 3 },
    { 0x5e7ad22f, 4 },
    { 0x2c4ad232, 3 },
    { 0x6fc6d230, 3 },
    { 0xcf26d237, 3 },
    { 0xb7e2d22e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf096d23b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e6d23e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2d246, 3 },
    { 0xa8bad247, 3 },
    { 0xe5ad247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ed24b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffad252, 3 },
    { 0, 0 },
    { 0x9362d254, 3 },
    { 0x168ad254, 3 },
    { 0, 0 },
    { 0xab6ad257, 3 },
    { 0x411ed258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e6d25c, 5 },
    { 0xa97ed25d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66d260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e46d264, 3 },
    { 0, 0 },
    { 0x4812d266, 3 },
    { 0x5412d267, 3 },
    { 0x29a2d268, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaad26c, 3 },
    { 0x7d72d26d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6ad270, 4 },
    { 0, 0 },
    { 0xb42ed272, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096d277, 4 },
    { 0x572ad278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4512d28a, 3 },
    { 0x32aad28b, 3 },
    { 0xce06d28c, 4 },
    { 0x7d22d28d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba86d291, 4 },
    { 0x93dad292, 3 },
    { 0x9646d293, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956d297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4736d29c, 4 },
    { 0xa9aed29d, 3 },
    { 0x8bfad29e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e22d2a6, 3 },
    { 0x1cdad2a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad2ae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df2d2b3, 3 },
    { 0x852d2b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd636d2b7, 3 },
    { 0x68a6d2b8, 3 },
    { 0xd5aed2b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5ed2c0, 3 },
    { 0, 0 },
    { 0x380ad2c2, 5 },
    { 0xfea6d2c3, 3 },
    { 0x6926d2c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636ad2ca, 5 },
    { 0xa57ed2cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4896d2cf, 5 },
    { 0xb04ed2d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23ed2d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2db, 3 },
    { 0x51bad2dc, 4 },
    { 0, 0 },
    { 0x73d6d2de, 3 },
    { 0xb5eed2de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa232d2e3, 3 },
    { 0xee2d2e4, 3 },
    { 0x2da2d2e3, 4 },
    { 0, 0 },
    { 0xe30ed2e7, 3 },
    { 0x1276d2e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fad2ec, 3 },
    { 0, 0 },
    { 0x253ad2ee, 3 },
    { 0xa11ad2ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21ed2f3, 3 },
    { 0x6d62d2f4, 3 },
    { 0x5f6ad2f3, 3 },
    { 0xffaed2f4, 4 },
    { 0x906d2f7, 4 },
    { 0x626ad2f6, 3 },
    { 0x7d52d2f9, 4 },
    { 0, 0 },
    { 0x3a9ed2fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673ed2ff, 3 },
    { 0xb46d300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d2d307, 3 },
    { 0x379ed308, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6d30c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de2d311, 4 },
    { 0, 0 },
    { 0xea02d313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ed319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e2d31f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7336d322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8ed326, 3 },
    { 0x2ae2d327, 3 },
    { 0, 0 },
    { 0xbdb2d329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cad32e, 3 },
    { 0, 0 },
    { 0xfa4ed330, 4 },
    { 0, 0 },
    { 0xfb46d332, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8076d336, 3 },
    { 0x8e6ad337, 3 },
    { 0, 0 },
    { 0xfe6ad339, 4 },
    { 0x56ead33a, 3 },
    { 0x8b92d33b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594ed33f, 4 },
    { 0, 0 },
    { 0xa3f2d341, 3 },
    { 0x1632d341, 3 },
    { 0x7c56d343, 3 },
    { 0, 0 },
    { 0x9c3ad345, 4 },
    { 0x4512d346, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a6d34a, 3 },
    { 0, 0 },
    { 0xecad34c, 3 },
    { 0, 0 },
    { 0xfceed34e, 4 },
    { 0, 0 },
    { 0x790ed350, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2ed353, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf462d357, 3 },
    { 0x214ed358, 4 },
    { 0x57ced357, 3 },
    { 0x820ad35a, 3 },
    { 0xd3ed35b, 3 },
    { 0x9bb2d358, 3 },
    { 0x29aad35d, 4 },
    { 0x154ad35e, 3 },
    { 0xe092d35f, 4 },
    { 0x15ead35f, 3 },
    { 0xd92d35f, 4 },
    { 0x9b96d361, 3 },
    { 0x1556d363, 4 },
    { 0x6fbed35d, 3 },
    { 0, 0 },
    { 0x79aad366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eed36a, 4 },
    { 0x3c62d36b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbad371, 3 },
    { 0, 0 },
    { 0x85b6d373, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefed378, 3 },
    { 0xb0f2d378, 3 },
    { 0xc092d37a, 3 },
    { 0, 0 },
    { 0x2502d37c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x336ed37f, 3 },
    { 0x2166d380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7986d383, 3 },
    { 0x384ad384, 3 },
    { 0, 0 },
    { 0x75cad386, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d42d389, 3 },
    { 0x5776d38a, 3 },
    { 0x93eed38b, 3 },
    { 0x8a8ad38b, 3 },
    { 0xd8f2d38b, 3 },
    { 0x3652d38e, 3 },
    { 0x71bad38f, 3 },
    { 0xb2b6d38e, 3 },
    { 0x9672d391, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b12d394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32d398, 3 },
    { 0, 0 },
    { 0x87dad39a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9ad3a2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97ed3ae, 3 },
    { 0x3632d3af, 3 },
    { 0xcbbed3af, 4 },
    { 0x82eed3b1, 3 },
    { 0xba4ed3b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x571ed3b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ead3ba, 3 },
    { 0xeb2d3ba, 4 },
    { 0x760ed3bc, 4 },
    { 0x34cad3bd, 3 },
    { 0x16b6d3bc, 4 },
    { 0x2596d3bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc6d3c7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122d3ca, 3 },
    { 0xe06d3ca, 4 },
    { 0xfe2ed3cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ad3db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd6d3df, 3 },
    { 0, 0 },
    { 0x4d2d3e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x186ad3e4, 3 },
    { 0, 0 },
    { 0x5396d3e6, 3 },
    { 0xe62d3e7, 4 },
    { 0x5a46d3e6, 3 },
    { 0x8bed3e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82d3f1, 3 },
    { 0xdfaad3f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79ad3f7, 3 },
    { 0xc4d2d3f8, 3 },
    { 0xb22d3f7, 3 },
    { 0x62bad3fa, 3 },
    { 0x6142d3fa, 3 },
    { 0xdd9ed3fa, 3 },
    { 0, 0 },
    { 0x76f6d3fe, 4 },
    { 0, 0 },
    { 0x932ed400, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8ad404, 3 },
    { 0xd086d405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aed40a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11ed417, 3 },
    { 0xa6c6d418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe316d41e, 5 },
    { 0xa802d41e, 3 },
    { 0x7e56d420, 3 },
    { 0x3ae2d421, 3 },
    { 0, 0 },
    { 0x13ed423, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74ad428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0ad42c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a96d430, 3 },
    { 0xc256d431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf776d434, 3 },
    { 0x61e6d434, 3 },
    { 0xfb9ad436, 3 },
    { 0xe636d437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe6d43a, 4 },
    { 0x379ad43b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b2d43e, 3 },
    { 0, 0 },
    { 0x9efad440, 4 },
    { 0xd2ad440, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8ad444, 3 },
    { 0xd08ad444, 4 },
    { 0x1c4ad446, 5 },
    { 0xc8b6d444, 3 },
    { 0, 0 },
    { 0xeeded449, 3 },
    { 0x3e0ed44a, 3 },
    { 0xf94ed44b, 3 },
    { 0xd526d44c, 3 },
    { 0x66ed44d, 3 },
    { 0x5726d44c, 3 },
    { 0x8492d44f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ed454, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7ed457, 3 },
    { 0x760ad458, 3 },
    { 0xaac2d459, 3 },
    { 0, 0 },
    { 0x8852d45b, 4 },
    { 0xdd2d45c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aead464, 4 },
    { 0x60ded465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e52d46b, 3 },
    { 0xedd6d46c, 3 },
    { 0x7fcad46c, 4 },
    { 0, 0 },
    { 0xcaaad46f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbed477, 3 },
    { 0x45ed478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536ad482, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a22d485, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c2d48b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ad490, 3 },
    { 0x636ed491, 3 },
    { 0xe8d2d492, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89ad497, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1ed49a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd6d4a7, 5 },
    { 0x271ed4a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d4ab, 3 },
    { 0x5e36d4ac, 4 },
    { 0x63f2d4ac, 4 },
    { 0xa306d4ae, 3 },
    { 0x9046d4af, 3 },
    { 0x96aed4b0, 3 },
    { 0x5aeed4b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cad4b6, 3 },
    { 0x1ca6d4b6, 3 },
    { 0xce86d4b8, 4 },
    { 0x1c42d4b7, 5 },
    { 0x199ad4ba, 4 },
    { 0x6e1ed4bb, 3 },
    { 0xa3c2d4b7, 3 },
    { 0x6d2ad4b8, 3 },
    { 0xc9ad4be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9ad4c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d4c5, 3 },
    { 0, 0 },
    { 0x92fed4c7, 3 },
    { 0x2d62d4c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56d4cc, 3 },
    { 0, 0 },
    { 0x2d92d4ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aad4d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561ad4d5, 3 },
    { 0, 0 },
    { 0xb56ed4d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4052d4de, 3 },
    { 0, 0 },
    { 0x1e22d4e0, 3 },
    { 0x6ed6d4e1, 3 },
    { 0xe1fed4e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c92d4e6, 3 },
    { 0, 0 },
    { 0x7152d4e8, 3 },
    { 0xb492d4e9, 3 },
    { 0x55f2d4ea, 4 },
    { 0xba9ed4eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe656d4f3, 3 },
    { 0xd296d4f4, 3 },
    { 0x2eaed4f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692d4fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86d4ff, 3 },
    { 0x83ced500, 3 },
    { 0x89ded501, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa32d504, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5ed507, 3 },
    { 0, 0 },
    { 0x115ad509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5222d50e, 3 },
    { 0, 0 },
    { 0x1e4ed510, 3 },
    { 0x36ed511, 3 },
    { 0, 0 },
    { 0xd412d513, 4 },
    { 0, 0 },
    { 0x5a2ed515, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ded51b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ad520, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caed525, 3 },
    { 0x4f2ad526, 3 },
    { 0xd9ead527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ad52a, 4 },
    { 0xb74ad52b, 3 },
    { 0x6152d52c, 3 },
    { 0, 0 },
    { 0xdcded52e, 3 },
    { 0xbbc2d52f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bad533, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed2d536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d2d539, 4 },
    { 0x46a2d53a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ad53d, 5 },
    { 0x1c76d53e, 3 },
    { 0xea06d53f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c2d542, 3 },
    { 0x32bad543, 4 },
    { 0x3fd2d543, 3 },
    { 0xc52ed545, 4 },
    { 0, 0 },
    { 0x563ed547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615ad54b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d6d551, 3 },
    { 0x60bed551, 3 },
    { 0xe3c6d553, 3 },
    { 0xb5c2d551, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc376d558, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ced561, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe966d565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6ad568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f6d56c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605ed571, 3 },
    { 0x9d76d572, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c6d576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2ad579, 4 },
    { 0x98c2d57a, 3 },
    { 0, 0 },
    { 0x2cf2d57c, 3 },
    { 0x3286d57d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2912d580, 3 },
    { 0xc44ed581, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36ad585, 3 },
    { 0xbe2ed586, 3 },
    { 0, 0 },
    { 0x6272d588, 3 },
    { 0x3e1ad588, 3 },
    { 0x1306d58a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ed58d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0ad590, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2d593, 3 },
    { 0x685ed594, 3 },
    { 0x4cf6d595, 4 },
    { 0, 0 },
    { 0xfa6ad597, 4 },
    { 0, 0 },
    { 0xca2d599, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fed5a1, 4 },
    { 0x9bd6d5a2, 3 },
    { 0x298ed5a3, 4 },
    { 0x3676d5a4, 4 },
    { 0x2122d5a4, 4 },
    { 0, 0 },
    { 0x13b2d5a7, 3 },
    { 0x141ed5a8, 3 },
    { 0, 0 },
    { 0x52a6d5aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0ed5ad, 3 },
    { 0xb86ed5ae, 5 },
    { 0, 0 },
    { 0x361ed5b0, 3 },
    { 0, 0 },
    { 0xac32d5b2, 3 },
    { 0x2b32d5b3, 3 },
    { 0xb70ed5b4, 3 },
    { 0x314ed5b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ed5b8, 3 },
    { 0x813ad5b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead5bf, 3 },
    { 0, 0 },
    { 0xf5eed5c1, 4 },
    { 0x4d96d5c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076d5c5, 3 },
    { 0, 0 },
    { 0xe2ead5c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ed5ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5ad5cd, 3 },
    { 0xbca6d5ce, 3 },
    { 0x42d5cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d56d5d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2d5d6, 3 },
    { 0, 0 },
    { 0x4c46d5d8, 3 },
    { 0xd196d5d9, 3 },
    { 0xd9c6d5da, 4 },
    { 0xdfa6d5db, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5496d5e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f36d5e5, 3 },
    { 0x14ded5e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266d5f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852d5f8, 3 },
    { 0xa762d5f9, 3 },
    { 0xaa4ad5f9, 3 },
    { 0xc5cad5f8, 3 },
    { 0x9dced5fc, 3 },
    { 0xcfead5fc, 3 },
    { 0xecaad5fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8ed601, 3 },
    { 0x896d602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8012d606, 3 },
    { 0x5e02d607, 3 },
    { 0, 0 },
    { 0x247ad609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e82d613, 3 },
    { 0x285ed614, 3 },
    { 0x9fed614, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3302d61a, 3 },
    { 0xa27ed61b, 3 },
    { 0x5fad61a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c62d620, 4 },
    { 0xec0ed620, 4 },
    { 0x5936d621, 3 },
    { 0xa736d623, 3 },
    { 0, 0 },
    { 0x550ed625, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6ed62b, 3 },
    { 0xbb36d62c, 3 },
    { 0x11aad62d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ced632, 3 },
    { 0xaabad633, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ed637, 3 },
    { 0x866ad637, 3 },
    { 0, 0 },
    { 0x5416d63a, 3 },
    { 0x2f76d63b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d641, 3 },
    { 0xf6a6d642, 4 },
    { 0, 0 },
    { 0x65e6d644, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2d648, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeed64b, 3 },
    { 0x9c2ad64b, 3 },
    { 0xc72d64b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ed652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1196d666, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eed66a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aded66e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef2d673, 3 },
    { 0xbc86d673, 3 },
    { 0xd2bed675, 4 },
    { 0, 0 },
    { 0x7b1ad677, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbad67c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1182d684, 3 },
    { 0x45f6d685, 3 },
    { 0x9f1ad685, 3 },
    { 0, 0 },
    { 0xda56d688, 3 },
    { 0xb16d689, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf32d692, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ad695, 3 },
    { 0x651ed696, 4 },
    { 0x602ed695, 5 },
    { 0x6646d698, 3 },
    { 0xe3c2d698, 3 },
    { 0xafa6d69a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288ed6a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56d6a7, 3 },
    { 0, 0 },
    { 0x8dead6a9, 4 },
    { 0, 0 },
    { 0xf4ed6ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc136d6ae, 3 },
    { 0, 0 },
    { 0xa3f6d6b0, 3 },
    { 0xa6a2d6b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2726d6b5, 3 },
    { 0xc996d6b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61ed6b9, 3 },
    { 0x2a3ed6b9, 3 },
    { 0xcb6d6bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ad6be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc956d6c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8322d6c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276d6cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd6d6d0, 3 },
    { 0, 0 },
    { 0x99c2d6d2, 4 },
    { 0x789ed6d2, 3 },
    { 0x9be6d6d2, 4 },
    { 0xff26d6d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cad6dd, 4 },
    { 0xca62d6dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6ed6e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce96d6e5, 3 },
    { 0, 0 },
    { 0x3d6ad6e7, 3 },
    { 0, 0 },
    { 0xaa32d6e9, 3 },
    { 0x4df2d6ea, 3 },
    { 0, 0 },
    { 0x9926d6ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd722d6ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e6d6f8, 3 },
    { 0x1832d6f9, 3 },
    { 0, 0 },
    { 0x560ed6fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b2d702, 3 },
    { 0xfd6ad702, 3 },
    { 0x631ad704, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b52d70d, 3 },
    { 0, 0 },
    { 0x920ad70f, 3 },
    { 0x3b92d70f, 3 },
    { 0xe282d711, 4 },
    { 0xf9ad711, 3 },
    { 0x767ed713, 5 },
    { 0, 0 },
    { 0xb07ed715, 4 },
    { 0, 0 },
    { 0x2aaad717, 3 },
    { 0, 0 },
    { 0x63aed719, 5 },
    { 0, 0 },
    { 0xbd2ad71b, 3 },
    { 0x9406d71c, 4 },
    { 0, 0 },
    { 0x8466d71e, 3 },
    { 0, 0 },
    { 0x8aaad720, 4 },
    { 0x5f92d721, 3 },
    { 0x7d9ad722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f42d729, 3 },
    { 0xe792d72a, 3 },
    { 0, 0 },
    { 0xfd76d72c, 3 },
    { 0x55fad72d, 3 },
    { 0x6656d72d, 3 },
    { 0x507ad72f, 4 },
    { 0, 0 },
    { 0xa8ead731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78ed734, 3 },
    { 0, 0 },
    { 0x78ead736, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397ed740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e16d749, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38ed74e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b2d751, 3 },
    { 0xa9b6d752, 3 },
    { 0xaa5ed753, 3 },
    { 0, 0 },
    { 0xaa02d755, 5 },
    { 0x7946d756, 3 },
    { 0, 0 },
    { 0xc52ad758, 3 },
    { 0xcad759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c96d75e, 4 },
    { 0x96ced75e, 3 },
    { 0x63ead760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc2d764, 3 },
    { 0, 0 },
    { 0x6f82d766, 3 },
    { 0xa66ed767, 4 },
    { 0xfb86d767, 3 },
    { 0x613ad767, 3 },
    { 0x85a6d76a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2d771, 3 },
    { 0, 0 },
    { 0x419ed773, 3 },
    { 0x331ad774, 4 },
    { 0x6816d775, 4 },
    { 0x992d776, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fed77a, 3 },
    { 0xd966d77b, 3 },
    { 0, 0 },
    { 0xc292d77d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1ed782, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc102d787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad78a, 3 },
    { 0xd246d78a, 3 },
    { 0, 0 },
    { 0x186d78d, 3 },
    { 0, 0 },
    { 0x93e6d78f, 3 },
    { 0xc29ed790, 3 },
    { 0xa966d791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb2d795, 4 },
    { 0xacd6d796, 3 },
    { 0xa50ad797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5ad79c, 3 },
    { 0xb976d79c, 3 },
    { 0x5036d79e, 3 },
    { 0, 0 },
    { 0x2366d7a0, 3 },
    { 0, 0 },
    { 0x9786d7a2, 3 },
    { 0x76f6d7a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbed7a8, 3 },
    { 0x9f6d7a9, 3 },
    { 0xceead7a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5ad7b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe716d7b8, 3 },
    { 0, 0 },
    { 0x8b2d7ba, 3 },
    { 0x857ed7ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6d7bf, 4 },
    { 0xfdb6d7bf, 4 },
    { 0x41e6d7bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55ad7cc, 3 },
    { 0x4a0ed7cc, 3 },
    { 0xae72d7ce, 3 },
    { 0, 0 },
    { 0xd27ad7d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa2d7d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532d7da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06d7dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8ed7e0, 4 },
    { 0xf57ed7e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2d7e4, 3 },
    { 0, 0 },
    { 0xc02d7e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bad7eb, 4 },
    { 0x6312d7eb, 3 },
    { 0xccced7ec, 3 },
    { 0x4202d7ee, 3 },
    { 0x38ead7ef, 3 },
    { 0xc1d6d7ee, 5 },
    { 0x3eed7f1, 3 },
    { 0xd1fed7f2, 4 },
    { 0x5846d7f3, 3 },
    { 0, 0 },
    { 0x983ad7f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01ad7fb, 3 },
    { 0xde9ed7fc, 3 },
    { 0xc56ad7fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3226d800, 3 },
    { 0, 0 },
    { 0x6e3ad802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d2d806, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756ad80d, 3 },
    { 0, 0 },
    { 0x1c1ad80f, 3 },
    { 0x319ed80f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0ed815, 4 },
    { 0x79ead816, 4 },
    { 0xf2d6d815, 3 },
    { 0, 0 },
    { 0x1aa2d819, 3 },
    { 0x664ed81a, 4 },
    { 0xc7dad81a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7082d81e, 3 },
    { 0xffb6d81f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d2d823, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e96d826, 3 },
    { 0x145ad826, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbded82b, 3 },
    { 0, 0 },
    { 0x7036d82d, 3 },
    { 0x695ad82e, 3 },
    { 0xb0a6d82f, 3 },
    { 0x29ad830, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ced835, 4 },
    { 0xcf76d835, 3 },
    { 0x18ad837, 3 },
    { 0xad66d837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ead83d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ed840, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14ad844, 4 },
    { 0x7c42d845, 4 },
    { 0xe406d846, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e32d84a, 4 },
    { 0x3b02d84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea32d84e, 4 },
    { 0xa4aad84f, 4 },
    { 0, 0 },
    { 0x472ed851, 3 },
    { 0x9ffed851, 3 },
    { 0x549ed851, 4 },
    { 0x7102d851, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dad85a, 3 },
    { 0, 0 },
    { 0xde3ad85c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5ad863, 3 },
    { 0, 0 },
    { 0xf746d865, 3 },
    { 0, 0 },
    { 0xddced867, 3 },
    { 0, 0 },
    { 0xddd6d869, 3 },
    { 0, 0 },
    { 0xc396d86b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ad871, 3 },
    { 0xac62d872, 3 },
    { 0, 0 },
    { 0x8fc6d874, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2d878, 4 },
    { 0xbf8ed879, 4 },
    { 0xa476d87a, 3 },
    { 0, 0 },
    { 0xcffad87c, 4 },
    { 0xee86d87d, 3 },
    { 0x6982d87c, 3 },
    { 0x171ad87c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ced883, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bed889, 3 },
    { 0x14a2d88a, 3 },
    { 0xc83ad88a, 3 },
    { 0, 0 },
    { 0x2666d88d, 3 },
    { 0xafdad88d, 3 },
    { 0x325ed88f, 3 },
    { 0, 0 },
    { 0xebdad891, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109ad895, 3 },
    { 0xce76d895, 4 },
    { 0xd40ad897, 3 },
    { 0x5bcad898, 3 },
    { 0xcd36d897, 4 },
    { 0xb262d89a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ad8a3, 3 },
    { 0xa466d8a3, 4 },
    { 0xfb02d8a5, 3 },
    { 0x7816d8a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2d8ac, 4 },
    { 0, 0 },
    { 0x7ee2d8ae, 3 },
    { 0x1156d8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df6d8b3, 3 },
    { 0xdff6d8b3, 3 },
    { 0, 0 },
    { 0x8a3ad8b6, 4 },
    { 0x2ddad8b6, 3 },
    { 0, 0 },
    { 0xa8fad8b9, 4 },
    { 0, 0 },
    { 0x1ea2d8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6932d8be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb492d8c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ed8cb, 3 },
    { 0x5c42d8cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb276d8cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc6d8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9deed8da, 4 },
    { 0x527ed8da, 3 },
    { 0, 0 },
    { 0xfeded8dd, 3 },
    { 0xca2ad8de, 3 },
    { 0x7632d8df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ed8e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b72d8e9, 4 },
    { 0, 0 },
    { 0x1696d8eb, 3 },
    { 0xa4ded8eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ed8ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a6d8f3, 3 },
    { 0, 0 },
    { 0xadded8f5, 3 },
    { 0xc1fed8f6, 3 },
    { 0xf322d8f7, 4 },
    { 0, 0 },
    { 0x9156d8f9, 3 },
    { 0x54ded8fa, 3 },
    { 0xba82d8f9, 3 },
    { 0, 0 },
    { 0x215ad8fd, 4 },
    { 0x58e2d8fe, 3 },
    { 0x70b6d8fd, 3 },
    { 0xf712d8ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9162d905, 4 },
    { 0x45dad906, 3 },
    { 0, 0 },
    { 0x552ed908, 3 },
    { 0xc9c2d909, 3 },
    { 0xd2e6d908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeed90d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa156d919, 3 },
    { 0x1e3ed91a, 3 },
    { 0, 0 },
    { 0xeb22d91c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d32d924, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3076d929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b6d931, 3 },
    { 0, 0 },
    { 0x15b6d933, 3 },
    { 0, 0 },
    { 0x7bfad935, 4 },
    { 0xa8e6d936, 3 },
    { 0xb7f2d937, 3 },
    { 0, 0 },
    { 0x25aad939, 3 },
    { 0xae02d939, 3 },
    { 0x7c2ad93b, 3 },
    { 0xc066d93b, 4 },
    { 0, 0 },
    { 0x8d7ed93e, 3 },
    { 0x8d8ad93e, 3 },
    { 0x97ded940, 5 },
    { 0x7972d940, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6ed946, 4 },
    { 0, 0 },
    { 0x93c2d948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd2d94b, 4 },
    { 0, 0 },
    { 0xcd2d94d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615ad952, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ad958, 3 },
    { 0xac1ad958, 3 },
    { 0xd682d95a, 3 },
    { 0x2e2d95b, 3 },
    { 0x1c2d95c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x451ad95f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad962, 3 },
    { 0x848ad962, 3 },
    { 0, 0 },
    { 0x52c6d965, 3 },
    { 0x2186d966, 3 },
    { 0xa1ad967, 3 },
    { 0, 0 },
    { 0x18dad969, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78ed96e, 3 },
    { 0x9f06d96f, 4 },
    { 0xd762d970, 3 },
    { 0x1306d96e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca46d974, 3 },
    { 0x89ded975, 3 },
    { 0x4142d976, 3 },
    { 0xdafed977, 3 },
    { 0xd982d978, 3 },
    { 0x7c82d978, 3 },
    { 0, 0 },
    { 0xafded97b, 3 },
    { 0xd826d97b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ad982, 3 },
    { 0, 0 },
    { 0x4ad6d984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52d987, 3 },
    { 0xa26d987, 3 },
    { 0x17bed989, 4 },
    { 0xa27ed989, 3 },
    { 0xa6d98b, 4 },
    { 0x2076d98c, 4 },
    { 0, 0 },
    { 0x95e2d98e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x878ad991, 3 },
    { 0x4c7ad992, 3 },
    { 0x386ad993, 5 },
    { 0, 0 },
    { 0xad8ed995, 3 },
    { 0x9feed995, 4 },
    { 0x95b2d997, 4 },
    { 0x9dc6d998, 3 },
    { 0x902ed999, 3 },
    { 0xeaced998, 4 },
    { 0x9d2d999, 3 },
    { 0, 0 },
    { 0x766ad99d, 4 },
    { 0x9626d99e, 3 },
    { 0x53aad99f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb236d9aa, 3 },
    { 0xcef6d9ab, 4 },
    { 0x4842d9ac, 3 },
    { 0x8826d9ac, 3 },
    { 0xffc2d9ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x162ed9b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d9b6, 3 },
    { 0x8bdad9b7, 3 },
    { 0xdafed9b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ad9bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736ad9c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ed9cb, 5 },
    { 0x1476d9cc, 3 },
    { 0xd076d9cb, 3 },
    { 0xd4bed9ce, 3 },
    { 0x1c6ad9ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319ad9d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e2d9dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb86d9e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebed9e5, 3 },
    { 0, 0 },
    { 0xef46d9e7, 3 },
    { 0, 0 },
    { 0x2592d9e9, 3 },
    { 0xf922d9e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbed9f0, 3 },
    { 0, 0 },
    { 0xfcaad9f2, 3 },
    { 0, 0 },
    { 0x7c1ed9f4, 3 },
    { 0x442ad9f5, 3 },
    { 0, 0 },
    { 0x2a3ed9f7, 3 },
    { 0xb286d9f8, 3 },
    { 0x4286d9f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c2d9fc, 3 },
    { 0, 0 },
    { 0x2e4ad9fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6da10, 4 },
    { 0, 0 },
    { 0xa0f2da12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62da1d, 4 },
    { 0xdd7ada1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4ada27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4056da2a, 3 },
    { 0x6206da2b, 3 },
    { 0x4092da2a, 3 },
    { 0x5e0eda2b, 3 },
    { 0x91d2da2e, 3 },
    { 0, 0 },
    { 0x3cc2da30, 3 },
    { 0x83d6da30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952da3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e6da41, 3 },
    { 0x1bf2da41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1eda49, 3 },
    { 0, 0 },
    { 0xf0bada4b, 4 },
    { 0x6912da4c, 3 },
    { 0, 0 },
    { 0x3212da4e, 3 },
    { 0x7f02da4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6da52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855eda59, 3 },
    { 0xef96da59, 3 },
    { 0x4e9eda5b, 3 },
    { 0xe4ceda59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353ada62, 3 },
    { 0x1732da63, 3 },
    { 0xa712da62, 3 },
    { 0xf846da65, 3 },
    { 0xf3c6da65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173eda6a, 4 },
    { 0, 0 },
    { 0xb3ceda6c, 4 },
    { 0, 0 },
    { 0x5caada6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffada71, 4 },
    { 0, 0 },
    { 0xc246da73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6da77, 3 },
    { 0, 0 },
    { 0x9916da79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd6da7d, 4 },
    { 0x2bf2da7e, 4 },
    { 0xa216da7f, 5 },
    { 0x8f06da7f, 3 },
    { 0x37aada81, 3 },
    { 0x4a66da7f, 3 },
    { 0xd882da83, 3 },
    { 0xbb72da83, 3 },
    { 0, 0 },
    { 0xd0b6da86, 3 },
    { 0, 0 },
    { 0xa1fada88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664eda91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eda95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1ada98, 4 },
    { 0x365eda99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92eda9c, 3 },
    { 0, 0 },
    { 0xeebeda9e, 3 },
    { 0, 0 },
    { 0xfea6daa0, 3 },
    { 0x7ad6daa0, 3 },
    { 0, 0 },
    { 0x3daedaa3, 3 },
    { 0, 0 },
    { 0xa2f6daa5, 3 },
    { 0, 0 },
    { 0x3ed2daa7, 3 },
    { 0x36d6daa8, 3 },
    { 0xe7fedaa9, 3 },
    { 0, 0 },
    { 0xc55adaab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaedaaf, 3 },
    { 0xdbbadab0, 3 },
    { 0xa8fedab0, 4 },
    { 0x6836dab2, 4 },
    { 0x6252dab3, 3 },
    { 0xc21adab4, 4 },
    { 0xfec2dab0, 3 },
    { 0, 0 },
    { 0xe95adab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79adabc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cedac1, 3 },
    { 0xbef6dac2, 4 },
    { 0xcfcedac3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6dac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adadacb, 4 },
    { 0x759edacc, 4 },
    { 0, 0 },
    { 0xd302dace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64edad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5406dad9, 3 },
    { 0xc062dad9, 3 },
    { 0xf312dadb, 3 },
    { 0x4772dadc, 3 },
    { 0x8006dadd, 4 },
    { 0xc66edadd, 4 },
    { 0xdc4edadf, 3 },
    { 0x45bedada, 5 },
    { 0, 0 },
    { 0x1e7adae2, 3 },
    { 0x293adae3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f2dae6, 3 },
    { 0xf47adae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f2daeb, 3 },
    { 0, 0 },
    { 0x6a26daed, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafedaf0, 5 },
    { 0, 0 },
    { 0x5072daf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4086daf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xda22daf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2dafc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0edaff, 3 },
    { 0, 0 },
    { 0xb67edb01, 3 },
    { 0, 0 },
    { 0x72cadb03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6db06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cedb0a, 4 },
    { 0xd1edb0b, 3 },
    { 0xc63adb0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16db13, 3 },
    { 0xa18adb13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e22db18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeadb20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1adb24, 3 },
    { 0x716edb25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7016db29, 3 },
    { 0x765edb2a, 3 },
    { 0xd38adb2b, 3 },
    { 0x4b72db2a, 3 },
    { 0xf9badb2d, 3 },
    { 0, 0 },
    { 0x533edb2f, 3 },
    { 0x3c7edb30, 3 },
    { 0x613adb2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec6db36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x169edb39, 3 },
    { 0, 0 },
    { 0xe89adb3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd026db40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cedb43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab2db48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7426db4e, 3 },
    { 0x7a5adb4f, 3 },
    { 0xf3b2db50, 3 },
    { 0xb48edb50, 4 },
    { 0xc5adb52, 4 },
    { 0x227edb53, 5 },
    { 0xfee2db53, 4 },
    { 0x5952db53, 3 },
    { 0x5c96db56, 3 },
    { 0xa44adb57, 3 },
    { 0xc98edb58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8026db5f, 3 },
    { 0x6232db60, 3 },
    { 0x84e2db61, 3 },
    { 0xa56db62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4adb66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3edb6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00adb6f, 3 },
    { 0x1feadb6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e6db77, 3 },
    { 0x9e06db77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7186db7c, 3 },
    { 0, 0 },
    { 0x7caedb7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3052db82, 4 },
    { 0x9326db83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cedb86, 3 },
    { 0x7ac2db87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746adb91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7adb99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x948edb9c, 4 },
    { 0x2e56db9c, 3 },
    { 0, 0 },
    { 0xb402db9f, 3 },
    { 0x8b5edb9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6dbac, 3 },
    { 0, 0 },
    { 0xa6eedbae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9602dbb2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dedbb5, 4 },
    { 0, 0 },
    { 0x8742dbb7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526edbbd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab2dbc1, 4 },
    { 0xe6e2dbc2, 3 },
    { 0x8b7adbc3, 3 },
    { 0, 0 },
    { 0xec8edbc5, 3 },
    { 0x5d8adbc6, 4 },
    { 0x662dbc7, 4 },
    { 0x6a6dbc8, 3 },
    { 0x232dbc8, 3 },
    { 0xd516dbca, 3 },
    { 0, 0 },
    { 0x54cadbcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb426dbd3, 3 },
    { 0xbd36dbd4, 3 },
    { 0, 0 },
    { 0x798adbd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32dbd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63edbdf, 3 },
    { 0, 0 },
    { 0xa2aedbe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27edbe5, 3 },
    { 0, 0 },
    { 0x9b3adbe7, 3 },
    { 0, 0 },
    { 0xfe8adbe9, 4 },
    { 0, 0 },
    { 0x6d4adbeb, 3 },
    { 0, 0 },
    { 0x410adbed, 3 },
    { 0, 0 },
    { 0x13d2dbef, 3 },
    { 0, 0 },
    { 0x5ebadbf1, 3 },
    { 0, 0 },
    { 0x87bedbf3, 3 },
    { 0, 0 },
    { 0xc7a6dbf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3986dbfa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638adc03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefadc08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732dc0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6dc0f, 4 },
    { 0x6552dc0f, 3 },
    { 0xa8eadc10, 3 },
    { 0, 0 },
    { 0x3fdedc13, 3 },
    { 0x48c6dc14, 3 },
    { 0, 0 },
    { 0x6e52dc16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb676dc1e, 3 },
    { 0x12aedc1e, 3 },
    { 0x8eaedc20, 4 },
    { 0, 0 },
    { 0x78f6dc22, 4 },
    { 0x7bd2dc23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x917adc26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62dc2a, 3 },
    { 0x6b3adc2b, 3 },
    { 0xf0dedc2b, 3 },
    { 0xf2a6dc2d, 3 },
    { 0x4d7edc2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c2dc31, 4 },
    { 0xd0a2dc32, 3 },
    { 0xb606dc33, 3 },
    { 0x39c6dc34, 5 },
    { 0x292edc34, 3 },
    { 0xaf82dc36, 3 },
    { 0, 0 },
    { 0xeba6dc38, 3 },
    { 0, 0 },
    { 0x43e2dc3a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82dc3e, 4 },
    { 0xc466dc3e, 3 },
    { 0x1726dc40, 3 },
    { 0x4c26dc3f, 5 },
    { 0xe90adc42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbadc45, 3 },
    { 0, 0 },
    { 0xab32dc47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabedc4b, 3 },
    { 0xdf42dc4c, 3 },
    { 0, 0 },
    { 0xcdeadc4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aadc51, 3 },
    { 0xd1aedc52, 3 },
    { 0x5b06dc53, 4 },
    { 0xb3adc54, 3 },
    { 0, 0 },
    { 0x52b6dc56, 4 },
    { 0xc036dc57, 3 },
    { 0x9172dc58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22dc5b, 3 },
    { 0x7b2edc5c, 5 },
    { 0, 0 },
    { 0x28badc5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x771edc61, 3 },
    { 0xedbedc62, 4 },
    { 0x9ce6dc62, 4 },
    { 0xa5aadc64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21adc6b, 4 },
    { 0x1266dc6c, 3 },
    { 0x51c2dc6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dedc70, 4 },
    { 0x9e46dc70, 3 },
    { 0, 0 },
    { 0x7de6dc73, 3 },
    { 0, 0 },
    { 0xd0aadc75, 3 },
    { 0x314edc76, 3 },
    { 0, 0 },
    { 0x1406dc78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b06dc7b, 3 },
    { 0, 0 },
    { 0x894edc7d, 5 },
    { 0x1566dc7e, 3 },
    { 0, 0 },
    { 0xf082dc80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96dc84, 3 },
    { 0x469adc85, 3 },
    { 0, 0 },
    { 0x39b6dc87, 3 },
    { 0x9b96dc88, 3 },
    { 0xa396dc89, 3 },
    { 0, 0 },
    { 0x6d6adc8b, 3 },
    { 0x5346dc8c, 3 },
    { 0xa982dc8d, 3 },
    { 0xbb86dc8e, 3 },
    { 0xebaadc8f, 4 },
    { 0x1606dc90, 4 },
    { 0x526dc91, 3 },
    { 0xe69edc92, 4 },
    { 0xe40adc92, 3 },
    { 0x7ceedc94, 3 },
    { 0x47adc92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eadc98, 3 },
    { 0, 0 },
    { 0x616adc9a, 4 },
    { 0x1fd2dc9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2adc9f, 4 },
    { 0x8e32dca0, 3 },
    { 0x4af6dca1, 3 },
    { 0x20dadc9f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc076dca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaadcb2, 3 },
    { 0x3fbedcb3, 4 },
    { 0xdcd2dcb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x524adcb7, 3 },
    { 0xdc2adcb7, 3 },
    { 0x60dedcb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5582dcbe, 3 },
    { 0x3b42dcbf, 3 },
    { 0x34cadcbe, 3 },
    { 0, 0 },
    { 0xa7c2dcc2, 3 },
    { 0, 0 },
    { 0x607edcc4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f42dcca, 3 },
    { 0xc70adccb, 4 },
    { 0x2cb2dcca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992edcd0, 3 },
    { 0, 0 },
    { 0x2d12dcd2, 3 },
    { 0, 0 },
    { 0x845adcd4, 3 },
    { 0, 0 },
    { 0x95c6dcd6, 4 },
    { 0x180edcd7, 3 },
    { 0, 0 },
    { 0xb57edcd9, 3 },
    { 0, 0 },
    { 0xe3e2dcdb, 3 },
    { 0, 0 },
    { 0xc0fedcdd, 3 },
    { 0x441edcde, 3 },
    { 0, 0 },
    { 0xc08edce0, 3 },
    { 0x719edce1, 4 },
    { 0, 0 },
    { 0x84d6dce3, 4 },
    { 0x3e5adce3, 3 },
    { 0xa792dce5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4adcea, 4 },
    { 0, 0 },
    { 0xc4c2dcec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7adcf1, 3 },
    { 0, 0 },
    { 0x4b4adcf3, 3 },
    { 0xe06edcf4, 3 },
    { 0, 0 },
    { 0x2f7edcf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36dcfe, 3 },
    { 0, 0 },
    { 0x4766dd00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9add03, 4 },
    { 0x4f32dd04, 3 },
    { 0x8292dd05, 3 },
    { 0, 0 },
    { 0x85b2dd07, 4 },
    { 0x1126dd07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19edd0b, 4 },
    { 0x4ccadd0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056dd0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a2dd13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36add17, 3 },
    { 0, 0 },
    { 0xeb42dd19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc62dd1c, 3 },
    { 0xfa1edd1c, 3 },
    { 0, 0 },
    { 0xc32dd1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b2dd22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686dd27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fedd2c, 4 },
    { 0xfdcadd2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6306dd30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741edd34, 3 },
    { 0x95cadd35, 3 },
    { 0xdbdadd34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7152dd3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4782dd42, 3 },
    { 0xd16dd43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e2dd47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53add4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e6dd4d, 3 },
    { 0xd05add4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32add55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226dd58, 4 },
    { 0, 0 },
    { 0x26bedd5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aedd61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31add65, 3 },
    { 0x5d86dd65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eeadd69, 3 },
    { 0xcee2dd6a, 4 },
    { 0xa5fadd6a, 3 },
    { 0, 0 },
    { 0x1206dd6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6dd70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52dd75, 4 },
    { 0, 0 },
    { 0x424edd77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d12dd7a, 3 },
    { 0xc6dd7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac6dd80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2dd86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334edd8a, 4 },
    { 0x6c1add8a, 3 },
    { 0xafaedd8c, 4 },
    { 0x13a6dd8b, 3 },
    { 0x7a6add8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6edd94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0add99, 3 },
    { 0, 0 },
    { 0x15aedd9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eedd9e, 4 },
    { 0x699add9e, 3 },
    { 0x2a8edda0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa6dda7, 4 },
    { 0, 0 },
    { 0x6d36dda9, 3 },
    { 0xe3baddaa, 3 },
    { 0x184adda9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95addb0, 3 },
    { 0x1b82ddb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ddb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7892ddb9, 3 },
    { 0, 0 },
    { 0x73faddbb, 4 },
    { 0xba8addbb, 3 },
    { 0xea8eddbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa322ddc3, 4 },
    { 0x7b3addc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf402ddc8, 3 },
    { 0, 0 },
    { 0xad16ddca, 5 },
    { 0xd552ddcb, 4 },
    { 0, 0 },
    { 0x805eddcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834addd3, 4 },
    { 0xaa8eddd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5626ddd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f6dddd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed56ddeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e02ddef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2de00, 3 },
    { 0xae86de01, 3 },
    { 0, 0 },
    { 0x307ade03, 3 },
    { 0x1ec2de04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576de08, 4 },
    { 0x4ede09, 3 },
    { 0, 0 },
    { 0x1592de0b, 3 },
    { 0x363ede0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9546de12, 3 },
    { 0xaf82de12, 3 },
    { 0x9e6ade13, 3 },
    { 0x38ede15, 3 },
    { 0x17cade16, 4 },
    { 0xccbede16, 3 },
    { 0x666ade18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc6de1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ede21, 3 },
    { 0x5586de22, 4 },
    { 0x97dade23, 3 },
    { 0xc636de24, 3 },
    { 0xa066de24, 3 },
    { 0x6e0ede26, 3 },
    { 0x55c6de27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x422de2a, 3 },
    { 0xcdcede2b, 3 },
    { 0, 0 },
    { 0x7886de2d, 3 },
    { 0, 0 },
    { 0xdc96de2f, 3 },
    { 0, 0 },
    { 0x573ade31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbede40, 3 },
    { 0, 0 },
    { 0x43dade42, 3 },
    { 0x4046de43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c6de46, 5 },
    { 0xd52ade46, 3 },
    { 0xafdede48, 3 },
    { 0xa6a2de46, 3 },
    { 0xa9c6de48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc032de50, 4 },
    { 0, 0 },
    { 0x175ade52, 4 },
    { 0x6032de52, 3 },
    { 0x4f96de54, 4 },
    { 0xb96de54, 3 },
    { 0x96a6de56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bade5b, 3 },
    { 0x4ad6de5c, 4 },
    { 0xf67ade5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7ade63, 3 },
    { 0xf9a6de64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eade67, 3 },
    { 0x8552de67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aede74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332de78, 3 },
    { 0x7c3ede79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbade7c, 3 },
    { 0xd192de7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e6de82, 4 },
    { 0, 0 },
    { 0x3272de84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6696de87, 3 },
    { 0x5a66de88, 3 },
    { 0, 0 },
    { 0x44cede8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df6de90, 3 },
    { 0x254ade90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b96de94, 3 },
    { 0, 0 },
    { 0x6146de96, 4 },
    { 0x7d52de97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ade9a, 3 },
    { 0, 0 },
    { 0xbd3ade9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6dea2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf06dea7, 3 },
    { 0xe0aadea8, 3 },
    { 0, 0 },
    { 0x8802deaa, 3 },
    { 0x1b76deaa, 3 },
    { 0xc2fadeaa, 3 },
    { 0x4946dead, 3 },
    { 0xf6aadeaa, 3 },
    { 0xea3edeaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eedeb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300adeba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1296debe, 3 },
    { 0x3676debf, 3 },
    { 0xa2d6dec0, 3 },
    { 0xdd7edec1, 4 },
    { 0x16fadec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf2deca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22ded1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4796ded7, 3 },
    { 0xc9deded7, 4 },
    { 0, 0 },
    { 0xe3cadeda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8862dee0, 3 },
    { 0x942dee1, 3 },
    { 0x5776dee0, 3 },
    { 0xae4edee3, 3 },
    { 0x6dbedee4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422deed, 4 },
    { 0xd6badeee, 3 },
    { 0xd69edeef, 3 },
    { 0xe406deef, 3 },
    { 0, 0 },
    { 0x4586def2, 3 },
    { 0x35fedef3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac22defc, 3 },
    { 0x4a22defc, 4 },
    { 0, 0 },
    { 0x70dadeff, 3 },
    { 0x638edf00, 3 },
    { 0x7e96df01, 4 },
    { 0x18a2df00, 3 },
    { 0x797adf02, 4 },
    { 0x6b1adf01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bedf09, 4 },
    { 0, 0 },
    { 0x7d06df0b, 3 },
    { 0x30cadf0b, 3 },
    { 0xadd6df0d, 4 },
    { 0x666adf0b, 3 },
    { 0, 0 },
    { 0x1bd6df10, 5 },
    { 0xbf9adf11, 3 },
    { 0xcbbadf11, 3 },
    { 0, 0 },
    { 0x7b92df14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cedf19, 3 },
    { 0, 0 },
    { 0x93aadf1b, 4 },
    { 0x6d6edf1c, 4 },
    { 0x7c92df1d, 4 },
    { 0, 0 },
    { 0xae2df1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefadf23, 3 },
    { 0xf6aedf24, 4 },
    { 0x7c52df24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a6df37, 3 },
    { 0, 0 },
    { 0xcab6df39, 3 },
    { 0, 0 },
    { 0xcb16df3b, 3 },
    { 0x671edf3c, 3 },
    { 0x89fadf3d, 4 },
    { 0x1996df3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2df42, 3 },
    { 0, 0 },
    { 0x80fadf44, 5 },
    { 0, 0 },
    { 0x31a2df46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc86df4e, 4 },
    { 0x7ae2df4f, 3 },
    { 0x4736df50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28adf55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5edf58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a2df5b, 3 },
    { 0xb54edf5b, 4 },
    { 0xba5adf5c, 4 },
    { 0x262edf5e, 4 },
    { 0xe13edf5f, 3 },
    { 0x469edf60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604adf67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f2df6a, 3 },
    { 0, 0 },
    { 0x7efadf6c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad2df71, 3 },
    { 0x7c5adf71, 3 },
    { 0xaecedf73, 3 },
    { 0, 0 },
    { 0x7886df75, 4 },
    { 0x549edf76, 3 },
    { 0, 0 },
    { 0x98c6df78, 3 },
    { 0x3ea6df79, 4 },
    { 0xc8d6df7a, 3 },
    { 0xe50adf7b, 3 },
    { 0x152edf7a, 3 },
    { 0, 0 },
    { 0x27fadf7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127edf84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2adf87, 4 },
    { 0xe12edf87, 5 },
    { 0x69cadf87, 3 },
    { 0xaafedf87, 3 },
    { 0xb932df8a, 3 },
    { 0x4cd2df8c, 3 },
    { 0xad2df8b, 3 },
    { 0x6ee6df89, 3 },
    { 0x8176df89, 3 },
    { 0x1f72df8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339adf97, 4 },
    { 0, 0 },
    { 0xb15adf99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f6df9d, 3 },
    { 0, 0 },
    { 0x462edf9f, 3 },
    { 0x27f6dfa0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eadfa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9edfac, 3 },
    { 0, 0 },
    { 0xe1c2dfae, 4 },
    { 0, 0 },
    { 0xcecedfb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539adfc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8132dfc4, 3 },
    { 0, 0 },
    { 0x5cb6dfc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe636dfcc, 5 },
    { 0x6edadfcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7782dfd5, 3 },
    { 0xff3adfd6, 3 },
    { 0xd75edfd6, 3 },
    { 0xa956dfd8, 3 },
    { 0xf96adfd8, 4 },
    { 0xabeedfd7, 3 },
    { 0x309adfdb, 3 },
    { 0xa472dfdc, 4 },
    { 0, 0 },
    { 0xb3f2dfde, 5 },
    { 0x318edfdf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dadfe9, 4 },
    { 0xeb4edfe9, 3 },
    { 0, 0 },
    { 0xd3f6dfec, 4 },
    { 0xe5dedfed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe302dff3, 3 },
    { 0xb5fadff4, 3 },
    { 0xf4eadff3, 3 },
    { 0x16e2dff6, 3 },
    { 0x6fc2dff7, 3 },
    { 0xa732dff8, 4 },
    { 0xb6d6dff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebedffe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ee001, 3 },
    { 0xee06e001, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ae008, 4 },
    { 0, 0 },
    { 0xd46ee00a, 3 },
    { 0xcad6e00b, 4 },
    { 0, 0 },
    { 0x5196e00d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe146e010, 4 },
    { 0xd7fae010, 3 },
    { 0x8a3ae012, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192e018, 4 },
    { 0xdceae019, 4 },
    { 0, 0 },
    { 0xcf16e01b, 3 },
    { 0xf0cee01c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c36e025, 3 },
    { 0, 0 },
    { 0xb692e027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x517ae02a, 3 },
    { 0, 0 },
    { 0xb1cee02c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f2e031, 3 },
    { 0xce3ee031, 4 },
    { 0x8db2e033, 3 },
    { 0xf342e034, 3 },
    { 0xeadae034, 4 },
    { 0x68ae036, 4 },
    { 0xb346e033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc46e03e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1186e041, 3 },
    { 0, 0 },
    { 0xf716e043, 3 },
    { 0x5aeae044, 3 },
    { 0xe44ee043, 3 },
    { 0xe5aee045, 3 },
    { 0x4fd6e046, 3 },
    { 0x71b6e048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652ae04e, 3 },
    { 0x8fd6e04e, 3 },
    { 0xb9d6e050, 3 },
    { 0xa286e050, 4 },
    { 0, 0 },
    { 0xbb9ae053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17ae058, 4 },
    { 0, 0 },
    { 0xe5fee05a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca36e05d, 3 },
    { 0x7abee05e, 3 },
    { 0x8e02e05e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a32e06e, 3 },
    { 0xf6c6e06f, 3 },
    { 0xaa46e06e, 4 },
    { 0xcec2e071, 3 },
    { 0xec0ae06f, 4 },
    { 0x8172e06e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a22e077, 3 },
    { 0x371ae078, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bae07d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dae080, 3 },
    { 0, 0 },
    { 0x5c6ee082, 3 },
    { 0, 0 },
    { 0xcb2e084, 3 },
    { 0xa20ee085, 4 },
    { 0x3e72e086, 4 },
    { 0x7172e085, 4 },
    { 0x23bee088, 3 },
    { 0xa4e6e084, 3 },
    { 0xbd46e08a, 3 },
    { 0, 0 },
    { 0xf93ae08c, 3 },
    { 0, 0 },
    { 0x8caae08e, 3 },
    { 0xd41ee08e, 3 },
    { 0x7adae090, 3 },
    { 0x3402e091, 3 },
    { 0, 0 },
    { 0xb39ee093, 4 },
    { 0x18f2e094, 3 },
    { 0x693ae094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x203ae098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fae09b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac92e0a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066e0a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723ee0b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bae0b9, 3 },
    { 0, 0 },
    { 0xb2b2e0bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dae0bf, 3 },
    { 0xf5eae0c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dee0c3, 3 },
    { 0, 0 },
    { 0x77e6e0c5, 3 },
    { 0x43fee0c6, 3 },
    { 0x1f5ae0c5, 3 },
    { 0xdf76e0c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ee0cb, 3 },
    { 0x7082e0cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3ee0d6, 3 },
    { 0x7156e0d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301ae0df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b06e0e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baae0ec, 3 },
    { 0, 0 },
    { 0x2832e0ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb346e0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa092e0fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe086e0fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x660ae100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5036e103, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbae107, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeee10a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fee10f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660ae114, 3 },
    { 0, 0 },
    { 0xded2e116, 3 },
    { 0x850ae117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822e11a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622e121, 3 },
    { 0x809ee121, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452ae129, 3 },
    { 0xa9c6e12a, 3 },
    { 0x1f3ae12b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c6e131, 3 },
    { 0, 0 },
    { 0x8ce6e133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8ae137, 3 },
    { 0xf34ee138, 3 },
    { 0xb5dee137, 4 },
    { 0, 0 },
    { 0x434ae13b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeee143, 3 },
    { 0x3bc2e144, 3 },
    { 0x36aee143, 3 },
    { 0xaefae145, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909ee14a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e2e14e, 3 },
    { 0x5542e14e, 4 },
    { 0, 0 },
    { 0x5d56e151, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9ee154, 3 },
    { 0x9696e154, 3 },
    { 0xc20ee154, 4 },
    { 0x5fcee156, 3 },
    { 0x626ee158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661ae15d, 3 },
    { 0xc0dae15e, 5 },
    { 0, 0 },
    { 0x6f72e160, 3 },
    { 0xc17ee161, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe752e16c, 3 },
    { 0x598ae16d, 4 },
    { 0x9066e16e, 3 },
    { 0x4556e16d, 5 },
    { 0xfbaae16e, 3 },
    { 0x8abee171, 3 },
    { 0x22eae172, 4 },
    { 0, 0 },
    { 0x2b2ae174, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829ee17a, 3 },
    { 0x5476e17b, 3 },
    { 0x7666e17b, 4 },
    { 0xcbd2e17d, 3 },
    { 0, 0 },
    { 0x675ee17f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc632e183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b6e187, 4 },
    { 0, 0 },
    { 0xdf02e189, 4 },
    { 0x24fae18a, 4 },
    { 0x6d6e18b, 4 },
    { 0x6fdae18c, 3 },
    { 0, 0 },
    { 0x87bae18e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecee191, 3 },
    { 0, 0 },
    { 0xd66ee193, 3 },
    { 0x809ee194, 3 },
    { 0x5b52e195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2e198, 3 },
    { 0x4f16e199, 4 },
    { 0xb546e19a, 3 },
    { 0x1ed2e19b, 3 },
    { 0xb216e19a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6426e19f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ae1a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aae1a7, 3 },
    { 0x46eee1a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6736e1ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177ee1b5, 3 },
    { 0, 0 },
    { 0xdecae1b7, 4 },
    { 0, 0 },
    { 0xb746e1b9, 4 },
    { 0x8562e1b9, 3 },
    { 0xd65ae1ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1772e1c1, 4 },
    { 0x905ae1c1, 3 },
    { 0x49d6e1c3, 3 },
    { 0xfb22e1c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc6e1c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1ee1cc, 3 },
    { 0x5566e1cc, 3 },
    { 0x491ae1ce, 4 },
    { 0x8b8ae1cf, 3 },
    { 0xda3ee1cf, 3 },
    { 0xa9aae1d1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e6e1d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc2e1d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd2e1de, 3 },
    { 0x8adee1de, 3 },
    { 0, 0 },
    { 0x6fcee1e1, 3 },
    { 0x9f5ae1e2, 4 },
    { 0x130ae1e3, 4 },
    { 0x257ee1e4, 3 },
    { 0xfb6ee1e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ae1ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83ae1f0, 3 },
    { 0x8922e1f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665ae1f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82e1fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046e20b, 3 },
    { 0, 0 },
    { 0x1906e20d, 3 },
    { 0, 0 },
    { 0x120ae20f, 3 },
    { 0x1c1ee20f, 3 },
    { 0, 0 },
    { 0x1feee212, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517ee216, 3 },
    { 0x9cae217, 4 },
    { 0x4c6ee216, 4 },
    { 0x8726e217, 3 },
    { 0x1d8ee21a, 3 },
    { 0x89dae21b, 3 },
    { 0xdb0ee21c, 5 },
    { 0, 0 },
    { 0xc4fee21e, 3 },
    { 0, 0 },
    { 0x563ee220, 3 },
    { 0, 0 },
    { 0x462e222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5ae226, 3 },
    { 0, 0 },
    { 0x3016e228, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9ee22e, 3 },
    { 0xc4e2e22e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cee232, 3 },
    { 0x2052e233, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae236, 4 },
    { 0x7a7ee237, 4 },
    { 0, 0 },
    { 0x24f6e239, 4 },
    { 0xa82e23a, 3 },
    { 0, 0 },
    { 0x43eae23c, 3 },
    { 0xf82e23d, 4 },
    { 0x50e2e23d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ee246, 3 },
    { 0xce1ae247, 3 },
    { 0xbbcee248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aee24c, 4 },
    { 0, 0 },
    { 0x8bc2e24e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3ae251, 3 },
    { 0, 0 },
    { 0x2db6e253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166ee257, 3 },
    { 0, 0 },
    { 0xb30ae259, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc566e25c, 3 },
    { 0x123ee25c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ee264, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x589ee267, 3 },
    { 0, 0 },
    { 0x3062e269, 4 },
    { 0xdfa2e269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ee26f, 4 },
    { 0x9486e270, 3 },
    { 0, 0 },
    { 0x9ef6e272, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee285, 4 },
    { 0x851ae286, 3 },
    { 0xd28ee286, 3 },
    { 0, 0 },
    { 0x254ee289, 3 },
    { 0x2d16e28a, 3 },
    { 0xd8f6e28b, 3 },
    { 0xb1cee28a, 4 },
    { 0x6c16e289, 4 },
    { 0x3ed2e28e, 3 },
    { 0xf256e28e, 3 },
    { 0xa4a6e290, 3 },
    { 0x9e86e291, 3 },
    { 0, 0 },
    { 0x4b1ee293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7626e298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eee29c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa612e2a0, 3 },
    { 0xdb4ae2a1, 3 },
    { 0x855ae2a2, 3 },
    { 0x60aee2a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f2e2a7, 3 },
    { 0xc1bae2a8, 3 },
    { 0xbfeee2a9, 3 },
    { 0x4eaae2aa, 3 },
    { 0, 0 },
    { 0xc6aee2ac, 4 },
    { 0xbf72e2ac, 4 },
    { 0xb246e2ac, 3 },
    { 0x9aaee2ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d76e2b2, 4 },
    { 0x5ac2e2b3, 4 },
    { 0, 0 },
    { 0xcf1ae2b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbae2ba, 3 },
    { 0, 0 },
    { 0x8792e2bc, 3 },
    { 0xc13ae2bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af6e2c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ee2ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ee2d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c2e2dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb6e2e6, 4 },
    { 0x5d62e2e7, 3 },
    { 0, 0 },
    { 0x939ae2e9, 3 },
    { 0, 0 },
    { 0x2382e2eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e22e2ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ee2f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e2f5, 3 },
    { 0x8e52e2f5, 3 },
    { 0xde26e2f7, 3 },
    { 0x3cc6e2f7, 3 },
    { 0, 0 },
    { 0x4586e2fa, 4 },
    { 0xf4e6e2fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3ee302, 3 },
    { 0, 0 },
    { 0x981ee304, 3 },
    { 0x66f6e305, 4 },
    { 0, 0 },
    { 0xe3e6e307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46e30b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba6e30f, 4 },
    { 0, 0 },
    { 0x3ef6e311, 3 },
    { 0, 0 },
    { 0x2e0ee313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ae319, 4 },
    { 0xac4ae31a, 3 },
    { 0x3122e31b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9876e324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2ae327, 3 },
    { 0xbf52e328, 3 },
    { 0, 0 },
    { 0xe9e2e32a, 3 },
    { 0x9892e32a, 3 },
    { 0x412e32c, 4 },
    { 0x4b0ae32d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90ae330, 3 },
    { 0xd372e331, 4 },
    { 0xe94ee330, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6e336, 5 },
    { 0x7542e337, 3 },
    { 0, 0 },
    { 0x524ae339, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd32e342, 3 },
    { 0xf7c2e342, 3 },
    { 0xa6d2e344, 3 },
    { 0xbcfee345, 3 },
    { 0, 0 },
    { 0x3afae347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beee34a, 5 },
    { 0xbcfae34b, 3 },
    { 0x6d66e34c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3492e34f, 3 },
    { 0xe496e350, 5 },
    { 0, 0 },
    { 0x3c92e352, 3 },
    { 0x2382e353, 4 },
    { 0x397ae354, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eee359, 3 },
    { 0x1f86e35a, 3 },
    { 0x1c7ae35b, 3 },
    { 0x252ee35c, 3 },
    { 0xffc2e35b, 3 },
    { 0x3db2e35e, 3 },
    { 0x5056e35f, 3 },
    { 0x6816e360, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4002e368, 3 },
    { 0, 0 },
    { 0x2106e36a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2e36d, 3 },
    { 0xe2ee36e, 3 },
    { 0x782e36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe392e377, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c2e37a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5916e37d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb2e382, 4 },
    { 0xde1ee383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1232e386, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9486e38d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ee390, 4 },
    { 0, 0 },
    { 0x22e2e392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fae395, 4 },
    { 0, 0 },
    { 0xafee397, 3 },
    { 0xc75ee398, 4 },
    { 0, 0 },
    { 0x682e39a, 3 },
    { 0, 0 },
    { 0x92e6e39c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6746e3a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb6e3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4212e3a9, 3 },
    { 0xcb22e3aa, 4 },
    { 0, 0 },
    { 0x559ee3ac, 3 },
    { 0x29e2e3ad, 3 },
    { 0xd7aae3ac, 3 },
    { 0, 0 },
    { 0x16aee3b0, 3 },
    { 0xc6bee3b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1636e3b5, 3 },
    { 0, 0 },
    { 0xf242e3b7, 5 },
    { 0xf7a6e3b7, 4 },
    { 0, 0 },
    { 0xe3f6e3ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36ae3bd, 4 },
    { 0xb96e3be, 3 },
    { 0x2922e3bf, 3 },
    { 0, 0 },
    { 0x1c7ee3c1, 3 },
    { 0x5286e3c1, 3 },
    { 0x5b7ee3c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6ae3c9, 4 },
    { 0, 0 },
    { 0x98c2e3cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6582e3d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb2e3d3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89ee3d6, 3 },
    { 0xfd6ee3d6, 4 },
    { 0x9492e3d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x642e3db, 3 },
    { 0xeaaae3dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e2e3e4, 3 },
    { 0x33aae3e5, 3 },
    { 0xc22ee3e5, 4 },
    { 0, 0 },
    { 0x75bae3e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdae3f1, 4 },
    { 0x88ee3f2, 3 },
    { 0x9282e3f1, 3 },
    { 0xd5f6e3f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fee3f7, 4 },
    { 0xa36ee3f8, 3 },
    { 0xda0ae3f9, 3 },
    { 0x207ee3fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe262e3fd, 3 },
    { 0, 0 },
    { 0x30b2e3ff, 3 },
    { 0xe9d6e400, 3 },
    { 0, 0 },
    { 0x6acee402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c2e407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6e40b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bee411, 3 },
    { 0xe4b6e411, 3 },
    { 0, 0 },
    { 0x3a76e414, 3 },
    { 0xae46e415, 4 },
    { 0xef66e415, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76ae41e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de6e422, 3 },
    { 0, 0 },
    { 0x9616e424, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edee427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd42e42a, 3 },
    { 0xd1ee42b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15ae42e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408ae432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548ae438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aae43d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x872e440, 3 },
    { 0x5196e441, 3 },
    { 0xabeee442, 3 },
    { 0x6fa6e440, 3 },
    { 0xb9d6e443, 4 },
    { 0x459ee440, 3 },
    { 0xa3bae445, 4 },
    { 0x8ea2e440, 4 },
    { 0xa32ae446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebee44e, 3 },
    { 0x2132e44f, 4 },
    { 0xbd52e44f, 4 },
    { 0x2912e451, 4 },
    { 0x2212e452, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcae458, 3 },
    { 0x3c0ae459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448ee45d, 5 },
    { 0, 0 },
    { 0xa69ee45f, 3 },
    { 0xed2ae460, 3 },
    { 0, 0 },
    { 0x76e6e462, 3 },
    { 0, 0 },
    { 0x370ae464, 3 },
    { 0x945ae465, 3 },
    { 0x80aae466, 4 },
    { 0x5442e467, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f6e46e, 3 },
    { 0xb5e6e46e, 4 },
    { 0x57fae470, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fae478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabee482, 3 },
    { 0xeeaee482, 4 },
    { 0xaa02e482, 3 },
    { 0x7e0ee483, 3 },
    { 0xb1f6e486, 3 },
    { 0x24fae487, 4 },
    { 0x6502e488, 4 },
    { 0xfa76e489, 3 },
    { 0x63e2e48a, 4 },
    { 0x4046e48b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e06e48f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe82e494, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282ae49b, 3 },
    { 0xa16ae49b, 3 },
    { 0, 0 },
    { 0x9e0ee49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eae4a9, 3 },
    { 0x316ae4aa, 3 },
    { 0, 0 },
    { 0x3cbee4ac, 3 },
    { 0, 0 },
    { 0x1cee4ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x767ee4b1, 3 },
    { 0x22bee4b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa42e4b5, 3 },
    { 0x4e86e4b6, 4 },
    { 0xf4aee4b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35eae4ba, 3 },
    { 0x59e6e4ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb6e4c0, 3 },
    { 0x9172e4c1, 4 },
    { 0x889ee4c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4712e4c5, 3 },
    { 0x21a2e4c6, 3 },
    { 0x279ee4c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4666e4cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1ae4cf, 5 },
    { 0x2fc2e4d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ae4d4, 4 },
    { 0x6926e4d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8202e4d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5ae4db, 3 },
    { 0xd9eae4dc, 3 },
    { 0, 0 },
    { 0x415ae4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628ee4e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x193ae4e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ae4ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceae4f1, 4 },
    { 0xc0dae4f1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e4f6, 3 },
    { 0, 0 },
    { 0xe8eae4f8, 4 },
    { 0xc03ae4f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472ee4ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bae503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2ee508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb02e510, 4 },
    { 0, 0 },
    { 0x593ae512, 4 },
    { 0, 0 },
    { 0x5982e514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dae517, 3 },
    { 0, 0 },
    { 0x1082e519, 3 },
    { 0x287ae51a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976e51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1ee525, 3 },
    { 0x8e32e526, 3 },
    { 0, 0 },
    { 0xb89ae528, 3 },
    { 0, 0 },
    { 0xd406e52a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9666e532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbee535, 4 },
    { 0x59a2e535, 3 },
    { 0, 0 },
    { 0x6c5ae538, 3 },
    { 0xc97ee538, 3 },
    { 0x40a6e53a, 4 },
    { 0xbe8ae53a, 3 },
    { 0x7efee53c, 3 },
    { 0, 0 },
    { 0xcb32e53e, 4 },
    { 0, 0 },
    { 0x358ee540, 3 },
    { 0x485ee540, 4 },
    { 0xa3a6e540, 3 },
    { 0, 0 },
    { 0x38cee544, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6ae54a, 4 },
    { 0x9586e54b, 3 },
    { 0x9072e54c, 4 },
    { 0x28aee54a, 4 },
    { 0, 0 },
    { 0x671ee54f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cee552, 4 },
    { 0, 0 },
    { 0xbd62e554, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97ae557, 3 },
    { 0, 0 },
    { 0xfc66e559, 3 },
    { 0, 0 },
    { 0x24eee55b, 3 },
    { 0, 0 },
    { 0x2ee55d, 3 },
    { 0x3b32e55e, 4 },
    { 0, 0 },
    { 0xd42e560, 4 },
    { 0, 0 },
    { 0x7b62e562, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fee565, 3 },
    { 0xa8a6e565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66ee56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ae573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee580, 5 },
    { 0, 0 },
    { 0xf1c6e582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2e589, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6ae58c, 4 },
    { 0xc59ae58d, 4 },
    { 0x494ee58e, 3 },
    { 0xc252e58c, 3 },
    { 0, 0 },
    { 0xd756e591, 3 },
    { 0xdd3ee591, 5 },
    { 0xa022e593, 3 },
    { 0x86d2e594, 4 },
    { 0x2376e593, 3 },
    { 0x1bd6e594, 4 },
    { 0x4e3ee597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cee59a, 3 },
    { 0x4baee59a, 3 },
    { 0x303ee59c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d36e5a1, 3 },
    { 0xf0cae5a2, 3 },
    { 0x7e76e5a2, 4 },
    { 0x5d7ae5a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c6e5ab, 3 },
    { 0xb25ae5ac, 3 },
    { 0xe7d6e5ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc32e5b3, 3 },
    { 0, 0 },
    { 0xaca2e5b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dae5b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2aae5be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3312e5c5, 3 },
    { 0x14b6e5c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7412e5d0, 3 },
    { 0x97b2e5d1, 5 },
    { 0, 0 },
    { 0x522ae5d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f32e5d6, 3 },
    { 0xdd7ee5d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82e5e1, 3 },
    { 0x413ae5e2, 3 },
    { 0x6d02e5e2, 4 },
    { 0x9deae5e1, 3 },
    { 0x55e2e5e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b22e5e9, 4 },
    { 0, 0 },
    { 0x4d4ee5eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f6e5ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9ae5f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ae5f7, 3 },
    { 0xb96e5f8, 3 },
    { 0xa5f2e5f8, 3 },
    { 0x64bee5fa, 4 },
    { 0xe12ae5fb, 3 },
    { 0xa852e5fc, 5 },
    { 0, 0 },
    { 0xdcd2e5fe, 3 },
    { 0, 0 },
    { 0xe4ee600, 4 },
    { 0, 0 },
    { 0xb302e602, 3 },
    { 0x91ee602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fae608, 3 },
    { 0x35b2e608, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7ee60c, 3 },
    { 0x982ae60d, 3 },
    { 0x5912e60c, 3 },
    { 0, 0 },
    { 0xd146e610, 4 },
    { 0x5962e611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ee615, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb172e620, 3 },
    { 0xce6e620, 4 },
    { 0xd1bee621, 3 },
    { 0x55ee620, 3 },
    { 0xa0a6e624, 3 },
    { 0x93bee625, 3 },
    { 0x28cee620, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cee62a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b2e630, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340ee634, 4 },
    { 0x9792e634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efae638, 3 },
    { 0, 0 },
    { 0x8c46e63a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb6e63e, 4 },
    { 0xa236e63f, 3 },
    { 0xc63ee63f, 3 },
    { 0, 0 },
    { 0xdb3ee642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96ee646, 4 },
    { 0xadeee646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf012e64d, 4 },
    { 0xd376e64d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ee651, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3102e654, 3 },
    { 0xffe6e655, 3 },
    { 0x3516e654, 5 },
    { 0xb002e654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c96e65d, 4 },
    { 0xff66e65e, 3 },
    { 0x903ee65d, 3 },
    { 0x743ae65f, 3 },
    { 0x3882e65f, 3 },
    { 0x8162e662, 3 },
    { 0xcf3ee663, 5 },
    { 0x51eee661, 3 },
    { 0, 0 },
    { 0x36ee666, 3 },
    { 0, 0 },
    { 0xef3ee668, 4 },
    { 0x87aae668, 3 },
    { 0x1a92e66a, 3 },
    { 0xcd26e66b, 3 },
    { 0x35eae66c, 3 },
    { 0x4aaee66d, 3 },
    { 0, 0 },
    { 0xae0ae66f, 3 },
    { 0xa732e670, 3 },
    { 0xce9ae671, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2e67a, 3 },
    { 0xf1e6e67b, 3 },
    { 0, 0 },
    { 0xc5aee67d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dee682, 3 },
    { 0x324ae682, 3 },
    { 0x7e2e684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff2e688, 3 },
    { 0, 0 },
    { 0xc692e68a, 4 },
    { 0x92eae68a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76e695, 3 },
    { 0x39a2e695, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d2e69e, 3 },
    { 0xdc4ee69f, 3 },
    { 0xd282e69f, 3 },
    { 0x95aee6a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2ee6a7, 3 },
    { 0, 0 },
    { 0x5c56e6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaae6af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab2e6b5, 3 },
    { 0x300ee6b6, 3 },
    { 0, 0 },
    { 0x29fae6b8, 3 },
    { 0xfd06e6b9, 3 },
    { 0, 0 },
    { 0x48dae6bb, 3 },
    { 0, 0 },
    { 0x99e2e6bd, 3 },
    { 0, 0 },
    { 0x662ae6bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fae6c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb42e6c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32e6c9, 4 },
    { 0xae4ae6c9, 3 },
    { 0, 0 },
    { 0x9192e6cc, 3 },
    { 0x47bae6cd, 4 },
    { 0xe12ae6ce, 4 },
    { 0x2946e6cd, 3 },
    { 0, 0 },
    { 0x56a6e6d1, 3 },
    { 0xff12e6d2, 3 },
    { 0xacaae6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426e6d7, 4 },
    { 0x6e36e6d8, 3 },
    { 0xe326e6d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ae6e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf86e6e6, 4 },
    { 0xb712e6e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916e6ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b2e6f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30ee6f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eee6fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2e700, 3 },
    { 0x9f02e701, 3 },
    { 0x63e2e701, 3 },
    { 0x3206e700, 3 },
    { 0x949ee704, 3 },
    { 0x38a6e705, 4 },
    { 0x97aee706, 3 },
    { 0xc952e707, 3 },
    { 0x644ae703, 3 },
    { 0, 0 },
    { 0xc766e70a, 3 },
    { 0x45dee70b, 3 },
    { 0x76aae70b, 4 },
    { 0xa282e70d, 3 },
    { 0x7f8ae70d, 3 },
    { 0x5206e70f, 4 },
    { 0, 0 },
    { 0xac3ae711, 4 },
    { 0x1a2ae712, 3 },
    { 0xbd6ee712, 3 },
    { 0xdaa2e712, 3 },
    { 0, 0 },
    { 0xb83ee716, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5002e71e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc952e725, 3 },
    { 0x670ae725, 3 },
    { 0x689ae726, 4 },
    { 0xe206e728, 4 },
    { 0x33ae729, 5 },
    { 0x8cf6e729, 3 },
    { 0x3e4ee729, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cee730, 3 },
    { 0x4982e731, 4 },
    { 0xac46e730, 3 },
    { 0xfac6e733, 3 },
    { 0x43d2e734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438ee738, 3 },
    { 0x49eae738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2752e73e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3842e744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11ae74b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2ee75c, 4 },
    { 0, 0 },
    { 0x83ae75e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2ae764, 4 },
    { 0, 0 },
    { 0xbac2e766, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26e76a, 3 },
    { 0xf986e76a, 3 },
    { 0, 0 },
    { 0xb7aae76d, 4 },
    { 0x7a6e76e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f6e771, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6e776, 3 },
    { 0xfe1ee777, 3 },
    { 0, 0 },
    { 0x102ee779, 3 },
    { 0x834ae779, 4 },
    { 0xb772e77b, 3 },
    { 0xf60ee779, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b6e782, 3 },
    { 0xcb42e782, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd842e786, 3 },
    { 0x252e786, 3 },
    { 0x899ae786, 3 },
    { 0xacee789, 3 },
    { 0x2072e787, 3 },
    { 0x6a0ae78b, 3 },
    { 0x89cee788, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec2e791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6e797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196e79c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4ae7a3, 5 },
    { 0x8ffae7a3, 3 },
    { 0xecb2e7a4, 3 },
    { 0xe45ae7a6, 3 },
    { 0x3fe2e7a6, 3 },
    { 0x8cd2e7a5, 3 },
    { 0x67aae7a9, 3 },
    { 0x4702e7aa, 4 },
    { 0x2856e7ab, 3 },
    { 0x6872e7ab, 3 },
    { 0x969ae7ad, 5 },
    { 0xed6ae7ae, 3 },
    { 0xe3eee7aa, 4 },
    { 0x7352e7b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430ee7b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a6e7bd, 3 },
    { 0x2e9ae7be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d16e7c4, 4 },
    { 0x10ae7c5, 3 },
    { 0xdb6e7c5, 3 },
    { 0xf42e7c7, 3 },
    { 0x733ee7c8, 3 },
    { 0xe7fae7c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde2e7cd, 3 },
    { 0, 0 },
    { 0x927ae7cf, 3 },
    { 0xd29ae7cf, 3 },
    { 0xaf2ee7d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e06e7d4, 3 },
    { 0x3fe2e7d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a6e7d9, 3 },
    { 0x9abae7d9, 3 },
    { 0xb7fee7db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe726e7de, 3 },
    { 0x3e66e7df, 3 },
    { 0x81f2e7df, 3 },
    { 0xa58ee7e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ae7e4, 3 },
    { 0x6e2e7e4, 3 },
    { 0xc952e7e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b32e7ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9ee7f4, 5 },
    { 0x8e0ae7f5, 3 },
    { 0, 0 },
    { 0x3afee7f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9ae7fe, 3 },
    { 0, 0 },
    { 0x42aee800, 3 },
    { 0, 0 },
    { 0x79a6e802, 3 },
    { 0x748ee803, 4 },
    { 0xc832e803, 3 },
    { 0x69a2e804, 3 },
    { 0, 0 },
    { 0xe02e807, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eee80a, 3 },
    { 0x49e2e80b, 4 },
    { 0, 0 },
    { 0x8bfae80d, 3 },
    { 0x6e62e80d, 3 },
    { 0, 0 },
    { 0x56aee810, 3 },
    { 0x2b16e811, 3 },
    { 0xa186e812, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05ae81c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfae823, 3 },
    { 0xed46e824, 3 },
    { 0xfbcae823, 3 },
    { 0, 0 },
    { 0xc25ae827, 3 },
    { 0x345ae828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915ae82c, 3 },
    { 0xf2e82c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37ae830, 3 },
    { 0x3b16e831, 4 },
    { 0x69aee831, 3 },
    { 0, 0 },
    { 0x6076e834, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dae837, 3 },
    { 0, 0 },
    { 0xdf7ee839, 4 },
    { 0xbbf6e839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ae83d, 3 },
    { 0, 0 },
    { 0x88ee83f, 3 },
    { 0xd1f2e83f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe26e843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e2e84a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc682e84d, 3 },
    { 0, 0 },
    { 0x35ee84f, 4 },
    { 0xe27ee850, 3 },
    { 0, 0 },
    { 0xfc0ae852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x541ae855, 3 },
    { 0, 0 },
    { 0xf0bae857, 3 },
    { 0x96bee857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd982e85f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa106e866, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67ae86a, 4 },
    { 0x75f6e86b, 4 },
    { 0xe3fee86c, 3 },
    { 0x894ae86c, 3 },
    { 0xeae86e, 3 },
    { 0x299ae86f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ae872, 3 },
    { 0x9e22e873, 4 },
    { 0, 0 },
    { 0x721ae875, 3 },
    { 0x9f42e876, 3 },
    { 0, 0 },
    { 0x4eb6e878, 3 },
    { 0xc332e879, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efae87c, 3 },
    { 0x5b7ee87c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ee880, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bae884, 3 },
    { 0, 0 },
    { 0x4b72e886, 3 },
    { 0, 0 },
    { 0x70b6e888, 3 },
    { 0xf656e889, 4 },
    { 0x76fae88a, 4 },
    { 0x122e88b, 3 },
    { 0x49aae88b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cae88f, 3 },
    { 0x8182e890, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x147ee893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefae896, 4 },
    { 0xb2e896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dee89a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ee89e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ae8a3, 3 },
    { 0, 0 },
    { 0x5266e8a5, 3 },
    { 0xe75ae8a6, 3 },
    { 0x8c2ee8a7, 4 },
    { 0x681ee8a8, 3 },
    { 0, 0 },
    { 0x405ee8aa, 3 },
    { 0x277ae8ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ae8b1, 3 },
    { 0x1a6e8b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426e8b5, 4 },
    { 0xad6e8b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed46e8b9, 3 },
    { 0xfb26e8b9, 4 },
    { 0, 0 },
    { 0x7b42e8bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819ee8c3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d86e8c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcee8cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99ee8cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85ae8d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ae8d7, 3 },
    { 0xca0ae8d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b36e8db, 4 },
    { 0, 0 },
    { 0xad2ae8dd, 3 },
    { 0xe2f6e8de, 4 },
    { 0xb922e8df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ee8e2, 3 },
    { 0x5bf2e8e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee8ed, 3 },
    { 0x2492e8ee, 3 },
    { 0xc94ee8ee, 4 },
    { 0xba66e8ee, 3 },
    { 0xaf5ee8ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5ae8f4, 3 },
    { 0x6f36e8f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bee8f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd92e8fb, 4 },
    { 0xfa0ee8fc, 4 },
    { 0, 0 },
    { 0xfca2e8fe, 4 },
    { 0, 0 },
    { 0x9eaee900, 3 },
    { 0, 0 },
    { 0x22d2e902, 3 },
    { 0x607ae903, 3 },
    { 0, 0 },
    { 0xa842e905, 4 },
    { 0x4c1ee906, 3 },
    { 0, 0 },
    { 0x191ee908, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d6e90b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fee90e, 3 },
    { 0xd29ae90f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980ae914, 4 },
    { 0xf6a6e915, 3 },
    { 0x39aee915, 3 },
    { 0x997ae917, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd76e91e, 4 },
    { 0xdf76e91f, 3 },
    { 0xaeeae920, 3 },
    { 0xb47ee920, 3 },
    { 0xa936e922, 3 },
    { 0x33a2e91e, 3 },
    { 0, 0 },
    { 0x5302e925, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91ae929, 4 },
    { 0x3a7ee92a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bae933, 3 },
    { 0, 0 },
    { 0x53b2e935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dae939, 3 },
    { 0, 0 },
    { 0x7bbee93b, 3 },
    { 0xd0ae93b, 3 },
    { 0, 0 },
    { 0x72a2e93e, 4 },
    { 0xf012e93e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x134ae942, 3 },
    { 0xea9ee942, 3 },
    { 0x7d96e944, 4 },
    { 0, 0 },
    { 0x8166e946, 3 },
    { 0x4dc2e946, 3 },
    { 0x277ae948, 4 },
    { 0x922e949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb242e94e, 4 },
    { 0x819ee94f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76ae952, 3 },
    { 0x176e953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1936e956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81ae960, 3 },
    { 0x2fa2e961, 4 },
    { 0xb14ae962, 3 },
    { 0xa25ee962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586e966, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2e96a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff6e96e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdae971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc6e976, 3 },
    { 0, 0 },
    { 0xfeeee978, 3 },
    { 0, 0 },
    { 0xae62e97a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d2e97d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3ae982, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6182e985, 3 },
    { 0, 0 },
    { 0xb722e987, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce2e98c, 3 },
    { 0, 0 },
    { 0x4676e98e, 3 },
    { 0x524ae98f, 5 },
    { 0, 0 },
    { 0x6526e991, 3 },
    { 0x4c66e992, 4 },
    { 0x841ee991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ee997, 3 },
    { 0xa2ee997, 3 },
    { 0xc5dee999, 3 },
    { 0, 0 },
    { 0xc0aae99b, 3 },
    { 0xf206e99c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954ae9a2, 3 },
    { 0, 0 },
    { 0x4a16e9a4, 3 },
    { 0x4f56e9a4, 3 },
    { 0xd056e9a5, 4 },
    { 0x5de2e9a4, 5 },
    { 0xc232e9a7, 3 },
    { 0, 0 },
    { 0x12fae9aa, 3 },
    { 0x189ee9ab, 3 },
    { 0x8776e9ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eae9af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ae9b3, 3 },
    { 0, 0 },
    { 0xa5b2e9b5, 3 },
    { 0x2dfae9b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64ae9ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bee9c1, 3 },
    { 0x235ae9c2, 3 },
    { 0xcd2ae9c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d2e9c7, 3 },
    { 0, 0 },
    { 0xa1d6e9c9, 3 },
    { 0x9f3ae9ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa536e9d2, 3 },
    { 0x4ad6e9d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913ee9d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446ee9df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x334ae9e2, 3 },
    { 0, 0 },
    { 0xe5eae9e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3ee9e9, 3 },
    { 0x6512e9ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ee9ed, 4 },
    { 0xbbcee9ed, 3 },
    { 0x35cee9ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6e9f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa2e9f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e26e9fe, 3 },
    { 0x76fae9ff, 3 },
    { 0xfe3ae9ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x136aea03, 4 },
    { 0xc7ceea04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8aea0b, 4 },
    { 0xa79eea0c, 3 },
    { 0, 0 },
    { 0xcc02ea0e, 3 },
    { 0x4836ea0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd2ea13, 3 },
    { 0, 0 },
    { 0xc32aea15, 4 },
    { 0xb446ea16, 3 },
    { 0x8506ea17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9eea1b, 3 },
    { 0x39f2ea1b, 3 },
    { 0xe87eea1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106ea22, 3 },
    { 0, 0 },
    { 0x154aea24, 3 },
    { 0xf6a2ea25, 4 },
    { 0xdc1aea26, 3 },
    { 0x15b2ea27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3aea3c, 4 },
    { 0x7e4aea3d, 3 },
    { 0xab76ea3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eeea42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaea45, 3 },
    { 0, 0 },
    { 0x404aea47, 3 },
    { 0, 0 },
    { 0xacbeea49, 3 },
    { 0, 0 },
    { 0x6b1aea4b, 3 },
    { 0xedd6ea4c, 3 },
    { 0x9126ea4d, 3 },
    { 0x4506ea4c, 3 },
    { 0xf42eea4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa846ea52, 3 },
    { 0xd322ea53, 3 },
    { 0x4406ea54, 3 },
    { 0xa8ceea55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37eea5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5eea63, 3 },
    { 0xa986ea63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4352ea68, 5 },
    { 0, 0 },
    { 0xa576ea6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af6ea71, 4 },
    { 0x7f02ea71, 3 },
    { 0, 0 },
    { 0x45aaea74, 3 },
    { 0, 0 },
    { 0x2c9eea76, 3 },
    { 0, 0 },
    { 0x5feaea78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eaea7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a12ea7f, 3 },
    { 0x3d7aea7f, 3 },
    { 0x2c52ea80, 3 },
    { 0xa376ea82, 3 },
    { 0x162eea83, 3 },
    { 0xe0e2ea82, 3 },
    { 0, 0 },
    { 0x4e5aea86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22ea89, 3 },
    { 0xa646ea89, 4 },
    { 0xe0aeea8b, 3 },
    { 0x8cd6ea89, 4 },
    { 0x4006ea8d, 3 },
    { 0x2b32ea8d, 3 },
    { 0x36c2ea8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52ea97, 3 },
    { 0x1412ea98, 3 },
    { 0x28eaea99, 4 },
    { 0x3f62ea9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285aea9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbaeaa3, 3 },
    { 0x336eeaa4, 3 },
    { 0, 0 },
    { 0x1096eaa6, 4 },
    { 0x5c4eeaa7, 3 },
    { 0, 0 },
    { 0x1c12eaa9, 3 },
    { 0, 0 },
    { 0xa9baeaab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaeab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeeeab7, 4 },
    { 0xe2caeab8, 3 },
    { 0xbdaaeab7, 3 },
    { 0xc2a6eab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d6eabd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9eeac6, 3 },
    { 0xbaeeac7, 3 },
    { 0xa75eeac7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaeeacb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x634aeace, 3 },
    { 0x94e2eacf, 4 },
    { 0xe4e2ead0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5deead5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x223aead8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69aeadb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32eeae0, 3 },
    { 0x8996eae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa286eae9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a72eaed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5042eaf0, 3 },
    { 0, 0 },
    { 0x7a22eaf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43eeaf5, 3 },
    { 0x31caeaf5, 5 },
    { 0, 0 },
    { 0x9e9aeaf8, 3 },
    { 0, 0 },
    { 0xb24aeafa, 3 },
    { 0, 0 },
    { 0x7de2eafc, 5 },
    { 0, 0 },
    { 0x6a3eeafe, 3 },
    { 0x13aeeaff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6eb06, 4 },
    { 0x824aeb07, 4 },
    { 0xde72eb07, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2eb16, 4 },
    { 0, 0 },
    { 0xa782eb18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e32eb1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c6eb22, 4 },
    { 0, 0 },
    { 0xd28aeb24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4aeb28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2aeb2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18aeb2e, 3 },
    { 0xf3feeb2f, 3 },
    { 0, 0 },
    { 0x416eb31, 4 },
    { 0x8faaeb32, 3 },
    { 0x94ceeb33, 3 },
    { 0, 0 },
    { 0x1adeeb35, 3 },
    { 0x13faeb36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276eb3c, 3 },
    { 0x9932eb3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88aaeb44, 3 },
    { 0x46aeeb45, 4 },
    { 0, 0 },
    { 0xef2eeb47, 3 },
    { 0, 0 },
    { 0xf8a6eb49, 4 },
    { 0x6d6aeb49, 4 },
    { 0, 0 },
    { 0xd6e2eb4c, 4 },
    { 0xd7d6eb4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143eeb54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a6eb58, 3 },
    { 0x7feeeb59, 4 },
    { 0x44eeeb59, 3 },
    { 0xf8e2eb59, 3 },
    { 0xe5e6eb5c, 3 },
    { 0x123aeb5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937eeb61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42eb65, 4 },
    { 0, 0 },
    { 0x98feeb67, 3 },
    { 0x4e56eb67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f6eb6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8836eb78, 4 },
    { 0xce0aeb79, 3 },
    { 0, 0 },
    { 0x212eb7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x544eeb7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3aeb82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f2eb8a, 3 },
    { 0xb126eb8b, 3 },
    { 0x9086eb8a, 4 },
    { 0, 0 },
    { 0x2eb2eb8e, 3 },
    { 0, 0 },
    { 0x7cdeeb90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eaeb93, 4 },
    { 0x4686eb94, 3 },
    { 0x6fbaeb93, 4 },
    { 0xda56eb93, 4 },
    { 0xe906eb96, 4 },
    { 0xaa2eb98, 3 },
    { 0x318eeb99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362eb9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aeba3, 3 },
    { 0x8e46eba4, 4 },
    { 0xaadaeba3, 3 },
    { 0xf926eba5, 3 },
    { 0xe91aeba3, 3 },
    { 0xe8baeba8, 4 },
    { 0x8abaeba8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d2ebb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad42ebb5, 3 },
    { 0xf6e6ebb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc636ebc0, 3 },
    { 0x56a2ebc1, 3 },
    { 0xb7d6ebc2, 3 },
    { 0x82aeebc0, 3 },
    { 0x1df2ebc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa346ebcb, 4 },
    { 0xf03eebcb, 3 },
    { 0, 0 },
    { 0x8c1aebce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39aebd2, 3 },
    { 0x2dbeebd2, 4 },
    { 0xb76ebd4, 3 },
    { 0x4572ebd5, 4 },
    { 0xc5aaebd5, 3 },
    { 0xe90aebd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a2ebe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x629eebe4, 3 },
    { 0x9f0eebe4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5faebe8, 3 },
    { 0xca12ebe9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d2ebec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aeebf0, 3 },
    { 0xc7baebf0, 3 },
    { 0xd7b2ebf2, 3 },
    { 0x9cfaebf3, 4 },
    { 0, 0 },
    { 0x84f2ebf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7beebfb, 4 },
    { 0, 0 },
    { 0x4d56ebfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x339aec00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aec07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5aec0a, 3 },
    { 0xe066ec0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daaec11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faec15, 3 },
    { 0xabb6ec15, 4 },
    { 0xe026ec15, 4 },
    { 0x817aec18, 3 },
    { 0xb196ec18, 3 },
    { 0x938aec1a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149eec1f, 3 },
    { 0, 0 },
    { 0xed4eec21, 3 },
    { 0xbe46ec22, 3 },
    { 0x651eec21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793eec27, 3 },
    { 0xb7aaec28, 3 },
    { 0, 0 },
    { 0x364aec2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082ec30, 3 },
    { 0x59e6ec31, 4 },
    { 0x34beec32, 3 },
    { 0x500eec33, 3 },
    { 0xaff6ec31, 3 },
    { 0x14aaec35, 3 },
    { 0x34b2ec36, 3 },
    { 0xe66ec37, 3 },
    { 0x286eec38, 4 },
    { 0x819eec39, 3 },
    { 0x12e2ec39, 3 },
    { 0xd64aec3b, 3 },
    { 0x188eec36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x68eec3f, 3 },
    { 0xb68aec40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd6ec43, 3 },
    { 0, 0 },
    { 0x937aec45, 3 },
    { 0x93aec46, 4 },
    { 0, 0 },
    { 0x463aec48, 4 },
    { 0, 0 },
    { 0xb2beec4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852ec4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2eec58, 3 },
    { 0x282aec58, 4 },
    { 0x74aeec5a, 3 },
    { 0x7102ec5a, 4 },
    { 0x6212ec5c, 4 },
    { 0x2e02ec5a, 3 },
    { 0x9a4eec5e, 3 },
    { 0xf4caec5f, 3 },
    { 0x6162ec60, 3 },
    { 0x7c36ec5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa6ec69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802ec6e, 3 },
    { 0xe0baec6f, 3 },
    { 0, 0 },
    { 0x6dd6ec71, 3 },
    { 0xb3c6ec72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab2ec76, 3 },
    { 0x859aec77, 3 },
    { 0xf0c2ec77, 3 },
    { 0, 0 },
    { 0x1ce6ec7a, 3 },
    { 0x2eeec7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26aec82, 3 },
    { 0xbf8aec83, 3 },
    { 0x523aec82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eeec8d, 3 },
    { 0x933eec8d, 3 },
    { 0x92d6ec8e, 3 },
    { 0, 0 },
    { 0xb98eec91, 3 },
    { 0, 0 },
    { 0xf44aec93, 4 },
    { 0x2d4aec94, 3 },
    { 0x4e8eec94, 3 },
    { 0xeb02ec96, 3 },
    { 0, 0 },
    { 0x9fcaec98, 4 },
    { 0, 0 },
    { 0xeafaec9a, 3 },
    { 0x5ab2ec9a, 3 },
    { 0xbc72ec9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10eec9f, 3 },
    { 0x7412eca0, 3 },
    { 0xd8c2eca0, 3 },
    { 0x62eca1, 4 },
    { 0, 0 },
    { 0x106aeca4, 3 },
    { 0xe106eca4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6eca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670eecaf, 3 },
    { 0xb91eecaf, 3 },
    { 0, 0 },
    { 0xec6eecb2, 3 },
    { 0x6186ecb3, 3 },
    { 0xc59eecb3, 3 },
    { 0, 0 },
    { 0x46b2ecb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774aecbb, 3 },
    { 0x9672ecbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795eecc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eaecc5, 3 },
    { 0x3662ecc6, 3 },
    { 0xa796ecc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4eeccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe6ecd1, 3 },
    { 0xc0feecd1, 4 },
    { 0, 0 },
    { 0x36b2ecd4, 5 },
    { 0x4bc2ecd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ecda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7eecde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6eece2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988aece6, 3 },
    { 0x3e1eece6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853aecf6, 3 },
    { 0xb7d2ecf7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6ecfa, 3 },
    { 0, 0 },
    { 0xc76eecfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0aed05, 3 },
    { 0x7d52ed06, 3 },
    { 0x62daed07, 4 },
    { 0xd0feed08, 5 },
    { 0, 0 },
    { 0x496aed0a, 4 },
    { 0xcad2ed0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeaed1b, 4 },
    { 0x60f6ed1b, 3 },
    { 0x9fbaed1c, 3 },
    { 0xd66aed1c, 3 },
    { 0x625eed1e, 4 },
    { 0xf9daed20, 4 },
    { 0xcb06ed21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1912ed26, 3 },
    { 0x532ed26, 3 },
    { 0, 0 },
    { 0x2352ed29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9866ed2d, 3 },
    { 0xa8aaed2e, 3 },
    { 0x83f6ed2f, 3 },
    { 0x7fbeed30, 4 },
    { 0xefb2ed31, 3 },
    { 0xab9eed2d, 3 },
    { 0x41baed33, 3 },
    { 0x9266ed34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaaed37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf916ed3e, 3 },
    { 0x3f9aed3f, 3 },
    { 0x2402ed40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eaed44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1eed4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9406ed50, 3 },
    { 0x440eed51, 3 },
    { 0, 0 },
    { 0x42eed53, 3 },
    { 0x976ed54, 3 },
    { 0xf7feed55, 3 },
    { 0x3932ed53, 3 },
    { 0, 0 },
    { 0x8842ed58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0aed5d, 4 },
    { 0xfc22ed5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a82ed62, 3 },
    { 0xfce2ed62, 3 },
    { 0xc46aed63, 4 },
    { 0x9576ed64, 3 },
    { 0xe22ed66, 4 },
    { 0xfeaeed67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaaed6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed6ed6f, 4 },
    { 0x2746ed70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ed74, 3 },
    { 0x914aed75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1eed79, 3 },
    { 0x6e1aed7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7562ed7f, 3 },
    { 0, 0 },
    { 0x4a96ed81, 3 },
    { 0x9746ed82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf26ed86, 3 },
    { 0xce96ed86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53aed8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dceed90, 3 },
    { 0x9ccaed91, 4 },
    { 0x2df6ed92, 3 },
    { 0xa1d2ed92, 3 },
    { 0xd7c2ed92, 4 },
    { 0x8b76ed95, 3 },
    { 0, 0 },
    { 0x8a5eed97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31eed9a, 3 },
    { 0xb1beed9b, 3 },
    { 0x5a2aed9b, 3 },
    { 0x42e6ed9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e76eda2, 4 },
    { 0xc14aeda2, 3 },
    { 0x65b6eda4, 3 },
    { 0x5196eda5, 3 },
    { 0x391aeda2, 3 },
    { 0x8e62eda7, 3 },
    { 0xbe56eda5, 3 },
    { 0x512eeda6, 4 },
    { 0x4f5eedaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaaedb2, 3 },
    { 0x3536edb2, 3 },
    { 0x5ec2edb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47aedb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec2edbe, 3 },
    { 0x92feedbe, 3 },
    { 0, 0 },
    { 0x5136edc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac6edc4, 3 },
    { 0xd4d2edc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36edc9, 3 },
    { 0x5c46edc9, 4 },
    { 0xd7baedc9, 3 },
    { 0, 0 },
    { 0xddceedcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90eedd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566edd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6eedd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eeeddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd512ede0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18eedf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c6edf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71aedfc, 3 },
    { 0x390eedfd, 3 },
    { 0xa172edfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b06ee02, 4 },
    { 0x2146ee03, 3 },
    { 0x9f32ee02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x102ee07, 3 },
    { 0x6daeee08, 3 },
    { 0x95feee07, 3 },
    { 0xf146ee0a, 5 },
    { 0xcc4aee07, 3 },
    { 0, 0 },
    { 0xb412ee0d, 3 },
    { 0x762eee0e, 5 },
    { 0x9d7aee0e, 3 },
    { 0xc326ee10, 3 },
    { 0xc2deee10, 3 },
    { 0x4832ee12, 3 },
    { 0xc0a6ee10, 4 },
    { 0x4eceee12, 4 },
    { 0x9d02ee0f, 3 },
    { 0, 0 },
    { 0x1292ee17, 3 },
    { 0, 0 },
    { 0x63d6ee19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5826ee1d, 3 },
    { 0, 0 },
    { 0xf192ee1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176ee24, 5 },
    { 0x80feee25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8622ee28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa106ee31, 4 },
    { 0x6a06ee32, 5 },
    { 0xf762ee33, 3 },
    { 0x18faee33, 3 },
    { 0, 0 },
    { 0x3a22ee36, 3 },
    { 0xd856ee36, 3 },
    { 0, 0 },
    { 0x7e2ee39, 3 },
    { 0x12ceee3a, 3 },
    { 0xc28aee3b, 3 },
    { 0x5862ee3b, 3 },
    { 0, 0 },
    { 0x7ed6ee3e, 3 },
    { 0x67b6ee3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faaee46, 3 },
    { 0xcde6ee46, 3 },
    { 0x61e6ee46, 3 },
    { 0xd5caee48, 3 },
    { 0x7222ee47, 3 },
    { 0x3eee4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb692ee51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5042ee54, 4 },
    { 0x63faee55, 3 },
    { 0x51e6ee56, 5 },
    { 0xb0c2ee57, 3 },
    { 0, 0 },
    { 0x5b6eee59, 3 },
    { 0x6f02ee5a, 3 },
    { 0x6056ee5b, 3 },
    { 0, 0 },
    { 0x59a2ee5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24feee60, 3 },
    { 0xed06ee61, 3 },
    { 0xcb22ee61, 3 },
    { 0x6d26ee62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2aee67, 3 },
    { 0x5d62ee68, 4 },
    { 0x79feee69, 3 },
    { 0xbbceee6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b6ee6d, 4 },
    { 0, 0 },
    { 0x9416ee6f, 3 },
    { 0x8022ee70, 3 },
    { 0x3edeee71, 3 },
    { 0xf10eee71, 3 },
    { 0x57c2ee70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4812ee7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810aee80, 4 },
    { 0xbd6aee81, 3 },
    { 0x3bc2ee82, 3 },
    { 0x5846ee82, 3 },
    { 0, 0 },
    { 0xc79eee85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eee88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7196ee8b, 3 },
    { 0x43eee8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebaee9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e2ee9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de6eea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942eea9, 3 },
    { 0, 0 },
    { 0x49faeeab, 3 },
    { 0x7cf2eeac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0eeeb0, 3 },
    { 0, 0 },
    { 0xf58aeeb2, 3 },
    { 0, 0 },
    { 0x91f6eeb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3aeebc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x272aeebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402eec6, 3 },
    { 0xb93eeec7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542eecb, 4 },
    { 0xe676eecc, 3 },
    { 0, 0 },
    { 0x7602eece, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e66eed2, 3 },
    { 0, 0 },
    { 0x37ceeed4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e2eedb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x200aeede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbaeee4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184aeee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845aeeef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acaeef4, 3 },
    { 0x608aeef5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbaeef8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebeef05, 4 },
    { 0x97aeef05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe2ef0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8baef11, 3 },
    { 0x6afeef12, 3 },
    { 0xffaeef11, 3 },
    { 0, 0 },
    { 0xd1ceef15, 3 },
    { 0, 0 },
    { 0x360eef17, 3 },
    { 0x1fb6ef18, 4 },
    { 0, 0 },
    { 0x4562ef1a, 3 },
    { 0, 0 },
    { 0x9d56ef1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b6ef21, 4 },
    { 0x85caef22, 4 },
    { 0x84d2ef21, 4 },
    { 0x8a82ef23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b6ef27, 4 },
    { 0, 0 },
    { 0x84eeef29, 3 },
    { 0, 0 },
    { 0x50daef2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0eef2f, 3 },
    { 0xf322ef30, 3 },
    { 0x7faef31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3eef35, 3 },
    { 0x6fbeef36, 3 },
    { 0, 0 },
    { 0x6c4eef38, 3 },
    { 0x891aef39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf02ef3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8216ef41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9aef4b, 3 },
    { 0, 0 },
    { 0x167eef4d, 3 },
    { 0xd7deef4e, 4 },
    { 0xfad6ef4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce2ef52, 3 },
    { 0x1c92ef53, 3 },
    { 0x659aef52, 3 },
    { 0xe082ef55, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9402ef5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17aef66, 4 },
    { 0xeeb2ef67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81aef6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea2ef6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636eef73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772ef76, 3 },
    { 0x30ceef77, 4 },
    { 0, 0 },
    { 0x21baef79, 3 },
    { 0x56e6ef7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af6ef7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807eef83, 3 },
    { 0xcc7eef84, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b6ef87, 3 },
    { 0xe5f2ef88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadaef8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29deef91, 3 },
    { 0, 0 },
    { 0xcd22ef93, 3 },
    { 0x94aef94, 4 },
    { 0x94eeef95, 3 },
    { 0xb1ceef93, 3 },
    { 0, 0 },
    { 0xe872ef98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaaef9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5eefa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a2efa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbaefa8, 3 },
    { 0x9cfaefa9, 3 },
    { 0x2322efa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eeefae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baefb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caefb7, 3 },
    { 0x4256efb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aaefbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eefc0, 3 },
    { 0x174aefc0, 3 },
    { 0x6daaefc0, 4 },
    { 0xd422efc1, 4 },
    { 0, 0 },
    { 0x24a2efc5, 3 },
    { 0xeb96efc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8eefc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2aaefcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc316efd1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f6efd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9816efd9, 4 },
    { 0x2a1aefda, 3 },
    { 0xd64aefdb, 3 },
    { 0x4cf2efdc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82efe3, 3 },
    { 0xbd4aefe4, 4 },
    { 0xfa4aefe5, 3 },
    { 0, 0 },
    { 0x18aeefe7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe722efed, 3 },
    { 0x580eefee, 3 },
    { 0xfaaeefef, 3 },
    { 0x52aefef, 3 },
    { 0, 0 },
    { 0x5026eff2, 3 },
    { 0xb326eff3, 3 },
    { 0, 0 },
    { 0x2e76eff5, 3 },
    { 0xa62eff5, 3 },
    { 0x105eeff6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f6effa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a6f001, 3 },
    { 0x4332f002, 4 },
    { 0x342f001, 3 },
    { 0x1c3ef004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abef007, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd2f00a, 3 },
    { 0xcc5ef00b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x258af00e, 4 },
    { 0xac7af00f, 3 },
    { 0xcceef00f, 3 },
    { 0x1deaf010, 3 },
    { 0xfc02f00f, 3 },
    { 0x5bb2f012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56ef018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0af01b, 3 },
    { 0, 0 },
    { 0xe0af01d, 3 },
    { 0, 0 },
    { 0x59a6f01f, 3 },
    { 0x697ef020, 3 },
    { 0xbd2af020, 3 },
    { 0xdb1af021, 3 },
    { 0xb2eaf023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb2f028, 3 },
    { 0x5836f029, 3 },
    { 0, 0 },
    { 0x5c1af02b, 3 },
    { 0x6acaf02b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364af032, 4 },
    { 0x6c4af033, 4 },
    { 0, 0 },
    { 0xd456f035, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x945af038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x760af03b, 3 },
    { 0x9c9ef03c, 3 },
    { 0xcb7af03d, 4 },
    { 0x57af03b, 4 },
    { 0xd472f03f, 3 },
    { 0x15daf040, 3 },
    { 0xdf0af040, 5 },
    { 0x295af041, 4 },
    { 0x47bef043, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4232f046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247ef04a, 3 },
    { 0xdec6f04a, 3 },
    { 0xde8ef04b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd46f056, 3 },
    { 0, 0 },
    { 0x6c56f058, 3 },
    { 0, 0 },
    { 0x2aef05a, 3 },
    { 0x6beef05b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe742f069, 4 },
    { 0xca8ef06a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b6f070, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ef07e, 3 },
    { 0xf59af07e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd726f083, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6f088, 3 },
    { 0x178ef089, 3 },
    { 0xb77ef08a, 3 },
    { 0x767ef08a, 3 },
    { 0x1e46f08b, 3 },
    { 0xe43ef08d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8ef094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586f097, 3 },
    { 0x9ef2f097, 3 },
    { 0xa156f098, 4 },
    { 0, 0 },
    { 0xc7a6f09b, 3 },
    { 0xe806f09b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236f0a9, 3 },
    { 0xf57ef0aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5ef0ae, 3 },
    { 0x299af0af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92f0b2, 3 },
    { 0x1752f0b3, 3 },
    { 0, 0 },
    { 0x12daf0b5, 4 },
    { 0, 0 },
    { 0xd7fef0b7, 3 },
    { 0x95f6f0b8, 3 },
    { 0, 0 },
    { 0x9ad6f0ba, 3 },
    { 0x6da6f0bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6816f0be, 3 },
    { 0, 0 },
    { 0xe1f6f0c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72af0cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616f0d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x499af0d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eef0d9, 3 },
    { 0x89bef0da, 4 },
    { 0x7916f0db, 4 },
    { 0x2ed2f0dc, 3 },
    { 0x5186f0dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe776f0e6, 3 },
    { 0xef86f0e7, 3 },
    { 0, 0 },
    { 0x9a72f0e9, 3 },
    { 0xca72f0ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3132f0ed, 3 },
    { 0, 0 },
    { 0x476ef0ef, 4 },
    { 0, 0 },
    { 0x4de2f0f1, 4 },
    { 0, 0 },
    { 0xcba6f0f3, 3 },
    { 0, 0 },
    { 0x24a2f0f5, 3 },
    { 0, 0 },
    { 0xde8af0f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6f0fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eaf0ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeef102, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2ef105, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4ef110, 3 },
    { 0xcd62f111, 3 },
    { 0x42c6f112, 4 },
    { 0xa416f113, 3 },
    { 0x7fbef113, 3 },
    { 0x8f46f115, 3 },
    { 0, 0 },
    { 0x2f82f117, 4 },
    { 0xecfef118, 3 },
    { 0xf34ef118, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2def11e, 3 },
    { 0xe16f11e, 4 },
    { 0xf71af11e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ef123, 3 },
    { 0x108af124, 3 },
    { 0x291af125, 3 },
    { 0x84faf125, 4 },
    { 0xb00ef126, 3 },
    { 0xa3aaf128, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x721ef12b, 3 },
    { 0x413af12c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336f131, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98baf138, 3 },
    { 0, 0 },
    { 0xe5af13a, 3 },
    { 0xde46f13b, 3 },
    { 0, 0 },
    { 0xaecaf13d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaaf140, 4 },
    { 0x92f2f141, 3 },
    { 0xc6e6f140, 3 },
    { 0x8746f143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41af148, 5 },
    { 0x5b32f149, 3 },
    { 0, 0 },
    { 0xd26f14b, 4 },
    { 0xfc5af14c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc2f152, 3 },
    { 0, 0 },
    { 0xbf6ef154, 3 },
    { 0x912f154, 3 },
    { 0x83faf155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ef15b, 4 },
    { 0xca92f15c, 4 },
    { 0xc812f15d, 3 },
    { 0x22c6f15e, 3 },
    { 0xdf5ef15f, 4 },
    { 0x1da6f15d, 4 },
    { 0xf192f15f, 4 },
    { 0, 0 },
    { 0x26daf163, 3 },
    { 0x3ccef163, 3 },
    { 0x1a26f165, 4 },
    { 0x7ea6f166, 3 },
    { 0xed0af163, 3 },
    { 0x8146f168, 4 },
    { 0, 0 },
    { 0x2c7af16a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x746ef16d, 3 },
    { 0, 0 },
    { 0x83def16f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4ef177, 4 },
    { 0x725af178, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e32f17b, 3 },
    { 0, 0 },
    { 0x2a3af17d, 3 },
    { 0, 0 },
    { 0x769ef17f, 4 },
    { 0x4396f180, 4 },
    { 0xd4f2f181, 3 },
    { 0xa886f182, 4 },
    { 0x3006f183, 4 },
    { 0xd592f184, 3 },
    { 0x8002f183, 3 },
    { 0x5772f186, 3 },
    { 0x76b6f187, 4 },
    { 0x358ef187, 4 },
    { 0x1a8ef189, 4 },
    { 0xd02ef18a, 3 },
    { 0x256f189, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06f190, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf2f193, 3 },
    { 0x279af194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x479ef197, 4 },
    { 0, 0 },
    { 0xc256f199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ef1a0, 3 },
    { 0x7f36f1a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ef1a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cef1aa, 3 },
    { 0x4bcef1ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00af1ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bef1b1, 3 },
    { 0, 0 },
    { 0x9e22f1b3, 4 },
    { 0x3626f1b3, 3 },
    { 0x9b5ef1b5, 3 },
    { 0x627af1b6, 3 },
    { 0x2372f1b7, 4 },
    { 0, 0 },
    { 0x49ef1b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2f1bc, 3 },
    { 0x834ef1bd, 4 },
    { 0xbabaf1be, 3 },
    { 0x482f1bf, 3 },
    { 0xe1def1c0, 3 },
    { 0, 0 },
    { 0x7652f1c2, 3 },
    { 0x3a52f1c2, 4 },
    { 0, 0 },
    { 0xc65ef1c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccef1c9, 3 },
    { 0x1f1ef1ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615af1d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e6f1dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d06f1e0, 4 },
    { 0, 0 },
    { 0x242ef1e2, 3 },
    { 0xffe2f1e2, 4 },
    { 0xddfef1e3, 3 },
    { 0x5ee2f1e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa106f1e8, 3 },
    { 0x2df2f1e9, 3 },
    { 0x4f06f1e9, 4 },
    { 0x48faf1eb, 3 },
    { 0x65f6f1eb, 3 },
    { 0xbe22f1eb, 3 },
    { 0xc406f1e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8ef1f8, 3 },
    { 0xbba6f1f9, 3 },
    { 0xfd12f1fa, 3 },
    { 0x4de6f1fa, 4 },
    { 0x695ef1fa, 3 },
    { 0xd0a2f1fd, 3 },
    { 0, 0 },
    { 0x8a42f1ff, 3 },
    { 0, 0 },
    { 0x8446f201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72f204, 3 },
    { 0xbd2f205, 3 },
    { 0xbeeaf206, 4 },
    { 0xf602f207, 3 },
    { 0, 0 },
    { 0x383af209, 3 },
    { 0x8ef20a, 4 },
    { 0xd052f209, 4 },
    { 0, 0 },
    { 0xa9baf20d, 4 },
    { 0x4852f20e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755ef213, 3 },
    { 0xafeaf213, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee6f219, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eaf220, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cef224, 3 },
    { 0x9112f225, 3 },
    { 0, 0 },
    { 0x3cdaf227, 3 },
    { 0x2f06f227, 3 },
    { 0x4172f229, 4 },
    { 0x3526f229, 3 },
    { 0, 0 },
    { 0xb73ef22c, 4 },
    { 0x136f22d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b6f232, 3 },
    { 0, 0 },
    { 0x7ad6f234, 3 },
    { 0xcbe2f235, 3 },
    { 0xbc3af236, 3 },
    { 0, 0 },
    { 0xcaeef238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc462f23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7ef243, 3 },
    { 0xa85af244, 4 },
    { 0, 0 },
    { 0x658af246, 3 },
    { 0, 0 },
    { 0xe782f248, 3 },
    { 0xa20ef248, 3 },
    { 0x41b6f24a, 3 },
    { 0x4f52f248, 3 },
    { 0xc03ef24c, 3 },
    { 0, 0 },
    { 0x49eef24e, 4 },
    { 0x8306f24e, 4 },
    { 0xd6aaf250, 3 },
    { 0, 0 },
    { 0xf15af252, 4 },
    { 0x63e2f253, 3 },
    { 0, 0 },
    { 0x475af255, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e96f262, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342ef266, 3 },
    { 0x2c36f266, 3 },
    { 0xa54af266, 3 },
    { 0, 0 },
    { 0xdd62f26a, 3 },
    { 0, 0 },
    { 0x9382f26c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef26f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbaf272, 3 },
    { 0xc4eef273, 3 },
    { 0xa8cef274, 3 },
    { 0x3ddaf273, 4 },
    { 0x96def273, 4 },
    { 0x3e1af277, 5 },
    { 0x622ef278, 3 },
    { 0x34fef279, 3 },
    { 0x5cc6f272, 3 },
    { 0x617ef27b, 4 },
    { 0x13fef27c, 3 },
    { 0x5dcaf27c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b46f281, 3 },
    { 0, 0 },
    { 0xa566f283, 3 },
    { 0xd1e2f283, 3 },
    { 0x6502f285, 5 },
    { 0, 0 },
    { 0xed0ef287, 3 },
    { 0x18baf288, 3 },
    { 0xc45af289, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0af290, 5 },
    { 0x981ef291, 3 },
    { 0, 0 },
    { 0x12def293, 3 },
    { 0x81caf293, 4 },
    { 0x81f6f295, 3 },
    { 0x8792f296, 3 },
    { 0x4bcef297, 5 },
    { 0x153ef297, 4 },
    { 0xfb86f297, 3 },
    { 0x989ef29a, 3 },
    { 0xe6e2f29b, 4 },
    { 0, 0 },
    { 0x43c6f29d, 3 },
    { 0x298af29e, 4 },
    { 0xcc2f29e, 3 },
    { 0xc346f29e, 4 },
    { 0xbe92f2a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6ef2a4, 3 },
    { 0x1ce6f2a5, 3 },
    { 0x1c7ef2a6, 4 },
    { 0x3b2af2a6, 4 },
    { 0x98af2a6, 4 },
    { 0x262af2a8, 3 },
    { 0x4626f2aa, 3 },
    { 0xe406f2ab, 3 },
    { 0xc1c2f2ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf02f2af, 3 },
    { 0xb6c6f2b0, 3 },
    { 0x3326f2af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9ef2b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3af2b8, 4 },
    { 0x987ef2b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77af2bd, 3 },
    { 0xe462f2bd, 3 },
    { 0, 0 },
    { 0xa00af2c0, 3 },
    { 0, 0 },
    { 0x6202f2c2, 3 },
    { 0x5672f2c2, 3 },
    { 0x66aef2c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ef2c8, 3 },
    { 0x5782f2c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x199ef2cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ef2cf, 4 },
    { 0x17eef2d0, 4 },
    { 0x6452f2d1, 3 },
    { 0xbfc2f2d2, 3 },
    { 0xb782f2d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57af2d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f06f2dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe72f2e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9192f2e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2ef2eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9066f2ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6f2f2, 3 },
    { 0x1e0af2f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369ef2f8, 3 },
    { 0x228ef2f8, 3 },
    { 0xa6c2f2f8, 5 },
    { 0xbbe2f2fb, 3 },
    { 0xa4e6f2fc, 3 },
    { 0xd0b2f2fc, 3 },
    { 0x7576f2fd, 3 },
    { 0xf85ef2fd, 4 },
    { 0x80aaf300, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d82f303, 3 },
    { 0x63fef304, 3 },
    { 0xf206f304, 3 },
    { 0, 0 },
    { 0x82a6f307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7662f30b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf242f30e, 3 },
    { 0xa2d6f30e, 3 },
    { 0xdf2af30e, 3 },
    { 0xca46f311, 4 },
    { 0xb5f6f312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aef317, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7ef31b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6f31f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x443af322, 3 },
    { 0x2d0af323, 3 },
    { 0, 0 },
    { 0xfdef325, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74caf32b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26baf330, 4 },
    { 0xc7f2f331, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7316f334, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797af33b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7af33f, 3 },
    { 0, 0 },
    { 0xad3ef341, 3 },
    { 0x2d2ef342, 3 },
    { 0x875af343, 3 },
    { 0x66d6f342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf142f349, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21ef34e, 3 },
    { 0x1ff6f34e, 3 },
    { 0xacaef34e, 3 },
    { 0xe5faf351, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de6f356, 5 },
    { 0xf522f357, 3 },
    { 0xea2f356, 3 },
    { 0, 0 },
    { 0x6cf2f35a, 3 },
    { 0xcc6ef35b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaf35e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe186f362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5ef367, 4 },
    { 0xe50ef367, 3 },
    { 0x941ef369, 4 },
    { 0, 0 },
    { 0x958af36b, 3 },
    { 0x8d66f36c, 3 },
    { 0, 0 },
    { 0x8e22f36e, 3 },
    { 0xd1b2f36f, 4 },
    { 0x18e6f36e, 3 },
    { 0, 0 },
    { 0x25aef372, 4 },
    { 0, 0 },
    { 0x4a06f374, 3 },
    { 0, 0 },
    { 0xfa76f376, 4 },
    { 0x974af377, 4 },
    { 0x577ef376, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6f37b, 3 },
    { 0, 0 },
    { 0x296f37d, 4 },
    { 0x6772f37e, 3 },
    { 0xe0aef37f, 3 },
    { 0xf83ef37d, 4 },
    { 0xdf6ef37f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x872af384, 4 },
    { 0xd08af385, 4 },
    { 0x703ef386, 3 },
    { 0x5aaef387, 4 },
    { 0, 0 },
    { 0x7c92f389, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909af38d, 3 },
    { 0x9ee2f38e, 3 },
    { 0, 0 },
    { 0xc436f390, 3 },
    { 0x418ef391, 3 },
    { 0xbf3af391, 4 },
    { 0, 0 },
    { 0x2bf2f394, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af2f39a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56af39d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3af3a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12f3ac, 3 },
    { 0xc262f3ad, 3 },
    { 0x89c2f3ac, 3 },
    { 0xfb8ef3ac, 3 },
    { 0, 0 },
    { 0x9a26f3b1, 5 },
    { 0xd65ef3b2, 3 },
    { 0x2a7ef3b2, 3 },
    { 0, 0 },
    { 0x4abaf3b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3842f3b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf6f3bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1472f3c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706f3ca, 4 },
    { 0xa4caf3ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36ef3ce, 3 },
    { 0, 0 },
    { 0xf0e6f3d0, 3 },
    { 0, 0 },
    { 0x845af3d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8902f3d7, 3 },
    { 0, 0 },
    { 0xffbaf3d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f06f3dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ef3e0, 4 },
    { 0, 0 },
    { 0x8e76f3e2, 3 },
    { 0xc25af3e3, 3 },
    { 0, 0 },
    { 0xcdbaf3e5, 4 },
    { 0x1bc6f3e6, 3 },
    { 0x95b2f3e5, 3 },
    { 0xa822f3e7, 4 },
    { 0x92faf3e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daf3ee, 3 },
    { 0xf772f3ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65ef3f3, 3 },
    { 0xecfef3f4, 3 },
    { 0x8a2af3f5, 3 },
    { 0xc1ef3f6, 4 },
    { 0xf7aaf3f7, 3 },
    { 0x2b72f3f8, 3 },
    { 0, 0 },
    { 0x59e6f3fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23af402, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f82f408, 3 },
    { 0x82fef408, 3 },
    { 0x9f06f40a, 3 },
    { 0, 0 },
    { 0xc09af40c, 3 },
    { 0, 0 },
    { 0xa6daf40e, 3 },
    { 0x6b42f40f, 3 },
    { 0x5086f410, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8066f416, 3 },
    { 0, 0 },
    { 0x851af418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a92f41d, 3 },
    { 0x4eef41e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473af424, 3 },
    { 0, 0 },
    { 0xcb1af426, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33faf42a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6f42d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eef430, 3 },
    { 0x42aaf431, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96f436, 4 },
    { 0xb97ef437, 3 },
    { 0xb06af436, 3 },
    { 0x294ef436, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2676f43d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7def44e, 4 },
    { 0x4616f44e, 5 },
    { 0x5a0ef450, 3 },
    { 0x570ef44f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e26f457, 3 },
    { 0x7ccaf458, 3 },
    { 0x2312f458, 3 },
    { 0x229ef45a, 3 },
    { 0x2992f45a, 3 },
    { 0xd53af45a, 4 },
    { 0xcb7ef45a, 4 },
    { 0, 0 },
    { 0x9112f45f, 3 },
    { 0x6892f45f, 3 },
    { 0x12fef460, 3 },
    { 0x2096f462, 3 },
    { 0xe6bef463, 4 },
    { 0, 0 },
    { 0x70fef465, 3 },
    { 0, 0 },
    { 0xfa5ef467, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73ef46a, 4 },
    { 0x8732f46b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af46e, 4 },
    { 0xe732f46e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd2f472, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9def478, 3 },
    { 0xd51ef478, 4 },
    { 0xe1af479, 4 },
    { 0, 0 },
    { 0xfa9af47c, 3 },
    { 0xc132f47d, 3 },
    { 0x642ef47e, 3 },
    { 0xf7a2f47e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8af482, 5 },
    { 0xbc6af483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731ef489, 3 },
    { 0x5c8af48a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd316f48d, 3 },
    { 0x6d4af48e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x155af491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76f498, 3 },
    { 0, 0 },
    { 0x47c6f49a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baaf49e, 4 },
    { 0x2ceaf49f, 3 },
    { 0x648ef4a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142f4a5, 3 },
    { 0x59eef4a5, 3 },
    { 0x6d2f4a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21af4ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762f4b1, 3 },
    { 0x2616f4b2, 5 },
    { 0, 0 },
    { 0xd46f4b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273af4bc, 3 },
    { 0, 0 },
    { 0x3b6f4be, 3 },
    { 0x5b56f4bf, 4 },
    { 0x8a52f4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ef4c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b22f4cd, 3 },
    { 0x861ef4cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64af4d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12f4dc, 3 },
    { 0, 0 },
    { 0xa39ef4de, 3 },
    { 0x47ef4de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2102f4e9, 4 },
    { 0, 0 },
    { 0x94baf4eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64eef4ee, 3 },
    { 0xb9b2f4ef, 4 },
    { 0, 0 },
    { 0xc13af4f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5216f4f6, 4 },
    { 0, 0 },
    { 0x2536f4f8, 3 },
    { 0, 0 },
    { 0x6916f4fa, 3 },
    { 0, 0 },
    { 0x61cef4fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7192f500, 3 },
    { 0xad52f501, 5 },
    { 0, 0 },
    { 0x843ef503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f6f506, 4 },
    { 0x1286f507, 3 },
    { 0x80a6f507, 5 },
    { 0xda5ef509, 3 },
    { 0x85b2f506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdef50e, 4 },
    { 0, 0 },
    { 0x3ddef510, 3 },
    { 0, 0 },
    { 0xfc56f512, 3 },
    { 0x484ef513, 3 },
    { 0x3b06f514, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x902af517, 3 },
    { 0xa346f517, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628af51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16af521, 4 },
    { 0xc916f522, 3 },
    { 0x9c2f523, 3 },
    { 0, 0 },
    { 0x8a6af525, 3 },
    { 0x7feaf526, 5 },
    { 0x1d22f527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776af538, 4 },
    { 0xf15ef538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0af540, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4192f545, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0ef54a, 4 },
    { 0x1722f54a, 4 },
    { 0x482ef54c, 5 },
    { 0, 0 },
    { 0x3446f54e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x836af551, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cef555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd036f55c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2f561, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2f56e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c26f572, 3 },
    { 0xcf2af573, 3 },
    { 0xc4e2f574, 3 },
    { 0x9e82f573, 3 },
    { 0, 0 },
    { 0x914af577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2f582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab72f58e, 4 },
    { 0xec16f58f, 3 },
    { 0x7782f590, 3 },
    { 0xc44af58e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820ef595, 3 },
    { 0, 0 },
    { 0x3216f597, 3 },
    { 0x7b2f597, 3 },
    { 0xf5b2f599, 3 },
    { 0, 0 },
    { 0x6336f59b, 3 },
    { 0xbe36f59b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ef5a2, 3 },
    { 0x8e4ef5a2, 3 },
    { 0x7cbaf5a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1af5a7, 4 },
    { 0xf3ef5a7, 3 },
    { 0xe42af5a9, 3 },
    { 0x42b6f5a8, 4 },
    { 0x7d8ef5ab, 3 },
    { 0xd2def5ac, 3 },
    { 0xc50af5ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1af5b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78baf5b3, 3 },
    { 0, 0 },
    { 0xb41ef5b5, 3 },
    { 0xf9ef5b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf56f5b9, 3 },
    { 0xdccef5b9, 4 },
    { 0xc422f5ba, 3 },
    { 0x79eef5bb, 3 },
    { 0, 0 },
    { 0x156af5be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc206f5c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ef5cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x86caf5cf, 4 },
    { 0, 0 },
    { 0x3e4af5d1, 4 },
    { 0, 0 },
    { 0x3a16f5d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16ef5db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e6f5de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aef5e1, 3 },
    { 0x3d62f5e2, 3 },
    { 0xe1f2f5e1, 3 },
    { 0x7eef5e4, 4 },
    { 0xb8ef5e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fef5eb, 3 },
    { 0xc40af5ec, 3 },
    { 0x2e2af5ec, 3 },
    { 0x7d92f5ed, 4 },
    { 0x6b26f5ed, 3 },
    { 0x5502f5ef, 4 },
    { 0x5bbef5ec, 3 },
    { 0x681ef5f2, 3 },
    { 0x991ef5f2, 3 },
    { 0, 0 },
    { 0x8076f5f5, 4 },
    { 0x766af5f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78af5fa, 3 },
    { 0xe39ef5fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x906af5fe, 3 },
    { 0x4f6af5ff, 3 },
    { 0, 0 },
    { 0xabdaf601, 3 },
    { 0xfa92f602, 4 },
    { 0x52faf602, 3 },
    { 0xf926f604, 4 },
    { 0, 0 },
    { 0x5806f606, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x441ef609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f60d, 4 },
    { 0, 0 },
    { 0x9fbef60f, 3 },
    { 0x58caf610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156f616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5752f619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x474af61c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad6f61f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9462f623, 4 },
    { 0x2ffef623, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812f627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f86f62d, 4 },
    { 0x437af62e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855af634, 3 },
    { 0xa006f635, 3 },
    { 0xa03af635, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc6f63c, 3 },
    { 0, 0 },
    { 0xf566f63e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b6f646, 3 },
    { 0xeb2f647, 3 },
    { 0xab32f647, 3 },
    { 0xee6af649, 4 },
    { 0, 0 },
    { 0xbb16f64b, 3 },
    { 0x5ceaf64b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8392f653, 3 },
    { 0, 0 },
    { 0x181ef655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e6f65a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3ef65e, 3 },
    { 0xc71af65f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc722f662, 3 },
    { 0, 0 },
    { 0x133af664, 3 },
    { 0x216ef665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92f668, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe2f66b, 3 },
    { 0x1efaf66c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778af673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590af677, 3 },
    { 0x5fb6f677, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aaf67f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7ef683, 3 },
    { 0xcbeef683, 3 },
    { 0x346af685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc822f689, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe212f68c, 3 },
    { 0, 0 },
    { 0x769af68e, 5 },
    { 0xe536f68e, 3 },
    { 0xd5f2f690, 3 },
    { 0xc782f691, 3 },
    { 0xa86af692, 3 },
    { 0x54aef693, 3 },
    { 0xafef693, 3 },
    { 0xd68af694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc82f69a, 3 },
    { 0xbb5af69a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42f6a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2262f6a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x626af6a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6f6ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bef6ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158af6b2, 4 },
    { 0xf93ef6b2, 3 },
    { 0x5416f6b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba12f6b7, 3 },
    { 0xe6b6f6b8, 3 },
    { 0, 0 },
    { 0xc6aaf6ba, 3 },
    { 0, 0 },
    { 0x8a32f6bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e2f6bf, 3 },
    { 0x7a2ef6bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aef6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddaf6cb, 4 },
    { 0x3f7ef6cb, 4 },
    { 0x1ed2f6cd, 3 },
    { 0x2086f6ce, 4 },
    { 0x1f56f6cc, 3 },
    { 0x62def6d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda42f6d5, 3 },
    { 0, 0 },
    { 0x438ef6d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3daf6db, 3 },
    { 0xa97ef6dc, 3 },
    { 0, 0 },
    { 0x1c6ef6de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cef6e2, 4 },
    { 0xfa2f6e3, 3 },
    { 0xfd7ef6e4, 4 },
    { 0, 0 },
    { 0x8eeaf6e6, 3 },
    { 0x9136f6e7, 3 },
    { 0, 0 },
    { 0x6c6f6e9, 3 },
    { 0x9a4ef6ea, 3 },
    { 0x6f46f6ea, 3 },
    { 0x7f92f6ea, 3 },
    { 0x1af2f6ed, 3 },
    { 0x2b52f6ee, 3 },
    { 0xabaf6ed, 3 },
    { 0x6a8af6ee, 3 },
    { 0xed2ef6f1, 3 },
    { 0xcf2f6ee, 3 },
    { 0x628ef6ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6836f6f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd052f6fb, 3 },
    { 0x4012f6fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fef6ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2ef705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48ef709, 3 },
    { 0x51faf70a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faef710, 3 },
    { 0x1f12f711, 3 },
    { 0xd1af712, 3 },
    { 0x3c86f713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbef71a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eef723, 3 },
    { 0xb15af724, 3 },
    { 0x5faaf724, 3 },
    { 0xc222f724, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2f72d, 4 },
    { 0, 0 },
    { 0xc31ef72f, 4 },
    { 0, 0 },
    { 0xa84ef731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68ef734, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccaf738, 4 },
    { 0x5802f738, 3 },
    { 0, 0 },
    { 0x1162f73b, 3 },
    { 0x1a72f73c, 4 },
    { 0, 0 },
    { 0x35b2f73e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4ef744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1af74e, 3 },
    { 0xf476f74f, 4 },
    { 0x8366f74f, 3 },
    { 0x24ef751, 3 },
    { 0, 0 },
    { 0x54b6f753, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512f756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c2f75a, 3 },
    { 0, 0 },
    { 0x2832f75c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ef765, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ef769, 3 },
    { 0x32baf76a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e2f772, 4 },
    { 0x92cef773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2f777, 3 },
    { 0x7fb2f777, 3 },
    { 0x6916f778, 3 },
    { 0xb32af779, 3 },
    { 0x6c0af77b, 5 },
    { 0, 0 },
    { 0x38a6f77d, 3 },
    { 0xdc62f77d, 3 },
    { 0xaf76f77f, 3 },
    { 0, 0 },
    { 0x959af781, 3 },
    { 0, 0 },
    { 0x97f6f783, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329af787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x915af78a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc6f790, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2526f793, 4 },
    { 0, 0 },
    { 0x22fef795, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e2f7a0, 4 },
    { 0, 0 },
    { 0x513ef7a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901af7ab, 3 },
    { 0, 0 },
    { 0x650af7ad, 3 },
    { 0xebd2f7ae, 4 },
    { 0x4f0ef7af, 3 },
    { 0xfd1af7b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7daf7b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691af7b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b6f7bb, 3 },
    { 0x1c5ef7bb, 4 },
    { 0x4b0ef7bd, 3 },
    { 0x5f9af7be, 3 },
    { 0x89c6f7bf, 4 },
    { 0xf4c6f7bf, 3 },
    { 0x32baf7be, 4 },
    { 0x82f2f7c2, 4 },
    { 0x3db6f7c2, 3 },
    { 0xd8ef7bc, 5 },
    { 0x7dbaf7c5, 4 },
    { 0x983ef7c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cef7c9, 3 },
    { 0, 0 },
    { 0x560af7cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ef7d1, 3 },
    { 0, 0 },
    { 0x7972f7d3, 4 },
    { 0, 0 },
    { 0x56eaf7d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a42f7d8, 3 },
    { 0xd402f7d9, 3 },
    { 0x84caf7d8, 3 },
    { 0x97a6f7d8, 3 },
    { 0xf73af7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefef7e0, 3 },
    { 0, 0 },
    { 0x4202f7e2, 3 },
    { 0xbc3af7e3, 4 },
    { 0, 0 },
    { 0xbf1af7e5, 3 },
    { 0x7076f7e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b6f7ea, 3 },
    { 0xfd0af7eb, 3 },
    { 0, 0 },
    { 0xb36af7ed, 4 },
    { 0x423ef7ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf7f4, 3 },
    { 0x5fdef7f5, 3 },
    { 0x2daef7f5, 4 },
    { 0xdc72f7f7, 3 },
    { 0xe9faf7f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e6f805, 4 },
    { 0x357ef806, 3 },
    { 0, 0 },
    { 0xef06f808, 3 },
    { 0xceaef808, 3 },
    { 0xf73ef808, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04af80d, 4 },
    { 0xf07ef80e, 3 },
    { 0, 0 },
    { 0xa0d2f810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1baf815, 3 },
    { 0, 0 },
    { 0x6596f817, 3 },
    { 0x4e2af817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2f81e, 3 },
    { 0xeaf6f81e, 3 },
    { 0, 0 },
    { 0xc72af821, 3 },
    { 0xd9eef822, 4 },
    { 0, 0 },
    { 0xd08af824, 3 },
    { 0xce96f825, 4 },
    { 0x4f16f824, 3 },
    { 0xe5aef824, 3 },
    { 0x45aef828, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c6f82b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642af830, 3 },
    { 0, 0 },
    { 0xc342f832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a2f83a, 3 },
    { 0xa7cef83b, 3 },
    { 0x79af83b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbef841, 3 },
    { 0xd3aef841, 3 },
    { 0x4afef843, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8ef849, 3 },
    { 0x96cef84a, 3 },
    { 0x9def84b, 3 },
    { 0xe91af849, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4116f854, 5 },
    { 0, 0 },
    { 0xa2faf856, 5 },
    { 0xcf96f857, 3 },
    { 0xefef858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd6f85b, 3 },
    { 0xad72f85b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x367af85f, 3 },
    { 0xf62ef85f, 4 },
    { 0x5192f861, 3 },
    { 0x9f2ef860, 3 },
    { 0x6572f85f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88daf866, 4 },
    { 0x5bcaf867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4af86a, 3 },
    { 0x98fef86b, 3 },
    { 0xfe86f86b, 3 },
    { 0x5482f86d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903af873, 3 },
    { 0xc032f873, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ef877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6f87a, 5 },
    { 0, 0 },
    { 0xb396f87c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb2f87f, 3 },
    { 0x3526f87f, 3 },
    { 0x467ef881, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ef886, 3 },
    { 0, 0 },
    { 0x6d5ef888, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285af88c, 3 },
    { 0, 0 },
    { 0x2402f88e, 4 },
    { 0, 0 },
    { 0xfecef890, 3 },
    { 0x6546f891, 3 },
    { 0, 0 },
    { 0x39c6f893, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1ef897, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf462f89d, 3 },
    { 0, 0 },
    { 0x4f3ef89f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x870af8a2, 4 },
    { 0x159ef8a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb282f8a7, 3 },
    { 0x35e2f8a8, 3 },
    { 0xd33ef8a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f66f8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf796f8b2, 3 },
    { 0x808af8b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16f8b9, 4 },
    { 0, 0 },
    { 0xa01ef8bb, 3 },
    { 0x758af8bb, 3 },
    { 0x554ef8bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c2f8c1, 3 },
    { 0xfd7ef8c2, 3 },
    { 0x4f52f8c3, 4 },
    { 0x6dcef8c4, 3 },
    { 0, 0 },
    { 0xaa6ef8c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b2f8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6af8d0, 4 },
    { 0, 0 },
    { 0x8feaf8d2, 4 },
    { 0xfa6f8d2, 3 },
    { 0xd8fef8d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956f8d8, 3 },
    { 0x33f2f8d9, 4 },
    { 0xb43af8d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885ef8de, 4 },
    { 0, 0 },
    { 0xedcef8e0, 3 },
    { 0, 0 },
    { 0xf5a6f8e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb6f8eb, 4 },
    { 0, 0 },
    { 0x1876f8ed, 4 },
    { 0xc8eaf8ed, 3 },
    { 0, 0 },
    { 0xbc12f8f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef8f3, 3 },
    { 0x9fdaf8f4, 4 },
    { 0x1caf8f3, 4 },
    { 0x731af8f3, 3 },
    { 0, 0 },
    { 0x5d82f8f8, 3 },
    { 0xcee6f8f9, 3 },
    { 0, 0 },
    { 0xc50af8fb, 3 },
    { 0, 0 },
    { 0x34b6f8fd, 3 },
    { 0xb84ef8fd, 3 },
    { 0x641ef8ff, 3 },
    { 0x1dc6f900, 3 },
    { 0x34b2f8ff, 3 },
    { 0, 0 },
    { 0x202ef903, 3 },
    { 0xd87ef904, 3 },
    { 0, 0 },
    { 0x6ea2f906, 3 },
    { 0x3f86f907, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5daf90a, 3 },
    { 0x40b6f90b, 3 },
    { 0x1b02f90c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c06f90f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbaf913, 4 },
    { 0x8346f914, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8caf91b, 3 },
    { 0, 0 },
    { 0x835ef91d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb6f923, 3 },
    { 0x962af923, 3 },
    { 0, 0 },
    { 0xc672f926, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2f92b, 3 },
    { 0x98a2f92c, 3 },
    { 0, 0 },
    { 0x98baf92e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fef937, 3 },
    { 0x5d2f938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfaf93c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2932f943, 3 },
    { 0x48bef944, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416f949, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fef94c, 3 },
    { 0, 0 },
    { 0x963af94e, 4 },
    { 0xaa06f94e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd866f952, 3 },
    { 0, 0 },
    { 0x5326f954, 3 },
    { 0x5c96f955, 3 },
    { 0x79af956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8af959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eef95c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a6f961, 4 },
    { 0xcecef961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82af966, 3 },
    { 0x64a2f967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbaf96b, 3 },
    { 0, 0 },
    { 0x312ef96d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1842f971, 3 },
    { 0x37f6f972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d6f977, 3 },
    { 0x7176f977, 4 },
    { 0x5b32f979, 4 },
    { 0x5de6f979, 5 },
    { 0x5fe2f97b, 4 },
    { 0, 0 },
    { 0xa156f97d, 3 },
    { 0x6702f97e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4ef984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6ef98a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66def98f, 3 },
    { 0, 0 },
    { 0xfb6af991, 3 },
    { 0x111af992, 3 },
    { 0x5362f992, 4 },
    { 0xe4e6f992, 4 },
    { 0xf402f995, 5 },
    { 0x8112f995, 3 },
    { 0x5f9ef997, 4 },
    { 0xde76f998, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d6f99f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a2f9a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda46f9a8, 3 },
    { 0, 0 },
    { 0x1b3ef9aa, 3 },
    { 0, 0 },
    { 0x4c2f9ac, 3 },
    { 0x3356f9ac, 4 },
    { 0, 0 },
    { 0x4c06f9af, 4 },
    { 0xf4a2f9af, 3 },
    { 0x4ddef9b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1032f9bf, 3 },
    { 0, 0 },
    { 0x19af9c1, 3 },
    { 0x875ef9c1, 3 },
    { 0xafc2f9c3, 3 },
    { 0, 0 },
    { 0x2b16f9c5, 3 },
    { 0x88faf9c6, 3 },
    { 0, 0 },
    { 0x1522f9c8, 3 },
    { 0x2e06f9c9, 3 },
    { 0xb13ef9c8, 3 },
    { 0xffdaf9c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb2f9ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6f9d5, 3 },
    { 0xdaa2f9d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa32f9dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cef9e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99def9eb, 4 },
    { 0x941ef9eb, 3 },
    { 0xc966f9ec, 4 },
    { 0x3af6f9ed, 4 },
    { 0xed8ef9ef, 3 },
    { 0x2a02f9eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec6f9f3, 3 },
    { 0, 0 },
    { 0xa92ef9f5, 3 },
    { 0, 0 },
    { 0x6a3af9f7, 3 },
    { 0x5b9af9f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9baf9fb, 3 },
    { 0xad0ef9fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb6fa02, 4 },
    { 0, 0 },
    { 0xf71efa04, 3 },
    { 0x9742fa05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2932fa08, 3 },
    { 0xaecefa09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cefa0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd2fa10, 3 },
    { 0, 0 },
    { 0x6962fa12, 3 },
    { 0xc52efa13, 3 },
    { 0, 0 },
    { 0x5a4efa15, 3 },
    { 0, 0 },
    { 0x1e86fa17, 4 },
    { 0xb596fa18, 4 },
    { 0xca72fa19, 3 },
    { 0xa9afa18, 3 },
    { 0, 0 },
    { 0xe766fa1c, 4 },
    { 0x86c6fa1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264efa22, 4 },
    { 0, 0 },
    { 0x1d92fa24, 3 },
    { 0xf0befa25, 3 },
    { 0x1216fa25, 3 },
    { 0xb5efa27, 3 },
    { 0xedd6fa28, 4 },
    { 0xa2dafa29, 3 },
    { 0x291efa26, 3 },
    { 0xd89efa25, 3 },
    { 0x20cafa26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x155afa2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c02fa33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe62fa39, 3 },
    { 0x71bafa3a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a6fa3f, 4 },
    { 0xd8a6fa40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507efa44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86fa47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dafa4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf302fa58, 3 },
    { 0xe752fa59, 3 },
    { 0x8c4afa59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea6fa60, 3 },
    { 0xee4afa61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f26fa65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb602fa6a, 3 },
    { 0xc136fa6b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dafa6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc262fa78, 3 },
    { 0xa00afa79, 3 },
    { 0x6f32fa78, 3 },
    { 0x7bbefa7b, 3 },
    { 0, 0 },
    { 0xa676fa7d, 3 },
    { 0, 0 },
    { 0xc15efa7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3efa87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcefa8a, 4 },
    { 0x7a5afa8a, 3 },
    { 0x6b82fa8b, 3 },
    { 0x3466fa8d, 3 },
    { 0xa86efa8c, 3 },
    { 0x7afa8f, 3 },
    { 0, 0 },
    { 0xb3efa91, 3 },
    { 0x2276fa92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeafa96, 3 },
    { 0xd22fa96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310afa9b, 3 },
    { 0x1ffefa9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d2fa9f, 4 },
    { 0, 0 },
    { 0x7472faa1, 4 },
    { 0, 0 },
    { 0x5c52faa3, 3 },
    { 0, 0 },
    { 0x4726faa5, 3 },
    { 0x8ad6faa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefab0, 3 },
    { 0xf116fab1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2fab6, 3 },
    { 0x8ddafab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc796fabd, 4 },
    { 0x483afabe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaefac1, 3 },
    { 0xab9afac2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546fac5, 4 },
    { 0x1e0afac6, 3 },
    { 0x353afac7, 4 },
    { 0x540afac8, 3 },
    { 0x423efac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f72facc, 3 },
    { 0x7c92facc, 3 },
    { 0, 0 },
    { 0x9e3afacf, 3 },
    { 0, 0 },
    { 0x81defad1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6fad4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1886fadb, 4 },
    { 0x77c6fadc, 3 },
    { 0xf8aafadd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2faafae6, 4 },
    { 0x3d3afae7, 3 },
    { 0x1d7efae8, 4 },
    { 0, 0 },
    { 0xc6f6faea, 4 },
    { 0xeadafaeb, 3 },
    { 0x7f8efaec, 3 },
    { 0x11cafaed, 3 },
    { 0, 0 },
    { 0xc98afaef, 3 },
    { 0x3802faf0, 3 },
    { 0x6ea6faf1, 4 },
    { 0, 0 },
    { 0x5466faf3, 4 },
    { 0xcea6faf4, 3 },
    { 0xf34afaf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e6faf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1066fb00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64efb06, 3 },
    { 0x5166fb07, 3 },
    { 0xe62afb07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f2fb0d, 3 },
    { 0, 0 },
    { 0x94d6fb0f, 4 },
    { 0, 0 },
    { 0x3dcafb11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2fb14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8afb19, 4 },
    { 0xc502fb1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e86fb1e, 3 },
    { 0xadfefb1f, 3 },
    { 0x2f3afb1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3996fb25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e16fb2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf96fb33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8afb36, 3 },
    { 0x16dafb37, 3 },
    { 0xf742fb37, 3 },
    { 0x9bbafb36, 3 },
    { 0x7e7efb3a, 4 },
    { 0xdcaafb3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baafb3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9362fb46, 3 },
    { 0x73dafb47, 3 },
    { 0x6f9efb48, 4 },
    { 0x624afb48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2afb52, 3 },
    { 0x2f76fb52, 3 },
    { 0x466fb54, 3 },
    { 0xf64afb55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a2fb5c, 3 },
    { 0, 0 },
    { 0xdeaafb5e, 3 },
    { 0xd1d2fb5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5182fb63, 4 },
    { 0, 0 },
    { 0x637efb65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe36fb68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a46fb6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b6fb6f, 3 },
    { 0, 0 },
    { 0x8652fb71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6fb76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6fb79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f66fb7c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afafb81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaefb87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2fb8a, 3 },
    { 0xf5c2fb8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479afb8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f2fb9a, 3 },
    { 0xac46fb9b, 4 },
    { 0x8cf6fb9c, 3 },
    { 0xcd6afb9c, 3 },
    { 0x5c3efb9d, 3 },
    { 0x9df2fb9f, 3 },
    { 0xf2a6fb9d, 3 },
    { 0x50b6fba1, 3 },
    { 0x3392fba2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae2fba5, 3 },
    { 0x7926fba6, 3 },
    { 0x7c86fba6, 3 },
    { 0x6fba5, 3 },
    { 0x76b2fba7, 3 },
    { 0, 0 },
    { 0x425efbab, 3 },
    { 0xed5afbac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6fbb5, 4 },
    { 0xab9afbb6, 3 },
    { 0x4aeafbb7, 3 },
    { 0x4b8afbb8, 3 },
    { 0, 0 },
    { 0xf516fbba, 4 },
    { 0xdfc6fbbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440afbc1, 3 },
    { 0x83fafbc1, 3 },
    { 0, 0 },
    { 0xcc2efbc4, 3 },
    { 0x6456fbc5, 3 },
    { 0xa186fbc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf62fbc9, 3 },
    { 0x821afbca, 3 },
    { 0, 0 },
    { 0xab42fbcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd2fbd6, 4 },
    { 0, 0 },
    { 0xfa32fbd8, 3 },
    { 0, 0 },
    { 0x40c2fbda, 4 },
    { 0xa4afbda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419efbdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132efbe4, 4 },
    { 0, 0 },
    { 0x9f46fbe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f6fbe9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe216fbed, 4 },
    { 0xbb32fbee, 3 },
    { 0x5926fbef, 3 },
    { 0, 0 },
    { 0x6b2efbf1, 3 },
    { 0x9b9afbf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5202fbf9, 3 },
    { 0, 0 },
    { 0xdb26fbfb, 3 },
    { 0, 0 },
    { 0x9346fbfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747efc02, 5 },
    { 0xc716fc03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b36fc0b, 4 },
    { 0, 0 },
    { 0x9396fc0d, 3 },
    { 0x79a2fc0d, 3 },
    { 0xcc4efc0f, 4 },
    { 0, 0 },
    { 0xe8fafc11, 4 },
    { 0xde1afc11, 3 },
    { 0, 0 },
    { 0xdfc6fc14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324afc1e, 3 },
    { 0, 0 },
    { 0xe5a6fc20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe996fc26, 3 },
    { 0, 0 },
    { 0xba7efc28, 3 },
    { 0, 0 },
    { 0xbfefc2a, 3 },
    { 0, 0 },
    { 0x1a56fc2c, 4 },
    { 0xbdfefc2c, 4 },
    { 0xd852fc2e, 3 },
    { 0x563efc2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf2fc34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0afc39, 4 },
    { 0x786afc3a, 3 },
    { 0x43e2fc39, 3 },
    { 0xf60afc3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c2fc3f, 4 },
    { 0xd852fc40, 4 },
    { 0x5de2fc41, 3 },
    { 0x58a2fc41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dafc46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5afc4d, 3 },
    { 0x19f6fc4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66fc57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc32fc5c, 3 },
    { 0x178efc5d, 4 },
    { 0xc76fc5d, 4 },
    { 0xa796fc5d, 4 },
    { 0xac12fc5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956fc63, 3 },
    { 0, 0 },
    { 0xeaa6fc65, 3 },
    { 0, 0 },
    { 0xdecafc67, 3 },
    { 0, 0 },
    { 0xb542fc69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd522fc6e, 3 },
    { 0x561efc6e, 3 },
    { 0xdde6fc6f, 3 },
    { 0, 0 },
    { 0x63fafc72, 4 },
    { 0x93dafc73, 3 },
    { 0x2e52fc74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0afc78, 3 },
    { 0xc48afc78, 3 },
    { 0x8bcafc7a, 3 },
    { 0x8aa6fc7b, 3 },
    { 0xc4a6fc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00efc84, 3 },
    { 0x366afc85, 3 },
    { 0xbe4efc86, 4 },
    { 0x4b52fc87, 4 },
    { 0x20eefc88, 3 },
    { 0x37a6fc86, 3 },
    { 0x548efc8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56fc8d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0befc90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aefc97, 3 },
    { 0xb52fc98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745afc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5afc9f, 4 },
    { 0xc1aafc9f, 3 },
    { 0x9fe2fc9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf642fca6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a6fcae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28efcb2, 3 },
    { 0, 0 },
    { 0x854efcb4, 4 },
    { 0x5e36fcb5, 3 },
    { 0x2f6afcb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549afcba, 3 },
    { 0x48aafcbb, 3 },
    { 0xe612fcbc, 4 },
    { 0x6e9efcbb, 3 },
    { 0xd89efcbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f32fcc4, 3 },
    { 0, 0 },
    { 0xe0d6fcc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442fcc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501afcd2, 3 },
    { 0, 0 },
    { 0x6952fcd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4efcd8, 3 },
    { 0x484efcd8, 3 },
    { 0xcfe2fcda, 3 },
    { 0xcb8afcdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99defcdf, 3 },
    { 0x4d42fce0, 3 },
    { 0, 0 },
    { 0x71aefce2, 3 },
    { 0xd326fce3, 3 },
    { 0x1752fce4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dafce7, 3 },
    { 0xde5afce7, 4 },
    { 0x7dcefce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bafcec, 3 },
    { 0xa40afcec, 4 },
    { 0xc726fcee, 3 },
    { 0, 0 },
    { 0x52eafcf0, 5 },
    { 0xd92afcf1, 3 },
    { 0x84d2fcf1, 3 },
    { 0x37f6fcf0, 3 },
    { 0x2a0afcf4, 3 },
    { 0xe81efcf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72fcfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1afcff, 3 },
    { 0, 0 },
    { 0x294efd01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63afd05, 3 },
    { 0, 0 },
    { 0x8d62fd07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b6fd0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977afd10, 3 },
    { 0, 0 },
    { 0x1b02fd12, 4 },
    { 0, 0 },
    { 0xfe42fd14, 4 },
    { 0, 0 },
    { 0xe55efd16, 3 },
    { 0, 0 },
    { 0xbadefd18, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f46fd1b, 3 },
    { 0x4beafd1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867afd21, 3 },
    { 0, 0 },
    { 0xe8cefd23, 4 },
    { 0, 0 },
    { 0xd2bafd25, 3 },
    { 0xe0eefd26, 3 },
    { 0x1636fd27, 4 },
    { 0x86defd26, 4 },
    { 0, 0 },
    { 0xa0e6fd2a, 3 },
    { 0x6f72fd2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x131afd2e, 4 },
    { 0, 0 },
    { 0xe5e2fd30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7afd37, 3 },
    { 0x8f32fd38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28efd3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a2fd44, 3 },
    { 0, 0 },
    { 0xe01afd46, 4 },
    { 0, 0 },
    { 0xbc32fd48, 3 },
    { 0, 0 },
    { 0x186fd4a, 3 },
    { 0, 0 },
    { 0x619afd4c, 3 },
    { 0, 0 },
    { 0x6902fd4e, 3 },
    { 0x3132fd4e, 4 },
    { 0x31c2fd4e, 3 },
    { 0, 0 },
    { 0x8c6afd52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd82fd56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb62fd59, 3 },
    { 0x5bc6fd5a, 3 },
    { 0, 0 },
    { 0xefafd5c, 3 },
    { 0xf792fd5d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2fd60, 3 },
    { 0, 0 },
    { 0x7b82fd62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90afd69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1efd6e, 3 },
    { 0, 0 },
    { 0xd46efd70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6fd74, 4 },
    { 0xfaeafd75, 3 },
    { 0x34fefd75, 5 },
    { 0x1ceafd77, 3 },
    { 0xa2fefd76, 3 },
    { 0, 0 },
    { 0x32b2fd7a, 3 },
    { 0, 0 },
    { 0x6cb2fd7c, 3 },
    { 0x96b6fd7c, 3 },
    { 0, 0 },
    { 0x1db2fd7f, 3 },
    { 0xfd82fd7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x803afd83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9afd88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf2fd8e, 3 },
    { 0, 0 },
    { 0xf952fd90, 3 },
    { 0xa30afd91, 3 },
    { 0xa2d6fd91, 4 },
    { 0x55c2fd93, 3 },
    { 0, 0 },
    { 0xf486fd95, 3 },
    { 0x1886fd95, 3 },
    { 0x889efd97, 3 },
    { 0x75efd97, 3 },
    { 0xb43afd98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7efd9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370efda0, 3 },
    { 0, 0 },
    { 0xeba6fda2, 4 },
    { 0x4a8efda2, 3 },
    { 0, 0 },
    { 0x55cafda5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caefda9, 4 },
    { 0x56f6fdaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73efdae, 3 },
    { 0x3b3afdae, 3 },
    { 0x8ec6fdb0, 3 },
    { 0x992afdb1, 4 },
    { 0x57fefdb2, 3 },
    { 0x3762fdae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b76fdb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c76fdba, 3 },
    { 0xfdd2fdbb, 3 },
    { 0xae02fdbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59afdc1, 3 },
    { 0x8db2fdc2, 4 },
    { 0x4392fdc1, 4 },
    { 0x8b26fdc2, 4 },
    { 0xaf76fdc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f6fdce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e92fdd2, 3 },
    { 0xebcefdd3, 3 },
    { 0x12eafdd4, 3 },
    { 0xbea2fdd5, 3 },
    { 0xc632fdd6, 4 },
    { 0xc9dafdd2, 3 },
    { 0x49f2fdd5, 3 },
    { 0, 0 },
    { 0xa7d6fdda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6efddd, 4 },
    { 0x1da6fddd, 3 },
    { 0x122afddd, 3 },
    { 0x726fde0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49efde5, 3 },
    { 0x133efde5, 4 },
    { 0, 0 },
    { 0x6a5afde8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74afded, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72fdf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11efdf3, 3 },
    { 0x9922fdf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e26fdfe, 5 },
    { 0xd556fdff, 4 },
    { 0x2966fe00, 3 },
    { 0, 0 },
    { 0xd82fe02, 3 },
    { 0, 0 },
    { 0x2a4afe04, 3 },
    { 0x9922fe05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7afe0f, 4 },
    { 0, 0 },
    { 0xe7befe11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6fe15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2086fe18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfefe1d, 3 },
    { 0x55d2fe1d, 3 },
    { 0xc1a2fe1f, 3 },
    { 0xfcf2fe20, 3 },
    { 0x785efe20, 3 },
    { 0x289efe22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb72fe25, 3 },
    { 0x8b96fe25, 3 },
    { 0x208afe25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12fe2d, 4 },
    { 0x6f7efe2e, 3 },
    { 0x943afe2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52fe34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b2fe37, 3 },
    { 0x32d6fe38, 3 },
    { 0x336afe37, 3 },
    { 0xabaafe38, 4 },
    { 0xf022fe39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dafe40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eafe48, 4 },
    { 0x8beefe49, 4 },
    { 0xdfdafe49, 4 },
    { 0x9b0afe4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b36fe4e, 3 },
    { 0, 0 },
    { 0x2766fe50, 4 },
    { 0x20aefe51, 3 },
    { 0xc206fe51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd2fe55, 4 },
    { 0, 0 },
    { 0x77f2fe57, 3 },
    { 0x6132fe58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6fe5e, 3 },
    { 0x5fe2fe5e, 3 },
    { 0x3062fe60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c82fe63, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbafe69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335afe6f, 3 },
    { 0xc526fe70, 4 },
    { 0x7dcafe71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787afe75, 3 },
    { 0x7c36fe75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f2fe79, 3 },
    { 0x3946fe7a, 3 },
    { 0x73fafe7b, 3 },
    { 0x809afe7a, 3 },
    { 0x709efe7d, 3 },
    { 0, 0 },
    { 0x39f6fe7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fafe85, 3 },
    { 0, 0 },
    { 0x44a2fe87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966fe8c, 3 },
    { 0xfc5efe8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aefe93, 3 },
    { 0, 0 },
    { 0x7752fe95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566fe99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16fe9c, 4 },
    { 0, 0 },
    { 0x9f8efe9e, 4 },
    { 0x7e3afe9e, 3 },
    { 0xe2f6fea0, 3 },
    { 0x52fafea0, 3 },
    { 0x4e1afea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd82feb0, 3 },
    { 0xfe0afeb1, 3 },
    { 0x509afeb0, 3 },
    { 0xb152feb3, 3 },
    { 0x9f72feb4, 3 },
    { 0xefeafeb3, 3 },
    { 0xcb7efeb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x623afeb9, 3 },
    { 0x520efeb9, 3 },
    { 0x334efebb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828afec0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x560efec3, 4 },
    { 0x854efec4, 3 },
    { 0x668afec5, 3 },
    { 0x6adafec4, 3 },
    { 0xd8e2fec4, 3 },
    { 0, 0 },
    { 0xc8a6fec9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0efecc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20efed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba02fed6, 3 },
    { 0, 0 },
    { 0x3cc2fed8, 3 },
    { 0xfdf2fed9, 3 },
    { 0xf6efeda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x785afedd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82fee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a76fee7, 3 },
    { 0x1376fee7, 3 },
    { 0xc4efee7, 3 },
    { 0xc62efeea, 3 },
    { 0xc696feea, 3 },
    { 0x9532feec, 3 },
    { 0xb64efee7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbafef0, 3 },
    { 0x4f4efef1, 3 },
    { 0x51c6fef2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6afef5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8befef8, 4 },
    { 0, 0 },
    { 0x694afefa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0afefd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf6ff01, 3 },
    { 0x2baff01, 3 },
    { 0xc3aeff01, 5 },
    { 0x6662ff04, 3 },
    { 0x93eff05, 4 },
    { 0x68e6ff05, 4 },
    { 0xd126ff05, 3 },
    { 0, 0 },
    { 0x3952ff09, 3 },
    { 0x1a72ff0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5552ff0e, 3 },
    { 0, 0 },
    { 0x3126ff10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68feff17, 3 },
    { 0x23e6ff18, 3 },
    { 0xfb36ff18, 3 },
    { 0xa712ff1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1aff22, 3 },
    { 0x4e6ff23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x143eff26, 3 },
    { 0x6122ff27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570aff2c, 3 },
    { 0x5ffaff2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292ff34, 3 },
    { 0x9ef2ff35, 3 },
    { 0xd882ff34, 3 },
    { 0xa12eff37, 5 },
    { 0x315eff37, 4 },
    { 0xf182ff38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97eff3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877eff44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb2ff4b, 3 },
    { 0, 0 },
    { 0xe3eff4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302ff51, 3 },
    { 0xf4eff51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e76ff57, 3 },
    { 0xe842ff57, 4 },
    { 0x120aff59, 3 },
    { 0x2a8aff5a, 3 },
    { 0x538eff59, 3 },
    { 0x8a46ff5b, 3 },
    { 0, 0 },
    { 0x4b1eff5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aaff64, 5 },
    { 0x331aff64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e2ff6e, 4 },
    { 0x20d6ff6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9366ff72, 4 },
    { 0x4602ff73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90eff7a, 3 },
    { 0, 0 },
    { 0x2c36ff7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afaff7f, 3 },
    { 0x41faff80, 3 },
    { 0xd36eff80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560aff85, 3 },
    { 0xa9e6ff86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1162ff89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe812ff90, 4 },
    { 0x165eff91, 3 },
    { 0, 0 },
    { 0x33a2ff93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5386ff9a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69affa4, 4 },
    { 0, 0 },
    { 0x8dfeffa6, 5 },
    { 0x4f0affa7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4beffac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37effaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ceffb2, 4 },
    { 0x3466ffb2, 4 },
    { 0xd126ffb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa6ffb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213affbd, 3 },
    { 0x4beffbd, 3 },
    { 0x3a0effbe, 3 },
    { 0x7cdeffc0, 4 },
    { 0, 0 },
    { 0xc12effc2, 3 },
    { 0x3eaeffc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32daffc7, 3 },
    { 0, 0 },
    { 0xc69effc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e6ffce, 3 },
    { 0xa5b2ffcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc216ffd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daffd7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afeffda, 4 },
    { 0x5fc2ffdb, 3 },
    { 0, 0 },
    { 0xe056ffdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa2ffe3, 4 },
    { 0x2dc6ffe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a6ffe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788effec, 3 },
    { 0, 0 },
    { 0x9ca6ffee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7afffb, 3 },
    { 0x3c6fffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f230004, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb0008, 4 },
    { 0, 0 },
    { 0x148b000a, 3 },
    { 0xb64f000b, 4 },
    { 0x932f000a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e370015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1f001b, 4 },
    { 0x8157001b, 4 },
    { 0x951f001b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237f0025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x501f0028, 3 },
    { 0x159f0028, 4 },
    { 0x72e7002a, 3 },
    { 0xf3e30029, 4 },
    { 0x64c7002c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5930030, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b0036, 4 },
    { 0xac4b0037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec30040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c30043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1027004a, 3 },
    { 0x4907004a, 3 },
    { 0xaf57004b, 4 },
    { 0, 0 },
    { 0xba43004e, 3 },
    { 0xfff7004f, 3 },
    { 0x870f004f, 4 },
    { 0xaa5b004f, 3 },
    { 0x99eb004f, 3 },
    { 0x472b0053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x643b0056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb005b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce30063, 4 },
    { 0, 0 },
    { 0x498b0065, 3 },
    { 0x228f0066, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e83006d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99470071, 3 },
    { 0xfd670072, 3 },
    { 0x913f0071, 3 },
    { 0, 0 },
    { 0xf0df0075, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb7007a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552f0080, 3 },
    { 0xcac30081, 3 },
    { 0x4bb70080, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd870086, 4 },
    { 0x29bf0087, 4 },
    { 0, 0 },
    { 0x57370089, 3 },
    { 0x69f30089, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55db0091, 3 },
    { 0, 0 },
    { 0xedd70093, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb0097, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39f009b, 3 },
    { 0xb477009c, 3 },
    { 0x7403009d, 3 },
    { 0x9e6f009c, 3 },
    { 0xc67b009b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93f00a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04300a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c700ac, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428f00b1, 3 },
    { 0xd3af00b2, 4 },
    { 0xa5ef00b1, 4 },
    { 0x958300b4, 4 },
    { 0x639f00b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664700bc, 5 },
    { 0x888b00bc, 3 },
    { 0x906300bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661b00c2, 4 },
    { 0x23ef00c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab300ca, 4 },
    { 0x74b00ca, 3 },
    { 0xd49700ca, 3 },
    { 0xa4d300cb, 4 },
    { 0x611300ce, 4 },
    { 0, 0 },
    { 0xd49b00d0, 5 },
    { 0x32b00d0, 3 },
    { 0x504b00d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad700d5, 3 },
    { 0x663f00d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900300da, 3 },
    { 0, 0 },
    { 0x93c300dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e700e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74700e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e300f0, 3 },
    { 0x8f00f1, 3 },
    { 0x2b3b00f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce300f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f00f9, 3 },
    { 0x11bb00f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608300fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf0102, 3 },
    { 0x905b0102, 3 },
    { 0xbb070103, 3 },
    { 0xa6f70105, 4 },
    { 0x1c0b0102, 3 },
    { 0x21a70107, 3 },
    { 0xa34b0108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa703010b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e30110, 4 },
    { 0, 0 },
    { 0xa2e70112, 3 },
    { 0xec3f0112, 3 },
    { 0x799b0113, 3 },
    { 0, 0 },
    { 0x304b0116, 3 },
    { 0, 0 },
    { 0x79eb0118, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33130129, 4 },
    { 0, 0 },
    { 0x18cb012b, 3 },
    { 0xa14f012c, 3 },
    { 0x7e1f012d, 3 },
    { 0, 0 },
    { 0xbacb012f, 3 },
    { 0x7ee70130, 3 },
    { 0, 0 },
    { 0xd61b0132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x400f0135, 3 },
    { 0xadaf0136, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fff013b, 4 },
    { 0, 0 },
    { 0xe5f013d, 3 },
    { 0xddb7013d, 3 },
    { 0x5953013e, 4 },
    { 0x9067013f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4af0144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba770147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d3014b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f0155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab0158, 3 },
    { 0x14970159, 3 },
    { 0x881b0158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ff015d, 5 },
    { 0, 0 },
    { 0x1233015f, 3 },
    { 0x3f630160, 3 },
    { 0x823b015f, 3 },
    { 0x8f670162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ef0167, 3 },
    { 0x2f4b0168, 3 },
    { 0xe7d30168, 3 },
    { 0xb73b0169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x298f016d, 3 },
    { 0, 0 },
    { 0xeeab016f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb0172, 3 },
    { 0x1eff0173, 4 },
    { 0xc4bf0173, 3 },
    { 0x6d330175, 4 },
    { 0xecb0176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca30179, 3 },
    { 0, 0 },
    { 0x8f13017b, 3 },
    { 0, 0 },
    { 0x697017d, 3 },
    { 0xa6bf017e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627f0183, 3 },
    { 0x103f0184, 3 },
    { 0, 0 },
    { 0x1f3f0186, 4 },
    { 0, 0 },
    { 0xbb270188, 3 },
    { 0xac0f0188, 4 },
    { 0x7443018a, 3 },
    { 0x14bb0189, 3 },
    { 0, 0 },
    { 0x645f018d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585f0193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23170196, 3 },
    { 0x562b0196, 3 },
    { 0x4ae70198, 4 },
    { 0, 0 },
    { 0x2453019a, 4 },
    { 0xca33019b, 4 },
    { 0xc7eb019c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d7019f, 4 },
    { 0xcb2701a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854301a4, 4 },
    { 0xc25701a4, 3 },
    { 0x2ed301a6, 3 },
    { 0x218301a7, 3 },
    { 0x85d701a7, 3 },
    { 0x33eb01a7, 3 },
    { 0xb1301aa, 3 },
    { 0x1a9f01ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x847f01ae, 3 },
    { 0xea4701af, 3 },
    { 0, 0 },
    { 0x588b01b1, 3 },
    { 0x895f01b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9b01b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5f01ba, 4 },
    { 0, 0 },
    { 0xa6f701bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b01c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b01ca, 3 },
    { 0x4bf01cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387701d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda701d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d701dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335301e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1b01e4, 3 },
    { 0xcb6701e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f01e8, 4 },
    { 0xf61301e9, 3 },
    { 0x5a5f01ea, 3 },
    { 0x6f8701eb, 3 },
    { 0x620b01e8, 3 },
    { 0x3e2f01e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80301f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x436f01f3, 3 },
    { 0x9e701f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b0202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15130206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe103020e, 4 },
    { 0x65a7020e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39b0213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf633021c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45130228, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79b022b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b022f, 3 },
    { 0, 0 },
    { 0x11970231, 4 },
    { 0x70ab0231, 3 },
    { 0xcaef0233, 5 },
    { 0xbe530234, 3 },
    { 0, 0 },
    { 0xf2eb0236, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70239, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404b0242, 4 },
    { 0x39530242, 3 },
    { 0x726f0244, 3 },
    { 0xd9630245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb730248, 3 },
    { 0x8a7f0249, 3 },
    { 0, 0 },
    { 0xf6eb024b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e17024e, 4 },
    { 0x5fcb024e, 3 },
    { 0, 0 },
    { 0x6d170251, 5 },
    { 0x6abf0252, 3 },
    { 0x4a9b0252, 3 },
    { 0x4cdb0254, 3 },
    { 0x87eb0252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2370258, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9130260, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd70268, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c70271, 3 },
    { 0xed8f0272, 3 },
    { 0x817b0271, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cb027b, 3 },
    { 0x3913027c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0b027f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b70282, 3 },
    { 0, 0 },
    { 0x5a770284, 5 },
    { 0, 0 },
    { 0x7f3b0286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45670289, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b87028c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36930294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c170297, 3 },
    { 0x38bf0298, 3 },
    { 0xea7f0298, 3 },
    { 0x2607029a, 4 },
    { 0xd803029a, 3 },
    { 0x18e7029c, 3 },
    { 0x425b029d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221702a7, 3 },
    { 0xc72702a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7302ab, 3 },
    { 0x71ff02ac, 3 },
    { 0xb22f02ad, 4 },
    { 0xd06702ae, 3 },
    { 0x676302ad, 4 },
    { 0xa5b02b0, 3 },
    { 0x7d4f02b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac302b8, 4 },
    { 0, 0 },
    { 0xd1db02ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49702c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f702c6, 3 },
    { 0x59eb02c7, 4 },
    { 0x283b02c8, 3 },
    { 0x717b02c7, 3 },
    { 0xde7702c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34b02d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b02e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171302e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d302ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb702f0, 3 },
    { 0xb11302f1, 3 },
    { 0xe40702f1, 3 },
    { 0xb91f02f0, 3 },
    { 0x289f02f4, 3 },
    { 0x906302f5, 4 },
    { 0xd70702f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea702fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb0300, 3 },
    { 0xf40f0300, 4 },
    { 0, 0 },
    { 0x855b0303, 3 },
    { 0, 0 },
    { 0x54d30305, 3 },
    { 0, 0 },
    { 0x940f0307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc077030b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b0310, 5 },
    { 0xc0b70311, 3 },
    { 0xcebb0311, 4 },
    { 0x89670310, 3 },
    { 0xdd430314, 3 },
    { 0, 0 },
    { 0x6d3f0316, 3 },
    { 0xd8030317, 3 },
    { 0, 0 },
    { 0xda5f0319, 4 },
    { 0x10af0319, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f7031f, 3 },
    { 0xdca3031f, 4 },
    { 0, 0 },
    { 0x6a270322, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f0329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b032d, 3 },
    { 0, 0 },
    { 0x1497032f, 3 },
    { 0xc6cb0330, 4 },
    { 0x5b670331, 3 },
    { 0x62430332, 3 },
    { 0xa7830333, 3 },
    { 0x746b0333, 3 },
    { 0xc33f0332, 3 },
    { 0x221f0336, 4 },
    { 0x34af0336, 3 },
    { 0xb3230338, 3 },
    { 0x4fc30338, 3 },
    { 0xd9c7033a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4f033d, 3 },
    { 0xf5cf033e, 3 },
    { 0x442f033f, 3 },
    { 0, 0 },
    { 0xb2170341, 3 },
    { 0xd47f0341, 3 },
    { 0xb4cb0341, 4 },
    { 0x24f70341, 3 },
    { 0xc51f0345, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8b0348, 3 },
    { 0xbef0349, 3 },
    { 0x4397034a, 4 },
    { 0x6a870349, 3 },
    { 0x352b034c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf0350, 3 },
    { 0xc56b0351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a370357, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367035f, 4 },
    { 0x664b035f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65170363, 3 },
    { 0x4d730364, 3 },
    { 0x932f0365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b93036e, 3 },
    { 0x259f036f, 3 },
    { 0x3bef0370, 4 },
    { 0x93f70371, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be30374, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30130379, 3 },
    { 0x28f30379, 3 },
    { 0, 0 },
    { 0x67e7037c, 3 },
    { 0xf23037c, 3 },
    { 0x6937037e, 3 },
    { 0x8267037c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d870383, 3 },
    { 0, 0 },
    { 0x6ce30385, 3 },
    { 0x61270386, 3 },
    { 0, 0 },
    { 0x3eeb0388, 4 },
    { 0x16370388, 4 },
    { 0xadbb038a, 3 },
    { 0x37bb038b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505f038f, 3 },
    { 0, 0 },
    { 0x6ef30391, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57770396, 3 },
    { 0, 0 },
    { 0x2de30398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497039c, 5 },
    { 0x875f039d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b03a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638b03a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c303aa, 5 },
    { 0, 0 },
    { 0x24bb03ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88af03b1, 3 },
    { 0xdf5703b2, 3 },
    { 0, 0 },
    { 0x44f703b4, 4 },
    { 0xa0a703b5, 4 },
    { 0xf6a703b5, 3 },
    { 0, 0 },
    { 0x6ad703b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9303bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6303c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x551b03c5, 4 },
    { 0xb1db03c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3703cc, 4 },
    { 0xc1fb03cd, 3 },
    { 0, 0 },
    { 0xf01b03cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a303d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b703db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f03eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d303ee, 4 },
    { 0, 0 },
    { 0x1e4f03f0, 3 },
    { 0x75e303f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec303f4, 3 },
    { 0, 0 },
    { 0xbd0f03f6, 4 },
    { 0x98303f7, 3 },
    { 0x106303f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ab03fb, 4 },
    { 0xa1c303fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b30402, 3 },
    { 0, 0 },
    { 0xdb6f0404, 3 },
    { 0x48530405, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77070408, 3 },
    { 0x885b0409, 3 },
    { 0x7e5b0408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10f041b, 4 },
    { 0, 0 },
    { 0xd0e7041d, 4 },
    { 0, 0 },
    { 0x9953041f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab0422, 4 },
    { 0x240b0422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ab0428, 3 },
    { 0x68230429, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f042c, 3 },
    { 0, 0 },
    { 0xfff3042e, 3 },
    { 0xb73042e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af30437, 3 },
    { 0x57ff0437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd267043c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd70440, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f0443, 3 },
    { 0xd2f70443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4df0447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f044a, 3 },
    { 0, 0 },
    { 0x5c77044c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5f0451, 3 },
    { 0x77cf0452, 3 },
    { 0xdd870453, 3 },
    { 0xb4eb0454, 4 },
    { 0x2f1b0451, 5 },
    { 0xfd070456, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbb045c, 4 },
    { 0x204f045d, 3 },
    { 0, 0 },
    { 0xbf37045f, 4 },
    { 0, 0 },
    { 0x291f0461, 3 },
    { 0, 0 },
    { 0xb7030463, 4 },
    { 0, 0 },
    { 0x1f6f0465, 4 },
    { 0xe3bb0465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62630469, 3 },
    { 0, 0 },
    { 0xa47046b, 4 },
    { 0xa90f046c, 3 },
    { 0x14e7046d, 4 },
    { 0xc59f046e, 3 },
    { 0xf32b046d, 3 },
    { 0x1f5f0470, 3 },
    { 0x32cf0471, 3 },
    { 0x9593046b, 3 },
    { 0xfed70473, 5 },
    { 0x4ac30474, 3 },
    { 0, 0 },
    { 0xc09b0476, 4 },
    { 0, 0 },
    { 0xcc7f0478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf047c, 3 },
    { 0, 0 },
    { 0xb75f047e, 3 },
    { 0, 0 },
    { 0x6d070480, 5 },
    { 0x78f70481, 3 },
    { 0x54eb0481, 4 },
    { 0xf12f0481, 3 },
    { 0x91eb0484, 3 },
    { 0, 0 },
    { 0x13ff0486, 3 },
    { 0x48870487, 3 },
    { 0x29eb0488, 3 },
    { 0xdadf0489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eb048c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bf049a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac7049e, 3 },
    { 0xd8cb049f, 3 },
    { 0x6e3049f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefb04a4, 3 },
    { 0, 0 },
    { 0x5de304a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa704aa, 3 },
    { 0x6ce704aa, 3 },
    { 0xc6c304ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936304b2, 4 },
    { 0x178f04b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345f04b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8304c0, 4 },
    { 0xab9704c1, 3 },
    { 0xa7ff04c1, 3 },
    { 0x7c9704c3, 3 },
    { 0x313b04c2, 3 },
    { 0x9ad704c5, 3 },
    { 0x52304c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9304c9, 3 },
    { 0, 0 },
    { 0x3d1704cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1db04d0, 3 },
    { 0, 0 },
    { 0x51c704d2, 3 },
    { 0xde0f04d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508304d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3f04da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785304e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77704ea, 3 },
    { 0, 0 },
    { 0x5e6b04ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ef04f0, 3 },
    { 0xc2b04f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689704f5, 3 },
    { 0xde4f04f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff04f9, 3 },
    { 0xfc2f04f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f030504, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930509, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abb050c, 4 },
    { 0, 0 },
    { 0x5e07050e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7370514, 4 },
    { 0x8c30514, 3 },
    { 0xba670515, 3 },
    { 0x53ef0516, 3 },
    { 0, 0 },
    { 0xadd70519, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827051e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cb0522, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f730525, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675f0529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebb052e, 3 },
    { 0xc73052e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb30536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7053c, 3 },
    { 0x997053c, 3 },
    { 0, 0 },
    { 0xc637053f, 3 },
    { 0x80570540, 3 },
    { 0x20eb053f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4530548, 3 },
    { 0x7fb30549, 4 },
    { 0x590b054a, 3 },
    { 0, 0 },
    { 0x154b054c, 3 },
    { 0, 0 },
    { 0x869f054e, 3 },
    { 0, 0 },
    { 0xa2d30550, 3 },
    { 0, 0 },
    { 0x15570552, 3 },
    { 0, 0 },
    { 0xfddb0554, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b3055b, 3 },
    { 0xdb3b055c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4db055f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a70562, 3 },
    { 0xdec70563, 3 },
    { 0x1e030564, 4 },
    { 0xcc4f0562, 3 },
    { 0x845f0566, 3 },
    { 0x87b0567, 3 },
    { 0, 0 },
    { 0xb8e70569, 3 },
    { 0xd597056a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a70571, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd70579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd197057d, 3 },
    { 0, 0 },
    { 0xf7ef057f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4b0582, 3 },
    { 0x1eaf0583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b058e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acb059b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b305a4, 3 },
    { 0xb19b05a5, 3 },
    { 0x1d4305a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf305ac, 3 },
    { 0x148f05ac, 3 },
    { 0xc27705ac, 3 },
    { 0x91fb05af, 3 },
    { 0x59eb05af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ff05b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc305b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x217b05ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf05be, 3 },
    { 0x7d4b05bf, 4 },
    { 0x203b05bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb05c4, 4 },
    { 0, 0 },
    { 0xf5d305c6, 3 },
    { 0, 0 },
    { 0x830b05c8, 3 },
    { 0x575305c9, 3 },
    { 0, 0 },
    { 0xd53305cb, 3 },
    { 0, 0 },
    { 0xe9f705cd, 3 },
    { 0xa52b05ce, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7b05d2, 3 },
    { 0xc57305d2, 3 },
    { 0x219705d4, 3 },
    { 0x3f0705d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee305da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b05e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x567f05e9, 3 },
    { 0x1f3f05e9, 3 },
    { 0x154305eb, 3 },
    { 0x3ac705eb, 3 },
    { 0xf9db05ea, 3 },
    { 0xbbaf05ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x393705f1, 4 },
    { 0xed7705f2, 4 },
    { 0x9b4b05f2, 4 },
    { 0, 0 },
    { 0x88d705f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x149705f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233f05fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f30602, 3 },
    { 0, 0 },
    { 0x5d370604, 4 },
    { 0, 0 },
    { 0xdabb0606, 3 },
    { 0x907b0607, 3 },
    { 0x28430608, 5 },
    { 0xfec70608, 3 },
    { 0xf30b060a, 3 },
    { 0xdf30609, 4 },
    { 0xe55f060c, 3 },
    { 0x8813060d, 3 },
    { 0x1ff060c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb70611, 4 },
    { 0, 0 },
    { 0x42570613, 4 },
    { 0, 0 },
    { 0x65870615, 3 },
    { 0x3f3f0616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701f061b, 3 },
    { 0x5657061c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239f0622, 3 },
    { 0, 0 },
    { 0x47b30624, 3 },
    { 0, 0 },
    { 0x755f0626, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778f062a, 4 },
    { 0x8973062b, 3 },
    { 0x42a7062c, 4 },
    { 0x6137062b, 3 },
    { 0, 0 },
    { 0x1993062f, 3 },
    { 0x25e70630, 3 },
    { 0, 0 },
    { 0x539b0632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f0635, 4 },
    { 0, 0 },
    { 0xdf0b0637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1717063a, 4 },
    { 0xed8b063b, 3 },
    { 0xab9b063c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f370644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae070647, 3 },
    { 0x74af0647, 3 },
    { 0x73af0647, 5 },
    { 0x7bb3064a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9b0653, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f065b, 3 },
    { 0x5ba7065c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a70660, 3 },
    { 0x73a70661, 3 },
    { 0x579f0662, 5 },
    { 0x203f0663, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89770668, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f066f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969b0673, 4 },
    { 0, 0 },
    { 0xc3170675, 4 },
    { 0xadfb0675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f37067a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9830681, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b7068a, 3 },
    { 0, 0 },
    { 0x938b068c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b3068f, 3 },
    { 0, 0 },
    { 0x50070691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1730696, 3 },
    { 0x4a0b0697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72f069b, 3 },
    { 0x5af7069b, 3 },
    { 0xcc07069d, 3 },
    { 0x15cb069b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01f06a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d306a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x681b06a9, 3 },
    { 0, 0 },
    { 0x7e3706ab, 3 },
    { 0, 0 },
    { 0x29a306ad, 3 },
    { 0xb44b06ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb06b2, 3 },
    { 0xf48f06b3, 3 },
    { 0xd20f06b4, 3 },
    { 0x24f06b5, 3 },
    { 0xb14f06b6, 3 },
    { 0, 0 },
    { 0xe33b06b8, 3 },
    { 0xe4bf06b9, 3 },
    { 0x809b06b9, 3 },
    { 0xf34b06ba, 3 },
    { 0x10b06b9, 3 },
    { 0xd76706bd, 3 },
    { 0xf5d706be, 3 },
    { 0x637f06be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9b06c2, 4 },
    { 0xcc1306c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00f06c6, 3 },
    { 0x47306c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a306cc, 3 },
    { 0, 0 },
    { 0x33eb06ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1706d2, 3 },
    { 0x70fb06d2, 4 },
    { 0x323b06d2, 3 },
    { 0xc13f06d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f06d8, 3 },
    { 0x138f06d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4706dc, 3 },
    { 0xefc706dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f06e6, 3 },
    { 0, 0 },
    { 0x74df06e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2b06ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4306fd, 3 },
    { 0x3af706fe, 4 },
    { 0, 0 },
    { 0xf02b0700, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f0703, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b0706, 4 },
    { 0x82630707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f070b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ff0718, 3 },
    { 0xf47b0718, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde7071c, 3 },
    { 0xd3af071c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cf0720, 3 },
    { 0, 0 },
    { 0xe4670722, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b0728, 4 },
    { 0xb42f0728, 3 },
    { 0xcd6b0728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9383072f, 3 },
    { 0x58db0730, 4 },
    { 0, 0 },
    { 0x139f0732, 5 },
    { 0, 0 },
    { 0x9d970734, 3 },
    { 0, 0 },
    { 0x1a730736, 4 },
    { 0x319f0737, 4 },
    { 0x87a70736, 3 },
    { 0x8ea70739, 3 },
    { 0x957f073a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a3074b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e30750, 3 },
    { 0, 0 },
    { 0x66070752, 3 },
    { 0xd60f0752, 4 },
    { 0, 0 },
    { 0x64db0755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefb0758, 3 },
    { 0xf1e70759, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7077075c, 4 },
    { 0xd003075d, 3 },
    { 0xbe23075d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3f0761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdf0764, 3 },
    { 0xeba70764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32b0768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b076f, 3 },
    { 0xc11b076f, 3 },
    { 0x960b0771, 3 },
    { 0x38eb0770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b70776, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adb0779, 3 },
    { 0x334b077a, 3 },
    { 0x9d9b077a, 5 },
    { 0x8f93077c, 4 },
    { 0x551b077c, 3 },
    { 0x81b7077e, 3 },
    { 0xaf7077f, 3 },
    { 0xc6e30780, 4 },
    { 0x519f0781, 3 },
    { 0xdc4b0782, 4 },
    { 0xa130782, 4 },
    { 0x16ab0782, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f0787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac3078b, 3 },
    { 0, 0 },
    { 0xc857078d, 3 },
    { 0x7683078d, 3 },
    { 0x4beb078f, 3 },
    { 0x7def0790, 3 },
    { 0, 0 },
    { 0x35170792, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f0796, 3 },
    { 0, 0 },
    { 0x83eb0798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7079d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1b07a0, 3 },
    { 0x251b07a1, 4 },
    { 0x2e1707a2, 3 },
    { 0, 0 },
    { 0xc45b07a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b07a8, 3 },
    { 0x5e7707a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39707ad, 4 },
    { 0, 0 },
    { 0xb3a307af, 3 },
    { 0x7f1707b0, 4 },
    { 0xd84707b1, 3 },
    { 0xd25307b2, 3 },
    { 0xa8a707b3, 3 },
    { 0x5b1307b0, 3 },
    { 0xb4ef07b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9b07bd, 5 },
    { 0x670707be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250f07c5, 3 },
    { 0xad9707c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f07ca, 3 },
    { 0, 0 },
    { 0xb3e307cc, 3 },
    { 0x107f07cd, 3 },
    { 0xed6307cc, 3 },
    { 0x7dd707cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6f07d3, 3 },
    { 0, 0 },
    { 0xa79f07d5, 3 },
    { 0xb16b07d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aff07da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6307df, 3 },
    { 0, 0 },
    { 0x559307e1, 3 },
    { 0xd2e707e2, 3 },
    { 0xb8df07e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973b07e7, 3 },
    { 0x835307e8, 3 },
    { 0x5b9307e7, 3 },
    { 0x49fb07e7, 3 },
    { 0xff4f07eb, 4 },
    { 0x255f07e7, 3 },
    { 0, 0 },
    { 0x780307ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x626707f1, 4 },
    { 0xa46707f2, 3 },
    { 0, 0 },
    { 0x23e707f4, 3 },
    { 0x9bbf07f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81707f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953b07fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b07ff, 3 },
    { 0xdab70800, 3 },
    { 0x9ba70801, 3 },
    { 0, 0 },
    { 0xdf1b0803, 3 },
    { 0x45f30803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0770807, 4 },
    { 0xa95b0808, 3 },
    { 0, 0 },
    { 0x1c3f080a, 3 },
    { 0xf91f080b, 3 },
    { 0x86d7080b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5b0811, 4 },
    { 0, 0 },
    { 0x113b0813, 3 },
    { 0x61ab0814, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3870818, 3 },
    { 0xa9f0819, 4 },
    { 0x24d7081a, 3 },
    { 0, 0 },
    { 0x7087081c, 3 },
    { 0, 0 },
    { 0xe2f081e, 4 },
    { 0xa4e3081f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5030823, 3 },
    { 0x3b8b0824, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cf0827, 3 },
    { 0, 0 },
    { 0xb170829, 3 },
    { 0x1bb7082a, 3 },
    { 0x1657082b, 3 },
    { 0x286f082c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbf082f, 4 },
    { 0x49370830, 4 },
    { 0x68b30830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787083b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2670840, 4 },
    { 0x6cdf0841, 3 },
    { 0, 0 },
    { 0x16b30843, 3 },
    { 0, 0 },
    { 0xfcab0845, 3 },
    { 0, 0 },
    { 0xa34f0847, 5 },
    { 0x8c0f0848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b7084b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f084e, 4 },
    { 0x747f084f, 3 },
    { 0x1c870850, 3 },
    { 0, 0 },
    { 0xb7730852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb0855, 3 },
    { 0, 0 },
    { 0x246b0857, 3 },
    { 0, 0 },
    { 0x7b7f0859, 3 },
    { 0xceeb0859, 3 },
    { 0x80ff085b, 3 },
    { 0x940b085c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1b085f, 4 },
    { 0, 0 },
    { 0x9ec70861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf0866, 4 },
    { 0, 0 },
    { 0x578f0868, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe630878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80f0882, 3 },
    { 0xa49f0882, 3 },
    { 0xd5530884, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc407088d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f30892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3270896, 4 },
    { 0xc7530896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b3089d, 3 },
    { 0x1eeb089e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79f08a2, 4 },
    { 0xbebb08a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750308a7, 4 },
    { 0x7d3f08a7, 3 },
    { 0xfa5708a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f708ae, 3 },
    { 0x6d5b08ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1b08b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e708b9, 3 },
    { 0, 0 },
    { 0x3cd308bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdf08c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45f08c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaf08cf, 3 },
    { 0x44e308d0, 3 },
    { 0x838708d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980f08d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a308da, 4 },
    { 0x376f08db, 3 },
    { 0xe94308db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a308e4, 3 },
    { 0, 0 },
    { 0x58f08e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x711f08e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411308f0, 3 },
    { 0, 0 },
    { 0x852b08f2, 3 },
    { 0x530708f3, 5 },
    { 0xaab08f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19b08f8, 3 },
    { 0, 0 },
    { 0xdabb08fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f708fd, 3 },
    { 0, 0 },
    { 0x406f08ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa30904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82b0909, 3 },
    { 0, 0 },
    { 0xf94f090b, 3 },
    { 0, 0 },
    { 0x56fb090d, 3 },
    { 0xff2b090d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac70913, 4 },
    { 0xba430914, 3 },
    { 0xdf0914, 3 },
    { 0x38eb0914, 3 },
    { 0x7c970916, 3 },
    { 0, 0 },
    { 0x3af30919, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945f091e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5870921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a30924, 4 },
    { 0x310f0924, 3 },
    { 0x456b0926, 4 },
    { 0, 0 },
    { 0xb4330928, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f092e, 3 },
    { 0x9ab092e, 3 },
    { 0x338b0930, 3 },
    { 0x701b0930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b70935, 3 },
    { 0, 0 },
    { 0x7bcf0937, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d3093b, 5 },
    { 0x54a3093c, 4 },
    { 0xa87093c, 4 },
    { 0x4cbb093e, 3 },
    { 0x1f8b093f, 3 },
    { 0x7c130940, 3 },
    { 0xdc5b0941, 4 },
    { 0xf7670941, 4 },
    { 0x12b30941, 4 },
    { 0x3a2b0944, 3 },
    { 0xd6970942, 4 },
    { 0xc6570946, 3 },
    { 0xe0bf0947, 3 },
    { 0xb5430945, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cf094d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb0950, 3 },
    { 0x8a2f0951, 4 },
    { 0x48bf0950, 4 },
    { 0xa4f70953, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b095a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eb095f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf730962, 4 },
    { 0x31370963, 3 },
    { 0, 0 },
    { 0x7f770965, 3 },
    { 0xadc70966, 3 },
    { 0, 0 },
    { 0x16570968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f096e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23230977, 3 },
    { 0, 0 },
    { 0x1a130979, 3 },
    { 0xd8ef097a, 3 },
    { 0x9d170979, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x977b097e, 3 },
    { 0x11e3097e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27db0982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b0987, 3 },
    { 0x5e630988, 3 },
    { 0xa4c30988, 3 },
    { 0x642f098a, 3 },
    { 0, 0 },
    { 0xe8cb098c, 3 },
    { 0, 0 },
    { 0xa137098e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fb0992, 4 },
    { 0xd6530992, 4 },
    { 0, 0 },
    { 0xd29f0995, 3 },
    { 0, 0 },
    { 0x163b0997, 3 },
    { 0xacf70998, 3 },
    { 0x7b2b0999, 3 },
    { 0x702f099a, 3 },
    { 0x186b0997, 3 },
    { 0xfedf099c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b09a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfb09a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x459309a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16709ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef309af, 3 },
    { 0, 0 },
    { 0x2e4709b1, 3 },
    { 0x6fe709b1, 4 },
    { 0xf14709b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9709b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98309ba, 5 },
    { 0x384b09ba, 3 },
    { 0xa2d709bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ff09bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8309c2, 3 },
    { 0x23709c3, 3 },
    { 0x7cb09c2, 4 },
    { 0, 0 },
    { 0x27709c6, 3 },
    { 0x96eb09c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7309cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x401309ce, 4 },
    { 0, 0 },
    { 0x20eb09d0, 3 },
    { 0xc39b09d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0309d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x713b09d8, 3 },
    { 0x5b4709d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad309e0, 3 },
    { 0x445709e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3309e5, 4 },
    { 0, 0 },
    { 0xd48309e7, 3 },
    { 0xfb0f09e7, 3 },
    { 0x717b09e9, 4 },
    { 0xf8c709ea, 3 },
    { 0xd8e309eb, 3 },
    { 0x1b0b09ec, 3 },
    { 0xfeff09eb, 3 },
    { 0x234f09eb, 4 },
    { 0xb26b09ef, 3 },
    { 0x2b0f09ed, 3 },
    { 0, 0 },
    { 0x5f6309f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d709f5, 3 },
    { 0x76af09f6, 3 },
    { 0xacbb09f7, 3 },
    { 0x3ce709f8, 3 },
    { 0x120b09f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c709fc, 3 },
    { 0xaee309fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c170a00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4b0a04, 4 },
    { 0xa68b0a05, 3 },
    { 0x7130a06, 4 },
    { 0x443f0a07, 3 },
    { 0, 0 },
    { 0x10030a09, 3 },
    { 0, 0 },
    { 0xbf330a0b, 3 },
    { 0xfd9f0a0c, 3 },
    { 0x317f0a0d, 4 },
    { 0x15930a0e, 3 },
    { 0xb7830a0f, 4 },
    { 0xaa730a10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab70a14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f30a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a830a25, 3 },
    { 0x525f0a26, 4 },
    { 0xe1f30a27, 3 },
    { 0, 0 },
    { 0x4c070a29, 3 },
    { 0x51570a2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7530a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65530a32, 3 },
    { 0, 0 },
    { 0x68670a34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b0a40, 3 },
    { 0x4edb0a41, 3 },
    { 0xd4c70a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f0a46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0370a49, 3 },
    { 0xa0d30a4a, 3 },
    { 0xcf430a4a, 4 },
    { 0x29d70a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61f0a4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef0a54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7230a58, 3 },
    { 0x849b0a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294f0a5f, 3 },
    { 0, 0 },
    { 0xdc1b0a61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee70a6a, 3 },
    { 0xf0c30a6b, 3 },
    { 0x84af0a6c, 3 },
    { 0x5a070a6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b0a70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f770a73, 3 },
    { 0x844b0a73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c30a7a, 4 },
    { 0, 0 },
    { 0x3370a7c, 3 },
    { 0x69c30a7d, 3 },
    { 0xc570a7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b0a89, 3 },
    { 0x53b70a8a, 4 },
    { 0xa17b0a8b, 3 },
    { 0x132f0a8c, 4 },
    { 0x82c30a8c, 4 },
    { 0x30f30a8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf0a91, 4 },
    { 0xb4870a92, 4 },
    { 0x6b030a93, 3 },
    { 0xb2b30a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fb0a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13830a9d, 3 },
    { 0, 0 },
    { 0xe5db0a9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8b0aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7f0aa6, 4 },
    { 0xcd1f0aa7, 3 },
    { 0x7ba30aa6, 3 },
    { 0, 0 },
    { 0x392b0aaa, 3 },
    { 0xbd6f0aaa, 3 },
    { 0x17230aac, 4 },
    { 0xb74f0aad, 3 },
    { 0x2ac70aae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3b0ab5, 4 },
    { 0xfb1b0ab6, 3 },
    { 0, 0 },
    { 0xf74f0ab8, 4 },
    { 0x455b0ab9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60630ac0, 5 },
    { 0, 0 },
    { 0x574f0ac2, 3 },
    { 0, 0 },
    { 0xf2e30ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ff0aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9b0ace, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b930ad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0b0ad9, 3 },
    { 0x8cf70ad9, 4 },
    { 0xf8330adb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9430ae0, 3 },
    { 0x1fcb0ae1, 3 },
    { 0, 0 },
    { 0x20630ae3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac70ae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdb0aeb, 3 },
    { 0x2a8b0aec, 3 },
    { 0xbbb0aec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5b0af1, 3 },
    { 0x370f0af2, 3 },
    { 0x203b0af2, 3 },
    { 0xa1ef0af2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77f0af8, 4 },
    { 0, 0 },
    { 0x2bc30afa, 3 },
    { 0x1ab0afb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8f0b07, 3 },
    { 0x30b07, 3 },
    { 0xf2bb0b07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf430b0c, 4 },
    { 0, 0 },
    { 0x2c930b0e, 4 },
    { 0, 0 },
    { 0x39630b10, 3 },
    { 0xef2b0b11, 3 },
    { 0xe2bb0b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f0b1a, 3 },
    { 0x575b0b1b, 3 },
    { 0, 0 },
    { 0x22b70b1d, 3 },
    { 0x52e70b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75630b21, 4 },
    { 0, 0 },
    { 0x8eef0b23, 3 },
    { 0, 0 },
    { 0xac9b0b25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46570b28, 3 },
    { 0x2acf0b29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7730b33, 3 },
    { 0, 0 },
    { 0xec5f0b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67f0b39, 3 },
    { 0, 0 },
    { 0x3f3b0b3b, 3 },
    { 0x837f0b3c, 3 },
    { 0xc90f0b3d, 4 },
    { 0, 0 },
    { 0x46b30b3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f70b43, 3 },
    { 0, 0 },
    { 0x86a30b45, 3 },
    { 0, 0 },
    { 0x9db30b47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcf0b4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8070b4e, 3 },
    { 0x70170b4f, 3 },
    { 0x717b0b50, 3 },
    { 0xdb1b0b51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a330b56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fb0b5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa730b64, 3 },
    { 0, 0 },
    { 0x503b0b66, 4 },
    { 0, 0 },
    { 0x9b530b68, 3 },
    { 0x18670b69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b0b70, 4 },
    { 0x17d70b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b870b78, 3 },
    { 0x93d30b79, 3 },
    { 0x36af0b78, 3 },
    { 0x478b0b7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90af0b7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eab0b82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f0b88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e270b90, 4 },
    { 0x55230b90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e330b95, 3 },
    { 0x90f70b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6b0b9a, 4 },
    { 0, 0 },
    { 0xa2c70b9c, 3 },
    { 0xec70b9d, 4 },
    { 0x77c30b9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca830ba2, 3 },
    { 0, 0 },
    { 0xd74b0ba4, 3 },
    { 0x441b0ba5, 3 },
    { 0x8dbf0ba6, 4 },
    { 0xee30ba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ab0baf, 4 },
    { 0xc9b30baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x571b0bb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cf0bb7, 3 },
    { 0, 0 },
    { 0xd77f0bb9, 3 },
    { 0x332b0bba, 3 },
    { 0x89970bbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71470bbf, 3 },
    { 0, 0 },
    { 0xf5830bc1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1570bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c70bc7, 4 },
    { 0x57eb0bc8, 4 },
    { 0xe89f0bc9, 4 },
    { 0x3b5f0bca, 3 },
    { 0, 0 },
    { 0xde7b0bcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a30bd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eb0bd9, 3 },
    { 0x899f0bda, 3 },
    { 0x7b30bd9, 3 },
    { 0x74f0bdc, 3 },
    { 0x918f0bdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f0be0, 5 },
    { 0, 0 },
    { 0x422b0be2, 3 },
    { 0, 0 },
    { 0x79370be4, 3 },
    { 0, 0 },
    { 0x59a70be6, 3 },
    { 0xbe630be7, 3 },
    { 0xc1670be6, 5 },
    { 0x6d430be8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12570bee, 3 },
    { 0xee1b0bee, 3 },
    { 0x9e5b0bef, 3 },
    { 0, 0 },
    { 0x403f0bf2, 3 },
    { 0xff7b0bf2, 4 },
    { 0xc52f0bf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd70bfc, 3 },
    { 0xa7f70bfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b0c04, 3 },
    { 0xa3a70c05, 4 },
    { 0xd7af0c05, 3 },
    { 0xc84b0c07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb30c0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fb0c12, 3 },
    { 0, 0 },
    { 0x68db0c14, 3 },
    { 0xef8b0c15, 3 },
    { 0, 0 },
    { 0xa1bf0c17, 3 },
    { 0xeeb30c17, 3 },
    { 0, 0 },
    { 0x81df0c1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f0c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c230c28, 3 },
    { 0x7a0f0c29, 3 },
    { 0, 0 },
    { 0xca4f0c2b, 3 },
    { 0, 0 },
    { 0x33730c2d, 5 },
    { 0x47370c2d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a630c35, 4 },
    { 0xccbb0c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81f0c3b, 4 },
    { 0, 0 },
    { 0x7d9f0c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d930c40, 3 },
    { 0x5f6b0c41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd570c44, 3 },
    { 0x82af0c45, 3 },
    { 0, 0 },
    { 0xac7b0c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5170c4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f0c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f0c58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2f0c5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e830c65, 3 },
    { 0x11130c66, 4 },
    { 0x5dbf0c67, 4 },
    { 0x709f0c66, 3 },
    { 0xfa030c69, 4 },
    { 0xb2d70c69, 3 },
    { 0x63bb0c6b, 4 },
    { 0x71d30c6c, 3 },
    { 0, 0 },
    { 0xc730c6e, 3 },
    { 0x42e30c6f, 4 },
    { 0, 0 },
    { 0x11c70c71, 4 },
    { 0, 0 },
    { 0xe3c30c73, 4 },
    { 0x4bef0c73, 3 },
    { 0x38c30c75, 3 },
    { 0x8370c73, 4 },
    { 0xd9e30c77, 3 },
    { 0xb3ab0c77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9df0c7c, 4 },
    { 0x2b670c7d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72430c81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65470c8d, 4 },
    { 0xa9830c8e, 3 },
    { 0x99270c8d, 3 },
    { 0, 0 },
    { 0xcb230c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6f0c96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddff0c9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe30ca4, 3 },
    { 0, 0 },
    { 0x56f0ca6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e430cab, 4 },
    { 0xacab0cac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0caf, 4 },
    { 0xebdb0cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6370cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97070cba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4b0cc0, 4 },
    { 0, 0 },
    { 0x2aff0cc2, 3 },
    { 0x87b0cc3, 3 },
    { 0x52130cc4, 3 },
    { 0x857b0cc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac70cc9, 3 },
    { 0x1c6b0cca, 3 },
    { 0xeecb0cca, 3 },
    { 0x7beb0cca, 3 },
    { 0, 0 },
    { 0x492f0cce, 4 },
    { 0, 0 },
    { 0x14070cd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b0cd4, 3 },
    { 0x12e30cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b730cdd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ab0ce1, 3 },
    { 0x37230ce2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f0ce6, 3 },
    { 0x81930ce7, 3 },
    { 0x95030ce7, 4 },
    { 0x7b5f0ce7, 4 },
    { 0x30bb0ce8, 3 },
    { 0xd72b0ce7, 4 },
    { 0x80e30ce8, 4 },
    { 0, 0 },
    { 0x2b870cee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7f0cf1, 3 },
    { 0x2b9b0cf2, 3 },
    { 0xc8830cf1, 4 },
    { 0x79c30cf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf670cf7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f70cff, 4 },
    { 0x613f0cff, 4 },
    { 0xa7b0cff, 4 },
    { 0x1f330d00, 5 },
    { 0xce830d03, 3 },
    { 0x5b330d04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f0d08, 4 },
    { 0xdc2f0d08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef930d10, 4 },
    { 0xb30b0d10, 3 },
    { 0x9cc30d12, 4 },
    { 0xb2770d13, 3 },
    { 0x4acf0d12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cab0d17, 4 },
    { 0, 0 },
    { 0x4b730d19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2930d1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bb0d23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7b0d26, 3 },
    { 0x95630d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161b0d2b, 3 },
    { 0x62670d2c, 3 },
    { 0x4ffb0d2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdab0d32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adb0d35, 3 },
    { 0, 0 },
    { 0x35930d37, 3 },
    { 0x319f0d37, 3 },
    { 0x64830d39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eef0d3c, 3 },
    { 0x53f0d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e470d41, 3 },
    { 0xc2070d41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21df0d45, 3 },
    { 0x41bf0d45, 3 },
    { 0xedc70d47, 3 },
    { 0, 0 },
    { 0x2bd70d49, 3 },
    { 0xfc870d4a, 3 },
    { 0x6c1f0d4b, 3 },
    { 0, 0 },
    { 0xa8c70d4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b0d50, 3 },
    { 0x7dff0d50, 3 },
    { 0xec870d52, 3 },
    { 0x96a70d53, 4 },
    { 0xb01b0d53, 3 },
    { 0, 0 },
    { 0xa6fb0d56, 3 },
    { 0xd6ff0d57, 3 },
    { 0x780b0d58, 3 },
    { 0, 0 },
    { 0x2d8f0d5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b0d5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79930d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55430d69, 3 },
    { 0x33ab0d69, 4 },
    { 0x525f0d69, 4 },
    { 0x13a30d6c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df0d6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782b0d73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49f0d77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b30d80, 3 },
    { 0x5dc70d80, 5 },
    { 0, 0 },
    { 0xaf1f0d83, 4 },
    { 0, 0 },
    { 0x26ab0d85, 4 },
    { 0, 0 },
    { 0x4fa30d87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfb0d8a, 3 },
    { 0x5d2f0d8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8530d93, 3 },
    { 0xc19b0d94, 5 },
    { 0, 0 },
    { 0xf4870d96, 3 },
    { 0, 0 },
    { 0xea6b0d98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11230d9e, 4 },
    { 0x2fff0d9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x521b0da2, 3 },
    { 0xcae30da3, 4 },
    { 0x7ac70da3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19530db0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4670db3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf0db7, 3 },
    { 0x7e970db8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10430dbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd670dbf, 3 },
    { 0x903b0dc0, 3 },
    { 0x1c3b0dc0, 5 },
    { 0xa9270dc2, 3 },
    { 0, 0 },
    { 0xb42f0dc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc630dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3670dca, 3 },
    { 0xb6ab0dcb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c670dd1, 3 },
    { 0x7970dd2, 3 },
    { 0x7b2f0dd2, 3 },
    { 0xec5f0dd4, 4 },
    { 0xac9f0dd4, 3 },
    { 0x1ccb0dd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddb0dda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95530de4, 4 },
    { 0, 0 },
    { 0x68630de6, 3 },
    { 0, 0 },
    { 0xfaf0de8, 4 },
    { 0xf9d70de9, 3 },
    { 0xeaf0dea, 3 },
    { 0x26cb0dea, 3 },
    { 0, 0 },
    { 0x124f0ded, 5 },
    { 0x29870ded, 3 },
    { 0xeb9f0dee, 3 },
    { 0, 0 },
    { 0xf6eb0df1, 3 },
    { 0x121b0df2, 3 },
    { 0x2a830df3, 3 },
    { 0, 0 },
    { 0x9d30df5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34270df9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184f0e00, 3 },
    { 0xbdd70e01, 3 },
    { 0x58b70e01, 3 },
    { 0xa9c70e03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824b0e0a, 3 },
    { 0, 0 },
    { 0xbc670e0c, 3 },
    { 0, 0 },
    { 0x6b1b0e0e, 3 },
    { 0xbb630e0e, 3 },
    { 0xd9b30e0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff0e16, 3 },
    { 0x8fd30e17, 3 },
    { 0xd7b70e17, 3 },
    { 0x39af0e19, 3 },
    { 0x69730e1a, 3 },
    { 0xee470e19, 4 },
    { 0x9c230e1a, 3 },
    { 0x873f0e1d, 4 },
    { 0xdb5f0e1d, 3 },
    { 0, 0 },
    { 0x9abb0e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f0e26, 3 },
    { 0x51570e26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3f0e2b, 3 },
    { 0x9fef0e2c, 4 },
    { 0xa6d30e2d, 3 },
    { 0x1ce70e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaef0e32, 3 },
    { 0, 0 },
    { 0x9b1b0e34, 3 },
    { 0, 0 },
    { 0x44b0e36, 3 },
    { 0x3a4f0e36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae30e3c, 3 },
    { 0, 0 },
    { 0xe4e30e3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86970e44, 3 },
    { 0x974f0e44, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb170e48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf0e4b, 4 },
    { 0xe7b70e4c, 3 },
    { 0, 0 },
    { 0x24cf0e4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86f0e57, 3 },
    { 0x38cb0e58, 3 },
    { 0x6bcf0e59, 3 },
    { 0, 0 },
    { 0x1b3f0e5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb30e60, 3 },
    { 0xee4b0e61, 3 },
    { 0, 0 },
    { 0x9feb0e63, 4 },
    { 0, 0 },
    { 0x4e970e65, 3 },
    { 0x56870e66, 4 },
    { 0x13470e67, 3 },
    { 0xbdd30e67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b30e6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edf0e70, 3 },
    { 0xe00b0e71, 3 },
    { 0x472f0e72, 3 },
    { 0x9f0b0e72, 3 },
    { 0xd10b0e74, 3 },
    { 0xd9570e75, 3 },
    { 0x4cbf0e70, 3 },
    { 0x66ef0e77, 3 },
    { 0x53f0e78, 3 },
    { 0x7eaf0e79, 3 },
    { 0xaf570e7a, 4 },
    { 0x652f0e72, 3 },
    { 0, 0 },
    { 0x73070e7d, 3 },
    { 0xe6cb0e7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35f0e85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f0e8a, 3 },
    { 0, 0 },
    { 0x92530e8c, 3 },
    { 0xa52f0e8d, 4 },
    { 0x211b0e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebf0e96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9b0e9c, 3 },
    { 0, 0 },
    { 0x65870e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4b0ea2, 3 },
    { 0xc4270ea3, 3 },
    { 0xf4870ea2, 5 },
    { 0x73c30ea4, 4 },
    { 0x26cb0ea5, 3 },
    { 0x28970ea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793b0eab, 3 },
    { 0x39d70eab, 3 },
    { 0x8b570ead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6f0eb1, 4 },
    { 0xbb70eb1, 3 },
    { 0x21470eb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50f0eb6, 3 },
    { 0xf3b30eb7, 4 },
    { 0, 0 },
    { 0x75b0eb9, 3 },
    { 0xfb0f0eba, 3 },
    { 0, 0 },
    { 0xe1830ebc, 4 },
    { 0, 0 },
    { 0x22470ebe, 3 },
    { 0x4e330ebe, 4 },
    { 0x29b0ec0, 4 },
    { 0x7a70ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc30ec5, 4 },
    { 0x158b0ec6, 3 },
    { 0, 0 },
    { 0xb1330ec8, 3 },
    { 0xb72f0ec8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82530ecd, 3 },
    { 0x49ab0ece, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f0ed2, 3 },
    { 0x441f0ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25b0ed6, 3 },
    { 0, 0 },
    { 0xca430ed8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a870ee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c270ee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb0ee6, 3 },
    { 0xacc70ee7, 3 },
    { 0x6e9b0ee6, 3 },
    { 0, 0 },
    { 0x4c670eea, 3 },
    { 0xdb870eea, 3 },
    { 0x3f070eec, 5 },
    { 0x28930eed, 3 },
    { 0x839b0eee, 3 },
    { 0, 0 },
    { 0x8c7b0ef0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556f0ef4, 3 },
    { 0, 0 },
    { 0xb8c70ef6, 4 },
    { 0x679b0ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f0efb, 3 },
    { 0x6a0f0efc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347b0f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c70f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea870f09, 4 },
    { 0, 0 },
    { 0x651f0f0b, 3 },
    { 0x3b1b0f0b, 3 },
    { 0x23970f0d, 4 },
    { 0, 0 },
    { 0xa1470f0f, 4 },
    { 0x40b70f0f, 3 },
    { 0, 0 },
    { 0x17f70f12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df0f17, 3 },
    { 0, 0 },
    { 0x22d30f19, 3 },
    { 0x3c4f0f1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55430f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc930f27, 4 },
    { 0xdb130f28, 4 },
    { 0, 0 },
    { 0x6dfb0f2a, 3 },
    { 0xc13b0f2b, 4 },
    { 0x185f0f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b0f30, 4 },
    { 0xa5db0f30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b0f38, 3 },
    { 0, 0 },
    { 0x22e70f3a, 3 },
    { 0xd93b0f3b, 3 },
    { 0xb570f3a, 3 },
    { 0, 0 },
    { 0x267f0f3e, 3 },
    { 0x2fff0f3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970f42, 3 },
    { 0x37830f43, 3 },
    { 0xf8ff0f44, 3 },
    { 0, 0 },
    { 0xd16b0f46, 4 },
    { 0x8cc30f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630f4a, 3 },
    { 0xe7a70f4b, 3 },
    { 0xda3b0f4c, 3 },
    { 0x91ab0f4d, 3 },
    { 0, 0 },
    { 0x37830f4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e70f52, 3 },
    { 0, 0 },
    { 0x5b230f54, 4 },
    { 0xbc030f55, 4 },
    { 0x96c70f55, 3 },
    { 0, 0 },
    { 0x27870f58, 3 },
    { 0xfc170f59, 3 },
    { 0xe3270f5a, 3 },
    { 0x14bf0f5b, 3 },
    { 0xc8eb0f5c, 4 },
    { 0, 0 },
    { 0xc88b0f5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b0f61, 5 },
    { 0x79630f62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15b0f67, 3 },
    { 0, 0 },
    { 0xc01b0f69, 3 },
    { 0xb6ff0f6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbf0f72, 3 },
    { 0, 0 },
    { 0x2f030f74, 3 },
    { 0x62fb0f75, 3 },
    { 0, 0 },
    { 0x65ef0f77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9530f7b, 3 },
    { 0x686b0f7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47770f7f, 3 },
    { 0xf4e30f7f, 3 },
    { 0xc3a70f81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833b0f85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb0f8b, 3 },
    { 0xcf770f8c, 3 },
    { 0x6fdf0f8d, 3 },
    { 0, 0 },
    { 0x58b30f8f, 3 },
    { 0x37330f8f, 3 },
    { 0, 0 },
    { 0xb9cb0f92, 3 },
    { 0xbdcb0f93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf30f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f30f99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cff0f9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61370fa4, 4 },
    { 0, 0 },
    { 0x98b30fa6, 3 },
    { 0xdd0f0fa7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26630faa, 4 },
    { 0x4fcb0fab, 3 },
    { 0x15df0fac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3370faf, 4 },
    { 0x65570fb0, 3 },
    { 0, 0 },
    { 0x85a30fb2, 4 },
    { 0, 0 },
    { 0xefc30fb4, 3 },
    { 0x40530fb4, 4 },
    { 0xb71f0fb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93370fb9, 3 },
    { 0x45df0fba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce70fc3, 3 },
    { 0, 0 },
    { 0x7930fc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38f0fc8, 4 },
    { 0x223b0fc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b0fcd, 3 },
    { 0, 0 },
    { 0x6b370fcf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ff0fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7330fd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102f0fdb, 3 },
    { 0x3bfb0fdb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f630fe5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9470fe8, 3 },
    { 0xab170fe9, 3 },
    { 0xda930fea, 3 },
    { 0xb3d30fea, 3 },
    { 0x85570fec, 3 },
    { 0x6ddf0fec, 3 },
    { 0x6f2f0fee, 3 },
    { 0, 0 },
    { 0xc1070ff0, 3 },
    { 0xacd70ff0, 3 },
    { 0xfa5f0ff2, 4 },
    { 0, 0 },
    { 0x3cc30ff4, 3 },
    { 0, 0 },
    { 0xf56f0ff6, 3 },
    { 0xb8d70ff7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35571003, 3 },
    { 0, 0 },
    { 0x78531005, 3 },
    { 0x9adb1005, 3 },
    { 0x74cf1007, 3 },
    { 0xaa5f1008, 3 },
    { 0x56c31009, 3 },
    { 0xc08f100a, 3 },
    { 0x26d7100a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdf1011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x172b1014, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8af1018, 3 },
    { 0xb8cf1019, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cf101c, 3 },
    { 0xc627101d, 3 },
    { 0x4097101e, 3 },
    { 0x8dab101e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e71026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac93102b, 4 },
    { 0x9c17102c, 3 },
    { 0x1d07102b, 3 },
    { 0, 0 },
    { 0x2253102f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d71032, 5 },
    { 0, 0 },
    { 0xd071034, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f31037, 3 },
    { 0xd23b1038, 3 },
    { 0, 0 },
    { 0xc8f7103a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84db103d, 4 },
    { 0xc137103d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed31044, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf423104c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f831051, 4 },
    { 0xd0e71051, 4 },
    { 0x534f1052, 4 },
    { 0x65f71054, 4 },
    { 0x259b1055, 4 },
    { 0, 0 },
    { 0x3fab1057, 3 },
    { 0x37571058, 3 },
    { 0x4a071057, 3 },
    { 0xb7f3105a, 3 },
    { 0, 0 },
    { 0x2d73105c, 3 },
    { 0xc0f105c, 3 },
    { 0, 0 },
    { 0x5e0f105f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33671062, 3 },
    { 0x73931062, 4 },
    { 0x5e0b1062, 3 },
    { 0x28671065, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1f1068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb106c, 3 },
    { 0x3583106d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0b1071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b1075, 4 },
    { 0x9f571076, 3 },
    { 0x8fb31077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de3107c, 3 },
    { 0, 0 },
    { 0x51cb107e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab31082, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5771085, 3 },
    { 0, 0 },
    { 0x68b1087, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d37108f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x258f1092, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f109a, 4 },
    { 0, 0 },
    { 0xef5f109c, 3 },
    { 0, 0 },
    { 0x3eb3109e, 3 },
    { 0xef3b109f, 4 },
    { 0x170b10a0, 3 },
    { 0x31a7109e, 5 },
    { 0x8d7310a2, 3 },
    { 0x230f10a0, 4 },
    { 0x232f10a4, 4 },
    { 0xd73710a5, 3 },
    { 0, 0 },
    { 0x74df10a7, 4 },
    { 0x690310a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfb10ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eb10b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e710ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f10c4, 4 },
    { 0xfc8f10c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b10ca, 4 },
    { 0xef6f10ca, 3 },
    { 0xdfdb10cc, 3 },
    { 0xb2df10cc, 3 },
    { 0x84ff10ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d710d1, 3 },
    { 0x5ddb10d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b10d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca710df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f10e3, 3 },
    { 0xbbbf10e3, 4 },
    { 0xe75f10e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f310e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d710ec, 3 },
    { 0xc9bf10ed, 4 },
    { 0xb67310ee, 3 },
    { 0, 0 },
    { 0x834b10f0, 4 },
    { 0x94ab10f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677b10f6, 3 },
    { 0x877b10f7, 3 },
    { 0x106f10f7, 3 },
    { 0xa54710f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba710fd, 3 },
    { 0x1c3f10fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a31104, 3 },
    { 0x70371104, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd167110a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x262f110d, 3 },
    { 0xdc97110e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b71114, 3 },
    { 0x48f71115, 3 },
    { 0, 0 },
    { 0x530f1117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655b111c, 3 },
    { 0, 0 },
    { 0x78a7111e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e71123, 4 },
    { 0x53bb1124, 3 },
    { 0x98b31123, 3 },
    { 0x17af1126, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f71129, 4 },
    { 0xea9f112a, 3 },
    { 0x2f37112a, 3 },
    { 0xbd07112c, 3 },
    { 0x925f112d, 3 },
    { 0x947112c, 3 },
    { 0xca631129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6b1132, 3 },
    { 0x7e831133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce71136, 3 },
    { 0, 0 },
    { 0xcb9b1138, 3 },
    { 0, 0 },
    { 0xdd03113a, 3 },
    { 0, 0 },
    { 0x6b27113c, 4 },
    { 0xbadf113d, 4 },
    { 0xc6d3113e, 3 },
    { 0x6d1b113f, 4 },
    { 0, 0 },
    { 0xdd671141, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43931145, 3 },
    { 0, 0 },
    { 0xc9a31147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d13114a, 3 },
    { 0, 0 },
    { 0xc2b7114c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130f1150, 3 },
    { 0xc2731151, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c31159, 3 },
    { 0, 0 },
    { 0xf947115b, 4 },
    { 0, 0 },
    { 0xc7d3115d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8af1160, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76171166, 3 },
    { 0x5f771167, 4 },
    { 0x6cab1168, 4 },
    { 0xf5971169, 3 },
    { 0x652b1169, 4 },
    { 0x6fbb116b, 4 },
    { 0x9817116b, 3 },
    { 0x9fa7116d, 3 },
    { 0xc867116c, 3 },
    { 0x4abf1168, 3 },
    { 0x2ec71168, 3 },
    { 0xe88f1171, 3 },
    { 0, 0 },
    { 0x426f1173, 3 },
    { 0, 0 },
    { 0xba6f1175, 3 },
    { 0, 0 },
    { 0x6a131177, 3 },
    { 0x42171178, 3 },
    { 0, 0 },
    { 0x747f117a, 3 },
    { 0x8a2f117b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf71181, 3 },
    { 0x59671182, 3 },
    { 0x93831183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12331186, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca31189, 3 },
    { 0x9e87118a, 3 },
    { 0x25e7118b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cf118e, 4 },
    { 0x774f118f, 3 },
    { 0, 0 },
    { 0x16cb1191, 3 },
    { 0x79eb1192, 3 },
    { 0xefb31193, 3 },
    { 0xc4071191, 3 },
    { 0xdde31192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13971198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce63119d, 3 },
    { 0x834b119e, 4 },
    { 0, 0 },
    { 0x5f2311a0, 3 },
    { 0x3b5311a0, 3 },
    { 0, 0 },
    { 0xfc5311a3, 3 },
    { 0, 0 },
    { 0x131711a5, 3 },
    { 0, 0 },
    { 0x197f11a7, 3 },
    { 0x14fb11a7, 3 },
    { 0, 0 },
    { 0xfadf11aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66311ad, 3 },
    { 0, 0 },
    { 0xc41711af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f11b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38b11ba, 3 },
    { 0x12cf11ba, 4 },
    { 0x9c4b11bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0311bf, 3 },
    { 0x86ff11c0, 3 },
    { 0x307311c1, 3 },
    { 0x20eb11c2, 3 },
    { 0x249f11c0, 3 },
    { 0x723b11c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724711c9, 3 },
    { 0xfc5f11ca, 5 },
    { 0x1ad711cb, 3 },
    { 0x941311ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbf11d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551311dc, 3 },
    { 0x3711dd, 3 },
    { 0xec7b11dc, 3 },
    { 0, 0 },
    { 0x123b11e0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db311e5, 3 },
    { 0, 0 },
    { 0xca6311e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b11eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x812f11ee, 3 },
    { 0, 0 },
    { 0x1de311f0, 4 },
    { 0x8a6711f1, 3 },
    { 0xf49f11f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e711ff, 3 },
    { 0x1ab711ff, 3 },
    { 0x989f1201, 3 },
    { 0x5fa711ff, 3 },
    { 0, 0 },
    { 0x2c0f1204, 3 },
    { 0xc0f71204, 3 },
    { 0, 0 },
    { 0x37b71207, 4 },
    { 0x2bf1208, 3 },
    { 0x76c31209, 3 },
    { 0x38771209, 3 },
    { 0x79b71209, 3 },
    { 0x8d1b120c, 3 },
    { 0xce07120c, 3 },
    { 0, 0 },
    { 0x897120f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x914f1212, 3 },
    { 0xe771213, 3 },
    { 0x18e71212, 4 },
    { 0x35971214, 4 },
    { 0x2b3f1216, 3 },
    { 0x21b1216, 4 },
    { 0xbbef1212, 3 },
    { 0xe93f1219, 3 },
    { 0x43e71219, 3 },
    { 0x4b77121b, 5 },
    { 0x7097121a, 3 },
    { 0xe70f1213, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b1220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e531225, 3 },
    { 0, 0 },
    { 0x62071227, 4 },
    { 0, 0 },
    { 0x332b1229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97122d, 3 },
    { 0x3747122d, 3 },
    { 0x8853122f, 3 },
    { 0x38e3122e, 3 },
    { 0, 0 },
    { 0xe29f1232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x997b1235, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2131238, 3 },
    { 0, 0 },
    { 0x171f123a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b97123e, 4 },
    { 0x1477123f, 3 },
    { 0x61fb123f, 3 },
    { 0xecdb123e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43631248, 3 },
    { 0x10bf1249, 3 },
    { 0xfd33124a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb124e, 3 },
    { 0xc60b124f, 3 },
    { 0x270b124e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20f1253, 4 },
    { 0, 0 },
    { 0xabef1255, 3 },
    { 0xf03f1256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6771259, 3 },
    { 0xb853125a, 3 },
    { 0xebff125b, 5 },
    { 0, 0 },
    { 0x579f125d, 3 },
    { 0xc97f125e, 4 },
    { 0x9297125f, 3 },
    { 0x2657125e, 3 },
    { 0xd4e31261, 3 },
    { 0x94b31262, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b1269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9627126d, 4 },
    { 0xb29b126e, 3 },
    { 0, 0 },
    { 0xb1d71270, 5 },
    { 0xfaa31270, 4 },
    { 0xf3b31272, 3 },
    { 0xab731273, 3 },
    { 0x519b1274, 4 },
    { 0x88af1275, 4 },
    { 0x2b771275, 3 },
    { 0x7c6b1275, 5 },
    { 0x603b1273, 4 },
    { 0xa29f1277, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e71282, 3 },
    { 0, 0 },
    { 0x2cb71284, 5 },
    { 0x6a831284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac7128a, 3 },
    { 0xb3c3128a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30231292, 3 },
    { 0x2e271293, 3 },
    { 0, 0 },
    { 0x959f1295, 3 },
    { 0xc4131296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee3129c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3f12a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f712a4, 3 },
    { 0xec7312a4, 4 },
    { 0xad4712a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548712aa, 3 },
    { 0xdfeb12ab, 4 },
    { 0xeec712ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f12af, 3 },
    { 0, 0 },
    { 0xc1712b1, 4 },
    { 0x58af12b1, 3 },
    { 0x26df12b1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b312b6, 4 },
    { 0x3b4f12b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bff12bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb712c0, 4 },
    { 0x20af12c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfab12c6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302b12ca, 3 },
    { 0x148712cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6712cf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84f12d2, 3 },
    { 0, 0 },
    { 0x881f12d4, 4 },
    { 0xa21f12d4, 4 },
    { 0, 0 },
    { 0x52b312d7, 4 },
    { 0x9ce712d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adf12db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd312de, 4 },
    { 0, 0 },
    { 0x94b712e0, 3 },
    { 0x8a5312e1, 3 },
    { 0xfa7f12e2, 3 },
    { 0xb28712e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x722312e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b12ec, 3 },
    { 0, 0 },
    { 0x1ea712ee, 3 },
    { 0xd41712ee, 4 },
    { 0x6e7b12ee, 3 },
    { 0, 0 },
    { 0xe68f12f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2312f6, 4 },
    { 0, 0 },
    { 0xd04b12f8, 3 },
    { 0xe2712f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f12fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70331300, 3 },
    { 0x39bf1301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f1304, 3 },
    { 0, 0 },
    { 0x4d5b1306, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34771312, 4 },
    { 0xdbfb1313, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x241f1316, 3 },
    { 0xe13b1317, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f131a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe963131d, 3 },
    { 0, 0 },
    { 0x8fef131f, 3 },
    { 0x53271320, 4 },
    { 0x6b9f1321, 3 },
    { 0, 0 },
    { 0x6b031323, 3 },
    { 0, 0 },
    { 0xf6e71325, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcf1328, 4 },
    { 0xf6bf1328, 5 },
    { 0x667132a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a7132d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x59731330, 3 },
    { 0x5a1b1331, 4 },
    { 0xaa0f1332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72af1337, 3 },
    { 0x78af1337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5f133b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7db1341, 3 },
    { 0x69bb1341, 4 },
    { 0, 0 },
    { 0xf0771344, 4 },
    { 0xc2271344, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c93134a, 4 },
    { 0x22f3134a, 3 },
    { 0x9687134a, 3 },
    { 0x7387134a, 3 },
    { 0x6ecf134d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e271354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eab135a, 3 },
    { 0xb5db135b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9271367, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fb136e, 3 },
    { 0xbfb136f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x912b1372, 3 },
    { 0xe3271373, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0631377, 3 },
    { 0xf83f1378, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfb137b, 3 },
    { 0x346f137c, 4 },
    { 0, 0 },
    { 0x90ef137e, 3 },
    { 0x549b137e, 4 },
    { 0x48fb137f, 4 },
    { 0, 0 },
    { 0xba1b1382, 3 },
    { 0, 0 },
    { 0xe1b31384, 4 },
    { 0xc01f1385, 4 },
    { 0x4e731386, 3 },
    { 0x998b1387, 3 },
    { 0x7e4f1388, 3 },
    { 0x8f131389, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b138c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b931394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x218f1397, 3 },
    { 0x15871398, 3 },
    { 0x9d7b1397, 3 },
    { 0x6a9b139a, 3 },
    { 0, 0 },
    { 0x7483139c, 4 },
    { 0x30cf139d, 3 },
    { 0x111f139c, 4 },
    { 0x3c93139c, 3 },
    { 0x9df3139d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbf13a5, 3 },
    { 0x8a713a6, 3 },
    { 0xaa7b13a7, 3 },
    { 0, 0 },
    { 0xef1713a9, 3 },
    { 0x6b713aa, 4 },
    { 0xa27b13aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd713b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b13b6, 3 },
    { 0xcb6713b7, 4 },
    { 0xb94b13b7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b13bc, 4 },
    { 0, 0 },
    { 0xa54f13be, 3 },
    { 0x1eb13bf, 4 },
    { 0x73af13bf, 3 },
    { 0xfaaf13c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25af13c8, 4 },
    { 0x253f13c9, 3 },
    { 0, 0 },
    { 0x309313cb, 3 },
    { 0xdca313cc, 3 },
    { 0x228713cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cb13d5, 3 },
    { 0, 0 },
    { 0x6de313d7, 3 },
    { 0x736f13d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52713db, 3 },
    { 0xeb0313dc, 3 },
    { 0x56bb13db, 3 },
    { 0, 0 },
    { 0xd3bb13df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84b13e3, 3 },
    { 0xf19713e4, 3 },
    { 0x322713e4, 3 },
    { 0x590713e5, 3 },
    { 0x2afb13e4, 4 },
    { 0xc18b13e8, 3 },
    { 0x66b713e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3713ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b13f1, 3 },
    { 0x794f13f1, 3 },
    { 0, 0 },
    { 0x91ab13f4, 3 },
    { 0, 0 },
    { 0x4e8313f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0313f9, 3 },
    { 0, 0 },
    { 0x2d6b13fb, 4 },
    { 0xae3713fc, 4 },
    { 0x6ab13fc, 3 },
    { 0x377f13fe, 3 },
    { 0xcf4713ff, 4 },
    { 0x2cf71400, 3 },
    { 0x659b13fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9631404, 3 },
    { 0x3eb1404, 3 },
    { 0, 0 },
    { 0xc5fb1407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993140c, 3 },
    { 0, 0 },
    { 0xb1ff140e, 3 },
    { 0x3cc3140f, 4 },
    { 0x96731410, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42431414, 3 },
    { 0x8b5f1415, 3 },
    { 0x80bf1414, 4 },
    { 0, 0 },
    { 0x11971418, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf773141f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b1422, 3 },
    { 0, 0 },
    { 0x37bb1424, 3 },
    { 0xe1331425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b71428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6b142d, 4 },
    { 0, 0 },
    { 0xbeb3142f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b1432, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2471436, 3 },
    { 0, 0 },
    { 0xe5b1438, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57143b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6f143e, 3 },
    { 0, 0 },
    { 0x4871440, 3 },
    { 0, 0 },
    { 0x961b1442, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75031446, 3 },
    { 0x2b771446, 3 },
    { 0, 0 },
    { 0x171449, 4 },
    { 0xc97f144a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89f144e, 3 },
    { 0x3097144e, 3 },
    { 0, 0 },
    { 0xb5fb1451, 5 },
    { 0xdf0f1452, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97af1456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb145d, 3 },
    { 0xf43f145e, 3 },
    { 0x8c17145f, 3 },
    { 0xaea31460, 5 },
    { 0x24ef1461, 3 },
    { 0xd19b1462, 3 },
    { 0x5aab1462, 3 },
    { 0x72d31462, 3 },
    { 0, 0 },
    { 0x54731466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df1469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62db146d, 3 },
    { 0x6cc3146e, 3 },
    { 0xf26f146f, 4 },
    { 0xdcb146e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a31473, 3 },
    { 0x705b1474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12f147c, 3 },
    { 0, 0 },
    { 0x3a43147e, 5 },
    { 0x6c6b147e, 3 },
    { 0xe7b7147e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef1484, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450b148c, 3 },
    { 0x877f148d, 3 },
    { 0, 0 },
    { 0x355b148f, 3 },
    { 0xb1a31490, 4 },
    { 0xa9471490, 4 },
    { 0xbd1b1492, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2f1495, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7149e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cf14a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490314a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f14ab, 3 },
    { 0x81a314ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7b14af, 3 },
    { 0xa7c314af, 3 },
    { 0xd85314b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e314b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367f14ba, 3 },
    { 0xdd3f14bb, 3 },
    { 0xce4f14bc, 3 },
    { 0, 0 },
    { 0x387f14be, 3 },
    { 0xbd3714bf, 3 },
    { 0x394314c0, 3 },
    { 0x6b8b14c0, 3 },
    { 0x86d314c1, 3 },
    { 0xcba714c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d714c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf314ca, 3 },
    { 0, 0 },
    { 0xc12b14cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57b14d0, 3 },
    { 0x92b314d1, 4 },
    { 0xed0314d2, 3 },
    { 0x3ff314d3, 4 },
    { 0xbfab14d4, 5 },
    { 0xbf7f14d4, 4 },
    { 0x886714d6, 4 },
    { 0x8b4b14d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd714db, 3 },
    { 0, 0 },
    { 0xd1df14dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6b14e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b14e4, 3 },
    { 0x427b14e5, 3 },
    { 0xb6e314e4, 3 },
    { 0, 0 },
    { 0x1ae314e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d314ed, 3 },
    { 0xba0314ee, 3 },
    { 0xc54b14ef, 3 },
    { 0x96b714f0, 3 },
    { 0x201714f1, 4 },
    { 0xc4314ef, 3 },
    { 0, 0 },
    { 0x2d9314f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f14f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f14fa, 3 },
    { 0, 0 },
    { 0xefa714fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb1500, 3 },
    { 0x23f1501, 3 },
    { 0xdb8f1501, 4 },
    { 0x91a71503, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9631507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df150a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89f150e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb71513, 3 },
    { 0xb1bf1514, 3 },
    { 0x7fbb1515, 3 },
    { 0x67971516, 3 },
    { 0x9b0b1517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b1520, 3 },
    { 0x4ccb1521, 3 },
    { 0x5edf1522, 4 },
    { 0xed5f1521, 4 },
    { 0x59231524, 4 },
    { 0xd3fb1521, 3 },
    { 0x1c071520, 4 },
    { 0x61cf1527, 4 },
    { 0x22831528, 3 },
    { 0x7c671529, 4 },
    { 0x7931528, 3 },
    { 0xcb9b1527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc773152e, 3 },
    { 0, 0 },
    { 0xb2a71530, 3 },
    { 0, 0 },
    { 0xb1b31532, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eb1535, 3 },
    { 0xf45b1536, 3 },
    { 0, 0 },
    { 0xd74f1538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bb153b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf643153e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2371545, 4 },
    { 0x60ff1546, 4 },
    { 0, 0 },
    { 0x89eb1548, 3 },
    { 0, 0 },
    { 0x376f154a, 4 },
    { 0x1e2b154a, 3 },
    { 0x597154c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba671550, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f1553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf3155d, 4 },
    { 0xe5ff155e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e31561, 4 },
    { 0x4ec71561, 3 },
    { 0, 0 },
    { 0x729f1564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388b1568, 3 },
    { 0xc2db1568, 3 },
    { 0xf92f156a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e31576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775b157c, 3 },
    { 0x6493157d, 3 },
    { 0xcaf3157c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde471581, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddb1585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5b1589, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03158d, 3 },
    { 0xa00f158e, 3 },
    { 0x451f158f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433b1596, 3 },
    { 0x70c31597, 3 },
    { 0, 0 },
    { 0xfab1599, 3 },
    { 0x6757159a, 3 },
    { 0x2303159b, 3 },
    { 0x2777159b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1b15a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbb15a6, 3 },
    { 0x3e0715a6, 5 },
    { 0x6db715a7, 3 },
    { 0x8acb15a8, 3 },
    { 0, 0 },
    { 0x2efb15ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fb15ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x935315b1, 4 },
    { 0xdecb15b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501b15b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5f15bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f15c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5f15c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d315c9, 3 },
    { 0x913715c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143715d2, 4 },
    { 0, 0 },
    { 0x20fb15d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x515f15d7, 3 },
    { 0x11cf15d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8715e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f315eb, 3 },
    { 0xa6b315ec, 3 },
    { 0x596f15ed, 3 },
    { 0xc0ff15ed, 4 },
    { 0, 0 },
    { 0x980b15f0, 3 },
    { 0, 0 },
    { 0xea1b15f2, 3 },
    { 0xa2a715f3, 4 },
    { 0xc7715f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e315f7, 4 },
    { 0x56b715f7, 4 },
    { 0x821315f7, 3 },
    { 0xbe7315fa, 4 },
    { 0xc21315fb, 3 },
    { 0xeb2f15fc, 4 },
    { 0x291715fc, 3 },
    { 0xb62f15fe, 4 },
    { 0x95ab15fe, 4 },
    { 0x595315f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d31608, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4b161a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de71620, 3 },
    { 0xc8f31621, 3 },
    { 0x68231622, 3 },
    { 0xdce71623, 4 },
    { 0x23771624, 3 },
    { 0x3e131625, 3 },
    { 0x93671626, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x583f1629, 3 },
    { 0x41f162a, 4 },
    { 0xe9df162a, 3 },
    { 0x2ff162a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67931631, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95931635, 3 },
    { 0, 0 },
    { 0xe8b1637, 3 },
    { 0x835b1638, 3 },
    { 0x868f1639, 4 },
    { 0, 0 },
    { 0xc3b7163b, 3 },
    { 0xe1b3163c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72471641, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91f1646, 3 },
    { 0x41d31646, 4 },
    { 0x2a1b1648, 3 },
    { 0xba7b1649, 4 },
    { 0xdcc7164a, 3 },
    { 0xad1f164a, 3 },
    { 0xff731646, 5 },
    { 0, 0 },
    { 0x84e3164e, 3 },
    { 0x1b33164f, 3 },
    { 0xcc1f1650, 3 },
    { 0x7cb1650, 4 },
    { 0xf52b1652, 3 },
    { 0x9831653, 3 },
    { 0xef7f1652, 3 },
    { 0xabfb1655, 3 },
    { 0x69231654, 3 },
    { 0, 0 },
    { 0x32571658, 3 },
    { 0x77cb1659, 3 },
    { 0xb7f165a, 3 },
    { 0xfaef165a, 3 },
    { 0xfd1f165b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517166a, 3 },
    { 0, 0 },
    { 0x71c7166c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ff1670, 3 },
    { 0xd30f1671, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859f1676, 4 },
    { 0x920f1676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5423167a, 3 },
    { 0xf663167a, 3 },
    { 0x4047167c, 3 },
    { 0x21f3167d, 3 },
    { 0x1da7167e, 3 },
    { 0xd2ab167f, 3 },
    { 0x12d7167f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0f1683, 3 },
    { 0, 0 },
    { 0xae4b1685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813168b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x273168e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x119b1691, 4 },
    { 0x8ac31691, 3 },
    { 0, 0 },
    { 0xb1171694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bb169c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2853169f, 4 },
    { 0x6b3f16a0, 3 },
    { 0x8fff16a1, 3 },
    { 0xc9df16a2, 3 },
    { 0x86ab16a3, 3 },
    { 0x182b16a0, 4 },
    { 0, 0 },
    { 0x1af316a6, 3 },
    { 0x662f16a6, 3 },
    { 0, 0 },
    { 0xd2e716a9, 4 },
    { 0x5a5316aa, 3 },
    { 0, 0 },
    { 0xc30f16ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdf16ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61316bd, 3 },
    { 0xb01b16bd, 3 },
    { 0x7db16bf, 3 },
    { 0xa81716c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436b16c6, 3 },
    { 0xbb8316c6, 4 },
    { 0xf82b16c8, 3 },
    { 0, 0 },
    { 0x8d0b16ca, 3 },
    { 0, 0 },
    { 0x241b16cc, 5 },
    { 0xe62316cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82716d2, 3 },
    { 0xf4b316d3, 3 },
    { 0xbeef16d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7b16d7, 3 },
    { 0x65b716d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68f16dd, 3 },
    { 0, 0 },
    { 0x3a0b16df, 3 },
    { 0, 0 },
    { 0x372f16e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2316e4, 3 },
    { 0, 0 },
    { 0xd32316e6, 3 },
    { 0xa4cf16e7, 3 },
    { 0, 0 },
    { 0xb65316e9, 3 },
    { 0, 0 },
    { 0x880316eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50716ee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f716f3, 3 },
    { 0xf08f16f4, 4 },
    { 0x467316f4, 3 },
    { 0xbdaf16f6, 3 },
    { 0xfffb16f7, 3 },
    { 0x754f16f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef16fb, 4 },
    { 0x1c5316fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46331700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x767f1703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f1708, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744b170d, 4 },
    { 0xfdab170e, 3 },
    { 0x5583170d, 3 },
    { 0, 0 },
    { 0xb4b71711, 3 },
    { 0x80c71711, 3 },
    { 0x9cb1713, 4 },
    { 0x3baf1713, 3 },
    { 0, 0 },
    { 0x6c471716, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53171b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf7171e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4b1721, 3 },
    { 0x9f271722, 4 },
    { 0x3e0b1722, 5 },
    { 0, 0 },
    { 0xb1b1725, 3 },
    { 0, 0 },
    { 0x133b1727, 4 },
    { 0, 0 },
    { 0xbdff1729, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b172e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0271731, 3 },
    { 0xdb1b1732, 3 },
    { 0xc6371733, 4 },
    { 0x4ed71734, 3 },
    { 0x91cb1733, 3 },
    { 0xdc771734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61371739, 3 },
    { 0xddab1739, 4 },
    { 0x910b173b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beb1742, 5 },
    { 0x6ee71742, 4 },
    { 0x50b1744, 4 },
    { 0x8ec31745, 4 },
    { 0, 0 },
    { 0xd0eb1747, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e7174e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd71751, 4 },
    { 0, 0 },
    { 0xae7b1753, 3 },
    { 0x3c331754, 3 },
    { 0x22031754, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc71759, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44f1760, 3 },
    { 0, 0 },
    { 0xa6df1762, 4 },
    { 0, 0 },
    { 0x494b1764, 3 },
    { 0xd7b31765, 3 },
    { 0, 0 },
    { 0x6c171767, 3 },
    { 0x910b1768, 3 },
    { 0xf5771769, 4 },
    { 0xbc1b176a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3843176e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff731771, 3 },
    { 0x49531772, 3 },
    { 0xf3eb1772, 3 },
    { 0x534b1774, 3 },
    { 0xa1e71775, 4 },
    { 0x97631776, 3 },
    { 0, 0 },
    { 0x11071778, 4 },
    { 0xc5b1779, 3 },
    { 0xce1b177a, 4 },
    { 0, 0 },
    { 0xf08b177c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e43177f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55871783, 3 },
    { 0, 0 },
    { 0xca4f1785, 3 },
    { 0, 0 },
    { 0x137b1787, 4 },
    { 0xbe531787, 3 },
    { 0, 0 },
    { 0x15eb178a, 3 },
    { 0, 0 },
    { 0x6ad3178c, 4 },
    { 0x28e7178d, 4 },
    { 0xd2b7178e, 3 },
    { 0, 0 },
    { 0x64731790, 3 },
    { 0x1b631791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f71794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55af1797, 3 },
    { 0, 0 },
    { 0xde6b1799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb213179c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9717a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58717ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62717b0, 3 },
    { 0x490f17b1, 3 },
    { 0x35c317b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8317b5, 3 },
    { 0, 0 },
    { 0x53a717b7, 3 },
    { 0, 0 },
    { 0xa69f17b9, 4 },
    { 0, 0 },
    { 0xac9f17bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf717c2, 3 },
    { 0x74af17c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849317c8, 3 },
    { 0xd88f17c9, 3 },
    { 0xc7a717c9, 3 },
    { 0xc3bb17cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05f17ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984b17d2, 3 },
    { 0xbef717d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19f17dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30717e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfef17e5, 3 },
    { 0xc7cf17e6, 3 },
    { 0x89fb17e5, 3 },
    { 0x2ff717e6, 3 },
    { 0xea2b17e9, 3 },
    { 0x6f7b17ea, 3 },
    { 0xd11317eb, 3 },
    { 0xca717ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf17ef, 3 },
    { 0x746317f0, 3 },
    { 0x4ab317f1, 4 },
    { 0xaf6b17ef, 3 },
    { 0, 0 },
    { 0xad7f17f4, 3 },
    { 0x3a6717f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f17fa, 3 },
    { 0, 0 },
    { 0xfe8b17fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48771800, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0571803, 4 },
    { 0, 0 },
    { 0xcdff1805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48471808, 3 },
    { 0x50131809, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bb180e, 3 },
    { 0x85cf180f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd3181b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657b181f, 3 },
    { 0x4be31820, 4 },
    { 0, 0 },
    { 0xc7c71822, 3 },
    { 0, 0 },
    { 0xe6131824, 3 },
    { 0xfba31825, 4 },
    { 0, 0 },
    { 0x7b031827, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c7182e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df1834, 5 },
    { 0, 0 },
    { 0xbbcf1836, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53171842, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b1845, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88f184e, 3 },
    { 0, 0 },
    { 0x346b1850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a431856, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7b185b, 3 },
    { 0xf00b185c, 4 },
    { 0xbc87185d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e31860, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab171863, 3 },
    { 0x7d731863, 5 },
    { 0xa7731863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7186a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf3186d, 3 },
    { 0x3d6f186d, 3 },
    { 0x2a13186d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69b1872, 4 },
    { 0xd03f1873, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4213187b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c3187f, 3 },
    { 0, 0 },
    { 0x54af1881, 4 },
    { 0x1bcb1882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b1889, 3 },
    { 0x1a27188a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b31892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9b1898, 3 },
    { 0x680b1898, 3 },
    { 0, 0 },
    { 0x7d77189b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669f189f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f18a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b718ad, 3 },
    { 0xcd2b18ae, 4 },
    { 0xef7b18ae, 3 },
    { 0x2d9f18b0, 3 },
    { 0x378318ad, 3 },
    { 0, 0 },
    { 0x7eb718b3, 3 },
    { 0x24e318b4, 4 },
    { 0xa27718b5, 4 },
    { 0x2d0718b6, 3 },
    { 0, 0 },
    { 0xc34718b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4f18bd, 3 },
    { 0, 0 },
    { 0xd6c318bf, 3 },
    { 0x38b318c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7b18c3, 3 },
    { 0x9ad718c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eef18c8, 3 },
    { 0x5dd718c9, 3 },
    { 0x669f18c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242b18ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7318d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a718da, 3 },
    { 0, 0 },
    { 0x882318dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb18e3, 3 },
    { 0xc1db18e3, 3 },
    { 0x642b18e4, 3 },
    { 0x804f18e6, 3 },
    { 0xec2b18e6, 3 },
    { 0xe2c718e4, 3 },
    { 0xfaaf18e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41b18f0, 3 },
    { 0x1d4318f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccb18f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f571900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bf1906, 3 },
    { 0x43631906, 3 },
    { 0x56731908, 3 },
    { 0x82131908, 3 },
    { 0x3a271908, 4 },
    { 0x4dab190b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781f190f, 3 },
    { 0x29431910, 3 },
    { 0xad67190f, 3 },
    { 0xc30f1912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bf1916, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cb191c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebf1922, 3 },
    { 0x9431923, 5 },
    { 0x2d971924, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba31929, 3 },
    { 0xa2b7192a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadb192e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26b1934, 4 },
    { 0x7def1935, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6f1938, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbf193e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd571944, 3 },
    { 0xaadb1945, 3 },
    { 0xf1b1944, 4 },
    { 0xdb5f1947, 3 },
    { 0xc1271946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ab194e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65f195a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa797195e, 3 },
    { 0x64ef195f, 3 },
    { 0x8cdb195f, 3 },
    { 0x79871961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50b1965, 3 },
    { 0x11771966, 5 },
    { 0x3c831967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b1972, 3 },
    { 0x87c71973, 4 },
    { 0x333b1974, 4 },
    { 0x64831974, 3 },
    { 0x58171975, 3 },
    { 0x98931974, 4 },
    { 0xd77b1978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549f197f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1471984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd71988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1b198d, 3 },
    { 0x517f198e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eb1992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cff1996, 3 },
    { 0x920f1997, 3 },
    { 0xa31f1996, 3 },
    { 0, 0 },
    { 0xd05f199a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897199d, 3 },
    { 0x26af199e, 3 },
    { 0x845b199e, 4 },
    { 0, 0 },
    { 0x526719a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab19a7, 4 },
    { 0x564719a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8319b2, 3 },
    { 0x97a319b3, 3 },
    { 0xf62f19b3, 4 },
    { 0x64c319b5, 3 },
    { 0xdf9319b4, 3 },
    { 0x3c5719b2, 4 },
    { 0x1a0319b7, 3 },
    { 0xeac319b9, 3 },
    { 0x68bb19ba, 3 },
    { 0xfcaf19b7, 4 },
    { 0x80b719ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b319bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5719c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f19c9, 4 },
    { 0xe95b19ca, 3 },
    { 0, 0 },
    { 0xf58f19cc, 3 },
    { 0, 0 },
    { 0xb42f19ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f19d2, 5 },
    { 0x1d4319d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef719d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3f19dd, 3 },
    { 0x30ef19dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f319e4, 4 },
    { 0, 0 },
    { 0xa91b19e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a719ea, 3 },
    { 0x91e719ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x707b19ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0f19f1, 4 },
    { 0x793319f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba719f9, 3 },
    { 0xc40b19fa, 4 },
    { 0xf20719fb, 3 },
    { 0x3c9f19fb, 3 },
    { 0x51319fb, 4 },
    { 0x8e6719fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cb1a02, 3 },
    { 0, 0 },
    { 0x479f1a04, 3 },
    { 0x17431a04, 3 },
    { 0xfd9b1a06, 4 },
    { 0x6f871a07, 4 },
    { 0, 0 },
    { 0xb1071a09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb31a0d, 3 },
    { 0, 0 },
    { 0x281f1a0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393f1a1c, 4 },
    { 0x63931a1d, 3 },
    { 0, 0 },
    { 0xe94b1a1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410f1a27, 3 },
    { 0x1831a28, 4 },
    { 0xdbdb1a29, 4 },
    { 0x506b1a28, 3 },
    { 0, 0 },
    { 0x6f4f1a2c, 3 },
    { 0xc92f1a2d, 3 },
    { 0x9f031a2d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48171a36, 4 },
    { 0xc33b1a36, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4531a3b, 3 },
    { 0, 0 },
    { 0xb9b31a3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x781f1a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52931a44, 3 },
    { 0x17ef1a45, 3 },
    { 0x27771a46, 3 },
    { 0x25bb1a47, 3 },
    { 0x537f1a47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192f1a4d, 4 },
    { 0, 0 },
    { 0x29af1a4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9071a56, 4 },
    { 0, 0 },
    { 0xc2431a58, 4 },
    { 0x83c31a58, 3 },
    { 0, 0 },
    { 0x82171a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94f1a61, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29f1a66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb1a6b, 3 },
    { 0xa2cf1a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b1a6f, 3 },
    { 0, 0 },
    { 0x51d71a71, 3 },
    { 0, 0 },
    { 0xa6d31a73, 3 },
    { 0, 0 },
    { 0xbb231a75, 4 },
    { 0x986f1a76, 3 },
    { 0x17171a76, 4 },
    { 0x85cb1a78, 3 },
    { 0, 0 },
    { 0x8b2b1a7a, 3 },
    { 0xd3131a7b, 3 },
    { 0xd7b71a7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e71a80, 4 },
    { 0xda031a81, 5 },
    { 0xb32b1a81, 3 },
    { 0, 0 },
    { 0x59371a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b271a87, 3 },
    { 0x61b31a87, 3 },
    { 0xd6db1a89, 3 },
    { 0xca8b1a8a, 3 },
    { 0xc7371a8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45b1a8f, 4 },
    { 0xbcf71a90, 4 },
    { 0x419f1a91, 4 },
    { 0x2b8f1a92, 3 },
    { 0, 0 },
    { 0xa6f1a94, 4 },
    { 0xc8af1a94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd31a99, 3 },
    { 0x499f1a9a, 3 },
    { 0, 0 },
    { 0x54571a9c, 3 },
    { 0, 0 },
    { 0x871b1a9e, 4 },
    { 0, 0 },
    { 0x9671aa0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba171aa5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75871aa8, 3 },
    { 0, 0 },
    { 0xbbcb1aaa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe531aad, 3 },
    { 0, 0 },
    { 0xaf731aaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58b1ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb831ab6, 3 },
    { 0x67571ab7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77f1aba, 3 },
    { 0, 0 },
    { 0xb32f1abc, 3 },
    { 0x808f1abd, 3 },
    { 0, 0 },
    { 0x3a771abf, 3 },
    { 0, 0 },
    { 0xf5771ac1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9231ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d731ac8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b571acb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216b1ad0, 3 },
    { 0xbd931ad1, 3 },
    { 0x2d3b1ad1, 3 },
    { 0xfda71ad3, 3 },
    { 0, 0 },
    { 0x52031ad5, 3 },
    { 0x64a71ad6, 4 },
    { 0xa0ff1ad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc71ada, 3 },
    { 0, 0 },
    { 0x351b1adc, 3 },
    { 0xf531add, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0b1ae3, 3 },
    { 0xe8df1ae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72b1ae8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f1aeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x980b1aee, 3 },
    { 0xf34b1aee, 5 },
    { 0, 0 },
    { 0xa93f1af1, 3 },
    { 0x84331af1, 4 },
    { 0x4a2f1af2, 3 },
    { 0, 0 },
    { 0x7c7f1af5, 3 },
    { 0xa80f1af5, 3 },
    { 0x75eb1af7, 3 },
    { 0x54131af7, 3 },
    { 0, 0 },
    { 0xd0c71afa, 3 },
    { 0xe5571afb, 3 },
    { 0, 0 },
    { 0xe46f1afd, 3 },
    { 0x1bf1afd, 3 },
    { 0x93271aff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d31b06, 3 },
    { 0x78271b06, 4 },
    { 0xe94b1b07, 4 },
    { 0x94331b06, 3 },
    { 0xd00b1b0a, 4 },
    { 0x1fbf1b0b, 3 },
    { 0xc3bf1b0c, 3 },
    { 0x2b731b0c, 3 },
    { 0x683f1b0e, 3 },
    { 0x11bb1b0c, 3 },
    { 0x75771b10, 3 },
    { 0xaeb31b10, 4 },
    { 0, 0 },
    { 0xe2f71b13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6131b1d, 3 },
    { 0x85c31b1e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef871b24, 3 },
    { 0x14d71b25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e31b28, 4 },
    { 0x96bf1b28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab31b2d, 3 },
    { 0, 0 },
    { 0x147f1b2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbb1b37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b71b3a, 3 },
    { 0xb24f1b3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb1b3e, 5 },
    { 0x8b7f1b3f, 5 },
    { 0xedef1b3f, 3 },
    { 0, 0 },
    { 0xf3631b42, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd31b4a, 3 },
    { 0, 0 },
    { 0x150b1b4c, 4 },
    { 0, 0 },
    { 0xb3271b4e, 3 },
    { 0xc33b1b4f, 3 },
    { 0x2ba71b50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa871b54, 3 },
    { 0xe0331b54, 4 },
    { 0x48d31b56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d31b5b, 4 },
    { 0xd46b1b5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96031b64, 3 },
    { 0x48b71b65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a71b6a, 4 },
    { 0xdbff1b6a, 4 },
    { 0xeadf1b6a, 3 },
    { 0, 0 },
    { 0x9e4f1b6e, 3 },
    { 0x939b1b6e, 3 },
    { 0, 0 },
    { 0xb1831b71, 3 },
    { 0, 0 },
    { 0x9ea71b73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x30031b76, 3 },
    { 0x74831b77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2771b7c, 3 },
    { 0xec471b7d, 3 },
    { 0xfc1b1b7e, 3 },
    { 0x8acb1b7f, 3 },
    { 0xb8ef1b80, 3 },
    { 0xe7ab1b80, 4 },
    { 0x7f871b82, 3 },
    { 0x54c71b7f, 3 },
    { 0, 0 },
    { 0xdfe31b85, 3 },
    { 0xca1b1b86, 4 },
    { 0xa2c71b87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10971b8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684b1b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98531b94, 3 },
    { 0, 0 },
    { 0xebe31b96, 4 },
    { 0x4bbf1b97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f1b9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ab1b9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1b1ba4, 4 },
    { 0, 0 },
    { 0xebef1ba6, 3 },
    { 0x8bb71ba7, 3 },
    { 0xb0d31ba7, 3 },
    { 0x571ba9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59671bb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c71bb8, 3 },
    { 0x1c3b1bb9, 3 },
    { 0x370f1bb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f1bbe, 4 },
    { 0x4b471bbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2f1bc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc71bc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177b1bcc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c71bd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8871bda, 3 },
    { 0x66231bdb, 4 },
    { 0x1c871bdb, 3 },
    { 0x3a431bdd, 3 },
    { 0x1d0b1bdd, 3 },
    { 0x84d31bdd, 4 },
    { 0x682b1be0, 3 },
    { 0x86db1be0, 3 },
    { 0x354f1bde, 4 },
    { 0x547b1be3, 3 },
    { 0xe85f1bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe531be8, 3 },
    { 0xe6c71be9, 3 },
    { 0x50871bea, 4 },
    { 0xeddb1be9, 4 },
    { 0xe3271be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d31bf0, 3 },
    { 0, 0 },
    { 0xd9ab1bf2, 3 },
    { 0, 0 },
    { 0x2fc71bf4, 4 },
    { 0, 0 },
    { 0x45271bf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e71bf9, 3 },
    { 0xc1631bf9, 5 },
    { 0x510f1bf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b31c03, 3 },
    { 0x68331c04, 3 },
    { 0x74771c04, 3 },
    { 0x8c331c06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f1c09, 4 },
    { 0x2ebb1c09, 3 },
    { 0x86ff1c0b, 4 },
    { 0, 0 },
    { 0x3caf1c0d, 3 },
    { 0, 0 },
    { 0x5ebf1c0f, 4 },
    { 0, 0 },
    { 0x8c2f1c11, 3 },
    { 0, 0 },
    { 0xbd171c13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2031c1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3971c21, 3 },
    { 0x83cf1c21, 5 },
    { 0x9cfb1c21, 3 },
    { 0x38b31c22, 3 },
    { 0x9aef1c25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e31c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91071c2d, 3 },
    { 0xfc4f1c2e, 3 },
    { 0, 0 },
    { 0xd5971c30, 3 },
    { 0x88cf1c31, 3 },
    { 0x26831c32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26471c35, 3 },
    { 0x75bb1c36, 3 },
    { 0x913b1c36, 3 },
    { 0xbd2f1c36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4f1c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bb1c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba71c4a, 3 },
    { 0x3c5b1c4b, 3 },
    { 0, 0 },
    { 0x9d431c4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db1c52, 3 },
    { 0, 0 },
    { 0x12eb1c54, 5 },
    { 0, 0 },
    { 0x2cbf1c56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84731c5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abf1c5f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f1c63, 3 },
    { 0x8a831c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60371c68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d971c6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb1c75, 3 },
    { 0x724b1c76, 3 },
    { 0xc76f1c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d431c7a, 3 },
    { 0x32a31c7a, 4 },
    { 0, 0 },
    { 0x4671c7d, 4 },
    { 0x3e371c7e, 4 },
    { 0, 0 },
    { 0x3acf1c80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49b1c84, 3 },
    { 0, 0 },
    { 0x732f1c86, 4 },
    { 0xf59f1c86, 4 },
    { 0x84d71c88, 3 },
    { 0x89171c88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b1c8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b1c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2671c97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f1c9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df71c9e, 3 },
    { 0x5b231c9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1f1ca3, 4 },
    { 0, 0 },
    { 0x3f431ca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b1ca8, 5 },
    { 0xf331ca9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68f1cac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf31caf, 3 },
    { 0x92bb1cb0, 3 },
    { 0x1f431cb0, 3 },
    { 0x8b231cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d31cb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bef1cbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebf1cbe, 4 },
    { 0x94071cbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca71cc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4771cc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15631cc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e731ccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f1ccf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bf1cd5, 3 },
    { 0x45c31cd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a531cd9, 3 },
    { 0x75131cda, 3 },
    { 0xd8071cdb, 4 },
    { 0, 0 },
    { 0xc64f1cdd, 3 },
    { 0xbf71cde, 3 },
    { 0x342f1cdd, 3 },
    { 0x881f1cdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6f1ce3, 3 },
    { 0x88371ce4, 5 },
    { 0x201f1ce3, 3 },
    { 0xef6b1ce6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6271cec, 3 },
    { 0x95631cec, 3 },
    { 0x948b1cec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8971cf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39271cfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2f1cff, 3 },
    { 0x91231cff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672f1d04, 3 },
    { 0xedbf1d04, 3 },
    { 0x3e131d05, 3 },
    { 0x94331d06, 3 },
    { 0x378f1d05, 3 },
    { 0, 0 },
    { 0xb7731d0a, 4 },
    { 0xdf771d0b, 3 },
    { 0, 0 },
    { 0x743f1d0d, 4 },
    { 0x74db1d0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4371d12, 3 },
    { 0xc14f1d13, 3 },
    { 0x80931d14, 3 },
    { 0x397f1d14, 3 },
    { 0x7bb31d15, 3 },
    { 0xcce31d17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1f1d1c, 3 },
    { 0x1dd71d1d, 3 },
    { 0x1e371d1e, 3 },
    { 0xf37f1d1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd31d27, 3 },
    { 0x505f1d28, 3 },
    { 0, 0 },
    { 0x1c131d2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54531d2e, 4 },
    { 0, 0 },
    { 0xac31d30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76671d36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f1d39, 3 },
    { 0, 0 },
    { 0x7bdb1d3b, 3 },
    { 0x4bb1d3b, 3 },
    { 0xd7db1d3d, 4 },
    { 0, 0 },
    { 0x8e831d3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bf1d43, 4 },
    { 0xe1371d43, 3 },
    { 0x918f1d45, 4 },
    { 0, 0 },
    { 0x998b1d47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77b1d4a, 3 },
    { 0, 0 },
    { 0x9fd31d4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1b1d51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1231d56, 3 },
    { 0x97b1d57, 3 },
    { 0x94131d58, 3 },
    { 0xb0ff1d58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eb1d5c, 4 },
    { 0, 0 },
    { 0xeaef1d5e, 3 },
    { 0, 0 },
    { 0xfc431d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86b1d74, 4 },
    { 0xdb31d75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42b1d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c31d7d, 4 },
    { 0, 0 },
    { 0x7231d7f, 3 },
    { 0, 0 },
    { 0xe5271d81, 3 },
    { 0, 0 },
    { 0x14971d83, 3 },
    { 0x90231d83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ff1d8b, 3 },
    { 0, 0 },
    { 0x5a831d8d, 3 },
    { 0xb8931d8e, 3 },
    { 0, 0 },
    { 0x63771d90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98031d95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e31d9e, 3 },
    { 0xe8f71d9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41231da6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x10371da9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5731dad, 3 },
    { 0, 0 },
    { 0xd56b1daf, 3 },
    { 0x9c771daf, 4 },
    { 0, 0 },
    { 0xe7031db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1f1db5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad31db9, 3 },
    { 0, 0 },
    { 0x81171dbb, 4 },
    { 0x91eb1dbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9f1dc4, 3 },
    { 0x52b1dc5, 3 },
    { 0xbe7b1dc6, 3 },
    { 0, 0 },
    { 0x597b1dc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f1dcd, 3 },
    { 0, 0 },
    { 0xcedb1dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de71dd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b1ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921f1de2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb1de7, 3 },
    { 0x67f1de8, 3 },
    { 0x664b1de8, 3 },
    { 0x9ecf1dea, 3 },
    { 0, 0 },
    { 0xb2e71dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb1def, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a931df3, 3 },
    { 0x7d631df4, 3 },
    { 0xb9831df4, 3 },
    { 0xc3871df6, 3 },
    { 0xce4f1df7, 3 },
    { 0x97bb1df6, 3 },
    { 0x1d031df9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8f1dfc, 3 },
    { 0xdd931dfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ff1e04, 3 },
    { 0x6f1b1e05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0971e08, 4 },
    { 0x6e671e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f1e11, 3 },
    { 0x73c31e12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87071e1d, 4 },
    { 0xa0db1e1e, 3 },
    { 0x995f1e1e, 4 },
    { 0xd2971e20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3431e27, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd71e2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83f1e31, 3 },
    { 0, 0 },
    { 0xd87f1e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c071e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ef1e44, 3 },
    { 0x571e45, 3 },
    { 0xece31e44, 3 },
    { 0x52c71e47, 3 },
    { 0x750f1e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x812f1e4b, 3 },
    { 0x38471e4b, 3 },
    { 0x17031e4b, 3 },
    { 0x3abf1e4d, 3 },
    { 0x53231e4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4f1e53, 4 },
    { 0, 0 },
    { 0xcb71e55, 3 },
    { 0xabfb1e55, 4 },
    { 0xfacb1e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b1e5c, 4 },
    { 0x19d71e5d, 3 },
    { 0xe18b1e5e, 4 },
    { 0x668b1e5f, 3 },
    { 0xc4f31e60, 3 },
    { 0xc64b1e61, 3 },
    { 0, 0 },
    { 0x4f431e63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14271e67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8531e6b, 3 },
    { 0x5aa31e6c, 4 },
    { 0xf2bf1e6d, 3 },
    { 0x67c71e6e, 4 },
    { 0x51671e6f, 3 },
    { 0, 0 },
    { 0x6b171e71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2371e75, 3 },
    { 0, 0 },
    { 0xc12f1e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c631e7c, 3 },
    { 0x9c371e7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f1e80, 4 },
    { 0x15c71e80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a431e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cf1e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf1e94, 3 },
    { 0x3f2b1e95, 3 },
    { 0x660f1e96, 4 },
    { 0, 0 },
    { 0xd7bb1e98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79f1e9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f31ea0, 3 },
    { 0xceab1ea1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29631ea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f1eb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf431ebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38531ec5, 3 },
    { 0, 0 },
    { 0xe92f1ec7, 3 },
    { 0xe8c31ec7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2031ecb, 3 },
    { 0x7a31ecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bb1ecf, 4 },
    { 0xdecb1ecf, 3 },
    { 0, 0 },
    { 0x7fe31ed2, 3 },
    { 0xee2b1ed3, 3 },
    { 0, 0 },
    { 0xfe0f1ed5, 3 },
    { 0x218f1ed5, 3 },
    { 0, 0 },
    { 0x176f1ed8, 3 },
    { 0x64231ed8, 4 },
    { 0x512f1eda, 4 },
    { 0x6e131edb, 3 },
    { 0x5b5b1edc, 4 },
    { 0x8f4f1eda, 3 },
    { 0x7ceb1edd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6571ee2, 4 },
    { 0x56531ee3, 5 },
    { 0x7be31ee3, 3 },
    { 0xe3771ee3, 4 },
    { 0xa4871ee2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67271ee9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577f1ef0, 3 },
    { 0, 0 },
    { 0x5aab1ef2, 3 },
    { 0, 0 },
    { 0x7f5b1ef4, 3 },
    { 0xbf871ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ef1ef9, 3 },
    { 0, 0 },
    { 0xc3731efb, 3 },
    { 0xa49f1efc, 3 },
    { 0x65ff1efd, 3 },
    { 0x595f1efe, 4 },
    { 0xf15b1eff, 3 },
    { 0xd21f1efb, 3 },
    { 0xc76b1f01, 4 },
    { 0x9bab1efe, 3 },
    { 0, 0 },
    { 0x59c31f04, 4 },
    { 0, 0 },
    { 0xa54f1f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96af1f0a, 4 },
    { 0xad671f0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf031f10, 3 },
    { 0x3b371f11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc531f17, 4 },
    { 0, 0 },
    { 0x5e771f19, 4 },
    { 0x42bb1f1a, 3 },
    { 0x93c31f1a, 3 },
    { 0xaedb1f1c, 4 },
    { 0, 0 },
    { 0x3ee31f1e, 3 },
    { 0xda4b1f1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01b1f22, 3 },
    { 0, 0 },
    { 0x190f1f24, 5 },
    { 0, 0 },
    { 0xd35f1f26, 3 },
    { 0x56871f26, 3 },
    { 0x66f31f26, 3 },
    { 0, 0 },
    { 0x70fb1f2a, 3 },
    { 0x361b1f2b, 3 },
    { 0, 0 },
    { 0xec5f1f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd471f39, 3 },
    { 0x4bb71f39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aef1f3f, 3 },
    { 0xa48f1f40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7b1f44, 4 },
    { 0xe6cb1f44, 3 },
    { 0xc9b31f44, 4 },
    { 0x31c31f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53571f4c, 3 },
    { 0xea131f4d, 3 },
    { 0x13ab1f4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b71f52, 3 },
    { 0x352f1f52, 3 },
    { 0x250b1f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b1f57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9071f5b, 3 },
    { 0x778b1f5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82571f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dff1f66, 3 },
    { 0x36b31f67, 4 },
    { 0xb5b71f67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0771f77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f31f7d, 3 },
    { 0, 0 },
    { 0x37731f7f, 3 },
    { 0, 0 },
    { 0x81271f81, 3 },
    { 0x5e5b1f82, 4 },
    { 0x8f871f83, 5 },
    { 0, 0 },
    { 0x5d31f85, 5 },
    { 0xeb7b1f86, 3 },
    { 0, 0 },
    { 0x2d631f88, 3 },
    { 0x1531f88, 3 },
    { 0xe3871f8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f231f95, 3 },
    { 0x5f71f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e71f99, 3 },
    { 0x137b1f99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b1f9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b171fa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc071fa5, 3 },
    { 0x47571fa5, 4 },
    { 0xe8331fa7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a31faa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0031fb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6671fba, 3 },
    { 0, 0 },
    { 0xc0ab1fbc, 3 },
    { 0, 0 },
    { 0xbf231fbe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419b1fc5, 3 },
    { 0x7bf71fc6, 3 },
    { 0, 0 },
    { 0xb30b1fc8, 3 },
    { 0, 0 },
    { 0x3f7b1fca, 3 },
    { 0xb2271fcb, 3 },
    { 0x6d571fcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cf1fcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55031fd3, 3 },
    { 0x23bf1fd4, 3 },
    { 0x65db1fd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbb1fdb, 3 },
    { 0, 0 },
    { 0x52ef1fdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b31fe0, 4 },
    { 0x70e31fe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b831fe7, 4 },
    { 0xc5971fe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64071fec, 3 },
    { 0, 0 },
    { 0x3c471fee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0771ff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f1ff9, 3 },
    { 0xfbc71ffa, 4 },
    { 0x8f71ffb, 3 },
    { 0x58a31ffc, 3 },
    { 0xafb31ffd, 4 },
    { 0x5cd31ffe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab2001, 5 },
    { 0, 0 },
    { 0xfd232003, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e32006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb2009, 3 },
    { 0, 0 },
    { 0x585f200b, 3 },
    { 0x2b67200c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9f2011, 3 },
    { 0, 0 },
    { 0x9b2013, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba32017, 3 },
    { 0xa3872018, 3 },
    { 0, 0 },
    { 0x42cf201a, 4 },
    { 0xd71f201b, 3 },
    { 0, 0 },
    { 0x341b201d, 4 },
    { 0xdf6b201e, 3 },
    { 0x14c7201f, 3 },
    { 0, 0 },
    { 0xdffb2021, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835f2026, 3 },
    { 0x15a32027, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4423202a, 3 },
    { 0xdbd7202b, 3 },
    { 0x2537202c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332030, 3 },
    { 0x128f2031, 4 },
    { 0x728f2032, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xadab2035, 3 },
    { 0x4f3f2035, 4 },
    { 0xfb572037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb703203a, 4 },
    { 0x65ef203b, 3 },
    { 0, 0 },
    { 0xf28b203d, 5 },
    { 0xa9b3203d, 3 },
    { 0x8e7f203f, 4 },
    { 0xf1672040, 3 },
    { 0xa67b203e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed072045, 3 },
    { 0, 0 },
    { 0x190f2047, 3 },
    { 0xe3f32047, 3 },
    { 0, 0 },
    { 0xdb1b204a, 3 },
    { 0, 0 },
    { 0xd5af204c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7204f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d32054, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x761f2057, 3 },
    { 0x68032057, 3 },
    { 0, 0 },
    { 0x8c9b205a, 4 },
    { 0x60eb205a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b32061, 3 },
    { 0, 0 },
    { 0x40eb2063, 4 },
    { 0xe73b2064, 4 },
    { 0xb9972063, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe557206f, 3 },
    { 0xb4d32070, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d872074, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3b2079, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d87207c, 4 },
    { 0x36ff207d, 4 },
    { 0xbe5f207e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8f2081, 3 },
    { 0x6d32082, 3 },
    { 0xa45b2083, 3 },
    { 0, 0 },
    { 0x23ff2085, 4 },
    { 0xa0d32085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef208c, 4 },
    { 0, 0 },
    { 0x6f208e, 3 },
    { 0x4d43208f, 3 },
    { 0, 0 },
    { 0xe6372091, 3 },
    { 0, 0 },
    { 0x7b632093, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78db2097, 3 },
    { 0, 0 },
    { 0x89a32099, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb209d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776b20a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91320a6, 3 },
    { 0, 0 },
    { 0x2d1720a8, 5 },
    { 0x260f20a9, 3 },
    { 0x220320a9, 3 },
    { 0x4ed320a9, 4 },
    { 0x35e720ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baf20b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b720ba, 3 },
    { 0x709720bb, 3 },
    { 0xf41b20bb, 3 },
    { 0xbddb20bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ff20c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0720c3, 3 },
    { 0, 0 },
    { 0x76c720c5, 4 },
    { 0, 0 },
    { 0x37f320c7, 3 },
    { 0x33e320c8, 4 },
    { 0, 0 },
    { 0x1ff320ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7320cd, 3 },
    { 0, 0 },
    { 0xb02320cf, 4 },
    { 0xb84f20d0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4320d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x700320d9, 3 },
    { 0x86df20d9, 3 },
    { 0xc1f720db, 3 },
    { 0xaa2f20dc, 4 },
    { 0, 0 },
    { 0xfacb20de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30720e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4720ef, 3 },
    { 0x153b20f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b320f7, 3 },
    { 0x9cd320f8, 3 },
    { 0x227320f8, 4 },
    { 0x514320fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x964b20fd, 3 },
    { 0xbdc320fd, 3 },
    { 0xed7720ff, 3 },
    { 0x342b20fd, 3 },
    { 0xe7172100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933f210c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d72110, 4 },
    { 0x684f2111, 3 },
    { 0, 0 },
    { 0x19e72113, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6b2116, 3 },
    { 0xa1e32116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd7211a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04b211e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b2122, 3 },
    { 0xdcb32123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b2127, 3 },
    { 0, 0 },
    { 0xe9b32129, 3 },
    { 0xc683212a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be32132, 3 },
    { 0, 0 },
    { 0x83072134, 3 },
    { 0xb79f2134, 4 },
    { 0xd1fb2135, 4 },
    { 0, 0 },
    { 0xc5232138, 4 },
    { 0, 0 },
    { 0xb1c7213a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66872142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e32145, 3 },
    { 0, 0 },
    { 0x886f2147, 3 },
    { 0x95132148, 3 },
    { 0x51032149, 3 },
    { 0xf883214a, 3 },
    { 0x64532147, 3 },
    { 0xe5272148, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32151, 4 },
    { 0x6c9b2152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15632155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab72158, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d7215c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6532162, 3 },
    { 0x43e72162, 3 },
    { 0x484b2164, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9b2170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db3217a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1887217f, 3 },
    { 0x8072180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2332185, 3 },
    { 0xd39f2186, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c93218c, 3 },
    { 0x705b218d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872191, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeab2197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08b219a, 3 },
    { 0xefc7219b, 3 },
    { 0xf16b219b, 3 },
    { 0x1bc7219d, 3 },
    { 0xa8ab219d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb21a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644b21a8, 3 },
    { 0x3e0721a9, 4 },
    { 0x627f21aa, 4 },
    { 0x8e6721ab, 3 },
    { 0xe8e321ab, 3 },
    { 0x835b21a9, 3 },
    { 0x5e4721ae, 3 },
    { 0xb9a321af, 3 },
    { 0x71f721ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6721b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1321ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69321bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f721c2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x559321c5, 3 },
    { 0x695321c6, 4 },
    { 0, 0 },
    { 0x562721c8, 3 },
    { 0xaf3721c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b21ce, 3 },
    { 0, 0 },
    { 0x381721d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabf21d4, 3 },
    { 0x1ad721d5, 3 },
    { 0x2a1f21d5, 3 },
    { 0xd45321d7, 3 },
    { 0xe5c721d4, 3 },
    { 0, 0 },
    { 0xa3721da, 3 },
    { 0xa38f21db, 3 },
    { 0xd55f21da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca721e0, 3 },
    { 0x1a4721e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254321ea, 4 },
    { 0, 0 },
    { 0xd2e721ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a721ef, 3 },
    { 0x2a5b21f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f21f5, 3 },
    { 0x8e7321f6, 3 },
    { 0, 0 },
    { 0x974721f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d721fc, 3 },
    { 0xcab721fd, 3 },
    { 0, 0 },
    { 0x244321ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37732202, 4 },
    { 0xf6bf2203, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4432208, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35df220e, 3 },
    { 0x40ff220f, 3 },
    { 0x546b2210, 3 },
    { 0x26b32211, 3 },
    { 0xfc6b2212, 3 },
    { 0x8eef2213, 3 },
    { 0x183b2211, 3 },
    { 0xfed72212, 3 },
    { 0xf97f2216, 5 },
    { 0x8f2b2212, 3 },
    { 0x42b72217, 4 },
    { 0, 0 },
    { 0x59b7221a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebb2220, 3 },
    { 0x2e7b2221, 3 },
    { 0xd5a32220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb2229, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1f222c, 3 },
    { 0x9073222d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ef2233, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f2238, 3 },
    { 0xd972238, 4 },
    { 0x88cb223a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef223d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4db2240, 3 },
    { 0xbc9f2241, 3 },
    { 0x54fb2240, 3 },
    { 0xe7cb2240, 3 },
    { 0, 0 },
    { 0x399f2245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3b2248, 4 },
    { 0x7aa32249, 3 },
    { 0x39af2249, 5 },
    { 0x1953224b, 3 },
    { 0, 0 },
    { 0xd5f3224d, 3 },
    { 0x6147224e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf2251, 3 },
    { 0x4e5b2252, 3 },
    { 0x52172253, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f2256, 3 },
    { 0x80b32257, 3 },
    { 0xac972257, 3 },
    { 0x1e232259, 3 },
    { 0xa4cf2259, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a72260, 3 },
    { 0xa5232261, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fb2266, 3 },
    { 0x7a472267, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c03226b, 3 },
    { 0x71af226c, 3 },
    { 0xf1bb226d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c172270, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50132278, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe113227c, 4 },
    { 0xf8f3227d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cf2284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7572287, 4 },
    { 0, 0 },
    { 0x8bf32289, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b43228d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91972297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e13229b, 3 },
    { 0xecf7229c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2f229f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db22a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449b22aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405f22af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b22b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9722bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cf22c0, 3 },
    { 0x358722c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae722c4, 3 },
    { 0xcc3722c5, 4 },
    { 0xb47b22c6, 3 },
    { 0xbc4f22c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345f22cb, 3 },
    { 0xab8b22cc, 3 },
    { 0, 0 },
    { 0x886722ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667b22d2, 4 },
    { 0x7cbb22d2, 4 },
    { 0x7e0f22d4, 3 },
    { 0x59f22d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46322d9, 3 },
    { 0x376722da, 3 },
    { 0x4f5b22db, 3 },
    { 0x895f22dc, 3 },
    { 0, 0 },
    { 0x31af22de, 3 },
    { 0, 0 },
    { 0x5e9f22e0, 3 },
    { 0xbdc322e1, 4 },
    { 0x29d322e1, 3 },
    { 0x342f22e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0322e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d322e9, 4 },
    { 0x1d9f22ea, 4 },
    { 0x8fff22eb, 3 },
    { 0x8fe322ea, 4 },
    { 0x59db22e9, 3 },
    { 0xb7c722eb, 3 },
    { 0, 0 },
    { 0x891722f0, 3 },
    { 0, 0 },
    { 0xb48722f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19722f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fb22fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73322fd, 3 },
    { 0xa89722fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb2302, 3 },
    { 0x54932302, 4 },
    { 0x545b2303, 3 },
    { 0xe3572302, 4 },
    { 0x95b72304, 4 },
    { 0, 0 },
    { 0x6a732308, 3 },
    { 0x91eb2309, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71932310, 3 },
    { 0x5dcb2311, 3 },
    { 0x4ec72312, 3 },
    { 0, 0 },
    { 0x8fb32314, 3 },
    { 0xe48f2315, 3 },
    { 0xd41f2315, 3 },
    { 0xc76b2315, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3f231e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266b2324, 3 },
    { 0x1fb2325, 3 },
    { 0x7d132326, 4 },
    { 0x1732327, 3 },
    { 0x6dcf2325, 3 },
    { 0xf6b32324, 4 },
    { 0x21f232a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e3232d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d772332, 3 },
    { 0, 0 },
    { 0x3aef2334, 3 },
    { 0x4bcb2334, 3 },
    { 0xb37b2336, 4 },
    { 0x13572335, 3 },
    { 0x3d772336, 3 },
    { 0x757f2339, 4 },
    { 0xeeb2338, 3 },
    { 0x77b7233b, 4 },
    { 0x69ab233c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc77233f, 3 },
    { 0xbeeb2340, 3 },
    { 0, 0 },
    { 0xc7532342, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64b2345, 3 },
    { 0, 0 },
    { 0x4cb2347, 4 },
    { 0x6c7b2348, 3 },
    { 0x2e32349, 4 },
    { 0x60b72348, 3 },
    { 0x708f234b, 3 },
    { 0xf91f234b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea872355, 3 },
    { 0x14db2356, 3 },
    { 0xfd132356, 3 },
    { 0, 0 },
    { 0xe58b2359, 3 },
    { 0x383f2359, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19b235d, 4 },
    { 0, 0 },
    { 0xd793235f, 3 },
    { 0x509b2360, 3 },
    { 0, 0 },
    { 0xf2c72362, 3 },
    { 0, 0 },
    { 0xbfff2364, 3 },
    { 0xfd2f2365, 3 },
    { 0x71cb2366, 4 },
    { 0x294f2367, 3 },
    { 0x4c972367, 3 },
    { 0x303b2369, 4 },
    { 0x4d77236a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323f2370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddf2376, 3 },
    { 0, 0 },
    { 0x1cb2378, 4 },
    { 0x19bf2378, 4 },
    { 0, 0 },
    { 0x8fb3237b, 3 },
    { 0x2847237b, 3 },
    { 0x12cb237d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5172384, 3 },
    { 0x91432384, 4 },
    { 0, 0 },
    { 0x9fcf2387, 4 },
    { 0, 0 },
    { 0x2b572389, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f27238f, 3 },
    { 0, 0 },
    { 0x24032391, 3 },
    { 0xa0072392, 3 },
    { 0x1072391, 4 },
    { 0x711f2393, 5 },
    { 0x6fd72394, 4 },
    { 0x2a872393, 3 },
    { 0, 0 },
    { 0xff4b2398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eff239d, 3 },
    { 0x88ab239e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f23a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f723a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bb23a9, 3 },
    { 0x29eb23aa, 3 },
    { 0xfa1723aa, 4 },
    { 0, 0 },
    { 0x566323ad, 3 },
    { 0x1dff23ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6723b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2723b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb23bb, 4 },
    { 0, 0 },
    { 0x4a0323bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c323c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24b23c5, 4 },
    { 0, 0 },
    { 0x1e3723c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02323cb, 3 },
    { 0x470f23cc, 4 },
    { 0, 0 },
    { 0xf9cb23ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203723d2, 3 },
    { 0xac0323d3, 3 },
    { 0, 0 },
    { 0x637b23d5, 3 },
    { 0x5bc723d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaf23d9, 3 },
    { 0x4fe723d9, 3 },
    { 0x325f23da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5723e0, 3 },
    { 0x7a3323e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x499b23e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b23e8, 3 },
    { 0x474723e8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81b23ed, 3 },
    { 0, 0 },
    { 0x4a3723ef, 4 },
    { 0xd3ab23f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x147f23f3, 3 },
    { 0xb71323f4, 3 },
    { 0, 0 },
    { 0x936723f6, 3 },
    { 0xb89f23f6, 4 },
    { 0x4f5323f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb23ff, 3 },
    { 0x5ceb23ff, 3 },
    { 0, 0 },
    { 0x80bf2402, 3 },
    { 0x60b2402, 4 },
    { 0x4b3b2404, 3 },
    { 0x53732405, 3 },
    { 0, 0 },
    { 0x47072407, 3 },
    { 0xb6a32407, 3 },
    { 0x830b2408, 3 },
    { 0xc3f240a, 3 },
    { 0x67ab240a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df240e, 4 },
    { 0x1c77240f, 3 },
    { 0x2fdb240f, 3 },
    { 0, 0 },
    { 0x81f72412, 3 },
    { 0, 0 },
    { 0x9ba72414, 3 },
    { 0xfed72414, 3 },
    { 0x57872416, 3 },
    { 0x73372416, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f07241c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98732423, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb232428, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb943242e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44f2435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x857f2438, 3 },
    { 0x21332438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634b2445, 4 },
    { 0x6cff2446, 4 },
    { 0x70f2446, 3 },
    { 0, 0 },
    { 0xb2449, 3 },
    { 0, 0 },
    { 0xc10b244b, 3 },
    { 0x96fb244c, 4 },
    { 0, 0 },
    { 0xb903244e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767b2455, 4 },
    { 0, 0 },
    { 0x883b2457, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1383245b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da32462, 3 },
    { 0x8672463, 3 },
    { 0xd5832464, 3 },
    { 0, 0 },
    { 0x71d72466, 4 },
    { 0xe76b2466, 3 },
    { 0xb9b72468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f246c, 3 },
    { 0x10b7246c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f2470, 5 },
    { 0xb8ef2471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9b2474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ef2478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509b2481, 4 },
    { 0, 0 },
    { 0x6cbf2483, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xede72486, 4 },
    { 0x40c32487, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359f248d, 3 },
    { 0x8eeb248e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1b2493, 3 },
    { 0xf0532494, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa733249c, 4 },
    { 0, 0 },
    { 0x28a7249e, 4 },
    { 0x9e1b249e, 4 },
    { 0, 0 },
    { 0xc59324a1, 3 },
    { 0x930b24a1, 4 },
    { 0xe6fb24a3, 3 },
    { 0x8f1b24a4, 3 },
    { 0, 0 },
    { 0x58c324a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e724a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe724ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65324b3, 3 },
    { 0, 0 },
    { 0x8cab24b5, 3 },
    { 0xadcf24b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ef24ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df24bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ff24c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eb24c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab24d1, 3 },
    { 0x3af724d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f24d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5b24d8, 3 },
    { 0, 0 },
    { 0x5a1f24da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1724de, 3 },
    { 0xa72324df, 3 },
    { 0x823f24e0, 3 },
    { 0, 0 },
    { 0x78e324e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b24e6, 4 },
    { 0x87d324e7, 5 },
    { 0x74b24e8, 3 },
    { 0xd6ef24e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8b24ed, 3 },
    { 0x940724ee, 4 },
    { 0x50a324ef, 3 },
    { 0xad8324ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0b24f5, 3 },
    { 0x1d8324f5, 4 },
    { 0, 0 },
    { 0xa58724f8, 3 },
    { 0, 0 },
    { 0x4dcf24fa, 3 },
    { 0x6c9724fb, 3 },
    { 0x7b2324fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b2502, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a72505, 4 },
    { 0, 0 },
    { 0x27ef2507, 3 },
    { 0xb5af2507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79db250b, 3 },
    { 0x903f250c, 3 },
    { 0, 0 },
    { 0x827b250e, 4 },
    { 0x9633250f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b72516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7251a, 4 },
    { 0xb66f251b, 4 },
    { 0, 0 },
    { 0x183f251d, 3 },
    { 0x2d47251e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22f2522, 3 },
    { 0, 0 },
    { 0x8f132524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6372527, 3 },
    { 0x26f32527, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f252b, 3 },
    { 0, 0 },
    { 0xe047252d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99272533, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e32539, 4 },
    { 0xe75b253a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2f253d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882f2541, 4 },
    { 0x43372542, 3 },
    { 0, 0 },
    { 0x9f2544, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b254a, 3 },
    { 0x2e3254b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdb2553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51af2559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2033255c, 3 },
    { 0x74df255c, 3 },
    { 0x129b255c, 3 },
    { 0xf21f255f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x790b2562, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81772566, 3 },
    { 0, 0 },
    { 0xbcab2568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb256d, 3 },
    { 0x8c67256e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd272575, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b257b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c7257e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe72583, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a32588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f17258c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db258f, 3 },
    { 0x9adb258f, 3 },
    { 0xbfd72591, 3 },
    { 0xbb232592, 3 },
    { 0x885f2593, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e72597, 3 },
    { 0x748f2598, 4 },
    { 0x24b2598, 3 },
    { 0xb2b7259a, 4 },
    { 0xff23259b, 3 },
    { 0x9823259b, 3 },
    { 0x2ec3259d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624f25a1, 3 },
    { 0xc5e725a2, 3 },
    { 0x41cf25a3, 3 },
    { 0, 0 },
    { 0xa0ab25a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c325a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ef25af, 3 },
    { 0, 0 },
    { 0x99a325b1, 3 },
    { 0x10325b2, 3 },
    { 0x435725b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x495f25b6, 4 },
    { 0, 0 },
    { 0xf80b25b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9325bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65725c0, 3 },
    { 0x7ed725c0, 3 },
    { 0x81d325c0, 3 },
    { 0x60cb25c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867b25c9, 4 },
    { 0x8acb25ca, 3 },
    { 0x6d8725cb, 3 },
    { 0x6c6725ca, 3 },
    { 0x174725cd, 3 },
    { 0x114b25cc, 3 },
    { 0x7fe725cf, 3 },
    { 0x691b25ca, 3 },
    { 0xdc3b25cb, 3 },
    { 0xe99b25d2, 3 },
    { 0, 0 },
    { 0xd06f25d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8325d7, 3 },
    { 0, 0 },
    { 0xa87b25d9, 3 },
    { 0xfc325d9, 4 },
    { 0, 0 },
    { 0x9be325dc, 4 },
    { 0xdaf725dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d725e6, 3 },
    { 0, 0 },
    { 0xbbe325e8, 3 },
    { 0xc3bb25e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9b25ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56b25f2, 3 },
    { 0x821f25f3, 3 },
    { 0xf61325f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db25f8, 4 },
    { 0, 0 },
    { 0xac5325fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f25ff, 4 },
    { 0x68672600, 3 },
    { 0x4f372601, 5 },
    { 0xd95b2602, 3 },
    { 0xa7832602, 3 },
    { 0xf8af25ff, 3 },
    { 0x25c72605, 3 },
    { 0xe9f72606, 3 },
    { 0x2f0f2606, 3 },
    { 0x24072608, 4 },
    { 0x38472600, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed87260c, 3 },
    { 0xc8ef260d, 4 },
    { 0x66ef260e, 3 },
    { 0xde43260f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b72612, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69b2619, 3 },
    { 0, 0 },
    { 0xcab261b, 4 },
    { 0x6cbb261c, 3 },
    { 0xfe6f261b, 3 },
    { 0x439f261e, 3 },
    { 0x5767261e, 3 },
    { 0x2f32620, 4 },
    { 0x7637261c, 4 },
    { 0x71232622, 4 },
    { 0, 0 },
    { 0xa7532624, 3 },
    { 0x6d32624, 3 },
    { 0, 0 },
    { 0xf9df2627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f47262b, 3 },
    { 0x625f262c, 4 },
    { 0x794b262d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d32633, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f2637, 3 },
    { 0xa7732637, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b263c, 3 },
    { 0x2567263d, 3 },
    { 0x9f73263d, 4 },
    { 0xe1ab263d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56332642, 3 },
    { 0x56a32643, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff72651, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7f2657, 3 },
    { 0xb0e32658, 3 },
    { 0x6d9f2659, 3 },
    { 0xe0632659, 3 },
    { 0xffdb2657, 3 },
    { 0, 0 },
    { 0x5537265d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69f2662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b72668, 4 },
    { 0xa7772669, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef266d, 3 },
    { 0, 0 },
    { 0x787266f, 3 },
    { 0x2a72670, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x915b2673, 3 },
    { 0xcab32673, 3 },
    { 0xdb532675, 4 },
    { 0x87772674, 3 },
    { 0, 0 },
    { 0x8fd72678, 3 },
    { 0, 0 },
    { 0xb0f3267a, 3 },
    { 0x8183267b, 5 },
    { 0x1d67267b, 4 },
    { 0x9d2b267a, 3 },
    { 0, 0 },
    { 0xab53267f, 3 },
    { 0x3cb72680, 3 },
    { 0x7b2b267f, 3 },
    { 0xccdf2682, 3 },
    { 0, 0 },
    { 0x335b2684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf2687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9b2691, 3 },
    { 0, 0 },
    { 0x754f2693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf269a, 3 },
    { 0x494f269b, 4 },
    { 0, 0 },
    { 0xd913269d, 3 },
    { 0x560b269d, 3 },
    { 0, 0 },
    { 0x7e4f26a0, 3 },
    { 0x40c326a1, 3 },
    { 0xa1df26a1, 3 },
    { 0x355726a3, 4 },
    { 0x5d4326a2, 4 },
    { 0, 0 },
    { 0x6dc726a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab26b4, 4 },
    { 0x4e7326b5, 3 },
    { 0xf68b26b6, 3 },
    { 0x956326b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e326c0, 3 },
    { 0x3cf26c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd326c4, 3 },
    { 0, 0 },
    { 0x5fb726c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443f26ca, 3 },
    { 0x94b26cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3b26d4, 4 },
    { 0xa1df26d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b26d8, 3 },
    { 0x8c9726d9, 3 },
    { 0, 0 },
    { 0x137f26db, 3 },
    { 0x342f26dc, 3 },
    { 0xf41f26dd, 3 },
    { 0, 0 },
    { 0x194726df, 3 },
    { 0xc91f26df, 3 },
    { 0x63c726e1, 3 },
    { 0x197726e2, 3 },
    { 0xdb0726e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a326e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f26e9, 4 },
    { 0x4cb26ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254f26f0, 4 },
    { 0, 0 },
    { 0xa40326f2, 3 },
    { 0x265b26f2, 3 },
    { 0xe6f26f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f26f7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f26fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26972703, 3 },
    { 0, 0 },
    { 0x155f2705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f03270a, 3 },
    { 0xee2b270b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fb270e, 3 },
    { 0x58db270f, 5 },
    { 0x8ddf270f, 4 },
    { 0x50d72711, 4 },
    { 0x5db32712, 4 },
    { 0x4bb2712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3763271c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3b2723, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabf2726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ab272a, 3 },
    { 0, 0 },
    { 0xf837272c, 3 },
    { 0x869b272c, 4 },
    { 0x1bc7272e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9b2732, 4 },
    { 0x6d932732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5f2737, 3 },
    { 0, 0 },
    { 0xfb2f2739, 3 },
    { 0, 0 },
    { 0x287b273b, 4 },
    { 0xbf97273c, 3 },
    { 0x5dcb273c, 3 },
    { 0xeec7273e, 4 },
    { 0xffcf273f, 3 },
    { 0x55472740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f2744, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9472749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b3274c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda372753, 3 },
    { 0x1b4b2754, 3 },
    { 0x53bb2755, 3 },
    { 0x4e732755, 4 },
    { 0x68732757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf33275a, 4 },
    { 0x2cc7275b, 4 },
    { 0, 0 },
    { 0xaf7275d, 3 },
    { 0x29ff275e, 3 },
    { 0, 0 },
    { 0x6ecb2760, 4 },
    { 0x1f372761, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd72764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a97276b, 3 },
    { 0x66b7276c, 3 },
    { 0x8a37276b, 3 },
    { 0, 0 },
    { 0x7f53276f, 3 },
    { 0xaa5f276f, 3 },
    { 0xda03276f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b2774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2b277f, 3 },
    { 0x4777277f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b72783, 3 },
    { 0x9cc32784, 3 },
    { 0, 0 },
    { 0xb7eb2786, 3 },
    { 0x93c32787, 3 },
    { 0xd36f2787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f278f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677f2794, 3 },
    { 0x880f2795, 3 },
    { 0x857f2796, 3 },
    { 0xab172797, 3 },
    { 0xa50b2797, 4 },
    { 0xd2ef2799, 3 },
    { 0xe7032799, 4 },
    { 0xdea72795, 3 },
    { 0x5087279a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b279f, 3 },
    { 0xf5d327a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570327a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d327ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88327b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcff27bc, 3 },
    { 0xa16f27bd, 4 },
    { 0x2eb27bd, 3 },
    { 0x938f27bf, 3 },
    { 0x848b27bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2327c6, 3 },
    { 0x644b27c7, 3 },
    { 0, 0 },
    { 0x868327c9, 3 },
    { 0xb5a327ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf27cd, 3 },
    { 0, 0 },
    { 0x4ceb27cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e727d3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e327d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef27db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93f27e0, 3 },
    { 0xb64b27e1, 4 },
    { 0x7d1f27e0, 4 },
    { 0x3b3b27e0, 3 },
    { 0, 0 },
    { 0xd02327e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cb27e8, 3 },
    { 0x1c3f27e9, 3 },
    { 0x5f0b27e9, 3 },
    { 0x709b27eb, 3 },
    { 0, 0 },
    { 0x750727ed, 3 },
    { 0x428327ee, 4 },
    { 0xb20b27ed, 3 },
    { 0, 0 },
    { 0x4dd327f1, 3 },
    { 0, 0 },
    { 0x305b27f3, 4 },
    { 0, 0 },
    { 0x762327f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94327f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8727ff, 3 },
    { 0x6c2f2800, 3 },
    { 0x8872801, 3 },
    { 0, 0 },
    { 0x45bb2803, 3 },
    { 0, 0 },
    { 0x90972805, 3 },
    { 0xb1df2805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b280a, 3 },
    { 0, 0 },
    { 0x94b7280c, 3 },
    { 0x81a3280d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b2810, 3 },
    { 0x9db32811, 3 },
    { 0, 0 },
    { 0x472f2813, 3 },
    { 0xf13f2814, 4 },
    { 0xe7bb2814, 3 },
    { 0xe4e32816, 3 },
    { 0x2ac72816, 3 },
    { 0x1f6f2816, 3 },
    { 0x6c5f2814, 3 },
    { 0x371b281a, 3 },
    { 0x2933281b, 3 },
    { 0, 0 },
    { 0xc80b281d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7372821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76872825, 3 },
    { 0, 0 },
    { 0xaee72827, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f282a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c572833, 3 },
    { 0xecdf2833, 3 },
    { 0, 0 },
    { 0xb3472836, 3 },
    { 0, 0 },
    { 0xbd9f2838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3283c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29f2845, 3 },
    { 0xb47b2845, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9193284a, 3 },
    { 0x468b284a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b2850, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eb2853, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d232856, 3 },
    { 0x2df2857, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed3285d, 3 },
    { 0, 0 },
    { 0xcaab285f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618f2863, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa32867, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ef286a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5f286d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4232871, 4 },
    { 0xc3772872, 3 },
    { 0x236f2872, 3 },
    { 0, 0 },
    { 0x6c432875, 5 },
    { 0x30172876, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb2885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32f2889, 3 },
    { 0, 0 },
    { 0x766f288b, 3 },
    { 0x326f288b, 4 },
    { 0xe003288d, 3 },
    { 0x87df288e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98df2893, 4 },
    { 0x39572893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12832898, 3 },
    { 0x67e32899, 4 },
    { 0xc567289a, 3 },
    { 0, 0 },
    { 0x4eaf289c, 3 },
    { 0x9aeb289d, 3 },
    { 0, 0 },
    { 0x2317289f, 3 },
    { 0, 0 },
    { 0x72ff28a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b28a5, 3 },
    { 0x78e728a5, 3 },
    { 0x23df28a7, 3 },
    { 0x5ac728a8, 3 },
    { 0x40eb28a9, 4 },
    { 0, 0 },
    { 0x609728ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x959328ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x333f28b1, 4 },
    { 0x3cab28b1, 3 },
    { 0, 0 },
    { 0x728728b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430728ba, 3 },
    { 0x49728bb, 3 },
    { 0xf0df28bb, 3 },
    { 0x850328bd, 3 },
    { 0x9e6728be, 4 },
    { 0xb05328be, 3 },
    { 0xe50b28c0, 3 },
    { 0, 0 },
    { 0xfc9328c2, 3 },
    { 0, 0 },
    { 0xc37f28c4, 4 },
    { 0x9328c4, 3 },
    { 0x9f5f28c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798328cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x738728ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f328d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22af28da, 3 },
    { 0x257b28da, 4 },
    { 0, 0 },
    { 0x29ab28dd, 3 },
    { 0xa04328de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4f28e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90f28e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525728f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35b28fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194f2904, 3 },
    { 0, 0 },
    { 0x5b32906, 4 },
    { 0, 0 },
    { 0xa10f2908, 3 },
    { 0xcbff2909, 3 },
    { 0x21932908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d7290d, 4 },
    { 0xbf47290e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cf2913, 3 },
    { 0xf4b72913, 3 },
    { 0, 0 },
    { 0xa7932916, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735b291a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x636f291d, 3 },
    { 0x7bf291e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bf2922, 4 },
    { 0, 0 },
    { 0x33a32924, 3 },
    { 0xe18f2925, 4 },
    { 0, 0 },
    { 0x8f272927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b292a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77872931, 3 },
    { 0, 0 },
    { 0x73432933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0f2938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f3293e, 3 },
    { 0xffbf293e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7f2942, 3 },
    { 0xaf9f2943, 3 },
    { 0x85872943, 3 },
    { 0x7e0f2945, 3 },
    { 0x6dcf2942, 3 },
    { 0x95ff2946, 3 },
    { 0xb72946, 3 },
    { 0xd99b2948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7294d, 4 },
    { 0xc9b3294e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x339f2951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51132956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798b295b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7637295e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b832962, 3 },
    { 0x616f2963, 4 },
    { 0xd92f2964, 3 },
    { 0x710f2963, 3 },
    { 0xa5eb2963, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bb296e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef2971, 3 },
    { 0x66232972, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed32979, 5 },
    { 0x4a03297a, 3 },
    { 0xc3f7297b, 3 },
    { 0xdef297c, 3 },
    { 0x19eb297d, 3 },
    { 0xcc57297e, 4 },
    { 0, 0 },
    { 0x29db2980, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75272986, 3 },
    { 0x721f2986, 3 },
    { 0xe6c72988, 3 },
    { 0xac8f2986, 3 },
    { 0x2d03298a, 4 },
    { 0x73e72989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f2991, 3 },
    { 0x9f7f2991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56f2995, 4 },
    { 0x80a32996, 4 },
    { 0x5f2b2996, 3 },
    { 0, 0 },
    { 0x4a6f2999, 3 },
    { 0xb49f299a, 3 },
    { 0, 0 },
    { 0x7d83299c, 3 },
    { 0, 0 },
    { 0xc33299e, 3 },
    { 0xce07299f, 3 },
    { 0x8a2729a0, 4 },
    { 0x3c3b29a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1f29a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b29b2, 4 },
    { 0xce2b29b3, 4 },
    { 0, 0 },
    { 0x1f6f29b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699f29b9, 3 },
    { 0x431729ba, 3 },
    { 0x48cb29ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6f29c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10329c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37729ca, 3 },
    { 0xb22f29ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83329ce, 4 },
    { 0, 0 },
    { 0xf329d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db29d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b29da, 3 },
    { 0x111f29da, 4 },
    { 0xebf29db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4329e1, 3 },
    { 0x719f29e2, 4 },
    { 0x5ab729e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4729e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x152729e9, 3 },
    { 0xae2f29e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2729f1, 3 },
    { 0x730729f2, 4 },
    { 0x27f29f3, 3 },
    { 0xae9b29f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4729f8, 4 },
    { 0, 0 },
    { 0x860f29fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33b29fd, 3 },
    { 0x17fb29fe, 3 },
    { 0, 0 },
    { 0x9d72a00, 3 },
    { 0x80272a01, 4 },
    { 0, 0 },
    { 0x92f32a03, 3 },
    { 0xb7cf2a04, 3 },
    { 0x531b2a05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9232a0a, 3 },
    { 0, 0 },
    { 0xc97f2a0c, 3 },
    { 0, 0 },
    { 0x32472a0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24f2a12, 4 },
    { 0x98b32a13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef2a16, 3 },
    { 0xd4b32a17, 4 },
    { 0xe3f32a16, 3 },
    { 0xa9332a16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7f2a1c, 4 },
    { 0xb2ab2a1c, 5 },
    { 0xde232a1c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61f2a22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf532a25, 3 },
    { 0, 0 },
    { 0x4e332a27, 3 },
    { 0xe9172a28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf2a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faf2a2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d032a32, 3 },
    { 0x29bb2a33, 3 },
    { 0xd15b2a34, 3 },
    { 0x395b2a33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b32a39, 3 },
    { 0x48cb2a39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c372a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f2a4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c532a4f, 3 },
    { 0xed0f2a4f, 4 },
    { 0xfc8f2a51, 5 },
    { 0x85f32a4f, 5 },
    { 0x39732a53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf32a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc32a5c, 4 },
    { 0x3a272a5d, 3 },
    { 0x4fbf2a5e, 3 },
    { 0, 0 },
    { 0xd8e32a60, 5 },
    { 0, 0 },
    { 0x30e72a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b232a69, 4 },
    { 0x58b72a69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf2a71, 4 },
    { 0xfb2f2a71, 3 },
    { 0xb06b2a73, 3 },
    { 0xbbb72a72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1f2a77, 3 },
    { 0xa75f2a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b2a7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce32a7e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7b2a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e72a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8972a87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295b2a8e, 3 },
    { 0x4b3f2a8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d72a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff632a98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e32a9c, 4 },
    { 0x7f7b2a9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f2aa0, 3 },
    { 0, 0 },
    { 0x78532aa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f2aa6, 4 },
    { 0xa5c32aa6, 3 },
    { 0x7b832aa6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48b2ab0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5f2ab3, 4 },
    { 0x30bf2ab4, 3 },
    { 0xc4af2ab3, 4 },
    { 0x33432ab6, 3 },
    { 0xd9ef2ab7, 3 },
    { 0, 0 },
    { 0x38332ab9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1532abc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a72ac0, 3 },
    { 0, 0 },
    { 0xc8b2ac2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa72ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19072acc, 4 },
    { 0xb72f2acd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80372ad0, 3 },
    { 0, 0 },
    { 0x8dcf2ad2, 4 },
    { 0x38272ad3, 3 },
    { 0, 0 },
    { 0x37f32ad5, 3 },
    { 0, 0 },
    { 0x6d432ad7, 4 },
    { 0, 0 },
    { 0xe8bb2ad9, 5 },
    { 0, 0 },
    { 0xf9c72adb, 4 },
    { 0x98232adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc72ae5, 3 },
    { 0xc2df2ae6, 4 },
    { 0xbda72ae7, 3 },
    { 0x2d2b2ae6, 5 },
    { 0x919f2ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b2aee, 3 },
    { 0xb15f2aee, 4 },
    { 0xd1eb2af0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e32af9, 3 },
    { 0x649b2af9, 3 },
    { 0x93572afb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be32b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519b2b04, 3 },
    { 0x18cf2b05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0432b08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5f2b0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c72b0f, 4 },
    { 0xa72b2b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2132b15, 3 },
    { 0x39e72b16, 4 },
    { 0x6f2b17, 4 },
    { 0, 0 },
    { 0x9ccb2b19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf2b1c, 3 },
    { 0, 0 },
    { 0x7e4f2b1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53b2b21, 3 },
    { 0xa2b2b22, 3 },
    { 0xe29f2b23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf2b26, 3 },
    { 0x582b2b27, 4 },
    { 0x62fb2b28, 3 },
    { 0xce872b29, 3 },
    { 0x92172b26, 3 },
    { 0x7e732b2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6b2b31, 3 },
    { 0x786b2b32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af2b35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f72b3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f072b40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6b2b44, 3 },
    { 0x757f2b45, 3 },
    { 0x6db2b46, 4 },
    { 0x68872b47, 3 },
    { 0x5d432b47, 3 },
    { 0x38472b44, 4 },
    { 0x30472b4a, 3 },
    { 0xa98b2b4b, 3 },
    { 0x938b2b4b, 3 },
    { 0x911b2b46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97072b52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f2b56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810f2b5f, 4 },
    { 0x2f772b60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71472b65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cf2b70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e72b73, 4 },
    { 0x6ee32b74, 3 },
    { 0x4e6b2b75, 3 },
    { 0x4a232b76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b2b79, 3 },
    { 0, 0 },
    { 0x16172b7b, 4 },
    { 0x4a7b2b7b, 3 },
    { 0xf9d72b7b, 3 },
    { 0, 0 },
    { 0xf7332b7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ab2b82, 4 },
    { 0xd8772b82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab2b8a, 4 },
    { 0, 0 },
    { 0x4b932b8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2b92, 5 },
    { 0, 0 },
    { 0x3a432b94, 3 },
    { 0xc1d32b95, 4 },
    { 0xf0832b95, 4 },
    { 0, 0 },
    { 0x9e7f2b98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bf2ba0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe272ba6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58f2ba9, 3 },
    { 0x9d632baa, 3 },
    { 0x8eff2bab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fb2baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e932bb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f2bb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd72bbc, 3 },
    { 0x85872bbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bb2bc4, 5 },
    { 0x6b1b2bc5, 3 },
    { 0xed532bc6, 4 },
    { 0x929f2bc7, 3 },
    { 0x3ebb2bc7, 3 },
    { 0xac0f2bc7, 3 },
    { 0xa7d32bc8, 4 },
    { 0x2bdf2bcb, 3 },
    { 0xc1cf2bc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50072bcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f2bd3, 3 },
    { 0xe6832bd4, 3 },
    { 0, 0 },
    { 0x69bf2bd6, 5 },
    { 0x9c232bd7, 3 },
    { 0, 0 },
    { 0x2fdf2bd9, 4 },
    { 0x7dc72bda, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f32bdd, 4 },
    { 0, 0 },
    { 0x29632bdf, 4 },
    { 0xd6632be0, 3 },
    { 0x54432be1, 4 },
    { 0x40032be0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914f2be8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f2bef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb32bf2, 3 },
    { 0xc8932bf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4f2bf7, 3 },
    { 0xb2d32bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e72bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea72c00, 5 },
    { 0, 0 },
    { 0x52732c02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f772c08, 3 },
    { 0x6f5f2c09, 3 },
    { 0, 0 },
    { 0x34cb2c0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84332c0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6672c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2f2c19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46b2c1e, 3 },
    { 0, 0 },
    { 0xdd9f2c20, 3 },
    { 0xe72c21, 3 },
    { 0xfe2f2c20, 3 },
    { 0x6f5f2c23, 3 },
    { 0x110b2c24, 3 },
    { 0x22472c25, 4 },
    { 0, 0 },
    { 0x4fab2c27, 3 },
    { 0xe52f2c28, 4 },
    { 0x8f3b2c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d72c2c, 3 },
    { 0x43832c2d, 3 },
    { 0xd73f2c2c, 4 },
    { 0x3b6b2c2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c972c35, 4 },
    { 0x57c32c36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e472c39, 3 },
    { 0x6d3b2c39, 4 },
    { 0x3a172c3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd72c3f, 3 },
    { 0x220f2c40, 3 },
    { 0x84532c40, 3 },
    { 0x67e72c42, 4 },
    { 0x6fdb2c43, 3 },
    { 0x460b2c3f, 4 },
    { 0x58632c45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d32c48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54f2c4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80272c52, 3 },
    { 0xcf6b2c53, 4 },
    { 0, 0 },
    { 0xf7b72c55, 3 },
    { 0x2a5f2c56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efb2c59, 3 },
    { 0x9b3f2c5a, 3 },
    { 0x857b2c5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6f2c6a, 4 },
    { 0, 0 },
    { 0xe54f2c6c, 3 },
    { 0, 0 },
    { 0xa30b2c6e, 3 },
    { 0xaa772c6f, 3 },
    { 0xaeaf2c70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf32c75, 4 },
    { 0, 0 },
    { 0x4be72c77, 3 },
    { 0x81772c77, 3 },
    { 0x5d4f2c79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f72c7e, 3 },
    { 0, 0 },
    { 0x67eb2c80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bb2c8b, 4 },
    { 0x25c32c8c, 4 },
    { 0x326f2c8d, 3 },
    { 0x17972c8e, 3 },
    { 0x8c572c8f, 3 },
    { 0, 0 },
    { 0x3c3f2c91, 3 },
    { 0x3e6f2c92, 3 },
    { 0x2aaf2c91, 3 },
    { 0x8872c93, 3 },
    { 0xb4532c94, 3 },
    { 0, 0 },
    { 0xfb572c97, 3 },
    { 0x7832c98, 5 },
    { 0x788b2c98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52af2c9f, 3 },
    { 0, 0 },
    { 0x710f2ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772caa, 3 },
    { 0, 0 },
    { 0xcaa32cac, 4 },
    { 0, 0 },
    { 0xcd732cae, 4 },
    { 0xa35b2caf, 3 },
    { 0x5e6b2cae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3072cb3, 3 },
    { 0, 0 },
    { 0xd55f2cb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20b2cbe, 3 },
    { 0, 0 },
    { 0xe10b2cc0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45f2cc8, 3 },
    { 0x604f2cc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce972cd3, 3 },
    { 0x3d472cd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af2cd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de32ce2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d472ce5, 3 },
    { 0xaf9f2ce5, 4 },
    { 0x18872ce7, 3 },
    { 0x3c3b2ce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72332cec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726f2cf1, 3 },
    { 0x8a0f2cf2, 3 },
    { 0, 0 },
    { 0x825b2cf4, 3 },
    { 0xc2c32cf5, 3 },
    { 0x89a72cf6, 3 },
    { 0x1a572cf5, 3 },
    { 0x2f2b2cf8, 3 },
    { 0x6f232cf9, 3 },
    { 0x732f2cfa, 3 },
    { 0x1932cf9, 3 },
    { 0, 0 },
    { 0xd7fb2cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793b2d01, 5 },
    { 0, 0 },
    { 0xd2a72d03, 3 },
    { 0, 0 },
    { 0xf2eb2d05, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8632d0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf2d10, 3 },
    { 0x30832d11, 3 },
    { 0x19172d12, 4 },
    { 0, 0 },
    { 0xae632d14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61972d1c, 3 },
    { 0x39a32d1c, 3 },
    { 0, 0 },
    { 0xe1032d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f72d23, 3 },
    { 0xa8eb2d23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cf2d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e32d2e, 3 },
    { 0xc3ef2d2f, 3 },
    { 0x38df2d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c32d37, 4 },
    { 0, 0 },
    { 0xe6832d39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffb2d42, 3 },
    { 0xdc2f2d43, 3 },
    { 0xdcbb2d44, 3 },
    { 0x5b0b2d42, 3 },
    { 0x9e3b2d44, 3 },
    { 0x7e0b2d42, 3 },
    { 0, 0 },
    { 0x42b2d49, 3 },
    { 0xd1332d49, 3 },
    { 0xc7472d49, 4 },
    { 0, 0 },
    { 0x2d932d4d, 3 },
    { 0, 0 },
    { 0xb4f2d4f, 3 },
    { 0x891b2d50, 4 },
    { 0, 0 },
    { 0xa43b2d52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50732d58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c32d5f, 3 },
    { 0x5eb72d60, 3 },
    { 0x74732d61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b32d64, 4 },
    { 0, 0 },
    { 0x988b2d66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b72d6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaef2d73, 3 },
    { 0xe6ab2d74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd32d7c, 3 },
    { 0, 0 },
    { 0x594b2d7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2132d81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81932d89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249b2d8d, 3 },
    { 0xefe32d8d, 3 },
    { 0x36d32d8f, 3 },
    { 0x329b2d90, 3 },
    { 0xcdff2d91, 3 },
    { 0, 0 },
    { 0x624b2d93, 4 },
    { 0x389f2d94, 5 },
    { 0x35f72d95, 3 },
    { 0xe1432d96, 3 },
    { 0x635b2d97, 3 },
    { 0xc0832d98, 3 },
    { 0xfba32d99, 4 },
    { 0x7a772d99, 4 },
    { 0xa4632d9b, 3 },
    { 0xebeb2d9c, 4 },
    { 0, 0 },
    { 0xe6eb2d9e, 3 },
    { 0x5c4b2d9f, 3 },
    { 0x1b7f2d9f, 3 },
    { 0, 0 },
    { 0xc4732da2, 4 },
    { 0, 0 },
    { 0x9aab2da4, 4 },
    { 0xd2d72da4, 3 },
    { 0, 0 },
    { 0xd38f2da7, 3 },
    { 0x5b6b2da8, 3 },
    { 0, 0 },
    { 0x753f2daa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f772dad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a72db0, 4 },
    { 0, 0 },
    { 0x9af32db2, 3 },
    { 0x8fff2db3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7232db9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0b2dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b2dc5, 3 },
    { 0, 0 },
    { 0xa0432dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bf2dd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a672dd9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b2ddd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd632de8, 3 },
    { 0x63cb2de9, 3 },
    { 0xe8232dea, 3 },
    { 0xb1832deb, 3 },
    { 0xd5d72deb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197b2df1, 3 },
    { 0xec772df2, 4 },
    { 0xf1972df1, 3 },
    { 0x35c72df3, 3 },
    { 0x37f32df5, 3 },
    { 0xfe832df5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adb2df9, 3 },
    { 0, 0 },
    { 0xba1f2dfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac32e00, 3 },
    { 0x697b2e00, 3 },
    { 0x9fef2e00, 4 },
    { 0, 0 },
    { 0x31332e04, 3 },
    { 0x790b2e05, 4 },
    { 0x10c72e06, 3 },
    { 0x63572e07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb2e0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f72e15, 4 },
    { 0x6cc32e16, 4 },
    { 0x2bb72e17, 3 },
    { 0x896f2e16, 5 },
    { 0x213f2e17, 3 },
    { 0xc7272e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b32e1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b2e21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04f2e26, 3 },
    { 0xa5e72e26, 3 },
    { 0, 0 },
    { 0x910f2e29, 3 },
    { 0x4f2e2a, 3 },
    { 0, 0 },
    { 0x65032e2c, 3 },
    { 0xa69b2e2c, 3 },
    { 0x35b2e2c, 3 },
    { 0xa0bb2e2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca32e32, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64732e38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfb2e3d, 4 },
    { 0, 0 },
    { 0x674f2e3f, 3 },
    { 0x7e772e40, 4 },
    { 0x4832e3f, 3 },
    { 0, 0 },
    { 0x190b2e43, 3 },
    { 0xe8fb2e44, 3 },
    { 0, 0 },
    { 0xf7032e46, 4 },
    { 0, 0 },
    { 0xee072e48, 4 },
    { 0x11332e49, 3 },
    { 0xabff2e48, 4 },
    { 0, 0 },
    { 0xe9c32e4c, 3 },
    { 0, 0 },
    { 0x78432e4e, 4 },
    { 0x2c7f2e4e, 4 },
    { 0xf5bb2e50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce72e56, 3 },
    { 0, 0 },
    { 0x532e58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc172e5c, 3 },
    { 0x15ab2e5d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76432e62, 3 },
    { 0xbc3f2e63, 3 },
    { 0, 0 },
    { 0xefcf2e65, 3 },
    { 0, 0 },
    { 0x93df2e67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08f2e6c, 3 },
    { 0xcf2f2e6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41f2e71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b2e74, 3 },
    { 0, 0 },
    { 0x5b172e76, 3 },
    { 0x3dcb2e77, 3 },
    { 0xec7f2e77, 3 },
    { 0xd832e76, 3 },
    { 0xd5732e7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd32e7d, 3 },
    { 0x3b132e7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf972e85, 3 },
    { 0xa2472e85, 4 },
    { 0, 0 },
    { 0x3a272e88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cb2e8b, 3 },
    { 0x3f372e8b, 3 },
    { 0, 0 },
    { 0xb3872e8e, 3 },
    { 0x464f2e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ab2e93, 3 },
    { 0xeae72e93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f2e97, 3 },
    { 0x19c32e98, 4 },
    { 0x4fbb2e97, 3 },
    { 0, 0 },
    { 0x2dcb2e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f2e9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb32ea1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x955f2ea4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bf2ead, 4 },
    { 0x8c8f2ead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab332eb3, 3 },
    { 0xd8cf2eb4, 3 },
    { 0x217f2eb5, 3 },
    { 0xb05b2eb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad32eba, 3 },
    { 0, 0 },
    { 0x93072ebc, 3 },
    { 0x93f2ebc, 3 },
    { 0xff272ebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a672ec3, 3 },
    { 0xf9132ec4, 3 },
    { 0xb6972ec5, 3 },
    { 0xbcc32ec4, 3 },
    { 0x10f2ec7, 3 },
    { 0, 0 },
    { 0x6d7b2ec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc72ecc, 3 },
    { 0, 0 },
    { 0x8ea32ece, 3 },
    { 0x5972ece, 3 },
    { 0xd3f72ed0, 3 },
    { 0xadab2ed1, 3 },
    { 0x29b72ed2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce932ed8, 3 },
    { 0x9fb32ed9, 3 },
    { 0x50032ed9, 3 },
    { 0xf4932ed9, 4 },
    { 0x951f2edc, 3 },
    { 0x9aff2ed9, 3 },
    { 0x8fbb2ede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef2ee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f032ee9, 3 },
    { 0x8e672ee9, 4 },
    { 0x6b872eeb, 3 },
    { 0xeebb2eec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73532ef1, 3 },
    { 0, 0 },
    { 0xbf3b2ef3, 3 },
    { 0, 0 },
    { 0x1eff2ef5, 4 },
    { 0x4332ef5, 3 },
    { 0, 0 },
    { 0x70232ef8, 3 },
    { 0xb2072ef8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ab2efc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeb2f00, 3 },
    { 0, 0 },
    { 0xf7432f02, 3 },
    { 0, 0 },
    { 0xba432f04, 3 },
    { 0xafef2f05, 3 },
    { 0x31e32f06, 4 },
    { 0xf1eb2f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x276f2f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f2f0e, 3 },
    { 0x626b2f0e, 4 },
    { 0x3c2b2f10, 3 },
    { 0x47a32f11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971b2f16, 3 },
    { 0xae5b2f17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b2f1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9f2f25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6872f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1972f34, 3 },
    { 0x6932f35, 4 },
    { 0x719b2f35, 4 },
    { 0xf73f2f35, 4 },
    { 0x69ab2f38, 4 },
    { 0xfea32f39, 3 },
    { 0xc54f2f37, 4 },
    { 0x4fff2f3b, 3 },
    { 0xfed32f3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df32f45, 4 },
    { 0x6cf72f45, 3 },
    { 0x85af2f47, 4 },
    { 0xfa72f48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791b2f4e, 3 },
    { 0xf9db2f4e, 3 },
    { 0, 0 },
    { 0x15e32f51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bf2f56, 4 },
    { 0x2e072f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e532f5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eb2f67, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf2f6b, 3 },
    { 0, 0 },
    { 0x811f2f6d, 4 },
    { 0, 0 },
    { 0xff4b2f6f, 3 },
    { 0x41f72f6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5232f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9b2f86, 3 },
    { 0x4eaf2f87, 4 },
    { 0x97332f88, 3 },
    { 0x788f2f88, 4 },
    { 0xc3e72f87, 3 },
    { 0, 0 },
    { 0x54b2f8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd932f8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc72f93, 4 },
    { 0xb1ab2f93, 3 },
    { 0x24272f95, 3 },
    { 0x24572f96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2b2f9b, 3 },
    { 0x9d572f9b, 3 },
    { 0x7bcb2f9d, 3 },
    { 0x530b2f9e, 4 },
    { 0x575f2f9f, 3 },
    { 0xebef2f9b, 3 },
    { 0xf13b2fa1, 3 },
    { 0x15472f9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961b2fa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eb2fad, 3 },
    { 0xcbbb2fad, 3 },
    { 0xf5cb2faf, 4 },
    { 0x1e232fae, 4 },
    { 0x60432fb1, 3 },
    { 0x732b2fae, 3 },
    { 0, 0 },
    { 0x48272fb4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea72fc0, 3 },
    { 0x52bf2fc1, 3 },
    { 0x248b2fc2, 3 },
    { 0, 0 },
    { 0xe67f2fc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13532fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf72fca, 3 },
    { 0xec0f2fcb, 3 },
    { 0, 0 },
    { 0x79ff2fcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db2fd4, 3 },
    { 0, 0 },
    { 0xac4f2fd6, 4 },
    { 0xd66b2fd7, 3 },
    { 0x769f2fd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2fdc, 3 },
    { 0, 0 },
    { 0xe4ff2fde, 3 },
    { 0, 0 },
    { 0xa2372fe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2f2fe4, 3 },
    { 0xfee72fe5, 3 },
    { 0x9d372fe5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34972fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84af2ff3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c72ff7, 3 },
    { 0, 0 },
    { 0x1d272ff9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38f2ffc, 3 },
    { 0x43b72ffc, 3 },
    { 0x26d72ffe, 4 },
    { 0x3ff32ffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b973009, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a7300d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9833012, 3 },
    { 0xdf573013, 4 },
    { 0xd7133014, 3 },
    { 0xf8fb3015, 3 },
    { 0x852f3016, 3 },
    { 0, 0 },
    { 0x56233018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7163301c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f3021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9f3028, 4 },
    { 0x7af33028, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b3036, 4 },
    { 0x5bd33037, 3 },
    { 0, 0 },
    { 0xcd473039, 3 },
    { 0x4763303a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b303d, 3 },
    { 0x289b303d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3333041, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26773044, 3 },
    { 0xc3173044, 3 },
    { 0x7cbf3046, 3 },
    { 0, 0 },
    { 0xbab33048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58eb304d, 3 },
    { 0, 0 },
    { 0x133b304f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c73053, 4 },
    { 0x25133054, 4 },
    { 0x179b3055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4513305b, 3 },
    { 0x4027305c, 3 },
    { 0, 0 },
    { 0x22cb305e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cb3064, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aab306a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x117f306d, 3 },
    { 0x8f67306e, 3 },
    { 0x17d7306d, 3 },
    { 0x293f3070, 3 },
    { 0xda4f3071, 3 },
    { 0xf5f73071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f3077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c33307c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a733087, 3 },
    { 0x12273088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab308b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5077308e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce83309b, 3 },
    { 0x6d9f309c, 4 },
    { 0x8cb7309b, 3 },
    { 0xc72f309b, 4 },
    { 0x1a8b309e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8f30a6, 4 },
    { 0x67b330a7, 3 },
    { 0, 0 },
    { 0xaa4f30a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x843730ac, 4 },
    { 0xddeb30ad, 4 },
    { 0x671b30ad, 3 },
    { 0, 0 },
    { 0xde1b30b0, 3 },
    { 0xe19730b1, 5 },
    { 0, 0 },
    { 0x7b0330b3, 3 },
    { 0, 0 },
    { 0x5ac330b5, 3 },
    { 0x24c330b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db30ba, 3 },
    { 0x3a5b30ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8730c1, 3 },
    { 0x4f7330c1, 3 },
    { 0x16ff30c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80330c7, 3 },
    { 0, 0 },
    { 0x88f730c9, 3 },
    { 0, 0 },
    { 0x791330cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x307730ce, 3 },
    { 0, 0 },
    { 0x473b30d0, 4 },
    { 0x3b8f30d0, 3 },
    { 0xacff30d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41b30d5, 3 },
    { 0, 0 },
    { 0x445730d7, 3 },
    { 0xc4c730d8, 3 },
    { 0, 0 },
    { 0xae5b30da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65730e0, 3 },
    { 0x42f330e0, 3 },
    { 0, 0 },
    { 0x14f30e3, 3 },
    { 0x5ce330e4, 3 },
    { 0xdc6f30e4, 3 },
    { 0x45b330e4, 3 },
    { 0, 0 },
    { 0x6c9730e8, 3 },
    { 0x3db730e8, 3 },
    { 0x308330ea, 4 },
    { 0x163330eb, 3 },
    { 0x53f730e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a330f7, 3 },
    { 0x772f30f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f30fe, 5 },
    { 0x262330ff, 4 },
    { 0, 0 },
    { 0x65b3101, 3 },
    { 0xb6d73102, 3 },
    { 0xf0673103, 3 },
    { 0, 0 },
    { 0xa3333105, 5 },
    { 0, 0 },
    { 0xa47b3107, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b573111, 3 },
    { 0x95073111, 4 },
    { 0xde6b3113, 4 },
    { 0, 0 },
    { 0x8e3b3115, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b311a, 5 },
    { 0, 0 },
    { 0x21e7311c, 3 },
    { 0, 0 },
    { 0xe113311e, 4 },
    { 0x6f37311f, 3 },
    { 0, 0 },
    { 0x299b3121, 3 },
    { 0x9a73122, 4 },
    { 0x37c73123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a433126, 3 },
    { 0xefbf3127, 3 },
    { 0x411b3127, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669b312f, 3 },
    { 0, 0 },
    { 0xf9a33131, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47f3139, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcb313c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b3140, 4 },
    { 0xe533141, 3 },
    { 0x1baf3142, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb33145, 3 },
    { 0xfb273146, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3314b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be3314e, 3 },
    { 0x5ddb314f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5533156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af315d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4f3161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab473166, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f73169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02f316c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d73171, 4 },
    { 0x97a33171, 3 },
    { 0x45533173, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe33176, 3 },
    { 0xaef33177, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33317c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6af3180, 3 },
    { 0x5bd33180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f318a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342b318e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333191, 3 },
    { 0xa02f3192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5373199, 3 },
    { 0xad93319a, 4 },
    { 0xba97319a, 4 },
    { 0, 0 },
    { 0x8163319d, 3 },
    { 0, 0 },
    { 0x65b7319f, 3 },
    { 0xeec331a0, 3 },
    { 0x2c1b319f, 3 },
    { 0xa21731a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b331a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6731ad, 3 },
    { 0, 0 },
    { 0xef1731af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0331b2, 4 },
    { 0x3a4f31b3, 3 },
    { 0, 0 },
    { 0x83df31b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1331b8, 3 },
    { 0xa18331b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a331bc, 3 },
    { 0x8faf31bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x984731c0, 3 },
    { 0, 0 },
    { 0x184f31c2, 3 },
    { 0, 0 },
    { 0x7e4f31c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x315f31c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1331cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30b31cf, 3 },
    { 0x340331d0, 3 },
    { 0, 0 },
    { 0x5dcf31d2, 4 },
    { 0, 0 },
    { 0xace331d4, 3 },
    { 0xd4cb31d4, 3 },
    { 0, 0 },
    { 0x3dcb31d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad731db, 3 },
    { 0x100f31db, 3 },
    { 0, 0 },
    { 0xb39f31de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65f31e3, 3 },
    { 0x6fab31e3, 3 },
    { 0x2e7b31e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eab31e8, 4 },
    { 0xe14331e9, 3 },
    { 0, 0 },
    { 0x218731eb, 3 },
    { 0, 0 },
    { 0x41bb31ed, 3 },
    { 0x958b31ee, 3 },
    { 0xa2db31ef, 3 },
    { 0x40ef31f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb731f4, 4 },
    { 0x4f8b31f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69f31f9, 3 },
    { 0xd4d731fa, 4 },
    { 0, 0 },
    { 0xe3c731fc, 3 },
    { 0xb3a331fc, 3 },
    { 0x57b731fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc73201, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641f3205, 3 },
    { 0xd52f3205, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x876f3209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7320f, 3 },
    { 0, 0 },
    { 0xeed73211, 3 },
    { 0xc9ff3212, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ef321a, 4 },
    { 0x3e7321b, 3 },
    { 0x5f0f321a, 3 },
    { 0x5c5b321b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cb3223, 3 },
    { 0x9b9f3224, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf3227, 3 },
    { 0x10833228, 3 },
    { 0x3db3229, 3 },
    { 0xe0e73228, 4 },
    { 0x49eb322a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb33230, 4 },
    { 0xba6b3231, 3 },
    { 0, 0 },
    { 0x7f33233, 3 },
    { 0x7dbb3234, 3 },
    { 0, 0 },
    { 0xcbbb3236, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9b323d, 3 },
    { 0xe0ef323e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f33244, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff33248, 4 },
    { 0x981f3248, 3 },
    { 0x6f833249, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b3324e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db33251, 5 },
    { 0x223b3251, 3 },
    { 0, 0 },
    { 0x29233254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83325b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab3263, 3 },
    { 0, 0 },
    { 0x93e73265, 3 },
    { 0x90bb3266, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde33269, 3 },
    { 0, 0 },
    { 0xc617326b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebf326e, 3 },
    { 0, 0 },
    { 0xb3cb3270, 3 },
    { 0, 0 },
    { 0xcf1b3272, 3 },
    { 0x273f3273, 3 },
    { 0x19df3274, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b3277, 4 },
    { 0, 0 },
    { 0x58c33279, 4 },
    { 0x9c5b327a, 3 },
    { 0, 0 },
    { 0x708f327c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b473281, 4 },
    { 0x804b3282, 3 },
    { 0xb2eb3281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e3328f, 3 },
    { 0, 0 },
    { 0xa0ab3291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60f3297, 3 },
    { 0x4a8b3297, 3 },
    { 0xef9b3297, 3 },
    { 0, 0 },
    { 0x6d7f329b, 3 },
    { 0, 0 },
    { 0xd06f329d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9df32a5, 3 },
    { 0x9b9332a5, 3 },
    { 0, 0 },
    { 0x7f0732a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcf32b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89b32b5, 4 },
    { 0x309b32b5, 3 },
    { 0x535732b5, 3 },
    { 0x731f32b8, 3 },
    { 0xba9f32b8, 3 },
    { 0x40d732b7, 4 },
    { 0x32bb32ba, 3 },
    { 0x4d6b32b5, 4 },
    { 0x6f6f32bd, 3 },
    { 0x386b32be, 3 },
    { 0x7c8332bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633732c5, 3 },
    { 0x18a732c6, 4 },
    { 0xa0732c7, 3 },
    { 0x726332c8, 4 },
    { 0xa26f32c9, 3 },
    { 0x306732c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb732cf, 4 },
    { 0xc5d732d0, 3 },
    { 0x7bc732d1, 3 },
    { 0, 0 },
    { 0x46d332d3, 3 },
    { 0, 0 },
    { 0xdab732d5, 3 },
    { 0x1a5f32d6, 5 },
    { 0x281f32d6, 3 },
    { 0x2f7732d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af32dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35732e3, 4 },
    { 0x9f6332e4, 3 },
    { 0x9dbf32e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961f32ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b32f3, 4 },
    { 0, 0 },
    { 0x77a332f5, 3 },
    { 0xc43b32f6, 3 },
    { 0xe4332f5, 4 },
    { 0xf90b32f8, 5 },
    { 0xd1df32f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287b3301, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcb3307, 3 },
    { 0x971b3308, 3 },
    { 0x5fcf3308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73233311, 3 },
    { 0xd9a73312, 4 },
    { 0x26073311, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b73316, 3 },
    { 0xc8ff3316, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5f331a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1233322, 4 },
    { 0x15533323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2537332c, 3 },
    { 0, 0 },
    { 0xc2c7332e, 3 },
    { 0x6adf332f, 3 },
    { 0, 0 },
    { 0xcbff3331, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1833334, 5 },
    { 0, 0 },
    { 0xf61f3336, 3 },
    { 0, 0 },
    { 0xb83b3338, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde033340, 3 },
    { 0xd86b3341, 3 },
    { 0x81073342, 4 },
    { 0x830f3343, 3 },
    { 0x9ca33341, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbb3347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811b334b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f334e, 3 },
    { 0x3df3334f, 3 },
    { 0, 0 },
    { 0x68773351, 4 },
    { 0, 0 },
    { 0xc16b3353, 3 },
    { 0x73bf3354, 3 },
    { 0xaad73355, 3 },
    { 0, 0 },
    { 0x1d233357, 4 },
    { 0, 0 },
    { 0x85533359, 3 },
    { 0xbeeb335a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ef335d, 3 },
    { 0, 0 },
    { 0xe3ab335f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ab3362, 3 },
    { 0x88eb3363, 3 },
    { 0x8073362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b33368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcb336b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f336e, 3 },
    { 0x2eaf336f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467b3375, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0b337b, 4 },
    { 0xd343337c, 4 },
    { 0x5ab3337c, 4 },
    { 0xb853337c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b733383, 3 },
    { 0x76b73383, 4 },
    { 0, 0 },
    { 0x4f033386, 3 },
    { 0xd6033386, 3 },
    { 0x7be33388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96f338d, 3 },
    { 0xa497338d, 3 },
    { 0, 0 },
    { 0x7b6b3390, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bff3393, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faf3396, 3 },
    { 0xa02f3397, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ff339f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b33a3, 3 },
    { 0x7acb33a3, 3 },
    { 0, 0 },
    { 0x88333a6, 3 },
    { 0xd5bb33a7, 4 },
    { 0x611333a7, 3 },
    { 0x2cdb33a9, 3 },
    { 0, 0 },
    { 0xfe4333ab, 3 },
    { 0x814333ab, 4 },
    { 0xd8f733ad, 3 },
    { 0xbd4f33ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cf33b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82333ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb33bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61b33c2, 3 },
    { 0x88df33c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fb33c6, 4 },
    { 0xa46b33c6, 3 },
    { 0xe1df33c6, 3 },
    { 0x3f1333c9, 3 },
    { 0, 0 },
    { 0xe37b33cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e733d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d733d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x326333d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2333db, 4 },
    { 0xf5e733dc, 5 },
    { 0x8aff33dc, 3 },
    { 0x3eb733de, 3 },
    { 0x78e333de, 4 },
    { 0x385733e0, 4 },
    { 0xea1333e1, 4 },
    { 0xaefb33e2, 3 },
    { 0x9a4733db, 3 },
    { 0x7e5733de, 4 },
    { 0xdef733db, 5 },
    { 0xf3bb33e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6733ea, 4 },
    { 0xad5333eb, 4 },
    { 0x6d7f33eb, 3 },
    { 0xe68b33ec, 3 },
    { 0xbc9b33ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65b33f1, 3 },
    { 0, 0 },
    { 0x72c733f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd733fc, 3 },
    { 0xf52f33fd, 3 },
    { 0x8c1b33fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf3408, 3 },
    { 0xcc973409, 3 },
    { 0x3c77340a, 3 },
    { 0x768b340a, 3 },
    { 0x2387340b, 4 },
    { 0, 0 },
    { 0xd28b340e, 3 },
    { 0x484b340e, 4 },
    { 0, 0 },
    { 0x5be33411, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146b3415, 3 },
    { 0x107f3415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6f3428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581b3431, 4 },
    { 0, 0 },
    { 0xc26f3433, 3 },
    { 0x7fbf3434, 3 },
    { 0x11ab3435, 3 },
    { 0x88b3435, 3 },
    { 0x6cbb3437, 4 },
    { 0x7f233438, 3 },
    { 0x212b3437, 3 },
    { 0x73af3435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b343e, 3 },
    { 0x833343f, 5 },
    { 0, 0 },
    { 0x53ff3441, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d673445, 3 },
    { 0x924f3446, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03344d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fb3451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b3458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67345c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe973461, 3 },
    { 0, 0 },
    { 0x12273463, 3 },
    { 0, 0 },
    { 0xbc6f3465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8b3468, 3 },
    { 0, 0 },
    { 0x5d1f346a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb346f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e73479, 3 },
    { 0x6bbb347a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283347f, 3 },
    { 0xb1ab3480, 3 },
    { 0xbd8f3481, 4 },
    { 0x467b3480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77b3485, 3 },
    { 0, 0 },
    { 0x2fa73487, 3 },
    { 0xc8f33487, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bf348d, 4 },
    { 0xad87348d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97233491, 4 },
    { 0x49db3492, 5 },
    { 0xc52f3493, 3 },
    { 0xbeab3494, 4 },
    { 0x80273493, 3 },
    { 0x64733496, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5733499, 3 },
    { 0, 0 },
    { 0x181b349b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3f349e, 4 },
    { 0xf39b349f, 5 },
    { 0x957f34a0, 4 },
    { 0x3f2f34a1, 4 },
    { 0, 0 },
    { 0xcd7f34a3, 3 },
    { 0xbd0334a4, 4 },
    { 0, 0 },
    { 0x7f5b34a6, 3 },
    { 0x3ddf34a6, 4 },
    { 0xd65734a7, 3 },
    { 0x5b1f34a8, 3 },
    { 0xb9ab34a8, 4 },
    { 0xee7b34ab, 3 },
    { 0, 0 },
    { 0x2d1b34ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91b34b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58734bc, 3 },
    { 0x518334bd, 4 },
    { 0, 0 },
    { 0x204f34bf, 4 },
    { 0xc69734c0, 3 },
    { 0x3c334c0, 3 },
    { 0, 0 },
    { 0xcaf34c3, 3 },
    { 0xe87b34c4, 3 },
    { 0, 0 },
    { 0xada734c6, 3 },
    { 0xfe4b34c7, 3 },
    { 0, 0 },
    { 0xa3d334c9, 4 },
    { 0x397734c9, 3 },
    { 0xb03734cb, 3 },
    { 0x5e3f34cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a734cf, 3 },
    { 0xaddf34cf, 5 },
    { 0xee7734d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205f34d7, 4 },
    { 0x494f34d7, 4 },
    { 0x931b34d8, 3 },
    { 0x10af34da, 3 },
    { 0x82ef34d7, 3 },
    { 0x244f34dc, 4 },
    { 0xbacf34d8, 3 },
    { 0x763334da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523734e4, 3 },
    { 0, 0 },
    { 0xa81734e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932334ea, 3 },
    { 0x6cc734eb, 3 },
    { 0x7ecf34ea, 4 },
    { 0, 0 },
    { 0xa7b734ee, 4 },
    { 0x28cb34ef, 3 },
    { 0xae9334ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7f34f3, 3 },
    { 0x56bb34f4, 3 },
    { 0xa68734f5, 3 },
    { 0x9d9334f6, 3 },
    { 0x253334f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x576734fa, 3 },
    { 0, 0 },
    { 0xf8e734fc, 3 },
    { 0x37b34fd, 3 },
    { 0x904734fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b933502, 3 },
    { 0xccdf3503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3f3506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d3350a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf3514, 4 },
    { 0xe2bb3514, 3 },
    { 0x65033516, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d47351c, 3 },
    { 0xf24f351c, 4 },
    { 0xaa87351e, 3 },
    { 0xd0ef351f, 3 },
    { 0x1657351c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44f3523, 3 },
    { 0, 0 },
    { 0xc4133525, 3 },
    { 0xcda33526, 3 },
    { 0x12333526, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14f352a, 4 },
    { 0xd6ef352b, 3 },
    { 0x58eb352c, 3 },
    { 0x851f352d, 3 },
    { 0x8a3352e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c133533, 4 },
    { 0x422b3534, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b7353a, 4 },
    { 0, 0 },
    { 0xf67353c, 3 },
    { 0, 0 },
    { 0x5de3353e, 3 },
    { 0, 0 },
    { 0xc6833540, 4 },
    { 0x2a7b3541, 4 },
    { 0, 0 },
    { 0xfd373543, 4 },
    { 0xf7073544, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f73549, 3 },
    { 0, 0 },
    { 0xb65f354b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x124f354e, 3 },
    { 0x32b7354e, 4 },
    { 0x811f3550, 3 },
    { 0, 0 },
    { 0x20d73552, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c33555, 3 },
    { 0x28233556, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b3560, 3 },
    { 0x6f473561, 3 },
    { 0x10fb3562, 4 },
    { 0, 0 },
    { 0xa1873564, 3 },
    { 0, 0 },
    { 0x34773566, 3 },
    { 0xe31b3566, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6767356b, 3 },
    { 0xf62f356c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3f3571, 3 },
    { 0x48df3572, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e73577, 3 },
    { 0, 0 },
    { 0x135f3579, 4 },
    { 0x2a8b357a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa803357d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282b3581, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50933585, 3 },
    { 0xcc9b3586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6733591, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc73595, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25f3598, 3 },
    { 0xcb4b3599, 4 },
    { 0xa31b359a, 3 },
    { 0, 0 },
    { 0x9cf3359c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735a0, 3 },
    { 0x5e6735a1, 4 },
    { 0xafb335a2, 3 },
    { 0x70b735a3, 3 },
    { 0xf7cf35a2, 3 },
    { 0x391f35a0, 3 },
    { 0xefdf35a2, 4 },
    { 0x68bb35a5, 4 },
    { 0x514b35a2, 3 },
    { 0x50cb35a0, 3 },
    { 0x99c335a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f35ae, 3 },
    { 0x611b35af, 4 },
    { 0xf07735ae, 4 },
    { 0x2b6335b1, 4 },
    { 0, 0 },
    { 0x25cb35b3, 3 },
    { 0x13f335b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71f35b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b35c0, 3 },
    { 0, 0 },
    { 0x74e335c2, 4 },
    { 0xe44b35c2, 4 },
    { 0xf5b735c2, 3 },
    { 0xd17b35c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c735ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76335cd, 4 },
    { 0xf02f35ce, 4 },
    { 0x93a735cf, 4 },
    { 0x6c7335d0, 4 },
    { 0, 0 },
    { 0xf66b35d2, 3 },
    { 0x9fd335d3, 3 },
    { 0, 0 },
    { 0x665f35d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb335da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2735de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4f35e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3335e5, 4 },
    { 0xca9335e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf35e9, 3 },
    { 0, 0 },
    { 0x7a7f35eb, 3 },
    { 0, 0 },
    { 0x13b35ed, 4 },
    { 0x797b35ee, 3 },
    { 0xf3db35ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6335f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49735f8, 3 },
    { 0, 0 },
    { 0xb75735fa, 3 },
    { 0xb74735fb, 3 },
    { 0x259b35fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b33603, 5 },
    { 0, 0 },
    { 0xd09f3605, 3 },
    { 0x25c73606, 4 },
    { 0xc34b3607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5db360d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e33611, 3 },
    { 0, 0 },
    { 0x4ee73613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb3617, 3 },
    { 0, 0 },
    { 0x738b3619, 3 },
    { 0x6ab73619, 5 },
    { 0x8193361b, 3 },
    { 0xe19f361c, 4 },
    { 0x2ea3361d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a73620, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80073624, 4 },
    { 0x44673624, 3 },
    { 0, 0 },
    { 0x397b3627, 3 },
    { 0xda6b3627, 4 },
    { 0, 0 },
    { 0xdcc3362a, 4 },
    { 0xfad7362a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbf3631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f73636, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963363a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30df363f, 3 },
    { 0xf90b3640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc773643, 4 },
    { 0xa3533644, 3 },
    { 0, 0 },
    { 0xd9733646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c33649, 4 },
    { 0xb9db364a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb364f, 4 },
    { 0xc00f3650, 3 },
    { 0, 0 },
    { 0xefef3652, 4 },
    { 0x4973653, 3 },
    { 0x91b33652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fb365a, 3 },
    { 0, 0 },
    { 0x3dab365c, 3 },
    { 0xf41f365d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b3664, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93df3668, 3 },
    { 0x135b3669, 4 },
    { 0x47b366a, 3 },
    { 0x1787366a, 3 },
    { 0xd353366a, 3 },
    { 0x9a57366a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912f3671, 3 },
    { 0xdf173672, 4 },
    { 0, 0 },
    { 0x290b3674, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7f367a, 4 },
    { 0, 0 },
    { 0xcaaf367c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb367f, 3 },
    { 0, 0 },
    { 0x406f3681, 3 },
    { 0, 0 },
    { 0x4abf3683, 3 },
    { 0, 0 },
    { 0x959f3685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c3368b, 3 },
    { 0x2eb368c, 4 },
    { 0, 0 },
    { 0xd21b368e, 3 },
    { 0x952b368f, 3 },
    { 0, 0 },
    { 0xab1f3691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebf3697, 3 },
    { 0x739b3698, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe437369b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ceb36a1, 3 },
    { 0, 0 },
    { 0x510f36a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f36a6, 3 },
    { 0x9b2736a6, 5 },
    { 0xa65336a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f36ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ab36ae, 3 },
    { 0, 0 },
    { 0xe2bb36b0, 3 },
    { 0, 0 },
    { 0x49c736b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0f36b7, 3 },
    { 0, 0 },
    { 0x8f1b36b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccb36bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25f36c2, 3 },
    { 0x5f6736c2, 3 },
    { 0, 0 },
    { 0xacff36c5, 3 },
    { 0xfdbb36c5, 5 },
    { 0x371f36c5, 3 },
    { 0x467b36c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856336cc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4af36d3, 4 },
    { 0, 0 },
    { 0xebb336d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9b36d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5336df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b36e5, 3 },
    { 0, 0 },
    { 0x350f36e7, 3 },
    { 0x7ae336e8, 4 },
    { 0x880736e9, 4 },
    { 0x27d736e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x328f36ed, 3 },
    { 0, 0 },
    { 0xac4736ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f36f2, 3 },
    { 0x11336f3, 3 },
    { 0x2ceb36f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6336f7, 4 },
    { 0xa9a336f8, 3 },
    { 0x137f36f8, 3 },
    { 0x72fb36fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b36ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b73703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d73708, 4 },
    { 0, 0 },
    { 0x667370a, 3 },
    { 0x313f370b, 3 },
    { 0xf377370b, 3 },
    { 0, 0 },
    { 0xb307370e, 4 },
    { 0x2faf370f, 4 },
    { 0xd59b3710, 3 },
    { 0x41eb3710, 3 },
    { 0xc0473712, 3 },
    { 0xa91f370e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80af3716, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2733719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e33720, 3 },
    { 0x74a73720, 3 },
    { 0xac933720, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaf3726, 3 },
    { 0x21db3726, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fb372b, 4 },
    { 0xcc33372c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08b3730, 3 },
    { 0x434f3731, 4 },
    { 0xb65b3732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f3738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26b373e, 4 },
    { 0x4a8f373f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19373743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9f3748, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb374c, 3 },
    { 0x19d7374c, 5 },
    { 0, 0 },
    { 0x1f1b374f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaef3752, 3 },
    { 0x14ef3753, 4 },
    { 0xc6733754, 3 },
    { 0x40133754, 3 },
    { 0, 0 },
    { 0xe7a73757, 3 },
    { 0xfbef3758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803375d, 3 },
    { 0x9823375e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a473764, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ef3767, 3 },
    { 0x614b3767, 3 },
    { 0, 0 },
    { 0xb787376a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5873772, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fb3775, 3 },
    { 0x74673776, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acf377d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb3780, 4 },
    { 0, 0 },
    { 0xfb473782, 4 },
    { 0, 0 },
    { 0x27c33784, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07f3788, 4 },
    { 0, 0 },
    { 0x6583378a, 3 },
    { 0xc517378b, 4 },
    { 0x6b0f378a, 3 },
    { 0x7087378d, 4 },
    { 0x217378c, 3 },
    { 0x637378f, 3 },
    { 0xd87f378d, 3 },
    { 0xd6133791, 3 },
    { 0xea73378d, 3 },
    { 0x6fd33793, 4 },
    { 0x365b3793, 3 },
    { 0x2a673795, 4 },
    { 0x2a533795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eb379e, 3 },
    { 0, 0 },
    { 0x97d337a0, 3 },
    { 0x3a0337a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5337a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb37ac, 4 },
    { 0, 0 },
    { 0x246f37ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6f37b1, 4 },
    { 0, 0 },
    { 0xd90f37b3, 3 },
    { 0, 0 },
    { 0x853f37b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40337b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0b37bc, 3 },
    { 0xfeab37bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb37c4, 3 },
    { 0xa3fb37c4, 3 },
    { 0x2eb337c6, 3 },
    { 0, 0 },
    { 0x159b37c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x720b37cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209337d0, 3 },
    { 0x362337d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97337d6, 4 },
    { 0x16c737d7, 4 },
    { 0x760b37d7, 3 },
    { 0xd29737d8, 3 },
    { 0xbc337da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96737df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39f37e2, 4 },
    { 0xfea337e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe337e8, 3 },
    { 0xfba337e8, 5 },
    { 0x3dd737ea, 3 },
    { 0xc50737eb, 3 },
    { 0xde2f37e9, 4 },
    { 0x682337eb, 3 },
    { 0, 0 },
    { 0x42b337ef, 3 },
    { 0x4c2337f0, 3 },
    { 0xe5b737ef, 4 },
    { 0x1abb37f2, 3 },
    { 0x250337f3, 4 },
    { 0x8a8f37f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f37fa, 4 },
    { 0x94f337fa, 4 },
    { 0x487b37fc, 4 },
    { 0x554337fb, 4 },
    { 0xa82f37fa, 3 },
    { 0xeda737fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf3805, 3 },
    { 0xc1e73806, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbb380b, 3 },
    { 0, 0 },
    { 0xed5f380d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8f3810, 3 },
    { 0, 0 },
    { 0x64c73812, 3 },
    { 0x10d73812, 3 },
    { 0x48bf3814, 3 },
    { 0xe52b3815, 3 },
    { 0xf6ef3816, 4 },
    { 0x889f3815, 3 },
    { 0x9deb3812, 3 },
    { 0, 0 },
    { 0x80cb381a, 3 },
    { 0xda93381b, 3 },
    { 0xeb17381c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3822, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74f3827, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eb382b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61f382e, 3 },
    { 0x329f382f, 4 },
    { 0x7ce7382f, 3 },
    { 0x9e473830, 3 },
    { 0, 0 },
    { 0x5e2b3833, 3 },
    { 0xb4d33834, 4 },
    { 0x1d773835, 4 },
    { 0, 0 },
    { 0xbe0b3837, 3 },
    { 0, 0 },
    { 0x99ef3839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe47383c, 3 },
    { 0x9383383c, 3 },
    { 0, 0 },
    { 0xe02f383f, 3 },
    { 0x4e973840, 3 },
    { 0xcef3840, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c3384a, 4 },
    { 0xcc43384b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a33850, 4 },
    { 0x3433851, 3 },
    { 0, 0 },
    { 0xdc1f3853, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad733856, 3 },
    { 0x8ceb3857, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bb3861, 3 },
    { 0x636f3862, 3 },
    { 0x90fb3863, 3 },
    { 0xda2b3864, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3b3869, 3 },
    { 0xf93386a, 3 },
    { 0xa197386b, 3 },
    { 0, 0 },
    { 0x545f386d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f3870, 3 },
    { 0, 0 },
    { 0xad033872, 3 },
    { 0x1b273873, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecf3876, 4 },
    { 0, 0 },
    { 0xd0273878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeb387b, 3 },
    { 0, 0 },
    { 0x5653387d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9f3883, 3 },
    { 0x69df3884, 3 },
    { 0xacaf3885, 3 },
    { 0xb30b3886, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2173889, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b27388d, 5 },
    { 0xb177388e, 3 },
    { 0x9bff388f, 3 },
    { 0x9f33890, 4 },
    { 0x453b3890, 3 },
    { 0x80ef3892, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb3898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b7389e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944b38a2, 4 },
    { 0, 0 },
    { 0x72a338a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ab38a7, 3 },
    { 0x9cf338a8, 3 },
    { 0x710f38a9, 4 },
    { 0xd06338aa, 3 },
    { 0xd4cf38aa, 3 },
    { 0x423338ac, 3 },
    { 0xbb7338ad, 3 },
    { 0x870738ae, 3 },
    { 0xde2b38af, 4 },
    { 0x6c5b38b0, 3 },
    { 0, 0 },
    { 0xc85f38b2, 4 },
    { 0x15e738b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf738b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf38b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b738bc, 3 },
    { 0x149338bd, 4 },
    { 0x520f38bd, 3 },
    { 0x37af38bf, 3 },
    { 0x5ea338bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931738c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe338cc, 4 },
    { 0xcb338cd, 3 },
    { 0, 0 },
    { 0x3b7738cf, 4 },
    { 0, 0 },
    { 0xa3e338d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994338d6, 3 },
    { 0, 0 },
    { 0x569f38d8, 3 },
    { 0xb6f738d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b38dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f38e0, 3 },
    { 0xc48738e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cf38ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1f38f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a338f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138738fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f27390a, 3 },
    { 0x2fbb390a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134b390f, 3 },
    { 0xdc5f3910, 3 },
    { 0x310f390f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2433916, 4 },
    { 0, 0 },
    { 0x7fb33918, 3 },
    { 0, 0 },
    { 0x9dcf391a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2433391f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad33924, 3 },
    { 0, 0 },
    { 0x912f3926, 3 },
    { 0, 0 },
    { 0x875f3928, 3 },
    { 0xb3ab3929, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0233934, 3 },
    { 0x571b3935, 3 },
    { 0xc0d73935, 3 },
    { 0x6d9b3937, 3 },
    { 0xd77f3938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3393e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa933941, 4 },
    { 0, 0 },
    { 0x951b3943, 5 },
    { 0x81533943, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98273952, 3 },
    { 0x50a33953, 5 },
    { 0x90933953, 3 },
    { 0, 0 },
    { 0x2d773956, 4 },
    { 0, 0 },
    { 0xa3df3958, 4 },
    { 0xee4f3959, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb3395c, 3 },
    { 0, 0 },
    { 0x15af395e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5df3968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f396b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bb3970, 4 },
    { 0xe16b3971, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdef3976, 3 },
    { 0x46af3976, 4 },
    { 0, 0 },
    { 0xd54f3979, 3 },
    { 0x1b3b397a, 3 },
    { 0x1bc3397a, 3 },
    { 0x4047397c, 3 },
    { 0x7347397c, 3 },
    { 0xf797397e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f3981, 3 },
    { 0x79973981, 3 },
    { 0, 0 },
    { 0x80b3984, 3 },
    { 0xc9db3985, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74273989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f3994, 3 },
    { 0xb3373995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3399b, 3 },
    { 0x6d33399c, 3 },
    { 0xa3bb399d, 3 },
    { 0x6feb399d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337739a3, 3 },
    { 0, 0 },
    { 0x937739a5, 4 },
    { 0xc6c739a6, 4 },
    { 0, 0 },
    { 0x81d739a8, 3 },
    { 0x75db39a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f39ac, 3 },
    { 0, 0 },
    { 0x5a339ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883f39b3, 3 },
    { 0x2d8739b4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x329339b7, 3 },
    { 0xddaf39b8, 3 },
    { 0xac4339b9, 3 },
    { 0x285b39ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56f39ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6f39ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x863f39d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36f39d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf39da, 4 },
    { 0xd4cb39db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76f39de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69339e1, 3 },
    { 0, 0 },
    { 0xafd739e3, 3 },
    { 0, 0 },
    { 0x46cb39e5, 3 },
    { 0xab7f39e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486739ea, 3 },
    { 0xa06b39eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2b39f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c339f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1339fd, 3 },
    { 0xf5cf39fe, 3 },
    { 0xd51739fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf33a02, 3 },
    { 0x787b3a03, 3 },
    { 0, 0 },
    { 0x254b3a05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf73a0a, 3 },
    { 0x23db3a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe173a0e, 3 },
    { 0x99733a0f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238f3a13, 4 },
    { 0x36f73a14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1373a18, 4 },
    { 0, 0 },
    { 0xcbf73a1a, 3 },
    { 0x200f3a1b, 3 },
    { 0, 0 },
    { 0x8bff3a1d, 3 },
    { 0, 0 },
    { 0x2f7b3a1f, 3 },
    { 0x4f6b3a20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88b3a27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b3a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44db3a35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0373a3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1b3a3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ef3a42, 4 },
    { 0xec1f3a43, 3 },
    { 0, 0 },
    { 0xe4f33a45, 4 },
    { 0x39333a45, 3 },
    { 0x54e33a47, 3 },
    { 0x18733a46, 3 },
    { 0, 0 },
    { 0x2f373a4a, 4 },
    { 0xf1ef3a4a, 4 },
    { 0xecb3a4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d73a50, 3 },
    { 0, 0 },
    { 0x3a33a52, 3 },
    { 0xe6ff3a52, 3 },
    { 0, 0 },
    { 0xeaaf3a55, 4 },
    { 0xfd233a56, 3 },
    { 0x7bdf3a57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd433a5a, 3 },
    { 0, 0 },
    { 0x9fa73a5c, 3 },
    { 0x4a233a5c, 4 },
    { 0, 0 },
    { 0x749b3a5f, 3 },
    { 0xfb733a60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82f3a63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9533a66, 3 },
    { 0x67e33a67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ff3a6b, 3 },
    { 0x35573a6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239b3a71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9633a78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87673a81, 4 },
    { 0x3dfb3a82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2033a8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b3a94, 3 },
    { 0xb1df3a94, 3 },
    { 0xd9df3a95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b3a99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adb3a9c, 3 },
    { 0xcd333a9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b33aa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f33aa6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac73ab1, 3 },
    { 0x8cbf3ab2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f3ab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd33ab8, 3 },
    { 0, 0 },
    { 0x3d173aba, 3 },
    { 0xb9ff3abb, 3 },
    { 0xe51b3abb, 4 },
    { 0, 0 },
    { 0xd99b3abe, 3 },
    { 0x63e33abe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72673ac4, 3 },
    { 0x1d6f3ac5, 4 },
    { 0x69773ac6, 3 },
    { 0x3a5b3ac7, 3 },
    { 0, 0 },
    { 0x952b3ac9, 3 },
    { 0x5fdf3aca, 4 },
    { 0xf2df3aca, 3 },
    { 0, 0 },
    { 0xc4d73acd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54f3ad0, 3 },
    { 0x1e873ad1, 4 },
    { 0xb0bf3ad2, 3 },
    { 0x47a73ad3, 3 },
    { 0x75073ad4, 3 },
    { 0xc7eb3ad4, 4 },
    { 0xf0373ad3, 3 },
    { 0, 0 },
    { 0x97eb3ad8, 4 },
    { 0x9e373ad9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97373ae0, 3 },
    { 0x6d733ae1, 3 },
    { 0x63e33ae2, 4 },
    { 0x66af3ae2, 3 },
    { 0, 0 },
    { 0xa8933ae5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac33af0, 3 },
    { 0xd6d33af0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b3af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d73af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a73afe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b3b07, 4 },
    { 0x1b273b07, 4 },
    { 0x39bb3b08, 3 },
    { 0xf5673b0a, 5 },
    { 0, 0 },
    { 0x744f3b0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6473b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddf3b13, 3 },
    { 0, 0 },
    { 0xf2933b15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3033b18, 3 },
    { 0, 0 },
    { 0xe5873b1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d33b1d, 3 },
    { 0, 0 },
    { 0xa6133b1f, 3 },
    { 0x8c3b3b20, 4 },
    { 0xf0d73b20, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526b3b25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f3b29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245b3b2d, 3 },
    { 0x804f3b2e, 4 },
    { 0, 0 },
    { 0x55333b30, 4 },
    { 0x310f3b30, 3 },
    { 0x48473b32, 3 },
    { 0xec73b33, 3 },
    { 0x7b873b33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272b3b3c, 3 },
    { 0, 0 },
    { 0x7f5b3b3e, 4 },
    { 0x550b3b3f, 3 },
    { 0, 0 },
    { 0xe3b73b41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b45, 4 },
    { 0x21573b46, 3 },
    { 0x59e33b46, 3 },
    { 0xbfe33b46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d33b4b, 3 },
    { 0, 0 },
    { 0xa0373b4d, 3 },
    { 0xe97b3b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e73b51, 3 },
    { 0x71af3b51, 3 },
    { 0xab9f3b53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9673b5c, 3 },
    { 0x8fd33b5c, 4 },
    { 0x532b3b5e, 3 },
    { 0x279b3b5f, 4 },
    { 0, 0 },
    { 0x2fbb3b61, 4 },
    { 0, 0 },
    { 0x81e73b63, 3 },
    { 0xd1433b64, 5 },
    { 0xa3433b63, 3 },
    { 0, 0 },
    { 0x7e3b3b67, 3 },
    { 0x20f33b68, 4 },
    { 0x415f3b68, 3 },
    { 0x5eaf3b67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655f3b6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a33b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af3b76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea73b79, 3 },
    { 0x89d73b7a, 3 },
    { 0x7acf3b79, 3 },
    { 0, 0 },
    { 0xec133b7d, 3 },
    { 0xd0f33b7e, 3 },
    { 0x719b3b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40873b82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c73b85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb3b8e, 3 },
    { 0xd6f73b8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da73b94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c073b97, 5 },
    { 0xbf3b98, 4 },
    { 0, 0 },
    { 0x82033b9a, 3 },
    { 0, 0 },
    { 0xf4f73b9c, 3 },
    { 0x6f5b3b9d, 3 },
    { 0x33773b9e, 3 },
    { 0x888f3b9f, 3 },
    { 0xa01f3ba0, 5 },
    { 0x96f3ba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912b3ba7, 3 },
    { 0, 0 },
    { 0x6af73ba9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33bb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6b3bbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa73bc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f3bcc, 3 },
    { 0x79133bcc, 3 },
    { 0, 0 },
    { 0x13133bcf, 3 },
    { 0xb2f73bd0, 3 },
    { 0x94173bd1, 4 },
    { 0xcaeb3bd2, 5 },
    { 0x5eb3bd2, 3 },
    { 0, 0 },
    { 0x3bc33bd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b3bda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92773bdd, 3 },
    { 0xf833bde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cb3be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f533be5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38233bea, 3 },
    { 0x20733beb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27db3bf2, 3 },
    { 0, 0 },
    { 0x50f33bf4, 3 },
    { 0x777f3bf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d73bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6173bfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b33c01, 3 },
    { 0x7773c02, 3 },
    { 0, 0 },
    { 0x5bcb3c04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f3c11, 3 },
    { 0x5cf3c12, 3 },
    { 0x66233c13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99433c1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073c20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8973c23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4433c26, 4 },
    { 0x5b433c26, 3 },
    { 0x9fbb3c28, 4 },
    { 0x875b3c26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea33c2e, 3 },
    { 0xe7033c2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c73c3a, 3 },
    { 0, 0 },
    { 0xbc73c3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6933c40, 4 },
    { 0x19b33c40, 3 },
    { 0x256f3c42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ef3c45, 3 },
    { 0xcb1f3c46, 3 },
    { 0x8fbf3c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88873c4b, 3 },
    { 0xa9573c4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b73c50, 4 },
    { 0, 0 },
    { 0x677f3c52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e973c57, 3 },
    { 0, 0 },
    { 0xd62b3c59, 3 },
    { 0x35b33c5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bb3c61, 3 },
    { 0x87ff3c61, 3 },
    { 0xd3fb3c63, 3 },
    { 0xd2573c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f933c6b, 4 },
    { 0, 0 },
    { 0x5b3c6d, 3 },
    { 0, 0 },
    { 0xdb5f3c6f, 3 },
    { 0xec133c70, 3 },
    { 0x28b33c70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a33c78, 3 },
    { 0x608b3c78, 3 },
    { 0, 0 },
    { 0x25e33c7b, 3 },
    { 0, 0 },
    { 0xe4773c7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bb3c82, 3 },
    { 0x3b373c83, 3 },
    { 0x50f3c84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807f3c8a, 3 },
    { 0xffc73c8b, 3 },
    { 0xbbbb3c8c, 3 },
    { 0x7df3c8d, 3 },
    { 0x8aa73c8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bf3c91, 3 },
    { 0x8f973c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e33c9a, 5 },
    { 0x90773c9a, 4 },
    { 0xe3273c9c, 3 },
    { 0x13bb3c9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52f3ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af3ca6, 3 },
    { 0xe1073ca6, 3 },
    { 0xe0533ca8, 3 },
    { 0x2a5b3ca9, 3 },
    { 0, 0 },
    { 0x32b73cab, 4 },
    { 0x628f3cac, 3 },
    { 0x51ff3cac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca073cb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3cb9, 3 },
    { 0xdeb73cba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29b3cc1, 3 },
    { 0x1e4f3cc1, 4 },
    { 0xbd2f3cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e73cc7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ef3cca, 3 },
    { 0, 0 },
    { 0x93bb3ccc, 3 },
    { 0, 0 },
    { 0x65833cce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8b3cd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e573cd5, 3 },
    { 0x71d73cd5, 3 },
    { 0, 0 },
    { 0xd6e73cd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e33cdc, 3 },
    { 0xe26b3cdd, 3 },
    { 0xd83f3cdc, 3 },
    { 0xebbf3cdf, 4 },
    { 0xc2673cdf, 3 },
    { 0x8d473ce1, 3 },
    { 0x11df3cdc, 3 },
    { 0x846b3ce3, 3 },
    { 0, 0 },
    { 0x63233ce5, 5 },
    { 0xd3cf3ce6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6773ce9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ab3cec, 4 },
    { 0, 0 },
    { 0x14233cee, 5 },
    { 0xb0173cef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da73cf2, 3 },
    { 0, 0 },
    { 0x300b3cf4, 4 },
    { 0, 0 },
    { 0x48933cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff3cf9, 4 },
    { 0x74733cf9, 3 },
    { 0x7f0f3cfb, 3 },
    { 0x767b3cfc, 3 },
    { 0x370f3cfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4f3d00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc133d04, 3 },
    { 0, 0 },
    { 0xfbb3d06, 3 },
    { 0x13ab3d07, 3 },
    { 0, 0 },
    { 0x57873d09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e933d0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b73d12, 5 },
    { 0xe1c33d12, 3 },
    { 0x78033d14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5873d18, 3 },
    { 0xbba33d19, 4 },
    { 0xabfb3d1a, 3 },
    { 0xf77b3d1b, 3 },
    { 0, 0 },
    { 0xa8733d1d, 3 },
    { 0x83233d1e, 4 },
    { 0xd6af3d1f, 3 },
    { 0x9fdf3d1f, 3 },
    { 0xc01b3d1f, 3 },
    { 0x87473d1f, 4 },
    { 0x81273d23, 4 },
    { 0x93eb3d24, 3 },
    { 0xa9ab3d24, 4 },
    { 0x292b3d25, 3 },
    { 0xfcc33d25, 3 },
    { 0x198b3d28, 3 },
    { 0x134f3d26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124b3d2d, 3 },
    { 0xb56f3d2e, 3 },
    { 0x17a73d2d, 3 },
    { 0x44873d30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a273d33, 3 },
    { 0xc77b3d34, 3 },
    { 0, 0 },
    { 0x83333d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d873d3b, 3 },
    { 0, 0 },
    { 0x31fb3d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33d44, 4 },
    { 0, 0 },
    { 0x2b3b3d46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af3d4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3b3d4f, 4 },
    { 0x10133d4f, 3 },
    { 0x549f3d51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2f3d59, 3 },
    { 0xb6bf3d5a, 3 },
    { 0x179b3d5a, 3 },
    { 0, 0 },
    { 0xb0833d5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb3d63, 3 },
    { 0x52b33d63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce533d67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bb3d6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680f3d71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34433d74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38073d83, 3 },
    { 0x6f833d83, 3 },
    { 0x263f3d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a33d88, 3 },
    { 0xa7f3d89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63f3d8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b33d94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b3d98, 4 },
    { 0x67733d99, 3 },
    { 0xa2033d9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6973d9e, 3 },
    { 0, 0 },
    { 0xf5ff3da0, 3 },
    { 0x60f33da1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955f3da6, 3 },
    { 0xa9733da6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f33dab, 3 },
    { 0, 0 },
    { 0x96bb3dad, 3 },
    { 0x785f3dae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94173db3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x123f3db6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f3dbc, 3 },
    { 0x56e33dbd, 3 },
    { 0xdaf33dbe, 3 },
    { 0xc38b3dbc, 5 },
    { 0x23f73dc0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x760b3dc3, 3 },
    { 0x58b33dc3, 3 },
    { 0xbfc73dc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9f3dcb, 3 },
    { 0xe133dcb, 3 },
    { 0xd16b3dcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe873dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bf3dda, 3 },
    { 0, 0 },
    { 0x83573ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90633de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83233de6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x702f3de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fb3df0, 3 },
    { 0x3ce73df0, 5 },
    { 0x4ee73df2, 4 },
    { 0, 0 },
    { 0x6da33df4, 3 },
    { 0, 0 },
    { 0xeb473df6, 4 },
    { 0, 0 },
    { 0x859f3df8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5973dfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcf3e03, 3 },
    { 0x2d0b3e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf633e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49933e0c, 3 },
    { 0, 0 },
    { 0x7c1f3e0e, 3 },
    { 0, 0 },
    { 0xec633e10, 3 },
    { 0x7ca33e10, 4 },
    { 0x28133e12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4733e16, 3 },
    { 0xe29b3e16, 4 },
    { 0x39af3e17, 3 },
    { 0x65ef3e19, 3 },
    { 0xeba73e18, 3 },
    { 0, 0 },
    { 0x6f573e1c, 4 },
    { 0xa41b3e1c, 4 },
    { 0x647b3e1e, 3 },
    { 0xd5df3e1f, 3 },
    { 0xeb433e1f, 3 },
    { 0xc0233e1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53b3e25, 3 },
    { 0xf67b3e26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b3e2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5f3e2e, 3 },
    { 0x76c33e2f, 3 },
    { 0xbd273e30, 3 },
    { 0x895b3e2f, 3 },
    { 0, 0 },
    { 0x60bf3e33, 3 },
    { 0x6af33e34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33e37, 3 },
    { 0xc9e33e38, 3 },
    { 0x13133e39, 4 },
    { 0x20c33e39, 4 },
    { 0x5cc33e39, 3 },
    { 0, 0 },
    { 0x6ffb3e3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa633e41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf73e4b, 3 },
    { 0xd3b33e4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a33e50, 3 },
    { 0x38a33e51, 3 },
    { 0x9a033e52, 3 },
    { 0xb2bb3e51, 4 },
    { 0x4d673e54, 4 },
    { 0x62173e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f073e5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8073e61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebb3e66, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d33e6a, 3 },
    { 0, 0 },
    { 0x481f3e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebb3e70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff3e73, 3 },
    { 0x9a4b3e73, 3 },
    { 0xbdcb3e75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c433e78, 3 },
    { 0, 0 },
    { 0xdf033e7a, 4 },
    { 0, 0 },
    { 0xf0f3e7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf3e7f, 3 },
    { 0xd81b3e80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8973e89, 3 },
    { 0xe0173e8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cf3e9b, 3 },
    { 0xfa673e9c, 3 },
    { 0, 0 },
    { 0xa7bf3e9e, 3 },
    { 0x534f3e9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5f3ea2, 3 },
    { 0, 0 },
    { 0x31133ea4, 3 },
    { 0x5a0b3ea5, 4 },
    { 0, 0 },
    { 0xab873ea7, 3 },
    { 0, 0 },
    { 0xf8873ea9, 4 },
    { 0xc97f3eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b3eaf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b3eb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b3eb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd73ebe, 3 },
    { 0xcfe33ebe, 3 },
    { 0x59f3ec0, 4 },
    { 0x3a133ec1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f3eca, 3 },
    { 0x2c833ecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff3ece, 3 },
    { 0xc96f3ece, 3 },
    { 0x91b73ece, 3 },
    { 0x80ef3ed1, 3 },
    { 0x21a73ed1, 3 },
    { 0, 0 },
    { 0x7ce33ed4, 3 },
    { 0xa8ef3ed5, 3 },
    { 0x7c673ed6, 3 },
    { 0x753b3ed5, 3 },
    { 0x50db3ed8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5f3edb, 3 },
    { 0, 0 },
    { 0xab8b3edd, 5 },
    { 0xba2f3ede, 4 },
    { 0, 0 },
    { 0x265f3ee0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4733ee8, 3 },
    { 0xbeeb3ee9, 3 },
    { 0xd7c33eea, 4 },
    { 0, 0 },
    { 0x80033eec, 5 },
    { 0xe1c73eed, 3 },
    { 0x56533eee, 4 },
    { 0xb0633eef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d33ef4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62f3ef7, 4 },
    { 0, 0 },
    { 0x23cb3ef9, 4 },
    { 0x55173efa, 3 },
    { 0, 0 },
    { 0xb7c33efc, 4 },
    { 0xd3f73efd, 4 },
    { 0xb533efd, 3 },
    { 0xa27b3efd, 4 },
    { 0x15a73f00, 3 },
    { 0x1b773f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fb3f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b73f08, 3 },
    { 0x716b3f08, 3 },
    { 0x37af3f09, 3 },
    { 0x6aef3f0b, 4 },
    { 0x1f1f3f0c, 3 },
    { 0x227f3f0b, 3 },
    { 0x64db3f0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6433f13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def3f19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8f3f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b733f1f, 3 },
    { 0x18b3f20, 3 },
    { 0x333b3f21, 3 },
    { 0, 0 },
    { 0x87a73f23, 3 },
    { 0x74cf3f24, 4 },
    { 0xc4b73f25, 3 },
    { 0x543f3f23, 4 },
    { 0x5d33f25, 3 },
    { 0, 0 },
    { 0x2f033f29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cb3f2e, 3 },
    { 0x71133f2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ab3f35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7b3f38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f3f3b, 3 },
    { 0xfa3f3f3b, 3 },
    { 0xde033f3d, 3 },
    { 0x9ccf3f3d, 3 },
    { 0, 0 },
    { 0x709b3f40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f3f45, 3 },
    { 0, 0 },
    { 0x245f3f47, 3 },
    { 0, 0 },
    { 0x84db3f49, 3 },
    { 0, 0 },
    { 0x73133f4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70b3f4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc73f53, 3 },
    { 0x61473f54, 3 },
    { 0x6b633f53, 4 },
    { 0x9373f56, 3 },
    { 0x5f7f3f56, 3 },
    { 0x51273f58, 3 },
    { 0xc0773f58, 3 },
    { 0x8bbb3f59, 3 },
    { 0x96933f5b, 4 },
    { 0x83873f5b, 3 },
    { 0xdabb3f5d, 3 },
    { 0, 0 },
    { 0x4edf3f5f, 4 },
    { 0xce73f60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72f3f63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16333f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f973f79, 3 },
    { 0, 0 },
    { 0x510b3f7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9f3f80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x34db3f83, 3 },
    { 0, 0 },
    { 0xdb7f3f85, 3 },
    { 0, 0 },
    { 0x66073f87, 3 },
    { 0, 0 },
    { 0x80a73f89, 3 },
    { 0, 0 },
    { 0x47733f8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd73f8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa33f91, 3 },
    { 0, 0 },
    { 0xfbc73f93, 3 },
    { 0xe2af3f94, 3 },
    { 0xf11f3f94, 3 },
    { 0xf2573f96, 3 },
    { 0x3e2f3f97, 4 },
    { 0x7ac73f97, 3 },
    { 0, 0 },
    { 0xf96b3f9a, 5 },
    { 0xf5a33f9b, 3 },
    { 0, 0 },
    { 0x66d73f9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52b3fa1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933fa6, 4 },
    { 0xb81f3fa6, 3 },
    { 0xbf333fa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb973fab, 3 },
    { 0xbe2b3fac, 3 },
    { 0x9ec33fad, 3 },
    { 0, 0 },
    { 0x78ff3faf, 4 },
    { 0, 0 },
    { 0xbe5b3fb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53373fb4, 3 },
    { 0, 0 },
    { 0x999b3fb6, 3 },
    { 0x985f3fb7, 3 },
    { 0x762b3fb8, 4 },
    { 0, 0 },
    { 0x56633fba, 3 },
    { 0, 0 },
    { 0xf0e73fbc, 4 },
    { 0x41f33fbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36733fc2, 3 },
    { 0xf6273fc3, 4 },
    { 0, 0 },
    { 0xbfaf3fc5, 3 },
    { 0, 0 },
    { 0xfbcb3fc7, 4 },
    { 0xb87f3fc7, 4 },
    { 0x926b3fc9, 4 },
    { 0x5e973fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb73fd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ef3fd3, 3 },
    { 0xb6233fd4, 4 },
    { 0x56733fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f3fd8, 3 },
    { 0xf2773fd9, 4 },
    { 0xaddb3fd9, 3 },
    { 0x44d33fd9, 3 },
    { 0x87033fd9, 3 },
    { 0x67d73fd8, 3 },
    { 0, 0 },
    { 0xef073fdf, 4 },
    { 0, 0 },
    { 0x42ff3fe1, 3 },
    { 0x1c233fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f3fe7, 3 },
    { 0, 0 },
    { 0x86e33fe9, 3 },
    { 0xb93b3fea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d73ffa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81973ffd, 3 },
    { 0, 0 },
    { 0xbaff3fff, 3 },
    { 0xf5c33fff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b4006, 4 },
    { 0x75134007, 4 },
    { 0, 0 },
    { 0x7feb4009, 5 },
    { 0xe08b400a, 3 },
    { 0x2aaf400b, 3 },
    { 0, 0 },
    { 0x2f4b400d, 4 },
    { 0x4c3b400d, 4 },
    { 0x6183400f, 4 },
    { 0, 0 },
    { 0xb5bf4011, 3 },
    { 0xb9934012, 3 },
    { 0x748f4013, 4 },
    { 0x4a774014, 3 },
    { 0x4fd74015, 3 },
    { 0xb3ef4015, 4 },
    { 0x5fdb4015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15df401e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1b4023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x779b4026, 3 },
    { 0xb99f4026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2db402b, 4 },
    { 0x317b402b, 3 },
    { 0x4453402d, 3 },
    { 0, 0 },
    { 0xbbb402f, 3 },
    { 0x333f402f, 4 },
    { 0xd9b402f, 4 },
    { 0xa26f4031, 3 },
    { 0x1d574033, 3 },
    { 0xe7a34034, 3 },
    { 0x75974034, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x233b4038, 3 },
    { 0xc2b34038, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bb4043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c74047, 3 },
    { 0x3f374048, 3 },
    { 0x61cb4049, 4 },
    { 0xefa74049, 3 },
    { 0xd3bb404a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07b4050, 3 },
    { 0, 0 },
    { 0x379b4052, 4 },
    { 0xbdf4053, 3 },
    { 0x9334052, 3 },
    { 0x5cd34055, 3 },
    { 0x16e34055, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7223405c, 3 },
    { 0, 0 },
    { 0x4313405e, 3 },
    { 0x47405e, 3 },
    { 0xe14b4060, 3 },
    { 0xfe534061, 4 },
    { 0, 0 },
    { 0xba2b4063, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21b4069, 3 },
    { 0x6113406a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bef4071, 4 },
    { 0xc3174071, 3 },
    { 0xa0174073, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f74076, 3 },
    { 0xe55b4077, 3 },
    { 0xeba34078, 3 },
    { 0x5cf74076, 3 },
    { 0, 0 },
    { 0x5e77407b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6b4080, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0b4086, 3 },
    { 0xc7c74087, 3 },
    { 0x6d8b4087, 3 },
    { 0x92334089, 3 },
    { 0x2a674088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf3408f, 4 },
    { 0xe6b7408f, 4 },
    { 0, 0 },
    { 0x51eb4092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74095, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb74099, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5517409c, 3 },
    { 0x3c57409d, 3 },
    { 0x27cf409e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613340a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e740a6, 4 },
    { 0, 0 },
    { 0xacf740a8, 4 },
    { 0xb3df40a9, 3 },
    { 0x1c0340a9, 4 },
    { 0, 0 },
    { 0x78e740ac, 3 },
    { 0, 0 },
    { 0x83f340ae, 3 },
    { 0xc35b40af, 3 },
    { 0x3ef340ae, 4 },
    { 0xd4d340b1, 3 },
    { 0xb71f40b0, 4 },
    { 0xe52b40b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bf40b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x120b40ba, 4 },
    { 0x4e9340bb, 3 },
    { 0xf4c740bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e340bf, 4 },
    { 0x17a740c0, 4 },
    { 0xf28340bf, 3 },
    { 0x551f40c1, 4 },
    { 0x274f40c2, 3 },
    { 0xefe340c4, 3 },
    { 0xbaeb40c5, 4 },
    { 0xb6bb40c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9340ca, 3 },
    { 0xaaf340cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91f40d1, 3 },
    { 0, 0 },
    { 0xd38f40d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436740d7, 4 },
    { 0x5db740d8, 3 },
    { 0x910b40d8, 3 },
    { 0, 0 },
    { 0x248b40db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173f40e2, 3 },
    { 0x3f7f40e3, 3 },
    { 0, 0 },
    { 0x1ebf40e5, 3 },
    { 0, 0 },
    { 0xbd4340e7, 3 },
    { 0x5f3340e8, 5 },
    { 0xe52f40e9, 3 },
    { 0, 0 },
    { 0x462340eb, 4 },
    { 0xdd6340ec, 5 },
    { 0xe19340ec, 4 },
    { 0x274f40eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f40f1, 4 },
    { 0x710f40f1, 4 },
    { 0x78c340f1, 4 },
    { 0, 0 },
    { 0xd6340f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf40f8, 3 },
    { 0, 0 },
    { 0xb42b40fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6b40fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d34108, 3 },
    { 0, 0 },
    { 0x851f410a, 4 },
    { 0, 0 },
    { 0x17d3410c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7853410f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab4112, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a34116, 3 },
    { 0x937b4116, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ef4123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fb412b, 4 },
    { 0, 0 },
    { 0x4db412d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef4130, 3 },
    { 0, 0 },
    { 0x1d7f4132, 5 },
    { 0xa63f4132, 3 },
    { 0, 0 },
    { 0x8fa34135, 3 },
    { 0, 0 },
    { 0x85334137, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438f413b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb413e, 3 },
    { 0x8a9f413f, 3 },
    { 0x87874140, 4 },
    { 0x39e74140, 3 },
    { 0x25e34142, 3 },
    { 0, 0 },
    { 0x71274144, 3 },
    { 0xb5b4145, 3 },
    { 0, 0 },
    { 0x721b4147, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67414a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e7414f, 3 },
    { 0, 0 },
    { 0xbdcb4151, 4 },
    { 0x8774151, 3 },
    { 0x10534153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24df4161, 3 },
    { 0xf5c34161, 3 },
    { 0, 0 },
    { 0x5b1b4164, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b74169, 3 },
    { 0xaf234169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb416d, 3 },
    { 0xc93f416d, 3 },
    { 0, 0 },
    { 0x19834170, 3 },
    { 0, 0 },
    { 0xf02f4172, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5274178, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12f417e, 4 },
    { 0xccb3417f, 3 },
    { 0xc38f417f, 4 },
    { 0x5b0b417e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d34185, 3 },
    { 0xd4734186, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b418a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff418d, 3 },
    { 0x28c7418e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d34192, 3 },
    { 0xa5bb4192, 5 },
    { 0x404b4192, 4 },
    { 0x1c074195, 3 },
    { 0xc32f4196, 4 },
    { 0, 0 },
    { 0xa35f4198, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b419d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8741a4, 3 },
    { 0xf7341a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73df41aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b341ae, 4 },
    { 0, 0 },
    { 0x1fa741b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a341b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38341ba, 3 },
    { 0, 0 },
    { 0xde0b41bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b741bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed741c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99341c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d741ca, 4 },
    { 0xf6d741cb, 3 },
    { 0x666f41ca, 5 },
    { 0x2d341cd, 3 },
    { 0xcf0f41ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79df41d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de341d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7741d8, 3 },
    { 0x13eb41d8, 3 },
    { 0x641341da, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d341e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x198341e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb741e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2f41ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b41ef, 3 },
    { 0x67e341f0, 4 },
    { 0xc44b41f1, 3 },
    { 0xa1b741f2, 3 },
    { 0x3c8341f3, 3 },
    { 0, 0 },
    { 0x881f41f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f41f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbb41ff, 3 },
    { 0, 0 },
    { 0xb1534201, 3 },
    { 0xe5734202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8334206, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63420a, 3 },
    { 0xc3b7420b, 3 },
    { 0, 0 },
    { 0xb167420d, 3 },
    { 0x533b420d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e374212, 3 },
    { 0xec1b4213, 3 },
    { 0x4a3f4214, 3 },
    { 0x153b4213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a774218, 3 },
    { 0x307f4218, 4 },
    { 0x1a1f421a, 3 },
    { 0xb247421b, 3 },
    { 0x686b4218, 3 },
    { 0x3574218, 3 },
    { 0xd543421e, 5 },
    { 0xe37f421f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaf4222, 4 },
    { 0x1afb4223, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def4227, 3 },
    { 0xa02b4228, 4 },
    { 0x1acb4228, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a7422c, 3 },
    { 0xfd6f422d, 3 },
    { 0, 0 },
    { 0xddfb422f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2234232, 3 },
    { 0xac0f4232, 4 },
    { 0x954f4234, 4 },
    { 0xf274235, 3 },
    { 0x26d74236, 3 },
    { 0x994b4237, 3 },
    { 0x681b4238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef423b, 3 },
    { 0xca43423b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7f4244, 4 },
    { 0, 0 },
    { 0x20ab4246, 3 },
    { 0x68ab4247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fb424a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec34250, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x505f4253, 3 },
    { 0, 0 },
    { 0x1ccb4255, 3 },
    { 0xad234256, 3 },
    { 0, 0 },
    { 0x23f74258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdb425f, 4 },
    { 0x20e3425f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20b4264, 4 },
    { 0xe81f4265, 3 },
    { 0x6e974266, 3 },
    { 0xe76b4267, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be7426d, 3 },
    { 0x8417426e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c34273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff34278, 3 },
    { 0, 0 },
    { 0x216f427a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b334286, 4 },
    { 0xe3274286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ab428a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3937428d, 3 },
    { 0, 0 },
    { 0x94f428f, 3 },
    { 0x1e434290, 3 },
    { 0xea3f4291, 3 },
    { 0x7ddf4292, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be74297, 3 },
    { 0xc6c34298, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ef429b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531b42ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b742ae, 3 },
    { 0x826b42af, 3 },
    { 0x805342b0, 3 },
    { 0x4b8342b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fab42b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x807342b7, 3 },
    { 0x4cb42b8, 3 },
    { 0x841342b7, 3 },
    { 0xe9f42b8, 3 },
    { 0, 0 },
    { 0xdf3b42bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f42bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff42c5, 3 },
    { 0x716342c5, 3 },
    { 0x380b42c6, 3 },
    { 0, 0 },
    { 0x310f42c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1342ce, 3 },
    { 0x263b42cf, 3 },
    { 0x85a342d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4742d3, 4 },
    { 0, 0 },
    { 0xf15b42d5, 3 },
    { 0x47db42d6, 3 },
    { 0x78cf42d5, 3 },
    { 0xf9a742d6, 3 },
    { 0, 0 },
    { 0x959742da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958b42de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb742e3, 3 },
    { 0x2abb42e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x478742e7, 3 },
    { 0x659342e8, 3 },
    { 0x7fab42e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b42ee, 3 },
    { 0, 0 },
    { 0xac1f42f0, 4 },
    { 0xe71742f1, 3 },
    { 0xf10342f2, 3 },
    { 0xec0b42f2, 3 },
    { 0xa67f42f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa342fc, 5 },
    { 0x207b42fd, 3 },
    { 0, 0 },
    { 0x736b42ff, 3 },
    { 0xfabf42ff, 3 },
    { 0xd86f42ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be74304, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x60034307, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3b430a, 4 },
    { 0, 0 },
    { 0x42e3430c, 3 },
    { 0xd6e7430c, 4 },
    { 0x67f430e, 3 },
    { 0xef7430f, 3 },
    { 0x1513430e, 3 },
    { 0xf8734311, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f4315, 3 },
    { 0xcc4b4316, 3 },
    { 0, 0 },
    { 0x310b4318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7431b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267431e, 3 },
    { 0, 0 },
    { 0x8d4b4320, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9874323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdb4326, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4f432a, 3 },
    { 0x2573432b, 3 },
    { 0x69a7432a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecf4330, 3 },
    { 0xe3bf4331, 3 },
    { 0x5ec34332, 3 },
    { 0x115b4333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x785f4336, 3 },
    { 0, 0 },
    { 0x42874338, 3 },
    { 0x511f4339, 3 },
    { 0xcddf4338, 3 },
    { 0, 0 },
    { 0xb207433c, 3 },
    { 0xc24b433d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38074340, 3 },
    { 0, 0 },
    { 0xcbd34342, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a34345, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e34348, 4 },
    { 0, 0 },
    { 0x235b434a, 4 },
    { 0x2fcb434a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c74352, 3 },
    { 0, 0 },
    { 0x65bf4354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf534357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f3435c, 4 },
    { 0xdabf435d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb4361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdb4364, 3 },
    { 0x6e5b4364, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c34368, 3 },
    { 0xfd074369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301b436e, 3 },
    { 0x12eb436f, 4 },
    { 0, 0 },
    { 0xd3af4371, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be34375, 3 },
    { 0, 0 },
    { 0xcd34377, 3 },
    { 0x94734377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b437b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3df437e, 3 },
    { 0xf9f3437f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a74387, 4 },
    { 0, 0 },
    { 0xdf334389, 4 },
    { 0x413438a, 3 },
    { 0xb83438a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f438f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cb4392, 3 },
    { 0, 0 },
    { 0xf47b4394, 3 },
    { 0, 0 },
    { 0x6534396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5f439b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b43a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f343a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323343ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cb43ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264343b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f743bc, 3 },
    { 0xba1743bd, 3 },
    { 0x58d343bc, 4 },
    { 0xe17f43be, 3 },
    { 0, 0 },
    { 0x832743c1, 3 },
    { 0xb7ef43c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6f43cb, 3 },
    { 0, 0 },
    { 0xcdc343cd, 3 },
    { 0x6ecf43ce, 3 },
    { 0xfcbf43cd, 3 },
    { 0xa47f43d0, 3 },
    { 0x557b43cd, 3 },
    { 0, 0 },
    { 0x850343d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4743d7, 5 },
    { 0x2feb43d8, 4 },
    { 0x3df343d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb43e0, 3 },
    { 0x99bb43e1, 4 },
    { 0x529743e0, 4 },
    { 0, 0 },
    { 0xdfb43e4, 3 },
    { 0x49343e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29743ea, 3 },
    { 0x4d7f43eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09f43f0, 3 },
    { 0xa58f43f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f43f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c343f8, 4 },
    { 0xb40343f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba343fc, 3 },
    { 0xcc0f43fc, 3 },
    { 0xc16743fe, 3 },
    { 0x626b43ff, 3 },
    { 0x65d34400, 3 },
    { 0xb31743fd, 3 },
    { 0xf98743fe, 3 },
    { 0x6a9b4402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb4408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcb440f, 3 },
    { 0, 0 },
    { 0x3f0f4411, 3 },
    { 0, 0 },
    { 0x44974413, 3 },
    { 0xa574414, 3 },
    { 0xb8974414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa357441b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86034420, 3 },
    { 0xe6974421, 3 },
    { 0x77334421, 3 },
    { 0xa1e74420, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb4426, 3 },
    { 0xdb5f4426, 3 },
    { 0xdc9b4426, 3 },
    { 0x4fc34429, 3 },
    { 0xbcab442a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7627442d, 3 },
    { 0, 0 },
    { 0x9c3b442f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adf4435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ef4439, 3 },
    { 0x89d3443a, 3 },
    { 0xbba3443b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acf4443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff7444d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abf4450, 3 },
    { 0x5db4451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d34455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3b4458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b4460, 3 },
    { 0x61174461, 4 },
    { 0xb6b4462, 4 },
    { 0x34c74460, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0f4469, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b4470, 4 },
    { 0xdbcb4471, 3 },
    { 0, 0 },
    { 0xa87b4473, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c74478, 3 },
    { 0x8a9b4479, 3 },
    { 0x70574479, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e34480, 5 },
    { 0, 0 },
    { 0x5f7f4482, 3 },
    { 0x8d5f4483, 3 },
    { 0, 0 },
    { 0x78b74485, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82034496, 4 },
    { 0x1a174497, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541f449b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf449e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f44a4, 4 },
    { 0x42344a5, 3 },
    { 0xf1eb44a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b744ae, 3 },
    { 0, 0 },
    { 0x5b0b44b0, 3 },
    { 0x33bf44b1, 3 },
    { 0, 0 },
    { 0xf39744b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82db44b9, 4 },
    { 0, 0 },
    { 0x685b44bb, 3 },
    { 0, 0 },
    { 0xf4ff44bd, 4 },
    { 0xd40744be, 3 },
    { 0x3bdf44bf, 5 },
    { 0xd16744c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704344c4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf44c7, 4 },
    { 0, 0 },
    { 0x2f6744c9, 3 },
    { 0x729744c9, 3 },
    { 0xf9b44cb, 4 },
    { 0xe28b44cb, 4 },
    { 0x698b44cb, 3 },
    { 0x149744ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9744d1, 3 },
    { 0x672744d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f744d6, 4 },
    { 0x406744d7, 3 },
    { 0xfb9f44d8, 3 },
    { 0x511344d9, 4 },
    { 0, 0 },
    { 0x271344db, 3 },
    { 0x7cf744dc, 3 },
    { 0xd8f44dd, 4 },
    { 0, 0 },
    { 0xfc8344df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1344e6, 3 },
    { 0, 0 },
    { 0x423f44e8, 3 },
    { 0x73f744e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31f44ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa744f4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27744f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4744fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540b4501, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e074506, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b34511, 3 },
    { 0x526b4511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a34518, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2703451c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a97451f, 3 },
    { 0x2a9b4520, 3 },
    { 0x8ad74520, 5 },
    { 0x90fb4522, 3 },
    { 0, 0 },
    { 0x18c34524, 3 },
    { 0x483f4524, 4 },
    { 0x67174526, 4 },
    { 0x62534526, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedef452d, 3 },
    { 0, 0 },
    { 0x9ddb452f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f74532, 3 },
    { 0x69f34532, 3 },
    { 0, 0 },
    { 0xb8d34535, 3 },
    { 0xe73b4536, 4 },
    { 0x2e3b4535, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3527453a, 3 },
    { 0xe4ab453a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b74541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af4546, 3 },
    { 0, 0 },
    { 0x61574548, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7454e, 3 },
    { 0xcf3f454f, 3 },
    { 0x6aaf4550, 4 },
    { 0x940b4550, 3 },
    { 0x40174552, 3 },
    { 0xb5ef4550, 4 },
    { 0, 0 },
    { 0x547b4555, 3 },
    { 0x7ed74556, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a7455a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbf455f, 4 },
    { 0x75b74560, 3 },
    { 0x371f4561, 4 },
    { 0xe12f455f, 3 },
    { 0x39ff4563, 3 },
    { 0, 0 },
    { 0xeb6f4565, 3 },
    { 0xc4bb4565, 4 },
    { 0x4ff34565, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ff456c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4374570, 3 },
    { 0, 0 },
    { 0x82934572, 4 },
    { 0, 0 },
    { 0x949f4574, 3 },
    { 0xe64b4574, 4 },
    { 0xb21b4574, 3 },
    { 0xaab74574, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e3457a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f457d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906b4585, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c34588, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485b458f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d34597, 3 },
    { 0xb8474597, 3 },
    { 0, 0 },
    { 0xe05f459a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb103459d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f745a0, 3 },
    { 0xbcfb45a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52af45a4, 4 },
    { 0x237b45a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bf45b7, 4 },
    { 0, 0 },
    { 0x5345b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf345be, 3 },
    { 0, 0 },
    { 0x4e8345c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4f45c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8f45cc, 4 },
    { 0, 0 },
    { 0xe1a345ce, 3 },
    { 0xdf8745ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bef45d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f45d8, 3 },
    { 0x499b45d8, 3 },
    { 0xab0345da, 3 },
    { 0x9f3f45da, 4 },
    { 0xd4e745dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f45df, 3 },
    { 0xcb2345df, 3 },
    { 0, 0 },
    { 0xa02745e2, 3 },
    { 0xf49f45e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f45e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3345e9, 3 },
    { 0x57c345ea, 4 },
    { 0x3c9745ea, 3 },
    { 0xd40b45ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41345ef, 4 },
    { 0x227745ef, 3 },
    { 0x407b45f0, 3 },
    { 0x591345f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaf45f6, 3 },
    { 0xcafb45f7, 3 },
    { 0x236345f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d534602, 3 },
    { 0, 0 },
    { 0x29534604, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ff460e, 3 },
    { 0x9dd3460f, 3 },
    { 0x19a7460e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b4617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b461b, 3 },
    { 0x9027461c, 3 },
    { 0xfe3b461d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1b4620, 5 },
    { 0xd1634621, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f4627, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9627462a, 3 },
    { 0x2dc3462b, 3 },
    { 0x51e3462c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f4630, 3 },
    { 0, 0 },
    { 0x5dfb4632, 3 },
    { 0x4f7f4633, 3 },
    { 0, 0 },
    { 0xa02b4635, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512b4639, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f83463c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15af4641, 3 },
    { 0x63474642, 4 },
    { 0xa29f4643, 3 },
    { 0x3e4b4644, 3 },
    { 0xd37f4641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4f4648, 3 },
    { 0x4d4f4649, 3 },
    { 0xf8bb4649, 3 },
    { 0, 0 },
    { 0xfd57464c, 3 },
    { 0, 0 },
    { 0x193b464e, 3 },
    { 0xb123464e, 4 },
    { 0x1e5b4650, 4 },
    { 0x918f4650, 3 },
    { 0x674f4652, 3 },
    { 0x2df4650, 4 },
    { 0xdd73464e, 3 },
    { 0xbf874655, 3 },
    { 0, 0 },
    { 0x5c234657, 3 },
    { 0, 0 },
    { 0x4e3b4659, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fb465c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad34663, 3 },
    { 0xca2b4664, 5 },
    { 0xb8af4665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd34669, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x904f466c, 3 },
    { 0x4e8b466d, 5 },
    { 0x7347466e, 3 },
    { 0x763b466c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d74673, 5 },
    { 0xf30b4673, 3 },
    { 0xcc974673, 3 },
    { 0x41ff4676, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f4679, 3 },
    { 0xb3df467a, 3 },
    { 0x8b5b467b, 3 },
    { 0x211f467c, 3 },
    { 0x2ea7467d, 3 },
    { 0, 0 },
    { 0x2d2b467f, 3 },
    { 0x28f34680, 3 },
    { 0xd98f4681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1df4687, 3 },
    { 0, 0 },
    { 0x29b4689, 4 },
    { 0x68e3468a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136f468e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44074692, 4 },
    { 0, 0 },
    { 0xd1cb4694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96634699, 3 },
    { 0x48b4699, 3 },
    { 0xcf9b469b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71746a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ab46a7, 3 },
    { 0, 0 },
    { 0x812b46a9, 3 },
    { 0, 0 },
    { 0x94f46ab, 3 },
    { 0x589f46ac, 3 },
    { 0x1bb346ac, 3 },
    { 0x16f746ae, 3 },
    { 0xfc9346ae, 3 },
    { 0, 0 },
    { 0xa8ef46b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa346b6, 4 },
    { 0, 0 },
    { 0x96c746b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca746c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6b46c4, 3 },
    { 0x4dd346c5, 4 },
    { 0, 0 },
    { 0x23ab46c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef346ce, 3 },
    { 0xcbf46cf, 3 },
    { 0, 0 },
    { 0xd73f46d1, 3 },
    { 0, 0 },
    { 0xc2ff46d3, 4 },
    { 0x2fbf46d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21346d8, 3 },
    { 0x55ff46d9, 3 },
    { 0x5a2346d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35346de, 3 },
    { 0x2e9b46de, 3 },
    { 0x96a746e0, 3 },
    { 0, 0 },
    { 0xe6a746e2, 4 },
    { 0xdd2b46e3, 4 },
    { 0xa6f346e2, 3 },
    { 0x145b46e5, 4 },
    { 0x9d346e4, 3 },
    { 0x8ef746e3, 3 },
    { 0xd5bf46e8, 3 },
    { 0xf00f46e9, 3 },
    { 0x1ca346e8, 5 },
    { 0xa3bb46eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x503746ee, 3 },
    { 0xcdb346ef, 3 },
    { 0xbd3746f0, 4 },
    { 0, 0 },
    { 0xa48f46f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4346f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcf4701, 3 },
    { 0x8c74702, 3 },
    { 0xdaff4702, 4 },
    { 0x64fb4704, 3 },
    { 0xefb74704, 4 },
    { 0x9f934703, 3 },
    { 0x6efb4707, 3 },
    { 0x134708, 3 },
    { 0, 0 },
    { 0x56fb470a, 3 },
    { 0x7b33470b, 4 },
    { 0xa263470a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x415f470f, 3 },
    { 0x121f4710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f4716, 3 },
    { 0x24fb4717, 4 },
    { 0, 0 },
    { 0x9b2b4719, 3 },
    { 0x87cf471a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf4720, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13b4725, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a934728, 4 },
    { 0, 0 },
    { 0x1e6f472a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb34731, 3 },
    { 0x6df4732, 3 },
    { 0, 0 },
    { 0x6ea74734, 4 },
    { 0x35ff4734, 5 },
    { 0x599b4736, 4 },
    { 0x46c34735, 3 },
    { 0x266f4734, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb473b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e34741, 4 },
    { 0, 0 },
    { 0x59074743, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137f4748, 3 },
    { 0xf78f4748, 3 },
    { 0xa4bf474a, 4 },
    { 0x49a7474b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72b474e, 3 },
    { 0xce5f474f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49b475e, 4 },
    { 0x5663475f, 4 },
    { 0xfb334760, 3 },
    { 0x3c5f4761, 3 },
    { 0, 0 },
    { 0x67cf4763, 3 },
    { 0, 0 },
    { 0xa7534765, 3 },
    { 0, 0 },
    { 0xa1534767, 3 },
    { 0, 0 },
    { 0x523f4769, 3 },
    { 0x1f2b476a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc674777, 5 },
    { 0, 0 },
    { 0x4ee74779, 3 },
    { 0x806f477a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd3477d, 3 },
    { 0, 0 },
    { 0x76bf477f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8574785, 3 },
    { 0x451b4785, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18b4790, 3 },
    { 0xcd74790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2df4798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f479f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab47a2, 4 },
    { 0x3c3347a3, 4 },
    { 0xa2ef47a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d347a7, 3 },
    { 0, 0 },
    { 0x6b5347a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f47ae, 4 },
    { 0x907347ae, 3 },
    { 0x65ef47b0, 3 },
    { 0xc69347b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828f47b6, 3 },
    { 0xe1ff47b6, 3 },
    { 0x76c747b6, 3 },
    { 0x13e747b9, 5 },
    { 0x667747b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9747bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb347c2, 3 },
    { 0x47b747c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffb47c6, 3 },
    { 0xc23f47c6, 4 },
    { 0, 0 },
    { 0x2a3f47c9, 3 },
    { 0xeb7347c9, 3 },
    { 0xbe1347ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42347d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384347d6, 4 },
    { 0, 0 },
    { 0x333747d8, 4 },
    { 0x8e0f47d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae347df, 3 },
    { 0x994b47df, 4 },
    { 0x839b47e1, 3 },
    { 0xd2ab47e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6f47e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f47ec, 3 },
    { 0xeaaf47ed, 3 },
    { 0x852b47ed, 3 },
    { 0x615347ef, 3 },
    { 0x20747f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1747f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20f47ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91f4807, 4 },
    { 0, 0 },
    { 0x801f4809, 4 },
    { 0xd00f480a, 3 },
    { 0, 0 },
    { 0xe64f480c, 4 },
    { 0xda8b480c, 3 },
    { 0x877480e, 4 },
    { 0x9e4f480f, 3 },
    { 0xf97b4810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cf4813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3434817, 4 },
    { 0x7cf74818, 3 },
    { 0xf97b4819, 3 },
    { 0, 0 },
    { 0xea93481b, 3 },
    { 0xc627481c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0af481f, 3 },
    { 0, 0 },
    { 0x9c8b4821, 4 },
    { 0x918f4822, 3 },
    { 0xe3034822, 3 },
    { 0, 0 },
    { 0xd1334825, 4 },
    { 0, 0 },
    { 0xcd734827, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f482f, 3 },
    { 0xb5634830, 3 },
    { 0xefcf482f, 3 },
    { 0, 0 },
    { 0x7cb4833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d74836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffb483a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2f483d, 3 },
    { 0, 0 },
    { 0x9857483f, 3 },
    { 0, 0 },
    { 0x285b4841, 3 },
    { 0xedf74841, 3 },
    { 0x615b4841, 3 },
    { 0xaa174843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffb4848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7484f, 3 },
    { 0x7daf484f, 3 },
    { 0, 0 },
    { 0x21fb4852, 4 },
    { 0x504b4853, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf34858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba3485e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a34864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5b4867, 3 },
    { 0, 0 },
    { 0xacaf4869, 3 },
    { 0x9843486a, 3 },
    { 0xdd3b486b, 4 },
    { 0x35a7486b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134870, 3 },
    { 0, 0 },
    { 0x8d334872, 4 },
    { 0, 0 },
    { 0x62034874, 3 },
    { 0, 0 },
    { 0x13474876, 4 },
    { 0x225f4877, 3 },
    { 0, 0 },
    { 0x1b734879, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9703487d, 3 },
    { 0xa807487e, 4 },
    { 0x9947487f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79b4883, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x736f4886, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7488c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a74892, 3 },
    { 0xb7834893, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b489a, 5 },
    { 0, 0 },
    { 0xf223489c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a748a1, 3 },
    { 0, 0 },
    { 0x1d2f48a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x326748a6, 4 },
    { 0xecff48a7, 3 },
    { 0x10d748a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ff48ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946f48b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ff48c1, 3 },
    { 0x5c3748c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db348c5, 3 },
    { 0xb8d348c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x727748c9, 3 },
    { 0, 0 },
    { 0xef9f48cb, 3 },
    { 0x847748cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450b48d0, 3 },
    { 0, 0 },
    { 0x15db48d2, 3 },
    { 0xe17f48d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd748d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb48e2, 3 },
    { 0xd68b48e3, 3 },
    { 0xe4ff48e3, 3 },
    { 0x1d1748e5, 4 },
    { 0xdd4348e6, 3 },
    { 0x67a348e7, 3 },
    { 0x2a2748e8, 3 },
    { 0x5b4348e9, 4 },
    { 0x2d0f48e9, 3 },
    { 0x122b48e7, 3 },
    { 0x3f1b48e4, 3 },
    { 0xf66348ed, 3 },
    { 0x82b748ed, 3 },
    { 0x20e348ed, 4 },
    { 0xc12748e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9b48f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f48f8, 3 },
    { 0x55d748f8, 3 },
    { 0, 0 },
    { 0x27df48fb, 3 },
    { 0xeee748fc, 3 },
    { 0xc30748fd, 3 },
    { 0xe66b48fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf4904, 3 },
    { 0x40a74904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc243490b, 4 },
    { 0xc14f490c, 4 },
    { 0, 0 },
    { 0x33bb490e, 3 },
    { 0xc9a7490f, 4 },
    { 0xe2cb4910, 3 },
    { 0, 0 },
    { 0x50bb4912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a234915, 3 },
    { 0x649b4916, 3 },
    { 0x20ef4916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1f491c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1523491f, 5 },
    { 0xeecb491f, 3 },
    { 0xfa274921, 3 },
    { 0, 0 },
    { 0xcde34923, 3 },
    { 0x6abb4924, 3 },
    { 0, 0 },
    { 0xa18b4926, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30f4929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a7492e, 3 },
    { 0xbb47492e, 4 },
    { 0xbfdf492f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2674934, 3 },
    { 0, 0 },
    { 0x89534936, 3 },
    { 0x71f34937, 3 },
    { 0, 0 },
    { 0xc3834939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb493d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f74941, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58834945, 3 },
    { 0xfb2b4946, 3 },
    { 0x17f74945, 4 },
    { 0xb60b4948, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63494c, 4 },
    { 0x6e8b494c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f4950, 3 },
    { 0x11274950, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeb4956, 3 },
    { 0xe4b4957, 3 },
    { 0x74174958, 4 },
    { 0xa70f4957, 4 },
    { 0xdc5b4958, 3 },
    { 0x8d87495b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ff4960, 3 },
    { 0xe34f4961, 3 },
    { 0, 0 },
    { 0x4c334963, 3 },
    { 0xd2a74963, 4 },
    { 0xede74965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cf496a, 3 },
    { 0x8233496b, 3 },
    { 0x31f3496c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c74971, 3 },
    { 0x532b4972, 4 },
    { 0xd8034973, 3 },
    { 0x4d574974, 3 },
    { 0x62f74975, 3 },
    { 0x36134976, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1b497b, 3 },
    { 0xa1eb497c, 3 },
    { 0xf687497d, 3 },
    { 0x208b497e, 3 },
    { 0x3f8b497e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d234983, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb74986, 3 },
    { 0xe47f4987, 3 },
    { 0x5d2b4988, 3 },
    { 0xfa174989, 3 },
    { 0x5827498a, 3 },
    { 0x37ff4988, 3 },
    { 0xf5674987, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe74990, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed734994, 5 },
    { 0, 0 },
    { 0xc5c34996, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993499b, 3 },
    { 0x9ff499c, 3 },
    { 0x8757499b, 3 },
    { 0x22bf499e, 4 },
    { 0x4327499e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f49a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50b49a6, 3 },
    { 0x577349a7, 3 },
    { 0x80d749a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986349af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5b49b6, 3 },
    { 0x47e349b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12349bd, 3 },
    { 0x56ef49bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5f49c3, 3 },
    { 0, 0 },
    { 0x432f49c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93749c9, 3 },
    { 0x90cb49ca, 3 },
    { 0xac349cb, 4 },
    { 0x17e349cc, 3 },
    { 0x6bb349cd, 3 },
    { 0, 0 },
    { 0x9b2b49cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ef49d4, 5 },
    { 0, 0 },
    { 0x21b349d6, 3 },
    { 0xb36b49d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9b49da, 3 },
    { 0x64ab49db, 4 },
    { 0xb1d349dc, 3 },
    { 0x581b49db, 3 },
    { 0x6ec749de, 3 },
    { 0x83c749df, 3 },
    { 0x220749e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5b49e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a749e7, 4 },
    { 0, 0 },
    { 0xc84349e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b49ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814349f4, 3 },
    { 0x848b49f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca749fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261749fe, 4 },
    { 0xbebb49fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75df4a04, 3 },
    { 0xfd434a05, 3 },
    { 0, 0 },
    { 0x6ff34a07, 3 },
    { 0x99134a08, 3 },
    { 0x111f4a07, 4 },
    { 0x42f34a08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f34a0d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9334a10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb834a14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0f4a17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94134a1a, 3 },
    { 0x690f4a1b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e434a20, 3 },
    { 0x4f874a21, 4 },
    { 0x4f3b4a21, 3 },
    { 0x138f4a23, 3 },
    { 0xdbe34a23, 4 },
    { 0, 0 },
    { 0x20fb4a26, 4 },
    { 0, 0 },
    { 0xdaaf4a28, 3 },
    { 0x87974a28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591b4a34, 3 },
    { 0xc0734a35, 3 },
    { 0, 0 },
    { 0x6fcf4a37, 3 },
    { 0, 0 },
    { 0xdae74a39, 4 },
    { 0xb8334a39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f074a3f, 3 },
    { 0x9b5f4a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9b4a43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1b4a4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f4a51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a74a54, 3 },
    { 0, 0 },
    { 0xad434a56, 4 },
    { 0x9bab4a56, 4 },
    { 0, 0 },
    { 0x476b4a59, 5 },
    { 0x63db4a59, 3 },
    { 0x546b4a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf4a61, 4 },
    { 0x1c634a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26074a66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b374a69, 3 },
    { 0, 0 },
    { 0x841b4a6b, 3 },
    { 0x352f4a6b, 3 },
    { 0x9c174a6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a34a75, 3 },
    { 0xca274a75, 3 },
    { 0x16174a76, 4 },
    { 0xbecf4a78, 3 },
    { 0x4e1f4a79, 3 },
    { 0x5f0f4a79, 3 },
    { 0x5efb4a7a, 3 },
    { 0, 0 },
    { 0x88f4a7d, 4 },
    { 0xe134a7d, 3 },
    { 0, 0 },
    { 0x4bf74a80, 4 },
    { 0x864b4a81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ff4a84, 5 },
    { 0, 0 },
    { 0xe3c74a86, 3 },
    { 0x301b4a87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e334a8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c74a92, 3 },
    { 0x3a6b4a93, 3 },
    { 0x7d834a94, 4 },
    { 0x8c074a95, 4 },
    { 0xec134a92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2f4a9e, 3 },
    { 0, 0 },
    { 0xca74aa0, 3 },
    { 0, 0 },
    { 0xba834aa2, 3 },
    { 0xb75b4aa2, 4 },
    { 0xcf634aa3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b4aa7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291b4aab, 3 },
    { 0x533b4aac, 3 },
    { 0xe96f4aad, 3 },
    { 0xe2974aae, 3 },
    { 0x3534aad, 3 },
    { 0xa1e34ab0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10034ab3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84674ab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a374aba, 3 },
    { 0x3c274abb, 3 },
    { 0xf43f4aba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ac0, 3 },
    { 0x89b4ac1, 3 },
    { 0x39db4ac0, 3 },
    { 0x4f374ac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909b4ad0, 4 },
    { 0, 0 },
    { 0xf04b4ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f4ad6, 4 },
    { 0, 0 },
    { 0xa7fb4ad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13034adc, 3 },
    { 0, 0 },
    { 0xa05b4ade, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46934ae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f4ae6, 3 },
    { 0x15b34ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdf4af5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15b4af8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8f4afd, 3 },
    { 0xf85b4afe, 3 },
    { 0x32eb4afd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef4b04, 4 },
    { 0, 0 },
    { 0xf1234b06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f4b10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f74b1b, 3 },
    { 0x4bef4b1b, 3 },
    { 0x1d534b1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19374b22, 4 },
    { 0x42b74b23, 4 },
    { 0x74434b24, 3 },
    { 0x98af4b25, 4 },
    { 0x1d4f4b26, 3 },
    { 0xa7574b22, 3 },
    { 0xfa4b4b28, 3 },
    { 0x2b6b4b28, 3 },
    { 0, 0 },
    { 0x2d974b2b, 4 },
    { 0x4f6b4b2c, 5 },
    { 0x6fdf4b2c, 3 },
    { 0xf9274b2b, 4 },
    { 0x1f274b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bf4b32, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670b4b38, 4 },
    { 0x18b4b39, 4 },
    { 0x29f4b3a, 3 },
    { 0xb2df4b3a, 5 },
    { 0x57034b3c, 4 },
    { 0xdcbb4b3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24474b40, 3 },
    { 0, 0 },
    { 0xe63b4b42, 4 },
    { 0xb2a74b42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ab4b48, 3 },
    { 0x15274b49, 4 },
    { 0xa1934b4a, 3 },
    { 0x940b4b4a, 4 },
    { 0x30234b4a, 4 },
    { 0xb6e74b4c, 4 },
    { 0x4d74b4b, 3 },
    { 0xedcf4b4f, 4 },
    { 0, 0 },
    { 0x913f4b51, 3 },
    { 0, 0 },
    { 0xde734b53, 3 },
    { 0, 0 },
    { 0x8534b55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x200b4b58, 4 },
    { 0xd04f4b59, 4 },
    { 0x44074b5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747f4b62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10034b6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f4b75, 3 },
    { 0x78a34b76, 4 },
    { 0x79f74b76, 3 },
    { 0xca274b78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffb4b7d, 4 },
    { 0, 0 },
    { 0x658f4b7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26334b82, 3 },
    { 0xebf4b83, 3 },
    { 0x584f4b83, 3 },
    { 0x79734b85, 3 },
    { 0xcb234b85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9f4b8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594b4b91, 3 },
    { 0, 0 },
    { 0x5d134b93, 3 },
    { 0x9bb4b94, 4 },
    { 0x9b974b95, 4 },
    { 0xd47f4b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f34b9e, 3 },
    { 0, 0 },
    { 0x58db4ba0, 3 },
    { 0xe0d74ba0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e74ba5, 4 },
    { 0xe89b4ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35f4ba9, 3 },
    { 0, 0 },
    { 0x69cb4bab, 3 },
    { 0, 0 },
    { 0x92d74bad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x875f4bb0, 4 },
    { 0, 0 },
    { 0xcf4b4bb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83f4bb6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f4bb9, 3 },
    { 0, 0 },
    { 0xcddf4bbb, 3 },
    { 0x360f4bbc, 4 },
    { 0xaddb4bbc, 3 },
    { 0x31cb4bbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f4bc1, 5 },
    { 0, 0 },
    { 0xd14f4bc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64f4bc8, 3 },
    { 0x33ff4bc9, 3 },
    { 0x92a74bca, 3 },
    { 0xbfeb4bcb, 4 },
    { 0, 0 },
    { 0xce1b4bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f74bd0, 3 },
    { 0x26334bd1, 3 },
    { 0x3b934bd2, 3 },
    { 0, 0 },
    { 0x3daf4bd4, 3 },
    { 0xd3a34bd5, 3 },
    { 0x7e234bd6, 4 },
    { 0x86ff4bd7, 3 },
    { 0xa57f4bd8, 3 },
    { 0xdef4bd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b4bdf, 3 },
    { 0xd3274be0, 4 },
    { 0x40574bdf, 4 },
    { 0x467f4be1, 3 },
    { 0, 0 },
    { 0x59c74be4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x94734be7, 4 },
    { 0, 0 },
    { 0xb4934be9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4f4bf0, 3 },
    { 0xf24f4bf1, 3 },
    { 0xb7bf4bf2, 4 },
    { 0x3b034bf3, 3 },
    { 0xd70f4bf2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95334bf7, 3 },
    { 0x9a534bf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4834bfd, 3 },
    { 0xf13f4bfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4b4c03, 4 },
    { 0x16434c04, 3 },
    { 0x7f274c03, 3 },
    { 0xd4a74c04, 3 },
    { 0x599b4c04, 4 },
    { 0xbbdb4c08, 3 },
    { 0x308f4c09, 3 },
    { 0x34674c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4674c0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3b4c13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1f4c1f, 3 },
    { 0x5e4f4c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d034c24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaff4c28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf4c2d, 3 },
    { 0x42af4c2d, 3 },
    { 0, 0 },
    { 0x9de34c30, 3 },
    { 0, 0 },
    { 0x2cb74c32, 3 },
    { 0, 0 },
    { 0x69b34c34, 3 },
    { 0x71334c34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf134c38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc34c3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f4c47, 4 },
    { 0, 0 },
    { 0xc974c49, 3 },
    { 0, 0 },
    { 0x63474c4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5334c4f, 4 },
    { 0x21f4c50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31334c57, 3 },
    { 0xc4eb4c58, 3 },
    { 0, 0 },
    { 0x39ff4c5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f4c5e, 3 },
    { 0x811b4c5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x614f4c62, 3 },
    { 0, 0 },
    { 0x3b734c64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bb4c6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66b4c7a, 3 },
    { 0xd3cb4c7b, 3 },
    { 0x99374c7c, 4 },
    { 0x629f4c7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8f4c81, 3 },
    { 0xe434c82, 3 },
    { 0x93ff4c83, 3 },
    { 0xfbcb4c84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x504b4c87, 4 },
    { 0x948f4c88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7f4c8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b774c98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcb4c9d, 3 },
    { 0, 0 },
    { 0x27bf4c9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88b4ca4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be34ca8, 4 },
    { 0, 0 },
    { 0xa22b4caa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fb4cad, 3 },
    { 0x82374cad, 4 },
    { 0, 0 },
    { 0x369f4cb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4734cb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ab4cb6, 3 },
    { 0xa4974cb7, 3 },
    { 0x20a34cb8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66b4cbc, 3 },
    { 0xe99f4cbd, 3 },
    { 0x1a034cbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d74cc5, 5 },
    { 0, 0 },
    { 0x627b4cc7, 3 },
    { 0x4f8b4cc8, 3 },
    { 0, 0 },
    { 0xdfd34cca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x987f4ccd, 3 },
    { 0, 0 },
    { 0xd28b4ccf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc74cd2, 4 },
    { 0, 0 },
    { 0xa93b4cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df4cd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb34cdd, 3 },
    { 0xf8674cdd, 3 },
    { 0x9bdb4cdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8f4ce2, 4 },
    { 0x681f4ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f34cea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d74ced, 3 },
    { 0, 0 },
    { 0x92734cef, 4 },
    { 0, 0 },
    { 0x68f34cf1, 3 },
    { 0, 0 },
    { 0x63134cf3, 3 },
    { 0xd6234cf3, 3 },
    { 0, 0 },
    { 0x2374cf6, 3 },
    { 0xaf74cf6, 3 },
    { 0xde1b4cf8, 4 },
    { 0xccd74cf9, 3 },
    { 0xa3334cf7, 4 },
    { 0xf7ff4cf8, 3 },
    { 0x86b34cf8, 4 },
    { 0xdba34cfd, 3 },
    { 0xae634cf9, 3 },
    { 0x7fa74cfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d774d03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acb4d09, 3 },
    { 0x73074d0a, 3 },
    { 0xc72f4d0b, 3 },
    { 0xef674d0a, 3 },
    { 0xf6ff4d0c, 3 },
    { 0x58874d0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48f4d18, 4 },
    { 0, 0 },
    { 0xbc134d1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58af4d1e, 4 },
    { 0x474d1e, 3 },
    { 0x54eb4d20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778b4d25, 3 },
    { 0x3a974d26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef4d29, 4 },
    { 0x2874d2a, 3 },
    { 0xe1734d2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ab4d34, 3 },
    { 0x7e374d34, 3 },
    { 0x392f4d36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8674d39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb4d41, 5 },
    { 0xedcf4d42, 3 },
    { 0, 0 },
    { 0xb57b4d44, 4 },
    { 0xb9f4d44, 4 },
    { 0x858b4d45, 3 },
    { 0x716b4d47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc74d4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90834d51, 4 },
    { 0, 0 },
    { 0x4bbb4d53, 3 },
    { 0x6f6b4d54, 3 },
    { 0x533b4d55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4b4d60, 3 },
    { 0x83cb4d60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f634d66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x469b4d69, 3 },
    { 0x20fb4d6a, 3 },
    { 0xb88f4d69, 3 },
    { 0, 0 },
    { 0xe3cf4d6d, 3 },
    { 0, 0 },
    { 0xcff74d6f, 4 },
    { 0x637b4d70, 3 },
    { 0x19c74d71, 3 },
    { 0xfff4d6f, 3 },
    { 0x30a74d73, 3 },
    { 0xe0ef4d74, 3 },
    { 0, 0 },
    { 0x45eb4d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb74d80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f34d83, 3 },
    { 0x5ef74d84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0074d88, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b4d8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61eb4d8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e74d92, 3 },
    { 0xc374d93, 4 },
    { 0xf6df4d94, 3 },
    { 0xa4ff4d94, 3 },
    { 0, 0 },
    { 0x7c0f4d97, 3 },
    { 0xd16b4d97, 3 },
    { 0, 0 },
    { 0x92af4d9a, 3 },
    { 0, 0 },
    { 0x98634d9c, 3 },
    { 0x20eb4d9c, 4 },
    { 0, 0 },
    { 0x2f874d9f, 4 },
    { 0, 0 },
    { 0x66174da1, 4 },
    { 0x3e434da2, 3 },
    { 0x11af4da1, 3 },
    { 0, 0 },
    { 0xa4774da5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f74da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f4dad, 4 },
    { 0x5b5b4dae, 3 },
    { 0x402b4daf, 3 },
    { 0, 0 },
    { 0xbe474db1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f4db4, 4 },
    { 0x4e774db5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eb4db8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e574dbd, 3 },
    { 0xe7a74dbd, 3 },
    { 0xc6e34dbf, 3 },
    { 0x9f134dbf, 3 },
    { 0x358b4dc1, 4 },
    { 0x45e74dc1, 3 },
    { 0x51134dc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55774dc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694f4dd2, 3 },
    { 0x436f4dd2, 3 },
    { 0xd6934dd4, 3 },
    { 0xad034dd5, 3 },
    { 0x3e9b4dd5, 4 },
    { 0x32074dd7, 3 },
    { 0x4eeb4dd5, 3 },
    { 0x9e774dd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x874f4ddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a34dea, 3 },
    { 0, 0 },
    { 0xf30b4dec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0574def, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b74dfd, 4 },
    { 0x7e874dfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84934e02, 3 },
    { 0xdf734e03, 3 },
    { 0x251b4e04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb034e07, 3 },
    { 0x5d174e08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb4e0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f4e0e, 3 },
    { 0x6ddf4e0f, 3 },
    { 0, 0 },
    { 0x2caf4e11, 4 },
    { 0xbd6f4e12, 3 },
    { 0x151b4e12, 4 },
    { 0x4a2f4e14, 3 },
    { 0x8e74e15, 3 },
    { 0xfed74e16, 3 },
    { 0xa63b4e15, 4 },
    { 0x9d5f4e13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac74e1b, 3 },
    { 0, 0 },
    { 0x28f34e1d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fb4e24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0b4e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1b4e2d, 3 },
    { 0, 0 },
    { 0x8b674e2f, 3 },
    { 0x9df34e2f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf74e33, 3 },
    { 0, 0 },
    { 0x77834e35, 3 },
    { 0, 0 },
    { 0xd3974e37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3634e41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e174e44, 3 },
    { 0xd44f4e45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a34e4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a74e4f, 3 },
    { 0x1b4b4e50, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fb4e53, 3 },
    { 0, 0 },
    { 0xaeeb4e55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d74e59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca74e5e, 3 },
    { 0xd34f4e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bb4e63, 3 },
    { 0, 0 },
    { 0x51ab4e65, 3 },
    { 0, 0 },
    { 0x840b4e67, 4 },
    { 0x91cb4e67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66334e70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f874e74, 4 },
    { 0x811f4e75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54634e79, 4 },
    { 0x270b4e79, 3 },
    { 0, 0 },
    { 0x63934e7c, 3 },
    { 0x1ebf4e7d, 5 },
    { 0, 0 },
    { 0xc6e34e7f, 3 },
    { 0, 0 },
    { 0xaa4b4e81, 3 },
    { 0x4dd74e81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30374e85, 4 },
    { 0x683b4e86, 3 },
    { 0x62f34e86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd034e8b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13674e92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f4e95, 3 },
    { 0, 0 },
    { 0x9a974e97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x203f4e9a, 3 },
    { 0x38e74e9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1f4e9f, 3 },
    { 0xec334ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2974ea5, 3 },
    { 0x24034ea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x837f4ea9, 3 },
    { 0x45c34eaa, 3 },
    { 0, 0 },
    { 0x4ddf4eac, 3 },
    { 0x2fb74ead, 3 },
    { 0xe97f4eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc574eb2, 4 },
    { 0, 0 },
    { 0x6d734eb4, 3 },
    { 0, 0 },
    { 0x2bf4eb6, 5 },
    { 0x619b4eb7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a634ebc, 3 },
    { 0, 0 },
    { 0x71874ebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd34ec4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b74ec8, 3 },
    { 0xa5034ec9, 3 },
    { 0, 0 },
    { 0x8cb4ecb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480b4ed1, 3 },
    { 0x1c874ed2, 3 },
    { 0x65d74ed3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b274ed7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f4edd, 3 },
    { 0x6a834ede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa34ee4, 4 },
    { 0, 0 },
    { 0x9234ee6, 3 },
    { 0, 0 },
    { 0xdae34ee8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb4eee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d34ef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3b4ef6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35274efc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4234f04, 3 },
    { 0x72e34f05, 4 },
    { 0x834f4f06, 4 },
    { 0, 0 },
    { 0xdf834f08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfb4f0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1534f13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef4f17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4474f1f, 3 },
    { 0x74f1f, 3 },
    { 0xd1e34f21, 4 },
    { 0x19734f21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29274f25, 3 },
    { 0x1e1f4f26, 3 },
    { 0x975f4f27, 3 },
    { 0x4ef34f28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb974f2f, 4 },
    { 0, 0 },
    { 0x4f534f31, 3 },
    { 0xc99f4f31, 3 },
    { 0, 0 },
    { 0xa8874f34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x487f4f37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a34f3a, 3 },
    { 0, 0 },
    { 0x348f4f3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0474f3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9834f42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e74f45, 3 },
    { 0x86f34f46, 3 },
    { 0x6fbf4f45, 5 },
    { 0xbca34f48, 3 },
    { 0x61734f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff4f4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff134f4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e234f53, 4 },
    { 0x50434f54, 3 },
    { 0xc41b4f54, 3 },
    { 0xd1534f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b4f5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479f4f62, 4 },
    { 0xf4674f63, 3 },
    { 0x1cc34f64, 4 },
    { 0x44ff4f65, 3 },
    { 0xd23b4f62, 3 },
    { 0x92ab4f67, 3 },
    { 0x3bb34f68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be34f6e, 3 },
    { 0, 0 },
    { 0x19b4f70, 4 },
    { 0x76174f71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf74f78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea74f81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78b4f88, 3 },
    { 0x28af4f89, 4 },
    { 0xdfb74f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30f4f8e, 3 },
    { 0xca6b4f8f, 5 },
    { 0, 0 },
    { 0xbe6b4f91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689f4f95, 4 },
    { 0, 0 },
    { 0x4cd34f97, 3 },
    { 0x6b274f98, 3 },
    { 0x298f4f99, 4 },
    { 0x98cb4f9a, 4 },
    { 0xc4bb4f9b, 3 },
    { 0x798f4f99, 3 },
    { 0x886f4f9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc134fa0, 4 },
    { 0x3a974fa1, 3 },
    { 0x73834fa2, 3 },
    { 0x3c674fa3, 4 },
    { 0xcc0f4fa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b4fa8, 3 },
    { 0xa2774fa8, 4 },
    { 0xf7674fa9, 3 },
    { 0x3ecf4faa, 3 },
    { 0x16f4fac, 3 },
    { 0x75474fad, 3 },
    { 0x87374fae, 3 },
    { 0, 0 },
    { 0x984b4fb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf734fbd, 3 },
    { 0x2e034fbe, 3 },
    { 0x68574fbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70974fc5, 3 },
    { 0xb11f4fc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f34fc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fb4fcc, 3 },
    { 0x8b274fcd, 3 },
    { 0x37034fcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e74fd2, 4 },
    { 0, 0 },
    { 0x6c3b4fd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480b4fd9, 3 },
    { 0x7bb4fda, 3 },
    { 0xba174fdb, 3 },
    { 0x4f234fdb, 3 },
    { 0, 0 },
    { 0xbdbf4fde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1134fe1, 3 },
    { 0x5e574fe1, 3 },
    { 0, 0 },
    { 0xd6b4fe4, 3 },
    { 0, 0 },
    { 0x17bf4fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12df4fe9, 3 },
    { 0xb9f34fe9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e774fef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29b4ff3, 4 },
    { 0, 0 },
    { 0x17ab4ff5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f4ffd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98935002, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e35006, 4 },
    { 0xfd6b5007, 3 },
    { 0x28775008, 3 },
    { 0xef7f5008, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3797500c, 3 },
    { 0, 0 },
    { 0x8f8f500e, 3 },
    { 0xc267500f, 4 },
    { 0, 0 },
    { 0xe56f5011, 4 },
    { 0xfe0b5011, 3 },
    { 0x4bab5013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30175018, 3 },
    { 0xa9c75019, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348f5021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b3502a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7502e, 3 },
    { 0xd427502f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66f5032, 3 },
    { 0xb98b5032, 4 },
    { 0, 0 },
    { 0x93cb5035, 3 },
    { 0x906f5035, 5 },
    { 0x6e6f5036, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c7503c, 3 },
    { 0x52df503c, 5 },
    { 0xf447503e, 3 },
    { 0xd9cf503f, 3 },
    { 0xad4b503f, 3 },
    { 0xcf57503e, 3 },
    { 0xdc6f5042, 3 },
    { 0, 0 },
    { 0x94b35044, 4 },
    { 0, 0 },
    { 0xab1f5046, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7504a, 3 },
    { 0x2657504b, 3 },
    { 0xdcab504c, 3 },
    { 0xfeab504b, 3 },
    { 0xee5f504d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c775058, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d7505b, 3 },
    { 0xe2d7505c, 4 },
    { 0x509b505c, 3 },
    { 0xff1b505b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05b5061, 4 },
    { 0xb4fb5062, 3 },
    { 0x915f5063, 3 },
    { 0, 0 },
    { 0x116b5065, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c3506a, 3 },
    { 0, 0 },
    { 0xc587506c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6027506f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5275075, 3 },
    { 0xf8775076, 3 },
    { 0xc5d75077, 4 },
    { 0xa1675076, 3 },
    { 0, 0 },
    { 0x8fdf507a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6f507d, 3 },
    { 0xd9a3507e, 5 },
    { 0xa0db507e, 3 },
    { 0x47475080, 3 },
    { 0x86435081, 3 },
    { 0, 0 },
    { 0xdc8f5083, 3 },
    { 0xcd4b5083, 3 },
    { 0, 0 },
    { 0xd3335086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ef5094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb527509a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3509e, 4 },
    { 0x828b509f, 3 },
    { 0x1ce7509f, 4 },
    { 0x9f7b50a1, 3 },
    { 0x55b350a1, 3 },
    { 0xb2b3509e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44b50a7, 3 },
    { 0x3f8b50a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb50ac, 3 },
    { 0x404f50ad, 3 },
    { 0xb45750ac, 3 },
    { 0xeb1350af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f50b3, 3 },
    { 0, 0 },
    { 0xa2d750b5, 3 },
    { 0, 0 },
    { 0xf95750b7, 3 },
    { 0xf08b50b8, 3 },
    { 0xb34350b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b50c2, 3 },
    { 0, 0 },
    { 0x965b50c4, 3 },
    { 0x529750c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4750c8, 5 },
    { 0x20df50c8, 3 },
    { 0x6e9750c9, 4 },
    { 0xd34b50cb, 3 },
    { 0xa7350cb, 3 },
    { 0xbca750cd, 4 },
    { 0x974750ce, 3 },
    { 0xe3d350cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300350d8, 3 },
    { 0xccdf50d9, 3 },
    { 0, 0 },
    { 0x7f5350db, 3 },
    { 0x201b50dc, 3 },
    { 0x374f50db, 3 },
    { 0xe57350de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d750e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b50e6, 3 },
    { 0x6e9350e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaf50eb, 3 },
    { 0, 0 },
    { 0x798f50ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652b50f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb50fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf5108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6763510c, 4 },
    { 0x65d3510d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff5111, 3 },
    { 0, 0 },
    { 0xea975113, 3 },
    { 0x628b5113, 3 },
    { 0x15775115, 3 },
    { 0x3375115, 3 },
    { 0xb2ef5115, 3 },
    { 0xbf035114, 4 },
    { 0x86575119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc35120, 3 },
    { 0, 0 },
    { 0xea475122, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb5125, 3 },
    { 0, 0 },
    { 0x1f9b5127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x692f512a, 4 },
    { 0xa05f512a, 3 },
    { 0x281f512b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b512f, 3 },
    { 0xbbfb5130, 3 },
    { 0xc6fb512f, 3 },
    { 0xe5ef5132, 4 },
    { 0x87b35132, 4 },
    { 0x99935134, 3 },
    { 0x3c735133, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0835138, 3 },
    { 0, 0 },
    { 0xd5ef513a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1b513d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3275142, 3 },
    { 0, 0 },
    { 0xfe8f5144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e7514b, 3 },
    { 0xc0cb514c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f35150, 3 },
    { 0xa5c35151, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad575157, 3 },
    { 0x421b5157, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3515b, 3 },
    { 0x9057515b, 3 },
    { 0x304f515d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fff5165, 3 },
    { 0xa6775166, 4 },
    { 0, 0 },
    { 0x9d075168, 3 },
    { 0x28775169, 3 },
    { 0x907516a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a975171, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2f5176, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe853517e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca175182, 4 },
    { 0, 0 },
    { 0x74c35184, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ff5189, 3 },
    { 0xf26f518a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c675194, 3 },
    { 0, 0 },
    { 0x99875196, 3 },
    { 0x438f5197, 3 },
    { 0xc20b5198, 3 },
    { 0x1535199, 3 },
    { 0x142b519a, 3 },
    { 0, 0 },
    { 0x9be7519c, 3 },
    { 0, 0 },
    { 0x5367519e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab351a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6751a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6751ab, 3 },
    { 0x727351ac, 3 },
    { 0x491f51ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3751b0, 3 },
    { 0xa7f351b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1f51ba, 4 },
    { 0xe67751bb, 4 },
    { 0x4bc751ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b751c3, 4 },
    { 0x7ab51c4, 4 },
    { 0xde0f51c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baf51c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daf51ce, 3 },
    { 0x20cb51cf, 3 },
    { 0xf5bf51d0, 3 },
    { 0, 0 },
    { 0xceb751d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74751d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5351d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bf51dc, 4 },
    { 0, 0 },
    { 0xf2ab51de, 3 },
    { 0x82a351df, 5 },
    { 0x6bdb51df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248751e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22b51ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a751f9, 3 },
    { 0xf69751fa, 3 },
    { 0xf29351fb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48b520d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d35210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6675216, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e35219, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3521d, 3 },
    { 0, 0 },
    { 0x51ef521f, 3 },
    { 0, 0 },
    { 0x6c435221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f522a, 4 },
    { 0, 0 },
    { 0x27a7522c, 3 },
    { 0x169f522d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f675230, 3 },
    { 0x88835231, 3 },
    { 0xeb235230, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6b5238, 4 },
    { 0x43f35239, 3 },
    { 0, 0 },
    { 0xf1ef523b, 3 },
    { 0xa793523b, 3 },
    { 0x6d7523d, 3 },
    { 0x568f523e, 3 },
    { 0, 0 },
    { 0x965f5240, 3 },
    { 0xb0df5240, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18235247, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f93524a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e3524d, 3 },
    { 0xcd97524e, 3 },
    { 0xb4bf524f, 3 },
    { 0x96cf524f, 5 },
    { 0x81e75251, 3 },
    { 0xd7f75252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c35255, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f5258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676b525d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd775263, 4 },
    { 0, 0 },
    { 0x5e275265, 3 },
    { 0, 0 },
    { 0x14535267, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767526a, 3 },
    { 0xe867526b, 3 },
    { 0xbbab526c, 4 },
    { 0x170b526d, 3 },
    { 0x878b526d, 3 },
    { 0, 0 },
    { 0x8a2f5270, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b5273, 4 },
    { 0x433f5273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4757527c, 3 },
    { 0x423f527c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ef5280, 3 },
    { 0xecd35281, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9675285, 3 },
    { 0xe3335286, 3 },
    { 0x48c35285, 4 },
    { 0, 0 },
    { 0x50535289, 4 },
    { 0, 0 },
    { 0x552b528b, 3 },
    { 0x30eb528c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f7528f, 3 },
    { 0x45eb5290, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f875293, 3 },
    { 0x4fab5293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757b52a2, 3 },
    { 0, 0 },
    { 0x993352a4, 3 },
    { 0, 0 },
    { 0xd53f52a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fb52ad, 3 },
    { 0xf7b752ae, 3 },
    { 0xcfb752af, 4 },
    { 0xa4752b0, 4 },
    { 0xf13f52b1, 3 },
    { 0x98bf52b1, 3 },
    { 0x750752b1, 5 },
    { 0, 0 },
    { 0x23cb52b5, 4 },
    { 0x948b52b5, 3 },
    { 0xd59b52b7, 5 },
    { 0xc2f752b8, 3 },
    { 0, 0 },
    { 0x309f52ba, 3 },
    { 0x9b4752bb, 3 },
    { 0x625752bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f52bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdf52c2, 3 },
    { 0x4e4b52c3, 3 },
    { 0x7e8b52c2, 4 },
    { 0, 0 },
    { 0x309352c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3b52cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3b52ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00f52d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f52d7, 3 },
    { 0, 0 },
    { 0xb03f52d9, 3 },
    { 0, 0 },
    { 0xf25752db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770f52e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x904752e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218b52e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605f52ed, 3 },
    { 0x13db52ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95752f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8af52f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df352fc, 3 },
    { 0, 0 },
    { 0xa80b52fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27530b, 4 },
    { 0x21ef530c, 3 },
    { 0, 0 },
    { 0xba0f530e, 3 },
    { 0xdef530f, 3 },
    { 0x8cdb530e, 3 },
    { 0x2163530e, 3 },
    { 0, 0 },
    { 0x3af35313, 3 },
    { 0xfe735314, 3 },
    { 0, 0 },
    { 0x76375316, 3 },
    { 0, 0 },
    { 0x221f5318, 4 },
    { 0x91c35319, 3 },
    { 0xbc3f5318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b531d, 3 },
    { 0, 0 },
    { 0xa47f531f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf5323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4835328, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6635332, 4 },
    { 0xf8875333, 3 },
    { 0, 0 },
    { 0x24df5335, 3 },
    { 0, 0 },
    { 0xe46b5337, 4 },
    { 0, 0 },
    { 0x96875339, 3 },
    { 0x5327533a, 3 },
    { 0x23c7533b, 3 },
    { 0x7d07533a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75475341, 3 },
    { 0x95075342, 3 },
    { 0xb27b5343, 3 },
    { 0x52275342, 3 },
    { 0x424b5345, 3 },
    { 0, 0 },
    { 0x6a7b5347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c27534c, 3 },
    { 0xe09f534c, 4 },
    { 0x48df534e, 3 },
    { 0xb65b534e, 4 },
    { 0x804b534e, 3 },
    { 0xe1cf5350, 3 },
    { 0x4d1b5352, 5 },
    { 0x370f5353, 3 },
    { 0x24775354, 3 },
    { 0x4cb5355, 4 },
    { 0xc0c75356, 3 },
    { 0xd475352, 3 },
    { 0xd8675355, 3 },
    { 0, 0 },
    { 0x61eb535a, 3 },
    { 0x7e97535b, 3 },
    { 0x6f17535c, 3 },
    { 0xc4cb535c, 4 },
    { 0x54cb535d, 3 },
    { 0x409f535d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af5368, 3 },
    { 0x5c935369, 3 },
    { 0, 0 },
    { 0xb3e3536b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e7536e, 3 },
    { 0, 0 },
    { 0x61535370, 3 },
    { 0, 0 },
    { 0x2cbf5372, 3 },
    { 0x78535372, 4 },
    { 0x5c375374, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9435377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b5381, 3 },
    { 0x9bd75382, 4 },
    { 0xd5835381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba235389, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241b5390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19b5393, 3 },
    { 0xd6ef5394, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb375397, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f3539a, 3 },
    { 0xdab539a, 3 },
    { 0xa64b539b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x422b539f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d353a2, 3 },
    { 0xaeff53a2, 3 },
    { 0x38cb53a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x322353a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c753ae, 3 },
    { 0xf34353ae, 4 },
    { 0, 0 },
    { 0xd77b53b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55353b8, 4 },
    { 0x956f53b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f53bc, 3 },
    { 0x770353bd, 3 },
    { 0x3ef53bd, 3 },
    { 0, 0 },
    { 0x7ecf53c0, 3 },
    { 0xe51753c0, 3 },
    { 0x84bb53c2, 3 },
    { 0x619f53c2, 3 },
    { 0xd18353c3, 3 },
    { 0x116b53c5, 4 },
    { 0xa9ff53c3, 4 },
    { 0x10f353c1, 3 },
    { 0x9a8353c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201353cd, 4 },
    { 0x907753ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf753d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba753d6, 3 },
    { 0x5ab353d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4353dc, 3 },
    { 0x60eb53dd, 3 },
    { 0x338f53de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582753e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3353e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bb53ee, 4 },
    { 0xe3d353ef, 3 },
    { 0xca5f53f0, 3 },
    { 0x885f53f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3b53f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c353fa, 4 },
    { 0x793b53fa, 3 },
    { 0xb35b53fc, 3 },
    { 0, 0 },
    { 0x669353fe, 3 },
    { 0xa48f53ff, 3 },
    { 0, 0 },
    { 0x9d1f5401, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3b5404, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8675408, 4 },
    { 0xd2475408, 3 },
    { 0x827f540a, 3 },
    { 0xfe7b540a, 4 },
    { 0xa9c3540c, 3 },
    { 0x3a5f540b, 4 },
    { 0xccbb5408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf5414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b5417, 3 },
    { 0, 0 },
    { 0xbdc35419, 3 },
    { 0, 0 },
    { 0xb143541b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2443541e, 4 },
    { 0xca47541f, 3 },
    { 0, 0 },
    { 0xd7e75421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea75424, 5 },
    { 0x12c35425, 3 },
    { 0x88f75426, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa683542a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61375431, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733b5437, 3 },
    { 0x3a135438, 3 },
    { 0, 0 },
    { 0xdefb543a, 3 },
    { 0xb6d3543b, 3 },
    { 0x58d3543a, 5 },
    { 0, 0 },
    { 0x287543e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eb5441, 3 },
    { 0x71bf5442, 3 },
    { 0, 0 },
    { 0x7bcf5444, 3 },
    { 0, 0 },
    { 0x826b5446, 4 },
    { 0x7fa75447, 3 },
    { 0xdb835446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e07544b, 3 },
    { 0, 0 },
    { 0x8e77544d, 3 },
    { 0, 0 },
    { 0x2a23544f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf35455, 3 },
    { 0x74675455, 3 },
    { 0xe375455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2f545a, 3 },
    { 0x91b3545b, 3 },
    { 0x19c3545c, 3 },
    { 0, 0 },
    { 0xff97545e, 3 },
    { 0xae43545f, 3 },
    { 0x7a6f5460, 3 },
    { 0xc84f5460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40875466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f546a, 3 },
    { 0x5e7b546b, 4 },
    { 0x4da7546a, 3 },
    { 0x6ecf546d, 4 },
    { 0, 0 },
    { 0x40d3546f, 3 },
    { 0x65f3546f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x888b5473, 3 },
    { 0, 0 },
    { 0xde735475, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b547c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f547f, 3 },
    { 0, 0 },
    { 0xd0435481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8f5484, 3 },
    { 0xe24b5484, 3 },
    { 0xfdcb5486, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff7548a, 4 },
    { 0x9adf548b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69f5494, 3 },
    { 0x2f035495, 4 },
    { 0xf2075496, 4 },
    { 0x313b5495, 4 },
    { 0, 0 },
    { 0xfc8b5499, 3 },
    { 0, 0 },
    { 0x70ff549b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462b54a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f54b0, 4 },
    { 0x7d9354b0, 4 },
    { 0x1dfb54b2, 3 },
    { 0x743b54b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55754b7, 3 },
    { 0x4c8754b8, 3 },
    { 0, 0 },
    { 0x5f8354ba, 4 },
    { 0x55ef54ba, 3 },
    { 0xd01b54bc, 4 },
    { 0x8b5b54bb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcb54c5, 3 },
    { 0, 0 },
    { 0x7a754c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5354cc, 3 },
    { 0xe80f54cd, 4 },
    { 0, 0 },
    { 0xc73f54cf, 3 },
    { 0xb92754cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebf54d4, 3 },
    { 0, 0 },
    { 0x927b54d6, 4 },
    { 0x622354d7, 3 },
    { 0xf8c354d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbf54de, 3 },
    { 0x1ec754de, 3 },
    { 0xbccf54e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43354e3, 4 },
    { 0, 0 },
    { 0x239b54e5, 3 },
    { 0, 0 },
    { 0xca7754e7, 3 },
    { 0x1fe354e8, 4 },
    { 0x3bd754e8, 3 },
    { 0x256f54e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cb54f2, 3 },
    { 0x681b54f3, 4 },
    { 0x1f0754f4, 4 },
    { 0, 0 },
    { 0xc3db54f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b54f9, 3 },
    { 0xc92754f9, 3 },
    { 0xb27754fb, 3 },
    { 0, 0 },
    { 0x4a9f54fd, 4 },
    { 0xa51354fd, 3 },
    { 0, 0 },
    { 0xc1375500, 4 },
    { 0xedcb5500, 3 },
    { 0xd20b5501, 3 },
    { 0x651b5503, 3 },
    { 0x1d6b5504, 4 },
    { 0x272b5504, 3 },
    { 0x1b2f5506, 4 },
    { 0, 0 },
    { 0xb14b5508, 3 },
    { 0xe4775509, 3 },
    { 0xed8f550a, 3 },
    { 0xe967550a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f5510, 3 },
    { 0xe9e35511, 4 },
    { 0, 0 },
    { 0xf7975513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bb5516, 4 },
    { 0x490b5517, 3 },
    { 0xc5cf5518, 3 },
    { 0xcebf5519, 3 },
    { 0xcc6b551a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5b551e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2b5523, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c35527, 3 },
    { 0xe01f5528, 3 },
    { 0x92035529, 3 },
    { 0x78bb552a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85b552d, 4 },
    { 0x8777552e, 3 },
    { 0x57d7552f, 3 },
    { 0xd2035530, 3 },
    { 0x6da7552e, 3 },
    { 0xea3b552d, 3 },
    { 0, 0 },
    { 0x55db5534, 3 },
    { 0, 0 },
    { 0xd7775536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e775539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d35541, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6535546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f3554c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7735551, 3 },
    { 0x67475552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b5557, 3 },
    { 0x3c035558, 3 },
    { 0x9e7b5557, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc57555e, 4 },
    { 0x263b555e, 3 },
    { 0x2a3b5560, 3 },
    { 0xd133555f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94b5564, 3 },
    { 0x3faf5565, 3 },
    { 0x1b075566, 3 },
    { 0x1ed35566, 3 },
    { 0xceff5568, 4 },
    { 0xbb375569, 3 },
    { 0x6ca3556a, 3 },
    { 0x8ef75568, 3 },
    { 0x5eb7556c, 3 },
    { 0xed63556d, 3 },
    { 0x7d77556c, 3 },
    { 0, 0 },
    { 0xca9b5570, 3 },
    { 0x5ba75570, 4 },
    { 0, 0 },
    { 0x47b75573, 4 },
    { 0x12375573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31835577, 3 },
    { 0x7fb5577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d3557b, 3 },
    { 0x8603557c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdf557f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beb5584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f5587, 3 },
    { 0xbe835588, 3 },
    { 0x19f5589, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff77558d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x644b5590, 3 },
    { 0x750b5591, 3 },
    { 0x55235591, 3 },
    { 0x505b5593, 3 },
    { 0x1b0f5591, 4 },
    { 0x701b5595, 3 },
    { 0x5a175592, 3 },
    { 0xeb5f5597, 3 },
    { 0xa4b75598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e355a0, 3 },
    { 0, 0 },
    { 0x3edb55a2, 4 },
    { 0xb9d755a2, 3 },
    { 0xd7b55a2, 5 },
    { 0x134755a2, 3 },
    { 0xe4d755a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f55aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef755af, 4 },
    { 0x7a5355af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b55b3, 3 },
    { 0xefa355b4, 3 },
    { 0, 0 },
    { 0x192355b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22755ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9755bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x382b55c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ab55c4, 4 },
    { 0xa76355c5, 3 },
    { 0x4fc355c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31755cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8df55d2, 3 },
    { 0, 0 },
    { 0xe4c755d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469755d8, 4 },
    { 0x13755d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d755e0, 3 },
    { 0xd4bf55e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11355e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297f55ec, 4 },
    { 0x1dbf55ec, 4 },
    { 0x1faf55ee, 3 },
    { 0xa1c755ee, 3 },
    { 0xcdcf55f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d355f5, 4 },
    { 0x682755f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x293b55f9, 5 },
    { 0x3d6755fa, 3 },
    { 0x531355fb, 3 },
    { 0xd88f55fb, 4 },
    { 0x268755fd, 3 },
    { 0xced355fa, 3 },
    { 0xa1b55fe, 3 },
    { 0xb3fb55fb, 3 },
    { 0x40135601, 3 },
    { 0xc14b55fd, 4 },
    { 0x5c4355fd, 3 },
    { 0x8bbb5604, 3 },
    { 0x28475604, 3 },
    { 0x157f5605, 4 },
    { 0x362b5605, 3 },
    { 0x3de75606, 3 },
    { 0x987f55fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f560c, 4 },
    { 0xdae7560d, 3 },
    { 0xee7b560e, 4 },
    { 0x52eb560d, 3 },
    { 0x71b7560f, 3 },
    { 0x883b5611, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbf5615, 3 },
    { 0xf1075615, 3 },
    { 0xed5b5615, 3 },
    { 0, 0 },
    { 0xd32f5619, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b5621, 3 },
    { 0x75e35621, 3 },
    { 0x3d035623, 3 },
    { 0xc0db5624, 3 },
    { 0xedf5625, 4 },
    { 0xaa4b5623, 4 },
    { 0x20eb5625, 4 },
    { 0xedb75628, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb562d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d935634, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f435637, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18b563a, 5 },
    { 0x568f563a, 3 },
    { 0x7abf563c, 3 },
    { 0, 0 },
    { 0x3aa7563e, 4 },
    { 0, 0 },
    { 0xb17b5640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf75644, 3 },
    { 0x77fb5644, 4 },
    { 0x584b5646, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb375649, 3 },
    { 0xee73564a, 3 },
    { 0, 0 },
    { 0xad4b564c, 3 },
    { 0x16fb564d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3535654, 3 },
    { 0, 0 },
    { 0xf3275656, 3 },
    { 0xa45f5657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63565a, 3 },
    { 0x96df565b, 4 },
    { 0x9827565c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eb5661, 5 },
    { 0x63bb5662, 4 },
    { 0, 0 },
    { 0xe60b5664, 4 },
    { 0x55935665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627566a, 3 },
    { 0xaf4b566b, 3 },
    { 0x327566a, 3 },
    { 0x816f566d, 3 },
    { 0, 0 },
    { 0xd217566f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde075674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2b5679, 3 },
    { 0xb5af567a, 3 },
    { 0x7223567b, 3 },
    { 0, 0 },
    { 0x779b567d, 3 },
    { 0x982f567e, 3 },
    { 0xd13f567e, 3 },
    { 0x99af5680, 3 },
    { 0xcfcb5680, 3 },
    { 0x38635682, 3 },
    { 0x8aef5683, 4 },
    { 0xf86b5684, 3 },
    { 0xf1cf5684, 3 },
    { 0x49cb5686, 3 },
    { 0x9e1f5687, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3703568a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cb568e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3635693, 3 },
    { 0, 0 },
    { 0x30b35695, 3 },
    { 0, 0 },
    { 0x44275697, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17569a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4997569d, 3 },
    { 0x4203569e, 3 },
    { 0xc1a3569e, 3 },
    { 0, 0 },
    { 0xa1cb56a1, 3 },
    { 0, 0 },
    { 0x3bc356a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b56a6, 3 },
    { 0x2fef56a7, 5 },
    { 0x75db56a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1756ac, 3 },
    { 0xd79756ac, 4 },
    { 0xef7756ae, 4 },
    { 0xa2ef56ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb56b2, 3 },
    { 0xd71756b3, 3 },
    { 0x24eb56b4, 3 },
    { 0xe43356b5, 3 },
    { 0, 0 },
    { 0x73e756b7, 3 },
    { 0, 0 },
    { 0x9dc356b9, 3 },
    { 0xf7a356ba, 3 },
    { 0x248356bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497b56c0, 3 },
    { 0, 0 },
    { 0x93f356c2, 3 },
    { 0, 0 },
    { 0x43756c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5b56cb, 3 },
    { 0, 0 },
    { 0x23f356cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f356d1, 3 },
    { 0, 0 },
    { 0xa03756d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69eb56dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b356e0, 3 },
    { 0x6ce356e1, 3 },
    { 0x5dd356e0, 3 },
    { 0, 0 },
    { 0xc06756e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb56e7, 3 },
    { 0x25df56e8, 3 },
    { 0x593356e7, 3 },
    { 0x725b56ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87b56ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927756f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78356f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee756fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6356ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd35703, 3 },
    { 0x8435704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437570b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25f570f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5b5713, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00f5718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf571c, 3 },
    { 0xb083571d, 3 },
    { 0, 0 },
    { 0xfbd3571f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdb5725, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3135729, 3 },
    { 0, 0 },
    { 0xd67f572b, 3 },
    { 0x16d7572c, 3 },
    { 0x4c13572c, 3 },
    { 0xf783572c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f475731, 3 },
    { 0x3a435731, 3 },
    { 0xae35732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce75738, 3 },
    { 0x78b35739, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9f573c, 3 },
    { 0xbde7573d, 4 },
    { 0x20f7573d, 3 },
    { 0x46ef573e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf5742, 3 },
    { 0xe8035743, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb5746, 5 },
    { 0xa5eb5747, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c3574d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d75756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c35759, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c7575d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a75760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1975765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed435768, 3 },
    { 0x3f335768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52db5770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde35774, 3 },
    { 0xe9875774, 5 },
    { 0x811b5776, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab75779, 3 },
    { 0xaf03577a, 3 },
    { 0xc5f7577b, 4 },
    { 0, 0 },
    { 0x876b577d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ff5783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcb5786, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe275789, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddf5790, 5 },
    { 0x69b5790, 3 },
    { 0xa0db5792, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecab5797, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763579b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527f57a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b757a6, 3 },
    { 0x65a357a7, 3 },
    { 0xe1bf57a8, 3 },
    { 0x476b57a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866757af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d757b9, 4 },
    { 0x35ff57ba, 3 },
    { 0xeb7357b9, 4 },
    { 0, 0 },
    { 0x2d7357bd, 3 },
    { 0xa94757be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2357c1, 3 },
    { 0x680b57c2, 3 },
    { 0x4cef57c3, 4 },
    { 0, 0 },
    { 0x599757c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c757d3, 4 },
    { 0x8bdf57d3, 3 },
    { 0xcdcf57d5, 3 },
    { 0x2f3357d4, 3 },
    { 0x662b57d4, 3 },
    { 0xfdb357d8, 3 },
    { 0xa68f57d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4357e2, 3 },
    { 0x3ad757e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9f57e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55357e9, 3 },
    { 0x36e357ea, 3 },
    { 0xfb6757eb, 3 },
    { 0xf2ef57ec, 3 },
    { 0x994b57ed, 3 },
    { 0x589f57ee, 4 },
    { 0x2e757ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c357f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b57fd, 3 },
    { 0x392357fd, 3 },
    { 0xfa9f57ff, 3 },
    { 0x4c1f5800, 3 },
    { 0x6b975801, 3 },
    { 0x7d675801, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a7580b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab5813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828b581e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88035823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cf582b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b582f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad75835, 3 },
    { 0x78f35835, 3 },
    { 0, 0 },
    { 0x17e35838, 3 },
    { 0xa3d75839, 3 },
    { 0x653583a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65f5846, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe075849, 3 },
    { 0xbf47584a, 4 },
    { 0x2c73584a, 3 },
    { 0x2a53584b, 3 },
    { 0, 0 },
    { 0xe36f584e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9f5853, 5 },
    { 0x60435854, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ab5857, 3 },
    { 0x3dff5857, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f585b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ff585f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4f5863, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82875869, 3 },
    { 0x95fb586a, 3 },
    { 0xff47586b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x557b586e, 4 },
    { 0x284f586f, 3 },
    { 0x3dab5870, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b5873, 3 },
    { 0, 0 },
    { 0x60975875, 3 },
    { 0, 0 },
    { 0xc3435877, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3587d, 3 },
    { 0, 0 },
    { 0x9e3587f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f35882, 3 },
    { 0x9b4f5883, 3 },
    { 0xb2cb5884, 3 },
    { 0xce4f5884, 3 },
    { 0, 0 },
    { 0x6b0b5887, 3 },
    { 0xc6375888, 3 },
    { 0x4e6f5887, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7588e, 3 },
    { 0x63c3588f, 4 },
    { 0, 0 },
    { 0x412f5891, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c35894, 4 },
    { 0x44075895, 3 },
    { 0xe88f5896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ff589a, 3 },
    { 0x19bf589b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07f589e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f758a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba758a8, 3 },
    { 0x418358a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da358b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2f58b5, 3 },
    { 0xb77758b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f58ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7f58bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7758c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b58c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17358c9, 3 },
    { 0xdbbf58ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c758ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x148b58d1, 3 },
    { 0x969f58d2, 3 },
    { 0xc64358d2, 3 },
    { 0xcc0b58d2, 3 },
    { 0xe8bb58d5, 3 },
    { 0x902758d5, 3 },
    { 0x7d4358d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09b58dd, 3 },
    { 0x11c358de, 3 },
    { 0x29ab58df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f358e4, 3 },
    { 0xef5758e5, 3 },
    { 0x306358e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x144358e9, 4 },
    { 0xbc5758ea, 4 },
    { 0, 0 },
    { 0x161358ec, 3 },
    { 0x32ff58ed, 3 },
    { 0, 0 },
    { 0xd77f58ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeff58f4, 3 },
    { 0x40b58f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf58f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f758fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135f5906, 3 },
    { 0, 0 },
    { 0x9b1f5908, 4 },
    { 0x614f5908, 3 },
    { 0x73ff590a, 4 },
    { 0x5c33590b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c75910, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc675916, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc87591a, 3 },
    { 0x553b591b, 3 },
    { 0x76ab591c, 3 },
    { 0x6ef7591b, 3 },
    { 0x893591b, 4 },
    { 0x68cb591f, 4 },
    { 0x192b591d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec75927, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3b5932, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de7593b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa037593f, 4 },
    { 0x1f1f593f, 3 },
    { 0x64c3593f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f5944, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6f5947, 3 },
    { 0x28375947, 3 },
    { 0xdb7f5947, 3 },
    { 0x7d3f594a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28df5953, 4 },
    { 0xbfb35954, 3 },
    { 0, 0 },
    { 0x8b375956, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac75959, 3 },
    { 0x2447595a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cb595d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cb5960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2835963, 3 },
    { 0xa9f35964, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37275968, 4 },
    { 0xe1e75968, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b596d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a35970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95df5973, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f5979, 3 },
    { 0, 0 },
    { 0x28c3597b, 3 },
    { 0x4ea7597c, 4 },
    { 0x9873597d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bf5986, 3 },
    { 0x6c1f5987, 4 },
    { 0, 0 },
    { 0x564b5989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6b598f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89675992, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff5996, 4 },
    { 0x8f335996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf599c, 4 },
    { 0xfd83599d, 4 },
    { 0x6403599e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75db59a3, 3 },
    { 0, 0 },
    { 0xa03b59a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58759ac, 3 },
    { 0, 0 },
    { 0x219759ae, 3 },
    { 0x4cb59af, 3 },
    { 0, 0 },
    { 0x383359b1, 3 },
    { 0, 0 },
    { 0x108759b3, 5 },
    { 0, 0 },
    { 0xe2f359b5, 3 },
    { 0xc78b59b6, 3 },
    { 0xda9359b6, 3 },
    { 0, 0 },
    { 0xded359b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672b59c0, 4 },
    { 0x5c1b59c1, 4 },
    { 0, 0 },
    { 0xb3cf59c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x436f59c6, 3 },
    { 0x78b359c7, 4 },
    { 0x2d3359c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9f59cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2b59d3, 3 },
    { 0xeb7359d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d759d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728759dc, 4 },
    { 0xf16759dc, 3 },
    { 0xc95f59de, 4 },
    { 0xacbb59df, 3 },
    { 0, 0 },
    { 0x43f759e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437359eb, 4 },
    { 0x49e359ec, 3 },
    { 0xf80b59ed, 3 },
    { 0x6b1759ee, 3 },
    { 0x34cf59ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28b59f5, 4 },
    { 0x75ab59f5, 3 },
    { 0, 0 },
    { 0xb24359f8, 4 },
    { 0xedfb59f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cf59fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241b5a00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90f5a03, 3 },
    { 0, 0 },
    { 0xfa535a05, 3 },
    { 0, 0 },
    { 0x89835a07, 4 },
    { 0x750b5a07, 3 },
    { 0, 0 },
    { 0x1a4f5a0a, 3 },
    { 0, 0 },
    { 0x167b5a0c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15935a12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1b5a1a, 3 },
    { 0x311f5a1b, 4 },
    { 0, 0 },
    { 0xc75b5a1d, 3 },
    { 0x93335a1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f5a21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2835a24, 3 },
    { 0, 0 },
    { 0x89eb5a26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59835a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908f5a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9135a33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc35a36, 3 },
    { 0, 0 },
    { 0x2e135a38, 4 },
    { 0x8a7f5a39, 4 },
    { 0xcaf35a39, 3 },
    { 0, 0 },
    { 0xcd6f5a3c, 3 },
    { 0xb275a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98575a40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c635a46, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c35a49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd35a4c, 4 },
    { 0, 0 },
    { 0xfc5b5a4e, 3 },
    { 0, 0 },
    { 0x6f2f5a50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2b5a57, 3 },
    { 0xc7775a58, 4 },
    { 0xe42b5a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a75a5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f5a62, 3 },
    { 0, 0 },
    { 0x12535a64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566f5a6d, 3 },
    { 0xb6eb5a6d, 3 },
    { 0x5fc75a6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce575a72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddf5a7a, 4 },
    { 0xeb8b5a7a, 3 },
    { 0xc1935a7a, 3 },
    { 0, 0 },
    { 0x19e35a7e, 3 },
    { 0x7c235a7e, 3 },
    { 0x3cd35a7e, 4 },
    { 0x3eff5a7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca835a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49f5a92, 3 },
    { 0, 0 },
    { 0x41f75a94, 3 },
    { 0, 0 },
    { 0xfa7f5a96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2b5a9a, 3 },
    { 0xe49f5a9b, 3 },
    { 0x6e6f5a9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4b5aa1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f75aa6, 4 },
    { 0x157b5aa6, 3 },
    { 0, 0 },
    { 0xbb9f5aa9, 3 },
    { 0x476b5aa9, 3 },
    { 0xf1075aab, 3 },
    { 0x7be75aac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b5aaf, 3 },
    { 0xce235ab0, 3 },
    { 0x313f5ab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf35ab7, 3 },
    { 0x93435ab8, 4 },
    { 0x36db5ab8, 3 },
    { 0x662b5aba, 3 },
    { 0x13375ab8, 3 },
    { 0, 0 },
    { 0xd9035abd, 4 },
    { 0, 0 },
    { 0x71ab5abf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8975ac2, 4 },
    { 0xc9eb5ac3, 3 },
    { 0x95cf5ac4, 3 },
    { 0xfcbf5ac2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84235ac9, 3 },
    { 0x16435ac9, 5 },
    { 0xd5b75acb, 4 },
    { 0x9cb5aca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f5acf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb35ad4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a75ad8, 3 },
    { 0xdb635ad8, 4 },
    { 0xa33b5ada, 5 },
    { 0xf80b5adb, 4 },
    { 0, 0 },
    { 0x819b5add, 3 },
    { 0, 0 },
    { 0x660b5adf, 3 },
    { 0, 0 },
    { 0xa08f5ae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cf5ae4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4035ae9, 3 },
    { 0x843f5ae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da75aef, 3 },
    { 0x99135aef, 3 },
    { 0x14835af1, 3 },
    { 0xaef35af1, 3 },
    { 0x20b35af0, 3 },
    { 0xca075af2, 3 },
    { 0, 0 },
    { 0x79f5af6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09b5b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc775b07, 3 },
    { 0xae6b5b07, 3 },
    { 0x1bbf5b07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9f5b0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c35b13, 3 },
    { 0, 0 },
    { 0x68b75b15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x658b5b18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35935b1b, 3 },
    { 0xbf575b1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0f5b28, 3 },
    { 0x8be75b29, 3 },
    { 0x14a75b2a, 3 },
    { 0x521f5b2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10135b30, 3 },
    { 0x59835b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8b5b35, 4 },
    { 0, 0 },
    { 0x9b375b37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec35b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535b43, 3 },
    { 0x50cb5b43, 3 },
    { 0x52475b45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f5b4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a75b4e, 3 },
    { 0xe675b4f, 3 },
    { 0, 0 },
    { 0xecbf5b51, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x342b5b54, 3 },
    { 0x54ab5b55, 3 },
    { 0xfec75b56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d675b62, 3 },
    { 0xe97f5b63, 3 },
    { 0, 0 },
    { 0x4a935b65, 3 },
    { 0, 0 },
    { 0xea4b5b67, 4 },
    { 0x5d275b67, 3 },
    { 0x3c3b5b68, 3 },
    { 0x931b5b68, 3 },
    { 0, 0 },
    { 0xce735b6c, 3 },
    { 0x828b5b6c, 3 },
    { 0xe5735b6d, 3 },
    { 0, 0 },
    { 0x9f675b70, 4 },
    { 0x3fc75b70, 3 },
    { 0, 0 },
    { 0xa6175b73, 3 },
    { 0, 0 },
    { 0x4c835b75, 4 },
    { 0x42875b76, 4 },
    { 0x9f475b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717b5b7c, 3 },
    { 0x50935b7d, 3 },
    { 0x73b5b7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a75b83, 4 },
    { 0xf6035b83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x142b5b87, 3 },
    { 0, 0 },
    { 0x45a75b89, 3 },
    { 0xe1f35b8a, 4 },
    { 0x7bf75b8b, 3 },
    { 0xddef5b89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f5b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b75b92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340f5b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d35ba6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9935bad, 4 },
    { 0, 0 },
    { 0x3f675baf, 3 },
    { 0, 0 },
    { 0x250b5bb1, 3 },
    { 0, 0 },
    { 0xb3f5bb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c35bb6, 4 },
    { 0x310b5bb7, 4 },
    { 0x3135bb7, 3 },
    { 0, 0 },
    { 0x30af5bba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e35bbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4775bc3, 4 },
    { 0xe2035bc3, 3 },
    { 0, 0 },
    { 0x5b8b5bc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b5bca, 3 },
    { 0, 0 },
    { 0x81e75bcc, 3 },
    { 0x80b35bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7835bd0, 3 },
    { 0, 0 },
    { 0x92eb5bd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf5bd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d35bd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad75be1, 3 },
    { 0, 0 },
    { 0x25ff5be3, 4 },
    { 0xc7275be3, 3 },
    { 0xb3175be4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bb5be8, 4 },
    { 0xf1df5be9, 4 },
    { 0xe6235bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e35bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f75bf1, 3 },
    { 0, 0 },
    { 0xa6735bf3, 4 },
    { 0, 0 },
    { 0x67375bf5, 3 },
    { 0x71ff5bf6, 3 },
    { 0x47a35bf6, 5 },
    { 0x8c975bf8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b5bfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2635c03, 3 },
    { 0xbfcb5c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab75c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22b5c0b, 3 },
    { 0xa4c35c0c, 3 },
    { 0xac575c0c, 3 },
    { 0x483b5c0e, 3 },
    { 0x5b75c0f, 4 },
    { 0x2bd75c10, 3 },
    { 0x36475c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5c15, 4 },
    { 0, 0 },
    { 0x2a8b5c17, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfab5c1a, 4 },
    { 0xef2b5c1b, 3 },
    { 0x77ab5c1c, 3 },
    { 0x8e9f5c1d, 3 },
    { 0x7c75c1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f35c27, 3 },
    { 0xfab35c27, 3 },
    { 0xc29f5c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525f5c30, 3 },
    { 0xea135c31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad675c35, 3 },
    { 0, 0 },
    { 0x130f5c37, 3 },
    { 0x14335c38, 4 },
    { 0xd8235c38, 5 },
    { 0, 0 },
    { 0xa3eb5c3b, 4 },
    { 0x9d9b5c3c, 4 },
    { 0x9d2b5c3d, 3 },
    { 0xfecb5c3e, 4 },
    { 0xd45b5c3b, 3 },
    { 0x7eff5c3f, 3 },
    { 0xc1f35c41, 3 },
    { 0, 0 },
    { 0x647b5c43, 4 },
    { 0x99f35c44, 4 },
    { 0, 0 },
    { 0xdb975c46, 3 },
    { 0xe7d75c47, 3 },
    { 0x52675c46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d35c4d, 3 },
    { 0xa3575c4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610f5c52, 3 },
    { 0xa0e35c52, 3 },
    { 0x1e1f5c53, 4 },
    { 0x34e35c55, 3 },
    { 0xea8b5c55, 3 },
    { 0, 0 },
    { 0x7c5f5c58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b35c61, 3 },
    { 0x8a875c62, 3 },
    { 0x915f5c63, 4 },
    { 0x14cb5c63, 3 },
    { 0, 0 },
    { 0x46e75c66, 4 },
    { 0x12575c67, 4 },
    { 0x85b5c68, 4 },
    { 0, 0 },
    { 0xdfcb5c6a, 3 },
    { 0x56ab5c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cb5c6e, 3 },
    { 0xbf7b5c6f, 3 },
    { 0x468f5c70, 4 },
    { 0xfa0b5c71, 4 },
    { 0xcd6f5c72, 3 },
    { 0xd3075c6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08f5c76, 4 },
    { 0xe26b5c77, 4 },
    { 0x4d575c76, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x18af5c7b, 3 },
    { 0xe9df5c7b, 3 },
    { 0x240f5c7d, 3 },
    { 0, 0 },
    { 0xdb835c7f, 3 },
    { 0x620b5c80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64735c83, 3 },
    { 0xd7875c83, 3 },
    { 0x21435c85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72b5c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42335c8d, 3 },
    { 0x20435c8e, 3 },
    { 0xf0e35c8f, 3 },
    { 0, 0 },
    { 0x6f235c91, 4 },
    { 0, 0 },
    { 0xcf575c93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f5c97, 3 },
    { 0x2a635c98, 4 },
    { 0x18a35c98, 3 },
    { 0, 0 },
    { 0x189b5c9b, 4 },
    { 0xbe975c9c, 3 },
    { 0, 0 },
    { 0x59db5c9e, 4 },
    { 0, 0 },
    { 0xd0375ca0, 5 },
    { 0xe05f5ca1, 3 },
    { 0x8c675ca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5575ca7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f675cae, 3 },
    { 0x78575caf, 4 },
    { 0x7c735cae, 3 },
    { 0xc62b5cb1, 3 },
    { 0xfe2f5cb2, 3 },
    { 0xe07f5cb3, 3 },
    { 0x886b5cb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94035cb9, 3 },
    { 0, 0 },
    { 0xfb9f5cbb, 3 },
    { 0x14cb5cbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bf5cbf, 3 },
    { 0x1cdf5cc0, 4 },
    { 0, 0 },
    { 0xa3835cc2, 3 },
    { 0, 0 },
    { 0xe60b5cc4, 3 },
    { 0xa17b5cc4, 3 },
    { 0x254f5cc6, 3 },
    { 0xe0e35cc6, 5 },
    { 0x640f5cc6, 4 },
    { 0xdc435cc6, 4 },
    { 0xc7c35cc4, 3 },
    { 0x976b5cc7, 3 },
    { 0, 0 },
    { 0xa8e35ccd, 3 },
    { 0xd1235ccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85135cd1, 3 },
    { 0, 0 },
    { 0xaaff5cd3, 3 },
    { 0xd59b5cd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe75cda, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f35cdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2475ce3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x475ce6, 3 },
    { 0x63135ce7, 3 },
    { 0, 0 },
    { 0x875b5ce9, 4 },
    { 0x7dd35cea, 3 },
    { 0, 0 },
    { 0x79c35cec, 4 },
    { 0x98075ced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48f5cf5, 3 },
    { 0, 0 },
    { 0xbbc35cf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df5cfb, 3 },
    { 0x1ceb5cfc, 5 },
    { 0xde0b5cfd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebf5d04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b5d08, 4 },
    { 0x4af75d08, 4 },
    { 0, 0 },
    { 0xe76f5d0b, 3 },
    { 0x8d6b5d0c, 3 },
    { 0xfcb5d0d, 3 },
    { 0, 0 },
    { 0x1e8f5d0f, 3 },
    { 0xd91f5d0f, 3 },
    { 0xe71f5d11, 4 },
    { 0, 0 },
    { 0xb4cb5d13, 5 },
    { 0x6ecf5d14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f5d1a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef35d28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46f5d2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d275d2e, 4 },
    { 0xd9ab5d2e, 3 },
    { 0x94b5d30, 3 },
    { 0, 0 },
    { 0x659b5d32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x75df5d35, 3 },
    { 0, 0 },
    { 0x81c35d37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a35d3b, 3 },
    { 0xb3b5d3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af5d42, 3 },
    { 0xad235d42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3475d47, 4 },
    { 0xd1b75d48, 3 },
    { 0x9a435d49, 4 },
    { 0xf0df5d47, 3 },
    { 0x583b5d4b, 3 },
    { 0x82bb5d4c, 3 },
    { 0x7b135d4d, 3 },
    { 0xc83b5d4b, 3 },
    { 0x9d975d4e, 3 },
    { 0xb2f75d50, 5 },
    { 0xe8db5d50, 3 },
    { 0xd3275d4d, 3 },
    { 0x6f975d53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a435d5b, 3 },
    { 0xd40b5d5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835d61, 3 },
    { 0xb7a75d62, 3 },
    { 0, 0 },
    { 0xeedb5d64, 3 },
    { 0x1a3b5d65, 3 },
    { 0, 0 },
    { 0xbb575d67, 3 },
    { 0, 0 },
    { 0x499b5d69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf835d6e, 3 },
    { 0, 0 },
    { 0x7ab5d70, 4 },
    { 0x824b5d71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2735d7a, 3 },
    { 0x46275d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff5d7e, 4 },
    { 0xeef35d7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b5d84, 3 },
    { 0x6e635d85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb5d88, 3 },
    { 0, 0 },
    { 0x2b1f5d8a, 3 },
    { 0, 0 },
    { 0x5a975d8c, 3 },
    { 0x76af5d8d, 3 },
    { 0x526f5d8d, 4 },
    { 0, 0 },
    { 0xd82f5d90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72975d93, 3 },
    { 0x2ee75d93, 3 },
    { 0xc62b5d95, 3 },
    { 0x66275d96, 4 },
    { 0x58bf5d97, 3 },
    { 0xc9b75d93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee35d9d, 5 },
    { 0x1ccb5d9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675da1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x51775da4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b5da8, 3 },
    { 0xc4ef5da9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e975dad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cb5db6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba75dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b5dc9, 3 },
    { 0x96f75dca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5b5dcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x451b5dd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bf5dd5, 3 },
    { 0x1bdf5dd6, 3 },
    { 0, 0 },
    { 0xaa7b5dd8, 3 },
    { 0xf3d35dd9, 3 },
    { 0x4bbb5dd8, 3 },
    { 0x2cf35ddb, 3 },
    { 0x43135ddc, 3 },
    { 0xba935ddd, 3 },
    { 0x50d75ddd, 3 },
    { 0xc0335dde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad335de2, 4 },
    { 0xea0f5de3, 3 },
    { 0x92975de4, 4 },
    { 0xd43f5de5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53275deb, 3 },
    { 0x95675deb, 3 },
    { 0xc91b5dec, 4 },
    { 0x55975dee, 3 },
    { 0xf4f5ded, 3 },
    { 0x7a5b5df0, 3 },
    { 0x393f5df1, 4 },
    { 0, 0 },
    { 0x51535df3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98f5df7, 3 },
    { 0, 0 },
    { 0xc8f5df9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8875dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb5e05, 3 },
    { 0xdd4b5e06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8175e09, 4 },
    { 0, 0 },
    { 0x76935e0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6af5e0e, 3 },
    { 0x91835e0f, 3 },
    { 0x71475e10, 3 },
    { 0x7a2b5e11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a775e15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f35e19, 3 },
    { 0x32835e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353f5e1e, 3 },
    { 0x56975e1f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf5e25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b5e29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2475e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a875e31, 3 },
    { 0, 0 },
    { 0xcbeb5e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c035e37, 3 },
    { 0xcaf35e38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea35e3b, 4 },
    { 0x308f5e3c, 3 },
    { 0, 0 },
    { 0x49835e3e, 3 },
    { 0x17f75e3e, 3 },
    { 0x62a35e40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x104b5e43, 3 },
    { 0x96675e43, 3 },
    { 0, 0 },
    { 0x915f5e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a035e49, 4 },
    { 0x25475e4a, 3 },
    { 0, 0 },
    { 0xa84f5e4c, 3 },
    { 0xf94b5e4d, 3 },
    { 0x78635e4e, 3 },
    { 0, 0 },
    { 0x9c675e50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135e53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5835e58, 3 },
    { 0xf55f5e59, 4 },
    { 0x4b975e58, 3 },
    { 0x7c5f5e5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f5e5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b5e64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ab5e67, 3 },
    { 0x222b5e68, 3 },
    { 0xe0235e68, 4 },
    { 0xa5fb5e6a, 3 },
    { 0, 0 },
    { 0xa8df5e6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eb5e72, 3 },
    { 0xa1b75e72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c75e76, 3 },
    { 0x7e275e77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9f5e7b, 3 },
    { 0xba6f5e7b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc175e89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43b5e8c, 3 },
    { 0xa5f35e8c, 3 },
    { 0x8ffb5e8e, 4 },
    { 0, 0 },
    { 0xf1635e90, 3 },
    { 0x45f35e91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4f5e97, 3 },
    { 0x99735e98, 3 },
    { 0xe3db5e99, 3 },
    { 0x97875e9a, 4 },
    { 0x6c235e9b, 3 },
    { 0xfc5f5e9c, 3 },
    { 0xfe0b5e9d, 3 },
    { 0x201f5e9e, 4 },
    { 0xb52b5e9d, 3 },
    { 0, 0 },
    { 0x52b35ea1, 4 },
    { 0, 0 },
    { 0x20f75ea3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aef5ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226b5ead, 3 },
    { 0xbb635ead, 3 },
    { 0xd775ead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b35eb3, 3 },
    { 0x2abf5eb3, 4 },
    { 0x692b5eb3, 3 },
    { 0xb5135eb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aef5ebe, 3 },
    { 0x93235ebe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f5ec7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bb5ecd, 4 },
    { 0xc9c75ece, 3 },
    { 0xb2635ecf, 3 },
    { 0, 0 },
    { 0xdb6f5ed1, 3 },
    { 0x8a835ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a035ed5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x838f5ed8, 3 },
    { 0, 0 },
    { 0x42275eda, 3 },
    { 0x2a835edb, 3 },
    { 0xab0b5edb, 3 },
    { 0x8a9f5edd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1535ee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b5ef1, 3 },
    { 0x4dc75ef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffb5ef5, 3 },
    { 0, 0 },
    { 0xc4c75ef7, 4 },
    { 0x52af5ef7, 3 },
    { 0x6b9b5ef9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f075efe, 4 },
    { 0x1f835eff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c75f02, 3 },
    { 0x547f5f03, 3 },
    { 0xf4235f04, 4 },
    { 0, 0 },
    { 0xf2675f06, 3 },
    { 0xf5575f07, 3 },
    { 0, 0 },
    { 0xb68f5f09, 4 },
    { 0, 0 },
    { 0x79d35f0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b775f10, 4 },
    { 0, 0 },
    { 0xabcf5f12, 3 },
    { 0x3f2f5f13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a35f16, 3 },
    { 0, 0 },
    { 0x4f635f18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68675f1c, 5 },
    { 0xba475f1d, 3 },
    { 0, 0 },
    { 0xc7eb5f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30f5f26, 3 },
    { 0x542b5f27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5735f2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20b5f3b, 3 },
    { 0, 0 },
    { 0x371f5f3d, 3 },
    { 0x5135f3e, 4 },
    { 0x3f535f3e, 3 },
    { 0xffcb5f40, 3 },
    { 0x38635f3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f35f45, 3 },
    { 0xceff5f46, 3 },
    { 0xa7cb5f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x656f5f4a, 3 },
    { 0, 0 },
    { 0x56675f4c, 3 },
    { 0xc92f5f4d, 3 },
    { 0x37375f4e, 4 },
    { 0x754b5f4f, 3 },
    { 0x6de75f4f, 5 },
    { 0, 0 },
    { 0xfff5f52, 3 },
    { 0x64735f52, 4 },
    { 0xfb8b5f54, 3 },
    { 0, 0 },
    { 0x80875f56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19375f5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9235f61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff5f68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed35f6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773f5f76, 4 },
    { 0x49ff5f77, 3 },
    { 0xc08b5f78, 3 },
    { 0x3ba35f78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x71db5f7c, 3 },
    { 0, 0 },
    { 0xf3db5f7e, 3 },
    { 0x239b5f7f, 3 },
    { 0xb19f5f7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43db5f89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbf5f8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c35f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x85035f95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9f5f9a, 3 },
    { 0xdb4f5f9b, 3 },
    { 0x50ef5f9a, 4 },
    { 0xfbab5f9d, 3 },
    { 0x28475f9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cf5fa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca475fa7, 4 },
    { 0, 0 },
    { 0x298b5fa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d75fac, 3 },
    { 0x82f75fad, 3 },
    { 0x9e475fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cf5fb2, 4 },
    { 0xca9f5fb2, 4 },
    { 0, 0 },
    { 0x276f5fb5, 3 },
    { 0, 0 },
    { 0x73735fb7, 3 },
    { 0, 0 },
    { 0xdb075fb9, 3 },
    { 0xc4bf5fba, 4 },
    { 0x74e35fbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1f5fbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5fc6, 3 },
    { 0xe2a35fc7, 3 },
    { 0xe38f5fc8, 4 },
    { 0x6b8f5fc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d75fcc, 3 },
    { 0, 0 },
    { 0xb2b75fce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cb5fd2, 3 },
    { 0, 0 },
    { 0x35835fd4, 4 },
    { 0x3af75fd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf35fde, 5 },
    { 0, 0 },
    { 0xd4d75fe0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x985f5fe3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeb5fe7, 3 },
    { 0, 0 },
    { 0x17af5fe9, 3 },
    { 0xb95b5fea, 3 },
    { 0x72e35feb, 3 },
    { 0x6af5fec, 3 },
    { 0, 0 },
    { 0xeef75fee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3b5ff1, 3 },
    { 0x6fe35ff2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ef5ff7, 3 },
    { 0x7b35ff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6b5ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfb6002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5db6005, 3 },
    { 0x6d836005, 3 },
    { 0x71f76006, 3 },
    { 0xaec36008, 3 },
    { 0x83736007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6536011, 4 },
    { 0x2a736012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5b6015, 3 },
    { 0x684f6016, 3 },
    { 0, 0 },
    { 0xca3f6018, 3 },
    { 0xe2af6019, 3 },
    { 0x3d7601a, 4 },
    { 0x8fdb601b, 3 },
    { 0x449f601c, 3 },
    { 0xf747601d, 3 },
    { 0xfe67601e, 3 },
    { 0x57601f, 3 },
    { 0x78036020, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec36024, 4 },
    { 0x18f6025, 4 },
    { 0x139f6026, 3 },
    { 0xdf776025, 4 },
    { 0x523f6025, 3 },
    { 0x31ab6028, 3 },
    { 0xa2536026, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793602d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2b6033, 3 },
    { 0, 0 },
    { 0x4b036035, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb603a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406b603e, 3 },
    { 0x564b603e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b36045, 3 },
    { 0x136b6046, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44776049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b604d, 3 },
    { 0x822b604e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54f6057, 3 },
    { 0, 0 },
    { 0x8b436059, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa083605c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cb605f, 3 },
    { 0x3abb605f, 3 },
    { 0xf5db6060, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb36065, 4 },
    { 0, 0 },
    { 0x787f6067, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9876071, 3 },
    { 0xc3b36072, 3 },
    { 0, 0 },
    { 0xd7cb6074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ff6078, 3 },
    { 0, 0 },
    { 0xbadf607a, 3 },
    { 0xffe3607b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da3607f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f76083, 4 },
    { 0, 0 },
    { 0x36cf6085, 3 },
    { 0, 0 },
    { 0x93b6087, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7608b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7608e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be36091, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf36096, 3 },
    { 0x846f6096, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eb609c, 3 },
    { 0, 0 },
    { 0x3e7b609e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d760a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cb60ab, 3 },
    { 0, 0 },
    { 0x591f60ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88760b0, 3 },
    { 0xbfe360b1, 3 },
    { 0, 0 },
    { 0xb6ef60b3, 4 },
    { 0, 0 },
    { 0x4fef60b5, 3 },
    { 0, 0 },
    { 0x5c8b60b7, 3 },
    { 0, 0 },
    { 0xebdb60b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202b60bd, 3 },
    { 0xebfb60be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf360c5, 3 },
    { 0xb1bf60c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b360cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b60d2, 3 },
    { 0x5ea760d3, 3 },
    { 0, 0 },
    { 0xf16b60d5, 3 },
    { 0xfdb760d6, 3 },
    { 0x45d360d5, 3 },
    { 0x409360d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62f60e1, 3 },
    { 0x962b60e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab60e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b60f5, 3 },
    { 0, 0 },
    { 0x588760f7, 3 },
    { 0xba9760f8, 3 },
    { 0x2cc760f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1760fe, 3 },
    { 0, 0 },
    { 0x8c076100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98636106, 4 },
    { 0x6c676107, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb063610c, 4 },
    { 0x644f610d, 3 },
    { 0xa38b610d, 3 },
    { 0x90a3610f, 4 },
    { 0x4307610f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e36117, 3 },
    { 0xd86b6118, 4 },
    { 0x82af6117, 3 },
    { 0xfe5b611a, 3 },
    { 0xb57b6119, 4 },
    { 0x877611c, 3 },
    { 0x15b611a, 3 },
    { 0x66ff611e, 3 },
    { 0xb103611f, 3 },
    { 0x7eaf611e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353f6129, 3 },
    { 0, 0 },
    { 0x1ae3612b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e36130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6b6134, 3 },
    { 0x94f36134, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d376138, 3 },
    { 0xbb0f6139, 4 },
    { 0xd5e36139, 3 },
    { 0xac0b6139, 3 },
    { 0xea23613a, 3 },
    { 0, 0 },
    { 0xa66b613e, 3 },
    { 0xf92b613f, 3 },
    { 0, 0 },
    { 0xb6e36141, 3 },
    { 0xb8236142, 3 },
    { 0, 0 },
    { 0x1ba76144, 3 },
    { 0, 0 },
    { 0x830b6146, 3 },
    { 0xc27b6147, 4 },
    { 0, 0 },
    { 0x834f6149, 3 },
    { 0, 0 },
    { 0x56f7614b, 4 },
    { 0x29eb614c, 3 },
    { 0xf30f614d, 3 },
    { 0xcf13614e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a36152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd733615f, 4 },
    { 0xff236160, 3 },
    { 0xaa6f6161, 3 },
    { 0x56cf6162, 3 },
    { 0xf1bb6162, 4 },
    { 0xcbb36163, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8f6168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207f616c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc476170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d36173, 3 },
    { 0xad036174, 5 },
    { 0x8f776175, 3 },
    { 0x5ac36174, 4 },
    { 0xccef6176, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e93617b, 3 },
    { 0x604f617b, 3 },
    { 0x2db617c, 3 },
    { 0x91c3617c, 3 },
    { 0x5f93617e, 3 },
    { 0x426f6180, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e76183, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b36186, 3 },
    { 0xcf0b6186, 3 },
    { 0x8bbf6186, 3 },
    { 0xb25b6189, 3 },
    { 0xc5876188, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df3618d, 3 },
    { 0x8b4b618e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da36192, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f6196, 5 },
    { 0xe86f6197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5b619c, 3 },
    { 0, 0 },
    { 0x3de7619e, 4 },
    { 0xfe53619f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00b61a8, 4 },
    { 0, 0 },
    { 0xe51b61aa, 4 },
    { 0x23b61aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e761ae, 4 },
    { 0, 0 },
    { 0xcf3361b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f61b4, 3 },
    { 0, 0 },
    { 0xb35761b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633f61bc, 3 },
    { 0, 0 },
    { 0x5fb61be, 3 },
    { 0, 0 },
    { 0xbc6b61c0, 4 },
    { 0x17fb61c0, 3 },
    { 0x56bb61c2, 4 },
    { 0x52df61c3, 3 },
    { 0xc2e761c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e761ce, 4 },
    { 0xd5f761cf, 4 },
    { 0x7f7f61d0, 3 },
    { 0x714f61d1, 4 },
    { 0xe5f761d1, 4 },
    { 0x7b1361d3, 3 },
    { 0, 0 },
    { 0xbb5b61d5, 4 },
    { 0, 0 },
    { 0xbd0761d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bab61dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5b61e3, 3 },
    { 0x189b61e3, 3 },
    { 0, 0 },
    { 0x480761e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb61eb, 3 },
    { 0x987361ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ef61f1, 4 },
    { 0xa7fb61f2, 3 },
    { 0x1ff761f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x608b61f6, 3 },
    { 0, 0 },
    { 0xd33f61f8, 4 },
    { 0x113361f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7136204, 4 },
    { 0x6f6204, 3 },
    { 0x40776206, 3 },
    { 0x3e8b6204, 3 },
    { 0x8bf76205, 4 },
    { 0, 0 },
    { 0x7e33620a, 3 },
    { 0, 0 },
    { 0xf5b7620c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c36210, 3 },
    { 0x5f3f6211, 3 },
    { 0xd27f6212, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5337621a, 3 },
    { 0xe847621b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d3622b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b622e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd836231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83676241, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83b6245, 3 },
    { 0, 0 },
    { 0x3e0f6247, 3 },
    { 0x8fbb6248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1b624c, 4 },
    { 0x32b7624d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6936255, 3 },
    { 0, 0 },
    { 0x904b6257, 4 },
    { 0, 0 },
    { 0x83636259, 3 },
    { 0xcceb6259, 4 },
    { 0x72f76259, 3 },
    { 0, 0 },
    { 0xd74b625d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6260, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837f6271, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e76275, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b336278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f627b, 3 },
    { 0x9593627c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5f627f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f736282, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f6285, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8236288, 3 },
    { 0x1d7f6288, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d236294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61636297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85362a0, 3 },
    { 0xa7d362a0, 3 },
    { 0x52ff62a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f362a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b362ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f62b2, 3 },
    { 0xf0d362b2, 3 },
    { 0x343f62b4, 3 },
    { 0x9e0762b5, 3 },
    { 0xbacf62b2, 4 },
    { 0, 0 },
    { 0x563f62b8, 3 },
    { 0xc92762b9, 3 },
    { 0, 0 },
    { 0x5d1f62bb, 3 },
    { 0, 0 },
    { 0x15b762bd, 4 },
    { 0x7c6362be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69762c2, 3 },
    { 0x853b62c2, 3 },
    { 0, 0 },
    { 0xd41362c5, 3 },
    { 0x664f62c5, 3 },
    { 0xd55762c7, 4 },
    { 0, 0 },
    { 0x74a362c9, 3 },
    { 0xe22362ca, 3 },
    { 0x6def62cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c362d1, 4 },
    { 0x7bbb62d2, 3 },
    { 0x201762d2, 3 },
    { 0x7e4762d3, 3 },
    { 0xf8ff62d1, 4 },
    { 0x99df62d4, 3 },
    { 0x704762d7, 3 },
    { 0xcb7f62d7, 3 },
    { 0x6d5b62d6, 3 },
    { 0, 0 },
    { 0x3c6f62db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bb62e1, 3 },
    { 0xa03b62e2, 3 },
    { 0x438362e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51362ee, 5 },
    { 0x23fb62ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0b62f3, 3 },
    { 0x969762f4, 4 },
    { 0, 0 },
    { 0x8c8b62f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2362fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87b62fe, 4 },
    { 0, 0 },
    { 0xbaaf6300, 3 },
    { 0xb14b6301, 4 },
    { 0x57636302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x233f6305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0276308, 4 },
    { 0xd5d36309, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f6312, 3 },
    { 0x9e476312, 3 },
    { 0xeb0f6312, 3 },
    { 0x66fb6315, 4 },
    { 0xc8a36316, 4 },
    { 0x481b6313, 3 },
    { 0xa21b6318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3f631c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c76321, 4 },
    { 0x4f9b6322, 3 },
    { 0xc96f6321, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101f6329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2017632c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf632f, 3 },
    { 0x56b76330, 4 },
    { 0x84df6331, 4 },
    { 0x68cb6332, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d3633c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf807634a, 4 },
    { 0xae67634b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3433634e, 4 },
    { 0xec97634f, 3 },
    { 0x1abf6350, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf636357, 3 },
    { 0x9fb76357, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd123635d, 3 },
    { 0, 0 },
    { 0xca1b635f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f76362, 5 },
    { 0x1e9f6363, 3 },
    { 0, 0 },
    { 0x76db6365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2f6368, 3 },
    { 0x78276369, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c7636e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462f6374, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28f6379, 4 },
    { 0, 0 },
    { 0x265f637b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db6382, 4 },
    { 0x8e636382, 3 },
    { 0xc9436382, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fb638b, 3 },
    { 0, 0 },
    { 0xf04f638d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c76390, 4 },
    { 0x396b6390, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1927639a, 4 },
    { 0x6583639b, 3 },
    { 0x6ae7639b, 4 },
    { 0xf067639d, 4 },
    { 0, 0 },
    { 0x4df639f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f63a3, 5 },
    { 0xd0363a4, 3 },
    { 0xa3c763a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1b63a8, 4 },
    { 0, 0 },
    { 0x900b63aa, 4 },
    { 0, 0 },
    { 0xaaab63ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b63af, 3 },
    { 0, 0 },
    { 0x750363b1, 3 },
    { 0x6a7763b2, 3 },
    { 0xb65f63b3, 3 },
    { 0, 0 },
    { 0x11a763b5, 3 },
    { 0, 0 },
    { 0x877f63b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf763bf, 3 },
    { 0x27c363bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55db63c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b63c6, 3 },
    { 0x7e363c7, 3 },
    { 0xf3cb63c8, 4 },
    { 0, 0 },
    { 0x95ef63ca, 4 },
    { 0x23af63cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8363d0, 4 },
    { 0, 0 },
    { 0xbf0f63d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40f63d5, 4 },
    { 0x571f63d5, 4 },
    { 0x21db63d7, 3 },
    { 0xeaff63d7, 4 },
    { 0xf43f63d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326f63de, 3 },
    { 0x15eb63df, 3 },
    { 0, 0 },
    { 0x3c9b63e1, 4 },
    { 0x7e3363e2, 3 },
    { 0x2ddb63e1, 3 },
    { 0, 0 },
    { 0x18bf63e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4763ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b63ed, 3 },
    { 0x31a363ed, 5 },
    { 0x990b63ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06363f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9f63f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624b63ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf636404, 3 },
    { 0x81736405, 3 },
    { 0xeafb6406, 3 },
    { 0x25076407, 3 },
    { 0xf7eb6408, 3 },
    { 0xb8a76404, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577640e, 3 },
    { 0x128f640f, 4 },
    { 0, 0 },
    { 0x94036411, 3 },
    { 0x93f6412, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x80376415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afb6418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5376420, 3 },
    { 0xcc136420, 3 },
    { 0xd5436422, 3 },
    { 0xfb736423, 3 },
    { 0xd6b6423, 3 },
    { 0xd1176425, 4 },
    { 0x34e36426, 4 },
    { 0x86276427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce3642e, 3 },
    { 0x96ab642e, 3 },
    { 0xe8df6430, 3 },
    { 0xd1a76431, 3 },
    { 0x181b6432, 3 },
    { 0xc9476431, 4 },
    { 0x77e36431, 3 },
    { 0x4c9f6430, 4 },
    { 0xefc76436, 3 },
    { 0xeccb6437, 3 },
    { 0x518f6438, 3 },
    { 0xa91f6439, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf533643f, 3 },
    { 0xa1076440, 3 },
    { 0xd7976441, 3 },
    { 0x301b6442, 4 },
    { 0x4436442, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f6449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5b644c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x389b644f, 3 },
    { 0, 0 },
    { 0x17736451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f6456, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd623645c, 4 },
    { 0xd1bf645c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecb6462, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bb6468, 3 },
    { 0x49cf6468, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f646d, 3 },
    { 0xc3db646e, 4 },
    { 0xafff646d, 5 },
    { 0, 0 },
    { 0x653f6471, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b6475, 4 },
    { 0, 0 },
    { 0x867b6477, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8b647f, 3 },
    { 0x8f376480, 4 },
    { 0x325f6480, 3 },
    { 0x1fa76482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ab6489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb648d, 5 },
    { 0x2aef648d, 3 },
    { 0, 0 },
    { 0xa0bb6490, 4 },
    { 0xebc36490, 3 },
    { 0x598b6490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb476496, 3 },
    { 0, 0 },
    { 0xcdd76498, 3 },
    { 0, 0 },
    { 0x733f649a, 3 },
    { 0x21cb649b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7649e, 3 },
    { 0, 0 },
    { 0x5fef64a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b64a4, 4 },
    { 0xc63f64a4, 4 },
    { 0x39af64a5, 3 },
    { 0x6c6764a7, 4 },
    { 0, 0 },
    { 0xd764a9, 4 },
    { 0xb54f64aa, 3 },
    { 0x874f64aa, 3 },
    { 0x92af64ab, 5 },
    { 0x672364ad, 3 },
    { 0x4ba764aa, 3 },
    { 0x6f7764ab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b64b3, 3 },
    { 0xf7bf64b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189364ba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x130764bd, 3 },
    { 0xb7d364be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fb64c6, 3 },
    { 0x117f64c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02f64cc, 3 },
    { 0xce0364cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5f64d0, 3 },
    { 0x695764d0, 5 },
    { 0x153364d2, 3 },
    { 0xa4e364d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x438764d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6764db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07f64de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2364e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7364e8, 4 },
    { 0x6a5f64e8, 4 },
    { 0, 0 },
    { 0x132764eb, 3 },
    { 0x4c3764ec, 3 },
    { 0x2b9b64ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b64f3, 3 },
    { 0x2a4364f4, 3 },
    { 0, 0 },
    { 0xad2764f6, 3 },
    { 0x1e3764f7, 4 },
    { 0x523764f8, 3 },
    { 0, 0 },
    { 0x23d364fa, 3 },
    { 0, 0 },
    { 0x9ba764fc, 3 },
    { 0, 0 },
    { 0x46df64fe, 4 },
    { 0x8fb364ff, 3 },
    { 0xc0bf64fe, 4 },
    { 0x59d764fe, 3 },
    { 0xfcb76502, 4 },
    { 0xc436503, 3 },
    { 0xfe3b6504, 3 },
    { 0xfad76502, 4 },
    { 0, 0 },
    { 0x2bcf6507, 3 },
    { 0xfb176508, 3 },
    { 0x847f6509, 3 },
    { 0x88b650a, 4 },
    { 0xc11f650b, 3 },
    { 0xb3650c, 3 },
    { 0, 0 },
    { 0x1497650e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13b6514, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b83651d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2076521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c36524, 3 },
    { 0x1d776525, 3 },
    { 0xa58f6526, 3 },
    { 0x1fb36526, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcf652a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x504b652d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b6530, 4 },
    { 0xc3736531, 3 },
    { 0x8d2b6530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15336535, 3 },
    { 0, 0 },
    { 0xf9a36537, 3 },
    { 0x331b6538, 3 },
    { 0x377f6539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a7653c, 3 },
    { 0, 0 },
    { 0x8057653e, 3 },
    { 0, 0 },
    { 0x248f6540, 3 },
    { 0xe76541, 3 },
    { 0, 0 },
    { 0xd7d36543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63536546, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3654a, 4 },
    { 0xc2b654b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb36550, 5 },
    { 0x5c6f6551, 4 },
    { 0x35576552, 3 },
    { 0x56836553, 5 },
    { 0x5eb76553, 3 },
    { 0x68cb6551, 3 },
    { 0x844f6556, 3 },
    { 0x52c36553, 4 },
    { 0xff176556, 3 },
    { 0x3a8b6554, 3 },
    { 0xc8eb655a, 3 },
    { 0xa8db655a, 3 },
    { 0xd8b655a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c536560, 4 },
    { 0x13a36560, 3 },
    { 0x7fff6562, 3 },
    { 0xa4ff6563, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a36568, 4 },
    { 0, 0 },
    { 0x9443656a, 3 },
    { 0xe7ef656b, 3 },
    { 0x24ef656b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d7656f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bf6572, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb6575, 3 },
    { 0x3abf6576, 3 },
    { 0xc9236576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aef657b, 3 },
    { 0, 0 },
    { 0x4d8f657d, 3 },
    { 0x9267657e, 3 },
    { 0, 0 },
    { 0x242f6580, 3 },
    { 0, 0 },
    { 0xa6ef6582, 3 },
    { 0, 0 },
    { 0x10cf6584, 3 },
    { 0x68ff6585, 5 },
    { 0xd80f6585, 3 },
    { 0x85f76586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa657658d, 3 },
    { 0xd567658e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3236598, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60f659c, 4 },
    { 0, 0 },
    { 0xa50b659e, 3 },
    { 0x52af659f, 4 },
    { 0xb93765a0, 4 },
    { 0xaa2365a1, 3 },
    { 0xfb27659e, 3 },
    { 0xb8bf659e, 4 },
    { 0xd1cb65a4, 3 },
    { 0xddc365a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856f65a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48765ad, 3 },
    { 0xb88765ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f65b7, 3 },
    { 0xf70765b8, 3 },
    { 0xf2af65b9, 3 },
    { 0xf65f65ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeb65be, 3 },
    { 0x937765bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f65c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaf65cc, 3 },
    { 0, 0 },
    { 0xa11365ce, 3 },
    { 0xb4cb65cf, 3 },
    { 0x3cdf65cf, 3 },
    { 0x427b65d1, 4 },
    { 0xead365d2, 3 },
    { 0, 0 },
    { 0x54365d4, 3 },
    { 0, 0 },
    { 0x376765d6, 3 },
    { 0, 0 },
    { 0xf9ab65d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf65dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddf65e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879765e8, 3 },
    { 0xbc5765e9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df65ef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eb65f3, 3 },
    { 0, 0 },
    { 0x2e8765f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813b65fa, 4 },
    { 0, 0 },
    { 0xf1b765fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0876603, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54b6606, 3 },
    { 0xbb9f6607, 3 },
    { 0xa88f6607, 3 },
    { 0xff0f6609, 3 },
    { 0x4f476609, 3 },
    { 0xe8d36609, 4 },
    { 0x9c27660b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96f660f, 4 },
    { 0xd1b7660f, 4 },
    { 0, 0 },
    { 0x3bf76612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80136615, 4 },
    { 0, 0 },
    { 0x26eb6617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67176622, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb6626, 3 },
    { 0x70f76627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f662a, 4 },
    { 0, 0 },
    { 0x66b7662c, 4 },
    { 0, 0 },
    { 0x92bb662e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcf6631, 3 },
    { 0x705b6632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec76636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3476639, 3 },
    { 0x20f36639, 4 },
    { 0, 0 },
    { 0xbfcb663c, 3 },
    { 0x61ab663d, 3 },
    { 0xb0e3663d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x215b6641, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956b6647, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2853664a, 3 },
    { 0x5437664b, 4 },
    { 0x6af664c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad37664f, 3 },
    { 0x136b6650, 3 },
    { 0x553f6650, 3 },
    { 0, 0 },
    { 0x9f776653, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cf665c, 3 },
    { 0x4287665c, 3 },
    { 0xc507665e, 4 },
    { 0xe947665e, 4 },
    { 0x25876660, 3 },
    { 0xc437665f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b76664, 5 },
    { 0xdc9b6664, 4 },
    { 0, 0 },
    { 0x97776667, 3 },
    { 0x4ff76667, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebf6670, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x182f6673, 3 },
    { 0x5f4f6674, 3 },
    { 0x28f6675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5b667d, 3 },
    { 0x6bc7667e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3176684, 3 },
    { 0x15d36685, 3 },
    { 0x7aeb6686, 5 },
    { 0xbcfb6685, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb668c, 3 },
    { 0x5b43668d, 5 },
    { 0, 0 },
    { 0xeb93668f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682f6694, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40af6698, 3 },
    { 0x91cf6698, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef669c, 3 },
    { 0x90d7669d, 3 },
    { 0x20df669e, 3 },
    { 0, 0 },
    { 0xef7b66a0, 3 },
    { 0xd9766a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82766a4, 4 },
    { 0x205b66a4, 3 },
    { 0xce7366a6, 4 },
    { 0x25bb66a7, 4 },
    { 0x35c766a6, 3 },
    { 0, 0 },
    { 0x3fd766aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e366af, 3 },
    { 0x471366b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ef66b5, 3 },
    { 0x69b66b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x521f66b9, 3 },
    { 0x122f66ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f66bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eb66c1, 3 },
    { 0x89a366c2, 3 },
    { 0xbcc366c3, 3 },
    { 0xe12f66c2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaf66c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff66cd, 3 },
    { 0, 0 },
    { 0xf8bb66cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f366d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734766d7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115f66db, 3 },
    { 0, 0 },
    { 0x831766dd, 4 },
    { 0xf96b66dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71366e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2766e6, 3 },
    { 0, 0 },
    { 0xd64b66e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103b66ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572b66f2, 3 },
    { 0, 0 },
    { 0xd6a766f4, 3 },
    { 0x7eb66f4, 3 },
    { 0xa42366f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3b66fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1636701, 4 },
    { 0x18f76702, 4 },
    { 0xb7af6703, 4 },
    { 0xdb976703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a76707, 3 },
    { 0, 0 },
    { 0xead36709, 3 },
    { 0, 0 },
    { 0x5ba7670b, 4 },
    { 0, 0 },
    { 0xabf7670d, 4 },
    { 0xedeb670e, 4 },
    { 0x1247670f, 3 },
    { 0x16336710, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f76719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f671d, 3 },
    { 0xbe5f671e, 3 },
    { 0x6043671d, 3 },
    { 0x25bf6720, 3 },
    { 0xdf03671d, 3 },
    { 0x2697671e, 3 },
    { 0xb807671f, 4 },
    { 0x7f876724, 3 },
    { 0xbd76725, 3 },
    { 0x62fb6726, 3 },
    { 0x523f671f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523f672e, 3 },
    { 0, 0 },
    { 0x8c0f6730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f76734, 5 },
    { 0xbf76734, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x938f6738, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec13673d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36742, 3 },
    { 0xb17b6742, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736749, 3 },
    { 0, 0 },
    { 0x9633674b, 4 },
    { 0x735f674b, 3 },
    { 0, 0 },
    { 0xfeeb674e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3f6752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17436755, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x282f6758, 4 },
    { 0, 0 },
    { 0x11ef675a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b675d, 3 },
    { 0xb6af675e, 3 },
    { 0x44cb675f, 3 },
    { 0x82376760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8236765, 3 },
    { 0xda576765, 3 },
    { 0, 0 },
    { 0x9de76768, 3 },
    { 0, 0 },
    { 0xdd03676a, 3 },
    { 0x853f676b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48036771, 4 },
    { 0xc08b6771, 3 },
    { 0, 0 },
    { 0x5a076774, 3 },
    { 0xaa7b6775, 3 },
    { 0x75936775, 4 },
    { 0xdb8b6775, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8487677b, 5 },
    { 0, 0 },
    { 0xd48f677d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab676786, 3 },
    { 0x8ec36786, 4 },
    { 0xc6636786, 3 },
    { 0x78776788, 3 },
    { 0x5bcb678a, 3 },
    { 0xa1d3678a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b678e, 3 },
    { 0, 0 },
    { 0x77436790, 3 },
    { 0, 0 },
    { 0xea7b6792, 4 },
    { 0x2a036793, 4 },
    { 0xe936794, 4 },
    { 0, 0 },
    { 0x306b6796, 4 },
    { 0, 0 },
    { 0x78c76798, 4 },
    { 0x6e9f6798, 3 },
    { 0, 0 },
    { 0x19d7679b, 3 },
    { 0, 0 },
    { 0x41f679d, 3 },
    { 0, 0 },
    { 0xf5df679f, 3 },
    { 0, 0 },
    { 0xedb367a1, 3 },
    { 0xa8bb67a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72f67a7, 3 },
    { 0, 0 },
    { 0x1a7b67a9, 3 },
    { 0xd6e367aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3367b1, 3 },
    { 0, 0 },
    { 0xfb5367b3, 4 },
    { 0, 0 },
    { 0xe7f67b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x460767b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb67be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5f67c6, 4 },
    { 0xb79f67c7, 3 },
    { 0xac2367c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c367cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7767d0, 3 },
    { 0x583767d1, 3 },
    { 0xfc9367d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46367d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e367dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b67e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4b67e4, 3 },
    { 0, 0 },
    { 0x7a4767e6, 3 },
    { 0, 0 },
    { 0xee8b67e8, 4 },
    { 0xda6367e8, 3 },
    { 0xd06367ea, 3 },
    { 0x8bef67eb, 4 },
    { 0x7b4367ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cb67f3, 3 },
    { 0x49c767f3, 4 },
    { 0x7b7b67f5, 3 },
    { 0, 0 },
    { 0x2b67f7, 4 },
    { 0, 0 },
    { 0x1ce767f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d36801, 3 },
    { 0xd1cb6802, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf6817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf53681a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24af681f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0036822, 3 },
    { 0x11f76823, 3 },
    { 0x44536824, 3 },
    { 0x799b6823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf383682a, 4 },
    { 0xd57f682b, 3 },
    { 0x6823682c, 3 },
    { 0x3807682c, 3 },
    { 0xde0b682c, 3 },
    { 0, 0 },
    { 0x656b6830, 3 },
    { 0xcc636831, 3 },
    { 0, 0 },
    { 0x86036833, 4 },
    { 0, 0 },
    { 0x6e676835, 4 },
    { 0xa1836836, 3 },
    { 0xc8776837, 3 },
    { 0x75a76838, 3 },
    { 0, 0 },
    { 0x747b683a, 3 },
    { 0x1bb7683a, 4 },
    { 0xbfb683c, 4 },
    { 0x41c7683d, 3 },
    { 0x19f7683e, 3 },
    { 0xe533683e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f6846, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8b6849, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f676852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7936855, 3 },
    { 0, 0 },
    { 0x72cf6857, 3 },
    { 0, 0 },
    { 0x344b6859, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a87685c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc36860, 3 },
    { 0xdf5f6861, 3 },
    { 0xcedb6862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed3686b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d36870, 3 },
    { 0xafef6871, 4 },
    { 0, 0 },
    { 0xaf836873, 3 },
    { 0x6ab6874, 3 },
    { 0xebe36875, 4 },
    { 0x43c36876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee3687a, 3 },
    { 0xfe1f687b, 4 },
    { 0xb8af687b, 3 },
    { 0x741f687d, 3 },
    { 0x2e2b687e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad236882, 3 },
    { 0xd7e76882, 4 },
    { 0, 0 },
    { 0x95d76885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81536893, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x34536896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b368a2, 3 },
    { 0, 0 },
    { 0x628b68a4, 4 },
    { 0x134768a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6b68a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7b68b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b368b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5b68b9, 3 },
    { 0xd4368ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742368c0, 3 },
    { 0xa2d368c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5f68c7, 3 },
    { 0, 0 },
    { 0xfbff68c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cf68cf, 3 },
    { 0x3fcb68cf, 3 },
    { 0, 0 },
    { 0xf62b68d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97768d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6b68dc, 3 },
    { 0, 0 },
    { 0xed1f68de, 3 },
    { 0xf19f68df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce768e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76768e5, 3 },
    { 0x27bf68e6, 4 },
    { 0xf88b68e6, 3 },
    { 0xcaab68e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cb68f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ff68f3, 4 },
    { 0x6bdf68f4, 3 },
    { 0xe69768f5, 3 },
    { 0, 0 },
    { 0x47e368f7, 3 },
    { 0x44af68f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964f68fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479b6904, 3 },
    { 0xd7db6905, 3 },
    { 0x5bd76906, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1af690c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad736917, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287f691c, 3 },
    { 0xa2b691d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f6921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab6926, 5 },
    { 0x30e76927, 3 },
    { 0x5e7f6926, 3 },
    { 0xfdd36927, 3 },
    { 0x8b0f692a, 3 },
    { 0xc4b36927, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cb6935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a7693e, 3 },
    { 0x6607693f, 3 },
    { 0, 0 },
    { 0xcf6f6941, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf6946, 4 },
    { 0x2dfb6946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f6951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae536957, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b695e, 3 },
    { 0, 0 },
    { 0x1ccf6960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb036967, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb696c, 3 },
    { 0xa603696c, 3 },
    { 0xae6b696d, 4 },
    { 0x8957696f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806f6973, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77697b, 4 },
    { 0, 0 },
    { 0x3937697d, 3 },
    { 0x60a7697e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e76985, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d876988, 4 },
    { 0, 0 },
    { 0xb2c3698a, 3 },
    { 0xc567698a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123698e, 3 },
    { 0xa9f698e, 3 },
    { 0xa7ab6990, 3 },
    { 0x4f2f6990, 3 },
    { 0x67cf6992, 3 },
    { 0x5b776991, 3 },
    { 0xcff36994, 4 },
    { 0x150f698e, 3 },
    { 0xfd176993, 3 },
    { 0x9b076996, 3 },
    { 0, 0 },
    { 0xaf336999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x352f699c, 4 },
    { 0xd95f699c, 3 },
    { 0xbff699d, 3 },
    { 0x640b699f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf369a5, 3 },
    { 0x9c4369a6, 3 },
    { 0x2fef69a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x618769aa, 3 },
    { 0xec5769aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b69ae, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b769b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b369b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb69ba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbf69be, 5 },
    { 0, 0 },
    { 0x4b2769c0, 4 },
    { 0x2a8769c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fb69c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27b69cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1369d1, 5 },
    { 0xf65f69d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472b69dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ab69e3, 3 },
    { 0xf56769e4, 4 },
    { 0xcb9369e3, 3 },
    { 0x714769e6, 3 },
    { 0xcbd769e7, 3 },
    { 0xccd369e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad769ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7b69f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf769f6, 3 },
    { 0xa06769f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868b69fd, 4 },
    { 0x2ebb69fd, 4 },
    { 0, 0 },
    { 0x606f6a00, 3 },
    { 0xbb8f6a01, 3 },
    { 0x6f36a02, 3 },
    { 0xa3936a00, 3 },
    { 0xf6cf6a04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e76a07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb6a0b, 3 },
    { 0, 0 },
    { 0x95336a0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f6a11, 3 },
    { 0x91336a12, 4 },
    { 0, 0 },
    { 0x4576a14, 3 },
    { 0x7c4f6a15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc476a23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759f6a2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10776a31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf6a35, 4 },
    { 0, 0 },
    { 0x14cb6a37, 3 },
    { 0xd4a76a37, 3 },
    { 0x477f6a39, 3 },
    { 0x1e876a3a, 4 },
    { 0x14576a3a, 3 },
    { 0xce076a3b, 3 },
    { 0, 0 },
    { 0x732b6a3e, 4 },
    { 0xc61f6a3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fb6a43, 3 },
    { 0xd18b6a43, 3 },
    { 0x11c76a43, 3 },
    { 0x641b6a46, 3 },
    { 0, 0 },
    { 0x54036a48, 3 },
    { 0x7d576a48, 3 },
    { 0x3c736a49, 3 },
    { 0x22076a4b, 3 },
    { 0x1f1b6a4c, 3 },
    { 0x36eb6a4d, 3 },
    { 0xf0276a4e, 4 },
    { 0x5636a4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9376a52, 3 },
    { 0, 0 },
    { 0x7ce36a54, 3 },
    { 0x31636a55, 3 },
    { 0x47076a55, 3 },
    { 0xb6e76a56, 3 },
    { 0x3b6b6a56, 3 },
    { 0, 0 },
    { 0x49336a5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3136a5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76136a62, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d76a68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf476a6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5f6a74, 3 },
    { 0xdbf76a75, 3 },
    { 0, 0 },
    { 0x1eaf6a77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56f6a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b336a85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f036a88, 4 },
    { 0x6e2b6a89, 3 },
    { 0xb75f6a89, 4 },
    { 0x90e36a89, 3 },
    { 0xf5cf6a8c, 3 },
    { 0x297f6a88, 3 },
    { 0x1ca36a8e, 3 },
    { 0x14c76a8f, 3 },
    { 0xa7136a89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac36a96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f76a9a, 3 },
    { 0x7a8f6a9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1f6a9f, 3 },
    { 0x874b6aa0, 4 },
    { 0x4d676aa1, 3 },
    { 0x56b36aa2, 3 },
    { 0x7fcf6aa2, 4 },
    { 0x245f6aa2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96136aaa, 3 },
    { 0, 0 },
    { 0xda8f6aac, 3 },
    { 0xe2d76aac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34976ab1, 3 },
    { 0x64df6ab2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618f6ab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2b6abc, 3 },
    { 0, 0 },
    { 0xece76abe, 3 },
    { 0xd6736abf, 3 },
    { 0xfbef6ac0, 3 },
    { 0, 0 },
    { 0xe7b6ac2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e736ac7, 3 },
    { 0x9cb36ac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c36ad0, 3 },
    { 0x9b3b6ad0, 3 },
    { 0x9fab6ad2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce336ad5, 5 },
    { 0xb2136ad5, 3 },
    { 0, 0 },
    { 0x4ee76ad8, 3 },
    { 0x80136ad9, 3 },
    { 0, 0 },
    { 0x6ac76adb, 5 },
    { 0xc9736adb, 3 },
    { 0xc8236add, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68536ae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16f6ae3, 4 },
    { 0x2aef6ae4, 3 },
    { 0, 0 },
    { 0x9e036ae6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843b6aeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df36aee, 3 },
    { 0x295f6aef, 3 },
    { 0xd5976af0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636b6af4, 3 },
    { 0, 0 },
    { 0x85ff6af6, 3 },
    { 0, 0 },
    { 0x3f76af8, 3 },
    { 0x85d36af9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edf6afc, 3 },
    { 0x5fbb6afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2676b04, 4 },
    { 0x876f6b04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b36b08, 3 },
    { 0x629b6b09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f76b0c, 3 },
    { 0x9e1f6b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef76b11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ef6b14, 4 },
    { 0x21536b15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceeb6b19, 3 },
    { 0, 0 },
    { 0x645f6b1b, 3 },
    { 0x77936b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72236b20, 4 },
    { 0xa69b6b21, 3 },
    { 0x71e76b20, 3 },
    { 0x760f6b21, 4 },
    { 0x383b6b24, 3 },
    { 0xdba76b25, 3 },
    { 0x383b6b23, 3 },
    { 0xc75b6b20, 4 },
    { 0xa476b23, 3 },
    { 0xa2c76b27, 3 },
    { 0, 0 },
    { 0x922f6b2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9f6b2e, 4 },
    { 0x807f6b2f, 4 },
    { 0x593b6b2f, 3 },
    { 0x128b6b2f, 3 },
    { 0x3a4b6b2f, 4 },
    { 0x725b6b2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f636b37, 4 },
    { 0x92c36b38, 3 },
    { 0, 0 },
    { 0x40636b3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15af6b41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28736b47, 4 },
    { 0xe676b48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4b6b4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881b6b51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa236b54, 4 },
    { 0xf8cb6b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf36b58, 3 },
    { 0, 0 },
    { 0x738f6b5a, 3 },
    { 0, 0 },
    { 0x73cb6b5c, 3 },
    { 0x7f676b5d, 3 },
    { 0xb7a76b5d, 4 },
    { 0x94476b5c, 3 },
    { 0x2a176b5f, 4 },
    { 0xb98f6b61, 3 },
    { 0xc9376b61, 3 },
    { 0x92736b63, 3 },
    { 0, 0 },
    { 0x457b6b65, 3 },
    { 0, 0 },
    { 0x355b6b67, 5 },
    { 0x80776b68, 3 },
    { 0, 0 },
    { 0xa5bb6b6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769f6b6e, 4 },
    { 0, 0 },
    { 0x4d876b70, 3 },
    { 0xee0b6b71, 3 },
    { 0, 0 },
    { 0x7ba36b73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421b6b79, 3 },
    { 0xe5a36b7a, 3 },
    { 0x1ce36b7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44936b80, 4 },
    { 0xeb2b6b81, 4 },
    { 0x92476b82, 3 },
    { 0x10e36b83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b6b89, 4 },
    { 0x54af6b89, 3 },
    { 0x8a2f6b8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b6b92, 3 },
    { 0, 0 },
    { 0x18e36b94, 3 },
    { 0, 0 },
    { 0x70db6b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd876b9a, 3 },
    { 0xd7d36b9b, 4 },
    { 0xb8676b9b, 3 },
    { 0x154f6b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f36ba1, 3 },
    { 0x64db6ba1, 3 },
    { 0x8a776ba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab6ba8, 3 },
    { 0xf8cf6ba9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675f6bb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cf6bb4, 3 },
    { 0x8a936bb4, 4 },
    { 0xafbb6bb6, 3 },
    { 0xbcf36bb7, 4 },
    { 0x86ab6bb8, 4 },
    { 0x74df6bb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x900f6bbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x463f6bbf, 4 },
    { 0xe42f6bbf, 3 },
    { 0x4bcf6bbf, 4 },
    { 0xd5376bbf, 4 },
    { 0, 0 },
    { 0xf9876bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d76bc9, 3 },
    { 0x4a5b6bc9, 3 },
    { 0x8e8f6bca, 3 },
    { 0, 0 },
    { 0xa91b6bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f36bd5, 3 },
    { 0x775b6bd6, 4 },
    { 0x2f7f6bd7, 4 },
    { 0x19076bd5, 3 },
    { 0x3a736bd9, 4 },
    { 0xf0376bda, 3 },
    { 0x4d4f6bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6bde, 3 },
    { 0xa98f6bdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60f6be5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df6bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64b6bee, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41f6bf4, 4 },
    { 0, 0 },
    { 0x56676bf6, 3 },
    { 0xce076bf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b776bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18436c0e, 3 },
    { 0, 0 },
    { 0x9a576c10, 3 },
    { 0xbccb6c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3b6c1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x938f6c22, 3 },
    { 0x70bb6c22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50676c27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e176c2e, 4 },
    { 0, 0 },
    { 0xec5b6c30, 3 },
    { 0x508b6c31, 3 },
    { 0xee636c32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf6c36, 3 },
    { 0xc00f6c37, 3 },
    { 0x60276c36, 3 },
    { 0x92676c39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713b6c4d, 3 },
    { 0, 0 },
    { 0xb2ab6c4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576c53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b36c56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa76c5b, 4 },
    { 0, 0 },
    { 0x6076c5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236c61, 3 },
    { 0, 0 },
    { 0xd6036c63, 3 },
    { 0, 0 },
    { 0xf236c65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f376c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3736c79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ef6c7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbef6c7f, 3 },
    { 0xe3836c80, 4 },
    { 0x2fff6c81, 4 },
    { 0xe5176c82, 4 },
    { 0xeb4b6c83, 5 },
    { 0, 0 },
    { 0x18ab6c85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35b6c8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb6c91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743f6c95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50936c9b, 3 },
    { 0xb6036c9c, 3 },
    { 0, 0 },
    { 0x10336c9e, 3 },
    { 0, 0 },
    { 0x947b6ca0, 4 },
    { 0x30b36ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd076ca6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcf6ca9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384b6caf, 3 },
    { 0xa3536caf, 4 },
    { 0xd0676cb1, 3 },
    { 0xf38b6cb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4136cb5, 3 },
    { 0x677b6cb6, 3 },
    { 0xcfab6cb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3f6cba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda736cc5, 3 },
    { 0xe9d36cc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b6ccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a36ccf, 3 },
    { 0xb0a36ccf, 3 },
    { 0, 0 },
    { 0x9d576cd2, 4 },
    { 0xbe836cd2, 4 },
    { 0x7fef6cd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3476cda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b976ce0, 3 },
    { 0x22276ce1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6df6ce5, 3 },
    { 0x6fab6ce6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6ceb, 4 },
    { 0x5d536ceb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452b6cf7, 5 },
    { 0xc75b6cf8, 3 },
    { 0x662f6cf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7b6cfd, 3 },
    { 0xcff36cfe, 3 },
    { 0, 0 },
    { 0xebc76d00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x693b6d03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bf6d07, 3 },
    { 0x2e976d07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e936d0e, 4 },
    { 0xbde76d0e, 3 },
    { 0, 0 },
    { 0x9e9b6d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0af6d14, 3 },
    { 0x7a1f6d15, 3 },
    { 0xb2376d16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dab6d1e, 4 },
    { 0x179b6d1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa976d28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f6d2b, 4 },
    { 0x77036d2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x986b6d2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625b6d33, 3 },
    { 0x9f4f6d34, 3 },
    { 0xc22f6d35, 3 },
    { 0x3aeb6d35, 3 },
    { 0xb0536d37, 3 },
    { 0x9c276d33, 3 },
    { 0, 0 },
    { 0x140b6d3a, 3 },
    { 0, 0 },
    { 0xcb476d3c, 4 },
    { 0x428b6d3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52f6d4b, 5 },
    { 0x7def6d4c, 4 },
    { 0, 0 },
    { 0xa0876d4e, 3 },
    { 0x7cdb6d4f, 3 },
    { 0xf05f6d4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44f6d56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30f6d59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf6d5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec436d5f, 3 },
    { 0x77d76d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf6d63, 4 },
    { 0xe2af6d64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93636d6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87f6d6e, 3 },
    { 0, 0 },
    { 0x3f1f6d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3f6d77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e76d7c, 4 },
    { 0, 0 },
    { 0x7e676d7e, 4 },
    { 0xbf3f6d7e, 3 },
    { 0, 0 },
    { 0x570b6d81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5f6d85, 3 },
    { 0xc44b6d86, 4 },
    { 0x11336d87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff6d8d, 3 },
    { 0x25ff6d8e, 3 },
    { 0x98076d8f, 3 },
    { 0xc71f6d90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c436d94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda576d9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf336da1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424f6da5, 3 },
    { 0xf5976da5, 4 },
    { 0x537b6da5, 3 },
    { 0xd14b6da8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91236daf, 3 },
    { 0xe6ab6db0, 3 },
    { 0, 0 },
    { 0xad036db2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04b6db6, 3 },
    { 0, 0 },
    { 0xae8f6db8, 4 },
    { 0xa46f6db9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d076dc6, 4 },
    { 0, 0 },
    { 0x4c436dc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0836dcb, 3 },
    { 0, 0 },
    { 0x70ff6dcd, 5 },
    { 0xe92f6dce, 3 },
    { 0x12bf6dce, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cf6dd6, 3 },
    { 0xd7376dd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1b6dde, 3 },
    { 0, 0 },
    { 0xf48b6de0, 3 },
    { 0x74736de1, 3 },
    { 0xc6236de2, 3 },
    { 0x30ff6de3, 3 },
    { 0x72b76de2, 5 },
    { 0xf39f6de5, 3 },
    { 0, 0 },
    { 0xcb476de7, 3 },
    { 0, 0 },
    { 0x5d4b6de9, 4 },
    { 0x7c1f6dea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcf6ded, 3 },
    { 0xe4db6dee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d336df2, 3 },
    { 0, 0 },
    { 0x80a36df4, 3 },
    { 0, 0 },
    { 0xeb436df6, 4 },
    { 0, 0 },
    { 0xb9036df8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb376e07, 3 },
    { 0, 0 },
    { 0xfdf76e09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dab6e0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb76e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf6e26, 3 },
    { 0x74d76e26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf936e2a, 5 },
    { 0, 0 },
    { 0x70f36e2c, 4 },
    { 0x296f6e2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b6e30, 3 },
    { 0, 0 },
    { 0x125f6e32, 4 },
    { 0xa4df6e33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f6e37, 3 },
    { 0xf83b6e38, 3 },
    { 0x1bb36e39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4b6e3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c76e40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff636e47, 4 },
    { 0xc9676e47, 3 },
    { 0x6d4f6e48, 3 },
    { 0, 0 },
    { 0x94536e4b, 3 },
    { 0x28ab6e4b, 5 },
    { 0x6a8b6e4d, 4 },
    { 0x583b6e4e, 3 },
    { 0, 0 },
    { 0x38fb6e50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad36e53, 3 },
    { 0x156f6e54, 3 },
    { 0, 0 },
    { 0x10976e56, 3 },
    { 0xe7976e57, 4 },
    { 0, 0 },
    { 0xbbaf6e59, 3 },
    { 0, 0 },
    { 0xaa236e5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a436e61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df6e68, 3 },
    { 0xc3276e69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b6e6c, 4 },
    { 0, 0 },
    { 0xa136e6e, 4 },
    { 0x92236e6f, 3 },
    { 0x92ab6e70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb6e75, 3 },
    { 0, 0 },
    { 0x9b176e77, 4 },
    { 0x278f6e78, 3 },
    { 0xdc936e78, 3 },
    { 0x7aff6e79, 3 },
    { 0x74136e77, 4 },
    { 0xea4f6e7c, 3 },
    { 0, 0 },
    { 0xdfcf6e7e, 3 },
    { 0, 0 },
    { 0x4d36e80, 4 },
    { 0x65976e81, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b6e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a36e87, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40536e8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd576e94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298b6e9c, 3 },
    { 0, 0 },
    { 0x51cb6e9e, 3 },
    { 0, 0 },
    { 0x82176ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4b6ea7, 3 },
    { 0x34e76ea7, 3 },
    { 0x358b6ea9, 3 },
    { 0x7436ea9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eb6eb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c76eb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837b6ebe, 3 },
    { 0xa2476ebf, 3 },
    { 0xf2ab6ec0, 3 },
    { 0x13cf6ec1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb176ec6, 4 },
    { 0, 0 },
    { 0x3e8f6ec8, 3 },
    { 0x1f036ec9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0376ecd, 3 },
    { 0xe0fb6ece, 4 },
    { 0xa2d76ece, 3 },
    { 0x74b36ecd, 3 },
    { 0xcb976ecd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf6ed5, 3 },
    { 0x15fb6ed6, 3 },
    { 0xda3f6ed6, 3 },
    { 0x5cc76ed5, 4 },
    { 0x496b6ed9, 3 },
    { 0xfb3b6ed7, 3 },
    { 0, 0 },
    { 0x9b9b6edc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cf6ee4, 4 },
    { 0, 0 },
    { 0xbe8f6ee6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d76eea, 3 },
    { 0xee36eeb, 4 },
    { 0x60476eec, 3 },
    { 0x2def6eec, 4 },
    { 0x8d6f6eee, 3 },
    { 0x9c76eee, 3 },
    { 0, 0 },
    { 0x17176ef1, 3 },
    { 0xdf876ef1, 3 },
    { 0x46476ef2, 3 },
    { 0x51a36ef4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59f6ef8, 3 },
    { 0, 0 },
    { 0xd3a76efa, 3 },
    { 0x5feb6efb, 4 },
    { 0x6d876efb, 3 },
    { 0xbf836efd, 3 },
    { 0xc6276efb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e576f03, 4 },
    { 0, 0 },
    { 0x6c336f05, 4 },
    { 0x7bef6f06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67836f09, 3 },
    { 0x197f6f0a, 3 },
    { 0x97976f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b6f0e, 3 },
    { 0x500f6f0e, 3 },
    { 0, 0 },
    { 0xd2ab6f11, 3 },
    { 0xbf576f11, 3 },
    { 0, 0 },
    { 0x19736f14, 4 },
    { 0, 0 },
    { 0x77ef6f16, 3 },
    { 0x6b5b6f17, 3 },
    { 0, 0 },
    { 0x52d76f19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e836f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff876f23, 3 },
    { 0x7d776f23, 3 },
    { 0x17b36f25, 3 },
    { 0xa94f6f25, 3 },
    { 0, 0 },
    { 0x86d36f28, 4 },
    { 0, 0 },
    { 0xaae36f2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cb6f2e, 4 },
    { 0x1d976f2f, 3 },
    { 0, 0 },
    { 0x37b6f31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f6f37, 4 },
    { 0, 0 },
    { 0x50af6f39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea436f3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68536f43, 4 },
    { 0xf70b6f44, 3 },
    { 0xc8436f44, 3 },
    { 0x5b6f46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8f6f4f, 3 },
    { 0x3a5f6f50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3576f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x636b6f57, 3 },
    { 0xe0e36f57, 3 },
    { 0x3b4f6f59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5436f5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2076f5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15b6f65, 3 },
    { 0x1abb6f66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad36f6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25f6f70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d76f73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e76f77, 3 },
    { 0x14576f77, 3 },
    { 0x24476f79, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39536f87, 4 },
    { 0, 0 },
    { 0x537b6f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79476f8c, 3 },
    { 0, 0 },
    { 0xf1f6f8e, 3 },
    { 0, 0 },
    { 0x8e7f6f90, 3 },
    { 0, 0 },
    { 0xb9036f92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0b6f9e, 3 },
    { 0x1c7f6f9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b6fa4, 3 },
    { 0x5acb6fa4, 3 },
    { 0x58b76fa6, 4 },
    { 0x916f6fa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c36fad, 3 },
    { 0x45eb6fae, 3 },
    { 0xf5a36faf, 4 },
    { 0xe0b76fb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46176fb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71f6fbb, 3 },
    { 0, 0 },
    { 0x425f6fbd, 3 },
    { 0xb98b6fbe, 3 },
    { 0xcb976fbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f6fc4, 4 },
    { 0xbb736fc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214f6fd1, 3 },
    { 0x8b2b6fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9f6fde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f6fe2, 4 },
    { 0xd43f6fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x722b6fe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32536fe9, 4 },
    { 0xef3b6fea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eb6fed, 3 },
    { 0, 0 },
    { 0xa8436fef, 4 },
    { 0xd6cb6ff0, 3 },
    { 0x1ad76ff1, 3 },
    { 0x33c36ff2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37b6ff7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319b6ffe, 3 },
    { 0x81736fff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cff7004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16477010, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b37013, 3 },
    { 0x2e7b7014, 4 },
    { 0xabfb7014, 4 },
    { 0xfaaf7016, 3 },
    { 0x33b77013, 4 },
    { 0x7fab7013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880b701f, 4 },
    { 0x27cb7020, 3 },
    { 0x8683701f, 4 },
    { 0x76d77022, 3 },
    { 0xe937022, 3 },
    { 0x5d8f7021, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c77027, 4 },
    { 0x3b3b7028, 3 },
    { 0x754b7029, 3 },
    { 0x5f03702a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eff702d, 3 },
    { 0x9bc3702e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d77032, 3 },
    { 0x7e3f7033, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaf7037, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f3703c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff7045, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d77048, 4 },
    { 0xea9b7049, 3 },
    { 0x9e077049, 3 },
    { 0xa93b704b, 3 },
    { 0xdcd7704c, 3 },
    { 0xfda3704a, 4 },
    { 0, 0 },
    { 0x2dcf704f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x118b7052, 4 },
    { 0x3d3f7053, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ff7056, 3 },
    { 0, 0 },
    { 0xbc337058, 3 },
    { 0x170b7058, 3 },
    { 0, 0 },
    { 0x1453705b, 4 },
    { 0x74ab705c, 3 },
    { 0x6523705d, 3 },
    { 0, 0 },
    { 0x8637705f, 4 },
    { 0xd26f7060, 4 },
    { 0x9d477061, 3 },
    { 0xe48f7062, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb7065, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f7068, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf487706e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8737079, 4 },
    { 0x9c677079, 4 },
    { 0xf8e3707b, 3 },
    { 0xe8cb707c, 4 },
    { 0x660f707b, 3 },
    { 0x470f707a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd37083, 4 },
    { 0, 0 },
    { 0xf77b7085, 3 },
    { 0x189f7086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527708b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f7091, 3 },
    { 0x85d37091, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ef7095, 4 },
    { 0x2f837096, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf387709b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf70a1, 3 },
    { 0x936370a2, 4 },
    { 0x97c370a2, 3 },
    { 0x6cb70a4, 3 },
    { 0xef5b70a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d770a8, 3 },
    { 0x68af70a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f370ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18f70b0, 4 },
    { 0xf45370b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab70b6, 4 },
    { 0, 0 },
    { 0x665770b8, 3 },
    { 0x633b70b8, 3 },
    { 0x36b370ba, 3 },
    { 0, 0 },
    { 0xa17f70bc, 3 },
    { 0, 0 },
    { 0x48d770be, 3 },
    { 0x3eeb70bf, 3 },
    { 0x675f70c0, 3 },
    { 0xf64b70be, 3 },
    { 0x95bb70c2, 3 },
    { 0, 0 },
    { 0xf73770c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddb70c7, 4 },
    { 0x872b70c8, 3 },
    { 0xdb4770c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98af70d0, 3 },
    { 0, 0 },
    { 0x630f70d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9f70d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb70da, 4 },
    { 0, 0 },
    { 0x7ae370dc, 3 },
    { 0xa9770dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50370e0, 3 },
    { 0xb9e370e1, 3 },
    { 0xd86b70e2, 3 },
    { 0xfdcf70e2, 3 },
    { 0xc6c770e4, 4 },
    { 0xe770e5, 3 },
    { 0x28d770e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1370ea, 3 },
    { 0xa99770eb, 3 },
    { 0xc5770ec, 3 },
    { 0xd6e770ed, 4 },
    { 0x2f0370eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcab70f1, 3 },
    { 0x7c8f70f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd370fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5f70fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bf7102, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f7109, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489b710e, 3 },
    { 0, 0 },
    { 0xba737110, 3 },
    { 0x30437111, 3 },
    { 0xb04b7111, 3 },
    { 0x822f7113, 4 },
    { 0x184b7114, 3 },
    { 0x263b7114, 3 },
    { 0xc30b7116, 3 },
    { 0, 0 },
    { 0x9137118, 4 },
    { 0, 0 },
    { 0x7e97711a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3711d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b7129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933712d, 3 },
    { 0, 0 },
    { 0x1923712f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2eb7133, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f037139, 3 },
    { 0xf08f7139, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8b7141, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1477145, 3 },
    { 0, 0 },
    { 0xa237147, 3 },
    { 0x67377148, 3 },
    { 0xa6e37149, 4 },
    { 0x3703714a, 3 },
    { 0xafcb7149, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7667714e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a137158, 3 },
    { 0, 0 },
    { 0x8a0f715a, 3 },
    { 0, 0 },
    { 0x703f715c, 3 },
    { 0x253f715d, 5 },
    { 0, 0 },
    { 0x572b715f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47537162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77b7165, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762b7177, 3 },
    { 0x9c937178, 3 },
    { 0x8cfb7179, 3 },
    { 0x6f0f7179, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74437183, 4 },
    { 0xbebb7184, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e77190, 4 },
    { 0x1c77190, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff7194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2577197, 3 },
    { 0x86bb7198, 3 },
    { 0xb2977197, 3 },
    { 0x59f7198, 3 },
    { 0x64a7719b, 3 },
    { 0, 0 },
    { 0xb92f719d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249b71a1, 3 },
    { 0, 0 },
    { 0xaa4f71a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b71ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56f71ae, 3 },
    { 0xf49371af, 3 },
    { 0xd71771b0, 4 },
    { 0, 0 },
    { 0xff3771b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b371b6, 4 },
    { 0, 0 },
    { 0x1deb71b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b71bb, 3 },
    { 0xe08f71bc, 3 },
    { 0, 0 },
    { 0xbe9371be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1371c1, 3 },
    { 0xc87371c2, 3 },
    { 0xdf6771c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59771c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93f71cb, 3 },
    { 0x51b771cb, 3 },
    { 0x4c7371cd, 3 },
    { 0x182371ce, 4 },
    { 0, 0 },
    { 0x61371d0, 3 },
    { 0xac5771d1, 3 },
    { 0x303b71d0, 3 },
    { 0x282771d3, 3 },
    { 0x83f371d4, 3 },
    { 0xf1bf71d5, 3 },
    { 0xdcf771d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1b71de, 3 },
    { 0x32cf71df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f371e3, 4 },
    { 0, 0 },
    { 0xb46771e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x909771e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0771ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85b71f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f71f4, 4 },
    { 0x943771f5, 3 },
    { 0x4dab71f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f71fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af7201, 4 },
    { 0x77fb7201, 4 },
    { 0x9637203, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4f7207, 4 },
    { 0, 0 },
    { 0xa0f77209, 4 },
    { 0, 0 },
    { 0x94fb720b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cf720e, 4 },
    { 0xcd63720f, 3 },
    { 0xb0977210, 3 },
    { 0x95fb7210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e437216, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x43af7219, 3 },
    { 0x850b721a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce877222, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f7225, 3 },
    { 0xf15f7226, 4 },
    { 0x77ef7227, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c3722c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaab722f, 3 },
    { 0xccff7230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4737233, 4 },
    { 0x22a37234, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84737237, 5 },
    { 0x24eb7238, 3 },
    { 0, 0 },
    { 0xbc5b723a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f7723d, 3 },
    { 0, 0 },
    { 0x8e57723f, 3 },
    { 0, 0 },
    { 0x20437241, 4 },
    { 0, 0 },
    { 0xf737243, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cb7246, 3 },
    { 0, 0 },
    { 0xb7737248, 3 },
    { 0, 0 },
    { 0x948b724a, 3 },
    { 0xb2a3724b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cf724e, 4 },
    { 0x2c3724f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f7725a, 3 },
    { 0x43a7725b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df725f, 3 },
    { 0xe1bb7260, 3 },
    { 0xe3f7261, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e77264, 3 },
    { 0xfca77265, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x599f7268, 3 },
    { 0x27ff7269, 3 },
    { 0, 0 },
    { 0x4ef3726b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x199b726e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fb7271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f7274, 3 },
    { 0, 0 },
    { 0x643b7276, 3 },
    { 0, 0 },
    { 0xca237278, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e7727b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea07727f, 3 },
    { 0x2303727f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f728d, 3 },
    { 0xe6f3728e, 4 },
    { 0, 0 },
    { 0xdb8b7290, 3 },
    { 0x95737291, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x103f7294, 3 },
    { 0x15eb7294, 3 },
    { 0x4edf7296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3729d, 3 },
    { 0xc427729e, 3 },
    { 0xf353729f, 3 },
    { 0x41d3729d, 4 },
    { 0x21f7729d, 3 },
    { 0x535b72a0, 3 },
    { 0xc27772a3, 4 },
    { 0, 0 },
    { 0x8dd772a5, 3 },
    { 0xee1772a6, 3 },
    { 0x70ab72a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218b72ac, 3 },
    { 0x884372ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcf72b0, 3 },
    { 0x4fb372b1, 3 },
    { 0x658372b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ff72b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a772b8, 3 },
    { 0x8cbb72b9, 3 },
    { 0, 0 },
    { 0xcef72bb, 3 },
    { 0, 0 },
    { 0xb0eb72bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6372c0, 3 },
    { 0x6e4372c0, 3 },
    { 0x3e4b72c1, 3 },
    { 0, 0 },
    { 0x58eb72c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299772c8, 3 },
    { 0, 0 },
    { 0x961772ca, 3 },
    { 0xec3f72cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93372d0, 3 },
    { 0x44c372d1, 3 },
    { 0x87c772d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f72d7, 3 },
    { 0xa2772d8, 3 },
    { 0x22eb72d9, 4 },
    { 0, 0 },
    { 0x35f72db, 3 },
    { 0xb79372dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74772df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5df72ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c372f0, 3 },
    { 0, 0 },
    { 0xc2372f2, 3 },
    { 0, 0 },
    { 0x34eb72f4, 4 },
    { 0, 0 },
    { 0xb1b772f6, 4 },
    { 0x645772f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b772ff, 3 },
    { 0xff7b72ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97037303, 3 },
    { 0xbda77304, 3 },
    { 0xfccf7304, 4 },
    { 0, 0 },
    { 0x4ca37307, 4 },
    { 0xdb1b7308, 4 },
    { 0, 0 },
    { 0x71e3730a, 3 },
    { 0, 0 },
    { 0x6be7730c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f7312, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bf7318, 4 },
    { 0xc6af7319, 3 },
    { 0x4977731a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc37322, 3 },
    { 0, 0 },
    { 0x3eeb7324, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25577328, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d7732b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f7330, 3 },
    { 0xbc4b7331, 3 },
    { 0, 0 },
    { 0x156f7333, 4 },
    { 0x49037334, 3 },
    { 0xb8ef7335, 4 },
    { 0xfa6f7335, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf733d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cb7340, 3 },
    { 0x1ff77341, 3 },
    { 0x35337340, 3 },
    { 0xb2d37343, 3 },
    { 0, 0 },
    { 0x9fb37345, 3 },
    { 0x4b837346, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bf734c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecb7351, 4 },
    { 0xb5537352, 3 },
    { 0, 0 },
    { 0xf1677354, 3 },
    { 0x7d9b7355, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b7359, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35f7361, 3 },
    { 0xbb277362, 4 },
    { 0xc6037363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a737369, 3 },
    { 0, 0 },
    { 0x53ff736b, 4 },
    { 0x2763736c, 4 },
    { 0xcdff736c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0f7372, 4 },
    { 0xfbdb7372, 4 },
    { 0x2a3b7372, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fff737a, 4 },
    { 0, 0 },
    { 0xbfdb737c, 3 },
    { 0xd99f737d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f77385, 3 },
    { 0x70d37386, 5 },
    { 0xdfaf7385, 3 },
    { 0x91cf7388, 4 },
    { 0xb3937389, 3 },
    { 0x65cf7388, 3 },
    { 0xd1177389, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09f738e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe053739b, 3 },
    { 0, 0 },
    { 0x17f739d, 4 },
    { 0, 0 },
    { 0x3a13739f, 3 },
    { 0, 0 },
    { 0x311373a1, 3 },
    { 0x19cb73a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b73ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ff73b0, 3 },
    { 0, 0 },
    { 0x4ae773b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd373b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54373b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4b73bc, 4 },
    { 0xeedb73bd, 4 },
    { 0x289f73be, 3 },
    { 0xed2b73bf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c373c2, 3 },
    { 0, 0 },
    { 0x94c373c4, 3 },
    { 0xdedf73c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6773cb, 3 },
    { 0x99ab73cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f73cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff73d4, 3 },
    { 0xc6eb73d5, 4 },
    { 0x3b4373d5, 3 },
    { 0x92d373d6, 3 },
    { 0x725f73d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x133f73db, 4 },
    { 0, 0 },
    { 0xcbbb73dd, 3 },
    { 0xbf3373de, 4 },
    { 0x58f773df, 3 },
    { 0xffbf73e0, 3 },
    { 0xdfb373de, 3 },
    { 0, 0 },
    { 0x9aa773e3, 3 },
    { 0xacf773e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625b73e8, 3 },
    { 0x428373e9, 4 },
    { 0xc05773e9, 3 },
    { 0, 0 },
    { 0x201773ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0f73f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530373f8, 3 },
    { 0x49bb73f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845f73ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737f7403, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bb7406, 3 },
    { 0x2ccf7406, 3 },
    { 0xe77f7408, 3 },
    { 0x7d537409, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377411, 3 },
    { 0xd8977411, 3 },
    { 0, 0 },
    { 0x7e3f7414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cf741c, 3 },
    { 0, 0 },
    { 0x1737741e, 3 },
    { 0xcde7741f, 3 },
    { 0x70337420, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd77423, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x629b7426, 3 },
    { 0x637427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e77435, 4 },
    { 0x85bb7435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f743d, 4 },
    { 0xbe9b743e, 3 },
    { 0xc18b743e, 3 },
    { 0x191f7440, 3 },
    { 0xfc87743f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f7446, 4 },
    { 0x5e0b7447, 4 },
    { 0, 0 },
    { 0x50ab7449, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cb744c, 4 },
    { 0, 0 },
    { 0x1eab744e, 4 },
    { 0x6d4f744f, 4 },
    { 0, 0 },
    { 0x94d77451, 3 },
    { 0x86fb7451, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4937455, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb97745e, 3 },
    { 0x42ab745f, 3 },
    { 0xc743745f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bb7463, 4 },
    { 0xdd4f7464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7b7467, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c7746a, 3 },
    { 0x6aeb746b, 3 },
    { 0x3e8b746c, 4 },
    { 0xcf57746d, 3 },
    { 0, 0 },
    { 0x2a73746f, 4 },
    { 0x21877470, 3 },
    { 0, 0 },
    { 0xc92f7472, 3 },
    { 0, 0 },
    { 0x52d77474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b57747c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ef7480, 3 },
    { 0xda8f7481, 3 },
    { 0xba037482, 3 },
    { 0xce737482, 3 },
    { 0x7d9f7481, 3 },
    { 0xe1c77485, 3 },
    { 0xf977483, 4 },
    { 0xb2c77485, 3 },
    { 0, 0 },
    { 0xd54f7489, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a33748d, 3 },
    { 0x7403748d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92b7493, 4 },
    { 0xa7cb7493, 3 },
    { 0xac6b7495, 4 },
    { 0, 0 },
    { 0x12a77497, 3 },
    { 0, 0 },
    { 0x18277499, 3 },
    { 0, 0 },
    { 0x7dcf749b, 3 },
    { 0x6613749c, 3 },
    { 0xc52b749d, 4 },
    { 0xfc83749d, 3 },
    { 0xf0bb749f, 4 },
    { 0x419b749e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edb74a4, 3 },
    { 0x842374a4, 3 },
    { 0, 0 },
    { 0x9c6f74a7, 3 },
    { 0xdc7774a7, 3 },
    { 0x17cf74a9, 3 },
    { 0xf57774aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f74b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c374b5, 3 },
    { 0, 0 },
    { 0x898774b7, 3 },
    { 0, 0 },
    { 0x6ba774b9, 4 },
    { 0, 0 },
    { 0xe65f74bb, 3 },
    { 0xda1374bc, 3 },
    { 0xa51774bb, 3 },
    { 0, 0 },
    { 0x249774bf, 4 },
    { 0xfd9774c0, 3 },
    { 0, 0 },
    { 0x2db374c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f74cf, 4 },
    { 0x216b74d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x147374d3, 3 },
    { 0xbb9774d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443774e0, 3 },
    { 0x333774e0, 3 },
    { 0xa02374e2, 4 },
    { 0, 0 },
    { 0x2cb774e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdf74eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f774ee, 3 },
    { 0x81b74ef, 3 },
    { 0x2fc774f0, 4 },
    { 0, 0 },
    { 0xde0374f2, 4 },
    { 0x80f774f3, 3 },
    { 0x2d7374f3, 3 },
    { 0x8fff74f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa774fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d774ff, 3 },
    { 0xff877500, 4 },
    { 0, 0 },
    { 0x81937502, 3 },
    { 0x79437503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fb7506, 5 },
    { 0, 0 },
    { 0xed777508, 3 },
    { 0x95377509, 3 },
    { 0xe3f750a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f07750f, 3 },
    { 0x57cf7510, 3 },
    { 0xfb1b7511, 3 },
    { 0x1df37511, 3 },
    { 0x3d37511, 4 },
    { 0, 0 },
    { 0x36fb7515, 3 },
    { 0, 0 },
    { 0x49177517, 3 },
    { 0, 0 },
    { 0xf277519, 3 },
    { 0, 0 },
    { 0x9c9f751b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd7752f, 3 },
    { 0x95777530, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cf7533, 3 },
    { 0, 0 },
    { 0x84177535, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23677538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe857753c, 3 },
    { 0x9b63753c, 3 },
    { 0x9207753e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8537541, 4 },
    { 0x1c877542, 3 },
    { 0x347f7542, 3 },
    { 0xb6bb7543, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a77549, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b7754e, 4 },
    { 0xebd7754e, 3 },
    { 0x90377550, 3 },
    { 0x748b754e, 3 },
    { 0xaf637551, 4 },
    { 0x1e137553, 3 },
    { 0, 0 },
    { 0x9ce77555, 3 },
    { 0xd27f7556, 3 },
    { 0x989b7557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe163755b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4977567, 3 },
    { 0x76637568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe497756b, 3 },
    { 0x4723756c, 3 },
    { 0x14bb756b, 3 },
    { 0, 0 },
    { 0x6b13756f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b7572, 3 },
    { 0x1c2f7573, 3 },
    { 0x85bb7574, 4 },
    { 0x4b2b7572, 3 },
    { 0x86d77576, 3 },
    { 0, 0 },
    { 0xbb337578, 3 },
    { 0xa0037579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77757d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1837581, 3 },
    { 0, 0 },
    { 0xd19b7583, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ff7588, 3 },
    { 0x4dd77589, 3 },
    { 0xd713758a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505b7593, 3 },
    { 0, 0 },
    { 0x4d7b7595, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cf7599, 3 },
    { 0, 0 },
    { 0xf0ab759b, 3 },
    { 0, 0 },
    { 0xfb9b759d, 3 },
    { 0, 0 },
    { 0x6f83759f, 3 },
    { 0x8bb375a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b75a5, 4 },
    { 0, 0 },
    { 0xfd8b75a7, 4 },
    { 0x5a1f75a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6775ab, 3 },
    { 0x5fa375ab, 3 },
    { 0x26bf75ad, 3 },
    { 0, 0 },
    { 0x917375af, 3 },
    { 0, 0 },
    { 0xab5b75b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db775b9, 3 },
    { 0xdbf375ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b75bf, 3 },
    { 0, 0 },
    { 0xc91375c1, 3 },
    { 0x63d375c2, 3 },
    { 0, 0 },
    { 0xf59b75c4, 3 },
    { 0xb74775c5, 4 },
    { 0xac7375c6, 3 },
    { 0xab9375c6, 4 },
    { 0x8edb75c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf75d3, 3 },
    { 0x2bb75d3, 3 },
    { 0xe42f75d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65775d8, 3 },
    { 0x405775d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a375df, 3 },
    { 0, 0 },
    { 0xf83b75e1, 3 },
    { 0xabc375e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95775e6, 4 },
    { 0x139b75e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ab75ec, 4 },
    { 0xf01375ed, 3 },
    { 0x1a9375ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15f75f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d375fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11277603, 3 },
    { 0x4d6b7603, 4 },
    { 0, 0 },
    { 0x59837606, 3 },
    { 0x78bf7607, 4 },
    { 0xf14f7607, 4 },
    { 0x741b7609, 3 },
    { 0, 0 },
    { 0x6c8b760b, 3 },
    { 0, 0 },
    { 0x2d47760d, 4 },
    { 0xac7760e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a77613, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad677619, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b761c, 3 },
    { 0xfa63761d, 3 },
    { 0x9fcb761e, 3 },
    { 0, 0 },
    { 0xee2f7620, 3 },
    { 0xc5877621, 3 },
    { 0xaadf7622, 3 },
    { 0, 0 },
    { 0xa3af7624, 3 },
    { 0xa9777624, 3 },
    { 0xe2077625, 3 },
    { 0xdff7627, 3 },
    { 0x732b7624, 3 },
    { 0x9cfb7629, 3 },
    { 0xebeb7628, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a63762e, 4 },
    { 0x4ef3762f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e77635, 3 },
    { 0x251f7636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23763a, 5 },
    { 0x4a13763b, 4 },
    { 0, 0 },
    { 0x95ab763d, 3 },
    { 0x58af763d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade77642, 3 },
    { 0, 0 },
    { 0xbd3b7644, 3 },
    { 0xe7637645, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cb7649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f764d, 4 },
    { 0xe7e3764e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65077652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82f7655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdb765b, 4 },
    { 0x5e8f765c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8b7662, 3 },
    { 0x1fb77663, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88037668, 3 },
    { 0, 0 },
    { 0x5dd7766a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcb766d, 3 },
    { 0x5a2b766d, 3 },
    { 0xbb5f766e, 3 },
    { 0xe7cf7670, 3 },
    { 0xd02f7671, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e77675, 5 },
    { 0x246f7676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb77679, 3 },
    { 0xe68b767a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8827767e, 3 },
    { 0x3ef7767e, 3 },
    { 0xd8577680, 3 },
    { 0x70537680, 4 },
    { 0x67037681, 4 },
    { 0x14db7683, 3 },
    { 0xf39b7684, 3 },
    { 0x94977684, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf837688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546b768d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5f7695, 4 },
    { 0x52cb7696, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b769c, 4 },
    { 0xb3ab769d, 3 },
    { 0x75c7769e, 3 },
    { 0xd0eb769f, 4 },
    { 0, 0 },
    { 0x249b76a1, 3 },
    { 0, 0 },
    { 0x359776a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7376a9, 3 },
    { 0xc3d776aa, 3 },
    { 0xef4b76aa, 3 },
    { 0x50db76ac, 3 },
    { 0xe3af76ab, 4 },
    { 0xf3c776ae, 3 },
    { 0x304376af, 3 },
    { 0x605776ac, 4 },
    { 0xeefb76ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b76b4, 4 },
    { 0x56bb76b4, 3 },
    { 0x72776b5, 3 },
    { 0x27cf76b7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1f76bc, 3 },
    { 0xd6a776bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650b76c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a376c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961376ca, 3 },
    { 0x484376cb, 3 },
    { 0x9b8776cc, 4 },
    { 0x8c6b76cb, 3 },
    { 0xac1776ce, 3 },
    { 0, 0 },
    { 0x4dd776d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503776d5, 3 },
    { 0, 0 },
    { 0x361b76d7, 3 },
    { 0x7e376d7, 3 },
    { 0x15376d9, 3 },
    { 0, 0 },
    { 0xd4db76db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5b76df, 4 },
    { 0xd47776e0, 3 },
    { 0, 0 },
    { 0x637376e2, 3 },
    { 0xef4b76e3, 3 },
    { 0x14d376e2, 3 },
    { 0xf3c376e3, 3 },
    { 0xd77376e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf76f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156f76f9, 3 },
    { 0x22f376fa, 3 },
    { 0x7ee376fb, 3 },
    { 0x5eaf76fc, 3 },
    { 0x7c9b76fd, 4 },
    { 0x545b76fe, 3 },
    { 0xe75b76fb, 3 },
    { 0, 0 },
    { 0x7ccf7701, 3 },
    { 0, 0 },
    { 0xdb7703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3df7706, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecf770a, 4 },
    { 0, 0 },
    { 0xfa2b770c, 3 },
    { 0, 0 },
    { 0xa983770e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246b7712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b7715, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7771b, 3 },
    { 0x32e3771c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd837720, 3 },
    { 0, 0 },
    { 0x89577722, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee37728, 4 },
    { 0, 0 },
    { 0x6f17772a, 3 },
    { 0xee03772a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e37772f, 3 },
    { 0x81ef7730, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb7773e, 3 },
    { 0x2beb773e, 4 },
    { 0, 0 },
    { 0x898b7741, 3 },
    { 0x6bfb7741, 4 },
    { 0, 0 },
    { 0x4e577744, 3 },
    { 0xd4837745, 3 },
    { 0xae177746, 3 },
    { 0x62177745, 3 },
    { 0x9f877747, 3 },
    { 0xa5f77749, 3 },
    { 0, 0 },
    { 0x846b774b, 3 },
    { 0x8aab774c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94877750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a77753, 3 },
    { 0xace37754, 3 },
    { 0, 0 },
    { 0x37637756, 3 },
    { 0, 0 },
    { 0x2cf7758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987775b, 3 },
    { 0xfbeb775b, 3 },
    { 0x73df775d, 3 },
    { 0, 0 },
    { 0x1837775f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9b7762, 3 },
    { 0xe55b7763, 3 },
    { 0x16977764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b7767, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x698f776a, 3 },
    { 0x6803776a, 3 },
    { 0xcc5b776c, 3 },
    { 0xdd37776d, 3 },
    { 0, 0 },
    { 0x209b776f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a77773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f677776, 4 },
    { 0, 0 },
    { 0xfd437778, 4 },
    { 0xae977778, 4 },
    { 0x56377778, 3 },
    { 0xef33777b, 3 },
    { 0x36c37778, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2b777f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce277785, 3 },
    { 0x3abb7785, 3 },
    { 0xf9b7787, 5 },
    { 0x20c37785, 3 },
    { 0x21db7786, 3 },
    { 0xb98b778a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d77793, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68177797, 4 },
    { 0, 0 },
    { 0xffbf7799, 3 },
    { 0xebdb7799, 3 },
    { 0, 0 },
    { 0x3ca7779c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b779f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ef77a7, 3 },
    { 0xf66f77a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00777ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afb77af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x598b77b2, 3 },
    { 0, 0 },
    { 0xdadb77b4, 4 },
    { 0xa43f77b4, 3 },
    { 0, 0 },
    { 0x87e377b7, 3 },
    { 0x568b77b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7b77bc, 3 },
    { 0, 0 },
    { 0xcd1377be, 3 },
    { 0xf6ab77be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8377c3, 3 },
    { 0x82f377c3, 4 },
    { 0x8a1b77c5, 3 },
    { 0x579377c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0b77ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x775777cd, 4 },
    { 0x18a377cd, 4 },
    { 0x553f77cd, 4 },
    { 0, 0 },
    { 0xc66377d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0377d4, 3 },
    { 0, 0 },
    { 0xe55777d6, 4 },
    { 0x111f77d7, 3 },
    { 0x120b77d8, 4 },
    { 0x471b77d9, 5 },
    { 0x522777da, 4 },
    { 0x606f77da, 3 },
    { 0x234b77dc, 3 },
    { 0x163777d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df777e3, 4 },
    { 0, 0 },
    { 0xd73b77e5, 3 },
    { 0x895f77e6, 4 },
    { 0x681f77e7, 3 },
    { 0x2d5377e6, 3 },
    { 0xfaa777e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df77ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22377f1, 4 },
    { 0x77f777f2, 4 },
    { 0, 0 },
    { 0xce8777f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x427777f7, 3 },
    { 0x1e6777f8, 3 },
    { 0xe4cb77f7, 3 },
    { 0x24d377f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed37800, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3037803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b77806, 4 },
    { 0, 0 },
    { 0x6d77808, 3 },
    { 0x72937808, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68b7810, 3 },
    { 0xaaa77811, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02b7818, 3 },
    { 0, 0 },
    { 0x3e03781a, 3 },
    { 0xb303781b, 3 },
    { 0x88bb781c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4b7820, 3 },
    { 0x1737821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af37824, 3 },
    { 0x517f7825, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fb782e, 3 },
    { 0xbfaf782f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xceff7832, 3 },
    { 0x1baf7833, 3 },
    { 0x942b7833, 3 },
    { 0xafeb7835, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff577839, 3 },
    { 0xfdc3783a, 3 },
    { 0, 0 },
    { 0x78ef783c, 3 },
    { 0x514b783c, 4 },
    { 0, 0 },
    { 0x29df783f, 4 },
    { 0, 0 },
    { 0xcd537841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c77848, 4 },
    { 0, 0 },
    { 0xf623784a, 5 },
    { 0, 0 },
    { 0xeeef784c, 4 },
    { 0xd87b784d, 3 },
    { 0xf2f7784e, 3 },
    { 0x3213784f, 4 },
    { 0x5d4b784f, 3 },
    { 0, 0 },
    { 0x4ee37852, 3 },
    { 0, 0 },
    { 0x4cd37854, 3 },
    { 0x641f7854, 3 },
    { 0x924f7856, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7b7859, 4 },
    { 0x32df785a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7785e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8337861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823b7868, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb786e, 4 },
    { 0xf14f786e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a77874, 4 },
    { 0, 0 },
    { 0xedff7876, 4 },
    { 0x7677877, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b7881, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1777886, 4 },
    { 0xd7037886, 3 },
    { 0x27177888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1db788f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ab7892, 4 },
    { 0xe4237893, 3 },
    { 0, 0 },
    { 0x38b77895, 3 },
    { 0x60f7895, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc173789e, 4 },
    { 0, 0 },
    { 0x78a378a0, 3 },
    { 0xf3bf78a0, 3 },
    { 0x3f1f78a0, 3 },
    { 0, 0 },
    { 0xadd778a4, 3 },
    { 0, 0 },
    { 0x72df78a6, 3 },
    { 0xbc4778a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf78ab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559778b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e778b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246f78bb, 3 },
    { 0x5e9b78bc, 3 },
    { 0, 0 },
    { 0xa5b378be, 5 },
    { 0xec8f78be, 3 },
    { 0x583f78c0, 3 },
    { 0xe8b78c1, 3 },
    { 0x75378c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a378c9, 4 },
    { 0x12df78ca, 4 },
    { 0xab78c9, 3 },
    { 0x544b78ca, 4 },
    { 0x721b78ca, 4 },
    { 0xaf4778c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44b78d3, 3 },
    { 0x71778d4, 4 },
    { 0xec8f78d5, 3 },
    { 0xb81378d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac378dc, 3 },
    { 0, 0 },
    { 0x7daf78de, 3 },
    { 0x7f778df, 5 },
    { 0xd0e378e0, 4 },
    { 0x275778e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e778e5, 3 },
    { 0, 0 },
    { 0x2c8778e7, 3 },
    { 0, 0 },
    { 0x26bf78e9, 3 },
    { 0, 0 },
    { 0x174778eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451f78ef, 3 },
    { 0x90ef78f0, 3 },
    { 0x7f8778f1, 3 },
    { 0, 0 },
    { 0xadaf78f3, 3 },
    { 0x4e5f78f3, 3 },
    { 0x614f78f4, 3 },
    { 0x6d6378f6, 3 },
    { 0xe84778f6, 3 },
    { 0xb22b78f7, 3 },
    { 0x8e378f9, 3 },
    { 0xa72378f9, 3 },
    { 0xd8b378fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab7900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f7906, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8993790c, 4 },
    { 0x3faf790d, 3 },
    { 0x9b5f790d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377913, 3 },
    { 0x77377913, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7737917, 4 },
    { 0, 0 },
    { 0x1dc37919, 3 },
    { 0xf9b791a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b791f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb7923, 4 },
    { 0x77737924, 3 },
    { 0, 0 },
    { 0x3c4b7926, 3 },
    { 0xd6977926, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf47792c, 4 },
    { 0x8447792d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x982f7930, 3 },
    { 0xa1277931, 3 },
    { 0xbcb7932, 4 },
    { 0x95c77931, 3 },
    { 0x6f2f7934, 4 },
    { 0xf7837930, 3 },
    { 0x4dd37936, 3 },
    { 0x1d3f7937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eb7941, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427f7946, 4 },
    { 0x24b37947, 4 },
    { 0x2bcf7947, 3 },
    { 0xf08f7948, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6f794c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2837951, 4 },
    { 0xa2377951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5777958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351f795e, 3 },
    { 0, 0 },
    { 0xd61b7960, 3 },
    { 0xd3977961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16f7966, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbb7969, 3 },
    { 0xe9eb7969, 3 },
    { 0xc3a7796b, 3 },
    { 0x3b6f796c, 3 },
    { 0x665f796d, 3 },
    { 0x5377796e, 5 },
    { 0x45ff796c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f37976, 4 },
    { 0x2abf7976, 3 },
    { 0xb5ff7978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8b797c, 3 },
    { 0x7ddf797d, 4 },
    { 0xf36f797d, 3 },
    { 0xfa2b797e, 3 },
    { 0x324f797f, 3 },
    { 0xfff37981, 4 },
    { 0, 0 },
    { 0x2fa77983, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895b7988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a7798b, 4 },
    { 0x437b798b, 3 },
    { 0, 0 },
    { 0x2bb3798e, 3 },
    { 0, 0 },
    { 0x23b37990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbb79a0, 3 },
    { 0xfd0b79a1, 3 },
    { 0xba0379a2, 4 },
    { 0x80f379a3, 3 },
    { 0xb48779a3, 4 },
    { 0, 0 },
    { 0x455f79a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a379a9, 5 },
    { 0x3f4779aa, 3 },
    { 0x32df79aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815379af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c779bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7779c1, 4 },
    { 0, 0 },
    { 0x2f8779c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87379c8, 3 },
    { 0x5b8779c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828379d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f79d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ab79da, 3 },
    { 0xab6379da, 3 },
    { 0, 0 },
    { 0xef9b79dd, 4 },
    { 0, 0 },
    { 0x1c9f79df, 4 },
    { 0xe4ab79e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553779e4, 3 },
    { 0x1fab79e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab379e8, 3 },
    { 0x924379e9, 3 },
    { 0x609379ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab379f0, 3 },
    { 0x397779f0, 3 },
    { 0x6a7379f0, 4 },
    { 0x6fc779f2, 3 },
    { 0xd6e779f2, 3 },
    { 0x134b79f5, 3 },
    { 0xb38b79f6, 4 },
    { 0, 0 },
    { 0x1df79f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25379fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ef7a01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f7a08, 3 },
    { 0, 0 },
    { 0x7db77a0a, 3 },
    { 0xa5db7a0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf7a0e, 3 },
    { 0x34977a0e, 3 },
    { 0x19f7a0e, 4 },
    { 0x9f5b7a11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45df7a19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9237a1c, 4 },
    { 0x2d077a1d, 3 },
    { 0x54577a1e, 3 },
    { 0xd9bf7a1f, 3 },
    { 0, 0 },
    { 0x50ab7a21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0b7a24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14277a29, 3 },
    { 0x82df7a2a, 3 },
    { 0xc577a2b, 3 },
    { 0xd84b7a2c, 3 },
    { 0, 0 },
    { 0x1eb77a2e, 3 },
    { 0x4337a2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3df7a37, 3 },
    { 0x253b7a38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b37a3b, 3 },
    { 0x7f437a3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8def7a40, 3 },
    { 0x94577a41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffb7a48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36f7a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d277a54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a877a5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b7a60, 4 },
    { 0xe1ab7a60, 3 },
    { 0x9ae37a62, 3 },
    { 0x137a63, 3 },
    { 0, 0 },
    { 0xadcf7a65, 3 },
    { 0x14337a66, 3 },
    { 0, 0 },
    { 0xf9f7a68, 3 },
    { 0x7dbf7a69, 3 },
    { 0xab7f7a6a, 4 },
    { 0x919f7a6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff7a6e, 3 },
    { 0x54c37a6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbef7a73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f777a7a, 3 },
    { 0, 0 },
    { 0xe6937a7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6f7a7f, 3 },
    { 0, 0 },
    { 0x77137a81, 3 },
    { 0xddff7a82, 4 },
    { 0x7cfb7a83, 3 },
    { 0x32137a81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de77a87, 3 },
    { 0x927b7a88, 3 },
    { 0, 0 },
    { 0x50b7a8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50db7a8d, 4 },
    { 0x8a8b7a8e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e77a92, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c77a95, 3 },
    { 0x68ab7a96, 3 },
    { 0x7877a96, 3 },
    { 0, 0 },
    { 0xbdab7a99, 3 },
    { 0, 0 },
    { 0x80b7a9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8977a9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c337aa2, 3 },
    { 0x8e337aa3, 3 },
    { 0x5f4f7aa4, 3 },
    { 0x1e677aa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb37aa8, 3 },
    { 0xbceb7aa9, 3 },
    { 0xaa837aaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55b7aae, 5 },
    { 0x35e77aae, 3 },
    { 0xa1537ab0, 4 },
    { 0x922b7ab0, 3 },
    { 0x10077aaf, 3 },
    { 0xb5cb7ab2, 3 },
    { 0xb3a37ab3, 3 },
    { 0xe6537ab2, 4 },
    { 0, 0 },
    { 0x5c6f7ab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf77abb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcb7abe, 3 },
    { 0, 0 },
    { 0xf55b7ac0, 3 },
    { 0x877f7ac1, 3 },
    { 0xfa8f7ac1, 4 },
    { 0x82437ac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bf7ac6, 3 },
    { 0x30ef7ac6, 3 },
    { 0x237ac8, 3 },
    { 0xf35f7ac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e77ace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc37ad1, 4 },
    { 0x44537ad2, 3 },
    { 0xffe77ad1, 5 },
    { 0xb13f7ad1, 3 },
    { 0xabff7ad5, 4 },
    { 0x8f777ad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf77ad9, 4 },
    { 0, 0 },
    { 0xbaa77adb, 3 },
    { 0xeebf7adc, 4 },
    { 0x74d77add, 3 },
    { 0, 0 },
    { 0xbb9b7adf, 3 },
    { 0x2fbb7ae0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95937ae4, 4 },
    { 0x41c77ae5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da77ae8, 3 },
    { 0x260f7ae8, 3 },
    { 0x57977aea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a837aef, 3 },
    { 0x19f77af0, 3 },
    { 0xe86f7af0, 3 },
    { 0x505f7af0, 3 },
    { 0x5877aef, 3 },
    { 0, 0 },
    { 0x83ff7af5, 4 },
    { 0x492b7af5, 4 },
    { 0x733b7af5, 3 },
    { 0x10037af5, 4 },
    { 0x310f7af9, 3 },
    { 0x22937af8, 3 },
    { 0, 0 },
    { 0x2b7b7afc, 4 },
    { 0x17fb7afd, 3 },
    { 0, 0 },
    { 0x68977aff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8af7b02, 3 },
    { 0xe7377b03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb7b0a, 3 },
    { 0, 0 },
    { 0x5caf7b0c, 4 },
    { 0x4f677b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8b7b11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde77b14, 4 },
    { 0, 0 },
    { 0x1cdb7b16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a77b19, 4 },
    { 0x8ebb7b19, 4 },
    { 0, 0 },
    { 0xad337b1c, 4 },
    { 0xad137b1c, 3 },
    { 0x2ebf7b1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a077b24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87577b29, 3 },
    { 0x7ddf7b2a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68b7b2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da77b34, 3 },
    { 0, 0 },
    { 0x4adb7b36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd37b3c, 4 },
    { 0, 0 },
    { 0xf7877b3e, 3 },
    { 0xadbb7b3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f7b44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8937b47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c77b4b, 3 },
    { 0, 0 },
    { 0x38db7b4d, 4 },
    { 0xa177b4e, 3 },
    { 0x5d777b4f, 4 },
    { 0, 0 },
    { 0x9b9f7b51, 3 },
    { 0xa5037b52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f37b57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177b60, 3 },
    { 0x39cf7b61, 4 },
    { 0x33877b60, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf777b65, 4 },
    { 0x6677b66, 4 },
    { 0x8af77b65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f037b6f, 5 },
    { 0xcdb77b70, 4 },
    { 0x9aa37b71, 3 },
    { 0xa1bf7b70, 3 },
    { 0, 0 },
    { 0x29af7b74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f7b79, 3 },
    { 0xd98f7b7a, 3 },
    { 0, 0 },
    { 0xff9f7b7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee177b8a, 3 },
    { 0xec277b8a, 3 },
    { 0x218b7b8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db7b8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f7b93, 3 },
    { 0xa7477b93, 4 },
    { 0xdfb7b95, 4 },
    { 0xd737b95, 3 },
    { 0x453f7b96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd37b9e, 4 },
    { 0xcfe37b9e, 3 },
    { 0xb5e77b9e, 3 },
    { 0xb0ff7ba1, 3 },
    { 0, 0 },
    { 0xbf377ba3, 3 },
    { 0xa2037ba4, 3 },
    { 0xd0577ba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cb7ba9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd77bac, 3 },
    { 0x44b7bac, 3 },
    { 0, 0 },
    { 0x2fcf7baf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba77bb4, 3 },
    { 0x28377bb5, 3 },
    { 0xa41b7bb6, 3 },
    { 0xe7af7bb6, 3 },
    { 0x96777bb6, 4 },
    { 0, 0 },
    { 0xec8f7bba, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9337bbf, 5 },
    { 0x33437bbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9f7bc4, 3 },
    { 0x8c077bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f7bca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904b7bd4, 3 },
    { 0x508f7bd4, 3 },
    { 0x1f577bd6, 4 },
    { 0x1c977bd4, 3 },
    { 0x89837bd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3b7bdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58f7be1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f77be4, 3 },
    { 0, 0 },
    { 0x7eb77be6, 5 },
    { 0, 0 },
    { 0x19137be8, 3 },
    { 0x17237be8, 3 },
    { 0x277b7bea, 3 },
    { 0xa2a77beb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cb7bee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92937bf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d77bfa, 3 },
    { 0xc0237bfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7177bff, 4 },
    { 0, 0 },
    { 0xff477c01, 3 },
    { 0, 0 },
    { 0x9b5b7c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5f7c06, 4 },
    { 0xa5037c07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba437c0a, 4 },
    { 0xfbeb7c0b, 3 },
    { 0, 0 },
    { 0xcd177c0d, 4 },
    { 0, 0 },
    { 0x6b977c0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe237c13, 3 },
    { 0xa4077c14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2837c1d, 3 },
    { 0, 0 },
    { 0x542f7c1f, 4 },
    { 0x9a677c20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97677c23, 5 },
    { 0x98cb7c23, 3 },
    { 0xd62f7c25, 3 },
    { 0xa37b7c24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x917b7c29, 3 },
    { 0x2d337c2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b7c2d, 4 },
    { 0xd3737c2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b37c31, 3 },
    { 0xf7af7c32, 4 },
    { 0xd3277c33, 4 },
    { 0xc4af7c33, 3 },
    { 0xa7af7c31, 3 },
    { 0x54cf7c36, 3 },
    { 0xaae77c37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77337c3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9f7c40, 3 },
    { 0x9a477c41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5f7c4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf7c52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3f7c55, 4 },
    { 0x9e777c56, 3 },
    { 0x154f7c55, 4 },
    { 0x35a37c58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c77c5b, 3 },
    { 0xa43f7c5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16877c5f, 3 },
    { 0x10bb7c60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf337c65, 4 },
    { 0, 0 },
    { 0x65237c67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e177c6d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2f7c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59477c77, 3 },
    { 0xd3877c78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x147f7c7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd37c7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb77c85, 4 },
    { 0, 0 },
    { 0xb3b7c87, 3 },
    { 0x8c037c87, 3 },
    { 0, 0 },
    { 0xd19f7c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23b7c90, 3 },
    { 0, 0 },
    { 0x381b7c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb337c95, 4 },
    { 0x62477c96, 3 },
    { 0xb2fb7c97, 3 },
    { 0xc8337c98, 3 },
    { 0x8a9b7c96, 3 },
    { 0x21d77c9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b7ca1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be77cac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b7caf, 3 },
    { 0x99277caf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f7cb4, 3 },
    { 0x9c77cb5, 3 },
    { 0xb5877cb6, 3 },
    { 0x30d77cb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b7cbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31f7cc0, 4 },
    { 0, 0 },
    { 0xffcb7cc2, 4 },
    { 0xd9777cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ef7cc6, 3 },
    { 0xd9f7cc7, 3 },
    { 0x3d3b7cc8, 3 },
    { 0xd3af7cc6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695f7cd3, 4 },
    { 0, 0 },
    { 0xac7b7cd5, 3 },
    { 0, 0 },
    { 0x6c237cd7, 3 },
    { 0x35e77cd7, 4 },
    { 0xec937cd7, 3 },
    { 0xb4f7cd9, 4 },
    { 0, 0 },
    { 0xb4c77cdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf377cdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bb7ce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63637ce5, 3 },
    { 0, 0 },
    { 0x7b7ce7, 4 },
    { 0x9eeb7ce8, 3 },
    { 0, 0 },
    { 0xc73b7cea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e77ced, 3 },
    { 0x5be37ced, 3 },
    { 0x834f7cef, 3 },
    { 0xb67f7cf0, 4 },
    { 0xba977cf1, 3 },
    { 0x60377cee, 3 },
    { 0, 0 },
    { 0x28977cf4, 3 },
    { 0, 0 },
    { 0x8c0b7cf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3f7cfc, 3 },
    { 0, 0 },
    { 0x96837cfe, 3 },
    { 0xd5277cfe, 3 },
    { 0xc9137cff, 4 },
    { 0xd1db7d01, 3 },
    { 0xb55f7cff, 5 },
    { 0xdff7d03, 3 },
    { 0xa5bb7d04, 3 },
    { 0xe4f37cfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd137d0a, 3 },
    { 0, 0 },
    { 0x8f277d0c, 3 },
    { 0x81cb7d0d, 3 },
    { 0x28e37d0d, 3 },
    { 0xe8477d0d, 3 },
    { 0, 0 },
    { 0xdf277d11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b377d18, 3 },
    { 0x4f2f7d19, 3 },
    { 0x11bb7d19, 3 },
    { 0x215b7d18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x34777d1e, 3 },
    { 0, 0 },
    { 0xaf177d20, 3 },
    { 0xf19f7d21, 4 },
    { 0, 0 },
    { 0xa71f7d23, 3 },
    { 0, 0 },
    { 0x682b7d25, 3 },
    { 0x72bb7d25, 3 },
    { 0x64037d27, 3 },
    { 0x7b237d28, 4 },
    { 0x96ef7d29, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce437d33, 3 },
    { 0xd5737d33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda477d3b, 4 },
    { 0xbd9f7d3c, 3 },
    { 0x932f7d3b, 3 },
    { 0xa2877d3e, 3 },
    { 0xdd3f7d3e, 3 },
    { 0x9d5b7d3d, 3 },
    { 0, 0 },
    { 0x17d77d42, 5 },
    { 0, 0 },
    { 0x3ea37d44, 3 },
    { 0, 0 },
    { 0x2aef7d46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fb7d49, 3 },
    { 0x52b37d4a, 3 },
    { 0x1e8f7d4b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0f7d50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f7d53, 3 },
    { 0xd7a77d53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50b7d57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc37d5e, 5 },
    { 0x51677d5f, 3 },
    { 0xd11f7d5f, 5 },
    { 0xd9fb7d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb37d67, 3 },
    { 0x53c37d68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7d6d, 5 },
    { 0, 0 },
    { 0xb377d6f, 4 },
    { 0x4f9f7d70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529f7d74, 3 },
    { 0, 0 },
    { 0xd6e37d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f7d7e, 3 },
    { 0, 0 },
    { 0x9b3b7d80, 3 },
    { 0x401b7d80, 3 },
    { 0x85277d80, 3 },
    { 0xa90b7d80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a37d87, 3 },
    { 0x510b7d88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f77d8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f7d96, 5 },
    { 0x3ebb7d97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6f7d9e, 3 },
    { 0xf337d9f, 3 },
    { 0x65b7da0, 4 },
    { 0xe59f7da1, 3 },
    { 0xdcc77da2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39f7da8, 3 },
    { 0x1aef7da9, 3 },
    { 0x11eb7daa, 4 },
    { 0x5d7f7da8, 4 },
    { 0, 0 },
    { 0x2ac37dad, 3 },
    { 0, 0 },
    { 0xd7637daf, 3 },
    { 0x607b7db0, 4 },
    { 0, 0 },
    { 0x1877db2, 3 },
    { 0, 0 },
    { 0x20037db4, 3 },
    { 0xf0977db5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1437dbe, 3 },
    { 0xb6d77dbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f7dc2, 3 },
    { 0x55477dc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf77dc6, 5 },
    { 0, 0 },
    { 0xc977dc8, 3 },
    { 0, 0 },
    { 0x8e9f7dca, 3 },
    { 0, 0 },
    { 0x87a37dcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f37dcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6337dd2, 4 },
    { 0x2dc37dd3, 3 },
    { 0, 0 },
    { 0xb0fb7dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c277dd9, 3 },
    { 0x4f637dda, 3 },
    { 0x35f7dda, 3 },
    { 0x7bab7dd9, 3 },
    { 0xcbcf7ddb, 3 },
    { 0x90437dde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcb7de5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f77dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7def, 3 },
    { 0xda2f7df0, 4 },
    { 0x99eb7df1, 3 },
    { 0x445b7df1, 3 },
    { 0xf7f77df3, 3 },
    { 0, 0 },
    { 0xc9b7df5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bb7df8, 4 },
    { 0x24f7df9, 3 },
    { 0x97b7dfa, 3 },
    { 0, 0 },
    { 0x63c37dfc, 3 },
    { 0x9ef77dfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32b7e01, 5 },
    { 0xebab7e02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b7e05, 3 },
    { 0xf85b7e05, 3 },
    { 0, 0 },
    { 0xeb937e08, 5 },
    { 0x5a977e09, 3 },
    { 0, 0 },
    { 0x42177e0b, 4 },
    { 0, 0 },
    { 0x83ef7e0d, 3 },
    { 0x18137e0e, 4 },
    { 0x75a77e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff037e12, 3 },
    { 0xfd3b7e13, 3 },
    { 0xed777e14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3077e25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c377e28, 4 },
    { 0x79b77e29, 3 },
    { 0xa9877e29, 3 },
    { 0xd5637e2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6f7e2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19277e36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed37e3c, 3 },
    { 0x4b577e3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d677e44, 3 },
    { 0xbf637e45, 4 },
    { 0x64037e46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cb7e52, 3 },
    { 0, 0 },
    { 0x1bf37e54, 3 },
    { 0x7e6f7e54, 4 },
    { 0, 0 },
    { 0xd6f7e57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4077e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c377e5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf137e61, 4 },
    { 0, 0 },
    { 0x3e737e63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3b7e66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9df7e6c, 3 },
    { 0xd3ef7e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49df7e75, 3 },
    { 0, 0 },
    { 0x774f7e77, 3 },
    { 0, 0 },
    { 0xc5d37e79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1677e7d, 3 },
    { 0xaaf37e7e, 3 },
    { 0xe7f7e7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x38977e82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7f7e88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d377e90, 3 },
    { 0, 0 },
    { 0xe57f7e92, 3 },
    { 0x3e2f7e93, 4 },
    { 0x369f7e94, 3 },
    { 0xf3677e94, 4 },
    { 0x3cb7e96, 3 },
    { 0, 0 },
    { 0x7c037e98, 3 },
    { 0x158b7e99, 4 },
    { 0xf637e98, 3 },
    { 0x1037e9b, 3 },
    { 0x6ceb7e9c, 3 },
    { 0xf0537e9d, 3 },
    { 0x8e9b7e9e, 3 },
    { 0, 0 },
    { 0x22237ea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be77ea4, 4 },
    { 0x212f7ea4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b37ea9, 3 },
    { 0x19af7eaa, 4 },
    { 0, 0 },
    { 0x648f7eac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf937eaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c77eb2, 3 },
    { 0x89177eb3, 3 },
    { 0x99737eb3, 3 },
    { 0, 0 },
    { 0xeab37eb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4f7eb9, 4 },
    { 0xb6f7eba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bab7ebd, 3 },
    { 0x629f7ebe, 3 },
    { 0x28ef7ebf, 3 },
    { 0x13e37ebf, 3 },
    { 0x1ecb7ec1, 3 },
    { 0x1c5f7ec2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963b7ec7, 3 },
    { 0x824b7ec7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d77ecb, 3 },
    { 0x38477ecc, 5 },
    { 0xe3ef7ecd, 3 },
    { 0xc1537ece, 3 },
    { 0x584b7ecf, 3 },
    { 0, 0 },
    { 0xc0577ed1, 3 },
    { 0x61cf7ed2, 3 },
    { 0xadfb7ed3, 3 },
    { 0xcbdf7ed4, 3 },
    { 0x48677ed3, 4 },
    { 0x37bb7ed4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afb7ede, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x340f7ee1, 4 },
    { 0xe9b37ee2, 3 },
    { 0x835f7ee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f7ee7, 3 },
    { 0x6c3f7ee8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe037eee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74937ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7337ef6, 3 },
    { 0x5ea77ef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f37efe, 3 },
    { 0xf5877eff, 3 },
    { 0, 0 },
    { 0x7df7f01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38977f04, 3 },
    { 0x66fb7f05, 3 },
    { 0xe1fb7f05, 3 },
    { 0x30a77f04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4f7f0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77137f0f, 4 },
    { 0x4ba77f10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab37f14, 3 },
    { 0, 0 },
    { 0x9fc77f16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a37f19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5037f22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff77f28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6b7f2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11337f33, 3 },
    { 0x784b7f34, 3 },
    { 0, 0 },
    { 0x66337f36, 4 },
    { 0x38377f36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d877f3a, 3 },
    { 0xc1437f3a, 3 },
    { 0xcc537f3c, 4 },
    { 0x60db7f3d, 3 },
    { 0xa45b7f3e, 3 },
    { 0x7e337f3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd37f45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2b7f49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26277f52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b7f56, 3 },
    { 0, 0 },
    { 0x392f7f58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b77f5b, 3 },
    { 0x8d337f5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8477f61, 5 },
    { 0x99677f62, 3 },
    { 0xcac77f61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d237f66, 3 },
    { 0xc42b7f67, 4 },
    { 0x59a37f66, 4 },
    { 0x41f37f69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b7f6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3daf7f71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79377f75, 3 },
    { 0x7fef7f76, 3 },
    { 0x6fff7f76, 3 },
    { 0x161f7f78, 3 },
    { 0x97fb7f79, 3 },
    { 0xc737f77, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb7f7e, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b237f83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb7f86, 4 },
    { 0xe34f7f87, 3 },
    { 0, 0 },
    { 0x5177f89, 3 },
    { 0x4dfb7f89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3237f8e, 3 },
    { 0xe5ff7f8e, 3 },
    { 0x1f177f90, 3 },
    { 0, 0 },
    { 0xd3037f92, 3 },
    { 0xedcf7f92, 3 },
    { 0xe1b7f94, 4 },
    { 0, 0 },
    { 0xb3e77f96, 3 },
    { 0x3cb77f97, 3 },
    { 0xd5477f96, 3 },
    { 0x75837f98, 3 },
    { 0xd5c37f99, 5 },
    { 0x72f37f9b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59f7fa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b7fa3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3af7fad, 3 },
    { 0x908f7fae, 4 },
    { 0xcf37fae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f7fb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdf7fb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cb7fbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb7fc4, 3 },
    { 0xe9637fc4, 3 },
    { 0x7f677fc6, 3 },
    { 0, 0 },
    { 0x95737fc8, 3 },
    { 0, 0 },
    { 0x8437fca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb7fd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02f7fd5, 3 },
    { 0, 0 },
    { 0x54637fd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527f7fdb, 4 },
    { 0, 0 },
    { 0x890b7fdd, 4 },
    { 0x3def7fde, 3 },
    { 0xb05b7fdf, 3 },
    { 0x50a37fe0, 3 },
    { 0, 0 },
    { 0xadf7fe2, 3 },
    { 0, 0 },
    { 0x3eb37fe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98f7fea, 3 },
    { 0xd7777fea, 4 },
    { 0x94177fec, 3 },
    { 0, 0 },
    { 0xf26b7fee, 3 },
    { 0xff1f7fef, 3 },
    { 0x50437ff0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e177ff5, 3 },
    { 0x74437ff5, 3 },
    { 0xbb6b7ff5, 4 },
    { 0x58cf7ff5, 3 },
    { 0, 0 },
    { 0x40bb7ffa, 3 },
    { 0x24ff7ffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438004, 5 },
    { 0, 0 },
    { 0x929b8006, 3 },
    { 0x4ee78007, 3 },
    { 0x8cb78008, 3 },
    { 0x38c78009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a7800f, 4 },
    { 0x7f8f8010, 3 },
    { 0x415b800f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdb8015, 3 },
    { 0xfc2b8016, 4 },
    { 0x20df8017, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f801d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46f8021, 3 },
    { 0x4b7b8022, 4 },
    { 0xf8ef8023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb278026, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51b802a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98678036, 3 },
    { 0, 0 },
    { 0xf4838038, 4 },
    { 0x98478039, 3 },
    { 0x30d7803a, 3 },
    { 0x2cb7803b, 3 },
    { 0x37cf803b, 3 },
    { 0xbed7803a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9f8040, 4 },
    { 0, 0 },
    { 0x678f8042, 3 },
    { 0xd8138043, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce78047, 3 },
    { 0xf8a78048, 3 },
    { 0xf9478048, 3 },
    { 0xdf27804a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413f8055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5467805b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5b805e, 3 },
    { 0xb477805f, 3 },
    { 0, 0 },
    { 0x4a4f8061, 3 },
    { 0, 0 },
    { 0xaacb8063, 3 },
    { 0xb7c78064, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f8069, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bb806d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b8071, 3 },
    { 0xe4a78072, 4 },
    { 0, 0 },
    { 0x13338074, 3 },
    { 0xb45f8075, 3 },
    { 0xcf8076, 3 },
    { 0xbe738077, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa057807a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a7807f, 3 },
    { 0x47e78080, 3 },
    { 0x70a78081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e038086, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba7808a, 3 },
    { 0x21d3808b, 3 },
    { 0xd18f808c, 3 },
    { 0x199b808d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b38091, 3 },
    { 0, 0 },
    { 0xe91b8093, 4 },
    { 0, 0 },
    { 0x491f8095, 3 },
    { 0x18778095, 5 },
    { 0x1c338097, 3 },
    { 0x4f238098, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81f809b, 3 },
    { 0, 0 },
    { 0x578b809d, 4 },
    { 0, 0 },
    { 0x673809f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48780a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf80a6, 4 },
    { 0xe78f80a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3780ab, 4 },
    { 0, 0 },
    { 0x9d380ad, 3 },
    { 0x733380ad, 3 },
    { 0, 0 },
    { 0xa28b80b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a780ba, 3 },
    { 0x5c0780bb, 4 },
    { 0xb19f80bc, 3 },
    { 0xafa780bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x152f80c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f780c6, 3 },
    { 0x7ddf80c7, 4 },
    { 0x268780c8, 3 },
    { 0xc6c380c9, 3 },
    { 0, 0 },
    { 0x5a5780cb, 3 },
    { 0x8a8780cc, 3 },
    { 0x8a1780cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0f80d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd780d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830380e4, 3 },
    { 0, 0 },
    { 0xe68b80e6, 4 },
    { 0xfc1f80e7, 3 },
    { 0, 0 },
    { 0xf7bf80e9, 3 },
    { 0x816f80ea, 3 },
    { 0x243b80eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6380f1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x666b80f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb80fc, 3 },
    { 0, 0 },
    { 0xe58b80fe, 4 },
    { 0xe90380ff, 3 },
    { 0, 0 },
    { 0x90ef8101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f78108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf07810d, 3 },
    { 0, 0 },
    { 0xbddb810f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0678112, 4 },
    { 0xcbc78113, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3678116, 3 },
    { 0x83478117, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47811a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427811e, 3 },
    { 0xd2e7811f, 3 },
    { 0xf4b8120, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a38124, 3 },
    { 0xaf278124, 3 },
    { 0xbf9b8126, 4 },
    { 0xcab8124, 4 },
    { 0x4b2b8126, 3 },
    { 0, 0 },
    { 0x10ab812a, 3 },
    { 0xe8b812a, 3 },
    { 0x55b812c, 3 },
    { 0, 0 },
    { 0x6477812e, 3 },
    { 0, 0 },
    { 0x814f8130, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f8136, 3 },
    { 0x7cb8137, 3 },
    { 0xf4778138, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94af813c, 3 },
    { 0, 0 },
    { 0xfdeb813e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399f8142, 3 },
    { 0, 0 },
    { 0x51338144, 3 },
    { 0x11078145, 3 },
    { 0x1f638145, 3 },
    { 0, 0 },
    { 0xc1038148, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc63814e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b8153, 3 },
    { 0xc28b8153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96af8158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cab815f, 3 },
    { 0x959b815f, 3 },
    { 0, 0 },
    { 0x38b8162, 3 },
    { 0, 0 },
    { 0x402b8164, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d738168, 3 },
    { 0x576f8168, 5 },
    { 0xde7816a, 4 },
    { 0xfb83816b, 3 },
    { 0xe64f816a, 4 },
    { 0, 0 },
    { 0xff07816e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f8175, 3 },
    { 0, 0 },
    { 0x8bbb8177, 3 },
    { 0x3a4b8178, 3 },
    { 0x6ebb8178, 3 },
    { 0x4083817a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e78181, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b8184, 3 },
    { 0x7f178185, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcb818c, 3 },
    { 0, 0 },
    { 0xe9d7818e, 4 },
    { 0x4143818e, 3 },
    { 0, 0 },
    { 0xb6c38191, 4 },
    { 0xb38192, 3 },
    { 0xd2638193, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841b8197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227819c, 3 },
    { 0x949f819d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcff81a3, 4 },
    { 0x494381a4, 4 },
    { 0x572781a3, 4 },
    { 0, 0 },
    { 0xa6e781a7, 3 },
    { 0x27c781a8, 3 },
    { 0xf40781a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59f81ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8381b2, 3 },
    { 0, 0 },
    { 0x23eb81b4, 4 },
    { 0, 0 },
    { 0x3e7781b6, 4 },
    { 0x448381b7, 3 },
    { 0xf6c781b6, 3 },
    { 0x62781b9, 3 },
    { 0x6ef381ba, 3 },
    { 0xa3c781bb, 3 },
    { 0x2ac381bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48781c0, 3 },
    { 0x818b81c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b381c5, 5 },
    { 0, 0 },
    { 0xcc2f81c7, 3 },
    { 0xeb3b81c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a381cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88381d4, 4 },
    { 0x6cff81d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70781d9, 4 },
    { 0, 0 },
    { 0x374f81db, 3 },
    { 0xb3e781db, 3 },
    { 0x8dcb81dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283781e6, 3 },
    { 0x1f9781e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x346381ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x212781ed, 3 },
    { 0x1fa381ed, 3 },
    { 0x478381ef, 3 },
    { 0x6f0781ee, 3 },
    { 0x852381ee, 4 },
    { 0x96ef81f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42df81f5, 3 },
    { 0, 0 },
    { 0xa2ef81f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8781fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b8202, 5 },
    { 0x71538202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f8207, 3 },
    { 0, 0 },
    { 0x91978209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc03820d, 3 },
    { 0, 0 },
    { 0xccf820f, 3 },
    { 0x247b820f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14678213, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0578216, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3821b, 3 },
    { 0x1927821b, 3 },
    { 0x5d67821b, 4 },
    { 0, 0 },
    { 0xd4ff821f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49038223, 4 },
    { 0xeff8223, 3 },
    { 0, 0 },
    { 0xe07b8226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b8229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3822e, 3 },
    { 0x56ab822f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x791f8232, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b8239, 3 },
    { 0xe4c38239, 3 },
    { 0xe587823b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bf823f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f78245, 3 },
    { 0xd6cb8246, 3 },
    { 0xd7db8247, 3 },
    { 0x1b2f8248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb824b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce78251, 3 },
    { 0xe4d78251, 3 },
    { 0, 0 },
    { 0x33ab8254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecf8259, 3 },
    { 0, 0 },
    { 0xfd0f825b, 3 },
    { 0x7627825b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e438262, 3 },
    { 0xe48b8262, 4 },
    { 0xb1cb8262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f478267, 3 },
    { 0xb91b8268, 3 },
    { 0, 0 },
    { 0xf19b826a, 4 },
    { 0, 0 },
    { 0x46a7826c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6af826f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610b8274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838279, 3 },
    { 0xd74f827a, 4 },
    { 0xe87f827b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870b8281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x203f8284, 3 },
    { 0x7ae38284, 3 },
    { 0xfe438284, 3 },
    { 0xb44f8287, 4 },
    { 0x4c178288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42938290, 3 },
    { 0x4bbb8291, 5 },
    { 0xbe5f8291, 3 },
    { 0, 0 },
    { 0x46938294, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50b8297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b829b, 5 },
    { 0xe067829b, 3 },
    { 0, 0 },
    { 0xcdc7829e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x453f82a1, 3 },
    { 0, 0 },
    { 0xf36b82a3, 3 },
    { 0x677382a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0782a7, 4 },
    { 0x4e9382a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab82ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c782b4, 4 },
    { 0, 0 },
    { 0x9b3382b6, 5 },
    { 0xfaf382b6, 3 },
    { 0xb9b82b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0b82be, 3 },
    { 0xdc0f82be, 4 },
    { 0x6b82be, 4 },
    { 0x269b82be, 4 },
    { 0x4f0f82c2, 4 },
    { 0xa40f82c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981382cb, 4 },
    { 0xa6b82cb, 3 },
    { 0xbf3382cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedf82d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6af82d9, 4 },
    { 0x370382d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4382dd, 3 },
    { 0xba0f82de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3782e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439782e5, 3 },
    { 0x16cb82e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45f82ed, 3 },
    { 0x1a9f82ee, 3 },
    { 0xe06f82ed, 4 },
    { 0x9ceb82f0, 3 },
    { 0xf0bf82f0, 3 },
    { 0x2d2b82ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7382f5, 3 },
    { 0x51a382f6, 3 },
    { 0x4e5782f7, 4 },
    { 0xc97b82f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec382fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a278301, 3 },
    { 0x264f8301, 4 },
    { 0, 0 },
    { 0xb9678304, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadf8308, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d93830b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b830e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feb8311, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678314, 3 },
    { 0, 0 },
    { 0x67fb8316, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680b831b, 3 },
    { 0xedc7831c, 3 },
    { 0xeff7831b, 3 },
    { 0x9b3b831e, 3 },
    { 0x5f7b831f, 3 },
    { 0, 0 },
    { 0xac578321, 3 },
    { 0x54bf8322, 3 },
    { 0x756b8323, 3 },
    { 0, 0 },
    { 0x17df8325, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f8329, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe7832c, 4 },
    { 0x51bb832c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed38335, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b78338, 3 },
    { 0, 0 },
    { 0x3e7f833a, 3 },
    { 0, 0 },
    { 0xf9ab833c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b834a, 3 },
    { 0x3cb834b, 3 },
    { 0x1743834c, 4 },
    { 0xd597834d, 3 },
    { 0xc6bf834b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da38356, 4 },
    { 0x21038356, 3 },
    { 0xc2b38356, 5 },
    { 0, 0 },
    { 0xc9ab835a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb835f, 3 },
    { 0, 0 },
    { 0x599f8361, 3 },
    { 0x24538361, 3 },
    { 0x576f8361, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f8369, 4 },
    { 0, 0 },
    { 0x80a3836b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe3836e, 3 },
    { 0xbcb3836f, 3 },
    { 0x96778370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcb8375, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7478378, 4 },
    { 0, 0 },
    { 0xa51f837a, 3 },
    { 0xbaf837a, 3 },
    { 0x49a3837a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823f8380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceb838f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e38393, 3 },
    { 0x451b8394, 4 },
    { 0xcb678393, 4 },
    { 0xd1778396, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb7839c, 3 },
    { 0x242b839d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294b83a6, 3 },
    { 0x5a783a7, 3 },
    { 0x588783a8, 4 },
    { 0x1ba383a8, 3 },
    { 0x2a4b83a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99783ad, 3 },
    { 0x675383ae, 4 },
    { 0x9c383ae, 3 },
    { 0, 0 },
    { 0xbd7f83b1, 3 },
    { 0xdeab83b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c783b9, 3 },
    { 0x902b83ba, 5 },
    { 0x859783bb, 3 },
    { 0x6ba383bc, 3 },
    { 0x123b83ba, 3 },
    { 0, 0 },
    { 0x905383bf, 5 },
    { 0xc6ef83c0, 3 },
    { 0x78bb83c0, 4 },
    { 0x5bc383c0, 4 },
    { 0x2c4f83c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27f83c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a783ca, 3 },
    { 0x87cb83ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x415383ce, 3 },
    { 0xe21783cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eb83d3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdab83d8, 3 },
    { 0xdd8f83d8, 5 },
    { 0x892383da, 3 },
    { 0x616383db, 4 },
    { 0xe31b83db, 3 },
    { 0x102f83da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadab83e0, 3 },
    { 0xa40f83e1, 3 },
    { 0x816783e2, 3 },
    { 0xda9b83e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9783e6, 3 },
    { 0, 0 },
    { 0x5d9b83e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bf83eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bb83f1, 3 },
    { 0xb5a383f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5383f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8b83fb, 3 },
    { 0x8feb83fb, 3 },
    { 0x827b83fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f8400, 3 },
    { 0x2b9f8401, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f678404, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e38407, 3 },
    { 0, 0 },
    { 0x69978409, 3 },
    { 0xb1f3840a, 3 },
    { 0x2aab840b, 3 },
    { 0x6e8f840b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71b8410, 3 },
    { 0, 0 },
    { 0x540f8412, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c738418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f841d, 3 },
    { 0xed8b841d, 3 },
    { 0x3507841f, 3 },
    { 0xd0b7841d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f8430, 3 },
    { 0x9eff8431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f8436, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79f8439, 3 },
    { 0x6c2f843a, 3 },
    { 0xdf3843a, 3 },
    { 0x9c5b843c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d878442, 4 },
    { 0, 0 },
    { 0xe4378444, 3 },
    { 0x5ecf8445, 3 },
    { 0xed7f8444, 3 },
    { 0xd0fb8447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf77844c, 3 },
    { 0, 0 },
    { 0xa373844e, 3 },
    { 0xa697844f, 3 },
    { 0xfe538450, 3 },
    { 0x2b83844e, 3 },
    { 0, 0 },
    { 0x310b8453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70638457, 3 },
    { 0, 0 },
    { 0xeb38459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c67845c, 3 },
    { 0x9fbb845c, 3 },
    { 0, 0 },
    { 0x79c7845f, 3 },
    { 0, 0 },
    { 0xce778461, 3 },
    { 0x994b8462, 3 },
    { 0x876f8461, 3 },
    { 0, 0 },
    { 0xd12b8465, 3 },
    { 0, 0 },
    { 0x7c7f8467, 3 },
    { 0x65338467, 3 },
    { 0, 0 },
    { 0x4563846a, 3 },
    { 0, 0 },
    { 0x10bf846c, 5 },
    { 0, 0 },
    { 0x24af846e, 4 },
    { 0xbe9f846e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f78474, 3 },
    { 0, 0 },
    { 0x3f578476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b847e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66f8483, 3 },
    { 0, 0 },
    { 0xc6e38485, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f8488, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5b848d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f8493, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2b8499, 3 },
    { 0, 0 },
    { 0xe953849b, 3 },
    { 0x4673849c, 4 },
    { 0x926b849d, 3 },
    { 0xa8af849e, 4 },
    { 0xe6cf849e, 3 },
    { 0x5323849f, 3 },
    { 0xcc384a1, 3 },
    { 0xe4b784a2, 3 },
    { 0x65d784a3, 3 },
    { 0, 0 },
    { 0xbdf84a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473f84a9, 3 },
    { 0x2e5784aa, 3 },
    { 0, 0 },
    { 0x4dbf84ac, 3 },
    { 0, 0 },
    { 0xa6e384ae, 3 },
    { 0x95b84af, 3 },
    { 0x350f84b0, 3 },
    { 0xf70784ae, 3 },
    { 0xe22784ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53384b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3384b8, 4 },
    { 0x4b2784b8, 3 },
    { 0x18a784ba, 3 },
    { 0, 0 },
    { 0x133784bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4384bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a784c4, 4 },
    { 0, 0 },
    { 0x834384c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ab84ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81384ce, 3 },
    { 0, 0 },
    { 0x6c8b84d0, 3 },
    { 0xc92b84d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbf84d5, 4 },
    { 0, 0 },
    { 0x9e3784d7, 3 },
    { 0xc38f84d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf84dd, 3 },
    { 0, 0 },
    { 0xf67384df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x180f84e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdf84ea, 3 },
    { 0xf12784ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48384ee, 4 },
    { 0, 0 },
    { 0xac4384f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414784f5, 4 },
    { 0x8bd384f6, 4 },
    { 0x255784f6, 4 },
    { 0x850784f8, 3 },
    { 0xd2ff84f8, 3 },
    { 0xcf9784fa, 3 },
    { 0x264b84f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x344f84fe, 3 },
    { 0, 0 },
    { 0x4e1f8500, 3 },
    { 0x104f8501, 4 },
    { 0, 0 },
    { 0x7cf8503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b878507, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae3850a, 3 },
    { 0x9a7b850b, 3 },
    { 0x2ef850c, 3 },
    { 0x6f33850d, 3 },
    { 0x929b850d, 3 },
    { 0xe4a3850f, 4 },
    { 0x9973850f, 3 },
    { 0x89bb8510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538515, 4 },
    { 0x74278516, 4 },
    { 0xb2af8517, 3 },
    { 0x2cfb8516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb37851b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8520, 3 },
    { 0x974b8521, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f8524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa878530, 3 },
    { 0xc9b78531, 3 },
    { 0x8e238532, 4 },
    { 0, 0 },
    { 0xc8438534, 3 },
    { 0x614b8535, 3 },
    { 0xa25f8536, 3 },
    { 0, 0 },
    { 0x42e78538, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601b853c, 3 },
    { 0x941f853c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff8540, 5 },
    { 0xb8338541, 4 },
    { 0x3b4b8542, 3 },
    { 0xe22f8543, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7db8546, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9983854a, 5 },
    { 0x5f7f854b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd7854e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x73778551, 3 },
    { 0xcb8552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340b8558, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e3855d, 5 },
    { 0, 0 },
    { 0x5ccf855f, 3 },
    { 0xf6878560, 3 },
    { 0xf6f3855f, 3 },
    { 0xe203855f, 4 },
    { 0, 0 },
    { 0xc4e78564, 3 },
    { 0x957f8564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e38572, 3 },
    { 0, 0 },
    { 0xe7cb8574, 4 },
    { 0, 0 },
    { 0x39a38576, 3 },
    { 0xb78576, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926b857d, 3 },
    { 0x51cb857d, 3 },
    { 0x3033857e, 4 },
    { 0, 0 },
    { 0x4ef78581, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6738588, 3 },
    { 0x266f8589, 3 },
    { 0, 0 },
    { 0xb987858b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec7858e, 4 },
    { 0xaf57858e, 4 },
    { 0x868f8590, 3 },
    { 0x4e9b8591, 3 },
    { 0x4eb78591, 3 },
    { 0x55d78593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423b8597, 3 },
    { 0x999b8598, 4 },
    { 0xa038599, 3 },
    { 0x272f859a, 3 },
    { 0x5753859a, 3 },
    { 0x872f859c, 3 },
    { 0, 0 },
    { 0xcb57859e, 3 },
    { 0x15f7859f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x658f85a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56385a6, 5 },
    { 0, 0 },
    { 0xafe385a8, 4 },
    { 0xfb5f85a9, 4 },
    { 0xfa8785aa, 3 },
    { 0xc66385ab, 4 },
    { 0x8dcf85ac, 3 },
    { 0, 0 },
    { 0xc03b85ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fb85b6, 3 },
    { 0, 0 },
    { 0x6f9785b8, 4 },
    { 0x55f85b9, 3 },
    { 0, 0 },
    { 0xd31385bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x412b85be, 3 },
    { 0xd12785be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58785c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa785c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb85d0, 3 },
    { 0x1b9385d1, 3 },
    { 0, 0 },
    { 0xafcf85d3, 3 },
    { 0xedbf85d4, 3 },
    { 0x5c1785d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaab85d9, 3 },
    { 0x946b85da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b85de, 3 },
    { 0x6ce785df, 3 },
    { 0x655f85df, 3 },
    { 0xa7e785e1, 3 },
    { 0x684785e0, 5 },
    { 0x2cbb85e3, 3 },
    { 0x201785e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806b85e8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e785f0, 3 },
    { 0x6cbf85f0, 4 },
    { 0, 0 },
    { 0x2a5f85f3, 3 },
    { 0xdb2f85f3, 3 },
    { 0xe9b85f5, 3 },
    { 0xf20785f6, 4 },
    { 0xfb5385f7, 3 },
    { 0x2bab85f3, 3 },
    { 0x4a0785f6, 3 },
    { 0xa8f785f9, 3 },
    { 0, 0 },
    { 0x556f85fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7385ff, 3 },
    { 0x78d785ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f860c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db8610, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff8613, 4 },
    { 0xab738613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf861c, 3 },
    { 0x983b861d, 3 },
    { 0, 0 },
    { 0xd78f861f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5978622, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae538626, 5 },
    { 0xd3bf8627, 3 },
    { 0, 0 },
    { 0x3c678629, 4 },
    { 0x52a3862a, 3 },
    { 0x5ae7862b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbb862e, 3 },
    { 0, 0 },
    { 0x58f78630, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4f8633, 4 },
    { 0, 0 },
    { 0x804b8635, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd053863a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24b863e, 3 },
    { 0x12cb863f, 3 },
    { 0, 0 },
    { 0xca438641, 4 },
    { 0x42f8642, 4 },
    { 0, 0 },
    { 0x1ef8644, 3 },
    { 0xcd1f8644, 3 },
    { 0xb2bb8646, 3 },
    { 0, 0 },
    { 0x45d78648, 3 },
    { 0x79b78648, 3 },
    { 0xcf6b8649, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bef8656, 3 },
    { 0x96ab8656, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1b8663, 3 },
    { 0x15f8664, 3 },
    { 0x91638665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18b8668, 4 },
    { 0xc2478669, 3 },
    { 0x9397866a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb866e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2238675, 4 },
    { 0x19438676, 3 },
    { 0xe6f78677, 3 },
    { 0x6a138678, 3 },
    { 0x46f8679, 3 },
    { 0x6d0f8678, 4 },
    { 0, 0 },
    { 0x6eab867c, 3 },
    { 0xe24b867c, 3 },
    { 0x6cb3867c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112f8686, 3 },
    { 0xa1f38687, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd457868c, 3 },
    { 0, 0 },
    { 0xc567868e, 3 },
    { 0x3c7f868f, 4 },
    { 0xbec78690, 3 },
    { 0, 0 },
    { 0xf3578692, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20b8696, 3 },
    { 0, 0 },
    { 0x1b278698, 4 },
    { 0x56cf8699, 3 },
    { 0xfc2f869a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a786a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1786b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0386b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8b86bd, 3 },
    { 0, 0 },
    { 0xb0786bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f86c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee386cc, 4 },
    { 0x402f86cd, 3 },
    { 0xf51386cc, 3 },
    { 0x569786cf, 3 },
    { 0x6ae386d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6b86d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10786db, 3 },
    { 0x976786dc, 4 },
    { 0, 0 },
    { 0xf17386de, 3 },
    { 0xed4b86de, 3 },
    { 0xf3b386de, 5 },
    { 0, 0 },
    { 0x5c6b86e2, 3 },
    { 0x92e386e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f86e6, 3 },
    { 0x463f86e7, 3 },
    { 0, 0 },
    { 0xe16f86e9, 3 },
    { 0, 0 },
    { 0xc42f86eb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54786f2, 3 },
    { 0x337386f3, 3 },
    { 0xf1d386f3, 4 },
    { 0xb30b86f3, 4 },
    { 0xe5cf86f2, 3 },
    { 0xc14f86f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x267386fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fb86fe, 4 },
    { 0xfe3b86ff, 3 },
    { 0, 0 },
    { 0x2c9b8701, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef8705, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceb870a, 4 },
    { 0x5ab3870a, 3 },
    { 0x3b63870c, 3 },
    { 0x1f9b870c, 4 },
    { 0x9ceb870e, 4 },
    { 0xea7f870f, 3 },
    { 0x258f870c, 3 },
    { 0xdb4f8711, 4 },
    { 0xdfef8712, 4 },
    { 0x12b8713, 3 },
    { 0x614f8712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x691f8717, 3 },
    { 0x49138718, 3 },
    { 0, 0 },
    { 0x71c7871a, 4 },
    { 0x655f871b, 3 },
    { 0xe933871c, 3 },
    { 0x4797871d, 3 },
    { 0xa71b871e, 3 },
    { 0xab43871d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6f8723, 3 },
    { 0x133b8724, 3 },
    { 0, 0 },
    { 0x135f8726, 4 },
    { 0x21a78726, 3 },
    { 0xcb2f8726, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f872b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x906b872e, 4 },
    { 0xcfd7872f, 3 },
    { 0x62bb8730, 3 },
    { 0xf67f8731, 4 },
    { 0x1e578731, 3 },
    { 0xb0b78733, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b078738, 5 },
    { 0, 0 },
    { 0x95eb873a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8b873d, 3 },
    { 0xcbc3873e, 4 },
    { 0, 0 },
    { 0xe4778740, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d378743, 3 },
    { 0, 0 },
    { 0x4f038745, 3 },
    { 0, 0 },
    { 0x592f8747, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977874a, 3 },
    { 0x8f57874b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdf874e, 3 },
    { 0x9877874e, 4 },
    { 0x685b8750, 3 },
    { 0x194b8750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d3875a, 3 },
    { 0x83f7875a, 3 },
    { 0x38bb875c, 3 },
    { 0, 0 },
    { 0x7613875e, 3 },
    { 0x1323875f, 3 },
    { 0x4df7875f, 3 },
    { 0x7e0f8761, 4 },
    { 0, 0 },
    { 0x2f9b8763, 3 },
    { 0x1fff8764, 3 },
    { 0, 0 },
    { 0x2a038766, 4 },
    { 0, 0 },
    { 0x11f78768, 3 },
    { 0xf3778769, 4 },
    { 0x1a7b8768, 4 },
    { 0, 0 },
    { 0x9d97876c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8b8770, 4 },
    { 0, 0 },
    { 0x91f8772, 4 },
    { 0x7bbb8772, 3 },
    { 0, 0 },
    { 0x9ecb8775, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed78779, 3 },
    { 0x412f877a, 3 },
    { 0x5ea3877b, 3 },
    { 0x94d7877b, 3 },
    { 0xb97f877d, 3 },
    { 0x20ff877e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b8786, 3 },
    { 0xce978787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3b878a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3878d, 3 },
    { 0xe0e3878e, 3 },
    { 0x1453878d, 3 },
    { 0x71c38790, 3 },
    { 0xa29f8791, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb38794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff7879b, 3 },
    { 0xaa87879b, 3 },
    { 0xd24b879d, 3 },
    { 0x9acf879b, 4 },
    { 0x74df879f, 3 },
    { 0x2c1387a0, 4 },
    { 0x4afb87a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x991b87a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b87a8, 3 },
    { 0x2eeb87a9, 4 },
    { 0x150387a8, 3 },
    { 0, 0 },
    { 0x95a787ac, 3 },
    { 0x6c0787ad, 3 },
    { 0xb6b87ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3387b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf787b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f87b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ab87be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b87c3, 3 },
    { 0x4dbb87c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x810f87c7, 3 },
    { 0, 0 },
    { 0xd02787c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x63df87cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9387d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84387d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b87d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b787da, 4 },
    { 0x8e2787db, 3 },
    { 0, 0 },
    { 0x2cf787dd, 3 },
    { 0x346787de, 3 },
    { 0xc2df87dd, 3 },
    { 0xb39f87e0, 4 },
    { 0, 0 },
    { 0xd37787e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158787e7, 3 },
    { 0x4a4f87e7, 3 },
    { 0, 0 },
    { 0xa08b87ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f387ef, 4 },
    { 0, 0 },
    { 0xb8bb87f1, 4 },
    { 0x982f87f1, 3 },
    { 0xa0787f3, 3 },
    { 0xb15f87f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76787fa, 3 },
    { 0, 0 },
    { 0xee3387fc, 5 },
    { 0xccff87fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b8805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736b8809, 3 },
    { 0, 0 },
    { 0x7c07880b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574b8810, 3 },
    { 0xe0578811, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e538817, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fb881a, 4 },
    { 0xb6b881b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f838825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93882a, 4 },
    { 0xd63b882b, 4 },
    { 0, 0 },
    { 0x1e37882d, 3 },
    { 0x971b882d, 3 },
    { 0x6dc7882f, 3 },
    { 0x902f882f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4678833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d78837, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f883a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a97883d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ef8841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8b8844, 3 },
    { 0xac3b8844, 3 },
    { 0xc84f8846, 3 },
    { 0, 0 },
    { 0x1c6b8848, 3 },
    { 0x93638848, 4 },
    { 0x33b3884a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbf884f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf8855, 4 },
    { 0xb4c78855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef885a, 4 },
    { 0xf07885a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beb885f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e38862, 4 },
    { 0x14678862, 3 },
    { 0x9ad78864, 3 },
    { 0x465b8865, 3 },
    { 0xa1ef8865, 3 },
    { 0x15cf8867, 3 },
    { 0x8f6f8862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb83886b, 4 },
    { 0, 0 },
    { 0x20df886d, 4 },
    { 0x83c7886e, 3 },
    { 0x10a7886f, 3 },
    { 0x54978870, 4 },
    { 0x40db886e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e38875, 3 },
    { 0x5c1b8876, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119f887b, 5 },
    { 0xcc7887c, 3 },
    { 0x2287887d, 3 },
    { 0x2a7f887b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a38882, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adf8885, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b8888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c87888b, 3 },
    { 0xafdf888c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x386f888f, 3 },
    { 0, 0 },
    { 0x161f8891, 3 },
    { 0x2c638892, 4 },
    { 0xf138893, 4 },
    { 0, 0 },
    { 0x4d078895, 4 },
    { 0xc3df8896, 3 },
    { 0x204b8896, 3 },
    { 0, 0 },
    { 0xdd778899, 3 },
    { 0xe403889a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d3889f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b88a5, 3 },
    { 0x76d388a5, 3 },
    { 0xce7b88a6, 3 },
    { 0xd75388a8, 3 },
    { 0xb2bf88a9, 3 },
    { 0x9e3b88a8, 3 },
    { 0xb68388ab, 3 },
    { 0x4a4f88ab, 4 },
    { 0xccb788a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80b88b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22b88b5, 3 },
    { 0xcbcf88b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29f88bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49f88c0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f88c5, 3 },
    { 0, 0 },
    { 0x95ef88c7, 4 },
    { 0x39388c7, 3 },
    { 0xe14388c9, 3 },
    { 0x4f4b88c9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f88d1, 4 },
    { 0x38fb88d1, 3 },
    { 0x185f88d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x879b88d6, 4 },
    { 0xc81788d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3788e4, 3 },
    { 0xdc5388e4, 4 },
    { 0, 0 },
    { 0x4a2388e7, 3 },
    { 0xf13f88e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10f88f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b88fb, 4 },
    { 0, 0 },
    { 0x700b88fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1678901, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50f8906, 4 },
    { 0x404f8906, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd43890a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b890e, 3 },
    { 0x7d4f890f, 3 },
    { 0x57cf8910, 4 },
    { 0xfcaf8911, 3 },
    { 0x1acf8912, 3 },
    { 0xf67b8911, 3 },
    { 0, 0 },
    { 0x9b1b8915, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6f8918, 4 },
    { 0x741f8919, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee578920, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4f8924, 3 },
    { 0x3c638925, 3 },
    { 0, 0 },
    { 0xaf978927, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c3892d, 3 },
    { 0x723892d, 3 },
    { 0, 0 },
    { 0x8cf38930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7b8938, 4 },
    { 0xe69b8938, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27893d, 3 },
    { 0xf137893d, 3 },
    { 0xe9f893f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9138944, 4 },
    { 0, 0 },
    { 0x8bcf8946, 3 },
    { 0xe6778947, 3 },
    { 0, 0 },
    { 0x14d78949, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf894e, 3 },
    { 0xb08b894f, 4 },
    { 0x61378950, 5 },
    { 0x3a47894f, 3 },
    { 0, 0 },
    { 0x2f978953, 3 },
    { 0xbb538953, 3 },
    { 0xb7078955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745f895e, 3 },
    { 0x9b6f895f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984b8964, 3 },
    { 0x62ef8964, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e3896a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1507896d, 3 },
    { 0xae87896e, 3 },
    { 0x21c3896f, 4 },
    { 0xb54b8970, 3 },
    { 0xae0f8970, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2f897e, 3 },
    { 0xa9ff897f, 4 },
    { 0x8463897e, 3 },
    { 0xbf8b8981, 4 },
    { 0xf52b8981, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79df8988, 4 },
    { 0x66178989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe913898c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b8991, 3 },
    { 0, 0 },
    { 0xeb278993, 3 },
    { 0x8f278994, 4 },
    { 0, 0 },
    { 0x1b78996, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x983f8999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57b899c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x342b899f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0b89a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cff89aa, 3 },
    { 0, 0 },
    { 0x46789ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eb89b4, 3 },
    { 0x5e8b89b4, 3 },
    { 0x1fff89b5, 3 },
    { 0xd37389b7, 3 },
    { 0xa5db89b5, 3 },
    { 0x647f89b9, 3 },
    { 0, 0 },
    { 0x76db89bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daf89c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693389c6, 3 },
    { 0, 0 },
    { 0x212b89c8, 3 },
    { 0xb4cf89c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4f89d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249389d5, 4 },
    { 0xf40b89d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59b89d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1389df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1789e2, 3 },
    { 0xd72f89e2, 4 },
    { 0xfcbf89e4, 3 },
    { 0xbd4f89e4, 3 },
    { 0, 0 },
    { 0x88e789e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537b89eb, 4 },
    { 0xbddb89eb, 3 },
    { 0xc9b789ed, 3 },
    { 0x277389ed, 4 },
    { 0, 0 },
    { 0x687f89f0, 3 },
    { 0, 0 },
    { 0xacdf89f2, 4 },
    { 0x87ff89f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f89fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b38a01, 4 },
    { 0x4ff8a02, 3 },
    { 0x4f8b8a03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2b8a06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0978a0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b8a0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53f8a11, 3 },
    { 0x81738a12, 3 },
    { 0x865f8a13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87f8a19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa038a1e, 3 },
    { 0x26bb8a1e, 4 },
    { 0x271b8a1e, 3 },
    { 0xe9f38a1f, 3 },
    { 0x1aa78a1e, 3 },
    { 0xf2d78a23, 3 },
    { 0x76b78a23, 3 },
    { 0x947b8a25, 3 },
    { 0, 0 },
    { 0x8a3f8a27, 3 },
    { 0x64b38a28, 4 },
    { 0x59978a29, 3 },
    { 0xeda38a2a, 4 },
    { 0x6b7f8a27, 4 },
    { 0, 0 },
    { 0x3d8b8a2d, 3 },
    { 0x67d38a2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3db8a32, 3 },
    { 0, 0 },
    { 0xe0c38a34, 3 },
    { 0xb7cb8a35, 3 },
    { 0xbd3f8a36, 4 },
    { 0x90878a37, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b078a3b, 3 },
    { 0, 0 },
    { 0xdf638a3d, 4 },
    { 0, 0 },
    { 0xbaf8a3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778a42, 3 },
    { 0, 0 },
    { 0x98f38a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcb8a4b, 3 },
    { 0xc0b8a4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x261b8a4f, 4 },
    { 0xb5e78a50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4738a5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15378a5f, 4 },
    { 0xa7738a5f, 3 },
    { 0, 0 },
    { 0x37038a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b8a65, 3 },
    { 0x4c3f8a65, 3 },
    { 0, 0 },
    { 0x744b8a68, 4 },
    { 0, 0 },
    { 0xfb3f8a6a, 3 },
    { 0x60038a6b, 4 },
    { 0x7fdf8a6c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68478a70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5f8a75, 3 },
    { 0x80cb8a75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b8a82, 4 },
    { 0, 0 },
    { 0x237f8a84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28f8a8b, 4 },
    { 0, 0 },
    { 0x898b8a8d, 3 },
    { 0x22938a8e, 3 },
    { 0xf5378a8f, 4 },
    { 0x45b8a8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b8a94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf8a97, 4 },
    { 0xab8a98, 4 },
    { 0xd2d38a99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c978a9d, 3 },
    { 0x15fb8a9d, 3 },
    { 0, 0 },
    { 0x52cf8aa0, 3 },
    { 0x358b8aa0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4f8aa4, 3 },
    { 0, 0 },
    { 0x2e4f8aa6, 3 },
    { 0x8e438aa7, 3 },
    { 0x43178aa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6138aab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64638ab1, 4 },
    { 0x4acb8ab2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68b8ab8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b38abd, 3 },
    { 0xf2bb8abd, 4 },
    { 0, 0 },
    { 0xb4a38ac0, 3 },
    { 0xcaab8ac0, 3 },
    { 0, 0 },
    { 0x57378ac3, 3 },
    { 0x42fb8ac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0db8acd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62278ad1, 3 },
    { 0, 0 },
    { 0xb4f78ad3, 3 },
    { 0, 0 },
    { 0x30978ad5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbb8ad9, 3 },
    { 0xe478ada, 3 },
    { 0, 0 },
    { 0xfeb78adc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbf8ae0, 3 },
    { 0xd7e38ae1, 3 },
    { 0x53238ae2, 5 },
    { 0, 0 },
    { 0xec5b8ae4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56538aef, 3 },
    { 0x174b8af0, 4 },
    { 0xbcdb8af0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60038afe, 3 },
    { 0x93ab8afe, 3 },
    { 0x7b078afe, 3 },
    { 0x4e238b01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cb8b0c, 3 },
    { 0x5be78b0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb8b10, 4 },
    { 0xceb8b11, 4 },
    { 0x1c338b12, 3 },
    { 0x6738b13, 3 },
    { 0xcc778b13, 3 },
    { 0xeac38b10, 3 },
    { 0, 0 },
    { 0xd7b8b17, 4 },
    { 0x20f8b18, 4 },
    { 0xec5b8b18, 3 },
    { 0x7ea38b19, 3 },
    { 0x61df8b1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77638b20, 4 },
    { 0, 0 },
    { 0x56bb8b22, 3 },
    { 0x22978b23, 3 },
    { 0xc42f8b24, 4 },
    { 0xe0df8b24, 4 },
    { 0xe4e38b24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ff8b2b, 3 },
    { 0, 0 },
    { 0xd8638b2d, 3 },
    { 0x984f8b2d, 5 },
    { 0x83bb8b2f, 3 },
    { 0, 0 },
    { 0x65b38b31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385f8b36, 3 },
    { 0xce838b37, 3 },
    { 0x84bb8b38, 3 },
    { 0x8d0f8b39, 3 },
    { 0x63038b39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131f8b3f, 3 },
    { 0, 0 },
    { 0x2bdb8b41, 4 },
    { 0, 0 },
    { 0x66778b43, 4 },
    { 0x9eaf8b44, 4 },
    { 0xb8938b44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86978b4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2b8b4f, 3 },
    { 0x37a78b50, 3 },
    { 0xaee38b51, 4 },
    { 0x95138b4f, 3 },
    { 0, 0 },
    { 0xc0ff8b54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fb8b58, 5 },
    { 0x97578b59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb8b5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b8b62, 3 },
    { 0xf59f8b63, 4 },
    { 0, 0 },
    { 0x650f8b65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6b8b70, 5 },
    { 0xe11b8b71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7078b78, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7678b7d, 3 },
    { 0x4c938b7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad338b84, 3 },
    { 0xa3fb8b85, 3 },
    { 0x89678b86, 3 },
    { 0x5ce38b87, 4 },
    { 0x99df8b88, 3 },
    { 0xf44f8b89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906f8b8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ab8b92, 3 },
    { 0xc22f8b93, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a578b98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6678b9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1b8b9f, 3 },
    { 0x1feb8ba0, 3 },
    { 0, 0 },
    { 0xd3db8ba2, 3 },
    { 0, 0 },
    { 0xe1a38ba4, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871b8ba8, 3 },
    { 0x296b8ba9, 3 },
    { 0x74f78baa, 3 },
    { 0xb54b8ba9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fb8bb3, 3 },
    { 0xd0978bb4, 3 },
    { 0x2a878bb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd38bb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66678bc2, 3 },
    { 0x13b38bc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34f8bc6, 3 },
    { 0x6ec78bc6, 3 },
    { 0xf9438bc8, 3 },
    { 0x2fb78bc9, 4 },
    { 0, 0 },
    { 0xeaab8bcb, 3 },
    { 0xc68f8bcc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934b8bd5, 3 },
    { 0, 0 },
    { 0x4ecf8bd7, 4 },
    { 0x7b978bd7, 4 },
    { 0, 0 },
    { 0x11878bda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f8bde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adb8bed, 3 },
    { 0, 0 },
    { 0xcc838bef, 4 },
    { 0x795f8bf0, 4 },
    { 0x7d938bef, 3 },
    { 0xa12f8bf0, 4 },
    { 0xfdf78bf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cb8bf8, 3 },
    { 0x12b38bf9, 4 },
    { 0, 0 },
    { 0x620f8bfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30038bfe, 3 },
    { 0x8eaf8bff, 3 },
    { 0xa0f78bfe, 3 },
    { 0x877f8c01, 4 },
    { 0xde7b8c01, 4 },
    { 0x91538c03, 4 },
    { 0xc6eb8c04, 3 },
    { 0xa57b8c05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda778c08, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7638c0c, 3 },
    { 0xf5df8c0c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a78c10, 4 },
    { 0x6d6b8c11, 4 },
    { 0xef638c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13b8c1c, 3 },
    { 0x99738c1d, 4 },
    { 0, 0 },
    { 0x59238c1f, 5 },
    { 0x43c78c1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c24, 3 },
    { 0x53b38c25, 3 },
    { 0x2f738c26, 3 },
    { 0, 0 },
    { 0x85f38c28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37b8c2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc38c2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4b8c31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2238c35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b8c3b, 3 },
    { 0xd37f8c3b, 3 },
    { 0, 0 },
    { 0xc4eb8c3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab8c41, 5 },
    { 0x57478c41, 3 },
    { 0, 0 },
    { 0xb2bb8c44, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a78c49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f8c4c, 3 },
    { 0x6df38c4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b8c54, 4 },
    { 0xbb378c54, 3 },
    { 0xdfaf8c56, 3 },
    { 0x415f8c56, 4 },
    { 0x6e6b8c58, 3 },
    { 0x8d8f8c54, 3 },
    { 0, 0 },
    { 0x1b0f8c5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856b8c60, 3 },
    { 0, 0 },
    { 0x19938c62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33f8c65, 3 },
    { 0x53f38c65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9b8c6b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcab8c6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f8c72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f8c77, 3 },
    { 0xf8b8c78, 3 },
    { 0xc1c38c77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b578c7e, 3 },
    { 0, 0 },
    { 0xddf78c80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee78c83, 3 },
    { 0x21478c84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef278c8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1038c99, 3 },
    { 0x5b638c9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d038c9e, 5 },
    { 0xbcab8c9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8678ca2, 3 },
    { 0xa65f8ca3, 3 },
    { 0x21eb8ca4, 4 },
    { 0xd0378ca5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bf8cab, 5 },
    { 0x3c378cac, 3 },
    { 0, 0 },
    { 0xac9f8cae, 4 },
    { 0xb2b38cae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a938cb4, 3 },
    { 0, 0 },
    { 0x71e38cb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x298f8cb9, 4 },
    { 0, 0 },
    { 0xdff38cbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b38cbf, 3 },
    { 0xa4238cbf, 3 },
    { 0x63bf8cc1, 3 },
    { 0xc7eb8cc2, 4 },
    { 0xc9bf8cc0, 3 },
    { 0xf04b8cc4, 3 },
    { 0x88b38cbf, 3 },
    { 0, 0 },
    { 0x245f8cc7, 3 },
    { 0x33978cc8, 4 },
    { 0x87b38cc8, 3 },
    { 0x4ec78cca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c78cce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccb8cd2, 3 },
    { 0, 0 },
    { 0x2cb78cd4, 3 },
    { 0, 0 },
    { 0xb6138cd6, 4 },
    { 0, 0 },
    { 0x912f8cd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da78cdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec38cdf, 3 },
    { 0, 0 },
    { 0x935b8ce1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3df8cea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2f8cf3, 3 },
    { 0xe7878cf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99b8d01, 3 },
    { 0x5d638d02, 3 },
    { 0xa2e38d01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bab8d0d, 3 },
    { 0, 0 },
    { 0x69f78d0f, 3 },
    { 0, 0 },
    { 0xebb78d11, 3 },
    { 0, 0 },
    { 0xa34b8d13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f38d16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36f8d1d, 3 },
    { 0x2a7b8d1e, 3 },
    { 0xfa878d1e, 3 },
    { 0, 0 },
    { 0xa9f78d21, 3 },
    { 0xe1fb8d22, 3 },
    { 0xd5df8d22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d78d2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af8d35, 4 },
    { 0x2a138d36, 4 },
    { 0x1ccf8d37, 3 },
    { 0x902b8d38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f38d3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f78d41, 3 },
    { 0xaff78d42, 4 },
    { 0, 0 },
    { 0x72f38d44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbf8d47, 4 },
    { 0xff538d48, 3 },
    { 0x124f8d48, 3 },
    { 0x9d4f8d4a, 3 },
    { 0xd25b8d4a, 4 },
    { 0xa0a38d4c, 3 },
    { 0x98e38d4d, 3 },
    { 0x6d678d4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f8d51, 3 },
    { 0x5838d51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d38d55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b178d58, 3 },
    { 0, 0 },
    { 0xfaf8d5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cb8d62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255f8d69, 3 },
    { 0x2c438d6a, 3 },
    { 0xd8eb8d6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb8d72, 3 },
    { 0xff478d72, 3 },
    { 0xdb5b8d74, 4 },
    { 0, 0 },
    { 0x27578d76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8838d79, 4 },
    { 0xabd78d7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96238d85, 4 },
    { 0, 0 },
    { 0x62e38d87, 3 },
    { 0xf1ab8d88, 3 },
    { 0x853f8d89, 4 },
    { 0, 0 },
    { 0x67cf8d8b, 3 },
    { 0, 0 },
    { 0x3d38d8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ab8d91, 3 },
    { 0xd9cf8d92, 3 },
    { 0x947f8d92, 4 },
    { 0x88cb8d94, 3 },
    { 0x655f8d94, 3 },
    { 0xcde38d95, 4 },
    { 0x9fcb8d91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8d9a, 4 },
    { 0x1e5b8d9a, 3 },
    { 0x4f638d9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54f8da5, 3 },
    { 0xe5eb8da5, 3 },
    { 0xa11f8da7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91338dad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322f8db1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec378db6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f8db9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23f8dbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd78dc1, 4 },
    { 0x7c138dc1, 3 },
    { 0xa23f8dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f8dcb, 4 },
    { 0, 0 },
    { 0xe1038dcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3338dd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e738dd5, 4 },
    { 0xf3e38dd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b8dd9, 4 },
    { 0, 0 },
    { 0x6df78ddb, 4 },
    { 0, 0 },
    { 0x77a38ddd, 5 },
    { 0xa978dde, 3 },
    { 0x8b38dde, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6b8de2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe78de7, 3 },
    { 0, 0 },
    { 0xba678de9, 4 },
    { 0, 0 },
    { 0xaaef8deb, 3 },
    { 0xfb0b8dec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b38df0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56b8df3, 4 },
    { 0, 0 },
    { 0xc438df5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe438dfb, 3 },
    { 0, 0 },
    { 0x272b8dfd, 3 },
    { 0, 0 },
    { 0x657b8dff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb8e02, 3 },
    { 0x96538e03, 3 },
    { 0x993b8e04, 3 },
    { 0x5c6f8e03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39f8e0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e038e10, 3 },
    { 0x547f8e10, 3 },
    { 0, 0 },
    { 0xfb3b8e13, 4 },
    { 0, 0 },
    { 0x438e15, 3 },
    { 0xb8ef8e15, 3 },
    { 0x179f8e17, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9b8e1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f8e1d, 3 },
    { 0, 0 },
    { 0xe7f8e1f, 3 },
    { 0xc08b8e20, 3 },
    { 0, 0 },
    { 0x49938e22, 3 },
    { 0xdf9f8e22, 4 },
    { 0x1aaf8e24, 3 },
    { 0x7eeb8e23, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d78e2c, 3 },
    { 0, 0 },
    { 0xd7cb8e2e, 3 },
    { 0, 0 },
    { 0xb6b78e30, 3 },
    { 0x498f8e31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37578e35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a538e3a, 3 },
    { 0xcdb8e3b, 3 },
    { 0, 0 },
    { 0x2338e3d, 3 },
    { 0xee78e3e, 4 },
    { 0xe2eb8e3f, 3 },
    { 0x31078e3f, 3 },
    { 0, 0 },
    { 0x5a278e42, 3 },
    { 0x52478e42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c78e4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7b8e4e, 3 },
    { 0x67af8e4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c78e52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acb8e57, 4 },
    { 0xfa0b8e58, 5 },
    { 0, 0 },
    { 0x10338e5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e78e60, 3 },
    { 0, 0 },
    { 0x1ca38e62, 3 },
    { 0xc3938e63, 4 },
    { 0xc3af8e63, 3 },
    { 0xb86b8e65, 3 },
    { 0xf44b8e66, 4 },
    { 0xbfbb8e63, 4 },
    { 0xb05b8e64, 3 },
    { 0x6e8f8e67, 3 },
    { 0x4fe78e6a, 4 },
    { 0x8be38e6a, 3 },
    { 0xefef8e6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75638e71, 3 },
    { 0x57fb8e71, 4 },
    { 0xbbff8e73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf78e78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b38e80, 4 },
    { 0xa21f8e80, 3 },
    { 0x9d0b8e81, 3 },
    { 0xff1f8e83, 4 },
    { 0xc5bf8e83, 3 },
    { 0xbb278e85, 3 },
    { 0x4a278e86, 3 },
    { 0, 0 },
    { 0xd1238e88, 3 },
    { 0x264b8e89, 3 },
    { 0x4a2f8e8a, 3 },
    { 0x10438e8b, 3 },
    { 0xaa2f8e8a, 4 },
    { 0x81238e89, 3 },
    { 0xdbe38e8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16b8e94, 3 },
    { 0xb9cb8e94, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddf8e9a, 3 },
    { 0xd3978e9a, 5 },
    { 0x7a4b8e9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3278ea5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e38ead, 3 },
    { 0xcd378eae, 3 },
    { 0x28ef8eaf, 4 },
    { 0x4eaf8eb0, 4 },
    { 0, 0 },
    { 0xc338eb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8f8eba, 3 },
    { 0, 0 },
    { 0x9fcb8ebc, 3 },
    { 0x9c5b8ebd, 5 },
    { 0xf03b8ebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476f8ec2, 3 },
    { 0, 0 },
    { 0x1b6b8ec4, 4 },
    { 0x3e178ec4, 3 },
    { 0xa1cf8ec6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f8edb, 3 },
    { 0x7d7b8edb, 3 },
    { 0x35038edd, 4 },
    { 0x30738ede, 3 },
    { 0x4278edf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4df8ee3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f578ee6, 3 },
    { 0x6acb8ee7, 3 },
    { 0x80c78ee7, 3 },
    { 0xb30f8ee7, 3 },
    { 0x67a78eea, 3 },
    { 0x75778ee7, 3 },
    { 0xda3b8eec, 3 },
    { 0xa83f8eed, 3 },
    { 0x57178ee9, 3 },
    { 0x35b78ee8, 3 },
    { 0xde678ef0, 3 },
    { 0x21f78ef1, 3 },
    { 0xb7778ef1, 4 },
    { 0x14d78ef3, 3 },
    { 0xa7af8ef3, 4 },
    { 0x81a78ef1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac678eff, 3 },
    { 0, 0 },
    { 0x1c238f01, 3 },
    { 0x1edf8f01, 3 },
    { 0x8a678f02, 3 },
    { 0xc21f8f02, 3 },
    { 0xdc2b8f05, 4 },
    { 0xfe0f8f01, 3 },
    { 0x474b8f07, 4 },
    { 0x87738f05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f38f0b, 5 },
    { 0, 0 },
    { 0x8eeb8f0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1f8f10, 3 },
    { 0x248f8f10, 3 },
    { 0x9738f12, 3 },
    { 0xdacb8f13, 3 },
    { 0x45038f14, 3 },
    { 0xc0338f14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62938f19, 3 },
    { 0xe4cb8f1a, 3 },
    { 0, 0 },
    { 0x23ab8f1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76178f1f, 3 },
    { 0x870b8f1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd138f29, 4 },
    { 0, 0 },
    { 0x71978f2b, 3 },
    { 0x5aaf8f2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f8f33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44f8f3e, 4 },
    { 0xfe0b8f3f, 3 },
    { 0x6f4b8f3f, 4 },
    { 0, 0 },
    { 0xc8678f42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccb8f45, 3 },
    { 0, 0 },
    { 0x743f8f47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8178f4c, 3 },
    { 0xfb178f4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f738f51, 4 },
    { 0x9a478f51, 3 },
    { 0x76078f53, 4 },
    { 0xa4d38f54, 3 },
    { 0x9d38f51, 3 },
    { 0xade78f54, 3 },
    { 0x944f8f56, 3 },
    { 0xab38f55, 3 },
    { 0x62478f58, 4 },
    { 0x98a38f5a, 3 },
    { 0x19278f5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6278f60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eab8f65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ff8f68, 3 },
    { 0xbacf8f69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f8f6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f278f70, 3 },
    { 0x83db8f70, 3 },
    { 0x24cf8f72, 3 },
    { 0, 0 },
    { 0x649f8f74, 4 },
    { 0x9caf8f75, 3 },
    { 0x42ef8f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc78f79, 4 },
    { 0x91078f7a, 4 },
    { 0x3ed38f7b, 3 },
    { 0x860b8f7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc38f80, 3 },
    { 0, 0 },
    { 0xfcf8f82, 4 },
    { 0x1bb78f83, 3 },
    { 0, 0 },
    { 0x82d38f85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c78f88, 4 },
    { 0xc3438f88, 4 },
    { 0x4c378f8a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56f8f8f, 3 },
    { 0x12e78f90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d378f93, 3 },
    { 0xe36f8f93, 3 },
    { 0x899b8f95, 3 },
    { 0xcf378f93, 3 },
    { 0xd78f8f97, 3 },
    { 0xce238f94, 4 },
    { 0x5f438f95, 3 },
    { 0, 0 },
    { 0xb6df8f9b, 3 },
    { 0xbad38f9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4af8fa0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc78fa6, 3 },
    { 0, 0 },
    { 0xadb38fa8, 3 },
    { 0x3f078fa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c78fac, 3 },
    { 0xfa178fad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab878fb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f8fb5, 3 },
    { 0xde938fb6, 3 },
    { 0x366b8fb7, 3 },
    { 0x38478fb6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aaf8fbc, 3 },
    { 0x15338fbd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19738fc6, 5 },
    { 0x72178fc7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780f8fcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcb8fd1, 3 },
    { 0, 0 },
    { 0xdc6b8fd3, 4 },
    { 0x5fbf8fd3, 3 },
    { 0xc5e38fd5, 4 },
    { 0x9edf8fd3, 3 },
    { 0, 0 },
    { 0x701b8fd8, 3 },
    { 0, 0 },
    { 0x6d738fda, 3 },
    { 0x1dfb8fda, 4 },
    { 0xbd638fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52378fe0, 3 },
    { 0x864b8fe1, 3 },
    { 0x9feb8fe2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b8fe6, 3 },
    { 0x99d38fe7, 4 },
    { 0x8feb8fe7, 3 },
    { 0, 0 },
    { 0x8edf8fea, 3 },
    { 0, 0 },
    { 0xb51b8fec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x33138fef, 3 },
    { 0, 0 },
    { 0x6dab8ff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a78ff5, 3 },
    { 0xbe5f8ff6, 3 },
    { 0x23e38ff7, 3 },
    { 0x4a538ff8, 3 },
    { 0x2b5f8ff9, 3 },
    { 0x9e1b8ffa, 4 },
    { 0x405f8ffa, 3 },
    { 0x55b38ff9, 3 },
    { 0x365f8ffb, 3 },
    { 0, 0 },
    { 0x234b8fff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b679002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec979006, 3 },
    { 0xb7f9007, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937f900c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x876f900f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb9012, 3 },
    { 0xe7b39012, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff9016, 3 },
    { 0x605b9017, 3 },
    { 0xd9c79017, 3 },
    { 0x5f9017, 4 },
    { 0x42eb9016, 3 },
    { 0xbf39016, 3 },
    { 0xc7bf901b, 4 },
    { 0xa5139019, 3 },
    { 0x2937901e, 4 },
    { 0x16d7901f, 3 },
    { 0xe93f9020, 3 },
    { 0x24ab9018, 3 },
    { 0, 0 },
    { 0xfc179023, 4 },
    { 0, 0 },
    { 0xf3279025, 3 },
    { 0xa7339025, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7f902c, 3 },
    { 0x4413902d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5139033, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96779037, 3 },
    { 0x9b3b9038, 3 },
    { 0, 0 },
    { 0xf537903a, 4 },
    { 0xcae7903b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7b9041, 3 },
    { 0, 0 },
    { 0x437f9043, 3 },
    { 0x3e7f9044, 4 },
    { 0x8b8f9045, 3 },
    { 0, 0 },
    { 0xe5e39047, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4b9052, 4 },
    { 0xb8839053, 4 },
    { 0, 0 },
    { 0x252f9055, 3 },
    { 0xb16f9055, 4 },
    { 0, 0 },
    { 0xac079058, 3 },
    { 0xe07f9059, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e39061, 3 },
    { 0xdc4b9061, 3 },
    { 0x5f5f9061, 3 },
    { 0xd2879064, 3 },
    { 0x26f9061, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235b9069, 4 },
    { 0, 0 },
    { 0x9657906b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fb906e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1db9071, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb9075, 3 },
    { 0xd1bb9075, 3 },
    { 0xd6bf9077, 3 },
    { 0x689f9078, 3 },
    { 0xf1eb9078, 4 },
    { 0x377f907a, 3 },
    { 0x98f79078, 3 },
    { 0x461b907b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c839081, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bf9085, 3 },
    { 0xdf339085, 3 },
    { 0x54239087, 3 },
    { 0x62ff9088, 4 },
    { 0x381b9086, 4 },
    { 0x7e7908a, 3 },
    { 0x4acb908a, 3 },
    { 0x66579085, 3 },
    { 0x5e7f908b, 4 },
    { 0x2dc3908e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b579091, 3 },
    { 0x30af9092, 3 },
    { 0xbf779093, 3 },
    { 0x4b5b9094, 5 },
    { 0x8ca39094, 4 },
    { 0, 0 },
    { 0xb9679097, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f909b, 3 },
    { 0x41f3909c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c790a0, 3 },
    { 0x299790a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74790a5, 5 },
    { 0xd74b90a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0db90ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d390b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b390b3, 3 },
    { 0x1be390b4, 3 },
    { 0x9ec790b5, 3 },
    { 0x1d4790b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff90be, 3 },
    { 0xc96790bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x739b90c2, 4 },
    { 0x7cab90c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00390c6, 4 },
    { 0x638790c6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24db90ce, 5 },
    { 0x47d790cf, 3 },
    { 0x5b1b90d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0790d4, 3 },
    { 0xfb8790d4, 3 },
    { 0, 0 },
    { 0x308f90d7, 3 },
    { 0x16f790d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x380b90db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6db90de, 4 },
    { 0xa8a790de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb90e2, 3 },
    { 0x85cb90e3, 3 },
    { 0x7ab90e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca390e7, 3 },
    { 0x648790e8, 3 },
    { 0x8c3790e9, 3 },
    { 0xd2ab90e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f390ef, 3 },
    { 0x421390ef, 3 },
    { 0x599b90f0, 4 },
    { 0, 0 },
    { 0xebf790f3, 3 },
    { 0xdc3790f4, 3 },
    { 0, 0 },
    { 0x48d390f6, 3 },
    { 0xfae790f7, 4 },
    { 0x711790f6, 3 },
    { 0x705f90f9, 3 },
    { 0, 0 },
    { 0x7c7f90fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bf90ff, 3 },
    { 0x4f3b9100, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251f9106, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c03910b, 3 },
    { 0xe4e7910c, 3 },
    { 0, 0 },
    { 0x990b910e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8739111, 5 },
    { 0x5f3b9112, 3 },
    { 0xcfb39113, 3 },
    { 0xfaf39114, 4 },
    { 0x1e6b9113, 3 },
    { 0xba6f9116, 3 },
    { 0x705b9116, 3 },
    { 0x23f9116, 3 },
    { 0xa51f9111, 3 },
    { 0xbd6f911a, 3 },
    { 0x9823911b, 3 },
    { 0xae17911c, 3 },
    { 0x215f911b, 3 },
    { 0xb933911a, 3 },
    { 0x42eb911b, 3 },
    { 0x3823911e, 3 },
    { 0x4a7b9121, 5 },
    { 0xa1ef9122, 3 },
    { 0x34ab911f, 4 },
    { 0x6f37911a, 3 },
    { 0xa7839124, 3 },
    { 0x1b739126, 4 },
    { 0x7fdf9127, 3 },
    { 0x2ccf9127, 4 },
    { 0xdce79129, 3 },
    { 0x1233912a, 3 },
    { 0xa4d3912a, 4 },
    { 0x8aa3912b, 5 },
    { 0x6eff9116, 3 },
    { 0xa88f9129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91f9131, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bb9135, 3 },
    { 0x78979136, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533913c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78db9144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401f914e, 3 },
    { 0x5dab914e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ff9153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c779156, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc179159, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb057915c, 3 },
    { 0xaa43915d, 4 },
    { 0, 0 },
    { 0xdb47915f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72339163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5f9169, 3 },
    { 0xab2f916a, 3 },
    { 0x4a0b916a, 3 },
    { 0x223f916c, 3 },
    { 0x203916c, 3 },
    { 0x6e83916b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b9173, 3 },
    { 0x81939174, 4 },
    { 0x987f9174, 3 },
    { 0xd1139176, 4 },
    { 0xaca39177, 3 },
    { 0x5acb9178, 3 },
    { 0x71df9179, 3 },
    { 0xb0eb917a, 3 },
    { 0xd99f917b, 3 },
    { 0x9477917a, 4 },
    { 0, 0 },
    { 0x3daf917e, 4 },
    { 0x9743917f, 3 },
    { 0, 0 },
    { 0x84bf9181, 3 },
    { 0x77639182, 3 },
    { 0xbf1b9183, 3 },
    { 0xca2f9184, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b37918a, 4 },
    { 0xc34f918b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaab918e, 3 },
    { 0x941b918f, 3 },
    { 0, 0 },
    { 0x971b9191, 3 },
    { 0x9e5b9192, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0b9195, 3 },
    { 0x84cf9196, 3 },
    { 0x429b9195, 3 },
    { 0xc9cb9198, 4 },
    { 0x85ff9199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a391a0, 3 },
    { 0x3f91a1, 3 },
    { 0xafe391a2, 3 },
    { 0xe63f91a2, 3 },
    { 0xacdf91a3, 3 },
    { 0, 0 },
    { 0x75b391a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0f91a9, 4 },
    { 0x99c791a9, 3 },
    { 0, 0 },
    { 0x6bf91ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f91b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2f91b5, 4 },
    { 0, 0 },
    { 0xce1791b7, 4 },
    { 0xbc9f91b7, 3 },
    { 0x31ef91b7, 3 },
    { 0x781391ba, 3 },
    { 0x18af91bb, 3 },
    { 0x33a791b8, 4 },
    { 0, 0 },
    { 0x589b91be, 4 },
    { 0x43d791be, 4 },
    { 0x728f91be, 3 },
    { 0, 0 },
    { 0x8391c2, 4 },
    { 0x203f91c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574791c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39f91ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402f91ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48791d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a791d7, 3 },
    { 0x8a4b91d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66f91dd, 3 },
    { 0, 0 },
    { 0x1e1391df, 3 },
    { 0, 0 },
    { 0x645b91e1, 3 },
    { 0x474791e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5b91eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b91f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54791f8, 3 },
    { 0x4e391f9, 3 },
    { 0, 0 },
    { 0xd00791fb, 3 },
    { 0xe67b91fb, 3 },
    { 0x792f91fd, 3 },
    { 0, 0 },
    { 0xdb5b91ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfeb9204, 4 },
    { 0, 0 },
    { 0x45579206, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e739209, 4 },
    { 0x46d3920a, 3 },
    { 0, 0 },
    { 0xcc2f920c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67439210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9579213, 4 },
    { 0x87bb9214, 4 },
    { 0x73139213, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c39218, 3 },
    { 0x3439219, 3 },
    { 0xd6539219, 3 },
    { 0x70b79219, 3 },
    { 0x7c0b921c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0979220, 3 },
    { 0xb88f9221, 4 },
    { 0x50039221, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb9226, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b3922d, 3 },
    { 0xe97922e, 3 },
    { 0, 0 },
    { 0x80739230, 3 },
    { 0x693b9231, 3 },
    { 0, 0 },
    { 0xe5f39233, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f9237, 3 },
    { 0xf0c79238, 3 },
    { 0x75fb9239, 3 },
    { 0, 0 },
    { 0x1fff923b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e39240, 3 },
    { 0xbf9241, 3 },
    { 0xa7b79241, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f9245, 4 },
    { 0x35a79246, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x995f9249, 3 },
    { 0, 0 },
    { 0xd167924b, 4 },
    { 0xbc7924c, 3 },
    { 0, 0 },
    { 0x6fdb924e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b9251, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf9256, 3 },
    { 0, 0 },
    { 0x4d4b9258, 3 },
    { 0xe1db9259, 3 },
    { 0xfccb9259, 3 },
    { 0, 0 },
    { 0x31c3925c, 3 },
    { 0x7e3925c, 4 },
    { 0x6a5b925c, 3 },
    { 0x8bb7925f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74b9265, 3 },
    { 0x69a79265, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b839269, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba79270, 3 },
    { 0, 0 },
    { 0xf1eb9272, 3 },
    { 0xd30f9273, 3 },
    { 0xea3f9274, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f9277, 3 },
    { 0, 0 },
    { 0xb39b9279, 3 },
    { 0xb5e7927a, 4 },
    { 0xc4bb927b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d079286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f679289, 3 },
    { 0x513928a, 3 },
    { 0, 0 },
    { 0xbfef928c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab9292, 3 },
    { 0xbd1f9293, 3 },
    { 0, 0 },
    { 0x9139295, 5 },
    { 0x693f9296, 4 },
    { 0x3379297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb929b, 4 },
    { 0x1f53929b, 4 },
    { 0x3193929b, 3 },
    { 0xaa23929e, 3 },
    { 0x280b929e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f92a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d392a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b92ab, 4 },
    { 0x3bbf92ac, 4 },
    { 0xdc1392ad, 4 },
    { 0xd68b92ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de792b5, 3 },
    { 0xae0b92b6, 4 },
    { 0x8a7792b5, 4 },
    { 0xde3f92b6, 5 },
    { 0xddab92b9, 3 },
    { 0xce9f92ba, 3 },
    { 0xf5bb92ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92f92be, 3 },
    { 0x9a8b92bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbf92c3, 3 },
    { 0x531792c4, 3 },
    { 0, 0 },
    { 0x649f92c6, 3 },
    { 0x445392c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892b92cc, 3 },
    { 0x4b6792cc, 3 },
    { 0xc9bf92ce, 5 },
    { 0xd9f92cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x792792d2, 3 },
    { 0x1a1392d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9392d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89b92e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7792e9, 3 },
    { 0, 0 },
    { 0x86a792eb, 3 },
    { 0x203392ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fb92f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4792f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129f92fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615f9302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38679306, 4 },
    { 0x447b9307, 3 },
    { 0x602f9307, 3 },
    { 0x9a4b9309, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca67930c, 3 },
    { 0x3e53930d, 3 },
    { 0x8017930d, 3 },
    { 0xd0bb930f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf9314, 3 },
    { 0x515f9315, 5 },
    { 0xbe8b9316, 3 },
    { 0xa26b9316, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35139321, 4 },
    { 0xd5279321, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7339327, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb23932c, 3 },
    { 0xa137932d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64f9330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9539333, 3 },
    { 0x100b9333, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddb933b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b39346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43934b, 3 },
    { 0x6977934b, 4 },
    { 0xe6b3934c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f79352, 3 },
    { 0xf3f39352, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b9359, 4 },
    { 0x103935a, 5 },
    { 0, 0 },
    { 0x5d07935c, 4 },
    { 0, 0 },
    { 0x80df935e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff9361, 3 },
    { 0xc4db9361, 3 },
    { 0xaac39363, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81679366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9faf9369, 3 },
    { 0xa4af936a, 3 },
    { 0x4c9b936b, 3 },
    { 0xcd3936c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7f9374, 3 },
    { 0, 0 },
    { 0x542f9376, 3 },
    { 0x5e739377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa23937c, 3 },
    { 0x629f937d, 4 },
    { 0, 0 },
    { 0x4bfb937f, 3 },
    { 0x49ab9380, 3 },
    { 0x485b9381, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dab9384, 3 },
    { 0x20839385, 4 },
    { 0x9139384, 3 },
    { 0xb21f9387, 3 },
    { 0xea979386, 3 },
    { 0xf90b9385, 3 },
    { 0xaa33938a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab39390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd79395, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f87939f, 4 },
    { 0, 0 },
    { 0xe19b93a1, 4 },
    { 0xb30b93a2, 4 },
    { 0x7e9f93a2, 4 },
    { 0x976b93a2, 3 },
    { 0x737b93a1, 3 },
    { 0x1fbf93a4, 4 },
    { 0xe65b93a7, 3 },
    { 0x1c3393a7, 5 },
    { 0, 0 },
    { 0x3eab93aa, 3 },
    { 0, 0 },
    { 0x23af93ac, 3 },
    { 0xb5df93ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f93b0, 3 },
    { 0x42e393b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x567f93b4, 3 },
    { 0xf65393b5, 3 },
    { 0xbc4793b4, 5 },
    { 0x913b93b7, 4 },
    { 0xf09b93b7, 4 },
    { 0x5f9b93b9, 3 },
    { 0xfab793ba, 3 },
    { 0x676f93b7, 3 },
    { 0xb08f93bc, 3 },
    { 0x36fb93bd, 4 },
    { 0x3eab93b4, 3 },
    { 0x830f93bf, 3 },
    { 0xaddb93c0, 3 },
    { 0x48a393c0, 4 },
    { 0xdf2393c0, 3 },
    { 0xf96393c3, 3 },
    { 0, 0 },
    { 0x88f793c5, 3 },
    { 0xa20793c6, 4 },
    { 0x972793c6, 4 },
    { 0x2cf793c7, 3 },
    { 0x96df93c9, 4 },
    { 0xe1e793ca, 3 },
    { 0x189b93ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9f93d3, 3 },
    { 0, 0 },
    { 0x6cf93d5, 4 },
    { 0x889393d6, 3 },
    { 0x521f93d7, 4 },
    { 0, 0 },
    { 0x9eb793d9, 4 },
    { 0x5f7393da, 4 },
    { 0x29bf93db, 3 },
    { 0x853393dc, 3 },
    { 0xa793dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0393e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f93e5, 3 },
    { 0xfef393e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fff93ec, 4 },
    { 0x8f7f93ec, 3 },
    { 0, 0 },
    { 0xe56393ef, 3 },
    { 0xb04793ef, 3 },
    { 0, 0 },
    { 0xedab93f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7b93f5, 3 },
    { 0x5aef93f6, 3 },
    { 0xfc7393f7, 5 },
    { 0x4edf93f7, 4 },
    { 0xdf9f93f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecb9404, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf079409, 3 },
    { 0x4327940a, 3 },
    { 0xcf0b940b, 3 },
    { 0, 0 },
    { 0xb043940d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65b9412, 3 },
    { 0, 0 },
    { 0x50b79414, 3 },
    { 0xea279415, 3 },
    { 0, 0 },
    { 0xde5b9417, 4 },
    { 0xd8c79418, 3 },
    { 0, 0 },
    { 0x63c7941a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b9424, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ab942c, 3 },
    { 0x666b942d, 3 },
    { 0x47b3942e, 3 },
    { 0x8d9b942f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e639436, 3 },
    { 0x72879436, 3 },
    { 0xe0779438, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9617943c, 4 },
    { 0x52db943d, 3 },
    { 0x9957943e, 3 },
    { 0x15df943d, 4 },
    { 0xaf0f943d, 3 },
    { 0x80e3943e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cf9444, 5 },
    { 0, 0 },
    { 0x9d1f9446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4563944a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f9453, 4 },
    { 0x5ebb9453, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4b9459, 4 },
    { 0x808f9459, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f3945f, 5 },
    { 0, 0 },
    { 0x34d79461, 5 },
    { 0xe8cb9461, 3 },
    { 0xcb8b9462, 3 },
    { 0, 0 },
    { 0xb9c39465, 3 },
    { 0x62b9466, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee3946a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68f946d, 4 },
    { 0, 0 },
    { 0x82b7946f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bb9472, 3 },
    { 0, 0 },
    { 0xa7eb9474, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bb9478, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bb947d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25179482, 4 },
    { 0xd75b9483, 3 },
    { 0, 0 },
    { 0xeff79485, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5247948d, 3 },
    { 0, 0 },
    { 0xe803948f, 3 },
    { 0xa3b3948f, 3 },
    { 0x4d5f9491, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b9496, 4 },
    { 0, 0 },
    { 0x309b9498, 4 },
    { 0, 0 },
    { 0x2ce7949a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b794a4, 3 },
    { 0x954394a4, 4 },
    { 0x904794a6, 3 },
    { 0x599b94a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x376b94aa, 3 },
    { 0, 0 },
    { 0x674794ac, 3 },
    { 0xa5794ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654394b2, 3 },
    { 0, 0 },
    { 0xb3ff94b4, 3 },
    { 0, 0 },
    { 0x972394b6, 3 },
    { 0x640794b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506394bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8b94c0, 3 },
    { 0x39394c1, 3 },
    { 0xb9ff94c2, 3 },
    { 0x40db94c2, 3 },
    { 0xf49b94c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x712794c7, 3 },
    { 0x47a794c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff94cc, 4 },
    { 0x4c8794cd, 4 },
    { 0x89b794cd, 3 },
    { 0xbac794ce, 3 },
    { 0xfe9394d0, 4 },
    { 0xb42b94d0, 3 },
    { 0x5ea394d1, 3 },
    { 0xdea794ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247b94dc, 3 },
    { 0x513f94dc, 4 },
    { 0xeb8394de, 3 },
    { 0x308394df, 3 },
    { 0xb0a394dd, 3 },
    { 0x43b94e0, 3 },
    { 0x126794e2, 3 },
    { 0, 0 },
    { 0xd4db94e4, 3 },
    { 0x2e3b94e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86794e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea394ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5f94f1, 3 },
    { 0x1d0f94f1, 3 },
    { 0xc79394f3, 3 },
    { 0x233794f2, 3 },
    { 0, 0 },
    { 0x35bf94f6, 3 },
    { 0, 0 },
    { 0x340b94f8, 3 },
    { 0, 0 },
    { 0x7af394fa, 4 },
    { 0xb82794fa, 3 },
    { 0x231794fc, 4 },
    { 0x50f94fc, 3 },
    { 0, 0 },
    { 0x1f2794ff, 3 },
    { 0, 0 },
    { 0xc8af9501, 3 },
    { 0x77479502, 5 },
    { 0x230b9503, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb7950c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db39510, 3 },
    { 0x184f9511, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02f951b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d87951f, 3 },
    { 0, 0 },
    { 0x5bff9521, 3 },
    { 0, 0 },
    { 0xf2cb9523, 3 },
    { 0x73bf9523, 3 },
    { 0x2d7f9525, 3 },
    { 0, 0 },
    { 0x161f9527, 3 },
    { 0x8cb9527, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597952b, 4 },
    { 0xe8b3952c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e39531, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x54139534, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b9537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab79541, 3 },
    { 0x47b9541, 3 },
    { 0x74ef9543, 3 },
    { 0xe0439543, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f954a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df954d, 4 },
    { 0x9647954d, 3 },
    { 0x29db954f, 3 },
    { 0x686f9550, 3 },
    { 0x97ab9551, 3 },
    { 0xaf6b9550, 3 },
    { 0x2ef79552, 4 },
    { 0xe939554, 3 },
    { 0xf5479555, 3 },
    { 0x218f9556, 3 },
    { 0x9c939557, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a83955a, 3 },
    { 0x9183955a, 4 },
    { 0, 0 },
    { 0x1973955d, 3 },
    { 0, 0 },
    { 0x3933955f, 3 },
    { 0, 0 },
    { 0x739b9561, 3 },
    { 0x7d1b9562, 3 },
    { 0xe839562, 3 },
    { 0x1a39564, 3 },
    { 0xb5df9565, 4 },
    { 0xd99f9565, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3b956c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe3957b, 3 },
    { 0xd8ab957b, 4 },
    { 0x4c8f957b, 3 },
    { 0xd663957e, 3 },
    { 0, 0 },
    { 0xd5279580, 4 },
    { 0, 0 },
    { 0xc3a39582, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615b9586, 3 },
    { 0, 0 },
    { 0x63879588, 3 },
    { 0, 0 },
    { 0x594f958a, 3 },
    { 0, 0 },
    { 0x179b958c, 3 },
    { 0, 0 },
    { 0xb123958e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcf9592, 3 },
    { 0x64ef9592, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f959d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb95a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799395a8, 3 },
    { 0, 0 },
    { 0x6d5f95aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f795b0, 3 },
    { 0xc0af95b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74f95bb, 3 },
    { 0xf0c795bc, 3 },
    { 0x494f95bd, 4 },
    { 0, 0 },
    { 0x223f95bf, 5 },
    { 0, 0 },
    { 0x26d395c1, 4 },
    { 0, 0 },
    { 0x275395c3, 3 },
    { 0x676f95c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76795ca, 3 },
    { 0, 0 },
    { 0x996795cc, 4 },
    { 0, 0 },
    { 0x280b95ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2b95d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626b95d6, 3 },
    { 0xe2eb95d7, 3 },
    { 0x287795d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12795e1, 3 },
    { 0, 0 },
    { 0x81bb95e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x677795e6, 3 },
    { 0x566795e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557795eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848b95ef, 4 },
    { 0x2fc395f0, 3 },
    { 0xa96795ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1b95f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b95fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97979600, 3 },
    { 0, 0 },
    { 0x81939602, 3 },
    { 0x22839603, 3 },
    { 0x5cf39603, 3 },
    { 0, 0 },
    { 0xf5bb9606, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6df9609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1507960d, 4 },
    { 0xc68b960e, 3 },
    { 0x4fdf960f, 3 },
    { 0x79f7960e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf79614, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecf9617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a3961d, 4 },
    { 0xd6f3961d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78f9624, 3 },
    { 0, 0 },
    { 0xc49f9626, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833962a, 3 },
    { 0x27ef962b, 3 },
    { 0xe487962b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c739631, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x814b9634, 3 },
    { 0x98b9635, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690b9639, 3 },
    { 0x3be7963a, 3 },
    { 0x13e7963b, 3 },
    { 0xbac7963c, 4 },
    { 0xd6cb963d, 3 },
    { 0x5a4b963d, 4 },
    { 0x5cd3963a, 3 },
    { 0xe76f9640, 3 },
    { 0xc7a39640, 3 },
    { 0x303b963e, 3 },
    { 0xedc7963c, 5 },
    { 0, 0 },
    { 0x80679645, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b7964e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a779654, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcb965b, 4 },
    { 0xd367965c, 3 },
    { 0xff7f965d, 3 },
    { 0, 0 },
    { 0x79f7965f, 3 },
    { 0xa9979660, 3 },
    { 0x4dd39660, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14939665, 3 },
    { 0x829b9665, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fb966d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e03967a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7947967d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab9680, 3 },
    { 0x1c1f9681, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45039688, 3 },
    { 0x8ddf9689, 3 },
    { 0, 0 },
    { 0xb393968b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3939693, 3 },
    { 0, 0 },
    { 0x6ddf9695, 4 },
    { 0xf20b9696, 3 },
    { 0, 0 },
    { 0xa6b39698, 3 },
    { 0x233f9698, 3 },
    { 0xa52f969a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f96a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f796a8, 4 },
    { 0, 0 },
    { 0xa4ff96aa, 3 },
    { 0xbbb396ab, 3 },
    { 0, 0 },
    { 0xdeab96ad, 4 },
    { 0x46cf96ae, 4 },
    { 0x32f96af, 3 },
    { 0x260396b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20796b6, 4 },
    { 0, 0 },
    { 0xb60f96b8, 3 },
    { 0x61d796b8, 3 },
    { 0, 0 },
    { 0x4f9396bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faf96be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e396c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710796c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf96ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7b96d3, 3 },
    { 0, 0 },
    { 0xbf7b96d5, 3 },
    { 0, 0 },
    { 0x4f1b96d7, 3 },
    { 0, 0 },
    { 0xeed396d9, 3 },
    { 0x1dd796d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3f96e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a396e8, 3 },
    { 0x894f96e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eb96ed, 3 },
    { 0x66f96ed, 3 },
    { 0x40e796ef, 3 },
    { 0xfd2f96f0, 4 },
    { 0x510796f1, 4 },
    { 0x64bb96f2, 3 },
    { 0x2e9f96f0, 3 },
    { 0x418b96f0, 3 },
    { 0x69b396f5, 3 },
    { 0x3e4396f6, 3 },
    { 0xef8396f6, 3 },
    { 0xccfb96f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b96fd, 3 },
    { 0x89f96fe, 3 },
    { 0x6eb796ff, 4 },
    { 0x893b96ff, 3 },
    { 0, 0 },
    { 0x17b39702, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a839705, 3 },
    { 0x89c39706, 4 },
    { 0x4f239707, 3 },
    { 0xe3379708, 4 },
    { 0x33979709, 4 },
    { 0xcc839706, 5 },
    { 0, 0 },
    { 0x4837970c, 3 },
    { 0, 0 },
    { 0x956f970e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47539715, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f971e, 4 },
    { 0x426b971f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fb9725, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87f9728, 3 },
    { 0x75cb9729, 3 },
    { 0xc3f7972a, 4 },
    { 0xc1c7972b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17972e, 3 },
    { 0, 0 },
    { 0x1fa39730, 4 },
    { 0, 0 },
    { 0xd9a39732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26f9735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b9739, 3 },
    { 0, 0 },
    { 0x747b973b, 3 },
    { 0, 0 },
    { 0xbd57973d, 3 },
    { 0, 0 },
    { 0x307b973f, 3 },
    { 0, 0 },
    { 0xf76f9741, 4 },
    { 0xa2539742, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448b9749, 4 },
    { 0x3fe39749, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f9750, 3 },
    { 0x91b79751, 5 },
    { 0xb4a39751, 4 },
    { 0xa5d39750, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f975e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c39762, 3 },
    { 0x2cb9762, 4 },
    { 0xe3739764, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4213976a, 4 },
    { 0x54f3976b, 3 },
    { 0xadf976c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cf976f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43f9774, 4 },
    { 0x9b1b9775, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2b9780, 4 },
    { 0, 0 },
    { 0xfdb79782, 3 },
    { 0xf2b9783, 3 },
    { 0xf20f9784, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb9787, 3 },
    { 0, 0 },
    { 0x7e739789, 3 },
    { 0xd2af978a, 3 },
    { 0x6f03978b, 4 },
    { 0x67d7978c, 3 },
    { 0x7937978c, 3 },
    { 0, 0 },
    { 0xebbb978f, 4 },
    { 0x8e139790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x11339793, 3 },
    { 0x9f6b9794, 3 },
    { 0x9ab79794, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6f9798, 3 },
    { 0xaeaf9799, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef979d, 3 },
    { 0, 0 },
    { 0x61af979f, 3 },
    { 0x68d397a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8397a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c797a8, 4 },
    { 0xf1397a9, 4 },
    { 0xd2ff97a9, 4 },
    { 0xc2c797ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36397b5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbf97b8, 3 },
    { 0x71c797b9, 3 },
    { 0, 0 },
    { 0x887f97bb, 3 },
    { 0x643797bb, 3 },
    { 0, 0 },
    { 0xe31b97be, 3 },
    { 0, 0 },
    { 0xb03b97c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x965797c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282b97ca, 3 },
    { 0, 0 },
    { 0x289f97cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xade797cf, 3 },
    { 0x749397cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ab97d5, 3 },
    { 0x26d397d6, 4 },
    { 0xe41f97d7, 3 },
    { 0x8e8f97d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ff97e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad397e4, 3 },
    { 0, 0 },
    { 0x8cf797e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53797ea, 4 },
    { 0x545397ea, 3 },
    { 0x49bb97ec, 3 },
    { 0x66a797ed, 3 },
    { 0x656397eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fb97f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b97fc, 3 },
    { 0xb46797fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f79806, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ab980b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ef980e, 3 },
    { 0x898b980e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a139813, 3 },
    { 0x612f9814, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd39817, 4 },
    { 0xccd79818, 3 },
    { 0x24379819, 3 },
    { 0x83cf981a, 5 },
    { 0x683981a, 4 },
    { 0xfe97981a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf9820, 3 },
    { 0x51d39820, 4 },
    { 0, 0 },
    { 0xc8379823, 3 },
    { 0x30fb9824, 3 },
    { 0x185b9825, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a7982c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bb9830, 4 },
    { 0x450b9831, 3 },
    { 0, 0 },
    { 0x251b9833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b79838, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc947983b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e3983f, 3 },
    { 0, 0 },
    { 0x1f5b9841, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329f984c, 3 },
    { 0x6db984c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3679855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3985a, 3 },
    { 0, 0 },
    { 0x4a8f985c, 3 },
    { 0xbf2f985d, 4 },
    { 0, 0 },
    { 0xbb87985f, 3 },
    { 0xc06b9860, 3 },
    { 0xaf679861, 4 },
    { 0x3fa39861, 3 },
    { 0x9a13985f, 3 },
    { 0x538f985f, 3 },
    { 0x1479860, 3 },
    { 0, 0 },
    { 0xf01b9867, 3 },
    { 0xd3e79867, 3 },
    { 0x5a0b9867, 4 },
    { 0xc5cf9867, 3 },
    { 0, 0 },
    { 0x2edf986c, 3 },
    { 0x663f986d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473b9873, 3 },
    { 0xe61f9873, 3 },
    { 0x8a2b9875, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b987a, 3 },
    { 0x15cb987b, 3 },
    { 0xcfff987a, 3 },
    { 0, 0 },
    { 0xaedb987e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab439881, 4 },
    { 0, 0 },
    { 0x4c8b9883, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c639887, 3 },
    { 0xbf939887, 3 },
    { 0, 0 },
    { 0xd7b3988a, 3 },
    { 0xf8f3988b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x431b988e, 3 },
    { 0, 0 },
    { 0xe8239890, 3 },
    { 0, 0 },
    { 0x96cf9892, 3 },
    { 0xd0e39893, 3 },
    { 0x32af9894, 3 },
    { 0xf5479895, 3 },
    { 0x2cff9892, 3 },
    { 0x38fb9896, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f989a, 3 },
    { 0xd487989b, 3 },
    { 0xff2f989b, 3 },
    { 0x58f7989d, 4 },
    { 0, 0 },
    { 0x75a7989f, 4 },
    { 0x770f989f, 3 },
    { 0, 0 },
    { 0xcc9798a2, 3 },
    { 0x809798a3, 3 },
    { 0xb7398a4, 3 },
    { 0x7d798a5, 4 },
    { 0x391398a6, 3 },
    { 0x97798a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0b98ac, 4 },
    { 0xad8b98ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9398b6, 3 },
    { 0x5e9398b7, 3 },
    { 0, 0 },
    { 0x4f98b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542398be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398c5, 3 },
    { 0, 0 },
    { 0x8d2398c7, 3 },
    { 0xf4c398c8, 4 },
    { 0xc48f98c8, 3 },
    { 0, 0 },
    { 0x793798cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f98cf, 3 },
    { 0, 0 },
    { 0x1ddb98d1, 3 },
    { 0xebfb98d2, 4 },
    { 0xc02798d3, 4 },
    { 0, 0 },
    { 0x82ab98d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba798db, 4 },
    { 0, 0 },
    { 0xf22b98dd, 3 },
    { 0x296798de, 3 },
    { 0x13a798dd, 3 },
    { 0, 0 },
    { 0xd6c798e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c398e6, 4 },
    { 0x200b98e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0df98ed, 3 },
    { 0xe1e398ee, 4 },
    { 0, 0 },
    { 0xee1f98f0, 3 },
    { 0x9b9398f1, 3 },
    { 0, 0 },
    { 0x84af98f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95798fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x178b98fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcb9902, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62239905, 3 },
    { 0x7b739905, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc39909, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3990d, 3 },
    { 0x99ef990e, 4 },
    { 0x9ec7990e, 3 },
    { 0xe20b9910, 4 },
    { 0x3f8b9910, 3 },
    { 0xe5fb990f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95f991b, 3 },
    { 0, 0 },
    { 0x57eb991d, 3 },
    { 0, 0 },
    { 0x6c8f991f, 4 },
    { 0, 0 },
    { 0x78f79921, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f9924, 3 },
    { 0x249b9925, 3 },
    { 0x189b9925, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x25539929, 3 },
    { 0, 0 },
    { 0xe6f7992b, 3 },
    { 0, 0 },
    { 0x9be7992d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9307993d, 3 },
    { 0, 0 },
    { 0x4683993f, 3 },
    { 0xaec7993f, 4 },
    { 0, 0 },
    { 0x88339942, 3 },
    { 0x237f9942, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb9949, 3 },
    { 0, 0 },
    { 0xc037994b, 3 },
    { 0, 0 },
    { 0xfbd7994d, 3 },
    { 0, 0 },
    { 0x12bb994f, 3 },
    { 0x26f7994f, 3 },
    { 0x47079950, 4 },
    { 0xfe2f9952, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x916b9955, 3 },
    { 0x61439956, 4 },
    { 0x13379957, 3 },
    { 0x3c2b9958, 3 },
    { 0x9cc79959, 3 },
    { 0x30eb9958, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c3995d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb79960, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf9963, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb9969, 3 },
    { 0xddf996a, 3 },
    { 0xbfc39969, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb79970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54179974, 3 },
    { 0x57b79974, 5 },
    { 0x1d4f9976, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab539979, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4af997c, 3 },
    { 0x554b997c, 4 },
    { 0xfd73997c, 3 },
    { 0xa0f997f, 3 },
    { 0, 0 },
    { 0xb3cb9981, 3 },
    { 0x98c39982, 3 },
    { 0x53379983, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x939b9986, 3 },
    { 0xcc079986, 4 },
    { 0x218f9988, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9b998b, 3 },
    { 0, 0 },
    { 0xd42b998d, 3 },
    { 0xee27998d, 3 },
    { 0x3397998e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a39992, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e79995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x47679998, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e3999b, 3 },
    { 0xf6eb999c, 5 },
    { 0xfa9b999c, 3 },
    { 0x859f999e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b399a3, 3 },
    { 0x6ae799a3, 4 },
    { 0, 0 },
    { 0x56b99a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e399a9, 4 },
    { 0, 0 },
    { 0xd83399ab, 3 },
    { 0x7ab799ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556399b8, 3 },
    { 0x1ca399b8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb399c2, 3 },
    { 0, 0 },
    { 0x24db99c4, 3 },
    { 0x17df99c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cb99ca, 3 },
    { 0xcd1799cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5f99d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f99d5, 3 },
    { 0x2ce799d5, 3 },
    { 0, 0 },
    { 0x6f8b99d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65df99df, 3 },
    { 0x5e9b99e0, 3 },
    { 0x84b99df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9799e6, 3 },
    { 0x67cb99e7, 3 },
    { 0x754799e7, 4 },
    { 0x5eaf99e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a799ed, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f99f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e799f6, 4 },
    { 0, 0 },
    { 0x371399f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b99fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c79a03, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f9a08, 3 },
    { 0, 0 },
    { 0x87279a0a, 3 },
    { 0x51b39a0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d79a12, 4 },
    { 0x833f9a13, 4 },
    { 0, 0 },
    { 0x52d79a15, 3 },
    { 0x7c79a15, 4 },
    { 0x6cef9a15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b679a1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e79a24, 3 },
    { 0x6239a25, 4 },
    { 0xaeab9a24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a79a29, 4 },
    { 0xe2839a2a, 3 },
    { 0x18af9a2a, 3 },
    { 0, 0 },
    { 0x828f9a2d, 3 },
    { 0, 0 },
    { 0x75579a2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28b9a37, 4 },
    { 0, 0 },
    { 0x73839a39, 3 },
    { 0, 0 },
    { 0x2f579a3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00b9a44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f9a48, 3 },
    { 0, 0 },
    { 0xda839a4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda639a4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b79a51, 3 },
    { 0, 0 },
    { 0x851b9a53, 4 },
    { 0xd8eb9a54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26df9a59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67f9a5d, 3 },
    { 0, 0 },
    { 0x57db9a5f, 3 },
    { 0, 0 },
    { 0xc05b9a61, 5 },
    { 0x9f739a62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99af9a66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b9a69, 3 },
    { 0x74bb9a6a, 4 },
    { 0x1479a6b, 4 },
    { 0xc5ab9a6a, 3 },
    { 0xf5c39a6d, 3 },
    { 0x86a79a6d, 3 },
    { 0x175b9a69, 3 },
    { 0xb6439a6a, 3 },
    { 0x17a79a71, 4 },
    { 0x217f9a71, 3 },
    { 0xb64f9a73, 4 },
    { 0xe57f9a69, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x14879a77, 4 },
    { 0xae279a78, 4 },
    { 0x9d7b9a79, 3 },
    { 0x3f579a78, 3 },
    { 0xf8939a77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1f9a80, 3 },
    { 0x8cf79a81, 3 },
    { 0xc89b9a81, 3 },
    { 0x7f5b9a82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf9a87, 3 },
    { 0, 0 },
    { 0xabcb9a89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545f9a90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd39a94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2b9a9a, 3 },
    { 0xcc979a9b, 4 },
    { 0xd78f9a9c, 3 },
    { 0x1c279a9b, 4 },
    { 0x3b9a9a, 3 },
    { 0xf9639a9f, 3 },
    { 0xe48b9a9d, 3 },
    { 0, 0 },
    { 0x145f9aa2, 4 },
    { 0xb3839aa3, 3 },
    { 0xc5fb9aa4, 3 },
    { 0x448f9aa4, 4 },
    { 0xd4679aa6, 3 },
    { 0x56d79aa6, 5 },
    { 0x35a79aa8, 3 },
    { 0x6e939aa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b9aaf, 3 },
    { 0xa6579aaf, 3 },
    { 0xe9d39ab1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c79ab4, 3 },
    { 0x460b9ab4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76439aba, 4 },
    { 0, 0 },
    { 0xee6f9abc, 4 },
    { 0x6bdf9abd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f9ac3, 4 },
    { 0, 0 },
    { 0x69e39ac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65f9ac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab9acc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x168f9acf, 3 },
    { 0x1bcf9acf, 3 },
    { 0, 0 },
    { 0xd2179ad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d739ad5, 3 },
    { 0x29ab9ad6, 3 },
    { 0xbf279ad6, 3 },
    { 0, 0 },
    { 0xad6b9ad9, 3 },
    { 0x7a5f9ada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x56079add, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26879ae2, 5 },
    { 0x6bf9ae3, 3 },
    { 0xfe879ae4, 3 },
    { 0xdb5f9ae5, 3 },
    { 0x7279ae6, 4 },
    { 0x933b9ae7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51db9aef, 3 },
    { 0, 0 },
    { 0xae7b9af1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c639af4, 3 },
    { 0xbbcf9af4, 3 },
    { 0xf5a79af6, 3 },
    { 0, 0 },
    { 0x8a979af8, 3 },
    { 0xe39b9af9, 3 },
    { 0xedb39afa, 3 },
    { 0xc73f9afb, 4 },
    { 0xc7b9afc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xccaf9aff, 3 },
    { 0x61839b00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b9b07, 3 },
    { 0x183f9b08, 3 },
    { 0x8f079b09, 3 },
    { 0x5b6f9b0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe79b11, 3 },
    { 0x1d739b12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x79379b15, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac79b1f, 3 },
    { 0xc0239b20, 3 },
    { 0xff6b9b20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc79b25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe939b2b, 3 },
    { 0x6aff9b2b, 5 },
    { 0x54239b2b, 4 },
    { 0xe0539b2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f79b37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6479b3c, 5 },
    { 0xe0279b3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59979b43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b9b47, 3 },
    { 0x75d39b48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc79b4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b439b54, 3 },
    { 0x25439b55, 4 },
    { 0, 0 },
    { 0x13b39b57, 4 },
    { 0, 0 },
    { 0x757b9b59, 5 },
    { 0xebe39b59, 3 },
    { 0xc0c79b5a, 3 },
    { 0x855f9b5b, 3 },
    { 0xb62f9b5a, 3 },
    { 0x150b9b5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c39b64, 3 },
    { 0xb1cb9b64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e139b68, 3 },
    { 0, 0 },
    { 0x97e39b6a, 3 },
    { 0x92bf9b6b, 3 },
    { 0xac3b9b6a, 3 },
    { 0x9cbb9b6a, 3 },
    { 0, 0 },
    { 0xf07f9b6f, 3 },
    { 0x129f9b6f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3739b75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639b78, 3 },
    { 0xdbc79b79, 3 },
    { 0xc63f9b7a, 3 },
    { 0, 0 },
    { 0x53879b7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78b9b7f, 5 },
    { 0x60b39b7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef9b83, 3 },
    { 0, 0 },
    { 0x369b9b85, 3 },
    { 0, 0 },
    { 0x75539b87, 3 },
    { 0x6c179b88, 3 },
    { 0x5f379b88, 3 },
    { 0x39f79b88, 3 },
    { 0x58379b89, 3 },
    { 0x1cb39b8c, 3 },
    { 0x273f9b89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474f9b92, 4 },
    { 0xfdbf9b92, 4 },
    { 0, 0 },
    { 0x4a039b95, 3 },
    { 0x4bbf9b95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4139b99, 3 },
    { 0x865f9b9a, 3 },
    { 0, 0 },
    { 0xe9979b9c, 3 },
    { 0x5c079b9d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edf9ba1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c79ba6, 4 },
    { 0x74b9ba6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6479bb0, 3 },
    { 0x290f9bb1, 4 },
    { 0xdcef9bb2, 3 },
    { 0xe4f39bb3, 3 },
    { 0x7b0b9bb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d39bb8, 3 },
    { 0x21db9bb9, 3 },
    { 0xe8a39bba, 3 },
    { 0x26a79bba, 3 },
    { 0xe5c79bbb, 3 },
    { 0x1f579bba, 3 },
    { 0x5439bba, 4 },
    { 0xb9ff9bbf, 3 },
    { 0x6bbb9bc0, 3 },
    { 0x703f9bc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d339bc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab9bca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x812f9bcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f79bd0, 4 },
    { 0xcc7f9bd1, 3 },
    { 0xaf279bd1, 3 },
    { 0x7cd79bd3, 3 },
    { 0x97b79bd4, 3 },
    { 0x48039bd0, 3 },
    { 0, 0 },
    { 0x952b9bd7, 3 },
    { 0, 0 },
    { 0xc4839bd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a39bdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54879be2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3f9bea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542b9bf3, 5 },
    { 0x1d5f9bf4, 3 },
    { 0x77179bf5, 3 },
    { 0x55f79bf5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2f9bfa, 4 },
    { 0x3eff9bfb, 3 },
    { 0x7b9bfb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5af9c00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f9c03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c879c08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290f9c0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97679c11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bb9c15, 3 },
    { 0x50479c16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb9c1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11b9c21, 3 },
    { 0x9e5f9c22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f9c26, 3 },
    { 0x1f0b9c26, 3 },
    { 0xe7979c28, 3 },
    { 0xdfab9c29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b79c2d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d79c36, 4 },
    { 0, 0 },
    { 0xa5b39c38, 4 },
    { 0, 0 },
    { 0x33139c3a, 3 },
    { 0x9bdf9c3a, 3 },
    { 0x9f179c3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339c41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda79c45, 3 },
    { 0xb5739c46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2f9c49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d039c4f, 4 },
    { 0x52639c4f, 3 },
    { 0xcd7f9c51, 3 },
    { 0x37879c52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b9c57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc179c5c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0f9c60, 4 },
    { 0xd38f9c60, 3 },
    { 0xa57b9c62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f9c65, 4 },
    { 0x466b9c66, 3 },
    { 0, 0 },
    { 0x9a439c68, 3 },
    { 0xae8b9c69, 4 },
    { 0xf58b9c69, 3 },
    { 0, 0 },
    { 0xab9c6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b79c70, 4 },
    { 0x24df9c71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f9c78, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579c81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf379c84, 3 },
    { 0, 0 },
    { 0x619f9c86, 4 },
    { 0, 0 },
    { 0x44b79c88, 3 },
    { 0, 0 },
    { 0xc5ff9c8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb9c8d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50f9c92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f579c97, 3 },
    { 0, 0 },
    { 0xaeef9c99, 3 },
    { 0, 0 },
    { 0xacff9c9b, 3 },
    { 0xc5fb9c9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11f9ca1, 3 },
    { 0, 0 },
    { 0xd7ef9ca3, 4 },
    { 0x167f9ca4, 4 },
    { 0xb5f39ca4, 3 },
    { 0x28cb9ca3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38f9caa, 3 },
    { 0xf9379cab, 4 },
    { 0xcf679cac, 3 },
    { 0x83379cab, 4 },
    { 0x9a79cae, 4 },
    { 0x259b9caf, 4 },
    { 0x9fbf9caa, 3 },
    { 0x35df9cb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6939cb8, 3 },
    { 0, 0 },
    { 0x861b9cba, 3 },
    { 0xc1ff9cba, 3 },
    { 0, 0 },
    { 0xf3f79cbd, 3 },
    { 0, 0 },
    { 0xb9ef9cbf, 3 },
    { 0xa7db9cc0, 3 },
    { 0x15c39cc0, 4 },
    { 0x7439cc2, 3 },
    { 0x72f39cc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d79cca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0579cce, 3 },
    { 0x52b39ccf, 3 },
    { 0x5dbf9cd0, 3 },
    { 0xf2eb9cd0, 3 },
    { 0xaf139ccf, 4 },
    { 0xc0579ccf, 3 },
    { 0x707b9ccf, 4 },
    { 0xf5b79cd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf779cdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45079ce5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf9ce8, 4 },
    { 0x4a2f9ce9, 3 },
    { 0, 0 },
    { 0x5e1b9ceb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c979cef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff9cf4, 3 },
    { 0x70b39cf5, 3 },
    { 0x13979cf6, 3 },
    { 0xf77f9cf7, 3 },
    { 0x5cdf9cf8, 3 },
    { 0xa2cf9cf8, 3 },
    { 0x1a439cf9, 3 },
    { 0xf4439cfb, 3 },
    { 0, 0 },
    { 0x5fc79cfd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x348b9d00, 5 },
    { 0xf5239d00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983b9d06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25df9d0c, 3 },
    { 0, 0 },
    { 0xdbe39d0e, 3 },
    { 0xfb539d0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb9d15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbf9d18, 3 },
    { 0xe94b9d18, 3 },
    { 0xfa2f9d18, 3 },
    { 0x63a39d1b, 3 },
    { 0x17979d19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83379d23, 3 },
    { 0x11ab9d24, 3 },
    { 0xe439d25, 4 },
    { 0x64539d24, 3 },
    { 0xc0f9d27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5f9d2f, 3 },
    { 0x928f9d30, 3 },
    { 0xfed79d31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5b9d37, 4 },
    { 0, 0 },
    { 0x7aab9d39, 3 },
    { 0xb4df9d39, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc79d3d, 3 },
    { 0x51039d3e, 3 },
    { 0x3ca39d3f, 4 },
    { 0, 0 },
    { 0x3eaf9d41, 5 },
    { 0xe68f9d42, 3 },
    { 0x315f9d43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238b9d49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45439d4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f9d52, 3 },
    { 0x5d39d52, 3 },
    { 0xc8579d53, 3 },
    { 0, 0 },
    { 0xef379d56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1b9d5d, 3 },
    { 0x44b39d5d, 3 },
    { 0xefdb9d5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b139d63, 4 },
    { 0xb4ff9d63, 3 },
    { 0xe15f9d65, 3 },
    { 0, 0 },
    { 0xae679d67, 4 },
    { 0xe4a79d68, 3 },
    { 0x5adb9d68, 3 },
    { 0x12839d6a, 3 },
    { 0x1b1b9d6b, 3 },
    { 0xe41b9d6c, 3 },
    { 0, 0 },
    { 0x91ab9d6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3f9d77, 4 },
    { 0, 0 },
    { 0xc7439d79, 3 },
    { 0xd10b9d7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41279d83, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b79d8b, 4 },
    { 0x5bd79d8b, 3 },
    { 0xd4c79d8d, 4 },
    { 0, 0 },
    { 0xdd9f9d8f, 3 },
    { 0x89ef9d90, 3 },
    { 0x7eb79d90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979d97, 5 },
    { 0x20a39d98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3af9d9d, 3 },
    { 0x80439d9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f9da1, 4 },
    { 0xb01b9da1, 3 },
    { 0xc0af9da1, 3 },
    { 0, 0 },
    { 0xd5c39da5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd139da9, 3 },
    { 0x8fa39daa, 4 },
    { 0xa67f9daa, 3 },
    { 0x1f9b9dab, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcb9db3, 4 },
    { 0x7c8b9db4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4079db8, 3 },
    { 0x5ccb9db9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f9dc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99439dc6, 3 },
    { 0x934b9dc6, 3 },
    { 0x9779dc8, 3 },
    { 0xc6bf9dc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81979dd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e79dd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f739ddb, 3 },
    { 0, 0 },
    { 0xdbdb9ddd, 4 },
    { 0x29679dde, 4 },
    { 0, 0 },
    { 0x93239de0, 3 },
    { 0x7e4b9de1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195b9de9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13179dec, 3 },
    { 0x881b9dec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7079df0, 3 },
    { 0x6839df1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb9df8, 3 },
    { 0, 0 },
    { 0xf79f9dfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ef9dfe, 3 },
    { 0x3a3f9dff, 4 },
    { 0, 0 },
    { 0xa0fb9e01, 3 },
    { 0, 0 },
    { 0x156b9e03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9139e0b, 3 },
    { 0, 0 },
    { 0xa1db9e0d, 4 },
    { 0, 0 },
    { 0x6c579e0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa39e13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0b9e19, 3 },
    { 0xaa6b9e1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3b9e1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9db9e23, 3 },
    { 0x7a479e24, 3 },
    { 0xef639e25, 3 },
    { 0x2b6b9e26, 4 },
    { 0xe03b9e27, 3 },
    { 0, 0 },
    { 0xfed79e29, 3 },
    { 0, 0 },
    { 0x49db9e2b, 4 },
    { 0xe0f9e2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84b9e2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa79e34, 3 },
    { 0, 0 },
    { 0x16579e36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f9e3a, 3 },
    { 0xe29f9e3b, 3 },
    { 0x527b9e3c, 3 },
    { 0x41479e3d, 3 },
    { 0x4fb9e3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f9e46, 3 },
    { 0x264b9e46, 3 },
    { 0x108b9e48, 3 },
    { 0x6bcb9e49, 4 },
    { 0x78139e49, 3 },
    { 0xebab9e4b, 3 },
    { 0x83af9e4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b39e51, 3 },
    { 0xa279e51, 3 },
    { 0x8d039e53, 4 },
    { 0x2ebf9e52, 3 },
    { 0xd4879e55, 3 },
    { 0x66af9e56, 3 },
    { 0xe82b9e57, 4 },
    { 0x394b9e56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2b9e5b, 4 },
    { 0, 0 },
    { 0xa98b9e5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26b9e60, 4 },
    { 0, 0 },
    { 0x87ab9e62, 3 },
    { 0, 0 },
    { 0xae939e64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x537f9e67, 3 },
    { 0xda1f9e68, 3 },
    { 0xd91b9e69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61af9e6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72b9e80, 3 },
    { 0, 0 },
    { 0xfa3b9e82, 4 },
    { 0, 0 },
    { 0xbeb9e84, 3 },
    { 0xb6179e85, 4 },
    { 0x70a79e84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28539e8b, 4 },
    { 0, 0 },
    { 0x4e2f9e8d, 4 },
    { 0x19279e8e, 3 },
    { 0, 0 },
    { 0xfaf39e90, 3 },
    { 0, 0 },
    { 0xa1bf9e92, 4 },
    { 0xf55f9e92, 3 },
    { 0xffdf9e92, 3 },
    { 0, 0 },
    { 0xda879e96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34879e9e, 3 },
    { 0, 0 },
    { 0xfab79ea0, 3 },
    { 0, 0 },
    { 0xbfc79ea2, 3 },
    { 0, 0 },
    { 0x2ab9ea4, 4 },
    { 0x8f1f9ea5, 3 },
    { 0x52cb9ea5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50f9eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c79eaf, 3 },
    { 0xb1ab9eaf, 3 },
    { 0xbfd39eaf, 3 },
    { 0, 0 },
    { 0xcc8f9eb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282f9eb8, 3 },
    { 0x57db9eb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b9ebf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c379ec5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9ece, 3 },
    { 0x33bf9ecf, 3 },
    { 0xedff9ed0, 3 },
    { 0xe7039ecf, 3 },
    { 0xe5239ed2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da39ed6, 3 },
    { 0, 0 },
    { 0xb2e79ed8, 3 },
    { 0x427f9ed8, 3 },
    { 0xb3879eda, 4 },
    { 0x95f79edb, 5 },
    { 0, 0 },
    { 0xe4639edd, 4 },
    { 0x6b39ede, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780f9ee7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36879eeb, 3 },
    { 0x4dfb9eeb, 3 },
    { 0xf0079eed, 3 },
    { 0xaeaf9eec, 4 },
    { 0x4cc79eef, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cf9ef3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48679ef8, 3 },
    { 0x3a579ef9, 3 },
    { 0, 0 },
    { 0x3daf9efb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6f9f00, 4 },
    { 0, 0 },
    { 0x6c7b9f02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9279f0b, 3 },
    { 0, 0 },
    { 0xe0b39f0d, 3 },
    { 0, 0 },
    { 0xe5fb9f0f, 4 },
    { 0xa79f9f10, 3 },
    { 0x237b9f11, 3 },
    { 0x8d079f12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b79f15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6739f1b, 3 },
    { 0, 0 },
    { 0xf03b9f1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedb9f20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4df9f25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf39f31, 3 },
    { 0xc8539f31, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3f9f38, 4 },
    { 0x29939f39, 3 },
    { 0x88939f38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b9f41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4179f44, 5 },
    { 0xbc679f45, 3 },
    { 0x9e079f45, 3 },
    { 0xa3db9f46, 3 },
    { 0xa9439f45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa79f4e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f9f54, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79879f5f, 3 },
    { 0xa5839f5f, 3 },
    { 0x6ba79f60, 3 },
    { 0xdba79f5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c39f65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94db9f69, 4 },
    { 0xc87f9f6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d39f6f, 3 },
    { 0xdc279f70, 4 },
    { 0x7dd79f71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x84af9f74, 4 },
    { 0x32c39f74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c39f79, 3 },
    { 0, 0 },
    { 0xd8439f7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9239f80, 3 },
    { 0, 0 },
    { 0x8a3f9f82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa979f85, 3 },
    { 0x39c39f85, 3 },
    { 0x50bf9f87, 3 },
    { 0xe01b9f88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed39f97, 4 },
    { 0, 0 },
    { 0xfab9f99, 4 },
    { 0x138b9f9a, 3 },
    { 0xabe39f9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3f9fa2, 3 },
    { 0x31979fa3, 3 },
    { 0x8b139fa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5839fa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7db9fae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcb9fb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb39fb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3b9fbe, 3 },
    { 0x7a5f9fbf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaab9fc4, 4 },
    { 0x46c79fc4, 4 },
    { 0x552b9fc5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa79fca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46639fcf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb9fdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efb9fe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67479fe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf39fec, 3 },
    { 0x6fe79fec, 3 },
    { 0x97d39fee, 3 },
    { 0x986f9fed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b9ff2, 3 },
    { 0xfbef9ff3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a79ffa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133a002, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ba00f, 3 },
    { 0x80dfa010, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f03a01b, 4 },
    { 0xb05fa01c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fa01f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7a022, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe387a027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ba02e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7663a031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3a034, 3 },
    { 0x33b7a034, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a7a044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fa047, 3 },
    { 0xb8c3a048, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bfa04b, 4 },
    { 0x3203a04c, 3 },
    { 0x875ba04d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee87a055, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4ba05b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6913a062, 3 },
    { 0x8f37a063, 3 },
    { 0x855ba062, 3 },
    { 0xd99fa062, 3 },
    { 0, 0 },
    { 0xa0c7a067, 4 },
    { 0xdb57a067, 3 },
    { 0x3bf3a069, 4 },
    { 0xd89fa06a, 3 },
    { 0x563ba069, 3 },
    { 0x6aaba069, 4 },
    { 0, 0 },
    { 0x3603a06e, 3 },
    { 0, 0 },
    { 0x3743a070, 3 },
    { 0xe6dba070, 5 },
    { 0, 0 },
    { 0x41efa073, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6203a07a, 3 },
    { 0xaefa07b, 3 },
    { 0, 0 },
    { 0xb50fa07d, 3 },
    { 0x3ac7a07e, 3 },
    { 0, 0 },
    { 0xd22ba080, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbffa085, 3 },
    { 0x646fa085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63a08a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46ba08e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a67a091, 3 },
    { 0x6e6ba092, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c27a099, 3 },
    { 0, 0 },
    { 0xbbcba09b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5477a09e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2367a0a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53a0a6, 3 },
    { 0, 0 },
    { 0x1ccfa0a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225fa0ad, 3 },
    { 0, 0 },
    { 0xc97ba0af, 4 },
    { 0, 0 },
    { 0xbf33a0b1, 3 },
    { 0x65afa0b2, 3 },
    { 0xf493a0b3, 4 },
    { 0x13f3a0b4, 3 },
    { 0xcc9ba0b5, 3 },
    { 0, 0 },
    { 0x9d27a0b7, 3 },
    { 0xaf5fa0b8, 3 },
    { 0xc9fa0b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7a0bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873a0c2, 3 },
    { 0xfab3a0c2, 3 },
    { 0x547a0c2, 3 },
    { 0x46eba0c5, 3 },
    { 0, 0 },
    { 0x6cbba0c7, 3 },
    { 0, 0 },
    { 0x27c7a0c9, 3 },
    { 0, 0 },
    { 0x617a0cb, 3 },
    { 0x9d77a0cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bfa0d7, 5 },
    { 0x6833a0d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393a0dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d7a0e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787a0e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efba0e9, 4 },
    { 0x1be7a0ea, 3 },
    { 0xe413a0ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37ba0ee, 3 },
    { 0x8c8ba0ef, 4 },
    { 0x9cfba0f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067a0f3, 3 },
    { 0, 0 },
    { 0x879fa0f5, 3 },
    { 0xaafa0f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x798fa0f9, 5 },
    { 0x1d83a0fa, 3 },
    { 0xa813a0f9, 4 },
    { 0x17cba0fc, 3 },
    { 0, 0 },
    { 0x5b1fa0fe, 5 },
    { 0xd72ba0fe, 3 },
    { 0xac0ba0ff, 3 },
    { 0xa6fba0ff, 3 },
    { 0x20b3a102, 4 },
    { 0x73a102, 3 },
    { 0x9987a102, 4 },
    { 0, 0 },
    { 0xa74fa106, 4 },
    { 0xcc87a107, 4 },
    { 0x95e7a107, 3 },
    { 0xda17a109, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e7a10c, 4 },
    { 0, 0 },
    { 0xb10ba10e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5fa112, 3 },
    { 0, 0 },
    { 0x9bafa114, 4 },
    { 0x5a57a115, 4 },
    { 0x3cffa116, 3 },
    { 0x9fcfa117, 3 },
    { 0xe36fa118, 3 },
    { 0xd643a119, 3 },
    { 0xf987a11a, 3 },
    { 0x7b1ba119, 4 },
    { 0x3d83a11c, 3 },
    { 0xf0b3a11a, 3 },
    { 0x41ffa11e, 3 },
    { 0xa8a3a11f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c27a123, 3 },
    { 0xda1fa124, 4 },
    { 0xce53a123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadba129, 3 },
    { 0x9c1fa12a, 3 },
    { 0x9d3a12b, 4 },
    { 0xa9fba12c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113a12f, 3 },
    { 0x9057a130, 3 },
    { 0xe877a12f, 3 },
    { 0xce57a132, 3 },
    { 0x7023a133, 3 },
    { 0xf5bfa134, 3 },
    { 0xc763a134, 3 },
    { 0xfb7a132, 3 },
    { 0x707ba137, 3 },
    { 0x4f3ba136, 3 },
    { 0, 0 },
    { 0xaeb7a13a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559ba140, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a87a14c, 3 },
    { 0, 0 },
    { 0x3f17a14e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be7a153, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a7a15c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3a160, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8987a163, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabba167, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e3a16d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfa171, 3 },
    { 0xe433a172, 3 },
    { 0x399ba173, 3 },
    { 0x46f7a174, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4affa17b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bba17f, 3 },
    { 0x2bd7a17f, 3 },
    { 0xb6cba181, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cba184, 3 },
    { 0x8c27a185, 4 },
    { 0, 0 },
    { 0xca9ba187, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61ba192, 4 },
    { 0, 0 },
    { 0xa943a194, 3 },
    { 0x78cfa195, 3 },
    { 0, 0 },
    { 0x7ae3a197, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc67a1a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93ba1a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53a1b1, 3 },
    { 0, 0 },
    { 0xcdd3a1b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07a1b7, 3 },
    { 0xf473a1b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567a1bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0afa1bf, 3 },
    { 0, 0 },
    { 0x94cfa1c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd7a1cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cfa1cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2913a1d4, 3 },
    { 0x3353a1d4, 5 },
    { 0x7fb7a1d6, 3 },
    { 0xe0f3a1d7, 4 },
    { 0x4a3ba1d7, 3 },
    { 0xd88ba1d6, 4 },
    { 0x702ba1d5, 4 },
    { 0x42dfa1d6, 3 },
    { 0x3443a1d4, 4 },
    { 0xcf77a1dd, 3 },
    { 0xc533a1de, 4 },
    { 0x9827a1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a1e7, 4 },
    { 0, 0 },
    { 0xcbcfa1e9, 3 },
    { 0, 0 },
    { 0xe933a1eb, 3 },
    { 0, 0 },
    { 0x610fa1ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9ba1f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8727a1fa, 3 },
    { 0xe987a1fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeba1ff, 3 },
    { 0, 0 },
    { 0x6a4fa201, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3a205, 4 },
    { 0x5477a206, 4 },
    { 0, 0 },
    { 0x70b7a208, 3 },
    { 0x6a0ba208, 3 },
    { 0x993a20a, 3 },
    { 0, 0 },
    { 0x919ba20c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ba20f, 3 },
    { 0, 0 },
    { 0xa5fba211, 3 },
    { 0x2cdba211, 3 },
    { 0xa71fa213, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58fa216, 3 },
    { 0x166ba217, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x223ba21a, 4 },
    { 0x141fa21b, 4 },
    { 0x1f83a21b, 4 },
    { 0x5e5ba21d, 3 },
    { 0, 0 },
    { 0xa37ba21f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ba225, 3 },
    { 0xc9bfa226, 3 },
    { 0xaf7ba227, 4 },
    { 0, 0 },
    { 0x7b97a229, 4 },
    { 0x253ba22a, 3 },
    { 0x6303a22b, 4 },
    { 0x4ddfa22c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ba231, 3 },
    { 0x8a03a231, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd53a235, 3 },
    { 0x79b3a236, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aba239, 5 },
    { 0xff2fa23a, 3 },
    { 0x865ba23b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f3a23f, 3 },
    { 0x40ba240, 4 },
    { 0x7763a23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7677a244, 3 },
    { 0xf523a245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb987a249, 3 },
    { 0x4607a24a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3a24d, 4 },
    { 0x5173a24d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b57a251, 4 },
    { 0x5e8fa251, 3 },
    { 0, 0 },
    { 0x772fa254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3a257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d3a25b, 3 },
    { 0x5bd3a25c, 3 },
    { 0xa957a25b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd93a260, 3 },
    { 0x3b3fa260, 3 },
    { 0xb71fa261, 3 },
    { 0, 0 },
    { 0x277a264, 3 },
    { 0xa237a264, 3 },
    { 0, 0 },
    { 0x5e1ba267, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adba26a, 3 },
    { 0xef3a26b, 3 },
    { 0x2283a26b, 3 },
    { 0, 0 },
    { 0xed4fa26e, 3 },
    { 0, 0 },
    { 0xa207a270, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fa273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dba278, 3 },
    { 0, 0 },
    { 0x939fa27a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304fa27f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f3a285, 3 },
    { 0x9613a285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192ba28b, 3 },
    { 0xec93a28b, 3 },
    { 0, 0 },
    { 0x5b5ba28e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bba293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1393a296, 3 },
    { 0x86cba297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d7a29b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfba29e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8fa2a4, 3 },
    { 0, 0 },
    { 0xf37ba2a6, 3 },
    { 0, 0 },
    { 0xf0f7a2a8, 3 },
    { 0xa0bfa2a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ffa2ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6fa2b0, 3 },
    { 0xe60fa2b1, 4 },
    { 0x958fa2b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2fa2b8, 3 },
    { 0x912ba2b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffa2bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fa2c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fba2c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fa2ca, 4 },
    { 0x3eeba2cb, 3 },
    { 0x5a93a2cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf3a2d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270fa2e1, 3 },
    { 0x33c7a2e2, 3 },
    { 0xfa07a2e3, 3 },
    { 0xaf0fa2e3, 4 },
    { 0xdfffa2e3, 4 },
    { 0, 0 },
    { 0x7217a2e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cba2f3, 3 },
    { 0xb237a2f4, 4 },
    { 0, 0 },
    { 0x913fa2f6, 3 },
    { 0xd6a3a2f7, 3 },
    { 0x3eb7a2f8, 4 },
    { 0, 0 },
    { 0xc387a2fa, 3 },
    { 0, 0 },
    { 0x6d47a2fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79ba301, 4 },
    { 0x56dba302, 4 },
    { 0, 0 },
    { 0x1103a304, 5 },
    { 0xa5e7a304, 3 },
    { 0x7663a306, 3 },
    { 0x5f83a307, 4 },
    { 0xe3f3a308, 3 },
    { 0xb2b3a307, 4 },
    { 0x3eafa30a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cfa30d, 3 },
    { 0, 0 },
    { 0xc183a30f, 3 },
    { 0xd8ffa310, 4 },
    { 0, 0 },
    { 0x1c0fa312, 3 },
    { 0x9287a312, 4 },
    { 0, 0 },
    { 0x4f03a315, 4 },
    { 0, 0 },
    { 0x51c3a317, 4 },
    { 0x2ea7a318, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdba31d, 3 },
    { 0, 0 },
    { 0xd7cba31f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbfa328, 3 },
    { 0xc0a3a329, 3 },
    { 0xd1bfa32a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297a331, 3 },
    { 0xb77ba332, 3 },
    { 0x805ba331, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942fa337, 3 },
    { 0x1e6ba338, 3 },
    { 0x6d7ba339, 5 },
    { 0x9693a339, 3 },
    { 0, 0 },
    { 0x6633a33c, 3 },
    { 0x936ba33d, 3 },
    { 0, 0 },
    { 0x336ba33f, 3 },
    { 0x179fa340, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x193a343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe203a346, 4 },
    { 0, 0 },
    { 0x7413a348, 3 },
    { 0xed7ba349, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7fa34f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe733a358, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb13a35b, 3 },
    { 0, 0 },
    { 0x896ba35d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dba362, 3 },
    { 0, 0 },
    { 0x20b7a364, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x223fa367, 3 },
    { 0, 0 },
    { 0x26a7a369, 4 },
    { 0x5b97a36a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b07a36d, 3 },
    { 0xc327a36e, 3 },
    { 0xa187a36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4ba373, 4 },
    { 0xeb27a374, 3 },
    { 0xdcb3a375, 3 },
    { 0xb993a376, 4 },
    { 0xc557a377, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb7a37a, 3 },
    { 0xa6fa37a, 4 },
    { 0xffb3a37c, 3 },
    { 0x35b3a37c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcfa380, 3 },
    { 0xf7efa381, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa883a393, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cba396, 3 },
    { 0, 0 },
    { 0x8baba398, 3 },
    { 0xdf07a399, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814fa3a1, 4 },
    { 0xbabba3a1, 4 },
    { 0x87d7a3a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff7a3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ba3ab, 3 },
    { 0x83dfa3ac, 3 },
    { 0x8a7a3ad, 3 },
    { 0x1b5ba3ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424ba3b5, 3 },
    { 0x5a73a3b6, 3 },
    { 0, 0 },
    { 0x9643a3b8, 3 },
    { 0xd053a3b9, 3 },
    { 0xa66ba3b9, 4 },
    { 0, 0 },
    { 0x215fa3bc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea7a3c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989fa3cb, 3 },
    { 0xff3ba3cc, 3 },
    { 0x87e7a3cd, 3 },
    { 0, 0 },
    { 0x14e7a3cf, 3 },
    { 0, 0 },
    { 0x6893a3d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d07a3d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37fa3d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acba3e1, 4 },
    { 0x184ba3e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875ba3e8, 4 },
    { 0xbfc7a3e9, 3 },
    { 0xcafa3ea, 4 },
    { 0x7f8ba3eb, 4 },
    { 0x4e0fa3ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197a3f5, 3 },
    { 0xdc5ba3f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dba3fa, 3 },
    { 0, 0 },
    { 0xdc73a3fc, 3 },
    { 0x5a27a3fd, 3 },
    { 0x1613a3fc, 3 },
    { 0xd73fa3ff, 4 },
    { 0x1783a3ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b23a404, 3 },
    { 0xbb4ba405, 3 },
    { 0, 0 },
    { 0xfa8fa407, 5 },
    { 0x6867a408, 3 },
    { 0, 0 },
    { 0xb393a40a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d3a40e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f73a415, 3 },
    { 0xac23a416, 3 },
    { 0x184ba415, 3 },
    { 0x4d2ba416, 4 },
    { 0x3133a419, 3 },
    { 0x5fb7a41a, 3 },
    { 0xf253a41b, 3 },
    { 0x3563a41a, 3 },
    { 0xc44fa41d, 3 },
    { 0x707fa41e, 3 },
    { 0x47e7a41f, 4 },
    { 0x4e8ba420, 3 },
    { 0x1087a415, 3 },
    { 0x8cbba415, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ba428, 3 },
    { 0x7a97a429, 4 },
    { 0x8393a42a, 3 },
    { 0x1607a42a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afa431, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa177a434, 3 },
    { 0x1a6fa435, 3 },
    { 0, 0 },
    { 0x2ff3a437, 3 },
    { 0xd37ba438, 3 },
    { 0xc757a439, 3 },
    { 0x5f77a43a, 3 },
    { 0x4e37a438, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb867a43e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb217a442, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3a446, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fba449, 4 },
    { 0x5787a44a, 3 },
    { 0x68ffa44b, 4 },
    { 0x5c9ba44b, 3 },
    { 0xa79fa44d, 3 },
    { 0x7953a44e, 3 },
    { 0, 0 },
    { 0xb8cfa450, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe613a454, 4 },
    { 0x3bd7a455, 4 },
    { 0x5b43a456, 4 },
    { 0xd5ffa457, 3 },
    { 0xa1bfa457, 3 },
    { 0, 0 },
    { 0x4517a45a, 3 },
    { 0xd1b3a45b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3ba45f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ba462, 3 },
    { 0x98f7a463, 3 },
    { 0x3397a462, 3 },
    { 0xa937a465, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0fa46d, 3 },
    { 0, 0 },
    { 0xe37fa46f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f7a475, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c3a478, 3 },
    { 0, 0 },
    { 0xd3cba47a, 3 },
    { 0x6cffa47b, 3 },
    { 0x6857a47c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37a484, 3 },
    { 0x13fba485, 3 },
    { 0x85aba486, 3 },
    { 0xa08fa487, 3 },
    { 0xccc7a487, 4 },
    { 0xbfd3a489, 4 },
    { 0x7e53a489, 3 },
    { 0xde4ba48b, 5 },
    { 0, 0 },
    { 0xaf5fa48d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4fa490, 3 },
    { 0x5087a490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47ba494, 3 },
    { 0x5993a495, 4 },
    { 0xa13ba495, 3 },
    { 0xef3fa494, 3 },
    { 0x6133a498, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809fa4a0, 3 },
    { 0xc86ba4a0, 4 },
    { 0x852ba4a2, 3 },
    { 0x5183a4a3, 3 },
    { 0xee43a4a3, 3 },
    { 0, 0 },
    { 0x737ba4a6, 3 },
    { 0xdc47a4a6, 3 },
    { 0, 0 },
    { 0xd6e7a4a9, 4 },
    { 0, 0 },
    { 0xb3aba4ab, 4 },
    { 0x9fc7a4ac, 4 },
    { 0x27a7a4ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbba4b1, 3 },
    { 0, 0 },
    { 0x5e07a4b3, 3 },
    { 0xa1e3a4b4, 4 },
    { 0x7547a4b4, 4 },
    { 0x7387a4b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda47a4bb, 3 },
    { 0x7317a4bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f3a4c1, 3 },
    { 0x968fa4c1, 3 },
    { 0xe80fa4c3, 4 },
    { 0x5d5ba4c4, 4 },
    { 0x49eba4c5, 5 },
    { 0x4adfa4c1, 3 },
    { 0x89aba4c7, 3 },
    { 0xef37a4c8, 3 },
    { 0, 0 },
    { 0x10bfa4ca, 5 },
    { 0x6be3a4cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123a4d5, 3 },
    { 0x6083a4d6, 3 },
    { 0x40b7a4d7, 3 },
    { 0, 0 },
    { 0xf05fa4d9, 3 },
    { 0, 0 },
    { 0x5dfba4db, 4 },
    { 0, 0 },
    { 0x15d3a4dd, 4 },
    { 0x9d37a4de, 3 },
    { 0, 0 },
    { 0xba37a4e0, 3 },
    { 0, 0 },
    { 0x9067a4e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x925ba4e5, 3 },
    { 0, 0 },
    { 0x6943a4e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fba4ea, 4 },
    { 0x3e97a4eb, 3 },
    { 0xa7ffa4ec, 4 },
    { 0xd753a4eb, 4 },
    { 0, 0 },
    { 0x5283a4ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bfa4f2, 3 },
    { 0, 0 },
    { 0x57f7a4f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ba4f7, 4 },
    { 0x1aefa4f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693fa4fc, 4 },
    { 0, 0 },
    { 0xdb2fa4fe, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76ba505, 3 },
    { 0, 0 },
    { 0x76e7a507, 3 },
    { 0x3cd7a508, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e3a50f, 4 },
    { 0xcbffa510, 3 },
    { 0x84ffa510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d3a514, 3 },
    { 0xb933a515, 3 },
    { 0x2bfa515, 3 },
    { 0x2593a514, 3 },
    { 0, 0 },
    { 0x6b17a519, 3 },
    { 0x3baba519, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3a520, 3 },
    { 0, 0 },
    { 0xb3c3a522, 3 },
    { 0, 0 },
    { 0x5eaba524, 4 },
    { 0, 0 },
    { 0xcca3a526, 4 },
    { 0, 0 },
    { 0x49bba528, 3 },
    { 0, 0 },
    { 0x15a7a52a, 3 },
    { 0x84bba52b, 3 },
    { 0xb697a52c, 3 },
    { 0x12fa52d, 3 },
    { 0xd817a52e, 3 },
    { 0x1ed7a52e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930fa538, 3 },
    { 0xb8ffa539, 3 },
    { 0, 0 },
    { 0xde53a53b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x471ba53e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc73a544, 3 },
    { 0, 0 },
    { 0x103fa546, 3 },
    { 0x978ba547, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ba54a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97ba54e, 3 },
    { 0xed6ba54f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430fa553, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de7a557, 3 },
    { 0x885ba558, 4 },
    { 0x4553a558, 4 },
    { 0xd44ba55a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a560, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b3a563, 3 },
    { 0xebafa564, 3 },
    { 0x7cd7a565, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4823a568, 3 },
    { 0, 0 },
    { 0xf603a56a, 4 },
    { 0xd783a56b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693a571, 3 },
    { 0x25bba572, 3 },
    { 0x55dfa573, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe67a577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32fa57a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x262fa57d, 4 },
    { 0, 0 },
    { 0x6827a57f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738fa585, 3 },
    { 0xcba3a586, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793a58d, 3 },
    { 0x272ba58e, 3 },
    { 0, 0 },
    { 0xc003a590, 3 },
    { 0, 0 },
    { 0x1eb7a592, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fa596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d3a59c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dfa5a4, 3 },
    { 0xe40fa5a5, 4 },
    { 0xd2ba5a6, 3 },
    { 0x4a8fa5a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26fa5ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3927a5b1, 3 },
    { 0x20e7a5b1, 3 },
    { 0xd307a5b1, 5 },
    { 0, 0 },
    { 0x5d5ba5b5, 4 },
    { 0, 0 },
    { 0xe487a5b7, 3 },
    { 0, 0 },
    { 0x6263a5b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa03a5bf, 3 },
    { 0x79d7a5c0, 3 },
    { 0x4117a5c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cba5c4, 3 },
    { 0xe413a5c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5773a5ca, 3 },
    { 0x3ea3a5cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e3a5ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ba5d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5ba5d6, 3 },
    { 0x1857a5d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adfa5da, 3 },
    { 0x240fa5db, 3 },
    { 0xc98ba5dc, 3 },
    { 0x761ba5dd, 3 },
    { 0x5483a5de, 3 },
    { 0xa75ba5df, 3 },
    { 0x674fa5df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7aba5e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1897a5e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba5ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77a5ef, 5 },
    { 0, 0 },
    { 0x98e7a5f1, 4 },
    { 0xb8b7a5f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7a5fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b43a605, 4 },
    { 0x7213a606, 3 },
    { 0x10c3a607, 4 },
    { 0, 0 },
    { 0xdb37a609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9fa60d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32fa610, 4 },
    { 0x8b93a611, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fa616, 4 },
    { 0x3d27a616, 4 },
    { 0x183fa618, 3 },
    { 0xdf63a619, 3 },
    { 0xbbcba61a, 4 },
    { 0, 0 },
    { 0x58ba61c, 3 },
    { 0xd367a61d, 3 },
    { 0x796fa61e, 4 },
    { 0xc013a61f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf7a629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624fa638, 3 },
    { 0x39ffa639, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cfa63e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fba644, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x774fa647, 3 },
    { 0x3883a648, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efa64e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d7a651, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bfa657, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce13a65b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb827a660, 3 },
    { 0xd683a661, 3 },
    { 0xb9a7a660, 3 },
    { 0x4a87a662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d7a66a, 5 },
    { 0x77a3a66b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3407a674, 3 },
    { 0, 0 },
    { 0xb2c7a676, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8fa679, 3 },
    { 0x73aba67a, 3 },
    { 0xc17a67b, 3 },
    { 0, 0 },
    { 0x1627a67d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bba685, 3 },
    { 0x1b3fa685, 3 },
    { 0x89ffa685, 5 },
    { 0xfdefa688, 3 },
    { 0, 0 },
    { 0x535ba68a, 3 },
    { 0xfa63a68a, 3 },
    { 0x62c7a68c, 3 },
    { 0x1613a68c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fba690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9fa693, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdfa698, 4 },
    { 0, 0 },
    { 0xde9ba69a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d07a6a0, 3 },
    { 0xbb93a6a1, 3 },
    { 0x71d7a6a1, 4 },
    { 0xce43a6a3, 3 },
    { 0x4d3ba6a0, 3 },
    { 0x6713a6a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f7a6ab, 3 },
    { 0x8663a6ac, 3 },
    { 0x2ffa6ad, 3 },
    { 0xf3fba6ae, 3 },
    { 0xdce7a6af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cba6b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fa6b6, 4 },
    { 0xdcefa6b6, 3 },
    { 0xf02fa6b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33a6bd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3a6c4, 4 },
    { 0xf45fa6c5, 3 },
    { 0x9f4ba6c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x462ba6c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5383a6cc, 3 },
    { 0, 0 },
    { 0xa357a6ce, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ba6d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e77a6d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6ba6de, 5 },
    { 0x6c57a6df, 3 },
    { 0x95f3a6df, 3 },
    { 0xb3e3a6de, 3 },
    { 0, 0 },
    { 0x1387a6e3, 3 },
    { 0, 0 },
    { 0xcec7a6e5, 3 },
    { 0xbb37a6e5, 3 },
    { 0x76eba6e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9073a6ec, 3 },
    { 0, 0 },
    { 0x5eafa6ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ba6f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffa6f6, 3 },
    { 0x8a7fa6f6, 4 },
    { 0x8fe7a6f7, 3 },
    { 0x9e43a6f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47a700, 3 },
    { 0xc367a700, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe437a705, 3 },
    { 0x7053a705, 3 },
    { 0xdb5fa705, 4 },
    { 0xc0ffa708, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7fa716, 3 },
    { 0, 0 },
    { 0x92f3a718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fba71b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a71f, 4 },
    { 0xc00ba720, 4 },
    { 0x7ba7a721, 3 },
    { 0xd503a721, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c07a726, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f3a72c, 3 },
    { 0xb2dba72d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ba731, 3 },
    { 0x86d3a731, 4 },
    { 0xac5ba733, 4 },
    { 0x9323a733, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a7a73a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2fa740, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7643a744, 3 },
    { 0x59fa744, 4 },
    { 0x6fe7a746, 3 },
    { 0x4c4ba747, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527a74d, 3 },
    { 0x11a7a74e, 4 },
    { 0xba47a74d, 4 },
    { 0xd89fa750, 3 },
    { 0, 0 },
    { 0xeeba752, 3 },
    { 0, 0 },
    { 0xe1aba754, 3 },
    { 0, 0 },
    { 0xcfcba756, 3 },
    { 0xc23fa757, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a75b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7a75e, 4 },
    { 0x27aba75f, 3 },
    { 0x192ba75e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7a764, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aba76a, 3 },
    { 0xbf97a76b, 3 },
    { 0x5123a76b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ba770, 4 },
    { 0x3dcba770, 4 },
    { 0xf82ba772, 3 },
    { 0xbdb7a773, 3 },
    { 0xe8ffa770, 3 },
    { 0, 0 },
    { 0xdeba776, 3 },
    { 0xfe53a777, 3 },
    { 0x956fa776, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3553a77c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d7a783, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c83a786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6ba78a, 3 },
    { 0x8b73a78b, 3 },
    { 0xfff3a78c, 3 },
    { 0xd9c7a78c, 4 },
    { 0x625fa78b, 3 },
    { 0, 0 },
    { 0xc4e7a790, 4 },
    { 0xbc6ba790, 4 },
    { 0x7663a792, 3 },
    { 0, 0 },
    { 0x43a794, 3 },
    { 0xbf37a794, 3 },
    { 0x6073a796, 3 },
    { 0, 0 },
    { 0xef83a798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1fa79c, 3 },
    { 0, 0 },
    { 0xad83a79e, 3 },
    { 0, 0 },
    { 0xb1dfa7a0, 3 },
    { 0xd2b7a7a1, 3 },
    { 0x9e7a7a2, 3 },
    { 0x6163a7a2, 3 },
    { 0xbe9ba7a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44ba7a9, 3 },
    { 0, 0 },
    { 0xd383a7ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47a7ae, 3 },
    { 0, 0 },
    { 0xd153a7b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cba7b3, 3 },
    { 0, 0 },
    { 0xe6bfa7b5, 3 },
    { 0xf7f3a7b6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc233a7bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defa7c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ba7ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877fa7ce, 3 },
    { 0x4d67a7cf, 3 },
    { 0x23dfa7d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ba7db, 4 },
    { 0, 0 },
    { 0x702fa7dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee7a7e0, 3 },
    { 0xa353a7e1, 3 },
    { 0, 0 },
    { 0x923ba7e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4023a7e6, 3 },
    { 0x80a7a7e7, 3 },
    { 0x8cba7e7, 3 },
    { 0, 0 },
    { 0x6da7a7ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152fa7f0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdba7f4, 3 },
    { 0x9da7a7f4, 3 },
    { 0, 0 },
    { 0xaf07a7f7, 3 },
    { 0xf16fa7f8, 3 },
    { 0, 0 },
    { 0x521fa7fa, 3 },
    { 0, 0 },
    { 0x93dfa7fc, 3 },
    { 0xbfa7a7fd, 3 },
    { 0x63c7a7fe, 4 },
    { 0x6a8ba7fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883ba803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b7a806, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bba80c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457fa813, 3 },
    { 0xa93fa813, 5 },
    { 0xc46ba814, 3 },
    { 0x34aba815, 3 },
    { 0x6453a813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7733a820, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52fa828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32ba82c, 4 },
    { 0, 0 },
    { 0xf323a82e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09fa832, 3 },
    { 0, 0 },
    { 0xbe2fa834, 4 },
    { 0, 0 },
    { 0xc8e3a836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d3a839, 4 },
    { 0x3b63a83a, 5 },
    { 0xf63fa83b, 3 },
    { 0xc14fa83c, 4 },
    { 0x2b2ba83b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa633a847, 3 },
    { 0xa4eba848, 3 },
    { 0, 0 },
    { 0x1cd3a84a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca3a84d, 4 },
    { 0x8ccba84d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbba854, 5 },
    { 0, 0 },
    { 0xb8bba856, 3 },
    { 0, 0 },
    { 0xa2aba858, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d7a85b, 4 },
    { 0xfe63a85c, 4 },
    { 0xf25fa85c, 3 },
    { 0, 0 },
    { 0x35e3a85f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3a865, 4 },
    { 0x286fa866, 3 },
    { 0x6693a867, 3 },
    { 0x2d3ba868, 3 },
    { 0x8d97a868, 3 },
    { 0x3947a86a, 3 },
    { 0xa5ba86b, 4 },
    { 0xe6ba86a, 4 },
    { 0x8ee7a86b, 4 },
    { 0x1ec7a86e, 4 },
    { 0, 0 },
    { 0x237a870, 4 },
    { 0x9a0ba871, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102fa878, 4 },
    { 0x7187a879, 4 },
    { 0xf21ba87a, 3 },
    { 0x31bba879, 4 },
    { 0x87cfa87b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fa87f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d87a883, 4 },
    { 0xcb4ba884, 3 },
    { 0, 0 },
    { 0x5d4ba886, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ba889, 3 },
    { 0, 0 },
    { 0xd2f7a88b, 4 },
    { 0xf863a88c, 3 },
    { 0xc69ba88d, 4 },
    { 0, 0 },
    { 0xa007a88f, 3 },
    { 0xce33a88f, 4 },
    { 0, 0 },
    { 0x51dba892, 3 },
    { 0xe397a893, 3 },
    { 0, 0 },
    { 0xac03a895, 3 },
    { 0xd613a896, 3 },
    { 0x3167a897, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b33a89a, 3 },
    { 0x4203a89b, 3 },
    { 0xfabfa89b, 3 },
    { 0xa403a89d, 4 },
    { 0, 0 },
    { 0xee53a89f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27a8a2, 4 },
    { 0x2a3ba8a3, 4 },
    { 0x5b27a8a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3273a8ad, 3 },
    { 0xc3d7a8ad, 3 },
    { 0x6213a8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a7a8b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8873a8ba, 3 },
    { 0, 0 },
    { 0xadc7a8bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437a8c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bba8c9, 3 },
    { 0x3bc7a8c9, 5 },
    { 0xe197a8cb, 4 },
    { 0x790fa8c9, 3 },
    { 0x5f17a8cd, 4 },
    { 0, 0 },
    { 0xa5cba8cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657a8d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ffa8d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ba8dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb563a8e2, 3 },
    { 0x78a3a8e3, 3 },
    { 0x4e8ba8e4, 3 },
    { 0xa167a8e5, 4 },
    { 0x4f4ba8e6, 3 },
    { 0x1607a8e7, 3 },
    { 0x549fa8e8, 3 },
    { 0xbe8fa8e2, 3 },
    { 0x21fba8e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53a8ef, 3 },
    { 0x5983a8ef, 3 },
    { 0, 0 },
    { 0xadafa8f2, 3 },
    { 0x770ba8f3, 3 },
    { 0x8417a8f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc507a8f8, 3 },
    { 0, 0 },
    { 0xf28fa8fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eba900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afba90a, 3 },
    { 0xfb87a90a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38ba913, 4 },
    { 0, 0 },
    { 0xec03a915, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x773ba918, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0ba91e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4527a922, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443ba92b, 4 },
    { 0xe373a92c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2fa933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8227a937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5157a93b, 3 },
    { 0x6c33a93c, 5 },
    { 0xec0fa93d, 4 },
    { 0, 0 },
    { 0xf8cfa93f, 3 },
    { 0xe4fba940, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4837a945, 4 },
    { 0x9443a945, 3 },
    { 0xb5fa945, 3 },
    { 0, 0 },
    { 0x9d4ba949, 3 },
    { 0x356ba94a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a94f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383a955, 3 },
    { 0, 0 },
    { 0x73a3a957, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ba95b, 3 },
    { 0, 0 },
    { 0x78a3a95d, 4 },
    { 0x4bb7a95d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6033a961, 3 },
    { 0x839ba962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc7a968, 4 },
    { 0x590fa968, 4 },
    { 0x8de7a96a, 3 },
    { 0xe66ba969, 5 },
    { 0xac2ba969, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1513a970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b3a973, 4 },
    { 0x783a974, 3 },
    { 0x7f67a975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834fa980, 3 },
    { 0, 0 },
    { 0xb023a982, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403a98b, 3 },
    { 0x58d3a98b, 3 },
    { 0x946fa98d, 4 },
    { 0xa777a98c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6753a996, 3 },
    { 0, 0 },
    { 0xf5e7a998, 3 },
    { 0xc1a7a999, 3 },
    { 0x7a87a99a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdba99d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec67a9a6, 3 },
    { 0, 0 },
    { 0xeb53a9a8, 4 },
    { 0, 0 },
    { 0xd1e3a9aa, 3 },
    { 0x7ca7a9ab, 4 },
    { 0xd5f7a9ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1effa9b0, 3 },
    { 0xa827a9b1, 3 },
    { 0x512fa9b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ba9b6, 3 },
    { 0xa3bfa9b7, 3 },
    { 0, 0 },
    { 0x3ebfa9b9, 3 },
    { 0, 0 },
    { 0x2b87a9bb, 3 },
    { 0, 0 },
    { 0x416fa9bd, 3 },
    { 0x216fa9bd, 3 },
    { 0x54aba9bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b7a9c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c3a9d0, 3 },
    { 0x6787a9d1, 3 },
    { 0xc6cfa9d0, 3 },
    { 0x6457a9d0, 3 },
    { 0x2ef3a9d0, 4 },
    { 0x160ba9d5, 3 },
    { 0, 0 },
    { 0x7a63a9d7, 4 },
    { 0x86dfa9d8, 3 },
    { 0x6ba7a9d8, 3 },
    { 0x620ba9d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d53a9dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcba9e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec83a9ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00ba9ee, 3 },
    { 0x39efa9ee, 3 },
    { 0xb61fa9ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4933a9f7, 3 },
    { 0x11efa9f7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x741fa9fb, 3 },
    { 0xfb57a9fc, 3 },
    { 0x7c17a9fd, 3 },
    { 0x4c83a9fe, 4 },
    { 0x2fbfa9fe, 3 },
    { 0x2503aa00, 3 },
    { 0x911faa00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e3aa04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7aa0a, 3 },
    { 0, 0 },
    { 0x1ef7aa0c, 3 },
    { 0xcfb7aa0c, 3 },
    { 0x938faa0c, 3 },
    { 0xac13aa0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7baa12, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1baa16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cffaa1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd937aa1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e3aa26, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911baa2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7897aa30, 4 },
    { 0, 0 },
    { 0x2773aa32, 3 },
    { 0, 0 },
    { 0xcac3aa34, 3 },
    { 0x2227aa35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe607aa38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9afaa3b, 4 },
    { 0xece3aa3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87faa41, 3 },
    { 0x7ca3aa41, 4 },
    { 0, 0 },
    { 0xe97baa44, 3 },
    { 0x1f7baa45, 3 },
    { 0x2277aa46, 5 },
    { 0xb9c3aa46, 3 },
    { 0x2dd7aa48, 3 },
    { 0xa53baa49, 4 },
    { 0, 0 },
    { 0xea27aa4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3aa4f, 3 },
    { 0, 0 },
    { 0x8293aa51, 3 },
    { 0x31c7aa51, 3 },
    { 0x7c0baa53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15baa58, 4 },
    { 0x412baa58, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe37aa5c, 4 },
    { 0xd7a3aa5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323aa65, 3 },
    { 0x3477aa66, 4 },
    { 0x56baa67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fbaa6a, 3 },
    { 0xb847aa6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeffaa6f, 3 },
    { 0xf39faa70, 3 },
    { 0xc607aa71, 3 },
    { 0x1a27aa72, 3 },
    { 0xde07aa72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7faa77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0faa7c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76baa84, 4 },
    { 0, 0 },
    { 0xa4fbaa86, 3 },
    { 0, 0 },
    { 0x9187aa88, 3 },
    { 0xed87aa88, 3 },
    { 0, 0 },
    { 0x762faa8b, 3 },
    { 0x1fefaa8c, 4 },
    { 0x184baa8c, 3 },
    { 0xf577aa8e, 3 },
    { 0xbc17aa8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d7aa94, 3 },
    { 0, 0 },
    { 0x507baa96, 3 },
    { 0x4513aa97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86baa9a, 3 },
    { 0, 0 },
    { 0xea4faa9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf07aaa5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4323aaa9, 5 },
    { 0x8e13aaaa, 4 },
    { 0x919baaab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0faaae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a7aab1, 4 },
    { 0x794faab2, 3 },
    { 0x7dc7aab3, 3 },
    { 0x7fbfaab4, 3 },
    { 0xb80faab5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1faaba, 3 },
    { 0x6337aabb, 3 },
    { 0x3383aaba, 3 },
    { 0xac6faabb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7207aac2, 4 },
    { 0xaf73aac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3373aac8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30faacc, 3 },
    { 0, 0 },
    { 0xe7d3aace, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x82afaad1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1773aad4, 3 },
    { 0xda4baad4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b13aad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24abaade, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de7aae3, 3 },
    { 0xf96baae4, 3 },
    { 0x2befaae5, 3 },
    { 0, 0 },
    { 0xa863aae7, 3 },
    { 0, 0 },
    { 0xa84faae9, 3 },
    { 0, 0 },
    { 0x2517aaeb, 3 },
    { 0, 0 },
    { 0x3bc3aaed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77aaf3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cfaaf9, 3 },
    { 0xecdbaaf9, 4 },
    { 0xece3aafa, 4 },
    { 0, 0 },
    { 0xff9baafd, 3 },
    { 0, 0 },
    { 0xafbfaaff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfab02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3547ab06, 3 },
    { 0, 0 },
    { 0x474fab08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dbab0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc77ab0f, 4 },
    { 0xc5e7ab10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4233ab13, 3 },
    { 0x76fbab14, 3 },
    { 0, 0 },
    { 0x92dbab16, 3 },
    { 0, 0 },
    { 0xbe67ab18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ab1d, 4 },
    { 0x8d4bab1e, 3 },
    { 0, 0 },
    { 0xb31bab20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688bab24, 3 },
    { 0xf633ab25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bab29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40bab2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7973ab32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cbab35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227ab3e, 4 },
    { 0xc5ebab3f, 3 },
    { 0x2b57ab40, 4 },
    { 0xd013ab41, 3 },
    { 0x6c07ab40, 3 },
    { 0, 0 },
    { 0xb8b7ab44, 3 },
    { 0, 0 },
    { 0xda67ab46, 5 },
    { 0xc6e7ab46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e47ab4c, 3 },
    { 0x102fab4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dbab50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4fab53, 3 },
    { 0x40b3ab54, 3 },
    { 0xfb13ab55, 3 },
    { 0x68cbab53, 3 },
    { 0, 0 },
    { 0x3493ab58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88bab5c, 3 },
    { 0xfe8fab5d, 4 },
    { 0x7a7bab5c, 4 },
    { 0xaaffab5c, 3 },
    { 0xaca3ab5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ffab67, 3 },
    { 0xb7c3ab68, 3 },
    { 0, 0 },
    { 0xbc7fab6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d37ab77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bab7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bab85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6bab8d, 4 },
    { 0xb8dbab8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497ab92, 3 },
    { 0x5e67ab93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7793ab96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccbab99, 3 },
    { 0x8153ab9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf7aba2, 3 },
    { 0xa27aba2, 3 },
    { 0x71dbaba4, 3 },
    { 0xd17baba4, 4 },
    { 0x9dc7aba6, 4 },
    { 0x3effaba4, 4 },
    { 0x8a67aba8, 3 },
    { 0xece7aba9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633abad, 4 },
    { 0x9cffabae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6fabb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7347abb4, 4 },
    { 0x5aa3abb5, 3 },
    { 0, 0 },
    { 0xf0cfabb7, 3 },
    { 0xed7babb8, 4 },
    { 0, 0 },
    { 0x427babba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d27abc2, 3 },
    { 0x383fabc2, 3 },
    { 0, 0 },
    { 0x64dbabc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313babcf, 4 },
    { 0x30bfabd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b13abd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07babd9, 3 },
    { 0, 0 },
    { 0x33d7abdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdafabde, 3 },
    { 0xd517abdf, 3 },
    { 0x1753abde, 3 },
    { 0x35ffabe0, 3 },
    { 0, 0 },
    { 0xf1dfabe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fabe7, 3 },
    { 0xa847abe7, 3 },
    { 0xf67abe8, 3 },
    { 0x664babea, 4 },
    { 0x340fabeb, 3 },
    { 0x51fabe7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43abf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd927abfe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503ac01, 4 },
    { 0x353fac02, 3 },
    { 0xe23bac01, 3 },
    { 0x48afac04, 4 },
    { 0x4123ac04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950bac0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fbac0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76fac15, 4 },
    { 0, 0 },
    { 0x7207ac17, 3 },
    { 0xe76fac17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc3ac1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370fac22, 4 },
    { 0x89fbac22, 3 },
    { 0, 0 },
    { 0x723fac25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bac28, 4 },
    { 0, 0 },
    { 0xae7fac2a, 3 },
    { 0x78bfac2b, 4 },
    { 0x9917ac2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394bac33, 3 },
    { 0xbe7bac34, 4 },
    { 0x402bac33, 4 },
    { 0x6217ac34, 3 },
    { 0xb6f3ac37, 3 },
    { 0x7e8bac38, 3 },
    { 0x5953ac38, 4 },
    { 0xb07bac37, 4 },
    { 0x114fac37, 4 },
    { 0x91f3ac37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb7ac3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87ac42, 5 },
    { 0xb65fac43, 4 },
    { 0xd707ac43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee3ac47, 3 },
    { 0xd5fbac48, 3 },
    { 0xe90bac48, 3 },
    { 0, 0 },
    { 0x9227ac4b, 4 },
    { 0x37cfac4b, 3 },
    { 0x96d7ac4d, 4 },
    { 0x811fac4e, 3 },
    { 0xdb43ac4e, 3 },
    { 0x91e3ac50, 3 },
    { 0xaecfac51, 4 },
    { 0xbdc3ac52, 3 },
    { 0xde3bac51, 3 },
    { 0x3283ac50, 4 },
    { 0x267bac4b, 3 },
    { 0x3283ac4c, 3 },
    { 0x85c7ac50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305fac5c, 3 },
    { 0x3ed7ac5c, 3 },
    { 0x135fac5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db7ac65, 3 },
    { 0x1a0fac66, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22fac69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bac6e, 3 },
    { 0, 0 },
    { 0xf7ffac70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9163ac74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332fac79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3627ac7d, 4 },
    { 0, 0 },
    { 0x89f3ac7f, 3 },
    { 0x1a27ac80, 3 },
    { 0x128fac7f, 3 },
    { 0x48dbac82, 4 },
    { 0x467ac82, 3 },
    { 0xa9cbac84, 3 },
    { 0x8057ac84, 4 },
    { 0x8aebac86, 3 },
    { 0, 0 },
    { 0x829bac88, 3 },
    { 0xbaabac89, 3 },
    { 0, 0 },
    { 0xafebac8b, 4 },
    { 0xea1fac8b, 5 },
    { 0x52c7ac8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb637ac92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23ac95, 4 },
    { 0, 0 },
    { 0xe5d3ac97, 3 },
    { 0, 0 },
    { 0xddbfac99, 4 },
    { 0x413fac9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7123aca0, 3 },
    { 0, 0 },
    { 0xeb8faca2, 3 },
    { 0x9a93aca3, 4 },
    { 0xa5ffaca4, 4 },
    { 0xc8baca4, 3 },
    { 0x390baca5, 5 },
    { 0x63e7aca7, 3 },
    { 0xca77aca4, 3 },
    { 0, 0 },
    { 0xc1ebacaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dbacaf, 3 },
    { 0x9b0facb0, 4 },
    { 0, 0 },
    { 0x7b9facb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x872bacb5, 4 },
    { 0xa2e3acb6, 4 },
    { 0x569facb7, 4 },
    { 0xc92facb5, 4 },
    { 0x9c1bacb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437acbc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dfacc1, 3 },
    { 0x8c53acc2, 4 },
    { 0x38afacc3, 3 },
    { 0x39b7acc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47acc8, 4 },
    { 0x83bacc9, 4 },
    { 0xaf43acca, 3 },
    { 0xb033acc8, 3 },
    { 0x5ac7accc, 4 },
    { 0, 0 },
    { 0x7087acce, 5 },
    { 0xb26bacce, 3 },
    { 0, 0 },
    { 0xbaffacd1, 3 },
    { 0x3887acd1, 3 },
    { 0xcf13acd1, 4 },
    { 0xf78bacd4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbffacd7, 3 },
    { 0x859facd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1facdd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44bace1, 3 },
    { 0, 0 },
    { 0x50c3ace3, 3 },
    { 0, 0 },
    { 0xcb93ace5, 3 },
    { 0xf0a7ace5, 4 },
    { 0xbc0bace7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b3acee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe193acf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7acf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6bacff, 4 },
    { 0x9b17ad00, 3 },
    { 0, 0 },
    { 0x28b7ad02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb807ad07, 3 },
    { 0x4dd7ad08, 3 },
    { 0x1bd3ad09, 3 },
    { 0, 0 },
    { 0x2a0fad0b, 4 },
    { 0x133ad0c, 4 },
    { 0, 0 },
    { 0xdb53ad0e, 4 },
    { 0xb8e7ad0f, 3 },
    { 0, 0 },
    { 0x888bad11, 3 },
    { 0x3917ad12, 3 },
    { 0, 0 },
    { 0xec8bad14, 3 },
    { 0x477bad14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1637ad18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x742bad1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533bad1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975bad26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9bad2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3ad2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ffad33, 4 },
    { 0x29bad34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fad38, 3 },
    { 0xb007ad39, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x462fad3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482fad47, 4 },
    { 0, 0 },
    { 0xd0bfad49, 3 },
    { 0x763ad49, 3 },
    { 0, 0 },
    { 0x336bad4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdbad4f, 3 },
    { 0x918bad4f, 4 },
    { 0xec2fad4f, 5 },
    { 0x6247ad51, 3 },
    { 0xe3bbad51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168bad58, 3 },
    { 0x31f7ad59, 3 },
    { 0x630fad5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ffad62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x638fad65, 4 },
    { 0xa39bad66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb3ad6f, 4 },
    { 0x18d3ad6f, 3 },
    { 0x567fad71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0fad74, 3 },
    { 0x130fad75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5fad7a, 3 },
    { 0xbbbbad7a, 3 },
    { 0x53ebad7c, 4 },
    { 0x5e63ad7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861fad81, 3 },
    { 0x8b67ad82, 3 },
    { 0xa2ffad83, 3 },
    { 0xcd27ad84, 3 },
    { 0xe7d7ad85, 4 },
    { 0x9d97ad84, 3 },
    { 0x17cfad87, 3 },
    { 0x75c3ad88, 3 },
    { 0x9293ad89, 3 },
    { 0xce1bad8a, 3 },
    { 0x6837ad85, 3 },
    { 0xb333ad89, 3 },
    { 0x7453ad8d, 3 },
    { 0x3607ad8e, 3 },
    { 0x4677ad8e, 3 },
    { 0x7193ad90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798bad94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a7ad99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b87ad9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c57adac, 4 },
    { 0x3ca3adad, 3 },
    { 0x5c7fadae, 3 },
    { 0, 0 },
    { 0x4e23adb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fadb4, 3 },
    { 0x9063adb5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75badb8, 3 },
    { 0, 0 },
    { 0xc1bfadba, 3 },
    { 0x530badbb, 3 },
    { 0, 0 },
    { 0xb2dbadbd, 4 },
    { 0x46cfadbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd77adc4, 3 },
    { 0xb8afadc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e63adcc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbadd0, 3 },
    { 0xbf1fadd1, 3 },
    { 0xab27add2, 4 },
    { 0x97c3add3, 4 },
    { 0xa283add4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9673add7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x918fadda, 3 },
    { 0x8207adda, 4 },
    { 0x941baddb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cbade1, 3 },
    { 0xa72fade2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cafadea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77adef, 3 },
    { 0, 0 },
    { 0x3933adf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x697badf4, 3 },
    { 0, 0 },
    { 0x1697adf6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6fadff, 3 },
    { 0x6727ae00, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588fae06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ebae09, 5 },
    { 0xa84bae0a, 3 },
    { 0xebc7ae0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fae11, 3 },
    { 0xfab7ae12, 3 },
    { 0xc633ae12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2313ae17, 3 },
    { 0xab53ae18, 3 },
    { 0xe967ae18, 3 },
    { 0x6f6bae1a, 3 },
    { 0xf57fae19, 3 },
    { 0, 0 },
    { 0x9b7bae1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e97ae20, 4 },
    { 0x896fae21, 3 },
    { 0xb267ae21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160bae27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5bae2a, 3 },
    { 0xf2a7ae2b, 3 },
    { 0, 0 },
    { 0x5dcfae2d, 4 },
    { 0xd6bae2e, 3 },
    { 0x45f3ae2e, 4 },
    { 0xa23bae2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecbae33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34bae39, 3 },
    { 0x2acbae3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef63ae3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75efae48, 3 },
    { 0x5767ae48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fbae4d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7597ae50, 3 },
    { 0xa633ae51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf137ae59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a3ae5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6343ae62, 3 },
    { 0x985bae63, 3 },
    { 0xd7afae64, 3 },
    { 0x6023ae64, 3 },
    { 0xc483ae66, 3 },
    { 0, 0 },
    { 0xc91fae68, 3 },
    { 0, 0 },
    { 0x9a37ae6a, 3 },
    { 0x206bae6b, 3 },
    { 0x573fae6c, 3 },
    { 0, 0 },
    { 0x54e3ae6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x81afae71, 3 },
    { 0xe0dfae72, 3 },
    { 0x99a3ae73, 3 },
    { 0xff67ae72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c3ae78, 3 },
    { 0xf4b7ae79, 3 },
    { 0x3d6fae79, 3 },
    { 0xf993ae78, 3 },
    { 0x8163ae7c, 3 },
    { 0xf897ae7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c57ae81, 4 },
    { 0x154bae81, 3 },
    { 0x41cbae83, 3 },
    { 0xf487ae82, 3 },
    { 0, 0 },
    { 0x5e03ae86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bfae89, 4 },
    { 0x54cbae89, 4 },
    { 0xd02fae8a, 4 },
    { 0xb033ae89, 3 },
    { 0x9bafae8d, 3 },
    { 0, 0 },
    { 0xc697ae8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe837ae92, 4 },
    { 0x40d3ae93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397ae9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d3aea3, 3 },
    { 0xa1bbaea4, 4 },
    { 0x20faea5, 3 },
    { 0, 0 },
    { 0x42baea7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x30afaeaa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5baeae, 3 },
    { 0xde77aeaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8baeb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7baebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5faec0, 4 },
    { 0, 0 },
    { 0x136baec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e3aec5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd257aeca, 3 },
    { 0, 0 },
    { 0x2ec7aecc, 3 },
    { 0, 0 },
    { 0x566baece, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83aed4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2093aed9, 4 },
    { 0, 0 },
    { 0xb257aedb, 3 },
    { 0xe7dbaedb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x721faedf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92faee3, 4 },
    { 0xc387aee4, 3 },
    { 0x473baee3, 3 },
    { 0x853aee6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ebaee9, 3 },
    { 0x9a07aeea, 4 },
    { 0x83faeeb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edbaef1, 3 },
    { 0x1dcbaef1, 3 },
    { 0x4cebaef2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e87aef7, 4 },
    { 0, 0 },
    { 0xb81baef9, 4 },
    { 0x7e3baefa, 3 },
    { 0x8eabaefb, 4 },
    { 0xe1c3aefc, 3 },
    { 0x8b1faefb, 3 },
    { 0x8033aefe, 3 },
    { 0xfcbaeff, 3 },
    { 0, 0 },
    { 0xce93af01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3977af04, 4 },
    { 0, 0 },
    { 0xc29faf06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58faf0b, 3 },
    { 0x52a3af0c, 3 },
    { 0xd297af0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ebaf11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9143af16, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0faf1d, 3 },
    { 0x687faf1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9faf22, 3 },
    { 0, 0 },
    { 0xa28faf24, 3 },
    { 0x4ac3af25, 3 },
    { 0, 0 },
    { 0x893baf27, 5 },
    { 0x696faf28, 3 },
    { 0, 0 },
    { 0xcb93af2a, 3 },
    { 0, 0 },
    { 0x3b53af2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc27af31, 3 },
    { 0x877af32, 3 },
    { 0x11abaf32, 3 },
    { 0, 0 },
    { 0x9f2faf35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2baf3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b03af3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7faf48, 3 },
    { 0x8d3baf49, 5 },
    { 0, 0 },
    { 0xa10baf4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca7af4e, 3 },
    { 0, 0 },
    { 0x4957af50, 3 },
    { 0, 0 },
    { 0x1623af52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97faf56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ffaf5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd7af5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7af65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbaf69, 5 },
    { 0x519baf6a, 3 },
    { 0x7c57af6a, 4 },
    { 0x7473af6c, 3 },
    { 0x7c4baf6d, 3 },
    { 0x6527af6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f7af72, 3 },
    { 0, 0 },
    { 0x6ea7af74, 3 },
    { 0xd557af75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3af7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbaf82, 3 },
    { 0x3783af82, 4 },
    { 0, 0 },
    { 0xb99baf85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef7af8a, 4 },
    { 0xe81baf8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1693af93, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1727af99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x13afaf9c, 3 },
    { 0x3b0baf9d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75bafad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95bafb1, 3 },
    { 0x5c87afb2, 4 },
    { 0, 0 },
    { 0x20e3afb4, 3 },
    { 0, 0 },
    { 0x64cbafb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6433afc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7317afd0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dbafd7, 3 },
    { 0x52bbafd8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa317afdd, 3 },
    { 0x7013afde, 3 },
    { 0x8b5bafdd, 4 },
    { 0x38fafde, 3 },
    { 0x71efafde, 4 },
    { 0xacabafe0, 3 },
    { 0xbaabafe2, 3 },
    { 0x8ad7afe4, 3 },
    { 0x1bafe2, 3 },
    { 0xa30fafe6, 3 },
    { 0x952fafdd, 3 },
    { 0xd98fafe8, 3 },
    { 0, 0 },
    { 0x40cbafea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3afed, 3 },
    { 0xf99fafed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffaff2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560faffa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9687affd, 3 },
    { 0xb003affd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb7b004, 3 },
    { 0, 0 },
    { 0x14a3b006, 4 },
    { 0x6f5bb006, 4 },
    { 0x5dcbb006, 3 },
    { 0xc057b009, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b00c, 3 },
    { 0xa117b00d, 3 },
    { 0x3f17b00e, 3 },
    { 0, 0 },
    { 0xd293b010, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3673b013, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e7b017, 3 },
    { 0, 0 },
    { 0x15a3b019, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1017b01c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xff83b01f, 3 },
    { 0x69dbb020, 3 },
    { 0xdd97b021, 3 },
    { 0, 0 },
    { 0x1663b023, 3 },
    { 0x9a83b024, 3 },
    { 0xa35fb023, 3 },
    { 0x4d2fb026, 3 },
    { 0xb34bb027, 4 },
    { 0x9a8bb027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7b02e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dbb031, 4 },
    { 0x42d3b032, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eebb039, 3 },
    { 0, 0 },
    { 0x87efb03b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a7b043, 4 },
    { 0x2777b044, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc633b049, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6fb04c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673b04f, 3 },
    { 0x63bb050, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97b053, 4 },
    { 0x63a3b054, 4 },
    { 0, 0 },
    { 0xf43bb056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833bb05b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9443b05e, 3 },
    { 0x7c03b05f, 3 },
    { 0xd693b060, 3 },
    { 0, 0 },
    { 0xc353b062, 3 },
    { 0, 0 },
    { 0xcee7b064, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fbb067, 3 },
    { 0x6cfbb067, 4 },
    { 0, 0 },
    { 0xa35fb06a, 4 },
    { 0xe86bb06b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc093b073, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8087b077, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696fb07b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e7b07e, 3 },
    { 0x67fb07e, 4 },
    { 0, 0 },
    { 0x8edbb081, 3 },
    { 0x8a8bb082, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7693b085, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5697b08d, 4 },
    { 0, 0 },
    { 0xf913b08f, 3 },
    { 0xf8b7b090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533bb096, 3 },
    { 0x72a3b097, 3 },
    { 0x470bb098, 4 },
    { 0x9aabb099, 3 },
    { 0xeef3b09a, 3 },
    { 0x1013b09b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b7b09e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f7b0a2, 5 },
    { 0x4f77b0a3, 3 },
    { 0, 0 },
    { 0x1a3b0a5, 3 },
    { 0xb853b0a6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f87b0ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8bb0b6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9693b0be, 4 },
    { 0, 0 },
    { 0x9adfb0c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3b0c7, 3 },
    { 0, 0 },
    { 0x46e7b0c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afb0cc, 3 },
    { 0xd33fb0cd, 3 },
    { 0x768fb0cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82efb0d5, 4 },
    { 0, 0 },
    { 0x36dbb0d7, 3 },
    { 0x1897b0d8, 4 },
    { 0, 0 },
    { 0x4d97b0da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891fb0df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f33b0e3, 3 },
    { 0xc56fb0e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87bb0e9, 3 },
    { 0, 0 },
    { 0x8b0fb0eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970fb0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fbb0fa, 3 },
    { 0, 0 },
    { 0x3463b0fc, 4 },
    { 0x6dffb0fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cbb101, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2bb104, 3 },
    { 0, 0 },
    { 0x980bb106, 4 },
    { 0xf3bbb106, 3 },
    { 0xc3abb108, 4 },
    { 0x56d3b109, 3 },
    { 0x540fb108, 4 },
    { 0x447fb10b, 3 },
    { 0x357fb10c, 3 },
    { 0x144bb10b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827bb114, 3 },
    { 0xf85bb115, 3 },
    { 0x62c3b116, 4 },
    { 0xee5fb116, 4 },
    { 0x867fb118, 3 },
    { 0, 0 },
    { 0x53e3b11a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e97b11d, 3 },
    { 0x3ba3b11e, 3 },
    { 0xc1fb11d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe853b123, 3 },
    { 0x3953b124, 4 },
    { 0x292bb125, 3 },
    { 0x72bbb126, 3 },
    { 0x82fb126, 4 },
    { 0, 0 },
    { 0x168fb129, 3 },
    { 0x3017b129, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877b12f, 5 },
    { 0x8f13b12f, 3 },
    { 0x17b131, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b93b135, 3 },
    { 0, 0 },
    { 0x273fb137, 3 },
    { 0x937b138, 4 },
    { 0xdc3b139, 5 },
    { 0x5f17b139, 4 },
    { 0, 0 },
    { 0xd263b13c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac7b141, 4 },
    { 0xecdbb142, 4 },
    { 0x6b83b142, 4 },
    { 0x1f8fb143, 3 },
    { 0x3e47b145, 3 },
    { 0x8a0bb146, 3 },
    { 0x8443b147, 3 },
    { 0x277b146, 3 },
    { 0x3dc3b141, 4 },
    { 0xea4fb147, 3 },
    { 0x1befb146, 3 },
    { 0xb74fb14c, 3 },
    { 0x8bbbb14c, 3 },
    { 0xb963b14e, 4 },
    { 0x55dfb14e, 3 },
    { 0x33bb14d, 3 },
    { 0x767b148, 3 },
    { 0x1e57b143, 3 },
    { 0, 0 },
    { 0x7eafb154, 3 },
    { 0x7fc7b155, 3 },
    { 0x2087b154, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc7b159, 3 },
    { 0, 0 },
    { 0x64efb15b, 3 },
    { 0xfb37b15b, 4 },
    { 0x8087b15b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71fb164, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931bb169, 3 },
    { 0, 0 },
    { 0x7523b16b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa73b16e, 4 },
    { 0, 0 },
    { 0xd9e7b170, 3 },
    { 0xff7bb171, 3 },
    { 0, 0 },
    { 0x6cc3b173, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237b178, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dfb17d, 3 },
    { 0, 0 },
    { 0x7103b17f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddbb182, 3 },
    { 0, 0 },
    { 0x4a97b184, 4 },
    { 0x8243b185, 4 },
    { 0x38ebb185, 3 },
    { 0xc44bb186, 4 },
    { 0x6f4fb187, 3 },
    { 0xb7bb184, 4 },
    { 0x9a2fb18a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5587b191, 3 },
    { 0x1f6bb192, 4 },
    { 0, 0 },
    { 0x98bb194, 4 },
    { 0, 0 },
    { 0x6e93b196, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x827bb199, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa427b1a2, 3 },
    { 0x74a3b1a2, 4 },
    { 0, 0 },
    { 0x47bbb1a5, 3 },
    { 0xd8b7b1a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05bb1a9, 3 },
    { 0x18dbb1a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf353b1b4, 3 },
    { 0xb7fb1b4, 3 },
    { 0, 0 },
    { 0x544fb1b7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43b1ba, 4 },
    { 0x154fb1bb, 5 },
    { 0x47b1bc, 3 },
    { 0x7177b1bd, 3 },
    { 0xbe7bb1be, 3 },
    { 0x5ebfb1be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fb1c2, 4 },
    { 0x4643b1c3, 3 },
    { 0, 0 },
    { 0x277b1c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24fb1c8, 3 },
    { 0x205bb1c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbb1d1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5fb1d5, 3 },
    { 0x3a3b1d5, 3 },
    { 0x2e0bb1d7, 3 },
    { 0xd0dfb1d6, 3 },
    { 0x8ab7b1d9, 5 },
    { 0x6adbb1da, 4 },
    { 0x10fb1db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3b1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26fb1e3, 3 },
    { 0x5c9fb1e3, 3 },
    { 0xe483b1e3, 4 },
    { 0, 0 },
    { 0x9693b1e7, 3 },
    { 0x3cefb1e8, 3 },
    { 0x81fbb1e9, 3 },
    { 0x8127b1e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84fb1ed, 4 },
    { 0, 0 },
    { 0xd02fb1ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d73b1f2, 4 },
    { 0x5eb3b1f3, 4 },
    { 0xaa8fb1f4, 3 },
    { 0, 0 },
    { 0x4977b1f6, 3 },
    { 0xac4fb1f7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dbb1fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9bb1fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588fb202, 3 },
    { 0xbbb202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55bb206, 3 },
    { 0, 0 },
    { 0x6d03b208, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x476bb20b, 4 },
    { 0xd53fb20c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8113b214, 4 },
    { 0xf93bb215, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924bb21a, 4 },
    { 0, 0 },
    { 0xd403b21c, 3 },
    { 0x3ce3b21c, 3 },
    { 0, 0 },
    { 0xeec7b21f, 3 },
    { 0x23cbb21f, 3 },
    { 0x8e87b221, 4 },
    { 0, 0 },
    { 0x2dafb223, 3 },
    { 0x75dbb224, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d83b227, 3 },
    { 0x8e4bb228, 3 },
    { 0, 0 },
    { 0x5d0bb22a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bb230, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c3b233, 4 },
    { 0, 0 },
    { 0xb337b235, 3 },
    { 0x2287b235, 4 },
    { 0xcb97b237, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64efb23a, 3 },
    { 0xfd03b23b, 3 },
    { 0x746bb23a, 4 },
    { 0x4ef7b23d, 3 },
    { 0, 0 },
    { 0xd1dfb23f, 3 },
    { 0x3707b240, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e13b246, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d7b24f, 3 },
    { 0x4e23b250, 4 },
    { 0xbf13b250, 3 },
    { 0x84bfb251, 3 },
    { 0x3eabb253, 3 },
    { 0x33fbb253, 3 },
    { 0xb313b251, 4 },
    { 0x313bb253, 3 },
    { 0x2be3b257, 3 },
    { 0x6113b258, 3 },
    { 0xd803b258, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffb25d, 3 },
    { 0x3bf7b25e, 3 },
    { 0x4067b25f, 3 },
    { 0, 0 },
    { 0xd1a3b261, 3 },
    { 0x9a0fb261, 3 },
    { 0x6263b263, 4 },
    { 0x6fb263, 3 },
    { 0x55dfb262, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359fb269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc87b26c, 3 },
    { 0, 0 },
    { 0x29efb26e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073b271, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc03b274, 3 },
    { 0, 0 },
    { 0x92cbb276, 3 },
    { 0x7eabb277, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5fb27a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa927b27f, 3 },
    { 0x1ec3b280, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23bb284, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9293b287, 3 },
    { 0xb7dfb287, 4 },
    { 0, 0 },
    { 0x24b7b28a, 3 },
    { 0xfd6fb28a, 3 },
    { 0x9333b28b, 4 },
    { 0xfdafb28d, 4 },
    { 0x1973b28b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8997b295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3307b29a, 4 },
    { 0xa72fb29b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6fb29f, 3 },
    { 0, 0 },
    { 0xa37bb2a1, 3 },
    { 0xbbbb2a2, 3 },
    { 0xd1e3b2a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2abb2a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc787b2ab, 4 },
    { 0xb7dbb2ab, 4 },
    { 0, 0 },
    { 0xf09fb2ae, 3 },
    { 0x33efb2af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dfb2b5, 3 },
    { 0xc4afb2b6, 4 },
    { 0x1ee7b2b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b87b2ba, 3 },
    { 0xe1b3b2ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe217b2bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4fb2c3, 4 },
    { 0xf8c7b2c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee7b2c7, 3 },
    { 0xbe87b2c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbb2cd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10efb2d2, 4 },
    { 0, 0 },
    { 0xcea7b2d4, 3 },
    { 0, 0 },
    { 0x3c0fb2d6, 3 },
    { 0x8de3b2d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe7b2db, 4 },
    { 0x7507b2dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499fb2e1, 4 },
    { 0x2973b2e2, 3 },
    { 0, 0 },
    { 0x5e43b2e4, 3 },
    { 0x7f5bb2e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c3b2e8, 3 },
    { 0xbe07b2e9, 3 },
    { 0x4f93b2ea, 4 },
    { 0, 0 },
    { 0x5cc3b2ec, 3 },
    { 0xd983b2ed, 3 },
    { 0x6563b2ed, 3 },
    { 0x3e17b2ee, 3 },
    { 0, 0 },
    { 0x4aa7b2f1, 3 },
    { 0x7947b2f2, 4 },
    { 0, 0 },
    { 0x68efb2f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1fb2f8, 3 },
    { 0, 0 },
    { 0x85cbb2fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1bb2ff, 4 },
    { 0xbfb2ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x647bb303, 3 },
    { 0x9e67b304, 3 },
    { 0xf943b305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53b308, 3 },
    { 0, 0 },
    { 0x78d3b30a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bb30d, 3 },
    { 0x29dfb30e, 4 },
    { 0x2393b30d, 3 },
    { 0xbe97b310, 3 },
    { 0x9467b310, 3 },
    { 0x241bb310, 4 },
    { 0xbf33b313, 4 },
    { 0x7d23b314, 3 },
    { 0x2b67b313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c7b318, 4 },
    { 0, 0 },
    { 0xcf0bb31a, 3 },
    { 0xe7efb31a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x212bb31e, 3 },
    { 0xab6fb31f, 4 },
    { 0x89cbb320, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2fb323, 3 },
    { 0x5ce7b323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737b330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6573b337, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba7b33f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3357b346, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecfb349, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995bb34d, 3 },
    { 0xd63fb34e, 3 },
    { 0x8003b34f, 3 },
    { 0x6423b34e, 3 },
    { 0xe65fb34e, 4 },
    { 0x1c4fb34e, 4 },
    { 0xd187b353, 4 },
    { 0xd2a7b351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726fb35b, 4 },
    { 0x1eebb35c, 3 },
    { 0, 0 },
    { 0x1613b35e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d3b362, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0bb367, 3 },
    { 0xd30bb367, 3 },
    { 0x6eebb369, 4 },
    { 0x9d7fb36a, 3 },
    { 0x97ebb367, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dbb36f, 3 },
    { 0x57fb36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7b374, 3 },
    { 0, 0 },
    { 0x61dfb376, 4 },
    { 0x538fb377, 3 },
    { 0x34f7b377, 3 },
    { 0x38c7b376, 4 },
    { 0x5e87b37a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e7b37d, 3 },
    { 0xc48bb37d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182fb383, 3 },
    { 0x855bb383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3bb388, 3 },
    { 0xa797b389, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbb38d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827bb398, 3 },
    { 0, 0 },
    { 0x9fcfb39a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8193b39f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3b3a4, 3 },
    { 0x69dbb3a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5833b3a8, 3 },
    { 0x2877b3a9, 3 },
    { 0, 0 },
    { 0xd91fb3ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9027b3b0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2bb3b5, 3 },
    { 0x413b3b6, 3 },
    { 0, 0 },
    { 0x16b7b3b8, 3 },
    { 0xdc77b3b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f13b3bf, 3 },
    { 0, 0 },
    { 0x853fb3c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472fb3c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9fb3ca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c7b3d1, 3 },
    { 0x6e3b3d2, 3 },
    { 0, 0 },
    { 0x6827b3d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7b3d7, 3 },
    { 0, 0 },
    { 0x9af7b3d9, 3 },
    { 0x9fb3da, 3 },
    { 0, 0 },
    { 0xac0fb3dc, 3 },
    { 0x10c3b3dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb3b3e1, 3 },
    { 0, 0 },
    { 0x2367b3e3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b03b3e6, 3 },
    { 0, 0 },
    { 0xb347b3e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1efb3eb, 3 },
    { 0x486fb3ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4bb3f2, 4 },
    { 0, 0 },
    { 0x5d0fb3f4, 3 },
    { 0xaf7fb3f5, 3 },
    { 0x8dcbb3f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273b3f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f3b400, 3 },
    { 0x72b7b401, 3 },
    { 0, 0 },
    { 0xc2e3b403, 3 },
    { 0, 0 },
    { 0x9563b405, 3 },
    { 0x3a2fb406, 4 },
    { 0xd8bb407, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d3b40d, 5 },
    { 0, 0 },
    { 0x3307b40f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d7b412, 3 },
    { 0, 0 },
    { 0x534bb414, 3 },
    { 0xf1f7b415, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e53b41d, 3 },
    { 0x421fb41e, 3 },
    { 0xe743b41d, 3 },
    { 0x9bffb420, 4 },
    { 0x537b420, 4 },
    { 0x9303b422, 4 },
    { 0x7793b423, 4 },
    { 0x247b422, 3 },
    { 0x5ffbb425, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47bb42c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e3b430, 3 },
    { 0xc843b430, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edfb434, 3 },
    { 0xedc3b435, 4 },
    { 0, 0 },
    { 0x2c17b437, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbb43d, 3 },
    { 0x74bb43e, 3 },
    { 0x287b43f, 3 },
    { 0xe93b440, 3 },
    { 0xef23b441, 5 },
    { 0, 0 },
    { 0x368fb443, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b23b452, 3 },
    { 0xf877b453, 3 },
    { 0xbd17b454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf157b458, 3 },
    { 0, 0 },
    { 0xaacfb45a, 3 },
    { 0x2b63b45b, 3 },
    { 0x28abb45c, 3 },
    { 0x380fb45b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c17b461, 3 },
    { 0x7883b462, 3 },
    { 0x72bbb463, 3 },
    { 0xc49fb461, 4 },
    { 0x54dfb465, 3 },
    { 0xed6bb465, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x931bb469, 3 },
    { 0x56b3b46a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd7b46d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe893b470, 4 },
    { 0x6697b471, 4 },
    { 0, 0 },
    { 0xcb73b473, 3 },
    { 0, 0 },
    { 0x38e3b475, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48bb478, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007b47b, 4 },
    { 0xc257b47c, 3 },
    { 0x9107b47c, 3 },
    { 0, 0 },
    { 0x4efb47f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e57b482, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7b486, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2297b48a, 3 },
    { 0xb32fb48b, 4 },
    { 0x6f63b48c, 4 },
    { 0xc563b48d, 4 },
    { 0xebdfb48e, 4 },
    { 0x312fb48d, 3 },
    { 0x612bb48f, 3 },
    { 0xbf3bb48b, 3 },
    { 0xf083b48d, 3 },
    { 0x4847b48c, 4 },
    { 0xc57bb494, 3 },
    { 0, 0 },
    { 0x992bb496, 4 },
    { 0x6c67b497, 4 },
    { 0x1367b497, 3 },
    { 0xab0bb496, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40afb49c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b4a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc67b4aa, 3 },
    { 0x3547b4ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d3b4b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcfb4b8, 4 },
    { 0xafe3b4b9, 3 },
    { 0x6f1bb4ba, 3 },
    { 0x82bfb4bb, 3 },
    { 0, 0 },
    { 0xe9fbb4bd, 3 },
    { 0xcefbb4bd, 3 },
    { 0x2dbbb4bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea3b4c2, 3 },
    { 0x8383b4c3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d93b4c6, 4 },
    { 0x753bb4c7, 3 },
    { 0x36dbb4c8, 3 },
    { 0x164bb4c9, 3 },
    { 0xb2bfb4ca, 3 },
    { 0xcb23b4cb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d3b4ce, 3 },
    { 0x151bb4cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab7b4de, 5 },
    { 0, 0 },
    { 0xee53b4e0, 3 },
    { 0x554bb4e1, 3 },
    { 0x314fb4e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cfb4e6, 3 },
    { 0x5b7b4e7, 3 },
    { 0x3f3b4e6, 3 },
    { 0xc8b7b4e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc73b4f1, 3 },
    { 0xd103b4f1, 3 },
    { 0xbfffb4f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19bb4f6, 3 },
    { 0, 0 },
    { 0x7a03b4f8, 4 },
    { 0, 0 },
    { 0xa167b4fa, 3 },
    { 0x7127b4fb, 4 },
    { 0x580bb4fc, 4 },
    { 0xaf57b4fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dfb501, 4 },
    { 0, 0 },
    { 0x8a53b503, 4 },
    { 0, 0 },
    { 0xb7b3b505, 3 },
    { 0, 0 },
    { 0x89cbb507, 4 },
    { 0xf5ffb508, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75bb50d, 5 },
    { 0x1d47b50e, 4 },
    { 0, 0 },
    { 0x4df7b510, 4 },
    { 0x3bffb510, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3b514, 3 },
    { 0, 0 },
    { 0x4a93b516, 3 },
    { 0x6803b516, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827b51b, 3 },
    { 0xd79fb51c, 3 },
    { 0x4263b51d, 3 },
    { 0xa79bb51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe343b522, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0fb526, 3 },
    { 0x17abb526, 3 },
    { 0x706bb527, 3 },
    { 0xbc2fb527, 4 },
    { 0x5517b52a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cfb530, 3 },
    { 0x5ecfb531, 3 },
    { 0x2ed7b530, 3 },
    { 0, 0 },
    { 0x956bb534, 3 },
    { 0xb9d7b535, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bafb542, 3 },
    { 0, 0 },
    { 0x9a93b544, 4 },
    { 0, 0 },
    { 0x1b5fb546, 3 },
    { 0xb2f3b547, 3 },
    { 0x3efb548, 3 },
    { 0xc3bfb549, 4 },
    { 0xf8bb54a, 3 },
    { 0x3853b54a, 3 },
    { 0xeb33b546, 3 },
    { 0xc60bb54d, 4 },
    { 0x29a3b54e, 3 },
    { 0x9353b54f, 3 },
    { 0x84ebb54f, 3 },
    { 0xf563b548, 3 },
    { 0x7c4bb552, 4 },
    { 0xdc07b552, 4 },
    { 0x926bb54d, 3 },
    { 0xab03b555, 3 },
    { 0x1ea3b556, 3 },
    { 0xed13b549, 5 },
    { 0, 0 },
    { 0x7e13b559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7bb561, 3 },
    { 0xc827b562, 4 },
    { 0, 0 },
    { 0x6ab7b564, 4 },
    { 0, 0 },
    { 0x536bb566, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8003b56a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefbb56d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11fb571, 4 },
    { 0x82cbb572, 3 },
    { 0x40abb573, 3 },
    { 0x412bb573, 3 },
    { 0x50afb575, 3 },
    { 0x3993b573, 3 },
    { 0x9677b577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ebb57b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29fb57e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49bb584, 3 },
    { 0x8563b584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88efb58c, 4 },
    { 0xd7cfb58c, 4 },
    { 0xf343b58e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ebb591, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267b59b, 4 },
    { 0x6c27b59b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5723b5a0, 3 },
    { 0x84bbb5a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7b5a5, 3 },
    { 0x847b5a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acfb5a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01bb5ac, 3 },
    { 0, 0 },
    { 0x7a8fb5ae, 4 },
    { 0x1a1bb5ae, 4 },
    { 0xe453b5af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1fb5b5, 3 },
    { 0xe557b5b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dbb5ba, 3 },
    { 0x5acbb5bb, 4 },
    { 0x72ffb5bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x44dbb5bf, 3 },
    { 0x2b4bb5c0, 3 },
    { 0xf3fb5c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac3b5cb, 3 },
    { 0xeb27b5cc, 3 },
    { 0x5cffb5cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfb5d1, 3 },
    { 0xef9fb5d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55efb5d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3b5dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce3b5e7, 3 },
    { 0xfe87b5e8, 3 },
    { 0, 0 },
    { 0xe387b5ea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2afb5ee, 3 },
    { 0xbb37b5ee, 4 },
    { 0x87d3b5ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f7b5f6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8bb5fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4bb602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9fb608, 3 },
    { 0xbad3b609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e3b60d, 4 },
    { 0x77d3b60e, 3 },
    { 0x8203b60e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03bb613, 4 },
    { 0x8a47b614, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d43b618, 3 },
    { 0xb327b618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5efb61e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82fb625, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf643b628, 3 },
    { 0xd783b629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc3b62e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd3b631, 3 },
    { 0x46abb631, 3 },
    { 0, 0 },
    { 0xdbd3b634, 3 },
    { 0, 0 },
    { 0xbe47b636, 4 },
    { 0x189fb637, 3 },
    { 0, 0 },
    { 0x9ccfb639, 4 },
    { 0x37b7b63a, 3 },
    { 0x6d27b63b, 5 },
    { 0xb4a7b63c, 3 },
    { 0x77fbb63c, 3 },
    { 0x3ff3b63e, 3 },
    { 0x1eafb63c, 3 },
    { 0x2e3b63a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cfb644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c7b648, 4 },
    { 0, 0 },
    { 0xefbbb64a, 3 },
    { 0x854fb64b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7fb650, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc3b65a, 5 },
    { 0x83ffb65a, 3 },
    { 0x7d27b65c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aabb661, 3 },
    { 0x8b0fb661, 4 },
    { 0xae0fb662, 3 },
    { 0x44d3b662, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c7b66a, 3 },
    { 0, 0 },
    { 0xa287b66c, 3 },
    { 0x5e0bb66c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3b672, 3 },
    { 0x473b673, 3 },
    { 0, 0 },
    { 0x288bb675, 3 },
    { 0x4b37b675, 3 },
    { 0, 0 },
    { 0x768bb678, 4 },
    { 0xf1f7b679, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e7b67c, 4 },
    { 0xec03b67d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3927b683, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cbb68b, 4 },
    { 0, 0 },
    { 0x4bfb68d, 3 },
    { 0x347bb68e, 3 },
    { 0, 0 },
    { 0x3b43b690, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x453bb693, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7b698, 3 },
    { 0xcdebb699, 3 },
    { 0x5cf7b69a, 3 },
    { 0, 0 },
    { 0xcd63b69c, 3 },
    { 0xad93b69d, 4 },
    { 0xa4afb69c, 3 },
    { 0, 0 },
    { 0xbc6fb6a0, 3 },
    { 0x6507b6a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6bb6a5, 3 },
    { 0, 0 },
    { 0x78dbb6a7, 4 },
    { 0xd9e3b6a7, 3 },
    { 0x76bbb6a9, 3 },
    { 0xbf3bb6aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7bb6b6, 3 },
    { 0, 0 },
    { 0xa2a3b6b8, 3 },
    { 0xda63b6b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7483b6bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42bb6bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9517b6c5, 3 },
    { 0xde8bb6c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x806bb6c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa7b6d2, 3 },
    { 0x4ad3b6d2, 3 },
    { 0, 0 },
    { 0xe987b6d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf83b6d9, 3 },
    { 0x519fb6da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973b6e2, 3 },
    { 0, 0 },
    { 0xcc77b6e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14abb6e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ebb6ec, 3 },
    { 0xfa6fb6ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edfb6f0, 3 },
    { 0, 0 },
    { 0x31dfb6f2, 3 },
    { 0x62ffb6f3, 3 },
    { 0xb01bb6f2, 3 },
    { 0xe493b6f3, 4 },
    { 0, 0 },
    { 0xe357b6f7, 3 },
    { 0xaeffb6f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36afb6fc, 3 },
    { 0x49cfb6fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c3b709, 4 },
    { 0xe6efb709, 3 },
    { 0x87afb70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebbb711, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec3b716, 3 },
    { 0xb9b3b717, 3 },
    { 0, 0 },
    { 0x6a67b719, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325fb71f, 3 },
    { 0x94cbb720, 3 },
    { 0x9f23b720, 3 },
    { 0xbe13b720, 3 },
    { 0xb86fb723, 3 },
    { 0xbf17b724, 3 },
    { 0x21e3b724, 3 },
    { 0xfdffb726, 3 },
    { 0, 0 },
    { 0x5aafb728, 3 },
    { 0xb8a3b728, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81fb72e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5fb732, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70bb735, 3 },
    { 0xca5bb735, 4 },
    { 0x43a7b737, 4 },
    { 0x8857b735, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2fb73b, 3 },
    { 0x52bfb73c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977b741, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ffb744, 3 },
    { 0x7e97b745, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910fb74b, 3 },
    { 0, 0 },
    { 0x959fb74d, 3 },
    { 0, 0 },
    { 0xb8ffb74f, 4 },
    { 0, 0 },
    { 0x601bb751, 4 },
    { 0xf3fbb752, 3 },
    { 0x4787b753, 4 },
    { 0xe56bb751, 5 },
    { 0x932bb755, 3 },
    { 0x6e43b756, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a87b759, 3 },
    { 0, 0 },
    { 0x29dbb75b, 3 },
    { 0, 0 },
    { 0x8bfbb75d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d57b760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bbb763, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2bb766, 3 },
    { 0xfcd7b766, 5 },
    { 0xa547b768, 4 },
    { 0x8263b769, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde7b76c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1137b775, 3 },
    { 0x5a27b775, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb77e, 4 },
    { 0x74ebb77e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2fb785, 4 },
    { 0xbb9fb786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180fb78b, 4 },
    { 0xa313b78c, 3 },
    { 0xf06bb78d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af7b790, 3 },
    { 0, 0 },
    { 0x7cebb792, 3 },
    { 0, 0 },
    { 0xc607b794, 3 },
    { 0x6567b794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3fb798, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8033b79b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d7b79e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8bb7a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4373b7a9, 3 },
    { 0x39cfb7aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80fb7af, 4 },
    { 0x4f17b7af, 3 },
    { 0xe34fb7b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7bb7b8, 4 },
    { 0xb1ffb7b9, 3 },
    { 0xd04fb7b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377fb7be, 3 },
    { 0, 0 },
    { 0x617fb7c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3b7c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313bb7cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d3b7d0, 3 },
    { 0, 0 },
    { 0x3d3bb7d2, 3 },
    { 0xea4fb7d2, 3 },
    { 0x27fbb7d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753b7d9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0bb7dd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d7b7e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7b7e9, 3 },
    { 0x3bdbb7ea, 3 },
    { 0x8073b7ea, 4 },
    { 0x6fdfb7ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x193b7ef, 3 },
    { 0xe4bbb7f0, 3 },
    { 0xb353b7f1, 3 },
    { 0x413fb7f1, 3 },
    { 0, 0 },
    { 0x8b5bb7f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999fb7f8, 3 },
    { 0, 0 },
    { 0xb4c7b7fa, 3 },
    { 0, 0 },
    { 0x86d3b7fc, 3 },
    { 0, 0 },
    { 0x3727b7fe, 3 },
    { 0x1a7bb7fe, 4 },
    { 0, 0 },
    { 0x18dfb801, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3363b80e, 4 },
    { 0x7c3b80f, 3 },
    { 0x91fbb810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bbb813, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820fb817, 3 },
    { 0, 0 },
    { 0xa86bb819, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bb81e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x321fb821, 3 },
    { 0x2f97b822, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1687b825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4bb828, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d87b82b, 3 },
    { 0x14e7b82c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf417b831, 4 },
    { 0x99a7b831, 3 },
    { 0xc453b832, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465fb838, 4 },
    { 0xdff3b838, 4 },
    { 0, 0 },
    { 0xd503b83b, 3 },
    { 0xc61fb83c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc3b83f, 3 },
    { 0xcfebb840, 3 },
    { 0xbb7bb841, 3 },
    { 0x69f7b842, 4 },
    { 0xdebb843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe7b846, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179fb850, 3 },
    { 0, 0 },
    { 0x5987b852, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1517b857, 3 },
    { 0xfe9bb857, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41fb85c, 3 },
    { 0x1f3fb85c, 3 },
    { 0xeaa7b85c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ffb866, 3 },
    { 0x1d3b867, 3 },
    { 0x774fb868, 3 },
    { 0xf9bfb869, 4 },
    { 0, 0 },
    { 0x5347b86b, 3 },
    { 0x70a7b86c, 4 },
    { 0x91ffb86d, 3 },
    { 0x41e3b86e, 3 },
    { 0xef8bb86b, 3 },
    { 0x847b86f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e27b874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5fb877, 3 },
    { 0xd3bb877, 3 },
    { 0xa26bb879, 4 },
    { 0, 0 },
    { 0x9a6bb87b, 4 },
    { 0, 0 },
    { 0x8aefb87d, 3 },
    { 0x4907b87e, 3 },
    { 0x4a2fb87f, 3 },
    { 0, 0 },
    { 0x7fcbb881, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc413b884, 4 },
    { 0xebf7b885, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2bb88a, 3 },
    { 0, 0 },
    { 0xc853b88c, 3 },
    { 0, 0 },
    { 0x847b88e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddbb891, 4 },
    { 0x94a3b892, 4 },
    { 0xc61bb893, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739fb89c, 3 },
    { 0x4e27b89d, 5 },
    { 0x94ebb89d, 3 },
    { 0xdeebb89f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83b8a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83fb8a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9bb8ac, 3 },
    { 0x187bb8ad, 3 },
    { 0xef3b8ae, 3 },
    { 0xe19bb8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63bb8b2, 3 },
    { 0, 0 },
    { 0x154fb8b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a7b8b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6067b8bc, 3 },
    { 0x3e7b8bc, 3 },
    { 0, 0 },
    { 0xf2ebb8bf, 3 },
    { 0, 0 },
    { 0x9897b8c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77afb8c5, 3 },
    { 0, 0 },
    { 0xeb47b8c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c47b8ce, 3 },
    { 0x88afb8ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fbb8d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55fb8d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b3b8e0, 4 },
    { 0x387fb8e1, 3 },
    { 0x4203b8e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d97b8e6, 3 },
    { 0x946fb8e7, 3 },
    { 0, 0 },
    { 0x17bfb8e9, 3 },
    { 0x188fb8ea, 4 },
    { 0x2823b8ea, 4 },
    { 0xe713b8eb, 3 },
    { 0xcdfb8eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e3b8f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3b8f4, 4 },
    { 0, 0 },
    { 0x97cbb8f6, 3 },
    { 0, 0 },
    { 0x56d3b8f8, 3 },
    { 0x7cf3b8f9, 4 },
    { 0x264fb8f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed57b903, 3 },
    { 0xf9e3b903, 3 },
    { 0x665fb905, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fb909, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x60abb90c, 5 },
    { 0, 0 },
    { 0xbcfbb90e, 3 },
    { 0xf017b90f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f7b912, 3 },
    { 0, 0 },
    { 0xefc7b914, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x526fb917, 3 },
    { 0, 0 },
    { 0xa073b919, 3 },
    { 0x7547b91a, 3 },
    { 0x5d8fb91b, 3 },
    { 0xff0bb91c, 3 },
    { 0xa23b919, 3 },
    { 0x376fb91a, 3 },
    { 0x862bb91d, 4 },
    { 0xfe1bb91a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbbb926, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7b929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eabb92e, 3 },
    { 0, 0 },
    { 0x9f67b930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc073b933, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47fb938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc7b93d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe643b941, 3 },
    { 0x333bb941, 3 },
    { 0xc8f3b941, 3 },
    { 0x3413b942, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1bb947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f87b94a, 3 },
    { 0xaa13b94b, 5 },
    { 0, 0 },
    { 0xa9f7b94d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3b952, 3 },
    { 0, 0 },
    { 0x9d5bb954, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37fb95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c03b95d, 3 },
    { 0xb597b95e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe57b963, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebbb966, 3 },
    { 0x18fbb967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7bb972, 3 },
    { 0, 0 },
    { 0xfdf7b974, 4 },
    { 0x748bb974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc737b979, 3 },
    { 0, 0 },
    { 0x470bb97b, 4 },
    { 0x5383b97c, 5 },
    { 0xfd7fb97b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8297b980, 3 },
    { 0x8aa3b981, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1bb988, 3 },
    { 0xe5f3b989, 4 },
    { 0xff2fb98a, 3 },
    { 0, 0 },
    { 0x9de3b98c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5073b990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7bb993, 3 },
    { 0, 0 },
    { 0xaab7b995, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2bb998, 3 },
    { 0xfe6fb999, 4 },
    { 0xdd7bb99a, 3 },
    { 0x91b7b998, 5 },
    { 0xc9a7b99c, 4 },
    { 0x72abb99d, 3 },
    { 0x28b7b99d, 3 },
    { 0x4bdfb99f, 3 },
    { 0xaa4bb9a0, 3 },
    { 0x160fb9a1, 3 },
    { 0, 0 },
    { 0xa903b9a3, 3 },
    { 0, 0 },
    { 0xfa87b9a5, 3 },
    { 0, 0 },
    { 0x2ecbb9a7, 3 },
    { 0x3603b9a8, 4 },
    { 0x53fb9a9, 3 },
    { 0xc74bb9aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbb9b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2327b9bc, 4 },
    { 0, 0 },
    { 0x80afb9be, 4 },
    { 0xbcabb9bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedefb9c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbb9ca, 3 },
    { 0, 0 },
    { 0xb55fb9cc, 3 },
    { 0x6427b9cc, 3 },
    { 0, 0 },
    { 0x2333b9cf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd327b9d2, 3 },
    { 0xbfc3b9d3, 3 },
    { 0xb3fb9d4, 4 },
    { 0xdc27b9d5, 5 },
    { 0, 0 },
    { 0x1347b9d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3b9dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3bb9df, 3 },
    { 0xcc67b9df, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccabb9e4, 3 },
    { 0, 0 },
    { 0x669fb9e6, 3 },
    { 0x37b9e7, 4 },
    { 0x8303b9e7, 3 },
    { 0x6f5fb9e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9797b9ec, 3 },
    { 0, 0 },
    { 0x4c23b9ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe237b9f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7863b9fa, 3 },
    { 0, 0 },
    { 0x68c3b9fc, 3 },
    { 0x2f43b9fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7333ba07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33ba0b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fba0e, 3 },
    { 0x56fbba0f, 3 },
    { 0x45a3ba10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453bba15, 3 },
    { 0xe33bba16, 5 },
    { 0x8a0bba16, 3 },
    { 0x2a2bba17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268bba1c, 3 },
    { 0, 0 },
    { 0xa8ebba1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7bba21, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb567ba25, 3 },
    { 0x7e07ba26, 3 },
    { 0x6ed3ba25, 4 },
    { 0x7ffbba28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd837ba2c, 3 },
    { 0x1d67ba2d, 4 },
    { 0x8abfba2e, 3 },
    { 0x5343ba2f, 3 },
    { 0xe72fba2f, 3 },
    { 0, 0 },
    { 0x5567ba32, 3 },
    { 0x3f7bba33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7fba36, 3 },
    { 0, 0 },
    { 0x6c13ba38, 3 },
    { 0, 0 },
    { 0x3e93ba3a, 3 },
    { 0xb70bba3a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bfba3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300fba43, 3 },
    { 0, 0 },
    { 0xd5dfba45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3ba48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2523ba4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70bba4f, 3 },
    { 0x307bba4f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91bba53, 3 },
    { 0x6797ba53, 3 },
    { 0x8a4bba55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1727ba58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947bba60, 4 },
    { 0x600fba60, 3 },
    { 0, 0 },
    { 0xd40fba63, 3 },
    { 0x5493ba64, 3 },
    { 0x7607ba65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d7ba69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa7ba6c, 4 },
    { 0xc4dfba6d, 3 },
    { 0, 0 },
    { 0xe5cfba6f, 3 },
    { 0, 0 },
    { 0x8dd7ba71, 3 },
    { 0xb09fba72, 3 },
    { 0x154fba73, 3 },
    { 0xce83ba74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bba77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4463ba7a, 3 },
    { 0, 0 },
    { 0x1f8bba7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4617ba7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e53ba82, 3 },
    { 0, 0 },
    { 0x6cffba84, 3 },
    { 0x9343ba85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa3ba8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca3ba95, 3 },
    { 0x4d0fba96, 3 },
    { 0, 0 },
    { 0xa46bba98, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8347baa4, 4 },
    { 0xc137baa5, 3 },
    { 0x4fe7baa6, 3 },
    { 0xaeabbaa6, 3 },
    { 0x9417baa8, 3 },
    { 0xf5abbaa9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e7baac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992bbab0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227bab3, 3 },
    { 0x772fbab4, 5 },
    { 0, 0 },
    { 0x693bbab6, 4 },
    { 0, 0 },
    { 0x913bab8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87bac1, 3 },
    { 0, 0 },
    { 0xf05bbac3, 3 },
    { 0, 0 },
    { 0xa55bbac5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbbac8, 3 },
    { 0, 0 },
    { 0xf43baca, 3 },
    { 0x6083baca, 3 },
    { 0x6327bacc, 4 },
    { 0x60bfbacc, 3 },
    { 0x843bace, 3 },
    { 0, 0 },
    { 0x6ff3bad0, 3 },
    { 0x1f7bbad0, 3 },
    { 0xbd93bad2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cbbad8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5fbadc, 3 },
    { 0xf2a3badc, 3 },
    { 0, 0 },
    { 0x6bf3badf, 3 },
    { 0x8db3bae0, 3 },
    { 0, 0 },
    { 0xe36bbae2, 3 },
    { 0x5377bae3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1fbaec, 4 },
    { 0, 0 },
    { 0xd067baee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7803baf1, 3 },
    { 0x39afbaf2, 3 },
    { 0, 0 },
    { 0x957baf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x22efbaf7, 4 },
    { 0x2a8fbaf8, 4 },
    { 0x1f7baf9, 3 },
    { 0xc36fbafa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12bbafd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe7bb01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93bb05, 4 },
    { 0xda4fbb05, 3 },
    { 0x6a7bbb07, 3 },
    { 0, 0 },
    { 0xca8bbb09, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa293bb0c, 3 },
    { 0xf68bbb0d, 3 },
    { 0x2d33bb0e, 3 },
    { 0xd413bb0f, 4 },
    { 0x4b47bb10, 3 },
    { 0x935fbb0e, 4 },
    { 0, 0 },
    { 0xec97bb13, 3 },
    { 0x7a6fbb14, 3 },
    { 0xa843bb15, 3 },
    { 0xf77bb13, 3 },
    { 0x4fefbb17, 3 },
    { 0xf953bb18, 5 },
    { 0xbda3bb18, 3 },
    { 0x140fbb1a, 4 },
    { 0x3a77bb1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773bb1f, 3 },
    { 0xf95bbb20, 4 },
    { 0x753bb21, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e37bb29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437bb2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3007bb36, 3 },
    { 0x1b1fbb37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63bb3e, 3 },
    { 0xb303bb3f, 3 },
    { 0, 0 },
    { 0x81d7bb41, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3bb44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfbb47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74fbb50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5307bb54, 4 },
    { 0x9fbbb55, 5 },
    { 0x1d0bbb55, 3 },
    { 0, 0 },
    { 0x3aafbb58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb537bb60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de3bb64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3bb67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750fbb6d, 3 },
    { 0x4abbb6e, 3 },
    { 0x40e7bb6f, 3 },
    { 0, 0 },
    { 0x63cfbb71, 3 },
    { 0xb283bb71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fffbb75, 3 },
    { 0x9bbbbb76, 4 },
    { 0, 0 },
    { 0x255fbb78, 3 },
    { 0xf8a3bb79, 3 },
    { 0x8e4fbb7a, 4 },
    { 0xce07bb79, 3 },
    { 0, 0 },
    { 0xaf0fbb7d, 3 },
    { 0, 0 },
    { 0xb5a3bb7f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842fbb85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdfbb89, 3 },
    { 0x1677bb8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c7bb8d, 4 },
    { 0x33b7bb8e, 5 },
    { 0x84ebbb8f, 3 },
    { 0x54dfbb8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x139fbb93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdfbb97, 3 },
    { 0x2457bb98, 4 },
    { 0x8313bb98, 5 },
    { 0x9ed7bb98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aefbb9e, 4 },
    { 0x7d8bbb9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f7bba3, 3 },
    { 0x3e7bba4, 3 },
    { 0x9453bba5, 3 },
    { 0x590fbba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0afbbaf, 3 },
    { 0, 0 },
    { 0xc6fbbb1, 4 },
    { 0x204bbbb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47bbbb5, 3 },
    { 0, 0 },
    { 0xee23bbb7, 3 },
    { 0, 0 },
    { 0xd63bbbb9, 3 },
    { 0, 0 },
    { 0xa597bbbb, 3 },
    { 0xc42bbbbb, 3 },
    { 0, 0 },
    { 0xfad3bbbe, 4 },
    { 0x2667bbbf, 3 },
    { 0x46cbbbc0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x953fbbc3, 4 },
    { 0x2edbbbc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40fbbca, 4 },
    { 0, 0 },
    { 0x548bbbcc, 4 },
    { 0x1e83bbcd, 3 },
    { 0xfca7bbcc, 4 },
    { 0, 0 },
    { 0xdbe3bbd0, 4 },
    { 0xf6d7bbd1, 4 },
    { 0x9787bbd2, 4 },
    { 0xea0bbbd3, 3 },
    { 0xeb7fbbd4, 3 },
    { 0xf5afbbd3, 3 },
    { 0xf063bbd6, 4 },
    { 0x29abbbd7, 3 },
    { 0x7cd7bbd8, 3 },
    { 0, 0 },
    { 0xd087bbda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d7bbdd, 3 },
    { 0, 0 },
    { 0x62c7bbdf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de3bbe3, 3 },
    { 0xb03bbe4, 4 },
    { 0x8103bbe4, 4 },
    { 0xc8cfbbe6, 4 },
    { 0, 0 },
    { 0x2abfbbe8, 4 },
    { 0x9cafbbe8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84bbbee, 3 },
    { 0xdfcfbbef, 4 },
    { 0xcdd7bbf0, 3 },
    { 0, 0 },
    { 0xeb07bbf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x822bbbf5, 4 },
    { 0, 0 },
    { 0x6bbbbbf7, 3 },
    { 0x527fbbf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6433bbfc, 3 },
    { 0x4917bbfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75fbc00, 4 },
    { 0, 0 },
    { 0x708fbc02, 4 },
    { 0xedb3bc03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fbbc06, 3 },
    { 0xd35fbc06, 4 },
    { 0xcc07bc07, 3 },
    { 0, 0 },
    { 0xbeafbc0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bbbc0f, 3 },
    { 0, 0 },
    { 0x341bbc11, 3 },
    { 0x58f7bc12, 4 },
    { 0xb7e7bc13, 3 },
    { 0x5a9fbc14, 4 },
    { 0xed33bc11, 4 },
    { 0, 0 },
    { 0x33f3bc17, 3 },
    { 0x364bbc17, 3 },
    { 0x733bc17, 4 },
    { 0, 0 },
    { 0xed8fbc1b, 3 },
    { 0x5f23bc1c, 4 },
    { 0x368fbc1d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27fbc20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3bbc24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417bc28, 3 },
    { 0x356bbc29, 3 },
    { 0, 0 },
    { 0x20ffbc2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dfbc2f, 3 },
    { 0, 0 },
    { 0x3453bc31, 3 },
    { 0x796fbc31, 3 },
    { 0x294bbc32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bc38, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb53bc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23abbc3e, 3 },
    { 0xf1d7bc3f, 3 },
    { 0x553fbc3f, 3 },
    { 0x1f13bc3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05bbc4b, 3 },
    { 0x7013bc4c, 3 },
    { 0xf79fbc4d, 3 },
    { 0xee7bc4c, 4 },
    { 0x8e1bbc4b, 5 },
    { 0xa223bc50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197bc5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x257bc5d, 3 },
    { 0xacd7bc5d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bbbc6c, 3 },
    { 0x1597bc6d, 3 },
    { 0x7c0bbc6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b3bc71, 3 },
    { 0x955fbc72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acbbc76, 4 },
    { 0x347bc77, 3 },
    { 0, 0 },
    { 0x9a07bc79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26abbc7c, 3 },
    { 0, 0 },
    { 0xe2e7bc7e, 3 },
    { 0x6267bc7e, 5 },
    { 0x6007bc80, 3 },
    { 0, 0 },
    { 0x57f3bc82, 3 },
    { 0, 0 },
    { 0xc54bbc84, 4 },
    { 0x373bc85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cbbc8f, 4 },
    { 0x47ebbc90, 3 },
    { 0xb023bc91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84abbc97, 3 },
    { 0x5e6fbc98, 3 },
    { 0xc4bbbc98, 3 },
    { 0x59d3bc9a, 3 },
    { 0, 0 },
    { 0xc2cfbc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6397bc9f, 3 },
    { 0x4583bca0, 3 },
    { 0, 0 },
    { 0x7c63bca2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2fbca5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20bbca8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x135bbcab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbcaf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fbcb5, 3 },
    { 0x6617bcb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2fbcba, 3 },
    { 0xab53bcbb, 4 },
    { 0xb473bcbb, 3 },
    { 0x9f37bcbd, 4 },
    { 0, 0 },
    { 0x9febbcbf, 4 },
    { 0xe74bbcc0, 3 },
    { 0, 0 },
    { 0x55bbcc2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d53bcc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422bbcca, 3 },
    { 0, 0 },
    { 0x11e7bccc, 4 },
    { 0x98efbccd, 4 },
    { 0, 0 },
    { 0x9c9fbccf, 4 },
    { 0, 0 },
    { 0xce6bbcd1, 5 },
    { 0, 0 },
    { 0x37fbcd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x313fbcd6, 5 },
    { 0, 0 },
    { 0x8847bcd8, 3 },
    { 0xe42bbcd8, 4 },
    { 0x6f3fbcd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fbcde, 3 },
    { 0, 0 },
    { 0x8d33bce0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cbbce3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15bbcee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cbbcf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113bbcfd, 3 },
    { 0x944fbcfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67bd08, 3 },
    { 0xa177bd08, 4 },
    { 0xc2e7bd0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de3bd0d, 3 },
    { 0x860bbd0d, 4 },
    { 0xfa73bd0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02fbd15, 3 },
    { 0x7757bd16, 3 },
    { 0x9a3fbd17, 3 },
    { 0xa033bd15, 3 },
    { 0x90efbd19, 3 },
    { 0x98bfbd1a, 3 },
    { 0xc637bd19, 3 },
    { 0x2d23bd1c, 3 },
    { 0xabdbbd1c, 3 },
    { 0xf4dfbd1b, 3 },
    { 0xf24fbd1f, 3 },
    { 0x6aefbd1d, 4 },
    { 0, 0 },
    { 0x15dbbd22, 3 },
    { 0xcbcfbd23, 4 },
    { 0x95fbd23, 3 },
    { 0, 0 },
    { 0x633fbd26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf453bd29, 4 },
    { 0xec27bd29, 4 },
    { 0, 0 },
    { 0x17f7bd2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x510fbd2f, 4 },
    { 0, 0 },
    { 0xae97bd31, 3 },
    { 0xff17bd31, 3 },
    { 0x31a7bd33, 3 },
    { 0, 0 },
    { 0x7087bd35, 3 },
    { 0xdd2bbd36, 3 },
    { 0x3673bd35, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246fbd3c, 4 },
    { 0x9503bd3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420bbd49, 3 },
    { 0xab3fbd4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85fbd4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c3bd51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7bd57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ebbd5d, 3 },
    { 0x19c7bd5e, 3 },
    { 0x2f73bd5f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d3bd64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d97bd67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfebbd6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827fbd73, 3 },
    { 0, 0 },
    { 0x2623bd75, 3 },
    { 0, 0 },
    { 0xc8a3bd77, 3 },
    { 0x5d9bbd77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107bbd7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbbbd89, 3 },
    { 0, 0 },
    { 0x20dbbd8b, 3 },
    { 0, 0 },
    { 0x54b7bd8d, 4 },
    { 0xc3b3bd8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f3bd91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef03bd95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f7bd98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcbbda4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bbda7, 4 },
    { 0, 0 },
    { 0xe7dbbda9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef7bdac, 3 },
    { 0x9897bdac, 3 },
    { 0, 0 },
    { 0xf263bdaf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe417bdb2, 4 },
    { 0, 0 },
    { 0xf8d3bdb4, 3 },
    { 0x4113bdb5, 3 },
    { 0xc43bdb6, 4 },
    { 0, 0 },
    { 0x732fbdb8, 3 },
    { 0x95d7bdb9, 4 },
    { 0x4037bdba, 4 },
    { 0xdbc7bdbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5033bdbe, 3 },
    { 0xe9f7bdbe, 3 },
    { 0x5413bdbe, 3 },
    { 0, 0 },
    { 0x448fbdc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1bbdc7, 4 },
    { 0x23ebbdc8, 3 },
    { 0x113bbdc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69bbdcc, 4 },
    { 0x967bbdcc, 3 },
    { 0xe753bdce, 3 },
    { 0xfa5fbdcf, 3 },
    { 0, 0 },
    { 0x3e3fbdd1, 4 },
    { 0xec03bdd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721bbdd6, 3 },
    { 0xe78fbdd7, 3 },
    { 0xbefbdd8, 3 },
    { 0xfec3bdd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904bbddf, 3 },
    { 0xd2c7bde0, 4 },
    { 0x2e1fbde1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ebbdea, 3 },
    { 0, 0 },
    { 0x8a4bbdec, 3 },
    { 0xd37fbdec, 4 },
    { 0, 0 },
    { 0x82efbdef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e7bdf3, 3 },
    { 0x3e67bdf4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cbbdfa, 3 },
    { 0xbb03bdfa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecfbe04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abfbe08, 3 },
    { 0xdfcfbe08, 5 },
    { 0x3f93be09, 4 },
    { 0xfa4bbe09, 3 },
    { 0xc33fbe0c, 3 },
    { 0, 0 },
    { 0xf067be0e, 3 },
    { 0x127be0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bbe13, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09fbe1a, 4 },
    { 0x64d3be1a, 3 },
    { 0x631fbe1a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a7be22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17be29, 3 },
    { 0xc73be2a, 4 },
    { 0xa9f7be2b, 3 },
    { 0x8963be29, 3 },
    { 0x5abbbe2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49fbe31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7fbe39, 3 },
    { 0xa83be3a, 3 },
    { 0x9837be3b, 3 },
    { 0xcefbe3c, 3 },
    { 0x1713be3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45bbe41, 3 },
    { 0, 0 },
    { 0xbd87be43, 5 },
    { 0x5fd3be43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23fbe4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6efbe51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83bbe54, 4 },
    { 0xbd53be55, 4 },
    { 0x5fbbe56, 3 },
    { 0x788bbe56, 4 },
    { 0, 0 },
    { 0x750bbe59, 3 },
    { 0x3457be5a, 3 },
    { 0x1bf7be5b, 3 },
    { 0xee7be5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x374fbe5f, 3 },
    { 0x3f57be60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26bbe64, 3 },
    { 0, 0 },
    { 0x14dfbe66, 3 },
    { 0xb4c3be67, 3 },
    { 0x96dbbe68, 4 },
    { 0xb3a3be68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa633be6c, 3 },
    { 0, 0 },
    { 0x73b3be6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc563be75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7743be7d, 4 },
    { 0x87e7be7e, 4 },
    { 0xdf07be7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7907be82, 3 },
    { 0x3277be82, 4 },
    { 0, 0 },
    { 0xe3b3be85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6737be8c, 3 },
    { 0, 0 },
    { 0x8783be8e, 3 },
    { 0x79a7be8f, 3 },
    { 0x4e77be90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82fbe94, 3 },
    { 0x205fbe94, 3 },
    { 0xfbe3be96, 3 },
    { 0x6ae3be97, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938fbe9c, 3 },
    { 0xb2fbe9d, 3 },
    { 0xb03bbe9e, 3 },
    { 0, 0 },
    { 0x59d7bea0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ffbea4, 4 },
    { 0, 0 },
    { 0x5a9bbea6, 3 },
    { 0xf0a7bea7, 3 },
    { 0x349bbea8, 3 },
    { 0xd3f3bea9, 3 },
    { 0xf1e3beaa, 4 },
    { 0xdd8fbeab, 3 },
    { 0, 0 },
    { 0xbb77bead, 3 },
    { 0xa42fbead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x223fbeb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x903bbeb4, 4 },
    { 0x3723beb5, 3 },
    { 0x8c9bbeb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7fbebb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507bec1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46bbec8, 4 },
    { 0x117bbec9, 3 },
    { 0, 0 },
    { 0xc48bbecb, 4 },
    { 0x463bbecc, 3 },
    { 0x71dfbecc, 4 },
    { 0, 0 },
    { 0x512bbecf, 3 },
    { 0x9d97becf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa297bedb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad7bede, 3 },
    { 0, 0 },
    { 0xf6e7bee0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95efbee4, 3 },
    { 0, 0 },
    { 0x4ed7bee6, 3 },
    { 0xf5bfbee7, 3 },
    { 0x55d7bee7, 3 },
    { 0x50f3bee8, 3 },
    { 0x8ed7bee7, 3 },
    { 0xb323bee9, 3 },
    { 0xfd83beec, 3 },
    { 0xac8bbeec, 3 },
    { 0x842bbeee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198fbef3, 3 },
    { 0xac7bef4, 3 },
    { 0x2a43bef5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77bef8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c3befb, 3 },
    { 0, 0 },
    { 0x99d7befd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd3bf03, 3 },
    { 0xeeffbf04, 3 },
    { 0x4a6bbf04, 4 },
    { 0, 0 },
    { 0x1b3bf07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16bbf0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cfbf11, 3 },
    { 0x7f0bbf12, 4 },
    { 0, 0 },
    { 0x602bbf14, 4 },
    { 0xc123bf15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7bf1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2bbf21, 3 },
    { 0x362bbf22, 3 },
    { 0xb6afbf23, 3 },
    { 0xb83bbf24, 3 },
    { 0, 0 },
    { 0x1d93bf26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x127fbf29, 5 },
    { 0, 0 },
    { 0x3377bf2b, 3 },
    { 0x6397bf2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237fbf34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ffbf3e, 3 },
    { 0x1477bf3e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fafbf43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142bbf47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf3bf4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58fbf51, 4 },
    { 0x9b5bbf51, 3 },
    { 0x54d7bf53, 5 },
    { 0x1d7bbf54, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1fbf5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9733bf63, 4 },
    { 0, 0 },
    { 0x50efbf65, 3 },
    { 0xb5fbbf65, 4 },
    { 0, 0 },
    { 0x90afbf68, 3 },
    { 0x2807bf69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad3bf6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac3bf71, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f47bf75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9997bf79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa57bf7c, 3 },
    { 0xb663bf7d, 3 },
    { 0xb90fbf7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d7bf81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf33bf84, 3 },
    { 0x24b7bf85, 3 },
    { 0, 0 },
    { 0xf98fbf87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x125bbf8a, 4 },
    { 0x8cabbf8b, 3 },
    { 0x323bf8b, 3 },
    { 0, 0 },
    { 0x8633bf8e, 3 },
    { 0, 0 },
    { 0xf5cfbf90, 3 },
    { 0xf6c3bf90, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35abbf95, 3 },
    { 0, 0 },
    { 0x45e3bf97, 3 },
    { 0, 0 },
    { 0xe427bf99, 3 },
    { 0x57bf99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff7bf9e, 3 },
    { 0, 0 },
    { 0x95fbfa0, 4 },
    { 0, 0 },
    { 0xf27bfa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a3bfab, 3 },
    { 0xece7bfac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a7bfaf, 3 },
    { 0, 0 },
    { 0xe703bfb1, 3 },
    { 0xe1ebbfb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x575fbfb5, 3 },
    { 0x2e9fbfb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71fbfbb, 3 },
    { 0x20b7bfbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9477bfc6, 3 },
    { 0, 0 },
    { 0x90e3bfc8, 3 },
    { 0x6f7bbfc8, 3 },
    { 0x4423bfc9, 3 },
    { 0x9793bfc8, 4 },
    { 0x1197bfcc, 3 },
    { 0xaf47bfcd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bbbfd1, 5 },
    { 0x9d1bbfd2, 3 },
    { 0xab67bfd3, 4 },
    { 0x90f3bfd4, 3 },
    { 0x1bc7bfd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bfd8, 3 },
    { 0x8a57bfd8, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a13bfe0, 3 },
    { 0, 0 },
    { 0xabd3bfe2, 3 },
    { 0x3bbbbfe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cfbfe6, 3 },
    { 0xae83bfe7, 3 },
    { 0x54e7bfe7, 3 },
    { 0xc8b3bfe9, 3 },
    { 0, 0 },
    { 0xa03fbfeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c7bff0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797bff4, 4 },
    { 0x98d3bff5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9bbffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4093c001, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f37c005, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3337c00b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcbc00e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79fc013, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadfc016, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2fc019, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69fc020, 4 },
    { 0x42f3c020, 3 },
    { 0x8b2bc020, 3 },
    { 0, 0 },
    { 0xea1bc024, 4 },
    { 0, 0 },
    { 0x2977c026, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5fc029, 3 },
    { 0, 0 },
    { 0xb0dbc02b, 3 },
    { 0xc46bc02c, 4 },
    { 0xf733c02d, 3 },
    { 0x4d4bc02d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98abc031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fc038, 3 },
    { 0xddefc039, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cfc03c, 3 },
    { 0xd07c03d, 3 },
    { 0xc333c03d, 5 },
    { 0x4713c03f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fbc043, 4 },
    { 0x516fc043, 4 },
    { 0x4b53c045, 3 },
    { 0, 0 },
    { 0x1247c047, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fc04a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c03c04f, 3 },
    { 0x5b93c050, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c054, 3 },
    { 0x630bc055, 3 },
    { 0x18e3c056, 3 },
    { 0xa9c3c056, 3 },
    { 0x8d03c055, 4 },
    { 0xd47c054, 3 },
    { 0x7983c055, 3 },
    { 0xfdc3c054, 4 },
    { 0xc607c05c, 4 },
    { 0x8dbc05d, 3 },
    { 0x97b7c05e, 3 },
    { 0x2edfc05f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853bc063, 3 },
    { 0x5dcbc063, 4 },
    { 0xa4c3c065, 3 },
    { 0, 0 },
    { 0xa17bc067, 4 },
    { 0x8ef3c068, 4 },
    { 0xe65bc068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4fc071, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ebc075, 3 },
    { 0xa04bc076, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe263c07a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3c081, 3 },
    { 0, 0 },
    { 0x549bc083, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30fc087, 4 },
    { 0x1267c088, 3 },
    { 0xffc7c089, 3 },
    { 0, 0 },
    { 0x163bc08b, 4 },
    { 0, 0 },
    { 0x6c8fc08d, 3 },
    { 0x2cd3c08e, 3 },
    { 0x97ebc08f, 4 },
    { 0x5f4fc090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a7c096, 4 },
    { 0, 0 },
    { 0x73f7c098, 4 },
    { 0, 0 },
    { 0xe36bc09a, 4 },
    { 0x3dabc09b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4bc09e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bfc0a1, 3 },
    { 0xff47c0a2, 3 },
    { 0x1f43c0a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe187c0a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87bc0ab, 3 },
    { 0, 0 },
    { 0xd95fc0ad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x239fc0b0, 3 },
    { 0xd6fc0b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f7c0b8, 3 },
    { 0xab0fc0b9, 3 },
    { 0xcf9fc0b9, 4 },
    { 0xfac7c0b8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6bc0be, 3 },
    { 0xab1fc0bf, 3 },
    { 0xf797c0bf, 3 },
    { 0x4233c0c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3c0c7, 3 },
    { 0x7717c0c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8913c0cb, 4 },
    { 0xdfe3c0cc, 3 },
    { 0, 0 },
    { 0xf197c0ce, 4 },
    { 0xe893c0ce, 3 },
    { 0xab8fc0cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33c0d5, 3 },
    { 0xed07c0d6, 3 },
    { 0xddefc0d7, 3 },
    { 0xb78bc0d8, 4 },
    { 0xf853c0d9, 3 },
    { 0, 0 },
    { 0x47cfc0db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bc0e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfc0e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2913c0eb, 3 },
    { 0, 0 },
    { 0x8a23c0ed, 3 },
    { 0, 0 },
    { 0xfa23c0ef, 3 },
    { 0xf90bc0f0, 3 },
    { 0x72d3c0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fc0f4, 3 },
    { 0, 0 },
    { 0x821bc0f6, 4 },
    { 0x40c7c0f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc101, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607fc105, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6fc116, 3 },
    { 0xe14fc117, 3 },
    { 0x42efc118, 4 },
    { 0, 0 },
    { 0x7e9bc11a, 3 },
    { 0xe12bc11a, 3 },
    { 0x363fc11c, 3 },
    { 0xe11fc11d, 3 },
    { 0, 0 },
    { 0x6dfc11f, 3 },
    { 0x30cbc120, 4 },
    { 0x9f83c121, 3 },
    { 0xd4dfc11f, 4 },
    { 0, 0 },
    { 0x6607c124, 3 },
    { 0x489fc125, 3 },
    { 0x81e3c125, 3 },
    { 0xc647c127, 3 },
    { 0x28bfc128, 3 },
    { 0x42d7c129, 3 },
    { 0x436fc12a, 3 },
    { 0x9ffc12b, 3 },
    { 0x2ba7c12c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf443c12f, 3 },
    { 0, 0 },
    { 0xed07c131, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dfc139, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313fc13d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0fc143, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea23c148, 3 },
    { 0xa19fc149, 5 },
    { 0x84bfc148, 4 },
    { 0xfce3c14b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9073c14e, 3 },
    { 0x4a87c14e, 4 },
    { 0x7a7bc14e, 4 },
    { 0, 0 },
    { 0x15a7c152, 3 },
    { 0, 0 },
    { 0x152fc154, 3 },
    { 0xe0e3c155, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xde87c158, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33c15c, 3 },
    { 0x1637c15d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf3c162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a37c166, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b7c169, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d7c16c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5effc171, 3 },
    { 0x7fe3c172, 3 },
    { 0x5413c173, 4 },
    { 0xa633c173, 3 },
    { 0xb343c174, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x73abc178, 3 },
    { 0xbde7c178, 3 },
    { 0x1393c17a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13c17f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d93c18e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957c195, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a7c19c, 4 },
    { 0xf99fc19d, 3 },
    { 0, 0 },
    { 0xa897c19f, 4 },
    { 0x5793c19f, 3 },
    { 0, 0 },
    { 0x399fc1a2, 3 },
    { 0x444bc1a3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc317c1a8, 3 },
    { 0x634bc1a9, 3 },
    { 0, 0 },
    { 0xf377c1ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe773c1b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ffc1b7, 3 },
    { 0x8f47c1b8, 3 },
    { 0xa9bc1b9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2463c1bd, 3 },
    { 0x9d7bc1be, 3 },
    { 0xb253c1bf, 3 },
    { 0x9237c1c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7bc1c4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bc1c8, 3 },
    { 0xcf17c1c8, 4 },
    { 0x5fcbc1c8, 4 },
    { 0xb15fc1cb, 3 },
    { 0, 0 },
    { 0x95d3c1cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a7c1d1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb37c1d4, 3 },
    { 0x34b7c1d5, 3 },
    { 0xe927c1d6, 3 },
    { 0xb6fc1d7, 3 },
    { 0x23bbc1d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dbc1db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07bc1df, 4 },
    { 0xfe07c1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc483c1e6, 3 },
    { 0x305bc1e6, 3 },
    { 0, 0 },
    { 0xa717c1e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe133c1ec, 3 },
    { 0x45fc1ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd73c1f4, 4 },
    { 0x9277c1f4, 3 },
    { 0x8c3fc1f6, 3 },
    { 0x5523c1f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619bc1fe, 4 },
    { 0x27afc1ff, 3 },
    { 0xef37c1ff, 4 },
    { 0x69efc201, 3 },
    { 0xea3c201, 3 },
    { 0xe2e3c203, 3 },
    { 0xed33c204, 3 },
    { 0x550fc205, 3 },
    { 0x107fc204, 3 },
    { 0xca9fc206, 3 },
    { 0x2bdbc202, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89bc20b, 3 },
    { 0, 0 },
    { 0x84f7c20d, 3 },
    { 0x3b53c20e, 4 },
    { 0, 0 },
    { 0xf907c210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7fc218, 3 },
    { 0xe137c219, 3 },
    { 0x23a3c21a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167c21d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bbc220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba7c225, 3 },
    { 0xd583c225, 3 },
    { 0xd32fc226, 3 },
    { 0x31a3c228, 3 },
    { 0xc48fc229, 3 },
    { 0x86fbc22a, 4 },
    { 0x96b3c229, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78fc230, 3 },
    { 0xff6bc231, 3 },
    { 0x7d1bc231, 3 },
    { 0xad6bc231, 3 },
    { 0x6d1bc234, 3 },
    { 0x5577c235, 3 },
    { 0, 0 },
    { 0xe9a3c237, 4 },
    { 0xa0ebc237, 3 },
    { 0x8ac7c238, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c33c23c, 3 },
    { 0x96bc23d, 3 },
    { 0xb36fc23c, 3 },
    { 0x425fc23f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318fc246, 4 },
    { 0, 0 },
    { 0x3b83c248, 3 },
    { 0x2be7c249, 3 },
    { 0x8dffc249, 3 },
    { 0, 0 },
    { 0x472fc24c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644fc251, 3 },
    { 0xe2cbc252, 3 },
    { 0xbdd3c252, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139fc257, 3 },
    { 0xde6fc257, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbc25c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73efc265, 3 },
    { 0, 0 },
    { 0x74b3c267, 3 },
    { 0xd393c268, 4 },
    { 0, 0 },
    { 0x730fc26a, 3 },
    { 0x82c3c26b, 3 },
    { 0, 0 },
    { 0x6327c26d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9bc274, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344fc279, 3 },
    { 0xfffc27a, 3 },
    { 0, 0 },
    { 0x9127c27c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fc280, 4 },
    { 0x6d37c281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316bc285, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb7c288, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699bc28d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7fc293, 3 },
    { 0x942bc294, 3 },
    { 0xd6ebc295, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80bc299, 3 },
    { 0x653fc299, 3 },
    { 0x673c29b, 3 },
    { 0x5fbfc29c, 3 },
    { 0x8ae7c29b, 3 },
    { 0xf0ebc29e, 3 },
    { 0x18b7c29c, 4 },
    { 0xcecbc2a0, 3 },
    { 0, 0 },
    { 0x68abc2a2, 3 },
    { 0x2507c2a3, 4 },
    { 0xeadfc2a3, 3 },
    { 0xa81fc2a5, 3 },
    { 0xf1bfc2a6, 4 },
    { 0xde03c2a6, 3 },
    { 0xedf3c2a7, 3 },
    { 0, 0 },
    { 0x55dfc2aa, 3 },
    { 0x9c7c2ab, 3 },
    { 0, 0 },
    { 0xca37c2ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3efc2b6, 3 },
    { 0x9743c2b7, 3 },
    { 0xae57c2b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7987c2be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4afc2c1, 3 },
    { 0, 0 },
    { 0x93c7c2c3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa3c2cd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3673c2d1, 3 },
    { 0, 0 },
    { 0x692fc2d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abfc2d9, 3 },
    { 0xc003c2da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3217c2e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecfc2e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7fc2ec, 4 },
    { 0, 0 },
    { 0x5173c2ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b27c2f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6fc2f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf7c2f9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfbc2fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cfc301, 3 },
    { 0x9e07c301, 4 },
    { 0x7267c303, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f67c310, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bfc318, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc31c, 3 },
    { 0xdbdbc31d, 3 },
    { 0, 0 },
    { 0x35c7c31f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd213c323, 3 },
    { 0xf4afc324, 3 },
    { 0x9563c324, 3 },
    { 0x28dfc323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b3c32a, 3 },
    { 0x5767c32b, 4 },
    { 0xa8e7c32c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abbc32f, 3 },
    { 0xe897c330, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360bc336, 3 },
    { 0, 0 },
    { 0x2f63c338, 4 },
    { 0xadabc339, 4 },
    { 0x17b3c33a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfc340, 3 },
    { 0x60c3c341, 3 },
    { 0x32dfc340, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05fc345, 3 },
    { 0xf1abc346, 3 },
    { 0x9c23c347, 3 },
    { 0x39a3c347, 4 },
    { 0xbcefc347, 3 },
    { 0xa0e3c346, 3 },
    { 0x2a13c34b, 3 },
    { 0xc273c34b, 3 },
    { 0xc3e7c34c, 3 },
    { 0x766fc345, 3 },
    { 0, 0 },
    { 0xef3c350, 3 },
    { 0xfe2fc351, 3 },
    { 0xe10fc351, 3 },
    { 0x536fc353, 4 },
    { 0x7b4fc354, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c358, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b3c35b, 3 },
    { 0x4107c35c, 3 },
    { 0x37a3c35b, 3 },
    { 0x300bc35e, 3 },
    { 0xfd63c35f, 3 },
    { 0, 0 },
    { 0xa15bc361, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ebc368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4bc36c, 3 },
    { 0xeb83c36d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6fc372, 3 },
    { 0, 0 },
    { 0xb0b3c374, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987c378, 3 },
    { 0, 0 },
    { 0x651fc37a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ffc37f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bc384, 3 },
    { 0, 0 },
    { 0x1cdfc386, 4 },
    { 0, 0 },
    { 0x6583c388, 3 },
    { 0, 0 },
    { 0xb283c38a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee7c38d, 3 },
    { 0x95c3c38e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c7c394, 3 },
    { 0x5f8fc395, 3 },
    { 0xe353c396, 4 },
    { 0xe76bc396, 3 },
    { 0xde7c398, 4 },
    { 0xd59bc394, 3 },
    { 0x33afc39a, 4 },
    { 0x5e73c39a, 3 },
    { 0, 0 },
    { 0xe093c39d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf3c3a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x152bc3a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3c3a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55bc3ac, 3 },
    { 0x14bbc3ad, 3 },
    { 0x86f3c3ae, 4 },
    { 0x207bc3ae, 3 },
    { 0, 0 },
    { 0x66bfc3b1, 3 },
    { 0xebc3c3b1, 3 },
    { 0xe697c3b1, 3 },
    { 0x3773c3b4, 4 },
    { 0xc1c7c3b3, 4 },
    { 0x781fc3b5, 3 },
    { 0xd60bc3b3, 4 },
    { 0x44fbc3b8, 3 },
    { 0x985bc3b1, 3 },
    { 0x8937c3ba, 3 },
    { 0x20afc3bb, 3 },
    { 0xc69bc3bc, 3 },
    { 0xf7e3c3b4, 3 },
    { 0x2517c3ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb57c3c1, 4 },
    { 0x8e43c3c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe813c3ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58bc3d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5907c3d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126fc3dc, 3 },
    { 0x1bf7c3dc, 4 },
    { 0, 0 },
    { 0xadbfc3df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ebc3e5, 3 },
    { 0x7d33c3e5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18fc3ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f7c3f1, 3 },
    { 0x7a37c3f2, 3 },
    { 0, 0 },
    { 0x15f7c3f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15fc3f9, 3 },
    { 0x353c3fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3c407, 3 },
    { 0x6f4fc408, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf23c40c, 3 },
    { 0xd92bc40d, 3 },
    { 0x394fc40c, 3 },
    { 0x4c57c40e, 3 },
    { 0, 0 },
    { 0xd33c411, 4 },
    { 0xac77c411, 3 },
    { 0x13fbc413, 3 },
    { 0x5c6bc414, 3 },
    { 0x1857c414, 3 },
    { 0xd167c416, 4 },
    { 0, 0 },
    { 0xca5fc418, 4 },
    { 0, 0 },
    { 0xb6f7c41a, 3 },
    { 0x754fc41b, 3 },
    { 0, 0 },
    { 0xfa57c41d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af3c422, 3 },
    { 0xd3afc423, 4 },
    { 0x5517c424, 5 },
    { 0, 0 },
    { 0xe187c426, 4 },
    { 0x990bc427, 3 },
    { 0xa2bc428, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2143c42c, 3 },
    { 0xd513c42c, 4 },
    { 0x6653c42e, 4 },
    { 0xe17bc42f, 3 },
    { 0xaa77c42d, 3 },
    { 0, 0 },
    { 0x5067c432, 3 },
    { 0x3d13c432, 4 },
    { 0, 0 },
    { 0xe04fc435, 3 },
    { 0xdd8fc436, 3 },
    { 0xe83fc435, 3 },
    { 0x5dcbc436, 3 },
    { 0, 0 },
    { 0x4047c43a, 3 },
    { 0xa81fc43b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2abc43e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ebc444, 3 },
    { 0, 0 },
    { 0xa85bc446, 4 },
    { 0x13c3c447, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32bc44a, 4 },
    { 0, 0 },
    { 0xec53c44c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c3c44f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bc454, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd263c45b, 4 },
    { 0xa5bfc45c, 3 },
    { 0, 0 },
    { 0xec2fc45e, 3 },
    { 0, 0 },
    { 0xa423c460, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x81afc463, 3 },
    { 0xd383c464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd233c467, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6863c46e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa283c472, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fbc476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bc47e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b7c481, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa093c489, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4bc48c, 3 },
    { 0xfde7c48d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f73c490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db7c494, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc27c49d, 3 },
    { 0x4557c49e, 3 },
    { 0, 0 },
    { 0x402bc4a0, 3 },
    { 0x4f93c4a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x488bc4a4, 3 },
    { 0x6443c4a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x239bc4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc7c4b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bbc4b7, 4 },
    { 0x2fefc4b8, 3 },
    { 0, 0 },
    { 0x9fcfc4ba, 3 },
    { 0x7ca3c4ba, 3 },
    { 0x11cbc4ba, 3 },
    { 0x989fc4bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813c4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80fc4c3, 4 },
    { 0, 0 },
    { 0xc0b3c4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b3c4d2, 3 },
    { 0x680bc4d2, 3 },
    { 0x655fc4d4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x679bc4d7, 3 },
    { 0xddc3c4d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf43c4db, 3 },
    { 0xda73c4dc, 3 },
    { 0xc427c4dd, 3 },
    { 0x2767c4de, 3 },
    { 0x1fe3c4de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3c4e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc4e7, 3 },
    { 0x1d37c4e8, 4 },
    { 0xe3d7c4e9, 3 },
    { 0xde33c4e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3fc4f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e7c4f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85fc4fb, 3 },
    { 0xddefc4fb, 4 },
    { 0x5223c4fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe207c503, 4 },
    { 0x93cbc503, 3 },
    { 0x93fc504, 3 },
    { 0, 0 },
    { 0xf5abc507, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeefc515, 3 },
    { 0x45e7c515, 3 },
    { 0x4473c517, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7bc51c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4377c524, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbc529, 3 },
    { 0x3afbc529, 4 },
    { 0x732bc52a, 3 },
    { 0x825fc52b, 3 },
    { 0, 0 },
    { 0x228bc52e, 4 },
    { 0x4a13c52f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bfc532, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb7c535, 3 },
    { 0x931fc535, 3 },
    { 0x70c3c537, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x767c53a, 4 },
    { 0x80efc53a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78bc543, 3 },
    { 0, 0 },
    { 0x9e2fc545, 4 },
    { 0, 0 },
    { 0x835bc547, 3 },
    { 0x764fc547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9247c54c, 3 },
    { 0, 0 },
    { 0xa46bc54e, 3 },
    { 0xfc73c54f, 3 },
    { 0xbd3c550, 3 },
    { 0, 0 },
    { 0xaadfc552, 3 },
    { 0x446bc553, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0fc557, 3 },
    { 0xd033c558, 3 },
    { 0xe413c559, 3 },
    { 0x38fbc559, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x920bc55d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673c563, 3 },
    { 0x5b73c564, 3 },
    { 0x1bf3c565, 3 },
    { 0, 0 },
    { 0x92cfc567, 3 },
    { 0x8fc568, 3 },
    { 0, 0 },
    { 0xaa6bc56a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bfc56d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3bc571, 4 },
    { 0xcafc572, 3 },
    { 0xe073c573, 3 },
    { 0x2d1bc572, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9bc578, 3 },
    { 0x2fdbc579, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3c57e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfbc585, 4 },
    { 0x44fc586, 3 },
    { 0xb67c585, 3 },
    { 0xdf9fc588, 3 },
    { 0, 0 },
    { 0x9723c58a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e7c591, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bc594, 3 },
    { 0, 0 },
    { 0xe9ffc596, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24bc59d, 3 },
    { 0x6767c59e, 3 },
    { 0xa90bc59e, 3 },
    { 0x72abc59f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713bc5a4, 3 },
    { 0x1197c5a5, 4 },
    { 0, 0 },
    { 0xd66bc5a7, 4 },
    { 0, 0 },
    { 0xdf7c5a9, 4 },
    { 0, 0 },
    { 0xede3c5ab, 3 },
    { 0x237bc5ac, 4 },
    { 0x2383c5ab, 3 },
    { 0xd893c5ae, 3 },
    { 0x691fc5ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6fc5b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6007c5b5, 3 },
    { 0, 0 },
    { 0x46fc5b7, 4 },
    { 0xcc87c5b8, 3 },
    { 0x8333c5b9, 3 },
    { 0x725fc5ba, 4 },
    { 0, 0 },
    { 0x2583c5bc, 3 },
    { 0xafcbc5bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9bc5c0, 3 },
    { 0, 0 },
    { 0x1707c5c2, 3 },
    { 0x69e7c5c3, 3 },
    { 0x873c5c4, 4 },
    { 0x128fc5c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42fc5cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecbc5d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbc5d5, 4 },
    { 0xbcffc5d6, 3 },
    { 0x8d4fc5d6, 4 },
    { 0xe903c5d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb57c5dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x738fc5e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e17c5e3, 3 },
    { 0x41afc5e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f03c5e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f3c5ed, 3 },
    { 0xfa6bc5ee, 3 },
    { 0x90cbc5ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa677c5f9, 4 },
    { 0, 0 },
    { 0x3cabc5fb, 3 },
    { 0x758bc5fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013c600, 3 },
    { 0xd96fc600, 4 },
    { 0x130bc602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80bc605, 3 },
    { 0x8ba3c606, 3 },
    { 0x7593c605, 3 },
    { 0xb953c608, 3 },
    { 0x7ac3c609, 3 },
    { 0xbfa7c609, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3bc610, 4 },
    { 0x8537c611, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0bc619, 3 },
    { 0, 0 },
    { 0xfd97c61b, 3 },
    { 0, 0 },
    { 0xd84fc61d, 3 },
    { 0x3c3bc61e, 3 },
    { 0x7a63c61f, 3 },
    { 0x5867c61f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fc623, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e63c627, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ebc62c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04bc62f, 3 },
    { 0xf8ffc62f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007c633, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80efc636, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc063c63a, 3 },
    { 0xf03c63b, 4 },
    { 0, 0 },
    { 0x9e67c63d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd337c640, 4 },
    { 0x167c640, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4687c64a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b03c650, 3 },
    { 0xfac7c651, 4 },
    { 0xbfbfc652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc497c656, 3 },
    { 0, 0 },
    { 0x8523c658, 4 },
    { 0x31bfc658, 3 },
    { 0x4fd7c65a, 3 },
    { 0, 0 },
    { 0xb6c3c65c, 3 },
    { 0x29d7c65d, 3 },
    { 0xc6a7c65d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e3c662, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26fc666, 4 },
    { 0x9933c667, 3 },
    { 0, 0 },
    { 0x3797c669, 3 },
    { 0x5b63c669, 3 },
    { 0, 0 },
    { 0x176bc66c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fc673, 4 },
    { 0, 0 },
    { 0x3693c675, 4 },
    { 0x847bc676, 3 },
    { 0, 0 },
    { 0xd2b3c678, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd063c67d, 3 },
    { 0x9603c67e, 3 },
    { 0x9653c67f, 3 },
    { 0x1bb7c680, 3 },
    { 0x116fc681, 4 },
    { 0xeedfc680, 3 },
    { 0xa087c683, 5 },
    { 0, 0 },
    { 0x813fc685, 3 },
    { 0x29c3c686, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7c68e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3fc691, 3 },
    { 0xac9bc691, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2657c698, 3 },
    { 0xa217c699, 3 },
    { 0x5e6bc699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bbc6a0, 3 },
    { 0, 0 },
    { 0x211fc6a2, 3 },
    { 0x6dc7c6a3, 3 },
    { 0xfd13c6a3, 3 },
    { 0x943c6a5, 3 },
    { 0xc483c6a5, 3 },
    { 0xd553c6a7, 3 },
    { 0xc6bc6a2, 3 },
    { 0xb423c6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4fc6af, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f43c6b5, 4 },
    { 0xd8ebc6b6, 3 },
    { 0x9553c6b7, 3 },
    { 0, 0 },
    { 0x140fc6b9, 3 },
    { 0xcaefc6ba, 3 },
    { 0xd9bbc6b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec7c6c6, 3 },
    { 0x70c7c6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713c6d0, 3 },
    { 0xb7cfc6d0, 3 },
    { 0x37f7c6d2, 4 },
    { 0xd1c3c6d2, 4 },
    { 0x8bf7c6d1, 3 },
    { 0, 0 },
    { 0xb6a3c6d6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30fc6d9, 3 },
    { 0x71e7c6da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc6e1, 3 },
    { 0x2c6bc6e1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bbc6e7, 3 },
    { 0, 0 },
    { 0x46e7c6e9, 3 },
    { 0x348bc6ea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d3c6ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b7c6f5, 3 },
    { 0x8e8bc6f6, 3 },
    { 0, 0 },
    { 0x62b3c6f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6897c6ff, 3 },
    { 0, 0 },
    { 0xd2e3c701, 3 },
    { 0, 0 },
    { 0xd1abc703, 3 },
    { 0xb98fc703, 3 },
    { 0x6dc7c704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5703c708, 3 },
    { 0x577c708, 3 },
    { 0, 0 },
    { 0xa3cfc70b, 3 },
    { 0, 0 },
    { 0xcdbfc70d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd147c712, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b3c717, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8babc721, 3 },
    { 0xbf27c722, 4 },
    { 0, 0 },
    { 0x279bc724, 3 },
    { 0xbcb3c725, 3 },
    { 0xd39fc726, 3 },
    { 0xcdbc727, 4 },
    { 0x75fc728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdffc72c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c7c735, 3 },
    { 0x3af3c736, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdbc73a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d7c73e, 3 },
    { 0x3d7fc73f, 3 },
    { 0x52bbc73f, 3 },
    { 0xe6fc741, 3 },
    { 0xe87fc741, 4 },
    { 0xb777c73f, 3 },
    { 0xde0fc744, 3 },
    { 0x2407c73e, 3 },
    { 0, 0 },
    { 0x327c747, 4 },
    { 0xfd37c748, 4 },
    { 0, 0 },
    { 0x2627c74a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da7c753, 3 },
    { 0x619bc754, 3 },
    { 0x14bbc754, 3 },
    { 0x7df3c756, 3 },
    { 0xe5fc757, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515bc75c, 3 },
    { 0x3bcbc75d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dffc760, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f3c766, 3 },
    { 0xbd37c767, 3 },
    { 0xd05bc766, 4 },
    { 0x15f7c769, 3 },
    { 0x1027c76a, 4 },
    { 0x9983c768, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17bc773, 3 },
    { 0x15cbc773, 3 },
    { 0x3dafc775, 4 },
    { 0x944fc774, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9267c779, 3 },
    { 0, 0 },
    { 0x3a37c77b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27c77e, 3 },
    { 0x3c47c77f, 4 },
    { 0x45afc780, 3 },
    { 0, 0 },
    { 0x87dfc782, 3 },
    { 0, 0 },
    { 0xa7c784, 3 },
    { 0xd7cbc785, 3 },
    { 0xa287c786, 3 },
    { 0x2f17c787, 3 },
    { 0x91ffc787, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5fc790, 3 },
    { 0, 0 },
    { 0xd03bc792, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x969bc795, 3 },
    { 0x5e03c796, 3 },
    { 0x8153c795, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74fc79d, 3 },
    { 0x589bc79e, 4 },
    { 0, 0 },
    { 0xfdefc7a0, 3 },
    { 0x523c7a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fc7a5, 3 },
    { 0, 0 },
    { 0x83d7c7a7, 3 },
    { 0, 0 },
    { 0x9547c7a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dffc7ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fbc7b1, 3 },
    { 0x87dbc7b1, 4 },
    { 0x571fc7b1, 3 },
    { 0x36dbc7b1, 3 },
    { 0xba5fc7b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe163c7b8, 4 },
    { 0xf277c7b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f67c7be, 4 },
    { 0, 0 },
    { 0x7ba3c7c0, 3 },
    { 0x34cfc7c1, 3 },
    { 0, 0 },
    { 0xad1fc7c3, 3 },
    { 0x3dfbc7c4, 3 },
    { 0x1967c7c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa3c7c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1fc7d0, 3 },
    { 0xab13c7d1, 3 },
    { 0x8003c7d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0fc7d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c7dc, 5 },
    { 0x187c7dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b7c7e3, 4 },
    { 0xe01bc7e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6167c7e7, 4 },
    { 0, 0 },
    { 0x723c7e9, 3 },
    { 0xcddbc7ea, 3 },
    { 0xa2d7c7ea, 3 },
    { 0xd6bbc7ec, 3 },
    { 0, 0 },
    { 0xfba7c7ee, 3 },
    { 0xfabfc7ee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8fc7f3, 3 },
    { 0xeeabc7f4, 3 },
    { 0, 0 },
    { 0x85ebc7f6, 3 },
    { 0, 0 },
    { 0x293bc7f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8113c7fe, 4 },
    { 0x4d4bc7ff, 3 },
    { 0x5b5fc800, 3 },
    { 0x505fc800, 3 },
    { 0xa6b7c800, 3 },
    { 0xdddfc801, 3 },
    { 0xd823c804, 3 },
    { 0x513c800, 3 },
    { 0x6197c806, 3 },
    { 0xb8e3c807, 4 },
    { 0x9edbc805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fefc80b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea7c80f, 3 },
    { 0xf9ffc80f, 3 },
    { 0x7a8fc811, 3 },
    { 0x78afc810, 3 },
    { 0x3173c813, 4 },
    { 0x4e87c813, 3 },
    { 0xd0c3c813, 3 },
    { 0x6f7fc810, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944bc81c, 3 },
    { 0x71cfc81d, 3 },
    { 0xf6fbc81d, 3 },
    { 0x5be3c81f, 3 },
    { 0xa72bc820, 3 },
    { 0xebb7c820, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebfc824, 3 },
    { 0, 0 },
    { 0xcc37c826, 3 },
    { 0x2d7c827, 3 },
    { 0, 0 },
    { 0xdcb3c829, 4 },
    { 0xb9abc82a, 3 },
    { 0x614fc82b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1307c831, 3 },
    { 0xa46fc831, 3 },
    { 0x57b3c833, 3 },
    { 0x7ab7c834, 3 },
    { 0x636bc832, 3 },
    { 0x25efc836, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aafc839, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc3c83d, 4 },
    { 0, 0 },
    { 0x988bc83f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b57c842, 4 },
    { 0xb7d7c843, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b7c849, 4 },
    { 0, 0 },
    { 0x22cfc84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1063c850, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb57c854, 3 },
    { 0xaf1bc854, 3 },
    { 0x15f7c856, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eefc85e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3c861, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a3c864, 3 },
    { 0, 0 },
    { 0xa1dbc866, 3 },
    { 0, 0 },
    { 0xd1f3c868, 3 },
    { 0, 0 },
    { 0x3837c86a, 3 },
    { 0x822bc86a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa687c86f, 3 },
    { 0xe38fc870, 3 },
    { 0, 0 },
    { 0x1fcbc872, 4 },
    { 0x5087c873, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb997c877, 4 },
    { 0x7a03c878, 4 },
    { 0xfd9bc878, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbfc87d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d3c882, 3 },
    { 0, 0 },
    { 0x2f77c884, 3 },
    { 0, 0 },
    { 0xb6cbc886, 3 },
    { 0x24efc887, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42fc88b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3113c88e, 3 },
    { 0xc187c88f, 3 },
    { 0xebf7c890, 3 },
    { 0xb8b3c890, 5 },
    { 0xc51fc890, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d7c89b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1857c8a4, 3 },
    { 0, 0 },
    { 0x5b1bc8a6, 4 },
    { 0xf923c8a6, 4 },
    { 0x403bc8a8, 5 },
    { 0xca5fc8a6, 4 },
    { 0xf433c8aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413c8b3, 3 },
    { 0x74afc8b3, 3 },
    { 0xd263c8b5, 4 },
    { 0xe7fc8b6, 4 },
    { 0x4cdbc8b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704fc8bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3bc8be, 4 },
    { 0x30afc8bf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b3c8c5, 3 },
    { 0x7b0bc8c6, 3 },
    { 0x9b3fc8c7, 3 },
    { 0xd0fbc8c8, 4 },
    { 0x395fc8c8, 3 },
    { 0x4b5fc8ca, 4 },
    { 0xba8fc8c8, 3 },
    { 0x158fc8c7, 3 },
    { 0x1ff3c8cd, 3 },
    { 0x835bc8cd, 3 },
    { 0, 0 },
    { 0xdd97c8d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e3c8d4, 3 },
    { 0xf0bbc8d5, 4 },
    { 0x86bbc8d6, 3 },
    { 0x385fc8d5, 4 },
    { 0x644bc8d5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6813c8dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77fc8df, 3 },
    { 0xbf33c8df, 4 },
    { 0xe387c8e1, 3 },
    { 0xcd07c8e2, 3 },
    { 0xabffc8e3, 3 },
    { 0x4d37c8e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bfc8e7, 4 },
    { 0xbd5fc8e8, 4 },
    { 0x4777c8e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573c8ed, 3 },
    { 0xdaefc8ed, 3 },
    { 0xe73c8ef, 3 },
    { 0, 0 },
    { 0x1a4fc8f1, 4 },
    { 0x9b17c8f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ebc8f7, 4 },
    { 0, 0 },
    { 0x640fc8f9, 3 },
    { 0x96e3c8fa, 3 },
    { 0x625bc8fa, 3 },
    { 0, 0 },
    { 0xaad3c8fd, 3 },
    { 0x325bc8fe, 3 },
    { 0, 0 },
    { 0x8087c900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df3c906, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5577c90a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efc90d, 4 },
    { 0xd897c90d, 3 },
    { 0x7fafc90f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a7c912, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c7c916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccbc919, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4493c923, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bfc92d, 3 },
    { 0, 0 },
    { 0x107c92f, 3 },
    { 0, 0 },
    { 0xb4cfc931, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8113c939, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde7c943, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5853c948, 3 },
    { 0xf757c949, 3 },
    { 0x33fc94a, 3 },
    { 0x3db3c94b, 3 },
    { 0x172fc94c, 3 },
    { 0x776fc94d, 3 },
    { 0, 0 },
    { 0x8a7fc94f, 5 },
    { 0, 0 },
    { 0xf593c951, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1823c954, 5 },
    { 0xe33c955, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cbc95c, 4 },
    { 0, 0 },
    { 0x949fc95e, 4 },
    { 0, 0 },
    { 0x313c960, 3 },
    { 0x95a7c961, 3 },
    { 0x434fc962, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x124bc965, 4 },
    { 0, 0 },
    { 0xb587c967, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ffc96b, 3 },
    { 0, 0 },
    { 0xc06fc96d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc317c970, 4 },
    { 0x5c07c970, 3 },
    { 0x5353c970, 3 },
    { 0x9703c972, 3 },
    { 0xdd0fc974, 3 },
    { 0, 0 },
    { 0x1e53c976, 3 },
    { 0x469bc977, 3 },
    { 0x3623c978, 4 },
    { 0x98afc979, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7fc97d, 3 },
    { 0xdca3c97e, 3 },
    { 0xcd9fc97e, 4 },
    { 0, 0 },
    { 0xe0b7c981, 3 },
    { 0, 0 },
    { 0xa6bc983, 3 },
    { 0xe837c983, 4 },
    { 0xf3afc985, 3 },
    { 0x445bc985, 3 },
    { 0x95c7c986, 3 },
    { 0, 0 },
    { 0x5e5fc989, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9bc990, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafafc994, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc33c999, 3 },
    { 0x402bc99a, 3 },
    { 0x9213c99a, 3 },
    { 0x980bc99c, 3 },
    { 0x9c5bc99d, 4 },
    { 0x19afc99e, 4 },
    { 0xe1abc99f, 3 },
    { 0, 0 },
    { 0x1afbc9a1, 5 },
    { 0, 0 },
    { 0xc3bfc9a3, 3 },
    { 0x7fb7c9a4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ffc9a7, 3 },
    { 0, 0 },
    { 0x3fe3c9a9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860fc9ad, 4 },
    { 0xcd9bc9ae, 3 },
    { 0xe91fc9ad, 3 },
    { 0, 0 },
    { 0xd53c9b1, 4 },
    { 0, 0 },
    { 0xc12fc9b3, 3 },
    { 0, 0 },
    { 0x8ebbc9b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be3c9bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b7c9bf, 3 },
    { 0x198fc9bf, 3 },
    { 0x85b7c9c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bbc9c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6917c9cb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ebc9ce, 4 },
    { 0xe343c9cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da7c9d7, 3 },
    { 0xd513c9d8, 4 },
    { 0x7107c9d8, 3 },
    { 0, 0 },
    { 0xa7b3c9db, 3 },
    { 0x7a23c9dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b17c9e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8bc9ed, 4 },
    { 0xe92bc9ee, 3 },
    { 0xcd1fc9ef, 3 },
    { 0x3427c9f0, 3 },
    { 0x5f83c9f1, 3 },
    { 0x89fbc9f1, 3 },
    { 0x942fc9f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x102bc9f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca17c9fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644fca00, 4 },
    { 0xca6bca01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5bca05, 3 },
    { 0xa5e3ca06, 3 },
    { 0, 0 },
    { 0xb1bbca08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d93ca18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc083ca1c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7157ca20, 3 },
    { 0x75ffca21, 4 },
    { 0x9cb3ca22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b17ca26, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7ca2b, 4 },
    { 0xa8afca2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22fca3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85bca40, 3 },
    { 0x46f7ca41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90fca45, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac93ca49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f7ca4c, 3 },
    { 0, 0 },
    { 0x91b7ca4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fca52, 4 },
    { 0x8d27ca52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe053ca5c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd753ca66, 3 },
    { 0x46e7ca67, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb767ca6a, 3 },
    { 0x227bca6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6653ca75, 3 },
    { 0x77abca75, 3 },
    { 0, 0 },
    { 0x7987ca78, 3 },
    { 0, 0 },
    { 0x31a7ca7a, 3 },
    { 0xf7d7ca7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbca7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddbca82, 4 },
    { 0xd717ca83, 4 },
    { 0xbd9bca83, 4 },
    { 0, 0 },
    { 0x9afca86, 4 },
    { 0, 0 },
    { 0x9187ca88, 4 },
    { 0x3d8bca88, 3 },
    { 0, 0 },
    { 0xf7fca8b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2203ca8e, 3 },
    { 0x1ba7ca8f, 4 },
    { 0xb0d7ca8f, 3 },
    { 0x5e5bca90, 3 },
    { 0x8de3ca92, 4 },
    { 0x5b7fca93, 3 },
    { 0x2b1fca91, 4 },
    { 0x1d9bca90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e53ca9e, 3 },
    { 0, 0 },
    { 0xd703caa0, 4 },
    { 0xdf4fcaa0, 4 },
    { 0x32afcaa2, 3 },
    { 0xefa3caa3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab7caaa, 3 },
    { 0, 0 },
    { 0x5fcaac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f97cabb, 3 },
    { 0, 0 },
    { 0x4947cabd, 3 },
    { 0x1cfbcabd, 4 },
    { 0x76abcabe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b7cac2, 3 },
    { 0x9fabcac2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4373cac6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8fcac9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003cacf, 3 },
    { 0, 0 },
    { 0xd45bcad1, 3 },
    { 0, 0 },
    { 0x7ffcad3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fcada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bfcae7, 3 },
    { 0, 0 },
    { 0x7abcae9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f43caec, 3 },
    { 0, 0 },
    { 0x2fabcaee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43caf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfcaf7, 3 },
    { 0, 0 },
    { 0xd6b3caf9, 4 },
    { 0x7b9fcaf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa743cb01, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999fcb05, 4 },
    { 0, 0 },
    { 0xe193cb07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecfcb0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfbcb11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fbcb14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ebcb1a, 3 },
    { 0, 0 },
    { 0x884fcb1c, 3 },
    { 0x786fcb1d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7cb21, 3 },
    { 0x453bcb21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7cb25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816bcb29, 3 },
    { 0xe9efcb2a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871fcb35, 4 },
    { 0xaebcb35, 3 },
    { 0xf763cb35, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0bcb3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97fcb3f, 3 },
    { 0xbfcfcb40, 3 },
    { 0xd513cb41, 4 },
    { 0x1eefcb42, 3 },
    { 0xce53cb41, 4 },
    { 0x3f1bcb43, 3 },
    { 0x8a3bcb42, 3 },
    { 0xb91bcb42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60bcb49, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fcb52, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6387cb5d, 3 },
    { 0xdc2bcb5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a63cb61, 3 },
    { 0xad13cb61, 4 },
    { 0xd4f3cb63, 3 },
    { 0x80ffcb62, 4 },
    { 0, 0 },
    { 0x6517cb66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675bcb6b, 3 },
    { 0, 0 },
    { 0x4263cb6d, 3 },
    { 0x4d7fcb6e, 4 },
    { 0xa8fbcb6f, 4 },
    { 0xa85fcb6d, 3 },
    { 0x624bcb71, 3 },
    { 0x38b7cb72, 3 },
    { 0, 0 },
    { 0x1463cb74, 3 },
    { 0xd0bcb75, 3 },
    { 0x252bcb75, 3 },
    { 0x986fcb77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ebcb85, 4 },
    { 0x428fcb86, 4 },
    { 0x2ce3cb87, 3 },
    { 0, 0 },
    { 0xb403cb89, 3 },
    { 0x2093cb8a, 3 },
    { 0x3e27cb8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2bcb8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7bcb92, 3 },
    { 0xdbabcb93, 3 },
    { 0, 0 },
    { 0x443bcb95, 3 },
    { 0, 0 },
    { 0x376bcb97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3633cb9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7787cba1, 3 },
    { 0, 0 },
    { 0xe5fbcba3, 3 },
    { 0x2147cba4, 3 },
    { 0x97dbcba4, 3 },
    { 0x5f3bcba6, 3 },
    { 0x13abcba7, 3 },
    { 0x5413cba8, 4 },
    { 0x1b63cba9, 3 },
    { 0x40d3cba9, 3 },
    { 0xbdffcbab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x415fcbae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe563cbb3, 3 },
    { 0, 0 },
    { 0x661fcbb5, 4 },
    { 0x34ebcbb6, 4 },
    { 0, 0 },
    { 0x43d7cbb8, 4 },
    { 0x912fcbb9, 4 },
    { 0xd413cbba, 4 },
    { 0x289bcbbb, 3 },
    { 0xa107cbbb, 4 },
    { 0xe7a3cbbc, 4 },
    { 0xcba3cbba, 3 },
    { 0, 0 },
    { 0xa8e7cbc0, 3 },
    { 0x6b17cbc1, 3 },
    { 0x767fcbc2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1fcbc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e7cbca, 3 },
    { 0x3cbcbcb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33cbd1, 3 },
    { 0, 0 },
    { 0x77dbcbd3, 3 },
    { 0x8fb7cbd4, 3 },
    { 0, 0 },
    { 0x488fcbd6, 3 },
    { 0, 0 },
    { 0xb703cbd8, 3 },
    { 0, 0 },
    { 0x973bcbda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd7cbe2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53cbe6, 4 },
    { 0x9d4fcbe7, 4 },
    { 0x30dbcbe7, 4 },
    { 0x3307cbe6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc83cbec, 3 },
    { 0x734bcbec, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe357cbf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549bcc03, 3 },
    { 0x4a8fcc04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebbcc07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3413cc0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949fcc13, 4 },
    { 0, 0 },
    { 0xdc47cc15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b7cc18, 3 },
    { 0xe67fcc19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bbcc1d, 3 },
    { 0xd9abcc1e, 3 },
    { 0, 0 },
    { 0x9c5bcc20, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7cc23, 3 },
    { 0, 0 },
    { 0x75ebcc25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc3cc29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aebcc2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e3cc33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf03cc36, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754fcc3b, 3 },
    { 0xc76bcc3b, 3 },
    { 0x1bdfcc3c, 3 },
    { 0xa81bcc3c, 3 },
    { 0x1c7cc3f, 3 },
    { 0x807bcc40, 3 },
    { 0xa8ffcc40, 3 },
    { 0, 0 },
    { 0x8e3bcc43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc223cc48, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23fcc4d, 3 },
    { 0x585bcc4e, 3 },
    { 0xd9b7cc4f, 3 },
    { 0xe30fcc4d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce3cc55, 3 },
    { 0, 0 },
    { 0x90c7cc57, 3 },
    { 0xb50bcc58, 3 },
    { 0xdb3bcc59, 3 },
    { 0x254bcc5a, 3 },
    { 0xf317cc5a, 3 },
    { 0x4297cc59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e3cc60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3343cc64, 3 },
    { 0xfdbfcc65, 3 },
    { 0, 0 },
    { 0xde9bcc67, 3 },
    { 0x17bfcc67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a83cc6c, 4 },
    { 0, 0 },
    { 0x466bcc6e, 3 },
    { 0, 0 },
    { 0xcb7fcc70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557bcc74, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be3cc7a, 3 },
    { 0, 0 },
    { 0x211bcc7c, 3 },
    { 0xfa67cc7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b7cc83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9fcc89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03cc93, 3 },
    { 0xd50fcc94, 4 },
    { 0x59dfcc94, 3 },
    { 0, 0 },
    { 0xa553cc97, 3 },
    { 0, 0 },
    { 0x89a7cc99, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bcc9c, 3 },
    { 0, 0 },
    { 0x9c63cc9e, 4 },
    { 0x3e8fcc9f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b7cca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe733cca7, 3 },
    { 0, 0 },
    { 0x954bcca9, 3 },
    { 0xc587ccaa, 3 },
    { 0, 0 },
    { 0x533bccac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe27ccaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f3ccb2, 4 },
    { 0x1473ccb3, 3 },
    { 0xa593ccb2, 3 },
    { 0x765fccb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280bccc1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5327ccc6, 4 },
    { 0x9b3bccc6, 3 },
    { 0x41c3ccc8, 3 },
    { 0x1abfccc8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9fcccc, 5 },
    { 0x755bcccc, 3 },
    { 0x9a2fcccc, 3 },
    { 0x3c03cccd, 3 },
    { 0xd563ccd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dfccd3, 5 },
    { 0, 0 },
    { 0x68e7ccd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c7ccd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41fccdb, 3 },
    { 0, 0 },
    { 0xe20fccdd, 3 },
    { 0xbe7bccdd, 4 },
    { 0, 0 },
    { 0xae7bcce0, 3 },
    { 0, 0 },
    { 0xa37cce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377cce9, 3 },
    { 0xafbbccea, 3 },
    { 0x5f27cceb, 3 },
    { 0x48b7ccec, 3 },
    { 0xe81bcceb, 3 },
    { 0xb833ccee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33ccf3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cfccfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbccff, 3 },
    { 0xcebfccff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf693cd03, 3 },
    { 0x2bffcd03, 3 },
    { 0xd88bcd05, 3 },
    { 0xaa3bcd06, 5 },
    { 0x4e0fcd03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21bcd12, 4 },
    { 0x7a53cd12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9fcd16, 3 },
    { 0xf21bcd17, 3 },
    { 0xd56fcd17, 3 },
    { 0x7073cd19, 3 },
    { 0, 0 },
    { 0x932fcd1b, 3 },
    { 0x3517cd1c, 4 },
    { 0xcd6bcd1d, 3 },
    { 0, 0 },
    { 0xc00bcd1f, 3 },
    { 0x821bcd20, 4 },
    { 0, 0 },
    { 0x658fcd22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc683cd2b, 4 },
    { 0xd5f7cd2c, 3 },
    { 0x6c43cd2d, 3 },
    { 0, 0 },
    { 0x6843cd2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3fcd36, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f03cd40, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334bcd44, 3 },
    { 0xdc63cd45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bbcd52, 4 },
    { 0x6377cd52, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6bcd58, 3 },
    { 0, 0 },
    { 0xa04fcd5a, 3 },
    { 0xb3e3cd5b, 4 },
    { 0xefcfcd5b, 3 },
    { 0xd973cd5d, 3 },
    { 0x50dfcd5d, 4 },
    { 0x99cbcd5e, 4 },
    { 0xff6fcd5d, 3 },
    { 0xc72bcd61, 3 },
    { 0xbb6fcd61, 3 },
    { 0x2febcd63, 3 },
    { 0x9607cd63, 4 },
    { 0x481fcd64, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dbcd69, 4 },
    { 0x1f3fcd69, 3 },
    { 0, 0 },
    { 0xfc57cd6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa653cd6f, 4 },
    { 0xc957cd70, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b57cd75, 3 },
    { 0, 0 },
    { 0x22afcd77, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbfcd7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e7cd7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bcd86, 4 },
    { 0, 0 },
    { 0xcacfcd88, 3 },
    { 0, 0 },
    { 0xd5a7cd8a, 3 },
    { 0xce0bcd8b, 3 },
    { 0x43b3cd8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277cd90, 3 },
    { 0, 0 },
    { 0xb3abcd92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7fcd9b, 3 },
    { 0xabbbcd9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2473cda1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3cda5, 4 },
    { 0x396fcda6, 3 },
    { 0x2d23cda5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbcdac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3cdb2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1077cdb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1abcdb9, 3 },
    { 0, 0 },
    { 0x9aabcdbb, 3 },
    { 0, 0 },
    { 0xf6d3cdbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97bcdc9, 4 },
    { 0x987cdca, 3 },
    { 0, 0 },
    { 0xdf37cdcc, 3 },
    { 0x7a6fcdcd, 3 },
    { 0x8ffbcdcc, 3 },
    { 0x8bcfcdcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x247bcdd2, 3 },
    { 0x5f37cdd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf907cdd8, 3 },
    { 0x7c43cdd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4007cddd, 3 },
    { 0xb127cddd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcde1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253cdeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecffcdf2, 3 },
    { 0xd30bcdf3, 4 },
    { 0xb2cfcdf4, 3 },
    { 0xf7cfcdf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4843cdf9, 3 },
    { 0xed97cdfa, 3 },
    { 0, 0 },
    { 0x30e3cdfc, 4 },
    { 0, 0 },
    { 0xda5fcdfe, 3 },
    { 0x3747cdff, 4 },
    { 0x63abcdfe, 3 },
    { 0, 0 },
    { 0xfc97ce02, 4 },
    { 0x534fce02, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd7ce09, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bce0c, 4 },
    { 0x647ce0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5287ce11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x460fce14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273fce19, 4 },
    { 0x41d7ce19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ce1f, 3 },
    { 0x6127ce20, 4 },
    { 0xe513ce21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e23ce24, 3 },
    { 0, 0 },
    { 0xa34fce26, 4 },
    { 0x6dcbce27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27ce2e, 3 },
    { 0, 0 },
    { 0x5167ce30, 3 },
    { 0xf443ce31, 3 },
    { 0, 0 },
    { 0x34cfce33, 3 },
    { 0x34ffce34, 4 },
    { 0xc6efce34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885bce3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbbce3e, 3 },
    { 0, 0 },
    { 0xa123ce40, 3 },
    { 0, 0 },
    { 0x6d8bce42, 3 },
    { 0x7c57ce43, 3 },
    { 0, 0 },
    { 0x4c17ce45, 3 },
    { 0x6ffce46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dfce4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe93ce56, 4 },
    { 0, 0 },
    { 0x9e3ce58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9febce5c, 4 },
    { 0x50dbce5d, 3 },
    { 0x964bce5e, 3 },
    { 0, 0 },
    { 0x8563ce60, 4 },
    { 0x596bce61, 3 },
    { 0x1be7ce61, 3 },
    { 0, 0 },
    { 0x6617ce64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99fce69, 3 },
    { 0x7adbce6a, 3 },
    { 0x5387ce6b, 3 },
    { 0x59c7ce69, 3 },
    { 0x30ebce6d, 3 },
    { 0, 0 },
    { 0xdaebce6f, 3 },
    { 0xcc7fce70, 4 },
    { 0, 0 },
    { 0x602bce72, 3 },
    { 0x1b27ce73, 3 },
    { 0x56d7ce73, 3 },
    { 0, 0 },
    { 0x55fce76, 3 },
    { 0x4937ce76, 3 },
    { 0x9017ce78, 3 },
    { 0xd553ce76, 3 },
    { 0x7fbfce7a, 3 },
    { 0xc98fce77, 3 },
    { 0xb873ce7c, 3 },
    { 0x71b3ce79, 3 },
    { 0x9f13ce7e, 3 },
    { 0xdef7ce7c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13ce90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3ce93, 4 },
    { 0, 0 },
    { 0xd9ffce95, 3 },
    { 0x8fb3ce96, 3 },
    { 0x2aefce96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd3ce9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf843ce9f, 5 },
    { 0x7cd7ce9f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6827cea3, 4 },
    { 0xf3d3cea4, 3 },
    { 0x68afcea4, 3 },
    { 0, 0 },
    { 0x88b3cea7, 3 },
    { 0, 0 },
    { 0x448fcea9, 3 },
    { 0, 0 },
    { 0xfd47ceab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc7ceaf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328bceb7, 3 },
    { 0x7047ceb8, 5 },
    { 0x1437ceb7, 3 },
    { 0xe5bfceb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e87cebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a27cec1, 4 },
    { 0x1367cec2, 3 },
    { 0x4453cec2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b3cec7, 3 },
    { 0, 0 },
    { 0x1ef3cec9, 3 },
    { 0, 0 },
    { 0x534bcecb, 3 },
    { 0, 0 },
    { 0xf083cecd, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abced7, 3 },
    { 0xd073ced8, 3 },
    { 0x2223ced8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445bcedd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2bcee0, 3 },
    { 0, 0 },
    { 0xb10fcee2, 3 },
    { 0x9743cee2, 3 },
    { 0x9cb3cee4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b7ceed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d07cef2, 3 },
    { 0xa4efcef3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f3cef8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7657cefb, 4 },
    { 0x2d97cefb, 3 },
    { 0x587bcefd, 4 },
    { 0x6563cefd, 3 },
    { 0x15dbceff, 3 },
    { 0xbf03cefd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7527cf03, 3 },
    { 0x62ffcf03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1bcf0b, 3 },
    { 0, 0 },
    { 0xe897cf0d, 4 },
    { 0x3ebbcf0e, 3 },
    { 0, 0 },
    { 0x7fbcf10, 3 },
    { 0x44d7cf11, 3 },
    { 0x4b77cf10, 4 },
    { 0xc31fcf13, 4 },
    { 0xb0dfcf14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bcf17, 3 },
    { 0, 0 },
    { 0x34f7cf19, 3 },
    { 0x93cf1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbbcf1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f83cf25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa3cf2e, 3 },
    { 0, 0 },
    { 0x66b7cf30, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6bcf36, 5 },
    { 0, 0 },
    { 0x71ebcf38, 3 },
    { 0xeadbcf38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e7cf3c, 3 },
    { 0x28c7cf3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1187cf43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c37cf4c, 3 },
    { 0x1dabcf4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8fcf51, 4 },
    { 0x1f17cf52, 3 },
    { 0xebfbcf53, 3 },
    { 0, 0 },
    { 0x4177cf55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2fcf58, 3 },
    { 0, 0 },
    { 0xdffcf5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x175fcf5d, 3 },
    { 0x250bcf5e, 4 },
    { 0xa99fcf5f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe7cf63, 4 },
    { 0xbb8fcf63, 3 },
    { 0xf0e7cf64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x837bcf68, 3 },
    { 0x8ba3cf69, 3 },
    { 0xbae3cf68, 3 },
    { 0x3d67cf6b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8363cf6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ffcf73, 4 },
    { 0x9f9fcf74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe3cf79, 3 },
    { 0xc063cf79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x359bcf7d, 4 },
    { 0x16e7cf7e, 3 },
    { 0x32f3cf7e, 4 },
    { 0xd2bbcf7f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726fcf89, 3 },
    { 0, 0 },
    { 0x3d3cf8b, 3 },
    { 0xd437cf8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc253cf93, 3 },
    { 0x5f7fcf93, 4 },
    { 0xd293cf95, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371fcfa0, 3 },
    { 0xd067cfa1, 3 },
    { 0x9a97cfa1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e3cfa6, 4 },
    { 0xc7cfcfa7, 3 },
    { 0x2a1fcfa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3cfad, 4 },
    { 0xdbabcfad, 3 },
    { 0x56b7cfaf, 5 },
    { 0x4813cfb0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fcfb6, 5 },
    { 0, 0 },
    { 0x106bcfb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebcfbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8fcfc1, 4 },
    { 0x54bfcfc2, 3 },
    { 0xf8ebcfc3, 3 },
    { 0xc313cfc3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903cfc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d7cfcd, 3 },
    { 0x4cfcfcd, 4 },
    { 0x973fcfcf, 3 },
    { 0xf157cfd0, 4 },
    { 0xa4f7cfd1, 3 },
    { 0, 0 },
    { 0x6383cfd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c53cfd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfbcfdc, 3 },
    { 0x1987cfdc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bfcfe0, 3 },
    { 0, 0 },
    { 0xe8afcfe2, 4 },
    { 0x707bcfe3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8673cfe6, 4 },
    { 0x9e53cfe7, 3 },
    { 0, 0 },
    { 0xa9a3cfe9, 3 },
    { 0x301fcfea, 3 },
    { 0xf207cfeb, 3 },
    { 0x597cfec, 4 },
    { 0, 0 },
    { 0x90bbcfee, 3 },
    { 0xbb9bcfef, 4 },
    { 0, 0 },
    { 0x1bc3cff1, 4 },
    { 0x8623cff1, 3 },
    { 0, 0 },
    { 0x28b3cff4, 4 },
    { 0, 0 },
    { 0x4043cff6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x121fcff9, 3 },
    { 0xfc23cffa, 3 },
    { 0xf62fcffb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229bcfff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95bd003, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c63d00b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02fd00e, 3 },
    { 0xa177d00f, 3 },
    { 0xa8d3d010, 3 },
    { 0xe25fd011, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f7d015, 4 },
    { 0, 0 },
    { 0x4f4bd017, 3 },
    { 0x71b3d018, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3d01f, 3 },
    { 0, 0 },
    { 0x4bb7d021, 3 },
    { 0, 0 },
    { 0x6d1fd023, 3 },
    { 0xaee3d024, 3 },
    { 0, 0 },
    { 0x4367d026, 4 },
    { 0, 0 },
    { 0x88fbd028, 3 },
    { 0, 0 },
    { 0x83fd02a, 3 },
    { 0x4c07d02b, 3 },
    { 0x5427d02c, 3 },
    { 0xa95bd02d, 3 },
    { 0x6a5fd02b, 4 },
    { 0xf00bd02a, 4 },
    { 0, 0 },
    { 0x17abd031, 4 },
    { 0xccfbd032, 3 },
    { 0x750bd032, 3 },
    { 0, 0 },
    { 0xa293d035, 3 },
    { 0x8777d036, 3 },
    { 0x10c3d037, 3 },
    { 0, 0 },
    { 0xb207d039, 3 },
    { 0x8bebd03a, 3 },
    { 0x12cfd03b, 3 },
    { 0xaf2fd03c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3d045, 3 },
    { 0, 0 },
    { 0xebb3d047, 3 },
    { 0xb1cfd048, 4 },
    { 0x46c7d048, 4 },
    { 0xd3dbd048, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2863d04d, 3 },
    { 0x55cfd04e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe3d052, 3 },
    { 0x8047d053, 4 },
    { 0x92afd054, 4 },
    { 0xe3ffd055, 3 },
    { 0xce9bd056, 3 },
    { 0xecd3d056, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8abd05e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa737d061, 3 },
    { 0, 0 },
    { 0xdc7bd063, 3 },
    { 0xc8a7d064, 3 },
    { 0x14b3d065, 4 },
    { 0, 0 },
    { 0xce23d067, 3 },
    { 0x9b3d068, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288fd06d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1bd075, 3 },
    { 0xcbf7d076, 3 },
    { 0, 0 },
    { 0x123bd078, 3 },
    { 0x6fd7d079, 3 },
    { 0x669bd07a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595fd081, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccfd084, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4577d087, 3 },
    { 0, 0 },
    { 0xe54bd089, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2bd08c, 3 },
    { 0x936bd08c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143d092, 3 },
    { 0xd86fd093, 3 },
    { 0xb62bd092, 3 },
    { 0x595bd095, 3 },
    { 0xfa13d096, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3d099, 4 },
    { 0xd943d09a, 3 },
    { 0x428fd09b, 3 },
    { 0xd8afd09c, 3 },
    { 0xf91bd09c, 3 },
    { 0xf0abd09e, 3 },
    { 0x8d6fd09d, 3 },
    { 0x5a2fd09c, 3 },
    { 0x996bd0a1, 3 },
    { 0x6a73d0a1, 4 },
    { 0x991bd0a3, 3 },
    { 0xe5bd0a0, 3 },
    { 0, 0 },
    { 0x727bd0a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7d0a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e7d0ad, 3 },
    { 0, 0 },
    { 0xd2e7d0af, 4 },
    { 0xe23fd0af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x690bd0b3, 3 },
    { 0xf1d7d0b4, 3 },
    { 0x68b7d0b5, 3 },
    { 0xc12bd0b4, 3 },
    { 0xedbbd0b3, 4 },
    { 0x4383d0b8, 3 },
    { 0x225fd0b9, 3 },
    { 0x15b7d0b8, 3 },
    { 0x602bd0b8, 3 },
    { 0x717fd0ba, 3 },
    { 0x4be7d0bd, 5 },
    { 0x8a0bd0b4, 3 },
    { 0x11b3d0bf, 4 },
    { 0, 0 },
    { 0x7783d0c1, 3 },
    { 0x820bd0c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bbd0c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x761bd0ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91bd0ce, 3 },
    { 0, 0 },
    { 0x6b23d0d0, 4 },
    { 0x847fd0d1, 3 },
    { 0x8d3d0d1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b37d0d7, 3 },
    { 0x51a3d0d8, 4 },
    { 0x8b5bd0d8, 3 },
    { 0x6bafd0da, 4 },
    { 0xa1b7d0db, 3 },
    { 0x2e77d0da, 3 },
    { 0, 0 },
    { 0x17e7d0de, 3 },
    { 0x5d53d0df, 3 },
    { 0x88c3d0df, 3 },
    { 0, 0 },
    { 0xdcfd0e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f63d0e7, 3 },
    { 0x51e7d0e8, 4 },
    { 0x7aa7d0e9, 3 },
    { 0, 0 },
    { 0x6f7fd0eb, 4 },
    { 0, 0 },
    { 0x6d27d0ed, 3 },
    { 0, 0 },
    { 0x774fd0ef, 4 },
    { 0, 0 },
    { 0xbd9fd0f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413d0f6, 3 },
    { 0x6fcfd0f7, 3 },
    { 0x8b4fd0f6, 3 },
    { 0xcafd0f9, 3 },
    { 0, 0 },
    { 0xaacbd0fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bfd104, 3 },
    { 0xa23fd104, 3 },
    { 0xae5fd105, 3 },
    { 0xde27d106, 3 },
    { 0x54e3d108, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91bd10b, 3 },
    { 0, 0 },
    { 0x4043d10d, 3 },
    { 0x2c0fd10e, 3 },
    { 0x2b9bd10e, 3 },
    { 0xda53d10f, 3 },
    { 0xb87d10d, 3 },
    { 0xd7d7d112, 3 },
    { 0xdd7fd113, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa753d118, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f97d120, 3 },
    { 0, 0 },
    { 0x8ec3d122, 4 },
    { 0x4b87d123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487d127, 3 },
    { 0x6affd127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3637d12c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cbd12f, 3 },
    { 0x7de3d130, 4 },
    { 0x4bc7d130, 3 },
    { 0x7b37d132, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de7d13c, 5 },
    { 0xfa17d13c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3fd142, 3 },
    { 0x4a77d143, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113bd148, 3 },
    { 0xdcc3d149, 4 },
    { 0, 0 },
    { 0x692fd14b, 3 },
    { 0, 0 },
    { 0x95abd14d, 3 },
    { 0, 0 },
    { 0x44e3d14f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37d154, 3 },
    { 0x58f7d154, 3 },
    { 0xb167d154, 4 },
    { 0x576bd157, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff7d15b, 3 },
    { 0x7697d15c, 4 },
    { 0x7407d15d, 3 },
    { 0x2863d15e, 3 },
    { 0xa77bd15e, 3 },
    { 0, 0 },
    { 0x3c57d161, 3 },
    { 0x1953d161, 3 },
    { 0xcbcbd161, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13fd167, 3 },
    { 0x5fd3d168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa227d16b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6627d170, 3 },
    { 0x960bd170, 3 },
    { 0xe22fd170, 3 },
    { 0, 0 },
    { 0xe507d174, 3 },
    { 0, 0 },
    { 0x120bd176, 3 },
    { 0xdbbd177, 3 },
    { 0, 0 },
    { 0xd777d179, 3 },
    { 0xa67bd17a, 3 },
    { 0, 0 },
    { 0xc933d17c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2693d181, 3 },
    { 0xccdfd182, 3 },
    { 0x84afd183, 3 },
    { 0x4e6bd184, 4 },
    { 0x9fb7d184, 4 },
    { 0x3f1fd185, 4 },
    { 0xef8bd187, 3 },
    { 0x5aefd185, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdbd18d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589bd192, 4 },
    { 0, 0 },
    { 0x670fd194, 3 },
    { 0x117d194, 3 },
    { 0x4cefd196, 3 },
    { 0xa84fd197, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8247d19a, 4 },
    { 0x4a0bd19b, 3 },
    { 0xc457d19c, 3 },
    { 0x1b3bd19d, 3 },
    { 0xa77fd19a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb3d1a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x596bd1a6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd533d1b3, 5 },
    { 0xed0bd1b4, 3 },
    { 0xa43d1b3, 3 },
    { 0xd47bd1b3, 3 },
    { 0x8ebbd1b7, 4 },
    { 0x8ebd1b5, 3 },
    { 0x7b73d1b9, 3 },
    { 0x271fd1b4, 3 },
    { 0xf4fd1bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6fd1c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ebd1c5, 3 },
    { 0x4f6fd1c6, 3 },
    { 0, 0 },
    { 0x9debd1c8, 3 },
    { 0x2617d1c8, 5 },
    { 0x89a7d1c9, 3 },
    { 0x44b3d1cb, 3 },
    { 0xb23d1cc, 3 },
    { 0x682fd1cd, 3 },
    { 0x5393d1ce, 3 },
    { 0x55e7d1cf, 5 },
    { 0x47e3d1d0, 3 },
    { 0x2d0bd1d1, 4 },
    { 0xf3afd1d2, 3 },
    { 0x7a07d1c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827fd1db, 3 },
    { 0, 0 },
    { 0x6993d1dd, 4 },
    { 0x7dd7d1de, 4 },
    { 0x14d7d1de, 3 },
    { 0xcc3d1df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3517d1e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaabd1eb, 4 },
    { 0x7c7bd1eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2bd1f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe33d1f6, 3 },
    { 0x8eabd1f7, 3 },
    { 0xa617d1f7, 3 },
    { 0x9cd3d1f6, 3 },
    { 0x4c13d1fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x574bd1fd, 3 },
    { 0x7bc3d1fd, 4 },
    { 0, 0 },
    { 0xce37d200, 3 },
    { 0xf9a3d201, 3 },
    { 0x6813d202, 3 },
    { 0, 0 },
    { 0xfc7bd204, 3 },
    { 0x4c5fd204, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917d210, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd3d215, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dfd219, 3 },
    { 0x27e3d21a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fd220, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc653d223, 3 },
    { 0x279fd223, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1677d228, 4 },
    { 0x1bb3d228, 3 },
    { 0xb0bbd228, 4 },
    { 0, 0 },
    { 0xe693d22c, 3 },
    { 0x7d47d22c, 3 },
    { 0x34dbd22e, 3 },
    { 0xbdc3d22d, 3 },
    { 0xfc9bd230, 3 },
    { 0x955fd231, 3 },
    { 0xd7dfd231, 3 },
    { 0xc77bd233, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7543d23a, 3 },
    { 0x65d3d23a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2087d23e, 3 },
    { 0, 0 },
    { 0x25efd240, 3 },
    { 0, 0 },
    { 0xeb77d242, 3 },
    { 0x5d3fd243, 3 },
    { 0x93ffd242, 3 },
    { 0xc9e3d242, 3 },
    { 0x5f73d246, 3 },
    { 0xf057d245, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453d254, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d7d25b, 3 },
    { 0x4567d25c, 3 },
    { 0xe2fd25c, 3 },
    { 0x8c57d25e, 4 },
    { 0, 0 },
    { 0x73c7d260, 3 },
    { 0x4da3d261, 3 },
    { 0xc3dbd261, 3 },
    { 0x7497d262, 3 },
    { 0x307d264, 3 },
    { 0xeecbd264, 4 },
    { 0x216fd265, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dbd26a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32afd26e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed83d271, 4 },
    { 0, 0 },
    { 0x8513d273, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd278, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6fd27c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c3d280, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x907d283, 3 },
    { 0x1537d284, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bd287, 3 },
    { 0, 0 },
    { 0xda93d289, 3 },
    { 0x88ebd28a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7d292, 3 },
    { 0xeff3d293, 3 },
    { 0, 0 },
    { 0xd31fd295, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abfd29c, 3 },
    { 0x2373d29c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2473d2a1, 3 },
    { 0xe43fd2a2, 3 },
    { 0x88fbd2a3, 3 },
    { 0, 0 },
    { 0xf6ffd2a5, 4 },
    { 0x1943d2a6, 4 },
    { 0, 0 },
    { 0xf3dbd2a8, 4 },
    { 0, 0 },
    { 0x7b53d2aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb73d2af, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77bd2b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7fd2b5, 4 },
    { 0x61c7d2b6, 4 },
    { 0, 0 },
    { 0x925bd2b8, 3 },
    { 0x41f3d2b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eabd2be, 3 },
    { 0, 0 },
    { 0x88bd2c0, 3 },
    { 0, 0 },
    { 0x7a4fd2c2, 3 },
    { 0x9e17d2c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a7d2ce, 3 },
    { 0, 0 },
    { 0xda53d2d0, 3 },
    { 0, 0 },
    { 0x9cbbd2d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767bd2d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc263d2da, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8587d2e3, 5 },
    { 0x20a7d2e4, 3 },
    { 0x6613d2e4, 3 },
    { 0x5b57d2e6, 3 },
    { 0x9cabd2e6, 3 },
    { 0x2c43d2e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fd2ed, 3 },
    { 0x1197d2ee, 3 },
    { 0x857fd2ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbfd2f2, 3 },
    { 0, 0 },
    { 0x87bd2f4, 3 },
    { 0x7b73d2f4, 3 },
    { 0xaddfd2f6, 4 },
    { 0, 0 },
    { 0x32cbd2f8, 3 },
    { 0, 0 },
    { 0x6ac7d2fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d27d2fd, 3 },
    { 0x487fd2fe, 3 },
    { 0xd4ffd2fd, 4 },
    { 0, 0 },
    { 0x2cb7d301, 3 },
    { 0x54a7d302, 4 },
    { 0x6b43d302, 3 },
    { 0xb313d304, 4 },
    { 0x1b9bd305, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5babd309, 3 },
    { 0xba4fd309, 3 },
    { 0xaf57d30b, 3 },
    { 0, 0 },
    { 0x4183d30d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9513d312, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ffd315, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239bd31a, 3 },
    { 0xb6fd31b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x771bd31e, 3 },
    { 0xf783d31e, 3 },
    { 0x568fd320, 3 },
    { 0x68e3d321, 3 },
    { 0xe723d31f, 4 },
    { 0x5ffd31f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc017d32e, 4 },
    { 0xfe3bd32f, 3 },
    { 0x5b2bd330, 3 },
    { 0xa787d331, 3 },
    { 0xabc7d32f, 3 },
    { 0x9573d333, 3 },
    { 0x18cfd334, 3 },
    { 0x1df3d335, 3 },
    { 0x1e0bd336, 3 },
    { 0xcd0bd336, 3 },
    { 0x69bfd330, 3 },
    { 0, 0 },
    { 0x2747d33a, 3 },
    { 0, 0 },
    { 0x9d37d33c, 3 },
    { 0xcb57d33d, 3 },
    { 0x28f3d33d, 3 },
    { 0x15b7d33e, 3 },
    { 0xd4efd340, 3 },
    { 0x631bd341, 3 },
    { 0, 0 },
    { 0x3a87d343, 3 },
    { 0x231bd343, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d47d347, 3 },
    { 0xb63d347, 3 },
    { 0x7883d349, 3 },
    { 0xd6b7d34a, 3 },
    { 0xdd7bd349, 3 },
    { 0x81d7d347, 3 },
    { 0xd907d34d, 3 },
    { 0, 0 },
    { 0x42b3d34f, 3 },
    { 0, 0 },
    { 0x441bd351, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3d355, 3 },
    { 0xe85fd355, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c7d35f, 3 },
    { 0, 0 },
    { 0xd777d361, 3 },
    { 0, 0 },
    { 0x9eabd363, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e3d367, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b3d36f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fbd377, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745fd37b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3653d380, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2bd385, 4 },
    { 0xb053d386, 4 },
    { 0xfef7d387, 3 },
    { 0xa3a3d388, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ebd38f, 3 },
    { 0x148bd390, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987d395, 3 },
    { 0xc26bd395, 3 },
    { 0x152bd397, 3 },
    { 0x7e2fd398, 4 },
    { 0xc10bd399, 3 },
    { 0, 0 },
    { 0x525fd39b, 4 },
    { 0xc50fd39b, 4 },
    { 0xc25fd39d, 3 },
    { 0x4277d39e, 4 },
    { 0x94a3d39d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5267d3a2, 3 },
    { 0x66f7d3a3, 3 },
    { 0x43b3d3a4, 3 },
    { 0x70c3d3a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725bd3ab, 4 },
    { 0x5a47d3ac, 3 },
    { 0xe443d3ab, 3 },
    { 0x98dfd3ac, 3 },
    { 0x4ab3d3af, 3 },
    { 0xfc37d3ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7d3bc, 3 },
    { 0xb51fd3bd, 3 },
    { 0, 0 },
    { 0xcec3d3bf, 3 },
    { 0x8287d3c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37fd3c5, 4 },
    { 0x8dd3d3c6, 3 },
    { 0x90bfd3c7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba67d3ce, 3 },
    { 0x6acfd3ce, 3 },
    { 0x4a67d3d0, 3 },
    { 0x29efd3d1, 3 },
    { 0, 0 },
    { 0x4a27d3d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b7d3d6, 4 },
    { 0x81b3d3d7, 3 },
    { 0x422fd3d8, 3 },
    { 0x57efd3d8, 4 },
    { 0xb9b3d3d8, 3 },
    { 0xe29bd3db, 3 },
    { 0xdc57d3da, 5 },
    { 0, 0 },
    { 0xbf2bd3de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69fd3e7, 4 },
    { 0x6c67d3e7, 4 },
    { 0, 0 },
    { 0x20e7d3ea, 3 },
    { 0xa683d3eb, 3 },
    { 0x3f5bd3ea, 4 },
    { 0x29d3d3ed, 3 },
    { 0, 0 },
    { 0x6c67d3ef, 3 },
    { 0x9137d3ef, 3 },
    { 0x2303d3f1, 3 },
    { 0x447fd3f2, 3 },
    { 0xe7d3f2, 4 },
    { 0x53dfd3f1, 3 },
    { 0xb46bd3f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697d3f8, 3 },
    { 0xe3bbd3f9, 3 },
    { 0, 0 },
    { 0x27a7d3fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf393d3ff, 3 },
    { 0x3e13d3ff, 4 },
    { 0xd7cfd401, 3 },
    { 0xc6efd401, 4 },
    { 0x185fd403, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389bd408, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd40b, 3 },
    { 0x3093d40c, 4 },
    { 0xb73fd40d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ebd412, 3 },
    { 0, 0 },
    { 0x437fd414, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x199bd417, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d03d41b, 3 },
    { 0xfefbd41c, 3 },
    { 0x14f3d41c, 4 },
    { 0x10e7d41e, 3 },
    { 0xe34fd41e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4fd422, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccfd425, 4 },
    { 0x2997d425, 3 },
    { 0x1b4fd427, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed53d42c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd42f, 3 },
    { 0, 0 },
    { 0x61f3d431, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab73d435, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e33d438, 4 },
    { 0xc5abd439, 3 },
    { 0x5353d43a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fbd43d, 4 },
    { 0x6277d43e, 4 },
    { 0x11f3d43e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01bd442, 3 },
    { 0x145fd442, 3 },
    { 0xc327d444, 4 },
    { 0xeb67d445, 3 },
    { 0x636fd446, 3 },
    { 0x23d3d442, 3 },
    { 0xf743d446, 3 },
    { 0x7d17d449, 4 },
    { 0x93dfd445, 3 },
    { 0xa4bd44b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d7d44f, 3 },
    { 0x44bd450, 3 },
    { 0, 0 },
    { 0xbeebd452, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d7d456, 3 },
    { 0x946bd457, 3 },
    { 0xbe93d458, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f87d45b, 3 },
    { 0, 0 },
    { 0xb54fd45d, 4 },
    { 0x66f3d45e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cbd462, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24fd468, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379fd46d, 3 },
    { 0x6417d46d, 4 },
    { 0x3c73d46f, 3 },
    { 0x453bd46e, 3 },
    { 0, 0 },
    { 0x71a7d472, 3 },
    { 0x1987d473, 3 },
    { 0x564fd473, 5 },
    { 0xd3d474, 3 },
    { 0xe7b3d476, 3 },
    { 0x8b17d476, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cfd47a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bd47f, 4 },
    { 0x97b7d47f, 3 },
    { 0x6037d480, 3 },
    { 0xb9fbd482, 3 },
    { 0x1707d482, 3 },
    { 0xf40fd481, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a7d487, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8213d48c, 4 },
    { 0xa347d48d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2613d490, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bd497, 3 },
    { 0, 0 },
    { 0x34dfd499, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647fd49f, 3 },
    { 0x5effd4a0, 3 },
    { 0x8053d4a0, 3 },
    { 0xb733d4a2, 3 },
    { 0xc633d4a3, 4 },
    { 0x8963d4a3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bfd4ad, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7d4b0, 4 },
    { 0x8ab3d4b1, 3 },
    { 0xd0e3d4b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9967d4ba, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3d4c2, 3 },
    { 0xe5bd4c3, 4 },
    { 0, 0 },
    { 0x2d63d4c5, 3 },
    { 0x2a6fd4c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d47d4cc, 3 },
    { 0x396fd4cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a13d4d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cbd4d3, 3 },
    { 0xe0cbd4d4, 4 },
    { 0xdc67d4d4, 4 },
    { 0x2987d4d3, 4 },
    { 0xa333d4d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b07d4e1, 3 },
    { 0xcdd3d4e1, 4 },
    { 0, 0 },
    { 0x35bd4e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc023d4e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365bd4f0, 3 },
    { 0x9d4fd4f0, 3 },
    { 0, 0 },
    { 0x4a53d4f3, 3 },
    { 0x7723d4f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7d4f9, 4 },
    { 0x2f5bd4fa, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xd493d4fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a57d502, 3 },
    { 0xe49bd502, 4 },
    { 0x744bd504, 3 },
    { 0x57bbd503, 3 },
    { 0x8e13d504, 4 },
    { 0, 0 },
    { 0x7053d508, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68bd50c, 3 },
    { 0, 0 },
    { 0x7fcfd50e, 3 },
    { 0, 0 },
    { 0xb043d510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4087d51b, 3 },
    { 0, 0 },
    { 0x9affd51d, 3 },
    { 0, 0 },
    { 0x29ebd51f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917fd526, 3 },
    { 0xb10fd527, 3 },
    { 0xc6dbd526, 4 },
    { 0x178fd529, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6efd52c, 3 },
    { 0, 0 },
    { 0xb857d52e, 3 },
    { 0xa87bd52f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246fd534, 4 },
    { 0x7aefd535, 4 },
    { 0, 0 },
    { 0xcde7d537, 3 },
    { 0xf5dbd538, 3 },
    { 0x452fd539, 3 },
    { 0xcac7d538, 4 },
    { 0x12efd539, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9fd542, 3 },
    { 0x863bd543, 3 },
    { 0x59fd542, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d73d549, 4 },
    { 0x33abd54a, 3 },
    { 0x63efd549, 3 },
    { 0xc837d54c, 3 },
    { 0, 0 },
    { 0x77f3d54e, 3 },
    { 0x873d54e, 3 },
    { 0xf77bd54e, 3 },
    { 0, 0 },
    { 0xca47d552, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3fd557, 3 },
    { 0xb0d7d558, 3 },
    { 0, 0 },
    { 0x7f43d55a, 3 },
    { 0x76bbd55b, 4 },
    { 0xa6efd55c, 4 },
    { 0xb977d55c, 3 },
    { 0x83abd55d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6febd561, 3 },
    { 0xfa53d561, 3 },
    { 0, 0 },
    { 0x3937d564, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xad73d567, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7d56a, 3 },
    { 0, 0 },
    { 0x51fbd56c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221bd571, 3 },
    { 0x5263d572, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f3d579, 4 },
    { 0, 0 },
    { 0xb187d57b, 3 },
    { 0xb077d57b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22fd57f, 3 },
    { 0, 0 },
    { 0xd16fd581, 3 },
    { 0, 0 },
    { 0x2fc7d583, 3 },
    { 0x9193d584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375fd588, 4 },
    { 0, 0 },
    { 0x9b87d58a, 3 },
    { 0xda9fd58b, 4 },
    { 0xc4c7d58a, 3 },
    { 0x6fdfd58a, 3 },
    { 0x33abd58a, 4 },
    { 0x5633d58f, 3 },
    { 0xb047d58d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3d594, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18bd597, 3 },
    { 0xa983d598, 4 },
    { 0x681fd599, 3 },
    { 0, 0 },
    { 0x441bd59b, 3 },
    { 0, 0 },
    { 0x1c03d59d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d3d5a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d3d5a5, 3 },
    { 0x6697d5a6, 3 },
    { 0x917d5a6, 3 },
    { 0, 0 },
    { 0xa1cbd5a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb7d5af, 3 },
    { 0x4d4fd5af, 4 },
    { 0xee7fd5b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cfd5bb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2fd5c0, 3 },
    { 0x55bd5c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2bd5c8, 3 },
    { 0x495fd5c8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7793d5d3, 4 },
    { 0x180bd5d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14fd5d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bfd5db, 4 },
    { 0, 0 },
    { 0x46a7d5dd, 3 },
    { 0xef4fd5dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bfd5e1, 3 },
    { 0xabf7d5e2, 3 },
    { 0x16dfd5e1, 4 },
    { 0xc7a3d5e2, 4 },
    { 0xe303d5e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf97d5e9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x873fd5ec, 4 },
    { 0xeae7d5ed, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb413d5f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4383d5f5, 3 },
    { 0, 0 },
    { 0x1aefd5f7, 3 },
    { 0x621fd5f8, 3 },
    { 0x763d5f9, 4 },
    { 0x6ed3d5fa, 3 },
    { 0x67a7d5f8, 3 },
    { 0xa363d5f7, 3 },
    { 0x7cbfd5fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e3d601, 3 },
    { 0, 0 },
    { 0xc3f3d603, 3 },
    { 0x5483d604, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228fd60c, 3 },
    { 0xe37bd60d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb983d611, 4 },
    { 0x4bcbd612, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cbd617, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7d622, 3 },
    { 0, 0 },
    { 0x37dfd624, 3 },
    { 0, 0 },
    { 0x7e93d626, 4 },
    { 0, 0 },
    { 0xd107d628, 4 },
    { 0x482bd629, 3 },
    { 0xe4bfd628, 3 },
    { 0x1a83d62b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5bd634, 3 },
    { 0xdd2fd635, 3 },
    { 0x7b67d634, 4 },
    { 0xd43d635, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3d63b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33d640, 4 },
    { 0x677d641, 3 },
    { 0xbc3d642, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b3d647, 4 },
    { 0, 0 },
    { 0xbdb3d649, 4 },
    { 0x4de7d649, 5 },
    { 0, 0 },
    { 0x82bfd64c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bd652, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ffd655, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77d65d, 3 },
    { 0x35cbd65e, 3 },
    { 0x882bd65f, 4 },
    { 0, 0 },
    { 0x298fd661, 3 },
    { 0xe9dfd662, 3 },
    { 0xc3bfd663, 3 },
    { 0x1a73d664, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaafd667, 4 },
    { 0x2b57d668, 3 },
    { 0, 0 },
    { 0x6d03d66a, 3 },
    { 0, 0 },
    { 0xd53bd66c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf777d66f, 4 },
    { 0xb77bd670, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c13d673, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6047d677, 3 },
    { 0xf6f7d678, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb3d67b, 3 },
    { 0xe9bfd67b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x298fd67f, 3 },
    { 0x483fd680, 3 },
    { 0xe6abd67f, 3 },
    { 0xfb67d680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d87d685, 3 },
    { 0xc69fd686, 3 },
    { 0x170fd687, 4 },
    { 0xa5f3d688, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7fd68e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf957d694, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fd699, 4 },
    { 0xf447d69a, 4 },
    { 0x4c7fd69b, 3 },
    { 0xbcf7d69c, 4 },
    { 0x871bd69d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e13d6a2, 3 },
    { 0xa50bd6a3, 3 },
    { 0xfd6a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25bd6a8, 3 },
    { 0x157d6a9, 4 },
    { 0x276fd6aa, 3 },
    { 0x4473d6ab, 3 },
    { 0x8e9fd6ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ffd6af, 3 },
    { 0x9407d6af, 3 },
    { 0x8cc7d6b1, 3 },
    { 0x8e9fd6b2, 3 },
    { 0x1903d6b1, 3 },
    { 0, 0 },
    { 0xc0a7d6b5, 4 },
    { 0xb463d6b6, 3 },
    { 0, 0 },
    { 0x319fd6b8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e7d6bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bfd6c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defd6cc, 3 },
    { 0x65d3d6cd, 3 },
    { 0, 0 },
    { 0xb0abd6cf, 3 },
    { 0, 0 },
    { 0xf43fd6d1, 4 },
    { 0, 0 },
    { 0xcf93d6d3, 3 },
    { 0x6243d6d4, 3 },
    { 0x27fd6d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bbd6db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5abd6de, 3 },
    { 0xf4dfd6df, 4 },
    { 0, 0 },
    { 0x925bd6e1, 4 },
    { 0, 0 },
    { 0x411fd6e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6457d6e6, 3 },
    { 0xa8b7d6e6, 4 },
    { 0xb337d6e8, 4 },
    { 0x830fd6e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29fd6ec, 3 },
    { 0x6fb7d6ed, 3 },
    { 0, 0 },
    { 0xf0fbd6ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23d6f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3fd6f8, 3 },
    { 0x370bd6f8, 4 },
    { 0x15f7d6fa, 3 },
    { 0, 0 },
    { 0x3b47d6fc, 4 },
    { 0xd6dfd6fd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d3d703, 3 },
    { 0x2d33d704, 3 },
    { 0xf2c7d705, 3 },
    { 0x7e73d703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7d70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2fd712, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdfd715, 3 },
    { 0xbc93d716, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430bd71c, 3 },
    { 0x7643d71d, 3 },
    { 0xfa3bd71e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8bd722, 3 },
    { 0, 0 },
    { 0x9257d724, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460fd72a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c3d730, 3 },
    { 0, 0 },
    { 0x93e7d732, 5 },
    { 0, 0 },
    { 0x13dbd734, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe093d739, 4 },
    { 0x6007d73a, 3 },
    { 0, 0 },
    { 0x5f9bd73c, 3 },
    { 0, 0 },
    { 0xa1ebd73e, 4 },
    { 0, 0 },
    { 0x926fd740, 4 },
    { 0xa407d741, 3 },
    { 0xcaabd742, 3 },
    { 0x5347d743, 3 },
    { 0, 0 },
    { 0x58c7d745, 4 },
    { 0x334bd745, 4 },
    { 0xa637d745, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96fd750, 3 },
    { 0, 0 },
    { 0x26e7d752, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3353d759, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x649fd75c, 3 },
    { 0xead3d75c, 3 },
    { 0, 0 },
    { 0x970fd75f, 3 },
    { 0xc807d75f, 3 },
    { 0xbb23d761, 3 },
    { 0xc723d762, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3d765, 3 },
    { 0x1387d766, 3 },
    { 0x79cfd767, 4 },
    { 0x45dbd768, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bd76e, 3 },
    { 0, 0 },
    { 0x950bd770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa17d773, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f3d77c, 3 },
    { 0, 0 },
    { 0xf95bd77e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233d781, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x489bd784, 3 },
    { 0x46e3d784, 3 },
    { 0xf963d786, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe617d789, 3 },
    { 0xdb9bd78a, 3 },
    { 0, 0 },
    { 0x179fd78c, 3 },
    { 0, 0 },
    { 0xd9b7d78e, 3 },
    { 0x5007d78f, 3 },
    { 0xf4c3d78e, 3 },
    { 0, 0 },
    { 0xfa47d792, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5bd795, 3 },
    { 0x6d73d796, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fbd79b, 4 },
    { 0x9a6bd79c, 3 },
    { 0x142bd79d, 3 },
    { 0x8117d79e, 3 },
    { 0x61b3d79b, 3 },
    { 0, 0 },
    { 0xbc93d7a1, 4 },
    { 0x20e3d7a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33d7a5, 3 },
    { 0, 0 },
    { 0x1447d7a7, 3 },
    { 0xf14fd7a7, 3 },
    { 0, 0 },
    { 0xb59fd7aa, 4 },
    { 0x7efbd7ab, 3 },
    { 0xb5bd7ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecfd7b5, 4 },
    { 0x5487d7b5, 4 },
    { 0x5597d7b6, 5 },
    { 0x5197d7b8, 3 },
    { 0x972bd7b9, 3 },
    { 0, 0 },
    { 0xf83d7bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9abd7bf, 3 },
    { 0xf01bd7c0, 3 },
    { 0xb1a3d7c1, 4 },
    { 0xbc9bd7c2, 4 },
    { 0xceb7d7c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a7d7d4, 4 },
    { 0, 0 },
    { 0x762fd7d6, 3 },
    { 0x191fd7d7, 3 },
    { 0xc777d7d8, 3 },
    { 0x1d87d7d6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2723d7e1, 5 },
    { 0, 0 },
    { 0xe63fd7e3, 3 },
    { 0x1163d7e4, 3 },
    { 0x655bd7e5, 3 },
    { 0xe143d7e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57bd7ea, 4 },
    { 0, 0 },
    { 0x57a7d7ec, 3 },
    { 0xf5fbd7ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787d7f1, 3 },
    { 0x82bbd7f2, 4 },
    { 0xcc53d7f1, 3 },
    { 0x5effd7f2, 3 },
    { 0x247bd7f5, 3 },
    { 0xe083d7f6, 3 },
    { 0x1043d7f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424bd7fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb143d800, 4 },
    { 0xbf0fd800, 3 },
    { 0xdc23d802, 3 },
    { 0xe52fd803, 3 },
    { 0xab4bd802, 3 },
    { 0x2d57d803, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec53d809, 5 },
    { 0xfd9bd80a, 3 },
    { 0, 0 },
    { 0x69cbd80c, 4 },
    { 0, 0 },
    { 0x52bd80e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc417d815, 3 },
    { 0xaa93d816, 3 },
    { 0xa593d816, 4 },
    { 0xf61fd818, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882bd81f, 4 },
    { 0xc8e7d81f, 3 },
    { 0x401fd820, 3 },
    { 0, 0 },
    { 0x2b4fd823, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x575fd826, 3 },
    { 0, 0 },
    { 0x69dbd828, 3 },
    { 0xb0dfd829, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161fd831, 4 },
    { 0x2fb3d832, 3 },
    { 0x180bd833, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bd837, 4 },
    { 0x7b87d838, 4 },
    { 0x9913d839, 3 },
    { 0xf727d838, 3 },
    { 0x3c3d838, 5 },
    { 0xd603d839, 3 },
    { 0, 0 },
    { 0xd1d3d83e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d57d843, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfbd846, 3 },
    { 0xcd7d847, 4 },
    { 0xa9cfd848, 3 },
    { 0x3bfd849, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dbd850, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d854, 4 },
    { 0x2e57d854, 3 },
    { 0x9cfd856, 4 },
    { 0, 0 },
    { 0x39e3d858, 4 },
    { 0xff7bd859, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254bd862, 3 },
    { 0x2173d863, 3 },
    { 0, 0 },
    { 0x564bd865, 4 },
    { 0, 0 },
    { 0x90f3d867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cfd86a, 3 },
    { 0, 0 },
    { 0x8ee7d86c, 3 },
    { 0xc907d86c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bbd872, 4 },
    { 0xa323d872, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bfd876, 4 },
    { 0xd153d876, 3 },
    { 0xeb77d877, 3 },
    { 0x6623d879, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbbd87d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953d880, 3 },
    { 0x84fbd881, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfd887, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253d891, 4 },
    { 0xcd9bd892, 4 },
    { 0x82bfd893, 3 },
    { 0xa953d893, 3 },
    { 0xe58bd895, 3 },
    { 0xd0fbd896, 4 },
    { 0x944fd897, 3 },
    { 0x79dfd898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7d89b, 3 },
    { 0x31e3d89c, 3 },
    { 0xf4e7d89c, 3 },
    { 0xdaffd89e, 3 },
    { 0, 0 },
    { 0x737bd8a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d7d8a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63bd8a7, 4 },
    { 0xe20fd8a7, 3 },
    { 0x61a3d8a9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xa237d8ac, 3 },
    { 0x8747d8ad, 3 },
    { 0x1efbd8ad, 4 },
    { 0xf947d8af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3bd8b2, 3 },
    { 0xe7bbd8b2, 4 },
    { 0x98a7d8b4, 3 },
    { 0xc067d8b5, 3 },
    { 0xcbbd8b6, 4 },
    { 0xe8d7d8b5, 3 },
    { 0x99d3d8b8, 3 },
    { 0, 0 },
    { 0x9e7d8ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13d8be, 3 },
    { 0xcf5bd8bf, 3 },
    { 0, 0 },
    { 0xe6abd8c1, 4 },
    { 0x90c3d8c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c73d8c7, 3 },
    { 0x8e33d8c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xac27d8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11fd8d0, 4 },
    { 0xac43d8d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153d8d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd327d8d9, 3 },
    { 0, 0 },
    { 0x4f3bd8db, 4 },
    { 0x32b3d8dc, 3 },
    { 0x2e4bd8dd, 3 },
    { 0x134bd8de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f73d8e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3bd8ed, 3 },
    { 0x5cabd8ee, 3 },
    { 0x43c7d8ef, 3 },
    { 0x3f3bd8ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07d8f4, 3 },
    { 0, 0 },
    { 0x2217d8f6, 4 },
    { 0, 0 },
    { 0x32efd8f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x823d8fb, 3 },
    { 0x836bd8fc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0bd902, 4 },
    { 0, 0 },
    { 0xf4ffd904, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697d90e, 4 },
    { 0x4f7bd90f, 3 },
    { 0xf81fd910, 4 },
    { 0xd61bd910, 3 },
    { 0x3e93d910, 4 },
    { 0xdc77d911, 3 },
    { 0, 0 },
    { 0x1c2fd915, 4 },
    { 0xe81fd916, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813d919, 3 },
    { 0x72abd91a, 4 },
    { 0xc9dbd91b, 3 },
    { 0x51c3d919, 4 },
    { 0, 0 },
    { 0x1ee3d91e, 4 },
    { 0, 0 },
    { 0x5c43d920, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4fd925, 4 },
    { 0x9f87d926, 3 },
    { 0x561fd925, 3 },
    { 0xae8fd927, 3 },
    { 0xee13d925, 3 },
    { 0x2ce3d92a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728bd931, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6fd934, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7383d937, 3 },
    { 0x9effd937, 4 },
    { 0xa417d939, 3 },
    { 0x766bd937, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a3d93e, 4 },
    { 0xf577d93f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433d946, 3 },
    { 0x71efd947, 3 },
    { 0xc7b7d947, 3 },
    { 0x8abfd948, 4 },
    { 0x53fd94a, 3 },
    { 0xd297d946, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6537d951, 3 },
    { 0xc10bd951, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c7d955, 3 },
    { 0, 0 },
    { 0x6dcfd957, 3 },
    { 0x36afd958, 3 },
    { 0x45b3d959, 4 },
    { 0, 0 },
    { 0x7b67d95b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cfd95e, 3 },
    { 0, 0 },
    { 0x4973d960, 4 },
    { 0xdf03d961, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7017d966, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53abd970, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbbd973, 4 },
    { 0, 0 },
    { 0xf157d975, 3 },
    { 0xea57d975, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7d979, 3 },
    { 0x8fdfd97a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8fd987, 3 },
    { 0x11afd988, 3 },
    { 0, 0 },
    { 0xae3bd98a, 3 },
    { 0x503fd98a, 3 },
    { 0, 0 },
    { 0xbefd98d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd3d991, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6fd999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dbd99c, 3 },
    { 0, 0 },
    { 0x687fd99e, 3 },
    { 0, 0 },
    { 0xe8b3d9a0, 3 },
    { 0xaa83d9a1, 4 },
    { 0, 0 },
    { 0xfeafd9a3, 3 },
    { 0, 0 },
    { 0xb1b3d9a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8fd9a8, 3 },
    { 0, 0 },
    { 0x3007d9aa, 3 },
    { 0x26bfd9aa, 3 },
    { 0x6573d9ac, 3 },
    { 0xd6cbd9ad, 3 },
    { 0x5ac3d9ad, 3 },
    { 0x66cbd9ac, 3 },
    { 0, 0 },
    { 0x3a63d9b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ffd9b4, 3 },
    { 0xfa37d9b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb753d9b9, 3 },
    { 0x47efd9ba, 3 },
    { 0, 0 },
    { 0x15bd9bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c7d9bf, 3 },
    { 0x6437d9c0, 4 },
    { 0, 0 },
    { 0xdf1fd9c2, 3 },
    { 0x28e3d9c3, 3 },
    { 0x5e0bd9c3, 3 },
    { 0xb763d9c5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd3d9c9, 3 },
    { 0, 0 },
    { 0x1bbbd9cb, 3 },
    { 0, 0 },
    { 0xf163d9cd, 5 },
    { 0x3927d9ce, 3 },
    { 0, 0 },
    { 0xa957d9d0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6907d9d5, 3 },
    { 0, 0 },
    { 0x2dcbd9d7, 3 },
    { 0x1cd7d9d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d53d9dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3207d9e1, 4 },
    { 0, 0 },
    { 0xa93fd9e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c97d9e7, 3 },
    { 0xfc1bd9e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927d9f3, 3 },
    { 0, 0 },
    { 0x789bd9f5, 3 },
    { 0, 0 },
    { 0xcaafd9f7, 3 },
    { 0x290bd9f8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x219fd9fb, 4 },
    { 0x860bd9fc, 3 },
    { 0x749bd9fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x178bda00, 3 },
    { 0, 0 },
    { 0xa307da02, 3 },
    { 0x65b7da03, 3 },
    { 0x5363da04, 3 },
    { 0, 0 },
    { 0xc2b7da06, 3 },
    { 0x670bda07, 3 },
    { 0, 0 },
    { 0x7e63da09, 3 },
    { 0x7e7fda09, 3 },
    { 0xf59bda0a, 4 },
    { 0x349bda0c, 3 },
    { 0, 0 },
    { 0x81dbda0e, 3 },
    { 0xef4fda0f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da12, 3 },
    { 0, 0 },
    { 0x2897da14, 3 },
    { 0x668fda15, 3 },
    { 0xb073da16, 3 },
    { 0xa14bda16, 4 },
    { 0xa2efda18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3da1f, 4 },
    { 0, 0 },
    { 0x85c3da21, 3 },
    { 0xacefda21, 5 },
    { 0, 0 },
    { 0x318bda24, 3 },
    { 0x6747da25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bbda28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x107bda2b, 4 },
    { 0x525bda2c, 4 },
    { 0xcc83da2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fda30, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bda33, 3 },
    { 0, 0 },
    { 0xdaf3da35, 3 },
    { 0, 0 },
    { 0xc5afda37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b3da3b, 5 },
    { 0x551bda3c, 3 },
    { 0xa56bda3d, 3 },
    { 0, 0 },
    { 0x1e53da3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5167da42, 4 },
    { 0xa1dbda42, 4 },
    { 0x8b2fda44, 3 },
    { 0, 0 },
    { 0x7963da46, 3 },
    { 0, 0 },
    { 0x69d7da48, 3 },
    { 0, 0 },
    { 0xf1bbda4a, 4 },
    { 0, 0 },
    { 0xb247da4c, 3 },
    { 0xbefda4c, 4 },
    { 0xed53da4c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe117da51, 3 },
    { 0x997da51, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa87da55, 3 },
    { 0x1a07da56, 3 },
    { 0x3a33da56, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb7da5a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333da60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05bda64, 4 },
    { 0, 0 },
    { 0x3d9fda66, 3 },
    { 0x3db3da67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x193fda6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a3da6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7da73, 3 },
    { 0x23c7da73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1bda85, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1fda89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dfda9c, 4 },
    { 0, 0 },
    { 0xa6dfda9e, 4 },
    { 0xf1abda9f, 3 },
    { 0xf437daa0, 3 },
    { 0x96ffda9f, 3 },
    { 0x92fdaa2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf993daa9, 3 },
    { 0, 0 },
    { 0x6f3daab, 3 },
    { 0x61e7daac, 3 },
    { 0, 0 },
    { 0x2303daae, 3 },
    { 0, 0 },
    { 0x133dab0, 3 },
    { 0xb597dab0, 3 },
    { 0x587fdab1, 3 },
    { 0x999bdab2, 3 },
    { 0x48cfdab3, 3 },
    { 0x3d6fdab4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dfdaba, 3 },
    { 0, 0 },
    { 0xd713dabc, 3 },
    { 0x8153dabd, 4 },
    { 0, 0 },
    { 0x6d6bdabf, 3 },
    { 0, 0 },
    { 0x43ebdac1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c7dac5, 3 },
    { 0, 0 },
    { 0xc307dac7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45fdace, 3 },
    { 0xa83dacf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bdad8, 5 },
    { 0x3637dad8, 3 },
    { 0x9b87dada, 3 },
    { 0x57c3dada, 5 },
    { 0x3e9fdadc, 3 },
    { 0xd18fdadd, 3 },
    { 0x578bdad8, 3 },
    { 0x2a6bdadf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bdae7, 3 },
    { 0xc657dae8, 3 },
    { 0x8bf3dae7, 4 },
    { 0, 0 },
    { 0x4103daeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71abdaef, 3 },
    { 0xeedbdaf0, 3 },
    { 0, 0 },
    { 0xfc93daf2, 4 },
    { 0, 0 },
    { 0xf87daf4, 4 },
    { 0, 0 },
    { 0x88c7daf6, 3 },
    { 0xed3daf7, 3 },
    { 0xc337daf8, 3 },
    { 0x92dfdaf9, 3 },
    { 0x2893daf9, 3 },
    { 0xbcafdaf9, 4 },
    { 0x5493dafc, 3 },
    { 0x3bbdafd, 4 },
    { 0x571fdaf8, 3 },
    { 0xe8c3daff, 3 },
    { 0xeec7daff, 3 },
    { 0xaca7dafe, 3 },
    { 0xf0cbdb02, 3 },
    { 0x6bf3db03, 3 },
    { 0x7ec7db03, 3 },
    { 0x8787db04, 3 },
    { 0xd10bdb06, 4 },
    { 0x79d7db04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871fdb0c, 3 },
    { 0, 0 },
    { 0xc353db0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bbdb12, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7db17, 3 },
    { 0x391bdb18, 3 },
    { 0, 0 },
    { 0xb8bbdb1a, 3 },
    { 0x5adbdb1b, 4 },
    { 0, 0 },
    { 0x7d0bdb1d, 3 },
    { 0x10b7db1e, 4 },
    { 0xee3db1d, 5 },
    { 0x1e3bdb1f, 3 },
    { 0, 0 },
    { 0xfd2bdb22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe403db26, 3 },
    { 0x521fdb26, 4 },
    { 0x6a33db28, 3 },
    { 0x7487db29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccbdb35, 3 },
    { 0, 0 },
    { 0x22b7db37, 3 },
    { 0, 0 },
    { 0x338bdb39, 5 },
    { 0x5f0fdb39, 3 },
    { 0x8ff3db3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc863db3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbdb42, 4 },
    { 0x9883db43, 4 },
    { 0xffbfdb44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd613db47, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cbdb4a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fafdb54, 3 },
    { 0xb22bdb54, 3 },
    { 0x53dfdb56, 4 },
    { 0xc653db57, 3 },
    { 0x1e5bdb57, 3 },
    { 0xb0b3db56, 3 },
    { 0xd3e3db5a, 3 },
    { 0xdafbdb5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x954bdb5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8613db62, 3 },
    { 0, 0 },
    { 0x6513db64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd37db6a, 3 },
    { 0, 0 },
    { 0x66f3db6c, 3 },
    { 0x173db6c, 3 },
    { 0x913fdb6e, 3 },
    { 0x983db6f, 4 },
    { 0xc4dfdb70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b73db74, 4 },
    { 0, 0 },
    { 0xa5bfdb76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22fdb7a, 5 },
    { 0xee8bdb7b, 3 },
    { 0xa20fdb7b, 3 },
    { 0xc6c3db7d, 3 },
    { 0xd33db7e, 3 },
    { 0, 0 },
    { 0x9acbdb80, 3 },
    { 0xb8fdb81, 3 },
    { 0, 0 },
    { 0x7f4bdb83, 3 },
    { 0, 0 },
    { 0xb937db85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3efdb88, 3 },
    { 0x714fdb89, 3 },
    { 0x7e3bdb8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ffdb96, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7bdb9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7bdb9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543bdba2, 3 },
    { 0xe57bdba3, 3 },
    { 0x601fdba4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af3dba7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809bdbb0, 3 },
    { 0xc7efdbb0, 3 },
    { 0, 0 },
    { 0x4dd3dbb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8bdbb7, 3 },
    { 0x456fdbb7, 3 },
    { 0x2117dbb7, 3 },
    { 0xb22bdbba, 4 },
    { 0x1f7dbba, 3 },
    { 0xa777dbbb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd043dbc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de3dbc9, 3 },
    { 0xb2fdbca, 4 },
    { 0xb2afdbca, 4 },
    { 0x11bfdbc9, 3 },
    { 0, 0 },
    { 0x39ffdbce, 3 },
    { 0xd95fdbce, 3 },
    { 0, 0 },
    { 0x4b37dbd1, 3 },
    { 0x50bbdbd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dafdbd5, 3 },
    { 0x7eafdbd5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50bdbdc, 3 },
    { 0x4e83dbdc, 4 },
    { 0xff9fdbde, 3 },
    { 0, 0 },
    { 0xada3dbe0, 3 },
    { 0, 0 },
    { 0xf37dbe2, 3 },
    { 0, 0 },
    { 0x5b4fdbe4, 3 },
    { 0xc8a7dbe5, 3 },
    { 0xa957dbe5, 4 },
    { 0x638bdbe5, 3 },
    { 0, 0 },
    { 0x2e13dbe9, 3 },
    { 0, 0 },
    { 0x31dfdbeb, 3 },
    { 0, 0 },
    { 0x8b13dbed, 3 },
    { 0xacb3dbee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb27dbf6, 3 },
    { 0x839fdbf7, 3 },
    { 0x745fdbf7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae7dbfd, 3 },
    { 0, 0 },
    { 0x453fdbff, 5 },
    { 0x5863dc00, 3 },
    { 0x15a7dc01, 3 },
    { 0, 0 },
    { 0xe5f7dc03, 3 },
    { 0x4fffdc03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ebdc0c, 3 },
    { 0x55fbdc0c, 3 },
    { 0x181bdc0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd83dc14, 3 },
    { 0, 0 },
    { 0x4fdbdc16, 3 },
    { 0x2853dc17, 3 },
    { 0xd5c3dc17, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x162bdc1b, 3 },
    { 0x2393dc1c, 3 },
    { 0x358bdc1c, 4 },
    { 0x91f7dc1e, 3 },
    { 0x414bdc1d, 4 },
    { 0x7ca7dc1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90fdc28, 4 },
    { 0x9ea3dc28, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626bdc2e, 4 },
    { 0x4b97dc2f, 3 },
    { 0, 0 },
    { 0x132fdc31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64bdc37, 4 },
    { 0xaae7dc38, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5073dc3e, 3 },
    { 0, 0 },
    { 0xe587dc40, 3 },
    { 0, 0 },
    { 0x73fdc42, 4 },
    { 0, 0 },
    { 0x9be7dc44, 3 },
    { 0, 0 },
    { 0x51a7dc46, 3 },
    { 0x43cfdc46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7dc4e, 3 },
    { 0, 0 },
    { 0xe383dc50, 3 },
    { 0, 0 },
    { 0x71ebdc52, 4 },
    { 0xba2bdc53, 3 },
    { 0xff5fdc53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ebdc5a, 3 },
    { 0x5437dc5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd27dc5f, 3 },
    { 0x82fdc60, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1357dc64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1637dc67, 3 },
    { 0x335fdc67, 3 },
    { 0, 0 },
    { 0x6c97dc6a, 3 },
    { 0xc833dc6a, 3 },
    { 0x9677dc6b, 3 },
    { 0x6987dc6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba67dc71, 4 },
    { 0, 0 },
    { 0x1bf7dc73, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x535bdc76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806fdc7a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x167dc7d, 3 },
    { 0x7e53dc7d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x52efdc81, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dfdc85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f7dc89, 3 },
    { 0x2903dc89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bdc94, 3 },
    { 0, 0 },
    { 0xf35fdc96, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb287dc9a, 4 },
    { 0, 0 },
    { 0xc7b3dc9c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8333dca2, 4 },
    { 0x47f3dca3, 3 },
    { 0x73f7dca4, 3 },
    { 0, 0 },
    { 0x4a77dca6, 3 },
    { 0, 0 },
    { 0xb2e7dca8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83bdcab, 4 },
    { 0xef2bdcab, 3 },
    { 0x148fdcac, 3 },
    { 0xbd3fdcae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913dcb5, 3 },
    { 0, 0 },
    { 0x6d33dcb7, 3 },
    { 0x5bc3dcb8, 3 },
    { 0x2d77dcb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236fdcc2, 3 },
    { 0, 0 },
    { 0xce33dcc4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93dcc8, 3 },
    { 0, 0 },
    { 0x9837dcca, 3 },
    { 0xa37bdcca, 3 },
    { 0x2f9bdccc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da3dcd0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1023dcd4, 3 },
    { 0x3e2bdcd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945bdcdf, 3 },
    { 0xcde7dce0, 3 },
    { 0, 0 },
    { 0x1947dce2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85fdce6, 3 },
    { 0x3ce3dce6, 3 },
    { 0xdf4bdce8, 3 },
    { 0, 0 },
    { 0x596bdcea, 3 },
    { 0xf8bbdceb, 3 },
    { 0xb0cfdceb, 3 },
    { 0xea0fdced, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fdcf3, 3 },
    { 0xf78fdcf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc3dcfb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb083dd06, 4 },
    { 0, 0 },
    { 0xcedbdd08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9077dd0b, 3 },
    { 0xce6bdd0b, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671bdd11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fdd14, 3 },
    { 0xbcafdd14, 3 },
    { 0xf5a7dd16, 3 },
    { 0x615fdd17, 3 },
    { 0xd9cbdd18, 3 },
    { 0x2f2fdd19, 4 },
    { 0x4fb7dd19, 3 },
    { 0x3dc3dd1b, 4 },
    { 0xa41bdd1c, 3 },
    { 0x65b3dd1d, 3 },
    { 0x307dd1b, 3 },
    { 0xc077dd1d, 3 },
    { 0x3823dd20, 3 },
    { 0xb4dfdd18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1587dd24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3dd27, 4 },
    { 0x9f23dd28, 3 },
    { 0x3eebdd28, 3 },
    { 0, 0 },
    { 0xf7a7dd2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2493dd31, 3 },
    { 0xee47dd31, 3 },
    { 0, 0 },
    { 0x7133dd34, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x852fdd37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ffdd3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324bdd40, 4 },
    { 0x296bdd40, 3 },
    { 0x1033dd42, 3 },
    { 0x60efdd43, 3 },
    { 0x7257dd44, 3 },
    { 0xecf3dd44, 3 },
    { 0xaeebdd43, 3 },
    { 0, 0 },
    { 0xea27dd48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd53dd4b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da7dd4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7397dd51, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff3dd5a, 3 },
    { 0, 0 },
    { 0xb173dd5c, 3 },
    { 0x559bdd5d, 3 },
    { 0xb0cfdd5d, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26afdd65, 4 },
    { 0xb873dd66, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e17dd6b, 3 },
    { 0xabffdd6c, 3 },
    { 0x22cbdd6c, 3 },
    { 0x1dc3dd6e, 3 },
    { 0x2917dd6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f3dd74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057dd77, 3 },
    { 0xa77bdd78, 4 },
    { 0, 0 },
    { 0x41ffdd7a, 3 },
    { 0, 0 },
    { 0x2cb7dd7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567dd82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc047dd88, 3 },
    { 0, 0 },
    { 0x6557dd8a, 3 },
    { 0x892bdd8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6023dd8e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7487dd91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6487dd94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5bdd99, 3 },
    { 0xaa07dd9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfdd9d, 3 },
    { 0, 0 },
    { 0x4bdfdd9f, 3 },
    { 0xf833dda0, 3 },
    { 0, 0 },
    { 0xd12fdda2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb697ddac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2453ddb0, 3 },
    { 0, 0 },
    { 0xe91bddb2, 3 },
    { 0x1687ddb3, 3 },
    { 0x10ebddb3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920fddb9, 3 },
    { 0xd50bddba, 3 },
    { 0xd903ddbb, 3 },
    { 0x5da7ddba, 4 },
    { 0x1f57ddb9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adbddc2, 3 },
    { 0xa3bbddc2, 5 },
    { 0x58ebddc2, 3 },
    { 0x6183ddc2, 3 },
    { 0xf5c3ddc4, 3 },
    { 0, 0 },
    { 0xb703ddc8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecfddcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4bddd7, 3 },
    { 0xfd3bddd8, 4 },
    { 0x16cfddd8, 4 },
    { 0x4c5fddda, 3 },
    { 0x960bdddb, 4 },
    { 0xf12bdddc, 4 },
    { 0x99bdddd, 4 },
    { 0xb763dddd, 3 },
    { 0x671fddda, 3 },
    { 0xc913dde0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d53dde3, 3 },
    { 0, 0 },
    { 0xc55bdde5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cbddeb, 3 },
    { 0x1d03ddec, 3 },
    { 0xb30fdded, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ffddf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf67ddf5, 4 },
    { 0x1d8fddf5, 4 },
    { 0, 0 },
    { 0xf5dfddf8, 3 },
    { 0x1e2bddf9, 3 },
    { 0x4ef3ddf9, 3 },
    { 0, 0 },
    { 0xa6afddfc, 3 },
    { 0xa87bddfd, 3 },
    { 0x3423ddfe, 3 },
    { 0xc223ddfc, 3 },
    { 0x7cfddfc, 3 },
    { 0xc1a3de01, 3 },
    { 0x98bfde02, 3 },
    { 0x95c3de02, 4 },
    { 0x571fde04, 4 },
    { 0xadffde03, 3 },
    { 0x39fbde06, 5 },
    { 0x14d7de04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x930fde0a, 3 },
    { 0x663de0b, 3 },
    { 0x711fde0b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a7de10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53de13, 4 },
    { 0xbe3bde14, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fbde1b, 4 },
    { 0x6a8bde1b, 3 },
    { 0, 0 },
    { 0x1defde1e, 4 },
    { 0x4cb7de1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512bde23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xad47de26, 4 },
    { 0, 0 },
    { 0xf467de28, 4 },
    { 0xa6bfde29, 4 },
    { 0x6ae3de29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff27de32, 3 },
    { 0x3403de33, 3 },
    { 0x113de32, 3 },
    { 0x6fabde35, 4 },
    { 0xe7fde35, 3 },
    { 0xd99fde37, 3 },
    { 0x3a13de38, 3 },
    { 0x22a7de34, 3 },
    { 0xf31bde3a, 4 },
    { 0x4cf7de3a, 4 },
    { 0xb433de32, 3 },
    { 0x97bfde3d, 3 },
    { 0x1d6fde3d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983de42, 3 },
    { 0xf1b7de42, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433de47, 3 },
    { 0, 0 },
    { 0xab43de49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63de4c, 4 },
    { 0x7edbde4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dfde50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8bde53, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b7de56, 3 },
    { 0x611bde57, 3 },
    { 0x9177de58, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177de5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85bde5e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x152fde61, 3 },
    { 0x16b7de62, 3 },
    { 0, 0 },
    { 0xa87fde64, 3 },
    { 0x5cfbde65, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bde68, 3 },
    { 0xcfbbde69, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e43de6e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd243de71, 3 },
    { 0xe883de71, 3 },
    { 0x2183de73, 4 },
    { 0x83e7de73, 3 },
    { 0, 0 },
    { 0x39a3de76, 3 },
    { 0xcadfde77, 3 },
    { 0xdb8fde78, 4 },
    { 0x60b7de79, 4 },
    { 0x502fde78, 4 },
    { 0, 0 },
    { 0x1767de7c, 3 },
    { 0xbac3de7c, 3 },
    { 0x88d3de7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a3de81, 3 },
    { 0x335bde81, 3 },
    { 0x16abde83, 3 },
    { 0x734bde84, 3 },
    { 0x84cbde84, 3 },
    { 0x7f33de86, 3 },
    { 0xd0e3de87, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c3de8a, 4 },
    { 0, 0 },
    { 0xd98bde8c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc517de90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a23de95, 3 },
    { 0xf93bde96, 3 },
    { 0x195bde97, 3 },
    { 0, 0 },
    { 0xa783de99, 4 },
    { 0, 0 },
    { 0xd947de9b, 3 },
    { 0xb47bde9c, 3 },
    { 0x953de9c, 3 },
    { 0x9137de9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc93dea3, 4 },
    { 0x5c8fdea4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e3deab, 3 },
    { 0xe597deab, 4 },
    { 0x5e8bdeab, 3 },
    { 0xe1bdead, 3 },
    { 0x837deae, 3 },
    { 0x2a3bdeac, 3 },
    { 0xa95fdeb1, 4 },
    { 0x7da3deab, 3 },
    { 0, 0 },
    { 0x35b7deb4, 3 },
    { 0x492bdeb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8fdebd, 3 },
    { 0x61afdebe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74bdec2, 4 },
    { 0x1a63dec3, 5 },
    { 0, 0 },
    { 0xabdfdec5, 3 },
    { 0x5783dec6, 4 },
    { 0x70e7dec7, 3 },
    { 0x2c3bdec8, 4 },
    { 0xbf4bdec6, 3 },
    { 0x782fdeca, 3 },
    { 0x754fdeca, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bbdece, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c07ded1, 3 },
    { 0, 0 },
    { 0x55ebded3, 3 },
    { 0, 0 },
    { 0xb76bded5, 3 },
    { 0xe9afded5, 5 },
    { 0, 0 },
    { 0xa4cbded8, 4 },
    { 0x813bded9, 3 },
    { 0, 0 },
    { 0x1fcbdedb, 3 },
    { 0x6eefdedb, 3 },
    { 0xa64fdedd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5fdee2, 4 },
    { 0xe55bdee3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa47deea, 3 },
    { 0xbd7bdeeb, 3 },
    { 0xb48bdeeb, 4 },
    { 0x1aafdeec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56bdef0, 3 },
    { 0x3ea3def0, 3 },
    { 0, 0 },
    { 0xcc5fdef3, 4 },
    { 0, 0 },
    { 0x2adbdef5, 3 },
    { 0, 0 },
    { 0xd573def7, 3 },
    { 0, 0 },
    { 0xfc2bdef9, 3 },
    { 0xbcc7defa, 3 },
    { 0x1747defa, 3 },
    { 0, 0 },
    { 0xc22fdefd, 3 },
    { 0xf5fdefd, 4 },
    { 0, 0 },
    { 0xff6fdf00, 3 },
    { 0xab5bdf01, 4 },
    { 0x41afdf02, 4 },
    { 0x1f67df00, 3 },
    { 0xe087df04, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207df0a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7787df0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3fdf12, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ffdf16, 4 },
    { 0x1a87df17, 5 },
    { 0x734bdf18, 3 },
    { 0xf423df18, 3 },
    { 0x4a9bdf19, 3 },
    { 0x9f9fdf1b, 3 },
    { 0x7a3bdf1c, 3 },
    { 0x64bbdf18, 3 },
    { 0x140fdf1e, 3 },
    { 0, 0 },
    { 0x7337df20, 3 },
    { 0x8e0bdf21, 3 },
    { 0, 0 },
    { 0x441fdf23, 3 },
    { 0x9cdbdf24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06bdf27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6bdf2b, 3 },
    { 0x38efdf2c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fdf2f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51bdf33, 3 },
    { 0, 0 },
    { 0x97fdf35, 5 },
    { 0, 0 },
    { 0xbf4fdf37, 3 },
    { 0x3cf7df38, 4 },
    { 0, 0 },
    { 0xe227df3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ebdf41, 3 },
    { 0x5ab7df42, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1bdf47, 3 },
    { 0xb01bdf48, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a3df4c, 3 },
    { 0x233bdf4d, 3 },
    { 0xbe5bdf4d, 3 },
    { 0xcd53df4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83df59, 3 },
    { 0x9a47df59, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17fdf5e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407df62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bdf65, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58bdf68, 5 },
    { 0, 0 },
    { 0xa43bdf6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9233df74, 3 },
    { 0, 0 },
    { 0xee1bdf76, 3 },
    { 0xcfa7df77, 4 },
    { 0xa7dfdf78, 3 },
    { 0x9d37df78, 3 },
    { 0x8d2bdf7a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb563df82, 3 },
    { 0, 0 },
    { 0xd8efdf84, 3 },
    { 0x7b5bdf84, 3 },
    { 0, 0 },
    { 0xc1f3df87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f57df8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963df8e, 3 },
    { 0, 0 },
    { 0x1563df90, 3 },
    { 0x245fdf91, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9683df94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b17df97, 3 },
    { 0, 0 },
    { 0x40ffdf99, 4 },
    { 0x3e43df9a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7dfa3, 3 },
    { 0x87ffdfa4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81fdfaa, 3 },
    { 0xfd97dfab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583dfb1, 3 },
    { 0xba93dfb2, 5 },
    { 0xf673dfb3, 3 },
    { 0x7a17dfb4, 3 },
    { 0, 0 },
    { 0x54b3dfb6, 3 },
    { 0x241bdfb7, 3 },
    { 0xf9fbdfb8, 3 },
    { 0, 0 },
    { 0x481bdfba, 4 },
    { 0, 0 },
    { 0x18b3dfbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4333dfbf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1fdfc3, 4 },
    { 0x1233dfc3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaafdfc9, 3 },
    { 0x199fdfca, 3 },
    { 0, 0 },
    { 0x88cfdfcc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bfdfd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdfdfd4, 4 },
    { 0xb96bdfd5, 4 },
    { 0x2b23dfd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22bdfd9, 3 },
    { 0xf3c7dfda, 4 },
    { 0x5177dfda, 4 },
    { 0xaf53dfd9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2193dfe0, 4 },
    { 0xb447dfe1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197dfea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1667dfed, 3 },
    { 0, 0 },
    { 0xbfc7dfef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f7dff3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7993dffb, 3 },
    { 0, 0 },
    { 0xcd7fdffd, 3 },
    { 0, 0 },
    { 0xf1ffdfff, 3 },
    { 0, 0 },
    { 0xd3e001, 4 },
    { 0, 0 },
    { 0x4c27e003, 3 },
    { 0xee47e004, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39be007, 3 },
    { 0, 0 },
    { 0xf25fe009, 4 },
    { 0xa817e00a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f3e00d, 3 },
    { 0, 0 },
    { 0x85e3e00f, 4 },
    { 0x692fe00f, 3 },
    { 0x42b3e011, 3 },
    { 0x7033e010, 4 },
    { 0xf23fe013, 3 },
    { 0x7227e010, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8fe017, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735be01b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ffe01e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc203e023, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0fe026, 4 },
    { 0xfdefe027, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbe02d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336be031, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x173fe034, 3 },
    { 0xc92fe035, 3 },
    { 0x7b8be036, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26be03a, 3 },
    { 0x33abe03b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x947be03e, 3 },
    { 0, 0 },
    { 0x1c1fe040, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd483e043, 3 },
    { 0x7cd3e044, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9227e052, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317be057, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcbe05b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e05e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313be064, 3 },
    { 0x506fe065, 3 },
    { 0, 0 },
    { 0xd9bbe067, 3 },
    { 0, 0 },
    { 0x9573e069, 3 },
    { 0, 0 },
    { 0x577be06b, 3 },
    { 0xf75fe06b, 3 },
    { 0x8673e06d, 3 },
    { 0x67cbe06e, 3 },
    { 0x60c7e06f, 3 },
    { 0xd8ebe06f, 3 },
    { 0xcb13e06d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbfe076, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fefe079, 3 },
    { 0xc79fe07a, 3 },
    { 0, 0 },
    { 0xedb7e07c, 3 },
    { 0, 0 },
    { 0x9907e07e, 4 },
    { 0x2183e07f, 4 },
    { 0xdef7e080, 4 },
    { 0x6f33e081, 3 },
    { 0x613e082, 3 },
    { 0x2e63e083, 3 },
    { 0xf707e084, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x987fe087, 3 },
    { 0x5fe7e088, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3e08e, 4 },
    { 0xf733e08e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f3e092, 3 },
    { 0x76cbe092, 3 },
    { 0x7cc7e093, 3 },
    { 0xbc8be095, 4 },
    { 0xcda7e095, 3 },
    { 0x5083e094, 3 },
    { 0xa5ebe094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c93e09d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbe0a0, 4 },
    { 0xd27fe0a0, 3 },
    { 0xf69be0a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53be0a9, 4 },
    { 0, 0 },
    { 0xced7e0ab, 4 },
    { 0x7f2fe0ac, 3 },
    { 0xcde7e0ad, 4 },
    { 0, 0 },
    { 0x9b57e0af, 3 },
    { 0, 0 },
    { 0xb2d7e0b1, 3 },
    { 0, 0 },
    { 0xa3afe0b3, 3 },
    { 0x8fd3e0b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7e0bc, 3 },
    { 0, 0 },
    { 0xb2e3e0be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f3e0c2, 3 },
    { 0x66dfe0c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6fe0cb, 3 },
    { 0xcd9fe0cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3447e0d3, 3 },
    { 0, 0 },
    { 0x3b17e0d5, 3 },
    { 0x90a7e0d6, 3 },
    { 0xd97e0d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d7e0e4, 3 },
    { 0x1e3be0e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b3e0e8, 4 },
    { 0xbf7e0e9, 4 },
    { 0x91fbe0ea, 4 },
    { 0xfc63e0eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2fe0f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca7e0f7, 3 },
    { 0, 0 },
    { 0xb237e0f9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30fe0ff, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbe102, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fe106, 4 },
    { 0x71d7e106, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cfe10a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103e10d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x535fe110, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1617e115, 3 },
    { 0x6b4be115, 3 },
    { 0xf4cfe117, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b27e11b, 3 },
    { 0xe85fe11c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8be120, 3 },
    { 0xa50fe120, 3 },
    { 0, 0 },
    { 0xb95be123, 3 },
    { 0x7b4be123, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde77e12b, 3 },
    { 0x5fcfe12c, 3 },
    { 0x9a5fe12d, 3 },
    { 0x8b3e12e, 4 },
    { 0, 0 },
    { 0x302be130, 3 },
    { 0x461be131, 3 },
    { 0x859be130, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bfe136, 3 },
    { 0xac6fe137, 3 },
    { 0xfccbe138, 3 },
    { 0x7973e138, 3 },
    { 0xedf7e138, 3 },
    { 0x9fafe13b, 3 },
    { 0x6b9fe13c, 3 },
    { 0, 0 },
    { 0x8e5be13e, 4 },
    { 0xa707e13f, 3 },
    { 0xf4d3e140, 3 },
    { 0, 0 },
    { 0xaf8be142, 3 },
    { 0xa173e142, 4 },
    { 0xcae7e144, 3 },
    { 0x77ffe145, 3 },
    { 0x2d57e144, 3 },
    { 0x597fe147, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aebe14c, 4 },
    { 0x68ebe14c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07fe151, 3 },
    { 0x587e152, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9be155, 3 },
    { 0x141fe156, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec3e15b, 4 },
    { 0x80a3e15c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e3e161, 3 },
    { 0xf45be162, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563e168, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe777e16d, 3 },
    { 0x2a07e16e, 3 },
    { 0, 0 },
    { 0xe1c7e170, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06fe174, 3 },
    { 0x6713e174, 4 },
    { 0x1fc3e174, 3 },
    { 0xfbbe177, 3 },
    { 0, 0 },
    { 0x3453e179, 3 },
    { 0xb087e17a, 3 },
    { 0, 0 },
    { 0x7dafe17c, 3 },
    { 0, 0 },
    { 0xb78fe17e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fe181, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ffe188, 3 },
    { 0, 0 },
    { 0x248be18a, 3 },
    { 0, 0 },
    { 0x4697e18c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cbe194, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27e19a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517e19f, 3 },
    { 0x19abe1a0, 4 },
    { 0xf8bbe19f, 4 },
    { 0x5657e19f, 3 },
    { 0x3517e19f, 3 },
    { 0x74a3e1a1, 4 },
    { 0x508fe1a1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cbe1a9, 3 },
    { 0, 0 },
    { 0xc6e7e1ab, 3 },
    { 0x5b27e1ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753e1af, 3 },
    { 0, 0 },
    { 0x168fe1b1, 4 },
    { 0, 0 },
    { 0x9767e1b3, 3 },
    { 0, 0 },
    { 0x677fe1b5, 4 },
    { 0x57bfe1b5, 3 },
    { 0xc0c7e1b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217e1ba, 3 },
    { 0, 0 },
    { 0xa30fe1bc, 3 },
    { 0x78bfe1bd, 4 },
    { 0x459fe1be, 3 },
    { 0, 0 },
    { 0x1aabe1c0, 4 },
    { 0x191be1c1, 3 },
    { 0, 0 },
    { 0x217e1c3, 3 },
    { 0x5fbbe1c4, 3 },
    { 0xfba7e1c3, 3 },
    { 0x6983e1c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3e1c9, 3 },
    { 0xeb2fe1c9, 3 },
    { 0x2257e1ca, 3 },
    { 0xb7f3e1c9, 3 },
    { 0x6d6be1cc, 3 },
    { 0x4b5be1ce, 3 },
    { 0x330fe1cf, 3 },
    { 0, 0 },
    { 0x24ebe1d1, 3 },
    { 0x204be1d2, 3 },
    { 0xfb13e1d2, 3 },
    { 0xae4fe1d2, 3 },
    { 0x45abe1d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d77e1db, 3 },
    { 0x867be1dc, 4 },
    { 0x340fe1db, 3 },
    { 0x6143e1de, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7fe1e1, 4 },
    { 0x3a3be1e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cfe1ef, 3 },
    { 0x781fe1f0, 4 },
    { 0x5befe1f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15fe1f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42be1f7, 4 },
    { 0, 0 },
    { 0x79ebe1f9, 3 },
    { 0x5f37e1f9, 4 },
    { 0xe517e1f9, 3 },
    { 0x3af7e1f9, 4 },
    { 0xf84be1fd, 3 },
    { 0xd96be1fe, 4 },
    { 0x8243e1fc, 3 },
    { 0xc40fe1fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbe203, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7fe207, 5 },
    { 0xe917e208, 3 },
    { 0xe6c7e209, 3 },
    { 0x95e7e20a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c7e20d, 3 },
    { 0xb92be20e, 3 },
    { 0x5373e20f, 3 },
    { 0x9affe20f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac7e219, 3 },
    { 0xff5be21a, 3 },
    { 0x58b7e21b, 3 },
    { 0, 0 },
    { 0x2713e21d, 4 },
    { 0, 0 },
    { 0x3783e21f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911fe223, 4 },
    { 0xb557e223, 4 },
    { 0x548be225, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb777e228, 3 },
    { 0, 0 },
    { 0xa08fe22a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7e22d, 4 },
    { 0x2c63e22e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b3e231, 4 },
    { 0xb87fe232, 5 },
    { 0x710fe233, 3 },
    { 0, 0 },
    { 0x1407e235, 3 },
    { 0x9067e236, 4 },
    { 0x2da3e237, 3 },
    { 0xe59fe236, 3 },
    { 0, 0 },
    { 0xa54fe23a, 3 },
    { 0x73afe23b, 3 },
    { 0xa2efe23c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc707e241, 3 },
    { 0, 0 },
    { 0x5bffe243, 4 },
    { 0x20ffe244, 3 },
    { 0, 0 },
    { 0x5f87e246, 3 },
    { 0x594be247, 3 },
    { 0x9e5fe248, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae3e24b, 3 },
    { 0xd89be24c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f33e252, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0be258, 5 },
    { 0x16bbe259, 3 },
    { 0x8e5be258, 4 },
    { 0x8cbbe25b, 5 },
    { 0x8353e25c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb3e265, 3 },
    { 0x697e266, 3 },
    { 0x6fe267, 3 },
    { 0, 0 },
    { 0x5443e269, 4 },
    { 0, 0 },
    { 0x687be26b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5fe273, 4 },
    { 0, 0 },
    { 0x9093e275, 3 },
    { 0x9f3be276, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2493e279, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ffe27f, 3 },
    { 0, 0 },
    { 0x5febe281, 4 },
    { 0xba4fe282, 3 },
    { 0x286fe283, 4 },
    { 0, 0 },
    { 0x81e7e285, 4 },
    { 0x2467e286, 3 },
    { 0xd883e286, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa317e28d, 3 },
    { 0, 0 },
    { 0x4a77e28f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e7e296, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf3e299, 5 },
    { 0x7e67e29a, 3 },
    { 0x4903e299, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97e2a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104be2a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f7e2ac, 3 },
    { 0xc53fe2ad, 3 },
    { 0x352be2ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c3e2b5, 3 },
    { 0, 0 },
    { 0xc12fe2b7, 3 },
    { 0xb60be2b7, 3 },
    { 0x50ffe2b8, 3 },
    { 0xb2a3e2b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437e2bd, 3 },
    { 0x1b87e2be, 4 },
    { 0, 0 },
    { 0xe5c3e2c0, 3 },
    { 0xfc97e2c1, 3 },
    { 0x2d13e2c2, 3 },
    { 0x3fffe2c3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fe2c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412be2cc, 4 },
    { 0, 0 },
    { 0x78a7e2ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fe2d2, 3 },
    { 0x8bebe2d3, 3 },
    { 0x1593e2d3, 3 },
    { 0xcf4be2d5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc277e2d8, 3 },
    { 0x2f17e2d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xceefe2dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bfe2e3, 3 },
    { 0x3a1fe2e3, 4 },
    { 0, 0 },
    { 0x177e2e6, 4 },
    { 0x3c3be2e6, 3 },
    { 0x348fe2e8, 3 },
    { 0xcb7be2e7, 3 },
    { 0x74e7e2ea, 5 },
    { 0xfa0be2e6, 3 },
    { 0xcdc3e2ec, 4 },
    { 0, 0 },
    { 0x484fe2ee, 4 },
    { 0x8297e2ef, 4 },
    { 0x4fd3e2ef, 3 },
    { 0x7d7fe2f1, 4 },
    { 0, 0 },
    { 0x593e2f3, 3 },
    { 0x53efe2f4, 4 },
    { 0xa47be2f5, 3 },
    { 0xe7a7e2f6, 3 },
    { 0x4217e2f6, 3 },
    { 0xa90be2f8, 3 },
    { 0xd45fe2f9, 3 },
    { 0xc73be2f5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccbe2fd, 3 },
    { 0xb66be2fe, 3 },
    { 0x14efe2ff, 3 },
    { 0x3bc7e300, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164be306, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fbe30c, 3 },
    { 0x5667e30c, 3 },
    { 0x688be30e, 3 },
    { 0xe40be30e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7e318, 4 },
    { 0xbabbe319, 3 },
    { 0, 0 },
    { 0xc60be31b, 3 },
    { 0, 0 },
    { 0x7f77e31d, 3 },
    { 0xb41fe31e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442be323, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e328, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf343e32e, 3 },
    { 0x2b47e32f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53e332, 3 },
    { 0xff8fe333, 3 },
    { 0, 0 },
    { 0x6d93e335, 3 },
    { 0xfbfe336, 3 },
    { 0x8283e337, 3 },
    { 0xe8a7e338, 3 },
    { 0, 0 },
    { 0x1f4fe33a, 3 },
    { 0x80fe33b, 4 },
    { 0, 0 },
    { 0x872be33d, 3 },
    { 0xa0cfe33d, 4 },
    { 0xd29be33f, 4 },
    { 0x4307e33d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28be344, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc7e347, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e3e34c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa3e353, 3 },
    { 0, 0 },
    { 0xe29be355, 3 },
    { 0x7ebfe356, 3 },
    { 0x535be355, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x635be35a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbfe35d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7fe362, 3 },
    { 0x91f7e362, 4 },
    { 0x1817e364, 3 },
    { 0xe597e364, 3 },
    { 0xf45be366, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90fe36a, 4 },
    { 0, 0 },
    { 0xd017e36c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883be370, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a27e375, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9027e379, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe7e37c, 4 },
    { 0, 0 },
    { 0x60bbe37e, 4 },
    { 0x290be37f, 3 },
    { 0xd2b7e380, 4 },
    { 0x82ffe381, 3 },
    { 0x22ebe37f, 3 },
    { 0x9ae3e383, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd863e387, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922be38b, 4 },
    { 0x538be38b, 3 },
    { 0x9dffe38d, 3 },
    { 0xee47e38e, 3 },
    { 0xbd7be38e, 3 },
    { 0x2c63e390, 3 },
    { 0xd33be390, 3 },
    { 0, 0 },
    { 0x3f43e393, 3 },
    { 0, 0 },
    { 0xf6f3e395, 3 },
    { 0, 0 },
    { 0xc5bbe397, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802fe39e, 3 },
    { 0, 0 },
    { 0xd57fe3a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935be3a6, 3 },
    { 0, 0 },
    { 0xa147e3a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3e3ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46fe3af, 3 },
    { 0x7a13e3b0, 4 },
    { 0x683e3b1, 3 },
    { 0x904be3af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbe3b5, 4 },
    { 0x2587e3b5, 4 },
    { 0xc633e3b7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f7e3bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81fe3c1, 3 },
    { 0xac77e3c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0fe3c5, 4 },
    { 0x9ec3e3c6, 4 },
    { 0xe06be3c5, 4 },
    { 0xd087e3c8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533e3cd, 3 },
    { 0, 0 },
    { 0xccafe3cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857e3d6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4be3d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3e3dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3fe3e2, 3 },
    { 0x6f57e3e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4743e3e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5407e3ef, 3 },
    { 0xf2c7e3f0, 3 },
    { 0x5157e3ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebe3fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf537e3ff, 3 },
    { 0xe4f7e400, 3 },
    { 0, 0 },
    { 0x9e1be402, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada3e406, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b67e40a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eebe40d, 3 },
    { 0xb16fe40d, 3 },
    { 0xf3e7e40f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4be421, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bbe424, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843e42a, 3 },
    { 0xba4fe42a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea93e42f, 4 },
    { 0xaef3e42f, 3 },
    { 0xc887e431, 3 },
    { 0x8b53e431, 3 },
    { 0x23d7e431, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87be43a, 5 },
    { 0, 0 },
    { 0xd237e43c, 4 },
    { 0x4317e43c, 4 },
    { 0x107e43e, 4 },
    { 0x467e43f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0fe443, 3 },
    { 0x174fe443, 3 },
    { 0x6103e445, 5 },
    { 0x63d3e446, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe3e451, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afe456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5effe459, 3 },
    { 0x75dbe45a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957e463, 3 },
    { 0x55e7e464, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabe46b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f3e470, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583e476, 3 },
    { 0, 0 },
    { 0xa52fe478, 3 },
    { 0x973be479, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ffe47c, 3 },
    { 0x939fe47d, 4 },
    { 0xaeefe47e, 4 },
    { 0, 0 },
    { 0xdd8be480, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c3e485, 3 },
    { 0x8c23e485, 4 },
    { 0x9587e487, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600fe48e, 3 },
    { 0xb8c7e48f, 3 },
    { 0, 0 },
    { 0xc2efe491, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb97e494, 3 },
    { 0xecfe494, 3 },
    { 0xbbb7e496, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb23e49a, 3 },
    { 0x4293e49a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eabe49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57be4a3, 3 },
    { 0, 0 },
    { 0x7e03e4a5, 3 },
    { 0x616be4a6, 3 },
    { 0, 0 },
    { 0x4b43e4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5be4ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f3e4ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3be4b1, 3 },
    { 0, 0 },
    { 0x82fe4b3, 4 },
    { 0, 0 },
    { 0x419fe4b5, 3 },
    { 0, 0 },
    { 0xd10be4b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737be4bb, 3 },
    { 0x8c43e4bc, 3 },
    { 0, 0 },
    { 0x866be4be, 3 },
    { 0xf80be4bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x67abe4c2, 4 },
    { 0, 0 },
    { 0x3b43e4c4, 3 },
    { 0xd4cfe4c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe7e4c8, 4 },
    { 0xd673e4c9, 4 },
    { 0x8f2be4c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x243fe4cd, 3 },
    { 0, 0 },
    { 0x6733e4cf, 3 },
    { 0, 0 },
    { 0xce37e4d1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae7e4d4, 3 },
    { 0, 0 },
    { 0x38b3e4d6, 4 },
    { 0x4a17e4d7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8597e4db, 3 },
    { 0xc98fe4dc, 4 },
    { 0xe51fe4dd, 3 },
    { 0, 0 },
    { 0x47bfe4df, 3 },
    { 0x19e7e4e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c13e4e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf83e4e6, 4 },
    { 0xb48be4e7, 5 },
    { 0x9ab7e4e6, 4 },
    { 0x4663e4e9, 3 },
    { 0, 0 },
    { 0x6837e4eb, 3 },
    { 0, 0 },
    { 0xa2fbe4ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853fe4f2, 3 },
    { 0x2b83e4f2, 3 },
    { 0, 0 },
    { 0x9de3e4f5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e33e4f8, 3 },
    { 0, 0 },
    { 0x3a6fe4fa, 3 },
    { 0x7b7be4fb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85be504, 3 },
    { 0x2d57e505, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5be50b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x120fe50e, 3 },
    { 0x866be50f, 3 },
    { 0x6937e510, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73e518, 3 },
    { 0, 0 },
    { 0x5bf7e51a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb067e51f, 4 },
    { 0x346be520, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2fe523, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd3e527, 4 },
    { 0x9dbe528, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7423e52d, 3 },
    { 0, 0 },
    { 0x2cbe52f, 3 },
    { 0xc99be52f, 3 },
    { 0x356be531, 3 },
    { 0, 0 },
    { 0x3c8be533, 3 },
    { 0xfebfe534, 4 },
    { 0x66a3e535, 4 },
    { 0xf667e536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa987e53d, 4 },
    { 0, 0 },
    { 0xbb6fe53f, 3 },
    { 0x7373e53f, 3 },
    { 0xbf33e541, 3 },
    { 0x9abbe542, 4 },
    { 0x4b33e543, 3 },
    { 0xa9e7e542, 4 },
    { 0, 0 },
    { 0x3dcfe546, 3 },
    { 0xfc1be547, 4 },
    { 0, 0 },
    { 0x4833e549, 3 },
    { 0, 0 },
    { 0xc053e54b, 3 },
    { 0x16c7e54c, 4 },
    { 0, 0 },
    { 0x96b7e54e, 3 },
    { 0, 0 },
    { 0x9efbe550, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fe554, 3 },
    { 0x4033e555, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63e55e, 3 },
    { 0, 0 },
    { 0xb3bbe560, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce7e56c, 4 },
    { 0, 0 },
    { 0xdd13e56e, 3 },
    { 0x49ebe56f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0efe579, 3 },
    { 0, 0 },
    { 0x4447e57b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85afe57f, 3 },
    { 0, 0 },
    { 0x6d4be581, 3 },
    { 0x84ffe581, 3 },
    { 0xe34fe581, 4 },
    { 0x2ddfe581, 4 },
    { 0x7937e584, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc17e588, 3 },
    { 0, 0 },
    { 0x1987e58a, 3 },
    { 0x8d83e58b, 3 },
    { 0x657be58c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf507e58f, 3 },
    { 0xb84be590, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf3e593, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71fe596, 4 },
    { 0xfcb3e597, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d83e59a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaefe59d, 4 },
    { 0xdd77e59d, 4 },
    { 0x165fe59e, 3 },
    { 0xfaefe5a0, 3 },
    { 0, 0 },
    { 0xb3bbe5a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b23e5a5, 3 },
    { 0, 0 },
    { 0x8c4be5a7, 3 },
    { 0, 0 },
    { 0x4977e5a9, 3 },
    { 0xf68be5aa, 4 },
    { 0x5bbbe5aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3e5b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9763e5b8, 3 },
    { 0xa43e5b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd423e5bc, 4 },
    { 0xefe7e5bc, 3 },
    { 0xd253e5be, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444fe5c2, 3 },
    { 0x35afe5c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbbe5c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dafe5ce, 3 },
    { 0, 0 },
    { 0x678be5d0, 3 },
    { 0, 0 },
    { 0x4cefe5d2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1007e5d8, 4 },
    { 0x4bdbe5d8, 3 },
    { 0, 0 },
    { 0x91bbe5db, 3 },
    { 0xdd7e5dc, 3 },
    { 0x4ee3e5dc, 4 },
    { 0xccfe5db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214be5e3, 3 },
    { 0x8697e5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2be5e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7e5ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d13e5f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcabe5fb, 3 },
    { 0xcad3e5fc, 4 },
    { 0x9b33e5fd, 3 },
    { 0xe197e5fe, 3 },
    { 0x9747e5fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb043e602, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e93e609, 3 },
    { 0x5d1fe609, 3 },
    { 0xa8c7e60b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5307e60e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60be612, 3 },
    { 0x3327e613, 3 },
    { 0x92cbe613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbe618, 3 },
    { 0, 0 },
    { 0xb777e61a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0fe61d, 3 },
    { 0xcae7e61e, 4 },
    { 0, 0 },
    { 0xb63fe620, 4 },
    { 0, 0 },
    { 0xc7abe622, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3693e627, 4 },
    { 0, 0 },
    { 0x42f7e629, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6be632, 3 },
    { 0xf0d3e632, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77fe639, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x838be63c, 3 },
    { 0x686be63d, 4 },
    { 0x10d7e63c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81be641, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dfe649, 3 },
    { 0x87e7e649, 3 },
    { 0xa7abe64b, 3 },
    { 0xb6c3e64c, 3 },
    { 0x6d7be64d, 3 },
    { 0, 0 },
    { 0x311be64f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e3e653, 3 },
    { 0x7977e654, 3 },
    { 0x6703e653, 3 },
    { 0, 0 },
    { 0x876fe657, 4 },
    { 0xb51fe657, 3 },
    { 0x4ad7e659, 3 },
    { 0xa8f3e65a, 4 },
    { 0xfb6fe65b, 3 },
    { 0x6a6fe65c, 4 },
    { 0x79dbe65c, 3 },
    { 0xdaefe659, 3 },
    { 0xabfe65b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d97e662, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c3e666, 3 },
    { 0, 0 },
    { 0xb4f3e668, 4 },
    { 0, 0 },
    { 0x486be66a, 3 },
    { 0x8b27e66a, 4 },
    { 0x29ffe66b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dfe66f, 4 },
    { 0xfb6be670, 3 },
    { 0x354be671, 3 },
    { 0x489be670, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19afe676, 3 },
    { 0x58cfe676, 3 },
    { 0, 0 },
    { 0x641be679, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc077e680, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c17e68b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ffe699, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5fe69f, 3 },
    { 0x649be6a0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e6a5, 3 },
    { 0xae37e6a5, 3 },
    { 0x5697e6a7, 4 },
    { 0, 0 },
    { 0x2ffe6a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580fe6ad, 3 },
    { 0x9e63e6ae, 3 },
    { 0x67b7e6ae, 3 },
    { 0x1d2be6b0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d67e6b3, 3 },
    { 0xf797e6b4, 3 },
    { 0x60ebe6b5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5693e6be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86be6c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927e6c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacbe6cf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e17e6d2, 3 },
    { 0x633be6d3, 3 },
    { 0xb47e6d3, 3 },
    { 0x669be6d4, 3 },
    { 0x5a1be6d4, 3 },
    { 0xbf33e6d4, 3 },
    { 0xc4dfe6d4, 3 },
    { 0, 0 },
    { 0x8ccfe6da, 3 },
    { 0x2d9fe6db, 4 },
    { 0, 0 },
    { 0xfdb3e6dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbfe6e3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b3e6e7, 3 },
    { 0x2507e6e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588be6f0, 3 },
    { 0x917fe6f1, 3 },
    { 0x604be6f1, 3 },
    { 0xea6be6f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dfe6f9, 4 },
    { 0x92afe6f9, 4 },
    { 0x32f3e6fb, 3 },
    { 0x19bbe6fb, 3 },
    { 0x5bdbe6fc, 4 },
    { 0x8103e6fd, 3 },
    { 0x59a3e6ff, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa057e704, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebe707, 3 },
    { 0xbc9fe708, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bebe70c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9113e710, 4 },
    { 0xe78fe711, 3 },
    { 0x3b57e712, 3 },
    { 0x7b7e713, 3 },
    { 0x4653e714, 4 },
    { 0x5be3e715, 4 },
    { 0x89bfe716, 3 },
    { 0, 0 },
    { 0x677be718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9be71c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf77e722, 3 },
    { 0, 0 },
    { 0x6fe724, 4 },
    { 0xe0b3e725, 3 },
    { 0xc0b3e726, 3 },
    { 0x2187e727, 3 },
    { 0x4aabe728, 3 },
    { 0x890fe727, 3 },
    { 0, 0 },
    { 0x79fe72b, 3 },
    { 0x765fe72c, 4 },
    { 0x6bcbe72c, 3 },
    { 0, 0 },
    { 0x2ec7e72f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1607e735, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78efe745, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfe750, 4 },
    { 0x36afe751, 4 },
    { 0, 0 },
    { 0xfa8be753, 3 },
    { 0, 0 },
    { 0xad37e755, 3 },
    { 0x2c67e756, 3 },
    { 0x9537e756, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe613e75a, 3 },
    { 0x6ab3e75b, 4 },
    { 0x421fe75b, 3 },
    { 0x42e3e75d, 3 },
    { 0, 0 },
    { 0xd637e75f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf83e762, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e97e765, 3 },
    { 0x4f77e766, 3 },
    { 0xb99fe767, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a87e76b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4fe774, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e3e778, 3 },
    { 0xd0b3e779, 4 },
    { 0x4ccbe77a, 3 },
    { 0x2d17e77b, 3 },
    { 0xcfafe77c, 3 },
    { 0, 0 },
    { 0x79bfe77e, 3 },
    { 0x328be77e, 4 },
    { 0xd2bbe77e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf117e783, 3 },
    { 0x7e23e784, 3 },
    { 0x8263e783, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fe78c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa683e790, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfe794, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea7e79c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63be7a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2523e7a6, 4 },
    { 0xa467e7a7, 3 },
    { 0xd47fe7a6, 4 },
    { 0xcf3e7a7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf027e7ac, 4 },
    { 0xc4efe7ad, 3 },
    { 0xbc93e7ac, 3 },
    { 0x529be7af, 4 },
    { 0x3567e7af, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a97e7bf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccfe7c3, 3 },
    { 0xa01fe7c3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed33e7c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4297e7d2, 3 },
    { 0x98fe7d2, 3 },
    { 0x8ddfe7d4, 3 },
    { 0, 0 },
    { 0xa623e7d6, 3 },
    { 0xe2d3e7d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x28efe7da, 3 },
    { 0, 0 },
    { 0xfe7be7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38be7e2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659fe7e7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453fe7f1, 3 },
    { 0x5a3be7f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a7e7f6, 4 },
    { 0, 0 },
    { 0xb81fe7f8, 3 },
    { 0, 0 },
    { 0x3f43e7fa, 3 },
    { 0x78f7e7fa, 3 },
    { 0xdb13e7fb, 4 },
    { 0, 0 },
    { 0xe12be7fe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8233e803, 4 },
    { 0, 0 },
    { 0xc4b3e805, 3 },
    { 0, 0 },
    { 0xaec7e807, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8103e80c, 3 },
    { 0xd69fe80c, 3 },
    { 0x895be80c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x497fe811, 3 },
    { 0, 0 },
    { 0x5f23e813, 4 },
    { 0xef47e814, 3 },
    { 0x1ab7e813, 3 },
    { 0xbdd7e816, 3 },
    { 0x9f8be816, 3 },
    { 0x5763e818, 3 },
    { 0xa06fe819, 4 },
    { 0xb097e81a, 3 },
    { 0x4fdbe81b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a43e81e, 3 },
    { 0, 0 },
    { 0xf4dbe820, 3 },
    { 0x3e77e821, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8be824, 3 },
    { 0x7277e825, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x889be828, 5 },
    { 0x3bd7e829, 3 },
    { 0x64bbe82a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae7e83a, 3 },
    { 0x6cb7e83a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dbe83e, 3 },
    { 0x351fe83e, 3 },
    { 0x28a3e83e, 3 },
    { 0xcbffe83f, 3 },
    { 0, 0 },
    { 0xaf6be843, 3 },
    { 0xe09be844, 4 },
    { 0xcc3e845, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e7e848, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edbe84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbefe853, 4 },
    { 0x273e854, 3 },
    { 0xaafe855, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610fe85a, 4 },
    { 0xda6be85a, 3 },
    { 0, 0 },
    { 0xa53e85d, 3 },
    { 0x57e3e85e, 4 },
    { 0x93b7e85f, 3 },
    { 0xc277e85f, 3 },
    { 0xfd33e861, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3e864, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x241be867, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c13e86b, 3 },
    { 0xb4afe86b, 3 },
    { 0xd203e86d, 3 },
    { 0x48dfe86e, 3 },
    { 0x273fe86c, 4 },
    { 0xb457e870, 3 },
    { 0xd47fe86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ebe876, 3 },
    { 0x3fabe876, 3 },
    { 0, 0 },
    { 0xd2ffe879, 3 },
    { 0xfda7e87a, 3 },
    { 0x96bfe87a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3be881, 3 },
    { 0, 0 },
    { 0x8d27e883, 3 },
    { 0xe4ffe884, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167e887, 3 },
    { 0x6983e888, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c73e88d, 3 },
    { 0xb0b3e88e, 3 },
    { 0xb823e88f, 4 },
    { 0xe793e88d, 3 },
    { 0x6fefe891, 3 },
    { 0x2963e88e, 3 },
    { 0x11c7e893, 3 },
    { 0x4c2be890, 4 },
    { 0x6cdbe895, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9183e899, 3 },
    { 0xba93e89a, 3 },
    { 0x699be899, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x727fe89e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb073e8a2, 3 },
    { 0x2823e8a3, 3 },
    { 0x6b67e8a3, 3 },
    { 0xa6abe8a3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e8af, 4 },
    { 0xdfffe8b0, 3 },
    { 0, 0 },
    { 0x887e8b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7177e8b6, 3 },
    { 0x6cabe8b7, 4 },
    { 0, 0 },
    { 0xf3ffe8b9, 3 },
    { 0xd64be8ba, 4 },
    { 0x27afe8ba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816be8c3, 3 },
    { 0, 0 },
    { 0x1ecfe8c5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fe8c8, 3 },
    { 0, 0 },
    { 0x6dbe8ca, 3 },
    { 0, 0 },
    { 0x35dfe8cc, 4 },
    { 0x3b67e8cd, 4 },
    { 0xd953e8cd, 5 },
    { 0, 0 },
    { 0xc4d7e8d0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc3e8d4, 3 },
    { 0, 0 },
    { 0x813be8d6, 4 },
    { 0, 0 },
    { 0xa787e8d8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3e8db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63fe8de, 4 },
    { 0, 0 },
    { 0x1a07e8e0, 3 },
    { 0, 0 },
    { 0x7643e8e2, 3 },
    { 0x5007e8e3, 3 },
    { 0xc62be8e4, 3 },
    { 0x8bc3e8e5, 3 },
    { 0x95dfe8e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe7e8e9, 3 },
    { 0, 0 },
    { 0x818fe8eb, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e3e8ef, 4 },
    { 0x44a7e8f0, 3 },
    { 0x21dbe8f0, 3 },
    { 0, 0 },
    { 0x50dfe8f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aefe8f7, 4 },
    { 0, 0 },
    { 0xa023e8f9, 3 },
    { 0, 0 },
    { 0xb257e8fb, 4 },
    { 0x183be8fc, 4 },
    { 0x81fbe8fc, 3 },
    { 0x33fbe8fd, 3 },
    { 0x69d7e8fd, 4 },
    { 0x7217e8fe, 5 },
    { 0x8ff3e901, 3 },
    { 0, 0 },
    { 0x9f5fe903, 3 },
    { 0x300fe903, 3 },
    { 0, 0 },
    { 0x1703e906, 3 },
    { 0x7b87e907, 3 },
    { 0x1437e908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597be90e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341be912, 3 },
    { 0, 0 },
    { 0xa01be914, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e918, 4 },
    { 0, 0 },
    { 0xe2d7e91a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f7e920, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e73e923, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d3e929, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53be932, 3 },
    { 0x8b5fe933, 3 },
    { 0xb63e934, 3 },
    { 0xb9c7e935, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3e93a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x600fe93d, 3 },
    { 0, 0 },
    { 0x66a7e93f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7be942, 3 },
    { 0, 0 },
    { 0xc8a3e944, 3 },
    { 0, 0 },
    { 0x15a3e946, 3 },
    { 0xce33e947, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8be94b, 4 },
    { 0x3d03e94c, 3 },
    { 0xfc3fe94c, 3 },
    { 0xcf23e94e, 3 },
    { 0xf8cbe94d, 3 },
    { 0x1a7be950, 3 },
    { 0x713fe951, 3 },
    { 0x623be952, 3 },
    { 0xff83e953, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7613e958, 4 },
    { 0, 0 },
    { 0xbc03e95a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867e95e, 3 },
    { 0xf4cbe95f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbfe965, 3 },
    { 0x8933e966, 4 },
    { 0x214fe967, 3 },
    { 0xc8dfe965, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b7e96c, 3 },
    { 0x9a7be96c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527fe972, 4 },
    { 0xed7e972, 3 },
    { 0x7cefe974, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7933e978, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6987e97c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x199fe97f, 3 },
    { 0, 0 },
    { 0x96e7e981, 4 },
    { 0x11fe981, 3 },
    { 0xf93be983, 3 },
    { 0x607fe984, 3 },
    { 0, 0 },
    { 0x36cfe986, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747e98c, 3 },
    { 0x57ffe98d, 3 },
    { 0x771fe98c, 4 },
    { 0x33ebe98f, 3 },
    { 0, 0 },
    { 0x1fb7e991, 4 },
    { 0xf36be991, 4 },
    { 0x2523e992, 3 },
    { 0, 0 },
    { 0x704be995, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999be99b, 3 },
    { 0xc5dbe99c, 3 },
    { 0xef97e99d, 4 },
    { 0, 0 },
    { 0xf06be99f, 4 },
    { 0x9e63e9a0, 3 },
    { 0, 0 },
    { 0xd83e9a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67e9a5, 3 },
    { 0, 0 },
    { 0x3d93e9a7, 3 },
    { 0x36a7e9a7, 4 },
    { 0xed4fe9a9, 3 },
    { 0x8cbbe9a8, 4 },
    { 0x89e3e9aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be9b2, 3 },
    { 0xe47e9b3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e9b8, 4 },
    { 0xe197e9b8, 3 },
    { 0xf72be9b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8be9cb, 3 },
    { 0x925fe9cc, 4 },
    { 0x2cbfe9cd, 4 },
    { 0, 0 },
    { 0xc417e9cf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8fe9d4, 3 },
    { 0xfd8fe9d5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616fe9d9, 3 },
    { 0x51dfe9d9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11fe9de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cbe9e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003e9e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656fe9ec, 3 },
    { 0x3a9be9ed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f3e9f1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa7e9f4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6be9f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ebe9fd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe3ea01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bea05, 3 },
    { 0x7b63ea05, 3 },
    { 0x9197ea05, 3 },
    { 0x2387ea07, 3 },
    { 0x5f17ea09, 3 },
    { 0xa5e3ea0a, 5 },
    { 0x34d7ea07, 4 },
    { 0, 0 },
    { 0x88fbea0d, 3 },
    { 0x6c0bea0e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5387ea13, 4 },
    { 0x9dbbea13, 3 },
    { 0xef37ea13, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf83ea1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2fea20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323bea29, 3 },
    { 0xb657ea2a, 4 },
    { 0x2857ea2b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfea2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5643ea39, 3 },
    { 0x4c73ea39, 3 },
    { 0x43afea3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3ea41, 3 },
    { 0xa283ea41, 3 },
    { 0, 0 },
    { 0xababea44, 3 },
    { 0x523fea44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc613ea4e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc37ea52, 4 },
    { 0x4b0fea53, 3 },
    { 0, 0 },
    { 0x4eb7ea55, 3 },
    { 0xfc9fea56, 3 },
    { 0x9ffea57, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f7ea5b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fea61, 4 },
    { 0x14afea62, 3 },
    { 0x97bea62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be3ea67, 3 },
    { 0xb8f3ea68, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bfea6b, 4 },
    { 0xfc7ea6c, 3 },
    { 0x689bea6c, 3 },
    { 0x4133ea6e, 3 },
    { 0x43ffea6b, 3 },
    { 0, 0 },
    { 0x126fea71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2efea75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de7ea7c, 3 },
    { 0x1d97ea7c, 3 },
    { 0, 0 },
    { 0x7577ea7f, 3 },
    { 0x4b5fea80, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6bea88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ffea8f, 3 },
    { 0, 0 },
    { 0x704bea91, 3 },
    { 0, 0 },
    { 0x8157ea93, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b3ea9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d87ea9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f3eaa1, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbeaa4, 4 },
    { 0x2827eaa4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dabeaa8, 3 },
    { 0xcd43eaa8, 4 },
    { 0, 0 },
    { 0x4903eaab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81beab8, 3 },
    { 0, 0 },
    { 0xb6c7eaba, 3 },
    { 0x2a43eabb, 4 },
    { 0, 0 },
    { 0x437eabd, 3 },
    { 0, 0 },
    { 0x3837eabf, 3 },
    { 0xef23eac0, 3 },
    { 0x29ffeabf, 3 },
    { 0xb3d7eac1, 3 },
    { 0x9fdfeac3, 4 },
    { 0x6bbfeac4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beacc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd37ead1, 3 },
    { 0x7833ead2, 3 },
    { 0xf38fead3, 3 },
    { 0x8c53ead4, 3 },
    { 0, 0 },
    { 0xce63ead6, 3 },
    { 0xd843ead6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd7eadb, 3 },
    { 0, 0 },
    { 0x7babeadd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6767eae1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f7eae4, 3 },
    { 0x5197eae5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x3667eae8, 3 },
    { 0, 0 },
    { 0xc15beaea, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe373eaee, 3 },
    { 0, 0 },
    { 0x463beaf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7eaf6, 3 },
    { 0xc2b7eaf6, 3 },
    { 0x58dfeaf8, 3 },
    { 0x4757eaf8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4feafc, 3 },
    { 0x2503eafc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b3eb02, 3 },
    { 0x9ef3eb03, 4 },
    { 0x8f27eb02, 4 },
    { 0, 0 },
    { 0x28ebeb06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9feb0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3eb10, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3feb18, 3 },
    { 0x5babeb19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92afeb1e, 3 },
    { 0xad07eb1f, 4 },
    { 0xa67feb20, 3 },
    { 0x275beb21, 3 },
    { 0x34d7eb1e, 3 },
    { 0x896beb23, 3 },
    { 0xc16beb24, 3 },
    { 0x757eb23, 5 },
    { 0x3e5beb24, 3 },
    { 0x898beb27, 3 },
    { 0x916beb28, 3 },
    { 0x10d7eb26, 3 },
    { 0x9b63eb2a, 3 },
    { 0x202feb2b, 3 },
    { 0xbcfbeb23, 3 },
    { 0x4173eb25, 3 },
    { 0xe18feb2e, 3 },
    { 0xe103eb22, 3 },
    { 0, 0 },
    { 0x7fffeb31, 3 },
    { 0xc517eb31, 3 },
    { 0x7557eb33, 3 },
    { 0x517feb34, 3 },
    { 0xa5a7eb35, 3 },
    { 0xe05beb35, 3 },
    { 0x6ae7eb36, 3 },
    { 0xcba3eb35, 3 },
    { 0x193eb37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ebeb3e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7feb41, 3 },
    { 0x108feb42, 3 },
    { 0x76e3eb43, 3 },
    { 0x26b3eb41, 3 },
    { 0xcb63eb42, 3 },
    { 0, 0 },
    { 0x26f3eb47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd7eb4a, 3 },
    { 0xb70feb4b, 3 },
    { 0x6903eb4c, 3 },
    { 0x4d57eb4b, 3 },
    { 0xaa5feb4d, 3 },
    { 0xfcc7eb4f, 4 },
    { 0xea93eb50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b67eb53, 4 },
    { 0x5073eb54, 3 },
    { 0, 0 },
    { 0xd4ebeb56, 4 },
    { 0xab03eb56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffeb60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7107eb63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24afeb68, 3 },
    { 0x69dbeb69, 3 },
    { 0x623feb6a, 4 },
    { 0xfe2feb69, 4 },
    { 0xc33beb6c, 3 },
    { 0, 0 },
    { 0x475beb6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cbeb71, 3 },
    { 0x6f13eb71, 4 },
    { 0xa5a7eb73, 3 },
    { 0, 0 },
    { 0xe2f3eb75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e77eb79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe223eb7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185beb82, 3 },
    { 0x6263eb83, 3 },
    { 0xa15feb83, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8feb89, 3 },
    { 0x8ff3eb8a, 3 },
    { 0, 0 },
    { 0x466beb8c, 3 },
    { 0x15f3eb8d, 3 },
    { 0x6ddfeb8e, 4 },
    { 0, 0 },
    { 0x16abeb90, 3 },
    { 0x7d97eb91, 4 },
    { 0xea23eb92, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x999beb95, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b13eb9c, 4 },
    { 0, 0 },
    { 0x2693eb9e, 3 },
    { 0x42afeb9e, 3 },
    { 0x8d7feb9e, 3 },
    { 0, 0 },
    { 0xea13eba2, 3 },
    { 0x5ccbeba3, 3 },
    { 0, 0 },
    { 0x2607eba5, 3 },
    { 0x9d6beba5, 5 },
    { 0x5f5feba5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabebebac, 3 },
    { 0, 0 },
    { 0x7a0febae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x545febb1, 4 },
    { 0x6ffebb2, 3 },
    { 0, 0 },
    { 0x6f9bebb4, 3 },
    { 0x28ebebb5, 3 },
    { 0xefc3ebb4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53febb9, 3 },
    { 0x4323ebb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207ebbd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfebc1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6febc4, 3 },
    { 0, 0 },
    { 0x6c87ebc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c03ebcb, 3 },
    { 0xfc47ebcc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abebd0, 3 },
    { 0x14fbebd0, 3 },
    { 0x8f3ebd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680bebd6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478febdc, 3 },
    { 0, 0 },
    { 0x3b3febde, 3 },
    { 0x9813ebdf, 3 },
    { 0xe38bebdf, 4 },
    { 0x3e67ebdf, 5 },
    { 0x80d3ebe0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7ebe6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17ebea, 3 },
    { 0, 0 },
    { 0xd867ebec, 4 },
    { 0x3323ebed, 3 },
    { 0, 0 },
    { 0xebd7ebef, 3 },
    { 0xf06bebf0, 3 },
    { 0, 0 },
    { 0xc3b7ebf2, 3 },
    { 0xb72febf3, 3 },
    { 0xc8f3ebf4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822febfd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640fec01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ebec07, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e3ec11, 4 },
    { 0xcf6bec12, 4 },
    { 0, 0 },
    { 0x5277ec14, 3 },
    { 0xd693ec15, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bec1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acfec1e, 3 },
    { 0xc337ec1f, 3 },
    { 0, 0 },
    { 0xf2afec21, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32efec26, 3 },
    { 0x786bec27, 3 },
    { 0, 0 },
    { 0xb33fec29, 3 },
    { 0x4f17ec2a, 3 },
    { 0, 0 },
    { 0xebefec2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b3ec30, 4 },
    { 0, 0 },
    { 0xebdbec32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921bec38, 3 },
    { 0x7737ec39, 3 },
    { 0, 0 },
    { 0xf56fec3b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae37ec3f, 3 },
    { 0xa403ec3f, 4 },
    { 0xa503ec40, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc203ec44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fec4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6fec4e, 4 },
    { 0x1353ec4f, 3 },
    { 0x3693ec50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56fec55, 3 },
    { 0xe7f7ec56, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8707ec5b, 3 },
    { 0, 0 },
    { 0x7fe3ec5d, 3 },
    { 0, 0 },
    { 0x8807ec5f, 3 },
    { 0x9b5fec60, 4 },
    { 0x1f6bec61, 5 },
    { 0x6917ec62, 4 },
    { 0xf563ec62, 3 },
    { 0xb537ec63, 3 },
    { 0x6a93ec61, 3 },
    { 0x2a6fec66, 3 },
    { 0x9877ec67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53ec6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa463ec74, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3fec78, 3 },
    { 0x35cbec79, 4 },
    { 0x403bec79, 4 },
    { 0xe267ec7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x155fec7e, 3 },
    { 0x95c3ec7f, 4 },
    { 0x156fec7e, 4 },
    { 0xa89fec7e, 3 },
    { 0x9ad3ec7e, 3 },
    { 0xbbf7ec83, 3 },
    { 0, 0 },
    { 0xa897ec85, 3 },
    { 0x95c3ec85, 4 },
    { 0xb953ec87, 3 },
    { 0xa26bec88, 3 },
    { 0, 0 },
    { 0xcb73ec8a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9bec8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6fec9a, 3 },
    { 0x73fbec9a, 3 },
    { 0x7df7ec9b, 3 },
    { 0x5587ec9d, 4 },
    { 0x638bec9e, 4 },
    { 0x4dd3ec9f, 4 },
    { 0xa78feca0, 3 },
    { 0, 0 },
    { 0x28a3eca2, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4becab, 3 },
    { 0x4a83ecab, 5 },
    { 0, 0 },
    { 0xb86fecae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23afecb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a23ecba, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbecbd, 3 },
    { 0, 0 },
    { 0xed5fecbf, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x887fecc2, 3 },
    { 0, 0 },
    { 0x3377ecc4, 3 },
    { 0x9c1fecc5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f3ecc9, 3 },
    { 0x2c97ecca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cbeccd, 3 },
    { 0xe73beccd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0becd2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7ecd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bfecda, 4 },
    { 0, 0 },
    { 0x38b7ecdc, 3 },
    { 0, 0 },
    { 0x13a3ecde, 4 },
    { 0xd4becde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda7ece3, 3 },
    { 0, 0 },
    { 0x6febece5, 4 },
    { 0, 0 },
    { 0xdefece7, 3 },
    { 0xa32bece8, 5 },
    { 0x80ebece7, 4 },
    { 0x61b3ecea, 3 },
    { 0x65bbeceb, 3 },
    { 0, 0 },
    { 0xd107eced, 3 },
    { 0x588becee, 3 },
    { 0x1a5fecef, 3 },
    { 0xdc0fecf0, 3 },
    { 0x274becf0, 4 },
    { 0x9937ecee, 3 },
    { 0xeb87ecf2, 5 },
    { 0x1157ecf4, 3 },
    { 0x78ebecef, 3 },
    { 0, 0 },
    { 0x9b0fecf7, 4 },
    { 0, 0 },
    { 0x8e63ecf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23fecfc, 4 },
    { 0x1fbbecfc, 4 },
    { 0x9ea7ecfc, 4 },
    { 0xbec7ecfc, 4 },
    { 0, 0 },
    { 0x3c57ed01, 4 },
    { 0x3667ed02, 4 },
    { 0x9a9fed02, 4 },
    { 0x14f3ed02, 3 },
    { 0x10abed05, 4 },
    { 0xd513ed06, 4 },
    { 0xd9c3ed03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcfed0c, 3 },
    { 0xd227ed0d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adbed11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff3ed14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344fed19, 3 },
    { 0x21bed1a, 3 },
    { 0, 0 },
    { 0xfdffed1c, 3 },
    { 0x1727ed1c, 3 },
    { 0, 0 },
    { 0xa35bed1f, 4 },
    { 0x1fbed20, 3 },
    { 0, 0 },
    { 0x98c3ed22, 3 },
    { 0, 0 },
    { 0x4463ed24, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbed27, 3 },
    { 0x3933ed28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8933ed2b, 4 },
    { 0x49a7ed2b, 3 },
    { 0x7487ed2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fed31, 4 },
    { 0x8a67ed31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e7ed37, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fed40, 3 },
    { 0x1ec3ed41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b13ed45, 3 },
    { 0, 0 },
    { 0x582fed47, 4 },
    { 0, 0 },
    { 0x5073ed49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x483ed4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a33ed52, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1fed55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf393ed5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2fed60, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd7ed64, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b93ed68, 4 },
    { 0, 0 },
    { 0x39c7ed6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff3ed6f, 3 },
    { 0x2e87ed6f, 3 },
    { 0xdbffed6f, 3 },
    { 0x8ebfed72, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3ed76, 4 },
    { 0x2ddfed76, 3 },
    { 0x90c7ed78, 3 },
    { 0x44ebed79, 3 },
    { 0x88e7ed76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6bed7d, 3 },
    { 0x2befed7e, 3 },
    { 0x8c93ed7e, 4 },
    { 0x5c3bed80, 3 },
    { 0, 0 },
    { 0xddbfed82, 3 },
    { 0, 0 },
    { 0x9607ed84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17fed88, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c7ed8e, 3 },
    { 0, 0 },
    { 0xdf1bed90, 3 },
    { 0xc19fed91, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ffed96, 3 },
    { 0x5897ed97, 4 },
    { 0xe0cfed98, 3 },
    { 0x7fd7ed96, 3 },
    { 0x879fed9a, 3 },
    { 0xeb73ed97, 3 },
    { 0x1843ed96, 3 },
    { 0x4a6bed97, 4 },
    { 0x111bed9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65efeda3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b7eda7, 3 },
    { 0xd6d3eda7, 3 },
    { 0, 0 },
    { 0x9b2bedaa, 3 },
    { 0, 0 },
    { 0x7123edac, 4 },
    { 0x89ebedad, 3 },
    { 0, 0 },
    { 0xb8a7edaf, 3 },
    { 0xcaf3edb0, 4 },
    { 0x66fbedb0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1bedb4, 3 },
    { 0x9197edb5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54bedb8, 3 },
    { 0, 0 },
    { 0x1603edba, 3 },
    { 0x17afedbb, 3 },
    { 0x143fedbb, 5 },
    { 0x3cb7edbb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42bedc8, 4 },
    { 0x9fc7edc9, 3 },
    { 0, 0 },
    { 0xfdafedcb, 4 },
    { 0xf53bedcb, 3 },
    { 0xafbfedcd, 3 },
    { 0xdb27edcb, 4 },
    { 0, 0 },
    { 0xa4dfedd0, 3 },
    { 0, 0 },
    { 0x455bedd2, 3 },
    { 0x899bedd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c3edd8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd417eddc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb3eddf, 4 },
    { 0x9b5beddf, 3 },
    { 0xda5beddf, 3 },
    { 0, 0 },
    { 0x3fafede3, 3 },
    { 0x743fede4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67edf3, 3 },
    { 0x317bedf4, 3 },
    { 0x9f17edf5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3edfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3ee00, 4 },
    { 0x3d53ee01, 4 },
    { 0x45a7ee02, 5 },
    { 0xc92fee03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63bee07, 4 },
    { 0, 0 },
    { 0x89d7ee09, 3 },
    { 0x1a53ee0a, 4 },
    { 0, 0 },
    { 0x73d7ee0c, 3 },
    { 0, 0 },
    { 0x460fee0e, 3 },
    { 0x8f4fee0e, 3 },
    { 0xdcbee10, 3 },
    { 0x8937ee10, 4 },
    { 0, 0 },
    { 0x391bee13, 5 },
    { 0, 0 },
    { 0x4d0bee15, 3 },
    { 0x3e0bee16, 4 },
    { 0x5b3fee17, 3 },
    { 0xeb57ee16, 3 },
    { 0x4317ee18, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2babee20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a7ee23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bfee27, 3 },
    { 0, 0 },
    { 0x466bee29, 4 },
    { 0, 0 },
    { 0x2ca7ee2b, 3 },
    { 0x8d2fee2c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb7ee36, 3 },
    { 0x9d8bee36, 3 },
    { 0xba6fee38, 3 },
    { 0x928fee39, 3 },
    { 0xeb9bee39, 3 },
    { 0x18bee3b, 4 },
    { 0x99e3ee38, 3 },
    { 0, 0 },
    { 0xc4bee3e, 3 },
    { 0x4113ee3f, 4 },
    { 0x1bc7ee40, 3 },
    { 0x2d3ee40, 3 },
    { 0xb55bee3f, 3 },
    { 0, 0 },
    { 0xb0f3ee44, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c3ee4f, 3 },
    { 0x6fc7ee50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0bee57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607ee60, 3 },
    { 0xe557ee61, 4 },
    { 0x2bdfee62, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a17ee6b, 4 },
    { 0, 0 },
    { 0x1b47ee6d, 3 },
    { 0xae87ee6e, 3 },
    { 0xfdc7ee6d, 3 },
    { 0x63a3ee6f, 3 },
    { 0x78efee71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40fee74, 3 },
    { 0, 0 },
    { 0x4a93ee76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b3ee7b, 4 },
    { 0x6a23ee7c, 3 },
    { 0xd567ee7d, 3 },
    { 0xf053ee7d, 3 },
    { 0x7fc3ee7f, 3 },
    { 0x44f3ee80, 4 },
    { 0, 0 },
    { 0x429fee82, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed13ee85, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcfee88, 3 },
    { 0x16f3ee89, 3 },
    { 0x3ebfee8a, 4 },
    { 0, 0 },
    { 0x3f7ee8c, 4 },
    { 0x1d3fee8c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fee94, 3 },
    { 0, 0 },
    { 0x9e4bee96, 3 },
    { 0, 0 },
    { 0xef23ee98, 3 },
    { 0x2563ee99, 3 },
    { 0x2977ee99, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73efee9e, 3 },
    { 0, 0 },
    { 0xc47beea0, 3 },
    { 0xbef3eea1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4357eea4, 3 },
    { 0, 0 },
    { 0xb677eea6, 3 },
    { 0x6113eea6, 3 },
    { 0x4c53eea8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9597eeab, 3 },
    { 0, 0 },
    { 0xd093eead, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa183eeb1, 3 },
    { 0x725feeb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfeeb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970beebd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f3eec2, 3 },
    { 0, 0 },
    { 0x6cb3eec4, 3 },
    { 0x98ebeec4, 4 },
    { 0xe5bfeec5, 4 },
    { 0xdc27eec7, 4 },
    { 0x970beec8, 3 },
    { 0x2e3feec9, 3 },
    { 0x9353eec8, 5 },
    { 0, 0 },
    { 0xffe3eecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88beed6, 3 },
    { 0, 0 },
    { 0xaec7eed8, 4 },
    { 0, 0 },
    { 0xf50feeda, 3 },
    { 0x96dfeedb, 3 },
    { 0x71b7eeda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6feee0, 3 },
    { 0x32ffeee1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8477eee7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb97eeea, 3 },
    { 0x84afeeeb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcbeeef, 3 },
    { 0x970feef0, 4 },
    { 0, 0 },
    { 0xe1d7eef2, 3 },
    { 0, 0 },
    { 0x6457eef4, 3 },
    { 0, 0 },
    { 0x75a7eef6, 3 },
    { 0x4fabeef7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7713eefa, 3 },
    { 0, 0 },
    { 0x234feefc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb893ef00, 3 },
    { 0x5d9bef01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcbef05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6fef0a, 4 },
    { 0, 0 },
    { 0x1217ef0c, 3 },
    { 0x8cabef0c, 3 },
    { 0, 0 },
    { 0x4587ef0f, 4 },
    { 0xbad3ef0f, 3 },
    { 0x530fef11, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc997ef15, 3 },
    { 0x5cbfef16, 3 },
    { 0, 0 },
    { 0x8db7ef18, 3 },
    { 0, 0 },
    { 0x9eb3ef1a, 3 },
    { 0xa373ef1a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cfef1f, 5 },
    { 0x984fef20, 3 },
    { 0x93ffef1f, 3 },
    { 0x6dbfef1f, 3 },
    { 0x45a3ef22, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7bef27, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7697ef2d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6bef32, 3 },
    { 0xc91fef32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3effef39, 3 },
    { 0x39fbef3a, 3 },
    { 0x3c7ef39, 3 },
    { 0x6143ef3c, 4 },
    { 0x5623ef3c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1bef40, 3 },
    { 0x69bbef40, 3 },
    { 0, 0 },
    { 0x9c1fef43, 3 },
    { 0xa47bef44, 3 },
    { 0x769bef45, 4 },
    { 0x50dbef45, 3 },
    { 0x6ab7ef47, 3 },
    { 0x915bef48, 3 },
    { 0xb523ef49, 4 },
    { 0x545bef45, 3 },
    { 0xd633ef4b, 3 },
    { 0x2653ef4c, 3 },
    { 0x427ef4d, 4 },
    { 0xa59fef4d, 4 },
    { 0x7bf7ef4f, 3 },
    { 0x9c37ef50, 3 },
    { 0x659bef50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbbef54, 4 },
    { 0x76e7ef55, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1fef61, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1fef67, 3 },
    { 0, 0 },
    { 0xc587ef69, 3 },
    { 0xb0afef6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2623ef71, 3 },
    { 0x17d3ef72, 3 },
    { 0x614fef73, 4 },
    { 0, 0 },
    { 0x267fef75, 3 },
    { 0xbc87ef75, 4 },
    { 0x2dfef75, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bef7d, 3 },
    { 0xbc87ef7d, 4 },
    { 0xa433ef7f, 3 },
    { 0x3c3bef80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e7ef83, 3 },
    { 0, 0 },
    { 0x4e37ef85, 5 },
    { 0x1c6bef86, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78afef8c, 3 },
    { 0xc45bef8d, 4 },
    { 0x9123ef8e, 3 },
    { 0, 0 },
    { 0xaf9fef90, 3 },
    { 0x11ffef91, 5 },
    { 0x300fef92, 4 },
    { 0, 0 },
    { 0xea37ef94, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x111bef97, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc7ef9b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14bef9e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fefa1, 3 },
    { 0x9363efa2, 4 },
    { 0, 0 },
    { 0x33bfefa4, 3 },
    { 0, 0 },
    { 0x797efa6, 3 },
    { 0xe393efa7, 3 },
    { 0xefbbefa8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3043efab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fbefaf, 3 },
    { 0xf78fefb0, 3 },
    { 0x2083efb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4397efb5, 3 },
    { 0x5137efb6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c3efba, 3 },
    { 0x8ffbefbb, 3 },
    { 0xe753efbc, 3 },
    { 0xab47efbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196fefc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81befc4, 3 },
    { 0x5cffefc5, 4 },
    { 0xe583efc6, 3 },
    { 0x6b7efc7, 4 },
    { 0x665befc6, 4 },
    { 0x25b7efc9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dfefcf, 3 },
    { 0, 0 },
    { 0x4603efd1, 3 },
    { 0x347befd1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557efdc, 3 },
    { 0x7417efdd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4befe3, 3 },
    { 0xb37fefe4, 3 },
    { 0xe2bfefe5, 3 },
    { 0x9a7fefe6, 3 },
    { 0x60a3efe6, 4 },
    { 0x7623efe3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7efef, 3 },
    { 0xa6a7eff0, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a17eff9, 4 },
    { 0x1467eff9, 3 },
    { 0, 0 },
    { 0x2edbeffc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acfefff, 3 },
    { 0xdc3bf000, 4 },
    { 0x5017f001, 3 },
    { 0xc87ff002, 4 },
    { 0x5333f003, 4 },
    { 0xf733f003, 3 },
    { 0, 0 },
    { 0xe157f006, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d3f00e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1bf011, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6ff014, 3 },
    { 0xf0aff014, 3 },
    { 0xf03ff016, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b7f01c, 4 },
    { 0xa8c7f01c, 3 },
    { 0x750ff01d, 4 },
    { 0x4faff01c, 3 },
    { 0x791ff01e, 3 },
    { 0, 0 },
    { 0xc00bf022, 4 },
    { 0, 0 },
    { 0x44a3f024, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbf02a, 3 },
    { 0x1eabf02a, 4 },
    { 0, 0 },
    { 0x70c7f02d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x807ff030, 3 },
    { 0xcafff031, 4 },
    { 0x5d87f032, 5 },
    { 0, 0 },
    { 0xcc17f034, 3 },
    { 0, 0 },
    { 0x47f3f036, 3 },
    { 0xd6bff037, 3 },
    { 0, 0 },
    { 0x8cc3f039, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x816bf03c, 3 },
    { 0x8b23f03d, 3 },
    { 0xacf3f03e, 3 },
    { 0x27d3f03f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dff045, 3 },
    { 0xca17f046, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14bf049, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd7f04c, 3 },
    { 0x6f9ff04d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a57f052, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd3f055, 3 },
    { 0, 0 },
    { 0x107f057, 3 },
    { 0x6b3bf058, 3 },
    { 0xc613f059, 3 },
    { 0x3753f058, 3 },
    { 0xb68bf05b, 4 },
    { 0x9863f05b, 3 },
    { 0x777f05d, 3 },
    { 0xcbb3f05e, 3 },
    { 0x716bf05e, 3 },
    { 0, 0 },
    { 0xb27ff061, 3 },
    { 0x34ff062, 3 },
    { 0x5897f061, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ff066, 4 },
    { 0xeb63f067, 3 },
    { 0x50ff066, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad7f06b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee7f070, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513f074, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307f079, 3 },
    { 0xe07ff07a, 3 },
    { 0x4663f07a, 3 },
    { 0, 0 },
    { 0xafbf07d, 3 },
    { 0, 0 },
    { 0xded7f07f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5bf082, 3 },
    { 0x9a23f083, 3 },
    { 0x1d3f084, 3 },
    { 0x66b7f084, 3 },
    { 0xd757f086, 4 },
    { 0, 0 },
    { 0x4a6ff088, 3 },
    { 0x1a9bf089, 3 },
    { 0x39e7f088, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701bf090, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3f093, 4 },
    { 0x3fabf094, 3 },
    { 0xfc37f094, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce7f099, 3 },
    { 0x3273f09a, 4 },
    { 0x8de3f09b, 3 },
    { 0, 0 },
    { 0x8c07f09d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f83f0a7, 3 },
    { 0xb97bf0a8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8387f0ac, 3 },
    { 0xb3bff0ad, 3 },
    { 0, 0 },
    { 0xa87ff0af, 3 },
    { 0, 0 },
    { 0x8f0ff0b1, 3 },
    { 0, 0 },
    { 0xa43bf0b3, 3 },
    { 0xba33f0b4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2213f0b7, 3 },
    { 0, 0 },
    { 0x5c33f0b9, 3 },
    { 0x3183f0ba, 3 },
    { 0x43a7f0bb, 4 },
    { 0x232ff0ba, 4 },
    { 0, 0 },
    { 0x1a3f0be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c7f0c9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x745ff0cc, 3 },
    { 0, 0 },
    { 0x30bf0ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963bf0d4, 4 },
    { 0, 0 },
    { 0x3a23f0d6, 4 },
    { 0, 0 },
    { 0x26f7f0d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d03f0df, 3 },
    { 0, 0 },
    { 0x5abff0e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6ff0e4, 3 },
    { 0, 0 },
    { 0xd627f0e6, 3 },
    { 0x4fbbf0e6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c3f0ea, 3 },
    { 0xce77f0eb, 4 },
    { 0xc723f0ec, 3 },
    { 0, 0 },
    { 0x7c97f0ee, 3 },
    { 0x1e03f0ef, 3 },
    { 0x54d7f0f0, 4 },
    { 0xfd2ff0f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63f0f4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dff0f9, 4 },
    { 0x6aebf0fa, 3 },
    { 0xc8b3f0f9, 3 },
    { 0, 0 },
    { 0x9e9ff0fd, 3 },
    { 0x992bf0fe, 3 },
    { 0x580ff0ff, 3 },
    { 0x5c3ff100, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40ff105, 3 },
    { 0, 0 },
    { 0xd66bf107, 4 },
    { 0xfab3f108, 3 },
    { 0x563f109, 3 },
    { 0, 0 },
    { 0xafcbf10b, 4 },
    { 0x73abf10c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a7f110, 3 },
    { 0xfe03f111, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ff119, 4 },
    { 0x7343f119, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7f120, 4 },
    { 0xec27f121, 3 },
    { 0x581ff122, 3 },
    { 0xe6b7f122, 4 },
    { 0, 0 },
    { 0x3697f125, 3 },
    { 0, 0 },
    { 0xfaff127, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eff12b, 4 },
    { 0xf5a3f12c, 3 },
    { 0x9e4bf12b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bff139, 3 },
    { 0, 0 },
    { 0x6feff13b, 3 },
    { 0xd7aff13c, 3 },
    { 0, 0 },
    { 0xaa6ff13e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eff141, 3 },
    { 0x35f3f142, 3 },
    { 0, 0 },
    { 0x36c3f144, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf23f148, 3 },
    { 0, 0 },
    { 0x10d3f14a, 3 },
    { 0xa7bff14a, 3 },
    { 0, 0 },
    { 0x1a4ff14d, 3 },
    { 0x98bff14e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbf152, 3 },
    { 0x349bf153, 3 },
    { 0xdefbf154, 3 },
    { 0xc9bff155, 4 },
    { 0x9527f156, 3 },
    { 0x63bbf154, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8057f15a, 3 },
    { 0, 0 },
    { 0x8987f15c, 4 },
    { 0, 0 },
    { 0x4bb7f15e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27f161, 3 },
    { 0x3d67f162, 3 },
    { 0x261bf163, 3 },
    { 0x88a3f163, 3 },
    { 0x8533f165, 3 },
    { 0x9f3ff165, 3 },
    { 0x763f165, 4 },
    { 0x5a87f165, 5 },
    { 0x5113f167, 3 },
    { 0xd237f165, 3 },
    { 0, 0 },
    { 0x326bf16c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4487f171, 3 },
    { 0xf0c7f171, 4 },
    { 0, 0 },
    { 0x6a77f174, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cbf17b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5ff180, 3 },
    { 0x41cbf180, 3 },
    { 0xcbf7f182, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7ff186, 3 },
    { 0xa9a3f186, 3 },
    { 0x8caff188, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9bf19b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc63f1a4, 3 },
    { 0, 0 },
    { 0xf05ff1a6, 4 },
    { 0x63dbf1a7, 5 },
    { 0x4eaff1a7, 3 },
    { 0, 0 },
    { 0x5147f1aa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799bf1b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a3f1be, 4 },
    { 0, 0 },
    { 0xb753f1c0, 4 },
    { 0xd9fbf1c1, 3 },
    { 0, 0 },
    { 0x53cbf1c3, 3 },
    { 0x4bf3f1c3, 3 },
    { 0, 0 },
    { 0xb1d3f1c6, 3 },
    { 0xa973f1c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7f1cb, 3 },
    { 0x972bf1cc, 3 },
    { 0xc99ff1cd, 3 },
    { 0xf95bf1cd, 3 },
    { 0x8fb7f1cf, 3 },
    { 0x2c2bf1d0, 3 },
    { 0, 0 },
    { 0x567bf1d2, 3 },
    { 0xdebbf1d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fbf1d8, 3 },
    { 0, 0 },
    { 0xda3ff1da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb043f1dd, 3 },
    { 0xe1b3f1de, 3 },
    { 0x8313f1df, 3 },
    { 0x4b3bf1de, 4 },
    { 0, 0 },
    { 0x9093f1e2, 3 },
    { 0x24d7f1e2, 4 },
    { 0, 0 },
    { 0xfe7f1e5, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f3f1e9, 4 },
    { 0, 0 },
    { 0x3a17f1eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e33f1f1, 3 },
    { 0x138bf1f2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbf1f5, 3 },
    { 0x6cabf1f6, 3 },
    { 0xbd37f1f7, 3 },
    { 0xe513f1f8, 3 },
    { 0x397bf1f8, 3 },
    { 0xf3ebf1fa, 3 },
    { 0x3c77f1fa, 3 },
    { 0, 0 },
    { 0xef83f1fd, 4 },
    { 0x8167f1fe, 3 },
    { 0x790ff1ff, 3 },
    { 0, 0 },
    { 0xbddbf201, 3 },
    { 0, 0 },
    { 0x708bf203, 4 },
    { 0xa3e3f203, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e3f209, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2ff212, 3 },
    { 0x5067f213, 3 },
    { 0x9087f213, 3 },
    { 0, 0 },
    { 0x254ff216, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f3f21d, 3 },
    { 0xbf37f21e, 3 },
    { 0xc9fbf21e, 3 },
    { 0xdc2ff220, 3 },
    { 0x658ff221, 4 },
    { 0x5df3f221, 3 },
    { 0, 0 },
    { 0x4adbf224, 3 },
    { 0, 0 },
    { 0xeb03f226, 4 },
    { 0xba7f227, 3 },
    { 0xd8aff226, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aff22c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710bf234, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xff97f237, 3 },
    { 0, 0 },
    { 0xe667f239, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3f245, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f33f248, 3 },
    { 0, 0 },
    { 0x907bf24a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44bf24f, 4 },
    { 0, 0 },
    { 0x1bc3f251, 3 },
    { 0x6a97f251, 3 },
    { 0xb0b3f253, 3 },
    { 0, 0 },
    { 0xffbf255, 3 },
    { 0x98bff256, 3 },
    { 0xfe13f256, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5053f25a, 3 },
    { 0xa027f25b, 4 },
    { 0xc8dff25c, 3 },
    { 0xa2abf25d, 3 },
    { 0x5c9bf25e, 3 },
    { 0xba4bf25f, 3 },
    { 0x4fff260, 3 },
    { 0x540bf25a, 3 },
    { 0x81a7f25d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff268, 3 },
    { 0xa903f269, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbbf26f, 3 },
    { 0x4f5ff270, 3 },
    { 0xb77bf270, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61bf275, 3 },
    { 0xfc67f276, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3803f27b, 3 },
    { 0xffe7f27b, 4 },
    { 0, 0 },
    { 0x34cbf27e, 3 },
    { 0, 0 },
    { 0x1efff280, 3 },
    { 0x9bfbf281, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caff286, 4 },
    { 0, 0 },
    { 0xbd97f288, 3 },
    { 0xd00ff289, 4 },
    { 0, 0 },
    { 0xc793f28b, 4 },
    { 0, 0 },
    { 0xd17bf28d, 3 },
    { 0xb66ff28e, 4 },
    { 0, 0 },
    { 0xd2c7f290, 3 },
    { 0, 0 },
    { 0x19cbf292, 3 },
    { 0xc733f293, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b7f297, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb7f29a, 4 },
    { 0, 0 },
    { 0x3eebf29c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267f2a1, 3 },
    { 0xd2dff2a2, 3 },
    { 0x2553f2a3, 3 },
    { 0x6d6bf2a4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eabf2a8, 4 },
    { 0x340bf2a9, 3 },
    { 0x6057f2a8, 3 },
    { 0x4b33f2ab, 3 },
    { 0xe037f2ac, 3 },
    { 0xbf0bf2ad, 3 },
    { 0, 0 },
    { 0xd73f2af, 3 },
    { 0, 0 },
    { 0x993ff2b1, 3 },
    { 0x9b97f2b2, 3 },
    { 0xa33ff2b3, 3 },
    { 0x2273f2b4, 3 },
    { 0x4fd3f2b3, 3 },
    { 0x8b9bf2b5, 3 },
    { 0xddc3f2b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597f2ba, 3 },
    { 0x2e9bf2bb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27f2c1, 3 },
    { 0xe8e3f2c1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dbf2cc, 3 },
    { 0xcc5ff2cd, 3 },
    { 0xab4ff2ce, 3 },
    { 0xfc23f2ce, 3 },
    { 0xaf1bf2ce, 3 },
    { 0x304ff2d1, 3 },
    { 0x5347f2d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b73f2d8, 3 },
    { 0x603bf2d9, 5 },
    { 0x2383f2da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cff2de, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efff2e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a7f2e7, 3 },
    { 0x5febf2e8, 3 },
    { 0xdaff2e9, 4 },
    { 0x8eff2ea, 4 },
    { 0x777ff2eb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe3f2f3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e3f2f6, 4 },
    { 0x98cbf2f6, 5 },
    { 0x6dc7f2f7, 4 },
    { 0xaaa3f2f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6ff2ff, 3 },
    { 0x72abf300, 3 },
    { 0, 0 },
    { 0xf95bf302, 3 },
    { 0x2c1ff302, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5367f30d, 3 },
    { 0xeff7f30e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf313, 3 },
    { 0x2e27f313, 4 },
    { 0xdfe3f314, 4 },
    { 0x385bf313, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974bf31d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7f322, 4 },
    { 0, 0 },
    { 0x31fff324, 3 },
    { 0, 0 },
    { 0x2a07f326, 4 },
    { 0, 0 },
    { 0x402ff328, 4 },
    { 0, 0 },
    { 0x9ea7f32a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ff32d, 3 },
    { 0x6623f32e, 3 },
    { 0, 0 },
    { 0x84b3f330, 3 },
    { 0, 0 },
    { 0xcaebf332, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa263f336, 3 },
    { 0xa487f337, 3 },
    { 0xd687f338, 3 },
    { 0xb8eff339, 4 },
    { 0xcd67f339, 3 },
    { 0x2957f33b, 4 },
    { 0, 0 },
    { 0x6fbbf33d, 4 },
    { 0x392bf33e, 3 },
    { 0xff7f33e, 3 },
    { 0xb55bf340, 4 },
    { 0x9733f33f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebff344, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ff34a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b3f350, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b03f353, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ff35a, 4 },
    { 0x94f3f35a, 4 },
    { 0xbab3f35c, 3 },
    { 0x8d6ff35c, 4 },
    { 0x3533f35e, 3 },
    { 0x8997f35f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ff362, 3 },
    { 0xf54bf363, 3 },
    { 0, 0 },
    { 0x66ebf365, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8bf368, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722bf36c, 3 },
    { 0x6103f36d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac7f377, 4 },
    { 0x39dff378, 3 },
    { 0xe2dff379, 3 },
    { 0x41eff378, 3 },
    { 0xdc9ff37b, 3 },
    { 0xab8ff37c, 3 },
    { 0xf90bf37c, 3 },
    { 0x3f63f37c, 3 },
    { 0x782ff37f, 3 },
    { 0, 0 },
    { 0x39dbf381, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebf388, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc3f38b, 3 },
    { 0x9e8bf38b, 3 },
    { 0xd097f38d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a73f392, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57abf398, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c3f39f, 3 },
    { 0xd6dbf39f, 3 },
    { 0x7f43f3a1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ff3a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc093f3a9, 3 },
    { 0x4f17f3a9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbff3ad, 3 },
    { 0x21aff3ad, 5 },
    { 0x8517f3ae, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7133f3b2, 4 },
    { 0xd0eff3b3, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe317f3b6, 4 },
    { 0x7ebf3b7, 3 },
    { 0xf61ff3b6, 3 },
    { 0x5d6ff3b9, 3 },
    { 0x3273f3ba, 3 },
    { 0xa9cff3bb, 3 },
    { 0x8b9ff3b9, 3 },
    { 0x690bf3bd, 4 },
    { 0x62f3f3be, 3 },
    { 0x5deff3be, 4 },
    { 0x6453f3bc, 3 },
    { 0x9323f3c1, 4 },
    { 0x45aff3c2, 4 },
    { 0, 0 },
    { 0x9b07f3c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3f3ca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fbf3d1, 3 },
    { 0, 0 },
    { 0x6abf3d3, 3 },
    { 0, 0 },
    { 0x2c17f3d5, 4 },
    { 0x49cbf3d6, 3 },
    { 0x2d13f3d7, 4 },
    { 0x375ff3d7, 3 },
    { 0x1927f3d6, 4 },
    { 0x9ac3f3da, 4 },
    { 0x40d3f3d9, 3 },
    { 0xc54bf3dc, 4 },
    { 0x50cbf3dd, 3 },
    { 0, 0 },
    { 0x97bbf3df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc3f3e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadbf3e5, 3 },
    { 0xc5d3f3e6, 3 },
    { 0x3773f3e5, 3 },
    { 0, 0 },
    { 0xdf2ff3e9, 3 },
    { 0, 0 },
    { 0x62dbf3eb, 3 },
    { 0x63e3f3ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe827f3f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ebf3f3, 3 },
    { 0xccff3f3, 3 },
    { 0, 0 },
    { 0x9c57f3f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7177f3fb, 3 },
    { 0xa227f3fc, 3 },
    { 0xfc6bf3fb, 3 },
    { 0xa407f3fe, 3 },
    { 0x91fff3fd, 3 },
    { 0xd0cff3ff, 4 },
    { 0, 0 },
    { 0x9757f402, 5 },
    { 0, 0 },
    { 0xfdebf404, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d33f407, 4 },
    { 0, 0 },
    { 0x7e77f409, 3 },
    { 0xa15bf409, 4 },
    { 0x676bf40b, 3 },
    { 0xc7a7f40c, 3 },
    { 0x88bf40b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7f410, 3 },
    { 0xa13ff410, 4 },
    { 0xee57f412, 4 },
    { 0x3afff410, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x242ff416, 3 },
    { 0, 0 },
    { 0x2197f418, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa373f41b, 3 },
    { 0x34dff41c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d7f424, 3 },
    { 0xd3f7f425, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d13f428, 3 },
    { 0xdbcff428, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a7f42e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7ff432, 3 },
    { 0, 0 },
    { 0xa32ff434, 3 },
    { 0x9dc3f434, 3 },
    { 0x6f2bf436, 5 },
    { 0x32dbf437, 4 },
    { 0xf553f434, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a3f43b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd23f43e, 3 },
    { 0xe43bf43e, 3 },
    { 0x6fb7f43f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebf449, 4 },
    { 0x2f83f44a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7f44d, 4 },
    { 0x7743f44e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf3f452, 3 },
    { 0x8feff453, 3 },
    { 0, 0 },
    { 0xda8bf455, 3 },
    { 0x727bf456, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdbf459, 4 },
    { 0x816ff45a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517f45f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x536ff462, 3 },
    { 0, 0 },
    { 0xf8f7f464, 4 },
    { 0, 0 },
    { 0x9db3f466, 3 },
    { 0x6273f467, 3 },
    { 0x25dff468, 4 },
    { 0, 0 },
    { 0x4743f46a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbff46d, 3 },
    { 0xc7cbf46d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2873f477, 4 },
    { 0xe067f478, 3 },
    { 0xf533f479, 3 },
    { 0x809ff478, 3 },
    { 0xab53f47b, 3 },
    { 0xffb7f47b, 3 },
    { 0, 0 },
    { 0xbf33f47e, 3 },
    { 0, 0 },
    { 0x1dfff480, 3 },
    { 0x67dbf480, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4463f484, 3 },
    { 0, 0 },
    { 0xd2d3f486, 3 },
    { 0xa9fff486, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf037f48b, 3 },
    { 0x48a3f48c, 3 },
    { 0, 0 },
    { 0x13b7f48e, 4 },
    { 0, 0 },
    { 0xff33f490, 3 },
    { 0x3297f491, 4 },
    { 0x740ff492, 3 },
    { 0x58a3f493, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0ff496, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcabf499, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdff49e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7f4a3, 4 },
    { 0, 0 },
    { 0x3b1ff4a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ebf4a8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb567f4ac, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa7f4af, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eff4b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83f4c0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf633f4c4, 3 },
    { 0, 0 },
    { 0x363f4c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5843f4cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e7f4d3, 4 },
    { 0, 0 },
    { 0x2ddbf4d5, 3 },
    { 0, 0 },
    { 0x7307f4d7, 3 },
    { 0xa043f4d7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xef43f4db, 3 },
    { 0, 0 },
    { 0x466bf4dd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feff4e2, 3 },
    { 0, 0 },
    { 0x14bf4e4, 4 },
    { 0x120ff4e4, 4 },
    { 0xcd27f4e6, 3 },
    { 0xa547f4e6, 3 },
    { 0xa96ff4e8, 4 },
    { 0x50bbf4e9, 3 },
    { 0x7a17f4e9, 3 },
    { 0, 0 },
    { 0x5bcbf4ec, 3 },
    { 0x5d13f4ed, 3 },
    { 0x733f4ee, 3 },
    { 0x90d7f4ef, 4 },
    { 0x9e17f4f0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacff4f3, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb93f4f6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc87f4fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4853f4fe, 4 },
    { 0xeeaff4fe, 4 },
    { 0xf517f4ff, 3 },
    { 0xb103f501, 3 },
    { 0x3cf7f500, 3 },
    { 0xd02ff501, 3 },
    { 0x293f4fe, 3 },
    { 0x4e1bf505, 3 },
    { 0xf813f506, 4 },
    { 0, 0 },
    { 0x4c1bf508, 3 },
    { 0xe5ebf509, 4 },
    { 0, 0 },
    { 0x8017f50b, 3 },
    { 0, 0 },
    { 0xaceff50d, 3 },
    { 0x9643f50e, 3 },
    { 0xfd57f50f, 3 },
    { 0xeba3f510, 3 },
    { 0x9377f50f, 3 },
    { 0x872ff512, 4 },
    { 0xe783f510, 3 },
    { 0x5e13f513, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5713f518, 3 },
    { 0xc363f518, 3 },
    { 0xa713f51a, 3 },
    { 0, 0 },
    { 0x1437f51c, 3 },
    { 0xb8fff51d, 4 },
    { 0, 0 },
    { 0xda1bf51f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe13f525, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bbf52d, 3 },
    { 0xaa0ff52e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c33f532, 3 },
    { 0xf3bff533, 3 },
    { 0xa273f534, 4 },
    { 0, 0 },
    { 0xceebf536, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3f539, 3 },
    { 0, 0 },
    { 0xdb27f53b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f87f53e, 4 },
    { 0x9143f53e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9bf546, 4 },
    { 0x5e8bf547, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1abf54b, 3 },
    { 0, 0 },
    { 0xb46ff54d, 3 },
    { 0x48bff54d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8ff553, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe353f556, 3 },
    { 0x8c97f556, 3 },
    { 0x64bff556, 4 },
    { 0x4debf559, 3 },
    { 0x5b37f55a, 3 },
    { 0xedbf55b, 3 },
    { 0x6a03f55a, 3 },
    { 0xe3ebf55d, 3 },
    { 0xa583f55d, 3 },
    { 0xa3f3f55d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ff562, 4 },
    { 0, 0 },
    { 0xb593f564, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea3f568, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bbf56c, 3 },
    { 0xb5dbf56c, 3 },
    { 0x6077f56e, 3 },
    { 0x4e47f56f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794bf577, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f27f57a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a07f57f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f585, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8ff589, 3 },
    { 0xa147f58a, 3 },
    { 0xda37f58b, 3 },
    { 0, 0 },
    { 0x6c0ff58d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa863f593, 3 },
    { 0x4307f594, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf597, 3 },
    { 0x5d97f597, 3 },
    { 0, 0 },
    { 0xb037f59a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a57f5a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01ff5a5, 3 },
    { 0x8e3f5a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e13f5a9, 4 },
    { 0, 0 },
    { 0x6ae3f5ab, 4 },
    { 0x6597f5ab, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a47f5b1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae3f5b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7f5b9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cff5be, 4 },
    { 0, 0 },
    { 0x995bf5c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x6587f5c3, 3 },
    { 0xc68bf5c4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74bf5c7, 4 },
    { 0x1f37f5c7, 3 },
    { 0x6f5bf5c7, 3 },
    { 0x29a7f5ca, 3 },
    { 0, 0 },
    { 0x415bf5cc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe33f5d2, 3 },
    { 0x7eebf5d3, 3 },
    { 0x4d3bf5d3, 3 },
    { 0xb127f5d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763f5db, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c3f5e2, 4 },
    { 0, 0 },
    { 0xe83bf5e4, 4 },
    { 0xf163f5e4, 5 },
    { 0x3a07f5e6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9abf5eb, 3 },
    { 0, 0 },
    { 0x556ff5ed, 3 },
    { 0xdff5ee, 3 },
    { 0x5427f5ef, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3bf5f7, 4 },
    { 0, 0 },
    { 0xcfbff5f9, 3 },
    { 0x81f7f5fa, 4 },
    { 0xc6bf5f9, 4 },
    { 0x3993f5fc, 3 },
    { 0x293f5fd, 4 },
    { 0xe9dbf5fb, 3 },
    { 0xbb7bf5ff, 3 },
    { 0xf4c3f600, 3 },
    { 0x3b1ff600, 3 },
    { 0, 0 },
    { 0x8e9bf603, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2bf607, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938bf610, 3 },
    { 0xe457f611, 3 },
    { 0, 0 },
    { 0xb63ff613, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fff616, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817f61b, 3 },
    { 0x2bdbf61c, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9ff625, 3 },
    { 0x8b33f626, 3 },
    { 0x438bf626, 3 },
    { 0, 0 },
    { 0x7223f629, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e7f62c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57ff62f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bf632, 3 },
    { 0x9413f632, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdff638, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f53f63e, 4 },
    { 0xa687f63f, 4 },
    { 0x2b53f63f, 3 },
    { 0xbd73f641, 4 },
    { 0, 0 },
    { 0xad57f643, 3 },
    { 0x8f37f644, 3 },
    { 0x697bf644, 3 },
    { 0x5ad3f646, 3 },
    { 0x519bf643, 3 },
    { 0x707ff646, 3 },
    { 0xe99bf644, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd3f64c, 3 },
    { 0xdec3f64d, 3 },
    { 0xe31ff64e, 4 },
    { 0x24b7f64f, 3 },
    { 0xdd83f650, 4 },
    { 0x846bf650, 3 },
    { 0xb293f651, 3 },
    { 0x6dbff650, 3 },
    { 0, 0 },
    { 0x415ff655, 3 },
    { 0x375ff656, 3 },
    { 0xa393f657, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf3f65a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7297f669, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73ff66d, 3 },
    { 0x4cebf66d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227f672, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcff675, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ff679, 4 },
    { 0x19c3f679, 3 },
    { 0xbddff67a, 3 },
    { 0, 0 },
    { 0x515ff67d, 3 },
    { 0xf36bf67e, 3 },
    { 0xd88bf67d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57f683, 3 },
    { 0, 0 },
    { 0xd2bff685, 5 },
    { 0, 0 },
    { 0xbdeff687, 3 },
    { 0x778ff688, 3 },
    { 0x35abf689, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd393f68d, 3 },
    { 0xd3f3f68e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x708bf691, 3 },
    { 0x990bf692, 3 },
    { 0x3773f691, 3 },
    { 0, 0 },
    { 0x8d73f695, 4 },
    { 0, 0 },
    { 0x698ff697, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xece3f69a, 5 },
    { 0x952ff69b, 3 },
    { 0xe87bf69b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bbf69f, 3 },
    { 0x58dbf6a0, 3 },
    { 0xa72bf6a0, 3 },
    { 0x78e7f69f, 3 },
    { 0xdd3bf6a3, 3 },
    { 0, 0 },
    { 0xaf77f6a5, 3 },
    { 0, 0 },
    { 0xdfcbf6a7, 3 },
    { 0xdedbf6a7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6967f6ac, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb7f6b2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7f6b6, 3 },
    { 0xb9a3f6b7, 3 },
    { 0x990ff6b8, 5 },
    { 0x6dff6b9, 3 },
    { 0x14abf6b9, 5 },
    { 0, 0 },
    { 0x540bf6bc, 3 },
    { 0, 0 },
    { 0x520bf6be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1493f6c1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ff6c6, 3 },
    { 0xc8ff6c6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53f6ce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64bf6d2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cff6d5, 3 },
    { 0x9e47f6d6, 3 },
    { 0xc847f6d5, 4 },
    { 0xe06bf6d8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eff6dd, 3 },
    { 0, 0 },
    { 0xff83f6df, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1807f6e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44bf6e5, 3 },
    { 0x95a7f6e5, 3 },
    { 0x174bf6e5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2ff6ea, 3 },
    { 0x6bc3f6ea, 3 },
    { 0xba27f6ec, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ff6f0, 3 },
    { 0xc997f6f0, 4 },
    { 0x6a57f6f2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bbf6f7, 3 },
    { 0x7167f6f8, 3 },
    { 0, 0 },
    { 0x8d4bf6fa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454bf6fe, 3 },
    { 0x816ff6ff, 3 },
    { 0, 0 },
    { 0x4703f701, 3 },
    { 0x99fbf701, 3 },
    { 0x68b3f703, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7bf706, 3 },
    { 0x2667f707, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608bf70b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3f718, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc7f723, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dff728, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf633f731, 3 },
    { 0xb32bf731, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff736, 3 },
    { 0, 0 },
    { 0xd9e3f738, 4 },
    { 0x963ff739, 3 },
    { 0, 0 },
    { 0x59c7f73b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecbf73e, 3 },
    { 0x4adff73f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9aff744, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bff748, 3 },
    { 0, 0 },
    { 0xea67f74a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31ff74f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ebf756, 4 },
    { 0xb55bf756, 3 },
    { 0x9403f758, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff75c, 4 },
    { 0xc24bf75d, 3 },
    { 0x12cbf75e, 4 },
    { 0x3307f75f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c3f765, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae7f768, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379bf76d, 3 },
    { 0x6413f76e, 4 },
    { 0x7bf7f76e, 3 },
    { 0x986ff76e, 3 },
    { 0xf3cff771, 4 },
    { 0x8e53f770, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840ff778, 4 },
    { 0x23e7f779, 3 },
    { 0x442bf77a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527f780, 3 },
    { 0x25cff781, 3 },
    { 0xd03bf781, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5317f78e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106bf793, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fbf796, 3 },
    { 0, 0 },
    { 0x464bf798, 4 },
    { 0x3f1ff799, 4 },
    { 0x85a3f79a, 3 },
    { 0xa983f79a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc117f7a1, 3 },
    { 0xe567f7a2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67f7a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d87f7ad, 3 },
    { 0x6a47f7ae, 3 },
    { 0x3cd3f7af, 4 },
    { 0, 0 },
    { 0xaa57f7b1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9ff7b4, 3 },
    { 0x934ff7b5, 3 },
    { 0xffe7f7b6, 4 },
    { 0x8913f7b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc87f7bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc947f7c1, 3 },
    { 0x12ff7c2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04bf7c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3727f7ca, 3 },
    { 0x938bf7cb, 3 },
    { 0, 0 },
    { 0x27fff7cd, 3 },
    { 0xca1ff7ce, 4 },
    { 0x62fbf7cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f3f7d8, 3 },
    { 0x2c93f7d8, 3 },
    { 0x7333f7d8, 4 },
    { 0, 0 },
    { 0xdd7f7dc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ff7e6, 4 },
    { 0xa27ff7e7, 3 },
    { 0x4d7bf7e7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ebf7eb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ebf7f0, 3 },
    { 0x95b7f7f1, 3 },
    { 0xc09bf7f1, 3 },
    { 0x1d33f7f1, 4 },
    { 0x8acbf7f1, 3 },
    { 0x8f97f7f3, 3 },
    { 0xcab3f7f6, 3 },
    { 0x86cff7f6, 3 },
    { 0x2577f7f8, 3 },
    { 0x2523f7f8, 3 },
    { 0x905ff7fa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a7f800, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c77f805, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7f80a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37f81b, 3 },
    { 0x373ff81b, 3 },
    { 0x7f5ff81d, 3 },
    { 0x3863f81e, 3 },
    { 0x5853f81b, 3 },
    { 0xf913f820, 4 },
    { 0x646bf820, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb113f828, 4 },
    { 0x619bf829, 3 },
    { 0, 0 },
    { 0xce87f82b, 3 },
    { 0, 0 },
    { 0x553f82d, 3 },
    { 0, 0 },
    { 0xd467f82f, 3 },
    { 0xb11bf830, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3517f833, 3 },
    { 0xd1bff833, 3 },
    { 0, 0 },
    { 0x29b3f836, 3 },
    { 0x864bf836, 3 },
    { 0x1983f838, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fff83c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3f83f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3f843, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a3f846, 4 },
    { 0xbdfff846, 4 },
    { 0x1b77f848, 3 },
    { 0xa63f849, 3 },
    { 0xb6b3f84a, 4 },
    { 0x6927f84b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48ff84e, 3 },
    { 0xa72bf84f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0abf852, 3 },
    { 0, 0 },
    { 0x3b3ff854, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656ff85d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3f862, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d7f86a, 3 },
    { 0x1c13f86b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afff86e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227f874, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336bf878, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f7f87b, 4 },
    { 0x83f87b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad7f87f, 4 },
    { 0x97b3f880, 4 },
    { 0x350bf87f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8ff885, 3 },
    { 0x4a77f886, 3 },
    { 0, 0 },
    { 0x4583f888, 3 },
    { 0x16e7f889, 3 },
    { 0xdc83f888, 4 },
    { 0, 0 },
    { 0xef5bf88c, 3 },
    { 0x3da3f88d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cff893, 4 },
    { 0x2963f894, 3 },
    { 0, 0 },
    { 0x1ca3f896, 3 },
    { 0x1153f897, 4 },
    { 0x5e27f898, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1043f89b, 3 },
    { 0, 0 },
    { 0x6ab3f89d, 4 },
    { 0xde03f89e, 5 },
    { 0xe53f89e, 3 },
    { 0x1a07f89d, 3 },
    { 0x4f43f8a1, 3 },
    { 0, 0 },
    { 0x8033f8a3, 3 },
    { 0xd553f8a4, 3 },
    { 0, 0 },
    { 0x76c3f8a6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657f8aa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb33f8b2, 3 },
    { 0, 0 },
    { 0x6f87f8b4, 3 },
    { 0xf3fbf8b5, 3 },
    { 0x7a0bf8b6, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c3f8bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49ff8bf, 4 },
    { 0, 0 },
    { 0x1247f8c1, 3 },
    { 0, 0 },
    { 0xc4d3f8c3, 3 },
    { 0x8d33f8c3, 3 },
    { 0x235ff8c5, 3 },
    { 0x56ff8c6, 3 },
    { 0x66b3f8c6, 5 },
    { 0xc6c3f8c7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bff8cb, 3 },
    { 0xacbf8cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86bf8d0, 4 },
    { 0, 0 },
    { 0xa327f8d2, 3 },
    { 0, 0 },
    { 0x6e7f8d4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7323f8d8, 3 },
    { 0x387f8d8, 3 },
    { 0xac77f8da, 3 },
    { 0xb22ff8d8, 3 },
    { 0, 0 },
    { 0x9eabf8dd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcff8e2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827f8e6, 3 },
    { 0, 0 },
    { 0x9e97f8e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6727f8f3, 4 },
    { 0, 0 },
    { 0xaf0ff8f5, 3 },
    { 0xcad7f8f6, 3 },
    { 0x296ff8f5, 3 },
    { 0x3983f8f7, 3 },
    { 0xe8fbf8f5, 3 },
    { 0xba03f8f8, 3 },
    { 0, 0 },
    { 0x4a17f8fc, 3 },
    { 0xc61ff8fc, 3 },
    { 0x7d7bf8fe, 3 },
    { 0, 0 },
    { 0x1e2ff900, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe533f903, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4ff907, 4 },
    { 0xa4cff908, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1307f90d, 3 },
    { 0x60bf90e, 3 },
    { 0, 0 },
    { 0x8acff910, 3 },
    { 0x7f83f911, 3 },
    { 0x6617f911, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7f919, 3 },
    { 0x3ebf91a, 3 },
    { 0x92c7f91a, 3 },
    { 0, 0 },
    { 0x834ff91d, 3 },
    { 0x89a7f91e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa973f922, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d3f926, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48bf92f, 3 },
    { 0x7167f930, 3 },
    { 0x420ff930, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55bf936, 3 },
    { 0, 0 },
    { 0x572ff938, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539bf93d, 3 },
    { 0, 0 },
    { 0xc153f93f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x399ff942, 4 },
    { 0, 0 },
    { 0x3b4ff944, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7f94a, 3 },
    { 0xda0ff94b, 4 },
    { 0x29ff94b, 3 },
    { 0x7083f94c, 4 },
    { 0x71c7f94e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e7f95a, 3 },
    { 0x71dbf95b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe957f95e, 3 },
    { 0x71b3f95f, 4 },
    { 0x8d63f95f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dbf964, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dff96c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39bf977, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3f97a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3f97e, 5 },
    { 0x1df7f97f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ff984, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287f98c, 3 },
    { 0xa5c3f98d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ff995, 3 },
    { 0xd75ff995, 3 },
    { 0xba0bf996, 3 },
    { 0, 0 },
    { 0x12dff999, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf137f99c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7f99f, 3 },
    { 0x1a3ff9a0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ebf9a6, 3 },
    { 0x196ff9a7, 3 },
    { 0, 0 },
    { 0xa4bff9a9, 5 },
    { 0x575ff9aa, 3 },
    { 0, 0 },
    { 0x2847f9ac, 4 },
    { 0x7253f9ad, 3 },
    { 0x5e97f9ae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eabf9b4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xed63f9b7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df7f9ba, 3 },
    { 0xca3bf9bb, 4 },
    { 0x4013f9bb, 4 },
    { 0x8ff3f9bb, 5 },
    { 0x2d2ff9be, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d3f9c2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bbf9c6, 4 },
    { 0, 0 },
    { 0x47ff9c8, 3 },
    { 0, 0 },
    { 0x93fff9ca, 3 },
    { 0, 0 },
    { 0x4f87f9cc, 3 },
    { 0xa9bff9cd, 4 },
    { 0xa79bf9cd, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf467f9d3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d3f9d6, 3 },
    { 0xdbe3f9d7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f7f9da, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766bf9e0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f3f9e4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fff9e8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3957f9ec, 4 },
    { 0, 0 },
    { 0x705ff9ee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3f9f1, 4 },
    { 0x8e3bf9f2, 3 },
    { 0xdbe7f9f3, 3 },
    { 0x329ff9f3, 3 },
    { 0x8ac3f9f3, 3 },
    { 0x157f9f3, 3 },
    { 0x98fbf9f7, 3 },
    { 0x169bf9f8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9913f9fe, 3 },
    { 0x6b3ff9ff, 3 },
    { 0xc3affa00, 3 },
    { 0x2b1ffa01, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdffa04, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e7fa07, 3 },
    { 0x1307fa07, 4 },
    { 0x812ffa09, 3 },
    { 0x178bfa07, 3 },
    { 0xf437fa08, 4 },
    { 0x1a17fa0c, 3 },
    { 0x3d57fa0a, 3 },
    { 0x7a63fa0c, 3 },
    { 0xe3effa0e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a83fa12, 4 },
    { 0, 0 },
    { 0x5f27fa14, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff97fa18, 3 },
    { 0x529bfa19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266bfa1e, 3 },
    { 0, 0 },
    { 0xa50ffa20, 4 },
    { 0xd0e3fa21, 3 },
    { 0x4773fa22, 3 },
    { 0x30f3fa23, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb83fa26, 3 },
    { 0x290ffa27, 3 },
    { 0x9467fa28, 3 },
    { 0xa6fbfa28, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37fa2c, 3 },
    { 0, 0 },
    { 0xdbbffa2e, 4 },
    { 0xbf77fa2e, 3 },
    { 0xf14ffa30, 3 },
    { 0xbcd7fa2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1263fa3a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x738ffa3d, 3 },
    { 0, 0 },
    { 0xc56bfa3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bfa42, 3 },
    { 0x1f73fa43, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a7fa46, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa607fa4b, 3 },
    { 0xc9d3fa4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba7fa4f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d7fa52, 3 },
    { 0x55affa52, 4 },
    { 0xc9effa52, 3 },
    { 0, 0 },
    { 0xdb73fa56, 5 },
    { 0x94a3fa57, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf957fa5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8ffa5d, 3 },
    { 0, 0 },
    { 0x457ffa5f, 3 },
    { 0, 0 },
    { 0x7f0ffa61, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd497fa65, 4 },
    { 0, 0 },
    { 0xca0bfa67, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f17fa6a, 3 },
    { 0x5a3fa6a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf663fa6e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681ffa72, 4 },
    { 0, 0 },
    { 0x9557fa74, 5 },
    { 0x535bfa75, 3 },
    { 0xc61ffa76, 3 },
    { 0x59bfa76, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fa7d, 3 },
    { 0xff5bfa7e, 3 },
    { 0, 0 },
    { 0x893bfa80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ffa83, 4 },
    { 0x9e73fa84, 3 },
    { 0, 0 },
    { 0x485ffa86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358ffa8a, 3 },
    { 0x5d4bfa8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7fa90, 4 },
    { 0, 0 },
    { 0x7a77fa92, 5 },
    { 0, 0 },
    { 0x9dcffa94, 4 },
    { 0, 0 },
    { 0xd4bfa96, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2bfa9c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720bfaa3, 4 },
    { 0x6c5bfaa4, 3 },
    { 0x4397faa3, 3 },
    { 0x117faa6, 4 },
    { 0, 0 },
    { 0xa2effaa8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da7faae, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faffab1, 4 },
    { 0x31f3fab1, 3 },
    { 0xde8ffab3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bfab6, 3 },
    { 0x51bffab7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe193fabb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x263ffabe, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de7fac1, 3 },
    { 0x18c7fac1, 3 },
    { 0xaacbfac3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd07fac8, 3 },
    { 0x865ffac9, 3 },
    { 0, 0 },
    { 0xaddbfacb, 3 },
    { 0x9a17facb, 4 },
    { 0xb97ffacd, 3 },
    { 0, 0 },
    { 0xf913facf, 3 },
    { 0xbc53fad0, 5 },
    { 0xcc17facf, 3 },
    { 0, 0 },
    { 0xcd97fad3, 3 },
    { 0xe85ffad4, 3 },
    { 0, 0 },
    { 0x854bfad6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddbfada, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e7fadd, 3 },
    { 0xff2bfade, 3 },
    { 0x9737fade, 3 },
    { 0x59b7fae0, 4 },
    { 0x44e3fae1, 4 },
    { 0x86ebfadd, 3 },
    { 0x9d73fae1, 4 },
    { 0x9f93fade, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2ffaed, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3faf0, 3 },
    { 0xf86ffaf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed13faf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1563fb00, 3 },
    { 0, 0 },
    { 0x946ffb02, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x70abfb05, 3 },
    { 0x2d23fb06, 5 },
    { 0, 0 },
    { 0x54dbfb08, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bbfb0d, 3 },
    { 0x1de7fb0d, 3 },
    { 0x8ad7fb0e, 3 },
    { 0, 0 },
    { 0xb76bfb11, 3 },
    { 0xfd1bfb11, 3 },
    { 0xb5fffb13, 3 },
    { 0x8293fb14, 3 },
    { 0x8a9ffb15, 4 },
    { 0, 0 },
    { 0xba2bfb17, 3 },
    { 0xf487fb18, 3 },
    { 0xb38bfb19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc423fb1d, 4 },
    { 0, 0 },
    { 0x56abfb1f, 3 },
    { 0, 0 },
    { 0x77d3fb21, 3 },
    { 0x9e47fb21, 3 },
    { 0x9253fb22, 3 },
    { 0xc193fb24, 3 },
    { 0, 0 },
    { 0x81bfb26, 3 },
    { 0x789ffb27, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899bfb2c, 3 },
    { 0, 0 },
    { 0xaa07fb2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140ffb32, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43fb37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88ffb3b, 3 },
    { 0x6a93fb3c, 3 },
    { 0x6cf3fb3d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e63fb41, 3 },
    { 0, 0 },
    { 0xc1cbfb43, 3 },
    { 0x1a5ffb44, 3 },
    { 0x9e0bfb45, 3 },
    { 0x58c7fb43, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47ffb4d, 3 },
    { 0x5e93fb4e, 3 },
    { 0x7fbffb4f, 3 },
    { 0xbd6ffb50, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x853ffb53, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b83fb57, 4 },
    { 0x373bfb58, 3 },
    { 0xebb3fb57, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xded3fb5c, 3 },
    { 0xc65bfb5d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f7fb63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42bfb67, 3 },
    { 0xedfbfb68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x641bfb6b, 3 },
    { 0x7eebfb6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63bfb72, 3 },
    { 0x1e03fb73, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d93fb78, 3 },
    { 0x6bcbfb79, 3 },
    { 0xa11ffb7a, 3 },
    { 0, 0 },
    { 0xbbd7fb7c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c93fb82, 3 },
    { 0, 0 },
    { 0x9457fb84, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245ffb8e, 4 },
    { 0, 0 },
    { 0xe95ffb90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3953fb95, 3 },
    { 0xc7fffb95, 4 },
    { 0xf72bfb97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x19abfb9a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c3fba2, 3 },
    { 0xee33fba3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393bfba7, 3 },
    { 0, 0 },
    { 0x80affba9, 3 },
    { 0x34cbfbaa, 3 },
    { 0x9fc3fbab, 3 },
    { 0, 0 },
    { 0xfe87fbad, 3 },
    { 0, 0 },
    { 0xc013fbaf, 3 },
    { 0, 0 },
    { 0x9837fbb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5403fbb4, 3 },
    { 0, 0 },
    { 0xb4effbb6, 3 },
    { 0xd1bfbb7, 4 },
    { 0x44d7fbb8, 3 },
    { 0x173ffbb7, 3 },
    { 0x309bfbba, 3 },
    { 0x50a7fbbb, 3 },
    { 0x30dffbb7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ffbbf, 3 },
    { 0, 0 },
    { 0x9937fbc1, 3 },
    { 0, 0 },
    { 0x78d7fbc3, 4 },
    { 0xd18bfbc4, 4 },
    { 0x4eeffbc5, 3 },
    { 0x1bd7fbc6, 3 },
    { 0, 0 },
    { 0x6143fbc8, 4 },
    { 0, 0 },
    { 0xcb0bfbca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb123fbce, 3 },
    { 0xbaa7fbcf, 3 },
    { 0x87d7fbcf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6897fbd7, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72abfbdb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1bfbdf, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3fbe4, 4 },
    { 0x60b3fbe5, 3 },
    { 0x66bffbe6, 3 },
    { 0xf447fbe6, 4 },
    { 0x1333fbe7, 3 },
    { 0xb33ffbe4, 4 },
    { 0xc063fbe4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf307fbef, 3 },
    { 0, 0 },
    { 0x7ea3fbf1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fffbf5, 3 },
    { 0, 0 },
    { 0x2c83fbf7, 3 },
    { 0xff93fbf7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d3fc01, 4 },
    { 0x5457fc01, 4 },
    { 0xeca3fc03, 4 },
    { 0x51c3fc02, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x328ffc07, 3 },
    { 0x4027fc07, 3 },
    { 0x347fc08, 4 },
    { 0, 0 },
    { 0x366bfc0b, 3 },
    { 0, 0 },
    { 0xba7ffc0d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x913fc10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4527fc15, 4 },
    { 0xb1bffc16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b7fc1b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3fc1e, 4 },
    { 0x126bfc1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dffc2a, 4 },
    { 0xbf07fc2b, 3 },
    { 0xe9a7fc2b, 3 },
    { 0x5647fc2d, 3 },
    { 0x3d27fc2a, 3 },
    { 0xd687fc2c, 3 },
    { 0, 0 },
    { 0xc27ffc31, 4 },
    { 0x2a43fc31, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ffc35, 5 },
    { 0x2c0ffc35, 3 },
    { 0, 0 },
    { 0x3db3fc38, 3 },
    { 0xafe3fc38, 3 },
    { 0, 0 },
    { 0x9323fc3b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63fc4a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3fc50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebfc54, 3 },
    { 0xb1c3fc55, 4 },
    { 0x3683fc56, 3 },
    { 0, 0 },
    { 0x8bb3fc58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x658bfc5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874bfc62, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc237fc65, 3 },
    { 0x7baffc66, 4 },
    { 0x1823fc67, 3 },
    { 0xe387fc68, 3 },
    { 0x541ffc69, 3 },
    { 0, 0 },
    { 0xc717fc6b, 4 },
    { 0x49c3fc6c, 3 },
    { 0x6957fc6d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8bfc70, 3 },
    { 0xfceffc71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c3fc77, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabbfc7b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd893fc7f, 3 },
    { 0x32d7fc80, 3 },
    { 0, 0 },
    { 0x7dc3fc82, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca83fc89, 3 },
    { 0, 0 },
    { 0x3347fc8b, 4 },
    { 0, 0 },
    { 0xcf7fc8d, 3 },
    { 0xeb4bfc8e, 3 },
    { 0xb9e3fc8f, 3 },
    { 0x2bcffc90, 4 },
    { 0, 0 },
    { 0xf233fc92, 3 },
    { 0xf42bfc92, 3 },
    { 0x934ffc93, 3 },
    { 0x523ffc95, 4 },
    { 0x119bfc93, 3 },
    { 0, 0 },
    { 0xad83fc98, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e27fc9f, 3 },
    { 0x62f7fca0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9bfca4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520bfca8, 4 },
    { 0xd9fbfca9, 4 },
    { 0, 0 },
    { 0xba33fcab, 4 },
    { 0x45c7fcab, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69ffcb1, 3 },
    { 0x82bffcb1, 3 },
    { 0x656bfcb3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ffcb6, 4 },
    { 0xd3effcb7, 4 },
    { 0, 0 },
    { 0xbfffcb9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27fcbc, 4 },
    { 0x2723fcbc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697fcc1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3097fcc4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37fcc8, 3 },
    { 0x2bfbfcc8, 3 },
    { 0, 0 },
    { 0x776bfccb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x6613fcce, 4 },
    { 0x3affcce, 4 },
    { 0xba77fcd0, 3 },
    { 0x1573fcd0, 3 },
    { 0xbf6bfccf, 5 },
    { 0, 0 },
    { 0x7f67fcd4, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a27fce0, 4 },
    { 0xa603fce0, 3 },
    { 0xaa03fce2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec7fce7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1163fcea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4827fcee, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efffcf1, 3 },
    { 0xda93fcf1, 3 },
    { 0xf0ffcf3, 3 },
    { 0x980ffcf2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd593fcf9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c3fcfc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e23fd00, 3 },
    { 0x1dcbfd00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba3fd04, 3 },
    { 0x226bfd05, 3 },
    { 0xbca7fd06, 4 },
    { 0xe9a7fd05, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29bfd10, 3 },
    { 0xd8fffd11, 3 },
    { 0x18ffd11, 3 },
    { 0x16c7fd11, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cbfd18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8ffd1e, 3 },
    { 0x26f7fd1f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf667fd25, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafffd28, 3 },
    { 0x5aebfd29, 3 },
    { 0, 0 },
    { 0xe19bfd2b, 3 },
    { 0x645bfd2b, 3 },
    { 0, 0 },
    { 0x724ffd2e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeffd32, 3 },
    { 0xb51bfd33, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3bfd3d, 3 },
    { 0x1203fd3e, 4 },
    { 0xe66ffd3f, 3 },
    { 0x5ffd3f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd7fd45, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf547fd49, 3 },
    { 0x510bfd49, 3 },
    { 0x47bbfd4b, 4 },
    { 0x6bdffd4c, 3 },
    { 0xd4a3fd4c, 4 },
    { 0x3577fd4e, 4 },
    { 0xe4cffd4c, 5 },
    { 0x60ebfd50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84bfd5a, 3 },
    { 0xbf27fd5b, 3 },
    { 0x148ffd5c, 4 },
    { 0x3bc3fd5a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd897fd63, 3 },
    { 0x65affd64, 4 },
    { 0x2cb7fd63, 3 },
    { 0xa033fd66, 4 },
    { 0xf8fbfd67, 4 },
    { 0, 0 },
    { 0x2767fd69, 4 },
    { 0x86fbfd6a, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c63fd71, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71ffd76, 3 },
    { 0xee4bfd77, 3 },
    { 0xfb7ffd76, 4 },
    { 0xc52ffd77, 3 },
    { 0x8e4bfd76, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79bfd7f, 4 },
    { 0x4243fd80, 3 },
    { 0x5f8bfd80, 3 },
    { 0x97abfd82, 3 },
    { 0xcf07fd83, 4 },
    { 0x971ffd84, 3 },
    { 0, 0 },
    { 0x91bfd86, 3 },
    { 0x42bbfd87, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca47fd8b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e7fd8f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x8013fd92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37ffd95, 4 },
    { 0x533ffd95, 4 },
    { 0x47abfd95, 3 },
    { 0, 0 },
    { 0x5ac7fd99, 3 },
    { 0xd39bfd9a, 4 },
    { 0, 0 },
    { 0xc843fd9c, 4 },
    { 0x6bb7fd9d, 3 },
    { 0x3bb7fd9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1293fda1, 3 },
    { 0, 0 },
    { 0xeb83fda3, 4 },
    { 0x761ffda4, 3 },
    { 0, 0 },
    { 0xaad3fda6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be7fda9, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac3fdb1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x831ffdb4, 3 },
    { 0x5e27fdb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbfdb9, 3 },
    { 0xdfaffdb9, 3 },
    { 0, 0 },
    { 0xe19ffdbc, 3 },
    { 0, 0 },
    { 0xbfa3fdbe, 3 },
    { 0xa493fdbe, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19effdc5, 3 },
    { 0x9973fdc6, 4 },
    { 0x9c5bfdc7, 3 },
    { 0xce1ffdc6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1043fdd2, 3 },
    { 0xe48bfdd3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ffdd7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7fdda, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afbfddf, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bfde4, 3 },
    { 0xb3cffde5, 3 },
    { 0xd553fde4, 4 },
    { 0x402bfde7, 5 },
    { 0x81e3fde5, 3 },
    { 0, 0 },
    { 0x9183fdea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae3fdee, 4 },
    { 0xca33fdef, 3 },
    { 0x3c23fdf0, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad27fdf6, 3 },
    { 0xa287fdf7, 4 },
    { 0, 0 },
    { 0xd717fdf9, 3 },
    { 0xe6bfdfa, 4 },
    { 0, 0 },
    { 0xfa5bfdfc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c13fe00, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08bfe03, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x3363fe06, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbffe0d, 4 },
    { 0x794ffe0e, 4 },
    { 0xf527fe0e, 3 },
    { 0x9fdbfe10, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdffe14, 3 },
    { 0xcc13fe15, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675bfe19, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587fe21, 3 },
    { 0xf407fe22, 3 },
    { 0xf44ffe23, 4 },
    { 0, 0 },
    { 0xa52bfe25, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263fe29, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bbfe2c, 3 },
    { 0, 0 },
    { 0xc043fe2e, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faffe31, 3 },
    { 0, 0 },
    { 0x5b1ffe33, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d3fe39, 4 },
    { 0x73f3fe39, 3 },
    { 0xe733fe3b, 4 },
    { 0x48d7fe3c, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332ffe41, 4 },
    { 0xb3fffe41, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763fe49, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3fe4c, 4 },
    { 0xc58ffe4d, 3 },
    { 0, 0 },
    { 0x3b53fe4f, 4 },
    { 0x48dbfe50, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cbfe57, 3 },
    { 0x657fe58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b53fe5c, 3 },
    { 0, 0 },
    { 0xcc93fe5e, 3 },
    { 0xc313fe5f, 4 },
    { 0xf88ffe5f, 3 },
    { 0x743ffe61, 3 },
    { 0, 0 },
    { 0xd633fe63, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfffe6b, 3 },
    { 0x30cbfe6b, 3 },
    { 0, 0 },
    { 0x5ecbfe6e, 3 },
    { 0x1f1bfe6f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8053fe75, 3 },
    { 0xb75ffe75, 3 },
    { 0xf473fe77, 3 },
    { 0xfc73fe78, 4 },
    { 0xf87fe79, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59ffe80, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2bfe86, 3 },
    { 0x9027fe86, 3 },
    { 0, 0 },
    { 0xe357fe89, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fabfe8f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f63fe96, 3 },
    { 0x27fffe97, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6abfe9b, 4 },
    { 0x1223fe9c, 3 },
    { 0x92bbfe9b, 4 },
    { 0xc50bfe9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc3fea3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x1daffea6, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4effea9, 3 },
    { 0x601bfea9, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3feaf, 3 },
    { 0, 0 },
    { 0x849bfeb1, 3 },
    { 0, 0 },
    { 0x6a1ffeb3, 3 },
    { 0, 0 },
    { 0x697feb5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x463feb8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecffebb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33fec2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ffec5, 4 },
    { 0, 0 },
    { 0x4a8bfec7, 3 },
    { 0, 0 },
    { 0xd6dffec9, 3 },
    { 0x8097feca, 4 },
    { 0, 0 },
    { 0xb24bfecc, 4 },
    { 0xa563fecc, 3 },
    { 0x32c3fecc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fffed1, 3 },
    { 0, 0 },
    { 0xaa1bfed3, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb3fed7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1973fedf, 3 },
    { 0x242bfee0, 4 },
    { 0, 0 },
    { 0xb177fee2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1bfee8, 4 },
    { 0, 0 },
    { 0x3897feea, 3 },
    { 0x3e67feeb, 3 },
    { 0xf8c3feea, 3 },
    { 0xf0bfeeb, 3 },
    { 0xab47feee, 4 },
    { 0xe08ffeea, 3 },
    { 0xc49ffeec, 3 },
    { 0x487ffef1, 3 },
    { 0xb183fef1, 3 },
    { 0x5e2bfef3, 3 },
    { 0xf39bfef4, 3 },
    { 0x6e17fef2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa943fefb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4817ff09, 3 },
    { 0x7787ff09, 3 },
    { 0x38dbff0a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d67ff12, 3 },
    { 0, 0 },
    { 0x4a2bff14, 3 },
    { 0, 0 },
    { 0xb5cfff16, 3 },
    { 0, 0 },
    { 0xc5afff18, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a7ff1b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdabff20, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23ff2a, 3 },
    { 0x7ff7ff2b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xa927ff2e, 4 },
    { 0xba57ff2f, 3 },
    { 0x6dbfff2f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf123ff33, 4 },
    { 0x41d7ff34, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9013ff3c, 3 },
    { 0xb5f3ff3d, 3 },
    { 0x891bff3e, 3 },
    { 0x60e3ff3f, 3 },
    { 0xd91fff3f, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88efff4a, 3 },
    { 0, 0 },
    { 0x9b17ff4c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac83ff58, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee3ff5b, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf7ff5f, 3 },
    { 0x74cbff60, 3 },
    { 0x91e7ff61, 3 },
    { 0, 0 },
    { 0xd6b7ff63, 3 },
    { 0, 0 },
    { 0x1db7ff65, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9efff68, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe347ff6b, 3 },
    { 0x5587ff6c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bfff70, 4 },
    { 0x6327ff70, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f7ff74, 4 },
    { 0x352bff74, 3 },
    { 0xc6c7ff76, 4 },
    { 0x466fff77, 4 },
    { 0, 0 },
    { 0x2f5fff79, 4 },
    { 0x67fbff79, 4 },
    { 0x49a3ff7a, 3 },
    { 0xbb1bff7c, 3 },
    { 0xbebbff7c, 4 },
    { 0x486bff7c, 3 },
    { 0xa217ff7f, 3 },
    { 0x8f3bff7e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x187fff83, 3 },
    { 0, 0 },
    { 0x2717ff85, 3 },
    { 0x40c3ff86, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07fff89, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413ff92, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903fff97, 3 },
    { 0x7e9fff98, 4 },
    { 0x65cbff99, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e7ff9d, 3 },
    { 0xb633ff9e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ffa8, 3 },
    { 0, 0 },
    { 0xfd7ffaa, 5 },
    { 0x8e87ffab, 3 },
    { 0, 0 },
    { 0x71f7ffad, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6fffb1, 3 },
    { 0x17bfffb1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1653ffb7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9577ffbb, 5 },
    { 0, 0 },
    { 0xb753ffbd, 4 },
    { 0x6f1fffbd, 3 },
    { 0, 0 },
    { 0x5b3ffc0, 3 },
    { 0, 0 },
    { 0xd1bbffc2, 3 },
    { 0, 0 },
    { 0xc27bffc4, 4 },
    { 0, 0 },
    { 0x89fbffc6, 3 },
    { 0, 0 },
    { 0xd60bffc8, 4 },
    { 0x1417ffc9, 3 },
    { 0xd62fffca, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa3ffce, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d57ffd2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc77ffd5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c7ffdd, 3 },
    { 0x6f83ffde, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7febffe2, 3 },
    { 0, 0 },
    { 0x9e83ffe4, 5 },
    { 0, 0 },
    { 0xe6b3ffe6, 3 },
    { 0x252bffe7, 3 },
    { 0xbdebffe7, 3 },
    { 0x399bffe9, 3 },
    { 0x2803ffea, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe343ffee, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a3fff1, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3bfff5, 3 },
    { 0x1003fff5, 3 },
    { 0x4d03fff7, 3 },
    { 0x2affff8, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6bfffb, 3 },
    { 0x173fffc, 3 },
    { 0, 0 },
    { 0xd8fffffe, 4 },
    { 0xa6e7ffff, 3 }
};


static inline label names_get_wc(uint32_t key) {
    size_t i = key & 0x3ffff;
    for (;;) {
        if (names[i].hash == key) return names[i].value;
        if (names[i].value == 0) return 0;
        i = (i + 1) & 0x3ffff;
    }
}
#define brown_size 0x100000

static const hash32_kv_label brown[brown_size] = {
    { 0x439fffff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7500004, 65 },
    { 0x86f00004, 33 },
    { 0x6c300006, 40 },
    { 0, 0 },
    { 0xc2300008, 90 },
    { 0xb5e00009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a800010, 61 },
    { 0, 0 },
    { 0x35300012, 78 },
    { 0x4ed00012, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b300018, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0001b, 90 },
    { 0xf2a0001c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d0002c, 33 },
    { 0, 0 },
    { 0xaa20002e, 55 },
    { 0x3d90002e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a300033, 90 },
    { 0x6be00034, 43 },
    { 0, 0 },
    { 0x62000036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0003a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720003d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa600040, 33 },
    { 0xa6700041, 78 },
    { 0x9a000042, 91 },
    { 0, 0 },
    { 0xe6400044, 33 },
    { 0xb2000045, 33 },
    { 0x30600044, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0004a, 90 },
    { 0x2e60004b, 33 },
    { 0, 0 },
    { 0x7740004d, 68 },
    { 0, 0 },
    { 0xf590004f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a600052, 61 },
    { 0, 0 },
    { 0x2a900054, 23 },
    { 0x73d00054, 90 },
    { 0x7c900056, 33 },
    { 0x8d700056, 95 },
    { 0xd3300058, 90 },
    { 0x7bd00057, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140005d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4600060, 12 },
    { 0, 0 },
    { 0x73900062, 19 },
    { 0xee600063, 33 },
    { 0x8200064, 61 },
    { 0x70800065, 90 },
    { 0x2db00064, 55 },
    { 0x39400067, 95 },
    { 0xb5c00064, 93 },
    { 0xde00069, 23 },
    { 0xd0200063, 66 },
    { 0x5000006a, 84 },
    { 0xf400006c, 55 },
    { 0xdcb0006c, 95 },
    { 0xdef00062, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e200073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9120007a, 95 },
    { 0xf5a0007b, 76 },
    { 0xaf90007a, 61 },
    { 0x9d90007d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x63200080, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec200087, 55 },
    { 0x28600088, 61 },
    { 0x9b700089, 95 },
    { 0x9a800089, 66 },
    { 0xec60008b, 33 },
    { 0xb900008c, 33 },
    { 0x5980008d, 90 },
    { 0x68a0008e, 99 },
    { 0x8ca0008b, 68 },
    { 0x5c500090, 33 },
    { 0x1d300091, 25 },
    { 0x98600092, 91 },
    { 0x8dd00093, 95 },
    { 0x43400094, 33 },
    { 0xaac00090, 100 },
    { 0x9e800095, 93 },
    { 0, 0 },
    { 0x17400098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9000a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c000a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1000ac, 33 },
    { 0xe5d000ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795000b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263000bd, 98 },
    { 0x82f000be, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4000c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39000c8, 91 },
    { 0x276000c9, 75 },
    { 0x1aa000c9, 33 },
    { 0, 0 },
    { 0x1ca000cc, 67 },
    { 0x599000cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff000d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6000e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493000ed, 95 },
    { 0xbb000ee, 61 },
    { 0x976000ef, 83 },
    { 0, 0 },
    { 0x448000f1, 90 },
    { 0x748000f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54000f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d000fd, 98 },
    { 0x49c000fe, 61 },
    { 0, 0 },
    { 0x2ab00100, 66 },
    { 0, 0 },
    { 0x97900102, 95 },
    { 0xf4700103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4100108, 46 },
    { 0xa900109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f00110, 82 },
    { 0xb5e00111, 18 },
    { 0x28000111, 46 },
    { 0xb2a00113, 99 },
    { 0, 0 },
    { 0xde900115, 78 },
    { 0xba100116, 55 },
    { 0xab100117, 33 },
    { 0, 0 },
    { 0xc5e00119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070011e, 90 },
    { 0x1c10011f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30400123, 81 },
    { 0x3b300124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba00127, 98 },
    { 0xe4000128, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b00137, 95 },
    { 0, 0 },
    { 0x58f00139, 43 },
    { 0xdb60013a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0300141, 52 },
    { 0xbb00141, 95 },
    { 0, 0 },
    { 0x73500144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3100148, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0014c, 40 },
    { 0, 0 },
    { 0xf390014e, 34 },
    { 0xcf0014f, 68 },
    { 0xff20014f, 95 },
    { 0xec600150, 93 },
    { 0x2800152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e00155, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c0015d, 23 },
    { 0x5840015d, 61 },
    { 0xeaf0015f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85900163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b00168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0016b, 55 },
    { 0x8e50016c, 78 },
    { 0x5250016d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33200170, 36 },
    { 0xdfd00171, 98 },
    { 0x25000172, 61 },
    { 0x66700173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b00177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa00183, 61 },
    { 0xd600183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce00187, 33 },
    { 0, 0 },
    { 0x95300189, 23 },
    { 0, 0 },
    { 0x49b0018b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e00196, 88 },
    { 0x46400196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940019c, 23 },
    { 0, 0 },
    { 0xc340019e, 46 },
    { 0x5630019f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb001a4, 61 },
    { 0xa18001a5, 61 },
    { 0, 0 },
    { 0x119001a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2001ae, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0x479001b1, 61 },
    { 0x1a4001b2, 43 },
    { 0, 0 },
    { 0xd2c001b4, 46 },
    { 0xf8a001b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24001bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575001c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384001c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78001cf, 17 },
    { 0xcd1001d0, 98 },
    { 0x578001cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185001d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705001d9, 96 },
    { 0xa68001da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25001e9, 23 },
    { 0xf60001ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d001f0, 23 },
    { 0xb91001f1, 55 },
    { 0xb32001f2, 76 },
    { 0xc3f001f3, 23 },
    { 0x1b1001f4, 33 },
    { 0x5e7001f5, 46 },
    { 0, 0 },
    { 0x118001f7, 90 },
    { 0, 0 },
    { 0x5c001f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5001fc, 36 },
    { 0, 0 },
    { 0xfc7001fe, 100 },
    { 0x1a0001ff, 76 },
    { 0x58200200, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab00208, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72100216, 95 },
    { 0xd7000217, 52 },
    { 0xa8700218, 33 },
    { 0xc6000219, 23 },
    { 0x71b0021a, 76 },
    { 0x1f40021a, 76 },
    { 0, 0 },
    { 0x570021d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b700220, 88 },
    { 0, 0 },
    { 0xeff00222, 98 },
    { 0xa2500222, 98 },
    { 0x6200224, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3400227, 55 },
    { 0x26900228, 95 },
    { 0xd6f00229, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf70022f, 34 },
    { 0, 0 },
    { 0x9f700231, 23 },
    { 0xa9500231, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb00239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b00240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e000247, 55 },
    { 0x12a00248, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf00251, 61 },
    { 0, 0 },
    { 0xb0500253, 75 },
    { 0, 0 },
    { 0xac000255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060025a, 98 },
    { 0, 0 },
    { 0xfe60025c, 60 },
    { 0, 0 },
    { 0x18b0025e, 43 },
    { 0, 0 },
    { 0x92300260, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b700263, 61 },
    { 0, 0 },
    { 0x43600265, 81 },
    { 0xa2400266, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380026b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d10026e, 95 },
    { 0x6a90026f, 28 },
    { 0x39f00270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77500278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50027c, 23 },
    { 0, 0 },
    { 0x4b50027e, 67 },
    { 0xd0027f, 61 },
    { 0x53d0027f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad700283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98600287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60028d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b00290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b00298, 69 },
    { 0x9300299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0029e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e002a2, 98 },
    { 0, 0 },
    { 0xd9f002a4, 59 },
    { 0x2ca002a5, 33 },
    { 0x4e002a6, 98 },
    { 0x6ea002a5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f002ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133002b5, 68 },
    { 0xe39002b6, 52 },
    { 0x285002b7, 77 },
    { 0xf22002b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a002bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519002c1, 90 },
    { 0x80002c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02002c5, 78 },
    { 0xcf6002c6, 76 },
    { 0, 0 },
    { 0x9cb002c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3002ce, 87 },
    { 0, 0 },
    { 0xf83002d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1002d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358002df, 69 },
    { 0x126002e0, 46 },
    { 0, 0 },
    { 0x4bd002e2, 33 },
    { 0x339002e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859002e7, 43 },
    { 0, 0 },
    { 0xba1002e9, 95 },
    { 0x13e002ea, 66 },
    { 0x919002eb, 95 },
    { 0, 0 },
    { 0xbdb002ed, 33 },
    { 0, 0 },
    { 0x562002ef, 88 },
    { 0xaeb002ef, 95 },
    { 0xd60002f1, 52 },
    { 0, 0 },
    { 0x9e7002f3, 19 },
    { 0, 0 },
    { 0x28f002f5, 33 },
    { 0xfe6002f6, 76 },
    { 0x570002f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a002fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8002ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5000306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd610030d, 33 },
    { 0xa0030e, 84 },
    { 0x34a0030f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b400313, 43 },
    { 0xe7e00313, 43 },
    { 0x8400313, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb260031a, 23 },
    { 0xf790031a, 99 },
    { 0, 0 },
    { 0x8600031d, 90 },
    { 0x8060031d, 34 },
    { 0, 0 },
    { 0xd7400320, 23 },
    { 0xc2f00321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a00324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45900328, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe900332, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60033a, 34 },
    { 0x2820033b, 98 },
    { 0xb500033c, 61 },
    { 0xc620033c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44900351, 87 },
    { 0x5cd00352, 90 },
    { 0x11b00353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d00356, 95 },
    { 0, 0 },
    { 0xd2300358, 77 },
    { 0xa6b00359, 66 },
    { 0xa1400358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70035d, 33 },
    { 0x4f40035e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae00364, 88 },
    { 0x2d700364, 61 },
    { 0xd9400366, 90 },
    { 0x3dc00367, 61 },
    { 0xba600368, 75 },
    { 0xbf600369, 78 },
    { 0x9eb0036a, 43 },
    { 0xf7d0036b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0036f, 12 },
    { 0x9cb00370, 33 },
    { 0x7b60036f, 33 },
    { 0xa4600372, 90 },
    { 0x86000372, 33 },
    { 0x60000374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe200379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2020037e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71800384, 98 },
    { 0xedf00385, 95 },
    { 0, 0 },
    { 0xed400387, 98 },
    { 0x1de00388, 43 },
    { 0xc500389, 93 },
    { 0, 0 },
    { 0xaf70038b, 91 },
    { 0, 0 },
    { 0x9d20038d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda600391, 52 },
    { 0x16600392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84600395, 85 },
    { 0, 0 },
    { 0x30300397, 100 },
    { 0xff400397, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1520039b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd820039e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x723003a1, 23 },
    { 0xc9b003a2, 23 },
    { 0xe62003a1, 34 },
    { 0, 0 },
    { 0xb4a003a5, 23 },
    { 0x240003a6, 95 },
    { 0, 0 },
    { 0x838003a8, 66 },
    { 0, 0 },
    { 0x6ef003aa, 33 },
    { 0, 0 },
    { 0x83d003ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a003b1, 56 },
    { 0xd76003b1, 33 },
    { 0x141003b1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813003c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de003c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e003c6, 85 },
    { 0, 0 },
    { 0xb51003c8, 43 },
    { 0x2f2003c9, 96 },
    { 0xab9003c9, 34 },
    { 0xb2f003ca, 60 },
    { 0x3fe003cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51003cf, 33 },
    { 0xde7003d0, 93 },
    { 0x6a4003d1, 56 },
    { 0, 0 },
    { 0x3d9003d3, 61 },
    { 0xa3a003d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14003d9, 90 },
    { 0, 0 },
    { 0x191003db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5003df, 34 },
    { 0xe6e003e0, 61 },
    { 0xf1003e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8003e5, 98 },
    { 0, 0 },
    { 0x8e7003e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb003ee, 23 },
    { 0, 0 },
    { 0x8f1003f0, 66 },
    { 0x7f9003f0, 77 },
    { 0, 0 },
    { 0xe70003f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08003f8, 23 },
    { 0xbd5003f8, 95 },
    { 0xe83003fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750003fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a500408, 48 },
    { 0xc1500409, 17 },
    { 0x7ba0040a, 39 },
    { 0, 0 },
    { 0x2b40040c, 55 },
    { 0x9760040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece00410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47800419, 83 },
    { 0, 0 },
    { 0x2ac0041b, 93 },
    { 0, 0 },
    { 0x9400041d, 77 },
    { 0x72f0041e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2500421, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78400426, 17 },
    { 0xda700427, 43 },
    { 0, 0 },
    { 0x48e00429, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540042c, 90 },
    { 0x7f60042d, 98 },
    { 0xae50042e, 46 },
    { 0x4ae0042f, 34 },
    { 0xfde00430, 90 },
    { 0xd70042f, 61 },
    { 0x5e400432, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda000436, 34 },
    { 0x9eb00437, 95 },
    { 0, 0 },
    { 0xdf100439, 27 },
    { 0, 0 },
    { 0x8730043b, 33 },
    { 0xd050043b, 95 },
    { 0xf0f0043d, 78 },
    { 0x3460043e, 61 },
    { 0xdc40043f, 80 },
    { 0xfde0043f, 36 },
    { 0x7e600441, 33 },
    { 0xea800442, 90 },
    { 0x1bb00443, 34 },
    { 0, 0 },
    { 0x53b00445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0100449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50044c, 90 },
    { 0x74d0044d, 95 },
    { 0xeaa0044e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47700454, 90 },
    { 0x30c00455, 61 },
    { 0, 0 },
    { 0xf1400457, 66 },
    { 0x59600458, 75 },
    { 0xd2e00459, 8 },
    { 0, 0 },
    { 0x82f0045b, 93 },
    { 0x3350045c, 34 },
    { 0, 0 },
    { 0xc7f0045e, 33 },
    { 0, 0 },
    { 0xeaf00460, 90 },
    { 0, 0 },
    { 0xf9000462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2700466, 90 },
    { 0xf3b00467, 66 },
    { 0, 0 },
    { 0xeb000469, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880046c, 90 },
    { 0x14b0046d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e900471, 77 },
    { 0, 0 },
    { 0x6b100473, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff00479, 34 },
    { 0, 0 },
    { 0xc80047b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f700481, 33 },
    { 0, 0 },
    { 0xd7000483, 95 },
    { 0x24400484, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b20048b, 80 },
    { 0xafb0048b, 98 },
    { 0x90e0048c, 87 },
    { 0x16a0048e, 76 },
    { 0, 0 },
    { 0x64400490, 98 },
    { 0x2e900491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4600496, 33 },
    { 0x84100497, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40049a, 23 },
    { 0x7ba0049b, 55 },
    { 0x2090049c, 52 },
    { 0x1430049d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6004a7, 80 },
    { 0xbe2004a8, 33 },
    { 0xbe3004a7, 90 },
    { 0x23e004a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4004ae, 23 },
    { 0x67b004af, 90 },
    { 0x564004af, 90 },
    { 0, 0 },
    { 0x4f004b2, 98 },
    { 0, 0 },
    { 0xc4d004b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x924004b7, 90 },
    { 0, 0 },
    { 0x5ee004b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc004c1, 90 },
    { 0x979004c2, 59 },
    { 0x71d004c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce004c7, 81 },
    { 0xd32004c8, 90 },
    { 0x67b004c9, 43 },
    { 0xb70004ca, 78 },
    { 0x16a004cb, 88 },
    { 0xfe0004ca, 52 },
    { 0xe4d004cd, 17 },
    { 0xec2004c9, 23 },
    { 0xe28004cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4004db, 33 },
    { 0x882004db, 87 },
    { 0, 0 },
    { 0x477004de, 98 },
    { 0, 0 },
    { 0x52d004e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119004e7, 91 },
    { 0, 0 },
    { 0xc5b004e9, 90 },
    { 0x2e9004ea, 88 },
    { 0, 0 },
    { 0x118004ec, 23 },
    { 0x7b2004ec, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af004f1, 87 },
    { 0xd3b004f2, 52 },
    { 0xb1e004f2, 90 },
    { 0x4d2004f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9004fa, 75 },
    { 0xa7a004fb, 75 },
    { 0, 0 },
    { 0x4ad004fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc00502, 17 },
    { 0xc8200503, 40 },
    { 0x22600502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b0050c, 55 },
    { 0, 0 },
    { 0x9f0050e, 12 },
    { 0x69f0050f, 78 },
    { 0x30800510, 33 },
    { 0x4ea0050f, 68 },
    { 0x2f00512, 90 },
    { 0x38800513, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f00518, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb0051b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c800520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d00524, 40 },
    { 0x9ac00525, 76 },
    { 0x7a700526, 85 },
    { 0, 0 },
    { 0x86000528, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0052b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf200533, 43 },
    { 0xf8600534, 91 },
    { 0xdf400535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0053a, 68 },
    { 0x2620053b, 98 },
    { 0, 0 },
    { 0x54e0053d, 78 },
    { 0, 0 },
    { 0xfc00053f, 98 },
    { 0x7900540, 34 },
    { 0x7250053f, 99 },
    { 0, 0 },
    { 0x8b000543, 90 },
    { 0x5ee00543, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d700547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7160054e, 95 },
    { 0xdad0054e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c00553, 87 },
    { 0xbea00554, 90 },
    { 0xcbd00553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf740055c, 43 },
    { 0xe8f0055d, 87 },
    { 0x58d0055d, 43 },
    { 0, 0 },
    { 0x76b00560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6700566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e800569, 90 },
    { 0x8650056a, 98 },
    { 0x3d70056a, 81 },
    { 0x1c80056c, 84 },
    { 0xbde0056b, 95 },
    { 0x37e0056c, 23 },
    { 0x9ac0056f, 91 },
    { 0x86f0056b, 17 },
    { 0x6c60056a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f00579, 66 },
    { 0xd600057a, 90 },
    { 0x9450057b, 76 },
    { 0xcae0057c, 65 },
    { 0x1030057d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea00582, 81 },
    { 0, 0 },
    { 0x3a300584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a00588, 23 },
    { 0, 0 },
    { 0xd660058a, 61 },
    { 0xd1f0058b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b0058e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab00592, 90 },
    { 0x65500593, 66 },
    { 0x24d00593, 43 },
    { 0xadd00595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef00598, 84 },
    { 0xd2800598, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66005a3, 46 },
    { 0xfc9005a4, 98 },
    { 0, 0 },
    { 0x282005a6, 61 },
    { 0xd92005a7, 95 },
    { 0x738005a8, 43 },
    { 0, 0 },
    { 0x171005aa, 98 },
    { 0x9b2005ab, 90 },
    { 0x2fb005ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09005b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96005ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3005bd, 33 },
    { 0x631005be, 100 },
    { 0, 0 },
    { 0x55005c0, 98 },
    { 0xf45005c1, 33 },
    { 0, 0 },
    { 0xa98005c3, 90 },
    { 0x799005c4, 100 },
    { 0xaa2005c5, 33 },
    { 0x570005c6, 90 },
    { 0, 0 },
    { 0x852005c8, 99 },
    { 0xe0c005c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x504005cc, 90 },
    { 0x9ca005cd, 23 },
    { 0, 0 },
    { 0x498005cf, 90 },
    { 0x840005cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06005da, 84 },
    { 0x9e6005db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f005e0, 95 },
    { 0, 0 },
    { 0xbcd005e2, 46 },
    { 0xb19005e2, 98 },
    { 0, 0 },
    { 0xbf005e5, 90 },
    { 0x28a005e6, 87 },
    { 0xe7a005e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3005eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f005ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90005f2, 61 },
    { 0, 0 },
    { 0xd3b005f4, 23 },
    { 0x13005f5, 33 },
    { 0, 0 },
    { 0xc92005f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7005fa, 91 },
    { 0xe53005fa, 90 },
    { 0x960005fc, 81 },
    { 0x117005fc, 55 },
    { 0, 0 },
    { 0xc6c005ff, 76 },
    { 0x7df00600, 46 },
    { 0xc9e00600, 66 },
    { 0x7f200602, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa00608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d500610, 91 },
    { 0, 0 },
    { 0x2d100612, 33 },
    { 0x9da00613, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55400616, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c00619, 97 },
    { 0xccf00619, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e10061d, 90 },
    { 0xfb70061e, 91 },
    { 0x8060061f, 17 },
    { 0x75e00620, 34 },
    { 0, 0 },
    { 0x6ca00622, 93 },
    { 0x31000623, 75 },
    { 0x85000624, 90 },
    { 0xff500625, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d0062a, 61 },
    { 0, 0 },
    { 0x5c70062c, 90 },
    { 0xc0e0062d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2400631, 33 },
    { 0xcc500632, 97 },
    { 0xe5600633, 23 },
    { 0x2ef00633, 76 },
    { 0, 0 },
    { 0x71f00636, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8600639, 23 },
    { 0xcfa00639, 61 },
    { 0x72e0063b, 76 },
    { 0xda0063c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf330063f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2100642, 46 },
    { 0xc9300643, 34 },
    { 0, 0 },
    { 0x73a00645, 23 },
    { 0x28b00646, 90 },
    { 0, 0 },
    { 0x68300648, 33 },
    { 0, 0 },
    { 0xff90064a, 33 },
    { 0x2fe0064a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4630064e, 77 },
    { 0, 0 },
    { 0x66a00650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef00653, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5500656, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97500662, 77 },
    { 0x5ea00663, 76 },
    { 0xb5e00662, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0200669, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x610066c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4100672, 23 },
    { 0, 0 },
    { 0x8a100674, 90 },
    { 0x53a00675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db0067a, 55 },
    { 0xf480067a, 34 },
    { 0xe620067c, 100 },
    { 0, 0 },
    { 0x35c0067e, 23 },
    { 0xd650067f, 95 },
    { 0x1b70067f, 98 },
    { 0x8ec00681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d00686, 99 },
    { 0x9fe00687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b50068c, 100 },
    { 0, 0 },
    { 0xe080068e, 84 },
    { 0xa9e0068f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d00698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23006a1, 55 },
    { 0, 0 },
    { 0x821006a3, 43 },
    { 0x4a2006a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52006a9, 98 },
    { 0x875006aa, 90 },
    { 0x629006a9, 66 },
    { 0x885006ac, 33 },
    { 0x511006ad, 93 },
    { 0x71a006a9, 12 },
    { 0x195006af, 61 },
    { 0x98a006b0, 61 },
    { 0x42a006b1, 34 },
    { 0x5c9006b0, 23 },
    { 0xc0d006b3, 61 },
    { 0x854006b1, 95 },
    { 0x443006b1, 76 },
    { 0, 0 },
    { 0x265006b7, 33 },
    { 0x3b0006b7, 46 },
    { 0x42d006b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650006c3, 100 },
    { 0x388006c3, 61 },
    { 0x749006c5, 43 },
    { 0, 0 },
    { 0x7c9006c7, 93 },
    { 0x6bb006c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb006d0, 67 },
    { 0, 0 },
    { 0x2f0006d2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1006d5, 90 },
    { 0x66a006d6, 100 },
    { 0, 0 },
    { 0x31b006d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911006e9, 43 },
    { 0xcde006ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd006ee, 90 },
    { 0x489006ef, 61 },
    { 0x5d006ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253006f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe006fa, 95 },
    { 0x8c4006fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4006fe, 90 },
    { 0x83d006ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b00703, 61 },
    { 0, 0 },
    { 0x3e200705, 52 },
    { 0xa6d00706, 33 },
    { 0x6f300707, 75 },
    { 0x4a100708, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b0070c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0700717, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde60071b, 78 },
    { 0xbde0071c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e80071f, 99 },
    { 0, 0 },
    { 0x43400721, 100 },
    { 0xcd600722, 23 },
    { 0, 0 },
    { 0x3c00724, 33 },
    { 0x45a00725, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0072a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870072e, 90 },
    { 0, 0 },
    { 0x17100730, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1500733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0073a, 33 },
    { 0x9070073b, 90 },
    { 0x34e0073b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e100741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7600745, 17 },
    { 0, 0 },
    { 0xbeb00747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a0074e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f00759, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c0075c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b200762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a600765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10076c, 90 },
    { 0xb650076c, 77 },
    { 0x6e30076e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc00772, 93 },
    { 0xada00773, 95 },
    { 0x1e700774, 23 },
    { 0x14000775, 76 },
    { 0xeb300774, 90 },
    { 0, 0 },
    { 0xd8700778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0077c, 90 },
    { 0, 0 },
    { 0x4910077e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65900782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc800785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60800788, 78 },
    { 0x49a00789, 91 },
    { 0, 0 },
    { 0x9780078b, 99 },
    { 0, 0 },
    { 0x7dc0078d, 46 },
    { 0x6ec0078d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b400794, 76 },
    { 0x26e00795, 23 },
    { 0xa300795, 90 },
    { 0, 0 },
    { 0xfef00798, 90 },
    { 0x6bc00798, 66 },
    { 0x80e0079a, 69 },
    { 0, 0 },
    { 0x1e60079c, 33 },
    { 0xe790079d, 85 },
    { 0x1f70079e, 90 },
    { 0x8b60079d, 96 },
    { 0x6140079f, 55 },
    { 0x6140079e, 33 },
    { 0x7c20079e, 66 },
    { 0x53e007a3, 98 },
    { 0xa04007a3, 75 },
    { 0x195007a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41007ac, 83 },
    { 0x786007ad, 33 },
    { 0xd72007ad, 77 },
    { 0xe3a007af, 90 },
    { 0x28c007b0, 90 },
    { 0x635007b1, 23 },
    { 0x679007b2, 100 },
    { 0, 0 },
    { 0x4ef007b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x898007b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57007bf, 43 },
    { 0, 0 },
    { 0xcb5007c1, 69 },
    { 0, 0 },
    { 0x77a007c3, 46 },
    { 0x2b9007c4, 66 },
    { 0, 0 },
    { 0x226007c6, 23 },
    { 0xf43007c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f007cf, 66 },
    { 0x20e007cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae007d4, 87 },
    { 0, 0 },
    { 0x383007d6, 98 },
    { 0, 0 },
    { 0x23a007d8, 61 },
    { 0x9f2007d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac007e2, 98 },
    { 0xf53007e2, 90 },
    { 0x5b5007e2, 84 },
    { 0xe73007e5, 78 },
    { 0x153007e6, 98 },
    { 0x172007e7, 46 },
    { 0xcb6007e8, 98 },
    { 0x500007e9, 90 },
    { 0xde5007e2, 75 },
    { 0, 0 },
    { 0x807007ec, 46 },
    { 0x572007ed, 76 },
    { 0xece007ed, 96 },
    { 0xc24007ef, 98 },
    { 0x370007f0, 95 },
    { 0xeaa007ed, 90 },
    { 0, 0 },
    { 0xafc007f3, 55 },
    { 0xd9b007f4, 98 },
    { 0x55f007f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1007fa, 76 },
    { 0x328007fa, 43 },
    { 0, 0 },
    { 0x3b7007fd, 95 },
    { 0xe3c007fe, 23 },
    { 0xf38007fe, 33 },
    { 0x8fa00800, 90 },
    { 0, 0 },
    { 0xe7300802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d00807, 90 },
    { 0, 0 },
    { 0xfb600809, 78 },
    { 0, 0 },
    { 0x68f0080b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680080e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee00814, 90 },
    { 0x3ff00815, 55 },
    { 0x56600816, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700081b, 33 },
    { 0x7f90081b, 61 },
    { 0, 0 },
    { 0x3230081e, 90 },
    { 0x6900081f, 43 },
    { 0x9740081e, 98 },
    { 0x86600821, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e00827, 61 },
    { 0xc9c00828, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970082c, 90 },
    { 0xeac0082d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600830, 98 },
    { 0xd6b00830, 33 },
    { 0x2400831, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10700836, 55 },
    { 0xa4400837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30083a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca0083d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82900842, 61 },
    { 0x34600842, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c0084e, 95 },
    { 0xdfa0084f, 97 },
    { 0xe600850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2900853, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf00856, 81 },
    { 0, 0 },
    { 0xa5c00858, 90 },
    { 0xfbe00859, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c0085e, 77 },
    { 0xa430085f, 98 },
    { 0x8f10085f, 33 },
    { 0x66200861, 61 },
    { 0x16f00862, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb340086d, 52 },
    { 0x7ad0086d, 33 },
    { 0x9280086d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64200874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc00877, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc30087b, 33 },
    { 0, 0 },
    { 0xfa0087d, 66 },
    { 0x3a20087d, 33 },
    { 0, 0 },
    { 0x7b00880, 98 },
    { 0xd00880, 17 },
    { 0x40b00880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d00886, 84 },
    { 0xa500886, 78 },
    { 0x29100887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb000088c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d0088f, 43 },
    { 0, 0 },
    { 0x8d400891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d00896, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf70089d, 90 },
    { 0xa1b0089e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3008a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552008a5, 90 },
    { 0xfc6008a5, 34 },
    { 0xcb6008a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14008ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5008b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807008b7, 66 },
    { 0, 0 },
    { 0xb7d008b9, 84 },
    { 0x3ae008ba, 61 },
    { 0xc82008bb, 90 },
    { 0x495008bc, 46 },
    { 0xb2c008bc, 34 },
    { 0, 0 },
    { 0x3ed008bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1008c2, 61 },
    { 0, 0 },
    { 0x83008c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62008ca, 90 },
    { 0x593008cb, 20 },
    { 0, 0 },
    { 0x325008cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e008d0, 91 },
    { 0xffb008d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e008d6, 34 },
    { 0, 0 },
    { 0x39e008d8, 34 },
    { 0x286008d9, 90 },
    { 0xbbb008d9, 78 },
    { 0x5fa008db, 43 },
    { 0xf52008dc, 40 },
    { 0, 0 },
    { 0xe4c008de, 46 },
    { 0, 0 },
    { 0x11b008e0, 98 },
    { 0xae5008e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3008f2, 56 },
    { 0x783008f2, 52 },
    { 0x5c2008f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e008f7, 33 },
    { 0xabe008f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a008fb, 61 },
    { 0x1df008fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db00900, 68 },
    { 0xe3300900, 90 },
    { 0, 0 },
    { 0x99c00903, 46 },
    { 0xd5600904, 95 },
    { 0xa6f00905, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c00908, 98 },
    { 0xf7400909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0090f, 23 },
    { 0x19c00910, 99 },
    { 0x93200911, 90 },
    { 0x42600912, 23 },
    { 0x87800913, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb800917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6460091b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100091e, 23 },
    { 0x8690091f, 61 },
    { 0x290091f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f00927, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca20092c, 90 },
    { 0x30b0092d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd00931, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e00934, 61 },
    { 0x46800935, 68 },
    { 0, 0 },
    { 0x26200937, 33 },
    { 0, 0 },
    { 0x3da00939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe00943, 95 },
    { 0, 0 },
    { 0x34200945, 40 },
    { 0x89b00946, 33 },
    { 0x92400947, 84 },
    { 0, 0 },
    { 0x3db00949, 23 },
    { 0, 0 },
    { 0x5b10094b, 90 },
    { 0xe940094c, 76 },
    { 0x7f90094d, 61 },
    { 0x3ee0094e, 90 },
    { 0x6b50094c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db00955, 90 },
    { 0, 0 },
    { 0xeb100957, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea10095d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e900961, 33 },
    { 0x2ad00962, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa600967, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0096c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5500971, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d00975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1200978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d0097e, 87 },
    { 0x4fd0097f, 90 },
    { 0x19b00980, 90 },
    { 0x98c0097e, 55 },
    { 0xb5600982, 23 },
    { 0x3a500983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f00986, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c00989, 46 },
    { 0xe010098a, 90 },
    { 0xc360098a, 33 },
    { 0x48a0098c, 90 },
    { 0x2610098c, 78 },
    { 0x8cd0098c, 90 },
    { 0x20b0098f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c600998, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6009a0, 34 },
    { 0xe79009a0, 61 },
    { 0x3b8009a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc009aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4009ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3009b3, 33 },
    { 0xd65009b3, 93 },
    { 0xb4d009b5, 66 },
    { 0, 0 },
    { 0x227009b7, 91 },
    { 0xac6009b8, 23 },
    { 0x5b0009b9, 90 },
    { 0x5ac009ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799009be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175009c6, 78 },
    { 0x46f009c7, 66 },
    { 0x206009c8, 46 },
    { 0, 0 },
    { 0x345009ca, 90 },
    { 0xeb009cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd009d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce009d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c009d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8009db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab009e1, 23 },
    { 0x90f009e1, 23 },
    { 0x44a009e3, 43 },
    { 0, 0 },
    { 0x49a009e5, 85 },
    { 0x5da009e6, 90 },
    { 0x485009e7, 43 },
    { 0, 0 },
    { 0xe9d009e9, 84 },
    { 0, 0 },
    { 0xf4d009eb, 77 },
    { 0x956009ec, 33 },
    { 0x8ad009ed, 78 },
    { 0x155009ec, 80 },
    { 0xb5009ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1009f9, 90 },
    { 0x9a5009fa, 61 },
    { 0xb7d009fb, 90 },
    { 0x9c4009fc, 97 },
    { 0, 0 },
    { 0x1fd009fe, 29 },
    { 0, 0 },
    { 0x5b900a00, 61 },
    { 0, 0 },
    { 0x67d00a02, 61 },
    { 0xfd900a02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39000a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a00a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a00a10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9700a16, 19 },
    { 0xb4900a16, 61 },
    { 0, 0 },
    { 0x5ee00a19, 47 },
    { 0xdfc00a1a, 34 },
    { 0x76100a1a, 93 },
    { 0, 0 },
    { 0xb5800a1d, 76 },
    { 0xadb00a1e, 76 },
    { 0xc1600a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa00a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b00a29, 23 },
    { 0x94b00a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d400a33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd00a37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e00a3c, 23 },
    { 0x85100a3d, 80 },
    { 0xd7200a3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d00a42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e100a4c, 98 },
    { 0xc800a4c, 90 },
    { 0x5f00a4e, 98 },
    { 0x7b900a4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4200a53, 95 },
    { 0x8b600a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f00a58, 33 },
    { 0xed800a58, 33 },
    { 0x52900a5a, 96 },
    { 0, 0 },
    { 0x60600a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef400a67, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50500a6c, 84 },
    { 0, 0 },
    { 0xbe100a6e, 33 },
    { 0xb2b00a6f, 34 },
    { 0x7f600a70, 83 },
    { 0x19f00a6f, 95 },
    { 0xebc00a72, 46 },
    { 0, 0 },
    { 0x41f00a74, 90 },
    { 0, 0 },
    { 0x63a00a76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39600a79, 98 },
    { 0xd6500a7a, 33 },
    { 0, 0 },
    { 0x49300a7c, 93 },
    { 0xd1f00a7c, 34 },
    { 0xf1900a7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb200a85, 90 },
    { 0xa8800a85, 61 },
    { 0, 0 },
    { 0x66000a88, 38 },
    { 0, 0 },
    { 0xd4100a8a, 98 },
    { 0xb1a00a8b, 100 },
    { 0, 0 },
    { 0xcdb00a8d, 97 },
    { 0xe1c00a8e, 87 },
    { 0x7cb00a8f, 33 },
    { 0x5ca00a8f, 61 },
    { 0x4a900a91, 33 },
    { 0, 0 },
    { 0xd6c00a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f00a99, 88 },
    { 0x3e900a99, 33 },
    { 0x8c00a9b, 66 },
    { 0x88b00a9b, 61 },
    { 0, 0 },
    { 0xd1c00a9e, 55 },
    { 0, 0 },
    { 0x30b00aa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b100aa5, 61 },
    { 0xa0a00aa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x68900aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e00aad, 34 },
    { 0xb3900aae, 90 },
    { 0x7c800aaf, 99 },
    { 0xa800ab0, 66 },
    { 0xd0600aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe200ab4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e00ab7, 98 },
    { 0xcc100ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd00abf, 43 },
    { 0, 0 },
    { 0x62600ac1, 76 },
    { 0x7b300ac2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee00ac6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d00ac9, 77 },
    { 0x31300ac9, 34 },
    { 0x9fd00aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde00ad5, 91 },
    { 0, 0 },
    { 0x2e00ad7, 23 },
    { 0, 0 },
    { 0x9100ad9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd500adc, 93 },
    { 0x5400add, 34 },
    { 0x6a300ade, 77 },
    { 0x8e800adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca500ae2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a00ae6, 61 },
    { 0, 0 },
    { 0x48f00ae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4500af3, 77 },
    { 0, 0 },
    { 0x55b00af5, 98 },
    { 0x8b900af6, 90 },
    { 0x9b00af5, 78 },
    { 0x96700af8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b500afb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d00b04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb00b07, 33 },
    { 0, 0 },
    { 0x74d00b09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab000b0e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e700b15, 95 },
    { 0, 0 },
    { 0x97300b17, 61 },
    { 0xdea00b18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d200b1d, 23 },
    { 0x4fa00b1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b000b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe00b24, 77 },
    { 0x76a00b25, 88 },
    { 0x42200b25, 95 },
    { 0x1f400b24, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6000b2c, 98 },
    { 0xb1c00b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57100b35, 76 },
    { 0x53000b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f000b3b, 61 },
    { 0, 0 },
    { 0x83d00b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd00b40, 23 },
    { 0x7cd00b41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19000b45, 33 },
    { 0, 0 },
    { 0xfd700b47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41100b4c, 98 },
    { 0, 0 },
    { 0x58e00b4e, 34 },
    { 0xd9600b4f, 33 },
    { 0xcf900b50, 33 },
    { 0x98c00b51, 96 },
    { 0x28600b52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c00b5b, 61 },
    { 0x19200b5c, 95 },
    { 0xa600b5d, 90 },
    { 0x49400b5e, 33 },
    { 0x7ad00b5b, 90 },
    { 0, 0 },
    { 0x8e600b61, 90 },
    { 0xfea00b62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24000b66, 52 },
    { 0, 0 },
    { 0x6a400b68, 52 },
    { 0x76d00b69, 98 },
    { 0, 0 },
    { 0xb3f00b6b, 90 },
    { 0, 0 },
    { 0x71d00b6d, 90 },
    { 0, 0 },
    { 0x1f600b6f, 76 },
    { 0x3f400b70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76200b74, 61 },
    { 0, 0 },
    { 0x6d900b76, 87 },
    { 0xd2100b76, 80 },
    { 0x72700b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe00b7d, 17 },
    { 0x88300b7d, 33 },
    { 0xc3200b7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed800b84, 33 },
    { 0x6ad00b85, 61 },
    { 0, 0 },
    { 0xc000b87, 90 },
    { 0x6d900b88, 76 },
    { 0, 0 },
    { 0x6ca00b8a, 55 },
    { 0x44900b8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c00b8f, 23 },
    { 0x90d00b90, 90 },
    { 0xb2400b91, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff00b96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5800b99, 61 },
    { 0, 0 },
    { 0x26d00b9b, 61 },
    { 0x1b200b9b, 95 },
    { 0x66700b9d, 77 },
    { 0x62700b9e, 61 },
    { 0x12000b9e, 76 },
    { 0x93200b9e, 43 },
    { 0x1f200ba1, 90 },
    { 0xbf700ba2, 76 },
    { 0xc000ba1, 43 },
    { 0x8fd00ba4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb800ba7, 91 },
    { 0xa8200ba8, 34 },
    { 0xb5c00ba9, 90 },
    { 0x22000ba9, 76 },
    { 0xd5100bab, 87 },
    { 0x6600bac, 66 },
    { 0x72700bab, 95 },
    { 0, 0 },
    { 0xd2c00baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5200bb2, 55 },
    { 0, 0 },
    { 0xfb300bb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd00bb8, 23 },
    { 0x56b00bb8, 87 },
    { 0xfde00bba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf100bbd, 81 },
    { 0xc5000bbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4800bcd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66800bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a900bd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef00bdd, 98 },
    { 0x1f700bdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6300be4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0600be8, 90 },
    { 0x4f600be9, 33 },
    { 0, 0 },
    { 0x30700beb, 95 },
    { 0xf1200bec, 98 },
    { 0, 0 },
    { 0xeeb00bee, 61 },
    { 0xdf600bef, 78 },
    { 0x5bc00bf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e300bf4, 95 },
    { 0x7f00bf4, 23 },
    { 0, 0 },
    { 0xe7d00bf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd00bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47200c02, 4 },
    { 0, 0 },
    { 0x4b400c04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac000c0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b00c10, 93 },
    { 0, 0 },
    { 0x18a00c12, 52 },
    { 0x5c600c12, 55 },
    { 0xb3100c14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b000c1b, 98 },
    { 0x6b000c1b, 100 },
    { 0xa9900c1d, 88 },
    { 0x24700c1d, 90 },
    { 0x5da00c1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9400c24, 98 },
    { 0xf700c24, 68 },
    { 0x6f300c26, 98 },
    { 0x8e200c27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a00c2a, 98 },
    { 0, 0 },
    { 0x5c00c2c, 61 },
    { 0xb7b00c2d, 93 },
    { 0xc5f00c2e, 61 },
    { 0x4bc00c2e, 17 },
    { 0xea400c2f, 23 },
    { 0xa2200c31, 61 },
    { 0x53f00c32, 34 },
    { 0xfba00c33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c00c3a, 78 },
    { 0xe5100c3a, 43 },
    { 0, 0 },
    { 0xc8700c3d, 87 },
    { 0x52200c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82300c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc400c45, 91 },
    { 0, 0 },
    { 0x82000c47, 95 },
    { 0x4800c48, 95 },
    { 0, 0 },
    { 0xe6900c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff00c50, 33 },
    { 0xd0f00c51, 61 },
    { 0x39e00c50, 52 },
    { 0, 0 },
    { 0xde600c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d300c5a, 93 },
    { 0, 0 },
    { 0x7aa00c5c, 61 },
    { 0x85400c5d, 43 },
    { 0x27000c5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc400c61, 98 },
    { 0xcfa00c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a00c66, 23 },
    { 0x19700c66, 93 },
    { 0, 0 },
    { 0xeff00c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99700c6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c800c6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f00c74, 98 },
    { 0x5c300c75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26700c79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2900c7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38400c84, 76 },
    { 0, 0 },
    { 0xcfb00c86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af00c8a, 90 },
    { 0xf0f00c8b, 67 },
    { 0xd500c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea00c92, 67 },
    { 0, 0 },
    { 0xb8600c94, 95 },
    { 0x55d00c95, 33 },
    { 0xebb00c94, 19 },
    { 0xc6e00c97, 78 },
    { 0x8000c98, 96 },
    { 0x92800c99, 98 },
    { 0xe7f00c9a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd00ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72700ca9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e100cb1, 95 },
    { 0x4c400cb1, 33 },
    { 0, 0 },
    { 0x72b00cb4, 96 },
    { 0x44500cb4, 46 },
    { 0xa3b00cb6, 55 },
    { 0xc4000cb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d00cbc, 98 },
    { 0x4b900cbd, 61 },
    { 0x58800cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee00cc3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d00cc8, 100 },
    { 0, 0 },
    { 0xfd500cca, 17 },
    { 0x1de00cca, 98 },
    { 0, 0 },
    { 0x4700ccd, 98 },
    { 0, 0 },
    { 0x32700ccf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d900cd2, 95 },
    { 0x96b00cd3, 23 },
    { 0xef700cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c900cda, 90 },
    { 0, 0 },
    { 0xe8c00cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec00ce5, 34 },
    { 0x3000ce6, 92 },
    { 0xb0c00ce7, 98 },
    { 0x52300ce8, 95 },
    { 0xa0c00ce9, 61 },
    { 0xa6300ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b00ced, 23 },
    { 0, 0 },
    { 0x9e100cef, 76 },
    { 0x4c100cef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89300cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46700cf7, 23 },
    { 0x90b00cf8, 66 },
    { 0xb6f00cf9, 95 },
    { 0xd4d00cfa, 17 },
    { 0, 0 },
    { 0x76500cfc, 43 },
    { 0, 0 },
    { 0xb3f00cfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f00d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e100d08, 61 },
    { 0xe4600d09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10300d0c, 61 },
    { 0x5000d0d, 90 },
    { 0x83700d0d, 61 },
    { 0x75000d0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41500d13, 67 },
    { 0xb9700d14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9600d1c, 67 },
    { 0x8a200d1d, 23 },
    { 0, 0 },
    { 0x7d600d1f, 76 },
    { 0, 0 },
    { 0x41e00d21, 98 },
    { 0x55b00d21, 23 },
    { 0, 0 },
    { 0x75e00d24, 46 },
    { 0, 0 },
    { 0xc3a00d26, 55 },
    { 0x8d700d27, 23 },
    { 0x5400d28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f900d30, 33 },
    { 0xb8500d30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c00d36, 34 },
    { 0xea000d37, 95 },
    { 0xa6900d38, 98 },
    { 0x4a100d38, 95 },
    { 0x34b00d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f00d3f, 33 },
    { 0, 0 },
    { 0xc0900d41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff00d46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb00d49, 87 },
    { 0x3f000d49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b00d4e, 78 },
    { 0, 0 },
    { 0x6ab00d50, 55 },
    { 0, 0 },
    { 0xc5600d52, 90 },
    { 0x9200d53, 99 },
    { 0xead00d54, 56 },
    { 0x21e00d55, 33 },
    { 0x9cd00d53, 98 },
    { 0x10400d57, 76 },
    { 0x46400d58, 61 },
    { 0x97500d58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a00d5d, 61 },
    { 0xea000d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc00d61, 97 },
    { 0x28500d62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b700d68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f00d6d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xac800d70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7400d73, 95 },
    { 0x1e200d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb00d7b, 61 },
    { 0x3c300d7b, 33 },
    { 0xa9900d7d, 98 },
    { 0xc7400d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87100d86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3300d8e, 76 },
    { 0, 0 },
    { 0x13600d90, 55 },
    { 0xda00d91, 90 },
    { 0x6e00d91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b600d95, 61 },
    { 0, 0 },
    { 0x99100d97, 43 },
    { 0, 0 },
    { 0x34e00d99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d400d9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a00da1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee300da9, 76 },
    { 0, 0 },
    { 0xd1b00dab, 46 },
    { 0x13c00dac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f00db0, 61 },
    { 0x3de00db1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b400dba, 98 },
    { 0x67700dbb, 46 },
    { 0x54d00dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb00dbf, 91 },
    { 0, 0 },
    { 0x7000dc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c00dc4, 90 },
    { 0x80700dc5, 98 },
    { 0, 0 },
    { 0x87500dc7, 33 },
    { 0x7d00dc8, 76 },
    { 0xa7d00dc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f00dcd, 33 },
    { 0, 0 },
    { 0x46700dcf, 23 },
    { 0x38c00dd0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b400dd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56200ddb, 90 },
    { 0xb4700ddb, 61 },
    { 0x85f00ddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7500de5, 61 },
    { 0, 0 },
    { 0x85b00de7, 33 },
    { 0xa3600de8, 98 },
    { 0, 0 },
    { 0x11900dea, 34 },
    { 0x56400deb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae00def, 34 },
    { 0, 0 },
    { 0xef200df1, 43 },
    { 0x4d900df1, 61 },
    { 0, 0 },
    { 0x72700df4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea00df8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a00dfb, 23 },
    { 0xcb700dfc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40300dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16900e03, 90 },
    { 0x1c400e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28700e09, 95 },
    { 0x67800e09, 90 },
    { 0, 0 },
    { 0xe1e00e0c, 55 },
    { 0x82500e0c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a100e13, 61 },
    { 0, 0 },
    { 0x7af00e15, 88 },
    { 0x7a200e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f100e1b, 61 },
    { 0x9ae00e1c, 98 },
    { 0x23c00e1d, 33 },
    { 0, 0 },
    { 0x58600e1f, 90 },
    { 0x85a00e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0700e23, 66 },
    { 0, 0 },
    { 0x48f00e25, 23 },
    { 0, 0 },
    { 0xfe400e27, 93 },
    { 0xc7300e28, 52 },
    { 0xb8100e29, 33 },
    { 0, 0 },
    { 0x41900e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e00e2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x45000e31, 55 },
    { 0x95c00e32, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b00e35, 46 },
    { 0, 0 },
    { 0xeae00e37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc900e3a, 55 },
    { 0xa9b00e3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24400e41, 23 },
    { 0x1e700e42, 97 },
    { 0x99100e43, 98 },
    { 0xc8000e44, 90 },
    { 0, 0 },
    { 0x76c00e46, 99 },
    { 0x1a000e47, 23 },
    { 0xadd00e48, 17 },
    { 0x92000e46, 52 },
    { 0x26a00e49, 33 },
    { 0x99b00e4a, 23 },
    { 0, 0 },
    { 0x54b00e4d, 33 },
    { 0, 0 },
    { 0x2a000e4f, 95 },
    { 0xabc00e4f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d00e5c, 33 },
    { 0x18000e5d, 46 },
    { 0x8b300e5e, 65 },
    { 0x10f00e5f, 34 },
    { 0xb0600e5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd00e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32700e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a00e6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50300e79, 33 },
    { 0x76200e7a, 65 },
    { 0, 0 },
    { 0x1bc00e7c, 45 },
    { 0xd1500e7c, 33 },
    { 0, 0 },
    { 0x43c00e7f, 65 },
    { 0x84a00e80, 61 },
    { 0xd2d00e81, 69 },
    { 0x7e00e7f, 98 },
    { 0xb3c00e83, 40 },
    { 0xbde00e84, 76 },
    { 0x97300e84, 34 },
    { 0x57600e86, 33 },
    { 0x99700e86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16500e8e, 43 },
    { 0, 0 },
    { 0x8cf00e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde500e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb00e99, 43 },
    { 0, 0 },
    { 0xd6300e9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e00ea0, 23 },
    { 0, 0 },
    { 0x46300ea2, 98 },
    { 0xb2c00ea3, 90 },
    { 0x900ea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d200ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3000eab, 34 },
    { 0xbc400eac, 95 },
    { 0x78a00ead, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0000eb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64600eb9, 34 },
    { 0, 0 },
    { 0xd9900ebb, 33 },
    { 0xe3200ebc, 68 },
    { 0, 0 },
    { 0xf5100ebe, 95 },
    { 0x7b000ebf, 95 },
    { 0xb7a00ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47800ec6, 90 },
    { 0, 0 },
    { 0x7fb00ec8, 61 },
    { 0, 0 },
    { 0xd3e00eca, 52 },
    { 0, 0 },
    { 0x14700ecc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1000ed7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc500eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b200edf, 33 },
    { 0x15100edf, 16 },
    { 0, 0 },
    { 0xcd700ee2, 36 },
    { 0, 0 },
    { 0x17500ee4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4500ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f00eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d00ef0, 61 },
    { 0xd6c00ef1, 91 },
    { 0, 0 },
    { 0xf1200ef3, 34 },
    { 0xcdf00ef3, 99 },
    { 0x7f700ef5, 61 },
    { 0xbc700ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68700efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd800f01, 95 },
    { 0x72000f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00f07, 23 },
    { 0xbd400f08, 78 },
    { 0x46200f09, 43 },
    { 0x38c00f09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21000f0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8300f11, 33 },
    { 0, 0 },
    { 0x2f000f13, 34 },
    { 0x9ab00f14, 19 },
    { 0, 0 },
    { 0x29000f16, 90 },
    { 0, 0 },
    { 0x17e00f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5300f1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe00f23, 90 },
    { 0xd6000f24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af00f29, 90 },
    { 0, 0 },
    { 0xbd600f2b, 95 },
    { 0, 0 },
    { 0xa0800f2d, 23 },
    { 0, 0 },
    { 0xc4500f2f, 76 },
    { 0xc9200f2f, 98 },
    { 0, 0 },
    { 0xf8200f32, 34 },
    { 0xcfb00f33, 98 },
    { 0x85300f33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa500f3d, 52 },
    { 0xad000f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14300f42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31800f46, 90 },
    { 0x34300f47, 33 },
    { 0x8e000f48, 23 },
    { 0, 0 },
    { 0xe000f4a, 91 },
    { 0x28300f4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e00f53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef00f56, 33 },
    { 0x66000f57, 90 },
    { 0x79a00f56, 91 },
    { 0xdb00f59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa600f5f, 98 },
    { 0, 0 },
    { 0xcf500f61, 46 },
    { 0xf7d00f62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400f66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d00f69, 33 },
    { 0x17100f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a00f6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a00f71, 76 },
    { 0x55700f71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10400f7a, 34 },
    { 0xdae00f7a, 98 },
    { 0x1ee00f7c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a00f80, 97 },
    { 0x6c900f80, 33 },
    { 0xabe00f82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56700f89, 98 },
    { 0x79a00f8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6000f91, 98 },
    { 0, 0 },
    { 0xf5100f93, 33 },
    { 0, 0 },
    { 0x59a00f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88500f98, 65 },
    { 0xa6500f99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e00fa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef00fa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4300fa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7600fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde00fbd, 55 },
    { 0xe2300fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0700fc5, 61 },
    { 0xa2900fc6, 85 },
    { 0xf4500fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca000fca, 87 },
    { 0x15600fcb, 66 },
    { 0x7c000fca, 33 },
    { 0x77400fcd, 87 },
    { 0, 0 },
    { 0x5ea00fcf, 84 },
    { 0xf8c00fcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5600fd5, 33 },
    { 0x3b300fd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e00fde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac400fe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b00fea, 95 },
    { 0, 0 },
    { 0x78900fec, 34 },
    { 0x1a00fec, 33 },
    { 0, 0 },
    { 0x74400fef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7800ff2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd00ff6, 90 },
    { 0xea900ff6, 100 },
    { 0xbd700ff8, 81 },
    { 0xbb600ff9, 95 },
    { 0x61500ff9, 84 },
    { 0, 0 },
    { 0xcd500ffc, 33 },
    { 0x22700ffc, 76 },
    { 0x5b800ffe, 40 },
    { 0xb9300ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba01003, 61 },
    { 0, 0 },
    { 0xc9f01005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3340100a, 33 },
    { 0xf3d0100b, 90 },
    { 0x26a0100b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570100f, 33 },
    { 0x37201010, 33 },
    { 0xdaf01011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a01014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0101b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2610101f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2201025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc01028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80102d, 76 },
    { 0xc910102d, 88 },
    { 0, 0 },
    { 0xe3401030, 78 },
    { 0, 0 },
    { 0x5da01032, 95 },
    { 0, 0 },
    { 0x86001034, 33 },
    { 0xef101035, 77 },
    { 0xcb201036, 89 },
    { 0xe8801037, 99 },
    { 0x34601037, 91 },
    { 0x9eb01036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b0103c, 23 },
    { 0, 0 },
    { 0x5a40103e, 61 },
    { 0, 0 },
    { 0xee901040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef301043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d01049, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4520104d, 33 },
    { 0x4390104d, 76 },
    { 0x52f0104f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4701058, 95 },
    { 0xb2601059, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8510105e, 95 },
    { 0, 0 },
    { 0x24301060, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa501064, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa101067, 85 },
    { 0x4a501068, 33 },
    { 0x31d01069, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010106c, 61 },
    { 0x2200106d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1401070, 53 },
    { 0xa9001071, 78 },
    { 0x6501070, 95 },
    { 0x23b01071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4101078, 46 },
    { 0x20c01079, 52 },
    { 0x98301079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0107e, 80 },
    { 0xc050107f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1370108e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be01091, 90 },
    { 0, 0 },
    { 0x62c01093, 98 },
    { 0x71f01093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe01098, 87 },
    { 0xef501098, 85 },
    { 0, 0 },
    { 0xf230109b, 85 },
    { 0x73f0109c, 23 },
    { 0x9c30109d, 66 },
    { 0, 0 },
    { 0xdc90109f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27010a4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a010a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71010ab, 34 },
    { 0x128010ac, 61 },
    { 0xc19010ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6010b0, 38 },
    { 0x24d010b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66010b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78010b8, 84 },
    { 0, 0 },
    { 0x869010ba, 61 },
    { 0x1da010bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0010be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8010c5, 61 },
    { 0x4de010c6, 51 },
    { 0x245010c7, 90 },
    { 0xfec010c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957010cc, 33 },
    { 0xb97010cd, 61 },
    { 0xcb4010ce, 61 },
    { 0x233010ce, 61 },
    { 0, 0 },
    { 0x215010d1, 90 },
    { 0x49f010d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x417010d5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68010da, 98 },
    { 0x82d010da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150010e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409010e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f010ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6010f0, 53 },
    { 0, 0 },
    { 0xb6c010f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467010f8, 91 },
    { 0xf33010f8, 66 },
    { 0, 0 },
    { 0xe61010fb, 91 },
    { 0x36e010fc, 33 },
    { 0x3cd010fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa301100, 95 },
    { 0x67e01101, 56 },
    { 0xba201101, 90 },
    { 0xb8701103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c801106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77101109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac50110c, 87 },
    { 0x81a0110c, 90 },
    { 0x3920110e, 85 },
    { 0xdcb0110f, 98 },
    { 0, 0 },
    { 0x58c01111, 99 },
    { 0x48c01112, 65 },
    { 0x8fc01113, 66 },
    { 0x66901113, 33 },
    { 0xdd501115, 90 },
    { 0xcfe01116, 47 },
    { 0x55101117, 87 },
    { 0, 0 },
    { 0x4f601119, 78 },
    { 0x33d0111a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13201120, 87 },
    { 0xd3401121, 95 },
    { 0x64e01121, 55 },
    { 0xf8701123, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b01129, 90 },
    { 0x28001129, 93 },
    { 0x3300112b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae0112f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1101132, 93 },
    { 0x43d01133, 100 },
    { 0xc7101134, 52 },
    { 0x55a01133, 34 },
    { 0x92501134, 55 },
    { 0, 0 },
    { 0x3bc01138, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda00113c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b01141, 90 },
    { 0xe2501142, 23 },
    { 0x25401143, 90 },
    { 0xf3d01143, 61 },
    { 0, 0 },
    { 0x7c001146, 56 },
    { 0x82101147, 34 },
    { 0xf3301146, 33 },
    { 0x80201149, 76 },
    { 0xc0a0114a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc580114e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b01151, 34 },
    { 0xafd01152, 100 },
    { 0xc7401152, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d01157, 33 },
    { 0xe4f01157, 61 },
    { 0x31201157, 90 },
    { 0, 0 },
    { 0x5880115b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d01161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24701167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50116b, 23 },
    { 0x66f0116c, 85 },
    { 0x9a10116b, 46 },
    { 0xf900116b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9701174, 76 },
    { 0x6c301174, 77 },
    { 0x3e101175, 55 },
    { 0x20f01175, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d0117b, 33 },
    { 0, 0 },
    { 0x4410117d, 65 },
    { 0x45c0117e, 85 },
    { 0, 0 },
    { 0x7a501180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb01188, 76 },
    { 0x15301189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9800118d, 68 },
    { 0x8e00118e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a01197, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc210119c, 90 },
    { 0x3320119c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7011a0, 66 },
    { 0xf92011a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2011a7, 38 },
    { 0xe34011a7, 91 },
    { 0xa1b011a8, 69 },
    { 0xba2011aa, 52 },
    { 0xad1011ab, 88 },
    { 0, 0 },
    { 0xe8011ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a011b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed011b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f011bb, 78 },
    { 0x9bb011bc, 99 },
    { 0xd3d011bd, 90 },
    { 0x3f2011bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24011c1, 77 },
    { 0x99b011c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398011c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43011cb, 55 },
    { 0, 0 },
    { 0xfc7011cd, 78 },
    { 0, 0 },
    { 0xc58011cf, 76 },
    { 0xbfa011cf, 61 },
    { 0, 0 },
    { 0x30d011d2, 77 },
    { 0, 0 },
    { 0x7af011d4, 95 },
    { 0xc4f011d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd011d9, 99 },
    { 0x141011d9, 90 },
    { 0xb9a011d9, 76 },
    { 0xacf011dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120011e1, 61 },
    { 0, 0 },
    { 0x139011e3, 34 },
    { 0x2b8011e4, 33 },
    { 0, 0 },
    { 0xd02011e6, 40 },
    { 0x108011e7, 90 },
    { 0xd63011e7, 23 },
    { 0x416011e9, 43 },
    { 0, 0 },
    { 0xbd2011eb, 34 },
    { 0xb68011ec, 90 },
    { 0, 0 },
    { 0x73c011ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f011f2, 90 },
    { 0, 0 },
    { 0x80f011f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952011f8, 61 },
    { 0, 0 },
    { 0x8a6011fa, 98 },
    { 0x979011fb, 91 },
    { 0x878011fc, 61 },
    { 0x56011fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b01201, 61 },
    { 0xb2e01202, 34 },
    { 0x8f301203, 90 },
    { 0x64001202, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701207, 68 },
    { 0x3d001207, 46 },
    { 0, 0 },
    { 0x5a40120a, 23 },
    { 0x4fa0120b, 90 },
    { 0x15d0120c, 34 },
    { 0x9110120d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd501210, 98 },
    { 0, 0 },
    { 0x25701212, 76 },
    { 0x57701213, 23 },
    { 0xb1e01214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db01217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0121c, 43 },
    { 0x5b70121d, 68 },
    { 0x9cc0121d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee01221, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70122b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f01232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f201235, 84 },
    { 0x73a01235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc01239, 90 },
    { 0xeee01239, 95 },
    { 0xeb001239, 65 },
    { 0, 0 },
    { 0x5960123d, 98 },
    { 0, 0 },
    { 0xae80123f, 76 },
    { 0, 0 },
    { 0x99301241, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c01244, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0101247, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350124c, 98 },
    { 0, 0 },
    { 0x6d30124e, 17 },
    { 0, 0 },
    { 0x5401250, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47301253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9610125e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6101263, 90 },
    { 0xa0201264, 90 },
    { 0xbcb01263, 78 },
    { 0x35601266, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b501269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e80126d, 34 },
    { 0x9aa0126e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb01277, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740127f, 43 },
    { 0x71701280, 23 },
    { 0xfbe01281, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e01285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde90128a, 66 },
    { 0x6ed0128b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa290128e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b701291, 78 },
    { 0, 0 },
    { 0xf9501293, 46 },
    { 0x23401294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b0129b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4012a1, 17 },
    { 0xa07012a2, 98 },
    { 0xda4012a1, 98 },
    { 0, 0 },
    { 0xd03012a5, 98 },
    { 0xec8012a5, 90 },
    { 0xcfd012a5, 91 },
    { 0, 0 },
    { 0x5e012a9, 33 },
    { 0, 0 },
    { 0xf89012ab, 52 },
    { 0xae2012ac, 93 },
    { 0x68d012ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9012b0, 52 },
    { 0xa7b012b0, 46 },
    { 0, 0 },
    { 0xc12012b3, 43 },
    { 0x355012b4, 90 },
    { 0x571012b5, 67 },
    { 0x94d012b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c012c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef012c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334012c8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x140012cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e012cf, 61 },
    { 0x7ad012d0, 66 },
    { 0x686012cf, 76 },
    { 0xf96012d2, 68 },
    { 0xee8012d3, 98 },
    { 0x42a012d2, 100 },
    { 0x390012d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb012db, 90 },
    { 0x239012dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663012e2, 67 },
    { 0x9bd012e3, 67 },
    { 0x234012e4, 95 },
    { 0xc93012e5, 33 },
    { 0x61b012e6, 76 },
    { 0xe5a012e6, 68 },
    { 0x136012e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c012ee, 34 },
    { 0xd47012ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03012f5, 84 },
    { 0xd1c012f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af012fb, 34 },
    { 0, 0 },
    { 0xbed012fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26401301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff01305, 83 },
    { 0xd9c01305, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43801310, 33 },
    { 0x6b501310, 90 },
    { 0xcae01312, 40 },
    { 0xa0001310, 95 },
    { 0x51a01312, 23 },
    { 0xd4701312, 90 },
    { 0x5ea01311, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10131b, 55 },
    { 0xf070131b, 43 },
    { 0x8ba0131d, 33 },
    { 0xbee0131e, 61 },
    { 0x5dd0131c, 61 },
    { 0, 0 },
    { 0x82801321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb901324, 95 },
    { 0, 0 },
    { 0x87301326, 36 },
    { 0xaaa01327, 61 },
    { 0, 0 },
    { 0x88501329, 33 },
    { 0xd3e0132a, 90 },
    { 0xcc50132b, 33 },
    { 0x36e0132c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef30132f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e01333, 23 },
    { 0x8bd01334, 95 },
    { 0, 0 },
    { 0x5af01336, 55 },
    { 0x4b301337, 23 },
    { 0, 0 },
    { 0xbe001339, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0133d, 85 },
    { 0xcf10133d, 55 },
    { 0, 0 },
    { 0x6301340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e101343, 90 },
    { 0, 0 },
    { 0x11801345, 90 },
    { 0, 0 },
    { 0x1a401347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0134b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa90134f, 33 },
    { 0xdfc01350, 90 },
    { 0xc0f01351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98101354, 61 },
    { 0x2af01355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db01359, 84 },
    { 0, 0 },
    { 0xac40135b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8801366, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6080136a, 55 },
    { 0xdc90136b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840136e, 61 },
    { 0xe080136f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b101375, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e401379, 55 },
    { 0x1ec0137a, 23 },
    { 0, 0 },
    { 0xcf20137c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b80137f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5901384, 66 },
    { 0x71301385, 91 },
    { 0, 0 },
    { 0xbab01387, 98 },
    { 0x1a501387, 24 },
    { 0, 0 },
    { 0xf290138a, 81 },
    { 0x1470138b, 95 },
    { 0xc00138a, 90 },
    { 0xd260138d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e01394, 34 },
    { 0x2ec01394, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf501398, 87 },
    { 0xf2601399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b0139e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946013a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff013a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24013aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4013ae, 8 },
    { 0xf34013af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d013b4, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0013b7, 61 },
    { 0, 0 },
    { 0x5d3013b9, 33 },
    { 0x12a013ba, 55 },
    { 0xbdc013bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4013be, 95 },
    { 0x9cc013be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4013c2, 61 },
    { 0x614013c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3013ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e013cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99013d4, 61 },
    { 0x49b013d5, 88 },
    { 0xea7013d5, 55 },
    { 0xe1013d7, 65 },
    { 0, 0 },
    { 0x250013d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe013dd, 23 },
    { 0xc84013dd, 33 },
    { 0x71a013de, 43 },
    { 0xdce013e0, 98 },
    { 0x210013e0, 33 },
    { 0x5c6013e1, 33 },
    { 0, 0 },
    { 0xad6013e4, 98 },
    { 0xb79013e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5013e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440013ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757013f0, 90 },
    { 0x1c0013f1, 87 },
    { 0, 0 },
    { 0xd72013f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5013fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26701404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd001408, 36 },
    { 0xe1f01408, 95 },
    { 0x90f0140a, 23 },
    { 0, 0 },
    { 0x1aa0140c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8210140f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d01412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5901415, 46 },
    { 0xda01416, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a00141f, 90 },
    { 0x7a70141f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da01426, 95 },
    { 0xa3901426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40142c, 61 },
    { 0x61a0142c, 87 },
    { 0xd90142e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42301433, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e01438, 33 },
    { 0, 0 },
    { 0x4c60143a, 61 },
    { 0x5e80143a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb80143e, 66 },
    { 0xbda0143f, 76 },
    { 0xfb901440, 90 },
    { 0xb340143e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8901445, 90 },
    { 0x22b01446, 76 },
    { 0x59701445, 88 },
    { 0xab001448, 91 },
    { 0xa4101449, 33 },
    { 0xae301447, 95 },
    { 0x97a01446, 90 },
    { 0xdfe0144c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a701452, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d01458, 75 },
    { 0xebe01459, 95 },
    { 0, 0 },
    { 0xe6d0145b, 90 },
    { 0x9260145c, 61 },
    { 0x7bd0145c, 95 },
    { 0xe350145c, 90 },
    { 0xcbe0145c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68701463, 93 },
    { 0, 0 },
    { 0xce501465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b01469, 34 },
    { 0xba01469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d90146f, 34 },
    { 0xf7f01470, 61 },
    { 0xbc401471, 33 },
    { 0x47501470, 95 },
    { 0x5ea01471, 43 },
    { 0x33101472, 43 },
    { 0x9a101470, 33 },
    { 0x77701472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd90147b, 90 },
    { 0, 0 },
    { 0xebf0147d, 95 },
    { 0, 0 },
    { 0x6a0147f, 98 },
    { 0xce01480, 43 },
    { 0, 0 },
    { 0x2d901482, 59 },
    { 0x4e101483, 98 },
    { 0x3e01484, 88 },
    { 0x1c701483, 46 },
    { 0x87201486, 91 },
    { 0xd3401487, 95 },
    { 0x9c001488, 99 },
    { 0xaec01489, 93 },
    { 0x8b01488, 76 },
    { 0x9a70148b, 33 },
    { 0xdfc0148c, 43 },
    { 0x2b50148d, 61 },
    { 0xaa20148d, 55 },
    { 0x7d201487, 90 },
    { 0xf9c01490, 61 },
    { 0x8310148a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50301497, 34 },
    { 0, 0 },
    { 0x99d01499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d0149e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938014a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb014aa, 43 },
    { 0, 0 },
    { 0x33b014ac, 87 },
    { 0x2f6014ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98014b3, 98 },
    { 0, 0 },
    { 0x738014b5, 98 },
    { 0x2bc014b5, 91 },
    { 0, 0 },
    { 0x615014b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5014bd, 90 },
    { 0x883014be, 52 },
    { 0, 0 },
    { 0x133014c0, 61 },
    { 0x407014c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b014c6, 33 },
    { 0, 0 },
    { 0x1ef014c8, 23 },
    { 0x100014c9, 80 },
    { 0x93d014ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40014ce, 90 },
    { 0, 0 },
    { 0x5c1014d0, 55 },
    { 0x649014d1, 52 },
    { 0x332014d2, 67 },
    { 0xffa014d0, 33 },
    { 0x540014d2, 85 },
    { 0, 0 },
    { 0xdbb014d6, 91 },
    { 0xe2f014d6, 93 },
    { 0x855014d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234014df, 34 },
    { 0xea2014df, 61 },
    { 0x333014df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b014e4, 23 },
    { 0, 0 },
    { 0x438014e6, 33 },
    { 0xc92014e6, 98 },
    { 0x680014e8, 95 },
    { 0x8d7014e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22014ec, 90 },
    { 0, 0 },
    { 0xea2014ee, 78 },
    { 0xfe7014ef, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89014f3, 90 },
    { 0x204014f4, 68 },
    { 0xa5f014f5, 55 },
    { 0xebb014f6, 34 },
    { 0, 0 },
    { 0xd90014f8, 90 },
    { 0x4a0014f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da014fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3001501, 77 },
    { 0, 0 },
    { 0x35c01503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0150e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc01514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3801517, 61 },
    { 0, 0 },
    { 0x26601519, 98 },
    { 0, 0 },
    { 0xf060151b, 80 },
    { 0, 0 },
    { 0x71c0151d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43801522, 23 },
    { 0x58701523, 34 },
    { 0xd1901524, 34 },
    { 0x92401524, 90 },
    { 0x59101526, 98 },
    { 0xbd001524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf00152b, 90 },
    { 0, 0 },
    { 0x2400152d, 91 },
    { 0xa9c0152e, 77 },
    { 0x1720152e, 83 },
    { 0x3a0152d, 98 },
    { 0, 0 },
    { 0x1ff01532, 88 },
    { 0x18201533, 33 },
    { 0, 0 },
    { 0x4c001535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x56601538, 33 },
    { 0, 0 },
    { 0x81d0153a, 61 },
    { 0, 0 },
    { 0x3140153c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2701540, 95 },
    { 0, 0 },
    { 0xe6201542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d01548, 90 },
    { 0x21d01549, 100 },
    { 0x2d001549, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed90154e, 98 },
    { 0x8d20154e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d01552, 52 },
    { 0x26c01553, 93 },
    { 0xecf01554, 90 },
    { 0x3ce01554, 95 },
    { 0x87201556, 99 },
    { 0xd5201557, 34 },
    { 0xbbf01555, 33 },
    { 0xfe501555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff701560, 61 },
    { 0x2101561, 33 },
    { 0x41301562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40801566, 76 },
    { 0xb2e01567, 33 },
    { 0x19d01568, 98 },
    { 0, 0 },
    { 0x15a0156a, 85 },
    { 0, 0 },
    { 0xead0156c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3201570, 67 },
    { 0, 0 },
    { 0xbab01572, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c01576, 67 },
    { 0x1fb01577, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa680157a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b0157e, 91 },
    { 0x6c60157e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af01583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f01586, 33 },
    { 0xc2201587, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0158a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d0158e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4101597, 90 },
    { 0, 0 },
    { 0x50701599, 99 },
    { 0x8320159a, 61 },
    { 0x88d0159a, 23 },
    { 0x1dd0159b, 34 },
    { 0xc7e0159d, 98 },
    { 0x8ee0159d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b015a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f015a6, 23 },
    { 0xbdd015a6, 23 },
    { 0x5015a6, 90 },
    { 0x71c015a9, 90 },
    { 0, 0 },
    { 0x538015ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480015b1, 95 },
    { 0xb1015b2, 93 },
    { 0, 0 },
    { 0xa2b015b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565015be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x587015c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea015c4, 90 },
    { 0xa06015c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434015c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b015cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315015d2, 76 },
    { 0, 0 },
    { 0xd90015d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67015d7, 98 },
    { 0x9b0015d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829015df, 76 },
    { 0x9ed015e0, 98 },
    { 0xbb3015df, 90 },
    { 0xa24015e2, 93 },
    { 0x84d015e3, 98 },
    { 0, 0 },
    { 0x94c015e5, 90 },
    { 0x40f015e6, 93 },
    { 0xb7f015e7, 95 },
    { 0xb8c015e5, 61 },
    { 0xe91015e9, 84 },
    { 0xa5f015ea, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82015ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc015f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95015fc, 91 },
    { 0, 0 },
    { 0xa2015fe, 33 },
    { 0, 0 },
    { 0xc7701600, 43 },
    { 0, 0 },
    { 0xd6801602, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c01606, 61 },
    { 0x1e501607, 66 },
    { 0xeaa01608, 33 },
    { 0xa5e01606, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e0160f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89801613, 46 },
    { 0xe3901613, 75 },
    { 0x8bc01615, 61 },
    { 0x4c01615, 78 },
    { 0x46c01617, 61 },
    { 0x5c101616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0161b, 17 },
    { 0x4550161c, 34 },
    { 0x6860161d, 85 },
    { 0, 0 },
    { 0x8590161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72301624, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b01627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb10162b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1350162e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42201632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2001637, 33 },
    { 0, 0 },
    { 0xc6d01639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c00163c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22101640, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4601645, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9150164d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c101650, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f701658, 98 },
    { 0xfc801659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1060165c, 66 },
    { 0, 0 },
    { 0x1440165e, 33 },
    { 0xb360165e, 33 },
    { 0xf2101660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8501666, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1001669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d0166d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75801677, 93 },
    { 0, 0 },
    { 0x2e001679, 23 },
    { 0xb9c0167a, 80 },
    { 0, 0 },
    { 0x1e80167c, 65 },
    { 0x810167d, 90 },
    { 0x52a0167c, 95 },
    { 0x4d90167c, 75 },
    { 0, 0 },
    { 0xddc01681, 66 },
    { 0x17f01682, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a601685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b01690, 61 },
    { 0, 0 },
    { 0x5c01692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a01695, 90 },
    { 0x5f501696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450169e, 94 },
    { 0x7a50169f, 61 },
    { 0x807016a0, 95 },
    { 0x6ea016a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a016a7, 61 },
    { 0x9e6016a8, 92 },
    { 0xc4e016a9, 90 },
    { 0x1ee016aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a016ae, 61 },
    { 0xb15016af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a016b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b016b7, 95 },
    { 0x45d016b8, 55 },
    { 0xa00016b9, 95 },
    { 0x9bc016b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca016c1, 55 },
    { 0xf44016c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9016c5, 91 },
    { 0x6a2016c6, 55 },
    { 0xcb8016c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae016cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96016cf, 34 },
    { 0x8ae016cf, 95 },
    { 0, 0 },
    { 0x4ff016d2, 66 },
    { 0, 0 },
    { 0xff9016d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8016d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6016da, 33 },
    { 0xd11016da, 93 },
    { 0, 0 },
    { 0xe6b016dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b016e1, 61 },
    { 0, 0 },
    { 0x2d016e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95016e6, 34 },
    { 0xf1e016e7, 43 },
    { 0, 0 },
    { 0x563016e9, 23 },
    { 0x1c9016ea, 33 },
    { 0, 0 },
    { 0xfe5016ec, 95 },
    { 0, 0 },
    { 0xad4016ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8016f1, 90 },
    { 0x3ea016f1, 98 },
    { 0x7a0016f3, 23 },
    { 0, 0 },
    { 0xecf016f5, 33 },
    { 0xa45016f5, 56 },
    { 0x132016f7, 93 },
    { 0x6ea016f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82016fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e301705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f0170b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed40170f, 95 },
    { 0xd5c01710, 68 },
    { 0x61701711, 34 },
    { 0xf8d01712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec801716, 43 },
    { 0x95c01717, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e01720, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x94201723, 98 },
    { 0xca801724, 36 },
    { 0x2d701724, 95 },
    { 0x43101726, 85 },
    { 0x24601727, 100 },
    { 0x15f01728, 76 },
    { 0xd9401723, 61 },
    { 0xf5c0172a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2670172e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd01733, 33 },
    { 0xae501733, 23 },
    { 0x8a401735, 76 },
    { 0x44501735, 100 },
    { 0x81001737, 98 },
    { 0x79701734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970173d, 88 },
    { 0xbbe0173e, 61 },
    { 0xbb90173f, 43 },
    { 0xa0d0173e, 90 },
    { 0x3420173d, 90 },
    { 0x98101742, 61 },
    { 0x49901742, 23 },
    { 0x6101744, 33 },
    { 0x8bc01744, 65 },
    { 0x10901745, 87 },
    { 0x36001747, 98 },
    { 0xbbd01748, 90 },
    { 0xc6501747, 95 },
    { 0x8ee01743, 34 },
    { 0x78d01748, 64 },
    { 0xeba0173e, 90 },
    { 0, 0 },
    { 0xef00174e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c01751, 53 },
    { 0, 0 },
    { 0x34301753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180175c, 95 },
    { 0x68e0175d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86501761, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c01765, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf380176d, 55 },
    { 0x7390176e, 23 },
    { 0x83c0176f, 100 },
    { 0x5d201770, 95 },
    { 0x4d60176d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c01777, 90 },
    { 0xe6e01777, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa00177b, 43 },
    { 0x1630177c, 90 },
    { 0x6910177d, 93 },
    { 0, 0 },
    { 0xcaa0177f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce901784, 77 },
    { 0, 0 },
    { 0x60801786, 95 },
    { 0xfd901786, 90 },
    { 0, 0 },
    { 0x42b01789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10178c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb070178f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32701794, 61 },
    { 0x54201795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1801798, 77 },
    { 0xb3d01799, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d0179d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8017a3, 61 },
    { 0, 0 },
    { 0xf32017a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb017a9, 97 },
    { 0x507017aa, 84 },
    { 0x2a4017a9, 98 },
    { 0, 0 },
    { 0x6ff017ad, 95 },
    { 0xeb017ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8017b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32017ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48017bd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52017c4, 33 },
    { 0xfd017c5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f017cc, 61 },
    { 0x29e017cd, 90 },
    { 0x67017cd, 93 },
    { 0xad9017cd, 55 },
    { 0xa37017cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e017d3, 34 },
    { 0, 0 },
    { 0x91c017d5, 98 },
    { 0xa53017d5, 90 },
    { 0x298017d5, 92 },
    { 0x407017d8, 75 },
    { 0, 0 },
    { 0x36e017da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a017e1, 33 },
    { 0x5cc017e2, 84 },
    { 0x576017e3, 78 },
    { 0x13d017e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870017ea, 99 },
    { 0, 0 },
    { 0x9bb017ec, 76 },
    { 0, 0 },
    { 0xeb7017ee, 78 },
    { 0, 0 },
    { 0xfe1017f0, 90 },
    { 0x88e017f1, 90 },
    { 0x3e8017f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd017f8, 85 },
    { 0, 0 },
    { 0xeae017fa, 76 },
    { 0xfab017fb, 52 },
    { 0x975017fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7501800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b01804, 69 },
    { 0, 0 },
    { 0xf3e01806, 78 },
    { 0, 0 },
    { 0xd1301808, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0180c, 46 },
    { 0, 0 },
    { 0x9e90180e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f901815, 52 },
    { 0xa2101816, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0181b, 80 },
    { 0, 0 },
    { 0xcd80181d, 23 },
    { 0x52f0181e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2701821, 91 },
    { 0xca601822, 76 },
    { 0xdb101823, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf01828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c0182b, 23 },
    { 0x41f0182c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b01830, 91 },
    { 0xf1b01831, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae01835, 98 },
    { 0, 0 },
    { 0x4a101837, 46 },
    { 0, 0 },
    { 0x32101839, 98 },
    { 0x7420183a, 98 },
    { 0, 0 },
    { 0x20183c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd840183f, 34 },
    { 0xc9601840, 61 },
    { 0xb4c01840, 93 },
    { 0xf1101842, 33 },
    { 0x28c01843, 98 },
    { 0xd2001842, 33 },
    { 0x68101841, 61 },
    { 0xa4601846, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb940184a, 93 },
    { 0xa050184a, 61 },
    { 0xbd40184c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd50184f, 90 },
    { 0xa4301850, 95 },
    { 0x4b901851, 95 },
    { 0x85901852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd701855, 98 },
    { 0x33801855, 61 },
    { 0xa1e01857, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0185b, 33 },
    { 0, 0 },
    { 0x7830185d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43401863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510186b, 61 },
    { 0x6fe0186c, 90 },
    { 0x2f70186c, 76 },
    { 0, 0 },
    { 0x6830186f, 43 },
    { 0xc7501870, 34 },
    { 0x60101871, 17 },
    { 0xe0001871, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6201879, 76 },
    { 0xdaf01879, 98 },
    { 0x6be01879, 98 },
    { 0x4140187b, 67 },
    { 0x7560187d, 8 },
    { 0, 0 },
    { 0xfc50187f, 55 },
    { 0xf601880, 61 },
    { 0x65c01880, 55 },
    { 0xefb01882, 33 },
    { 0xd1801883, 78 },
    { 0xfe301881, 33 },
    { 0x69501881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0188c, 33 },
    { 0xde50188c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a01892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46601896, 91 },
    { 0x1dc01897, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f0189d, 33 },
    { 0, 0 },
    { 0xc620189f, 87 },
    { 0, 0 },
    { 0x967018a1, 76 },
    { 0x50d018a1, 90 },
    { 0, 0 },
    { 0x511018a4, 98 },
    { 0x566018a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a018a8, 23 },
    { 0xbbf018a8, 43 },
    { 0xdb8018aa, 76 },
    { 0x36b018ab, 23 },
    { 0x410018ac, 99 },
    { 0x63c018ab, 94 },
    { 0xf4018ac, 34 },
    { 0x697018aa, 23 },
    { 0x8d1018b0, 85 },
    { 0x91b018b0, 61 },
    { 0x49b018b2, 61 },
    { 0xd95018b3, 95 },
    { 0x37e018b0, 95 },
    { 0, 0 },
    { 0x7f7018b6, 95 },
    { 0xcbb018b6, 90 },
    { 0x17b018b8, 33 },
    { 0, 0 },
    { 0x432018ba, 33 },
    { 0xd04018bb, 90 },
    { 0x92c018bc, 33 },
    { 0, 0 },
    { 0x7d8018be, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57018c1, 61 },
    { 0x16f018c2, 77 },
    { 0xd29018c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632018c8, 76 },
    { 0x348018c8, 33 },
    { 0, 0 },
    { 0x1f9018cb, 91 },
    { 0x5c5018cc, 55 },
    { 0xb9c018cc, 84 },
    { 0x1e7018ce, 98 },
    { 0x38f018cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1018d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44018d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0018dc, 90 },
    { 0x67a018dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x419018e0, 46 },
    { 0x627018e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x428018e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70018e7, 95 },
    { 0x941018e8, 93 },
    { 0x581018e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886018ed, 61 },
    { 0x67d018ee, 77 },
    { 0, 0 },
    { 0x460018f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7018f6, 43 },
    { 0xfe2018f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b018fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2018fe, 95 },
    { 0xc62018ff, 33 },
    { 0x5cf018fe, 90 },
    { 0xafe018fe, 90 },
    { 0x55d01900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d201905, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c01908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc470190c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d601912, 90 },
    { 0x65801913, 95 },
    { 0xe4f01912, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3890191a, 46 },
    { 0xfcf0191b, 33 },
    { 0x8aa0191c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d0191f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18101924, 61 },
    { 0, 0 },
    { 0x56201926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c0192c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc01931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c201934, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9301939, 90 },
    { 0xb670193a, 90 },
    { 0x5270193a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70601940, 84 },
    { 0, 0 },
    { 0x9001942, 46 },
    { 0x6f801943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a0194a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0194e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e301953, 67 },
    { 0x8ba01954, 88 },
    { 0xbee01955, 91 },
    { 0, 0 },
    { 0x34801957, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0195b, 100 },
    { 0x89f0195c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3201962, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c601968, 90 },
    { 0x5d301968, 91 },
    { 0, 0 },
    { 0x6f20196b, 55 },
    { 0x63b0196b, 76 },
    { 0, 0 },
    { 0xad80196e, 90 },
    { 0, 0 },
    { 0x8ee01970, 43 },
    { 0x33801971, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1101974, 66 },
    { 0, 0 },
    { 0xa4001976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4730197a, 96 },
    { 0, 0 },
    { 0xbfa0197c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150197f, 23 },
    { 0xb8201980, 99 },
    { 0x39f01981, 33 },
    { 0x2801982, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c01985, 61 },
    { 0x8ab01986, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990198d, 67 },
    { 0xf100198e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93301991, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9710199e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a019a1, 61 },
    { 0x790019a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f019a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc019a8, 95 },
    { 0x173019a8, 61 },
    { 0, 0 },
    { 0x5cf019ab, 98 },
    { 0, 0 },
    { 0xde8019ad, 90 },
    { 0xfd2019ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8019b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f019b8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58019bb, 100 },
    { 0x317019bc, 100 },
    { 0x418019bc, 93 },
    { 0x492019be, 69 },
    { 0x5ed019bf, 92 },
    { 0x65b019bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456019c4, 55 },
    { 0x920019c4, 95 },
    { 0xc35019c6, 98 },
    { 0xfe8019c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1019ca, 90 },
    { 0, 0 },
    { 0x358019cc, 46 },
    { 0x4fe019cd, 33 },
    { 0xb3b019ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2019d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2019d6, 95 },
    { 0x9e9019d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d019db, 81 },
    { 0x6ce019dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1019e1, 55 },
    { 0x29019e2, 66 },
    { 0, 0 },
    { 0x20a019e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b019e7, 76 },
    { 0, 0 },
    { 0x1a4019e9, 66 },
    { 0, 0 },
    { 0xec6019eb, 33 },
    { 0x915019ec, 33 },
    { 0x4ac019ec, 23 },
    { 0xd99019ee, 99 },
    { 0, 0 },
    { 0xce2019f0, 38 },
    { 0, 0 },
    { 0x4b019f2, 80 },
    { 0x8fa019f3, 85 },
    { 0xfa2019f4, 75 },
    { 0x265019f4, 76 },
    { 0x37c019f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab019f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76401a00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca01a05, 76 },
    { 0, 0 },
    { 0x32d01a07, 90 },
    { 0, 0 },
    { 0xb3501a09, 43 },
    { 0, 0 },
    { 0x81601a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1101a11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb01a18, 76 },
    { 0, 0 },
    { 0xbfb01a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66601a20, 61 },
    { 0x61b01a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3801a26, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a001a2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77401a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a01a35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73801a3a, 23 },
    { 0x73d01a3b, 46 },
    { 0, 0 },
    { 0xcaa01a3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50901a42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d01a48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff01a4e, 33 },
    { 0x48e01a4f, 87 },
    { 0, 0 },
    { 0xb7f01a51, 37 },
    { 0, 0 },
    { 0x1bb01a53, 87 },
    { 0x98201a54, 34 },
    { 0x32801a53, 98 },
    { 0xd9e01a56, 33 },
    { 0x60301a53, 90 },
    { 0x6f101a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c701a5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb01a5e, 34 },
    { 0xaa101a5f, 98 },
    { 0x92901a5f, 33 },
    { 0xd7001a60, 61 },
    { 0x54501a60, 90 },
    { 0x1f001a63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63301a66, 87 },
    { 0xc0401a67, 34 },
    { 0xb0301a68, 61 },
    { 0x4df01a69, 43 },
    { 0xeec01a68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c01a70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c301a75, 80 },
    { 0, 0 },
    { 0x3b501a77, 98 },
    { 0, 0 },
    { 0xb0801a79, 61 },
    { 0x55601a7a, 76 },
    { 0x97a01a7b, 88 },
    { 0x14601a7c, 90 },
    { 0xee501a7b, 90 },
    { 0x95d01a7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28701a88, 98 },
    { 0, 0 },
    { 0xa1a01a8a, 80 },
    { 0xb6a01a8a, 95 },
    { 0xc1701a8b, 23 },
    { 0, 0 },
    { 0x9001a8e, 4 },
    { 0xb4101a8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb01a93, 93 },
    { 0x4701a94, 99 },
    { 0xffd01a95, 95 },
    { 0x29201a96, 98 },
    { 0x4c201a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e101a9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7301aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47501aa3, 95 },
    { 0, 0 },
    { 0x7e301aa5, 55 },
    { 0xb7001aa6, 95 },
    { 0xc0001aa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e01aac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63501ab0, 33 },
    { 0x58001ab0, 76 },
    { 0x3301ab0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61001aba, 88 },
    { 0xfa401abb, 43 },
    { 0x2b901abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99701abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68201ac6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b01acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1501ad1, 98 },
    { 0x6801ad1, 90 },
    { 0x26101ad3, 98 },
    { 0xf3701ad1, 93 },
    { 0x46501ad1, 61 },
    { 0, 0 },
    { 0x87401ad7, 61 },
    { 0, 0 },
    { 0x3b901ad9, 87 },
    { 0x37d01ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d01ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1801ae1, 23 },
    { 0xfcd01ae2, 43 },
    { 0x87b01ae3, 61 },
    { 0xaa801ae1, 98 },
    { 0, 0 },
    { 0xe401ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e101aef, 90 },
    { 0x73201af0, 100 },
    { 0xc2501af1, 52 },
    { 0x44b01af0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf401af5, 90 },
    { 0, 0 },
    { 0xce701af7, 100 },
    { 0xc1501af8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b01afc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401aff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec01b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35901b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58901b16, 34 },
    { 0x37201b17, 23 },
    { 0x95a01b18, 95 },
    { 0xcd01b19, 33 },
    { 0x3e401b1a, 76 },
    { 0x9de01b16, 95 },
    { 0, 0 },
    { 0xc6e01b1d, 33 },
    { 0x42701b1d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a801b21, 90 },
    { 0, 0 },
    { 0x14801b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd01b29, 87 },
    { 0x3ad01b29, 84 },
    { 0, 0 },
    { 0x51901b2c, 90 },
    { 0xa5e01b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46901b30, 61 },
    { 0, 0 },
    { 0xc7701b32, 76 },
    { 0, 0 },
    { 0x52e01b34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60801b40, 33 },
    { 0, 0 },
    { 0x96801b42, 61 },
    { 0xfd001b43, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b01b4f, 90 },
    { 0xcc101b4f, 40 },
    { 0xfaa01b4f, 93 },
    { 0xd7301b52, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2501b57, 90 },
    { 0, 0 },
    { 0xabc01b59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2401b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b01b68, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb601b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e01b71, 52 },
    { 0x7eb01b71, 90 },
    { 0xafa01b71, 95 },
    { 0x2ad01b74, 61 },
    { 0x3a401b73, 61 },
    { 0xe4401b76, 84 },
    { 0x10601b77, 91 },
    { 0xe9e01b75, 98 },
    { 0x1a801b78, 66 },
    { 0x83f01b77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e01b7d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48001b82, 61 },
    { 0, 0 },
    { 0xe1001b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41601b8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c01b92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb01b97, 85 },
    { 0, 0 },
    { 0xcb601b99, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce01b9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca01b9f, 77 },
    { 0, 0 },
    { 0xa4001ba1, 23 },
    { 0xca201ba2, 61 },
    { 0xd2301ba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6501ba8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b01bac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c301bb0, 93 },
    { 0x45301bb1, 33 },
    { 0xc9f01bb1, 87 },
    { 0x93001bb3, 55 },
    { 0xddf01bb4, 61 },
    { 0x95401bb5, 61 },
    { 0x47401bb6, 91 },
    { 0x6f901bb1, 91 },
    { 0, 0 },
    { 0xee301bb9, 34 },
    { 0xefc01bba, 91 },
    { 0xe0001bba, 61 },
    { 0, 0 },
    { 0xdd801bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45901bc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a301bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb01bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b701bd5, 33 },
    { 0x46d01bd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9001bda, 95 },
    { 0x35d01bda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1901bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82001be7, 55 },
    { 0x8d601be7, 90 },
    { 0x9ed01be9, 90 },
    { 0xfba01bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c501bf1, 95 },
    { 0, 0 },
    { 0x68801bf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad01bf7, 55 },
    { 0xdc901bf8, 93 },
    { 0x4a601bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e01bfd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a01c01, 98 },
    { 0, 0 },
    { 0x67201c03, 98 },
    { 0xc8e01c04, 56 },
    { 0x46e01c03, 78 },
    { 0x68801c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf601c0a, 46 },
    { 0xe8801c0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xced01c0e, 61 },
    { 0xe1801c0f, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd801c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c01c15, 85 },
    { 0xec501c15, 98 },
    { 0x37d01c17, 87 },
    { 0x61101c17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0101c1e, 23 },
    { 0xcb801c1f, 33 },
    { 0xead01c1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79801c23, 23 },
    { 0x96301c24, 61 },
    { 0x24501c23, 67 },
    { 0, 0 },
    { 0x11801c27, 61 },
    { 0xdc101c28, 75 },
    { 0xa5101c29, 99 },
    { 0x94d01c27, 23 },
    { 0x99b01c28, 95 },
    { 0, 0 },
    { 0x5601c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e101c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc701c3f, 90 },
    { 0, 0 },
    { 0x46e01c41, 68 },
    { 0, 0 },
    { 0x59201c43, 52 },
    { 0x23001c44, 52 },
    { 0x39a01c45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46401c4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5001c4d, 52 },
    { 0, 0 },
    { 0xdfd01c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f01c52, 91 },
    { 0x96301c53, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c01c57, 90 },
    { 0x4e501c57, 61 },
    { 0x11801c57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x57301c5c, 99 },
    { 0x9e701c5d, 61 },
    { 0x6e901c5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef501c61, 93 },
    { 0xb6201c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b01c66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c01c6d, 87 },
    { 0xb0d01c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a01c71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc01c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28201c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d701c7f, 65 },
    { 0, 0 },
    { 0x25901c81, 90 },
    { 0, 0 },
    { 0x1aa01c83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5801c87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6001c8b, 61 },
    { 0x3e801c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd01c90, 93 },
    { 0xc2001c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f801c96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c01c99, 23 },
    { 0x86201c99, 80 },
    { 0xb1501c9b, 23 },
    { 0x29601c9c, 40 },
    { 0xa9901c9b, 55 },
    { 0x8b701c9e, 98 },
    { 0x2d601c9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30801ca3, 67 },
    { 0, 0 },
    { 0x4501ca5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d01ca8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9501cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a501cb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e01cb6, 61 },
    { 0x68101cb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde601cbb, 91 },
    { 0, 0 },
    { 0x41e01cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83101cc0, 23 },
    { 0, 0 },
    { 0xe1801cc2, 90 },
    { 0, 0 },
    { 0xd9101cc4, 93 },
    { 0xfcb01cc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa001cc8, 53 },
    { 0xf5901cc9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad01ccc, 34 },
    { 0xfee01ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b701cd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6601cdc, 61 },
    { 0xc5701cdc, 43 },
    { 0, 0 },
    { 0xbd101cdf, 33 },
    { 0, 0 },
    { 0x8e001ce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97001ce4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a01ce9, 87 },
    { 0xd8401cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1601cf1, 33 },
    { 0x43501cf2, 91 },
    { 0, 0 },
    { 0xcab01cf4, 33 },
    { 0x87601cf4, 61 },
    { 0xcd701cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24701cfb, 76 },
    { 0xc6c01cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc401cff, 61 },
    { 0, 0 },
    { 0x99801d01, 77 },
    { 0x3b101d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41701d08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca01d11, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32801d18, 98 },
    { 0, 0 },
    { 0xc7b01d1a, 34 },
    { 0xd0f01d1a, 47 },
    { 0xf4001d1c, 76 },
    { 0x68201d1c, 23 },
    { 0x8f001d1e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df01d21, 95 },
    { 0x9b501d22, 76 },
    { 0x27d01d23, 98 },
    { 0x17601d23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa901d27, 90 },
    { 0, 0 },
    { 0xc7e01d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01d2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af01d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd01d35, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63101d38, 98 },
    { 0x34a01d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b01d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35001d44, 33 },
    { 0xae01d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea01d4b, 38 },
    { 0xe1b01d4c, 66 },
    { 0x1a001d4b, 38 },
    { 0x9b401d4e, 48 },
    { 0xddb01d4f, 95 },
    { 0x1c101d4f, 90 },
    { 0xc2d01d50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df01d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c901d58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0001d5b, 61 },
    { 0, 0 },
    { 0x92501d5d, 93 },
    { 0, 0 },
    { 0x41001d5f, 61 },
    { 0, 0 },
    { 0x47e01d61, 90 },
    { 0, 0 },
    { 0x50501d63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80801d6c, 46 },
    { 0xcdb01d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf801d71, 90 },
    { 0, 0 },
    { 0x1ba01d73, 33 },
    { 0, 0 },
    { 0x4a801d75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff101d78, 100 },
    { 0x2ed01d78, 55 },
    { 0xe7901d7a, 23 },
    { 0x7d901d7b, 78 },
    { 0x79f01d7b, 61 },
    { 0x73001d7a, 95 },
    { 0x7fa01d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5001d84, 23 },
    { 0x3e601d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd901d89, 46 },
    { 0x2b901d8a, 96 },
    { 0xea201d8a, 46 },
    { 0x61101d8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f01d8f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34001d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd01d98, 33 },
    { 0xef301d98, 55 },
    { 0xa001d9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6201d9d, 98 },
    { 0, 0 },
    { 0x2b301d9f, 93 },
    { 0xbaf01da0, 90 },
    { 0x63001d9f, 93 },
    { 0xa5601da2, 33 },
    { 0x34901da2, 90 },
    { 0xb1b01da4, 52 },
    { 0x32301da3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc01daa, 99 },
    { 0x2c801daa, 85 },
    { 0x97701dac, 61 },
    { 0xaa001dab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91401db7, 76 },
    { 0x59701db8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x98201dbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e501dbf, 75 },
    { 0x7b501dbf, 33 },
    { 0, 0 },
    { 0xc5b01dc2, 87 },
    { 0xbd301dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12901dc7, 95 },
    { 0, 0 },
    { 0x83101dc9, 40 },
    { 0, 0 },
    { 0x4a801dcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65801dce, 95 },
    { 0x13801dcf, 61 },
    { 0xa3301dd0, 93 },
    { 0x87301dd1, 90 },
    { 0x6ed01dd2, 77 },
    { 0x4b501dd2, 85 },
    { 0x94001dd4, 85 },
    { 0x80d01dd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d01ddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c101de5, 61 },
    { 0x79701de6, 67 },
    { 0x87201de7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0301df3, 76 },
    { 0, 0 },
    { 0x3ab01df5, 76 },
    { 0xa2b01df6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41401dfa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54701e00, 59 },
    { 0, 0 },
    { 0xc1f01e02, 33 },
    { 0xe3701e03, 67 },
    { 0, 0 },
    { 0x9bf01e05, 87 },
    { 0, 0 },
    { 0x77201e07, 81 },
    { 0xfd01e08, 96 },
    { 0x46701e09, 55 },
    { 0x39501e08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e701e0d, 34 },
    { 0x47c01e0d, 99 },
    { 0, 0 },
    { 0xd801e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb01e14, 100 },
    { 0x52f01e15, 23 },
    { 0x44d01e14, 95 },
    { 0x80601e14, 85 },
    { 0x34e01e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c01e21, 98 },
    { 0xbb301e22, 34 },
    { 0xeeb01e23, 55 },
    { 0x8401e24, 55 },
    { 0x6201e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13201e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf901e39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea601e3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3701e41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c01e4b, 95 },
    { 0x91601e4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a01e54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0801e57, 66 },
    { 0, 0 },
    { 0xeb801e59, 61 },
    { 0x3f401e5a, 76 },
    { 0xdf901e5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe01e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb01e65, 23 },
    { 0, 0 },
    { 0x43101e67, 90 },
    { 0xca401e68, 46 },
    { 0x58e01e68, 98 },
    { 0xf1f01e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc501e6e, 61 },
    { 0x28201e6e, 95 },
    { 0x67201e6e, 59 },
    { 0xabc01e71, 61 },
    { 0xa0301e72, 40 },
    { 0xaa801e73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5601e78, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d01e7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11201e86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d01e89, 76 },
    { 0xaaa01e89, 43 },
    { 0xe3e01e8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd01e8e, 78 },
    { 0, 0 },
    { 0xbdf01e90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d801e93, 61 },
    { 0xa3801e94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da01e97, 84 },
    { 0xbe201e97, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb01e9e, 34 },
    { 0x29501e9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a01ea3, 42 },
    { 0x6c101ea3, 61 },
    { 0, 0 },
    { 0x32501ea6, 78 },
    { 0, 0 },
    { 0xebf01ea8, 78 },
    { 0x1d501ea9, 55 },
    { 0x32601ea8, 68 },
    { 0x63b01ea8, 55 },
    { 0x30c01ea8, 23 },
    { 0x32a01ead, 66 },
    { 0xa901ead, 52 },
    { 0x40801ea9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b101eb2, 90 },
    { 0x83e01eb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b01eb6, 43 },
    { 0x89b01eb7, 61 },
    { 0xcf001eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67001ebb, 77 },
    { 0x87201ebb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70601ebf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf01ec2, 90 },
    { 0xa3301ec2, 78 },
    { 0, 0 },
    { 0x55201ec5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9601ecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d201ece, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b01ed3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f01ede, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa01ee2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7801ee5, 61 },
    { 0x3c601ee6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df01eeb, 82 },
    { 0x79b01eec, 100 },
    { 0, 0 },
    { 0xe9101eee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda101ef2, 67 },
    { 0xbe301ef3, 55 },
    { 0x2dd01ef4, 77 },
    { 0xa3a01ef4, 90 },
    { 0xcd001ef5, 98 },
    { 0xb0001ef4, 90 },
    { 0x1cd01ef7, 93 },
    { 0x59f01ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b01f04, 61 },
    { 0x34701f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf801f0c, 33 },
    { 0xc3501f0c, 23 },
    { 0xb7401f0e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed01f15, 78 },
    { 0, 0 },
    { 0x4d01f17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d701f1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c01f22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4501f27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec01f2a, 95 },
    { 0, 0 },
    { 0x2a601f2c, 90 },
    { 0x3a001f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd101f32, 98 },
    { 0x2b601f33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf501f3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd501f3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e501f41, 61 },
    { 0x36d01f41, 95 },
    { 0, 0 },
    { 0xc0701f44, 90 },
    { 0, 0 },
    { 0xddb01f46, 55 },
    { 0, 0 },
    { 0x69401f48, 95 },
    { 0xe4601f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c501f50, 94 },
    { 0, 0 },
    { 0xab001f52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1501f55, 33 },
    { 0x89901f56, 91 },
    { 0x6e701f55, 75 },
    { 0, 0 },
    { 0xd6b01f59, 34 },
    { 0, 0 },
    { 0xacc01f5b, 61 },
    { 0xc8b01f5c, 33 },
    { 0x6b01f5c, 90 },
    { 0x87801f5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f01f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb501f6e, 98 },
    { 0x84f01f6f, 34 },
    { 0x9ab01f70, 78 },
    { 0, 0 },
    { 0x9a101f72, 95 },
    { 0, 0 },
    { 0x72501f74, 95 },
    { 0, 0 },
    { 0x71b01f76, 90 },
    { 0, 0 },
    { 0x21e01f78, 33 },
    { 0x3cb01f79, 55 },
    { 0xc0901f78, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4701f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf801f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f301f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79001f8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e01f98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda01f9b, 90 },
    { 0x7c401f9c, 52 },
    { 0x94901f9c, 87 },
    { 0xc4601f9e, 34 },
    { 0x7f401f9f, 33 },
    { 0x6e001f9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f01faa, 33 },
    { 0, 0 },
    { 0xc001fac, 98 },
    { 0, 0 },
    { 0xad001fae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2001fb8, 96 },
    { 0xf4d01fb8, 95 },
    { 0x9001fb8, 33 },
    { 0x6df01fbb, 65 },
    { 0xf8701fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec01fc0, 99 },
    { 0x58901fc1, 76 },
    { 0xfad01fc2, 76 },
    { 0xcc401fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f501fc8, 55 },
    { 0, 0 },
    { 0x15901fca, 90 },
    { 0x9f201fcb, 67 },
    { 0xc6201fcc, 33 },
    { 0x64901fcd, 91 },
    { 0xc3101fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab01fd1, 90 },
    { 0x19701fd2, 33 },
    { 0xc1801fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50801fd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31101fdf, 90 },
    { 0x7d01fe0, 23 },
    { 0xec701fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c01feb, 95 },
    { 0x37601fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98501ff2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59601ff8, 95 },
    { 0, 0 },
    { 0x12401ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd01fff, 95 },
    { 0x2f902000, 98 },
    { 0x81c02000, 33 },
    { 0xfe602000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302005, 78 },
    { 0x8102005, 95 },
    { 0x73e02007, 76 },
    { 0, 0 },
    { 0xda002009, 99 },
    { 0x7ed02009, 67 },
    { 0xe470200b, 77 },
    { 0xa060200c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x150200f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92402012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330201d, 68 },
    { 0xd0c0201e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d02023, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a0203b, 98 },
    { 0xc750203b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3430203f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb802044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c02047, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb200204a, 84 },
    { 0, 0 },
    { 0x8ee0204c, 33 },
    { 0xc2a0204c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xff002050, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0802055, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa60205b, 33 },
    { 0xd6b0205b, 95 },
    { 0x1f0205d, 90 },
    { 0x85c0205d, 98 },
    { 0xdb0205e, 83 },
    { 0xf4e02060, 23 },
    { 0x4f702061, 33 },
    { 0, 0 },
    { 0xc8002063, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20206c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde102072, 23 },
    { 0, 0 },
    { 0x38b02074, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5302077, 98 },
    { 0x6fa02077, 61 },
    { 0xa1802079, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a40207c, 76 },
    { 0xe7e0207d, 84 },
    { 0x9c40207e, 81 },
    { 0xbe20207e, 66 },
    { 0xff902080, 55 },
    { 0x4100207f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d02089, 76 },
    { 0, 0 },
    { 0xec0208b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd690208f, 84 },
    { 0xd5f02090, 43 },
    { 0xa102091, 23 },
    { 0x3bd02091, 23 },
    { 0x63a02091, 33 },
    { 0x98b02094, 65 },
    { 0x21e02094, 61 },
    { 0x6d402096, 61 },
    { 0x12602092, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b0209b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1020a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7020a4, 43 },
    { 0, 0 },
    { 0x3f1020a6, 75 },
    { 0xf5e020a7, 90 },
    { 0xf83020a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661020b0, 98 },
    { 0x740020b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a020b6, 33 },
    { 0x81020b6, 99 },
    { 0x633020b6, 90 },
    { 0xc60020b9, 61 },
    { 0xea4020b9, 33 },
    { 0xf48020ba, 61 },
    { 0, 0 },
    { 0x598020bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c020c0, 90 },
    { 0xee9020c0, 51 },
    { 0xc65020c1, 23 },
    { 0xc17020c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x243020c6, 84 },
    { 0xb3b020c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed020cb, 95 },
    { 0x7da020cc, 91 },
    { 0x33e020cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff020d0, 95 },
    { 0, 0 },
    { 0x4f3020d2, 88 },
    { 0x1c1020d3, 95 },
    { 0x6d4020d2, 23 },
    { 0, 0 },
    { 0x38d020d6, 90 },
    { 0xb39020d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca020da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee020df, 33 },
    { 0, 0 },
    { 0xfe020e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1020e4, 76 },
    { 0xbb8020e5, 98 },
    { 0xd9a020e5, 34 },
    { 0, 0 },
    { 0xdfb020e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e020eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962020ef, 43 },
    { 0xcd1020f0, 55 },
    { 0, 0 },
    { 0xff8020f2, 61 },
    { 0x409020f3, 34 },
    { 0x9d9020f4, 61 },
    { 0x31a020f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c020f8, 61 },
    { 0, 0 },
    { 0x3d0020fa, 93 },
    { 0x397020fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d02100, 67 },
    { 0, 0 },
    { 0x1f702102, 24 },
    { 0, 0 },
    { 0x23b02104, 77 },
    { 0x36d02105, 64 },
    { 0x94702106, 95 },
    { 0, 0 },
    { 0xd3b02108, 90 },
    { 0, 0 },
    { 0x93e0210a, 23 },
    { 0, 0 },
    { 0x2ba0210c, 85 },
    { 0xbb80210c, 98 },
    { 0xdf70210c, 33 },
    { 0xaba0210c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a02112, 55 },
    { 0, 0 },
    { 0xd8b02114, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd550211d, 34 },
    { 0, 0 },
    { 0x4370211f, 95 },
    { 0, 0 },
    { 0x60102121, 43 },
    { 0x68602122, 23 },
    { 0xc6f02123, 43 },
    { 0x4cb02123, 23 },
    { 0xf9f02125, 34 },
    { 0x79302125, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced0212a, 33 },
    { 0, 0 },
    { 0x68d0212c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0212f, 43 },
    { 0xe2402130, 95 },
    { 0xfc602131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9702134, 33 },
    { 0, 0 },
    { 0xed402136, 33 },
    { 0xe9502136, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e60213c, 66 },
    { 0, 0 },
    { 0x39f0213e, 61 },
    { 0, 0 },
    { 0x2fe02140, 34 },
    { 0xd3902140, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa02145, 99 },
    { 0x10402146, 61 },
    { 0x2b802147, 33 },
    { 0x42602145, 95 },
    { 0, 0 },
    { 0x9b30214a, 66 },
    { 0x77b0214b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f02150, 93 },
    { 0x4c602150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd902156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45202159, 33 },
    { 0xb3a0215a, 23 },
    { 0, 0 },
    { 0xd090215c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d702160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b02166, 61 },
    { 0, 0 },
    { 0xfab02168, 61 },
    { 0xea302169, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a50216c, 34 },
    { 0x2410216d, 61 },
    { 0x5d0216d, 84 },
    { 0x3220216f, 40 },
    { 0, 0 },
    { 0xd7f02171, 43 },
    { 0x2ca02172, 23 },
    { 0xd2002172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d0217b, 33 },
    { 0, 0 },
    { 0x8e30217d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d02185, 76 },
    { 0, 0 },
    { 0xf3502187, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440218d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92202192, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f10219a, 90 },
    { 0, 0 },
    { 0x1be0219c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x110219f, 23 },
    { 0, 0 },
    { 0x99b021a1, 55 },
    { 0xe0c021a2, 76 },
    { 0, 0 },
    { 0x2e021a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca021a9, 95 },
    { 0x87021aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122021ae, 61 },
    { 0x6f8021af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b021b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc021b8, 46 },
    { 0x246021b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22021bd, 61 },
    { 0xcb1021be, 98 },
    { 0, 0 },
    { 0x903021c0, 98 },
    { 0, 0 },
    { 0x715021c2, 84 },
    { 0xad0021c2, 48 },
    { 0x409021c4, 43 },
    { 0x20d021c2, 23 },
    { 0x8cd021c6, 90 },
    { 0, 0 },
    { 0x663021c8, 95 },
    { 0xb84021c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9021db, 55 },
    { 0, 0 },
    { 0x355021dd, 90 },
    { 0, 0 },
    { 0xd94021df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69021e7, 52 },
    { 0xd38021e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92021eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76021f0, 90 },
    { 0, 0 },
    { 0x2e021f2, 91 },
    { 0xa6e021f3, 81 },
    { 0x798021f3, 95 },
    { 0x384021f5, 75 },
    { 0, 0 },
    { 0x565021f7, 76 },
    { 0x2a1021f8, 90 },
    { 0xfc0021f9, 99 },
    { 0x8ab021fa, 98 },
    { 0x5eb021f7, 59 },
    { 0, 0 },
    { 0x5b3021fd, 23 },
    { 0x71a021fe, 76 },
    { 0x644021fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02204, 98 },
    { 0xf7602205, 23 },
    { 0xae802206, 95 },
    { 0xdf702207, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870220a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb0220f, 65 },
    { 0x5c02210, 23 },
    { 0xbd602211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74202215, 69 },
    { 0x3fd02216, 33 },
    { 0xa9402216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee60221d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d02220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d02224, 100 },
    { 0x7bf02225, 46 },
    { 0xd2e02225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe050222a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe02231, 95 },
    { 0, 0 },
    { 0x60f02233, 90 },
    { 0x9b502234, 33 },
    { 0, 0 },
    { 0xb5002236, 87 },
    { 0xb3b02236, 47 },
    { 0xf4402238, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6700223b, 34 },
    { 0, 0 },
    { 0xa3c0223d, 61 },
    { 0x30c0223d, 95 },
    { 0x92f0223f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1902245, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9930224e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87702256, 76 },
    { 0x91c02257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9480225e, 90 },
    { 0, 0 },
    { 0x52f02260, 85 },
    { 0xc1102261, 90 },
    { 0x1cd02262, 78 },
    { 0xb5b02262, 52 },
    { 0xb0202264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940226a, 90 },
    { 0, 0 },
    { 0x6fb0226c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59502270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d02274, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d02277, 90 },
    { 0x4bd02278, 87 },
    { 0, 0 },
    { 0x3ea0227a, 76 },
    { 0, 0 },
    { 0x9c10227c, 23 },
    { 0xcc0227d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c702280, 76 },
    { 0xd5402281, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a02285, 34 },
    { 0x78102285, 33 },
    { 0x33402287, 77 },
    { 0x10602287, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9240228b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8102292, 98 },
    { 0, 0 },
    { 0x8c402294, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a80229a, 95 },
    { 0x4f30229b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6022a1, 78 },
    { 0x53c022a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54022a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67022aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd022af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d022b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7022b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb022ba, 61 },
    { 0xb05022ba, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec022c3, 55 },
    { 0x1a0022c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b022d0, 98 },
    { 0xe9b022d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4022d5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x411022d8, 91 },
    { 0xc3d022d9, 61 },
    { 0x371022da, 90 },
    { 0, 0 },
    { 0xbf4022dc, 34 },
    { 0xc7022dc, 98 },
    { 0x9cb022de, 43 },
    { 0xc6e022de, 76 },
    { 0xd23022df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586022e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932022ed, 33 },
    { 0x461022ed, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x433022f1, 43 },
    { 0xc5a022f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b022f6, 66 },
    { 0x3fe022f7, 23 },
    { 0x444022f8, 23 },
    { 0x739022f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4502302, 90 },
    { 0, 0 },
    { 0x66802304, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a02307, 95 },
    { 0x2df02308, 23 },
    { 0xc7602309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0230c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54602311, 90 },
    { 0, 0 },
    { 0x17302313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55402316, 97 },
    { 0xe4f02317, 61 },
    { 0x77702316, 61 },
    { 0, 0 },
    { 0x5c80231a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd80231d, 98 },
    { 0xf010231e, 34 },
    { 0xdcd0231f, 34 },
    { 0xb8a0231d, 90 },
    { 0x26b02321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d802324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80202329, 80 },
    { 0x86502329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a90232e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf002331, 91 },
    { 0, 0 },
    { 0x65202333, 85 },
    { 0x80e02334, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf802339, 98 },
    { 0, 0 },
    { 0xffe0233b, 36 },
    { 0x67b0233c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0233f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1602347, 90 },
    { 0, 0 },
    { 0x2be02349, 98 },
    { 0x7fa02349, 33 },
    { 0, 0 },
    { 0x2730234c, 23 },
    { 0x1730234d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a02356, 61 },
    { 0xd4402357, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6050235a, 95 },
    { 0, 0 },
    { 0xbd70235c, 90 },
    { 0, 0 },
    { 0xbbc0235e, 90 },
    { 0x7510235f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96202366, 81 },
    { 0x3ac02367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140236a, 55 },
    { 0xd710236a, 33 },
    { 0x8ce0236c, 98 },
    { 0x6ba0236c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7802372, 87 },
    { 0x76902372, 43 },
    { 0x7dd02374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb902377, 93 },
    { 0x51d02378, 61 },
    { 0xaa902379, 61 },
    { 0, 0 },
    { 0xad70237b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a02381, 93 },
    { 0x6bb02382, 43 },
    { 0x3f402383, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b02387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa0238c, 95 },
    { 0, 0 },
    { 0x5fa0238e, 77 },
    { 0xed30238e, 69 },
    { 0xc950238e, 61 },
    { 0x16c02391, 90 },
    { 0x63c02390, 61 },
    { 0x45402393, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c02398, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80239f, 98 },
    { 0xb3e023a0, 87 },
    { 0xfe4023a0, 90 },
    { 0xe5e023a1, 98 },
    { 0x6d1023a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b023a6, 40 },
    { 0x14b023a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72023ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75023b3, 90 },
    { 0, 0 },
    { 0x492023b5, 66 },
    { 0, 0 },
    { 0x686023b7, 98 },
    { 0, 0 },
    { 0x677023b9, 95 },
    { 0, 0 },
    { 0x17023bb, 90 },
    { 0, 0 },
    { 0x749023bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec023c0, 98 },
    { 0x593023c1, 91 },
    { 0, 0 },
    { 0xcd7023c3, 55 },
    { 0x85e023c4, 69 },
    { 0x91a023c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62023c8, 46 },
    { 0, 0 },
    { 0xec4023ca, 76 },
    { 0xd44023cb, 76 },
    { 0xf54023cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42023de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436023e2, 90 },
    { 0x855023e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1023e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357023e9, 61 },
    { 0x10c023ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d023ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39023f0, 61 },
    { 0x555023f1, 98 },
    { 0xac7023f2, 23 },
    { 0, 0 },
    { 0x73a023f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb023f9, 98 },
    { 0x3c2023fa, 33 },
    { 0x32b023f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23602403, 98 },
    { 0xf3e02404, 76 },
    { 0x4d902405, 90 },
    { 0, 0 },
    { 0x53902407, 61 },
    { 0, 0 },
    { 0x67702409, 23 },
    { 0, 0 },
    { 0x8c40240b, 85 },
    { 0xd720240c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602412, 99 },
    { 0x9bb02413, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f402418, 90 },
    { 0x3ba02419, 95 },
    { 0x90e0241a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a02421, 23 },
    { 0, 0 },
    { 0xae502423, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e02427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa0242c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a02432, 90 },
    { 0, 0 },
    { 0x5bc02434, 90 },
    { 0xc0202435, 100 },
    { 0x42002436, 33 },
    { 0xa1602436, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0243a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7902441, 90 },
    { 0xcbf02442, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f02445, 61 },
    { 0xad02446, 33 },
    { 0, 0 },
    { 0x97e02448, 55 },
    { 0x4cc02448, 95 },
    { 0x7680244a, 23 },
    { 0xab70244a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670244f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1880245b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f02463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b102469, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f0246e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19202472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca02475, 98 },
    { 0x2902475, 76 },
    { 0xde202477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2002480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa802485, 33 },
    { 0x4eb02486, 23 },
    { 0, 0 },
    { 0xa1302488, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc880248c, 23 },
    { 0xbd40248d, 95 },
    { 0, 0 },
    { 0x7120248f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d02495, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce02498, 34 },
    { 0, 0 },
    { 0xe3f0249a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9970249e, 43 },
    { 0x2eb0249f, 88 },
    { 0xa450249f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8024a3, 9 },
    { 0, 0 },
    { 0xf9f024a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x445024a8, 34 },
    { 0, 0 },
    { 0x2d9024aa, 100 },
    { 0, 0 },
    { 0x59b024ac, 55 },
    { 0xfd4024ad, 90 },
    { 0x267024ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd024b3, 61 },
    { 0x17e024b4, 90 },
    { 0x77b024b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4024b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b024bc, 33 },
    { 0x7fd024bd, 85 },
    { 0xa01024bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85024c2, 25 },
    { 0xeb8024c3, 61 },
    { 0, 0 },
    { 0xbe6024c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9024c8, 87 },
    { 0xb62024c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329024ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2024d1, 88 },
    { 0, 0 },
    { 0x20c024d3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x342024d6, 23 },
    { 0, 0 },
    { 0x16f024d8, 33 },
    { 0x333024d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e024dc, 66 },
    { 0x40f024dd, 52 },
    { 0xf31024de, 76 },
    { 0x667024de, 80 },
    { 0x2f3024e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c024e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce024e8, 23 },
    { 0x633024e8, 55 },
    { 0x40a024ea, 95 },
    { 0x37a024eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85024ee, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57024f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b024f6, 98 },
    { 0, 0 },
    { 0xb84024f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8024fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12602503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec602506, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee0250c, 91 },
    { 0, 0 },
    { 0xa540250e, 61 },
    { 0xe2e0250f, 61 },
    { 0, 0 },
    { 0x2a202511, 33 },
    { 0x8d902511, 34 },
    { 0, 0 },
    { 0xc5602514, 55 },
    { 0xbd602515, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45102519, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0251f, 33 },
    { 0xdd402520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b0252d, 90 },
    { 0x8b70252e, 95 },
    { 0xcb00252d, 82 },
    { 0x5aa02530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10253a, 34 },
    { 0, 0 },
    { 0xb960253c, 61 },
    { 0xf460253c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa02541, 90 },
    { 0x4eb02542, 34 },
    { 0x1a302542, 95 },
    { 0x5ab02544, 93 },
    { 0x7ca02545, 33 },
    { 0x38402546, 55 },
    { 0xb1e02544, 97 },
    { 0, 0 },
    { 0x9c102549, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a0254e, 93 },
    { 0x95d0254f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a902552, 43 },
    { 0x45802553, 90 },
    { 0xd3f02554, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac02557, 76 },
    { 0x33902558, 33 },
    { 0, 0 },
    { 0x6a10255a, 91 },
    { 0xdd90255b, 76 },
    { 0, 0 },
    { 0x1260255d, 33 },
    { 0, 0 },
    { 0x2580255f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd602563, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc0256a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8210256e, 61 },
    { 0xc160256e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91502572, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54402576, 91 },
    { 0x79502576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c0257b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0257e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xed802581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc02585, 66 },
    { 0, 0 },
    { 0x7a002587, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x790258a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60258e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32402591, 23 },
    { 0xac202591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a02595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2502598, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf10259f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57025a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3025a9, 34 },
    { 0x271025a9, 58 },
    { 0xc5c025ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93025b0, 69 },
    { 0, 0 },
    { 0x7b3025b2, 98 },
    { 0x1bb025b3, 46 },
    { 0x92f025b4, 55 },
    { 0x161025b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446025bb, 61 },
    { 0x744025bb, 34 },
    { 0, 0 },
    { 0x56c025be, 61 },
    { 0xde0025bf, 98 },
    { 0x8d1025bf, 91 },
    { 0, 0 },
    { 0x787025c2, 61 },
    { 0, 0 },
    { 0xdbb025c4, 55 },
    { 0xf62025c5, 98 },
    { 0xa67025c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda025cc, 23 },
    { 0xbf8025cc, 67 },
    { 0x957025cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc025d1, 99 },
    { 0xd68025d2, 23 },
    { 0, 0 },
    { 0x12a025d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91025d7, 26 },
    { 0x7af025d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1025db, 34 },
    { 0x7d0025dc, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c025e1, 77 },
    { 0, 0 },
    { 0x7da025e3, 67 },
    { 0xfd8025e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d025e9, 91 },
    { 0x78e025e9, 43 },
    { 0x736025eb, 76 },
    { 0xd3025ec, 90 },
    { 0xa1e025ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f025f0, 43 },
    { 0, 0 },
    { 0xe80025f2, 70 },
    { 0x8d9025f2, 55 },
    { 0x863025f4, 83 },
    { 0xbe8025f5, 98 },
    { 0, 0 },
    { 0x3fc025f7, 33 },
    { 0xbd6025f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9025fe, 93 },
    { 0xf58025fe, 81 },
    { 0x1dd02600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e02605, 95 },
    { 0x6502606, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380260c, 98 },
    { 0x6fe0260c, 91 },
    { 0, 0 },
    { 0xbc20260f, 33 },
    { 0xeca02610, 23 },
    { 0x6580260f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88402619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a50261d, 33 },
    { 0x9920261e, 46 },
    { 0x6920261f, 23 },
    { 0x8730261e, 17 },
    { 0x35002620, 33 },
    { 0xc720261e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62702626, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c202629, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650262d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc602631, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a502639, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0263d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8602640, 43 },
    { 0, 0 },
    { 0x78702642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98102645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce02648, 99 },
    { 0xe4002649, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9202651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe402659, 90 },
    { 0xd3d02659, 33 },
    { 0x4f70265b, 95 },
    { 0x9b50265b, 84 },
    { 0xa010265c, 98 },
    { 0, 0 },
    { 0xd530265f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb902665, 76 },
    { 0xd0d02666, 90 },
    { 0x90f02665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f00266a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85002671, 68 },
    { 0xc2e02671, 76 },
    { 0x28202672, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53502678, 61 },
    { 0xfda02679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d0267e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6902681, 90 },
    { 0xdb802682, 90 },
    { 0, 0 },
    { 0x9602684, 23 },
    { 0, 0 },
    { 0xdbc02686, 95 },
    { 0xa7e02687, 93 },
    { 0x52802688, 66 },
    { 0x39e02688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa280268d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e02691, 98 },
    { 0x45302692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d0269c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c0269f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x661026a2, 23 },
    { 0x45c026a3, 52 },
    { 0x159026a3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d026a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8026ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4026af, 76 },
    { 0, 0 },
    { 0xa1e026b1, 76 },
    { 0x670026b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c026bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4026c3, 36 },
    { 0x7c2026c4, 33 },
    { 0xbcf026c3, 85 },
    { 0x7fc026c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8026c9, 84 },
    { 0, 0 },
    { 0xa4c026cb, 61 },
    { 0x9dc026cb, 97 },
    { 0xc1026cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111026d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e026db, 76 },
    { 0x37f026dc, 46 },
    { 0xd84026dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df026e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72026e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0026ee, 90 },
    { 0x633026ee, 95 },
    { 0, 0 },
    { 0x500026f1, 98 },
    { 0xe54026f2, 90 },
    { 0xaa8026f3, 95 },
    { 0xda8026f4, 61 },
    { 0xab0026f4, 91 },
    { 0x4f1026f2, 34 },
    { 0x62026f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d026fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed002703, 77 },
    { 0xeb602703, 61 },
    { 0, 0 },
    { 0xc4502706, 55 },
    { 0, 0 },
    { 0xe7302708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2720270b, 65 },
    { 0, 0 },
    { 0x750270d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27302713, 76 },
    { 0, 0 },
    { 0xcea02715, 84 },
    { 0x55e02716, 90 },
    { 0, 0 },
    { 0x68802718, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c30271b, 33 },
    { 0, 0 },
    { 0x62b0271d, 52 },
    { 0x9f30271d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x38702721, 81 },
    { 0x8d002722, 90 },
    { 0, 0 },
    { 0x4e902724, 55 },
    { 0xa3802725, 90 },
    { 0, 0 },
    { 0xe0e02727, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d0272c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da02730, 55 },
    { 0xaa702730, 33 },
    { 0x5b802730, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db02737, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d0273b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d02741, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83102745, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0274d, 23 },
    { 0, 0 },
    { 0x8020274f, 96 },
    { 0xf6502750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0802753, 61 },
    { 0x8eb02753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20275e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c02766, 23 },
    { 0x10602767, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf330276b, 65 },
    { 0, 0 },
    { 0xe7c0276d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd02771, 95 },
    { 0xca602772, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc02775, 43 },
    { 0x26e02775, 33 },
    { 0, 0 },
    { 0x51e02778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a00277b, 61 },
    { 0xe3b0277c, 90 },
    { 0xd180277c, 61 },
    { 0xe810277e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37702782, 78 },
    { 0x73d02782, 33 },
    { 0x1cb02783, 43 },
    { 0x75802785, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x55102788, 98 },
    { 0x56f02789, 91 },
    { 0xfb002788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7302794, 61 },
    { 0x78702795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e90279a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef0279d, 91 },
    { 0xe2f0279e, 55 },
    { 0x6d0279e, 23 },
    { 0xd01027a0, 46 },
    { 0, 0 },
    { 0x246027a2, 93 },
    { 0, 0 },
    { 0xb36027a4, 95 },
    { 0x862027a5, 95 },
    { 0x879027a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4027a9, 100 },
    { 0, 0 },
    { 0x1a027ab, 23 },
    { 0, 0 },
    { 0xf9a027ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8027b1, 81 },
    { 0, 0 },
    { 0xefe027b3, 90 },
    { 0x43a027b4, 90 },
    { 0xa8027b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20027b8, 90 },
    { 0, 0 },
    { 0xb78027ba, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34027be, 33 },
    { 0, 0 },
    { 0x593027c0, 34 },
    { 0x102027c1, 90 },
    { 0xc11027c2, 23 },
    { 0, 0 },
    { 0x159027c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2027c8, 66 },
    { 0x791027c9, 61 },
    { 0x26a027c9, 52 },
    { 0xc47027cb, 95 },
    { 0, 0 },
    { 0x39b027cd, 52 },
    { 0, 0 },
    { 0x572027cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0027d2, 61 },
    { 0, 0 },
    { 0xbcb027d4, 43 },
    { 0x886027d5, 55 },
    { 0x537027d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95027d9, 56 },
    { 0x3fe027da, 61 },
    { 0x800027db, 61 },
    { 0, 0 },
    { 0x222027dd, 61 },
    { 0, 0 },
    { 0x5fb027df, 33 },
    { 0, 0 },
    { 0x2b1027e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe027ee, 43 },
    { 0x9c7027ef, 43 },
    { 0, 0 },
    { 0xb9c027f1, 38 },
    { 0xce4027f2, 98 },
    { 0x883027f2, 33 },
    { 0, 0 },
    { 0xabb027f5, 33 },
    { 0xc45027f5, 78 },
    { 0, 0 },
    { 0x200027f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541027ff, 98 },
    { 0, 0 },
    { 0xedd02801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b02807, 61 },
    { 0, 0 },
    { 0x62902809, 92 },
    { 0xb3a0280a, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a0280d, 90 },
    { 0, 0 },
    { 0xfd00280f, 33 },
    { 0xdd002810, 87 },
    { 0, 0 },
    { 0xf1902812, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69302817, 90 },
    { 0, 0 },
    { 0x9dc02819, 77 },
    { 0x6a902819, 90 },
    { 0xc300281b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda0281e, 75 },
    { 0, 0 },
    { 0x53002820, 91 },
    { 0x95102820, 100 },
    { 0, 0 },
    { 0x49302823, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77702827, 95 },
    { 0x62f02827, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2630282c, 8 },
    { 0, 0 },
    { 0x2020282e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea502831, 98 },
    { 0, 0 },
    { 0xaee02833, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b20283b, 33 },
    { 0, 0 },
    { 0xa150283d, 98 },
    { 0x6900283e, 85 },
    { 0x6b30283e, 95 },
    { 0x1ca02840, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc02843, 34 },
    { 0x47202844, 90 },
    { 0x18102845, 52 },
    { 0, 0 },
    { 0xad102847, 33 },
    { 0xfd402848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb0284b, 98 },
    { 0x9850284b, 61 },
    { 0x6470284d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a02855, 55 },
    { 0x21d02856, 90 },
    { 0xff002857, 95 },
    { 0x8eb02855, 34 },
    { 0xf3902859, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f0285d, 33 },
    { 0, 0 },
    { 0xd840285f, 33 },
    { 0xb202860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53802864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd520286a, 96 },
    { 0x9570286b, 46 },
    { 0x7230286b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba0286f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe02872, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2802876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c0287c, 84 },
    { 0x5cc0287d, 34 },
    { 0xb310287c, 33 },
    { 0x5680287f, 91 },
    { 0xd2b02880, 95 },
    { 0xeb02880, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45602885, 61 },
    { 0x61502886, 61 },
    { 0, 0 },
    { 0xc0902888, 78 },
    { 0, 0 },
    { 0x2160288a, 34 },
    { 0, 0 },
    { 0xaac0288c, 61 },
    { 0x15f0288c, 61 },
    { 0x9260288c, 90 },
    { 0x1c50288f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a02892, 17 },
    { 0xeca02893, 98 },
    { 0x56202894, 97 },
    { 0xe4b02894, 23 },
    { 0, 0 },
    { 0x23102897, 61 },
    { 0, 0 },
    { 0x28502899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4040289d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d028a2, 76 },
    { 0, 0 },
    { 0x434028a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57028a9, 90 },
    { 0x19c028a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd028ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202028b3, 98 },
    { 0xb78028b4, 90 },
    { 0x52d028b5, 90 },
    { 0xbdc028b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b028b9, 95 },
    { 0x9df028b9, 40 },
    { 0x1fe028bb, 33 },
    { 0x685028b9, 64 },
    { 0x650028bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd028c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818028c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626028cd, 66 },
    { 0x702028ce, 43 },
    { 0xcc7028ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1028d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76028d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x679028db, 33 },
    { 0x4d9028db, 61 },
    { 0xf17028dd, 84 },
    { 0xefe028de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29028e1, 95 },
    { 0x5b3028e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692028ed, 80 },
    { 0, 0 },
    { 0xcd0028ef, 98 },
    { 0x1d1028f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c028f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b028f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09028fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x895028fe, 95 },
    { 0, 0 },
    { 0x88202900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c302904, 99 },
    { 0x80802904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa002908, 90 },
    { 0xfe302909, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd020290f, 95 },
    { 0, 0 },
    { 0xe3c02911, 67 },
    { 0xc6502912, 61 },
    { 0xa7d02913, 77 },
    { 0x7a002914, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa710291e, 33 },
    { 0xf970291e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70102922, 66 },
    { 0xe3302923, 85 },
    { 0x7ba02924, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32402928, 23 },
    { 0, 0 },
    { 0xce80292a, 90 },
    { 0xd220292b, 33 },
    { 0x1a0292c, 65 },
    { 0xc4f0292d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58902932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc02937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd0293b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0293e, 87 },
    { 0xfe20293e, 33 },
    { 0x4fa0293f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba02943, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce02948, 99 },
    { 0, 0 },
    { 0x1ca0294a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e0294f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef602952, 48 },
    { 0x2b202952, 61 },
    { 0x95002954, 98 },
    { 0x93402955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e02958, 77 },
    { 0, 0 },
    { 0x7a80295a, 76 },
    { 0, 0 },
    { 0xec0295c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa02961, 78 },
    { 0x41402962, 100 },
    { 0xbb502962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa02966, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c90296a, 76 },
    { 0xcd30296b, 95 },
    { 0xfb30296c, 61 },
    { 0xb800296d, 98 },
    { 0x320296d, 66 },
    { 0, 0 },
    { 0xc0502970, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57902975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b90297a, 33 },
    { 0, 0 },
    { 0x4f40297c, 34 },
    { 0x53b0297d, 75 },
    { 0, 0 },
    { 0xe320297f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e802984, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59102988, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7302990, 85 },
    { 0x6b02991, 67 },
    { 0x8b102992, 40 },
    { 0x5e802993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67802999, 77 },
    { 0, 0 },
    { 0x2cd0299b, 100 },
    { 0x3000299b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c0299f, 76 },
    { 0x798029a0, 95 },
    { 0xce3029a1, 90 },
    { 0, 0 },
    { 0x804029a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f029a6, 100 },
    { 0xd1e029a7, 98 },
    { 0x4b029a8, 98 },
    { 0, 0 },
    { 0xebb029aa, 91 },
    { 0xbec029ab, 90 },
    { 0x477029ac, 33 },
    { 0x588029ad, 84 },
    { 0x309029ae, 76 },
    { 0, 0 },
    { 0x753029b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a029b3, 40 },
    { 0x1ba029b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae029b7, 84 },
    { 0, 0 },
    { 0xa16029b9, 33 },
    { 0x831029ba, 34 },
    { 0, 0 },
    { 0x16b029bc, 33 },
    { 0, 0 },
    { 0xb91029be, 23 },
    { 0x40b029bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7029c2, 23 },
    { 0xd58029c2, 75 },
    { 0x7a6029c4, 98 },
    { 0x840029c5, 90 },
    { 0x243029c6, 52 },
    { 0xa1b029c7, 95 },
    { 0x2c2029c8, 33 },
    { 0x295029c9, 33 },
    { 0x916029c8, 55 },
    { 0x1f6029cb, 55 },
    { 0xef7029c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd029cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e029d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118029da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0029e4, 95 },
    { 0x1b4029e4, 76 },
    { 0x43a029e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873029ea, 76 },
    { 0, 0 },
    { 0xae0029ec, 87 },
    { 0, 0 },
    { 0x76b029ee, 33 },
    { 0xdb0029ef, 33 },
    { 0x75b029f0, 98 },
    { 0xe29029ef, 93 },
    { 0xd25029ef, 90 },
    { 0x974029f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c029f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7202a03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9402a06, 95 },
    { 0, 0 },
    { 0x83602a08, 34 },
    { 0, 0 },
    { 0x8d902a0a, 61 },
    { 0x1ff02a0b, 76 },
    { 0x8b402a0c, 88 },
    { 0xa102a0a, 95 },
    { 0xb6b02a0e, 95 },
    { 0x99202a0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb02a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d02a16, 66 },
    { 0xad002a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68902a22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a002a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a702a2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f02a30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e02a36, 90 },
    { 0x5b402a37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae02a3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e02a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c02a4b, 33 },
    { 0x42202a4c, 33 },
    { 0xe3702a4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b02a63, 100 },
    { 0x2a702a64, 61 },
    { 0x42302a65, 38 },
    { 0x5302a65, 90 },
    { 0xd1402a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90802a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde002a6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad02a73, 95 },
    { 0x57402a74, 90 },
    { 0, 0 },
    { 0x60602a76, 81 },
    { 0x72102a77, 98 },
    { 0xc6102a77, 69 },
    { 0, 0 },
    { 0x69102a7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13902a7e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7902a87, 76 },
    { 0xac02a88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8802a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee502a93, 69 },
    { 0, 0 },
    { 0x2be02a95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x24902a98, 33 },
    { 0x8b702a98, 34 },
    { 0x29c02a9a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af02a9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1802aa1, 23 },
    { 0xcc102aa1, 23 },
    { 0x1f502aa3, 91 },
    { 0x88a02aa4, 61 },
    { 0xe9f02aa5, 34 },
    { 0xf9c02aa6, 98 },
    { 0x51502aa3, 98 },
    { 0xdf02aa7, 90 },
    { 0, 0 },
    { 0xb3c02aaa, 65 },
    { 0x2202aab, 90 },
    { 0, 0 },
    { 0x2dc02aad, 76 },
    { 0xc4b02aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c02ab2, 90 },
    { 0xbee02ab3, 34 },
    { 0, 0 },
    { 0x64402ab5, 23 },
    { 0x4fa02ab6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf602abc, 95 },
    { 0x29502abd, 61 },
    { 0xd7202abe, 87 },
    { 0, 0 },
    { 0x22902ac0, 61 },
    { 0x83402ac1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f102ac6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f02acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e02ad7, 23 },
    { 0x13902ad7, 34 },
    { 0, 0 },
    { 0x1e302ada, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf02ade, 90 },
    { 0, 0 },
    { 0x74702ae0, 91 },
    { 0, 0 },
    { 0x99c02ae2, 93 },
    { 0xfa902ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d02ae7, 52 },
    { 0x2e102ae8, 94 },
    { 0xedc02ae8, 34 },
    { 0, 0 },
    { 0xd6702aeb, 23 },
    { 0xe7702aec, 95 },
    { 0, 0 },
    { 0x3d402aee, 33 },
    { 0x3d202aef, 61 },
    { 0, 0 },
    { 0xe2e02af1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa02af4, 33 },
    { 0, 0 },
    { 0xc9602af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d02afb, 98 },
    { 0, 0 },
    { 0x69602afd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4402b01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf02b09, 87 },
    { 0, 0 },
    { 0xf102b0b, 23 },
    { 0x67b02b0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc02b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37402b18, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63402b1c, 81 },
    { 0xdce02b1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d02b24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e02b29, 33 },
    { 0xf7302b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea02b2d, 33 },
    { 0, 0 },
    { 0x29402b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71002b32, 67 },
    { 0x74102b33, 33 },
    { 0x97a02b32, 61 },
    { 0xc2802b34, 100 },
    { 0, 0 },
    { 0x4b502b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d02b3b, 90 },
    { 0xba302b3b, 23 },
    { 0, 0 },
    { 0x69102b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50002b41, 33 },
    { 0, 0 },
    { 0xdbf02b43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea02b46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca02b4a, 61 },
    { 0x65d02b4a, 66 },
    { 0x16502b4c, 98 },
    { 0, 0 },
    { 0xb0902b4e, 95 },
    { 0x5f802b4f, 76 },
    { 0xa0d02b50, 90 },
    { 0x27a02b51, 61 },
    { 0xedd02b50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b702b55, 55 },
    { 0, 0 },
    { 0x75502b57, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52002b5d, 98 },
    { 0x33402b5d, 55 },
    { 0, 0 },
    { 0xd9f02b60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf02b67, 33 },
    { 0x76e02b68, 84 },
    { 0x97902b68, 99 },
    { 0x41802b6a, 90 },
    { 0, 0 },
    { 0xebc02b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f02b70, 67 },
    { 0, 0 },
    { 0xd5802b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab02b79, 33 },
    { 0, 0 },
    { 0x65602b7b, 97 },
    { 0x4a502b7c, 98 },
    { 0, 0 },
    { 0x27402b7e, 95 },
    { 0xb6e02b7e, 55 },
    { 0x20102b7e, 34 },
    { 0xa2902b7f, 36 },
    { 0, 0 },
    { 0x6a702b83, 34 },
    { 0, 0 },
    { 0x2002b85, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0102b88, 23 },
    { 0, 0 },
    { 0x90f02b8a, 33 },
    { 0x2ee02b8b, 90 },
    { 0xa9102b8b, 55 },
    { 0, 0 },
    { 0xe8a02b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79402b91, 61 },
    { 0x51702b92, 90 },
    { 0x54402b93, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a02b96, 23 },
    { 0xf1802b96, 23 },
    { 0, 0 },
    { 0xe8602b99, 69 },
    { 0xab902b9a, 98 },
    { 0x65602b9a, 33 },
    { 0x9b102b9c, 61 },
    { 0x30a02b9d, 76 },
    { 0xdeb02b9e, 61 },
    { 0x54802b9f, 77 },
    { 0xec402b9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47102ba3, 81 },
    { 0x49c02ba4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7202ba9, 66 },
    { 0, 0 },
    { 0x6e502bab, 55 },
    { 0, 0 },
    { 0x8f202bad, 52 },
    { 0x40102bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb002bb9, 90 },
    { 0x36002bba, 95 },
    { 0xc4c02bbb, 95 },
    { 0x62a02bbc, 85 },
    { 0x1202bbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74402bc1, 77 },
    { 0x33e02bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12202bca, 46 },
    { 0, 0 },
    { 0x63902bcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f02bd1, 76 },
    { 0, 0 },
    { 0xe4b02bd3, 84 },
    { 0xbba02bd3, 76 },
    { 0x73302bd5, 90 },
    { 0, 0 },
    { 0x82002bd7, 91 },
    { 0, 0 },
    { 0xee402bd9, 95 },
    { 0x6b502bda, 33 },
    { 0x49802bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c702be8, 36 },
    { 0x67802be9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf02bef, 76 },
    { 0x7b402bf0, 90 },
    { 0x85a02bf1, 61 },
    { 0, 0 },
    { 0x4dd02bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb02bf7, 78 },
    { 0, 0 },
    { 0x75202bf9, 76 },
    { 0, 0 },
    { 0x2002bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd02c02, 55 },
    { 0xc1d02c03, 78 },
    { 0x70b02c04, 23 },
    { 0x6db02c05, 76 },
    { 0x56702c06, 77 },
    { 0, 0 },
    { 0x9a802c08, 83 },
    { 0x2ec02c09, 43 },
    { 0x49002c09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a02c0e, 85 },
    { 0xf3702c0f, 34 },
    { 0x56a02c10, 90 },
    { 0x5e802c10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c402c14, 95 },
    { 0, 0 },
    { 0xc6802c16, 33 },
    { 0x4bc02c17, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd02c1a, 87 },
    { 0x45f02c1b, 95 },
    { 0x59e02c1c, 61 },
    { 0x7c802c1d, 61 },
    { 0x71802c1c, 33 },
    { 0x4b202c1d, 90 },
    { 0x19702c1c, 75 },
    { 0x66602c21, 90 },
    { 0, 0 },
    { 0x30c02c23, 33 },
    { 0, 0 },
    { 0x66902c25, 70 },
    { 0x3302c26, 76 },
    { 0x8e02c27, 98 },
    { 0, 0 },
    { 0x40002c29, 90 },
    { 0x14302c29, 55 },
    { 0, 0 },
    { 0xb8c02c2c, 87 },
    { 0x7b102c2c, 90 },
    { 0xabf02c2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91402c33, 98 },
    { 0x9dc02c34, 64 },
    { 0x59c02c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee02c48, 95 },
    { 0xcac02c49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c502c4e, 88 },
    { 0x3fe02c4f, 95 },
    { 0xfcf02c50, 78 },
    { 0x34f02c51, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e02c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac02c5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81302c63, 97 },
    { 0xe8702c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff302c68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1102c6d, 81 },
    { 0, 0 },
    { 0x8fd02c6f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a302c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70502c76, 88 },
    { 0, 0 },
    { 0x80902c78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed902c80, 81 },
    { 0x53102c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26202c8d, 98 },
    { 0, 0 },
    { 0x20c02c8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f02c98, 76 },
    { 0x21e02c99, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee02c9c, 38 },
    { 0x54302c9d, 90 },
    { 0xc1302c9c, 95 },
    { 0xc7502c9f, 43 },
    { 0, 0 },
    { 0x7802ca1, 90 },
    { 0xa4002ca2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39802ca5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3302caa, 43 },
    { 0, 0 },
    { 0xed602cac, 33 },
    { 0xeb02cad, 90 },
    { 0xdeb02cad, 61 },
    { 0x3f002caf, 93 },
    { 0xbdd02cb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc02cb4, 90 },
    { 0x9ee02cb5, 33 },
    { 0x60c02cb4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db02cbb, 55 },
    { 0, 0 },
    { 0xd6802cbd, 61 },
    { 0x36002cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e02cc1, 52 },
    { 0x29502cc1, 85 },
    { 0xf7d02cc3, 76 },
    { 0x83102cc3, 33 },
    { 0xc6702cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2102cc9, 90 },
    { 0, 0 },
    { 0x31802ccb, 98 },
    { 0x1702ccc, 39 },
    { 0x91802ccb, 23 },
    { 0, 0 },
    { 0xf3902ccf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91502cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1402cda, 90 },
    { 0xe6c02cda, 23 },
    { 0xcf02cdc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87902ce0, 90 },
    { 0, 0 },
    { 0xc7902ce2, 81 },
    { 0x43602ce2, 90 },
    { 0xafa02ce3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff902ceb, 85 },
    { 0xb3202cec, 33 },
    { 0x23202ced, 98 },
    { 0, 0 },
    { 0xd8402cef, 23 },
    { 0, 0 },
    { 0xd0102cf1, 90 },
    { 0x68302cf2, 55 },
    { 0, 0 },
    { 0x52702cf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad02cfa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3802cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902d04, 78 },
    { 0xab302d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0202d09, 76 },
    { 0, 0 },
    { 0xdc102d0b, 46 },
    { 0x86002d0b, 61 },
    { 0xfea02d0c, 23 },
    { 0, 0 },
    { 0x9f402d0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91102d14, 34 },
    { 0x7d002d14, 55 },
    { 0, 0 },
    { 0x55402d17, 98 },
    { 0, 0 },
    { 0x8d002d19, 33 },
    { 0, 0 },
    { 0xd0702d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f02d1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc802d22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a02d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f02d30, 61 },
    { 0xe3e02d31, 98 },
    { 0x62b02d32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87602d37, 90 },
    { 0xded02d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d02d3e, 61 },
    { 0x8e702d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4102d43, 34 },
    { 0x57302d43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f502d4e, 88 },
    { 0x89002d4f, 76 },
    { 0xbdc02d50, 77 },
    { 0x59a02d51, 90 },
    { 0x9a302d52, 48 },
    { 0xe9602d53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d802d56, 61 },
    { 0x6ae02d57, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc702d5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c02d61, 43 },
    { 0x75702d62, 17 },
    { 0, 0 },
    { 0xee902d64, 90 },
    { 0xc5f02d65, 91 },
    { 0xe5a02d65, 23 },
    { 0x4f902d67, 90 },
    { 0x4e302d68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe602d76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57302d7b, 98 },
    { 0x96302d7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0702d83, 90 },
    { 0x1b402d84, 43 },
    { 0x5be02d85, 93 },
    { 0x28102d86, 46 },
    { 0xf2d02d87, 61 },
    { 0, 0 },
    { 0x3de02d89, 61 },
    { 0xbcc02d8a, 99 },
    { 0xebf02d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a602d8e, 90 },
    { 0x7ec02d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1102d92, 61 },
    { 0, 0 },
    { 0xac502d94, 90 },
    { 0x15702d95, 33 },
    { 0, 0 },
    { 0xe3c02d97, 93 },
    { 0, 0 },
    { 0xd3902d99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60202d9e, 36 },
    { 0xac902d9f, 55 },
    { 0, 0 },
    { 0xa6a02da1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7202da6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc02dae, 33 },
    { 0x8bb02daf, 65 },
    { 0xd3e02daf, 23 },
    { 0xcc602db1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11802db8, 55 },
    { 0xe0b02db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6802dbc, 61 },
    { 0xe0202dbd, 76 },
    { 0xc3d02dbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3202dc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b02dc8, 90 },
    { 0x30b02dc8, 95 },
    { 0x3a402dca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a02dd2, 33 },
    { 0, 0 },
    { 0x9a402dd4, 76 },
    { 0, 0 },
    { 0x9df02dd6, 61 },
    { 0x92b02dd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72102ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a02de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db02de7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba102deb, 52 },
    { 0x6ad02dec, 95 },
    { 0x7ac02dec, 66 },
    { 0x81e02dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b02df1, 90 },
    { 0xb7c02df1, 94 },
    { 0xa7102df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae02df8, 76 },
    { 0x2d002df9, 95 },
    { 0x4a902df9, 55 },
    { 0xf2802df9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88702dfe, 95 },
    { 0x1c902dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4602e06, 23 },
    { 0, 0 },
    { 0x75e02e08, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd802e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e502e11, 61 },
    { 0x93702e11, 98 },
    { 0, 0 },
    { 0xad402e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4302e17, 34 },
    { 0x64802e17, 33 },
    { 0, 0 },
    { 0xa9502e1a, 33 },
    { 0, 0 },
    { 0xdc202e1c, 33 },
    { 0xf0802e1d, 64 },
    { 0x93602e1c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c02e21, 98 },
    { 0, 0 },
    { 0x5c202e23, 100 },
    { 0x46a02e23, 33 },
    { 0x9c502e25, 95 },
    { 0xf3902e26, 89 },
    { 0xb0c02e27, 78 },
    { 0xab302e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99402e2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3802e37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b602e3b, 55 },
    { 0x23c02e3b, 61 },
    { 0xdce02e3d, 67 },
    { 0, 0 },
    { 0xe402e3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b602e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f02e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c602e58, 33 },
    { 0, 0 },
    { 0x38702e5a, 43 },
    { 0xcc702e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d02e60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c702e68, 33 },
    { 0x14f02e69, 78 },
    { 0, 0 },
    { 0x1fd02e6b, 90 },
    { 0x39402e6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf02e6f, 55 },
    { 0x9ab02e70, 23 },
    { 0, 0 },
    { 0xf8902e72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d02e76, 52 },
    { 0x65a02e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab02e7d, 23 },
    { 0xb5a02e7e, 87 },
    { 0xf9902e7f, 93 },
    { 0, 0 },
    { 0x2fc02e81, 33 },
    { 0xaad02e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a802e85, 43 },
    { 0xe7402e85, 33 },
    { 0xe7e02e85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa802e90, 43 },
    { 0x9b302e91, 84 },
    { 0x46502e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c02e95, 99 },
    { 0xbb802e96, 34 },
    { 0, 0 },
    { 0x1b002e98, 80 },
    { 0xc6102e99, 75 },
    { 0x56c02e9a, 43 },
    { 0xfc102e98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba02e9f, 33 },
    { 0x60202ea0, 43 },
    { 0, 0 },
    { 0x5a102ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e202ea7, 98 },
    { 0x14202ea7, 76 },
    { 0x7a302ea7, 34 },
    { 0, 0 },
    { 0x52502eab, 33 },
    { 0xba802eac, 87 },
    { 0x59902eac, 95 },
    { 0xdf102eae, 61 },
    { 0x16002eaf, 23 },
    { 0xc1702eb0, 17 },
    { 0xd4702eae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c902eb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c02eba, 23 },
    { 0xbe002ebb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d02ebe, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9902ec1, 67 },
    { 0xc4f02ec2, 87 },
    { 0xc3102ec3, 98 },
    { 0, 0 },
    { 0xccf02ec5, 76 },
    { 0x54202ec5, 61 },
    { 0xdd402ec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76002ecf, 48 },
    { 0x2af02ed0, 95 },
    { 0, 0 },
    { 0x98a02ed2, 61 },
    { 0xcdb02ed2, 33 },
    { 0x84402ed2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c402ed7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf02eda, 34 },
    { 0x6dc02edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61302edf, 100 },
    { 0, 0 },
    { 0xee402ee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5002ee4, 98 },
    { 0, 0 },
    { 0x4e702ee6, 90 },
    { 0xf6e02ee7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40002ef3, 87 },
    { 0x3d502ef4, 90 },
    { 0x63802ef4, 33 },
    { 0xb7d02ef6, 86 },
    { 0, 0 },
    { 0xd3b02ef8, 61 },
    { 0x65902ef9, 95 },
    { 0, 0 },
    { 0x87002efb, 61 },
    { 0, 0 },
    { 0x5d402efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74902f03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca02f06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c02f0c, 95 },
    { 0xb4102f0d, 94 },
    { 0, 0 },
    { 0x46602f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94202f19, 23 },
    { 0, 0 },
    { 0x59a02f1b, 90 },
    { 0xaf902f1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93702f23, 23 },
    { 0xe1c02f23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5302f27, 33 },
    { 0x97b02f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b02f33, 95 },
    { 0xe1702f34, 77 },
    { 0x68f02f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2002f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c02f45, 76 },
    { 0x41602f46, 33 },
    { 0, 0 },
    { 0x12802f48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d02f4c, 76 },
    { 0xe4902f4d, 61 },
    { 0xc6802f4d, 38 },
    { 0x40402f4f, 61 },
    { 0, 0 },
    { 0xd9702f51, 55 },
    { 0, 0 },
    { 0xb4602f53, 23 },
    { 0, 0 },
    { 0x64802f55, 56 },
    { 0x1ad02f55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15602f5c, 99 },
    { 0x9d702f5d, 61 },
    { 0, 0 },
    { 0x50802f5f, 95 },
    { 0xe2702f60, 61 },
    { 0x4ef02f61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea502f67, 61 },
    { 0x58002f68, 61 },
    { 0, 0 },
    { 0xaff02f6a, 98 },
    { 0xe8602f6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6702f70, 46 },
    { 0x19f02f70, 76 },
    { 0xbf302f72, 33 },
    { 0xfa502f73, 91 },
    { 0, 0 },
    { 0x1b302f75, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b02f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd002f7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86902f81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba002f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46802f8f, 93 },
    { 0, 0 },
    { 0xcf602f91, 95 },
    { 0x59702f92, 33 },
    { 0xe9902f91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95502f96, 90 },
    { 0x5b02f96, 95 },
    { 0xf4e02f98, 98 },
    { 0x3d202f99, 34 },
    { 0x4b202f96, 95 },
    { 0xe3902f9b, 90 },
    { 0x85402f9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84202fa0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b202fa3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f02faa, 40 },
    { 0xdcc02fab, 33 },
    { 0xefa02fac, 95 },
    { 0xd7802fad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x19502fb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f02fb4, 77 },
    { 0x84502fb5, 33 },
    { 0, 0 },
    { 0x402fb7, 90 },
    { 0xbff02fb8, 33 },
    { 0x5f802fb9, 98 },
    { 0xb4802fba, 61 },
    { 0xe3b02fba, 61 },
    { 0, 0 },
    { 0xdc102fbd, 90 },
    { 0x32b02fbe, 76 },
    { 0xd1002fbe, 34 },
    { 0x47202fc0, 88 },
    { 0x5ef02fc0, 61 },
    { 0x73002fc2, 55 },
    { 0xb4a02fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3002fc6, 85 },
    { 0, 0 },
    { 0x5b502fc8, 90 },
    { 0, 0 },
    { 0x37a02fca, 65 },
    { 0xe5502fcb, 33 },
    { 0xa2f02fcb, 34 },
    { 0xc5a02fcd, 33 },
    { 0x25602fcd, 66 },
    { 0xff902fce, 90 },
    { 0xbdc02fcd, 84 },
    { 0x96302fcf, 66 },
    { 0x4d402fcc, 98 },
    { 0xf7d02fd3, 61 },
    { 0, 0 },
    { 0x9702fd5, 33 },
    { 0xb5402fd5, 33 },
    { 0, 0 },
    { 0x90802fd8, 93 },
    { 0xc7402fd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb002fdd, 95 },
    { 0x55402fde, 61 },
    { 0xada02fdf, 90 },
    { 0x11602fe0, 46 },
    { 0xdcb02fde, 90 },
    { 0x2b202fe2, 61 },
    { 0xe4802fe2, 90 },
    { 0x26d02fe4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa02fe8, 98 },
    { 0xa2402fe9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb02fec, 90 },
    { 0x96b02fed, 38 },
    { 0x29c02fee, 76 },
    { 0x5db02fef, 76 },
    { 0, 0 },
    { 0x18402ff1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa02ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd302ffb, 61 },
    { 0, 0 },
    { 0x26b02ffd, 75 },
    { 0x2a802ffe, 90 },
    { 0, 0 },
    { 0x10003000, 61 },
    { 0xc5203000, 70 },
    { 0x67503000, 53 },
    { 0xdc403003, 98 },
    { 0x5cd03004, 93 },
    { 0x7c103003, 90 },
    { 0x30b03006, 55 },
    { 0x27203006, 55 },
    { 0x47103008, 87 },
    { 0x91303009, 61 },
    { 0x61a0300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d0300d, 90 },
    { 0x49e0300d, 43 },
    { 0xa1c0300d, 61 },
    { 0xbb10300f, 23 },
    { 0x55d03011, 95 },
    { 0, 0 },
    { 0x60e03013, 43 },
    { 0x2cd03014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0301d, 100 },
    { 0, 0 },
    { 0x94a0301f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa03022, 33 },
    { 0xa8803022, 93 },
    { 0xd7303022, 43 },
    { 0x80403024, 81 },
    { 0x4da03026, 90 },
    { 0, 0 },
    { 0xbb703028, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc500302b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf0302f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd03032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31703035, 61 },
    { 0x5d203036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb03039, 90 },
    { 0, 0 },
    { 0x2d30303b, 66 },
    { 0x5880303c, 61 },
    { 0, 0 },
    { 0x80c0303e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c703041, 98 },
    { 0xcc003042, 52 },
    { 0x91203043, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00304a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd03050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90003054, 68 },
    { 0, 0 },
    { 0xafc03056, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe0305c, 33 },
    { 0x4900305c, 46 },
    { 0, 0 },
    { 0xa730305f, 33 },
    { 0, 0 },
    { 0x53803061, 23 },
    { 0x54e03062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330306d, 76 },
    { 0xed60306d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39503072, 61 },
    { 0x85e03073, 55 },
    { 0xa2d03072, 23 },
    { 0xe603073, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2303079, 55 },
    { 0xda503079, 76 },
    { 0xa720307b, 61 },
    { 0, 0 },
    { 0xc460307d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e03082, 95 },
    { 0x4f903082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9380308a, 95 },
    { 0xccb0308b, 95 },
    { 0x4910308b, 38 },
    { 0, 0 },
    { 0xf140308e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c03093, 52 },
    { 0x4a303094, 69 },
    { 0xc0603095, 55 },
    { 0x8f03096, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3390309b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f030a3, 78 },
    { 0x7d1030a4, 53 },
    { 0xf42030a5, 76 },
    { 0x7d3030a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x962030a9, 76 },
    { 0, 0 },
    { 0x172030ab, 23 },
    { 0xa52030ab, 33 },
    { 0xf7b030ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e030b1, 33 },
    { 0x2b3030b2, 23 },
    { 0xc4b030b2, 90 },
    { 0x998030b4, 90 },
    { 0x2d5030b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455030bb, 95 },
    { 0, 0 },
    { 0x68d030bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da030c2, 95 },
    { 0x3c2030c3, 43 },
    { 0x4c4030c4, 33 },
    { 0x4ac030c5, 75 },
    { 0, 0 },
    { 0x950030c7, 90 },
    { 0x623030c8, 34 },
    { 0, 0 },
    { 0x420030ca, 34 },
    { 0, 0 },
    { 0xfa2030cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5030d0, 95 },
    { 0, 0 },
    { 0xe49030d2, 23 },
    { 0xeea030d3, 98 },
    { 0, 0 },
    { 0xf21030d5, 43 },
    { 0, 0 },
    { 0xc03030d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785030e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f030e8, 46 },
    { 0xa74030e9, 55 },
    { 0, 0 },
    { 0xbec030eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc030f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x757030f3, 81 },
    { 0xd19030f3, 90 },
    { 0xa25030f5, 55 },
    { 0, 0 },
    { 0x2b8030f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899030fe, 61 },
    { 0x331030fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef03103, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75903108, 95 },
    { 0x26903109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf70310e, 90 },
    { 0x96e0310f, 95 },
    { 0xb7403110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab03114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a30311c, 46 },
    { 0, 0 },
    { 0xbda0311e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd603121, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6303125, 23 },
    { 0x3ed03126, 98 },
    { 0, 0 },
    { 0x83b03128, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b0312b, 90 },
    { 0x7a70312c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990312f, 43 },
    { 0xd0f03130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56203134, 76 },
    { 0x24403135, 61 },
    { 0x4a803135, 55 },
    { 0x6a103137, 61 },
    { 0x60003138, 55 },
    { 0x12603139, 33 },
    { 0, 0 },
    { 0x7d0313b, 75 },
    { 0xb810313c, 93 },
    { 0x7550313d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5903143, 95 },
    { 0xbda03144, 65 },
    { 0x42b03143, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc03150, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79703153, 84 },
    { 0x13803154, 46 },
    { 0x20d03154, 81 },
    { 0x7e03155, 95 },
    { 0x99903157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9040315f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f03162, 75 },
    { 0xa4c03163, 55 },
    { 0x7b903163, 68 },
    { 0, 0 },
    { 0x30a03166, 55 },
    { 0, 0 },
    { 0x2ba03168, 34 },
    { 0, 0 },
    { 0xc070316a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb403170, 61 },
    { 0x95c03171, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d03174, 33 },
    { 0xa4603174, 33 },
    { 0xf5203176, 100 },
    { 0x59903177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e20317d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c03183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe803189, 76 },
    { 0, 0 },
    { 0xf40318b, 95 },
    { 0xa640318c, 87 },
    { 0xafc0318b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3503190, 76 },
    { 0x51e03191, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede03199, 98 },
    { 0, 0 },
    { 0x9b30319b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee031a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4031a4, 33 },
    { 0, 0 },
    { 0x4cc031a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531031ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c031ae, 84 },
    { 0xeb0031af, 93 },
    { 0xa6f031af, 95 },
    { 0xcd8031b1, 98 },
    { 0x808031b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6031b6, 95 },
    { 0x953031b6, 95 },
    { 0x6e2031b8, 55 },
    { 0x16f031b7, 93 },
    { 0x679031ba, 98 },
    { 0x705031ba, 52 },
    { 0x61c031bb, 100 },
    { 0x39a031bd, 91 },
    { 0x7f1031be, 98 },
    { 0x18f031bd, 87 },
    { 0x19b031bd, 61 },
    { 0x1b3031c1, 43 },
    { 0x7c1031c2, 52 },
    { 0xf11031bb, 23 },
    { 0, 0 },
    { 0x19d031c5, 95 },
    { 0x79c031c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd031cb, 34 },
    { 0xa7a031cb, 33 },
    { 0x2b7031cd, 80 },
    { 0xf14031cd, 85 },
    { 0xa5c031cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x223031d2, 55 },
    { 0, 0 },
    { 0x2f0031d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9031d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c031dd, 61 },
    { 0, 0 },
    { 0x800031df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f031e3, 33 },
    { 0xe8f031e3, 65 },
    { 0, 0 },
    { 0x218031e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252031eb, 33 },
    { 0x306031eb, 66 },
    { 0x1e1031ec, 33 },
    { 0x4a0031ee, 95 },
    { 0x2a1031ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6031f4, 61 },
    { 0x85d031f4, 93 },
    { 0x393031f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x793031f9, 77 },
    { 0x685031fa, 46 },
    { 0, 0 },
    { 0x28e031fc, 33 },
    { 0x305031fd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41803204, 55 },
    { 0xe4903205, 61 },
    { 0xa703204, 43 },
    { 0x26703207, 33 },
    { 0x13903207, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570320e, 90 },
    { 0x8e60320f, 33 },
    { 0, 0 },
    { 0x58703211, 95 },
    { 0, 0 },
    { 0x65f03213, 61 },
    { 0x4eb03213, 90 },
    { 0, 0 },
    { 0xd8603216, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45103219, 85 },
    { 0xf5303219, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf600321f, 46 },
    { 0, 0 },
    { 0x2d903221, 99 },
    { 0xb2e03222, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae03227, 23 },
    { 0, 0 },
    { 0xaa703229, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf490322c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0322f, 33 },
    { 0x7490322f, 89 },
    { 0xd3003231, 33 },
    { 0xad0322f, 34 },
    { 0xbe903232, 52 },
    { 0xba403234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f803237, 95 },
    { 0x87403238, 55 },
    { 0xb1103239, 90 },
    { 0x160323a, 91 },
    { 0, 0 },
    { 0xcde0323c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a0323f, 77 },
    { 0xd2803240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b03244, 43 },
    { 0, 0 },
    { 0x8f103246, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b0324a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe520324e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4903253, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff03258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4103263, 98 },
    { 0xc0e03264, 55 },
    { 0x91d03264, 61 },
    { 0x5a103266, 61 },
    { 0x6903267, 90 },
    { 0x5dd03266, 43 },
    { 0x35203269, 33 },
    { 0x5e703266, 33 },
    { 0x9cb03265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70326e, 61 },
    { 0x7b30326f, 66 },
    { 0, 0 },
    { 0xf503271, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274, 85 },
    { 0x62403274, 90 },
    { 0, 0 },
    { 0x68503277, 33 },
    { 0x3fb03278, 95 },
    { 0xa8d03279, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb100327c, 90 },
    { 0xf10327d, 76 },
    { 0xe130327c, 78 },
    { 0xbf10327d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e03282, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81303290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d503293, 23 },
    { 0x74003293, 76 },
    { 0, 0 },
    { 0x55a03296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa0329e, 61 },
    { 0x6fb0329e, 33 },
    { 0x6f30329f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf032a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc032a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc032ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c032b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2032b7, 90 },
    { 0xf82032b7, 33 },
    { 0xa43032b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97032bc, 61 },
    { 0xf30032bd, 90 },
    { 0x60b032bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f032c1, 98 },
    { 0xac1032c2, 90 },
    { 0xd98032c3, 76 },
    { 0x992032c3, 61 },
    { 0xd80032c3, 17 },
    { 0xa46032c1, 61 },
    { 0, 0 },
    { 0xbf6032c8, 90 },
    { 0, 0 },
    { 0x934032ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae032cf, 55 },
    { 0xc5b032d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef032d7, 61 },
    { 0, 0 },
    { 0xb95032d9, 33 },
    { 0, 0 },
    { 0x385032db, 95 },
    { 0xa85032dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d032df, 76 },
    { 0xf0032df, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8032e4, 76 },
    { 0x209032e5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327032ec, 61 },
    { 0, 0 },
    { 0xd8c032ee, 90 },
    { 0xb98032ef, 76 },
    { 0, 0 },
    { 0x72e032f1, 55 },
    { 0x74c032f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b032f7, 76 },
    { 0xf02032f7, 90 },
    { 0xc8c032f7, 23 },
    { 0x52c032f7, 23 },
    { 0xd35032f8, 98 },
    { 0xec8032fc, 65 },
    { 0xa6032fd, 77 },
    { 0xa1f032fd, 61 },
    { 0x867032ff, 98 },
    { 0xe0e03300, 88 },
    { 0x1a503300, 33 },
    { 0xe2c03300, 23 },
    { 0xd3e03303, 93 },
    { 0xc09032ff, 23 },
    { 0, 0 },
    { 0xbc003306, 76 },
    { 0, 0 },
    { 0x85703308, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd60330e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac03314, 87 },
    { 0xf8c03314, 34 },
    { 0x5d603316, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5203319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9390331c, 34 },
    { 0, 0 },
    { 0x43d0331e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00332a, 69 },
    { 0, 0 },
    { 0xfa60332c, 23 },
    { 0x7c30332d, 19 },
    { 0, 0 },
    { 0x8250332f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb03334, 33 },
    { 0x75d03335, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9460333a, 90 },
    { 0x7c10333a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12503340, 34 },
    { 0x1a203341, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xece03344, 78 },
    { 0xf0303345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e90334b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab503352, 90 },
    { 0xafe03352, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e303357, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310335b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df0335e, 33 },
    { 0xbb30335f, 90 },
    { 0, 0 },
    { 0xcb603361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8080336b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e03371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a03376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83103379, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790337e, 33 },
    { 0x9820337f, 33 },
    { 0xd170337f, 43 },
    { 0xd103381, 66 },
    { 0x7eb0337f, 90 },
    { 0x5600337f, 90 },
    { 0x1370337e, 98 },
    { 0xe1403385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960338f, 95 },
    { 0x81a03390, 82 },
    { 0, 0 },
    { 0x7a303392, 61 },
    { 0, 0 },
    { 0xe303394, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8340339d, 90 },
    { 0x3c80339e, 85 },
    { 0xe510339d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70033a4, 76 },
    { 0x264033a5, 34 },
    { 0xd57033a4, 34 },
    { 0xe45033a7, 90 },
    { 0x8bc033a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0033ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6033af, 76 },
    { 0, 0 },
    { 0x7f9033b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c033b4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x791033b7, 95 },
    { 0x4d7033b8, 61 },
    { 0x2c4033b9, 23 },
    { 0x2fd033ba, 95 },
    { 0x584033ba, 33 },
    { 0xdbd033b8, 34 },
    { 0x324033ba, 76 },
    { 0xfe7033be, 98 },
    { 0x60a033be, 91 },
    { 0xe52033bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe033c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556033c9, 90 },
    { 0x26e033c9, 23 },
    { 0x1af033cb, 58 },
    { 0xd59033ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06033d1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f033d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922033dd, 76 },
    { 0xaf1033dd, 61 },
    { 0x48033df, 90 },
    { 0x5ae033de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193033e5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d033e8, 23 },
    { 0x6b0033e9, 23 },
    { 0x5a033ea, 33 },
    { 0x8e5033ea, 90 },
    { 0, 0 },
    { 0x4b6033ed, 88 },
    { 0x49b033ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6033f1, 90 },
    { 0, 0 },
    { 0xaa5033f3, 66 },
    { 0x9e4033f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597033ff, 100 },
    { 0, 0 },
    { 0x6e303401, 99 },
    { 0, 0 },
    { 0xec103403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe03407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0340b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0340e, 90 },
    { 0x4a70340f, 90 },
    { 0x37603410, 93 },
    { 0xcfd03410, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79503414, 61 },
    { 0xca503415, 98 },
    { 0, 0 },
    { 0x98003417, 76 },
    { 0xe8703417, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f0341c, 87 },
    { 0x2f50341d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74703420, 52 },
    { 0xda203420, 95 },
    { 0x10403421, 33 },
    { 0x2b03420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92403427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990342a, 76 },
    { 0xe170342a, 98 },
    { 0xc5e0342c, 98 },
    { 0x8870342a, 90 },
    { 0xfb40342e, 76 },
    { 0, 0 },
    { 0x30403430, 76 },
    { 0x5e603430, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d03439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080343c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf503441, 61 },
    { 0x24503441, 23 },
    { 0x8c003442, 33 },
    { 0x1f03443, 75 },
    { 0x84c03445, 23 },
    { 0x9b303445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b90344d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb03450, 93 },
    { 0x70803450, 33 },
    { 0xaea03451, 95 },
    { 0, 0 },
    { 0x92d03454, 91 },
    { 0x6fb03454, 33 },
    { 0xf4103456, 61 },
    { 0x10403455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc0345a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba30345d, 33 },
    { 0x2ae0345d, 17 },
    { 0, 0 },
    { 0xa3603460, 95 },
    { 0x74503461, 40 },
    { 0x72c03461, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8300346d, 23 },
    { 0x3280346d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d03471, 90 },
    { 0x65603472, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb03477, 76 },
    { 0x7fc03477, 93 },
    { 0x19603477, 95 },
    { 0x5ad03478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7710347d, 33 },
    { 0xe150347e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42503485, 61 },
    { 0x1d503485, 23 },
    { 0x5a703487, 98 },
    { 0, 0 },
    { 0xd8103489, 88 },
    { 0xac603489, 87 },
    { 0, 0 },
    { 0x7e50348c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61603495, 95 },
    { 0xeb203495, 90 },
    { 0xb6903497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220349d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9034a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43034a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62034a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54034ac, 61 },
    { 0x5b2034ac, 95 },
    { 0xc3d034ae, 33 },
    { 0, 0 },
    { 0xa6034b0, 43 },
    { 0, 0 },
    { 0xa12034b2, 34 },
    { 0, 0 },
    { 0xa6c034b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97034b7, 98 },
    { 0x103034b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14034c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca034c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b034cc, 33 },
    { 0, 0 },
    { 0x337034ce, 76 },
    { 0x284034ce, 61 },
    { 0, 0 },
    { 0xbca034d1, 90 },
    { 0xee034d1, 90 },
    { 0xe36034d1, 99 },
    { 0xea7034d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e034d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd034dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b034e5, 52 },
    { 0xbc4034e6, 76 },
    { 0x605034e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x174034ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12034ee, 95 },
    { 0xd33034ee, 61 },
    { 0x251034ee, 84 },
    { 0x845034f0, 55 },
    { 0x8d3034ef, 31 },
    { 0xb39034f1, 99 },
    { 0x15d034f1, 53 },
    { 0xf5e034f5, 84 },
    { 0, 0 },
    { 0xbeb034f7, 61 },
    { 0xa41034f8, 33 },
    { 0xf32034f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd703500, 33 },
    { 0, 0 },
    { 0xb9603502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98403507, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a303515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4830351c, 90 },
    { 0x7a50351d, 34 },
    { 0, 0 },
    { 0xece0351f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50703522, 95 },
    { 0xa2003523, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e60352a, 61 },
    { 0xdff0352b, 81 },
    { 0, 0 },
    { 0x4050352d, 34 },
    { 0, 0 },
    { 0xcd60352f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28803534, 76 },
    { 0, 0 },
    { 0x7cf03536, 34 },
    { 0x4e403536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c0353b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a80353e, 33 },
    { 0xd030353e, 33 },
    { 0, 0 },
    { 0x7a203541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0354a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c0354f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580355c, 90 },
    { 0x8180355d, 90 },
    { 0, 0 },
    { 0xb9f0355f, 98 },
    { 0x3aa03560, 61 },
    { 0xc9803561, 64 },
    { 0xd6c03560, 95 },
    { 0x8b80355f, 98 },
    { 0x69c03564, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade03568, 55 },
    { 0x60f03569, 61 },
    { 0, 0 },
    { 0xbd20356b, 33 },
    { 0, 0 },
    { 0x8c00356d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d03572, 33 },
    { 0x80a03573, 90 },
    { 0x7503574, 23 },
    { 0xf2303575, 99 },
    { 0xd2f03574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f0357e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f03582, 34 },
    { 0x6203583, 66 },
    { 0xb4103582, 33 },
    { 0x9403585, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51103589, 23 },
    { 0x85e0358a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a03592, 33 },
    { 0x9503592, 61 },
    { 0x61f03594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30503597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa680359c, 55 },
    { 0x6d90359d, 52 },
    { 0x3530359e, 23 },
    { 0xc6e0359f, 61 },
    { 0x4480359c, 67 },
    { 0, 0 },
    { 0x61035a2, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78035a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1035ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363035b2, 76 },
    { 0x3d6035b3, 22 },
    { 0x18e035b3, 90 },
    { 0xa47035b2, 23 },
    { 0x753035b6, 34 },
    { 0xa0c035b5, 33 },
    { 0xa4a035b6, 17 },
    { 0xd03035b9, 98 },
    { 0xde5035b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9035bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x880035c0, 95 },
    { 0, 0 },
    { 0x951035c2, 23 },
    { 0x17c035c2, 76 },
    { 0xe8e035c2, 43 },
    { 0, 0 },
    { 0x91b035c6, 61 },
    { 0, 0 },
    { 0x943035c8, 66 },
    { 0x907035c9, 99 },
    { 0xb67035c9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27035ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685035d2, 95 },
    { 0, 0 },
    { 0xcda035d4, 23 },
    { 0x119035d5, 55 },
    { 0x3b3035d4, 98 },
    { 0x491035d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132035db, 43 },
    { 0xb07035dc, 98 },
    { 0x2c7035dd, 90 },
    { 0x5b035de, 34 },
    { 0, 0 },
    { 0x5a035e0, 43 },
    { 0x55d035e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be035e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3035ec, 95 },
    { 0x55b035ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7035f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1035f7, 43 },
    { 0, 0 },
    { 0xdd035f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0035fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x747035ff, 95 },
    { 0x59303600, 98 },
    { 0, 0 },
    { 0x1e603602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea03605, 90 },
    { 0, 0 },
    { 0xa1403607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590360c, 98 },
    { 0x620360d, 61 },
    { 0x1570360e, 33 },
    { 0x8060360f, 90 },
    { 0, 0 },
    { 0xa3903611, 98 },
    { 0x16e03612, 61 },
    { 0xc6e03613, 33 },
    { 0x93903614, 33 },
    { 0xc1f03612, 33 },
    { 0, 0 },
    { 0x3e03617, 34 },
    { 0x3a103618, 87 },
    { 0xd4503619, 23 },
    { 0x3df03617, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c0361d, 90 },
    { 0x9990361e, 65 },
    { 0xf710361e, 33 },
    { 0x3750361e, 33 },
    { 0xb0d03621, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61003628, 61 },
    { 0, 0 },
    { 0x3790362a, 95 },
    { 0, 0 },
    { 0xd620362c, 90 },
    { 0xeb20362d, 53 },
    { 0x9900362e, 80 },
    { 0xf150362e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e03632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f03637, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc0363a, 95 },
    { 0, 0 },
    { 0x4b10363c, 98 },
    { 0xc6a0363d, 81 },
    { 0xb340363e, 46 },
    { 0, 0 },
    { 0x9fe03640, 67 },
    { 0x99803641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46303645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd090364a, 91 },
    { 0x97c0364a, 98 },
    { 0, 0 },
    { 0xe7f0364d, 90 },
    { 0xe430364e, 78 },
    { 0xc30364f, 65 },
    { 0xb4403650, 65 },
    { 0xa4403650, 33 },
    { 0x76903652, 43 },
    { 0x32803652, 61 },
    { 0x3d403652, 66 },
    { 0xfbc03650, 77 },
    { 0x72a03655, 46 },
    { 0x7d803650, 90 },
    { 0xcbb0364d, 100 },
    { 0x58303654, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60365d, 34 },
    { 0, 0 },
    { 0xae90365f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f03665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc03669, 80 },
    { 0, 0 },
    { 0xcc30366b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5803670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa503674, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf03679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64303685, 90 },
    { 0, 0 },
    { 0x82303687, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8020368a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be03690, 98 },
    { 0xd7b03691, 76 },
    { 0x8dc03692, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1803695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410369f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414036a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8036b0, 76 },
    { 0x90c036b1, 93 },
    { 0x760036b2, 90 },
    { 0, 0 },
    { 0x632036b4, 33 },
    { 0, 0 },
    { 0x4d3036b6, 65 },
    { 0xacf036b7, 95 },
    { 0x4e7036b8, 90 },
    { 0x260036b8, 78 },
    { 0, 0 },
    { 0xb8a036bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1036c0, 76 },
    { 0x7f9036c1, 58 },
    { 0x5ab036c2, 91 },
    { 0, 0 },
    { 0xe01036c4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5036c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be036cb, 78 },
    { 0x2e1036cb, 91 },
    { 0xcb9036cb, 94 },
    { 0, 0 },
    { 0x12e036cf, 69 },
    { 0x5d8036cf, 33 },
    { 0, 0 },
    { 0x2aa036d2, 90 },
    { 0xbe7036d3, 97 },
    { 0x7b1036d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380036dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9036e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6036e5, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536036e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc036ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0036f2, 17 },
    { 0x111036f3, 43 },
    { 0, 0 },
    { 0xe16036f5, 34 },
    { 0, 0 },
    { 0xcd1036f7, 34 },
    { 0, 0 },
    { 0xbae036f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b03702, 78 },
    { 0, 0 },
    { 0x5bf03704, 98 },
    { 0xc2f03705, 77 },
    { 0x36a03704, 95 },
    { 0x55a03707, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e10370b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7700370e, 46 },
    { 0, 0 },
    { 0x7f303710, 95 },
    { 0x22703710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb703715, 76 },
    { 0xa7203716, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22903719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b0371c, 90 },
    { 0, 0 },
    { 0x5bc0371e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66903729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840372d, 98 },
    { 0x7290372e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a03732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f03735, 61 },
    { 0, 0 },
    { 0x96003737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db0373c, 95 },
    { 0x23b0373d, 46 },
    { 0, 0 },
    { 0x8490373f, 90 },
    { 0, 0 },
    { 0x57703741, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c03747, 55 },
    { 0xb3e03748, 34 },
    { 0xaa303749, 98 },
    { 0x42a03749, 95 },
    { 0x52f0374b, 95 },
    { 0x28f03747, 33 },
    { 0, 0 },
    { 0x3910374e, 78 },
    { 0, 0 },
    { 0xb2a03750, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae03758, 55 },
    { 0x3ab03758, 76 },
    { 0x1503758, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b303760, 90 },
    { 0, 0 },
    { 0xbef03762, 33 },
    { 0x95103763, 46 },
    { 0x79903763, 77 },
    { 0xa5e03764, 99 },
    { 0, 0 },
    { 0xe1e03767, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0376b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6703770, 76 },
    { 0x42903771, 98 },
    { 0, 0 },
    { 0x2a303773, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670377b, 46 },
    { 0x11e0377c, 75 },
    { 0x5fd0377d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f03781, 93 },
    { 0xa903782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0403786, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10378b, 90 },
    { 0xd090378b, 34 },
    { 0, 0 },
    { 0x1f90378e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92403795, 61 },
    { 0x3ac03796, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e03799, 33 },
    { 0, 0 },
    { 0x450379b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231037a2, 66 },
    { 0, 0 },
    { 0xfff037a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb037ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a037b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38037b6, 55 },
    { 0, 0 },
    { 0x82f037b8, 95 },
    { 0, 0 },
    { 0x442037ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611037c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7037c7, 55 },
    { 0x1e9037c8, 90 },
    { 0, 0 },
    { 0xefe037ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc037ce, 76 },
    { 0xa037cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037d2, 95 },
    { 0, 0 },
    { 0x550037d4, 88 },
    { 0x9fe037d4, 61 },
    { 0x85c037d5, 98 },
    { 0x1c8037d7, 61 },
    { 0xcd9037d8, 80 },
    { 0x946037d9, 61 },
    { 0x9c0037d9, 34 },
    { 0x94e037d5, 95 },
    { 0, 0 },
    { 0xe38037dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732037e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e037ec, 90 },
    { 0x2f4037ed, 90 },
    { 0x8f037ec, 68 },
    { 0x9c0037ef, 94 },
    { 0x871037ef, 34 },
    { 0x823037ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51037f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x331037f9, 98 },
    { 0x9f1037f9, 78 },
    { 0xf3a037fa, 55 },
    { 0xdec037fc, 61 },
    { 0x6e3037fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f003803, 33 },
    { 0, 0 },
    { 0xf1303805, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b0380d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb103811, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d03815, 95 },
    { 0x92b03816, 33 },
    { 0x3e203817, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a0381a, 33 },
    { 0, 0 },
    { 0xbf50381c, 61 },
    { 0x5950381d, 100 },
    { 0xe0d0381e, 46 },
    { 0, 0 },
    { 0x6fe03820, 65 },
    { 0, 0 },
    { 0xc8803822, 61 },
    { 0, 0 },
    { 0x3a303824, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15503829, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4470382d, 90 },
    { 0, 0 },
    { 0xafd0382f, 23 },
    { 0xb2b0382f, 43 },
    { 0xcc303831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30403834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c03837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c03840, 95 },
    { 0, 0 },
    { 0x3b303842, 90 },
    { 0x8a703843, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61203849, 23 },
    { 0xbe703849, 90 },
    { 0x8bd0384b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93403857, 90 },
    { 0xb0d03858, 99 },
    { 0x3fd03859, 23 },
    { 0x1d403858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9350385d, 76 },
    { 0, 0 },
    { 0x3540385f, 23 },
    { 0xf9803860, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0003868, 33 },
    { 0, 0 },
    { 0x640386a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd0386e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e03874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xef303877, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0387a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a00387d, 93 },
    { 0x4600387d, 95 },
    { 0, 0 },
    { 0x30703880, 90 },
    { 0, 0 },
    { 0xf5603882, 33 },
    { 0xc3c03883, 98 },
    { 0x82603884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00388b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc03890, 40 },
    { 0xdb603891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd03894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20389a, 90 },
    { 0xe0b0389b, 90 },
    { 0xd400389c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c038a0, 99 },
    { 0xa59038a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f038a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8038ab, 90 },
    { 0x4aa038ab, 99 },
    { 0, 0 },
    { 0x24038ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40038b2, 52 },
    { 0, 0 },
    { 0x38b038b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9038b8, 34 },
    { 0x3f3038b9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc038c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40038cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d038cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb038d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3038da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef038dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452038e8, 23 },
    { 0x4fc038e9, 95 },
    { 0x911038ea, 90 },
    { 0, 0 },
    { 0x9f4038ec, 95 },
    { 0x794038ec, 33 },
    { 0, 0 },
    { 0x171038ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c038f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87038f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c038fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da038ff, 77 },
    { 0, 0 },
    { 0x57803901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d03907, 40 },
    { 0, 0 },
    { 0x8e803909, 90 },
    { 0, 0 },
    { 0x9d20390b, 98 },
    { 0x7030390b, 36 },
    { 0x56a0390d, 100 },
    { 0x7200390d, 87 },
    { 0x3c70390f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49703912, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c403915, 55 },
    { 0, 0 },
    { 0x77103917, 23 },
    { 0x5c803917, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2240391d, 23 },
    { 0x7d00391e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d503922, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0392c, 75 },
    { 0xb1b0392d, 95 },
    { 0x92b0392d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2703933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff03938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d0393d, 87 },
    { 0xdda0393e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f03945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420394a, 61 },
    { 0x6460394b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0394e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98503952, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d03958, 95 },
    { 0xea403959, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc260395d, 34 },
    { 0, 0 },
    { 0x7cf0395f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80403962, 27 },
    { 0xcd03962, 33 },
    { 0xb7a03962, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e00396a, 33 },
    { 0x72f0396b, 84 },
    { 0x490396b, 90 },
    { 0xdff0396b, 69 },
    { 0xe940396e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8903971, 81 },
    { 0x64603971, 91 },
    { 0x5d303972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39803978, 98 },
    { 0x1503979, 85 },
    { 0, 0 },
    { 0x35d0397b, 78 },
    { 0xa6d0397c, 33 },
    { 0, 0 },
    { 0x5ae0397e, 95 },
    { 0x35e0397e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f03984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f03988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f0398b, 56 },
    { 0x2ad0398b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c0398f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5460399b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f00399f, 46 },
    { 0xec60399f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b039a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6039ae, 90 },
    { 0, 0 },
    { 0xafc039b0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19039b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87039bb, 90 },
    { 0xb78039bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3039c1, 76 },
    { 0xcb5039c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f039ce, 61 },
    { 0x35f039cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0039d3, 81 },
    { 0, 0 },
    { 0xc76039d5, 98 },
    { 0x143039d6, 43 },
    { 0, 0 },
    { 0xec039d8, 90 },
    { 0, 0 },
    { 0x798039da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de039de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e039e1, 90 },
    { 0xd07039e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6039e6, 61 },
    { 0, 0 },
    { 0x379039e8, 90 },
    { 0x4c4039e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6039ee, 55 },
    { 0x414039ef, 33 },
    { 0xbc8039f0, 76 },
    { 0xf18039f1, 76 },
    { 0xdc2039f1, 55 },
    { 0x914039f2, 23 },
    { 0xca3039f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85039f7, 88 },
    { 0xae3039f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61039ff, 99 },
    { 0xcc603a00, 84 },
    { 0x2e703a01, 69 },
    { 0, 0 },
    { 0x40703a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903a09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf03a0d, 23 },
    { 0x36f03a0e, 90 },
    { 0xb2003a0f, 95 },
    { 0xcd203a10, 91 },
    { 0x38e03a11, 76 },
    { 0xa4403a12, 65 },
    { 0x3ac03a13, 35 },
    { 0xe4f03a14, 40 },
    { 0x97703a0f, 95 },
    { 0, 0 },
    { 0x4803a17, 23 },
    { 0, 0 },
    { 0x68103a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c03a20, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3503a23, 84 },
    { 0x3c803a24, 93 },
    { 0x70903a25, 23 },
    { 0, 0 },
    { 0xad603a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c403a2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd03a30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f03a34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e303a37, 76 },
    { 0x1b503a38, 33 },
    { 0, 0 },
    { 0xc0a03a3a, 90 },
    { 0, 0 },
    { 0x75603a3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49203a3f, 98 },
    { 0xc3a03a40, 98 },
    { 0x1803a41, 80 },
    { 0, 0 },
    { 0xd3c03a43, 94 },
    { 0, 0 },
    { 0xfe003a45, 61 },
    { 0xc6903a46, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad03a4c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03a4f, 81 },
    { 0, 0 },
    { 0x41103a51, 76 },
    { 0x8b603a52, 34 },
    { 0x7f703a53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61703a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52703a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54003a61, 43 },
    { 0x66f03a62, 61 },
    { 0x4203a63, 87 },
    { 0x66503a62, 61 },
    { 0x34403a65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36903a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x89803a6c, 23 },
    { 0x23b03a6d, 95 },
    { 0, 0 },
    { 0xd5003a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c503a73, 76 },
    { 0xb2503a74, 90 },
    { 0xfbb03a74, 34 },
    { 0x8e703a76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6603a7f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da03a85, 84 },
    { 0, 0 },
    { 0x3d503a87, 98 },
    { 0x78903a87, 43 },
    { 0x23003a89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57403a8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a03a90, 33 },
    { 0xdd03a90, 61 },
    { 0xa803a92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e103a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72603a9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff03aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23403aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc03aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a203ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85403aba, 61 },
    { 0, 0 },
    { 0xd9903abc, 95 },
    { 0x8bf03abd, 90 },
    { 0x8d403abe, 93 },
    { 0, 0 },
    { 0x49e03ac0, 95 },
    { 0x24603ac1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d403ac8, 43 },
    { 0xe8e03ac9, 98 },
    { 0, 0 },
    { 0x1b603acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f803ad0, 46 },
    { 0x30c03ad0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce03ad5, 90 },
    { 0xf6303ad6, 61 },
    { 0xe7803ad7, 33 },
    { 0x36603ad8, 87 },
    { 0x7ed03ad7, 34 },
    { 0x8e503ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a703add, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7503ae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c03ae6, 88 },
    { 0xb1203ae7, 80 },
    { 0x26903ae6, 78 },
    { 0xe4803ae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d03aec, 98 },
    { 0x17e03aed, 33 },
    { 0, 0 },
    { 0xa5103aef, 90 },
    { 0xa7c03af0, 33 },
    { 0, 0 },
    { 0x24603af2, 33 },
    { 0xc8003af3, 95 },
    { 0x8bb03af4, 98 },
    { 0x63c03af5, 95 },
    { 0xc0b03af6, 61 },
    { 0xef103af6, 99 },
    { 0xe7003af8, 61 },
    { 0x59c03af9, 84 },
    { 0x79b03af9, 55 },
    { 0, 0 },
    { 0x7f503afc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30703b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a03b03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce603b0a, 91 },
    { 0xf9403b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f03b14, 99 },
    { 0xad503b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c03b19, 98 },
    { 0, 0 },
    { 0xf8e03b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b03b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a03b21, 78 },
    { 0, 0 },
    { 0x8b803b23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f03b26, 61 },
    { 0, 0 },
    { 0xff03b28, 98 },
    { 0xf7d03b28, 98 },
    { 0x6d903b2a, 98 },
    { 0x72903b29, 34 },
    { 0, 0 },
    { 0x4af03b2d, 33 },
    { 0x1303b2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab03b31, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a703b36, 55 },
    { 0x85203b37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31203b3a, 33 },
    { 0x39703b3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36403b40, 43 },
    { 0xcaa03b41, 23 },
    { 0xcbb03b41, 46 },
    { 0x5103b43, 83 },
    { 0xdd103b43, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52103b49, 61 },
    { 0x5c703b4a, 98 },
    { 0x64103b4a, 61 },
    { 0x7103b4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9903b50, 98 },
    { 0x13803b50, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f703b57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e03b5d, 90 },
    { 0, 0 },
    { 0xa8d03b5f, 90 },
    { 0xa0f03b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56803b65, 61 },
    { 0xa5003b66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc803b6a, 90 },
    { 0x20903b6b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb703b6e, 76 },
    { 0xfc03b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e03b77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f03b7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c03b7d, 76 },
    { 0xdbb03b7e, 23 },
    { 0xaa203b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f203b83, 46 },
    { 0xbe503b84, 90 },
    { 0xbfe03b83, 33 },
    { 0x67903b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc03b89, 98 },
    { 0, 0 },
    { 0xc2903b8b, 90 },
    { 0x1b603b8c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f03b91, 93 },
    { 0, 0 },
    { 0xfc203b93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb03b99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77503b9e, 77 },
    { 0, 0 },
    { 0x95503ba0, 76 },
    { 0x22b03ba1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c03ba5, 98 },
    { 0x71003ba6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8003bae, 99 },
    { 0xbec03baf, 46 },
    { 0x8b403bb0, 91 },
    { 0xee003bae, 23 },
    { 0x4f703bb0, 90 },
    { 0, 0 },
    { 0x52c03bb4, 34 },
    { 0xb6203bb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e03bbe, 95 },
    { 0xe2c03bbf, 90 },
    { 0x56c03bbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40303bc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae03bcc, 34 },
    { 0xf9403bcd, 61 },
    { 0, 0 },
    { 0x9e103bcf, 33 },
    { 0x3f403bd0, 61 },
    { 0xb0903bd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd003bd4, 55 },
    { 0x22a03bd5, 33 },
    { 0x98e03bd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a603bda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b103be2, 81 },
    { 0xf2103be2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d303be7, 52 },
    { 0xc2d03be8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0303bef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b303bf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9703bfd, 55 },
    { 0x13503bfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40003c02, 90 },
    { 0x70e03c02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73203c0c, 33 },
    { 0x8503c0d, 43 },
    { 0x10603c0d, 43 },
    { 0xffe03c0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8803c18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84803c1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f03c22, 69 },
    { 0, 0 },
    { 0xa2403c24, 98 },
    { 0, 0 },
    { 0xfb203c26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac03c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e03c2c, 81 },
    { 0x87103c2c, 90 },
    { 0x26503c2e, 23 },
    { 0xf2103c2c, 75 },
    { 0, 0 },
    { 0x7d103c31, 33 },
    { 0x3fa03c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c03c38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d03c3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e03c40, 95 },
    { 0x6603c40, 76 },
    { 0x90c03c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32603c48, 95 },
    { 0, 0 },
    { 0x12a03c4a, 55 },
    { 0x45403c4a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f603c4e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x12403c51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad03c54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c03c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4703c5b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x45203c5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7303c61, 85 },
    { 0x9cf03c61, 33 },
    { 0xd9d03c63, 43 },
    { 0x7503c63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c03c68, 95 },
    { 0x41703c69, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x26703c6c, 34 },
    { 0xb0103c6c, 95 },
    { 0, 0 },
    { 0x8fe03c6f, 93 },
    { 0x6d803c70, 33 },
    { 0xcd603c71, 43 },
    { 0x9bb03c72, 23 },
    { 0x28f03c71, 61 },
    { 0, 0 },
    { 0x9b603c75, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd03c7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1003c7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39503c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e03c87, 34 },
    { 0x50703c88, 85 },
    { 0x33c03c87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1103c8e, 69 },
    { 0x31a03c8f, 75 },
    { 0, 0 },
    { 0x76303c91, 67 },
    { 0xd8d03c92, 61 },
    { 0x87f03c93, 52 },
    { 0, 0 },
    { 0x75503c95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e03c9c, 55 },
    { 0, 0 },
    { 0xd0403c9e, 23 },
    { 0x8db03c9e, 100 },
    { 0x94f03ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5803ca6, 91 },
    { 0x23703ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e103cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d803cb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17103cb7, 33 },
    { 0, 0 },
    { 0x34b03cb9, 90 },
    { 0xeca03cba, 69 },
    { 0x9c203cb9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8503cbe, 93 },
    { 0x76903cbf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf803cc3, 91 },
    { 0, 0 },
    { 0xc6703cc5, 68 },
    { 0x21003cc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f903cca, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac903cce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa303cd5, 43 },
    { 0, 0 },
    { 0xc0403cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32703cda, 100 },
    { 0xc1603cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7603cdf, 33 },
    { 0, 0 },
    { 0xcf503ce1, 33 },
    { 0xdf603ce2, 99 },
    { 0x44903ce3, 90 },
    { 0xd8d03ce1, 98 },
    { 0x54303ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e503ce9, 46 },
    { 0x74603cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7703cef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca303cf3, 98 },
    { 0, 0 },
    { 0x31803cf5, 43 },
    { 0x57c03cf6, 23 },
    { 0xcfc03cf6, 98 },
    { 0xc4303cf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff03d00, 96 },
    { 0xbc403d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70203d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03d0b, 90 },
    { 0x1c103d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d03d0f, 61 },
    { 0xd8a03d10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98003d13, 95 },
    { 0xc2203d14, 90 },
    { 0xca03d15, 91 },
    { 0x29d03d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57403d1a, 90 },
    { 0, 0 },
    { 0x1e203d1c, 34 },
    { 0xe2103d1d, 61 },
    { 0x6b003d1d, 61 },
    { 0x77403d1f, 83 },
    { 0xe4703d20, 23 },
    { 0x4c703d1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a03d28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b03d2b, 99 },
    { 0x2a703d2c, 95 },
    { 0, 0 },
    { 0x1a803d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef903d34, 85 },
    { 0, 0 },
    { 0xd5403d36, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd03d39, 93 },
    { 0, 0 },
    { 0xb2c03d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43203d41, 95 },
    { 0x3b803d42, 90 },
    { 0xdc503d43, 99 },
    { 0xa0b03d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94803d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88803d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca503d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1603d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b03d5f, 46 },
    { 0x22203d5f, 61 },
    { 0x21503d60, 89 },
    { 0, 0 },
    { 0x30d03d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e03d66, 90 },
    { 0xb3203d67, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb903d70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6203d74, 23 },
    { 0x79d03d75, 98 },
    { 0x29903d75, 69 },
    { 0, 0 },
    { 0x15103d78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef03d7c, 43 },
    { 0xe7703d7d, 95 },
    { 0x1b603d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5003d86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f03d8c, 75 },
    { 0, 0 },
    { 0x77903d8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c03d91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b03d94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d03d97, 34 },
    { 0, 0 },
    { 0xeb103d99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca03d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2403da1, 55 },
    { 0x20403da2, 98 },
    { 0x10c03da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff03dab, 46 },
    { 0xe1803dab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa103db6, 90 },
    { 0, 0 },
    { 0x65203db8, 98 },
    { 0x66103db9, 98 },
    { 0xfbd03db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4303dbf, 33 },
    { 0x76f03dc0, 46 },
    { 0x96f03dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b03dc5, 23 },
    { 0xf5703dc6, 33 },
    { 0x5e003dc7, 46 },
    { 0x2bc03dc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da03dcb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c103dd4, 87 },
    { 0xdc003dd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a03dd8, 90 },
    { 0x35203dd8, 75 },
    { 0, 0 },
    { 0x31103ddb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6303de4, 93 },
    { 0, 0 },
    { 0xe4403de6, 33 },
    { 0x56303de7, 99 },
    { 0, 0 },
    { 0xa7b03de9, 46 },
    { 0x10403de9, 78 },
    { 0xe3503dea, 33 },
    { 0xa4d03deb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe903df3, 33 },
    { 0xe603df3, 43 },
    { 0, 0 },
    { 0xada03df6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e003dfa, 76 },
    { 0x7c903dfa, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0703dff, 61 },
    { 0x43903e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a03e03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68403e07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1603e0a, 45 },
    { 0x3fe03e0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2603e0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f403e12, 91 },
    { 0, 0 },
    { 0x6a203e14, 100 },
    { 0, 0 },
    { 0xba703e16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf03e1c, 98 },
    { 0, 0 },
    { 0x70103e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81103e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec703e2d, 33 },
    { 0, 0 },
    { 0x9df03e2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2303e36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d203e3a, 87 },
    { 0xa9d03e3a, 87 },
    { 0, 0 },
    { 0xf5f03e3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8803e42, 95 },
    { 0x10003e43, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb003e48, 90 },
    { 0, 0 },
    { 0x12b03e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee03e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf903e51, 66 },
    { 0x9b403e52, 100 },
    { 0x64a03e53, 52 },
    { 0xf0d03e52, 98 },
    { 0xe8703e52, 61 },
    { 0xe3503e54, 55 },
    { 0xdb203e57, 46 },
    { 0x16403e58, 23 },
    { 0xbf703e58, 34 },
    { 0xbfd03e5a, 76 },
    { 0xa4b03e57, 81 },
    { 0, 0 },
    { 0x90a03e5d, 33 },
    { 0xaf203e5e, 90 },
    { 0x54b03e5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b03e65, 33 },
    { 0x8f003e66, 66 },
    { 0x65c03e65, 33 },
    { 0x16103e68, 90 },
    { 0x53503e66, 69 },
    { 0x60903e6a, 52 },
    { 0x73303e67, 95 },
    { 0xf8103e65, 61 },
    { 0xe2603e6c, 61 },
    { 0x31203e66, 61 },
    { 0x34003e6f, 97 },
    { 0x26b03e70, 34 },
    { 0x6a703e71, 34 },
    { 0x4b403e71, 95 },
    { 0x69303e6c, 77 },
    { 0xe4903e74, 46 },
    { 0, 0 },
    { 0x99003e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8503e7a, 61 },
    { 0, 0 },
    { 0xcba03e7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c03e87, 45 },
    { 0, 0 },
    { 0xc6303e89, 74 },
    { 0xdca03e8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d03e8e, 34 },
    { 0, 0 },
    { 0xa6b03e90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3203e97, 91 },
    { 0x6b03e97, 88 },
    { 0x54903e98, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b03e9d, 8 },
    { 0x8cb03e9e, 61 },
    { 0, 0 },
    { 0x4603ea0, 17 },
    { 0, 0 },
    { 0x29003ea2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc903ea5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b03eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4203ead, 34 },
    { 0x7f903eae, 43 },
    { 0, 0 },
    { 0x54703eb0, 84 },
    { 0, 0 },
    { 0xb0203eb2, 61 },
    { 0x1b303eb2, 66 },
    { 0, 0 },
    { 0x4f003eb5, 77 },
    { 0xeb903eb5, 95 },
    { 0x86a03eb7, 34 },
    { 0x31b03eb7, 78 },
    { 0x69103eb9, 34 },
    { 0x1a603eb8, 95 },
    { 0xf7303ebb, 33 },
    { 0xa7a03eb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65003ec3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd903ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b03edb, 95 },
    { 0x55d03edc, 23 },
    { 0x92e03edd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad03ee1, 93 },
    { 0, 0 },
    { 0x7ee03ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b03ee9, 84 },
    { 0xe4203ee9, 91 },
    { 0xb5803eeb, 95 },
    { 0xd9d03eec, 33 },
    { 0x86f03eed, 34 },
    { 0x1b03eec, 100 },
    { 0x9d603eef, 76 },
    { 0x77d03eee, 98 },
    { 0xb5503eec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe203ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e003ef8, 99 },
    { 0x12403ef9, 88 },
    { 0xc4103efa, 23 },
    { 0xcd603efa, 23 },
    { 0x5f403efc, 98 },
    { 0xe0003efc, 90 },
    { 0x70f03efc, 90 },
    { 0x5f903efe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de03f07, 43 },
    { 0x5ed03f08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f03f11, 52 },
    { 0xf2203f12, 33 },
    { 0x32403f13, 55 },
    { 0x3f13, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce03f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c03f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba03f25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da03f28, 90 },
    { 0xe1903f29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0803f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb03f34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a603f37, 90 },
    { 0, 0 },
    { 0xf0403f39, 33 },
    { 0xfeb03f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea903f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab03f45, 61 },
    { 0x67903f46, 91 },
    { 0xb2c03f46, 33 },
    { 0, 0 },
    { 0x75603f49, 33 },
    { 0xb7c03f4a, 23 },
    { 0xdb203f49, 27 },
    { 0xc5803f4c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e303f4f, 33 },
    { 0x5cb03f50, 43 },
    { 0xd9803f50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa903f55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed03f58, 84 },
    { 0xecb03f59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f03f62, 95 },
    { 0, 0 },
    { 0x4d003f64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b803f68, 66 },
    { 0x58803f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55903f6c, 88 },
    { 0xc3403f6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b03f72, 76 },
    { 0xe7f03f73, 90 },
    { 0x1b603f74, 90 },
    { 0, 0 },
    { 0xa4703f76, 90 },
    { 0xae103f77, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa03f7d, 23 },
    { 0x68b03f7e, 85 },
    { 0x4d003f7f, 90 },
    { 0xc3b03f80, 67 },
    { 0xbc003f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7803f84, 23 },
    { 0x54903f85, 55 },
    { 0x42b03f86, 43 },
    { 0xff003f85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1003f8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0803f8d, 67 },
    { 0, 0 },
    { 0x62903f8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c803f92, 99 },
    { 0, 0 },
    { 0xd8803f94, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f703f99, 90 },
    { 0, 0 },
    { 0x57003f9b, 61 },
    { 0, 0 },
    { 0xa2303f9d, 95 },
    { 0xd6403f9e, 98 },
    { 0x50f03f9d, 99 },
    { 0x14303fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9303fa4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8903fab, 33 },
    { 0xc7703fab, 43 },
    { 0x97803fac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x54403fb0, 98 },
    { 0x82703fb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc03fbb, 33 },
    { 0, 0 },
    { 0xa4e03fbd, 61 },
    { 0x3cb03fbe, 90 },
    { 0x9b403fbe, 33 },
    { 0x69f03fc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef03fcb, 61 },
    { 0xd0d03fcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f03fd0, 76 },
    { 0xad103fd0, 90 },
    { 0, 0 },
    { 0xdf103fd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e03fdf, 98 },
    { 0x48a03fe0, 87 },
    { 0x18c03fe1, 93 },
    { 0x3c603fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac503fe7, 61 },
    { 0x4b203fe8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a03feb, 90 },
    { 0, 0 },
    { 0x3a703fed, 84 },
    { 0x8ae03fee, 95 },
    { 0xae403fee, 90 },
    { 0xcb603fed, 95 },
    { 0xe0203fef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27103ff8, 95 },
    { 0xec303ff9, 33 },
    { 0, 0 },
    { 0x64103ffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89103fff, 98 },
    { 0, 0 },
    { 0x31904001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3504008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd310400d, 23 },
    { 0, 0 },
    { 0x8d20400f, 33 },
    { 0, 0 },
    { 0x27b04011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c704015, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c04018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd90401f, 90 },
    { 0xd1804020, 61 },
    { 0x6ac04021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904024, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50304029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa730402e, 33 },
    { 0x1df0402f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf104032, 55 },
    { 0xa4004033, 61 },
    { 0xed704034, 46 },
    { 0, 0 },
    { 0x57304036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69704039, 40 },
    { 0x1850403a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0403e, 61 },
    { 0, 0 },
    { 0xc3b04040, 80 },
    { 0x1fc04041, 33 },
    { 0x9f604041, 90 },
    { 0xf4204043, 90 },
    { 0xf0904044, 91 },
    { 0, 0 },
    { 0xfa204046, 55 },
    { 0xd1304047, 95 },
    { 0xd9004048, 23 },
    { 0x95404047, 84 },
    { 0xe604048, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010404f, 98 },
    { 0, 0 },
    { 0xb004051, 81 },
    { 0xeff04052, 76 },
    { 0, 0 },
    { 0x2f104054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f20405d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2304068, 33 },
    { 0x54504069, 73 },
    { 0xcbc04069, 90 },
    { 0x1dc04069, 95 },
    { 0x1fe04068, 76 },
    { 0x4900406a, 76 },
    { 0x7d70406e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59004071, 90 },
    { 0xb1e04071, 95 },
    { 0, 0 },
    { 0x11304074, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd150407a, 88 },
    { 0, 0 },
    { 0xacb0407c, 24 },
    { 0x7f0407d, 53 },
    { 0x6d50407c, 90 },
    { 0xa460407f, 90 },
    { 0xa5104080, 90 },
    { 0xf0804080, 95 },
    { 0xbf104082, 52 },
    { 0x60504081, 78 },
    { 0x1f20407f, 76 },
    { 0x65204085, 98 },
    { 0x6e04086, 55 },
    { 0xbef04087, 46 },
    { 0x34104088, 95 },
    { 0x19004088, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020408d, 61 },
    { 0, 0 },
    { 0x7d60408f, 33 },
    { 0, 0 },
    { 0xa5604091, 91 },
    { 0, 0 },
    { 0x92804093, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d04097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0409a, 99 },
    { 0x32b0409b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb040a5, 91 },
    { 0xb7d040a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x467040a9, 65 },
    { 0x5f8040aa, 33 },
    { 0xda9040ab, 33 },
    { 0x6d6040ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec040af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38040b4, 61 },
    { 0x695040b4, 61 },
    { 0xba8040b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c040b9, 53 },
    { 0xc39040b9, 55 },
    { 0, 0 },
    { 0xd4a040bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b040c2, 90 },
    { 0, 0 },
    { 0x754040c4, 101 },
    { 0x452040c5, 98 },
    { 0x4db040c6, 33 },
    { 0x137040c4, 95 },
    { 0x149040c7, 61 },
    { 0x5b5040c9, 33 },
    { 0xafa040ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489040d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3040d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x166040d7, 34 },
    { 0x3ce040d7, 75 },
    { 0, 0 },
    { 0x252040da, 33 },
    { 0xefb040db, 33 },
    { 0x90d040db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58040e1, 61 },
    { 0x950040e2, 93 },
    { 0xb17040e3, 66 },
    { 0x8fb040e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf040e7, 55 },
    { 0x9c8040e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227040ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891040f0, 61 },
    { 0, 0 },
    { 0x6ab040f2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f040fc, 66 },
    { 0x42d040fd, 23 },
    { 0xba9040fe, 78 },
    { 0x58b040fe, 33 },
    { 0, 0 },
    { 0x17404101, 17 },
    { 0xf0304102, 91 },
    { 0xf5104102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf804106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27504109, 95 },
    { 0xc750410a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80410e, 87 },
    { 0xf350410f, 93 },
    { 0, 0 },
    { 0x4f704111, 78 },
    { 0xde904112, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f04118, 84 },
    { 0x23704119, 98 },
    { 0xba304118, 90 },
    { 0x2f20411b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04120, 33 },
    { 0x3d004121, 75 },
    { 0x8c104122, 23 },
    { 0xe0904122, 95 },
    { 0x14e04123, 76 },
    { 0, 0 },
    { 0xce404126, 66 },
    { 0x55804127, 90 },
    { 0x66104127, 33 },
    { 0, 0 },
    { 0x9440412a, 61 },
    { 0x9d10412b, 96 },
    { 0x4760412c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc504130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82304133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9904136, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c04139, 33 },
    { 0xa130413a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf0413d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f604141, 77 },
    { 0x56004141, 64 },
    { 0xa004141, 90 },
    { 0x2c504141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0414a, 90 },
    { 0, 0 },
    { 0xa2f0414c, 85 },
    { 0x55a0414d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf04153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb104157, 99 },
    { 0x34204157, 81 },
    { 0, 0 },
    { 0xaaf0415a, 85 },
    { 0, 0 },
    { 0xbcf0415c, 90 },
    { 0, 0 },
    { 0xe130415e, 99 },
    { 0xdcd0415e, 33 },
    { 0x4380415f, 90 },
    { 0, 0 },
    { 0x25f04162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f304165, 85 },
    { 0xcb304166, 95 },
    { 0xdaa04167, 90 },
    { 0x69104165, 100 },
    { 0x94a04167, 43 },
    { 0xb7f0416a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890417b, 55 },
    { 0, 0 },
    { 0xe0417d, 76 },
    { 0, 0 },
    { 0x73a0417f, 100 },
    { 0x29404180, 91 },
    { 0xc2b0417f, 61 },
    { 0x2a30417f, 61 },
    { 0x82504183, 98 },
    { 0xfa504180, 33 },
    { 0xcba04185, 33 },
    { 0, 0 },
    { 0xa2004187, 34 },
    { 0, 0 },
    { 0x3e304189, 23 },
    { 0x8ab0418a, 84 },
    { 0, 0 },
    { 0x8090418c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43004192, 66 },
    { 0x49204193, 68 },
    { 0x93604193, 34 },
    { 0xc5a04192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e04199, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390419c, 90 },
    { 0x51d0419d, 98 },
    { 0, 0 },
    { 0x2b60419f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70041a2, 23 },
    { 0x55041a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8041a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b041af, 76 },
    { 0, 0 },
    { 0x717041b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45041b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d041b9, 61 },
    { 0xc93041ba, 90 },
    { 0, 0 },
    { 0x8cd041bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8041c0, 55 },
    { 0, 0 },
    { 0x11041c2, 87 },
    { 0xab2041c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62041c6, 100 },
    { 0x6dc041c7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb041ce, 68 },
    { 0xb6e041cf, 61 },
    { 0, 0 },
    { 0x82f041d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7041d5, 61 },
    { 0xcb7041d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa041e0, 33 },
    { 0xdf9041e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d041e4, 33 },
    { 0x20f041e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631041ee, 61 },
    { 0, 0 },
    { 0x6c3041f0, 84 },
    { 0x985041f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640041fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b504203, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe04208, 76 },
    { 0xb3804209, 33 },
    { 0, 0 },
    { 0x14c0420b, 98 },
    { 0, 0 },
    { 0x6790420d, 17 },
    { 0x9d70420e, 77 },
    { 0, 0 },
    { 0x5ff04210, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e204217, 90 },
    { 0x13504218, 33 },
    { 0, 0 },
    { 0xcc20421a, 33 },
    { 0, 0 },
    { 0x9740421c, 23 },
    { 0xea30421d, 52 },
    { 0x8050421e, 100 },
    { 0, 0 },
    { 0x39804220, 95 },
    { 0, 0 },
    { 0x91f04222, 89 },
    { 0, 0 },
    { 0x22b04224, 95 },
    { 0x8cb04225, 23 },
    { 0, 0 },
    { 0x67d04227, 88 },
    { 0x14d04228, 95 },
    { 0, 0 },
    { 0x5150422a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6004233, 34 },
    { 0x4e404234, 93 },
    { 0, 0 },
    { 0x60a04236, 90 },
    { 0x8f604237, 95 },
    { 0xe2304236, 76 },
    { 0x22404237, 55 },
    { 0x3d70423a, 90 },
    { 0x3470423a, 61 },
    { 0x83e04238, 24 },
    { 0, 0 },
    { 0x9020423e, 88 },
    { 0x1c40423e, 90 },
    { 0, 0 },
    { 0x58d04241, 43 },
    { 0x4bf04241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45104245, 90 },
    { 0x8e904245, 98 },
    { 0, 0 },
    { 0x42c04248, 90 },
    { 0x16904249, 76 },
    { 0x35c04249, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd20424f, 98 },
    { 0, 0 },
    { 0xf504251, 52 },
    { 0, 0 },
    { 0x35204253, 61 },
    { 0x61f04254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0425b, 76 },
    { 0x75a0425c, 98 },
    { 0xbb10425c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc04260, 98 },
    { 0, 0 },
    { 0xd1004262, 43 },
    { 0, 0 },
    { 0xf3304264, 23 },
    { 0xbfc04265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1810426d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26904272, 62 },
    { 0x32104273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de04278, 17 },
    { 0x75d04279, 96 },
    { 0x18d0427a, 55 },
    { 0x47c0427b, 61 },
    { 0xfd70427a, 90 },
    { 0, 0 },
    { 0xbea0427e, 91 },
    { 0xd2e0427e, 98 },
    { 0x5bd0427e, 33 },
    { 0xb800427f, 90 },
    { 0x89a0427e, 95 },
    { 0xc8004283, 66 },
    { 0x78b0427f, 75 },
    { 0x10504280, 90 },
    { 0x8ba04286, 23 },
    { 0, 0 },
    { 0xe7804288, 52 },
    { 0x69a04289, 61 },
    { 0x4d80428a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950428d, 90 },
    { 0xf620428d, 61 },
    { 0, 0 },
    { 0x68304290, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2404295, 100 },
    { 0x92204295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db04299, 34 },
    { 0x3ab0429a, 66 },
    { 0, 0 },
    { 0xdf70429c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb042a3, 76 },
    { 0, 0 },
    { 0x339042a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e042ac, 95 },
    { 0xc7d042ad, 93 },
    { 0x79d042ad, 61 },
    { 0xc75042ac, 66 },
    { 0, 0 },
    { 0xbcf042b1, 46 },
    { 0, 0 },
    { 0x24a042b3, 17 },
    { 0xf0b042b4, 33 },
    { 0xe27042b3, 95 },
    { 0, 0 },
    { 0xade042b7, 90 },
    { 0xe6e042b8, 46 },
    { 0, 0 },
    { 0x928042ba, 34 },
    { 0xb60042ba, 99 },
    { 0, 0 },
    { 0x655042bd, 98 },
    { 0x6be042bd, 17 },
    { 0, 0 },
    { 0x4c6042c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785042c5, 81 },
    { 0x52b042c5, 76 },
    { 0xf45042c7, 84 },
    { 0x12c042c8, 98 },
    { 0x1ec042c7, 99 },
    { 0, 0 },
    { 0x2df042cb, 34 },
    { 0xfb6042cb, 55 },
    { 0xf75042cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d042d2, 90 },
    { 0x782042d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f042d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8042e0, 61 },
    { 0x6fa042e0, 98 },
    { 0xd22042e2, 95 },
    { 0x166042e2, 55 },
    { 0xcfe042e4, 17 },
    { 0x1a6042e5, 33 },
    { 0x959042e4, 76 },
    { 0x59c042e7, 90 },
    { 0xe3c042e8, 33 },
    { 0x7b2042e5, 61 },
    { 0x462042e4, 55 },
    { 0, 0 },
    { 0xb96042ec, 68 },
    { 0, 0 },
    { 0xf47042ee, 52 },
    { 0xa95042ef, 91 },
    { 0x2a0042ef, 23 },
    { 0x733042f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f042f7, 23 },
    { 0, 0 },
    { 0xafe042f9, 61 },
    { 0x5b5042fa, 80 },
    { 0x1c3042f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8042ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb04304, 52 },
    { 0xfc604304, 61 },
    { 0xe0504306, 84 },
    { 0x48b04304, 33 },
    { 0xf7d04306, 46 },
    { 0xa4904309, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540430c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb04311, 100 },
    { 0xdb104312, 90 },
    { 0xadb04313, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda40431d, 66 },
    { 0x3250431e, 23 },
    { 0xca90431f, 23 },
    { 0, 0 },
    { 0x4a704321, 34 },
    { 0, 0 },
    { 0x2e404323, 95 },
    { 0xba404323, 23 },
    { 0, 0 },
    { 0x81804326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290432a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90804330, 33 },
    { 0xc6504331, 56 },
    { 0xe3a04331, 68 },
    { 0xbdb04331, 33 },
    { 0x1e404334, 61 },
    { 0x44704334, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1004338, 61 },
    { 0x1a04339, 95 },
    { 0xeb304338, 96 },
    { 0x24c04339, 78 },
    { 0x91a0433c, 61 },
    { 0xaba0433d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7504343, 17 },
    { 0xac704344, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5304347, 91 },
    { 0xa8d04348, 90 },
    { 0x9bb04348, 76 },
    { 0, 0 },
    { 0x4c40434b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a04352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac60435e, 23 },
    { 0x7580435f, 23 },
    { 0x53d0435e, 99 },
    { 0xfbf04361, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a04366, 40 },
    { 0, 0 },
    { 0x25604368, 33 },
    { 0xe8f04369, 23 },
    { 0x42304368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba20436d, 34 },
    { 0xba40436e, 76 },
    { 0xe150436f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3904373, 65 },
    { 0, 0 },
    { 0x26e04375, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7900437b, 33 },
    { 0x570437c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b104387, 95 },
    { 0x58604388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4030438e, 61 },
    { 0xacd0438f, 95 },
    { 0xbf004390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b04394, 95 },
    { 0xfca04395, 46 },
    { 0, 0 },
    { 0xb2404397, 98 },
    { 0xe0c04398, 75 },
    { 0xa9904399, 91 },
    { 0x62d04399, 32 },
    { 0x3b40439b, 52 },
    { 0, 0 },
    { 0xf2d0439d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1043a2, 90 },
    { 0x8c2043a3, 61 },
    { 0, 0 },
    { 0x7b1043a5, 59 },
    { 0xaf6043a5, 65 },
    { 0xacb043a7, 61 },
    { 0, 0 },
    { 0x244043a9, 43 },
    { 0x4b6043aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a043af, 98 },
    { 0x68b043b0, 75 },
    { 0xeee043b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92043b6, 23 },
    { 0x78a043b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26043be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860043c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce043cf, 76 },
    { 0, 0 },
    { 0x9c8043d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4043d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80043da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2043df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914043e3, 90 },
    { 0xbb043e4, 99 },
    { 0xc38043e3, 90 },
    { 0xd16043e4, 76 },
    { 0x7c8043e6, 85 },
    { 0, 0 },
    { 0xc8a043e9, 33 },
    { 0xdf7043e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512043ef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f043f4, 17 },
    { 0x781043f5, 98 },
    { 0, 0 },
    { 0x109043f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2043fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51104403, 40 },
    { 0xece04404, 99 },
    { 0xd9204405, 98 },
    { 0xab204406, 61 },
    { 0x30304403, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c70440a, 91 },
    { 0x4a90440a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba04414, 76 },
    { 0xf8904415, 61 },
    { 0x9fe04415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2750441b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc04422, 33 },
    { 0x34904423, 61 },
    { 0xdab04424, 90 },
    { 0x31404422, 90 },
    { 0, 0 },
    { 0xe6404427, 61 },
    { 0x7d404427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8770442b, 98 },
    { 0x4400442c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57204432, 98 },
    { 0x52b04433, 61 },
    { 0x4af04434, 66 },
    { 0xf9304432, 90 },
    { 0x41d04432, 90 },
    { 0x75504437, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a0443a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa020443d, 61 },
    { 0xe2e0443d, 95 },
    { 0x7e20443f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f04445, 23 },
    { 0xc5c04446, 23 },
    { 0x27304447, 81 },
    { 0x5ee04448, 46 },
    { 0xb3d04449, 66 },
    { 0xc900444a, 90 },
    { 0x5e504447, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b804458, 46 },
    { 0x90d04459, 33 },
    { 0x9150445a, 33 },
    { 0, 0 },
    { 0xc560445c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d04464, 90 },
    { 0x29504465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6004468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba40446c, 46 },
    { 0x6fa0446d, 87 },
    { 0, 0 },
    { 0xbdf0446f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39204474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c404477, 76 },
    { 0, 0 },
    { 0xce204479, 90 },
    { 0xa390447a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a0447e, 90 },
    { 0x79a0447f, 95 },
    { 0x59604480, 93 },
    { 0xd5c04481, 61 },
    { 0x3a004482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5504487, 93 },
    { 0, 0 },
    { 0x32904489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e0448c, 90 },
    { 0, 0 },
    { 0x7940448e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e04491, 33 },
    { 0xc1e04492, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16504496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0449c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40449f, 84 },
    { 0xc2a0449f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6044a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0044a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389044ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x933044af, 46 },
    { 0xc51044b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8044b3, 77 },
    { 0, 0 },
    { 0x43f044b5, 33 },
    { 0, 0 },
    { 0xff7044b7, 95 },
    { 0xe74044b7, 98 },
    { 0xe58044b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47044bc, 90 },
    { 0xc38044bc, 33 },
    { 0, 0 },
    { 0x3d6044bf, 61 },
    { 0, 0 },
    { 0x387044c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24044c4, 43 },
    { 0x682044c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310044cc, 36 },
    { 0x93e044cc, 36 },
    { 0x289044ce, 23 },
    { 0, 0 },
    { 0xe46044d0, 75 },
    { 0, 0 },
    { 0x4c1044d2, 61 },
    { 0, 0 },
    { 0xb2e044d4, 23 },
    { 0x499044d5, 90 },
    { 0xc6d044d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b044dd, 55 },
    { 0x415044de, 90 },
    { 0x359044dd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1044e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2044e6, 95 },
    { 0x7f044e7, 33 },
    { 0xdc9044e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6044ee, 61 },
    { 0x8c7044ef, 23 },
    { 0x2e2044ee, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68044f4, 95 },
    { 0xfbe044f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69044fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b404507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4900450c, 33 },
    { 0, 0 },
    { 0x8000450e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43204511, 81 },
    { 0x17504511, 90 },
    { 0xd5704511, 34 },
    { 0x97604514, 61 },
    { 0xa4f04515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c00451f, 33 },
    { 0, 0 },
    { 0x35604521, 77 },
    { 0, 0 },
    { 0x604523, 100 },
    { 0xf5d04523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c0452b, 97 },
    { 0x1440452c, 46 },
    { 0x1c0452c, 78 },
    { 0, 0 },
    { 0x2b20452f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c04532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f0453a, 76 },
    { 0, 0 },
    { 0xfaa0453c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35604543, 46 },
    { 0xddc04544, 87 },
    { 0x39804545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a10454c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57904550, 98 },
    { 0x82e04551, 95 },
    { 0x72304552, 76 },
    { 0x46c04551, 90 },
    { 0x51404551, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460455b, 90 },
    { 0, 0 },
    { 0x4ce0455d, 95 },
    { 0x1640455e, 40 },
    { 0, 0 },
    { 0xfe104560, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96404563, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81104566, 81 },
    { 0xf5b04567, 77 },
    { 0xcd404568, 66 },
    { 0xbc204568, 90 },
    { 0, 0 },
    { 0xec90456b, 61 },
    { 0x7370456c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff0456f, 90 },
    { 0x68d04570, 61 },
    { 0xdfb04571, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9704575, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c04578, 84 },
    { 0, 0 },
    { 0x28d0457a, 84 },
    { 0, 0 },
    { 0x1750457c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a04584, 43 },
    { 0xc9a04584, 61 },
    { 0x41004584, 76 },
    { 0xbc304586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00458a, 61 },
    { 0xf3a0458b, 53 },
    { 0xd0e0458c, 61 },
    { 0x3820458c, 46 },
    { 0x8f70458d, 61 },
    { 0x8380458d, 23 },
    { 0x7d30458c, 23 },
    { 0xa8404591, 98 },
    { 0x4540458c, 98 },
    { 0x2d30458e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0704597, 76 },
    { 0x92204598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80459c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0459f, 61 },
    { 0x8d7045a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a045a4, 67 },
    { 0xf8f045a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66045a9, 33 },
    { 0, 0 },
    { 0x17b045ab, 43 },
    { 0, 0 },
    { 0x950045ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1045b1, 93 },
    { 0x924045b1, 33 },
    { 0x3e2045b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1045bc, 48 },
    { 0, 0 },
    { 0x3e045be, 33 },
    { 0, 0 },
    { 0xc8045c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120045c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6045ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386045d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9045d3, 61 },
    { 0, 0 },
    { 0x269045d5, 97 },
    { 0x975045d6, 90 },
    { 0x22a045d7, 91 },
    { 0, 0 },
    { 0xeda045d9, 91 },
    { 0xcfd045da, 33 },
    { 0x468045da, 93 },
    { 0, 0 },
    { 0xb7e045dd, 43 },
    { 0x688045dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3045e5, 23 },
    { 0x7b0045e6, 95 },
    { 0, 0 },
    { 0x71c045e8, 76 },
    { 0xaec045e8, 61 },
    { 0xbef045ea, 93 },
    { 0x87c045ea, 55 },
    { 0xd9a045eb, 23 },
    { 0, 0 },
    { 0x854045ee, 95 },
    { 0xdfd045ef, 43 },
    { 0xf6d045f0, 61 },
    { 0xe10045ef, 23 },
    { 0, 0 },
    { 0xb60045f3, 33 },
    { 0xf9e045f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x315045f7, 61 },
    { 0x32b045f8, 23 },
    { 0x2e5045f9, 46 },
    { 0x37e045f9, 90 },
    { 0xad7045fb, 87 },
    { 0x1fd045fc, 52 },
    { 0xcee045fc, 66 },
    { 0xe2b045fc, 90 },
    { 0, 0 },
    { 0x94304600, 95 },
    { 0, 0 },
    { 0xd504602, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc904605, 34 },
    { 0, 0 },
    { 0x11704607, 90 },
    { 0xa2e04608, 98 },
    { 0xe2604609, 36 },
    { 0x12804607, 33 },
    { 0x6e0460b, 33 },
    { 0, 0 },
    { 0x4420460d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca104610, 90 },
    { 0xb3e04611, 55 },
    { 0x53b04612, 61 },
    { 0, 0 },
    { 0x86204614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76904617, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f20461d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c04620, 87 },
    { 0x7404621, 61 },
    { 0xa8804620, 55 },
    { 0x40804623, 66 },
    { 0, 0 },
    { 0xfc04625, 23 },
    { 0xe7c04625, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d0462b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe880462f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46504636, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c0463b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6860463e, 91 },
    { 0x7040463f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1604644, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0704648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a0464b, 90 },
    { 0x84c0464b, 61 },
    { 0xf890464d, 55 },
    { 0, 0 },
    { 0x3840464f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7504656, 90 },
    { 0, 0 },
    { 0x1d304658, 34 },
    { 0, 0 },
    { 0x6280465a, 96 },
    { 0x5480465b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c0465e, 33 },
    { 0x1240465e, 100 },
    { 0xc2b04660, 52 },
    { 0, 0 },
    { 0x8da04662, 31 },
    { 0xa1d04663, 61 },
    { 0, 0 },
    { 0x22904665, 92 },
    { 0x1ac04665, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4504669, 61 },
    { 0x19304669, 61 },
    { 0xff40466a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xef50466e, 76 },
    { 0x3900466f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff04673, 82 },
    { 0xd6e04674, 33 },
    { 0xd7304674, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8304678, 23 },
    { 0x38604679, 46 },
    { 0x2904679, 61 },
    { 0xe90467b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db04683, 33 },
    { 0xbe504683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0468a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b0468e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a04695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb04698, 23 },
    { 0x45204699, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09046a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19046a6, 23 },
    { 0, 0 },
    { 0xb2046a8, 75 },
    { 0x495046a9, 27 },
    { 0xf21046aa, 63 },
    { 0x272046ab, 33 },
    { 0x7c9046aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8046b0, 65 },
    { 0x77f046b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17046b5, 61 },
    { 0xb02046b6, 38 },
    { 0x394046b5, 95 },
    { 0xe60046b8, 93 },
    { 0, 0 },
    { 0x8d8046ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316046be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8046c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531046c6, 67 },
    { 0x683046c7, 90 },
    { 0xa5d046c8, 76 },
    { 0x566046c9, 33 },
    { 0x96d046c9, 90 },
    { 0x314046c6, 87 },
    { 0xa3d046cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e046d5, 61 },
    { 0x88b046d6, 91 },
    { 0x968046d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34046dc, 61 },
    { 0, 0 },
    { 0x4f046de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f046e4, 93 },
    { 0x394046e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc046e9, 33 },
    { 0x528046ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d046ee, 76 },
    { 0x182046ef, 23 },
    { 0x9b5046f0, 61 },
    { 0xda0046ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b046f7, 23 },
    { 0, 0 },
    { 0x4a046f9, 66 },
    { 0x349046fa, 98 },
    { 0x4f9046fb, 46 },
    { 0, 0 },
    { 0xfcb046fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d104708, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de0470c, 34 },
    { 0x35e0470c, 95 },
    { 0x57c0470e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5804715, 98 },
    { 0xa6704716, 55 },
    { 0x2304717, 81 },
    { 0x76904718, 76 },
    { 0xa4504719, 90 },
    { 0x5ee04717, 100 },
    { 0x85604716, 55 },
    { 0, 0 },
    { 0x6410471d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55804720, 66 },
    { 0x22e04721, 72 },
    { 0x40a04721, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5104725, 88 },
    { 0, 0 },
    { 0xc3a04727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc430472b, 98 },
    { 0x4d90472c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020472f, 90 },
    { 0x86f04730, 55 },
    { 0x33b04730, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc04737, 98 },
    { 0x1c904738, 98 },
    { 0x2cd04739, 34 },
    { 0x43c04737, 95 },
    { 0, 0 },
    { 0xa9a0473c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b04740, 90 },
    { 0x8bf04740, 99 },
    { 0xebc04741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89304746, 98 },
    { 0xd5004747, 55 },
    { 0xde004747, 90 },
    { 0xf3204749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe750474f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9004753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf04757, 61 },
    { 0, 0 },
    { 0xf9804759, 90 },
    { 0x67404759, 99 },
    { 0xbd30475b, 76 },
    { 0x6e00475c, 33 },
    { 0x1600475a, 55 },
    { 0xbe30475e, 66 },
    { 0x42b0475e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d04767, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe030476a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0476e, 95 },
    { 0x3ee0476e, 93 },
    { 0, 0 },
    { 0x1bb04771, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x78504774, 98 },
    { 0xfa804774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96704779, 98 },
    { 0, 0 },
    { 0x8120477b, 87 },
    { 0xd8e0477c, 23 },
    { 0x3b90477d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da04782, 17 },
    { 0xe9404782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b0478d, 90 },
    { 0xa800478e, 23 },
    { 0x5e80478f, 76 },
    { 0xa704790, 34 },
    { 0x38c04791, 33 },
    { 0x18504792, 95 },
    { 0, 0 },
    { 0xcd804794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e04798, 43 },
    { 0xc4704799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9430479e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a047a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5047a9, 33 },
    { 0x22d047a9, 99 },
    { 0xbd9047ab, 100 },
    { 0xc23047ac, 67 },
    { 0x11047ad, 100 },
    { 0xfc9047ae, 88 },
    { 0xa1b047ad, 90 },
    { 0x7cc047b0, 98 },
    { 0x9dd047ae, 90 },
    { 0x37f047b2, 33 },
    { 0x595047b3, 90 },
    { 0x302047b0, 34 },
    { 0x967047b3, 90 },
    { 0xd23047ae, 76 },
    { 0x4f0047b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9047bf, 66 },
    { 0, 0 },
    { 0xbb1047c1, 90 },
    { 0, 0 },
    { 0x195047c3, 52 },
    { 0x56047c4, 83 },
    { 0x4b0047c4, 95 },
    { 0x774047c6, 61 },
    { 0x889047c7, 61 },
    { 0xfc5047c8, 61 },
    { 0x34c047c8, 100 },
    { 0xe0a047c8, 90 },
    { 0, 0 },
    { 0x8c6047cc, 77 },
    { 0, 0 },
    { 0xa19047ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1047d2, 34 },
    { 0x356047d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b047d7, 66 },
    { 0xc7b047d8, 77 },
    { 0x50f047d9, 90 },
    { 0xcf4047d9, 43 },
    { 0, 0 },
    { 0x18c047dc, 49 },
    { 0xa1047dd, 95 },
    { 0x3e9047de, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x500047e1, 33 },
    { 0x17c047e2, 98 },
    { 0x4a0047e3, 61 },
    { 0, 0 },
    { 0xc45047e5, 43 },
    { 0, 0 },
    { 0xd68047e7, 33 },
    { 0x6fa047e8, 90 },
    { 0x31f047e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b047ec, 67 },
    { 0xa6f047ed, 23 },
    { 0xfd0047ee, 98 },
    { 0x28047ef, 60 },
    { 0x827047f0, 91 },
    { 0, 0 },
    { 0xf8a047f2, 33 },
    { 0, 0 },
    { 0x41a047f4, 90 },
    { 0x2e9047f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3047f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b04801, 80 },
    { 0x84204801, 95 },
    { 0x33c04803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0480b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1500480f, 81 },
    { 0, 0 },
    { 0xbb904811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a04815, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14204818, 98 },
    { 0, 0 },
    { 0xf4d0481a, 91 },
    { 0, 0 },
    { 0xeca0481c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd04823, 95 },
    { 0xdfc04823, 85 },
    { 0xd9804824, 34 },
    { 0, 0 },
    { 0x3a604827, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b50482e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e04832, 33 },
    { 0x9e604832, 61 },
    { 0, 0 },
    { 0x2bc04835, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea04838, 95 },
    { 0, 0 },
    { 0x6cd0483a, 93 },
    { 0x43b0483b, 95 },
    { 0, 0 },
    { 0xad0483d, 55 },
    { 0xd450483e, 76 },
    { 0x3720483f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904847, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb604850, 40 },
    { 0x17f04851, 87 },
    { 0xdc504852, 61 },
    { 0x33e04852, 33 },
    { 0x45904853, 90 },
    { 0, 0 },
    { 0xec404856, 90 },
    { 0x59d04857, 91 },
    { 0x24904856, 61 },
    { 0, 0 },
    { 0xdf0485a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0485e, 53 },
    { 0, 0 },
    { 0x48304860, 23 },
    { 0xaf104860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a04866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9804869, 52 },
    { 0x7c90486a, 90 },
    { 0x84f0486b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640486e, 40 },
    { 0, 0 },
    { 0x58b04870, 100 },
    { 0x34004871, 90 },
    { 0x99c04871, 23 },
    { 0xe6704873, 61 },
    { 0, 0 },
    { 0x40304875, 84 },
    { 0, 0 },
    { 0xf5404877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5650487b, 61 },
    { 0x6bf0487c, 23 },
    { 0xf2f0487d, 23 },
    { 0xb280487e, 69 },
    { 0x4700487f, 99 },
    { 0xebc04880, 90 },
    { 0x2820487e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc104884, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93104887, 34 },
    { 0, 0 },
    { 0x53804889, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b0488c, 88 },
    { 0x7d20488d, 37 },
    { 0, 0 },
    { 0x1e20488f, 90 },
    { 0x82e0488f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd504895, 56 },
    { 0x71c04896, 46 },
    { 0x40704897, 93 },
    { 0x35e04898, 55 },
    { 0xdd404896, 95 },
    { 0, 0 },
    { 0x92a0489b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb430489e, 23 },
    { 0, 0 },
    { 0xdd048a0, 98 },
    { 0xa07048a1, 55 },
    { 0xf08048a2, 61 },
    { 0x27d048a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3048a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb048ad, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f048b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca048b4, 61 },
    { 0xbd5048b5, 65 },
    { 0x99a048b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82048bd, 34 },
    { 0x928048bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x784048c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc048c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121048d3, 96 },
    { 0xe18048d3, 96 },
    { 0xfce048d5, 90 },
    { 0xe45048d3, 23 },
    { 0, 0 },
    { 0x189048d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a048e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40048e3, 76 },
    { 0x6ce048e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5048e7, 40 },
    { 0, 0 },
    { 0x7b0048e9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f048ef, 93 },
    { 0, 0 },
    { 0x209048f1, 61 },
    { 0xaa4048f2, 23 },
    { 0, 0 },
    { 0xa81048f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e048f9, 68 },
    { 0, 0 },
    { 0xe83048fb, 90 },
    { 0xc7e048fc, 95 },
    { 0, 0 },
    { 0x4b6048fe, 94 },
    { 0, 0 },
    { 0x6c604900, 93 },
    { 0x3104900, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31804906, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9680490b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70490f, 78 },
    { 0xef704910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45104914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25304918, 33 },
    { 0xed804919, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57304924, 90 },
    { 0, 0 },
    { 0x45c04926, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e30492b, 33 },
    { 0xd100492c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f60492f, 98 },
    { 0xcca04930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18204934, 84 },
    { 0xa5304935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d30493a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e04942, 33 },
    { 0x65e04943, 66 },
    { 0xc5704944, 90 },
    { 0x5e904945, 76 },
    { 0xe6b04944, 52 },
    { 0xa1804947, 90 },
    { 0x7604944, 33 },
    { 0x8404942, 23 },
    { 0, 0 },
    { 0x5ba0494b, 34 },
    { 0x30d0494c, 48 },
    { 0x58f0494c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e504951, 61 },
    { 0xb9e04951, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70204956, 43 },
    { 0, 0 },
    { 0xf4404958, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d04965, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f04969, 33 },
    { 0x9940496a, 33 },
    { 0x62d0496b, 33 },
    { 0xf5c0496b, 23 },
    { 0, 0 },
    { 0x7400496e, 77 },
    { 0xd390496f, 98 },
    { 0xb490496e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c104977, 33 },
    { 0x73e04978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0497b, 46 },
    { 0x74d0497b, 95 },
    { 0x72c0497c, 90 },
    { 0xa2a0497e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27204985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb00498c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e04993, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0804998, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b0499e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738049a1, 33 },
    { 0x36a049a2, 95 },
    { 0x412049a3, 61 },
    { 0xe3c049a4, 33 },
    { 0xa2a049a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12049a9, 34 },
    { 0, 0 },
    { 0xcb4049ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a049b2, 36 },
    { 0xb6d049b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4049b8, 33 },
    { 0x946049b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f049bd, 90 },
    { 0xadf049be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59049c1, 23 },
    { 0xa86049c2, 33 },
    { 0, 0 },
    { 0x45d049c4, 46 },
    { 0xff5049c5, 98 },
    { 0x6b7049c4, 76 },
    { 0, 0 },
    { 0x264049c8, 91 },
    { 0, 0 },
    { 0xbb7049ca, 99 },
    { 0, 0 },
    { 0x33c049cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73049d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x504049d5, 61 },
    { 0, 0 },
    { 0x604049d7, 23 },
    { 0, 0 },
    { 0xf83049d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8049de, 98 },
    { 0x1d5049df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2049e4, 90 },
    { 0x48d049e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592049ea, 76 },
    { 0x183049eb, 90 },
    { 0x96e049eb, 95 },
    { 0x56d049ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689049f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35049f7, 43 },
    { 0xdd7049f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7049fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e104a02, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf04a05, 90 },
    { 0x14904a06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f204a09, 95 },
    { 0x3e204a09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae04a10, 100 },
    { 0xced04a11, 93 },
    { 0x4af04a11, 34 },
    { 0xa1f04a13, 93 },
    { 0xc0904a13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e104a18, 66 },
    { 0, 0 },
    { 0xd104a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89904a1d, 61 },
    { 0, 0 },
    { 0xf4904a1f, 85 },
    { 0x36204a1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504a28, 87 },
    { 0, 0 },
    { 0xdc504a2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d104a32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56104a35, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c04a3a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd04a3d, 98 },
    { 0x4db04a3e, 98 },
    { 0xf804a3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf04a47, 33 },
    { 0xa1504a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e04a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3404a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75604a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8404a5d, 100 },
    { 0x48e04a5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34004a61, 90 },
    { 0x3d504a61, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35004a67, 33 },
    { 0xadd04a68, 95 },
    { 0x38404a68, 33 },
    { 0x6a04a68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69904a6d, 87 },
    { 0x32304a6e, 61 },
    { 0x16604a6f, 91 },
    { 0x5d004a70, 66 },
    { 0, 0 },
    { 0xa7004a72, 43 },
    { 0, 0 },
    { 0x46b04a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd04a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5804a7d, 90 },
    { 0x7e04a7d, 95 },
    { 0xccc04a7f, 91 },
    { 0, 0 },
    { 0xef904a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0304a86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d204a8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b504a92, 88 },
    { 0x55704a92, 88 },
    { 0xd2204a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa404a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20104a9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed304aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd04aaa, 43 },
    { 0, 0 },
    { 0x2a704aac, 75 },
    { 0x66e04aad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde404ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89004ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82204acd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8304ad0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e04ad4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e04ad7, 98 },
    { 0xded04ad8, 76 },
    { 0xa7d04ad7, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a04ae0, 77 },
    { 0, 0 },
    { 0xd7b04ae2, 76 },
    { 0x1f804ae3, 52 },
    { 0xb0c04ae2, 69 },
    { 0xe904ae4, 61 },
    { 0, 0 },
    { 0x9a404ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9204aec, 95 },
    { 0, 0 },
    { 0x36204aee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b04af2, 85 },
    { 0x8504af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c04af6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd04af9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a404afc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88104aff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e04b04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8004b07, 99 },
    { 0, 0 },
    { 0xb7e04b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25204b0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad404b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f04b16, 55 },
    { 0x63104b17, 90 },
    { 0xb2104b17, 23 },
    { 0xa7204b16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb604b1c, 61 },
    { 0x5ea04b1c, 91 },
    { 0, 0 },
    { 0x43604b1f, 90 },
    { 0, 0 },
    { 0xeb504b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82404b26, 37 },
    { 0x87504b27, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x25904b2a, 37 },
    { 0x67304b2b, 43 },
    { 0x6a804b2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90304b32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe404b36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa04b39, 33 },
    { 0xf2a04b3a, 33 },
    { 0xb3504b3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e04b3e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e04b42, 61 },
    { 0x27204b42, 90 },
    { 0x33604b44, 90 },
    { 0, 0 },
    { 0x2b204b46, 34 },
    { 0xa1204b47, 98 },
    { 0xa2404b48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca04b4b, 78 },
    { 0x5ee04b4b, 55 },
    { 0xb2504b4c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f04b53, 99 },
    { 0x7c204b54, 43 },
    { 0x2ca04b53, 61 },
    { 0xf8d04b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb004b59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a504b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d04b5f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2304b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2504b69, 95 },
    { 0, 0 },
    { 0xe2404b6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4304b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc104b75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a04b7c, 64 },
    { 0x1b704b7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf04b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b04b8a, 90 },
    { 0xe7404b8b, 23 },
    { 0xfb604b8c, 61 },
    { 0x64c04b8c, 55 },
    { 0, 0 },
    { 0x54d04b8f, 93 },
    { 0x28304b8f, 40 },
    { 0, 0 },
    { 0x42a04b92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c104b95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x15804b98, 61 },
    { 0x34904b98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c04b9d, 95 },
    { 0, 0 },
    { 0x3f704b9f, 33 },
    { 0x90904ba0, 76 },
    { 0, 0 },
    { 0x39d04ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b204ba5, 93 },
    { 0x27f04ba6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b04baa, 90 },
    { 0xe9c04bab, 95 },
    { 0x30604baa, 98 },
    { 0x5e04bab, 34 },
    { 0xb7d04bae, 33 },
    { 0, 0 },
    { 0x14904bb0, 90 },
    { 0x95504bb0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde804bb4, 98 },
    { 0, 0 },
    { 0x5e804bb6, 23 },
    { 0xc5004bb6, 33 },
    { 0, 0 },
    { 0x75104bb9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24104bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52104bc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97804bc7, 90 },
    { 0x42d04bc8, 98 },
    { 0, 0 },
    { 0x25a04bca, 34 },
    { 0xa6c04bcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb404bd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb804bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73904bda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f04be1, 61 },
    { 0xa8104be2, 98 },
    { 0x4904be3, 99 },
    { 0x28304be2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67204be9, 98 },
    { 0x6204bea, 98 },
    { 0x1e104beb, 76 },
    { 0x5f704bec, 90 },
    { 0xa5804bed, 88 },
    { 0, 0 },
    { 0x73d04bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f04bfa, 98 },
    { 0, 0 },
    { 0x9af04bfc, 46 },
    { 0xf8f04bfd, 33 },
    { 0x9bd04bfd, 95 },
    { 0, 0 },
    { 0xade04c00, 92 },
    { 0, 0 },
    { 0x9fa04c02, 90 },
    { 0xce204c02, 43 },
    { 0, 0 },
    { 0xf5e04c05, 23 },
    { 0x3f504c06, 39 },
    { 0, 0 },
    { 0x35404c08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82504c0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95204c11, 55 },
    { 0xb5304c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28204c15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6004c19, 33 },
    { 0x91f04c1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4304c22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4104c26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6504c2f, 76 },
    { 0x67704c2f, 33 },
    { 0xb0204c31, 76 },
    { 0xc0104c30, 34 },
    { 0x67304c2f, 33 },
    { 0, 0 },
    { 0xe1004c35, 33 },
    { 0x84a04c35, 33 },
    { 0, 0 },
    { 0x28404c38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b104c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74104c40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3904c47, 33 },
    { 0xd8e04c48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51604c4d, 76 },
    { 0xbf904c4e, 90 },
    { 0x2ef04c4f, 52 },
    { 0xb2b04c50, 61 },
    { 0x15904c4d, 93 },
    { 0xcc904c4f, 95 },
    { 0xc1a04c53, 88 },
    { 0x44604c53, 55 },
    { 0x9be04c55, 90 },
    { 0x84804c4f, 55 },
    { 0x97204c57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8604c5b, 98 },
    { 0, 0 },
    { 0x93404c5d, 67 },
    { 0x44b04c5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b04c62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec04c66, 55 },
    { 0, 0 },
    { 0x2e704c68, 61 },
    { 0x69704c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59804c6c, 95 },
    { 0x8d604c6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a04c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6704c77, 46 },
    { 0xe2904c77, 75 },
    { 0, 0 },
    { 0x74f04c7a, 33 },
    { 0x8004c7a, 76 },
    { 0x2e504c7a, 67 },
    { 0xef604c7c, 17 },
    { 0x5ba04c7b, 46 },
    { 0x15004c7a, 95 },
    { 0x82404c80, 33 },
    { 0x9fd04c80, 33 },
    { 0xa4b04c82, 98 },
    { 0x6b504c7d, 95 },
    { 0x9a004c84, 85 },
    { 0, 0 },
    { 0x52304c86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f04c89, 98 },
    { 0xf204c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1304c8d, 98 },
    { 0, 0 },
    { 0x7e704c8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5804c93, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f204c96, 98 },
    { 0, 0 },
    { 0x10804c98, 23 },
    { 0x7c604c98, 95 },
    { 0xef704c9a, 90 },
    { 0x49e04c9b, 65 },
    { 0, 0 },
    { 0xfa704c9d, 23 },
    { 0x66f04c9e, 76 },
    { 0, 0 },
    { 0x2c004ca0, 23 },
    { 0xef104ca0, 23 },
    { 0x70f04ca1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8204caa, 61 },
    { 0xa3304caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec04cae, 23 },
    { 0, 0 },
    { 0xfa004cb0, 23 },
    { 0x17304cb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d04cb4, 61 },
    { 0xad904cb5, 88 },
    { 0, 0 },
    { 0xe5104cb7, 93 },
    { 0xc6e04cb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b04cbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee04cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5204cc5, 52 },
    { 0, 0 },
    { 0xad104cc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef04cca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b04ccf, 61 },
    { 0, 0 },
    { 0x21704cd1, 98 },
    { 0, 0 },
    { 0xcd104cd3, 65 },
    { 0x77804cd4, 23 },
    { 0x89e04cd3, 90 },
    { 0xd0c04cd6, 66 },
    { 0xb4704cd6, 23 },
    { 0, 0 },
    { 0xaff04cd9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x77204cdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe04ce1, 77 },
    { 0, 0 },
    { 0x3f604ce3, 49 },
    { 0x2ed04ce3, 23 },
    { 0, 0 },
    { 0x23a04ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea04cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d04cf7, 43 },
    { 0xddd04cf7, 95 },
    { 0x28604cf9, 98 },
    { 0xbe504cf9, 34 },
    { 0x45f04cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e704cfe, 34 },
    { 0x2ea04cff, 99 },
    { 0xf1704cfe, 98 },
    { 0x5504d01, 23 },
    { 0, 0 },
    { 0xb7004d03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e04d09, 76 },
    { 0x23904d09, 98 },
    { 0x9ab04d09, 34 },
    { 0x67704d0c, 90 },
    { 0x16e04d0c, 52 },
    { 0xfd604d0b, 95 },
    { 0x66404d0f, 33 },
    { 0x7cb04d0b, 90 },
    { 0xc4404d11, 95 },
    { 0x25f04d11, 23 },
    { 0x86804d11, 65 },
    { 0xad504d0c, 85 },
    { 0, 0 },
    { 0x56c04d16, 95 },
    { 0xf5e04d17, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a04d1c, 90 },
    { 0xe5504d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a04d20, 98 },
    { 0x6ac04d21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de04d29, 93 },
    { 0, 0 },
    { 0x3004d2b, 61 },
    { 0xcb804d2c, 90 },
    { 0, 0 },
    { 0x32e04d2e, 95 },
    { 0, 0 },
    { 0x4ad04d30, 23 },
    { 0x1e004d31, 23 },
    { 0x85e04d32, 55 },
    { 0, 0 },
    { 0xdcb04d34, 33 },
    { 0xb3104d35, 90 },
    { 0x40104d36, 75 },
    { 0x12404d35, 90 },
    { 0, 0 },
    { 0x75b04d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e04d3c, 98 },
    { 0xc804d3d, 33 },
    { 0xc7604d3d, 98 },
    { 0x26b04d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7904d43, 95 },
    { 0, 0 },
    { 0xfcb04d45, 33 },
    { 0x74804d46, 61 },
    { 0x5c604d47, 66 },
    { 0x3f204d47, 61 },
    { 0, 0 },
    { 0x1fc04d4a, 23 },
    { 0xc7904d4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7604d51, 33 },
    { 0xa5204d51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc04d57, 61 },
    { 0x28004d58, 34 },
    { 0, 0 },
    { 0x90004d5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa04d5e, 100 },
    { 0xce304d5f, 91 },
    { 0, 0 },
    { 0xa5a04d61, 46 },
    { 0x18f04d62, 98 },
    { 0, 0 },
    { 0xeda04d64, 98 },
    { 0x98a04d65, 90 },
    { 0x4604d66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b004d69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee04d6c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x11104d6f, 90 },
    { 0, 0 },
    { 0xc7604d71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x67204d74, 100 },
    { 0x14904d75, 81 },
    { 0x72904d74, 95 },
    { 0xe7204d77, 100 },
    { 0xa8704d75, 95 },
    { 0x67b04d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d404d7f, 34 },
    { 0x46904d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33604d83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a04d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b04d8b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10804d8e, 23 },
    { 0xd0d04d8e, 90 },
    { 0x46e04d90, 90 },
    { 0xd5b04d91, 68 },
    { 0x93304d91, 69 },
    { 0xe1304d92, 52 },
    { 0xc0204d94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5704d9b, 95 },
    { 0x74d04d9c, 95 },
    { 0, 0 },
    { 0x2b804d9e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb04da3, 91 },
    { 0x12204da4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9904da8, 90 },
    { 0x58304da8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef04daf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf04db2, 61 },
    { 0xdbe04db3, 33 },
    { 0xa0504db4, 17 },
    { 0x81304db2, 90 },
    { 0x40004db2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10004db9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57904dbc, 85 },
    { 0, 0 },
    { 0xc9904dbe, 95 },
    { 0xad904dbf, 33 },
    { 0x3a504dc0, 98 },
    { 0xeb204dc1, 65 },
    { 0, 0 },
    { 0x41904dc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7704dc7, 90 },
    { 0, 0 },
    { 0x2ea04dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a04dce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac04dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d04dde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f04de1, 33 },
    { 0x9a804de1, 98 },
    { 0xa2904de3, 23 },
    { 0x1cf04de4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda04dea, 61 },
    { 0x5c304dea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c604df0, 95 },
    { 0x92104df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28504df9, 95 },
    { 0, 0 },
    { 0xd1c04dfb, 66 },
    { 0x4e604dfc, 85 },
    { 0xd3c04dfd, 90 },
    { 0x82a04dfe, 91 },
    { 0x7ff04dff, 23 },
    { 0x33a04e00, 61 },
    { 0x9604e01, 61 },
    { 0x6db04e02, 76 },
    { 0x95304dfe, 34 },
    { 0xcd104e04, 76 },
    { 0xc2704e05, 76 },
    { 0xfca04e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9204e0c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c04e13, 55 },
    { 0x1be04e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d04e17, 85 },
    { 0xc2e04e18, 90 },
    { 0x32704e19, 55 },
    { 0x5904e1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50104e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92004e24, 33 },
    { 0x94504e24, 61 },
    { 0x6ca04e26, 33 },
    { 0x24604e27, 88 },
    { 0xac304e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f204e2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e04e30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a04e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93504e40, 90 },
    { 0x51504e41, 33 },
    { 0x24804e42, 90 },
    { 0xeb904e42, 33 },
    { 0xede04e44, 76 },
    { 0x5a504e44, 55 },
    { 0x5f304e45, 65 },
    { 0x68704e46, 90 },
    { 0xef304e48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e104e4c, 33 },
    { 0xebe04e4d, 88 },
    { 0x2a904e4c, 90 },
    { 0x71f04e4f, 34 },
    { 0xa8204e4c, 81 },
    { 0xaa104e50, 87 },
    { 0x94c04e4e, 46 },
    { 0x84804e53, 23 },
    { 0x2cd04e4e, 43 },
    { 0xfed04e55, 61 },
    { 0x19e04e4c, 23 },
    { 0x70604e4e, 65 },
    { 0, 0 },
    { 0x30c04e59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf04e5d, 46 },
    { 0x7304e5e, 99 },
    { 0xb7d04e5d, 23 },
    { 0x17404e60, 33 },
    { 0x3df04e61, 34 },
    { 0x67b04e62, 33 },
    { 0xb2d04e62, 61 },
    { 0xea04e63, 84 },
    { 0x54f04e63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce204e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec104e71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d04e77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd404e7e, 77 },
    { 0xe8e04e7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce04e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a04e89, 94 },
    { 0xb1504e89, 46 },
    { 0xfd004e89, 98 },
    { 0, 0 },
    { 0x8a704e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d204e94, 34 },
    { 0, 0 },
    { 0x21304e96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80204e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78204ea0, 95 },
    { 0xaa804ea0, 77 },
    { 0xe6f04ea0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef004eaa, 40 },
    { 0x84c04eaa, 76 },
    { 0xa5704eac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8004eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef304eb3, 23 },
    { 0xae804eb4, 61 },
    { 0xa1504eb4, 41 },
    { 0, 0 },
    { 0x72f04eb7, 46 },
    { 0, 0 },
    { 0x86504eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c04ec1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15904ec7, 74 },
    { 0, 0 },
    { 0xcde04ec9, 33 },
    { 0, 0 },
    { 0x7fb04ecb, 78 },
    { 0, 0 },
    { 0xab204ecd, 52 },
    { 0, 0 },
    { 0x8c504ecf, 33 },
    { 0, 0 },
    { 0x73a04ed1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8804ed8, 84 },
    { 0x5604ed9, 84 },
    { 0x32304eda, 53 },
    { 0, 0 },
    { 0x89a04edc, 23 },
    { 0xc1004edd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba004ee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8604ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c04ef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d04efc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78304f00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a04f05, 61 },
    { 0x15104f06, 81 },
    { 0x6c04f07, 67 },
    { 0x5e204f08, 43 },
    { 0xa2604f07, 34 },
    { 0x2c04f05, 23 },
    { 0, 0 },
    { 0x77e04f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84904f0f, 90 },
    { 0, 0 },
    { 0xb1904f11, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc04f1b, 61 },
    { 0, 0 },
    { 0x92104f1d, 61 },
    { 0x72904f1e, 61 },
    { 0xeea04f1f, 52 },
    { 0x11004f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af04f25, 76 },
    { 0, 0 },
    { 0x22104f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62404f2c, 90 },
    { 0x83f04f2d, 55 },
    { 0x41f04f2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99204f32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59404f35, 34 },
    { 0x4e404f35, 52 },
    { 0xe04f36, 33 },
    { 0, 0 },
    { 0x49504f39, 95 },
    { 0x31204f39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6004f3f, 17 },
    { 0x4df04f40, 33 },
    { 0x88d04f41, 66 },
    { 0xb8c04f42, 90 },
    { 0xc8804f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74804f46, 43 },
    { 0x45304f47, 90 },
    { 0, 0 },
    { 0xba904f49, 61 },
    { 0, 0 },
    { 0xafc04f4b, 77 },
    { 0, 0 },
    { 0xf6c04f4d, 98 },
    { 0xf0104f4e, 93 },
    { 0x14f04f4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef04f53, 23 },
    { 0xf4a04f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa04f60, 78 },
    { 0x3c104f61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5604f66, 92 },
    { 0, 0 },
    { 0xb9204f68, 61 },
    { 0, 0 },
    { 0x8aa04f6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804f6f, 76 },
    { 0xeef04f6f, 91 },
    { 0x79204f71, 95 },
    { 0x42304f72, 43 },
    { 0x64d04f73, 95 },
    { 0x30a04f74, 66 },
    { 0x17b04f75, 95 },
    { 0xffe04f75, 23 },
    { 0xfe204f73, 87 },
    { 0x23404f70, 35 },
    { 0xe6b04f79, 33 },
    { 0xa9b04f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b04f7e, 75 },
    { 0xc2b04f7f, 61 },
    { 0x56004f7f, 90 },
    { 0x81204f81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe04f84, 43 },
    { 0x3cb04f84, 33 },
    { 0x5cb04f86, 93 },
    { 0, 0 },
    { 0xfd004f88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe04f8f, 93 },
    { 0x75f04f90, 23 },
    { 0, 0 },
    { 0x8e204f92, 95 },
    { 0x2de04f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f004f96, 43 },
    { 0, 0 },
    { 0xafd04f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc104f9b, 33 },
    { 0xcec04f9c, 34 },
    { 0xd2604f9d, 91 },
    { 0, 0 },
    { 0xa9b04f9f, 34 },
    { 0xc7f04fa0, 33 },
    { 0x2ff04fa1, 23 },
    { 0xf5c04fa2, 90 },
    { 0x10204fa3, 43 },
    { 0xc9504fa0, 80 },
    { 0x83e04fa5, 69 },
    { 0, 0 },
    { 0xe5204fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a04faa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x61904fad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda404fb0, 33 },
    { 0, 0 },
    { 0xf04fb2, 90 },
    { 0x8aa04fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7604fb7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9404fbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d04fc0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e04fc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45304fca, 95 },
    { 0, 0 },
    { 0x72f04fcc, 100 },
    { 0x12504fcc, 85 },
    { 0, 0 },
    { 0x6c304fcf, 61 },
    { 0, 0 },
    { 0xf7104fd1, 90 },
    { 0xa4a04fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da04fda, 61 },
    { 0x3cf04fdb, 81 },
    { 0x20504fdc, 55 },
    { 0x48704fdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf04fe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3704fe4, 23 },
    { 0x5dc04fe4, 98 },
    { 0x5a404fe6, 33 },
    { 0x10504fe7, 90 },
    { 0xe8204fe6, 95 },
    { 0, 0 },
    { 0x1e204fea, 95 },
    { 0xa2d04feb, 43 },
    { 0x4fb04feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36104fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae04ff3, 90 },
    { 0x9fc04ff4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e04ff8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16904ffb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x63604ffe, 84 },
    { 0, 0 },
    { 0xddf05000, 66 },
    { 0x72605000, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9505004, 33 },
    { 0xd7b05005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc05009, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b0500d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a505012, 43 },
    { 0x98f05013, 90 },
    { 0xd1705014, 55 },
    { 0xfbf05012, 43 },
    { 0xa7105016, 80 },
    { 0xc8e05014, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb00501c, 61 },
    { 0x9be0501d, 23 },
    { 0, 0 },
    { 0x38f0501f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x605027, 87 },
    { 0x47005027, 52 },
    { 0xed105029, 91 },
    { 0xb4e05027, 23 },
    { 0xeea0502b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7860502f, 90 },
    { 0x40605030, 95 },
    { 0, 0 },
    { 0xc3b05032, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd05041, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38105048, 76 },
    { 0xe7e05049, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0504d, 65 },
    { 0x7060504d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33705052, 23 },
    { 0xd4405053, 76 },
    { 0x7d005054, 95 },
    { 0xcd305053, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0505a, 61 },
    { 0x1970505b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0505f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x83905062, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2520506a, 5 },
    { 0, 0 },
    { 0x1640506c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c905071, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57305076, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a90507c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf730507f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c05082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a05085, 93 },
    { 0xe6f05085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec05089, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad0508d, 90 },
    { 0x670508e, 90 },
    { 0x2ae0508f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d705093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd805096, 76 },
    { 0xadd05097, 90 },
    { 0, 0 },
    { 0x4b405099, 95 },
    { 0, 0 },
    { 0xeac0509b, 90 },
    { 0, 0 },
    { 0xe3a0509d, 98 },
    { 0x44c0509e, 61 },
    { 0xc830509f, 95 },
    { 0x3670509f, 33 },
    { 0x63f0509f, 33 },
    { 0xfcf050a2, 33 },
    { 0x4f6050a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2050ab, 98 },
    { 0, 0 },
    { 0x15e050ad, 55 },
    { 0xcd6050ad, 95 },
    { 0x690050af, 81 },
    { 0x349050b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17050b3, 55 },
    { 0xcb050b4, 90 },
    { 0x3d6050b4, 65 },
    { 0xc5f050b6, 55 },
    { 0xf57050b7, 55 },
    { 0x3fb050b8, 76 },
    { 0x73a050b6, 100 },
    { 0x141050ba, 61 },
    { 0x369050bb, 90 },
    { 0xade050ba, 34 },
    { 0xd3d050b8, 33 },
    { 0xf7050b3, 33 },
    { 0xd80050bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3050c3, 98 },
    { 0x583050c4, 36 },
    { 0, 0 },
    { 0xba1050c6, 61 },
    { 0x54b050c7, 93 },
    { 0xf18050c8, 61 },
    { 0x23d050c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe050d0, 33 },
    { 0xd6a050d1, 95 },
    { 0xbb6050d1, 95 },
    { 0x3f2050d3, 95 },
    { 0x1d0050d3, 90 },
    { 0x8f7050d4, 99 },
    { 0x762050d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6050de, 76 },
    { 0x51a050df, 76 },
    { 0x32050df, 46 },
    { 0x1ed050e1, 23 },
    { 0x11e050e1, 90 },
    { 0x775050e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b050e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9050e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801050ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc050f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x424050f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98050f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65050ff, 23 },
    { 0x95105100, 76 },
    { 0x42a05101, 33 },
    { 0xb8305102, 61 },
    { 0xc0005102, 61 },
    { 0x4c405102, 94 },
    { 0xd4f05105, 33 },
    { 0xe3505106, 55 },
    { 0xb9b05103, 55 },
    { 0, 0 },
    { 0xcf505109, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd170510c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef0510f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62705112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2605116, 33 },
    { 0, 0 },
    { 0x9d005118, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390511b, 84 },
    { 0x5250511c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6005120, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf05123, 66 },
    { 0xcc705124, 61 },
    { 0x29c05124, 90 },
    { 0xeb705125, 17 },
    { 0x14c05124, 96 },
    { 0x89505128, 98 },
    { 0xc4205128, 23 },
    { 0, 0 },
    { 0xe5d0512b, 66 },
    { 0xfa30512c, 61 },
    { 0x8ce0512d, 81 },
    { 0, 0 },
    { 0xfc30512f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32205135, 95 },
    { 0xdd705136, 34 },
    { 0xe9e05137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b0513b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b805145, 61 },
    { 0xa2b05146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0514a, 23 },
    { 0x29e0514b, 43 },
    { 0x9d00514c, 90 },
    { 0x4b50514d, 43 },
    { 0, 0 },
    { 0x3a80514f, 33 },
    { 0xf5305150, 76 },
    { 0, 0 },
    { 0xcbc05152, 23 },
    { 0x5d305153, 76 },
    { 0, 0 },
    { 0xd9605155, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef20515b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3550515e, 61 },
    { 0xf5f0515f, 33 },
    { 0xf530515f, 90 },
    { 0xaa005161, 61 },
    { 0xafa05161, 95 },
    { 0x34305161, 33 },
    { 0x7fb0515e, 100 },
    { 0x2f905165, 95 },
    { 0x3100515e, 95 },
    { 0, 0 },
    { 0xad005168, 55 },
    { 0x8e805169, 77 },
    { 0, 0 },
    { 0x9630516b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5405172, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad05175, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9230517a, 61 },
    { 0x51f0517b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0605180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b905184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc805187, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf90518c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a0519b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0051a2, 98 },
    { 0x80b051a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a051a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188051af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8051b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf051be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f051c3, 34 },
    { 0x31c051c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8051cb, 46 },
    { 0x901051cb, 76 },
    { 0x3ec051cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9051d0, 46 },
    { 0xb18051d1, 90 },
    { 0, 0 },
    { 0x43e051d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb051d7, 90 },
    { 0x502051d8, 55 },
    { 0xfda051d8, 61 },
    { 0x6bb051d9, 85 },
    { 0x8ba051db, 34 },
    { 0x7c4051dc, 33 },
    { 0, 0 },
    { 0x914051de, 33 },
    { 0, 0 },
    { 0x7b8051e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d051ed, 55 },
    { 0, 0 },
    { 0x264051ef, 76 },
    { 0x325051ef, 23 },
    { 0xb1f051f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a051f4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af051f7, 23 },
    { 0, 0 },
    { 0x611051f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c05200, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7305203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36905208, 17 },
    { 0x86e05208, 17 },
    { 0, 0 },
    { 0xc310520b, 90 },
    { 0x9c70520c, 33 },
    { 0xde90520d, 90 },
    { 0xec90520d, 17 },
    { 0x9060520d, 33 },
    { 0x2280520e, 55 },
    { 0xae05211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f05214, 61 },
    { 0, 0 },
    { 0x81705216, 61 },
    { 0xf3905217, 33 },
    { 0x60505218, 34 },
    { 0x45405216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a005220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe0522c, 72 },
    { 0xc5f0522c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5805237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0523b, 95 },
    { 0, 0 },
    { 0x6df0523d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c005240, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f05243, 61 },
    { 0xdb05244, 33 },
    { 0, 0 },
    { 0xdbe05246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57405249, 98 },
    { 0x8360524a, 61 },
    { 0xe7a0524a, 61 },
    { 0xe880524c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a05251, 76 },
    { 0xf5105252, 95 },
    { 0, 0 },
    { 0x12e05254, 74 },
    { 0, 0 },
    { 0x58e05256, 33 },
    { 0xead05257, 81 },
    { 0x9a005257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440525b, 90 },
    { 0x8480525c, 33 },
    { 0, 0 },
    { 0x5c10525e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5305264, 61 },
    { 0xf3405264, 33 },
    { 0, 0 },
    { 0x3a205267, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e10526f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f05273, 95 },
    { 0x77905274, 34 },
    { 0x34005275, 98 },
    { 0xc5805275, 95 },
    { 0x85705277, 43 },
    { 0x21905278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5405281, 46 },
    { 0, 0 },
    { 0x29305283, 95 },
    { 0x7e205284, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed05288, 99 },
    { 0xc0f05289, 76 },
    { 0x47c0528a, 98 },
    { 0x3040528b, 46 },
    { 0xf360528c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab00528f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc05294, 95 },
    { 0, 0 },
    { 0x1f405296, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db0529a, 81 },
    { 0xb8a0529b, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2052a3, 33 },
    { 0x794052a4, 33 },
    { 0x8ce052a4, 90 },
    { 0x1e8052a3, 33 },
    { 0xbe8052a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6052aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67052b3, 68 },
    { 0x71c052b4, 43 },
    { 0xd40052b4, 98 },
    { 0, 0 },
    { 0xefc052b7, 55 },
    { 0x99c052b8, 90 },
    { 0, 0 },
    { 0xab9052ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150052bf, 34 },
    { 0xd61052c0, 23 },
    { 0x106052c1, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb052ca, 76 },
    { 0x970052cb, 23 },
    { 0x409052cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8052d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b052dc, 43 },
    { 0x7fa052dd, 23 },
    { 0x554052de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49052e3, 46 },
    { 0xfb4052e4, 76 },
    { 0xe9d052e5, 85 },
    { 0x79b052e3, 99 },
    { 0, 0 },
    { 0x4d6052e8, 61 },
    { 0x1ac052e9, 84 },
    { 0xe9f052e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x535052ed, 61 },
    { 0, 0 },
    { 0xe64052ef, 68 },
    { 0, 0 },
    { 0x932052f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80052fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5205305, 96 },
    { 0, 0 },
    { 0x81e05307, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3360530a, 61 },
    { 0xe520530a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7780530f, 61 },
    { 0x70f05310, 95 },
    { 0xf420530f, 90 },
    { 0x82205312, 23 },
    { 0x55105312, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2805316, 76 },
    { 0x6aa05317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4780531a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb0531d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14605321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26805324, 90 },
    { 0, 0 },
    { 0xf0505326, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280532a, 67 },
    { 0xf960532b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73405332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc205335, 78 },
    { 0, 0 },
    { 0xf6705337, 89 },
    { 0xa4005338, 96 },
    { 0xab405339, 61 },
    { 0x74d05338, 66 },
    { 0xc1305337, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1090533f, 67 },
    { 0x1e205340, 91 },
    { 0xc3105341, 33 },
    { 0x15705342, 77 },
    { 0x7e70533f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b05346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d20534b, 90 },
    { 0, 0 },
    { 0x470534d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a05352, 90 },
    { 0x59e05353, 75 },
    { 0x35205354, 61 },
    { 0xe6205354, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc10535c, 53 },
    { 0, 0 },
    { 0x3700535e, 66 },
    { 0xed80535f, 95 },
    { 0x8105360, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b05364, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6405367, 76 },
    { 0, 0 },
    { 0x4ca05369, 23 },
    { 0xe110536a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1120536e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea05373, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7605378, 67 },
    { 0xa6805379, 61 },
    { 0xa4f05379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360537f, 23 },
    { 0x1dd05380, 46 },
    { 0x30805380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b05384, 33 },
    { 0x48c05385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a05389, 95 },
    { 0x58505389, 98 },
    { 0, 0 },
    { 0xefd0538c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f05396, 98 },
    { 0x9b005396, 46 },
    { 0xc0e05396, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a0539d, 61 },
    { 0x1720539d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb053a3, 38 },
    { 0x7bc053a4, 61 },
    { 0xf13053a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1053a9, 33 },
    { 0, 0 },
    { 0xea5053ab, 95 },
    { 0x187053ab, 43 },
    { 0x370053ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a053b1, 93 },
    { 0x1c3053b2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00053b5, 90 },
    { 0, 0 },
    { 0x5fa053b7, 98 },
    { 0xf2053b8, 55 },
    { 0x423053b9, 43 },
    { 0x9c4053ba, 58 },
    { 0x5fd053bb, 33 },
    { 0x950053bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x747053bf, 43 },
    { 0x38b053c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae053c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x339053c8, 98 },
    { 0x1da053c8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de053cc, 43 },
    { 0xafa053cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51053d1, 83 },
    { 0, 0 },
    { 0xb0d053d3, 33 },
    { 0x1d2053d3, 99 },
    { 0x325053d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94053de, 46 },
    { 0xc4b053df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76053ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8053ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e053f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc053f7, 23 },
    { 0xe05053f8, 77 },
    { 0x975053f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce05401, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c05406, 99 },
    { 0, 0 },
    { 0x97a05408, 95 },
    { 0x94705409, 95 },
    { 0xcef05409, 33 },
    { 0x83e0540a, 23 },
    { 0, 0 },
    { 0xfc80540d, 95 },
    { 0, 0 },
    { 0xb270540f, 93 },
    { 0x3cf05410, 84 },
    { 0xdd605410, 91 },
    { 0x2fa05411, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b05419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950541f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35205422, 43 },
    { 0xde905423, 34 },
    { 0x31a05424, 76 },
    { 0x8a805424, 78 },
    { 0x21505424, 75 },
    { 0x64d05427, 65 },
    { 0xc4105428, 67 },
    { 0xb5805423, 90 },
    { 0x8a405426, 33 },
    { 0xd9b05422, 76 },
    { 0x4c005425, 23 },
    { 0x70e05427, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b805435, 90 },
    { 0xd2505436, 95 },
    { 0xc7c05436, 23 },
    { 0x68d05436, 76 },
    { 0x72d05438, 98 },
    { 0xa280543a, 33 },
    { 0xc3505438, 61 },
    { 0xc40543c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d05440, 98 },
    { 0, 0 },
    { 0x55605442, 90 },
    { 0xc9805443, 34 },
    { 0, 0 },
    { 0xfd605445, 46 },
    { 0x7d605445, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd20544a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0544e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c05453, 87 },
    { 0x33205453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a0545a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb105466, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec0546a, 23 },
    { 0x1060546a, 52 },
    { 0xf610546a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10546f, 33 },
    { 0x79f05470, 91 },
    { 0x50605471, 55 },
    { 0xee705472, 95 },
    { 0xed505473, 43 },
    { 0x8205473, 93 },
    { 0xc6805475, 85 },
    { 0x83805471, 95 },
    { 0, 0 },
    { 0x56605478, 95 },
    { 0x3ac05479, 87 },
    { 0x41205478, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e005481, 87 },
    { 0, 0 },
    { 0x4e105483, 23 },
    { 0x52005483, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70548a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe990548d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b05490, 95 },
    { 0xf7f05491, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd0549a, 90 },
    { 0, 0 },
    { 0xad30549c, 23 },
    { 0x6d80549c, 95 },
    { 0, 0 },
    { 0xb0e0549f, 95 },
    { 0x68c0549f, 90 },
    { 0x60d054a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638054a7, 61 },
    { 0xf58054a7, 33 },
    { 0xf9054a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1054ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef054b4, 95 },
    { 0x52a054b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc054bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71054bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82054c2, 76 },
    { 0xa05054c2, 90 },
    { 0x58b054c2, 66 },
    { 0, 0 },
    { 0x198054c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0054cb, 55 },
    { 0xa60054cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88054cf, 93 },
    { 0, 0 },
    { 0xe8d054d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f054d6, 34 },
    { 0xa29054d6, 99 },
    { 0, 0 },
    { 0x386054d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc054de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e054e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e054e6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308054ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x700054ef, 23 },
    { 0x547054ef, 95 },
    { 0x32a054f1, 38 },
    { 0x91c054f1, 43 },
    { 0, 0 },
    { 0x121054f4, 43 },
    { 0xcd3054f5, 98 },
    { 0x944054f4, 87 },
    { 0x2dc054f6, 81 },
    { 0, 0 },
    { 0x763054f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05054fc, 66 },
    { 0x74b054fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa05504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe105509, 90 },
    { 0xb540550a, 95 },
    { 0x8f50550b, 99 },
    { 0x4ea0550c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19905511, 78 },
    { 0xfcd05512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c05518, 33 },
    { 0xa6205518, 33 },
    { 0x6c90551a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60552b, 33 },
    { 0x5c0552c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570552f, 61 },
    { 0x3e10552f, 84 },
    { 0x30305531, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0553c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1505540, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d05543, 98 },
    { 0, 0 },
    { 0xf0d05545, 95 },
    { 0x17005546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f30554a, 98 },
    { 0, 0 },
    { 0xb000554c, 33 },
    { 0x2020554d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0105553, 90 },
    { 0x46605554, 78 },
    { 0, 0 },
    { 0x8ac05556, 76 },
    { 0x14105557, 95 },
    { 0xccb05558, 23 },
    { 0x8cb05559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4330555c, 90 },
    { 0x72e0555c, 90 },
    { 0x6a00555e, 97 },
    { 0xcd20555d, 98 },
    { 0x38b0555c, 75 },
    { 0xa9605561, 53 },
    { 0xf5805562, 96 },
    { 0x2d405561, 46 },
    { 0x61805561, 33 },
    { 0, 0 },
    { 0x43d05566, 95 },
    { 0xafa05567, 33 },
    { 0x3ea05568, 23 },
    { 0xa8805569, 33 },
    { 0xe30556a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0556e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e605574, 95 },
    { 0x4eb05574, 17 },
    { 0x39305576, 61 },
    { 0x8ef05577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd720557c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c105580, 78 },
    { 0, 0 },
    { 0x4cb05582, 33 },
    { 0xaa305583, 90 },
    { 0x70005584, 90 },
    { 0x9705585, 85 },
    { 0xb0e05585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe680558a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10558e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab05592, 90 },
    { 0, 0 },
    { 0x99705594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b705597, 52 },
    { 0xd005598, 43 },
    { 0x67f05598, 98 },
    { 0x5820559a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c20559f, 61 },
    { 0xa19055a0, 90 },
    { 0xa6e055a0, 100 },
    { 0xde1055a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807055a6, 96 },
    { 0x981055a7, 95 },
    { 0x9cc055a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb055ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb055b4, 78 },
    { 0xc56055b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9055b8, 43 },
    { 0, 0 },
    { 0xb7e055ba, 76 },
    { 0xc3055bb, 90 },
    { 0xfea055bb, 98 },
    { 0x541055bb, 55 },
    { 0xea5055bd, 23 },
    { 0, 0 },
    { 0x5b6055c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776055c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836055c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe055cc, 33 },
    { 0, 0 },
    { 0xbff055ce, 33 },
    { 0xa44055ce, 61 },
    { 0xf38055d0, 33 },
    { 0xa1f055d0, 90 },
    { 0x4c6055ce, 17 },
    { 0x4e2055d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47055d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7055dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362055e0, 95 },
    { 0x386055e0, 61 },
    { 0, 0 },
    { 0x852055e3, 46 },
    { 0xb6b055e4, 95 },
    { 0xa52055e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x903055e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5055ed, 90 },
    { 0, 0 },
    { 0xca5055ef, 81 },
    { 0, 0 },
    { 0xd69055f1, 55 },
    { 0, 0 },
    { 0x7e055f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e055f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05055fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad505602, 90 },
    { 0x24f05603, 78 },
    { 0, 0 },
    { 0xfbe05605, 33 },
    { 0, 0 },
    { 0xa4305607, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410560b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e505619, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d0561c, 95 },
    { 0x3c10561c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26405620, 55 },
    { 0x68f05621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8805629, 23 },
    { 0x45d0562a, 55 },
    { 0xeda0562a, 84 },
    { 0x6180562b, 98 },
    { 0xb040562d, 90 },
    { 0x33f0562d, 93 },
    { 0x640562e, 90 },
    { 0xf0c0562c, 98 },
    { 0, 0 },
    { 0xdaa05632, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe905637, 93 },
    { 0x99d05638, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1770563b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff505643, 97 },
    { 0, 0 },
    { 0x8d005645, 84 },
    { 0x21105646, 76 },
    { 0xdf905646, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x430564a, 98 },
    { 0x23d0564b, 87 },
    { 0xbcf0564c, 97 },
    { 0x410564d, 69 },
    { 0xd5b0564d, 33 },
    { 0xaa50564d, 17 },
    { 0x56005650, 61 },
    { 0xe0105651, 90 },
    { 0x5ee0564d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab05655, 90 },
    { 0x83d05656, 98 },
    { 0xa1005657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e80565d, 64 },
    { 0xe5e0565e, 98 },
    { 0xdca0565f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83605662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42605667, 55 },
    { 0x95e05667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b05672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a05677, 77 },
    { 0, 0 },
    { 0x4fd05679, 34 },
    { 0x41a0567a, 61 },
    { 0, 0 },
    { 0x80b0567c, 87 },
    { 0x7a10567c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3305683, 100 },
    { 0x8ce05684, 81 },
    { 0xf2b05684, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x90905688, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba0568b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1570568e, 61 },
    { 0x1f70568e, 46 },
    { 0xdff05690, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70405694, 93 },
    { 0x4d205695, 17 },
    { 0xb7905696, 91 },
    { 0x8ad05697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390569f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x732056a2, 46 },
    { 0xd4f056a2, 33 },
    { 0xfa6056a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5056a7, 61 },
    { 0xfb7056a8, 23 },
    { 0x11c056a7, 65 },
    { 0x8b5056aa, 90 },
    { 0x736056ab, 90 },
    { 0, 0 },
    { 0x8f5056ad, 76 },
    { 0x52a056ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77056b1, 84 },
    { 0xe16056b2, 33 },
    { 0xead056b3, 36 },
    { 0x297056b4, 76 },
    { 0x50056b5, 52 },
    { 0x148056b1, 90 },
    { 0, 0 },
    { 0xad8056b8, 95 },
    { 0, 0 },
    { 0xb64056ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671056c0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1056cb, 98 },
    { 0, 0 },
    { 0xff7056cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb056d4, 68 },
    { 0, 0 },
    { 0x8d2056d6, 98 },
    { 0, 0 },
    { 0xc7056d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4056dd, 34 },
    { 0xb6056de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95056e5, 23 },
    { 0xd8c056e5, 90 },
    { 0xe85056e7, 33 },
    { 0xfa5056e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60056ec, 61 },
    { 0xaa8056ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x646056f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569056f6, 36 },
    { 0x933056f7, 95 },
    { 0xb65056f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c056fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x438056fe, 87 },
    { 0x6cb056ff, 66 },
    { 0xff905700, 76 },
    { 0x8e305701, 98 },
    { 0, 0 },
    { 0x87405703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e05706, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x19805709, 98 },
    { 0x1c90570a, 90 },
    { 0, 0 },
    { 0x99f0570c, 40 },
    { 0x6d60570d, 92 },
    { 0x8ad0570e, 33 },
    { 0x6990570d, 90 },
    { 0x6df05710, 76 },
    { 0x9bf05711, 90 },
    { 0, 0 },
    { 0xe5505713, 99 },
    { 0, 0 },
    { 0x78b05715, 68 },
    { 0, 0 },
    { 0x6ba05717, 90 },
    { 0, 0 },
    { 0xe6e05719, 90 },
    { 0xa8a0571a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10571d, 42 },
    { 0xdb0571d, 67 },
    { 0x2a20571f, 66 },
    { 0, 0 },
    { 0x95505721, 98 },
    { 0x11d05722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d305727, 34 },
    { 0, 0 },
    { 0xd4505729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df0572d, 76 },
    { 0, 0 },
    { 0xcd40572f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43705732, 55 },
    { 0x27a05733, 68 },
    { 0x60f05734, 91 },
    { 0xf3505734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f05738, 23 },
    { 0, 0 },
    { 0x9340573a, 66 },
    { 0x5d00573b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2490573f, 53 },
    { 0x4e105740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70305747, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20574a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90574f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f05754, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf305757, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0575c, 33 },
    { 0xe990575d, 23 },
    { 0xa630575e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe005764, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb05767, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8270576a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf290576e, 85 },
    { 0x2b0576e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c05773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf05776, 90 },
    { 0x3ae05776, 98 },
    { 0, 0 },
    { 0x9205779, 90 },
    { 0x1120577a, 69 },
    { 0x4c80577a, 98 },
    { 0xb510577c, 61 },
    { 0x9380577a, 98 },
    { 0xf230577e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e705783, 52 },
    { 0x74705784, 33 },
    { 0x33205783, 90 },
    { 0, 0 },
    { 0x58e05787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c80578a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b05792, 46 },
    { 0xb6805792, 33 },
    { 0x92a05794, 61 },
    { 0xb6305792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d605798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b0579b, 98 },
    { 0x4cf0579c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a057a4, 90 },
    { 0x73a057a5, 81 },
    { 0, 0 },
    { 0x224057a7, 98 },
    { 0x360057a8, 76 },
    { 0xaf0057a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285057bc, 90 },
    { 0x8e8057bd, 34 },
    { 0xc5a057bd, 81 },
    { 0, 0 },
    { 0xc37057c0, 33 },
    { 0, 0 },
    { 0x99b057c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40057c5, 52 },
    { 0x96c057c6, 33 },
    { 0x961057c5, 90 },
    { 0x568057c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84057cc, 23 },
    { 0, 0 },
    { 0xd96057ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66057d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc057d5, 33 },
    { 0x80e057d6, 23 },
    { 0x758057d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3057dc, 95 },
    { 0xca2057dd, 61 },
    { 0x1aa057de, 90 },
    { 0x347057df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a057e6, 95 },
    { 0xa9f057e7, 76 },
    { 0x411057e8, 99 },
    { 0x7f7057e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628057ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26057f5, 98 },
    { 0x116057f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a057fe, 90 },
    { 0, 0 },
    { 0x66a05800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a505803, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df05806, 52 },
    { 0x86505807, 61 },
    { 0xba05808, 33 },
    { 0xfdc05809, 93 },
    { 0x2d60580a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70580f, 61 },
    { 0x5a80580f, 77 },
    { 0x3d305810, 98 },
    { 0xf7905812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0405817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170581c, 61 },
    { 0, 0 },
    { 0x1810581e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c05822, 23 },
    { 0xd4005823, 66 },
    { 0xdc405824, 76 },
    { 0x47105822, 33 },
    { 0xf1105826, 76 },
    { 0xb8805824, 91 },
    { 0, 0 },
    { 0x8e705829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4470582d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd705831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e05840, 52 },
    { 0x13905840, 17 },
    { 0xce105842, 33 },
    { 0x95205840, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c05849, 85 },
    { 0, 0 },
    { 0x2100584b, 87 },
    { 0x2c70584c, 23 },
    { 0x3670584d, 100 },
    { 0x7260584c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1405859, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0585c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0405870, 90 },
    { 0xc0c05870, 33 },
    { 0x22705871, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d805878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990587c, 55 },
    { 0xcd70587d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f05880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b505883, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f205887, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc930588d, 96 },
    { 0xa230588d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f05893, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x86105896, 68 },
    { 0xa6805897, 33 },
    { 0xa4505896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65058a1, 34 },
    { 0, 0 },
    { 0x4c058a3, 93 },
    { 0x9a0058a4, 90 },
    { 0, 0 },
    { 0x257058a6, 59 },
    { 0xa5e058a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b058aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809058ae, 88 },
    { 0xd51058af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba058b3, 97 },
    { 0, 0 },
    { 0x23c058b5, 52 },
    { 0x809058b5, 33 },
    { 0, 0 },
    { 0xdae058b8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403058bc, 90 },
    { 0xae7058bd, 90 },
    { 0xa1e058be, 84 },
    { 0x44d058bc, 61 },
    { 0x674058c0, 33 },
    { 0, 0 },
    { 0xdaf058c2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e058c6, 95 },
    { 0xea8058c7, 98 },
    { 0x22e058c8, 43 },
    { 0x60058c8, 84 },
    { 0x373058ca, 90 },
    { 0x9bb058ca, 17 },
    { 0xbdb058cc, 40 },
    { 0x886058cc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08058d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e058d9, 76 },
    { 0x9e8058da, 33 },
    { 0xe19058da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433058df, 43 },
    { 0xb3a058df, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26058e5, 61 },
    { 0xb80058e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x226058e9, 78 },
    { 0, 0 },
    { 0x9fe058eb, 90 },
    { 0, 0 },
    { 0x2ec058ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83058f4, 90 },
    { 0x8fb058f5, 91 },
    { 0, 0 },
    { 0xf66058f7, 23 },
    { 0, 0 },
    { 0xe0058f9, 33 },
    { 0, 0 },
    { 0xd09058fb, 23 },
    { 0xdeb058fc, 78 },
    { 0, 0 },
    { 0x511058fe, 95 },
    { 0xa6d058ff, 46 },
    { 0x1ed058ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf240590a, 75 },
    { 0x10d0590b, 34 },
    { 0xd0e0590c, 61 },
    { 0x4ff0590c, 17 },
    { 0, 0 },
    { 0x79e0590f, 55 },
    { 0xc220590f, 76 },
    { 0x4da05911, 92 },
    { 0, 0 },
    { 0xdbe05913, 76 },
    { 0x49705914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95405917, 100 },
    { 0, 0 },
    { 0xef605919, 76 },
    { 0xc770591a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e0591d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c505929, 95 },
    { 0xe8c0592a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe700592d, 23 },
    { 0x2b30592e, 90 },
    { 0x1610592d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16805933, 55 },
    { 0, 0 },
    { 0x7f505935, 78 },
    { 0x97c05935, 23 },
    { 0xcb405936, 91 },
    { 0x18005935, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf60593c, 61 },
    { 0x8900593c, 98 },
    { 0xff00593e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a505945, 23 },
    { 0, 0 },
    { 0xdf805947, 93 },
    { 0, 0 },
    { 0x20b05949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd00594d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd105959, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580595e, 61 },
    { 0xf8e0595e, 40 },
    { 0x46d05960, 66 },
    { 0x91205961, 95 },
    { 0x5c105961, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de05969, 23 },
    { 0x7ce05969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x596d, 87 },
    { 0x27c0596e, 90 },
    { 0x3130596f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65805972, 98 },
    { 0x36e05973, 23 },
    { 0, 0 },
    { 0xbcb05975, 97 },
    { 0x9c05976, 99 },
    { 0x4ca05977, 76 },
    { 0x69805975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe780597b, 90 },
    { 0x3e10597c, 68 },
    { 0x43d0597c, 90 },
    { 0x7360597b, 33 },
    { 0x6a0597f, 98 },
    { 0xdbe0597d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70605983, 95 },
    { 0, 0 },
    { 0x4dd05985, 98 },
    { 0xa9105986, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a05989, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0598f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb405992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d05997, 33 },
    { 0, 0 },
    { 0xe6e05999, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7020599c, 78 },
    { 0x9690599d, 23 },
    { 0x16b0599d, 88 },
    { 0xd3e0599f, 23 },
    { 0xd05059a0, 55 },
    { 0xe65059a1, 84 },
    { 0x140059a2, 91 },
    { 0x14c0599d, 95 },
    { 0x11b059a1, 55 },
    { 0xd7f059a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0059a8, 85 },
    { 0x654059a8, 23 },
    { 0x990059aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4059ae, 95 },
    { 0, 0 },
    { 0x61d059b0, 33 },
    { 0x5e6059b1, 61 },
    { 0xedd059b2, 85 },
    { 0x267059b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e059b9, 93 },
    { 0, 0 },
    { 0xdb059bb, 90 },
    { 0x5c4059bc, 90 },
    { 0, 0 },
    { 0x7bd059be, 56 },
    { 0x32d059bf, 90 },
    { 0xa46059c0, 81 },
    { 0x627059c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d059c6, 84 },
    { 0, 0 },
    { 0x7d6059c8, 99 },
    { 0xe55059c9, 34 },
    { 0x7dd059ca, 93 },
    { 0x726059ca, 23 },
    { 0xe0e059c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922059d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb059d6, 33 },
    { 0, 0 },
    { 0xf0c059d8, 76 },
    { 0xd27059d9, 81 },
    { 0xe97059d9, 98 },
    { 0xe40059d8, 34 },
    { 0xf2a059db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc059e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30059e6, 98 },
    { 0x4ac059e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e059ea, 81 },
    { 0xfae059eb, 55 },
    { 0x46d059ec, 55 },
    { 0x8c059ed, 23 },
    { 0x74059ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7059f2, 90 },
    { 0xd1d059f3, 90 },
    { 0xee7059f2, 96 },
    { 0xf84059f3, 33 },
    { 0xe63059f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60059f9, 53 },
    { 0x2a3059f9, 98 },
    { 0x553059fb, 61 },
    { 0xbb059f9, 33 },
    { 0xc8a059f9, 33 },
    { 0x8ef059fe, 90 },
    { 0x6b059ff, 33 },
    { 0x293059fe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2905a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d705a06, 100 },
    { 0xcb505a07, 98 },
    { 0x84f05a07, 68 },
    { 0x5be05a08, 98 },
    { 0xf5a05a07, 43 },
    { 0x97205a07, 76 },
    { 0x7d205a08, 23 },
    { 0, 0 },
    { 0xd9105a0e, 67 },
    { 0x7c005a0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05a16, 99 },
    { 0x4c405a16, 95 },
    { 0x33605a18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef05a1c, 33 },
    { 0, 0 },
    { 0x37705a1e, 98 },
    { 0xd4f05a1f, 98 },
    { 0xb4c05a20, 37 },
    { 0, 0 },
    { 0x39205a22, 76 },
    { 0xd7605a23, 76 },
    { 0xf4905a24, 90 },
    { 0xc8405a24, 33 },
    { 0xdcc05a23, 84 },
    { 0x66305a24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c05a2d, 33 },
    { 0xfb105a2e, 85 },
    { 0, 0 },
    { 0xdec05a30, 84 },
    { 0, 0 },
    { 0xe6505a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0605a35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee805a3a, 23 },
    { 0, 0 },
    { 0xe4905a3c, 95 },
    { 0xb7705a3c, 34 },
    { 0x64905a3e, 90 },
    { 0xa8305a3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93705a48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45605a4c, 66 },
    { 0x2c505a4c, 46 },
    { 0x7505a4e, 19 },
    { 0, 0 },
    { 0x25a05a50, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85205a54, 23 },
    { 0, 0 },
    { 0x5a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f105a59, 98 },
    { 0x5df05a5a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e05a5d, 95 },
    { 0xad705a5e, 81 },
    { 0x83e05a5d, 90 },
    { 0x81905a60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb805a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d805a6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0105a6e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b05a72, 52 },
    { 0, 0 },
    { 0xf9405a74, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3705a77, 23 },
    { 0xfc805a78, 40 },
    { 0x6c05a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd405a7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a05a80, 100 },
    { 0xd9b05a81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87205a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5205a87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe05a8a, 61 },
    { 0, 0 },
    { 0x75805a8c, 61 },
    { 0x6b05a8d, 52 },
    { 0x12c05a8d, 33 },
    { 0x98005a8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8105a93, 78 },
    { 0x5d805a94, 34 },
    { 0x2f05a94, 87 },
    { 0xd4e05a96, 33 },
    { 0, 0 },
    { 0x1c805a98, 55 },
    { 0, 0 },
    { 0xb3305a9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab705a9d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9305aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9005aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb105aa9, 43 },
    { 0x77305aaa, 76 },
    { 0xb7005aab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf05ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9205ab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e205ab9, 61 },
    { 0x56f05aba, 95 },
    { 0, 0 },
    { 0x2ad05abc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db05abf, 88 },
    { 0x22205ac0, 33 },
    { 0, 0 },
    { 0xaae05ac2, 90 },
    { 0, 0 },
    { 0x9e605ac4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3305aca, 95 },
    { 0, 0 },
    { 0xd4c05acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c05ad0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd205ad4, 90 },
    { 0, 0 },
    { 0xb4505ad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8605adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f05ae0, 86 },
    { 0x49505ae1, 78 },
    { 0x36505ae2, 33 },
    { 0xaca05ae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b05aec, 90 },
    { 0x91405aec, 90 },
    { 0xc1005aee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a05af3, 90 },
    { 0, 0 },
    { 0x5ca05af5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58305afa, 85 },
    { 0x5c605afb, 93 },
    { 0, 0 },
    { 0x54605afd, 23 },
    { 0, 0 },
    { 0x9e405aff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b05b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb05b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d05b17, 90 },
    { 0x13b05b18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x98905b1b, 76 },
    { 0x37f05b1b, 61 },
    { 0xa8e05b1d, 95 },
    { 0xb0005b1e, 87 },
    { 0xb2c05b1e, 33 },
    { 0x74b05b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a05b30, 33 },
    { 0, 0 },
    { 0x44105b32, 33 },
    { 0x7d905b33, 55 },
    { 0, 0 },
    { 0x805b35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c805b38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4705b3b, 90 },
    { 0xda305b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5305b3f, 85 },
    { 0xea105b40, 95 },
    { 0, 0 },
    { 0x59e05b42, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7505b4a, 91 },
    { 0x8ac05b4b, 61 },
    { 0x6205b4c, 75 },
    { 0xa9205b4d, 85 },
    { 0x33205b4e, 90 },
    { 0x2d805b4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad05b52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d005b56, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c305b59, 90 },
    { 0, 0 },
    { 0x6fd05b5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c405b5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6705b63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a05b66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee05b6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71805b72, 100 },
    { 0, 0 },
    { 0x70105b74, 91 },
    { 0, 0 },
    { 0x39a05b76, 90 },
    { 0, 0 },
    { 0xd1805b78, 61 },
    { 0, 0 },
    { 0x57e05b7a, 33 },
    { 0x58805b7b, 90 },
    { 0x2d105b7c, 33 },
    { 0xd505b7d, 90 },
    { 0, 0 },
    { 0xdc705b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22205b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef05b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e05b8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce05b8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5505b92, 61 },
    { 0xb105b93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b05b96, 43 },
    { 0xab05b97, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905b9b, 68 },
    { 0x19205b9c, 90 },
    { 0xb8905b9d, 95 },
    { 0xca905b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26205ba5, 17 },
    { 0x59305ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da05ba9, 33 },
    { 0, 0 },
    { 0x40a05bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96805bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc05bb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86805bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d05bc2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d05bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f05bc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10805bce, 67 },
    { 0x97a05bcf, 33 },
    { 0x41c05bcf, 80 },
    { 0xe705bd1, 69 },
    { 0x24405bd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d105bd7, 53 },
    { 0x4ed05bd8, 90 },
    { 0x4ec05bd9, 34 },
    { 0x1e205bda, 90 },
    { 0xc2b05bdb, 98 },
    { 0x61505bdc, 99 },
    { 0x48205bdc, 95 },
    { 0, 0 },
    { 0xd705bdf, 34 },
    { 0, 0 },
    { 0x40005be1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a405be4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62805bea, 87 },
    { 0x3dd05bea, 33 },
    { 0x4e105bea, 66 },
    { 0x5be05bed, 61 },
    { 0x63005bee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x86805bf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51405bf4, 95 },
    { 0x87905bf5, 90 },
    { 0xb5405bf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c05bf9, 78 },
    { 0x9e305bfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de05bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c705c00, 76 },
    { 0x60805c01, 61 },
    { 0xccd05c02, 33 },
    { 0, 0 },
    { 0x1f505c04, 33 },
    { 0x9c05c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe905c09, 51 },
    { 0x7fa05c0a, 95 },
    { 0, 0 },
    { 0x10e05c0c, 64 },
    { 0, 0 },
    { 0x9d105c0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad05c1a, 66 },
    { 0x605c1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a05c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d05c2e, 76 },
    { 0, 0 },
    { 0x6a305c30, 85 },
    { 0xea005c31, 61 },
    { 0x31e05c32, 34 },
    { 0xe4f05c30, 23 },
    { 0xa9205c32, 90 },
    { 0x49005c35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac05c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f05c3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5805c47, 87 },
    { 0xf6005c47, 91 },
    { 0, 0 },
    { 0x87805c4a, 43 },
    { 0, 0 },
    { 0x5d805c4c, 67 },
    { 0xe7905c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde05c53, 98 },
    { 0xe8705c54, 90 },
    { 0xfe005c55, 77 },
    { 0, 0 },
    { 0xd305c57, 43 },
    { 0, 0 },
    { 0xd7905c59, 90 },
    { 0xb2805c5a, 98 },
    { 0xa3d05c5b, 61 },
    { 0x4b505c5c, 95 },
    { 0, 0 },
    { 0x5d605c5e, 98 },
    { 0xce405c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f05c66, 81 },
    { 0, 0 },
    { 0x28305c68, 33 },
    { 0, 0 },
    { 0x50d05c6a, 43 },
    { 0xb1205c6b, 33 },
    { 0x51205c6b, 23 },
    { 0xf0f05c6d, 43 },
    { 0x94705c6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb05c74, 61 },
    { 0x54705c74, 33 },
    { 0, 0 },
    { 0x38705c77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc05c7a, 95 },
    { 0xa9705c7b, 93 },
    { 0x96d05c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c05c80, 46 },
    { 0xccf05c81, 90 },
    { 0x6b05c82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff05c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9705c8a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c05c8d, 55 },
    { 0x55005c8e, 78 },
    { 0x86f05c8f, 77 },
    { 0x10205c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac05c99, 61 },
    { 0x6cf05c9a, 78 },
    { 0xb7a05c9a, 95 },
    { 0x91e05c99, 53 },
    { 0, 0 },
    { 0x35805c9e, 17 },
    { 0x2d05c9f, 61 },
    { 0x84505ca0, 33 },
    { 0x8c905ca1, 34 },
    { 0x90905ca2, 61 },
    { 0, 0 },
    { 0x43205ca4, 85 },
    { 0x2a905ca5, 33 },
    { 0xf5b05ca6, 95 },
    { 0x46c05ca5, 55 },
    { 0x22a05ca8, 61 },
    { 0, 0 },
    { 0xa1e05caa, 93 },
    { 0, 0 },
    { 0x63405cac, 55 },
    { 0x7ab05cad, 76 },
    { 0xb0005cae, 99 },
    { 0x94b05cad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e05cb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae05cb8, 61 },
    { 0x9f005cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb05cbe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16305cc2, 93 },
    { 0, 0 },
    { 0x13b05cc4, 95 },
    { 0, 0 },
    { 0x25a05cc6, 33 },
    { 0x4dc05cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75705cca, 90 },
    { 0xff405ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61005cce, 90 },
    { 0xfc405ccf, 55 },
    { 0xf1705ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e05cd3, 23 },
    { 0x28105cd3, 100 },
    { 0xdb105cd5, 90 },
    { 0x11c05cd3, 98 },
    { 0x3c705cd6, 95 },
    { 0xb2b05cd7, 76 },
    { 0xaa205cd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed05cdf, 33 },
    { 0x26905cdf, 95 },
    { 0x8cf05ce0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16905ce5, 95 },
    { 0xbe905ce5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d05cee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e105cf4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a05cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d05cfe, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9305d04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78705d08, 78 },
    { 0xb205d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6205d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd05d12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a305d17, 43 },
    { 0xb6005d18, 100 },
    { 0, 0 },
    { 0xfc005d1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd05d1f, 52 },
    { 0x30205d20, 55 },
    { 0x73205d1f, 95 },
    { 0x89905d22, 90 },
    { 0xca105d20, 98 },
    { 0x41c05d20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x85405d27, 85 },
    { 0, 0 },
    { 0xdf005d29, 43 },
    { 0x69c05d2a, 33 },
    { 0xdb005d2b, 90 },
    { 0, 0 },
    { 0x89f05d2d, 61 },
    { 0xd6205d2e, 61 },
    { 0, 0 },
    { 0x49c05d30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54605d34, 93 },
    { 0x32a05d35, 23 },
    { 0xe9405d36, 43 },
    { 0xd8e05d37, 90 },
    { 0x88305d38, 99 },
    { 0x4b005d39, 46 },
    { 0xa8805d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14505d3e, 95 },
    { 0, 0 },
    { 0xa7105d40, 84 },
    { 0xa6505d40, 90 },
    { 0, 0 },
    { 0x8bf05d43, 85 },
    { 0, 0 },
    { 0xc705d45, 76 },
    { 0, 0 },
    { 0x7ba05d47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb605d4a, 78 },
    { 0, 0 },
    { 0xf0005d4c, 23 },
    { 0, 0 },
    { 0xb9705d4e, 66 },
    { 0, 0 },
    { 0x86605d50, 90 },
    { 0, 0 },
    { 0xe4a05d52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51705d55, 61 },
    { 0x1c405d56, 23 },
    { 0x81005d57, 43 },
    { 0x26705d58, 93 },
    { 0x86c05d58, 55 },
    { 0, 0 },
    { 0xa7205d5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e05d61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e05d68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95805d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a805d71, 95 },
    { 0, 0 },
    { 0x59105d73, 61 },
    { 0, 0 },
    { 0xd05d75, 68 },
    { 0x69205d76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49005d7a, 90 },
    { 0, 0 },
    { 0x1005d7c, 98 },
    { 0x28005d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f05d80, 88 },
    { 0x7ee05d81, 91 },
    { 0x21205d82, 98 },
    { 0xe6d05d83, 33 },
    { 0, 0 },
    { 0xe5d05d85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50505d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c05d8c, 33 },
    { 0xacb05d8d, 98 },
    { 0xbe805d8e, 55 },
    { 0xb6b05d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d05d93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43605d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d05d9b, 90 },
    { 0, 0 },
    { 0x43805d9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a05da5, 90 },
    { 0x53a05da5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2705dab, 77 },
    { 0, 0 },
    { 0x1c405dad, 34 },
    { 0xd1f05dad, 33 },
    { 0x4fc05daf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce05db6, 33 },
    { 0xc0e05db7, 87 },
    { 0x91c05db8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb05dc2, 55 },
    { 0x71b05dc3, 33 },
    { 0xb8505dc4, 85 },
    { 0x6f905dc3, 94 },
    { 0x60205dc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b905dcc, 90 },
    { 0x7cf05dcd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab05dd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf905dd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13405dde, 23 },
    { 0x76405dde, 90 },
    { 0x4f405de0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb305de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b605ded, 98 },
    { 0xf6605ded, 95 },
    { 0, 0 },
    { 0xe3905df0, 97 },
    { 0x7f905df0, 98 },
    { 0, 0 },
    { 0x18105df3, 61 },
    { 0x9cd05df3, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd05df8, 90 },
    { 0x82205df9, 99 },
    { 0xe9005df8, 90 },
    { 0, 0 },
    { 0x51c05dfc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba05e03, 98 },
    { 0, 0 },
    { 0x2105e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f05e09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa05e11, 98 },
    { 0xa5005e12, 95 },
    { 0x3f05e13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e405e1b, 85 },
    { 0x99205e1c, 52 },
    { 0x13305e1c, 81 },
    { 0x48b05e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2605e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac105e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe605e2d, 33 },
    { 0xbaa05e2e, 77 },
    { 0xb7705e2d, 52 },
    { 0xd1805e30, 76 },
    { 0x69405e2e, 33 },
    { 0x8c505e2f, 87 },
    { 0x24a05e30, 33 },
    { 0x705e32, 66 },
    { 0x56e05e30, 97 },
    { 0xb2305e33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64105e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5105e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb105e47, 46 },
    { 0xc1605e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd05e4b, 88 },
    { 0x99d05e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xee505e4f, 61 },
    { 0x95d05e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb05e54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca05e57, 99 },
    { 0x82905e58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17105e5c, 33 },
    { 0x1db05e5d, 95 },
    { 0xedf05e5e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c05e61, 91 },
    { 0, 0 },
    { 0x33e05e63, 93 },
    { 0x47905e64, 85 },
    { 0x4ff05e65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea05e6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf205e6d, 33 },
    { 0x3a305e6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1905e73, 52 },
    { 0xc1b05e74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c605e7a, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x83205e7d, 23 },
    { 0, 0 },
    { 0xbfc05e7f, 95 },
    { 0, 0 },
    { 0x92005e81, 33 },
    { 0, 0 },
    { 0x56805e83, 90 },
    { 0x99105e84, 61 },
    { 0x84c05e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe05e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de05e8c, 61 },
    { 0x45305e8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9505e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab05e95, 61 },
    { 0x5f505e95, 55 },
    { 0, 0 },
    { 0xdc005e98, 34 },
    { 0, 0 },
    { 0xa8805e9a, 90 },
    { 0x85c05e9b, 23 },
    { 0, 0 },
    { 0xd6b05e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81305ea4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e605eaa, 33 },
    { 0, 0 },
    { 0x4c605eac, 90 },
    { 0x97f05ead, 97 },
    { 0xde605eae, 33 },
    { 0xb8305eae, 88 },
    { 0xa1305eac, 43 },
    { 0xe1105eb1, 55 },
    { 0x4c005eb2, 33 },
    { 0x55c05eae, 61 },
    { 0xd8105eb4, 61 },
    { 0x47105eb5, 95 },
    { 0, 0 },
    { 0x6dc05eb7, 61 },
    { 0x3d205eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b05ebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d05ec0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba05ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff05ec8, 100 },
    { 0xf505ec8, 23 },
    { 0, 0 },
    { 0x4e405ecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b805ecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa005ed2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c605ed8, 61 },
    { 0xa1a05ed9, 33 },
    { 0x3c605eda, 23 },
    { 0x65105edb, 78 },
    { 0xe8f05edc, 45 },
    { 0x5c905edd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c05ee5, 99 },
    { 0xd5005ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2405ee9, 90 },
    { 0x98205ee9, 75 },
    { 0, 0 },
    { 0x70005eec, 66 },
    { 0xfcf05eec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e005ef0, 61 },
    { 0, 0 },
    { 0x76e05ef2, 90 },
    { 0x1005ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e05f04, 90 },
    { 0x7b205f05, 33 },
    { 0xc6b05f06, 65 },
    { 0x7e05f06, 33 },
    { 0x2c605f08, 98 },
    { 0xa7a05f08, 90 },
    { 0xf9f05f0a, 76 },
    { 0x89d05f0b, 90 },
    { 0x7a005f0b, 23 },
    { 0, 0 },
    { 0xef105f0e, 23 },
    { 0x85405f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf705f12, 95 },
    { 0xb9a05f12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac05f19, 76 },
    { 0xabe05f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35105f1d, 23 },
    { 0x5a905f1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d705f26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec05f35, 56 },
    { 0, 0 },
    { 0xcdb05f37, 33 },
    { 0x7e405f38, 90 },
    { 0xc0d05f39, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f05f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4205f48, 61 },
    { 0xc7405f49, 34 },
    { 0x40d05f4a, 33 },
    { 0, 0 },
    { 0xf0905f4c, 36 },
    { 0x1005f4c, 96 },
    { 0x6bb05f4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd005f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab05f55, 87 },
    { 0, 0 },
    { 0x82005f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4205f5c, 52 },
    { 0xa3305f5d, 61 },
    { 0x47305f5e, 52 },
    { 0, 0 },
    { 0x5be05f60, 33 },
    { 0xb9f05f60, 33 },
    { 0x7f605f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e05f66, 61 },
    { 0x95005f67, 55 },
    { 0, 0 },
    { 0x33705f69, 56 },
    { 0x8de05f69, 91 },
    { 0x93105f6b, 65 },
    { 0xfc05f69, 23 },
    { 0, 0 },
    { 0x2eb05f6e, 90 },
    { 0, 0 },
    { 0x2d705f70, 33 },
    { 0x13a05f70, 61 },
    { 0x19b05f72, 95 },
    { 0x7b305f73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad205f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe305f7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3405f8a, 100 },
    { 0xa4705f8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f05f8f, 61 },
    { 0x34605f8f, 61 },
    { 0, 0 },
    { 0xefa05f92, 67 },
    { 0, 0 },
    { 0xa8405f94, 91 },
    { 0xe6b05f95, 34 },
    { 0, 0 },
    { 0x7ec05f97, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2005f9c, 33 },
    { 0, 0 },
    { 0x7eb05f9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4805fa5, 61 },
    { 0, 0 },
    { 0xca805fa7, 81 },
    { 0x3da05fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4105fac, 43 },
    { 0xb6e05fad, 90 },
    { 0, 0 },
    { 0x9fe05faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde805fb2, 93 },
    { 0xaf105fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc605fb6, 23 },
    { 0xef505fb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d05fba, 66 },
    { 0x14e05fbb, 95 },
    { 0xd0805fbc, 99 },
    { 0x86105fbd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a05fc1, 61 },
    { 0, 0 },
    { 0x705fc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8605fc6, 52 },
    { 0, 0 },
    { 0xa7905fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b205fcb, 61 },
    { 0x15005fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d05fdc, 33 },
    { 0x8cc05fdd, 23 },
    { 0x19c05fdd, 34 },
    { 0x7bd05fdf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05fe3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c05fe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db05fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5805ff2, 67 },
    { 0, 0 },
    { 0x79105ff4, 90 },
    { 0, 0 },
    { 0x7c205ff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12206001, 90 },
    { 0, 0 },
    { 0x4b706003, 61 },
    { 0x35506004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0600a, 66 },
    { 0, 0 },
    { 0x3770600c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda06010, 98 },
    { 0xcf006010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d06014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d06017, 61 },
    { 0xbed06018, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f60601c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b006021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1906025, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa670602c, 33 },
    { 0xfee0602d, 23 },
    { 0, 0 },
    { 0x5330602f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc06035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e06039, 90 },
    { 0, 0 },
    { 0x22d0603b, 33 },
    { 0xc7b0603c, 34 },
    { 0x5930603c, 33 },
    { 0x5760603e, 88 },
    { 0, 0 },
    { 0x10e06040, 78 },
    { 0x82506041, 33 },
    { 0x66906042, 64 },
    { 0xf6806042, 23 },
    { 0, 0 },
    { 0xf8906045, 23 },
    { 0, 0 },
    { 0xc9c06047, 23 },
    { 0xfd06047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110604f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79406058, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0605c, 52 },
    { 0, 0 },
    { 0xf530605e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f06061, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc06067, 43 },
    { 0x22b06067, 23 },
    { 0x32a06069, 95 },
    { 0x18c06069, 90 },
    { 0xc4e0606b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c0606f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d06072, 66 },
    { 0x93706073, 76 },
    { 0xe7c06074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c406079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc160607d, 87 },
    { 0xa530607e, 28 },
    { 0x8700607d, 95 },
    { 0x67506080, 55 },
    { 0, 0 },
    { 0x60306082, 33 },
    { 0, 0 },
    { 0x90d06084, 76 },
    { 0xe5206085, 76 },
    { 0, 0 },
    { 0x45306087, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71006090, 91 },
    { 0, 0 },
    { 0x84b06092, 99 },
    { 0, 0 },
    { 0x76306094, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00609d, 95 },
    { 0x7c20609d, 98 },
    { 0x8f00609f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d060a3, 61 },
    { 0xfea060a3, 61 },
    { 0x45e060a5, 98 },
    { 0xae7060a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc060a9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51060ae, 94 },
    { 0x448060af, 43 },
    { 0, 0 },
    { 0x2cf060b1, 90 },
    { 0x845060b2, 43 },
    { 0xf96060b3, 34 },
    { 0xf1e060b1, 61 },
    { 0x18b060b5, 93 },
    { 0xfac060b6, 84 },
    { 0x83e060b6, 75 },
    { 0x82b060b7, 98 },
    { 0x9dc060b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628060be, 87 },
    { 0, 0 },
    { 0xfbf060c0, 84 },
    { 0x5d1060c0, 90 },
    { 0xd40060c2, 90 },
    { 0, 0 },
    { 0xb1a060c4, 61 },
    { 0x359060c5, 33 },
    { 0xced060c5, 34 },
    { 0xe3e060c6, 90 },
    { 0x2c5060c8, 95 },
    { 0, 0 },
    { 0x10c060ca, 38 },
    { 0, 0 },
    { 0xdfd060cc, 61 },
    { 0xd39060cd, 61 },
    { 0xac7060ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x793060d1, 34 },
    { 0x77e060d2, 93 },
    { 0x94c060d3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1060d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5060db, 76 },
    { 0xc4f060dc, 98 },
    { 0xed1060dd, 34 },
    { 0, 0 },
    { 0xfed060df, 98 },
    { 0x616060e0, 90 },
    { 0xaa4060e1, 55 },
    { 0x420060e1, 33 },
    { 0xe32060e3, 61 },
    { 0x532060e4, 95 },
    { 0, 0 },
    { 0xfa9060e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab060ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d060ed, 33 },
    { 0x266060ee, 91 },
    { 0, 0 },
    { 0x61e060f0, 52 },
    { 0xe1060f1, 81 },
    { 0, 0 },
    { 0x6f7060f3, 61 },
    { 0, 0 },
    { 0xda4060f5, 95 },
    { 0x7f2060f6, 93 },
    { 0, 0 },
    { 0xed060f8, 90 },
    { 0x889060f8, 98 },
    { 0xfe7060fa, 61 },
    { 0xc5b060f8, 90 },
    { 0x43d060f8, 33 },
    { 0xa55060f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb06101, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b06105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58106108, 61 },
    { 0x1e006109, 33 },
    { 0x4f50610a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8206110, 61 },
    { 0, 0 },
    { 0xb8906112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8906118, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad0611c, 85 },
    { 0, 0 },
    { 0x5b80611e, 66 },
    { 0, 0 },
    { 0x71806120, 84 },
    { 0xeae06121, 98 },
    { 0xd8206121, 23 },
    { 0x33c06122, 90 },
    { 0x9cb06123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e906127, 88 },
    { 0xdee06128, 49 },
    { 0xf4106129, 78 },
    { 0xe7f0612a, 34 },
    { 0x2f00612b, 90 },
    { 0xc1f0612c, 92 },
    { 0x26c0612d, 90 },
    { 0x43a0612d, 33 },
    { 0x340612f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95406133, 95 },
    { 0, 0 },
    { 0x1ff06135, 33 },
    { 0xf3006135, 64 },
    { 0x59606137, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0613d, 91 },
    { 0x59f0613e, 76 },
    { 0, 0 },
    { 0x5c406140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65806145, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f0614a, 34 },
    { 0, 0 },
    { 0xfcb0614c, 34 },
    { 0xfd00614d, 44 },
    { 0, 0 },
    { 0xac00614f, 90 },
    { 0xdab06150, 61 },
    { 0xdce06150, 64 },
    { 0xfa00614f, 87 },
    { 0x46306153, 59 },
    { 0xa506154, 33 },
    { 0x82106155, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7780615e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d06161, 90 },
    { 0xb9206161, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7130616a, 61 },
    { 0xedf0616b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17006174, 75 },
    { 0x11b06175, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad0617f, 95 },
    { 0x4c806180, 76 },
    { 0x76706181, 43 },
    { 0xe3706182, 34 },
    { 0x57a06182, 98 },
    { 0xc5e06183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a06187, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0618c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f70618f, 40 },
    { 0x9a00618f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4406194, 33 },
    { 0x4e006195, 84 },
    { 0xbb06195, 98 },
    { 0x40306197, 33 },
    { 0, 0 },
    { 0xc1406199, 77 },
    { 0x18a06199, 95 },
    { 0, 0 },
    { 0xa990619c, 97 },
    { 0x5960619c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0061a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e061a5, 95 },
    { 0xb29061a6, 61 },
    { 0xbb6061a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x270061aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7061ae, 33 },
    { 0x695061af, 68 },
    { 0x9da061b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43061b4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac061b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef061bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x265061c0, 76 },
    { 0x507061c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17061c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9061ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee061d5, 43 },
    { 0xace061d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a061da, 90 },
    { 0xdf4061da, 90 },
    { 0x60a061dc, 85 },
    { 0x830061dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58061e2, 34 },
    { 0x61f061e3, 52 },
    { 0xea7061e4, 98 },
    { 0, 0 },
    { 0x997061e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6061ea, 93 },
    { 0, 0 },
    { 0xbe1061ec, 87 },
    { 0xf9a061ec, 33 },
    { 0x4f5061ec, 42 },
    { 0x60061ef, 34 },
    { 0xc61061f0, 78 },
    { 0x9f9061f1, 90 },
    { 0xcef061f2, 34 },
    { 0x4ba061f1, 90 },
    { 0, 0 },
    { 0xe03061f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886061ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4306203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79406206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36706209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9906210, 90 },
    { 0x4ab06211, 98 },
    { 0x31b06212, 23 },
    { 0x9bb06212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f06216, 81 },
    { 0x51406217, 90 },
    { 0xe7f06217, 23 },
    { 0, 0 },
    { 0xf670621a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4980621f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f906223, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d06228, 23 },
    { 0xe0706229, 77 },
    { 0x7160622a, 98 },
    { 0xb1d0622a, 98 },
    { 0x1060622a, 43 },
    { 0x420622c, 23 },
    { 0x2600622e, 90 },
    { 0x83d0622b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f06232, 61 },
    { 0, 0 },
    { 0xfd506234, 99 },
    { 0xae806234, 61 },
    { 0xab206234, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c106239, 23 },
    { 0x11506239, 23 },
    { 0xe6e0623b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a06240, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e506243, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d006246, 93 },
    { 0, 0 },
    { 0x89306248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5680624d, 76 },
    { 0xad50624e, 95 },
    { 0xfa10624f, 66 },
    { 0x50f0624e, 55 },
    { 0x7ef06250, 43 },
    { 0x1eb06251, 90 },
    { 0x5e20624e, 66 },
    { 0xfe60624d, 81 },
    { 0xd1506255, 81 },
    { 0xdea06250, 55 },
    { 0xb1d06251, 23 },
    { 0, 0 },
    { 0x79806259, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320625c, 99 },
    { 0x5270625d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c06260, 90 },
    { 0, 0 },
    { 0xe2606262, 90 },
    { 0xd7706263, 61 },
    { 0xd4806264, 85 },
    { 0xbf606265, 85 },
    { 0x8f06266, 34 },
    { 0xc8b06267, 96 },
    { 0x17a06265, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0626d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d06271, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9706274, 33 },
    { 0, 0 },
    { 0x4ed06276, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80627d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7806283, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f606286, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0628a, 77 },
    { 0, 0 },
    { 0x3060628c, 90 },
    { 0xa2a0628d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c06293, 90 },
    { 0, 0 },
    { 0xf8806295, 55 },
    { 0, 0 },
    { 0x96d06297, 77 },
    { 0, 0 },
    { 0x9ff06299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f0629c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790629f, 100 },
    { 0x9b6062a0, 33 },
    { 0x41c062a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52062a5, 99 },
    { 0, 0 },
    { 0xcc6062a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321062ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1062b5, 98 },
    { 0x77d062b5, 81 },
    { 0x5b0062b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee062bd, 61 },
    { 0xd98062bd, 98 },
    { 0x1d7062bf, 98 },
    { 0, 0 },
    { 0x363062c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8062c4, 95 },
    { 0x6f7062c4, 33 },
    { 0, 0 },
    { 0x15a062c7, 78 },
    { 0x6e6062c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27062d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4062d6, 76 },
    { 0x2dc062d7, 52 },
    { 0xb4a062d7, 76 },
    { 0, 0 },
    { 0x1a0062da, 36 },
    { 0x942062da, 46 },
    { 0xe0d062dc, 98 },
    { 0xef6062dd, 90 },
    { 0x1da062dd, 55 },
    { 0x3f9062df, 98 },
    { 0x5bc062e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45062e3, 84 },
    { 0x572062e4, 90 },
    { 0xe0062e5, 38 },
    { 0xe34062e5, 66 },
    { 0x8a8062e6, 87 },
    { 0x598062e5, 90 },
    { 0x873062e9, 61 },
    { 0x665062e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6062f0, 76 },
    { 0xcc9062f0, 98 },
    { 0xa7e062f2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e062f6, 23 },
    { 0x7f4062f7, 93 },
    { 0xb9d062f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x758062fb, 61 },
    { 0x3b8062fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37606300, 95 },
    { 0x6e106301, 90 },
    { 0, 0 },
    { 0xaef06303, 87 },
    { 0x18206304, 88 },
    { 0x3ea06304, 87 },
    { 0, 0 },
    { 0xd1d06307, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb0630e, 90 },
    { 0x26a0630f, 90 },
    { 0xa490630e, 33 },
    { 0x8706310, 98 },
    { 0xd0206312, 98 },
    { 0x12806313, 95 },
    { 0xbf406313, 98 },
    { 0xdeb0630e, 33 },
    { 0, 0 },
    { 0xa3006317, 90 },
    { 0, 0 },
    { 0xc8906319, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5760631f, 95 },
    { 0, 0 },
    { 0x99706321, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c0632a, 33 },
    { 0xf5f0632b, 61 },
    { 0x8400632c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c06332, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa06336, 77 },
    { 0x3f606337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d0633a, 66 },
    { 0, 0 },
    { 0x11d0633c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc06340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76906344, 38 },
    { 0xcb906344, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b06348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb050634b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb660634f, 52 },
    { 0, 0 },
    { 0x9b406351, 23 },
    { 0, 0 },
    { 0xf4f06353, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f06357, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0635a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b06362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1606365, 34 },
    { 0xf8d06365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec906374, 40 },
    { 0x89206375, 100 },
    { 0x57106376, 49 },
    { 0xf9806377, 23 },
    { 0xff806378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a0637c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b06381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5706388, 90 },
    { 0x6b706389, 94 },
    { 0, 0 },
    { 0xbac0638b, 98 },
    { 0, 0 },
    { 0x7240638d, 90 },
    { 0, 0 },
    { 0xa570638f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d106398, 99 },
    { 0x55a06399, 76 },
    { 0, 0 },
    { 0xd830639b, 84 },
    { 0, 0 },
    { 0xbee0639d, 98 },
    { 0x7de0639e, 97 },
    { 0, 0 },
    { 0xe063a0, 69 },
    { 0x2e8063a0, 33 },
    { 0xdcc063a0, 67 },
    { 0x187063a3, 43 },
    { 0x654063a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x381063a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50063aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4063af, 23 },
    { 0, 0 },
    { 0xf37063b1, 93 },
    { 0xf29063b1, 90 },
    { 0, 0 },
    { 0xb64063b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77063b7, 46 },
    { 0, 0 },
    { 0x792063b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c063bc, 34 },
    { 0, 0 },
    { 0xb23063be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4063c5, 55 },
    { 0, 0 },
    { 0x11f063c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17063cd, 33 },
    { 0x272063cd, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x751063d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28063d4, 95 },
    { 0x9ca063d4, 61 },
    { 0xfca063d6, 90 },
    { 0xc4a063d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e063dc, 90 },
    { 0, 0 },
    { 0x229063de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634063e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6063e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd063ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802063f3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675063f7, 33 },
    { 0xf67063f8, 95 },
    { 0xbc063f9, 33 },
    { 0xa5063fa, 33 },
    { 0, 0 },
    { 0x8fe063fc, 46 },
    { 0xd04063fd, 23 },
    { 0xf83063fd, 80 },
    { 0xb2c063ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9006406, 68 },
    { 0x3ba06407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540640a, 33 },
    { 0x4900640b, 61 },
    { 0x5150640c, 76 },
    { 0xd4d0640b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b206416, 83 },
    { 0xf7306416, 33 },
    { 0x22306416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe980641c, 33 },
    { 0x9230641d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac606422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac06425, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d206428, 61 },
    { 0x2bd06428, 33 },
    { 0x1b10642a, 76 },
    { 0x89a0642a, 95 },
    { 0xe360642b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd410642f, 95 },
    { 0xf9c06430, 66 },
    { 0x21e06430, 98 },
    { 0xc1906432, 55 },
    { 0xc4906432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75706438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b06443, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06446, 91 },
    { 0x7af06446, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3650644d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa006452, 91 },
    { 0, 0 },
    { 0xa0606454, 77 },
    { 0xd1d06455, 90 },
    { 0x6be06456, 55 },
    { 0, 0 },
    { 0xad906458, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0645e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9006462, 66 },
    { 0x23906463, 90 },
    { 0, 0 },
    { 0x5b006465, 56 },
    { 0x6c006466, 91 },
    { 0x33f06465, 90 },
    { 0x5006468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f0646b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46006471, 76 },
    { 0x89206471, 33 },
    { 0x3ef06473, 95 },
    { 0x71d06474, 61 },
    { 0x4c106474, 46 },
    { 0x13d06475, 90 },
    { 0x44b06473, 33 },
    { 0xa5006478, 95 },
    { 0xa6006478, 67 },
    { 0x59a0647a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0647e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24706481, 33 },
    { 0x48a06481, 83 },
    { 0x2cb06483, 90 },
    { 0x4ea06483, 46 },
    { 0x37706485, 61 },
    { 0x5a906486, 98 },
    { 0xaf106486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a06491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba06495, 99 },
    { 0xa7306496, 95 },
    { 0, 0 },
    { 0x83f06498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb0649b, 87 },
    { 0x3120649c, 76 },
    { 0, 0 },
    { 0xd7d0649e, 23 },
    { 0xeaf0649e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d064a5, 34 },
    { 0x6d3064a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48064a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a064b0, 90 },
    { 0xeaf064b1, 61 },
    { 0xd6d064b0, 83 },
    { 0, 0 },
    { 0x8f1064b4, 90 },
    { 0x4c5064b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc064bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee064c1, 33 },
    { 0x8e9064c2, 93 },
    { 0xc2d064c2, 87 },
    { 0x166064c1, 76 },
    { 0x26a064c5, 34 },
    { 0xc064c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea064cb, 84 },
    { 0x6db064cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631064d3, 61 },
    { 0xa65064d4, 61 },
    { 0x60a064d3, 43 },
    { 0x870064d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288064dc, 87 },
    { 0x279064dc, 95 },
    { 0, 0 },
    { 0xafa064df, 80 },
    { 0xc95064df, 90 },
    { 0, 0 },
    { 0x17d064e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6064e7, 34 },
    { 0x961064e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904064ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39064f2, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73064f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b064fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x343064fe, 76 },
    { 0x265064ff, 43 },
    { 0x89c06500, 77 },
    { 0, 0 },
    { 0x28906502, 78 },
    { 0x4da06503, 95 },
    { 0x2cd06504, 98 },
    { 0xe0806504, 55 },
    { 0, 0 },
    { 0xf0606507, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b40650b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1310650f, 87 },
    { 0x3706510, 23 },
    { 0, 0 },
    { 0x8d606512, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3906515, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d306519, 33 },
    { 0xfbb0651a, 87 },
    { 0xcb40651b, 66 },
    { 0xdd80651c, 91 },
    { 0xd100651a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e506520, 33 },
    { 0xd2e06521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32306524, 100 },
    { 0, 0 },
    { 0xb1706526, 95 },
    { 0xfd606527, 33 },
    { 0x30006527, 61 },
    { 0, 0 },
    { 0xc230652a, 98 },
    { 0, 0 },
    { 0xae60652c, 90 },
    { 0x8a00652c, 33 },
    { 0xa5e0652e, 61 },
    { 0, 0 },
    { 0xb7006530, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15906535, 99 },
    { 0, 0 },
    { 0xa0706537, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb440653a, 75 },
    { 0xff40653b, 67 },
    { 0, 0 },
    { 0x9180653d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06540, 33 },
    { 0, 0 },
    { 0x91c06542, 93 },
    { 0, 0 },
    { 0x7d206544, 40 },
    { 0xe8506545, 33 },
    { 0x50606545, 90 },
    { 0x62f06547, 98 },
    { 0xefb06548, 34 },
    { 0xf1006549, 87 },
    { 0xa5c06549, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff0654d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b106550, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf0655a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90655d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106561, 61 },
    { 0x69e06562, 66 },
    { 0xf5b06563, 23 },
    { 0xc6806564, 61 },
    { 0x6c006565, 23 },
    { 0x3bf06565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f06569, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb0656c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5240656f, 61 },
    { 0, 0 },
    { 0x60206571, 95 },
    { 0xf2b06572, 95 },
    { 0x86306573, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9006576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c0657b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca06581, 55 },
    { 0xc5106582, 32 },
    { 0, 0 },
    { 0xc4006584, 34 },
    { 0xbae06585, 52 },
    { 0xb0906585, 76 },
    { 0x8be06587, 76 },
    { 0x80a06587, 34 },
    { 0, 0 },
    { 0x2d60658a, 52 },
    { 0, 0 },
    { 0x7d0658c, 61 },
    { 0xa1f0658c, 33 },
    { 0x9130658e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b306594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2606598, 85 },
    { 0x66206598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0659e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba065a2, 98 },
    { 0, 0 },
    { 0x506065a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166065a9, 33 },
    { 0xc4b065aa, 46 },
    { 0, 0 },
    { 0x1a065ac, 76 },
    { 0, 0 },
    { 0xabf065ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2065b3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b065b6, 95 },
    { 0x7f8065b7, 86 },
    { 0x968065b6, 95 },
    { 0, 0 },
    { 0x6c1065ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e065be, 33 },
    { 0, 0 },
    { 0xef9065c0, 90 },
    { 0x335065c1, 98 },
    { 0x327065c1, 43 },
    { 0xb1c065c1, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799065ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378065d0, 33 },
    { 0, 0 },
    { 0x29a065d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43065d6, 90 },
    { 0, 0 },
    { 0x8bf065d8, 66 },
    { 0, 0 },
    { 0xd55065da, 56 },
    { 0xd9e065db, 81 },
    { 0, 0 },
    { 0x354065dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac065e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7065e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28065e6, 23 },
    { 0xf4b065e6, 80 },
    { 0x328065e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47065ee, 33 },
    { 0xbde065ee, 76 },
    { 0x964065f0, 23 },
    { 0x3f2065ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6065f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777065fa, 87 },
    { 0, 0 },
    { 0xe6f065fc, 61 },
    { 0x565065fd, 43 },
    { 0xf76065fe, 66 },
    { 0, 0 },
    { 0x3b006600, 75 },
    { 0, 0 },
    { 0x8706602, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88906607, 95 },
    { 0, 0 },
    { 0x86206609, 90 },
    { 0x5170660a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f006611, 33 },
    { 0xb7306612, 33 },
    { 0, 0 },
    { 0xe9a06614, 90 },
    { 0xfa906615, 92 },
    { 0x19906616, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d906619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4710661c, 90 },
    { 0xed60661c, 90 },
    { 0x1220661c, 51 },
    { 0, 0 },
    { 0x98d06620, 91 },
    { 0xef706621, 33 },
    { 0xd7906622, 90 },
    { 0x9e06623, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d006627, 66 },
    { 0xb4c06628, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc0662c, 98 },
    { 0, 0 },
    { 0x2ab0662e, 52 },
    { 0x60b0662f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f006634, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54506637, 66 },
    { 0x62406637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb90663b, 33 },
    { 0xd0c0663c, 33 },
    { 0x9310663d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0664b, 95 },
    { 0x1970664c, 66 },
    { 0xf840664d, 97 },
    { 0, 0 },
    { 0x4910664f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc806654, 61 },
    { 0xb2c06655, 78 },
    { 0x56606655, 34 },
    { 0xc8206654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5970665c, 98 },
    { 0xcde0665d, 65 },
    { 0, 0 },
    { 0x4050665f, 76 },
    { 0, 0 },
    { 0x10e06661, 23 },
    { 0xca906662, 61 },
    { 0x50c06661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d06666, 33 },
    { 0x34106667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d40666a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d506676, 91 },
    { 0x2af06677, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0667b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa840667e, 78 },
    { 0, 0 },
    { 0xd7906680, 95 },
    { 0, 0 },
    { 0xcf06682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2506688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50668b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e06693, 91 },
    { 0x9d106693, 33 },
    { 0, 0 },
    { 0xceb06696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c60669a, 61 },
    { 0, 0 },
    { 0xe120669c, 90 },
    { 0, 0 },
    { 0x48c0669e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33066a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47066aa, 85 },
    { 0xe5b066aa, 61 },
    { 0x865066ac, 61 },
    { 0xba4066ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6066b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28066b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa066ba, 67 },
    { 0x6b8066bb, 33 },
    { 0x3e0066ba, 33 },
    { 0x9ea066bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0066c9, 33 },
    { 0x7d9066ca, 95 },
    { 0x33a066ca, 52 },
    { 0x2f7066c9, 90 },
    { 0xe47066cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea066d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38066d6, 98 },
    { 0x260066d7, 93 },
    { 0x56f066d7, 33 },
    { 0, 0 },
    { 0x19d066da, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0066e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d066ec, 52 },
    { 0xffc066ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24066f0, 98 },
    { 0x27b066f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a066f6, 23 },
    { 0x1ce066f7, 34 },
    { 0x557066f8, 23 },
    { 0x45f066f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8066ff, 25 },
    { 0x28006700, 33 },
    { 0xa4d06700, 91 },
    { 0xae306702, 84 },
    { 0x4406702, 98 },
    { 0x70406700, 61 },
    { 0xd9a06702, 61 },
    { 0, 0 },
    { 0x32806707, 23 },
    { 0, 0 },
    { 0x48806709, 61 },
    { 0xc9506709, 90 },
    { 0x5330670b, 61 },
    { 0x77606709, 91 },
    { 0, 0 },
    { 0x5490670e, 90 },
    { 0x69e0670f, 75 },
    { 0x42806710, 33 },
    { 0x9f006711, 40 },
    { 0, 0 },
    { 0xddd06713, 91 },
    { 0xb8006714, 23 },
    { 0x34206714, 93 },
    { 0xbd206715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640671a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d06722, 52 },
    { 0x71306723, 61 },
    { 0x5b006724, 43 },
    { 0x92106725, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520672d, 33 },
    { 0, 0 },
    { 0x6c80672f, 33 },
    { 0x2850672f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6706733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f106736, 33 },
    { 0x52906737, 17 },
    { 0xb6d06737, 61 },
    { 0xe8906739, 90 },
    { 0xefe06737, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b306741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f106744, 98 },
    { 0x68906745, 90 },
    { 0xeb306745, 61 },
    { 0x7eb06747, 46 },
    { 0x3706748, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15006753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49906757, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac0675a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df0675d, 90 },
    { 0xe100675e, 90 },
    { 0x8de0675f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b06762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe206768, 23 },
    { 0, 0 },
    { 0xca50676a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360676f, 90 },
    { 0x8980676f, 56 },
    { 0x5d706771, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae806777, 76 },
    { 0x6dd06778, 90 },
    { 0x30306778, 76 },
    { 0xb4306779, 61 },
    { 0xf6d0677b, 77 },
    { 0xabf0677c, 61 },
    { 0x45c0677b, 90 },
    { 0, 0 },
    { 0x2e40677f, 84 },
    { 0, 0 },
    { 0x3bb06781, 98 },
    { 0, 0 },
    { 0x1e006783, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a0678d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a106791, 76 },
    { 0, 0 },
    { 0x7d206793, 33 },
    { 0xec906794, 91 },
    { 0xc4106795, 66 },
    { 0xd6406795, 90 },
    { 0xd406795, 100 },
    { 0, 0 },
    { 0xad706799, 61 },
    { 0, 0 },
    { 0x2360679b, 33 },
    { 0, 0 },
    { 0x1800679d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f067a1, 76 },
    { 0, 0 },
    { 0x17f067a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x541067a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27067a9, 33 },
    { 0x4067aa, 33 },
    { 0x8f6067aa, 85 },
    { 0x10c067ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x164067af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2067b6, 33 },
    { 0xd8f067b7, 61 },
    { 0x187067b7, 99 },
    { 0, 0 },
    { 0x622067ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82067bd, 90 },
    { 0x158067bd, 33 },
    { 0x3e2067bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80067c3, 33 },
    { 0, 0 },
    { 0xa61067c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x901067c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2067ce, 33 },
    { 0x138067cf, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef067d2, 33 },
    { 0x868067d3, 52 },
    { 0, 0 },
    { 0xe7067d5, 90 },
    { 0x353067d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x341067d9, 52 },
    { 0, 0 },
    { 0xf5b067db, 90 },
    { 0, 0 },
    { 0xa71067dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9067e4, 98 },
    { 0x214067e5, 23 },
    { 0x4fc067e6, 61 },
    { 0, 0 },
    { 0x4f5067e8, 76 },
    { 0xaf7067e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e067ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c067f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c067f7, 76 },
    { 0xff6067f8, 95 },
    { 0x2ef067f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c706800, 33 },
    { 0x8b506800, 43 },
    { 0xd8006802, 91 },
    { 0, 0 },
    { 0x42806804, 76 },
    { 0, 0 },
    { 0x5ea06806, 23 },
    { 0x71006807, 81 },
    { 0xe8c06808, 80 },
    { 0, 0 },
    { 0xc640680a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb80680d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f06814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9406818, 91 },
    { 0x2c306818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac0681c, 75 },
    { 0, 0 },
    { 0xf2f0681e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf306822, 98 },
    { 0xea206823, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42606827, 34 },
    { 0x98606827, 98 },
    { 0x1206827, 98 },
    { 0, 0 },
    { 0x7d70682b, 43 },
    { 0, 0 },
    { 0x39d0682d, 78 },
    { 0xb8f0682e, 78 },
    { 0x52a0682e, 65 },
    { 0x16c06830, 76 },
    { 0xd3e06831, 100 },
    { 0x43b06832, 66 },
    { 0xd506832, 33 },
    { 0x88a06834, 91 },
    { 0x88706835, 95 },
    { 0x6b806836, 61 },
    { 0x4c10682e, 89 },
    { 0, 0 },
    { 0xcfe06839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b0683f, 98 },
    { 0x47506840, 76 },
    { 0x13006841, 84 },
    { 0xa4706840, 90 },
    { 0x2fb06843, 61 },
    { 0x5406842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0684b, 33 },
    { 0, 0 },
    { 0xb6c0684d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f306852, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a06858, 55 },
    { 0x96606859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c706861, 61 },
    { 0xf3606861, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83906867, 90 },
    { 0xcbd06868, 87 },
    { 0xa6906868, 52 },
    { 0xc550686a, 33 },
    { 0x9810686b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730686e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80106871, 93 },
    { 0, 0 },
    { 0x40106873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff806876, 78 },
    { 0x8aa06877, 61 },
    { 0x49e06876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3460687f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a06884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93806888, 46 },
    { 0, 0 },
    { 0x56d0688a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe706892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4106896, 76 },
    { 0x17806896, 66 },
    { 0x1f06896, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d0689c, 90 },
    { 0x3930689c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35068a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53068a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5068ae, 23 },
    { 0x103068ae, 90 },
    { 0xa56068ae, 55 },
    { 0x995068b1, 100 },
    { 0x2a5068b2, 61 },
    { 0x72d068b3, 99 },
    { 0xdc7068b3, 52 },
    { 0, 0 },
    { 0x34e068b6, 36 },
    { 0, 0 },
    { 0x827068b8, 43 },
    { 0x444068b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f068bc, 33 },
    { 0xa58068bd, 90 },
    { 0x641068be, 84 },
    { 0xda9068bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58068c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab068c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf068ca, 23 },
    { 0, 0 },
    { 0x19c068cc, 33 },
    { 0x389068cd, 55 },
    { 0, 0 },
    { 0xba4068cf, 99 },
    { 0x995068d0, 75 },
    { 0xef3068d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6068d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4068d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x227068da, 46 },
    { 0x3f2068da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1068df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86068e2, 76 },
    { 0xd71068e2, 33 },
    { 0xbfe068e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4068e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81068eb, 61 },
    { 0xf23068ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x453068ef, 90 },
    { 0, 0 },
    { 0x672068f1, 90 },
    { 0x16068f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60068f5, 55 },
    { 0xbcf068f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617068fc, 91 },
    { 0x15a068fc, 33 },
    { 0x15e068fe, 61 },
    { 0xdd068ff, 52 },
    { 0x60006900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51106907, 61 },
    { 0xf506908, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60690d, 91 },
    { 0xab10690e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b606915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9860691a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf06921, 99 },
    { 0, 0 },
    { 0x27306923, 52 },
    { 0, 0 },
    { 0x4e306925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9406928, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9040692f, 95 },
    { 0x7a06930, 23 },
    { 0xd2306931, 33 },
    { 0xaf306932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef206936, 98 },
    { 0xac706936, 23 },
    { 0x4d906938, 23 },
    { 0x24506939, 85 },
    { 0, 0 },
    { 0x99a0693b, 33 },
    { 0, 0 },
    { 0x8010693d, 95 },
    { 0x5ad0693e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea06942, 55 },
    { 0, 0 },
    { 0x19206944, 90 },
    { 0x20206944, 90 },
    { 0, 0 },
    { 0x87a06947, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf10694a, 33 },
    { 0xbca0694a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13906954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c806958, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc290695c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b06965, 33 },
    { 0, 0 },
    { 0xbcd06967, 76 },
    { 0, 0 },
    { 0xa1006969, 76 },
    { 0x1190696a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb06973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50697a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb220697f, 64 },
    { 0, 0 },
    { 0x3906981, 34 },
    { 0x87806982, 81 },
    { 0xa2306983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe06987, 55 },
    { 0x4e006988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe0698b, 76 },
    { 0, 0 },
    { 0x4600698d, 99 },
    { 0, 0 },
    { 0xe040698f, 98 },
    { 0x4260698f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d06998, 33 },
    { 0x47d06999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd700699e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541069a2, 23 },
    { 0, 0 },
    { 0x5e8069a4, 33 },
    { 0, 0 },
    { 0x7c8069a6, 61 },
    { 0x7c6069a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658069ab, 68 },
    { 0xb28069ab, 98 },
    { 0x201069ad, 61 },
    { 0x753069ac, 78 },
    { 0x798069ac, 61 },
    { 0x16d069b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be069b4, 43 },
    { 0, 0 },
    { 0x8af069b6, 90 },
    { 0x688069b7, 46 },
    { 0, 0 },
    { 0x6b9069b9, 95 },
    { 0x42f069ba, 98 },
    { 0, 0 },
    { 0xa2c069bc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15069c2, 55 },
    { 0, 0 },
    { 0xa74069c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa069cb, 90 },
    { 0x226069cc, 34 },
    { 0x9fd069cc, 33 },
    { 0xe2d069ce, 90 },
    { 0xf49069cf, 76 },
    { 0xaf1069d0, 77 },
    { 0xba8069ce, 99 },
    { 0x625069cb, 61 },
    { 0x621069cc, 98 },
    { 0x226069d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc069de, 95 },
    { 0xb16069de, 99 },
    { 0, 0 },
    { 0xfdb069e1, 61 },
    { 0x3a0069e2, 26 },
    { 0, 0 },
    { 0xb70069e4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d069e8, 33 },
    { 0x2b0069e9, 46 },
    { 0xe06069ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760069ee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56069fb, 98 },
    { 0xaae069fb, 33 },
    { 0x9f6069fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed06a06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa06a09, 98 },
    { 0x91f06a0a, 99 },
    { 0xd0306a0b, 98 },
    { 0x44706a0a, 43 },
    { 0, 0 },
    { 0x35f06a0e, 98 },
    { 0x80e06a0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d06a13, 55 },
    { 0x16f06a13, 33 },
    { 0x5a006a15, 17 },
    { 0xf5d06a14, 100 },
    { 0, 0 },
    { 0xa5e06a18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d06a1c, 90 },
    { 0, 0 },
    { 0x92606a1e, 90 },
    { 0xff806a1f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd06a24, 66 },
    { 0x14306a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa106a28, 78 },
    { 0xc3506a28, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a06a2d, 55 },
    { 0, 0 },
    { 0x2e706a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63606a32, 95 },
    { 0, 0 },
    { 0xbd706a34, 95 },
    { 0, 0 },
    { 0xeee06a36, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d06a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb06a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c806a4b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88806a4f, 91 },
    { 0, 0 },
    { 0x1bc06a51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe006a54, 33 },
    { 0, 0 },
    { 0x20f06a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8206a5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe306a5d, 33 },
    { 0, 0 },
    { 0x2c606a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f06a63, 23 },
    { 0x1f606a63, 43 },
    { 0x61906a65, 23 },
    { 0x85306a65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96606a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae406a72, 98 },
    { 0, 0 },
    { 0x61506a74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f606a77, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b06a7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75206a80, 90 },
    { 0x12906a80, 55 },
    { 0xfbd06a81, 34 },
    { 0xeb906a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43206a88, 100 },
    { 0, 0 },
    { 0x1b306a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46606a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3006aa5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99506aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd306ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1306ab5, 75 },
    { 0, 0 },
    { 0x88806ab7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1806aba, 96 },
    { 0x9c206abb, 33 },
    { 0xe4e06abc, 76 },
    { 0x65806abc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82506ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a06ac8, 95 },
    { 0x48606ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c906acc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d06acf, 77 },
    { 0x18706acf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de06ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b06add, 23 },
    { 0xa8106ade, 38 },
    { 0x4be06adf, 33 },
    { 0x80006add, 98 },
    { 0, 0 },
    { 0xf5506ae2, 46 },
    { 0, 0 },
    { 0x52c06ae4, 42 },
    { 0x6e06ae5, 90 },
    { 0xea906ae4, 75 },
    { 0xd6806ae7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3906aec, 68 },
    { 0xc06aed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d406af4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f06afc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f06b01, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0606b05, 23 },
    { 0x5ec06b06, 23 },
    { 0xea006b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e06b0c, 95 },
    { 0x1f006b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f06b14, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d06b1c, 85 },
    { 0xe7506b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f106b20, 93 },
    { 0xd7d06b20, 34 },
    { 0x38e06b22, 65 },
    { 0, 0 },
    { 0xaea06b24, 90 },
    { 0x1ef06b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e06b29, 98 },
    { 0x62b06b2a, 98 },
    { 0x7a506b2b, 90 },
    { 0, 0 },
    { 0x65106b2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73206b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f06b35, 33 },
    { 0xa5d06b36, 95 },
    { 0, 0 },
    { 0x72606b38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06b41, 33 },
    { 0xe5d06b42, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4706b47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe406b4a, 61 },
    { 0x9ce06b4b, 78 },
    { 0xd3906b4c, 91 },
    { 0x1a906b4d, 76 },
    { 0xd0a06b4a, 61 },
    { 0, 0 },
    { 0x54106b50, 43 },
    { 0, 0 },
    { 0xde706b52, 61 },
    { 0x49206b53, 68 },
    { 0x8306b54, 23 },
    { 0x4a106b53, 100 },
    { 0, 0 },
    { 0x34906b57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e406b5a, 23 },
    { 0x46f06b5b, 55 },
    { 0xe0606b5c, 61 },
    { 0x93906b5d, 23 },
    { 0x7e06b5d, 95 },
    { 0x65b06b5c, 90 },
    { 0, 0 },
    { 0xc0106b61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af06b6e, 46 },
    { 0x6d06b6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1006b76, 93 },
    { 0x80c06b77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb906b7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f706b7f, 95 },
    { 0xa3906b80, 61 },
    { 0xe1d06b81, 98 },
    { 0xf5a06b81, 87 },
    { 0x7be06b81, 98 },
    { 0xa5e06b81, 33 },
    { 0x38b06b82, 87 },
    { 0x77c06b83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf606b8b, 61 },
    { 0, 0 },
    { 0x9e906b8d, 61 },
    { 0, 0 },
    { 0x14106b8f, 33 },
    { 0, 0 },
    { 0x60806b91, 61 },
    { 0, 0 },
    { 0x3c306b93, 90 },
    { 0, 0 },
    { 0x1206b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa06b98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f06b9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9506ba8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3006bab, 99 },
    { 0, 0 },
    { 0x50d06bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d06bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe406bb4, 66 },
    { 0xade06bb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d06bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17206bbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff06bbf, 95 },
    { 0x2506bc0, 76 },
    { 0, 0 },
    { 0x4fb06bc2, 98 },
    { 0, 0 },
    { 0xae506bc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54906bc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc06bcb, 33 },
    { 0xbfe06bcb, 23 },
    { 0xf9606bcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf06bd9, 33 },
    { 0x4506bd9, 91 },
    { 0x45006bdb, 98 },
    { 0x39b06bda, 55 },
    { 0x64e06bdd, 95 },
    { 0xc1106bde, 23 },
    { 0x80a06bde, 23 },
    { 0, 0 },
    { 0x36706be1, 23 },
    { 0x27506be1, 90 },
    { 0xff006be3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4106be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29706bec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6506bf9, 61 },
    { 0x7db06bfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b06c01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b06c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f06c0c, 90 },
    { 0xeba06c0d, 91 },
    { 0xd3606c0d, 91 },
    { 0, 0 },
    { 0x89a06c10, 33 },
    { 0xea606c11, 84 },
    { 0x20e06c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60906c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf806c1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c306c1f, 100 },
    { 0x5fc06c1f, 98 },
    { 0xa2806c21, 43 },
    { 0x51d06c20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97306c27, 22 },
    { 0x2e506c27, 23 },
    { 0xbf506c29, 76 },
    { 0x8ce06c29, 77 },
    { 0xa3306c2a, 56 },
    { 0xaa806c27, 43 },
    { 0x43b06c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc06c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb206c35, 91 },
    { 0x91e06c35, 67 },
    { 0x47c06c36, 34 },
    { 0xbf606c35, 61 },
    { 0x24f06c35, 61 },
    { 0xb6d06c3a, 95 },
    { 0x2d206c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac06c42, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c906c49, 33 },
    { 0x8a006c4a, 76 },
    { 0x10006c4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92506c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad906c54, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de06c58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f406c5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac806c5e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8506c61, 76 },
    { 0xb6106c62, 23 },
    { 0x16506c63, 34 },
    { 0x6dd06c64, 23 },
    { 0x3f06c65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda406c69, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99706c6f, 52 },
    { 0, 0 },
    { 0xd06c71, 78 },
    { 0x94f06c72, 33 },
    { 0xf0606c73, 85 },
    { 0, 0 },
    { 0x5a306c75, 61 },
    { 0x20806c76, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b606c79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec006c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40706c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb06c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a206c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b106c92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2606c95, 98 },
    { 0x45506c95, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a606c99, 90 },
    { 0x40a06c99, 52 },
    { 0, 0 },
    { 0x6e506c9c, 98 },
    { 0x74206c9d, 43 },
    { 0x2cd06c9e, 91 },
    { 0xf7506c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80406ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b506ca5, 90 },
    { 0x5c706ca6, 95 },
    { 0xd8806ca6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f906cb2, 61 },
    { 0x85306cb3, 52 },
    { 0xa2b06cb4, 23 },
    { 0xd606cb4, 43 },
    { 0x2bc06cb6, 55 },
    { 0, 0 },
    { 0xd8706cb8, 95 },
    { 0xe8a06cb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45406cbf, 80 },
    { 0x63006cbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f506cc6, 61 },
    { 0x8e506cc6, 33 },
    { 0x25d06cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6806cd6, 33 },
    { 0xb7206cd7, 33 },
    { 0, 0 },
    { 0x93106cd9, 98 },
    { 0x71e06cda, 23 },
    { 0x54a06cdb, 46 },
    { 0, 0 },
    { 0xd4f06cdd, 77 },
    { 0x2ae06cde, 91 },
    { 0x1ac06cdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9106ce2, 33 },
    { 0x13c06ce3, 52 },
    { 0, 0 },
    { 0x64c06ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa506ce9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf06cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9406cf0, 10 },
    { 0x1d206cf0, 66 },
    { 0x42006cf2, 95 },
    { 0, 0 },
    { 0x58006cf4, 33 },
    { 0, 0 },
    { 0x13006cf6, 76 },
    { 0x27d06cf7, 95 },
    { 0x4ef06cf7, 33 },
    { 0x88606cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d06cfc, 33 },
    { 0, 0 },
    { 0xae506cfe, 93 },
    { 0, 0 },
    { 0x8606d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d06d03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69506d06, 61 },
    { 0, 0 },
    { 0x13106d08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5806d0c, 23 },
    { 0xb6d06d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2906d11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35506d18, 23 },
    { 0, 0 },
    { 0x69606d1a, 81 },
    { 0x87306d1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd706d24, 90 },
    { 0xfb606d25, 98 },
    { 0xf1906d26, 24 },
    { 0x63f06d27, 33 },
    { 0x74b06d28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e606d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d806d31, 55 },
    { 0xc8906d32, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d706d39, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x37706d3c, 23 },
    { 0x86206d3d, 44 },
    { 0xa1206d3e, 33 },
    { 0xaff06d3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39006d44, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c06d47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56406d4c, 43 },
    { 0x29206d4d, 33 },
    { 0, 0 },
    { 0x96006d4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93206d56, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a906d5d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c06d61, 17 },
    { 0, 0 },
    { 0xde806d63, 84 },
    { 0xb8906d64, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3206d6a, 87 },
    { 0, 0 },
    { 0x75106d6c, 95 },
    { 0xe8b06d6d, 52 },
    { 0xfb806d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a06d71, 23 },
    { 0x53b06d71, 95 },
    { 0x3bb06d73, 23 },
    { 0x14e06d72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a106d78, 34 },
    { 0x5cd06d79, 99 },
    { 0x9f106d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b806d7e, 43 },
    { 0, 0 },
    { 0x9c806d80, 33 },
    { 0xc0806d81, 88 },
    { 0, 0 },
    { 0x4d06d83, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c06d8a, 23 },
    { 0x1e706d8a, 77 },
    { 0, 0 },
    { 0x5b806d8d, 95 },
    { 0x35b06d8e, 66 },
    { 0, 0 },
    { 0x69d06d90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26506d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff06d9a, 23 },
    { 0xeac06d9a, 33 },
    { 0x55206d9c, 98 },
    { 0x9ba06d9d, 55 },
    { 0xa8d06d9e, 84 },
    { 0xc2406d9e, 61 },
    { 0xcef06da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc06da3, 93 },
    { 0x4aa06da3, 90 },
    { 0xe3a06da4, 98 },
    { 0, 0 },
    { 0x94f06da7, 95 },
    { 0x91906da8, 43 },
    { 0x90906da8, 95 },
    { 0xa8a06da8, 33 },
    { 0, 0 },
    { 0x68506dac, 33 },
    { 0x4f506dac, 51 },
    { 0xf906dae, 95 },
    { 0x2a506dae, 65 },
    { 0x29f06dae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a06db6, 98 },
    { 0x47306db7, 33 },
    { 0x21a06db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35406dbb, 90 },
    { 0, 0 },
    { 0x37706dbd, 90 },
    { 0xafb06dbd, 68 },
    { 0, 0 },
    { 0x78806dc0, 55 },
    { 0x68806dc0, 95 },
    { 0, 0 },
    { 0x73406dc3, 93 },
    { 0x28106dc3, 23 },
    { 0, 0 },
    { 0xcad06dc6, 90 },
    { 0, 0 },
    { 0x42006dc8, 66 },
    { 0x4bc06dc8, 61 },
    { 0x55b06dca, 40 },
    { 0x8b206dcb, 33 },
    { 0x87106dcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c806dd0, 98 },
    { 0x3fc06dd0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d606dd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1706dd7, 91 },
    { 0, 0 },
    { 0x64406dd9, 75 },
    { 0x88e06dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4506ddd, 76 },
    { 0xece06dde, 61 },
    { 0x8b506ddf, 95 },
    { 0, 0 },
    { 0xd5c06de1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac806de4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4006de9, 90 },
    { 0, 0 },
    { 0x7aa06deb, 17 },
    { 0, 0 },
    { 0xf4906ded, 95 },
    { 0xd5806ded, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa06df2, 100 },
    { 0x44b06df3, 98 },
    { 0, 0 },
    { 0x1b06df5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa06df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e06dfd, 55 },
    { 0, 0 },
    { 0xf7306dff, 76 },
    { 0xbcd06dff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b606e05, 90 },
    { 0x5a306e06, 96 },
    { 0xe0306e06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c606e0d, 83 },
    { 0xa2506e0e, 95 },
    { 0, 0 },
    { 0xb6f06e10, 61 },
    { 0, 0 },
    { 0x69906e12, 23 },
    { 0x3cf06e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a06e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e06e1a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45106e23, 100 },
    { 0, 0 },
    { 0x85906e25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7306e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1806e2b, 77 },
    { 0xec06e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81906e33, 61 },
    { 0, 0 },
    { 0x45106e35, 61 },
    { 0xe1b06e36, 98 },
    { 0xf5d06e35, 75 },
    { 0xa2b06e38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5406e3d, 65 },
    { 0x2b106e3e, 33 },
    { 0x49306e3f, 67 },
    { 0xce106e40, 34 },
    { 0xde406e3f, 43 },
    { 0, 0 },
    { 0xfeb06e43, 76 },
    { 0xaa06e44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06e47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab06e4d, 55 },
    { 0x77906e4d, 94 },
    { 0x9aa06e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e906e52, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda606e55, 76 },
    { 0xc6406e55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9706e5d, 76 },
    { 0xe7906e5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb06e62, 91 },
    { 0x2c506e62, 33 },
    { 0x51306e63, 38 },
    { 0x85506e65, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7706e69, 90 },
    { 0xe1c06e6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21506e73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee06e77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6206e82, 68 },
    { 0xbb106e83, 76 },
    { 0xfdc06e83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb906e87, 91 },
    { 0x25f06e87, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a06e93, 43 },
    { 0xb0a06e94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd606e98, 33 },
    { 0xa1606e98, 95 },
    { 0, 0 },
    { 0xa3706e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c406e9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36106ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a706ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2706ead, 23 },
    { 0xdbf06eae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59106eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23006eb4, 61 },
    { 0, 0 },
    { 0x5c406eb6, 61 },
    { 0, 0 },
    { 0xd1906eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda206ebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a06ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2706ec9, 40 },
    { 0xe9c06eca, 46 },
    { 0, 0 },
    { 0x3af06ecc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06ed2, 43 },
    { 0, 0 },
    { 0x45306ed4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe06ed7, 85 },
    { 0x54106ed7, 17 },
    { 0, 0 },
    { 0x4eb06eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd206edf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x17506ee2, 65 },
    { 0xdd906ee2, 53 },
    { 0x3906ee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d006ee8, 61 },
    { 0x3c06ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99506eec, 90 },
    { 0, 0 },
    { 0xf3b06eee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe106ef8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56906efe, 33 },
    { 0x38c06efe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f06f03, 55 },
    { 0xee206f04, 91 },
    { 0, 0 },
    { 0x9d06f06, 67 },
    { 0, 0 },
    { 0x93606f08, 76 },
    { 0, 0 },
    { 0x14906f0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x56706f0d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f06f11, 90 },
    { 0, 0 },
    { 0x43806f13, 90 },
    { 0x5e606f13, 33 },
    { 0, 0 },
    { 0xa0106f16, 76 },
    { 0, 0 },
    { 0x76706f18, 76 },
    { 0xeaf06f19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d06f1c, 90 },
    { 0x47106f1d, 33 },
    { 0x45b06f1e, 95 },
    { 0xbe406f1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17006f26, 88 },
    { 0, 0 },
    { 0x1a606f28, 33 },
    { 0, 0 },
    { 0x3fd06f2a, 61 },
    { 0x9ad06f2b, 90 },
    { 0x4ce06f2c, 61 },
    { 0xe3406f2d, 99 },
    { 0x5fb06f2d, 43 },
    { 0xd4206f2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d06f34, 92 },
    { 0, 0 },
    { 0x6ba06f36, 33 },
    { 0x57806f37, 84 },
    { 0x5e506f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc306f3b, 95 },
    { 0x2a506f3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a06f41, 76 },
    { 0x1dc06f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39206f47, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df06f4a, 95 },
    { 0xec06f4a, 33 },
    { 0x46c06f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e06f4f, 85 },
    { 0x10506f50, 90 },
    { 0x5b06f51, 61 },
    { 0xa5a06f51, 93 },
    { 0xaa106f53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6906f56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d506f5b, 46 },
    { 0x53a06f5c, 98 },
    { 0xbe206f5b, 33 },
    { 0x9bf06f5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b06f6a, 77 },
    { 0x6c406f6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73306f73, 48 },
    { 0x65006f74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c06f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce906f7c, 76 },
    { 0xbcf06f7d, 98 },
    { 0xc4506f7c, 99 },
    { 0x81106f7f, 61 },
    { 0x5cf06f80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7106f84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec06f87, 77 },
    { 0, 0 },
    { 0x31906f89, 98 },
    { 0, 0 },
    { 0x49406f8b, 95 },
    { 0, 0 },
    { 0x8ef06f8d, 90 },
    { 0, 0 },
    { 0xb7e06f8f, 76 },
    { 0xdce06f90, 91 },
    { 0x8106f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46906f99, 77 },
    { 0x75206f9a, 98 },
    { 0xfa106f99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3406f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c06fa6, 34 },
    { 0x98a06fa6, 95 },
    { 0, 0 },
    { 0x7b906fa9, 80 },
    { 0x76006fa9, 91 },
    { 0, 0 },
    { 0xdda06fac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee606fb1, 87 },
    { 0x27806fb2, 46 },
    { 0, 0 },
    { 0x47e06fb4, 59 },
    { 0xde406fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b406fbb, 83 },
    { 0x75e06fbc, 61 },
    { 0, 0 },
    { 0xdfe06fbe, 26 },
    { 0x4a06fbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35006fc5, 83 },
    { 0x92606fc5, 98 },
    { 0, 0 },
    { 0x30e06fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24306fd0, 100 },
    { 0xc7b06fd0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd06fd5, 95 },
    { 0, 0 },
    { 0xa3906fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5106fdb, 33 },
    { 0x23606fdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43406fdf, 68 },
    { 0, 0 },
    { 0x5f806fe1, 33 },
    { 0xc9e06fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a06fe5, 98 },
    { 0, 0 },
    { 0xc6206fe7, 90 },
    { 0x59406fe8, 98 },
    { 0x67106fe8, 93 },
    { 0x4d206fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e06fee, 61 },
    { 0, 0 },
    { 0x6ba06ff0, 34 },
    { 0, 0 },
    { 0x48f06ff2, 90 },
    { 0, 0 },
    { 0x30606ff4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d606ff7, 91 },
    { 0, 0 },
    { 0xb2006ff9, 91 },
    { 0xc8c06ff9, 61 },
    { 0, 0 },
    { 0xa9406ffc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd406fff, 90 },
    { 0x82707000, 90 },
    { 0, 0 },
    { 0xd5507002, 33 },
    { 0x12b07003, 99 },
    { 0x2c807004, 67 },
    { 0, 0 },
    { 0x83c07006, 87 },
    { 0, 0 },
    { 0x84507008, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3810700b, 34 },
    { 0x7eb0700b, 95 },
    { 0xb390700b, 61 },
    { 0x1b30700b, 95 },
    { 0x19f0700f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b0701d, 33 },
    { 0, 0 },
    { 0xe590701f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7807024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0702b, 84 },
    { 0, 0 },
    { 0xd660702d, 61 },
    { 0x1df0702e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77507036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0703a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7380703d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75307040, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2107044, 76 },
    { 0x84f07045, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20704a, 91 },
    { 0xc600704b, 43 },
    { 0xe7c0704c, 23 },
    { 0xcc30704d, 90 },
    { 0x42a0704e, 61 },
    { 0x7920704e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd07052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f07055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00705d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4207060, 50 },
    { 0, 0 },
    { 0x21f07062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19307068, 55 },
    { 0x6b807069, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf30706c, 90 },
    { 0, 0 },
    { 0x47c0706e, 33 },
    { 0x8890706f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e07074, 84 },
    { 0x1c807075, 95 },
    { 0xad607076, 90 },
    { 0x20b07077, 98 },
    { 0x6e207074, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a0707b, 92 },
    { 0xed80707c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83507080, 76 },
    { 0xf1007080, 95 },
    { 0x27b07080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e007086, 95 },
    { 0xfe507087, 55 },
    { 0x4807087, 23 },
    { 0xebf07088, 76 },
    { 0xcec0708a, 90 },
    { 0x55a07087, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2290708e, 61 },
    { 0, 0 },
    { 0x55207090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3607094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca607099, 55 },
    { 0x8260709a, 98 },
    { 0x8920709a, 90 },
    { 0x2bb0709a, 23 },
    { 0x3c60709d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806070a8, 95 },
    { 0x8b1070a9, 98 },
    { 0x7d4070aa, 65 },
    { 0x874070ab, 61 },
    { 0xc37070ab, 23 },
    { 0x135070ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x661070b0, 33 },
    { 0, 0 },
    { 0x379070b2, 98 },
    { 0, 0 },
    { 0x1d2070b4, 34 },
    { 0xb70070b5, 98 },
    { 0x89d070b5, 33 },
    { 0x6fc070b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798070bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84070c4, 99 },
    { 0x797070c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956070ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af070d1, 87 },
    { 0x963070d2, 95 },
    { 0, 0 },
    { 0xbb5070d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0070d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd070dd, 33 },
    { 0xa8070dd, 61 },
    { 0, 0 },
    { 0x823070e0, 43 },
    { 0, 0 },
    { 0x529070e2, 33 },
    { 0x190070e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0070e7, 53 },
    { 0x3ea070e8, 33 },
    { 0xb28070e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624070ef, 99 },
    { 0xa07070ef, 91 },
    { 0xddf070ef, 95 },
    { 0xe070f1, 90 },
    { 0x2b3070f3, 46 },
    { 0x174070f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79070f8, 98 },
    { 0, 0 },
    { 0x150070fa, 100 },
    { 0xc61070fa, 23 },
    { 0xcc3070fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x544070ff, 23 },
    { 0xa0307100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba07104, 95 },
    { 0, 0 },
    { 0xf1607106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90710d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56807119, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc440711e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x24807121, 33 },
    { 0x22a07122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c07126, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2007131, 78 },
    { 0, 0 },
    { 0xaa407133, 95 },
    { 0, 0 },
    { 0xdb307135, 23 },
    { 0x34007136, 52 },
    { 0x46d07135, 61 },
    { 0x67e07138, 43 },
    { 0, 0 },
    { 0xc460713a, 90 },
    { 0, 0 },
    { 0xb110713c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc40713f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d07142, 84 },
    { 0xa1007143, 99 },
    { 0xd9207144, 61 },
    { 0x2a507143, 33 },
    { 0x76407143, 90 },
    { 0x99d07147, 61 },
    { 0x8c207143, 90 },
    { 0xf7c07144, 90 },
    { 0xb6c0714a, 55 },
    { 0xcfc0714a, 93 },
    { 0xbfe0714b, 33 },
    { 0x2590714d, 78 },
    { 0xb1107143, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f907152, 87 },
    { 0, 0 },
    { 0x54a07154, 33 },
    { 0x16107155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0107158, 52 },
    { 0xdfa07159, 43 },
    { 0x90a07159, 76 },
    { 0xdbd07159, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0715e, 98 },
    { 0x2fc0715f, 97 },
    { 0xd0507160, 61 },
    { 0x4ea0715e, 61 },
    { 0x8440715e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c07169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500716e, 17 },
    { 0x3100716f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7407172, 66 },
    { 0xda407172, 34 },
    { 0x1307173, 98 },
    { 0x13107175, 52 },
    { 0x20207176, 90 },
    { 0x95a07177, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7030717a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12207181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0907185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e0718a, 95 },
    { 0x1500718a, 36 },
    { 0xca30718b, 95 },
    { 0xe0a0718d, 98 },
    { 0xe5a0718c, 90 },
    { 0x8000718d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78707192, 90 },
    { 0xdc007193, 90 },
    { 0xe4b07193, 34 },
    { 0xd3407195, 23 },
    { 0x9b907195, 55 },
    { 0x3ec07197, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5980719d, 39 },
    { 0x4400719e, 87 },
    { 0xe1c0719f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00071a4, 61 },
    { 0x380071a5, 37 },
    { 0xd63071a6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1071ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142071b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df071b6, 93 },
    { 0, 0 },
    { 0xec1071b8, 33 },
    { 0x63d071b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2071bc, 23 },
    { 0x6c071bc, 76 },
    { 0x9a1071bc, 61 },
    { 0x57e071bf, 95 },
    { 0, 0 },
    { 0x7d3071c1, 61 },
    { 0x929071c2, 34 },
    { 0xc5f071c3, 61 },
    { 0, 0 },
    { 0x75b071c5, 55 },
    { 0x77071c5, 76 },
    { 0xc03071c7, 33 },
    { 0xdfe071c5, 91 },
    { 0xc51071c9, 55 },
    { 0, 0 },
    { 0x73071cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8071d2, 95 },
    { 0xb1d071d2, 33 },
    { 0x536071d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96071d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c071dd, 98 },
    { 0xec4071dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932071e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528071e6, 61 },
    { 0xbd071e7, 43 },
    { 0, 0 },
    { 0x2f5071e9, 98 },
    { 0, 0 },
    { 0x3af071eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5071ee, 88 },
    { 0, 0 },
    { 0x695071f0, 61 },
    { 0x53a071f1, 90 },
    { 0, 0 },
    { 0x58e071f3, 46 },
    { 0x2e7071f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16071f7, 61 },
    { 0x762071f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27071fb, 33 },
    { 0xaf2071fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b07200, 90 },
    { 0, 0 },
    { 0x4ab07202, 17 },
    { 0x4607203, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c807208, 87 },
    { 0xa607209, 76 },
    { 0xbc70720a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67607210, 33 },
    { 0x8e007210, 64 },
    { 0x7fc07212, 55 },
    { 0xa4007213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b07216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c07219, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa07221, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a07224, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1000722c, 78 },
    { 0xaa00722d, 43 },
    { 0xf1f0722c, 34 },
    { 0x8a70722f, 90 },
    { 0x11107230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef07233, 33 },
    { 0x92107234, 76 },
    { 0x4fd07235, 90 },
    { 0, 0 },
    { 0x61607237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd480723d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75807242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c07245, 61 },
    { 0xee507246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e0724b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc207250, 95 },
    { 0x43c07251, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6307254, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f0725b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3960725e, 66 },
    { 0xa010725f, 99 },
    { 0xa80725f, 33 },
    { 0, 0 },
    { 0xb1307262, 23 },
    { 0xfd907263, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc807268, 23 },
    { 0, 0 },
    { 0x4670726a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810726f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1407273, 90 },
    { 0x7d007274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe530727c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e07286, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8590728e, 95 },
    { 0x3020728f, 34 },
    { 0, 0 },
    { 0x18507291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7307297, 100 },
    { 0, 0 },
    { 0x7bc07299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b072a1, 33 },
    { 0, 0 },
    { 0x598072a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156072ac, 43 },
    { 0xcbb072ad, 75 },
    { 0x111072ae, 33 },
    { 0x36d072af, 99 },
    { 0xd4d072af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b072b3, 33 },
    { 0xd27072b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79072b7, 19 },
    { 0, 0 },
    { 0xa19072b9, 96 },
    { 0xc6072b9, 33 },
    { 0x7c3072bb, 43 },
    { 0xa44072bc, 90 },
    { 0xb15072bd, 23 },
    { 0xd27072bd, 33 },
    { 0x808072b9, 98 },
    { 0, 0 },
    { 0x7bf072c1, 34 },
    { 0, 0 },
    { 0xc5c072c3, 90 },
    { 0xc81072c4, 78 },
    { 0xc72072c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56072cc, 90 },
    { 0x914072cd, 90 },
    { 0x279072cc, 43 },
    { 0x98072cf, 95 },
    { 0xf14072d0, 98 },
    { 0, 0 },
    { 0xb64072d2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110072d7, 23 },
    { 0xc072d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x588072db, 61 },
    { 0x1bf072dc, 61 },
    { 0xd8f072dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572072e1, 34 },
    { 0x871072e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70072e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170072ef, 90 },
    { 0, 0 },
    { 0xaa5072f1, 90 },
    { 0xb22072f2, 98 },
    { 0x670072f3, 67 },
    { 0x6f6072f4, 90 },
    { 0xfdf072f2, 43 },
    { 0x784072f6, 33 },
    { 0, 0 },
    { 0x488072f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e072fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b907301, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19807309, 90 },
    { 0, 0 },
    { 0x4da0730b, 97 },
    { 0, 0 },
    { 0xafa0730d, 65 },
    { 0x90c0730d, 40 },
    { 0xc650730f, 84 },
    { 0xee407310, 33 },
    { 0, 0 },
    { 0xa4e07312, 61 },
    { 0xa4807312, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x80607316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be07319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90731c, 95 },
    { 0x47a0731c, 67 },
    { 0x6410731c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46007322, 78 },
    { 0x79a07323, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe507326, 68 },
    { 0, 0 },
    { 0x6507328, 23 },
    { 0xb9b07329, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe580732f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e70733b, 2 },
    { 0x3930733c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0733f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f07342, 95 },
    { 0x9ee07343, 87 },
    { 0x4f507344, 80 },
    { 0, 0 },
    { 0x71a07346, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c507349, 98 },
    { 0xf907349, 95 },
    { 0x8270734b, 23 },
    { 0x1b30734c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e07351, 90 },
    { 0, 0 },
    { 0xaec07353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a07357, 76 },
    { 0, 0 },
    { 0xb8c07359, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920735c, 23 },
    { 0, 0 },
    { 0x8e10735e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b07362, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d07367, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e0736a, 95 },
    { 0x9280736b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6107371, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e07377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330737b, 33 },
    { 0xece0737b, 34 },
    { 0x26f0737b, 99 },
    { 0, 0 },
    { 0x16e0737f, 43 },
    { 0x17d07380, 75 },
    { 0x7fa07381, 67 },
    { 0, 0 },
    { 0xb8d07383, 78 },
    { 0x8ed07383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26007389, 100 },
    { 0x6640738a, 43 },
    { 0x2650738b, 90 },
    { 0x66c0738a, 61 },
    { 0x6360738c, 34 },
    { 0, 0 },
    { 0x8100738f, 46 },
    { 0x4d80738f, 90 },
    { 0x6c007391, 98 },
    { 0, 0 },
    { 0x53f07393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8040739c, 80 },
    { 0, 0 },
    { 0x7de0739e, 46 },
    { 0xdc50739f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea073a5, 95 },
    { 0x3af073a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87073ab, 100 },
    { 0x3b6073ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b073af, 80 },
    { 0, 0 },
    { 0x160073b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd073ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64073bf, 95 },
    { 0, 0 },
    { 0x957073c1, 55 },
    { 0, 0 },
    { 0x74073c3, 90 },
    { 0xd86073c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1073cb, 76 },
    { 0x989073cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2073cf, 98 },
    { 0x68b073d0, 98 },
    { 0, 0 },
    { 0xbfe073d2, 90 },
    { 0xc80073d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5073d7, 67 },
    { 0x55073d7, 78 },
    { 0, 0 },
    { 0xa2b073da, 95 },
    { 0xb64073da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1073e3, 33 },
    { 0x883073e3, 34 },
    { 0x22e073e5, 77 },
    { 0, 0 },
    { 0xa08073e7, 44 },
    { 0x766073e8, 61 },
    { 0, 0 },
    { 0x6fe073ea, 55 },
    { 0xd0073eb, 66 },
    { 0xb50073ec, 90 },
    { 0xfc2073eb, 23 },
    { 0xd18073ea, 88 },
    { 0x56c073ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x504073f2, 46 },
    { 0, 0 },
    { 0x494073f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a073f7, 87 },
    { 0x722073f8, 95 },
    { 0x729073f8, 98 },
    { 0xd7e073f8, 76 },
    { 0xccf073f8, 55 },
    { 0, 0 },
    { 0x940073fd, 90 },
    { 0x4c5073fe, 61 },
    { 0x6a8073ff, 61 },
    { 0x704073ff, 61 },
    { 0xbc707401, 77 },
    { 0xd3207402, 93 },
    { 0, 0 },
    { 0x42807404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f0740a, 90 },
    { 0, 0 },
    { 0xa5c0740c, 66 },
    { 0x2330740d, 90 },
    { 0xb80740d, 34 },
    { 0x9890740c, 23 },
    { 0x5150740f, 90 },
    { 0xc3907411, 98 },
    { 0x11107411, 95 },
    { 0xd707411, 95 },
    { 0x2300740e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5707417, 90 },
    { 0xab807418, 34 },
    { 0, 0 },
    { 0x1790741a, 33 },
    { 0, 0 },
    { 0x4070741c, 43 },
    { 0x2520741d, 43 },
    { 0x6a30741e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa07422, 85 },
    { 0x33807422, 90 },
    { 0x9a807424, 33 },
    { 0xa207425, 43 },
    { 0xa2407426, 78 },
    { 0x74907425, 95 },
    { 0xbd207428, 61 },
    { 0xf0107428, 95 },
    { 0xbd707422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c07430, 34 },
    { 0xbc807431, 33 },
    { 0xb0907431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db07440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79307445, 98 },
    { 0x24c07445, 90 },
    { 0, 0 },
    { 0xb8507448, 61 },
    { 0x89907449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae07451, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3107456, 99 },
    { 0, 0 },
    { 0x65d07458, 76 },
    { 0xa1407459, 23 },
    { 0x60207458, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d0745f, 85 },
    { 0, 0 },
    { 0xb4c07461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f507466, 90 },
    { 0x5ad07466, 95 },
    { 0x2a607468, 23 },
    { 0, 0 },
    { 0x4910746a, 33 },
    { 0x36d0746a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7507472, 99 },
    { 0xdfe07473, 81 },
    { 0xc5307473, 90 },
    { 0x4d907473, 23 },
    { 0, 0 },
    { 0xe2a07477, 91 },
    { 0xda407477, 78 },
    { 0x2cc07479, 88 },
    { 0x19c0747a, 92 },
    { 0x94107477, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf07488, 91 },
    { 0xbe107489, 33 },
    { 0xb5d07489, 90 },
    { 0x54b07489, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b07491, 99 },
    { 0xe1d07492, 55 },
    { 0xe2507493, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9707496, 61 },
    { 0x6b907497, 23 },
    { 0, 0 },
    { 0xf5207499, 52 },
    { 0xc970749a, 90 },
    { 0, 0 },
    { 0x64d0749c, 23 },
    { 0x6640749d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f074a2, 33 },
    { 0x9c9074a3, 76 },
    { 0x11f074a4, 46 },
    { 0x5c0074a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996074a9, 90 },
    { 0xead074aa, 33 },
    { 0xbab074aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb074b1, 34 },
    { 0x6bd074b1, 99 },
    { 0x641074b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b074b8, 98 },
    { 0xf91074b8, 83 },
    { 0xce074ba, 90 },
    { 0x5f2074bb, 95 },
    { 0xc42074bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a074c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6074c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4074cd, 33 },
    { 0x69d074ce, 76 },
    { 0xa3b074cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x495074d2, 99 },
    { 0x1b0074d3, 52 },
    { 0xf6a074d3, 61 },
    { 0x18a074d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4074da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x607074dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5074e0, 23 },
    { 0xe82074e1, 33 },
    { 0x412074e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c074e8, 98 },
    { 0, 0 },
    { 0x84b074ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9074ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430074f1, 90 },
    { 0x34c074f2, 91 },
    { 0, 0 },
    { 0xd6c074f4, 95 },
    { 0x816074f5, 90 },
    { 0xc4a074f6, 52 },
    { 0xddc074f7, 67 },
    { 0x7dc074f4, 90 },
    { 0xf35074f4, 55 },
    { 0x59b074f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73074ff, 90 },
    { 0x318074ff, 55 },
    { 0, 0 },
    { 0xa7507502, 98 },
    { 0x8b007502, 68 },
    { 0x2b007504, 43 },
    { 0x6bd07502, 33 },
    { 0x39107506, 76 },
    { 0, 0 },
    { 0xd6507508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e0750e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a07512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7507515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920751a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7907529, 91 },
    { 0, 0 },
    { 0xb380752b, 90 },
    { 0x9fe0752c, 43 },
    { 0x8e60752d, 67 },
    { 0xf20752d, 34 },
    { 0, 0 },
    { 0xd7107530, 34 },
    { 0xb6207531, 82 },
    { 0x5df07532, 95 },
    { 0xed407533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0753f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9007542, 61 },
    { 0, 0 },
    { 0x9a407544, 17 },
    { 0x82b07545, 67 },
    { 0x46307546, 91 },
    { 0x1807547, 95 },
    { 0xe4707547, 93 },
    { 0x3c207549, 23 },
    { 0, 0 },
    { 0x4dd0754b, 98 },
    { 0x1510754b, 95 },
    { 0x8a20754b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb907559, 81 },
    { 0x3950755a, 34 },
    { 0x6290755b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee0755f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0707563, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6007566, 98 },
    { 0x7707567, 46 },
    { 0x73d07568, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f07570, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6507573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63307576, 66 },
    { 0xc4607576, 84 },
    { 0xe8a07578, 90 },
    { 0xa5a07579, 33 },
    { 0x12707579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a707580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9107583, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e007588, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc610758d, 61 },
    { 0, 0 },
    { 0x47f0758f, 69 },
    { 0, 0 },
    { 0xa2207591, 98 },
    { 0xf5907591, 17 },
    { 0x59407593, 95 },
    { 0xcc707594, 90 },
    { 0x2ed07595, 43 },
    { 0xaf807596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0759c, 93 },
    { 0, 0 },
    { 0x65d0759e, 46 },
    { 0x77e0759e, 40 },
    { 0x7d90759f, 90 },
    { 0, 0 },
    { 0x69a075a2, 46 },
    { 0x932075a2, 23 },
    { 0xf9d075a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d075a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2075ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x134075af, 95 },
    { 0x853075b0, 60 },
    { 0xc58075b1, 90 },
    { 0, 0 },
    { 0x385075b3, 66 },
    { 0, 0 },
    { 0xe2075b5, 87 },
    { 0, 0 },
    { 0x253075b7, 43 },
    { 0x3d1075b8, 76 },
    { 0x9fb075b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858075be, 99 },
    { 0x42b075bf, 91 },
    { 0x6e075c0, 78 },
    { 0x998075c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x725075c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f075c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b075ce, 68 },
    { 0, 0 },
    { 0x562075d0, 34 },
    { 0x8de075d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d075d4, 90 },
    { 0xf3b075d4, 88 },
    { 0xdfb075d5, 87 },
    { 0xc49075d7, 90 },
    { 0x8f8075d7, 98 },
    { 0x3b2075d9, 59 },
    { 0xc8a075da, 90 },
    { 0x515075da, 61 },
    { 0xb0a075d9, 65 },
    { 0x663075d8, 88 },
    { 0xef4075de, 88 },
    { 0x34c075df, 90 },
    { 0xdaf075e0, 66 },
    { 0xf25075e0, 91 },
    { 0x553075e2, 55 },
    { 0, 0 },
    { 0xc8b075e4, 43 },
    { 0, 0 },
    { 0xfa8075e6, 90 },
    { 0x398075e7, 34 },
    { 0xff7075e8, 90 },
    { 0xfc3075e8, 90 },
    { 0, 0 },
    { 0xfbd075eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80075ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac075f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98075fc, 33 },
    { 0x1ff075fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a07603, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c907607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b07610, 95 },
    { 0x2c107610, 55 },
    { 0xf3807611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22607615, 34 },
    { 0x5ac07616, 33 },
    { 0xef407617, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa720761c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07620, 95 },
    { 0, 0 },
    { 0x20d07622, 56 },
    { 0, 0 },
    { 0x24a07624, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6407629, 46 },
    { 0, 0 },
    { 0x1e40762b, 17 },
    { 0, 0 },
    { 0x762d, 69 },
    { 0xbd90762e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b207632, 90 },
    { 0, 0 },
    { 0x65807634, 81 },
    { 0, 0 },
    { 0x3bb07636, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330763c, 95 },
    { 0x5850763d, 61 },
    { 0, 0 },
    { 0x76d0763f, 34 },
    { 0xbfa07640, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd807643, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c07647, 80 },
    { 0, 0 },
    { 0xdc207649, 46 },
    { 0, 0 },
    { 0xf8d0764b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b70764f, 98 },
    { 0xe5207650, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f07654, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0765a, 34 },
    { 0xf700765b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80765e, 43 },
    { 0, 0 },
    { 0x7407660, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8707663, 43 },
    { 0, 0 },
    { 0xef107665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x207668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d107670, 52 },
    { 0x9f407671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23607679, 33 },
    { 0xe1b0767a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d407680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be07683, 65 },
    { 0x85707684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee07689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b00768f, 76 },
    { 0xaa607690, 99 },
    { 0, 0 },
    { 0xf0c07692, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa07696, 23 },
    { 0x97407696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3076a1, 76 },
    { 0, 0 },
    { 0x338076a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904076aa, 91 },
    { 0xac0076ab, 91 },
    { 0x3ee076ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f076b0, 88 },
    { 0x248076b0, 95 },
    { 0xc4076b0, 90 },
    { 0x9cf076b3, 33 },
    { 0, 0 },
    { 0x3ed076b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105076bd, 34 },
    { 0x426076be, 23 },
    { 0x3d7076bd, 95 },
    { 0, 0 },
    { 0x6a6076c1, 46 },
    { 0x68e076c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae076c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102076cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba076d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1076d9, 33 },
    { 0, 0 },
    { 0x95a076db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x434076de, 61 },
    { 0xbc1076df, 99 },
    { 0x6a2076e0, 98 },
    { 0x21f076e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb076e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d076e8, 34 },
    { 0x819076e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128076ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3076f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772076f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28207705, 43 },
    { 0xb3d07706, 100 },
    { 0xcc707706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a0770c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30907716, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7007722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6807725, 23 },
    { 0x1fe07726, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e307730, 90 },
    { 0x6d507731, 76 },
    { 0, 0 },
    { 0x81807733, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7107736, 23 },
    { 0x78307737, 95 },
    { 0x6e207738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc950773b, 90 },
    { 0x63c0773c, 46 },
    { 0xfc20773d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2907742, 33 },
    { 0x29507743, 61 },
    { 0, 0 },
    { 0xaca07745, 34 },
    { 0, 0 },
    { 0x3f407747, 61 },
    { 0xd7107748, 67 },
    { 0xa8007748, 66 },
    { 0x7140774a, 43 },
    { 0x5480774a, 76 },
    { 0xc130774b, 98 },
    { 0xa470774d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf07750, 95 },
    { 0x21707751, 90 },
    { 0x2cb07752, 95 },
    { 0xc5307753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68407756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1230775c, 95 },
    { 0x7b30775c, 34 },
    { 0, 0 },
    { 0xd8f0775f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a607769, 55 },
    { 0, 0 },
    { 0x1fe0776b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce07771, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb107777, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f0777a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x530777d, 33 },
    { 0, 0 },
    { 0x3830777f, 66 },
    { 0x73407780, 23 },
    { 0x8e207781, 98 },
    { 0, 0 },
    { 0x69e07783, 61 },
    { 0x79407783, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d07787, 76 },
    { 0xb8307788, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60778e, 90 },
    { 0x6580778f, 90 },
    { 0x98a0778f, 34 },
    { 0x52b07791, 76 },
    { 0xaf407792, 43 },
    { 0xa4a07792, 85 },
    { 0, 0 },
    { 0xea007795, 46 },
    { 0x1807796, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690779a, 33 },
    { 0x8eb0779b, 55 },
    { 0, 0 },
    { 0x190779d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3077a4, 66 },
    { 0x2ec077a5, 90 },
    { 0xe97077a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9077a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e077ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7077b4, 84 },
    { 0x342077b5, 18 },
    { 0, 0 },
    { 0x57b077b7, 96 },
    { 0xae0077b8, 90 },
    { 0xf10077b9, 55 },
    { 0x764077b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b077bd, 90 },
    { 0x58d077be, 91 },
    { 0xa68077bd, 25 },
    { 0x346077c0, 33 },
    { 0xb44077c1, 90 },
    { 0, 0 },
    { 0xcff077c3, 65 },
    { 0, 0 },
    { 0x741077c5, 34 },
    { 0x3d3077c6, 68 },
    { 0x32e077c5, 97 },
    { 0, 0 },
    { 0x4c8077c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5077cc, 98 },
    { 0x437077cd, 23 },
    { 0xd4077ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7077d3, 95 },
    { 0x4ac077d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1077d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba077dd, 91 },
    { 0, 0 },
    { 0x625077df, 90 },
    { 0x292077df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af077e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x202077e7, 95 },
    { 0, 0 },
    { 0x2c4077e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116077f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503077f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca077f9, 61 },
    { 0x8a3077fa, 91 },
    { 0x581077fa, 78 },
    { 0x867077fc, 61 },
    { 0xf93077fd, 61 },
    { 0x51f077fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95307805, 95 },
    { 0xd5e07806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10780a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80780d, 90 },
    { 0x93d0780e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1807813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57607816, 76 },
    { 0x76e07816, 90 },
    { 0x78a07817, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0707823, 98 },
    { 0xb9207824, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14607829, 98 },
    { 0xf8707829, 98 },
    { 0xc690782b, 23 },
    { 0xbce0782c, 55 },
    { 0, 0 },
    { 0xf90782e, 36 },
    { 0x59b0782f, 23 },
    { 0, 0 },
    { 0x9fa07831, 81 },
    { 0x68f07831, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae07839, 23 },
    { 0xbb0783a, 33 },
    { 0x6df0783b, 90 },
    { 0x42e0783c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb880783f, 61 },
    { 0, 0 },
    { 0x70f07841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a007846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a0784d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde07854, 38 },
    { 0x2fa07855, 96 },
    { 0x97007856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00785f, 95 },
    { 0, 0 },
    { 0x4a107861, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf407864, 90 },
    { 0x1ba07865, 76 },
    { 0x35f07864, 87 },
    { 0, 0 },
    { 0xfe207868, 90 },
    { 0, 0 },
    { 0xd6c0786a, 23 },
    { 0x9900786b, 90 },
    { 0x6f90786b, 80 },
    { 0x5470786d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6907870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd07873, 43 },
    { 0x75507874, 93 },
    { 0x5ca07873, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95207878, 23 },
    { 0x32407879, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30787c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca07884, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5407887, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb30788c, 43 },
    { 0, 0 },
    { 0x2b50788e, 95 },
    { 0xf260788f, 90 },
    { 0xee0788e, 93 },
    { 0, 0 },
    { 0x7c907892, 90 },
    { 0xe9607893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770789a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf60789e, 90 },
    { 0, 0 },
    { 0x7e6078a0, 66 },
    { 0, 0 },
    { 0x7cf078a2, 66 },
    { 0x70e078a2, 23 },
    { 0x54a078a4, 90 },
    { 0x8a4078a3, 93 },
    { 0xe6a078a6, 91 },
    { 0x4a5078a7, 99 },
    { 0, 0 },
    { 0xda7078a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483078ad, 90 },
    { 0, 0 },
    { 0x566078af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4078b6, 95 },
    { 0, 0 },
    { 0xf31078b8, 66 },
    { 0, 0 },
    { 0x93d078ba, 34 },
    { 0, 0 },
    { 0x33f078bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f078c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766078ca, 77 },
    { 0xec8078cb, 61 },
    { 0x89d078cb, 55 },
    { 0xf93078cd, 91 },
    { 0x4df078ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350078d2, 83 },
    { 0x7e9078d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7078da, 94 },
    { 0x22c078db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173078e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21078e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92078e8, 23 },
    { 0, 0 },
    { 0xbae078ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8078ed, 46 },
    { 0, 0 },
    { 0xb90078ef, 90 },
    { 0x4ca078f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5078f7, 95 },
    { 0x14c078f7, 33 },
    { 0x4a0078f9, 85 },
    { 0x2b8078f9, 76 },
    { 0xbb4078fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d807902, 33 },
    { 0xebf07902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd0790f, 95 },
    { 0, 0 },
    { 0xe6907911, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43007918, 46 },
    { 0x72a07918, 23 },
    { 0x9890791a, 90 },
    { 0x1ec0791b, 95 },
    { 0x41e0791c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc807920, 76 },
    { 0, 0 },
    { 0xe9207922, 98 },
    { 0x38107923, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b07926, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0792e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20907934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b0793a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c07944, 84 },
    { 0x72007945, 92 },
    { 0x50e07945, 23 },
    { 0xe8007947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf620794c, 55 },
    { 0, 0 },
    { 0xa630794e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe507952, 88 },
    { 0, 0 },
    { 0xbf307954, 66 },
    { 0x63107955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b00795f, 46 },
    { 0x6fd0795f, 34 },
    { 0x59e0795f, 95 },
    { 0x71907962, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2807965, 95 },
    { 0xcc007966, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87407969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8530796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71707972, 33 },
    { 0x61807973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d07976, 95 },
    { 0x7b007977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed0797b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0797f, 93 },
    { 0xff407980, 90 },
    { 0x9fc07981, 84 },
    { 0, 0 },
    { 0x96407983, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7707986, 90 },
    { 0, 0 },
    { 0x50207988, 99 },
    { 0, 0 },
    { 0x3a30798a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360798e, 77 },
    { 0xc6c0798f, 34 },
    { 0xe400798e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d507996, 17 },
    { 0x21b07997, 52 },
    { 0x2fc07998, 76 },
    { 0x45907998, 55 },
    { 0x31007998, 43 },
    { 0, 0 },
    { 0x6d80799c, 61 },
    { 0, 0 },
    { 0x9570799e, 55 },
    { 0xc330799f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x495079a2, 33 },
    { 0, 0 },
    { 0x92a079a4, 98 },
    { 0xa0a079a4, 95 },
    { 0, 0 },
    { 0xa4d079a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93079ad, 46 },
    { 0x2d0079ad, 52 },
    { 0, 0 },
    { 0x1a1079b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159079b4, 93 },
    { 0xbe7079b5, 34 },
    { 0xf079b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37079c3, 93 },
    { 0, 0 },
    { 0x8ff079c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea079cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x535079ce, 33 },
    { 0xb84079cf, 98 },
    { 0x8a0079cf, 99 },
    { 0xa36079d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03079d8, 61 },
    { 0x49b079d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40079dc, 23 },
    { 0xd28079dd, 33 },
    { 0, 0 },
    { 0xe2f079df, 23 },
    { 0, 0 },
    { 0x336079e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41079e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef079eb, 33 },
    { 0xa47079ec, 43 },
    { 0xfda079ed, 93 },
    { 0x7be079ed, 61 },
    { 0x9b079ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e079f2, 61 },
    { 0x8f8079f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f079f7, 34 },
    { 0x896079f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b079fc, 23 },
    { 0x356079fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f707a01, 67 },
    { 0xac07a02, 43 },
    { 0, 0 },
    { 0xb8207a04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95507a0c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6607a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c07a17, 83 },
    { 0x83b07a18, 98 },
    { 0x37e07a19, 51 },
    { 0xaf207a1a, 33 },
    { 0x9507a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2207a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6907a21, 33 },
    { 0xcb307a21, 93 },
    { 0xc2907a23, 55 },
    { 0x42207a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b607a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5207a2d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b907a35, 90 },
    { 0xe307a36, 93 },
    { 0x7db07a37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b07a3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007a41, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb07a44, 95 },
    { 0x48f07a44, 43 },
    { 0xfcf07a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2107a4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3807a51, 33 },
    { 0, 0 },
    { 0xe907a53, 55 },
    { 0xcc807a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc07a57, 69 },
    { 0, 0 },
    { 0xb0907a59, 23 },
    { 0xe3e07a5a, 55 },
    { 0xeeb07a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93907a5e, 33 },
    { 0, 0 },
    { 0xa4807a60, 95 },
    { 0x37407a60, 61 },
    { 0xfc207a62, 23 },
    { 0, 0 },
    { 0xfff07a64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62407a68, 33 },
    { 0xf2c07a69, 95 },
    { 0x52207a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a07a6d, 24 },
    { 0x97807a6e, 90 },
    { 0x74507a6f, 84 },
    { 0x65307a6f, 90 },
    { 0xa9d07a71, 90 },
    { 0xdc007a72, 90 },
    { 0xdf107a73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c507a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c307a7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f307a7e, 43 },
    { 0, 0 },
    { 0x5b207a80, 87 },
    { 0xd3007a81, 23 },
    { 0, 0 },
    { 0xef407a83, 93 },
    { 0x28207a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83607a8d, 23 },
    { 0xeea07a8e, 34 },
    { 0xa7807a8f, 33 },
    { 0x87f07a90, 61 },
    { 0x96c07a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66407a94, 99 },
    { 0xf6c07a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97307a98, 23 },
    { 0, 0 },
    { 0xb9307a9a, 84 },
    { 0, 0 },
    { 0xbe907a9c, 33 },
    { 0x9ef07a9d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab007aa3, 98 },
    { 0x7c207aa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5907aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c507aac, 90 },
    { 0xd2e07aad, 77 },
    { 0x7f407aae, 82 },
    { 0x77507aae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4107ab2, 33 },
    { 0xcd007ab3, 43 },
    { 0xbd807ab3, 69 },
    { 0xb9507ab4, 76 },
    { 0xf8907ab6, 85 },
    { 0, 0 },
    { 0x81207ab8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c107abc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c07ac4, 90 },
    { 0xba907ac4, 46 },
    { 0xa8e07ac5, 77 },
    { 0xddf07ac5, 61 },
    { 0x36207ac8, 43 },
    { 0x54207ac9, 23 },
    { 0x3d107ac7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5007acd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b07ad1, 65 },
    { 0x17a07ad1, 33 },
    { 0x9607ad3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b307adb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3207ae5, 98 },
    { 0, 0 },
    { 0xaa607ae7, 93 },
    { 0x56007ae8, 65 },
    { 0xdfe07ae9, 98 },
    { 0, 0 },
    { 0x29f07aeb, 78 },
    { 0x8a907aec, 97 },
    { 0xb207aed, 23 },
    { 0, 0 },
    { 0xa3a07aef, 33 },
    { 0, 0 },
    { 0x16207af1, 55 },
    { 0x84407af2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f07af9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3207b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e507b0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a07b11, 43 },
    { 0, 0 },
    { 0xe4207b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a07b16, 61 },
    { 0, 0 },
    { 0xc7507b18, 95 },
    { 0, 0 },
    { 0xac307b1a, 55 },
    { 0x2d907b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc07b1e, 90 },
    { 0xddf07b1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8307b25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac607b2d, 52 },
    { 0xf2707b2e, 85 },
    { 0x97d07b2d, 90 },
    { 0x80907b2d, 78 },
    { 0xbd307b2f, 93 },
    { 0xbed07b32, 17 },
    { 0x23907b32, 61 },
    { 0x1c007b32, 23 },
    { 0x4cf07b35, 93 },
    { 0x5e107b36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07b3a, 33 },
    { 0xec607b3a, 55 },
    { 0xa5407b3b, 43 },
    { 0x82507b3a, 44 },
    { 0x5ec07b3c, 65 },
    { 0x51307b3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf307b43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20507b4a, 61 },
    { 0x5a707b4b, 51 },
    { 0xafc07b4c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c07b4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62007b55, 75 },
    { 0, 0 },
    { 0x3d207b57, 55 },
    { 0x76107b58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c07b5b, 23 },
    { 0xe3407b5b, 85 },
    { 0, 0 },
    { 0x90f07b5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0107b61, 33 },
    { 0, 0 },
    { 0x5f107b63, 46 },
    { 0, 0 },
    { 0xe007b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f07b68, 61 },
    { 0x97e07b69, 43 },
    { 0x8607b6a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e607b6d, 91 },
    { 0, 0 },
    { 0xc6f07b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3907b74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf107b7e, 55 },
    { 0, 0 },
    { 0x51307b80, 55 },
    { 0x30007b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f07b84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e407b8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe507b8e, 43 },
    { 0x7e007b8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f807b92, 93 },
    { 0x1307b93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c07b97, 98 },
    { 0xf0907b97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed07b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a07ba4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25107ba8, 95 },
    { 0, 0 },
    { 0xc8107baa, 51 },
    { 0, 0 },
    { 0xcfa07bac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40307bb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb07bb5, 100 },
    { 0x2c007bb6, 52 },
    { 0x25b07bb6, 61 },
    { 0xfca07bb8, 98 },
    { 0xce07bb9, 46 },
    { 0xf2607bba, 23 },
    { 0x40c07bb8, 67 },
    { 0, 0 },
    { 0x73307bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed07bc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x25707bc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce07bc8, 34 },
    { 0, 0 },
    { 0xe0d07bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54007bce, 55 },
    { 0xc0a07bcf, 23 },
    { 0x32907bcf, 95 },
    { 0x91407bd1, 98 },
    { 0x25707bce, 55 },
    { 0, 0 },
    { 0x4e507bd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a07bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6607bda, 33 },
    { 0, 0 },
    { 0xece07bdc, 84 },
    { 0xe0707bdd, 95 },
    { 0x99007bde, 98 },
    { 0xd3907bdf, 61 },
    { 0xc9007be0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89707be6, 90 },
    { 0, 0 },
    { 0xe6007be8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07bec, 98 },
    { 0, 0 },
    { 0x99307bee, 88 },
    { 0x91407bef, 98 },
    { 0x77d07bf0, 100 },
    { 0x54f07bf1, 56 },
    { 0xf5207bf1, 33 },
    { 0xb5807bf3, 23 },
    { 0x75c07bf4, 76 },
    { 0, 0 },
    { 0x47607bf6, 75 },
    { 0x9be07bf7, 80 },
    { 0xf3607bf8, 61 },
    { 0x8be07bf9, 76 },
    { 0xa4e07bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb07c01, 38 },
    { 0x3907c02, 90 },
    { 0x78207c02, 33 },
    { 0x47907c03, 76 },
    { 0x57107c05, 95 },
    { 0x91807c06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa907c0f, 100 },
    { 0x80e07c10, 23 },
    { 0x8a707c10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf807c14, 90 },
    { 0x7e607c14, 76 },
    { 0, 0 },
    { 0xb6507c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de07c1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b07c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6107c23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de07c28, 46 },
    { 0x20c07c29, 33 },
    { 0, 0 },
    { 0xfe007c2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2207c2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3107c31, 61 },
    { 0xcf507c32, 68 },
    { 0x15d07c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1007c36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2307c3a, 84 },
    { 0, 0 },
    { 0x34607c3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d07c43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77007c47, 98 },
    { 0xbc307c48, 61 },
    { 0x92807c48, 61 },
    { 0xe4f07c4a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa07c53, 55 },
    { 0x20e07c54, 68 },
    { 0xf0c07c54, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe07c59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e107c60, 77 },
    { 0xdf307c60, 43 },
    { 0x92c07c62, 33 },
    { 0xa5e07c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c07c68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd07c6d, 98 },
    { 0x86607c6e, 88 },
    { 0x59c07c6f, 99 },
    { 0x5a07c70, 23 },
    { 0xa5a07c71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f07c75, 90 },
    { 0xd2b07c75, 98 },
    { 0xd3307c76, 46 },
    { 0xed407c78, 67 },
    { 0, 0 },
    { 0xadc07c7a, 61 },
    { 0x4c007c7b, 76 },
    { 0x86b07c7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3907c7f, 90 },
    { 0xd2a07c7f, 46 },
    { 0xe8f07c81, 51 },
    { 0xfaf07c82, 23 },
    { 0x2f207c83, 87 },
    { 0, 0 },
    { 0xa3907c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c07c89, 61 },
    { 0x6e107c89, 90 },
    { 0x99a07c89, 70 },
    { 0, 0 },
    { 0x1c207c8d, 78 },
    { 0xb2f07c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42707c91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b07c97, 99 },
    { 0x5f07c98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a407c9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a107ca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e407caa, 61 },
    { 0, 0 },
    { 0x2af07cac, 98 },
    { 0xdb307cac, 55 },
    { 0x7a907cad, 34 },
    { 0x1da07caf, 34 },
    { 0xd1207caf, 90 },
    { 0x6d107cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe007cb6, 43 },
    { 0x67507cb7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad07cbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa07cbf, 67 },
    { 0, 0 },
    { 0xbed07cc1, 98 },
    { 0, 0 },
    { 0xeac07cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1307cc7, 95 },
    { 0, 0 },
    { 0x58707cc9, 61 },
    { 0x89907cc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28407ccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07cd4, 76 },
    { 0, 0 },
    { 0xfea07cd6, 90 },
    { 0x12407cd7, 23 },
    { 0, 0 },
    { 0xd5907cd9, 23 },
    { 0x45e07cda, 61 },
    { 0x33d07cdb, 98 },
    { 0x28e07cd9, 90 },
    { 0xcf507cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69907ce3, 91 },
    { 0, 0 },
    { 0xd4a07ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf307ce8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d07cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de07cf4, 80 },
    { 0, 0 },
    { 0x73f07cf6, 61 },
    { 0xf8407cf6, 33 },
    { 0x79707cf6, 66 },
    { 0x79507cf9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7007cfc, 75 },
    { 0, 0 },
    { 0x2d507cfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87807d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd907d04, 33 },
    { 0, 0 },
    { 0x8dc07d06, 33 },
    { 0x1e807d06, 98 },
    { 0xf2a07d08, 90 },
    { 0, 0 },
    { 0xd6a07d0a, 33 },
    { 0xd7707d0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f07d0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f07d12, 43 },
    { 0, 0 },
    { 0x9d207d14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73607d1c, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f07d22, 75 },
    { 0x98407d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef107d2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x93707d2d, 34 },
    { 0x50e07d2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a07d33, 95 },
    { 0, 0 },
    { 0x62507d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df07d3e, 98 },
    { 0, 0 },
    { 0x6a807d40, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce07d44, 61 },
    { 0xd3c07d45, 43 },
    { 0, 0 },
    { 0x9c907d47, 100 },
    { 0x42d07d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62607d4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff07d4e, 33 },
    { 0x61607d4f, 33 },
    { 0x7b607d50, 81 },
    { 0x80507d4e, 95 },
    { 0x74607d52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58107d55, 22 },
    { 0, 0 },
    { 0x72207d57, 77 },
    { 0, 0 },
    { 0x38207d59, 55 },
    { 0xa8e07d59, 33 },
    { 0xbfe07d5b, 95 },
    { 0xb9c07d5c, 55 },
    { 0, 0 },
    { 0x28207d5e, 55 },
    { 0x86607d5f, 98 },
    { 0x63f07d60, 23 },
    { 0x29207d60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1807d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b07d6c, 100 },
    { 0x19107d6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28207d71, 23 },
    { 0xd3407d72, 75 },
    { 0xc4207d73, 90 },
    { 0xa2a07d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43007d78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007d7f, 93 },
    { 0x50107d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4607d84, 98 },
    { 0xdda07d85, 65 },
    { 0xdf907d85, 77 },
    { 0, 0 },
    { 0x52007d88, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5907d8d, 33 },
    { 0xa4207d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94407d91, 43 },
    { 0x91107d92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e07d9c, 100 },
    { 0, 0 },
    { 0xad507d9e, 23 },
    { 0x3bd07d9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc07da5, 95 },
    { 0xa7607da6, 76 },
    { 0x91607da6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f07daa, 23 },
    { 0, 0 },
    { 0x5ab07dac, 68 },
    { 0, 0 },
    { 0x11507dae, 33 },
    { 0, 0 },
    { 0x5f507db0, 69 },
    { 0x94e07db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e707dbd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f507dc0, 87 },
    { 0xc6b07dc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca07dc6, 23 },
    { 0, 0 },
    { 0xde107dc8, 95 },
    { 0, 0 },
    { 0xed407dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7207dd6, 90 },
    { 0, 0 },
    { 0x6f407dd8, 67 },
    { 0xdd607dd8, 95 },
    { 0x91807dda, 23 },
    { 0, 0 },
    { 0xc0b07ddc, 95 },
    { 0x1c007ddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36107de2, 95 },
    { 0, 0 },
    { 0xfe707de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007dea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6107ded, 98 },
    { 0, 0 },
    { 0x3a307def, 76 },
    { 0, 0 },
    { 0xcf007df1, 33 },
    { 0x14d07df2, 78 },
    { 0, 0 },
    { 0x9f407df4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef07dfb, 98 },
    { 0, 0 },
    { 0x2d407dfd, 90 },
    { 0xd7307dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d307e04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee07e07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf607e0c, 76 },
    { 0x16307e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df07e13, 66 },
    { 0x63307e14, 61 },
    { 0xbfb07e14, 90 },
    { 0x1e107e16, 43 },
    { 0x59e07e17, 61 },
    { 0x1c407e18, 84 },
    { 0x79407e18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d07e22, 33 },
    { 0xd9f07e23, 99 },
    { 0xa8807e24, 98 },
    { 0xe2a07e24, 33 },
    { 0xf8a07e26, 98 },
    { 0xfde07e27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae07e2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc07e2e, 33 },
    { 0x66907e2f, 61 },
    { 0xce007e30, 23 },
    { 0x4bc07e30, 93 },
    { 0xa2107e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe307e38, 46 },
    { 0xa5607e39, 61 },
    { 0xd5907e3a, 61 },
    { 0x4e607e3b, 23 },
    { 0xc9c07e3a, 61 },
    { 0x38107e3a, 34 },
    { 0x8fc07e3e, 61 },
    { 0xe9507e3f, 17 },
    { 0x35707e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10707e43, 91 },
    { 0x81d07e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe07e4d, 33 },
    { 0xb3c07e4e, 84 },
    { 0xe9607e4f, 33 },
    { 0xec507e4d, 95 },
    { 0, 0 },
    { 0x9b707e52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0307e5d, 90 },
    { 0, 0 },
    { 0x9aa07e5f, 65 },
    { 0xd3d07e60, 61 },
    { 0xed107e61, 46 },
    { 0x96607e5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f407e69, 61 },
    { 0x9ed07e6a, 33 },
    { 0xd8e07e6b, 23 },
    { 0x84807e6a, 98 },
    { 0xb607e69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b07e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f07e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c107e79, 34 },
    { 0x3e207e7a, 34 },
    { 0, 0 },
    { 0xf7e07e7c, 34 },
    { 0x91407e7d, 93 },
    { 0, 0 },
    { 0xb407e7f, 78 },
    { 0x95c07e80, 61 },
    { 0xbb507e80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61907e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf07e88, 46 },
    { 0x5a907e88, 83 },
    { 0x92007e8a, 47 },
    { 0, 0 },
    { 0xefa07e8c, 85 },
    { 0x1d07e8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45607e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb07e95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b707e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11807ea1, 98 },
    { 0x94407ea2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d07eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f707eae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95607eba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f07ebe, 90 },
    { 0, 0 },
    { 0x35407ec0, 55 },
    { 0x41507ec1, 33 },
    { 0x71d07ec2, 100 },
    { 0, 0 },
    { 0xde607ec4, 100 },
    { 0xb4f07ec5, 90 },
    { 0, 0 },
    { 0xd0907ec7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f307ecc, 90 },
    { 0xb1807ecc, 95 },
    { 0x2b07ece, 90 },
    { 0x67707ecf, 33 },
    { 0xe907ece, 33 },
    { 0, 0 },
    { 0xb9307ed2, 95 },
    { 0, 0 },
    { 0xedf07ed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa807edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93807ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa07ee1, 69 },
    { 0x42307ee2, 33 },
    { 0x61a07ee3, 90 },
    { 0x6b507ee4, 90 },
    { 0x6d307ee3, 66 },
    { 0x53707ee6, 43 },
    { 0x15707ee3, 33 },
    { 0xafb07ee8, 34 },
    { 0x9b007ee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8207eed, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9007ef2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a07eff, 52 },
    { 0x54507f00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9807f07, 34 },
    { 0x88007f08, 95 },
    { 0x7e107f07, 23 },
    { 0, 0 },
    { 0xc3407f0b, 43 },
    { 0xd8c07f0c, 90 },
    { 0xd7407f0c, 78 },
    { 0x25907f0e, 52 },
    { 0, 0 },
    { 0xec807f10, 97 },
    { 0, 0 },
    { 0x7ed07f12, 33 },
    { 0xc0a07f12, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb07f17, 84 },
    { 0xfad07f18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e07f1b, 76 },
    { 0xc4a07f1c, 85 },
    { 0xde407f1c, 61 },
    { 0x31507f1e, 64 },
    { 0x59b07f1f, 76 },
    { 0x4a007f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e07f25, 61 },
    { 0x93307f25, 90 },
    { 0xf7407f27, 99 },
    { 0x11b07f27, 34 },
    { 0xf1507f27, 95 },
    { 0xd0907f28, 43 },
    { 0x20507f28, 100 },
    { 0x6fc07f27, 99 },
    { 0x49607f2d, 95 },
    { 0xffb07f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b07f31, 85 },
    { 0, 0 },
    { 0xf1407f33, 93 },
    { 0x74507f34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea607f3f, 76 },
    { 0, 0 },
    { 0x7a707f41, 61 },
    { 0, 0 },
    { 0x96507f43, 43 },
    { 0, 0 },
    { 0x47407f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd507f48, 76 },
    { 0x6be07f48, 33 },
    { 0xdc007f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3807f4f, 33 },
    { 0, 0 },
    { 0xc0d07f51, 33 },
    { 0xc6507f51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c907f55, 91 },
    { 0x36907f56, 83 },
    { 0x17407f57, 95 },
    { 0x7107f58, 23 },
    { 0xff907f59, 23 },
    { 0x83707f5a, 98 },
    { 0x82707f5b, 55 },
    { 0, 0 },
    { 0x27a07f5d, 48 },
    { 0, 0 },
    { 0xa6f07f5f, 93 },
    { 0x78a07f60, 76 },
    { 0, 0 },
    { 0xdbe07f62, 33 },
    { 0xd8607f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce807f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44207f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07f70, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe007f75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c07f79, 81 },
    { 0, 0 },
    { 0x85907f7b, 61 },
    { 0, 0 },
    { 0xd4107f7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf07f86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca07f8a, 87 },
    { 0x56b07f8b, 90 },
    { 0x9d007f8c, 76 },
    { 0, 0 },
    { 0x71a07f8e, 90 },
    { 0x40307f8f, 61 },
    { 0xa9207f90, 66 },
    { 0x1db07f91, 90 },
    { 0xb6407f8f, 80 },
    { 0, 0 },
    { 0x3ef07f94, 67 },
    { 0xc2b07f95, 23 },
    { 0x1a907f95, 85 },
    { 0x9fc07f97, 23 },
    { 0x66b07f98, 90 },
    { 0xcea07f99, 98 },
    { 0x9e607f9a, 34 },
    { 0xf9a07f98, 23 },
    { 0x8c707f96, 91 },
    { 0x59d07f9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3807fa6, 61 },
    { 0xdf107fa6, 90 },
    { 0, 0 },
    { 0x10d07fa9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b407fac, 61 },
    { 0x60307fac, 97 },
    { 0x6c07fac, 91 },
    { 0x73f07fac, 67 },
    { 0x91007fb0, 49 },
    { 0, 0 },
    { 0xf7907fb2, 33 },
    { 0x30d07fb2, 23 },
    { 0, 0 },
    { 0x2da07fb5, 34 },
    { 0x3fa07fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca07fb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62307fbd, 23 },
    { 0x25707fbe, 15 },
    { 0x10707fbe, 33 },
    { 0xd3007fbd, 91 },
    { 0x6e707fc1, 99 },
    { 0x9fb07fc2, 86 },
    { 0xe0807fc0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac07fc6, 90 },
    { 0xadf07fc7, 98 },
    { 0x6be07fc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d07fce, 98 },
    { 0xade07fcf, 43 },
    { 0xe4607fd0, 33 },
    { 0x5dc07fd1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e207fd4, 34 },
    { 0, 0 },
    { 0xcd807fd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd107fdb, 90 },
    { 0x29d07fdc, 33 },
    { 0, 0 },
    { 0x3d207fde, 34 },
    { 0, 0 },
    { 0x5fd07fe0, 66 },
    { 0x2ce07fe1, 76 },
    { 0, 0 },
    { 0x4c207fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1807fe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d07fed, 95 },
    { 0, 0 },
    { 0xabd07fef, 23 },
    { 0, 0 },
    { 0xc5c07ff1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5007ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c07fff, 80 },
    { 0, 0 },
    { 0x8e008001, 33 },
    { 0x8e608001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa708005, 90 },
    { 0x29d08006, 85 },
    { 0, 0 },
    { 0xb6f08008, 95 },
    { 0, 0 },
    { 0xb6f0800a, 33 },
    { 0, 0 },
    { 0x4fb0800c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5620800f, 55 },
    { 0, 0 },
    { 0x5c708011, 23 },
    { 0, 0 },
    { 0x37d08013, 90 },
    { 0xc3d08014, 95 },
    { 0, 0 },
    { 0x42e08016, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c508019, 61 },
    { 0x16e08019, 66 },
    { 0x1f00801b, 23 },
    { 0x5d50801c, 46 },
    { 0x8f70801c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7308025, 99 },
    { 0x2dc08026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8008029, 95 },
    { 0x65b0802a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a0802d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27608034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64108037, 33 },
    { 0x4d908037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x180803b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77108041, 33 },
    { 0xf7208041, 87 },
    { 0xdfe08043, 61 },
    { 0x3b208043, 33 },
    { 0xae708045, 84 },
    { 0x3f108043, 95 },
    { 0x67508047, 78 },
    { 0x5c08048, 34 },
    { 0x41b08045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87808050, 46 },
    { 0, 0 },
    { 0xfb708052, 93 },
    { 0x7308053, 91 },
    { 0xc8708053, 98 },
    { 0x50308054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e08058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0805b, 33 },
    { 0x95e0805b, 78 },
    { 0xfb90805c, 61 },
    { 0, 0 },
    { 0xd830805f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba08063, 100 },
    { 0x8b808064, 90 },
    { 0, 0 },
    { 0x1c108066, 23 },
    { 0x26808066, 76 },
    { 0, 0 },
    { 0xa1c08069, 61 },
    { 0x2c60806a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0806f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c208072, 76 },
    { 0x6e908073, 43 },
    { 0, 0 },
    { 0x5dd08075, 55 },
    { 0x3fe08075, 90 },
    { 0, 0 },
    { 0x6eb08078, 46 },
    { 0x6c408078, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20807d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af08082, 46 },
    { 0xf2808083, 33 },
    { 0x82908084, 34 },
    { 0x7f308082, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e08088, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f90808e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4108094, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d0809b, 87 },
    { 0, 0 },
    { 0x990809d, 55 },
    { 0x7b0809d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc080a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66080a7, 85 },
    { 0xdc2080a7, 98 },
    { 0x8ac080a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51080ac, 87 },
    { 0, 0 },
    { 0x1c080ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x173080b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43080b4, 95 },
    { 0xfab080b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04080bb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4080be, 61 },
    { 0, 0 },
    { 0x80e080c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592080cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b080d0, 61 },
    { 0, 0 },
    { 0x960080d2, 91 },
    { 0x92080d3, 68 },
    { 0xc15080d3, 43 },
    { 0xfd3080d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8080d8, 76 },
    { 0, 0 },
    { 0x9c9080da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332080df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd080e2, 17 },
    { 0, 0 },
    { 0x69080e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3080e7, 73 },
    { 0xa03080e7, 23 },
    { 0x21b080e9, 23 },
    { 0, 0 },
    { 0x4ac080eb, 98 },
    { 0, 0 },
    { 0xcd4080ed, 55 },
    { 0xaa080ee, 100 },
    { 0, 0 },
    { 0xa02080f0, 90 },
    { 0, 0 },
    { 0x20a080f2, 64 },
    { 0x46e080f3, 80 },
    { 0xeb4080f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d080f8, 36 },
    { 0, 0 },
    { 0x6c080fa, 55 },
    { 0x5b1080fa, 33 },
    { 0xfcc080fb, 23 },
    { 0, 0 },
    { 0xb59080fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f08102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb08109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710810f, 78 },
    { 0xbb00810f, 95 },
    { 0xa4c08111, 80 },
    { 0xd1508110, 100 },
    { 0x1b408113, 33 },
    { 0xeea08110, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c008118, 95 },
    { 0, 0 },
    { 0x8390811a, 90 },
    { 0, 0 },
    { 0x9ac0811c, 77 },
    { 0x6430811d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d08120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42908127, 52 },
    { 0, 0 },
    { 0x53c08129, 90 },
    { 0xe7b08129, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df08130, 90 },
    { 0x31e08131, 81 },
    { 0x78808132, 76 },
    { 0x29d08132, 43 },
    { 0x1ee08134, 64 },
    { 0x52b08135, 98 },
    { 0x85908136, 91 },
    { 0x70608135, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a0813b, 23 },
    { 0xe390813c, 52 },
    { 0x7890813d, 98 },
    { 0x2f10813d, 90 },
    { 0, 0 },
    { 0x1e108140, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c408143, 46 },
    { 0x73508144, 97 },
    { 0xdc608145, 61 },
    { 0x40b08146, 61 },
    { 0, 0 },
    { 0x49b08148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f0814c, 95 },
    { 0x8f00814d, 61 },
    { 0, 0 },
    { 0xb410814f, 76 },
    { 0xda908150, 52 },
    { 0x9fd0814f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7108155, 34 },
    { 0, 0 },
    { 0x1db08157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b0815e, 98 },
    { 0, 0 },
    { 0xfe708160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e508163, 90 },
    { 0xdfb08164, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a408167, 99 },
    { 0xc308168, 99 },
    { 0xfc708168, 55 },
    { 0x74b0816a, 68 },
    { 0x87008168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c08170, 23 },
    { 0x46908171, 33 },
    { 0x8b008172, 88 },
    { 0xfa108172, 33 },
    { 0x41508173, 87 },
    { 0, 0 },
    { 0x6da08176, 98 },
    { 0x1a808176, 90 },
    { 0, 0 },
    { 0x1db08179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0817d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a108180, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x56408183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f50818e, 33 },
    { 0, 0 },
    { 0xdac08190, 95 },
    { 0x64308191, 95 },
    { 0xac108192, 95 },
    { 0x4d408192, 90 },
    { 0xa5708192, 76 },
    { 0x52008190, 84 },
    { 0x6b808196, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700819d, 33 },
    { 0xf7f0819d, 51 },
    { 0, 0 },
    { 0x9d2081a0, 23 },
    { 0x453081a1, 98 },
    { 0x8fa081a2, 34 },
    { 0, 0 },
    { 0x8d9081a4, 69 },
    { 0, 0 },
    { 0x27a081a6, 23 },
    { 0, 0 },
    { 0xdea081a8, 61 },
    { 0x9c4081a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x233081ac, 52 },
    { 0xb8b081ad, 95 },
    { 0, 0 },
    { 0x8e081af, 78 },
    { 0, 0 },
    { 0x39081b1, 61 },
    { 0x943081b1, 76 },
    { 0, 0 },
    { 0x461081b4, 33 },
    { 0xe4d081b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41081b8, 48 },
    { 0xf21081b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb081bd, 43 },
    { 0xdd7081be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae081c5, 23 },
    { 0x120081c5, 17 },
    { 0, 0 },
    { 0xca4081c8, 53 },
    { 0, 0 },
    { 0x8eb081ca, 90 },
    { 0x8fb081cb, 61 },
    { 0x656081cc, 61 },
    { 0x569081cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb081d0, 35 },
    { 0x9ee081d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d081d6, 90 },
    { 0, 0 },
    { 0xa44081d8, 34 },
    { 0xc38081d8, 93 },
    { 0x6ba081d9, 95 },
    { 0, 0 },
    { 0x822081dc, 33 },
    { 0x1c8081dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x519081e0, 90 },
    { 0xb6d081e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616081e8, 61 },
    { 0x971081e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4081ec, 68 },
    { 0, 0 },
    { 0xbf081ee, 90 },
    { 0xb09081ef, 61 },
    { 0xf3a081f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209081f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a081fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc008203, 61 },
    { 0, 0 },
    { 0x31908205, 90 },
    { 0xe7508206, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a30820d, 33 },
    { 0, 0 },
    { 0x3c40820f, 61 },
    { 0x9c80820f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e308214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73108218, 77 },
    { 0, 0 },
    { 0x1bd0821a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc508220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f708228, 61 },
    { 0xf7d08229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a0822c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61908230, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680823e, 98 },
    { 0x88d0823f, 61 },
    { 0x70e08240, 23 },
    { 0x6f00823f, 23 },
    { 0, 0 },
    { 0x20908243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2708246, 81 },
    { 0x16808246, 23 },
    { 0, 0 },
    { 0xa0208249, 43 },
    { 0, 0 },
    { 0xcba0824b, 98 },
    { 0x6e0824b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58508252, 53 },
    { 0x8e608253, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca508256, 87 },
    { 0xce108257, 77 },
    { 0x40d08258, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60825d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c08263, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c208268, 99 },
    { 0x66908269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb470826f, 95 },
    { 0, 0 },
    { 0x69708271, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed608275, 90 },
    { 0, 0 },
    { 0x9f808277, 23 },
    { 0x9f708278, 52 },
    { 0xf6208278, 61 },
    { 0, 0 },
    { 0xb030827b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2390827e, 90 },
    { 0xed90827f, 78 },
    { 0x64208280, 90 },
    { 0xab708281, 85 },
    { 0x54c08282, 76 },
    { 0, 0 },
    { 0xc7408284, 90 },
    { 0xc3708285, 88 },
    { 0xf4a08286, 90 },
    { 0xcf608287, 33 },
    { 0x76608285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d0828d, 98 },
    { 0, 0 },
    { 0x310828f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10008292, 43 },
    { 0x76308292, 69 },
    { 0xbcb08293, 33 },
    { 0x69408292, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c90829f, 98 },
    { 0, 0 },
    { 0xc2a082a1, 46 },
    { 0x19c082a2, 52 },
    { 0x4ae082a3, 33 },
    { 0x3f2082a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82082aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b082b0, 91 },
    { 0x8b9082b0, 93 },
    { 0x888082b0, 55 },
    { 0, 0 },
    { 0xb26082b4, 95 },
    { 0, 0 },
    { 0x8c6082b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2082ba, 32 },
    { 0, 0 },
    { 0x6fe082bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d082c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b082c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7082c9, 61 },
    { 0x7cf082ca, 90 },
    { 0x914082cb, 55 },
    { 0, 0 },
    { 0xde7082cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b082d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b082d9, 46 },
    { 0xb69082d9, 61 },
    { 0xa9082db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x522082de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b082e1, 33 },
    { 0x415082e2, 52 },
    { 0xd9082e3, 33 },
    { 0, 0 },
    { 0x5082e5, 90 },
    { 0, 0 },
    { 0x662082e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718082ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2082f5, 96 },
    { 0x3d082f6, 91 },
    { 0x745082f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859082fb, 91 },
    { 0, 0 },
    { 0xf38082fd, 95 },
    { 0x13e082fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af08306, 76 },
    { 0x6cd08307, 66 },
    { 0, 0 },
    { 0xb5e08309, 99 },
    { 0x6d30830a, 88 },
    { 0xeaf0830a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d0830e, 61 },
    { 0xc440830f, 93 },
    { 0x9c008310, 81 },
    { 0xa8f08311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd108314, 61 },
    { 0, 0 },
    { 0x85108316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21208319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3120831c, 81 },
    { 0x4e40831d, 46 },
    { 0xf620831c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e08323, 46 },
    { 0xbcd08324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0808329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe110832c, 95 },
    { 0, 0 },
    { 0x4a80832e, 91 },
    { 0, 0 },
    { 0xef308330, 61 },
    { 0xaca08331, 95 },
    { 0x7f208332, 75 },
    { 0x17608333, 76 },
    { 0xa5b08334, 95 },
    { 0x3d008335, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3740833c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff308343, 33 },
    { 0x6be08344, 95 },
    { 0x5a408345, 33 },
    { 0xe2908346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d0834a, 33 },
    { 0, 0 },
    { 0x4400834c, 90 },
    { 0xcd70834c, 61 },
    { 0, 0 },
    { 0x9110834f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52808352, 98 },
    { 0x34a08353, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f08358, 95 },
    { 0xa008359, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0835e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7508361, 95 },
    { 0xece08362, 98 },
    { 0, 0 },
    { 0x8ff08364, 33 },
    { 0xda708364, 43 },
    { 0x54908366, 53 },
    { 0x1ad08367, 19 },
    { 0x1bd08367, 46 },
    { 0xa0508368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650836d, 33 },
    { 0xcbe0836e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e08371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f708375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f0837b, 90 },
    { 0x8bb0837c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d08382, 76 },
    { 0x8b208382, 46 },
    { 0x27008384, 90 },
    { 0, 0 },
    { 0xa8508386, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d108392, 93 },
    { 0xdf508392, 67 },
    { 0xf7208394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a08397, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb0839f, 90 },
    { 0, 0 },
    { 0x1a083a1, 61 },
    { 0xa4e083a1, 76 },
    { 0x2b5083a3, 76 },
    { 0, 0 },
    { 0xef8083a5, 64 },
    { 0x639083a5, 95 },
    { 0x1d7083a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc083aa, 90 },
    { 0x11e083aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8083ae, 33 },
    { 0x569083af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27083b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb083b5, 77 },
    { 0x152083b6, 90 },
    { 0, 0 },
    { 0x760083b8, 23 },
    { 0, 0 },
    { 0xaac083ba, 23 },
    { 0x9be083ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda083be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b083c2, 98 },
    { 0xa58083c3, 52 },
    { 0xf86083c4, 90 },
    { 0x87d083c3, 68 },
    { 0x204083c6, 38 },
    { 0x3c1083c3, 66 },
    { 0x5a5083c3, 76 },
    { 0xb41083c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30083ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb083d6, 43 },
    { 0x21b083d7, 80 },
    { 0xecf083d8, 33 },
    { 0x9db083d7, 99 },
    { 0xfa7083da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6083df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a083e2, 95 },
    { 0x88d083e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c083e6, 94 },
    { 0, 0 },
    { 0xc2b083e8, 100 },
    { 0, 0 },
    { 0x489083ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129083f3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41083f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28308400, 55 },
    { 0, 0 },
    { 0x66808402, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f08406, 33 },
    { 0x9f608407, 52 },
    { 0, 0 },
    { 0x61708409, 17 },
    { 0, 0 },
    { 0x14e0840b, 34 },
    { 0xed80840b, 52 },
    { 0x1770840c, 90 },
    { 0x7e00840e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb308411, 98 },
    { 0, 0 },
    { 0x38408413, 90 },
    { 0x81b08414, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91908418, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb320841c, 61 },
    { 0x4f50841c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c08420, 90 },
    { 0x46708421, 34 },
    { 0xffb08421, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e08427, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0842b, 76 },
    { 0, 0 },
    { 0x9d30842d, 90 },
    { 0xaa90842d, 33 },
    { 0x9440842d, 67 },
    { 0x6708430, 61 },
    { 0, 0 },
    { 0xeb008432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2508435, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59108438, 68 },
    { 0, 0 },
    { 0x2920843a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740843e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x12708441, 98 },
    { 0xcbf08441, 46 },
    { 0x7ff08443, 93 },
    { 0x6d308444, 100 },
    { 0xa8308445, 33 },
    { 0x84d08446, 90 },
    { 0xa2208447, 61 },
    { 0xeab08448, 76 },
    { 0, 0 },
    { 0x8be0844a, 90 },
    { 0, 0 },
    { 0xc40844c, 61 },
    { 0xcd40844d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0108452, 68 },
    { 0x21308453, 98 },
    { 0x88908453, 90 },
    { 0, 0 },
    { 0xab608456, 61 },
    { 0x1c08457, 61 },
    { 0xc3908458, 98 },
    { 0x73008457, 84 },
    { 0x1a808457, 55 },
    { 0x94a0845b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f0845f, 90 },
    { 0, 0 },
    { 0xcb708461, 17 },
    { 0, 0 },
    { 0x5cd08463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96108469, 46 },
    { 0x68108469, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f08470, 33 },
    { 0x55808471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08474, 33 },
    { 0xfad08475, 23 },
    { 0xe1b08475, 66 },
    { 0, 0 },
    { 0x9c808478, 23 },
    { 0, 0 },
    { 0xcf00847a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b0847e, 84 },
    { 0, 0 },
    { 0x99708480, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c08487, 34 },
    { 0xbd108487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5300848f, 23 },
    { 0x3f008490, 35 },
    { 0, 0 },
    { 0xf6308492, 33 },
    { 0xc5a08493, 23 },
    { 0xabb08494, 90 },
    { 0x64108494, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5740849b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470849e, 98 },
    { 0xb10849e, 90 },
    { 0xd980849e, 90 },
    { 0x343084a1, 95 },
    { 0x712084a2, 78 },
    { 0xa0b084a3, 66 },
    { 0xe02084a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23084a9, 33 },
    { 0x217084a9, 75 },
    { 0xb45084ab, 90 },
    { 0, 0 },
    { 0x843084ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36084b3, 95 },
    { 0xbef084b4, 95 },
    { 0, 0 },
    { 0x8aa084b6, 76 },
    { 0, 0 },
    { 0x989084b8, 95 },
    { 0xbb9084b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18084bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x737084bf, 90 },
    { 0x3c7084bf, 61 },
    { 0xd35084c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x104084c4, 99 },
    { 0, 0 },
    { 0x79c084c6, 85 },
    { 0x9ec084c7, 23 },
    { 0x3d5084c8, 23 },
    { 0x2f1084c8, 61 },
    { 0xe95084c7, 61 },
    { 0xf74084cb, 67 },
    { 0xb7f084cc, 34 },
    { 0x25e084cc, 76 },
    { 0xcac084cb, 98 },
    { 0, 0 },
    { 0xb48084d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57084d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11084d7, 33 },
    { 0xfa3084d7, 76 },
    { 0, 0 },
    { 0x8af084da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4084dd, 77 },
    { 0x7e4084de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d084e1, 35 },
    { 0x75f084e2, 33 },
    { 0x5b7084e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577084ea, 34 },
    { 0xa36084eb, 85 },
    { 0x5b7084ea, 95 },
    { 0, 0 },
    { 0x474084ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x535084f1, 61 },
    { 0x7b084f1, 34 },
    { 0xcad084f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90084f7, 33 },
    { 0x186084f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45084fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b08503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c0850e, 61 },
    { 0xa220850e, 90 },
    { 0x84e0850f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100851a, 90 },
    { 0xac90851b, 33 },
    { 0x57d0851b, 23 },
    { 0, 0 },
    { 0x700851e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed08524, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d08528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60852b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f00852e, 66 },
    { 0x92e0852f, 98 },
    { 0x9360852f, 67 },
    { 0x8008531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46908534, 99 },
    { 0x6a908535, 52 },
    { 0, 0 },
    { 0x3bb08537, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10853e, 91 },
    { 0x8d90853f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a608543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59608547, 61 },
    { 0xd1908548, 66 },
    { 0x23a08547, 85 },
    { 0xaee0854a, 98 },
    { 0, 0 },
    { 0x3d40854c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7570854f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b708553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa450855a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1320855f, 76 },
    { 0xf6d08560, 43 },
    { 0xe4508560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf08566, 90 },
    { 0x42f08567, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e008571, 65 },
    { 0xbcf08572, 90 },
    { 0xff308572, 55 },
    { 0, 0 },
    { 0x1bf08575, 61 },
    { 0xd9408576, 69 },
    { 0, 0 },
    { 0x8d708578, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1750857b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31308584, 33 },
    { 0, 0 },
    { 0x37d08586, 90 },
    { 0x6dd08587, 77 },
    { 0, 0 },
    { 0x99108589, 95 },
    { 0x4508589, 23 },
    { 0x3f00858b, 75 },
    { 0x4d008589, 34 },
    { 0x2ce0858d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12708590, 90 },
    { 0xf3908591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e08594, 61 },
    { 0x3e408595, 43 },
    { 0xace08595, 69 },
    { 0xc1708597, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e0859a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd900859d, 90 },
    { 0, 0 },
    { 0xc00859f, 23 },
    { 0x3d70859f, 98 },
    { 0x21085a0, 99 },
    { 0xe49085a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236085a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76085ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f085b4, 55 },
    { 0, 0 },
    { 0xdc0085b6, 99 },
    { 0x583085b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52085c0, 84 },
    { 0, 0 },
    { 0xbee085c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded085c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af085cc, 66 },
    { 0x263085cd, 46 },
    { 0x19d085ce, 55 },
    { 0x9b3085cc, 66 },
    { 0xd2a085d0, 95 },
    { 0, 0 },
    { 0xbcb085d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105085d8, 61 },
    { 0x76f085d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd085dd, 99 },
    { 0xd2d085de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db085e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba085e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78085eb, 61 },
    { 0x968085ec, 36 },
    { 0x412085ed, 61 },
    { 0xd9c085ed, 66 },
    { 0xd05085ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60085f2, 65 },
    { 0xc00085f2, 55 },
    { 0xe5b085f4, 52 },
    { 0xf1a085f4, 43 },
    { 0xaab085f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x895085f9, 61 },
    { 0, 0 },
    { 0x6c8085fb, 33 },
    { 0xf0d085fb, 90 },
    { 0xded085fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f08602, 61 },
    { 0xfb808603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c908606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a908610, 68 },
    { 0x69108610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f90861a, 75 },
    { 0xb910861a, 78 },
    { 0xe670861b, 93 },
    { 0x6660861d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d608621, 98 },
    { 0x12a08622, 61 },
    { 0x8cb08623, 93 },
    { 0x68b08623, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a508627, 55 },
    { 0x30d08627, 23 },
    { 0, 0 },
    { 0x3ab0862a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b0862d, 90 },
    { 0x4de0862d, 95 },
    { 0x9da0862f, 23 },
    { 0x27608630, 76 },
    { 0xb2f0862f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d08634, 23 },
    { 0x22308635, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910863d, 88 },
    { 0x4f70863e, 67 },
    { 0x8870863e, 90 },
    { 0x2080863f, 95 },
    { 0x7cc0863e, 91 },
    { 0x4be08642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0208645, 33 },
    { 0xcbe08646, 98 },
    { 0, 0 },
    { 0x46e08648, 90 },
    { 0x98608648, 90 },
    { 0x5b508649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb20864e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b108652, 75 },
    { 0x79508652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e08656, 99 },
    { 0, 0 },
    { 0x6e08658, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0865b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7360865f, 33 },
    { 0xdca0865f, 98 },
    { 0x98f08661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee08664, 90 },
    { 0x71108665, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee208669, 69 },
    { 0x3210866a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e608671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb08674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e08677, 90 },
    { 0, 0 },
    { 0x65408679, 23 },
    { 0, 0 },
    { 0x3d60867b, 93 },
    { 0, 0 },
    { 0x3010867d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80208683, 100 },
    { 0x44508684, 52 },
    { 0x5cb08684, 95 },
    { 0x97d08684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5600868f, 98 },
    { 0xb3a0868f, 100 },
    { 0xb7408691, 43 },
    { 0xcd308691, 4 },
    { 0x70708693, 33 },
    { 0x52108694, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff708697, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694086a2, 46 },
    { 0, 0 },
    { 0x69086a4, 33 },
    { 0xb23086a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a086aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45086ae, 96 },
    { 0xf6086af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405086bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32086c3, 61 },
    { 0xff9086c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e086c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478086ce, 33 },
    { 0x153086ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7086d5, 61 },
    { 0xeb5086d6, 66 },
    { 0xc04086d7, 78 },
    { 0x5fc086d5, 67 },
    { 0x8fb086d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686086e2, 61 },
    { 0x233086e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701086e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5086ea, 46 },
    { 0xf78086ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d086ee, 78 },
    { 0x593086ef, 61 },
    { 0xad4086f0, 90 },
    { 0, 0 },
    { 0x7d9086f2, 98 },
    { 0, 0 },
    { 0x39f086f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8086f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9086ff, 61 },
    { 0, 0 },
    { 0xe308701, 33 },
    { 0xb7808702, 33 },
    { 0x4f08703, 34 },
    { 0x37d08704, 33 },
    { 0xd3708705, 98 },
    { 0, 0 },
    { 0xd3008707, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b50870e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed08712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f08715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9770871b, 81 },
    { 0xe120871c, 33 },
    { 0x720871d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e08725, 66 },
    { 0xa4208726, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea0872b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a0872e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b208731, 23 },
    { 0x7ca08731, 99 },
    { 0xd2e08733, 61 },
    { 0x43208734, 69 },
    { 0x1fd08733, 61 },
    { 0xbe808734, 23 },
    { 0xb7208737, 98 },
    { 0x25b08733, 20 },
    { 0x2bc08739, 90 },
    { 0x4e0873a, 90 },
    { 0xa430873b, 85 },
    { 0xea708739, 55 },
    { 0, 0 },
    { 0xc650873e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74808747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x820874a, 98 },
    { 0x8bf0874b, 55 },
    { 0xf6a0874c, 90 },
    { 0x300874d, 84 },
    { 0x5710874e, 64 },
    { 0x29d0874f, 91 },
    { 0x80b08750, 95 },
    { 0x8530874d, 95 },
    { 0x6b308752, 100 },
    { 0x2050874d, 98 },
    { 0x1b308754, 76 },
    { 0, 0 },
    { 0xd9908756, 85 },
    { 0x78708756, 34 },
    { 0x9e808758, 98 },
    { 0x4ad08758, 69 },
    { 0x6310875a, 55 },
    { 0x8040875b, 61 },
    { 0, 0 },
    { 0x8d60875d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b708760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c08766, 33 },
    { 0x87208767, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50876a, 76 },
    { 0xe690876b, 99 },
    { 0x20e0876b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e10876f, 33 },
    { 0x94708770, 33 },
    { 0xee608771, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0108774, 78 },
    { 0xcf208775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e0877b, 50 },
    { 0, 0 },
    { 0x1770877d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34408780, 33 },
    { 0xce608781, 61 },
    { 0x7b708780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e008785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9910878b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c08792, 95 },
    { 0x90f08792, 76 },
    { 0x90508794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca08798, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1860879b, 98 },
    { 0x30879b, 23 },
    { 0xa810879d, 67 },
    { 0, 0 },
    { 0x1730879f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4087a3, 36 },
    { 0x47d087a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35087a7, 77 },
    { 0, 0 },
    { 0xdbf087a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d087ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d087af, 46 },
    { 0, 0 },
    { 0xc53087b1, 46 },
    { 0xc3f087b2, 75 },
    { 0xf20087b3, 84 },
    { 0xb33087b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4087b7, 90 },
    { 0xff5087b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x219087bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad087c0, 99 },
    { 0, 0 },
    { 0x590087c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a087c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x912087cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7087ce, 95 },
    { 0x74d087cf, 61 },
    { 0xf07087d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4087d6, 33 },
    { 0, 0 },
    { 0xc6c087d8, 23 },
    { 0xce5087d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5087dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec087e4, 61 },
    { 0xd46087e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82087ef, 33 },
    { 0, 0 },
    { 0xf8a087f1, 74 },
    { 0, 0 },
    { 0x5fe087f3, 23 },
    { 0x64f087f3, 100 },
    { 0, 0 },
    { 0xe3b087f6, 43 },
    { 0, 0 },
    { 0x904087f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb087fb, 98 },
    { 0x28c087fc, 33 },
    { 0xf54087fc, 90 },
    { 0x5e3087fe, 84 },
    { 0x1e6087ff, 55 },
    { 0xb0208800, 90 },
    { 0x1fd087fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4508809, 98 },
    { 0, 0 },
    { 0x9450880b, 95 },
    { 0x3fc0880c, 33 },
    { 0, 0 },
    { 0x8710880e, 95 },
    { 0xb680880f, 61 },
    { 0x3b80880f, 34 },
    { 0xd5208811, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c08816, 87 },
    { 0x7a508817, 55 },
    { 0xec08818, 61 },
    { 0x19408819, 66 },
    { 0xdb108819, 90 },
    { 0, 0 },
    { 0x6f70881c, 23 },
    { 0x41a0881d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee08820, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a508823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880882c, 33 },
    { 0x510882d, 33 },
    { 0, 0 },
    { 0x72d0882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d508833, 98 },
    { 0xa1108834, 93 },
    { 0, 0 },
    { 0x4f708836, 76 },
    { 0x1c08837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd20883c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51808843, 99 },
    { 0xdae08844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c08847, 90 },
    { 0, 0 },
    { 0xa3708849, 90 },
    { 0x5ff0884a, 46 },
    { 0, 0 },
    { 0xd840884c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c40884f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec908852, 88 },
    { 0x3f608853, 91 },
    { 0x50e08852, 33 },
    { 0x57c08855, 93 },
    { 0xa0f08856, 34 },
    { 0xff508856, 88 },
    { 0x5508858, 33 },
    { 0xd708855, 23 },
    { 0xb5408855, 34 },
    { 0x14508858, 95 },
    { 0x52c08858, 90 },
    { 0xa0408857, 77 },
    { 0, 0 },
    { 0x340885f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e08863, 98 },
    { 0xf5308864, 61 },
    { 0x42b08864, 34 },
    { 0x28a08865, 55 },
    { 0xa7508867, 85 },
    { 0x27208865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c40886b, 61 },
    { 0x88e0886c, 61 },
    { 0x4460886d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed08871, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8508876, 95 },
    { 0, 0 },
    { 0xa708878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f0887b, 34 },
    { 0x95b0887b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a508889, 91 },
    { 0, 0 },
    { 0x3500888b, 77 },
    { 0xb980888b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60888f, 90 },
    { 0xbfb08890, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33208895, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0889b, 61 },
    { 0x8b80889c, 84 },
    { 0x21e0889c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91088a1, 61 },
    { 0xbf0088a2, 78 },
    { 0x9ee088a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3088a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25088ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d088b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e088b5, 95 },
    { 0xe32088b6, 90 },
    { 0x8da088b5, 98 },
    { 0xb3c088b8, 23 },
    { 0xde9088b8, 33 },
    { 0xe38088ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5088bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943088c0, 90 },
    { 0x3ff088c1, 33 },
    { 0x958088c1, 95 },
    { 0x679088c1, 55 },
    { 0, 0 },
    { 0xffc088c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c088c8, 78 },
    { 0x4c1088c9, 90 },
    { 0xf30088c9, 76 },
    { 0xbb2088cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x883088ce, 55 },
    { 0, 0 },
    { 0xf3b088d0, 33 },
    { 0, 0 },
    { 0x1ea088d2, 75 },
    { 0, 0 },
    { 0x420088d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e088d8, 90 },
    { 0, 0 },
    { 0x5db088da, 95 },
    { 0, 0 },
    { 0x4c3088dc, 33 },
    { 0x261088dd, 61 },
    { 0x84c088de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f088e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a088e7, 76 },
    { 0x139088e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17088eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x246088ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e088f1, 93 },
    { 0xd9d088f2, 61 },
    { 0, 0 },
    { 0xa0c088f4, 90 },
    { 0, 0 },
    { 0x464088f6, 85 },
    { 0, 0 },
    { 0x650088f8, 83 },
    { 0, 0 },
    { 0xe7088fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806088ff, 33 },
    { 0, 0 },
    { 0x76508901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b08904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d608909, 99 },
    { 0xca20890a, 61 },
    { 0xfa40890b, 33 },
    { 0x6cd08909, 55 },
    { 0xaf10890d, 90 },
    { 0x20f0890d, 98 },
    { 0x8950890a, 99 },
    { 0xb9d08910, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a08914, 61 },
    { 0x85c08915, 85 },
    { 0xfcb08914, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d40891c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15808920, 95 },
    { 0x5ef08921, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf08924, 68 },
    { 0xb4c08925, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6108929, 33 },
    { 0xf790892a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6208931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45008936, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b708939, 78 },
    { 0, 0 },
    { 0xdaa0893b, 91 },
    { 0xa6f0893c, 90 },
    { 0, 0 },
    { 0xa380893e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e08949, 90 },
    { 0, 0 },
    { 0x4480894b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c0894e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf08951, 84 },
    { 0xe1908952, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43008959, 91 },
    { 0xa5f08959, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d0895e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d08962, 95 },
    { 0x81e08962, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6508966, 90 },
    { 0xdd808967, 95 },
    { 0x6a208968, 66 },
    { 0x6c508969, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080896c, 66 },
    { 0xc120896c, 76 },
    { 0x3a40896c, 90 },
    { 0x1c90896f, 90 },
    { 0xbfa08970, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa08979, 61 },
    { 0, 0 },
    { 0xec30897b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd0897f, 98 },
    { 0xb8f0897f, 33 },
    { 0, 0 },
    { 0x76208982, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708988, 90 },
    { 0x82c08988, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1830898e, 90 },
    { 0xdd80898f, 77 },
    { 0x68c08990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a808998, 55 },
    { 0xeb608999, 87 },
    { 0, 0 },
    { 0x9950899b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf0899e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651089a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a089a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4089ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e089b2, 61 },
    { 0xab3089b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5089bd, 96 },
    { 0, 0 },
    { 0x795089bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f089c2, 95 },
    { 0xcf8089c3, 90 },
    { 0, 0 },
    { 0x598089c5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43089cb, 84 },
    { 0xe3b089cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1089d0, 33 },
    { 0xb24089d1, 91 },
    { 0, 0 },
    { 0xf13089d3, 33 },
    { 0x6c089d3, 48 },
    { 0, 0 },
    { 0xb4a089d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6089da, 23 },
    { 0xb5f089da, 95 },
    { 0x6df089dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b089e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df089e4, 55 },
    { 0xe8c089e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8089eb, 90 },
    { 0xc2089ec, 65 },
    { 0xc20089ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d089f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3089f5, 98 },
    { 0, 0 },
    { 0x75a089f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a089fa, 42 },
    { 0x594089fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd089ff, 90 },
    { 0xd5508a00, 84 },
    { 0x9f508a01, 78 },
    { 0xef708a01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb508a05, 23 },
    { 0x51608a05, 33 },
    { 0xf808a07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4408a0a, 98 },
    { 0x90408a0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb08a15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f108a18, 23 },
    { 0x1f908a19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e08a1d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc608a21, 98 },
    { 0, 0 },
    { 0xe4508a23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb008a26, 93 },
    { 0xd5208a27, 34 },
    { 0x4bf08a26, 33 },
    { 0x1e908a29, 33 },
    { 0xb2b08a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb08a30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b208a36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b08a3b, 75 },
    { 0xc1808a3c, 22 },
    { 0x49c08a3d, 77 },
    { 0x98608a3d, 99 },
    { 0x99c08a3f, 90 },
    { 0xc708a40, 33 },
    { 0x44b08a3e, 98 },
    { 0x73f08a42, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc508a47, 61 },
    { 0x22b08a48, 66 },
    { 0, 0 },
    { 0x14e08a4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d08a4f, 34 },
    { 0, 0 },
    { 0xed808a51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59208a58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25708a5c, 90 },
    { 0x2bf08a5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8708a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2408a6a, 95 },
    { 0, 0 },
    { 0x1c608a6c, 90 },
    { 0x84308a6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb508a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad708a74, 78 },
    { 0xafc08a75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b08a78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08a7d, 67 },
    { 0, 0 },
    { 0x99108a7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82908a89, 90 },
    { 0, 0 },
    { 0x91408a8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a408a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96008a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d408a96, 55 },
    { 0xdc308a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae708a9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e08aa8, 90 },
    { 0xb3c08aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa08aae, 90 },
    { 0, 0 },
    { 0x60008ab0, 76 },
    { 0x56608ab1, 98 },
    { 0xe8008ab1, 98 },
    { 0x30208ab3, 61 },
    { 0x42708ab4, 23 },
    { 0xb0308ab3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f08ab9, 95 },
    { 0x67508aba, 85 },
    { 0x14208ab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f208abe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b908ac2, 87 },
    { 0xdb008ac3, 23 },
    { 0x3f208ac2, 95 },
    { 0x45a08ac5, 77 },
    { 0x3bf08ac6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44108ac9, 34 },
    { 0, 0 },
    { 0x70708acb, 33 },
    { 0xdde08acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2108acf, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x23508ad2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6208ad5, 85 },
    { 0xcca08ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27608ad9, 95 },
    { 0x83608ada, 87 },
    { 0xb7808adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e08ae0, 46 },
    { 0, 0 },
    { 0xe1908ae2, 33 },
    { 0x6d108ae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f08ae7, 90 },
    { 0, 0 },
    { 0x8fd08ae9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f08aec, 95 },
    { 0x26008aed, 61 },
    { 0xda008aed, 42 },
    { 0xa0208aed, 90 },
    { 0, 0 },
    { 0x25e08af1, 61 },
    { 0xa3308af2, 95 },
    { 0, 0 },
    { 0x95908af4, 90 },
    { 0x6b408af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6208afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc08b01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e08b04, 91 },
    { 0, 0 },
    { 0x64408b06, 78 },
    { 0x41a08b06, 95 },
    { 0x8b308b08, 88 },
    { 0xdac08b09, 23 },
    { 0x75b08b0a, 84 },
    { 0xdcf08b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd208b15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83508b19, 95 },
    { 0, 0 },
    { 0xb7d08b1b, 90 },
    { 0x42d08b1b, 93 },
    { 0x3f608b1b, 55 },
    { 0, 0 },
    { 0xcea08b1f, 34 },
    { 0x37108b1f, 55 },
    { 0, 0 },
    { 0x66a08b22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97308b27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34808b2f, 40 },
    { 0, 0 },
    { 0x50508b31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60008b37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b08b3c, 23 },
    { 0, 0 },
    { 0x70a08b3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d08b44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c08b4c, 91 },
    { 0x28208b4d, 33 },
    { 0x15f08b4c, 23 },
    { 0x23508b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66108b56, 61 },
    { 0x9e108b56, 95 },
    { 0xb4e08b57, 90 },
    { 0xebc08b58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60808b5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10408b64, 99 },
    { 0, 0 },
    { 0x1e308b66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf508b6a, 77 },
    { 0xb4308b6b, 33 },
    { 0x45408b6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x24408b6f, 66 },
    { 0, 0 },
    { 0x68008b71, 23 },
    { 0, 0 },
    { 0x16308b73, 78 },
    { 0x5e708b74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c08b78, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e08b80, 46 },
    { 0xf7b08b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80208b89, 77 },
    { 0x71e08b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8608b95, 95 },
    { 0xff108b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa408b99, 76 },
    { 0x6a008b99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba008b9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1708ba5, 95 },
    { 0x81808ba5, 101 },
    { 0x4c308ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9408bb8, 88 },
    { 0, 0 },
    { 0xf6408bba, 23 },
    { 0x54308bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d08bbe, 74 },
    { 0xd9708bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98708bc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90308bc8, 98 },
    { 0x1dd08bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30908bd1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f08bd6, 90 },
    { 0x6b908bd7, 33 },
    { 0x9908bd7, 23 },
    { 0x16208bd8, 34 },
    { 0x80e08bd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26808bde, 90 },
    { 0x2ab08bdf, 95 },
    { 0, 0 },
    { 0xd308be1, 95 },
    { 0, 0 },
    { 0xb5d08be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2308be6, 98 },
    { 0xa108be7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82408beb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab008bee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c08bf1, 33 },
    { 0, 0 },
    { 0xda508bf3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a08bf8, 33 },
    { 0, 0 },
    { 0x81e08bfa, 90 },
    { 0x6f408bfb, 33 },
    { 0x78208bfc, 90 },
    { 0x2ff08bfb, 33 },
    { 0x99508bfe, 55 },
    { 0, 0 },
    { 0x7b808c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b108c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d08c0c, 23 },
    { 0xc8908c0d, 55 },
    { 0xa5808c0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb08c17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c608c1c, 95 },
    { 0, 0 },
    { 0x9c608c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08c22, 33 },
    { 0xbc08c22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b08c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10008c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d08c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc08c37, 61 },
    { 0x69e08c37, 76 },
    { 0, 0 },
    { 0xd4808c3a, 43 },
    { 0, 0 },
    { 0xcbf08c3c, 23 },
    { 0x78508c3d, 33 },
    { 0xc8908c3e, 99 },
    { 0x32f08c3f, 90 },
    { 0x81b08c40, 61 },
    { 0xd2d08c41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55808c47, 33 },
    { 0x8a308c48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c108c4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99308c51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d08c58, 77 },
    { 0x44208c59, 95 },
    { 0x30d08c5a, 98 },
    { 0x96108c58, 77 },
    { 0x31708c5c, 40 },
    { 0x35c08c5a, 90 },
    { 0, 0 },
    { 0xa5708c5f, 66 },
    { 0x66608c5f, 61 },
    { 0xdf308c61, 23 },
    { 0xaeb08c5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd08c65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b08c69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5308c6e, 34 },
    { 0xa8508c6e, 61 },
    { 0, 0 },
    { 0x5d508c71, 61 },
    { 0x68708c71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8408c75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d208c7a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5108c7d, 95 },
    { 0xf3008c7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a08c88, 33 },
    { 0xea708c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa808c8c, 95 },
    { 0, 0 },
    { 0xe0908c8e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e08c92, 55 },
    { 0xd3d08c92, 93 },
    { 0x2e808c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1408c97, 33 },
    { 0, 0 },
    { 0xf2408c99, 95 },
    { 0xa6108c99, 90 },
    { 0x8f208c9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b08c9e, 23 },
    { 0, 0 },
    { 0x4208ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23008ca4, 100 },
    { 0xbc708ca4, 55 },
    { 0xfa008ca6, 52 },
    { 0xe9b08ca7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92308cae, 95 },
    { 0, 0 },
    { 0x2b408cb0, 23 },
    { 0xb8208cb1, 76 },
    { 0xa8808cb1, 99 },
    { 0x4e708cb1, 88 },
    { 0xb3508cb2, 87 },
    { 0, 0 },
    { 0x4c308cb6, 46 },
    { 0x14208cb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de08cbd, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb08cc0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9008cc3, 95 },
    { 0xc2608cc4, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd08cc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b08ccf, 78 },
    { 0x53d08ccf, 90 },
    { 0x4f108ccf, 61 },
    { 0xd7d08ccf, 100 },
    { 0x42808cd3, 95 },
    { 0xd4c08cd4, 68 },
    { 0x27208cd4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef08cd9, 23 },
    { 0xd0b08cda, 69 },
    { 0xcdf08cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e08ce1, 98 },
    { 0x43f08ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df08ce7, 33 },
    { 0xe1808ce7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7008cee, 40 },
    { 0, 0 },
    { 0x1e008cf0, 99 },
    { 0x7d208cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18508cf4, 88 },
    { 0, 0 },
    { 0xb7e08cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64708cf9, 33 },
    { 0xaa08cfa, 33 },
    { 0xf1f08cfa, 55 },
    { 0x1b08cfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b08d02, 46 },
    { 0, 0 },
    { 0x98308d04, 95 },
    { 0x56508d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d08d0c, 33 },
    { 0x55308d0d, 90 },
    { 0xd5e08d0e, 95 },
    { 0xa0208d0e, 33 },
    { 0x40808d10, 34 },
    { 0xbc108d0f, 90 },
    { 0, 0 },
    { 0xb0908d13, 17 },
    { 0x15a08d14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb508d1a, 23 },
    { 0xfd008d1b, 78 },
    { 0x50408d1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f08d20, 61 },
    { 0, 0 },
    { 0xaa408d22, 61 },
    { 0xb3108d23, 75 },
    { 0, 0 },
    { 0x42508d25, 98 },
    { 0x20508d26, 33 },
    { 0xa9b08d27, 61 },
    { 0, 0 },
    { 0x72e08d29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d08d2c, 98 },
    { 0x35608d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92408d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8708d34, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde08d38, 98 },
    { 0, 0 },
    { 0x7e008d3a, 95 },
    { 0x13d08d3b, 23 },
    { 0x90808d3a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x69108d3f, 65 },
    { 0x3e708d3f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa08d45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf208d49, 61 },
    { 0xdd708d4a, 67 },
    { 0x90308d4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c608d4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf08d56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c08d59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70908d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0608d65, 95 },
    { 0, 0 },
    { 0x95808d67, 24 },
    { 0x4b108d68, 43 },
    { 0x72708d67, 90 },
    { 0xdb708d6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e808d6e, 98 },
    { 0xc2308d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a08d72, 84 },
    { 0x1ac08d73, 76 },
    { 0xd7b08d72, 98 },
    { 0xb6208d75, 61 },
    { 0xa1308d76, 52 },
    { 0x1ed08d74, 61 },
    { 0xbf808d78, 98 },
    { 0x19c08d76, 61 },
    { 0, 0 },
    { 0x4408d7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd108d7e, 55 },
    { 0xce108d7e, 33 },
    { 0x2fe08d80, 99 },
    { 0xe3708d81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba508d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc208d88, 61 },
    { 0xce708d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42208d8c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed208d90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f08d9f, 33 },
    { 0, 0 },
    { 0x92f08da1, 95 },
    { 0, 0 },
    { 0xc3008da3, 33 },
    { 0, 0 },
    { 0x7b208da5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb08dad, 61 },
    { 0xc9108dae, 33 },
    { 0x5a708daf, 90 },
    { 0x14608db0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b08db6, 46 },
    { 0, 0 },
    { 0x22108db8, 69 },
    { 0, 0 },
    { 0x8108dba, 90 },
    { 0x35c08dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf308dbe, 52 },
    { 0x108dbe, 90 },
    { 0, 0 },
    { 0x57f08dc1, 95 },
    { 0, 0 },
    { 0x7df08dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc08dc7, 22 },
    { 0x8e108dc8, 91 },
    { 0x20408dc9, 90 },
    { 0xd6708dc7, 76 },
    { 0x69f08dcb, 98 },
    { 0xf6908dcc, 90 },
    { 0xfa008dcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2608dd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e08dd9, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d08ddc, 61 },
    { 0xb3b08ddd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e08de0, 34 },
    { 0xc2d08de1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f08de5, 69 },
    { 0x8fb08de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15208deb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b08df3, 61 },
    { 0, 0 },
    { 0x14608df5, 90 },
    { 0x14208df6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf08df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78108dfc, 23 },
    { 0, 0 },
    { 0xdb808dfe, 33 },
    { 0x7ac08dff, 28 },
    { 0xc1d08dfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73808e09, 95 },
    { 0x9a108e0a, 77 },
    { 0xbf408e0b, 98 },
    { 0x7a908e0c, 98 },
    { 0xfed08e0d, 56 },
    { 0x89f08e0c, 33 },
    { 0, 0 },
    { 0xe3608e10, 61 },
    { 0x55508e11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb08e15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0408e18, 33 },
    { 0, 0 },
    { 0x49e08e1a, 66 },
    { 0, 0 },
    { 0x44208e1c, 33 },
    { 0xfde08e1d, 61 },
    { 0xf0808e1e, 68 },
    { 0x9d208e1d, 61 },
    { 0xf2d08e20, 95 },
    { 0xc9f08e20, 93 },
    { 0x3c408e1d, 31 },
    { 0xf5e08e23, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd08e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c08e2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e108e31, 98 },
    { 0, 0 },
    { 0x92608e33, 88 },
    { 0xc4308e34, 43 },
    { 0, 0 },
    { 0x11208e36, 23 },
    { 0xbdf08e37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18208e3a, 55 },
    { 0xeb708e3a, 33 },
    { 0x32408e3c, 93 },
    { 0xb3708e3d, 66 },
    { 0xd408e3e, 98 },
    { 0xd3a08e3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7008e43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85108e47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2808e4b, 33 },
    { 0, 0 },
    { 0xe2808e4d, 33 },
    { 0x6c008e4e, 98 },
    { 0, 0 },
    { 0x29a08e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff08e58, 66 },
    { 0xe708e59, 55 },
    { 0xba708e58, 95 },
    { 0, 0 },
    { 0x3c708e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd08e61, 46 },
    { 0x8a008e61, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c08e65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08e6b, 65 },
    { 0x48108e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9908e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6308e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66208e7b, 76 },
    { 0xc9408e7c, 52 },
    { 0xf9408e7c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b608e82, 88 },
    { 0x95908e82, 61 },
    { 0xa7b08e84, 55 },
    { 0x6ae08e85, 33 },
    { 0x61908e86, 76 },
    { 0, 0 },
    { 0x32d08e88, 90 },
    { 0, 0 },
    { 0x8bd08e8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2808e8d, 43 },
    { 0xea508e8e, 95 },
    { 0xf6f08e8e, 91 },
    { 0, 0 },
    { 0x3d108e91, 93 },
    { 0x53108e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08e96, 95 },
    { 0xcba08e97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98908ea5, 90 },
    { 0xcc608ea6, 65 },
    { 0x38e08ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e08ead, 98 },
    { 0xc1408eae, 43 },
    { 0x2dd08eaf, 91 },
    { 0, 0 },
    { 0x3b08eb1, 89 },
    { 0x54e08eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee808eb6, 95 },
    { 0, 0 },
    { 0x3ea08eb8, 76 },
    { 0xd7308eb9, 76 },
    { 0xdd508eb8, 61 },
    { 0, 0 },
    { 0x2ba08ebc, 93 },
    { 0x38108ebd, 88 },
    { 0xbf608ebd, 67 },
    { 0, 0 },
    { 0xeb108ec0, 66 },
    { 0x2c608ec1, 93 },
    { 0, 0 },
    { 0x82b08ec3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9308ec6, 91 },
    { 0, 0 },
    { 0x40808ec8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a508ecb, 99 },
    { 0x32008ecc, 90 },
    { 0xdd208ecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29808ed0, 84 },
    { 0, 0 },
    { 0x68108ed2, 98 },
    { 0, 0 },
    { 0xbaf08ed4, 95 },
    { 0x7db08ed4, 61 },
    { 0, 0 },
    { 0xb1208ed7, 23 },
    { 0, 0 },
    { 0x52f08ed9, 55 },
    { 0x56308ed9, 98 },
    { 0xadf08ed9, 93 },
    { 0, 0 },
    { 0xb4c08edd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2408ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b08ee3, 33 },
    { 0, 0 },
    { 0x8608ee5, 95 },
    { 0, 0 },
    { 0x27b08ee7, 34 },
    { 0x2e608ee8, 78 },
    { 0xa7008ee7, 34 },
    { 0xe3c08eea, 55 },
    { 0x28908eeb, 61 },
    { 0x12908eec, 61 },
    { 0xf2e08eec, 43 },
    { 0xd8808eec, 98 },
    { 0x9f708eef, 52 },
    { 0x12008ef0, 66 },
    { 0x47408ef0, 43 },
    { 0x93508ef1, 34 },
    { 0xa9d08ef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d708ef6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f908efa, 66 },
    { 0x7808efb, 85 },
    { 0xfe908efa, 78 },
    { 0xbc908efc, 76 },
    { 0x5da08efb, 95 },
    { 0xf1208eff, 61 },
    { 0, 0 },
    { 0xca08f01, 61 },
    { 0, 0 },
    { 0xb3808f03, 90 },
    { 0x62808f03, 66 },
    { 0x14108f05, 78 },
    { 0xea108f05, 43 },
    { 0, 0 },
    { 0x31f08f08, 97 },
    { 0x1e908f09, 93 },
    { 0x97208f0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2108f0f, 98 },
    { 0, 0 },
    { 0x46108f11, 98 },
    { 0xca008f12, 76 },
    { 0x12e08f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc608f18, 23 },
    { 0x8e408f18, 84 },
    { 0x52308f19, 33 },
    { 0x7f408f1b, 94 },
    { 0, 0 },
    { 0xb7a08f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb808f22, 93 },
    { 0x71508f23, 95 },
    { 0x48008f23, 67 },
    { 0x40108f25, 90 },
    { 0x12608f26, 23 },
    { 0x44608f27, 33 },
    { 0xd9808f22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b08f2b, 17 },
    { 0x9bd08f2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad08f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b08f33, 34 },
    { 0xec608f33, 61 },
    { 0, 0 },
    { 0xe3308f36, 55 },
    { 0x6b808f36, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x57608f3a, 64 },
    { 0x8ff08f3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd08f44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97208f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f608f4a, 43 },
    { 0xc6808f4b, 76 },
    { 0x54908f4b, 98 },
    { 0, 0 },
    { 0xdaa08f4e, 95 },
    { 0, 0 },
    { 0xdee08f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42708f54, 33 },
    { 0xd4a08f55, 55 },
    { 0xc7708f56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de08f59, 95 },
    { 0xfab08f5a, 90 },
    { 0xb4608f5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb08f63, 99 },
    { 0, 0 },
    { 0xdbb08f65, 84 },
    { 0xc4208f65, 98 },
    { 0, 0 },
    { 0x35d08f68, 67 },
    { 0xc0308f68, 33 },
    { 0, 0 },
    { 0x46f08f6b, 61 },
    { 0, 0 },
    { 0xd3408f6d, 98 },
    { 0x36508f6d, 76 },
    { 0xb8f08f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf08f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f08f81, 95 },
    { 0x61408f82, 8 },
    { 0x64d08f81, 98 },
    { 0x88908f84, 76 },
    { 0xfde08f85, 95 },
    { 0x42308f85, 33 },
    { 0xde008f87, 46 },
    { 0x88708f85, 33 },
    { 0x97208f89, 95 },
    { 0xf9c08f88, 76 },
    { 0x12308f87, 87 },
    { 0xacf08f84, 61 },
    { 0xc9b08f8a, 61 },
    { 0xe1d08f8e, 65 },
    { 0x81508f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d008f93, 98 },
    { 0x43008f94, 17 },
    { 0xccd08f95, 98 },
    { 0xcf408f95, 98 },
    { 0, 0 },
    { 0x1bd08f98, 34 },
    { 0xf8108f98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f08f9c, 34 },
    { 0xc8108f9d, 98 },
    { 0x28008f9e, 23 },
    { 0x83908f9f, 61 },
    { 0x12008f9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a08fa6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62708fb6, 98 },
    { 0, 0 },
    { 0xd8b08fb8, 33 },
    { 0x13808fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef208fbd, 78 },
    { 0x24408fbd, 85 },
    { 0xc3b08fbe, 95 },
    { 0, 0 },
    { 0x82a08fc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08fc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd408fc9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e08fcc, 33 },
    { 0xa5b08fcd, 33 },
    { 0x6a408fce, 90 },
    { 0x8fe08fcf, 61 },
    { 0x79308fd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93908fd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b008fd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24808fe0, 100 },
    { 0x35f08fe0, 31 },
    { 0xc7108fe2, 55 },
    { 0x6de08fe1, 75 },
    { 0xaa308fe4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d08fe8, 66 },
    { 0xaf208fe9, 95 },
    { 0, 0 },
    { 0x63c08feb, 76 },
    { 0xcc608fec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d408ff3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d08ffc, 95 },
    { 0x25d08ffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a09000, 70 },
    { 0, 0 },
    { 0xe1209002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc930900a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109012, 33 },
    { 0x37209012, 98 },
    { 0, 0 },
    { 0xe1509015, 90 },
    { 0x2b009016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26409021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be09027, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf370902b, 66 },
    { 0x7260902c, 98 },
    { 0x5de0902b, 80 },
    { 0x65b0902e, 46 },
    { 0x95b0902e, 78 },
    { 0x89e09030, 95 },
    { 0x3e20902e, 36 },
    { 0x5f09032, 90 },
    { 0, 0 },
    { 0x6d09034, 95 },
    { 0xfee09034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed509039, 61 },
    { 0, 0 },
    { 0x43d0903b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee009040, 61 },
    { 0x4ac09041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34609045, 88 },
    { 0xfe609045, 66 },
    { 0, 0 },
    { 0x7fc09048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4140904c, 33 },
    { 0x4550904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a09054, 76 },
    { 0x88609055, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0905f, 66 },
    { 0, 0 },
    { 0x97909061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409064, 100 },
    { 0x45b09065, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc830906a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a0906f, 95 },
    { 0x38409070, 98 },
    { 0x25509071, 23 },
    { 0xcdb09072, 98 },
    { 0, 0 },
    { 0x3b309074, 75 },
    { 0, 0 },
    { 0x23309076, 52 },
    { 0, 0 },
    { 0xd5b09078, 90 },
    { 0, 0 },
    { 0x1fb0907a, 90 },
    { 0, 0 },
    { 0xfac0907c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c0907f, 68 },
    { 0x1b0907f, 90 },
    { 0x97209081, 90 },
    { 0xd5509082, 90 },
    { 0xbf209083, 52 },
    { 0x38609083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f09088, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0908c, 55 },
    { 0xfdf0908d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48409095, 90 },
    { 0xd5509096, 90 },
    { 0xacb09095, 95 },
    { 0x38b09095, 90 },
    { 0xca309099, 23 },
    { 0x5e80909a, 55 },
    { 0x9850909b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54090a0, 90 },
    { 0xb30090a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157090a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644090ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8090ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e090b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad090b9, 91 },
    { 0x1d3090ba, 36 },
    { 0x337090b9, 76 },
    { 0xae3090b9, 76 },
    { 0, 0 },
    { 0x15090be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d090c2, 61 },
    { 0, 0 },
    { 0xbe3090c4, 95 },
    { 0xbc5090c4, 75 },
    { 0xe53090c6, 90 },
    { 0, 0 },
    { 0x645090c8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57090cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1090d0, 76 },
    { 0, 0 },
    { 0xfe0090d2, 52 },
    { 0xf16090d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c090e0, 100 },
    { 0, 0 },
    { 0xfc6090e2, 43 },
    { 0x2b5090e3, 33 },
    { 0xeb1090e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b090e7, 33 },
    { 0, 0 },
    { 0x52090e9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c090f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4090f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde090fa, 61 },
    { 0x4d2090fa, 61 },
    { 0, 0 },
    { 0xa42090fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be09100, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x28609103, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc709109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10910c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95709110, 34 },
    { 0x50509111, 66 },
    { 0, 0 },
    { 0x6da09113, 95 },
    { 0, 0 },
    { 0xf1a09115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e09119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44709120, 55 },
    { 0x2aa09121, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75209124, 90 },
    { 0, 0 },
    { 0x80209126, 69 },
    { 0x61409127, 23 },
    { 0, 0 },
    { 0x32309129, 67 },
    { 0, 0 },
    { 0xd2c0912b, 90 },
    { 0x1610912c, 95 },
    { 0x1350912d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf09131, 53 },
    { 0x1ad09131, 76 },
    { 0x10b09131, 23 },
    { 0x74c09132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa900913a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0913d, 33 },
    { 0xc510913e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba309145, 65 },
    { 0x41309146, 17 },
    { 0xe5f09147, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d0914a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1370914d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f309150, 55 },
    { 0xa8709151, 23 },
    { 0, 0 },
    { 0xedd09153, 33 },
    { 0, 0 },
    { 0xef109155, 87 },
    { 0x12c09155, 33 },
    { 0x70e09156, 76 },
    { 0xc2d09158, 68 },
    { 0xe9809159, 90 },
    { 0, 0 },
    { 0xd070915b, 76 },
    { 0, 0 },
    { 0x56e0915d, 68 },
    { 0x6a60915e, 98 },
    { 0x2f50915d, 77 },
    { 0xef709160, 34 },
    { 0xd290915f, 93 },
    { 0xdad09162, 90 },
    { 0x45c09162, 65 },
    { 0x8bf09164, 78 },
    { 0xcac09161, 34 },
    { 0x60709166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd09170, 90 },
    { 0x91d09171, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf609177, 46 },
    { 0, 0 },
    { 0x2a909179, 55 },
    { 0, 0 },
    { 0x860917b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f09183, 46 },
    { 0x30c09184, 90 },
    { 0x6e309184, 96 },
    { 0xedb09186, 65 },
    { 0x99909185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f0918b, 61 },
    { 0, 0 },
    { 0xf290918d, 40 },
    { 0xd0d0918e, 33 },
    { 0, 0 },
    { 0x23909190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63809193, 93 },
    { 0, 0 },
    { 0xb1a09195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7820919d, 76 },
    { 0x630919e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1091a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b091aa, 69 },
    { 0xc14091ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2091af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d091b3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b091b6, 76 },
    { 0x3d4091b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5091ba, 66 },
    { 0xfce091ba, 46 },
    { 0xe6091bc, 61 },
    { 0xe84091bd, 95 },
    { 0, 0 },
    { 0x63a091bf, 33 },
    { 0x435091c0, 61 },
    { 0xc30091c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4091c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce091c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879091cf, 78 },
    { 0x24b091d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b091d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311091db, 55 },
    { 0xc8091dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520091e1, 76 },
    { 0x7ca091e2, 33 },
    { 0xb57091e2, 25 },
    { 0xccc091e3, 34 },
    { 0x11d091e5, 55 },
    { 0, 0 },
    { 0xd96091e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387091ec, 46 },
    { 0, 0 },
    { 0x76a091ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a091f1, 98 },
    { 0x49c091f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b091f5, 34 },
    { 0xb53091f6, 84 },
    { 0, 0 },
    { 0x5cc091f8, 98 },
    { 0xd28091f9, 61 },
    { 0x13a091fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e09200, 98 },
    { 0x90d09200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9209207, 33 },
    { 0, 0 },
    { 0x27909209, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9309211, 49 },
    { 0xe7609212, 90 },
    { 0x7be09212, 90 },
    { 0, 0 },
    { 0x77f09215, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc0921b, 75 },
    { 0, 0 },
    { 0x850921d, 95 },
    { 0xe280921e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x44409221, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae909225, 98 },
    { 0xf8c09226, 43 },
    { 0xba509226, 85 },
    { 0x1f309228, 46 },
    { 0xf8909228, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb609233, 90 },
    { 0xb0409234, 23 },
    { 0x5a609235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d709239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d90923d, 81 },
    { 0x2310923d, 84 },
    { 0x3c90923e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59609242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42309249, 87 },
    { 0xe340924a, 61 },
    { 0x8700924b, 33 },
    { 0x9130924b, 90 },
    { 0x25b09249, 33 },
    { 0x7bb0924e, 33 },
    { 0xd700924c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb509252, 90 },
    { 0x63f09253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd309256, 43 },
    { 0x95309257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c0925b, 23 },
    { 0, 0 },
    { 0x2eb0925d, 98 },
    { 0xc0b0925e, 76 },
    { 0, 0 },
    { 0xfab09260, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a09268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780926c, 23 },
    { 0, 0 },
    { 0x3920926e, 90 },
    { 0, 0 },
    { 0x27b09270, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b09274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea0927b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f0927e, 40 },
    { 0, 0 },
    { 0xddb09280, 33 },
    { 0, 0 },
    { 0x10609282, 95 },
    { 0, 0 },
    { 0x1309284, 43 },
    { 0xab509285, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d0928b, 61 },
    { 0x2b50928c, 90 },
    { 0x8230928d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a09293, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c09296, 23 },
    { 0x8e409297, 61 },
    { 0x32209297, 33 },
    { 0x1d209297, 90 },
    { 0x56f09297, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb0929d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20092a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6092a7, 43 },
    { 0, 0 },
    { 0x432092a9, 91 },
    { 0xa01092aa, 99 },
    { 0x8ad092aa, 90 },
    { 0, 0 },
    { 0xeeb092ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f092b1, 33 },
    { 0, 0 },
    { 0x35b092b3, 23 },
    { 0xd40092b4, 61 },
    { 0x112092b3, 90 },
    { 0x16c092b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x978092b9, 43 },
    { 0, 0 },
    { 0x8c4092bb, 95 },
    { 0, 0 },
    { 0xa08092bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c092c0, 23 },
    { 0xb76092c1, 46 },
    { 0x4d7092c2, 43 },
    { 0x651092c3, 46 },
    { 0x555092c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68092c7, 93 },
    { 0, 0 },
    { 0x2f8092c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x302092cc, 66 },
    { 0x7c6092cd, 78 },
    { 0x615092cd, 98 },
    { 0xf95092cf, 36 },
    { 0x6b4092d0, 95 },
    { 0xf8e092d1, 90 },
    { 0, 0 },
    { 0x499092d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7092d7, 45 },
    { 0, 0 },
    { 0xdf6092d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934092e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4092e4, 90 },
    { 0, 0 },
    { 0x4f1092e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x113092e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c092ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d092ef, 95 },
    { 0, 0 },
    { 0x260092f1, 23 },
    { 0x4ae092f2, 61 },
    { 0, 0 },
    { 0x156092f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7092f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50092fc, 90 },
    { 0x3f1092fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72909300, 95 },
    { 0xea109301, 61 },
    { 0xa3709302, 61 },
    { 0x6ba09302, 78 },
    { 0xddf09303, 61 },
    { 0x54c09305, 91 },
    { 0x2d109302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb60930b, 90 },
    { 0xa6f0930c, 90 },
    { 0x1970930d, 98 },
    { 0x4cc0930c, 95 },
    { 0xeeb0930b, 55 },
    { 0x3ed09310, 80 },
    { 0xfb509310, 32 },
    { 0xc20930d, 55 },
    { 0x9120930f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf030931a, 43 },
    { 0, 0 },
    { 0xa530931c, 33 },
    { 0xdac0931c, 77 },
    { 0, 0 },
    { 0x60a0931f, 33 },
    { 0, 0 },
    { 0x42a09321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47809325, 90 },
    { 0, 0 },
    { 0x45e09327, 95 },
    { 0xc9e09328, 77 },
    { 0, 0 },
    { 0x30a0932a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc480932d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2809330, 61 },
    { 0xec709331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0809336, 90 },
    { 0xead09336, 98 },
    { 0x86209338, 33 },
    { 0xf9b09336, 90 },
    { 0, 0 },
    { 0x3980933b, 23 },
    { 0, 0 },
    { 0xb300933d, 90 },
    { 0x18c0933d, 90 },
    { 0xafd0933f, 67 },
    { 0xbdf09340, 43 },
    { 0, 0 },
    { 0x88309342, 33 },
    { 0x41309342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf709347, 90 },
    { 0, 0 },
    { 0x1a509349, 95 },
    { 0, 0 },
    { 0x8d80934b, 98 },
    { 0x2570934c, 66 },
    { 0xdbc0934d, 83 },
    { 0xbdb0934e, 23 },
    { 0x45e0934f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99309355, 34 },
    { 0x33009356, 52 },
    { 0xb5109356, 95 },
    { 0, 0 },
    { 0x74109359, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb09361, 33 },
    { 0x95909362, 90 },
    { 0x49e09363, 90 },
    { 0x64509363, 23 },
    { 0x1ef09365, 76 },
    { 0xc6909365, 36 },
    { 0x20309364, 33 },
    { 0xfb409367, 23 },
    { 0xb2709369, 23 },
    { 0x5b60936a, 53 },
    { 0x6ff0936a, 98 },
    { 0xe5509361, 76 },
    { 0x2300936d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e09370, 90 },
    { 0, 0 },
    { 0x9409372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a209378, 76 },
    { 0xe2209378, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e0937c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd140937f, 76 },
    { 0x6aa09380, 61 },
    { 0x54809381, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55009386, 61 },
    { 0xab09387, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab809399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30939e, 97 },
    { 0, 0 },
    { 0xf1093a0, 76 },
    { 0x808093a1, 17 },
    { 0x7a4093a2, 55 },
    { 0, 0 },
    { 0xec5093a4, 23 },
    { 0xe26093a4, 65 },
    { 0, 0 },
    { 0x585093a7, 23 },
    { 0, 0 },
    { 0x497093a9, 34 },
    { 0x159093aa, 76 },
    { 0, 0 },
    { 0xc8a093ac, 98 },
    { 0x79f093ad, 90 },
    { 0x29e093ae, 76 },
    { 0, 0 },
    { 0xb3a093b0, 61 },
    { 0x5ce093b0, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8093b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3093c2, 55 },
    { 0, 0 },
    { 0xb98093c4, 43 },
    { 0, 0 },
    { 0x251093c6, 23 },
    { 0xeb2093c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2093cb, 90 },
    { 0xed4093cc, 68 },
    { 0x312093cd, 34 },
    { 0x40e093cd, 61 },
    { 0, 0 },
    { 0xa43093d0, 33 },
    { 0x793093d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b093d6, 90 },
    { 0xfc6093d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550093dc, 17 },
    { 0, 0 },
    { 0xcb4093de, 87 },
    { 0x54d093de, 34 },
    { 0xa57093df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b093e5, 75 },
    { 0, 0 },
    { 0xfda093e7, 23 },
    { 0x46f093e8, 23 },
    { 0x316093e9, 87 },
    { 0x2fd093ea, 87 },
    { 0xde7093e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b093f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768093f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb093fa, 95 },
    { 0, 0 },
    { 0x7df093fc, 61 },
    { 0x2e8093fc, 90 },
    { 0, 0 },
    { 0xd0c093ff, 95 },
    { 0x10209400, 95 },
    { 0, 0 },
    { 0x3d09402, 95 },
    { 0x38d09403, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c309408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380940d, 95 },
    { 0xfa60940e, 78 },
    { 0xbbe0940f, 90 },
    { 0x2910940e, 33 },
    { 0, 0 },
    { 0xe6909412, 34 },
    { 0, 0 },
    { 0xf8009414, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74809417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a60941d, 33 },
    { 0x1d60941d, 90 },
    { 0x3d10941f, 34 },
    { 0x9d20941e, 33 },
    { 0xeb809421, 90 },
    { 0x7470941e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f09426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a0942a, 61 },
    { 0, 0 },
    { 0x7060942c, 84 },
    { 0x9870942c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b09430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a09434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d0943b, 76 },
    { 0xe4f0943c, 52 },
    { 0x930943c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73409440, 76 },
    { 0xa3f09441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d509444, 55 },
    { 0x52a09444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e0944a, 33 },
    { 0x23b0944a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d0944e, 34 },
    { 0, 0 },
    { 0x46809450, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65409454, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f009458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c20945b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd50945f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5609465, 95 },
    { 0, 0 },
    { 0x99709467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e50946a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4260946d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43709476, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd0947c, 43 },
    { 0x49a0947d, 98 },
    { 0xdf70947e, 90 },
    { 0x6af0947f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8009483, 91 },
    { 0x82609483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98109488, 33 },
    { 0x9b809489, 61 },
    { 0xf7a0948a, 55 },
    { 0, 0 },
    { 0xe860948c, 95 },
    { 0x8ac0948d, 61 },
    { 0xc8d0948e, 95 },
    { 0xd8d0948c, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0109494, 65 },
    { 0xf3709494, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4709499, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a094a4, 98 },
    { 0x76e094a4, 43 },
    { 0x861094a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156094be, 8 },
    { 0x795094be, 34 },
    { 0x33b094bf, 75 },
    { 0x1f3094bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x496094c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce094c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6094cb, 98 },
    { 0x5df094cc, 77 },
    { 0x357094cc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a094d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c094d9, 23 },
    { 0xe5b094da, 33 },
    { 0x5fe094db, 88 },
    { 0xe9f094db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442094e5, 43 },
    { 0x64c094e5, 43 },
    { 0, 0 },
    { 0xf6f094e8, 52 },
    { 0x10094e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06094ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0094f1, 98 },
    { 0x99c094f2, 78 },
    { 0xda5094f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e094f8, 90 },
    { 0xa68094f8, 98 },
    { 0, 0 },
    { 0x853094fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735094ff, 55 },
    { 0, 0 },
    { 0x4af09501, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad609504, 93 },
    { 0x60409505, 33 },
    { 0xf9609506, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e0950b, 61 },
    { 0xc050950c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b0950f, 54 },
    { 0x12109510, 33 },
    { 0xe3b0950f, 29 },
    { 0, 0 },
    { 0xe0109513, 33 },
    { 0xb5009514, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb40951c, 23 },
    { 0, 0 },
    { 0x9b90951e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e09527, 93 },
    { 0, 0 },
    { 0x5c209529, 90 },
    { 0, 0 },
    { 0x3390952b, 99 },
    { 0, 0 },
    { 0xedb0952d, 95 },
    { 0x3b10952e, 23 },
    { 0xd20952f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3309533, 95 },
    { 0x57609533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc09537, 93 },
    { 0x3309538, 34 },
    { 0xb9609537, 76 },
    { 0x7330953a, 87 },
    { 0x46209538, 78 },
    { 0xe680953c, 99 },
    { 0, 0 },
    { 0xd2f0953e, 95 },
    { 0x4930953e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d09547, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680954a, 33 },
    { 0x9320954b, 33 },
    { 0x2390954c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e0954f, 98 },
    { 0x24509550, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440955a, 67 },
    { 0xe3f0955a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b209561, 46 },
    { 0x8f209561, 90 },
    { 0, 0 },
    { 0xf1109564, 76 },
    { 0x3e009565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa140956d, 52 },
    { 0x24f0956d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc709571, 84 },
    { 0x49c09572, 98 },
    { 0, 0 },
    { 0x49909574, 33 },
    { 0x3f909574, 65 },
    { 0xc4309576, 99 },
    { 0, 0 },
    { 0xd609578, 90 },
    { 0x6c809579, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7850957c, 33 },
    { 0xc4d0957d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70909581, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a09584, 55 },
    { 0, 0 },
    { 0x16009586, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7109591, 98 },
    { 0, 0 },
    { 0xf009593, 33 },
    { 0xe0709594, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf709599, 91 },
    { 0x5b60959a, 33 },
    { 0x6050959b, 76 },
    { 0x5080959b, 87 },
    { 0x5ec0959a, 61 },
    { 0x8eb0959e, 95 },
    { 0xd70959d, 90 },
    { 0x7ea0959b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c095a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd095a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc095ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c095b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2095b4, 33 },
    { 0x149095b4, 46 },
    { 0x3f9095b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8095ba, 93 },
    { 0, 0 },
    { 0x8c8095bc, 23 },
    { 0x8c8095bd, 34 },
    { 0xf54095be, 76 },
    { 0xd4c095bf, 88 },
    { 0, 0 },
    { 0xdd5095c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b095c7, 68 },
    { 0x21c095c7, 33 },
    { 0x2fb095c9, 66 },
    { 0x4bf095ca, 95 },
    { 0x3b9095cb, 95 },
    { 0xfc9095cb, 95 },
    { 0, 0 },
    { 0x824095ce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b095d1, 65 },
    { 0, 0 },
    { 0xff7095d3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0095d6, 55 },
    { 0, 0 },
    { 0xda1095d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794095dc, 92 },
    { 0x2b9095dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12095e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5095e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8095ea, 76 },
    { 0, 0 },
    { 0x737095ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897095f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b095f4, 90 },
    { 0, 0 },
    { 0x441095f6, 100 },
    { 0, 0 },
    { 0x94095f8, 34 },
    { 0x252095f9, 90 },
    { 0, 0 },
    { 0xb5f095fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd095ff, 36 },
    { 0, 0 },
    { 0x99309601, 34 },
    { 0x94909602, 46 },
    { 0xbf309603, 76 },
    { 0xcf09603, 59 },
    { 0xa1709601, 95 },
    { 0x56209606, 23 },
    { 0x92b09607, 61 },
    { 0, 0 },
    { 0xf6d09609, 68 },
    { 0xaf80960a, 98 },
    { 0xbde0960b, 95 },
    { 0, 0 },
    { 0xaff0960d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf609611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1309616, 99 },
    { 0, 0 },
    { 0xc5109618, 81 },
    { 0x19409618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4430961c, 45 },
    { 0xc4e0961d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c609621, 23 },
    { 0xd9609621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c509626, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d20962f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b0963e, 23 },
    { 0, 0 },
    { 0x70f09640, 61 },
    { 0, 0 },
    { 0x1bb09642, 51 },
    { 0x51c09643, 95 },
    { 0xa2c09644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7509648, 33 },
    { 0xf5509649, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x830964c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480964f, 23 },
    { 0, 0 },
    { 0x45609651, 93 },
    { 0xf4e09651, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e09656, 90 },
    { 0, 0 },
    { 0x78909658, 90 },
    { 0, 0 },
    { 0x7610965a, 55 },
    { 0, 0 },
    { 0xe0b0965c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3320966c, 55 },
    { 0xd800966d, 88 },
    { 0xa850966e, 43 },
    { 0xb9c0966f, 95 },
    { 0xd6509670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe509675, 66 },
    { 0xc4b09675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd900967a, 61 },
    { 0x1270967a, 76 },
    { 0, 0 },
    { 0xa710967d, 95 },
    { 0x3930967e, 95 },
    { 0xa550967f, 34 },
    { 0xdc60967f, 23 },
    { 0x7a209681, 88 },
    { 0x96809682, 61 },
    { 0, 0 },
    { 0xa6009684, 95 },
    { 0x27b09685, 43 },
    { 0x52909686, 91 },
    { 0xa0e09686, 46 },
    { 0x4cd09686, 61 },
    { 0, 0 },
    { 0x9670968a, 52 },
    { 0xe050968b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60968f, 55 },
    { 0x7da09690, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef60969a, 61 },
    { 0, 0 },
    { 0x6c00969c, 55 },
    { 0x7200969c, 90 },
    { 0, 0 },
    { 0xce20969f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651096a5, 33 },
    { 0xbfc096a6, 61 },
    { 0x879096a6, 33 },
    { 0x182096a7, 95 },
    { 0xae1096a9, 33 },
    { 0xac2096a9, 43 },
    { 0x571096ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59096af, 93 },
    { 0xaec096af, 90 },
    { 0xfd5096b1, 76 },
    { 0x357096b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4096b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6096be, 98 },
    { 0xbfa096bf, 43 },
    { 0, 0 },
    { 0xb6d096c1, 84 },
    { 0x4fe096c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf096c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x559096c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488096cd, 76 },
    { 0x7a7096ce, 88 },
    { 0xd17096cf, 95 },
    { 0x3e096d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e096d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad096d9, 19 },
    { 0x6ff096da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a096e0, 61 },
    { 0xceb096e0, 61 },
    { 0x856096e2, 33 },
    { 0x526096e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c096e7, 87 },
    { 0x8c6096e8, 34 },
    { 0, 0 },
    { 0x744096ea, 99 },
    { 0xcf2096eb, 95 },
    { 0, 0 },
    { 0x8fa096ed, 61 },
    { 0x56c096ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29096f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58096fb, 36 },
    { 0x660096fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc09700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3809704, 61 },
    { 0x4cd09704, 95 },
    { 0, 0 },
    { 0x1f009707, 98 },
    { 0, 0 },
    { 0x98b09709, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d80970c, 55 },
    { 0x1ce0970c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d309710, 55 },
    { 0, 0 },
    { 0xd3209712, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36709717, 33 },
    { 0, 0 },
    { 0xe6009719, 76 },
    { 0x71e0971a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6870971d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad709723, 33 },
    { 0x8e909723, 98 },
    { 0, 0 },
    { 0x4db09726, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0972a, 90 },
    { 0x8110972b, 23 },
    { 0xecf0972c, 23 },
    { 0xfa20972b, 76 },
    { 0x1880972e, 33 },
    { 0xd220972f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b09736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec09739, 95 },
    { 0x72709739, 43 },
    { 0x3000973a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5470973e, 76 },
    { 0xbbd0973f, 85 },
    { 0xb3709740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77609743, 90 },
    { 0xf2909743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0974f, 99 },
    { 0, 0 },
    { 0x96609751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f09756, 90 },
    { 0, 0 },
    { 0x31f09758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6430975b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50975e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db09761, 42 },
    { 0x77609761, 95 },
    { 0xcb409762, 61 },
    { 0, 0 },
    { 0xa7809765, 61 },
    { 0xc5b09766, 61 },
    { 0, 0 },
    { 0x12509768, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab80976b, 76 },
    { 0x2640976c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4300976f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d09774, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c709777, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a0977a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87409781, 90 },
    { 0x94a09781, 95 },
    { 0x63309781, 91 },
    { 0xaef09784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0978c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc009793, 60 },
    { 0x29209794, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a409798, 95 },
    { 0, 0 },
    { 0x18f0979a, 90 },
    { 0xcb0979b, 61 },
    { 0xd3f0979b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac097a2, 98 },
    { 0, 0 },
    { 0xafd097a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844097a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d097b1, 95 },
    { 0, 0 },
    { 0xd79097b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c097b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d097b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c097be, 81 },
    { 0, 0 },
    { 0xea4097c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e097c7, 29 },
    { 0xdf2097c7, 33 },
    { 0xc94097c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1097cc, 90 },
    { 0xaab097cc, 100 },
    { 0x3e097ce, 33 },
    { 0xf31097cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832097d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8097d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb097df, 81 },
    { 0x1c1097df, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4097e3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30097e6, 34 },
    { 0x552097e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7097ed, 90 },
    { 0xafc097ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311097f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f097f5, 90 },
    { 0x985097f6, 23 },
    { 0x6e9097f7, 90 },
    { 0x568097f7, 61 },
    { 0x29b097f7, 66 },
    { 0x8da097fa, 61 },
    { 0xf9d097f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b097fe, 40 },
    { 0x2f8097ff, 76 },
    { 0, 0 },
    { 0x33009801, 76 },
    { 0, 0 },
    { 0x1ca09803, 61 },
    { 0xd4a09804, 46 },
    { 0x35109805, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a0980a, 98 },
    { 0x1560980b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f09811, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f609815, 67 },
    { 0x22b09816, 80 },
    { 0xebe09817, 100 },
    { 0x30709815, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e10981e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b09822, 90 },
    { 0, 0 },
    { 0x309824, 33 },
    { 0, 0 },
    { 0xea509826, 85 },
    { 0, 0 },
    { 0x69309828, 43 },
    { 0x4f909828, 33 },
    { 0x2cc0982a, 23 },
    { 0x9ba0982b, 23 },
    { 0x9a409829, 55 },
    { 0x37b0982d, 90 },
    { 0x6f80982e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66909834, 99 },
    { 0xd2109834, 77 },
    { 0x49409835, 90 },
    { 0xdf309834, 52 },
    { 0x3ba09837, 33 },
    { 0xfe509838, 90 },
    { 0, 0 },
    { 0x6640983b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f10983f, 98 },
    { 0xccc09840, 34 },
    { 0, 0 },
    { 0xd9709842, 46 },
    { 0xedd09843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d09848, 65 },
    { 0x5e109848, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b0984d, 55 },
    { 0xab70984d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce609851, 90 },
    { 0, 0 },
    { 0xd3409853, 90 },
    { 0x93509854, 100 },
    { 0, 0 },
    { 0xd4109856, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f09859, 40 },
    { 0x72f0985a, 33 },
    { 0xc9209859, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92409860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9009869, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550986d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09870, 43 },
    { 0x31309871, 68 },
    { 0xd9a09872, 33 },
    { 0, 0 },
    { 0x4e009874, 55 },
    { 0, 0 },
    { 0xd4909876, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9180987a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0987d, 66 },
    { 0x3e70987e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9009884, 64 },
    { 0x68a09885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a80988a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa990988f, 90 },
    { 0xb6e0988f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae909893, 46 },
    { 0xa6a09894, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b209897, 12 },
    { 0x6d09898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8330989f, 23 },
    { 0, 0 },
    { 0xe28098a1, 61 },
    { 0x697098a2, 55 },
    { 0, 0 },
    { 0x7e5098a4, 23 },
    { 0x4ae098a5, 98 },
    { 0xf7098a6, 95 },
    { 0x45098a4, 61 },
    { 0x725098a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720098ac, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249098b0, 43 },
    { 0x25098b0, 43 },
    { 0x67c098b2, 55 },
    { 0x9dd098b3, 95 },
    { 0, 0 },
    { 0x40098b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b098b9, 69 },
    { 0x133098ba, 33 },
    { 0xa8e098ba, 61 },
    { 0xabc098bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f098c0, 98 },
    { 0x719098c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e098c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01098cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d098d0, 34 },
    { 0x5b5098d1, 98 },
    { 0xa66098d0, 98 },
    { 0x869098d3, 84 },
    { 0, 0 },
    { 0xd13098d5, 55 },
    { 0, 0 },
    { 0x55d098d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2098db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc098df, 34 },
    { 0x5b8098df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x238098e3, 61 },
    { 0, 0 },
    { 0x8a1098e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e098e9, 75 },
    { 0, 0 },
    { 0x2b8098eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd098f0, 93 },
    { 0xf6a098f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69098f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e098fa, 64 },
    { 0x343098fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf098ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb09904, 33 },
    { 0xca609905, 33 },
    { 0xc7409904, 67 },
    { 0x9de09904, 98 },
    { 0x9cf09908, 55 },
    { 0x9e09906, 98 },
    { 0x37309908, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10990d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6009913, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9309918, 61 },
    { 0x41509918, 95 },
    { 0x63709919, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d40991d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e609922, 61 },
    { 0x2e909923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b409937, 90 },
    { 0xefb09938, 61 },
    { 0xd909939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe40993e, 76 },
    { 0x45c0993e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae09944, 91 },
    { 0xbf809945, 61 },
    { 0xd8209945, 76 },
    { 0x9e909947, 90 },
    { 0xd2e09948, 76 },
    { 0x4ea09949, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e0994f, 43 },
    { 0, 0 },
    { 0x6cc09951, 75 },
    { 0x4ee09952, 95 },
    { 0, 0 },
    { 0xf509954, 61 },
    { 0, 0 },
    { 0x36609956, 80 },
    { 0xaf009957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0995c, 17 },
    { 0, 0 },
    { 0x4920995e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7809962, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe09967, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe10996a, 99 },
    { 0x20c0996b, 90 },
    { 0x5c70996c, 76 },
    { 0xee50996d, 61 },
    { 0x8260996e, 98 },
    { 0x5f20996c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca09972, 95 },
    { 0, 0 },
    { 0xa9609974, 46 },
    { 0xc8209975, 61 },
    { 0xce409976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef00997c, 99 },
    { 0, 0 },
    { 0xdba0997e, 43 },
    { 0xec40997f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d09982, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33509988, 82 },
    { 0x8af09988, 61 },
    { 0xa3f0998a, 33 },
    { 0x4db0998a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9480998e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f09991, 61 },
    { 0, 0 },
    { 0xbd709993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6309999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d0999e, 78 },
    { 0x3ef0999f, 98 },
    { 0x3ea0999f, 61 },
    { 0x643099a0, 23 },
    { 0x7f9099a1, 46 },
    { 0, 0 },
    { 0x982099a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c099a7, 90 },
    { 0xf76099a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f099af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6099b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66099b9, 52 },
    { 0, 0 },
    { 0xfc8099bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e099c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e099cd, 98 },
    { 0xf7c099cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1099df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e099e2, 33 },
    { 0x2f4099e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6099eb, 23 },
    { 0xb7e099ec, 76 },
    { 0x632099eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477099f1, 61 },
    { 0, 0 },
    { 0xe71099f3, 33 },
    { 0x106099f3, 23 },
    { 0x71e099f5, 17 },
    { 0x761099f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e099f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db099fd, 84 },
    { 0xc3e099fe, 88 },
    { 0xa66099fe, 95 },
    { 0x329099ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61609a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4409a08, 69 },
    { 0xea809a09, 33 },
    { 0x6f309a08, 76 },
    { 0x3c209a0b, 98 },
    { 0xa7e09a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5309a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf109a12, 43 },
    { 0xd3409a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3009a16, 99 },
    { 0x2d409a17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b709a1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed209a1d, 93 },
    { 0x64609a1e, 76 },
    { 0xd6609a1f, 55 },
    { 0x27e09a20, 61 },
    { 0, 0 },
    { 0x1f09a22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35609a2b, 77 },
    { 0x56109a2c, 98 },
    { 0, 0 },
    { 0x1209a2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5009a35, 90 },
    { 0x67609a36, 61 },
    { 0x6cf09a35, 95 },
    { 0xc3c09a38, 55 },
    { 0x2fb09a37, 67 },
    { 0xce009a39, 64 },
    { 0xce309a3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80309a40, 62 },
    { 0x59109a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79909a44, 98 },
    { 0, 0 },
    { 0xcd109a46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10209a4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c09a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b09a62, 34 },
    { 0xba209a62, 99 },
    { 0xd1509a63, 52 },
    { 0, 0 },
    { 0xe6109a66, 61 },
    { 0x5d009a67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a09a6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9509a6d, 98 },
    { 0xa7709a6e, 78 },
    { 0x96b09a6f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b09a72, 61 },
    { 0xd0f09a73, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b09a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20409a7d, 65 },
    { 0x38509a7e, 23 },
    { 0x88209a7e, 46 },
    { 0x5d709a80, 90 },
    { 0x4e009a81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae09a85, 33 },
    { 0x24f09a85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4709a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6909a8f, 95 },
    { 0x7bc09a90, 91 },
    { 0, 0 },
    { 0x19009a92, 33 },
    { 0x68d09a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09a96, 78 },
    { 0, 0 },
    { 0xc109a98, 23 },
    { 0x27909a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16309a9e, 61 },
    { 0xe1309a9e, 90 },
    { 0xfa09aa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c209aa3, 78 },
    { 0xbcc09aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b09aaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c909aad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25309ab1, 51 },
    { 0xeb609ab2, 61 },
    { 0x41809ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42309ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d09abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d09ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa109ac4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x28809ac7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95509aca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68009acf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be09ad3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4309ad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c309add, 55 },
    { 0xf1509ade, 88 },
    { 0xa7509add, 95 },
    { 0, 0 },
    { 0xd909ae1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75309ae4, 95 },
    { 0x1ed09ae5, 23 },
    { 0x59909ae6, 99 },
    { 0, 0 },
    { 0x90909ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47609aec, 23 },
    { 0xdd009aec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38109af0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92109af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9109afc, 95 },
    { 0, 0 },
    { 0x59e09afe, 61 },
    { 0xe8309aff, 43 },
    { 0, 0 },
    { 0xb6d09b01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba09b05, 69 },
    { 0, 0 },
    { 0x63009b07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e09b0d, 56 },
    { 0x5c509b0e, 33 },
    { 0, 0 },
    { 0x9509b10, 46 },
    { 0, 0 },
    { 0xb2b09b12, 61 },
    { 0x76709b13, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a09b18, 76 },
    { 0, 0 },
    { 0x3d509b1a, 77 },
    { 0x3b409b1a, 98 },
    { 0x7d109b1b, 61 },
    { 0x8cb09b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db09b22, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40109b28, 46 },
    { 0xeee09b29, 68 },
    { 0, 0 },
    { 0xff09b2b, 52 },
    { 0x29a09b2c, 33 },
    { 0xb9e09b2d, 23 },
    { 0xbf609b2e, 23 },
    { 0x4b409b2b, 55 },
    { 0x91509b30, 61 },
    { 0x6909b30, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e09b38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf709b3d, 55 },
    { 0xced09b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c09b44, 52 },
    { 0xda009b45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57309b48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53309b50, 98 },
    { 0xba309b51, 85 },
    { 0x59d09b51, 90 },
    { 0, 0 },
    { 0x74d09b54, 52 },
    { 0x40c09b55, 76 },
    { 0x71e09b55, 64 },
    { 0, 0 },
    { 0x3df09b58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e09b60, 76 },
    { 0xe8609b61, 46 },
    { 0xc2209b62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d09b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2909b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf09b70, 34 },
    { 0x56109b71, 98 },
    { 0x7bc09b72, 100 },
    { 0xfe409b73, 34 },
    { 0xdf309b70, 55 },
    { 0x8cd09b74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69909b78, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78709b81, 23 },
    { 0x63809b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb409b86, 55 },
    { 0, 0 },
    { 0xa7809b88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29609b91, 98 },
    { 0, 0 },
    { 0x98109b93, 23 },
    { 0x8ea09b94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa909b98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64509b9d, 56 },
    { 0x3ea09b9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a309ba1, 91 },
    { 0, 0 },
    { 0xa6e09ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c09ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d09bae, 90 },
    { 0x2f09baf, 76 },
    { 0x9bc09baf, 33 },
    { 0x89709baf, 61 },
    { 0x83309bb1, 76 },
    { 0x6d909bb3, 100 },
    { 0xc6a09bb4, 75 },
    { 0, 0 },
    { 0xc3609bb6, 84 },
    { 0, 0 },
    { 0x6a809bb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0309bbd, 85 },
    { 0x45509bbd, 47 },
    { 0x36709bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10509bc3, 93 },
    { 0xfc09bc3, 66 },
    { 0x8d509bc5, 90 },
    { 0x23609bc3, 77 },
    { 0x77309bc3, 77 },
    { 0xb209bc8, 36 },
    { 0x12909bc9, 33 },
    { 0, 0 },
    { 0x3df09bcb, 90 },
    { 0xbb109bcc, 43 },
    { 0xa5009bcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d09bd4, 55 },
    { 0, 0 },
    { 0xb3509bd6, 95 },
    { 0, 0 },
    { 0x89509bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d09bdc, 78 },
    { 0, 0 },
    { 0x67009bde, 95 },
    { 0, 0 },
    { 0x3e409be0, 33 },
    { 0x25909be0, 55 },
    { 0x4d09be2, 91 },
    { 0xa4309be3, 61 },
    { 0x6f009be0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x11709be7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c09beb, 81 },
    { 0x77209beb, 100 },
    { 0xf409beb, 34 },
    { 0xe1f09bee, 98 },
    { 0xdbb09beb, 19 },
    { 0x3a209bee, 76 },
    { 0, 0 },
    { 0x9909bf2, 98 },
    { 0xa7a09bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31009bf8, 62 },
    { 0x97709bf9, 34 },
    { 0xa5009bf9, 99 },
    { 0x52909bfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3409bff, 78 },
    { 0x73c09c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a09c08, 85 },
    { 0x44509c09, 52 },
    { 0, 0 },
    { 0x71e09c0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc09c16, 33 },
    { 0x86509c16, 93 },
    { 0, 0 },
    { 0x68209c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67309c1e, 76 },
    { 0, 0 },
    { 0x21a09c20, 76 },
    { 0x3dd09c21, 90 },
    { 0, 0 },
    { 0xfd309c23, 90 },
    { 0x7e209c24, 38 },
    { 0x7a809c24, 95 },
    { 0, 0 },
    { 0x68f09c27, 61 },
    { 0xb8d09c27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b09c2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47209c32, 76 },
    { 0x18e09c33, 90 },
    { 0xb7809c34, 55 },
    { 0x45209c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b09c38, 61 },
    { 0x8de09c38, 34 },
    { 0xa1909c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e09c45, 33 },
    { 0x4c09c46, 98 },
    { 0x6e609c46, 67 },
    { 0x36e09c48, 46 },
    { 0xe2809c49, 69 },
    { 0xfa909c4a, 61 },
    { 0xf2b09c4a, 61 },
    { 0xead09c4c, 99 },
    { 0xed809c48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b09c50, 87 },
    { 0xd3509c51, 33 },
    { 0x66609c51, 90 },
    { 0xf9f09c50, 23 },
    { 0x73509c54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d209c57, 78 },
    { 0x27609c58, 77 },
    { 0xc5909c57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa09c5e, 23 },
    { 0x1a409c5e, 95 },
    { 0x2ff09c60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0109c63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef09c6d, 52 },
    { 0x94b09c6e, 90 },
    { 0xc6909c6d, 90 },
    { 0xc8009c6d, 90 },
    { 0x42409c71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d209c74, 40 },
    { 0, 0 },
    { 0x58b09c76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3609c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a09c87, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e09c8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e609c91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b09c97, 55 },
    { 0xae09c98, 87 },
    { 0, 0 },
    { 0x76b09c9a, 98 },
    { 0xf0409c9b, 61 },
    { 0xbf709c9c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf909ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d09ca4, 90 },
    { 0x50e09ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e09ca9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98209caf, 61 },
    { 0x8f09caf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc09cb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5409cba, 33 },
    { 0, 0 },
    { 0x37109cbc, 91 },
    { 0x91909cbc, 33 },
    { 0x7f209cbe, 61 },
    { 0, 0 },
    { 0x92d09cc0, 53 },
    { 0xffa09cc1, 95 },
    { 0xeb409cc2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xca309cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e09ccb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d09ccf, 90 },
    { 0xf6709ccf, 23 },
    { 0x36309cd1, 88 },
    { 0x5cd09cd2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88409ce1, 23 },
    { 0xfc009ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb09ce5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54609ce8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a09ceb, 23 },
    { 0x87d09cec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e509cf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97109cf9, 61 },
    { 0xb5b09cfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a09cfd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd09d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa09d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24909d0d, 23 },
    { 0x3f009d0d, 77 },
    { 0xa7209d0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47409d13, 90 },
    { 0x4b309d13, 61 },
    { 0x4c109d15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e09d18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d09d1b, 40 },
    { 0x46a09d1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde109d20, 76 },
    { 0x5d109d21, 34 },
    { 0xc3809d22, 65 },
    { 0x6bb09d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a909d28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73509d2b, 23 },
    { 0x2af09d2c, 55 },
    { 0x75009d2c, 78 },
    { 0xc5d09d2e, 91 },
    { 0x9a509d2f, 76 },
    { 0xa6d09d2f, 93 },
    { 0x19609d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e09d35, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x79209d38, 34 },
    { 0, 0 },
    { 0x21809d3a, 92 },
    { 0x57d09d3b, 61 },
    { 0, 0 },
    { 0xffc09d3d, 33 },
    { 0xf6209d3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14709d43, 33 },
    { 0x4de09d43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8109d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3709d53, 90 },
    { 0, 0 },
    { 0xbb409d55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d09d5c, 93 },
    { 0x1b709d5d, 66 },
    { 0, 0 },
    { 0x82409d5f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1409d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f809d68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e09d6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8009d73, 91 },
    { 0x28c09d74, 101 },
    { 0xb5c09d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b09d7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b09d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da09d88, 65 },
    { 0, 0 },
    { 0xcf809d8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15709d90, 90 },
    { 0x23e09d91, 77 },
    { 0, 0 },
    { 0x66309d93, 93 },
    { 0xff509d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d09da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc709da4, 90 },
    { 0x64c09da5, 23 },
    { 0x44009da5, 34 },
    { 0x8ea09da7, 33 },
    { 0x28b09da8, 95 },
    { 0xd2d09da4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e09dad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a09db0, 91 },
    { 0x28c09db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a909db8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a309dbc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709dc3, 52 },
    { 0, 0 },
    { 0x8ed09dc5, 46 },
    { 0x5a609dc5, 23 },
    { 0x36f09dc7, 95 },
    { 0, 0 },
    { 0xaea09dc9, 33 },
    { 0x609dca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95109dce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90609dd7, 85 },
    { 0xafa09dd8, 77 },
    { 0x37009dd8, 66 },
    { 0, 0 },
    { 0x23709ddb, 91 },
    { 0, 0 },
    { 0x5ba09ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23609de3, 90 },
    { 0, 0 },
    { 0x7c609de5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17609df0, 33 },
    { 0x7ae09df1, 90 },
    { 0x56e09df2, 61 },
    { 0xe2309df2, 61 },
    { 0x2209df4, 61 },
    { 0xe209df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e09dfa, 53 },
    { 0x37e09dfa, 93 },
    { 0x4809dfb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e09e00, 66 },
    { 0x3aa09e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d09e0e, 76 },
    { 0, 0 },
    { 0xc1d09e10, 52 },
    { 0, 0 },
    { 0xcba09e12, 91 },
    { 0xcb609e13, 88 },
    { 0x47309e14, 46 },
    { 0xbdb09e15, 58 },
    { 0x24609e16, 88 },
    { 0x11b09e16, 84 },
    { 0x80409e18, 66 },
    { 0x1f209e16, 98 },
    { 0x66209e19, 98 },
    { 0, 0 },
    { 0x75409e1c, 69 },
    { 0x8cb09e1c, 23 },
    { 0x34009e1e, 93 },
    { 0, 0 },
    { 0xbcd09e20, 55 },
    { 0xbe109e20, 33 },
    { 0x98809e22, 93 },
    { 0x1c209e23, 98 },
    { 0x2a009e23, 33 },
    { 0, 0 },
    { 0x62c09e26, 75 },
    { 0x74b09e27, 23 },
    { 0x4d409e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef909e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec009e33, 95 },
    { 0x82809e34, 64 },
    { 0, 0 },
    { 0x4ae09e36, 33 },
    { 0, 0 },
    { 0xe0909e38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa409e3b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x42709e3e, 33 },
    { 0xc9709e3f, 67 },
    { 0x1609e3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6309e44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc09e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8809e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56509e57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9409e5b, 23 },
    { 0x16509e5c, 88 },
    { 0x80d09e5d, 95 },
    { 0x52e09e5c, 101 },
    { 0xd4b09e5f, 90 },
    { 0x11809e60, 43 },
    { 0x26c09e61, 90 },
    { 0xac609e5d, 95 },
    { 0xb9609e5f, 47 },
    { 0xcc709e5f, 84 },
    { 0x1d509e5e, 95 },
    { 0x15a09e66, 33 },
    { 0, 0 },
    { 0xc0909e68, 33 },
    { 0x95709e69, 76 },
    { 0x22809e69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a09e6d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d09e74, 23 },
    { 0, 0 },
    { 0x2f09e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409e7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a709e81, 46 },
    { 0, 0 },
    { 0xe5e09e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4209e8b, 43 },
    { 0x9e009e8b, 33 },
    { 0, 0 },
    { 0xb809e8e, 23 },
    { 0, 0 },
    { 0xa2b09e90, 90 },
    { 0xce509e91, 91 },
    { 0, 0 },
    { 0x61709e93, 46 },
    { 0x2509e94, 34 },
    { 0xfc109e93, 78 },
    { 0xa0009e96, 43 },
    { 0x20409e94, 17 },
    { 0x38309e98, 87 },
    { 0x9d209e93, 91 },
    { 0x70f09e9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa09e9f, 95 },
    { 0x3f609ea0, 90 },
    { 0, 0 },
    { 0x2a909ea2, 98 },
    { 0xbb909ea3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda409ea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a09eae, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d09eb1, 61 },
    { 0, 0 },
    { 0x52309eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5409eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9109ebb, 61 },
    { 0xafd09ebc, 33 },
    { 0, 0 },
    { 0xcb409ebe, 90 },
    { 0xbfb09ebe, 95 },
    { 0, 0 },
    { 0x3c409ec1, 96 },
    { 0xf4f09ec1, 87 },
    { 0xfde09ec3, 33 },
    { 0xcc809ec2, 33 },
    { 0x48509ec3, 98 },
    { 0xdc209ec6, 90 },
    { 0xc0409ec7, 33 },
    { 0xd2909ec4, 33 },
    { 0xc7109ec9, 61 },
    { 0x48009eca, 33 },
    { 0x36e09ec6, 100 },
    { 0, 0 },
    { 0xce909ecd, 98 },
    { 0xb6109ece, 23 },
    { 0xe0709ece, 55 },
    { 0x2b409ecd, 43 },
    { 0x8fc09ed1, 90 },
    { 0x8c409ed2, 90 },
    { 0x20109ed3, 66 },
    { 0x48109ed0, 90 },
    { 0x8d709ed5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d909ed9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xca209edc, 61 },
    { 0xc6409edd, 90 },
    { 0x86809ede, 61 },
    { 0, 0 },
    { 0xae409ee0, 80 },
    { 0xc3e09ee0, 61 },
    { 0x53609ee0, 33 },
    { 0x2ae09ee3, 95 },
    { 0, 0 },
    { 0x76109ee5, 91 },
    { 0, 0 },
    { 0xea09ee7, 46 },
    { 0, 0 },
    { 0xacf09ee9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa09eed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda09ef7, 95 },
    { 0, 0 },
    { 0x96309ef9, 78 },
    { 0x55b09ef9, 99 },
    { 0x4a309efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6209eff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b09f03, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd409f0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8509f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac09f11, 85 },
    { 0x45909f11, 95 },
    { 0x32f09f13, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a209f16, 95 },
    { 0, 0 },
    { 0x4a609f18, 90 },
    { 0, 0 },
    { 0xa1109f1a, 33 },
    { 0x20e09f1b, 52 },
    { 0x50c09f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e09f21, 84 },
    { 0, 0 },
    { 0x7a009f23, 98 },
    { 0x7f809f24, 98 },
    { 0xcb509f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc09f28, 55 },
    { 0xea409f29, 40 },
    { 0x70009f29, 33 },
    { 0, 0 },
    { 0xd2009f2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4509f2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c09f38, 99 },
    { 0x2b109f38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e309f43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf09f46, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a609f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c09f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e09f55, 95 },
    { 0xa1809f56, 61 },
    { 0xc0a09f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f909f5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17409f60, 34 },
    { 0, 0 },
    { 0x77a09f62, 90 },
    { 0xeaa09f63, 99 },
    { 0, 0 },
    { 0x41109f65, 91 },
    { 0x7ea09f66, 23 },
    { 0xbc709f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd509f6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b809f6d, 84 },
    { 0xe1b09f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee09f72, 90 },
    { 0, 0 },
    { 0xf6409f74, 96 },
    { 0xd0509f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af09f7e, 100 },
    { 0x23709f7f, 33 },
    { 0, 0 },
    { 0xad109f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96109f87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81009f8f, 33 },
    { 0x47c09f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f09f93, 66 },
    { 0xd5c09f93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae09f98, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84609f9c, 61 },
    { 0x10309f9c, 95 },
    { 0x85a09f9c, 93 },
    { 0x43209f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0509fa2, 90 },
    { 0x3b109fa3, 61 },
    { 0xd8709fa4, 55 },
    { 0xd5a09fa3, 96 },
    { 0, 0 },
    { 0xa3909fa7, 95 },
    { 0xef909fa8, 98 },
    { 0x5ea09fa9, 98 },
    { 0xeb409fa8, 90 },
    { 0xea709fab, 43 },
    { 0x23909fac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6109fb6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd09fbf, 90 },
    { 0, 0 },
    { 0xc3309fc1, 36 },
    { 0x3f909fc2, 33 },
    { 0x74009fc3, 46 },
    { 0xa4309fc4, 84 },
    { 0x25b09fc5, 90 },
    { 0x40709fc6, 61 },
    { 0, 0 },
    { 0x93609fc8, 90 },
    { 0x70409fc8, 76 },
    { 0xfd609fc8, 34 },
    { 0xda09fcb, 76 },
    { 0, 0 },
    { 0xdb009fcd, 46 },
    { 0x96c09fcd, 90 },
    { 0xd7709fcf, 61 },
    { 0x1509fd0, 66 },
    { 0xa8609fcd, 95 },
    { 0xd7609fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0609fd5, 90 },
    { 0, 0 },
    { 0x36f09fd7, 98 },
    { 0x5309fd8, 33 },
    { 0x3e309fd8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee409fde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8109fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62409fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf909fed, 90 },
    { 0, 0 },
    { 0xba709fef, 90 },
    { 0x91209ff0, 61 },
    { 0xec909ff0, 43 },
    { 0, 0 },
    { 0x309ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6609ff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c409ffa, 33 },
    { 0x72f09ffb, 46 },
    { 0xc2c09ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b0a004, 85 },
    { 0xb5b0a004, 52 },
    { 0x4c50a005, 23 },
    { 0x8330a007, 90 },
    { 0, 0 },
    { 0xfb90a009, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0a00c, 61 },
    { 0x9890a00c, 33 },
    { 0xe4c0a00e, 23 },
    { 0xc1d0a00f, 33 },
    { 0xd220a00e, 61 },
    { 0, 0 },
    { 0xb550a012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb60a019, 61 },
    { 0x380a01a, 34 },
    { 0xf550a01a, 90 },
    { 0xb1a0a01b, 33 },
    { 0x61e0a01d, 76 },
    { 0, 0 },
    { 0xab10a01f, 93 },
    { 0, 0 },
    { 0x8b80a021, 98 },
    { 0x50e0a022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa0a028, 76 },
    { 0x70f0a029, 90 },
    { 0, 0 },
    { 0xb590a02b, 95 },
    { 0x1330a02b, 95 },
    { 0x5170a02d, 61 },
    { 0, 0 },
    { 0xf150a02f, 90 },
    { 0xec40a030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0a037, 55 },
    { 0x7e80a038, 76 },
    { 0xd380a039, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8300a03d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b30a043, 33 },
    { 0xb3e0a044, 90 },
    { 0xc290a043, 98 },
    { 0xb510a046, 33 },
    { 0x40f0a045, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a00a04a, 76 },
    { 0xdfa0a04b, 17 },
    { 0xbe50a04c, 33 },
    { 0x9010a04d, 61 },
    { 0xc610a04e, 94 },
    { 0xde50a04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf320a054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea50a05c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60a05f, 90 },
    { 0, 0 },
    { 0xc620a061, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8760a065, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870a068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d0a06b, 76 },
    { 0xd3e0a06c, 23 },
    { 0, 0 },
    { 0xdcd0a06e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1200a077, 55 },
    { 0x1760a077, 91 },
    { 0x2890a079, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd540a07e, 65 },
    { 0, 0 },
    { 0x6c00a080, 43 },
    { 0x1020a081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce60a086, 33 },
    { 0, 0 },
    { 0x29c0a088, 77 },
    { 0xb170a089, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e0a094, 61 },
    { 0x90a095, 33 },
    { 0xac20a095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c0a09b, 95 },
    { 0xdb0a09c, 61 },
    { 0x4530a09d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50a0a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60a0ab, 61 },
    { 0x3450a0ac, 46 },
    { 0xac60a0ad, 77 },
    { 0x5900a0ae, 77 },
    { 0x6f50a0ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d0a0b4, 99 },
    { 0x8a60a0b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b70a0b8, 67 },
    { 0xded0a0b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec0a0bc, 33 },
    { 0, 0 },
    { 0x4cd0a0be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c0a0c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20a0c5, 43 },
    { 0, 0 },
    { 0x22d0a0c7, 61 },
    { 0x4ef0a0c8, 61 },
    { 0, 0 },
    { 0xc160a0ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0a0ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350a0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90a0dd, 43 },
    { 0x5470a0dd, 90 },
    { 0, 0 },
    { 0x8a50a0e0, 61 },
    { 0xa550a0e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40a0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a0a0ef, 55 },
    { 0xa7b0a0f0, 17 },
    { 0, 0 },
    { 0xe20a0f2, 43 },
    { 0xaef0a0f3, 23 },
    { 0x3c50a0f4, 55 },
    { 0x2a10a0f5, 38 },
    { 0x8830a0f6, 43 },
    { 0, 0 },
    { 0x1e10a0f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2360a0fb, 23 },
    { 0x7700a0fc, 33 },
    { 0xc8a0a0fd, 76 },
    { 0x87d0a0fb, 61 },
    { 0x12f0a0ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180a103, 55 },
    { 0x7fe0a104, 95 },
    { 0xc630a105, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7240a108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1590a10b, 98 },
    { 0x560a10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf220a111, 90 },
    { 0xf450a112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f60a116, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd480a11b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a0a124, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe450a128, 23 },
    { 0, 0 },
    { 0x6a10a12a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0a130, 95 },
    { 0xfb30a131, 99 },
    { 0xef90a130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0a135, 76 },
    { 0x62f0a135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a139, 90 },
    { 0x2700a13a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7890a13d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570a143, 61 },
    { 0, 0 },
    { 0x2e00a145, 61 },
    { 0, 0 },
    { 0x1ba0a147, 61 },
    { 0x2560a147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120a14c, 90 },
    { 0x5ce0a14d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd880a153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a0a158, 87 },
    { 0xc990a158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a15e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3040a162, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050a165, 23 },
    { 0x4670a166, 49 },
    { 0x6e20a167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e50a16e, 33 },
    { 0x5870a16f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c0a172, 46 },
    { 0xf680a172, 61 },
    { 0, 0 },
    { 0x6990a175, 66 },
    { 0xb820a175, 100 },
    { 0, 0 },
    { 0x4c40a178, 83 },
    { 0x2220a178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40a17d, 23 },
    { 0x7600a17e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1740a181, 76 },
    { 0x3f60a182, 90 },
    { 0xc8d0a183, 46 },
    { 0, 0 },
    { 0xa80a185, 90 },
    { 0x5330a186, 76 },
    { 0x2520a185, 90 },
    { 0x7880a188, 33 },
    { 0xab80a189, 33 },
    { 0x8a50a18a, 98 },
    { 0xead0a18b, 85 },
    { 0x1010a18a, 33 },
    { 0x2440a18b, 65 },
    { 0x69d0a18e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7790a195, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf110a198, 61 },
    { 0x7cf0a199, 90 },
    { 0x4900a19a, 23 },
    { 0xfff0a199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840a19e, 61 },
    { 0x3830a19f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd0a1a3, 86 },
    { 0x3890a1a3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a1ab, 33 },
    { 0, 0 },
    { 0x4930a1ad, 61 },
    { 0x7ed0a1ae, 61 },
    { 0, 0 },
    { 0x4910a1b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e0a1b3, 70 },
    { 0x9bc0a1b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1570a1b8, 23 },
    { 0, 0 },
    { 0xf930a1ba, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9110a1be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680a1c7, 76 },
    { 0x130a1c8, 61 },
    { 0xe260a1c8, 95 },
    { 0, 0 },
    { 0x8ef0a1cb, 23 },
    { 0, 0 },
    { 0x9880a1cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1d4, 23 },
    { 0x55a0a1d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc90a1d8, 55 },
    { 0x8660a1d9, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a0a1de, 64 },
    { 0x95c0a1de, 98 },
    { 0, 0 },
    { 0xd990a1e1, 76 },
    { 0x3680a1e1, 33 },
    { 0xc4c0a1e3, 33 },
    { 0xa840a1e4, 23 },
    { 0xcc0a1e5, 95 },
    { 0x28c0a1e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7860a1ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c20a1f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe160a1f8, 88 },
    { 0x85f0a1f9, 47 },
    { 0x9bb0a1fa, 90 },
    { 0, 0 },
    { 0xc080a1fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e30a1ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720a203, 99 },
    { 0x82f0a204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0a207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc0a20d, 78 },
    { 0x6510a20e, 34 },
    { 0, 0 },
    { 0x460a210, 34 },
    { 0xcac0a211, 34 },
    { 0xbbf0a212, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f0a215, 93 },
    { 0x6cb0a216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800a21d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b70a220, 34 },
    { 0xd050a220, 55 },
    { 0x54b0a222, 55 },
    { 0x9a90a223, 85 },
    { 0x9b70a222, 33 },
    { 0xeab0a224, 100 },
    { 0x2ac0a226, 95 },
    { 0xe180a225, 23 },
    { 0x2a50a228, 98 },
    { 0x13c0a228, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd010a22e, 55 },
    { 0xde40a22f, 88 },
    { 0xbc10a230, 95 },
    { 0, 0 },
    { 0x8c0a232, 98 },
    { 0, 0 },
    { 0x5510a234, 59 },
    { 0x850a235, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9600a239, 95 },
    { 0x9c50a239, 23 },
    { 0x77c0a23a, 33 },
    { 0, 0 },
    { 0x3bd0a23d, 55 },
    { 0x7fb0a23e, 91 },
    { 0xf9d0a23e, 23 },
    { 0, 0 },
    { 0x540a241, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d0a244, 84 },
    { 0xa780a245, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0a249, 61 },
    { 0, 0 },
    { 0xb2d0a24b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a24e, 90 },
    { 0xd8a0a24e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d60a255, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f30a25b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40a25e, 76 },
    { 0, 0 },
    { 0x26d0a260, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70a263, 93 },
    { 0x91d0a264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe410a267, 95 },
    { 0x3900a268, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590a26f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4770a278, 61 },
    { 0x1aa0a278, 23 },
    { 0x5b10a27a, 87 },
    { 0xbba0a27b, 67 },
    { 0x4140a27c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa10a280, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c0a283, 90 },
    { 0, 0 },
    { 0x2e00a285, 95 },
    { 0xae70a286, 55 },
    { 0x6c00a287, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c0a292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0a29a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0a29e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0a2a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0a2a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30a2a9, 23 },
    { 0x3ce0a2aa, 23 },
    { 0xb470a2ab, 76 },
    { 0x1ae0a2ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a2b0, 61 },
    { 0xb500a2b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10a2b4, 55 },
    { 0xcb50a2b5, 95 },
    { 0x6770a2b6, 33 },
    { 0, 0 },
    { 0xe6e0a2b8, 78 },
    { 0, 0 },
    { 0xd8c0a2ba, 98 },
    { 0xe410a2bb, 90 },
    { 0x4140a2bb, 33 },
    { 0, 0 },
    { 0x7620a2be, 80 },
    { 0x26a0a2bf, 46 },
    { 0x890a2c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0a2c3, 76 },
    { 0x7940a2c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d80a2c8, 23 },
    { 0, 0 },
    { 0x1c10a2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb110a2cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0a2d0, 93 },
    { 0x61d0a2d1, 59 },
    { 0, 0 },
    { 0x2770a2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d0a2d6, 90 },
    { 0x5510a2d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe650a2e0, 76 },
    { 0, 0 },
    { 0xe940a2e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9840a2e9, 69 },
    { 0, 0 },
    { 0xebc0a2eb, 65 },
    { 0x9450a2eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630a2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b0a2f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c0a2fb, 33 },
    { 0x8730a2fb, 87 },
    { 0x57b0a2fc, 75 },
    { 0xb1e0a2fb, 87 },
    { 0x3530a2fb, 93 },
    { 0xd680a2fb, 93 },
    { 0xb1d0a301, 33 },
    { 0xb080a302, 43 },
    { 0xac00a303, 95 },
    { 0x95f0a303, 98 },
    { 0, 0 },
    { 0xcfe0a306, 90 },
    { 0x9b80a306, 76 },
    { 0xa360a308, 43 },
    { 0, 0 },
    { 0x780a30a, 95 },
    { 0xa720a30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480a30e, 98 },
    { 0x36f0a30f, 84 },
    { 0x38f0a310, 90 },
    { 0x4470a310, 76 },
    { 0x76b0a312, 59 },
    { 0x33e0a311, 61 },
    { 0, 0 },
    { 0xa280a315, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9630a31b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3610a31f, 55 },
    { 0, 0 },
    { 0x86c0a321, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e80a324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae0a327, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e0a32d, 66 },
    { 0, 0 },
    { 0x3c90a32f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920a335, 61 },
    { 0, 0 },
    { 0x67f0a337, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf440a33f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d0a343, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d0a346, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd10a34b, 90 },
    { 0, 0 },
    { 0xafe0a34d, 61 },
    { 0, 0 },
    { 0x3ad0a34f, 52 },
    { 0xba10a34f, 98 },
    { 0x38d0a351, 43 },
    { 0x84d0a352, 76 },
    { 0, 0 },
    { 0xfae0a354, 100 },
    { 0xbf30a355, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a0a35a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe190a35e, 91 },
    { 0, 0 },
    { 0xa830a360, 33 },
    { 0x8dc0a361, 61 },
    { 0, 0 },
    { 0x2e40a363, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0a369, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00a36c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20a36f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d0a37f, 55 },
    { 0xa1b0a37f, 66 },
    { 0x5ab0a380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400a38d, 90 },
    { 0, 0 },
    { 0xfd80a38f, 91 },
    { 0, 0 },
    { 0x4f00a391, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0a397, 33 },
    { 0xc460a397, 61 },
    { 0x2fe0a399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd910a39d, 95 },
    { 0, 0 },
    { 0xf5f0a39f, 64 },
    { 0x9350a3a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e0a3a4, 18 },
    { 0x4230a3a5, 52 },
    { 0x7aa0a3a5, 84 },
    { 0xede0a3a7, 34 },
    { 0xfeb0a3a5, 23 },
    { 0xbb70a3a9, 33 },
    { 0x1740a3aa, 98 },
    { 0, 0 },
    { 0x8510a3ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8580a3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0a3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad30a3b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490a3bf, 87 },
    { 0x6530a3c0, 23 },
    { 0xbdc0a3c1, 75 },
    { 0x4820a3bf, 96 },
    { 0xbdd0a3c3, 43 },
    { 0xd030a3c3, 95 },
    { 0xc340a3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f0a3c8, 33 },
    { 0, 0 },
    { 0x5b30a3ca, 95 },
    { 0xeb70a3cb, 23 },
    { 0xde90a3cc, 98 },
    { 0xf420a3cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80a3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b0a3d7, 23 },
    { 0xc720a3d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc0a3dc, 23 },
    { 0x5b00a3dd, 87 },
    { 0x7850a3de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0a3e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a3e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0a3e7, 34 },
    { 0, 0 },
    { 0x2ed0a3e9, 43 },
    { 0, 0 },
    { 0x8020a3eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d0a3ee, 33 },
    { 0xee20a3ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f0a3f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e40a3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90a3fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa280a403, 36 },
    { 0x4f60a403, 98 },
    { 0xa1d0a405, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0a411, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1690a415, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90a41a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be0a424, 34 },
    { 0x23e0a425, 33 },
    { 0, 0 },
    { 0xeab0a427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790a42c, 41 },
    { 0x4210a42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740a431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10a434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0a438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef0a43e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a0a443, 88 },
    { 0xd950a444, 95 },
    { 0, 0 },
    { 0xdf60a446, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa590a449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1390a44c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f0a450, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a0a455, 85 },
    { 0x2bb0a455, 55 },
    { 0xa950a456, 43 },
    { 0xe380a458, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f0a45d, 98 },
    { 0, 0 },
    { 0xc970a45f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0a462, 85 },
    { 0, 0 },
    { 0x4150a464, 90 },
    { 0xc80a465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b70a46b, 52 },
    { 0, 0 },
    { 0xb3b0a46d, 77 },
    { 0, 0 },
    { 0x18f0a46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e0a478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20a47b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60a47f, 34 },
    { 0, 0 },
    { 0xf4b0a481, 78 },
    { 0xc610a481, 27 },
    { 0x4ff0a483, 66 },
    { 0xbdf0a484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6180a48c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0a490, 95 },
    { 0xf2d0a490, 23 },
    { 0x4f30a492, 95 },
    { 0xf1a0a493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a0a49c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4310a4a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910a4a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea0a4ad, 36 },
    { 0, 0 },
    { 0x8000a4af, 100 },
    { 0x7e70a4b0, 77 },
    { 0x89f0a4af, 87 },
    { 0x2b60a4b2, 34 },
    { 0xa310a4b3, 66 },
    { 0x17c0a4b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a90a4bb, 33 },
    { 0x6f0a4bc, 43 },
    { 0xc260a4bc, 33 },
    { 0x5f80a4be, 61 },
    { 0x61a0a4be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880a4ca, 23 },
    { 0x3160a4cb, 67 },
    { 0x9b50a4cc, 52 },
    { 0x2240a4cc, 34 },
    { 0x2700a4cb, 98 },
    { 0xddd0a4ce, 33 },
    { 0x5650a4d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d30a4d6, 90 },
    { 0, 0 },
    { 0xa0c0a4d8, 90 },
    { 0xcd00a4d8, 98 },
    { 0xbb90a4d8, 95 },
    { 0x8330a4db, 23 },
    { 0x2ba0a4dc, 77 },
    { 0xdc40a4db, 98 },
    { 0x6990a4d8, 98 },
    { 0x8e0a4da, 55 },
    { 0, 0 },
    { 0xed00a4e1, 78 },
    { 0xb540a4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c0a4e6, 93 },
    { 0, 0 },
    { 0x7360a4e8, 91 },
    { 0xc360a4e9, 23 },
    { 0, 0 },
    { 0x8770a4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110a4ee, 34 },
    { 0x1830a4ef, 100 },
    { 0x4240a4f0, 55 },
    { 0x6880a4f0, 43 },
    { 0x5690a4f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a4f7, 90 },
    { 0x6420a4f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a4fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c30a4ff, 99 },
    { 0xdb70a500, 98 },
    { 0, 0 },
    { 0x8740a502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0a506, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b0a50b, 98 },
    { 0x63e0a50b, 90 },
    { 0x550a50d, 23 },
    { 0x2100a50d, 43 },
    { 0xc340a50e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf0a515, 33 },
    { 0x5b00a516, 95 },
    { 0x2490a516, 98 },
    { 0x5820a518, 90 },
    { 0xeba0a519, 90 },
    { 0xb3d0a519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5120a521, 90 },
    { 0, 0 },
    { 0xa2f0a523, 95 },
    { 0x35e0a524, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb240a52b, 23 },
    { 0, 0 },
    { 0x40b0a52d, 66 },
    { 0x1090a52e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd0a531, 92 },
    { 0, 0 },
    { 0xa070a533, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b0a538, 23 },
    { 0xa5f0a538, 24 },
    { 0, 0 },
    { 0x24f0a53b, 43 },
    { 0x2d40a53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a00a540, 95 },
    { 0, 0 },
    { 0x2a70a542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb700a545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab0a54c, 33 },
    { 0x3a30a54d, 99 },
    { 0xa660a54e, 23 },
    { 0xb250a54f, 100 },
    { 0, 0 },
    { 0x1920a551, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00a555, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290a559, 33 },
    { 0, 0 },
    { 0x8ba0a55b, 76 },
    { 0xc0e0a55b, 97 },
    { 0xce50a55d, 87 },
    { 0xf3b0a55d, 61 },
    { 0x7b50a55f, 17 },
    { 0xab10a560, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9450a565, 66 },
    { 0x61d0a566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c0a56a, 78 },
    { 0xadc0a56b, 33 },
    { 0, 0 },
    { 0x8b30a56d, 93 },
    { 0x8be0a56e, 76 },
    { 0x3e30a56d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560a576, 76 },
    { 0x3080a577, 46 },
    { 0xb770a577, 52 },
    { 0, 0 },
    { 0x75e0a57a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0a57e, 46 },
    { 0x7570a57f, 95 },
    { 0x34a0a580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40a584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa070a587, 95 },
    { 0x79a0a588, 76 },
    { 0x40c0a588, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1740a58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0a592, 100 },
    { 0, 0 },
    { 0xc210a594, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2930a59a, 33 },
    { 0x8880a59b, 91 },
    { 0, 0 },
    { 0x56d0a59d, 66 },
    { 0x5850a59e, 90 },
    { 0xcd0a59e, 33 },
    { 0x9440a59f, 43 },
    { 0xae50a5a0, 35 },
    { 0xf410a5a0, 76 },
    { 0xa700a5a3, 98 },
    { 0x1b00a5a4, 33 },
    { 0, 0 },
    { 0x510a5a6, 34 },
    { 0xfe10a5a7, 33 },
    { 0x5a60a5a7, 61 },
    { 0x3480a5a9, 23 },
    { 0xf4f0a5aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f0a5b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b0a5b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca0a5bc, 98 },
    { 0x2620a5bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0a5c0, 40 },
    { 0xc1f0a5c1, 90 },
    { 0x4450a5c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e0a5c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80a5ca, 46 },
    { 0, 0 },
    { 0x1d60a5cc, 88 },
    { 0xdee0a5cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700a5d2, 55 },
    { 0x740a5d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250a5d8, 77 },
    { 0x9150a5d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e0a5df, 95 },
    { 0, 0 },
    { 0xa770a5e1, 76 },
    { 0xda0a5e2, 23 },
    { 0x7280a5e3, 43 },
    { 0, 0 },
    { 0xa630a5e5, 78 },
    { 0x4100a5e6, 78 },
    { 0, 0 },
    { 0xdd70a5e8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0a5ee, 61 },
    { 0xdff0a5ee, 52 },
    { 0x3e60a5f0, 95 },
    { 0, 0 },
    { 0x3a40a5f2, 76 },
    { 0xe1a0a5f2, 95 },
    { 0xe210a5f2, 61 },
    { 0, 0 },
    { 0x9db0a5f6, 40 },
    { 0, 0 },
    { 0x6f20a5f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa00a603, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac0a60a, 78 },
    { 0xb510a60b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd80a60e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80a612, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650a616, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530a61a, 76 },
    { 0xb30a61b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50a627, 66 },
    { 0x8250a628, 23 },
    { 0x2ad0a629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00a62f, 46 },
    { 0xa150a630, 43 },
    { 0xec10a630, 95 },
    { 0, 0 },
    { 0xf70a633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d20a636, 95 },
    { 0, 0 },
    { 0x46d0a638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d0a63b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090a643, 17 },
    { 0x4670a643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a0a64b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f0a650, 95 },
    { 0, 0 },
    { 0x3060a652, 33 },
    { 0x7c50a653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a50a662, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0a666, 36 },
    { 0x8420a666, 93 },
    { 0x5300a668, 23 },
    { 0x5230a669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0a66c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030a673, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0a67b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a67e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0a681, 90 },
    { 0xadd0a682, 91 },
    { 0x3700a682, 55 },
    { 0, 0 },
    { 0xd0d0a685, 43 },
    { 0x94c0a686, 33 },
    { 0x5840a686, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120a68a, 67 },
    { 0, 0 },
    { 0xcde0a68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b70a68f, 76 },
    { 0, 0 },
    { 0x4e00a691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0a695, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c0a699, 23 },
    { 0x3960a69a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a0a69f, 61 },
    { 0, 0 },
    { 0x61c0a6a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d40a6a4, 87 },
    { 0x5380a6a5, 78 },
    { 0, 0 },
    { 0x1980a6a7, 76 },
    { 0x3b90a6a7, 98 },
    { 0x4330a6a9, 67 },
    { 0x2b70a6aa, 23 },
    { 0x2eb0a6ab, 61 },
    { 0, 0 },
    { 0xfc40a6ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0a6b3, 43 },
    { 0xa8c0a6b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c0a6bb, 61 },
    { 0x9510a6bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0a6bf, 61 },
    { 0x5dc0a6c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00a6c6, 90 },
    { 0x2c80a6c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00a6cb, 23 },
    { 0xe1d0a6cc, 75 },
    { 0x88a0a6cd, 61 },
    { 0xf490a6ce, 95 },
    { 0xff20a6cf, 55 },
    { 0x5530a6d0, 34 },
    { 0x5610a6d0, 90 },
    { 0x8880a6d2, 77 },
    { 0xa990a6d0, 59 },
    { 0xc5f0a6d2, 95 },
    { 0x7830a6d4, 65 },
    { 0x8100a6d6, 23 },
    { 0x4c60a6d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80a6db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710a6df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6010a6e6, 17 },
    { 0xd890a6e6, 98 },
    { 0xbec0a6e7, 78 },
    { 0, 0 },
    { 0x6120a6ea, 91 },
    { 0, 0 },
    { 0xdbf0a6ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda40a6f0, 27 },
    { 0x6130a6f1, 33 },
    { 0xde60a6f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa510a6f7, 85 },
    { 0x9500a6f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f0a6fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690a700, 61 },
    { 0xe3c0a701, 61 },
    { 0xd260a702, 90 },
    { 0, 0 },
    { 0x2700a704, 85 },
    { 0, 0 },
    { 0x1250a706, 33 },
    { 0x8480a707, 23 },
    { 0x4a10a708, 66 },
    { 0, 0 },
    { 0x6330a70a, 33 },
    { 0, 0 },
    { 0xa440a70c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850a711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf0a714, 33 },
    { 0x95d0a715, 55 },
    { 0, 0 },
    { 0x8200a717, 95 },
    { 0, 0 },
    { 0x4690a719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d0a71c, 91 },
    { 0x9e90a71d, 90 },
    { 0xba00a71e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb620a721, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6160a72f, 85 },
    { 0x3a00a730, 100 },
    { 0x6250a731, 76 },
    { 0, 0 },
    { 0xe4a0a733, 34 },
    { 0xd0c0a734, 90 },
    { 0, 0 },
    { 0x8680a736, 98 },
    { 0x6d40a737, 55 },
    { 0, 0 },
    { 0xdb40a739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd860a73c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690a740, 23 },
    { 0, 0 },
    { 0xf3b0a742, 77 },
    { 0x970a743, 90 },
    { 0x6740a742, 33 },
    { 0x8ad0a745, 46 },
    { 0, 0 },
    { 0x1b90a747, 95 },
    { 0, 0 },
    { 0xe850a749, 76 },
    { 0xd0d0a74a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0a74d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0a755, 95 },
    { 0x30b0a756, 55 },
    { 0, 0 },
    { 0x3b90a758, 98 },
    { 0, 0 },
    { 0x6640a75a, 100 },
    { 0x8390a75b, 23 },
    { 0xe1e0a75c, 90 },
    { 0xea30a75d, 89 },
    { 0xe8f0a75e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc970a762, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a0a766, 95 },
    { 0x7f0a767, 98 },
    { 0x83e0a768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970a76c, 95 },
    { 0x9280a76d, 61 },
    { 0xef0a76d, 61 },
    { 0x16e0a76c, 95 },
    { 0xa240a770, 76 },
    { 0x59d0a76e, 43 },
    { 0, 0 },
    { 0xb740a773, 55 },
    { 0xdef0a774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6970a778, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1190a77c, 55 },
    { 0xf030a77d, 76 },
    { 0x93c0a77d, 85 },
    { 0x480a77f, 23 },
    { 0x8790a780, 98 },
    { 0xf940a781, 95 },
    { 0xb7a0a782, 90 },
    { 0, 0 },
    { 0x9820a784, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e40a794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220a79a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d0a79f, 33 },
    { 0x7340a79f, 66 },
    { 0xce80a7a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d50a7a5, 33 },
    { 0, 0 },
    { 0x3f60a7a7, 93 },
    { 0x51a0a7a7, 93 },
    { 0xcf00a7a9, 95 },
    { 0, 0 },
    { 0xdc90a7ab, 61 },
    { 0, 0 },
    { 0xae00a7ad, 23 },
    { 0xf100a7ad, 23 },
    { 0, 0 },
    { 0xc410a7b0, 99 },
    { 0x68b0a7b1, 97 },
    { 0x8e00a7b2, 68 },
    { 0x6560a7b1, 33 },
    { 0xa490a7b3, 23 },
    { 0, 0 },
    { 0x2060a7b6, 33 },
    { 0, 0 },
    { 0x2310a7b8, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd0a7bc, 99 },
    { 0x4720a7bd, 90 },
    { 0xaa30a7bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2510a7cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c0a7d1, 91 },
    { 0xd0a0a7d2, 55 },
    { 0x3650a7d1, 67 },
    { 0xb170a7d4, 43 },
    { 0, 0 },
    { 0x3350a7d6, 34 },
    { 0xf0e0a7d7, 87 },
    { 0x70a7d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf0a7df, 93 },
    { 0x7070a7df, 55 },
    { 0, 0 },
    { 0x7a50a7e2, 66 },
    { 0x530a7e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb210a7e6, 46 },
    { 0x4d00a7e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e0a7ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0a7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2130a7f5, 98 },
    { 0x1370a7f6, 23 },
    { 0x9530a7f7, 33 },
    { 0xb890a7f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb50a7fc, 98 },
    { 0, 0 },
    { 0xb5c0a7fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b0a803, 43 },
    { 0, 0 },
    { 0x7c70a805, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb40a80a, 34 },
    { 0x13c0a80b, 76 },
    { 0, 0 },
    { 0xdfb0a80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3720a811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0a818, 83 },
    { 0x1b80a819, 87 },
    { 0xb4c0a819, 23 },
    { 0x67d0a81b, 98 },
    { 0x8940a81a, 33 },
    { 0xe4d0a81d, 90 },
    { 0x46a0a81d, 23 },
    { 0, 0 },
    { 0x4f00a820, 43 },
    { 0x4f10a821, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40a824, 23 },
    { 0x6060a825, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900a829, 55 },
    { 0, 0 },
    { 0xfb80a82b, 51 },
    { 0x5090a82b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4530a831, 95 },
    { 0xf650a832, 66 },
    { 0xb790a831, 61 },
    { 0x5370a834, 55 },
    { 0, 0 },
    { 0x6c0a836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc0a83c, 44 },
    { 0x6ad0a83d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b0a846, 43 },
    { 0x3040a846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c80a84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8900a84f, 33 },
    { 0, 0 },
    { 0xd350a851, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7300a854, 76 },
    { 0x9510a854, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b80a858, 33 },
    { 0xb760a859, 90 },
    { 0xd640a859, 90 },
    { 0xb9b0a85b, 81 },
    { 0xebb0a859, 95 },
    { 0xf340a85d, 61 },
    { 0, 0 },
    { 0xb340a85f, 85 },
    { 0x3890a860, 23 },
    { 0xeca0a860, 90 },
    { 0x9a0a862, 33 },
    { 0xaf70a85f, 33 },
    { 0x71a0a861, 98 },
    { 0x5be0a865, 62 },
    { 0xe4d0a866, 33 },
    { 0x3810a865, 87 },
    { 0x5310a868, 95 },
    { 0xc6c0a869, 98 },
    { 0x1680a86a, 99 },
    { 0xa290a86a, 61 },
    { 0x8090a86c, 66 },
    { 0x9180a86d, 91 },
    { 0xaa50a86b, 61 },
    { 0x8e60a861, 100 },
    { 0xeeb0a870, 90 },
    { 0xd380a871, 23 },
    { 0xbfc0a870, 36 },
    { 0x53f0a873, 90 },
    { 0, 0 },
    { 0xaa60a875, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff0a878, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0a87b, 98 },
    { 0, 0 },
    { 0xe860a87d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7240a881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd40a884, 23 },
    { 0, 0 },
    { 0xc640a886, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4060a889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60a88c, 89 },
    { 0x1020a88d, 61 },
    { 0x5910a88e, 33 },
    { 0, 0 },
    { 0xe0c0a890, 61 },
    { 0x1d90a891, 95 },
    { 0x6290a891, 23 },
    { 0x360a893, 53 },
    { 0x3920a894, 56 },
    { 0x63f0a895, 95 },
    { 0xd1e0a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0a899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da0a89c, 95 },
    { 0xf770a89d, 34 },
    { 0x7c50a89e, 34 },
    { 0x8650a89f, 93 },
    { 0xe580a8a0, 98 },
    { 0xe5f0a8a1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4840a8a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd0a8a9, 32 },
    { 0x4fe0a8aa, 33 },
    { 0, 0 },
    { 0x4a0a8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1190a8b4, 43 },
    { 0xbfe0a8b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980a8b8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0a8c6, 91 },
    { 0x6190a8c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0a8cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0a8ce, 91 },
    { 0x5250a8ce, 98 },
    { 0xa100a8d0, 80 },
    { 0x620a8d1, 43 },
    { 0x40f0a8cf, 23 },
    { 0, 0 },
    { 0x8f40a8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3450a8d9, 98 },
    { 0, 0 },
    { 0x3750a8db, 55 },
    { 0, 0 },
    { 0xfb20a8dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60a8e0, 99 },
    { 0x1ca0a8e1, 80 },
    { 0x5780a8e1, 95 },
    { 0xf90a8e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b0a8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e70a8e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d0a8ec, 45 },
    { 0x1720a8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540a8f0, 43 },
    { 0xb440a8f1, 95 },
    { 0x5590a8f2, 90 },
    { 0xf0c0a8f3, 33 },
    { 0x50a8f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc520a8f7, 90 },
    { 0x3840a8f8, 80 },
    { 0xcdc0a8f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6400a8fd, 95 },
    { 0xb0c0a8fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a901, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd20a904, 88 },
    { 0x34a0a905, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0a90f, 75 },
    { 0x8f20a90f, 93 },
    { 0xdff0a911, 90 },
    { 0xfe90a911, 98 },
    { 0xe5a0a90f, 76 },
    { 0x7f50a911, 67 },
    { 0, 0 },
    { 0xb990a916, 87 },
    { 0x5950a917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a90a91a, 87 },
    { 0, 0 },
    { 0xd9f0a91c, 52 },
    { 0, 0 },
    { 0xacd0a91e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e70a92f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c0a933, 93 },
    { 0xa2c0a934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0a938, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e70a93e, 17 },
    { 0x8860a93f, 98 },
    { 0xdd80a93f, 76 },
    { 0xfee0a941, 61 },
    { 0x22d0a941, 75 },
    { 0xf690a943, 69 },
    { 0xac0a944, 33 },
    { 0xa0b0a940, 55 },
    { 0xf990a93e, 88 },
    { 0, 0 },
    { 0x6290a948, 46 },
    { 0xbd80a949, 90 },
    { 0xb3c0a948, 23 },
    { 0, 0 },
    { 0xa2b0a94c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e0a94f, 87 },
    { 0xb0e0a950, 98 },
    { 0x1220a94f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3920a954, 98 },
    { 0xab40a955, 98 },
    { 0, 0 },
    { 0x3d50a957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b0a964, 33 },
    { 0, 0 },
    { 0x4ef0a966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20a96d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f0a970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3370a973, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0a978, 34 },
    { 0, 0 },
    { 0x7aa0a97a, 33 },
    { 0x9f70a97b, 61 },
    { 0xe870a97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac00a983, 23 },
    { 0, 0 },
    { 0x46a0a985, 33 },
    { 0x6b10a986, 86 },
    { 0x3c10a985, 78 },
    { 0x8430a988, 66 },
    { 0xb410a985, 98 },
    { 0x4b50a985, 23 },
    { 0, 0 },
    { 0xd690a98c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f60a993, 75 },
    { 0x1fe0a994, 95 },
    { 0x5dd0a995, 55 },
    { 0, 0 },
    { 0xd3d0a997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790a99b, 43 },
    { 0, 0 },
    { 0xb960a99d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f0a9a2, 61 },
    { 0x12f0a9a3, 98 },
    { 0x5c50a9a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b0a9a9, 81 },
    { 0x3100a9a9, 90 },
    { 0xf0a9aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0a9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4460a9ba, 76 },
    { 0x1910a9bb, 61 },
    { 0x2510a9bc, 33 },
    { 0x54e0a9bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe680a9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e0a9c5, 76 },
    { 0, 0 },
    { 0xf5f0a9c7, 77 },
    { 0x48a0a9c8, 33 },
    { 0x2530a9c9, 43 },
    { 0xdd90a9c8, 90 },
    { 0, 0 },
    { 0xc850a9cc, 61 },
    { 0, 0 },
    { 0x47f0a9ce, 64 },
    { 0, 0 },
    { 0xb980a9d0, 34 },
    { 0, 0 },
    { 0xd70a9d2, 55 },
    { 0, 0 },
    { 0x8260a9d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7730a9d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0a9db, 69 },
    { 0x22f0a9dc, 46 },
    { 0, 0 },
    { 0x6a60a9de, 55 },
    { 0x6930a9de, 78 },
    { 0, 0 },
    { 0x6d90a9e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe810a9e4, 33 },
    { 0xab80a9e5, 95 },
    { 0, 0 },
    { 0xf0c0a9e7, 90 },
    { 0, 0 },
    { 0xcd40a9e9, 61 },
    { 0, 0 },
    { 0x7a40a9eb, 85 },
    { 0xaf00a9ec, 85 },
    { 0xd070a9ed, 93 },
    { 0x78a0a9ee, 55 },
    { 0x7d0a9ee, 61 },
    { 0xf380a9ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a40a9fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0aa05, 33 },
    { 0, 0 },
    { 0x500aa07, 33 },
    { 0, 0 },
    { 0xd4d0aa09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe040aa0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10aa12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c0aa15, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f0aa19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c60aa1c, 98 },
    { 0, 0 },
    { 0x4850aa1e, 99 },
    { 0x6330aa1f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680aa24, 76 },
    { 0xb870aa24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820aa2c, 34 },
    { 0xe9f0aa2d, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0aa31, 23 },
    { 0x1dc0aa31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1420aa37, 33 },
    { 0, 0 },
    { 0xe6b0aa39, 95 },
    { 0x8580aa3a, 88 },
    { 0, 0 },
    { 0x1cc0aa3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0aa41, 61 },
    { 0, 0 },
    { 0x5d00aa43, 98 },
    { 0, 0 },
    { 0x7a30aa45, 78 },
    { 0x2ec0aa45, 84 },
    { 0x2900aa47, 93 },
    { 0xc580aa48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa080aa4d, 17 },
    { 0, 0 },
    { 0x3a0aa4f, 95 },
    { 0xed40aa4f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c0aa56, 61 },
    { 0, 0 },
    { 0xd800aa58, 84 },
    { 0xd0f0aa58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed70aa5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8380aa61, 95 },
    { 0, 0 },
    { 0x7fc0aa63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410aa66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7610aa6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc630aa70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd90aa74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9390aa77, 98 },
    { 0, 0 },
    { 0x9080aa79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a70aa87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b70aa8c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f0aa96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0aa9a, 90 },
    { 0x5590aa9b, 61 },
    { 0xbd30aa9c, 93 },
    { 0x9ec0aa9d, 53 },
    { 0xc190aa9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a0aaa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880aaa8, 84 },
    { 0x1490aaa9, 91 },
    { 0x6300aaaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c0aaaf, 78 },
    { 0, 0 },
    { 0xca70aab1, 33 },
    { 0xf750aab2, 67 },
    { 0xb1f0aab3, 38 },
    { 0x3880aab3, 61 },
    { 0, 0 },
    { 0x77f0aab6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7760aabb, 43 },
    { 0x3050aabb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xce20aabf, 75 },
    { 0xd50aabf, 55 },
    { 0, 0 },
    { 0xd6e0aac2, 38 },
    { 0, 0 },
    { 0x3690aac4, 75 },
    { 0x1470aac5, 34 },
    { 0x3e00aac6, 23 },
    { 0x80aac6, 90 },
    { 0x97c0aac4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50aacf, 52 },
    { 0x5f30aacf, 76 },
    { 0x1790aad1, 23 },
    { 0x2360aad1, 55 },
    { 0, 0 },
    { 0x30c0aad4, 95 },
    { 0x8b60aad5, 98 },
    { 0x9340aad6, 33 },
    { 0x7bd0aad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0aada, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f80aadd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0aae1, 23 },
    { 0x6160aae2, 78 },
    { 0xa500aae3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc550aae7, 55 },
    { 0x3270aae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0aaee, 51 },
    { 0xfdf0aaef, 61 },
    { 0x6fd0aaef, 76 },
    { 0xa100aaee, 52 },
    { 0xc5e0aaee, 90 },
    { 0x3cd0aaf3, 66 },
    { 0x4000aaf4, 76 },
    { 0x5080aaf3, 93 },
    { 0x8050aaf3, 98 },
    { 0xd4a0aaf7, 78 },
    { 0xad60aaf7, 90 },
    { 0x8fa0aaf9, 77 },
    { 0x8780aaf9, 91 },
    { 0xc040aaf7, 38 },
    { 0x4b70aafc, 33 },
    { 0xb400aafd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5110ab03, 95 },
    { 0x8f80ab03, 34 },
    { 0xf5a0ab04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf60ab0b, 43 },
    { 0x370ab0c, 98 },
    { 0x7dd0ab0c, 65 },
    { 0x4950ab0e, 76 },
    { 0xbe20ab0e, 61 },
    { 0x3b0ab0f, 40 },
    { 0, 0 },
    { 0xa850ab12, 61 },
    { 0xa950ab13, 95 },
    { 0xf9e0ab13, 33 },
    { 0, 0 },
    { 0x9810ab16, 46 },
    { 0xbec0ab16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9980ab1a, 66 },
    { 0, 0 },
    { 0x7860ab1c, 33 },
    { 0xea60ab1c, 52 },
    { 0xa5c0ab1e, 33 },
    { 0, 0 },
    { 0x8c20ab20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2510ab25, 98 },
    { 0xc9d0ab25, 46 },
    { 0xf360ab26, 46 },
    { 0, 0 },
    { 0x1ff0ab29, 95 },
    { 0xf7a0ab2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880ab31, 90 },
    { 0x9b10ab31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x820ab35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa0ab38, 84 },
    { 0x72d0ab38, 61 },
    { 0xfca0ab3a, 90 },
    { 0x2990ab3a, 90 },
    { 0x52c0ab3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae30ab3f, 23 },
    { 0xed50ab40, 93 },
    { 0x3110ab40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd70ab44, 78 },
    { 0x64b0ab44, 90 },
    { 0xf130ab45, 99 },
    { 0, 0 },
    { 0xaa70ab48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa90ab4e, 90 },
    { 0x1f20ab4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c0ab5b, 34 },
    { 0xd6b0ab5c, 95 },
    { 0, 0 },
    { 0x12c0ab5e, 87 },
    { 0, 0 },
    { 0x4f60ab60, 98 },
    { 0x4df0ab60, 34 },
    { 0, 0 },
    { 0xa450ab63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8950ab6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c40ab71, 55 },
    { 0xe9c0ab71, 44 },
    { 0, 0 },
    { 0x7510ab74, 61 },
    { 0xc90ab75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ab79, 55 },
    { 0xf780ab7a, 95 },
    { 0x44c0ab7b, 33 },
    { 0x2750ab7a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00ab7f, 23 },
    { 0, 0 },
    { 0x7ff0ab81, 88 },
    { 0x19d0ab81, 98 },
    { 0xb1b0ab83, 95 },
    { 0xf3a0ab83, 43 },
    { 0x5910ab85, 33 },
    { 0x8010ab82, 100 },
    { 0x8750ab84, 61 },
    { 0x9ff0ab88, 78 },
    { 0xd780ab81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf0ab8d, 100 },
    { 0xc650ab8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10ab91, 90 },
    { 0, 0 },
    { 0x3c90ab93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a0ab98, 98 },
    { 0x11e0ab99, 90 },
    { 0, 0 },
    { 0xcad0ab9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c0ab9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480aba1, 90 },
    { 0x5150aba2, 55 },
    { 0xede0aba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930abb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330abb6, 61 },
    { 0x27c0abb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490abbc, 93 },
    { 0x33c0abbd, 98 },
    { 0x4be0abbe, 43 },
    { 0x4620abbe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8140abc4, 43 },
    { 0xe540abc5, 33 },
    { 0xaad0abc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e0abc9, 33 },
    { 0xbca0abca, 95 },
    { 0x37f0abcb, 33 },
    { 0xc0c0abcc, 98 },
    { 0x14e0abcd, 86 },
    { 0x3320abcd, 99 },
    { 0, 0 },
    { 0xddf0abd0, 95 },
    { 0x62c0abd1, 78 },
    { 0xd610abd1, 68 },
    { 0, 0 },
    { 0xdb60abd4, 95 },
    { 0, 0 },
    { 0x8c90abd6, 33 },
    { 0xa490abd6, 93 },
    { 0x30d0abd7, 61 },
    { 0, 0 },
    { 0x8410abda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b0abdf, 90 },
    { 0, 0 },
    { 0x6a10abe1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b0abe6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa0abea, 95 },
    { 0x1c30abea, 43 },
    { 0xd2f0abec, 66 },
    { 0x7a20abec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350abf2, 93 },
    { 0xf2d0abf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c0abf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2640abfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c30ac00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ac04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00ac09, 48 },
    { 0xf640ac0a, 98 },
    { 0xfcd0ac09, 95 },
    { 0x2d90ac0c, 77 },
    { 0xe9a0ac0d, 87 },
    { 0x4e40ac0e, 90 },
    { 0x8b50ac0f, 93 },
    { 0x1110ac0e, 84 },
    { 0, 0 },
    { 0x49c0ac12, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa040ac15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60ac1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360ac1d, 46 },
    { 0x8e0ac1e, 55 },
    { 0x6a00ac1f, 23 },
    { 0, 0 },
    { 0x860ac21, 65 },
    { 0x3d50ac22, 61 },
    { 0xd350ac23, 46 },
    { 0x40d0ac24, 98 },
    { 0x5c60ac23, 23 },
    { 0x31e0ac24, 33 },
    { 0x1e80ac22, 94 },
    { 0x1760ac28, 98 },
    { 0x8010ac27, 95 },
    { 0xd760ac2a, 33 },
    { 0xd770ac2a, 95 },
    { 0x59f0ac2b, 67 },
    { 0x2830ac23, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c0ac33, 99 },
    { 0x94e0ac34, 88 },
    { 0x26f0ac34, 61 },
    { 0, 0 },
    { 0x5ba0ac37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50ac3b, 76 },
    { 0, 0 },
    { 0x86d0ac3d, 46 },
    { 0x8a60ac3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b40ac45, 98 },
    { 0, 0 },
    { 0x3230ac47, 33 },
    { 0x3b60ac48, 76 },
    { 0, 0 },
    { 0x4b0ac4a, 99 },
    { 0x6950ac4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20ac4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb30ac51, 98 },
    { 0x3190ac52, 90 },
    { 0, 0 },
    { 0x8370ac54, 76 },
    { 0, 0 },
    { 0xe730ac56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab0ac61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0ac6a, 84 },
    { 0xc880ac6b, 95 },
    { 0, 0 },
    { 0x2f10ac6d, 90 },
    { 0, 0 },
    { 0x2320ac6f, 67 },
    { 0x2130ac70, 99 },
    { 0xe5e0ac71, 95 },
    { 0xd520ac71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6550ac75, 93 },
    { 0xd970ac76, 95 },
    { 0x7c70ac76, 65 },
    { 0xef40ac78, 90 },
    { 0x5bd0ac79, 76 },
    { 0x4dd0ac7a, 33 },
    { 0x60b0ac7b, 23 },
    { 0x5220ac7c, 92 },
    { 0x2d50ac7d, 52 },
    { 0x7f40ac7d, 96 },
    { 0, 0 },
    { 0xee30ac80, 55 },
    { 0xefb0ac80, 67 },
    { 0, 0 },
    { 0x4330ac83, 98 },
    { 0, 0 },
    { 0xa080ac85, 61 },
    { 0x5470ac85, 46 },
    { 0, 0 },
    { 0x31b0ac88, 61 },
    { 0xd690ac88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff50ac8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c30ac95, 76 },
    { 0xcfc0ac96, 95 },
    { 0xe710ac96, 66 },
    { 0xd7e0ac98, 33 },
    { 0xbbf0ac99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0ac9d, 23 },
    { 0x8700ac9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee0aca2, 34 },
    { 0xfff0aca2, 19 },
    { 0x3e0aca4, 75 },
    { 0x1f60aca5, 34 },
    { 0x52b0aca4, 95 },
    { 0x85b0aca7, 46 },
    { 0, 0 },
    { 0xf220aca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60acad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0acb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x830acb8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0acbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8590acbf, 61 },
    { 0, 0 },
    { 0xf0acc1, 46 },
    { 0x2730acc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de0acc5, 94 },
    { 0xd8b0acc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50accd, 93 },
    { 0x7770acce, 25 },
    { 0xc720accd, 90 },
    { 0xf6c0accd, 36 },
    { 0xa8a0acd1, 91 },
    { 0, 0 },
    { 0x5930acd3, 55 },
    { 0, 0 },
    { 0xaa20acd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00ace0, 95 },
    { 0xe810ace0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7870ace9, 61 },
    { 0xea00acea, 33 },
    { 0x3e70aceb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de0acf0, 95 },
    { 0xd010acf0, 91 },
    { 0xc590acf0, 46 },
    { 0x57f0acf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb30acf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490acfe, 55 },
    { 0xde10acfe, 98 },
    { 0x3860ad00, 67 },
    { 0, 0 },
    { 0xa7b0ad02, 52 },
    { 0x1290ad03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960ad07, 67 },
    { 0x6f0ad08, 90 },
    { 0xbda0ad09, 23 },
    { 0x4460ad08, 90 },
    { 0, 0 },
    { 0xba80ad0c, 95 },
    { 0xb250ad0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0ad10, 43 },
    { 0, 0 },
    { 0x8020ad12, 98 },
    { 0x6790ad13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490ad16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0ad1c, 43 },
    { 0, 0 },
    { 0xfd60ad1e, 61 },
    { 0x1100ad1f, 98 },
    { 0, 0 },
    { 0x3cd0ad21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e80ad26, 95 },
    { 0, 0 },
    { 0xae40ad28, 90 },
    { 0xe230ad29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0ad2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30ad30, 23 },
    { 0xc520ad30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ad35, 33 },
    { 0x60a0ad36, 67 },
    { 0x27a0ad37, 66 },
    { 0x6d10ad38, 61 },
    { 0, 0 },
    { 0xec0ad3a, 23 },
    { 0x4350ad3a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd950ad42, 85 },
    { 0, 0 },
    { 0xfd00ad44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a90ad49, 23 },
    { 0x2f70ad4a, 95 },
    { 0x9e20ad4b, 61 },
    { 0, 0 },
    { 0x3ea0ad4d, 95 },
    { 0x1c20ad4e, 95 },
    { 0x6ed0ad4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5330ad52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ad55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470ad5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba70ad60, 76 },
    { 0xfbd0ad61, 90 },
    { 0x8c80ad62, 17 },
    { 0x3ea0ad63, 90 },
    { 0x92e0ad64, 98 },
    { 0x3b80ad65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d0ad68, 67 },
    { 0x3260ad68, 90 },
    { 0xb6c0ad68, 66 },
    { 0x53e0ad68, 99 },
    { 0, 0 },
    { 0x91a0ad6d, 98 },
    { 0, 0 },
    { 0x7b30ad6f, 87 },
    { 0xfdb0ad6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710ad79, 95 },
    { 0x2b50ad79, 78 },
    { 0x8470ad7b, 76 },
    { 0xf020ad7a, 34 },
    { 0x1130ad7d, 93 },
    { 0x85b0ad7a, 33 },
    { 0xc8d0ad7f, 91 },
    { 0, 0 },
    { 0x7d50ad81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c0ad88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6370ad8d, 87 },
    { 0x72a0ad8e, 76 },
    { 0x83e0ad8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60ad99, 98 },
    { 0xbfe0ad99, 61 },
    { 0x62b0ad99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1650ad9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce90ada2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220ada7, 67 },
    { 0xe7e0ada8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa140adab, 99 },
    { 0x8fc0adac, 23 },
    { 0xcc00adad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60adb0, 81 },
    { 0x53a0adb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c0adb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa770adb9, 99 },
    { 0, 0 },
    { 0xeb90adbb, 33 },
    { 0xe090adbc, 98 },
    { 0x320adbb, 61 },
    { 0x2600adbe, 99 },
    { 0x2b60adbe, 90 },
    { 0x8bd0adc0, 66 },
    { 0xf8f0adbe, 33 },
    { 0, 0 },
    { 0x9290adc3, 90 },
    { 0x7f20adc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1650adc7, 90 },
    { 0, 0 },
    { 0x9bf0adc9, 88 },
    { 0x4510adca, 91 },
    { 0xa600adca, 33 },
    { 0x6990adcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c80adcf, 98 },
    { 0xed0add0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2100add5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f0addb, 85 },
    { 0, 0 },
    { 0xf530addd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9300ade3, 100 },
    { 0x3710ade4, 81 },
    { 0, 0 },
    { 0xe8c0ade6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe150ade9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b90adee, 23 },
    { 0xfc90adef, 46 },
    { 0x6c0adf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae0adf7, 78 },
    { 0x8b20adf7, 90 },
    { 0xf120adf9, 33 },
    { 0x6850adfa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee50ae01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f0ae05, 23 },
    { 0x2680ae06, 33 },
    { 0, 0 },
    { 0x17e0ae08, 90 },
    { 0xe010ae09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef0ae0f, 68 },
    { 0, 0 },
    { 0xaee0ae11, 76 },
    { 0, 0 },
    { 0x6680ae13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a10ae17, 87 },
    { 0xc560ae18, 33 },
    { 0xc810ae18, 75 },
    { 0, 0 },
    { 0x2a10ae1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20ae20, 100 },
    { 0, 0 },
    { 0x5c00ae22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0ae25, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc110ae28, 52 },
    { 0xa3c0ae28, 76 },
    { 0x8b0ae2a, 23 },
    { 0, 0 },
    { 0x55b0ae2c, 55 },
    { 0x2640ae2d, 76 },
    { 0xa6d0ae2e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6130ae33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc350ae39, 61 },
    { 0, 0 },
    { 0x46f0ae3b, 33 },
    { 0x63c0ae3c, 98 },
    { 0x30f0ae3d, 78 },
    { 0x70d0ae3c, 91 },
    { 0x90ae3c, 23 },
    { 0xce60ae40, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc130ae45, 93 },
    { 0xd7c0ae46, 33 },
    { 0x6070ae47, 93 },
    { 0, 0 },
    { 0x7190ae49, 43 },
    { 0x14c0ae4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840ae52, 33 },
    { 0, 0 },
    { 0x1fb0ae54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce0ae57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ae5a, 91 },
    { 0, 0 },
    { 0x1680ae5c, 98 },
    { 0x1030ae5d, 55 },
    { 0xcdc0ae5e, 99 },
    { 0xd430ae5e, 23 },
    { 0, 0 },
    { 0xcff0ae61, 76 },
    { 0x27f0ae61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e50ae66, 55 },
    { 0x9400ae66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e40ae6a, 90 },
    { 0x4030ae6b, 34 },
    { 0xcbd0ae6a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0ae71, 95 },
    { 0x6b70ae72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa0ae7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d80ae81, 76 },
    { 0x6f70ae82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1780ae87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6430ae8c, 67 },
    { 0, 0 },
    { 0xc460ae8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc400ae92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790ae95, 98 },
    { 0x7d50ae95, 43 },
    { 0x37f0ae96, 43 },
    { 0, 0 },
    { 0x1d10ae99, 61 },
    { 0x4940ae99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0aea3, 61 },
    { 0xb530aea4, 33 },
    { 0, 0 },
    { 0x5ef0aea6, 78 },
    { 0x660aea7, 90 },
    { 0x39b0aea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10aeb1, 61 },
    { 0, 0 },
    { 0xd810aeb3, 33 },
    { 0x33c0aeb4, 76 },
    { 0x2130aeb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d0aeb8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1180aebf, 55 },
    { 0, 0 },
    { 0x9220aec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf80aec7, 61 },
    { 0x68a0aec7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60aecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd60aed2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2230aed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90aedc, 52 },
    { 0xea70aedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2770aee0, 76 },
    { 0x9110aee0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6280aee7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0aeef, 17 },
    { 0xaee0aeef, 85 },
    { 0, 0 },
    { 0xbaa0aef2, 81 },
    { 0x2850aef3, 78 },
    { 0xb6a0aef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4020aef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d0aefc, 90 },
    { 0, 0 },
    { 0x7dc0aefe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a0af0b, 91 },
    { 0x360af0c, 55 },
    { 0, 0 },
    { 0xb0c0af0e, 23 },
    { 0x8de0af0e, 66 },
    { 0xaeb0af10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540af14, 99 },
    { 0, 0 },
    { 0xf680af16, 55 },
    { 0, 0 },
    { 0x5fa0af18, 76 },
    { 0, 0 },
    { 0xc3f0af1a, 68 },
    { 0, 0 },
    { 0xf3c0af1c, 34 },
    { 0xb580af1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0af22, 33 },
    { 0, 0 },
    { 0x8980af24, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd0af2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb550af2f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2610af32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920af40, 66 },
    { 0xb980af40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8820af45, 43 },
    { 0xbc40af46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0af49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180af4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf70af53, 23 },
    { 0xb830af53, 98 },
    { 0x8820af54, 64 },
    { 0x71c0af56, 34 },
    { 0, 0 },
    { 0x8250af58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0af5b, 90 },
    { 0, 0 },
    { 0x8370af5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe0af62, 61 },
    { 0, 0 },
    { 0x7a0af64, 33 },
    { 0x5040af65, 23 },
    { 0x6530af65, 61 },
    { 0x9fa0af67, 55 },
    { 0xb430af68, 43 },
    { 0x4eb0af69, 23 },
    { 0xff0af69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf060af76, 80 },
    { 0xa3e0af76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5530af7b, 55 },
    { 0x4b20af7b, 40 },
    { 0, 0 },
    { 0xabc0af7e, 98 },
    { 0xa460af7f, 61 },
    { 0x1e70af80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60af83, 90 },
    { 0x7470af84, 96 },
    { 0x1fd0af85, 95 },
    { 0x6500af85, 87 },
    { 0x83e0af87, 68 },
    { 0, 0 },
    { 0xa940af89, 33 },
    { 0, 0 },
    { 0x67b0af8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c00af8e, 43 },
    { 0x9680af8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2220af94, 100 },
    { 0xd480af94, 98 },
    { 0xd830af94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390af9d, 78 },
    { 0x4210af9e, 55 },
    { 0x9130af9e, 76 },
    { 0x40e0afa0, 61 },
    { 0x3590afa1, 43 },
    { 0x47f0afa2, 98 },
    { 0x9a40afa0, 34 },
    { 0x8970afa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0afa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0afaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0afb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6750afbd, 83 },
    { 0xf0d0afbe, 33 },
    { 0xd4c0afbf, 61 },
    { 0, 0 },
    { 0xf5f0afc1, 36 },
    { 0, 0 },
    { 0x3360afc3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100afc6, 33 },
    { 0, 0 },
    { 0x4ea0afc8, 95 },
    { 0xd640afc8, 61 },
    { 0xc340afca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf90afd0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0afd4, 61 },
    { 0xe4f0afd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba0afdc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9250afe5, 95 },
    { 0, 0 },
    { 0x65c0afe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d0afeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a0afef, 95 },
    { 0xc80afef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2220affd, 98 },
    { 0x8a70affe, 76 },
    { 0x28a0afff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3920b005, 23 },
    { 0x7db0b006, 98 },
    { 0x64d0b006, 23 },
    { 0, 0 },
    { 0x52a0b009, 90 },
    { 0, 0 },
    { 0xf6e0b00b, 38 },
    { 0xed00b00c, 76 },
    { 0xee50b00d, 43 },
    { 0, 0 },
    { 0xc020b00f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80b015, 98 },
    { 0xfbc0b016, 46 },
    { 0x5180b016, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c50b01a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030b01d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3000b023, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030b02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f0b031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0b035, 87 },
    { 0, 0 },
    { 0x5890b037, 68 },
    { 0xc9a0b038, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc0b042, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4170b045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010b048, 55 },
    { 0, 0 },
    { 0xecb0b04a, 98 },
    { 0, 0 },
    { 0xa50b04c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a50b050, 23 },
    { 0xa580b051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680b054, 61 },
    { 0xb410b055, 98 },
    { 0x6400b056, 98 },
    { 0xd7e0b056, 61 },
    { 0xed00b054, 33 },
    { 0x2d80b059, 69 },
    { 0, 0 },
    { 0x9150b05b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410b060, 93 },
    { 0x4030b061, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb170b065, 77 },
    { 0xc620b066, 61 },
    { 0x9850b066, 99 },
    { 0, 0 },
    { 0xa150b069, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa50b072, 95 },
    { 0, 0 },
    { 0x2120b074, 95 },
    { 0, 0 },
    { 0x3b10b076, 78 },
    { 0xc900b077, 90 },
    { 0, 0 },
    { 0x8290b079, 38 },
    { 0xbd90b07a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6690b07e, 76 },
    { 0xbcb0b07e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5210b082, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e0b086, 90 },
    { 0x7f40b087, 55 },
    { 0x28e0b088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb0b08b, 43 },
    { 0, 0 },
    { 0xd600b08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800b091, 76 },
    { 0xb9d0b091, 90 },
    { 0xd760b091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0b096, 61 },
    { 0xfd50b097, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0b09c, 23 },
    { 0x2d20b09d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa720b0a5, 17 },
    { 0xac10b0a6, 93 },
    { 0x44a0b0a7, 43 },
    { 0x2c40b0a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420b0ac, 90 },
    { 0x23b0b0ad, 90 },
    { 0x9430b0ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be0b0b1, 33 },
    { 0x6f0b0b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede0b0b6, 61 },
    { 0x22b0b0b7, 95 },
    { 0x5af0b0b7, 43 },
    { 0xa470b0b6, 46 },
    { 0x8b80b0b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb050b0be, 61 },
    { 0, 0 },
    { 0xfea0b0c0, 61 },
    { 0xcd00b0c1, 33 },
    { 0xc800b0c2, 33 },
    { 0x1c10b0c3, 75 },
    { 0x4d80b0c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60b0ca, 55 },
    { 0, 0 },
    { 0x1a60b0cc, 100 },
    { 0x6c10b0cc, 46 },
    { 0xfeb0b0ce, 55 },
    { 0xc3d0b0cf, 23 },
    { 0x7d00b0d0, 91 },
    { 0xbae0b0cf, 95 },
    { 0xd950b0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0b0d5, 55 },
    { 0x3fd0b0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4420b0d9, 76 },
    { 0x5200b0da, 78 },
    { 0xde80b0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b0df, 76 },
    { 0xd4d0b0df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d0b0e3, 90 },
    { 0xd0b0b0e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7300b0ea, 61 },
    { 0x8910b0eb, 90 },
    { 0xc70b0ec, 23 },
    { 0, 0 },
    { 0xcce0b0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf970b0f3, 55 },
    { 0, 0 },
    { 0x19f0b0f5, 87 },
    { 0x6a10b0f6, 78 },
    { 0x49c0b0f7, 38 },
    { 0x6640b0f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac80b0fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b0b102, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050b107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a90b10c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde60b10f, 99 },
    { 0x2bf0b10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020b115, 55 },
    { 0xc050b116, 61 },
    { 0x270b116, 78 },
    { 0x7350b118, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50b11c, 99 },
    { 0, 0 },
    { 0x9f10b11e, 95 },
    { 0xdf10b11f, 78 },
    { 0xd280b120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9900b126, 23 },
    { 0x4220b126, 33 },
    { 0x8f10b128, 66 },
    { 0x6c80b127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9600b12e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb0b131, 43 },
    { 0x9670b131, 98 },
    { 0xc700b133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0b13b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0b13f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff0b143, 76 },
    { 0x4590b144, 98 },
    { 0xbc80b143, 93 },
    { 0x1c10b146, 85 },
    { 0x7040b144, 87 },
    { 0x59b0b148, 23 },
    { 0x4440b143, 23 },
    { 0, 0 },
    { 0x3fc0b14b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b50b14e, 90 },
    { 0xda00b14f, 93 },
    { 0, 0 },
    { 0x7ae0b151, 87 },
    { 0x3900b152, 94 },
    { 0x9c0b152, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70b157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700b15d, 95 },
    { 0x310b15e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef0b164, 95 },
    { 0xd670b165, 95 },
    { 0xb590b164, 90 },
    { 0x5290b167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0b16a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf400b16d, 98 },
    { 0xbd0b16d, 33 },
    { 0xed10b16d, 93 },
    { 0xf3e0b16f, 33 },
    { 0, 0 },
    { 0x6630b172, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100b17a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b0b182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0b189, 43 },
    { 0, 0 },
    { 0xd760b18b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e0b18f, 75 },
    { 0, 0 },
    { 0x1430b191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c20b194, 33 },
    { 0, 0 },
    { 0xfed0b196, 43 },
    { 0x3010b197, 98 },
    { 0xede0b198, 76 },
    { 0x2860b199, 90 },
    { 0x520b198, 90 },
    { 0xddd0b197, 33 },
    { 0x1c50b19c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8240b19f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540b1a2, 95 },
    { 0x4be0b1a3, 34 },
    { 0x7a90b1a4, 98 },
    { 0xeb0b1a4, 23 },
    { 0x8850b1a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa60b1a9, 90 },
    { 0, 0 },
    { 0x5630b1ab, 23 },
    { 0xdc0b1ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe700b1b3, 90 },
    { 0, 0 },
    { 0x2c00b1b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4240b1bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0b1c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3930b1c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6800b1d1, 95 },
    { 0xbcd0b1d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce0b1d5, 23 },
    { 0xe140b1d6, 95 },
    { 0x2ba0b1d6, 23 },
    { 0x2f70b1d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0b1df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650b1e4, 55 },
    { 0x8190b1e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20b1eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd600b1ee, 23 },
    { 0, 0 },
    { 0xf760b1f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b1f6, 55 },
    { 0xbe60b1f7, 69 },
    { 0x7d40b1f6, 90 },
    { 0x26a0b1f6, 61 },
    { 0x9730b1f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5070b1ff, 65 },
    { 0x8450b1ff, 98 },
    { 0xf9d0b1ff, 46 },
    { 0x84e0b202, 98 },
    { 0xb1ff, 84 },
    { 0, 0 },
    { 0x6b80b205, 65 },
    { 0xf3f0b206, 90 },
    { 0x1720b207, 61 },
    { 0x66a0b207, 95 },
    { 0x1470b209, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba0b20d, 43 },
    { 0x84c0b20e, 78 },
    { 0x5730b20e, 95 },
    { 0xf660b210, 46 },
    { 0x4fe0b211, 84 },
    { 0, 0 },
    { 0xf5a0b213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1350b217, 34 },
    { 0x2720b218, 23 },
    { 0xf7c0b218, 61 },
    { 0x35c0b218, 34 },
    { 0xbb00b21b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb720b21f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480b22c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf40b22f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240b234, 23 },
    { 0x78a0b234, 95 },
    { 0x2060b236, 43 },
    { 0, 0 },
    { 0x58b0b238, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe70b23b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a0b23e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc90b243, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd750b24b, 41 },
    { 0, 0 },
    { 0xe190b24d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac60b250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930b255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8180b25a, 33 },
    { 0x52d0b25b, 55 },
    { 0, 0 },
    { 0xf4a0b25d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810b260, 90 },
    { 0xca90b261, 33 },
    { 0x4270b261, 95 },
    { 0x9920b263, 69 },
    { 0, 0 },
    { 0x8a80b265, 90 },
    { 0, 0 },
    { 0x7710b267, 52 },
    { 0, 0 },
    { 0x2d30b269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc0b26c, 90 },
    { 0x9d30b26d, 98 },
    { 0, 0 },
    { 0x7f90b26f, 33 },
    { 0x47d0b270, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb550b274, 76 },
    { 0xbc30b275, 61 },
    { 0x1110b274, 90 },
    { 0, 0 },
    { 0x97d0b278, 76 },
    { 0x2430b279, 98 },
    { 0, 0 },
    { 0x5580b27b, 23 },
    { 0x36c0b27b, 95 },
    { 0xd800b27d, 95 },
    { 0xa320b27e, 98 },
    { 0x6590b27f, 93 },
    { 0xf380b27f, 76 },
    { 0, 0 },
    { 0x3db0b282, 76 },
    { 0, 0 },
    { 0xcfc0b284, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30b28b, 66 },
    { 0xde60b28c, 77 },
    { 0x6440b28c, 34 },
    { 0x5250b28e, 90 },
    { 0x30c0b28f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440b292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb360b295, 36 },
    { 0xd1b0b296, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0b2a1, 90 },
    { 0x64d0b2a2, 90 },
    { 0xd280b2a3, 95 },
    { 0xfff0b2a4, 76 },
    { 0, 0 },
    { 0xd1c0b2a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb10b2ad, 55 },
    { 0, 0 },
    { 0x2ad0b2af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c10b2b4, 91 },
    { 0x12b0b2b5, 55 },
    { 0xe8e0b2b5, 61 },
    { 0, 0 },
    { 0x99e0b2b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0b2be, 66 },
    { 0xd460b2be, 66 },
    { 0, 0 },
    { 0x2b60b2c1, 90 },
    { 0, 0 },
    { 0xd320b2c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb160b2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe780b2cb, 52 },
    { 0x8d70b2cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd0b2d2, 98 },
    { 0xda70b2d2, 67 },
    { 0xea20b2d4, 90 },
    { 0x7f50b2d5, 95 },
    { 0x2b50b2d6, 33 },
    { 0xe610b2d7, 90 },
    { 0xdaf0b2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0b2db, 98 },
    { 0xb040b2dc, 61 },
    { 0x8340b2db, 95 },
    { 0x89c0b2de, 61 },
    { 0, 0 },
    { 0xd0c0b2e0, 66 },
    { 0xe030b2e0, 98 },
    { 0xff60b2e2, 33 },
    { 0, 0 },
    { 0xc470b2e4, 75 },
    { 0, 0 },
    { 0x36d0b2e6, 91 },
    { 0, 0 },
    { 0xf970b2e8, 34 },
    { 0, 0 },
    { 0x35f0b2ea, 67 },
    { 0, 0 },
    { 0xd510b2ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff10b2f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410b2f6, 34 },
    { 0x2ae0b2f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f0b2fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a20b2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890b304, 76 },
    { 0, 0 },
    { 0xbb70b306, 90 },
    { 0, 0 },
    { 0xdf10b308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30b30c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0b313, 90 },
    { 0xdd0b313, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb0b31e, 98 },
    { 0xac10b31f, 78 },
    { 0xfc30b31f, 33 },
    { 0x4b90b321, 17 },
    { 0x240b321, 34 },
    { 0x7f20b320, 33 },
    { 0xab80b31f, 44 },
    { 0x6d00b31e, 66 },
    { 0x2930b326, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb610b331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50b33a, 23 },
    { 0, 0 },
    { 0x6d0b33c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0b33f, 90 },
    { 0, 0 },
    { 0xe1b0b341, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50b344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450b348, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf50b34d, 78 },
    { 0xbe30b34d, 90 },
    { 0xb950b34f, 46 },
    { 0x2b90b34e, 46 },
    { 0xa0c0b351, 93 },
    { 0x6fd0b352, 99 },
    { 0xa330b352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd0b35b, 34 },
    { 0x3800b35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00b364, 94 },
    { 0x54d0b365, 91 },
    { 0xd730b365, 98 },
    { 0xce30b367, 95 },
    { 0x1ec0b368, 43 },
    { 0, 0 },
    { 0xbcd0b36a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a00b36e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1500b371, 40 },
    { 0, 0 },
    { 0x2d50b373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970b377, 61 },
    { 0x5530b378, 98 },
    { 0x6510b379, 33 },
    { 0xe920b37a, 46 },
    { 0xb2d0b379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c50b37f, 23 },
    { 0x2e20b380, 90 },
    { 0x5c30b37f, 33 },
    { 0xf0b382, 61 },
    { 0, 0 },
    { 0x9100b384, 33 },
    { 0, 0 },
    { 0x3c40b386, 75 },
    { 0x6990b387, 95 },
    { 0x1310b388, 33 },
    { 0xf040b388, 92 },
    { 0xe250b389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a0b38d, 90 },
    { 0xde30b38d, 81 },
    { 0x6a00b38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70b397, 33 },
    { 0x5020b398, 98 },
    { 0x87f0b399, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x270b39c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd940b3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f0b3a4, 17 },
    { 0x3a80b3a4, 33 },
    { 0, 0 },
    { 0x6230b3a7, 66 },
    { 0xdc20b3a8, 90 },
    { 0x4430b3a9, 34 },
    { 0x3970b3aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8950b3ad, 85 },
    { 0xf940b3ae, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d0b3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6570b3b5, 78 },
    { 0x3e30b3b5, 33 },
    { 0, 0 },
    { 0x2d10b3b8, 100 },
    { 0xbdd0b3b9, 33 },
    { 0, 0 },
    { 0x7000b3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170b3c0, 90 },
    { 0x3af0b3c1, 84 },
    { 0, 0 },
    { 0x2c60b3c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e50b3c9, 90 },
    { 0x8e90b3ca, 23 },
    { 0xcf90b3c9, 92 },
    { 0, 0 },
    { 0x370b3cd, 85 },
    { 0x9ac0b3ce, 85 },
    { 0xb330b3ce, 98 },
    { 0x1f10b3d0, 34 },
    { 0x37b0b3cf, 23 },
    { 0, 0 },
    { 0xfec0b3d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0b3d8, 76 },
    { 0, 0 },
    { 0x9a80b3da, 77 },
    { 0xb4d0b3db, 98 },
    { 0, 0 },
    { 0x1ed0b3dd, 85 },
    { 0xe0d0b3de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e80b3e1, 66 },
    { 0x3c40b3e2, 98 },
    { 0x3be0b3e2, 90 },
    { 0x9550b3e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3440b3e7, 95 },
    { 0x6700b3e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1300b3ed, 43 },
    { 0, 0 },
    { 0xc3f0b3ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde20b3f2, 36 },
    { 0xbc40b3f3, 80 },
    { 0x56b0b3f4, 61 },
    { 0x3e00b3f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe560b3f8, 76 },
    { 0, 0 },
    { 0xd0a0b3fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba0b3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570b403, 61 },
    { 0x14e0b404, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b70b40c, 100 },
    { 0, 0 },
    { 0x88d0b40e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60b411, 91 },
    { 0xbef0b411, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0b416, 33 },
    { 0xa3d0b417, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa0b41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410b422, 23 },
    { 0xc80b423, 76 },
    { 0xc210b424, 76 },
    { 0xbb00b425, 58 },
    { 0x5ed0b426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110b42a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b0b42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70b431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e0b435, 90 },
    { 0, 0 },
    { 0x99b0b437, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e0b442, 43 },
    { 0x970b443, 33 },
    { 0x7ab0b444, 23 },
    { 0x4540b444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f90b449, 51 },
    { 0, 0 },
    { 0x5e50b44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3770b44f, 76 },
    { 0x3b60b450, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a0b455, 99 },
    { 0x71a0b455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe30b45b, 90 },
    { 0, 0 },
    { 0x6290b45d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d00b464, 52 },
    { 0x5820b465, 98 },
    { 0x9040b465, 61 },
    { 0xd5d0b465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d0b46a, 23 },
    { 0, 0 },
    { 0xfb0b46c, 23 },
    { 0xf180b46d, 98 },
    { 0, 0 },
    { 0x8390b46f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8120b472, 34 },
    { 0x4320b473, 55 },
    { 0xe730b474, 99 },
    { 0x3ee0b472, 46 },
    { 0, 0 },
    { 0x25f0b477, 90 },
    { 0, 0 },
    { 0xdf20b479, 55 },
    { 0x2f70b47a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50b47d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c0b483, 66 },
    { 0, 0 },
    { 0x5bb0b485, 55 },
    { 0xd90b485, 23 },
    { 0x7110b486, 52 },
    { 0x9d20b488, 95 },
    { 0xa400b487, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b48d, 33 },
    { 0xcb50b48d, 43 },
    { 0x3d40b48f, 75 },
    { 0x9de0b48e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b0b497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f0b49b, 58 },
    { 0, 0 },
    { 0x8e60b49d, 33 },
    { 0xaf30b49d, 53 },
    { 0xbd80b49d, 98 },
    { 0x12c0b4a0, 96 },
    { 0xe810b49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c0b4ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c0b4b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7400b4b6, 61 },
    { 0x6800b4b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe960b4ba, 23 },
    { 0xb740b4bb, 33 },
    { 0x81f0b4bc, 95 },
    { 0x4a0b4bc, 33 },
    { 0, 0 },
    { 0x86c0b4bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9050b4cb, 23 },
    { 0xb8c0b4cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0b4d6, 33 },
    { 0x1600b4d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c80b4dd, 33 },
    { 0, 0 },
    { 0x8da0b4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d0b4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0b4ea, 95 },
    { 0x8440b4eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c0b4ee, 55 },
    { 0xb360b4ee, 43 },
    { 0xa8f0b4f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7160b4f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580b4f9, 23 },
    { 0x6630b4f9, 90 },
    { 0x1180b4fa, 93 },
    { 0x7480b4fa, 23 },
    { 0xb170b4fd, 76 },
    { 0x9150b4fe, 33 },
    { 0x1300b4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810b509, 76 },
    { 0, 0 },
    { 0x200b50b, 90 },
    { 0x1c90b50c, 98 },
    { 0, 0 },
    { 0xd610b50e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b0b516, 23 },
    { 0xede0b517, 66 },
    { 0x7ab0b516, 98 },
    { 0, 0 },
    { 0xb350b51a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f0b520, 17 },
    { 0, 0 },
    { 0x10d0b522, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990b525, 61 },
    { 0x6550b526, 85 },
    { 0xc890b527, 43 },
    { 0xc2e0b528, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b80b52f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8920b532, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f0b537, 90 },
    { 0xc690b537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a90b53b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7260b545, 61 },
    { 0xb380b546, 33 },
    { 0x9420b547, 76 },
    { 0x83e0b548, 90 },
    { 0, 0 },
    { 0xdc20b54a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580b54e, 61 },
    { 0x2bc0b54e, 98 },
    { 0, 0 },
    { 0x2860b551, 95 },
    { 0, 0 },
    { 0xdef0b553, 46 },
    { 0xcc10b554, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8850b55b, 96 },
    { 0x2960b55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f0b55f, 96 },
    { 0xac00b560, 98 },
    { 0x5fd0b561, 23 },
    { 0x1ac0b562, 91 },
    { 0x7500b562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae90b568, 64 },
    { 0xf920b569, 80 },
    { 0x9f50b569, 90 },
    { 0xd120b56b, 76 },
    { 0x9be0b56b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0b570, 65 },
    { 0xd3c0b571, 84 },
    { 0xc330b570, 87 },
    { 0xd980b573, 98 },
    { 0, 0 },
    { 0xb180b575, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540b579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990b57d, 75 },
    { 0x13e0b57d, 34 },
    { 0, 0 },
    { 0xc0f0b580, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e90b587, 90 },
    { 0x8190b588, 98 },
    { 0x33f0b588, 98 },
    { 0xd1e0b58a, 46 },
    { 0, 0 },
    { 0xd490b58c, 55 },
    { 0x96a0b58c, 42 },
    { 0x66f0b58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9260b594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c0b5a0, 84 },
    { 0, 0 },
    { 0xb9f0b5a2, 90 },
    { 0, 0 },
    { 0x2e70b5a4, 46 },
    { 0, 0 },
    { 0x2300b5a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290b5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4950b5ac, 98 },
    { 0xa570b5ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef0b5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe490b5b7, 90 },
    { 0x18f0b5b8, 23 },
    { 0x6a10b5b9, 98 },
    { 0x7940b5b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf0b5be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc440b5c2, 99 },
    { 0, 0 },
    { 0xab0b5c4, 43 },
    { 0x7580b5c4, 95 },
    { 0xd820b5c6, 95 },
    { 0xea70b5c7, 96 },
    { 0, 0 },
    { 0x5400b5c9, 99 },
    { 0x43d0b5ca, 66 },
    { 0x8410b5ca, 95 },
    { 0, 0 },
    { 0x6630b5cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd40b5d1, 88 },
    { 0x8670b5d2, 52 },
    { 0x15c0b5d2, 68 },
    { 0x3e50b5d3, 61 },
    { 0x3460b5d1, 34 },
    { 0, 0 },
    { 0xc830b5d7, 46 },
    { 0, 0 },
    { 0xdae0b5d9, 95 },
    { 0, 0 },
    { 0x8cc0b5db, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60b5e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec0b5e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810b5ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x440b5f0, 98 },
    { 0xf20b5f0, 64 },
    { 0x7870b5f0, 55 },
    { 0xe40b5f2, 34 },
    { 0, 0 },
    { 0xaa0b5f5, 33 },
    { 0x53d0b5f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e0b5f9, 98 },
    { 0, 0 },
    { 0xa6f0b5fb, 23 },
    { 0x4710b5fc, 43 },
    { 0xeaf0b5fd, 69 },
    { 0x8d90b5fd, 23 },
    { 0, 0 },
    { 0xfb30b600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570b605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0b60b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe90b610, 90 },
    { 0, 0 },
    { 0x60d0b612, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b617, 23 },
    { 0, 0 },
    { 0x9170b619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc160b61e, 55 },
    { 0xb540b61e, 95 },
    { 0x79b0b61e, 91 },
    { 0x4430b621, 61 },
    { 0xda50b622, 76 },
    { 0x76f0b61f, 34 },
    { 0xe110b624, 98 },
    { 0, 0 },
    { 0xb030b626, 33 },
    { 0xa8b0b627, 78 },
    { 0x47c0b628, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a80b62c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f0b630, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50b633, 22 },
    { 0x4f0b634, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00b637, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6860b63c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c0b641, 61 },
    { 0, 0 },
    { 0x68f0b643, 90 },
    { 0xc410b644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380b64b, 90 },
    { 0x8560b64c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba0b650, 90 },
    { 0xfc30b650, 91 },
    { 0x6390b652, 61 },
    { 0xfdf0b652, 34 },
    { 0x5990b653, 61 },
    { 0x3cf0b655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180b659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc0b65c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710b672, 93 },
    { 0x2c50b673, 98 },
    { 0xd80b674, 33 },
    { 0x4c90b672, 75 },
    { 0x8ca0b676, 61 },
    { 0, 0 },
    { 0x5e20b678, 61 },
    { 0, 0 },
    { 0xe8e0b67a, 76 },
    { 0x28b0b67a, 98 },
    { 0xd7b0b67c, 34 },
    { 0, 0 },
    { 0x4150b67e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c0b682, 43 },
    { 0xba20b683, 98 },
    { 0xbd90b682, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0b687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b690, 33 },
    { 0xe130b690, 93 },
    { 0x6ed0b692, 85 },
    { 0x2980b691, 67 },
    { 0, 0 },
    { 0xf6b0b695, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00b699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0b69d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f60b6a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0b6b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f0b6b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0b6ba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7640b6bf, 98 },
    { 0, 0 },
    { 0x6fc0b6c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da0b6c4, 100 },
    { 0, 0 },
    { 0x7e60b6c6, 55 },
    { 0xf8c0b6c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0b6ca, 61 },
    { 0x6670b6ca, 67 },
    { 0x8840b6cb, 68 },
    { 0, 0 },
    { 0x580b6ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa600b6e0, 95 },
    { 0x1550b6e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d70b6e7, 76 },
    { 0x710b6e8, 23 },
    { 0, 0 },
    { 0x9f30b6ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd620b6ed, 98 },
    { 0, 0 },
    { 0x1070b6ef, 28 },
    { 0x7a10b6ef, 97 },
    { 0x50d0b6f1, 61 },
    { 0x3840b6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a60b6f5, 88 },
    { 0xba30b6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b6fa, 23 },
    { 0x6530b6fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c90b6fe, 90 },
    { 0xbed0b6ff, 87 },
    { 0x5110b6ff, 100 },
    { 0xbc00b701, 95 },
    { 0xfe0b701, 61 },
    { 0x6520b703, 33 },
    { 0xcb60b700, 55 },
    { 0x1640b705, 33 },
    { 0x84e0b706, 90 },
    { 0xe3e0b706, 23 },
    { 0x4810b700, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd470b70b, 77 },
    { 0, 0 },
    { 0x8230b70d, 91 },
    { 0x5290b70e, 33 },
    { 0x27d0b70d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7510b716, 77 },
    { 0x1cc0b717, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8540b71a, 95 },
    { 0, 0 },
    { 0xecb0b71c, 23 },
    { 0xe720b71d, 95 },
    { 0x6500b71e, 81 },
    { 0xc1f0b71f, 95 },
    { 0x9ed0b720, 84 },
    { 0x6650b720, 43 },
    { 0x58c0b721, 78 },
    { 0xd780b722, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0b727, 33 },
    { 0xcca0b728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0b72b, 76 },
    { 0xd930b72b, 34 },
    { 0, 0 },
    { 0x77f0b72e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b70b732, 33 },
    { 0xc180b733, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa370b736, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f20b739, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe0b73d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde40b741, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9310b746, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb60b749, 61 },
    { 0x4b10b74a, 90 },
    { 0x50a0b749, 84 },
    { 0, 0 },
    { 0x11f0b74d, 52 },
    { 0xf840b74d, 33 },
    { 0x79e0b74d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f50b752, 90 },
    { 0, 0 },
    { 0x2640b754, 43 },
    { 0x3590b755, 33 },
    { 0xf5e0b756, 98 },
    { 0x2dd0b757, 52 },
    { 0xb1b0b758, 87 },
    { 0x9d50b756, 66 },
    { 0, 0 },
    { 0xa220b75b, 33 },
    { 0, 0 },
    { 0x1b20b75d, 23 },
    { 0xe810b75d, 78 },
    { 0x27d0b75e, 90 },
    { 0, 0 },
    { 0x1bf0b761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b767, 23 },
    { 0xbe10b768, 86 },
    { 0, 0 },
    { 0xd8b0b76a, 66 },
    { 0x1b60b76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed0b770, 90 },
    { 0xba00b771, 98 },
    { 0x4260b771, 34 },
    { 0x9000b770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b776, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf0b779, 95 },
    { 0x8f60b779, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e0b780, 46 },
    { 0xbeb0b781, 76 },
    { 0xf980b780, 90 },
    { 0, 0 },
    { 0xa490b784, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d0b788, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0b78e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f0b794, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050b797, 55 },
    { 0x910b798, 61 },
    { 0xcd80b798, 91 },
    { 0, 0 },
    { 0xc40b79b, 34 },
    { 0x4c0b79c, 95 },
    { 0, 0 },
    { 0xd5c0b79e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0b7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af0b7a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0b7a9, 61 },
    { 0x5c70b7aa, 91 },
    { 0xb200b7ab, 99 },
    { 0x51e0b7ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500b7b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0b7b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f30b7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20b7be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa240b7c2, 90 },
    { 0x7840b7c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0b7ca, 43 },
    { 0, 0 },
    { 0x4f60b7cc, 76 },
    { 0xa9d0b7cd, 95 },
    { 0xc660b7ce, 90 },
    { 0xa140b7ce, 90 },
    { 0xa700b7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1990b7d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0b7d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9200b7dd, 61 },
    { 0xda50b7dd, 93 },
    { 0, 0 },
    { 0xb2b0b7e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe30b7e3, 90 },
    { 0x9b90b7e4, 66 },
    { 0xcd00b7e3, 46 },
    { 0, 0 },
    { 0x4850b7e7, 51 },
    { 0xf450b7e8, 90 },
    { 0x7f90b7e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930b7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe630b7f7, 58 },
    { 0x2d20b7f8, 93 },
    { 0, 0 },
    { 0x7a20b7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650b7fd, 99 },
    { 0xe610b7fe, 90 },
    { 0, 0 },
    { 0x92a0b800, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20b806, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600b810, 55 },
    { 0xe4c0b811, 55 },
    { 0x5860b810, 33 },
    { 0xdb20b813, 90 },
    { 0xec50b813, 90 },
    { 0x22b0b813, 76 },
    { 0x2410b816, 61 },
    { 0xab70b816, 68 },
    { 0xfb20b818, 76 },
    { 0xfa0b814, 98 },
    { 0, 0 },
    { 0xe9f0b81b, 90 },
    { 0x28e0b81b, 93 },
    { 0x3520b81b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9020b822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8900b827, 98 },
    { 0xf9f0b828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac0b82c, 61 },
    { 0x88f0b82c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf0b831, 93 },
    { 0, 0 },
    { 0x7860b833, 91 },
    { 0x3f10b834, 95 },
    { 0xb420b835, 93 },
    { 0, 0 },
    { 0x9b0b837, 98 },
    { 0, 0 },
    { 0x4fd0b839, 55 },
    { 0xf600b839, 33 },
    { 0, 0 },
    { 0x76f0b83c, 40 },
    { 0x6d40b83d, 23 },
    { 0x1830b83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b0b841, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d0b846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2120b84b, 61 },
    { 0x56d0b84c, 66 },
    { 0, 0 },
    { 0xe520b84e, 100 },
    { 0x71f0b84e, 95 },
    { 0x3670b850, 46 },
    { 0x2560b850, 34 },
    { 0x2140b84e, 98 },
    { 0xb850b853, 55 },
    { 0x3080b853, 95 },
    { 0x2ef0b855, 33 },
    { 0, 0 },
    { 0x5330b857, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2400b85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e0b85e, 93 },
    { 0x27b0b85f, 55 },
    { 0x5060b860, 76 },
    { 0x99f0b861, 61 },
    { 0x9a80b85e, 40 },
    { 0x4b90b863, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb0b869, 80 },
    { 0, 0 },
    { 0xa290b86b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee0b86e, 46 },
    { 0xa1b0b86f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010b876, 33 },
    { 0, 0 },
    { 0x4d50b878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6880b87b, 90 },
    { 0, 0 },
    { 0xc3c0b87d, 67 },
    { 0x22b0b87d, 81 },
    { 0x63e0b87f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8450b884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7510b888, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae00b88d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e70b890, 100 },
    { 0x7770b891, 23 },
    { 0x6670b892, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f30b895, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4530b898, 76 },
    { 0x75d0b899, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b8a0, 84 },
    { 0, 0 },
    { 0x78f0b8a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b8a6, 90 },
    { 0x52e0b8a7, 95 },
    { 0x9940b8a8, 76 },
    { 0, 0 },
    { 0x79f0b8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b80b8b0, 55 },
    { 0, 0 },
    { 0xec80b8b2, 17 },
    { 0x9370b8b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf200b8b7, 84 },
    { 0xd0b8b7, 98 },
    { 0, 0 },
    { 0xa0a0b8ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc900b8bf, 23 },
    { 0xd930b8c0, 98 },
    { 0, 0 },
    { 0x2690b8c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd290b8c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb0b8c9, 33 },
    { 0, 0 },
    { 0x4f20b8cb, 55 },
    { 0, 0 },
    { 0xc650b8cd, 52 },
    { 0, 0 },
    { 0x9d50b8cf, 74 },
    { 0x1220b8cf, 23 },
    { 0x4660b8d1, 55 },
    { 0xa730b8d2, 76 },
    { 0x3b70b8d2, 61 },
    { 0, 0 },
    { 0xa6b0b8d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c0b8d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc090b8db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c0b8df, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6940b8e3, 93 },
    { 0xaa30b8e3, 75 },
    { 0xb640b8e5, 61 },
    { 0xfe00b8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70b8ed, 61 },
    { 0x5220b8ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db0b8f3, 43 },
    { 0, 0 },
    { 0xb5c0b8f5, 15 },
    { 0xd740b8f6, 33 },
    { 0, 0 },
    { 0x22f0b8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0b8fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac0b900, 95 },
    { 0x83e0b901, 90 },
    { 0xd190b902, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad0b906, 23 },
    { 0xab50b907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2720b90d, 33 },
    { 0x90b0b90e, 90 },
    { 0xd600b90e, 60 },
    { 0, 0 },
    { 0xcbd0b911, 95 },
    { 0, 0 },
    { 0x4e00b913, 61 },
    { 0xb420b914, 61 },
    { 0x7fd0b915, 19 },
    { 0x7c90b916, 33 },
    { 0xaff0b917, 33 },
    { 0, 0 },
    { 0x9bc0b919, 95 },
    { 0xe460b91a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9980b91d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8260b920, 68 },
    { 0x48c0b920, 61 },
    { 0x3800b922, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc480b929, 33 },
    { 0x26f0b92a, 23 },
    { 0xce10b92a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90b92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b933, 98 },
    { 0, 0 },
    { 0x7c70b935, 88 },
    { 0x1510b935, 66 },
    { 0x3c70b935, 34 },
    { 0, 0 },
    { 0x21d0b939, 81 },
    { 0x5590b93a, 99 },
    { 0x39b0b93b, 55 },
    { 0x3690b93c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d0b940, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb560b943, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a0b949, 90 },
    { 0, 0 },
    { 0x3800b94b, 34 },
    { 0x1020b94c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d70b94f, 90 },
    { 0x5560b950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0b954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060b959, 52 },
    { 0xe240b959, 88 },
    { 0, 0 },
    { 0x21c0b95c, 33 },
    { 0xda60b95d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50b965, 84 },
    { 0xf600b965, 90 },
    { 0x4580b967, 95 },
    { 0xccd0b968, 90 },
    { 0xf3d0b969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8750b96d, 67 },
    { 0x8eb0b96e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f0b972, 52 },
    { 0xba10b973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610b976, 99 },
    { 0, 0 },
    { 0x29b0b978, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920b97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5960b983, 61 },
    { 0, 0 },
    { 0xc0b985, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc0b993, 64 },
    { 0xe0c0b994, 38 },
    { 0xe120b995, 43 },
    { 0xad70b996, 55 },
    { 0x4390b996, 33 },
    { 0x35d0b996, 33 },
    { 0x6300b997, 85 },
    { 0x8a50b99a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2230b99d, 34 },
    { 0, 0 },
    { 0xa110b99f, 33 },
    { 0x6700b9a0, 90 },
    { 0, 0 },
    { 0x8510b9a2, 90 },
    { 0xd910b9a2, 46 },
    { 0xdd0b9a4, 68 },
    { 0x6510b9a2, 44 },
    { 0xe0d0b9a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a60b9aa, 52 },
    { 0xfa10b9ab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4030b9b1, 77 },
    { 0, 0 },
    { 0x7fc0b9b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780b9b8, 88 },
    { 0, 0 },
    { 0x8d00b9ba, 90 },
    { 0xa90b9ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40b9c1, 77 },
    { 0x6290b9c2, 55 },
    { 0xd0f0b9c3, 61 },
    { 0xc250b9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c0b9c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020b9cb, 33 },
    { 0x7910b9cc, 95 },
    { 0xdef0b9cd, 43 },
    { 0xd250b9ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b00b9d1, 76 },
    { 0x9040b9d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b9d5, 23 },
    { 0xd740b9d6, 95 },
    { 0x8fd0b9d6, 90 },
    { 0x3a00b9d6, 40 },
    { 0, 0 },
    { 0xd8f0b9da, 49 },
    { 0xd860b9db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a0b9df, 23 },
    { 0x55c0b9e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7270b9e9, 61 },
    { 0x3270b9ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0b9f3, 98 },
    { 0x6ed0b9f4, 91 },
    { 0xc870b9f5, 67 },
    { 0, 0 },
    { 0xfe40b9f7, 100 },
    { 0, 0 },
    { 0x3c20b9f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8930b9fd, 90 },
    { 0xdff0b9fe, 43 },
    { 0x9d60b9ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a80ba02, 23 },
    { 0, 0 },
    { 0x67d0ba04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da0ba08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3410ba0b, 90 },
    { 0x6450ba0b, 90 },
    { 0x28f0ba0b, 43 },
    { 0x1180ba0d, 34 },
    { 0x9960ba0f, 23 },
    { 0x3100ba10, 87 },
    { 0, 0 },
    { 0x23d0ba12, 33 },
    { 0x2360ba13, 33 },
    { 0xd850ba14, 61 },
    { 0xc820ba15, 90 },
    { 0x9ea0ba15, 96 },
    { 0, 0 },
    { 0x410ba18, 90 },
    { 0x3dc0ba19, 46 },
    { 0x880ba1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10ba1f, 76 },
    { 0xa890ba1f, 88 },
    { 0x4b20ba21, 4 },
    { 0x69d0ba1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340ba26, 67 },
    { 0x1a30ba27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5870ba2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190ba2e, 98 },
    { 0, 0 },
    { 0x2ec0ba30, 40 },
    { 0x7370ba31, 33 },
    { 0x8fc0ba31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60ba38, 76 },
    { 0, 0 },
    { 0xa530ba3a, 87 },
    { 0x79a0ba3a, 90 },
    { 0x9650ba3c, 23 },
    { 0x2320ba3c, 90 },
    { 0x6fd0ba3a, 62 },
    { 0x7600ba3f, 98 },
    { 0xd380ba3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0ba43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda50ba48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0ba4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a30ba4e, 33 },
    { 0, 0 },
    { 0xc220ba50, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb70ba55, 23 },
    { 0x49c0ba56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ba5b, 98 },
    { 0xb6c0ba5b, 33 },
    { 0xa620ba5d, 98 },
    { 0, 0 },
    { 0x58c0ba5f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9520ba66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae80ba6a, 90 },
    { 0, 0 },
    { 0xf640ba6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0ba6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa0ba77, 90 },
    { 0x91c0ba78, 90 },
    { 0xdaa0ba79, 23 },
    { 0x85d0ba77, 17 },
    { 0xba80ba7b, 33 },
    { 0xc3d0ba7c, 93 },
    { 0x8240ba7d, 90 },
    { 0x7ba0ba7c, 98 },
    { 0xf620ba7f, 34 },
    { 0x9cf0ba80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe800ba84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d0ba87, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550ba8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b0ba91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce20ba96, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0ba99, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca0ba9c, 33 },
    { 0, 0 },
    { 0xc0b0ba9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc390baab, 90 },
    { 0x5c70baac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a0baaf, 90 },
    { 0, 0 },
    { 0x1610bab1, 98 },
    { 0, 0 },
    { 0x5b20bab3, 46 },
    { 0, 0 },
    { 0x8b00bab5, 66 },
    { 0x59c0bab6, 87 },
    { 0xe390bab6, 33 },
    { 0, 0 },
    { 0x41a0bab9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef90bac6, 34 },
    { 0, 0 },
    { 0xd40bac8, 90 },
    { 0xd860bac9, 90 },
    { 0x2590bac9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0bad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00badf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa0bae3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70bae6, 42 },
    { 0x2bb0bae7, 87 },
    { 0, 0 },
    { 0x77e0bae9, 61 },
    { 0xb860bae9, 90 },
    { 0, 0 },
    { 0xd100baec, 61 },
    { 0, 0 },
    { 0x3310baee, 90 },
    { 0xed00baef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0baf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0bb02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e30bb05, 98 },
    { 0xb10bb05, 90 },
    { 0x7990bb07, 95 },
    { 0, 0 },
    { 0x4900bb09, 95 },
    { 0x6a60bb0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e0bb0d, 75 },
    { 0xe8e0bb0e, 93 },
    { 0x3dd0bb0d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c00bb12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc0bb15, 23 },
    { 0xaa80bb16, 90 },
    { 0, 0 },
    { 0x5270bb18, 33 },
    { 0x93b0bb18, 33 },
    { 0x8ed0bb18, 77 },
    { 0x4e30bb1b, 55 },
    { 0xfa70bb1c, 88 },
    { 0, 0 },
    { 0xd040bb1e, 52 },
    { 0x1d90bb1e, 33 },
    { 0x6c70bb1f, 61 },
    { 0x54f0bb21, 23 },
    { 0xefc0bb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0bb26, 100 },
    { 0, 0 },
    { 0x7630bb28, 66 },
    { 0, 0 },
    { 0x2740bb2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df0bb2e, 98 },
    { 0, 0 },
    { 0xeba0bb30, 81 },
    { 0x36f0bb31, 77 },
    { 0x7f50bb31, 81 },
    { 0x3cf0bb30, 98 },
    { 0x34c0bb30, 93 },
    { 0xd520bb34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6110bb39, 95 },
    { 0, 0 },
    { 0x3020bb3b, 61 },
    { 0x2450bb3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10bb3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140bb42, 90 },
    { 0xe40bb43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d0bb46, 33 },
    { 0, 0 },
    { 0xeba0bb48, 95 },
    { 0x2400bb49, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb0bb4e, 23 },
    { 0xb230bb4f, 90 },
    { 0x7e50bb50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20bb56, 98 },
    { 0, 0 },
    { 0x78d0bb58, 88 },
    { 0xd220bb58, 33 },
    { 0xac10bb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca0bb5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00bb6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1970bb72, 61 },
    { 0, 0 },
    { 0xce60bb74, 33 },
    { 0xa5a0bb75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb0bb78, 61 },
    { 0xf4d0bb79, 83 },
    { 0x4ad0bb7a, 61 },
    { 0x59b0bb79, 76 },
    { 0, 0 },
    { 0x3800bb7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840bb80, 100 },
    { 0, 0 },
    { 0xa960bb82, 90 },
    { 0, 0 },
    { 0xa9e0bb84, 33 },
    { 0, 0 },
    { 0xc790bb86, 40 },
    { 0xfd10bb86, 46 },
    { 0x26c0bb88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7580bb8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0bb91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e0bb94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540bb97, 76 },
    { 0x860bb97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e0bba0, 99 },
    { 0x2160bba1, 23 },
    { 0xd8a0bba2, 23 },
    { 0x3590bba2, 95 },
    { 0xb520bba4, 33 },
    { 0, 0 },
    { 0x7d50bba6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d0bba9, 87 },
    { 0x74c0bbaa, 90 },
    { 0x3220bbab, 33 },
    { 0x1760bbac, 95 },
    { 0x9050bbac, 77 },
    { 0xcd10bbae, 78 },
    { 0x60c0bbaf, 95 },
    { 0x3320bbb0, 91 },
    { 0xcce0bbb1, 93 },
    { 0x7c50bbae, 46 },
    { 0x710bbae, 98 },
    { 0x9d70bbb4, 67 },
    { 0x2b20bbae, 98 },
    { 0, 0 },
    { 0x47e0bbb7, 98 },
    { 0x4160bbb8, 55 },
    { 0x6710bbb8, 61 },
    { 0x3400bbb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba0bbc2, 98 },
    { 0xe000bbc3, 93 },
    { 0xb6e0bbc4, 23 },
    { 0xc3c0bbc5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0bbca, 76 },
    { 0xae0bbcb, 61 },
    { 0x8b60bbcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e60bbcf, 90 },
    { 0xc640bbd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0bbd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b0bbde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4170bbe1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bbe5, 17 },
    { 0x8c0bbe5, 52 },
    { 0x95d0bbe7, 23 },
    { 0xac60bbe8, 78 },
    { 0x7b20bbe9, 23 },
    { 0x5300bbe6, 61 },
    { 0xeb20bbeb, 33 },
    { 0, 0 },
    { 0x6680bbed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8740bbf2, 55 },
    { 0, 0 },
    { 0xe860bbf4, 95 },
    { 0, 0 },
    { 0x4d10bbf6, 95 },
    { 0x7750bbf7, 33 },
    { 0x910bbf8, 43 },
    { 0x3f30bbf7, 98 },
    { 0xb2c0bbf9, 90 },
    { 0x5e0bbf7, 90 },
    { 0x39e0bbfc, 91 },
    { 0xfb50bbfd, 23 },
    { 0, 0 },
    { 0x7dc0bbff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80bc02, 34 },
    { 0, 0 },
    { 0xb960bc04, 23 },
    { 0x55c0bc05, 66 },
    { 0, 0 },
    { 0xd2f0bc07, 61 },
    { 0x8890bc08, 33 },
    { 0x9ab0bc08, 98 },
    { 0xd430bc0a, 76 },
    { 0x4180bc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf00bc11, 61 },
    { 0x2a20bc11, 23 },
    { 0x3c10bc13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9660bc18, 95 },
    { 0, 0 },
    { 0x5c10bc1a, 52 },
    { 0, 0 },
    { 0x4fa0bc1c, 43 },
    { 0x4490bc1d, 95 },
    { 0, 0 },
    { 0xf520bc1f, 66 },
    { 0xb710bc1f, 90 },
    { 0x5b80bc21, 55 },
    { 0, 0 },
    { 0x49c0bc23, 95 },
    { 0, 0 },
    { 0x640bc25, 33 },
    { 0xde10bc26, 61 },
    { 0x1f60bc26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bc2d, 23 },
    { 0, 0 },
    { 0xd290bc2f, 90 },
    { 0, 0 },
    { 0xfe50bc31, 95 },
    { 0x2410bc31, 46 },
    { 0, 0 },
    { 0x3620bc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4920bc37, 23 },
    { 0x13e0bc38, 23 },
    { 0x65f0bc39, 85 },
    { 0xe570bc3a, 80 },
    { 0x720bc38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac0bc3e, 33 },
    { 0xcdd0bc3f, 33 },
    { 0x1d0bc3f, 33 },
    { 0xf410bc41, 33 },
    { 0xfcf0bc3e, 91 },
    { 0, 0 },
    { 0x1930bc44, 43 },
    { 0x2270bc45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f60bc51, 98 },
    { 0, 0 },
    { 0xd350bc53, 90 },
    { 0xf20bc54, 33 },
    { 0x9ba0bc55, 33 },
    { 0x2500bc56, 46 },
    { 0x7e70bc54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa980bc5b, 87 },
    { 0xf9e0bc5b, 95 },
    { 0xaca0bc5b, 68 },
    { 0xdfc0bc5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a0bc63, 23 },
    { 0, 0 },
    { 0x6420bc65, 23 },
    { 0, 0 },
    { 0x7f40bc67, 61 },
    { 0x2f10bc68, 33 },
    { 0xd1d0bc69, 17 },
    { 0xd240bc6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc880bc6d, 61 },
    { 0, 0 },
    { 0xa30bc6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1030bc73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50bc7c, 61 },
    { 0xa5a0bc7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80bc83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d0bc86, 98 },
    { 0xcb30bc87, 98 },
    { 0x4c40bc87, 61 },
    { 0xea60bc87, 33 },
    { 0x5a80bc88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc0bc8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1340bc93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5460bc99, 96 },
    { 0, 0 },
    { 0x4590bc9b, 43 },
    { 0x1370bc9c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b90bc9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0bca2, 98 },
    { 0x7830bca3, 23 },
    { 0x6f10bca2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd780bcaa, 61 },
    { 0x2680bcaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880bcae, 61 },
    { 0, 0 },
    { 0x530bcb0, 46 },
    { 0xd570bcb1, 33 },
    { 0xe6b0bcb1, 34 },
    { 0x1500bcb3, 46 },
    { 0x65c0bcb3, 94 },
    { 0, 0 },
    { 0x85a0bcb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb0bcb9, 88 },
    { 0xb630bcba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0bcbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa010bcc3, 17 },
    { 0x9090bcc4, 34 },
    { 0xcc80bcc4, 34 },
    { 0x9450bcc6, 34 },
    { 0x3030bcc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0bcd3, 55 },
    { 0, 0 },
    { 0xe7b0bcd5, 95 },
    { 0xfee0bcd5, 93 },
    { 0x8220bcd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750bcda, 78 },
    { 0xe620bcdb, 33 },
    { 0, 0 },
    { 0x2880bcdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0bce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0bce6, 75 },
    { 0x6fb0bce7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a0bcf1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60bcf5, 76 },
    { 0, 0 },
    { 0xfe90bcf7, 98 },
    { 0xc250bcf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80bcff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f0bd11, 52 },
    { 0, 0 },
    { 0x3360bd13, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0bd18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0bd1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c0bd1f, 78 },
    { 0x9fa0bd1f, 90 },
    { 0x3740bd21, 93 },
    { 0, 0 },
    { 0x6df0bd23, 66 },
    { 0x540bd24, 61 },
    { 0x75e0bd25, 67 },
    { 0, 0 },
    { 0x78f0bd27, 84 },
    { 0, 0 },
    { 0xff10bd29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d0bd2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650bd30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa40bd33, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd680bd37, 81 },
    { 0xaf0bd38, 98 },
    { 0xa980bd39, 95 },
    { 0xdeb0bd37, 42 },
    { 0, 0 },
    { 0xcf80bd3c, 90 },
    { 0, 0 },
    { 0x74c0bd3e, 91 },
    { 0xa20bd3f, 46 },
    { 0x5ae0bd3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190bd44, 91 },
    { 0, 0 },
    { 0x5740bd46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80bd4d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c60bd50, 96 },
    { 0, 0 },
    { 0xade0bd52, 33 },
    { 0, 0 },
    { 0xae50bd54, 43 },
    { 0xe560bd55, 80 },
    { 0, 0 },
    { 0x4d10bd57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd40bd5e, 90 },
    { 0, 0 },
    { 0x8d20bd60, 76 },
    { 0x9690bd61, 76 },
    { 0xda60bd60, 33 },
    { 0x4870bd63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a0bd66, 55 },
    { 0x99b0bd66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e0bd6b, 88 },
    { 0xb5f0bd6b, 95 },
    { 0x3dd0bd6b, 61 },
    { 0, 0 },
    { 0x150bd6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd800bd72, 33 },
    { 0x8f90bd73, 34 },
    { 0xb9e0bd74, 52 },
    { 0xc4a0bd75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260bd7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0bd80, 84 },
    { 0xb5d0bd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9030bd84, 33 },
    { 0x4640bd84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa760bd88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780bd8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3370bd90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60bd96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf670bd9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0bda4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20bda7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd370bdab, 34 },
    { 0xa7d0bdac, 43 },
    { 0xee30bdad, 33 },
    { 0xfd10bdae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a20bdb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb410bdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e0bdc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b00bdc3, 34 },
    { 0, 0 },
    { 0x5d50bdc5, 98 },
    { 0xa9a0bdc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf290bdcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac0bdd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e0bdda, 90 },
    { 0xf7e0bdda, 61 },
    { 0, 0 },
    { 0x88d0bddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a0bde0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6650bde4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e90bde7, 98 },
    { 0xc0bde8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20bdec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b0bdf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3820bdf3, 55 },
    { 0, 0 },
    { 0x8af0bdf5, 61 },
    { 0x9340bdf6, 76 },
    { 0x3ea0bdf7, 43 },
    { 0, 0 },
    { 0x37e0bdf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c90bdfc, 23 },
    { 0xb080bdfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950be01, 61 },
    { 0x23c0be02, 90 },
    { 0, 0 },
    { 0x94d0be04, 98 },
    { 0x3500be05, 88 },
    { 0xff10be04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0be09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb0be12, 90 },
    { 0x38c0be13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4130be18, 76 },
    { 0, 0 },
    { 0xc120be1a, 33 },
    { 0x7770be1b, 81 },
    { 0x9d30be1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc60be20, 59 },
    { 0x9e10be20, 93 },
    { 0x34d0be22, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60be26, 34 },
    { 0x1a90be26, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a0be2d, 17 },
    { 0x9540be2e, 34 },
    { 0xf630be2d, 34 },
    { 0, 0 },
    { 0x30b0be31, 90 },
    { 0x24f0be31, 55 },
    { 0x8820be33, 98 },
    { 0xce90be32, 90 },
    { 0x7630be31, 90 },
    { 0x8310be36, 55 },
    { 0xce10be36, 61 },
    { 0xacc0be35, 98 },
    { 0x6db0be39, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf940be3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0be43, 46 },
    { 0xfb70be44, 34 },
    { 0xbcc0be43, 90 },
    { 0x20a0be45, 23 },
    { 0xa210be46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c40be4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb920be54, 52 },
    { 0x8fa0be54, 76 },
    { 0xd8f0be56, 23 },
    { 0xc5d0be57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a0be5a, 98 },
    { 0xef30be5b, 68 },
    { 0xd750be5b, 61 },
    { 0x8fd0be5a, 76 },
    { 0xcd30be5d, 33 },
    { 0x64d0be5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5610be62, 95 },
    { 0x4c30be63, 99 },
    { 0xf340be64, 68 },
    { 0x7970be65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0be6a, 33 },
    { 0x9d0be6a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd750be70, 66 },
    { 0xf670be71, 77 },
    { 0x3830be72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d0be7a, 90 },
    { 0x120be7b, 15 },
    { 0x28e0be7a, 43 },
    { 0, 0 },
    { 0xb9c0be7e, 91 },
    { 0x8d20be7f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e0be84, 98 },
    { 0, 0 },
    { 0xf080be86, 90 },
    { 0xe780be87, 85 },
    { 0xbbf0be88, 76 },
    { 0, 0 },
    { 0xa550be8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f0be8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d0be91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x340be94, 98 },
    { 0, 0 },
    { 0x3a80be96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c0bea0, 61 },
    { 0, 0 },
    { 0xc010bea2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780bea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0bead, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d40beb0, 33 },
    { 0xc530beb1, 80 },
    { 0, 0 },
    { 0xc0c0beb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf0beb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0beba, 76 },
    { 0x9e50bebb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd0bec1, 80 },
    { 0, 0 },
    { 0x7600bec3, 93 },
    { 0, 0 },
    { 0xde70bec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f40bec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0becc, 77 },
    { 0x14e0becd, 76 },
    { 0x8d10bece, 87 },
    { 0x8da0becc, 91 },
    { 0x5b70bed0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890bed5, 61 },
    { 0x18d0bed5, 33 },
    { 0x1e80bed6, 77 },
    { 0xc5a0bed8, 95 },
    { 0x89d0bed9, 95 },
    { 0x5a50beda, 91 },
    { 0x7300bed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940bedf, 43 },
    { 0xcaf0bee0, 33 },
    { 0x96d0bee1, 95 },
    { 0x8b10bee2, 55 },
    { 0, 0 },
    { 0xa9c0bee4, 98 },
    { 0xe5c0bee5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd0bee8, 98 },
    { 0, 0 },
    { 0x7100beea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec0beee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xba30bef1, 61 },
    { 0, 0 },
    { 0xf480bef3, 52 },
    { 0x5470bef3, 88 },
    { 0x4800bef5, 95 },
    { 0xf60bef5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b80beff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140bf02, 99 },
    { 0xe0b0bf03, 90 },
    { 0xd8c0bf04, 78 },
    { 0x3140bf05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed0bf08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bf0d, 34 },
    { 0xb320bf0d, 100 },
    { 0xa40bf0f, 66 },
    { 0x51c0bf10, 78 },
    { 0, 0 },
    { 0x9d60bf12, 78 },
    { 0xb290bf12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30bf1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a50bf20, 88 },
    { 0xdfe0bf21, 34 },
    { 0x8f10bf21, 99 },
    { 0xb370bf20, 33 },
    { 0xefa0bf24, 98 },
    { 0x32c0bf24, 66 },
    { 0, 0 },
    { 0x86f0bf27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8070bf2a, 84 },
    { 0, 0 },
    { 0x84a0bf2c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6540bf32, 84 },
    { 0x9070bf33, 90 },
    { 0, 0 },
    { 0x35e0bf35, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea0bf38, 75 },
    { 0xda10bf38, 93 },
    { 0x8d50bf39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b0bf3e, 65 },
    { 0, 0 },
    { 0xe5b0bf40, 90 },
    { 0xf1e0bf41, 90 },
    { 0xb1a0bf42, 34 },
    { 0x10b0bf41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0bf47, 69 },
    { 0x50a0bf47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00bf4b, 90 },
    { 0, 0 },
    { 0x7590bf4d, 23 },
    { 0x1590bf4d, 76 },
    { 0xb2f0bf4d, 90 },
    { 0xddc0bf4d, 35 },
    { 0, 0 },
    { 0xac60bf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad0bf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640bf60, 98 },
    { 0x6460bf61, 33 },
    { 0, 0 },
    { 0x6810bf63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef0bf68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d30bf6b, 55 },
    { 0xc6e0bf6c, 90 },
    { 0x8a10bf6d, 34 },
    { 0xf380bf6e, 61 },
    { 0xf370bf6f, 95 },
    { 0x9000bf70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c0bf74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970bf78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1770bf7d, 33 },
    { 0x1d70bf7e, 66 },
    { 0x9d20bf7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f40bf88, 55 },
    { 0, 0 },
    { 0x85c0bf8a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9410bf8e, 90 },
    { 0xb3f0bf8f, 81 },
    { 0x4f20bf8f, 95 },
    { 0xd920bf90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0bf94, 23 },
    { 0, 0 },
    { 0xf130bf96, 95 },
    { 0xcd40bf96, 23 },
    { 0xfb20bf97, 33 },
    { 0, 0 },
    { 0x6b30bf9a, 34 },
    { 0x52e0bf9b, 46 },
    { 0xb6f0bf9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30bf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6880bfa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c0bfa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f0bfaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f0bfb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab10bfbc, 61 },
    { 0, 0 },
    { 0x1b80bfbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba0bfc1, 34 },
    { 0x6820bfc2, 95 },
    { 0x4440bfc3, 23 },
    { 0x83b0bfc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc570bfcd, 52 },
    { 0xbf30bfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b0bfd1, 94 },
    { 0, 0 },
    { 0x3860bfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc60bfdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf990bfe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d0bfe3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab0bfe7, 46 },
    { 0xb630bfe8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c0bfeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed10bff1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8860bff4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9430bff7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d0bffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350bfff, 55 },
    { 0x5d80c000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c0c009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c00c, 33 },
    { 0x81c0c00d, 97 },
    { 0x79a0c00e, 33 },
    { 0, 0 },
    { 0x8800c010, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5650c013, 61 },
    { 0xe3a0c014, 69 },
    { 0xa40c014, 95 },
    { 0x56f0c015, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca50c019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea40c01c, 90 },
    { 0xa570c01d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2370c023, 78 },
    { 0x30b0c023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e0c027, 98 },
    { 0x1030c027, 61 },
    { 0xa70c029, 100 },
    { 0xb560c02a, 55 },
    { 0x33a0c02a, 98 },
    { 0xe930c02b, 93 },
    { 0, 0 },
    { 0x870c02e, 34 },
    { 0x5550c02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5800c032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0c035, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa0c039, 23 },
    { 0x46f0c03a, 66 },
    { 0x8c30c03a, 78 },
    { 0, 0 },
    { 0x44b0c03d, 90 },
    { 0, 0 },
    { 0xdda0c03f, 61 },
    { 0x76b0c040, 98 },
    { 0, 0 },
    { 0x42d0c042, 95 },
    { 0x9cf0c043, 33 },
    { 0xee90c044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8270c052, 43 },
    { 0x2d00c053, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8670c056, 76 },
    { 0x4960c057, 61 },
    { 0x6480c058, 98 },
    { 0, 0 },
    { 0x11f0c05a, 94 },
    { 0, 0 },
    { 0xf80c05c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a0c05f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a0c068, 95 },
    { 0x7c30c068, 95 },
    { 0x8ec0c06a, 90 },
    { 0x2530c06a, 68 },
    { 0x1c80c06c, 90 },
    { 0xecb0c06d, 100 },
    { 0x2fe0c06c, 61 },
    { 0, 0 },
    { 0x99a0c070, 43 },
    { 0x5c60c071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7130c07c, 55 },
    { 0xd720c07c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950c080, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8670c087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc540c08b, 90 },
    { 0x9940c08b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c08f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310c094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2410c099, 90 },
    { 0x2590c09a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa300c09e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80c0a1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a70c0a7, 55 },
    { 0x2c20c0a8, 66 },
    { 0x5000c0a9, 90 },
    { 0x9090c0aa, 95 },
    { 0xf9c0c0ab, 98 },
    { 0x3120c0ac, 87 },
    { 0x2580c0a9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4340c0b0, 17 },
    { 0, 0 },
    { 0x9e30c0b2, 95 },
    { 0x1e10c0b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0c0b9, 43 },
    { 0x2030c0ba, 66 },
    { 0, 0 },
    { 0x7780c0bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad50c0bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe140c0c3, 46 },
    { 0xe8a0c0c4, 90 },
    { 0x7a50c0c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80c0c9, 98 },
    { 0x7b50c0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc670c0d1, 90 },
    { 0x8620c0d2, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4950c0da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd770c0dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2430c0e2, 99 },
    { 0xf940c0e3, 70 },
    { 0xc080c0e4, 99 },
    { 0xad30c0e4, 33 },
    { 0x5810c0e6, 23 },
    { 0x7660c0e6, 99 },
    { 0x3070c0e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2990c0eb, 17 },
    { 0, 0 },
    { 0x8840c0ed, 55 },
    { 0, 0 },
    { 0x6f30c0ef, 76 },
    { 0, 0 },
    { 0x36b0c0f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7670c0f4, 55 },
    { 0xe610c0f5, 99 },
    { 0, 0 },
    { 0xae00c0f7, 61 },
    { 0, 0 },
    { 0x3c10c0f9, 95 },
    { 0x9ca0c0f9, 85 },
    { 0x85a0c0fb, 76 },
    { 0xfc70c0fc, 61 },
    { 0x7940c0f9, 90 },
    { 0xbb90c0fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd450c10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf930c113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b0c118, 98 },
    { 0xf480c119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0c120, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c40c126, 95 },
    { 0x4b30c126, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20c12d, 99 },
    { 0x5890c12e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a0c131, 68 },
    { 0, 0 },
    { 0xb8b0c133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d0c138, 98 },
    { 0, 0 },
    { 0xd450c13a, 90 },
    { 0x6cc0c13a, 75 },
    { 0, 0 },
    { 0xc8f0c13d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf0c147, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8230c14c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50c14f, 61 },
    { 0, 0 },
    { 0x8560c151, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c157, 23 },
    { 0x2960c158, 66 },
    { 0x88f0c159, 61 },
    { 0xffa0c159, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c15e, 46 },
    { 0xed00c15e, 90 },
    { 0x97e0c160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80c163, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd0c16a, 78 },
    { 0, 0 },
    { 0x2020c16c, 65 },
    { 0xc980c16d, 66 },
    { 0x9760c16e, 87 },
    { 0x7a90c16e, 96 },
    { 0, 0 },
    { 0xf960c171, 23 },
    { 0x2e20c172, 99 },
    { 0x30c0c173, 61 },
    { 0xc280c174, 87 },
    { 0x2ea0c172, 93 },
    { 0xd9a0c174, 43 },
    { 0xe30c175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2390c17d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0c18e, 33 },
    { 0xe490c18e, 95 },
    { 0x5ce0c190, 91 },
    { 0xaa20c18f, 95 },
    { 0x8ba0c190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf760c195, 23 },
    { 0xd940c196, 46 },
    { 0, 0 },
    { 0xe340c198, 34 },
    { 0x1af0c199, 34 },
    { 0xc060c19a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a0c19d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d40c1aa, 61 },
    { 0, 0 },
    { 0xe00c1ac, 99 },
    { 0xf480c1ad, 90 },
    { 0x4e90c1ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf0c1b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250c1be, 33 },
    { 0x6e90c1bf, 23 },
    { 0x93d0c1bf, 90 },
    { 0xac30c1be, 61 },
    { 0, 0 },
    { 0x9ef0c1c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630c1c6, 87 },
    { 0, 0 },
    { 0x70d0c1c8, 61 },
    { 0x9010c1c9, 33 },
    { 0x2330c1c8, 95 },
    { 0xcf00c1ca, 93 },
    { 0x4a40c1cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda20c1cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc10c1d2, 33 },
    { 0, 0 },
    { 0xd4e0c1d4, 33 },
    { 0, 0 },
    { 0x1260c1d6, 43 },
    { 0x96d0c1d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe520c1dc, 101 },
    { 0xd810c1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd100c1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a70c1e6, 46 },
    { 0, 0 },
    { 0xc0c0c1e8, 95 },
    { 0x9450c1e9, 17 },
    { 0, 0 },
    { 0x4a30c1eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df0c1ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a0c1f2, 55 },
    { 0x880c1f2, 23 },
    { 0xf8d0c1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf590c1f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f0c1fd, 55 },
    { 0, 0 },
    { 0x9f60c1ff, 61 },
    { 0, 0 },
    { 0x2360c201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1710c205, 90 },
    { 0x65d0c206, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0c214, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0c217, 95 },
    { 0x31c0c218, 61 },
    { 0, 0 },
    { 0x2aa0c21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0c21d, 33 },
    { 0x5c80c21e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f0c223, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c0c22b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c22f, 43 },
    { 0xe5e0c22f, 99 },
    { 0x6bd0c231, 23 },
    { 0xf3f0c231, 61 },
    { 0x5560c233, 90 },
    { 0xf7f0c233, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7530c23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e0c23f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810c245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf610c249, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90c24c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60c250, 23 },
    { 0, 0 },
    { 0xb000c252, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc20c255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7810c258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0c25c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a90c261, 98 },
    { 0, 0 },
    { 0x4960c263, 84 },
    { 0x1e20c264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db0c268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790c26f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a30c273, 90 },
    { 0x6a0c273, 78 },
    { 0x8380c275, 61 },
    { 0x7df0c276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c0c280, 76 },
    { 0xa5d0c281, 23 },
    { 0, 0 },
    { 0x85f0c283, 33 },
    { 0xd4e0c283, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560c288, 23 },
    { 0xcbe0c289, 84 },
    { 0xbdd0c28a, 95 },
    { 0x1d0c28a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0c28e, 77 },
    { 0x8a00c28f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd690c293, 66 },
    { 0x27b0c294, 33 },
    { 0, 0 },
    { 0x4940c296, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf870c299, 46 },
    { 0x6790c29a, 61 },
    { 0x3620c29a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70c29e, 99 },
    { 0xcbe0c29e, 33 },
    { 0x1b60c29f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8840c2a4, 76 },
    { 0x6260c2a4, 33 },
    { 0x72c0c2a6, 76 },
    { 0xd10c2a6, 95 },
    { 0xcee0c2a8, 98 },
    { 0x66a0c2a9, 61 },
    { 0x8c10c2a6, 95 },
    { 0, 0 },
    { 0x8190c2ac, 95 },
    { 0x6cd0c2ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80c2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2660c2b5, 55 },
    { 0x7620c2b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b0c2bc, 33 },
    { 0x4a00c2bd, 81 },
    { 0x80d0c2be, 98 },
    { 0xebc0c2bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe880c2c7, 90 },
    { 0xc0d0c2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230c2d2, 95 },
    { 0, 0 },
    { 0x8160c2d4, 100 },
    { 0, 0 },
    { 0x9e80c2d6, 61 },
    { 0x6180c2d7, 61 },
    { 0xb0b0c2d8, 93 },
    { 0x8680c2d8, 95 },
    { 0x5310c2da, 78 },
    { 0x7cb0c2d9, 95 },
    { 0xa70c2dc, 95 },
    { 0x700c2dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4620c2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780c2e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b40c2e8, 23 },
    { 0xcdf0c2e9, 61 },
    { 0, 0 },
    { 0x5f70c2eb, 90 },
    { 0xc5d0c2ec, 65 },
    { 0xb7b0c2ed, 61 },
    { 0xe8d0c2ec, 76 },
    { 0, 0 },
    { 0x3a50c2f0, 77 },
    { 0xfad0c2f1, 34 },
    { 0, 0 },
    { 0xe440c2f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680c2f7, 55 },
    { 0xa070c2f8, 33 },
    { 0xac50c2f8, 61 },
    { 0xeb40c2fa, 9 },
    { 0x70f0c2f8, 61 },
    { 0x280c2fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb40c303, 61 },
    { 0xb600c303, 90 },
    { 0x4760c305, 33 },
    { 0xbc50c306, 98 },
    { 0x9870c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7280c30a, 90 },
    { 0xdd0c30b, 46 },
    { 0xbe00c30c, 98 },
    { 0x4800c30d, 43 },
    { 0x8ba0c30c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a0c316, 95 },
    { 0x3280c317, 100 },
    { 0, 0 },
    { 0x3a70c319, 67 },
    { 0x2bd0c31a, 61 },
    { 0, 0 },
    { 0xde20c31c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20c31f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba0c322, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d0c329, 90 },
    { 0x610c32a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0c32d, 36 },
    { 0, 0 },
    { 0x1640c32f, 90 },
    { 0, 0 },
    { 0xee20c331, 90 },
    { 0xb7d0c332, 98 },
    { 0, 0 },
    { 0x29a0c334, 71 },
    { 0, 0 },
    { 0x6f70c336, 98 },
    { 0xa860c337, 60 },
    { 0xff80c338, 93 },
    { 0xba80c339, 23 },
    { 0xa8b0c339, 23 },
    { 0xac90c336, 90 },
    { 0xea90c33c, 23 },
    { 0, 0 },
    { 0x5960c33e, 46 },
    { 0, 0 },
    { 0x2b90c340, 23 },
    { 0xed50c341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9060c348, 23 },
    { 0, 0 },
    { 0x9190c34a, 33 },
    { 0, 0 },
    { 0x4e80c34c, 85 },
    { 0, 0 },
    { 0x88e0c34e, 33 },
    { 0x6bf0c34f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd520c357, 52 },
    { 0, 0 },
    { 0x87d0c359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d10c35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7750c35f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9110c362, 55 },
    { 0xc150c363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc660c366, 88 },
    { 0xb730c366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b0c36a, 90 },
    { 0xf360c36b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc0c373, 33 },
    { 0x90d0c374, 61 },
    { 0x60c374, 98 },
    { 0xb7c0c376, 95 },
    { 0x6110c376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0c37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800c388, 90 },
    { 0x15e0c389, 66 },
    { 0x8560c38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa0c38e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0c391, 23 },
    { 0x5080c392, 76 },
    { 0x8740c391, 87 },
    { 0x1850c394, 95 },
    { 0x5a50c394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10c39c, 51 },
    { 0x45e0c39c, 61 },
    { 0, 0 },
    { 0xc7e0c39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4010c3a5, 98 },
    { 0xcb40c3a5, 87 },
    { 0x3a50c3a7, 87 },
    { 0x8840c3a8, 17 },
    { 0xa970c3a8, 66 },
    { 0xace0c3a8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9120c3ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd90c3b1, 90 },
    { 0x560c3b2, 70 },
    { 0xd590c3b3, 95 },
    { 0, 0 },
    { 0x3c30c3b5, 98 },
    { 0, 0 },
    { 0x6140c3b7, 90 },
    { 0x7240c3b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a0c3bb, 98 },
    { 0x88b0c3bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0c3c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5170c3c3, 91 },
    { 0, 0 },
    { 0xcb20c3c5, 90 },
    { 0x65f0c3c6, 61 },
    { 0xa860c3c7, 98 },
    { 0xef10c3c7, 76 },
    { 0xca80c3c9, 40 },
    { 0xbea0c3c9, 90 },
    { 0x2ca0c3cb, 99 },
    { 0xf0e0c3cb, 98 },
    { 0x6f80c3cd, 61 },
    { 0xab90c3cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f0c3d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50c3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840c3e1, 95 },
    { 0, 0 },
    { 0x8cf0c3e3, 43 },
    { 0, 0 },
    { 0xa950c3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d0c3e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0c3ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc530c3f6, 65 },
    { 0, 0 },
    { 0xbc00c3f8, 78 },
    { 0xe9a0c3f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf70c401, 45 },
    { 0x3550c402, 93 },
    { 0xd980c402, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b0c406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2560c40b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c410, 33 },
    { 0xf220c411, 90 },
    { 0, 0 },
    { 0x5aa0c413, 68 },
    { 0x330c413, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f60c41f, 66 },
    { 0x7e20c420, 23 },
    { 0x9460c420, 98 },
    { 0xba30c422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b0c428, 33 },
    { 0x9780c429, 55 },
    { 0xff0c428, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0c436, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0c439, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6150c43e, 61 },
    { 0, 0 },
    { 0xa720c440, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xf840c443, 34 },
    { 0, 0 },
    { 0xb900c445, 43 },
    { 0x1d70c446, 23 },
    { 0, 0 },
    { 0x88b0c448, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c44f, 90 },
    { 0, 0 },
    { 0x8f60c451, 33 },
    { 0x5350c452, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0c455, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a20c458, 90 },
    { 0, 0 },
    { 0x9190c45a, 87 },
    { 0x8010c45b, 23 },
    { 0, 0 },
    { 0x6c90c45d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10c463, 46 },
    { 0xeaf0c464, 33 },
    { 0, 0 },
    { 0xd350c466, 90 },
    { 0, 0 },
    { 0x5310c468, 78 },
    { 0xa130c469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc0c46e, 61 },
    { 0x35e0c46f, 95 },
    { 0xe510c470, 95 },
    { 0x8cf0c470, 91 },
    { 0x5810c470, 55 },
    { 0x8ef0c473, 33 },
    { 0x6490c474, 76 },
    { 0x8720c471, 55 },
    { 0xad50c476, 100 },
    { 0xf970c477, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4420c47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd70c480, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5310c486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3210c48b, 84 },
    { 0, 0 },
    { 0x3620c48d, 91 },
    { 0, 0 },
    { 0x8370c48f, 34 },
    { 0, 0 },
    { 0x90d0c491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6340c495, 23 },
    { 0xe30c496, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5550c499, 90 },
    { 0, 0 },
    { 0x7980c49b, 33 },
    { 0x1000c49c, 61 },
    { 0x9540c49d, 76 },
    { 0, 0 },
    { 0xfe60c49f, 91 },
    { 0xd840c49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60c4a6, 49 },
    { 0xa880c4a7, 61 },
    { 0x99c0c4a8, 23 },
    { 0x3680c4a9, 23 },
    { 0x5cf0c4aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440c4b2, 61 },
    { 0x2660c4b2, 93 },
    { 0xd250c4b4, 78 },
    { 0x23c0c4b5, 55 },
    { 0, 0 },
    { 0x2800c4b7, 33 },
    { 0xd6c0c4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a90c4bd, 90 },
    { 0x6220c4be, 67 },
    { 0x6260c4bf, 95 },
    { 0xc1f0c4bd, 40 },
    { 0x5f0c4c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0c4cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa0c4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10c4db, 61 },
    { 0xc190c4dc, 61 },
    { 0, 0 },
    { 0xec40c4de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190c4e1, 36 },
    { 0x99d0c4e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c0c4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f0c4ea, 61 },
    { 0x9b00c4eb, 78 },
    { 0xaf00c4ec, 95 },
    { 0xaf80c4ed, 33 },
    { 0x2720c4ec, 98 },
    { 0xa320c4ef, 33 },
    { 0x66f0c4f0, 99 },
    { 0xa3e0c4f1, 78 },
    { 0x2af0c4f0, 23 },
    { 0xbc80c4f3, 23 },
    { 0, 0 },
    { 0xfc60c4f5, 33 },
    { 0xc380c4f6, 55 },
    { 0xcf50c4f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0c4fb, 90 },
    { 0x3ef0c4fb, 33 },
    { 0x3d30c4fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b0c505, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee0c508, 90 },
    { 0, 0 },
    { 0xfb70c50a, 90 },
    { 0xd7c0c50b, 90 },
    { 0x82d0c50c, 95 },
    { 0x36d0c50d, 90 },
    { 0, 0 },
    { 0x8740c50f, 98 },
    { 0, 0 },
    { 0x1420c511, 27 },
    { 0xdbf0c512, 55 },
    { 0xa760c511, 61 },
    { 0x51d0c513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f0c521, 33 },
    { 0x55c0c521, 99 },
    { 0x1420c523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40c526, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a00c529, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850c538, 97 },
    { 0, 0 },
    { 0x2620c53a, 23 },
    { 0, 0 },
    { 0x3460c53c, 76 },
    { 0xf3e0c53d, 61 },
    { 0x8c00c53e, 33 },
    { 0xbb80c53c, 98 },
    { 0, 0 },
    { 0xd180c541, 93 },
    { 0, 0 },
    { 0x3650c543, 66 },
    { 0, 0 },
    { 0xaaf0c545, 33 },
    { 0x6ba0c546, 87 },
    { 0x2da0c547, 61 },
    { 0x2a20c548, 90 },
    { 0x2810c545, 34 },
    { 0x8a60c54a, 90 },
    { 0x8990c54b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110c54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d0c557, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd040c55b, 68 },
    { 0x3e90c55b, 84 },
    { 0, 0 },
    { 0xd50c55e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0c561, 66 },
    { 0xa750c562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1050c56b, 90 },
    { 0, 0 },
    { 0xdd90c56d, 61 },
    { 0xb0c0c56e, 93 },
    { 0x2900c56d, 33 },
    { 0xae00c56e, 90 },
    { 0x910c571, 23 },
    { 0xa550c56d, 90 },
    { 0x960c573, 61 },
    { 0x46d0c573, 98 },
    { 0x8f30c575, 65 },
    { 0x89e0c576, 55 },
    { 0xf2d0c577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0c580, 90 },
    { 0xbac0c580, 46 },
    { 0x9420c581, 46 },
    { 0x4fb0c583, 23 },
    { 0x7820c581, 96 },
    { 0xb850c585, 43 },
    { 0x8610c585, 61 },
    { 0xd250c580, 98 },
    { 0x3140c588, 90 },
    { 0x3ed0c586, 93 },
    { 0x9510c587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f0c58e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb890c591, 87 },
    { 0x1770c591, 95 },
    { 0xca00c593, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b70c59a, 90 },
    { 0xdc80c59a, 76 },
    { 0x6ae0c59c, 61 },
    { 0xede0c59c, 98 },
    { 0x2a90c59d, 100 },
    { 0xf150c59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd770c5a3, 76 },
    { 0xa30c5a3, 23 },
    { 0xa9e0c5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0c5a8, 81 },
    { 0x7080c5a8, 90 },
    { 0x2810c5aa, 23 },
    { 0x99d0c5ab, 95 },
    { 0x3430c5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2280c5af, 98 },
    { 0xc9f0c5b0, 87 },
    { 0xea30c5af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf0c5b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f0c5ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230c5c1, 34 },
    { 0, 0 },
    { 0x17c0c5c3, 66 },
    { 0xf7e0c5c4, 43 },
    { 0xc690c5c5, 94 },
    { 0xff20c5c6, 68 },
    { 0x7d40c5c6, 66 },
    { 0xb10c5c7, 61 },
    { 0xe720c5c6, 95 },
    { 0xa900c5c6, 55 },
    { 0x99d0c5c4, 85 },
    { 0x1eb0c5cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc520c5d2, 43 },
    { 0xd7d0c5d2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca0c5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf30c5e2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a0c5e8, 75 },
    { 0x2d80c5e9, 76 },
    { 0, 0 },
    { 0xc800c5eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa890c5f0, 98 },
    { 0x8720c5f1, 33 },
    { 0xefb0c5f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50c5f5, 98 },
    { 0xe000c5f6, 95 },
    { 0, 0 },
    { 0xa8a0c5f8, 33 },
    { 0, 0 },
    { 0xfa60c5fa, 33 },
    { 0xbbe0c5fb, 61 },
    { 0x52d0c5fc, 99 },
    { 0x4890c5fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded0c602, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000c605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb60c609, 61 },
    { 0x37c0c609, 87 },
    { 0, 0 },
    { 0x3390c60c, 23 },
    { 0x1570c60d, 43 },
    { 0x4700c60c, 90 },
    { 0xe570c60f, 33 },
    { 0x6460c610, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120c614, 43 },
    { 0x9700c615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b0c619, 90 },
    { 0, 0 },
    { 0x48c0c61b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c621, 95 },
    { 0x7180c622, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910c625, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630c629, 19 },
    { 0xd760c62a, 34 },
    { 0x1890c629, 90 },
    { 0x5ec0c62c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0c62f, 23 },
    { 0xbb20c62f, 61 },
    { 0xf780c62f, 61 },
    { 0xbad0c631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d20c635, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150c639, 98 },
    { 0x2850c63a, 61 },
    { 0, 0 },
    { 0x5ee0c63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c640, 43 },
    { 0, 0 },
    { 0xd910c642, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d0c645, 61 },
    { 0, 0 },
    { 0x8720c647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c64c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3710c650, 66 },
    { 0x2c30c651, 46 },
    { 0, 0 },
    { 0x9470c653, 98 },
    { 0x8290c654, 98 },
    { 0xf280c655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c65b, 23 },
    { 0x4050c65b, 87 },
    { 0xd6d0c65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf080c660, 64 },
    { 0x8fd0c660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b10c665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710c669, 61 },
    { 0x1900c66a, 99 },
    { 0x2b90c669, 33 },
    { 0x4f50c66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80c670, 33 },
    { 0, 0 },
    { 0x63b0c672, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b0c679, 90 },
    { 0x44c0c67a, 85 },
    { 0xc600c679, 17 },
    { 0, 0 },
    { 0xfe80c67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240c682, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190c685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d0c68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0c690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0c697, 43 },
    { 0x55c0c698, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd920c69c, 88 },
    { 0x8c20c69d, 52 },
    { 0xc110c69d, 23 },
    { 0, 0 },
    { 0xd320c6a0, 23 },
    { 0x94f0c6a1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9200c6a6, 90 },
    { 0x9de0c6a7, 85 },
    { 0xfd0c6a8, 90 },
    { 0x1bc0c6a6, 75 },
    { 0x770c6aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320c6ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b0c6b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0c6b6, 37 },
    { 0, 0 },
    { 0x5a50c6b8, 53 },
    { 0xe330c6b8, 23 },
    { 0x7490c6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4910c6be, 33 },
    { 0xc60c6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd0c6c9, 96 },
    { 0xc8b0c6c9, 84 },
    { 0, 0 },
    { 0x9010c6cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1720c6cf, 34 },
    { 0x9e90c6cf, 85 },
    { 0x19f0c6d1, 23 },
    { 0x4550c6d1, 98 },
    { 0xc340c6d1, 43 },
    { 0xc320c6d2, 33 },
    { 0x8030c6d5, 61 },
    { 0x57b0c6d6, 90 },
    { 0x5440c6d6, 66 },
    { 0xe990c6d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100c6db, 90 },
    { 0, 0 },
    { 0x9de0c6dd, 68 },
    { 0, 0 },
    { 0x2b20c6df, 90 },
    { 0xed80c6e0, 97 },
    { 0, 0 },
    { 0x250c6e2, 95 },
    { 0x23c0c6e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440c6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b0c6ed, 76 },
    { 0x7db0c6ed, 34 },
    { 0x110c6ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0c6f5, 88 },
    { 0x2920c6f5, 95 },
    { 0, 0 },
    { 0x9e10c6f8, 46 },
    { 0x6630c6f9, 23 },
    { 0xd800c6fa, 61 },
    { 0x6f40c6fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc670c700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5640c703, 95 },
    { 0x19b0c704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe0c70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0c70e, 85 },
    { 0x2540c70e, 90 },
    { 0, 0 },
    { 0xba90c711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40c714, 34 },
    { 0, 0 },
    { 0xe4a0c716, 46 },
    { 0xdb40c716, 95 },
    { 0xe220c718, 61 },
    { 0x1760c719, 61 },
    { 0, 0 },
    { 0x1670c71b, 61 },
    { 0xe9a0c71c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490c722, 101 },
    { 0x44c0c722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2830c728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd0c72c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f30c72f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0c733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0c736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9010c73f, 95 },
    { 0x1920c740, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410c743, 17 },
    { 0xcd0c744, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac40c74a, 33 },
    { 0xf6b0c74a, 95 },
    { 0, 0 },
    { 0xaea0c74d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220c757, 33 },
    { 0x1830c758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c30c75c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020c75f, 61 },
    { 0xab50c760, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf650c764, 95 },
    { 0xefc0c765, 98 },
    { 0x3e70c764, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b0c76a, 55 },
    { 0xec90c76b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4210c771, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da0c776, 55 },
    { 0xd5a0c777, 98 },
    { 0xd520c778, 84 },
    { 0x9490c776, 93 },
    { 0x1b80c776, 76 },
    { 0x7060c77b, 85 },
    { 0x1a20c77c, 67 },
    { 0xbc20c77b, 95 },
    { 0xc420c77b, 95 },
    { 0xe960c77f, 23 },
    { 0x9dc0c77c, 44 },
    { 0, 0 },
    { 0x7990c782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c78c, 66 },
    { 0x8c30c78c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f0c790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f70c794, 90 },
    { 0, 0 },
    { 0x4ca0c796, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860c799, 90 },
    { 0, 0 },
    { 0x5350c79b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a0c79f, 61 },
    { 0xa130c7a0, 90 },
    { 0xd750c7a1, 43 },
    { 0x9a60c7a2, 43 },
    { 0xd340c7a3, 76 },
    { 0x9d20c7a4, 84 },
    { 0xc9d0c7a5, 55 },
    { 0xea50c7a6, 98 },
    { 0x5b80c7a5, 33 },
    { 0x75e0c7a4, 61 },
    { 0xbe0c7a9, 23 },
    { 0xe00c7a9, 74 },
    { 0, 0 },
    { 0x2ff0c7ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0c7af, 98 },
    { 0x640c7af, 33 },
    { 0, 0 },
    { 0x2b10c7b2, 99 },
    { 0x7750c7b3, 33 },
    { 0x3800c7b3, 66 },
    { 0x9610c7b2, 95 },
    { 0, 0 },
    { 0x1150c7b7, 67 },
    { 0x770c7b8, 92 },
    { 0, 0 },
    { 0xe380c7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540c7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb90c7c3, 98 },
    { 0, 0 },
    { 0x1930c7c5, 34 },
    { 0xac20c7c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280c7c9, 61 },
    { 0xe2f0c7ca, 23 },
    { 0x7d40c7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920c7d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1160c7da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee0c7e1, 61 },
    { 0x6410c7e1, 78 },
    { 0x5670c7e3, 90 },
    { 0xa80c7e4, 85 },
    { 0xcb70c7e3, 67 },
    { 0x6f40c7e2, 33 },
    { 0x2e50c7e7, 78 },
    { 0, 0 },
    { 0xfd00c7e9, 76 },
    { 0x9f10c7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6350c7ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5480c7f1, 34 },
    { 0, 0 },
    { 0x9f50c7f3, 69 },
    { 0x3fb0c7f4, 93 },
    { 0x9ea0c7f5, 78 },
    { 0xd230c7f4, 65 },
    { 0x94a0c7f7, 90 },
    { 0x8e60c7f8, 17 },
    { 0x17c0c7f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff10c7fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc0c802, 98 },
    { 0, 0 },
    { 0x72e0c804, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a0c80e, 78 },
    { 0x58c0c80e, 23 },
    { 0x5bd0c80f, 98 },
    { 0x48c0c811, 69 },
    { 0, 0 },
    { 0xa3c0c813, 98 },
    { 0xd1e0c814, 95 },
    { 0, 0 },
    { 0xe110c816, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a0c81c, 67 },
    { 0x2340c81d, 33 },
    { 0xdc10c81e, 23 },
    { 0x7d80c81f, 43 },
    { 0x6c50c820, 90 },
    { 0xe040c81c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4570c824, 61 },
    { 0x5280c824, 33 },
    { 0x73d0c824, 90 },
    { 0, 0 },
    { 0xa200c828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4330c82c, 90 },
    { 0x1f50c82d, 95 },
    { 0x2d70c82d, 98 },
    { 0xccc0c82f, 98 },
    { 0xb330c82f, 98 },
    { 0x7940c831, 90 },
    { 0xd090c832, 33 },
    { 0xcc0c82c, 98 },
    { 0x75f0c831, 46 },
    { 0x3610c82e, 33 },
    { 0, 0 },
    { 0x490c837, 61 },
    { 0, 0 },
    { 0x9c20c839, 34 },
    { 0, 0 },
    { 0xd9c0c83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7520c83f, 33 },
    { 0, 0 },
    { 0xc8d0c841, 52 },
    { 0x79e0c842, 90 },
    { 0x630c843, 85 },
    { 0, 0 },
    { 0xd800c845, 33 },
    { 0x7020c845, 81 },
    { 0x26a0c846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf260c84a, 90 },
    { 0xbb10c84b, 34 },
    { 0x8bc0c84b, 23 },
    { 0, 0 },
    { 0x8260c84e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8920c855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270c858, 33 },
    { 0x5630c859, 55 },
    { 0x5290c85a, 34 },
    { 0x8610c85b, 19 },
    { 0, 0 },
    { 0x9e60c85d, 98 },
    { 0xe630c85e, 90 },
    { 0x3960c85d, 90 },
    { 0xaeb0c85f, 23 },
    { 0xcc70c85e, 53 },
    { 0, 0 },
    { 0xa8c0c863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf30c867, 66 },
    { 0xe240c867, 34 },
    { 0x8e50c869, 88 },
    { 0, 0 },
    { 0x9760c86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd310c874, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5330c877, 61 },
    { 0, 0 },
    { 0x1320c879, 53 },
    { 0, 0 },
    { 0x3680c87b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610c880, 33 },
    { 0xa5a0c880, 95 },
    { 0, 0 },
    { 0xeb30c883, 77 },
    { 0, 0 },
    { 0xd760c885, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd620c888, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9320c88c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1580c88f, 69 },
    { 0x7a60c890, 91 },
    { 0xf6e0c891, 23 },
    { 0x6120c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c0c895, 46 },
    { 0, 0 },
    { 0x2a70c897, 100 },
    { 0, 0 },
    { 0x9f40c899, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d00c89e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf0c8a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a10c8a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0c8a7, 98 },
    { 0, 0 },
    { 0x66d0c8a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00c8b0, 23 },
    { 0, 0 },
    { 0x5c30c8b2, 91 },
    { 0, 0 },
    { 0x14d0c8b4, 61 },
    { 0x42f0c8b4, 66 },
    { 0xd250c8b6, 90 },
    { 0xb5f0c8b7, 95 },
    { 0x2820c8b8, 36 },
    { 0xe320c8b8, 76 },
    { 0x58e0c8b7, 95 },
    { 0x15d0c8b7, 90 },
    { 0x2820c8bc, 95 },
    { 0x2bf0c8b9, 90 },
    { 0xe1d0c8b5, 66 },
    { 0xbcf0c8b9, 80 },
    { 0x8a00c8c0, 33 },
    { 0x62e0c8ba, 93 },
    { 0x8f90c8be, 75 },
    { 0xf940c8c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb0c8c7, 43 },
    { 0, 0 },
    { 0x2d0c8c9, 33 },
    { 0, 0 },
    { 0xcc50c8cb, 33 },
    { 0, 0 },
    { 0x1100c8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae0c8d1, 81 },
    { 0x1020c8d2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310c8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa0c8d9, 76 },
    { 0x6e80c8d9, 95 },
    { 0, 0 },
    { 0xd190c8dc, 43 },
    { 0xcef0c8dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0c8e1, 55 },
    { 0xc410c8e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60c8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd010c8eb, 100 },
    { 0, 0 },
    { 0xfd50c8ed, 98 },
    { 0xbcd0c8ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe0c8f3, 61 },
    { 0, 0 },
    { 0x22e0c8f5, 61 },
    { 0x7d20c8f6, 33 },
    { 0x4910c8f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0c8fd, 61 },
    { 0x8f10c8fd, 33 },
    { 0, 0 },
    { 0x5f70c900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7970c904, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb10c907, 81 },
    { 0xf390c907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80c90b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c910, 76 },
    { 0x97b0c911, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f0c916, 36 },
    { 0x36f0c917, 34 },
    { 0xf8b0c916, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c91c, 76 },
    { 0x9390c91c, 55 },
    { 0x52e0c91e, 33 },
    { 0xafb0c91d, 95 },
    { 0, 0 },
    { 0xe580c921, 46 },
    { 0x61b0c922, 93 },
    { 0xb750c923, 76 },
    { 0x2d00c924, 90 },
    { 0, 0 },
    { 0xccc0c926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c20c92e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a60c931, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2490c93a, 95 },
    { 0, 0 },
    { 0xfe30c93c, 61 },
    { 0, 0 },
    { 0xc130c93e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd510c942, 94 },
    { 0x9aa0c943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000c94d, 76 },
    { 0x88a0c94e, 93 },
    { 0xe1a0c94f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca00c954, 61 },
    { 0x3db0c955, 76 },
    { 0x300c956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f40c959, 61 },
    { 0xead0c95a, 52 },
    { 0xb4e0c95b, 81 },
    { 0, 0 },
    { 0x800c95d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e0c960, 95 },
    { 0xcf80c961, 81 },
    { 0xcb20c961, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc80c967, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb0c96d, 46 },
    { 0x6a30c96e, 55 },
    { 0xdd40c96f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd0c975, 33 },
    { 0x9550c976, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f0c97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0c980, 90 },
    { 0, 0 },
    { 0x2bb0c982, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de0c98a, 95 },
    { 0x3b50c98b, 33 },
    { 0xcef0c98a, 98 },
    { 0xc000c98b, 98 },
    { 0xbce0c98e, 55 },
    { 0, 0 },
    { 0xdba0c990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe290c993, 43 },
    { 0, 0 },
    { 0x92e0c995, 33 },
    { 0x2970c996, 61 },
    { 0xbcf0c997, 33 },
    { 0xdb60c997, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0c99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c0c99f, 93 },
    { 0, 0 },
    { 0x4fc0c9a1, 33 },
    { 0, 0 },
    { 0x3f40c9a3, 61 },
    { 0xeb10c9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0c9a8, 23 },
    { 0, 0 },
    { 0x3ac0c9aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae40c9af, 98 },
    { 0x9ae0c9af, 90 },
    { 0x4900c9b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d0c9b8, 91 },
    { 0xda30c9b9, 90 },
    { 0, 0 },
    { 0x5650c9bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb0c9bf, 17 },
    { 0xeb00c9c0, 61 },
    { 0xff10c9bf, 36 },
    { 0x22c0c9bf, 33 },
    { 0x6810c9c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0c9c6, 90 },
    { 0x4880c9c7, 23 },
    { 0, 0 },
    { 0x9d80c9c9, 17 },
    { 0x39f0c9c9, 52 },
    { 0, 0 },
    { 0xb7f0c9cc, 98 },
    { 0xca80c9cd, 95 },
    { 0, 0 },
    { 0x41e0c9cf, 98 },
    { 0x8350c9d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100c9d5, 76 },
    { 0, 0 },
    { 0x34b0c9d7, 55 },
    { 0xcf20c9d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1210c9dc, 76 },
    { 0xd8b0c9dd, 66 },
    { 0, 0 },
    { 0xffa0c9df, 95 },
    { 0x2e50c9e0, 52 },
    { 0, 0 },
    { 0xc50c9e2, 55 },
    { 0x31c0c9e3, 90 },
    { 0xd070c9e4, 55 },
    { 0x95e0c9e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe720c9eb, 98 },
    { 0, 0 },
    { 0xb0c0c9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050c9f1, 66 },
    { 0xe040c9f2, 33 },
    { 0x8480c9f3, 90 },
    { 0xfc00c9f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40c9f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee60c9fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b0c9ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa0ca02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce0ca05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0ca0a, 90 },
    { 0xf080ca0a, 99 },
    { 0x1220ca0c, 33 },
    { 0xb1f0ca0d, 98 },
    { 0xed10ca0e, 33 },
    { 0x7bc0ca0f, 34 },
    { 0, 0 },
    { 0x6420ca11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1050ca19, 61 },
    { 0x8190ca1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf300ca25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd0ca28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f0ca2c, 95 },
    { 0, 0 },
    { 0x8830ca2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f00ca31, 61 },
    { 0x10b0ca31, 17 },
    { 0xd3e0ca32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf0ca36, 84 },
    { 0, 0 },
    { 0x23c0ca38, 90 },
    { 0, 0 },
    { 0x8440ca3a, 84 },
    { 0x1bc0ca3a, 61 },
    { 0x1a90ca3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4550ca41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf90ca50, 78 },
    { 0xd540ca51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x830ca54, 76 },
    { 0, 0 },
    { 0xb280ca56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b0ca5a, 90 },
    { 0x85c0ca5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690ca60, 84 },
    { 0x42e0ca61, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0ca64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5790ca67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc0ca6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f0ca6d, 98 },
    { 0x6ab0ca6e, 80 },
    { 0, 0 },
    { 0xc30ca70, 33 },
    { 0, 0 },
    { 0x8480ca72, 95 },
    { 0, 0 },
    { 0xc750ca74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060ca79, 97 },
    { 0xf650ca7a, 34 },
    { 0x6db0ca7b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980ca80, 88 },
    { 0, 0 },
    { 0x4040ca82, 67 },
    { 0x1950ca83, 90 },
    { 0x5730ca84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640ca8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b0ca93, 61 },
    { 0x28e0ca93, 87 },
    { 0xc50ca94, 87 },
    { 0, 0 },
    { 0x4de0ca97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20ca9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240ca9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e0caa4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa580caa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb320caab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0cab1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b30cab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f0cab9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0cabd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f0cac0, 93 },
    { 0, 0 },
    { 0xeb00cac2, 68 },
    { 0xc3b0cac3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9890cac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80caca, 52 },
    { 0xd780cacb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce60cad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc420cad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0cade, 95 },
    { 0, 0 },
    { 0x7140cae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70cae4, 90 },
    { 0xb980cae5, 66 },
    { 0xb310cae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70caea, 33 },
    { 0xdf0caeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a0caee, 78 },
    { 0, 0 },
    { 0x7c80caf0, 23 },
    { 0, 0 },
    { 0x14b0caf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50caf5, 23 },
    { 0x4440caf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060cafb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6110caff, 61 },
    { 0x5220cb00, 95 },
    { 0x1a20cb01, 34 },
    { 0x1340cb02, 90 },
    { 0x31b0cb03, 90 },
    { 0x68c0cb02, 33 },
    { 0x8fe0cb05, 23 },
    { 0x1870cb06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0cb0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e60cb10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0cb13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da0cb16, 99 },
    { 0, 0 },
    { 0x3a30cb18, 33 },
    { 0, 0 },
    { 0x1af0cb1a, 98 },
    { 0xb840cb1b, 95 },
    { 0xbb60cb1c, 61 },
    { 0x32e0cb1d, 76 },
    { 0, 0 },
    { 0x4690cb1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d0cb22, 87 },
    { 0x6910cb23, 85 },
    { 0x3cd0cb24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810cb29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10cb2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330cb32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0cb36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e0cb3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe590cb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c0cb41, 33 },
    { 0, 0 },
    { 0x8840cb43, 61 },
    { 0xf0b0cb43, 55 },
    { 0xf30cb45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6900cb48, 98 },
    { 0, 0 },
    { 0xd760cb4a, 61 },
    { 0, 0 },
    { 0x16c0cb4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2640cb52, 95 },
    { 0xd360cb53, 90 },
    { 0x6350cb53, 95 },
    { 0, 0 },
    { 0x1150cb56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf0cb5a, 65 },
    { 0, 0 },
    { 0xc7f0cb5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1330cb60, 55 },
    { 0x4290cb61, 34 },
    { 0x4080cb60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860cb67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb890cb6b, 95 },
    { 0xf4e0cb6b, 33 },
    { 0xf230cb6c, 98 },
    { 0x87e0cb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570cb71, 45 },
    { 0x9ca0cb72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cb76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9870cb79, 23 },
    { 0, 0 },
    { 0x9590cb7b, 90 },
    { 0xb610cb7c, 78 },
    { 0x2470cb7d, 76 },
    { 0x4140cb7b, 95 },
    { 0x4070cb7f, 61 },
    { 0x20f0cb7c, 96 },
    { 0x2d60cb81, 23 },
    { 0, 0 },
    { 0x59c0cb83, 90 },
    { 0x2410cb83, 91 },
    { 0xb990cb83, 22 },
    { 0xe8a0cb86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0cb8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150cb93, 88 },
    { 0x5130cb93, 33 },
    { 0x4e10cb95, 93 },
    { 0x2d40cb96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0cb9c, 61 },
    { 0xaa30cb9d, 95 },
    { 0, 0 },
    { 0xe450cb9f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d70cba3, 90 },
    { 0x7e80cba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e0cba7, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0cbab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x300cbae, 98 },
    { 0x49c0cbaf, 75 },
    { 0, 0 },
    { 0x72b0cbb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d0cbb4, 61 },
    { 0x4a00cbb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3270cbba, 98 },
    { 0x7c0cbbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80cbbf, 61 },
    { 0x5170cbbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b0cbce, 52 },
    { 0xa290cbcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0cbd3, 93 },
    { 0x5b40cbd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c0cbdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a0cbe1, 85 },
    { 0xb9f0cbe2, 47 },
    { 0x23c0cbe3, 95 },
    { 0x31c0cbe2, 33 },
    { 0x7b60cbe1, 81 },
    { 0xcc20cbe6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150cbeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e50cbf2, 98 },
    { 0, 0 },
    { 0x76a0cbf4, 95 },
    { 0, 0 },
    { 0xd5a0cbf6, 61 },
    { 0x6340cbf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb0cbfc, 76 },
    { 0x26c0cbfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80cc00, 98 },
    { 0, 0 },
    { 0xdef0cc02, 33 },
    { 0xe4e0cc03, 88 },
    { 0x5890cc04, 98 },
    { 0xae50cc03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb0cc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80cc19, 33 },
    { 0xc670cc19, 46 },
    { 0x18f0cc1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e50cc1e, 98 },
    { 0xdc70cc1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20cc24, 90 },
    { 0, 0 },
    { 0x5890cc26, 46 },
    { 0x7670cc27, 95 },
    { 0x4be0cc26, 68 },
    { 0xc820cc29, 33 },
    { 0, 0 },
    { 0x5a10cc2b, 77 },
    { 0xe3d0cc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50cc2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc290cc33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970cc36, 90 },
    { 0x4a40cc37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f70cc3c, 81 },
    { 0x7b60cc3c, 23 },
    { 0x3900cc3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0cc41, 43 },
    { 0x6ed0cc41, 84 },
    { 0, 0 },
    { 0xdc00cc44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b0cc47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630cc4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410cc52, 98 },
    { 0x9100cc53, 90 },
    { 0x7f00cc53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf000cc59, 98 },
    { 0x5f00cc59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc90cc5e, 33 },
    { 0x8bd0cc5e, 52 },
    { 0xdf40cc60, 87 },
    { 0xd030cc61, 23 },
    { 0x320cc62, 33 },
    { 0, 0 },
    { 0x1fd0cc64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b0cc68, 90 },
    { 0xdb00cc69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa80cc77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250cc7b, 46 },
    { 0xe320cc7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40cc85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e0cc88, 36 },
    { 0xb640cc89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640cc8c, 34 },
    { 0, 0 },
    { 0x2970cc8e, 33 },
    { 0xe00cc8f, 33 },
    { 0, 0 },
    { 0xdc30cc91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0cc94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e0cc97, 33 },
    { 0, 0 },
    { 0x4b00cc99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9330cc9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0cca1, 61 },
    { 0, 0 },
    { 0x7010cca3, 61 },
    { 0xd100cca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2090cca9, 81 },
    { 0xa910cca9, 61 },
    { 0xb0b0ccab, 90 },
    { 0, 0 },
    { 0xc240ccad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0ccb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8660ccb7, 95 },
    { 0x7590ccb8, 95 },
    { 0, 0 },
    { 0x9fe0ccba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe60ccc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d0ccc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f0ccc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2680ccd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2740ccd8, 84 },
    { 0x5a60ccd9, 23 },
    { 0, 0 },
    { 0xea10ccdb, 34 },
    { 0x83b0ccdc, 90 },
    { 0, 0 },
    { 0x8ac0ccde, 98 },
    { 0, 0 },
    { 0xc2f0cce0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3240cce3, 87 },
    { 0, 0 },
    { 0xaab0cce5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1700ccf2, 23 },
    { 0xf9d0ccf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff0ccff, 69 },
    { 0xd9c0ccff, 87 },
    { 0, 0 },
    { 0x4e0cd02, 98 },
    { 0x8030cd02, 52 },
    { 0xc540cd04, 61 },
    { 0xa0cd04, 90 },
    { 0xb0e0cd06, 95 },
    { 0x9a30cd07, 98 },
    { 0, 0 },
    { 0xb510cd09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc830cd0c, 23 },
    { 0xd7a0cd0d, 61 },
    { 0x9eb0cd0e, 33 },
    { 0x3480cd0f, 90 },
    { 0x39a0cd10, 33 },
    { 0xc3f0cd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c0cd18, 53 },
    { 0x62f0cd18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b0cd1e, 43 },
    { 0xc40cd1f, 23 },
    { 0x3810cd1f, 81 },
    { 0x99d0cd21, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0cd24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e10cd29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0cd31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d30cd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930cd39, 68 },
    { 0x5430cd39, 55 },
    { 0, 0 },
    { 0xb0a0cd3c, 90 },
    { 0, 0 },
    { 0xf470cd3e, 33 },
    { 0x40f0cd3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10cd4a, 91 },
    { 0x71a0cd4a, 90 },
    { 0x6020cd4c, 95 },
    { 0x9ae0cd4d, 93 },
    { 0xc710cd4e, 90 },
    { 0xe2b0cd4f, 61 },
    { 0x41a0cd4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb20cd53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320cd57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee0cd5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0cd5e, 90 },
    { 0, 0 },
    { 0x5410cd60, 93 },
    { 0x6b0cd60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8070cd64, 33 },
    { 0xc010cd65, 61 },
    { 0xd030cd66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d0cd6b, 90 },
    { 0x7180cd6c, 98 },
    { 0x9a20cd6c, 98 },
    { 0xeb40cd6c, 61 },
    { 0xdfe0cd6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce30cd79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c30cd7d, 99 },
    { 0xab0cd7e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0cd82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4970cd86, 66 },
    { 0, 0 },
    { 0x1a20cd88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7700cd8b, 46 },
    { 0x1a10cd8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb260cd8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef70cd93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd40cd96, 99 },
    { 0x1d10cd97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8850cd9c, 99 },
    { 0x9740cd9c, 23 },
    { 0x89e0cd9e, 76 },
    { 0x51b0cd9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330cda3, 76 },
    { 0xcd20cda4, 33 },
    { 0x1150cda4, 98 },
    { 0x3ec0cda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5740cda9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6000cdac, 90 },
    { 0xb960cdad, 61 },
    { 0xce00cdae, 90 },
    { 0xd3d0cdaf, 93 },
    { 0, 0 },
    { 0x9bd0cdb1, 61 },
    { 0x1520cdb2, 80 },
    { 0xace0cdb3, 90 },
    { 0xa410cdb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0cdb9, 90 },
    { 0x3190cdba, 61 },
    { 0xf350cdbb, 98 },
    { 0, 0 },
    { 0xbdd0cdbd, 95 },
    { 0x87b0cdbd, 61 },
    { 0, 0 },
    { 0xddd0cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990cdc4, 76 },
    { 0x4600cdc4, 77 },
    { 0xd930cdc5, 33 },
    { 0x1b70cdc6, 43 },
    { 0, 0 },
    { 0xc460cdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd120cdce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c0cdd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3120cdd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c80cddd, 87 },
    { 0xcae0cdde, 91 },
    { 0x6420cddd, 47 },
    { 0x4b0cddd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4750cde5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd0cdee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e0cdf3, 75 },
    { 0, 0 },
    { 0x1a60cdf5, 84 },
    { 0, 0 },
    { 0xe720cdf7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa950cdfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e0cdfe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850ce04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0ce0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0ce13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9690ce1b, 61 },
    { 0x5f30ce1c, 81 },
    { 0xad90ce1d, 93 },
    { 0x9e60ce1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b0ce21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0ce27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db0ce2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e0ce32, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b70ce3a, 90 },
    { 0xa0b0ce3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280ce40, 55 },
    { 0x1970ce41, 43 },
    { 0x850ce41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc0ce45, 23 },
    { 0xeb20ce46, 33 },
    { 0, 0 },
    { 0x52e0ce48, 98 },
    { 0x7e0ce48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c90ce50, 33 },
    { 0, 0 },
    { 0xa430ce52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad0ce56, 23 },
    { 0, 0 },
    { 0x7430ce58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8710ce5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc0ce60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c00ce63, 98 },
    { 0, 0 },
    { 0x1f20ce65, 61 },
    { 0x3100ce66, 23 },
    { 0xb940ce66, 34 },
    { 0, 0 },
    { 0x74e0ce69, 22 },
    { 0x4ae0ce69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad90ce6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a30ce71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f0ce77, 98 },
    { 0x1350ce78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a70ce7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a0ce81, 95 },
    { 0x2a50ce82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990ce86, 98 },
    { 0xedb0ce87, 61 },
    { 0, 0 },
    { 0x54d0ce89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190ce8d, 77 },
    { 0x8f0ce8e, 46 },
    { 0, 0 },
    { 0x3170ce90, 68 },
    { 0x2490ce91, 90 },
    { 0xa2a0ce90, 76 },
    { 0, 0 },
    { 0x20a0ce94, 61 },
    { 0x9350ce94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba0ce9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff90cea3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6930cea6, 33 },
    { 0xb270cea6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ceac, 23 },
    { 0xfae0cead, 67 },
    { 0x4f20cead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440ceb6, 78 },
    { 0, 0 },
    { 0x9c80ceb8, 99 },
    { 0, 0 },
    { 0xa900ceba, 33 },
    { 0xcfa0cebb, 23 },
    { 0xb940ceba, 95 },
    { 0x70cebd, 91 },
    { 0, 0 },
    { 0xc8c0cebf, 52 },
    { 0xfb90cec0, 43 },
    { 0xc980cebf, 55 },
    { 0xae70cec2, 90 },
    { 0, 0 },
    { 0xab00cec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70cecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e0ced4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe460cedb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390cee2, 23 },
    { 0, 0 },
    { 0x43e0cee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020ceed, 76 },
    { 0, 0 },
    { 0x5970ceef, 33 },
    { 0, 0 },
    { 0xa170cef1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd0cef7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf650cefe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca0cf03, 23 },
    { 0xbde0cf04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a0cf08, 90 },
    { 0x8fb0cf08, 91 },
    { 0xdbf0cf08, 90 },
    { 0xbde0cf0b, 55 },
    { 0x5ad0cf0a, 93 },
    { 0xab20cf0d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00cf11, 98 },
    { 0, 0 },
    { 0x5150cf13, 66 },
    { 0x9860cf14, 46 },
    { 0xcba0cf15, 98 },
    { 0xe360cf15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f40cf1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb0cf24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc0cf28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde10cf2f, 87 },
    { 0, 0 },
    { 0x93a0cf31, 67 },
    { 0, 0 },
    { 0xdaf0cf33, 90 },
    { 0xe8c0cf34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d40cf37, 95 },
    { 0x9c80cf38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0cf3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0cf40, 87 },
    { 0, 0 },
    { 0x8de0cf42, 87 },
    { 0x39d0cf42, 23 },
    { 0, 0 },
    { 0xc080cf45, 33 },
    { 0, 0 },
    { 0x6070cf47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630cf4a, 95 },
    { 0x7760cf4b, 90 },
    { 0, 0 },
    { 0x3c50cf4d, 76 },
    { 0xad70cf4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f0cf52, 33 },
    { 0, 0 },
    { 0x8460cf54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa840cf59, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b20cf5d, 98 },
    { 0x45a0cf5e, 90 },
    { 0x9a40cf5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0cf62, 85 },
    { 0xf950cf63, 80 },
    { 0x50cf64, 95 },
    { 0, 0 },
    { 0x75d0cf66, 75 },
    { 0x9a10cf66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0cf6a, 90 },
    { 0xef30cf6a, 55 },
    { 0, 0 },
    { 0x6ec0cf6d, 81 },
    { 0xf610cf6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20cf71, 23 },
    { 0x9140cf72, 23 },
    { 0x47f0cf71, 55 },
    { 0x8a30cf74, 95 },
    { 0, 0 },
    { 0xc8c0cf76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810cf7a, 33 },
    { 0, 0 },
    { 0x1f0cf7c, 99 },
    { 0xdf0cf7d, 90 },
    { 0x92e0cf7e, 95 },
    { 0xe820cf7e, 76 },
    { 0x9840cf80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600cf83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc10cf88, 91 },
    { 0xcef0cf88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5450cf8c, 33 },
    { 0xb4a0cf8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b0cf93, 55 },
    { 0x8cb0cf94, 77 },
    { 0x61a0cf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6140cf9c, 99 },
    { 0x80cf9d, 33 },
    { 0x45e0cf9d, 33 },
    { 0x5e10cf9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b0cfa4, 93 },
    { 0x3e20cfa4, 33 },
    { 0xf2b0cfa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50cfad, 33 },
    { 0x2dd0cfae, 46 },
    { 0x3a0cfad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20cfb2, 52 },
    { 0xcb90cfb3, 61 },
    { 0xb6c0cfb4, 61 },
    { 0, 0 },
    { 0x9f30cfb6, 87 },
    { 0x52d0cfb7, 100 },
    { 0xaae0cfb8, 52 },
    { 0xc170cfb9, 82 },
    { 0xcf20cfb7, 90 },
    { 0, 0 },
    { 0xb3c0cfbc, 95 },
    { 0x6c90cfbc, 23 },
    { 0, 0 },
    { 0x20d0cfbf, 99 },
    { 0xa820cfc0, 90 },
    { 0xec20cfc1, 33 },
    { 0x2380cfc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6500cfc5, 55 },
    { 0, 0 },
    { 0x8220cfc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50cfcf, 66 },
    { 0x4530cfd0, 46 },
    { 0, 0 },
    { 0xdd40cfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50cfd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10cfd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f0cfdd, 98 },
    { 0, 0 },
    { 0x8c10cfdf, 76 },
    { 0x16a0cfe0, 46 },
    { 0xca10cfe1, 46 },
    { 0, 0 },
    { 0x25f0cfe3, 65 },
    { 0xfeb0cfe4, 98 },
    { 0x7bf0cfe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80cfea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70cfed, 34 },
    { 0x51e0cfed, 33 },
    { 0, 0 },
    { 0x3cc0cff0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b0cff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710cff8, 76 },
    { 0x5230cff9, 95 },
    { 0xd410cffa, 85 },
    { 0x2da0cffb, 90 },
    { 0x5d00cffc, 23 },
    { 0x3ce0cffb, 73 },
    { 0x1950cff8, 76 },
    { 0xda50cffb, 66 },
    { 0x78d0cffa, 67 },
    { 0x8db0d001, 33 },
    { 0x66b0cffb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a20d007, 100 },
    { 0, 0 },
    { 0x2180d009, 61 },
    { 0, 0 },
    { 0x1560d00b, 90 },
    { 0x1600d00c, 66 },
    { 0x4510d00d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c0d01b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc0d01f, 61 },
    { 0, 0 },
    { 0x76e0d021, 33 },
    { 0xf4e0d022, 68 },
    { 0xb1d0d021, 43 },
    { 0, 0 },
    { 0x2670d025, 88 },
    { 0x8df0d026, 100 },
    { 0xd2c0d025, 95 },
    { 0xc5e0d025, 20 },
    { 0xb200d029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c50d030, 33 },
    { 0x9360d031, 69 },
    { 0xc1b0d030, 90 },
    { 0x2150d033, 46 },
    { 0xce30d032, 34 },
    { 0, 0 },
    { 0x9420d036, 43 },
    { 0x4c30d036, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd20d03b, 33 },
    { 0x95b0d03c, 34 },
    { 0x74f0d03b, 33 },
    { 0xa150d03e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca30d042, 61 },
    { 0xa610d043, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9540d048, 99 },
    { 0x7630d049, 34 },
    { 0, 0 },
    { 0x4be0d04b, 61 },
    { 0, 0 },
    { 0xa440d04d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b0d052, 61 },
    { 0xba40d052, 34 },
    { 0xeea0d054, 75 },
    { 0, 0 },
    { 0x5830d056, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c0d05c, 61 },
    { 0x3e50d05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60d060, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730d065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf30d068, 98 },
    { 0, 0 },
    { 0xd8d0d06a, 90 },
    { 0xf620d06a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6490d070, 95 },
    { 0, 0 },
    { 0x6f60d072, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0d080, 95 },
    { 0x3080d081, 95 },
    { 0x7d60d082, 98 },
    { 0, 0 },
    { 0x9f20d084, 23 },
    { 0x9210d084, 40 },
    { 0, 0 },
    { 0x8290d087, 23 },
    { 0xbab0d088, 90 },
    { 0x38e0d087, 61 },
    { 0x2310d08a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d08d, 33 },
    { 0, 0 },
    { 0x9270d08f, 23 },
    { 0xd310d08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30d093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000d097, 95 },
    { 0x8fc0d098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe80d09b, 23 },
    { 0x530d09b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde30d0a0, 23 },
    { 0x3980d0a0, 61 },
    { 0, 0 },
    { 0x6f10d0a3, 46 },
    { 0x23b0d0a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390d0a8, 33 },
    { 0x17c0d0a9, 91 },
    { 0, 0 },
    { 0xe6c0d0ab, 76 },
    { 0, 0 },
    { 0x4b70d0ad, 90 },
    { 0, 0 },
    { 0x9490d0af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40d0b2, 98 },
    { 0, 0 },
    { 0x9840d0b4, 87 },
    { 0x8fd0d0b5, 52 },
    { 0, 0 },
    { 0xcfb0d0b7, 90 },
    { 0xcec0d0b8, 33 },
    { 0, 0 },
    { 0x86c0d0ba, 61 },
    { 0, 0 },
    { 0xb800d0bc, 90 },
    { 0x7870d0bd, 78 },
    { 0xfcb0d0be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0d0c1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf050d0c4, 77 },
    { 0xf2d0d0c5, 61 },
    { 0xdad0d0c6, 98 },
    { 0x4b90d0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400d0cf, 98 },
    { 0x5260d0d0, 61 },
    { 0xafc0d0d1, 85 },
    { 0x6c90d0d2, 88 },
    { 0xeb40d0d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540d0d9, 43 },
    { 0x5ad0d0d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec0d0e0, 34 },
    { 0x1430d0e1, 61 },
    { 0xcd90d0e1, 64 },
    { 0, 0 },
    { 0x97a0d0e4, 87 },
    { 0x4ff0d0e5, 90 },
    { 0xa440d0e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d00d0e9, 34 },
    { 0xaff0d0e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da0d0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc0d0fa, 95 },
    { 0xcbf0d0fb, 33 },
    { 0x1b80d0fc, 23 },
    { 0xd9c0d0fc, 90 },
    { 0, 0 },
    { 0x890d0ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e70d108, 23 },
    { 0xb640d109, 98 },
    { 0xd200d10a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20d10d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570d117, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c0d11a, 95 },
    { 0x13c0d11b, 90 },
    { 0x4aa0d11c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd550d11f, 66 },
    { 0x98c0d120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc20d127, 100 },
    { 0x9580d128, 34 },
    { 0, 0 },
    { 0x4e00d12a, 61 },
    { 0xfb0d12b, 90 },
    { 0xcf20d12c, 61 },
    { 0x2510d12b, 98 },
    { 0x4ba0d12e, 83 },
    { 0, 0 },
    { 0xd6e0d130, 76 },
    { 0x1d40d131, 98 },
    { 0x2320d132, 55 },
    { 0x3890d133, 95 },
    { 0x19a0d133, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5790d138, 95 },
    { 0, 0 },
    { 0x7ab0d13a, 61 },
    { 0x8ed0d13b, 90 },
    { 0x9020d13b, 33 },
    { 0x5080d13d, 67 },
    { 0x2b10d13e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe180d143, 36 },
    { 0x41d0d144, 38 },
    { 0xce0d145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00d148, 52 },
    { 0xf330d149, 33 },
    { 0, 0 },
    { 0xf9d0d14b, 77 },
    { 0, 0 },
    { 0x61d0d14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1820d151, 43 },
    { 0xe3b0d152, 23 },
    { 0, 0 },
    { 0x1830d154, 90 },
    { 0xd690d155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c20d158, 34 },
    { 0xf4a0d159, 55 },
    { 0x1cb0d158, 90 },
    { 0, 0 },
    { 0xee40d15c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0d163, 84 },
    { 0xae40d164, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc0d16c, 100 },
    { 0, 0 },
    { 0xef50d16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df0d176, 23 },
    { 0xfa90d176, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220d17c, 90 },
    { 0xcae0d17d, 98 },
    { 0xd870d17c, 100 },
    { 0xa350d17f, 46 },
    { 0, 0 },
    { 0xf350d181, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf20d184, 90 },
    { 0xcea0d184, 34 },
    { 0, 0 },
    { 0xda40d187, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0d18c, 56 },
    { 0x4610d18c, 53 },
    { 0x4ca0d18e, 33 },
    { 0x4bb0d18f, 90 },
    { 0xccc0d18c, 46 },
    { 0xad90d18f, 33 },
    { 0x2250d192, 61 },
    { 0x1ac0d190, 98 },
    { 0x1350d18d, 61 },
    { 0, 0 },
    { 0x4810d196, 67 },
    { 0x8780d197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0d19a, 90 },
    { 0xd0a0d19a, 23 },
    { 0xd60d19a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750d1a5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0d1aa, 99 },
    { 0x6d20d1aa, 98 },
    { 0xec20d1ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a0d1af, 95 },
    { 0x1970d1b0, 61 },
    { 0x3350d1b0, 33 },
    { 0x6020d1b1, 85 },
    { 0xe310d1b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c0d1b6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb220d1bb, 34 },
    { 0, 0 },
    { 0x86a0d1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b0d1c2, 98 },
    { 0, 0 },
    { 0xd040d1c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3070d1c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6990d1cb, 90 },
    { 0, 0 },
    { 0xe3e0d1cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0d1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2830d1df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff60d1e3, 78 },
    { 0, 0 },
    { 0xc1b0d1e5, 84 },
    { 0x82b0d1e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece0d1e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c40d1ec, 95 },
    { 0x1ef0d1ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4290d1f1, 84 },
    { 0x6a60d1f2, 90 },
    { 0xbf10d1f3, 61 },
    { 0x22f0d1f2, 61 },
    { 0xe600d1f5, 84 },
    { 0x94c0d1f6, 33 },
    { 0, 0 },
    { 0x4880d1f8, 98 },
    { 0x78d0d1f8, 33 },
    { 0, 0 },
    { 0x6700d1fb, 89 },
    { 0x1090d1fb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10d200, 95 },
    { 0x3140d201, 33 },
    { 0xfc70d202, 98 },
    { 0, 0 },
    { 0x4890d204, 33 },
    { 0xf0c0d204, 87 },
    { 0x92e0d206, 43 },
    { 0xf480d204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70d20c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10d214, 78 },
    { 0x8bc0d215, 23 },
    { 0xcb70d216, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f0d21a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2320d223, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3650d226, 33 },
    { 0x9ed0d227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e0d22a, 93 },
    { 0x38f0d22b, 90 },
    { 0xb7a0d22b, 55 },
    { 0xbb00d22d, 61 },
    { 0xaba0d22e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa710d238, 76 },
    { 0x3070d239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b0d23d, 33 },
    { 0x3710d23e, 90 },
    { 0x70d23f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c0d243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0d248, 90 },
    { 0, 0 },
    { 0xf5b0d24a, 76 },
    { 0x77d0d24b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb850d250, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f0d259, 90 },
    { 0x6100d259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be0d260, 67 },
    { 0, 0 },
    { 0x5500d262, 100 },
    { 0xf3d0d263, 38 },
    { 0x5760d264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0d268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50d26b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6140d26f, 95 },
    { 0x15d0d270, 66 },
    { 0xb1a0d270, 69 },
    { 0x8e70d270, 95 },
    { 0x6470d270, 61 },
    { 0x71c0d274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0d277, 93 },
    { 0xacd0d278, 88 },
    { 0, 0 },
    { 0x7300d27a, 84 },
    { 0, 0 },
    { 0x200d27c, 98 },
    { 0xee60d27c, 75 },
    { 0x5870d27e, 23 },
    { 0, 0 },
    { 0x5840d280, 81 },
    { 0x9830d280, 96 },
    { 0xf9e0d282, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130d289, 90 },
    { 0xc000d289, 23 },
    { 0x3f20d28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe400d291, 33 },
    { 0, 0 },
    { 0x76a0d293, 100 },
    { 0xdd00d294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70d297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0d2a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210d2ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6500d2b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0d2ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe770d2bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba0d2c2, 34 },
    { 0x3980d2c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0d2c6, 90 },
    { 0x8e70d2c6, 95 },
    { 0, 0 },
    { 0xc00d2c9, 87 },
    { 0, 0 },
    { 0xfc30d2cb, 98 },
    { 0xab20d2cc, 80 },
    { 0x3130d2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30d2d6, 43 },
    { 0xf940d2d7, 55 },
    { 0x5f60d2d7, 90 },
    { 0x6ff0d2d9, 95 },
    { 0, 0 },
    { 0x4a50d2db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60d2de, 100 },
    { 0, 0 },
    { 0xf2e0d2e0, 99 },
    { 0x1dd0d2e0, 99 },
    { 0xb9f0d2e2, 61 },
    { 0xdb90d2e0, 75 },
    { 0x4f80d2e4, 55 },
    { 0xa280d2e4, 98 },
    { 0x8200d2e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7750d2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0d2f4, 81 },
    { 0xabc0d2f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0d2f9, 98 },
    { 0, 0 },
    { 0x45f0d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0d300, 77 },
    { 0x5170d301, 61 },
    { 0x7530d301, 61 },
    { 0x9e30d302, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a00d306, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6930d30a, 98 },
    { 0, 0 },
    { 0x4310d30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b0d312, 76 },
    { 0xa6f0d313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c0d318, 69 },
    { 0x9ce0d318, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500d31e, 84 },
    { 0x9dd0d31f, 68 },
    { 0x9a20d31e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400d324, 95 },
    { 0, 0 },
    { 0xabb0d326, 98 },
    { 0, 0 },
    { 0x1440d328, 95 },
    { 0, 0 },
    { 0x1e60d32a, 93 },
    { 0xa130d32a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350d32e, 98 },
    { 0x1440d32f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0d334, 33 },
    { 0, 0 },
    { 0xba90d336, 98 },
    { 0xc120d336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe0d340, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780d343, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf380d348, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec0d34c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d80d350, 23 },
    { 0, 0 },
    { 0x6a70d352, 33 },
    { 0x4150d352, 23 },
    { 0, 0 },
    { 0x7fc0d355, 90 },
    { 0, 0 },
    { 0xb6f0d357, 93 },
    { 0, 0 },
    { 0xd60d359, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920d360, 90 },
    { 0, 0 },
    { 0xcd40d362, 76 },
    { 0xd540d362, 34 },
    { 0x4b50d364, 65 },
    { 0xe860d363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b0d368, 99 },
    { 0x1bc0d369, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050d36d, 98 },
    { 0x4030d36e, 23 },
    { 0x5100d36e, 90 },
    { 0x6600d36f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180d373, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10d378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa820d37d, 98 },
    { 0x100d37d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580d381, 55 },
    { 0, 0 },
    { 0x3360d383, 55 },
    { 0x1820d384, 33 },
    { 0, 0 },
    { 0x9370d386, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740d389, 90 },
    { 0x93f0d38a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060d38d, 93 },
    { 0x6be0d38d, 55 },
    { 0x7830d38f, 90 },
    { 0xe8b0d390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570d394, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3820d397, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970d39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020d3a2, 40 },
    { 0x5aa0d3a2, 98 },
    { 0xdfe0d3a4, 46 },
    { 0, 0 },
    { 0x77b0d3a6, 90 },
    { 0xcc30d3a7, 98 },
    { 0xb3c0d3a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a0d3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730d3b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570d3b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3620d3b7, 90 },
    { 0x2e00d3b8, 61 },
    { 0, 0 },
    { 0x3a60d3ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0d3bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0d3c4, 98 },
    { 0x77b0d3c4, 67 },
    { 0x9ae0d3c6, 17 },
    { 0xd840d3c7, 76 },
    { 0xb590d3c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc420d3cf, 23 },
    { 0xf3b0d3cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480d3d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0d3d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf660d3dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0d3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf0d3e3, 64 },
    { 0, 0 },
    { 0x3c80d3e5, 89 },
    { 0x8ac0d3e6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7370d3ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf510d3ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90d3f3, 95 },
    { 0, 0 },
    { 0xec10d3f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af0d3f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490d3fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc120d3ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0d403, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5640d407, 66 },
    { 0xc0e0d407, 33 },
    { 0x68e0d409, 52 },
    { 0xeff0d409, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab40d40e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3230d412, 40 },
    { 0xb860d412, 61 },
    { 0x1a90d413, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b0d41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf880d424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f0d427, 61 },
    { 0, 0 },
    { 0x6900d429, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xef30d42c, 23 },
    { 0x9d20d42d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3000d435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80d440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db0d445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b50d448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4840d44d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb00d455, 55 },
    { 0x5370d456, 46 },
    { 0xc3e0d456, 61 },
    { 0x1630d456, 77 },
    { 0x3ee0d457, 100 },
    { 0, 0 },
    { 0x8a10d45b, 43 },
    { 0x3100d45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b0d463, 68 },
    { 0, 0 },
    { 0x89b0d465, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc430d471, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e40d477, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f0d47a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4220d480, 55 },
    { 0, 0 },
    { 0x8190d482, 23 },
    { 0x5b80d483, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0d486, 23 },
    { 0, 0 },
    { 0x72e0d488, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50d48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb0d48f, 33 },
    { 0x8410d490, 55 },
    { 0, 0 },
    { 0x7730d492, 46 },
    { 0, 0 },
    { 0xca30d494, 61 },
    { 0x9c50d495, 33 },
    { 0x82a0d495, 43 },
    { 0x1650d497, 77 },
    { 0, 0 },
    { 0xed00d499, 61 },
    { 0x6c50d49a, 23 },
    { 0xbfa0d49b, 78 },
    { 0xb180d49a, 61 },
    { 0, 0 },
    { 0x4d30d49e, 91 },
    { 0x1aa0d49f, 52 },
    { 0, 0 },
    { 0x8700d4a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce0d4a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9220d4a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0d4ac, 98 },
    { 0x6390d4ac, 95 },
    { 0, 0 },
    { 0x16b0d4af, 61 },
    { 0xaf40d4af, 53 },
    { 0x5a50d4b0, 34 },
    { 0xb4b0d4b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff0d4b7, 53 },
    { 0, 0 },
    { 0x8e00d4b9, 65 },
    { 0x3000d4b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc380d4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae0d4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6320d4c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc0d4ca, 88 },
    { 0, 0 },
    { 0x39d0d4cc, 46 },
    { 0, 0 },
    { 0xc30d4ce, 61 },
    { 0x8500d4cf, 98 },
    { 0, 0 },
    { 0x3040d4d1, 90 },
    { 0, 0 },
    { 0x21d0d4d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b80d4d6, 23 },
    { 0x15b0d4d7, 76 },
    { 0xf3f0d4d8, 33 },
    { 0x9440d4d9, 100 },
    { 0x7440d4d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb0d4e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b0d4e6, 61 },
    { 0, 0 },
    { 0x3930d4e8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d50d4eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0d4ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d0d4f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960d4fa, 46 },
    { 0x9680d4fb, 23 },
    { 0xf410d4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f60d501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000d504, 88 },
    { 0x78e0d505, 90 },
    { 0x86b0d506, 23 },
    { 0x4e60d507, 90 },
    { 0, 0 },
    { 0xe230d509, 87 },
    { 0x1350d50a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d50f, 76 },
    { 0xde90d50f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb340d515, 61 },
    { 0xa100d516, 61 },
    { 0, 0 },
    { 0xa6f0d518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9850d51b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e0d51e, 64 },
    { 0xc760d51e, 68 },
    { 0x7f00d520, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9890d52d, 33 },
    { 0x6670d52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570d532, 33 },
    { 0xcb10d533, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f0d537, 67 },
    { 0, 0 },
    { 0xc690d539, 23 },
    { 0x5550d53a, 52 },
    { 0x2600d53b, 60 },
    { 0, 0 },
    { 0x9430d53d, 90 },
    { 0x7950d53d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0d543, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa60d546, 61 },
    { 0x2850d547, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe230d54d, 95 },
    { 0x3a00d54e, 90 },
    { 0x23a0d54f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d552, 84 },
    { 0xcab0d553, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0d556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50d559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790d55e, 23 },
    { 0x26b0d55f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7340d564, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d0d569, 90 },
    { 0, 0 },
    { 0x4db0d56b, 98 },
    { 0x400d56b, 95 },
    { 0, 0 },
    { 0x9500d56e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb400d572, 76 },
    { 0x5460d573, 90 },
    { 0, 0 },
    { 0x8670d575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e0d579, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0d57d, 33 },
    { 0x86f0d57e, 61 },
    { 0x77a0d57f, 43 },
    { 0xb970d580, 55 },
    { 0, 0 },
    { 0x3f20d582, 98 },
    { 0xf10d582, 33 },
    { 0xd00d584, 85 },
    { 0x9650d583, 93 },
    { 0, 0 },
    { 0x6b20d587, 90 },
    { 0x9300d588, 91 },
    { 0, 0 },
    { 0xee60d58a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0d58e, 81 },
    { 0, 0 },
    { 0x6360d590, 55 },
    { 0xa3b0d591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb460d594, 23 },
    { 0, 0 },
    { 0xbb00d596, 76 },
    { 0x770d597, 91 },
    { 0xca80d598, 98 },
    { 0, 0 },
    { 0x3440d59a, 76 },
    { 0xa160d59b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510d59e, 43 },
    { 0x30a0d59e, 95 },
    { 0, 0 },
    { 0xc0f0d5a1, 55 },
    { 0, 0 },
    { 0xf5b0d5a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa470d5a6, 55 },
    { 0, 0 },
    { 0xa4e0d5a8, 90 },
    { 0xf4b0d5a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed30d5ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f20d5b6, 90 },
    { 0xd750d5b7, 33 },
    { 0, 0 },
    { 0x6680d5b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780d5bc, 78 },
    { 0x7090d5bd, 78 },
    { 0x7200d5bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40d5c1, 61 },
    { 0x59f0d5c2, 68 },
    { 0x27d0d5c2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e0d5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0d5c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0d5d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520d5d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9140d5db, 95 },
    { 0xb8f0d5dc, 90 },
    { 0, 0 },
    { 0x2e80d5de, 61 },
    { 0x8f50d5de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7940d5e3, 100 },
    { 0x2850d5e4, 33 },
    { 0, 0 },
    { 0x8a80d5e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10d5ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c0d5f1, 95 },
    { 0xaf20d5f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e0d5fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0d5fd, 69 },
    { 0x8f40d5fe, 61 },
    { 0, 0 },
    { 0xacd0d600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f0d603, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b30d607, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0d60a, 98 },
    { 0x67a0d60b, 99 },
    { 0x380d60c, 81 },
    { 0x8e80d60b, 95 },
    { 0x9030d60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa70d611, 98 },
    { 0x8b10d612, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0d617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00d61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0d61d, 96 },
    { 0xe7e0d61e, 61 },
    { 0x1d80d61f, 40 },
    { 0x7ae0d620, 33 },
    { 0x2ae0d621, 23 },
    { 0xeb0d621, 98 },
    { 0x7870d621, 76 },
    { 0xab70d623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510d628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710d62b, 33 },
    { 0xee50d62c, 96 },
    { 0xb4d0d62b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20d631, 76 },
    { 0x73e0d632, 61 },
    { 0xce70d633, 61 },
    { 0x8560d634, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0d639, 90 },
    { 0x2f80d639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0d640, 95 },
    { 0x5ed0d641, 90 },
    { 0, 0 },
    { 0x84c0d643, 67 },
    { 0x90d643, 55 },
    { 0x370d643, 55 },
    { 0x6c20d645, 61 },
    { 0xfbe0d647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda0d650, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560d654, 93 },
    { 0xadb0d655, 91 },
    { 0xc5e0d655, 84 },
    { 0, 0 },
    { 0x3b90d658, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab0d661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f20d672, 95 },
    { 0x7840d672, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00d677, 95 },
    { 0x1560d678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca0d67b, 61 },
    { 0, 0 },
    { 0x53f0d67d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c0d682, 55 },
    { 0, 0 },
    { 0xd0e0d684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6580d687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0d68a, 33 },
    { 0xa7f0d68b, 90 },
    { 0xa310d68c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc890d691, 60 },
    { 0x6730d691, 42 },
    { 0x580d692, 23 },
    { 0x2710d694, 23 },
    { 0, 0 },
    { 0x2de0d696, 91 },
    { 0x4ff0d696, 33 },
    { 0xb0d0d698, 34 },
    { 0, 0 },
    { 0x9790d69a, 95 },
    { 0, 0 },
    { 0x3190d69c, 33 },
    { 0xcd20d69d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1040d6a3, 95 },
    { 0, 0 },
    { 0x5f00d6a5, 17 },
    { 0x68c0d6a6, 43 },
    { 0xabf0d6a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70d6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0d6ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0d6b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc0d6b3, 95 },
    { 0, 0 },
    { 0x370d6b5, 98 },
    { 0, 0 },
    { 0x90d6b7, 52 },
    { 0x1300d6b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc510d6bb, 43 },
    { 0xef40d6bc, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0d6c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970d6cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe060d6cf, 95 },
    { 0, 0 },
    { 0x9640d6d1, 66 },
    { 0x8ff0d6d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc170d6db, 95 },
    { 0xfc70d6dc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f90d6e0, 23 },
    { 0xcb20d6e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x590d6e4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990d6e7, 23 },
    { 0, 0 },
    { 0x7de0d6e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a10d6ec, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6730d6f1, 77 },
    { 0xdde0d6f2, 76 },
    { 0x78c0d6f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad0d6f6, 33 },
    { 0, 0 },
    { 0x55e0d6f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c0d6fd, 33 },
    { 0x6960d6fe, 100 },
    { 0, 0 },
    { 0x23d0d700, 61 },
    { 0x9820d700, 84 },
    { 0xfa40d702, 90 },
    { 0x4ff0d703, 46 },
    { 0x8140d700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc450d708, 23 },
    { 0x20f0d708, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe370d70e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc0d717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7090d71a, 33 },
    { 0xf800d71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e60d71e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700d723, 76 },
    { 0, 0 },
    { 0xd760d725, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f40d729, 61 },
    { 0xf0a0d729, 61 },
    { 0x56b0d72a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a0d732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b60d736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a0d739, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd0d742, 34 },
    { 0xd2c0d742, 98 },
    { 0, 0 },
    { 0xd120d745, 84 },
    { 0xdf40d746, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0d74b, 87 },
    { 0xc5d0d74b, 78 },
    { 0xc90d74d, 93 },
    { 0, 0 },
    { 0x5b80d74f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00d753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad00d756, 61 },
    { 0xca30d756, 90 },
    { 0x5a30d758, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0d75b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a60d75e, 33 },
    { 0, 0 },
    { 0x9b80d760, 87 },
    { 0xcc0d761, 76 },
    { 0x9450d761, 100 },
    { 0xb530d763, 46 },
    { 0, 0 },
    { 0x43c0d765, 81 },
    { 0x6670d766, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1450d769, 24 },
    { 0x9690d76a, 76 },
    { 0, 0 },
    { 0x49d0d76c, 61 },
    { 0xa1e0d76d, 95 },
    { 0x84b0d76d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790d772, 68 },
    { 0, 0 },
    { 0xa590d774, 99 },
    { 0, 0 },
    { 0xcd20d776, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf850d77f, 90 },
    { 0x2970d77f, 33 },
    { 0x2d60d781, 23 },
    { 0x82a0d782, 55 },
    { 0x5a0d782, 98 },
    { 0xef00d781, 55 },
    { 0x1a0d785, 95 },
    { 0x7450d786, 77 },
    { 0xa270d785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0d78c, 88 },
    { 0xb5f0d78d, 23 },
    { 0, 0 },
    { 0x3a80d78f, 93 },
    { 0xd650d78f, 33 },
    { 0, 0 },
    { 0xfd80d792, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0d796, 34 },
    { 0x1b20d796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d80d79c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f0d79f, 90 },
    { 0, 0 },
    { 0x30c0d7a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb90d7a6, 95 },
    { 0, 0 },
    { 0x76a0d7a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260d7ac, 100 },
    { 0xd8d0d7ac, 23 },
    { 0xc6b0d7ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0d7b2, 23 },
    { 0xa10d7b2, 43 },
    { 0x7a30d7b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee0d7b8, 23 },
    { 0x6480d7b9, 33 },
    { 0x5c10d7ba, 33 },
    { 0xab90d7b8, 99 },
    { 0xa810d7bc, 34 },
    { 0, 0 },
    { 0xc30d7be, 33 },
    { 0, 0 },
    { 0x67c0d7c0, 68 },
    { 0xc340d7c1, 33 },
    { 0x66a0d7c1, 38 },
    { 0xa740d7c3, 61 },
    { 0, 0 },
    { 0x3a30d7c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae80d7cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870d7d1, 76 },
    { 0x7b70d7d2, 95 },
    { 0xfe70d7d3, 92 },
    { 0x7bb0d7d3, 33 },
    { 0xce40d7d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c0d7d8, 34 },
    { 0xde60d7d9, 33 },
    { 0x3210d7d9, 95 },
    { 0xa380d7d8, 76 },
    { 0x1f50d7dc, 33 },
    { 0xfc20d7da, 34 },
    { 0xa050d7d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a0d7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd350d7e8, 61 },
    { 0, 0 },
    { 0xe230d7ea, 23 },
    { 0, 0 },
    { 0xf130d7ec, 90 },
    { 0xaa50d7ed, 46 },
    { 0x6680d7ed, 55 },
    { 0x6b40d7ef, 84 },
    { 0xee20d7ef, 23 },
    { 0x3740d7f0, 95 },
    { 0x72f0d7f0, 53 },
    { 0x7530d7f3, 95 },
    { 0xe710d7ef, 95 },
    { 0, 0 },
    { 0x7090d7f6, 99 },
    { 0x950d7f7, 90 },
    { 0x1ba0d7f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb950d7fc, 90 },
    { 0x23c0d7fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7940d802, 23 },
    { 0x33c0d802, 33 },
    { 0x41a0d802, 61 },
    { 0xa440d805, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe30d808, 33 },
    { 0xf280d808, 66 },
    { 0xe200d80a, 34 },
    { 0x8650d80b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340d80e, 34 },
    { 0x56d0d80e, 33 },
    { 0xbfd0d80f, 95 },
    { 0x6db0d80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f0d814, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9920d817, 90 },
    { 0x4ef0d817, 65 },
    { 0x8b10d818, 90 },
    { 0, 0 },
    { 0x4950d81b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc800d81f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890d822, 73 },
    { 0x4c80d823, 98 },
    { 0x5920d822, 95 },
    { 0xaf90d822, 87 },
    { 0x2060d826, 76 },
    { 0xa460d822, 95 },
    { 0, 0 },
    { 0x3290d829, 23 },
    { 0xbe70d82a, 52 },
    { 0xa1d0d82b, 34 },
    { 0xab70d82c, 90 },
    { 0xf170d82d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc0d830, 98 },
    { 0xdd10d830, 98 },
    { 0x7a70d831, 90 },
    { 0, 0 },
    { 0x6160d834, 100 },
    { 0xbed0d835, 33 },
    { 0x1910d836, 61 },
    { 0x7aa0d836, 55 },
    { 0xbc60d838, 90 },
    { 0xb850d836, 53 },
    { 0, 0 },
    { 0x25d0d83b, 98 },
    { 0x56d0d83b, 98 },
    { 0x4e10d83d, 43 },
    { 0xc060d83b, 93 },
    { 0xbbb0d83e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7230d844, 93 },
    { 0, 0 },
    { 0xac90d846, 87 },
    { 0x1090d847, 43 },
    { 0, 0 },
    { 0xf470d849, 95 },
    { 0x1d50d849, 76 },
    { 0xa240d84b, 91 },
    { 0xbee0d84c, 83 },
    { 0xe50d84c, 88 },
    { 0x180d84e, 90 },
    { 0xa340d84e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7630d854, 90 },
    { 0x5490d855, 43 },
    { 0x73b0d856, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xd320d859, 83 },
    { 0xf8c0d85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80d85f, 98 },
    { 0x1ad0d85f, 95 },
    { 0x1990d861, 33 },
    { 0xab40d862, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80d865, 43 },
    { 0xaf50d865, 90 },
    { 0, 0 },
    { 0x5a30d868, 98 },
    { 0x99b0d868, 95 },
    { 0xb3a0d869, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840d86e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0d875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a0d87e, 78 },
    { 0xc570d87f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370d884, 76 },
    { 0xa260d885, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f0d889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef0d88c, 61 },
    { 0x4a70d88c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b0d894, 65 },
    { 0x1270d895, 55 },
    { 0x91c0d895, 90 },
    { 0, 0 },
    { 0x2100d898, 43 },
    { 0xea40d899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0d89f, 98 },
    { 0, 0 },
    { 0xcae0d8a1, 55 },
    { 0x78d0d8a2, 64 },
    { 0xcaf0d8a3, 61 },
    { 0, 0 },
    { 0x4680d8a5, 91 },
    { 0, 0 },
    { 0x2290d8a7, 63 },
    { 0, 0 },
    { 0xbc30d8a9, 61 },
    { 0xe4a0d8aa, 90 },
    { 0x8d80d8ab, 33 },
    { 0, 0 },
    { 0xa250d8ad, 95 },
    { 0, 0 },
    { 0xa060d8af, 95 },
    { 0, 0 },
    { 0x8390d8b1, 33 },
    { 0, 0 },
    { 0xa890d8b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60d8bb, 43 },
    { 0xa310d8bc, 84 },
    { 0xcb40d8bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd760d8c0, 76 },
    { 0, 0 },
    { 0xca20d8c2, 90 },
    { 0x6c70d8c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa480d8c6, 61 },
    { 0x5020d8c7, 87 },
    { 0xf790d8c7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0d8cb, 90 },
    { 0x1c50d8cc, 23 },
    { 0x5f20d8cc, 100 },
    { 0xb3c0d8cd, 90 },
    { 0x4e90d8cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf910d8d2, 61 },
    { 0, 0 },
    { 0xe7f0d8d4, 90 },
    { 0, 0 },
    { 0xe970d8d6, 95 },
    { 0xfc60d8d7, 76 },
    { 0xfca0d8d7, 100 },
    { 0, 0 },
    { 0xd610d8da, 90 },
    { 0x61d0d8db, 61 },
    { 0xaea0d8dc, 34 },
    { 0xa7e0d8dd, 33 },
    { 0, 0 },
    { 0xe50d8df, 95 },
    { 0x3550d8e0, 23 },
    { 0xa000d8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d0d8e5, 61 },
    { 0, 0 },
    { 0x8180d8e7, 52 },
    { 0, 0 },
    { 0xfff0d8e9, 33 },
    { 0x7a70d8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3500d8ed, 76 },
    { 0, 0 },
    { 0xf6f0d8ef, 90 },
    { 0, 0 },
    { 0x580d8f1, 61 },
    { 0xbf0d8f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1700d8f9, 90 },
    { 0xc7b0d8fa, 98 },
    { 0, 0 },
    { 0x5780d8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6300d902, 66 },
    { 0xfe50d902, 90 },
    { 0xf3a0d903, 95 },
    { 0, 0 },
    { 0x36b0d906, 55 },
    { 0x17d0d906, 33 },
    { 0xf2b0d907, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0d90d, 69 },
    { 0x7120d90e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f0d912, 90 },
    { 0, 0 },
    { 0x5600d914, 46 },
    { 0x25d0d915, 90 },
    { 0xa9b0d916, 46 },
    { 0x9230d917, 90 },
    { 0x56a0d918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750d91c, 23 },
    { 0xc0a0d91c, 98 },
    { 0x45e0d91e, 61 },
    { 0x1e80d91d, 46 },
    { 0x7e50d91c, 33 },
    { 0x5d30d921, 95 },
    { 0xe170d921, 33 },
    { 0, 0 },
    { 0x6f40d924, 39 },
    { 0xe250d925, 95 },
    { 0xe60d926, 98 },
    { 0, 0 },
    { 0x4ff0d928, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980d92b, 33 },
    { 0, 0 },
    { 0x1ed0d92d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a0d932, 84 },
    { 0x90b0d933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa270d937, 33 },
    { 0, 0 },
    { 0x7840d939, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f0d93e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00d943, 61 },
    { 0xf8a0d943, 23 },
    { 0, 0 },
    { 0x6750d946, 90 },
    { 0x1b20d947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0d94a, 84 },
    { 0, 0 },
    { 0x1980d94c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800d951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950d954, 95 },
    { 0, 0 },
    { 0xd960d956, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6970d959, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe50d95e, 61 },
    { 0x8ba0d95f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00d962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc0d967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70d96a, 99 },
    { 0x9a40d96a, 34 },
    { 0x4150d96c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00d970, 33 },
    { 0xba30d971, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670d97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9030d982, 33 },
    { 0xadc0d983, 61 },
    { 0x9fb0d984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000d987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4790d98b, 98 },
    { 0x5fd0d98b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5550d990, 78 },
    { 0x1df0d991, 90 },
    { 0x8500d991, 46 },
    { 0xb590d992, 61 },
    { 0xc0a0d994, 91 },
    { 0, 0 },
    { 0x23e0d996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50d99e, 90 },
    { 0xc50d99f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa950d9a5, 55 },
    { 0x43e0d9a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe100d9ad, 33 },
    { 0x57f0d9ae, 55 },
    { 0x15c0d9ae, 43 },
    { 0xfee0d9b0, 88 },
    { 0x9a0d9ad, 23 },
    { 0x22a0d9b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef0d9b5, 33 },
    { 0x760d9b6, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20d9bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360d9c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4080d9c4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe040d9c7, 90 },
    { 0, 0 },
    { 0xc430d9c9, 90 },
    { 0, 0 },
    { 0xd130d9cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1070d9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e0d9d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed10d9d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb370d9da, 46 },
    { 0xc810d9db, 78 },
    { 0xa210d9dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b20d9df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00d9e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca0d9e6, 98 },
    { 0x40a0d9e7, 78 },
    { 0x66f0d9e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340d9eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea30d9ef, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b0d9f2, 61 },
    { 0x7ca0d9f3, 95 },
    { 0, 0 },
    { 0x9250d9f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d9fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd420d9fd, 68 },
    { 0, 0 },
    { 0xdcb0d9ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10da04, 23 },
    { 0x640da04, 100 },
    { 0, 0 },
    { 0xd060da07, 90 },
    { 0, 0 },
    { 0xc520da09, 95 },
    { 0xf700da0a, 33 },
    { 0xc190da0b, 17 },
    { 0x15d0da0a, 95 },
    { 0xe480da0b, 90 },
    { 0x15a0da0e, 98 },
    { 0x4bb0da0f, 33 },
    { 0, 0 },
    { 0xac90da11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620da15, 91 },
    { 0, 0 },
    { 0x8290da17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00da1a, 95 },
    { 0xa8e0da1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850da22, 90 },
    { 0xd90da23, 61 },
    { 0x8f20da22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0da2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d30da31, 75 },
    { 0x5c10da31, 33 },
    { 0x9ad0da33, 46 },
    { 0xe110da31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf280da37, 76 },
    { 0xdc90da38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb50da3d, 33 },
    { 0, 0 },
    { 0x4940da3f, 23 },
    { 0xa790da3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790da44, 34 },
    { 0, 0 },
    { 0x6ca0da46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea0da49, 33 },
    { 0x3430da49, 93 },
    { 0x66d0da4b, 34 },
    { 0xf420da4c, 52 },
    { 0xfb30da4c, 81 },
    { 0xa200da4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe490da52, 34 },
    { 0, 0 },
    { 0xb070da54, 61 },
    { 0x5fe0da54, 95 },
    { 0xe270da56, 61 },
    { 0xc4c0da57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2270da5c, 66 },
    { 0, 0 },
    { 0x7270da5e, 23 },
    { 0xb3d0da5f, 95 },
    { 0x6780da60, 33 },
    { 0, 0 },
    { 0x6120da62, 61 },
    { 0xe7b0da63, 76 },
    { 0, 0 },
    { 0xa40da65, 23 },
    { 0x2560da66, 46 },
    { 0, 0 },
    { 0xca00da68, 95 },
    { 0x6580da69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a0da6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb10da7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c0da82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00da89, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d0da90, 43 },
    { 0, 0 },
    { 0xec0da92, 98 },
    { 0xe590da93, 98 },
    { 0xa0d0da93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc190da99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c0da9c, 49 },
    { 0x6320da9c, 55 },
    { 0xfa00da9c, 55 },
    { 0xd9e0da9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c0daa2, 61 },
    { 0, 0 },
    { 0xaf40daa4, 95 },
    { 0, 0 },
    { 0xdd20daa6, 95 },
    { 0, 0 },
    { 0x1a90daa8, 95 },
    { 0, 0 },
    { 0xa290daaa, 23 },
    { 0, 0 },
    { 0xd280daac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb0dabb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f30dabe, 95 },
    { 0, 0 },
    { 0x8470dac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60dac3, 90 },
    { 0, 0 },
    { 0xe680dac5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef40daca, 55 },
    { 0xe7c0dacb, 61 },
    { 0x70b0dacc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0dadb, 100 },
    { 0x2f70dadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe070dae1, 80 },
    { 0xd230dae2, 75 },
    { 0x7f00dae3, 33 },
    { 0, 0 },
    { 0x90dae5, 23 },
    { 0x5c30dae5, 79 },
    { 0x31e0dae7, 90 },
    { 0x83a0dae8, 91 },
    { 0, 0 },
    { 0x2f00daea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f0daef, 33 },
    { 0x5b80daf0, 80 },
    { 0x4770daf1, 76 },
    { 0x12a0daf2, 76 },
    { 0, 0 },
    { 0x32c0daf4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a0dafc, 95 },
    { 0xb120dafd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7840db02, 46 },
    { 0xfaf0db03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640db08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f0db0b, 23 },
    { 0x5fe0db0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0db0f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c0db15, 76 },
    { 0, 0 },
    { 0xa970db17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b0db1c, 34 },
    { 0x57a0db1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de0db20, 90 },
    { 0, 0 },
    { 0xc730db22, 76 },
    { 0, 0 },
    { 0x1530db24, 55 },
    { 0, 0 },
    { 0xe5a0db26, 55 },
    { 0x8d10db27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9240db2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660db2e, 33 },
    { 0x2820db2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e0db32, 76 },
    { 0, 0 },
    { 0xf560db34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2000db3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4390db3f, 77 },
    { 0x4580db40, 78 },
    { 0xc6a0db41, 55 },
    { 0xd0a0db40, 99 },
    { 0x7aa0db42, 23 },
    { 0x33b0db44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0db47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd720db4d, 98 },
    { 0xbfa0db4e, 33 },
    { 0x99c0db4f, 44 },
    { 0x3bc0db4e, 98 },
    { 0, 0 },
    { 0xd5f0db52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f80db56, 55 },
    { 0xe0db56, 55 },
    { 0x8080db58, 46 },
    { 0x3970db59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e20db5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30db61, 78 },
    { 0x4420db62, 23 },
    { 0x16f0db63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0db66, 46 },
    { 0x50e0db66, 98 },
    { 0x2300db67, 61 },
    { 0x1140db69, 43 },
    { 0x3c0db6a, 61 },
    { 0, 0 },
    { 0x5b70db6c, 43 },
    { 0xb560db6d, 33 },
    { 0xf780db6e, 34 },
    { 0xa510db6e, 95 },
    { 0xf4c0db70, 64 },
    { 0x960db71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c20db75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd410db7d, 55 },
    { 0x8c10db7e, 61 },
    { 0x8120db7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd070db8a, 23 },
    { 0xc460db8b, 55 },
    { 0x8c30db8b, 34 },
    { 0x5810db8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa0db92, 61 },
    { 0x7c40db93, 33 },
    { 0x1400db94, 33 },
    { 0, 0 },
    { 0xcae0db96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4370db9a, 43 },
    { 0xa210db9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c60dba5, 98 },
    { 0xd200dba5, 61 },
    { 0x88e0dba6, 61 },
    { 0x3690dba8, 66 },
    { 0x5ea0dba9, 46 },
    { 0x3db0dba6, 23 },
    { 0xca30dbaa, 90 },
    { 0xdc90dbac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e0dbaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d60dbb2, 36 },
    { 0x5760dbb3, 69 },
    { 0, 0 },
    { 0x7990dbb5, 61 },
    { 0x4030dbb5, 27 },
    { 0, 0 },
    { 0x58c0dbb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690dbbf, 100 },
    { 0x4ed0dbbf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9220dbc7, 67 },
    { 0x85e0dbc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe890dbcb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad0dbcf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab0dbd3, 98 },
    { 0xaaf0dbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9530dbd8, 95 },
    { 0xbc0dbd9, 90 },
    { 0, 0 },
    { 0x12f0dbdb, 33 },
    { 0x2710dbdc, 61 },
    { 0x15e0dbdd, 99 },
    { 0, 0 },
    { 0x7e00dbdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f70dbe3, 66 },
    { 0xd550dbe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb0dbe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390dbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d20dbf6, 80 },
    { 0, 0 },
    { 0x4840dbf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b0dbfb, 34 },
    { 0xdae0dbfc, 98 },
    { 0xad00dbfc, 33 },
    { 0xaf30dbfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5590dc02, 90 },
    { 0, 0 },
    { 0xe320dc04, 90 },
    { 0xac80dc04, 40 },
    { 0x91e0dc04, 90 },
    { 0x8f90dc07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf900dc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70dc0d, 98 },
    { 0, 0 },
    { 0x64c0dc0f, 55 },
    { 0x560dc10, 83 },
    { 0xd3a0dc0f, 61 },
    { 0x6ad0dc12, 61 },
    { 0x5b60dc13, 43 },
    { 0x4cf0dc14, 33 },
    { 0x39d0dc10, 33 },
    { 0xe470dc16, 91 },
    { 0x4fc0dc10, 61 },
    { 0x440dc18, 34 },
    { 0xcd80dc19, 91 },
    { 0, 0 },
    { 0x1390dc1b, 66 },
    { 0, 0 },
    { 0x3e20dc1d, 76 },
    { 0x5b80dc1e, 61 },
    { 0xfc30dc1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa790dc24, 46 },
    { 0, 0 },
    { 0xf3f0dc26, 97 },
    { 0x290dc26, 94 },
    { 0x95a0dc27, 99 },
    { 0x1dd0dc26, 84 },
    { 0x7e70dc26, 93 },
    { 0x5380dc28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90dc2f, 55 },
    { 0x5f40dc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30dc33, 43 },
    { 0xf40dc34, 76 },
    { 0x72d0dc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd980dc3c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000dc3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc0dc42, 33 },
    { 0x4690dc43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa0dc46, 23 },
    { 0, 0 },
    { 0xe6e0dc48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x750dc4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0dc50, 90 },
    { 0xd1b0dc50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990dc57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780dc5a, 33 },
    { 0x9860dc5b, 61 },
    { 0, 0 },
    { 0xa5b0dc5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e30dc62, 33 },
    { 0x7820dc62, 84 },
    { 0xd660dc64, 33 },
    { 0xdb50dc62, 95 },
    { 0xb000dc66, 78 },
    { 0, 0 },
    { 0xc60dc68, 90 },
    { 0, 0 },
    { 0x87e0dc6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50dc73, 55 },
    { 0x14c0dc74, 81 },
    { 0x3ed0dc75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9930dc79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0dc82, 98 },
    { 0x7b00dc83, 87 },
    { 0xec90dc83, 52 },
    { 0, 0 },
    { 0xf670dc86, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3530dc8a, 23 },
    { 0x5f00dc8b, 95 },
    { 0xb30dc8a, 90 },
    { 0x2330dc8d, 23 },
    { 0xf140dc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840dc93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690dc96, 34 },
    { 0x9720dc97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100dc9a, 91 },
    { 0xf60dc9a, 90 },
    { 0, 0 },
    { 0xf380dc9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e70dca2, 61 },
    { 0xe000dca2, 55 },
    { 0, 0 },
    { 0x5d40dca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c40dca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f0dcad, 95 },
    { 0xec30dcae, 81 },
    { 0xe840dcae, 76 },
    { 0xfec0dcb0, 46 },
    { 0x68d0dcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850dcb6, 98 },
    { 0, 0 },
    { 0x6d10dcb8, 33 },
    { 0, 0 },
    { 0x1640dcba, 95 },
    { 0xa200dcbb, 90 },
    { 0x5a10dcbc, 33 },
    { 0xbb20dcbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd0dcc3, 66 },
    { 0, 0 },
    { 0x3b70dcc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00dcca, 33 },
    { 0xf6c0dcca, 23 },
    { 0xf8f0dccc, 17 },
    { 0xffb0dccd, 90 },
    { 0x88c0dcce, 77 },
    { 0xda60dccd, 87 },
    { 0x10dcce, 55 },
    { 0x7730dcce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130dcd5, 33 },
    { 0x1880dcd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b0dcd9, 90 },
    { 0xa3a0dcd9, 33 },
    { 0, 0 },
    { 0x1cb0dcdc, 43 },
    { 0, 0 },
    { 0xc3a0dcde, 88 },
    { 0xd5f0dcde, 93 },
    { 0x9810dce0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc400dce9, 55 },
    { 0, 0 },
    { 0x7950dceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a0dcef, 19 },
    { 0x8590dcf0, 88 },
    { 0, 0 },
    { 0xe390dcf2, 23 },
    { 0x5580dcf3, 91 },
    { 0x9bf0dcf4, 100 },
    { 0x4cf0dcf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d0dcfa, 61 },
    { 0x3950dcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5930dd00, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20dd07, 55 },
    { 0, 0 },
    { 0x6390dd09, 81 },
    { 0x2ee0dd09, 100 },
    { 0x2090dd0b, 75 },
    { 0, 0 },
    { 0xd800dd0d, 78 },
    { 0x4f0dd0e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0dd18, 98 },
    { 0, 0 },
    { 0x83e0dd1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390dd1d, 90 },
    { 0x9560dd1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1310dd22, 66 },
    { 0, 0 },
    { 0x6120dd24, 95 },
    { 0x80b0dd25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb770dd2b, 61 },
    { 0, 0 },
    { 0xc3a0dd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690dd37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd3b, 61 },
    { 0x52a0dd3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0dd41, 23 },
    { 0, 0 },
    { 0x2de0dd43, 67 },
    { 0, 0 },
    { 0x7a90dd45, 33 },
    { 0xb810dd46, 95 },
    { 0, 0 },
    { 0xa380dd48, 90 },
    { 0xbe40dd48, 61 },
    { 0x6550dd49, 76 },
    { 0x27e0dd4b, 76 },
    { 0xfe70dd4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd0dd50, 55 },
    { 0x2300dd51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1040dd54, 61 },
    { 0x1b10dd54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b0dd59, 90 },
    { 0xd8b0dd5a, 98 },
    { 0xb8e0dd59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb140dd5f, 46 },
    { 0, 0 },
    { 0x8480dd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7720dd64, 90 },
    { 0, 0 },
    { 0x3ff0dd66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be0dd69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30dd6d, 23 },
    { 0x2400dd6e, 98 },
    { 0xa7e0dd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60dd74, 93 },
    { 0xd010dd75, 77 },
    { 0xa850dd76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7550dd7a, 52 },
    { 0x4770dd7b, 95 },
    { 0xd4b0dd7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf70dd81, 98 },
    { 0x4200dd82, 61 },
    { 0xa880dd81, 83 },
    { 0x1b80dd84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340dd8a, 98 },
    { 0x9130dd8b, 90 },
    { 0x5650dd8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b0dd91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710dd96, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480dd99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd0dd9f, 93 },
    { 0xb3d0dda0, 84 },
    { 0, 0 },
    { 0xeb50dda2, 33 },
    { 0x33e0dda3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d70dda6, 97 },
    { 0xc8d0dda6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b40ddb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0ddb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb0ddbd, 66 },
    { 0xfdd0ddbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca20ddc1, 99 },
    { 0x5740ddc2, 98 },
    { 0xea90ddc3, 66 },
    { 0x6c30ddc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d0ddc7, 96 },
    { 0x7640ddc8, 33 },
    { 0, 0 },
    { 0xfbb0ddca, 34 },
    { 0xabf0ddcb, 33 },
    { 0x3ff0ddca, 87 },
    { 0x5e20ddcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8710ddd0, 34 },
    { 0xade0ddd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0ddd6, 87 },
    { 0, 0 },
    { 0x6530ddd8, 43 },
    { 0x8060ddd9, 84 },
    { 0x84d0ddda, 100 },
    { 0xb900dddb, 93 },
    { 0x46b0ddd8, 33 },
    { 0xa550dddd, 68 },
    { 0x83b0dddd, 90 },
    { 0, 0 },
    { 0xe730dde0, 90 },
    { 0, 0 },
    { 0xbf00dde2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0dde7, 90 },
    { 0xf860dde7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d60ddef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b0ddf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb0ddf9, 34 },
    { 0xa330ddfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3400ddfe, 98 },
    { 0x7e10ddff, 33 },
    { 0xa990de00, 47 },
    { 0xa520ddff, 95 },
    { 0, 0 },
    { 0x9070de03, 98 },
    { 0x33b0de03, 98 },
    { 0x5780de05, 23 },
    { 0x38a0de05, 90 },
    { 0, 0 },
    { 0x8670de08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf360de0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00de10, 33 },
    { 0, 0 },
    { 0xc020de12, 23 },
    { 0, 0 },
    { 0x1660de14, 100 },
    { 0xf460de15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70de1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae50de21, 68 },
    { 0x16d0de22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e0de27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb0de2a, 98 },
    { 0x3670de2a, 23 },
    { 0x8e50de2b, 90 },
    { 0xe750de2b, 90 },
    { 0, 0 },
    { 0x1970de2f, 91 },
    { 0xb8c0de2f, 95 },
    { 0x9de0de31, 90 },
    { 0xb130de30, 51 },
    { 0x69b0de2f, 55 },
    { 0xcf0de34, 61 },
    { 0x2270de34, 33 },
    { 0x92a0de36, 88 },
    { 0x1640de37, 23 },
    { 0, 0 },
    { 0x9200de39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540de3d, 90 },
    { 0, 0 },
    { 0x6ab0de3f, 46 },
    { 0x3aa0de40, 61 },
    { 0xae30de3f, 95 },
    { 0xe990de41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9490de45, 95 },
    { 0x34a0de45, 98 },
    { 0, 0 },
    { 0xbcb0de48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d0de4b, 93 },
    { 0, 0 },
    { 0x16c0de4d, 90 },
    { 0x8a10de4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b0de52, 61 },
    { 0x5d70de53, 78 },
    { 0x7810de52, 95 },
    { 0xd2e0de55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f0de5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d90de61, 93 },
    { 0x8360de62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6850de65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0de68, 98 },
    { 0, 0 },
    { 0x750de6a, 34 },
    { 0, 0 },
    { 0x5b30de6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5350de70, 69 },
    { 0x39e0de71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410de75, 23 },
    { 0x9e10de76, 33 },
    { 0, 0 },
    { 0x8700de78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9750de7c, 66 },
    { 0, 0 },
    { 0x5f60de7e, 98 },
    { 0, 0 },
    { 0xfd40de80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb0de84, 76 },
    { 0x61a0de84, 99 },
    { 0xd0e0de85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac40de8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa0de90, 9 },
    { 0x2750de90, 95 },
    { 0xbcf0de90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0de95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c0de98, 90 },
    { 0xe640de99, 90 },
    { 0x59d0de9a, 87 },
    { 0, 0 },
    { 0x50f0de9c, 33 },
    { 0x75b0de9d, 83 },
    { 0x8900de9e, 66 },
    { 0, 0 },
    { 0x5790dea0, 52 },
    { 0xc2e0dea1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60dea4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe0dea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd00deb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10deb9, 98 },
    { 0, 0 },
    { 0x3700debb, 34 },
    { 0x33e0debc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d0dec0, 61 },
    { 0x5ec0dec0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c0dec4, 66 },
    { 0x1e80dec4, 61 },
    { 0, 0 },
    { 0xfdc0dec7, 76 },
    { 0xe220dec8, 33 },
    { 0xa5d0dec9, 99 },
    { 0x94f0deca, 33 },
    { 0x5150decb, 84 },
    { 0x86b0deca, 92 },
    { 0, 0 },
    { 0x3f50dece, 52 },
    { 0, 0 },
    { 0x7680ded0, 90 },
    { 0, 0 },
    { 0x8720ded2, 91 },
    { 0x30ded2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840ded9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc510dedd, 90 },
    { 0x1b30dede, 83 },
    { 0xa8d0dedf, 61 },
    { 0x900dee0, 84 },
    { 0, 0 },
    { 0x79a0dee2, 23 },
    { 0x7df0dee3, 33 },
    { 0xf930dee2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad50dee9, 53 },
    { 0xac50deea, 61 },
    { 0x6ba0dee9, 99 },
    { 0x3900deec, 90 },
    { 0xa930deed, 69 },
    { 0x8b20deec, 98 },
    { 0x5530deef, 77 },
    { 0x69a0def0, 83 },
    { 0x37f0deef, 91 },
    { 0x10f0def2, 23 },
    { 0x9390def1, 98 },
    { 0xca0def3, 90 },
    { 0x81d0def5, 98 },
    { 0x9130def4, 95 },
    { 0x7500deef, 69 },
    { 0x12d0def3, 98 },
    { 0, 0 },
    { 0x80f0defa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd0df03, 76 },
    { 0x7f70df03, 90 },
    { 0xec30df05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0df08, 98 },
    { 0, 0 },
    { 0xbed0df0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0df0e, 91 },
    { 0xd6e0df0f, 67 },
    { 0x4480df10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0df15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f0df1b, 61 },
    { 0, 0 },
    { 0xf8a0df1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba0df22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f80df2a, 85 },
    { 0x99b0df2b, 77 },
    { 0x1f80df2b, 46 },
    { 0x77d0df2d, 61 },
    { 0x2bb0df2e, 65 },
    { 0x7aa0df2f, 88 },
    { 0xb360df2d, 23 },
    { 0, 0 },
    { 0x3600df32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0df37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940df3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a50df43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790df47, 33 },
    { 0xfd10df48, 23 },
    { 0x3c00df48, 61 },
    { 0, 0 },
    { 0xc950df4b, 62 },
    { 0xade0df4b, 98 },
    { 0xc7b0df4d, 23 },
    { 0, 0 },
    { 0xb130df4f, 76 },
    { 0, 0 },
    { 0xbd90df51, 23 },
    { 0, 0 },
    { 0x2560df53, 90 },
    { 0x1080df54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a0df57, 90 },
    { 0xe760df58, 90 },
    { 0xd430df57, 61 },
    { 0x3650df57, 90 },
    { 0, 0 },
    { 0x1980df5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf200df5f, 33 },
    { 0x4ab0df5f, 33 },
    { 0, 0 },
    { 0xdc30df62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd10df65, 74 },
    { 0xc3f0df66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb790df69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890df71, 67 },
    { 0xee90df71, 61 },
    { 0x5520df73, 98 },
    { 0x35f0df71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2860df79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d10df80, 90 },
    { 0, 0 },
    { 0xef20df82, 95 },
    { 0x31b0df83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950df8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c0df8f, 90 },
    { 0, 0 },
    { 0x78f0df91, 61 },
    { 0x15f0df91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0df96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f0df9b, 19 },
    { 0xc370df9c, 61 },
    { 0x1260df9d, 98 },
    { 0, 0 },
    { 0x93e0df9f, 55 },
    { 0xe7b0dfa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3210dfa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d0dfa8, 91 },
    { 0xb5e0dfa9, 95 },
    { 0x1df0dfa9, 95 },
    { 0, 0 },
    { 0x6af0dfac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0dfb0, 76 },
    { 0, 0 },
    { 0xc140dfb2, 77 },
    { 0xd2b0dfb3, 76 },
    { 0xe660dfb4, 98 },
    { 0x9ad0dfb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0dfc3, 33 },
    { 0, 0 },
    { 0x45c0dfc5, 23 },
    { 0, 0 },
    { 0x60dfc7, 33 },
    { 0x9ee0dfc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20dfcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2890dfce, 91 },
    { 0xb870dfce, 99 },
    { 0x51c0dfce, 23 },
    { 0, 0 },
    { 0xa520dfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0dfda, 90 },
    { 0x6ba0dfdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e0dfe5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00dfe8, 84 },
    { 0, 0 },
    { 0xd2f0dfea, 98 },
    { 0xba50dfea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa830dfef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f80dff4, 90 },
    { 0x31e0dff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070dffa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410dffd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7340e001, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130e008, 34 },
    { 0xd980e009, 23 },
    { 0xdb60e00a, 61 },
    { 0xb980e00b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410e014, 90 },
    { 0x6ac0e014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa00e01c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca0e024, 95 },
    { 0xd3c0e024, 75 },
    { 0, 0 },
    { 0x14d0e027, 98 },
    { 0xd1c0e028, 61 },
    { 0x4e30e027, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1810e031, 98 },
    { 0x1460e031, 77 },
    { 0, 0 },
    { 0x2d60e034, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0e039, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9910e03c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0e040, 95 },
    { 0x400e041, 43 },
    { 0xa060e040, 55 },
    { 0x8a60e042, 33 },
    { 0xfc80e040, 33 },
    { 0x1fb0e042, 23 },
    { 0x1440e044, 23 },
    { 0x640e047, 76 },
    { 0xa700e047, 61 },
    { 0x42e0e049, 34 },
    { 0xb20e044, 33 },
    { 0xd4f0e04b, 95 },
    { 0xad80e043, 90 },
    { 0xcf70e04d, 95 },
    { 0x81f0e046, 68 },
    { 0xdbc0e046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e0e054, 23 },
    { 0x9f30e054, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e062, 33 },
    { 0, 0 },
    { 0x6ba0e064, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550e06d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3020e070, 95 },
    { 0x1ad0e071, 33 },
    { 0x9910e072, 98 },
    { 0x7750e073, 60 },
    { 0x1380e074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80e077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630e07b, 90 },
    { 0, 0 },
    { 0x7b60e07d, 33 },
    { 0xfb10e07e, 61 },
    { 0x8d30e07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8880e087, 95 },
    { 0, 0 },
    { 0x4610e089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb070e08c, 67 },
    { 0, 0 },
    { 0x7c80e08e, 88 },
    { 0, 0 },
    { 0x9e0e090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0e09a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc640e09d, 33 },
    { 0xd860e09d, 76 },
    { 0, 0 },
    { 0xfcd0e0a0, 61 },
    { 0x8110e0a1, 33 },
    { 0, 0 },
    { 0x1cb0e0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0e0a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2100e0ab, 98 },
    { 0x4fb0e0ac, 90 },
    { 0, 0 },
    { 0xc0e0e0ae, 33 },
    { 0x7a30e0af, 23 },
    { 0x51e0e0b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf60e0b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5610e0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f0e0c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890e0cd, 98 },
    { 0x4610e0ce, 33 },
    { 0xed90e0cd, 46 },
    { 0x7da0e0cf, 96 },
    { 0x18c0e0ce, 33 },
    { 0, 0 },
    { 0x2ec0e0d3, 98 },
    { 0x9fe0e0d4, 67 },
    { 0, 0 },
    { 0xcbb0e0d6, 90 },
    { 0x4760e0d7, 93 },
    { 0x4720e0d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc0e0de, 55 },
    { 0, 0 },
    { 0x6740e0e0, 90 },
    { 0xbf50e0e1, 52 },
    { 0xa030e0e2, 87 },
    { 0x5ce0e0e3, 8 },
    { 0x2db0e0e1, 55 },
    { 0x4430e0e4, 55 },
    { 0x30b0e0e6, 93 },
    { 0xdd50e0e6, 17 },
    { 0x4640e0e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d50e0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f0e0f3, 55 },
    { 0xe440e0f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a60e0f7, 59 },
    { 0xcee0e0f8, 90 },
    { 0, 0 },
    { 0x3be0e0fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa750e0fe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e0e109, 24 },
    { 0x6880e10a, 90 },
    { 0x2d30e109, 76 },
    { 0xc8b0e10c, 34 },
    { 0x4770e10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690e111, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150e117, 61 },
    { 0x9320e118, 95 },
    { 0x1d80e117, 33 },
    { 0xd570e11a, 98 },
    { 0xc210e11a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330e11e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f0e123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d0e126, 23 },
    { 0x9280e127, 90 },
    { 0x9bf0e127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0e12b, 46 },
    { 0xd4d0e12b, 77 },
    { 0x8c60e12d, 55 },
    { 0xd300e12d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00e131, 46 },
    { 0xb3a0e131, 36 },
    { 0, 0 },
    { 0x1e90e134, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa910e137, 76 },
    { 0x2460e138, 35 },
    { 0, 0 },
    { 0x3340e13a, 95 },
    { 0, 0 },
    { 0x2e0e13c, 90 },
    { 0x86c0e13d, 95 },
    { 0x2930e13e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e30e148, 66 },
    { 0, 0 },
    { 0x4770e14a, 90 },
    { 0, 0 },
    { 0xfc00e14c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d90e153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad0e159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f0e15c, 33 },
    { 0x86b0e15d, 98 },
    { 0, 0 },
    { 0x6080e15f, 33 },
    { 0, 0 },
    { 0x19c0e161, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e166, 61 },
    { 0xcc50e166, 52 },
    { 0xee20e168, 77 },
    { 0, 0 },
    { 0x2c90e16a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890e16d, 23 },
    { 0xae10e16d, 23 },
    { 0xb850e16f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca0e174, 78 },
    { 0xa830e174, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba0e17a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c20e17d, 90 },
    { 0x99f0e17d, 34 },
    { 0xaa90e17f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30e185, 83 },
    { 0x3320e186, 46 },
    { 0x89b0e187, 84 },
    { 0xc640e186, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910e18d, 90 },
    { 0x83e0e18d, 38 },
    { 0xfa40e18f, 55 },
    { 0x5750e190, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b0e195, 93 },
    { 0xae00e196, 61 },
    { 0x60e197, 66 },
    { 0, 0 },
    { 0x92b0e199, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6830e19f, 48 },
    { 0x5ad0e19f, 75 },
    { 0xc3a0e19f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d40e1a4, 66 },
    { 0x7bf0e1a5, 90 },
    { 0x3a00e1a4, 97 },
    { 0x8ef0e1a7, 61 },
    { 0xcb10e1a8, 98 },
    { 0xc9a0e1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e0e1b2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc40e1b6, 100 },
    { 0xf6e0e1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf870e1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40e1ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e1d1, 85 },
    { 0, 0 },
    { 0x4bc0e1d3, 95 },
    { 0, 0 },
    { 0xf680e1d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550e1da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e0e1e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640e1e3, 90 },
    { 0, 0 },
    { 0xd710e1e5, 43 },
    { 0x89d0e1e6, 33 },
    { 0x6ed0e1e7, 43 },
    { 0x2ed0e1e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd60e1eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc010e1f0, 77 },
    { 0, 0 },
    { 0xfcd0e1f2, 98 },
    { 0x6d50e1f3, 34 },
    { 0xd680e1f4, 55 },
    { 0, 0 },
    { 0x9830e1f6, 95 },
    { 0x2400e1f7, 55 },
    { 0x5e90e1f8, 76 },
    { 0x4200e1f9, 33 },
    { 0xd880e1fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650e20a, 51 },
    { 0, 0 },
    { 0x20d0e20c, 78 },
    { 0, 0 },
    { 0x8c90e20e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0e214, 84 },
    { 0xfe70e214, 87 },
    { 0x2fa0e216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a0e21c, 67 },
    { 0x790e21d, 98 },
    { 0x5eb0e21d, 52 },
    { 0x80d0e21d, 98 },
    { 0xb160e220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30e223, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb0e227, 95 },
    { 0xb400e228, 86 },
    { 0x8db0e227, 44 },
    { 0, 0 },
    { 0x45e0e22b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40e22f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a30e232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe660e239, 90 },
    { 0, 0 },
    { 0x3a50e23b, 76 },
    { 0x5f0e23b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea0e240, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b90e245, 33 },
    { 0xa80e246, 98 },
    { 0xea60e247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf20e24b, 34 },
    { 0x9bb0e24b, 43 },
    { 0x2e90e24c, 61 },
    { 0x540e24b, 69 },
    { 0x5ab0e24f, 55 },
    { 0x50f0e250, 33 },
    { 0x1470e24e, 90 },
    { 0, 0 },
    { 0x61f0e253, 96 },
    { 0, 0 },
    { 0x710e255, 32 },
    { 0, 0 },
    { 0xa4e0e257, 66 },
    { 0, 0 },
    { 0x20c0e259, 34 },
    { 0, 0 },
    { 0x5f30e25b, 98 },
    { 0, 0 },
    { 0x8ca0e25d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250e261, 61 },
    { 0x5b00e262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c40e265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0e269, 99 },
    { 0x410e26a, 61 },
    { 0, 0 },
    { 0xce30e26c, 99 },
    { 0, 0 },
    { 0xff90e26e, 61 },
    { 0x7b40e26e, 84 },
    { 0x91e0e270, 90 },
    { 0x25e0e270, 77 },
    { 0x180e272, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0e275, 100 },
    { 0xd790e276, 93 },
    { 0xc890e277, 61 },
    { 0xc990e278, 76 },
    { 0x7a50e277, 36 },
    { 0x58c0e279, 46 },
    { 0x1110e27a, 88 },
    { 0, 0 },
    { 0x4ed0e27d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb660e280, 91 },
    { 0xd9e0e281, 81 },
    { 0x1ce0e281, 84 },
    { 0x720e282, 23 },
    { 0x91a0e282, 34 },
    { 0, 0 },
    { 0xab20e286, 34 },
    { 0x5090e287, 33 },
    { 0, 0 },
    { 0x5880e289, 99 },
    { 0, 0 },
    { 0x3840e28b, 61 },
    { 0x73b0e28b, 91 },
    { 0, 0 },
    { 0x2a10e28e, 95 },
    { 0x1a90e28f, 100 },
    { 0xcbc0e28e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb680e295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc70e298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa200e2a4, 43 },
    { 0x9220e2a5, 98 },
    { 0xf7f0e2a6, 43 },
    { 0xe7a0e2a7, 95 },
    { 0, 0 },
    { 0xd800e2a9, 95 },
    { 0xda70e2aa, 69 },
    { 0x5c80e2ab, 95 },
    { 0x2bf0e2a9, 93 },
    { 0x5920e2ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c0e2b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe560e2b9, 90 },
    { 0x3540e2ba, 98 },
    { 0, 0 },
    { 0xb240e2bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0e2bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f0e2c5, 90 },
    { 0x5ca0e2c6, 33 },
    { 0x3460e2c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0e2cd, 55 },
    { 0, 0 },
    { 0x6600e2cf, 52 },
    { 0x1fa0e2cf, 33 },
    { 0, 0 },
    { 0x7d90e2d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf270e2db, 53 },
    { 0, 0 },
    { 0xe5e0e2dd, 95 },
    { 0x5880e2de, 94 },
    { 0xbaa0e2de, 90 },
    { 0xf6b0e2e0, 33 },
    { 0, 0 },
    { 0xffa0e2e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350e2e7, 95 },
    { 0x7ed0e2e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e2ec, 78 },
    { 0xa3c0e2ec, 66 },
    { 0x62e0e2ed, 61 },
    { 0, 0 },
    { 0xcf40e2f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0e2f4, 61 },
    { 0x3ce0e2f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a0e2f9, 76 },
    { 0x4260e2fa, 53 },
    { 0, 0 },
    { 0xc150e2fc, 23 },
    { 0, 0 },
    { 0xcb0e2fe, 90 },
    { 0xafe0e2fe, 34 },
    { 0xbba0e300, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b0e307, 23 },
    { 0xd0f0e307, 33 },
    { 0x1680e309, 95 },
    { 0xe880e30a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5130e312, 98 },
    { 0x9c60e312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef80e316, 43 },
    { 0x5d30e317, 80 },
    { 0x2f90e318, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca0e31b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9170e322, 33 },
    { 0x6da0e322, 98 },
    { 0, 0 },
    { 0xc680e325, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3490e329, 99 },
    { 0xd490e32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd080e32e, 34 },
    { 0xfce0e32f, 90 },
    { 0x1970e32e, 23 },
    { 0x6250e331, 66 },
    { 0xe8e0e331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060e336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa10e342, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050e345, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab0e34a, 61 },
    { 0x52c0e34b, 90 },
    { 0x6730e34b, 95 },
    { 0x6b0e34c, 95 },
    { 0xed80e34c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa510e352, 68 },
    { 0, 0 },
    { 0xae00e354, 77 },
    { 0x72b0e355, 61 },
    { 0, 0 },
    { 0xb450e357, 17 },
    { 0x47b0e357, 76 },
    { 0, 0 },
    { 0xa380e35a, 85 },
    { 0, 0 },
    { 0xb860e35c, 95 },
    { 0xbd00e35d, 61 },
    { 0, 0 },
    { 0x7ac0e35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb090e362, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee70e36e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf100e371, 78 },
    { 0, 0 },
    { 0x58e0e373, 4 },
    { 0, 0 },
    { 0x1370e375, 90 },
    { 0, 0 },
    { 0x3420e377, 61 },
    { 0, 0 },
    { 0x610e379, 90 },
    { 0, 0 },
    { 0x21f0e37b, 52 },
    { 0xc6c0e37c, 90 },
    { 0xe020e37d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac0e380, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a0e384, 61 },
    { 0xdd0e384, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8610e38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd0e393, 90 },
    { 0xa770e394, 23 },
    { 0xacf0e395, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf530e399, 61 },
    { 0, 0 },
    { 0x5ab0e39b, 23 },
    { 0x6080e39c, 33 },
    { 0xf890e39d, 43 },
    { 0, 0 },
    { 0x40d0e39f, 95 },
    { 0x4c50e3a0, 66 },
    { 0x5c70e3a1, 23 },
    { 0, 0 },
    { 0x7e90e3a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120e3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb00e3b0, 34 },
    { 0, 0 },
    { 0xf800e3b2, 33 },
    { 0x5f0e3b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc500e3b7, 33 },
    { 0xcbe0e3b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730e3bc, 33 },
    { 0x8c00e3bd, 33 },
    { 0x5bb0e3bd, 33 },
    { 0, 0 },
    { 0x5e70e3c0, 33 },
    { 0, 0 },
    { 0x87f0e3c2, 46 },
    { 0xfc0e3c3, 65 },
    { 0xf160e3c2, 95 },
    { 0xdc80e3c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf270e3cc, 98 },
    { 0, 0 },
    { 0x690e3ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb0e3d2, 95 },
    { 0x61d0e3d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe140e3d9, 76 },
    { 0x70e3d9, 52 },
    { 0xcb90e3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4020e3df, 88 },
    { 0x8650e3df, 99 },
    { 0, 0 },
    { 0x3f50e3e2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c80e3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf110e3ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f0e3f2, 34 },
    { 0xac20e3f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0e3f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20e3fe, 84 },
    { 0xa850e3ff, 76 },
    { 0x29c0e400, 33 },
    { 0, 0 },
    { 0x3b0e402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae0e407, 43 },
    { 0xad30e408, 68 },
    { 0x50a0e409, 88 },
    { 0xcb0e409, 95 },
    { 0x85c0e408, 33 },
    { 0x4510e40a, 61 },
    { 0x5380e40b, 90 },
    { 0x1d20e40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf150e411, 100 },
    { 0x3ed0e411, 61 },
    { 0, 0 },
    { 0xa7c0e414, 23 },
    { 0x3230e414, 76 },
    { 0x4dc0e415, 33 },
    { 0x4170e415, 66 },
    { 0x6440e418, 23 },
    { 0x5f80e415, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f0e41c, 81 },
    { 0x480e41d, 95 },
    { 0x67d0e41d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb0e423, 40 },
    { 0, 0 },
    { 0x69d0e425, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c0e42b, 78 },
    { 0, 0 },
    { 0x9f30e42d, 23 },
    { 0x5a40e42d, 34 },
    { 0, 0 },
    { 0xc9d0e430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf710e433, 61 },
    { 0xd130e434, 87 },
    { 0x2a80e435, 98 },
    { 0x5ce0e433, 55 },
    { 0xee20e437, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600e43b, 90 },
    { 0, 0 },
    { 0x7770e43d, 23 },
    { 0, 0 },
    { 0xc3a0e43f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa10e444, 91 },
    { 0x66b0e445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d00e449, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b0e44c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d30e452, 33 },
    { 0xd330e453, 76 },
    { 0, 0 },
    { 0x9100e455, 23 },
    { 0x9fc0e455, 52 },
    { 0xd040e455, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f10e45b, 87 },
    { 0xc590e45c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90e461, 33 },
    { 0x2d60e461, 90 },
    { 0, 0 },
    { 0x2af0e464, 33 },
    { 0x84b0e464, 55 },
    { 0xed00e466, 53 },
    { 0xe4a0e465, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0e46c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2800e47c, 90 },
    { 0x4a10e47d, 90 },
    { 0, 0 },
    { 0xd3c0e47f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef40e483, 90 },
    { 0xbb90e484, 33 },
    { 0xdee0e485, 76 },
    { 0xbc00e486, 78 },
    { 0xc5d0e487, 95 },
    { 0, 0 },
    { 0x9e00e489, 33 },
    { 0xe1e0e48a, 52 },
    { 0xc820e48a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410e490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce0e497, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea40e4a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x910e4a4, 95 },
    { 0x5160e4a4, 90 },
    { 0x9c90e4a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d0e4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10e4ac, 100 },
    { 0x4570e4ac, 90 },
    { 0, 0 },
    { 0xde0e4af, 33 },
    { 0x2480e4b0, 23 },
    { 0x7e40e4b1, 33 },
    { 0xe610e4b1, 33 },
    { 0x3b20e4b1, 55 },
    { 0, 0 },
    { 0xa3e0e4b5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c70e4c0, 95 },
    { 0, 0 },
    { 0xd6b0e4c2, 33 },
    { 0, 0 },
    { 0xe2e0e4c4, 98 },
    { 0x8930e4c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060e4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8000e4cc, 71 },
    { 0x8280e4cc, 98 },
    { 0x4bd0e4cc, 90 },
    { 0, 0 },
    { 0x8a80e4d0, 88 },
    { 0xf6c0e4d0, 61 },
    { 0, 0 },
    { 0xaf20e4d3, 61 },
    { 0x68b0e4d3, 98 },
    { 0x3e80e4d3, 95 },
    { 0x440e4d6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea90e4da, 55 },
    { 0x8830e4db, 90 },
    { 0x2030e4da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd170e4e0, 95 },
    { 0x69f0e4e0, 98 },
    { 0x87f0e4e2, 66 },
    { 0xc90e4e3, 76 },
    { 0xb330e4e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50e4ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e4f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc010e4f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690e4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c0e500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c60e50a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d90e50f, 61 },
    { 0x5d80e510, 90 },
    { 0x2c40e511, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650e514, 77 },
    { 0x4ca0e515, 33 },
    { 0x630e515, 23 },
    { 0x8df0e516, 89 },
    { 0xcfb0e518, 23 },
    { 0xe4a0e519, 98 },
    { 0x2e0e51a, 98 },
    { 0x70a0e517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc0e523, 66 },
    { 0xb2a0e524, 52 },
    { 0, 0 },
    { 0xf470e526, 95 },
    { 0x3eb0e527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc910e52c, 34 },
    { 0x8170e52d, 33 },
    { 0, 0 },
    { 0xf330e52f, 99 },
    { 0x86f0e530, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb0e536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2800e53e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0e545, 99 },
    { 0, 0 },
    { 0x9e20e547, 64 },
    { 0, 0 },
    { 0x1c50e549, 98 },
    { 0, 0 },
    { 0x4bc0e54b, 33 },
    { 0xdda0e54b, 61 },
    { 0, 0 },
    { 0x1490e54e, 90 },
    { 0x7970e54f, 23 },
    { 0x2470e550, 90 },
    { 0xa720e550, 61 },
    { 0x9e90e552, 80 },
    { 0x16c0e552, 98 },
    { 0xd290e552, 90 },
    { 0, 0 },
    { 0xca60e556, 61 },
    { 0xc2f0e556, 95 },
    { 0xdc70e558, 23 },
    { 0, 0 },
    { 0x5080e55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180e55f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f90e569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d50e570, 61 },
    { 0xa4f0e571, 43 },
    { 0xad90e572, 33 },
    { 0x7110e570, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540e576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5690e579, 93 },
    { 0x1f80e57a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce00e57d, 98 },
    { 0x4b60e57e, 61 },
    { 0xdb90e57f, 38 },
    { 0, 0 },
    { 0xabf0e581, 95 },
    { 0x8b10e581, 61 },
    { 0, 0 },
    { 0xa2f0e584, 94 },
    { 0xd050e585, 98 },
    { 0xa740e586, 43 },
    { 0x2b70e585, 61 },
    { 0x3520e588, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990e58b, 98 },
    { 0, 0 },
    { 0xc6b0e58d, 61 },
    { 0xf890e58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f60e594, 77 },
    { 0, 0 },
    { 0xfd30e596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc90e599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70e59f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd590e5a2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed0e5a5, 98 },
    { 0, 0 },
    { 0x2f10e5a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b50e5ab, 78 },
    { 0x6ef0e5ab, 48 },
    { 0, 0 },
    { 0x9f20e5ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350e5b1, 93 },
    { 0x8fa0e5b1, 100 },
    { 0x49f0e5b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc110e5b6, 17 },
    { 0, 0 },
    { 0xc970e5b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e0e5bc, 91 },
    { 0x9690e5bd, 46 },
    { 0xd010e5be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea0e5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba40e5c6, 95 },
    { 0x7d80e5c6, 23 },
    { 0x260e5c6, 99 },
    { 0x9120e5c9, 95 },
    { 0x16a0e5ca, 76 },
    { 0xba90e5c8, 36 },
    { 0x3480e5cc, 55 },
    { 0x4680e5c9, 33 },
    { 0, 0 },
    { 0xb8f0e5cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be0e5d4, 85 },
    { 0x60a0e5d5, 90 },
    { 0, 0 },
    { 0x5010e5d7, 93 },
    { 0x930e5d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a0e5db, 96 },
    { 0, 0 },
    { 0x2d00e5dd, 34 },
    { 0xcb20e5de, 23 },
    { 0x3ac0e5df, 100 },
    { 0xe530e5e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b00e5e4, 43 },
    { 0xa6a0e5e5, 43 },
    { 0x4950e5e5, 61 },
    { 0xbbc0e5e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9290e5ea, 95 },
    { 0x59a0e5eb, 90 },
    { 0x7f10e5ec, 43 },
    { 0x74a0e5ed, 76 },
    { 0x58a0e5ee, 90 },
    { 0x1660e5ea, 61 },
    { 0xafb0e5f0, 90 },
    { 0x3920e5f1, 87 },
    { 0xe270e5f1, 61 },
    { 0xd110e5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e5f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd560e5fa, 90 },
    { 0x7c40e5fb, 93 },
    { 0x65a0e5fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de0e600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630e603, 90 },
    { 0x99a0e604, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50e607, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b40e60a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910e60f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf030e616, 61 },
    { 0x59b0e617, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250e61a, 98 },
    { 0xac00e61b, 61 },
    { 0, 0 },
    { 0x6b40e61d, 55 },
    { 0x2200e61e, 33 },
    { 0, 0 },
    { 0x2bd0e620, 99 },
    { 0x3cc0e621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee80e624, 84 },
    { 0, 0 },
    { 0x1410e626, 52 },
    { 0x6de0e627, 55 },
    { 0x8b80e626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f70e62b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c0e62f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6300e636, 23 },
    { 0xb2e0e637, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00e63a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40e63d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0e641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0e646, 77 },
    { 0x2060e646, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3980e654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0e659, 98 },
    { 0, 0 },
    { 0xe620e65b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf970e669, 95 },
    { 0, 0 },
    { 0xd520e66b, 84 },
    { 0x1b50e66b, 95 },
    { 0xf6f0e66b, 61 },
    { 0x5310e66e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d0e671, 69 },
    { 0x50f0e672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450e678, 76 },
    { 0x8230e679, 88 },
    { 0xc310e67a, 90 },
    { 0x5c80e67b, 34 },
    { 0x48c0e679, 90 },
    { 0, 0 },
    { 0xdbe0e67e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba20e682, 91 },
    { 0x4fa0e683, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0e687, 33 },
    { 0x32d0e687, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e0e68b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff0e68f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3600e694, 76 },
    { 0, 0 },
    { 0xc6a0e696, 55 },
    { 0, 0 },
    { 0x250e698, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f40e69b, 84 },
    { 0xdf60e69c, 90 },
    { 0xf250e69d, 23 },
    { 0, 0 },
    { 0xf7b0e69f, 23 },
    { 0x76a0e6a0, 83 },
    { 0xf950e6a1, 55 },
    { 0, 0 },
    { 0x35d0e6a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa370e6a6, 43 },
    { 0xa690e6a7, 55 },
    { 0xa210e6a8, 98 },
    { 0xf0e0e6a9, 95 },
    { 0, 0 },
    { 0x8f70e6ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4960e6b0, 84 },
    { 0x77d0e6b1, 95 },
    { 0x97e0e6b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e6ba, 90 },
    { 0, 0 },
    { 0x7560e6bc, 61 },
    { 0x18d0e6bd, 52 },
    { 0xc1b0e6bd, 90 },
    { 0x2500e6bd, 87 },
    { 0xd850e6bd, 23 },
    { 0, 0 },
    { 0x8220e6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f70e6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b20e6c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa290e6d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150e6d4, 98 },
    { 0x5150e6d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780e6dd, 61 },
    { 0xcb40e6de, 61 },
    { 0, 0 },
    { 0x65e0e6e0, 84 },
    { 0x7870e6e1, 90 },
    { 0, 0 },
    { 0x3a10e6e3, 23 },
    { 0x2480e6e4, 23 },
    { 0x6370e6e4, 33 },
    { 0xd80e6e5, 98 },
    { 0x30e0e6e5, 66 },
    { 0xc380e6e7, 95 },
    { 0xf450e6e9, 9 },
    { 0xc100e6e7, 90 },
    { 0xb8e0e6ea, 90 },
    { 0, 0 },
    { 0x92e0e6ed, 95 },
    { 0x65e0e6ee, 55 },
    { 0xdf40e6ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3360e6f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0e6fa, 61 },
    { 0x8930e6fb, 76 },
    { 0x4730e6fc, 23 },
    { 0x9940e6fd, 23 },
    { 0x2120e6fe, 100 },
    { 0x5da0e6fd, 90 },
    { 0x6130e6fd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e706, 76 },
    { 0, 0 },
    { 0xa070e708, 98 },
    { 0, 0 },
    { 0x87d0e70a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330e70d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed0e710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e60e713, 90 },
    { 0x2c40e714, 67 },
    { 0xeff0e715, 85 },
    { 0, 0 },
    { 0xf2a0e717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4320e71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260e71e, 95 },
    { 0xfbf0e71f, 33 },
    { 0, 0 },
    { 0x7c70e721, 23 },
    { 0x3620e722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e0e725, 90 },
    { 0x2580e725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d0e729, 33 },
    { 0, 0 },
    { 0xdf50e72b, 61 },
    { 0, 0 },
    { 0xd820e72d, 55 },
    { 0x25d0e72d, 93 },
    { 0xed50e72f, 61 },
    { 0, 0 },
    { 0x3360e731, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a60e734, 61 },
    { 0x2390e735, 66 },
    { 0xf800e734, 76 },
    { 0, 0 },
    { 0x22e0e738, 77 },
    { 0, 0 },
    { 0x3230e73a, 95 },
    { 0xa310e73b, 46 },
    { 0x9f60e73b, 61 },
    { 0x9fc0e73d, 17 },
    { 0, 0 },
    { 0xf080e73f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb300e745, 23 },
    { 0x5870e746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00e74a, 83 },
    { 0xb870e74b, 90 },
    { 0xc090e74c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8120e752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e757, 99 },
    { 0x4ca0e758, 66 },
    { 0x1df0e758, 90 },
    { 0x8fa0e757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff0e75d, 90 },
    { 0xbc20e75e, 77 },
    { 0xf130e75d, 93 },
    { 0xd9a0e75f, 93 },
    { 0xa9a0e761, 90 },
    { 0xc090e761, 61 },
    { 0xeba0e763, 33 },
    { 0x9850e764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6610e767, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0e76d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920e770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f60e773, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d0e779, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f0e77c, 33 },
    { 0x90a0e77d, 85 },
    { 0x3630e77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf720e781, 81 },
    { 0x4e0e782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640e785, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9160e78b, 99 },
    { 0, 0 },
    { 0x5ae0e78d, 55 },
    { 0x1600e78e, 92 },
    { 0xa850e78f, 95 },
    { 0xed00e790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5810e793, 100 },
    { 0xca60e793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff0e799, 58 },
    { 0, 0 },
    { 0xe510e79b, 98 },
    { 0, 0 },
    { 0xa730e79d, 34 },
    { 0, 0 },
    { 0x7340e79f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e7a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0e7a8, 61 },
    { 0x6ae0e7a8, 61 },
    { 0x5b30e7a8, 90 },
    { 0x93d0e7a9, 33 },
    { 0xa960e7ac, 61 },
    { 0x38b0e7ab, 85 },
    { 0x4dc0e7ae, 99 },
    { 0x6300e7ad, 33 },
    { 0x6cc0e7a8, 65 },
    { 0x73b0e7b1, 84 },
    { 0, 0 },
    { 0xbf80e7b3, 76 },
    { 0xecd0e7b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0e7b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e0e7bd, 99 },
    { 0x7ad0e7be, 91 },
    { 0xc9a0e7bf, 90 },
    { 0xfaa0e7bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80e7c7, 91 },
    { 0xf410e7c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf260e7cf, 23 },
    { 0, 0 },
    { 0xfc60e7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e7d8, 90 },
    { 0x7dd0e7d9, 78 },
    { 0x30e7d8, 43 },
    { 0xc680e7da, 34 },
    { 0x7190e7d8, 99 },
    { 0x4f90e7dd, 98 },
    { 0xf440e7de, 90 },
    { 0, 0 },
    { 0x7a00e7e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa430e7e5, 90 },
    { 0xd210e7e5, 100 },
    { 0xd660e7e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff00e7ed, 93 },
    { 0xde90e7ee, 23 },
    { 0, 0 },
    { 0x66d0e7f0, 23 },
    { 0, 0 },
    { 0x790e7f2, 90 },
    { 0x1360e7f3, 76 },
    { 0xb2a0e7f4, 93 },
    { 0x7df0e7f5, 90 },
    { 0x1300e7f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e30e7fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0e801, 75 },
    { 0xa0f0e802, 84 },
    { 0x5e0e802, 33 },
    { 0xca70e804, 98 },
    { 0xfd50e805, 61 },
    { 0x270e805, 55 },
    { 0xb5e0e807, 90 },
    { 0xc0f0e808, 90 },
    { 0, 0 },
    { 0xbd00e80a, 33 },
    { 0x4c40e80b, 81 },
    { 0x6950e80a, 33 },
    { 0x6f90e80c, 95 },
    { 0xdd70e80c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540e818, 40 },
    { 0x4520e819, 100 },
    { 0, 0 },
    { 0xcb00e81b, 76 },
    { 0x3880e81c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5270e821, 91 },
    { 0, 0 },
    { 0x22e0e823, 76 },
    { 0x76a0e823, 33 },
    { 0xfc60e825, 90 },
    { 0xa000e826, 23 },
    { 0x41c0e825, 95 },
    { 0x7ef0e828, 95 },
    { 0x1640e828, 34 },
    { 0x1ed0e82a, 98 },
    { 0xd330e82b, 90 },
    { 0, 0 },
    { 0x7b60e82d, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e0e832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd0e838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9880e83b, 90 },
    { 0, 0 },
    { 0xce20e83d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0e84a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea0e84f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c0e853, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0e85b, 23 },
    { 0x810e85b, 90 },
    { 0, 0 },
    { 0x5cb0e85e, 65 },
    { 0xfc0e85e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0e865, 90 },
    { 0xd20e866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8970e86b, 95 },
    { 0xaf20e86b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a0e872, 98 },
    { 0xdbe0e873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70e87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320e87f, 61 },
    { 0x6300e880, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa760e885, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2910e888, 61 },
    { 0xea60e889, 99 },
    { 0x2f70e888, 61 },
    { 0, 0 },
    { 0x1c20e88c, 84 },
    { 0xcc00e88c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f70e892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e0e895, 90 },
    { 0x4880e896, 61 },
    { 0xb0e0e896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d0e89a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed0e89f, 91 },
    { 0xd6b0e8a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa80e8a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc890e8ac, 100 },
    { 0xbc0e8ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690e8b0, 33 },
    { 0x39c0e8b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d90e8b6, 61 },
    { 0, 0 },
    { 0x15a0e8b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530e8bb, 91 },
    { 0x1ac0e8bc, 33 },
    { 0, 0 },
    { 0x2f70e8be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc0e8c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400e8c8, 90 },
    { 0, 0 },
    { 0x1760e8ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0e8ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40e8d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd80e8d4, 23 },
    { 0x4870e8d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070e8d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b0e8e0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f30e8e5, 34 },
    { 0, 0 },
    { 0xdca0e8e7, 33 },
    { 0, 0 },
    { 0x8e60e8e9, 23 },
    { 0, 0 },
    { 0xde80e8eb, 95 },
    { 0xc4a0e8ec, 46 },
    { 0x6f00e8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe790e8f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5300e8fb, 99 },
    { 0x3bc0e8fc, 55 },
    { 0xbaf0e8fb, 76 },
    { 0x7dd0e8fb, 91 },
    { 0x7f10e8ff, 90 },
    { 0x3cf0e8ff, 100 },
    { 0x68c0e901, 55 },
    { 0xdb30e901, 84 },
    { 0x66b0e902, 99 },
    { 0, 0 },
    { 0xc210e905, 97 },
    { 0x5d70e905, 67 },
    { 0xe580e907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9860e90c, 61 },
    { 0x5ed0e90d, 87 },
    { 0xc690e90e, 97 },
    { 0x8960e90f, 61 },
    { 0xf630e910, 53 },
    { 0xe150e911, 23 },
    { 0xf720e912, 87 },
    { 0xcb70e913, 77 },
    { 0xd520e914, 40 },
    { 0xefe0e914, 46 },
    { 0x6080e913, 76 },
    { 0xbf30e90c, 43 },
    { 0xb3c0e90e, 33 },
    { 0xd8b0e919, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf0e920, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f40e929, 90 },
    { 0xa690e929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d0e92e, 90 },
    { 0x3e60e92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80e933, 90 },
    { 0xb170e934, 46 },
    { 0, 0 },
    { 0x8160e936, 90 },
    { 0x48a0e937, 91 },
    { 0x41b0e938, 85 },
    { 0x8110e939, 33 },
    { 0x3c10e93a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a0e93d, 95 },
    { 0x4e00e93e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a60e941, 61 },
    { 0x6dd0e942, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db0e946, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70e951, 93 },
    { 0, 0 },
    { 0xa840e953, 90 },
    { 0xefd0e954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0e95a, 100 },
    { 0x7f10e95b, 76 },
    { 0, 0 },
    { 0xc00e95d, 34 },
    { 0xcaf0e95e, 33 },
    { 0xd70e95e, 91 },
    { 0xe840e960, 90 },
    { 0x3da0e960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7360e967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0e96a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd0e96e, 33 },
    { 0xb3b0e96f, 90 },
    { 0, 0 },
    { 0xfbb0e971, 33 },
    { 0xcdb0e971, 43 },
    { 0xb850e972, 90 },
    { 0x2d0e974, 95 },
    { 0xb1a0e974, 88 },
    { 0x1860e976, 93 },
    { 0xb5d0e977, 61 },
    { 0xd0e978, 55 },
    { 0xfe40e979, 90 },
    { 0xdb30e978, 90 },
    { 0x36b0e97b, 97 },
    { 0, 0 },
    { 0xdd30e97d, 95 },
    { 0, 0 },
    { 0x6170e97f, 33 },
    { 0xe5e0e980, 33 },
    { 0xb200e97f, 95 },
    { 0xa1b0e981, 90 },
    { 0, 0 },
    { 0xcfa0e984, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7120e990, 90 },
    { 0xe4b0e991, 33 },
    { 0, 0 },
    { 0x8740e993, 33 },
    { 0x46b0e993, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570e999, 91 },
    { 0x910e999, 23 },
    { 0x6040e99b, 78 },
    { 0xc2f0e99c, 91 },
    { 0x71c0e999, 95 },
    { 0x2760e99e, 23 },
    { 0xb490e99f, 33 },
    { 0x9e40e9a0, 91 },
    { 0x15e0e99f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c0e9a5, 90 },
    { 0xbfa0e9a5, 65 },
    { 0xaba0e9a5, 23 },
    { 0x6fd0e9a7, 90 },
    { 0, 0 },
    { 0xfc80e9aa, 87 },
    { 0x36e0e9ab, 23 },
    { 0x9920e9aa, 34 },
    { 0, 0 },
    { 0xb730e9ae, 76 },
    { 0x74a0e9ae, 90 },
    { 0, 0 },
    { 0x44c0e9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0e9b4, 23 },
    { 0x4ea0e9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380e9b8, 55 },
    { 0x37a0e9b9, 77 },
    { 0xb8b0e9ba, 23 },
    { 0x7750e9bb, 91 },
    { 0x9000e9bb, 33 },
    { 0x4f10e9bd, 40 },
    { 0, 0 },
    { 0x6700e9bf, 61 },
    { 0, 0 },
    { 0xc4e0e9c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6340e9c4, 88 },
    { 0xff80e9c5, 98 },
    { 0xb140e9c6, 98 },
    { 0x9930e9c6, 98 },
    { 0xb100e9c6, 93 },
    { 0, 0 },
    { 0xdf20e9ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3060e9ce, 33 },
    { 0xc2f0e9ce, 90 },
    { 0xb730e9cf, 33 },
    { 0xea0e9d1, 95 },
    { 0x4760e9d0, 98 },
    { 0x6490e9d1, 43 },
    { 0x6670e9d4, 87 },
    { 0x6050e9d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d50e9d9, 46 },
    { 0xce10e9da, 48 },
    { 0x59c0e9db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b70e9e1, 34 },
    { 0, 0 },
    { 0x15a0e9e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4890e9e9, 95 },
    { 0xd5b0e9ea, 95 },
    { 0x4a30e9eb, 99 },
    { 0x9960e9ec, 33 },
    { 0x2610e9ed, 52 },
    { 0, 0 },
    { 0x3650e9ef, 98 },
    { 0x6830e9f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb0e9fa, 95 },
    { 0x35a0e9fb, 90 },
    { 0, 0 },
    { 0x8ce0e9fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c0ea05, 92 },
    { 0, 0 },
    { 0xab90ea07, 23 },
    { 0, 0 },
    { 0x5880ea09, 98 },
    { 0, 0 },
    { 0x9a60ea0b, 98 },
    { 0xbf10ea0b, 90 },
    { 0x5b0ea0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb0ea10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x210ea13, 98 },
    { 0xa430ea13, 55 },
    { 0x7310ea15, 90 },
    { 0xe180ea16, 98 },
    { 0x4c00ea14, 55 },
    { 0x7ae0ea18, 19 },
    { 0xcc60ea19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb780ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380ea20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10ea23, 61 },
    { 0, 0 },
    { 0x8e90ea25, 90 },
    { 0xe620ea25, 90 },
    { 0xac80ea26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea60ea2a, 98 },
    { 0, 0 },
    { 0x1040ea2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920ea36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb540ea3d, 61 },
    { 0x8f40ea3e, 91 },
    { 0xae90ea3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2570ea42, 81 },
    { 0x3d50ea42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe990ea4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780ea4f, 61 },
    { 0x56a0ea50, 98 },
    { 0x23c0ea50, 52 },
    { 0xb710ea52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0ea57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d0ea60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ea65, 98 },
    { 0x23b0ea66, 33 },
    { 0, 0 },
    { 0xe1f0ea68, 98 },
    { 0x67a0ea68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e40ea6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f60ea71, 90 },
    { 0xb4e0ea72, 90 },
    { 0x1c80ea72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b0ea7b, 61 },
    { 0x3470ea7c, 55 },
    { 0xbd30ea7d, 33 },
    { 0, 0 },
    { 0x6510ea7f, 61 },
    { 0xe070ea7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7220ea84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0ea88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf430ea8d, 98 },
    { 0, 0 },
    { 0x2720ea8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f0ea92, 34 },
    { 0, 0 },
    { 0x4ac0ea94, 23 },
    { 0xe460ea95, 52 },
    { 0xb000ea96, 61 },
    { 0, 0 },
    { 0x4520ea98, 93 },
    { 0x6fc0ea99, 95 },
    { 0xde90ea9a, 90 },
    { 0, 0 },
    { 0x6460ea9c, 43 },
    { 0, 0 },
    { 0x6950ea9e, 33 },
    { 0x61a0ea9f, 23 },
    { 0xf6a0eaa0, 98 },
    { 0x9d20eaa0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0eaa4, 90 },
    { 0x2e40eaa5, 98 },
    { 0, 0 },
    { 0x73d0eaa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf10eaab, 77 },
    { 0x2990eaac, 95 },
    { 0xcae0eaac, 98 },
    { 0x7030eaab, 61 },
    { 0x72a0eaaf, 36 },
    { 0x4640eaaf, 90 },
    { 0x2cf0eaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5240eab4, 90 },
    { 0, 0 },
    { 0x8c20eab6, 90 },
    { 0x6a70eab7, 61 },
    { 0, 0 },
    { 0x2e50eab9, 34 },
    { 0, 0 },
    { 0x8b30eabb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6700eac0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1620eac3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550eaca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650eace, 61 },
    { 0xb2e0eacf, 61 },
    { 0x4340eace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc150ead9, 23 },
    { 0x99e0eada, 95 },
    { 0x7aa0eadb, 98 },
    { 0x5140eadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120eae0, 93 },
    { 0x5790eae1, 99 },
    { 0x5650eae0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6080eae9, 67 },
    { 0xadc0eaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b0eaed, 90 },
    { 0xad80eaed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e0eaf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f0eaf8, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7850eafe, 61 },
    { 0xc340eafe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1150eb02, 33 },
    { 0xbb0eb02, 96 },
    { 0xcaf0eb04, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa0eb09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc620eb0e, 61 },
    { 0, 0 },
    { 0xd320eb10, 93 },
    { 0xee10eb11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380eb18, 17 },
    { 0xb830eb19, 95 },
    { 0xcd80eb19, 90 },
    { 0xc0eb1b, 77 },
    { 0, 0 },
    { 0xe6a0eb1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf00eb26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x710eb29, 33 },
    { 0xb030eb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0eb2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a0eb33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc0eb37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b0eb3a, 33 },
    { 0x5a10eb3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc800eb40, 69 },
    { 0xaa0eb41, 77 },
    { 0, 0 },
    { 0xab90eb43, 99 },
    { 0xcd10eb44, 34 },
    { 0, 0 },
    { 0x4c70eb46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1710eb4d, 90 },
    { 0xcad0eb4e, 90 },
    { 0xdeb0eb4e, 95 },
    { 0, 0 },
    { 0xff80eb51, 90 },
    { 0, 0 },
    { 0xa9b0eb53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6490eb56, 33 },
    { 0xcec0eb57, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x710eb5a, 90 },
    { 0, 0 },
    { 0x5380eb5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2170eb60, 85 },
    { 0xf460eb61, 33 },
    { 0xde30eb61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c50eb71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac0eb7a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb610eb7d, 90 },
    { 0x2230eb7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810eb84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d0eb87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f90eb8a, 61 },
    { 0, 0 },
    { 0xc720eb8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f0eb8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd0eb99, 33 },
    { 0, 0 },
    { 0x6020eb9b, 61 },
    { 0xd350eb9c, 61 },
    { 0xc4b0eb9c, 34 },
    { 0x66a0eb9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0eba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e0ebaa, 33 },
    { 0xca50ebab, 85 },
    { 0x6b80ebab, 99 },
    { 0x2100ebad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7010ebb0, 98 },
    { 0x7fe0ebb0, 43 },
    { 0xac80ebb2, 98 },
    { 0x14a0ebb0, 23 },
    { 0xce50ebb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f0ebb7, 93 },
    { 0xb800ebb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80ebbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870ebc5, 33 },
    { 0, 0 },
    { 0x180ebc7, 61 },
    { 0, 0 },
    { 0x72b0ebc9, 78 },
    { 0x9970ebca, 61 },
    { 0x58e0ebcb, 33 },
    { 0x4d30ebcc, 96 },
    { 0x30e0ebcd, 98 },
    { 0x400ebce, 34 },
    { 0xa730ebce, 98 },
    { 0xc610ebcb, 76 },
    { 0xfce0ebd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca0ebd6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc0ebda, 90 },
    { 0xf150ebdb, 33 },
    { 0x7620ebdc, 99 },
    { 0xa7e0ebda, 33 },
    { 0xd030ebdd, 33 },
    { 0x15a0ebdf, 98 },
    { 0x38d0ebe0, 76 },
    { 0xb810ebe1, 90 },
    { 0xfba0ebdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb0ebe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30ebeb, 80 },
    { 0x3570ebec, 90 },
    { 0x7ee0ebed, 55 },
    { 0x2590ebed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf160ebf1, 55 },
    { 0x2890ebf2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0ebfc, 95 },
    { 0, 0 },
    { 0x7bb0ebfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c30ec07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed0ec0e, 46 },
    { 0x7250ec0f, 80 },
    { 0, 0 },
    { 0x8340ec11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e80ec14, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc210ec1c, 46 },
    { 0, 0 },
    { 0x9c0ec1e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140ec21, 33 },
    { 0x9f50ec22, 55 },
    { 0x92b0ec23, 61 },
    { 0xbeb0ec24, 93 },
    { 0x73f0ec25, 61 },
    { 0, 0 },
    { 0xdc50ec27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7330ec2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320ec31, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd0ec35, 33 },
    { 0, 0 },
    { 0xcbe0ec37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ec3a, 98 },
    { 0xa230ec3b, 98 },
    { 0x3c30ec3c, 52 },
    { 0x1aa0ec3c, 33 },
    { 0xffb0ec3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d0ec4a, 23 },
    { 0, 0 },
    { 0x9280ec4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b90ec54, 66 },
    { 0, 0 },
    { 0x78a0ec56, 33 },
    { 0x2650ec57, 98 },
    { 0x6fa0ec57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec5b, 76 },
    { 0, 0 },
    { 0x4270ec5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50ec62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00ec67, 76 },
    { 0x8560ec68, 33 },
    { 0x1280ec67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3150ec6c, 84 },
    { 0, 0 },
    { 0x83d0ec6e, 90 },
    { 0x8980ec6f, 23 },
    { 0x2a30ec70, 100 },
    { 0xa4a0ec6f, 93 },
    { 0xb160ec6f, 23 },
    { 0xbea0ec73, 55 },
    { 0x3870ec73, 61 },
    { 0xdd20ec71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4350ec7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790ec7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110ec87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb60ec8b, 98 },
    { 0x1330ec8c, 46 },
    { 0, 0 },
    { 0x8750ec8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b60ec96, 78 },
    { 0x24c0ec96, 81 },
    { 0x9a10ec98, 90 },
    { 0x2000ec98, 93 },
    { 0x3380ec9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb10ec9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b50eca0, 90 },
    { 0xe4d0eca1, 90 },
    { 0xb400eca0, 33 },
    { 0, 0 },
    { 0x97c0eca4, 23 },
    { 0x6930eca5, 76 },
    { 0x970eca5, 77 },
    { 0, 0 },
    { 0xf0b0eca8, 9 },
    { 0x76d0eca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb900ecae, 90 },
    { 0, 0 },
    { 0xa1d0ecb0, 94 },
    { 0xb4d0ecb1, 46 },
    { 0x38f0ecb0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790ecba, 77 },
    { 0x41f0ecbb, 46 },
    { 0, 0 },
    { 0xcc0ecbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a0ecc0, 34 },
    { 0xfb40ecc1, 95 },
    { 0x4580ecc2, 80 },
    { 0x2af0ecc3, 33 },
    { 0, 0 },
    { 0x8130ecc5, 98 },
    { 0x410ecc5, 61 },
    { 0x8d40ecc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf040eccb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd150ecce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d0ecd1, 43 },
    { 0, 0 },
    { 0x6a90ecd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac0ecd8, 68 },
    { 0xf3c0ecd8, 66 },
    { 0, 0 },
    { 0xd0a0ecdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9070ecde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0ece7, 43 },
    { 0xcd40ece8, 90 },
    { 0x9650ece9, 95 },
    { 0x6f50ece8, 61 },
    { 0x6890eceb, 61 },
    { 0x1780eceb, 84 },
    { 0xe60eceb, 33 },
    { 0xa170ecee, 61 },
    { 0xe870ecef, 90 },
    { 0x2b90ecf0, 61 },
    { 0x7810ecf1, 95 },
    { 0xc5a0ecf2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d0ecf6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6220ecfa, 93 },
    { 0xab50ecfb, 76 },
    { 0x88e0ecfa, 93 },
    { 0x9f10ecfd, 46 },
    { 0x6c10ecfd, 61 },
    { 0xf550ecff, 98 },
    { 0xbec0ed00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b00ed04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c0ed07, 17 },
    { 0, 0 },
    { 0xab90ed09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9550ed10, 23 },
    { 0, 0 },
    { 0x8700ed12, 87 },
    { 0x9830ed13, 95 },
    { 0x4f0ed13, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb100ed18, 87 },
    { 0x69c0ed19, 46 },
    { 0xfa60ed19, 33 },
    { 0xa270ed1b, 46 },
    { 0xfa10ed19, 46 },
    { 0x24b0ed1d, 23 },
    { 0xfc90ed1e, 43 },
    { 0xb190ed1e, 95 },
    { 0, 0 },
    { 0x32e0ed21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd90ed28, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ed2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3690ed3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d50ed3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00ed44, 55 },
    { 0xfef0ed44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560ed49, 66 },
    { 0xb7b0ed4a, 95 },
    { 0x49f0ed4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060ed4e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1240ed53, 61 },
    { 0x4cd0ed53, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9620ed57, 33 },
    { 0xb970ed58, 23 },
    { 0, 0 },
    { 0xfcb0ed5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc590ed5d, 64 },
    { 0x4b90ed5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ed61, 77 },
    { 0x9dc0ed62, 88 },
    { 0, 0 },
    { 0x8570ed64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef0ed68, 93 },
    { 0xce20ed68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb800ed6c, 93 },
    { 0x6660ed6d, 33 },
    { 0x2e00ed6e, 59 },
    { 0x1250ed6f, 95 },
    { 0x7270ed70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540ed73, 56 },
    { 0xc4e0ed74, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac0ed78, 90 },
    { 0x9cf0ed78, 91 },
    { 0xe730ed7a, 98 },
    { 0xb430ed7b, 23 },
    { 0, 0 },
    { 0x960ed7d, 61 },
    { 0, 0 },
    { 0xcc30ed7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc980ed83, 95 },
    { 0x2dd0ed84, 98 },
    { 0xc9f0ed85, 33 },
    { 0x8ce0ed86, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca60ed8f, 33 },
    { 0xe690ed90, 73 },
    { 0xe130ed91, 33 },
    { 0, 0 },
    { 0x6c80ed93, 90 },
    { 0x14f0ed93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce40ed98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ed9e, 98 },
    { 0x4450ed9f, 61 },
    { 0xc050ed9f, 75 },
    { 0x4510ed9f, 91 },
    { 0, 0 },
    { 0x56c0eda3, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a0eda6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc0eda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210edaf, 33 },
    { 0, 0 },
    { 0x4270edb1, 87 },
    { 0xaf00edb2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c0edb6, 46 },
    { 0, 0 },
    { 0xeb00edb8, 66 },
    { 0x3a0edb9, 23 },
    { 0x7e20edba, 61 },
    { 0x8ed0edba, 23 },
    { 0x9ef0edbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1640edc1, 23 },
    { 0xd250edc1, 23 },
    { 0x1080edc1, 61 },
    { 0, 0 },
    { 0x5d50edc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff0edc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae00edcb, 92 },
    { 0x7b30edcc, 81 },
    { 0, 0 },
    { 0xd6d0edce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b30edd3, 66 },
    { 0x17f0edd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0edd9, 90 },
    { 0x66a0edda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a80eddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd260ede2, 95 },
    { 0x3fc0ede3, 95 },
    { 0, 0 },
    { 0x8dc0ede5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc0ede9, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630edec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac0edf0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30edf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb850edfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5820ee03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f50ee09, 86 },
    { 0, 0 },
    { 0x3d0ee0b, 90 },
    { 0x4c90ee0b, 61 },
    { 0x1930ee0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9940ee10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810ee15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd10ee18, 61 },
    { 0xb400ee19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe00ee1c, 55 },
    { 0x39a0ee1d, 66 },
    { 0xa820ee1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00ee21, 95 },
    { 0x3150ee21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc0ee26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0ee2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c0ee39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0ee3e, 40 },
    { 0x16b0ee3f, 61 },
    { 0x1360ee3f, 33 },
    { 0x3c80ee41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740ee46, 17 },
    { 0xf4b0ee47, 95 },
    { 0xbb30ee47, 61 },
    { 0xbfe0ee46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20ee4c, 95 },
    { 0xca90ee4d, 24 },
    { 0xc120ee4e, 98 },
    { 0x4b70ee4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8880ee53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c00ee57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c10ee5a, 55 },
    { 0xe0d0ee5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d30ee5e, 95 },
    { 0x7de0ee5f, 60 },
    { 0x7990ee60, 33 },
    { 0x2190ee60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50ee6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0ee6e, 55 },
    { 0x9e10ee6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60ee76, 95 },
    { 0x3f10ee76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90ee7a, 76 },
    { 0, 0 },
    { 0x5430ee7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe0ee84, 90 },
    { 0xef50ee84, 91 },
    { 0x6450ee84, 61 },
    { 0x1e60ee87, 61 },
    { 0x5af0ee87, 61 },
    { 0xda20ee88, 98 },
    { 0xd0b0ee89, 98 },
    { 0xff30ee8b, 76 },
    { 0x6190ee8c, 33 },
    { 0xdef0ee8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc560ee90, 46 },
    { 0xd170ee91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4830ee97, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f0ee9a, 61 },
    { 0, 0 },
    { 0x3b60ee9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad0ee9f, 101 },
    { 0x1430ee9f, 52 },
    { 0x2120eea0, 61 },
    { 0xf2c0eea0, 85 },
    { 0xaaf0eea3, 99 },
    { 0x73f0eea3, 80 },
    { 0xa6f0eea3, 87 },
    { 0x5850eea5, 98 },
    { 0, 0 },
    { 0x3980eea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd0eeae, 61 },
    { 0x5fb0eeaf, 61 },
    { 0x27f0eeb0, 61 },
    { 0x2720eeb1, 98 },
    { 0xba80eeb2, 90 },
    { 0, 0 },
    { 0x7900eeb4, 55 },
    { 0xbc90eeb4, 61 },
    { 0x8e80eeb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad00eeba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b60eec1, 46 },
    { 0x2d00eec1, 61 },
    { 0, 0 },
    { 0x310eec4, 98 },
    { 0x40b0eec4, 84 },
    { 0x4250eec4, 34 },
    { 0, 0 },
    { 0x91a0eec8, 55 },
    { 0, 0 },
    { 0x7c80eeca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4790eed4, 40 },
    { 0x850eed5, 98 },
    { 0x3890eed5, 98 },
    { 0x76c0eed4, 92 },
    { 0, 0 },
    { 0x9530eed9, 52 },
    { 0x53f0eed9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0eede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec10eee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc0eee9, 84 },
    { 0xa1f0eeea, 95 },
    { 0x89b0eeeb, 90 },
    { 0x9220eeeb, 81 },
    { 0, 0 },
    { 0xb670eeee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720eef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0eef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540eefe, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e60ef05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9700ef08, 61 },
    { 0x5fa0ef09, 100 },
    { 0x51b0ef0a, 98 },
    { 0xa7d0ef0a, 69 },
    { 0x1990ef0c, 61 },
    { 0xa00ef0c, 90 },
    { 0x47f0ef0e, 77 },
    { 0xa000ef0f, 90 },
    { 0x7d40ef10, 100 },
    { 0x4ab0ef0a, 33 },
    { 0x1e10ef12, 55 },
    { 0x93a0ef12, 64 },
    { 0x5560ef0b, 33 },
    { 0x1810ef15, 95 },
    { 0xef90ef10, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xf770ef19, 98 },
    { 0x3520ef1a, 34 },
    { 0xb130ef19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0ef1e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1450ef21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8240ef2a, 87 },
    { 0xef60ef2b, 38 },
    { 0xebf0ef2c, 95 },
    { 0, 0 },
    { 0x2c0ef2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910ef31, 23 },
    { 0, 0 },
    { 0xe540ef33, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a40ef36, 33 },
    { 0x66e0ef37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f10ef3a, 55 },
    { 0xb600ef3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b0ef3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7820ef46, 99 },
    { 0, 0 },
    { 0xfa60ef48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6820ef4f, 75 },
    { 0, 0 },
    { 0xd5b0ef51, 95 },
    { 0x81c0ef52, 90 },
    { 0xf400ef52, 95 },
    { 0x270ef52, 90 },
    { 0x81d0ef52, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0ef59, 95 },
    { 0, 0 },
    { 0xa630ef5b, 90 },
    { 0x7190ef5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0ef65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa030ef68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ef6b, 87 },
    { 0, 0 },
    { 0x200ef6d, 34 },
    { 0, 0 },
    { 0xc6e0ef6f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e0ef72, 90 },
    { 0xdb00ef73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0ef79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0ef7d, 23 },
    { 0, 0 },
    { 0xb6e0ef7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1950ef83, 53 },
    { 0, 0 },
    { 0xee80ef85, 55 },
    { 0xd730ef86, 23 },
    { 0, 0 },
    { 0xb300ef88, 93 },
    { 0xb410ef88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680ef8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df0ef94, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c0ef98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0ef9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb90ef9e, 98 },
    { 0xf620ef9e, 40 },
    { 0xf290ef9f, 34 },
    { 0, 0 },
    { 0x700efa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4940efa6, 23 },
    { 0, 0 },
    { 0xe790efa8, 98 },
    { 0x9b50efa9, 23 },
    { 0x5d10efa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60efad, 95 },
    { 0x1c90efad, 81 },
    { 0x680efae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda0efb4, 51 },
    { 0, 0 },
    { 0x9590efb6, 45 },
    { 0xd330efb7, 61 },
    { 0xd7f0efb7, 61 },
    { 0x14a0efb9, 78 },
    { 0x8fc0efb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf70efbe, 100 },
    { 0x770efbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0efc2, 99 },
    { 0x6440efc3, 98 },
    { 0x3b30efc4, 88 },
    { 0, 0 },
    { 0x66b0efc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb00efc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff0efcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf640efd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee0efd7, 98 },
    { 0x6a80efd8, 95 },
    { 0, 0 },
    { 0x8980efda, 61 },
    { 0x7780efda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3230efde, 55 },
    { 0xe870efde, 61 },
    { 0xc080efe0, 65 },
    { 0x5d20efe1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b0efe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380efea, 61 },
    { 0x60c0efeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f0efef, 46 },
    { 0, 0 },
    { 0x2030eff1, 61 },
    { 0x170eff2, 98 },
    { 0x8760eff3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390eff6, 43 },
    { 0x7d70eff7, 77 },
    { 0xbe70eff6, 23 },
    { 0x1390eff9, 65 },
    { 0x3190eff8, 55 },
    { 0x3c70eff6, 33 },
    { 0, 0 },
    { 0x1420effd, 95 },
    { 0x6ac0effe, 88 },
    { 0x6870effe, 33 },
    { 0x5f40f000, 98 },
    { 0, 0 },
    { 0x9b40f002, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f70f007, 55 },
    { 0xe700f007, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb0f00c, 33 },
    { 0x4470f00d, 76 },
    { 0xdde0f00e, 33 },
    { 0x7e60f00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f30f013, 95 },
    { 0xb1b0f013, 52 },
    { 0xcf50f015, 66 },
    { 0xc6d0f014, 96 },
    { 0xf990f017, 90 },
    { 0xb7e0f014, 99 },
    { 0, 0 },
    { 0x2c0f01a, 55 },
    { 0x2ba0f01a, 46 },
    { 0x2e30f01c, 81 },
    { 0, 0 },
    { 0xebe0f01e, 53 },
    { 0, 0 },
    { 0xa50f020, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e00f023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe760f026, 90 },
    { 0x86a0f026, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a0f037, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3470f03c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2330f042, 34 },
    { 0x5230f043, 90 },
    { 0x70b0f043, 61 },
    { 0xbb30f045, 78 },
    { 0x400f043, 33 },
    { 0x8860f047, 95 },
    { 0x4840f047, 61 },
    { 0xc6f0f048, 95 },
    { 0x80c0f047, 90 },
    { 0xdce0f04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0f055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd0f058, 91 },
    { 0x3cf0f058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd220f05c, 85 },
    { 0, 0 },
    { 0x84a0f05e, 95 },
    { 0x7a40f05f, 36 },
    { 0x690f060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d30f064, 76 },
    { 0, 0 },
    { 0x5580f066, 77 },
    { 0xe9c0f067, 91 },
    { 0, 0 },
    { 0x8840f069, 77 },
    { 0, 0 },
    { 0xfd90f06b, 90 },
    { 0, 0 },
    { 0x5750f06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80f076, 33 },
    { 0xb450f077, 61 },
    { 0x97e0f077, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2450f07e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d0f082, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa980f085, 95 },
    { 0xee20f085, 91 },
    { 0x4a70f085, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e0f091, 23 },
    { 0, 0 },
    { 0x3940f093, 33 },
    { 0, 0 },
    { 0xcc80f095, 33 },
    { 0x5ff0f096, 84 },
    { 0x1e90f095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec0f09b, 93 },
    { 0x5f70f09c, 66 },
    { 0, 0 },
    { 0x6a30f09e, 95 },
    { 0xb550f09e, 61 },
    { 0x6820f0a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00f0a4, 61 },
    { 0x6f20f0a5, 33 },
    { 0xfc50f0a5, 43 },
    { 0xd3c0f0a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6850f0aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a0f0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d0f0b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020f0b9, 99 },
    { 0xb7d0f0b9, 78 },
    { 0xe240f0ba, 34 },
    { 0x2790f0bc, 61 },
    { 0x23f0f0bd, 93 },
    { 0x740f0be, 77 },
    { 0x4130f0bf, 93 },
    { 0, 0 },
    { 0x6d20f0c1, 90 },
    { 0, 0 },
    { 0xe8c0f0c3, 90 },
    { 0x6a0f0c3, 67 },
    { 0xf650f0c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e0f0c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0f0ce, 33 },
    { 0xe140f0cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e30f0d7, 46 },
    { 0, 0 },
    { 0x3a20f0d9, 85 },
    { 0, 0 },
    { 0x100f0db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d60f0de, 98 },
    { 0x4980f0df, 43 },
    { 0x8390f0de, 61 },
    { 0, 0 },
    { 0x4820f0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c50f0e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6800f0ed, 17 },
    { 0, 0 },
    { 0x2170f0ef, 91 },
    { 0xd610f0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe470f0f3, 61 },
    { 0x8060f0f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c60f0f9, 34 },
    { 0x2380f0fa, 99 },
    { 0x6a20f0f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0f102, 55 },
    { 0x9440f103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf550f106, 95 },
    { 0xc40f106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0f10c, 78 },
    { 0x91f0f10d, 55 },
    { 0x1300f10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad10f119, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570f11c, 61 },
    { 0, 0 },
    { 0x31f0f11e, 33 },
    { 0x53e0f11f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d0f123, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0f12a, 90 },
    { 0x7d70f12b, 68 },
    { 0x6eb0f12c, 90 },
    { 0xf3b0f12b, 98 },
    { 0xe390f12e, 33 },
    { 0xa030f12e, 67 },
    { 0x3d20f130, 98 },
    { 0xf400f12d, 33 },
    { 0xe2c0f12c, 34 },
    { 0x140f12a, 90 },
    { 0x4680f131, 99 },
    { 0xef90f135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c0f13f, 33 },
    { 0xef70f140, 95 },
    { 0x7a60f140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe650f145, 43 },
    { 0x7840f146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee80f14e, 23 },
    { 0, 0 },
    { 0x3790f150, 98 },
    { 0x99f0f151, 52 },
    { 0xe430f152, 34 },
    { 0xa7d0f153, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c30f15a, 48 },
    { 0, 0 },
    { 0x8890f15c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c0f15f, 52 },
    { 0xe8a0f160, 91 },
    { 0, 0 },
    { 0x1740f162, 87 },
    { 0x97a0f162, 17 },
    { 0x21e0f164, 33 },
    { 0x5b50f165, 90 },
    { 0, 0 },
    { 0x1430f167, 90 },
    { 0xaa20f168, 52 },
    { 0x3170f169, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee0f170, 61 },
    { 0, 0 },
    { 0x6190f172, 55 },
    { 0, 0 },
    { 0x4350f174, 40 },
    { 0xd550f174, 33 },
    { 0, 0 },
    { 0xc070f177, 93 },
    { 0x6cb0f177, 95 },
    { 0x9520f177, 34 },
    { 0x13a0f17a, 77 },
    { 0, 0 },
    { 0x94d0f17c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe280f180, 23 },
    { 0, 0 },
    { 0x40b0f182, 76 },
    { 0x1a90f182, 98 },
    { 0xc530f184, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8470f189, 33 },
    { 0xa740f18a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae50f18e, 91 },
    { 0, 0 },
    { 0x85b0f190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf010f196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd0f19c, 46 },
    { 0x6a20f19d, 33 },
    { 0xe50f19e, 69 },
    { 0, 0 },
    { 0x700f1a0, 34 },
    { 0x8640f1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b0f1a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7770f1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c0f1ac, 90 },
    { 0x4ab0f1ad, 90 },
    { 0x3dc0f1ad, 90 },
    { 0x2300f1af, 95 },
    { 0x8780f1ae, 23 },
    { 0, 0 },
    { 0xa250f1b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3400f1b5, 23 },
    { 0, 0 },
    { 0xc460f1b7, 100 },
    { 0xd460f1b7, 34 },
    { 0xea30f1b9, 61 },
    { 0x9b60f1b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e0f1c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa0f1c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f90f1c9, 33 },
    { 0x9440f1ca, 99 },
    { 0x9cd0f1cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe080f1cf, 66 },
    { 0x4f0f1cf, 84 },
    { 0x4660f1d1, 61 },
    { 0x3cc0f1cf, 76 },
    { 0x4520f1d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f20f1db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb490f1e0, 90 },
    { 0x1770f1e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330f1e5, 46 },
    { 0, 0 },
    { 0xae70f1e7, 76 },
    { 0, 0 },
    { 0x9750f1e9, 90 },
    { 0x9cc0f1e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f1f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a10f1f5, 33 },
    { 0x53a0f1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530f1f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f40f1ff, 23 },
    { 0x8c90f200, 66 },
    { 0, 0 },
    { 0x9c50f202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab0f206, 93 },
    { 0x5900f207, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050f212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0f215, 55 },
    { 0xd4e0f216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f0f219, 55 },
    { 0x22f0f21a, 61 },
    { 0x6530f21b, 87 },
    { 0x460f21c, 90 },
    { 0x6a30f21c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca0f223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb520f227, 61 },
    { 0xff0f228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed0f22b, 61 },
    { 0xecc0f22c, 90 },
    { 0xae20f22d, 98 },
    { 0xd20f22d, 98 },
    { 0xa640f22f, 33 },
    { 0x59a0f230, 34 },
    { 0, 0 },
    { 0xefc0f232, 33 },
    { 0x6970f233, 66 },
    { 0x61f0f233, 61 },
    { 0x91f0f234, 88 },
    { 0x700f234, 34 },
    { 0x16d0f237, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d60f23b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c70f23f, 90 },
    { 0xc560f240, 84 },
    { 0x89b0f240, 23 },
    { 0x9020f242, 2 },
    { 0x92e0f243, 72 },
    { 0x7c70f244, 61 },
    { 0x59f0f245, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2300f248, 46 },
    { 0, 0 },
    { 0xcd90f24a, 68 },
    { 0xa3c0f24b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b0f250, 80 },
    { 0x6540f250, 64 },
    { 0x5fa0f252, 61 },
    { 0, 0 },
    { 0xed30f254, 98 },
    { 0x4a0f254, 90 },
    { 0, 0 },
    { 0xee80f257, 95 },
    { 0x3c90f258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910f25b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc0f25f, 85 },
    { 0x7500f260, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f26f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4010f275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c80f279, 90 },
    { 0x1f00f279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd080f282, 33 },
    { 0x30e0f283, 90 },
    { 0x3980f283, 61 },
    { 0xb190f285, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b60f288, 33 },
    { 0, 0 },
    { 0x1f70f28a, 90 },
    { 0, 0 },
    { 0x5560f28c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd0f294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380f297, 90 },
    { 0xc710f298, 95 },
    { 0, 0 },
    { 0x6f80f29a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2690f29f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0f2a2, 98 },
    { 0xbfc0f2a3, 43 },
    { 0x7860f2a4, 78 },
    { 0, 0 },
    { 0x8330f2a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6780f2ab, 95 },
    { 0x2be0f2ac, 90 },
    { 0xf7f0f2ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf520f2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0f2b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280f2bb, 95 },
    { 0x78f0f2bc, 55 },
    { 0x2450f2bb, 80 },
    { 0x4cf0f2be, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc0f2c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc50f2ce, 43 },
    { 0xa5a0f2cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170f2d2, 55 },
    { 0, 0 },
    { 0xfc30f2d4, 78 },
    { 0xb500f2d5, 80 },
    { 0xfcf0f2d6, 52 },
    { 0x6550f2d7, 55 },
    { 0xaf80f2d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9710f2dd, 26 },
    { 0, 0 },
    { 0x8fa0f2df, 98 },
    { 0, 0 },
    { 0x1fb0f2e1, 68 },
    { 0xf700f2e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960f2e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e70f2ec, 90 },
    { 0x9f50f2ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5590f2fa, 95 },
    { 0, 0 },
    { 0xd0f2fc, 98 },
    { 0x2a10f2fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650f300, 23 },
    { 0x6b70f300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d0f304, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940f30b, 61 },
    { 0xf350f30c, 61 },
    { 0x1130f30d, 43 },
    { 0xfc90f30e, 76 },
    { 0x7180f30e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7100f313, 84 },
    { 0xaa70f314, 33 },
    { 0xe340f315, 90 },
    { 0xf4b0f316, 61 },
    { 0xb560f316, 98 },
    { 0xb060f318, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0f31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed80f321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f30f324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d40f329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed80f32e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd000f338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50f33d, 90 },
    { 0xa50f33e, 83 },
    { 0x7180f33f, 33 },
    { 0, 0 },
    { 0xf7b0f341, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5410f345, 23 },
    { 0xb100f346, 95 },
    { 0x8370f347, 45 },
    { 0x3540f346, 90 },
    { 0xd120f349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c20f34d, 41 },
    { 0x16e0f34d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0f354, 36 },
    { 0xc1e0f354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570f359, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a0f35d, 66 },
    { 0x4550f35e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1380f362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7610f365, 90 },
    { 0, 0 },
    { 0xaeb0f367, 90 },
    { 0, 0 },
    { 0x7d40f369, 78 },
    { 0, 0 },
    { 0x7980f36b, 61 },
    { 0xec80f36c, 61 },
    { 0x8560f36d, 23 },
    { 0x74c0f36e, 90 },
    { 0xe710f36c, 59 },
    { 0xf540f370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40f374, 65 },
    { 0xc7d0f374, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6960f378, 98 },
    { 0, 0 },
    { 0xd550f37a, 95 },
    { 0, 0 },
    { 0x7470f37c, 90 },
    { 0x9450f37c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e00f380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c0f384, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750f387, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0f38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0f38f, 93 },
    { 0xa60f390, 56 },
    { 0x3180f391, 61 },
    { 0x22f0f392, 98 },
    { 0x3f70f392, 94 },
    { 0xdec0f390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f399, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360f3a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810f3a9, 34 },
    { 0x4e80f3a9, 34 },
    { 0, 0 },
    { 0xab30f3ac, 76 },
    { 0, 0 },
    { 0x7cb0f3ae, 76 },
    { 0xe990f3af, 61 },
    { 0xa260f3af, 98 },
    { 0xabd0f3af, 76 },
    { 0x7f90f3b2, 23 },
    { 0, 0 },
    { 0x8670f3b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b80f3b9, 85 },
    { 0xa710f3ba, 61 },
    { 0xe8a0f3ba, 96 },
    { 0xab0f3bb, 34 },
    { 0x38c0f3bd, 84 },
    { 0, 0 },
    { 0xf9c0f3bf, 98 },
    { 0xfef0f3c0, 90 },
    { 0, 0 },
    { 0x9f80f3c2, 95 },
    { 0x32f0f3c3, 78 },
    { 0xf1b0f3c4, 36 },
    { 0x6820f3c5, 52 },
    { 0xf740f3c5, 93 },
    { 0x8470f3c7, 52 },
    { 0, 0 },
    { 0xbad0f3c9, 46 },
    { 0xce90f3c9, 61 },
    { 0x2f50f3ca, 61 },
    { 0xd050f3cc, 90 },
    { 0xcb50f3c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e50f3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b50f3d7, 55 },
    { 0x3290f3d7, 23 },
    { 0x6190f3d7, 66 },
    { 0x1860f3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb50f3dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c40f3e5, 88 },
    { 0xe160f3e6, 90 },
    { 0xb650f3e7, 17 },
    { 0xe860f3e8, 94 },
    { 0, 0 },
    { 0x4270f3ea, 23 },
    { 0, 0 },
    { 0x3590f3ec, 90 },
    { 0, 0 },
    { 0x3ef0f3ee, 76 },
    { 0x6380f3ef, 66 },
    { 0xe080f3ee, 34 },
    { 0, 0 },
    { 0x19e0f3f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2120f3fc, 33 },
    { 0, 0 },
    { 0x1120f3fe, 34 },
    { 0, 0 },
    { 0xf610f400, 33 },
    { 0x3750f401, 33 },
    { 0, 0 },
    { 0x3c50f403, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1000f408, 93 },
    { 0, 0 },
    { 0x4cf0f40a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1820f40e, 34 },
    { 0, 0 },
    { 0x8810f410, 61 },
    { 0, 0 },
    { 0x9be0f412, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8250f415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e0f41b, 35 },
    { 0, 0 },
    { 0xfd80f41d, 55 },
    { 0, 0 },
    { 0x6ca0f41f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c70f423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120f426, 34 },
    { 0xab20f427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4940f42a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80f430, 52 },
    { 0xf10f431, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0f437, 95 },
    { 0, 0 },
    { 0xdf60f439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620f43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b0f442, 99 },
    { 0, 0 },
    { 0xe070f444, 23 },
    { 0xa7a0f445, 76 },
    { 0xc260f446, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640f44b, 90 },
    { 0x5ac0f44c, 66 },
    { 0xaf80f44d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe010f450, 98 },
    { 0x6570f450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2840f456, 84 },
    { 0xe4b0f457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0f45b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f0f45e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280f462, 90 },
    { 0, 0 },
    { 0xa820f464, 87 },
    { 0x9d20f464, 23 },
    { 0xf160f466, 90 },
    { 0x8c80f467, 90 },
    { 0, 0 },
    { 0x3520f469, 61 },
    { 0x980f46a, 61 },
    { 0, 0 },
    { 0xc630f46c, 61 },
    { 0xf1e0f46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0f473, 55 },
    { 0xab90f474, 23 },
    { 0xb0b0f475, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f0f479, 99 },
    { 0, 0 },
    { 0x5950f47b, 95 },
    { 0, 0 },
    { 0x7a10f47d, 84 },
    { 0x1490f47e, 33 },
    { 0x78a0f47e, 33 },
    { 0x2780f47f, 34 },
    { 0xa640f481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f50f487, 87 },
    { 0x7270f488, 33 },
    { 0, 0 },
    { 0x5290f48a, 61 },
    { 0xb300f48b, 55 },
    { 0, 0 },
    { 0x3670f48d, 33 },
    { 0, 0 },
    { 0xee20f48f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc340f493, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa50f496, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b0f499, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc260f4a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0f4ac, 90 },
    { 0xb130f4ad, 43 },
    { 0xb8f0f4ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f50f4b5, 23 },
    { 0xf100f4b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0f4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0f4c0, 23 },
    { 0xcaa0f4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba10f4c5, 90 },
    { 0xe780f4c6, 90 },
    { 0, 0 },
    { 0xede0f4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda20f4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630f4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd710f4e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730f4e5, 33 },
    { 0, 0 },
    { 0xefc0f4e7, 34 },
    { 0xc6c0f4e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd240f4ef, 37 },
    { 0x73e0f4f0, 90 },
    { 0x6640f4f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b0f4f4, 90 },
    { 0xaf00f4f5, 93 },
    { 0xabe0f4f5, 61 },
    { 0xf710f4f5, 23 },
    { 0x2370f4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70f4fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc820f502, 95 },
    { 0x5d0f502, 90 },
    { 0xca70f504, 52 },
    { 0, 0 },
    { 0xbaf0f506, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0f50e, 34 },
    { 0x5970f50f, 68 },
    { 0x53d0f510, 77 },
    { 0xaf70f510, 66 },
    { 0xa6b0f512, 65 },
    { 0x32b0f513, 76 },
    { 0x39a0f514, 85 },
    { 0x1be0f515, 98 },
    { 0x2230f516, 23 },
    { 0xbe90f517, 17 },
    { 0x8700f50f, 61 },
    { 0, 0 },
    { 0x8d90f51a, 59 },
    { 0, 0 },
    { 0x7370f51c, 61 },
    { 0, 0 },
    { 0xb5f0f51e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb760f521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd0f528, 42 },
    { 0x6810f529, 33 },
    { 0x3ca0f528, 61 },
    { 0, 0 },
    { 0x91e0f52c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab00f531, 94 },
    { 0xea70f531, 90 },
    { 0x3ef0f533, 90 },
    { 0, 0 },
    { 0x50a0f535, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd70f538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80f53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb700f542, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0f546, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5530f54b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4120f552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe470f55a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560f55f, 23 },
    { 0xfc80f560, 43 },
    { 0x1c40f55f, 61 },
    { 0x7b80f562, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe90f569, 98 },
    { 0xbb40f56a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe370f56d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0f570, 90 },
    { 0xe4d0f570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec0f574, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80f579, 90 },
    { 0x6d60f579, 61 },
    { 0xa40f57a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00f57e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0f584, 93 },
    { 0x8230f584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b0f588, 23 },
    { 0x3640f589, 90 },
    { 0, 0 },
    { 0xfbd0f58b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f0f590, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe420f593, 88 },
    { 0, 0 },
    { 0x54f0f595, 98 },
    { 0x7f10f596, 98 },
    { 0xc1e0f596, 64 },
    { 0x6c30f595, 46 },
    { 0xfff0f597, 95 },
    { 0x6a40f59a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed70f5a0, 100 },
    { 0x1240f5a0, 95 },
    { 0x6f20f5a1, 90 },
    { 0x8c0f5a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf080f5a8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0f5af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7320f5b2, 61 },
    { 0xa8a0f5b2, 33 },
    { 0x2300f5b4, 98 },
    { 0, 0 },
    { 0xd940f5b6, 43 },
    { 0x35d0f5b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0f5be, 81 },
    { 0, 0 },
    { 0x7930f5c0, 77 },
    { 0, 0 },
    { 0xb2d0f5c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf40f5c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0f5c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b90f5cb, 90 },
    { 0, 0 },
    { 0x54f0f5cd, 90 },
    { 0x9190f5ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec0f5d2, 23 },
    { 0xa400f5d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0f5d7, 43 },
    { 0, 0 },
    { 0x9f10f5d9, 33 },
    { 0x69c0f5da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0f5de, 85 },
    { 0xb6b0f5df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1070f5e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc80f5ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00f5f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc550f5f8, 55 },
    { 0xb5c0f5f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe350f5fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf0f603, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3040f612, 61 },
    { 0x4750f612, 55 },
    { 0, 0 },
    { 0x56e0f615, 37 },
    { 0xfe70f616, 61 },
    { 0xf910f616, 95 },
    { 0, 0 },
    { 0xb40f619, 90 },
    { 0x6700f619, 23 },
    { 0xdc90f61b, 33 },
    { 0xa20f61b, 55 },
    { 0xf940f61b, 93 },
    { 0x48f0f619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120f622, 87 },
    { 0x9d30f623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae60f628, 76 },
    { 0x31d0f629, 98 },
    { 0, 0 },
    { 0x3d90f62b, 34 },
    { 0xe040f62b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb0f630, 33 },
    { 0, 0 },
    { 0xddf0f632, 90 },
    { 0, 0 },
    { 0xc020f634, 91 },
    { 0, 0 },
    { 0xd870f636, 95 },
    { 0xf540f636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20f63e, 46 },
    { 0xb270f63f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f643, 23 },
    { 0, 0 },
    { 0xb000f645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8610f64a, 76 },
    { 0xdcc0f64b, 76 },
    { 0, 0 },
    { 0xbc0f64d, 61 },
    { 0xb2c0f64d, 23 },
    { 0x7e80f64e, 77 },
    { 0, 0 },
    { 0x5bb0f651, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa250f654, 95 },
    { 0xe450f655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f0f658, 34 },
    { 0xb60f658, 66 },
    { 0xd4f0f658, 95 },
    { 0x3cc0f659, 38 },
    { 0, 0 },
    { 0x3840f65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0f661, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0f668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f66b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa850f670, 66 },
    { 0xc150f671, 33 },
    { 0xcd70f672, 55 },
    { 0x3300f673, 23 },
    { 0xb6c0f673, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410f67f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60f684, 34 },
    { 0x5170f685, 61 },
    { 0x1200f686, 61 },
    { 0xcac0f686, 55 },
    { 0x2ea0f688, 90 },
    { 0, 0 },
    { 0xed50f68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e0f68d, 61 },
    { 0x8d0f68e, 87 },
    { 0x27c0f68f, 77 },
    { 0xfa40f68f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f0f694, 100 },
    { 0xfbc0f695, 40 },
    { 0xc2f0f696, 68 },
    { 0x6370f694, 98 },
    { 0x4640f697, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7200f69b, 33 },
    { 0xf4d0f69b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd300f6a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230f6aa, 61 },
    { 0xfa0f6aa, 43 },
    { 0x9070f6ac, 93 },
    { 0x7d90f6ab, 52 },
    { 0xa810f6ae, 95 },
    { 0x2f30f6af, 33 },
    { 0x4f30f6ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0f6b5, 90 },
    { 0x12e0f6b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa870f6b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa0f6bf, 100 },
    { 0xc390f6c0, 61 },
    { 0x4420f6c1, 35 },
    { 0xdf30f6bf, 33 },
    { 0x9d20f6c1, 34 },
    { 0xdc80f6c4, 61 },
    { 0xfcd0f6c1, 24 },
    { 0, 0 },
    { 0x67f0f6c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0f6cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf670f6d2, 43 },
    { 0x4ea0f6d3, 98 },
    { 0, 0 },
    { 0xf1f0f6d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b00f6da, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b0f6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb0f6e0, 84 },
    { 0, 0 },
    { 0x7390f6e2, 88 },
    { 0xce20f6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0f6e7, 55 },
    { 0, 0 },
    { 0xb260f6e9, 23 },
    { 0x4360f6ea, 23 },
    { 0x1850f6eb, 77 },
    { 0, 0 },
    { 0x2500f6ed, 33 },
    { 0, 0 },
    { 0x4f00f6ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2680f6f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f6fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0f6fe, 34 },
    { 0xc150f6ff, 52 },
    { 0xb1e0f700, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb030f704, 34 },
    { 0, 0 },
    { 0x6330f706, 90 },
    { 0, 0 },
    { 0xc00f708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1780f70b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e20f70f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810f714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa730f71b, 33 },
    { 0xb140f71c, 83 },
    { 0xfbb0f71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a0f720, 93 },
    { 0x3060f720, 90 },
    { 0x2c10f720, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700f72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310f72d, 52 },
    { 0x3980f72d, 23 },
    { 0xaf30f72d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd510f736, 43 },
    { 0, 0 },
    { 0xb690f738, 43 },
    { 0, 0 },
    { 0xc7a0f73a, 61 },
    { 0x2b10f73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b0f743, 98 },
    { 0xdf90f744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb0f74c, 98 },
    { 0x96d0f74d, 90 },
    { 0x4660f74d, 29 },
    { 0x1b50f74d, 66 },
    { 0x2bf0f74e, 93 },
    { 0x7260f74f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c0f754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb20f759, 88 },
    { 0x24c0f75a, 69 },
    { 0x8f90f75a, 34 },
    { 0, 0 },
    { 0x57f0f75d, 90 },
    { 0xdee0f75e, 98 },
    { 0x8300f75f, 61 },
    { 0xfae0f760, 55 },
    { 0x8360f760, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0f766, 33 },
    { 0xc740f766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40f76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b80f76e, 61 },
    { 0x4e00f76e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e0f775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb470f778, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6270f784, 90 },
    { 0, 0 },
    { 0x3c10f786, 33 },
    { 0x6de0f787, 90 },
    { 0x3b40f787, 33 },
    { 0xf170f787, 90 },
    { 0x38c0f788, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa790f792, 85 },
    { 0x4720f793, 98 },
    { 0xceb0f794, 76 },
    { 0xa4d0f794, 53 },
    { 0xb970f795, 33 },
    { 0x2ac0f797, 95 },
    { 0x80b0f793, 61 },
    { 0x9f10f797, 96 },
    { 0x7470f79a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4680f79d, 90 },
    { 0xf030f79d, 90 },
    { 0x51c0f79e, 98 },
    { 0x5ec0f7a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea0f7a7, 61 },
    { 0, 0 },
    { 0x2260f7a9, 90 },
    { 0xa7a0f7aa, 28 },
    { 0x9330f7aa, 84 },
    { 0x3af0f7ac, 23 },
    { 0xa5f0f7ad, 77 },
    { 0x8df0f7ad, 33 },
    { 0xf120f7aa, 76 },
    { 0x23c0f7ae, 90 },
    { 0x89a0f7b1, 88 },
    { 0xf280f7b1, 96 },
    { 0x1ca0f7b3, 61 },
    { 0x2640f7ac, 93 },
    { 0, 0 },
    { 0x8c60f7b6, 90 },
    { 0, 0 },
    { 0x42b0f7b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790f7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e0f7be, 55 },
    { 0, 0 },
    { 0x3340f7c0, 43 },
    { 0x83a0f7c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b30f7c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0f7c7, 75 },
    { 0x5770f7c8, 95 },
    { 0xe720f7c9, 91 },
    { 0xb680f7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc40f7d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850f7d4, 33 },
    { 0x6e20f7d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f0f7da, 40 },
    { 0x3bb0f7db, 12 },
    { 0x6060f7dc, 90 },
    { 0x12a0f7dc, 99 },
    { 0x3fa0f7de, 90 },
    { 0x8270f7df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8180f7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6760f7e7, 33 },
    { 0xc250f7e8, 39 },
    { 0x6530f7e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00f7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f7f4, 61 },
    { 0x3230f7f5, 91 },
    { 0x9550f7f6, 95 },
    { 0xd8c0f7f5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0f7fc, 99 },
    { 0x5220f7fd, 68 },
    { 0xc510f7fe, 23 },
    { 0xfb20f7fd, 95 },
    { 0x45c0f800, 52 },
    { 0x6450f7ff, 75 },
    { 0x2ad0f7ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80f807, 76 },
    { 0, 0 },
    { 0x33a0f809, 98 },
    { 0xd950f809, 33 },
    { 0x5070f80b, 75 },
    { 0xd880f80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0f810, 33 },
    { 0, 0 },
    { 0x2190f812, 90 },
    { 0x8410f813, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8570f818, 23 },
    { 0xdb20f819, 90 },
    { 0xb350f81a, 75 },
    { 0x7580f81a, 95 },
    { 0x7b90f81c, 23 },
    { 0x43d0f81d, 33 },
    { 0, 0 },
    { 0x42c0f81f, 87 },
    { 0x9600f81f, 93 },
    { 0xa020f81f, 33 },
    { 0, 0 },
    { 0xa7f0f823, 61 },
    { 0xc8d0f824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310f828, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610f82f, 34 },
    { 0, 0 },
    { 0x5250f831, 55 },
    { 0xde20f831, 95 },
    { 0, 0 },
    { 0xff20f834, 55 },
    { 0x22e0f835, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a0f83c, 34 },
    { 0xd160f83d, 52 },
    { 0, 0 },
    { 0xf9f0f83f, 90 },
    { 0x76e0f840, 95 },
    { 0, 0 },
    { 0x59c0f842, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0f848, 95 },
    { 0, 0 },
    { 0xb630f84a, 95 },
    { 0x6e70f84a, 84 },
    { 0, 0 },
    { 0x6680f84d, 33 },
    { 0xe670f84d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f0f852, 53 },
    { 0x3990f853, 33 },
    { 0xf90f853, 23 },
    { 0x1790f855, 55 },
    { 0x6120f856, 90 },
    { 0xd580f853, 23 },
    { 0xe4d0f858, 98 },
    { 0xc8e0f859, 98 },
    { 0x36d0f857, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd00f85d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe0f862, 90 },
    { 0x4680f862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b10f86a, 98 },
    { 0, 0 },
    { 0x6a20f86c, 61 },
    { 0, 0 },
    { 0xfa00f86e, 76 },
    { 0, 0 },
    { 0x31a0f870, 95 },
    { 0xb3b0f871, 90 },
    { 0xd7d0f872, 87 },
    { 0x86f0f872, 84 },
    { 0x7c80f871, 43 },
    { 0xf80f875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e0f87d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f880, 33 },
    { 0x96d0f881, 52 },
    { 0, 0 },
    { 0xf70f883, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800f888, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef70f891, 98 },
    { 0x8a10f892, 90 },
    { 0xe4f0f892, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480f89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0f8a4, 98 },
    { 0x9740f8a4, 90 },
    { 0xf50f8a6, 43 },
    { 0x2670f8a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6590f8af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b0f8b3, 99 },
    { 0x8460f8b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d30f8b8, 33 },
    { 0xeab0f8b8, 76 },
    { 0xe830f8ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f0f8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f8c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0f8c5, 93 },
    { 0, 0 },
    { 0x7da0f8c7, 33 },
    { 0x21e0f8c7, 95 },
    { 0x1100f8c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5580f8d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c40f8d9, 23 },
    { 0xbdd0f8da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b0f8dd, 76 },
    { 0x3700f8de, 98 },
    { 0x1600f8dd, 82 },
    { 0x6b20f8e0, 46 },
    { 0xe50f8e0, 34 },
    { 0xbdd0f8e0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60f8eb, 33 },
    { 0x9c70f8ec, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe570f8f2, 90 },
    { 0, 0 },
    { 0x6580f8f4, 93 },
    { 0, 0 },
    { 0xddd0f8f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920f8fa, 33 },
    { 0xfc60f8fa, 55 },
    { 0x99b0f8fa, 90 },
    { 0x3900f8fd, 52 },
    { 0x3350f8fc, 76 },
    { 0xd670f8ff, 98 },
    { 0xa530f900, 95 },
    { 0x4dc0f901, 78 },
    { 0xf740f901, 98 },
    { 0x3d80f903, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f909, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b0f916, 100 },
    { 0, 0 },
    { 0x2020f918, 23 },
    { 0xd230f919, 95 },
    { 0, 0 },
    { 0x9a00f91b, 55 },
    { 0x9be0f91b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d0f91f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b0f927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1250f92a, 76 },
    { 0, 0 },
    { 0x8a0f92c, 20 },
    { 0x5fd0f92d, 76 },
    { 0x31f0f92c, 33 },
    { 0, 0 },
    { 0xee0f930, 90 },
    { 0, 0 },
    { 0xa160f932, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0f936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f93b, 40 },
    { 0, 0 },
    { 0xd460f93d, 61 },
    { 0x31f0f93e, 43 },
    { 0x1060f93f, 90 },
    { 0xee30f940, 96 },
    { 0xc010f940, 98 },
    { 0xa80f942, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1750f947, 98 },
    { 0xaa50f948, 68 },
    { 0xca40f949, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd0f94e, 61 },
    { 0x21d0f94f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c0f953, 55 },
    { 0x2e20f954, 99 },
    { 0, 0 },
    { 0xc140f956, 33 },
    { 0, 0 },
    { 0x7100f958, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0f95f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0f963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f90f966, 76 },
    { 0, 0 },
    { 0xb9a0f968, 33 },
    { 0xb620f968, 75 },
    { 0, 0 },
    { 0x5f40f96b, 90 },
    { 0x4b10f96c, 40 },
    { 0, 0 },
    { 0xd4f0f96e, 90 },
    { 0xebb0f96f, 23 },
    { 0xfbc0f970, 99 },
    { 0x4880f96f, 98 },
    { 0xece0f96e, 80 },
    { 0, 0 },
    { 0x47c0f974, 95 },
    { 0, 0 },
    { 0xd840f976, 33 },
    { 0x6980f976, 90 },
    { 0x3d90f978, 33 },
    { 0, 0 },
    { 0xea90f97a, 52 },
    { 0x92d0f97a, 90 },
    { 0x4ad0f97c, 61 },
    { 0, 0 },
    { 0xf0e0f97e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50f985, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2410f98b, 23 },
    { 0xe220f98c, 85 },
    { 0, 0 },
    { 0xc420f98e, 91 },
    { 0x43d0f98f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0f992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f997, 91 },
    { 0, 0 },
    { 0x4870f999, 95 },
    { 0x1310f99a, 34 },
    { 0, 0 },
    { 0xc6d0f99c, 33 },
    { 0x64a0f99d, 66 },
    { 0x90d0f99c, 33 },
    { 0, 0 },
    { 0x4780f9a0, 98 },
    { 0x98d0f9a1, 88 },
    { 0x5010f9a2, 40 },
    { 0xce50f9a1, 61 },
    { 0, 0 },
    { 0x7db0f9a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0f9a8, 90 },
    { 0xd6a0f9a8, 95 },
    { 0xeb10f9a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580f9b0, 61 },
    { 0x80f9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0f9b5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40f9be, 100 },
    { 0, 0 },
    { 0x380f9c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650f9c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc0f9cb, 99 },
    { 0, 0 },
    { 0x8e90f9cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70f9d4, 98 },
    { 0x9040f9d5, 95 },
    { 0, 0 },
    { 0x87c0f9d7, 98 },
    { 0x75c0f9d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f0f9dd, 61 },
    { 0xe940f9de, 66 },
    { 0x7580f9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f9e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00f9e8, 93 },
    { 0x3450f9e9, 90 },
    { 0xdbc0f9e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40f9ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf070f9f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d80f9fc, 90 },
    { 0x3560f9fd, 55 },
    { 0xe1d0f9fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6180fa01, 90 },
    { 0, 0 },
    { 0x5650fa03, 100 },
    { 0, 0 },
    { 0x23a0fa05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2840fa0c, 33 },
    { 0, 0 },
    { 0xc920fa0e, 95 },
    { 0x6770fa0e, 98 },
    { 0xedb0fa0f, 90 },
    { 0xa160fa11, 93 },
    { 0x69d0fa12, 98 },
    { 0xd250fa13, 90 },
    { 0, 0 },
    { 0xf50fa15, 78 },
    { 0xc3d0fa16, 34 },
    { 0, 0 },
    { 0xfa10fa18, 83 },
    { 0xba0fa19, 53 },
    { 0x2a90fa1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac0fa1f, 91 },
    { 0, 0 },
    { 0x7c50fa21, 78 },
    { 0xc1c0fa22, 91 },
    { 0x10e0fa22, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d0fa26, 33 },
    { 0, 0 },
    { 0x6eb0fa28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5950fa2f, 55 },
    { 0xd6f0fa2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b50fa3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0fa42, 85 },
    { 0, 0 },
    { 0x77b0fa44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790fa49, 80 },
    { 0x2460fa4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0fa4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f80fa51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb0fa56, 34 },
    { 0, 0 },
    { 0x8780fa58, 95 },
    { 0xb70fa59, 90 },
    { 0x8870fa58, 87 },
    { 0x7f00fa58, 93 },
    { 0x1920fa58, 33 },
    { 0x4810fa5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3090fa60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0fa66, 98 },
    { 0x6590fa67, 91 },
    { 0xf4f0fa68, 90 },
    { 0x9d50fa69, 90 },
    { 0x5310fa69, 100 },
    { 0, 0 },
    { 0xaf20fa6c, 90 },
    { 0x30b0fa6d, 98 },
    { 0xf600fa6e, 23 },
    { 0xde60fa6f, 99 },
    { 0x5ae0fa70, 43 },
    { 0xf5e0fa71, 90 },
    { 0x8e30fa6e, 61 },
    { 0x8d0fa73, 98 },
    { 0x79f0fa72, 23 },
    { 0xeef0fa75, 98 },
    { 0x6d20fa71, 46 },
    { 0x8cf0fa6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8970fa7a, 61 },
    { 0x22d0fa7b, 33 },
    { 0x8990fa7a, 46 },
    { 0xe4d0fa7c, 66 },
    { 0x5a40fa7e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee0fa82, 76 },
    { 0xea20fa83, 95 },
    { 0x7870fa84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540fa89, 98 },
    { 0xa9c0fa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b0fa8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0fa91, 55 },
    { 0x3eb0fa91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730fa9a, 90 },
    { 0xb200fa9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6690faad, 98 },
    { 0x1400faae, 77 },
    { 0xc5e0faaf, 90 },
    { 0xc4f0faae, 23 },
    { 0, 0 },
    { 0x3510fab2, 95 },
    { 0, 0 },
    { 0xe070fab4, 98 },
    { 0x2cb0fab5, 53 },
    { 0x65f0fab5, 90 },
    { 0, 0 },
    { 0x5d90fab8, 75 },
    { 0, 0 },
    { 0xe0c0faba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf620fabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180fac5, 46 },
    { 0x7650fac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc0faca, 23 },
    { 0xda30facb, 33 },
    { 0x1210facb, 67 },
    { 0x3790facb, 77 },
    { 0x5920facc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0fad2, 90 },
    { 0x35d0fad3, 98 },
    { 0xf00fad4, 90 },
    { 0xb60fad5, 91 },
    { 0x4a90fad4, 17 },
    { 0, 0 },
    { 0x2660fad8, 68 },
    { 0x53a0fad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290fade, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b10fae4, 61 },
    { 0x7b10fae5, 33 },
    { 0x2ce0fae4, 98 },
    { 0, 0 },
    { 0x6910fae8, 55 },
    { 0, 0 },
    { 0x70c0faea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b0faed, 85 },
    { 0xaf30faee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0faf1, 40 },
    { 0x14a0faf2, 100 },
    { 0xa820faf2, 95 },
    { 0x6f70faf4, 93 },
    { 0xf210faf3, 33 },
    { 0x2e10faf1, 68 },
    { 0x8b0faf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0fafa, 77 },
    { 0x14c0fafa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c0fafe, 23 },
    { 0, 0 },
    { 0xcb80fb00, 98 },
    { 0, 0 },
    { 0x3f40fb02, 98 },
    { 0xab60fb02, 90 },
    { 0x88f0fb04, 78 },
    { 0xfcd0fb05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0fb0b, 33 },
    { 0x1800fb0c, 80 },
    { 0x14d0fb0d, 51 },
    { 0xd450fb0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c0fb11, 91 },
    { 0x1540fb12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c0fb18, 95 },
    { 0, 0 },
    { 0xcb70fb1a, 95 },
    { 0xec10fb1b, 76 },
    { 0xd240fb1c, 55 },
    { 0, 0 },
    { 0x320fb1e, 76 },
    { 0x83e0fb1e, 55 },
    { 0x9ee0fb20, 95 },
    { 0x8a20fb20, 90 },
    { 0, 0 },
    { 0x16b0fb23, 91 },
    { 0, 0 },
    { 0x8a60fb25, 33 },
    { 0xc7f0fb26, 66 },
    { 0xcdc0fb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60fb2c, 61 },
    { 0xe840fb2c, 55 },
    { 0x9630fb2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990fb35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5630fb38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850fb3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0fb3e, 61 },
    { 0xa720fb3f, 68 },
    { 0x2930fb40, 95 },
    { 0x13b0fb3e, 77 },
    { 0, 0 },
    { 0xece0fb43, 23 },
    { 0xe080fb44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd0fb47, 23 },
    { 0x4d60fb47, 77 },
    { 0xb800fb49, 90 },
    { 0xbcb0fb4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210fb54, 33 },
    { 0, 0 },
    { 0x87e0fb56, 23 },
    { 0x68d0fb57, 52 },
    { 0xac50fb56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00fb5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe600fb66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d20fb69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0fb6d, 90 },
    { 0x7260fb6e, 46 },
    { 0x81f0fb6f, 61 },
    { 0x2360fb6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c20fb7e, 34 },
    { 0x25f0fb7f, 95 },
    { 0xa830fb80, 65 },
    { 0, 0 },
    { 0x9870fb82, 95 },
    { 0, 0 },
    { 0xc20fb84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0fb91, 93 },
    { 0xe0e0fb92, 66 },
    { 0xb4c0fb93, 88 },
    { 0xcca0fb94, 76 },
    { 0x5250fb93, 34 },
    { 0, 0 },
    { 0x9800fb97, 33 },
    { 0x17b0fb98, 34 },
    { 0x8700fb97, 33 },
    { 0x78a0fb9a, 33 },
    { 0x3010fb98, 61 },
    { 0x2390fb9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8200fba0, 43 },
    { 0, 0 },
    { 0x3430fba2, 43 },
    { 0, 0 },
    { 0xe560fba4, 61 },
    { 0xc780fba5, 61 },
    { 0, 0 },
    { 0x9b00fba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd560fbab, 23 },
    { 0x380fbac, 33 },
    { 0xde80fbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc300fbb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2460fbb5, 98 },
    { 0x3190fbb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf230fbb9, 98 },
    { 0x51b0fbb9, 34 },
    { 0xd5e0fbbb, 33 },
    { 0, 0 },
    { 0x6df0fbbd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0fbc0, 90 },
    { 0x54d0fbc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc0fbc8, 64 },
    { 0, 0 },
    { 0xd6c0fbca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b0fbce, 99 },
    { 0xa050fbcf, 52 },
    { 0xbcb0fbd0, 95 },
    { 0xfb20fbd1, 88 },
    { 0xa630fbd0, 93 },
    { 0xe2a0fbd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e0fbd9, 95 },
    { 0xaba0fbda, 68 },
    { 0xd050fbd9, 98 },
    { 0xc10fbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb10fbe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb20fbe6, 33 },
    { 0x22e0fbe7, 78 },
    { 0xc5e0fbe7, 98 },
    { 0x9890fbe7, 33 },
    { 0x8a40fbe9, 95 },
    { 0xd260fbe9, 23 },
    { 0xc300fbe8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa650fbf4, 61 },
    { 0x6090fbf4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd40fbf8, 76 },
    { 0x7d0fbf9, 98 },
    { 0x43f0fbf9, 34 },
    { 0x8b40fbfb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400fc01, 95 },
    { 0, 0 },
    { 0xca70fc03, 90 },
    { 0xbac0fc04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4710fc08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c0fc13, 23 },
    { 0, 0 },
    { 0x9e40fc15, 95 },
    { 0xdd90fc16, 85 },
    { 0x4310fc16, 61 },
    { 0, 0 },
    { 0xd950fc19, 33 },
    { 0x12d0fc1a, 90 },
    { 0xd410fc1b, 34 },
    { 0x7a10fc1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990fc20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c0fc2a, 43 },
    { 0x4b70fc2b, 90 },
    { 0, 0 },
    { 0x37d0fc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0fc3a, 90 },
    { 0, 0 },
    { 0x6a00fc3c, 52 },
    { 0x5670fc3d, 33 },
    { 0x29a0fc3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe0fc41, 70 },
    { 0, 0 },
    { 0xc480fc43, 33 },
    { 0x2930fc44, 55 },
    { 0xd8e0fc45, 93 },
    { 0xd8b0fc45, 98 },
    { 0xd2d0fc47, 23 },
    { 0x850fc48, 98 },
    { 0x6c00fc49, 76 },
    { 0x6d40fc49, 90 },
    { 0, 0 },
    { 0x2b20fc4c, 46 },
    { 0, 0 },
    { 0xd4c0fc4e, 98 },
    { 0, 0 },
    { 0xb3e0fc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0fc54, 55 },
    { 0x6510fc54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f40fc58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa50fc5b, 95 },
    { 0xa580fc5c, 76 },
    { 0x17f0fc5d, 90 },
    { 0, 0 },
    { 0xbe50fc5f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c0fc65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd90fc6b, 90 },
    { 0x9e40fc6b, 78 },
    { 0xc1a0fc6b, 61 },
    { 0x1e20fc6e, 99 },
    { 0xe20fc6f, 61 },
    { 0, 0 },
    { 0xd700fc71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2910fc76, 91 },
    { 0, 0 },
    { 0xb910fc78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e0fc7d, 90 },
    { 0x3470fc7e, 95 },
    { 0x640fc7e, 61 },
    { 0x6720fc7f, 95 },
    { 0, 0 },
    { 0xa170fc82, 90 },
    { 0xa600fc83, 73 },
    { 0x6550fc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860fc89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc80fc8d, 23 },
    { 0xedb0fc8d, 33 },
    { 0x91b0fc8d, 90 },
    { 0x90d0fc90, 23 },
    { 0x61d0fc8e, 33 },
    { 0xe860fc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0fc97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be0fc9a, 90 },
    { 0, 0 },
    { 0xa170fc9c, 98 },
    { 0x8dc0fc9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f80fca7, 33 },
    { 0xce70fca8, 90 },
    { 0x8d10fca9, 76 },
    { 0x5790fca8, 90 },
    { 0xe070fcaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40fcae, 98 },
    { 0xb8c0fcaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420fcb6, 94 },
    { 0xdea0fcb7, 23 },
    { 0x1260fcb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640fcbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf570fcc1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc200fcc4, 76 },
    { 0x8cb0fcc5, 61 },
    { 0xead0fcc6, 61 },
    { 0xeeb0fcc7, 77 },
    { 0x9160fcc8, 90 },
    { 0x2500fcc8, 23 },
    { 0x5a50fcc9, 65 },
    { 0x3820fcc7, 23 },
    { 0x1bb0fccc, 87 },
    { 0, 0 },
    { 0x3a40fcce, 90 },
    { 0xc210fccf, 96 },
    { 0x5f60fcd0, 100 },
    { 0x40a0fccf, 34 },
    { 0x6cb0fcd2, 93 },
    { 0xed70fccf, 23 },
    { 0x4040fcd4, 23 },
    { 0xb550fcd4, 90 },
    { 0xebc0fcd5, 61 },
    { 0x5570fcd4, 33 },
    { 0, 0 },
    { 0xce50fcd9, 76 },
    { 0x9d0fcda, 19 },
    { 0xf0d0fcdb, 68 },
    { 0x7030fcdc, 17 },
    { 0xbce0fcdd, 95 },
    { 0x85b0fcdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00fce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae0fce5, 91 },
    { 0x7f40fce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad0fce9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb0fcee, 65 },
    { 0xb040fcef, 52 },
    { 0x5600fcf0, 90 },
    { 0xd280fcf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b00fcf5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf720fcf8, 22 },
    { 0, 0 },
    { 0x940fcfa, 95 },
    { 0x2070fcfb, 61 },
    { 0x4290fcfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710fd01, 33 },
    { 0, 0 },
    { 0xb00fd03, 55 },
    { 0x6e40fd04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce0fd07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0fd0c, 33 },
    { 0, 0 },
    { 0x2d90fd0e, 61 },
    { 0, 0 },
    { 0x9c40fd10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0fd17, 33 },
    { 0xa380fd17, 76 },
    { 0, 0 },
    { 0xebc0fd1a, 23 },
    { 0xd230fd1b, 34 },
    { 0xcee0fd1b, 95 },
    { 0xb90fd1b, 2 },
    { 0x3bb0fd1b, 46 },
    { 0, 0 },
    { 0x5820fd20, 76 },
    { 0x3bc0fd20, 34 },
    { 0x8890fd20, 90 },
    { 0xe660fd23, 61 },
    { 0xc4f0fd23, 23 },
    { 0x56c0fd25, 33 },
    { 0xf500fd26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4750fd29, 90 },
    { 0xf3d0fd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7900fd32, 90 },
    { 0x5a50fd33, 90 },
    { 0x2610fd33, 91 },
    { 0x3d0fd35, 95 },
    { 0x1690fd36, 98 },
    { 0x4420fd37, 61 },
    { 0xf6c0fd37, 33 },
    { 0xa270fd32, 97 },
    { 0x3160fd3a, 23 },
    { 0x5a70fd3a, 61 },
    { 0x50e0fd3c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b40fd3f, 98 },
    { 0, 0 },
    { 0x6da0fd41, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1470fd45, 100 },
    { 0xe780fd45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9160fd49, 4 },
    { 0x59e0fd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b0fd52, 61 },
    { 0x8ae0fd53, 23 },
    { 0xece0fd52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b60fd5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7970fd60, 98 },
    { 0x4030fd61, 98 },
    { 0, 0 },
    { 0x2710fd63, 33 },
    { 0, 0 },
    { 0xb40fd65, 99 },
    { 0x5fb0fd66, 90 },
    { 0, 0 },
    { 0x1330fd68, 98 },
    { 0x2c60fd69, 17 },
    { 0x6860fd69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350fd70, 53 },
    { 0xbce0fd70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3960fd75, 33 },
    { 0x1630fd76, 90 },
    { 0x7af0fd77, 88 },
    { 0x3490fd78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4850fd81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0fd84, 93 },
    { 0xcc0fd85, 61 },
    { 0x1d90fd85, 17 },
    { 0xfab0fd85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe940fd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af0fd8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e50fd92, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f0fd96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00fd9a, 78 },
    { 0x6dc0fd9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd00fda1, 61 },
    { 0, 0 },
    { 0x2340fda3, 100 },
    { 0x7530fda4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950fdaa, 90 },
    { 0x3010fdaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f40fdae, 55 },
    { 0x5bc0fdae, 90 },
    { 0xd60fdaf, 67 },
    { 0xa5e0fdb1, 95 },
    { 0x7740fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d0fdb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e70fdbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70fdc4, 33 },
    { 0, 0 },
    { 0xd3a0fdc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba0fdca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440fdd1, 66 },
    { 0, 0 },
    { 0x8e30fdd3, 72 },
    { 0, 0 },
    { 0xaac0fdd5, 90 },
    { 0xf0b0fdd5, 95 },
    { 0, 0 },
    { 0x51d0fdd8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1280fddb, 90 },
    { 0, 0 },
    { 0xfb10fddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b0fde0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8870fde3, 46 },
    { 0, 0 },
    { 0x7370fde5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3770fdee, 44 },
    { 0x54c0fdef, 52 },
    { 0x4cc0fdf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc940fdfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80fe00, 33 },
    { 0x4c50fe01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050fe0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0fe11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf0fe14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9410fe17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550fe20, 40 },
    { 0x20d0fe20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a0fe27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e0fe2c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xed00fe2f, 76 },
    { 0xe780fe2f, 23 },
    { 0x9280fe31, 81 },
    { 0x7580fe32, 61 },
    { 0xa0fe33, 23 },
    { 0xf90fe32, 43 },
    { 0x63d0fe2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90fe3a, 46 },
    { 0x8830fe3b, 97 },
    { 0xbb00fe3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0fe3f, 43 },
    { 0x1890fe3f, 55 },
    { 0, 0 },
    { 0xc1e0fe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d0fe45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6580fe48, 36 },
    { 0, 0 },
    { 0x34d0fe4a, 25 },
    { 0x47d0fe4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210fe4e, 33 },
    { 0, 0 },
    { 0x6fa0fe50, 90 },
    { 0x13e0fe51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940fe55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0fe59, 23 },
    { 0x3c0fe5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300fe5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90fe65, 98 },
    { 0xd8d0fe66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40fe6e, 34 },
    { 0x8870fe6e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b0fe72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e20fe77, 23 },
    { 0xd320fe78, 61 },
    { 0, 0 },
    { 0x4740fe7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80fe7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a0fe80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3290fe83, 98 },
    { 0, 0 },
    { 0xcbf0fe85, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e0fe88, 23 },
    { 0x41a0fe89, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e90fe8c, 95 },
    { 0x1410fe8d, 76 },
    { 0x14b0fe8c, 90 },
    { 0x8eb0fe8f, 33 },
    { 0x50b0fe90, 33 },
    { 0xa490fe8f, 33 },
    { 0x8210fe92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb780fe9c, 23 },
    { 0, 0 },
    { 0xd910fe9e, 85 },
    { 0, 0 },
    { 0x3350fea0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770fea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0fea9, 33 },
    { 0, 0 },
    { 0x6a80feab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d0feb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530feb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1270febc, 69 },
    { 0, 0 },
    { 0x2f0febe, 61 },
    { 0xdd70febe, 98 },
    { 0, 0 },
    { 0x1fa0fec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d0fec8, 23 },
    { 0xe990fec8, 55 },
    { 0x9d50fec9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820fecf, 23 },
    { 0xfbe0fed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd340fed4, 61 },
    { 0, 0 },
    { 0x850fed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf230fed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde0fedc, 61 },
    { 0xbca0fedd, 91 },
    { 0x5c40fedc, 34 },
    { 0, 0 },
    { 0xfbc0fee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d50fee3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0fee9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa830feef, 93 },
    { 0, 0 },
    { 0x5e30fef1, 46 },
    { 0x8da0fef2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00fef6, 90 },
    { 0, 0 },
    { 0x5640fef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b50ff01, 99 },
    { 0x8aa0ff01, 87 },
    { 0xea20ff02, 95 },
    { 0x1170ff04, 90 },
    { 0x9d70ff04, 95 },
    { 0x3080ff04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0ff0b, 43 },
    { 0, 0 },
    { 0xe00ff0d, 90 },
    { 0x3480ff0e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710ff16, 17 },
    { 0xfda0ff17, 90 },
    { 0, 0 },
    { 0xe20ff19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd0ff1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960ff23, 43 },
    { 0xd7c0ff24, 98 },
    { 0x1730ff25, 95 },
    { 0x2f50ff25, 34 },
    { 0, 0 },
    { 0x8e40ff28, 56 },
    { 0, 0 },
    { 0x60b0ff2a, 33 },
    { 0xf420ff2b, 98 },
    { 0, 0 },
    { 0xdb20ff2d, 43 },
    { 0x8560ff2e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab40ff32, 78 },
    { 0xd650ff33, 88 },
    { 0x6550ff34, 33 },
    { 0xd6f0ff34, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90ff39, 99 },
    { 0, 0 },
    { 0x74e0ff3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8040ff3e, 95 },
    { 0xb630ff3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de0ff48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0ff4d, 61 },
    { 0, 0 },
    { 0x8a90ff4f, 34 },
    { 0, 0 },
    { 0xc5f0ff51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c0ff57, 61 },
    { 0x8ac0ff57, 61 },
    { 0, 0 },
    { 0x3050ff5a, 95 },
    { 0, 0 },
    { 0xffb0ff5c, 98 },
    { 0x2c10ff5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c10ff61, 90 },
    { 0xb4e0ff61, 75 },
    { 0, 0 },
    { 0x4a70ff64, 91 },
    { 0xc810ff65, 23 },
    { 0x94f0ff66, 61 },
    { 0, 0 },
    { 0x1220ff68, 55 },
    { 0xe770ff69, 93 },
    { 0xd840ff6a, 95 },
    { 0xbad0ff6a, 23 },
    { 0x5150ff6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc0ff6f, 97 },
    { 0xc400ff70, 61 },
    { 0x8cd0ff71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870ff74, 55 },
    { 0, 0 },
    { 0x6bf0ff76, 98 },
    { 0xf3f0ff77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c0ff7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa200ff84, 61 },
    { 0, 0 },
    { 0xd810ff86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda80ff8e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80ff93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70ff97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1930ff9a, 84 },
    { 0xb6c0ff9b, 81 },
    { 0, 0 },
    { 0x89b0ff9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd0ffa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac0ffaf, 75 },
    { 0xbb70ffb0, 33 },
    { 0x7990ffaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700ffb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0ffbf, 23 },
    { 0, 0 },
    { 0xf150ffc1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca0ffc8, 23 },
    { 0xc5e0ffc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a0ffcc, 33 },
    { 0, 0 },
    { 0xdc50ffce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0ffd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0ffd7, 76 },
    { 0x5c40ffd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0ffde, 75 },
    { 0xc370ffdf, 61 },
    { 0, 0 },
    { 0x17b0ffe1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2050ffe9, 23 },
    { 0, 0 },
    { 0x7410ffeb, 46 },
    { 0xca50ffec, 91 },
    { 0xc3c0ffed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a00fff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8200fff3, 88 },
    { 0x52b0fff4, 81 },
    { 0x5dd0fff5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0fffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5320ffff, 55 },
    { 0, 0 },
    { 0xab10001, 95 },
    { 0xb4310002, 61 },
    { 0, 0 },
    { 0x5eb10004, 44 },
    { 0, 0 },
    { 0xf8d10006, 98 },
    { 0x62a10007, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51000a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d31000e, 90 },
    { 0, 0 },
    { 0x10010, 87 },
    { 0x2ad10010, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1110016, 90 },
    { 0xfee10016, 23 },
    { 0x7b310017, 90 },
    { 0xf110019, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b1001e, 75 },
    { 0, 0 },
    { 0xb2110020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94710024, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9421002d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610031, 90 },
    { 0x39810031, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff310036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31610045, 95 },
    { 0, 0 },
    { 0xae410047, 98 },
    { 0, 0 },
    { 0x53110049, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be1004f, 33 },
    { 0, 0 },
    { 0xc0b10051, 61 },
    { 0, 0 },
    { 0xd5910053, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b10059, 95 },
    { 0, 0 },
    { 0xfc21005b, 46 },
    { 0xc771005c, 33 },
    { 0x8071005c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1710062, 33 },
    { 0x2c610063, 95 },
    { 0x26410064, 98 },
    { 0x59410064, 43 },
    { 0x20710063, 90 },
    { 0xc5010066, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531006c, 90 },
    { 0xde61006c, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c10070, 23 },
    { 0x14210071, 46 },
    { 0xbc010072, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa110075, 90 },
    { 0, 0 },
    { 0xb3510077, 90 },
    { 0, 0 },
    { 0x47810079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f1007c, 46 },
    { 0xba01007d, 33 },
    { 0xa601007d, 100 },
    { 0x96f1007e, 33 },
    { 0xe0f1007e, 90 },
    { 0xce21007d, 46 },
    { 0x44610082, 90 },
    { 0x54c10082, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99610088, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d1008c, 90 },
    { 0, 0 },
    { 0xba51008e, 46 },
    { 0x101008f, 75 },
    { 0x3d810090, 55 },
    { 0x54010091, 76 },
    { 0x7a210090, 52 },
    { 0xed1008e, 90 },
    { 0x92610094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf10097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b51009a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b1009e, 66 },
    { 0xa901009f, 17 },
    { 0xff4100a0, 95 },
    { 0, 0 },
    { 0x7a7100a2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d100a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca100aa, 90 },
    { 0, 0 },
    { 0x2ea100ac, 93 },
    { 0xeb100ad, 95 },
    { 0xd73100ae, 34 },
    { 0x553100af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0100b8, 23 },
    { 0, 0 },
    { 0x1e7100ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368100be, 43 },
    { 0x3a7100bf, 52 },
    { 0xe82100c0, 95 },
    { 0xfb5100c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f100c5, 61 },
    { 0x705100c5, 98 },
    { 0, 0 },
    { 0x724100c8, 90 },
    { 0xaf3100c9, 33 },
    { 0x3c1100ca, 23 },
    { 0, 0 },
    { 0xcc4100cc, 26 },
    { 0, 0 },
    { 0x1e8100ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b100d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe100da, 46 },
    { 0xd00100db, 61 },
    { 0x718100dc, 58 },
    { 0x453100dc, 34 },
    { 0x95e100de, 34 },
    { 0x2ec100dd, 33 },
    { 0xa96100e0, 23 },
    { 0xd1a100dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d100e7, 23 },
    { 0x817100e7, 75 },
    { 0, 0 },
    { 0x835100ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0100ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0100f2, 40 },
    { 0x314100f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1100f6, 34 },
    { 0xc5d100f7, 23 },
    { 0x352100f6, 63 },
    { 0xbb3100f9, 80 },
    { 0x801100fa, 61 },
    { 0xd21100fb, 90 },
    { 0xd7b100f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5100ff, 52 },
    { 0x88110100, 90 },
    { 0x83a10101, 33 },
    { 0xd9810101, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8510107, 90 },
    { 0x2e610108, 98 },
    { 0x1f610109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b61010d, 90 },
    { 0, 0 },
    { 0xacd1010f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee610116, 64 },
    { 0, 0 },
    { 0x47310118, 84 },
    { 0x3ba10118, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb261011c, 52 },
    { 0x32a1011d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42610122, 77 },
    { 0, 0 },
    { 0xf1a10124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe10127, 23 },
    { 0x26b10127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd61012b, 34 },
    { 0, 0 },
    { 0x95b1012d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8210130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a10139, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f1013c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd810140, 77 },
    { 0x90a10141, 55 },
    { 0xd5d10141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80010147, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f1014b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d1014e, 93 },
    { 0xa091014f, 52 },
    { 0xa871014f, 33 },
    { 0, 0 },
    { 0xaf610152, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b310157, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8441015f, 46 },
    { 0, 0 },
    { 0xccb10161, 61 },
    { 0, 0 },
    { 0x33810163, 66 },
    { 0xee710163, 43 },
    { 0, 0 },
    { 0x77110166, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4501016a, 67 },
    { 0, 0 },
    { 0x5631016c, 33 },
    { 0x2be1016d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c10171, 61 },
    { 0xec310172, 34 },
    { 0, 0 },
    { 0xdfd10174, 95 },
    { 0xb3310174, 77 },
    { 0x33e10176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8471017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401017d, 61 },
    { 0xb4a1017e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6810181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210184, 33 },
    { 0, 0 },
    { 0x86b10186, 90 },
    { 0xa3810187, 90 },
    { 0x8a410188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce1018c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1018f, 33 },
    { 0, 0 },
    { 0x7af10191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5910194, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d10198, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00101a3, 64 },
    { 0x879101a4, 77 },
    { 0x7ea101a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b101a9, 61 },
    { 0x746101a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x165101ad, 95 },
    { 0x5f101ae, 95 },
    { 0x7e1101af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d101b5, 90 },
    { 0, 0 },
    { 0x50e101b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e101bc, 98 },
    { 0x2de101bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4101c9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x212101cc, 61 },
    { 0x10c101cd, 33 },
    { 0, 0 },
    { 0xe2a101cf, 35 },
    { 0xb3f101d0, 23 },
    { 0x217101d0, 85 },
    { 0xcf101d2, 81 },
    { 0x9ae101d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2101d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2101da, 43 },
    { 0x629101db, 61 },
    { 0x332101db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8101e0, 76 },
    { 0xa09101e1, 61 },
    { 0x5a6101e2, 91 },
    { 0xe93101e3, 35 },
    { 0x7e0101e3, 76 },
    { 0, 0 },
    { 0xc94101e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25101eb, 61 },
    { 0xe0d101eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6101ef, 66 },
    { 0x2c0101f0, 33 },
    { 0xd22101f1, 61 },
    { 0, 0 },
    { 0xb6e101f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5101f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc101fc, 23 },
    { 0x34a101fd, 33 },
    { 0x38b101fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61910207, 95 },
    { 0x9a110208, 61 },
    { 0xcf210208, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b1020d, 98 },
    { 0x3331020d, 61 },
    { 0xc731020e, 61 },
    { 0x8b31020d, 4 },
    { 0x4f810211, 34 },
    { 0xe091020f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91021d, 43 },
    { 0, 0 },
    { 0x35a1021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d10222, 33 },
    { 0, 0 },
    { 0x7210224, 46 },
    { 0, 0 },
    { 0x6c010226, 23 },
    { 0xa2b10226, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6911022c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce10232, 76 },
    { 0x6e410232, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde810237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e1023b, 61 },
    { 0x3441023c, 43 },
    { 0, 0 },
    { 0x2741023e, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2910242, 99 },
    { 0xdbc10243, 33 },
    { 0, 0 },
    { 0x67610245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4381024a, 33 },
    { 0, 0 },
    { 0xbbd1024c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf410257, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b10260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca10263, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29910268, 88 },
    { 0, 0 },
    { 0x7da1026a, 55 },
    { 0, 0 },
    { 0xb481026c, 51 },
    { 0x89b1026d, 67 },
    { 0x7821026d, 33 },
    { 0x6661026f, 88 },
    { 0x30d10270, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf010274, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27910279, 23 },
    { 0x8cd1027a, 98 },
    { 0x9cc1027b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1027f, 33 },
    { 0x98f10280, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe110285, 33 },
    { 0, 0 },
    { 0x7b910287, 84 },
    { 0, 0 },
    { 0xd7210289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f1028d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a310292, 43 },
    { 0xc2010293, 23 },
    { 0x89310293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe561029a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad102a0, 90 },
    { 0, 0 },
    { 0xda7102a2, 88 },
    { 0x2bb102a2, 55 },
    { 0, 0 },
    { 0x89102a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927102ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b102b0, 61 },
    { 0xf58102b1, 95 },
    { 0xe82102b2, 61 },
    { 0x420102b3, 23 },
    { 0, 0 },
    { 0x401102b5, 34 },
    { 0x650102b6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea102bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e102c2, 95 },
    { 0, 0 },
    { 0xc3d102c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85102cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976102d1, 95 },
    { 0, 0 },
    { 0xb9d102d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57102d6, 99 },
    { 0x71c102d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d102dc, 33 },
    { 0, 0 },
    { 0x2bb102de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6102e1, 78 },
    { 0, 0 },
    { 0xbd3102e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a102e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b102e9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be102ec, 76 },
    { 0, 0 },
    { 0xb97102ee, 78 },
    { 0xc1a102ef, 88 },
    { 0x8b8102f0, 90 },
    { 0xe76102f0, 95 },
    { 0x44f102ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4102f6, 33 },
    { 0xd50102f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967102fc, 74 },
    { 0x8ab102fd, 78 },
    { 0x3c5102fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe510307, 33 },
    { 0x2c410308, 76 },
    { 0x51810309, 67 },
    { 0x7aa10309, 23 },
    { 0xeb71030a, 91 },
    { 0x8d71030a, 76 },
    { 0, 0 },
    { 0xed71030e, 95 },
    { 0x9421030f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d10312, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41610316, 40 },
    { 0x21210316, 55 },
    { 0x71210318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f010322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3310325, 52 },
    { 0x60110326, 95 },
    { 0, 0 },
    { 0x8e910328, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb010330, 64 },
    { 0x11710330, 61 },
    { 0x89610332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea710335, 34 },
    { 0xf110335, 94 },
    { 0xfa810337, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71033c, 66 },
    { 0xaa91033d, 52 },
    { 0x2ee1033c, 81 },
    { 0, 0 },
    { 0x1cd10340, 61 },
    { 0x6d110341, 49 },
    { 0x4fd10340, 95 },
    { 0xb1310343, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2710348, 88 },
    { 0x2f710349, 81 },
    { 0x88e1034a, 33 },
    { 0x2541034b, 23 },
    { 0x7f21034b, 55 },
    { 0x6771034b, 98 },
    { 0x74d10348, 90 },
    { 0xa251034f, 90 },
    { 0x5fd10350, 98 },
    { 0x1eb1034f, 55 },
    { 0xc9910352, 81 },
    { 0xd2d10353, 33 },
    { 0xd071034f, 90 },
    { 0x1210355, 61 },
    { 0x8c210356, 61 },
    { 0, 0 },
    { 0xb6910358, 23 },
    { 0, 0 },
    { 0x8491035a, 90 },
    { 0x3031035b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa731035e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35710366, 84 },
    { 0, 0 },
    { 0xa2110368, 98 },
    { 0xf0510369, 90 },
    { 0xd121036a, 76 },
    { 0x2210369, 55 },
    { 0, 0 },
    { 0xbfa1036d, 23 },
    { 0x5f61036e, 61 },
    { 0xc681036f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0610377, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1037a, 98 },
    { 0xe911037b, 81 },
    { 0xa671037c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69710380, 85 },
    { 0xdb110381, 93 },
    { 0, 0 },
    { 0x81910383, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331038a, 95 },
    { 0x5881038b, 76 },
    { 0xccc1038c, 81 },
    { 0xb821038d, 93 },
    { 0xc131038e, 55 },
    { 0x40d1038f, 90 },
    { 0xd1610390, 61 },
    { 0x5c110391, 23 },
    { 0x4d310391, 93 },
    { 0xf2e10391, 95 },
    { 0xccd10394, 55 },
    { 0x6a710395, 76 },
    { 0x48810396, 55 },
    { 0, 0 },
    { 0xc1f10398, 90 },
    { 0x99110398, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed1039d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a103a2, 95 },
    { 0x37d103a3, 37 },
    { 0x2d0103a2, 55 },
    { 0, 0 },
    { 0x228103a6, 85 },
    { 0x5a4103a7, 33 },
    { 0x101103a7, 99 },
    { 0xea8103a6, 55 },
    { 0xead103aa, 95 },
    { 0x292103aa, 68 },
    { 0x825103ac, 88 },
    { 0x25103ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3103b0, 33 },
    { 0x7b4103b0, 91 },
    { 0, 0 },
    { 0x589103b3, 67 },
    { 0xd28103b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7103b7, 33 },
    { 0x530103b8, 90 },
    { 0xcd0103b9, 61 },
    { 0xf6a103ba, 90 },
    { 0x29b103ba, 33 },
    { 0xd8b103bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf103bf, 38 },
    { 0xe08103bf, 33 },
    { 0, 0 },
    { 0xf67103c2, 61 },
    { 0xb17103c3, 85 },
    { 0x1cd103c4, 66 },
    { 0xb5d103c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b103cb, 75 },
    { 0x1c8103cc, 75 },
    { 0x337103cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5103d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237103d8, 33 },
    { 0x7b3103d9, 23 },
    { 0xf14103da, 90 },
    { 0x8f2103db, 77 },
    { 0xdf103dc, 50 },
    { 0x65d103dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e103e6, 40 },
    { 0, 0 },
    { 0xd52103e8, 86 },
    { 0xb90103e9, 56 },
    { 0x9c103e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19103ed, 61 },
    { 0xead103ed, 33 },
    { 0x849103ef, 23 },
    { 0x1a6103ef, 94 },
    { 0, 0 },
    { 0x276103f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba103f7, 56 },
    { 0, 0 },
    { 0xd95103f9, 84 },
    { 0x517103f9, 61 },
    { 0x160103fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa510402, 46 },
    { 0, 0 },
    { 0xc4010404, 100 },
    { 0xed810405, 23 },
    { 0, 0 },
    { 0xdec10407, 100 },
    { 0xe610408, 98 },
    { 0, 0 },
    { 0xf3e1040a, 34 },
    { 0, 0 },
    { 0x5a51040c, 61 },
    { 0x9341040c, 100 },
    { 0x3541040e, 33 },
    { 0xfbc1040f, 55 },
    { 0x50610410, 23 },
    { 0xd9a10411, 90 },
    { 0x95010412, 55 },
    { 0x851040f, 95 },
    { 0x2a210414, 69 },
    { 0x88d10410, 23 },
    { 0xdf10416, 98 },
    { 0xe1110417, 76 },
    { 0xdbd10418, 98 },
    { 0x40910412, 95 },
    { 0x5dc10417, 34 },
    { 0xad71040d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca1041e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb510421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa10424, 95 },
    { 0x44d10424, 61 },
    { 0xf6f10426, 34 },
    { 0xc8610426, 98 },
    { 0x7f810425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef11042d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f10430, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e710433, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea510445, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e10448, 95 },
    { 0x67610448, 23 },
    { 0, 0 },
    { 0x9cc1044b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5491044e, 95 },
    { 0, 0 },
    { 0xc3910450, 93 },
    { 0x78610451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae10457, 90 },
    { 0x90610457, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e10468, 43 },
    { 0xba310468, 33 },
    { 0xb141046a, 98 },
    { 0, 0 },
    { 0x2b91046c, 90 },
    { 0xa6f1046d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2810472, 46 },
    { 0x89d10473, 33 },
    { 0x71d10474, 52 },
    { 0x8fb10474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa10479, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1047d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1110482, 66 },
    { 0x76810482, 98 },
    { 0x43d10482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1910488, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x641048b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21048e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83910491, 34 },
    { 0, 0 },
    { 0xe7c10493, 61 },
    { 0x55c10493, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b10499, 99 },
    { 0x66a1049a, 33 },
    { 0xf5b1049b, 33 },
    { 0, 0 },
    { 0xc811049d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c104a0, 38 },
    { 0, 0 },
    { 0xb45104a2, 33 },
    { 0x929104a3, 95 },
    { 0, 0 },
    { 0x481104a5, 61 },
    { 0x561104a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0104ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46104ae, 33 },
    { 0x47b104af, 93 },
    { 0, 0 },
    { 0x30f104b1, 61 },
    { 0xa67104b2, 78 },
    { 0x3d2104b2, 69 },
    { 0x287104b4, 85 },
    { 0xeef104b5, 87 },
    { 0x8e7104b5, 17 },
    { 0x86d104b2, 93 },
    { 0x496104b8, 66 },
    { 0x97a104b9, 91 },
    { 0x20e104ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c104be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce104c3, 61 },
    { 0xaec104c4, 33 },
    { 0xdf9104c5, 46 },
    { 0x849104c6, 78 },
    { 0, 0 },
    { 0x928104c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7104cb, 23 },
    { 0, 0 },
    { 0x590104cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5104d1, 52 },
    { 0, 0 },
    { 0xca4104d3, 76 },
    { 0xab0104d4, 77 },
    { 0, 0 },
    { 0xfca104d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x216104d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e104dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5104e3, 33 },
    { 0xd71104e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539104e9, 89 },
    { 0x134104e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf104ef, 98 },
    { 0xd26104ef, 90 },
    { 0, 0 },
    { 0xcbb104f2, 52 },
    { 0x194104f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58104f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07104fb, 33 },
    { 0, 0 },
    { 0xa26104fd, 33 },
    { 0, 0 },
    { 0x3e1104ff, 95 },
    { 0x11510500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee710506, 55 },
    { 0x4d510506, 23 },
    { 0x11e10507, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841050d, 90 },
    { 0xe111050e, 61 },
    { 0x51d1050f, 76 },
    { 0xd1d10510, 99 },
    { 0x14910511, 55 },
    { 0, 0 },
    { 0x32510513, 55 },
    { 0xe2610514, 55 },
    { 0, 0 },
    { 0xb2610516, 93 },
    { 0, 0 },
    { 0x2be10518, 17 },
    { 0x9bf10519, 33 },
    { 0x60c10519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c91051d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef10529, 98 },
    { 0x7291052a, 75 },
    { 0, 0 },
    { 0x6561052c, 61 },
    { 0xf7a1052c, 76 },
    { 0x5b11052c, 24 },
    { 0x1141052f, 33 },
    { 0x751052f, 90 },
    { 0x22910531, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6710535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94010538, 23 },
    { 0, 0 },
    { 0x4e1053a, 55 },
    { 0xb951053b, 33 },
    { 0xe621053c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c10543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef10549, 55 },
    { 0x8cb1054a, 85 },
    { 0x6cb1054b, 75 },
    { 0xdee1054a, 90 },
    { 0, 0 },
    { 0x5dd1054e, 46 },
    { 0x89e1054f, 76 },
    { 0x4a310550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74610556, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b10559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd410560, 95 },
    { 0, 0 },
    { 0x8a210562, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2601056f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f10572, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac410576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c10579, 90 },
    { 0xbe81057a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e41057f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39610584, 87 },
    { 0x30710585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c91058d, 66 },
    { 0x4761058e, 79 },
    { 0, 0 },
    { 0xd1910590, 55 },
    { 0, 0 },
    { 0xb8510592, 67 },
    { 0xc3110593, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c10597, 67 },
    { 0, 0 },
    { 0xf9d10599, 23 },
    { 0xe881059a, 61 },
    { 0xd4710599, 90 },
    { 0x96610599, 98 },
    { 0x5b81059d, 67 },
    { 0x47f1059b, 61 },
    { 0xffe1059c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a105a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855105b7, 33 },
    { 0x82105b8, 90 },
    { 0x404105b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33105bd, 93 },
    { 0, 0 },
    { 0x1a9105bf, 93 },
    { 0, 0 },
    { 0xc3b105c1, 33 },
    { 0, 0 },
    { 0xb10105c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189105c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192105ca, 90 },
    { 0x444105cb, 98 },
    { 0xd8f105ca, 90 },
    { 0x5c0105cd, 95 },
    { 0x49c105ce, 90 },
    { 0, 0 },
    { 0xe9e105d0, 95 },
    { 0x588105d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b105d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958105dd, 76 },
    { 0, 0 },
    { 0x860105df, 91 },
    { 0x280105df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76105e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d105e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e105eb, 76 },
    { 0xc105ec, 78 },
    { 0xb88105ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5105f5, 55 },
    { 0x13c105f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f10600, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6110606, 52 },
    { 0x63610607, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81710610, 76 },
    { 0, 0 },
    { 0x62f10612, 68 },
    { 0, 0 },
    { 0xa7710614, 95 },
    { 0x32f10615, 91 },
    { 0xb1710615, 55 },
    { 0xa0710617, 93 },
    { 0, 0 },
    { 0x59c10619, 89 },
    { 0, 0 },
    { 0x3001061b, 95 },
    { 0x49d1061b, 90 },
    { 0xd891061c, 67 },
    { 0x77d1061b, 98 },
    { 0xbfc1061f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce010624, 90 },
    { 0, 0 },
    { 0xc3410626, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c10629, 23 },
    { 0x6101062a, 52 },
    { 0x41410629, 40 },
    { 0x661062c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26310633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b10637, 33 },
    { 0, 0 },
    { 0x22c10639, 52 },
    { 0, 0 },
    { 0x3611063b, 76 },
    { 0xbb21063c, 81 },
    { 0x6c21063c, 99 },
    { 0x9931063c, 34 },
    { 0x4e81063c, 80 },
    { 0x6b010640, 56 },
    { 0x2610641, 77 },
    { 0xd3d10642, 61 },
    { 0x86910641, 98 },
    { 0xe6f10640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e10648, 90 },
    { 0xfad10648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b1064d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a10652, 34 },
    { 0x3f010652, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c10657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11065a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa1065d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea10668, 53 },
    { 0, 0 },
    { 0xfc21066a, 76 },
    { 0x7471066a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6071066e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54810671, 61 },
    { 0xf6910672, 90 },
    { 0x3f10671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf510676, 33 },
    { 0xbf510676, 33 },
    { 0xac510678, 98 },
    { 0x4ad10679, 61 },
    { 0, 0 },
    { 0xe651067b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52010681, 61 },
    { 0x40410682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6210685, 81 },
    { 0x52f10685, 95 },
    { 0xde510687, 23 },
    { 0, 0 },
    { 0x3f910689, 55 },
    { 0x7861068a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971068d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc210694, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310697, 55 },
    { 0x67c10697, 61 },
    { 0, 0 },
    { 0x3071069a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691069e, 55 },
    { 0x9121069f, 52 },
    { 0x41d1069f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5106a4, 91 },
    { 0x955106a4, 23 },
    { 0, 0 },
    { 0xfb4106a7, 55 },
    { 0x505106a8, 61 },
    { 0xbd3106a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d106ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f106b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x575106bc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46106c0, 90 },
    { 0, 0 },
    { 0x555106c2, 43 },
    { 0x6ad106c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39106c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d106cd, 81 },
    { 0xb95106ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9106d1, 33 },
    { 0, 0 },
    { 0x15d106d3, 76 },
    { 0x62c106d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b106d7, 98 },
    { 0xf9f106d8, 33 },
    { 0x442106d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46106dd, 81 },
    { 0xf01106dd, 33 },
    { 0x52106df, 61 },
    { 0, 0 },
    { 0x454106e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357106ea, 34 },
    { 0, 0 },
    { 0x75106ec, 93 },
    { 0xc1a106ed, 36 },
    { 0, 0 },
    { 0x12106ef, 98 },
    { 0xf15106f0, 61 },
    { 0xe17106f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5106fd, 76 },
    { 0x612106fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb10703, 88 },
    { 0xf7810704, 34 },
    { 0xb4a10705, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051070a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10712, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a10716, 65 },
    { 0x4b410716, 84 },
    { 0x44410718, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901071b, 69 },
    { 0xd391071c, 95 },
    { 0, 0 },
    { 0x8ea1071e, 90 },
    { 0, 0 },
    { 0x3d410720, 43 },
    { 0xe2010721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b10724, 98 },
    { 0, 0 },
    { 0x2c810726, 90 },
    { 0, 0 },
    { 0x3e310728, 98 },
    { 0, 0 },
    { 0x18b1072a, 81 },
    { 0x9811072b, 46 },
    { 0xd831072b, 98 },
    { 0xdd1072b, 61 },
    { 0xb191072e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac10734, 64 },
    { 0xec310734, 98 },
    { 0x73f10735, 43 },
    { 0, 0 },
    { 0xd610738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251073d, 23 },
    { 0xe711073d, 46 },
    { 0x7491073f, 17 },
    { 0, 0 },
    { 0x28410741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed110746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5021074a, 67 },
    { 0x7e1074b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1661074e, 23 },
    { 0, 0 },
    { 0xf8910750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f310757, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91075d, 98 },
    { 0xfb71075d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67810761, 90 },
    { 0xd3f10761, 56 },
    { 0x5610763, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45510768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2721076d, 33 },
    { 0x1601076d, 98 },
    { 0x43c1076f, 78 },
    { 0x9141076e, 95 },
    { 0x2d110771, 95 },
    { 0x7bd1076d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8261077b, 23 },
    { 0x2691077b, 84 },
    { 0x6ef1077c, 33 },
    { 0x3a71077e, 33 },
    { 0x3921077d, 91 },
    { 0x8d51077d, 100 },
    { 0xc7c10781, 34 },
    { 0xd5a10782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d1078b, 88 },
    { 0xbcb1078b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c10790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df1079b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b107a3, 90 },
    { 0x933107a4, 90 },
    { 0x3f9107a5, 60 },
    { 0x789107a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28107aa, 23 },
    { 0x6e7107aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286107af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x940107b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c107b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef107bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34107c0, 95 },
    { 0xe93107c1, 75 },
    { 0x10f107c2, 100 },
    { 0x1ce107c2, 90 },
    { 0x5d4107c4, 93 },
    { 0, 0 },
    { 0x6fc107c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662107ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172107cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0107d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb107d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x341107da, 90 },
    { 0, 0 },
    { 0x301107dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd107e0, 33 },
    { 0x2c9107e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b107e4, 34 },
    { 0x73b107e5, 98 },
    { 0xff3107e6, 23 },
    { 0, 0 },
    { 0xd54107e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6107ed, 33 },
    { 0xafa107ee, 56 },
    { 0xd94107ef, 98 },
    { 0xa1107f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c107f3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3107fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x905107ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c10804, 98 },
    { 0x20a10805, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b10808, 33 },
    { 0x64a10809, 23 },
    { 0xbcc1080a, 90 },
    { 0, 0 },
    { 0x41d1080c, 34 },
    { 0, 0 },
    { 0x7751080e, 55 },
    { 0x6fa1080f, 75 },
    { 0xc331080e, 34 },
    { 0xb6b10811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd10816, 76 },
    { 0, 0 },
    { 0x4a310818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f1081c, 98 },
    { 0x75b1081d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3510820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d10824, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1082a, 33 },
    { 0x1171082b, 46 },
    { 0xb661082b, 33 },
    { 0xec51082a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1110835, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde71083b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab1083e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9461084b, 2 },
    { 0, 0 },
    { 0x6a21084d, 93 },
    { 0, 0 },
    { 0x64a1084f, 61 },
    { 0x1181084f, 68 },
    { 0x23510851, 78 },
    { 0, 0 },
    { 0x61e10853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73910858, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a1085d, 61 },
    { 0x6331085e, 55 },
    { 0x1181085f, 23 },
    { 0xed010860, 65 },
    { 0, 0 },
    { 0xa6410862, 80 },
    { 0x9f110863, 99 },
    { 0, 0 },
    { 0x59c10865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3861086a, 95 },
    { 0, 0 },
    { 0xf571086c, 95 },
    { 0, 0 },
    { 0xd881086e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d310871, 33 },
    { 0x6b910871, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f10875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010879, 23 },
    { 0xb6c1087a, 95 },
    { 0, 0 },
    { 0x6ec1087c, 87 },
    { 0xe981087c, 90 },
    { 0x2001087e, 46 },
    { 0xcf71087f, 77 },
    { 0x6df1087f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee1088a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1210892, 33 },
    { 0x3dd10893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f510897, 52 },
    { 0x37f10897, 91 },
    { 0xf9810899, 23 },
    { 0x35210899, 61 },
    { 0xa0c1089a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4801089f, 37 },
    { 0xba81089f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106108a4, 47 },
    { 0xdb3108a4, 55 },
    { 0x755108a4, 43 },
    { 0xf92108a7, 88 },
    { 0x8fd108a7, 95 },
    { 0x15c108a9, 98 },
    { 0x3ee108aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf108af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0108b2, 38 },
    { 0xa2f108b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e108b6, 78 },
    { 0x7c8108b7, 76 },
    { 0, 0 },
    { 0xfab108b9, 33 },
    { 0x434108ba, 76 },
    { 0x4aa108b9, 78 },
    { 0xa0c108b9, 55 },
    { 0xf9d108ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69108c7, 90 },
    { 0xb37108c8, 98 },
    { 0x845108c9, 76 },
    { 0x9f1108ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8108d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04108d9, 33 },
    { 0xb64108d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3108df, 93 },
    { 0x99e108df, 61 },
    { 0x136108df, 43 },
    { 0xa49108e2, 90 },
    { 0x958108e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df108e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f108ea, 25 },
    { 0x58a108eb, 23 },
    { 0, 0 },
    { 0x73d108ed, 23 },
    { 0x73e108ee, 90 },
    { 0xff0108ee, 99 },
    { 0x77108f0, 33 },
    { 0, 0 },
    { 0x4a3108f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124108fa, 59 },
    { 0xee5108fb, 23 },
    { 0, 0 },
    { 0x25a108fd, 23 },
    { 0, 0 },
    { 0xe1e108ff, 78 },
    { 0, 0 },
    { 0x53610901, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56810904, 33 },
    { 0xbf610905, 98 },
    { 0, 0 },
    { 0xe0910907, 34 },
    { 0x14410908, 23 },
    { 0, 0 },
    { 0xb571090a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5491090f, 99 },
    { 0x1ca1090f, 95 },
    { 0xfbb10911, 87 },
    { 0x7b410912, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd10915, 33 },
    { 0xd0c10916, 90 },
    { 0x13e10917, 90 },
    { 0xe2910916, 33 },
    { 0xab410917, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa1091d, 52 },
    { 0x57a1091d, 55 },
    { 0, 0 },
    { 0x18e10920, 98 },
    { 0x48e10921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc10924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a10928, 33 },
    { 0x1fd10928, 95 },
    { 0x8601092a, 43 },
    { 0xf2c1092b, 61 },
    { 0x8ee1092b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911092f, 61 },
    { 0x4b710930, 33 },
    { 0xc6d10931, 46 },
    { 0xfe11092f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b10936, 75 },
    { 0, 0 },
    { 0xc2410938, 61 },
    { 0xc3210939, 98 },
    { 0, 0 },
    { 0xd021093b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x651093e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76910945, 33 },
    { 0x6d110946, 72 },
    { 0xff510946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a10956, 95 },
    { 0xc0810957, 55 },
    { 0, 0 },
    { 0x51210959, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091095c, 98 },
    { 0xa801095c, 52 },
    { 0x1a21095c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17710961, 98 },
    { 0, 0 },
    { 0xff710963, 43 },
    { 0x79610964, 95 },
    { 0x5d810965, 90 },
    { 0xa510966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b1096a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5231096e, 66 },
    { 0xb81096f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b10974, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a110979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba1097c, 85 },
    { 0x6c61097d, 76 },
    { 0xefe1097d, 90 },
    { 0x1ec1097f, 67 },
    { 0x6c11097c, 95 },
    { 0, 0 },
    { 0x97310982, 76 },
    { 0xa9b10983, 34 },
    { 0x7e910984, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd10987, 55 },
    { 0xcb910988, 61 },
    { 0x99110988, 76 },
    { 0x12d1098a, 85 },
    { 0x4671098b, 61 },
    { 0x6be1098b, 34 },
    { 0x20110987, 23 },
    { 0x52a10988, 87 },
    { 0x2e81098a, 23 },
    { 0, 0 },
    { 0x96f10991, 43 },
    { 0, 0 },
    { 0x92910993, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c10997, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a1099b, 23 },
    { 0, 0 },
    { 0x51a1099d, 90 },
    { 0x1d21099e, 33 },
    { 0xf371099f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3109a4, 95 },
    { 0xfc8109a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125109aa, 91 },
    { 0x221109aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3109bc, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7109c1, 40 },
    { 0, 0 },
    { 0xc48109c3, 100 },
    { 0x726109c3, 76 },
    { 0x209109c4, 90 },
    { 0xab1109c6, 40 },
    { 0, 0 },
    { 0xe29109c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x510109cb, 90 },
    { 0x381109cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e109d0, 90 },
    { 0x873109d1, 99 },
    { 0x388109d2, 84 },
    { 0xbe9109d3, 77 },
    { 0xa4e109d2, 93 },
    { 0, 0 },
    { 0xe77109d6, 40 },
    { 0x213109d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57109dc, 46 },
    { 0, 0 },
    { 0x87f109de, 40 },
    { 0x5ec109de, 87 },
    { 0, 0 },
    { 0x623109e1, 33 },
    { 0, 0 },
    { 0x665109e3, 78 },
    { 0, 0 },
    { 0xfa1109e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4109ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda109f6, 33 },
    { 0, 0 },
    { 0x57e109f8, 76 },
    { 0, 0 },
    { 0x400109fa, 85 },
    { 0xcb2109fb, 84 },
    { 0, 0 },
    { 0x79109fd, 61 },
    { 0, 0 },
    { 0xe6109ff, 90 },
    { 0, 0 },
    { 0x32010a01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf10a05, 95 },
    { 0xfe810a06, 90 },
    { 0, 0 },
    { 0x62a10a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa610a12, 23 },
    { 0x96910a13, 55 },
    { 0, 0 },
    { 0xecc10a15, 61 },
    { 0xe0c10a15, 95 },
    { 0x18d10a17, 95 },
    { 0xf410a18, 47 },
    { 0, 0 },
    { 0xa3810a1a, 55 },
    { 0x37010a1a, 98 },
    { 0x4010a1c, 23 },
    { 0, 0 },
    { 0x2af10a1e, 55 },
    { 0xc9110a1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a210a26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd710a2e, 88 },
    { 0xae110a2f, 66 },
    { 0xd0910a2f, 98 },
    { 0, 0 },
    { 0x74710a32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e10a40, 90 },
    { 0x18510a41, 46 },
    { 0x3f10a40, 98 },
    { 0x3c410a40, 43 },
    { 0xeba10a43, 90 },
    { 0x31210a45, 91 },
    { 0xd3010a45, 61 },
    { 0, 0 },
    { 0x1d010a48, 33 },
    { 0x23b10a48, 98 },
    { 0x53e10a4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23110a4d, 90 },
    { 0xc0310a4e, 61 },
    { 0x11610a4f, 43 },
    { 0x7d210a50, 72 },
    { 0xc8710a51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe210a57, 33 },
    { 0xdb110a58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed610a5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a310a61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e10a64, 95 },
    { 0x7ce10a65, 23 },
    { 0xeed10a66, 23 },
    { 0xa2f10a67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d10a6f, 61 },
    { 0, 0 },
    { 0x49710a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd10a74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a10a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88310a7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6410a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a85, 23 },
    { 0, 0 },
    { 0xcb610a87, 90 },
    { 0x4e910a88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de10a92, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f10a95, 17 },
    { 0xc3510a96, 98 },
    { 0x2c410a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e10a9b, 99 },
    { 0, 0 },
    { 0xf8510a9d, 95 },
    { 0x42510a9e, 90 },
    { 0, 0 },
    { 0x86410aa0, 46 },
    { 0xeac10aa0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5310aa5, 90 },
    { 0x7ff10aa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0110aaa, 95 },
    { 0x2c710aab, 91 },
    { 0xbd410aac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db10aaf, 100 },
    { 0, 0 },
    { 0x6e210ab1, 95 },
    { 0, 0 },
    { 0xbc10ab3, 98 },
    { 0, 0 },
    { 0x2f010ab5, 36 },
    { 0x9bc10ab5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83910abe, 43 },
    { 0, 0 },
    { 0x15f10ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34510ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c610ac8, 55 },
    { 0, 0 },
    { 0xd2b10aca, 98 },
    { 0x51a10acb, 90 },
    { 0, 0 },
    { 0xb0a10acd, 55 },
    { 0, 0 },
    { 0x74b10acf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3110ad2, 93 },
    { 0, 0 },
    { 0x11810ad4, 33 },
    { 0xe3d10ad5, 99 },
    { 0x9d010ad6, 95 },
    { 0xa0810ad7, 98 },
    { 0xa3710ad8, 23 },
    { 0, 0 },
    { 0xed310ada, 78 },
    { 0x2c210adb, 69 },
    { 0x9f010ada, 90 },
    { 0, 0 },
    { 0xd6f10ade, 61 },
    { 0xb5810adf, 90 },
    { 0xb8710ade, 88 },
    { 0x45310ae0, 61 },
    { 0, 0 },
    { 0x40d10ae3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2810ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91510af2, 23 },
    { 0xbad10af3, 61 },
    { 0x3ff10af4, 33 },
    { 0xc6010af5, 87 },
    { 0x8e810af6, 33 },
    { 0x31910af7, 90 },
    { 0xd7110af7, 91 },
    { 0xd2210af9, 85 },
    { 0x7b710af2, 34 },
    { 0x8d910afb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c10b01, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e710b04, 61 },
    { 0xe410b05, 98 },
    { 0xbb410b04, 76 },
    { 0, 0 },
    { 0x41010b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e10b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb410b0f, 61 },
    { 0x13d10b10, 84 },
    { 0, 0 },
    { 0x18b10b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8010b16, 43 },
    { 0, 0 },
    { 0xdd710b18, 100 },
    { 0x8a410b19, 93 },
    { 0x67b10b1a, 43 },
    { 0x56710b1b, 34 },
    { 0x47910b1b, 96 },
    { 0xe0c10b1d, 33 },
    { 0x11b10b1e, 61 },
    { 0x83210b1d, 88 },
    { 0x66110b1d, 95 },
    { 0x3d810b1b, 33 },
    { 0x9f110b1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34610b26, 33 },
    { 0, 0 },
    { 0x92d10b28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8710b2e, 93 },
    { 0xab710b2f, 90 },
    { 0, 0 },
    { 0xb4510b31, 93 },
    { 0x22710b32, 85 },
    { 0x79a10b33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5810b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96210b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde110b42, 86 },
    { 0, 0 },
    { 0xd8e10b44, 61 },
    { 0x1b010b45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb10b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb10b4b, 95 },
    { 0, 0 },
    { 0x21510b4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d610b52, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd10b55, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb710b5b, 90 },
    { 0x5d210b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a10b60, 100 },
    { 0, 0 },
    { 0x16310b62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53010b6e, 33 },
    { 0x14310b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4910b73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3810b7c, 34 },
    { 0x7ef10b7d, 101 },
    { 0xaac10b7e, 61 },
    { 0x43310b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a510b84, 76 },
    { 0, 0 },
    { 0xf2c10b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7010b8a, 93 },
    { 0x4e110b8b, 76 },
    { 0xa7310b8a, 87 },
    { 0x95710b8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3210b91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69110b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab710ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63110ba4, 43 },
    { 0x99a10ba4, 34 },
    { 0x97b10ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c910baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55610bae, 85 },
    { 0x34510bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3410bb9, 76 },
    { 0xc4010bba, 85 },
    { 0xac810bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10810bc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde510bc7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x90910bca, 23 },
    { 0x3b010bca, 46 },
    { 0x8c910bca, 99 },
    { 0x94b10bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1910bd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f10bd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1910bdb, 95 },
    { 0, 0 },
    { 0x48610bdd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2310be4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a410be7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc310bef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67710bf5, 66 },
    { 0x210bf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32110bf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99710bfc, 43 },
    { 0, 0 },
    { 0x9ae10bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce10c01, 98 },
    { 0, 0 },
    { 0xa3910c03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93210c08, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47810c0e, 34 },
    { 0xca610c0e, 61 },
    { 0x2210c10, 96 },
    { 0, 0 },
    { 0xe4610c12, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d010c15, 98 },
    { 0, 0 },
    { 0x99810c17, 66 },
    { 0xcf410c17, 34 },
    { 0xbd010c19, 17 },
    { 0x88310c1a, 76 },
    { 0, 0 },
    { 0x94210c1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75410c1f, 78 },
    { 0, 0 },
    { 0xe5310c21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a10c24, 98 },
    { 0x8df10c24, 90 },
    { 0x77110c24, 67 },
    { 0xd8210c27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010c2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5310c34, 76 },
    { 0x91710c34, 34 },
    { 0, 0 },
    { 0xf3210c37, 90 },
    { 0xe2910c37, 87 },
    { 0, 0 },
    { 0x88610c3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81110c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc710c43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d810c46, 95 },
    { 0x31b10c47, 77 },
    { 0xb9a10c46, 95 },
    { 0, 0 },
    { 0x3a510c4a, 33 },
    { 0, 0 },
    { 0x91810c4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70510c50, 33 },
    { 0x33210c51, 23 },
    { 0x45d10c52, 33 },
    { 0, 0 },
    { 0xb3a10c54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63010c57, 34 },
    { 0x39810c57, 93 },
    { 0x58310c58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23210c5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d210c5f, 95 },
    { 0xdb10c60, 33 },
    { 0, 0 },
    { 0xfae10c62, 56 },
    { 0x40e10c63, 78 },
    { 0xdfe10c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e810c68, 98 },
    { 0, 0 },
    { 0xc8d10c6a, 83 },
    { 0x20d10c6a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x57210c6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8410c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29010c75, 98 },
    { 0, 0 },
    { 0x80410c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf10c7c, 98 },
    { 0xc910c7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca10c81, 95 },
    { 0xde410c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45610c87, 99 },
    { 0xd1c10c88, 33 },
    { 0x9de10c87, 33 },
    { 0x2bd10c8a, 90 },
    { 0x73410c88, 23 },
    { 0xe4610c8c, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77610c93, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69110c99, 78 },
    { 0, 0 },
    { 0x8de10c9b, 61 },
    { 0xb4110c9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9610c9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d110ca5, 90 },
    { 0x6b810ca6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec010ca9, 33 },
    { 0, 0 },
    { 0x4db10cab, 95 },
    { 0x4aa10cac, 69 },
    { 0x9aa10cad, 90 },
    { 0xf8010cae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1910cb2, 93 },
    { 0xd1f10cb3, 23 },
    { 0xc110cb3, 96 },
    { 0xf7510cb3, 33 },
    { 0x2da10cb6, 95 },
    { 0, 0 },
    { 0x9ce10cb8, 76 },
    { 0x9df10cb9, 43 },
    { 0x50b10cba, 91 },
    { 0x88510cb8, 95 },
    { 0xee710cbc, 33 },
    { 0x62710cbc, 84 },
    { 0x4bc10cb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d210cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c10ccc, 61 },
    { 0, 0 },
    { 0x6110cce, 75 },
    { 0x58810ccf, 33 },
    { 0x2f010cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8210cd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd10cdc, 93 },
    { 0, 0 },
    { 0xa6110cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf10ce2, 33 },
    { 0xcc910ce3, 23 },
    { 0xb1d10ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23710ce7, 100 },
    { 0, 0 },
    { 0xd110ce9, 61 },
    { 0x8c210cea, 98 },
    { 0x6fb10ceb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e610cee, 38 },
    { 0x96410cee, 78 },
    { 0xeea10cee, 55 },
    { 0x19410cef, 55 },
    { 0x50910cf1, 33 },
    { 0xec510cf2, 66 },
    { 0x6510cf4, 61 },
    { 0xf3010cf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb010cf9, 100 },
    { 0x1010cfa, 34 },
    { 0, 0 },
    { 0xb5010cfc, 95 },
    { 0xe7410cfd, 87 },
    { 0, 0 },
    { 0xffb10cff, 99 },
    { 0x92010d00, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a110d06, 42 },
    { 0xb010d06, 67 },
    { 0x23910d08, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f010d10, 67 },
    { 0xa0110d11, 43 },
    { 0x31c10d11, 61 },
    { 0xd8310d11, 90 },
    { 0x13d10d11, 95 },
    { 0xc1310d15, 76 },
    { 0x15f10d15, 33 },
    { 0, 0 },
    { 0x17710d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82110d1b, 23 },
    { 0, 0 },
    { 0xa5910d1d, 67 },
    { 0, 0 },
    { 0xbec10d1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6010d2d, 90 },
    { 0x66110d2d, 34 },
    { 0xefe10d2f, 76 },
    { 0, 0 },
    { 0x81f10d31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e210d34, 76 },
    { 0x14810d34, 66 },
    { 0x9eb10d34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c10d3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32210d41, 61 },
    { 0xd6010d42, 34 },
    { 0x12c10d43, 33 },
    { 0x9aa10d44, 55 },
    { 0xac610d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e10d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab10d4c, 90 },
    { 0xb2710d4d, 51 },
    { 0xcd510d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2910d59, 33 },
    { 0, 0 },
    { 0xb6c10d5b, 98 },
    { 0xca010d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c710d63, 87 },
    { 0x12110d64, 98 },
    { 0xc0510d65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6410d6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d610d6d, 19 },
    { 0x46210d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b10d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7010d79, 61 },
    { 0x23110d7a, 100 },
    { 0, 0 },
    { 0xf3410d7c, 90 },
    { 0x66910d7d, 76 },
    { 0xfec10d7e, 61 },
    { 0xce910d7f, 76 },
    { 0xeba10d80, 98 },
    { 0xbf310d80, 88 },
    { 0x68210d82, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10d86, 38 },
    { 0x8a910d86, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xba710d8a, 90 },
    { 0xccf10d8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d10d96, 61 },
    { 0, 0 },
    { 0x92f10d98, 75 },
    { 0x610d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49810da2, 99 },
    { 0, 0 },
    { 0x6b510da4, 43 },
    { 0, 0 },
    { 0xbd110da6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69210db0, 99 },
    { 0x2d210db0, 76 },
    { 0, 0 },
    { 0x5a110db3, 90 },
    { 0x5c310db4, 55 },
    { 0xce410db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b10dbb, 81 },
    { 0, 0 },
    { 0x12410dbd, 55 },
    { 0xd5910dbd, 55 },
    { 0x5eb10dbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d10dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32010dd0, 90 },
    { 0, 0 },
    { 0x64510dd2, 17 },
    { 0xa1810dd3, 75 },
    { 0x2f210dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba10dd9, 95 },
    { 0, 0 },
    { 0xf6b10ddb, 84 },
    { 0x2d410ddc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37610de1, 75 },
    { 0, 0 },
    { 0x25c10de3, 90 },
    { 0xab310de4, 23 },
    { 0x75c10de4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba410dea, 90 },
    { 0, 0 },
    { 0xca810dec, 52 },
    { 0, 0 },
    { 0xc9310dee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91610df1, 95 },
    { 0, 0 },
    { 0x3d010df3, 98 },
    { 0x7fe10df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4310dfc, 95 },
    { 0x38610dfd, 76 },
    { 0x18f10dfc, 23 },
    { 0, 0 },
    { 0xaee10e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c10e03, 77 },
    { 0, 0 },
    { 0x67210e05, 34 },
    { 0x6fa10e05, 90 },
    { 0x6a410e07, 23 },
    { 0, 0 },
    { 0xc6810e09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1810e0c, 95 },
    { 0xa3510e0d, 80 },
    { 0x6e610e0e, 90 },
    { 0xd6c10e0e, 90 },
    { 0x5e110e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed910e14, 61 },
    { 0xd8210e15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb610e1a, 43 },
    { 0x93110e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79810e20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e10e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f10e2d, 98 },
    { 0xcd410e2d, 33 },
    { 0x16710e2d, 33 },
    { 0xec410e30, 87 },
    { 0, 0 },
    { 0x27210e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e10e39, 55 },
    { 0x39210e39, 95 },
    { 0xe710e3a, 81 },
    { 0x6ca10e39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d10e3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x99410e42, 61 },
    { 0xf5f10e43, 33 },
    { 0x82310e44, 90 },
    { 0x6010e44, 66 },
    { 0, 0 },
    { 0xce810e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77310e4b, 76 },
    { 0, 0 },
    { 0x24010e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9810e50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d10e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e810e56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce310e59, 90 },
    { 0x4cc10e5a, 95 },
    { 0, 0 },
    { 0x61310e5c, 61 },
    { 0xd5110e5c, 98 },
    { 0x2f210e5d, 33 },
    { 0xbfb10e5c, 61 },
    { 0, 0 },
    { 0xe5e10e61, 46 },
    { 0, 0 },
    { 0x63f10e63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f010e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f610e6a, 23 },
    { 0xca910e6b, 46 },
    { 0xfd610e6b, 33 },
    { 0xfe310e6d, 90 },
    { 0x6f610e6c, 55 },
    { 0xd9210e6a, 34 },
    { 0x1cf10e70, 66 },
    { 0x80110e70, 98 },
    { 0x46610e72, 83 },
    { 0x85510e6b, 33 },
    { 0x62510e72, 52 },
    { 0, 0 },
    { 0x3c610e76, 100 },
    { 0, 0 },
    { 0xdda10e78, 93 },
    { 0, 0 },
    { 0x3ef10e7a, 23 },
    { 0, 0 },
    { 0xb9d10e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32210e81, 61 },
    { 0xe310e81, 66 },
    { 0x9d310e81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d910e87, 87 },
    { 0, 0 },
    { 0xfdf10e89, 95 },
    { 0xb2510e8a, 61 },
    { 0x9e410e8b, 95 },
    { 0x20510e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f10e8f, 61 },
    { 0, 0 },
    { 0x1a510e91, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84210e96, 98 },
    { 0x50f10e97, 61 },
    { 0x76d10e96, 90 },
    { 0xe4610e98, 98 },
    { 0xfde10e99, 23 },
    { 0, 0 },
    { 0x2cd10e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71510e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb110ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde510eac, 90 },
    { 0, 0 },
    { 0x43110eae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7110eb1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7510eb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef910eb9, 33 },
    { 0xfc10eba, 91 },
    { 0xf6910eba, 4 },
    { 0xfd810ebc, 34 },
    { 0xcd510ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51410ec3, 61 },
    { 0, 0 },
    { 0xb3610ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f10ec9, 17 },
    { 0x6c310eca, 36 },
    { 0, 0 },
    { 0xa5910ecc, 61 },
    { 0x30310ecd, 87 },
    { 0, 0 },
    { 0x39810ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe10ed2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae10ed5, 23 },
    { 0x3eb10ed5, 33 },
    { 0x4a110ed7, 97 },
    { 0x17910ed7, 98 },
    { 0x41410ed9, 52 },
    { 0x87d10ed8, 61 },
    { 0x8bb10edb, 95 },
    { 0xdb410edc, 98 },
    { 0x9de10edd, 95 },
    { 0, 0 },
    { 0xbc710edf, 99 },
    { 0xf6210ee0, 61 },
    { 0x23710ee0, 34 },
    { 0xf3410ee2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa10ee8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94610eeb, 33 },
    { 0x8eb10eec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab10eef, 99 },
    { 0x6af10ef0, 98 },
    { 0x50c10eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20410ef4, 94 },
    { 0x85710ef5, 98 },
    { 0, 0 },
    { 0x6ba10ef7, 61 },
    { 0xfd410ef8, 33 },
    { 0x24610ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf10f01, 90 },
    { 0x79310f02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b10f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb710f09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e010f0e, 66 },
    { 0, 0 },
    { 0x2aa10f10, 40 },
    { 0xc3210f11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9710f14, 55 },
    { 0x5cc10f15, 95 },
    { 0x9f910f16, 95 },
    { 0xf4910f16, 98 },
    { 0x6d310f18, 84 },
    { 0xfa910f19, 22 },
    { 0, 0 },
    { 0xdb910f1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef610f1e, 61 },
    { 0x3b410f1f, 90 },
    { 0x5ea10f1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45910f23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c10f26, 80 },
    { 0x7b710f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96210f2d, 33 },
    { 0x72110f2e, 33 },
    { 0x3a710f2e, 90 },
    { 0xe5d10f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f10f33, 46 },
    { 0x41310f33, 98 },
    { 0x57110f35, 90 },
    { 0x5a710f36, 33 },
    { 0, 0 },
    { 0x93810f38, 90 },
    { 0x64110f39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79510f40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea410f44, 66 },
    { 0, 0 },
    { 0x44c10f46, 99 },
    { 0xc9f10f46, 76 },
    { 0xa7d10f47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e610f4b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d10f4e, 61 },
    { 0x14f10f4f, 46 },
    { 0xce810f50, 46 },
    { 0, 0 },
    { 0xff810f52, 23 },
    { 0x6c610f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a10f57, 90 },
    { 0xd2510f57, 34 },
    { 0xb1810f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f10f5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40010f61, 61 },
    { 0xd2d10f62, 61 },
    { 0x83610f63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a10f66, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e10f69, 23 },
    { 0, 0 },
    { 0x2bf10f6b, 98 },
    { 0x83910f6c, 17 },
    { 0, 0 },
    { 0xb5710f6e, 61 },
    { 0, 0 },
    { 0x78d10f70, 55 },
    { 0xb1e10f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4210f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d710f78, 20 },
    { 0xa8510f78, 100 },
    { 0x5b210f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a810f80, 55 },
    { 0xda510f80, 82 },
    { 0x98d10f82, 33 },
    { 0xdc810f83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90210f87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e10f91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92810f9a, 76 },
    { 0, 0 },
    { 0x3e010f9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd10fa0, 61 },
    { 0, 0 },
    { 0x67810fa2, 43 },
    { 0, 0 },
    { 0xd410fa4, 93 },
    { 0xfd910fa5, 33 },
    { 0, 0 },
    { 0x2a710fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0410fad, 91 },
    { 0x7f810fad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21610fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb10fb8, 90 },
    { 0xad110fb9, 43 },
    { 0x6fb10fb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d910fbd, 76 },
    { 0xea710fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6310fc5, 91 },
    { 0xa5810fc6, 33 },
    { 0x9710fc7, 87 },
    { 0xa3310fc5, 77 },
    { 0x91010fc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f910fcf, 33 },
    { 0, 0 },
    { 0xf7c10fd1, 33 },
    { 0xac10fd2, 23 },
    { 0x9e710fd3, 53 },
    { 0xd0510fd3, 93 },
    { 0xda910fd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad710fda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6210fde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb10fe1, 95 },
    { 0x47810fe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c910fe6, 33 },
    { 0xdf010fe6, 43 },
    { 0x1c910fe7, 90 },
    { 0, 0 },
    { 0xa0d10fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b10fed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb10ff2, 34 },
    { 0x33710ff3, 33 },
    { 0xb8b10ff4, 61 },
    { 0xcde10ff5, 46 },
    { 0x5b10ff6, 95 },
    { 0x6d010ff6, 34 },
    { 0xc4b10ff5, 84 },
    { 0x3ea10ff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e10ffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45810fff, 34 },
    { 0x40611000, 48 },
    { 0x69811001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4611007, 93 },
    { 0x9f611008, 95 },
    { 0, 0 },
    { 0x78b1100a, 95 },
    { 0x5d01100b, 77 },
    { 0x5421100c, 95 },
    { 0x47b1100d, 23 },
    { 0, 0 },
    { 0x56c1100f, 12 },
    { 0, 0 },
    { 0x90011011, 100 },
    { 0, 0 },
    { 0xc3f11013, 23 },
    { 0x86711014, 90 },
    { 0, 0 },
    { 0x63d11016, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251101a, 36 },
    { 0, 0 },
    { 0xaa91101c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1681101f, 34 },
    { 0x72811020, 46 },
    { 0x69d11021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4211102a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa11031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5411041, 55 },
    { 0xaa611042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d11045, 43 },
    { 0xfa111046, 46 },
    { 0, 0 },
    { 0x60911048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf561104c, 33 },
    { 0xb2a1104c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15411051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d11055, 76 },
    { 0, 0 },
    { 0x4d911057, 51 },
    { 0x53e11058, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc711105e, 98 },
    { 0xb3d1105e, 23 },
    { 0x1641105f, 33 },
    { 0x8e11061, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a611066, 90 },
    { 0x39d11066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad11070, 61 },
    { 0, 0 },
    { 0x96e11072, 23 },
    { 0x6b311073, 23 },
    { 0x11911074, 78 },
    { 0, 0 },
    { 0x90011076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e11079, 95 },
    { 0, 0 },
    { 0xed21107b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12211082, 91 },
    { 0xab11083, 61 },
    { 0, 0 },
    { 0x9a611085, 55 },
    { 0x89e11085, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3761108b, 78 },
    { 0x5fe1108b, 55 },
    { 0x90f1108d, 91 },
    { 0x1c61108e, 95 },
    { 0x6191108f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50111093, 76 },
    { 0, 0 },
    { 0x10911095, 95 },
    { 0x85e11096, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553110a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f110ab, 76 },
    { 0, 0 },
    { 0x323110ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x337110b0, 93 },
    { 0, 0 },
    { 0xc39110b2, 23 },
    { 0xcff110b3, 90 },
    { 0x129110b4, 91 },
    { 0, 0 },
    { 0xcdf110b6, 90 },
    { 0xde0110b6, 76 },
    { 0, 0 },
    { 0xfc4110b9, 67 },
    { 0xfcf110ba, 23 },
    { 0x2e3110b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9110be, 55 },
    { 0x249110bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693110c5, 34 },
    { 0x53b110c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2110ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2110d1, 90 },
    { 0x780110d1, 90 },
    { 0xd27110d3, 84 },
    { 0x69110d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402110d9, 76 },
    { 0, 0 },
    { 0xd80110db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x264110de, 55 },
    { 0x150110df, 90 },
    { 0x2c8110de, 34 },
    { 0x62110e1, 90 },
    { 0x810110e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d110e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24110eb, 40 },
    { 0xe99110eb, 85 },
    { 0, 0 },
    { 0x26f110ee, 33 },
    { 0xe15110ee, 100 },
    { 0, 0 },
    { 0x6fb110f1, 90 },
    { 0, 0 },
    { 0x1ae110f3, 84 },
    { 0xeda110f3, 90 },
    { 0x9aa110f5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d110f8, 75 },
    { 0, 0 },
    { 0x9e8110fa, 93 },
    { 0xa7f110fb, 95 },
    { 0x4a2110fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc11100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11103, 67 },
    { 0, 0 },
    { 0x85111105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c211108, 98 },
    { 0, 0 },
    { 0x2771110a, 61 },
    { 0x8e91110b, 98 },
    { 0x6f41110a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1110f, 98 },
    { 0x5461110f, 98 },
    { 0x17a11111, 34 },
    { 0x9dd11112, 93 },
    { 0x5f311113, 23 },
    { 0, 0 },
    { 0x67911115, 61 },
    { 0x22711116, 93 },
    { 0, 0 },
    { 0x29011118, 23 },
    { 0xe8411119, 23 },
    { 0, 0 },
    { 0x2c61111b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e11121, 34 },
    { 0x56711122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121112a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1112d, 33 },
    { 0xdfd1112d, 95 },
    { 0xacf1112d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e11133, 69 },
    { 0x5c511134, 23 },
    { 0xddb11135, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad1113b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9561113e, 23 },
    { 0x6c11113f, 78 },
    { 0x1861113f, 23 },
    { 0x44a11141, 61 },
    { 0, 0 },
    { 0x32c11143, 58 },
    { 0, 0 },
    { 0x44f11145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad1114b, 90 },
    { 0, 0 },
    { 0x6681114d, 91 },
    { 0x3c61114e, 99 },
    { 0, 0 },
    { 0x28011150, 23 },
    { 0x81a11150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb11154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd811115b, 84 },
    { 0xadc1115b, 55 },
    { 0, 0 },
    { 0x7a91115e, 33 },
    { 0, 0 },
    { 0x8511160, 76 },
    { 0xfe811161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab11165, 90 },
    { 0, 0 },
    { 0x48111167, 61 },
    { 0x32511168, 61 },
    { 0x7d911168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd01116f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53611176, 55 },
    { 0xb5c11176, 69 },
    { 0, 0 },
    { 0x7b711179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b21117f, 88 },
    { 0x2a211180, 87 },
    { 0xb0111181, 90 },
    { 0, 0 },
    { 0xcd711183, 65 },
    { 0, 0 },
    { 0x6111185, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01118a, 61 },
    { 0x2951118b, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f211191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5111194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5721119c, 61 },
    { 0xfad1119c, 90 },
    { 0x1091119e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd111a3, 33 },
    { 0xbad111a3, 98 },
    { 0xf50111a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a111aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4111b3, 61 },
    { 0xde4111b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407111ba, 90 },
    { 0xf2b111ba, 55 },
    { 0x8f111bc, 95 },
    { 0xd4a111bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5111c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f111c9, 90 },
    { 0, 0 },
    { 0x205111cb, 38 },
    { 0xe38111cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f111d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f111d6, 76 },
    { 0x111d7, 55 },
    { 0, 0 },
    { 0x8cc111d9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb111de, 90 },
    { 0, 0 },
    { 0x5bd111e0, 23 },
    { 0xb40111e1, 53 },
    { 0xd57111e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c111e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2111e8, 34 },
    { 0, 0 },
    { 0x4b5111ea, 23 },
    { 0, 0 },
    { 0x3c0111ec, 93 },
    { 0x3e0111ed, 68 },
    { 0x3b5111ee, 85 },
    { 0x8f7111ee, 33 },
    { 0x5d6111ee, 55 },
    { 0xa22111f1, 68 },
    { 0xca5111f2, 52 },
    { 0, 0 },
    { 0x8b5111f4, 40 },
    { 0, 0 },
    { 0x156111f6, 90 },
    { 0, 0 },
    { 0xc24111f8, 90 },
    { 0xc63111f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35111fe, 98 },
    { 0, 0 },
    { 0xe9211200, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d11205, 99 },
    { 0xffd11205, 90 },
    { 0x8f11206, 76 },
    { 0xef711206, 33 },
    { 0xae511209, 61 },
    { 0xc4f11209, 33 },
    { 0xd3d1120b, 75 },
    { 0x51e11206, 34 },
    { 0x2ae1120d, 34 },
    { 0x6cb1120d, 100 },
    { 0xc001120f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f11213, 90 },
    { 0, 0 },
    { 0xcc811215, 61 },
    { 0xfe211216, 61 },
    { 0xe7d11215, 23 },
    { 0, 0 },
    { 0x86e11219, 61 },
    { 0, 0 },
    { 0x44c1121b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75811220, 55 },
    { 0x58311221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e911224, 33 },
    { 0x4c211224, 33 },
    { 0x45711225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1122a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c111230, 17 },
    { 0xaaf11231, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b11238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1531123c, 23 },
    { 0xe9a1123d, 90 },
    { 0x83c1123e, 81 },
    { 0xfd51123e, 90 },
    { 0x87e11240, 95 },
    { 0xa141123e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68011244, 55 },
    { 0x31911244, 43 },
    { 0x34b11246, 76 },
    { 0x8f111247, 98 },
    { 0, 0 },
    { 0xdd711249, 98 },
    { 0xc6511249, 43 },
    { 0xac51124b, 46 },
    { 0xcc911249, 66 },
    { 0xa1f1124d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae11250, 95 },
    { 0xf5e11250, 90 },
    { 0xbcc11252, 90 },
    { 0xec211252, 95 },
    { 0, 0 },
    { 0xca11255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9511267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc31126e, 23 },
    { 0x411126f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e11272, 99 },
    { 0xc7311273, 43 },
    { 0xe6811274, 84 },
    { 0x7d811275, 90 },
    { 0x92111276, 34 },
    { 0, 0 },
    { 0x11111278, 43 },
    { 0x7e111278, 23 },
    { 0xc2811278, 34 },
    { 0, 0 },
    { 0x88e1127c, 23 },
    { 0, 0 },
    { 0xc511127e, 95 },
    { 0x5d91127f, 87 },
    { 0xeb411280, 23 },
    { 0xf6311280, 81 },
    { 0, 0 },
    { 0x55611283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b411286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71128a, 61 },
    { 0, 0 },
    { 0xb781128c, 65 },
    { 0, 0 },
    { 0x9c11128e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc211293, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30011298, 46 },
    { 0, 0 },
    { 0xc891129a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1129e, 90 },
    { 0x53f1129f, 98 },
    { 0xbf81129f, 85 },
    { 0x8ea112a1, 23 },
    { 0xb8f112a0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2112aa, 90 },
    { 0xac0112aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a112b2, 90 },
    { 0x1e7112b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32112b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733112be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3112c3, 84 },
    { 0x222112c4, 34 },
    { 0x8e8112c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9112c9, 95 },
    { 0x786112ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d112cf, 33 },
    { 0, 0 },
    { 0x895112d1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892112d5, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359112d9, 99 },
    { 0x52b112da, 23 },
    { 0x2b7112da, 98 },
    { 0x7d8112d9, 65 },
    { 0, 0 },
    { 0x54e112de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c112e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9112e9, 98 },
    { 0x434112ea, 61 },
    { 0x983112eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef112ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635112f3, 81 },
    { 0xee1112f3, 78 },
    { 0, 0 },
    { 0xe64112f6, 61 },
    { 0xd04112f7, 33 },
    { 0x4ff112f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd112fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac811301, 90 },
    { 0, 0 },
    { 0xebb11303, 85 },
    { 0xd0811303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52111308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c11313, 85 },
    { 0x36911313, 95 },
    { 0, 0 },
    { 0xf7711316, 87 },
    { 0xfc311317, 90 },
    { 0, 0 },
    { 0x57511319, 52 },
    { 0xb41131a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41131d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8111323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0111329, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1132d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d511334, 76 },
    { 0, 0 },
    { 0xba11336, 95 },
    { 0xff611336, 90 },
    { 0x6fb11338, 98 },
    { 0xfa811337, 90 },
    { 0xb1d1133a, 78 },
    { 0x65f1133a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a1133e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xef811341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45611344, 23 },
    { 0, 0 },
    { 0xe9311346, 61 },
    { 0x3db11347, 90 },
    { 0, 0 },
    { 0xd1d11349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1134c, 76 },
    { 0, 0 },
    { 0x7491134e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6611352, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab11356, 61 },
    { 0x2e911356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb1135a, 76 },
    { 0xac71135a, 51 },
    { 0x4241135c, 78 },
    { 0, 0 },
    { 0x4a11135e, 98 },
    { 0xd2c1135f, 95 },
    { 0, 0 },
    { 0x26511361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81511369, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7621136c, 90 },
    { 0x9031136c, 37 },
    { 0x7e31136e, 34 },
    { 0x4ec1136f, 56 },
    { 0, 0 },
    { 0x39311371, 95 },
    { 0x93511372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99511376, 98 },
    { 0xfe011376, 98 },
    { 0, 0 },
    { 0xec111379, 90 },
    { 0x86e1137a, 88 },
    { 0x9591137a, 66 },
    { 0x28211379, 90 },
    { 0x25b1137d, 33 },
    { 0x8f81137e, 95 },
    { 0x2c01137f, 33 },
    { 0x711380, 46 },
    { 0x84a11380, 84 },
    { 0x4b11137a, 33 },
    { 0x63a1137e, 76 },
    { 0xafb11384, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be11387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2411138d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xede11390, 61 },
    { 0, 0 },
    { 0xa3f11392, 34 },
    { 0xa1c11393, 55 },
    { 0, 0 },
    { 0x57e11395, 33 },
    { 0x3bd11395, 99 },
    { 0x24311397, 61 },
    { 0x27611396, 95 },
    { 0, 0 },
    { 0xf3d1139a, 100 },
    { 0x6af1139b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276113a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0113a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838113b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39113b9, 61 },
    { 0x7f0113ba, 95 },
    { 0xea0113bb, 95 },
    { 0xf7d113bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f113c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1113c6, 43 },
    { 0x95f113c7, 85 },
    { 0x703113c8, 90 },
    { 0, 0 },
    { 0x1ae113ca, 33 },
    { 0, 0 },
    { 0x567113cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a113d0, 90 },
    { 0, 0 },
    { 0x437113d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b113d7, 65 },
    { 0, 0 },
    { 0xab7113d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676113de, 95 },
    { 0, 0 },
    { 0x872113e0, 95 },
    { 0, 0 },
    { 0x7b3113e2, 34 },
    { 0xcdb113e3, 33 },
    { 0, 0 },
    { 0x189113e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c113e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04113ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6113f3, 93 },
    { 0x2d5113f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7113fa, 55 },
    { 0, 0 },
    { 0xb89113fc, 23 },
    { 0x9e0113fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a11400, 61 },
    { 0xf7311401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc51140b, 61 },
    { 0x1091140b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f51140f, 40 },
    { 0, 0 },
    { 0x40b11411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38711414, 78 },
    { 0, 0 },
    { 0x19d11416, 90 },
    { 0xd0811417, 98 },
    { 0xd911418, 36 },
    { 0xeeb11419, 98 },
    { 0x13c1141a, 46 },
    { 0xadc1141b, 23 },
    { 0x62e1141c, 91 },
    { 0x15a1141b, 96 },
    { 0x90b1141c, 33 },
    { 0x1ef1141a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc11427, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c1142b, 23 },
    { 0x6541142c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61142f, 84 },
    { 0x81611430, 33 },
    { 0x58111430, 33 },
    { 0xc0c11430, 76 },
    { 0x50811433, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811437, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e1143a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a11447, 78 },
    { 0xc4011448, 98 },
    { 0, 0 },
    { 0x9361144a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b61144f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef11454, 33 },
    { 0xe8e11455, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901145b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a111460, 17 },
    { 0xdf011461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80111465, 61 },
    { 0xf1e11466, 85 },
    { 0xf3011467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11146c, 23 },
    { 0x25d1146c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67111470, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c811473, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3811476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221147b, 33 },
    { 0, 0 },
    { 0xbb11147d, 76 },
    { 0x7c41147d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37811487, 23 },
    { 0xc2411488, 23 },
    { 0, 0 },
    { 0x2f71148a, 90 },
    { 0x4e11148b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12711490, 61 },
    { 0x64011491, 91 },
    { 0xe2411491, 95 },
    { 0x76f11491, 76 },
    { 0xd9911493, 90 },
    { 0x96911495, 76 },
    { 0xdb611491, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca11499, 55 },
    { 0xdb1149a, 98 },
    { 0x8ce1149b, 66 },
    { 0xf271149c, 98 },
    { 0x2bf1149d, 84 },
    { 0xd821149d, 90 },
    { 0x3951149e, 98 },
    { 0, 0 },
    { 0x100114a1, 90 },
    { 0x748114a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50114ad, 85 },
    { 0x40f114ae, 43 },
    { 0xb7b114af, 76 },
    { 0x58c114ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22114b5, 46 },
    { 0, 0 },
    { 0xdc7114b7, 81 },
    { 0xc8d114b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40114c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95114c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459114c8, 61 },
    { 0xd63114c9, 93 },
    { 0, 0 },
    { 0xa84114cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2114cf, 33 },
    { 0, 0 },
    { 0x6a2114d1, 93 },
    { 0, 0 },
    { 0xb44114d3, 23 },
    { 0xff6114d4, 61 },
    { 0x7f4114d5, 94 },
    { 0xb53114d6, 76 },
    { 0xbd2114d5, 90 },
    { 0x8f2114d6, 61 },
    { 0x7f1114d9, 55 },
    { 0, 0 },
    { 0x3f0114db, 44 },
    { 0xf77114dc, 98 },
    { 0x64e114dd, 91 },
    { 0, 0 },
    { 0xd3114df, 55 },
    { 0x46114df, 67 },
    { 0x4d8114e0, 95 },
    { 0x2a5114e2, 61 },
    { 0, 0 },
    { 0xeb7114e4, 23 },
    { 0x865114e4, 93 },
    { 0, 0 },
    { 0xa30114e7, 78 },
    { 0, 0 },
    { 0x356114e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade114ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf114f1, 85 },
    { 0x1ef114f2, 95 },
    { 0, 0 },
    { 0xe3c114f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a114f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584114fe, 69 },
    { 0, 0 },
    { 0xb3011500, 98 },
    { 0x37311500, 43 },
    { 0, 0 },
    { 0xcdc11503, 90 },
    { 0, 0 },
    { 0x5ea11505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641150a, 36 },
    { 0xc451150b, 33 },
    { 0, 0 },
    { 0xd631150d, 55 },
    { 0xb361150e, 90 },
    { 0x5c91150e, 68 },
    { 0x79311510, 76 },
    { 0x25911511, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99311516, 87 },
    { 0xb9a11517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d81151b, 33 },
    { 0xef91151c, 33 },
    { 0xe181151d, 23 },
    { 0x6441151e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b11521, 61 },
    { 0xe6411522, 77 },
    { 0x74711522, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61152b, 33 },
    { 0, 0 },
    { 0xd831152d, 61 },
    { 0, 0 },
    { 0x5281152f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c511538, 33 },
    { 0xedb11539, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221153c, 91 },
    { 0x1211153d, 90 },
    { 0, 0 },
    { 0xdaa1153f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9811542, 95 },
    { 0, 0 },
    { 0x24611544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc11549, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6741154c, 55 },
    { 0xad1154d, 61 },
    { 0xde01154e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb1155a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3361155d, 99 },
    { 0x9401155e, 96 },
    { 0xcfa1155d, 96 },
    { 0x97a11560, 99 },
    { 0xc0711560, 95 },
    { 0, 0 },
    { 0x95311563, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d11568, 42 },
    { 0x40f11569, 33 },
    { 0xdd1156a, 61 },
    { 0x1481156b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1156e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad711574, 90 },
    { 0x23911575, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd71157c, 76 },
    { 0xb61157d, 76 },
    { 0xa151157e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53111586, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2401158a, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4951158f, 95 },
    { 0x46e11590, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091159a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd115a1, 95 },
    { 0x8b2115a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342115ad, 34 },
    { 0, 0 },
    { 0x271115af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e115b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5115b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2115bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0115c8, 98 },
    { 0xb4a115c8, 98 },
    { 0, 0 },
    { 0xf0115cb, 55 },
    { 0xafe115cc, 33 },
    { 0, 0 },
    { 0x526115ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4115d5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd115da, 98 },
    { 0xd30115db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547115e3, 95 },
    { 0x5ea115e4, 52 },
    { 0x11d115e5, 90 },
    { 0xd22115e6, 34 },
    { 0x422115e3, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96115ed, 61 },
    { 0xabb115ee, 87 },
    { 0x1ea115ee, 39 },
    { 0x588115ef, 55 },
    { 0xdf4115f1, 55 },
    { 0xd83115f2, 61 },
    { 0xfe115f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101115f7, 33 },
    { 0x6e2115f8, 90 },
    { 0xd3a115f9, 61 },
    { 0xb81115fa, 87 },
    { 0x6d2115fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86411605, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e11609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb801160f, 23 },
    { 0, 0 },
    { 0xbfe11611, 81 },
    { 0xd3f11612, 23 },
    { 0xea211611, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a11619, 95 },
    { 0xc801161a, 99 },
    { 0x36e11619, 76 },
    { 0x7071161c, 77 },
    { 0x561161d, 92 },
    { 0x291161c, 90 },
    { 0, 0 },
    { 0x90711620, 87 },
    { 0xb2511621, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a11624, 77 },
    { 0xbf011624, 98 },
    { 0x1b411624, 90 },
    { 0xdb11627, 66 },
    { 0xb1111627, 68 },
    { 0x8fd11629, 76 },
    { 0x9c611624, 36 },
    { 0, 0 },
    { 0xbf91162c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe411631, 43 },
    { 0x3bc11632, 98 },
    { 0x38711633, 76 },
    { 0x7a11633, 100 },
    { 0x10a11633, 95 },
    { 0x80911636, 66 },
    { 0x81211635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb11163a, 95 },
    { 0xe521163b, 91 },
    { 0x6521163c, 61 },
    { 0x4061163b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c11644, 67 },
    { 0, 0 },
    { 0xfd211646, 33 },
    { 0x58211647, 23 },
    { 0xefc11648, 34 },
    { 0x9b211649, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91164f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15311653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc11657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d1165a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f1165d, 61 },
    { 0x22e1165d, 93 },
    { 0, 0 },
    { 0xeec11660, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81166a, 95 },
    { 0x6c71166b, 34 },
    { 0x4d71166c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36111674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61167e, 76 },
    { 0xc0e1167e, 75 },
    { 0, 0 },
    { 0x16211681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f11686, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e1168c, 33 },
    { 0x6c31168d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a311693, 33 },
    { 0xa5811693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11511698, 61 },
    { 0xb911698, 65 },
    { 0, 0 },
    { 0x231169b, 61 },
    { 0x96c1169c, 90 },
    { 0, 0 },
    { 0x2111169e, 55 },
    { 0, 0 },
    { 0x27d116a0, 33 },
    { 0, 0 },
    { 0x89a116a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca116a7, 90 },
    { 0, 0 },
    { 0x31c116a9, 68 },
    { 0xc8116a9, 61 },
    { 0, 0 },
    { 0x472116ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481116b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38116b5, 98 },
    { 0xba7116b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8116b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8116be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa116ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd116cd, 23 },
    { 0xd08116ce, 66 },
    { 0x56c116ce, 95 },
    { 0x3fe116d0, 33 },
    { 0xaca116d1, 23 },
    { 0x13f116d2, 84 },
    { 0x3a7116d2, 33 },
    { 0, 0 },
    { 0x5cb116d5, 23 },
    { 0x4a4116d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e116db, 99 },
    { 0, 0 },
    { 0xff2116dd, 95 },
    { 0x615116de, 97 },
    { 0, 0 },
    { 0xd04116e0, 23 },
    { 0x939116e1, 90 },
    { 0xf23116e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7116e6, 90 },
    { 0, 0 },
    { 0xca4116e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952116ec, 69 },
    { 0, 0 },
    { 0x457116ee, 33 },
    { 0x9a9116ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5116f3, 95 },
    { 0xb5116f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400116f8, 78 },
    { 0x844116f9, 55 },
    { 0x4a116fa, 46 },
    { 0x3a3116fb, 23 },
    { 0xe1a116fc, 43 },
    { 0xe8d116fd, 90 },
    { 0x5da116fe, 98 },
    { 0x72f116fd, 52 },
    { 0x15611700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86311706, 78 },
    { 0, 0 },
    { 0xe0911708, 33 },
    { 0x6fb11709, 67 },
    { 0xfbd1170a, 61 },
    { 0x6e211709, 95 },
    { 0x54c1170c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b11713, 76 },
    { 0xca711714, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3311171d, 2 },
    { 0x3be1171d, 33 },
    { 0x6d31171f, 76 },
    { 0x27b1171d, 90 },
    { 0x21711720, 33 },
    { 0, 0 },
    { 0x4ca11723, 34 },
    { 0xdbc11724, 95 },
    { 0xacf11725, 88 },
    { 0xfdf11723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c1172a, 61 },
    { 0, 0 },
    { 0x50e1172c, 90 },
    { 0, 0 },
    { 0xbcf1172e, 53 },
    { 0x57c1172e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6711732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a11736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1173e, 77 },
    { 0xb3f1173f, 84 },
    { 0xd1b11740, 90 },
    { 0xe3e11741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc11747, 99 },
    { 0xe4511747, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e1174e, 66 },
    { 0x2991174f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde111754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a11759, 95 },
    { 0x76311759, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e11175d, 55 },
    { 0xae61175d, 75 },
    { 0, 0 },
    { 0x88b11760, 61 },
    { 0x70611761, 23 },
    { 0, 0 },
    { 0xc2011763, 98 },
    { 0x3e611764, 33 },
    { 0, 0 },
    { 0x29c11766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7961176c, 85 },
    { 0, 0 },
    { 0xb4c1176e, 43 },
    { 0x1671176e, 58 },
    { 0xdce11770, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57811776, 95 },
    { 0, 0 },
    { 0x8111778, 78 },
    { 0x95411779, 90 },
    { 0, 0 },
    { 0x4291177b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd01177f, 90 },
    { 0, 0 },
    { 0xcc711781, 97 },
    { 0, 0 },
    { 0xc6d11783, 67 },
    { 0xa8911784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e11789, 100 },
    { 0, 0 },
    { 0xb001178b, 61 },
    { 0x2901178c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1178f, 90 },
    { 0x65e1178f, 55 },
    { 0, 0 },
    { 0x43411792, 98 },
    { 0x43b11792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f611798, 61 },
    { 0xfc211799, 87 },
    { 0x3711179a, 23 },
    { 0x4b51179a, 23 },
    { 0xc591179b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b117a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53117ad, 95 },
    { 0, 0 },
    { 0x199117af, 95 },
    { 0x1ff117b0, 98 },
    { 0xe76117b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590117b4, 34 },
    { 0, 0 },
    { 0xe0a117b6, 61 },
    { 0x913117b7, 95 },
    { 0x39117b8, 33 },
    { 0x582117b6, 33 },
    { 0x34d117b9, 31 },
    { 0x40d117b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f117be, 98 },
    { 0, 0 },
    { 0x9ee117c0, 91 },
    { 0xdb7117c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d117c8, 98 },
    { 0x58e117c8, 90 },
    { 0, 0 },
    { 0x79b117cb, 38 },
    { 0x763117cc, 52 },
    { 0xed1117cc, 52 },
    { 0xaeb117ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc117d2, 84 },
    { 0x713117d3, 91 },
    { 0, 0 },
    { 0x305117d5, 33 },
    { 0x9bb117d6, 100 },
    { 0xe06117d6, 22 },
    { 0xd8b117d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63117db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0117e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b117e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x505117e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6117ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5117f2, 33 },
    { 0, 0 },
    { 0x375117f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f117fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a11800, 61 },
    { 0, 0 },
    { 0x74911802, 23 },
    { 0x92711803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf11806, 69 },
    { 0xd0911807, 23 },
    { 0x59e11808, 61 },
    { 0x22e11806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11180f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c611817, 90 },
    { 0, 0 },
    { 0x6d311819, 66 },
    { 0x5be1181a, 76 },
    { 0x3451181b, 68 },
    { 0xcef1181c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6881181f, 68 },
    { 0xae611820, 68 },
    { 0, 0 },
    { 0xa611822, 69 },
    { 0xd9611823, 68 },
    { 0x18011823, 55 },
    { 0xd0611824, 66 },
    { 0xb5011826, 95 },
    { 0x6d11825, 23 },
    { 0x73c11824, 34 },
    { 0x4be11829, 98 },
    { 0xd8611826, 98 },
    { 0xb2911826, 61 },
    { 0xfb01182c, 90 },
    { 0, 0 },
    { 0xf381182e, 61 },
    { 0x7731182f, 77 },
    { 0xf1d1182f, 43 },
    { 0x25a11830, 87 },
    { 0xe3311832, 34 },
    { 0x57911833, 33 },
    { 0xe6e11833, 95 },
    { 0xe2a11830, 98 },
    { 0, 0 },
    { 0x25c11837, 100 },
    { 0xa1011837, 93 },
    { 0x41611839, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8611840, 93 },
    { 0xd4211841, 34 },
    { 0x49e11842, 33 },
    { 0, 0 },
    { 0x7c411844, 33 },
    { 0xcf511845, 90 },
    { 0x1f711846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921184a, 98 },
    { 0x4101184b, 33 },
    { 0x4291184a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d1184f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe611856, 33 },
    { 0xbef11857, 90 },
    { 0xad311858, 95 },
    { 0x60111858, 34 },
    { 0, 0 },
    { 0x2541185b, 23 },
    { 0, 0 },
    { 0x8a71185d, 85 },
    { 0x3db1185e, 98 },
    { 0xa601185f, 33 },
    { 0, 0 },
    { 0xd8011861, 76 },
    { 0xc2111862, 66 },
    { 0x28411863, 90 },
    { 0xb8411862, 99 },
    { 0x2c511865, 33 },
    { 0xa5a11866, 99 },
    { 0xff11864, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b1186b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda1186f, 33 },
    { 0, 0 },
    { 0x5a711871, 38 },
    { 0xeac11872, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b11875, 90 },
    { 0xa2a11876, 61 },
    { 0xecb11876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe021187c, 61 },
    { 0xfe51187d, 90 },
    { 0x2f51187d, 76 },
    { 0xfa51187d, 98 },
    { 0, 0 },
    { 0xad011881, 43 },
    { 0xaba11882, 90 },
    { 0xb2611882, 58 },
    { 0xedb11884, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf911887, 68 },
    { 0xdd011888, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a11894, 17 },
    { 0x24e11894, 52 },
    { 0x2b111894, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab1189a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391189d, 34 },
    { 0, 0 },
    { 0x2be1189f, 61 },
    { 0x27c118a0, 23 },
    { 0x97b118a0, 98 },
    { 0, 0 },
    { 0xf2a118a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec118a7, 85 },
    { 0, 0 },
    { 0xbb1118a9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d118ac, 90 },
    { 0, 0 },
    { 0x3f2118ae, 61 },
    { 0x10e118ae, 84 },
    { 0x308118b0, 33 },
    { 0xc48118b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6118b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f118bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e118c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c118c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d118c9, 88 },
    { 0x2ed118c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e118cd, 98 },
    { 0x32f118ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533118da, 78 },
    { 0x697118da, 98 },
    { 0x159118dc, 93 },
    { 0x796118dc, 33 },
    { 0xd69118de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279118e5, 55 },
    { 0xbf6118e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c118f9, 38 },
    { 0x889118f9, 52 },
    { 0xd28118f9, 61 },
    { 0xb86118fa, 33 },
    { 0, 0 },
    { 0xdad118fe, 34 },
    { 0, 0 },
    { 0x1cd11900, 46 },
    { 0, 0 },
    { 0x9b211902, 46 },
    { 0, 0 },
    { 0xecb11904, 95 },
    { 0xc6811904, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e11908, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1190d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151191c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58011920, 98 },
    { 0, 0 },
    { 0x4f811922, 90 },
    { 0x1d611923, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d11926, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2911929, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11192c, 33 },
    { 0xe251192d, 66 },
    { 0x6b71192e, 55 },
    { 0xdff1192e, 17 },
    { 0x6161192f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee611934, 43 },
    { 0xee111935, 55 },
    { 0x56411936, 93 },
    { 0, 0 },
    { 0x60311938, 61 },
    { 0, 0 },
    { 0x7df1193a, 34 },
    { 0, 0 },
    { 0x2491193c, 33 },
    { 0x4d61193d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f11940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5311943, 61 },
    { 0xc3b11944, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c1194a, 95 },
    { 0xdb71194a, 78 },
    { 0xac61194a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5581194f, 90 },
    { 0xac511950, 95 },
    { 0x77211950, 95 },
    { 0x7d611950, 52 },
    { 0x71411953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671195e, 95 },
    { 0, 0 },
    { 0xa4d11960, 33 },
    { 0x6e811961, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f11969, 61 },
    { 0, 0 },
    { 0xd211196b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1196e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f211971, 95 },
    { 0x71d11972, 52 },
    { 0xcc811973, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911976, 93 },
    { 0x1c311977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf521197b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4011984, 56 },
    { 0x14111984, 61 },
    { 0xb9811984, 95 },
    { 0xfe211984, 23 },
    { 0, 0 },
    { 0xe2111989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb831198c, 100 },
    { 0xd8f1198d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b911991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33311994, 43 },
    { 0xb4311995, 95 },
    { 0x70811995, 33 },
    { 0xaa111997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3651199b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6021199f, 93 },
    { 0, 0 },
    { 0xc86119a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8119a6, 23 },
    { 0xbfd119a6, 46 },
    { 0xf66119a7, 33 },
    { 0xb7a119a9, 61 },
    { 0x4ec119aa, 90 },
    { 0xdd119ab, 90 },
    { 0x355119ac, 34 },
    { 0xb41119ad, 33 },
    { 0xb33119aa, 76 },
    { 0xde119a9, 34 },
    { 0x6c7119b0, 100 },
    { 0x90b119b1, 61 },
    { 0, 0 },
    { 0xc74119b3, 87 },
    { 0x5e7119b4, 90 },
    { 0, 0 },
    { 0xd6b119b6, 80 },
    { 0xa45119b6, 34 },
    { 0x50f119b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12119bb, 33 },
    { 0, 0 },
    { 0xd1c119bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51119c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56119cb, 90 },
    { 0x468119cc, 90 },
    { 0, 0 },
    { 0x923119ce, 33 },
    { 0x78f119cf, 68 },
    { 0x9fc119ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x340119d3, 90 },
    { 0, 0 },
    { 0x757119d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2119d8, 88 },
    { 0xc4e119d9, 76 },
    { 0x9f5119da, 61 },
    { 0xdca119db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c119de, 40 },
    { 0, 0 },
    { 0x5fc119e0, 61 },
    { 0, 0 },
    { 0x43d119e2, 95 },
    { 0x162119e3, 90 },
    { 0, 0 },
    { 0x8d5119e5, 61 },
    { 0, 0 },
    { 0xc0c119e7, 48 },
    { 0x6fc119e8, 96 },
    { 0x80c119e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b119f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63119f4, 87 },
    { 0x94e119f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644119fa, 33 },
    { 0x357119fa, 67 },
    { 0x7a119fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0111a13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a11a16, 43 },
    { 0x2d511a17, 61 },
    { 0xc4b11a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf11a1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e11a21, 95 },
    { 0xada11a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9611a26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94211a2d, 84 },
    { 0, 0 },
    { 0x2bc11a2f, 61 },
    { 0xcb411a30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6111a33, 43 },
    { 0x94e11a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae11a37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a111a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c11a49, 34 },
    { 0x3f711a4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28611a55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89911a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29811a66, 76 },
    { 0xd811a67, 23 },
    { 0x26811a67, 78 },
    { 0x9c411a69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c11a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5911a71, 95 },
    { 0x3d811a72, 23 },
    { 0x93911a73, 98 },
    { 0, 0 },
    { 0xbb411a75, 55 },
    { 0x9cb11a75, 34 },
    { 0, 0 },
    { 0x2da11a78, 34 },
    { 0x71e11a79, 52 },
    { 0xfe211a7a, 81 },
    { 0x74111a79, 98 },
    { 0x16111a7c, 61 },
    { 0xac211a79, 95 },
    { 0x42c11a7e, 95 },
    { 0xaa511a7a, 55 },
    { 0x3b311a7f, 61 },
    { 0xcf311a7d, 95 },
    { 0xee511a7e, 68 },
    { 0xa4011a83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89311a8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac11a8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85411a95, 90 },
    { 0, 0 },
    { 0xa7011a97, 43 },
    { 0xf6711a97, 99 },
    { 0xe6e11a99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e611aa5, 33 },
    { 0xa1a11aa6, 52 },
    { 0xfd611aa7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87911aac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe11ab1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18711ab8, 33 },
    { 0, 0 },
    { 0xaea11aba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c11abe, 100 },
    { 0xdda11abe, 61 },
    { 0x9f311ac0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb11ac4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76211ac7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89411acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca11ad6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce811ade, 55 },
    { 0xc011adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e11ae6, 85 },
    { 0x75c11ae7, 100 },
    { 0x83b11ae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde711aeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4111aef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d011af2, 95 },
    { 0, 0 },
    { 0x79a11af4, 33 },
    { 0x39611af5, 93 },
    { 0x4ce11af4, 34 },
    { 0xc0e11af7, 69 },
    { 0xcae11af8, 49 },
    { 0xec511af5, 98 },
    { 0x80e11af4, 23 },
    { 0, 0 },
    { 0x77411afc, 81 },
    { 0x33a11afc, 75 },
    { 0x77a11afd, 23 },
    { 0x56911aff, 55 },
    { 0, 0 },
    { 0x63611b01, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11711b07, 95 },
    { 0, 0 },
    { 0x5bb11b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9211b0e, 98 },
    { 0x2dc11b0e, 33 },
    { 0, 0 },
    { 0xc3211b11, 61 },
    { 0x39911b11, 61 },
    { 0x8b411b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d11b16, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b011b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a11b20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c11b24, 40 },
    { 0, 0 },
    { 0xfdd11b26, 98 },
    { 0x30211b27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc11b2a, 43 },
    { 0, 0 },
    { 0x79911b2c, 100 },
    { 0, 0 },
    { 0xcb311b2e, 90 },
    { 0xae711b2e, 78 },
    { 0xb2911b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef11b33, 90 },
    { 0x49611b34, 90 },
    { 0x3ce11b35, 98 },
    { 0x87d11b34, 23 },
    { 0xf1a11b37, 52 },
    { 0x7d611b36, 85 },
    { 0xd5411b36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a211b3d, 88 },
    { 0x3411b3e, 93 },
    { 0x95211b3e, 33 },
    { 0x8ee11b3d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65311b44, 100 },
    { 0, 0 },
    { 0x16311b46, 98 },
    { 0, 0 },
    { 0xe4c11b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f511b52, 66 },
    { 0xcfb11b52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a11b58, 34 },
    { 0, 0 },
    { 0x8e111b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e11b65, 33 },
    { 0, 0 },
    { 0x6c111b67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8311b6b, 55 },
    { 0x1fb11b6c, 33 },
    { 0x9db11b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50911b71, 75 },
    { 0, 0 },
    { 0x2d411b73, 33 },
    { 0, 0 },
    { 0xcd711b75, 34 },
    { 0xff11b76, 90 },
    { 0x6de11b76, 34 },
    { 0x49011b78, 61 },
    { 0xc9f11b77, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89711b7d, 46 },
    { 0x99511b7e, 99 },
    { 0, 0 },
    { 0xff311b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf611b83, 65 },
    { 0xdc811b84, 93 },
    { 0x2a611b85, 61 },
    { 0x5c611b86, 58 },
    { 0xcc611b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee411b8a, 62 },
    { 0, 0 },
    { 0xc0d11b8c, 45 },
    { 0, 0 },
    { 0x4f511b8e, 95 },
    { 0xf4611b8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d011b92, 98 },
    { 0xcf511b92, 91 },
    { 0xd8311b94, 77 },
    { 0xa8511b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a11b9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c911ba1, 46 },
    { 0, 0 },
    { 0xd711ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66711ba8, 90 },
    { 0x5211ba8, 33 },
    { 0, 0 },
    { 0xc7111bab, 78 },
    { 0x38511bab, 95 },
    { 0, 0 },
    { 0xae211bae, 76 },
    { 0x17e11baf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5311bb3, 93 },
    { 0x15611bb4, 67 },
    { 0xdd611bb5, 93 },
    { 0xbd011bb6, 66 },
    { 0x8a111bb7, 90 },
    { 0x20e11bb4, 33 },
    { 0x68a11bb9, 99 },
    { 0xfbc11bba, 95 },
    { 0xbf511bbb, 95 },
    { 0x7111bbc, 33 },
    { 0x5e311bbc, 95 },
    { 0x40f11bbe, 90 },
    { 0xf9511bbf, 55 },
    { 0xab711bbb, 33 },
    { 0xb1711bc1, 23 },
    { 0x65111bc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe311bca, 40 },
    { 0xc3811bcb, 23 },
    { 0x9db11bcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d611bcf, 34 },
    { 0x4311bd0, 99 },
    { 0x41611bd0, 76 },
    { 0xbef11bd2, 33 },
    { 0x9e411bd2, 78 },
    { 0xc8311bd4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30611bd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19911bdd, 90 },
    { 0x6a411bde, 55 },
    { 0x9ef11bdf, 33 },
    { 0x7b811bde, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac11be5, 23 },
    { 0x36811be6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c311beb, 87 },
    { 0x82e11bec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b11bef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e11bf2, 98 },
    { 0x12711bf3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9911bfa, 40 },
    { 0xdf011bfb, 87 },
    { 0, 0 },
    { 0xf6611bfd, 34 },
    { 0xb3a11bfe, 33 },
    { 0xebc11bff, 75 },
    { 0xfc411bfd, 98 },
    { 0xc5811bff, 100 },
    { 0xdb711bfe, 90 },
    { 0, 0 },
    { 0xe6111c04, 76 },
    { 0x3311c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1411c08, 98 },
    { 0x81b11c08, 90 },
    { 0x82b11c0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59111c0d, 90 },
    { 0xa1311c0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6811c11, 95 },
    { 0xfc111c12, 74 },
    { 0xb7811c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54111c16, 99 },
    { 0, 0 },
    { 0x4f611c18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce411c1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f11c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0511c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e11c32, 33 },
    { 0xa5c11c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e11c38, 61 },
    { 0xc7611c38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c11c3d, 43 },
    { 0xdb111c3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff911c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7011c47, 65 },
    { 0x5d411c48, 23 },
    { 0x45411c49, 90 },
    { 0xfb911c4a, 23 },
    { 0, 0 },
    { 0xb7511c4c, 98 },
    { 0x8df11c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe11c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d11c59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc711c5c, 90 },
    { 0xd8511c5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e11c60, 76 },
    { 0x80e11c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5811c67, 95 },
    { 0, 0 },
    { 0x41a11c69, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a611c6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f211c74, 55 },
    { 0xf4211c75, 33 },
    { 0xe411c76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2211c79, 76 },
    { 0x29711c7a, 91 },
    { 0xb2611c7b, 76 },
    { 0x3cb11c7b, 33 },
    { 0xc6511c7a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60811c8c, 43 },
    { 0x82911c8d, 90 },
    { 0x2f911c8e, 61 },
    { 0x7b11c8f, 100 },
    { 0x2ec11c90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a811c93, 61 },
    { 0, 0 },
    { 0x911c95, 33 },
    { 0xd7511c96, 61 },
    { 0x38e11c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93811c9b, 99 },
    { 0xa7111c9c, 61 },
    { 0x4911c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe11ca6, 87 },
    { 0x51911ca7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd11caa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e111cae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63711cc2, 90 },
    { 0xa0c11cc3, 78 },
    { 0x92711cc2, 98 },
    { 0xbcb11cc3, 33 },
    { 0, 0 },
    { 0x4c011cc7, 76 },
    { 0xb9211cc8, 76 },
    { 0xed111cc7, 55 },
    { 0xfc211cc9, 33 },
    { 0xc1011cc7, 78 },
    { 0xd2811ccc, 90 },
    { 0xee711ccd, 76 },
    { 0xf0411ccb, 55 },
    { 0xbdb11cc7, 33 },
    { 0x17e11ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38411cd7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7511cda, 98 },
    { 0xa1611cda, 52 },
    { 0, 0 },
    { 0x30511cdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7911ce0, 33 },
    { 0, 0 },
    { 0x18811ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0411ce9, 61 },
    { 0x49f11cea, 76 },
    { 0x4f11ce9, 61 },
    { 0xd0b11cec, 33 },
    { 0xb0311ced, 61 },
    { 0xd8611cec, 90 },
    { 0xd2e11cec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a611cf2, 90 },
    { 0x68c11cf3, 90 },
    { 0x34711cf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8511cf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f11cfa, 23 },
    { 0x22411cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99011d03, 61 },
    { 0, 0 },
    { 0x5db11d05, 42 },
    { 0xa8711d06, 52 },
    { 0, 0 },
    { 0xbd611d08, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1511d12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23811d15, 33 },
    { 0, 0 },
    { 0xce511d17, 33 },
    { 0, 0 },
    { 0xe0a11d19, 23 },
    { 0x5d011d1a, 87 },
    { 0xae411d1b, 46 },
    { 0x34211d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53611d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e11d2c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe11d34, 55 },
    { 0xa4911d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb11d3b, 95 },
    { 0x2e11d3c, 55 },
    { 0x69811d3c, 90 },
    { 0, 0 },
    { 0xd8411d3f, 98 },
    { 0, 0 },
    { 0xcf211d41, 98 },
    { 0x14811d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28711d4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea11d52, 41 },
    { 0xf0411d53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f11d57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6811d5a, 77 },
    { 0x7a011d5b, 23 },
    { 0, 0 },
    { 0xd5c11d5d, 23 },
    { 0x6c311d5e, 90 },
    { 0x47011d5f, 84 },
    { 0xdb711d60, 76 },
    { 0xcdc11d61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22911d64, 43 },
    { 0, 0 },
    { 0xe1911d66, 78 },
    { 0x9d911d66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd11d6b, 91 },
    { 0x43511d6c, 46 },
    { 0x77211d6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b11d70, 95 },
    { 0x79a11d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e11d74, 98 },
    { 0x98011d75, 33 },
    { 0xe7811d76, 84 },
    { 0, 0 },
    { 0xf2011d78, 61 },
    { 0, 0 },
    { 0x33f11d7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa611d7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf11d88, 23 },
    { 0x50111d89, 33 },
    { 0, 0 },
    { 0xba611d8b, 23 },
    { 0x80811d8c, 61 },
    { 0x2311d8d, 90 },
    { 0x50411d8e, 55 },
    { 0x91111d8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a11d92, 99 },
    { 0xc2711d93, 76 },
    { 0x7dd11d92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7611d9a, 90 },
    { 0x4a011d9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb711da2, 95 },
    { 0xc3011da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9011da7, 66 },
    { 0, 0 },
    { 0x42b11da9, 90 },
    { 0x91411da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e211dae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d11db2, 76 },
    { 0x42e11db3, 100 },
    { 0, 0 },
    { 0xef811db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33411db8, 33 },
    { 0xe1f11db9, 98 },
    { 0x44c11dba, 34 },
    { 0x9d911dbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f11dbe, 43 },
    { 0x95511dbf, 33 },
    { 0xb7111dc0, 66 },
    { 0, 0 },
    { 0xffb11dc2, 95 },
    { 0, 0 },
    { 0xe7311dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e11dc7, 84 },
    { 0x5bb11dc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b11dce, 98 },
    { 0x34d11dcf, 52 },
    { 0xad811dcf, 95 },
    { 0xfc711dce, 87 },
    { 0xe5a11dcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3411dd5, 90 },
    { 0xa6c11dd6, 90 },
    { 0xa5111dd5, 95 },
    { 0xee011dd8, 90 },
    { 0x93311dd5, 95 },
    { 0, 0 },
    { 0x71f11ddb, 75 },
    { 0, 0 },
    { 0x2a11ddd, 90 },
    { 0xbcd11dde, 23 },
    { 0, 0 },
    { 0x36711de0, 33 },
    { 0x3dc11de1, 95 },
    { 0xed211de2, 95 },
    { 0x2a011de3, 93 },
    { 0, 0 },
    { 0xe9311de5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe911de9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b11dec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x34811def, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e11dfa, 90 },
    { 0x93b11dfb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73411e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22811e09, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb911e0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78211e11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d311e16, 88 },
    { 0x73b11e17, 35 },
    { 0xf8111e17, 23 },
    { 0x52611e18, 33 },
    { 0x24011e19, 61 },
    { 0x6211e17, 22 },
    { 0xbc11e18, 95 },
    { 0xca111e1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3611e21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86811e26, 98 },
    { 0xa4911e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18511e2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f11e30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16011e33, 55 },
    { 0xf6411e34, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c711e38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87611e3b, 93 },
    { 0, 0 },
    { 0xb9111e3d, 33 },
    { 0xb7711e3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e511e42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd11e4a, 56 },
    { 0xacc11e4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d11e4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc311e52, 46 },
    { 0, 0 },
    { 0xd5a11e54, 93 },
    { 0xca411e55, 61 },
    { 0, 0 },
    { 0xc4f11e57, 19 },
    { 0, 0 },
    { 0xd6611e59, 90 },
    { 0, 0 },
    { 0x7ff11e5b, 95 },
    { 0, 0 },
    { 0x94a11e5d, 33 },
    { 0xc6e11e5e, 66 },
    { 0, 0 },
    { 0x1f111e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a11e67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a611e6f, 33 },
    { 0, 0 },
    { 0x93811e71, 33 },
    { 0, 0 },
    { 0xce711e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9211e77, 76 },
    { 0xacb11e77, 98 },
    { 0x9fd11e78, 46 },
    { 0x4a411e7a, 33 },
    { 0x82011e7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16511e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5211e82, 55 },
    { 0, 0 },
    { 0x94e11e84, 46 },
    { 0x5b311e85, 100 },
    { 0x2711e86, 98 },
    { 0x19611e87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3411e8b, 90 },
    { 0x4ee11e8b, 43 },
    { 0x71a11e8d, 23 },
    { 0x1ea11e8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c11e91, 61 },
    { 0xb1111e91, 61 },
    { 0xaa611e93, 90 },
    { 0x57911e94, 43 },
    { 0xc1111e94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19811e9b, 90 },
    { 0, 0 },
    { 0xb4611e9d, 85 },
    { 0xb2511e9e, 76 },
    { 0xfea11e9f, 81 },
    { 0x2a11ea0, 34 },
    { 0x3bd11ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44411ea5, 34 },
    { 0x1ef11ea5, 55 },
    { 0x41111ea6, 76 },
    { 0xc411ea6, 90 },
    { 0x6f211ea7, 66 },
    { 0xa5811ea9, 88 },
    { 0x8d411eab, 43 },
    { 0xd9011eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac611eaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6111eb3, 76 },
    { 0, 0 },
    { 0xd6b11eb5, 33 },
    { 0x4d111eb5, 90 },
    { 0x15011eb7, 90 },
    { 0xf0811eb6, 98 },
    { 0, 0 },
    { 0x44111eba, 61 },
    { 0xf1311eba, 76 },
    { 0x1de11ebc, 90 },
    { 0x18011ebd, 34 },
    { 0x89411ebe, 84 },
    { 0xad311ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611ec7, 74 },
    { 0, 0 },
    { 0x92a11ec9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd911ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce11ed8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b11edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c711ee2, 33 },
    { 0x3e611ee3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e611ee9, 75 },
    { 0x4c411eea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c11eee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63211ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83011ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc11efb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f11efe, 55 },
    { 0x93c11efe, 95 },
    { 0x46611f00, 76 },
    { 0, 0 },
    { 0x84d11f02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff11f05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54611f0b, 34 },
    { 0x64211f0c, 98 },
    { 0x9ab11f0d, 61 },
    { 0, 0 },
    { 0xb3611f0f, 95 },
    { 0, 0 },
    { 0xe4711f11, 53 },
    { 0x10b11f12, 43 },
    { 0x77711f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13311f16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e511f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c711f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63211f2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7111f31, 90 },
    { 0, 0 },
    { 0xdaa11f33, 61 },
    { 0x82311f34, 77 },
    { 0x3c11f33, 99 },
    { 0, 0 },
    { 0xbf711f37, 93 },
    { 0x6011f37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a211f3f, 25 },
    { 0, 0 },
    { 0x7d711f41, 90 },
    { 0x66811f41, 66 },
    { 0, 0 },
    { 0x51711f44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf111f47, 90 },
    { 0x60e11f48, 61 },
    { 0x73c11f49, 100 },
    { 0xe0f11f49, 46 },
    { 0, 0 },
    { 0x11911f4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a11f4f, 90 },
    { 0, 0 },
    { 0x35911f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27211f54, 34 },
    { 0x25211f55, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x80311f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94811f5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a11f5f, 90 },
    { 0x64b11f60, 95 },
    { 0xd1911f60, 90 },
    { 0xf6b11f62, 33 },
    { 0x4ed11f62, 90 },
    { 0xf7f11f64, 23 },
    { 0x45011f65, 76 },
    { 0x22711f65, 59 },
    { 0xc7511f65, 95 },
    { 0x7cc11f67, 55 },
    { 0xfb111f69, 76 },
    { 0x5e611f69, 61 },
    { 0x19711f6b, 61 },
    { 0xa8811f5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78311f71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7211f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f11f79, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11f7f, 90 },
    { 0, 0 },
    { 0x9f411f81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a211f87, 33 },
    { 0, 0 },
    { 0x27f11f89, 64 },
    { 0x46f11f8a, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3311f8e, 76 },
    { 0xebc11f8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb11f94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff811f97, 90 },
    { 0xad011f98, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48811f9e, 61 },
    { 0xece11f9f, 100 },
    { 0x6f011f9f, 46 },
    { 0x78811fa1, 33 },
    { 0xbee11fa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b711fa7, 98 },
    { 0x50e11fa8, 33 },
    { 0, 0 },
    { 0xce511faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab11fb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e411fb5, 76 },
    { 0, 0 },
    { 0x20211fb7, 93 },
    { 0x8c911fb8, 33 },
    { 0x38011fb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88011fbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6211fc6, 78 },
    { 0, 0 },
    { 0x89a11fc8, 33 },
    { 0, 0 },
    { 0x94111fca, 23 },
    { 0x4bf11fcb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d411fd0, 61 },
    { 0x30611fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd11fd4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35211fdd, 33 },
    { 0, 0 },
    { 0xda211fdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5611fe2, 61 },
    { 0x52a11fe3, 33 },
    { 0x84811fe4, 93 },
    { 0x8311fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b211fe9, 23 },
    { 0x2e411fea, 64 },
    { 0x68e11fe9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52411fef, 95 },
    { 0, 0 },
    { 0xbe211ff1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7311ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c12002, 98 },
    { 0xa7b12002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81512007, 40 },
    { 0x42912008, 99 },
    { 0, 0 },
    { 0xb061200a, 99 },
    { 0, 0 },
    { 0x85a1200c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171200f, 23 },
    { 0x6b312010, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d12013, 76 },
    { 0xe812014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba12017, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6931201a, 34 },
    { 0x53f1201a, 84 },
    { 0x1f61201c, 8 },
    { 0, 0 },
    { 0x1de1201e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2712026, 88 },
    { 0xc9412027, 23 },
    { 0x96912028, 46 },
    { 0x5d112029, 43 },
    { 0xcc61202a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad1202e, 93 },
    { 0x1f1202f, 23 },
    { 0x4b512030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa712034, 90 },
    { 0, 0 },
    { 0xa2112036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc512041, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b812046, 33 },
    { 0x7be12047, 95 },
    { 0x4cd12046, 84 },
    { 0x6b112048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21204f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc612052, 90 },
    { 0, 0 },
    { 0xfaf12054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d01205d, 61 },
    { 0xd801205e, 23 },
    { 0, 0 },
    { 0x8aa12060, 55 },
    { 0xe5412061, 33 },
    { 0x26012060, 33 },
    { 0, 0 },
    { 0xaa512064, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8051206a, 100 },
    { 0x94d1206b, 98 },
    { 0x3311206a, 61 },
    { 0, 0 },
    { 0xafa1206e, 46 },
    { 0x4af1206e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c212073, 33 },
    { 0xced12073, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e12079, 43 },
    { 0x6fa1207a, 34 },
    { 0xbbc1207b, 90 },
    { 0xd451207b, 98 },
    { 0xeab1207c, 67 },
    { 0x2161207b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b712081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e912084, 90 },
    { 0xfc12084, 46 },
    { 0x3d712085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21208b, 52 },
    { 0x5981208c, 61 },
    { 0x5281208d, 98 },
    { 0x20d1208d, 24 },
    { 0x1051208d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a12093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a12098, 90 },
    { 0xf3012098, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371209d, 23 },
    { 0x2591209e, 93 },
    { 0, 0 },
    { 0x428120a0, 52 },
    { 0, 0 },
    { 0x60d120a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e120a7, 61 },
    { 0, 0 },
    { 0xfc3120a9, 61 },
    { 0, 0 },
    { 0x411120ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185120b1, 78 },
    { 0xa62120b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839120b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983120bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63120c1, 23 },
    { 0, 0 },
    { 0x7b0120c3, 90 },
    { 0x6f1120c4, 85 },
    { 0xf3f120c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d120c8, 88 },
    { 0xf49120c9, 87 },
    { 0x237120c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d120d5, 33 },
    { 0x83c120d5, 33 },
    { 0xa1d120d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94120da, 33 },
    { 0, 0 },
    { 0x60a120dc, 91 },
    { 0x557120dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290120e1, 46 },
    { 0x2e2120e2, 76 },
    { 0, 0 },
    { 0x1d1120e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x301120e7, 75 },
    { 0, 0 },
    { 0x3cc120e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9120ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf120ef, 23 },
    { 0x39c120f0, 19 },
    { 0x645120f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf120f8, 34 },
    { 0x710120f9, 100 },
    { 0x32d120fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4120ff, 53 },
    { 0xc8512100, 33 },
    { 0xb54120ff, 87 },
    { 0, 0 },
    { 0x26a12103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f61210b, 34 },
    { 0x9dd1210c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df12110, 23 },
    { 0, 0 },
    { 0x3f512112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b12117, 23 },
    { 0xef512118, 61 },
    { 0x7712118, 34 },
    { 0x8bb1211a, 33 },
    { 0x38612119, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8851211e, 55 },
    { 0xb601211f, 93 },
    { 0, 0 },
    { 0x71d12121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1212d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6812132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7612137, 34 },
    { 0, 0 },
    { 0xf9f12139, 34 },
    { 0, 0 },
    { 0xd401213b, 95 },
    { 0xe8a1213b, 90 },
    { 0x9fa1213b, 90 },
    { 0xb4a1213b, 84 },
    { 0, 0 },
    { 0x7db12140, 61 },
    { 0xc3712141, 52 },
    { 0x46312141, 46 },
    { 0x7d812143, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e12146, 85 },
    { 0x5c912146, 93 },
    { 0x20d12147, 101 },
    { 0, 0 },
    { 0xf7c1214a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f1214e, 93 },
    { 0xc281214f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c12152, 95 },
    { 0xca712153, 93 },
    { 0x4db12153, 87 },
    { 0, 0 },
    { 0x20a12156, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e1215c, 76 },
    { 0x3291215d, 23 },
    { 0, 0 },
    { 0xe121215f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7212164, 90 },
    { 0, 0 },
    { 0xc9112166, 33 },
    { 0, 0 },
    { 0xb2512168, 100 },
    { 0x30012168, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b1216d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33612173, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88512177, 77 },
    { 0, 0 },
    { 0x65412179, 95 },
    { 0, 0 },
    { 0x2331217b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5921217e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe012181, 95 },
    { 0, 0 },
    { 0xfc12183, 84 },
    { 0x81012184, 85 },
    { 0, 0 },
    { 0x73e12186, 24 },
    { 0x20612187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1218c, 76 },
    { 0x3671218d, 90 },
    { 0x95f1218e, 33 },
    { 0x23b1218f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1712195, 43 },
    { 0, 0 },
    { 0x87c12197, 43 },
    { 0x5d012197, 23 },
    { 0x9f212197, 55 },
    { 0x9111219a, 55 },
    { 0x9811219b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20121a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991121a6, 34 },
    { 0x98a121a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4121ab, 90 },
    { 0, 0 },
    { 0x95f121ad, 23 },
    { 0x9cb121ad, 90 },
    { 0x2ab121ad, 75 },
    { 0, 0 },
    { 0xdfa121b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd121b7, 34 },
    { 0x72f121b8, 95 },
    { 0x138121b8, 46 },
    { 0x6c5121ba, 28 },
    { 0xe55121b9, 93 },
    { 0xe0a121bc, 34 },
    { 0x641121bd, 23 },
    { 0x249121b9, 95 },
    { 0x533121bb, 90 },
    { 0x268121c0, 23 },
    { 0x17121c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x601121c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d121c7, 99 },
    { 0x392121c7, 76 },
    { 0, 0 },
    { 0x201121ca, 95 },
    { 0, 0 },
    { 0x149121cc, 34 },
    { 0x51f121cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e121d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3121dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b121e3, 90 },
    { 0, 0 },
    { 0xa92121e5, 61 },
    { 0, 0 },
    { 0xaa7121e7, 2 },
    { 0x886121e7, 55 },
    { 0xf1c121e9, 91 },
    { 0xf27121ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824121ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c121f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1121f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f121fc, 93 },
    { 0, 0 },
    { 0xc72121fe, 61 },
    { 0xf9c121ff, 90 },
    { 0x397121ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae612205, 61 },
    { 0x26212206, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b1220a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4801220e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56312212, 90 },
    { 0x16c12213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87112219, 90 },
    { 0xaee1221a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6912220, 33 },
    { 0xe4412220, 76 },
    { 0xf1b12221, 87 },
    { 0xf9112223, 84 },
    { 0, 0 },
    { 0xab312225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b12228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83612231, 55 },
    { 0xbf412231, 93 },
    { 0x58812233, 23 },
    { 0xb6612234, 90 },
    { 0x27312235, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe971223a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5061223d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23912240, 52 },
    { 0x8c312241, 75 },
    { 0xd4912242, 98 },
    { 0xff412243, 34 },
    { 0x4b412244, 61 },
    { 0x5f612244, 76 },
    { 0xf3f12246, 90 },
    { 0, 0 },
    { 0x7b612248, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c1224b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75212250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa012257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e012262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b12265, 66 },
    { 0xa6012265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea1226c, 90 },
    { 0xdc11226d, 90 },
    { 0x7271226c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e12273, 33 },
    { 0, 0 },
    { 0xfd912275, 90 },
    { 0, 0 },
    { 0x62412277, 52 },
    { 0, 0 },
    { 0xa1e12279, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd91227d, 61 },
    { 0, 0 },
    { 0x2541227f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b812283, 98 },
    { 0, 0 },
    { 0x78d12285, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af1228b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af12294, 66 },
    { 0x7d412295, 94 },
    { 0xbb212296, 33 },
    { 0x9b412297, 64 },
    { 0x10f12298, 61 },
    { 0x45712297, 83 },
    { 0x4821229a, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f1229e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8122a2, 90 },
    { 0xe30122a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a122a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d122a9, 23 },
    { 0x4e8122a9, 90 },
    { 0, 0 },
    { 0x550122ac, 64 },
    { 0, 0 },
    { 0x2f9122ae, 81 },
    { 0x2fa122af, 55 },
    { 0x5e7122b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891122b5, 23 },
    { 0x822122b6, 61 },
    { 0x43b122b7, 23 },
    { 0xc85122b8, 98 },
    { 0, 0 },
    { 0xc3e122ba, 95 },
    { 0xcf1122bb, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f122bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5122c3, 98 },
    { 0xd87122c4, 78 },
    { 0x444122c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f122c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66122cc, 95 },
    { 0x688122cd, 87 },
    { 0x330122ce, 65 },
    { 0xb68122ce, 93 },
    { 0x4d9122d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c122d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f122e2, 95 },
    { 0x6aa122e3, 23 },
    { 0, 0 },
    { 0x993122e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28122ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb122ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72122f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af122f5, 95 },
    { 0, 0 },
    { 0x6cb122f7, 85 },
    { 0, 0 },
    { 0xc14122f9, 61 },
    { 0, 0 },
    { 0xe1c122fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a712304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75312307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2451230a, 34 },
    { 0xf611230a, 95 },
    { 0x31e1230c, 43 },
    { 0, 0 },
    { 0x5311230e, 61 },
    { 0x92b1230e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b12314, 76 },
    { 0x81a12315, 88 },
    { 0, 0 },
    { 0x93f12317, 61 },
    { 0, 0 },
    { 0xbee12319, 61 },
    { 0xc941231a, 76 },
    { 0x4cc1231b, 33 },
    { 0xdf512319, 34 },
    { 0x3451231d, 90 },
    { 0x311231a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35412321, 61 },
    { 0xf9112322, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1232a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5431232d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e12331, 38 },
    { 0, 0 },
    { 0xcd512333, 95 },
    { 0x26612334, 80 },
    { 0xf5d12335, 98 },
    { 0xbc112335, 69 },
    { 0x2ad12333, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c1233b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb12345, 43 },
    { 0, 0 },
    { 0x7d412347, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b1234a, 23 },
    { 0xda41234b, 76 },
    { 0xafa1234a, 75 },
    { 0, 0 },
    { 0xefd1234e, 23 },
    { 0xf951234e, 61 },
    { 0x1234e, 34 },
    { 0xcb312351, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2512354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e812357, 34 },
    { 0, 0 },
    { 0x85f12359, 85 },
    { 0, 0 },
    { 0xf51235b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8312361, 90 },
    { 0, 0 },
    { 0xbd12363, 23 },
    { 0xdac12363, 61 },
    { 0x7f712363, 55 },
    { 0xdcd12366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7812369, 91 },
    { 0, 0 },
    { 0xa641236b, 91 },
    { 0x7b21236c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21812370, 61 },
    { 0, 0 },
    { 0xa7812372, 76 },
    { 0, 0 },
    { 0x9f112374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2991237a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6141237d, 61 },
    { 0, 0 },
    { 0x2e51237f, 95 },
    { 0, 0 },
    { 0xf2112381, 43 },
    { 0, 0 },
    { 0xe7912383, 76 },
    { 0x8ff12384, 95 },
    { 0xcac12385, 90 },
    { 0x54512386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1238c, 98 },
    { 0x2e1238c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67112393, 19 },
    { 0, 0 },
    { 0xe2d12395, 52 },
    { 0, 0 },
    { 0x24412397, 91 },
    { 0, 0 },
    { 0x23d12399, 100 },
    { 0xde81239a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb41239f, 55 },
    { 0xd8a123a0, 85 },
    { 0x637123a1, 100 },
    { 0xe7b123a0, 23 },
    { 0xadb123a0, 66 },
    { 0x8d9123a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92123a7, 23 },
    { 0xc62123a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1123ab, 48 },
    { 0x354123ab, 61 },
    { 0x2d3123ad, 67 },
    { 0xc77123ad, 76 },
    { 0x5b4123ac, 98 },
    { 0x1f0123af, 34 },
    { 0, 0 },
    { 0x397123b2, 90 },
    { 0xd38123b3, 90 },
    { 0xf8f123b4, 87 },
    { 0, 0 },
    { 0xb3a123b6, 52 },
    { 0xfeb123b7, 85 },
    { 0xa07123b8, 84 },
    { 0xaa123b7, 78 },
    { 0x577123ba, 61 },
    { 0xe4123bb, 90 },
    { 0, 0 },
    { 0x2fa123bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1123c7, 76 },
    { 0, 0 },
    { 0x415123c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87123cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a123d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0123d4, 67 },
    { 0x71a123d5, 55 },
    { 0x914123d6, 61 },
    { 0xb95123d5, 33 },
    { 0xb06123d4, 76 },
    { 0, 0 },
    { 0x296123da, 23 },
    { 0xd96123db, 33 },
    { 0x9ef123dc, 61 },
    { 0, 0 },
    { 0x1c3123de, 66 },
    { 0, 0 },
    { 0x870123e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd123e3, 85 },
    { 0x352123e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa123e7, 90 },
    { 0xe9b123e7, 43 },
    { 0, 0 },
    { 0x75123ea, 87 },
    { 0x5f0123eb, 66 },
    { 0xe26123eb, 90 },
    { 0, 0 },
    { 0xb6e123ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb123f1, 90 },
    { 0x146123f2, 90 },
    { 0xb25123f1, 26 },
    { 0xec1123f4, 75 },
    { 0x17a123f1, 61 },
    { 0xf81123f6, 93 },
    { 0x945123f7, 95 },
    { 0x32d123f3, 66 },
    { 0, 0 },
    { 0x54123fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xca123fd, 23 },
    { 0x71c123fd, 46 },
    { 0x1f1123ff, 98 },
    { 0x9ca12400, 26 },
    { 0, 0 },
    { 0xa0e12402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f12405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e12409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c1240c, 95 },
    { 0, 0 },
    { 0xf5c1240e, 88 },
    { 0, 0 },
    { 0xc1812410, 33 },
    { 0xa4e12411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2412417, 90 },
    { 0, 0 },
    { 0x29812419, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a1241c, 61 },
    { 0x89b1241c, 93 },
    { 0x2551241c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce12424, 97 },
    { 0, 0 },
    { 0xf6212426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a91242a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee91242d, 23 },
    { 0xaec1242d, 33 },
    { 0xf2d1242e, 43 },
    { 0x5d012430, 77 },
    { 0x92c12430, 98 },
    { 0x7eb12432, 90 },
    { 0x2c012432, 23 },
    { 0x62912434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1243a, 85 },
    { 0xdba1243b, 90 },
    { 0x9cf1243c, 76 },
    { 0x3b71243b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb412440, 87 },
    { 0x2d412441, 61 },
    { 0, 0 },
    { 0xce612443, 90 },
    { 0xbb512444, 77 },
    { 0x6bf12445, 98 },
    { 0x2a812446, 61 },
    { 0x48312447, 46 },
    { 0, 0 },
    { 0x72612449, 61 },
    { 0x7b512449, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b41244e, 76 },
    { 0xd5f1244e, 90 },
    { 0, 0 },
    { 0xf9112451, 93 },
    { 0x91f12452, 43 },
    { 0xbd312453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c12459, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c12466, 84 },
    { 0x4ae12467, 59 },
    { 0x91f12466, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xede1246b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35112475, 91 },
    { 0, 0 },
    { 0x27212477, 61 },
    { 0, 0 },
    { 0xb0512479, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71248b, 95 },
    { 0, 0 },
    { 0xe621248d, 98 },
    { 0xebd1248d, 23 },
    { 0x8381248d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e12495, 33 },
    { 0xc4012496, 33 },
    { 0x7d512497, 33 },
    { 0x31312498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a1249b, 99 },
    { 0, 0 },
    { 0x7c41249d, 33 },
    { 0x7e61249e, 61 },
    { 0xae21249f, 33 },
    { 0xa9c1249d, 34 },
    { 0x3df124a1, 33 },
    { 0x92e124a2, 90 },
    { 0xb42124a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde124ab, 23 },
    { 0x426124ac, 33 },
    { 0xb14124ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204124b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a124bb, 46 },
    { 0xc15124bc, 61 },
    { 0, 0 },
    { 0x64a124be, 33 },
    { 0x33f124bf, 90 },
    { 0xeb7124c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708124c3, 95 },
    { 0xc2f124c4, 75 },
    { 0xa87124c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c124c9, 98 },
    { 0xfe3124ca, 33 },
    { 0, 0 },
    { 0xb04124cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a124d2, 17 },
    { 0x996124d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f124d6, 78 },
    { 0x3db124d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739124de, 100 },
    { 0, 0 },
    { 0x8b2124e0, 33 },
    { 0xe90124e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551124e5, 34 },
    { 0xf70124e6, 55 },
    { 0x6fa124e5, 23 },
    { 0x7e3124e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770124ec, 23 },
    { 0x80d124ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394124f5, 90 },
    { 0xd7e124f6, 95 },
    { 0x82d124f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f412501, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x57012504, 55 },
    { 0, 0 },
    { 0xcc12506, 33 },
    { 0xbf912506, 23 },
    { 0xbf012506, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661250b, 93 },
    { 0x1081250c, 55 },
    { 0xfe51250b, 90 },
    { 0xc8a1250e, 55 },
    { 0xe1a1250d, 96 },
    { 0x6151250b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3412513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7212517, 61 },
    { 0xa7412517, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1251c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf751251f, 34 },
    { 0x4ce12520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c21252a, 61 },
    { 0x49a1252b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d12530, 43 },
    { 0x81812530, 94 },
    { 0x51d12530, 23 },
    { 0x62312532, 23 },
    { 0x8ea12530, 98 },
    { 0x71512533, 34 },
    { 0xcc012530, 90 },
    { 0x2d912537, 66 },
    { 0x39512531, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8181253b, 83 },
    { 0, 0 },
    { 0x1361253d, 92 },
    { 0x23c1253e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b12546, 48 },
    { 0x2e012547, 93 },
    { 0, 0 },
    { 0x36212549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961254c, 95 },
    { 0, 0 },
    { 0x64c1254e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7312551, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd712558, 17 },
    { 0x2f112559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1255f, 81 },
    { 0x17f1255f, 43 },
    { 0, 0 },
    { 0xca912562, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9471256a, 34 },
    { 0x1d21256b, 66 },
    { 0, 0 },
    { 0xbb21256d, 23 },
    { 0, 0 },
    { 0x38f1256f, 55 },
    { 0x28812570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7212576, 90 },
    { 0, 0 },
    { 0x31412578, 23 },
    { 0, 0 },
    { 0x4a11257a, 90 },
    { 0xe4e1257b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af1257e, 61 },
    { 0, 0 },
    { 0x75e12580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c12586, 98 },
    { 0, 0 },
    { 0x8a212588, 98 },
    { 0x6bd12588, 76 },
    { 0x30a1258a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa811258f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81312592, 23 },
    { 0x37212593, 23 },
    { 0x5ec12594, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c512597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461259d, 3 },
    { 0x8931259e, 77 },
    { 0, 0 },
    { 0x95b125a0, 95 },
    { 0, 0 },
    { 0xe8d125a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d125a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b125aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432125ae, 61 },
    { 0x230125af, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99125b8, 43 },
    { 0x1a3125b9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x135125bc, 33 },
    { 0x51b125bc, 68 },
    { 0xab0125be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2125c4, 98 },
    { 0xf125c5, 33 },
    { 0xade125c6, 23 },
    { 0x6d6125c6, 43 },
    { 0xfd6125c8, 61 },
    { 0xecd125c9, 43 },
    { 0x3ea125c5, 76 },
    { 0x5bc125c4, 66 },
    { 0x69a125c8, 66 },
    { 0xfa1125cd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836125d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd125d8, 50 },
    { 0xb0e125d9, 87 },
    { 0xe12125da, 34 },
    { 0x579125da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee125de, 52 },
    { 0, 0 },
    { 0x35e125e0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f125e4, 98 },
    { 0x153125e4, 85 },
    { 0xc73125e4, 23 },
    { 0, 0 },
    { 0x1c1125e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef125eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2125f5, 95 },
    { 0x221125f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1125fd, 34 },
    { 0x1db125fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3812608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4141260b, 99 },
    { 0xbff1260c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9212616, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x77912619, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071261d, 90 },
    { 0xc601261e, 33 },
    { 0x9011261e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27412625, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311262d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11112633, 90 },
    { 0x21212634, 61 },
    { 0x4dc12633, 90 },
    { 0xafa12635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e1263b, 91 },
    { 0x99d1263c, 33 },
    { 0xcd21263d, 46 },
    { 0xb021263e, 91 },
    { 0xb1f1263e, 87 },
    { 0xc8012640, 61 },
    { 0x4fa1263d, 84 },
    { 0xcdc12642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca12649, 33 },
    { 0x3e712649, 98 },
    { 0xd6d1264b, 55 },
    { 0, 0 },
    { 0x1691264d, 61 },
    { 0, 0 },
    { 0xdd41264f, 40 },
    { 0, 0 },
    { 0x89312651, 33 },
    { 0, 0 },
    { 0xd312653, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d012657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf1265b, 23 },
    { 0x7931265b, 23 },
    { 0, 0 },
    { 0x1891265e, 100 },
    { 0x7a31265f, 65 },
    { 0, 0 },
    { 0xdd312661, 55 },
    { 0x9fa12661, 98 },
    { 0, 0 },
    { 0x8b812664, 61 },
    { 0, 0 },
    { 0x35412666, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb1266a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a41266e, 95 },
    { 0, 0 },
    { 0x66012670, 98 },
    { 0x7dd12670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c012679, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa271267d, 55 },
    { 0x9ad1267e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb012681, 34 },
    { 0xeb912682, 98 },
    { 0xa9f12683, 88 },
    { 0x7ff12684, 52 },
    { 0x86212684, 34 },
    { 0x3f712686, 76 },
    { 0x31d12684, 66 },
    { 0xde512688, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141268c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b1268f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4612692, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e12695, 76 },
    { 0x18912696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b12699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221269e, 98 },
    { 0xf311269f, 23 },
    { 0xa42126a0, 91 },
    { 0x4cb126a1, 61 },
    { 0x241126a2, 98 },
    { 0x8dc126a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419126a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc126ad, 90 },
    { 0xec9126ae, 93 },
    { 0xed3126af, 77 },
    { 0, 0 },
    { 0x40a126b1, 61 },
    { 0, 0 },
    { 0xca7126b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6126b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd126b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba126bf, 76 },
    { 0xf27126bf, 61 },
    { 0xb87126bf, 55 },
    { 0xdb9126c2, 52 },
    { 0, 0 },
    { 0x75f126c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd126c8, 85 },
    { 0, 0 },
    { 0x87f126ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0126d4, 76 },
    { 0xd4126d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4126e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f126ea, 95 },
    { 0xd4b126ea, 98 },
    { 0x188126ec, 23 },
    { 0x2a2126ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1126f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b126f5, 33 },
    { 0, 0 },
    { 0x8ee126f7, 23 },
    { 0, 0 },
    { 0xe80126f9, 98 },
    { 0x34f126fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214126fe, 43 },
    { 0x62c126ff, 33 },
    { 0xfc9126fe, 61 },
    { 0x9f112701, 90 },
    { 0x83f12702, 53 },
    { 0x1b112701, 90 },
    { 0x79e12702, 95 },
    { 0xc7012705, 78 },
    { 0xe2112705, 33 },
    { 0x7d812707, 61 },
    { 0xcd6126fe, 23 },
    { 0, 0 },
    { 0x63d1270a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121270e, 90 },
    { 0xbea1270e, 43 },
    { 0xc651270e, 95 },
    { 0x9ad12711, 67 },
    { 0x87612711, 37 },
    { 0, 0 },
    { 0x84612714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f12717, 67 },
    { 0xfb612717, 55 },
    { 0xe012719, 81 },
    { 0xbe31271a, 95 },
    { 0xf821271b, 76 },
    { 0x25a1271c, 33 },
    { 0x8591271a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d12720, 61 },
    { 0xa0012721, 33 },
    { 0x61212722, 25 },
    { 0x22b12723, 95 },
    { 0x73912724, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe721272d, 23 },
    { 0x9491272e, 61 },
    { 0xef01272f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1512738, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e1273d, 33 },
    { 0xe471273e, 90 },
    { 0x89f1273e, 77 },
    { 0, 0 },
    { 0xa9712741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d12746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc241274c, 34 },
    { 0, 0 },
    { 0x7a91274e, 55 },
    { 0x99a1274f, 47 },
    { 0xf7d12750, 23 },
    { 0x4551274f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3612757, 90 },
    { 0x71612758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8891275c, 78 },
    { 0x981275c, 81 },
    { 0xdeb1275e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f12762, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5812765, 61 },
    { 0x2112766, 55 },
    { 0, 0 },
    { 0xd4f12768, 91 },
    { 0x47412769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191276e, 34 },
    { 0xae41276f, 90 },
    { 0xae012770, 91 },
    { 0xaab12770, 91 },
    { 0xbb912770, 98 },
    { 0x60712773, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f12779, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f12787, 55 },
    { 0x15312788, 95 },
    { 0x42d12789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd812791, 55 },
    { 0, 0 },
    { 0xe7a12793, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20212799, 90 },
    { 0x6dc1279a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2127a0, 43 },
    { 0, 0 },
    { 0x499127a2, 33 },
    { 0x230127a3, 98 },
    { 0, 0 },
    { 0x3d127a5, 65 },
    { 0x5f3127a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf127ab, 46 },
    { 0x4d9127ac, 88 },
    { 0x5a7127ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf127b1, 53 },
    { 0xf43127b2, 38 },
    { 0x477127b2, 84 },
    { 0xb44127b2, 23 },
    { 0xc8c127b2, 76 },
    { 0xc4b127b2, 100 },
    { 0, 0 },
    { 0xf05127b8, 23 },
    { 0, 0 },
    { 0x27c127ba, 23 },
    { 0, 0 },
    { 0xfcd127bc, 76 },
    { 0x423127bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15127c0, 49 },
    { 0xa54127c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2127cb, 76 },
    { 0x9c8127cc, 100 },
    { 0xac9127cd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0127d1, 61 },
    { 0x655127d1, 23 },
    { 0xaa1127d2, 84 },
    { 0xa10127d2, 95 },
    { 0x867127d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc127d9, 65 },
    { 0, 0 },
    { 0x4a7127db, 23 },
    { 0, 0 },
    { 0x51d127dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45127e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15127e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6127ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2127f1, 23 },
    { 0x651127f2, 61 },
    { 0xebb127f1, 80 },
    { 0xd2b127f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79127f7, 90 },
    { 0x3da127f8, 90 },
    { 0xcf0127f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b127fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee012803, 81 },
    { 0, 0 },
    { 0x10512805, 61 },
    { 0x3de12806, 33 },
    { 0x8c512807, 99 },
    { 0x1e212808, 61 },
    { 0x6c812808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3821280d, 90 },
    { 0, 0 },
    { 0x54e1280f, 61 },
    { 0, 0 },
    { 0x5c012811, 61 },
    { 0x3c12811, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581281a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b12825, 90 },
    { 0x17112826, 81 },
    { 0x5812827, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa891282a, 101 },
    { 0, 0 },
    { 0xec1282c, 56 },
    { 0x3891282d, 98 },
    { 0x8db1282c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c12834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee12837, 93 },
    { 0x96512838, 98 },
    { 0xf3312837, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291283e, 90 },
    { 0, 0 },
    { 0xdbb12840, 34 },
    { 0x13012840, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd312850, 95 },
    { 0x82112851, 33 },
    { 0x4f912851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6812857, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e312862, 61 },
    { 0x42312863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971286a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1791286e, 23 },
    { 0xd801286f, 43 },
    { 0xcd712870, 23 },
    { 0xbaa12871, 61 },
    { 0xec312872, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96612876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc1287e, 90 },
    { 0xe801287f, 67 },
    { 0x6d812880, 97 },
    { 0x3cc12880, 75 },
    { 0, 0 },
    { 0x6d512883, 61 },
    { 0x55012883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d12888, 23 },
    { 0, 0 },
    { 0x571288a, 55 },
    { 0, 0 },
    { 0x8f81288c, 98 },
    { 0xf41288d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x13612890, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x61412893, 52 },
    { 0xf4612893, 90 },
    { 0x57412895, 98 },
    { 0x96312896, 95 },
    { 0, 0 },
    { 0xec12898, 100 },
    { 0, 0 },
    { 0xf6c1289a, 23 },
    { 0xcfa1289b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc71289e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x891128a1, 95 },
    { 0x1dd128a1, 34 },
    { 0x1aa128a2, 90 },
    { 0x492128a1, 91 },
    { 0xdfb128a5, 90 },
    { 0x916128a1, 78 },
    { 0x563128a7, 43 },
    { 0x9ce128a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4128ab, 33 },
    { 0, 0 },
    { 0xe89128ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x623128b0, 78 },
    { 0xe65128b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3128b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b128b9, 65 },
    { 0, 0 },
    { 0x414128bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be128be, 99 },
    { 0x85128bf, 98 },
    { 0x384128c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a128c3, 66 },
    { 0xab9128c4, 33 },
    { 0xef128c3, 52 },
    { 0x9d4128c3, 34 },
    { 0, 0 },
    { 0x87b128c8, 90 },
    { 0xe1c128c9, 82 },
    { 0, 0 },
    { 0x58f128cb, 69 },
    { 0x9d128cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd128d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0128d8, 98 },
    { 0, 0 },
    { 0x9a7128da, 90 },
    { 0x9df128db, 95 },
    { 0x69c128dc, 76 },
    { 0x366128db, 98 },
    { 0x196128de, 67 },
    { 0x895128df, 61 },
    { 0x6bc128dd, 52 },
    { 0x720128e1, 33 },
    { 0xcd1128e2, 43 },
    { 0x67d128e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649128ea, 55 },
    { 0, 0 },
    { 0x95b128ec, 61 },
    { 0x76a128ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1128f0, 80 },
    { 0, 0 },
    { 0x77e128f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230128f8, 76 },
    { 0, 0 },
    { 0xccf128fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe712903, 90 },
    { 0x7c012903, 90 },
    { 0x8cc12905, 66 },
    { 0x45312906, 61 },
    { 0x48112907, 23 },
    { 0x3a812904, 95 },
    { 0, 0 },
    { 0x5dc1290a, 95 },
    { 0, 0 },
    { 0x6fc1290c, 90 },
    { 0x6481290d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32012913, 93 },
    { 0x5ba12913, 52 },
    { 0, 0 },
    { 0x69012916, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb12919, 33 },
    { 0x6c12919, 61 },
    { 0, 0 },
    { 0x3f61291c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac12925, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1012928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a41292d, 85 },
    { 0, 0 },
    { 0x6e21292f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1212933, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f412937, 85 },
    { 0x6ef12938, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a1293c, 43 },
    { 0x181293c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912941, 90 },
    { 0x18312942, 61 },
    { 0xb0712942, 75 },
    { 0, 0 },
    { 0x20c12945, 43 },
    { 0xe7b12945, 51 },
    { 0x8c912947, 65 },
    { 0xb2b12948, 88 },
    { 0xb712949, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21012954, 90 },
    { 0x7fb12954, 37 },
    { 0x20412956, 95 },
    { 0x73912955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca1295a, 17 },
    { 0x39e1295b, 90 },
    { 0x6751295b, 46 },
    { 0, 0 },
    { 0xdb21295e, 67 },
    { 0x8e51295e, 99 },
    { 0xb11295e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9112964, 61 },
    { 0xcdf12965, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b712968, 76 },
    { 0xeb312969, 46 },
    { 0x3f91296a, 33 },
    { 0xe9b12968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36012974, 90 },
    { 0x6fa12975, 61 },
    { 0xa8412976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4271297b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca12981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f12984, 90 },
    { 0, 0 },
    { 0x4b312986, 75 },
    { 0, 0 },
    { 0x39912988, 95 },
    { 0xc4712989, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7031298e, 17 },
    { 0x4b1298f, 33 },
    { 0x24a1298e, 61 },
    { 0xb8c12991, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe12994, 90 },
    { 0x84512995, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5241299d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa129a0, 52 },
    { 0x8d1129a1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c129a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a129aa, 55 },
    { 0xa11129aa, 55 },
    { 0, 0 },
    { 0x20f129ad, 33 },
    { 0x5b6129ae, 90 },
    { 0xab4129ae, 87 },
    { 0xa60129ae, 23 },
    { 0x601129af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e129b6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db129ba, 61 },
    { 0, 0 },
    { 0xa9a129bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d129bf, 56 },
    { 0xea3129c0, 76 },
    { 0x466129c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae129cb, 61 },
    { 0x48c129cb, 61 },
    { 0x232129cd, 98 },
    { 0, 0 },
    { 0x663129cf, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5129d2, 23 },
    { 0x96f129d3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7129e2, 34 },
    { 0x13c129e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a129e8, 87 },
    { 0, 0 },
    { 0xeb6129ea, 34 },
    { 0xe78129eb, 98 },
    { 0xe2b129ec, 23 },
    { 0, 0 },
    { 0xb4d129ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc129f5, 83 },
    { 0xeb6129f6, 69 },
    { 0xa62129f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5129fb, 23 },
    { 0x258129fb, 34 },
    { 0x25d129fd, 61 },
    { 0xe5129fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba12a01, 88 },
    { 0x88312a01, 99 },
    { 0x65312a03, 33 },
    { 0, 0 },
    { 0xda412a05, 93 },
    { 0x14212a05, 88 },
    { 0xd2b12a07, 81 },
    { 0x18312a08, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f12a12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17912a15, 93 },
    { 0, 0 },
    { 0xed12a17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc12a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c412a1f, 93 },
    { 0xc7d12a20, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0812a23, 61 },
    { 0xf4f12a23, 23 },
    { 0xefd12a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5612a29, 95 },
    { 0x6c712a2a, 90 },
    { 0, 0 },
    { 0xa4912a2c, 61 },
    { 0x8b212a2d, 98 },
    { 0, 0 },
    { 0x60312a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe012a33, 17 },
    { 0, 0 },
    { 0xff012a35, 67 },
    { 0x96312a36, 95 },
    { 0xde212a37, 98 },
    { 0xafd12a35, 76 },
    { 0x1fd12a39, 98 },
    { 0x3f312a36, 52 },
    { 0x23a12a3b, 33 },
    { 0x87a12a3c, 90 },
    { 0, 0 },
    { 0x11412a3e, 98 },
    { 0x94a12a3f, 33 },
    { 0, 0 },
    { 0x90512a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf12a44, 90 },
    { 0, 0 },
    { 0xc9312a46, 76 },
    { 0x14212a46, 90 },
    { 0xb9812a48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed12a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d12a4f, 23 },
    { 0xebb12a50, 33 },
    { 0xda612a4f, 33 },
    { 0x92312a52, 61 },
    { 0x51712a53, 33 },
    { 0, 0 },
    { 0x96212a55, 78 },
    { 0, 0 },
    { 0xd3f12a57, 95 },
    { 0, 0 },
    { 0xef812a59, 98 },
    { 0xd9e12a59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3412a60, 33 },
    { 0x74812a60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef12a68, 68 },
    { 0x95912a69, 23 },
    { 0, 0 },
    { 0x21e12a6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f12a6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f12a80, 61 },
    { 0, 0 },
    { 0x3d512a82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a12a8c, 85 },
    { 0x95512a8d, 85 },
    { 0x6b512a8d, 43 },
    { 0x4712a8e, 33 },
    { 0xd6312a8f, 76 },
    { 0xa7a12a8d, 93 },
    { 0, 0 },
    { 0x4b212a93, 95 },
    { 0, 0 },
    { 0x4cb12a95, 76 },
    { 0, 0 },
    { 0x42a12a97, 46 },
    { 0, 0 },
    { 0x94112a99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d812a9d, 98 },
    { 0x1e12a9e, 95 },
    { 0xfbf12a9e, 55 },
    { 0x10512aa0, 95 },
    { 0xaeb12aa0, 55 },
    { 0, 0 },
    { 0x3212aa3, 99 },
    { 0xabc12aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59412aab, 61 },
    { 0x36e12aac, 55 },
    { 0xa1112aab, 43 },
    { 0x19712aae, 53 },
    { 0x4f112aaf, 95 },
    { 0x80012ab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14812ab5, 52 },
    { 0x58a12ab6, 99 },
    { 0xffa12ab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a512ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88412ac4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6012ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6412acc, 90 },
    { 0xfce12acc, 33 },
    { 0, 0 },
    { 0xeb912acf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f12ad3, 76 },
    { 0xf7712ad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b212ad8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d12adc, 90 },
    { 0, 0 },
    { 0xb0212ade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66712ae4, 84 },
    { 0xa2412ae5, 90 },
    { 0, 0 },
    { 0xac912ae7, 33 },
    { 0x1fa12ae8, 91 },
    { 0xe4e12ae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb12aec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c12aef, 34 },
    { 0x20e12af0, 90 },
    { 0, 0 },
    { 0xdbc12af2, 90 },
    { 0x76c12af2, 93 },
    { 0x65612af4, 90 },
    { 0x1d712af5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44112af8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14112afb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb12b02, 98 },
    { 0, 0 },
    { 0x98e12b04, 55 },
    { 0x65312b05, 75 },
    { 0x35712b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86212b0c, 98 },
    { 0xeb812b0d, 61 },
    { 0xe3512b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0112b12, 40 },
    { 0x83512b13, 95 },
    { 0, 0 },
    { 0x52912b15, 33 },
    { 0xc0512b16, 48 },
    { 0x12512b17, 90 },
    { 0, 0 },
    { 0x37012b19, 55 },
    { 0x46e12b19, 98 },
    { 0x5d412b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4912b1e, 95 },
    { 0, 0 },
    { 0xf4312b20, 95 },
    { 0, 0 },
    { 0x33812b22, 90 },
    { 0xe012b23, 23 },
    { 0x47812b23, 61 },
    { 0x69212b25, 90 },
    { 0xd5f12b26, 90 },
    { 0xa0d12b27, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88112b2b, 78 },
    { 0, 0 },
    { 0xc5a12b2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9112b30, 61 },
    { 0, 0 },
    { 0x7ed12b32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a612b36, 88 },
    { 0x97212b36, 23 },
    { 0x22912b36, 90 },
    { 0x9ae12b39, 61 },
    { 0x2d512b39, 33 },
    { 0xe3c12b3b, 98 },
    { 0x73212b3a, 90 },
    { 0, 0 },
    { 0x82512b3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e12b43, 91 },
    { 0xdb12b43, 98 },
    { 0x53d12b45, 23 },
    { 0x8f412b45, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a12b57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd12b5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4212b61, 55 },
    { 0xffd12b62, 44 },
    { 0x43212b63, 23 },
    { 0xf6a12b64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c12b68, 67 },
    { 0, 0 },
    { 0xc0112b6a, 95 },
    { 0, 0 },
    { 0x8b012b6c, 61 },
    { 0x1ed12b6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9812b75, 33 },
    { 0xf4412b75, 100 },
    { 0x50312b77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b712b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b12b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16612b80, 44 },
    { 0xeff12b81, 17 },
    { 0x9fd12b82, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x51312b85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c12b8a, 95 },
    { 0x8f612b8b, 33 },
    { 0x46a12b8b, 61 },
    { 0x54a12b8d, 17 },
    { 0xcbd12b8c, 76 },
    { 0, 0 },
    { 0xf9c12b90, 100 },
    { 0, 0 },
    { 0x35112b92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d12b99, 90 },
    { 0, 0 },
    { 0x8da12b9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b12ba2, 46 },
    { 0xcb312ba2, 95 },
    { 0x60412ba4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe12ba9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35012bb1, 68 },
    { 0x94e12bb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1012bbb, 90 },
    { 0, 0 },
    { 0xf8912bbd, 90 },
    { 0x3212bbe, 98 },
    { 0x79f12bbe, 67 },
    { 0x64c12bc0, 80 },
    { 0xdd312bc1, 66 },
    { 0xa2412bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c12bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3212bce, 43 },
    { 0x49812bcf, 76 },
    { 0xf9f12bce, 90 },
    { 0xa412bd1, 78 },
    { 0x46312bd2, 98 },
    { 0xadb12bd1, 98 },
    { 0, 0 },
    { 0x73812bd5, 55 },
    { 0x89612bd5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1212bd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4712be0, 85 },
    { 0x84112be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb912be7, 43 },
    { 0, 0 },
    { 0xea112be9, 90 },
    { 0x3d712bea, 99 },
    { 0xa5212bea, 91 },
    { 0x8912bec, 95 },
    { 0xa6b12bec, 81 },
    { 0xd9012bec, 33 },
    { 0x47212bef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12012bf8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90812bfc, 23 },
    { 0xdcc12bfd, 23 },
    { 0, 0 },
    { 0x9e512bff, 93 },
    { 0, 0 },
    { 0xfdf12c01, 90 },
    { 0, 0 },
    { 0x67812c03, 33 },
    { 0, 0 },
    { 0x23712c05, 23 },
    { 0, 0 },
    { 0xb0912c07, 98 },
    { 0xeb412c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12912c12, 90 },
    { 0, 0 },
    { 0xe7612c14, 91 },
    { 0x29e12c14, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc712c19, 90 },
    { 0xb5912c1a, 33 },
    { 0, 0 },
    { 0xdd412c1c, 23 },
    { 0xba12c1d, 95 },
    { 0xbf612c1e, 43 },
    { 0x7ad12c1f, 76 },
    { 0xb2f12c1e, 90 },
    { 0x71912c1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96612c2b, 95 },
    { 0x5c312c2c, 90 },
    { 0x65d12c2d, 34 },
    { 0, 0 },
    { 0x5c712c2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2212c36, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb312c3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912c44, 95 },
    { 0, 0 },
    { 0x8a12c46, 76 },
    { 0x77d12c47, 23 },
    { 0, 0 },
    { 0x2ca12c49, 81 },
    { 0x8d12c49, 61 },
    { 0xf1a12c4b, 93 },
    { 0xa2a12c4c, 40 },
    { 0x29d12c4d, 23 },
    { 0x78012c4a, 76 },
    { 0x19612c4a, 23 },
    { 0x79812c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb412c55, 61 },
    { 0x19912c56, 90 },
    { 0x9c412c57, 61 },
    { 0x26912c56, 55 },
    { 0x9c512c59, 99 },
    { 0x3312c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1712c5d, 97 },
    { 0, 0 },
    { 0x98c12c5f, 23 },
    { 0xfaf12c5f, 98 },
    { 0xb412c61, 52 },
    { 0xa1d12c61, 90 },
    { 0xb1c12c60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e12c68, 33 },
    { 0x7fa12c69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e512c6c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d12c72, 76 },
    { 0, 0 },
    { 0x13612c74, 33 },
    { 0xd4012c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e12c78, 90 },
    { 0x49412c79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1412c7f, 90 },
    { 0xea012c80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d812c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b12c87, 33 },
    { 0x54a12c88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30012c91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e612c94, 90 },
    { 0xd5c12c95, 65 },
    { 0, 0 },
    { 0x7d812c97, 76 },
    { 0xe2412c97, 95 },
    { 0x1712c98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a412c9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc12c9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34512ca6, 93 },
    { 0, 0 },
    { 0x43312ca8, 90 },
    { 0x4e712ca9, 76 },
    { 0xce312ca8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e912cad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d12cb1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x96312cb4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba12cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6112cbb, 81 },
    { 0xe2c12cbc, 61 },
    { 0xb2412cbd, 75 },
    { 0x15312cbd, 23 },
    { 0xf2a12cbb, 98 },
    { 0xda12cbd, 55 },
    { 0x64d12cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212cc8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d12ccc, 52 },
    { 0x72212ccd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d12cd3, 99 },
    { 0x24b12cd4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8912cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf12cda, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12cde, 95 },
    { 0xecf12cdf, 93 },
    { 0x93412ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d12ce3, 35 },
    { 0, 0 },
    { 0x2f812ce5, 85 },
    { 0, 0 },
    { 0x92d12ce7, 45 },
    { 0, 0 },
    { 0x63212ce9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f12ced, 55 },
    { 0x3d512ced, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70812cf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e712cf8, 33 },
    { 0x3e312cf8, 90 },
    { 0xd7a12cfa, 76 },
    { 0x68e12cfb, 90 },
    { 0xbf212cfb, 34 },
    { 0x5c112cfd, 90 },
    { 0x27712cfd, 33 },
    { 0xc8412cff, 46 },
    { 0x20d12cf9, 90 },
    { 0xeaf12d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c12d08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5612d16, 95 },
    { 0x2c112d17, 98 },
    { 0x67e12d17, 33 },
    { 0x18312d19, 55 },
    { 0x2d512d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a12d1d, 100 },
    { 0, 0 },
    { 0xedd12d1f, 66 },
    { 0x36412d20, 23 },
    { 0x25f12d20, 55 },
    { 0x28212d22, 61 },
    { 0xb8512d23, 68 },
    { 0xc1612d23, 55 },
    { 0xfc612d21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b12d29, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x90212d2c, 97 },
    { 0, 0 },
    { 0x79f12d2e, 76 },
    { 0, 0 },
    { 0x7fe12d30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6412d35, 76 },
    { 0, 0 },
    { 0x52512d37, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe712d3a, 56 },
    { 0x73012d3b, 36 },
    { 0, 0 },
    { 0xe0112d3d, 98 },
    { 0x53b12d3e, 23 },
    { 0x44f12d3f, 90 },
    { 0xcbb12d3d, 96 },
    { 0x39712d3e, 61 },
    { 0xe4a12d3e, 61 },
    { 0, 0 },
    { 0x26c12d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b112d48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b12d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98612d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b12d55, 91 },
    { 0xf4512d55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f112d5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c012d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3712d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d512d6b, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f212d75, 61 },
    { 0x32012d76, 23 },
    { 0xac812d77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e12d7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a12d81, 46 },
    { 0, 0 },
    { 0x3f412d83, 99 },
    { 0x23612d84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c12d8a, 43 },
    { 0, 0 },
    { 0x6aa12d8c, 75 },
    { 0x65712d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28912d9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44712da0, 43 },
    { 0xe2f12da0, 76 },
    { 0x5a012da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f312da5, 23 },
    { 0x4912da6, 90 },
    { 0x22812da7, 23 },
    { 0xfd512da8, 43 },
    { 0x5e612da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c12dae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d12dbc, 95 },
    { 0x50512dbd, 17 },
    { 0xdf12dbd, 100 },
    { 0, 0 },
    { 0xd912dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e12dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a112dcb, 23 },
    { 0x97b12dcc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xe712dcf, 33 },
    { 0x5b512dd0, 55 },
    { 0, 0 },
    { 0xe6612dd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7312dda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c212ddd, 98 },
    { 0x10612dde, 33 },
    { 0, 0 },
    { 0xc7312de0, 78 },
    { 0, 0 },
    { 0x9c712de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf912de8, 40 },
    { 0x30912de9, 33 },
    { 0x7f212de8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e512ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb612df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2312df3, 23 },
    { 0x13312df4, 95 },
    { 0xea512df4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d12df8, 33 },
    { 0xd0412df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d712dfc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc112dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db12e03, 80 },
    { 0xe9f12e04, 61 },
    { 0x4e512e03, 90 },
    { 0xdaf12e05, 61 },
    { 0x7d912e04, 99 },
    { 0xaf212e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff12e0b, 33 },
    { 0x8bb12e0c, 95 },
    { 0x9e712e0c, 93 },
    { 0x25512e0c, 93 },
    { 0xd4c12e0f, 23 },
    { 0, 0 },
    { 0x45812e11, 93 },
    { 0xd1c12e11, 98 },
    { 0, 0 },
    { 0x52812e14, 23 },
    { 0x52712e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf412e1a, 33 },
    { 0xc2312e1b, 90 },
    { 0, 0 },
    { 0xbc212e1d, 61 },
    { 0, 0 },
    { 0xbda12e1f, 91 },
    { 0, 0 },
    { 0x8db12e21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d912e25, 55 },
    { 0x2d12e26, 61 },
    { 0x27812e27, 98 },
    { 0xaf412e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad12e2c, 33 },
    { 0x5c412e2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f512e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d12e3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59712e42, 55 },
    { 0x66812e43, 23 },
    { 0x8112e44, 55 },
    { 0, 0 },
    { 0x3d412e46, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3112e49, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b12e4e, 90 },
    { 0, 0 },
    { 0x2b512e50, 33 },
    { 0x1db12e51, 78 },
    { 0x40412e51, 23 },
    { 0x39e12e50, 61 },
    { 0x7dc12e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69612e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad12e5a, 34 },
    { 0, 0 },
    { 0x7d212e5c, 23 },
    { 0x5be12e5d, 23 },
    { 0xfe912e5e, 61 },
    { 0x4af12e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa212e67, 77 },
    { 0xf1f12e67, 23 },
    { 0x25412e69, 61 },
    { 0xbb312e69, 95 },
    { 0x6712e67, 90 },
    { 0, 0 },
    { 0x46e12e6d, 90 },
    { 0, 0 },
    { 0xea812e6f, 90 },
    { 0xdd512e70, 23 },
    { 0x52b12e71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37812e7f, 95 },
    { 0xdb312e80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb12e87, 33 },
    { 0xc2c12e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95512e8c, 23 },
    { 0x23812e8d, 19 },
    { 0xcdf12e8e, 66 },
    { 0x21d12e8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e012e9c, 95 },
    { 0, 0 },
    { 0x68c12e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60012ea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90212ea7, 76 },
    { 0xea212ea8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba512eae, 66 },
    { 0x11c12eaf, 90 },
    { 0, 0 },
    { 0x2ad12eb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5312eb4, 90 },
    { 0, 0 },
    { 0x5e912eb6, 43 },
    { 0xfd712eb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4112eba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f812ebe, 55 },
    { 0x4dd12ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80312ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1912ec8, 61 },
    { 0x7c512ec8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc512ed1, 33 },
    { 0xb7912ed1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d12ed6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3512edc, 2 },
    { 0xd3112edd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2012ee2, 98 },
    { 0x7fd12ee3, 68 },
    { 0xdc912ee3, 43 },
    { 0, 0 },
    { 0x30712ee6, 33 },
    { 0xed312ee7, 98 },
    { 0xdf612ee7, 55 },
    { 0x7a012ee7, 91 },
    { 0xdb812ee8, 90 },
    { 0, 0 },
    { 0xf6812eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30912ef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf112ef4, 43 },
    { 0x62c12ef5, 33 },
    { 0x79412ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d312ef9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c912efd, 43 },
    { 0, 0 },
    { 0xb7412eff, 55 },
    { 0x33a12f00, 52 },
    { 0x19012f01, 61 },
    { 0xd8712f01, 46 },
    { 0xd4312f01, 95 },
    { 0x90612f03, 90 },
    { 0x58012f05, 61 },
    { 0x82512f06, 90 },
    { 0xbea12f03, 76 },
    { 0xbbd12f08, 46 },
    { 0x97a12f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79912f11, 61 },
    { 0x49e12f12, 91 },
    { 0x6e112f11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a012f16, 90 },
    { 0x93612f17, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf12f1a, 66 },
    { 0, 0 },
    { 0xa3d12f1c, 91 },
    { 0, 0 },
    { 0xd5412f1e, 81 },
    { 0xf7512f1e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f12f24, 33 },
    { 0xb4b12f25, 43 },
    { 0xd7912f25, 23 },
    { 0x19112f26, 99 },
    { 0xc7212f27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2212f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34712f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca12f3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc112f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d12f46, 91 },
    { 0x77c12f47, 95 },
    { 0xa4912f47, 75 },
    { 0xa7d12f47, 55 },
    { 0, 0 },
    { 0xa8b12f4b, 88 },
    { 0, 0 },
    { 0xeb112f4d, 93 },
    { 0x7d512f4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65512f55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9212f58, 90 },
    { 0x9ed12f59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e12f63, 90 },
    { 0x5a512f64, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55012f69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb12f6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf12f73, 66 },
    { 0x85512f73, 33 },
    { 0, 0 },
    { 0xa2f12f76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41112f7c, 33 },
    { 0, 0 },
    { 0x3bf12f7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12f83, 77 },
    { 0, 0 },
    { 0xb4212f85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13712f88, 95 },
    { 0x95012f89, 98 },
    { 0x2d412f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a12f8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc12f91, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac12f95, 55 },
    { 0xadc12f96, 33 },
    { 0, 0 },
    { 0xf1d12f98, 95 },
    { 0xcb712f99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f12fa0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39812fa4, 48 },
    { 0xcab12fa5, 61 },
    { 0, 0 },
    { 0xb2212fa7, 98 },
    { 0, 0 },
    { 0x71b12fa9, 80 },
    { 0x24012faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88312fae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81312fb1, 91 },
    { 0x46c12fb2, 43 },
    { 0x23012fb3, 90 },
    { 0, 0 },
    { 0xc1c12fb5, 83 },
    { 0, 0 },
    { 0x4de12fb7, 33 },
    { 0x5db12fb8, 97 },
    { 0xc7b12fb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c12fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec12fc3, 99 },
    { 0xd2d12fc3, 95 },
    { 0, 0 },
    { 0x3612fc6, 93 },
    { 0x83012fc7, 90 },
    { 0x81412fc8, 61 },
    { 0x23f12fc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4712fcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15912fd3, 47 },
    { 0x11d12fd3, 98 },
    { 0xce912fd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce12fdb, 55 },
    { 0xf6912fdc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec12fe7, 34 },
    { 0xca712fe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f12feb, 17 },
    { 0x76a12feb, 61 },
    { 0xf5712fed, 95 },
    { 0x22d12fee, 77 },
    { 0x14212fef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2512ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32012ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39412ff9, 95 },
    { 0, 0 },
    { 0x8612ffb, 61 },
    { 0xf6412ffc, 61 },
    { 0x72112ffd, 90 },
    { 0x2a712ffe, 83 },
    { 0xa312ffc, 33 },
    { 0xa8713000, 33 },
    { 0, 0 },
    { 0xed13002, 90 },
    { 0xc3513003, 66 },
    { 0, 0 },
    { 0xdb813005, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b01300a, 46 },
    { 0x3bd1300b, 91 },
    { 0x5241300c, 98 },
    { 0x48c1300b, 33 },
    { 0xab71300e, 23 },
    { 0x43a1300e, 90 },
    { 0x8691300a, 90 },
    { 0x1821300a, 40 },
    { 0x2f713011, 95 },
    { 0x7981300f, 90 },
    { 0xa0013014, 23 },
    { 0x26313014, 61 },
    { 0, 0 },
    { 0x6fd13017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c1301a, 99 },
    { 0xc1e1301b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd013021, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b13024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c13028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21302b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891302f, 46 },
    { 0xff41302f, 55 },
    { 0, 0 },
    { 0x12d13032, 95 },
    { 0, 0 },
    { 0xd8613034, 61 },
    { 0x6a813034, 98 },
    { 0x68b13036, 77 },
    { 0, 0 },
    { 0xb1113038, 33 },
    { 0xcee13039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa881303c, 67 },
    { 0xd7a1303d, 90 },
    { 0x9b01303e, 55 },
    { 0xb321303f, 90 },
    { 0xb1713040, 23 },
    { 0xbe713041, 66 },
    { 0x3b413042, 77 },
    { 0, 0 },
    { 0x9eb13044, 64 },
    { 0, 0 },
    { 0x73213046, 95 },
    { 0x46e13047, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c1304d, 94 },
    { 0xb481304e, 99 },
    { 0x4631304e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b413054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1305b, 61 },
    { 0, 0 },
    { 0xc1b1305d, 90 },
    { 0x73e1305e, 23 },
    { 0x6ed1305f, 55 },
    { 0, 0 },
    { 0x1113061, 97 },
    { 0x28913061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813067, 90 },
    { 0xec213068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b11306b, 81 },
    { 0x7531306c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941306f, 61 },
    { 0x55213070, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed13074, 90 },
    { 0x16d13075, 98 },
    { 0, 0 },
    { 0xbd813077, 90 },
    { 0, 0 },
    { 0x7b913079, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d1307d, 61 },
    { 0xe8c1307e, 91 },
    { 0x2d21307f, 33 },
    { 0x7401307e, 85 },
    { 0xe6c13081, 33 },
    { 0x8dd1307e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec213087, 33 },
    { 0, 0 },
    { 0x1c613089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa551308c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5613090, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7113099, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf991309d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523130a1, 95 },
    { 0x61130a2, 23 },
    { 0xd10130a3, 90 },
    { 0xc55130a3, 90 },
    { 0x986130a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba130a8, 77 },
    { 0, 0 },
    { 0xcb3130aa, 98 },
    { 0xe130aa, 23 },
    { 0x48b130ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be130b8, 34 },
    { 0, 0 },
    { 0xb50130ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7130be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30130c1, 61 },
    { 0xb87130c1, 43 },
    { 0xaed130c1, 90 },
    { 0, 0 },
    { 0x4b7130c5, 55 },
    { 0xc50130c6, 90 },
    { 0, 0 },
    { 0x9ff130c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff130cd, 76 },
    { 0xd95130cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27130d1, 85 },
    { 0xff9130d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef130d5, 76 },
    { 0, 0 },
    { 0x89130d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae130db, 90 },
    { 0, 0 },
    { 0x91e130dd, 78 },
    { 0xb1f130de, 33 },
    { 0x44b130df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31130e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393130e6, 91 },
    { 0x9a7130e7, 90 },
    { 0, 0 },
    { 0xc91130e9, 98 },
    { 0x2be130ea, 33 },
    { 0, 0 },
    { 0x893130ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb130f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa130f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc130f9, 46 },
    { 0x6c7130f9, 95 },
    { 0, 0 },
    { 0xce4130fc, 33 },
    { 0x4d9130fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a13100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68413104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5613107, 61 },
    { 0xa9013108, 75 },
    { 0, 0 },
    { 0x8721310a, 95 },
    { 0xba81310b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d913112, 23 },
    { 0xcbc13113, 98 },
    { 0x60a13114, 85 },
    { 0, 0 },
    { 0x3e113116, 55 },
    { 0, 0 },
    { 0xcf713118, 95 },
    { 0xcbd13119, 76 },
    { 0x7b013118, 33 },
    { 0xa7713118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a71311f, 33 },
    { 0xe491311f, 66 },
    { 0xdf213121, 90 },
    { 0, 0 },
    { 0x55b13123, 78 },
    { 0x66813123, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39513128, 76 },
    { 0xe6213129, 77 },
    { 0x8091312a, 91 },
    { 0, 0 },
    { 0x2791312c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7071312f, 46 },
    { 0xda13130, 76 },
    { 0x7dd13131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30613135, 99 },
    { 0xe8613136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d1313b, 98 },
    { 0xb121313c, 23 },
    { 0, 0 },
    { 0x311313e, 43 },
    { 0xcdf1313f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1314a, 43 },
    { 0x62d1314b, 75 },
    { 0xbed1314c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3613151, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf513155, 95 },
    { 0x68a13155, 100 },
    { 0xb413157, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c1315d, 98 },
    { 0, 0 },
    { 0xa531315f, 33 },
    { 0x52b1315f, 43 },
    { 0, 0 },
    { 0x5b513162, 91 },
    { 0x91d13162, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2413167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851316a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf91316d, 23 },
    { 0x161316d, 33 },
    { 0x2dc1316d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55113173, 99 },
    { 0x94213174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2013177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81317d, 61 },
    { 0xd2d1317d, 61 },
    { 0xc721317d, 90 },
    { 0x5b313180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7113184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13187, 33 },
    { 0x7f113188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a31318d, 33 },
    { 0xa851318e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c013192, 55 },
    { 0, 0 },
    { 0x75913194, 55 },
    { 0, 0 },
    { 0x66b13196, 90 },
    { 0xf2313197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378131a1, 90 },
    { 0x190131a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74131a5, 23 },
    { 0, 0 },
    { 0xb38131a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a131ab, 81 },
    { 0, 0 },
    { 0xfe8131ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c131b0, 23 },
    { 0, 0 },
    { 0xeb3131b2, 61 },
    { 0x83b131b2, 33 },
    { 0, 0 },
    { 0xc0c131b5, 66 },
    { 0x598131b6, 90 },
    { 0x471131b6, 93 },
    { 0, 0 },
    { 0x2d0131b9, 75 },
    { 0, 0 },
    { 0x512131bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c131be, 23 },
    { 0, 0 },
    { 0x55d131c0, 93 },
    { 0x6f9131c1, 99 },
    { 0x9d3131c2, 23 },
    { 0x5bd131c2, 34 },
    { 0xe98131c3, 90 },
    { 0x1f3131c5, 34 },
    { 0x4b5131c6, 76 },
    { 0x336131c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26131cd, 90 },
    { 0, 0 },
    { 0x4b7131cf, 77 },
    { 0xf35131cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c131e5, 46 },
    { 0, 0 },
    { 0x6e4131e7, 95 },
    { 0xfed131e8, 67 },
    { 0xfb131e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1131ee, 90 },
    { 0x181131ef, 23 },
    { 0x2ad131f0, 23 },
    { 0xd94131ef, 93 },
    { 0x738131f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e131f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf131fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8513203, 90 },
    { 0xc813204, 91 },
    { 0, 0 },
    { 0x78b13206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e1320a, 33 },
    { 0xbbc1320a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91320e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa613215, 23 },
    { 0, 0 },
    { 0xeaf13217, 95 },
    { 0x41513218, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2041321d, 23 },
    { 0, 0 },
    { 0x63f1321f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57613222, 33 },
    { 0x2bb13223, 95 },
    { 0x91813224, 23 },
    { 0xc6f13225, 33 },
    { 0x1fa13226, 34 },
    { 0xf0e13227, 61 },
    { 0x2aa13223, 90 },
    { 0x4913225, 95 },
    { 0x8a713226, 68 },
    { 0x71713225, 93 },
    { 0xfa91322a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b213230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69213234, 81 },
    { 0, 0 },
    { 0x10a13236, 46 },
    { 0x46613237, 90 },
    { 0xf6113237, 100 },
    { 0x60a13239, 23 },
    { 0xd0e1323a, 61 },
    { 0, 0 },
    { 0x2b51323c, 76 },
    { 0, 0 },
    { 0xb741323e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b213241, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff13244, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc961324a, 76 },
    { 0x5571324a, 36 },
    { 0x8501324c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc13253, 34 },
    { 0xb2a13254, 66 },
    { 0x16813255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c51325a, 34 },
    { 0xa971325a, 76 },
    { 0xe1b1325a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5451325f, 98 },
    { 0, 0 },
    { 0x86e13261, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b713264, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b13268, 91 },
    { 0, 0 },
    { 0x95c1326a, 34 },
    { 0x12b1326a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7421326f, 85 },
    { 0x19813270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e13273, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f1327b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d313280, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca13286, 76 },
    { 0, 0 },
    { 0x1d013288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5331328f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2313297, 76 },
    { 0, 0 },
    { 0xcf813299, 84 },
    { 0x5911329a, 34 },
    { 0, 0 },
    { 0x5711329c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de132a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d132a5, 59 },
    { 0x132132a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09132b4, 90 },
    { 0xc42132b5, 43 },
    { 0xff5132b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1132bb, 33 },
    { 0x5b3132bc, 84 },
    { 0x152132bd, 100 },
    { 0xbec132bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119132c2, 95 },
    { 0x701132c3, 34 },
    { 0x1dd132c4, 91 },
    { 0x275132c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c132ca, 98 },
    { 0, 0 },
    { 0xa55132cc, 23 },
    { 0x8d5132cc, 88 },
    { 0xa66132ce, 33 },
    { 0x931132cc, 38 },
    { 0xcd8132d0, 46 },
    { 0, 0 },
    { 0x56f132d2, 76 },
    { 0x755132d3, 98 },
    { 0xf30132d4, 85 },
    { 0xdc7132d4, 66 },
    { 0, 0 },
    { 0xcb3132d7, 66 },
    { 0x400132d8, 95 },
    { 0x449132d8, 33 },
    { 0xcb6132d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e132e3, 95 },
    { 0, 0 },
    { 0xcc132e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7132ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x707132f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2132fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f113303, 90 },
    { 0x16c13303, 34 },
    { 0, 0 },
    { 0x10b13306, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e613309, 87 },
    { 0xce1330a, 23 },
    { 0xe9d1330b, 25 },
    { 0x9201330c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b1330f, 43 },
    { 0x18613310, 34 },
    { 0xc813311, 23 },
    { 0xa4613312, 95 },
    { 0x51513313, 95 },
    { 0x29b13310, 23 },
    { 0x5241330f, 96 },
    { 0x74013312, 95 },
    { 0xa1a13310, 43 },
    { 0xc5213318, 90 },
    { 0x2fa13319, 99 },
    { 0x88f1331a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac11331e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b13325, 90 },
    { 0xbf13326, 43 },
    { 0xd8813326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8911332a, 85 },
    { 0x5e1332b, 66 },
    { 0xfa11332c, 61 },
    { 0xc5a1332d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xae113330, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37013334, 95 },
    { 0, 0 },
    { 0x59913336, 67 },
    { 0x13113337, 61 },
    { 0x18813338, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe11333b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd913341, 100 },
    { 0xa2313341, 95 },
    { 0x42c13343, 33 },
    { 0xad813343, 98 },
    { 0xfba13341, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe13348, 91 },
    { 0, 0 },
    { 0xd3d1334a, 76 },
    { 0x6971334a, 61 },
    { 0, 0 },
    { 0x74e1334d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f13359, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921335c, 98 },
    { 0x121335d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61413361, 95 },
    { 0x1d813362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa313365, 23 },
    { 0x34113366, 33 },
    { 0xee613366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc1336a, 95 },
    { 0x3d1336b, 66 },
    { 0x2a61336c, 99 },
    { 0x8ba1336c, 55 },
    { 0x6051336e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x213371, 90 },
    { 0x68513372, 23 },
    { 0x94c13373, 65 },
    { 0, 0 },
    { 0x5bf13375, 33 },
    { 0x22613376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90913379, 61 },
    { 0x8781337a, 68 },
    { 0x6a31337a, 95 },
    { 0xf401337a, 66 },
    { 0, 0 },
    { 0xa961337e, 43 },
    { 0xdb11337f, 93 },
    { 0xe0313380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae13385, 95 },
    { 0, 0 },
    { 0xa7d13387, 46 },
    { 0xc0c13388, 55 },
    { 0x8a913387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911338f, 87 },
    { 0, 0 },
    { 0x84613391, 91 },
    { 0, 0 },
    { 0xcd513393, 85 },
    { 0xd4c13394, 61 },
    { 0x6a013395, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31339a, 23 },
    { 0xc721339a, 61 },
    { 0xec71339a, 90 },
    { 0x2e61339d, 99 },
    { 0xf6d1339a, 43 },
    { 0, 0 },
    { 0x50d133a0, 98 },
    { 0xe48133a0, 80 },
    { 0x737133a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639133a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4133ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36133ae, 78 },
    { 0, 0 },
    { 0x424133b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a133b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb133bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf133c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd133c6, 99 },
    { 0xccd133c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3133ca, 91 },
    { 0, 0 },
    { 0x8a3133cc, 87 },
    { 0xa49133cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133d0, 33 },
    { 0xe59133d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3133d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6133da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c133e3, 23 },
    { 0, 0 },
    { 0x4d0133e5, 93 },
    { 0xc1e133e6, 52 },
    { 0xcca133e7, 95 },
    { 0, 0 },
    { 0x905133e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432133ed, 87 },
    { 0, 0 },
    { 0xa07133ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8133f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d133f7, 33 },
    { 0xa5c133f8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1713403, 88 },
    { 0x10113403, 55 },
    { 0xf1413403, 78 },
    { 0xb0113406, 91 },
    { 0x14513407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591340c, 73 },
    { 0xe1c1340c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c13415, 84 },
    { 0x47213415, 33 },
    { 0x43213417, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21341c, 55 },
    { 0xc141341c, 46 },
    { 0, 0 },
    { 0x6f41341f, 43 },
    { 0x9c41341f, 90 },
    { 0xc361341f, 90 },
    { 0x4ad13420, 98 },
    { 0, 0 },
    { 0xbf813424, 98 },
    { 0x36c13424, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92513428, 90 },
    { 0x64413429, 90 },
    { 0x26e13428, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1342e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95413432, 34 },
    { 0, 0 },
    { 0x5f013434, 95 },
    { 0, 0 },
    { 0xc0313436, 40 },
    { 0xdec13436, 90 },
    { 0, 0 },
    { 0xe4813439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb591343f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4413442, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74613445, 85 },
    { 0x95113446, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61344a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61344d, 33 },
    { 0x9cd1344e, 69 },
    { 0x8341344f, 8 },
    { 0, 0 },
    { 0x59713451, 91 },
    { 0xe3913451, 93 },
    { 0xd6013453, 23 },
    { 0, 0 },
    { 0xba313455, 43 },
    { 0xd1913456, 55 },
    { 0x79b13457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf981345f, 87 },
    { 0xaa21345f, 98 },
    { 0x61513461, 80 },
    { 0xf5213460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f113465, 99 },
    { 0x26613466, 61 },
    { 0x66f13466, 23 },
    { 0x45b13465, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31346d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89413470, 46 },
    { 0x34a13471, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9213476, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad13479, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf831347f, 95 },
    { 0x9981347f, 76 },
    { 0xfff13481, 61 },
    { 0x79b13482, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43313487, 76 },
    { 0x75f13488, 93 },
    { 0x6313489, 61 },
    { 0xfd013489, 33 },
    { 0xaad1348a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31348e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35213493, 33 },
    { 0xea613493, 33 },
    { 0xf013495, 56 },
    { 0, 0 },
    { 0x2b613497, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21349a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d134a0, 33 },
    { 0xdfd134a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949134ae, 61 },
    { 0xd67134af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd134b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86134b5, 61 },
    { 0xd58134b6, 33 },
    { 0xcc1134b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100134c0, 90 },
    { 0xf61134c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x930134c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81134c7, 61 },
    { 0x37c134c8, 95 },
    { 0x6a2134c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x841134cc, 76 },
    { 0x22c134cc, 99 },
    { 0x560134cd, 90 },
    { 0xfd2134cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x771134d2, 77 },
    { 0xe31134d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32134da, 33 },
    { 0x250134db, 33 },
    { 0x8df134dc, 91 },
    { 0xd30134dd, 98 },
    { 0xafe134dd, 90 },
    { 0x3ad134df, 76 },
    { 0xdb2134e0, 61 },
    { 0xfb5134df, 90 },
    { 0, 0 },
    { 0x8d2134e3, 91 },
    { 0x48134e3, 84 },
    { 0, 0 },
    { 0x590134e6, 33 },
    { 0, 0 },
    { 0x736134e8, 95 },
    { 0x10e134e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed134ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8134f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e113501, 55 },
    { 0xd5b13501, 61 },
    { 0xc4413502, 33 },
    { 0x69c13504, 34 },
    { 0x62913505, 90 },
    { 0x63b13506, 61 },
    { 0, 0 },
    { 0x1b213508, 33 },
    { 0xb1f13508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81013512, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0413515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b713519, 84 },
    { 0x181351a, 23 },
    { 0x8971351b, 98 },
    { 0xaee1351b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5051351f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f313526, 76 },
    { 0xcdc13527, 61 },
    { 0x33513528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d1352b, 90 },
    { 0x6731352b, 23 },
    { 0xd091352c, 83 },
    { 0, 0 },
    { 0xd5f1352f, 43 },
    { 0, 0 },
    { 0xfd913531, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7013536, 90 },
    { 0, 0 },
    { 0xdfe13538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d91354c, 87 },
    { 0xdde1354d, 93 },
    { 0xe3f1354d, 34 },
    { 0, 0 },
    { 0x3d513550, 53 },
    { 0xf0913551, 61 },
    { 0x8e513552, 34 },
    { 0x4a213552, 76 },
    { 0x8c213553, 55 },
    { 0, 0 },
    { 0xf0513556, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67113559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf971355e, 66 },
    { 0xb1f1355f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61013563, 34 },
    { 0xc6513564, 33 },
    { 0xab213565, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd051356f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36613573, 95 },
    { 0, 0 },
    { 0x1e113575, 61 },
    { 0, 0 },
    { 0x8c913577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1991357b, 34 },
    { 0x9e81357c, 44 },
    { 0x9841357d, 90 },
    { 0, 0 },
    { 0xb201357f, 95 },
    { 0xb4813580, 93 },
    { 0x58113581, 67 },
    { 0xe3413581, 98 },
    { 0x9a13583, 33 },
    { 0xabe13581, 98 },
    { 0x30113584, 23 },
    { 0x13713586, 98 },
    { 0xb5f13580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071358b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151358e, 43 },
    { 0, 0 },
    { 0x8df13590, 55 },
    { 0, 0 },
    { 0x10413592, 88 },
    { 0, 0 },
    { 0xb5313594, 95 },
    { 0xb3513595, 76 },
    { 0x9ab13596, 95 },
    { 0xc4613597, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221359d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded135a6, 23 },
    { 0xd10135a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x78135aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800135b1, 90 },
    { 0xb41135b1, 90 },
    { 0x458135b3, 33 },
    { 0x501135b4, 84 },
    { 0x77f135b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b135b9, 66 },
    { 0x3b5135b9, 90 },
    { 0xacf135bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24135c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46135c4, 43 },
    { 0, 0 },
    { 0x4fd135c6, 61 },
    { 0x268135c6, 90 },
    { 0x2da135c8, 93 },
    { 0xf45135c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f135cd, 91 },
    { 0x79c135ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa135d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01135d9, 34 },
    { 0xefb135da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d135e2, 84 },
    { 0, 0 },
    { 0xa1b135e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba135e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb135ec, 61 },
    { 0, 0 },
    { 0xd30135ee, 23 },
    { 0xee6135ef, 98 },
    { 0x469135ef, 91 },
    { 0, 0 },
    { 0xb54135f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83135f6, 91 },
    { 0x246135f7, 90 },
    { 0x469135f8, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1135fb, 100 },
    { 0x6ba135fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca13607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1360c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1360f, 23 },
    { 0x9811360f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d13615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1361b, 91 },
    { 0x631361c, 23 },
    { 0x86a1361d, 23 },
    { 0, 0 },
    { 0x3cc1361f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11713623, 33 },
    { 0x4e513624, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a13627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d31362a, 52 },
    { 0, 0 },
    { 0xb331362c, 95 },
    { 0x4961362c, 52 },
    { 0, 0 },
    { 0xff81362f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f113632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f13635, 23 },
    { 0xfcb13636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761363a, 23 },
    { 0xd2a1363b, 98 },
    { 0, 0 },
    { 0x26e1363d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e413641, 76 },
    { 0x58613642, 98 },
    { 0xbab13642, 77 },
    { 0x92f13643, 90 },
    { 0, 0 },
    { 0x1b213646, 93 },
    { 0x7c813647, 60 },
    { 0xfef13648, 76 },
    { 0, 0 },
    { 0x5071364a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e013652, 46 },
    { 0xe3a13653, 67 },
    { 0xfc113652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e13658, 90 },
    { 0x21e13659, 76 },
    { 0, 0 },
    { 0x4d11365b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a413660, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3613663, 34 },
    { 0x2d13664, 76 },
    { 0xf9e13663, 23 },
    { 0xc9013666, 23 },
    { 0xa2c13667, 55 },
    { 0x3cb13668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20213672, 33 },
    { 0x6f513672, 29 },
    { 0xed913674, 90 },
    { 0, 0 },
    { 0xbcd13676, 93 },
    { 0x99713676, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c1367a, 23 },
    { 0x8121367b, 43 },
    { 0xcc41367c, 95 },
    { 0xfa91367d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4913682, 61 },
    { 0xb1213683, 33 },
    { 0, 0 },
    { 0x46a13685, 43 },
    { 0, 0 },
    { 0xdf313687, 85 },
    { 0xf813688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d913690, 76 },
    { 0xcc513691, 33 },
    { 0xbf913692, 90 },
    { 0x64513693, 23 },
    { 0x86413692, 95 },
    { 0xe5913695, 61 },
    { 0, 0 },
    { 0x6da13697, 98 },
    { 0x89513698, 81 },
    { 0x43113698, 90 },
    { 0x76d13699, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f31369d, 77 },
    { 0, 0 },
    { 0x9511369f, 33 },
    { 0x348136a0, 90 },
    { 0x5e136a0, 65 },
    { 0xbb1136a1, 90 },
    { 0x29c136a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42136a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x438136aa, 98 },
    { 0xb5e136ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70136ae, 61 },
    { 0x1e6136af, 61 },
    { 0x961136b0, 76 },
    { 0x351136b0, 55 },
    { 0xb6136b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39136b9, 52 },
    { 0, 0 },
    { 0xc05136bb, 76 },
    { 0x7a2136bb, 46 },
    { 0x117136bb, 23 },
    { 0xaa0136bb, 95 },
    { 0x5df136bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120136c4, 90 },
    { 0x6cb136c5, 23 },
    { 0, 0 },
    { 0xf5f136c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6136ca, 93 },
    { 0xeed136cb, 34 },
    { 0, 0 },
    { 0x8c136cd, 33 },
    { 0x4d7136ce, 90 },
    { 0xce0136cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4136d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d136db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x285136de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84136e1, 23 },
    { 0x356136e1, 87 },
    { 0x33d136e1, 93 },
    { 0, 0 },
    { 0xea136e5, 34 },
    { 0xba7136e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d136ee, 55 },
    { 0x8cb136ee, 99 },
    { 0x583136ef, 43 },
    { 0, 0 },
    { 0xdcc136f2, 91 },
    { 0x3ee136f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242136f8, 95 },
    { 0x1ab136f9, 98 },
    { 0xc57136fa, 34 },
    { 0x8dc136f9, 93 },
    { 0, 0 },
    { 0xf7b136fd, 93 },
    { 0x41e136fe, 23 },
    { 0xe97136fe, 33 },
    { 0xc2d13700, 95 },
    { 0xf4913701, 61 },
    { 0xaf113701, 46 },
    { 0, 0 },
    { 0x81013704, 67 },
    { 0xb8f13705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10513709, 52 },
    { 0x7641370a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77413710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29013713, 17 },
    { 0x8e013713, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27413717, 33 },
    { 0xc9e13718, 84 },
    { 0x76913717, 87 },
    { 0x7c41371a, 95 },
    { 0x6ac13719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1371e, 99 },
    { 0, 0 },
    { 0xdce13720, 88 },
    { 0x15d13720, 95 },
    { 0xcaf13721, 87 },
    { 0x12913722, 23 },
    { 0, 0 },
    { 0xa6913725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa13728, 76 },
    { 0xd8e13728, 90 },
    { 0x28c1372a, 61 },
    { 0xeed1372b, 87 },
    { 0x36b13728, 61 },
    { 0x44213728, 95 },
    { 0xbd61372c, 61 },
    { 0xbc013728, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99413733, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a513736, 55 },
    { 0xe9113737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff1373a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71373d, 66 },
    { 0x4ca1373d, 46 },
    { 0x3631373f, 23 },
    { 0xd3e1373d, 34 },
    { 0xfbc13741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c13748, 98 },
    { 0, 0 },
    { 0x2c71374a, 67 },
    { 0xc1f1374b, 98 },
    { 0, 0 },
    { 0x1f21374d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d613751, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7513755, 93 },
    { 0x4ad13756, 90 },
    { 0x41f13757, 34 },
    { 0x1c13756, 23 },
    { 0xf8613758, 76 },
    { 0xbda1375a, 85 },
    { 0xe01375a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c13760, 90 },
    { 0xc213760, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1613764, 95 },
    { 0x90c13764, 67 },
    { 0, 0 },
    { 0xe5413767, 55 },
    { 0x6a813768, 86 },
    { 0, 0 },
    { 0xd821376a, 46 },
    { 0xef11376a, 38 },
    { 0xb501376c, 98 },
    { 0xfb61376a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf013770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x39813773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff13779, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68413780, 90 },
    { 0, 0 },
    { 0x40113782, 23 },
    { 0x87813783, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf13786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6651378e, 66 },
    { 0xed21378e, 61 },
    { 0xda31378e, 76 },
    { 0, 0 },
    { 0xdbe13792, 90 },
    { 0x4213793, 85 },
    { 0xf2513793, 55 },
    { 0, 0 },
    { 0x57a13796, 33 },
    { 0x1c613797, 85 },
    { 0xa1f13798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d1379c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d137a1, 34 },
    { 0, 0 },
    { 0x4a137a3, 95 },
    { 0x4bb137a3, 47 },
    { 0, 0 },
    { 0xa87137a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272137ab, 61 },
    { 0, 0 },
    { 0x32f137ad, 90 },
    { 0, 0 },
    { 0xd33137af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f137b7, 23 },
    { 0xa6137b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42137bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4137c1, 52 },
    { 0xfcd137c2, 90 },
    { 0x9e9137c2, 43 },
    { 0x6bb137c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269137ca, 17 },
    { 0x481137cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac137ce, 76 },
    { 0xcd7137cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09137dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09137df, 69 },
    { 0x737137df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4137e3, 98 },
    { 0xfc5137e4, 34 },
    { 0xeb8137e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527137e9, 34 },
    { 0, 0 },
    { 0xbbf137eb, 23 },
    { 0xf1e137eb, 66 },
    { 0xb47137ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70137f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5137f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc137fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e313801, 90 },
    { 0xf6b13802, 90 },
    { 0, 0 },
    { 0xc0813804, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f13807, 33 },
    { 0x6d313808, 40 },
    { 0xaa813808, 43 },
    { 0x6e31380a, 95 },
    { 0, 0 },
    { 0xedb1380c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b13810, 34 },
    { 0, 0 },
    { 0x24a13812, 55 },
    { 0, 0 },
    { 0x68413814, 33 },
    { 0, 0 },
    { 0xd7a13816, 52 },
    { 0xd513816, 90 },
    { 0xa1413818, 87 },
    { 0x28b13818, 33 },
    { 0xed513819, 33 },
    { 0, 0 },
    { 0x80e1381c, 98 },
    { 0xd491381c, 81 },
    { 0x5051381c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46113823, 90 },
    { 0xaa713823, 90 },
    { 0, 0 },
    { 0x78c13826, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f13829, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb751382e, 65 },
    { 0x64c1382f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68113835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xece13838, 33 },
    { 0, 0 },
    { 0x77b1383a, 90 },
    { 0xe791383b, 95 },
    { 0x3bf1383c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72313841, 91 },
    { 0x2f413842, 23 },
    { 0x1f113842, 61 },
    { 0xbc513844, 33 },
    { 0x2f313845, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57813858, 76 },
    { 0xd0413859, 67 },
    { 0x4641385a, 98 },
    { 0, 0 },
    { 0x1ea1385c, 90 },
    { 0xb131385c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a913866, 61 },
    { 0x60913866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e1386a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3213871, 23 },
    { 0x45613872, 77 },
    { 0xf4413873, 23 },
    { 0xdd413874, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d13877, 99 },
    { 0xfa113878, 68 },
    { 0xb0113879, 99 },
    { 0x4e13879, 55 },
    { 0, 0 },
    { 0x9091387c, 98 },
    { 0x4761387d, 95 },
    { 0x7d1387d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f13882, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1113886, 98 },
    { 0xe5613887, 52 },
    { 0xd0713887, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6391388d, 43 },
    { 0, 0 },
    { 0x6211388f, 95 },
    { 0x4f113890, 76 },
    { 0, 0 },
    { 0x22513892, 87 },
    { 0xb9c13892, 91 },
    { 0x60f13894, 23 },
    { 0x81113893, 85 },
    { 0, 0 },
    { 0xe9313897, 78 },
    { 0, 0 },
    { 0xfbf13899, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b81389e, 43 },
    { 0, 0 },
    { 0x872138a0, 66 },
    { 0x48138a0, 55 },
    { 0x1c6138a2, 33 },
    { 0xc0e138a0, 99 },
    { 0xb08138a4, 33 },
    { 0x643138a5, 33 },
    { 0, 0 },
    { 0xaf2138a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x514138aa, 95 },
    { 0x14138aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38138af, 66 },
    { 0xc49138b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67138b5, 23 },
    { 0xd43138b6, 93 },
    { 0x124138b5, 23 },
    { 0xacc138b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8138bb, 95 },
    { 0x54c138bc, 23 },
    { 0x6db138bc, 33 },
    { 0x2fc138bd, 34 },
    { 0x47a138bf, 66 },
    { 0xcf1138c0, 84 },
    { 0x15b138be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e138c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6138c8, 23 },
    { 0, 0 },
    { 0xddb138ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e138cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6138d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226138d5, 76 },
    { 0xb3a138d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x593138d9, 90 },
    { 0x3ab138da, 77 },
    { 0x5a0138d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2138e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c138e5, 78 },
    { 0xaaf138e6, 76 },
    { 0x96b138e7, 33 },
    { 0x7df138e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e138ed, 84 },
    { 0x66e138ee, 33 },
    { 0xcfd138ef, 46 },
    { 0xa52138ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x856138f3, 88 },
    { 0xd42138f3, 95 },
    { 0x5a8138f5, 61 },
    { 0x6dd138f6, 33 },
    { 0x270138f4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd813900, 76 },
    { 0xe813900, 99 },
    { 0x7c013902, 67 },
    { 0xf4c13900, 90 },
    { 0x59b13903, 23 },
    { 0x33913905, 98 },
    { 0x66813905, 23 },
    { 0, 0 },
    { 0x49113908, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f1390c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3013910, 95 },
    { 0x1f113910, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce413915, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc813918, 40 },
    { 0xfb613919, 90 },
    { 0xe9c1391a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b13921, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a13924, 43 },
    { 0xaf313925, 61 },
    { 0x28813926, 90 },
    { 0x2ec13926, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1392d, 90 },
    { 0, 0 },
    { 0xbb81392f, 93 },
    { 0x3491392f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x22113933, 81 },
    { 0, 0 },
    { 0x13c13935, 90 },
    { 0, 0 },
    { 0x57d13937, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1393b, 95 },
    { 0xc431393b, 33 },
    { 0x1821393b, 23 },
    { 0xaca1393e, 46 },
    { 0xbd91393f, 98 },
    { 0x1cc1393e, 33 },
    { 0x3131393f, 99 },
    { 0xdde13942, 95 },
    { 0xea713943, 99 },
    { 0xef51393d, 76 },
    { 0x14313945, 43 },
    { 0, 0 },
    { 0xb6613947, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81394c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4313951, 95 },
    { 0, 0 },
    { 0x4ad13953, 23 },
    { 0xc1d13954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a313957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd1395d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7313964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c13967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2031396c, 90 },
    { 0xf1a1396d, 77 },
    { 0xf041396e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a513973, 93 },
    { 0, 0 },
    { 0x60913975, 69 },
    { 0xa113976, 95 },
    { 0x9ce13975, 99 },
    { 0, 0 },
    { 0x3ee13979, 55 },
    { 0, 0 },
    { 0x8741397b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43513982, 33 },
    { 0, 0 },
    { 0x3c13984, 90 },
    { 0x94d13985, 52 },
    { 0x5e213984, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1931398a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf1398d, 84 },
    { 0, 0 },
    { 0x7ba1398f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a13996, 90 },
    { 0xc1d13997, 61 },
    { 0x9b713996, 61 },
    { 0, 0 },
    { 0xb6b1399a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e1399d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6139a6, 90 },
    { 0, 0 },
    { 0x2a7139a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548139b6, 95 },
    { 0, 0 },
    { 0x9d4139b8, 91 },
    { 0x3ec139b8, 66 },
    { 0xc99139ba, 38 },
    { 0x3f3139ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2139bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775139c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b139c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661139ce, 93 },
    { 0xab9139ce, 90 },
    { 0xf0139cf, 98 },
    { 0x432139d1, 33 },
    { 0xeef139d1, 33 },
    { 0x308139d0, 66 },
    { 0, 0 },
    { 0x45a139d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39139dc, 61 },
    { 0x309139dd, 33 },
    { 0x208139dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8139e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a139eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595139f1, 23 },
    { 0x911139f1, 61 },
    { 0xcf2139f3, 99 },
    { 0x294139f3, 55 },
    { 0, 0 },
    { 0x81a139f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611139fd, 33 },
    { 0, 0 },
    { 0x84b139ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc13a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6013a19, 90 },
    { 0x43513a1a, 100 },
    { 0xcd613a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7113a21, 43 },
    { 0xc3013a21, 90 },
    { 0x79413a23, 93 },
    { 0x93713a24, 98 },
    { 0x47e13a25, 65 },
    { 0x4c213a26, 43 },
    { 0xf7913a21, 77 },
    { 0x26313a28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36213a2e, 55 },
    { 0, 0 },
    { 0xec813a30, 34 },
    { 0x59513a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24613a34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b13a39, 90 },
    { 0xc2413a3a, 98 },
    { 0x4bc13a3b, 23 },
    { 0x55213a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16313a3f, 61 },
    { 0, 0 },
    { 0xe13a41, 99 },
    { 0x44e13a42, 61 },
    { 0x60613a43, 61 },
    { 0xb7c13a43, 23 },
    { 0x2e013a45, 90 },
    { 0xea013a46, 98 },
    { 0x15813a46, 88 },
    { 0, 0 },
    { 0x8cf13a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5013a4c, 23 },
    { 0x52513a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82813a52, 77 },
    { 0x6a713a53, 78 },
    { 0xc4713a54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f613a57, 99 },
    { 0xcad13a58, 99 },
    { 0xee813a59, 61 },
    { 0x2ac13a57, 23 },
    { 0x1c113a57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59113a60, 33 },
    { 0xf513a61, 23 },
    { 0x41013a61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20213a65, 46 },
    { 0, 0 },
    { 0xef513a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45813a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63513a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd13a75, 61 },
    { 0, 0 },
    { 0x8a713a77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba013a7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4113a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d813a88, 55 },
    { 0x79a13a89, 66 },
    { 0x28d13a8a, 61 },
    { 0xacb13a8b, 91 },
    { 0x9eb13a8c, 95 },
    { 0x6b13a8d, 23 },
    { 0x90013a89, 84 },
    { 0xe4213a8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85613a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90613a9a, 34 },
    { 0xcfc13a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d913a9f, 91 },
    { 0xe9713aa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19513aa3, 61 },
    { 0, 0 },
    { 0x34513aa5, 83 },
    { 0, 0 },
    { 0xb0d13aa7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c13aac, 76 },
    { 0x1fa13aad, 46 },
    { 0x30c13aae, 100 },
    { 0x35013aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14813ab3, 90 },
    { 0xe0d13ab3, 76 },
    { 0x51313ab5, 93 },
    { 0xb8a13ab5, 33 },
    { 0xd3b13ab7, 78 },
    { 0x8bd13ab7, 46 },
    { 0x9b913ab8, 98 },
    { 0xb2413ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26813abd, 61 },
    { 0x92a13abe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5813ac3, 95 },
    { 0, 0 },
    { 0xa3e13ac5, 33 },
    { 0x4a813ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76313acb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a113ad1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc13ad8, 34 },
    { 0, 0 },
    { 0x29d13ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7613ae2, 33 },
    { 0, 0 },
    { 0x1e913ae4, 95 },
    { 0x45113ae5, 33 },
    { 0, 0 },
    { 0x5e213ae7, 98 },
    { 0x22413ae8, 52 },
    { 0x22f13ae9, 90 },
    { 0x65413aea, 75 },
    { 0xe9413aea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17313aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea813af3, 33 },
    { 0, 0 },
    { 0x60c13af5, 98 },
    { 0xd0e13af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe813af9, 61 },
    { 0x8e313af9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e813b00, 46 },
    { 0, 0 },
    { 0xccc13b02, 84 },
    { 0x3bb13b03, 33 },
    { 0xbc713b04, 95 },
    { 0xe2513b03, 88 },
    { 0xdf413b06, 55 },
    { 0x25c13b07, 76 },
    { 0xd2213b07, 98 },
    { 0xfb013b09, 61 },
    { 0x1bd13b0a, 68 },
    { 0xc6413b06, 65 },
    { 0x1fc13b0b, 61 },
    { 0xd1813b0d, 33 },
    { 0xec413b0d, 98 },
    { 0xb2b13b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d13b14, 61 },
    { 0, 0 },
    { 0x87713b16, 90 },
    { 0xcef13b16, 55 },
    { 0xf1013b17, 23 },
    { 0x89413b16, 46 },
    { 0x69213b1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f013b21, 90 },
    { 0, 0 },
    { 0x33a13b23, 52 },
    { 0, 0 },
    { 0x87e13b25, 33 },
    { 0x2be13b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded13b2b, 46 },
    { 0xd7d13b2c, 33 },
    { 0xf1d13b2d, 52 },
    { 0, 0 },
    { 0x28c13b2f, 98 },
    { 0x64213b30, 84 },
    { 0x81113b2f, 84 },
    { 0x6f313b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61613b38, 53 },
    { 0, 0 },
    { 0xcee13b3a, 55 },
    { 0x49813b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec13b40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d13b44, 55 },
    { 0, 0 },
    { 0xb1a13b46, 90 },
    { 0xfeb13b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af13b4e, 67 },
    { 0, 0 },
    { 0x55613b50, 66 },
    { 0, 0 },
    { 0x80213b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8313b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47513b59, 61 },
    { 0x3dd13b59, 95 },
    { 0x1ee13b5b, 23 },
    { 0x413b5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e213b5f, 54 },
    { 0xb5d13b60, 98 },
    { 0, 0 },
    { 0xbd13b62, 98 },
    { 0x34f13b62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f413b69, 91 },
    { 0x41613b69, 96 },
    { 0xbfd13b6b, 90 },
    { 0, 0 },
    { 0xbb113b6d, 46 },
    { 0, 0 },
    { 0x4d713b6f, 90 },
    { 0, 0 },
    { 0xd7513b71, 23 },
    { 0x79e13b72, 33 },
    { 0x32813b72, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd13b76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6013b7e, 90 },
    { 0x15313b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38413b82, 23 },
    { 0, 0 },
    { 0x83113b84, 46 },
    { 0x9b013b84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f413b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b13b8f, 52 },
    { 0x6c813b8f, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf13b96, 76 },
    { 0, 0 },
    { 0xb2d13b98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83413ba0, 77 },
    { 0x85c13ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a13ba6, 84 },
    { 0x95313ba7, 84 },
    { 0xf4713ba8, 61 },
    { 0, 0 },
    { 0xd8513baa, 55 },
    { 0x61813bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb513bae, 68 },
    { 0x9d413baf, 55 },
    { 0, 0 },
    { 0xc3013bb1, 23 },
    { 0x8e13bb2, 46 },
    { 0, 0 },
    { 0xa9e13bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1213bb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61313bbd, 90 },
    { 0xa8913bbe, 23 },
    { 0x36613bbf, 90 },
    { 0xbc613bc0, 46 },
    { 0x78c13bc1, 33 },
    { 0x5f13bbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f13bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd413bcb, 66 },
    { 0x3a413bcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a13bd2, 46 },
    { 0x20613bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2713bd6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca13bdf, 61 },
    { 0xe9513be0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe813be3, 52 },
    { 0xff813be4, 90 },
    { 0xa0213be3, 61 },
    { 0x9a13be6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde13bea, 90 },
    { 0x9c713beb, 90 },
    { 0x7aa13bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a13bf3, 90 },
    { 0x5013bf3, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a13bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d513c05, 95 },
    { 0, 0 },
    { 0x37213c07, 76 },
    { 0x1e113c08, 100 },
    { 0x59513c09, 90 },
    { 0xb813c0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d013c0f, 76 },
    { 0xc6713c0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc13c13, 46 },
    { 0x43a13c14, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a713c17, 43 },
    { 0x98513c18, 46 },
    { 0xc8413c17, 91 },
    { 0x28113c1a, 95 },
    { 0, 0 },
    { 0x73e13c1c, 91 },
    { 0xeec13c1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10813c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d13c29, 33 },
    { 0x7e813c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72113c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9513c3a, 93 },
    { 0xe4913c3a, 23 },
    { 0xe7713c3a, 33 },
    { 0xc913c3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b13c40, 99 },
    { 0, 0 },
    { 0x11813c42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71213c48, 55 },
    { 0, 0 },
    { 0xb4b13c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b13c4e, 55 },
    { 0x4e413c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d13c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46013c60, 61 },
    { 0, 0 },
    { 0x27f13c62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a513c69, 17 },
    { 0x1b113c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46013c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e13c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8313c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a13c7e, 90 },
    { 0, 0 },
    { 0xe9f13c80, 84 },
    { 0x71213c81, 75 },
    { 0x68413c81, 84 },
    { 0xc0713c82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60813c86, 33 },
    { 0xafd13c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63613c8a, 93 },
    { 0xf0213c8a, 95 },
    { 0x26413c8c, 34 },
    { 0x4ff13c8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b13c91, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0913c94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9713c98, 55 },
    { 0x16b13c99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45013c9f, 23 },
    { 0xbcb13c9f, 90 },
    { 0, 0 },
    { 0x16f13ca2, 90 },
    { 0x9c13ca2, 46 },
    { 0x113ca4, 36 },
    { 0x89913ca5, 33 },
    { 0x3af13ca5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48813cb0, 87 },
    { 0x42e13cb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c13cb6, 93 },
    { 0x99b13cb7, 90 },
    { 0, 0 },
    { 0x68913cb9, 90 },
    { 0, 0 },
    { 0x10813cbb, 61 },
    { 0, 0 },
    { 0x70b13cbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb13cc2, 90 },
    { 0, 0 },
    { 0x16e13cc4, 56 },
    { 0, 0 },
    { 0x90713cc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2513ccc, 90 },
    { 0x70e13ccd, 91 },
    { 0x57513ccc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80113cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b13cd4, 100 },
    { 0xc8b13cd5, 87 },
    { 0x70613cd6, 34 },
    { 0xea013cd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9913cdd, 61 },
    { 0xa2e13cde, 46 },
    { 0xf7c13cdf, 76 },
    { 0, 0 },
    { 0xa9013ce1, 98 },
    { 0x3b113ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e13ce5, 98 },
    { 0x3c613ce6, 67 },
    { 0x3e413ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1613cea, 95 },
    { 0, 0 },
    { 0xd4e13cec, 98 },
    { 0, 0 },
    { 0xed713cee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a113cf1, 99 },
    { 0, 0 },
    { 0x93b13cf3, 76 },
    { 0x21a13cf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb113cf7, 95 },
    { 0xcf713cf8, 90 },
    { 0xbe13cf8, 23 },
    { 0, 0 },
    { 0x6f413cfb, 78 },
    { 0x90213cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d13d01, 76 },
    { 0, 0 },
    { 0xe3f13d03, 65 },
    { 0xac813d04, 90 },
    { 0x2ec13d04, 46 },
    { 0, 0 },
    { 0x96d13d07, 98 },
    { 0xe9513d08, 93 },
    { 0x4ca13d09, 38 },
    { 0xb8613d0a, 33 },
    { 0, 0 },
    { 0x8cd13d0c, 61 },
    { 0x1f113d0d, 95 },
    { 0x48513d0e, 87 },
    { 0x55113d0e, 95 },
    { 0xac13d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a13d13, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82413d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62113d1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad13d1e, 78 },
    { 0x32813d1f, 91 },
    { 0x50013d1e, 61 },
    { 0, 0 },
    { 0x2a13d22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a13d25, 61 },
    { 0, 0 },
    { 0x7f113d27, 33 },
    { 0xf3913d28, 61 },
    { 0xaa013d28, 99 },
    { 0xbd813d2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b113d2e, 95 },
    { 0, 0 },
    { 0x8e813d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda713d33, 61 },
    { 0, 0 },
    { 0x63c13d35, 33 },
    { 0, 0 },
    { 0xd4113d37, 90 },
    { 0xfa513d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86413d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b13d3f, 40 },
    { 0, 0 },
    { 0x5f313d41, 33 },
    { 0xe5a13d42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0813d46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f113d49, 90 },
    { 0x36113d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c413d4e, 46 },
    { 0xd2813d4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x15413d52, 98 },
    { 0x9f213d53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce13d57, 33 },
    { 0x9c13d57, 66 },
    { 0x37813d58, 65 },
    { 0xcd213d57, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f13d5d, 91 },
    { 0x36a13d5d, 33 },
    { 0x38813d5f, 91 },
    { 0x80613d60, 91 },
    { 0, 0 },
    { 0xdf713d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd13d65, 96 },
    { 0x96513d66, 55 },
    { 0x36913d67, 52 },
    { 0x6bd13d68, 95 },
    { 0x65c13d69, 23 },
    { 0, 0 },
    { 0x84a13d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce113d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78913d71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72613d74, 84 },
    { 0, 0 },
    { 0x50a13d76, 93 },
    { 0xea513d77, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41013d7b, 68 },
    { 0, 0 },
    { 0x49c13d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88513d84, 90 },
    { 0x7cf13d84, 88 },
    { 0x42d13d86, 34 },
    { 0x31e13d86, 100 },
    { 0x9c513d86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad13d8c, 55 },
    { 0x1db13d8c, 90 },
    { 0x6213d8e, 90 },
    { 0x10713d8f, 33 },
    { 0, 0 },
    { 0x84413d91, 99 },
    { 0, 0 },
    { 0xb5213d93, 98 },
    { 0xb1a13d94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e13d98, 84 },
    { 0x13c13d99, 36 },
    { 0, 0 },
    { 0x7a413d9b, 90 },
    { 0x59213d9c, 66 },
    { 0x4bf13d9d, 43 },
    { 0x24f13d9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee13da3, 95 },
    { 0xdb113da3, 55 },
    { 0x12513da5, 67 },
    { 0, 0 },
    { 0xdf013da7, 100 },
    { 0, 0 },
    { 0xeed13da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3313dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e013db3, 95 },
    { 0, 0 },
    { 0x81b13db5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a13dbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc613dc7, 67 },
    { 0x18b13dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f13dcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24613dd1, 43 },
    { 0xa2d13dd1, 23 },
    { 0x75713dd3, 61 },
    { 0x63413dd4, 61 },
    { 0x5a413dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd13ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1513ddf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b13de9, 90 },
    { 0, 0 },
    { 0x46813deb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36613dee, 33 },
    { 0, 0 },
    { 0xe1f13df0, 61 },
    { 0x75e13df1, 76 },
    { 0xd013df2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc13df7, 90 },
    { 0xa5a13df8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec013dfe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb13e04, 93 },
    { 0x63413e04, 55 },
    { 0, 0 },
    { 0x97413e07, 83 },
    { 0, 0 },
    { 0xb2713e09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34713e0d, 90 },
    { 0xcd613e0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db13e11, 88 },
    { 0xba013e11, 33 },
    { 0x11413e11, 90 },
    { 0x5b913e13, 66 },
    { 0xdf613e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa113e18, 76 },
    { 0xf4513e19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a513e1c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a713e1f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4113e22, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55813e29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee13e2c, 98 },
    { 0xeae13e2c, 61 },
    { 0, 0 },
    { 0x5cb13e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee13e36, 66 },
    { 0, 0 },
    { 0xaf113e38, 23 },
    { 0xe9013e39, 76 },
    { 0xea413e3a, 78 },
    { 0x79b13e3b, 99 },
    { 0xe2513e39, 61 },
    { 0xcd113e3d, 23 },
    { 0xa9213e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed13e43, 23 },
    { 0x4d913e44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4213e48, 90 },
    { 0, 0 },
    { 0xe5113e4a, 55 },
    { 0x73d13e4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c13e4e, 95 },
    { 0xa9413e4f, 19 },
    { 0x30813e50, 90 },
    { 0x59f13e50, 90 },
    { 0x31e13e50, 33 },
    { 0x31313e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27613e56, 34 },
    { 0x2c313e57, 98 },
    { 0xe2013e57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c13e5c, 67 },
    { 0, 0 },
    { 0x8cd13e5e, 43 },
    { 0, 0 },
    { 0xdd313e60, 90 },
    { 0, 0 },
    { 0x4413e62, 33 },
    { 0x63c13e63, 99 },
    { 0xdb513e62, 95 },
    { 0, 0 },
    { 0x2ab13e66, 90 },
    { 0, 0 },
    { 0x8f513e68, 85 },
    { 0, 0 },
    { 0x84b13e6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc813e71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d013e75, 98 },
    { 0, 0 },
    { 0x25a13e77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51713e7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0113e85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc513e89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45713e8d, 68 },
    { 0x70513e8e, 98 },
    { 0xdbb13e8e, 56 },
    { 0x30813e90, 61 },
    { 0x65613e90, 33 },
    { 0xee513e92, 55 },
    { 0xea313e93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d113e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e13e9c, 23 },
    { 0xd5713e9c, 90 },
    { 0, 0 },
    { 0x3f013e9f, 61 },
    { 0x5a413e9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e13ea5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d113ea8, 100 },
    { 0x80413ea9, 78 },
    { 0x68a13eaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6113eaf, 61 },
    { 0, 0 },
    { 0x73613eb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35513eb9, 91 },
    { 0x17d13eba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42613ebd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e613ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13ec5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a013eca, 90 },
    { 0x7c13eca, 76 },
    { 0x75e13ecc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1713ed4, 95 },
    { 0x2c813ed5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6513ede, 43 },
    { 0, 0 },
    { 0xfd13ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85213ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c13ee6, 98 },
    { 0xaff13ee7, 90 },
    { 0x87f13ee6, 55 },
    { 0x59313ee9, 61 },
    { 0x7f313ee6, 66 },
    { 0x8b913eeb, 99 },
    { 0x6f713ee9, 89 },
    { 0, 0 },
    { 0xf2c13eee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8913ef1, 88 },
    { 0xde213ef2, 90 },
    { 0x79113ef2, 88 },
    { 0x7d713ef4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c413ef8, 95 },
    { 0, 0 },
    { 0xac113efa, 99 },
    { 0xfc113efb, 61 },
    { 0xeb913efa, 34 },
    { 0, 0 },
    { 0xd5813efe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80313f02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e13f06, 34 },
    { 0x52113f06, 33 },
    { 0xb2013f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c13f12, 66 },
    { 0, 0 },
    { 0x2513f14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c13f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a13f24, 46 },
    { 0x5c113f24, 75 },
    { 0, 0 },
    { 0x2d413f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36113f2a, 90 },
    { 0xc5a13f2b, 90 },
    { 0x5c413f2c, 95 },
    { 0, 0 },
    { 0xa413f2e, 61 },
    { 0xd1413f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8113f33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51313f39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e613f3c, 90 },
    { 0xd2313f3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29213f42, 76 },
    { 0, 0 },
    { 0x15f13f44, 33 },
    { 0, 0 },
    { 0xff213f46, 84 },
    { 0x91013f47, 95 },
    { 0x2113f48, 17 },
    { 0x3f613f48, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4013f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9113f4f, 33 },
    { 0x3aa13f50, 34 },
    { 0x3ac13f51, 61 },
    { 0x7b613f51, 95 },
    { 0, 0 },
    { 0xbaa13f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613f58, 80 },
    { 0x78013f58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c513f5f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d13f62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22413f65, 91 },
    { 0, 0 },
    { 0x8db13f67, 95 },
    { 0x7f713f68, 76 },
    { 0x4cf13f68, 61 },
    { 0xb2113f6a, 33 },
    { 0x9ee13f6b, 33 },
    { 0x62f13f69, 65 },
    { 0x99513f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d13f72, 33 },
    { 0xd2913f72, 61 },
    { 0, 0 },
    { 0xe7c13f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7013f79, 84 },
    { 0x2ea13f7a, 78 },
    { 0, 0 },
    { 0x54513f7c, 33 },
    { 0x2e713f7d, 46 },
    { 0xe9913f7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e13f81, 76 },
    { 0xae13f82, 100 },
    { 0xc1013f82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77813f88, 65 },
    { 0x8bf13f89, 17 },
    { 0x21d13f8a, 61 },
    { 0x75013f8b, 46 },
    { 0x44f13f8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44213f90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf413f95, 59 },
    { 0, 0 },
    { 0xf6b13f97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed13f9c, 55 },
    { 0x53413f9d, 93 },
    { 0x36313f9e, 84 },
    { 0xd4a13f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19213fa2, 61 },
    { 0, 0 },
    { 0xfa613fa4, 90 },
    { 0x9b713fa5, 33 },
    { 0xfe513fa4, 61 },
    { 0, 0 },
    { 0xa7a13fa8, 52 },
    { 0x9dc13fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea913fac, 46 },
    { 0, 0 },
    { 0x37413fae, 2 },
    { 0, 0 },
    { 0x50213fb0, 33 },
    { 0x57f13fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c13fb5, 61 },
    { 0, 0 },
    { 0x54d13fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38013fbf, 61 },
    { 0, 0 },
    { 0x1313fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5713fc5, 90 },
    { 0xde213fc6, 61 },
    { 0xe2c13fc6, 95 },
    { 0xd8213fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5113fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c13fd2, 77 },
    { 0xa5713fd3, 98 },
    { 0, 0 },
    { 0x3c13fd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf313fdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d013fe7, 78 },
    { 0xe9213fe8, 90 },
    { 0xf5113fe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c13fec, 61 },
    { 0xfba13fec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc513ff0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75113ffc, 76 },
    { 0xe5113ffd, 34 },
    { 0x98613ffe, 23 },
    { 0, 0 },
    { 0x81714000, 99 },
    { 0, 0 },
    { 0xae714002, 93 },
    { 0x30414003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81314006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe14009, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc21400d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71714010, 48 },
    { 0, 0 },
    { 0x9d614012, 95 },
    { 0, 0 },
    { 0x21a14014, 98 },
    { 0x81514015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe14019, 61 },
    { 0xb311401a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e14021, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c714024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a814027, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc71402b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a1402e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e14031, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac314034, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34214037, 95 },
    { 0x16414038, 90 },
    { 0xaae14038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd1403c, 84 },
    { 0, 0 },
    { 0xc601403e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88714047, 66 },
    { 0x95a14048, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b71404c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c714050, 75 },
    { 0x11514051, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef014056, 90 },
    { 0, 0 },
    { 0xd4a14058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8681405b, 93 },
    { 0xdb11405c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd14061, 90 },
    { 0, 0 },
    { 0xf6c14063, 17 },
    { 0x34d14064, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a1406d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe314071, 23 },
    { 0x95414071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90914078, 90 },
    { 0, 0 },
    { 0x6cd1407a, 84 },
    { 0, 0 },
    { 0x2031407c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b14082, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111408c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34414091, 76 },
    { 0, 0 },
    { 0xba714093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb14097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1409b, 90 },
    { 0x9e41409b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191409f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935140a3, 93 },
    { 0, 0 },
    { 0x812140a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09140aa, 23 },
    { 0, 0 },
    { 0x63a140ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40140b0, 33 },
    { 0xbf1140b0, 17 },
    { 0, 0 },
    { 0x517140b3, 95 },
    { 0xab1140b3, 95 },
    { 0, 0 },
    { 0x840140b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10140ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326140bf, 66 },
    { 0xad7140c0, 90 },
    { 0xd00140c1, 61 },
    { 0xe54140bf, 43 },
    { 0, 0 },
    { 0x4dc140c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee140c8, 23 },
    { 0x573140c9, 100 },
    { 0x3c6140ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863140cf, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24140d6, 33 },
    { 0, 0 },
    { 0x21c140d8, 69 },
    { 0xa8140d9, 91 },
    { 0xe44140da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf140dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa140e4, 23 },
    { 0, 0 },
    { 0x214140e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4140ea, 61 },
    { 0x9a6140ea, 90 },
    { 0xac8140ec, 98 },
    { 0xd17140eb, 95 },
    { 0x8b1140ee, 98 },
    { 0x9c9140ef, 98 },
    { 0xee6140ec, 61 },
    { 0xdfb140f1, 100 },
    { 0xdc2140f2, 33 },
    { 0x3b140f1, 95 },
    { 0x8ab140ed, 91 },
    { 0, 0 },
    { 0x8c140f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e140fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc461410c, 90 },
    { 0x57f1410d, 100 },
    { 0xc271410e, 95 },
    { 0x8e31410f, 61 },
    { 0xb181410e, 33 },
    { 0xa0914111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2214115, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51411b, 33 },
    { 0, 0 },
    { 0x56c1411d, 43 },
    { 0x2d31411e, 95 },
    { 0xbcf1411e, 33 },
    { 0x59f14120, 90 },
    { 0, 0 },
    { 0x7f814122, 46 },
    { 0, 0 },
    { 0x78214124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5514127, 84 },
    { 0x94014128, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b01412d, 90 },
    { 0xc0b1412e, 91 },
    { 0x3001412f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21413a, 100 },
    { 0xcd11413b, 33 },
    { 0x47f1413b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8114143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91414b, 65 },
    { 0x4ef1414c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2931414f, 90 },
    { 0, 0 },
    { 0x18f14151, 76 },
    { 0, 0 },
    { 0x32414153, 23 },
    { 0, 0 },
    { 0x9414155, 61 },
    { 0x9b414155, 43 },
    { 0xde314157, 78 },
    { 0x7814158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771415b, 33 },
    { 0x9941415c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c14161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab014166, 23 },
    { 0x17e14167, 84 },
    { 0xaf814168, 33 },
    { 0x73f14169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8791416c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e21416f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19414172, 23 },
    { 0, 0 },
    { 0x77b14174, 85 },
    { 0xfe14174, 23 },
    { 0xb8914174, 43 },
    { 0x31d14174, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d14180, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f14187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e11418a, 61 },
    { 0, 0 },
    { 0xb9c1418c, 46 },
    { 0, 0 },
    { 0xb511418e, 33 },
    { 0x3e61418f, 55 },
    { 0xd6914190, 33 },
    { 0x8c71418f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a14196, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a914199, 90 },
    { 0xc171419a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7141a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6141a5, 76 },
    { 0x4b0141a6, 82 },
    { 0xe4b141a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb141ab, 61 },
    { 0xea0141ac, 90 },
    { 0x329141ad, 98 },
    { 0x59a141ad, 55 },
    { 0x23f141af, 65 },
    { 0x829141af, 80 },
    { 0, 0 },
    { 0x4ed141b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x33141b5, 34 },
    { 0xd70141b5, 90 },
    { 0x4a141b6, 43 },
    { 0x5b4141b8, 61 },
    { 0xb3c141b8, 33 },
    { 0x9c6141ba, 98 },
    { 0xf14141bb, 88 },
    { 0x6af141bc, 86 },
    { 0xfba141bc, 23 },
    { 0x20d141be, 67 },
    { 0x33f141b5, 61 },
    { 0xa06141b8, 99 },
    { 0, 0 },
    { 0x5ef141c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c141c5, 96 },
    { 0x113141c6, 98 },
    { 0xffc141c7, 96 },
    { 0, 0 },
    { 0xde4141c9, 98 },
    { 0xd30141c9, 34 },
    { 0, 0 },
    { 0xf51141cc, 67 },
    { 0, 0 },
    { 0xd7f141ce, 55 },
    { 0x9cc141ce, 93 },
    { 0x2bc141ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba141db, 96 },
    { 0x7e0141db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd141df, 93 },
    { 0, 0 },
    { 0xe2a141e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c141eb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504141f6, 75 },
    { 0x836141f7, 61 },
    { 0xd61141f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a141fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb1420b, 90 },
    { 0, 0 },
    { 0x1d81420d, 98 },
    { 0xa091420e, 90 },
    { 0x3271420d, 55 },
    { 0, 0 },
    { 0xeb214211, 23 },
    { 0x8014212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79014222, 67 },
    { 0xfce14223, 91 },
    { 0, 0 },
    { 0xb0a14225, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe14229, 61 },
    { 0, 0 },
    { 0x7081422b, 61 },
    { 0x1a71422c, 55 },
    { 0xf2c1422d, 98 },
    { 0xc231422d, 33 },
    { 0x63a1422f, 93 },
    { 0xe0414230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a14234, 61 },
    { 0xa8214235, 95 },
    { 0x32b14236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5114239, 24 },
    { 0x5d81423a, 34 },
    { 0x58c14239, 98 },
    { 0x341423c, 23 },
    { 0x43d1423d, 43 },
    { 0, 0 },
    { 0xf0d1423f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40414245, 33 },
    { 0, 0 },
    { 0x5c714247, 23 },
    { 0xad014247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71424e, 69 },
    { 0x1701424f, 98 },
    { 0xe2214250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5114253, 34 },
    { 0, 0 },
    { 0x37714255, 61 },
    { 0, 0 },
    { 0xb6514257, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb1425d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2714267, 81 },
    { 0xa8e14268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd61426c, 80 },
    { 0xb871426d, 91 },
    { 0x5d41426e, 75 },
    { 0, 0 },
    { 0x19614270, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc14273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1014278, 80 },
    { 0x96f14278, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c14281, 90 },
    { 0, 0 },
    { 0xfef14283, 55 },
    { 0x57614284, 95 },
    { 0xe4714285, 61 },
    { 0xbd814286, 61 },
    { 0x26a14284, 98 },
    { 0x91914288, 34 },
    { 0xa7f14283, 91 },
    { 0x16b14284, 55 },
    { 0x2531428b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b1428f, 85 },
    { 0x5da1428f, 61 },
    { 0x64714291, 98 },
    { 0x6c614291, 61 },
    { 0x2d814293, 95 },
    { 0, 0 },
    { 0x3e014295, 88 },
    { 0x47414295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81429b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f142a0, 66 },
    { 0x94a142a0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975142a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a142ad, 55 },
    { 0x18d142ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b142b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab142ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51142bf, 33 },
    { 0x751142c0, 78 },
    { 0xac142bf, 66 },
    { 0, 0 },
    { 0x855142c3, 78 },
    { 0, 0 },
    { 0xf7f142c5, 66 },
    { 0x2c142c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739142cb, 76 },
    { 0xd17142cc, 33 },
    { 0x75e142cd, 93 },
    { 0, 0 },
    { 0x24b142cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f142d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda142d9, 90 },
    { 0x828142da, 76 },
    { 0x748142db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67142e2, 95 },
    { 0x6cc142e3, 98 },
    { 0x3d8142e2, 33 },
    { 0xc90142e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585142eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613142f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a142f9, 34 },
    { 0x236142f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb142ff, 34 },
    { 0x9db142ff, 33 },
    { 0x8cf14301, 65 },
    { 0, 0 },
    { 0x48714303, 100 },
    { 0, 0 },
    { 0xdb114305, 34 },
    { 0x6a414306, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f14309, 98 },
    { 0xda114309, 23 },
    { 0xac21430b, 33 },
    { 0xd5a1430c, 17 },
    { 0xb431430d, 98 },
    { 0xdd21430e, 98 },
    { 0x4601430b, 67 },
    { 0, 0 },
    { 0x8d814311, 33 },
    { 0x76314312, 33 },
    { 0, 0 },
    { 0x58c14314, 33 },
    { 0x1f514315, 33 },
    { 0, 0 },
    { 0xcdd14317, 66 },
    { 0x4614318, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb791431b, 61 },
    { 0, 0 },
    { 0xf0f1431d, 90 },
    { 0x98c1431e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9314322, 98 },
    { 0xb4614323, 33 },
    { 0xf914323, 98 },
    { 0, 0 },
    { 0x12714326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac71432b, 76 },
    { 0x66b1432b, 61 },
    { 0, 0 },
    { 0xef91432e, 68 },
    { 0x7901432e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67714333, 88 },
    { 0x73d14334, 33 },
    { 0x35614335, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc311433b, 34 },
    { 0x12d1433c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5414340, 23 },
    { 0x52514341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24614345, 34 },
    { 0xaa414346, 98 },
    { 0xde314347, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf01434b, 90 },
    { 0x9ec1434c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd14350, 23 },
    { 0xe1114351, 33 },
    { 0xb1d14350, 91 },
    { 0x5c314353, 61 },
    { 0x6fc14354, 87 },
    { 0x7b814353, 87 },
    { 0xc4b14356, 100 },
    { 0x23314357, 23 },
    { 0xc0a14353, 23 },
    { 0xe4014351, 34 },
    { 0x35114350, 95 },
    { 0x61f14350, 95 },
    { 0xadd14353, 46 },
    { 0xeb31435d, 33 },
    { 0xa141435d, 61 },
    { 0xde31435f, 90 },
    { 0x3ce14360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24814363, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3714368, 90 },
    { 0, 0 },
    { 0xe01436a, 99 },
    { 0, 0 },
    { 0xd5a1436c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb014371, 61 },
    { 0, 0 },
    { 0x90f14373, 90 },
    { 0xe2714374, 34 },
    { 0x2be14373, 17 },
    { 0x23414376, 87 },
    { 0, 0 },
    { 0xf1d14378, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c1437c, 95 },
    { 0xcef1437d, 95 },
    { 0x8311437e, 78 },
    { 0, 0 },
    { 0xc514380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21714383, 95 },
    { 0, 0 },
    { 0x1bb14385, 33 },
    { 0x1ec14386, 69 },
    { 0x40814385, 91 },
    { 0x8da14388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf21438b, 66 },
    { 0xd2f1438c, 40 },
    { 0x66a1438d, 61 },
    { 0xa1f1438d, 66 },
    { 0xf4e1438f, 46 },
    { 0xdd214390, 23 },
    { 0xdb314390, 98 },
    { 0x5a71438f, 33 },
    { 0x22714393, 61 },
    { 0x3ec14393, 76 },
    { 0x8d514394, 98 },
    { 0xd0914392, 85 },
    { 0xa4a1438e, 84 },
    { 0xd0714390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1439f, 76 },
    { 0x82f1439f, 33 },
    { 0xc01143a1, 96 },
    { 0, 0 },
    { 0xb96143a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790143ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b143b7, 90 },
    { 0, 0 },
    { 0x76c143b9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb143bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad143c2, 33 },
    { 0xf44143c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750143c6, 95 },
    { 0x4d5143c6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e143ca, 99 },
    { 0x77c143cb, 61 },
    { 0xbd143cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x784143cf, 34 },
    { 0x532143cf, 90 },
    { 0, 0 },
    { 0x3b6143d2, 90 },
    { 0, 0 },
    { 0xedf143d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96143da, 55 },
    { 0, 0 },
    { 0x693143dc, 33 },
    { 0x157143dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3143e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7143e9, 98 },
    { 0x3f5143ea, 78 },
    { 0, 0 },
    { 0x607143ec, 98 },
    { 0x46b143ec, 98 },
    { 0xd0f143ee, 52 },
    { 0xdf9143ef, 88 },
    { 0x257143f0, 90 },
    { 0x483143f1, 93 },
    { 0xc9c143f2, 90 },
    { 0x6e7143f3, 98 },
    { 0xad5143f1, 26 },
    { 0xbe5143f5, 48 },
    { 0xf87143f6, 90 },
    { 0xda5143f7, 90 },
    { 0x89a143f8, 23 },
    { 0x404143f1, 90 },
    { 0xafb143fa, 34 },
    { 0x576143fb, 95 },
    { 0, 0 },
    { 0x666143fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b14402, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f814406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821440a, 98 },
    { 0, 0 },
    { 0x2c91440c, 95 },
    { 0x77f1440d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d14414, 90 },
    { 0, 0 },
    { 0xa114416, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x52014419, 78 },
    { 0x6991441a, 23 },
    { 0xf341441b, 93 },
    { 0, 0 },
    { 0x74a1441d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35914421, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0x714424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b814427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071442a, 42 },
    { 0, 0 },
    { 0x7d31442c, 90 },
    { 0x8a91442c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a114432, 98 },
    { 0xd5214433, 61 },
    { 0, 0 },
    { 0x6914435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d1443c, 34 },
    { 0, 0 },
    { 0x73d1443e, 95 },
    { 0xad11443f, 98 },
    { 0x9cf14440, 23 },
    { 0xdbd14441, 95 },
    { 0xef714440, 98 },
    { 0x11614443, 61 },
    { 0xf014443, 23 },
    { 0, 0 },
    { 0x63214446, 78 },
    { 0xe714447, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b1444b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5914450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae514457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa801445c, 61 },
    { 0, 0 },
    { 0xd6a1445e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa014466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4171446b, 61 },
    { 0xe8f1446c, 61 },
    { 0x6961446c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc14470, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e14473, 94 },
    { 0xe214473, 61 },
    { 0xd7414473, 76 },
    { 0x3d214476, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf14479, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf31447f, 33 },
    { 0xd0f14480, 55 },
    { 0x62614480, 92 },
    { 0xd2e14480, 34 },
    { 0x76f14483, 61 },
    { 0x3c714482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd14489, 85 },
    { 0xa651448a, 99 },
    { 0x6e71448b, 26 },
    { 0x8fd1448c, 38 },
    { 0xec41448d, 90 },
    { 0xded1448e, 23 },
    { 0x2f1448e, 33 },
    { 0xef614490, 90 },
    { 0xf714489, 95 },
    { 0x6d514489, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc014495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011449a, 46 },
    { 0xe31449b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a144aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0144b3, 76 },
    { 0xdae144b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3144bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c144c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd144cd, 88 },
    { 0x62b144ce, 95 },
    { 0xe1f144ce, 90 },
    { 0x93b144ce, 43 },
    { 0xc1f144d1, 76 },
    { 0x8bc144d2, 100 },
    { 0x21c144d2, 98 },
    { 0x860144d4, 43 },
    { 0x464144d5, 33 },
    { 0x413144cd, 76 },
    { 0x7ec144d6, 95 },
    { 0, 0 },
    { 0x81f144d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae144e0, 99 },
    { 0x4fc144e0, 93 },
    { 0xcce144e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8144e6, 81 },
    { 0, 0 },
    { 0x22c144e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db144eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee144ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49144f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a144f7, 85 },
    { 0xbd6144f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e214504, 61 },
    { 0xbe414505, 36 },
    { 0xbf14505, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51450a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd814511, 17 },
    { 0, 0 },
    { 0xbdf14513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe141451b, 23 },
    { 0, 0 },
    { 0x8e61451d, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x75014520, 90 },
    { 0x9ca14521, 90 },
    { 0xdaa14521, 61 },
    { 0x80514522, 55 },
    { 0x6f314524, 76 },
    { 0xb5414524, 98 },
    { 0xa3714526, 81 },
    { 0x56914520, 34 },
    { 0xcbb14526, 61 },
    { 0, 0 },
    { 0x4551452a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1452d, 23 },
    { 0xc491452e, 90 },
    { 0xa361452f, 55 },
    { 0, 0 },
    { 0xaa314531, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a414535, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb071453b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5414544, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52314548, 67 },
    { 0, 0 },
    { 0x5641454a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3314550, 55 },
    { 0x2fd14551, 23 },
    { 0xdfd14551, 61 },
    { 0x7e414553, 98 },
    { 0x3ba14553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd001455b, 98 },
    { 0x1c51455c, 33 },
    { 0xf5f1455d, 98 },
    { 0x8591455d, 61 },
    { 0x74c1455f, 40 },
    { 0xe081455f, 90 },
    { 0x99b1455c, 99 },
    { 0x74914561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f14565, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b14568, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1456f, 76 },
    { 0, 0 },
    { 0xac814571, 61 },
    { 0, 0 },
    { 0xae514573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92114576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b914579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1457d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e14583, 33 },
    { 0, 0 },
    { 0xc1314585, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a21458a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33114593, 33 },
    { 0x6a914593, 35 },
    { 0, 0 },
    { 0x91714596, 61 },
    { 0xc5b14596, 23 },
    { 0xf8a14598, 93 },
    { 0xf9214596, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd31459e, 52 },
    { 0, 0 },
    { 0xf62145a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce145a9, 88 },
    { 0xca1145aa, 90 },
    { 0x872145ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124145af, 34 },
    { 0x22a145b0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55145b6, 33 },
    { 0x1b9145b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e145be, 33 },
    { 0x84c145bf, 90 },
    { 0x941145be, 61 },
    { 0, 0 },
    { 0xa33145c2, 55 },
    { 0x1c0145c3, 76 },
    { 0x8c1145c2, 95 },
    { 0x336145c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b145c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a145cb, 75 },
    { 0, 0 },
    { 0x894145cd, 76 },
    { 0, 0 },
    { 0x3c1145cf, 43 },
    { 0x68145d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c145d3, 97 },
    { 0x497145d4, 61 },
    { 0x5d8145d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79145d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72145df, 84 },
    { 0, 0 },
    { 0xad0145e1, 52 },
    { 0x292145e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3145e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a145ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc145ef, 76 },
    { 0, 0 },
    { 0x859145f1, 34 },
    { 0xe38145f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97145f5, 25 },
    { 0x1a145f6, 87 },
    { 0x6bc145f7, 90 },
    { 0x64a145f8, 95 },
    { 0xbab145f9, 93 },
    { 0xb10145fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a14601, 98 },
    { 0x48414602, 33 },
    { 0, 0 },
    { 0x4b14604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96314609, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98314612, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6614615, 90 },
    { 0xa3814615, 46 },
    { 0, 0 },
    { 0xe0014618, 55 },
    { 0xcfb14619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7851461c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d14623, 43 },
    { 0xfea14624, 96 },
    { 0x32414623, 84 },
    { 0x1814624, 99 },
    { 0x46614627, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1462a, 90 },
    { 0, 0 },
    { 0x4971462c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e14632, 43 },
    { 0xcd314633, 33 },
    { 0x38714634, 34 },
    { 0x6c214634, 23 },
    { 0xe4214636, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x18514639, 46 },
    { 0, 0 },
    { 0x1ea1463b, 91 },
    { 0x3541463c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2114644, 33 },
    { 0x5a914644, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4014652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f14655, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1465e, 81 },
    { 0, 0 },
    { 0x21214660, 90 },
    { 0x9cc14661, 98 },
    { 0x83d14660, 23 },
    { 0x3df14661, 38 },
    { 0x56314661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e14668, 91 },
    { 0x67914668, 98 },
    { 0x1661466a, 33 },
    { 0xe741466b, 88 },
    { 0xc11466b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de1466f, 23 },
    { 0x241466f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f14679, 65 },
    { 0xb741467a, 95 },
    { 0, 0 },
    { 0x7b21467c, 87 },
    { 0x5d51467d, 98 },
    { 0xadd1467c, 23 },
    { 0, 0 },
    { 0xc5914680, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b114684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1468b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51468f, 61 },
    { 0x4ec14690, 95 },
    { 0x2a914690, 55 },
    { 0x8f114692, 95 },
    { 0x76914691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe14698, 24 },
    { 0x4ec14699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6061469c, 33 },
    { 0, 0 },
    { 0x8891469e, 84 },
    { 0, 0 },
    { 0x52b146a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4146a3, 33 },
    { 0x414146a4, 33 },
    { 0xb5a146a3, 80 },
    { 0x968146a6, 55 },
    { 0xb7c146a5, 55 },
    { 0x773146a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f146b0, 53 },
    { 0x411146b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3146b9, 93 },
    { 0, 0 },
    { 0x4f146bb, 95 },
    { 0x251146bc, 95 },
    { 0x846146bd, 23 },
    { 0xce7146be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf146c1, 80 },
    { 0x15f146c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9146c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea146d8, 98 },
    { 0x465146d9, 23 },
    { 0x36b146da, 61 },
    { 0x529146da, 33 },
    { 0x9a5146d8, 98 },
    { 0x2f3146d8, 23 },
    { 0xcf3146dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a146e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6146f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5146fe, 61 },
    { 0x3de146ff, 17 },
    { 0x5ba14700, 93 },
    { 0, 0 },
    { 0xc2914702, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14705, 34 },
    { 0x1ee14705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78214709, 61 },
    { 0x4371470a, 23 },
    { 0x28d1470b, 83 },
    { 0, 0 },
    { 0xc8f1470d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc214711, 23 },
    { 0xfcc14712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f14718, 81 },
    { 0xea514719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b1471e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47114727, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901472c, 33 },
    { 0x38b1472d, 100 },
    { 0xd091472e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa14734, 46 },
    { 0x41714735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b14739, 66 },
    { 0xa981473a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8671473d, 33 },
    { 0x4d51473e, 98 },
    { 0, 0 },
    { 0x47014740, 61 },
    { 0x2814741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb14746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7051475a, 98 },
    { 0, 0 },
    { 0xfad1475c, 93 },
    { 0x1221475c, 23 },
    { 0x61a1475c, 33 },
    { 0x6ba1475f, 26 },
    { 0x2de1475f, 68 },
    { 0xfa51475f, 90 },
    { 0, 0 },
    { 0x45214763, 61 },
    { 0x15914764, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87114769, 43 },
    { 0x75d14769, 84 },
    { 0, 0 },
    { 0xf691476c, 43 },
    { 0xa811476d, 33 },
    { 0xc3e1476d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b14775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1477b, 100 },
    { 0, 0 },
    { 0xc3a1477d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47014781, 90 },
    { 0x89114782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c14787, 69 },
    { 0xe1d14788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea1479c, 55 },
    { 0xa801479c, 76 },
    { 0x7b71479e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0147a1, 76 },
    { 0x783147a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a147a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb147a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27147ad, 55 },
    { 0xa2c147ae, 98 },
    { 0, 0 },
    { 0xc80147b0, 46 },
    { 0x36d147b1, 81 },
    { 0x78a147b2, 90 },
    { 0x41b147b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e147b6, 93 },
    { 0x66f147b7, 55 },
    { 0xf2d147b8, 33 },
    { 0xc01147b9, 98 },
    { 0, 0 },
    { 0xcb7147bb, 61 },
    { 0, 0 },
    { 0xe84147bd, 90 },
    { 0xd95147bd, 90 },
    { 0x6f2147bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0147c4, 98 },
    { 0xd7a147c5, 84 },
    { 0x8f6147c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351147cf, 33 },
    { 0xb62147d0, 46 },
    { 0x77d147d1, 52 },
    { 0x79e147d1, 90 },
    { 0x7fa147cf, 43 },
    { 0x9b2147d4, 46 },
    { 0xd8a147d3, 66 },
    { 0x312147d5, 90 },
    { 0x6e147d7, 98 },
    { 0x6c9147d8, 33 },
    { 0xe03147d9, 100 },
    { 0, 0 },
    { 0x65a147db, 61 },
    { 0x706147db, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09147e2, 78 },
    { 0x162147e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f147e9, 33 },
    { 0xd98147ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1147ee, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7147f3, 34 },
    { 0, 0 },
    { 0x3a7147f5, 95 },
    { 0x802147f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6147fa, 93 },
    { 0x13e147fb, 95 },
    { 0x4a6147fc, 33 },
    { 0x32b147fd, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14800, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6414803, 23 },
    { 0xd9014803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8014809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac714810, 87 },
    { 0xb7914810, 23 },
    { 0x57514810, 23 },
    { 0x36c14813, 67 },
    { 0, 0 },
    { 0x55214815, 34 },
    { 0xd3314816, 43 },
    { 0xcc714816, 23 },
    { 0x70714816, 43 },
    { 0, 0 },
    { 0x35f1481a, 76 },
    { 0x7d81481b, 78 },
    { 0xd2c1481c, 90 },
    { 0x9bd1481b, 95 },
    { 0, 0 },
    { 0xf8c1481f, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c14822, 61 },
    { 0x3b214823, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe451482a, 95 },
    { 0xaf21482b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d91482f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1483b, 43 },
    { 0, 0 },
    { 0x8f71483d, 91 },
    { 0x72f1483e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b714841, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4914847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb21484a, 33 },
    { 0xe821484b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa501484f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e14855, 99 },
    { 0xc2214856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d1485a, 61 },
    { 0, 0 },
    { 0x1c41485c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18114861, 34 },
    { 0x7f014862, 90 },
    { 0, 0 },
    { 0x91114864, 76 },
    { 0x25514864, 61 },
    { 0xc4c14865, 90 },
    { 0x34a14867, 55 },
    { 0xfaf14867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa1486b, 33 },
    { 0x21e1486c, 95 },
    { 0x1d41486b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e14873, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e14877, 93 },
    { 0, 0 },
    { 0x19414879, 66 },
    { 0x3a41487a, 95 },
    { 0, 0 },
    { 0xf931487c, 84 },
    { 0x4491487d, 90 },
    { 0xc7d1487e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x90114881, 61 },
    { 0x26114881, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1488f, 46 },
    { 0xcad1488f, 61 },
    { 0xdc514891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a14899, 33 },
    { 0, 0 },
    { 0x6831489b, 97 },
    { 0, 0 },
    { 0x15d1489d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f148a0, 76 },
    { 0x139148a0, 85 },
    { 0x2f4148a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6148a7, 90 },
    { 0, 0 },
    { 0x9bc148a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1148ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2148b6, 90 },
    { 0, 0 },
    { 0x59c148b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2148bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x972148be, 98 },
    { 0x608148bf, 61 },
    { 0x683148c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x180148c3, 52 },
    { 0x537148c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc148c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d148d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61148d8, 83 },
    { 0x8e9148d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d148dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c148e2, 34 },
    { 0x759148e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f148e7, 61 },
    { 0, 0 },
    { 0xd28148e9, 85 },
    { 0x3be148ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa148ed, 61 },
    { 0x197148ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00148f4, 43 },
    { 0, 0 },
    { 0xc91148f6, 85 },
    { 0xb81148f7, 98 },
    { 0, 0 },
    { 0x420148f9, 66 },
    { 0xf53148fa, 90 },
    { 0x50c148fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43148fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc214901, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad214905, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61490f, 61 },
    { 0x8ac14910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf014916, 84 },
    { 0xd0014917, 23 },
    { 0, 0 },
    { 0xe7014919, 90 },
    { 0x36514919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d14920, 90 },
    { 0xbf14921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7714927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451492d, 46 },
    { 0xe9f1492e, 98 },
    { 0x6351492f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca414933, 98 },
    { 0x56f14934, 98 },
    { 0x54514935, 65 },
    { 0xdac14936, 95 },
    { 0xcac14933, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc1493c, 33 },
    { 0xfb71493d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95114941, 23 },
    { 0, 0 },
    { 0x1be14943, 61 },
    { 0x38814943, 90 },
    { 0x4cb14944, 33 },
    { 0x60a14945, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf941494e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc14951, 33 },
    { 0x5b414952, 33 },
    { 0xc8114952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db1495f, 95 },
    { 0x9e714960, 88 },
    { 0x39714961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571496c, 98 },
    { 0x3801496d, 93 },
    { 0x1621496d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f14972, 87 },
    { 0x21614973, 23 },
    { 0x43714974, 33 },
    { 0x2d914975, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3691497d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d14983, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75014988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd21498c, 33 },
    { 0, 0 },
    { 0x201498e, 90 },
    { 0x411498f, 61 },
    { 0, 0 },
    { 0xdbc14991, 90 },
    { 0x3c214991, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c14995, 95 },
    { 0, 0 },
    { 0xc1e14997, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd851499a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611499e, 100 },
    { 0x2911499f, 76 },
    { 0x46b149a0, 77 },
    { 0x90a149a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62149a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593149ad, 90 },
    { 0xffb149ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f149b4, 100 },
    { 0, 0 },
    { 0xe5c149b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46149bb, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5149c5, 98 },
    { 0x626149c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7149c9, 90 },
    { 0x16149ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769149d0, 61 },
    { 0, 0 },
    { 0x555149d2, 61 },
    { 0, 0 },
    { 0xf6a149d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977149d8, 33 },
    { 0xe42149d9, 93 },
    { 0, 0 },
    { 0x10f149db, 33 },
    { 0, 0 },
    { 0x49149dd, 46 },
    { 0x219149de, 93 },
    { 0x3e1149df, 98 },
    { 0xcbd149dd, 23 },
    { 0x4e149e1, 95 },
    { 0xe00149e0, 67 },
    { 0x994149df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395149ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34149ed, 95 },
    { 0, 0 },
    { 0x6ef149ef, 17 },
    { 0, 0 },
    { 0xb149f1, 66 },
    { 0xa72149f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f149f6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da149f9, 66 },
    { 0x83e149f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2149fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d14a05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9114a08, 65 },
    { 0, 0 },
    { 0xf2514a0a, 34 },
    { 0xecd14a0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14a14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c14a18, 43 },
    { 0xc2314a19, 91 },
    { 0, 0 },
    { 0x45d14a1b, 76 },
    { 0, 0 },
    { 0xc2314a1d, 90 },
    { 0xbce14a1e, 100 },
    { 0x8b014a1e, 36 },
    { 0x2d414a1e, 85 },
    { 0x63214a20, 23 },
    { 0x73714a20, 33 },
    { 0x2c14a21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8614a27, 98 },
    { 0, 0 },
    { 0x81814a29, 43 },
    { 0, 0 },
    { 0xf0914a2b, 34 },
    { 0x1a414a2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xade14a2f, 61 },
    { 0, 0 },
    { 0x5fc14a31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b14a34, 34 },
    { 0x14114a34, 23 },
    { 0x5a914a36, 64 },
    { 0x57614a37, 98 },
    { 0x2cb14a38, 23 },
    { 0x9cc14a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4214a3d, 23 },
    { 0xb1814a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd14a42, 87 },
    { 0x4d014a43, 67 },
    { 0, 0 },
    { 0xf0414a45, 84 },
    { 0xd0114a46, 61 },
    { 0, 0 },
    { 0xaa714a48, 90 },
    { 0xc3414a49, 56 },
    { 0, 0 },
    { 0x48e14a4b, 90 },
    { 0x5cd14a4c, 76 },
    { 0x9cb14a4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b014a51, 66 },
    { 0xddf14a52, 61 },
    { 0xe9514a51, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b14a59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8414a65, 98 },
    { 0x68214a66, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b14a6a, 76 },
    { 0x81414a6a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b14a6e, 93 },
    { 0x3ec14a6e, 33 },
    { 0x47714a70, 38 },
    { 0xa2714a70, 69 },
    { 0xdf314a72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f714a75, 40 },
    { 0xc214a76, 33 },
    { 0x22514a77, 33 },
    { 0x1cd14a78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64914a7f, 96 },
    { 0, 0 },
    { 0x29614a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e14a8d, 55 },
    { 0x5d814a8e, 77 },
    { 0x9d114a8f, 61 },
    { 0, 0 },
    { 0x23414a91, 97 },
    { 0x7d714a92, 69 },
    { 0x7af14a91, 33 },
    { 0x61414a94, 34 },
    { 0xb714a95, 52 },
    { 0x6d014a95, 33 },
    { 0, 0 },
    { 0x23514a98, 23 },
    { 0x25e14a98, 33 },
    { 0xfe814a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8514a9d, 90 },
    { 0, 0 },
    { 0x36214a9f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x82114aa2, 55 },
    { 0x8a914aa3, 61 },
    { 0x33414aa2, 43 },
    { 0x12b14aa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead14aaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15514ab2, 61 },
    { 0xf7a14ab2, 100 },
    { 0, 0 },
    { 0xc7a14ab5, 90 },
    { 0xfbc14ab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9614aba, 46 },
    { 0xa9814abb, 95 },
    { 0x7e614abc, 33 },
    { 0x3b714abd, 34 },
    { 0, 0 },
    { 0x4e214abf, 34 },
    { 0, 0 },
    { 0x13214ac1, 55 },
    { 0x64814ac2, 81 },
    { 0, 0 },
    { 0xd6514ac4, 90 },
    { 0x87814ac5, 61 },
    { 0, 0 },
    { 0xf5114ac7, 46 },
    { 0xf4014ac8, 23 },
    { 0x26214ac9, 46 },
    { 0x44c14ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50714ace, 98 },
    { 0, 0 },
    { 0xb9614ad0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50014ad8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e14adc, 33 },
    { 0, 0 },
    { 0xfe114ade, 98 },
    { 0x1814adf, 85 },
    { 0x5f14ae0, 55 },
    { 0x3c014ae0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc814ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b14ae9, 33 },
    { 0x1f14aea, 90 },
    { 0, 0 },
    { 0xc5514aec, 23 },
    { 0x75014aec, 61 },
    { 0, 0 },
    { 0x4ba14aef, 34 },
    { 0, 0 },
    { 0xbbf14af1, 61 },
    { 0x5e914af2, 90 },
    { 0xeb414af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a14af6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e914afa, 61 },
    { 0x80614afb, 87 },
    { 0x18914afc, 100 },
    { 0x92a14afd, 76 },
    { 0xf8214afa, 93 },
    { 0xb1a14aff, 33 },
    { 0x64f14aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f14b04, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f214b08, 76 },
    { 0x4c614b09, 33 },
    { 0xea814b09, 33 },
    { 0x55414b0b, 78 },
    { 0, 0 },
    { 0x74414b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18414b10, 93 },
    { 0xff014b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc14b18, 61 },
    { 0xaf114b19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f14b1c, 83 },
    { 0xd0214b1d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d014b22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb214b25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd414b29, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e14b2c, 33 },
    { 0x13014b2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a14b35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a14b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c14b3b, 75 },
    { 0x34e14b3c, 34 },
    { 0xf3d14b3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c14b44, 95 },
    { 0x4c114b44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10314b4c, 23 },
    { 0, 0 },
    { 0x77a14b4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f14b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd14b5c, 23 },
    { 0xdd414b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7314b61, 93 },
    { 0, 0 },
    { 0xaa514b63, 61 },
    { 0, 0 },
    { 0x4ed14b65, 78 },
    { 0, 0 },
    { 0xffb14b67, 33 },
    { 0xd0914b67, 33 },
    { 0xe0314b67, 93 },
    { 0x30114b6a, 23 },
    { 0xa7514b6a, 76 },
    { 0xd2114b6c, 19 },
    { 0x73614b6c, 17 },
    { 0, 0 },
    { 0xc0614b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe14b72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f14b75, 61 },
    { 0x3a814b76, 93 },
    { 0x6d14b76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c414b7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a14b82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2314b89, 98 },
    { 0x69314b8a, 55 },
    { 0x55514b8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd414b8e, 87 },
    { 0, 0 },
    { 0x4fa14b90, 91 },
    { 0, 0 },
    { 0x60914b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9314b95, 98 },
    { 0xd9814b95, 46 },
    { 0x1bc14b97, 90 },
    { 0xa0814b98, 46 },
    { 0x5fe14b98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51614b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd214ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e314baf, 95 },
    { 0x2f214bb0, 88 },
    { 0x55d14bb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25814bb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8614bbb, 23 },
    { 0x24d14bbc, 33 },
    { 0x6a414bbd, 68 },
    { 0, 0 },
    { 0x4bc14bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a014bc4, 95 },
    { 0, 0 },
    { 0xb6814bc6, 67 },
    { 0x93114bc7, 95 },
    { 0x9e514bc7, 33 },
    { 0x6b414bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f14bcf, 33 },
    { 0, 0 },
    { 0x87314bd1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a14bd7, 90 },
    { 0x72214bd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42314bdc, 23 },
    { 0x8ce14bdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14be0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6414be3, 100 },
    { 0xdea14be3, 76 },
    { 0x29014be5, 43 },
    { 0, 0 },
    { 0x7e014be7, 95 },
    { 0, 0 },
    { 0x43a14be9, 90 },
    { 0x52514bea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3714bef, 82 },
    { 0, 0 },
    { 0xed114bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b14bf8, 90 },
    { 0, 0 },
    { 0x5314bfa, 76 },
    { 0, 0 },
    { 0x2fe14bfc, 78 },
    { 0x5e014bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed14c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf14c03, 55 },
    { 0, 0 },
    { 0x6d314c05, 90 },
    { 0, 0 },
    { 0x67714c07, 93 },
    { 0xf6314c08, 77 },
    { 0x9e014c08, 23 },
    { 0x8c314c09, 55 },
    { 0x8f414c07, 75 },
    { 0x38114c0c, 75 },
    { 0x6214c07, 95 },
    { 0x3114c0e, 90 },
    { 0x8d114c0f, 55 },
    { 0xad614c10, 67 },
    { 0x37f14c11, 43 },
    { 0xab514c12, 55 },
    { 0xa7c14c11, 90 },
    { 0x5b214c0e, 100 },
    { 0x9b414c15, 90 },
    { 0xf1214c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba14c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4114c1d, 43 },
    { 0xf814c1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4014c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a14c2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39514c2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10114c34, 97 },
    { 0xdfa14c34, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e14c38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd14c3c, 78 },
    { 0x5314c3c, 76 },
    { 0, 0 },
    { 0x24114c3f, 98 },
    { 0xf9514c40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc14c45, 99 },
    { 0xf9a14c46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41714c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7314c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb614c57, 76 },
    { 0x26514c58, 90 },
    { 0, 0 },
    { 0x6a114c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3914c5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24314c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f514c65, 98 },
    { 0, 0 },
    { 0x88314c67, 77 },
    { 0, 0 },
    { 0x29a14c69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb314c6e, 55 },
    { 0x9b814c6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b14c72, 23 },
    { 0x13714c72, 87 },
    { 0xb8214c74, 34 },
    { 0xd7414c75, 76 },
    { 0xde614c74, 76 },
    { 0xfcd14c74, 98 },
    { 0x4a014c78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9214c7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e14c7f, 95 },
    { 0xfe614c7f, 34 },
    { 0x38514c80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8714c84, 78 },
    { 0, 0 },
    { 0x83e14c86, 23 },
    { 0x85114c87, 95 },
    { 0x42414c86, 23 },
    { 0x9514c88, 95 },
    { 0, 0 },
    { 0xf9414c8b, 85 },
    { 0x35914c8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f14c94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b14c98, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e14ca4, 68 },
    { 0x23e14ca5, 55 },
    { 0x38b14ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52014cab, 55 },
    { 0xe1e14cac, 89 },
    { 0x38914cad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a914cb0, 87 },
    { 0x26514cb1, 33 },
    { 0xb4414cb0, 23 },
    { 0xfe114cb1, 33 },
    { 0xa4b14cb4, 88 },
    { 0x2ef14cb1, 43 },
    { 0x58614cb6, 55 },
    { 0x3af14cb7, 98 },
    { 0xe4314cb7, 52 },
    { 0, 0 },
    { 0xe0d14cba, 33 },
    { 0xa4014cbb, 53 },
    { 0xae014cbb, 68 },
    { 0x70114cbc, 95 },
    { 0x7f014cbe, 90 },
    { 0x914cbe, 98 },
    { 0x9a114cc0, 43 },
    { 0x7f314cbe, 33 },
    { 0xc614cc2, 43 },
    { 0x57414cc2, 90 },
    { 0xff314cc4, 85 },
    { 0x80014cc5, 33 },
    { 0x48714cc5, 61 },
    { 0xa1c14cc1, 99 },
    { 0xfa314cc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e14cd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd14cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75814ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a14ce9, 88 },
    { 0x66014cea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0014cee, 66 },
    { 0x61b14cee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57214cf4, 95 },
    { 0xd014cf5, 33 },
    { 0x1d814cf6, 90 },
    { 0, 0 },
    { 0x3f914cf8, 52 },
    { 0, 0 },
    { 0xdb214cfa, 95 },
    { 0, 0 },
    { 0x89914cfc, 33 },
    { 0x64214cfd, 98 },
    { 0xec814cfd, 95 },
    { 0x8de14cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc14d02, 43 },
    { 0x14114d02, 75 },
    { 0, 0 },
    { 0xcc114d05, 66 },
    { 0, 0 },
    { 0x47e14d07, 90 },
    { 0x64914d08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6714d0d, 23 },
    { 0x56214d0e, 95 },
    { 0x55914d0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb614d14, 33 },
    { 0x85e14d15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde14d18, 98 },
    { 0x82814d19, 23 },
    { 0, 0 },
    { 0x56f14d1b, 87 },
    { 0x45714d1b, 99 },
    { 0xd7d14d1d, 98 },
    { 0xcb114d1e, 61 },
    { 0, 0 },
    { 0xb6414d20, 52 },
    { 0, 0 },
    { 0x60b14d22, 93 },
    { 0x5ee14d23, 76 },
    { 0x33514d24, 66 },
    { 0xdeb14d24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c14d2a, 66 },
    { 0x40514d2b, 61 },
    { 0x15b14d2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e14d32, 69 },
    { 0, 0 },
    { 0x30c14d34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa14d3a, 87 },
    { 0xd7a14d3b, 23 },
    { 0x4e914d3c, 55 },
    { 0x27514d3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea614d42, 98 },
    { 0, 0 },
    { 0xfc114d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c14d49, 69 },
    { 0x49b14d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e914d4d, 91 },
    { 0, 0 },
    { 0x77014d4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4114d52, 95 },
    { 0, 0 },
    { 0xaa414d54, 43 },
    { 0x4fb14d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48814d58, 95 },
    { 0, 0 },
    { 0xab614d5a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8014d5e, 46 },
    { 0xce414d5f, 95 },
    { 0x47614d60, 76 },
    { 0x5f914d5e, 76 },
    { 0, 0 },
    { 0xca314d63, 46 },
    { 0x4dd14d64, 98 },
    { 0xa0b14d63, 23 },
    { 0xe0514d65, 90 },
    { 0x4814d67, 74 },
    { 0x33e14d67, 90 },
    { 0xe4614d68, 93 },
    { 0xfbe14d63, 95 },
    { 0xa3e14d6a, 76 },
    { 0x3e014d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa514d72, 55 },
    { 0x6f114d72, 33 },
    { 0xc2714d74, 71 },
    { 0x5614d75, 95 },
    { 0x86214d75, 95 },
    { 0x49814d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87514d7a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c14d7e, 90 },
    { 0, 0 },
    { 0x24214d80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78214d86, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e114d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1414d8c, 61 },
    { 0xab214d8d, 90 },
    { 0, 0 },
    { 0x4a814d8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5814d92, 90 },
    { 0, 0 },
    { 0xa7714d94, 33 },
    { 0x6bc14d95, 76 },
    { 0x69d14d96, 93 },
    { 0, 0 },
    { 0xb3914d98, 93 },
    { 0, 0 },
    { 0xa4914d9a, 52 },
    { 0x17214d9a, 55 },
    { 0xccc14d9c, 88 },
    { 0x79214d9d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac14da0, 23 },
    { 0x7114da0, 33 },
    { 0x60514da2, 99 },
    { 0xbf14da3, 23 },
    { 0, 0 },
    { 0xfab14da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa214da8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe314dac, 91 },
    { 0xd1814dac, 95 },
    { 0x8cf14dad, 52 },
    { 0x14014daf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c14db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c314db9, 95 },
    { 0x75514db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e214dbd, 23 },
    { 0xb8514dbe, 95 },
    { 0x1e914dbf, 61 },
    { 0x8c914dbf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95214dc9, 43 },
    { 0x8b114dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e414dcd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb14dd1, 53 },
    { 0, 0 },
    { 0x32814dd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb14dd6, 95 },
    { 0x6bb14dd6, 87 },
    { 0x4c114dd8, 55 },
    { 0, 0 },
    { 0x6a914dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa14ddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d514de3, 36 },
    { 0x97d14de4, 33 },
    { 0x81d14de5, 55 },
    { 0xcdc14de6, 61 },
    { 0x1114de5, 34 },
    { 0x9614de8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c14df0, 90 },
    { 0x77e14df0, 67 },
    { 0x21414df2, 67 },
    { 0xc8214df3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14dfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4314e02, 61 },
    { 0, 0 },
    { 0x4a114e04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a14e07, 99 },
    { 0x4cb14e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8414e0b, 100 },
    { 0x93014e0c, 23 },
    { 0x76f14e0b, 65 },
    { 0x3ad14e0e, 85 },
    { 0x58a14e0c, 90 },
    { 0xfec14e10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9714e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce114e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0014e25, 34 },
    { 0x5ee14e26, 34 },
    { 0xffe14e25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d14e2a, 33 },
    { 0, 0 },
    { 0xac14e2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b414e35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab514e3c, 23 },
    { 0x2b14e3d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf14e42, 55 },
    { 0, 0 },
    { 0xba514e44, 61 },
    { 0xf914e44, 90 },
    { 0x93914e46, 61 },
    { 0xdf114e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec114e4a, 23 },
    { 0x28814e4a, 75 },
    { 0xcdb14e4c, 83 },
    { 0xf9014e4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb414e51, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb714e54, 43 },
    { 0x48614e54, 90 },
    { 0, 0 },
    { 0xc2214e57, 79 },
    { 0xad314e57, 77 },
    { 0xb6314e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e14e5c, 95 },
    { 0, 0 },
    { 0x614e5e, 34 },
    { 0x32414e5e, 23 },
    { 0x2b014e60, 52 },
    { 0, 0 },
    { 0x1fc14e62, 90 },
    { 0, 0 },
    { 0x5214e64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c14e68, 43 },
    { 0, 0 },
    { 0x95414e6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0614e6d, 23 },
    { 0x7cd14e6d, 76 },
    { 0xcc14e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e14e73, 61 },
    { 0, 0 },
    { 0x32b14e75, 95 },
    { 0x44914e76, 81 },
    { 0xd7814e75, 98 },
    { 0x75914e76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53414e7c, 78 },
    { 0xc0e14e7d, 77 },
    { 0x8a814e7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9014e83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d14e8b, 52 },
    { 0, 0 },
    { 0x66814e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf14e94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e14e97, 90 },
    { 0x80d14e98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad14e9c, 61 },
    { 0xce214e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44514ea3, 52 },
    { 0, 0 },
    { 0x28d14ea5, 98 },
    { 0xa8c14ea6, 78 },
    { 0, 0 },
    { 0x68114ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae14ead, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7014eb0, 90 },
    { 0, 0 },
    { 0x9a414eb2, 61 },
    { 0xd2114eb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e14eb9, 97 },
    { 0x1f014eba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73614ebf, 76 },
    { 0x75714ec0, 23 },
    { 0x2914ec1, 90 },
    { 0xfb714ec2, 91 },
    { 0xc5514ec3, 98 },
    { 0x1e514ec3, 91 },
    { 0xd3614ec5, 95 },
    { 0, 0 },
    { 0x20814ec7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf14eca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad414ed5, 61 },
    { 0x93414ed6, 98 },
    { 0xbd14ed6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73414eda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4714edd, 23 },
    { 0, 0 },
    { 0x47814edf, 61 },
    { 0x69114ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b914ee5, 76 },
    { 0x26814ee6, 99 },
    { 0xc6814ee7, 90 },
    { 0, 0 },
    { 0x95914ee9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16414eed, 33 },
    { 0, 0 },
    { 0x5b214eef, 55 },
    { 0xd2314ef0, 98 },
    { 0, 0 },
    { 0x2de14ef2, 88 },
    { 0x9a614ef2, 66 },
    { 0xbf214ef2, 90 },
    { 0, 0 },
    { 0xa0314ef6, 95 },
    { 0xf0714ef7, 33 },
    { 0x64214ef8, 61 },
    { 0, 0 },
    { 0x49114efa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc14f01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9214f04, 43 },
    { 0x8ad14f05, 98 },
    { 0x23314f06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90214f09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22614f0d, 81 },
    { 0xf614f0e, 90 },
    { 0xb3e14f0f, 34 },
    { 0xd6014f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf14f13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a14f1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a14f20, 81 },
    { 0xa1d14f21, 33 },
    { 0x4c114f20, 55 },
    { 0x24e14f23, 90 },
    { 0xf2114f24, 46 },
    { 0, 0 },
    { 0x7cf14f26, 33 },
    { 0x9f514f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe314f2b, 78 },
    { 0x9bc14f2c, 90 },
    { 0xda214f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf014f30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0214f35, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x21514f38, 23 },
    { 0x36a14f39, 23 },
    { 0, 0 },
    { 0x56814f3b, 90 },
    { 0x57814f3c, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d14f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94714f47, 33 },
    { 0xd1414f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59314f52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c214f55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63514f59, 43 },
    { 0, 0 },
    { 0x61314f5b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda14f5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5214f62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7614f6a, 76 },
    { 0x56814f6a, 76 },
    { 0x68a14f6a, 98 },
    { 0, 0 },
    { 0xcf714f6e, 90 },
    { 0xf6714f6f, 91 },
    { 0, 0 },
    { 0xe0114f71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e714f76, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14f79, 93 },
    { 0x18d14f7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda014f7f, 90 },
    { 0, 0 },
    { 0xe0514f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a14f85, 90 },
    { 0, 0 },
    { 0x10b14f87, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5314f8a, 98 },
    { 0, 0 },
    { 0xff114f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74714f91, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14f99, 95 },
    { 0, 0 },
    { 0xf5814f9b, 33 },
    { 0x1dc14f9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad514fa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x37314fa3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a14fa6, 98 },
    { 0xa9e14fa6, 17 },
    { 0x6e614fa7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55014fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54914fb1, 61 },
    { 0xab614fb2, 99 },
    { 0, 0 },
    { 0xa14fb4, 95 },
    { 0x42414fb4, 95 },
    { 0xd5114fb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf214fba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14fc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9514fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58414fca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15314fcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2814fd2, 61 },
    { 0, 0 },
    { 0xe3314fd4, 61 },
    { 0x1fe14fd5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87314fda, 33 },
    { 0, 0 },
    { 0x80514fdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62914fdf, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x70714fe2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b14fe7, 33 },
    { 0, 0 },
    { 0x8fe14fe9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76414fed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51314ff0, 98 },
    { 0x84614ff1, 75 },
    { 0xf8814ff2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa14ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b414ffb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac14ffe, 46 },
    { 0x48214fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f15004, 23 },
    { 0xfc015005, 23 },
    { 0x82f15006, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221500d, 88 },
    { 0, 0 },
    { 0xf7a1500f, 46 },
    { 0x26e15010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e15014, 43 },
    { 0xc2815015, 90 },
    { 0xca315016, 33 },
    { 0x44a15015, 23 },
    { 0, 0 },
    { 0x43d15019, 61 },
    { 0xe961501a, 23 },
    { 0xfa1501a, 76 },
    { 0, 0 },
    { 0x1b91501d, 33 },
    { 0, 0 },
    { 0xb271501f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e015023, 33 },
    { 0x2c215024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47915028, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691502d, 98 },
    { 0xe4c1502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe915031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3415034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa151503f, 23 },
    { 0x1c515040, 33 },
    { 0x6bb15041, 81 },
    { 0, 0 },
    { 0x14215043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f01504a, 61 },
    { 0xc071504b, 90 },
    { 0xaf1504b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c15050, 77 },
    { 0xcfa15051, 55 },
    { 0xa7315052, 91 },
    { 0xde915052, 98 },
    { 0xc8f15054, 87 },
    { 0x97b15052, 95 },
    { 0xfd015052, 95 },
    { 0xcca15057, 93 },
    { 0xc0515058, 23 },
    { 0x58015059, 76 },
    { 0x8f71505a, 67 },
    { 0x2915058, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53515068, 34 },
    { 0xef115069, 46 },
    { 0xa5a1506a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1506f, 61 },
    { 0xb8d15070, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab115074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef15079, 64 },
    { 0xd4d1507a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251507f, 33 },
    { 0x24615080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15083, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30315088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc301508d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde515091, 61 },
    { 0, 0 },
    { 0x27215093, 90 },
    { 0x79015094, 34 },
    { 0xfdc15094, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f615099, 84 },
    { 0xc591509a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671509f, 76 },
    { 0xc4f1509f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e150a5, 90 },
    { 0x586150a6, 61 },
    { 0, 0 },
    { 0xf5e150a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x441150ab, 50 },
    { 0, 0 },
    { 0x493150ad, 68 },
    { 0, 0 },
    { 0x83c150af, 98 },
    { 0xd04150b0, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a150b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x947150b6, 55 },
    { 0x282150b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78150be, 90 },
    { 0xa9b150bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924150c7, 55 },
    { 0x56c150c8, 90 },
    { 0xde6150c7, 95 },
    { 0xc7e150ca, 87 },
    { 0x23b150cb, 34 },
    { 0xbce150cb, 95 },
    { 0xf37150c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b150d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4150dd, 90 },
    { 0xe8b150dd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9150e5, 52 },
    { 0xb5f150e6, 90 },
    { 0x697150e7, 90 },
    { 0xa05150e8, 95 },
    { 0x5bd150e5, 55 },
    { 0x5e8150e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x947150ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x471150f0, 90 },
    { 0xe3f150f1, 33 },
    { 0, 0 },
    { 0xa46150f3, 90 },
    { 0, 0 },
    { 0x6a8150f5, 43 },
    { 0xac8150f6, 23 },
    { 0xd8b150f6, 33 },
    { 0xbc3150f7, 91 },
    { 0x3fe150f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e15105, 95 },
    { 0, 0 },
    { 0xd2415107, 23 },
    { 0xb8815107, 95 },
    { 0x2c015109, 67 },
    { 0x14f15109, 90 },
    { 0, 0 },
    { 0x3111510c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe215110, 61 },
    { 0x4115111, 95 },
    { 0x2cc15112, 76 },
    { 0xa2115113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f15116, 98 },
    { 0xe7215117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd15122, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8615126, 90 },
    { 0, 0 },
    { 0x93215128, 61 },
    { 0, 0 },
    { 0x9091512a, 64 },
    { 0x8e61512b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5615130, 61 },
    { 0x9c115130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac715138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d1513d, 33 },
    { 0, 0 },
    { 0x7481513f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f715142, 43 },
    { 0x24d15143, 33 },
    { 0x7ad15143, 33 },
    { 0xab615144, 33 },
    { 0xeeb15146, 90 },
    { 0xf2815143, 61 },
    { 0x69a15145, 34 },
    { 0x47e15149, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce115150, 55 },
    { 0xe6215150, 33 },
    { 0xad915152, 90 },
    { 0x3c115152, 95 },
    { 0x43715154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd15158, 76 },
    { 0x22015159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1515c, 96 },
    { 0x3e61515d, 91 },
    { 0x3e51515c, 23 },
    { 0xfe01515c, 23 },
    { 0x21415160, 90 },
    { 0xc6615161, 80 },
    { 0x8b015162, 43 },
    { 0x3b81515c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a515168, 93 },
    { 0x7a115169, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc271516e, 23 },
    { 0xd8a1516f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed215174, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f415177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa971517a, 87 },
    { 0xaa81517b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d41517f, 34 },
    { 0xeb215180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e15183, 90 },
    { 0, 0 },
    { 0x68a15185, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e91518f, 85 },
    { 0, 0 },
    { 0xce815191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a715195, 100 },
    { 0xa9715196, 34 },
    { 0, 0 },
    { 0x37715198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71519e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50151a3, 33 },
    { 0, 0 },
    { 0xe8b151a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7151a9, 91 },
    { 0xbd8151a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x793151ad, 85 },
    { 0xd75151ae, 61 },
    { 0, 0 },
    { 0x738151b0, 76 },
    { 0x88b151b0, 61 },
    { 0, 0 },
    { 0x631151b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x397151b6, 33 },
    { 0x3be151b7, 98 },
    { 0xaa5151b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8151bb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17151be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e151c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815151c8, 91 },
    { 0x430151c9, 61 },
    { 0x35d151c9, 90 },
    { 0x716151c9, 87 },
    { 0xbb9151cc, 23 },
    { 0x690151cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2151d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd151d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c151d8, 46 },
    { 0xfe4151d9, 98 },
    { 0x477151d9, 80 },
    { 0x4e1151db, 33 },
    { 0xc66151dc, 88 },
    { 0xc34151dd, 53 },
    { 0x5a1151de, 55 },
    { 0x3b9151df, 93 },
    { 0xa3f151db, 40 },
    { 0x14b151e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7151e7, 33 },
    { 0x1da151e8, 61 },
    { 0x7ba151e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10151ed, 44 },
    { 0x796151ee, 93 },
    { 0xd70151ef, 100 },
    { 0x5de151ef, 81 },
    { 0xd88151ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c151f8, 44 },
    { 0xc7a151f9, 90 },
    { 0x6ef151f9, 76 },
    { 0x7f3151fb, 61 },
    { 0xdd2151fc, 90 },
    { 0xd98151fd, 23 },
    { 0xbcf151fb, 61 },
    { 0x544151fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b15202, 80 },
    { 0x76c15202, 61 },
    { 0, 0 },
    { 0xb9e15205, 23 },
    { 0, 0 },
    { 0xb7b15207, 33 },
    { 0, 0 },
    { 0xd8915209, 93 },
    { 0, 0 },
    { 0xc6d1520b, 76 },
    { 0xf351520b, 23 },
    { 0, 0 },
    { 0x83f1520e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae15212, 23 },
    { 0, 0 },
    { 0x7a015214, 98 },
    { 0x52515214, 61 },
    { 0, 0 },
    { 0x1b215217, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec41521c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a1521f, 61 },
    { 0x79f15220, 87 },
    { 0xb8615221, 33 },
    { 0x3a915222, 33 },
    { 0, 0 },
    { 0xf1c15224, 23 },
    { 0x6e415225, 61 },
    { 0x36715226, 55 },
    { 0x8f115226, 61 },
    { 0x6f815227, 95 },
    { 0, 0 },
    { 0x6921522a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a1522e, 76 },
    { 0x4e31522f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb15232, 23 },
    { 0xd1315232, 23 },
    { 0xc4815232, 43 },
    { 0x6f715235, 80 },
    { 0xbce15235, 76 },
    { 0xeeb15234, 33 },
    { 0x58115238, 99 },
    { 0xb7715239, 87 },
    { 0x8561523a, 98 },
    { 0xf9e15237, 88 },
    { 0xf0415238, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81523f, 80 },
    { 0, 0 },
    { 0xeff15241, 33 },
    { 0x5f15242, 98 },
    { 0x87b15242, 23 },
    { 0x33415244, 67 },
    { 0x89915244, 64 },
    { 0xe415243, 98 },
    { 0xf0e15247, 61 },
    { 0xc2415248, 40 },
    { 0x95b15248, 92 },
    { 0xa7f15242, 90 },
    { 0xf9a15241, 23 },
    { 0x6341524c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33415250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f215253, 46 },
    { 0x7ea15254, 98 },
    { 0x4315254, 98 },
    { 0x94a15255, 81 },
    { 0, 0 },
    { 0xeb815258, 91 },
    { 0x82c15259, 87 },
    { 0, 0 },
    { 0x8481525b, 34 },
    { 0x4401525c, 95 },
    { 0xe011525d, 33 },
    { 0xfc71525e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd15263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3541526e, 98 },
    { 0, 0 },
    { 0x93e15270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2315277, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b1527a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9315280, 95 },
    { 0x7a415281, 90 },
    { 0xe7715282, 98 },
    { 0x65d15283, 76 },
    { 0xbf015284, 67 },
    { 0x5f515285, 78 },
    { 0x24a15286, 33 },
    { 0x64f15287, 76 },
    { 0, 0 },
    { 0x95d15289, 93 },
    { 0xa771528a, 76 },
    { 0x8921528b, 23 },
    { 0x471528c, 23 },
    { 0x59b1528d, 33 },
    { 0xd2f1528e, 98 },
    { 0x4db1528f, 95 },
    { 0x2d15290, 92 },
    { 0x415291, 90 },
    { 0x3cc15292, 93 },
    { 0xf7c15293, 98 },
    { 0x5f015290, 90 },
    { 0xd2f15295, 78 },
    { 0x89a15295, 23 },
    { 0xfb215297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e1529b, 23 },
    { 0, 0 },
    { 0xcdb1529d, 36 },
    { 0xcbf1529e, 95 },
    { 0x47f1529f, 98 },
    { 0x241152a0, 90 },
    { 0xc3c152a0, 90 },
    { 0xd10152a2, 87 },
    { 0xcc152a2, 33 },
    { 0xa66152a2, 23 },
    { 0xc81152a0, 33 },
    { 0x20b152a6, 33 },
    { 0xe5d152a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf152ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd152b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b152b6, 34 },
    { 0x8e9152b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78152bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0152c0, 66 },
    { 0x37152c1, 90 },
    { 0x1c1152c2, 76 },
    { 0, 0 },
    { 0x9a9152c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961152c9, 33 },
    { 0xbbd152ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561152d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x171152d7, 76 },
    { 0, 0 },
    { 0xe3152d9, 47 },
    { 0x745152da, 100 },
    { 0, 0 },
    { 0x17c152dc, 81 },
    { 0xf82152dd, 81 },
    { 0x150152dc, 33 },
    { 0xc9b152df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd152e8, 90 },
    { 0, 0 },
    { 0x5c152ea, 33 },
    { 0, 0 },
    { 0xf06152ec, 78 },
    { 0x344152ed, 61 },
    { 0x384152ee, 76 },
    { 0xf08152ee, 93 },
    { 0x624152ee, 95 },
    { 0xbc152f0, 38 },
    { 0, 0 },
    { 0xec0152f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x871152f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31152fa, 93 },
    { 0xbe8152fa, 61 },
    { 0, 0 },
    { 0xd23152fd, 90 },
    { 0, 0 },
    { 0x384152ff, 76 },
    { 0x49f15300, 33 },
    { 0x61315300, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e15307, 85 },
    { 0xcc815308, 43 },
    { 0, 0 },
    { 0x11d1530a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37015314, 98 },
    { 0, 0 },
    { 0xba115316, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6815319, 61 },
    { 0, 0 },
    { 0xe131531b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1531f, 66 },
    { 0x4bb15320, 93 },
    { 0x59115321, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a15327, 65 },
    { 0x27515328, 66 },
    { 0x9d915329, 23 },
    { 0x2231532a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841532d, 55 },
    { 0x1b51532e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d15334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f315338, 38 },
    { 0x5aa15339, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b21533d, 23 },
    { 0x6611533e, 87 },
    { 0xdc81533d, 95 },
    { 0, 0 },
    { 0x5df15341, 93 },
    { 0x1b115342, 93 },
    { 0x69015342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4215346, 43 },
    { 0x10215347, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1231534e, 88 },
    { 0xfbb1534f, 98 },
    { 0xd5b15350, 76 },
    { 0xa1e15350, 61 },
    { 0xf4615352, 87 },
    { 0x9731534f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4815359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c1535f, 61 },
    { 0, 0 },
    { 0xdd615361, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88515366, 33 },
    { 0, 0 },
    { 0x93b15368, 90 },
    { 0xdf315369, 76 },
    { 0x63e15368, 98 },
    { 0xd6d15368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f1536e, 91 },
    { 0x9481536e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e415372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8461537b, 76 },
    { 0xbe91537c, 17 },
    { 0x7271537b, 95 },
    { 0x1751537d, 46 },
    { 0xd321537e, 99 },
    { 0x32d15380, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff15386, 91 },
    { 0xb5c15387, 52 },
    { 0x5ba15388, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a1538c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74815392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea15395, 33 },
    { 0x7915396, 61 },
    { 0, 0 },
    { 0x36615398, 98 },
    { 0xa6215399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2881539c, 61 },
    { 0, 0 },
    { 0x3341539e, 61 },
    { 0x9061539f, 43 },
    { 0x8831539f, 43 },
    { 0xf9153a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b153a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83153ab, 55 },
    { 0xd71153ac, 99 },
    { 0, 0 },
    { 0x7de153ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7153b1, 90 },
    { 0, 0 },
    { 0x7b3153b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1153b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56153bd, 91 },
    { 0x24a153bd, 77 },
    { 0x6b153bf, 33 },
    { 0x938153be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68153c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc153c7, 95 },
    { 0xfe7153c8, 65 },
    { 0xe46153c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd153cd, 68 },
    { 0, 0 },
    { 0x2d153cf, 76 },
    { 0, 0 },
    { 0xfb2153d1, 38 },
    { 0, 0 },
    { 0x14b153d3, 87 },
    { 0x59b153d3, 90 },
    { 0x1c2153d5, 100 },
    { 0x1d153d6, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929153dd, 61 },
    { 0x8b5153de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9153e3, 90 },
    { 0xdb153e4, 33 },
    { 0, 0 },
    { 0x250153e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804153eb, 61 },
    { 0, 0 },
    { 0xf88153ed, 95 },
    { 0x94c153ee, 33 },
    { 0, 0 },
    { 0xabd153f0, 90 },
    { 0xf92153f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a153f4, 76 },
    { 0x191153f4, 98 },
    { 0, 0 },
    { 0x37c153f7, 33 },
    { 0xd4c153f7, 43 },
    { 0xfa3153f9, 66 },
    { 0xea6153f9, 99 },
    { 0xbf3153fb, 43 },
    { 0x1af153f9, 61 },
    { 0x419153f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b15401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf215407, 33 },
    { 0x76a15408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a31540d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd15410, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79315415, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1541b, 52 },
    { 0, 0 },
    { 0x2231541d, 61 },
    { 0xa381541e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7815421, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b15425, 85 },
    { 0, 0 },
    { 0x63515427, 88 },
    { 0x34815427, 78 },
    { 0, 0 },
    { 0x2cc1542a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21542d, 69 },
    { 0xbb11542e, 17 },
    { 0xb951542d, 95 },
    { 0, 0 },
    { 0x9515431, 95 },
    { 0, 0 },
    { 0x3f815433, 98 },
    { 0x51515434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff15439, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc141543e, 55 },
    { 0, 0 },
    { 0x3bb15440, 43 },
    { 0xdc415441, 85 },
    { 0x16d15442, 34 },
    { 0x1b815443, 33 },
    { 0x27015440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49015447, 98 },
    { 0x28315448, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a91544f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b15458, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78815460, 66 },
    { 0, 0 },
    { 0xfe315462, 98 },
    { 0, 0 },
    { 0xd8115464, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e15467, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61546c, 90 },
    { 0x6e51546d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1415472, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a415475, 90 },
    { 0xa9415475, 94 },
    { 0xe8f15477, 33 },
    { 0x14415478, 61 },
    { 0xacf15475, 76 },
    { 0xcee1547a, 90 },
    { 0x6821547b, 55 },
    { 0x7e615475, 98 },
    { 0xfb715477, 33 },
    { 0x20e1547e, 99 },
    { 0x81a1547b, 96 },
    { 0, 0 },
    { 0x97015481, 90 },
    { 0, 0 },
    { 0x88415483, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba15487, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1548b, 64 },
    { 0x83b1548c, 55 },
    { 0x1531548d, 100 },
    { 0x9851548c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf815493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa1549e, 43 },
    { 0x5ae1549f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d154a2, 61 },
    { 0, 0 },
    { 0xa84154a4, 69 },
    { 0x6cf154a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835154ab, 99 },
    { 0xd61154ac, 90 },
    { 0x348154ad, 33 },
    { 0, 0 },
    { 0x73b154af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a154b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e154b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0154c4, 61 },
    { 0xae9154c4, 95 },
    { 0xa7a154c5, 33 },
    { 0x9f1154c7, 48 },
    { 0x3dc154c5, 87 },
    { 0x84e154c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40154cd, 95 },
    { 0x865154ce, 90 },
    { 0, 0 },
    { 0x340154d0, 46 },
    { 0x2ca154d1, 93 },
    { 0x530154d1, 34 },
    { 0x790154d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b154d6, 43 },
    { 0xdd2154d7, 100 },
    { 0xfe0154d7, 76 },
    { 0xd5d154d7, 55 },
    { 0xcc2154d6, 76 },
    { 0x32d154d9, 84 },
    { 0xe4d154dc, 87 },
    { 0xcbe154dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84154e0, 55 },
    { 0xa27154e1, 23 },
    { 0x334154e2, 78 },
    { 0, 0 },
    { 0x4ef154e4, 61 },
    { 0xf3e154e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335154ec, 55 },
    { 0x3a2154ed, 76 },
    { 0x67b154ec, 95 },
    { 0x8df154ef, 56 },
    { 0, 0 },
    { 0x9ac154f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c154f7, 84 },
    { 0, 0 },
    { 0x26a154f9, 61 },
    { 0xd67154fa, 60 },
    { 0x1ea154fb, 43 },
    { 0, 0 },
    { 0x12a154fd, 34 },
    { 0xda3154fe, 93 },
    { 0x6ba154fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98715503, 76 },
    { 0, 0 },
    { 0x26615505, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21615509, 95 },
    { 0x33f1550a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1515512, 61 },
    { 0xbea15513, 75 },
    { 0, 0 },
    { 0xc2915515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb115519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf1551f, 34 },
    { 0xf5e1551f, 100 },
    { 0xd1215521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b615524, 33 },
    { 0x2e415525, 90 },
    { 0, 0 },
    { 0x30415527, 52 },
    { 0, 0 },
    { 0x42815529, 61 },
    { 0, 0 },
    { 0xcb31552b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191552e, 34 },
    { 0x8651552f, 99 },
    { 0x5315530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3115534, 61 },
    { 0x8c615534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe15538, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101553d, 100 },
    { 0x3f81553d, 33 },
    { 0x1371553d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee15542, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd815545, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3191555a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b15560, 61 },
    { 0, 0 },
    { 0x8fe15562, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28515566, 95 },
    { 0xb5c15567, 98 },
    { 0x9b915568, 34 },
    { 0x85315568, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801556e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79415575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451557b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6971557e, 87 },
    { 0x9ea1557f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41515583, 98 },
    { 0, 0 },
    { 0x54015585, 43 },
    { 0, 0 },
    { 0xbb315587, 95 },
    { 0x5df15588, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd1558b, 23 },
    { 0x3ed1558b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b15590, 74 },
    { 0x75515591, 23 },
    { 0x8ea15591, 33 },
    { 0x59b15592, 61 },
    { 0xa015594, 23 },
    { 0, 0 },
    { 0xec415596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5815599, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb901559e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7155a1, 23 },
    { 0x3dc155a1, 90 },
    { 0xd37155a3, 23 },
    { 0, 0 },
    { 0x28e155a5, 66 },
    { 0, 0 },
    { 0x18e155a7, 90 },
    { 0xd6155a8, 99 },
    { 0, 0 },
    { 0xc58155aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb155b2, 77 },
    { 0xb57155b3, 52 },
    { 0, 0 },
    { 0x7ce155b5, 33 },
    { 0xbb155b6, 33 },
    { 0x149155b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160155bc, 33 },
    { 0x383155bd, 95 },
    { 0xc0f155be, 91 },
    { 0x7f4155bc, 46 },
    { 0x2e9155c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6155c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0155cf, 98 },
    { 0, 0 },
    { 0x804155d1, 98 },
    { 0x34155d1, 33 },
    { 0xba8155d2, 33 },
    { 0, 0 },
    { 0x356155d5, 33 },
    { 0xf41155d6, 23 },
    { 0x49d155d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6155dc, 95 },
    { 0, 0 },
    { 0x6b6155de, 91 },
    { 0x8c2155de, 61 },
    { 0xa5f155de, 55 },
    { 0, 0 },
    { 0x7e9155e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a155e5, 98 },
    { 0, 0 },
    { 0x3f3155e7, 99 },
    { 0x42c155e8, 23 },
    { 0x7bf155e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x923155ec, 76 },
    { 0x7bf155ed, 61 },
    { 0xe91155ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5155f1, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b155f4, 76 },
    { 0xf75155f5, 43 },
    { 0xa8e155f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84155f9, 75 },
    { 0x100155fa, 95 },
    { 0, 0 },
    { 0x5f3155fc, 34 },
    { 0xd8e155fd, 23 },
    { 0xba155fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x98015601, 46 },
    { 0x1fa15601, 46 },
    { 0, 0 },
    { 0xf0e15604, 96 },
    { 0x2fe15604, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e15611, 84 },
    { 0x97615611, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6631561b, 87 },
    { 0x2bc1561c, 90 },
    { 0x291561b, 22 },
    { 0xe501561e, 66 },
    { 0x1871561f, 87 },
    { 0x7601561f, 33 },
    { 0x51315620, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c15626, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d1562b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d61563a, 93 },
    { 0xc251563b, 33 },
    { 0, 0 },
    { 0x4d81563d, 98 },
    { 0xcf1563d, 52 },
    { 0x7a81563f, 67 },
    { 0xb0c1563e, 61 },
    { 0xadf15641, 90 },
    { 0x76c15642, 98 },
    { 0x77c15643, 53 },
    { 0x3ea15640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af15652, 23 },
    { 0, 0 },
    { 0xf1b15654, 90 },
    { 0, 0 },
    { 0xdc515656, 95 },
    { 0xe6815657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5121565b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b15661, 61 },
    { 0xaf715662, 33 },
    { 0x415663, 90 },
    { 0xd6e15664, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27815667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd515672, 87 },
    { 0xfcf15673, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b15677, 90 },
    { 0x7da15678, 67 },
    { 0xcb15677, 61 },
    { 0xdf315677, 66 },
    { 0xf261567b, 90 },
    { 0xf1d1567b, 93 },
    { 0x4c61567c, 33 },
    { 0x261567e, 52 },
    { 0x8ff1567f, 90 },
    { 0xae415680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9515683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6315688, 33 },
    { 0x66915689, 76 },
    { 0x22e15689, 55 },
    { 0xbde1568b, 46 },
    { 0xa5d1568c, 90 },
    { 0x5ea1568c, 90 },
    { 0x6ad1568d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99715692, 61 },
    { 0x3f15692, 61 },
    { 0, 0 },
    { 0x92a15695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e215698, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d156a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22156a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06156ab, 98 },
    { 0, 0 },
    { 0x6bc156ad, 95 },
    { 0, 0 },
    { 0x67a156af, 23 },
    { 0xf45156b0, 98 },
    { 0, 0 },
    { 0xe7b156b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d156b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c156ba, 61 },
    { 0x128156bb, 81 },
    { 0xf0c156bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3156c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a156c8, 90 },
    { 0xb02156c9, 34 },
    { 0xaf7156c9, 33 },
    { 0, 0 },
    { 0x8a5156cc, 77 },
    { 0, 0 },
    { 0xaf9156ce, 87 },
    { 0, 0 },
    { 0xe1b156d0, 61 },
    { 0xdd5156d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a156d6, 55 },
    { 0, 0 },
    { 0xa60156d8, 90 },
    { 0xa56156d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73156dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b156e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861156e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68156ea, 95 },
    { 0x97d156ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2156ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb156f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1156f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f156f8, 55 },
    { 0xcf7156f9, 91 },
    { 0x35a156fa, 82 },
    { 0, 0 },
    { 0x1de156fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x914156ff, 90 },
    { 0, 0 },
    { 0xdc515701, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a15704, 76 },
    { 0xd1315705, 23 },
    { 0, 0 },
    { 0xf9515707, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc41570a, 33 },
    { 0x3c71570b, 61 },
    { 0x9191570c, 69 },
    { 0x3531570d, 100 },
    { 0x97e1570d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea215712, 55 },
    { 0x6fe15713, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7961571d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72115721, 90 },
    { 0x76115722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731572e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50915731, 98 },
    { 0x78c15732, 52 },
    { 0x35015733, 87 },
    { 0xdad15732, 61 },
    { 0xb815732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc1573d, 84 },
    { 0xf531573d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2815745, 94 },
    { 0, 0 },
    { 0xf0815747, 90 },
    { 0x6315747, 65 },
    { 0x67715749, 78 },
    { 0x1901574a, 53 },
    { 0x2801574b, 46 },
    { 0x91115749, 90 },
    { 0, 0 },
    { 0x9451574e, 77 },
    { 0xb291574f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371575b, 34 },
    { 0x5f31575b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23515764, 33 },
    { 0, 0 },
    { 0x69715766, 55 },
    { 0xbba15767, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1576a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1576f, 77 },
    { 0x2061576f, 98 },
    { 0xa1c15770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff15774, 94 },
    { 0x12d15775, 33 },
    { 0, 0 },
    { 0x5db15777, 33 },
    { 0x1c115778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f315786, 78 },
    { 0x4e715786, 33 },
    { 0, 0 },
    { 0x97515789, 67 },
    { 0x63b1578a, 61 },
    { 0, 0 },
    { 0xc9a1578c, 33 },
    { 0x2951578c, 33 },
    { 0xf041578c, 33 },
    { 0xd951578e, 34 },
    { 0xe8f1578f, 33 },
    { 0x6541578f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd115796, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x12515799, 61 },
    { 0xf2b1579a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1579d, 34 },
    { 0, 0 },
    { 0x5691579f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa157a6, 55 },
    { 0xc64157a7, 55 },
    { 0x9ca157a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301157ae, 34 },
    { 0x2f1157ae, 61 },
    { 0, 0 },
    { 0xc3157b1, 84 },
    { 0xe9d157b2, 61 },
    { 0x93a157b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e157be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24157c1, 95 },
    { 0, 0 },
    { 0xe3b157c3, 78 },
    { 0x57d157c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05157ca, 43 },
    { 0xcbe157cb, 61 },
    { 0x666157cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49157cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112157d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7157e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157f9, 33 },
    { 0, 0 },
    { 0xaa4157fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x765157fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3915801, 67 },
    { 0x35f15802, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99015808, 43 },
    { 0x24615809, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8281580d, 100 },
    { 0, 0 },
    { 0x2971580f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc15813, 78 },
    { 0x28115814, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa615818, 90 },
    { 0, 0 },
    { 0x95e1581a, 93 },
    { 0xf261581b, 55 },
    { 0x20f1581c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f15822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7715826, 90 },
    { 0xc1515827, 95 },
    { 0x59515828, 90 },
    { 0x3fc15829, 33 },
    { 0xfbc1582a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa1582d, 77 },
    { 0x4941582d, 69 },
    { 0x821582e, 90 },
    { 0xab115830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21715836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c15839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce815841, 23 },
    { 0x83115842, 33 },
    { 0, 0 },
    { 0xd7e15844, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1584c, 70 },
    { 0xf691584d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b15854, 87 },
    { 0x58315855, 33 },
    { 0xe4315856, 33 },
    { 0x74215857, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f1585a, 98 },
    { 0xf7d1585b, 34 },
    { 0x4341585b, 55 },
    { 0x1441585b, 99 },
    { 0, 0 },
    { 0xcca1585f, 55 },
    { 0x3c815860, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015863, 33 },
    { 0, 0 },
    { 0x1b815865, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a15868, 34 },
    { 0, 0 },
    { 0x331586a, 76 },
    { 0, 0 },
    { 0x81586c, 33 },
    { 0xc1a1586c, 55 },
    { 0x3401586e, 33 },
    { 0, 0 },
    { 0x65915870, 98 },
    { 0xe4e15871, 98 },
    { 0x73d15871, 23 },
    { 0, 0 },
    { 0xdb915874, 99 },
    { 0xd2015875, 90 },
    { 0xbde15875, 67 },
    { 0x1d815877, 97 },
    { 0xa9515874, 98 },
    { 0x62615879, 90 },
    { 0x24a1587a, 95 },
    { 0x3e215879, 98 },
    { 0x5c51587c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d01587f, 33 },
    { 0, 0 },
    { 0xccd15881, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c15887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a61588a, 46 },
    { 0x59c1588b, 52 },
    { 0xd3c1588b, 23 },
    { 0, 0 },
    { 0xed71588e, 55 },
    { 0xce41588f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58115892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb615896, 91 },
    { 0xf3e15897, 90 },
    { 0xb3115898, 99 },
    { 0, 0 },
    { 0x7491589a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1589e, 34 },
    { 0xd1f1589f, 34 },
    { 0x82c1589f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x67158a3, 17 },
    { 0xaf8158a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff158a7, 61 },
    { 0xe0d158a8, 33 },
    { 0, 0 },
    { 0x30f158aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x395158ad, 87 },
    { 0xd51158ad, 55 },
    { 0xb3c158af, 61 },
    { 0x7b8158b0, 98 },
    { 0x86c158b1, 88 },
    { 0, 0 },
    { 0x4ca158b3, 61 },
    { 0xea7158b4, 61 },
    { 0x96b158b4, 40 },
    { 0, 0 },
    { 0x1a4158b7, 33 },
    { 0x23b158b8, 67 },
    { 0x97a158b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7158bc, 87 },
    { 0xd3d158bc, 98 },
    { 0x3f8158be, 98 },
    { 0, 0 },
    { 0x859158c0, 43 },
    { 0x6e6158c1, 90 },
    { 0xdf9158c1, 61 },
    { 0x6f4158c3, 90 },
    { 0xa04158c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44158cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa158d0, 68 },
    { 0xa34158d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567158de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96158e1, 59 },
    { 0xca4158e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2158e5, 98 },
    { 0xbb5158e6, 64 },
    { 0x75f158e6, 90 },
    { 0x404158e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e158ed, 55 },
    { 0x882158ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff158f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4158fb, 98 },
    { 0x7f7158fc, 84 },
    { 0x94e158fd, 90 },
    { 0xad7158fe, 90 },
    { 0x2dc158ff, 97 },
    { 0, 0 },
    { 0x45615901, 84 },
    { 0, 0 },
    { 0x4a615903, 33 },
    { 0x7bd15904, 91 },
    { 0x38215905, 33 },
    { 0xf0115905, 91 },
    { 0x84615905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a1590b, 84 },
    { 0x5321590c, 76 },
    { 0, 0 },
    { 0x4f1590e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3115912, 67 },
    { 0xe1215913, 23 },
    { 0x64615914, 66 },
    { 0xa8215912, 33 },
    { 0xedb15916, 87 },
    { 0, 0 },
    { 0xaab15918, 34 },
    { 0, 0 },
    { 0x4b21591a, 98 },
    { 0, 0 },
    { 0x5861591c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e15921, 84 },
    { 0x5f315922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b015926, 61 },
    { 0x2e315926, 34 },
    { 0x2d515927, 90 },
    { 0xbc815927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb691592f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a15933, 61 },
    { 0x3a815933, 90 },
    { 0xbfe15934, 33 },
    { 0x42b15936, 78 },
    { 0xa515935, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081593c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b715944, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x85015947, 95 },
    { 0x10815948, 33 },
    { 0x7c15949, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75615952, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a15955, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d15958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921595d, 17 },
    { 0x5cb1595e, 93 },
    { 0xa3c1595f, 90 },
    { 0x8191595f, 76 },
    { 0x25a15961, 33 },
    { 0, 0 },
    { 0x3a515963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6215969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe031596c, 33 },
    { 0xc41596d, 90 },
    { 0xbdd1596e, 98 },
    { 0xfd71596e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d715973, 90 },
    { 0, 0 },
    { 0xd615975, 92 },
    { 0, 0 },
    { 0xd0215977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf081597b, 95 },
    { 0x611597b, 23 },
    { 0, 0 },
    { 0xfdc1597e, 98 },
    { 0, 0 },
    { 0x38b15980, 96 },
    { 0xc9b15981, 98 },
    { 0x4b015982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee715986, 84 },
    { 0, 0 },
    { 0x4fd15988, 97 },
    { 0xa8015989, 33 },
    { 0x45415989, 95 },
    { 0xaac15989, 98 },
    { 0xd631598c, 90 },
    { 0, 0 },
    { 0x4931598e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a15991, 23 },
    { 0xdc115992, 83 },
    { 0xc2315991, 61 },
    { 0x38c15994, 23 },
    { 0x1bb15995, 98 },
    { 0xd8315995, 91 },
    { 0x22115992, 34 },
    { 0xe4a15998, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e21599c, 95 },
    { 0xe7f1599d, 61 },
    { 0xd3b1599c, 61 },
    { 0, 0 },
    { 0x9cf159a0, 33 },
    { 0x28a159a1, 87 },
    { 0xa45159a2, 33 },
    { 0x224159a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98159a7, 91 },
    { 0x9d2159a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00159ab, 90 },
    { 0x829159ac, 98 },
    { 0x603159ac, 95 },
    { 0x987159ae, 61 },
    { 0x46b159ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8159b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e159bb, 61 },
    { 0, 0 },
    { 0xf67159bd, 93 },
    { 0x9b1159be, 98 },
    { 0xe99159bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f159c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9159cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38159d7, 23 },
    { 0, 0 },
    { 0x9ab159d9, 90 },
    { 0x158159da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0159de, 98 },
    { 0xeec159de, 66 },
    { 0xe7e159de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc159e3, 90 },
    { 0, 0 },
    { 0x86b159e5, 66 },
    { 0, 0 },
    { 0x3a5159e7, 23 },
    { 0x64e159e8, 55 },
    { 0xf36159e9, 23 },
    { 0x278159e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a159ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8159f1, 98 },
    { 0, 0 },
    { 0x1f0159f3, 76 },
    { 0x53a159f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9159f7, 55 },
    { 0, 0 },
    { 0xbe5159f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d159fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15a00, 91 },
    { 0x6e715a01, 76 },
    { 0, 0 },
    { 0xfd415a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b215a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54315a12, 91 },
    { 0, 0 },
    { 0xc0515a14, 90 },
    { 0xa2315a15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a515a20, 98 },
    { 0, 0 },
    { 0x41815a22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f15a26, 95 },
    { 0, 0 },
    { 0xb3215a28, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b15a2b, 33 },
    { 0xd5d15a2c, 76 },
    { 0x69715a2d, 100 },
    { 0xbbc15a2d, 91 },
    { 0x54815a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd15a35, 23 },
    { 0x7b915a35, 95 },
    { 0xdfd15a37, 95 },
    { 0x50f15a38, 23 },
    { 0xa3d15a39, 23 },
    { 0x75615a39, 95 },
    { 0x58c15a37, 33 },
    { 0xeb215a37, 84 },
    { 0x28815a38, 98 },
    { 0xfb815a39, 66 },
    { 0xff015a3f, 23 },
    { 0x26d15a40, 52 },
    { 0x6df15a3f, 95 },
    { 0x92115a3a, 33 },
    { 0xca615a3c, 76 },
    { 0x59c15a38, 43 },
    { 0x9a15a45, 90 },
    { 0x70c15a45, 95 },
    { 0x83815a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e215a4b, 61 },
    { 0x43c15a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d15a4f, 53 },
    { 0x4c115a50, 59 },
    { 0, 0 },
    { 0x9c615a52, 100 },
    { 0x5e515a52, 80 },
    { 0, 0 },
    { 0xb9715a55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91915a59, 90 },
    { 0x5c115a59, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x28315a5d, 84 },
    { 0xe2d15a5e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed15a68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b215a6e, 61 },
    { 0x65f15a6f, 65 },
    { 0x7d515a6f, 76 },
    { 0xcb615a71, 23 },
    { 0x58515a72, 55 },
    { 0, 0 },
    { 0xf915a74, 95 },
    { 0x92e15a74, 76 },
    { 0x83915a76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43715a7c, 98 },
    { 0x46e15a7c, 46 },
    { 0xf0e15a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae15a83, 66 },
    { 0x85a15a83, 33 },
    { 0xaf215a84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e215a8d, 34 },
    { 0x1e15a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8815a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d15a9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd215a9f, 33 },
    { 0xf5715a9f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b15aa5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93915aa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e515aac, 90 },
    { 0x11715aac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73715ab0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x44315ab3, 95 },
    { 0x40b15ab3, 78 },
    { 0x89a15ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf15ab8, 61 },
    { 0x93115ab9, 46 },
    { 0, 0 },
    { 0x76415abb, 43 },
    { 0, 0 },
    { 0x4e115abd, 90 },
    { 0xef715abe, 33 },
    { 0, 0 },
    { 0x4cb15ac0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10515ac5, 95 },
    { 0xe9815ac6, 43 },
    { 0x53f15ac6, 90 },
    { 0xf4815ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f15acb, 90 },
    { 0x61315acb, 98 },
    { 0xbf315acc, 95 },
    { 0xb1c15ace, 61 },
    { 0x25d15acf, 52 },
    { 0x32015ace, 91 },
    { 0x3a915acf, 99 },
    { 0xeb715acd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a15ad5, 67 },
    { 0, 0 },
    { 0xb0915ad7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96915ada, 90 },
    { 0x70615adb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x96615ade, 93 },
    { 0xd8415adf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27015ae2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7415ae5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7115ae9, 98 },
    { 0x84a15aea, 98 },
    { 0x61315aea, 33 },
    { 0x5b815aeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd915af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a15af7, 23 },
    { 0xb9d15af8, 23 },
    { 0xbe815af7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f015afc, 93 },
    { 0xd9815afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a15b04, 23 },
    { 0x7cc15b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e515b09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f15b0c, 98 },
    { 0x89715b0c, 61 },
    { 0xa2e15b0e, 98 },
    { 0xbf515b0f, 17 },
    { 0xfb615b10, 46 },
    { 0x28f15b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc15b16, 61 },
    { 0x9fc15b17, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c415b1c, 61 },
    { 0xfb215b1d, 31 },
    { 0, 0 },
    { 0x2ee15b1f, 90 },
    { 0xaa415b20, 98 },
    { 0xad315b21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f15b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6715b2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1415b33, 76 },
    { 0x3db15b33, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf15b37, 33 },
    { 0x8ec15b37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6015b3b, 55 },
    { 0, 0 },
    { 0x8c515b3d, 98 },
    { 0x77b15b3e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815b41, 53 },
    { 0xd7a15b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85515b49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa15b4c, 38 },
    { 0x9a515b4d, 68 },
    { 0x41e15b4e, 98 },
    { 0, 0 },
    { 0xc3a15b50, 33 },
    { 0xc2015b51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb15b54, 33 },
    { 0xf6e15b55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4615b5a, 33 },
    { 0, 0 },
    { 0x75715b5c, 87 },
    { 0x89815b5c, 33 },
    { 0, 0 },
    { 0x42915b5f, 68 },
    { 0xa2215b5f, 90 },
    { 0xd115b61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d15b68, 46 },
    { 0, 0 },
    { 0xeb615b6a, 43 },
    { 0x7f115b6b, 40 },
    { 0x9d415b6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad15b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c015b82, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4015b85, 61 },
    { 0, 0 },
    { 0x46215b87, 90 },
    { 0xe9815b88, 97 },
    { 0x44115b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11415b8c, 61 },
    { 0, 0 },
    { 0xa6515b8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1115b91, 93 },
    { 0x55f15b92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8115b95, 88 },
    { 0xd215b96, 33 },
    { 0x36615b97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb315b9d, 91 },
    { 0x6a015b9e, 52 },
    { 0x4215b9f, 95 },
    { 0x80415ba0, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8715ba3, 77 },
    { 0x4d215ba4, 90 },
    { 0x8b915ba3, 61 },
    { 0, 0 },
    { 0x30815ba7, 61 },
    { 0xca915ba8, 23 },
    { 0xbef15ba8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x46115bac, 90 },
    { 0xb1a15bad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc515bb1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17615bb4, 46 },
    { 0x34615bb5, 90 },
    { 0xbbe15bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85115bbe, 95 },
    { 0x3ea15bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c915bc2, 33 },
    { 0, 0 },
    { 0x38e15bc4, 61 },
    { 0x8ff15bc5, 90 },
    { 0x84f15bc6, 90 },
    { 0, 0 },
    { 0x8ec15bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6315bcb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x55615bce, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0215bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38615bda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e15bdd, 61 },
    { 0x9be15bde, 46 },
    { 0xa0915bde, 43 },
    { 0x47215be0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef615be4, 17 },
    { 0xb7515be5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b215bea, 67 },
    { 0x5e615beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e15bee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xded15bf1, 66 },
    { 0xd4315bf1, 85 },
    { 0, 0 },
    { 0x98415bf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc15bfe, 23 },
    { 0, 0 },
    { 0xd3415c00, 33 },
    { 0x38d15c00, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815c09, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b15c15, 85 },
    { 0x26315c15, 95 },
    { 0, 0 },
    { 0x7c515c18, 95 },
    { 0x5ad15c19, 87 },
    { 0xfaa15c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f15c1d, 55 },
    { 0x9c415c1d, 76 },
    { 0x68015c1d, 99 },
    { 0x75615c20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a15c26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a15c29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3815c2f, 33 },
    { 0xca115c30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd15c34, 61 },
    { 0x6f715c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a15c38, 46 },
    { 0, 0 },
    { 0xb4f15c3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f15c3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8815c42, 23 },
    { 0x42115c43, 76 },
    { 0, 0 },
    { 0xc2815c45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8115c49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b415c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4015c52, 34 },
    { 0, 0 },
    { 0x11815c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f15c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f15c61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a15c66, 88 },
    { 0x79a15c66, 33 },
    { 0x85c15c68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63615c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f715c72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e315c75, 95 },
    { 0, 0 },
    { 0x5c415c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715c7b, 98 },
    { 0xcef15c7b, 67 },
    { 0xe215c7d, 99 },
    { 0x53015c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b115c82, 46 },
    { 0, 0 },
    { 0x71115c84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c215c87, 66 },
    { 0xfbf15c87, 76 },
    { 0xbf15c89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb15c8c, 61 },
    { 0, 0 },
    { 0xbbf15c8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a015c92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63915c96, 83 },
    { 0x20f15c97, 61 },
    { 0x40415c97, 91 },
    { 0, 0 },
    { 0xfef15c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9515c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415ca1, 33 },
    { 0, 0 },
    { 0x4db15ca3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6515ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1115cac, 66 },
    { 0xd0e15cad, 90 },
    { 0xe1c15cae, 52 },
    { 0, 0 },
    { 0x29f15cb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc15cb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d15cba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e015cbf, 85 },
    { 0, 0 },
    { 0x9fc15cc1, 61 },
    { 0x85615cc2, 52 },
    { 0, 0 },
    { 0xa0c15cc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e15cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd15cca, 55 },
    { 0x7e915ccb, 87 },
    { 0x7e815ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64915ccf, 64 },
    { 0, 0 },
    { 0x63b15cd1, 90 },
    { 0x8df15cd2, 76 },
    { 0x99a15cd3, 95 },
    { 0x91315cd4, 98 },
    { 0xa9115cd4, 100 },
    { 0xbc15cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b15cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e15ce4, 96 },
    { 0x4c115ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34215ce8, 66 },
    { 0xd7215ce8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51315cec, 78 },
    { 0x13615cec, 90 },
    { 0x1d515cee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2715cf3, 93 },
    { 0x6ed15cf3, 33 },
    { 0x2f015cf5, 46 },
    { 0, 0 },
    { 0xd4915cf7, 33 },
    { 0x40a15cf8, 33 },
    { 0x66e15cf8, 52 },
    { 0, 0 },
    { 0xd1315cfb, 23 },
    { 0x1f015cfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9015cff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3015d03, 95 },
    { 0x82415d03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c715d07, 95 },
    { 0x53715d08, 33 },
    { 0xfa415d09, 55 },
    { 0x1c015d0a, 24 },
    { 0x38215d0b, 90 },
    { 0, 0 },
    { 0x10515d0d, 56 },
    { 0, 0 },
    { 0xfa15d0f, 33 },
    { 0xcb315d10, 90 },
    { 0, 0 },
    { 0xa2315d12, 65 },
    { 0xa7e15d13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef15d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7515d1a, 95 },
    { 0, 0 },
    { 0xe415d1c, 17 },
    { 0, 0 },
    { 0xe0b15d1e, 33 },
    { 0xf7d15d1f, 95 },
    { 0x41d15d1f, 33 },
    { 0xd9d15d1e, 98 },
    { 0, 0 },
    { 0x17315d23, 23 },
    { 0x13515d24, 61 },
    { 0x7a515d25, 90 },
    { 0, 0 },
    { 0x60915d27, 55 },
    { 0x8215d27, 61 },
    { 0x5ce15d27, 93 },
    { 0xe9b15d2a, 95 },
    { 0xd1815d29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de15d33, 23 },
    { 0x5d715d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad015d3b, 90 },
    { 0x2e315d3c, 55 },
    { 0xc9c15d3d, 95 },
    { 0xdba15d3e, 91 },
    { 0xc0815d3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8915d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d15d4d, 46 },
    { 0x11915d4e, 61 },
    { 0xc5915d4d, 76 },
    { 0, 0 },
    { 0xfa315d51, 23 },
    { 0, 0 },
    { 0x29e15d53, 60 },
    { 0x7ee15d54, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96515d57, 99 },
    { 0, 0 },
    { 0x49415d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0115d5c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc815d5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f15d63, 33 },
    { 0x3ec15d64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6715d67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35815d6a, 85 },
    { 0, 0 },
    { 0xc0115d6c, 90 },
    { 0xf1215d6c, 61 },
    { 0x8515d6e, 43 },
    { 0x39115d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c15d73, 55 },
    { 0x44d15d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc15d7b, 93 },
    { 0x36d15d7c, 90 },
    { 0x18915d7d, 91 },
    { 0x87215d7e, 40 },
    { 0xb015d7f, 87 },
    { 0xec515d80, 46 },
    { 0xf7415d7c, 61 },
    { 0xcc115d7c, 100 },
    { 0x41a15d82, 46 },
    { 0x5e015d81, 55 },
    { 0x84015d85, 94 },
    { 0xf1015d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e15d89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18215d8d, 81 },
    { 0xd3815d8e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b15d95, 96 },
    { 0, 0 },
    { 0x24f15d97, 66 },
    { 0xd5215d98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f15d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d15da4, 33 },
    { 0, 0 },
    { 0x12915da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd15daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc715db3, 95 },
    { 0, 0 },
    { 0x11b15db5, 23 },
    { 0xb2315db6, 61 },
    { 0, 0 },
    { 0x90815db8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef15dbf, 98 },
    { 0x5a615dc0, 98 },
    { 0, 0 },
    { 0xc4315dc2, 55 },
    { 0x48a15dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc15dc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a615dcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e15dcf, 23 },
    { 0xa2f15dd0, 90 },
    { 0xcdd15dcf, 95 },
    { 0, 0 },
    { 0x8c015dd3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31015dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5215de0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d15de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b015dee, 33 },
    { 0x3fd15def, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1515df7, 23 },
    { 0xcdb15df7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67615dfb, 46 },
    { 0, 0 },
    { 0x37b15dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e15e03, 17 },
    { 0xb0515e04, 67 },
    { 0x77015e04, 88 },
    { 0xb4515e03, 23 },
    { 0x14715e07, 87 },
    { 0xbf15e03, 90 },
    { 0xad615e05, 78 },
    { 0x67d15e0a, 67 },
    { 0, 0 },
    { 0x31415e0c, 77 },
    { 0, 0 },
    { 0xa8915e0e, 23 },
    { 0, 0 },
    { 0x97515e10, 88 },
    { 0xa1215e10, 93 },
    { 0, 0 },
    { 0xcfc15e13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a15e17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60615e1a, 34 },
    { 0xcfd15e1b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a015e1e, 53 },
    { 0x9d515e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d15e22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41115e29, 98 },
    { 0x47c15e2a, 23 },
    { 0x3ae15e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c515e31, 84 },
    { 0, 0 },
    { 0x86015e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5615e36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215e39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6415e3d, 33 },
    { 0x9515e3e, 61 },
    { 0, 0 },
    { 0x14015e40, 33 },
    { 0x31515e41, 76 },
    { 0x2b915e40, 55 },
    { 0, 0 },
    { 0xbf115e44, 33 },
    { 0x34815e44, 56 },
    { 0x9b815e46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa915e4d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c15e51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f15e55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84215e58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb15e5e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x96115e61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2215e67, 55 },
    { 0xe015e68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b15e71, 99 },
    { 0xe6315e72, 98 },
    { 0x15215e73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee15e77, 76 },
    { 0, 0 },
    { 0x29b15e79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83415e7c, 43 },
    { 0, 0 },
    { 0x1f315e7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82415e84, 45 },
    { 0, 0 },
    { 0xaed15e86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82715e8a, 61 },
    { 0x7eb15e8b, 76 },
    { 0x1c815e8c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b215e91, 90 },
    { 0x18115e92, 90 },
    { 0x58015e93, 76 },
    { 0xc8f15e94, 90 },
    { 0, 0 },
    { 0x4aa15e96, 85 },
    { 0x30b15e96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10615e9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac115ea2, 75 },
    { 0xb015ea3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd15ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af15eac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b15eb1, 35 },
    { 0xc8515eb2, 76 },
    { 0, 0 },
    { 0xbd115eb4, 33 },
    { 0x7f215eb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7615eb8, 76 },
    { 0x5215eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6915ebf, 34 },
    { 0, 0 },
    { 0x63115ec1, 98 },
    { 0x23115ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea15ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88015ed1, 98 },
    { 0, 0 },
    { 0x8f915ed3, 93 },
    { 0x75215ed4, 93 },
    { 0x74615ed4, 33 },
    { 0xfa715ed3, 90 },
    { 0x73715ed7, 33 },
    { 0xb8915ed8, 23 },
    { 0, 0 },
    { 0xc1115eda, 43 },
    { 0, 0 },
    { 0xe5715edc, 14 },
    { 0x7c615edd, 55 },
    { 0, 0 },
    { 0x7e615edf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84915ee2, 55 },
    { 0x77915ee3, 95 },
    { 0x6cc15ee2, 61 },
    { 0xb6115ee5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb515eea, 42 },
    { 0x8aa15eeb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15eee, 90 },
    { 0x4ce15eef, 33 },
    { 0xda615ef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8715ef6, 61 },
    { 0xcb215ef7, 61 },
    { 0x1b15ef7, 23 },
    { 0x16e15ef8, 95 },
    { 0x74615ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e15efe, 88 },
    { 0xbad15efe, 98 },
    { 0, 0 },
    { 0x48215f01, 33 },
    { 0x81a15f02, 34 },
    { 0xb4e15f02, 33 },
    { 0x5ba15f04, 33 },
    { 0xc6b15f05, 55 },
    { 0x42715f05, 85 },
    { 0xc2815f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66915f0e, 90 },
    { 0, 0 },
    { 0xaf415f10, 91 },
    { 0x4c115f10, 90 },
    { 0, 0 },
    { 0xe4215f13, 61 },
    { 0xaec15f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3315f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc115f20, 77 },
    { 0x9815f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60415f25, 91 },
    { 0xecd15f25, 80 },
    { 0xb4d15f27, 61 },
    { 0xd4215f28, 90 },
    { 0, 0 },
    { 0x89f15f2a, 90 },
    { 0, 0 },
    { 0x85015f2c, 61 },
    { 0xfed15f2d, 66 },
    { 0, 0 },
    { 0x6c815f2f, 95 },
    { 0x78915f30, 23 },
    { 0xe5d15f31, 61 },
    { 0, 0 },
    { 0x97d15f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c415f39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c15f3f, 67 },
    { 0, 0 },
    { 0x94615f41, 90 },
    { 0x38115f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e615f46, 61 },
    { 0x34715f47, 90 },
    { 0x93715f46, 67 },
    { 0xb7015f49, 98 },
    { 0, 0 },
    { 0x1d115f4b, 76 },
    { 0xb615f4c, 23 },
    { 0xdf315f4b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15f50, 67 },
    { 0, 0 },
    { 0x18e15f52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb15f58, 61 },
    { 0xe9715f58, 61 },
    { 0, 0 },
    { 0xdb815f5b, 91 },
    { 0x23815f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015f61, 81 },
    { 0x20415f61, 33 },
    { 0xd9815f63, 55 },
    { 0xc6415f64, 36 },
    { 0x18915f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec815f68, 33 },
    { 0x1b115f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb615f6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86015f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c315f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca15f7f, 61 },
    { 0, 0 },
    { 0x20715f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12415f89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad515f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc15f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7715f94, 77 },
    { 0, 0 },
    { 0x69515f96, 67 },
    { 0xaa315f96, 33 },
    { 0xf7715f98, 95 },
    { 0xc1a15f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf515fa0, 76 },
    { 0x66515fa1, 64 },
    { 0x2fc15fa1, 90 },
    { 0x1d815fa2, 65 },
    { 0, 0 },
    { 0xe8215fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4615fa8, 93 },
    { 0, 0 },
    { 0xceb15faa, 81 },
    { 0x50515faa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13215fae, 98 },
    { 0x41615faf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8015fb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a15fb5, 100 },
    { 0xcdb15fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b15fba, 43 },
    { 0xbb515fbb, 99 },
    { 0xcad15fbb, 61 },
    { 0xe0415fbc, 84 },
    { 0x7e115fbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5315fc7, 23 },
    { 0x80915fc7, 33 },
    { 0xb1115fc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa15fcf, 33 },
    { 0x59815fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf15fdb, 55 },
    { 0xed215fdc, 46 },
    { 0x35315fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec15fe2, 61 },
    { 0x10815fe3, 23 },
    { 0x81c15fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb15fe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c15fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b15ff3, 99 },
    { 0xec415ff4, 23 },
    { 0x1c815ff4, 61 },
    { 0x47315ff6, 84 },
    { 0x47e15ff5, 91 },
    { 0, 0 },
    { 0x3a15ff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a015ffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27016003, 90 },
    { 0x9616004, 99 },
    { 0, 0 },
    { 0x78816006, 34 },
    { 0, 0 },
    { 0xb8916008, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x3931600b, 78 },
    { 0, 0 },
    { 0x61a1600d, 84 },
    { 0, 0 },
    { 0xed71600f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e516016, 98 },
    { 0x12d16017, 91 },
    { 0xd1816017, 61 },
    { 0xee816017, 43 },
    { 0x90316017, 34 },
    { 0x22616017, 90 },
    { 0xbde1601c, 23 },
    { 0, 0 },
    { 0x7151601e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30516022, 76 },
    { 0x33016022, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab16027, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf216033, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18516038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba1603b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1603e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c1604a, 90 },
    { 0x54e1604b, 85 },
    { 0xc491604c, 66 },
    { 0xa5b1604d, 33 },
    { 0x6551604d, 38 },
    { 0x25f1604f, 23 },
    { 0xc8c16050, 33 },
    { 0x6631604d, 93 },
    { 0x3db1604e, 33 },
    { 0xd9d16053, 68 },
    { 0x22416053, 75 },
    { 0xc0216055, 90 },
    { 0xab716056, 23 },
    { 0xa671604a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e1605c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d016064, 66 },
    { 0x62216065, 80 },
    { 0xa8816066, 95 },
    { 0x2db16064, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b1606a, 98 },
    { 0xdb51606a, 61 },
    { 0, 0 },
    { 0x171606d, 40 },
    { 0xfb41606e, 67 },
    { 0x2431606f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43716078, 90 },
    { 0xe5116079, 61 },
    { 0x25916079, 95 },
    { 0, 0 },
    { 0xaca1607c, 65 },
    { 0xecd1607d, 90 },
    { 0xcca1607e, 55 },
    { 0xac91607e, 61 },
    { 0x701607e, 34 },
    { 0, 0 },
    { 0xdb616082, 61 },
    { 0xf16083, 52 },
    { 0x46916084, 81 },
    { 0xe8d16084, 93 },
    { 0x4e916084, 36 },
    { 0xad916087, 88 },
    { 0xe9f16088, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf851608c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d16092, 66 },
    { 0, 0 },
    { 0x6d116094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b1609d, 61 },
    { 0x9ca1609e, 98 },
    { 0, 0 },
    { 0x93d160a0, 33 },
    { 0x73160a0, 43 },
    { 0xd8e160a2, 61 },
    { 0x2e4160a3, 55 },
    { 0xb2c160a4, 52 },
    { 0xfe2160a5, 46 },
    { 0xbc1160a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d160aa, 98 },
    { 0, 0 },
    { 0xdce160ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e160b3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31160b7, 33 },
    { 0x14e160b8, 15 },
    { 0x7b8160b8, 85 },
    { 0xac9160ba, 95 },
    { 0x441160ba, 90 },
    { 0xaf4160b8, 98 },
    { 0x4d1160bd, 36 },
    { 0x462160bd, 76 },
    { 0, 0 },
    { 0x225160c0, 76 },
    { 0x8c3160c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af160c4, 77 },
    { 0, 0 },
    { 0x481160c6, 43 },
    { 0xc30160c7, 98 },
    { 0x9d3160c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914160cd, 78 },
    { 0xbf0160cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0160d2, 98 },
    { 0xb39160d3, 23 },
    { 0xe5c160d3, 95 },
    { 0xdca160d3, 95 },
    { 0, 0 },
    { 0x16f160d7, 93 },
    { 0x4a7160d8, 95 },
    { 0, 0 },
    { 0xa09160da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1160de, 61 },
    { 0xfdc160df, 95 },
    { 0x94b160df, 76 },
    { 0xf65160e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x999160e4, 43 },
    { 0xdb0160e5, 77 },
    { 0xb9c160e5, 98 },
    { 0x422160e7, 23 },
    { 0x4f8160e5, 77 },
    { 0x35160e9, 90 },
    { 0xeb4160ea, 36 },
    { 0, 0 },
    { 0x8b8160ec, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874160f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5160f9, 61 },
    { 0xef7160fa, 81 },
    { 0xcc160fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16100, 98 },
    { 0, 0 },
    { 0xeaf16102, 17 },
    { 0xdfe16103, 33 },
    { 0xbc816103, 87 },
    { 0x1ac16104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851610b, 76 },
    { 0x9a01610c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xded1610f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c16114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6616118, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9561611e, 46 },
    { 0, 0 },
    { 0x49b16120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40316125, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411612a, 55 },
    { 0, 0 },
    { 0x6bb1612c, 91 },
    { 0x8681612d, 98 },
    { 0, 0 },
    { 0x5c91612f, 90 },
    { 0xc2d16130, 43 },
    { 0, 0 },
    { 0x96316132, 23 },
    { 0x51816132, 98 },
    { 0x60816134, 33 },
    { 0x2e916134, 91 },
    { 0, 0 },
    { 0x9a416137, 43 },
    { 0xf4f16138, 76 },
    { 0x72216139, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf931613c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62216142, 91 },
    { 0, 0 },
    { 0x5df16144, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31614b, 95 },
    { 0xa441614c, 34 },
    { 0, 0 },
    { 0xe71614e, 75 },
    { 0x46f1614f, 90 },
    { 0x6931614f, 33 },
    { 0x5091614e, 90 },
    { 0x52716151, 55 },
    { 0x68b1614e, 33 },
    { 0xc2a16153, 75 },
    { 0xad716155, 95 },
    { 0x2b16155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd116159, 78 },
    { 0xde016159, 23 },
    { 0x5921615b, 99 },
    { 0x2fe1615b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7616160, 34 },
    { 0, 0 },
    { 0xed816162, 90 },
    { 0, 0 },
    { 0x98716164, 88 },
    { 0x73d16165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1481616d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea516175, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d01617b, 43 },
    { 0xb871617c, 76 },
    { 0x4881617d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc16182, 90 },
    { 0x77e16183, 100 },
    { 0x5616182, 34 },
    { 0x4f316185, 46 },
    { 0x86716186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f1618a, 61 },
    { 0x2501618a, 98 },
    { 0xd6f1618b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd16191, 33 },
    { 0x9c316192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d16198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd1619b, 99 },
    { 0xd761619c, 84 },
    { 0x6f61619d, 68 },
    { 0x5981619e, 98 },
    { 0xc0b1619f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7161a3, 46 },
    { 0x479161a3, 87 },
    { 0xb58161a3, 43 },
    { 0, 0 },
    { 0x668161a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5161ad, 95 },
    { 0x1cb161ae, 61 },
    { 0, 0 },
    { 0xb1e161b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1161b4, 84 },
    { 0xc13161b5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1161bb, 61 },
    { 0xf5161bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83161bf, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1161c2, 67 },
    { 0x765161c3, 61 },
    { 0xf13161c4, 66 },
    { 0xd65161c3, 95 },
    { 0, 0 },
    { 0x397161c7, 38 },
    { 0, 0 },
    { 0x5f6161c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558161cd, 76 },
    { 0, 0 },
    { 0x4b9161cf, 33 },
    { 0, 0 },
    { 0x3aa161d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d161db, 61 },
    { 0, 0 },
    { 0x310161dd, 52 },
    { 0, 0 },
    { 0x8b0161df, 52 },
    { 0, 0 },
    { 0x1ee161e1, 90 },
    { 0x3ec161e1, 55 },
    { 0, 0 },
    { 0x837161e4, 23 },
    { 0xfd5161e5, 95 },
    { 0x987161e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e161e9, 88 },
    { 0x37b161e9, 68 },
    { 0x541161eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161ef, 98 },
    { 0x18c161f0, 55 },
    { 0x514161f0, 95 },
    { 0x9a7161f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6161f6, 43 },
    { 0xdc6161f6, 90 },
    { 0, 0 },
    { 0xe02161f9, 17 },
    { 0xd83161f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x164161fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b016201, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916204, 34 },
    { 0xef216205, 43 },
    { 0x9d916206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda216209, 95 },
    { 0, 0 },
    { 0x3ca1620b, 76 },
    { 0, 0 },
    { 0xe5f1620d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e16211, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54416215, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d916218, 95 },
    { 0x61416219, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd31621c, 95 },
    { 0x4901621d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb716225, 76 },
    { 0, 0 },
    { 0xbe516227, 48 },
    { 0x9e016227, 43 },
    { 0x6916229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3331622d, 65 },
    { 0, 0 },
    { 0x2381622f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c016234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5616237, 85 },
    { 0, 0 },
    { 0xc1216239, 99 },
    { 0x6dc1623a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb381623f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c16242, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9316245, 90 },
    { 0x25316245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33016249, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f1624c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84416250, 40 },
    { 0x65616250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a016258, 17 },
    { 0x44a16258, 76 },
    { 0xdf91625a, 33 },
    { 0, 0 },
    { 0xba11625c, 77 },
    { 0x9f81625d, 23 },
    { 0, 0 },
    { 0xb101625f, 66 },
    { 0x81625f, 78 },
    { 0x53d16261, 78 },
    { 0, 0 },
    { 0x10016263, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c16267, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc016270, 90 },
    { 0xd2216271, 98 },
    { 0x72816271, 61 },
    { 0xb3116271, 90 },
    { 0x16716274, 90 },
    { 0xbc516273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b1627a, 95 },
    { 0xd0f1627b, 95 },
    { 0xa0e1627c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0016280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c16286, 85 },
    { 0, 0 },
    { 0xb7316288, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b1628e, 98 },
    { 0x18c1628e, 95 },
    { 0x7b216290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec16294, 98 },
    { 0, 0 },
    { 0x4aa16296, 65 },
    { 0x3a716297, 23 },
    { 0x7c516296, 33 },
    { 0xcc416299, 33 },
    { 0x25916299, 61 },
    { 0x50d16299, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3162a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28162a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2162a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x194162aa, 34 },
    { 0xc20162ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a162af, 43 },
    { 0x2f9162b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5162b3, 90 },
    { 0xdca162b4, 23 },
    { 0x699162b3, 17 },
    { 0xa8162b4, 90 },
    { 0xa3d162b4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb162bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4162be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391162c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3162c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134162ca, 98 },
    { 0, 0 },
    { 0x22a162cc, 98 },
    { 0x314162cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a162d0, 33 },
    { 0xfef162d1, 88 },
    { 0x618162d2, 98 },
    { 0xd22162d0, 33 },
    { 0xc30162d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17162dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f162df, 91 },
    { 0x649162e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7162e4, 33 },
    { 0x6f9162e5, 90 },
    { 0x216162e5, 91 },
    { 0xff8162e7, 23 },
    { 0xdf5162e8, 90 },
    { 0x9d5162e5, 76 },
    { 0x303162e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x509162ed, 98 },
    { 0x9f2162ed, 95 },
    { 0, 0 },
    { 0xaed162f0, 76 },
    { 0x924162f1, 34 },
    { 0x515162f2, 76 },
    { 0xb5e162f2, 95 },
    { 0xbed162f4, 76 },
    { 0xcef162f5, 33 },
    { 0x9d4162f6, 101 },
    { 0x5c9162f1, 98 },
    { 0, 0 },
    { 0x2b9162f9, 91 },
    { 0, 0 },
    { 0x368162fb, 98 },
    { 0x8a162fb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0162ff, 23 },
    { 0xd88162ff, 55 },
    { 0, 0 },
    { 0xedc16302, 43 },
    { 0xeda16303, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e616314, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc816317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61216324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd16328, 55 },
    { 0xa2316329, 92 },
    { 0xc0516329, 46 },
    { 0x4251632b, 33 },
    { 0xe1b1632b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13316332, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f416337, 99 },
    { 0xb9516337, 84 },
    { 0xcd116337, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe1633c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee16344, 98 },
    { 0xc5216345, 23 },
    { 0, 0 },
    { 0x5ce16347, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1941634d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb16352, 81 },
    { 0xc9516353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d16358, 100 },
    { 0, 0 },
    { 0xc2d1635a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb871635e, 23 },
    { 0, 0 },
    { 0x3f916360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e216363, 93 },
    { 0x9a816364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd16369, 55 },
    { 0xaac1636a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d1636f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df1637a, 33 },
    { 0xa5e1637b, 93 },
    { 0x6201637c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62616380, 67 },
    { 0, 0 },
    { 0x50716382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57416385, 33 },
    { 0, 0 },
    { 0x3f316387, 80 },
    { 0xe7816388, 33 },
    { 0x52216388, 67 },
    { 0xe061638a, 98 },
    { 0, 0 },
    { 0x82c1638c, 98 },
    { 0x62f1638d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a16391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6016397, 98 },
    { 0, 0 },
    { 0x65e16399, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd51639d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x911163a0, 65 },
    { 0x57e163a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3163a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e163a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727163ad, 84 },
    { 0xf79163ad, 95 },
    { 0, 0 },
    { 0x6e163b0, 58 },
    { 0xc85163b1, 90 },
    { 0x62d163b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48163b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55163c2, 95 },
    { 0, 0 },
    { 0x845163c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e163ca, 17 },
    { 0, 0 },
    { 0x4f0163cc, 76 },
    { 0x36b163cc, 43 },
    { 0x5163ce, 34 },
    { 0x4bf163cf, 61 },
    { 0x2fe163ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61163d3, 46 },
    { 0, 0 },
    { 0x673163d5, 98 },
    { 0xda2163d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc163de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171163e3, 61 },
    { 0xcff163e3, 34 },
    { 0x26163e3, 78 },
    { 0, 0 },
    { 0xdb5163e7, 78 },
    { 0xb76163e8, 34 },
    { 0x2ab163e9, 90 },
    { 0x19d163e9, 52 },
    { 0xd28163ea, 90 },
    { 0, 0 },
    { 0xa73163ed, 61 },
    { 0, 0 },
    { 0xccc163ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92163f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469163fc, 95 },
    { 0x342163fd, 76 },
    { 0x83a163fe, 97 },
    { 0x72d163fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64116404, 55 },
    { 0x79016405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f1640a, 61 },
    { 0x8dc1640b, 93 },
    { 0x9d31640c, 55 },
    { 0x38a1640d, 55 },
    { 0x1951640d, 23 },
    { 0xddf1640c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79216414, 46 },
    { 0x1fd16415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c16422, 52 },
    { 0xfdf16422, 23 },
    { 0x49516423, 98 },
    { 0x58416423, 46 },
    { 0xb6916426, 33 },
    { 0xc8e16426, 61 },
    { 0x2816425, 95 },
    { 0xf5516428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a1642c, 23 },
    { 0x4fd1642d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25816434, 23 },
    { 0xe3516434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2016438, 100 },
    { 0x8c816439, 61 },
    { 0x3e116439, 85 },
    { 0x3ef16439, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7431643f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9416443, 90 },
    { 0x4da16444, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a1644b, 90 },
    { 0, 0 },
    { 0x1b11644d, 90 },
    { 0x2cf1644d, 98 },
    { 0xc8b1644f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6316452, 33 },
    { 0xed16453, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12616456, 76 },
    { 0, 0 },
    { 0x75516458, 34 },
    { 0xc8916459, 75 },
    { 0xadc1645a, 76 },
    { 0xdff1645b, 33 },
    { 0x82e1645c, 33 },
    { 0x10a1645d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23616463, 88 },
    { 0x4a416464, 23 },
    { 0, 0 },
    { 0x6ec16466, 95 },
    { 0x89216467, 43 },
    { 0, 0 },
    { 0x78016469, 95 },
    { 0x8041646a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83916470, 95 },
    { 0xc6e16471, 90 },
    { 0x7e116472, 33 },
    { 0xf2316470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4616478, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591647b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61647e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58216481, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7616484, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc981648a, 90 },
    { 0x6ff1648a, 81 },
    { 0xe991648c, 90 },
    { 0x39b1648c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b16494, 8 },
    { 0, 0 },
    { 0x58f16496, 61 },
    { 0xf2216497, 33 },
    { 0x3bd16496, 61 },
    { 0xd5916499, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd21649d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b164a2, 95 },
    { 0, 0 },
    { 0xb9d164a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb164a9, 76 },
    { 0xabe164aa, 90 },
    { 0, 0 },
    { 0x54d164ac, 81 },
    { 0x406164ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd164b4, 17 },
    { 0x7af164b4, 100 },
    { 0x1d0164b4, 34 },
    { 0x63b164b7, 55 },
    { 0, 0 },
    { 0xa33164b9, 23 },
    { 0, 0 },
    { 0x8d6164bb, 88 },
    { 0xe27164bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2164bf, 78 },
    { 0, 0 },
    { 0x427164c1, 90 },
    { 0xee3164c1, 91 },
    { 0xc19164c1, 100 },
    { 0xb02164c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d164c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad164cc, 91 },
    { 0, 0 },
    { 0xaa4164ce, 19 },
    { 0, 0 },
    { 0xe9b164d0, 69 },
    { 0xaf3164d0, 91 },
    { 0, 0 },
    { 0x5bd164d3, 83 },
    { 0xa68164d4, 87 },
    { 0x254164d5, 91 },
    { 0x60b164d5, 95 },
    { 0x764164d5, 23 },
    { 0, 0 },
    { 0xf1b164d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a164dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850164e0, 43 },
    { 0x759164e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b164e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d164e9, 46 },
    { 0, 0 },
    { 0xa9b164eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf164f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b16501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb416508, 61 },
    { 0, 0 },
    { 0x12e1650a, 55 },
    { 0, 0 },
    { 0xaa41650c, 85 },
    { 0x73c1650c, 23 },
    { 0xa921650c, 78 },
    { 0xac01650f, 52 },
    { 0xa4716510, 61 },
    { 0xf5716511, 91 },
    { 0xcd816512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b816516, 33 },
    { 0, 0 },
    { 0x9a816518, 33 },
    { 0, 0 },
    { 0x2c41651a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1651d, 46 },
    { 0x80e1651e, 76 },
    { 0x3fe1651f, 90 },
    { 0x50016520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33716524, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x88116527, 93 },
    { 0, 0 },
    { 0xca416529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef1652c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c21652f, 87 },
    { 0x43a16530, 55 },
    { 0xdd516531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e16534, 98 },
    { 0xb9e16534, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5431653a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3571653e, 23 },
    { 0x52f1653e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d16546, 61 },
    { 0x3b16547, 75 },
    { 0xa2416548, 95 },
    { 0, 0 },
    { 0x7d11654a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94016553, 93 },
    { 0x68816554, 60 },
    { 0x8d416554, 61 },
    { 0, 0 },
    { 0xfe716557, 61 },
    { 0, 0 },
    { 0xce816559, 52 },
    { 0, 0 },
    { 0xb551655b, 61 },
    { 0x7991655c, 90 },
    { 0xe9b1655b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7716564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a716567, 90 },
    { 0, 0 },
    { 0x3ca16569, 90 },
    { 0xee316569, 78 },
    { 0, 0 },
    { 0x1e31656c, 88 },
    { 0, 0 },
    { 0x27a1656e, 95 },
    { 0x4821656e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3516572, 78 },
    { 0x88a16572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e16576, 33 },
    { 0, 0 },
    { 0xd0b16578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27116582, 95 },
    { 0, 0 },
    { 0xa716584, 67 },
    { 0xcc616585, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec1658e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af16594, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa516599, 33 },
    { 0, 0 },
    { 0x4791659b, 84 },
    { 0x7ab1659c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d1659f, 95 },
    { 0, 0 },
    { 0xfbd165a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373165a6, 90 },
    { 0x2fe165a7, 87 },
    { 0x5e9165a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69165af, 98 },
    { 0x54165b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667165b7, 61 },
    { 0x3fa165b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c165bb, 95 },
    { 0xf4f165bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a165c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc165c6, 76 },
    { 0xe59165c7, 33 },
    { 0, 0 },
    { 0xb9e165c9, 81 },
    { 0xc83165ca, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c165ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18165d2, 67 },
    { 0x2ca165d3, 61 },
    { 0xd4c165d4, 36 },
    { 0, 0 },
    { 0xfa1165d6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c165dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a165e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e165e3, 84 },
    { 0, 0 },
    { 0xca9165e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x760165e8, 23 },
    { 0xeb2165e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3165f3, 33 },
    { 0xe16165f4, 95 },
    { 0x85165f5, 55 },
    { 0xf87165f5, 61 },
    { 0xc73165f4, 84 },
    { 0xca3165f8, 90 },
    { 0, 0 },
    { 0x99d165fa, 90 },
    { 0xb85165fb, 61 },
    { 0, 0 },
    { 0x2e0165fd, 95 },
    { 0xa6d165fd, 36 },
    { 0x4ce165ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b416604, 98 },
    { 0x94516604, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91660e, 76 },
    { 0xb691660f, 90 },
    { 0x5eb16610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d516616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61661b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad16620, 43 },
    { 0, 0 },
    { 0x6ce16622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d16626, 88 },
    { 0x3c316627, 34 },
    { 0xdf616628, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb1662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71662f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea16632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2816635, 52 },
    { 0x5ce16636, 98 },
    { 0x63816635, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3251663c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46316640, 84 },
    { 0xedf16641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32716644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9316648, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa601664c, 98 },
    { 0, 0 },
    { 0xe321664e, 95 },
    { 0, 0 },
    { 0xf5316650, 23 },
    { 0x8a516651, 33 },
    { 0xae916652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e16655, 101 },
    { 0, 0 },
    { 0x8c16657, 61 },
    { 0xb0016658, 98 },
    { 0x84b16659, 23 },
    { 0xf0d16659, 23 },
    { 0x87d16659, 34 },
    { 0x33916658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f1665f, 95 },
    { 0, 0 },
    { 0x52e16661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a16664, 65 },
    { 0xe6016664, 90 },
    { 0x5cc16665, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5816669, 90 },
    { 0x3fa1666a, 46 },
    { 0x2b61666b, 90 },
    { 0x8a81666b, 90 },
    { 0xe501666d, 46 },
    { 0xc551666e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d16671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5616674, 61 },
    { 0, 0 },
    { 0x56716676, 84 },
    { 0x9216677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b1667b, 90 },
    { 0x53a1667c, 85 },
    { 0, 0 },
    { 0x2fc1667e, 61 },
    { 0xe4d1667f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf116686, 76 },
    { 0x92116686, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e61668d, 52 },
    { 0x63e1668e, 23 },
    { 0, 0 },
    { 0xb4b16690, 33 },
    { 0, 0 },
    { 0x40e16692, 23 },
    { 0x93916693, 43 },
    { 0x36916694, 33 },
    { 0x4e416694, 90 },
    { 0x6a16696, 23 },
    { 0, 0 },
    { 0x2f616698, 33 },
    { 0xf9616699, 76 },
    { 0, 0 },
    { 0x6421669b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4166a3, 98 },
    { 0x836166a3, 90 },
    { 0x20b166a5, 66 },
    { 0xcb8166a6, 93 },
    { 0x9e6166a5, 43 },
    { 0xd3166a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x239166ab, 98 },
    { 0x418166ac, 17 },
    { 0x4c4166ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679166b5, 95 },
    { 0x21a166b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc166c1, 90 },
    { 0x5fc166c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1166c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f166cb, 33 },
    { 0x2d5166cc, 61 },
    { 0, 0 },
    { 0xb42166ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1166d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991166d6, 61 },
    { 0xcb5166d6, 98 },
    { 0xf4f166d8, 81 },
    { 0x542166d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519166e0, 91 },
    { 0, 0 },
    { 0x26d166e2, 90 },
    { 0x2b0166e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993166ec, 61 },
    { 0xdab166ec, 87 },
    { 0x7d2166ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x646166f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526166fa, 34 },
    { 0x5ec166fb, 90 },
    { 0, 0 },
    { 0x57f166fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1616702, 96 },
    { 0x67516703, 66 },
    { 0x83f16704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd916709, 68 },
    { 0x16e1670a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd16711, 76 },
    { 0x82f16711, 65 },
    { 0xe0616713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c16718, 90 },
    { 0x4cd16719, 61 },
    { 0, 0 },
    { 0xf2d1671b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471671e, 93 },
    { 0xaf1671f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae16724, 100 },
    { 0x89116725, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7751672a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac16730, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f216736, 55 },
    { 0xd5c16737, 95 },
    { 0xf3016737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e1673c, 61 },
    { 0x50f1673d, 98 },
    { 0xe1a1673e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16751, 33 },
    { 0x65916752, 90 },
    { 0, 0 },
    { 0x50d16754, 46 },
    { 0x13e16755, 75 },
    { 0, 0 },
    { 0xa0916757, 55 },
    { 0xdaf16758, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c1675f, 98 },
    { 0, 0 },
    { 0x85a16761, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d816767, 90 },
    { 0, 0 },
    { 0x86216769, 52 },
    { 0x20316769, 56 },
    { 0x1261676b, 67 },
    { 0xf371676b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x491676f, 66 },
    { 0x47216770, 33 },
    { 0xc516771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c016774, 23 },
    { 0x61f16775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c16779, 35 },
    { 0xd491677a, 99 },
    { 0x9f01677b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a11677f, 23 },
    { 0xa1816780, 90 },
    { 0xa3416780, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed16785, 61 },
    { 0x5e816785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e16789, 90 },
    { 0, 0 },
    { 0x3551678b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca1678e, 90 },
    { 0xf881678f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f16793, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d16797, 93 },
    { 0, 0 },
    { 0x49716799, 34 },
    { 0x64016799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f167a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a167a5, 90 },
    { 0xf2b167a5, 95 },
    { 0x973167a7, 82 },
    { 0x943167a7, 98 },
    { 0x91c167a8, 83 },
    { 0xa04167aa, 90 },
    { 0x1cc167ab, 77 },
    { 0x865167ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9167af, 38 },
    { 0x558167b0, 99 },
    { 0, 0 },
    { 0x289167b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9167b5, 33 },
    { 0, 0 },
    { 0x81f167b7, 90 },
    { 0xdac167b8, 51 },
    { 0xd11167b7, 33 },
    { 0xcd8167b7, 84 },
    { 0x602167b7, 90 },
    { 0, 0 },
    { 0x8d167bd, 76 },
    { 0x33f167be, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77167c2, 90 },
    { 0, 0 },
    { 0x573167c4, 95 },
    { 0xef0167c5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99167d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b167db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e167de, 41 },
    { 0, 0 },
    { 0x323167e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42167e4, 33 },
    { 0x47a167e5, 38 },
    { 0x317167e6, 95 },
    { 0x4ed167e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d167ea, 23 },
    { 0x11167eb, 33 },
    { 0x92e167ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1167ef, 95 },
    { 0xf8c167ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246167f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad167f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1167fc, 46 },
    { 0x8fc167fd, 95 },
    { 0, 0 },
    { 0x6c2167ff, 23 },
    { 0x8fd167ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa16804, 55 },
    { 0xd6416805, 33 },
    { 0x52016806, 95 },
    { 0x5e216804, 67 },
    { 0xf5f16808, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6971680b, 38 },
    { 0x5281680c, 23 },
    { 0x9b31680d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c516815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b16818, 76 },
    { 0xcfe16819, 23 },
    { 0x1ad1681a, 43 },
    { 0x7011681a, 93 },
    { 0x20216819, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39016820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f16823, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23816827, 33 },
    { 0x49a16828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe661682b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c51682f, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b16833, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e16838, 87 },
    { 0xc1616839, 23 },
    { 0xb6716839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381683e, 90 },
    { 0x4701683f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c16843, 61 },
    { 0x7e316843, 33 },
    { 0xbda16843, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d11684b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa341684f, 98 },
    { 0x3ae16850, 95 },
    { 0, 0 },
    { 0x85a16852, 98 },
    { 0x63316853, 33 },
    { 0x51516853, 90 },
    { 0x30616855, 33 },
    { 0xdfe16856, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c61685f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e416862, 87 },
    { 0x62f16863, 34 },
    { 0xee216862, 93 },
    { 0x2d716865, 95 },
    { 0xa6316862, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x23916869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc1686c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3201686f, 95 },
    { 0, 0 },
    { 0xa6716871, 90 },
    { 0x97d16872, 23 },
    { 0xc1216873, 67 },
    { 0x6e616874, 57 },
    { 0x94816875, 85 },
    { 0, 0 },
    { 0x6bc16877, 87 },
    { 0, 0 },
    { 0x1c116879, 98 },
    { 0x6391687a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b116880, 61 },
    { 0, 0 },
    { 0xb6c16882, 33 },
    { 0, 0 },
    { 0x4b16884, 33 },
    { 0, 0 },
    { 0xe2a16886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94616889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3561688d, 43 },
    { 0xbff1688e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca616892, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf916897, 55 },
    { 0xcf716897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1689b, 33 },
    { 0x6fc1689c, 61 },
    { 0x3921689c, 90 },
    { 0xf01689e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a168a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe168a5, 61 },
    { 0, 0 },
    { 0x31b168a7, 17 },
    { 0xae6168a8, 33 },
    { 0x887168a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325168ae, 52 },
    { 0, 0 },
    { 0x6ba168b0, 87 },
    { 0x5aa168b0, 90 },
    { 0, 0 },
    { 0x92c168b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668168bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9168c0, 34 },
    { 0x429168c1, 43 },
    { 0, 0 },
    { 0xe9168c3, 43 },
    { 0xd98168c3, 68 },
    { 0, 0 },
    { 0xd48168c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899168c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7168cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992168d1, 40 },
    { 0xfed168d1, 61 },
    { 0, 0 },
    { 0x536168d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f168d7, 69 },
    { 0xd3f168d7, 100 },
    { 0x2ec168d7, 95 },
    { 0xcc2168da, 61 },
    { 0x849168da, 43 },
    { 0xdf2168dc, 78 },
    { 0x9e3168dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x331168e0, 99 },
    { 0x83e168e1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6168e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e168ee, 76 },
    { 0xe70168ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33168f2, 98 },
    { 0xcf2168f3, 33 },
    { 0x774168f4, 98 },
    { 0xadc168f3, 61 },
    { 0x9c8168f6, 98 },
    { 0x354168f3, 90 },
    { 0xc3f168f8, 84 },
    { 0xa5c168f9, 34 },
    { 0x471168f5, 81 },
    { 0xe65168fb, 52 },
    { 0x660168fc, 84 },
    { 0, 0 },
    { 0xff6168fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0416901, 61 },
    { 0x1c416901, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24316906, 93 },
    { 0, 0 },
    { 0x5aa16908, 101 },
    { 0xc1d16909, 98 },
    { 0, 0 },
    { 0x60b1690b, 78 },
    { 0, 0 },
    { 0xa391690d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x81616910, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9516916, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4331691c, 68 },
    { 0, 0 },
    { 0x5591691e, 34 },
    { 0x7c61691e, 33 },
    { 0x88516920, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3516924, 90 },
    { 0, 0 },
    { 0x88916926, 76 },
    { 0x5ba16926, 90 },
    { 0x40516928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd791692f, 23 },
    { 0x93f16930, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15916936, 23 },
    { 0, 0 },
    { 0xaaf16938, 95 },
    { 0x7c416939, 23 },
    { 0x2e61693a, 80 },
    { 0x4c316938, 52 },
    { 0, 0 },
    { 0x1871693d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee16943, 23 },
    { 0x94f16944, 98 },
    { 0xc4916944, 61 },
    { 0x1b516946, 78 },
    { 0, 0 },
    { 0x7f416948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4951694d, 76 },
    { 0xdc91694e, 76 },
    { 0, 0 },
    { 0xc9516950, 95 },
    { 0x53116950, 98 },
    { 0x8de16952, 100 },
    { 0x8e216952, 95 },
    { 0xf7316952, 46 },
    { 0xc1116951, 33 },
    { 0xb116954, 40 },
    { 0x1d016953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c1695a, 61 },
    { 0xf671695b, 85 },
    { 0x3921695b, 91 },
    { 0xc8f1695d, 55 },
    { 0xe961695b, 95 },
    { 0xd7b1695f, 33 },
    { 0xf516960, 98 },
    { 0xce11695d, 33 },
    { 0xb9416962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9416965, 34 },
    { 0xbdf16965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58816969, 23 },
    { 0x6cb16969, 66 },
    { 0xfa71696b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab21696f, 95 },
    { 0, 0 },
    { 0xec16971, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0516974, 8 },
    { 0x49416975, 43 },
    { 0x57b16976, 34 },
    { 0x3dd16977, 33 },
    { 0, 0 },
    { 0x50b16979, 95 },
    { 0, 0 },
    { 0xf291697b, 61 },
    { 0xa311697c, 90 },
    { 0, 0 },
    { 0xf851697e, 61 },
    { 0, 0 },
    { 0x38616980, 95 },
    { 0, 0 },
    { 0x4a16982, 23 },
    { 0x9316983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7916989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe81698d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e16992, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83816997, 55 },
    { 0, 0 },
    { 0x1d916999, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa41699c, 85 },
    { 0, 0 },
    { 0x6351699e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b169a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d169a7, 52 },
    { 0, 0 },
    { 0xb07169a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4169b3, 90 },
    { 0x472169b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5169c0, 33 },
    { 0xd0a169c1, 42 },
    { 0x8cc169c2, 17 },
    { 0xd03169c3, 38 },
    { 0, 0 },
    { 0x1a4169c5, 99 },
    { 0x412169c6, 95 },
    { 0xcf169c7, 95 },
    { 0xc92169c6, 78 },
    { 0x418169c9, 78 },
    { 0x777169ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba169cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee169d1, 43 },
    { 0xab8169d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165169d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46169db, 55 },
    { 0x7a169db, 33 },
    { 0x14b169dd, 95 },
    { 0x232169dd, 23 },
    { 0x8b0169dd, 98 },
    { 0, 0 },
    { 0xfd5169e1, 43 },
    { 0xa67169e2, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab169ee, 25 },
    { 0, 0 },
    { 0xed0169f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c169f3, 98 },
    { 0x62f169f4, 23 },
    { 0xc08169f5, 33 },
    { 0x7a6169f3, 61 },
    { 0x409169f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4169fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea169fd, 61 },
    { 0x537169fe, 90 },
    { 0xc3e169ff, 33 },
    { 0x1a3169ff, 33 },
    { 0x6f0169fd, 52 },
    { 0, 0 },
    { 0x7e816a03, 83 },
    { 0xc1716a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e16a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf916a0d, 81 },
    { 0x4aa16a0d, 93 },
    { 0x1fa16a0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3216a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae616a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15116a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0316a23, 90 },
    { 0x96316a23, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe016a27, 61 },
    { 0, 0 },
    { 0x26516a29, 98 },
    { 0x79616a2a, 33 },
    { 0x5e016a29, 95 },
    { 0xe7f16a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98716a2f, 34 },
    { 0x66116a30, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de16a33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16816a37, 23 },
    { 0xf5916a38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b116a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15616a41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e16a45, 98 },
    { 0xc5d16a45, 34 },
    { 0, 0 },
    { 0x82a16a48, 43 },
    { 0x13716a48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b16a4c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e916a4f, 33 },
    { 0, 0 },
    { 0x51016a51, 95 },
    { 0, 0 },
    { 0x21f16a53, 65 },
    { 0xf7716a54, 38 },
    { 0x64c16a53, 55 },
    { 0x9716a55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b216a59, 33 },
    { 0, 0 },
    { 0xe7916a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec016a60, 98 },
    { 0x28816a61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4016a65, 91 },
    { 0x5bd16a66, 55 },
    { 0, 0 },
    { 0x44c16a68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb816a6d, 90 },
    { 0x36f16a6e, 23 },
    { 0x9a516a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d16a72, 84 },
    { 0x3c116a73, 90 },
    { 0xa1816a74, 33 },
    { 0x44f16a72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b16a79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63416a85, 46 },
    { 0xa5f16a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de16a8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf716a90, 76 },
    { 0, 0 },
    { 0xf2d16a92, 90 },
    { 0x6bc16a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b16a96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d16a9a, 93 },
    { 0xbd616a9a, 46 },
    { 0xc9d16a9c, 90 },
    { 0, 0 },
    { 0xaf616a9e, 80 },
    { 0, 0 },
    { 0xf1a16aa0, 90 },
    { 0x52316aa0, 33 },
    { 0xee116aa0, 100 },
    { 0, 0 },
    { 0x51116aa4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec716aa9, 69 },
    { 0x4e416aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0816ab8, 45 },
    { 0x50d16ab8, 23 },
    { 0x53416aba, 17 },
    { 0x56f16abb, 33 },
    { 0x66a16abb, 76 },
    { 0x74116abd, 65 },
    { 0xd2c16abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2716ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd16ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90316acc, 90 },
    { 0xec316acc, 55 },
    { 0x4fd16ace, 95 },
    { 0, 0 },
    { 0xc3116ad0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d16ad4, 76 },
    { 0xc4916ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91816ada, 76 },
    { 0, 0 },
    { 0x13e16adc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1016ae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb16ae8, 33 },
    { 0, 0 },
    { 0x6c316aea, 61 },
    { 0xa7e16aea, 67 },
    { 0xfa616aec, 91 },
    { 0xcea16aec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25916afa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d16b05, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c816b0a, 23 },
    { 0, 0 },
    { 0xabf16b0c, 98 },
    { 0xfc116b0c, 90 },
    { 0x9b216b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4116b16, 76 },
    { 0x19d16b16, 33 },
    { 0, 0 },
    { 0x43116b19, 46 },
    { 0x84e16b1a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd16b1d, 95 },
    { 0, 0 },
    { 0x8db16b1f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a16b22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9916b25, 12 },
    { 0x71c16b26, 85 },
    { 0, 0 },
    { 0x40716b28, 90 },
    { 0x19f16b29, 90 },
    { 0xba516b28, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34616b2e, 36 },
    { 0, 0 },
    { 0xe2816b30, 90 },
    { 0, 0 },
    { 0xc0416b32, 95 },
    { 0xde716b33, 76 },
    { 0, 0 },
    { 0x3d816b35, 76 },
    { 0xe3516b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9716b40, 34 },
    { 0xa2d16b40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47816b49, 90 },
    { 0, 0 },
    { 0x4aa16b4b, 93 },
    { 0xf8116b4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f16b51, 100 },
    { 0x48316b52, 98 },
    { 0, 0 },
    { 0x13916b54, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x10716b57, 95 },
    { 0xfd216b58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63616b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5316b60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa216b63, 75 },
    { 0xf9516b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e16b6d, 91 },
    { 0, 0 },
    { 0x83c16b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a16b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64816b7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a816b83, 85 },
    { 0xb3d16b83, 87 },
    { 0xd9f16b85, 78 },
    { 0x53416b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c16b89, 56 },
    { 0xb3916b89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a16b8e, 46 },
    { 0x7916b8f, 61 },
    { 0xd1a16b90, 98 },
    { 0xb2b16b91, 55 },
    { 0x88716b91, 90 },
    { 0, 0 },
    { 0x65a16b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f516b97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6816b9b, 61 },
    { 0x81e16b9b, 34 },
    { 0x7a316b9b, 90 },
    { 0xb416b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc16ba1, 34 },
    { 0xcf416ba2, 98 },
    { 0x2cc16ba3, 75 },
    { 0x5c816ba4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e16ba7, 84 },
    { 0xa0b16ba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80116bab, 61 },
    { 0x83516bab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b16baf, 33 },
    { 0x7416bb0, 68 },
    { 0x25816bb1, 55 },
    { 0x5cf16bb0, 85 },
    { 0xdba16bb3, 61 },
    { 0x5e516bb2, 95 },
    { 0xd3b16bb5, 61 },
    { 0x94b16bb5, 23 },
    { 0x60216bb5, 23 },
    { 0x68816bb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a316bbc, 76 },
    { 0, 0 },
    { 0xa5d16bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93216bc2, 55 },
    { 0x83916bc3, 82 },
    { 0, 0 },
    { 0x40a16bc5, 43 },
    { 0x27f16bc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9016bcb, 90 },
    { 0x8b716bcc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d16bcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d416bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc16bde, 34 },
    { 0, 0 },
    { 0x42516be0, 65 },
    { 0x3eb16be0, 61 },
    { 0x2dc16be2, 90 },
    { 0, 0 },
    { 0x34416be4, 80 },
    { 0xe5116be4, 90 },
    { 0xd016be6, 95 },
    { 0, 0 },
    { 0x24416be8, 76 },
    { 0, 0 },
    { 0x2e416bea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87416bee, 95 },
    { 0xc3516bee, 90 },
    { 0x7d916bee, 98 },
    { 0xb8816bf0, 93 },
    { 0x38316bf2, 90 },
    { 0xfde16bf3, 93 },
    { 0x93016bf4, 76 },
    { 0xf5116bf2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe416bf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f16c00, 98 },
    { 0xcbf16c01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c16c04, 33 },
    { 0xe416c05, 98 },
    { 0xc4816c06, 76 },
    { 0, 0 },
    { 0xc2016c08, 61 },
    { 0x61f16c09, 90 },
    { 0x79916c08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e416c10, 66 },
    { 0x37016c10, 98 },
    { 0, 0 },
    { 0x57f16c13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b16c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71016c20, 90 },
    { 0x90316c21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d916c24, 98 },
    { 0, 0 },
    { 0x4d616c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d16c2e, 51 },
    { 0, 0 },
    { 0xd2d16c30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94516c36, 90 },
    { 0xa9016c37, 90 },
    { 0xf8c16c37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb16c3d, 93 },
    { 0, 0 },
    { 0x8c016c3f, 66 },
    { 0xab116c3f, 23 },
    { 0x84d16c41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f816c44, 87 },
    { 0xe8a16c44, 98 },
    { 0xb0216c44, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e016c49, 34 },
    { 0x78816c4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f716c56, 98 },
    { 0x2e816c57, 95 },
    { 0xc7c16c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4016c5c, 61 },
    { 0x56816c5c, 43 },
    { 0x7d916c5e, 76 },
    { 0, 0 },
    { 0x8016c60, 67 },
    { 0x81116c61, 36 },
    { 0, 0 },
    { 0x80e16c63, 61 },
    { 0x8e716c64, 61 },
    { 0xa2c16c65, 95 },
    { 0xc4516c66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa816c69, 76 },
    { 0, 0 },
    { 0xe3816c6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x64716c6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x89716c71, 90 },
    { 0x9c716c72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c16c7a, 61 },
    { 0x3e316c7b, 100 },
    { 0, 0 },
    { 0x3ee16c7d, 23 },
    { 0x25216c7e, 61 },
    { 0x40416c7f, 23 },
    { 0, 0 },
    { 0x5f816c81, 98 },
    { 0, 0 },
    { 0x12a16c83, 95 },
    { 0x79216c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9516c87, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff16c8f, 40 },
    { 0xbd516c90, 61 },
    { 0, 0 },
    { 0x8c716c92, 76 },
    { 0x92416c93, 33 },
    { 0x2116c93, 66 },
    { 0xd316c93, 95 },
    { 0xfc16c96, 83 },
    { 0x67016c97, 90 },
    { 0x27516c92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20616c9d, 67 },
    { 0x8f216c9d, 95 },
    { 0x89b16c9d, 95 },
    { 0x28516ca0, 93 },
    { 0xa7e16c9e, 23 },
    { 0x99616c9d, 61 },
    { 0, 0 },
    { 0x28416ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ca7, 61 },
    { 0xa9f16ca7, 10 },
    { 0x9df16ca9, 84 },
    { 0x6ac16ca8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de16cad, 90 },
    { 0, 0 },
    { 0x4d16caf, 98 },
    { 0x6e916cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e916cb3, 23 },
    { 0, 0 },
    { 0x6916cb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816cbb, 43 },
    { 0, 0 },
    { 0x7fe16cbd, 90 },
    { 0xabf16cbd, 23 },
    { 0xb4e16cbd, 33 },
    { 0, 0 },
    { 0x53c16cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d116cc7, 77 },
    { 0x90216cc7, 34 },
    { 0xc8516cc8, 33 },
    { 0x7016cca, 23 },
    { 0xb9e16ccb, 33 },
    { 0xbb816ccc, 53 },
    { 0, 0 },
    { 0xe6d16cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8616cd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63616cd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f16cd9, 91 },
    { 0, 0 },
    { 0x3ab16cdb, 23 },
    { 0xcc16cdb, 80 },
    { 0x9ab16cdd, 95 },
    { 0xfab16cde, 43 },
    { 0x72a16cde, 33 },
    { 0, 0 },
    { 0x9d316ce1, 61 },
    { 0x7fb16ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416ce5, 55 },
    { 0, 0 },
    { 0xc6c16ce7, 98 },
    { 0x5ce16ce8, 90 },
    { 0x73816ce9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a16cef, 91 },
    { 0x9cf16cf0, 95 },
    { 0xed016cf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x87516cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c16cfa, 98 },
    { 0x84716cfb, 33 },
    { 0x39516cfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36516d00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41216d07, 90 },
    { 0x79816d08, 93 },
    { 0, 0 },
    { 0xf8e16d0a, 34 },
    { 0x60416d0b, 43 },
    { 0, 0 },
    { 0x116d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f16d10, 33 },
    { 0x1da16d11, 61 },
    { 0x1ef16d12, 23 },
    { 0x1b416d13, 90 },
    { 0, 0 },
    { 0x2c816d15, 99 },
    { 0, 0 },
    { 0x9fd16d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4316d1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9316d21, 78 },
    { 0x11516d21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1916d25, 90 },
    { 0xdc316d25, 91 },
    { 0x20a16d25, 33 },
    { 0xf2816d27, 69 },
    { 0x57016d29, 95 },
    { 0x9c416d2a, 93 },
    { 0, 0 },
    { 0x3b216d2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb416d2f, 61 },
    { 0, 0 },
    { 0xedd16d31, 90 },
    { 0, 0 },
    { 0x80716d33, 90 },
    { 0, 0 },
    { 0xe0a16d35, 52 },
    { 0x42a16d36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a16d3a, 90 },
    { 0x86016d3b, 61 },
    { 0x34016d3b, 95 },
    { 0, 0 },
    { 0xe0e16d3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e16d42, 23 },
    { 0x1616d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e16d48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81916d4c, 90 },
    { 0xbe16d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb16d50, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71616d54, 95 },
    { 0, 0 },
    { 0x22716d56, 40 },
    { 0xd5516d57, 76 },
    { 0x62516d58, 78 },
    { 0x57016d56, 98 },
    { 0, 0 },
    { 0x47916d5b, 55 },
    { 0xcb916d5c, 91 },
    { 0xbf16d5d, 93 },
    { 0x4e016d5e, 77 },
    { 0x85016d5f, 33 },
    { 0x89f16d5e, 93 },
    { 0xae016d61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb16d66, 76 },
    { 0x47416d67, 68 },
    { 0xaba16d67, 36 },
    { 0, 0 },
    { 0x38016d6a, 23 },
    { 0x5d516d6b, 23 },
    { 0x4a916d6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1516d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a616d72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36416d75, 61 },
    { 0x18316d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc516d79, 55 },
    { 0xf9516d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c16d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d16d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f016d8a, 34 },
    { 0x35f16d8b, 55 },
    { 0, 0 },
    { 0xaf616d8d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb616d92, 98 },
    { 0xe4f16d93, 76 },
    { 0x55116d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44616d9a, 100 },
    { 0x6416d9b, 90 },
    { 0x98416d9b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x57316d9f, 95 },
    { 0, 0 },
    { 0x89916da1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98316da7, 34 },
    { 0xdfa16da8, 98 },
    { 0xfde16da9, 95 },
    { 0xd4116da9, 91 },
    { 0, 0 },
    { 0x9ce16dac, 90 },
    { 0xa3116dad, 55 },
    { 0, 0 },
    { 0xde816daf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe416db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b16dc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f16dce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd16dd1, 23 },
    { 0xa2e16dd2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a16dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa16ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c016ddf, 46 },
    { 0x14e16de0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98916de5, 46 },
    { 0x8b216de6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c916deb, 36 },
    { 0xc5416dec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71816df0, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e16df7, 78 },
    { 0x9e816df8, 90 },
    { 0xb4416df9, 90 },
    { 0xab216df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27016dfe, 84 },
    { 0, 0 },
    { 0x31116e00, 85 },
    { 0xd3516e00, 98 },
    { 0x4bd16e02, 61 },
    { 0x96416e03, 34 },
    { 0x82c16e04, 98 },
    { 0xd8816e05, 90 },
    { 0x95d16e05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66416e0d, 52 },
    { 0xbcf16e0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a16e11, 33 },
    { 0x26c16e11, 98 },
    { 0x2ea16e12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd516e1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c016e27, 43 },
    { 0x45416e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc416e2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3716e31, 44 },
    { 0x44c16e32, 95 },
    { 0xe3416e32, 43 },
    { 0x82f16e31, 23 },
    { 0xd7116e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c16e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28516e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a16e41, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9816e44, 23 },
    { 0, 0 },
    { 0x29716e46, 85 },
    { 0xe1e16e47, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad716e4a, 33 },
    { 0xb8616e4b, 55 },
    { 0x67d16e4c, 95 },
    { 0x6c816e4d, 66 },
    { 0x3e016e4e, 59 },
    { 0xc216e4c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4316e55, 100 },
    { 0xa316e56, 90 },
    { 0x73216e56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0916e5a, 33 },
    { 0, 0 },
    { 0x82916e5c, 17 },
    { 0x9ff16e5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81816e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88816e68, 33 },
    { 0x34c16e69, 91 },
    { 0x1ca16e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11216e6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f516e75, 52 },
    { 0xe916e76, 33 },
    { 0xbf16e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67116e7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f416e80, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec916e86, 43 },
    { 0, 0 },
    { 0x57a16e88, 43 },
    { 0, 0 },
    { 0x8a516e8a, 83 },
    { 0xd4b16e8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f16e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f16e93, 34 },
    { 0, 0 },
    { 0xd5416e95, 91 },
    { 0xbe116e95, 90 },
    { 0xe4f16e97, 91 },
    { 0x2a416e98, 69 },
    { 0x26816e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf16e9c, 95 },
    { 0, 0 },
    { 0x23816e9e, 61 },
    { 0xf616e9f, 77 },
    { 0, 0 },
    { 0xb116ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8916ea9, 90 },
    { 0xacd16ea9, 23 },
    { 0xe6516eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec16eaf, 90 },
    { 0, 0 },
    { 0x53f16eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d916ec1, 95 },
    { 0x23816ec2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf16ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d16ecb, 98 },
    { 0xde616ecb, 33 },
    { 0x1bf16ecc, 61 },
    { 0, 0 },
    { 0x23c16ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b16ed6, 23 },
    { 0xfb216ed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a316edb, 90 },
    { 0, 0 },
    { 0xafd16edd, 46 },
    { 0, 0 },
    { 0x85a16edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5016ee3, 66 },
    { 0x68116ee4, 34 },
    { 0xb5616ee3, 84 },
    { 0x4c216ee6, 66 },
    { 0, 0 },
    { 0xe9b16ee8, 61 },
    { 0xf916ee8, 95 },
    { 0xe0816ee8, 95 },
    { 0, 0 },
    { 0x28e16eec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf216ef0, 88 },
    { 0, 0 },
    { 0x1b416ef2, 33 },
    { 0, 0 },
    { 0x3016ef4, 90 },
    { 0x4516ef5, 61 },
    { 0xae816ef5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec16eff, 61 },
    { 0x70316eff, 98 },
    { 0x38716f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e616f06, 61 },
    { 0, 0 },
    { 0x9ee16f08, 46 },
    { 0xdfa16f09, 33 },
    { 0, 0 },
    { 0x10116f0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd116f0e, 77 },
    { 0x74216f0f, 33 },
    { 0x37b16f10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a16f13, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f16f18, 95 },
    { 0, 0 },
    { 0x57416f1a, 76 },
    { 0x9c516f1a, 43 },
    { 0, 0 },
    { 0xb5816f1d, 90 },
    { 0, 0 },
    { 0x1e416f1f, 76 },
    { 0x3eb16f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc016f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e16f28, 95 },
    { 0, 0 },
    { 0xf2216f2a, 33 },
    { 0xa7e16f2b, 85 },
    { 0, 0 },
    { 0x72a16f2d, 33 },
    { 0xe0016f2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e316f32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee16f38, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc16f3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c16f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda16f46, 33 },
    { 0, 0 },
    { 0x7fb16f48, 52 },
    { 0, 0 },
    { 0x7e316f4a, 95 },
    { 0, 0 },
    { 0x88316f4c, 23 },
    { 0, 0 },
    { 0xd9f16f4e, 61 },
    { 0x4316f4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a16f54, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd216f5b, 78 },
    { 0, 0 },
    { 0x2d816f5d, 68 },
    { 0x95016f5d, 90 },
    { 0x23e16f5f, 95 },
    { 0x36116f60, 33 },
    { 0xaee16f61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea16f6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x98416f72, 95 },
    { 0x87d16f73, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xee616f76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77416f79, 98 },
    { 0, 0 },
    { 0x21a16f7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa016f81, 61 },
    { 0xfd916f82, 46 },
    { 0x45716f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba16f86, 93 },
    { 0, 0 },
    { 0x716f88, 37 },
    { 0, 0 },
    { 0xe5116f8a, 95 },
    { 0, 0 },
    { 0x88a16f8c, 55 },
    { 0x11216f8c, 65 },
    { 0, 0 },
    { 0xccb16f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae16f9c, 55 },
    { 0x45116f9d, 33 },
    { 0, 0 },
    { 0x89216f9f, 98 },
    { 0, 0 },
    { 0xf9616fa1, 76 },
    { 0x83816fa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b016fb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47816fb6, 61 },
    { 0xabb16fb6, 95 },
    { 0x61216fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e016fbc, 23 },
    { 0, 0 },
    { 0x9f416fbe, 66 },
    { 0xe4016fbf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77516fc5, 93 },
    { 0xf5e16fc5, 33 },
    { 0xadd16fc7, 90 },
    { 0xc0416fc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c316fcc, 40 },
    { 0, 0 },
    { 0xa5916fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da16fd2, 61 },
    { 0, 0 },
    { 0x84616fd4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af16fd9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b16fdc, 98 },
    { 0xbe616fdd, 67 },
    { 0x71516fdd, 95 },
    { 0xc8c16fdf, 33 },
    { 0x82416fde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f16fe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d16fe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e816fee, 90 },
    { 0x5f316fee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x83116ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26716ff7, 34 },
    { 0x51316ff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67117002, 34 },
    { 0x27317002, 43 },
    { 0, 0 },
    { 0xae217005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa371700a, 46 },
    { 0, 0 },
    { 0xfd21700c, 95 },
    { 0, 0 },
    { 0xbf71700e, 34 },
    { 0x2d21700f, 61 },
    { 0x3291700f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e17013, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10017016, 90 },
    { 0x19e17016, 90 },
    { 0x86417018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb111701c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed1701f, 56 },
    { 0, 0 },
    { 0x91717021, 33 },
    { 0x9fe17021, 87 },
    { 0xc917023, 43 },
    { 0x4e617023, 95 },
    { 0x6fb17025, 61 },
    { 0xf1017025, 55 },
    { 0x4a017027, 33 },
    { 0x3c517028, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e1702f, 100 },
    { 0, 0 },
    { 0x13f17031, 98 },
    { 0xd1a17032, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c17035, 90 },
    { 0xb2b17035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f417046, 44 },
    { 0, 0 },
    { 0x29b17048, 75 },
    { 0, 0 },
    { 0xc5a1704a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2601704d, 33 },
    { 0x9bb1704d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb517051, 95 },
    { 0, 0 },
    { 0xe7517053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9621705a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301705d, 88 },
    { 0x21705e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a17061, 76 },
    { 0, 0 },
    { 0x29c17063, 87 },
    { 0, 0 },
    { 0xa6217065, 23 },
    { 0x6ca17065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d17069, 33 },
    { 0, 0 },
    { 0x13a1706b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8217076, 76 },
    { 0x65b17076, 55 },
    { 0xb1717077, 33 },
    { 0xf2117079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc1707c, 90 },
    { 0x4921707d, 23 },
    { 0x17b1707d, 33 },
    { 0x4c11707f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36817084, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43917088, 93 },
    { 0x31717088, 76 },
    { 0xe9c1708a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab1708f, 99 },
    { 0, 0 },
    { 0x12d17091, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7317095, 95 },
    { 0xef717096, 90 },
    { 0xa2917096, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81709a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd01709d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab170a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a170a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56170aa, 77 },
    { 0x949170aa, 90 },
    { 0x1e2170ac, 66 },
    { 0x9a9170ac, 76 },
    { 0, 0 },
    { 0x320170af, 61 },
    { 0xda2170b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80170b7, 55 },
    { 0, 0 },
    { 0x33170b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8170bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f170c2, 23 },
    { 0xe78170c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c170c8, 76 },
    { 0xe3d170c9, 96 },
    { 0xe53170ca, 98 },
    { 0x426170c9, 92 },
    { 0xc37170cc, 23 },
    { 0xc07170cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3170d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9170d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9170e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e170e7, 95 },
    { 0, 0 },
    { 0xe97170e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37170ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f170f0, 43 },
    { 0, 0 },
    { 0x50c170f2, 95 },
    { 0, 0 },
    { 0xf02170f4, 90 },
    { 0x6dd170f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47170f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581170fc, 90 },
    { 0xa45170fc, 69 },
    { 0, 0 },
    { 0xb91170ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b617103, 61 },
    { 0xdfc17104, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed17107, 73 },
    { 0xd3317108, 55 },
    { 0xc4d17109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41710d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7617112, 90 },
    { 0x2ce17112, 90 },
    { 0, 0 },
    { 0x42b17115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11711c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7491711f, 33 },
    { 0x70317120, 61 },
    { 0xabe17121, 76 },
    { 0xeb517122, 46 },
    { 0x7a117120, 90 },
    { 0, 0 },
    { 0x28d17125, 93 },
    { 0x60817125, 99 },
    { 0xdc617127, 93 },
    { 0x29d17126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41712b, 76 },
    { 0x68d1712c, 55 },
    { 0x5f11712d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x70717130, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe17134, 33 },
    { 0xea817135, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5717139, 90 },
    { 0x9581713a, 19 },
    { 0x25b1713b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db1713e, 23 },
    { 0x2a01713f, 90 },
    { 0, 0 },
    { 0x7217141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79617145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f017149, 95 },
    { 0xe5f1714a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee81714f, 34 },
    { 0x11d1714f, 61 },
    { 0, 0 },
    { 0xb2c17152, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3317156, 95 },
    { 0, 0 },
    { 0x6ad17158, 33 },
    { 0x13c17159, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c1715c, 23 },
    { 0xfd81715d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf17165, 55 },
    { 0xe3a17166, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831716b, 40 },
    { 0, 0 },
    { 0x4271716d, 61 },
    { 0, 0 },
    { 0xc7b1716f, 23 },
    { 0, 0 },
    { 0xd4817171, 33 },
    { 0, 0 },
    { 0xfbe17173, 66 },
    { 0x91517174, 90 },
    { 0xd4c17174, 90 },
    { 0x64717176, 84 },
    { 0x3a817177, 43 },
    { 0x40e17178, 43 },
    { 0x66817177, 99 },
    { 0xc3b17179, 40 },
    { 0xc0f17177, 95 },
    { 0, 0 },
    { 0xd6b1717d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40917181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f17186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811718e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef217191, 34 },
    { 0xc4a17192, 40 },
    { 0x5c117193, 98 },
    { 0x9f717194, 91 },
    { 0xeab17195, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f217198, 91 },
    { 0, 0 },
    { 0xd21719a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7141719f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731171a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1171aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4171ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3171b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903171bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7171c0, 90 },
    { 0x548171c1, 91 },
    { 0xa27171c0, 88 },
    { 0, 0 },
    { 0xf05171c4, 23 },
    { 0xc12171c4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3171c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x539171cb, 61 },
    { 0x286171cb, 46 },
    { 0x705171cd, 43 },
    { 0x4ef171ce, 61 },
    { 0, 0 },
    { 0xe42171d0, 76 },
    { 0xa02171d0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df171d9, 98 },
    { 0xe46171da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5171e1, 98 },
    { 0x11d171e2, 90 },
    { 0xa1b171e1, 33 },
    { 0x2d0171e3, 90 },
    { 0, 0 },
    { 0x9c6171e6, 46 },
    { 0x732171e7, 48 },
    { 0, 0 },
    { 0xcaa171e9, 90 },
    { 0xc3b171ea, 23 },
    { 0, 0 },
    { 0x1e4171ec, 46 },
    { 0x63b171ed, 91 },
    { 0x65d171ed, 61 },
    { 0xe47171ed, 95 },
    { 0x700171ef, 55 },
    { 0x7e5171f1, 61 },
    { 0xab6171f1, 46 },
    { 0xbb8171f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40171f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0171fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e171fd, 90 },
    { 0xd1a171fe, 33 },
    { 0xb66171ff, 61 },
    { 0xc0e171fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3217203, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7117208, 76 },
    { 0, 0 },
    { 0xa671720a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31720f, 23 },
    { 0x18c17210, 33 },
    { 0xd0117211, 100 },
    { 0x36217210, 85 },
    { 0x8a017213, 46 },
    { 0, 0 },
    { 0xe8f17215, 61 },
    { 0x95917215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e1721a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb41721d, 95 },
    { 0x8831721e, 95 },
    { 0x6c1721e, 101 },
    { 0x5aa1721f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x75717223, 65 },
    { 0xfc417224, 61 },
    { 0xa0d17223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d117228, 99 },
    { 0xf2317229, 61 },
    { 0x1611722a, 78 },
    { 0, 0 },
    { 0x56f1722c, 98 },
    { 0, 0 },
    { 0xc1a1722e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d17235, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc41723b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3191723f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23017242, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d617246, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a17249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1724d, 23 },
    { 0, 0 },
    { 0x39c1724f, 95 },
    { 0x8241724f, 99 },
    { 0x5d817251, 98 },
    { 0xe3717252, 43 },
    { 0, 0 },
    { 0x92e17254, 43 },
    { 0xd217255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f17259, 98 },
    { 0, 0 },
    { 0xda81725b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc1725f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27617264, 61 },
    { 0xd017265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77517269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71726e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac917274, 76 },
    { 0x71217275, 55 },
    { 0x94617274, 46 },
    { 0x67517274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571727f, 61 },
    { 0x5cb17280, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f417285, 43 },
    { 0x3cf17286, 33 },
    { 0x46a17287, 93 },
    { 0xed117288, 34 },
    { 0xdad17289, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1728c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce717290, 66 },
    { 0xdb417290, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1371729b, 40 },
    { 0x4b81729b, 95 },
    { 0x20c1729c, 55 },
    { 0xadd1729e, 55 },
    { 0xd8c1729f, 67 },
    { 0x3971729b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55172a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a172a6, 34 },
    { 0xfea172a6, 61 },
    { 0xe33172a8, 81 },
    { 0x253172a9, 81 },
    { 0x6ce172a6, 61 },
    { 0, 0 },
    { 0xee9172ac, 98 },
    { 0x287172ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15172b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x100172b3, 75 },
    { 0, 0 },
    { 0x306172b5, 75 },
    { 0xc1f172b6, 55 },
    { 0, 0 },
    { 0x55d172b8, 90 },
    { 0xc08172b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9172bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x507172bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a172c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0172c8, 96 },
    { 0xf1c172c9, 61 },
    { 0x613172c9, 23 },
    { 0xcdf172cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd172d3, 33 },
    { 0xb45172d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x715172d7, 23 },
    { 0, 0 },
    { 0xd92172d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573172e1, 23 },
    { 0, 0 },
    { 0xa28172e3, 46 },
    { 0x1c0172e4, 61 },
    { 0, 0 },
    { 0x860172e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46172ea, 34 },
    { 0, 0 },
    { 0x58b172ec, 90 },
    { 0x6af172ed, 93 },
    { 0xf2e172ec, 98 },
    { 0, 0 },
    { 0xc1d172f0, 46 },
    { 0, 0 },
    { 0x515172f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e172f8, 46 },
    { 0, 0 },
    { 0xe8b172fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0172fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e17300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb17303, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a17308, 33 },
    { 0, 0 },
    { 0x7921730a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa121730e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac17313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2117316, 91 },
    { 0, 0 },
    { 0xf7017318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae1731e, 33 },
    { 0xb331731f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4717327, 95 },
    { 0x73517328, 85 },
    { 0, 0 },
    { 0x7ce1732a, 90 },
    { 0x8f11732a, 98 },
    { 0x2591732c, 91 },
    { 0, 0 },
    { 0x6bf1732e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5017336, 56 },
    { 0xc7717337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8891733c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c417343, 69 },
    { 0xe5217343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c1734d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71817350, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d417353, 99 },
    { 0x17817354, 85 },
    { 0xe7317354, 90 },
    { 0xb4917355, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd41735a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631735f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e217362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf17366, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb1736a, 17 },
    { 0, 0 },
    { 0x5721736c, 84 },
    { 0xc4e1736d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b17372, 78 },
    { 0, 0 },
    { 0xc3917374, 93 },
    { 0xcec17374, 23 },
    { 0x3d817376, 98 },
    { 0xc417376, 23 },
    { 0xf3617378, 98 },
    { 0xd6017379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a17380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc17383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8451738a, 96 },
    { 0, 0 },
    { 0x16c1738c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8917392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1739c, 8 },
    { 0x1761739c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2173a1, 43 },
    { 0x74b173a2, 61 },
    { 0xc3a173a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb173a6, 98 },
    { 0, 0 },
    { 0xa5a173a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741173ac, 98 },
    { 0x290173ad, 66 },
    { 0x679173ae, 91 },
    { 0x7aa173af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf173b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e173b7, 23 },
    { 0x198173b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e173be, 90 },
    { 0x9cb173bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7173c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122173ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0173cf, 43 },
    { 0x8bf173d0, 46 },
    { 0x206173d0, 33 },
    { 0x63b173d2, 99 },
    { 0xe10173d2, 46 },
    { 0x2cc173d2, 76 },
    { 0x8dd173d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x865173d8, 33 },
    { 0x909173d9, 90 },
    { 0x6dd173da, 61 },
    { 0x54c173d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56173e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc173e4, 52 },
    { 0x98c173e5, 90 },
    { 0xa29173e6, 23 },
    { 0, 0 },
    { 0xb31173e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd173eb, 34 },
    { 0, 0 },
    { 0x51a173ed, 23 },
    { 0xf01173ee, 46 },
    { 0, 0 },
    { 0xfac173f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e173f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4173fb, 55 },
    { 0xec3173fc, 55 },
    { 0xd56173fd, 33 },
    { 0, 0 },
    { 0xe3f173ff, 76 },
    { 0xfc117400, 61 },
    { 0, 0 },
    { 0xf5317402, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d617405, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c517408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf91740f, 33 },
    { 0x36c1740f, 78 },
    { 0x5e417411, 90 },
    { 0, 0 },
    { 0x3d217413, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf517416, 95 },
    { 0, 0 },
    { 0x80017418, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a21741c, 76 },
    { 0, 0 },
    { 0x9dd1741e, 23 },
    { 0, 0 },
    { 0x56d17420, 52 },
    { 0x94217421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94217426, 52 },
    { 0x39b17426, 61 },
    { 0xdc317428, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8471742b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541742e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea417431, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33317436, 33 },
    { 0x17417436, 61 },
    { 0x29617438, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d1743c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89717440, 97 },
    { 0, 0 },
    { 0xc5217442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e17448, 95 },
    { 0x72117449, 95 },
    { 0xd0317449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9061744d, 61 },
    { 0xa9a1744d, 34 },
    { 0x351744e, 95 },
    { 0x1e317450, 98 },
    { 0x8c217451, 90 },
    { 0xc5d1744d, 95 },
    { 0xa1017452, 61 },
    { 0, 0 },
    { 0x61817455, 46 },
    { 0x41d17456, 90 },
    { 0x12c17457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6271745b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1745e, 90 },
    { 0xc4e1745f, 46 },
    { 0x3bb1745f, 95 },
    { 0xca617461, 61 },
    { 0xb1317462, 66 },
    { 0, 0 },
    { 0xda17464, 23 },
    { 0x8be17465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab917468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e1746c, 78 },
    { 0, 0 },
    { 0xfaf1746e, 90 },
    { 0xba81746f, 94 },
    { 0x3a61746f, 43 },
    { 0x15917470, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a317475, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c517478, 90 },
    { 0, 0 },
    { 0x1231747a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517485, 90 },
    { 0, 0 },
    { 0xdb17487, 23 },
    { 0, 0 },
    { 0x42917489, 33 },
    { 0x6fa17489, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6031748e, 52 },
    { 0x471748f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0517494, 23 },
    { 0, 0 },
    { 0x58217496, 66 },
    { 0xf9f17497, 90 },
    { 0, 0 },
    { 0xe8f17499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e1749c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x331749f, 91 },
    { 0, 0 },
    { 0x416174a1, 98 },
    { 0, 0 },
    { 0xe6d174a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c174a7, 90 },
    { 0x58a174a8, 81 },
    { 0xaac174a7, 23 },
    { 0x7a0174a7, 90 },
    { 0, 0 },
    { 0x97a174ac, 61 },
    { 0x8cb174ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477174b1, 78 },
    { 0, 0 },
    { 0x19e174b3, 33 },
    { 0xb1c174b3, 95 },
    { 0x25a174b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba174b8, 98 },
    { 0, 0 },
    { 0x3b1174ba, 95 },
    { 0x859174ba, 33 },
    { 0x9f8174bb, 83 },
    { 0x222174bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe174c1, 95 },
    { 0x90174c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420174cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78174d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b174d7, 95 },
    { 0, 0 },
    { 0x58e174d9, 88 },
    { 0x412174d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e174e1, 61 },
    { 0xaa2174e2, 90 },
    { 0xde5174e3, 85 },
    { 0x58d174e3, 33 },
    { 0xb87174e5, 33 },
    { 0x3eb174e5, 55 },
    { 0x4a3174e3, 61 },
    { 0x16d174e4, 43 },
    { 0, 0 },
    { 0xdba174ea, 95 },
    { 0, 0 },
    { 0x9c8174ec, 43 },
    { 0, 0 },
    { 0x94174ee, 68 },
    { 0, 0 },
    { 0x8e2174f0, 46 },
    { 0, 0 },
    { 0xee5174f2, 85 },
    { 0xfe7174f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e174f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9174f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd174fe, 76 },
    { 0x9fc174ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c17504, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa17507, 33 },
    { 0, 0 },
    { 0x96417509, 90 },
    { 0, 0 },
    { 0x33f1750b, 55 },
    { 0, 0 },
    { 0xa961750d, 90 },
    { 0x38c1750d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a17512, 76 },
    { 0x9db17513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c117519, 100 },
    { 0, 0 },
    { 0x8941751b, 98 },
    { 0xd651751c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc17527, 43 },
    { 0x4c717528, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41752b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53617536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3631753a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e1753f, 55 },
    { 0x4c917540, 90 },
    { 0x11017540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b817544, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b17548, 90 },
    { 0x8bb17549, 76 },
    { 0xb1a1754a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d1754d, 100 },
    { 0, 0 },
    { 0x1031754f, 34 },
    { 0, 0 },
    { 0x2c17551, 98 },
    { 0xee317552, 90 },
    { 0xa0a17551, 90 },
    { 0x49e17551, 95 },
    { 0x19017555, 33 },
    { 0xc8d17556, 61 },
    { 0xec117556, 95 },
    { 0x917551, 99 },
    { 0x8ae17559, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631755c, 61 },
    { 0, 0 },
    { 0xc5d1755e, 85 },
    { 0xd491755e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75317567, 76 },
    { 0, 0 },
    { 0x6f217569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd1756d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e1757a, 34 },
    { 0xea01757b, 98 },
    { 0x4241757a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f617584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89917588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd831758b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1758f, 33 },
    { 0x50017590, 84 },
    { 0, 0 },
    { 0x6f117592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46917595, 90 },
    { 0x82d17596, 91 },
    { 0xb0417597, 43 },
    { 0x37117598, 95 },
    { 0xed117596, 33 },
    { 0x1db17596, 95 },
    { 0x2551759b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1175a4, 67 },
    { 0x3d8175a5, 61 },
    { 0, 0 },
    { 0xd2b175a7, 84 },
    { 0, 0 },
    { 0xbdb175a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab175ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9175b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a175b7, 67 },
    { 0x3a175b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963175c4, 61 },
    { 0, 0 },
    { 0x6f8175c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x569175c9, 87 },
    { 0x11c175ca, 72 },
    { 0xa25175c9, 98 },
    { 0x8a7175cb, 90 },
    { 0x8ae175cd, 98 },
    { 0x4b7175ce, 67 },
    { 0xa20175cf, 95 },
    { 0xdb3175cf, 33 },
    { 0x975175cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f175d4, 66 },
    { 0xfe3175d5, 53 },
    { 0x7d1175d5, 55 },
    { 0x246175d4, 95 },
    { 0xa3175d6, 95 },
    { 0x3f4175d9, 61 },
    { 0xc63175d9, 81 },
    { 0xcc3175da, 43 },
    { 0x5f7175dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8175e1, 23 },
    { 0, 0 },
    { 0x977175e3, 61 },
    { 0x7a7175e4, 66 },
    { 0xf1e175e5, 95 },
    { 0, 0 },
    { 0xe6d175e7, 98 },
    { 0xa97175e7, 98 },
    { 0, 0 },
    { 0x340175ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9175ef, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763175f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4175fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d17601, 55 },
    { 0xfb517602, 95 },
    { 0xd9017603, 61 },
    { 0, 0 },
    { 0xaa517605, 100 },
    { 0xdaf17606, 68 },
    { 0x1e217607, 55 },
    { 0xa8317608, 65 },
    { 0xe1017605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3771760d, 95 },
    { 0, 0 },
    { 0x13a1760f, 43 },
    { 0x80c1760f, 34 },
    { 0, 0 },
    { 0x9b717612, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a317616, 93 },
    { 0xc1e17617, 100 },
    { 0, 0 },
    { 0x57517619, 88 },
    { 0, 0 },
    { 0x9141761b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b1761f, 78 },
    { 0, 0 },
    { 0xa5117621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea217626, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1762b, 61 },
    { 0xbf01762b, 33 },
    { 0, 0 },
    { 0x8671762e, 46 },
    { 0xd861762f, 90 },
    { 0x6811762f, 33 },
    { 0xa801762e, 61 },
    { 0xb6117630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e17636, 46 },
    { 0x5e517637, 23 },
    { 0x40517637, 98 },
    { 0xbeb17638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee01763d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14917640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba1764a, 91 },
    { 0xd7d1764b, 76 },
    { 0xdde1764c, 78 },
    { 0x8c31764b, 84 },
    { 0x1881764d, 43 },
    { 0x37e1764f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc717653, 91 },
    { 0xfa317654, 43 },
    { 0x88117655, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x73417658, 33 },
    { 0x8a117658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f17662, 23 },
    { 0, 0 },
    { 0x13317664, 64 },
    { 0, 0 },
    { 0x6d17666, 61 },
    { 0x67717667, 55 },
    { 0, 0 },
    { 0xc3317669, 99 },
    { 0xa9d1766a, 78 },
    { 0x851766b, 23 },
    { 0xf181766a, 98 },
    { 0xed31766d, 98 },
    { 0, 0 },
    { 0x5b81766f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x45517672, 78 },
    { 0x1db17672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d17676, 61 },
    { 0, 0 },
    { 0xfd517678, 95 },
    { 0xb7f17679, 91 },
    { 0, 0 },
    { 0x8b31767b, 99 },
    { 0x7571767c, 17 },
    { 0x7531767d, 90 },
    { 0xbc61767d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf17681, 78 },
    { 0xca417681, 61 },
    { 0, 0 },
    { 0x88817684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd317689, 34 },
    { 0x3431768a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7717698, 33 },
    { 0x63117699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca1769d, 91 },
    { 0, 0 },
    { 0xcf1769f, 55 },
    { 0x18d1769f, 34 },
    { 0x3c41769f, 93 },
    { 0x9ab176a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2176a6, 93 },
    { 0, 0 },
    { 0x417176a8, 98 },
    { 0, 0 },
    { 0x4ba176aa, 33 },
    { 0xf97176aa, 33 },
    { 0x9ab176ab, 33 },
    { 0, 0 },
    { 0xd65176ae, 33 },
    { 0x499176af, 90 },
    { 0x485176b0, 85 },
    { 0xde9176ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x321176b4, 61 },
    { 0, 0 },
    { 0xe27176b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06176b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f176bf, 33 },
    { 0, 0 },
    { 0x6de176c1, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170176cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0176d0, 33 },
    { 0x979176d0, 61 },
    { 0x9be176d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382176db, 34 },
    { 0, 0 },
    { 0xa16176dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51176e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e176e8, 66 },
    { 0, 0 },
    { 0x74d176ea, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb176f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3176fd, 95 },
    { 0, 0 },
    { 0xd60176ff, 95 },
    { 0xd2d17700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0417703, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc617707, 95 },
    { 0x5f717708, 98 },
    { 0x30b17709, 23 },
    { 0x38a17707, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27317710, 98 },
    { 0, 0 },
    { 0xbbb17712, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70317715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb17719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b01771c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a817722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2917725, 40 },
    { 0, 0 },
    { 0xedf17727, 38 },
    { 0xd2017728, 46 },
    { 0, 0 },
    { 0xee31772a, 76 },
    { 0x2251772a, 33 },
    { 0, 0 },
    { 0x2af1772d, 99 },
    { 0xada1772d, 23 },
    { 0, 0 },
    { 0x37f17730, 90 },
    { 0x75b17731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2311773c, 90 },
    { 0x7271773d, 34 },
    { 0xc971773e, 33 },
    { 0x1f51773f, 85 },
    { 0xe5c17740, 55 },
    { 0x5e21773d, 33 },
    { 0, 0 },
    { 0xae917743, 78 },
    { 0x6a717744, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe17747, 95 },
    { 0, 0 },
    { 0xfdb17749, 55 },
    { 0xe8817749, 46 },
    { 0x7517749, 34 },
    { 0xceb1774c, 98 },
    { 0x9081774c, 33 },
    { 0x9561774e, 76 },
    { 0xab417749, 33 },
    { 0xd7817750, 77 },
    { 0xa551774a, 76 },
    { 0xdb217752, 95 },
    { 0xdab17752, 96 },
    { 0x22b17754, 91 },
    { 0xd7217749, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c17759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1775c, 55 },
    { 0, 0 },
    { 0xdfa1775e, 23 },
    { 0x6851775e, 23 },
    { 0xd9f17760, 98 },
    { 0x16e17761, 87 },
    { 0, 0 },
    { 0x6f917763, 33 },
    { 0xf8117764, 43 },
    { 0, 0 },
    { 0xfee17766, 93 },
    { 0x60217767, 61 },
    { 0x7eb17767, 69 },
    { 0, 0 },
    { 0x6f21776a, 90 },
    { 0, 0 },
    { 0xa0d1776c, 81 },
    { 0, 0 },
    { 0x8491776e, 93 },
    { 0x7591776f, 61 },
    { 0x5f51776e, 95 },
    { 0xc2717771, 23 },
    { 0x4ef17772, 43 },
    { 0x9317771, 23 },
    { 0x87f17771, 76 },
    { 0, 0 },
    { 0x23f17776, 34 },
    { 0x30417776, 67 },
    { 0x4d317778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a21777b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22117785, 99 },
    { 0, 0 },
    { 0xd0717787, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b01778a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68017792, 87 },
    { 0x83917793, 98 },
    { 0x74417794, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3717797, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf177aa, 77 },
    { 0x30e177ab, 76 },
    { 0xea7177aa, 33 },
    { 0xaa6177ad, 34 },
    { 0, 0 },
    { 0x592177af, 95 },
    { 0, 0 },
    { 0x6aa177b1, 90 },
    { 0x678177b2, 55 },
    { 0x648177b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f177b6, 85 },
    { 0, 0 },
    { 0x322177b8, 23 },
    { 0x209177b9, 90 },
    { 0xc94177ba, 84 },
    { 0x794177bb, 61 },
    { 0, 0 },
    { 0x2a8177bd, 93 },
    { 0x358177be, 61 },
    { 0x374177bd, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157177c3, 23 },
    { 0x192177c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a177ca, 78 },
    { 0xe23177cb, 88 },
    { 0x94d177ca, 52 },
    { 0, 0 },
    { 0xac6177ce, 90 },
    { 0x8e6177cf, 76 },
    { 0x629177d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f177d3, 95 },
    { 0, 0 },
    { 0x89177d5, 34 },
    { 0, 0 },
    { 0x22f177d7, 23 },
    { 0x59e177d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4177dc, 55 },
    { 0x1b5177dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b177e3, 95 },
    { 0xe48177e4, 34 },
    { 0, 0 },
    { 0x6f1177e6, 34 },
    { 0xdce177e7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4177ec, 90 },
    { 0x41c177ed, 91 },
    { 0x570177ed, 66 },
    { 0xce0177ef, 90 },
    { 0xe50177ed, 61 },
    { 0, 0 },
    { 0x843177f2, 76 },
    { 0, 0 },
    { 0x1fb177f4, 52 },
    { 0, 0 },
    { 0xe71177f6, 61 },
    { 0, 0 },
    { 0xde3177f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a617800, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb17806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f71780b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b17811, 61 },
    { 0xc4317812, 76 },
    { 0xdbd17811, 66 },
    { 0x42017811, 61 },
    { 0xdfc17815, 87 },
    { 0x80617813, 33 },
    { 0x11017811, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41781a, 76 },
    { 0x6391781b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf17820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e17829, 61 },
    { 0xebf17829, 55 },
    { 0x7641782b, 29 },
    { 0xa1e1782c, 87 },
    { 0xa721782c, 95 },
    { 0xb941782e, 52 },
    { 0xbe91782e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c17838, 23 },
    { 0xc2417839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e17840, 85 },
    { 0xa7017840, 61 },
    { 0xbb317842, 76 },
    { 0x47317841, 33 },
    { 0x6c617843, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021784c, 23 },
    { 0, 0 },
    { 0x60c1784e, 55 },
    { 0, 0 },
    { 0x38c17850, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc617857, 61 },
    { 0x6c217858, 23 },
    { 0xf2c17859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae1785c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca217860, 98 },
    { 0, 0 },
    { 0xfeb17862, 61 },
    { 0, 0 },
    { 0x73317864, 61 },
    { 0, 0 },
    { 0x85c17866, 90 },
    { 0x10317866, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71786b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17017870, 98 },
    { 0x51117871, 23 },
    { 0x21317872, 17 },
    { 0, 0 },
    { 0x81117874, 23 },
    { 0x29417875, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95717879, 85 },
    { 0xa061787a, 88 },
    { 0xdc81787a, 87 },
    { 0, 0 },
    { 0xd821787d, 76 },
    { 0xca01787d, 98 },
    { 0x1691787e, 90 },
    { 0x6d517880, 99 },
    { 0x891787d, 55 },
    { 0x7081787d, 23 },
    { 0xb6417883, 98 },
    { 0x4d61787e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e017887, 95 },
    { 0xd8217888, 95 },
    { 0xa7717888, 33 },
    { 0xc41788a, 95 },
    { 0xef11788b, 33 },
    { 0xed1788b, 90 },
    { 0, 0 },
    { 0x3bb1788e, 91 },
    { 0, 0 },
    { 0x58017890, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a17896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91617899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb1789e, 90 },
    { 0, 0 },
    { 0x2ab178a0, 46 },
    { 0xc8e178a1, 84 },
    { 0, 0 },
    { 0x6e178a3, 33 },
    { 0x1dc178a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304178ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c178af, 34 },
    { 0xa86178af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77178b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d178b7, 33 },
    { 0xfe0178b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132178bc, 33 },
    { 0, 0 },
    { 0x818178be, 90 },
    { 0, 0 },
    { 0xd8e178c0, 78 },
    { 0x31a178c0, 98 },
    { 0x86f178c2, 98 },
    { 0x9fa178c2, 33 },
    { 0x959178c2, 99 },
    { 0, 0 },
    { 0x57178c6, 46 },
    { 0x8d1178c6, 61 },
    { 0xca6178c8, 95 },
    { 0x118178c9, 17 },
    { 0x873178ca, 90 },
    { 0x70178c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a178ce, 100 },
    { 0, 0 },
    { 0x8fa178d0, 76 },
    { 0, 0 },
    { 0x2db178d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe178d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66178da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1178dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66178e1, 90 },
    { 0x1ea178e1, 90 },
    { 0x103178e3, 91 },
    { 0, 0 },
    { 0x58a178e5, 61 },
    { 0x51d178e6, 81 },
    { 0x204178e6, 77 },
    { 0x607178e7, 56 },
    { 0x4ef178e9, 98 },
    { 0x2c178ea, 23 },
    { 0xa37178e7, 90 },
    { 0, 0 },
    { 0xd7b178ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82178f5, 95 },
    { 0x69a178f5, 90 },
    { 0xaea178f7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84178ff, 24 },
    { 0x5d617900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d117906, 24 },
    { 0x36917906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821790c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a17911, 88 },
    { 0, 0 },
    { 0xf17913, 78 },
    { 0x91017914, 33 },
    { 0x13217914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f31791b, 90 },
    { 0x9411791c, 90 },
    { 0x3211791d, 68 },
    { 0x6601791b, 90 },
    { 0xba91791d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c17924, 90 },
    { 0, 0 },
    { 0x7bd17926, 23 },
    { 0xf2417927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9421792e, 33 },
    { 0, 0 },
    { 0x9f717930, 23 },
    { 0xe6617931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd17934, 43 },
    { 0x65417934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c17938, 43 },
    { 0xf6b17939, 95 },
    { 0x7eb17939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb411793e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221794f, 34 },
    { 0, 0 },
    { 0xfa917951, 34 },
    { 0x77f17952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d17955, 91 },
    { 0, 0 },
    { 0xdc217957, 55 },
    { 0x7df17958, 33 },
    { 0, 0 },
    { 0x181795a, 65 },
    { 0xefd1795b, 90 },
    { 0, 0 },
    { 0x90a1795d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3317963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae81796e, 98 },
    { 0x2cf1796f, 90 },
    { 0x70f1796f, 90 },
    { 0, 0 },
    { 0x4e717972, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a517976, 55 },
    { 0, 0 },
    { 0x99917978, 90 },
    { 0, 0 },
    { 0x3271797a, 23 },
    { 0xa751797a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd281797f, 97 },
    { 0xe131797f, 98 },
    { 0x6af17981, 43 },
    { 0, 0 },
    { 0x9e717983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1517987, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c1798c, 33 },
    { 0xda21798d, 46 },
    { 0xcc61798d, 95 },
    { 0x6e01798f, 23 },
    { 0x57717990, 90 },
    { 0x6301798f, 33 },
    { 0xdb41798d, 67 },
    { 0x53217991, 61 },
    { 0x3fa17994, 24 },
    { 0xb7e1798c, 93 },
    { 0xffb17996, 33 },
    { 0xe8b17997, 95 },
    { 0xe2017998, 46 },
    { 0x67617999, 90 },
    { 0xf6417997, 90 },
    { 0, 0 },
    { 0x3b01799c, 75 },
    { 0xcb91799d, 76 },
    { 0, 0 },
    { 0xfe81799f, 90 },
    { 0, 0 },
    { 0xe38179a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190179aa, 23 },
    { 0, 0 },
    { 0x8c3179ac, 43 },
    { 0x41a179ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92179b2, 81 },
    { 0x79d179b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b179b8, 33 },
    { 0xf97179b9, 88 },
    { 0x939179ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a179bd, 61 },
    { 0, 0 },
    { 0x582179bf, 95 },
    { 0xa77179c0, 90 },
    { 0x7c2179c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e179c7, 61 },
    { 0x5c0179c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20179cd, 76 },
    { 0x3d6179cd, 90 },
    { 0, 0 },
    { 0x1b1179d0, 90 },
    { 0, 0 },
    { 0xae179d2, 91 },
    { 0, 0 },
    { 0x119179d4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2179d8, 61 },
    { 0, 0 },
    { 0x3cc179da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7179e1, 98 },
    { 0, 0 },
    { 0x310179e3, 61 },
    { 0x5ad179e3, 33 },
    { 0x4b2179e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e179eb, 61 },
    { 0x226179ec, 98 },
    { 0xad2179ed, 33 },
    { 0x87a179ee, 90 },
    { 0xeca179ec, 46 },
    { 0x559179ef, 90 },
    { 0x9a179f1, 95 },
    { 0, 0 },
    { 0x442179f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7179fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce179fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf17a02, 87 },
    { 0xd0617a03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c17a06, 34 },
    { 0x91917a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31917a0e, 61 },
    { 0x82817a0e, 87 },
    { 0x4317a10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40517a1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8417a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d17a27, 93 },
    { 0xfd717a28, 61 },
    { 0, 0 },
    { 0x3de17a2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb17a2d, 68 },
    { 0, 0 },
    { 0xca317a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42317a3e, 98 },
    { 0, 0 },
    { 0x2617a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d317a43, 84 },
    { 0, 0 },
    { 0x6dc17a45, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba17a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70917a4d, 46 },
    { 0, 0 },
    { 0x27117a4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8317a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d17a58, 95 },
    { 0x52317a58, 43 },
    { 0xc0417a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c17a5e, 83 },
    { 0x73d17a5f, 100 },
    { 0, 0 },
    { 0x4dc17a61, 33 },
    { 0x19917a62, 77 },
    { 0, 0 },
    { 0xed417a64, 46 },
    { 0x3ea17a65, 99 },
    { 0, 0 },
    { 0x82f17a67, 55 },
    { 0, 0 },
    { 0x9d717a69, 84 },
    { 0xb5017a6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c17a6d, 23 },
    { 0, 0 },
    { 0x73c17a6f, 76 },
    { 0x9e917a6f, 33 },
    { 0xd7f17a70, 95 },
    { 0, 0 },
    { 0x40317a73, 61 },
    { 0, 0 },
    { 0xb2617a75, 78 },
    { 0xe6717a76, 88 },
    { 0x4f717a76, 33 },
    { 0, 0 },
    { 0x1d117a79, 85 },
    { 0x87c17a7a, 33 },
    { 0x10117a7b, 55 },
    { 0xee717a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea17a80, 95 },
    { 0x1da17a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc17a84, 46 },
    { 0xa5017a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d17a8d, 23 },
    { 0x24517a8e, 67 },
    { 0xdc17a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf17a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f17a98, 90 },
    { 0x69c17a99, 33 },
    { 0xf4c17a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8517a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed117aa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717aa6, 85 },
    { 0, 0 },
    { 0x37817aa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73717aab, 88 },
    { 0x56517aac, 55 },
    { 0, 0 },
    { 0xbe117aae, 33 },
    { 0x1d117aaf, 93 },
    { 0xa2017aae, 43 },
    { 0x37417ab0, 66 },
    { 0x32917ab2, 61 },
    { 0, 0 },
    { 0x2e117ab4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb717aba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3717abd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd17ac3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff017ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b17ace, 33 },
    { 0x6fe17ace, 100 },
    { 0x1e117ad0, 33 },
    { 0x71f17ad1, 90 },
    { 0, 0 },
    { 0x7b517ad3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x30117ad6, 55 },
    { 0, 0 },
    { 0x9b17ad8, 34 },
    { 0xa717ad9, 90 },
    { 0xe7d17ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda17ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40817ae5, 34 },
    { 0x6a117ae5, 99 },
    { 0x6ec17ae6, 23 },
    { 0, 0 },
    { 0xcfb17ae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e517aec, 55 },
    { 0x40e17aec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87717af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf617af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa17afc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20917aff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f17b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f17b05, 98 },
    { 0, 0 },
    { 0x49a17b07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30017b0b, 95 },
    { 0x14217b0c, 46 },
    { 0xcfc17b0c, 77 },
    { 0x5b917b0d, 91 },
    { 0xafa17b0f, 76 },
    { 0xa017b0d, 76 },
    { 0x11617b0d, 23 },
    { 0xcc417b12, 93 },
    { 0xb1117b13, 90 },
    { 0x40217b12, 90 },
    { 0x80417b12, 82 },
    { 0, 0 },
    { 0x37017b17, 98 },
    { 0x14f17b17, 76 },
    { 0x23317b19, 17 },
    { 0x3317b1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f17b1e, 100 },
    { 0, 0 },
    { 0x71117b20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55217b25, 90 },
    { 0x62017b26, 93 },
    { 0xea117b26, 61 },
    { 0, 0 },
    { 0xd4417b29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf717b2c, 95 },
    { 0xa017b2d, 34 },
    { 0, 0 },
    { 0xf9617b2f, 95 },
    { 0xaa17b30, 33 },
    { 0x59b17b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80917b39, 90 },
    { 0x4fa17b3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b817b42, 33 },
    { 0, 0 },
    { 0xe3817b44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8917b48, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92417b51, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd17b5c, 34 },
    { 0x5f17b5d, 33 },
    { 0xb5417b5e, 34 },
    { 0xbb917b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94017b6a, 90 },
    { 0xb5117b6a, 75 },
    { 0x2ad17b6c, 61 },
    { 0x38017b6a, 69 },
    { 0, 0 },
    { 0x82017b6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70117b72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d217b75, 85 },
    { 0x7da17b76, 93 },
    { 0x51917b77, 88 },
    { 0xd6917b75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17b83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e17b87, 33 },
    { 0x24d17b88, 65 },
    { 0, 0 },
    { 0xfc417b8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d417b8f, 90 },
    { 0x8d717b8f, 90 },
    { 0xb9417b8f, 23 },
    { 0x97417b8f, 56 },
    { 0x40717b8f, 23 },
    { 0, 0 },
    { 0x20217b95, 69 },
    { 0, 0 },
    { 0x27117b97, 76 },
    { 0xfd017b98, 61 },
    { 0, 0 },
    { 0x1ef17b9a, 85 },
    { 0x1cc17b9a, 62 },
    { 0x22f17b9a, 90 },
    { 0, 0 },
    { 0x5b817b9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f317ba1, 100 },
    { 0x87d17ba1, 76 },
    { 0, 0 },
    { 0x47b17ba4, 76 },
    { 0, 0 },
    { 0x24317ba6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded17baa, 95 },
    { 0x3ab17bab, 93 },
    { 0xdca17bac, 81 },
    { 0xfda17bab, 69 },
    { 0x24c17bad, 34 },
    { 0xf6617baa, 61 },
    { 0, 0 },
    { 0x51117bb1, 77 },
    { 0x98017bb1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7917bb6, 43 },
    { 0x9bf17bb7, 33 },
    { 0, 0 },
    { 0x39117bb9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19317bc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c17bc3, 61 },
    { 0, 0 },
    { 0xfc517bc5, 52 },
    { 0x8c17bc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47417bc9, 90 },
    { 0x59917bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee17bcd, 95 },
    { 0x4a817bcd, 76 },
    { 0x2bc17bcf, 90 },
    { 0x4b917bd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc17bdb, 90 },
    { 0x19417bdb, 98 },
    { 0xf5d17bdc, 33 },
    { 0, 0 },
    { 0x8ff17bdf, 34 },
    { 0xb1517be0, 67 },
    { 0x75a17be0, 40 },
    { 0xcb117be2, 34 },
    { 0xe817be3, 55 },
    { 0, 0 },
    { 0xdff17be5, 33 },
    { 0, 0 },
    { 0xae917be7, 90 },
    { 0xb5617be8, 23 },
    { 0x2a017be9, 33 },
    { 0x8cf17bea, 43 },
    { 0x4b917beb, 52 },
    { 0xea17bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b17bf1, 95 },
    { 0x1d017bf2, 33 },
    { 0, 0 },
    { 0x27917bf4, 66 },
    { 0x22317bf5, 34 },
    { 0xf1e17bf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64417bfc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82617c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a717c0e, 90 },
    { 0, 0 },
    { 0x1f317c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51417c13, 83 },
    { 0, 0 },
    { 0xc5c17c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e117c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17c1c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab17c21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x41217c24, 61 },
    { 0x48117c25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61117c2b, 61 },
    { 0x3e917c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45217c31, 95 },
    { 0xb2717c32, 76 },
    { 0x4d317c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea17c37, 43 },
    { 0xf9817c37, 33 },
    { 0x77717c38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f17c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df17c48, 55 },
    { 0, 0 },
    { 0x82317c4a, 61 },
    { 0, 0 },
    { 0x3e217c4c, 55 },
    { 0, 0 },
    { 0x12517c4e, 64 },
    { 0xdeb17c4e, 69 },
    { 0xc9917c4f, 100 },
    { 0xa417c51, 76 },
    { 0x88317c51, 100 },
    { 0x8f217c4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e17c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea17c5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7117c60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51317c64, 17 },
    { 0x80c17c65, 90 },
    { 0x7ba17c66, 61 },
    { 0, 0 },
    { 0x21317c68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c17c6c, 88 },
    { 0x1cf17c6c, 93 },
    { 0xc0317c6e, 34 },
    { 0, 0 },
    { 0xbf017c70, 34 },
    { 0x19717c71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b617c79, 34 },
    { 0, 0 },
    { 0x8d317c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9217c7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2617c88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b17c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77217c93, 43 },
    { 0xbcf17c94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd17c99, 33 },
    { 0, 0 },
    { 0x1ed17c9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc17ca2, 34 },
    { 0xe1a17ca2, 76 },
    { 0x68117ca4, 90 },
    { 0x89417ca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a217ca8, 90 },
    { 0xf9017ca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c717cac, 23 },
    { 0x12b17cad, 98 },
    { 0xbe817cae, 76 },
    { 0, 0 },
    { 0xbb817cb0, 76 },
    { 0xeee17cb1, 61 },
    { 0xd2317cb0, 33 },
    { 0xdcf17cb3, 90 },
    { 0, 0 },
    { 0x58917cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb17cbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af17cbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8317cc3, 61 },
    { 0, 0 },
    { 0x85c17cc5, 90 },
    { 0xed517cc5, 95 },
    { 0, 0 },
    { 0x6d517cc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63217ccc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1017ccf, 66 },
    { 0x71617cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d417cd6, 52 },
    { 0x5e117cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1417cda, 98 },
    { 0, 0 },
    { 0x7e917cdc, 61 },
    { 0x99417cdc, 76 },
    { 0, 0 },
    { 0xde317cdf, 23 },
    { 0xeda17ce0, 77 },
    { 0xd4817ce1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f917ce5, 33 },
    { 0, 0 },
    { 0xcc17ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b17ceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde017cf1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d17cf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa17cfa, 33 },
    { 0, 0 },
    { 0xfc917cfc, 76 },
    { 0x48b17cfd, 95 },
    { 0x27717cfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24417d03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c117d0e, 80 },
    { 0, 0 },
    { 0x7d817d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df17d15, 55 },
    { 0, 0 },
    { 0xf5117d17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b17d1c, 76 },
    { 0x87d17d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34417d2d, 90 },
    { 0x56017d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de17d33, 43 },
    { 0xf4717d34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0217d3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c17d3f, 33 },
    { 0, 0 },
    { 0x5d317d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b17d48, 55 },
    { 0x41d17d48, 23 },
    { 0x2f617d48, 55 },
    { 0, 0 },
    { 0xb1817d4c, 67 },
    { 0x10817d4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37817d52, 90 },
    { 0xec117d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c17d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f17d60, 23 },
    { 0x8a817d60, 90 },
    { 0x44b17d60, 90 },
    { 0x5a917d60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93617d66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b17d69, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e17d6f, 76 },
    { 0, 0 },
    { 0xe5d17d71, 33 },
    { 0, 0 },
    { 0x77e17d73, 91 },
    { 0x7b717d74, 61 },
    { 0x9d717d74, 33 },
    { 0xc9917d76, 90 },
    { 0xc0c17d77, 76 },
    { 0x31917d77, 55 },
    { 0, 0 },
    { 0xbf17d7a, 23 },
    { 0x47717d7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0917d7f, 98 },
    { 0, 0 },
    { 0x3e917d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f217d88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb217d8b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0817d90, 55 },
    { 0x5e617d91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde517d96, 76 },
    { 0x9b817d97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e17d9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5617d9e, 55 },
    { 0, 0 },
    { 0x52617da0, 33 },
    { 0xe0e17da0, 33 },
    { 0, 0 },
    { 0x57617da3, 98 },
    { 0, 0 },
    { 0x7fd17da5, 34 },
    { 0x3ce17da6, 46 },
    { 0x33717da7, 95 },
    { 0x7f317da8, 88 },
    { 0xc0f17da6, 97 },
    { 0x9b317daa, 43 },
    { 0xca317dab, 43 },
    { 0, 0 },
    { 0xda17dad, 87 },
    { 0x6a17dae, 100 },
    { 0xffc17daf, 98 },
    { 0x51b17dae, 23 },
    { 0x9da17db0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc17db4, 98 },
    { 0, 0 },
    { 0x3bf17db6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45117db9, 95 },
    { 0xd2d17dba, 98 },
    { 0x74f17dbb, 76 },
    { 0x9d817dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb17dbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c17dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8617dca, 55 },
    { 0xffd17dcb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d17dd2, 33 },
    { 0x2e617dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b917dda, 90 },
    { 0x67717ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe17de1, 55 },
    { 0x4617de1, 90 },
    { 0, 0 },
    { 0x65517de4, 33 },
    { 0, 0 },
    { 0x8e217de6, 98 },
    { 0x71c17de7, 23 },
    { 0x4ca17de7, 55 },
    { 0x87617de8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27417dec, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c17df3, 61 },
    { 0x91f17df3, 98 },
    { 0x38c17df4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x43017df8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83217dfb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba17dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7817e04, 90 },
    { 0, 0 },
    { 0xdb617e06, 90 },
    { 0x98017e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717e0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5017e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95217e16, 40 },
    { 0, 0 },
    { 0x36417e18, 93 },
    { 0xd717e19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e317e1c, 85 },
    { 0x74717e1d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae117e22, 76 },
    { 0x37817e23, 91 },
    { 0, 0 },
    { 0x8dc17e25, 94 },
    { 0x77d17e26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b17e30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99417e33, 88 },
    { 0, 0 },
    { 0x9d517e35, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2617e3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c117e45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf317e48, 91 },
    { 0xd6d17e48, 61 },
    { 0x7bf17e48, 33 },
    { 0, 0 },
    { 0xdfb17e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca17e51, 98 },
    { 0x94217e52, 95 },
    { 0xdf317e52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa17e58, 90 },
    { 0, 0 },
    { 0x63117e5a, 90 },
    { 0xadb17e5a, 23 },
    { 0xe9017e5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85117e61, 52 },
    { 0xaea17e62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf17e65, 61 },
    { 0x58a17e66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b717e6d, 61 },
    { 0xd8c17e6e, 23 },
    { 0x7e117e6f, 61 },
    { 0x38c17e70, 43 },
    { 0, 0 },
    { 0x64a17e72, 81 },
    { 0xf817e72, 43 },
    { 0xd5517e74, 90 },
    { 0, 0 },
    { 0x55517e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c517e79, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b17e7c, 98 },
    { 0, 0 },
    { 0x8a117e7e, 93 },
    { 0, 0 },
    { 0xc4317e80, 52 },
    { 0, 0 },
    { 0xa3017e82, 76 },
    { 0x8ae17e83, 55 },
    { 0x1e717e84, 85 },
    { 0xb2817e85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf317e8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89417e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e95, 23 },
    { 0x33517e95, 87 },
    { 0x2d717e97, 23 },
    { 0xeb517e97, 98 },
    { 0xdd617e99, 34 },
    { 0xadd17e97, 93 },
    { 0, 0 },
    { 0x2e317e9c, 90 },
    { 0x24b17e9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11717ea1, 55 },
    { 0xc6d17ea1, 36 },
    { 0, 0 },
    { 0xa7d17ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a17ea7, 84 },
    { 0x18817ea8, 90 },
    { 0xaa017ea9, 61 },
    { 0x1fa17ea9, 55 },
    { 0, 0 },
    { 0x78b17eac, 23 },
    { 0, 0 },
    { 0xaef17eae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc17eb2, 23 },
    { 0, 0 },
    { 0x89517eb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e817eb7, 40 },
    { 0x6417eb8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e117ec0, 100 },
    { 0, 0 },
    { 0xc6717ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe17ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e17ed3, 90 },
    { 0x4bd17ed3, 98 },
    { 0xaf917ed5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c17eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e617edd, 98 },
    { 0x52317ede, 23 },
    { 0x9bc17ede, 61 },
    { 0x3017ee0, 61 },
    { 0xd9c17ee1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae617ee5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xda717ee8, 78 },
    { 0, 0 },
    { 0xba717eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec17ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e17ef4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a17ef8, 61 },
    { 0xd1717ef9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae17efd, 100 },
    { 0xb7417efe, 90 },
    { 0, 0 },
    { 0xff117f00, 98 },
    { 0x3b717f01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85117f08, 77 },
    { 0xc5017f09, 76 },
    { 0, 0 },
    { 0xaed17f0b, 100 },
    { 0, 0 },
    { 0x94c17f0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb617f10, 23 },
    { 0, 0 },
    { 0x6d117f12, 95 },
    { 0xb3817f12, 34 },
    { 0x71317f12, 61 },
    { 0xfaa17f15, 85 },
    { 0x9fd17f16, 23 },
    { 0x68617f14, 99 },
    { 0, 0 },
    { 0xfc417f19, 95 },
    { 0x22317f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54517f20, 67 },
    { 0x81217f20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c817f29, 23 },
    { 0x87917f2a, 76 },
    { 0x93217f2b, 90 },
    { 0xf4617f2a, 52 },
    { 0x3f817f2d, 77 },
    { 0xb117f2d, 84 },
    { 0xc5317f2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c017f35, 95 },
    { 0, 0 },
    { 0xf7817f37, 78 },
    { 0x58217f38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd717f3b, 100 },
    { 0xd1a17f3b, 61 },
    { 0x14c17f3d, 34 },
    { 0, 0 },
    { 0xbb117f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41117f4c, 33 },
    { 0, 0 },
    { 0x1617f4e, 98 },
    { 0x9fd17f4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d17f54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e17f57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12217f5e, 90 },
    { 0x5cc17f5f, 98 },
    { 0, 0 },
    { 0x96817f61, 66 },
    { 0, 0 },
    { 0x21b17f63, 23 },
    { 0, 0 },
    { 0x6cd17f65, 56 },
    { 0, 0 },
    { 0x93d17f67, 55 },
    { 0xb217f67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70517f6c, 46 },
    { 0x1d217f6c, 55 },
    { 0x56c17f6c, 40 },
    { 0xc9217f6d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a017f73, 61 },
    { 0xeef17f74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62017f7a, 81 },
    { 0x73617f7b, 85 },
    { 0x5cd17f7b, 61 },
    { 0xc9717f7d, 61 },
    { 0x5b517f7d, 61 },
    { 0, 0 },
    { 0x9017f80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9817f83, 98 },
    { 0, 0 },
    { 0x65f17f85, 34 },
    { 0xeaa17f86, 91 },
    { 0x71f17f87, 95 },
    { 0x44617f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd17f8b, 90 },
    { 0x71d17f8b, 33 },
    { 0xa5317f8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa17f91, 84 },
    { 0x51817f92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd17f95, 33 },
    { 0x2d917f95, 88 },
    { 0x40c17f97, 23 },
    { 0xd2317f97, 76 },
    { 0x1b17f98, 55 },
    { 0xcde17f9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e17fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66017fa8, 33 },
    { 0, 0 },
    { 0x2d417faa, 33 },
    { 0x7d617fab, 95 },
    { 0xd5d17fac, 67 },
    { 0xf7617fad, 61 },
    { 0x5e817faa, 90 },
    { 0, 0 },
    { 0xf9717fb0, 80 },
    { 0, 0 },
    { 0x65517fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f517fb7, 61 },
    { 0x92317fb8, 98 },
    { 0xe2317fb8, 90 },
    { 0, 0 },
    { 0x31117fbb, 55 },
    { 0x80217fbc, 76 },
    { 0, 0 },
    { 0x71f17fbe, 33 },
    { 0xa3f17fbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5817fc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8617fc6, 98 },
    { 0, 0 },
    { 0x53717fc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a717fcb, 55 },
    { 0, 0 },
    { 0xbd17fcd, 46 },
    { 0xf6c17fce, 55 },
    { 0, 0 },
    { 0x58817fd0, 90 },
    { 0x82a17fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe417fd6, 46 },
    { 0xe9617fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc17fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57917fde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19717fe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee917fe7, 66 },
    { 0, 0 },
    { 0xed317fe9, 34 },
    { 0x57217fea, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d17ff4, 98 },
    { 0x34d17ff5, 90 },
    { 0x57617ff6, 33 },
    { 0x6ee17ff7, 77 },
    { 0x99017ff8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb417ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c18000, 55 },
    { 0x2a118001, 61 },
    { 0x7ca18002, 96 },
    { 0x48818003, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6518006, 46 },
    { 0, 0 },
    { 0x6e218008, 88 },
    { 0, 0 },
    { 0xb281800a, 78 },
    { 0, 0 },
    { 0xe61800c, 33 },
    { 0x3991800d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45318012, 90 },
    { 0x2a218013, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb18017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc1801d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f518021, 33 },
    { 0, 0 },
    { 0xf5518023, 93 },
    { 0, 0 },
    { 0x7bf18025, 77 },
    { 0x1b018025, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d21802b, 33 },
    { 0x9a31802c, 48 },
    { 0, 0 },
    { 0x4f61802e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e918034, 90 },
    { 0x85d18035, 33 },
    { 0x62c18034, 90 },
    { 0, 0 },
    { 0xf7018038, 99 },
    { 0x3dc18039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8951803d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb018042, 90 },
    { 0x35418043, 33 },
    { 0, 0 },
    { 0x44318045, 61 },
    { 0, 0 },
    { 0xcbb18047, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e41804d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5018056, 95 },
    { 0, 0 },
    { 0x3c018058, 78 },
    { 0x6618058, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8018060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74518066, 46 },
    { 0, 0 },
    { 0x19818068, 23 },
    { 0x6618069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc711806d, 78 },
    { 0x6451806e, 67 },
    { 0, 0 },
    { 0xcf118070, 34 },
    { 0, 0 },
    { 0x11718072, 90 },
    { 0xa0f18072, 38 },
    { 0x3a218073, 93 },
    { 0, 0 },
    { 0xccd18076, 66 },
    { 0xf9018076, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc161807a, 88 },
    { 0xbff1807a, 33 },
    { 0x9a81807b, 61 },
    { 0x8161807d, 99 },
    { 0xcd21807b, 95 },
    { 0, 0 },
    { 0xadb18080, 33 },
    { 0x36118080, 90 },
    { 0xdf918082, 95 },
    { 0x99518082, 85 },
    { 0x59518084, 76 },
    { 0x12c18082, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a1808d, 93 },
    { 0, 0 },
    { 0xeb61808f, 33 },
    { 0x3f018090, 33 },
    { 0xe418091, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a18094, 93 },
    { 0, 0 },
    { 0x65b18096, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab1809a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a1809d, 33 },
    { 0xad31809e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e180a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd180ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527180b6, 87 },
    { 0xbab180b7, 67 },
    { 0x313180b8, 76 },
    { 0, 0 },
    { 0x912180ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b180bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d180c2, 95 },
    { 0xc51180c3, 87 },
    { 0xb71180c4, 23 },
    { 0, 0 },
    { 0xb8a180c6, 95 },
    { 0x1ec180c6, 17 },
    { 0, 0 },
    { 0xcef180c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x671180cc, 90 },
    { 0, 0 },
    { 0x552180ce, 33 },
    { 0x98f180cf, 90 },
    { 0, 0 },
    { 0x88d180d1, 43 },
    { 0, 0 },
    { 0xacf180d3, 90 },
    { 0, 0 },
    { 0xd49180d5, 98 },
    { 0x3f3180d5, 33 },
    { 0x6ac180d7, 17 },
    { 0xcb5180d6, 23 },
    { 0x6eb180d6, 61 },
    { 0xf3f180d5, 90 },
    { 0x6a180db, 23 },
    { 0xc77180dc, 95 },
    { 0, 0 },
    { 0x691180de, 61 },
    { 0x2c1180de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826180e6, 69 },
    { 0, 0 },
    { 0xc4f180e8, 95 },
    { 0x52e180e9, 43 },
    { 0xf7c180ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3180ee, 87 },
    { 0x57180ef, 98 },
    { 0, 0 },
    { 0x449180f1, 23 },
    { 0xbb5180f1, 23 },
    { 0, 0 },
    { 0x32e180f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3180f8, 61 },
    { 0x569180f9, 93 },
    { 0x5d9180f9, 90 },
    { 0x351180fb, 37 },
    { 0xb22180fc, 61 },
    { 0xaa2180fb, 90 },
    { 0, 0 },
    { 0xf78180ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e18107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb1810a, 76 },
    { 0x96b1810b, 90 },
    { 0x9ba1810c, 76 },
    { 0x1321810c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d318111, 55 },
    { 0, 0 },
    { 0x44318113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af18116, 95 },
    { 0, 0 },
    { 0x1e318118, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501811e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32818122, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x32518125, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab1812a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4001812e, 61 },
    { 0x9921812f, 98 },
    { 0x9f01812f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a618134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3818137, 43 },
    { 0x65818138, 23 },
    { 0x33618139, 61 },
    { 0xb9d18139, 95 },
    { 0xe0918139, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b518143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b18147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61814d, 65 },
    { 0x4531814e, 66 },
    { 0xef51814f, 93 },
    { 0xbbe18150, 98 },
    { 0xd751814f, 69 },
    { 0x60b1814e, 87 },
    { 0x721814f, 100 },
    { 0xf1f18154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1815b, 34 },
    { 0, 0 },
    { 0x4e81815d, 80 },
    { 0, 0 },
    { 0xab01815f, 33 },
    { 0xda018160, 34 },
    { 0x2fa18161, 91 },
    { 0, 0 },
    { 0x9ee18163, 99 },
    { 0xe418164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb18167, 33 },
    { 0, 0 },
    { 0x4ac18169, 23 },
    { 0, 0 },
    { 0xe4a1816b, 90 },
    { 0, 0 },
    { 0x5801816d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c918171, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f18175, 95 },
    { 0x56218176, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5718179, 55 },
    { 0, 0 },
    { 0x30e1817b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa801817e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac18181, 59 },
    { 0, 0 },
    { 0x29e18183, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9318187, 61 },
    { 0, 0 },
    { 0x51e18189, 98 },
    { 0x20b1818a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x7481818d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd18190, 84 },
    { 0, 0 },
    { 0xf2d18192, 90 },
    { 0xc8d18193, 55 },
    { 0x51318192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d18197, 81 },
    { 0xa5d18197, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1819c, 43 },
    { 0x9431819d, 78 },
    { 0x9e41819d, 87 },
    { 0x41c1819f, 56 },
    { 0, 0 },
    { 0x626181a1, 23 },
    { 0x276181a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648181a6, 100 },
    { 0x549181a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a181aa, 23 },
    { 0, 0 },
    { 0xd68181ac, 90 },
    { 0x822181ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb181b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76181b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe181b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61181be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e181c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f181ca, 95 },
    { 0, 0 },
    { 0xce8181cc, 61 },
    { 0, 0 },
    { 0xb35181ce, 90 },
    { 0, 0 },
    { 0x230181d0, 95 },
    { 0xd02181d1, 67 },
    { 0, 0 },
    { 0xc87181d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b181d6, 23 },
    { 0xd9a181d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18181db, 93 },
    { 0, 0 },
    { 0xaad181dd, 90 },
    { 0, 0 },
    { 0xcf1181df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x273181e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2181e5, 33 },
    { 0, 0 },
    { 0x7d5181e7, 34 },
    { 0x917181e8, 61 },
    { 0, 0 },
    { 0x668181ea, 33 },
    { 0xf41181eb, 65 },
    { 0xce4181ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687181f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4181f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f8, 55 },
    { 0xb72181f8, 67 },
    { 0x935181fa, 93 },
    { 0, 0 },
    { 0x2a9181fc, 98 },
    { 0x694181fc, 95 },
    { 0x1e2181fe, 61 },
    { 0xdf6181ff, 97 },
    { 0xb70181fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e18204, 100 },
    { 0, 0 },
    { 0xb7518206, 33 },
    { 0x8db18207, 33 },
    { 0x6118208, 66 },
    { 0x72618209, 90 },
    { 0, 0 },
    { 0xffb1820b, 90 },
    { 0xa931820b, 91 },
    { 0xfb21820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2118215, 17 },
    { 0x6a118216, 90 },
    { 0x22618216, 33 },
    { 0xf3118216, 77 },
    { 0x23a18219, 8 },
    { 0x13018215, 23 },
    { 0x7b518219, 98 },
    { 0xd9e1821c, 40 },
    { 0x19918218, 98 },
    { 0, 0 },
    { 0x60f1821f, 43 },
    { 0x1e218220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91918223, 68 },
    { 0, 0 },
    { 0x63018225, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c118228, 99 },
    { 0x518228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88618231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d318234, 90 },
    { 0x7a318235, 88 },
    { 0xb2718235, 43 },
    { 0, 0 },
    { 0x89518238, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d1823e, 90 },
    { 0, 0 },
    { 0x4e18240, 59 },
    { 0, 0 },
    { 0x88b18242, 90 },
    { 0, 0 },
    { 0x93618244, 77 },
    { 0, 0 },
    { 0x1c818246, 90 },
    { 0x50b18247, 38 },
    { 0x4618248, 11 },
    { 0x21a18248, 34 },
    { 0x28218246, 61 },
    { 0x6521824b, 95 },
    { 0x92f1824b, 61 },
    { 0x9311824b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df18250, 69 },
    { 0xe918251, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f18254, 90 },
    { 0, 0 },
    { 0x80518256, 78 },
    { 0, 0 },
    { 0xf0518258, 98 },
    { 0xdac18258, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd1825e, 95 },
    { 0x2401825f, 88 },
    { 0x501825f, 23 },
    { 0xcd01825e, 84 },
    { 0xe3c18262, 68 },
    { 0x35f18260, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e18268, 95 },
    { 0, 0 },
    { 0x2eb1826a, 99 },
    { 0, 0 },
    { 0x1981826c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab18270, 43 },
    { 0x29618271, 90 },
    { 0x61918271, 55 },
    { 0x18c18273, 43 },
    { 0, 0 },
    { 0x9bb18275, 95 },
    { 0x8fe18276, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85918279, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a1827d, 33 },
    { 0x74a1827d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b18284, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x75518287, 76 },
    { 0x75f18288, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe231828f, 81 },
    { 0x95d1828f, 99 },
    { 0, 0 },
    { 0xece18292, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc618295, 23 },
    { 0x69c18296, 33 },
    { 0x95a18296, 23 },
    { 0xd2118298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba01829c, 98 },
    { 0xacb1829d, 23 },
    { 0x5b31829c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3182aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca182b1, 66 },
    { 0x2bc182b2, 61 },
    { 0x260182b2, 61 },
    { 0x23b182b2, 61 },
    { 0x8e4182b5, 90 },
    { 0x7c5182b2, 52 },
    { 0x828182b7, 33 },
    { 0x90b182b3, 55 },
    { 0x18182b2, 95 },
    { 0x3fb182b1, 40 },
    { 0xd1f182b5, 40 },
    { 0xdf7182bc, 74 },
    { 0xb99182b8, 90 },
    { 0x869182be, 33 },
    { 0xfee182b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f182c7, 95 },
    { 0x4da182c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5182d0, 98 },
    { 0x105182d1, 93 },
    { 0x7b2182d2, 76 },
    { 0xaf8182d0, 61 },
    { 0xb3a182d4, 33 },
    { 0xa36182d4, 91 },
    { 0xc1182d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d182dc, 76 },
    { 0x6a182dd, 61 },
    { 0x79b182dc, 38 },
    { 0, 0 },
    { 0x53f182e0, 95 },
    { 0, 0 },
    { 0xdf7182e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x438182e5, 55 },
    { 0x3b1182e6, 68 },
    { 0x292182e6, 66 },
    { 0xa07182e8, 93 },
    { 0x2c7182e9, 33 },
    { 0x90e182e9, 66 },
    { 0x6f4182eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x394182ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285182f2, 66 },
    { 0xb63182f2, 33 },
    { 0, 0 },
    { 0x774182f5, 91 },
    { 0xc81182f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x569182f9, 61 },
    { 0xb02182fa, 74 },
    { 0x7b0182fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d18303, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e18307, 95 },
    { 0x7af18308, 95 },
    { 0x67718308, 81 },
    { 0xa7b1830a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d31830f, 87 },
    { 0x3c718310, 52 },
    { 0, 0 },
    { 0x64918312, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30118315, 78 },
    { 0x73418315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71831b, 61 },
    { 0, 0 },
    { 0xa6f1831d, 33 },
    { 0x5db1831e, 23 },
    { 0xc6a1831d, 34 },
    { 0x3bb18320, 33 },
    { 0x63d1831d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e318325, 34 },
    { 0xe8218326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e318329, 90 },
    { 0, 0 },
    { 0x3671832b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b18335, 55 },
    { 0, 0 },
    { 0x19b18337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51833b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d81833e, 90 },
    { 0xb4f1833f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c18344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5618349, 98 },
    { 0x92518349, 84 },
    { 0xc9e1834b, 33 },
    { 0xf3e1834c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb18353, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b18356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9611835e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69718365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b18377, 98 },
    { 0xac118378, 95 },
    { 0x7a018377, 75 },
    { 0xab61837a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371837e, 36 },
    { 0x3651837f, 95 },
    { 0xdd41837f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5418385, 87 },
    { 0xb918386, 87 },
    { 0xa5a18386, 65 },
    { 0xd9b18387, 98 },
    { 0x1f018385, 99 },
    { 0, 0 },
    { 0xd4f1838b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd321838e, 33 },
    { 0x47f1838f, 33 },
    { 0x13618390, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52218395, 77 },
    { 0xcae18396, 95 },
    { 0xb4a18397, 61 },
    { 0, 0 },
    { 0x91718399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251839c, 43 },
    { 0, 0 },
    { 0x76d1839e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959183a5, 98 },
    { 0xdfa183a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5183a9, 98 },
    { 0x6c8183aa, 76 },
    { 0, 0 },
    { 0xc38183ac, 55 },
    { 0x88b183ad, 90 },
    { 0xe91183ae, 90 },
    { 0xb65183af, 98 },
    { 0, 0 },
    { 0xad2183b1, 99 },
    { 0, 0 },
    { 0x351183b3, 91 },
    { 0x918183b3, 34 },
    { 0x3a5183b5, 23 },
    { 0x32183b6, 98 },
    { 0, 0 },
    { 0xe4e183b8, 92 },
    { 0xe2183b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62183bc, 95 },
    { 0xa07183bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601183c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xac183c4, 61 },
    { 0xc98183c4, 96 },
    { 0x775183c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02183c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e183ce, 23 },
    { 0x8c1183cf, 83 },
    { 0x7d7183d0, 95 },
    { 0x824183d1, 35 },
    { 0xd61183d1, 98 },
    { 0x42e183d1, 34 },
    { 0x4da183d4, 93 },
    { 0xec4183d5, 95 },
    { 0x87d183d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29183da, 90 },
    { 0, 0 },
    { 0x69c183dc, 23 },
    { 0, 0 },
    { 0x5ef183de, 61 },
    { 0x7e6183df, 99 },
    { 0x928183e0, 77 },
    { 0x23c183e0, 43 },
    { 0x8a0183e2, 90 },
    { 0xea183e1, 67 },
    { 0x30f183e4, 90 },
    { 0xfa5183e4, 92 },
    { 0, 0 },
    { 0x73183e7, 93 },
    { 0x2b7183e8, 77 },
    { 0x860183e7, 98 },
    { 0, 0 },
    { 0xa5d183eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7183f0, 80 },
    { 0x546183f0, 100 },
    { 0xc6f183f0, 98 },
    { 0, 0 },
    { 0x32e183f4, 95 },
    { 0x937183f5, 87 },
    { 0x845183f5, 90 },
    { 0x916183f7, 33 },
    { 0x1b4183f5, 98 },
    { 0xf74183f9, 90 },
    { 0xe32183fa, 98 },
    { 0x893183fb, 23 },
    { 0x48d183fb, 90 },
    { 0xd63183fd, 76 },
    { 0x136183fd, 95 },
    { 0x909183ff, 76 },
    { 0xa1d18400, 76 },
    { 0xf73183ff, 33 },
    { 0x619183fb, 33 },
    { 0xbfa18403, 91 },
    { 0xf2618404, 68 },
    { 0, 0 },
    { 0xa18406, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf18409, 95 },
    { 0xbb81840a, 23 },
    { 0x1501840a, 98 },
    { 0xa1e1840a, 82 },
    { 0xab11840c, 43 },
    { 0x2dd1840e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52318412, 55 },
    { 0x10718413, 90 },
    { 0xfaf18413, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90918418, 67 },
    { 0, 0 },
    { 0x63f1841a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161841f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af18425, 34 },
    { 0, 0 },
    { 0x68418427, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021842e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39918432, 88 },
    { 0x6d518433, 67 },
    { 0xd6918434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65918437, 61 },
    { 0, 0 },
    { 0xe6c18439, 90 },
    { 0x39f1843a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd1843e, 36 },
    { 0xdb91843e, 55 },
    { 0, 0 },
    { 0xbca18441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b18445, 33 },
    { 0x7bb18446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18449, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1844d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b18452, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97818456, 33 },
    { 0xe0618457, 90 },
    { 0xcfe18457, 23 },
    { 0xd0618457, 33 },
    { 0x5cb1845a, 43 },
    { 0xe3b18456, 43 },
    { 0, 0 },
    { 0x4f41845d, 33 },
    { 0, 0 },
    { 0x9fe1845f, 90 },
    { 0x70718460, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff18464, 61 },
    { 0x5fb18464, 90 },
    { 0x57a18464, 33 },
    { 0, 0 },
    { 0xc4618468, 33 },
    { 0x66d18469, 23 },
    { 0xfc118469, 33 },
    { 0, 0 },
    { 0xbf41846c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff18470, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe718474, 93 },
    { 0xd9f18474, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb311847a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e18482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64218486, 91 },
    { 0x71618487, 61 },
    { 0x6de18486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc1848f, 33 },
    { 0x45118490, 61 },
    { 0, 0 },
    { 0xa8218492, 95 },
    { 0xa6218493, 23 },
    { 0x80118494, 95 },
    { 0, 0 },
    { 0x1c818496, 33 },
    { 0, 0 },
    { 0x97b18498, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1849c, 76 },
    { 0x6881849c, 100 },
    { 0, 0 },
    { 0x1301849f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b184a2, 43 },
    { 0x22e184a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab184a7, 87 },
    { 0x6b2184a8, 33 },
    { 0x668184a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x394184ac, 61 },
    { 0x153184ad, 76 },
    { 0x6aa184ac, 98 },
    { 0x48b184af, 61 },
    { 0x7ad184af, 33 },
    { 0xc6f184b1, 93 },
    { 0x565184b2, 61 },
    { 0, 0 },
    { 0x29a184b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2184b7, 33 },
    { 0xed2184b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1184c0, 91 },
    { 0x7a6184c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753184c5, 87 },
    { 0x480184c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db184c9, 85 },
    { 0, 0 },
    { 0x2dc184cb, 83 },
    { 0, 0 },
    { 0xf52184cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655184d2, 61 },
    { 0, 0 },
    { 0x4a4184d4, 33 },
    { 0x555184d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d184db, 93 },
    { 0x873184db, 87 },
    { 0x8cf184dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5184e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85184ea, 76 },
    { 0xece184eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6184f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26184fc, 76 },
    { 0x237184fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e18502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d18508, 90 },
    { 0, 0 },
    { 0xea01850a, 61 },
    { 0, 0 },
    { 0xcc91850c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc081850f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e18514, 65 },
    { 0x8c518515, 95 },
    { 0, 0 },
    { 0xdce18517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe51851b, 36 },
    { 0xce71851b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c18520, 98 },
    { 0xe4418521, 93 },
    { 0x2ac18522, 85 },
    { 0xf6018523, 61 },
    { 0, 0 },
    { 0x4fd18525, 100 },
    { 0, 0 },
    { 0xbb318527, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f1852c, 23 },
    { 0x9d91852c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x16918530, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c18536, 90 },
    { 0, 0 },
    { 0x89618538, 43 },
    { 0, 0 },
    { 0x9e01853a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4371853e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85518541, 81 },
    { 0xb7818542, 33 },
    { 0xca718543, 81 },
    { 0, 0 },
    { 0x7ae18545, 61 },
    { 0xd18546, 65 },
    { 0x17718545, 61 },
    { 0xac318548, 84 },
    { 0xe7818549, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1751854d, 33 },
    { 0x1d1854e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74718551, 33 },
    { 0x37518552, 66 },
    { 0, 0 },
    { 0xde718554, 95 },
    { 0x5db18554, 76 },
    { 0, 0 },
    { 0x95c18557, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8241855a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f91855f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e18562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd018568, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae418570, 61 },
    { 0, 0 },
    { 0x35d18572, 61 },
    { 0x7de18573, 76 },
    { 0xd3118573, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18218577, 34 },
    { 0, 0 },
    { 0x9f118579, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071857c, 61 },
    { 0xe6c1857d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf18580, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e918587, 78 },
    { 0x59318587, 90 },
    { 0x3cc18587, 33 },
    { 0x48b1858a, 98 },
    { 0x2591858a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6601858e, 98 },
    { 0, 0 },
    { 0xee718590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb18594, 99 },
    { 0x8be18595, 90 },
    { 0x47018596, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd31859b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26185a0, 88 },
    { 0, 0 },
    { 0x79c185a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9185a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68185ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586185b0, 76 },
    { 0xfeb185b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff185b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03185c2, 33 },
    { 0xdd6185c3, 95 },
    { 0xafb185c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c185c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64185ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408185ce, 33 },
    { 0x2eb185cf, 90 },
    { 0, 0 },
    { 0xdae185d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e185d6, 90 },
    { 0x12c185d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4185dd, 91 },
    { 0xd61185de, 83 },
    { 0x7e185dd, 95 },
    { 0, 0 },
    { 0x444185e1, 66 },
    { 0x35a185e2, 33 },
    { 0x341185e1, 76 },
    { 0x531185e4, 23 },
    { 0xea4185e5, 34 },
    { 0x5fa185e3, 99 },
    { 0x754185e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7185eb, 52 },
    { 0x13a185ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16185ef, 98 },
    { 0x45d185f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2185f4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c185f8, 84 },
    { 0, 0 },
    { 0x9db185fa, 84 },
    { 0x9c8185fb, 94 },
    { 0xad1185fc, 61 },
    { 0, 0 },
    { 0x2ee185fe, 75 },
    { 0, 0 },
    { 0xc5718600, 90 },
    { 0x7ca18601, 33 },
    { 0x89e18601, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4818605, 98 },
    { 0, 0 },
    { 0xe7218607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc781860a, 33 },
    { 0x321860b, 98 },
    { 0, 0 },
    { 0xbb61860d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd018612, 90 },
    { 0x1d118612, 84 },
    { 0x8dd18613, 98 },
    { 0xa6018612, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d418618, 61 },
    { 0x1eb18619, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7331861d, 80 },
    { 0x9b51861e, 33 },
    { 0x4981861d, 55 },
    { 0x92b1861d, 90 },
    { 0xc9418621, 55 },
    { 0x73a1861e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1618625, 100 },
    { 0, 0 },
    { 0x12e18627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe71862c, 84 },
    { 0, 0 },
    { 0x8df1862e, 99 },
    { 0x7ba1862f, 78 },
    { 0x4418630, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f918634, 98 },
    { 0xe0418635, 90 },
    { 0x58118635, 19 },
    { 0xb7e18637, 95 },
    { 0x27318637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d1863f, 61 },
    { 0, 0 },
    { 0xd6518641, 61 },
    { 0, 0 },
    { 0x78018643, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9318646, 61 },
    { 0xa218647, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1865a, 87 },
    { 0x8261865b, 33 },
    { 0xfa31865a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d318668, 23 },
    { 0x35f18668, 98 },
    { 0x1c51866a, 61 },
    { 0x1311866b, 90 },
    { 0x5c018668, 76 },
    { 0xa661866b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb018670, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e18673, 61 },
    { 0, 0 },
    { 0x93518675, 33 },
    { 0x7e518675, 67 },
    { 0x82318677, 33 },
    { 0x99918677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211867e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc18682, 95 },
    { 0xeae18683, 76 },
    { 0xf1718684, 76 },
    { 0x90c18684, 61 },
    { 0x72718684, 66 },
    { 0x89d18686, 61 },
    { 0, 0 },
    { 0xc5418689, 23 },
    { 0, 0 },
    { 0xdc91868b, 34 },
    { 0xe311868b, 61 },
    { 0x4e51868d, 67 },
    { 0x4dd1868e, 90 },
    { 0, 0 },
    { 0x5ed18690, 97 },
    { 0xb1418691, 61 },
    { 0x88718692, 55 },
    { 0xd3518692, 34 },
    { 0xabc18692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1869e, 99 },
    { 0x97c1869f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff186a4, 55 },
    { 0x42186a5, 34 },
    { 0x7e6186a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a186a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119186b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b186b6, 69 },
    { 0xe9f186b6, 65 },
    { 0x791186b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b186bb, 46 },
    { 0, 0 },
    { 0x7d0186bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb186c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c186cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57186d0, 61 },
    { 0x632186d1, 95 },
    { 0xa75186d2, 34 },
    { 0, 0 },
    { 0xe23186d4, 90 },
    { 0xd86186d5, 98 },
    { 0x4f4186d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3186dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a186e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91186e4, 87 },
    { 0x21a186e4, 95 },
    { 0x66b186e6, 98 },
    { 0xb60186e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1186ed, 61 },
    { 0x8d1186ed, 98 },
    { 0, 0 },
    { 0xcca186f0, 91 },
    { 0, 0 },
    { 0x19c186f2, 84 },
    { 0xd99186f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957186f7, 88 },
    { 0, 0 },
    { 0x9ac186f9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b186fd, 61 },
    { 0, 0 },
    { 0xa26186ff, 61 },
    { 0, 0 },
    { 0xe0f18701, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43418705, 46 },
    { 0xc0c18705, 23 },
    { 0xb6018707, 93 },
    { 0x39318705, 95 },
    { 0xbe418709, 95 },
    { 0, 0 },
    { 0x7f11870b, 95 },
    { 0, 0 },
    { 0x64c1870d, 93 },
    { 0, 0 },
    { 0x6f51870f, 95 },
    { 0xbb418710, 23 },
    { 0x63518711, 46 },
    { 0x56a18710, 55 },
    { 0xf6d18713, 55 },
    { 0, 0 },
    { 0xacc18715, 88 },
    { 0, 0 },
    { 0x6c418717, 52 },
    { 0x4118718, 75 },
    { 0, 0 },
    { 0x41871a, 98 },
    { 0x1101871a, 61 },
    { 0x7271871c, 43 },
    { 0xfc91871b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff818722, 77 },
    { 0x7d118723, 33 },
    { 0xf6e18723, 61 },
    { 0x3d918723, 34 },
    { 0, 0 },
    { 0x1f318727, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1872a, 93 },
    { 0xbb01872a, 90 },
    { 0xd891872c, 23 },
    { 0, 0 },
    { 0x8541872e, 61 },
    { 0xfbc1872f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f18733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea18739, 98 },
    { 0, 0 },
    { 0x7dd1873b, 67 },
    { 0, 0 },
    { 0xc871873d, 34 },
    { 0x2371873e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6218741, 90 },
    { 0, 0 },
    { 0x4d818743, 98 },
    { 0x1ad18744, 95 },
    { 0xa7418744, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8518748, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4461874c, 34 },
    { 0, 0 },
    { 0x3401874e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e18754, 66 },
    { 0, 0 },
    { 0x1e218756, 100 },
    { 0xeb718757, 76 },
    { 0x4e918757, 87 },
    { 0x60918759, 98 },
    { 0x2a31875a, 61 },
    { 0x1df1875b, 61 },
    { 0, 0 },
    { 0x8f1875d, 61 },
    { 0x3c21875e, 76 },
    { 0xcb81875f, 69 },
    { 0x3c41875f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16018763, 90 },
    { 0x8c618764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8591876b, 34 },
    { 0x2691876c, 33 },
    { 0, 0 },
    { 0x9191876e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88618771, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e18779, 23 },
    { 0xd061877a, 98 },
    { 0xf8f1877a, 98 },
    { 0xfac1877c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63218781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97318787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1878d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9918790, 99 },
    { 0xce718790, 81 },
    { 0x52618791, 91 },
    { 0x30118791, 61 },
    { 0x42518794, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d818798, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x611879b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2241879f, 23 },
    { 0x3a3187a0, 23 },
    { 0xe54187a1, 46 },
    { 0x3c2187a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee187a7, 23 },
    { 0x2b187a8, 33 },
    { 0xf98187a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4187ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8187b0, 90 },
    { 0xb2d187b1, 36 },
    { 0xdf3187b2, 76 },
    { 0xb55187b3, 98 },
    { 0xca3187b0, 61 },
    { 0xf8187b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5187b8, 46 },
    { 0x330187b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67187c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6187c9, 95 },
    { 0xd05187ca, 76 },
    { 0x54d187cb, 91 },
    { 0, 0 },
    { 0x992187cd, 95 },
    { 0, 0 },
    { 0x53e187cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7187d9, 33 },
    { 0xbca187da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e187e5, 99 },
    { 0xf54187e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4187ea, 67 },
    { 0x67c187eb, 61 },
    { 0x9cc187ea, 95 },
    { 0xe2187ed, 90 },
    { 0, 0 },
    { 0x30b187ef, 34 },
    { 0xd5d187f0, 23 },
    { 0, 0 },
    { 0xa42187f2, 61 },
    { 0xcfd187f2, 33 },
    { 0xba9187f4, 34 },
    { 0x515187f5, 90 },
    { 0x8ac187f6, 93 },
    { 0, 0 },
    { 0x230187f8, 99 },
    { 0, 0 },
    { 0x4c8187fa, 90 },
    { 0x7cd187fb, 55 },
    { 0xb66187fa, 61 },
    { 0x74f187fd, 98 },
    { 0x11d187fb, 33 },
    { 0x4ca187ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e18802, 68 },
    { 0x1f718802, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d118807, 61 },
    { 0xa2218808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e1880d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab18814, 61 },
    { 0, 0 },
    { 0xaca18816, 93 },
    { 0, 0 },
    { 0xc8118818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f1881c, 33 },
    { 0x79f1881d, 34 },
    { 0x4301881c, 81 },
    { 0x4ca1881d, 90 },
    { 0, 0 },
    { 0x14218821, 95 },
    { 0, 0 },
    { 0xa218823, 77 },
    { 0xbf318823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58518827, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4741882a, 84 },
    { 0, 0 },
    { 0xe081882c, 46 },
    { 0, 0 },
    { 0x7ec1882e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21418834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d18838, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21883b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4631883f, 55 },
    { 0, 0 },
    { 0x54118841, 93 },
    { 0, 0 },
    { 0x42c18843, 23 },
    { 0x7d218844, 55 },
    { 0x93c18844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b51884b, 93 },
    { 0xc751884c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2781884f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a18855, 52 },
    { 0, 0 },
    { 0xfaf18857, 69 },
    { 0x12718858, 23 },
    { 0xafa18859, 98 },
    { 0x9d918859, 85 },
    { 0xe2c18859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e618860, 76 },
    { 0xba818861, 98 },
    { 0x24018862, 33 },
    { 0x96b18863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae418867, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21886a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f618874, 98 },
    { 0x61718874, 87 },
    { 0xd0b18875, 98 },
    { 0, 0 },
    { 0x88418878, 33 },
    { 0x79118879, 93 },
    { 0, 0 },
    { 0xd531887b, 33 },
    { 0xdd31887c, 99 },
    { 0x5811887c, 33 },
    { 0xbbf1887c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d718886, 61 },
    { 0xc6f18887, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1888f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76518892, 95 },
    { 0, 0 },
    { 0x84718894, 61 },
    { 0x1a718895, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98188a0, 23 },
    { 0xf8b188a1, 90 },
    { 0x7f8188a1, 55 },
    { 0x8d6188a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de188a6, 95 },
    { 0x809188a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1188aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833188af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401188b6, 33 },
    { 0, 0 },
    { 0x1b188b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371188c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2188cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f188d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be188d8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01188dd, 61 },
    { 0, 0 },
    { 0x3ec188df, 90 },
    { 0xcf9188e0, 67 },
    { 0, 0 },
    { 0x8cb188e2, 66 },
    { 0, 0 },
    { 0xe9188e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58188e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c188ee, 55 },
    { 0xd2e188ee, 23 },
    { 0, 0 },
    { 0x348188f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0188f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269188f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502188fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1118901, 33 },
    { 0, 0 },
    { 0xf518903, 61 },
    { 0xcb818903, 33 },
    { 0x8318905, 61 },
    { 0x90818906, 61 },
    { 0x21918907, 69 },
    { 0x7bc18907, 55 },
    { 0xfee18909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7018912, 90 },
    { 0xa3618913, 98 },
    { 0, 0 },
    { 0xc3c18915, 61 },
    { 0, 0 },
    { 0xcfa18917, 46 },
    { 0xdad18918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f1891b, 100 },
    { 0xb811891c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1718924, 82 },
    { 0, 0 },
    { 0xefe18926, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1892d, 76 },
    { 0x1111892e, 46 },
    { 0x7601892e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65718935, 55 },
    { 0xbd818936, 98 },
    { 0, 0 },
    { 0x13918938, 46 },
    { 0xcbc18938, 95 },
    { 0x47118939, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc451893e, 36 },
    { 0, 0 },
    { 0xfba18940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda18944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4418948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb031894d, 33 },
    { 0, 0 },
    { 0x8211894f, 55 },
    { 0xf1a18950, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75918956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc021895d, 34 },
    { 0, 0 },
    { 0xd5c1895f, 33 },
    { 0xfe518960, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8218966, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f71896c, 93 },
    { 0xd2f1896d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6718970, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7118973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b18976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51897c, 90 },
    { 0x7861897d, 66 },
    { 0, 0 },
    { 0xf471897f, 55 },
    { 0x9ca18980, 84 },
    { 0x2c518980, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c018984, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518987, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd511898d, 23 },
    { 0, 0 },
    { 0xea31898f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e618992, 46 },
    { 0xe3b18992, 90 },
    { 0, 0 },
    { 0xe3a18995, 76 },
    { 0, 0 },
    { 0xafe18997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7371899d, 91 },
    { 0x5501899e, 90 },
    { 0xb3e1899d, 58 },
    { 0x3d21899d, 33 },
    { 0, 0 },
    { 0xab5189a2, 66 },
    { 0, 0 },
    { 0xd5d189a4, 95 },
    { 0, 0 },
    { 0xae0189a6, 23 },
    { 0x529189a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595189b1, 98 },
    { 0, 0 },
    { 0xb5e189b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0189b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66189bb, 99 },
    { 0, 0 },
    { 0xa05189bd, 99 },
    { 0xb36189be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c189c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166189ce, 90 },
    { 0, 0 },
    { 0xb5c189d0, 76 },
    { 0, 0 },
    { 0xae3189d2, 23 },
    { 0x421189d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6189d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3189db, 47 },
    { 0, 0 },
    { 0xb74189dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9189e1, 76 },
    { 0, 0 },
    { 0xc25189e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7189e8, 17 },
    { 0x461189e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b189f6, 43 },
    { 0xc26189f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b189fb, 93 },
    { 0xc6b189fc, 95 },
    { 0, 0 },
    { 0xe2f189fe, 33 },
    { 0, 0 },
    { 0x55318a00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b18a08, 17 },
    { 0x55418a09, 23 },
    { 0x40018a08, 84 },
    { 0xe2e18a0b, 80 },
    { 0x71318a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53318a10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb918a17, 33 },
    { 0, 0 },
    { 0x96118a19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd18a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a118a23, 90 },
    { 0, 0 },
    { 0xdf118a25, 98 },
    { 0x7fa18a26, 48 },
    { 0xb8618a25, 33 },
    { 0, 0 },
    { 0xa2a18a29, 84 },
    { 0, 0 },
    { 0xb4318a2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83618a2e, 33 },
    { 0xad818a2f, 95 },
    { 0, 0 },
    { 0xd5118a31, 52 },
    { 0xcb818a31, 23 },
    { 0xb6918a33, 99 },
    { 0x69f18a32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e18a37, 34 },
    { 0x10018a37, 88 },
    { 0xceb18a37, 33 },
    { 0x7b218a3a, 98 },
    { 0xacc18a3b, 91 },
    { 0x29c18a3c, 66 },
    { 0, 0 },
    { 0xf9718a3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb718a48, 90 },
    { 0xd3418a48, 65 },
    { 0, 0 },
    { 0xed918a4b, 90 },
    { 0, 0 },
    { 0x38b18a4d, 98 },
    { 0, 0 },
    { 0x21318a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c18a54, 61 },
    { 0xf1d18a55, 61 },
    { 0x25518a55, 90 },
    { 0, 0 },
    { 0x9c218a58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c18a5c, 61 },
    { 0, 0 },
    { 0x26918a5e, 66 },
    { 0xc3118a5f, 95 },
    { 0, 0 },
    { 0x95c18a61, 76 },
    { 0xc018a61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218a67, 61 },
    { 0, 0 },
    { 0xf2218a69, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10118a6e, 55 },
    { 0xe918a6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75018a73, 90 },
    { 0, 0 },
    { 0xb2c18a75, 100 },
    { 0, 0 },
    { 0x32918a77, 40 },
    { 0x7d518a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d218a80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd818a83, 90 },
    { 0x77b18a84, 33 },
    { 0x8e18a85, 91 },
    { 0, 0 },
    { 0xd0c18a87, 61 },
    { 0x2f618a88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae518a95, 61 },
    { 0, 0 },
    { 0x3da18a97, 65 },
    { 0, 0 },
    { 0x96d18a99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb18a9c, 55 },
    { 0, 0 },
    { 0x56e18a9e, 55 },
    { 0, 0 },
    { 0x32218aa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c18aa5, 23 },
    { 0xaa318aa6, 95 },
    { 0x21f18aa7, 66 },
    { 0xf0d18aa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f18aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9418ab0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b18ab6, 76 },
    { 0x6a018ab6, 77 },
    { 0xbd518ab8, 90 },
    { 0x70618ab7, 45 },
    { 0xf2718aba, 55 },
    { 0x99618ab7, 76 },
    { 0x46618ab8, 100 },
    { 0xced18ab9, 95 },
    { 0xb4c18ab9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2618ac2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b18ac9, 33 },
    { 0x2fe18aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c18acd, 90 },
    { 0xe0618acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb18ad2, 90 },
    { 0xcf918ad3, 76 },
    { 0x58018ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab18ada, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a18ae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b818ae7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf18aea, 76 },
    { 0xe4918aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b118af0, 90 },
    { 0x81d18af0, 23 },
    { 0x6c218af2, 95 },
    { 0x59d18af0, 95 },
    { 0x78b18af4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76918af8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a118afb, 90 },
    { 0x50918afb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be18aff, 33 },
    { 0x35a18aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27218b03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa18b06, 98 },
    { 0x3a118b06, 67 },
    { 0x86418b06, 100 },
    { 0, 0 },
    { 0xeb718b0a, 90 },
    { 0, 0 },
    { 0x7318b0c, 78 },
    { 0xc3918b0d, 85 },
    { 0x63118b0e, 61 },
    { 0x68418b0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b818b15, 65 },
    { 0x43218b16, 99 },
    { 0xd9218b17, 23 },
    { 0, 0 },
    { 0x63518b19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d18b21, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x50218b24, 95 },
    { 0, 0 },
    { 0xcc718b26, 33 },
    { 0x5e418b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5718b2b, 98 },
    { 0, 0 },
    { 0xd1118b2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f118b33, 34 },
    { 0, 0 },
    { 0x10918b35, 23 },
    { 0x7f418b35, 91 },
    { 0x97718b37, 34 },
    { 0x92518b38, 55 },
    { 0x5d718b39, 55 },
    { 0x3ea18b39, 55 },
    { 0xfce18b37, 61 },
    { 0xca418b3c, 78 },
    { 0xf8218b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed118b41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6118b46, 98 },
    { 0x2b218b46, 90 },
    { 0, 0 },
    { 0xf0318b49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17518b4d, 95 },
    { 0x2218b4e, 90 },
    { 0x1e718b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0518b55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa18b5b, 23 },
    { 0x13b18b5b, 36 },
    { 0, 0 },
    { 0x6818b5e, 61 },
    { 0, 0 },
    { 0x62818b60, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x54218b63, 33 },
    { 0, 0 },
    { 0xb6918b65, 90 },
    { 0x1d518b66, 98 },
    { 0, 0 },
    { 0x9c618b68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15318b74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc18b77, 23 },
    { 0x518b78, 90 },
    { 0xe8f18b77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60218b85, 90 },
    { 0x68218b86, 100 },
    { 0, 0 },
    { 0x34518b88, 98 },
    { 0xb6118b88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91018b8e, 96 },
    { 0, 0 },
    { 0x3e818b90, 61 },
    { 0x4ea18b90, 98 },
    { 0xe5018b90, 23 },
    { 0x17618b91, 93 },
    { 0, 0 },
    { 0x87018b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3018b99, 90 },
    { 0, 0 },
    { 0xc1d18b9b, 61 },
    { 0xf7218b9c, 95 },
    { 0x6118b9c, 98 },
    { 0x9f118b9b, 95 },
    { 0x93a18b9f, 66 },
    { 0xbb918ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93018ba4, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb18ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21818baa, 90 },
    { 0, 0 },
    { 0x8df18bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca18bb0, 90 },
    { 0xffc18bb1, 98 },
    { 0xbea18bb1, 61 },
    { 0x85918bb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66918bb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86018bbc, 17 },
    { 0, 0 },
    { 0x75d18bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe018bc2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a18bc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63818bc9, 88 },
    { 0x6cc18bc9, 98 },
    { 0x8bc18bca, 23 },
    { 0x47c18bcb, 90 },
    { 0xa4218bcd, 23 },
    { 0xa6118bce, 33 },
    { 0xc3e18bce, 78 },
    { 0x81e18bd0, 33 },
    { 0, 0 },
    { 0x88918bd2, 43 },
    { 0, 0 },
    { 0xfbf18bd4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35618bd8, 98 },
    { 0, 0 },
    { 0x79518bda, 90 },
    { 0x20718bdb, 90 },
    { 0x4aa18bdc, 34 },
    { 0x57f18bdd, 34 },
    { 0x80718bdc, 91 },
    { 0xb7518bde, 34 },
    { 0x20818be0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f18be5, 55 },
    { 0, 0 },
    { 0x98918be7, 61 },
    { 0, 0 },
    { 0xe2f18be9, 33 },
    { 0x13018be9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c18bf1, 91 },
    { 0x3a218bf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e518bf8, 93 },
    { 0x61918bf8, 98 },
    { 0, 0 },
    { 0xed18bfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc318c02, 61 },
    { 0x7b318c03, 98 },
    { 0xe6418c04, 77 },
    { 0x1fd18c04, 55 },
    { 0xc5318c04, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c818c0e, 95 },
    { 0x43e18c0e, 90 },
    { 0x2c718c0f, 90 },
    { 0x38418c11, 46 },
    { 0x17e18c11, 76 },
    { 0x72b18c13, 61 },
    { 0xf2418c12, 34 },
    { 0xf9118c15, 95 },
    { 0, 0 },
    { 0x90118c17, 33 },
    { 0xbd718c17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa918c1e, 34 },
    { 0, 0 },
    { 0xbd718c20, 99 },
    { 0x6cb18c21, 90 },
    { 0xe4818c22, 95 },
    { 0x6ba18c23, 98 },
    { 0, 0 },
    { 0x88c18c25, 55 },
    { 0x38618c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b318c2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42818c31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1718c34, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6818c38, 98 },
    { 0x84418c39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d018c3c, 34 },
    { 0, 0 },
    { 0xd5c18c3e, 33 },
    { 0x50e18c3f, 95 },
    { 0xdf118c40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0818c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f18c4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e18c50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c718c53, 90 },
    { 0x90318c54, 33 },
    { 0x35518c53, 95 },
    { 0x73f18c56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce18c5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a18c60, 69 },
    { 0x2218c61, 61 },
    { 0xe0518c62, 95 },
    { 0x58f18c63, 67 },
    { 0xfd218c60, 55 },
    { 0, 0 },
    { 0x86118c66, 33 },
    { 0, 0 },
    { 0xfc318c68, 33 },
    { 0x75818c68, 61 },
    { 0xd5a18c6a, 34 },
    { 0x94018c6b, 58 },
    { 0xfee18c6b, 95 },
    { 0, 0 },
    { 0x6fa18c6e, 61 },
    { 0x61418c6f, 98 },
    { 0x30918c70, 99 },
    { 0x51118c71, 46 },
    { 0xf0f18c70, 98 },
    { 0x9ce18c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6818c77, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f18c82, 97 },
    { 0x6c18c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818c86, 66 },
    { 0x6ce18c87, 33 },
    { 0xcbf18c87, 98 },
    { 0, 0 },
    { 0xcba18c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd18c91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c18c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e118c99, 91 },
    { 0, 0 },
    { 0x1e218c9b, 33 },
    { 0xd9e18c9c, 33 },
    { 0x33718c9d, 84 },
    { 0xa3718c9e, 46 },
    { 0, 0 },
    { 0x76c18ca0, 55 },
    { 0xe1118ca1, 33 },
    { 0, 0 },
    { 0xb6c18ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5718ca7, 23 },
    { 0xe8e18ca7, 98 },
    { 0x5c418ca8, 98 },
    { 0x44118caa, 98 },
    { 0x19618ca7, 99 },
    { 0x1b118cac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d318cb2, 33 },
    { 0x27918cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41618cb7, 90 },
    { 0xabc18cb8, 98 },
    { 0x69618cb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a618cbd, 91 },
    { 0xc818cbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf18cc6, 85 },
    { 0x19618cc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed318cce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4018cd1, 61 },
    { 0x6ae18cd1, 66 },
    { 0x6f318cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3318cd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9018cde, 98 },
    { 0x46718cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a18ce5, 95 },
    { 0x99918ce6, 91 },
    { 0x17e18ce6, 33 },
    { 0x92918ce7, 95 },
    { 0xd2318ce8, 55 },
    { 0, 0 },
    { 0xb8918ceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d618cf2, 61 },
    { 0x6918cf3, 46 },
    { 0x34b18cf4, 95 },
    { 0x58b18cf3, 40 },
    { 0x7b318cf6, 33 },
    { 0, 0 },
    { 0xa5218cf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b18cfb, 100 },
    { 0x2018cfc, 46 },
    { 0xc9018cfd, 90 },
    { 0x37518cfd, 55 },
    { 0xb8418cfe, 76 },
    { 0x4c418cff, 67 },
    { 0xc0418d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a18d06, 68 },
    { 0xfee18d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad18d0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b118d11, 98 },
    { 0x92518d11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca218d16, 98 },
    { 0xdc918d17, 23 },
    { 0xc2118d16, 76 },
    { 0xf0c18d19, 23 },
    { 0x9bc18d1a, 33 },
    { 0x97818d1a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8418d1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b318d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab18d27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd618d2e, 90 },
    { 0x39b18d2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf18d32, 98 },
    { 0, 0 },
    { 0xc8818d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57918d3d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa718d40, 95 },
    { 0, 0 },
    { 0x5e318d42, 90 },
    { 0x8c918d42, 93 },
    { 0xee518d44, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7318d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c18d51, 98 },
    { 0, 0 },
    { 0x8fd18d53, 98 },
    { 0x46b18d53, 90 },
    { 0, 0 },
    { 0xfa818d56, 87 },
    { 0, 0 },
    { 0xf0418d58, 43 },
    { 0, 0 },
    { 0xe9318d5a, 76 },
    { 0, 0 },
    { 0x90a18d5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b718d61, 55 },
    { 0xe2b18d61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6318d65, 76 },
    { 0x88b18d66, 46 },
    { 0xe2918d67, 90 },
    { 0x7c018d68, 90 },
    { 0xf2118d65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f018d6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c318d72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27618d77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f18d7b, 78 },
    { 0, 0 },
    { 0x45418d7d, 99 },
    { 0x78718d7d, 55 },
    { 0x9c418d7f, 98 },
    { 0xc4718d80, 90 },
    { 0x78f18d81, 66 },
    { 0x62f18d80, 98 },
    { 0xfad18d80, 33 },
    { 0x14118d82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a18d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31218d90, 90 },
    { 0xf4718d91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8918d95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28918d98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff18d9d, 95 },
    { 0, 0 },
    { 0x62b18d9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf418da3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f18daf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63618db5, 90 },
    { 0, 0 },
    { 0x8b018db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85718dc2, 98 },
    { 0xdbf18dc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418dc7, 91 },
    { 0x36c18dc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c18dcb, 23 },
    { 0, 0 },
    { 0xef918dcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f18dd4, 98 },
    { 0, 0 },
    { 0xd2318dd6, 90 },
    { 0x35e18dd7, 78 },
    { 0x69718dd7, 40 },
    { 0xab118dd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e618ddc, 87 },
    { 0, 0 },
    { 0x80618dde, 66 },
    { 0x83f18dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd18de2, 95 },
    { 0x20218de2, 23 },
    { 0xd9618de4, 90 },
    { 0x7f18de3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b518de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf018dec, 90 },
    { 0xeb618ded, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8218df2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2618df9, 98 },
    { 0x46418dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d118dfe, 66 },
    { 0x2be18dfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19718e03, 84 },
    { 0, 0 },
    { 0xe3218e05, 48 },
    { 0x89918e06, 76 },
    { 0xf0118e06, 88 },
    { 0x49b18e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14718e0b, 61 },
    { 0, 0 },
    { 0x9d918e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b18e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80118e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c18e21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5318e24, 66 },
    { 0, 0 },
    { 0xe8518e26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d18e29, 99 },
    { 0xc7318e2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14618e32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e18e36, 98 },
    { 0x4ee18e37, 61 },
    { 0x28318e37, 61 },
    { 0xa0918e36, 93 },
    { 0xf1a18e38, 43 },
    { 0xd4e18e39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f518e3e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79718e47, 61 },
    { 0, 0 },
    { 0x26118e49, 46 },
    { 0, 0 },
    { 0xc7c18e4b, 95 },
    { 0, 0 },
    { 0x86018e4d, 33 },
    { 0, 0 },
    { 0x41618e4f, 90 },
    { 0, 0 },
    { 0x52a18e51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63518e54, 33 },
    { 0x5518e55, 61 },
    { 0x4ba18e56, 98 },
    { 0x9a518e55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18e5a, 43 },
    { 0, 0 },
    { 0x68a18e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6318e61, 76 },
    { 0xdc518e62, 67 },
    { 0, 0 },
    { 0xe5618e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2318e69, 55 },
    { 0xed818e6a, 68 },
    { 0xda918e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b18e6e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc518e76, 33 },
    { 0, 0 },
    { 0xec118e78, 23 },
    { 0, 0 },
    { 0x6de18e7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78018e7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d18e86, 34 },
    { 0, 0 },
    { 0x25c18e88, 85 },
    { 0x2ff18e89, 64 },
    { 0, 0 },
    { 0x3e618e8b, 23 },
    { 0x5c118e8c, 33 },
    { 0, 0 },
    { 0xc4418e8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3118e93, 90 },
    { 0xdce18e94, 80 },
    { 0x12e18e94, 66 },
    { 0xa5518e96, 87 },
    { 0x1ff18e97, 61 },
    { 0x4de18e98, 33 },
    { 0xe1a18e93, 93 },
    { 0xb2c18e9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7418ea0, 61 },
    { 0xd7818ea1, 98 },
    { 0xa3018ea2, 61 },
    { 0, 0 },
    { 0xeb18ea4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea18ea7, 76 },
    { 0x2e618ea7, 77 },
    { 0xbf018ea8, 55 },
    { 0, 0 },
    { 0xc0f18eab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e718eaf, 95 },
    { 0x16f18eaf, 55 },
    { 0xb818eb1, 76 },
    { 0xdf818eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c18eb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e18ebf, 91 },
    { 0x2e718ec0, 90 },
    { 0x35718ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36118ec6, 90 },
    { 0, 0 },
    { 0x1a318ec8, 90 },
    { 0, 0 },
    { 0x4d018eca, 23 },
    { 0x80e18eca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3618ece, 98 },
    { 0x7f218ecf, 90 },
    { 0x71c18ece, 90 },
    { 0xc1918ed1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40518ed6, 91 },
    { 0xb5018ed6, 34 },
    { 0xec018ed8, 23 },
    { 0xf1a18ed8, 95 },
    { 0xbf618eda, 80 },
    { 0, 0 },
    { 0x67018edc, 90 },
    { 0x4a18edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c18ee0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b018ee4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a018ee9, 90 },
    { 0, 0 },
    { 0xd2e18eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a918ef0, 33 },
    { 0x11318ef1, 33 },
    { 0x57018ef2, 46 },
    { 0x6d18ef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba18ef8, 65 },
    { 0x30a18ef8, 25 },
    { 0x47818ef9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d18efe, 55 },
    { 0xa8c18efe, 64 },
    { 0, 0 },
    { 0x3d918f01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce318f06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5318f09, 61 },
    { 0, 0 },
    { 0x3c818f0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a418f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c18f11, 61 },
    { 0, 0 },
    { 0x74d18f13, 67 },
    { 0x30b18f13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab18f18, 33 },
    { 0x84d18f19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b18f1c, 55 },
    { 0x90818f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d18f23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54518f28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6718f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6018f33, 98 },
    { 0x90818f34, 34 },
    { 0xe5518f35, 76 },
    { 0xaee18f36, 33 },
    { 0xfe318f37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3218f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d518f3e, 55 },
    { 0, 0 },
    { 0x44718f40, 33 },
    { 0x81a18f41, 85 },
    { 0xcf418f41, 23 },
    { 0xfb318f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d718f49, 95 },
    { 0x1da18f4a, 90 },
    { 0xa4d18f4b, 61 },
    { 0xbdf18f4c, 55 },
    { 0x5a918f4a, 33 },
    { 0x35b18f4e, 95 },
    { 0xd5c18f4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6918f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd718f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18f5e, 24 },
    { 0x57118f5f, 88 },
    { 0xf0a18f60, 33 },
    { 0, 0 },
    { 0xa7218f62, 23 },
    { 0x10118f62, 33 },
    { 0x88418f62, 23 },
    { 0, 0 },
    { 0x5b418f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc218f69, 95 },
    { 0x9918f6a, 33 },
    { 0x8f418f6a, 90 },
    { 0, 0 },
    { 0x5b18f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97518f71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17118f77, 90 },
    { 0xa7d18f77, 93 },
    { 0x4f718f78, 95 },
    { 0x2ff18f7a, 95 },
    { 0xfda18f7b, 99 },
    { 0xad518f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5518f84, 84 },
    { 0x4718f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb18f89, 23 },
    { 0x73018f89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86218f8e, 81 },
    { 0x8c518f8e, 76 },
    { 0, 0 },
    { 0x5ed18f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6718f94, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6718f9b, 99 },
    { 0x6e518f9b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe418fa0, 23 },
    { 0x86b18fa0, 97 },
    { 0x2618fa2, 90 },
    { 0x8418fa2, 76 },
    { 0, 0 },
    { 0x29518fa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62718fa8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61318fac, 90 },
    { 0x55718fac, 33 },
    { 0, 0 },
    { 0x2ce18faf, 43 },
    { 0, 0 },
    { 0x83118fb1, 61 },
    { 0, 0 },
    { 0x90518fb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe118fb8, 99 },
    { 0, 0 },
    { 0xf2b18fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69818fbe, 87 },
    { 0, 0 },
    { 0xcea18fc0, 61 },
    { 0, 0 },
    { 0xe2818fc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c18fc7, 85 },
    { 0xe1018fc8, 33 },
    { 0x35418fc7, 52 },
    { 0xd6e18fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b18fce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e18fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a18fd5, 76 },
    { 0, 0 },
    { 0x80018fd7, 98 },
    { 0x6d318fd7, 55 },
    { 0x57118fd9, 55 },
    { 0x1218fd9, 93 },
    { 0x11318fdb, 95 },
    { 0x84418fdc, 64 },
    { 0x57418fdd, 75 },
    { 0x18518fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d18fea, 99 },
    { 0x5d218feb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8218fef, 52 },
    { 0xd5618ff0, 23 },
    { 0xbe18ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc618ff8, 56 },
    { 0xe5618ff9, 43 },
    { 0xd1a18ff8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18018fff, 85 },
    { 0x60c18fff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28219004, 52 },
    { 0xb8819005, 75 },
    { 0x91819006, 65 },
    { 0xb0219007, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491900c, 78 },
    { 0, 0 },
    { 0x38e1900e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f19012, 76 },
    { 0x3f519013, 98 },
    { 0xa3d19014, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b619019, 61 },
    { 0x8991901a, 88 },
    { 0x8821901b, 61 },
    { 0, 0 },
    { 0xb1901d, 84 },
    { 0, 0 },
    { 0xeb91901f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47519022, 90 },
    { 0xf5e19023, 52 },
    { 0x29a19023, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc619027, 23 },
    { 0x6ce19028, 98 },
    { 0, 0 },
    { 0xe731902a, 61 },
    { 0, 0 },
    { 0xe281902c, 23 },
    { 0x20a1902d, 84 },
    { 0xd531902c, 53 },
    { 0, 0 },
    { 0xdaf19030, 87 },
    { 0, 0 },
    { 0x8c219032, 78 },
    { 0, 0 },
    { 0x10019034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d19039, 95 },
    { 0, 0 },
    { 0x7c11903b, 34 },
    { 0, 0 },
    { 0x6741903d, 82 },
    { 0xd641903e, 75 },
    { 0xa7c1903f, 90 },
    { 0x1f71903f, 95 },
    { 0x1ef19041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c19048, 61 },
    { 0xcf519049, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331904f, 90 },
    { 0x56019050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36019054, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96819058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9691905e, 23 },
    { 0xde81905f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2919063, 23 },
    { 0, 0 },
    { 0xfa619065, 47 },
    { 0, 0 },
    { 0x96219067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a41906c, 23 },
    { 0x55f1906d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff919070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8919073, 97 },
    { 0, 0 },
    { 0x77519075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3519079, 56 },
    { 0x3821907a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd19080, 66 },
    { 0xffb19080, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d619084, 90 },
    { 0, 0 },
    { 0x55919086, 61 },
    { 0xccf19087, 61 },
    { 0x4e119086, 95 },
    { 0, 0 },
    { 0xb8a1908a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6019095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1519099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6190a0, 90 },
    { 0xd2190a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f190a9, 91 },
    { 0, 0 },
    { 0x671190ab, 90 },
    { 0, 0 },
    { 0x53d190ad, 53 },
    { 0, 0 },
    { 0xd10190af, 65 },
    { 0x6bd190b0, 39 },
    { 0x2dc190b0, 34 },
    { 0x435190b2, 23 },
    { 0xa1a190b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7190bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45190c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f190c5, 23 },
    { 0xda2190c5, 90 },
    { 0x67f190c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a190d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x811190d4, 33 },
    { 0xa25190d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72190db, 90 },
    { 0x213190dc, 34 },
    { 0xf44190dd, 34 },
    { 0, 0 },
    { 0xa0b190df, 34 },
    { 0x30b190e0, 77 },
    { 0x3a3190e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381190e9, 55 },
    { 0x675190ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44190ef, 100 },
    { 0x780190f0, 23 },
    { 0xa2e190f1, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455190f5, 34 },
    { 0x57c190f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711190fa, 43 },
    { 0x4d3190fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95519100, 95 },
    { 0xee319101, 67 },
    { 0xd5f19102, 90 },
    { 0x19102, 61 },
    { 0x39619103, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd11910a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22019110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519113, 100 },
    { 0x3b919114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27519117, 96 },
    { 0x58319118, 43 },
    { 0x71c19118, 77 },
    { 0, 0 },
    { 0xc701911b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed19121, 38 },
    { 0x62919122, 23 },
    { 0x5ac19123, 95 },
    { 0x23219124, 43 },
    { 0x82e19124, 33 },
    { 0xfe419126, 61 },
    { 0x95919125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c01912a, 61 },
    { 0x4b1912b, 90 },
    { 0, 0 },
    { 0x8261912d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b19130, 76 },
    { 0x9b419130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed819135, 23 },
    { 0, 0 },
    { 0xc719137, 55 },
    { 0xf2e19137, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11913d, 98 },
    { 0, 0 },
    { 0x9711913f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc819142, 34 },
    { 0x1f019143, 61 },
    { 0x92919144, 90 },
    { 0x77019145, 96 },
    { 0x6c419146, 61 },
    { 0x86419143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a01914d, 91 },
    { 0x2e71914d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31919153, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee319159, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad71915d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce19167, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61916d, 98 },
    { 0x73a1916d, 95 },
    { 0, 0 },
    { 0x50519170, 25 },
    { 0, 0 },
    { 0x44c19172, 90 },
    { 0x21519172, 93 },
    { 0xce919172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6741917d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19183, 55 },
    { 0x70f19184, 90 },
    { 0x3fb19185, 85 },
    { 0x82919185, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db19189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7461918e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60419192, 93 },
    { 0, 0 },
    { 0x39d19194, 95 },
    { 0xf3519195, 93 },
    { 0x95e19195, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c61919a, 23 },
    { 0x4311919b, 99 },
    { 0xd7f1919c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19191a0, 99 },
    { 0x3a6191a1, 61 },
    { 0x47f191a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8191aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375191af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f191b2, 34 },
    { 0x812191b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08191b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8191bd, 100 },
    { 0x7c191be, 23 },
    { 0xb48191be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0191c3, 61 },
    { 0xf00191c4, 61 },
    { 0x7df191c4, 61 },
    { 0x9ec191c5, 90 },
    { 0x81191c4, 98 },
    { 0x532191c8, 61 },
    { 0x33d191c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c191cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59191d4, 23 },
    { 0xcd1191d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67191e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925191e7, 76 },
    { 0xd1a191e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b191ec, 43 },
    { 0xae7191ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39191f0, 77 },
    { 0xb7a191f0, 98 },
    { 0x693191f1, 33 },
    { 0x867191f2, 61 },
    { 0x90d191f1, 33 },
    { 0xdbf191f5, 43 },
    { 0, 0 },
    { 0xb76191f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9191fb, 76 },
    { 0, 0 },
    { 0xe9c191fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d919202, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74219206, 95 },
    { 0x3a19206, 90 },
    { 0, 0 },
    { 0x4bd19209, 76 },
    { 0xa3119209, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb1920e, 98 },
    { 0xec01920e, 78 },
    { 0x90f19210, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4819213, 93 },
    { 0x34619213, 95 },
    { 0x4f19213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd951921a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb19227, 33 },
    { 0x49a19228, 33 },
    { 0xbfe19229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1421922e, 33 },
    { 0x3561922f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b19233, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52319236, 61 },
    { 0, 0 },
    { 0xc619238, 61 },
    { 0xec619239, 34 },
    { 0x4a219239, 61 },
    { 0x7f41923b, 78 },
    { 0, 0 },
    { 0x2a11923d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e19241, 34 },
    { 0xdbc19241, 33 },
    { 0x98919243, 43 },
    { 0x18119241, 90 },
    { 0, 0 },
    { 0x7da19246, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f1924c, 61 },
    { 0x37b1924d, 98 },
    { 0xdc11924c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb19254, 55 },
    { 0xcc919254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941925b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7719266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d219269, 85 },
    { 0xfa319269, 33 },
    { 0x8971926b, 55 },
    { 0x4231926a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da1926f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e19272, 33 },
    { 0, 0 },
    { 0x1b719274, 33 },
    { 0xf0819275, 66 },
    { 0x98d19276, 98 },
    { 0xf419276, 98 },
    { 0xfad19277, 93 },
    { 0x11019277, 67 },
    { 0x3a21927a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf19289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071928c, 68 },
    { 0x68b1928c, 78 },
    { 0, 0 },
    { 0xc811928f, 59 },
    { 0xd9a19290, 65 },
    { 0xeed19290, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf319296, 61 },
    { 0, 0 },
    { 0x23b19298, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff01929c, 48 },
    { 0x1621929c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x374192a0, 17 },
    { 0x71c192a1, 98 },
    { 0, 0 },
    { 0xbba192a3, 95 },
    { 0x8a0192a4, 88 },
    { 0, 0 },
    { 0x1d3192a6, 33 },
    { 0xf41192a7, 77 },
    { 0, 0 },
    { 0x3ee192a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5192ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380192b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d192b5, 91 },
    { 0x7f6192b6, 23 },
    { 0, 0 },
    { 0x710192b8, 77 },
    { 0, 0 },
    { 0x607192ba, 87 },
    { 0, 0 },
    { 0xb84192bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7192bf, 45 },
    { 0, 0 },
    { 0x3ce192c1, 55 },
    { 0, 0 },
    { 0xca6192c3, 61 },
    { 0x34a192c4, 95 },
    { 0x69192c5, 52 },
    { 0, 0 },
    { 0x2b6192c7, 78 },
    { 0, 0 },
    { 0x144192c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e192cf, 34 },
    { 0, 0 },
    { 0x413192d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab192da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631192e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da192e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08192eb, 90 },
    { 0, 0 },
    { 0xc83192ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c192f1, 90 },
    { 0xeb192f2, 33 },
    { 0xa1d192f2, 76 },
    { 0x200192f4, 46 },
    { 0x9c8192f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2192fd, 90 },
    { 0xb63192fe, 90 },
    { 0, 0 },
    { 0xc0419300, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1819309, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01930c, 52 },
    { 0, 0 },
    { 0x30c1930e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x23519311, 23 },
    { 0xd6019311, 33 },
    { 0, 0 },
    { 0x93c19314, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x47119317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc1931b, 61 },
    { 0xa731931c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa091931f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87919322, 66 },
    { 0x8a519323, 98 },
    { 0, 0 },
    { 0x30c19325, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3619328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871932c, 95 },
    { 0x4651932d, 55 },
    { 0xa421932d, 76 },
    { 0x6b1932c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8019332, 34 },
    { 0x2df19333, 23 },
    { 0xc2919332, 33 },
    { 0, 0 },
    { 0x7fe19336, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1933f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b19342, 90 },
    { 0x1fc19343, 61 },
    { 0xa9119343, 76 },
    { 0xf3519343, 98 },
    { 0x5e219346, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c81934a, 66 },
    { 0xfe1934a, 34 },
    { 0x4c71934a, 61 },
    { 0, 0 },
    { 0xb731934e, 43 },
    { 0x14e1934f, 95 },
    { 0xd5c1934e, 98 },
    { 0xba61934f, 61 },
    { 0, 0 },
    { 0xbee19353, 55 },
    { 0x99319353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d01935d, 33 },
    { 0x5b31935d, 99 },
    { 0xdff1935f, 23 },
    { 0xcb519360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce19363, 23 },
    { 0x1b919364, 43 },
    { 0x77f19364, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9181936a, 23 },
    { 0, 0 },
    { 0xafe1936c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c19374, 61 },
    { 0, 0 },
    { 0x59319376, 61 },
    { 0xc7419377, 66 },
    { 0x77119377, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e51937e, 93 },
    { 0, 0 },
    { 0x7c619380, 98 },
    { 0xe3219381, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a1938a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891938d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9719393, 90 },
    { 0x6c719394, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc131939a, 90 },
    { 0x1691939b, 55 },
    { 0, 0 },
    { 0x44e1939d, 91 },
    { 0x1691939d, 90 },
    { 0xab71939e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd193a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb193a9, 90 },
    { 0x3fe193aa, 46 },
    { 0x8a1193aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de193b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c193c5, 23 },
    { 0x671193c6, 61 },
    { 0x71c193c7, 67 },
    { 0x475193c8, 90 },
    { 0x231193c8, 98 },
    { 0, 0 },
    { 0x88e193cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e193d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935193d7, 76 },
    { 0xc5c193d7, 75 },
    { 0x6eb193d9, 95 },
    { 0x56e193da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d193e1, 23 },
    { 0x35e193e2, 68 },
    { 0xee8193e3, 95 },
    { 0xc2b193e4, 95 },
    { 0xe63193e4, 46 },
    { 0x306193e4, 99 },
    { 0xe28193e4, 33 },
    { 0xd2a193e8, 23 },
    { 0x341193e9, 23 },
    { 0xa3f193e9, 90 },
    { 0x1d193eb, 34 },
    { 0x433193e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1193f1, 33 },
    { 0x483193f1, 33 },
    { 0x891193f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29193f9, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90193fd, 90 },
    { 0xd04193fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0419402, 93 },
    { 0, 0 },
    { 0xb919404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19919409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1940e, 67 },
    { 0x4ef1940e, 34 },
    { 0xb9f19410, 38 },
    { 0xba819411, 23 },
    { 0xfa19412, 100 },
    { 0xcf01940e, 34 },
    { 0x8c019414, 95 },
    { 0x28219413, 95 },
    { 0xd6619413, 95 },
    { 0xe9719417, 95 },
    { 0, 0 },
    { 0xc9919419, 43 },
    { 0xe4f1941a, 23 },
    { 0, 0 },
    { 0x5121941c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c1941f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb19423, 85 },
    { 0xab819424, 90 },
    { 0xa1019424, 90 },
    { 0xc8e19426, 46 },
    { 0xe7019424, 95 },
    { 0, 0 },
    { 0xb2219429, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d1942d, 91 },
    { 0x2e41942e, 46 },
    { 0x2981942f, 90 },
    { 0, 0 },
    { 0x5b019431, 23 },
    { 0, 0 },
    { 0xef419433, 98 },
    { 0x80019433, 61 },
    { 0x54419434, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc901943a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9871943f, 88 },
    { 0x39119440, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a19444, 43 },
    { 0xde419445, 33 },
    { 0, 0 },
    { 0xe2219447, 98 },
    { 0x63619448, 61 },
    { 0xd6819449, 61 },
    { 0, 0 },
    { 0x9b71944b, 33 },
    { 0x93b1944c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3919450, 40 },
    { 0, 0 },
    { 0xe9b19452, 39 },
    { 0x44919453, 46 },
    { 0x29e19452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa19459, 98 },
    { 0x43019459, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e19461, 46 },
    { 0x8a719461, 23 },
    { 0, 0 },
    { 0x12d19464, 39 },
    { 0xae019464, 78 },
    { 0, 0 },
    { 0x70219467, 91 },
    { 0, 0 },
    { 0x81719469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c91946c, 90 },
    { 0xb071946d, 23 },
    { 0x1f01946e, 90 },
    { 0x3841946e, 61 },
    { 0x9421946d, 33 },
    { 0x4011946f, 98 },
    { 0x2b119472, 90 },
    { 0x5b419470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5119477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e1947c, 55 },
    { 0x6a91947c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf119480, 33 },
    { 0, 0 },
    { 0xc7119482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b919487, 90 },
    { 0, 0 },
    { 0x79019489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd11948c, 66 },
    { 0x7a21948d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a19493, 95 },
    { 0xa5519493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa19497, 95 },
    { 0x33619498, 90 },
    { 0xb8f19497, 23 },
    { 0xde419498, 43 },
    { 0x7a11949b, 23 },
    { 0x6e11949c, 33 },
    { 0, 0 },
    { 0x5691949e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a194a1, 90 },
    { 0x7df194a2, 90 },
    { 0, 0 },
    { 0x32194a4, 87 },
    { 0x682194a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de194a8, 33 },
    { 0x359194a9, 55 },
    { 0xebc194a9, 55 },
    { 0xc6b194ab, 99 },
    { 0x512194ac, 61 },
    { 0x6e4194ac, 46 },
    { 0x751194ae, 78 },
    { 0xe21194af, 93 },
    { 0xd19194aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830194b5, 61 },
    { 0xcc2194b6, 61 },
    { 0, 0 },
    { 0x8b3194b8, 46 },
    { 0, 0 },
    { 0xcd6194ba, 93 },
    { 0xe7e194bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab194bf, 43 },
    { 0, 0 },
    { 0x37a194c1, 61 },
    { 0x83a194c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2194c7, 98 },
    { 0x326194c8, 98 },
    { 0x5a7194c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0194cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac194d0, 8 },
    { 0x901194d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e194d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182194e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25194e8, 78 },
    { 0xf55194e9, 34 },
    { 0x49a194e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x105194ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8194f1, 33 },
    { 0x176194f2, 91 },
    { 0x520194f2, 85 },
    { 0x50a194f2, 61 },
    { 0xb5e194f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8194f9, 67 },
    { 0x1bd194fa, 33 },
    { 0x2dd194fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a19501, 84 },
    { 0, 0 },
    { 0x5fe19503, 95 },
    { 0x8e019503, 95 },
    { 0xebc19503, 34 },
    { 0x77819506, 99 },
    { 0xdb119507, 34 },
    { 0x7119508, 87 },
    { 0, 0 },
    { 0x9f41950a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f19511, 33 },
    { 0, 0 },
    { 0x59c19513, 84 },
    { 0x96f19513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59819518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4461951b, 61 },
    { 0x6731951c, 52 },
    { 0x13b1951d, 38 },
    { 0xa011951e, 23 },
    { 0xdce1951d, 46 },
    { 0xe8c1951f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3419528, 78 },
    { 0x9da19529, 23 },
    { 0x2271952a, 39 },
    { 0x6d11952a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7919531, 33 },
    { 0x97419531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99419536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78119545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9219549, 95 },
    { 0, 0 },
    { 0x641954b, 76 },
    { 0, 0 },
    { 0x1f61954d, 88 },
    { 0x3131954e, 43 },
    { 0x14b1954f, 85 },
    { 0x16e19550, 66 },
    { 0xdba1954f, 95 },
    { 0, 0 },
    { 0x7e519553, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd19556, 93 },
    { 0x4ae19556, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f11955b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91955e, 33 },
    { 0xc151955f, 76 },
    { 0x3d719560, 55 },
    { 0x5b019561, 76 },
    { 0x80e19562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15819566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19570, 17 },
    { 0x80419570, 33 },
    { 0xb9f19572, 23 },
    { 0x6c19572, 33 },
    { 0x77319572, 33 },
    { 0x38519575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60119579, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e019586, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e1958b, 98 },
    { 0xa371958b, 35 },
    { 0x9c61958d, 33 },
    { 0x4251958d, 84 },
    { 0x8a1958f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e319593, 95 },
    { 0, 0 },
    { 0x2ab19595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b19598, 43 },
    { 0x1a219599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d11959e, 23 },
    { 0, 0 },
    { 0xe44195a0, 61 },
    { 0x763195a1, 33 },
    { 0, 0 },
    { 0x467195a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05195a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0195ab, 90 },
    { 0, 0 },
    { 0xc03195ad, 95 },
    { 0, 0 },
    { 0x778195af, 76 },
    { 0x9e3195b0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af195b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x411195b8, 34 },
    { 0, 0 },
    { 0x8ef195ba, 61 },
    { 0xf37195bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896195bf, 52 },
    { 0, 0 },
    { 0x5f4195c1, 55 },
    { 0, 0 },
    { 0xa57195c3, 23 },
    { 0xeec195c4, 85 },
    { 0x81a195c5, 76 },
    { 0x15b195c6, 23 },
    { 0x6b3195c4, 33 },
    { 0xe11195c8, 23 },
    { 0x367195c9, 33 },
    { 0x5ed195c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba195cd, 90 },
    { 0xf4f195ce, 67 },
    { 0xd25195cf, 90 },
    { 0, 0 },
    { 0x3f3195d1, 66 },
    { 0x5cc195d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af195d7, 33 },
    { 0x494195d7, 23 },
    { 0, 0 },
    { 0x68b195da, 66 },
    { 0, 0 },
    { 0xa54195dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4195df, 90 },
    { 0, 0 },
    { 0x4f5195e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb195e4, 85 },
    { 0xb68195e4, 95 },
    { 0x487195e6, 61 },
    { 0xac195e7, 100 },
    { 0xf8e195e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a195eb, 23 },
    { 0xd62195eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1195ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6195f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9195f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13195fd, 88 },
    { 0xc6195fe, 95 },
    { 0xc7195ff, 34 },
    { 0xc15195fd, 93 },
    { 0, 0 },
    { 0x20319602, 90 },
    { 0x10019602, 23 },
    { 0x33119604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7119607, 61 },
    { 0x67c19608, 33 },
    { 0, 0 },
    { 0xfbc1960a, 84 },
    { 0x5141960b, 90 },
    { 0, 0 },
    { 0x13b1960d, 76 },
    { 0x38a1960e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2219616, 61 },
    { 0xc1219617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1961a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601961e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a19621, 69 },
    { 0x4a719621, 87 },
    { 0, 0 },
    { 0xcc919624, 23 },
    { 0, 0 },
    { 0x53b19626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31962e, 43 },
    { 0, 0 },
    { 0x77b19630, 90 },
    { 0, 0 },
    { 0xfd619632, 83 },
    { 0x36819633, 33 },
    { 0x39219632, 33 },
    { 0x49419633, 33 },
    { 0xca419636, 91 },
    { 0, 0 },
    { 0x83a19638, 85 },
    { 0, 0 },
    { 0xeab1963a, 83 },
    { 0, 0 },
    { 0xe491963c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c1963f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15019642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e19649, 33 },
    { 0xcd519649, 33 },
    { 0, 0 },
    { 0x9d21964c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e1964f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22619652, 43 },
    { 0xbf719653, 34 },
    { 0, 0 },
    { 0x20219655, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271965d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b19661, 34 },
    { 0xd5e19662, 98 },
    { 0, 0 },
    { 0xb7a19664, 95 },
    { 0xd8d19665, 61 },
    { 0x45019665, 67 },
    { 0x29919667, 34 },
    { 0xfc219668, 100 },
    { 0x99219668, 98 },
    { 0x5d21966a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e1966d, 23 },
    { 0x861966e, 87 },
    { 0x44e1966e, 93 },
    { 0, 0 },
    { 0x49a19671, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62919677, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc471967b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e419681, 55 },
    { 0x8a419681, 90 },
    { 0x5b119683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db19687, 23 },
    { 0xf6419687, 90 },
    { 0xcc819689, 98 },
    { 0xaf919688, 23 },
    { 0xe4d1968b, 33 },
    { 0, 0 },
    { 0xa391968d, 55 },
    { 0x98f1968e, 77 },
    { 0xf341968f, 55 },
    { 0x32019690, 90 },
    { 0x75719690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f019694, 76 },
    { 0, 0 },
    { 0xea419696, 91 },
    { 0, 0 },
    { 0xdd219698, 78 },
    { 0x40819699, 23 },
    { 0x92519698, 100 },
    { 0, 0 },
    { 0x84c1969c, 80 },
    { 0, 0 },
    { 0xb851969e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7196a2, 98 },
    { 0, 0 },
    { 0xc9196a4, 67 },
    { 0xdd2196a4, 99 },
    { 0, 0 },
    { 0x157196a7, 33 },
    { 0x608196a8, 99 },
    { 0xafa196a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2196b0, 23 },
    { 0x932196b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c196b5, 33 },
    { 0, 0 },
    { 0x91c196b7, 33 },
    { 0xbfd196b8, 98 },
    { 0xad5196b9, 90 },
    { 0x73d196b9, 51 },
    { 0x429196bb, 61 },
    { 0xf1f196bb, 33 },
    { 0xb60196bd, 91 },
    { 0x4ab196be, 87 },
    { 0xcc6196bf, 95 },
    { 0x1e4196bc, 95 },
    { 0xd35196c1, 98 },
    { 0x948196c0, 33 },
    { 0xe8196c2, 78 },
    { 0x4e5196c0, 55 },
    { 0xf53196c5, 43 },
    { 0, 0 },
    { 0x398196c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9196ca, 23 },
    { 0x493196ca, 34 },
    { 0x779196cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719196d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79196d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1196d8, 23 },
    { 0xa80196d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87196de, 98 },
    { 0x19c196df, 90 },
    { 0, 0 },
    { 0x9aa196e1, 23 },
    { 0x86b196e1, 90 },
    { 0xb5d196e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23196e6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40196e9, 61 },
    { 0x645196ea, 76 },
    { 0xe42196eb, 95 },
    { 0x1e9196ea, 43 },
    { 0xf3b196e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce196f8, 100 },
    { 0xcc1196f9, 90 },
    { 0, 0 },
    { 0xf24196fb, 90 },
    { 0, 0 },
    { 0xf5e196fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66019701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f919705, 40 },
    { 0, 0 },
    { 0xfcd19707, 35 },
    { 0x5e319708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1970b, 87 },
    { 0xb441970c, 33 },
    { 0x7821970b, 76 },
    { 0x2d41970e, 23 },
    { 0x9f41970f, 46 },
    { 0x29519710, 43 },
    { 0x1b319711, 76 },
    { 0xf4e19710, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf31971a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81971e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b619722, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a819727, 100 },
    { 0, 0 },
    { 0xe719729, 33 },
    { 0x4c11972a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9419735, 66 },
    { 0xfdd19736, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa961973b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b719740, 98 },
    { 0xe3519741, 76 },
    { 0x8a919741, 96 },
    { 0x75f19743, 46 },
    { 0x2c619742, 90 },
    { 0, 0 },
    { 0x16c19746, 88 },
    { 0, 0 },
    { 0xe1319748, 34 },
    { 0, 0 },
    { 0xf9d1974a, 46 },
    { 0xb681974b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa19751, 61 },
    { 0xb19752, 93 },
    { 0xe4f19751, 90 },
    { 0x8b519754, 85 },
    { 0x9f119755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd819759, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf151975d, 99 },
    { 0, 0 },
    { 0x6e1975f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219763, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe11976b, 94 },
    { 0xeba1976c, 95 },
    { 0x4641976b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c419770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b19773, 33 },
    { 0xa9619774, 46 },
    { 0xd4d19774, 33 },
    { 0x51e19776, 90 },
    { 0, 0 },
    { 0x44819778, 95 },
    { 0, 0 },
    { 0x8c41977a, 61 },
    { 0x8151977b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a919780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d419783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b019786, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xab219789, 55 },
    { 0x92a19789, 23 },
    { 0x7341978b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1978f, 55 },
    { 0xbab1978f, 33 },
    { 0xaa619791, 98 },
    { 0x1a419792, 100 },
    { 0x12119793, 61 },
    { 0x52019794, 23 },
    { 0xa3b19793, 61 },
    { 0, 0 },
    { 0xf3119797, 95 },
    { 0, 0 },
    { 0x3cd19799, 95 },
    { 0x1c519799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8197a1, 90 },
    { 0, 0 },
    { 0x7d7197a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e197a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0197ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8197ae, 61 },
    { 0xe8197af, 2 },
    { 0x2d6197af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7197ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f197be, 46 },
    { 0xa42197be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97197ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d197ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85197d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de197d4, 78 },
    { 0x762197d5, 61 },
    { 0xe76197d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0197d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7197dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b197e4, 78 },
    { 0, 0 },
    { 0x90d197e6, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63197ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c197f0, 66 },
    { 0xc39197f1, 90 },
    { 0, 0 },
    { 0xa98197f3, 55 },
    { 0x7fd197f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85197f7, 34 },
    { 0, 0 },
    { 0x677197f9, 90 },
    { 0, 0 },
    { 0x60b197fb, 76 },
    { 0x324197fb, 43 },
    { 0x4c6197fd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d19806, 87 },
    { 0x3d519806, 78 },
    { 0xf8919808, 61 },
    { 0, 0 },
    { 0x3061980a, 93 },
    { 0, 0 },
    { 0xd2d1980c, 98 },
    { 0xc261980c, 61 },
    { 0, 0 },
    { 0x44a1980f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca1981e, 34 },
    { 0xec41981f, 61 },
    { 0xfea19820, 33 },
    { 0, 0 },
    { 0xcfc19822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab519825, 34 },
    { 0, 0 },
    { 0x2ca19827, 61 },
    { 0, 0 },
    { 0x95719829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c1982e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93319837, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1983c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf19841, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea19844, 55 },
    { 0x7c519845, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a19848, 95 },
    { 0, 0 },
    { 0xbd1984a, 90 },
    { 0x1891984a, 34 },
    { 0x99e1984c, 17 },
    { 0xd521984d, 66 },
    { 0, 0 },
    { 0x5c1984f, 61 },
    { 0x1211984f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb919853, 52 },
    { 0, 0 },
    { 0x79f19855, 93 },
    { 0x43c19856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981985d, 98 },
    { 0x5091985e, 34 },
    { 0xc0b1985f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a419866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc441986d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e19871, 46 },
    { 0x72319872, 23 },
    { 0xd4a19873, 66 },
    { 0xca119872, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81919877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf41987b, 48 },
    { 0x6c11987c, 61 },
    { 0, 0 },
    { 0x2d31987e, 61 },
    { 0, 0 },
    { 0x19e19880, 98 },
    { 0, 0 },
    { 0x23f19882, 93 },
    { 0xec519882, 98 },
    { 0, 0 },
    { 0x8d819885, 95 },
    { 0x25019886, 23 },
    { 0xbbe19887, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9451988e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec419892, 98 },
    { 0x37f19892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f19898, 55 },
    { 0xd719899, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9441989d, 76 },
    { 0, 0 },
    { 0x5121989f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570198a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f198a7, 59 },
    { 0xe15198a7, 33 },
    { 0x813198a7, 98 },
    { 0x191198aa, 78 },
    { 0xe3e198ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0198ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4198b1, 55 },
    { 0, 0 },
    { 0x579198b3, 90 },
    { 0x4e198b4, 33 },
    { 0x508198b3, 99 },
    { 0, 0 },
    { 0x344198b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4198ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f198be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645198c5, 61 },
    { 0xea0198c6, 23 },
    { 0, 0 },
    { 0xe6c198c8, 61 },
    { 0xd76198c9, 95 },
    { 0, 0 },
    { 0x20b198cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09198d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284198da, 23 },
    { 0xf74198da, 98 },
    { 0xa8b198dc, 98 },
    { 0, 0 },
    { 0x247198de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83198e1, 90 },
    { 0x607198e2, 76 },
    { 0xb69198e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0198ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d198f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8198f4, 90 },
    { 0x72c198f4, 95 },
    { 0x260198f6, 34 },
    { 0, 0 },
    { 0x9aa198f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07198fd, 67 },
    { 0xd0198fe, 23 },
    { 0, 0 },
    { 0x37219900, 61 },
    { 0x6e19900, 85 },
    { 0x65819901, 44 },
    { 0, 0 },
    { 0xcd919904, 90 },
    { 0x1b19904, 98 },
    { 0xb19906, 98 },
    { 0, 0 },
    { 0xf9e19908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211990c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8019910, 33 },
    { 0, 0 },
    { 0x84a19912, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f19915, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7819920, 76 },
    { 0, 0 },
    { 0x1fc19922, 55 },
    { 0x1b719923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42819929, 61 },
    { 0, 0 },
    { 0xae81992b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431992e, 40 },
    { 0, 0 },
    { 0xc5719930, 90 },
    { 0, 0 },
    { 0xbb219932, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d19937, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431993a, 76 },
    { 0xc511993b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b81993e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3919941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c71994b, 66 },
    { 0x3d01994b, 77 },
    { 0x19d1994d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0519951, 90 },
    { 0x7cf19952, 40 },
    { 0xaf19953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1219956, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31995b, 61 },
    { 0, 0 },
    { 0x4ee1995d, 61 },
    { 0x7f71995e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef419962, 55 },
    { 0xb719962, 34 },
    { 0x4b319964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf19967, 43 },
    { 0xc8a19968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1996b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e119971, 90 },
    { 0, 0 },
    { 0x27119973, 90 },
    { 0x13319973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97819977, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e81997b, 32 },
    { 0xdd91997c, 23 },
    { 0, 0 },
    { 0x9491997e, 89 },
    { 0x7c1997f, 55 },
    { 0x6181997f, 33 },
    { 0x83d19981, 67 },
    { 0x81e1997e, 90 },
    { 0x7801997f, 61 },
    { 0x6661997e, 33 },
    { 0x96819985, 98 },
    { 0x9dc19985, 90 },
    { 0x70519987, 66 },
    { 0x4b719987, 76 },
    { 0xff19985, 78 },
    { 0x28419985, 55 },
    { 0x41d1998b, 90 },
    { 0, 0 },
    { 0x9041998d, 95 },
    { 0x501998d, 33 },
    { 0x5b41998f, 98 },
    { 0x7019990, 87 },
    { 0x34f19991, 85 },
    { 0x3f119992, 17 },
    { 0x33719991, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69919998, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc1999c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac1999f, 98 },
    { 0xfa4199a0, 95 },
    { 0x90b199a0, 67 },
    { 0xbe2199a2, 66 },
    { 0x678199a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de199a9, 87 },
    { 0, 0 },
    { 0x714199ab, 96 },
    { 0x672199ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f199b0, 98 },
    { 0x73a199b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3199b4, 55 },
    { 0x670199b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1199c0, 61 },
    { 0x919199c0, 90 },
    { 0xcad199c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b199c6, 55 },
    { 0xe8e199c7, 61 },
    { 0xdbc199c8, 85 },
    { 0xb46199c6, 96 },
    { 0, 0 },
    { 0x9e3199cb, 33 },
    { 0x7d9199cb, 76 },
    { 0x2c4199cd, 61 },
    { 0xe1b199ce, 66 },
    { 0xc5f199cf, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7199d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0199d5, 67 },
    { 0, 0 },
    { 0xfcb199d7, 33 },
    { 0, 0 },
    { 0xefc199d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e199e0, 88 },
    { 0, 0 },
    { 0x789199e2, 43 },
    { 0x54b199e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0199e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d199ee, 26 },
    { 0x6bb199ef, 98 },
    { 0x639199ee, 95 },
    { 0x773199f1, 76 },
    { 0xa39199f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df199f6, 23 },
    { 0xcef199f6, 95 },
    { 0x4c5199f8, 43 },
    { 0x98f199f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee19a00, 85 },
    { 0x77419a00, 53 },
    { 0xa9e19a02, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8019a05, 61 },
    { 0xa7919a06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc19a0a, 88 },
    { 0xa6519a0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a19a11, 33 },
    { 0xf0819a11, 23 },
    { 0x8e619a13, 50 },
    { 0x8d419a11, 98 },
    { 0xe5819a15, 88 },
    { 0x8c119a15, 61 },
    { 0xe5119a16, 66 },
    { 0x51419a16, 58 },
    { 0xf119a15, 90 },
    { 0xa9019a11, 87 },
    { 0xc0919a14, 61 },
    { 0xd1d19a1c, 55 },
    { 0x54719a1b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c19a22, 23 },
    { 0x29c19a22, 23 },
    { 0x1f419a24, 87 },
    { 0x3a319a25, 95 },
    { 0, 0 },
    { 0xb9319a27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42019a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe619a2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb19a33, 98 },
    { 0, 0 },
    { 0x3f919a35, 55 },
    { 0x93519a36, 75 },
    { 0x6019a37, 90 },
    { 0x9919a38, 33 },
    { 0x71119a36, 90 },
    { 0xb2919a36, 90 },
    { 0x47719a3b, 98 },
    { 0xc919a39, 90 },
    { 0xa0219a3d, 99 },
    { 0xe2e19a3d, 93 },
    { 0x93f19a3f, 93 },
    { 0, 0 },
    { 0xe3919a41, 33 },
    { 0x34a19a42, 95 },
    { 0xc4719a41, 76 },
    { 0x64019a44, 23 },
    { 0, 0 },
    { 0x89b19a46, 61 },
    { 0x47119a47, 23 },
    { 0xd4f19a48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f019a4b, 61 },
    { 0x92a19a4b, 40 },
    { 0xf6c19a4d, 90 },
    { 0x8bb19a4e, 23 },
    { 0x23019a4f, 90 },
    { 0x2319a4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d19a55, 98 },
    { 0x61e19a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4119a59, 52 },
    { 0xa919a59, 90 },
    { 0xd1419a59, 34 },
    { 0xac019a5c, 84 },
    { 0xf7619a5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a419a60, 90 },
    { 0, 0 },
    { 0xf8919a62, 78 },
    { 0x79319a63, 43 },
    { 0x97a19a62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f19a69, 66 },
    { 0x14219a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c19a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f19a74, 100 },
    { 0x67419a75, 87 },
    { 0x9f219a74, 55 },
    { 0x4cb19a77, 43 },
    { 0, 0 },
    { 0xa6a19a79, 84 },
    { 0, 0 },
    { 0xf6219a7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f19a7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67219a82, 61 },
    { 0, 0 },
    { 0xf3a19a84, 90 },
    { 0x6c219a84, 95 },
    { 0, 0 },
    { 0xc3419a87, 91 },
    { 0x5c19a87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39519a8b, 69 },
    { 0, 0 },
    { 0x42119a8d, 85 },
    { 0x72019a8e, 33 },
    { 0x83619a8f, 95 },
    { 0x45819a90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4819a93, 43 },
    { 0xceb19a93, 90 },
    { 0, 0 },
    { 0x2b19a96, 90 },
    { 0, 0 },
    { 0xfb719a98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43219a9b, 95 },
    { 0, 0 },
    { 0xf0419a9d, 33 },
    { 0xf1319a9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61719aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f119aaa, 61 },
    { 0xa2019aab, 88 },
    { 0xbca19aaa, 90 },
    { 0x6a319aad, 95 },
    { 0xa4419aad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e019ab1, 93 },
    { 0x68219ab2, 33 },
    { 0x8e619ab3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a19ab9, 69 },
    { 0xc5219aba, 98 },
    { 0, 0 },
    { 0x5e19abc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc19abf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17619ac4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec19ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41719acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16519ad2, 90 },
    { 0x21d19ad3, 75 },
    { 0, 0 },
    { 0x87d19ad5, 77 },
    { 0xe6319ad6, 23 },
    { 0xab619ad6, 34 },
    { 0xace19ad8, 93 },
    { 0xa6719ad7, 90 },
    { 0x6b519ada, 65 },
    { 0x40a19adb, 91 },
    { 0x4e119adb, 99 },
    { 0xfcb19adb, 90 },
    { 0x5f119adc, 33 },
    { 0x1af19adf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b419ae5, 100 },
    { 0x1e619ae6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12619aea, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de19aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f419af2, 81 },
    { 0, 0 },
    { 0xc5119af4, 78 },
    { 0xc2419af5, 33 },
    { 0x48319af6, 95 },
    { 0x18b19af7, 66 },
    { 0, 0 },
    { 0xb8d19af9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67019b00, 61 },
    { 0, 0 },
    { 0x52719b02, 55 },
    { 0xb7019b02, 95 },
    { 0, 0 },
    { 0xc9119b05, 23 },
    { 0, 0 },
    { 0x80a19b07, 33 },
    { 0x85119b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9819b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f19b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d19b11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac19b1a, 61 },
    { 0xa2319b1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb19b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22719b28, 76 },
    { 0xb2919b29, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff19b2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e19b30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8619b36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f619b3c, 75 },
    { 0x4aa19b3d, 95 },
    { 0xe4019b3c, 95 },
    { 0x2b19b3f, 66 },
    { 0xed919b40, 90 },
    { 0x83b19b41, 77 },
    { 0xd6719b3e, 23 },
    { 0, 0 },
    { 0x79919b44, 95 },
    { 0x85319b45, 61 },
    { 0x37919b45, 46 },
    { 0x14f19b47, 91 },
    { 0x8b519b45, 40 },
    { 0, 0 },
    { 0x61e19b4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a419b4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3919b52, 69 },
    { 0, 0 },
    { 0xb6f19b54, 33 },
    { 0, 0 },
    { 0x7da19b56, 93 },
    { 0x19d19b56, 76 },
    { 0xe3e19b58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0719b5e, 76 },
    { 0, 0 },
    { 0x54e19b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82519b66, 85 },
    { 0, 0 },
    { 0xf5519b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c19b6c, 46 },
    { 0x94819b6d, 55 },
    { 0x3a819b6d, 61 },
    { 0xdc919b6d, 68 },
    { 0x5bc19b70, 61 },
    { 0x4b319b6f, 84 },
    { 0xfa319b6d, 61 },
    { 0xe7019b73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a19b76, 33 },
    { 0, 0 },
    { 0xa6f19b78, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d919b7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde19b80, 95 },
    { 0x48219b81, 98 },
    { 0x2b119b82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede19b87, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e19b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa19b8d, 33 },
    { 0xd4519b8d, 17 },
    { 0, 0 },
    { 0xc1519b90, 98 },
    { 0, 0 },
    { 0x3819b92, 90 },
    { 0x7219b93, 100 },
    { 0x1f119b93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58319b97, 90 },
    { 0x4e719b98, 33 },
    { 0x22819b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a19b9c, 61 },
    { 0x27019b9c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x52319ba0, 66 },
    { 0x5fa19ba1, 76 },
    { 0, 0 },
    { 0xc9c19ba3, 61 },
    { 0x61b19ba4, 55 },
    { 0, 0 },
    { 0x44519ba6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86619baa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2219bae, 43 },
    { 0, 0 },
    { 0xe7519bb0, 55 },
    { 0xd319bb1, 23 },
    { 0x7d319bb2, 78 },
    { 0xf5019bb0, 90 },
    { 0x97a19bb4, 66 },
    { 0xf9419bb5, 55 },
    { 0xbbd19bb4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e19bc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd19bc5, 33 },
    { 0, 0 },
    { 0xadd19bc7, 34 },
    { 0x78b19bc8, 99 },
    { 0, 0 },
    { 0xe9f19bca, 61 },
    { 0, 0 },
    { 0xe1b19bcc, 93 },
    { 0xe2819bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0319bd5, 77 },
    { 0x72519bd6, 95 },
    { 0x8c119bd7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b19bda, 95 },
    { 0x16519bdb, 95 },
    { 0xa0e19bda, 61 },
    { 0x36019bdd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b19be1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca119be4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa019bea, 77 },
    { 0xcaa19bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f19bef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31019bf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70319bfb, 98 },
    { 0x17919bfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a419c00, 99 },
    { 0x15019c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8319c06, 99 },
    { 0, 0 },
    { 0x2c19c08, 61 },
    { 0x25e19c09, 23 },
    { 0, 0 },
    { 0xd6e19c0b, 90 },
    { 0x2be19c0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb319c0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f19c1b, 83 },
    { 0xdcc19c1b, 43 },
    { 0, 0 },
    { 0x23819c1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f19c21, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb19c24, 34 },
    { 0xf3c19c25, 61 },
    { 0x11119c26, 46 },
    { 0xb1a19c26, 99 },
    { 0xb4819c28, 23 },
    { 0x4c219c28, 55 },
    { 0x14f19c2a, 33 },
    { 0x65d19c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b919c2e, 61 },
    { 0xa9919c2f, 23 },
    { 0x6e519c2f, 34 },
    { 0xfe019c2e, 90 },
    { 0xcbd19c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a19c39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e819c41, 90 },
    { 0x31919c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea719c45, 91 },
    { 0xff919c45, 90 },
    { 0x3b519c47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff19c4a, 23 },
    { 0x28119c4a, 17 },
    { 0, 0 },
    { 0xc8519c4d, 52 },
    { 0xdf919c4d, 85 },
    { 0x7e119c4f, 90 },
    { 0, 0 },
    { 0xa3e19c51, 99 },
    { 0x70819c52, 46 },
    { 0x3be19c53, 61 },
    { 0, 0 },
    { 0x35319c55, 90 },
    { 0, 0 },
    { 0x7b19c57, 77 },
    { 0x73a19c57, 95 },
    { 0xdd119c59, 52 },
    { 0x7b519c5a, 61 },
    { 0, 0 },
    { 0x79d19c5c, 61 },
    { 0x74b19c5d, 90 },
    { 0xb7719c5d, 55 },
    { 0x3b119c5e, 90 },
    { 0x17319c5f, 98 },
    { 0xafb19c61, 95 },
    { 0x82219c5f, 46 },
    { 0, 0 },
    { 0x15419c64, 98 },
    { 0, 0 },
    { 0xb1b19c66, 34 },
    { 0, 0 },
    { 0xbd319c68, 90 },
    { 0, 0 },
    { 0x17719c6a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb19c6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec19c70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18519c75, 98 },
    { 0x54a19c76, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce19c7a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb619c7f, 46 },
    { 0x5f919c7f, 34 },
    { 0, 0 },
    { 0x1f419c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1519c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa19c8b, 55 },
    { 0x38119c8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd119c91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c519c94, 84 },
    { 0, 0 },
    { 0x21b19c96, 90 },
    { 0x53f19c97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3319c9c, 61 },
    { 0x3b419c9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db19ca0, 46 },
    { 0xa0519ca0, 23 },
    { 0x39619ca2, 61 },
    { 0, 0 },
    { 0x7db19ca4, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91719ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2619cac, 43 },
    { 0x16c19cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad19cb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda19cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86719cc1, 76 },
    { 0xee019cc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d719cc5, 98 },
    { 0, 0 },
    { 0xe8719cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c19ccb, 90 },
    { 0x73519ccc, 69 },
    { 0xb9219ccc, 93 },
    { 0, 0 },
    { 0xd7f19ccf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1119cd4, 93 },
    { 0, 0 },
    { 0x14219cd6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43919cd9, 61 },
    { 0xf6d19cda, 95 },
    { 0xf2119cdb, 46 },
    { 0xe0919cdb, 99 },
    { 0x81919cdc, 85 },
    { 0x7ec19cde, 99 },
    { 0x76a19cdc, 33 },
    { 0x56a19cdb, 23 },
    { 0xbad19cda, 24 },
    { 0x20d19ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b19ce9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b19cf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e19cf9, 95 },
    { 0, 0 },
    { 0x7c119cfb, 55 },
    { 0x43d19cfc, 93 },
    { 0x39819cfd, 85 },
    { 0xbc219cfd, 46 },
    { 0x51c19cfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c819d02, 33 },
    { 0, 0 },
    { 0x7ac19d04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e19d0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68419d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37119d13, 95 },
    { 0x7db19d13, 46 },
    { 0xdda19d15, 34 },
    { 0, 0 },
    { 0x5a019d17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a619d1b, 98 },
    { 0, 0 },
    { 0x32019d1d, 90 },
    { 0, 0 },
    { 0xf7819d1f, 87 },
    { 0xd1519d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51719d24, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2619d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be19d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26719d31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a19d35, 95 },
    { 0x91319d36, 77 },
    { 0x72519d35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d319d3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3019d3d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f19d42, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea19d48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e219d4e, 33 },
    { 0x8f119d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe19d53, 23 },
    { 0x2f619d54, 97 },
    { 0xd5019d54, 94 },
    { 0x82a19d56, 66 },
    { 0xfbc19d53, 95 },
    { 0xd0619d58, 80 },
    { 0, 0 },
    { 0xfce19d5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a19d5d, 99 },
    { 0x60619d5e, 95 },
    { 0, 0 },
    { 0xfd619d60, 61 },
    { 0xa1519d60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df19d66, 76 },
    { 0x49f19d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed19d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec19d73, 97 },
    { 0, 0 },
    { 0xf0019d75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719d7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8119d81, 84 },
    { 0x77e19d82, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc19d87, 61 },
    { 0, 0 },
    { 0xb5319d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50119d8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3419d91, 52 },
    { 0x5f719d91, 90 },
    { 0x4b19d93, 33 },
    { 0x38719d94, 76 },
    { 0xaf219d95, 33 },
    { 0x98819d95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d19d9a, 87 },
    { 0, 0 },
    { 0x4f619d9c, 88 },
    { 0x13e19d9c, 23 },
    { 0x9da19d9c, 45 },
    { 0x49619d9f, 81 },
    { 0x60819d9f, 67 },
    { 0x33919da0, 38 },
    { 0x41719d9d, 61 },
    { 0, 0 },
    { 0x52b19da4, 75 },
    { 0x8f719da5, 61 },
    { 0x2ee19da4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a19dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a119daf, 90 },
    { 0xa1319db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c919db3, 92 },
    { 0x4a919db3, 34 },
    { 0x51519db5, 61 },
    { 0, 0 },
    { 0x71119db7, 95 },
    { 0x94419db8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c519dbb, 66 },
    { 0x8b19dbc, 90 },
    { 0, 0 },
    { 0x53219dbe, 61 },
    { 0, 0 },
    { 0x92419dc0, 90 },
    { 0, 0 },
    { 0x6e719dc2, 90 },
    { 0x7b719dc3, 61 },
    { 0x92d19dc4, 81 },
    { 0xf9a19dc4, 90 },
    { 0x10d19dc2, 101 },
    { 0x7fc19dc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf19dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19dcf, 61 },
    { 0x11c19dd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c19dd3, 100 },
    { 0, 0 },
    { 0x40d19dd5, 12 },
    { 0, 0 },
    { 0x54619dd7, 56 },
    { 0xfd019dd7, 99 },
    { 0x46719dd9, 95 },
    { 0x55e19dda, 23 },
    { 0x1bf19ddb, 76 },
    { 0, 0 },
    { 0x58119ddd, 93 },
    { 0xfc819ddd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19de5, 43 },
    { 0x9f419de6, 76 },
    { 0, 0 },
    { 0x16719de8, 90 },
    { 0x54419de9, 90 },
    { 0, 0 },
    { 0x53c19deb, 64 },
    { 0xce819dec, 90 },
    { 0xa6619ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23919df1, 87 },
    { 0, 0 },
    { 0xc8319df3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8419df6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b19df9, 61 },
    { 0xd7d19dfa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de19e07, 64 },
    { 0, 0 },
    { 0x42919e09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c19e0d, 33 },
    { 0, 0 },
    { 0xc3c19e0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f19e13, 46 },
    { 0x2b19e14, 90 },
    { 0xa2919e15, 55 },
    { 0x2e219e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e019e1a, 52 },
    { 0xf6719e1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419e1e, 33 },
    { 0, 0 },
    { 0x81519e20, 98 },
    { 0x8ce19e21, 76 },
    { 0, 0 },
    { 0xf8f19e23, 88 },
    { 0, 0 },
    { 0xe0e19e25, 98 },
    { 0x7f619e25, 90 },
    { 0x22019e27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd819e2f, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25619e34, 78 },
    { 0x8619e35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b19e38, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c319e3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57619e41, 98 },
    { 0, 0 },
    { 0x54519e43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f119e52, 68 },
    { 0, 0 },
    { 0xda619e54, 98 },
    { 0x14e19e55, 61 },
    { 0x7ac19e55, 91 },
    { 0x83a19e55, 90 },
    { 0, 0 },
    { 0xea519e59, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc19e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1119e6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4219e6f, 42 },
    { 0xa7219e70, 61 },
    { 0xba419e71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe419e74, 43 },
    { 0x96619e75, 76 },
    { 0x17419e76, 61 },
    { 0, 0 },
    { 0x75419e78, 52 },
    { 0, 0 },
    { 0xfa719e7a, 95 },
    { 0, 0 },
    { 0xfec19e7c, 87 },
    { 0, 0 },
    { 0x9ce19e7e, 98 },
    { 0x4f119e7f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e219e82, 43 },
    { 0x97e19e83, 98 },
    { 0x28c19e84, 97 },
    { 0, 0 },
    { 0x43a19e86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae19e8a, 95 },
    { 0xdcf19e8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b19e8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf819e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe19e95, 91 },
    { 0x32419e96, 65 },
    { 0xc7919e95, 75 },
    { 0x52219e98, 23 },
    { 0xd3919e98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4419ea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff19ea8, 93 },
    { 0xd3b19ea8, 98 },
    { 0xc9719ea9, 33 },
    { 0, 0 },
    { 0x6c719eac, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x76019eaf, 90 },
    { 0xa3119eb0, 23 },
    { 0x43719eb1, 90 },
    { 0x34c19eb0, 90 },
    { 0xb6119eb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89919eb8, 90 },
    { 0xd4019eb8, 78 },
    { 0x8c519eba, 76 },
    { 0, 0 },
    { 0x17819ebc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae619ec1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c219ec5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4519eca, 46 },
    { 0x9b619eca, 90 },
    { 0, 0 },
    { 0x6e419ecd, 33 },
    { 0xdf019ece, 81 },
    { 0xaa19ecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f19ed3, 46 },
    { 0, 0 },
    { 0xdde19ed5, 87 },
    { 0x67b19ed6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a19ede, 33 },
    { 0x11019ede, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52019ee2, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd19ee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb19ee9, 43 },
    { 0, 0 },
    { 0x9da19eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b119eee, 76 },
    { 0, 0 },
    { 0xec19ef0, 33 },
    { 0x30d19ef1, 90 },
    { 0x8db19ef1, 95 },
    { 0, 0 },
    { 0xe6a19ef4, 98 },
    { 0, 0 },
    { 0x97619ef6, 46 },
    { 0x4f419ef7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f19efa, 34 },
    { 0x6b119efa, 93 },
    { 0xeab19efb, 90 },
    { 0x7f019efd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb119f00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28519f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f719f08, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54419f0c, 90 },
    { 0xbc519f0d, 98 },
    { 0x3cb19f0e, 76 },
    { 0xb7919f0f, 61 },
    { 0x5bd19f10, 23 },
    { 0x4d319f11, 95 },
    { 0xb0519f0d, 61 },
    { 0xaae19f11, 61 },
    { 0x5a519f14, 33 },
    { 0x46b19f15, 69 },
    { 0x4ee19f14, 24 },
    { 0xf419f17, 90 },
    { 0, 0 },
    { 0x20319f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61019f1e, 67 },
    { 0, 0 },
    { 0xb1919f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68019f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e719f2d, 85 },
    { 0x28819f2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f19f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed19f38, 17 },
    { 0xb9219f38, 61 },
    { 0, 0 },
    { 0x5ff19f3b, 98 },
    { 0x1d319f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b419f46, 95 },
    { 0xbd519f47, 33 },
    { 0xf1519f46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc919f4c, 100 },
    { 0x56619f4d, 52 },
    { 0xa1319f4d, 99 },
    { 0xef619f4d, 66 },
    { 0, 0 },
    { 0xfdc19f51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5419f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82919f57, 55 },
    { 0, 0 },
    { 0x2d019f59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd19f5f, 33 },
    { 0x8a319f60, 67 },
    { 0x1c19f60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2019f64, 90 },
    { 0, 0 },
    { 0xb7a19f66, 76 },
    { 0x21b19f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65619f6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f19f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4719f75, 98 },
    { 0x35d19f76, 26 },
    { 0, 0 },
    { 0x2f419f78, 67 },
    { 0x66119f78, 55 },
    { 0x67219f7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3719f83, 33 },
    { 0xcff19f83, 23 },
    { 0x48819f85, 61 },
    { 0x90919f84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b19f89, 98 },
    { 0xca019f8a, 33 },
    { 0xf8819f8a, 78 },
    { 0x1b319f8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c19f91, 61 },
    { 0, 0 },
    { 0x37219f93, 99 },
    { 0, 0 },
    { 0xc1d19f95, 77 },
    { 0xff419f95, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b19f99, 90 },
    { 0xcff19f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7419f9d, 98 },
    { 0, 0 },
    { 0x8a019f9f, 76 },
    { 0x37f19fa0, 55 },
    { 0, 0 },
    { 0x1f919fa2, 90 },
    { 0x66f19fa2, 90 },
    { 0xa6219fa4, 90 },
    { 0x42619fa5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f19fa9, 33 },
    { 0, 0 },
    { 0x4ab19fab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b19fae, 61 },
    { 0x71d19faf, 52 },
    { 0xb2d19fb0, 83 },
    { 0x90319fae, 33 },
    { 0xaa919fb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb719fb5, 76 },
    { 0xd9d19fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60419fbb, 23 },
    { 0x3b819fbc, 33 },
    { 0xcaa19fbb, 90 },
    { 0x6719fbc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f519fc3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x14819fc6, 95 },
    { 0xbf219fc7, 93 },
    { 0x70819fc7, 78 },
    { 0x25f19fc9, 90 },
    { 0xd5b19fc9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d19fcd, 61 },
    { 0x57419fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef919fd4, 23 },
    { 0, 0 },
    { 0x2ea19fd6, 95 },
    { 0x41419fd7, 95 },
    { 0x18c19fd8, 23 },
    { 0xc4b19fd9, 55 },
    { 0, 0 },
    { 0x6bb19fdb, 95 },
    { 0x2dc19fdc, 85 },
    { 0x42b19fdd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x76019fe0, 81 },
    { 0x98919fe1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad19fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a19fe7, 32 },
    { 0x2319fe8, 64 },
    { 0x29819fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0219fef, 78 },
    { 0x19d19fef, 76 },
    { 0x42819ff1, 43 },
    { 0x2f519ff1, 99 },
    { 0xf6119ff3, 78 },
    { 0xc0219fef, 99 },
    { 0, 0 },
    { 0xec319ff6, 33 },
    { 0, 0 },
    { 0x36c19ff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b1a000, 68 },
    { 0, 0 },
    { 0x2a71a002, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb1a005, 35 },
    { 0, 0 },
    { 0xec1a007, 90 },
    { 0x7d01a007, 33 },
    { 0x5201a009, 65 },
    { 0x94f1a009, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf151a00d, 90 },
    { 0, 0 },
    { 0x8681a00f, 98 },
    { 0xa131a010, 95 },
    { 0x6b81a010, 93 },
    { 0x6de1a012, 91 },
    { 0, 0 },
    { 0xbee1a014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a1a01a, 61 },
    { 0xac31a01b, 46 },
    { 0x4a61a01b, 46 },
    { 0xb8c1a01d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a1a020, 100 },
    { 0, 0 },
    { 0xcc71a022, 91 },
    { 0, 0 },
    { 0xa751a024, 90 },
    { 0xd8c1a024, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb721a028, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf161a033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe341a038, 43 },
    { 0x571a038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff1a046, 93 },
    { 0xb1b1a047, 61 },
    { 0x91f1a046, 95 },
    { 0, 0 },
    { 0x931a04a, 33 },
    { 0x6791a04b, 33 },
    { 0xe561a04c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f61a04f, 93 },
    { 0x6701a04f, 88 },
    { 0xe3a1a051, 98 },
    { 0x1b31a052, 69 },
    { 0x13b1a053, 61 },
    { 0xbff1a054, 55 },
    { 0x3f31a055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671a058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821a05c, 88 },
    { 0, 0 },
    { 0x1641a05e, 66 },
    { 0xfe1a05f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b11a063, 85 },
    { 0xc921a064, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e1a06a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891a072, 61 },
    { 0xd761a073, 66 },
    { 0, 0 },
    { 0x2d31a075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771a078, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f1a080, 95 },
    { 0x2f81a080, 23 },
    { 0x14f1a082, 98 },
    { 0x55a1a082, 55 },
    { 0, 0 },
    { 0x8d61a085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1a088, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1a08c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a21a099, 90 },
    { 0x5e11a099, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41a09d, 40 },
    { 0, 0 },
    { 0x2711a09f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe21a0a4, 98 },
    { 0, 0 },
    { 0x7931a0a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b1a0a9, 33 },
    { 0xba01a0aa, 90 },
    { 0xa1b1a0ab, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc991a0b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a0b7, 34 },
    { 0x8a1a0b8, 33 },
    { 0x2e51a0b9, 90 },
    { 0xa191a0ba, 90 },
    { 0x54c1a0bb, 33 },
    { 0xfec1a0ba, 61 },
    { 0, 0 },
    { 0x5221a0be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc671a0c2, 61 },
    { 0x1601a0c2, 95 },
    { 0, 0 },
    { 0x3a91a0c5, 98 },
    { 0xe51a0c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc91a0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e1a0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4471a0d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e1a0dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701a0e0, 46 },
    { 0x51d1a0e1, 33 },
    { 0xd8f1a0e0, 76 },
    { 0x6561a0e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c1a0e8, 34 },
    { 0x2861a0e9, 34 },
    { 0, 0 },
    { 0x56d1a0eb, 90 },
    { 0xd591a0ec, 33 },
    { 0xc5f1a0ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01a0f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b01a0f9, 90 },
    { 0x1941a0f9, 98 },
    { 0x6ab1a0fb, 76 },
    { 0x78e1a0fc, 76 },
    { 0x5831a0fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1a100, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x7271a103, 75 },
    { 0x1341a104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf291a10a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb571a116, 23 },
    { 0, 0 },
    { 0x4721a118, 78 },
    { 0xbd01a118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c01a11c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b11a123, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a128, 23 },
    { 0xcbe1a129, 78 },
    { 0, 0 },
    { 0x1c1a12b, 33 },
    { 0x1e41a12c, 61 },
    { 0x8f81a12d, 90 },
    { 0xf6d1a12e, 55 },
    { 0xc6a1a12e, 76 },
    { 0xbf91a130, 61 },
    { 0x99c1a12f, 23 },
    { 0xa1b1a12d, 84 },
    { 0xc601a12f, 23 },
    { 0xe951a134, 95 },
    { 0x59b1a135, 90 },
    { 0xaeb1a12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71a139, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061a13d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee1a144, 33 },
    { 0x961a145, 84 },
    { 0xd591a146, 33 },
    { 0x2041a145, 74 },
    { 0x2b21a148, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf1a14f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a154, 23 },
    { 0x411a154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc1a160, 34 },
    { 0x6e91a161, 98 },
    { 0xe51a162, 61 },
    { 0x1671a163, 98 },
    { 0x1d31a164, 84 },
    { 0x181a165, 87 },
    { 0, 0 },
    { 0x9791a167, 91 },
    { 0x9801a167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a16b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe181a171, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41a175, 85 },
    { 0xad41a175, 52 },
    { 0x4f81a175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771a17a, 86 },
    { 0x5e51a17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3701a181, 87 },
    { 0xb8b1a182, 76 },
    { 0xabe1a183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c1a188, 98 },
    { 0x56d1a189, 17 },
    { 0xe131a189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd11a191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad01a195, 23 },
    { 0xaa71a195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1521a19c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d1a1a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3391a1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da1a1ac, 95 },
    { 0x8c71a1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121a1b1, 55 },
    { 0x2241a1b2, 43 },
    { 0x3a51a1b2, 64 },
    { 0x4441a1b2, 90 },
    { 0, 0 },
    { 0x6e01a1b6, 95 },
    { 0x1b21a1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f81a1bb, 98 },
    { 0x7d11a1bc, 61 },
    { 0, 0 },
    { 0x18d1a1be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621a1c1, 23 },
    { 0x451a1c2, 90 },
    { 0x4711a1c3, 90 },
    { 0x5e21a1c3, 68 },
    { 0xa131a1c3, 90 },
    { 0, 0 },
    { 0x56b1a1c7, 34 },
    { 0xb391a1c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1a1cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301a1cf, 33 },
    { 0, 0 },
    { 0x8e51a1d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6371a1da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1a1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf511a1e0, 50 },
    { 0x9e41a1e1, 17 },
    { 0xdca1a1e1, 23 },
    { 0xe4b1a1e1, 90 },
    { 0, 0 },
    { 0x5561a1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e01a1e9, 91 },
    { 0x5611a1e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a1ed, 43 },
    { 0, 0 },
    { 0x9fc1a1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21a1f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631a1f9, 95 },
    { 0, 0 },
    { 0x6451a1fb, 65 },
    { 0, 0 },
    { 0x60a1a1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3851a206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6711a20a, 91 },
    { 0xf331a20b, 33 },
    { 0, 0 },
    { 0x4191a20d, 23 },
    { 0x1f11a20d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf711a212, 90 },
    { 0, 0 },
    { 0xe541a214, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671a219, 33 },
    { 0x1b21a21a, 98 },
    { 0x8391a21b, 95 },
    { 0x91e1a21b, 23 },
    { 0x60c1a21c, 93 },
    { 0x9e31a21e, 90 },
    { 0xd5c1a21f, 76 },
    { 0, 0 },
    { 0x2731a221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081a227, 84 },
    { 0x2781a228, 100 },
    { 0, 0 },
    { 0x2ee1a22a, 88 },
    { 0, 0 },
    { 0xab1a22c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b21a230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1a238, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e1a23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x711a23f, 55 },
    { 0x961a23f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf1a243, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b31a248, 77 },
    { 0, 0 },
    { 0x9581a24a, 68 },
    { 0x3aa1a24a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1a250, 95 },
    { 0x4f01a251, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1a259, 91 },
    { 0, 0 },
    { 0xc3a1a25b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa691a25f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021a267, 93 },
    { 0xa221a267, 98 },
    { 0, 0 },
    { 0xa7b1a26a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21a26f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae1a272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41a276, 61 },
    { 0x71b1a277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc531a27a, 98 },
    { 0, 0 },
    { 0x2d61a27c, 90 },
    { 0, 0 },
    { 0x5431a27e, 76 },
    { 0, 0 },
    { 0x2c51a280, 46 },
    { 0, 0 },
    { 0xa171a282, 52 },
    { 0x60f1a283, 33 },
    { 0x84a1a284, 23 },
    { 0x2701a283, 33 },
    { 0, 0 },
    { 0x7a1a287, 52 },
    { 0, 0 },
    { 0x26d1a289, 76 },
    { 0x8fd1a28a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa391a28d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41a290, 98 },
    { 0, 0 },
    { 0x9051a292, 90 },
    { 0, 0 },
    { 0x8d31a294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f1a298, 90 },
    { 0, 0 },
    { 0x91b1a29a, 90 },
    { 0xb541a29b, 52 },
    { 0x1591a29c, 33 },
    { 0xfc71a29b, 90 },
    { 0x5b91a29b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61a2a2, 93 },
    { 0x17f1a2a2, 23 },
    { 0, 0 },
    { 0xfa61a2a5, 33 },
    { 0, 0 },
    { 0x9b01a2a7, 33 },
    { 0xb811a2a8, 94 },
    { 0x78c1a2a9, 88 },
    { 0x6b31a2aa, 90 },
    { 0xcf81a2a7, 61 },
    { 0xb941a2ac, 66 },
    { 0xc621a2ad, 90 },
    { 0x7031a2ae, 23 },
    { 0x65a1a2ae, 81 },
    { 0xf451a2ae, 87 },
    { 0x2461a2b1, 69 },
    { 0xf351a2b1, 61 },
    { 0xbe91a2b1, 87 },
    { 0x9101a2b4, 93 },
    { 0xc7a1a2b4, 90 },
    { 0x8471a2a9, 95 },
    { 0x1871a2b7, 33 },
    { 0xe291a2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841a2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5351a2c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e1a2c8, 85 },
    { 0x5a41a2c9, 23 },
    { 0, 0 },
    { 0x9841a2cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b1a2d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f1a2db, 93 },
    { 0xf9f1a2dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa111a2e1, 33 },
    { 0x25a1a2e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1a2e5, 61 },
    { 0x1d01a2e6, 90 },
    { 0xc3b1a2e7, 61 },
    { 0x4431a2e8, 95 },
    { 0x36e1a2e6, 61 },
    { 0xb591a2ea, 93 },
    { 0x4c91a2e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf371a2f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e1a2f4, 76 },
    { 0x8e61a2f5, 61 },
    { 0x6a81a2f6, 90 },
    { 0xafd1a2f5, 33 },
    { 0, 0 },
    { 0x21c1a2f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071a2fc, 98 },
    { 0, 0 },
    { 0x6441a2fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f01a303, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331a30a, 98 },
    { 0, 0 },
    { 0x5351a30c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9801a30f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a1a312, 99 },
    { 0x3761a313, 88 },
    { 0, 0 },
    { 0xdeb1a315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae51a31a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b1a31e, 65 },
    { 0, 0 },
    { 0x1411a320, 76 },
    { 0x7261a321, 61 },
    { 0x1191a322, 43 },
    { 0x9741a323, 88 },
    { 0xb9b1a323, 34 },
    { 0x85d1a325, 85 },
    { 0xb211a326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4521a32a, 55 },
    { 0xe161a32b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf1a330, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e1a337, 90 },
    { 0x4331a337, 93 },
    { 0x8211a338, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891a341, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea01a34c, 85 },
    { 0x8111a34c, 46 },
    { 0xf0e1a34e, 36 },
    { 0x2851a34f, 97 },
    { 0x1631a350, 66 },
    { 0x52d1a350, 98 },
    { 0xebf1a350, 40 },
    { 0xbe31a351, 90 },
    { 0x4cc1a354, 5 },
    { 0, 0 },
    { 0x4c1a356, 78 },
    { 0x41a1a357, 33 },
    { 0x31f1a357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781a35b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761a35e, 78 },
    { 0, 0 },
    { 0x94a1a360, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd781a365, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451a36b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee1a36e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e1a376, 76 },
    { 0xe501a376, 91 },
    { 0xcc81a378, 34 },
    { 0x4701a379, 61 },
    { 0xe1c1a379, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b61a37d, 85 },
    { 0x9ae1a37e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4881a382, 33 },
    { 0x6e1a383, 88 },
    { 0xe1e1a384, 61 },
    { 0x2ef1a382, 90 },
    { 0x5c91a386, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1a38e, 90 },
    { 0xdea1a38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c1a397, 33 },
    { 0x9551a398, 90 },
    { 0, 0 },
    { 0x4a61a39a, 61 },
    { 0, 0 },
    { 0x7681a39c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9291a3a1, 83 },
    { 0x9051a3a2, 55 },
    { 0xbf71a3a3, 94 },
    { 0xc981a3a4, 90 },
    { 0, 0 },
    { 0xa2a1a3a6, 91 },
    { 0, 0 },
    { 0xedd1a3a8, 23 },
    { 0x3971a3a9, 84 },
    { 0x41b1a3a8, 43 },
    { 0xa5d1a3a8, 23 },
    { 0x6f11a3ac, 64 },
    { 0, 0 },
    { 0x6f61a3ae, 87 },
    { 0x9cf1a3ae, 91 },
    { 0, 0 },
    { 0x2031a3b1, 61 },
    { 0, 0 },
    { 0xbdf1a3b3, 76 },
    { 0x18c1a3b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571a3b7, 76 },
    { 0x9451a3b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11a3c0, 81 },
    { 0x51a1a3c0, 95 },
    { 0, 0 },
    { 0xf011a3c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd811a3c9, 34 },
    { 0x28c1a3ca, 55 },
    { 0x62b1a3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b11a3ce, 61 },
    { 0, 0 },
    { 0x8571a3d0, 93 },
    { 0xff71a3d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101a3d5, 34 },
    { 0, 0 },
    { 0x4ae1a3d7, 33 },
    { 0xae61a3d8, 95 },
    { 0, 0 },
    { 0xd4d1a3da, 33 },
    { 0x48d1a3da, 43 },
    { 0x5c11a3da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d1a3e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31a3f2, 23 },
    { 0, 0 },
    { 0xfeb1a3f4, 33 },
    { 0xac31a3f4, 23 },
    { 0x15d1a3f4, 98 },
    { 0xddd1a3f7, 46 },
    { 0xa5f1a3f7, 90 },
    { 0xa5c1a3f5, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x5301a3fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01a407, 99 },
    { 0, 0 },
    { 0xb8f1a409, 55 },
    { 0x1081a40a, 61 },
    { 0x2181a40a, 95 },
    { 0x4f61a40c, 98 },
    { 0xacf1a40c, 90 },
    { 0, 0 },
    { 0xbcd1a40f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b41a412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91a417, 87 },
    { 0x4e21a418, 23 },
    { 0x64e1a418, 98 },
    { 0xcbe1a419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e1a41d, 46 },
    { 0x8351a41d, 95 },
    { 0xd771a41f, 76 },
    { 0, 0 },
    { 0x8be1a421, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f1a426, 52 },
    { 0x3c81a426, 85 },
    { 0x13f1a427, 98 },
    { 0xd2d1a427, 55 },
    { 0x5ff1a42a, 76 },
    { 0x6c81a428, 95 },
    { 0, 0 },
    { 0x47c1a42d, 78 },
    { 0x6331a42d, 90 },
    { 0x1b51a42f, 95 },
    { 0, 0 },
    { 0xb561a431, 90 },
    { 0x621a431, 87 },
    { 0x2e81a431, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f61a436, 100 },
    { 0, 0 },
    { 0x2df1a438, 33 },
    { 0x7c91a439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee1a43f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a1a443, 33 },
    { 0x2161a443, 76 },
    { 0x471a443, 91 },
    { 0xf061a445, 77 },
    { 0xfee1a447, 90 },
    { 0, 0 },
    { 0xa631a449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc91a44c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e1a453, 61 },
    { 0xf721a454, 61 },
    { 0x3021a455, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d1a458, 87 },
    { 0, 0 },
    { 0xce11a45a, 90 },
    { 0, 0 },
    { 0x1701a45c, 55 },
    { 0, 0 },
    { 0x7ac1a45e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1a464, 55 },
    { 0, 0 },
    { 0xb6c1a466, 33 },
    { 0x2d21a467, 23 },
    { 0xa041a468, 23 },
    { 0x7301a467, 87 },
    { 0xaa81a467, 98 },
    { 0xa411a466, 23 },
    { 0xcd1a469, 76 },
    { 0xfa1a46d, 33 },
    { 0xd641a46d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe411a471, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e1a475, 91 },
    { 0xd11a475, 61 },
    { 0xc9f1a477, 33 },
    { 0xe5b1a477, 75 },
    { 0x5981a479, 65 },
    { 0x8071a475, 33 },
    { 0, 0 },
    { 0xbe21a47c, 33 },
    { 0xbcc1a47d, 90 },
    { 0, 0 },
    { 0x7461a47f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b1a482, 55 },
    { 0, 0 },
    { 0x981a484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191a48c, 90 },
    { 0, 0 },
    { 0xce01a48e, 33 },
    { 0x9831a48f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1a493, 98 },
    { 0x1c21a493, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe571a497, 61 },
    { 0x4151a498, 33 },
    { 0x71b1a498, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c1a4a1, 61 },
    { 0x3f91a4a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8991a4ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d61a4b1, 33 },
    { 0, 0 },
    { 0xb5e1a4b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae1a4b6, 33 },
    { 0x18f1a4b7, 90 },
    { 0x3481a4b6, 61 },
    { 0x1331a4b9, 43 },
    { 0x97e1a4b9, 95 },
    { 0xbd41a4b7, 90 },
    { 0xbb21a4bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f41a4c2, 90 },
    { 0x1cb1a4c2, 90 },
    { 0xc571a4c4, 23 },
    { 0x5181a4c5, 66 },
    { 0xe611a4c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf21a4ca, 95 },
    { 0, 0 },
    { 0xf801a4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11a4d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281a4d6, 84 },
    { 0xa4a1a4d7, 78 },
    { 0x5ac1a4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1a4dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5621a4e2, 33 },
    { 0x1b41a4e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf251a4e7, 66 },
    { 0, 0 },
    { 0xb771a4e9, 78 },
    { 0x7ab1a4ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf11a4ee, 61 },
    { 0, 0 },
    { 0x34f1a4f0, 61 },
    { 0xe6d1a4f1, 97 },
    { 0, 0 },
    { 0x8a41a4f3, 61 },
    { 0x70f1a4f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951a4f8, 76 },
    { 0xf811a4f9, 61 },
    { 0xde21a4f8, 93 },
    { 0xaee1a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351a4fe, 76 },
    { 0x2411a4ff, 61 },
    { 0, 0 },
    { 0xd9b1a501, 88 },
    { 0x71c1a501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc21a508, 95 },
    { 0x7491a509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071a50e, 85 },
    { 0x6e71a50f, 84 },
    { 0, 0 },
    { 0x6b01a511, 90 },
    { 0xdd71a512, 55 },
    { 0x45c1a512, 93 },
    { 0, 0 },
    { 0x29e1a515, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b91a51b, 43 },
    { 0, 0 },
    { 0x3111a51d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1831a521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2811a525, 66 },
    { 0, 0 },
    { 0x1081a527, 98 },
    { 0, 0 },
    { 0xa131a529, 90 },
    { 0x2261a52a, 78 },
    { 0x9371a52b, 90 },
    { 0x74a1a52a, 90 },
    { 0xc441a52c, 98 },
    { 0x8a41a529, 84 },
    { 0x5011a52a, 66 },
    { 0xcb51a52c, 99 },
    { 0, 0 },
    { 0x5021a532, 93 },
    { 0, 0 },
    { 0x7d21a534, 46 },
    { 0xfbb1a535, 51 },
    { 0, 0 },
    { 0xb211a537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851a53b, 43 },
    { 0x7131a53c, 95 },
    { 0, 0 },
    { 0x1421a53e, 98 },
    { 0x4651a53f, 34 },
    { 0xcdd1a540, 61 },
    { 0, 0 },
    { 0x52c1a542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd861a54b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1a54e, 65 },
    { 0, 0 },
    { 0x3bc1a550, 93 },
    { 0xa521a550, 100 },
    { 0x8371a551, 33 },
    { 0xac41a553, 98 },
    { 0x3931a554, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b51a55a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d1a55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3741a560, 33 },
    { 0, 0 },
    { 0xbf61a562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b1a56a, 91 },
    { 0x6a51a56a, 23 },
    { 0x1771a56c, 46 },
    { 0x8f61a56d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7951a572, 87 },
    { 0, 0 },
    { 0xebb1a574, 98 },
    { 0xf471a575, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e1a57d, 85 },
    { 0xcaa1a57d, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe061a589, 76 },
    { 0x6f11a58a, 98 },
    { 0xfa81a58b, 90 },
    { 0x4531a58c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f1a591, 93 },
    { 0, 0 },
    { 0x3ac1a593, 52 },
    { 0, 0 },
    { 0x74d1a595, 90 },
    { 0x451a596, 61 },
    { 0x9f91a597, 33 },
    { 0xda01a597, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1a59d, 90 },
    { 0xaff1a59e, 78 },
    { 0xcfc1a59e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01a5a2, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db1a5a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e21a5a9, 34 },
    { 0x13d1a5aa, 95 },
    { 0x6a91a5a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6311a5af, 23 },
    { 0, 0 },
    { 0x83c1a5b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501a5b5, 33 },
    { 0x87b1a5b6, 43 },
    { 0xbbe1a5b6, 90 },
    { 0xe9d1a5b8, 33 },
    { 0, 0 },
    { 0xea41a5ba, 34 },
    { 0x5921a5bb, 33 },
    { 0, 0 },
    { 0x1391a5bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a5c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b21a5c4, 34 },
    { 0x8721a5c5, 90 },
    { 0x54d1a5c6, 34 },
    { 0xdfe1a5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c1a5cb, 76 },
    { 0, 0 },
    { 0xa011a5cd, 90 },
    { 0x67d1a5ce, 90 },
    { 0x40a1a5cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xef11a5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de1a5d5, 99 },
    { 0, 0 },
    { 0xaad1a5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391a5dc, 88 },
    { 0, 0 },
    { 0xab51a5de, 95 },
    { 0x521a5df, 95 },
    { 0x3e91a5e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa61a5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7241a5e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1a5ed, 55 },
    { 0x9271a5ed, 90 },
    { 0xa6b1a5ef, 93 },
    { 0xe201a5f0, 76 },
    { 0x7f81a5f0, 34 },
    { 0, 0 },
    { 0x9f91a5f3, 23 },
    { 0, 0 },
    { 0x2061a5f5, 46 },
    { 0x98b1a5f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1a5fc, 33 },
    { 0x7c11a5fd, 95 },
    { 0x98d1a5fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b71a601, 33 },
    { 0x45f1a602, 85 },
    { 0xe3d1a602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71a609, 76 },
    { 0xacb1a609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1401a60e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb1a614, 90 },
    { 0x1511a614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51a61a, 61 },
    { 0, 0 },
    { 0x9f81a61c, 55 },
    { 0, 0 },
    { 0x74f1a61e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71a622, 38 },
    { 0xd21a623, 55 },
    { 0xdb31a624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a1a627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6311a62b, 95 },
    { 0xb8f1a62c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba1a630, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda1a637, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21a63c, 98 },
    { 0, 0 },
    { 0x2501a63e, 46 },
    { 0xa3b1a63f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71a645, 90 },
    { 0xb5b1a646, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f1a64b, 90 },
    { 0x1431a64b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf61a651, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3291a655, 33 },
    { 0xd1a656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd21a65d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d1a661, 55 },
    { 0, 0 },
    { 0x3cb1a663, 23 },
    { 0xab61a664, 90 },
    { 0x7231a665, 95 },
    { 0, 0 },
    { 0xab91a667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5511a66b, 51 },
    { 0x8181a66c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1951a66f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e1a674, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa821a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341a67b, 33 },
    { 0, 0 },
    { 0x35f1a67d, 95 },
    { 0, 0 },
    { 0x1be1a67f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691a682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf1a685, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe461a68a, 90 },
    { 0, 0 },
    { 0x5b21a68c, 33 },
    { 0x3461a68c, 76 },
    { 0x7881a68c, 90 },
    { 0xbf51a68e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c1a692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b1a695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c1a69c, 23 },
    { 0x6731a69d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc1a6a2, 90 },
    { 0, 0 },
    { 0xad81a6a4, 93 },
    { 0xf7d1a6a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1a6a9, 55 },
    { 0xcbf1a6a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a6af, 90 },
    { 0, 0 },
    { 0xb351a6b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd531a6b4, 98 },
    { 0, 0 },
    { 0x3651a6b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1a6ba, 90 },
    { 0x5771a6ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561a6bf, 55 },
    { 0, 0 },
    { 0x30d1a6c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a91a6c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b1a6d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x391a6d6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51a6d9, 75 },
    { 0, 0 },
    { 0x64e1a6db, 85 },
    { 0, 0 },
    { 0xb611a6dd, 84 },
    { 0x95d1a6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71a6e6, 75 },
    { 0x6561a6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde91a6ef, 61 },
    { 0xc971a6f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca41a6f6, 46 },
    { 0xe4f1a6f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1a6fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb781a703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1a708, 3 },
    { 0x3541a709, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb41a70c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa821a70f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471a712, 61 },
    { 0xac81a713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381a718, 99 },
    { 0xe101a719, 52 },
    { 0x1c81a71a, 92 },
    { 0x4791a71b, 98 },
    { 0x4ae1a71a, 87 },
    { 0xb2f1a71d, 99 },
    { 0x13d1a71e, 95 },
    { 0x5181a71f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a723, 87 },
    { 0x7c41a724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1a727, 17 },
    { 0, 0 },
    { 0x1751a729, 85 },
    { 0xfa81a72a, 81 },
    { 0, 0 },
    { 0xb501a72c, 33 },
    { 0xf2f1a72c, 98 },
    { 0xf261a72d, 61 },
    { 0x1b21a72f, 61 },
    { 0xedb1a72d, 95 },
    { 0x46f1a72e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb871a739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d61a73e, 76 },
    { 0x4351a73f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c51a743, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac41a746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff1a749, 61 },
    { 0x6661a749, 61 },
    { 0x66f1a74b, 85 },
    { 0xd4a1a74c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f1a750, 23 },
    { 0, 0 },
    { 0xb671a752, 33 },
    { 0, 0 },
    { 0xe071a754, 98 },
    { 0x29e1a755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf731a758, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b1a75b, 88 },
    { 0xccc1a75b, 98 },
    { 0x28e1a75d, 46 },
    { 0x9871a75d, 34 },
    { 0x44b1a75e, 100 },
    { 0, 0 },
    { 0xdd1a761, 46 },
    { 0xe8e1a762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1701a766, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841a76b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1a771, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9371a776, 100 },
    { 0xd571a776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a1a77d, 98 },
    { 0x1481a77e, 52 },
    { 0x2b71a77e, 43 },
    { 0, 0 },
    { 0x9111a781, 23 },
    { 0xa831a782, 90 },
    { 0x3ff1a783, 100 },
    { 0, 0 },
    { 0xc991a785, 100 },
    { 0x78c1a786, 38 },
    { 0xcb21a785, 95 },
    { 0x53a1a788, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2581a78c, 23 },
    { 0xfb81a78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71a790, 34 },
    { 0, 0 },
    { 0x3b81a792, 98 },
    { 0xc381a792, 33 },
    { 0xbc1a794, 23 },
    { 0x76f1a795, 78 },
    { 0x5591a795, 33 },
    { 0x1491a794, 95 },
    { 0xa7e1a798, 95 },
    { 0xa3d1a795, 61 },
    { 0x1251a79a, 33 },
    { 0x4ba1a79a, 90 },
    { 0x7e21a79a, 61 },
    { 0x6751a79c, 93 },
    { 0x3d81a79e, 33 },
    { 0x4b61a79f, 77 },
    { 0xef61a79c, 34 },
    { 0x4071a79b, 95 },
    { 0xa151a7a2, 95 },
    { 0xa931a7a0, 33 },
    { 0x83f1a79c, 76 },
    { 0, 0 },
    { 0x2881a7a6, 76 },
    { 0xe211a7a7, 33 },
    { 0x5cc1a7a8, 76 },
    { 0x5d41a7a9, 33 },
    { 0x6671a7aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11a7ad, 98 },
    { 0xea1a7ad, 33 },
    { 0xf81a7af, 75 },
    { 0x2501a7b0, 55 },
    { 0, 0 },
    { 0x55a1a7b2, 90 },
    { 0, 0 },
    { 0xb431a7b4, 52 },
    { 0, 0 },
    { 0x1d61a7b6, 87 },
    { 0x8ce1a7b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1a7bc, 98 },
    { 0, 0 },
    { 0xb261a7be, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691a7c3, 55 },
    { 0, 0 },
    { 0x79d1a7c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81a7c9, 56 },
    { 0xa621a7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1a7d4, 43 },
    { 0x7261a7d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a7db, 33 },
    { 0x7151a7dc, 78 },
    { 0x1a81a7dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9711a7e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e1a7e5, 95 },
    { 0x811a7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41a7e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c1a7f1, 75 },
    { 0x9151a7f2, 76 },
    { 0x59d1a7f2, 33 },
    { 0x4f1a7f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe41a7ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e21a803, 33 },
    { 0x6e31a804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd691a807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c1a80a, 98 },
    { 0, 0 },
    { 0x7871a80c, 19 },
    { 0x4dc1a80d, 55 },
    { 0x6051a80c, 52 },
    { 0xc3a1a80d, 43 },
    { 0x59a1a810, 90 },
    { 0x4461a811, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11a816, 93 },
    { 0x5ec1a816, 87 },
    { 0xd241a818, 17 },
    { 0x8181a817, 65 },
    { 0x7571a81a, 33 },
    { 0xe51a81b, 23 },
    { 0x6df1a81c, 33 },
    { 0, 0 },
    { 0xb941a81e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c1a821, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481a82c, 61 },
    { 0x7171a82d, 98 },
    { 0x7f51a82c, 90 },
    { 0x31c1a82e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7801a832, 61 },
    { 0xac81a832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81a836, 98 },
    { 0x13e1a837, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3921a83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc691a843, 61 },
    { 0x6d41a843, 55 },
    { 0x2be1a844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1a84a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631a852, 36 },
    { 0xa0e1a853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf451a858, 95 },
    { 0x8321a858, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a61a85e, 90 },
    { 0x2421a85f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb81a86c, 90 },
    { 0x1441a86c, 55 },
    { 0x9b21a86e, 93 },
    { 0x6d11a86f, 46 },
    { 0x7981a86f, 23 },
    { 0x71a1a870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5481a878, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb221a87b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9851a880, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a1a884, 53 },
    { 0, 0 },
    { 0xe11a886, 95 },
    { 0, 0 },
    { 0x371a888, 87 },
    { 0xdea1a888, 65 },
    { 0xa361a88a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f71a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1a895, 56 },
    { 0x3911a896, 91 },
    { 0, 0 },
    { 0x6841a898, 78 },
    { 0x5cb1a898, 66 },
    { 0, 0 },
    { 0x8b71a89b, 90 },
    { 0x3ec1a89c, 33 },
    { 0x4271a89d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61a8a2, 34 },
    { 0xd551a8a2, 67 },
    { 0x30c1a8a4, 33 },
    { 0x7dc1a8a5, 23 },
    { 0x1d91a8a6, 96 },
    { 0x7511a8a4, 33 },
    { 0x5c01a8a2, 99 },
    { 0xdce1a8a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431a8b0, 23 },
    { 0xc621a8b0, 76 },
    { 0, 0 },
    { 0x7331a8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c1a8b8, 23 },
    { 0x8ea1a8b9, 23 },
    { 0xdb31a8b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1a8bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1a8c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1a8c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891a8ca, 98 },
    { 0xfb1a8ca, 34 },
    { 0x47e1a8cc, 23 },
    { 0, 0 },
    { 0xbae1a8ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab31a8d1, 99 },
    { 0, 0 },
    { 0xff11a8d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa041a8db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3501a8e1, 53 },
    { 0x1f01a8e2, 33 },
    { 0, 0 },
    { 0x6cc1a8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3351a8e7, 98 },
    { 0, 0 },
    { 0xae01a8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f41a8ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d91a8f1, 46 },
    { 0xe141a8f2, 61 },
    { 0xfb81a8f2, 76 },
    { 0x99c1a8f4, 87 },
    { 0x3321a8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9271a8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d1a8fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31a903, 87 },
    { 0x4351a903, 90 },
    { 0xdd31a905, 34 },
    { 0, 0 },
    { 0x6d21a907, 64 },
    { 0x8451a908, 33 },
    { 0x6a81a909, 90 },
    { 0x24b1a909, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b71a910, 95 },
    { 0xe061a911, 23 },
    { 0x1e71a911, 98 },
    { 0x25f1a913, 33 },
    { 0, 0 },
    { 0xe251a915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x701a918, 90 },
    { 0x6ab1a918, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251a91c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea1a91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xede1a922, 33 },
    { 0xc0c1a923, 58 },
    { 0xa351a924, 85 },
    { 0x8091a925, 90 },
    { 0xc851a925, 98 },
    { 0xd811a927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4261a92e, 95 },
    { 0xae11a92f, 98 },
    { 0, 0 },
    { 0x3761a931, 33 },
    { 0xa241a932, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8681a937, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521a942, 53 },
    { 0x6831a942, 23 },
    { 0xdc51a944, 84 },
    { 0x8dc1a944, 95 },
    { 0xa921a946, 90 },
    { 0, 0 },
    { 0x7e1a948, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9651a94b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe291a954, 33 },
    { 0xb231a954, 76 },
    { 0, 0 },
    { 0xbeb1a957, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e1a95b, 98 },
    { 0xe61a95c, 33 },
    { 0x2f71a95b, 76 },
    { 0x73a1a95d, 90 },
    { 0xf01a95d, 17 },
    { 0x8321a960, 76 },
    { 0xee81a960, 76 },
    { 0x9ad1a961, 90 },
    { 0x1b51a963, 23 },
    { 0xeb71a964, 91 },
    { 0x71d1a965, 61 },
    { 0x8be1a966, 98 },
    { 0, 0 },
    { 0x3041a968, 90 },
    { 0x9261a968, 61 },
    { 0, 0 },
    { 0x64d1a96b, 33 },
    { 0xb351a96c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11a970, 91 },
    { 0, 0 },
    { 0xf2d1a972, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c81a977, 98 },
    { 0xd1d1a978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe961a97e, 33 },
    { 0xddf1a97e, 91 },
    { 0xf281a97e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf71a986, 23 },
    { 0x5681a986, 95 },
    { 0xf961a988, 90 },
    { 0x7a21a986, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1a98e, 98 },
    { 0x98e1a98f, 61 },
    { 0x2c71a990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc11a993, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031a996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a81a999, 43 },
    { 0xcbd1a99a, 93 },
    { 0x1e1a999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1a9a2, 67 },
    { 0x3f61a9a2, 23 },
    { 0x1cc1a9a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1a9a7, 90 },
    { 0xfad1a9a8, 98 },
    { 0xcb11a9a8, 95 },
    { 0x4751a9a9, 91 },
    { 0x9a41a9ab, 93 },
    { 0xf61a9ab, 98 },
    { 0x5ac1a9ad, 66 },
    { 0x541a9ae, 34 },
    { 0, 0 },
    { 0xafc1a9b0, 34 },
    { 0x4561a9b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c1a9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf31a9bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1411a9c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2141a9c3, 90 },
    { 0x8a21a9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe851a9ca, 90 },
    { 0xafc1a9cb, 91 },
    { 0, 0 },
    { 0xb971a9cd, 90 },
    { 0xa4d1a9ce, 33 },
    { 0xafd1a9cf, 93 },
    { 0, 0 },
    { 0x8b1a9d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8741a9d9, 46 },
    { 0xae81a9da, 46 },
    { 0x4c91a9d9, 83 },
    { 0x3731a9dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a1a9df, 95 },
    { 0, 0 },
    { 0xe321a9e1, 76 },
    { 0xb781a9e2, 90 },
    { 0xed1a9e1, 76 },
    { 0x2061a9e4, 33 },
    { 0xd0a1a9e5, 99 },
    { 0x4d51a9e2, 84 },
    { 0, 0 },
    { 0xfd21a9e8, 23 },
    { 0x6c51a9e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141a9ec, 93 },
    { 0xa6b1a9ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1a9f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a9ff, 43 },
    { 0, 0 },
    { 0x7691aa01, 78 },
    { 0xd091aa01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1aa05, 43 },
    { 0xb581aa05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111aa0a, 33 },
    { 0x7551aa0b, 90 },
    { 0xa871aa0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df1aa10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3761aa15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b91aa1a, 95 },
    { 0xba1aa1b, 65 },
    { 0x8ba1aa1c, 76 },
    { 0x9cc1aa1b, 95 },
    { 0, 0 },
    { 0xbce1aa1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd1aa23, 99 },
    { 0xb131aa24, 78 },
    { 0, 0 },
    { 0x9b11aa26, 61 },
    { 0x6901aa26, 55 },
    { 0xcf11aa27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a1aa2b, 90 },
    { 0xcb31aa2c, 61 },
    { 0x8d71aa2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1aa30, 33 },
    { 0, 0 },
    { 0x7ac1aa32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41aa3a, 76 },
    { 0x26d1aa3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1621aa3e, 88 },
    { 0, 0 },
    { 0xe551aa40, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61aa43, 52 },
    { 0x4d31aa44, 34 },
    { 0, 0 },
    { 0x56a1aa46, 61 },
    { 0xa941aa46, 85 },
    { 0x3281aa48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe101aa4b, 33 },
    { 0x1f81aa4c, 78 },
    { 0x8841aa4d, 33 },
    { 0xc711aa4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091aa53, 90 },
    { 0, 0 },
    { 0x98d1aa55, 95 },
    { 0, 0 },
    { 0x16e1aa57, 61 },
    { 0xc9f1aa58, 34 },
    { 0x61b1aa57, 95 },
    { 0x57d1aa5a, 78 },
    { 0x3101aa5a, 55 },
    { 0, 0 },
    { 0x6d41aa5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91aa61, 23 },
    { 0xc771aa62, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6341aa6a, 84 },
    { 0, 0 },
    { 0x4321aa6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d1aa70, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9021aa74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8961aa7c, 33 },
    { 0, 0 },
    { 0x7f31aa7e, 98 },
    { 0x7e1aa7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed31aa83, 68 },
    { 0x2851aa83, 80 },
    { 0xff41aa83, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221aa8a, 97 },
    { 0x2e31aa8b, 76 },
    { 0, 0 },
    { 0x3771aa8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa001aa93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a01aa96, 99 },
    { 0xaca1aa97, 97 },
    { 0xa251aa96, 66 },
    { 0x5381aa99, 33 },
    { 0x6dc1aa9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb51aa9f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b81aaa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191aaa5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1aaab, 5 },
    { 0xcf91aaac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac61aaaf, 88 },
    { 0, 0 },
    { 0xecd1aab1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2281aab6, 95 },
    { 0xdf31aab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4311aabb, 66 },
    { 0, 0 },
    { 0xb121aabd, 78 },
    { 0xb171aabe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e1aac1, 88 },
    { 0x2c51aac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb171aaca, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21aacd, 95 },
    { 0, 0 },
    { 0x2cf1aacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141aad5, 43 },
    { 0x4f21aad6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd51aad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb1aae4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61aaea, 95 },
    { 0, 0 },
    { 0x7801aaec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf931aaf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41aaf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251aafa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b1aafe, 68 },
    { 0xa401aaff, 91 },
    { 0xa221ab00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41ab03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671ab06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b51ab0d, 61 },
    { 0x69f1ab0e, 34 },
    { 0x7421ab0f, 34 },
    { 0xf761ab0e, 93 },
    { 0, 0 },
    { 0x92c1ab12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491ab16, 98 },
    { 0x3e61ab17, 98 },
    { 0, 0 },
    { 0xb301ab19, 98 },
    { 0xcca1ab19, 43 },
    { 0xcf1ab1b, 90 },
    { 0xde71ab1b, 77 },
    { 0xf2c1ab1c, 33 },
    { 0, 0 },
    { 0x5d11ab1f, 99 },
    { 0, 0 },
    { 0xd6a1ab21, 55 },
    { 0, 0 },
    { 0x9811ab23, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1ab26, 76 },
    { 0xa601ab27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b1ab2a, 61 },
    { 0xecd1ab2a, 66 },
    { 0xd3e1ab2a, 76 },
    { 0, 0 },
    { 0x1971ab2e, 52 },
    { 0x77f1ab2e, 94 },
    { 0x7881ab30, 100 },
    { 0xb991ab31, 55 },
    { 0, 0 },
    { 0x9191ab33, 61 },
    { 0, 0 },
    { 0x2031ab35, 78 },
    { 0x3461ab35, 76 },
    { 0xbc41ab36, 67 },
    { 0x60c1ab36, 98 },
    { 0x6d91ab39, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc1ab3c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ab41, 70 },
    { 0x16d1ab41, 95 },
    { 0, 0 },
    { 0xb381ab44, 95 },
    { 0x3a51ab45, 95 },
    { 0xd951ab46, 33 },
    { 0x3bc1ab47, 95 },
    { 0x7cc1ab48, 43 },
    { 0xbd1ab44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3811ab4c, 34 },
    { 0, 0 },
    { 0x3391ab4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac31ab52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa931ab55, 23 },
    { 0, 0 },
    { 0x2051ab57, 61 },
    { 0x211ab58, 61 },
    { 0x9461ab57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef1ab5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8701ab60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f1ab63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda1ab68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c61ab6d, 98 },
    { 0xd211ab6e, 11 },
    { 0x6321ab6f, 90 },
    { 0x14b1ab6f, 66 },
    { 0x75a1ab6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831ab74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5021ab77, 90 },
    { 0xa491ab78, 95 },
    { 0xf8e1ab79, 95 },
    { 0xf7f1ab77, 76 },
    { 0x3831ab77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7221ab84, 33 },
    { 0xe341ab84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c1ab8c, 90 },
    { 0xab41ab8d, 98 },
    { 0xea81ab8e, 36 },
    { 0xa21ab8c, 91 },
    { 0x9671ab8d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541ab95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321ab9b, 61 },
    { 0xcad1ab9c, 94 },
    { 0, 0 },
    { 0x3841ab9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9881aba4, 43 },
    { 0x12a1aba4, 52 },
    { 0x5b31aba6, 88 },
    { 0x5241aba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1abad, 52 },
    { 0x6b51abae, 85 },
    { 0x22a1abaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9351abb2, 55 },
    { 0, 0 },
    { 0xec61abb4, 34 },
    { 0xeb81abb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6001abbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af1abc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1abca, 95 },
    { 0x2f81abcb, 90 },
    { 0xf641abcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8971abd1, 93 },
    { 0, 0 },
    { 0xe001abd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961abd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1abdf, 90 },
    { 0, 0 },
    { 0x9a31abe1, 98 },
    { 0x6e31abe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e51abe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8771abea, 93 },
    { 0x5761abea, 23 },
    { 0xb641abeb, 34 },
    { 0, 0 },
    { 0xf3b1abee, 55 },
    { 0xa951abef, 23 },
    { 0x8161abef, 90 },
    { 0x1b21abef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921abf6, 23 },
    { 0x7c21abf6, 66 },
    { 0xa11abf8, 90 },
    { 0x8131abf8, 33 },
    { 0, 0 },
    { 0xf8c1abfb, 100 },
    { 0xa3b1abfc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc1abff, 61 },
    { 0, 0 },
    { 0x3191ac01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa271ac0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7551ac14, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71ac17, 90 },
    { 0xf971ac17, 61 },
    { 0, 0 },
    { 0xa091ac1a, 90 },
    { 0x8761ac1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x901ac1e, 99 },
    { 0xba41ac1f, 97 },
    { 0xe641ac20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1ac26, 88 },
    { 0x9011ac26, 87 },
    { 0xb031ac27, 78 },
    { 0xf721ac26, 98 },
    { 0x7591ac2a, 33 },
    { 0x35a1ac26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b31ac2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad1ac36, 33 },
    { 0x3571ac37, 98 },
    { 0, 0 },
    { 0x67a1ac39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81ac3c, 61 },
    { 0, 0 },
    { 0x4331ac3e, 90 },
    { 0xcd31ac3f, 76 },
    { 0xba61ac3f, 98 },
    { 0x1301ac41, 46 },
    { 0, 0 },
    { 0x4171ac43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c31ac46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a71ac4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc1ac50, 34 },
    { 0, 0 },
    { 0x2a01ac52, 95 },
    { 0x2821ac53, 61 },
    { 0xb7b1ac54, 78 },
    { 0x3141ac55, 33 },
    { 0xda41ac56, 90 },
    { 0, 0 },
    { 0xdd31ac58, 90 },
    { 0, 0 },
    { 0xa581ac5a, 95 },
    { 0x7311ac5a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941ac5f, 23 },
    { 0x59c1ac5f, 30 },
    { 0x66b1ac60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e51ac65, 46 },
    { 0, 0 },
    { 0xe091ac67, 95 },
    { 0x3451ac67, 75 },
    { 0x5821ac69, 94 },
    { 0x9711ac6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe321ac76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f1ac7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c1ac7e, 40 },
    { 0xd541ac7e, 78 },
    { 0x90b1ac7f, 90 },
    { 0, 0 },
    { 0x8cb1ac82, 90 },
    { 0xcc81ac83, 33 },
    { 0, 0 },
    { 0x8d51ac85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91ac89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d41ac8c, 55 },
    { 0, 0 },
    { 0xf191ac8e, 78 },
    { 0x75e1ac8f, 91 },
    { 0, 0 },
    { 0xb6f1ac91, 98 },
    { 0xc5d1ac92, 76 },
    { 0xd751ac92, 95 },
    { 0xcd31ac92, 98 },
    { 0, 0 },
    { 0x1641ac96, 59 },
    { 0xfec1ac97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4151ac9a, 78 },
    { 0, 0 },
    { 0x8471ac9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec1aca2, 91 },
    { 0, 0 },
    { 0xe8a1aca4, 55 },
    { 0x5941aca5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf411aca9, 95 },
    { 0, 0 },
    { 0x5f91acab, 33 },
    { 0xc601acac, 33 },
    { 0x16b1acac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc021acb0, 61 },
    { 0xd911acb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121acb6, 23 },
    { 0x9da1acb6, 83 },
    { 0xe121acb8, 61 },
    { 0, 0 },
    { 0x2b1acba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701acbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1acc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba1acc4, 65 },
    { 0, 0 },
    { 0x38a1acc6, 67 },
    { 0xc3c1acc7, 46 },
    { 0xf101acc8, 61 },
    { 0x1a01acc7, 91 },
    { 0xffc1acc9, 61 },
    { 0x191acc6, 90 },
    { 0xe21acc8, 95 },
    { 0, 0 },
    { 0x15e1acce, 40 },
    { 0x9f91accf, 98 },
    { 0xf3d1accf, 95 },
    { 0x401accf, 33 },
    { 0, 0 },
    { 0x6f71acd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11acd6, 61 },
    { 0xc501acd7, 23 },
    { 0x9e61acd8, 100 },
    { 0x941acd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a71acdd, 90 },
    { 0, 0 },
    { 0xac91acdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb741ace8, 52 },
    { 0, 0 },
    { 0x9e11acea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641aced, 98 },
    { 0x3dd1acee, 98 },
    { 0, 0 },
    { 0xa5f1acf0, 75 },
    { 0xa871acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d1acf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b1acfb, 78 },
    { 0xef91acfb, 90 },
    { 0x3b91acfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc1ad06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d1ad0f, 90 },
    { 0xa1f1ad0f, 65 },
    { 0x1ad0f, 43 },
    { 0x8091ad12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991ad16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781ad1e, 95 },
    { 0xca21ad1e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e1ad25, 90 },
    { 0x4901ad25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4631ad2f, 90 },
    { 0x6121ad2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb841ad33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be1ad36, 61 },
    { 0, 0 },
    { 0x3101ad38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8601ad3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb101ad47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3161ad4b, 43 },
    { 0, 0 },
    { 0xc521ad4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f1ad50, 61 },
    { 0, 0 },
    { 0x58a1ad52, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c1ad55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a1ad5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21ad61, 91 },
    { 0x96e1ad62, 55 },
    { 0x8061ad63, 90 },
    { 0x3d81ad64, 90 },
    { 0x91d1ad65, 90 },
    { 0xcb91ad66, 61 },
    { 0x3cc1ad65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7841ad6c, 95 },
    { 0xdf1ad6d, 55 },
    { 0, 0 },
    { 0x26b1ad6f, 61 },
    { 0, 0 },
    { 0xa431ad71, 98 },
    { 0x9571ad71, 95 },
    { 0xd5a1ad73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa1ad76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe901ad7a, 23 },
    { 0x7b91ad7b, 23 },
    { 0xa3a1ad7c, 100 },
    { 0x311ad7a, 52 },
    { 0x7871ad7e, 98 },
    { 0x2e31ad7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa441ad84, 76 },
    { 0x2361ad84, 43 },
    { 0x81c1ad84, 78 },
    { 0xe6d1ad86, 76 },
    { 0xd421ad87, 61 },
    { 0xe71ad89, 52 },
    { 0x2c61ad89, 90 },
    { 0x7851ad85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4721ad93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb431ad99, 98 },
    { 0, 0 },
    { 0x93a1ad9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb1ad9e, 23 },
    { 0xc1e1ad9f, 98 },
    { 0xc6e1ada0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ada3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a1ada7, 23 },
    { 0, 0 },
    { 0x8e41ada9, 34 },
    { 0x761adaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d1adb2, 95 },
    { 0, 0 },
    { 0x2181adb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1adb8, 90 },
    { 0, 0 },
    { 0x88d1adba, 76 },
    { 0x1b61adba, 33 },
    { 0, 0 },
    { 0x5f21adbd, 90 },
    { 0x9911adbd, 90 },
    { 0, 0 },
    { 0xa4e1adc0, 95 },
    { 0, 0 },
    { 0x2481adc2, 84 },
    { 0xea51adc2, 33 },
    { 0x8371adc4, 90 },
    { 0x4cb1adc5, 99 },
    { 0x4ad1adc6, 70 },
    { 0x7751adc5, 95 },
    { 0xd6a1adc8, 80 },
    { 0, 0 },
    { 0x8351adca, 92 },
    { 0xac01adcb, 98 },
    { 0, 0 },
    { 0xf3b1adcd, 91 },
    { 0x5f21adcd, 95 },
    { 0x5ad1adcf, 90 },
    { 0xbc51adcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2291add3, 90 },
    { 0, 0 },
    { 0xfba1add5, 99 },
    { 0x78d1add6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2071addf, 95 },
    { 0x89d1ade0, 33 },
    { 0x48f1addf, 95 },
    { 0xec1ade2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1ade6, 61 },
    { 0, 0 },
    { 0x8af1ade8, 43 },
    { 0x37d1ade9, 90 },
    { 0x8ea1adea, 76 },
    { 0x50b1ade9, 93 },
    { 0xe11adec, 23 },
    { 0x701aded, 33 },
    { 0xb261adeb, 90 },
    { 0xd4a1aded, 84 },
    { 0x78b1adec, 61 },
    { 0xeb71adf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091adf7, 68 },
    { 0x3eb1adf7, 43 },
    { 0x75f1adf8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c91adfc, 85 },
    { 0, 0 },
    { 0xf1f1adfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9201ae01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c81ae08, 23 },
    { 0x651ae09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1ae0d, 75 },
    { 0x10c1ae0e, 91 },
    { 0, 0 },
    { 0x6111ae10, 87 },
    { 0, 0 },
    { 0xeeb1ae12, 95 },
    { 0x3f01ae13, 93 },
    { 0xfe31ae14, 85 },
    { 0xabd1ae13, 100 },
    { 0, 0 },
    { 0x2391ae17, 2 },
    { 0x7ad1ae18, 46 },
    { 0xc291ae19, 91 },
    { 0x6a61ae18, 100 },
    { 0x90a1ae1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7051ae1f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121ae22, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f1ae27, 17 },
    { 0x76e1ae28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b1ae2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1ae30, 46 },
    { 0xd601ae31, 34 },
    { 0, 0 },
    { 0x56c1ae33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c1ae36, 98 },
    { 0, 0 },
    { 0x9801ae38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8341ae3c, 33 },
    { 0, 0 },
    { 0x95a1ae3e, 78 },
    { 0xcd61ae3f, 33 },
    { 0xe1f1ae40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621ae43, 98 },
    { 0, 0 },
    { 0x1dc1ae45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a1ae4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd641ae53, 93 },
    { 0, 0 },
    { 0xf4c1ae55, 90 },
    { 0, 0 },
    { 0x2a91ae57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xab51ae5a, 91 },
    { 0xfdb1ae5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1ae62, 33 },
    { 0xf121ae63, 90 },
    { 0xbfe1ae64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5261ae69, 23 },
    { 0xd7b1ae69, 61 },
    { 0xff61ae6a, 90 },
    { 0x38d1ae6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1ae79, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c1ae7c, 33 },
    { 0x9601ae7c, 87 },
    { 0, 0 },
    { 0x6731ae7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb1ae85, 76 },
    { 0x21e1ae85, 76 },
    { 0xac31ae86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7291ae97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d1ae9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1ae9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa1aea1, 98 },
    { 0, 0 },
    { 0xc2f1aea3, 91 },
    { 0xd6c1aea4, 49 },
    { 0x1f41aea4, 95 },
    { 0xb581aea6, 90 },
    { 0x1321aea4, 67 },
    { 0x4e51aea5, 23 },
    { 0x9071aea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4011aeb0, 99 },
    { 0xf9e1aeb1, 95 },
    { 0, 0 },
    { 0x87e1aeb3, 90 },
    { 0x4a41aeb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d31aeb8, 95 },
    { 0, 0 },
    { 0xa9e1aeba, 46 },
    { 0x2991aebb, 99 },
    { 0, 0 },
    { 0xaa41aebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8271aec3, 91 },
    { 0xd701aec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa11aec7, 90 },
    { 0x9321aec8, 90 },
    { 0xb351aec9, 95 },
    { 0xb191aec9, 78 },
    { 0x2521aec9, 95 },
    { 0x4bb1aecc, 33 },
    { 0x8371aecc, 95 },
    { 0xc391aecc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c71aed1, 61 },
    { 0x44e1aed2, 95 },
    { 0xca01aed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c21aede, 66 },
    { 0, 0 },
    { 0x8e61aee0, 34 },
    { 0xc391aee1, 84 },
    { 0x5d51aee2, 87 },
    { 0x3061aee1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c71aeee, 61 },
    { 0x4a11aeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5871aef3, 95 },
    { 0x221aef4, 65 },
    { 0x4a51aef5, 33 },
    { 0xc361aef6, 90 },
    { 0x25f1aef6, 61 },
    { 0x29f1aef7, 95 },
    { 0xd4c1aef8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf541aefd, 90 },
    { 0xb8c1aefd, 90 },
    { 0x2391aefd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf81af02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1af05, 90 },
    { 0xbd51af05, 43 },
    { 0xc761af07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1af0a, 98 },
    { 0x1341af0b, 90 },
    { 0xc381af0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151af0f, 33 },
    { 0, 0 },
    { 0x55c1af11, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1af14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051af1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b1af26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3881af29, 33 },
    { 0, 0 },
    { 0xd7b1af2b, 61 },
    { 0, 0 },
    { 0x3791af2d, 84 },
    { 0xef41af2d, 33 },
    { 0, 0 },
    { 0x7ae1af30, 43 },
    { 0x4fe1af31, 55 },
    { 0xdfe1af30, 34 },
    { 0, 0 },
    { 0x48a1af34, 61 },
    { 0x94c1af35, 98 },
    { 0x80d1af36, 23 },
    { 0, 0 },
    { 0x1e61af38, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01af40, 93 },
    { 0x83b1af41, 93 },
    { 0x7f71af41, 23 },
    { 0x1b01af40, 65 },
    { 0xfe91af44, 33 },
    { 0x7721af44, 98 },
    { 0, 0 },
    { 0xf021af47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81af52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b1af57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1af5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741af5d, 86 },
    { 0xe721af5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1af65, 91 },
    { 0, 0 },
    { 0x8901af67, 33 },
    { 0x9bc1af68, 55 },
    { 0xb241af68, 46 },
    { 0, 0 },
    { 0x77f1af6b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad1af6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc241af73, 87 },
    { 0xd6e1af74, 90 },
    { 0xf441af74, 76 },
    { 0xfee1af76, 76 },
    { 0, 0 },
    { 0xfbd1af78, 33 },
    { 0x1e31af79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4311af7d, 83 },
    { 0x17b1af7d, 99 },
    { 0x1b71af7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981af86, 95 },
    { 0x5f81af87, 55 },
    { 0xf371af86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c1af8e, 98 },
    { 0x60c1af8e, 95 },
    { 0x1e1af90, 43 },
    { 0x4bc1af90, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d1af94, 76 },
    { 0, 0 },
    { 0x5b51af96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631af9b, 69 },
    { 0xb1d1af9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3991afa0, 23 },
    { 0xf9a1afa1, 95 },
    { 0xa801afa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2951afa5, 46 },
    { 0, 0 },
    { 0x5481afa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f91afab, 90 },
    { 0x2111afab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d1afb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db1afb7, 43 },
    { 0, 0 },
    { 0x23d1afb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e61afbd, 67 },
    { 0x3351afbe, 46 },
    { 0, 0 },
    { 0x50d1afc0, 90 },
    { 0, 0 },
    { 0x4c01afc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091afc6, 34 },
    { 0x5241afc7, 33 },
    { 0xe741afc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e1afd0, 98 },
    { 0, 0 },
    { 0x48c1afd2, 90 },
    { 0x1141afd3, 34 },
    { 0, 0 },
    { 0xd3f1afd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001afdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b1afdf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a91afe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc651aff3, 84 },
    { 0, 0 },
    { 0x3551aff5, 90 },
    { 0xc821aff6, 95 },
    { 0x2541aff7, 91 },
    { 0x5241aff8, 98 },
    { 0, 0 },
    { 0x1e51affa, 87 },
    { 0x77f1affa, 61 },
    { 0x9401affc, 61 },
    { 0, 0 },
    { 0xa641affe, 98 },
    { 0xbea1affe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c11b007, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b011, 95 },
    { 0, 0 },
    { 0xd851b013, 23 },
    { 0, 0 },
    { 0xa921b015, 33 },
    { 0, 0 },
    { 0xfc21b017, 90 },
    { 0x9b41b017, 39 },
    { 0, 0 },
    { 0x4a31b01a, 33 },
    { 0x60a1b01b, 91 },
    { 0x4dd1b01a, 33 },
    { 0, 0 },
    { 0xf951b01e, 34 },
    { 0x97e1b01e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c1b024, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc141b02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c31b036, 75 },
    { 0x9711b037, 33 },
    { 0xf811b038, 90 },
    { 0x2821b037, 90 },
    { 0xbd01b03a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d1b041, 87 },
    { 0x9d71b042, 23 },
    { 0x98a1b043, 98 },
    { 0xd901b044, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b1b048, 75 },
    { 0x9ea1b048, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d91b051, 33 },
    { 0, 0 },
    { 0x2291b053, 98 },
    { 0x63e1b054, 43 },
    { 0x8f51b055, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5861b05a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc61b05f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc181b062, 66 },
    { 0xe6d1b063, 90 },
    { 0xa841b064, 61 },
    { 0x5771b065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1b068, 34 },
    { 0, 0 },
    { 0x9031b06a, 61 },
    { 0x46c1b06b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b1b071, 61 },
    { 0, 0 },
    { 0x30d1b073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e1b076, 91 },
    { 0, 0 },
    { 0x6081b078, 33 },
    { 0x5631b079, 98 },
    { 0, 0 },
    { 0xb8a1b07b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb11b081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c61b084, 76 },
    { 0, 0 },
    { 0x83f1b086, 91 },
    { 0xeaa1b087, 33 },
    { 0xde71b088, 95 },
    { 0x2dc1b089, 55 },
    { 0x6351b089, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb241b08d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf721b09e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b0a2, 23 },
    { 0x2441b0a3, 78 },
    { 0, 0 },
    { 0x1c71b0a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1731b0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9441b0ab, 23 },
    { 0x11e1b0ac, 33 },
    { 0xc3c1b0ad, 40 },
    { 0x3c31b0ae, 43 },
    { 0x3371b0af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb41b0b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1b0b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21b0b8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841b0bc, 67 },
    { 0x9441b0bc, 93 },
    { 0x3e71b0bd, 46 },
    { 0x2c71b0bf, 95 },
    { 0x3c51b0be, 43 },
    { 0x67d1b0c1, 99 },
    { 0xb8b1b0c2, 95 },
    { 0xe1a1b0c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc01b0c6, 23 },
    { 0x5c41b0c7, 46 },
    { 0x59c1b0c8, 61 },
    { 0, 0 },
    { 0x9fe1b0ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d1b0ce, 43 },
    { 0xf81b0cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f1b0d2, 33 },
    { 0x3131b0d2, 61 },
    { 0x1051b0d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1b0d8, 90 },
    { 0, 0 },
    { 0x9b1b0da, 90 },
    { 0xa991b0db, 82 },
    { 0xb451b0dc, 23 },
    { 0xecd1b0dc, 95 },
    { 0xce71b0de, 96 },
    { 0x4f11b0de, 81 },
    { 0x58e1b0e0, 75 },
    { 0, 0 },
    { 0xee1b0e2, 33 },
    { 0x8f31b0e3, 99 },
    { 0x5861b0e4, 46 },
    { 0x6821b0e5, 87 },
    { 0x8c31b0e4, 55 },
    { 0, 0 },
    { 0x6751b0e8, 33 },
    { 0x88d1b0e8, 23 },
    { 0, 0 },
    { 0x2ae1b0eb, 76 },
    { 0, 0 },
    { 0x10c1b0ed, 85 },
    { 0x4651b0ed, 76 },
    { 0xa141b0ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1b0f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b0f6, 23 },
    { 0, 0 },
    { 0x9f61b0f8, 91 },
    { 0, 0 },
    { 0x2361b0fa, 78 },
    { 0x1121b0fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451b0ff, 51 },
    { 0x6ce1b0ff, 23 },
    { 0, 0 },
    { 0x5a31b102, 47 },
    { 0, 0 },
    { 0xdf11b104, 40 },
    { 0xed91b105, 67 },
    { 0x5af1b106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e1b109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a91b10d, 23 },
    { 0, 0 },
    { 0xf721b10f, 90 },
    { 0, 0 },
    { 0xedb1b111, 52 },
    { 0xd851b112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61b117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8391b11a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8521b11e, 95 },
    { 0xd0f1b11f, 76 },
    { 0xac31b11f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3521b124, 33 },
    { 0x3631b124, 66 },
    { 0xf341b126, 61 },
    { 0xd051b124, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991b12b, 66 },
    { 0x24b1b12c, 90 },
    { 0, 0 },
    { 0x6de1b12e, 55 },
    { 0xec61b12f, 61 },
    { 0, 0 },
    { 0xdb21b131, 66 },
    { 0x20a1b132, 76 },
    { 0, 0 },
    { 0xe3c1b134, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb31b137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4751b13d, 95 },
    { 0x5871b13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221b143, 52 },
    { 0, 0 },
    { 0x2481b145, 91 },
    { 0x5d61b146, 91 },
    { 0x9c01b145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7411b14a, 90 },
    { 0x58d1b14b, 93 },
    { 0, 0 },
    { 0xc901b14d, 61 },
    { 0x3a51b14d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b151, 46 },
    { 0x1201b151, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd1b155, 87 },
    { 0xdce1b155, 66 },
    { 0x801b155, 55 },
    { 0xd231b157, 48 },
    { 0x5091b159, 98 },
    { 0xb8f1b15a, 87 },
    { 0x58e1b15a, 61 },
    { 0xe8e1b15c, 78 },
    { 0x84a1b156, 76 },
    { 0x8d51b15e, 100 },
    { 0x9d91b15e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8071b164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b1b167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81b175, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251b179, 98 },
    { 0, 0 },
    { 0x5951b17b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521b17f, 23 },
    { 0, 0 },
    { 0xde61b181, 55 },
    { 0, 0 },
    { 0x5e21b183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631b187, 43 },
    { 0, 0 },
    { 0xb321b189, 17 },
    { 0, 0 },
    { 0x5ca1b18b, 95 },
    { 0x1a31b18b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821b190, 33 },
    { 0, 0 },
    { 0x6b81b192, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xade1b195, 61 },
    { 0xfa81b196, 84 },
    { 0x8301b197, 33 },
    { 0x6d41b197, 23 },
    { 0xe291b199, 78 },
    { 0x9be1b199, 66 },
    { 0xdd71b19b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c71b1a1, 93 },
    { 0x65a1b1a1, 33 },
    { 0x7561b1a3, 90 },
    { 0, 0 },
    { 0x3981b1a5, 46 },
    { 0xcb11b1a5, 87 },
    { 0xaca1b1a7, 52 },
    { 0x93a1b1a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7901b1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b1ae, 33 },
    { 0x9251b1ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4251b1b5, 93 },
    { 0xaa41b1b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f01b1b9, 56 },
    { 0x7b91b1ba, 90 },
    { 0x6591b1b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281b1be, 76 },
    { 0, 0 },
    { 0xea61b1c0, 98 },
    { 0, 0 },
    { 0x4201b1c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1121b1c6, 93 },
    { 0, 0 },
    { 0xec91b1c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7591b1d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c1b1d5, 66 },
    { 0, 0 },
    { 0xe3b1b1d7, 33 },
    { 0, 0 },
    { 0x6611b1d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f1b1dc, 98 },
    { 0x81a1b1dc, 23 },
    { 0x1781b1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a31b1e1, 55 },
    { 0xc0b1b1e1, 34 },
    { 0x6981b1e3, 95 },
    { 0, 0 },
    { 0xd1b1e5, 99 },
    { 0xc71b1e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7071b1eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad1b1f1, 43 },
    { 0xaba1b1f1, 38 },
    { 0, 0 },
    { 0x5e71b1f4, 23 },
    { 0x2391b1f5, 33 },
    { 0xc7e1b1f6, 90 },
    { 0xf641b1f6, 61 },
    { 0xc7e1b1f5, 98 },
    { 0, 0 },
    { 0x1861b1fa, 90 },
    { 0, 0 },
    { 0xa461b1fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6401b200, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e1b20b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd021b20f, 43 },
    { 0xcca1b20f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631b214, 90 },
    { 0xe571b215, 46 },
    { 0xaf61b216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa131b21c, 43 },
    { 0x70d1b21d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb031b225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9871b22b, 33 },
    { 0x8441b22b, 90 },
    { 0x4861b22c, 98 },
    { 0x7fd1b22e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6951b231, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a1b234, 61 },
    { 0, 0 },
    { 0xf5a1b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca01b239, 33 },
    { 0, 0 },
    { 0x5491b23b, 84 },
    { 0x7d61b23b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381b241, 43 },
    { 0x1811b242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa211b245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b24f, 88 },
    { 0x1411b250, 78 },
    { 0x66e1b250, 61 },
    { 0x6f51b251, 43 },
    { 0xfe11b253, 46 },
    { 0x4a21b24f, 90 },
    { 0x14d1b252, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891b258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd681b25b, 34 },
    { 0, 0 },
    { 0x90a1b25d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1b261, 33 },
    { 0x1ea1b262, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071b268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631b26b, 78 },
    { 0xedb1b26c, 95 },
    { 0xed81b26c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11b271, 46 },
    { 0x1ba1b271, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611b275, 23 },
    { 0x5c01b276, 33 },
    { 0xa9d1b276, 80 },
    { 0x3cf1b278, 23 },
    { 0xaba1b276, 98 },
    { 0x8ba1b27a, 66 },
    { 0, 0 },
    { 0xb201b27c, 61 },
    { 0x4271b27d, 43 },
    { 0, 0 },
    { 0xc011b27f, 90 },
    { 0xba81b27f, 66 },
    { 0xd811b281, 98 },
    { 0xb3c1b280, 61 },
    { 0xa5c1b283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d1b28d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb81b290, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91b293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1b296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac1b29a, 23 },
    { 0xaa81b29a, 66 },
    { 0xc401b29c, 90 },
    { 0x4411b29d, 92 },
    { 0x551b29e, 3 },
    { 0xd3e1b29e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261b2a2, 90 },
    { 0xc011b2a2, 90 },
    { 0, 0 },
    { 0x17b1b2a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21b2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf781b2b1, 46 },
    { 0xb4d1b2b2, 61 },
    { 0x8bc1b2b2, 91 },
    { 0xae41b2b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec11b2b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71b2c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd11b2cc, 90 },
    { 0, 0 },
    { 0xdfa1b2ce, 23 },
    { 0x59f1b2cf, 78 },
    { 0, 0 },
    { 0x19d1b2d1, 98 },
    { 0, 0 },
    { 0xad01b2d3, 85 },
    { 0, 0 },
    { 0x5ec1b2d5, 33 },
    { 0xbd1b2d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91b2d9, 95 },
    { 0xfa91b2d9, 90 },
    { 0x6ba1b2db, 98 },
    { 0, 0 },
    { 0x7501b2dd, 80 },
    { 0, 0 },
    { 0xa8b1b2df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb901b2e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1b2e7, 77 },
    { 0x2191b2e8, 90 },
    { 0x8401b2e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1b2ed, 90 },
    { 0x2661b2ed, 61 },
    { 0xbfe1b2ee, 90 },
    { 0, 0 },
    { 0x4a1b2f1, 46 },
    { 0xb5e1b2f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161b2f5, 52 },
    { 0x3d91b2f6, 90 },
    { 0, 0 },
    { 0x98c1b2f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1b2fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1b301, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1111b306, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc931b30a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde31b30d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6721b310, 95 },
    { 0, 0 },
    { 0x3321b312, 90 },
    { 0xe8d1b313, 55 },
    { 0x25f1b312, 65 },
    { 0, 0 },
    { 0x43a1b316, 46 },
    { 0x1bd1b317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1b31b, 90 },
    { 0xa141b31c, 76 },
    { 0x231b31c, 98 },
    { 0, 0 },
    { 0x7ea1b31f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d51b323, 23 },
    { 0, 0 },
    { 0x2881b325, 96 },
    { 0xbc41b325, 23 },
    { 0xcc11b327, 85 },
    { 0x9b01b328, 98 },
    { 0xbd01b327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b21b330, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501b335, 75 },
    { 0, 0 },
    { 0x10b1b337, 33 },
    { 0xbf81b337, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511b33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c1b345, 55 },
    { 0, 0 },
    { 0xda11b347, 33 },
    { 0, 0 },
    { 0x7f81b349, 52 },
    { 0x6c21b349, 33 },
    { 0x9501b34b, 61 },
    { 0xe741b34c, 90 },
    { 0xc181b34b, 95 },
    { 0xf881b34a, 90 },
    { 0x8d31b34f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce1b357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf1b35a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f1b35d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521b361, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb41b368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1b36b, 23 },
    { 0x34e1b36c, 33 },
    { 0xe421b36b, 33 },
    { 0, 0 },
    { 0x62c1b36f, 90 },
    { 0, 0 },
    { 0x3381b371, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691b379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb311b37c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc31b37f, 76 },
    { 0x9dc1b380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1b387, 17 },
    { 0x49f1b387, 24 },
    { 0x8191b389, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81b392, 68 },
    { 0x1ef1b392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a1b399, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x941b39c, 98 },
    { 0xb161b39d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf961b3a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e41b3a6, 61 },
    { 0xeef1b3a6, 61 },
    { 0, 0 },
    { 0xadd1b3a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501b3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd991b3b1, 87 },
    { 0x11b1b3b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301b3b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b3ba, 43 },
    { 0xc481b3bb, 61 },
    { 0x6241b3bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa591b3bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61b3c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b1b3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5131b3ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651b3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b3d1, 87 },
    { 0, 0 },
    { 0x23b1b3d3, 36 },
    { 0, 0 },
    { 0x9491b3d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d1b3db, 90 },
    { 0x9ff1b3dc, 55 },
    { 0x57a1b3db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b3e5, 76 },
    { 0x5031b3e6, 78 },
    { 0xb6e1b3e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371b3eb, 81 },
    { 0x9c61b3eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91b3f2, 85 },
    { 0x8a41b3f3, 95 },
    { 0x3761b3f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251b3f9, 61 },
    { 0x3a11b3fa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe341b3ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e1b407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71b414, 90 },
    { 0xd2c1b414, 52 },
    { 0x6c51b416, 99 },
    { 0, 0 },
    { 0xd311b418, 87 },
    { 0, 0 },
    { 0x9571b41a, 95 },
    { 0xdd41b41b, 33 },
    { 0xea71b41b, 90 },
    { 0xa001b41c, 93 },
    { 0x3401b41c, 100 },
    { 0x4911b41f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a11b424, 43 },
    { 0x7a51b424, 61 },
    { 0xe081b426, 51 },
    { 0x6ab1b426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9311b42d, 90 },
    { 0xbab1b42e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee41b435, 33 },
    { 0, 0 },
    { 0xd091b437, 95 },
    { 0, 0 },
    { 0xb8c1b439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3821b447, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901b44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b31b450, 76 },
    { 0xf811b451, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21b457, 33 },
    { 0x6af1b458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11b461, 90 },
    { 0x7d71b462, 98 },
    { 0x84b1b463, 95 },
    { 0xd0c1b464, 33 },
    { 0x5c71b465, 95 },
    { 0x5001b463, 61 },
    { 0x36b1b463, 55 },
    { 0x7211b467, 34 },
    { 0x5c11b464, 76 },
    { 0, 0 },
    { 0xa381b46b, 88 },
    { 0x9511b46b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9411b477, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491b47b, 98 },
    { 0xf51b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf1b47f, 40 },
    { 0x6d31b480, 87 },
    { 0xa981b480, 68 },
    { 0xe321b482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb1b485, 76 },
    { 0, 0 },
    { 0xf741b487, 34 },
    { 0xcff1b488, 23 },
    { 0, 0 },
    { 0xc521b48a, 33 },
    { 0x1bb1b48a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b1b490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b493, 90 },
    { 0xab21b494, 95 },
    { 0x52d1b493, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1301b498, 100 },
    { 0x8db1b499, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc291b49e, 90 },
    { 0, 0 },
    { 0xef61b4a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc611b4a6, 43 },
    { 0, 0 },
    { 0x5ae1b4a8, 48 },
    { 0xd221b4a9, 33 },
    { 0, 0 },
    { 0xd8b1b4ab, 22 },
    { 0x3cb1b4ac, 93 },
    { 0x16a1b4ac, 55 },
    { 0, 0 },
    { 0x1fc1b4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1b4b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771b4b9, 98 },
    { 0xe721b4b9, 34 },
    { 0xdd71b4bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7311b4c1, 95 },
    { 0xfdb1b4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e11b4c7, 43 },
    { 0x2431b4c8, 68 },
    { 0x7781b4c7, 99 },
    { 0x60e1b4c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b1b4cd, 23 },
    { 0xd1e1b4ce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a1b4d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f71b4d8, 74 },
    { 0xf6c1b4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee41b4dd, 33 },
    { 0, 0 },
    { 0x37b1b4df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f61b4e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871b4ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb01b4f3, 90 },
    { 0xa6f1b4f4, 76 },
    { 0, 0 },
    { 0x37f1b4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a51b4fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac71b4fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f1b502, 99 },
    { 0x3de1b502, 84 },
    { 0x1871b503, 98 },
    { 0xdd11b505, 61 },
    { 0, 0 },
    { 0x471b507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1b50a, 23 },
    { 0x9421b50b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad71b510, 61 },
    { 0, 0 },
    { 0x7f11b512, 17 },
    { 0x2781b512, 95 },
    { 0xd2f1b514, 87 },
    { 0x67e1b515, 87 },
    { 0x84d1b515, 83 },
    { 0x9081b517, 99 },
    { 0x93e1b518, 90 },
    { 0x20c1b519, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4131b51c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b11b521, 76 },
    { 0x4901b521, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa501b525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b1b52a, 98 },
    { 0x27d1b52b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841b535, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d81b539, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b1b53e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b41b545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5971b548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7511b54b, 23 },
    { 0, 0 },
    { 0xb001b54d, 84 },
    { 0, 0 },
    { 0xdb91b54f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1b556, 75 },
    { 0xec01b557, 61 },
    { 0x78b1b558, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3721b55c, 93 },
    { 0x8141b55d, 95 },
    { 0, 0 },
    { 0xeda1b55f, 98 },
    { 0x7871b55f, 43 },
    { 0xd761b55f, 90 },
    { 0x68e1b562, 95 },
    { 0xfc71b55f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd241b566, 85 },
    { 0, 0 },
    { 0x3bb1b568, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1b56c, 95 },
    { 0, 0 },
    { 0x4bc1b56e, 36 },
    { 0x8d61b56e, 61 },
    { 0x43f1b56e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901b574, 87 },
    { 0x5941b575, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a31b57b, 90 },
    { 0xfcd1b57b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb21b582, 61 },
    { 0, 0 },
    { 0xfbe1b584, 61 },
    { 0xa451b584, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1b58a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1b58e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd01b592, 23 },
    { 0x6591b592, 87 },
    { 0x2931b592, 46 },
    { 0xd161b595, 84 },
    { 0x9191b596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1b599, 61 },
    { 0x2dc1b599, 98 },
    { 0xce1b59b, 36 },
    { 0x2fb1b59b, 100 },
    { 0xd921b59b, 95 },
    { 0, 0 },
    { 0x6981b59f, 17 },
    { 0xac91b59f, 61 },
    { 0x6021b5a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f1b5a4, 23 },
    { 0xd351b5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f1b5aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea1b5af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1b5b4, 33 },
    { 0xba91b5b5, 55 },
    { 0xfeb1b5b6, 61 },
    { 0, 0 },
    { 0x27e1b5b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa761b5c3, 67 },
    { 0, 0 },
    { 0xc7a1b5c5, 76 },
    { 0x9b51b5c6, 90 },
    { 0xca41b5c7, 55 },
    { 0, 0 },
    { 0xaf1b5c9, 33 },
    { 0xef31b5ca, 90 },
    { 0xba81b5cb, 33 },
    { 0x4b11b5ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1351b5cf, 61 },
    { 0xb3f1b5d0, 90 },
    { 0xd3d1b5d1, 99 },
    { 0x7ef1b5d2, 84 },
    { 0x1561b5cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc171b5d7, 34 },
    { 0, 0 },
    { 0xe5f1b5d9, 93 },
    { 0x9301b5da, 44 },
    { 0x9d41b5db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd421b5e0, 61 },
    { 0, 0 },
    { 0x4ee1b5e2, 95 },
    { 0x5d01b5e2, 87 },
    { 0x9aa1b5e4, 90 },
    { 0x8911b5e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1b5ec, 53 },
    { 0x4f1b5ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b5f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a1b5f4, 23 },
    { 0, 0 },
    { 0x5f21b5f6, 33 },
    { 0xa8f1b5f7, 61 },
    { 0xb01b5f7, 61 },
    { 0x4561b5f9, 95 },
    { 0, 0 },
    { 0x5dd1b5fb, 80 },
    { 0x2761b5fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b5ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea61b604, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb051b60a, 90 },
    { 0xa6b1b60b, 98 },
    { 0x42e1b60c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a1b613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1b616, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e1b61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a1b620, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c1b626, 90 },
    { 0, 0 },
    { 0xef71b628, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11b62d, 98 },
    { 0, 0 },
    { 0xfc51b62f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8341b632, 98 },
    { 0, 0 },
    { 0x67a1b634, 55 },
    { 0xed31b635, 81 },
    { 0, 0 },
    { 0x8c11b637, 84 },
    { 0xfc91b637, 61 },
    { 0, 0 },
    { 0x9c41b63a, 90 },
    { 0x6c1b63b, 77 },
    { 0x6bb1b63c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51b640, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb11b643, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851b647, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c61b64b, 61 },
    { 0x3ce1b64c, 52 },
    { 0x72c1b64c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b11b652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b1b658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1b65d, 61 },
    { 0, 0 },
    { 0xd9a1b65f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef1b663, 98 },
    { 0xb0a1b663, 98 },
    { 0xe581b664, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8661b668, 90 },
    { 0xb311b669, 98 },
    { 0xfad1b66a, 33 },
    { 0xdc91b66b, 61 },
    { 0xca41b66c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2751b66f, 46 },
    { 0xa131b670, 23 },
    { 0x2411b670, 98 },
    { 0xb5f1b672, 34 },
    { 0x3241b673, 61 },
    { 0xbd11b674, 90 },
    { 0x3b61b675, 93 },
    { 0x1ce1b676, 33 },
    { 0, 0 },
    { 0x2741b678, 23 },
    { 0x3861b679, 95 },
    { 0xeb71b67a, 98 },
    { 0x5bc1b67a, 61 },
    { 0, 0 },
    { 0x3131b67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5391b680, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd881b684, 61 },
    { 0xfa91b684, 46 },
    { 0xc821b686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b68c, 55 },
    { 0x2491b68c, 95 },
    { 0xffd1b68e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca71b691, 33 },
    { 0, 0 },
    { 0x49a1b693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471b69d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc71b6a3, 81 },
    { 0xbec1b6a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f1b6a7, 61 },
    { 0x29e1b6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c1b6af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321b6b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b1b6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae71b6bc, 98 },
    { 0, 0 },
    { 0x8c91b6be, 91 },
    { 0x6ba1b6bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc1b6c4, 46 },
    { 0, 0 },
    { 0x6231b6c6, 34 },
    { 0x6b11b6c7, 76 },
    { 0, 0 },
    { 0xecf1b6c9, 33 },
    { 0, 0 },
    { 0xd021b6cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba1b6ce, 85 },
    { 0xb721b6cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e1b6da, 95 },
    { 0, 0 },
    { 0x53f1b6dc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd921b6e2, 61 },
    { 0xd3e1b6e3, 33 },
    { 0xb931b6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b6e7, 98 },
    { 0, 0 },
    { 0xc6a1b6e9, 76 },
    { 0x3201b6ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a51b6ed, 61 },
    { 0, 0 },
    { 0x1371b6ef, 90 },
    { 0x9361b6f0, 90 },
    { 0, 0 },
    { 0x9241b6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb1b6f7, 34 },
    { 0xaa91b6f8, 90 },
    { 0xacb1b6f7, 93 },
    { 0x7d61b6fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2471b6ff, 95 },
    { 0xf7a1b700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3491b708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf391b70b, 95 },
    { 0xbf11b70c, 46 },
    { 0, 0 },
    { 0xb431b70e, 91 },
    { 0xca1b70f, 33 },
    { 0xf791b70e, 99 },
    { 0xbe31b711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c1b714, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9491b71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2331b71d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2001b722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa011b726, 23 },
    { 0xf1c1b727, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd11b72b, 61 },
    { 0x7991b72b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8571b733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761b736, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51b73b, 61 },
    { 0x9b51b73c, 90 },
    { 0x3ae1b73c, 98 },
    { 0x17b1b73e, 90 },
    { 0xca41b73f, 93 },
    { 0x8e21b740, 93 },
    { 0xbd11b741, 90 },
    { 0x6931b742, 52 },
    { 0x8ca1b742, 61 },
    { 0x561b742, 84 },
    { 0x74b1b743, 46 },
    { 0xcb1b746, 87 },
    { 0x6791b746, 33 },
    { 0, 0 },
    { 0xb1b749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281b754, 84 },
    { 0x1a91b754, 33 },
    { 0x1ff1b756, 33 },
    { 0x6e51b757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb11b75d, 89 },
    { 0xb21b75d, 85 },
    { 0xeff1b75e, 78 },
    { 0xbc41b75f, 46 },
    { 0x2811b761, 95 },
    { 0x3a91b762, 98 },
    { 0xc871b763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe561b769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571b76e, 99 },
    { 0x6971b76f, 93 },
    { 0x71a1b76f, 61 },
    { 0x3771b76e, 95 },
    { 0x37a1b76e, 78 },
    { 0x3041b76e, 75 },
    { 0, 0 },
    { 0xd171b775, 98 },
    { 0x4e01b775, 52 },
    { 0x94e1b777, 81 },
    { 0xe391b778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c51b780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91b783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d1b788, 23 },
    { 0xaa41b789, 33 },
    { 0, 0 },
    { 0xff41b78b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd011b792, 34 },
    { 0x7c71b793, 68 },
    { 0x49f1b792, 61 },
    { 0xcab1b793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf401b79c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1b79f, 34 },
    { 0, 0 },
    { 0x8ff1b7a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981b7a5, 33 },
    { 0, 0 },
    { 0xaac1b7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c1b7aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3441b7ad, 17 },
    { 0, 0 },
    { 0xdb31b7af, 55 },
    { 0xfa41b7b0, 76 },
    { 0, 0 },
    { 0xb111b7b2, 34 },
    { 0, 0 },
    { 0x23f1b7b4, 33 },
    { 0xafb1b7b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7671b7b9, 98 },
    { 0x2931b7ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be1b7c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1b7c3, 95 },
    { 0x6e81b7c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31b7c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581b7cd, 81 },
    { 0xef21b7cd, 95 },
    { 0x8b01b7ce, 98 },
    { 0xf251b7ce, 66 },
    { 0x7e1b7d1, 52 },
    { 0x4041b7d2, 68 },
    { 0x68f1b7d3, 90 },
    { 0x7281b7d4, 61 },
    { 0x91b1b7d4, 33 },
    { 0x34a1b7d1, 61 },
    { 0x3351b7d7, 33 },
    { 0xf551b7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111b7dd, 98 },
    { 0x161b7de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e1b7e4, 88 },
    { 0x86d1b7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef01b7e9, 101 },
    { 0xea01b7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671b7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f1b7f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f1b7fb, 68 },
    { 0x8e1b7fc, 61 },
    { 0x3111b7fc, 67 },
    { 0x5501b7fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd41b804, 61 },
    { 0xf901b805, 46 },
    { 0, 0 },
    { 0x2261b807, 19 },
    { 0x4021b807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc1b80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f21b810, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811b819, 33 },
    { 0x16d1b819, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8261b81d, 98 },
    { 0xc991b81d, 76 },
    { 0xe341b81f, 90 },
    { 0xebb1b81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda71b828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9601b82c, 43 },
    { 0x63b1b82d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb1b831, 66 },
    { 0xfc61b832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561b835, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb971b83a, 90 },
    { 0x40e1b83b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa971b83f, 33 },
    { 0xf2b1b83f, 76 },
    { 0xbf41b841, 46 },
    { 0x1ea1b841, 100 },
    { 0x8c01b841, 46 },
    { 0x9391b842, 90 },
    { 0xfd31b845, 75 },
    { 0x6531b841, 98 },
    { 0, 0 },
    { 0x4ba1b848, 33 },
    { 0, 0 },
    { 0xbb01b84a, 46 },
    { 0x5f51b84a, 75 },
    { 0, 0 },
    { 0xcde1b84d, 33 },
    { 0xadd1b84e, 55 },
    { 0, 0 },
    { 0xef51b850, 90 },
    { 0, 0 },
    { 0xfa41b852, 90 },
    { 0, 0 },
    { 0xf2c1b854, 33 },
    { 0x2e1b855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf81b85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221b85f, 33 },
    { 0x1aa1b85f, 84 },
    { 0, 0 },
    { 0xc2e1b862, 77 },
    { 0, 0 },
    { 0x26e1b864, 23 },
    { 0x6b21b865, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf041b86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221b86e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f1b874, 23 },
    { 0, 0 },
    { 0x5801b876, 88 },
    { 0x1561b876, 90 },
    { 0xaba1b878, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51b87c, 67 },
    { 0x3621b87d, 34 },
    { 0x3831b87d, 84 },
    { 0x2051b87f, 91 },
    { 0x76e1b880, 61 },
    { 0x7c81b880, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb611b884, 33 },
    { 0xda21b884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f1b892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451b897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1b89b, 90 },
    { 0x28c1b89c, 75 },
    { 0x4651b89c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c81b8a0, 81 },
    { 0x2c11b8a1, 55 },
    { 0xe811b8a1, 43 },
    { 0, 0 },
    { 0x9ad1b8a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa1b8a7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f1b8aa, 33 },
    { 0x2631b8ab, 34 },
    { 0xad61b8ac, 66 },
    { 0xe2e1b8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431b8b0, 87 },
    { 0x2b21b8b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b51b8b5, 91 },
    { 0xc761b8b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e31b8b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1b8bc, 76 },
    { 0x55e1b8bc, 61 },
    { 0, 0 },
    { 0x4bb1b8bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda81b8c2, 66 },
    { 0x8bb1b8c3, 85 },
    { 0x44e1b8c4, 23 },
    { 0x1fc1b8c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe581b8c8, 90 },
    { 0x2271b8c9, 76 },
    { 0x931b8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741b8ce, 61 },
    { 0, 0 },
    { 0xb1f1b8d0, 33 },
    { 0x5d11b8d1, 23 },
    { 0xdbd1b8d0, 95 },
    { 0x2ae1b8d0, 99 },
    { 0xa1c1b8d3, 23 },
    { 0xf301b8d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261b8d8, 77 },
    { 0xc851b8d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501b8dd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121b8e0, 94 },
    { 0, 0 },
    { 0x31c1b8e2, 85 },
    { 0x54c1b8e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc951b8e6, 65 },
    { 0xf121b8e6, 65 },
    { 0, 0 },
    { 0x49b1b8e9, 55 },
    { 0xe6a1b8e9, 55 },
    { 0x4581b8eb, 98 },
    { 0xfdc1b8ec, 90 },
    { 0x3221b8ed, 91 },
    { 0xcd01b8ed, 95 },
    { 0x3951b8eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451b8f2, 85 },
    { 0, 0 },
    { 0xed31b8f4, 98 },
    { 0, 0 },
    { 0x5e1b8f6, 90 },
    { 0x1e21b8f6, 90 },
    { 0, 0 },
    { 0xea11b8f9, 95 },
    { 0x9981b8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b21b8ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c71b904, 23 },
    { 0, 0 },
    { 0x7f1b906, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401b909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa71b90c, 33 },
    { 0x2f81b90c, 61 },
    { 0xee1b90e, 99 },
    { 0xe111b90f, 90 },
    { 0, 0 },
    { 0x7ab1b911, 98 },
    { 0, 0 },
    { 0xff91b913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b919, 46 },
    { 0, 0 },
    { 0x5081b91b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e1b922, 98 },
    { 0, 0 },
    { 0x7361b924, 85 },
    { 0xde81b925, 52 },
    { 0xf8e1b926, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd371b92d, 98 },
    { 0x9a31b92d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc01b939, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2181b93c, 33 },
    { 0xe821b93c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7911b944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa31b948, 47 },
    { 0, 0 },
    { 0x921b94a, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b1b94f, 23 },
    { 0x9e51b950, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141b955, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd581b95c, 77 },
    { 0xd351b95c, 99 },
    { 0xb2e1b95d, 100 },
    { 0xec1b95f, 36 },
    { 0x7601b95f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df1b963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591b966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac1b96b, 33 },
    { 0x67e1b96b, 33 },
    { 0, 0 },
    { 0xe951b96e, 90 },
    { 0x6961b96f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b975, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad31b97a, 77 },
    { 0, 0 },
    { 0xb961b97c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec51b981, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8081b985, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc931b989, 33 },
    { 0xa91b98a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3321b98d, 93 },
    { 0x9591b98e, 65 },
    { 0xfc81b98e, 90 },
    { 0x8551b98e, 55 },
    { 0xa601b98e, 98 },
    { 0, 0 },
    { 0x6af1b993, 91 },
    { 0xc521b994, 98 },
    { 0, 0 },
    { 0x36e1b996, 61 },
    { 0, 0 },
    { 0x73a1b998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961b99b, 61 },
    { 0x88c1b99c, 23 },
    { 0, 0 },
    { 0x94b1b99e, 61 },
    { 0xc61b99f, 33 },
    { 0xbf41b99f, 23 },
    { 0xa841b9a1, 91 },
    { 0xab91b99f, 87 },
    { 0x1d71b9a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401b9a7, 61 },
    { 0xe1a1b9a8, 90 },
    { 0xb7a1b9a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091b9ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa261b9b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c1b9b4, 65 },
    { 0x6d61b9b5, 36 },
    { 0x78e1b9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd951b9bc, 98 },
    { 0x1f31b9bd, 90 },
    { 0, 0 },
    { 0x53b1b9bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa1b9c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921b9c7, 90 },
    { 0x91d1b9c8, 15 },
    { 0x7e31b9c9, 90 },
    { 0x56b1b9c8, 90 },
    { 0xefc1b9cb, 95 },
    { 0x821b9cc, 90 },
    { 0x90b1b9cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b61b9d1, 81 },
    { 0xdf21b9d2, 76 },
    { 0x6e41b9d3, 95 },
    { 0xa921b9d4, 34 },
    { 0, 0 },
    { 0x5e11b9d6, 36 },
    { 0x6ca1b9d7, 33 },
    { 0x38f1b9d8, 90 },
    { 0x92f1b9d7, 90 },
    { 0xc571b9da, 46 },
    { 0xf141b9da, 33 },
    { 0x3c41b9d8, 33 },
    { 0xa531b9dd, 48 },
    { 0x1581b9dd, 90 },
    { 0xcc61b9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a51b9e7, 76 },
    { 0x3231b9e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6881b9ee, 33 },
    { 0x9941b9ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b9f2, 55 },
    { 0xb71b9f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1b9f7, 80 },
    { 0x2131b9f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c71b9fe, 78 },
    { 0, 0 },
    { 0x3d51ba00, 91 },
    { 0xe0f1ba00, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f1ba05, 23 },
    { 0x8071ba06, 95 },
    { 0, 0 },
    { 0xd041ba08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda1ba0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb1ba12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ba17, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x6571ba1a, 23 },
    { 0xaae1ba1b, 98 },
    { 0xb651ba1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d81ba23, 95 },
    { 0, 0 },
    { 0xe751ba25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31ba2c, 66 },
    { 0xa2c1ba2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1ba30, 90 },
    { 0, 0 },
    { 0x2a21ba32, 67 },
    { 0xa2d1ba32, 43 },
    { 0x6f91ba34, 61 },
    { 0x17b1ba35, 78 },
    { 0x6ff1ba35, 61 },
    { 0x6791ba37, 98 },
    { 0xd9b1ba34, 34 },
    { 0x8d11ba34, 61 },
    { 0xa431ba3a, 55 },
    { 0xa4a1ba3b, 40 },
    { 0x1661ba3a, 55 },
    { 0x4f01ba3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6441ba43, 33 },
    { 0, 0 },
    { 0x71e1ba45, 61 },
    { 0, 0 },
    { 0x4ac1ba47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb1ba4a, 76 },
    { 0x18c1ba4b, 52 },
    { 0, 0 },
    { 0x3d21ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b1ba55, 87 },
    { 0, 0 },
    { 0x6a71ba57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d1ba5c, 61 },
    { 0x1cf1ba5d, 61 },
    { 0x5f61ba5d, 88 },
    { 0x6b91ba5f, 95 },
    { 0x7c21ba60, 33 },
    { 0x8081ba61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb01ba65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f1ba6a, 95 },
    { 0, 0 },
    { 0x5c71ba6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa341ba6f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e71ba72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b61ba75, 55 },
    { 0, 0 },
    { 0x7251ba77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501ba7b, 90 },
    { 0, 0 },
    { 0x6de1ba7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1ba80, 95 },
    { 0xca81ba81, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad1ba88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71ba8d, 61 },
    { 0, 0 },
    { 0xabc1ba8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1ba92, 23 },
    { 0xf421ba92, 23 },
    { 0x3b11ba94, 100 },
    { 0xfd71ba95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf911ba98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1baa1, 98 },
    { 0x5541baa2, 91 },
    { 0, 0 },
    { 0x80c1baa4, 55 },
    { 0xfc51baa4, 38 },
    { 0, 0 },
    { 0xa441baa7, 61 },
    { 0xf481baa8, 55 },
    { 0, 0 },
    { 0xaeb1baaa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651baae, 87 },
    { 0xdaf1baae, 43 },
    { 0, 0 },
    { 0x4a1bab1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41bab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6751bab8, 90 },
    { 0x4e01bab8, 52 },
    { 0x72d1bab9, 46 },
    { 0, 0 },
    { 0x3c11babc, 23 },
    { 0xaee1babc, 76 },
    { 0xb941babc, 95 },
    { 0x7c91babc, 95 },
    { 0x4fb1bac0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9921bac5, 61 },
    { 0xad71bac6, 98 },
    { 0, 0 },
    { 0x4401bac8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2331bad2, 85 },
    { 0x51a1bad3, 80 },
    { 0x2221bad4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c11bad8, 90 },
    { 0xcd71bad9, 90 },
    { 0x4fd1bada, 34 },
    { 0x60f1badb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff1badf, 43 },
    { 0, 0 },
    { 0xee31bae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c1baeb, 68 },
    { 0, 0 },
    { 0xcd01baed, 55 },
    { 0, 0 },
    { 0xc6f1baef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a41baf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771baf5, 43 },
    { 0x4481baf6, 95 },
    { 0x2621baf6, 23 },
    { 0x1011baf8, 81 },
    { 0xd0f1baf9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001bafd, 23 },
    { 0x9241bafe, 52 },
    { 0x8dd1baff, 61 },
    { 0xf611bb00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c1bb06, 87 },
    { 0x3d81bb06, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2381bb0f, 34 },
    { 0xa4f1bb10, 77 },
    { 0, 0 },
    { 0x5641bb12, 67 },
    { 0, 0 },
    { 0x20b1bb14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f1bb20, 99 },
    { 0, 0 },
    { 0x1421bb22, 55 },
    { 0x4ae1bb23, 83 },
    { 0x68a1bb23, 90 },
    { 0x79f1bb25, 100 },
    { 0x4261bb26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c21bb29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea91bb34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301bb38, 33 },
    { 0xf481bb39, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed11bb3d, 78 },
    { 0x9271bb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf11bb41, 78 },
    { 0, 0 },
    { 0x4601bb43, 84 },
    { 0xaa31bb44, 98 },
    { 0, 0 },
    { 0x6841bb46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de1bb4c, 90 },
    { 0x78e1bb4d, 80 },
    { 0, 0 },
    { 0x8741bb4f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71bb53, 98 },
    { 0, 0 },
    { 0x5041bb55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b41bb5b, 95 },
    { 0xc791bb5b, 85 },
    { 0x6e61bb5b, 17 },
    { 0, 0 },
    { 0x44f1bb5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8391bb62, 33 },
    { 0, 0 },
    { 0x9941bb64, 90 },
    { 0x10b1bb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe031bb68, 88 },
    { 0x9991bb68, 95 },
    { 0, 0 },
    { 0x1031bb6b, 61 },
    { 0xaa71bb6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c1bb70, 32 },
    { 0x5aa1bb70, 23 },
    { 0, 0 },
    { 0x2d1bb73, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb041bb76, 95 },
    { 0xa0e1bb76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1bb7b, 67 },
    { 0x8d71bb7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f1bb81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1901bb87, 85 },
    { 0xfc71bb88, 90 },
    { 0x1961bb89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1bb8e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd271bb92, 61 },
    { 0x62f1bb92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021bb96, 55 },
    { 0xe081bb97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21bb9a, 43 },
    { 0x17e1bb9b, 23 },
    { 0x7381bb9b, 77 },
    { 0x5361bb9d, 76 },
    { 0x8a01bb9e, 61 },
    { 0xd051bb9c, 90 },
    { 0, 0 },
    { 0x5a51bba1, 90 },
    { 0, 0 },
    { 0x5b11bba3, 34 },
    { 0x1d21bba4, 68 },
    { 0x1241bba5, 91 },
    { 0x3ad1bba6, 33 },
    { 0xa2f1bba5, 61 },
    { 0, 0 },
    { 0x11a1bba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc1bbad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc1bbb3, 90 },
    { 0, 0 },
    { 0x1181bbb5, 40 },
    { 0, 0 },
    { 0xc31bbb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6161bbbb, 43 },
    { 0xad01bbbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901bbbf, 17 },
    { 0, 0 },
    { 0xec1bbc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb481bbc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1bbcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551bbd4, 46 },
    { 0x2c91bbd4, 90 },
    { 0x251bbd6, 98 },
    { 0x4f21bbd7, 55 },
    { 0xe581bbd4, 60 },
    { 0x9d91bbd4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd1bbdc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b1bbdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba1bbe5, 60 },
    { 0, 0 },
    { 0xbde1bbe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8991bbea, 98 },
    { 0x2c01bbeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc1bbee, 91 },
    { 0x41a1bbee, 76 },
    { 0xe021bbf0, 43 },
    { 0x1351bbf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d1bbf5, 90 },
    { 0x4aa1bbf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961bc01, 23 },
    { 0xd8a1bc02, 98 },
    { 0x3181bc02, 80 },
    { 0xb3c1bc04, 34 },
    { 0x4751bc05, 98 },
    { 0x99f1bc04, 33 },
    { 0x2c51bc06, 80 },
    { 0xae31bc04, 81 },
    { 0x8cb1bc09, 53 },
    { 0xf4b1bc0a, 95 },
    { 0x2ed1bc08, 33 },
    { 0xf2b1bc02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1bc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6701bc13, 55 },
    { 0, 0 },
    { 0x6cb1bc15, 67 },
    { 0xa51bc16, 98 },
    { 0x7d11bc17, 23 },
    { 0x6e01bc18, 33 },
    { 0x5fa1bc17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491bc24, 98 },
    { 0x4421bc25, 95 },
    { 0, 0 },
    { 0x7831bc27, 61 },
    { 0, 0 },
    { 0xecf1bc29, 90 },
    { 0xace1bc2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261bc2f, 76 },
    { 0x2aa1bc30, 49 },
    { 0x1f1bc30, 95 },
    { 0x1a71bc31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a1bc37, 95 },
    { 0x3a31bc38, 23 },
    { 0x5621bc38, 95 },
    { 0x6221bc3a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e51bc43, 33 },
    { 0x9891bc44, 81 },
    { 0x7b21bc45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1bc48, 76 },
    { 0xa371bc49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc1bc51, 100 },
    { 0x5961bc52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f31bc56, 85 },
    { 0xc881bc57, 33 },
    { 0xbcb1bc57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11bc5b, 33 },
    { 0x2d91bc5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa461bc62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6461bc66, 90 },
    { 0x84f1bc66, 90 },
    { 0x6c1bc66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1bc6b, 75 },
    { 0, 0 },
    { 0xa981bc6d, 33 },
    { 0x8fd1bc6d, 23 },
    { 0, 0 },
    { 0x371bc70, 33 },
    { 0, 0 },
    { 0x5251bc72, 66 },
    { 0, 0 },
    { 0xad51bc74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f81bc77, 90 },
    { 0x5261bc78, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec61bc7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af1bc80, 61 },
    { 0x1b81bc81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71bc86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf91bc8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea1bc8e, 98 },
    { 0, 0 },
    { 0xd1f1bc90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d01bc95, 98 },
    { 0xd811bc96, 76 },
    { 0xf4b1bc96, 55 },
    { 0x9b81bc97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a1bc9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd221bca7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d31bcab, 100 },
    { 0x751bcab, 90 },
    { 0xe851bcab, 95 },
    { 0, 0 },
    { 0x5eb1bcaf, 98 },
    { 0x8611bcb0, 55 },
    { 0, 0 },
    { 0x1551bcb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11bcb7, 90 },
    { 0x46e1bcb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e1bcbe, 84 },
    { 0x7771bcbe, 90 },
    { 0xf701bcc0, 90 },
    { 0x9871bcc1, 52 },
    { 0x6441bcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1bcc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd671bccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd31bcdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021bcdf, 34 },
    { 0x5bd1bce0, 95 },
    { 0x6b11bce1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e91bce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf31bcee, 33 },
    { 0x1ee1bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9751bcf2, 88 },
    { 0xb881bcf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e1bcf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21bcfb, 98 },
    { 0x9201bcfc, 23 },
    { 0x5ed1bcfc, 33 },
    { 0x58f1bcfe, 95 },
    { 0x6a31bcff, 61 },
    { 0, 0 },
    { 0x4761bd01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce1bd04, 68 },
    { 0xc051bd05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1bd08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe1bd0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891bd11, 33 },
    { 0x79c1bd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d91bd1a, 11 },
    { 0, 0 },
    { 0xa661bd1c, 90 },
    { 0x2f61bd1d, 77 },
    { 0x4e51bd1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61bd23, 33 },
    { 0x37f1bd24, 55 },
    { 0x1d01bd25, 34 },
    { 0xdea1bd25, 33 },
    { 0x38e1bd27, 46 },
    { 0x2af1bd28, 61 },
    { 0, 0 },
    { 0x851bd2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4921bd31, 23 },
    { 0x1851bd32, 90 },
    { 0xf821bd31, 96 },
    { 0x3071bd33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba1bd44, 40 },
    { 0xdf41bd44, 33 },
    { 0, 0 },
    { 0xfaa1bd47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc1bd4b, 40 },
    { 0xa501bd4c, 33 },
    { 0, 0 },
    { 0x68d1bd4e, 90 },
    { 0, 0 },
    { 0xaf61bd50, 61 },
    { 0x2a91bd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd1bd55, 33 },
    { 0x3e01bd55, 55 },
    { 0, 0 },
    { 0x52b1bd58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee51bd5b, 90 },
    { 0xa351bd5b, 90 },
    { 0xd2d1bd5b, 93 },
    { 0xee91bd5e, 93 },
    { 0xcda1bd5f, 61 },
    { 0x3b41bd60, 33 },
    { 0xbc91bd5f, 98 },
    { 0x9a91bd5b, 91 },
    { 0xd271bd5d, 90 },
    { 0x2341bd64, 49 },
    { 0x1401bd64, 33 },
    { 0x8401bd66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41bd6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1bd73, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461bd7f, 98 },
    { 0, 0 },
    { 0xb401bd81, 17 },
    { 0, 0 },
    { 0xc6c1bd83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11bd88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec91bd8c, 93 },
    { 0, 0 },
    { 0xb791bd8e, 95 },
    { 0xfe21bd8f, 43 },
    { 0, 0 },
    { 0xc21bd91, 76 },
    { 0x24e1bd92, 60 },
    { 0x3c31bd93, 46 },
    { 0x90e1bd92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f1bd9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a21bda0, 66 },
    { 0x74c1bda1, 95 },
    { 0xf911bda2, 43 },
    { 0, 0 },
    { 0x8411bda4, 98 },
    { 0x5211bda5, 93 },
    { 0x5131bda4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb1bda9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d51bdae, 61 },
    { 0xb3f1bdaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c1bdb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c1bdbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd501bdc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf911bdc4, 83 },
    { 0x1a1bdc4, 95 },
    { 0x49e1bdc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc1bdc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1bdcc, 99 },
    { 0x7141bdcc, 100 },
    { 0xc181bdce, 90 },
    { 0x9bf1bdce, 95 },
    { 0x85e1bdcf, 55 },
    { 0xed31bdcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac1bdda, 33 },
    { 0x2f41bdda, 85 },
    { 0x51bdda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4911bde0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d1bde5, 90 },
    { 0x4081bde6, 76 },
    { 0x1401bde6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac51bdf6, 23 },
    { 0xe321bdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1bdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1911be03, 23 },
    { 0, 0 },
    { 0x6631be05, 33 },
    { 0x1c61be06, 100 },
    { 0x8e1be07, 90 },
    { 0x85b1be07, 51 },
    { 0xc341be09, 90 },
    { 0x67c1be08, 61 },
    { 0x301be0a, 76 },
    { 0x58f1be0c, 93 },
    { 0x40b1be07, 33 },
    { 0, 0 },
    { 0x8791be0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b1be16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71be1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91be2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe821be32, 55 },
    { 0x4d51be32, 43 },
    { 0, 0 },
    { 0xc051be35, 67 },
    { 0x4821be36, 95 },
    { 0x5391be35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91be3a, 61 },
    { 0xe171be3b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1be43, 85 },
    { 0xc5a1be44, 93 },
    { 0xfd81be45, 61 },
    { 0x9be1be46, 23 },
    { 0x98a1be45, 78 },
    { 0x68d1be44, 99 },
    { 0x55b1be49, 98 },
    { 0x5041be43, 61 },
    { 0, 0 },
    { 0x9961be4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381be51, 14 },
    { 0, 0 },
    { 0xa5f1be53, 23 },
    { 0, 0 },
    { 0x77e1be55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591be59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c81be5f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81be62, 36 },
    { 0xd841be63, 90 },
    { 0x9921be64, 95 },
    { 0x9a61be65, 68 },
    { 0x7f51be65, 84 },
    { 0x1ae1be67, 85 },
    { 0x9121be63, 23 },
    { 0, 0 },
    { 0xac81be6a, 33 },
    { 0, 0 },
    { 0xf261be6c, 33 },
    { 0x2f21be6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1be74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde41be7b, 61 },
    { 0xb2e1be7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6201be80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a1be84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd601be91, 66 },
    { 0, 0 },
    { 0x6241be93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41be99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671be9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4981bea2, 23 },
    { 0xf901bea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51beaa, 23 },
    { 0xa941beab, 90 },
    { 0xa071beac, 90 },
    { 0x39f1bead, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211beb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6151beb4, 90 },
    { 0, 0 },
    { 0x8931beb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb941bebe, 37 },
    { 0, 0 },
    { 0xf481bec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca1bec6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd1becd, 84 },
    { 0, 0 },
    { 0x2641becf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7441bed9, 77 },
    { 0x5031bed9, 66 },
    { 0xb4a1beda, 95 },
    { 0x62f1bedc, 52 },
    { 0, 0 },
    { 0xf8f1bede, 90 },
    { 0, 0 },
    { 0xa551bee0, 90 },
    { 0xcd91bee1, 82 },
    { 0x9bd1bee2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9051bee8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f61beec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b1bef3, 55 },
    { 0xb721bef4, 90 },
    { 0x64a1bef5, 61 },
    { 0xce61bef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba91beff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b1bf03, 33 },
    { 0, 0 },
    { 0x6991bf05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341bf0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e81bf11, 33 },
    { 0xfdb1bf12, 65 },
    { 0xa951bf11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d1bf18, 64 },
    { 0, 0 },
    { 0x9851bf1a, 55 },
    { 0xf4d1bf1a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321bf1e, 95 },
    { 0xba81bf1f, 98 },
    { 0x69f1bf20, 33 },
    { 0x9071bf21, 34 },
    { 0xaf01bf1f, 90 },
    { 0x34b1bf20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921bf29, 90 },
    { 0x2d21bf29, 77 },
    { 0xa1d1bf2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c1bf2f, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb881bf35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121bf3a, 23 },
    { 0x5601bf3b, 33 },
    { 0x9761bf3b, 33 },
    { 0xabb1bf3b, 34 },
    { 0, 0 },
    { 0x9071bf3f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1bf43, 90 },
    { 0xb3b1bf43, 90 },
    { 0x43f1bf43, 90 },
    { 0x5b31bf45, 17 },
    { 0x70f1bf47, 23 },
    { 0x6151bf48, 46 },
    { 0xbfe1bf49, 66 },
    { 0x4a1bf4a, 33 },
    { 0x9291bf48, 55 },
    { 0xaf31bf4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd41bf52, 55 },
    { 0, 0 },
    { 0xa1c1bf54, 90 },
    { 0x86d1bf55, 23 },
    { 0x3791bf54, 36 },
    { 0xf331bf57, 90 },
    { 0xdc71bf58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe741bf5c, 34 },
    { 0, 0 },
    { 0x9571bf5e, 33 },
    { 0, 0 },
    { 0x2981bf60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae1bf66, 76 },
    { 0xf741bf67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71bf6a, 61 },
    { 0, 0 },
    { 0x6491bf6c, 90 },
    { 0xd511bf6d, 98 },
    { 0x6451bf6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1bf71, 66 },
    { 0x65e1bf71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1bf75, 33 },
    { 0, 0 },
    { 0x571bf77, 23 },
    { 0, 0 },
    { 0xcf1bf79, 90 },
    { 0x1e81bf7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a21bf7d, 95 },
    { 0, 0 },
    { 0xf351bf7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb1bf86, 76 },
    { 0x41c1bf87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb1bf8a, 91 },
    { 0xf921bf8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5681bf8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec1bf91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1851bf94, 95 },
    { 0xe1c1bf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb381bf98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8041bf9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2631bfa3, 61 },
    { 0x8411bfa4, 98 },
    { 0xf961bfa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31bfaa, 90 },
    { 0, 0 },
    { 0x4371bfac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351bfb4, 100 },
    { 0x5fd1bfb5, 85 },
    { 0xf71bfb4, 36 },
    { 0xbfe1bfb7, 23 },
    { 0xf711bfb5, 85 },
    { 0x3531bfb5, 33 },
    { 0x12a1bfba, 84 },
    { 0x181bfb9, 34 },
    { 0x6741bfbc, 95 },
    { 0, 0 },
    { 0x8391bfbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31bfc1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f1bfc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1401bfca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a1bfce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf461bfd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc821bfd8, 90 },
    { 0xca21bfd9, 95 },
    { 0, 0 },
    { 0x1011bfdb, 85 },
    { 0, 0 },
    { 0x3ea1bfdd, 55 },
    { 0x9da1bfde, 78 },
    { 0x96f1bfdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101bfee, 34 },
    { 0xda51bfef, 92 },
    { 0x2b51bfee, 23 },
    { 0xc491bff1, 34 },
    { 0x5a71bff2, 46 },
    { 0x7531bff3, 61 },
    { 0x61e1bff2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc791bff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb821bffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1431bffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3981c004, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd591c009, 98 },
    { 0x3f81c00a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe1c010, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c016, 76 },
    { 0x34a1c017, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c1c01c, 87 },
    { 0x3351c01d, 77 },
    { 0xe111c01e, 99 },
    { 0xb21c01c, 91 },
    { 0x6941c020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9681c023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b1c026, 33 },
    { 0x4d1c026, 93 },
    { 0, 0 },
    { 0xae71c029, 23 },
    { 0x95c1c029, 76 },
    { 0x1de1c02b, 76 },
    { 0xdb1c02c, 33 },
    { 0x2781c02d, 91 },
    { 0xe911c02e, 23 },
    { 0xe0b1c02f, 76 },
    { 0xb371c02e, 33 },
    { 0, 0 },
    { 0xa9d1c032, 55 },
    { 0xf7b1c032, 93 },
    { 0xcb91c034, 33 },
    { 0x5f91c035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7331c039, 90 },
    { 0, 0 },
    { 0x1d51c03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a71c042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d11c04c, 46 },
    { 0, 0 },
    { 0xd4d1c04e, 76 },
    { 0x5131c04f, 34 },
    { 0x13f1c050, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde1c053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11c05b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2131c05f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e21c062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451c066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a1c06b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1c06e, 61 },
    { 0, 0 },
    { 0x3ab1c070, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e1c076, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xd741c079, 43 },
    { 0xd9c1c07a, 98 },
    { 0x2601c07b, 76 },
    { 0x1941c07b, 43 },
    { 0xcd91c07b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451c081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241c085, 90 },
    { 0, 0 },
    { 0x5251c087, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571c08d, 90 },
    { 0x151c08e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa771c09f, 65 },
    { 0x2931c0a0, 98 },
    { 0x871c0a1, 55 },
    { 0xfa51c0a2, 76 },
    { 0, 0 },
    { 0xcd11c0a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041c0a8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf651c0ac, 61 },
    { 0xb981c0ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f1c0b1, 61 },
    { 0xa9c1c0b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11c0b9, 75 },
    { 0x3b21c0ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a1c0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9611c0c4, 65 },
    { 0xc201c0c5, 23 },
    { 0x97d1c0c6, 33 },
    { 0, 0 },
    { 0x7e01c0c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631c0cf, 85 },
    { 0x31d1c0cf, 76 },
    { 0x21a1c0cf, 88 },
    { 0x1df1c0d2, 34 },
    { 0x6471c0cf, 61 },
    { 0x1871c0cf, 43 },
    { 0x74e1c0d5, 43 },
    { 0x4c51c0d5, 61 },
    { 0x8131c0d1, 98 },
    { 0x95e1c0d8, 90 },
    { 0x6991c0d8, 61 },
    { 0xf281c0d7, 75 },
    { 0xc441c0d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431c0e5, 98 },
    { 0, 0 },
    { 0x2061c0e7, 76 },
    { 0xf8a1c0e8, 66 },
    { 0xb8e1c0e9, 95 },
    { 0xdea1c0ea, 67 },
    { 0xeb31c0eb, 55 },
    { 0x9a61c0ec, 76 },
    { 0, 0 },
    { 0x53e1c0ee, 61 },
    { 0xf4c1c0ef, 43 },
    { 0xf861c0f0, 76 },
    { 0xf101c0f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee1c0f5, 95 },
    { 0xc8f1c0f5, 76 },
    { 0, 0 },
    { 0xb671c0f8, 61 },
    { 0xf1d1c0f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c0fc, 75 },
    { 0x5e21c0fd, 98 },
    { 0, 0 },
    { 0x1761c0ff, 61 },
    { 0x3461c100, 95 },
    { 0, 0 },
    { 0x3dd1c102, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891c106, 84 },
    { 0, 0 },
    { 0x5381c108, 19 },
    { 0, 0 },
    { 0x7851c10a, 33 },
    { 0, 0 },
    { 0xb001c10c, 33 },
    { 0xaef1c10d, 46 },
    { 0xa991c10e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4241c112, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1c119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1c11d, 23 },
    { 0, 0 },
    { 0x2fa1c11f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1c123, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f1c128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae91c12c, 61 },
    { 0xc491c12c, 33 },
    { 0, 0 },
    { 0x6ac1c12f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41c133, 61 },
    { 0x8681c134, 95 },
    { 0x7b81c134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61c13a, 90 },
    { 0x8eb1c13b, 52 },
    { 0xd41c13b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c1c146, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd491c149, 33 },
    { 0, 0 },
    { 0xf7d1c14b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d1c158, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b1c15c, 100 },
    { 0, 0 },
    { 0xf091c15e, 98 },
    { 0xfb1c15e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241c166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f1c16a, 46 },
    { 0, 0 },
    { 0xdb81c16c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae1c172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71c176, 46 },
    { 0xd8a1c176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2381c17c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d1c17f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c01c182, 90 },
    { 0x24f1c182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31c186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701c18b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e1c18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941c197, 61 },
    { 0, 0 },
    { 0x5781c199, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9221c19f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71c1a4, 34 },
    { 0x6eb1c1a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1c1a8, 46 },
    { 0x6aa1c1a9, 78 },
    { 0xf521c1a8, 90 },
    { 0x7211c1aa, 100 },
    { 0x8dc1c1a8, 61 },
    { 0xf711c1ad, 69 },
    { 0, 0 },
    { 0x3fe1c1af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1c1b3, 98 },
    { 0xd141c1b4, 33 },
    { 0, 0 },
    { 0xfb01c1b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11c1b9, 90 },
    { 0, 0 },
    { 0xf11c1bb, 90 },
    { 0, 0 },
    { 0x7d31c1bd, 84 },
    { 0, 0 },
    { 0x82c1c1bf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71c1c3, 66 },
    { 0x421c1c4, 75 },
    { 0, 0 },
    { 0x4b21c1c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5301c1c9, 83 },
    { 0, 0 },
    { 0xb301c1cb, 55 },
    { 0x7591c1cc, 53 },
    { 0xaad1c1cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e1c1d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6601c1da, 68 },
    { 0x45a1c1da, 61 },
    { 0xc91c1dc, 34 },
    { 0xa981c1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7521c1e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c1c1e4, 34 },
    { 0, 0 },
    { 0x9311c1e6, 33 },
    { 0xba1c1e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5191c1ea, 61 },
    { 0xf481c1eb, 95 },
    { 0x9e71c1eb, 80 },
    { 0x27c1c1ed, 61 },
    { 0x5d91c1ee, 23 },
    { 0xeb01c1ed, 17 },
    { 0x42c1c1ec, 78 },
    { 0x7fd1c1ef, 61 },
    { 0, 0 },
    { 0xa11c1f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b1c1f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d1c1fa, 33 },
    { 0xb141c1fa, 90 },
    { 0x7ff1c1fb, 23 },
    { 0xac51c1fd, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781c200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba01c205, 91 },
    { 0, 0 },
    { 0x5a51c207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91c20e, 61 },
    { 0, 0 },
    { 0x7c21c210, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1951c214, 98 },
    { 0x88a1c214, 17 },
    { 0x23f1c216, 78 },
    { 0, 0 },
    { 0xadb1c218, 43 },
    { 0, 0 },
    { 0x2d41c21a, 61 },
    { 0xa611c21b, 95 },
    { 0x6e71c21c, 95 },
    { 0xc031c21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4731c224, 90 },
    { 0xe441c225, 23 },
    { 0xce91c226, 90 },
    { 0, 0 },
    { 0x4c01c228, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11c22d, 33 },
    { 0xb631c22e, 95 },
    { 0, 0 },
    { 0xaf1c230, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c1c234, 98 },
    { 0, 0 },
    { 0x2a71c236, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a1c239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc1c240, 98 },
    { 0, 0 },
    { 0x7761c242, 98 },
    { 0, 0 },
    { 0xf71c244, 68 },
    { 0x12c1c244, 69 },
    { 0, 0 },
    { 0xd291c247, 91 },
    { 0, 0 },
    { 0x9f81c249, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f1c24e, 61 },
    { 0, 0 },
    { 0xecf1c250, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021c255, 23 },
    { 0xd5f1c255, 76 },
    { 0x3a81c257, 23 },
    { 0x9751c258, 43 },
    { 0x86f1c259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf21c25d, 99 },
    { 0, 0 },
    { 0xf8c1c25f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71c265, 67 },
    { 0x3e51c265, 75 },
    { 0xff91c267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4451c26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111c270, 61 },
    { 0x3701c270, 33 },
    { 0xf061c271, 61 },
    { 0, 0 },
    { 0x4f51c274, 17 },
    { 0, 0 },
    { 0x7be1c276, 43 },
    { 0x2fd1c276, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a1c27a, 61 },
    { 0xbf31c27b, 98 },
    { 0x6f91c27c, 90 },
    { 0xdb61c27b, 55 },
    { 0x1011c27e, 76 },
    { 0x6381c27e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef71c286, 61 },
    { 0, 0 },
    { 0xd621c288, 33 },
    { 0x6e61c289, 90 },
    { 0x8cb1c28a, 43 },
    { 0x6d71c28b, 17 },
    { 0x3c41c28c, 23 },
    { 0, 0 },
    { 0x45f1c28e, 55 },
    { 0xcaa1c28f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf711c292, 90 },
    { 0xc891c293, 99 },
    { 0x3fa1c294, 33 },
    { 0xb091c293, 33 },
    { 0, 0 },
    { 0x63b1c297, 33 },
    { 0xc1c1c298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6261c29e, 76 },
    { 0, 0 },
    { 0x9641c2a0, 33 },
    { 0xa311c2a0, 48 },
    { 0xaba1c2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd81c2a5, 78 },
    { 0x571c2a5, 95 },
    { 0x96e1c2a7, 93 },
    { 0x1d81c2a8, 83 },
    { 0, 0 },
    { 0x4e81c2aa, 90 },
    { 0, 0 },
    { 0xe5d1c2ac, 23 },
    { 0x8c31c2ad, 80 },
    { 0, 0 },
    { 0x15f1c2af, 46 },
    { 0xf71c2b0, 33 },
    { 0xf671c2b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc161c2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe091c2b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c2be, 53 },
    { 0, 0 },
    { 0x5291c2c0, 86 },
    { 0xa4c1c2c1, 78 },
    { 0x5541c2c0, 87 },
    { 0xda1c2c3, 93 },
    { 0xc8c1c2c4, 90 },
    { 0x82a1c2c1, 33 },
    { 0, 0 },
    { 0xbb61c2c7, 67 },
    { 0x82f1c2c8, 90 },
    { 0xc381c2c9, 90 },
    { 0x6b01c2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1c2cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241c2d1, 23 },
    { 0, 0 },
    { 0xe571c2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1c2da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac1c2df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd81c2eb, 76 },
    { 0x91f1c2eb, 76 },
    { 0xab91c2ed, 66 },
    { 0xe821c2eb, 55 },
    { 0x851c2ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb71c2f2, 61 },
    { 0xf8e1c2f3, 33 },
    { 0x5671c2f4, 55 },
    { 0x11d1c2f4, 90 },
    { 0xcce1c2f6, 95 },
    { 0x3ee1c2f7, 95 },
    { 0x7721c2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a91c2fb, 95 },
    { 0x6f01c2fc, 90 },
    { 0xb9d1c2fc, 90 },
    { 0x6cb1c2fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8821c302, 88 },
    { 0, 0 },
    { 0x511c304, 98 },
    { 0, 0 },
    { 0x2581c306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd91c30b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5851c30e, 75 },
    { 0x901c30f, 76 },
    { 0xd381c30f, 95 },
    { 0x7271c30f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e11c31a, 34 },
    { 0xf71c31a, 87 },
    { 0x1851c31b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b1c324, 95 },
    { 0, 0 },
    { 0xc381c326, 46 },
    { 0, 0 },
    { 0x9bd1c328, 77 },
    { 0, 0 },
    { 0x331c32a, 43 },
    { 0x69a1c32a, 34 },
    { 0xc0f1c32c, 81 },
    { 0x9761c32a, 98 },
    { 0x83a1c32e, 39 },
    { 0x64f1c32b, 40 },
    { 0, 0 },
    { 0xf781c331, 95 },
    { 0, 0 },
    { 0x3e1c333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd661c338, 36 },
    { 0, 0 },
    { 0x7f51c33a, 36 },
    { 0x3241c33a, 95 },
    { 0, 0 },
    { 0xc421c33d, 33 },
    { 0x67a1c33e, 43 },
    { 0x9801c33e, 33 },
    { 0x8351c340, 90 },
    { 0xc831c341, 95 },
    { 0xd601c340, 69 },
    { 0x5cc1c342, 55 },
    { 0x75e1c344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b1c34c, 69 },
    { 0, 0 },
    { 0xa0e1c34e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef1c352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb541c35a, 66 },
    { 0xb431c35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811c363, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1c36a, 99 },
    { 0x6441c36b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2591c36e, 76 },
    { 0, 0 },
    { 0x3031c370, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c81c374, 95 },
    { 0xde31c375, 61 },
    { 0x6271c374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b1c379, 95 },
    { 0xaec1c37a, 46 },
    { 0x65f1c37a, 90 },
    { 0xe891c37b, 98 },
    { 0xb1f1c37b, 93 },
    { 0, 0 },
    { 0x9d31c37f, 61 },
    { 0xbac1c380, 55 },
    { 0x211c381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d1c384, 33 },
    { 0x2161c384, 90 },
    { 0, 0 },
    { 0x84f1c387, 95 },
    { 0, 0 },
    { 0xe231c389, 98 },
    { 0x65e1c38a, 98 },
    { 0, 0 },
    { 0xd961c38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d91c392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec1c396, 34 },
    { 0xc21c397, 61 },
    { 0x77f1c398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3511c39d, 78 },
    { 0x3411c39e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2611c3a3, 98 },
    { 0x76b1c3a4, 61 },
    { 0x26c1c3a5, 33 },
    { 0x841c3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf1c3ae, 95 },
    { 0xd871c3af, 46 },
    { 0x3761c3ae, 23 },
    { 0xb001c3b0, 61 },
    { 0x8181c3b1, 75 },
    { 0x75e1c3ae, 34 },
    { 0x42a1c3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21c3c1, 55 },
    { 0x8b01c3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed31c3cb, 66 },
    { 0xf981c3cc, 33 },
    { 0x2111c3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa981c3d2, 26 },
    { 0, 0 },
    { 0x9941c3d4, 90 },
    { 0, 0 },
    { 0xee31c3d6, 77 },
    { 0x83e1c3d7, 90 },
    { 0, 0 },
    { 0xc141c3d9, 88 },
    { 0x37b1c3d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181c3df, 33 },
    { 0, 0 },
    { 0x2fe1c3e1, 34 },
    { 0xf741c3e1, 90 },
    { 0xf3e1c3e3, 76 },
    { 0, 0 },
    { 0xbb91c3e5, 69 },
    { 0, 0 },
    { 0x86d1c3e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf861c3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01c3f2, 33 },
    { 0xb9d1c3f2, 98 },
    { 0x91d1c3f4, 98 },
    { 0x9531c3f5, 55 },
    { 0xf0d1c3f6, 61 },
    { 0, 0 },
    { 0xede1c3f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb61c3fb, 85 },
    { 0x2551c3fc, 81 },
    { 0xdcc1c3fc, 61 },
    { 0x681c3fe, 23 },
    { 0x9841c3ff, 77 },
    { 0xeb1c400, 34 },
    { 0x6f11c400, 46 },
    { 0x4281c3fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6941c40c, 65 },
    { 0x1791c40c, 46 },
    { 0x2a81c40d, 19 },
    { 0xa201c40f, 95 },
    { 0x80e1c40d, 33 },
    { 0, 0 },
    { 0x2521c412, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1c415, 61 },
    { 0, 0 },
    { 0xcea1c417, 78 },
    { 0xfd1c417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111c41e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1c423, 43 },
    { 0xbfd1c424, 61 },
    { 0x2021c423, 95 },
    { 0x9361c426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411c429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc541c42e, 55 },
    { 0x3541c42f, 23 },
    { 0, 0 },
    { 0x97f1c431, 90 },
    { 0, 0 },
    { 0x7871c433, 93 },
    { 0xae01c434, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41c43c, 61 },
    { 0xd5e1c43d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e1c440, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9661c444, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51c448, 90 },
    { 0x7321c448, 33 },
    { 0xc221c44a, 66 },
    { 0x4bc1c449, 93 },
    { 0x6321c44c, 68 },
    { 0x7c71c44c, 61 },
    { 0x4751c44a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f51c458, 76 },
    { 0, 0 },
    { 0xffa1c45a, 91 },
    { 0x2191c45b, 55 },
    { 0x8361c45a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec11c463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931c469, 90 },
    { 0x30d1c469, 69 },
    { 0x63f1c469, 98 },
    { 0x29c1c46c, 34 },
    { 0x1cd1c46d, 76 },
    { 0, 0 },
    { 0x42e1c46f, 77 },
    { 0, 0 },
    { 0xa511c471, 76 },
    { 0, 0 },
    { 0xeb11c473, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf141c479, 55 },
    { 0, 0 },
    { 0x91d1c47b, 43 },
    { 0x7a91c47c, 38 },
    { 0, 0 },
    { 0x2811c47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e1c488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7861c495, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c1c49b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1c49e, 33 },
    { 0x4de1c49e, 66 },
    { 0x2cb1c4a0, 84 },
    { 0x34f1c4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf661c4ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc151c4b1, 78 },
    { 0, 0 },
    { 0x46a1c4b3, 61 },
    { 0xaa51c4b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c4bb, 34 },
    { 0x7961c4bb, 67 },
    { 0xda81c4bc, 90 },
    { 0xb431c4be, 61 },
    { 0x3221c4be, 43 },
    { 0, 0 },
    { 0xbd21c4c1, 90 },
    { 0, 0 },
    { 0xc491c4c3, 48 },
    { 0xdf1c4c4, 98 },
    { 0xaf81c4c4, 90 },
    { 0xd151c4c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1c4cb, 61 },
    { 0x5631c4cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111c4d0, 33 },
    { 0x2661c4d1, 98 },
    { 0x52e1c4d2, 98 },
    { 0x6621c4d1, 52 },
    { 0x28e1c4d4, 99 },
    { 0x9f1c4d2, 84 },
    { 0xf8d1c4d1, 33 },
    { 0x9cd1c4d2, 98 },
    { 0x7c51c4d5, 99 },
    { 0xf131c4d9, 33 },
    { 0x521c4d7, 55 },
    { 0xc9d1c4d3, 33 },
    { 0xf041c4db, 61 },
    { 0, 0 },
    { 0xb6e1c4de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb51c4e1, 95 },
    { 0, 0 },
    { 0x89c1c4e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa151c4e8, 43 },
    { 0, 0 },
    { 0x72c1c4ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5161c4f0, 55 },
    { 0xc501c4f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771c4f4, 48 },
    { 0, 0 },
    { 0x16e1c4f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e31c4fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe121c4fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa61c503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021c506, 8 },
    { 0x6321c507, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f1c50b, 33 },
    { 0xcf61c50c, 23 },
    { 0xfb91c50c, 34 },
    { 0, 0 },
    { 0x1481c50f, 34 },
    { 0, 0 },
    { 0xac31c511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a1c514, 84 },
    { 0xe3a1c515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651c51b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf571c51f, 23 },
    { 0, 0 },
    { 0xa4e1c521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71c524, 34 },
    { 0xe221c525, 52 },
    { 0xadc1c526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071c52b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c1c534, 99 },
    { 0xe71c535, 46 },
    { 0xadc1c534, 98 },
    { 0x85b1c537, 33 },
    { 0x22a1c537, 89 },
    { 0x7ff1c537, 19 },
    { 0x7721c536, 98 },
    { 0, 0 },
    { 0x4cd1c53c, 76 },
    { 0, 0 },
    { 0xa7d1c53e, 33 },
    { 0x8cf1c53f, 91 },
    { 0, 0 },
    { 0xd081c541, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861c544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1c548, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a1c54f, 33 },
    { 0xc601c54f, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121c555, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21c55d, 66 },
    { 0xc5b1c55d, 33 },
    { 0, 0 },
    { 0x6241c560, 93 },
    { 0xade1c560, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6861c56a, 43 },
    { 0x3131c56a, 23 },
    { 0x2ce1c56a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3111c574, 33 },
    { 0xf6e1c575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3041c578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e1c57b, 90 },
    { 0, 0 },
    { 0x38d1c57d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551c580, 91 },
    { 0xa711c581, 34 },
    { 0, 0 },
    { 0x3271c583, 76 },
    { 0, 0 },
    { 0xc881c585, 93 },
    { 0xf471c585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631c58a, 61 },
    { 0x9451c58a, 95 },
    { 0x42d1c58c, 90 },
    { 0xf081c58d, 43 },
    { 0x6031c58e, 33 },
    { 0xc2c1c58f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931c594, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd291c5a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b61c5a7, 93 },
    { 0, 0 },
    { 0x6ed1c5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b31c5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5841c5b3, 77 },
    { 0, 0 },
    { 0x4301c5b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71c5ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb1c5bf, 66 },
    { 0x40a1c5c0, 101 },
    { 0x4581c5c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321c5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6681c5cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1c5ce, 85 },
    { 0x4d51c5ce, 34 },
    { 0xf7a1c5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6521c5d4, 17 },
    { 0x4ff1c5d5, 98 },
    { 0xc161c5d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4221c5d9, 90 },
    { 0, 0 },
    { 0x2791c5db, 76 },
    { 0xe9f1c5dc, 96 },
    { 0x6391c5dc, 76 },
    { 0xfb41c5dc, 90 },
    { 0xecb1c5df, 3 },
    { 0x21c5df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa441c5ee, 98 },
    { 0, 0 },
    { 0x7b1c5f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1c5f4, 78 },
    { 0x8c31c5f5, 23 },
    { 0x2c81c5f5, 66 },
    { 0x6421c5f7, 95 },
    { 0xc421c5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211c5fb, 23 },
    { 0x28b1c5fc, 95 },
    { 0xc301c5fd, 23 },
    { 0xbbf1c5fd, 55 },
    { 0xd251c5fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a41c602, 85 },
    { 0xe931c603, 45 },
    { 0xbf11c602, 98 },
    { 0xcab1c604, 33 },
    { 0x12e1c606, 100 },
    { 0xf921c607, 91 },
    { 0x6661c608, 23 },
    { 0x8b31c609, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c21c60d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e31c613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361c617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead1c61b, 95 },
    { 0, 0 },
    { 0xa191c61d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c61c622, 33 },
    { 0xfc61c622, 61 },
    { 0x2041c624, 100 },
    { 0x3a41c625, 55 },
    { 0, 0 },
    { 0x1c11c627, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391c62f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321c639, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41c63d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a1c640, 93 },
    { 0x4c21c641, 61 },
    { 0x7a01c642, 52 },
    { 0x7ee1c642, 23 },
    { 0, 0 },
    { 0x1f61c645, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5541c648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc221c64e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2691c652, 40 },
    { 0, 0 },
    { 0xc471c654, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc1c659, 43 },
    { 0xa4f1c65a, 43 },
    { 0xbde1c65b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391c661, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5141c667, 90 },
    { 0x7071c668, 77 },
    { 0x431c667, 76 },
    { 0xfc91c668, 34 },
    { 0x40b1c66b, 46 },
    { 0xbbc1c668, 75 },
    { 0xa841c66d, 98 },
    { 0, 0 },
    { 0x5201c66f, 23 },
    { 0x7281c66f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb041c675, 33 },
    { 0x5db1c676, 100 },
    { 0x2731c677, 78 },
    { 0x51c1c677, 41 },
    { 0x5d51c675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec01c67d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2531c688, 66 },
    { 0, 0 },
    { 0x3371c68a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1c690, 33 },
    { 0xb461c691, 95 },
    { 0, 0 },
    { 0xc751c693, 88 },
    { 0x8111c693, 23 },
    { 0xba11c693, 76 },
    { 0x63d1c696, 55 },
    { 0x5d61c697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa181c69b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c91c69e, 98 },
    { 0x1c41c69f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd1c6a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21c6a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a81c6ad, 61 },
    { 0xf431c6ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb1c6b1, 90 },
    { 0x7ea1c6b2, 98 },
    { 0x92a1c6b2, 90 },
    { 0x2e41c6b4, 33 },
    { 0x6741c6b5, 98 },
    { 0xdd31c6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11c6ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981c6c5, 34 },
    { 0, 0 },
    { 0x5281c6c7, 23 },
    { 0x5191c6c8, 95 },
    { 0x99f1c6c7, 65 },
    { 0, 0 },
    { 0xd331c6cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a51c6ce, 61 },
    { 0xfc41c6cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb1c6d2, 61 },
    { 0xbd01c6d3, 95 },
    { 0x6671c6d3, 34 },
    { 0x5251c6d5, 78 },
    { 0x7141c6d6, 90 },
    { 0x32d1c6d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f61c6da, 90 },
    { 0, 0 },
    { 0xaf21c6dc, 43 },
    { 0x36f1c6dd, 61 },
    { 0xe31c6de, 94 },
    { 0, 0 },
    { 0x7fd1c6e0, 23 },
    { 0xbea1c6e0, 70 },
    { 0x52f1c6e0, 87 },
    { 0, 0 },
    { 0x9e41c6e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5411c6e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1c6eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c6ef, 33 },
    { 0x2ac1c6f0, 93 },
    { 0x6331c6ef, 67 },
    { 0, 0 },
    { 0x321c6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1931c700, 95 },
    { 0x8d01c700, 95 },
    { 0xa351c702, 23 },
    { 0, 0 },
    { 0xf4e1c704, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8321c707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d91c70c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd1c70f, 90 },
    { 0x6e71c70f, 33 },
    { 0xe551c70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd371c71a, 93 },
    { 0x7291c71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591c71f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce1c724, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea1c729, 23 },
    { 0xaec1c72a, 98 },
    { 0xda11c72b, 23 },
    { 0xe7f1c72b, 76 },
    { 0xc831c72a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f51c730, 23 },
    { 0, 0 },
    { 0x4481c732, 90 },
    { 0x53f1c733, 98 },
    { 0xa711c734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef71c738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc131c73c, 80 },
    { 0x97c1c73d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1c741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf1c744, 76 },
    { 0x7ef1c744, 61 },
    { 0xbb01c744, 34 },
    { 0x3791c744, 33 },
    { 0x2d71c748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e81c74d, 52 },
    { 0xfc41c74e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b1c752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a1c755, 90 },
    { 0x4a81c756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5571c75a, 95 },
    { 0xed1c75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61c75e, 100 },
    { 0x7381c75f, 95 },
    { 0x7901c75f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651c763, 90 },
    { 0x8571c764, 35 },
    { 0x6d51c765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b1c768, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d41c76c, 90 },
    { 0xaa51c76d, 90 },
    { 0, 0 },
    { 0x6821c76f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2461c775, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1861c779, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd1c781, 95 },
    { 0, 0 },
    { 0x7fd1c783, 61 },
    { 0, 0 },
    { 0x8191c785, 61 },
    { 0xde61c786, 90 },
    { 0, 0 },
    { 0x8fb1c788, 66 },
    { 0, 0 },
    { 0xbc51c78a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7901c790, 90 },
    { 0, 0 },
    { 0x2291c792, 77 },
    { 0x3171c793, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1c798, 61 },
    { 0x2871c798, 55 },
    { 0x7f01c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5781c7a2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf061c7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1c7a9, 33 },
    { 0, 0 },
    { 0xe81c7ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111c7ae, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e1c7b4, 93 },
    { 0x21c1c7b4, 52 },
    { 0xf2f1c7b6, 83 },
    { 0x5581c7b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2461c7bb, 91 },
    { 0xfc61c7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c31c7c1, 77 },
    { 0x2351c7c2, 78 },
    { 0x28e1c7c2, 52 },
    { 0x8d61c7c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a1c7c7, 33 },
    { 0x1c7c8, 90 },
    { 0x5381c7c9, 61 },
    { 0x1761c7ca, 90 },
    { 0x61b1c7cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111c7ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff1c7d7, 93 },
    { 0, 0 },
    { 0x7141c7d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61c7dc, 78 },
    { 0, 0 },
    { 0x13c1c7de, 94 },
    { 0, 0 },
    { 0x6ea1c7e0, 90 },
    { 0x7b61c7e1, 55 },
    { 0xac1c7e0, 33 },
    { 0xb921c7e2, 95 },
    { 0x7351c7e1, 85 },
    { 0x6a31c7e1, 77 },
    { 0, 0 },
    { 0x7071c7e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb1c7ec, 93 },
    { 0, 0 },
    { 0xd761c7ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7761c7f2, 51 },
    { 0x34f1c7f3, 23 },
    { 0xe761c7f4, 98 },
    { 0, 0 },
    { 0x93e1c7f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef1c7fa, 95 },
    { 0xec11c7fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c21c7fe, 23 },
    { 0xaed1c7ff, 90 },
    { 0x1ae1c800, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671c803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8201c807, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f1c80c, 90 },
    { 0x7641c80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf1c815, 95 },
    { 0x76e1c816, 95 },
    { 0, 0 },
    { 0xe361c818, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21c81b, 88 },
    { 0, 0 },
    { 0x9401c81d, 23 },
    { 0xd4e1c81d, 83 },
    { 0, 0 },
    { 0x9131c820, 77 },
    { 0x2091c820, 96 },
    { 0, 0 },
    { 0xdb81c823, 46 },
    { 0xb731c823, 90 },
    { 0x4e71c825, 33 },
    { 0xc921c825, 90 },
    { 0, 0 },
    { 0xca31c828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd21c82d, 76 },
    { 0xb121c82e, 90 },
    { 0, 0 },
    { 0x2471c830, 76 },
    { 0, 0 },
    { 0xb4b1c832, 55 },
    { 0xecb1c833, 33 },
    { 0xf911c834, 95 },
    { 0, 0 },
    { 0xc481c836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161c83a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f1c83e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4441c843, 95 },
    { 0x3c11c844, 76 },
    { 0x9681c845, 33 },
    { 0x9661c845, 90 },
    { 0x3261c846, 78 },
    { 0x5fc1c846, 98 },
    { 0xc651c844, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e1c850, 23 },
    { 0xe21c851, 95 },
    { 0xf311c850, 90 },
    { 0xc851c852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891c858, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c11c863, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb451c866, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691c86b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951c872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb1c876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841c87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf581c87f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571c883, 55 },
    { 0xaf91c884, 33 },
    { 0x1531c884, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e1c88b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc321c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec1c892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd461c89a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f41c89d, 85 },
    { 0x9941c89d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371c8a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c8a6, 61 },
    { 0x32c1c8a6, 61 },
    { 0, 0 },
    { 0x50c1c8a9, 76 },
    { 0x57f1c8a9, 55 },
    { 0x5071c8aa, 95 },
    { 0x6a31c8ab, 61 },
    { 0x5fd1c8ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2621c8b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1651c8b6, 98 },
    { 0x5d01c8b7, 98 },
    { 0, 0 },
    { 0x3ef1c8b9, 77 },
    { 0x7831c8ba, 95 },
    { 0x8ee1c8ba, 100 },
    { 0xea91c8bc, 45 },
    { 0xb1a1c8bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e71c8c2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c1c8c6, 95 },
    { 0x18b1c8c7, 78 },
    { 0x2b81c8c7, 23 },
    { 0x3d41c8c9, 80 },
    { 0x8f11c8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c31c8cd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041c8d3, 90 },
    { 0xbc71c8d4, 33 },
    { 0x1d01c8d3, 90 },
    { 0xf771c8d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041c8da, 90 },
    { 0x7c61c8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f1c8de, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d1c8e4, 68 },
    { 0, 0 },
    { 0x1281c8e6, 91 },
    { 0xc531c8e6, 33 },
    { 0x9c61c8e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf01c8ef, 43 },
    { 0x4791c8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf1c8f4, 33 },
    { 0, 0 },
    { 0x1881c8f6, 100 },
    { 0xe201c8f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd1c900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c61c90d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd381c915, 33 },
    { 0, 0 },
    { 0x7af1c917, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e01c91b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a1c91f, 33 },
    { 0, 0 },
    { 0xa821c921, 90 },
    { 0x8f01c922, 43 },
    { 0x9601c923, 95 },
    { 0xabb1c924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1c927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea1c92a, 23 },
    { 0x9a61c92a, 23 },
    { 0, 0 },
    { 0x31f1c92d, 76 },
    { 0, 0 },
    { 0x34f1c92f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c1c932, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a1c935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac1c93a, 33 },
    { 0, 0 },
    { 0x60f1c93c, 23 },
    { 0x5211c93c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231c948, 33 },
    { 0xcc81c949, 95 },
    { 0xcc01c94a, 36 },
    { 0x6af1c94b, 46 },
    { 0, 0 },
    { 0x9901c94d, 84 },
    { 0, 0 },
    { 0x8631c94f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1c953, 90 },
    { 0x4711c954, 67 },
    { 0xc8c1c955, 91 },
    { 0x3d61c956, 46 },
    { 0xb001c956, 43 },
    { 0x6101c954, 33 },
    { 0xd101c959, 61 },
    { 0xf6f1c953, 52 },
    { 0xffc1c958, 17 },
    { 0x6d1c95c, 36 },
    { 0, 0 },
    { 0xf271c95e, 61 },
    { 0x7fe1c95f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x261c962, 88 },
    { 0xc3c1c963, 35 },
    { 0x6b81c962, 55 },
    { 0x90f1c965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e11c969, 98 },
    { 0x9741c969, 90 },
    { 0, 0 },
    { 0xb981c96c, 33 },
    { 0x2b91c96c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7551c977, 61 },
    { 0, 0 },
    { 0x5131c979, 61 },
    { 0, 0 },
    { 0x9ed1c97b, 98 },
    { 0x35c1c97b, 90 },
    { 0, 0 },
    { 0xa621c97e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7531c983, 34 },
    { 0, 0 },
    { 0xecd1c985, 98 },
    { 0, 0 },
    { 0x14e1c987, 91 },
    { 0, 0 },
    { 0xb4b1c989, 98 },
    { 0xa8e1c989, 95 },
    { 0xabc1c98a, 99 },
    { 0x2671c98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091c99c, 95 },
    { 0x99b1c99d, 23 },
    { 0x3d11c99d, 61 },
    { 0xccb1c99f, 23 },
    { 0x3281c99f, 61 },
    { 0, 0 },
    { 0x93d1c9a2, 23 },
    { 0, 0 },
    { 0x6fc1c9a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d81c9ad, 43 },
    { 0xa961c9ae, 33 },
    { 0, 0 },
    { 0x51c1c9b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce61c9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61c9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1c9c0, 61 },
    { 0x4331c9c0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7291c9c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331c9c7, 46 },
    { 0xbb31c9c8, 95 },
    { 0x8bb1c9c9, 43 },
    { 0x7841c9c7, 76 },
    { 0x9c21c9cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed1c9d0, 81 },
    { 0x1ae1c9d0, 23 },
    { 0xa781c9d2, 84 },
    { 0xf161c9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61c9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1c9dd, 43 },
    { 0, 0 },
    { 0xcfe1c9df, 61 },
    { 0, 0 },
    { 0xf001c9e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc791c9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce1c9e7, 33 },
    { 0xcfe1c9e8, 84 },
    { 0xa2a1c9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7471c9ec, 99 },
    { 0x22a1c9ed, 76 },
    { 0xdf31c9ed, 61 },
    { 0xa921c9ee, 96 },
    { 0xbd11c9f0, 33 },
    { 0xbcf1c9f1, 52 },
    { 0xd241c9f1, 61 },
    { 0, 0 },
    { 0x1531c9f4, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1c9f7, 95 },
    { 0x9d1c9f8, 61 },
    { 0x6e1c9f7, 95 },
    { 0, 0 },
    { 0x2401c9fb, 61 },
    { 0x3ba1c9fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f1ca02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4441ca05, 61 },
    { 0xbff1ca06, 55 },
    { 0xb7f1ca05, 90 },
    { 0x87f1ca08, 67 },
    { 0x3351ca07, 78 },
    { 0xc8c1ca08, 34 },
    { 0, 0 },
    { 0x3cb1ca0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f31ca15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb51ca19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681ca1d, 88 },
    { 0x3281ca1d, 34 },
    { 0x8b41ca1f, 95 },
    { 0x9761ca1e, 95 },
    { 0xebf1ca1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e1ca24, 95 },
    { 0x7091ca25, 76 },
    { 0x52a1ca24, 35 },
    { 0x8f91ca27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5611ca2a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7201ca2e, 55 },
    { 0, 0 },
    { 0x15f1ca30, 85 },
    { 0x9411ca31, 43 },
    { 0x7321ca31, 98 },
    { 0xb531ca33, 85 },
    { 0x9f31ca34, 88 },
    { 0x3191ca35, 66 },
    { 0x2fd1ca36, 98 },
    { 0x7c31ca34, 84 },
    { 0xca71ca38, 76 },
    { 0x941ca38, 90 },
    { 0x5c41ca39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c1ca40, 55 },
    { 0, 0 },
    { 0x4361ca42, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111ca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca1ca48, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ca4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5071ca4f, 95 },
    { 0, 0 },
    { 0xbe01ca51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca1ca54, 33 },
    { 0xf51ca55, 23 },
    { 0xa311ca55, 76 },
    { 0x3261ca56, 87 },
    { 0xb221ca58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf791ca5b, 65 },
    { 0x2b81ca5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd11ca61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b91ca65, 43 },
    { 0, 0 },
    { 0x14f1ca67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb341ca6a, 95 },
    { 0x4c61ca6b, 46 },
    { 0x1291ca6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31ca70, 23 },
    { 0xf7f1ca71, 33 },
    { 0xa081ca72, 98 },
    { 0x6ce1ca73, 52 },
    { 0x66a1ca71, 55 },
    { 0x3f71ca75, 55 },
    { 0xa6d1ca74, 76 },
    { 0x6e71ca77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1ca7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e1ca7d, 34 },
    { 0xe691ca7e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a1ca86, 61 },
    { 0, 0 },
    { 0x6f61ca88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1ca8c, 95 },
    { 0x3a01ca8d, 33 },
    { 0x8e71ca8d, 95 },
    { 0x4dd1ca8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae1ca94, 97 },
    { 0, 0 },
    { 0x2801ca96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41ca99, 23 },
    { 0xa111ca99, 68 },
    { 0, 0 },
    { 0xb8c1ca9c, 98 },
    { 0, 0 },
    { 0xad11ca9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4501caa2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a1caa5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031caab, 33 },
    { 0, 0 },
    { 0xe8d1caad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda1cab6, 95 },
    { 0xbef1cab6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf611cabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e1cac0, 61 },
    { 0, 0 },
    { 0xfa41cac2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da1cac9, 55 },
    { 0x8081cac9, 98 },
    { 0x5c21cacb, 76 },
    { 0, 0 },
    { 0xc851cacd, 93 },
    { 0x3381cace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b71cad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d91cad9, 76 },
    { 0x9441cada, 88 },
    { 0xb6f1cad9, 94 },
    { 0x40e1cad9, 98 },
    { 0x31c1cadd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1cae0, 61 },
    { 0, 0 },
    { 0x47c1cae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1cae7, 33 },
    { 0xf441cae8, 93 },
    { 0xe6e1cae8, 23 },
    { 0xd081cae7, 55 },
    { 0x88e1cae7, 90 },
    { 0, 0 },
    { 0x9b61caed, 93 },
    { 0, 0 },
    { 0x3c51caef, 46 },
    { 0, 0 },
    { 0xb6f1caf1, 46 },
    { 0xac1caf1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1caf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3381cafb, 33 },
    { 0x7961cafc, 23 },
    { 0xc191cafd, 100 },
    { 0xbff1cafb, 34 },
    { 0xd1d1cafe, 98 },
    { 0xe6f1cafe, 33 },
    { 0, 0 },
    { 0x6fe1cb02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461cb05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71cb0c, 48 },
    { 0x1941cb0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5201cb12, 76 },
    { 0xdc71cb13, 90 },
    { 0, 0 },
    { 0xd371cb15, 23 },
    { 0xd411cb15, 88 },
    { 0xd401cb17, 55 },
    { 0, 0 },
    { 0x3181cb19, 99 },
    { 0x981cb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc051cb1f, 99 },
    { 0x5381cb20, 61 },
    { 0xa031cb20, 23 },
    { 0x1d1cb20, 95 },
    { 0x3ff1cb23, 31 },
    { 0xda11cb24, 23 },
    { 0xdc61cb23, 33 },
    { 0xe511cb26, 90 },
    { 0xf2e1cb27, 33 },
    { 0x2c81cb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031cb2b, 76 },
    { 0xdac1cb2b, 88 },
    { 0xf7f1cb2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81cb30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b1cb33, 33 },
    { 0xe651cb33, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd311cb3b, 98 },
    { 0x28b1cb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e1cb3f, 100 },
    { 0, 0 },
    { 0x5e21cb41, 46 },
    { 0xc231cb42, 55 },
    { 0xcc91cb42, 23 },
    { 0, 0 },
    { 0x6e61cb45, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c71cb48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a1cb4b, 68 },
    { 0xe6d1cb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21cb4f, 67 },
    { 0, 0 },
    { 0x3ba1cb51, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1cb54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6051cb57, 23 },
    { 0x59d1cb58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01cb5c, 61 },
    { 0xfb21cb5d, 33 },
    { 0x55e1cb5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed1cb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e1cb67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521cb6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1cb70, 33 },
    { 0x7891cb71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f01cb76, 93 },
    { 0x1ec1cb77, 98 },
    { 0x12f1cb78, 75 },
    { 0, 0 },
    { 0xdd81cb7a, 93 },
    { 0xaac1cb7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd151cb80, 91 },
    { 0xe801cb80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7341cb88, 77 },
    { 0x27d1cb89, 33 },
    { 0, 0 },
    { 0xddb1cb8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631cb8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f1cb92, 76 },
    { 0x3cd1cb93, 80 },
    { 0x3e81cb93, 66 },
    { 0, 0 },
    { 0x9211cb96, 98 },
    { 0, 0 },
    { 0x69b1cb98, 33 },
    { 0x8bf1cb99, 78 },
    { 0x34d1cb9a, 95 },
    { 0, 0 },
    { 0x2871cb9c, 61 },
    { 0x4331cb9d, 61 },
    { 0, 0 },
    { 0xdd61cb9f, 46 },
    { 0, 0 },
    { 0x8cb1cba1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31cba6, 95 },
    { 0x5971cba6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a41cbb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331cbb6, 91 },
    { 0x5c81cbb7, 46 },
    { 0xf7c1cbb8, 34 },
    { 0, 0 },
    { 0xe8a1cbba, 98 },
    { 0x62b1cbbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c1cbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e1cbc2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f1cbcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1cbce, 23 },
    { 0x1571cbce, 33 },
    { 0x3081cbd0, 90 },
    { 0x19c1cbd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1cbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b1cbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1cbdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a1cbe2, 17 },
    { 0, 0 },
    { 0x3bb1cbe4, 23 },
    { 0x97b1cbe4, 43 },
    { 0, 0 },
    { 0x2f01cbe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5321cbec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11cbf1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c1cbf5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251cbf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f1cbff, 40 },
    { 0x5f91cc00, 95 },
    { 0, 0 },
    { 0xec01cc02, 62 },
    { 0x5f11cc03, 23 },
    { 0x9c91cc04, 88 },
    { 0x6311cc05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1811cc0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891cc15, 43 },
    { 0x7801cc15, 17 },
    { 0, 0 },
    { 0x7961cc18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b61cc1c, 61 },
    { 0x2f41cc1c, 33 },
    { 0x2dc1cc1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda1cc22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd1cc25, 76 },
    { 0, 0 },
    { 0x9041cc27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a1cc2e, 95 },
    { 0, 0 },
    { 0x45a1cc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3081cc35, 96 },
    { 0x7601cc36, 90 },
    { 0xe81cc37, 75 },
    { 0xfd41cc38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4101cc3b, 61 },
    { 0, 0 },
    { 0xe611cc3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xff91cc40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8811cc46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe1cc4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1661cc51, 33 },
    { 0, 0 },
    { 0xab91cc53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1cc5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1861cc5e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x7831cc61, 87 },
    { 0x42d1cc61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b1cc66, 84 },
    { 0xab11cc67, 33 },
    { 0x43a1cc68, 61 },
    { 0, 0 },
    { 0x91f1cc6a, 76 },
    { 0, 0 },
    { 0xd681cc6c, 61 },
    { 0x4231cc6d, 46 },
    { 0xbbd1cc6d, 23 },
    { 0x9d81cc6f, 76 },
    { 0x7691cc70, 98 },
    { 0x1ba1cc6e, 83 },
    { 0xd3a1cc6e, 34 },
    { 0xec11cc72, 95 },
    { 0x6041cc74, 34 },
    { 0x2801cc75, 100 },
    { 0x7381cc75, 55 },
    { 0x47e1cc77, 76 },
    { 0x6e61cc77, 55 },
    { 0x7661cc79, 65 },
    { 0xf5e1cc76, 33 },
    { 0x9e81cc74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf421cc7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb161cc82, 66 },
    { 0, 0 },
    { 0xe121cc84, 98 },
    { 0, 0 },
    { 0xaac1cc86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc41cc8c, 33 },
    { 0, 0 },
    { 0x261cc8e, 90 },
    { 0xef51cc8f, 87 },
    { 0, 0 },
    { 0xc271cc91, 76 },
    { 0x8a31cc92, 43 },
    { 0x6321cc93, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461cc99, 90 },
    { 0x6cb1cc9a, 76 },
    { 0x5181cc9a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7911cc9e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e1cca5, 55 },
    { 0xd571cca6, 61 },
    { 0, 0 },
    { 0xd4d1cca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9461ccab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc431ccb2, 93 },
    { 0xb8e1ccb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce1ccb7, 95 },
    { 0x61d1ccb8, 66 },
    { 0x4d61ccb9, 91 },
    { 0x2481ccba, 61 },
    { 0, 0 },
    { 0x56f1ccbc, 23 },
    { 0x9c71ccbc, 23 },
    { 0, 0 },
    { 0x2131ccbf, 40 },
    { 0xb761ccc0, 61 },
    { 0, 0 },
    { 0x8251ccc2, 87 },
    { 0xcbc1ccc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601ccc6, 34 },
    { 0xd1a1ccc6, 33 },
    { 0xe241ccc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e1cccc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de1ccd3, 93 },
    { 0x8111ccd4, 98 },
    { 0xf661ccd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4541ccd9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a71ccec, 33 },
    { 0, 0 },
    { 0x24a1ccee, 55 },
    { 0x1c51ccef, 53 },
    { 0x64c1ccee, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc1ccf6, 33 },
    { 0xe071ccf6, 17 },
    { 0x3791ccf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7381ccfb, 23 },
    { 0, 0 },
    { 0x541ccfd, 76 },
    { 0, 0 },
    { 0x9d91ccff, 65 },
    { 0x8771cd00, 61 },
    { 0xa3d1ccff, 55 },
    { 0, 0 },
    { 0x171cd03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71cd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4741cd09, 34 },
    { 0x2c51cd0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1cd0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901cd10, 65 },
    { 0xa1f1cd11, 32 },
    { 0, 0 },
    { 0x2981cd13, 90 },
    { 0x48a1cd13, 40 },
    { 0, 0 },
    { 0x5d11cd16, 23 },
    { 0, 0 },
    { 0x8c01cd18, 76 },
    { 0, 0 },
    { 0xae71cd1a, 88 },
    { 0x5f1cd1b, 90 },
    { 0x6ba1cd1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed51cd26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3131cd2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51cd2d, 23 },
    { 0, 0 },
    { 0x4401cd2f, 52 },
    { 0, 0 },
    { 0xc251cd31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e1cd3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd51cd3f, 17 },
    { 0xb9e1cd3f, 100 },
    { 0xbe41cd40, 67 },
    { 0x3871cd42, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cd48, 90 },
    { 0x4de1cd48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d1cd4e, 55 },
    { 0, 0 },
    { 0x8421cd50, 68 },
    { 0, 0 },
    { 0x7511cd52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f1cd56, 33 },
    { 0x7011cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811cd5b, 61 },
    { 0, 0 },
    { 0x6ee1cd5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f1cd60, 76 },
    { 0, 0 },
    { 0xa051cd62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d61cd65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421cd69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d1cd6d, 76 },
    { 0xd01cd6e, 34 },
    { 0, 0 },
    { 0xc991cd70, 23 },
    { 0x9a01cd71, 23 },
    { 0xe2f1cd71, 90 },
    { 0, 0 },
    { 0xa981cd74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11cd7d, 33 },
    { 0xfe21cd7d, 81 },
    { 0, 0 },
    { 0x241cd80, 91 },
    { 0xfbd1cd80, 90 },
    { 0, 0 },
    { 0xf461cd83, 90 },
    { 0xa2f1cd83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341cd88, 85 },
    { 0x41f1cd89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c01cd90, 90 },
    { 0xe491cd91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61cd96, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a1cd9c, 65 },
    { 0x9421cd9d, 77 },
    { 0, 0 },
    { 0x88f1cd9f, 90 },
    { 0xde31cda0, 33 },
    { 0x5b01cda1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1cda4, 69 },
    { 0, 0 },
    { 0xf6a1cda6, 52 },
    { 0, 0 },
    { 0x9af1cda8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c1cdac, 78 },
    { 0xabe1cdad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf601cdb1, 34 },
    { 0x9571cdb2, 95 },
    { 0xf6e1cdb3, 93 },
    { 0, 0 },
    { 0xc11cdb5, 55 },
    { 0, 0 },
    { 0x16c1cdb7, 76 },
    { 0, 0 },
    { 0x3b71cdb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb661cdc0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff21cdcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d1cdcf, 77 },
    { 0x6a31cdd0, 33 },
    { 0xe791cdd1, 98 },
    { 0, 0 },
    { 0xe951cdd3, 33 },
    { 0x13a1cdd4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f41cdd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1cddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6081cde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3401cde6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3621cde9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e1cded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a1cdff, 100 },
    { 0x2d31ce00, 61 },
    { 0xeb51ce00, 61 },
    { 0x5a81ce02, 23 },
    { 0, 0 },
    { 0xe591ce04, 84 },
    { 0x6b11ce05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f1ce09, 76 },
    { 0x7521ce0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe431ce10, 90 },
    { 0x4d41ce10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501ce14, 90 },
    { 0, 0 },
    { 0x3a91ce16, 98 },
    { 0, 0 },
    { 0xcb31ce18, 34 },
    { 0, 0 },
    { 0xcbb1ce1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc81ce21, 98 },
    { 0, 0 },
    { 0xfdf1ce23, 87 },
    { 0, 0 },
    { 0x5641ce25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1371ce28, 33 },
    { 0, 0 },
    { 0x5b71ce2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e1ce31, 95 },
    { 0x9e21ce32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c1ce39, 61 },
    { 0, 0 },
    { 0x1361ce3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde41ce3e, 23 },
    { 0x5e51ce3f, 85 },
    { 0xd351ce40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e1ce43, 51 },
    { 0xa521ce43, 100 },
    { 0, 0 },
    { 0x3e81ce46, 55 },
    { 0, 0 },
    { 0x9c31ce48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5111ce4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x961ce50, 95 },
    { 0x1d71ce51, 93 },
    { 0, 0 },
    { 0x2dd1ce53, 34 },
    { 0xbe41ce54, 98 },
    { 0xafa1ce53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1ce5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b1ce61, 52 },
    { 0x3591ce62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f1ce67, 98 },
    { 0xba11ce67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c1ce6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d61ce78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d1ce7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1ce84, 95 },
    { 0x1d01ce85, 95 },
    { 0x13d1ce86, 33 },
    { 0, 0 },
    { 0xec01ce88, 23 },
    { 0x59a1ce89, 40 },
    { 0, 0 },
    { 0x7b51ce8b, 90 },
    { 0xe2c1ce8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae41ce97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf51ce9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11ce9e, 90 },
    { 0x98f1ce9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a1cea5, 98 },
    { 0x8191cea6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861cea9, 17 },
    { 0xef41cea9, 91 },
    { 0, 0 },
    { 0xe8d1ceac, 84 },
    { 0x5141cead, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e91ceb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b91ceb6, 91 },
    { 0xc0c1ceb7, 23 },
    { 0, 0 },
    { 0xdbf1ceb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4891cebe, 64 },
    { 0xf611cebf, 78 },
    { 0, 0 },
    { 0xf921cec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade1cec8, 48 },
    { 0, 0 },
    { 0x6251ceca, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a1ced1, 55 },
    { 0x181ced1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21ced8, 90 },
    { 0x6481ced8, 66 },
    { 0x4581ceda, 46 },
    { 0x5951ceda, 98 },
    { 0, 0 },
    { 0xd7f1cedd, 61 },
    { 0xf121cedd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db1cee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5281cee5, 90 },
    { 0, 0 },
    { 0x9f01cee7, 33 },
    { 0xf941cee8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071ceee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111cef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1cef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c01cefe, 90 },
    { 0x83d1ceff, 90 },
    { 0x2aa1cf00, 98 },
    { 0xc11cf00, 23 },
    { 0xba71ceff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c71cf06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe1cf11, 33 },
    { 0x1d41cf12, 61 },
    { 0xe321cf12, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71cf20, 95 },
    { 0x9641cf21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9861cf24, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c1cf2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1cf31, 88 },
    { 0x1401cf32, 34 },
    { 0xc341cf31, 98 },
    { 0x1781cf34, 93 },
    { 0x51f1cf35, 91 },
    { 0x29a1cf32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea1cf3b, 65 },
    { 0xa2b1cf3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf661cf41, 9 },
    { 0, 0 },
    { 0x5261cf43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a1cf47, 90 },
    { 0, 0 },
    { 0x1641cf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac01cf4c, 61 },
    { 0, 0 },
    { 0xfb11cf4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1691cf51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3131cf55, 99 },
    { 0, 0 },
    { 0x27d1cf57, 61 },
    { 0x20b1cf58, 33 },
    { 0xb791cf59, 90 },
    { 0, 0 },
    { 0x7401cf5b, 33 },
    { 0x2a01cf5c, 61 },
    { 0xe721cf5b, 76 },
    { 0x16e1cf5e, 88 },
    { 0x4171cf5f, 33 },
    { 0x5001cf60, 98 },
    { 0x7441cf61, 34 },
    { 0x9111cf61, 64 },
    { 0xe0b1cf62, 24 },
    { 0x8561cf5d, 34 },
    { 0, 0 },
    { 0xe881cf66, 90 },
    { 0, 0 },
    { 0x95d1cf68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a1cf6c, 61 },
    { 0xbf51cf6c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc881cf70, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc001cf73, 90 },
    { 0x8a1cf74, 55 },
    { 0, 0 },
    { 0x561cf76, 76 },
    { 0xca21cf77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae81cf7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d1cf81, 93 },
    { 0x1921cf82, 33 },
    { 0x5721cf81, 33 },
    { 0x43f1cf82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe281cf8d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8821cf93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e1cf96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9111cf9b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f1cfa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1cfab, 23 },
    { 0xee1cfac, 66 },
    { 0, 0 },
    { 0x80f1cfae, 90 },
    { 0xce81cfaf, 91 },
    { 0xe081cfb0, 46 },
    { 0xb191cfb1, 30 },
    { 0x8651cfb1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141cfbd, 61 },
    { 0, 0 },
    { 0xdc31cfbf, 43 },
    { 0, 0 },
    { 0x8e91cfc1, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5131cfc9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a1cfcd, 90 },
    { 0, 0 },
    { 0x8be1cfcf, 75 },
    { 0xfbf1cfd0, 33 },
    { 0x491cfd0, 34 },
    { 0x6671cfd2, 78 },
    { 0x5c51cfd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c1cfd9, 90 },
    { 0, 0 },
    { 0xe151cfdb, 66 },
    { 0, 0 },
    { 0x6bf1cfdd, 95 },
    { 0, 0 },
    { 0x8bc1cfdf, 67 },
    { 0x7b21cfdf, 90 },
    { 0x4bb1cfe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b1cfe7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf51cfeb, 61 },
    { 0x42b1cfec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d21cfef, 33 },
    { 0x5481cfef, 98 },
    { 0xd331cff1, 33 },
    { 0xc801cff2, 46 },
    { 0, 0 },
    { 0x171cff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91cff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d51cfff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e1d005, 65 },
    { 0, 0 },
    { 0xcc31d007, 90 },
    { 0xb701d008, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781d011, 55 },
    { 0, 0 },
    { 0xc401d013, 68 },
    { 0xf761d013, 98 },
    { 0xb2c1d015, 81 },
    { 0x9601d013, 98 },
    { 0x2801d017, 34 },
    { 0, 0 },
    { 0xaf01d019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1891d01d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1d020, 33 },
    { 0x601d020, 95 },
    { 0, 0 },
    { 0xf2f1d023, 95 },
    { 0xebf1d023, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb61d027, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc1d02b, 99 },
    { 0x9351d02c, 33 },
    { 0xc2b1d02d, 91 },
    { 0x3ba1d02d, 43 },
    { 0xa811d02b, 61 },
    { 0x3161d02e, 33 },
    { 0x5a51d031, 43 },
    { 0x64f1d032, 93 },
    { 0x1e61d02c, 78 },
    { 0, 0 },
    { 0xae1d035, 98 },
    { 0x6381d035, 90 },
    { 0x3521d035, 67 },
    { 0x40e1d038, 61 },
    { 0x681d039, 61 },
    { 0xc051d03a, 55 },
    { 0, 0 },
    { 0xe2b1d03c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa1d041, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf1d047, 34 },
    { 0, 0 },
    { 0xba31d049, 61 },
    { 0x1e61d04a, 90 },
    { 0xa121d04b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe41d054, 78 },
    { 0, 0 },
    { 0xa91d056, 95 },
    { 0xc241d057, 98 },
    { 0x6681d057, 33 },
    { 0x43d1d058, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf1d05c, 87 },
    { 0xf9d1d05d, 52 },
    { 0xc161d05d, 95 },
    { 0x81d05f, 76 },
    { 0xa31d060, 95 },
    { 0x701d05c, 75 },
    { 0x2b01d05f, 93 },
    { 0x831d063, 33 },
    { 0x3d21d05c, 61 },
    { 0, 0 },
    { 0xaba1d066, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc561d06e, 75 },
    { 0xf9e1d06f, 90 },
    { 0x6b1d06f, 78 },
    { 0xe3a1d071, 23 },
    { 0xf351d071, 34 },
    { 0xa981d073, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1d07a, 35 },
    { 0x6a41d07a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6581d07e, 66 },
    { 0x7991d07f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4421d082, 61 },
    { 0x9a71d083, 76 },
    { 0, 0 },
    { 0xf861d085, 61 },
    { 0x6101d086, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851d089, 78 },
    { 0xccd1d089, 61 },
    { 0xb0a1d08a, 98 },
    { 0x7291d08c, 33 },
    { 0x6fc1d08c, 90 },
    { 0x4d1d08e, 76 },
    { 0x8d71d08b, 98 },
    { 0x10d1d090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2161d093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021d097, 46 },
    { 0xd671d098, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f1d09b, 17 },
    { 0, 0 },
    { 0x2aa1d09d, 76 },
    { 0xd8b1d09e, 61 },
    { 0x18f1d09e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb1d0a2, 33 },
    { 0x8d51d0a3, 68 },
    { 0, 0 },
    { 0xa741d0a5, 33 },
    { 0, 0 },
    { 0x8791d0a7, 90 },
    { 0xe5a1d0a8, 33 },
    { 0x2f11d0a9, 66 },
    { 0x2621d0aa, 33 },
    { 0x59d1d0ab, 55 },
    { 0x87d1d0aa, 43 },
    { 0x7c11d0a9, 23 },
    { 0x77d1d0ae, 55 },
    { 0xffc1d0af, 95 },
    { 0xfa01d0aa, 23 },
    { 0x6851d0b1, 95 },
    { 0x4771d0b2, 76 },
    { 0x1a51d0b2, 23 },
    { 0xee11d0ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a1d0b7, 55 },
    { 0x89b1d0b7, 23 },
    { 0, 0 },
    { 0x6831d0ba, 61 },
    { 0x3611d0bb, 90 },
    { 0x1581d0ba, 66 },
    { 0, 0 },
    { 0xe9c1d0be, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71d0c1, 66 },
    { 0, 0 },
    { 0x4e41d0c3, 33 },
    { 0x8a81d0c4, 90 },
    { 0x9071d0c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d0c9, 51 },
    { 0, 0 },
    { 0xf761d0cb, 84 },
    { 0xfb31d0cb, 59 },
    { 0x3341d0cd, 91 },
    { 0xd71d0cc, 90 },
    { 0, 0 },
    { 0x5e1d0d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4471d0d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251d0d7, 87 },
    { 0x411d0d7, 23 },
    { 0, 0 },
    { 0x6cd1d0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6701d0e0, 95 },
    { 0x7811d0e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d31d0e5, 23 },
    { 0xf421d0e6, 90 },
    { 0x70c1d0e7, 33 },
    { 0xd751d0e7, 100 },
    { 0x6351d0e8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd1d0f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb1d0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871d0fc, 88 },
    { 0, 0 },
    { 0xa6d1d0fe, 94 },
    { 0xcd91d0ff, 90 },
    { 0x21d100, 91 },
    { 0xb571d100, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf741d107, 61 },
    { 0, 0 },
    { 0xdee1d109, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf851d10e, 93 },
    { 0xff31d10e, 84 },
    { 0x9e11d110, 83 },
    { 0x6c71d10f, 98 },
    { 0x7bc1d112, 90 },
    { 0x58a1d113, 17 },
    { 0xe21d112, 53 },
    { 0x6231d115, 68 },
    { 0x39c1d115, 75 },
    { 0x33d1d117, 78 },
    { 0x3dd1d117, 65 },
    { 0xcde1d117, 95 },
    { 0, 0 },
    { 0xcd51d11b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4931d11e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b11d122, 33 },
    { 0xbfe1d122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1d126, 98 },
    { 0x6671d127, 90 },
    { 0x6d1d127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4701d12d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f1d134, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5941d13c, 77 },
    { 0xfde1d13d, 55 },
    { 0xd3d1d13e, 87 },
    { 0x9351d13f, 99 },
    { 0, 0 },
    { 0x42f1d141, 76 },
    { 0xd191d142, 61 },
    { 0x3381d142, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e1d147, 90 },
    { 0x79b1d147, 96 },
    { 0xb7a1d149, 90 },
    { 0x311d14a, 95 },
    { 0x9ff1d14b, 98 },
    { 0xb971d14c, 75 },
    { 0x5361d14d, 33 },
    { 0x7471d14d, 95 },
    { 0x7571d14e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e71d152, 88 },
    { 0x23c1d153, 46 },
    { 0x34a1d153, 43 },
    { 0xf11d154, 95 },
    { 0x6041d156, 99 },
    { 0x4d21d157, 46 },
    { 0x95e1d158, 23 },
    { 0xe831d155, 61 },
    { 0, 0 },
    { 0xba1d15b, 34 },
    { 0, 0 },
    { 0x4561d15d, 33 },
    { 0xcca1d15d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91d163, 90 },
    { 0x63e1d164, 95 },
    { 0x10a1d165, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a1d16e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f21d177, 99 },
    { 0x8401d177, 81 },
    { 0x19b1d179, 61 },
    { 0x8b91d177, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c91d17f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd551d183, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb61d189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb411d191, 23 },
    { 0x6bd1d192, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d1d19c, 90 },
    { 0xe641d19d, 98 },
    { 0xb41d19e, 40 },
    { 0x2b61d19d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f81d1a5, 61 },
    { 0xc141d1a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac1d1a9, 98 },
    { 0x1eb1d1aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391d1ad, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a71d1b1, 95 },
    { 0, 0 },
    { 0xb8c1d1b3, 33 },
    { 0, 0 },
    { 0xcda1d1b5, 99 },
    { 0x6631d1b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac1d1b9, 61 },
    { 0xc71d1ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5351d1c2, 90 },
    { 0, 0 },
    { 0x21a1d1c4, 33 },
    { 0x4491d1c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5871d1cf, 17 },
    { 0x5041d1d0, 76 },
    { 0xf9b1d1cf, 90 },
    { 0x83e1d1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a1d1d5, 81 },
    { 0x2241d1d6, 98 },
    { 0x9a41d1d6, 23 },
    { 0x8041d1d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41d1e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc11d1ed, 46 },
    { 0, 0 },
    { 0xb0c1d1ef, 78 },
    { 0x1fa1d1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b1d1f4, 98 },
    { 0, 0 },
    { 0xefe1d1f6, 95 },
    { 0, 0 },
    { 0x8ac1d1f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd81d1fb, 23 },
    { 0xb911d1fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e41d201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8701d207, 38 },
    { 0, 0 },
    { 0xfde1d209, 81 },
    { 0x8551d209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b1d20e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5181d211, 76 },
    { 0xa3f1d212, 33 },
    { 0, 0 },
    { 0xd2d1d214, 95 },
    { 0xa451d215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de1d21c, 66 },
    { 0x1c31d21d, 34 },
    { 0, 0 },
    { 0x72f1d21f, 98 },
    { 0xc731d21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e1d223, 87 },
    { 0, 0 },
    { 0x3261d225, 61 },
    { 0x3ce1d225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611d232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f31d23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e01d241, 81 },
    { 0xa0f1d241, 34 },
    { 0xc7a1d243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8631d24d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71d251, 42 },
    { 0xa4d1d252, 93 },
    { 0x401d251, 34 },
    { 0, 0 },
    { 0xd91d255, 80 },
    { 0x6dd1d256, 23 },
    { 0, 0 },
    { 0x8631d258, 33 },
    { 0, 0 },
    { 0x5b01d25a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d1d25e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41d263, 33 },
    { 0x64b1d264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa01d267, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951d26a, 90 },
    { 0x1581d26b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121d270, 90 },
    { 0xe81d270, 90 },
    { 0x94e1d271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61d278, 34 },
    { 0xe2a1d279, 52 },
    { 0xe3f1d27a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6661d27f, 61 },
    { 0xfc01d280, 83 },
    { 0xae81d280, 23 },
    { 0x8561d282, 93 },
    { 0xc981d283, 76 },
    { 0x7171d281, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1d287, 93 },
    { 0, 0 },
    { 0xf401d289, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051d28f, 93 },
    { 0x9c01d28f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881d295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9001d29b, 33 },
    { 0xf201d29c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d29f, 61 },
    { 0x5571d29f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf11d2a7, 33 },
    { 0xe1c1d2a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b1d2ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2771d2b2, 33 },
    { 0, 0 },
    { 0x66d1d2b4, 76 },
    { 0, 0 },
    { 0x6041d2b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4211d2ba, 90 },
    { 0x14b1d2bb, 46 },
    { 0x51a1d2bc, 23 },
    { 0xa271d2bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091d2c0, 93 },
    { 0x531d2c0, 90 },
    { 0, 0 },
    { 0x351d2c3, 87 },
    { 0x4c21d2c4, 69 },
    { 0x68d1d2c4, 90 },
    { 0x3bc1d2c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3461d2c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9911d2ce, 90 },
    { 0x62c1d2ce, 55 },
    { 0, 0 },
    { 0xa511d2d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8561d2d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5211d2da, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e1d2dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf1d2e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d2e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c1d2e9, 61 },
    { 0xf4a1d2ea, 90 },
    { 0, 0 },
    { 0x1aa1d2ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf1d2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1d2f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31d2fd, 99 },
    { 0, 0 },
    { 0xa4c1d2ff, 23 },
    { 0, 0 },
    { 0xf81d301, 98 },
    { 0xdc91d302, 98 },
    { 0xf151d302, 93 },
    { 0xb371d303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x591d307, 17 },
    { 0xa1a1d308, 82 },
    { 0, 0 },
    { 0x4c61d30a, 76 },
    { 0x1181d30b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9581d30f, 23 },
    { 0xc5f1d310, 90 },
    { 0x81e1d310, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631d314, 61 },
    { 0, 0 },
    { 0xe981d316, 67 },
    { 0x8d91d317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1d31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91d324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d32d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb421d337, 88 },
    { 0, 0 },
    { 0x9821d339, 95 },
    { 0, 0 },
    { 0xb3a1d33b, 55 },
    { 0x28d1d33c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91d33f, 17 },
    { 0xfb1d340, 77 },
    { 0xd01d341, 98 },
    { 0xcfa1d342, 78 },
    { 0x8601d342, 52 },
    { 0xec41d341, 34 },
    { 0x9101d345, 23 },
    { 0xc461d340, 23 },
    { 0x6cf1d346, 95 },
    { 0, 0 },
    { 0x3761d349, 43 },
    { 0x2d61d349, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e51d34e, 90 },
    { 0x1b21d34f, 90 },
    { 0x91c1d34f, 75 },
    { 0x4231d351, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f1d357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d35f, 43 },
    { 0x7a11d360, 55 },
    { 0xec41d361, 90 },
    { 0xd751d362, 64 },
    { 0, 0 },
    { 0xd201d364, 43 },
    { 0x48f1d365, 66 },
    { 0, 0 },
    { 0x45b1d367, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1d36b, 98 },
    { 0, 0 },
    { 0xe981d36d, 90 },
    { 0xc2e1d36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331d371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb01d377, 61 },
    { 0xd061d378, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb1d37c, 98 },
    { 0x27f1d37d, 69 },
    { 0xc5d1d37d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca11d382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1d387, 98 },
    { 0xc261d388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf1d38e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841d392, 90 },
    { 0, 0 },
    { 0x7bc1d394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381d399, 33 },
    { 0, 0 },
    { 0x1611d39b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f21d3a0, 90 },
    { 0x1b11d3a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf271d3a4, 61 },
    { 0, 0 },
    { 0xb241d3a6, 33 },
    { 0x8321d3a7, 69 },
    { 0x1af1d3a7, 76 },
    { 0x6fe1d3a8, 23 },
    { 0xa711d3aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d3ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a1d3b2, 46 },
    { 0x4f11d3b3, 23 },
    { 0xb141d3b3, 43 },
    { 0xd331d3b3, 91 },
    { 0x1fa1d3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c1d3c0, 55 },
    { 0x9ee1d3c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5801d3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e21d3cc, 84 },
    { 0, 0 },
    { 0xc471d3ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f1d3d2, 52 },
    { 0xb211d3d2, 96 },
    { 0x7611d3d4, 84 },
    { 0x6b1d3d2, 98 },
    { 0, 0 },
    { 0x6321d3d7, 93 },
    { 0x1f41d3d8, 98 },
    { 0x9c31d3d9, 76 },
    { 0x2301d3d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1d3dd, 95 },
    { 0x3951d3de, 43 },
    { 0x62e1d3dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a1d3e6, 55 },
    { 0, 0 },
    { 0x5091d3e8, 61 },
    { 0, 0 },
    { 0xece1d3ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771d3f5, 90 },
    { 0x1b1d3f6, 98 },
    { 0xc301d3f7, 61 },
    { 0xc4e1d3f8, 95 },
    { 0xad61d3f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb1d3fd, 33 },
    { 0xdef1d3fe, 33 },
    { 0x22d1d3fe, 23 },
    { 0x6501d3ff, 55 },
    { 0x6011d401, 33 },
    { 0, 0 },
    { 0x6431d403, 101 },
    { 0x371d404, 61 },
    { 0, 0 },
    { 0x2d31d406, 90 },
    { 0, 0 },
    { 0x8271d408, 34 },
    { 0, 0 },
    { 0xdf01d40a, 98 },
    { 0, 0 },
    { 0x7181d40c, 92 },
    { 0xf3d1d40c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21d410, 48 },
    { 0x5ea1d411, 61 },
    { 0xa1c1d412, 34 },
    { 0x44e1d413, 78 },
    { 0x1d51d413, 61 },
    { 0x7f01d412, 95 },
    { 0x4521d413, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1d41b, 98 },
    { 0, 0 },
    { 0x69f1d41d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4661d421, 95 },
    { 0xc401d422, 37 },
    { 0, 0 },
    { 0x16e1d424, 23 },
    { 0x5f51d425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41d42c, 33 },
    { 0x7321d42c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe221d430, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6001d434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd511d438, 33 },
    { 0x9011d439, 95 },
    { 0x6f1d43a, 81 },
    { 0x6471d43a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a1d43e, 52 },
    { 0, 0 },
    { 0xcb21d440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1d444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1d448, 36 },
    { 0, 0 },
    { 0x4a01d44a, 61 },
    { 0, 0 },
    { 0xa3d1d44c, 40 },
    { 0x3b01d44c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351d450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671d455, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1d459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631d45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf1d462, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e11d465, 90 },
    { 0, 0 },
    { 0x2ce1d467, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81d46d, 34 },
    { 0, 0 },
    { 0xe491d46f, 76 },
    { 0xc61d46f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3641d474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b51d479, 67 },
    { 0, 0 },
    { 0xb051d47b, 67 },
    { 0xd5c1d47c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7801d481, 95 },
    { 0x40a1d482, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf521d485, 33 },
    { 0, 0 },
    { 0xc751d487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91d492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3881d498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a1d49e, 61 },
    { 0x5051d49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc41d4a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9301d4a5, 36 },
    { 0x11d1d4a6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1d4aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa391d4ae, 33 },
    { 0xfd81d4af, 95 },
    { 0x1c31d4b0, 95 },
    { 0xed61d4b1, 52 },
    { 0xaed1d4b2, 91 },
    { 0x2401d4b3, 95 },
    { 0xadf1d4b4, 90 },
    { 0xebe1d4b5, 43 },
    { 0x1b61d4b5, 23 },
    { 0x11d1d4b7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a1d4ba, 61 },
    { 0xb891d4bb, 95 },
    { 0xac21d4bc, 67 },
    { 0x54d1d4ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x911d4c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9511d4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd01d4c6, 34 },
    { 0x1531d4c7, 66 },
    { 0x4cc1d4c7, 55 },
    { 0x94c1d4c7, 34 },
    { 0x92a1d4c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d4ce, 91 },
    { 0x3c81d4ce, 43 },
    { 0x9ac1d4cf, 100 },
    { 0, 0 },
    { 0xfcb1d4d2, 23 },
    { 0, 0 },
    { 0x411d4d4, 98 },
    { 0x1971d4d5, 95 },
    { 0x2bd1d4d6, 46 },
    { 0, 0 },
    { 0x75e1d4d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b1d4df, 81 },
    { 0x88a1d4e0, 34 },
    { 0x9dc1d4df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee1d4e8, 100 },
    { 0xdf31d4e9, 61 },
    { 0, 0 },
    { 0x6ab1d4eb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1d4f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb281d4f9, 55 },
    { 0x351d4f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3551d4fd, 98 },
    { 0, 0 },
    { 0x6981d4ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac1d503, 55 },
    { 0x5221d504, 91 },
    { 0x7441d505, 90 },
    { 0x591d506, 33 },
    { 0xe261d505, 46 },
    { 0x3e51d508, 90 },
    { 0, 0 },
    { 0xe361d50a, 100 },
    { 0xcda1d50a, 100 },
    { 0, 0 },
    { 0xef1d50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf81d511, 48 },
    { 0x6f41d512, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea1d515, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e81d51a, 52 },
    { 0xc211d51b, 33 },
    { 0xbca1d51b, 87 },
    { 0, 0 },
    { 0x3e81d51e, 33 },
    { 0x18d1d51f, 55 },
    { 0x3ee1d51e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a11d525, 76 },
    { 0, 0 },
    { 0x5e41d527, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a1d52a, 91 },
    { 0xfc61d52b, 98 },
    { 0x1ad1d52c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e1d532, 85 },
    { 0xc4b1d533, 61 },
    { 0x8581d533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f1d53d, 43 },
    { 0x271d53e, 99 },
    { 0x38b1d53e, 33 },
    { 0xebd1d53d, 23 },
    { 0xbb11d53f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec1d544, 61 },
    { 0x90d1d545, 52 },
    { 0xe791d546, 55 },
    { 0, 0 },
    { 0xaea1d548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a1d54d, 76 },
    { 0x8dc1d54e, 77 },
    { 0xd9d1d54e, 43 },
    { 0, 0 },
    { 0xc921d551, 91 },
    { 0, 0 },
    { 0xbc1d553, 34 },
    { 0x4881d554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7041d558, 61 },
    { 0x22d1d559, 77 },
    { 0xed01d55a, 8 },
    { 0xe411d55b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61d55f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf191d562, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c61d565, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1d56f, 90 },
    { 0, 0 },
    { 0x3ee1d571, 76 },
    { 0, 0 },
    { 0xd591d573, 76 },
    { 0xf5f1d573, 33 },
    { 0, 0 },
    { 0xd511d576, 98 },
    { 0xadf1d577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b1d57d, 83 },
    { 0x87e1d57d, 98 },
    { 0, 0 },
    { 0xada1d580, 95 },
    { 0, 0 },
    { 0x7491d582, 90 },
    { 0x1611d583, 85 },
    { 0xf281d583, 99 },
    { 0x9c01d585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f1d58a, 93 },
    { 0xb551d58a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891d590, 93 },
    { 0, 0 },
    { 0x1701d592, 95 },
    { 0xfd1d593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b31d598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e1d5a3, 78 },
    { 0xebc1d5a3, 99 },
    { 0x86c1d5a5, 61 },
    { 0x39e1d5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa421d5aa, 33 },
    { 0, 0 },
    { 0xdc91d5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211d5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221d5b2, 87 },
    { 0x3811d5b2, 61 },
    { 0x6081d5b4, 17 },
    { 0, 0 },
    { 0xe2c1d5b6, 90 },
    { 0xc0c1d5b6, 95 },
    { 0, 0 },
    { 0xb161d5b9, 46 },
    { 0xa8b1d5b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae1d5c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e1d5c3, 90 },
    { 0xe871d5c4, 96 },
    { 0xd231d5c5, 68 },
    { 0, 0 },
    { 0x7b41d5c7, 61 },
    { 0x8f11d5c8, 90 },
    { 0x5e91d5c7, 43 },
    { 0x8b21d5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d5cd, 97 },
    { 0, 0 },
    { 0xe271d5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef1d5d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1d5d9, 58 },
    { 0xaf1d5da, 91 },
    { 0xf7f1d5d9, 99 },
    { 0xe451d5d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed1d5e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd1d5e8, 85 },
    { 0x83f1d5e8, 61 },
    { 0x6d31d5ea, 44 },
    { 0x4f81d5eb, 90 },
    { 0xce21d5ec, 61 },
    { 0xf321d5e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe351d5f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b1d5fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6641d5fe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1d601, 23 },
    { 0x3351d602, 98 },
    { 0xf091d602, 76 },
    { 0, 0 },
    { 0xfdf1d605, 55 },
    { 0xae11d606, 95 },
    { 0x36a1d607, 87 },
    { 0, 0 },
    { 0x5fa1d609, 33 },
    { 0x4741d60a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe161d612, 23 },
    { 0, 0 },
    { 0xdc91d614, 33 },
    { 0x4721d615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4341d618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde31d61c, 43 },
    { 0x8c31d61d, 55 },
    { 0, 0 },
    { 0x88c1d61f, 95 },
    { 0x8241d620, 33 },
    { 0xd6d1d621, 99 },
    { 0x9ea1d622, 95 },
    { 0x8241d622, 33 },
    { 0x1501d624, 93 },
    { 0x10b1d625, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271d62d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb1d630, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c51d635, 90 },
    { 0, 0 },
    { 0x2ea1d637, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d1d63a, 84 },
    { 0x4a31d63b, 95 },
    { 0x5521d63a, 98 },
    { 0x5ad1d63d, 18 },
    { 0x5591d63d, 34 },
    { 0x8331d63f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb91d644, 93 },
    { 0xdec1d644, 90 },
    { 0xdcb1d646, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a1d64a, 88 },
    { 0xa251d64a, 67 },
    { 0xb321d64b, 76 },
    { 0xcfb1d64c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a1d655, 55 },
    { 0x5c31d656, 46 },
    { 0, 0 },
    { 0xf5a1d658, 34 },
    { 0, 0 },
    { 0x25a1d65a, 95 },
    { 0x6cf1d65a, 69 },
    { 0x9f1d65b, 34 },
    { 0xcb31d65c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591d660, 89 },
    { 0xfc11d661, 66 },
    { 0xc991d660, 55 },
    { 0xce1d663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f1d668, 93 },
    { 0x52c1d669, 61 },
    { 0, 0 },
    { 0xa61d66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261d671, 95 },
    { 0, 0 },
    { 0x9771d673, 98 },
    { 0x7d51d674, 85 },
    { 0, 0 },
    { 0xf691d676, 97 },
    { 0x8631d676, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d67b, 78 },
    { 0x10a1d67b, 34 },
    { 0x4751d67c, 46 },
    { 0xdf21d67e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a1d681, 90 },
    { 0x1921d682, 66 },
    { 0, 0 },
    { 0xa401d684, 46 },
    { 0x6421d685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e31d68f, 90 },
    { 0, 0 },
    { 0xd251d691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371d694, 99 },
    { 0x19f1d695, 17 },
    { 0xc611d695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7381d69d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191d6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c41d6ae, 93 },
    { 0xde21d6af, 66 },
    { 0x9d71d6ae, 90 },
    { 0x8971d6b1, 33 },
    { 0x8cc1d6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf171d6b6, 61 },
    { 0x9361d6b6, 33 },
    { 0x18f1d6b6, 90 },
    { 0x8771d6b9, 98 },
    { 0x9661d6ba, 23 },
    { 0xd3f1d6bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe91d6c2, 23 },
    { 0x62c1d6c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc771d6c6, 75 },
    { 0, 0 },
    { 0x5431d6c8, 95 },
    { 0, 0 },
    { 0x4171d6ca, 61 },
    { 0, 0 },
    { 0xc201d6cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc411d6d0, 84 },
    { 0x46a1d6d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d6da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc01d6dd, 90 },
    { 0x3991d6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161d6e1, 23 },
    { 0xeb1d6e2, 95 },
    { 0, 0 },
    { 0xef41d6e4, 80 },
    { 0x8371d6e4, 33 },
    { 0, 0 },
    { 0x8941d6e7, 33 },
    { 0x9841d6e8, 33 },
    { 0x311d6e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8791d6ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d71d6f3, 55 },
    { 0x5371d6f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641d6f9, 55 },
    { 0, 0 },
    { 0xf271d6fb, 84 },
    { 0x76c1d6fb, 90 },
    { 0x6ea1d6fd, 46 },
    { 0x1381d6fd, 33 },
    { 0xc971d6fd, 34 },
    { 0xf3c1d700, 93 },
    { 0x6321d6ff, 61 },
    { 0x4361d702, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b31d705, 43 },
    { 0x6ed1d705, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1d709, 66 },
    { 0x8571d70a, 61 },
    { 0, 0 },
    { 0xd5a1d70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf751d712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b11d716, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3901d71b, 77 },
    { 0x67c1d71c, 98 },
    { 0xb3a1d71d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb791d727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171d72b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811d72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b71d732, 33 },
    { 0, 0 },
    { 0xa3f1d734, 93 },
    { 0, 0 },
    { 0x6711d736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1111d73a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d1d743, 81 },
    { 0x5871d744, 76 },
    { 0xbfc1d743, 90 },
    { 0x4c1d746, 40 },
    { 0xc8c1d747, 19 },
    { 0xa101d748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1d74e, 90 },
    { 0, 0 },
    { 0xae31d750, 98 },
    { 0, 0 },
    { 0xa4a1d752, 33 },
    { 0, 0 },
    { 0x82c1d754, 95 },
    { 0x3131d754, 93 },
    { 0x2411d756, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf431d75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a91d75f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf1d770, 19 },
    { 0xb491d771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1961d778, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a1d77d, 90 },
    { 0xddb1d77d, 78 },
    { 0xa651d77f, 95 },
    { 0, 0 },
    { 0xdb1d781, 87 },
    { 0xa391d782, 55 },
    { 0xc931d783, 93 },
    { 0, 0 },
    { 0x1561d785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661d789, 61 },
    { 0x331d789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd771d78e, 98 },
    { 0x8eb1d78f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f1d798, 80 },
    { 0xec81d799, 90 },
    { 0xefc1d798, 93 },
    { 0xf491d79b, 61 },
    { 0x5e91d79b, 77 },
    { 0x1be1d79a, 34 },
    { 0x1071d79e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1d7a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff81d7a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d7a8, 61 },
    { 0, 0 },
    { 0x7f21d7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab1d7b0, 46 },
    { 0, 0 },
    { 0x92c1d7b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691d7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4381d7b9, 95 },
    { 0xa5f1d7ba, 23 },
    { 0, 0 },
    { 0x8561d7bc, 76 },
    { 0xe0e1d7bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f1d7c0, 33 },
    { 0, 0 },
    { 0xb381d7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2361d7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6541d7ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1d7d1, 55 },
    { 0, 0 },
    { 0xfed1d7d3, 67 },
    { 0xa9c1d7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe071d7d7, 23 },
    { 0x6951d7d7, 76 },
    { 0xc781d7d9, 85 },
    { 0x9821d7da, 85 },
    { 0xd451d7d7, 90 },
    { 0xbe71d7d8, 66 },
    { 0xdbf1d7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f1d7e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a1d7e9, 90 },
    { 0x3e61d7e9, 23 },
    { 0xe121d7eb, 53 },
    { 0x8d31d7e9, 99 },
    { 0x46c1d7ed, 93 },
    { 0xdf61d7ee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d7f2, 34 },
    { 0, 0 },
    { 0x6a81d7f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b1d7fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1d800, 56 },
    { 0xa5b1d801, 91 },
    { 0x61c1d800, 75 },
    { 0x41d1d803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae1d807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9921d80a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e1d80d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11d812, 95 },
    { 0x9551d813, 90 },
    { 0x3121d814, 90 },
    { 0xee81d814, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c1d819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2931d81c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1d821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41d824, 90 },
    { 0xbe81d825, 98 },
    { 0xcce1d826, 66 },
    { 0xd271d825, 33 },
    { 0x74b1d828, 23 },
    { 0x2301d828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631d832, 95 },
    { 0, 0 },
    { 0x80d1d834, 90 },
    { 0xa4d1d835, 98 },
    { 0x4491d835, 77 },
    { 0xd3a1d837, 66 },
    { 0x8731d835, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d83b, 33 },
    { 0xaf41d83c, 98 },
    { 0xcad1d83d, 98 },
    { 0x67e1d83d, 95 },
    { 0x6791d83e, 90 },
    { 0, 0 },
    { 0x6bb1d841, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1d846, 33 },
    { 0, 0 },
    { 0x1251d848, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd521d851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1d854, 77 },
    { 0, 0 },
    { 0x5bf1d856, 90 },
    { 0x4ce1d857, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x4161d85a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d85f, 23 },
    { 0x8a01d85f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a1d864, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f71d868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e1d86c, 33 },
    { 0xdb61d86d, 66 },
    { 0xfe1d86e, 55 },
    { 0x3ea1d86e, 87 },
    { 0, 0 },
    { 0x6f91d871, 17 },
    { 0xfc81d871, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2821d87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3001d87f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411d883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4171d886, 68 },
    { 0x57c1d887, 98 },
    { 0xdd01d888, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101d88d, 98 },
    { 0x131d88e, 55 },
    { 0x6061d88d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb321d893, 66 },
    { 0x2a61d893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501d899, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe391d89c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7211d8a7, 77 },
    { 0x8f11d8a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3831d8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd1d8b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31d8b5, 26 },
    { 0, 0 },
    { 0xcc11d8b7, 99 },
    { 0, 0 },
    { 0x6041d8b9, 68 },
    { 0, 0 },
    { 0xd8d1d8bb, 61 },
    { 0x9491d8bc, 34 },
    { 0xcea1d8bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec1d8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1741d8c8, 94 },
    { 0x4171d8c9, 90 },
    { 0xf001d8ca, 61 },
    { 0x9731d8cb, 61 },
    { 0xd6b1d8cc, 34 },
    { 0x3ac1d8cb, 23 },
    { 0xa801d8ce, 44 },
    { 0x6bb1d8c8, 78 },
    { 0x24a1d8d0, 95 },
    { 0x2ed1d8cd, 91 },
    { 0x8a31d8cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61d8d6, 55 },
    { 0xf2d1d8d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901d8db, 35 },
    { 0, 0 },
    { 0x3b1d8dd, 91 },
    { 0x69e1d8dd, 23 },
    { 0, 0 },
    { 0xca61d8e0, 76 },
    { 0x9621d8e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b1d8eb, 55 },
    { 0x8681d8ec, 33 },
    { 0x6701d8ec, 43 },
    { 0xf521d8ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf1d8f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b31d8f6, 33 },
    { 0x1b01d8f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91d8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da1d903, 46 },
    { 0xcde1d903, 52 },
    { 0xb7e1d905, 98 },
    { 0x3a1d906, 43 },
    { 0xba81d907, 33 },
    { 0x6491d903, 67 },
    { 0xdaa1d909, 90 },
    { 0xcf31d906, 95 },
    { 0, 0 },
    { 0x9521d90c, 23 },
    { 0xdd41d90d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b1d911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe871d914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca1d918, 91 },
    { 0, 0 },
    { 0x9541d91a, 99 },
    { 0xe7d1d91b, 76 },
    { 0x3d31d91c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd1d920, 98 },
    { 0x451d921, 95 },
    { 0x1fd1d922, 33 },
    { 0x6521d923, 65 },
    { 0x7861d920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc1d928, 23 },
    { 0, 0 },
    { 0x9351d92a, 98 },
    { 0xd961d92b, 77 },
    { 0, 0 },
    { 0xcc1d92d, 61 },
    { 0, 0 },
    { 0xa471d92f, 43 },
    { 0x6de1d930, 95 },
    { 0xb1b1d931, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd1d936, 93 },
    { 0x6571d937, 78 },
    { 0x3331d937, 90 },
    { 0x7a91d938, 23 },
    { 0x7641d93a, 87 },
    { 0, 0 },
    { 0xde61d93c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321d93f, 98 },
    { 0xcee1d940, 33 },
    { 0, 0 },
    { 0x3061d942, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab1d947, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed21d94b, 95 },
    { 0, 0 },
    { 0x9ad1d94d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6281d951, 61 },
    { 0x7491d951, 61 },
    { 0, 0 },
    { 0x86f1d954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1681d95f, 98 },
    { 0, 0 },
    { 0x2f11d961, 98 },
    { 0, 0 },
    { 0x8621d963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c41d966, 78 },
    { 0x1881d967, 61 },
    { 0xbd41d967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab11d96c, 36 },
    { 0, 0 },
    { 0x72e1d96e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1331d973, 61 },
    { 0x75d1d974, 90 },
    { 0x5cf1d974, 46 },
    { 0x6be1d973, 34 },
    { 0, 0 },
    { 0xb8c1d978, 98 },
    { 0xb1e1d979, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261d97c, 95 },
    { 0x3441d97d, 90 },
    { 0xebb1d97c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d1d983, 34 },
    { 0, 0 },
    { 0x2651d985, 33 },
    { 0xfc1d985, 52 },
    { 0x7ee1d985, 76 },
    { 0x5011d988, 98 },
    { 0x77d1d989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb1d98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1d992, 77 },
    { 0xe81d992, 91 },
    { 0x4b01d994, 46 },
    { 0x9801d992, 52 },
    { 0x14e1d996, 98 },
    { 0x88a1d995, 61 },
    { 0xe871d998, 91 },
    { 0xf081d998, 61 },
    { 0x70f1d996, 95 },
    { 0xea11d995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961d99f, 33 },
    { 0x9701d9a0, 98 },
    { 0, 0 },
    { 0xc201d9a2, 90 },
    { 0x261d9a3, 76 },
    { 0, 0 },
    { 0xed1d9a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d1d9a8, 68 },
    { 0x21b1d9a9, 56 },
    { 0xaf01d9aa, 98 },
    { 0xed81d9aa, 64 },
    { 0x3501d9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81d9af, 33 },
    { 0x2091d9af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571d9b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e91d9c1, 52 },
    { 0xd3a1d9c2, 55 },
    { 0, 0 },
    { 0xa261d9c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c1d9c7, 33 },
    { 0, 0 },
    { 0xb181d9c9, 90 },
    { 0x28d1d9ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be1d9cd, 90 },
    { 0, 0 },
    { 0xa881d9cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb531d9d2, 33 },
    { 0, 0 },
    { 0xe4e1d9d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2521d9d9, 23 },
    { 0xfc81d9da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631d9e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d9e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd941d9ea, 23 },
    { 0xf761d9ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a1d9f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc71d9f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2831d9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1da04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6531da07, 90 },
    { 0x5681da07, 61 },
    { 0xb5a1da09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b81da0d, 87 },
    { 0, 0 },
    { 0x9691da0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e1da12, 95 },
    { 0, 0 },
    { 0xa041da14, 76 },
    { 0x8731da14, 90 },
    { 0x7e21da16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6441da1a, 65 },
    { 0xbba1da1b, 34 },
    { 0x2691da1b, 95 },
    { 0x4a51da1d, 90 },
    { 0xa561da1a, 90 },
    { 0xfc81da1f, 61 },
    { 0xa761da20, 98 },
    { 0xcb81da20, 90 },
    { 0x11d1da22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f41da26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c91da2b, 98 },
    { 0x21e1da2c, 61 },
    { 0x7891da2c, 95 },
    { 0x6fa1da2e, 95 },
    { 0x7211da2f, 91 },
    { 0x14c1da2c, 23 },
    { 0xa4c1da2e, 76 },
    { 0x7871da2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a1da35, 61 },
    { 0x3ae1da36, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271da39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c11da3c, 90 },
    { 0x93a1da3d, 98 },
    { 0x6541da3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed41da45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe201da48, 36 },
    { 0, 0 },
    { 0xfd21da4a, 34 },
    { 0x14c1da4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec91da4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71da59, 61 },
    { 0x1da5a, 61 },
    { 0x4001da5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71da61, 34 },
    { 0x8ff1da61, 33 },
    { 0x63a1da63, 61 },
    { 0x6b01da64, 34 },
    { 0x6d1da65, 61 },
    { 0x2d51da63, 84 },
    { 0, 0 },
    { 0xf761da68, 98 },
    { 0, 0 },
    { 0xa1f1da6a, 52 },
    { 0xf6c1da6a, 65 },
    { 0x9251da6a, 87 },
    { 0xb6e1da6b, 34 },
    { 0x3531da6e, 91 },
    { 0x3301da6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011da73, 84 },
    { 0, 0 },
    { 0xdeb1da75, 33 },
    { 0x1f91da75, 76 },
    { 0, 0 },
    { 0xd031da78, 75 },
    { 0, 0 },
    { 0xd651da7a, 95 },
    { 0x2b91da7b, 76 },
    { 0x1111da7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb711da83, 85 },
    { 0xbbe1da84, 90 },
    { 0x3e71da85, 76 },
    { 0xbdd1da86, 78 },
    { 0x5c31da87, 75 },
    { 0, 0 },
    { 0x5451da89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe81da8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51da91, 43 },
    { 0xf6f1da92, 52 },
    { 0xad81da91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb011da96, 98 },
    { 0x6271da97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4281da9b, 90 },
    { 0xaa51da9b, 34 },
    { 0, 0 },
    { 0x7cd1da9e, 52 },
    { 0x7501da9e, 23 },
    { 0x8cb1daa0, 93 },
    { 0x78e1daa1, 76 },
    { 0x3551daa2, 64 },
    { 0xdc01daa2, 78 },
    { 0x1bc1daa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1dab3, 81 },
    { 0x4cf1dab4, 95 },
    { 0x3fd1dab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7951dab9, 33 },
    { 0, 0 },
    { 0x7501dabb, 33 },
    { 0xb631dabc, 33 },
    { 0x8401dabc, 76 },
    { 0xcf41dabc, 34 },
    { 0, 0 },
    { 0x5f51dac0, 95 },
    { 0x2ef1dac1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x321dac4, 98 },
    { 0, 0 },
    { 0x9761dac6, 76 },
    { 0, 0 },
    { 0x77c1dac8, 82 },
    { 0, 0 },
    { 0x9421daca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b11dacd, 100 },
    { 0, 0 },
    { 0xe351dacf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1dad2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd531dad5, 33 },
    { 0xffc1dad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c71dadd, 23 },
    { 0, 0 },
    { 0x45f1dadf, 95 },
    { 0, 0 },
    { 0x3db1dae1, 99 },
    { 0x6f11dae1, 43 },
    { 0, 0 },
    { 0x83f1dae4, 55 },
    { 0xb7c1dae5, 90 },
    { 0xdc01dae6, 52 },
    { 0x49d1dae7, 65 },
    { 0x6bb1dae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411daec, 61 },
    { 0xfb1daed, 33 },
    { 0x5c11daee, 85 },
    { 0xe31daef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a1daf2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b31daf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1dafd, 76 },
    { 0, 0 },
    { 0x9fc1daff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1db04, 52 },
    { 0x50c1db05, 95 },
    { 0xd471db06, 43 },
    { 0, 0 },
    { 0x6dd1db08, 43 },
    { 0x84e1db09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba1db0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91db14, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1db18, 76 },
    { 0, 0 },
    { 0x3b41db1a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d1db1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71db24, 33 },
    { 0xca41db24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c41db2d, 46 },
    { 0x3a11db2e, 34 },
    { 0x5d31db2f, 67 },
    { 0xcf91db30, 76 },
    { 0xb721db2f, 90 },
    { 0, 0 },
    { 0xb6a1db33, 85 },
    { 0, 0 },
    { 0xd9f1db35, 55 },
    { 0x3091db36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe371db39, 55 },
    { 0, 0 },
    { 0x7c51db3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa361db3e, 43 },
    { 0xed1db3e, 66 },
    { 0, 0 },
    { 0xc4e1db41, 33 },
    { 0x5901db42, 95 },
    { 0xd541db43, 64 },
    { 0xec51db41, 95 },
    { 0x5cc1db45, 33 },
    { 0x67f1db46, 55 },
    { 0x4421db47, 55 },
    { 0x1701db41, 75 },
    { 0x1351db49, 92 },
    { 0xa8e1db4a, 80 },
    { 0, 0 },
    { 0x1651db4c, 61 },
    { 0xd551db4c, 61 },
    { 0xa0b1db4e, 33 },
    { 0x4b51db4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc411db54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d11db58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1db5b, 34 },
    { 0x2fd1db5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee1db5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee81db64, 76 },
    { 0x5f41db64, 55 },
    { 0, 0 },
    { 0xc3c1db67, 85 },
    { 0x91c1db68, 23 },
    { 0x6c61db68, 100 },
    { 0x36e1db68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1db6d, 98 },
    { 0, 0 },
    { 0x6291db6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e41db77, 85 },
    { 0x33c1db78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11db7d, 76 },
    { 0xa011db7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1181db82, 78 },
    { 0x271db83, 34 },
    { 0x14c1db82, 90 },
    { 0, 0 },
    { 0x9e21db86, 84 },
    { 0x4bb1db87, 90 },
    { 0x83f1db88, 61 },
    { 0x55f1db89, 98 },
    { 0xb7b1db8a, 98 },
    { 0x9111db8b, 98 },
    { 0xcca1db8c, 98 },
    { 0x3491db8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611db94, 33 },
    { 0xfc61db95, 91 },
    { 0x7de1db96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1db9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b1dba3, 33 },
    { 0xd1f1dba4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1dbac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51dbaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71dbbd, 61 },
    { 0x81d1dbbe, 76 },
    { 0xe9b1dbbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf281dbc5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc671dbc8, 46 },
    { 0x1c31dbc8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d1dbcc, 95 },
    { 0, 0 },
    { 0x94c1dbce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771dbd1, 23 },
    { 0x2011dbd2, 90 },
    { 0x74f1dbd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11dbdc, 23 },
    { 0, 0 },
    { 0x3951dbde, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb531dbe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d1dbe6, 55 },
    { 0, 0 },
    { 0xfff1dbe8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1dbf0, 76 },
    { 0x32d1dbf0, 93 },
    { 0x7251dbf2, 17 },
    { 0xc961dbf2, 76 },
    { 0, 0 },
    { 0xedc1dbf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701dbf8, 17 },
    { 0x2e71dbf8, 61 },
    { 0x8ff1dbf9, 33 },
    { 0x541dbfb, 61 },
    { 0x7211dbfb, 95 },
    { 0, 0 },
    { 0x89b1dbfe, 77 },
    { 0x54f1dbff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01dc04, 98 },
    { 0, 0 },
    { 0x1211dc06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b1dc09, 23 },
    { 0, 0 },
    { 0x2741dc0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271dc0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771dc13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41dc19, 76 },
    { 0, 0 },
    { 0xa311dc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a21dc1e, 33 },
    { 0x3a01dc1e, 90 },
    { 0, 0 },
    { 0x75c1dc21, 90 },
    { 0x6a91dc22, 88 },
    { 0, 0 },
    { 0x2101dc24, 52 },
    { 0, 0 },
    { 0x2391dc26, 33 },
    { 0, 0 },
    { 0x451dc28, 100 },
    { 0x3281dc28, 76 },
    { 0x9271dc28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab71dc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3551dc31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf551dc3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5201dc46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a1dc4a, 55 },
    { 0, 0 },
    { 0xd5c1dc4c, 93 },
    { 0x9761dc4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21dc50, 34 },
    { 0, 0 },
    { 0x5c41dc52, 90 },
    { 0xb161dc52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6121dc58, 33 },
    { 0, 0 },
    { 0xba11dc5a, 69 },
    { 0x2371dc5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a1dc5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81dc63, 33 },
    { 0x6f51dc64, 23 },
    { 0x2fb1dc63, 33 },
    { 0xbc11dc66, 90 },
    { 0x17d1dc67, 66 },
    { 0xc641dc67, 95 },
    { 0xe161dc69, 33 },
    { 0xa1c1dc69, 75 },
    { 0x6d81dc69, 55 },
    { 0x38d1dc6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9041dc72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe601dc77, 90 },
    { 0x78d1dc77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151dc7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f1dc7e, 66 },
    { 0x7051dc7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51dc82, 46 },
    { 0xcd41dc83, 33 },
    { 0x3ee1dc84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1dc88, 61 },
    { 0xbcb1dc89, 91 },
    { 0x56b1dc88, 40 },
    { 0x1be1dc8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1dc91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401dc95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1dc98, 23 },
    { 0x3e21dc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3061dc9c, 95 },
    { 0, 0 },
    { 0x6dd1dc9e, 23 },
    { 0, 0 },
    { 0xf301dca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6781dca4, 91 },
    { 0, 0 },
    { 0x4e21dca6, 33 },
    { 0x7521dca7, 34 },
    { 0, 0 },
    { 0x8681dca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101dcb1, 76 },
    { 0xc701dcb1, 80 },
    { 0xa131dcb2, 23 },
    { 0, 0 },
    { 0x49a1dcb5, 99 },
    { 0xcae1dcb6, 34 },
    { 0xd21dcb7, 34 },
    { 0xd031dcb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101dcc1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041dcc4, 33 },
    { 0x1861dcc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6711dcc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b1dccf, 66 },
    { 0x9911dcd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c1dcd3, 34 },
    { 0, 0 },
    { 0xbb1dcd5, 61 },
    { 0x8cb1dcd6, 98 },
    { 0xa0f1dcd6, 34 },
    { 0xb6b1dcd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6491dcdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf161dce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91dce6, 90 },
    { 0x7ff1dce6, 61 },
    { 0xc4d1dce8, 99 },
    { 0x9fd1dce8, 97 },
    { 0x7af1dce8, 46 },
    { 0x3f41dceb, 93 },
    { 0, 0 },
    { 0x69a1dced, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b1dcf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1dcf3, 84 },
    { 0, 0 },
    { 0x2531dcf5, 61 },
    { 0x8251dcf6, 98 },
    { 0x57b1dcf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe231dcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe11dd03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d1dd08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed1dd16, 90 },
    { 0, 0 },
    { 0x8581dd18, 23 },
    { 0, 0 },
    { 0x6a41dd1a, 34 },
    { 0, 0 },
    { 0x6b31dd1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd231dd1f, 95 },
    { 0, 0 },
    { 0x5461dd21, 98 },
    { 0xeac1dd21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1dd2a, 61 },
    { 0xdd61dd2a, 90 },
    { 0x2011dd2c, 90 },
    { 0, 0 },
    { 0xaf71dd2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e51dd35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa231dd38, 98 },
    { 0, 0 },
    { 0xcf71dd3a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e1dd40, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1dd43, 46 },
    { 0x84f1dd44, 87 },
    { 0, 0 },
    { 0x69b1dd46, 43 },
    { 0xaed1dd47, 23 },
    { 0xd711dd48, 61 },
    { 0x6641dd49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d1dd4c, 90 },
    { 0xd4e1dd4c, 68 },
    { 0x39b1dd4c, 34 },
    { 0xbfb1dd4f, 67 },
    { 0x2551dd4f, 95 },
    { 0xdac1dd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1dd55, 64 },
    { 0, 0 },
    { 0xb541dd57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61dd5b, 90 },
    { 0x67c1dd5c, 61 },
    { 0, 0 },
    { 0x95e1dd5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811dd6a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d1dd6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e1dd72, 98 },
    { 0x4e61dd72, 84 },
    { 0xd281dd74, 95 },
    { 0x3b11dd72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f1dd80, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4571dd84, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae71dd8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd8e, 23 },
    { 0x2741dd8f, 90 },
    { 0, 0 },
    { 0x5ec1dd91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281dd95, 33 },
    { 0, 0 },
    { 0xf621dd97, 34 },
    { 0, 0 },
    { 0x96b1dd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c51dd9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381dda1, 36 },
    { 0, 0 },
    { 0x5191dda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1dda6, 92 },
    { 0xa171dda7, 77 },
    { 0x6c1dda8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c81ddac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21ddb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3951ddb7, 34 },
    { 0x68f1ddb7, 90 },
    { 0x7411ddb9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1ddbc, 66 },
    { 0x97e1ddbc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1ddc0, 61 },
    { 0, 0 },
    { 0xf041ddc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb191ddc5, 99 },
    { 0, 0 },
    { 0x53a1ddc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9451ddca, 61 },
    { 0, 0 },
    { 0xf4f1ddcc, 33 },
    { 0xe1e1ddcc, 98 },
    { 0x2d01ddce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b1ddd2, 100 },
    { 0xd1e1ddd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc511ddda, 98 },
    { 0x83d1dddb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1dddf, 33 },
    { 0x6361dddf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071dde8, 75 },
    { 0x4391dde8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7591ddec, 55 },
    { 0, 0 },
    { 0xfd71ddee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e1ddf2, 61 },
    { 0x91d1ddf3, 23 },
    { 0x47f1ddf4, 38 },
    { 0x94b1ddf4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a31ddf9, 90 },
    { 0xc621ddfa, 55 },
    { 0xa1d1ddfb, 23 },
    { 0x7a01ddfc, 46 },
    { 0x1161ddfc, 93 },
    { 0, 0 },
    { 0xade1ddff, 76 },
    { 0xe851ddff, 33 },
    { 0x8d81de01, 46 },
    { 0xc4e1de02, 46 },
    { 0x98d1de00, 33 },
    { 0x2221de04, 46 },
    { 0x35d1de05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4571de0a, 76 },
    { 0x88e1de0b, 65 },
    { 0, 0 },
    { 0x4171de0d, 80 },
    { 0, 0 },
    { 0xfc91de0f, 33 },
    { 0xbc91de10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e1de19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81de1f, 74 },
    { 0xafb1de20, 90 },
    { 0x17f1de20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf591de25, 99 },
    { 0, 0 },
    { 0xc931de27, 40 },
    { 0x2fd1de28, 99 },
    { 0xe8f1de29, 90 },
    { 0, 0 },
    { 0x4a41de2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1de2e, 76 },
    { 0, 0 },
    { 0xaa1de30, 43 },
    { 0x8c51de31, 34 },
    { 0, 0 },
    { 0x36b1de33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf1de36, 90 },
    { 0x64e1de37, 95 },
    { 0x3101de37, 61 },
    { 0xb061de38, 55 },
    { 0, 0 },
    { 0x3a71de3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc1de3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc741de43, 90 },
    { 0x7061de44, 98 },
    { 0xd9a1de44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c1de49, 43 },
    { 0xd8d1de4a, 23 },
    { 0x2701de4b, 95 },
    { 0xfcc1de4c, 23 },
    { 0xb4f1de49, 98 },
    { 0x7be1de4e, 76 },
    { 0xaa1de4f, 101 },
    { 0xd8f1de4c, 95 },
    { 0, 0 },
    { 0x40d1de52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101de56, 77 },
    { 0x5681de57, 98 },
    { 0xccd1de58, 33 },
    { 0xa611de59, 33 },
    { 0x6cf1de59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1de5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61de60, 76 },
    { 0x9d01de60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631de67, 56 },
    { 0xfac1de67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d31de6d, 33 },
    { 0x4a11de6d, 87 },
    { 0, 0 },
    { 0x69e1de70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1de73, 91 },
    { 0xf121de73, 93 },
    { 0x3091de75, 90 },
    { 0, 0 },
    { 0x27a1de77, 90 },
    { 0, 0 },
    { 0x7ef1de79, 61 },
    { 0, 0 },
    { 0x1001de7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb1de81, 76 },
    { 0x83f1de82, 76 },
    { 0xd441de83, 23 },
    { 0xd6d1de83, 90 },
    { 0xe3c1de83, 67 },
    { 0x48a1de84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb21de89, 38 },
    { 0xb701de89, 33 },
    { 0x3d01de8a, 23 },
    { 0, 0 },
    { 0xb871de8d, 19 },
    { 0x60a1de8d, 87 },
    { 0xc701de8d, 61 },
    { 0, 0 },
    { 0x4841de91, 90 },
    { 0xa131de92, 77 },
    { 0xe651de92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf261de97, 90 },
    { 0x20d1de98, 33 },
    { 0xf791de99, 61 },
    { 0x7a31de98, 33 },
    { 0, 0 },
    { 0x35e1de9c, 75 },
    { 0, 0 },
    { 0x45a1de9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1741dea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581dea6, 99 },
    { 0, 0 },
    { 0x11c1dea8, 61 },
    { 0x87e1dea9, 64 },
    { 0x4651deaa, 76 },
    { 0xf721dea9, 61 },
    { 0xdea1deac, 43 },
    { 0, 0 },
    { 0x8ba1deae, 33 },
    { 0xd371deaf, 95 },
    { 0, 0 },
    { 0x8161deb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd971deb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651dec3, 66 },
    { 0x1da1dec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51ded0, 55 },
    { 0xa7a1ded1, 61 },
    { 0x62b1ded2, 90 },
    { 0xc6a1ded0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb1ded6, 93 },
    { 0xffb1ded7, 78 },
    { 0x3f51ded7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a1dedb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291dee5, 40 },
    { 0, 0 },
    { 0x1e1dee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071deee, 23 },
    { 0x2de1deee, 33 },
    { 0x96d1def0, 61 },
    { 0xddc1def1, 23 },
    { 0, 0 },
    { 0x7f01def3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391defa, 91 },
    { 0x2d41defb, 90 },
    { 0x5701defa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7361deff, 66 },
    { 0x9241deff, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451df0e, 100 },
    { 0x37f1df0e, 84 },
    { 0xecb1df10, 90 },
    { 0x8e01df11, 66 },
    { 0xf7c1df12, 74 },
    { 0xb0d1df0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b1df1a, 61 },
    { 0, 0 },
    { 0x9671df1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc1df1f, 43 },
    { 0x2401df20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c1df28, 61 },
    { 0xc381df28, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd21df32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb1df35, 46 },
    { 0x6451df35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21df3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61df41, 90 },
    { 0, 0 },
    { 0x6881df43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5311df47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc1df4c, 91 },
    { 0xd141df4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea1df50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf081df54, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf11df57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1df5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c1df5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f1df62, 68 },
    { 0, 0 },
    { 0x6971df64, 90 },
    { 0x20b1df65, 33 },
    { 0x5501df65, 61 },
    { 0, 0 },
    { 0xdf51df68, 95 },
    { 0xa561df68, 43 },
    { 0, 0 },
    { 0x6881df6b, 95 },
    { 0x8ae1df6c, 34 },
    { 0x53f1df6b, 91 },
    { 0, 0 },
    { 0x4631df6f, 95 },
    { 0x1c01df70, 46 },
    { 0x4c1df70, 34 },
    { 0x7de1df72, 93 },
    { 0x3661df73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc51df7c, 51 },
    { 0xcec1df7d, 91 },
    { 0, 0 },
    { 0xb661df7f, 98 },
    { 0xdfa1df80, 33 },
    { 0xbe71df81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c31df84, 34 },
    { 0x34a1df85, 36 },
    { 0, 0 },
    { 0x20d1df87, 81 },
    { 0xb8b1df87, 69 },
    { 0xab1df87, 55 },
    { 0x551df8a, 95 },
    { 0xb371df8a, 84 },
    { 0, 0 },
    { 0xd291df8d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa891df90, 84 },
    { 0xb421df91, 89 },
    { 0xb751df92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd981df95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d1df99, 33 },
    { 0x111df9a, 55 },
    { 0x7d91df9b, 93 },
    { 0xa311df9c, 75 },
    { 0x2461df9d, 90 },
    { 0x5071df9e, 61 },
    { 0x1391df9d, 98 },
    { 0xe8e1df9f, 94 },
    { 0x9e21dfa1, 34 },
    { 0x7061dfa2, 76 },
    { 0xa1a1dfa3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f1dfa7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa1dfad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f1dfb2, 75 },
    { 0, 0 },
    { 0x69f1dfb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1dfbb, 76 },
    { 0x3581dfbc, 55 },
    { 0, 0 },
    { 0x6961dfbe, 77 },
    { 0xc501dfbf, 61 },
    { 0x10b1dfc0, 40 },
    { 0xf041dfc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61dfc5, 90 },
    { 0xd691dfc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c31dfcc, 33 },
    { 0x1001dfcc, 22 },
    { 0x9ff1dfce, 46 },
    { 0xe711dfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa1dfd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c1dfdb, 91 },
    { 0x9ae1dfdc, 98 },
    { 0, 0 },
    { 0x4061dfde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6631dfe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec1dfec, 93 },
    { 0x9921dfed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb021dff1, 61 },
    { 0x6291dff2, 23 },
    { 0, 0 },
    { 0x5701dff4, 94 },
    { 0xd981dff4, 77 },
    { 0xc3c1dff6, 52 },
    { 0x27b1dff6, 98 },
    { 0, 0 },
    { 0xb71dff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf011dffd, 95 },
    { 0, 0 },
    { 0x66f1dfff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81e009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b01e00d, 40 },
    { 0, 0 },
    { 0x9a81e00f, 81 },
    { 0xc421e00f, 23 },
    { 0x7f61e011, 52 },
    { 0xb421e012, 81 },
    { 0xa211e00f, 90 },
    { 0x90b1e014, 48 },
    { 0, 0 },
    { 0x9d81e016, 91 },
    { 0xf531e017, 90 },
    { 0x23d1e018, 33 },
    { 0, 0 },
    { 0xeeb1e01a, 98 },
    { 0, 0 },
    { 0x3791e01c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c91e021, 76 },
    { 0x2661e022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd361e026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261e02a, 61 },
    { 0xb1a1e02b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9341e02f, 95 },
    { 0x4ac1e030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b1e034, 52 },
    { 0, 0 },
    { 0x6481e036, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f1e047, 98 },
    { 0x2da1e048, 81 },
    { 0x5eb1e049, 61 },
    { 0x4a11e048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1e04d, 90 },
    { 0x4351e04e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11e055, 40 },
    { 0xb271e056, 66 },
    { 0, 0 },
    { 0x79c1e058, 49 },
    { 0x3181e059, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da1e05c, 90 },
    { 0xf361e05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe791e060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e064, 84 },
    { 0x2331e065, 98 },
    { 0, 0 },
    { 0x42c1e067, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac1e06b, 52 },
    { 0xd051e06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001e076, 85 },
    { 0x6231e076, 33 },
    { 0x9ae1e076, 90 },
    { 0x7581e079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451e07d, 81 },
    { 0xdd11e07d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e083, 98 },
    { 0x6981e084, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb461e091, 90 },
    { 0xbbd1e092, 87 },
    { 0x33b1e093, 23 },
    { 0x75d1e091, 34 },
    { 0xfc21e095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1e09d, 87 },
    { 0x3ea1e09e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251e0a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1e0a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9791e0af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8141e0b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f81e0b9, 90 },
    { 0x34e1e0ba, 76 },
    { 0x1cb1e0bb, 93 },
    { 0xcbd1e0bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a1e0c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e1e0c4, 52 },
    { 0, 0 },
    { 0x51c1e0c6, 76 },
    { 0x4051e0c7, 46 },
    { 0x3e41e0c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1e0d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c1e0da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b1e0e3, 90 },
    { 0xbc61e0e3, 90 },
    { 0xb891e0e4, 90 },
    { 0, 0 },
    { 0xc8d1e0e7, 55 },
    { 0x8151e0e8, 61 },
    { 0x80d1e0e9, 33 },
    { 0xcfd1e0e9, 23 },
    { 0x8141e0eb, 90 },
    { 0xbd1e0e9, 43 },
    { 0x14a1e0ed, 99 },
    { 0xa8e1e0ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b91e0f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6481e0f4, 83 },
    { 0x8751e0f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab1e0f8, 98 },
    { 0xc771e0f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1021e101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae31e106, 76 },
    { 0xfa11e107, 76 },
    { 0, 0 },
    { 0x2eb1e109, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e1e10d, 90 },
    { 0, 0 },
    { 0xeed1e10f, 17 },
    { 0xfe41e110, 33 },
    { 0xb821e111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1e114, 90 },
    { 0xb261e115, 93 },
    { 0x1b11e115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4451e11a, 43 },
    { 0xeff1e11b, 90 },
    { 0x6411e11b, 23 },
    { 0xdd01e11d, 76 },
    { 0x53e1e11d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1e124, 33 },
    { 0, 0 },
    { 0xdd81e126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781e133, 98 },
    { 0xe4b1e134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7461e137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc1e13a, 99 },
    { 0x4561e13b, 61 },
    { 0x5ed1e13b, 61 },
    { 0x4bd1e13d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff71e141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b91e14a, 76 },
    { 0x531e14a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab1e14f, 33 },
    { 0x3b11e150, 95 },
    { 0x53b1e150, 65 },
    { 0xc1c1e150, 75 },
    { 0, 0 },
    { 0x87b1e154, 43 },
    { 0xa411e155, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1e159, 95 },
    { 0xae1e15a, 77 },
    { 0x3701e15b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe731e160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4821e163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd91e167, 98 },
    { 0x3951e168, 34 },
    { 0xfbb1e169, 52 },
    { 0x8e61e168, 78 },
    { 0x1d1e16b, 95 },
    { 0xac51e168, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae1e16f, 61 },
    { 0xbd21e170, 65 },
    { 0xd9b1e16f, 91 },
    { 0x95c1e172, 93 },
    { 0x5c41e173, 61 },
    { 0x4a11e171, 34 },
    { 0xd1e175, 17 },
    { 0x58c1e176, 55 },
    { 0xe5f1e177, 34 },
    { 0x7901e178, 91 },
    { 0, 0 },
    { 0x3cc1e17a, 100 },
    { 0x55a1e17b, 33 },
    { 0xd6a1e17b, 90 },
    { 0xe7e1e17b, 98 },
    { 0xfa81e17b, 95 },
    { 0, 0 },
    { 0xf551e180, 95 },
    { 0xa111e181, 93 },
    { 0x1581e182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d1e18c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e1e191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5601e197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a1e19b, 55 },
    { 0x92a1e19b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d61e1a4, 61 },
    { 0, 0 },
    { 0x8701e1a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6871e1ae, 33 },
    { 0x4ca1e1ae, 98 },
    { 0xa571e1b0, 61 },
    { 0, 0 },
    { 0x3421e1b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6721e1ba, 100 },
    { 0x83e1e1bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41e1bf, 48 },
    { 0xeef1e1bf, 55 },
    { 0, 0 },
    { 0x5d91e1c2, 61 },
    { 0xc271e1c2, 85 },
    { 0, 0 },
    { 0x23f1e1c5, 23 },
    { 0x86b1e1c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a1e1cb, 55 },
    { 0x1831e1cc, 52 },
    { 0x55d1e1cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b1e1d1, 33 },
    { 0, 0 },
    { 0xf941e1d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241e1d8, 33 },
    { 0x3861e1d9, 34 },
    { 0, 0 },
    { 0xa781e1db, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa1e1e3, 98 },
    { 0x2b71e1e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb841e1e7, 53 },
    { 0xdb91e1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4101e1ef, 95 },
    { 0x80e1e1ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc681e1f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1e1fd, 67 },
    { 0xe771e1fe, 88 },
    { 0x3421e1fe, 76 },
    { 0, 0 },
    { 0x6011e201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f1e206, 55 },
    { 0xb361e206, 47 },
    { 0, 0 },
    { 0xfa71e209, 95 },
    { 0, 0 },
    { 0x5251e20b, 61 },
    { 0, 0 },
    { 0xb451e20d, 88 },
    { 0xba41e20d, 90 },
    { 0xd81e20f, 61 },
    { 0x4ac1e20f, 61 },
    { 0xdd71e211, 23 },
    { 0x8f01e20e, 61 },
    { 0x3711e213, 95 },
    { 0x3c11e213, 94 },
    { 0x4fb1e215, 33 },
    { 0, 0 },
    { 0x2ab1e217, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9751e21b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde61e224, 34 },
    { 0xde81e225, 33 },
    { 0xa2c1e226, 46 },
    { 0x7f01e227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d91e22d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9931e230, 76 },
    { 0, 0 },
    { 0xfce1e232, 33 },
    { 0, 0 },
    { 0x3641e234, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac1e237, 46 },
    { 0x31a1e238, 48 },
    { 0xd391e239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6911e23d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791e242, 33 },
    { 0, 0 },
    { 0x551e244, 40 },
    { 0x4a71e244, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1e249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a1e24d, 43 },
    { 0, 0 },
    { 0x74e1e24f, 33 },
    { 0xf501e250, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d51e254, 33 },
    { 0x9c71e254, 46 },
    { 0, 0 },
    { 0xd1a1e257, 76 },
    { 0x8b51e258, 46 },
    { 0x7911e258, 90 },
    { 0x19f1e25a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51e261, 33 },
    { 0x1561e261, 100 },
    { 0xd161e263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b1e268, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be1e26e, 75 },
    { 0, 0 },
    { 0xd871e270, 33 },
    { 0, 0 },
    { 0xca1e272, 23 },
    { 0, 0 },
    { 0x6201e274, 33 },
    { 0x1c1e274, 93 },
    { 0x3f81e276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1e280, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71e285, 77 },
    { 0xd851e285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1e289, 33 },
    { 0, 0 },
    { 0xe641e28b, 93 },
    { 0, 0 },
    { 0xdde1e28d, 90 },
    { 0, 0 },
    { 0x2921e28f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e81e294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e1e297, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c1e29a, 66 },
    { 0xe8e1e29b, 75 },
    { 0x3491e29a, 90 },
    { 0x2281e29d, 38 },
    { 0xa11e29d, 67 },
    { 0, 0 },
    { 0x32c1e2a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba1e2a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721e2ac, 33 },
    { 0x7dc1e2ac, 76 },
    { 0x2071e2ae, 81 },
    { 0, 0 },
    { 0x9281e2b0, 61 },
    { 0x13d1e2b1, 95 },
    { 0xc561e2b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2371e2b5, 36 },
    { 0, 0 },
    { 0x1221e2b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251e2bb, 93 },
    { 0x2d91e2bb, 47 },
    { 0x98d1e2bd, 90 },
    { 0, 0 },
    { 0x15d1e2bf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741e2c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc81e2cc, 55 },
    { 0x5fb1e2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941e2d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31e2d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1e2dd, 33 },
    { 0xeca1e2de, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2091e2e1, 68 },
    { 0x7e11e2e2, 95 },
    { 0, 0 },
    { 0x8d81e2e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc31e2e8, 55 },
    { 0xb091e2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde1e2ec, 23 },
    { 0, 0 },
    { 0x4351e2ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb021e2f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81e2f8, 78 },
    { 0, 0 },
    { 0x3e51e2fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c1e2fd, 66 },
    { 0x38a1e2fd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541e301, 93 },
    { 0x1d21e302, 77 },
    { 0x4ac1e301, 61 },
    { 0, 0 },
    { 0xf671e305, 98 },
    { 0x3f91e306, 33 },
    { 0x5201e306, 23 },
    { 0x4021e308, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b1e30d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1e310, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941e318, 55 },
    { 0x2631e319, 90 },
    { 0, 0 },
    { 0xb4b1e31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51e31e, 33 },
    { 0x1d51e31e, 46 },
    { 0xd681e320, 55 },
    { 0x1f51e320, 36 },
    { 0, 0 },
    { 0x21c1e323, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b01e32b, 88 },
    { 0, 0 },
    { 0xc911e32d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411e330, 33 },
    { 0x7131e331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd31e33b, 77 },
    { 0, 0 },
    { 0x8411e33d, 88 },
    { 0x1b41e33e, 51 },
    { 0x8b21e33f, 95 },
    { 0x9741e340, 66 },
    { 0xa3b1e341, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1e344, 90 },
    { 0, 0 },
    { 0xa701e346, 33 },
    { 0x1541e346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b91e34e, 98 },
    { 0x6931e34f, 99 },
    { 0, 0 },
    { 0xb671e351, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc431e357, 66 },
    { 0, 0 },
    { 0xbde1e359, 43 },
    { 0xd941e35a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861e35d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121e360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff1e366, 95 },
    { 0, 0 },
    { 0xf9e1e368, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c1e36b, 95 },
    { 0, 0 },
    { 0xcf81e36d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe1e370, 46 },
    { 0xf331e371, 90 },
    { 0xc051e372, 98 },
    { 0x3581e373, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a1e378, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f51e383, 100 },
    { 0xf61e383, 91 },
    { 0xa781e385, 95 },
    { 0x93c1e383, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c1e38a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51e38d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd81e392, 68 },
    { 0, 0 },
    { 0xe9e1e394, 34 },
    { 0x1e01e394, 98 },
    { 0xff11e396, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8721e39a, 98 },
    { 0xe4b1e39a, 95 },
    { 0xfe71e39c, 95 },
    { 0x7701e39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b1e3a0, 93 },
    { 0, 0 },
    { 0x471e3a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb851e3aa, 83 },
    { 0xe5e1e3ab, 55 },
    { 0xa6d1e3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761e3b0, 95 },
    { 0, 0 },
    { 0x7b91e3b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651e3bc, 100 },
    { 0, 0 },
    { 0x3671e3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4771e3c1, 33 },
    { 0x8321e3c2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71e3c5, 43 },
    { 0x7741e3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b1e3ca, 23 },
    { 0x491e3cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7871e3d9, 85 },
    { 0x9c31e3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa881e3de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01e3e1, 98 },
    { 0, 0 },
    { 0x6891e3e3, 61 },
    { 0xa671e3e4, 85 },
    { 0x2431e3e4, 87 },
    { 0x1321e3e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41e3e9, 95 },
    { 0x5751e3e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4151e3ed, 23 },
    { 0x6001e3ee, 76 },
    { 0xd9d1e3ee, 93 },
    { 0xbfa1e3f0, 90 },
    { 0x3221e3ee, 90 },
    { 0xc851e3f1, 90 },
    { 0xce51e3f2, 33 },
    { 0xb9a1e3f4, 76 },
    { 0x6771e3f5, 78 },
    { 0xb501e3f6, 36 },
    { 0x6c71e3f7, 23 },
    { 0x941e3f6, 91 },
    { 0x5481e3f6, 98 },
    { 0x5d71e3fa, 93 },
    { 0x4581e3fb, 23 },
    { 0, 0 },
    { 0x9b51e3fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481e400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e1e403, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3101e40f, 90 },
    { 0xb71e40f, 99 },
    { 0, 0 },
    { 0xada1e412, 34 },
    { 0x23f1e413, 76 },
    { 0, 0 },
    { 0xe2c1e415, 80 },
    { 0x9891e416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd651e419, 34 },
    { 0, 0 },
    { 0xb451e41b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c1e41f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731e42b, 90 },
    { 0, 0 },
    { 0x48f1e42d, 87 },
    { 0, 0 },
    { 0x46e1e42f, 55 },
    { 0, 0 },
    { 0xd331e431, 53 },
    { 0x1621e432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b1e435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab1e439, 67 },
    { 0, 0 },
    { 0x3c91e43b, 98 },
    { 0x66f1e43b, 81 },
    { 0xf1f1e43c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d1e443, 33 },
    { 0, 0 },
    { 0xf981e445, 90 },
    { 0x5301e446, 67 },
    { 0x6511e447, 14 },
    { 0, 0 },
    { 0x7461e449, 66 },
    { 0, 0 },
    { 0xfd11e44b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf941e44e, 33 },
    { 0xcf41e44e, 23 },
    { 0xcfd1e450, 76 },
    { 0xee91e451, 88 },
    { 0x6ba1e452, 85 },
    { 0xe641e451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab1e459, 90 },
    { 0x59b1e45a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121e45d, 82 },
    { 0x3461e45e, 61 },
    { 0, 0 },
    { 0xe621e460, 2 },
    { 0xb461e461, 76 },
    { 0x3081e462, 90 },
    { 0x1dc1e462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf71e466, 90 },
    { 0xd3e1e466, 87 },
    { 0xb901e467, 100 },
    { 0x6f01e469, 33 },
    { 0x7bf1e467, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c1e46f, 98 },
    { 0x4551e470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1e473, 33 },
    { 0xeaf1e474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d41e477, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f11e47f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d1e483, 46 },
    { 0xde51e484, 90 },
    { 0, 0 },
    { 0x5801e486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461e48b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a21e490, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea81e494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c1e49a, 23 },
    { 0xa671e49b, 61 },
    { 0xadb1e49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd1e49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc1e4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6641e4ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f1e4b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf681e4b7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1e4be, 33 },
    { 0x4541e4be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001e4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371e4c5, 93 },
    { 0x8591e4c5, 76 },
    { 0, 0 },
    { 0xe3c1e4c8, 34 },
    { 0, 0 },
    { 0xe181e4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc1e4cd, 84 },
    { 0, 0 },
    { 0xc3f1e4cf, 78 },
    { 0, 0 },
    { 0x44f1e4d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a1e4da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a51e4e4, 69 },
    { 0, 0 },
    { 0xd0d1e4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b1e4eb, 55 },
    { 0x93e1e4ec, 67 },
    { 0xead1e4ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111e4f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd091e4f6, 68 },
    { 0x3851e4f6, 55 },
    { 0, 0 },
    { 0xff61e4f9, 34 },
    { 0, 0 },
    { 0x2ac1e4fb, 52 },
    { 0x76d1e4fc, 34 },
    { 0x7911e4fb, 33 },
    { 0, 0 },
    { 0x8121e4ff, 34 },
    { 0xe961e4ff, 43 },
    { 0, 0 },
    { 0xd121e502, 90 },
    { 0xc101e503, 90 },
    { 0x3171e504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7311e508, 61 },
    { 0xd311e509, 34 },
    { 0xc1a1e509, 61 },
    { 0x9e51e50a, 90 },
    { 0x3811e50c, 69 },
    { 0x5911e50c, 55 },
    { 0xd661e50e, 55 },
    { 0, 0 },
    { 0x1561e510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd1e515, 33 },
    { 0, 0 },
    { 0x62b1e517, 95 },
    { 0x81b1e517, 91 },
    { 0x45b1e518, 81 },
    { 0, 0 },
    { 0x6871e51b, 93 },
    { 0xe3f1e51c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef1e51f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b1e522, 23 },
    { 0x7c81e523, 90 },
    { 0x1a1e524, 43 },
    { 0x5841e525, 23 },
    { 0xcc41e525, 99 },
    { 0x7811e523, 23 },
    { 0, 0 },
    { 0x6311e529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf621e52e, 46 },
    { 0x34c1e52f, 85 },
    { 0xad51e52f, 93 },
    { 0, 0 },
    { 0x4981e532, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef1e535, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc11e538, 75 },
    { 0, 0 },
    { 0x9b71e53a, 95 },
    { 0xbc21e53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1e541, 76 },
    { 0, 0 },
    { 0x7cf1e543, 95 },
    { 0, 0 },
    { 0x251e545, 61 },
    { 0, 0 },
    { 0x9ca1e547, 53 },
    { 0x4221e548, 23 },
    { 0x5091e549, 78 },
    { 0x8e11e54a, 61 },
    { 0, 0 },
    { 0x4371e54c, 95 },
    { 0x9371e54d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001e551, 84 },
    { 0x4971e552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d71e555, 33 },
    { 0x9861e556, 90 },
    { 0, 0 },
    { 0x2621e558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1e55c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a1e55f, 61 },
    { 0xc371e55f, 93 },
    { 0x49e1e561, 98 },
    { 0x9a41e561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b1e569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa491e572, 66 },
    { 0x8f11e573, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc091e576, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1e579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581e57e, 98 },
    { 0, 0 },
    { 0xc1b1e580, 61 },
    { 0, 0 },
    { 0x8181e582, 43 },
    { 0x971e582, 78 },
    { 0, 0 },
    { 0x4141e585, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71e58f, 61 },
    { 0xb271e590, 98 },
    { 0xd781e591, 55 },
    { 0xd061e590, 33 },
    { 0xa4c1e58f, 98 },
    { 0, 0 },
    { 0x82f1e595, 99 },
    { 0, 0 },
    { 0x3bf1e597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff1e59b, 55 },
    { 0xfe31e59b, 90 },
    { 0, 0 },
    { 0x8921e59e, 95 },
    { 0x26d1e59f, 95 },
    { 0x9811e59f, 61 },
    { 0x26b1e59f, 98 },
    { 0, 0 },
    { 0x3d1e5a3, 71 },
    { 0x4d21e5a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1e5a9, 68 },
    { 0, 0 },
    { 0x7e41e5ab, 90 },
    { 0x4fd1e5ac, 55 },
    { 0x5d21e5ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed41e5b6, 34 },
    { 0, 0 },
    { 0xe811e5b8, 33 },
    { 0, 0 },
    { 0x9eb1e5ba, 34 },
    { 0x8a31e5bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf31e5be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1e5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1e5c6, 100 },
    { 0x61f1e5c7, 84 },
    { 0x6b1e5c6, 93 },
    { 0x87c1e5c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d1e5d0, 33 },
    { 0xfa01e5d0, 83 },
    { 0x2f71e5d2, 76 },
    { 0, 0 },
    { 0x69e1e5d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc1e5da, 23 },
    { 0, 0 },
    { 0x9e11e5dc, 95 },
    { 0x6ef1e5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f1e5e2, 51 },
    { 0, 0 },
    { 0x8a41e5e4, 33 },
    { 0, 0 },
    { 0x4e71e5e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3811e5ee, 98 },
    { 0x9fc1e5ef, 98 },
    { 0, 0 },
    { 0x4b31e5f1, 61 },
    { 0x4961e5f2, 64 },
    { 0x5611e5f2, 95 },
    { 0xe9a1e5f4, 90 },
    { 0x2001e5f3, 90 },
    { 0xeba1e5f4, 55 },
    { 0, 0 },
    { 0x2231e5f8, 96 },
    { 0, 0 },
    { 0xc981e5fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x521e5fd, 95 },
    { 0x6b1e5fe, 90 },
    { 0x2f81e5ff, 90 },
    { 0x3c01e5ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd1e615, 33 },
    { 0x7101e616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61e619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db1e61c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a81e623, 61 },
    { 0x31c1e624, 77 },
    { 0x26e1e625, 90 },
    { 0x2341e626, 61 },
    { 0xa611e626, 90 },
    { 0xf5a1e624, 95 },
    { 0xc3b1e629, 98 },
    { 0xddf1e629, 98 },
    { 0x1861e629, 52 },
    { 0xa781e62c, 76 },
    { 0xd771e628, 33 },
    { 0x3561e62e, 90 },
    { 0x1281e62a, 90 },
    { 0, 0 },
    { 0x6b21e631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1e636, 33 },
    { 0x81e637, 95 },
    { 0x2581e638, 53 },
    { 0x9c61e637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a51e63c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e63f, 34 },
    { 0xf3c1e640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1731e643, 46 },
    { 0x6761e643, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace1e649, 76 },
    { 0, 0 },
    { 0xfde1e64b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd471e64f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa1e652, 90 },
    { 0, 0 },
    { 0x131e654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef01e660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9911e667, 90 },
    { 0x89c1e668, 75 },
    { 0x53e1e668, 67 },
    { 0xf681e66a, 90 },
    { 0xfc21e66b, 61 },
    { 0, 0 },
    { 0xa2a1e66d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa131e671, 98 },
    { 0, 0 },
    { 0x3c51e673, 86 },
    { 0, 0 },
    { 0xf6a1e675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a1e678, 40 },
    { 0, 0 },
    { 0x3ab1e67a, 61 },
    { 0xdf21e67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2041e680, 23 },
    { 0, 0 },
    { 0xff81e682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd41e688, 61 },
    { 0x7f51e689, 23 },
    { 0x34e1e689, 73 },
    { 0x64a1e688, 33 },
    { 0x7c21e689, 23 },
    { 0, 0 },
    { 0x3851e68e, 23 },
    { 0x8761e68f, 98 },
    { 0, 0 },
    { 0xf561e691, 76 },
    { 0, 0 },
    { 0x7b11e693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb831e69b, 53 },
    { 0x2f91e69c, 81 },
    { 0, 0 },
    { 0xd1f1e69e, 90 },
    { 0xa051e69f, 61 },
    { 0x83b1e6a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e1e6a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9051e6a9, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xc421e6ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e21e6af, 52 },
    { 0xa911e6b0, 55 },
    { 0, 0 },
    { 0xd221e6b2, 90 },
    { 0x2551e6b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf31e6b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681e6b9, 23 },
    { 0x1d01e6ba, 66 },
    { 0x68e1e6bb, 23 },
    { 0xfaf1e6bc, 61 },
    { 0xab81e6ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61e6c1, 90 },
    { 0x57a1e6c1, 97 },
    { 0xe701e6c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81e6ce, 90 },
    { 0xc441e6cf, 90 },
    { 0x7811e6cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1e6d3, 61 },
    { 0xc641e6d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521e6d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71e6db, 98 },
    { 0x35b1e6dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe671e6e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe351e6e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311e6ea, 98 },
    { 0, 0 },
    { 0x1441e6ec, 61 },
    { 0xaef1e6ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d1e6f0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x8521e6f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991e6fa, 61 },
    { 0xf8b1e6fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa941e6ff, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81e705, 95 },
    { 0x2a51e706, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e1e70a, 90 },
    { 0x36a1e70a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e712, 33 },
    { 0x4bb1e713, 93 },
    { 0xe8c1e712, 69 },
    { 0xdf11e715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e1e71c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd561e71f, 93 },
    { 0xe291e720, 100 },
    { 0, 0 },
    { 0x96c1e722, 76 },
    { 0, 0 },
    { 0x3291e724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1e72d, 23 },
    { 0x2301e72e, 46 },
    { 0xdf91e72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a1e738, 33 },
    { 0xf8f1e739, 23 },
    { 0x1471e739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b41e73d, 93 },
    { 0, 0 },
    { 0xa991e73f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e71e743, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401e748, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d1e74c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4251e74f, 33 },
    { 0x5b41e750, 98 },
    { 0, 0 },
    { 0x78e1e752, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc51e756, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7771e759, 85 },
    { 0x27e1e75a, 88 },
    { 0x98c1e759, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671e75e, 55 },
    { 0xc861e75e, 33 },
    { 0xfca1e760, 43 },
    { 0x42d1e75e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd1e769, 91 },
    { 0xecc1e769, 55 },
    { 0xe1c1e76a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841e774, 77 },
    { 0x6e61e775, 92 },
    { 0x7d91e774, 61 },
    { 0x1001e775, 55 },
    { 0xfa41e778, 66 },
    { 0xa5f1e778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b1e77d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e61e780, 90 },
    { 0x7e41e781, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e784, 66 },
    { 0x5f91e785, 43 },
    { 0x89a1e785, 46 },
    { 0x6171e787, 33 },
    { 0, 0 },
    { 0x9f21e789, 55 },
    { 0x781e78a, 93 },
    { 0x77d1e78b, 90 },
    { 0xe2b1e78c, 61 },
    { 0x731e78d, 23 },
    { 0x5eb1e78c, 90 },
    { 0x1261e78f, 87 },
    { 0xf761e78d, 38 },
    { 0x83b1e791, 55 },
    { 0x1531e78d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911e795, 34 },
    { 0x7a41e795, 88 },
    { 0, 0 },
    { 0xece1e798, 84 },
    { 0, 0 },
    { 0xc5e1e79a, 33 },
    { 0x96b1e79b, 61 },
    { 0, 0 },
    { 0x6901e79d, 33 },
    { 0, 0 },
    { 0x7a81e79f, 95 },
    { 0, 0 },
    { 0x4491e7a1, 61 },
    { 0x5bb1e7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81e7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771e7a9, 98 },
    { 0xcca1e7aa, 80 },
    { 0xaee1e7aa, 98 },
    { 0xf61e7ac, 33 },
    { 0xd511e7ac, 66 },
    { 0xe9d1e7ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf321e7b5, 67 },
    { 0xcb81e7b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d1e7bb, 47 },
    { 0x2231e7bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5621e7bf, 61 },
    { 0, 0 },
    { 0x2331e7c1, 95 },
    { 0x3ab1e7c2, 23 },
    { 0x8ae1e7c3, 98 },
    { 0x8171e7c3, 33 },
    { 0xf9c1e7c4, 90 },
    { 0xfbc1e7c3, 33 },
    { 0x8721e7c7, 23 },
    { 0xb521e7c8, 100 },
    { 0x72e1e7c1, 95 },
    { 0xcc01e7c7, 95 },
    { 0x60a1e7c5, 100 },
    { 0xa331e7cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151e7cf, 85 },
    { 0xab61e7d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f1e7d9, 34 },
    { 0xd91e7d9, 76 },
    { 0, 0 },
    { 0xcc1e7dc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e7e1, 61 },
    { 0, 0 },
    { 0xc911e7e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2851e7e6, 90 },
    { 0, 0 },
    { 0x5bd1e7e8, 90 },
    { 0x9f81e7e8, 98 },
    { 0x8121e7ea, 87 },
    { 0x71d1e7eb, 33 },
    { 0x7a51e7e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b01e7ef, 90 },
    { 0x81e7f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81e7f6, 98 },
    { 0x8421e7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991e7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3961e7fe, 90 },
    { 0x9c41e7ff, 95 },
    { 0, 0 },
    { 0xf811e801, 87 },
    { 0x74c1e802, 34 },
    { 0, 0 },
    { 0xbba1e804, 93 },
    { 0, 0 },
    { 0xd051e806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1e80c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5941e810, 98 },
    { 0, 0 },
    { 0x2791e812, 90 },
    { 0x8251e813, 61 },
    { 0x1531e814, 34 },
    { 0x9161e815, 55 },
    { 0, 0 },
    { 0x5121e817, 78 },
    { 0xa821e818, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e1e81b, 81 },
    { 0xf381e81c, 76 },
    { 0x1d81e81d, 61 },
    { 0xabb1e81e, 34 },
    { 0xa361e81f, 95 },
    { 0xd011e81d, 77 },
    { 0, 0 },
    { 0x3091e822, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe441e829, 61 },
    { 0x1151e82a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2831e82f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a1e833, 75 },
    { 0x3661e834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa091e839, 34 },
    { 0xf5e1e83a, 90 },
    { 0, 0 },
    { 0x5931e83c, 87 },
    { 0x5151e83d, 55 },
    { 0xa161e83d, 78 },
    { 0x4d1e83c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581e847, 33 },
    { 0x1a61e848, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411e84b, 90 },
    { 0x3921e84c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb71e84f, 90 },
    { 0x5851e850, 46 },
    { 0x1791e850, 95 },
    { 0x1231e850, 76 },
    { 0, 0 },
    { 0x9be1e854, 34 },
    { 0, 0 },
    { 0xddb1e856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e1e85a, 33 },
    { 0x5f11e85a, 78 },
    { 0xe731e85c, 66 },
    { 0, 0 },
    { 0x9531e85e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc1e863, 95 },
    { 0, 0 },
    { 0x2fd1e865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d1e86c, 61 },
    { 0x7c61e86d, 95 },
    { 0x9261e86e, 100 },
    { 0x9501e86f, 91 },
    { 0xeb1e86f, 98 },
    { 0, 0 },
    { 0xba71e872, 61 },
    { 0, 0 },
    { 0xb621e874, 95 },
    { 0x74b1e875, 84 },
    { 0xc31e874, 95 },
    { 0, 0 },
    { 0xf801e878, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71e87b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d1e87f, 90 },
    { 0xf101e880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491e884, 61 },
    { 0x8401e884, 14 },
    { 0x29c1e886, 33 },
    { 0x92c1e887, 61 },
    { 0, 0 },
    { 0x7681e889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811e88e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed1e892, 34 },
    { 0xc6f1e893, 85 },
    { 0, 0 },
    { 0x2551e895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a1e899, 34 },
    { 0xad11e89a, 67 },
    { 0x5771e89a, 33 },
    { 0, 0 },
    { 0x2cc1e89d, 30 },
    { 0xa591e89d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e8a2, 90 },
    { 0, 0 },
    { 0xdad1e8a4, 61 },
    { 0x9321e8a4, 33 },
    { 0xa6f1e8a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d41e8ac, 33 },
    { 0x3111e8ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1e8b0, 85 },
    { 0x5231e8b0, 43 },
    { 0xad1e8b1, 43 },
    { 0xebd1e8b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb661e8b7, 61 },
    { 0x641e8b8, 23 },
    { 0x8cd1e8b7, 90 },
    { 0xf0b1e8b7, 90 },
    { 0xfcc1e8b8, 76 },
    { 0x2341e8bc, 87 },
    { 0x60f1e8bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e31e8c1, 90 },
    { 0xb661e8c1, 95 },
    { 0x3821e8c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd651e8cd, 92 },
    { 0x3ae1e8ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1e8d1, 61 },
    { 0, 0 },
    { 0xe7c1e8d3, 76 },
    { 0, 0 },
    { 0xb7b1e8d5, 76 },
    { 0x1181e8d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3871e8da, 90 },
    { 0x6481e8da, 95 },
    { 0xf621e8dc, 95 },
    { 0x5af1e8da, 55 },
    { 0x10b1e8dd, 55 },
    { 0xd261e8dc, 55 },
    { 0x6521e8e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e1e8e4, 61 },
    { 0, 0 },
    { 0x3c21e8e6, 61 },
    { 0x5ab1e8e7, 90 },
    { 0, 0 },
    { 0x7a21e8e9, 85 },
    { 0xc671e8e9, 34 },
    { 0, 0 },
    { 0x9d11e8ec, 23 },
    { 0xe741e8ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c1e8f2, 90 },
    { 0x77f1e8f3, 97 },
    { 0xc8b1e8f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5971e8f7, 68 },
    { 0x50c1e8f8, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651e8fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391e900, 61 },
    { 0, 0 },
    { 0xbe81e902, 23 },
    { 0x2c31e902, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4131e909, 91 },
    { 0xe7d1e90a, 98 },
    { 0xf3c1e90a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed1e90f, 23 },
    { 0, 0 },
    { 0xb541e911, 66 },
    { 0x36a1e912, 34 },
    { 0x98c1e912, 33 },
    { 0xa001e914, 90 },
    { 0x40c1e915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c1e91a, 78 },
    { 0x89f1e91a, 99 },
    { 0xd571e91c, 78 },
    { 0x62b1e91d, 98 },
    { 0x6041e91b, 85 },
    { 0, 0 },
    { 0xfa61e920, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821e923, 17 },
    { 0x3e01e923, 23 },
    { 0x861e923, 46 },
    { 0xefe1e926, 61 },
    { 0x15f1e927, 90 },
    { 0xc191e924, 66 },
    { 0, 0 },
    { 0xf5a1e92a, 90 },
    { 0xb871e92b, 23 },
    { 0, 0 },
    { 0x3441e92d, 90 },
    { 0x5921e92d, 23 },
    { 0xb8c1e92e, 76 },
    { 0x5be1e92f, 33 },
    { 0xff51e931, 85 },
    { 0x5671e931, 84 },
    { 0xa6e1e92e, 90 },
    { 0x5d51e934, 99 },
    { 0x9a11e92d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1e93a, 23 },
    { 0xaed1e93b, 90 },
    { 0x5281e93b, 61 },
    { 0xf421e93d, 23 },
    { 0, 0 },
    { 0x4f11e93f, 43 },
    { 0x5011e93f, 46 },
    { 0x1741e941, 98 },
    { 0, 0 },
    { 0x6a21e943, 61 },
    { 0x3f11e944, 61 },
    { 0x4151e945, 93 },
    { 0xfda1e945, 33 },
    { 0xa6e1e945, 33 },
    { 0x1f21e944, 55 },
    { 0x2b21e943, 90 },
    { 0, 0 },
    { 0xeec1e94b, 61 },
    { 0x44d1e94c, 100 },
    { 0x4d91e94d, 98 },
    { 0x731e94e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec1e951, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b01e956, 23 },
    { 0, 0 },
    { 0xa571e958, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581e95c, 23 },
    { 0x6941e95d, 55 },
    { 0xc341e95e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1e963, 61 },
    { 0xeaa1e964, 90 },
    { 0x9bd1e963, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1e969, 23 },
    { 0, 0 },
    { 0x311e96b, 98 },
    { 0x1a71e96b, 23 },
    { 0x5401e96b, 90 },
    { 0xbc1e96c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb1e971, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1331e976, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba1e979, 23 },
    { 0xb441e979, 34 },
    { 0x17b1e97b, 95 },
    { 0x5231e97c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161e982, 61 },
    { 0x6d61e982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f31e986, 75 },
    { 0xdb61e986, 33 },
    { 0xad41e988, 43 },
    { 0, 0 },
    { 0x7711e98a, 61 },
    { 0, 0 },
    { 0x1871e98c, 46 },
    { 0xebd1e98d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a1e994, 85 },
    { 0x4c11e994, 43 },
    { 0x3b21e996, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a81e99a, 43 },
    { 0, 0 },
    { 0xb821e99c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea41e99f, 98 },
    { 0, 0 },
    { 0xe631e9a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d1e9a4, 66 },
    { 0x3321e9a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111e9a8, 98 },
    { 0x1751e9a9, 43 },
    { 0x7101e9aa, 68 },
    { 0x89e1e9ab, 23 },
    { 0x5111e9aa, 17 },
    { 0x7621e9ac, 65 },
    { 0, 0 },
    { 0xcce1e9af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591e9b4, 85 },
    { 0x1ab1e9b5, 98 },
    { 0x7931e9b6, 23 },
    { 0, 0 },
    { 0x56e1e9b8, 76 },
    { 0xf411e9b9, 61 },
    { 0xdf21e9b8, 33 },
    { 0x1ee1e9bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef1e9be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41e9c6, 23 },
    { 0x68b1e9c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1e9d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b1e9d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c1e9db, 52 },
    { 0xf0b1e9dc, 81 },
    { 0xc31e9dc, 5 },
    { 0, 0 },
    { 0x17e1e9df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21e9e4, 69 },
    { 0, 0 },
    { 0x51e1e9e6, 33 },
    { 0, 0 },
    { 0x39b1e9e8, 76 },
    { 0, 0 },
    { 0x6921e9ea, 98 },
    { 0x5251e9eb, 98 },
    { 0x3271e9ec, 88 },
    { 0x5001e9ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe1e9f1, 66 },
    { 0xf661e9f2, 43 },
    { 0x1e01e9f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71e9f6, 61 },
    { 0xad01e9f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5641e9fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd31ea00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff41ea03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e1ea0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1271ea11, 78 },
    { 0xfff1ea12, 98 },
    { 0x8151ea12, 33 },
    { 0, 0 },
    { 0x8db1ea15, 40 },
    { 0, 0 },
    { 0x1d51ea17, 90 },
    { 0x8381ea18, 98 },
    { 0, 0 },
    { 0x6f81ea1a, 93 },
    { 0x9801ea1b, 76 },
    { 0x9df1ea1c, 46 },
    { 0x72c1ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8621ea22, 85 },
    { 0x7791ea23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1ea2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1ea31, 98 },
    { 0x7db1ea31, 17 },
    { 0xf921ea32, 99 },
    { 0x80e1ea31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3981ea3a, 46 },
    { 0x94e1ea3a, 69 },
    { 0x3521ea3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1131ea40, 98 },
    { 0x7f01ea41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1ea47, 46 },
    { 0, 0 },
    { 0xd4e1ea49, 90 },
    { 0x3d91ea49, 38 },
    { 0, 0 },
    { 0x1da1ea4c, 90 },
    { 0, 0 },
    { 0x2d01ea4e, 90 },
    { 0x8511ea4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b1ea5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341ea60, 98 },
    { 0xbb11ea60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5511ea65, 98 },
    { 0x8eb1ea65, 95 },
    { 0, 0 },
    { 0xeed1ea68, 23 },
    { 0x2c1ea68, 23 },
    { 0x3cf1ea69, 98 },
    { 0x3b01ea6b, 91 },
    { 0x8e21ea69, 90 },
    { 0xdd41ea6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f21ea72, 61 },
    { 0x8d31ea73, 23 },
    { 0, 0 },
    { 0x1d81ea75, 95 },
    { 0x18b1ea75, 99 },
    { 0x81b1ea75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea51ea7a, 90 },
    { 0xcb01ea7a, 88 },
    { 0xa261ea7c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd981ea81, 61 },
    { 0, 0 },
    { 0xeee1ea83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2391ea86, 84 },
    { 0xfc21ea87, 83 },
    { 0, 0 },
    { 0x4d91ea89, 81 },
    { 0x301ea8a, 61 },
    { 0xe11ea8a, 98 },
    { 0, 0 },
    { 0xfca1ea8d, 99 },
    { 0, 0 },
    { 0xc761ea8f, 98 },
    { 0x4d61ea90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271ea96, 61 },
    { 0xda31ea97, 30 },
    { 0, 0 },
    { 0x8c61ea99, 33 },
    { 0, 0 },
    { 0xe671ea9b, 90 },
    { 0, 0 },
    { 0x5261ea9d, 88 },
    { 0, 0 },
    { 0x92b1ea9f, 52 },
    { 0x4711ea9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eaa5, 23 },
    { 0x1341eaa6, 43 },
    { 0x4e21eaa7, 33 },
    { 0x3351eaa8, 90 },
    { 0xd3d1eaa9, 90 },
    { 0xb531eaa8, 90 },
    { 0x1811eaaa, 98 },
    { 0xba91eaac, 91 },
    { 0x37d1eaac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681eab2, 90 },
    { 0xe501eab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf61eab8, 90 },
    { 0, 0 },
    { 0x2311eaba, 69 },
    { 0xa4f1eaba, 61 },
    { 0x7c21eabc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2081eac2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61eac9, 93 },
    { 0x2381eaca, 98 },
    { 0x151eacb, 84 },
    { 0, 0 },
    { 0xfd31eacd, 23 },
    { 0x1441eace, 34 },
    { 0, 0 },
    { 0x2551ead0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad1ead3, 99 },
    { 0, 0 },
    { 0x7751ead5, 33 },
    { 0xe21ead6, 43 },
    { 0xd8e1ead7, 69 },
    { 0, 0 },
    { 0x5da1ead9, 33 },
    { 0xe5f1eada, 69 },
    { 0xbbf1eadb, 95 },
    { 0, 0 },
    { 0x9351eadd, 90 },
    { 0x20b1eade, 33 },
    { 0, 0 },
    { 0x1f1eae0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1eae4, 76 },
    { 0xf301eae4, 55 },
    { 0xbbb1eae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1eae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61eaec, 52 },
    { 0x5ed1eaed, 87 },
    { 0x1011eaee, 33 },
    { 0x3181eaee, 33 },
    { 0x82c1eaf0, 90 },
    { 0x7901eaf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61eaf4, 33 },
    { 0x1c1eaf5, 95 },
    { 0xf2c1eaf6, 90 },
    { 0x9841eaf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba81eafa, 95 },
    { 0xfa21eafb, 61 },
    { 0x6e1eafc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd1eb00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e21eb03, 33 },
    { 0, 0 },
    { 0xdbf1eb05, 46 },
    { 0x27d1eb05, 93 },
    { 0xb1d1eb07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe411eb0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7411eb11, 81 },
    { 0, 0 },
    { 0x2121eb13, 23 },
    { 0x5fc1eb14, 61 },
    { 0xdde1eb15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc81eb1c, 84 },
    { 0xcd81eb1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc41eb21, 90 },
    { 0xbeb1eb22, 43 },
    { 0xe831eb21, 95 },
    { 0, 0 },
    { 0xe151eb25, 95 },
    { 0x68f1eb26, 65 },
    { 0, 0 },
    { 0x6201eb28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7481eb2f, 61 },
    { 0, 0 },
    { 0xeb61eb31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb51eb38, 46 },
    { 0x5851eb38, 23 },
    { 0x2611eb39, 43 },
    { 0, 0 },
    { 0xb1e1eb3c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4421eb3f, 100 },
    { 0x7da1eb3f, 33 },
    { 0x63f1eb3f, 93 },
    { 0x5a41eb3f, 90 },
    { 0x8541eb42, 33 },
    { 0xcea1eb44, 98 },
    { 0, 0 },
    { 0x8941eb46, 61 },
    { 0xab61eb47, 91 },
    { 0xdd91eb47, 90 },
    { 0xa6b1eb49, 98 },
    { 0xe401eb4a, 61 },
    { 0x2e41eb4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf181eb54, 23 },
    { 0, 0 },
    { 0xe5f1eb56, 66 },
    { 0x2bc1eb56, 33 },
    { 0, 0 },
    { 0xbaa1eb59, 33 },
    { 0x4a71eb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c1eb63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f51eb66, 43 },
    { 0xdf61eb67, 55 },
    { 0x3631eb68, 100 },
    { 0xc771eb66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b1eb6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e1eb71, 85 },
    { 0xae31eb71, 95 },
    { 0, 0 },
    { 0xf841eb74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e1eb7c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1831eb7f, 68 },
    { 0xeb81eb80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad31eb84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb701eb89, 95 },
    { 0, 0 },
    { 0xcb21eb8b, 95 },
    { 0, 0 },
    { 0x5f71eb8d, 23 },
    { 0, 0 },
    { 0xd381eb8f, 95 },
    { 0x3281eb90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2211eb93, 84 },
    { 0xe761eb94, 100 },
    { 0x18d1eb94, 40 },
    { 0xfec1eb96, 96 },
    { 0xa41eb97, 95 },
    { 0xfc91eb98, 90 },
    { 0x6bb1eb99, 33 },
    { 0x6481eb99, 99 },
    { 0x4aa1eb98, 80 },
    { 0xf991eb9c, 46 },
    { 0xc951eb9c, 61 },
    { 0x29c1eb98, 90 },
    { 0xe061eb9f, 17 },
    { 0x9351eba0, 46 },
    { 0xd3f1eb98, 99 },
    { 0xa711eba2, 95 },
    { 0xe371eba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff1ebac, 61 },
    { 0, 0 },
    { 0x10a1ebae, 95 },
    { 0x2a01ebae, 61 },
    { 0, 0 },
    { 0x2da1ebb1, 93 },
    { 0x5491ebb2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x871ebb5, 98 },
    { 0x9b31ebb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc91ebba, 98 },
    { 0, 0 },
    { 0xe81ebbc, 33 },
    { 0x3fb1ebbd, 90 },
    { 0x3851ebbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61ebc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f81ebc7, 33 },
    { 0xaa81ebc8, 98 },
    { 0x8591ebc9, 88 },
    { 0x9e11ebc8, 46 },
    { 0, 0 },
    { 0xa3d1ebcc, 38 },
    { 0, 0 },
    { 0xd7e1ebce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f1ebd3, 95 },
    { 0xbb91ebd4, 98 },
    { 0xd7c1ebd5, 95 },
    { 0, 0 },
    { 0x2621ebd7, 61 },
    { 0xb5f1ebd8, 33 },
    { 0xef01ebd8, 95 },
    { 0x1381ebda, 88 },
    { 0xf3b1ebd9, 95 },
    { 0x4da1ebdc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d1ebe3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d1ebe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ebea, 61 },
    { 0xfbc1ebeb, 34 },
    { 0xbe01ebea, 75 },
    { 0, 0 },
    { 0x71d1ebee, 90 },
    { 0, 0 },
    { 0xd061ebf0, 93 },
    { 0, 0 },
    { 0xe51ebf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1ebf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731ebfb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2731ebfe, 76 },
    { 0x3371ebff, 43 },
    { 0, 0 },
    { 0xc291ec01, 34 },
    { 0, 0 },
    { 0x4931ec03, 98 },
    { 0x8d71ec04, 95 },
    { 0x1711ec05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef1ec11, 91 },
    { 0xc181ec12, 52 },
    { 0x21f1ec12, 61 },
    { 0, 0 },
    { 0x9471ec15, 43 },
    { 0x2d21ec16, 34 },
    { 0x2701ec16, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6541ec1c, 33 },
    { 0x97d1ec1d, 38 },
    { 0, 0 },
    { 0x5c71ec1f, 85 },
    { 0, 0 },
    { 0xe11ec21, 76 },
    { 0x9b41ec22, 82 },
    { 0x8e81ec22, 30 },
    { 0xdd1ec23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2081ec2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d1ec30, 68 },
    { 0x331ec31, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31ec36, 78 },
    { 0xedb1ec37, 33 },
    { 0x1b1ec38, 33 },
    { 0xa71ec39, 43 },
    { 0xa5f1ec3a, 55 },
    { 0, 0 },
    { 0x84a1ec3c, 65 },
    { 0xe4b1ec3d, 90 },
    { 0x16a1ec3c, 61 },
    { 0xd9f1ec3f, 76 },
    { 0xe7d1ec3f, 84 },
    { 0x38a1ec41, 46 },
    { 0x53b1ec41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2851ec48, 100 },
    { 0x2701ec49, 98 },
    { 0, 0 },
    { 0xd4f1ec4b, 76 },
    { 0x62a1ec4c, 76 },
    { 0, 0 },
    { 0x16e1ec4e, 55 },
    { 0xf771ec4e, 33 },
    { 0x2041ec50, 23 },
    { 0xd9c1ec4e, 23 },
    { 0xec31ec51, 61 },
    { 0x4671ec53, 100 },
    { 0xe371ec54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6551ec59, 93 },
    { 0x5491ec5a, 85 },
    { 0xd81ec5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1ec5e, 67 },
    { 0, 0 },
    { 0x66a1ec60, 90 },
    { 0x2e1ec61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1ec64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc471ec67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da1ec6a, 23 },
    { 0x2841ec6a, 33 },
    { 0xc611ec6a, 17 },
    { 0, 0 },
    { 0xda31ec6e, 61 },
    { 0, 0 },
    { 0xba21ec70, 46 },
    { 0x6f41ec71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431ec79, 34 },
    { 0, 0 },
    { 0xba21ec7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1ec82, 23 },
    { 0x7f21ec83, 33 },
    { 0x6f21ec84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21ec88, 94 },
    { 0, 0 },
    { 0x1a21ec8a, 68 },
    { 0xbf31ec8a, 23 },
    { 0x88c1ec8a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d1ec90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee1ec96, 34 },
    { 0xcd81ec96, 100 },
    { 0xb591ec96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61ec9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe121eca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e01eca5, 33 },
    { 0, 0 },
    { 0x5801eca7, 33 },
    { 0x8fe1eca7, 55 },
    { 0x8f41eca9, 76 },
    { 0x2bc1ecaa, 33 },
    { 0xff91ecab, 55 },
    { 0x3ac1ecac, 95 },
    { 0xeca1eca7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3161ecb2, 90 },
    { 0x96a1ecb3, 34 },
    { 0, 0 },
    { 0x93e1ecb5, 23 },
    { 0x5e41ecb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e1ecba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1ecc0, 87 },
    { 0, 0 },
    { 0xc041ecc2, 88 },
    { 0xa331ecc3, 76 },
    { 0x8591ecc3, 99 },
    { 0xf411ecc5, 85 },
    { 0, 0 },
    { 0x9721ecc7, 93 },
    { 0x6811ecc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d1eccc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b1ecd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d1ecd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde1ecd9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4201ecde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8781ece1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe161ece6, 66 },
    { 0x451ece7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841ecf9, 33 },
    { 0, 0 },
    { 0xb51ecfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e1ecff, 77 },
    { 0xf0b1ed00, 90 },
    { 0x22b1ed00, 90 },
    { 0, 0 },
    { 0xf001ed03, 90 },
    { 0x2db1ed04, 76 },
    { 0x3891ed03, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f11ed08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf331ed0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd481ed10, 76 },
    { 0, 0 },
    { 0xc71ed12, 78 },
    { 0, 0 },
    { 0xf741ed14, 95 },
    { 0x2d91ed15, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1ed1d, 87 },
    { 0x20d1ed1e, 95 },
    { 0xd6c1ed1f, 93 },
    { 0x3ba1ed1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be1ed29, 90 },
    { 0xbbe1ed2a, 90 },
    { 0xee01ed2b, 90 },
    { 0xcae1ed2a, 66 },
    { 0, 0 },
    { 0x4e61ed2e, 85 },
    { 0x2b41ed2f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe281ed33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf591ed36, 95 },
    { 0x15f1ed37, 34 },
    { 0xf481ed38, 76 },
    { 0xabb1ed39, 33 },
    { 0x4811ed39, 67 },
    { 0x321ed3b, 61 },
    { 0xdea1ed3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf01ed40, 98 },
    { 0x8671ed40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3961ed44, 99 },
    { 0xf51ed45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01ed4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541ed53, 23 },
    { 0x2041ed54, 33 },
    { 0x3801ed55, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb1ed59, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1ed5c, 81 },
    { 0x5ab1ed5d, 90 },
    { 0x6ca1ed5e, 89 },
    { 0x74f1ed5c, 90 },
    { 0, 0 },
    { 0xa821ed61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc1ed66, 90 },
    { 0xf0f1ed67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d1ed6a, 87 },
    { 0x7b31ed6b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1ed6f, 99 },
    { 0xcf71ed70, 75 },
    { 0xc6e1ed6f, 33 },
    { 0x1da1ed72, 33 },
    { 0x9801ed70, 34 },
    { 0x1821ed74, 91 },
    { 0x77a1ed74, 23 },
    { 0x70f1ed6f, 66 },
    { 0x4d1ed73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531ed7a, 43 },
    { 0x1f91ed7a, 93 },
    { 0x5e21ed7c, 43 },
    { 0, 0 },
    { 0x59f1ed7e, 52 },
    { 0xe391ed7e, 95 },
    { 0x7381ed80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1ed84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b31ed8c, 23 },
    { 0, 0 },
    { 0x79f1ed8e, 23 },
    { 0, 0 },
    { 0xcc41ed90, 61 },
    { 0x17c1ed91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ed94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1ed98, 61 },
    { 0x1201ed99, 43 },
    { 0xb861ed9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8011eda0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc531eda3, 33 },
    { 0, 0 },
    { 0x3091eda5, 87 },
    { 0, 0 },
    { 0x2ba1eda7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7701edad, 55 },
    { 0xdcd1edae, 76 },
    { 0, 0 },
    { 0xf2d1edb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb601edb3, 36 },
    { 0, 0 },
    { 0x78f1edb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb1edb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc11edbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1edc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131edc7, 98 },
    { 0xc9f1edc8, 84 },
    { 0x7ee1edc7, 33 },
    { 0x2fb1edca, 90 },
    { 0, 0 },
    { 0x7361edcc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11edd1, 61 },
    { 0xaa81edd1, 87 },
    { 0x48d1edd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ede0, 23 },
    { 0, 0 },
    { 0x8b41ede2, 23 },
    { 0, 0 },
    { 0xdd1ede4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71ede7, 98 },
    { 0xa81ede8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d1eded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1edf1, 90 },
    { 0, 0 },
    { 0xd311edf3, 33 },
    { 0, 0 },
    { 0x291edf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff1edf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc1ee02, 90 },
    { 0x7f11ee03, 100 },
    { 0x4fc1ee04, 98 },
    { 0xce81ee05, 43 },
    { 0xee61ee05, 61 },
    { 0xbb51ee07, 33 },
    { 0xa241ee08, 76 },
    { 0x54a1ee09, 67 },
    { 0x7b11ee0a, 98 },
    { 0, 0 },
    { 0x82c1ee0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a1ee10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f71ee14, 90 },
    { 0xd511ee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8731ee18, 99 },
    { 0x5071ee19, 33 },
    { 0x2da1ee1a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21ee23, 88 },
    { 0xd8d1ee24, 33 },
    { 0xd791ee24, 68 },
    { 0xb281ee26, 88 },
    { 0x6331ee26, 66 },
    { 0x47e1ee23, 61 },
    { 0x3351ee26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b1ee2d, 33 },
    { 0x1061ee2e, 61 },
    { 0xb8f1ee2d, 96 },
    { 0, 0 },
    { 0x40b1ee31, 85 },
    { 0xec1ee31, 67 },
    { 0x4241ee32, 61 },
    { 0x9061ee34, 98 },
    { 0x4351ee35, 43 },
    { 0x6521ee36, 46 },
    { 0xab91ee32, 34 },
    { 0xc781ee33, 43 },
    { 0xa6e1ee31, 23 },
    { 0x3ae1ee3a, 46 },
    { 0x8671ee3a, 76 },
    { 0x131ee3c, 23 },
    { 0x1dc1ee34, 23 },
    { 0x6bb1ee3d, 97 },
    { 0, 0 },
    { 0xcad1ee40, 52 },
    { 0xd851ee41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc591ee45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81ee4a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d81ee54, 61 },
    { 0x4081ee54, 87 },
    { 0x1f61ee56, 78 },
    { 0xd11ee54, 33 },
    { 0xcd71ee55, 93 },
    { 0xa5d1ee55, 61 },
    { 0xef41ee5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d21ee5d, 33 },
    { 0x9641ee5e, 52 },
    { 0, 0 },
    { 0xd451ee60, 95 },
    { 0x9d21ee60, 67 },
    { 0xce71ee62, 61 },
    { 0xbc1ee63, 76 },
    { 0x2a31ee60, 95 },
    { 0x361ee65, 33 },
    { 0x2e81ee62, 98 },
    { 0xcf01ee62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2161ee6e, 84 },
    { 0x9161ee6f, 46 },
    { 0xcaa1ee70, 51 },
    { 0xe3c1ee71, 17 },
    { 0xb171ee72, 66 },
    { 0x5a11ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe781ee77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b81ee7d, 100 },
    { 0xdc81ee7e, 33 },
    { 0x8021ee7e, 55 },
    { 0x15d1ee7d, 61 },
    { 0, 0 },
    { 0xe91ee82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f91ee86, 34 },
    { 0x10b1ee86, 52 },
    { 0xa051ee88, 98 },
    { 0xb8e1ee89, 78 },
    { 0x3331ee8a, 76 },
    { 0xf971ee89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3931ee92, 98 },
    { 0xdd81ee92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d1ee97, 33 },
    { 0x251ee97, 75 },
    { 0x3781ee99, 90 },
    { 0xc1ee9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf51ee9e, 34 },
    { 0, 0 },
    { 0x9941eea0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e1eea6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b31eead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd191eeb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1eeb5, 43 },
    { 0x9211eeb6, 85 },
    { 0x5ba1eeb7, 93 },
    { 0x9451eeb8, 77 },
    { 0x8441eeb7, 55 },
    { 0x3511eeb8, 33 },
    { 0xe351eebb, 98 },
    { 0xbdd1eeb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde1eebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8361eec2, 78 },
    { 0x2cf1eec3, 78 },
    { 0x8761eec3, 95 },
    { 0x7cd1eec3, 69 },
    { 0xac91eec6, 98 },
    { 0xe5b1eec6, 93 },
    { 0x5b11eec2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9331eecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a1eed1, 33 },
    { 0, 0 },
    { 0x8091eed3, 23 },
    { 0, 0 },
    { 0x1811eed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551eedd, 55 },
    { 0x8231eedd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c51eee4, 95 },
    { 0xec41eee4, 23 },
    { 0x5071eee6, 46 },
    { 0x5061eee6, 55 },
    { 0xf6d1eee8, 66 },
    { 0x40c1eee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6201eeee, 83 },
    { 0x9e1eeef, 33 },
    { 0x19b1eef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd1eef4, 90 },
    { 0x21d1eef4, 84 },
    { 0x6301eef6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b1eefa, 93 },
    { 0, 0 },
    { 0xda41eefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe761eeff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b1ef03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2691ef06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad1ef09, 23 },
    { 0x2501ef0a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf361ef0d, 76 },
    { 0x9a91ef0e, 93 },
    { 0x7c01ef0f, 81 },
    { 0xa8d1ef10, 55 },
    { 0x7ce1ef11, 33 },
    { 0xf41ef12, 61 },
    { 0x12c1ef13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561ef18, 87 },
    { 0, 0 },
    { 0xbc71ef1a, 64 },
    { 0xece1ef1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef1ef22, 95 },
    { 0x7a81ef23, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a1ef29, 90 },
    { 0x74b1ef2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a71ef2e, 75 },
    { 0xe9a1ef2f, 90 },
    { 0x7df1ef2e, 23 },
    { 0xee41ef31, 98 },
    { 0x1361ef32, 33 },
    { 0x7f51ef32, 66 },
    { 0x9231ef34, 43 },
    { 0x75e1ef34, 93 },
    { 0, 0 },
    { 0x5ee1ef37, 85 },
    { 0x2be1ef37, 91 },
    { 0x8301ef38, 33 },
    { 0, 0 },
    { 0x5511ef3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f1ef42, 95 },
    { 0x4491ef43, 90 },
    { 0x92b1ef43, 61 },
    { 0x29f1ef43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf1ef4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31ef4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6771ef52, 95 },
    { 0x47d1ef53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ef5d, 91 },
    { 0x5da1ef5d, 55 },
    { 0xd521ef5f, 87 },
    { 0, 0 },
    { 0x5961ef61, 33 },
    { 0, 0 },
    { 0x6771ef63, 49 },
    { 0x3cc1ef64, 81 },
    { 0xc2b1ef65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe981ef6a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9301ef6d, 66 },
    { 0x3f41ef6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c1ef71, 61 },
    { 0x7371ef72, 33 },
    { 0xaf31ef72, 67 },
    { 0xe4d1ef74, 33 },
    { 0x1751ef72, 61 },
    { 0x9271ef71, 90 },
    { 0x3ab1ef77, 33 },
    { 0x5d41ef78, 33 },
    { 0x3471ef79, 90 },
    { 0xa171ef78, 33 },
    { 0xe181ef78, 23 },
    { 0x9bc1ef7c, 61 },
    { 0xe211ef79, 65 },
    { 0x19c1ef7e, 23 },
    { 0xfa41ef7f, 91 },
    { 0x9b01ef80, 87 },
    { 0x8db1ef74, 61 },
    { 0x53d1ef82, 58 },
    { 0, 0 },
    { 0xe7e1ef84, 100 },
    { 0xda51ef85, 88 },
    { 0x1e61ef86, 23 },
    { 0xe1ef87, 90 },
    { 0xe041ef88, 23 },
    { 0x2c71ef88, 55 },
    { 0x3331ef88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1ef8d, 33 },
    { 0x171ef8e, 90 },
    { 0x8e61ef8f, 76 },
    { 0, 0 },
    { 0x9541ef91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c91ef94, 95 },
    { 0x81e1ef95, 33 },
    { 0x3fc1ef95, 61 },
    { 0x4c21ef97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a61ef9d, 61 },
    { 0xd171ef9e, 52 },
    { 0x8a21ef9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac1efa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71efa7, 46 },
    { 0x191efa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a31efad, 23 },
    { 0, 0 },
    { 0x8a21efaf, 90 },
    { 0x3dd1efaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a1efb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b1efb8, 61 },
    { 0, 0 },
    { 0x1ed1efba, 23 },
    { 0, 0 },
    { 0xbcd1efbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7731efc0, 33 },
    { 0x3d81efc1, 33 },
    { 0, 0 },
    { 0x13e1efc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b1efcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec21efd0, 33 },
    { 0x4fd1efd0, 87 },
    { 0xea51efd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111efd7, 46 },
    { 0, 0 },
    { 0xc61efd9, 55 },
    { 0x7cc1efda, 81 },
    { 0, 0 },
    { 0xd91efdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5241efe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb1efe6, 33 },
    { 0x4ea1efe7, 43 },
    { 0x7fe1efe6, 43 },
    { 0x90f1efe9, 98 },
    { 0xb201efea, 23 },
    { 0x3dc1efeb, 23 },
    { 0x3cc1efec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891efef, 61 },
    { 0xaf61eff0, 52 },
    { 0xb8c1eff1, 76 },
    { 0xb631eff1, 98 },
    { 0, 0 },
    { 0xd9d1eff4, 77 },
    { 0xb1a1eff5, 98 },
    { 0, 0 },
    { 0x67b1eff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc1effa, 98 },
    { 0x5c11effb, 75 },
    { 0x17e1effc, 61 },
    { 0xe191effb, 90 },
    { 0, 0 },
    { 0x6d61efff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac21f002, 55 },
    { 0x921f003, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5451f007, 85 },
    { 0x6761f008, 33 },
    { 0xb311f009, 88 },
    { 0x6021f00a, 23 },
    { 0xfba1f00b, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f21f00f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1f012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e1f017, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf1f01a, 90 },
    { 0x7241f01b, 99 },
    { 0x4341f01c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xed51f01f, 34 },
    { 0xc21f01f, 65 },
    { 0, 0 },
    { 0xaf41f022, 98 },
    { 0x5cc1f022, 33 },
    { 0x9f91f024, 78 },
    { 0x4f91f025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101f02f, 43 },
    { 0x9cd1f030, 93 },
    { 0, 0 },
    { 0x1071f032, 83 },
    { 0xd2d1f033, 59 },
    { 0x9131f034, 33 },
    { 0x8fa1f034, 23 },
    { 0x7b11f035, 40 },
    { 0x6111f036, 38 },
    { 0x2e31f038, 33 },
    { 0xea81f039, 76 },
    { 0x3eb1f03a, 34 },
    { 0x5d11f03a, 23 },
    { 0xdc91f03b, 33 },
    { 0, 0 },
    { 0xed31f03e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61f041, 85 },
    { 0x5e31f041, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5671f047, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1f04a, 66 },
    { 0, 0 },
    { 0x4b51f04c, 68 },
    { 0, 0 },
    { 0xe231f04e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6511f053, 33 },
    { 0, 0 },
    { 0xc591f055, 33 },
    { 0, 0 },
    { 0xe401f057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e1f05b, 99 },
    { 0x6ad1f05b, 84 },
    { 0x1811f05d, 46 },
    { 0, 0 },
    { 0x5431f05f, 61 },
    { 0xa51f060, 95 },
    { 0x7d41f05f, 76 },
    { 0x341f060, 46 },
    { 0, 0 },
    { 0x3631f064, 98 },
    { 0, 0 },
    { 0x8531f066, 95 },
    { 0, 0 },
    { 0x12a1f068, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51f06b, 95 },
    { 0xd551f06c, 100 },
    { 0x9c61f06c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb251f076, 23 },
    { 0x5941f077, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa1f07c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c1f07f, 61 },
    { 0x5831f080, 98 },
    { 0x13b1f081, 98 },
    { 0xf481f081, 49 },
    { 0xd121f083, 61 },
    { 0xbb1f082, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb441f089, 55 },
    { 0x1b41f08a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1531f08d, 95 },
    { 0, 0 },
    { 0xf151f08f, 76 },
    { 0xf211f090, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef1f094, 33 },
    { 0x7361f094, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901f098, 95 },
    { 0xf981f099, 33 },
    { 0x131f09a, 43 },
    { 0x691f099, 23 },
    { 0x6801f09c, 95 },
    { 0, 0 },
    { 0x4cc1f09e, 55 },
    { 0x1c21f09e, 90 },
    { 0x95b1f0a0, 55 },
    { 0xc9f1f0a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f91f0a6, 61 },
    { 0x5081f0a7, 55 },
    { 0x12c1f0a8, 98 },
    { 0, 0 },
    { 0x4821f0aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301f0b2, 96 },
    { 0, 0 },
    { 0xd681f0b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db1f0ba, 67 },
    { 0x2401f0ba, 38 },
    { 0x4a71f0bc, 41 },
    { 0xb131f0bd, 33 },
    { 0xff31f0ba, 52 },
    { 0, 0 },
    { 0xb171f0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa181f0c4, 93 },
    { 0, 0 },
    { 0x36d1f0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21f0cb, 61 },
    { 0x8e91f0cc, 85 },
    { 0xb4c1f0cc, 95 },
    { 0x7b71f0ce, 66 },
    { 0x80e1f0ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8801f0d9, 61 },
    { 0xf7c1f0da, 95 },
    { 0x4be1f0db, 61 },
    { 0xfd1f0da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb941f0e2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c91f0e8, 98 },
    { 0xfd11f0e9, 23 },
    { 0x88e1f0e9, 33 },
    { 0x8b1f0e8, 98 },
    { 0, 0 },
    { 0x11f0ed, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d1f0f0, 90 },
    { 0x1261f0f1, 68 },
    { 0x1271f0f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d91f0f5, 68 },
    { 0, 0 },
    { 0x8731f0f7, 90 },
    { 0, 0 },
    { 0xa91f0f9, 23 },
    { 0x3d71f0fa, 23 },
    { 0x5d61f0fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a1f0ff, 33 },
    { 0xa6b1f100, 46 },
    { 0, 0 },
    { 0xe801f102, 34 },
    { 0xf851f103, 95 },
    { 0xdcf1f104, 43 },
    { 0xabb1f104, 55 },
    { 0x91d1f106, 33 },
    { 0, 0 },
    { 0x2621f108, 61 },
    { 0, 0 },
    { 0x3c31f10a, 90 },
    { 0xf441f10b, 100 },
    { 0x4d11f10c, 34 },
    { 0, 0 },
    { 0x4f81f10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511f113, 67 },
    { 0xad81f114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa111f119, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a1f11c, 61 },
    { 0, 0 },
    { 0xca51f11e, 23 },
    { 0xaee1f11e, 95 },
    { 0xfeb1f11f, 43 },
    { 0x1af1f121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc731f129, 61 },
    { 0xbbc1f129, 61 },
    { 0, 0 },
    { 0x29a1f12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3311f12f, 23 },
    { 0x6891f130, 81 },
    { 0, 0 },
    { 0x4d51f132, 34 },
    { 0x2031f133, 34 },
    { 0xf821f133, 33 },
    { 0x31f1f135, 23 },
    { 0x2c81f133, 95 },
    { 0, 0 },
    { 0xcdf1f138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f1f13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae21f13e, 33 },
    { 0, 0 },
    { 0x3e41f140, 33 },
    { 0, 0 },
    { 0x4a21f142, 55 },
    { 0x7d41f143, 64 },
    { 0x3ad1f144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae61f148, 33 },
    { 0, 0 },
    { 0xe241f14a, 77 },
    { 0xe281f14b, 87 },
    { 0xd181f14a, 34 },
    { 0xb031f14d, 66 },
    { 0, 0 },
    { 0xa31f14f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee1f156, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1f159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f31f15d, 55 },
    { 0x7e01f15e, 90 },
    { 0, 0 },
    { 0xf5d1f160, 98 },
    { 0xeec1f161, 87 },
    { 0, 0 },
    { 0x4b41f163, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f167, 99 },
    { 0x1951f167, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1f16b, 43 },
    { 0xc8b1f16c, 90 },
    { 0x8721f16d, 56 },
    { 0x28a1f16e, 55 },
    { 0x8a71f16f, 23 },
    { 0x7c21f16b, 91 },
    { 0xba61f171, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31f174, 61 },
    { 0x1551f175, 55 },
    { 0, 0 },
    { 0x2261f177, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1f17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef41f17f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9721f184, 61 },
    { 0xa371f185, 90 },
    { 0, 0 },
    { 0x8761f187, 98 },
    { 0, 0 },
    { 0x18d1f189, 61 },
    { 0x4581f18a, 101 },
    { 0xc781f18b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7771f18f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa771f192, 55 },
    { 0, 0 },
    { 0x8191f194, 33 },
    { 0x8f41f194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481f198, 23 },
    { 0x69d1f199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01f1a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc031f1a4, 36 },
    { 0xec51f1a5, 90 },
    { 0x61b1f1a5, 87 },
    { 0x6d81f1a6, 98 },
    { 0xd231f1a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e91f1ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5421f1b5, 90 },
    { 0x3f81f1b6, 84 },
    { 0, 0 },
    { 0xaf61f1b8, 95 },
    { 0xdc61f1b8, 95 },
    { 0x1d1f1ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2631f1bd, 75 },
    { 0x26f1f1be, 95 },
    { 0xa5e1f1bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f1c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd1f1c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa161f1ca, 34 },
    { 0xccc1f1cb, 61 },
    { 0xc121f1cb, 66 },
    { 0x3fd1f1cc, 55 },
    { 0, 0 },
    { 0x4a11f1cf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f1d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe461f1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe311f1db, 96 },
    { 0xb01f1dc, 77 },
    { 0, 0 },
    { 0xdeb1f1de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4091f1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e1f1e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc1f1ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1f1f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b1f1f8, 33 },
    { 0, 0 },
    { 0x7b91f1fa, 46 },
    { 0, 0 },
    { 0xf251f1fc, 76 },
    { 0, 0 },
    { 0xee31f1fe, 17 },
    { 0, 0 },
    { 0xf0f1f200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1f206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1f209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c1f20d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501f210, 55 },
    { 0xab81f211, 91 },
    { 0x3351f212, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f1f215, 61 },
    { 0x12f1f216, 33 },
    { 0xe5f1f216, 90 },
    { 0x3e91f218, 90 },
    { 0xcc51f219, 98 },
    { 0, 0 },
    { 0x2f91f21b, 33 },
    { 0xa201f21b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe641f220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691f223, 93 },
    { 0x6b61f224, 43 },
    { 0x31a1f224, 98 },
    { 0x2fe1f226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b1f229, 90 },
    { 0x42f1f22a, 90 },
    { 0x6af1f22b, 33 },
    { 0x30c1f22b, 55 },
    { 0x29a1f22d, 84 },
    { 0xdbf1f22e, 96 },
    { 0xde01f22f, 33 },
    { 0x8e71f230, 90 },
    { 0xa701f22f, 90 },
    { 0xd9d1f231, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1f23f, 36 },
    { 0x7c11f23f, 61 },
    { 0xdeb1f240, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f1f244, 62 },
    { 0x761f245, 33 },
    { 0x6be1f246, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd1f24c, 43 },
    { 0, 0 },
    { 0xb901f24e, 98 },
    { 0, 0 },
    { 0x4c1f250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f257, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e1f25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5231f25e, 34 },
    { 0x81e1f25e, 90 },
    { 0x5751f260, 46 },
    { 0x8741f25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde11f264, 23 },
    { 0x6711f265, 95 },
    { 0x8581f266, 90 },
    { 0xab51f265, 98 },
    { 0x6a1f267, 33 },
    { 0, 0 },
    { 0x8e41f26a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d1f26d, 61 },
    { 0x23a1f26d, 33 },
    { 0xc911f26e, 23 },
    { 0, 0 },
    { 0xa141f271, 61 },
    { 0x3251f272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb621f275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c1f27b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7561f27f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871f283, 61 },
    { 0x7b91f283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291f288, 93 },
    { 0x7dd1f289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671f28e, 33 },
    { 0, 0 },
    { 0xf951f290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5611f295, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f1f29b, 90 },
    { 0xf51f29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf771f2a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd21f2a7, 95 },
    { 0x98b1f2a7, 85 },
    { 0x4cd1f2a8, 36 },
    { 0x5de1f2a7, 67 },
    { 0x45a1f2ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf61f2b1, 76 },
    { 0x51b1f2b2, 96 },
    { 0xd5f1f2b3, 91 },
    { 0x5fd1f2b3, 95 },
    { 0x3191f2b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d51f2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621f2be, 76 },
    { 0x93a1f2bf, 90 },
    { 0x1841f2c0, 61 },
    { 0x1861f2c1, 76 },
    { 0xc601f2c2, 90 },
    { 0, 0 },
    { 0xa5d1f2c4, 61 },
    { 0, 0 },
    { 0x1da1f2c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f1f2cf, 90 },
    { 0x63f1f2d0, 43 },
    { 0, 0 },
    { 0x2ce1f2d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5791f2d9, 90 },
    { 0, 0 },
    { 0x9321f2db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc1f2e4, 88 },
    { 0x8901f2e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf461f2ed, 91 },
    { 0x7061f2ee, 17 },
    { 0, 0 },
    { 0x4401f2f0, 52 },
    { 0, 0 },
    { 0xf901f2f2, 100 },
    { 0, 0 },
    { 0xed41f2f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671f2f8, 33 },
    { 0, 0 },
    { 0x9b41f2fa, 55 },
    { 0xd7f1f2fb, 91 },
    { 0xf851f2fc, 95 },
    { 0x6511f2fd, 34 },
    { 0x61a1f2fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821f302, 33 },
    { 0x4fd1f302, 55 },
    { 0x8021f303, 46 },
    { 0, 0 },
    { 0x35e1f306, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91f309, 76 },
    { 0x5931f309, 76 },
    { 0x7f71f30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551f310, 61 },
    { 0, 0 },
    { 0xba31f312, 33 },
    { 0x48e1f313, 93 },
    { 0xfdf1f314, 33 },
    { 0x9801f314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2761f319, 90 },
    { 0x6481f319, 91 },
    { 0xe81f31b, 98 },
    { 0xba01f31c, 33 },
    { 0x4f51f31d, 90 },
    { 0x43c1f31c, 33 },
    { 0xa1f1f31f, 77 },
    { 0x6ab1f320, 23 },
    { 0x5d71f319, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b1f326, 93 },
    { 0, 0 },
    { 0x6351f328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31f32b, 33 },
    { 0x57e1f32c, 75 },
    { 0xafe1f32d, 52 },
    { 0x9c61f32c, 33 },
    { 0x931f32c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe51f332, 43 },
    { 0xac61f333, 100 },
    { 0x8f61f334, 33 },
    { 0, 0 },
    { 0xf861f336, 46 },
    { 0x1ee1f336, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7701f33c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11f349, 93 },
    { 0x3da1f34a, 34 },
    { 0xcba1f34b, 95 },
    { 0, 0 },
    { 0x9961f34d, 33 },
    { 0x29b1f34e, 76 },
    { 0xd3f1f34f, 87 },
    { 0xda01f34d, 98 },
    { 0, 0 },
    { 0x1d61f352, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b1f35f, 52 },
    { 0x6d31f360, 93 },
    { 0xa641f361, 33 },
    { 0xbc81f362, 90 },
    { 0x8211f363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee31f367, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61f36a, 33 },
    { 0xb161f36b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1f370, 93 },
    { 0xb91f371, 46 },
    { 0xf271f372, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa561f375, 95 },
    { 0xb3b1f376, 98 },
    { 0, 0 },
    { 0x32a1f378, 98 },
    { 0x3311f378, 55 },
    { 0x8eb1f37a, 95 },
    { 0xa31f37b, 55 },
    { 0, 0 },
    { 0x62d1f37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab21f380, 34 },
    { 0xa1f381, 98 },
    { 0, 0 },
    { 0x3031f383, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7261f387, 91 },
    { 0x261f388, 23 },
    { 0xea21f387, 98 },
    { 0x31b1f38a, 33 },
    { 0, 0 },
    { 0x8831f38c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5731f390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1141f395, 95 },
    { 0x2251f395, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31f39d, 98 },
    { 0x7f91f39d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5721f3a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b1f3a7, 61 },
    { 0x321f3a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f31f3ab, 90 },
    { 0x5e01f3ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1f3b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf021f3b3, 84 },
    { 0x35f1f3b3, 98 },
    { 0, 0 },
    { 0x86b1f3b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501f3ba, 64 },
    { 0x52c1f3bb, 61 },
    { 0x2231f3ba, 75 },
    { 0, 0 },
    { 0xdab1f3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71f3c1, 99 },
    { 0xde61f3c1, 23 },
    { 0xd621f3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b1f3c9, 98 },
    { 0x4cd1f3c9, 91 },
    { 0x3c71f3c9, 98 },
    { 0x34d1f3ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6391f3d5, 90 },
    { 0xff71f3d6, 90 },
    { 0x7561f3d7, 93 },
    { 0, 0 },
    { 0xa541f3d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9251f3e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c91f3e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e11f3ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa921f3f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1f3f3, 87 },
    { 0x6f31f3f3, 101 },
    { 0x5a61f3f5, 61 },
    { 0x1ed1f3f5, 91 },
    { 0xcb21f3f7, 55 },
    { 0xee81f3f7, 34 },
    { 0x3e51f3f5, 46 },
    { 0x5211f3f7, 61 },
    { 0x9fc1f3f6, 33 },
    { 0x9bd1f3fb, 95 },
    { 0xf581f3fd, 90 },
    { 0, 0 },
    { 0x5a81f3ff, 93 },
    { 0xacd1f400, 33 },
    { 0x6ad1f401, 61 },
    { 0x3141f401, 55 },
    { 0x8e91f401, 98 },
    { 0x3981f404, 33 },
    { 0, 0 },
    { 0xb871f406, 90 },
    { 0x1451f407, 76 },
    { 0xa681f408, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce1f40b, 61 },
    { 0x1251f40b, 23 },
    { 0xcd71f40d, 76 },
    { 0x7da1f40e, 23 },
    { 0xdf1f40f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd581f414, 90 },
    { 0, 0 },
    { 0x61a1f416, 33 },
    { 0x94b1f417, 90 },
    { 0, 0 },
    { 0xc61f419, 3 },
    { 0xd531f41a, 35 },
    { 0xe051f41b, 61 },
    { 0, 0 },
    { 0x3ca1f41d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff91f424, 76 },
    { 0x9d61f425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec1f428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd421f42b, 33 },
    { 0x41a1f42c, 33 },
    { 0, 0 },
    { 0x11c1f42e, 61 },
    { 0x3111f42f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1f434, 61 },
    { 0xab31f434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5481f438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01f43c, 61 },
    { 0xde91f43d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd1f441, 55 },
    { 0x3f91f442, 61 },
    { 0x7451f442, 55 },
    { 0x3841f444, 38 },
    { 0x6651f445, 66 },
    { 0xdf41f444, 95 },
    { 0x53c1f443, 33 },
    { 0xd841f448, 46 },
    { 0, 0 },
    { 0xfb81f44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1f453, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c1f45c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411f45f, 33 },
    { 0x5d31f460, 90 },
    { 0xf8e1f45f, 66 },
    { 0x8dd1f462, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e1f466, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b1f46b, 95 },
    { 0xa071f46c, 33 },
    { 0, 0 },
    { 0x83d1f46e, 90 },
    { 0xdb21f46e, 55 },
    { 0x7251f46e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab91f474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71f477, 81 },
    { 0, 0 },
    { 0xa441f479, 98 },
    { 0x8441f47a, 34 },
    { 0x3b41f47b, 90 },
    { 0x4da1f47a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5781f47f, 17 },
    { 0, 0 },
    { 0x7f21f481, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe691f484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f48d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1f491, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591f498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd081f4a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf81f4aa, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7241f4ae, 76 },
    { 0xeb31f4ae, 24 },
    { 0xf461f4b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d01f4b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6781f4b7, 61 },
    { 0xb7a1f4b7, 98 },
    { 0xd5e1f4b8, 90 },
    { 0x7811f4ba, 95 },
    { 0, 0 },
    { 0x5e51f4bc, 67 },
    { 0x1c1f4bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xca61f4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f4c3, 95 },
    { 0, 0 },
    { 0xae51f4c5, 81 },
    { 0xaa51f4c5, 34 },
    { 0, 0 },
    { 0x21d1f4c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5911f4cf, 61 },
    { 0x9a91f4d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901f4d4, 34 },
    { 0, 0 },
    { 0xeb51f4d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61f4dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc151f4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e1f4e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2011f4e7, 90 },
    { 0, 0 },
    { 0xb811f4e9, 98 },
    { 0x8881f4ea, 66 },
    { 0x5ac1f4eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61f4ee, 95 },
    { 0xbe1f4ef, 66 },
    { 0x5321f4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df1f4fc, 33 },
    { 0, 0 },
    { 0x40e1f4fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21f503, 90 },
    { 0x4031f504, 48 },
    { 0x2231f504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca1f508, 98 },
    { 0x2e11f509, 90 },
    { 0, 0 },
    { 0xedd1f50b, 23 },
    { 0x4421f50c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f1f511, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb341f514, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e51f518, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d1f51b, 23 },
    { 0x3671f51c, 98 },
    { 0, 0 },
    { 0xfb41f51e, 101 },
    { 0xeec1f51f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b1f524, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1f527, 34 },
    { 0x63b1f528, 90 },
    { 0xdb01f528, 90 },
    { 0x70f1f52a, 88 },
    { 0x99d1f528, 98 },
    { 0xd551f52c, 90 },
    { 0xf631f52d, 98 },
    { 0xce61f52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a11f535, 33 },
    { 0x30e1f536, 75 },
    { 0xbed1f535, 90 },
    { 0xc9d1f538, 98 },
    { 0xaad1f539, 90 },
    { 0xa1f1f536, 95 },
    { 0x4b1f53b, 33 },
    { 0x7741f539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51f544, 69 },
    { 0x1f21f545, 61 },
    { 0x2a21f545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc61f54a, 76 },
    { 0x94f1f54b, 99 },
    { 0, 0 },
    { 0x8491f54d, 34 },
    { 0x3d31f54d, 90 },
    { 0x7a11f54f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b1f552, 95 },
    { 0x4031f552, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd1f557, 23 },
    { 0, 0 },
    { 0x9a31f559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3541f55e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7271f567, 90 },
    { 0, 0 },
    { 0x89f1f569, 33 },
    { 0xc481f569, 90 },
    { 0x3ef1f56b, 90 },
    { 0xee91f56b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101f56f, 81 },
    { 0xb431f56f, 40 },
    { 0, 0 },
    { 0x7751f572, 84 },
    { 0x7f61f573, 33 },
    { 0xf4c1f573, 43 },
    { 0x75d1f573, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1f579, 75 },
    { 0x6931f57a, 95 },
    { 0x9ed1f579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a1f57f, 87 },
    { 0, 0 },
    { 0x3641f581, 55 },
    { 0x2bc1f582, 34 },
    { 0x36a1f581, 34 },
    { 0x6ab1f584, 61 },
    { 0xea31f585, 99 },
    { 0x5cb1f585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01f58b, 95 },
    { 0x8c91f58b, 52 },
    { 0, 0 },
    { 0xbf1f58e, 17 },
    { 0, 0 },
    { 0x4631f590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa611f594, 90 },
    { 0xb8f1f595, 23 },
    { 0xb01f595, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db1f599, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae11f5a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f71f5ae, 98 },
    { 0, 0 },
    { 0xee01f5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3571f5b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa711f5b7, 61 },
    { 0xfee1f5b8, 90 },
    { 0xc321f5b9, 55 },
    { 0x9351f5ba, 55 },
    { 0xc1f1f5bb, 33 },
    { 0x3e51f5bc, 90 },
    { 0x84f1f5bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41f5c1, 91 },
    { 0x7651f5c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1f5cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971f5d0, 46 },
    { 0xbe1f5d1, 98 },
    { 0, 0 },
    { 0xfe01f5d3, 76 },
    { 0x8dd1f5d4, 76 },
    { 0x98d1f5d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c1f5d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1121f5dc, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301f5e3, 90 },
    { 0, 0 },
    { 0x4291f5e5, 95 },
    { 0x3aa1f5e6, 33 },
    { 0xa9f1f5e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb1f5ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d1f5f0, 55 },
    { 0x6031f5f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661f5f4, 61 },
    { 0xd831f5f5, 43 },
    { 0x2931f5f4, 95 },
    { 0x90f1f5f5, 33 },
    { 0x8a61f5f4, 61 },
    { 0x9cb1f5f8, 23 },
    { 0x63a1f5fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1f5fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2761f603, 90 },
    { 0x9641f604, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831f608, 76 },
    { 0, 0 },
    { 0x8921f60a, 87 },
    { 0, 0 },
    { 0x9221f60c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611f612, 90 },
    { 0x5b01f613, 88 },
    { 0xcf01f614, 23 },
    { 0x6c61f613, 34 },
    { 0xd1c1f616, 95 },
    { 0x5f51f617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e81f61c, 33 },
    { 0xf431f61c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b1f620, 95 },
    { 0, 0 },
    { 0x5761f622, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd1f625, 33 },
    { 0x6901f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa021f62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141f635, 98 },
    { 0, 0 },
    { 0x4801f637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d81f63e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c61f647, 69 },
    { 0, 0 },
    { 0xd11f649, 23 },
    { 0x571f64a, 34 },
    { 0x7111f64b, 66 },
    { 0xd9a1f64c, 33 },
    { 0xe7b1f64d, 76 },
    { 0x9ad1f64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b41f655, 87 },
    { 0x9541f655, 98 },
    { 0x9d91f657, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21f65b, 52 },
    { 0x2821f65b, 23 },
    { 0x55d1f65c, 75 },
    { 0, 0 },
    { 0x2cd1f65f, 100 },
    { 0xca11f660, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb41f663, 23 },
    { 0, 0 },
    { 0xe331f665, 76 },
    { 0xa991f665, 23 },
    { 0x5601f666, 34 },
    { 0xb2a1f668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3991f66e, 33 },
    { 0x4881f66f, 61 },
    { 0xc5b1f670, 84 },
    { 0x8001f671, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de1f679, 34 },
    { 0, 0 },
    { 0xd211f67b, 33 },
    { 0x7c41f67c, 88 },
    { 0xfd31f67d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551f682, 61 },
    { 0, 0 },
    { 0xb011f684, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa851f687, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9701f68a, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b41f68f, 66 },
    { 0x1911f68f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a1f694, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4091f697, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe1f69e, 91 },
    { 0, 0 },
    { 0x9bc1f6a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31f6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2501f6ad, 83 },
    { 0, 0 },
    { 0xc0f1f6af, 76 },
    { 0x8a91f6b0, 66 },
    { 0x2d81f6af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7231f6b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb581f6ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1f6c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1f6c8, 95 },
    { 0xdd11f6c9, 33 },
    { 0, 0 },
    { 0xa311f6cb, 88 },
    { 0, 0 },
    { 0x2241f6cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa1f6d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a81f6db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f21f6e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a1f6e8, 46 },
    { 0x12c1f6e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81f6ed, 55 },
    { 0x13b1f6ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41f6f1, 78 },
    { 0, 0 },
    { 0xa941f6f3, 95 },
    { 0x8ab1f6f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d1f6fa, 98 },
    { 0x2211f6fb, 78 },
    { 0xa8d1f6fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc1f705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381f708, 43 },
    { 0x9871f708, 36 },
    { 0x3dd1f708, 87 },
    { 0x351f70b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b1f710, 17 },
    { 0x70a1f711, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1f714, 76 },
    { 0x7db1f715, 95 },
    { 0x4411f714, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd091f71a, 33 },
    { 0x1b71f71b, 98 },
    { 0, 0 },
    { 0x1f71f71d, 98 },
    { 0xd9c1f71d, 90 },
    { 0x2d71f71f, 95 },
    { 0x47a1f720, 90 },
    { 0xfb41f71d, 8 },
    { 0, 0 },
    { 0xf4e1f723, 90 },
    { 0xac1f724, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961f729, 23 },
    { 0, 0 },
    { 0xeae1f72b, 75 },
    { 0x14d1f72c, 96 },
    { 0, 0 },
    { 0x33a1f72e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d1f731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa1f734, 34 },
    { 0, 0 },
    { 0xa0b1f736, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71f739, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3491f73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f1f741, 100 },
    { 0x2cb1f742, 23 },
    { 0x3631f743, 77 },
    { 0x62b1f744, 61 },
    { 0xf391f745, 23 },
    { 0xcd41f746, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d1f749, 98 },
    { 0xe8b1f74a, 90 },
    { 0x3ab1f74b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb81f753, 90 },
    { 0x491f754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7821f757, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba91f75a, 76 },
    { 0x71f1f75b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c1f75f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3421f765, 98 },
    { 0x651f765, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d71f769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f51f76e, 68 },
    { 0, 0 },
    { 0xa911f770, 34 },
    { 0xf0d1f771, 90 },
    { 0x7891f772, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2051f778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241f77f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2671f787, 75 },
    { 0, 0 },
    { 0xe781f789, 90 },
    { 0, 0 },
    { 0x6271f78b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d41f793, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d1f797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1f79b, 76 },
    { 0x7241f79c, 90 },
    { 0x111f79d, 43 },
    { 0, 0 },
    { 0x2af1f79f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8481f7a3, 55 },
    { 0xc671f7a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2011f7a9, 90 },
    { 0x5071f7a9, 61 },
    { 0xfbf1f7a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf581f7b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241f7b6, 90 },
    { 0, 0 },
    { 0xee41f7b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b1f7bc, 77 },
    { 0xd3e1f7bc, 78 },
    { 0xfef1f7be, 46 },
    { 0xde1f7bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e11f7c3, 90 },
    { 0xc651f7c4, 90 },
    { 0xf531f7c5, 33 },
    { 0xd2a1f7c6, 98 },
    { 0x51f7c7, 23 },
    { 0x59b1f7c8, 69 },
    { 0xc081f7c9, 76 },
    { 0xad1f7ca, 90 },
    { 0xdbd1f7ca, 90 },
    { 0x9651f7cb, 33 },
    { 0x6591f7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6041f7d2, 90 },
    { 0x4e71f7d3, 46 },
    { 0x2861f7d4, 95 },
    { 0x5841f7d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd771f7d8, 90 },
    { 0x26d1f7d9, 90 },
    { 0x11a1f7da, 34 },
    { 0xc821f7d8, 90 },
    { 0xdc81f7dc, 99 },
    { 0, 0 },
    { 0xfae1f7de, 61 },
    { 0x14d1f7df, 90 },
    { 0x7d1f7df, 90 },
    { 0xd661f7e1, 23 },
    { 0, 0 },
    { 0x90c1f7e3, 52 },
    { 0xf581f7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01f7ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xac21f7ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21f7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b1f7f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f91f7fb, 90 },
    { 0x521f7fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871f801, 99 },
    { 0x7d61f802, 76 },
    { 0xeed1f803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd1f80f, 34 },
    { 0xff91f810, 61 },
    { 0x7f61f811, 98 },
    { 0x1381f811, 90 },
    { 0xb611f813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3151f81b, 77 },
    { 0xf131f81c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8311f821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b1f826, 98 },
    { 0, 0 },
    { 0xf81f828, 76 },
    { 0x2a21f829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef1f82c, 66 },
    { 0x78c1f82d, 33 },
    { 0xdbe1f82e, 90 },
    { 0x2cc1f82d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd681f834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561f838, 95 },
    { 0xa031f838, 55 },
    { 0xf921f839, 33 },
    { 0x5ca1f839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1f83f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd591f844, 55 },
    { 0xdf91f845, 33 },
    { 0, 0 },
    { 0x2841f847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e21f84d, 56 },
    { 0x2cd1f84d, 33 },
    { 0, 0 },
    { 0x16e1f850, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d01f856, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f85c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f21f865, 33 },
    { 0xc8d1f865, 33 },
    { 0, 0 },
    { 0xd881f868, 95 },
    { 0x30d1f869, 98 },
    { 0xe4f1f86a, 61 },
    { 0xa391f86b, 23 },
    { 0x7c91f86b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb1f870, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb251f873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c1f87b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1f87f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961f884, 68 },
    { 0xd21f884, 76 },
    { 0x4571f886, 33 },
    { 0x8bd1f884, 23 },
    { 0, 0 },
    { 0x50e1f889, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1f88e, 34 },
    { 0xdaa1f88f, 23 },
    { 0x7d1f890, 64 },
    { 0x981f890, 23 },
    { 0x5741f88e, 98 },
    { 0xecf1f892, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d1f897, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb111f89c, 61 },
    { 0, 0 },
    { 0x1c1f89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e71f8a1, 61 },
    { 0xce51f8a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b1f8a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d1f8ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e1f8b4, 23 },
    { 0x1f71f8b5, 76 },
    { 0, 0 },
    { 0x2b21f8b7, 90 },
    { 0, 0 },
    { 0x49e1f8b9, 33 },
    { 0xa8a1f8ba, 61 },
    { 0xc991f8bb, 90 },
    { 0x1731f8ba, 61 },
    { 0x1821f8bd, 99 },
    { 0x1451f8be, 84 },
    { 0x1a61f8bf, 87 },
    { 0x3a41f8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd601f8c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e01f8c9, 95 },
    { 0xfae1f8ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791f8ce, 61 },
    { 0, 0 },
    { 0xd0c1f8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f8d3, 33 },
    { 0x6b01f8d4, 95 },
    { 0, 0 },
    { 0x2e61f8d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11f8da, 66 },
    { 0, 0 },
    { 0xd631f8dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f1f8e9, 43 },
    { 0, 0 },
    { 0xd181f8eb, 93 },
    { 0x5551f8eb, 23 },
    { 0x13d1f8ed, 46 },
    { 0xef41f8eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a21f8f1, 61 },
    { 0x8491f8f2, 95 },
    { 0xdc41f8f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa251f8f8, 99 },
    { 0x18b1f8f8, 95 },
    { 0, 0 },
    { 0x9381f8fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d41f8fe, 84 },
    { 0x7f11f8fe, 43 },
    { 0xbe1f900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51f907, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41f90c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2511f912, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1f919, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51f91c, 43 },
    { 0x8111f91d, 23 },
    { 0xcca1f91e, 65 },
    { 0xf991f91e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f1f923, 95 },
    { 0, 0 },
    { 0xde21f925, 90 },
    { 0xac1f926, 61 },
    { 0x4e51f926, 98 },
    { 0x1d61f928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21f92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001f933, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb931f936, 85 },
    { 0xb611f937, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa41f93b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51f93e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581f943, 61 },
    { 0x39a1f943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec1f94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5151f95b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1f95e, 94 },
    { 0xcf51f95f, 76 },
    { 0xcda1f95f, 33 },
    { 0xde31f961, 61 },
    { 0, 0 },
    { 0x38f1f963, 85 },
    { 0x2ab1f963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551f967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f96d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd211f972, 43 },
    { 0xd981f973, 95 },
    { 0x7421f972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a91f977, 33 },
    { 0xb891f977, 98 },
    { 0, 0 },
    { 0x401f97a, 46 },
    { 0xe391f97b, 95 },
    { 0, 0 },
    { 0x2771f97d, 76 },
    { 0, 0 },
    { 0x9a71f97f, 46 },
    { 0xe3c1f97f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5711f987, 52 },
    { 0x85e1f988, 61 },
    { 0x3f1f987, 93 },
    { 0, 0 },
    { 0x6db1f98b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b1f990, 67 },
    { 0x6661f991, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce91f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe441f999, 100 },
    { 0xb701f999, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e91f99d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511f9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8031f9aa, 40 },
    { 0, 0 },
    { 0x1b41f9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c1f9bb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3061f9c6, 55 },
    { 0x19f1f9c7, 98 },
    { 0, 0 },
    { 0x9631f9c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f9ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a51f9da, 95 },
    { 0, 0 },
    { 0x5e11f9dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde61f9df, 51 },
    { 0xcbf1f9df, 55 },
    { 0xc3a1f9e1, 90 },
    { 0x8f01f9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171f9e5, 34 },
    { 0xba01f9e6, 96 },
    { 0x6481f9e7, 23 },
    { 0x9071f9e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb1f9ec, 91 },
    { 0x9561f9ed, 43 },
    { 0, 0 },
    { 0x5a41f9ef, 51 },
    { 0, 0 },
    { 0x3f81f9f1, 87 },
    { 0x2cb1f9f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6431f9fa, 99 },
    { 0, 0 },
    { 0xdc11f9fc, 61 },
    { 0x64e1f9fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1181fa00, 61 },
    { 0, 0 },
    { 0x8c71fa02, 90 },
    { 0x8f21fa03, 87 },
    { 0, 0 },
    { 0xa0d1fa05, 98 },
    { 0, 0 },
    { 0xcf51fa07, 90 },
    { 0, 0 },
    { 0x7551fa09, 43 },
    { 0, 0 },
    { 0x6ea1fa0b, 87 },
    { 0x8331fa0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be1fa0f, 33 },
    { 0xced1fa10, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5271fa15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781fa1b, 100 },
    { 0x80b1fa1c, 33 },
    { 0xefe1fa1d, 98 },
    { 0, 0 },
    { 0xad61fa1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e21fa23, 61 },
    { 0x87d1fa24, 98 },
    { 0xb5c1fa25, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd1fa2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce11fa36, 23 },
    { 0x20d1fa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161fa3c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f1fa41, 52 },
    { 0, 0 },
    { 0x4cf1fa43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1901fa47, 87 },
    { 0x3131fa48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5041fa4b, 95 },
    { 0x77c1fa4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf31fa56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1fa59, 98 },
    { 0x4641fa59, 33 },
    { 0xa2a1fa5b, 34 },
    { 0x2e61fa5b, 34 },
    { 0xb791fa5c, 95 },
    { 0, 0 },
    { 0x27a1fa5f, 23 },
    { 0xd971fa60, 77 },
    { 0xb551fa60, 98 },
    { 0xfd81fa5f, 23 },
    { 0, 0 },
    { 0xd3d1fa64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141fa69, 95 },
    { 0, 0 },
    { 0x111fa6b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1fa73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca1fa76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671fa7d, 66 },
    { 0x9b11fa7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d41fa86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c91fa89, 61 },
    { 0x3161fa8a, 78 },
    { 0xf621fa8a, 34 },
    { 0x84c1fa8b, 34 },
    { 0x74d1fa8a, 33 },
    { 0xdc91fa8c, 90 },
    { 0x2861fa8f, 90 },
    { 0x8b91fa89, 99 },
    { 0xd731fa91, 90 },
    { 0xc91fa91, 66 },
    { 0xa161fa92, 23 },
    { 0, 0 },
    { 0x7b41fa95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81fa98, 91 },
    { 0x6841fa98, 23 },
    { 0x8dc1fa9a, 80 },
    { 0x1971fa98, 88 },
    { 0xace1fa9a, 66 },
    { 0x33b1fa9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211faa0, 61 },
    { 0xf541faa1, 52 },
    { 0xa761faa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e1faa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1faa9, 90 },
    { 0, 0 },
    { 0x121faab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb191fab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd131fab3, 87 },
    { 0x1101fab3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4921faba, 84 },
    { 0xc071faba, 23 },
    { 0x281fabb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d1fac4, 76 },
    { 0x9121fac5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761fac9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde11facd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e01fad1, 95 },
    { 0x6dc1fad1, 99 },
    { 0xa031fad1, 55 },
    { 0, 0 },
    { 0x5ae1fad5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91fad8, 40 },
    { 0x9c61fad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea41fadc, 23 },
    { 0, 0 },
    { 0xb1f1fade, 33 },
    { 0x7601fadf, 95 },
    { 0xd521fade, 98 },
    { 0x89e1fae0, 90 },
    { 0xb0c1fae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81fae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff1fae8, 98 },
    { 0x3b01fae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4911faee, 95 },
    { 0x8561faef, 76 },
    { 0x1d61faef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5311faf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c1fafc, 55 },
    { 0x3961fafc, 23 },
    { 0xb51fafc, 43 },
    { 0xc611fafe, 95 },
    { 0, 0 },
    { 0x8281fb01, 23 },
    { 0x3421fb02, 33 },
    { 0, 0 },
    { 0xaf01fb04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd91fb0c, 90 },
    { 0xad1fb0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c1fb14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb51fb19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1fb1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf311fb22, 40 },
    { 0x9ae1fb22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8251fb2a, 33 },
    { 0xdf91fb2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc01fb2f, 61 },
    { 0, 0 },
    { 0xd5d1fb31, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd731fb34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c1fb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b1fb3a, 43 },
    { 0, 0 },
    { 0x4a11fb3c, 61 },
    { 0, 0 },
    { 0xeb41fb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe1fb41, 99 },
    { 0, 0 },
    { 0xcc11fb43, 23 },
    { 0x69c1fb44, 90 },
    { 0, 0 },
    { 0x7ea1fb46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5521fb52, 76 },
    { 0x4c61fb53, 90 },
    { 0x8d11fb54, 61 },
    { 0x70b1fb53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc1fb5c, 42 },
    { 0xe6b1fb5c, 95 },
    { 0x1f1fb5e, 76 },
    { 0x6e41fb5d, 23 },
    { 0xf851fb5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e1fb67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1fb6e, 61 },
    { 0x7201fb6e, 34 },
    { 0, 0 },
    { 0x7371fb71, 95 },
    { 0xfa91fb72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa261fb76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f1fb79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa601fb7e, 52 },
    { 0xef21fb7f, 99 },
    { 0x20f1fb80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291fb86, 90 },
    { 0, 0 },
    { 0x91b1fb88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec1fb90, 33 },
    { 0x7d51fb90, 77 },
    { 0xa041fb92, 34 },
    { 0x4031fb93, 90 },
    { 0, 0 },
    { 0xe091fb95, 84 },
    { 0, 0 },
    { 0xd161fb97, 78 },
    { 0, 0 },
    { 0xf691fb99, 52 },
    { 0x48a1fb99, 34 },
    { 0x7de1fb99, 95 },
    { 0, 0 },
    { 0x7941fb9d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc401fba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9311fba4, 76 },
    { 0, 0 },
    { 0x6761fba6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd061fba9, 90 },
    { 0x5ad1fbaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8651fbae, 85 },
    { 0, 0 },
    { 0xbb31fbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d41fbb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91fbb7, 91 },
    { 0, 0 },
    { 0x4be1fbb9, 34 },
    { 0xcac1fbb9, 61 },
    { 0, 0 },
    { 0x42b1fbbc, 46 },
    { 0xe3c1fbbc, 68 },
    { 0, 0 },
    { 0xe2e1fbbf, 90 },
    { 0, 0 },
    { 0xcf21fbc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1fbc4, 33 },
    { 0, 0 },
    { 0x7271fbc6, 90 },
    { 0x9d1fbc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf231fbcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821fbd0, 97 },
    { 0xb1c1fbd1, 99 },
    { 0xea41fbd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb91fbd5, 90 },
    { 0xc811fbd5, 68 },
    { 0x58a1fbd7, 66 },
    { 0xc9c1fbd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021fbdd, 33 },
    { 0x151fbde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911fbe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea1fbe9, 95 },
    { 0x3321fbea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f1fbed, 33 },
    { 0xdc41fbee, 98 },
    { 0x1281fbef, 95 },
    { 0, 0 },
    { 0x7c31fbf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7851fbf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5321fc01, 76 },
    { 0xa2c1fc02, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c81fc0b, 87 },
    { 0xc4b1fc0c, 93 },
    { 0, 0 },
    { 0xffa1fc0e, 95 },
    { 0xcd51fc0f, 52 },
    { 0x6f21fc10, 23 },
    { 0xc511fc11, 98 },
    { 0xa2e1fc11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c1fc1a, 46 },
    { 0x57b1fc1b, 84 },
    { 0, 0 },
    { 0x5001fc1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a1fc20, 61 },
    { 0, 0 },
    { 0x63f1fc22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f91fc25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331fc29, 90 },
    { 0xe3d1fc2a, 66 },
    { 0x1711fc2b, 90 },
    { 0x7ec1fc2c, 52 },
    { 0, 0 },
    { 0x8681fc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e1fc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd1fc37, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa61fc3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf061fc3f, 67 },
    { 0x4bb1fc40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1fc47, 46 },
    { 0x50d1fc48, 17 },
    { 0x40b1fc49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3861fc4d, 23 },
    { 0, 0 },
    { 0x4731fc4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f51fc52, 76 },
    { 0x7e71fc52, 61 },
    { 0, 0 },
    { 0xc441fc55, 90 },
    { 0x211fc56, 51 },
    { 0xfa11fc57, 55 },
    { 0xe4b1fc58, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381fc5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a91fc60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1fc63, 33 },
    { 0x7b11fc63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9241fc67, 76 },
    { 0x5201fc68, 23 },
    { 0x96f1fc69, 90 },
    { 0xd291fc6a, 61 },
    { 0xf841fc67, 99 },
    { 0xde1fc6c, 76 },
    { 0xf001fc6d, 64 },
    { 0, 0 },
    { 0x99d1fc6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111fc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d1fc76, 55 },
    { 0x7eb1fc76, 90 },
    { 0xd941fc78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fc7e, 98 },
    { 0x1f01fc7e, 40 },
    { 0x201fc80, 92 },
    { 0x2e11fc81, 87 },
    { 0, 0 },
    { 0x1b51fc83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591fc87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc1fc8a, 33 },
    { 0, 0 },
    { 0x6661fc8c, 90 },
    { 0x6891fc8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b1fc93, 95 },
    { 0xa771fc94, 81 },
    { 0xe351fc95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9401fc98, 81 },
    { 0x9751fc99, 46 },
    { 0x7da1fc98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb361fc9e, 91 },
    { 0x8f11fc9e, 97 },
    { 0x57b1fca0, 87 },
    { 0x6e31fc9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa031fca4, 43 },
    { 0, 0 },
    { 0x43f1fca6, 95 },
    { 0x5221fca6, 46 },
    { 0x8111fca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f1fcab, 96 },
    { 0x18f1fcab, 43 },
    { 0, 0 },
    { 0x5351fcae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a1fcb1, 98 },
    { 0x401fcb1, 98 },
    { 0x85f1fcb3, 99 },
    { 0xd281fcb4, 89 },
    { 0x88e1fcb5, 33 },
    { 0x4121fcb4, 34 },
    { 0x5e81fcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81fcba, 99 },
    { 0, 0 },
    { 0x4ba1fcbc, 97 },
    { 0xd6c1fcbc, 91 },
    { 0xdc51fcbe, 61 },
    { 0x811fcbe, 95 },
    { 0x2f51fcbe, 76 },
    { 0x5051fcbe, 61 },
    { 0x8d61fcbd, 33 },
    { 0xeb11fcc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb41fcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1fcc9, 23 },
    { 0xabc1fcc9, 95 },
    { 0xccd1fccb, 90 },
    { 0x64a1fcca, 34 },
    { 0x6251fccc, 98 },
    { 0xe7a1fcca, 75 },
    { 0x9d81fcc9, 55 },
    { 0, 0 },
    { 0x3b71fcd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db1fcd4, 90 },
    { 0, 0 },
    { 0x5b01fcd6, 33 },
    { 0xa601fcd6, 87 },
    { 0xc841fcd8, 22 },
    { 0xa3d1fcd9, 99 },
    { 0xeb51fcd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb51fcdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf51fce0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd041fcea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1fced, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11fcf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5981fcf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461fcf8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fd01, 95 },
    { 0xeb81fd02, 95 },
    { 0, 0 },
    { 0x42c1fd04, 90 },
    { 0, 0 },
    { 0xd291fd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331fd0e, 55 },
    { 0x47a1fd0e, 23 },
    { 0xee11fd10, 68 },
    { 0x4db1fd0e, 55 },
    { 0xf7f1fd11, 61 },
    { 0, 0 },
    { 0xdea1fd14, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a1fd1d, 33 },
    { 0, 0 },
    { 0xe441fd1f, 33 },
    { 0xfb81fd20, 95 },
    { 0xe851fd21, 95 },
    { 0x8b1fd22, 33 },
    { 0x3b71fd22, 46 },
    { 0x7341fd23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241fd29, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf1fd2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8741fd36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb11fd3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681fd3f, 66 },
    { 0x2251fd40, 65 },
    { 0xd1e1fd41, 89 },
    { 0x8e41fd41, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21fd45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901fd48, 33 },
    { 0xa4b1fd49, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81fd4f, 95 },
    { 0x9921fd50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841fd53, 100 },
    { 0x8881fd53, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf1fd57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec21fd5b, 95 },
    { 0, 0 },
    { 0x7d51fd5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9601fd60, 43 },
    { 0x6451fd61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db1fd64, 90 },
    { 0, 0 },
    { 0xb291fd66, 33 },
    { 0, 0 },
    { 0x7531fd68, 91 },
    { 0x1781fd69, 76 },
    { 0x8351fd6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541fd71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111fd74, 46 },
    { 0x3be1fd75, 17 },
    { 0x5fb1fd76, 90 },
    { 0x1041fd74, 90 },
    { 0x94c1fd74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2961fd7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a1fd81, 33 },
    { 0, 0 },
    { 0x1331fd83, 66 },
    { 0xaec1fd84, 33 },
    { 0x17e1fd83, 87 },
    { 0xd5a1fd85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe81fd89, 55 },
    { 0, 0 },
    { 0xb4c1fd8b, 33 },
    { 0x3751fd8c, 98 },
    { 0x12a1fd8d, 55 },
    { 0x1bd1fd8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe571fd93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a01fd96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4341fd9e, 85 },
    { 0x8b51fd9f, 98 },
    { 0xab51fda0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81fda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8711fda6, 22 },
    { 0xef1fda6, 98 },
    { 0x9cb1fda8, 43 },
    { 0, 0 },
    { 0x1951fdaa, 33 },
    { 0x4d11fdab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa691fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed1fdb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba1fdba, 95 },
    { 0, 0 },
    { 0xe941fdbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1fdc1, 33 },
    { 0x67e1fdc2, 33 },
    { 0x79d1fdc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11fdc7, 61 },
    { 0xb3d1fdc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8431fdcd, 43 },
    { 0xa391fdce, 55 },
    { 0, 0 },
    { 0x8e41fdd0, 61 },
    { 0xe9a1fdd0, 95 },
    { 0xa501fdd2, 98 },
    { 0x7781fdd3, 84 },
    { 0xfcb1fdd4, 90 },
    { 0xda71fdd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd41fddc, 46 },
    { 0xddd1fddd, 69 },
    { 0xa821fdde, 33 },
    { 0, 0 },
    { 0x7891fde0, 90 },
    { 0, 0 },
    { 0x7001fde2, 90 },
    { 0xded1fde2, 33 },
    { 0x6ed1fde4, 85 },
    { 0x7b01fde5, 78 },
    { 0, 0 },
    { 0x6d01fde7, 46 },
    { 0xfc51fde8, 61 },
    { 0x9ce1fde9, 96 },
    { 0x88a1fdea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1fded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e1fdf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c51fdf3, 34 },
    { 0x4dc1fdf3, 76 },
    { 0, 0 },
    { 0x9621fdf6, 40 },
    { 0x5e21fdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c1fdfb, 33 },
    { 0x2241fdfc, 66 },
    { 0, 0 },
    { 0xb911fdfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c31fe01, 33 },
    { 0xe211fe01, 33 },
    { 0x4681fe03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1411fe07, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151fe0e, 84 },
    { 0, 0 },
    { 0xa921fe10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb961fe17, 93 },
    { 0x4761fe18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd1fe1f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be1fe22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1fe29, 40 },
    { 0, 0 },
    { 0xd991fe2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf1fe30, 69 },
    { 0xea81fe30, 98 },
    { 0xd131fe30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe971fe36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21fe3d, 61 },
    { 0xf221fe3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8481fe42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11fe46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401fe4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891fe4e, 95 },
    { 0xa881fe4e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9731fe54, 96 },
    { 0xbd31fe55, 90 },
    { 0xd391fe54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b11fe5e, 23 },
    { 0xde81fe5f, 52 },
    { 0x1bd1fe60, 61 },
    { 0x6b21fe61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761fe65, 33 },
    { 0x85d1fe65, 61 },
    { 0x6f41fe65, 33 },
    { 0x7401fe68, 34 },
    { 0, 0 },
    { 0xf801fe6a, 91 },
    { 0x5981fe6a, 43 },
    { 0x6841fe6c, 33 },
    { 0xab61fe6d, 55 },
    { 0xcfe1fe6e, 98 },
    { 0x1d61fe6e, 33 },
    { 0xcd1fe70, 34 },
    { 0, 0 },
    { 0xa961fe72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb31fe79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d51fe7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd1fe7f, 34 },
    { 0x6441fe80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf351fe85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1fe8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2511fe99, 90 },
    { 0xbd61fe9a, 95 },
    { 0xb311fe9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6511fea1, 46 },
    { 0x59d1fea2, 61 },
    { 0x7891fea2, 80 },
    { 0, 0 },
    { 0xd331fea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551fea8, 61 },
    { 0, 0 },
    { 0x76a1feaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871feb0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee1feb3, 46 },
    { 0x6e51feb4, 33 },
    { 0x40d1feb4, 34 },
    { 0xeaa1feb6, 98 },
    { 0x9fd1feb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2751febb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa1febf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c1fec2, 33 },
    { 0, 0 },
    { 0x9511fec4, 97 },
    { 0x7ca1fec5, 95 },
    { 0xcf91fec5, 91 },
    { 0x84e1fec6, 67 },
    { 0x2ec1fec4, 33 },
    { 0, 0 },
    { 0xca61feca, 43 },
    { 0x1e71fecb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e1fece, 90 },
    { 0xece1fece, 40 },
    { 0, 0 },
    { 0x53a1fed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1fed8, 98 },
    { 0, 0 },
    { 0xc41feda, 80 },
    { 0x4b71fedb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce1fee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4281fee5, 90 },
    { 0, 0 },
    { 0x9be1fee7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd671fef0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd91fef3, 61 },
    { 0x6111fef4, 52 },
    { 0xd431fef5, 90 },
    { 0, 0 },
    { 0xc4c1fef7, 34 },
    { 0, 0 },
    { 0x2031fef9, 90 },
    { 0xea01fefa, 84 },
    { 0x22c1fefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb01fefe, 77 },
    { 0x9b21feff, 61 },
    { 0, 0 },
    { 0xd141ff01, 56 },
    { 0x56e1ff01, 90 },
    { 0x7371ff03, 33 },
    { 0xf81ff04, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e1ff08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f11ff0b, 23 },
    { 0x7361ff0b, 34 },
    { 0x7031ff0d, 48 },
    { 0xfb31ff0e, 61 },
    { 0x1101ff0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b1ff12, 43 },
    { 0, 0 },
    { 0x33f1ff14, 87 },
    { 0xf521ff14, 43 },
    { 0xdc01ff15, 90 },
    { 0x5811ff14, 34 },
    { 0x1c41ff18, 90 },
    { 0x2141ff19, 61 },
    { 0, 0 },
    { 0x5f61ff1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091ff1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb1ff24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3791ff2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e1ff39, 19 },
    { 0, 0 },
    { 0xa561ff3b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d1ff41, 17 },
    { 0x9c61ff42, 39 },
    { 0xf6d1ff42, 95 },
    { 0xacf1ff42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc441ff48, 84 },
    { 0x21c1ff49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9721ff51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd301ff55, 90 },
    { 0, 0 },
    { 0x7961ff57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21ff5f, 61 },
    { 0xa9e1ff5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581ff64, 95 },
    { 0x83c1ff65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xef21ff68, 91 },
    { 0, 0 },
    { 0x2fc1ff6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81ff71, 34 },
    { 0xf261ff72, 98 },
    { 0x5e91ff71, 61 },
    { 0, 0 },
    { 0xf131ff75, 43 },
    { 0x26b1ff76, 23 },
    { 0x6041ff77, 98 },
    { 0, 0 },
    { 0x8ff1ff79, 88 },
    { 0, 0 },
    { 0x7631ff7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb001ff80, 98 },
    { 0, 0 },
    { 0x3ed1ff82, 98 },
    { 0xd391ff83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51ff87, 33 },
    { 0x3b01ff87, 76 },
    { 0, 0 },
    { 0x1fc1ff8a, 33 },
    { 0x9871ff8b, 76 },
    { 0x4b51ff8c, 53 },
    { 0xd791ff8d, 33 },
    { 0x8f81ff8e, 55 },
    { 0xe11ff8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5211ff9e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021ffa2, 87 },
    { 0x7141ffa3, 93 },
    { 0xb601ffa2, 33 },
    { 0x5d31ffa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd1ffaa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb1ffae, 88 },
    { 0, 0 },
    { 0x2841ffb0, 34 },
    { 0x8661ffb1, 46 },
    { 0x2611ffb1, 78 },
    { 0xdec1ffb1, 34 },
    { 0, 0 },
    { 0x9831ffb5, 23 },
    { 0x33c1ffb6, 67 },
    { 0, 0 },
    { 0x5351ffb8, 33 },
    { 0x55f1ffb9, 95 },
    { 0, 0 },
    { 0x6231ffbb, 43 },
    { 0xd101ffbc, 98 },
    { 0xba51ffbc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e1ffc0, 69 },
    { 0xfb41ffc1, 43 },
    { 0x9f61ffc1, 90 },
    { 0x9d31ffc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb91ffc7, 46 },
    { 0x2e31ffc8, 92 },
    { 0xbc11ffc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61ffcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551ffd3, 81 },
    { 0, 0 },
    { 0x941ffd5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1ffd9, 81 },
    { 0xa4e1ffda, 46 },
    { 0x89f1ffdb, 95 },
    { 0xffe1ffd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9371ffe2, 100 },
    { 0x5281ffe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc521ffeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ffee, 61 },
    { 0x7aa1ffef, 90 },
    { 0xfd1ffee, 33 },
    { 0x7b41fff1, 46 },
    { 0x471fff2, 48 },
    { 0xf6d1fff2, 95 },
    { 0x2491fff4, 48 },
    { 0, 0 },
    { 0x31a1fff6, 91 },
    { 0x8a01fff7, 61 },
    { 0xf6d1fff8, 43 },
    { 0x8cd1fff9, 77 },
    { 0x4b41fffa, 67 },
    { 0, 0 },
    { 0x39f1fffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3720005, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6582000a, 95 },
    { 0x28b2000b, 95 },
    { 0xd1d2000c, 94 },
    { 0x9c2000d, 93 },
    { 0xe0e2000e, 95 },
    { 0xbd82000f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e20013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58620018, 55 },
    { 0, 0 },
    { 0x54b2001a, 34 },
    { 0xdea2001b, 33 },
    { 0x5762001a, 23 },
    { 0, 0 },
    { 0x4ee2001e, 96 },
    { 0, 0 },
    { 0xc4d20020, 91 },
    { 0xb3520021, 78 },
    { 0xa5520022, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe420027, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262002a, 94 },
    { 0, 0 },
    { 0x6be2002c, 90 },
    { 0xfcb2002d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c220030, 52 },
    { 0xdec20030, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78220036, 90 },
    { 0, 0 },
    { 0x2f320038, 61 },
    { 0, 0 },
    { 0x2ea2003a, 46 },
    { 0, 0 },
    { 0x4a52003c, 34 },
    { 0, 0 },
    { 0x9062003e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7720043, 33 },
    { 0xaaa20043, 69 },
    { 0, 0 },
    { 0x44b20046, 61 },
    { 0, 0 },
    { 0xd8b20048, 76 },
    { 0, 0 },
    { 0x2f22004a, 53 },
    { 0xc5d2004a, 34 },
    { 0, 0 },
    { 0x7d52004d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x89020050, 68 },
    { 0x69820051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e2006c, 90 },
    { 0x6152006d, 23 },
    { 0xebe2006e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f20076, 61 },
    { 0x48620077, 98 },
    { 0, 0 },
    { 0x48620079, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd42007c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98520080, 61 },
    { 0, 0 },
    { 0xb0b20082, 55 },
    { 0x42020083, 76 },
    { 0xa5b20083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb32008a, 33 },
    { 0x1902008b, 52 },
    { 0xcc22008c, 45 },
    { 0xda62008d, 98 },
    { 0x4ce2008c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50820094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf42009e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed200a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22200ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1200b1, 98 },
    { 0xf83200b2, 23 },
    { 0x75b200b3, 33 },
    { 0xc82200b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4200b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d200bc, 46 },
    { 0xcb2200bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51200c2, 33 },
    { 0, 0 },
    { 0x28d200c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e200c7, 85 },
    { 0xc8b200c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe200d2, 61 },
    { 0, 0 },
    { 0x86d200d4, 93 },
    { 0xaa5200d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea200dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e200e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151200e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c200eb, 84 },
    { 0, 0 },
    { 0x8a9200ed, 43 },
    { 0x403200ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad200f4, 61 },
    { 0x380200f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5200fa, 98 },
    { 0x76c200fb, 23 },
    { 0xe15200fc, 90 },
    { 0x7eb200fd, 55 },
    { 0x61b200fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11820104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74120108, 61 },
    { 0, 0 },
    { 0x7722010a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b20110, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47820116, 83 },
    { 0xfc320116, 33 },
    { 0x72020118, 87 },
    { 0xddd20117, 96 },
    { 0xc582011a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192011e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f420122, 65 },
    { 0, 0 },
    { 0xee120124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9020128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22012b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90620130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab20133, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20136, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d20144, 90 },
    { 0xf7220145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c520149, 34 },
    { 0, 0 },
    { 0xc912014b, 76 },
    { 0, 0 },
    { 0x6c62014d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86420153, 33 },
    { 0xaa120154, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a20159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c2015d, 90 },
    { 0, 0 },
    { 0xd3e2015f, 90 },
    { 0x3542015f, 90 },
    { 0, 0 },
    { 0x7ad20162, 33 },
    { 0, 0 },
    { 0xf2920164, 33 },
    { 0x3a20164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf182016d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac20170, 90 },
    { 0x69e20170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f20178, 33 },
    { 0, 0 },
    { 0x6b32017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652017e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe20183, 99 },
    { 0x6b120184, 90 },
    { 0x14d20185, 90 },
    { 0xcd520185, 34 },
    { 0x3420185, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292018c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf20195, 90 },
    { 0, 0 },
    { 0x46b20197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1462019b, 33 },
    { 0, 0 },
    { 0x99a2019d, 75 },
    { 0, 0 },
    { 0xa652019f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20201a9, 98 },
    { 0xbdf201a9, 77 },
    { 0x517201aa, 55 },
    { 0, 0 },
    { 0x74b201ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c201b4, 76 },
    { 0xbd201b4, 84 },
    { 0, 0 },
    { 0x1fe201b7, 43 },
    { 0, 0 },
    { 0xe7201b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a201bc, 95 },
    { 0, 0 },
    { 0x360201be, 93 },
    { 0x3c4201bf, 43 },
    { 0x167201c0, 46 },
    { 0xbf9201c1, 88 },
    { 0x8dc201c1, 95 },
    { 0x5a4201c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x208201c6, 98 },
    { 0, 0 },
    { 0x3ee201c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd201cc, 85 },
    { 0x198201cd, 46 },
    { 0x958201ce, 33 },
    { 0xc79201cf, 43 },
    { 0x21b201d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42201d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73201dd, 85 },
    { 0, 0 },
    { 0xf7f201df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89201e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986201e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x117201ea, 33 },
    { 0xf6d201ea, 98 },
    { 0x5db201ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b201f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16201f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827201fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb20200, 84 },
    { 0, 0 },
    { 0xdde20202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89420205, 43 },
    { 0, 0 },
    { 0xb6920207, 33 },
    { 0x61120208, 47 },
    { 0x71e20208, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf62020c, 98 },
    { 0xbf32020d, 23 },
    { 0x2622020c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d920212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43420218, 90 },
    { 0x80020219, 23 },
    { 0x81120219, 23 },
    { 0x5402021b, 90 },
    { 0, 0 },
    { 0xba42021d, 55 },
    { 0xc712021e, 98 },
    { 0, 0 },
    { 0xf0720220, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2920224, 37 },
    { 0, 0 },
    { 0x3020226, 87 },
    { 0x4b020227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f2022d, 98 },
    { 0, 0 },
    { 0xc6e2022f, 98 },
    { 0, 0 },
    { 0x80a20231, 68 },
    { 0x19320231, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8120235, 23 },
    { 0, 0 },
    { 0x6c420237, 93 },
    { 0x65720237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1622023c, 52 },
    { 0x86b2023c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae720245, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f20248, 78 },
    { 0, 0 },
    { 0x9ec2024a, 98 },
    { 0xf532024b, 95 },
    { 0x28c2024b, 53 },
    { 0x4c52024c, 23 },
    { 0x5992024e, 76 },
    { 0xa932024e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db20252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9720259, 55 },
    { 0xf2b2025a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362025d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45220270, 68 },
    { 0xb5d20271, 61 },
    { 0x38520272, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f20275, 33 },
    { 0xce120275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e02027a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e2027f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5220283, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c20287, 33 },
    { 0xa3120288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b2028c, 33 },
    { 0x64d2028c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9920292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2029d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3202a5, 87 },
    { 0x423202a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459202ad, 55 },
    { 0x669202ae, 33 },
    { 0x331202ad, 95 },
    { 0xd88202b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84202b5, 98 },
    { 0x670202b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b202b9, 90 },
    { 0, 0 },
    { 0x73d202bb, 77 },
    { 0, 0 },
    { 0x998202bd, 76 },
    { 0, 0 },
    { 0xd4f202bf, 33 },
    { 0x281202bf, 33 },
    { 0xf18202c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318202c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69202d1, 66 },
    { 0x5a7202d1, 34 },
    { 0, 0 },
    { 0x557202d4, 90 },
    { 0, 0 },
    { 0x6c5202d6, 34 },
    { 0, 0 },
    { 0x714202d8, 90 },
    { 0, 0 },
    { 0x298202da, 75 },
    { 0x144202db, 33 },
    { 0xe57202db, 93 },
    { 0x93d202dd, 90 },
    { 0x9d202dd, 78 },
    { 0x774202dc, 61 },
    { 0x9cb202da, 66 },
    { 0xb9202e1, 87 },
    { 0, 0 },
    { 0xe66202e3, 61 },
    { 0xc43202e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05202ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1202ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x701202f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e202f9, 61 },
    { 0xbeb202fa, 93 },
    { 0xd6b202fa, 55 },
    { 0xada202fc, 90 },
    { 0, 0 },
    { 0x54c202fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c20302, 95 },
    { 0, 0 },
    { 0xe7720304, 67 },
    { 0x9d520305, 46 },
    { 0xc3520305, 95 },
    { 0x9220307, 66 },
    { 0xadd20308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42030b, 61 },
    { 0x24a2030c, 65 },
    { 0, 0 },
    { 0x182030e, 61 },
    { 0, 0 },
    { 0x20220310, 87 },
    { 0xd0820311, 99 },
    { 0, 0 },
    { 0xd5a20313, 90 },
    { 0xbee20314, 33 },
    { 0xfb120315, 55 },
    { 0x14d20313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f2031a, 33 },
    { 0x3a52031a, 87 },
    { 0x5e2031b, 90 },
    { 0x99a2031d, 61 },
    { 0xc572031b, 90 },
    { 0x4ea2031f, 33 },
    { 0x69420320, 95 },
    { 0x1652031b, 78 },
    { 0, 0 },
    { 0x7c220323, 78 },
    { 0xd4320324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f20328, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c2032b, 76 },
    { 0, 0 },
    { 0xff42032d, 61 },
    { 0x6d2032d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a20337, 33 },
    { 0x7fd20338, 33 },
    { 0xd5f20339, 90 },
    { 0x31c2033a, 95 },
    { 0x3fb20337, 61 },
    { 0xc542033c, 100 },
    { 0xff2033c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68820340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b220344, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35220347, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2682034c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87320355, 95 },
    { 0x36c20355, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2035a, 76 },
    { 0, 0 },
    { 0x95c2035c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94120369, 33 },
    { 0, 0 },
    { 0x9dd2036b, 75 },
    { 0, 0 },
    { 0x38f2036d, 88 },
    { 0xcc82036d, 94 },
    { 0x5c2036f, 100 },
    { 0xc2420370, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e320374, 90 },
    { 0xa2d20375, 98 },
    { 0x8b620376, 43 },
    { 0x2a220377, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5422037b, 61 },
    { 0x5572037c, 55 },
    { 0xda32037d, 76 },
    { 0xcb2037e, 33 },
    { 0xcf2037f, 66 },
    { 0x4432037b, 78 },
    { 0xb3f20381, 65 },
    { 0x34720382, 98 },
    { 0x5f920381, 90 },
    { 0x2bf2037f, 67 },
    { 0, 0 },
    { 0xa4d20386, 34 },
    { 0, 0 },
    { 0x49a20388, 55 },
    { 0x7da20389, 40 },
    { 0, 0 },
    { 0xe5b2038b, 100 },
    { 0, 0 },
    { 0x6c72038d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc20392, 23 },
    { 0x48820393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d20396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2039c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2039f, 17 },
    { 0x9af203a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e203a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03203a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371203ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x465203ae, 90 },
    { 0x4b2203af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7203b7, 61 },
    { 0xdf0203b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d203bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x219203be, 33 },
    { 0x3e3203bf, 88 },
    { 0xab0203c0, 98 },
    { 0x43b203c0, 95 },
    { 0xf43203c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30203c6, 55 },
    { 0xfcc203c7, 95 },
    { 0x1a5203c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d203cc, 61 },
    { 0x575203cd, 17 },
    { 0xe2b203cd, 23 },
    { 0x8f1203cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x188203d2, 33 },
    { 0x687203d3, 61 },
    { 0x229203d4, 61 },
    { 0xcce203d5, 61 },
    { 0x690203d6, 61 },
    { 0x32b203d5, 23 },
    { 0x15203d4, 93 },
    { 0xa8203d9, 98 },
    { 0x1bf203d2, 100 },
    { 0xea0203d2, 76 },
    { 0x4a9203dc, 98 },
    { 0x2d4203d4, 100 },
    { 0x83e203de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b203ed, 17 },
    { 0, 0 },
    { 0x410203ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f203f4, 65 },
    { 0, 0 },
    { 0x887203f6, 68 },
    { 0xd36203f7, 91 },
    { 0xb00203f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b20404, 93 },
    { 0xef120405, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e2040a, 90 },
    { 0x43c2040a, 61 },
    { 0x7d32040c, 61 },
    { 0xd702040c, 55 },
    { 0xf372040c, 98 },
    { 0xf032040f, 68 },
    { 0xfe92040f, 99 },
    { 0x50d20411, 95 },
    { 0xc2b2040a, 90 },
    { 0, 0 },
    { 0x23620414, 55 },
    { 0x5620415, 90 },
    { 0x120416, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f20425, 91 },
    { 0xb1e20426, 76 },
    { 0xb8920427, 75 },
    { 0x3b320428, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32042c, 90 },
    { 0xfb92042d, 90 },
    { 0x222042d, 55 },
    { 0x422042f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2420432, 61 },
    { 0, 0 },
    { 0xfd520434, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee22043d, 66 },
    { 0x9e92043e, 76 },
    { 0x8d92043f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0320442, 76 },
    { 0, 0 },
    { 0x73020444, 100 },
    { 0xac20445, 99 },
    { 0, 0 },
    { 0x3e20447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d2044b, 66 },
    { 0xc472044b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2122044f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a420454, 90 },
    { 0, 0 },
    { 0xa2d20456, 99 },
    { 0x4ad20456, 78 },
    { 0x5b320458, 90 },
    { 0xa1b20459, 78 },
    { 0x72020459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a20460, 23 },
    { 0, 0 },
    { 0x5c120462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93120468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92046b, 33 },
    { 0x4d82046c, 76 },
    { 0x972046d, 76 },
    { 0, 0 },
    { 0x5602046f, 85 },
    { 0, 0 },
    { 0xc2e20471, 33 },
    { 0x70820472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff820477, 99 },
    { 0xbf920478, 61 },
    { 0x7ba20477, 88 },
    { 0, 0 },
    { 0x7192047b, 61 },
    { 0x2f42047b, 65 },
    { 0x4532047d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa20480, 55 },
    { 0x7d520481, 33 },
    { 0x1bd20482, 98 },
    { 0, 0 },
    { 0xf7720484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c20488, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9482048b, 75 },
    { 0x9912048b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2048f, 17 },
    { 0x88320490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3420493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d20496, 52 },
    { 0x86920497, 76 },
    { 0xb3920498, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6102049f, 98 },
    { 0x2fa204a0, 90 },
    { 0, 0 },
    { 0xe16204a2, 61 },
    { 0x446204a3, 76 },
    { 0xd8d204a4, 77 },
    { 0, 0 },
    { 0x107204a6, 61 },
    { 0, 0 },
    { 0x797204a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6204b0, 66 },
    { 0x715204b1, 55 },
    { 0xe09204b2, 61 },
    { 0x706204b2, 98 },
    { 0, 0 },
    { 0xf31204b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b204ba, 84 },
    { 0xbd8204bb, 93 },
    { 0x712204ba, 43 },
    { 0xd9d204bd, 90 },
    { 0x6d2204be, 81 },
    { 0x5b2204bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29204c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e204cb, 33 },
    { 0, 0 },
    { 0x8b6204cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f204d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4204d6, 23 },
    { 0xae9204d7, 98 },
    { 0, 0 },
    { 0x71f204d9, 77 },
    { 0, 0 },
    { 0xb10204db, 33 },
    { 0x5d1204dc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87204e1, 76 },
    { 0x7d4204e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c204e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d204ef, 76 },
    { 0x268204f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x142204f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05204f8, 33 },
    { 0, 0 },
    { 0x640204fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c20501, 78 },
    { 0x68120502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a20505, 85 },
    { 0x92220506, 100 },
    { 0xce820507, 68 },
    { 0x2fa20507, 78 },
    { 0xfd920509, 88 },
    { 0xdae20508, 95 },
    { 0, 0 },
    { 0x4ce2050c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89720513, 33 },
    { 0x5e820514, 17 },
    { 0, 0 },
    { 0x71d20516, 33 },
    { 0, 0 },
    { 0x17a20518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5582051c, 87 },
    { 0x2ee2051c, 23 },
    { 0, 0 },
    { 0x9df2051f, 90 },
    { 0, 0 },
    { 0x31e20521, 33 },
    { 0x33e20522, 98 },
    { 0, 0 },
    { 0x46b20524, 91 },
    { 0, 0 },
    { 0x3d020526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e42052a, 100 },
    { 0, 0 },
    { 0xaa2052c, 43 },
    { 0xc322052d, 98 },
    { 0x3aa2052e, 55 },
    { 0x4c42052f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5220536, 98 },
    { 0x9b820537, 90 },
    { 0x20f20538, 98 },
    { 0, 0 },
    { 0x1042053a, 23 },
    { 0, 0 },
    { 0x5022053c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48520541, 93 },
    { 0x4ed20542, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c920548, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc2054c, 88 },
    { 0x5632054d, 95 },
    { 0x2e82054d, 98 },
    { 0x5622054f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df20554, 60 },
    { 0x36920555, 23 },
    { 0xe7a20556, 77 },
    { 0xa3e20554, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b2055a, 38 },
    { 0x66a2055b, 95 },
    { 0xfa62055c, 46 },
    { 0, 0 },
    { 0x5242055e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64320563, 76 },
    { 0xea720563, 87 },
    { 0x50620565, 78 },
    { 0, 0 },
    { 0x15020567, 61 },
    { 0xed020568, 33 },
    { 0xfc220568, 98 },
    { 0xf3720567, 76 },
    { 0xc0f2056a, 38 },
    { 0, 0 },
    { 0x8e62056d, 34 },
    { 0x91f2056e, 33 },
    { 0x47f2056f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f20572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f720575, 90 },
    { 0x12220575, 95 },
    { 0xc8320577, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a72057c, 61 },
    { 0x1172057d, 61 },
    { 0, 0 },
    { 0xa842057f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd20584, 55 },
    { 0, 0 },
    { 0x49720586, 17 },
    { 0, 0 },
    { 0x10820588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf52058c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c2058f, 90 },
    { 0xbcb20590, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b20593, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6520596, 33 },
    { 0x84520596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd352059e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x654205a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x461205a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd205aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f205af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f205b4, 91 },
    { 0x1a4205b5, 90 },
    { 0xd72205b4, 93 },
    { 0x1bc205b6, 90 },
    { 0x34d205b8, 90 },
    { 0xa59205b9, 37 },
    { 0x5dc205b9, 90 },
    { 0xdf0205bb, 66 },
    { 0xe5d205b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2205bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17205c2, 76 },
    { 0x2f3205c3, 61 },
    { 0x7ac205c3, 90 },
    { 0xd63205c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321205cc, 99 },
    { 0xfcd205cc, 93 },
    { 0x50c205ce, 76 },
    { 0x2c3205cf, 52 },
    { 0xaf4205cd, 43 },
    { 0x65b205d1, 33 },
    { 0x1b6205cd, 61 },
    { 0x1c1205cc, 93 },
    { 0x622205d4, 76 },
    { 0x352205d5, 17 },
    { 0xa01205d6, 76 },
    { 0x288205d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x439205da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6205e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90205e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa205e9, 23 },
    { 0xfe9205ea, 33 },
    { 0, 0 },
    { 0x6205ec, 33 },
    { 0, 0 },
    { 0x127205ee, 98 },
    { 0, 0 },
    { 0x3ce205f0, 98 },
    { 0xc6b205f1, 55 },
    { 0x362205f0, 33 },
    { 0xad8205f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd205f9, 33 },
    { 0xad8205fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7205ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48020606, 33 },
    { 0x12920606, 55 },
    { 0x99320608, 76 },
    { 0xef820608, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c2060d, 51 },
    { 0xa842060e, 90 },
    { 0, 0 },
    { 0xceb20610, 33 },
    { 0x90320610, 90 },
    { 0, 0 },
    { 0x24920613, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd20618, 90 },
    { 0, 0 },
    { 0x91c2061a, 100 },
    { 0x84a2061b, 67 },
    { 0x1452061c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87720623, 95 },
    { 0xca320624, 90 },
    { 0xdca20625, 98 },
    { 0x6aa20626, 33 },
    { 0x83c20627, 98 },
    { 0xf8a20627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe522062b, 85 },
    { 0x9a2062b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b420634, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78620637, 95 },
    { 0x94b20638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b20640, 34 },
    { 0, 0 },
    { 0xfc820642, 98 },
    { 0x66020642, 76 },
    { 0x94d20643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f420648, 33 },
    { 0x40820649, 23 },
    { 0xdb920649, 34 },
    { 0x9b32064b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa20651, 76 },
    { 0x83e20652, 85 },
    { 0x75720653, 61 },
    { 0x43720652, 87 },
    { 0x7c620652, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e320658, 43 },
    { 0, 0 },
    { 0x7f62065a, 90 },
    { 0, 0 },
    { 0x2772065c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde20660, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49520665, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38620668, 61 },
    { 0, 0 },
    { 0xb772066a, 23 },
    { 0x6cd2066b, 61 },
    { 0, 0 },
    { 0x9d82066d, 90 },
    { 0x72d2066d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd520671, 34 },
    { 0x90b20671, 23 },
    { 0xf7e20673, 61 },
    { 0, 0 },
    { 0x9fd20675, 61 },
    { 0xa9d20676, 34 },
    { 0, 0 },
    { 0x86a20678, 78 },
    { 0xe9220679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1452067e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5120681, 98 },
    { 0, 0 },
    { 0xbf420683, 98 },
    { 0x7b020683, 76 },
    { 0x2ca20683, 55 },
    { 0x67420686, 98 },
    { 0xb8020685, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9312068a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142068f, 90 },
    { 0, 0 },
    { 0x63c20691, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a220694, 90 },
    { 0x8e320695, 34 },
    { 0x47020696, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2069b, 90 },
    { 0, 0 },
    { 0xed12069d, 61 },
    { 0, 0 },
    { 0xc7d2069f, 84 },
    { 0, 0 },
    { 0x790206a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a206a5, 33 },
    { 0x3a0206a5, 90 },
    { 0, 0 },
    { 0xa57206a8, 98 },
    { 0xb24206a9, 53 },
    { 0xd50206aa, 34 },
    { 0xb17206aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859206b0, 93 },
    { 0x802206b0, 98 },
    { 0xe4e206b0, 95 },
    { 0xc0d206b1, 90 },
    { 0x221206b4, 98 },
    { 0x1da206b5, 23 },
    { 0x8d3206b4, 77 },
    { 0xf05206b4, 43 },
    { 0, 0 },
    { 0xd2f206b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a206bd, 93 },
    { 0, 0 },
    { 0x844206bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2206c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45206c6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03206cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9206d2, 98 },
    { 0x366206d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4206da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde206de, 100 },
    { 0x9a3206df, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300206e3, 43 },
    { 0xdf9206e4, 98 },
    { 0x10206e4, 33 },
    { 0x37a206e4, 61 },
    { 0x68e206e3, 43 },
    { 0, 0 },
    { 0x98b206e9, 100 },
    { 0xc33206ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198206ef, 67 },
    { 0, 0 },
    { 0xd37206f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614206f8, 87 },
    { 0xe1f206f8, 90 },
    { 0x5f6206f9, 93 },
    { 0x1ff206fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5206ff, 81 },
    { 0xd8d20700, 93 },
    { 0x6ba206ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692070a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6642070d, 23 },
    { 0xa102070d, 23 },
    { 0x232070e, 90 },
    { 0, 0 },
    { 0x66520711, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e20717, 43 },
    { 0xea20717, 61 },
    { 0x34620719, 98 },
    { 0, 0 },
    { 0x4092071b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37320723, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12920727, 80 },
    { 0xcc320728, 81 },
    { 0x9d720727, 23 },
    { 0x49a2072a, 61 },
    { 0x4f92072b, 85 },
    { 0xee82072a, 98 },
    { 0x6e12072d, 46 },
    { 0x7082072e, 77 },
    { 0xaf02072d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef820735, 33 },
    { 0, 0 },
    { 0xe2020737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca42073b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc732073e, 66 },
    { 0, 0 },
    { 0x1d920740, 77 },
    { 0, 0 },
    { 0x79020742, 55 },
    { 0x20a20743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80420747, 93 },
    { 0, 0 },
    { 0xbae20749, 93 },
    { 0x90720749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2074d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed520754, 33 },
    { 0x96720754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd2075c, 55 },
    { 0x90e2075d, 90 },
    { 0x2a02075e, 23 },
    { 0x9e82075f, 33 },
    { 0xe3a2075d, 33 },
    { 0x4892075c, 98 },
    { 0, 0 },
    { 0xb2820763, 34 },
    { 0xf1e20764, 98 },
    { 0, 0 },
    { 0xd8320766, 98 },
    { 0x98c20766, 34 },
    { 0, 0 },
    { 0x39920769, 46 },
    { 0x32020769, 64 },
    { 0, 0 },
    { 0x8f2076c, 98 },
    { 0x1e52076d, 76 },
    { 0x4532076e, 34 },
    { 0x3652076e, 46 },
    { 0x8b22076d, 61 },
    { 0x1d220771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd220775, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd602077c, 66 },
    { 0xade2077d, 90 },
    { 0xfdc2077e, 85 },
    { 0x8bd2077f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7920782, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a20785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda52078c, 95 },
    { 0x6142078d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0520796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b2079a, 59 },
    { 0x9092079b, 98 },
    { 0x46f2079a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf462079f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b207a5, 33 },
    { 0, 0 },
    { 0x427207a7, 90 },
    { 0x4cd207a8, 99 },
    { 0xebf207a9, 98 },
    { 0x4ab207a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x575207ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b207b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203207bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f207bf, 90 },
    { 0xfb8207bf, 61 },
    { 0x8d0207c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82207c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8207c7, 78 },
    { 0, 0 },
    { 0x914207c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645207d0, 98 },
    { 0x39a207d0, 33 },
    { 0x48e207d1, 98 },
    { 0xd18207d2, 93 },
    { 0, 0 },
    { 0xf54207d5, 98 },
    { 0x9f207d5, 76 },
    { 0xe0a207d7, 90 },
    { 0x5e4207d7, 85 },
    { 0xfaa207d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25207dd, 61 },
    { 0, 0 },
    { 0xbcb207df, 76 },
    { 0x136207e0, 33 },
    { 0, 0 },
    { 0x956207e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb207e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd207eb, 23 },
    { 0xc05207ec, 76 },
    { 0x7b6207eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce207f2, 68 },
    { 0xf40207f2, 46 },
    { 0x98f207f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e207fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512207fe, 49 },
    { 0xca1207fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46220802, 95 },
    { 0xcb920803, 33 },
    { 0xd6820802, 75 },
    { 0x2df20805, 43 },
    { 0, 0 },
    { 0x52e20807, 61 },
    { 0x89220808, 98 },
    { 0, 0 },
    { 0xadc2080a, 2 },
    { 0xe482080b, 33 },
    { 0xbc92080c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d62080f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e20813, 100 },
    { 0xe3d20814, 87 },
    { 0x6ef20815, 90 },
    { 0x57520814, 61 },
    { 0x4d120816, 90 },
    { 0x12920818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf72081c, 23 },
    { 0, 0 },
    { 0xcae2081e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6220826, 46 },
    { 0x79f20826, 90 },
    { 0, 0 },
    { 0xb420829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf262082f, 95 },
    { 0, 0 },
    { 0xc5d20831, 93 },
    { 0xeee20832, 23 },
    { 0x2020833, 90 },
    { 0x77d20831, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b20837, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d02083f, 76 },
    { 0x89020840, 90 },
    { 0xea120841, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66820845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e20849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e2084d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b220850, 33 },
    { 0x6b520851, 52 },
    { 0, 0 },
    { 0xe3120853, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de20858, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d20862, 61 },
    { 0x78520863, 61 },
    { 0x8cb20864, 43 },
    { 0x3c420862, 2 },
    { 0, 0 },
    { 0x28220867, 95 },
    { 0xcfd20868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2086f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d20873, 75 },
    { 0x6de20873, 99 },
    { 0x41020874, 87 },
    { 0x1a420873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf72087d, 23 },
    { 0, 0 },
    { 0x2c22087f, 90 },
    { 0x1242087f, 33 },
    { 0x7a920881, 81 },
    { 0xf6220882, 90 },
    { 0x94b20883, 95 },
    { 0x4020882, 95 },
    { 0xa7120882, 33 },
    { 0xc4820886, 34 },
    { 0x77820887, 61 },
    { 0x74f20888, 65 },
    { 0x1cf20887, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe142088c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e20893, 76 },
    { 0x98d20894, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120898, 55 },
    { 0, 0 },
    { 0xaa82089a, 95 },
    { 0, 0 },
    { 0x9342089c, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2089f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c208a2, 52 },
    { 0x4fe208a3, 90 },
    { 0x96b208a4, 34 },
    { 0x61208a5, 77 },
    { 0x3f0208a5, 67 },
    { 0xde208a7, 33 },
    { 0x89a208a8, 61 },
    { 0xc14208a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38208af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48208b6, 34 },
    { 0, 0 },
    { 0x267208b8, 61 },
    { 0x482208b9, 55 },
    { 0, 0 },
    { 0x6bb208bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16208bf, 94 },
    { 0, 0 },
    { 0x7e8208c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86208c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb208ca, 100 },
    { 0xabc208ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373208d1, 52 },
    { 0x4ed208d1, 55 },
    { 0xcb2208d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda208d7, 77 },
    { 0, 0 },
    { 0x474208d9, 46 },
    { 0, 0 },
    { 0xfb7208db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c208df, 33 },
    { 0, 0 },
    { 0x8d7208e1, 91 },
    { 0xbe7208e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1208e6, 23 },
    { 0xdcb208e7, 67 },
    { 0x717208e8, 61 },
    { 0xd29208e9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f208ec, 81 },
    { 0, 0 },
    { 0xd1a208ee, 34 },
    { 0, 0 },
    { 0x677208f0, 95 },
    { 0x513208f1, 95 },
    { 0, 0 },
    { 0xb1b208f3, 88 },
    { 0x232208f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x517208f7, 95 },
    { 0, 0 },
    { 0xa74208f9, 33 },
    { 0xac0208fa, 76 },
    { 0xd94208fb, 90 },
    { 0, 0 },
    { 0x9f5208fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d20905, 84 },
    { 0x99020905, 100 },
    { 0xde620906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d92090a, 48 },
    { 0, 0 },
    { 0xcb22090c, 66 },
    { 0, 0 },
    { 0x5862090e, 98 },
    { 0, 0 },
    { 0x75d20910, 88 },
    { 0xdd320911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7620919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c20922, 43 },
    { 0x4e620923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39020929, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd20930, 66 },
    { 0x6d520931, 90 },
    { 0x6db20931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9420935, 34 },
    { 0xc9520936, 91 },
    { 0x4ed20936, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9612093d, 23 },
    { 0xd492093e, 98 },
    { 0xfa42093f, 80 },
    { 0, 0 },
    { 0x11a20941, 90 },
    { 0xc7d20941, 99 },
    { 0, 0 },
    { 0xc1720944, 80 },
    { 0xcc820945, 23 },
    { 0x55720945, 93 },
    { 0x3f520946, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0620950, 88 },
    { 0x79a20950, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc20955, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1720959, 92 },
    { 0xc032095a, 77 },
    { 0x6242095b, 36 },
    { 0x2002095c, 90 },
    { 0x9b32095d, 34 },
    { 0x6a220959, 90 },
    { 0x2c42095f, 52 },
    { 0x342095f, 80 },
    { 0, 0 },
    { 0xa6320962, 90 },
    { 0xae020963, 95 },
    { 0x7e820964, 99 },
    { 0, 0 },
    { 0x2d320966, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8620969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2096e, 81 },
    { 0, 0 },
    { 0x5ef20970, 81 },
    { 0x72020970, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b20977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd462097b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692097e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b520982, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a20987, 52 },
    { 0xc9420987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2098e, 95 },
    { 0, 0 },
    { 0x41020990, 90 },
    { 0, 0 },
    { 0x8e220992, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52320998, 34 },
    { 0, 0 },
    { 0x3b22099a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef209a0, 91 },
    { 0xd54209a1, 90 },
    { 0xadc209a1, 61 },
    { 0xc6a209a3, 76 },
    { 0x268209a4, 43 },
    { 0x19209a5, 99 },
    { 0xcdc209a5, 66 },
    { 0xd08209a7, 23 },
    { 0x18f209a7, 52 },
    { 0xc32209a9, 67 },
    { 0xc20209a2, 47 },
    { 0x970209a5, 90 },
    { 0, 0 },
    { 0xe8e209ad, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7209b0, 61 },
    { 0x44f209b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90209b5, 91 },
    { 0xe2e209b5, 61 },
    { 0x7d5209b7, 33 },
    { 0xb6209b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96209bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d209c3, 23 },
    { 0x6e0209c3, 55 },
    { 0x21f209c3, 33 },
    { 0xfbe209c6, 67 },
    { 0x37e209c3, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2209cc, 100 },
    { 0xba4209cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280209d1, 55 },
    { 0xdab209d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c209d6, 33 },
    { 0xbfd209d6, 33 },
    { 0, 0 },
    { 0x22c209d9, 95 },
    { 0x2c6209d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x933209dd, 98 },
    { 0xb0b209de, 98 },
    { 0x454209de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e209ea, 61 },
    { 0x6a5209eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9209ee, 95 },
    { 0xbb1209ef, 61 },
    { 0xad3209f0, 33 },
    { 0xb13209f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8209f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf20a01, 90 },
    { 0x7fa20a01, 90 },
    { 0x3aa20a03, 46 },
    { 0x68220a04, 90 },
    { 0, 0 },
    { 0xd0d20a06, 90 },
    { 0x1e320a07, 95 },
    { 0xdd20a08, 66 },
    { 0x38520a06, 61 },
    { 0x71d20a06, 95 },
    { 0xb20a0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb420a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c620a12, 61 },
    { 0x8ea20a13, 43 },
    { 0x53820a14, 90 },
    { 0xea420a15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020a18, 76 },
    { 0xd8820a19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c20a1e, 61 },
    { 0x57020a1e, 61 },
    { 0xaa820a20, 33 },
    { 0x9420a1e, 90 },
    { 0x7fa20a22, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x43520a25, 33 },
    { 0x64220a26, 95 },
    { 0, 0 },
    { 0x63e20a28, 76 },
    { 0x89720a29, 61 },
    { 0, 0 },
    { 0x9e820a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb120a30, 93 },
    { 0, 0 },
    { 0x8b20a32, 87 },
    { 0xd7f20a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6120a36, 84 },
    { 0x17020a37, 95 },
    { 0x7e920a38, 95 },
    { 0x4e820a39, 61 },
    { 0x7bc20a3a, 66 },
    { 0x8ea20a3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51020a3f, 90 },
    { 0x70720a3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa420a44, 98 },
    { 0x5b120a45, 95 },
    { 0x5ed20a44, 46 },
    { 0x3bf20a47, 61 },
    { 0, 0 },
    { 0xed520a49, 76 },
    { 0xaa220a4a, 98 },
    { 0, 0 },
    { 0x82c20a4c, 55 },
    { 0x13220a4d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49020a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94020a5c, 36 },
    { 0x4bb20a5d, 23 },
    { 0xb8f20a5d, 33 },
    { 0x56420a5f, 90 },
    { 0xdda20a60, 75 },
    { 0, 0 },
    { 0xcf220a62, 90 },
    { 0x28520a62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71320a66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27320a6a, 33 },
    { 0x4c620a6b, 95 },
    { 0x3db20a6a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c20a70, 90 },
    { 0xe5220a70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18920a75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c20a7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd920a7e, 98 },
    { 0x60b20a7f, 55 },
    { 0x37520a80, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a20a87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd20a90, 55 },
    { 0x50620a91, 91 },
    { 0xb2820a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0920a95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd20a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20a9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20220aa2, 85 },
    { 0xec020aa2, 76 },
    { 0, 0 },
    { 0x22c20aa5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36820aa8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d20aae, 44 },
    { 0x5a420aaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84720ab2, 77 },
    { 0xd2320ab3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e20ab9, 100 },
    { 0, 0 },
    { 0x40220abb, 33 },
    { 0xa0a20abc, 34 },
    { 0x15720abb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b20ac0, 33 },
    { 0, 0 },
    { 0x45920ac2, 95 },
    { 0, 0 },
    { 0xec020ac4, 23 },
    { 0x93220ac5, 95 },
    { 0, 0 },
    { 0x23420ac7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f020acb, 95 },
    { 0x3ca20acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d20ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f20ad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c20add, 90 },
    { 0x4bf20add, 91 },
    { 0, 0 },
    { 0x31620ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82320aeb, 34 },
    { 0xab220aec, 23 },
    { 0x48d20aed, 34 },
    { 0xec920aee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c220af6, 22 },
    { 0, 0 },
    { 0xb5320af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78120b06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e20b0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24320b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91920b19, 97 },
    { 0xed20b1a, 55 },
    { 0x30820b19, 95 },
    { 0xad220b1c, 87 },
    { 0xb7820b1d, 98 },
    { 0x1b320b1e, 46 },
    { 0x2f020b1e, 100 },
    { 0xf9920b20, 23 },
    { 0, 0 },
    { 0x72120b22, 76 },
    { 0, 0 },
    { 0xb3f20b24, 43 },
    { 0, 0 },
    { 0xf0f20b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6420b29, 98 },
    { 0x14420b2a, 95 },
    { 0x29520b2a, 33 },
    { 0, 0 },
    { 0xe0020b2d, 23 },
    { 0, 0 },
    { 0x1a120b2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd20b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53120b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e420b39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd120b3f, 101 },
    { 0, 0 },
    { 0x92d20b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc920b44, 77 },
    { 0x4c20b44, 43 },
    { 0x2cb20b45, 33 },
    { 0xb7720b45, 61 },
    { 0x43e20b48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6820b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66020b54, 55 },
    { 0x7720b55, 55 },
    { 0, 0 },
    { 0xbf520b57, 95 },
    { 0x1e920b58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c20b5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88620b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4720b64, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58120b68, 78 },
    { 0x87020b69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d20b6e, 33 },
    { 0x2da20b6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf720b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e820b76, 99 },
    { 0, 0 },
    { 0xe9c20b78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b920b7c, 98 },
    { 0x4b520b7d, 91 },
    { 0xda220b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5620b82, 90 },
    { 0xb0620b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53620b8f, 90 },
    { 0x77b20b90, 61 },
    { 0x49320b90, 34 },
    { 0, 0 },
    { 0x15a20b93, 77 },
    { 0, 0 },
    { 0x1720b95, 93 },
    { 0x1720b96, 93 },
    { 0x90320b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15220b9d, 90 },
    { 0x3d320b9d, 11 },
    { 0x8d420b9f, 77 },
    { 0x1c20b9d, 61 },
    { 0x7cb20ba1, 55 },
    { 0x68520ba2, 33 },
    { 0xefd20ba1, 98 },
    { 0x9b20ba2, 97 },
    { 0xc9520ba5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66320ba9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91520bb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e20bba, 46 },
    { 0xa5a20bba, 33 },
    { 0xf5c20bbc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xda020bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3120bc3, 91 },
    { 0xeea20bc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820bcb, 90 },
    { 0x98e20bcc, 61 },
    { 0xf4520bcc, 66 },
    { 0, 0 },
    { 0x77120bcf, 95 },
    { 0x2ae20bd0, 90 },
    { 0, 0 },
    { 0x30e20bd2, 98 },
    { 0x9a220bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5020bde, 61 },
    { 0x4b820bde, 85 },
    { 0, 0 },
    { 0x1ba20be1, 99 },
    { 0, 0 },
    { 0x10220be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee20be9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d20bed, 33 },
    { 0, 0 },
    { 0xff220bef, 61 },
    { 0x1f220bf0, 34 },
    { 0x85e20bf0, 78 },
    { 0x75220bf1, 61 },
    { 0x7c120bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f320bf8, 76 },
    { 0xd3720bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9220bfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac20c00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c20c03, 87 },
    { 0, 0 },
    { 0x86d20c05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9220c09, 23 },
    { 0x52f20c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b20c0e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b20c14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c20c1c, 81 },
    { 0x3af20c1d, 34 },
    { 0xc1820c1c, 93 },
    { 0xfe820c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9020c24, 99 },
    { 0xd7520c24, 34 },
    { 0x63220c25, 68 },
    { 0, 0 },
    { 0x92120c28, 93 },
    { 0x88620c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c820c2c, 61 },
    { 0xcd720c2d, 95 },
    { 0xb820c2c, 95 },
    { 0, 0 },
    { 0x56b20c30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f20c33, 81 },
    { 0xf3920c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc20c37, 84 },
    { 0x2d220c37, 33 },
    { 0xad820c37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b20c3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120c43, 78 },
    { 0xdaf20c44, 90 },
    { 0x48120c44, 61 },
    { 0x16b20c44, 90 },
    { 0x47720c47, 33 },
    { 0x64d20c48, 61 },
    { 0xa1620c48, 33 },
    { 0x34720c44, 95 },
    { 0x93d20c4b, 33 },
    { 0xfc320c48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae20c55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24120c5b, 95 },
    { 0xceb20c5c, 95 },
    { 0x67720c5d, 84 },
    { 0x33220c5e, 98 },
    { 0x13a20c5f, 61 },
    { 0xcd720c60, 98 },
    { 0x36420c5b, 61 },
    { 0x53820c62, 91 },
    { 0x34020c63, 84 },
    { 0x1c620c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a20c67, 61 },
    { 0x1cf20c68, 93 },
    { 0xfc20c67, 33 },
    { 0x2c220c68, 98 },
    { 0xe2920c6b, 23 },
    { 0x7c920c6b, 95 },
    { 0x15820c6b, 98 },
    { 0x95c20c67, 76 },
    { 0x9ab20c6f, 33 },
    { 0xc3a20c70, 91 },
    { 0x76b20c70, 53 },
    { 0x46420c6f, 99 },
    { 0x81c20c73, 55 },
    { 0x48020c67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39420c77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2220c7b, 33 },
    { 0x2f420c7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc220c83, 61 },
    { 0xfac20c84, 90 },
    { 0xb1320c85, 76 },
    { 0xde620c83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba20c89, 33 },
    { 0xa0020c89, 34 },
    { 0x7a620c8b, 23 },
    { 0x58f20c8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d20c92, 90 },
    { 0x4ec20c93, 61 },
    { 0x6ab20c94, 17 },
    { 0xb2a20c94, 34 },
    { 0x50920c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea20c99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5020ca1, 34 },
    { 0, 0 },
    { 0xbb020ca3, 56 },
    { 0xc1920ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de20ca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87020cac, 95 },
    { 0, 0 },
    { 0xa5420cae, 100 },
    { 0x35820caf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa20cb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a20cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f20cbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d220cc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4820ccb, 98 },
    { 0xb1120ccc, 33 },
    { 0x8a220ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0620cd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac20cd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab20cdc, 33 },
    { 0x3f620cdc, 66 },
    { 0xc5d20cdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65120ce5, 98 },
    { 0x56720ce5, 90 },
    { 0x6520ce7, 91 },
    { 0, 0 },
    { 0x3f020ce9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1620cee, 23 },
    { 0, 0 },
    { 0xd6620cf0, 90 },
    { 0x88120cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc20cf6, 87 },
    { 0x2c320cf7, 34 },
    { 0xea020cf8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23420cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27120d00, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c20d04, 76 },
    { 0x1b020d04, 90 },
    { 0x8a920d06, 46 },
    { 0, 0 },
    { 0xbe020d08, 40 },
    { 0x8de20d08, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f20d10, 90 },
    { 0xbfb20d11, 95 },
    { 0x6ae20d12, 75 },
    { 0x6c120d13, 43 },
    { 0xf3220d14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1920d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7720d1c, 98 },
    { 0x89920d1d, 52 },
    { 0xa8120d1e, 90 },
    { 0x89020d1c, 65 },
    { 0xc2f20d1c, 95 },
    { 0, 0 },
    { 0xe7e20d22, 33 },
    { 0x47720d23, 66 },
    { 0x1e420d24, 33 },
    { 0x48020d23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee520d2a, 95 },
    { 0, 0 },
    { 0x71020d2c, 90 },
    { 0xfdb20d2d, 55 },
    { 0, 0 },
    { 0xf9c20d2f, 76 },
    { 0x85c20d30, 76 },
    { 0x70c20d31, 34 },
    { 0, 0 },
    { 0x43720d33, 98 },
    { 0, 0 },
    { 0xb0720d35, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa320d3b, 46 },
    { 0, 0 },
    { 0x77520d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab20d49, 43 },
    { 0, 0 },
    { 0x6f020d4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6020d50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c20d53, 61 },
    { 0xe5c20d53, 61 },
    { 0x35220d55, 95 },
    { 0xf8820d55, 23 },
    { 0xf4120d57, 47 },
    { 0xbd420d54, 100 },
    { 0x8ed20d55, 90 },
    { 0x2a320d57, 33 },
    { 0, 0 },
    { 0x6a620d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e520d5f, 66 },
    { 0xb120d60, 95 },
    { 0, 0 },
    { 0x9f920d62, 23 },
    { 0xd2e20d62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa20d69, 99 },
    { 0xc420d69, 46 },
    { 0, 0 },
    { 0x40b20d6c, 97 },
    { 0x34b20d6d, 61 },
    { 0xde920d6e, 77 },
    { 0, 0 },
    { 0xf6d20d70, 53 },
    { 0xa2620d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b20d76, 52 },
    { 0, 0 },
    { 0x51d20d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58820d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a20d83, 33 },
    { 0xd5720d84, 55 },
    { 0x90320d85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb520d8a, 43 },
    { 0, 0 },
    { 0xc7720d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f20d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51220d94, 33 },
    { 0xa20d94, 98 },
    { 0x3d120d94, 52 },
    { 0x6c720d97, 25 },
    { 0x53420d98, 34 },
    { 0x97120d95, 40 },
    { 0, 0 },
    { 0xca520d9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa20d9e, 23 },
    { 0x38f20d9f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b320da2, 93 },
    { 0xd9220da3, 90 },
    { 0x94220da2, 76 },
    { 0xfd120da2, 91 },
    { 0, 0 },
    { 0x57b20da7, 95 },
    { 0xca020da8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x20120dab, 46 },
    { 0x89b20dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a20db1, 90 },
    { 0xe1e20db1, 61 },
    { 0xffe20db3, 55 },
    { 0, 0 },
    { 0x7520db5, 32 },
    { 0x9f620db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb20db9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c820dc0, 99 },
    { 0, 0 },
    { 0x9d420dc2, 88 },
    { 0xaec20dc2, 95 },
    { 0xed620dc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c720dd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea20dd4, 61 },
    { 0xd4420dd4, 76 },
    { 0xd8b20dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d20ddc, 94 },
    { 0xf4120ddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e20de0, 98 },
    { 0xe5b20de1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b20de4, 84 },
    { 0, 0 },
    { 0x7e520de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d20dea, 33 },
    { 0x69120deb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20def, 80 },
    { 0, 0 },
    { 0x1da20df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53820df5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc20df8, 33 },
    { 0, 0 },
    { 0xa0e20dfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f20e03, 99 },
    { 0, 0 },
    { 0xe0120e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6920e0f, 33 },
    { 0x3ab20e10, 95 },
    { 0xfb620e10, 90 },
    { 0xed620e12, 61 },
    { 0x60320e13, 46 },
    { 0, 0 },
    { 0x52e20e15, 55 },
    { 0x89320e16, 76 },
    { 0xd0620e17, 90 },
    { 0x19e20e17, 85 },
    { 0, 0 },
    { 0x21f20e1a, 76 },
    { 0xb4620e1b, 77 },
    { 0, 0 },
    { 0x8ac20e1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a20e21, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e820e25, 61 },
    { 0x48720e26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3620e34, 78 },
    { 0x1f420e35, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a20e3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3920e43, 43 },
    { 0x53f20e44, 85 },
    { 0x80820e45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b20e48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff020e4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae20e55, 43 },
    { 0x84520e56, 52 },
    { 0x43420e56, 98 },
    { 0x72820e56, 96 },
    { 0x1a20e59, 43 },
    { 0xbe920e57, 66 },
    { 0, 0 },
    { 0x30320e5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e20e61, 78 },
    { 0, 0 },
    { 0xbb320e63, 85 },
    { 0, 0 },
    { 0x6c520e65, 89 },
    { 0, 0 },
    { 0x96320e67, 90 },
    { 0, 0 },
    { 0x48220e69, 98 },
    { 0, 0 },
    { 0xe220e6b, 95 },
    { 0, 0 },
    { 0x88d20e6d, 33 },
    { 0, 0 },
    { 0xa1420e6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54220e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c20e81, 77 },
    { 0x18f20e82, 84 },
    { 0x6db20e83, 33 },
    { 0xacd20e82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39620e88, 76 },
    { 0, 0 },
    { 0x80120e8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f20e8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7320e93, 61 },
    { 0xae120e93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0120e98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16220ea4, 52 },
    { 0x23920ea5, 33 },
    { 0, 0 },
    { 0x7c920ea7, 91 },
    { 0x52320ea7, 93 },
    { 0x23720ea9, 33 },
    { 0xfe20ea7, 23 },
    { 0x4a820eab, 48 },
    { 0xb5720eac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c20eb4, 90 },
    { 0, 0 },
    { 0x4eb20eb6, 33 },
    { 0, 0 },
    { 0x33e20eb8, 95 },
    { 0, 0 },
    { 0x92f20eba, 23 },
    { 0x45d20eba, 23 },
    { 0x70e20ebc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0420ec4, 90 },
    { 0x5f420ec5, 95 },
    { 0xadd20ec5, 2 },
    { 0x74820ec6, 34 },
    { 0x62620ec8, 84 },
    { 0x22520ec5, 90 },
    { 0x62120eca, 61 },
    { 0x77a20ec6, 61 },
    { 0xbc220ec5, 61 },
    { 0x26120ec6, 90 },
    { 0, 0 },
    { 0x86220ecf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x74320ed2, 98 },
    { 0x9a520ed2, 55 },
    { 0xc4320ed3, 88 },
    { 0xc7a20ed4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60520edb, 61 },
    { 0, 0 },
    { 0xe5920edd, 23 },
    { 0xb1d20ede, 61 },
    { 0xf4320ede, 34 },
    { 0x27c20ede, 84 },
    { 0x6420ee0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d20ee4, 98 },
    { 0, 0 },
    { 0x72720ee6, 99 },
    { 0, 0 },
    { 0x68520ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa20eeb, 90 },
    { 0, 0 },
    { 0x84e20eed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8820ef6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d920efa, 76 },
    { 0xc5820efb, 17 },
    { 0xaa420efc, 46 },
    { 0x4bf20efd, 98 },
    { 0x76c20efe, 80 },
    { 0x54420efe, 98 },
    { 0, 0 },
    { 0x62220f01, 61 },
    { 0, 0 },
    { 0x1fd20f03, 23 },
    { 0x26b20f03, 55 },
    { 0xe5720f05, 90 },
    { 0, 0 },
    { 0x82420f07, 68 },
    { 0x67220f08, 43 },
    { 0x6e420f08, 90 },
    { 0, 0 },
    { 0x81f20f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e20f11, 77 },
    { 0xd9820f12, 33 },
    { 0x9be20f13, 78 },
    { 0, 0 },
    { 0xcfb20f15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e20f19, 93 },
    { 0, 0 },
    { 0x5a920f1b, 95 },
    { 0x6f220f1c, 46 },
    { 0, 0 },
    { 0xa1120f1e, 95 },
    { 0, 0 },
    { 0xba820f20, 93 },
    { 0xb6820f21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a20f25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da20f2e, 90 },
    { 0xfc120f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e20f33, 60 },
    { 0x8b820f34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d420f37, 90 },
    { 0x91b20f38, 39 },
    { 0x29820f39, 98 },
    { 0xc9720f38, 55 },
    { 0, 0 },
    { 0xc0c20f3c, 46 },
    { 0xca20f3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a20f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e20f44, 40 },
    { 0xd9b20f44, 55 },
    { 0x62220f46, 37 },
    { 0x49120f47, 91 },
    { 0xe5f20f48, 33 },
    { 0, 0 },
    { 0x7520f4a, 34 },
    { 0xeb720f4b, 78 },
    { 0xe4b20f4b, 61 },
    { 0x40f20f4a, 95 },
    { 0xbf120f4e, 93 },
    { 0xbd220f4f, 55 },
    { 0x44220f50, 99 },
    { 0xb9920f4e, 23 },
    { 0x7d120f51, 61 },
    { 0xe5f20f53, 33 },
    { 0x76220f53, 34 },
    { 0xdbb20f4f, 91 },
    { 0xaea20f52, 33 },
    { 0xab220f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b20f5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a720f5d, 90 },
    { 0xf820f5e, 82 },
    { 0x5e520f5f, 33 },
    { 0xa720f60, 61 },
    { 0x6bd20f5f, 33 },
    { 0xe5420f5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84020f65, 26 },
    { 0x3a620f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8620f76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4820f7b, 66 },
    { 0x8be20f7c, 100 },
    { 0x2a420f7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d20f81, 34 },
    { 0, 0 },
    { 0x1df20f83, 23 },
    { 0, 0 },
    { 0x5a20f85, 69 },
    { 0x3a720f85, 75 },
    { 0xd3420f87, 33 },
    { 0xfea20f88, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd20f8b, 84 },
    { 0, 0 },
    { 0xa2420f8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c20f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81820f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d420f9d, 65 },
    { 0, 0 },
    { 0x70020f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff920fa6, 100 },
    { 0x81c20fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15320fab, 55 },
    { 0x71c20fac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae820faf, 90 },
    { 0xfd420fb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82520fb4, 23 },
    { 0, 0 },
    { 0x47520fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b20fbc, 98 },
    { 0, 0 },
    { 0x48c20fbe, 95 },
    { 0, 0 },
    { 0x47e20fc0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7720fc3, 52 },
    { 0x6f320fc3, 61 },
    { 0x95b20fc3, 98 },
    { 0x25620fc4, 65 },
    { 0xc3b20fc5, 98 },
    { 0x8b520fc8, 93 },
    { 0x2dd20fc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6620fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec020fdc, 95 },
    { 0xce220fdd, 81 },
    { 0, 0 },
    { 0x7db20fdf, 98 },
    { 0x1bd20fe0, 55 },
    { 0x12520fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e120fe6, 76 },
    { 0, 0 },
    { 0xe8c20fe8, 85 },
    { 0xbad20fe9, 90 },
    { 0x8d620fea, 93 },
    { 0x20020feb, 90 },
    { 0xfd420fea, 98 },
    { 0, 0 },
    { 0xbe920fee, 43 },
    { 0, 0 },
    { 0x43a20ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb20ff3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb20ff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5021000, 46 },
    { 0x7b121000, 90 },
    { 0x2dd21002, 90 },
    { 0x69521003, 61 },
    { 0xd7721001, 34 },
    { 0x6fc21005, 98 },
    { 0x94021006, 33 },
    { 0x6b921007, 23 },
    { 0xa3821007, 95 },
    { 0xd0421008, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02100c, 23 },
    { 0x3ff2100d, 76 },
    { 0x5962100d, 93 },
    { 0x48b2100e, 33 },
    { 0, 0 },
    { 0x8f521011, 98 },
    { 0x1b321011, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e121015, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e21019, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2101d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d621023, 76 },
    { 0xf7c21024, 96 },
    { 0x80b21025, 33 },
    { 0x28521026, 98 },
    { 0x8e421027, 100 },
    { 0x85621028, 81 },
    { 0x8d721024, 61 },
    { 0, 0 },
    { 0xbbc2102b, 90 },
    { 0, 0 },
    { 0x7882102d, 91 },
    { 0x2232102e, 33 },
    { 0x4d12102f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee21036, 69 },
    { 0xe1021036, 43 },
    { 0xe121038, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b21042, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3921049, 33 },
    { 0, 0 },
    { 0x76b2104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b2104e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf621051, 33 },
    { 0xd0921052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e21057, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8152105f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1421065, 61 },
    { 0x62821066, 98 },
    { 0, 0 },
    { 0xe7721068, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1892106b, 33 },
    { 0, 0 },
    { 0x6582106d, 100 },
    { 0xef42106e, 33 },
    { 0xcf72106f, 90 },
    { 0x22121070, 95 },
    { 0, 0 },
    { 0x90221072, 69 },
    { 0, 0 },
    { 0x27921074, 61 },
    { 0, 0 },
    { 0x54121076, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac2107d, 76 },
    { 0, 0 },
    { 0xda22107f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90821082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d21085, 88 },
    { 0xbed21085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32108e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57821094, 33 },
    { 0xef721095, 90 },
    { 0, 0 },
    { 0xd7621097, 90 },
    { 0, 0 },
    { 0x9aa21099, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd2109e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x541210a1, 93 },
    { 0xef2210a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2210a5, 33 },
    { 0, 0 },
    { 0x49c210a7, 91 },
    { 0xca5210a7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961210b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6210b7, 61 },
    { 0x57e210b7, 76 },
    { 0xccb210b7, 46 },
    { 0, 0 },
    { 0x257210bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60210c0, 52 },
    { 0x873210c0, 95 },
    { 0xfc7210c1, 90 },
    { 0xd38210c1, 61 },
    { 0x637210c1, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30210c8, 91 },
    { 0x570210c9, 90 },
    { 0, 0 },
    { 0x88a210cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb210d3, 93 },
    { 0xfe7210d3, 61 },
    { 0xac7210d5, 100 },
    { 0, 0 },
    { 0x66210d7, 76 },
    { 0xa5b210d8, 98 },
    { 0x524210d7, 99 },
    { 0x7f9210da, 52 },
    { 0xb74210da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86210e1, 76 },
    { 0x1db210e1, 43 },
    { 0x101210e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7210e7, 95 },
    { 0xd210e8, 98 },
    { 0xa62210e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8210ed, 34 },
    { 0, 0 },
    { 0x9a210ef, 69 },
    { 0xe67210ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66210f7, 98 },
    { 0x945210f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3210fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83221103, 95 },
    { 0xb3e21104, 61 },
    { 0, 0 },
    { 0xa6a21106, 93 },
    { 0, 0 },
    { 0xf2c21108, 95 },
    { 0x2bd21109, 55 },
    { 0xf921109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c2110d, 34 },
    { 0, 0 },
    { 0x2a72110f, 46 },
    { 0x9792110f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c21113, 90 },
    { 0xec21114, 61 },
    { 0xb8221115, 98 },
    { 0xf8921115, 90 },
    { 0xcf321117, 90 },
    { 0x1c721113, 55 },
    { 0xa9421118, 61 },
    { 0, 0 },
    { 0x98e2111b, 64 },
    { 0xf7d2111c, 83 },
    { 0x5ad2111c, 99 },
    { 0xd9e2111b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45921128, 85 },
    { 0x4f721129, 76 },
    { 0xbf621128, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1921130, 67 },
    { 0x8e821131, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3321135, 98 },
    { 0x93b21136, 40 },
    { 0, 0 },
    { 0x36521138, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c21146, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb612114a, 76 },
    { 0x3262114b, 87 },
    { 0, 0 },
    { 0x6cd2114d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d821152, 23 },
    { 0x12921153, 90 },
    { 0, 0 },
    { 0x3d321155, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d21158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72115b, 95 },
    { 0x2aa2115c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f421160, 90 },
    { 0xafe21161, 52 },
    { 0x3d221160, 55 },
    { 0x63f21163, 43 },
    { 0, 0 },
    { 0x9c821165, 93 },
    { 0x6dc21166, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3052116d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21170, 90 },
    { 0x87e21171, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6521174, 88 },
    { 0, 0 },
    { 0x71f21176, 53 },
    { 0, 0 },
    { 0x1f21178, 34 },
    { 0, 0 },
    { 0xe302117a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2117f, 34 },
    { 0x9e121180, 23 },
    { 0xcf82117f, 78 },
    { 0, 0 },
    { 0x94b21183, 90 },
    { 0x88921183, 98 },
    { 0x97121185, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f2118e, 78 },
    { 0, 0 },
    { 0x8b121190, 33 },
    { 0x45021191, 98 },
    { 0, 0 },
    { 0xe9121193, 34 },
    { 0, 0 },
    { 0x6a921195, 90 },
    { 0x5b421196, 90 },
    { 0x22021196, 95 },
    { 0x7b21196, 98 },
    { 0x8b221199, 76 },
    { 0x7992119a, 90 },
    { 0x43421198, 95 },
    { 0xa0221196, 76 },
    { 0x15a21195, 95 },
    { 0xdb121196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4211a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6211a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5211a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e211ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2211b1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754211b6, 61 },
    { 0, 0 },
    { 0x5ed211b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5211bd, 98 },
    { 0, 0 },
    { 0x43b211bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d211c9, 78 },
    { 0x4cb211ca, 68 },
    { 0xbc2211c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3211ce, 33 },
    { 0x5eb211cf, 98 },
    { 0x24e211d0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166211d4, 23 },
    { 0, 0 },
    { 0x615211d6, 23 },
    { 0x66b211d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f211db, 80 },
    { 0, 0 },
    { 0x40f211dd, 95 },
    { 0xf81211de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8211e5, 98 },
    { 0x6dc211e6, 87 },
    { 0x438211e7, 77 },
    { 0x751211e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8211eb, 33 },
    { 0x68e211ec, 95 },
    { 0x71c211ed, 67 },
    { 0xe8a211ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2211f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb211f7, 88 },
    { 0, 0 },
    { 0x3ef211f9, 90 },
    { 0x66a211f9, 17 },
    { 0, 0 },
    { 0x156211fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b21200, 52 },
    { 0x9cc21200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63321204, 69 },
    { 0, 0 },
    { 0x32d21206, 78 },
    { 0x5a421207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c2120b, 17 },
    { 0x2422120c, 98 },
    { 0x5d82120d, 90 },
    { 0x1c82120d, 81 },
    { 0xcff2120c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d021214, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9221219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde21221, 33 },
    { 0xd721222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a921225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42122b, 84 },
    { 0, 0 },
    { 0x4bb2122d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f721230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf21234, 67 },
    { 0x6bd21235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf521238, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7742123c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb21241, 61 },
    { 0, 0 },
    { 0x14f21243, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b21246, 23 },
    { 0xd6d21247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2124a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef22124e, 66 },
    { 0, 0 },
    { 0x4e21250, 33 },
    { 0xf2521250, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad21258, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2125b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c21262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32126c, 61 },
    { 0x7732126c, 93 },
    { 0x40a2126c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d21271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0321275, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97321278, 90 },
    { 0, 0 },
    { 0x42f2127a, 76 },
    { 0x7dc2127a, 67 },
    { 0xff52127c, 74 },
    { 0xbc92127c, 95 },
    { 0xe662127e, 90 },
    { 0x17f2127e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af21282, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b21286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7582128a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192128e, 99 },
    { 0x2c32128e, 98 },
    { 0x39c21290, 33 },
    { 0xb9a21290, 33 },
    { 0x68021292, 55 },
    { 0x6be21291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a421298, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42129d, 100 },
    { 0x1ca2129d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6212a2, 76 },
    { 0, 0 },
    { 0x143212a4, 95 },
    { 0xe3e212a4, 23 },
    { 0x8ff212a6, 61 },
    { 0x832212a4, 55 },
    { 0x7b6212a8, 84 },
    { 0, 0 },
    { 0x807212aa, 61 },
    { 0xbe9212ab, 55 },
    { 0x192212aa, 98 },
    { 0x34c212ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2212b1, 95 },
    { 0xd9c212b1, 98 },
    { 0xb7a212b3, 23 },
    { 0xebb212b4, 95 },
    { 0x164212b5, 90 },
    { 0x79c212b6, 90 },
    { 0x8a9212b7, 33 },
    { 0xbd9212b7, 23 },
    { 0x9b1212b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6212be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248212c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c212c9, 90 },
    { 0, 0 },
    { 0xa16212cb, 61 },
    { 0xaa5212cc, 75 },
    { 0x1a5212cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e212d3, 34 },
    { 0, 0 },
    { 0x27e212d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x356212d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e212db, 76 },
    { 0x4c212dc, 91 },
    { 0x87a212dc, 90 },
    { 0x276212dc, 91 },
    { 0x17b212dc, 66 },
    { 0x619212dd, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2212ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6212f6, 95 },
    { 0xdbf212f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a212fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02212fd, 85 },
    { 0xa12212fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c21303, 99 },
    { 0xa8a21304, 87 },
    { 0, 0 },
    { 0x4fa21306, 23 },
    { 0xe6221307, 90 },
    { 0xa6221308, 66 },
    { 0x9e921309, 95 },
    { 0xea21309, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2130d, 61 },
    { 0, 0 },
    { 0x392130f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b221312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e21315, 95 },
    { 0x53021315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8721319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c21325, 33 },
    { 0xf4f21326, 90 },
    { 0, 0 },
    { 0xe2a21328, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932132c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21336, 99 },
    { 0, 0 },
    { 0xe321338, 46 },
    { 0, 0 },
    { 0x5a42133a, 55 },
    { 0x4c32133b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x822133e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28621341, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d21347, 95 },
    { 0x3421348, 52 },
    { 0x56d21348, 99 },
    { 0xfab21349, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d21352, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e21355, 98 },
    { 0, 0 },
    { 0x57921357, 93 },
    { 0, 0 },
    { 0xbd621359, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7952135d, 90 },
    { 0x85c2135e, 95 },
    { 0xf9b2135d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2421362, 33 },
    { 0xb0621363, 60 },
    { 0, 0 },
    { 0x39021365, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5821368, 75 },
    { 0, 0 },
    { 0xe4c2136a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5712136d, 52 },
    { 0x9682136d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93421375, 33 },
    { 0x77621375, 75 },
    { 0, 0 },
    { 0xd9921378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca2137b, 98 },
    { 0x6a32137b, 76 },
    { 0, 0 },
    { 0xda12137e, 23 },
    { 0x6572137f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd021383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20221388, 61 },
    { 0x12b21389, 61 },
    { 0, 0 },
    { 0x2112138b, 90 },
    { 0xb932138b, 93 },
    { 0xcdd2138d, 66 },
    { 0x5fb2138e, 95 },
    { 0xa622138f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e21395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b721398, 93 },
    { 0x97321398, 95 },
    { 0x1921398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104213a0, 23 },
    { 0, 0 },
    { 0xd17213a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4213a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4213ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690213af, 90 },
    { 0x95f213b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x823213b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3213b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d213bc, 40 },
    { 0, 0 },
    { 0xba6213be, 95 },
    { 0xd04213bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98213c2, 87 },
    { 0x8d2213c3, 93 },
    { 0xb72213c4, 68 },
    { 0, 0 },
    { 0x1e8213c6, 61 },
    { 0, 0 },
    { 0x5a1213c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f213cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae213ce, 90 },
    { 0x29b213cf, 61 },
    { 0, 0 },
    { 0xfdf213d1, 90 },
    { 0, 0 },
    { 0x29e213d3, 33 },
    { 0, 0 },
    { 0xe86213d5, 98 },
    { 0xcd3213d6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1213db, 34 },
    { 0x96f213dc, 90 },
    { 0, 0 },
    { 0x6c4213de, 61 },
    { 0, 0 },
    { 0x2b8213e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe213e9, 76 },
    { 0, 0 },
    { 0x284213eb, 33 },
    { 0x599213ec, 47 },
    { 0xeb8213eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x445213f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d213f3, 81 },
    { 0, 0 },
    { 0x27e213f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f213fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8221400, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2021408, 33 },
    { 0x7f21408, 61 },
    { 0xd3321408, 100 },
    { 0xec621408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93621412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2141c, 52 },
    { 0, 0 },
    { 0x6792141e, 40 },
    { 0, 0 },
    { 0xd7921420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c21429, 61 },
    { 0x3cf21429, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2142d, 61 },
    { 0x9752142e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf321431, 61 },
    { 0, 0 },
    { 0xbd721433, 98 },
    { 0x20821433, 34 },
    { 0, 0 },
    { 0xd7e21436, 43 },
    { 0, 0 },
    { 0x19621438, 33 },
    { 0, 0 },
    { 0xb162143a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2143f, 23 },
    { 0xfc72143f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94321443, 33 },
    { 0x73521443, 33 },
    { 0xd3321444, 34 },
    { 0, 0 },
    { 0xc8921447, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a42144d, 93 },
    { 0x7db2144e, 85 },
    { 0x9d32144f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b21452, 69 },
    { 0, 0 },
    { 0xfa821454, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x76021457, 95 },
    { 0x52521458, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2145e, 76 },
    { 0xba52145e, 33 },
    { 0, 0 },
    { 0x7eb21461, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd21468, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2146d, 61 },
    { 0x3952146d, 85 },
    { 0xc1b2146e, 99 },
    { 0xcd42146f, 98 },
    { 0, 0 },
    { 0x83f21472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c21477, 90 },
    { 0xfcb21478, 61 },
    { 0x6bc21478, 90 },
    { 0xe022147a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb472147f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d621483, 98 },
    { 0x9b21484, 77 },
    { 0x1c921484, 69 },
    { 0, 0 },
    { 0x6621487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372148c, 95 },
    { 0, 0 },
    { 0xd0a2148e, 95 },
    { 0x31f2148f, 98 },
    { 0xf392148e, 95 },
    { 0x60d21491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a21494, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66921498, 98 },
    { 0, 0 },
    { 0x47b2149a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b2149e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5214a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df214a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6214ac, 36 },
    { 0, 0 },
    { 0xa0b214ae, 55 },
    { 0, 0 },
    { 0x17b214b0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59214ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c214c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1214c3, 66 },
    { 0x5da214c3, 95 },
    { 0x59c214c5, 100 },
    { 0, 0 },
    { 0xb50214c7, 66 },
    { 0x53a214c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f214ce, 95 },
    { 0, 0 },
    { 0xc3a214d0, 52 },
    { 0xc20214d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca214d9, 33 },
    { 0x8bb214da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b214e0, 87 },
    { 0xa5e214e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1214e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84214ec, 78 },
    { 0x58214ed, 78 },
    { 0xd21214ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536214f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51214fa, 38 },
    { 0x832214fb, 78 },
    { 0xa9214fc, 98 },
    { 0, 0 },
    { 0x5a8214fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab721502, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a321506, 95 },
    { 0xfd721507, 56 },
    { 0xce821507, 91 },
    { 0xa4921509, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32150d, 61 },
    { 0x7f62150d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a621511, 23 },
    { 0xcf721512, 33 },
    { 0, 0 },
    { 0x64721514, 95 },
    { 0x46b21515, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea221518, 90 },
    { 0, 0 },
    { 0x3992151a, 23 },
    { 0, 0 },
    { 0x8002151c, 87 },
    { 0xde92151d, 76 },
    { 0, 0 },
    { 0x15e2151f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e21524, 61 },
    { 0x2af21525, 90 },
    { 0xc2721525, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86921529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e2152d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55721533, 90 },
    { 0x20321533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd21538, 33 },
    { 0x88721539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9462153c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3521540, 55 },
    { 0x64e21541, 98 },
    { 0x66d21541, 33 },
    { 0x80021543, 75 },
    { 0, 0 },
    { 0x1e21545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a21548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682154b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062154e, 61 },
    { 0, 0 },
    { 0x7bf21550, 99 },
    { 0x58f21551, 61 },
    { 0x82c21552, 61 },
    { 0x79321550, 4 },
    { 0xb8621554, 78 },
    { 0x3821555, 99 },
    { 0x45021551, 76 },
    { 0x30621557, 90 },
    { 0xc8a21558, 33 },
    { 0, 0 },
    { 0xf0e2155a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3552155f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b821563, 87 },
    { 0xb4521563, 100 },
    { 0, 0 },
    { 0xbec21566, 33 },
    { 0, 0 },
    { 0x87521568, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4782156b, 90 },
    { 0x4972156b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd21577, 99 },
    { 0x74b21578, 34 },
    { 0x49521579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a52157c, 33 },
    { 0x1ec2157c, 98 },
    { 0, 0 },
    { 0x9622157f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521582, 95 },
    { 0x22f21583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3372158a, 52 },
    { 0x90c2158b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2342158e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a721595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d21598, 87 },
    { 0x33c21598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981215a9, 93 },
    { 0, 0 },
    { 0xc27215ab, 91 },
    { 0x3c9215ab, 80 },
    { 0x44f215ab, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374215b4, 88 },
    { 0xc02215b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2215b9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99215bd, 64 },
    { 0x650215be, 53 },
    { 0x851215bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe215c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148215ca, 90 },
    { 0x6aa215cb, 33 },
    { 0xdbd215cc, 33 },
    { 0x2a1215cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd215d3, 34 },
    { 0x917215d4, 61 },
    { 0x8e5215d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49215d8, 81 },
    { 0xa1d215d9, 98 },
    { 0x4f5215d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a215e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96215e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33215e9, 23 },
    { 0, 0 },
    { 0xe3b215eb, 66 },
    { 0xf6f215ec, 68 },
    { 0, 0 },
    { 0xc9c215ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360215f7, 98 },
    { 0, 0 },
    { 0xd7e215f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518215fe, 61 },
    { 0x169215fe, 65 },
    { 0xa2b215ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca21603, 33 },
    { 0, 0 },
    { 0x13d21605, 76 },
    { 0, 0 },
    { 0x19221607, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf32160a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a2160e, 95 },
    { 0, 0 },
    { 0x57321610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d21617, 76 },
    { 0xc4d21618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722161b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd2161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b21622, 55 },
    { 0, 0 },
    { 0x10b21624, 33 },
    { 0xa8921625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a2162d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f821635, 93 },
    { 0xbab21635, 34 },
    { 0, 0 },
    { 0xebd21638, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee321641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a21647, 76 },
    { 0, 0 },
    { 0x7ec21649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5421650, 46 },
    { 0xd5e21651, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b421654, 61 },
    { 0x96021655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae2165b, 93 },
    { 0x2962165c, 90 },
    { 0xbbe2165d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf21660, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed21668, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb21674, 33 },
    { 0x3b521675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4021679, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d42167c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab21684, 90 },
    { 0x33c21684, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd621688, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812168d, 90 },
    { 0x9bd2168d, 33 },
    { 0x4292168f, 95 },
    { 0xf7c21690, 55 },
    { 0x7d621690, 76 },
    { 0x6ad21692, 98 },
    { 0xfc921693, 90 },
    { 0xb6821694, 95 },
    { 0xfb221694, 61 },
    { 0x23b21695, 33 },
    { 0x52121691, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd2169a, 34 },
    { 0x47e2169a, 55 },
    { 0, 0 },
    { 0x6642169d, 33 },
    { 0x76a2169d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33216a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f216a7, 33 },
    { 0xfab216a8, 98 },
    { 0, 0 },
    { 0x1d8216aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec216ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8216b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4216b6, 46 },
    { 0, 0 },
    { 0xa23216b8, 91 },
    { 0xbb1216b9, 91 },
    { 0x264216b9, 77 },
    { 0x654216bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe216bf, 90 },
    { 0x146216c0, 99 },
    { 0x217216c1, 76 },
    { 0x95b216c2, 61 },
    { 0, 0 },
    { 0xf07216c4, 61 },
    { 0xb78216c5, 55 },
    { 0xc78216c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3216c9, 95 },
    { 0x7bd216c9, 95 },
    { 0x7d2216cb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf216d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282216d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38216d9, 75 },
    { 0xa42216da, 95 },
    { 0x69d216d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6216de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb216e1, 43 },
    { 0xa40216e1, 95 },
    { 0x4e6216e3, 90 },
    { 0xeb8216e1, 43 },
    { 0xc6a216e5, 90 },
    { 0x17f216e4, 67 },
    { 0xe60216e7, 98 },
    { 0, 0 },
    { 0xdbe216e9, 95 },
    { 0x91e216ea, 17 },
    { 0, 0 },
    { 0xfbf216ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541216f2, 33 },
    { 0x676216f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0216f7, 88 },
    { 0x617216f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54216fb, 95 },
    { 0, 0 },
    { 0xf3e216fd, 77 },
    { 0, 0 },
    { 0xe9f216ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa21707, 33 },
    { 0x91e21708, 52 },
    { 0x86521708, 43 },
    { 0x6d21707, 43 },
    { 0xfd02170b, 33 },
    { 0x5cf2170b, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf21713, 55 },
    { 0, 0 },
    { 0x20f21715, 55 },
    { 0, 0 },
    { 0xff421717, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2732171b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e421720, 33 },
    { 0x9d821721, 55 },
    { 0xec721722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a21729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2172c, 98 },
    { 0x5d62172d, 90 },
    { 0x1712172d, 36 },
    { 0xa8c2172f, 84 },
    { 0x92721730, 61 },
    { 0xd0c21731, 36 },
    { 0xf5a21730, 88 },
    { 0x4bf21732, 61 },
    { 0x3e721730, 33 },
    { 0, 0 },
    { 0x82621736, 95 },
    { 0x2bd21737, 33 },
    { 0, 0 },
    { 0x99321739, 61 },
    { 0, 0 },
    { 0x23e2173b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2173e, 98 },
    { 0x79e2173e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c621742, 66 },
    { 0x80121742, 61 },
    { 0xed721744, 100 },
    { 0x88b21742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652174b, 88 },
    { 0x36e2174b, 91 },
    { 0xc032174d, 76 },
    { 0xa882174b, 5 },
    { 0x7942174e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5421756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9221759, 52 },
    { 0xbc12175a, 98 },
    { 0x8682175b, 91 },
    { 0x1932175c, 33 },
    { 0x9c2175d, 52 },
    { 0x25c2175c, 70 },
    { 0xb642175c, 90 },
    { 0xde21760, 23 },
    { 0xa0021761, 17 },
    { 0, 0 },
    { 0xfe021763, 91 },
    { 0x78021764, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a21767, 95 },
    { 0, 0 },
    { 0x86621769, 99 },
    { 0, 0 },
    { 0x2772176b, 90 },
    { 0xfea2176b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd21778, 33 },
    { 0xd4a21779, 75 },
    { 0, 0 },
    { 0x8842177b, 91 },
    { 0, 0 },
    { 0xc4f2177d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92021781, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d21785, 90 },
    { 0, 0 },
    { 0x24a21787, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42178b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de21792, 90 },
    { 0, 0 },
    { 0xb2a21794, 90 },
    { 0xe6421794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62179d, 52 },
    { 0xc932179e, 99 },
    { 0x9942179f, 17 },
    { 0x7aa217a0, 61 },
    { 0x7f3217a1, 61 },
    { 0xa49217a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b217ab, 76 },
    { 0xf0a217ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3217af, 98 },
    { 0xdfb217b0, 99 },
    { 0x555217b0, 93 },
    { 0x3a4217b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0217ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578217be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714217c2, 33 },
    { 0xf36217c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c217c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b217cb, 55 },
    { 0, 0 },
    { 0xc08217cd, 61 },
    { 0x5b8217ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e217d5, 23 },
    { 0, 0 },
    { 0xdf1217d7, 43 },
    { 0, 0 },
    { 0xa1c217d9, 66 },
    { 0xd6c217da, 61 },
    { 0x6d6217db, 67 },
    { 0xe39217dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67217e0, 90 },
    { 0, 0 },
    { 0x90217e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35217e5, 46 },
    { 0x123217e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e217e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd217f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa217f4, 95 },
    { 0x992217f5, 61 },
    { 0x61217f5, 61 },
    { 0x75a217f7, 90 },
    { 0x5d7217f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6217fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x741217fe, 90 },
    { 0x3d4217ff, 34 },
    { 0xb10217ff, 95 },
    { 0x1e021801, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91021806, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8922180b, 76 },
    { 0xefc2180c, 9 },
    { 0x5532180d, 98 },
    { 0xc702180e, 52 },
    { 0x2242180d, 46 },
    { 0x9372180f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7321813, 95 },
    { 0xa4121814, 33 },
    { 0xff221813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44721818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512181e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2021822, 95 },
    { 0xd9621822, 95 },
    { 0x60a21824, 78 },
    { 0x4fc21823, 76 },
    { 0, 0 },
    { 0x82021827, 95 },
    { 0xaf121828, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f21833, 90 },
    { 0, 0 },
    { 0x5f321835, 97 },
    { 0x82521836, 76 },
    { 0x8df21837, 34 },
    { 0x8fd21836, 100 },
    { 0x5e321839, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24821840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d421845, 52 },
    { 0xb7a21845, 68 },
    { 0xbd121847, 55 },
    { 0x29621848, 36 },
    { 0xb7d21848, 101 },
    { 0, 0 },
    { 0xa2b2184b, 61 },
    { 0xbda2184c, 95 },
    { 0, 0 },
    { 0x6b12184e, 55 },
    { 0x7fd2184f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34021853, 61 },
    { 0xce921853, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb21857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2185b, 84 },
    { 0, 0 },
    { 0xe242185d, 33 },
    { 0, 0 },
    { 0x2a92185f, 95 },
    { 0x24d2185f, 55 },
    { 0xcc421861, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56521866, 23 },
    { 0x36221867, 91 },
    { 0x5c321868, 90 },
    { 0x12c21869, 87 },
    { 0x45021869, 98 },
    { 0x1f921869, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f21872, 93 },
    { 0, 0 },
    { 0x9ea21874, 98 },
    { 0, 0 },
    { 0x2021876, 76 },
    { 0, 0 },
    { 0xbb521878, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb72187d, 84 },
    { 0xa662187e, 100 },
    { 0x6a82187e, 52 },
    { 0x22521880, 91 },
    { 0x83921880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d821885, 90 },
    { 0x4e821886, 90 },
    { 0xe1521886, 95 },
    { 0xec821885, 90 },
    { 0x3f821889, 76 },
    { 0xacb21888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2582188d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4121891, 98 },
    { 0, 0 },
    { 0xfc021893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed21896, 23 },
    { 0x81421897, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4192189a, 23 },
    { 0, 0 },
    { 0x52a2189c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e218a0, 61 },
    { 0, 0 },
    { 0xd0f218a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8218a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d218a8, 33 },
    { 0, 0 },
    { 0xb52218aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23218ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8218b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x38218b6, 33 },
    { 0, 0 },
    { 0xb56218b8, 98 },
    { 0x2db218b9, 77 },
    { 0x604218ba, 91 },
    { 0, 0 },
    { 0x4f1218bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8218c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f218c4, 52 },
    { 0xf4e218c4, 69 },
    { 0, 0 },
    { 0xfc5218c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b218d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561218d9, 98 },
    { 0xbe5218d9, 33 },
    { 0xdec218db, 68 },
    { 0x570218d9, 23 },
    { 0xd15218d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc218e1, 55 },
    { 0xe4d218e1, 61 },
    { 0, 0 },
    { 0x527218e4, 34 },
    { 0, 0 },
    { 0x68f218e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750218f0, 90 },
    { 0x326218f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e218f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f218f9, 98 },
    { 0x788218f9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153218fe, 95 },
    { 0, 0 },
    { 0x63921900, 77 },
    { 0x46221901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17921907, 46 },
    { 0xb6a21908, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2190d, 34 },
    { 0x5352190e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f621915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2191a, 98 },
    { 0xb152191b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f32191f, 93 },
    { 0xa9a21920, 99 },
    { 0x2172191f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f21926, 43 },
    { 0x3e821927, 61 },
    { 0, 0 },
    { 0x10921929, 88 },
    { 0, 0 },
    { 0x18d2192b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6121932, 61 },
    { 0x25021933, 46 },
    { 0xf8d21934, 32 },
    { 0xe3721934, 33 },
    { 0xe1a21936, 23 },
    { 0xe6a21937, 90 },
    { 0, 0 },
    { 0x41021939, 46 },
    { 0xa772193a, 61 },
    { 0xc432193a, 58 },
    { 0x2c32193c, 52 },
    { 0xa4c2193d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93321944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0221947, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6262194a, 61 },
    { 0, 0 },
    { 0x1452194c, 61 },
    { 0, 0 },
    { 0xf2c2194e, 33 },
    { 0, 0 },
    { 0x8b521950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53221954, 55 },
    { 0x8d621955, 61 },
    { 0x6a221954, 46 },
    { 0xd4f21954, 95 },
    { 0x37421958, 34 },
    { 0x1921959, 98 },
    { 0x6cc21956, 46 },
    { 0xe182195b, 85 },
    { 0x2fb2195c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2902195f, 61 },
    { 0x7c021960, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8221963, 99 },
    { 0, 0 },
    { 0xfb21965, 98 },
    { 0xfe21965, 98 },
    { 0x95f21965, 76 },
    { 0, 0 },
    { 0x2bb21969, 36 },
    { 0x6221969, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21972, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5121978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e2197c, 95 },
    { 0x7292197d, 95 },
    { 0, 0 },
    { 0xe612197f, 95 },
    { 0x61f21980, 91 },
    { 0x64e2197f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a821985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc02198a, 33 },
    { 0x47b2198b, 95 },
    { 0xdc52198c, 90 },
    { 0xd332198d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a221995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d21999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3332199c, 61 },
    { 0, 0 },
    { 0xc8f2199e, 88 },
    { 0xdf72199f, 90 },
    { 0x6e72199f, 46 },
    { 0x3e0219a1, 61 },
    { 0xf13219a1, 55 },
    { 0xf8b219a3, 52 },
    { 0xb2219a3, 98 },
    { 0x2b5219a5, 23 },
    { 0xfd0219a4, 95 },
    { 0, 0 },
    { 0xefa219a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e219b2, 33 },
    { 0x74a219b2, 84 },
    { 0xcad219b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a219b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8219bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6219c1, 55 },
    { 0x9ef219c2, 76 },
    { 0x3a2219c3, 93 },
    { 0, 0 },
    { 0xbd4219c5, 23 },
    { 0x335219c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x222219c9, 17 },
    { 0, 0 },
    { 0xd21219cb, 90 },
    { 0xca0219cb, 90 },
    { 0x20a219cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2219d4, 34 },
    { 0x585219d5, 84 },
    { 0, 0 },
    { 0xd01219d7, 61 },
    { 0xb8a219d8, 61 },
    { 0xfe6219d9, 55 },
    { 0, 0 },
    { 0x8f3219db, 90 },
    { 0, 0 },
    { 0x811219dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c219e0, 33 },
    { 0, 0 },
    { 0xff4219e2, 95 },
    { 0x547219e3, 99 },
    { 0x35d219e4, 64 },
    { 0, 0 },
    { 0x5fe219e6, 76 },
    { 0xb03219e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb219f0, 23 },
    { 0, 0 },
    { 0xd74219f2, 25 },
    { 0xd3b219f3, 91 },
    { 0x85219f4, 95 },
    { 0xe7a219f4, 30 },
    { 0, 0 },
    { 0x49c219f7, 88 },
    { 0xf4e219f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x206219fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e21a01, 83 },
    { 0, 0 },
    { 0x77821a03, 76 },
    { 0xc1521a03, 43 },
    { 0xf8a21a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea21a0c, 61 },
    { 0, 0 },
    { 0x3bd21a0e, 98 },
    { 0x94e21a0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb321a16, 61 },
    { 0xb5021a16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92121a1a, 33 },
    { 0x1ab21a1b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c21a1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6721a24, 91 },
    { 0x2621a25, 23 },
    { 0xa8621a26, 66 },
    { 0, 0 },
    { 0xfaa21a28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d21a2b, 55 },
    { 0xcfc21a2c, 80 },
    { 0x7f521a2d, 98 },
    { 0, 0 },
    { 0x43e21a2f, 98 },
    { 0x1e221a2f, 23 },
    { 0x8b921a30, 78 },
    { 0xb3621a30, 76 },
    { 0xd8a21a33, 33 },
    { 0, 0 },
    { 0x11821a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa21a38, 25 },
    { 0xd1621a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21a3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74921a42, 67 },
    { 0, 0 },
    { 0x62421a44, 33 },
    { 0x86f21a45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4521a4a, 55 },
    { 0x54321a4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf721a52, 95 },
    { 0, 0 },
    { 0xa3321a54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e121a5a, 93 },
    { 0, 0 },
    { 0x2f21a5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021a64, 46 },
    { 0, 0 },
    { 0x16621a66, 95 },
    { 0x81221a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17521a6b, 55 },
    { 0, 0 },
    { 0x1cb21a6d, 46 },
    { 0xb6121a6d, 90 },
    { 0xe6c21a6f, 68 },
    { 0xef521a6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb21a73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7721a76, 61 },
    { 0, 0 },
    { 0x67021a78, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd21a7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a21a7e, 90 },
    { 0, 0 },
    { 0x96621a80, 43 },
    { 0x98d21a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35121a8a, 43 },
    { 0x79621a8b, 98 },
    { 0, 0 },
    { 0x7ab21a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce21a93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48821a9f, 68 },
    { 0x4c321a9f, 81 },
    { 0xcd421aa1, 38 },
    { 0x33921a9f, 76 },
    { 0x5e621aa3, 84 },
    { 0x62521aa4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac21aa7, 33 },
    { 0x55a21aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed21aab, 33 },
    { 0x82c21aab, 33 },
    { 0, 0 },
    { 0x54321aae, 91 },
    { 0x29a21aaf, 23 },
    { 0x18921ab0, 52 },
    { 0x79121aaf, 65 },
    { 0xf0b21ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d321ab8, 97 },
    { 0x6f021ab9, 98 },
    { 0x70c21aba, 55 },
    { 0x65d21ab9, 36 },
    { 0x59121abc, 90 },
    { 0x8d921abd, 90 },
    { 0x21321abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc21ac1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31421ac5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9021ac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c21acb, 68 },
    { 0x3fb21acb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b21ad4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6321ad8, 90 },
    { 0xce721ad9, 43 },
    { 0x4eb21ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee21ade, 95 },
    { 0, 0 },
    { 0x34721ae0, 98 },
    { 0x3ae21ae1, 34 },
    { 0xf7921ae2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20221ae8, 95 },
    { 0, 0 },
    { 0x5821aea, 23 },
    { 0x60821aeb, 83 },
    { 0, 0 },
    { 0x5c921aed, 90 },
    { 0xe3721aed, 61 },
    { 0x45f21aed, 76 },
    { 0, 0 },
    { 0xcb221af1, 95 },
    { 0xb6f21af2, 52 },
    { 0, 0 },
    { 0xc3421af4, 81 },
    { 0xcb921af4, 53 },
    { 0x5da21af5, 90 },
    { 0xb3421af7, 91 },
    { 0x8f821af6, 90 },
    { 0, 0 },
    { 0xf3621afa, 42 },
    { 0x40f21afb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be21b00, 33 },
    { 0x77d21b01, 46 },
    { 0xb0421b00, 17 },
    { 0x10621b00, 93 },
    { 0, 0 },
    { 0xf5f21b05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d21b08, 61 },
    { 0x2a821b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca21b0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29021b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b21b20, 93 },
    { 0, 0 },
    { 0xc0421b22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8221b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24021b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf21b2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf321b30, 61 },
    { 0, 0 },
    { 0x55621b32, 90 },
    { 0xcfb21b33, 90 },
    { 0xfd521b34, 95 },
    { 0xf3921b35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b821b38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc21b3c, 23 },
    { 0, 0 },
    { 0xdd921b3e, 81 },
    { 0xc4521b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca321b42, 38 },
    { 0, 0 },
    { 0x9e121b44, 52 },
    { 0x9dd21b44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee21b49, 98 },
    { 0x98421b4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f21b4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78521b50, 95 },
    { 0xaa621b50, 98 },
    { 0xdcf21b52, 67 },
    { 0, 0 },
    { 0x9ab21b54, 93 },
    { 0xa9d21b54, 91 },
    { 0xccc21b56, 61 },
    { 0x8c321b57, 23 },
    { 0xb1921b56, 95 },
    { 0x32721b57, 95 },
    { 0x10921b5a, 98 },
    { 0, 0 },
    { 0xb0c21b5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f21b61, 33 },
    { 0, 0 },
    { 0xcca21b63, 23 },
    { 0x77521b64, 93 },
    { 0, 0 },
    { 0xf5721b66, 33 },
    { 0, 0 },
    { 0x2f321b68, 43 },
    { 0, 0 },
    { 0xaa621b6a, 93 },
    { 0, 0 },
    { 0x41821b6c, 61 },
    { 0, 0 },
    { 0x3b21b6e, 67 },
    { 0x44721b6f, 76 },
    { 0x5af21b70, 38 },
    { 0x8ba21b71, 90 },
    { 0x73821b72, 90 },
    { 0x42e21b72, 91 },
    { 0, 0 },
    { 0x2a421b75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e21b7a, 88 },
    { 0x99b21b7b, 90 },
    { 0xcdc21b7c, 33 },
    { 0x41721b7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87721b89, 100 },
    { 0x83021b8a, 58 },
    { 0x32921b8b, 76 },
    { 0x21b21b8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64021b8f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21b92, 90 },
    { 0xfb421b93, 93 },
    { 0x25421b94, 23 },
    { 0xd5321b95, 95 },
    { 0x19a21b93, 98 },
    { 0xe1821b97, 88 },
    { 0x6f421b98, 90 },
    { 0x54c21b97, 100 },
    { 0x67f21b98, 43 },
    { 0x9b521b98, 98 },
    { 0x79a21b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32321ba2, 33 },
    { 0x9321ba3, 90 },
    { 0x96d21ba2, 99 },
    { 0, 0 },
    { 0xeac21ba6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84821baa, 61 },
    { 0, 0 },
    { 0x3821bac, 90 },
    { 0xd5b21bac, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x37821bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a21bb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c21bbe, 52 },
    { 0xeb421bbe, 83 },
    { 0xa7821bc0, 56 },
    { 0, 0 },
    { 0x23b21bc2, 90 },
    { 0, 0 },
    { 0x28a21bc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f521bc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a21bcc, 98 },
    { 0x62621bcd, 23 },
    { 0xe5121bce, 100 },
    { 0, 0 },
    { 0x86221bd0, 70 },
    { 0x2621bd1, 61 },
    { 0xc0c21bd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1621bd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf321bdc, 77 },
    { 0xb2021bdd, 99 },
    { 0x4e521bdd, 99 },
    { 0x13321bdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df21be2, 23 },
    { 0xeb821be3, 95 },
    { 0, 0 },
    { 0xd4821be5, 98 },
    { 0, 0 },
    { 0x49121be7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6821beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a921bee, 95 },
    { 0, 0 },
    { 0x7d621bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79021bf5, 46 },
    { 0x97421bf6, 95 },
    { 0x3eb21bf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85721bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b21c02, 90 },
    { 0, 0 },
    { 0x9d521c04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72221c07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee21c0d, 33 },
    { 0, 0 },
    { 0x92721c0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6921c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2821c1c, 98 },
    { 0, 0 },
    { 0xe8021c1e, 61 },
    { 0x57521c1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78121c26, 33 },
    { 0, 0 },
    { 0x6d421c28, 87 },
    { 0x52421c29, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x28421c2c, 23 },
    { 0, 0 },
    { 0xed221c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d21c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69221c35, 34 },
    { 0, 0 },
    { 0xc0d21c37, 95 },
    { 0, 0 },
    { 0xbe921c39, 95 },
    { 0xf6621c3a, 40 },
    { 0xeb121c39, 76 },
    { 0xd5221c3c, 23 },
    { 0xdbd21c3a, 98 },
    { 0xde821c3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f021c41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a021c46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67121c4a, 23 },
    { 0, 0 },
    { 0xdee21c4c, 87 },
    { 0x9ff21c4d, 90 },
    { 0, 0 },
    { 0xdff21c4f, 61 },
    { 0x5f521c4f, 76 },
    { 0xffe21c51, 23 },
    { 0x53821c52, 55 },
    { 0x56d21c53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed221c5a, 96 },
    { 0x94a21c5b, 46 },
    { 0x64121c5c, 98 },
    { 0xabc21c5d, 98 },
    { 0x28e21c5d, 95 },
    { 0x93f21c5d, 67 },
    { 0xdee21c5e, 93 },
    { 0xb5821c61, 20 },
    { 0x5d521c62, 87 },
    { 0xd2921c63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d21c66, 43 },
    { 0x90921c67, 66 },
    { 0, 0 },
    { 0x6b521c69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac221c6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2121c70, 98 },
    { 0xb421c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9121c75, 76 },
    { 0xd1121c76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3021c7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43021c81, 67 },
    { 0, 0 },
    { 0x4c021c83, 33 },
    { 0, 0 },
    { 0xe121c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b21c8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49321c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf821c97, 43 },
    { 0x2ee21c97, 90 },
    { 0x82121c98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8421c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf821ca0, 95 },
    { 0, 0 },
    { 0xaef21ca2, 77 },
    { 0x6e021ca3, 97 },
    { 0xf2521ca3, 55 },
    { 0xeba21ca5, 23 },
    { 0xfc421ca5, 23 },
    { 0x60c21ca7, 98 },
    { 0x3fc21ca6, 33 },
    { 0x3e21ca9, 45 },
    { 0, 0 },
    { 0x51821cab, 87 },
    { 0x2b921cac, 98 },
    { 0, 0 },
    { 0xf2321cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c21cb2, 55 },
    { 0x85a21cb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f21cb6, 61 },
    { 0x3f421cb7, 90 },
    { 0x67b21cb8, 76 },
    { 0xfaa21cb8, 17 },
    { 0xc1321cb9, 33 },
    { 0x84721cb9, 98 },
    { 0x3b421cbc, 43 },
    { 0, 0 },
    { 0xc7621cbe, 56 },
    { 0x6f721cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee21cc2, 55 },
    { 0x42a21cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a21ccc, 90 },
    { 0x53321ccc, 90 },
    { 0x7321ccc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89721cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2721cda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28321ce3, 46 },
    { 0, 0 },
    { 0x30521ce5, 98 },
    { 0xe4921ce6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5121ce9, 61 },
    { 0xf5f21cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x57121ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d721cf0, 46 },
    { 0xe1021cf1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe21cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe21cf9, 90 },
    { 0xffa21cfa, 95 },
    { 0x42a21cfb, 33 },
    { 0, 0 },
    { 0xf3e21cfd, 61 },
    { 0x1a521cfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7521d02, 98 },
    { 0x921d03, 67 },
    { 0xa7921d04, 33 },
    { 0xeaf21d04, 91 },
    { 0xe6521d06, 23 },
    { 0x69b21d06, 33 },
    { 0x89521d08, 98 },
    { 0xa4a21d08, 85 },
    { 0, 0 },
    { 0x22421d0b, 17 },
    { 0x31e21d0c, 98 },
    { 0x8c921d0c, 98 },
    { 0xe6021d0e, 84 },
    { 0xff21d0f, 100 },
    { 0xdcc21d0e, 91 },
    { 0x68a21d0f, 33 },
    { 0x42921d12, 100 },
    { 0xe5a21d13, 67 },
    { 0x60a21d14, 34 },
    { 0, 0 },
    { 0xe0b21d16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c21d19, 99 },
    { 0x3a821d19, 15 },
    { 0xded21d19, 90 },
    { 0xacc21d1c, 90 },
    { 0x58a21d1d, 33 },
    { 0x1d21d1c, 100 },
    { 0x76121d1d, 33 },
    { 0xf6021d20, 43 },
    { 0x6ea21d1b, 90 },
    { 0, 0 },
    { 0x41821d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57621d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f21d32, 77 },
    { 0x3ed21d33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59521d37, 33 },
    { 0xf0f21d37, 95 },
    { 0x26721d38, 95 },
    { 0, 0 },
    { 0x3d621d3b, 34 },
    { 0x92b21d3c, 91 },
    { 0, 0 },
    { 0xf6f21d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76121d41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d21d4b, 93 },
    { 0xebf21d4c, 33 },
    { 0x76321d4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd221d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc21d56, 98 },
    { 0xdda21d57, 23 },
    { 0xb3e21d57, 33 },
    { 0xe2321d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c721d5e, 34 },
    { 0x24b21d5f, 90 },
    { 0x86121d5f, 34 },
    { 0x84321d61, 33 },
    { 0x24221d61, 87 },
    { 0xf1421d5f, 34 },
    { 0x1a521d64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b21d69, 61 },
    { 0, 0 },
    { 0x6d221d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e21d71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51421d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5221d7d, 99 },
    { 0xaec21d7e, 84 },
    { 0x93821d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b21d82, 52 },
    { 0x49421d82, 99 },
    { 0x1dd21d84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b21d87, 95 },
    { 0, 0 },
    { 0x73e21d89, 99 },
    { 0, 0 },
    { 0x58e21d8b, 43 },
    { 0x5ea21d8c, 33 },
    { 0x96621d8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80321d92, 76 },
    { 0x79521d92, 55 },
    { 0, 0 },
    { 0xaf121d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa521d9d, 56 },
    { 0xe1021d9e, 33 },
    { 0x88c21d9f, 33 },
    { 0, 0 },
    { 0xe2221da1, 98 },
    { 0x3df21da1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15021da5, 95 },
    { 0x2c321da5, 95 },
    { 0, 0 },
    { 0x6c421da8, 55 },
    { 0xff021da9, 90 },
    { 0x72721da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d21db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33121db4, 34 },
    { 0x5b21db5, 87 },
    { 0, 0 },
    { 0xe5121db7, 23 },
    { 0, 0 },
    { 0x12621db9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21dbc, 46 },
    { 0xf8821dbd, 33 },
    { 0x94621dbd, 43 },
    { 0x10921dbf, 15 },
    { 0, 0 },
    { 0x3b721dc1, 68 },
    { 0x14721dc1, 95 },
    { 0, 0 },
    { 0x2c821dc4, 34 },
    { 0x55621dc4, 90 },
    { 0x25e21dc6, 33 },
    { 0, 0 },
    { 0xb21dc8, 98 },
    { 0x58d21dc8, 75 },
    { 0xfa421dca, 93 },
    { 0, 0 },
    { 0x66d21dcc, 23 },
    { 0x58421dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf21dd0, 61 },
    { 0, 0 },
    { 0x24721dd2, 52 },
    { 0x29121dd2, 23 },
    { 0x78121dd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c721dd8, 82 },
    { 0x46f21dd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55921de1, 90 },
    { 0x18921de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f221de5, 52 },
    { 0xf021de6, 66 },
    { 0, 0 },
    { 0x79921de8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c721deb, 95 },
    { 0x3ec21dec, 61 },
    { 0x17821ded, 76 },
    { 0x80b21deb, 90 },
    { 0x27821def, 66 },
    { 0, 0 },
    { 0xbd21df1, 91 },
    { 0x6ab21df2, 95 },
    { 0, 0 },
    { 0x4da21df4, 33 },
    { 0x2ed21df4, 46 },
    { 0xb7f21df4, 90 },
    { 0, 0 },
    { 0x18521df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e21dfc, 33 },
    { 0, 0 },
    { 0x1bd21dfe, 33 },
    { 0x26b21dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef21e06, 80 },
    { 0x32521e07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66921e0f, 90 },
    { 0, 0 },
    { 0x55a21e11, 91 },
    { 0, 0 },
    { 0x7b621e13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f21e19, 33 },
    { 0x30721e19, 90 },
    { 0x9c21e1b, 67 },
    { 0x20a21e1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf721e1f, 90 },
    { 0xd3421e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b21e25, 91 },
    { 0xda021e26, 33 },
    { 0x43821e27, 85 },
    { 0xf721e25, 76 },
    { 0x2c921e28, 88 },
    { 0, 0 },
    { 0x5a821e2b, 91 },
    { 0xa6621e2c, 95 },
    { 0x61b21e2d, 23 },
    { 0, 0 },
    { 0xe6921e2f, 23 },
    { 0xed921e30, 33 },
    { 0x9f321e31, 33 },
    { 0xbb821e32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2321e37, 80 },
    { 0xdef21e38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d21e40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4621e45, 90 },
    { 0xe8c21e46, 93 },
    { 0x12321e46, 34 },
    { 0, 0 },
    { 0xcab21e49, 61 },
    { 0xb2021e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13421e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c121e53, 90 },
    { 0x6821e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21e57, 90 },
    { 0x81121e58, 52 },
    { 0xa8821e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39621e5f, 23 },
    { 0x61921e60, 76 },
    { 0x56921e60, 76 },
    { 0x93821e60, 23 },
    { 0xfa821e63, 95 },
    { 0x9b821e63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb821e6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf421e70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d21e77, 17 },
    { 0x51821e78, 99 },
    { 0x38721e79, 61 },
    { 0x28021e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd21e7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf421e83, 61 },
    { 0xd2f21e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c21e87, 76 },
    { 0, 0 },
    { 0xed321e89, 64 },
    { 0, 0 },
    { 0xd0e21e8b, 90 },
    { 0xc3921e8c, 90 },
    { 0, 0 },
    { 0xf1e21e8e, 85 },
    { 0x42721e8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff321e93, 91 },
    { 0xe9e21e94, 33 },
    { 0x6da21e94, 95 },
    { 0x7a21e93, 61 },
    { 0xf4821e97, 43 },
    { 0x6cd21e98, 55 },
    { 0xe9721e98, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f21ea2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f21eaa, 98 },
    { 0x7821eab, 33 },
    { 0xb621eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11821eaf, 23 },
    { 0, 0 },
    { 0xd2c21eb1, 34 },
    { 0x4fe21eb2, 61 },
    { 0x3b321eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba21eb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a21ebf, 90 },
    { 0xbbe21ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc21ec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e221ecb, 33 },
    { 0x32321ecb, 88 },
    { 0x8bf21ecb, 90 },
    { 0x66421ece, 90 },
    { 0xfde21ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa21ed4, 69 },
    { 0, 0 },
    { 0x80521ed6, 90 },
    { 0xfe821ed7, 95 },
    { 0xbac21ed7, 84 },
    { 0x4321ed9, 90 },
    { 0x2ab21eda, 98 },
    { 0x421ed9, 95 },
    { 0x64f21ed6, 99 },
    { 0xed621edd, 55 },
    { 0x85a21ede, 95 },
    { 0x78921ede, 61 },
    { 0x2aa21ee0, 52 },
    { 0x59421ee1, 68 },
    { 0xf5c21ee0, 90 },
    { 0, 0 },
    { 0x54021ee4, 23 },
    { 0xcfb21ee5, 23 },
    { 0xbca21ee4, 76 },
    { 0x92921ee7, 95 },
    { 0xa7021ee8, 66 },
    { 0x3f121ee9, 78 },
    { 0x70721ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521eee, 23 },
    { 0xc2921eef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39921ef3, 95 },
    { 0, 0 },
    { 0x52821ef5, 55 },
    { 0, 0 },
    { 0x57221ef7, 98 },
    { 0x37221ef8, 34 },
    { 0xad021ef9, 53 },
    { 0x6eb21ef7, 43 },
    { 0x23b21ef8, 93 },
    { 0, 0 },
    { 0xdda21efd, 94 },
    { 0x88e21efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b21f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61721f07, 98 },
    { 0x91a21f08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7021f0d, 76 },
    { 0, 0 },
    { 0x82621f0f, 55 },
    { 0, 0 },
    { 0x33821f11, 95 },
    { 0, 0 },
    { 0x73621f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da21f17, 83 },
    { 0x8e021f18, 69 },
    { 0xaa021f19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c121f20, 99 },
    { 0x5e421f21, 96 },
    { 0xc6621f22, 94 },
    { 0xcdc21f23, 85 },
    { 0xd7421f24, 23 },
    { 0, 0 },
    { 0x5e321f26, 61 },
    { 0x68c21f26, 43 },
    { 0x73a21f28, 88 },
    { 0xc21f27, 78 },
    { 0, 0 },
    { 0xb5421f2b, 87 },
    { 0x43021f2b, 98 },
    { 0x1cd21f2c, 75 },
    { 0x5121f2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8021f32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a21f3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e21f42, 33 },
    { 0, 0 },
    { 0xea721f44, 33 },
    { 0x26521f45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87121f4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a21f51, 90 },
    { 0x28521f52, 76 },
    { 0x4ab21f53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37621f56, 33 },
    { 0x57921f57, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c21f5b, 61 },
    { 0x27d21f5c, 2 },
    { 0xff021f5c, 66 },
    { 0x5ab21f5c, 90 },
    { 0, 0 },
    { 0x77c21f60, 66 },
    { 0, 0 },
    { 0xe7321f62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93421f69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96821f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a21f72, 55 },
    { 0, 0 },
    { 0xbaf21f74, 66 },
    { 0x77f21f74, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d21f7a, 98 },
    { 0, 0 },
    { 0xdb621f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c21f81, 98 },
    { 0, 0 },
    { 0x67121f83, 93 },
    { 0, 0 },
    { 0xe1421f85, 34 },
    { 0xb1c21f86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95321f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e21f97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a821f9b, 85 },
    { 0, 0 },
    { 0x5b821f9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9021fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da21faa, 98 },
    { 0xae621faa, 93 },
    { 0xb9121fac, 77 },
    { 0xd0821fac, 66 },
    { 0, 0 },
    { 0x57a21faf, 83 },
    { 0x88321faf, 90 },
    { 0xd9721fb1, 61 },
    { 0xd9421faf, 76 },
    { 0xeb721fb3, 33 },
    { 0, 0 },
    { 0x16c21fb5, 90 },
    { 0, 0 },
    { 0xc8021fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4221fbc, 78 },
    { 0, 0 },
    { 0xa2221fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e21fc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121fcc, 90 },
    { 0x98421fcd, 23 },
    { 0xb8521fcd, 61 },
    { 0xd6721fce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81021fd2, 77 },
    { 0x5e21fd3, 99 },
    { 0x4a021fd2, 93 },
    { 0x14421fd3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f721fd9, 33 },
    { 0xec21fda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c21fde, 98 },
    { 0xed721fde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d21fe2, 87 },
    { 0xac721fe3, 76 },
    { 0x8dc21fe4, 77 },
    { 0x9f221fe3, 43 },
    { 0x3aa21fe4, 90 },
    { 0xbad21fe7, 46 },
    { 0x5e621fe8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef21feb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f021fef, 91 },
    { 0, 0 },
    { 0x12621ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4721ff4, 76 },
    { 0, 0 },
    { 0x68f21ff6, 61 },
    { 0x74421ff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf21ffe, 90 },
    { 0x62321ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f22003, 96 },
    { 0xc5522004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35922008, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822200b, 61 },
    { 0xbef2200c, 23 },
    { 0x6ee2200d, 93 },
    { 0x9f2200c, 46 },
    { 0xd8f2200f, 23 },
    { 0x58f22010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16022013, 98 },
    { 0x28d22014, 90 },
    { 0xd4822015, 78 },
    { 0x8ae22016, 85 },
    { 0, 0 },
    { 0x9822018, 98 },
    { 0, 0 },
    { 0x4a72201a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf12201d, 61 },
    { 0x732201e, 38 },
    { 0, 0 },
    { 0x66522020, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe22027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c2202a, 95 },
    { 0xe032202b, 33 },
    { 0xb172202b, 99 },
    { 0xb862202b, 90 },
    { 0x2952202b, 93 },
    { 0xe922202e, 55 },
    { 0xf042202a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d22033, 43 },
    { 0, 0 },
    { 0x3a122035, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e22038, 17 },
    { 0xab922039, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b42203c, 98 },
    { 0xf832203d, 95 },
    { 0x71b2203c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e522041, 91 },
    { 0x49622042, 95 },
    { 0x91022043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c2204b, 90 },
    { 0x3cc2204b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2204f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d22052, 95 },
    { 0x322053, 65 },
    { 0xb8e22054, 76 },
    { 0, 0 },
    { 0x34c22056, 98 },
    { 0x5af22056, 33 },
    { 0x70b22058, 46 },
    { 0xa2022058, 34 },
    { 0, 0 },
    { 0xb6f2205b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9322205e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f822061, 84 },
    { 0xd22061, 34 },
    { 0, 0 },
    { 0x83622064, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3622069, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe22070, 85 },
    { 0x66222071, 88 },
    { 0, 0 },
    { 0xbd222073, 76 },
    { 0, 0 },
    { 0x69c22075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e2207e, 90 },
    { 0, 0 },
    { 0xcbd22080, 61 },
    { 0, 0 },
    { 0xe3022082, 46 },
    { 0, 0 },
    { 0x90522084, 90 },
    { 0, 0 },
    { 0x53222086, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6022090, 76 },
    { 0xa3e22091, 55 },
    { 0xe6d22091, 75 },
    { 0xb5722093, 90 },
    { 0, 0 },
    { 0x4dc22095, 46 },
    { 0x69622096, 60 },
    { 0x15122097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b72209a, 67 },
    { 0x7f32209b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637220a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a220a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a220aa, 90 },
    { 0x9ed220ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2220af, 55 },
    { 0x5b220af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6220b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b220ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b220bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x378220c0, 98 },
    { 0xbed220c0, 33 },
    { 0x657220c2, 90 },
    { 0, 0 },
    { 0xe0f220c4, 93 },
    { 0xf1a220c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7220cb, 91 },
    { 0x277220cc, 61 },
    { 0xfbe220cd, 33 },
    { 0x56b220cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x327220d1, 98 },
    { 0x18b220d2, 85 },
    { 0x2ae220d3, 23 },
    { 0xf86220d1, 77 },
    { 0x421220d3, 95 },
    { 0x65b220d6, 33 },
    { 0x506220d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x456220da, 34 },
    { 0, 0 },
    { 0x241220dc, 61 },
    { 0x9c6220dd, 23 },
    { 0xe2f220dd, 84 },
    { 0, 0 },
    { 0x442220e0, 46 },
    { 0x9a7220e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27220e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4220ec, 33 },
    { 0, 0 },
    { 0x97a220ee, 90 },
    { 0, 0 },
    { 0x5d0220f0, 98 },
    { 0, 0 },
    { 0x4cc220f2, 99 },
    { 0, 0 },
    { 0x424220f4, 97 },
    { 0, 0 },
    { 0xa0f220f6, 98 },
    { 0x68d220f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487220fb, 76 },
    { 0xcf7220fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x188220ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb22107, 43 },
    { 0xfab22107, 91 },
    { 0, 0 },
    { 0x2a2210a, 33 },
    { 0xd7d2210b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2210f, 55 },
    { 0x65822110, 33 },
    { 0x9022111, 61 },
    { 0, 0 },
    { 0xe9722113, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b222116, 90 },
    { 0xc6a22117, 98 },
    { 0x98b22116, 90 },
    { 0x71122119, 23 },
    { 0x79e2211a, 95 },
    { 0, 0 },
    { 0xef72211c, 95 },
    { 0, 0 },
    { 0xdd02211e, 61 },
    { 0, 0 },
    { 0x58d22120, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb22128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae2212b, 90 },
    { 0xdac2212b, 90 },
    { 0, 0 },
    { 0x2a32212e, 90 },
    { 0xcc92212e, 90 },
    { 0, 0 },
    { 0x27d22131, 95 },
    { 0, 0 },
    { 0x4a822133, 33 },
    { 0, 0 },
    { 0xd0522135, 76 },
    { 0, 0 },
    { 0xfee22137, 55 },
    { 0, 0 },
    { 0x61b22139, 23 },
    { 0x4572213a, 90 },
    { 0xb9c2213a, 90 },
    { 0x62e2213c, 61 },
    { 0xf3e2213c, 90 },
    { 0x6f022139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd22142, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18822146, 33 },
    { 0xac922147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a2214b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21922152, 23 },
    { 0xff122153, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52322156, 90 },
    { 0x34222156, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22215e, 55 },
    { 0xbb72215f, 90 },
    { 0x9ee2215f, 55 },
    { 0xe2422160, 55 },
    { 0x6ce22160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56422165, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde422169, 33 },
    { 0, 0 },
    { 0x2512216b, 33 },
    { 0, 0 },
    { 0xefd2216d, 76 },
    { 0x9102216e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62722173, 90 },
    { 0x99f22173, 23 },
    { 0xa0522175, 76 },
    { 0xf8b22176, 90 },
    { 0xd3322176, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf72217c, 33 },
    { 0x26b2217c, 61 },
    { 0x12c2217e, 95 },
    { 0xe952217f, 61 },
    { 0x9ee22180, 98 },
    { 0x1382217d, 34 },
    { 0x22322182, 76 },
    { 0xb2f22183, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde922187, 89 },
    { 0x86722188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1072218b, 90 },
    { 0xf002218c, 33 },
    { 0, 0 },
    { 0x2022218e, 90 },
    { 0, 0 },
    { 0xd9422190, 90 },
    { 0x8a222190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88222195, 76 },
    { 0x6e22196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc452219c, 68 },
    { 0x772219d, 84 },
    { 0x9752219d, 34 },
    { 0x2872219f, 23 },
    { 0x2219f, 33 },
    { 0x66d221a1, 61 },
    { 0x3842219d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a221a5, 95 },
    { 0x8ed221a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d221aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8221ad, 98 },
    { 0, 0 },
    { 0xef4221af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe221b2, 91 },
    { 0x917221b3, 55 },
    { 0xb66221b2, 90 },
    { 0, 0 },
    { 0xac3221b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c221b9, 46 },
    { 0x2f6221ba, 36 },
    { 0, 0 },
    { 0x5ba221bc, 66 },
    { 0x25c221bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x431221c0, 84 },
    { 0, 0 },
    { 0xb2e221c2, 23 },
    { 0xc58221c3, 55 },
    { 0xb37221c4, 33 },
    { 0xb74221c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a221c8, 55 },
    { 0x7a5221c9, 17 },
    { 0xcb6221c9, 52 },
    { 0, 0 },
    { 0x468221cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b221d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e221d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1221e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab221e7, 76 },
    { 0xb96221e8, 95 },
    { 0, 0 },
    { 0x7a5221ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e221ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579221f5, 99 },
    { 0x980221f5, 93 },
    { 0xa51221f7, 90 },
    { 0, 0 },
    { 0x5ad221f9, 95 },
    { 0, 0 },
    { 0x1f6221fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58122205, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x47922208, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce52220d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35622213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352221c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e22221, 95 },
    { 0, 0 },
    { 0x27822223, 68 },
    { 0x12e22224, 33 },
    { 0x5b622224, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b22228, 43 },
    { 0xfd822229, 55 },
    { 0xff72222a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2222d, 61 },
    { 0, 0 },
    { 0x5c42222f, 61 },
    { 0x16722230, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b22234, 33 },
    { 0x5c822234, 90 },
    { 0xda022236, 90 },
    { 0x13922237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7462223f, 52 },
    { 0x73f22240, 33 },
    { 0xa2b22240, 95 },
    { 0x4c02223f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c022247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5792224b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b22250, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42822253, 78 },
    { 0, 0 },
    { 0xd3422255, 23 },
    { 0x8b622255, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd052225a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a22262, 34 },
    { 0x16e22263, 76 },
    { 0x57d22263, 76 },
    { 0x42722262, 61 },
    { 0x18622266, 98 },
    { 0x36222266, 91 },
    { 0xfdd22267, 87 },
    { 0xa2422269, 95 },
    { 0, 0 },
    { 0x9602226b, 90 },
    { 0x6062226c, 91 },
    { 0, 0 },
    { 0x7712226e, 95 },
    { 0x622226e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d222272, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb222276, 66 },
    { 0x1cf22276, 65 },
    { 0xa4d22276, 33 },
    { 0xeee22279, 52 },
    { 0x3702227a, 23 },
    { 0xa132227b, 55 },
    { 0xe7f2227b, 61 },
    { 0, 0 },
    { 0xc582227e, 48 },
    { 0x3932227f, 84 },
    { 0, 0 },
    { 0x71c22281, 99 },
    { 0, 0 },
    { 0x4422283, 23 },
    { 0x90b22284, 90 },
    { 0, 0 },
    { 0xcea22286, 33 },
    { 0x41922287, 90 },
    { 0x3f622287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2732228b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb2228e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40122291, 36 },
    { 0x6e822292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad2229b, 76 },
    { 0xa552229c, 43 },
    { 0xb922229b, 33 },
    { 0x1a02229b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45222a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x238222a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406222ae, 43 },
    { 0, 0 },
    { 0xf9c222b0, 90 },
    { 0x142222b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb222b6, 90 },
    { 0x946222b7, 33 },
    { 0x2ea222b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59222bf, 23 },
    { 0x2222c0, 52 },
    { 0xd6c222bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11222c4, 78 },
    { 0, 0 },
    { 0x6de222c6, 23 },
    { 0, 0 },
    { 0xd62222c8, 90 },
    { 0, 0 },
    { 0x943222ca, 61 },
    { 0x4c6222cb, 55 },
    { 0xf63222cc, 76 },
    { 0xb81222cd, 61 },
    { 0, 0 },
    { 0x18f222cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x757222d2, 61 },
    { 0x47b222d3, 42 },
    { 0xdb5222d4, 100 },
    { 0x376222d5, 55 },
    { 0x24222d6, 33 },
    { 0xbda222d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3222df, 61 },
    { 0xaf0222e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68222e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206222e8, 98 },
    { 0x466222e9, 55 },
    { 0x651222e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526222f0, 55 },
    { 0, 0 },
    { 0xdce222f2, 23 },
    { 0x56d222f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98222f6, 90 },
    { 0, 0 },
    { 0x857222f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58222fe, 90 },
    { 0xc9c222ff, 93 },
    { 0, 0 },
    { 0xb8722301, 69 },
    { 0x59722302, 61 },
    { 0x42922303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2230c, 33 },
    { 0xba2230d, 78 },
    { 0x4262230d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d22311, 76 },
    { 0, 0 },
    { 0x26c22313, 43 },
    { 0, 0 },
    { 0xf1c22315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8502231b, 33 },
    { 0x5c52231c, 90 },
    { 0x7cd2231d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f22322, 98 },
    { 0, 0 },
    { 0x26822324, 48 },
    { 0xe2d22325, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3022328, 91 },
    { 0xafb22328, 55 },
    { 0, 0 },
    { 0xbb02232b, 95 },
    { 0x33b2232b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef22334, 34 },
    { 0x21222334, 61 },
    { 0xfe722335, 90 },
    { 0xef422337, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2233e, 58 },
    { 0xea12233f, 61 },
    { 0xd1f22340, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf22345, 98 },
    { 0x2a222346, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52234a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f22350, 67 },
    { 0, 0 },
    { 0xa3622352, 46 },
    { 0, 0 },
    { 0x3a322354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78622358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2235f, 95 },
    { 0x2e52235f, 90 },
    { 0, 0 },
    { 0x8b022362, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f52236a, 95 },
    { 0x47b2236a, 100 },
    { 0x14f2236b, 33 },
    { 0x3f2236d, 55 },
    { 0x58e2236e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b22371, 46 },
    { 0x7be22372, 61 },
    { 0xd5b22372, 95 },
    { 0x6722373, 43 },
    { 0, 0 },
    { 0x9e422376, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8122237a, 76 },
    { 0, 0 },
    { 0x6a92237c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5802237f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b22387, 90 },
    { 0x39522388, 90 },
    { 0xf122387, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62238c, 38 },
    { 0x9c62238d, 46 },
    { 0, 0 },
    { 0x63e2238f, 91 },
    { 0x10922390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a22394, 34 },
    { 0, 0 },
    { 0x17522396, 98 },
    { 0x86b22397, 95 },
    { 0xe1322398, 98 },
    { 0xac422399, 33 },
    { 0x3652239a, 34 },
    { 0, 0 },
    { 0xe3c2239c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2239f, 98 },
    { 0x4f62239f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d223a3, 23 },
    { 0xf2223a4, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69223a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff223ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x929223b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a223b4, 55 },
    { 0x716223b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865223b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x360223bc, 33 },
    { 0, 0 },
    { 0xb91223be, 84 },
    { 0xe11223bf, 77 },
    { 0x9ed223be, 33 },
    { 0xe5a223be, 101 },
    { 0xa3b223c2, 98 },
    { 0x4e4223c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881223cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f223d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd223d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d223db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85223f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x504223f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797223f7, 23 },
    { 0x19b223f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1223fd, 23 },
    { 0xae7223fe, 66 },
    { 0xdea223ff, 100 },
    { 0xe7222400, 75 },
    { 0xe4222400, 75 },
    { 0xfc222402, 90 },
    { 0x20922403, 66 },
    { 0x84522403, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd22408, 95 },
    { 0x23222408, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf22411, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23822417, 46 },
    { 0xbf222418, 33 },
    { 0xa9722419, 61 },
    { 0x80722417, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3802241e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752242a, 61 },
    { 0, 0 },
    { 0x5162242c, 52 },
    { 0x7392242d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e22431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f22434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95022438, 90 },
    { 0xaab22439, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2243d, 98 },
    { 0, 0 },
    { 0x29b2243f, 76 },
    { 0xbac22440, 34 },
    { 0, 0 },
    { 0xb0522442, 84 },
    { 0, 0 },
    { 0x98422444, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94222448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642244d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac022455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71422458, 33 },
    { 0x39322458, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2245e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1922462, 98 },
    { 0xc9622463, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3722468, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92246d, 76 },
    { 0x2892246e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60822472, 91 },
    { 0, 0 },
    { 0x99322474, 87 },
    { 0x76522475, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92247a, 33 },
    { 0, 0 },
    { 0xb032247c, 84 },
    { 0xc242247c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa22480, 61 },
    { 0, 0 },
    { 0xc0522482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf222486, 90 },
    { 0x65922487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e32248a, 66 },
    { 0x3002248b, 90 },
    { 0x1792248c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62248f, 33 },
    { 0x3912248f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222493, 33 },
    { 0xf022494, 61 },
    { 0x46322494, 91 },
    { 0x19922496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352249a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602249e, 90 },
    { 0, 0 },
    { 0x509224a0, 55 },
    { 0xe3e224a0, 76 },
    { 0, 0 },
    { 0x291224a3, 33 },
    { 0, 0 },
    { 0x947224a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636224aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83224ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82224b1, 34 },
    { 0, 0 },
    { 0xdbc224b3, 88 },
    { 0x3c6224b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678224b8, 84 },
    { 0xe3c224b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea224ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc224cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x525224d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f224d8, 83 },
    { 0, 0 },
    { 0x270224da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c224dd, 85 },
    { 0, 0 },
    { 0x24d224df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c224e5, 33 },
    { 0, 0 },
    { 0x6bc224e7, 61 },
    { 0x50e224e8, 61 },
    { 0x612224e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1224ec, 76 },
    { 0xe2a224ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x949224f0, 34 },
    { 0x272224f0, 34 },
    { 0x5e3224f2, 52 },
    { 0x752224f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f224f6, 93 },
    { 0, 0 },
    { 0xffb224f8, 55 },
    { 0xfee224f9, 90 },
    { 0x919224f9, 61 },
    { 0x615224fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff422500, 95 },
    { 0xb2e22500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b22505, 76 },
    { 0x18422506, 23 },
    { 0x2d222507, 23 },
    { 0x29322507, 43 },
    { 0x38122509, 90 },
    { 0xba52250a, 90 },
    { 0x3a22250b, 65 },
    { 0x4372250a, 23 },
    { 0xd5f22508, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3122511, 36 },
    { 0, 0 },
    { 0x86a22513, 91 },
    { 0x17b22514, 43 },
    { 0, 0 },
    { 0x21c22516, 61 },
    { 0, 0 },
    { 0xd1e22518, 93 },
    { 0x6b822519, 95 },
    { 0x6362251a, 76 },
    { 0x67a2251b, 43 },
    { 0xbaa2251c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6422521, 61 },
    { 0, 0 },
    { 0x50022523, 87 },
    { 0, 0 },
    { 0x4622525, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77222529, 88 },
    { 0x83f2252a, 76 },
    { 0, 0 },
    { 0x9fb2252c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f32252f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40622532, 98 },
    { 0x5d122532, 85 },
    { 0x7422534, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f22538, 100 },
    { 0, 0 },
    { 0x1d82253a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802253d, 43 },
    { 0x7de2253e, 61 },
    { 0, 0 },
    { 0xcd922540, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d022547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c2254a, 52 },
    { 0xce02254b, 55 },
    { 0xee62254c, 23 },
    { 0x7b02254d, 98 },
    { 0x3322254c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9622557, 90 },
    { 0, 0 },
    { 0x2d822559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd022564, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442256e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb2257b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e22581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec22587, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd82258a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb2258d, 93 },
    { 0x5fe2258d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f522591, 76 },
    { 0x38422591, 95 },
    { 0x66c22593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34422596, 17 },
    { 0, 0 },
    { 0x93622598, 33 },
    { 0x77222599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f2259e, 66 },
    { 0, 0 },
    { 0x5fa225a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5225a8, 55 },
    { 0, 0 },
    { 0x629225aa, 96 },
    { 0xd56225ab, 78 },
    { 0x48e225ab, 61 },
    { 0x83e225ab, 95 },
    { 0x53a225ad, 91 },
    { 0x663225ab, 43 },
    { 0, 0 },
    { 0xbf7225b1, 67 },
    { 0, 0 },
    { 0xe35225b3, 76 },
    { 0xd225b3, 69 },
    { 0x26a225b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3225b8, 84 },
    { 0x3b225b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d225bd, 91 },
    { 0x174225be, 87 },
    { 0x6ac225bf, 46 },
    { 0xc89225be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839225ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9225ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e225d2, 98 },
    { 0, 0 },
    { 0x5b0225d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9225db, 43 },
    { 0xa35225dc, 61 },
    { 0xfd7225dd, 55 },
    { 0, 0 },
    { 0x38e225df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f225e8, 52 },
    { 0x956225e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef225ed, 98 },
    { 0x6c5225ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30225f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d225f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b225fc, 87 },
    { 0x6fd225fc, 55 },
    { 0, 0 },
    { 0x3d5225ff, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c22605, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c22609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb722610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd722613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7722619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99622621, 23 },
    { 0x3422621, 61 },
    { 0xa622622, 76 },
    { 0xd4222624, 95 },
    { 0x4a022625, 100 },
    { 0xf7822626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d822629, 93 },
    { 0x3a02262a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c222632, 100 },
    { 0xf2b22632, 23 },
    { 0x21d22633, 60 },
    { 0xed622635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a922639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8242263c, 98 },
    { 0x9222263d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a522643, 90 },
    { 0, 0 },
    { 0x9ab22645, 99 },
    { 0x2d622646, 89 },
    { 0x47722646, 78 },
    { 0, 0 },
    { 0xafd22649, 80 },
    { 0x7d92264a, 84 },
    { 0x14c2264b, 61 },
    { 0, 0 },
    { 0xbfa2264d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13122652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f22656, 98 },
    { 0x3db22656, 23 },
    { 0xa4522657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a222663, 61 },
    { 0, 0 },
    { 0x81522665, 90 },
    { 0xbd322666, 43 },
    { 0x6c622665, 61 },
    { 0x5c122668, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb32266c, 55 },
    { 0xb9d2266d, 95 },
    { 0, 0 },
    { 0xe862266f, 90 },
    { 0x5562266f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb522674, 85 },
    { 0, 0 },
    { 0x59c22676, 85 },
    { 0, 0 },
    { 0x67422678, 76 },
    { 0x1cd22679, 98 },
    { 0x2b022678, 90 },
    { 0xa6f2267b, 88 },
    { 0xdec2267b, 61 },
    { 0xfea2267c, 91 },
    { 0x9c2267a, 84 },
    { 0x8882267f, 90 },
    { 0x5aa2267f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f22684, 99 },
    { 0xfc122685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312268a, 90 },
    { 0, 0 },
    { 0x3f82268c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1622690, 90 },
    { 0x21c22690, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98722694, 90 },
    { 0, 0 },
    { 0xd9522696, 93 },
    { 0, 0 },
    { 0x6ef22698, 55 },
    { 0xb2a22699, 46 },
    { 0xb9a22699, 90 },
    { 0, 0 },
    { 0x7f02269c, 61 },
    { 0xe672269d, 34 },
    { 0xdae2269e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd226a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73226a9, 88 },
    { 0xbba226a9, 52 },
    { 0xb8a226ab, 61 },
    { 0x923226a9, 33 },
    { 0x158226ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba226b5, 33 },
    { 0, 0 },
    { 0x96226b7, 76 },
    { 0x6ec226b8, 43 },
    { 0x675226b9, 43 },
    { 0, 0 },
    { 0x3eb226bb, 95 },
    { 0x5bf226bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8226c0, 95 },
    { 0xa60226c1, 90 },
    { 0xc226c0, 90 },
    { 0x1c8226c2, 84 },
    { 0xe42226c4, 98 },
    { 0, 0 },
    { 0x861226c6, 98 },
    { 0x6a3226c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x686226ca, 98 },
    { 0xd78226cb, 98 },
    { 0x745226cb, 30 },
    { 0xd8f226cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43226d2, 23 },
    { 0x68c226d3, 99 },
    { 0, 0 },
    { 0xc82226d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f226d8, 55 },
    { 0xa69226d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a226dc, 34 },
    { 0x706226dd, 23 },
    { 0x252226dd, 55 },
    { 0, 0 },
    { 0xe24226e0, 51 },
    { 0xfd226e1, 95 },
    { 0xa92226e1, 33 },
    { 0xc9f226e3, 76 },
    { 0xa72226e4, 98 },
    { 0x94b226e5, 95 },
    { 0x727226e6, 76 },
    { 0, 0 },
    { 0x3a1226e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x802226eb, 95 },
    { 0, 0 },
    { 0x4fa226ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a226f1, 23 },
    { 0x66c226f1, 61 },
    { 0, 0 },
    { 0x40b226f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41226fa, 77 },
    { 0, 0 },
    { 0x520226fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c022701, 23 },
    { 0xe3d22701, 90 },
    { 0x2c222703, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d22706, 34 },
    { 0x67922707, 93 },
    { 0xb7422708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e22711, 34 },
    { 0x1e722711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2322715, 95 },
    { 0x81e22716, 91 },
    { 0x6222716, 43 },
    { 0x14e22715, 23 },
    { 0xab522716, 95 },
    { 0x19c2271a, 90 },
    { 0xcb2271a, 61 },
    { 0xc0522719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed22720, 100 },
    { 0xe9422721, 46 },
    { 0x43c22722, 76 },
    { 0xf2b22722, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f22729, 52 },
    { 0x9132272a, 88 },
    { 0, 0 },
    { 0x5422272c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec322732, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd222735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e422738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc572273b, 90 },
    { 0x6f22273b, 90 },
    { 0, 0 },
    { 0x6762273e, 23 },
    { 0x69d2273f, 23 },
    { 0x7302273e, 90 },
    { 0x85f2273e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf022746, 90 },
    { 0x24722747, 61 },
    { 0xb0322748, 52 },
    { 0xe2f22749, 87 },
    { 0xaff2274a, 90 },
    { 0x2062274b, 90 },
    { 0x7272274c, 65 },
    { 0x14e22749, 46 },
    { 0, 0 },
    { 0x6f32274f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2822752, 90 },
    { 0x16322753, 61 },
    { 0xfbb22754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19722758, 81 },
    { 0x1d922759, 90 },
    { 0, 0 },
    { 0x91b2275b, 98 },
    { 0, 0 },
    { 0xa712275d, 68 },
    { 0x7cb2275e, 90 },
    { 0xb1f2275f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2022763, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d322768, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd2276b, 33 },
    { 0x842276c, 100 },
    { 0xff52276c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52522772, 23 },
    { 0x22222773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe222778, 98 },
    { 0x71322778, 90 },
    { 0xd3e22778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e22787, 84 },
    { 0, 0 },
    { 0x2ca22789, 33 },
    { 0x21422789, 33 },
    { 0xf172278b, 95 },
    { 0x4f32278c, 93 },
    { 0xa612278d, 90 },
    { 0x1cb2278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f22793, 99 },
    { 0x37322793, 95 },
    { 0, 0 },
    { 0xe6922796, 61 },
    { 0x13022797, 34 },
    { 0, 0 },
    { 0xa6022799, 46 },
    { 0xa432279a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f227a9, 84 },
    { 0xa82227aa, 98 },
    { 0xc12227aa, 55 },
    { 0, 0 },
    { 0x830227ad, 23 },
    { 0, 0 },
    { 0xd7d227af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78227b6, 100 },
    { 0, 0 },
    { 0xd01227b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a227bb, 91 },
    { 0xa71227bb, 90 },
    { 0x83f227bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8227c1, 33 },
    { 0, 0 },
    { 0xae6227c3, 84 },
    { 0x4aa227c4, 8 },
    { 0x5a1227c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e227c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2227cb, 33 },
    { 0x11c227cc, 46 },
    { 0xe1d227cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff227d0, 34 },
    { 0x2d7227d1, 90 },
    { 0x63c227d2, 55 },
    { 0x95e227d3, 99 },
    { 0x2b6227d4, 91 },
    { 0x4a6227d4, 98 },
    { 0, 0 },
    { 0x8227d7, 19 },
    { 0, 0 },
    { 0x6a5227d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0227de, 67 },
    { 0xfe9227de, 33 },
    { 0x8b5227df, 97 },
    { 0x69f227e1, 33 },
    { 0x453227df, 23 },
    { 0, 0 },
    { 0xa4e227e4, 33 },
    { 0, 0 },
    { 0xbfa227e6, 98 },
    { 0x1bc227e7, 33 },
    { 0, 0 },
    { 0x889227e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1227ee, 23 },
    { 0x498227ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea227f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479227fd, 34 },
    { 0, 0 },
    { 0xde5227ff, 90 },
    { 0x46622800, 55 },
    { 0x781227ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b122807, 23 },
    { 0xccc22808, 90 },
    { 0x32f22809, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2280f, 43 },
    { 0, 0 },
    { 0x7da22811, 76 },
    { 0, 0 },
    { 0x31922813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40922818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde522823, 33 },
    { 0x71322824, 90 },
    { 0x82222825, 4 },
    { 0x5f922826, 78 },
    { 0x22822827, 34 },
    { 0x20822828, 76 },
    { 0, 0 },
    { 0x1022282a, 33 },
    { 0, 0 },
    { 0x2ad2282c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c922835, 76 },
    { 0xe4b22836, 34 },
    { 0x68e22836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e2283a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb2283f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x11722842, 55 },
    { 0, 0 },
    { 0xfbb22844, 93 },
    { 0xd4d22844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52284a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f2284d, 98 },
    { 0x8e12284e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a22851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda22855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa822858, 68 },
    { 0, 0 },
    { 0xcc12285a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2285d, 98 },
    { 0x1022285e, 95 },
    { 0x20e2285f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c522862, 33 },
    { 0, 0 },
    { 0xda122864, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f2286b, 61 },
    { 0xf242286c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b22872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71122881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b422884, 69 },
    { 0x77622885, 55 },
    { 0xad922885, 98 },
    { 0xc9622887, 90 },
    { 0, 0 },
    { 0xc022889, 33 },
    { 0xefc2288a, 76 },
    { 0, 0 },
    { 0xd532288c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5522892, 61 },
    { 0, 0 },
    { 0xe0922894, 52 },
    { 0x9dc22895, 65 },
    { 0xe4a22894, 33 },
    { 0x6522896, 90 },
    { 0xaeb22897, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876228a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c228a8, 91 },
    { 0x6ec228a9, 91 },
    { 0, 0 },
    { 0x5c7228ab, 74 },
    { 0, 0 },
    { 0xbf9228ad, 76 },
    { 0, 0 },
    { 0x5e4228af, 98 },
    { 0x1ac228b0, 61 },
    { 0xd93228af, 33 },
    { 0, 0 },
    { 0xac4228b3, 36 },
    { 0x9e2228b3, 98 },
    { 0, 0 },
    { 0x9ba228b6, 61 },
    { 0, 0 },
    { 0x36b228b8, 61 },
    { 0x79c228b8, 33 },
    { 0x57d228ba, 61 },
    { 0x89d228bb, 77 },
    { 0x7b3228b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea228c1, 40 },
    { 0, 0 },
    { 0x7e7228c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902228c8, 52 },
    { 0x60a228c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5228cc, 61 },
    { 0x42b228cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43228d0, 61 },
    { 0, 0 },
    { 0x192228d2, 98 },
    { 0x9c7228d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5228d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5228da, 85 },
    { 0x4b4228db, 33 },
    { 0x71a228db, 96 },
    { 0, 0 },
    { 0x3b7228de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x926228e1, 61 },
    { 0xc59228e2, 23 },
    { 0x6ec228e3, 61 },
    { 0x897228e4, 75 },
    { 0xb9d228e5, 37 },
    { 0x638228e3, 90 },
    { 0xd21228e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257228ed, 95 },
    { 0, 0 },
    { 0x1ad228ef, 95 },
    { 0, 0 },
    { 0x269228f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70228f8, 33 },
    { 0, 0 },
    { 0xab4228fa, 90 },
    { 0, 0 },
    { 0x8f1228fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1822900, 98 },
    { 0xb0122901, 66 },
    { 0x7b222902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c2290a, 100 },
    { 0xc4d2290a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7412290e, 93 },
    { 0x6e32290f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e22916, 33 },
    { 0xfdf22917, 33 },
    { 0x9c322918, 38 },
    { 0x68522919, 33 },
    { 0x95d22917, 76 },
    { 0x7fd2291b, 93 },
    { 0xb8e2291a, 93 },
    { 0, 0 },
    { 0xa62291e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a222921, 98 },
    { 0x6d22922, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2292c, 95 },
    { 0x1352292d, 90 },
    { 0x7002292e, 52 },
    { 0xe432292f, 84 },
    { 0xb0922930, 95 },
    { 0, 0 },
    { 0xeed22932, 93 },
    { 0x12622933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2922937, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62293d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b22941, 61 },
    { 0, 0 },
    { 0x3f922943, 98 },
    { 0xce122943, 95 },
    { 0x87222945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a2294c, 93 },
    { 0x57b2294d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93222956, 95 },
    { 0, 0 },
    { 0x4122958, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d922967, 90 },
    { 0x96922968, 88 },
    { 0, 0 },
    { 0x1e2296a, 33 },
    { 0, 0 },
    { 0x1ba2296c, 61 },
    { 0xa552296d, 67 },
    { 0x96f2296c, 33 },
    { 0x3a02296f, 98 },
    { 0x5d82296f, 76 },
    { 0x52122971, 53 },
    { 0x97222971, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed222976, 43 },
    { 0xf22977, 90 },
    { 0x86122978, 34 },
    { 0x5fd22977, 90 },
    { 0, 0 },
    { 0x6972297b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa162297f, 66 },
    { 0xcf42297f, 34 },
    { 0, 0 },
    { 0x7d322982, 99 },
    { 0x4b322982, 61 },
    { 0xd7122984, 55 },
    { 0xd5722983, 34 },
    { 0x9c922985, 33 },
    { 0xea722987, 66 },
    { 0x4df22988, 81 },
    { 0x2d722988, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722998, 23 },
    { 0x32622998, 33 },
    { 0x8832299a, 43 },
    { 0, 0 },
    { 0x1f12299c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee2299f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291229a5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8229aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb229b4, 23 },
    { 0x754229b5, 52 },
    { 0x9fb229b5, 61 },
    { 0xe71229b7, 98 },
    { 0x6de229b8, 34 },
    { 0x17e229b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee229c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177229c6, 61 },
    { 0xe2d229c7, 33 },
    { 0, 0 },
    { 0x3f9229c9, 95 },
    { 0x9bd229c9, 23 },
    { 0x27f229ca, 90 },
    { 0x2b1229cc, 23 },
    { 0x59d229cd, 76 },
    { 0xb01229cd, 97 },
    { 0x6db229cd, 95 },
    { 0, 0 },
    { 0x58a229d1, 61 },
    { 0xef4229d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430229d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684229dc, 76 },
    { 0x5aa229dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe229e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423229ea, 43 },
    { 0, 0 },
    { 0xcad229ec, 65 },
    { 0, 0 },
    { 0xede229ee, 90 },
    { 0x6ea229ee, 98 },
    { 0xfa1229ee, 33 },
    { 0, 0 },
    { 0xa5229f2, 90 },
    { 0x39229f3, 61 },
    { 0x25b229f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb229f8, 95 },
    { 0, 0 },
    { 0xcba229fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888229fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e22a03, 75 },
    { 0xb0222a04, 61 },
    { 0xccf22a05, 33 },
    { 0x9af22a06, 86 },
    { 0, 0 },
    { 0xacf22a08, 23 },
    { 0x2ec22a09, 99 },
    { 0x91622a0a, 76 },
    { 0, 0 },
    { 0x72222a0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb22a1a, 90 },
    { 0x6a322a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62022a1f, 98 },
    { 0x70522a1f, 76 },
    { 0, 0 },
    { 0xb1922a22, 23 },
    { 0x1d522a22, 34 },
    { 0xe5a22a24, 90 },
    { 0, 0 },
    { 0x66f22a26, 55 },
    { 0x92f22a26, 69 },
    { 0x73622a28, 61 },
    { 0xe2b22a26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30422a30, 61 },
    { 0x80922a30, 33 },
    { 0x25722a32, 99 },
    { 0x6be22a33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae22a39, 55 },
    { 0, 0 },
    { 0x2322a3b, 78 },
    { 0, 0 },
    { 0xade22a3d, 100 },
    { 0xf5222a3d, 61 },
    { 0, 0 },
    { 0x8122a40, 95 },
    { 0x88822a40, 17 },
    { 0x2b22a42, 81 },
    { 0xfc522a42, 35 },
    { 0xf0422a42, 33 },
    { 0, 0 },
    { 0x50422a46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54922a56, 77 },
    { 0, 0 },
    { 0xdf622a58, 61 },
    { 0xd4422a59, 55 },
    { 0x42122a59, 93 },
    { 0, 0 },
    { 0x3fa22a5c, 23 },
    { 0, 0 },
    { 0xa22a5e, 61 },
    { 0, 0 },
    { 0x37322a60, 33 },
    { 0x23a22a60, 33 },
    { 0x42c22a62, 33 },
    { 0, 0 },
    { 0xde422a64, 81 },
    { 0x62122a65, 17 },
    { 0, 0 },
    { 0x57422a67, 76 },
    { 0, 0 },
    { 0x29722a69, 56 },
    { 0x15922a6a, 33 },
    { 0xd7222a6b, 90 },
    { 0, 0 },
    { 0x20722a6d, 95 },
    { 0x75022a6e, 33 },
    { 0xc8922a6f, 33 },
    { 0xae022a70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71922a74, 90 },
    { 0x83b22a75, 81 },
    { 0x16222a75, 66 },
    { 0x7be22a77, 33 },
    { 0, 0 },
    { 0xa4022a79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c22a7d, 61 },
    { 0xd8e22a7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622a82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f822a88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd22a8b, 91 },
    { 0, 0 },
    { 0x93e22a8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb22a92, 90 },
    { 0x6b222a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed22a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c22a9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd122a9f, 61 },
    { 0x4ef22a9f, 33 },
    { 0x6fb22aa1, 100 },
    { 0x74522aa2, 90 },
    { 0xc8822aa1, 95 },
    { 0xb4222aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d22aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5522ab2, 34 },
    { 0xde022ab3, 23 },
    { 0xb022ab4, 52 },
    { 0, 0 },
    { 0x8ee22ab6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b22aba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed722ac4, 90 },
    { 0x56d22ac5, 43 },
    { 0x98822ac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19422acb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9622ad0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff22ad5, 90 },
    { 0x90522ad6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2322add, 23 },
    { 0x9c22ade, 23 },
    { 0xfc822adf, 100 },
    { 0, 0 },
    { 0x28a22ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be22ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81522ae8, 93 },
    { 0xbaf22ae9, 55 },
    { 0x90622aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c22aed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74522af2, 91 },
    { 0xaea22af3, 100 },
    { 0x5a822af3, 99 },
    { 0x23022af5, 90 },
    { 0x1d722af5, 33 },
    { 0, 0 },
    { 0x19922af8, 23 },
    { 0x32022af8, 95 },
    { 0x3cc22afa, 52 },
    { 0, 0 },
    { 0xce322afc, 98 },
    { 0x44522afd, 95 },
    { 0xdfe22afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90722b05, 93 },
    { 0, 0 },
    { 0x8e022b07, 93 },
    { 0xb4422b08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6022b0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba22b0e, 90 },
    { 0x77122b0e, 90 },
    { 0x85322b10, 87 },
    { 0x9b922b10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5322b15, 93 },
    { 0x6ee22b15, 93 },
    { 0, 0 },
    { 0xab22b18, 93 },
    { 0x4f322b19, 87 },
    { 0x31f22b1a, 66 },
    { 0xe6b22b1b, 23 },
    { 0x40522b1c, 23 },
    { 0xf5322b1c, 43 },
    { 0x6dd22b1c, 61 },
    { 0x3cd22b1f, 23 },
    { 0x32f22b20, 23 },
    { 0x6f922b1c, 34 },
    { 0, 0 },
    { 0x2e022b23, 43 },
    { 0x5ed22b24, 43 },
    { 0xd9822b25, 64 },
    { 0xfec22b25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d622b2d, 91 },
    { 0xaae22b2d, 90 },
    { 0x29222b2f, 98 },
    { 0x5de22b2f, 90 },
    { 0xb9922b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e422b38, 95 },
    { 0xcc722b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c622b3c, 34 },
    { 0, 0 },
    { 0xc1722b3e, 33 },
    { 0xa3522b3f, 23 },
    { 0x71a22b3f, 90 },
    { 0x8fa22b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1422b45, 53 },
    { 0x50522b46, 87 },
    { 0xe422b45, 61 },
    { 0, 0 },
    { 0x9c822b49, 46 },
    { 0x4622b49, 34 },
    { 0xdf322b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d22b55, 78 },
    { 0xe4322b56, 33 },
    { 0x2a22b57, 90 },
    { 0xee522b57, 90 },
    { 0x26722b57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed22b60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43322b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74622b6d, 95 },
    { 0x1fa22b6e, 61 },
    { 0xfe022b6f, 90 },
    { 0xe6a22b6f, 98 },
    { 0x35a22b6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9122b74, 67 },
    { 0xb8c22b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a22b78, 33 },
    { 0x61a22b79, 99 },
    { 0, 0 },
    { 0x11422b7b, 33 },
    { 0xfd122b7b, 91 },
    { 0x50f22b7c, 98 },
    { 0, 0 },
    { 0xfd22b7f, 90 },
    { 0xdb522b7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa22b83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca22b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97022b90, 61 },
    { 0x51f22b91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0422b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf22b97, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed122b9b, 46 },
    { 0x2e022b9c, 76 },
    { 0x2b222b9c, 100 },
    { 0xb822b9e, 76 },
    { 0, 0 },
    { 0x4622ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb22ba3, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c22ba9, 90 },
    { 0, 0 },
    { 0xd5b22bab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e722bb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb322bb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f22bbc, 52 },
    { 0x63522bbc, 61 },
    { 0xf6822bbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80922bc2, 33 },
    { 0x1d522bc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58122bc8, 78 },
    { 0, 0 },
    { 0x99422bca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9022bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae522bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22bd4, 33 },
    { 0, 0 },
    { 0x3dc22bd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c922bd9, 78 },
    { 0, 0 },
    { 0x6d922bdb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a22be3, 84 },
    { 0x1b222be4, 78 },
    { 0, 0 },
    { 0xc4522be6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef822beb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b22bef, 90 },
    { 0xbb922bef, 72 },
    { 0xe7d22bf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60122bf4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe22bf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e922bfc, 76 },
    { 0, 0 },
    { 0x78d22bfe, 85 },
    { 0xc6a22bfe, 81 },
    { 0xd8b22bfe, 33 },
    { 0xf5c22bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12222c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc22c0c, 17 },
    { 0, 0 },
    { 0x88822c0e, 91 },
    { 0xf6722c0f, 33 },
    { 0xa0c22c10, 65 },
    { 0, 0 },
    { 0x91422c12, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df22c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc222c1c, 98 },
    { 0x6a722c1c, 76 },
    { 0x6f422c1e, 90 },
    { 0x5a022c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68522c24, 95 },
    { 0x5bd22c24, 90 },
    { 0x1a322c26, 95 },
    { 0xf522c27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x32422c2a, 95 },
    { 0x74422c2b, 23 },
    { 0x74022c2c, 90 },
    { 0x25022c2d, 90 },
    { 0xb1922c2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53022c32, 23 },
    { 0xf6822c33, 98 },
    { 0x6f322c33, 85 },
    { 0, 0 },
    { 0xe7222c36, 61 },
    { 0, 0 },
    { 0xf6622c38, 61 },
    { 0x82e22c39, 85 },
    { 0xa7122c3a, 97 },
    { 0, 0 },
    { 0xa6d22c3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88222c47, 91 },
    { 0, 0 },
    { 0x87d22c49, 77 },
    { 0xb8822c4a, 52 },
    { 0, 0 },
    { 0xff922c4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b22c50, 90 },
    { 0, 0 },
    { 0xa9022c52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x38122c55, 95 },
    { 0xa3c22c56, 95 },
    { 0, 0 },
    { 0x19b22c58, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f422c5e, 67 },
    { 0x3c322c5f, 33 },
    { 0x15122c5f, 91 },
    { 0xb3022c61, 23 },
    { 0x36e22c61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb922c66, 55 },
    { 0x44522c67, 61 },
    { 0xfd622c68, 99 },
    { 0, 0 },
    { 0xeb022c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c022c6d, 85 },
    { 0, 0 },
    { 0xbdb22c6f, 23 },
    { 0x60122c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec822c75, 52 },
    { 0xe6422c75, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46822c7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0622c88, 33 },
    { 0xb2222c89, 95 },
    { 0x31122c8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20822c8e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2422c92, 95 },
    { 0x56c22c93, 98 },
    { 0x33522c93, 33 },
    { 0, 0 },
    { 0x54422c96, 55 },
    { 0xfb822c97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71822c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69922ca4, 23 },
    { 0xdea22ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc22ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89822cae, 78 },
    { 0x26422cae, 90 },
    { 0x7c122cb0, 100 },
    { 0x55e22cb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9122cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7022cb9, 90 },
    { 0, 0 },
    { 0x9e322cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f522cbf, 17 },
    { 0x35122cbf, 90 },
    { 0, 0 },
    { 0x15c22cc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9722cc6, 43 },
    { 0, 0 },
    { 0xd1822cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22cce, 95 },
    { 0xace22ccf, 98 },
    { 0x60122cd0, 95 },
    { 0x22322ccf, 99 },
    { 0x49d22cd2, 23 },
    { 0, 0 },
    { 0x36022cd4, 66 },
    { 0xb8a22cd5, 61 },
    { 0xfd622cd4, 95 },
    { 0xc8722cd7, 90 },
    { 0x40622cd8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56222cde, 33 },
    { 0x49522cde, 66 },
    { 0, 0 },
    { 0x2e722ce1, 66 },
    { 0x65c22ce2, 33 },
    { 0x81022ce2, 85 },
    { 0xfc722ce4, 95 },
    { 0xdc322ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d222ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27222cef, 34 },
    { 0xa2722cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1922cf4, 61 },
    { 0, 0 },
    { 0x95f22cf6, 67 },
    { 0xebd22cf7, 68 },
    { 0x7de22cf8, 40 },
    { 0x9922cf8, 87 },
    { 0x91922cf6, 34 },
    { 0x5a822cfb, 34 },
    { 0x85722cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xade22cff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7022d03, 78 },
    { 0xe322d04, 33 },
    { 0, 0 },
    { 0x8c822d06, 90 },
    { 0xb1522d06, 82 },
    { 0, 0 },
    { 0xb3c22d09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2722d0d, 17 },
    { 0, 0 },
    { 0xc3822d0f, 95 },
    { 0x77722d0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db22d13, 90 },
    { 0xe6d22d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b22d18, 61 },
    { 0x58322d18, 90 },
    { 0, 0 },
    { 0x8a522d1b, 76 },
    { 0, 0 },
    { 0xbfa22d1d, 46 },
    { 0, 0 },
    { 0x83822d1f, 17 },
    { 0x43922d20, 95 },
    { 0x7ed22d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d922d26, 33 },
    { 0xb2722d27, 65 },
    { 0x65a22d26, 98 },
    { 0x29222d26, 95 },
    { 0x36e22d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f22d2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93622d35, 78 },
    { 0xadf22d36, 53 },
    { 0xe1f22d36, 33 },
    { 0x39b22d38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0222d3c, 99 },
    { 0, 0 },
    { 0x5af22d3e, 52 },
    { 0x7a922d3f, 98 },
    { 0x7522d40, 98 },
    { 0, 0 },
    { 0x66422d42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422d46, 34 },
    { 0xe7222d46, 77 },
    { 0x64b22d48, 38 },
    { 0, 0 },
    { 0xe3f22d4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b22d4d, 61 },
    { 0, 0 },
    { 0x86622d4f, 66 },
    { 0, 0 },
    { 0x38322d51, 100 },
    { 0, 0 },
    { 0x5f222d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a22d57, 90 },
    { 0, 0 },
    { 0x26322d59, 52 },
    { 0xf6422d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f922d5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b922d65, 33 },
    { 0, 0 },
    { 0x32222d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d22d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7222d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f622d73, 23 },
    { 0xcf022d74, 68 },
    { 0x6b422d73, 98 },
    { 0xcea22d76, 33 },
    { 0x7f222d77, 88 },
    { 0xa8522d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d22d7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d22d7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b22d86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd422d8b, 90 },
    { 0xc8722d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1222d91, 65 },
    { 0x91d22d91, 55 },
    { 0, 0 },
    { 0xa9a22d94, 87 },
    { 0x26f22d94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e22d98, 61 },
    { 0, 0 },
    { 0x3b722d9a, 33 },
    { 0xac422d9b, 76 },
    { 0x13822d9c, 68 },
    { 0, 0 },
    { 0x34222d9e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77822da8, 33 },
    { 0x24322da8, 55 },
    { 0x9822daa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3722db4, 36 },
    { 0x7fb22db4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea22db9, 95 },
    { 0, 0 },
    { 0x2b822dbb, 76 },
    { 0xc8f22dbc, 90 },
    { 0xd5922dbc, 33 },
    { 0x51e22dbb, 61 },
    { 0, 0 },
    { 0xa2e22dc0, 33 },
    { 0x9022dc1, 61 },
    { 0xce822dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b22dc9, 98 },
    { 0x5ed22dca, 34 },
    { 0xa5122dcb, 33 },
    { 0x5c122dca, 66 },
    { 0, 0 },
    { 0x44f22dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d22dd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66622dd8, 34 },
    { 0x8f522dd9, 90 },
    { 0, 0 },
    { 0x18022ddb, 90 },
    { 0, 0 },
    { 0x40a22ddd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6122de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1422de5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f22de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a22dec, 76 },
    { 0x5f522ded, 95 },
    { 0x7b122dee, 90 },
    { 0, 0 },
    { 0x9c922df0, 23 },
    { 0, 0 },
    { 0xf7d22df2, 76 },
    { 0xfd822df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e22df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a22dfb, 78 },
    { 0xd0822dfc, 81 },
    { 0xd4422dfc, 67 },
    { 0, 0 },
    { 0x36922dff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed322e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a22e05, 77 },
    { 0x96e22e06, 43 },
    { 0x9f122e07, 90 },
    { 0xa2722e08, 55 },
    { 0, 0 },
    { 0x6aa22e0a, 65 },
    { 0xada22e0a, 95 },
    { 0x65322e0c, 46 },
    { 0x63f22e0a, 33 },
    { 0, 0 },
    { 0xe3f22e0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32122e13, 33 },
    { 0x822e13, 90 },
    { 0, 0 },
    { 0x60c22e16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92422e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf922e1e, 55 },
    { 0x22722e1f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4222e22, 46 },
    { 0x53222e23, 61 },
    { 0x31122e23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e22e27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f222e2b, 46 },
    { 0, 0 },
    { 0xb4822e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb322e30, 46 },
    { 0xb5822e30, 90 },
    { 0xa0e22e32, 23 },
    { 0xd1322e31, 90 },
    { 0, 0 },
    { 0xf2622e35, 93 },
    { 0x11b22e35, 33 },
    { 0x61922e36, 46 },
    { 0x61822e35, 23 },
    { 0xe0d22e35, 55 },
    { 0xcc522e35, 93 },
    { 0x35222e39, 90 },
    { 0, 0 },
    { 0xef822e3d, 67 },
    { 0x34f22e3e, 90 },
    { 0, 0 },
    { 0x89622e40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0522e43, 46 },
    { 0xecf22e44, 68 },
    { 0, 0 },
    { 0x20c22e46, 95 },
    { 0, 0 },
    { 0x9fe22e48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f422e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c22e4f, 43 },
    { 0, 0 },
    { 0x60522e51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27422e57, 33 },
    { 0, 0 },
    { 0x1be22e59, 90 },
    { 0xeb22e5a, 23 },
    { 0xbd622e59, 94 },
    { 0, 0 },
    { 0x52f22e5d, 98 },
    { 0x13e22e5d, 98 },
    { 0xcd922e5f, 93 },
    { 0x45822e60, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a22e64, 61 },
    { 0, 0 },
    { 0xf0e22e66, 99 },
    { 0xfe022e67, 46 },
    { 0x2bf22e67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d22e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f122e72, 90 },
    { 0xed322e72, 98 },
    { 0x70922e74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f22e7a, 34 },
    { 0x9f622e7b, 61 },
    { 0x2ba22e7c, 95 },
    { 0x7ff22e7c, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42022e83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b22e87, 84 },
    { 0xf6222e87, 43 },
    { 0, 0 },
    { 0xc4c22e8a, 78 },
    { 0xae622e8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422e8e, 33 },
    { 0x5b522e8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d22e93, 33 },
    { 0, 0 },
    { 0x2e22e95, 23 },
    { 0, 0 },
    { 0xf1422e97, 90 },
    { 0xb6222e97, 66 },
    { 0x56422e98, 61 },
    { 0xeb222e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d22e9f, 77 },
    { 0, 0 },
    { 0x7c822ea1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29522ea6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e22eaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d22eba, 43 },
    { 0xb7f22ebb, 36 },
    { 0, 0 },
    { 0xc122ebd, 91 },
    { 0x20c22ebd, 90 },
    { 0x30c22ebf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20622ec7, 90 },
    { 0x5a522ec8, 33 },
    { 0x7a822ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46422ecd, 66 },
    { 0x85822ecd, 98 },
    { 0x77822ecf, 33 },
    { 0, 0 },
    { 0xaf822ed1, 90 },
    { 0xe822ed2, 36 },
    { 0xa7722ed2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60822ed8, 61 },
    { 0x8c522ed9, 85 },
    { 0x66322ed9, 61 },
    { 0xb4722edb, 98 },
    { 0, 0 },
    { 0x77522edd, 98 },
    { 0x75922edd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4522ee2, 43 },
    { 0x10d22ee2, 76 },
    { 0, 0 },
    { 0xdd122ee5, 34 },
    { 0xbe022ee6, 40 },
    { 0xd6622ee6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4122eea, 43 },
    { 0x38222eeb, 90 },
    { 0x3e222eec, 91 },
    { 0x80422eec, 61 },
    { 0, 0 },
    { 0x8c122eef, 61 },
    { 0x7b822ef0, 23 },
    { 0, 0 },
    { 0xc8022ef2, 76 },
    { 0x92f22ef3, 88 },
    { 0xb9822ef4, 85 },
    { 0, 0 },
    { 0xe2622ef6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8522efc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c22eff, 78 },
    { 0, 0 },
    { 0x62422f01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f422f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8422f0d, 33 },
    { 0, 0 },
    { 0x1df22f0f, 90 },
    { 0x50a22f10, 61 },
    { 0, 0 },
    { 0x6922f12, 77 },
    { 0, 0 },
    { 0x2d522f14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b522f1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf22f1f, 67 },
    { 0, 0 },
    { 0xfc422f21, 98 },
    { 0x3bd22f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c22f28, 83 },
    { 0x80722f29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a22f30, 78 },
    { 0, 0 },
    { 0x2422f32, 98 },
    { 0xb9522f33, 90 },
    { 0, 0 },
    { 0xe0622f35, 66 },
    { 0, 0 },
    { 0xdae22f37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50522f3b, 55 },
    { 0xfa322f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c22f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad22f44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e22f48, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c922f53, 61 },
    { 0xf8522f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f22f6a, 81 },
    { 0x50622f6b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb122f6f, 52 },
    { 0xcec22f6f, 34 },
    { 0x52622f71, 55 },
    { 0x5b922f72, 34 },
    { 0, 0 },
    { 0x6922f74, 55 },
    { 0x61522f75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca22f78, 33 },
    { 0x7222f79, 93 },
    { 0, 0 },
    { 0x2d922f7b, 61 },
    { 0, 0 },
    { 0x322f7d, 90 },
    { 0, 0 },
    { 0x53a22f7f, 84 },
    { 0xca22f80, 23 },
    { 0xdd22f81, 97 },
    { 0x37d22f7f, 34 },
    { 0xa1722f83, 23 },
    { 0x9da22f84, 95 },
    { 0x12f22f80, 90 },
    { 0x38122f86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa522f8b, 95 },
    { 0x79b22f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22f94, 95 },
    { 0xb3322f95, 87 },
    { 0x12522f95, 90 },
    { 0xefc22f95, 55 },
    { 0, 0 },
    { 0x22222f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56522f9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc222fa2, 61 },
    { 0x76922fa2, 99 },
    { 0x34022fa4, 66 },
    { 0x9bf22fa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c22fa9, 76 },
    { 0x37a22faa, 77 },
    { 0xcfc22faa, 67 },
    { 0x67722fac, 34 },
    { 0xc2822fad, 66 },
    { 0x28422fac, 61 },
    { 0x3fb22fad, 33 },
    { 0, 0 },
    { 0x5f722fb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63922fb9, 34 },
    { 0x7e622fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1922fc1, 52 },
    { 0, 0 },
    { 0xe9522fc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b222fc6, 76 },
    { 0xe0b22fc7, 55 },
    { 0xa2b22fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8422fd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96322fd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86522fd7, 23 },
    { 0, 0 },
    { 0xd1f22fd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea22fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a22fe1, 55 },
    { 0x90822fe2, 34 },
    { 0x26122fe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27322fe6, 23 },
    { 0x50822fe6, 76 },
    { 0, 0 },
    { 0x6d222fe9, 17 },
    { 0, 0 },
    { 0x79a22feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf22fee, 76 },
    { 0, 0 },
    { 0x11022ff0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98822ff3, 100 },
    { 0, 0 },
    { 0xaa322ff5, 90 },
    { 0xef722ff6, 90 },
    { 0xaf22ff5, 23 },
    { 0x6f122ff8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x23122ffb, 87 },
    { 0x92922ffb, 55 },
    { 0, 0 },
    { 0xb4022ffe, 68 },
    { 0, 0 },
    { 0xd7423000, 40 },
    { 0xd1f23001, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad23007, 95 },
    { 0, 0 },
    { 0xea823009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b2300e, 78 },
    { 0, 0 },
    { 0x6a923010, 90 },
    { 0x82f23011, 65 },
    { 0xf8c23012, 90 },
    { 0, 0 },
    { 0x34523014, 36 },
    { 0xf5a23014, 90 },
    { 0, 0 },
    { 0x59f23017, 90 },
    { 0, 0 },
    { 0xef223019, 84 },
    { 0xac12301a, 84 },
    { 0x30b2301a, 90 },
    { 0, 0 },
    { 0x5b32301d, 23 },
    { 0xe5f2301e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9823026, 61 },
    { 0x2f923027, 96 },
    { 0xf1123028, 46 },
    { 0, 0 },
    { 0xdb82302a, 98 },
    { 0x7152302b, 90 },
    { 0xdd12302b, 90 },
    { 0xdcd2302b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4023030, 55 },
    { 0, 0 },
    { 0x25f23032, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4323036, 33 },
    { 0xe4c23037, 61 },
    { 0x89723038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f23045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f723048, 23 },
    { 0x2ea23048, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f2304e, 33 },
    { 0x5f32304f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa23052, 34 },
    { 0x45523053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e823057, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a623061, 76 },
    { 0x3f323062, 95 },
    { 0x8723061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6192306b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722306e, 33 },
    { 0x402306f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f23072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78923078, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca2307e, 99 },
    { 0xb012307e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab23082, 66 },
    { 0xefd23082, 33 },
    { 0, 0 },
    { 0x98a23085, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362308b, 90 },
    { 0x3662308c, 33 },
    { 0, 0 },
    { 0x6ee2308e, 95 },
    { 0, 0 },
    { 0xa9b23090, 52 },
    { 0, 0 },
    { 0xe7d23092, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61123096, 69 },
    { 0xa0123097, 99 },
    { 0xf1923096, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12309c, 98 },
    { 0xab52309c, 61 },
    { 0xcd62309c, 90 },
    { 0xec2309f, 61 },
    { 0x742309f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x205230a3, 55 },
    { 0xa65230a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x285230a7, 61 },
    { 0x3bf230a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d230b0, 84 },
    { 0x8d0230b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965230b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x182230b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16230c9, 12 },
    { 0xa97230ca, 55 },
    { 0, 0 },
    { 0xc9d230cc, 55 },
    { 0, 0 },
    { 0x513230ce, 76 },
    { 0, 0 },
    { 0xed4230d0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34230d6, 61 },
    { 0xabb230d7, 91 },
    { 0x8dc230d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe230db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80230e0, 61 },
    { 0, 0 },
    { 0xc76230e2, 78 },
    { 0x185230e2, 93 },
    { 0x32c230e2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c230e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6230eb, 55 },
    { 0, 0 },
    { 0xeb5230ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d230f0, 33 },
    { 0, 0 },
    { 0x312230f2, 90 },
    { 0xa46230f2, 43 },
    { 0xfa230f4, 33 },
    { 0x8a8230f5, 23 },
    { 0x7ba230f6, 8 },
    { 0x2a9230f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e230fd, 77 },
    { 0x67f230fd, 33 },
    { 0, 0 },
    { 0x85723100, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x33723103, 99 },
    { 0x8c223104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51623107, 93 },
    { 0, 0 },
    { 0xaa923109, 34 },
    { 0x9d22310a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x612310d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa623110, 95 },
    { 0xf323110, 43 },
    { 0xeeb23112, 75 },
    { 0xc8123110, 34 },
    { 0x88b23114, 91 },
    { 0x80123115, 93 },
    { 0xc0823113, 61 },
    { 0, 0 },
    { 0x65f23118, 33 },
    { 0xd0723119, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9552311c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf32311f, 23 },
    { 0x3d2311f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c123123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2312a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5023132, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d323139, 99 },
    { 0x60b2313a, 40 },
    { 0xdd52313a, 23 },
    { 0, 0 },
    { 0x82e2313d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b923144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa23148, 98 },
    { 0xd4b23149, 90 },
    { 0, 0 },
    { 0xcea2314b, 33 },
    { 0x3362314b, 76 },
    { 0x8452314b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf23151, 95 },
    { 0xf9c23152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72315b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e23160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a223166, 68 },
    { 0x4723167, 61 },
    { 0x8db23168, 65 },
    { 0x96c23169, 87 },
    { 0xfbf2316a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de23171, 76 },
    { 0x8d23172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c023176, 33 },
    { 0, 0 },
    { 0x94823178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2317b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9332317f, 90 },
    { 0x52b23180, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62723184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd23189, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b2318d, 90 },
    { 0x2662318e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef523192, 52 },
    { 0, 0 },
    { 0x58723194, 33 },
    { 0, 0 },
    { 0xe2e23196, 98 },
    { 0x6ab23197, 55 },
    { 0xfe523197, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d231a1, 23 },
    { 0, 0 },
    { 0x9ff231a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a231a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8231ab, 77 },
    { 0xd50231ac, 61 },
    { 0x647231ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf231b2, 95 },
    { 0, 0 },
    { 0xea231b4, 68 },
    { 0xf5a231b5, 33 },
    { 0xdb4231b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f231be, 78 },
    { 0xb4231be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f231c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162231c8, 23 },
    { 0x62b231c8, 95 },
    { 0xd01231c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c231d0, 76 },
    { 0, 0 },
    { 0xbf6231d2, 33 },
    { 0xa7b231d2, 61 },
    { 0x84f231d4, 23 },
    { 0, 0 },
    { 0xe35231d6, 40 },
    { 0x8c2231d6, 33 },
    { 0x493231d6, 85 },
    { 0x5aa231d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df231dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d231e0, 95 },
    { 0x621231e1, 23 },
    { 0xb16231e2, 33 },
    { 0x810231e3, 90 },
    { 0, 0 },
    { 0xc61231e5, 78 },
    { 0x15e231e6, 99 },
    { 0, 0 },
    { 0x21b231e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a231eb, 33 },
    { 0x62d231ec, 90 },
    { 0xa00231ec, 33 },
    { 0x30d231ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947231f9, 65 },
    { 0, 0 },
    { 0xc89231fb, 17 },
    { 0xb0c231fb, 23 },
    { 0x108231fb, 69 },
    { 0xe78231fb, 38 },
    { 0x204231ff, 68 },
    { 0x76f231fc, 90 },
    { 0, 0 },
    { 0x50123202, 43 },
    { 0xa8923202, 90 },
    { 0, 0 },
    { 0x81f23205, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b923209, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a2320e, 61 },
    { 0, 0 },
    { 0x20123210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e323214, 76 },
    { 0xa1e23214, 76 },
    { 0xca523216, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f2321d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23221, 76 },
    { 0x85123221, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf23226, 98 },
    { 0x8d723227, 33 },
    { 0x10c23227, 36 },
    { 0x46823229, 52 },
    { 0xa902322a, 64 },
    { 0, 0 },
    { 0xa052322c, 87 },
    { 0xa8a2322d, 90 },
    { 0x4512322d, 61 },
    { 0x1b42322f, 61 },
    { 0xdc22322f, 76 },
    { 0x10823231, 90 },
    { 0xa632322f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd23236, 38 },
    { 0x8fc23236, 69 },
    { 0, 0 },
    { 0x92023239, 53 },
    { 0xf8f23239, 43 },
    { 0x5252323b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99323246, 93 },
    { 0xce223247, 34 },
    { 0x91223246, 95 },
    { 0x90c23249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2324d, 98 },
    { 0, 0 },
    { 0x76d2324f, 95 },
    { 0x22b23250, 55 },
    { 0, 0 },
    { 0x20d23252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a723255, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c023258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2325c, 90 },
    { 0x4e92325d, 33 },
    { 0x7a42325e, 100 },
    { 0xe9c2325f, 98 },
    { 0, 0 },
    { 0xaa323261, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45723266, 61 },
    { 0x80e23267, 98 },
    { 0x98e23266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e2326d, 90 },
    { 0x13b2326e, 48 },
    { 0x692326f, 55 },
    { 0, 0 },
    { 0xc0023271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a23274, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb023277, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372327b, 43 },
    { 0x8662327b, 23 },
    { 0x28b2327b, 23 },
    { 0x8eb2327e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e23289, 61 },
    { 0x4292328a, 46 },
    { 0xeb32328a, 61 },
    { 0, 0 },
    { 0x45d2328d, 95 },
    { 0xb9a2328e, 33 },
    { 0x2a42328e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0923293, 90 },
    { 0x5d023293, 90 },
    { 0x72323294, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5822329a, 85 },
    { 0xfbe2329b, 77 },
    { 0xb062329c, 95 },
    { 0xe5e2329b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8232a1, 90 },
    { 0x482232a2, 76 },
    { 0xacb232a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e232a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8232ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x758232ae, 88 },
    { 0, 0 },
    { 0x12d232b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x535232b3, 68 },
    { 0, 0 },
    { 0xe35232b5, 33 },
    { 0x4ce232b5, 91 },
    { 0x75232b7, 76 },
    { 0x1f4232b7, 61 },
    { 0x831232b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7232bd, 90 },
    { 0x397232be, 66 },
    { 0xfb1232bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8232c4, 61 },
    { 0xe90232c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54232c8, 52 },
    { 0x3bf232c9, 65 },
    { 0x78a232ca, 93 },
    { 0xd39232c9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a232d2, 23 },
    { 0xcda232d3, 68 },
    { 0xec2232d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2232da, 61 },
    { 0x3f232da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad232e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0232e6, 33 },
    { 0x74a232e7, 55 },
    { 0xd72232e8, 23 },
    { 0x247232e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe232ec, 95 },
    { 0x254232ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708232f3, 61 },
    { 0, 0 },
    { 0x575232f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4232f8, 23 },
    { 0x47c232f9, 55 },
    { 0x25f232fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb232fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4923306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92923312, 23 },
    { 0xa9523312, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1323316, 66 },
    { 0x22923317, 90 },
    { 0, 0 },
    { 0xc8c23319, 95 },
    { 0, 0 },
    { 0xd7d2331b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b32331e, 93 },
    { 0, 0 },
    { 0x63f23320, 61 },
    { 0xe9023321, 34 },
    { 0xce123320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852332a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad523330, 91 },
    { 0xa5a23331, 33 },
    { 0, 0 },
    { 0x95823333, 43 },
    { 0xdd823333, 95 },
    { 0x10f23334, 61 },
    { 0x85a23335, 95 },
    { 0xb2f23336, 61 },
    { 0, 0 },
    { 0x6eb23339, 99 },
    { 0, 0 },
    { 0x5252333b, 76 },
    { 0xb2f2333c, 76 },
    { 0x2972333d, 90 },
    { 0x6ec2333e, 91 },
    { 0xb742333f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f323342, 90 },
    { 0x3e223343, 88 },
    { 0, 0 },
    { 0x29d23345, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2334e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a23352, 91 },
    { 0, 0 },
    { 0xfe023354, 93 },
    { 0, 0 },
    { 0x45a23356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b23359, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4772335d, 61 },
    { 0, 0 },
    { 0x57c2335f, 33 },
    { 0x27823360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8023363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932336e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e823372, 93 },
    { 0, 0 },
    { 0x72423374, 90 },
    { 0xd3d23375, 90 },
    { 0xc0623375, 98 },
    { 0xa1023377, 95 },
    { 0x6ef23378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e2337e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f423384, 34 },
    { 0x3d123384, 23 },
    { 0, 0 },
    { 0x323387, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa72338c, 47 },
    { 0, 0 },
    { 0x5742338e, 34 },
    { 0, 0 },
    { 0x30323390, 23 },
    { 0x2c223390, 19 },
    { 0xa4423392, 61 },
    { 0xdc423392, 76 },
    { 0x53323394, 23 },
    { 0xd2323393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f323398, 17 },
    { 0x59d23399, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad2339e, 95 },
    { 0xd482339e, 93 },
    { 0x2b5233a0, 98 },
    { 0xe8d233a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2233a7, 43 },
    { 0x11f233a8, 59 },
    { 0x725233a9, 34 },
    { 0xbcc233a9, 81 },
    { 0x253233a8, 43 },
    { 0xfd7233ac, 61 },
    { 0xc28233ad, 33 },
    { 0xef233ab, 76 },
    { 0xe2233ac, 95 },
    { 0x4bc233b0, 67 },
    { 0x63e233b1, 78 },
    { 0xb89233af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9233b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae233bf, 90 },
    { 0, 0 },
    { 0xb38233c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567233c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09233c9, 61 },
    { 0, 0 },
    { 0x632233cb, 85 },
    { 0, 0 },
    { 0x312233cd, 55 },
    { 0x45233ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958233d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883233d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7233de, 43 },
    { 0, 0 },
    { 0x819233e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6233e3, 68 },
    { 0x788233e4, 33 },
    { 0x85f233e5, 90 },
    { 0xc63233e4, 61 },
    { 0xd2b233e6, 61 },
    { 0xc07233e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb233ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94233f4, 50 },
    { 0x25c233f4, 99 },
    { 0, 0 },
    { 0xd09233f7, 23 },
    { 0, 0 },
    { 0x599233f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb233fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e23405, 90 },
    { 0x11723406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c2340b, 55 },
    { 0x16d2340c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12423414, 99 },
    { 0x7a523415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f23418, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2341f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7823422, 33 },
    { 0xbfd23423, 55 },
    { 0x6a023424, 98 },
    { 0xba323425, 61 },
    { 0x9e123426, 23 },
    { 0xf6623426, 84 },
    { 0xfc923425, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082342b, 23 },
    { 0, 0 },
    { 0x2ed2342d, 34 },
    { 0x2b42342e, 76 },
    { 0xa872342d, 43 },
    { 0x2a52342d, 17 },
    { 0, 0 },
    { 0xa6223432, 61 },
    { 0, 0 },
    { 0xb6323434, 91 },
    { 0x70023435, 90 },
    { 0x66e23434, 76 },
    { 0, 0 },
    { 0xe1c23438, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b2343d, 61 },
    { 0xb4e2343d, 90 },
    { 0x6e22343d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71623447, 33 },
    { 0, 0 },
    { 0x69323449, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32344c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed23450, 98 },
    { 0xe6d23451, 90 },
    { 0, 0 },
    { 0xcd923453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4323456, 33 },
    { 0, 0 },
    { 0x73b23458, 76 },
    { 0x5d823459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812345d, 40 },
    { 0, 0 },
    { 0x8e82345f, 55 },
    { 0x64f23460, 90 },
    { 0xb4f23461, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb23464, 55 },
    { 0x74823465, 85 },
    { 0x9ec23466, 98 },
    { 0x5f223467, 23 },
    { 0, 0 },
    { 0xfae23469, 23 },
    { 0x71c23469, 90 },
    { 0xc482346b, 90 },
    { 0, 0 },
    { 0x98f2346d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89423472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d23476, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed23479, 68 },
    { 0x5d02347a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b23481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a2348b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062348e, 90 },
    { 0, 0 },
    { 0x6e623490, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6323493, 98 },
    { 0, 0 },
    { 0xac623495, 90 },
    { 0x13e23496, 65 },
    { 0x9ce23495, 33 },
    { 0, 0 },
    { 0x80d23499, 78 },
    { 0x7e02349a, 60 },
    { 0, 0 },
    { 0x3cf2349c, 23 },
    { 0, 0 },
    { 0x5272349e, 67 },
    { 0xa9f2349f, 90 },
    { 0x64c234a0, 23 },
    { 0x7da234a1, 33 },
    { 0, 0 },
    { 0x53f234a3, 23 },
    { 0x1c2234a4, 90 },
    { 0x99a234a5, 81 },
    { 0, 0 },
    { 0x86d234a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced234ac, 76 },
    { 0xd7a234ad, 33 },
    { 0xe16234ae, 33 },
    { 0x70d234af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627234b3, 95 },
    { 0xca6234b3, 17 },
    { 0, 0 },
    { 0xccd234b6, 95 },
    { 0x653234b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed234bb, 34 },
    { 0xe5e234bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8234c1, 23 },
    { 0x649234c1, 95 },
    { 0, 0 },
    { 0x824234c4, 93 },
    { 0xfea234c5, 93 },
    { 0, 0 },
    { 0xb35234c7, 33 },
    { 0x36a234c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe234cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x859234ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4234d2, 85 },
    { 0, 0 },
    { 0x51d234d4, 34 },
    { 0xf35234d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187234d9, 78 },
    { 0x159234da, 90 },
    { 0, 0 },
    { 0x4fc234dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471234e0, 34 },
    { 0x96a234e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x265234e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85234e8, 84 },
    { 0xd2e234e9, 61 },
    { 0x1c9234ea, 33 },
    { 0xdf9234eb, 81 },
    { 0x642234ec, 17 },
    { 0xbe0234ed, 46 },
    { 0xaea234ed, 75 },
    { 0x564234ef, 61 },
    { 0x19234f0, 35 },
    { 0x928234ed, 95 },
    { 0, 0 },
    { 0xf81234f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4234f6, 23 },
    { 0xbae234f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f923500, 61 },
    { 0xd0e23501, 90 },
    { 0x46c23501, 61 },
    { 0x1123500, 69 },
    { 0xee23502, 33 },
    { 0, 0 },
    { 0x72823506, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae2350c, 33 },
    { 0x2a42350d, 53 },
    { 0x3732350e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa23512, 46 },
    { 0, 0 },
    { 0xa4723514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f23517, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8723520, 33 },
    { 0, 0 },
    { 0x18423522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70423527, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2352a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x432352d, 90 },
    { 0, 0 },
    { 0xe12352f, 61 },
    { 0xd7a2352f, 34 },
    { 0, 0 },
    { 0x5d323532, 98 },
    { 0x7a223532, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe23536, 34 },
    { 0x53023536, 85 },
    { 0x47d23536, 85 },
    { 0, 0 },
    { 0x4f22353a, 90 },
    { 0, 0 },
    { 0x5262353c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d22353f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c23548, 78 },
    { 0xb3c23549, 90 },
    { 0, 0 },
    { 0x6682354b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36323550, 8 },
    { 0, 0 },
    { 0x8b623552, 34 },
    { 0x8a523553, 23 },
    { 0xe2923554, 91 },
    { 0x29e23552, 99 },
    { 0x15123556, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c623559, 69 },
    { 0xba72355a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42355e, 90 },
    { 0x89b2355f, 23 },
    { 0x7fb2355f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb423565, 95 },
    { 0x6e23565, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca02356a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c223570, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6623575, 90 },
    { 0xed823576, 61 },
    { 0x2bf23577, 54 },
    { 0xa2b23577, 34 },
    { 0, 0 },
    { 0x4ec2357a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2672357e, 33 },
    { 0xa232357f, 61 },
    { 0xddd2357f, 90 },
    { 0x2b823581, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xad023584, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d523587, 55 },
    { 0x8d623587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99623590, 98 },
    { 0xa7123591, 40 },
    { 0x62a23592, 34 },
    { 0x1fe23590, 76 },
    { 0x8d523594, 95 },
    { 0, 0 },
    { 0x91b23596, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c23599, 98 },
    { 0xcec2359a, 61 },
    { 0x13c23599, 55 },
    { 0, 0 },
    { 0x8a62359d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf235a0, 90 },
    { 0x1ed235a1, 90 },
    { 0x357235a1, 95 },
    { 0xada235a1, 61 },
    { 0xbd3235a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f235a8, 49 },
    { 0x131235a8, 90 },
    { 0x2dc235a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e235ad, 61 },
    { 0xefb235ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b235b1, 93 },
    { 0x741235b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93235b6, 60 },
    { 0x588235b7, 19 },
    { 0x324235b8, 23 },
    { 0xc70235b9, 90 },
    { 0xe99235ba, 43 },
    { 0, 0 },
    { 0x9a5235bc, 34 },
    { 0x339235bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6235c3, 98 },
    { 0x2ac235c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4235c9, 19 },
    { 0xc3f235ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08235cf, 33 },
    { 0, 0 },
    { 0x289235d1, 46 },
    { 0xff235d1, 43 },
    { 0, 0 },
    { 0x7cb235d4, 95 },
    { 0, 0 },
    { 0x531235d6, 95 },
    { 0x53c235d6, 85 },
    { 0, 0 },
    { 0x155235d9, 78 },
    { 0xab0235da, 55 },
    { 0, 0 },
    { 0xa4c235dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d235df, 98 },
    { 0, 0 },
    { 0x325235e1, 19 },
    { 0x240235e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd235e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a235e9, 95 },
    { 0x1d5235ea, 90 },
    { 0x393235eb, 95 },
    { 0xae0235ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b235ef, 91 },
    { 0x3b6235f0, 95 },
    { 0xabe235f0, 43 },
    { 0, 0 },
    { 0xed0235f3, 33 },
    { 0x79f235f3, 61 },
    { 0, 0 },
    { 0x4c235f6, 46 },
    { 0, 0 },
    { 0xa32235f8, 33 },
    { 0xc91235f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d235fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae23605, 33 },
    { 0x2b923606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc2360b, 61 },
    { 0, 0 },
    { 0xbf52360d, 46 },
    { 0, 0 },
    { 0xee42360f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x91323612, 23 },
    { 0x13423612, 90 },
    { 0x5f23613, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76023618, 61 },
    { 0x1a223619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e22361c, 34 },
    { 0xf022361d, 100 },
    { 0, 0 },
    { 0x2bd2361f, 55 },
    { 0, 0 },
    { 0x62b23621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c23626, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf23629, 33 },
    { 0, 0 },
    { 0xe02362b, 61 },
    { 0xfed2362c, 90 },
    { 0, 0 },
    { 0x32e2362e, 55 },
    { 0x35a2362e, 59 },
    { 0, 0 },
    { 0x1a423631, 76 },
    { 0xb8223632, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13023635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9423638, 90 },
    { 0x6f223639, 98 },
    { 0x8d223638, 75 },
    { 0xb4a2363b, 23 },
    { 0xf592363c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3132363f, 61 },
    { 0, 0 },
    { 0x76123641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1923644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a423647, 61 },
    { 0x67323648, 90 },
    { 0x60523649, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc492364f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83823655, 52 },
    { 0xd5d23655, 95 },
    { 0, 0 },
    { 0xb1a23658, 98 },
    { 0xea423659, 33 },
    { 0x1eb2365a, 95 },
    { 0, 0 },
    { 0x47a2365c, 76 },
    { 0x6932365d, 33 },
    { 0xce92365e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36523664, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2366b, 90 },
    { 0x2522366b, 23 },
    { 0xf042366c, 34 },
    { 0xb0d2366b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f923674, 95 },
    { 0x6ff23675, 61 },
    { 0x3c123675, 33 },
    { 0x72423677, 87 },
    { 0x90023678, 61 },
    { 0, 0 },
    { 0x74e2367a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f2367e, 95 },
    { 0x36a2367f, 95 },
    { 0x162367f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd123684, 90 },
    { 0x6bf23685, 23 },
    { 0x19f23686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b23689, 77 },
    { 0x45e2368a, 33 },
    { 0, 0 },
    { 0x3b82368c, 23 },
    { 0x4de2368d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e323691, 46 },
    { 0xe0a23691, 75 },
    { 0, 0 },
    { 0x1af23694, 95 },
    { 0xf823695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf662369b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822369f, 66 },
    { 0xfe02369f, 38 },
    { 0x602236a1, 33 },
    { 0, 0 },
    { 0xe7c236a3, 33 },
    { 0x195236a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b236a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236ad, 23 },
    { 0x954236ae, 90 },
    { 0x2e8236ad, 98 },
    { 0, 0 },
    { 0xf2236b1, 61 },
    { 0, 0 },
    { 0x32c236b3, 97 },
    { 0xcfa236b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c236b8, 61 },
    { 0xdf3236b9, 95 },
    { 0x3e5236ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314236be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7236c4, 33 },
    { 0x5af236c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b236cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c236d0, 69 },
    { 0x871236d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b236d5, 33 },
    { 0x8a4236d6, 90 },
    { 0x69a236d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x534236da, 84 },
    { 0x741236da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7236e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a236e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309236ec, 90 },
    { 0xf86236ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88236f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77236f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14236fc, 46 },
    { 0, 0 },
    { 0xe80236fe, 48 },
    { 0xddc236fe, 100 },
    { 0, 0 },
    { 0x11d23701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c23704, 95 },
    { 0x10c23705, 99 },
    { 0, 0 },
    { 0xfa023707, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2370a, 51 },
    { 0x9a12370b, 43 },
    { 0x7112370c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8422370f, 90 },
    { 0x10c23710, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed223713, 90 },
    { 0, 0 },
    { 0x99f23715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b23719, 34 },
    { 0x57d2371a, 95 },
    { 0xdc62371b, 98 },
    { 0xfa22371c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772371f, 43 },
    { 0xd152371f, 36 },
    { 0xe5a2371f, 46 },
    { 0xd9a23722, 78 },
    { 0x8aa23722, 84 },
    { 0xb4d23722, 98 },
    { 0x45123725, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3323728, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672372e, 76 },
    { 0x9972372f, 67 },
    { 0, 0 },
    { 0x6c423731, 98 },
    { 0x5af23731, 43 },
    { 0xfed23731, 90 },
    { 0, 0 },
    { 0xec923735, 33 },
    { 0, 0 },
    { 0xe2223737, 99 },
    { 0x88c23738, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee82373e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c923744, 33 },
    { 0xd5623745, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d2374e, 95 },
    { 0, 0 },
    { 0xcb123750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86923754, 61 },
    { 0xbcb23754, 61 },
    { 0x3ef23756, 85 },
    { 0xfe023756, 77 },
    { 0xfdd23757, 61 },
    { 0, 0 },
    { 0xbdc2375a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2375e, 75 },
    { 0xda92375f, 33 },
    { 0, 0 },
    { 0xc0023761, 99 },
    { 0xf3223761, 23 },
    { 0x7a623763, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f023768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b22376e, 53 },
    { 0xfe22376f, 98 },
    { 0xc1623770, 78 },
    { 0xd072376e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b23774, 76 },
    { 0x2d123775, 33 },
    { 0xf9f23776, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7052377a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b23780, 78 },
    { 0, 0 },
    { 0x29923782, 95 },
    { 0x55623783, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec823787, 46 },
    { 0xb6d23788, 47 },
    { 0xc4a23789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47123791, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed23798, 33 },
    { 0x81b23799, 67 },
    { 0x5de2379a, 95 },
    { 0xabc2379a, 34 },
    { 0xba82379c, 33 },
    { 0x24323799, 91 },
    { 0x91c2379e, 33 },
    { 0, 0 },
    { 0x6c5237a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6237a6, 33 },
    { 0, 0 },
    { 0xc3f237a8, 55 },
    { 0x7237a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7237b3, 98 },
    { 0x681237b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80237b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d237bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64237c4, 84 },
    { 0xb45237c5, 66 },
    { 0x156237c6, 33 },
    { 0xa64237c7, 61 },
    { 0xe49237c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618237cc, 93 },
    { 0x730237cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2237d1, 93 },
    { 0x6cc237d2, 34 },
    { 0x2f2237d2, 98 },
    { 0xa27237d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81237d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6237dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6237df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f237e4, 77 },
    { 0, 0 },
    { 0xe05237e6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x418237e9, 76 },
    { 0xafd237e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792237ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a237f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7237f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc237fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca237ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40523803, 95 },
    { 0xcc23804, 98 },
    { 0xb8423803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb512380b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2380f, 95 },
    { 0xaf023810, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c23816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf23819, 17 },
    { 0x2ed2381a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f2381d, 90 },
    { 0, 0 },
    { 0xe522381f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d23822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb662382a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2382e, 64 },
    { 0xf452382e, 100 },
    { 0x39423830, 8 },
    { 0xac623831, 90 },
    { 0, 0 },
    { 0x4e223833, 33 },
    { 0, 0 },
    { 0x11523835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3823838, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2383d, 90 },
    { 0xe142383e, 43 },
    { 0x8a82383d, 33 },
    { 0, 0 },
    { 0xf3b23841, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23846, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf23849, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea2384c, 90 },
    { 0, 0 },
    { 0x1df2384e, 19 },
    { 0x4722384f, 93 },
    { 0, 0 },
    { 0x3423851, 67 },
    { 0x64823851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64123855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2385c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a23860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe23868, 90 },
    { 0x50723869, 33 },
    { 0xa3f2386a, 24 },
    { 0xa762386b, 58 },
    { 0xda62386c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc023870, 100 },
    { 0xdf323870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15523875, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12023879, 95 },
    { 0x492387a, 46 },
    { 0, 0 },
    { 0x9aa2387c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f23881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc223884, 34 },
    { 0, 0 },
    { 0x7bc23886, 95 },
    { 0x66c23887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca23891, 95 },
    { 0, 0 },
    { 0x94a23893, 61 },
    { 0x50023893, 90 },
    { 0xd923895, 93 },
    { 0x58a23895, 98 },
    { 0x40523897, 98 },
    { 0x8f123898, 78 },
    { 0, 0 },
    { 0x37f2389a, 23 },
    { 0x3f72389b, 43 },
    { 0, 0 },
    { 0x8f62389d, 23 },
    { 0xa7b2389e, 23 },
    { 0, 0 },
    { 0xdfc238a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x169238a3, 55 },
    { 0, 0 },
    { 0x30d238a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9238a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69238ab, 55 },
    { 0, 0 },
    { 0xee4238ad, 33 },
    { 0, 0 },
    { 0xf81238af, 61 },
    { 0, 0 },
    { 0xc57238b1, 61 },
    { 0xb56238b2, 95 },
    { 0, 0 },
    { 0x5a2238b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81238ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x745238bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5238c8, 93 },
    { 0, 0 },
    { 0x1e0238ca, 75 },
    { 0, 0 },
    { 0x10d238cc, 61 },
    { 0, 0 },
    { 0xfd9238ce, 67 },
    { 0xff5238cf, 90 },
    { 0x510238ce, 61 },
    { 0x491238cf, 43 },
    { 0xc15238d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69238da, 34 },
    { 0, 0 },
    { 0xe90238dc, 43 },
    { 0, 0 },
    { 0x51f238de, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651238e2, 90 },
    { 0xb65238e3, 90 },
    { 0x5fc238e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd238f3, 95 },
    { 0xc05238f3, 23 },
    { 0x386238f5, 33 },
    { 0, 0 },
    { 0x183238f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b238fb, 33 },
    { 0x75f238fb, 98 },
    { 0, 0 },
    { 0x5dd238fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca623902, 90 },
    { 0x77623903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf223907, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f2390d, 98 },
    { 0xbd22390e, 61 },
    { 0xfde2390f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f23916, 17 },
    { 0x85623917, 88 },
    { 0, 0 },
    { 0x5223919, 69 },
    { 0x5b32391a, 98 },
    { 0xa772391b, 92 },
    { 0x18a2391a, 87 },
    { 0x19023919, 33 },
    { 0x4f82391e, 76 },
    { 0x5b2391a, 90 },
    { 0xc7d23920, 98 },
    { 0x8c92391d, 98 },
    { 0, 0 },
    { 0xc1823923, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2392e, 61 },
    { 0, 0 },
    { 0xd3023930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22023934, 95 },
    { 0, 0 },
    { 0xba723936, 23 },
    { 0, 0 },
    { 0xc1223938, 55 },
    { 0x3a223939, 90 },
    { 0, 0 },
    { 0x62c2393b, 90 },
    { 0, 0 },
    { 0xe552393d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39223941, 61 },
    { 0x59223942, 76 },
    { 0x3d823941, 76 },
    { 0x41123942, 61 },
    { 0xed423944, 33 },
    { 0x1d623946, 95 },
    { 0x69023947, 95 },
    { 0x2d023947, 98 },
    { 0xb9e23949, 84 },
    { 0x54723941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d23952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c23957, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f2395e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd523961, 76 },
    { 0, 0 },
    { 0x69623963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d2396c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda123971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac623974, 84 },
    { 0xcc223975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2397a, 90 },
    { 0x4ab2397a, 99 },
    { 0, 0 },
    { 0xc8e2397d, 93 },
    { 0x7202397e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32223982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822398d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c23992, 43 },
    { 0xcc423993, 43 },
    { 0x1ea23994, 77 },
    { 0xe0623994, 23 },
    { 0x28e23996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb42399d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45239b0, 40 },
    { 0x616239b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae239b4, 23 },
    { 0x35f239b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f239bb, 84 },
    { 0xf5239bc, 93 },
    { 0, 0 },
    { 0xbe9239be, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed239cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a239d2, 85 },
    { 0x1e3239d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed239d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754239dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e239e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c239ec, 52 },
    { 0xfd239ed, 23 },
    { 0x5e6239ee, 33 },
    { 0x57b239ef, 61 },
    { 0x536239f0, 33 },
    { 0xdc1239f1, 95 },
    { 0x849239f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50239f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a239fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e239ff, 33 },
    { 0, 0 },
    { 0x9ed23a01, 90 },
    { 0xf0723a02, 91 },
    { 0x2fb23a01, 90 },
    { 0, 0 },
    { 0x37923a05, 99 },
    { 0, 0 },
    { 0xc3823a07, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f23a0e, 95 },
    { 0x61723a0f, 76 },
    { 0x97b23a0e, 78 },
    { 0x1bc23a10, 56 },
    { 0xa1a23a12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b23a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3523a1d, 98 },
    { 0x66123a1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9923a21, 81 },
    { 0x95223a21, 93 },
    { 0x65723a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa23a28, 33 },
    { 0x79c23a29, 90 },
    { 0xd5423a2a, 68 },
    { 0x19d23a2a, 98 },
    { 0xc8523a2b, 100 },
    { 0x8e223a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b23a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e823a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc023a3d, 98 },
    { 0x30623a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d23a42, 61 },
    { 0xb6a23a42, 64 },
    { 0, 0 },
    { 0xae823a45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc223a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5023a4d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e723a53, 99 },
    { 0x64923a54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe23a59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23a5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92323a67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8123a6b, 90 },
    { 0x84323a6c, 33 },
    { 0, 0 },
    { 0x67323a6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x96323a71, 64 },
    { 0xcd423a72, 100 },
    { 0x69823a72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e23a7c, 90 },
    { 0xef423a7d, 61 },
    { 0x1d023a7e, 33 },
    { 0xe5523a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9423a82, 23 },
    { 0x61b23a82, 52 },
    { 0xbf923a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30923a88, 93 },
    { 0x36023a89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac23a8f, 67 },
    { 0x8c423a8f, 68 },
    { 0xb5723a91, 86 },
    { 0x87323a92, 87 },
    { 0x13d23a93, 87 },
    { 0xdd523a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77123a97, 96 },
    { 0, 0 },
    { 0x5e723a99, 33 },
    { 0xb1f23a9a, 76 },
    { 0, 0 },
    { 0x2b423a9c, 55 },
    { 0xb5523a9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67523aa2, 43 },
    { 0, 0 },
    { 0xae423aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9423aa7, 61 },
    { 0xf8423aa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f23aab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a23aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e23abc, 76 },
    { 0x5b523abd, 84 },
    { 0xe2a23abe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c23ac4, 95 },
    { 0xa3023ac5, 34 },
    { 0, 0 },
    { 0x74723ac7, 52 },
    { 0xe5c23ac7, 95 },
    { 0, 0 },
    { 0xd1523aca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86523ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5723ada, 66 },
    { 0, 0 },
    { 0x8ff23adc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c823ae2, 66 },
    { 0xe8823ae3, 34 },
    { 0x7e523ae4, 87 },
    { 0, 0 },
    { 0x26d23ae6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a23ae9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52823aed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc23af0, 34 },
    { 0xa9623af1, 95 },
    { 0x12123af2, 77 },
    { 0, 0 },
    { 0x44423af4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d523afc, 46 },
    { 0x57823afc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc423b02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb923b05, 96 },
    { 0xde523b06, 10 },
    { 0x2c923b05, 34 },
    { 0x63223b05, 61 },
    { 0x2e423b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6823b0e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6323b11, 98 },
    { 0x89b23b12, 23 },
    { 0x33f23b11, 90 },
    { 0xdcc23b14, 88 },
    { 0x8923b12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a23b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc23b20, 90 },
    { 0xa6a23b20, 33 },
    { 0x34a23b20, 33 },
    { 0x1d023b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a323b27, 67 },
    { 0xc6123b28, 34 },
    { 0, 0 },
    { 0x69723b2a, 95 },
    { 0, 0 },
    { 0xbcf23b2c, 55 },
    { 0xbc323b2c, 43 },
    { 0x91523b2e, 90 },
    { 0, 0 },
    { 0x4323b30, 67 },
    { 0, 0 },
    { 0x21023b32, 43 },
    { 0xc3f23b32, 34 },
    { 0x3ae23b33, 61 },
    { 0x2bd23b35, 99 },
    { 0x17623b36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6823b39, 33 },
    { 0xb6123b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f323b3e, 43 },
    { 0xc3d23b3f, 95 },
    { 0xe1d23b40, 85 },
    { 0x6c223b3f, 33 },
    { 0x68523b42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca23b47, 61 },
    { 0x93023b47, 45 },
    { 0x8e023b49, 91 },
    { 0xb4b23b4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3923b4e, 61 },
    { 0, 0 },
    { 0xac223b50, 90 },
    { 0xecb23b51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae23b56, 90 },
    { 0, 0 },
    { 0x54e23b58, 99 },
    { 0xed23b59, 61 },
    { 0xa9f23b59, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87723b61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95223b66, 46 },
    { 0x7ae23b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99323b6a, 95 },
    { 0, 0 },
    { 0x5b723b6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c23b6f, 52 },
    { 0xcb423b6f, 90 },
    { 0x61223b70, 43 },
    { 0xfac23b72, 33 },
    { 0xc9f23b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf523b78, 23 },
    { 0xb1923b79, 98 },
    { 0xb2d23b7a, 23 },
    { 0xd7e23b78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d23b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8223b87, 55 },
    { 0x8c423b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23b90, 95 },
    { 0xff623b90, 90 },
    { 0x3f223b92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd023ba0, 33 },
    { 0x81223ba1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5823bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f23bb0, 95 },
    { 0xcad23bb0, 90 },
    { 0xc8223bb2, 83 },
    { 0xb0123bb2, 98 },
    { 0xff723bb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e123bb8, 75 },
    { 0, 0 },
    { 0x8e223bba, 78 },
    { 0xab323bbb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba23bbe, 95 },
    { 0, 0 },
    { 0x49323bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6523bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e23bc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x81023bcb, 12 },
    { 0, 0 },
    { 0x3a123bcd, 90 },
    { 0xf9c23bce, 33 },
    { 0x32623bcd, 75 },
    { 0xfc923bd0, 23 },
    { 0, 0 },
    { 0xcf523bd2, 85 },
    { 0xefa23bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf23bd6, 23 },
    { 0x5c323bd7, 95 },
    { 0xe4d23bd6, 90 },
    { 0xd6c23bd9, 61 },
    { 0, 0 },
    { 0x71923bdb, 85 },
    { 0xc9223bdc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd023bdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89323be3, 61 },
    { 0x4e23be4, 91 },
    { 0x32d23be5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb723bf0, 91 },
    { 0xb7223bf1, 93 },
    { 0x9a423bf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64423bf5, 85 },
    { 0x29d23bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c23bf9, 90 },
    { 0x3dc23bfa, 33 },
    { 0xc7c23bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f23bfe, 84 },
    { 0x48e23bff, 87 },
    { 0xd5923c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52023c06, 88 },
    { 0xf3723c07, 88 },
    { 0xdfc23c08, 61 },
    { 0x44c23c09, 23 },
    { 0xddb23c0a, 77 },
    { 0, 0 },
    { 0x96a23c0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d323c0f, 98 },
    { 0, 0 },
    { 0x60d23c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26023c18, 35 },
    { 0xaa923c19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae23c1e, 91 },
    { 0, 0 },
    { 0xee723c20, 90 },
    { 0x12323c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa723c25, 23 },
    { 0x58523c25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34823c33, 78 },
    { 0x61623c34, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68223c38, 95 },
    { 0x43a23c38, 76 },
    { 0, 0 },
    { 0x1cb23c3b, 52 },
    { 0xc1823c3c, 33 },
    { 0x96f23c3c, 55 },
    { 0, 0 },
    { 0x5fc23c3f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef023c43, 98 },
    { 0xc9923c44, 90 },
    { 0, 0 },
    { 0x87923c46, 46 },
    { 0x7d223c46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f23c4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad23c4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f23c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3023c5c, 99 },
    { 0, 0 },
    { 0xaa523c5e, 90 },
    { 0xf3d23c5e, 34 },
    { 0xb5123c60, 98 },
    { 0, 0 },
    { 0x3b423c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c523c6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26023c6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb23c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92723c73, 33 },
    { 0, 0 },
    { 0x1d423c75, 90 },
    { 0xbf323c76, 61 },
    { 0, 0 },
    { 0x74a23c78, 33 },
    { 0, 0 },
    { 0x5d823c7a, 98 },
    { 0xba923c7b, 33 },
    { 0x5fb23c7c, 90 },
    { 0, 0 },
    { 0x4ad23c7e, 33 },
    { 0x44223c7f, 98 },
    { 0xbe023c80, 84 },
    { 0xeec23c81, 77 },
    { 0xce723c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f23c85, 95 },
    { 0x9323c86, 76 },
    { 0xab23c87, 90 },
    { 0x5f23c86, 78 },
    { 0x37623c88, 33 },
    { 0xb0623c8a, 55 },
    { 0xa9023c8b, 100 },
    { 0xe6723c8a, 85 },
    { 0, 0 },
    { 0x8e23c8e, 76 },
    { 0x5f723c8f, 78 },
    { 0, 0 },
    { 0x16c23c91, 90 },
    { 0, 0 },
    { 0xdce23c93, 90 },
    { 0x54e23c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd23c98, 23 },
    { 0x41423c98, 90 },
    { 0, 0 },
    { 0x3ff23c9b, 61 },
    { 0xa3a23c9b, 91 },
    { 0x46923c9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb623ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c23ca5, 93 },
    { 0x6c23ca6, 78 },
    { 0x44323ca7, 64 },
    { 0x5d123ca7, 87 },
    { 0xad823ca9, 91 },
    { 0, 0 },
    { 0xbbc23cab, 52 },
    { 0x1b723cac, 90 },
    { 0x6c223cad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12723cb0, 84 },
    { 0xdb623cb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40023cb7, 78 },
    { 0x5bb23cb7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26923cbc, 33 },
    { 0x3e523cbc, 40 },
    { 0, 0 },
    { 0x22823cbf, 95 },
    { 0, 0 },
    { 0xd1323cc1, 90 },
    { 0, 0 },
    { 0x88423cc3, 23 },
    { 0, 0 },
    { 0x23c23cc5, 90 },
    { 0, 0 },
    { 0x63623cc7, 43 },
    { 0x37a23cc8, 90 },
    { 0x97823cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37823ccf, 33 },
    { 0x98623cd0, 43 },
    { 0x40223cd1, 90 },
    { 0x82a23cd2, 87 },
    { 0xa0323cd3, 43 },
    { 0x52f23cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7923cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92423cdd, 98 },
    { 0x35d23cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d723ce3, 46 },
    { 0x80823ce3, 23 },
    { 0x27423ce5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1323ce9, 75 },
    { 0, 0 },
    { 0xfee23ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4223cef, 65 },
    { 0xd5d23cf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5123cf3, 66 },
    { 0, 0 },
    { 0x4e623cf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed123cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7523cfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba623cfe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9223d01, 90 },
    { 0xeb923d02, 33 },
    { 0x83b23d02, 55 },
    { 0x56123d04, 95 },
    { 0x48123d05, 78 },
    { 0x8f023d06, 85 },
    { 0, 0 },
    { 0x62923d08, 98 },
    { 0x9fe23d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed923d0d, 78 },
    { 0xf6d23d0e, 80 },
    { 0, 0 },
    { 0x6d723d10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb323d14, 66 },
    { 0, 0 },
    { 0xa4423d16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef23d1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12023d23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5823d26, 40 },
    { 0x8c523d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7223d2b, 34 },
    { 0x21f23d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26123d2f, 91 },
    { 0x68623d30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56423d36, 23 },
    { 0xc5d23d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec23d3b, 98 },
    { 0xf9823d3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf23d42, 91 },
    { 0x323d42, 66 },
    { 0x58123d43, 95 },
    { 0x44b23d45, 93 },
    { 0x57d23d46, 40 },
    { 0x6fd23d42, 76 },
    { 0, 0 },
    { 0x8cc23d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de23d55, 23 },
    { 0xf5c23d55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f23d5d, 98 },
    { 0xc7323d5e, 93 },
    { 0, 0 },
    { 0xf5a23d60, 98 },
    { 0x89c23d61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec23d64, 95 },
    { 0x7e323d65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b23d6b, 90 },
    { 0x95b23d6b, 78 },
    { 0x64b23d6b, 23 },
    { 0x3b323d6d, 76 },
    { 0x3a323d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0423d72, 91 },
    { 0x5b523d73, 85 },
    { 0xc3923d73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d523d77, 98 },
    { 0x72a23d78, 55 },
    { 0xf3523d78, 91 },
    { 0, 0 },
    { 0x7ea23d7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f023d83, 55 },
    { 0xe0423d83, 23 },
    { 0, 0 },
    { 0x6b423d86, 93 },
    { 0, 0 },
    { 0x4bf23d88, 63 },
    { 0, 0 },
    { 0xfdc23d8a, 55 },
    { 0, 0 },
    { 0xf7f23d8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0523d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d23d98, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c23d9b, 90 },
    { 0x65523d9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e523d9f, 34 },
    { 0x26c23da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc23da4, 95 },
    { 0x17223da5, 33 },
    { 0xad123da6, 91 },
    { 0x33623da7, 23 },
    { 0x73723da7, 61 },
    { 0xa623da8, 43 },
    { 0xca23da8, 55 },
    { 0xcae23da5, 33 },
    { 0xd2d23da9, 33 },
    { 0xa123dad, 95 },
    { 0x19523da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x11323db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c823db7, 68 },
    { 0, 0 },
    { 0x13523db9, 33 },
    { 0xee023db9, 99 },
    { 0xef723db9, 76 },
    { 0x4b323dba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66823dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0723dc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c23dca, 98 },
    { 0xb4e23dcb, 46 },
    { 0, 0 },
    { 0xf1123dcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a23ddf, 90 },
    { 0xb3e23de0, 33 },
    { 0xe3323de1, 61 },
    { 0x53e23de0, 34 },
    { 0xccc23ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c23de6, 93 },
    { 0x6123de7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8123dee, 33 },
    { 0, 0 },
    { 0x35123df0, 87 },
    { 0x3c723df1, 98 },
    { 0x91e23df2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6123df6, 90 },
    { 0, 0 },
    { 0xe1b23df8, 98 },
    { 0xa2423df8, 95 },
    { 0x3223dfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a23dfd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd323e01, 23 },
    { 0xc6423e02, 34 },
    { 0xacc23e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4723e0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da23e0d, 90 },
    { 0xa7623e0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8223e11, 66 },
    { 0, 0 },
    { 0x1723e13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec23e18, 52 },
    { 0, 0 },
    { 0xc8e23e1a, 66 },
    { 0xd2823e1b, 98 },
    { 0x25523e1a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29823e20, 65 },
    { 0x7d823e21, 75 },
    { 0, 0 },
    { 0x84923e23, 43 },
    { 0, 0 },
    { 0x4da23e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe23e30, 76 },
    { 0xdeb23e31, 55 },
    { 0xb6a23e30, 95 },
    { 0xb5e23e33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b923e3b, 96 },
    { 0xe2623e3c, 76 },
    { 0xa2223e3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d23e41, 43 },
    { 0x62c23e42, 90 },
    { 0xaff23e43, 61 },
    { 0xb3623e43, 52 },
    { 0xa0e23e44, 99 },
    { 0x50b23e41, 90 },
    { 0x78d23e46, 23 },
    { 0, 0 },
    { 0x4d123e49, 55 },
    { 0x13e23e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3823e4d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x36023e50, 95 },
    { 0x80923e50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a623e56, 33 },
    { 0x38f23e57, 46 },
    { 0, 0 },
    { 0x92123e59, 75 },
    { 0x1e023e59, 90 },
    { 0x38323e59, 93 },
    { 0, 0 },
    { 0x68223e5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64323e60, 61 },
    { 0xc6e23e61, 90 },
    { 0xba323e62, 33 },
    { 0x9d223e60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28123e67, 75 },
    { 0, 0 },
    { 0xff123e69, 61 },
    { 0xe3c23e6a, 23 },
    { 0x84223e6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a523e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9223e76, 46 },
    { 0x46f23e77, 95 },
    { 0xc6623e76, 23 },
    { 0xc0b23e76, 76 },
    { 0xbcc23e7a, 34 },
    { 0, 0 },
    { 0x9d623e7c, 34 },
    { 0xfbe23e7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c423e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7423e88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae23e8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2423e90, 90 },
    { 0, 0 },
    { 0x8923e92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6223ea0, 61 },
    { 0x9c023ea1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f23ea8, 90 },
    { 0xe1d23ea9, 95 },
    { 0x5f323eaa, 33 },
    { 0xb2e23eab, 33 },
    { 0x9dd23ea9, 33 },
    { 0xfd723ead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd623eb0, 52 },
    { 0, 0 },
    { 0x85a23eb2, 78 },
    { 0, 0 },
    { 0x47323eb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b23eb7, 45 },
    { 0x7bf23eb8, 55 },
    { 0xd4723eb9, 61 },
    { 0x75d23eb8, 98 },
    { 0x14123ebb, 61 },
    { 0, 0 },
    { 0xe2123ebd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a423ec0, 91 },
    { 0xcdf23ec1, 61 },
    { 0x10923ec1, 61 },
    { 0, 0 },
    { 0xa8523ec4, 66 },
    { 0x98323ec5, 55 },
    { 0xaf523ec6, 61 },
    { 0, 0 },
    { 0xf0423ec8, 34 },
    { 0, 0 },
    { 0xdb823eca, 61 },
    { 0x1e723ecb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f23ed3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd223ed7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18123ee7, 23 },
    { 0, 0 },
    { 0xaa723ee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5623eef, 99 },
    { 0xc2a23ef0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b23ef5, 55 },
    { 0x50323ef6, 95 },
    { 0xa8123ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6623efc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae23eff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63323f06, 100 },
    { 0x65323f07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd23f0b, 43 },
    { 0, 0 },
    { 0xd6b23f0d, 90 },
    { 0x1ec23f0d, 33 },
    { 0xfa223f0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41923f13, 64 },
    { 0x21623f14, 90 },
    { 0x95b23f14, 84 },
    { 0, 0 },
    { 0xd7e23f17, 34 },
    { 0xb9323f17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4923f1d, 90 },
    { 0x8a223f1d, 76 },
    { 0x41423f1e, 98 },
    { 0x27b23f20, 98 },
    { 0xd4023f20, 93 },
    { 0, 0 },
    { 0xc5723f23, 43 },
    { 0x25723f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a123f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34023f2b, 76 },
    { 0x81323f2c, 46 },
    { 0xf0623f2d, 61 },
    { 0, 0 },
    { 0xdcf23f2f, 33 },
    { 0xb6523f30, 61 },
    { 0x10723f30, 93 },
    { 0, 0 },
    { 0x5c923f33, 23 },
    { 0, 0 },
    { 0x37823f35, 75 },
    { 0, 0 },
    { 0x73523f37, 66 },
    { 0x44223f38, 61 },
    { 0x8e623f38, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x19623f3c, 76 },
    { 0x91923f3c, 23 },
    { 0x71623f3e, 41 },
    { 0, 0 },
    { 0x63723f40, 43 },
    { 0xfe323f40, 90 },
    { 0x2b323f42, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4723f46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c223f49, 46 },
    { 0xa2f23f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f423f51, 33 },
    { 0, 0 },
    { 0x12623f53, 98 },
    { 0x6a23f54, 95 },
    { 0xd7a23f54, 56 },
    { 0x37e23f54, 84 },
    { 0, 0 },
    { 0x4b023f58, 68 },
    { 0, 0 },
    { 0x93323f5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4623f5d, 23 },
    { 0, 0 },
    { 0x6d123f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd23f62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f23f66, 93 },
    { 0, 0 },
    { 0xe0923f68, 33 },
    { 0x2b123f68, 33 },
    { 0, 0 },
    { 0x32e23f6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4623f6e, 34 },
    { 0, 0 },
    { 0x6ee23f70, 23 },
    { 0x57723f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed23f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe123f7a, 98 },
    { 0x52323f7a, 55 },
    { 0xed723f7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a123f8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e23f94, 90 },
    { 0x29123f95, 17 },
    { 0x7a223f94, 67 },
    { 0x44f23f97, 95 },
    { 0, 0 },
    { 0x88723f99, 55 },
    { 0xba923f9a, 90 },
    { 0x2a523f9b, 98 },
    { 0x39b23f99, 100 },
    { 0xf6723f9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f123fa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf623fa7, 90 },
    { 0x9b323fa8, 50 },
    { 0, 0 },
    { 0xd4223faa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb23fae, 33 },
    { 0x8ce23faf, 33 },
    { 0xc3b23fae, 23 },
    { 0xedc23fb1, 98 },
    { 0x7e623fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b423fb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef923fb8, 90 },
    { 0, 0 },
    { 0xaa423fba, 61 },
    { 0, 0 },
    { 0x7f223fbc, 88 },
    { 0x4323fbc, 90 },
    { 0x54e23fbc, 98 },
    { 0x47923fbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1723fc6, 90 },
    { 0, 0 },
    { 0x74723fc8, 90 },
    { 0, 0 },
    { 0xc8b23fca, 61 },
    { 0, 0 },
    { 0x3f823fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df23fd2, 98 },
    { 0, 0 },
    { 0x3523fd4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc23fd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a23fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7723fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae223fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35423ff7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77723ffb, 34 },
    { 0x2bc23ffb, 95 },
    { 0, 0 },
    { 0xe8023ffe, 85 },
    { 0x8ee23ffe, 91 },
    { 0x43124000, 23 },
    { 0x8e224000, 78 },
    { 0xe8d23ffe, 98 },
    { 0xb1124003, 61 },
    { 0x67e24004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c124009, 23 },
    { 0xc892400a, 77 },
    { 0xfa924009, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62400f, 96 },
    { 0x63924010, 76 },
    { 0x8f224010, 82 },
    { 0x1a624012, 95 },
    { 0x4e624010, 76 },
    { 0xe9724014, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d024019, 23 },
    { 0x2562401a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a24021, 23 },
    { 0x30824021, 77 },
    { 0x69624023, 76 },
    { 0, 0 },
    { 0x2ae24025, 93 },
    { 0x12124026, 61 },
    { 0x1924026, 33 },
    { 0xf2c24026, 23 },
    { 0x7c024029, 98 },
    { 0x29224027, 23 },
    { 0x6b724028, 95 },
    { 0x9542402c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8124030, 33 },
    { 0x85924030, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c24038, 95 },
    { 0xe1924039, 81 },
    { 0, 0 },
    { 0x7652403b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7524040, 58 },
    { 0x87c24040, 90 },
    { 0, 0 },
    { 0x2824043, 33 },
    { 0x1af24043, 85 },
    { 0xb9524044, 90 },
    { 0xe3624044, 98 },
    { 0xd7024047, 90 },
    { 0x62824048, 90 },
    { 0xe1a24048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12404c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc92404f, 91 },
    { 0xa462404f, 100 },
    { 0x4d424051, 94 },
    { 0xfc824052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e024056, 34 },
    { 0x8d924057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf832405a, 95 },
    { 0, 0 },
    { 0x93b2405c, 38 },
    { 0xdce2405c, 90 },
    { 0x1652405e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8224064, 93 },
    { 0x3a24065, 99 },
    { 0, 0 },
    { 0xfd24067, 95 },
    { 0xd4d24068, 87 },
    { 0x58524069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28624070, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df24073, 23 },
    { 0, 0 },
    { 0x72b24075, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda624078, 95 },
    { 0, 0 },
    { 0x5072407a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce2407e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca24081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a24086, 90 },
    { 0, 0 },
    { 0x3c524088, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf282408d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79124091, 95 },
    { 0xf8124092, 90 },
    { 0xace24093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c424099, 33 },
    { 0x28b2409a, 33 },
    { 0x2262409b, 34 },
    { 0, 0 },
    { 0xdb42409d, 33 },
    { 0xf612409e, 88 },
    { 0x7b62409f, 23 },
    { 0xbfa240a0, 76 },
    { 0xb8f240a1, 66 },
    { 0xfbc240a1, 90 },
    { 0xcb4240a2, 23 },
    { 0xc3c240a4, 33 },
    { 0x6c0240a5, 52 },
    { 0, 0 },
    { 0x5e4240a7, 98 },
    { 0x36240a7, 33 },
    { 0x1a3240a9, 77 },
    { 0xab9240a9, 47 },
    { 0x8d2240ab, 99 },
    { 0xfb1240ac, 23 },
    { 0x15f240ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61240b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d240bc, 90 },
    { 0, 0 },
    { 0xab6240be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746240c8, 23 },
    { 0, 0 },
    { 0xe66240ca, 98 },
    { 0xb9b240cb, 34 },
    { 0xbfe240cb, 69 },
    { 0, 0 },
    { 0x64d240ce, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd240d1, 33 },
    { 0xa51240d1, 33 },
    { 0x3e5240d3, 98 },
    { 0, 0 },
    { 0x795240d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a240d8, 95 },
    { 0x36240d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5240dc, 95 },
    { 0x509240dd, 78 },
    { 0x92e240de, 88 },
    { 0x40f240df, 34 },
    { 0xa9240df, 80 },
    { 0x8cc240e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a240e5, 90 },
    { 0x5bf240e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9240e9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x169240ec, 61 },
    { 0x338240ed, 95 },
    { 0x3b9240ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08240f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871240f7, 34 },
    { 0, 0 },
    { 0x23c240f9, 52 },
    { 0x9e0240fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529240fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee224101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3524104, 17 },
    { 0x5ab24104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2410a, 78 },
    { 0x9152410a, 55 },
    { 0, 0 },
    { 0x93d2410d, 90 },
    { 0xf7c2410e, 17 },
    { 0x6562410f, 55 },
    { 0xfdc24110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f24113, 23 },
    { 0, 0 },
    { 0x59324115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe24119, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e92411f, 90 },
    { 0xccf2411f, 95 },
    { 0, 0 },
    { 0x3bc24122, 61 },
    { 0xe3924123, 98 },
    { 0, 0 },
    { 0x33b24125, 61 },
    { 0x3ca24126, 98 },
    { 0x24924127, 23 },
    { 0xb6124128, 90 },
    { 0xb8024129, 95 },
    { 0, 0 },
    { 0xd052412b, 50 },
    { 0x2f52412b, 45 },
    { 0x6ec2412d, 55 },
    { 0, 0 },
    { 0x1a02412f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d24137, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb92413b, 76 },
    { 0, 0 },
    { 0x62d2413d, 80 },
    { 0xac62413e, 43 },
    { 0x2732413f, 33 },
    { 0xcf024140, 85 },
    { 0x6de24141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e24145, 95 },
    { 0, 0 },
    { 0x6dd24147, 33 },
    { 0, 0 },
    { 0x6924149, 98 },
    { 0, 0 },
    { 0xce62414b, 98 },
    { 0, 0 },
    { 0x6532414d, 75 },
    { 0xd3e2414e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e24151, 67 },
    { 0xc9a24152, 34 },
    { 0, 0 },
    { 0x1224154, 100 },
    { 0, 0 },
    { 0x58124156, 90 },
    { 0, 0 },
    { 0x90824158, 23 },
    { 0x21b24159, 76 },
    { 0x85c2415a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2415e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba24161, 98 },
    { 0x66124162, 56 },
    { 0xabf24162, 67 },
    { 0x34d24164, 43 },
    { 0x8f124164, 95 },
    { 0x97324165, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4424169, 78 },
    { 0, 0 },
    { 0x26d2416b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63224179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6442417c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24183, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2418e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea24192, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8424198, 52 },
    { 0xe5124199, 90 },
    { 0xfcb24198, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c2419f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f241a9, 76 },
    { 0x745241a9, 99 },
    { 0x360241ab, 95 },
    { 0xcee241ab, 23 },
    { 0x14e241ad, 34 },
    { 0x317241ab, 17 },
    { 0x1d1241aa, 90 },
    { 0x4cc241b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x454241b3, 98 },
    { 0x9e5241b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9241b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0241bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4241c0, 61 },
    { 0xfb6241c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa241c4, 61 },
    { 0xb1d241c5, 23 },
    { 0, 0 },
    { 0x9f4241c7, 75 },
    { 0xe7241c7, 33 },
    { 0, 0 },
    { 0xf4a241ca, 65 },
    { 0x492241cb, 23 },
    { 0x805241cb, 90 },
    { 0x44f241ca, 61 },
    { 0, 0 },
    { 0x8b6241cf, 90 },
    { 0x1b6241d0, 61 },
    { 0xbdd241d1, 80 },
    { 0xacd241d1, 34 },
    { 0, 0 },
    { 0xe26241d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94241d7, 55 },
    { 0x1a6241d8, 76 },
    { 0, 0 },
    { 0x39e241da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1241dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e241e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6241e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e241ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39241f0, 34 },
    { 0, 0 },
    { 0x31c241f2, 99 },
    { 0xa90241f2, 95 },
    { 0x274241f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2241f8, 98 },
    { 0x54a241f9, 33 },
    { 0x4a2241f9, 91 },
    { 0xdb5241f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad24206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac2420c, 61 },
    { 0, 0 },
    { 0x8652420e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3224211, 52 },
    { 0x96024212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82324216, 33 },
    { 0xf6c24216, 33 },
    { 0, 0 },
    { 0x26124219, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2421c, 90 },
    { 0, 0 },
    { 0x5c82421e, 61 },
    { 0x8392421e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f24223, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2422a, 33 },
    { 0, 0 },
    { 0x7322422c, 61 },
    { 0, 0 },
    { 0xd9d2422e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66724235, 61 },
    { 0xe2324235, 90 },
    { 0, 0 },
    { 0x31e24238, 95 },
    { 0xffa24239, 55 },
    { 0xbea24239, 43 },
    { 0x43024239, 98 },
    { 0xf522423a, 98 },
    { 0xdea2423c, 46 },
    { 0x16f24238, 90 },
    { 0, 0 },
    { 0xa2a24240, 61 },
    { 0, 0 },
    { 0x7a524242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb924246, 61 },
    { 0, 0 },
    { 0x80024248, 33 },
    { 0x27c24249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82424e, 33 },
    { 0x4ee2424f, 98 },
    { 0x93c24250, 76 },
    { 0x1872424e, 40 },
    { 0x6a924252, 34 },
    { 0, 0 },
    { 0xf2b24254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a2425e, 68 },
    { 0, 0 },
    { 0x5a924260, 46 },
    { 0, 0 },
    { 0x64524262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde124268, 95 },
    { 0, 0 },
    { 0xec22426a, 61 },
    { 0xfa92426b, 66 },
    { 0x18f2426c, 17 },
    { 0xcf82426d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412427a, 40 },
    { 0, 0 },
    { 0x1a62427c, 34 },
    { 0, 0 },
    { 0xa0d2427e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44024286, 46 },
    { 0xff424287, 90 },
    { 0xa6524287, 61 },
    { 0x10524286, 61 },
    { 0xcd24289, 90 },
    { 0x44b24287, 43 },
    { 0, 0 },
    { 0x4372428d, 90 },
    { 0x5802428e, 98 },
    { 0x2c42428f, 33 },
    { 0x3292428f, 91 },
    { 0xaa524291, 98 },
    { 0xf7824292, 87 },
    { 0xe2224293, 87 },
    { 0x7de24291, 84 },
    { 0, 0 },
    { 0xb8224296, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262429c, 76 },
    { 0, 0 },
    { 0x41a2429e, 90 },
    { 0xf282429f, 85 },
    { 0xc5f2429f, 23 },
    { 0x3d9242a1, 85 },
    { 0xc91242a2, 84 },
    { 0xafc242a2, 98 },
    { 0, 0 },
    { 0xf87242a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e242a9, 61 },
    { 0, 0 },
    { 0x68d242ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1242b1, 98 },
    { 0xa3242b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b242b5, 90 },
    { 0x701242b5, 99 },
    { 0xd11242b6, 98 },
    { 0x551242b8, 98 },
    { 0x48b242b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5242bc, 61 },
    { 0, 0 },
    { 0xcda242be, 55 },
    { 0xcab242be, 78 },
    { 0xfaa242c0, 100 },
    { 0x742242c1, 33 },
    { 0, 0 },
    { 0xa4d242c3, 61 },
    { 0, 0 },
    { 0xa0e242c5, 55 },
    { 0x17f242c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda242cb, 33 },
    { 0x29b242cc, 98 },
    { 0x42c242cc, 66 },
    { 0x704242ce, 43 },
    { 0x1e5242cb, 88 },
    { 0, 0 },
    { 0x6c2242d1, 81 },
    { 0x9cd242d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75242d9, 23 },
    { 0x7f7242d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c242de, 33 },
    { 0, 0 },
    { 0xc58242e0, 76 },
    { 0, 0 },
    { 0xa5c242e2, 34 },
    { 0xeea242e3, 90 },
    { 0, 0 },
    { 0x873242e5, 38 },
    { 0xd08242e6, 43 },
    { 0, 0 },
    { 0x41d242e8, 55 },
    { 0, 0 },
    { 0x452242ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0242ed, 33 },
    { 0, 0 },
    { 0x40d242ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0242f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x282242f8, 33 },
    { 0, 0 },
    { 0x97c242fa, 98 },
    { 0x4ba242fb, 87 },
    { 0xe1d242fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424303, 78 },
    { 0, 0 },
    { 0xba924305, 90 },
    { 0x10d24306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2430a, 98 },
    { 0, 0 },
    { 0xd512430c, 43 },
    { 0x10d2430d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef024311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3424317, 98 },
    { 0xc1824317, 65 },
    { 0x11524319, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2431c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19924322, 61 },
    { 0x6f124322, 40 },
    { 0xc9f24323, 76 },
    { 0xb924325, 55 },
    { 0, 0 },
    { 0x18424327, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c2432c, 100 },
    { 0xcca2432d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab24333, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa824336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212433b, 78 },
    { 0xc5a2433c, 93 },
    { 0x1d32433d, 61 },
    { 0, 0 },
    { 0x1c52433f, 75 },
    { 0x70424340, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1124345, 23 },
    { 0x2ad24346, 55 },
    { 0x95b24345, 33 },
    { 0x48924348, 61 },
    { 0x51e24346, 23 },
    { 0xdbd24345, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9742434d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc24350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a24355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2435e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4024361, 36 },
    { 0xcf424361, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b324366, 23 },
    { 0x24024367, 35 },
    { 0x2d924368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4332436c, 23 },
    { 0xa972436d, 76 },
    { 0x2dd2436e, 33 },
    { 0xf82436f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c24373, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b82437a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb24381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb24384, 95 },
    { 0, 0 },
    { 0x9ec24386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a2438a, 83 },
    { 0xaa32438b, 90 },
    { 0, 0 },
    { 0xe002438d, 78 },
    { 0x4bf2438d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c24394, 95 },
    { 0x64024394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a724399, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe2439c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1243a1, 33 },
    { 0x2f243a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f243a5, 91 },
    { 0, 0 },
    { 0x9eb243a7, 100 },
    { 0, 0 },
    { 0x1f8243a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736243ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0243c1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4243c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b243c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e243ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x787243cd, 55 },
    { 0, 0 },
    { 0x396243cf, 90 },
    { 0, 0 },
    { 0xc66243d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a243d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855243df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0243ea, 95 },
    { 0xbda243eb, 88 },
    { 0x37f243ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82243f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8243f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba243fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b24404, 81 },
    { 0, 0 },
    { 0x9a124406, 98 },
    { 0x99224407, 67 },
    { 0, 0 },
    { 0xc2924409, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f62440d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324411, 90 },
    { 0, 0 },
    { 0x88e24413, 90 },
    { 0, 0 },
    { 0xfd024415, 100 },
    { 0, 0 },
    { 0x2124417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442441b, 88 },
    { 0, 0 },
    { 0xeeb2441d, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f24422, 61 },
    { 0x1c424422, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf124427, 76 },
    { 0xdb824428, 90 },
    { 0, 0 },
    { 0xc6a2442a, 90 },
    { 0x9822442b, 55 },
    { 0x79d2442c, 33 },
    { 0x8862442d, 88 },
    { 0x27f2442e, 33 },
    { 0x19d2442f, 90 },
    { 0x5b24430, 85 },
    { 0x45c24431, 93 },
    { 0x162442b, 98 },
    { 0xb5a2442e, 55 },
    { 0x30224434, 69 },
    { 0, 0 },
    { 0x35024436, 61 },
    { 0xfb224437, 23 },
    { 0, 0 },
    { 0xd0824439, 55 },
    { 0, 0 },
    { 0xb292443b, 61 },
    { 0xc102443c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b624448, 61 },
    { 0x57724449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e24450, 95 },
    { 0xd7424451, 75 },
    { 0x3b324452, 23 },
    { 0x9ac24453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c24457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32445a, 95 },
    { 0x4242445b, 87 },
    { 0, 0 },
    { 0xb1b2445d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1624462, 78 },
    { 0xbda24462, 99 },
    { 0, 0 },
    { 0x61124465, 68 },
    { 0, 0 },
    { 0x2c924467, 85 },
    { 0xd1424468, 34 },
    { 0x16724469, 23 },
    { 0x8da2446a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea24471, 34 },
    { 0xcb624471, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa2447b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70524482, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x61524485, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a124488, 98 },
    { 0x35524489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca924493, 76 },
    { 0, 0 },
    { 0xb8c24495, 33 },
    { 0xe9324496, 88 },
    { 0xe8124497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a82449b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb92449e, 33 },
    { 0x1c62449e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6244a8, 23 },
    { 0xb50244a9, 23 },
    { 0, 0 },
    { 0xf77244ab, 61 },
    { 0xfa9244ac, 65 },
    { 0x2dd244ac, 33 },
    { 0x56a244ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3244b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e244b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782244b8, 61 },
    { 0x56e244b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1244be, 100 },
    { 0x3be244bf, 98 },
    { 0x880244c0, 33 },
    { 0, 0 },
    { 0x583244c2, 76 },
    { 0x18c244c3, 56 },
    { 0x616244c4, 93 },
    { 0x278244c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8244ca, 40 },
    { 0x8e3244cb, 23 },
    { 0xaae244cc, 91 },
    { 0x55f244cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3244d2, 61 },
    { 0xfff244d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e244d9, 78 },
    { 0x327244d9, 91 },
    { 0, 0 },
    { 0x230244dc, 76 },
    { 0x900244dd, 23 },
    { 0, 0 },
    { 0x8ef244df, 55 },
    { 0x2e0244e0, 23 },
    { 0xd8244e1, 90 },
    { 0x6dc244e2, 93 },
    { 0xf10244e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83244e6, 61 },
    { 0x2cd244e7, 90 },
    { 0x7dd244e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77244ee, 47 },
    { 0xc90244ee, 90 },
    { 0, 0 },
    { 0x29b244f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555244f8, 78 },
    { 0x748244f8, 33 },
    { 0x977244fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82244fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15824501, 33 },
    { 0x91724502, 81 },
    { 0x1a424503, 38 },
    { 0x6e924502, 46 },
    { 0x80a24503, 23 },
    { 0x4ff24506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c52450f, 23 },
    { 0x99224510, 61 },
    { 0x56624510, 76 },
    { 0x91624512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7424515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f12451d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe224523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4232452b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42452e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f24532, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e24536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832453a, 23 },
    { 0xae12453b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85524542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c24545, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7272454e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30624552, 61 },
    { 0xa9824553, 44 },
    { 0, 0 },
    { 0x624555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1472455e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80624562, 92 },
    { 0x3ec24562, 33 },
    { 0, 0 },
    { 0x9624565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a224568, 34 },
    { 0x88724569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd2456e, 90 },
    { 0, 0 },
    { 0xe7e24570, 95 },
    { 0, 0 },
    { 0xabe24572, 93 },
    { 0xc824573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c24576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e2457f, 88 },
    { 0x7b124580, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0924586, 23 },
    { 0x29024587, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2458a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9312458e, 90 },
    { 0x5262458e, 85 },
    { 0xacf2458e, 98 },
    { 0x10524591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae24598, 90 },
    { 0x8c924599, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b2459c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf2459f, 17 },
    { 0, 0 },
    { 0x805245a1, 46 },
    { 0, 0 },
    { 0xc8245a3, 34 },
    { 0x22e245a4, 76 },
    { 0x593245a4, 76 },
    { 0x8a3245a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369245b1, 43 },
    { 0xd08245b2, 23 },
    { 0x76245b3, 17 },
    { 0x654245b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2245b9, 96 },
    { 0, 0 },
    { 0x7e6245bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96245be, 43 },
    { 0x72d245bf, 55 },
    { 0xd1c245bf, 55 },
    { 0x71c245c1, 52 },
    { 0, 0 },
    { 0xec6245c3, 93 },
    { 0xff2245c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388245cf, 99 },
    { 0xac5245d0, 85 },
    { 0xd1d245d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28245d8, 61 },
    { 0x2af245d9, 90 },
    { 0, 0 },
    { 0xf00245db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3245de, 61 },
    { 0x9b5245df, 33 },
    { 0x4fd245df, 75 },
    { 0x753245de, 23 },
    { 0xe87245e1, 25 },
    { 0x9b9245e3, 23 },
    { 0xacb245e4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x320245e7, 90 },
    { 0xa63245e7, 61 },
    { 0, 0 },
    { 0x9e6245ea, 87 },
    { 0x3f3245ea, 98 },
    { 0xa1a245ea, 23 },
    { 0x6d8245ea, 95 },
    { 0x307245ee, 23 },
    { 0, 0 },
    { 0x444245f0, 88 },
    { 0x3cc245f0, 33 },
    { 0, 0 },
    { 0x75f245f3, 35 },
    { 0x7b9245f4, 78 },
    { 0, 0 },
    { 0x354245f6, 56 },
    { 0xe99245f7, 91 },
    { 0x79a245f7, 76 },
    { 0x328245f9, 76 },
    { 0x8bf245f7, 90 },
    { 0xaf1245f8, 55 },
    { 0xec245fc, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7924602, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d724608, 33 },
    { 0xdd024608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed52460f, 90 },
    { 0, 0 },
    { 0xeb624611, 33 },
    { 0x64224612, 98 },
    { 0x95824613, 76 },
    { 0xc3a24614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd2461f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33624622, 76 },
    { 0x42c24623, 97 },
    { 0x52124624, 61 },
    { 0x3e924624, 95 },
    { 0x7ba24626, 33 },
    { 0x3ed24624, 34 },
    { 0xfa24626, 61 },
    { 0x24c24623, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2462d, 55 },
    { 0x79e2462d, 76 },
    { 0x23c2462f, 78 },
    { 0x95e2462f, 88 },
    { 0x4482462e, 67 },
    { 0x5a824632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b24635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c2463a, 100 },
    { 0xbce2463a, 61 },
    { 0xb9d2463a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21824642, 61 },
    { 0, 0 },
    { 0x53324644, 33 },
    { 0, 0 },
    { 0xa9a24646, 61 },
    { 0x62324646, 76 },
    { 0x91524648, 87 },
    { 0, 0 },
    { 0x5c2464a, 90 },
    { 0, 0 },
    { 0xd172464c, 90 },
    { 0xdb12464d, 75 },
    { 0, 0 },
    { 0x6622464f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba024654, 78 },
    { 0xb0f24655, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed82465b, 95 },
    { 0x2722465b, 99 },
    { 0, 0 },
    { 0xe9b2465e, 98 },
    { 0x9012465e, 69 },
    { 0xe1e24660, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb24663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49524667, 23 },
    { 0, 0 },
    { 0x85d24669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc772466e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e124671, 34 },
    { 0x6a24672, 33 },
    { 0, 0 },
    { 0x40024674, 91 },
    { 0, 0 },
    { 0xf2c24676, 23 },
    { 0x14d24676, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002467d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc24680, 23 },
    { 0xc3024681, 23 },
    { 0x91824681, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a824685, 34 },
    { 0x76f24686, 84 },
    { 0x5024687, 34 },
    { 0xb24688, 77 },
    { 0x44124689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2024692, 90 },
    { 0x46324693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38924696, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f92469a, 23 },
    { 0xc692469b, 100 },
    { 0xbc12469c, 67 },
    { 0xe8a2469d, 55 },
    { 0xaee2469e, 55 },
    { 0xe362469f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753246a3, 23 },
    { 0x62e246a4, 33 },
    { 0, 0 },
    { 0xa29246a6, 90 },
    { 0, 0 },
    { 0x29a246a8, 33 },
    { 0x33d246a9, 93 },
    { 0, 0 },
    { 0x97e246ab, 43 },
    { 0xc7246ac, 98 },
    { 0xd5e246ad, 95 },
    { 0x5c9246ae, 23 },
    { 0x8ac246ab, 33 },
    { 0xd1f246b0, 98 },
    { 0, 0 },
    { 0x519246b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271246b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220246bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c246bf, 33 },
    { 0xdcc246c0, 90 },
    { 0x427246c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7246c4, 61 },
    { 0x83d246c5, 76 },
    { 0, 0 },
    { 0xc22246c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c246cd, 43 },
    { 0x95246ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35246d3, 61 },
    { 0x6e4246d3, 33 },
    { 0, 0 },
    { 0x19a246d6, 93 },
    { 0, 0 },
    { 0xeb0246d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2246dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d246e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8246e3, 98 },
    { 0x8c3246e3, 98 },
    { 0, 0 },
    { 0x424246e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e246eb, 33 },
    { 0xa5246eb, 66 },
    { 0xc75246eb, 98 },
    { 0x29d246ed, 33 },
    { 0x45c246ef, 86 },
    { 0xefe246f0, 61 },
    { 0xb21246f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291246f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2246fa, 78 },
    { 0xd4c246fa, 99 },
    { 0, 0 },
    { 0xe7e246fd, 95 },
    { 0x996246fe, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6824707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e2470a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2470d, 43 },
    { 0x4f82470e, 61 },
    { 0x7f52470e, 93 },
    { 0, 0 },
    { 0x49224711, 90 },
    { 0, 0 },
    { 0x43024713, 76 },
    { 0x12324713, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c24719, 66 },
    { 0x26b2471a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7072471f, 40 },
    { 0xe082471f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b124727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7722472a, 23 },
    { 0x28d2472b, 95 },
    { 0x18c2472c, 90 },
    { 0, 0 },
    { 0x8332472e, 95 },
    { 0xacb2472f, 23 },
    { 0x88524730, 34 },
    { 0xf4424731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54824738, 90 },
    { 0, 0 },
    { 0x4952473a, 76 },
    { 0, 0 },
    { 0x98d2473c, 68 },
    { 0x4db2473c, 90 },
    { 0x1c42473c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d724742, 64 },
    { 0xe9924742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272474c, 23 },
    { 0x45e2474d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d624750, 33 },
    { 0xab224751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d52475b, 95 },
    { 0x5df2475b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692475f, 33 },
    { 0x17724760, 98 },
    { 0xa0c24761, 34 },
    { 0x5ca24760, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc24765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa24775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b524779, 61 },
    { 0xf062477a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692477d, 61 },
    { 0xf8c2477e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74024783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d024786, 90 },
    { 0xe3b24787, 76 },
    { 0x4b524787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32478c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a2478f, 99 },
    { 0, 0 },
    { 0x61124791, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91824797, 52 },
    { 0, 0 },
    { 0x6124799, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef22479d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0247a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x836247a3, 99 },
    { 0xa00247a3, 23 },
    { 0, 0 },
    { 0x89247a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a247ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15247b3, 33 },
    { 0xeaf247b3, 55 },
    { 0x642247b4, 33 },
    { 0x19247b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0247b9, 78 },
    { 0xd83247b9, 99 },
    { 0, 0 },
    { 0x84a247bc, 34 },
    { 0x788247bd, 61 },
    { 0x9d9247be, 98 },
    { 0xcc7247be, 77 },
    { 0x4b1247c0, 23 },
    { 0x555247c1, 90 },
    { 0x772247c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3247c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8247ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951247d3, 61 },
    { 0x36e247d3, 61 },
    { 0x40f247d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6247db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff247e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b247e7, 95 },
    { 0x40247e8, 93 },
    { 0x335247e9, 61 },
    { 0, 0 },
    { 0x65a247eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c247f2, 90 },
    { 0x77c247f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef247f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc22480b, 98 },
    { 0xfa72480c, 61 },
    { 0x5792480b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d24810, 55 },
    { 0x43724811, 33 },
    { 0xcba24811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924816, 23 },
    { 0xeb424816, 100 },
    { 0x4e324818, 99 },
    { 0xdf024819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2481c, 23 },
    { 0x7392481d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce124820, 23 },
    { 0x4f424821, 43 },
    { 0xfbf24821, 93 },
    { 0x96524823, 33 },
    { 0xb7d24822, 61 },
    { 0xec324824, 95 },
    { 0x87124825, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xace24829, 80 },
    { 0x18d2482a, 44 },
    { 0xa052482b, 88 },
    { 0xa3d24829, 99 },
    { 0x4762482d, 33 },
    { 0xae22482b, 55 },
    { 0x3662482f, 95 },
    { 0x2a224830, 76 },
    { 0xb8f24830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b224838, 66 },
    { 0xa3624839, 52 },
    { 0, 0 },
    { 0x59f2483b, 100 },
    { 0xaa32483c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf24840, 33 },
    { 0x9d424841, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e724846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d2484c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88124851, 55 },
    { 0x59c24852, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe24857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f2485a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972485d, 95 },
    { 0xbf72485e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf624865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd24868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc72486b, 87 },
    { 0xeff2486c, 90 },
    { 0xf922486b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b124870, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b024875, 98 },
    { 0x7d824875, 78 },
    { 0x8fa24877, 66 },
    { 0xc5524877, 52 },
    { 0xcdb24879, 76 },
    { 0x4c12487a, 23 },
    { 0xf2a2487a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72424883, 90 },
    { 0x20024884, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89424889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45124892, 77 },
    { 0, 0 },
    { 0xdd424894, 23 },
    { 0, 0 },
    { 0xb224896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed2489f, 95 },
    { 0xd8a248a0, 61 },
    { 0x3b5248a0, 98 },
    { 0x982248a1, 33 },
    { 0x34e248a3, 55 },
    { 0xf2e248a4, 33 },
    { 0x73c248a3, 33 },
    { 0x5a4248a6, 98 },
    { 0x980248a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec248ac, 96 },
    { 0x667248ad, 55 },
    { 0xd2a248ac, 43 },
    { 0, 0 },
    { 0x616248b0, 61 },
    { 0, 0 },
    { 0x8fc248b2, 90 },
    { 0x1f0248b2, 90 },
    { 0xe1248b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f248bd, 95 },
    { 0, 0 },
    { 0xd2248bf, 67 },
    { 0x277248c0, 93 },
    { 0, 0 },
    { 0x402248c2, 52 },
    { 0, 0 },
    { 0xa02248c4, 90 },
    { 0, 0 },
    { 0xb83248c6, 40 },
    { 0x493248c7, 33 },
    { 0xc5e248c6, 76 },
    { 0, 0 },
    { 0x166248ca, 96 },
    { 0x3f5248cb, 23 },
    { 0xfa4248cb, 80 },
    { 0x4ac248cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d248d4, 90 },
    { 0, 0 },
    { 0xe16248d6, 98 },
    { 0xdf4248d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b248da, 67 },
    { 0, 0 },
    { 0x610248dc, 87 },
    { 0xfad248dc, 90 },
    { 0xe2e248de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416248e2, 81 },
    { 0, 0 },
    { 0xa63248e4, 100 },
    { 0xc3c248e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x592248e8, 23 },
    { 0xcd248e9, 55 },
    { 0, 0 },
    { 0x555248eb, 90 },
    { 0x187248ec, 23 },
    { 0, 0 },
    { 0x3f7248ee, 68 },
    { 0, 0 },
    { 0x2b1248f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80248f5, 55 },
    { 0xc55248f5, 90 },
    { 0, 0 },
    { 0x3db248f8, 98 },
    { 0, 0 },
    { 0xf6c248fa, 90 },
    { 0x378248fb, 98 },
    { 0xde5248fb, 95 },
    { 0, 0 },
    { 0x21d248fe, 61 },
    { 0xa12248ff, 46 },
    { 0, 0 },
    { 0x3ea24901, 33 },
    { 0xc4924901, 98 },
    { 0, 0 },
    { 0x1c24904, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1442490a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f2490d, 90 },
    { 0x3882490d, 33 },
    { 0xad42490f, 90 },
    { 0, 0 },
    { 0x4724911, 33 },
    { 0xf0624912, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa924918, 98 },
    { 0x65324919, 76 },
    { 0x27524918, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b2491d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59224920, 66 },
    { 0xf1624921, 33 },
    { 0x60024922, 91 },
    { 0xc0e24920, 61 },
    { 0xd2b24920, 23 },
    { 0x52324922, 87 },
    { 0xc224925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3942492a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7912492e, 95 },
    { 0x99b2492f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e24932, 87 },
    { 0xcbc24932, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77124939, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8752493c, 23 },
    { 0x6f42493c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e24942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8424945, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b24949, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5162494c, 84 },
    { 0xedd2494c, 33 },
    { 0, 0 },
    { 0xef52494f, 34 },
    { 0, 0 },
    { 0x8a724951, 90 },
    { 0x78724952, 90 },
    { 0x8c24953, 66 },
    { 0, 0 },
    { 0xae724955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd2495c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac24960, 68 },
    { 0x7824961, 83 },
    { 0x37d24962, 95 },
    { 0, 0 },
    { 0x2ac24964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7102496f, 33 },
    { 0xf892496f, 99 },
    { 0x8b624971, 66 },
    { 0, 0 },
    { 0xac024973, 90 },
    { 0xe4e24974, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12324977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6262497b, 78 },
    { 0x8392497b, 46 },
    { 0xac12497b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6924984, 23 },
    { 0, 0 },
    { 0x4224986, 81 },
    { 0x20c24987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4822498b, 95 },
    { 0x98e2498b, 98 },
    { 0x7a42498d, 61 },
    { 0x8f92498e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e24995, 91 },
    { 0x47d24995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf982499a, 66 },
    { 0x3132499a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a2499e, 90 },
    { 0x31e2499f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f249b3, 23 },
    { 0x352249b3, 33 },
    { 0, 0 },
    { 0x216249b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9249ba, 33 },
    { 0, 0 },
    { 0x21f249bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf249c8, 90 },
    { 0, 0 },
    { 0x176249ca, 23 },
    { 0x961249cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985249d2, 52 },
    { 0xce249d3, 33 },
    { 0x99e249d4, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377249da, 66 },
    { 0xa48249db, 91 },
    { 0, 0 },
    { 0xff6249dd, 90 },
    { 0xa1b249de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831249e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57249e6, 98 },
    { 0x988249e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c249ea, 67 },
    { 0, 0 },
    { 0x741249ec, 33 },
    { 0, 0 },
    { 0x8e0249ee, 55 },
    { 0xa70249ef, 93 },
    { 0x8eb249f0, 95 },
    { 0x1b8249f1, 98 },
    { 0x4e9249f2, 33 },
    { 0xd07249f2, 91 },
    { 0x755249f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e249f7, 95 },
    { 0, 0 },
    { 0x6db249f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8249ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a24a05, 93 },
    { 0x4724a06, 61 },
    { 0xb0324a07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc24a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab24a12, 98 },
    { 0xd324a13, 95 },
    { 0x7ff24a13, 33 },
    { 0x86724a15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8224a1b, 88 },
    { 0x6f924a1b, 76 },
    { 0, 0 },
    { 0x3e324a1e, 61 },
    { 0xb5424a1f, 69 },
    { 0xb0b24a20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f024a23, 93 },
    { 0, 0 },
    { 0x49524a25, 33 },
    { 0x31124a25, 93 },
    { 0x8fa24a26, 34 },
    { 0x15124a28, 98 },
    { 0x30624a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f124a34, 91 },
    { 0, 0 },
    { 0xf0724a36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f924a43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e924a47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b24a4a, 61 },
    { 0x4b224a4b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0924a50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4524a54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b24a57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9424a5c, 34 },
    { 0, 0 },
    { 0xd5424a5e, 48 },
    { 0xeb824a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56024a64, 33 },
    { 0xdbb24a64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x51224a68, 61 },
    { 0xb6924a68, 78 },
    { 0xc0724a69, 61 },
    { 0, 0 },
    { 0x6df24a6c, 65 },
    { 0x36924a6c, 90 },
    { 0x8cf24a6e, 46 },
    { 0x2e024a6d, 90 },
    { 0xa6424a6c, 67 },
    { 0xb3e24a6c, 90 },
    { 0x59f24a6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67124a76, 84 },
    { 0, 0 },
    { 0x5bd24a78, 23 },
    { 0xce924a78, 43 },
    { 0xd0024a78, 95 },
    { 0xebd24a7a, 76 },
    { 0xd5a24a7a, 23 },
    { 0, 0 },
    { 0x66024a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab724a82, 61 },
    { 0, 0 },
    { 0xb5724a84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4024a87, 98 },
    { 0x69e24a88, 100 },
    { 0x79424a87, 90 },
    { 0x6a124a8a, 95 },
    { 0xaf024a8a, 75 },
    { 0xdec24a87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6424a97, 98 },
    { 0x17324a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc924a9b, 40 },
    { 0xc8224a9c, 75 },
    { 0x73224a9c, 75 },
    { 0x5924a9e, 34 },
    { 0xa2424a9f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad324aa8, 61 },
    { 0, 0 },
    { 0xbe724aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35824aad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40024ab2, 95 },
    { 0, 0 },
    { 0x7fd24ab4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e124ab9, 61 },
    { 0x5e324aba, 99 },
    { 0, 0 },
    { 0xb2024abc, 55 },
    { 0xf0524abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e524ac2, 77 },
    { 0xcba24ac3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0124ac9, 76 },
    { 0xb0b24ac9, 76 },
    { 0, 0 },
    { 0x84824acc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe24ad1, 95 },
    { 0, 0 },
    { 0x25d24ad3, 34 },
    { 0, 0 },
    { 0xfdb24ad5, 45 },
    { 0x75624ad6, 23 },
    { 0, 0 },
    { 0xf6724ad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba24adb, 69 },
    { 0xb1e24adc, 33 },
    { 0, 0 },
    { 0x42124ade, 34 },
    { 0x38b24ade, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c24ae2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87924ae5, 66 },
    { 0x3e324ae5, 55 },
    { 0, 0 },
    { 0x35424ae8, 90 },
    { 0, 0 },
    { 0xe4624aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe724aed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb724af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c424af3, 98 },
    { 0x2e324af4, 33 },
    { 0xefa24af3, 90 },
    { 0, 0 },
    { 0xc1a24af7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa824b02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56524b06, 34 },
    { 0, 0 },
    { 0x54224b08, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24324b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f624b18, 55 },
    { 0x9d124b19, 33 },
    { 0xf5124b19, 34 },
    { 0xc2424b1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3124b26, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81924b2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73624b30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a324b35, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a24b39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d24b3c, 68 },
    { 0x6f624b3c, 23 },
    { 0x2aa24b3e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x24724b41, 61 },
    { 0xe824b42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce724b49, 23 },
    { 0, 0 },
    { 0xff724b4b, 24 },
    { 0xeac24b4b, 88 },
    { 0x4a424b4b, 90 },
    { 0xd7524b4c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1024b56, 23 },
    { 0, 0 },
    { 0x5a724b58, 23 },
    { 0x1c24b59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c24b5e, 48 },
    { 0, 0 },
    { 0x34324b60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e24b66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de24b6f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58824b74, 95 },
    { 0x28724b74, 98 },
    { 0x7bb24b75, 23 },
    { 0, 0 },
    { 0x7dc24b78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc24b7b, 95 },
    { 0x60d24b7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e24b7f, 90 },
    { 0x66324b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c24b87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b324b8a, 93 },
    { 0, 0 },
    { 0x84924b8c, 88 },
    { 0, 0 },
    { 0xd8724b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3724b91, 95 },
    { 0xf7624b92, 55 },
    { 0x7c024b93, 43 },
    { 0xa3524b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f424b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d24ba4, 98 },
    { 0xd4224ba5, 23 },
    { 0xda624ba6, 61 },
    { 0x32224ba7, 19 },
    { 0, 0 },
    { 0xf824ba9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb624bae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da24bb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab24bba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30524bbd, 100 },
    { 0xc2e24bbe, 33 },
    { 0x79f24bbe, 90 },
    { 0x7224bbf, 33 },
    { 0, 0 },
    { 0x82524bc2, 84 },
    { 0xb3c24bc3, 90 },
    { 0xa9624bc4, 76 },
    { 0x38524bc4, 90 },
    { 0x3b624bc6, 95 },
    { 0, 0 },
    { 0x92024bc8, 75 },
    { 0x11824bc9, 33 },
    { 0, 0 },
    { 0x84f24bcb, 76 },
    { 0x9d24bcc, 61 },
    { 0xf1224bcc, 55 },
    { 0xde124bce, 90 },
    { 0xdcb24bcf, 78 },
    { 0xa2f24bcf, 23 },
    { 0x5b124bcf, 61 },
    { 0x61624bcc, 43 },
    { 0xebb24bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c224bd6, 78 },
    { 0x36e24bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a24bdb, 34 },
    { 0, 0 },
    { 0x4ae24bdd, 33 },
    { 0xf8724bdd, 33 },
    { 0xc0d24bde, 61 },
    { 0x13724be0, 52 },
    { 0xaba24be0, 98 },
    { 0x7ac24be2, 95 },
    { 0xa9224be3, 55 },
    { 0, 0 },
    { 0xe7424be5, 85 },
    { 0x11c24be6, 33 },
    { 0x7c624be7, 98 },
    { 0x77b24be7, 90 },
    { 0x7b124be7, 61 },
    { 0x96e24bea, 98 },
    { 0, 0 },
    { 0x9df24bec, 76 },
    { 0, 0 },
    { 0x28424bee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db24bf5, 93 },
    { 0xae524bf6, 93 },
    { 0xee624bf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f24bfd, 88 },
    { 0xc4b24bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0524c03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f824c09, 90 },
    { 0x34424c09, 33 },
    { 0xf8e24c0a, 46 },
    { 0x76e24c09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95324c15, 33 },
    { 0x1aa24c16, 43 },
    { 0x68624c17, 98 },
    { 0, 0 },
    { 0xf9324c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a624c23, 23 },
    { 0, 0 },
    { 0x2f024c25, 34 },
    { 0xe7c24c26, 90 },
    { 0, 0 },
    { 0xf9024c28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9524c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57124c30, 95 },
    { 0x90924c31, 61 },
    { 0x28924c32, 36 },
    { 0x56c24c33, 23 },
    { 0x67a24c34, 98 },
    { 0x66324c34, 95 },
    { 0, 0 },
    { 0x15d24c37, 90 },
    { 0x2c924c37, 23 },
    { 0xcdf24c38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d224c3e, 95 },
    { 0xa9224c3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97424c42, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad24c45, 61 },
    { 0xf0124c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11324c50, 23 },
    { 0xed824c51, 87 },
    { 0xe5324c52, 95 },
    { 0, 0 },
    { 0xb2124c54, 23 },
    { 0xb9c24c55, 87 },
    { 0xf9c24c55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56124c59, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a24c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f124c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13024c65, 61 },
    { 0xe8024c65, 43 },
    { 0xa9e24c65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb24c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24824c77, 55 },
    { 0, 0 },
    { 0x30624c79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf324c7d, 17 },
    { 0x9fb24c7e, 43 },
    { 0xcf224c7d, 55 },
    { 0, 0 },
    { 0x9bb24c81, 95 },
    { 0, 0 },
    { 0xffc24c83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9324c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e24c8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee324c95, 66 },
    { 0x2e24c96, 88 },
    { 0x6bb24c96, 98 },
    { 0x6e924c95, 98 },
    { 0x1de24c99, 95 },
    { 0xea124c9a, 68 },
    { 0xdef24c97, 95 },
    { 0x7d024c97, 75 },
    { 0, 0 },
    { 0xba224c9e, 76 },
    { 0x16324c9f, 46 },
    { 0xc5924c9f, 33 },
    { 0, 0 },
    { 0x13c24ca2, 76 },
    { 0, 0 },
    { 0xddf24ca4, 87 },
    { 0, 0 },
    { 0x87d24ca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66024ca9, 95 },
    { 0x56824caa, 34 },
    { 0xe9124cab, 90 },
    { 0xc7c24cac, 91 },
    { 0x7d524cad, 99 },
    { 0x5ab24cac, 98 },
    { 0x65624caf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f24cb5, 68 },
    { 0, 0 },
    { 0xee724cb7, 55 },
    { 0x6d624cb8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd24cc4, 61 },
    { 0, 0 },
    { 0xd124cc6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf324ccb, 100 },
    { 0x84324ccc, 33 },
    { 0, 0 },
    { 0x39624cce, 34 },
    { 0xf3624ccf, 68 },
    { 0x94724cd0, 90 },
    { 0x40624cd1, 43 },
    { 0, 0 },
    { 0x98c24cd3, 90 },
    { 0xc6b24cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc924cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20224ce3, 33 },
    { 0, 0 },
    { 0x50b24ce5, 98 },
    { 0x47f24ce6, 23 },
    { 0x54124ce7, 23 },
    { 0, 0 },
    { 0xe8524ce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed24ced, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b724cf2, 90 },
    { 0x54a24cf2, 33 },
    { 0x9ca24cf4, 90 },
    { 0xa3e24cf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a424cf8, 75 },
    { 0, 0 },
    { 0xd2224cfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde24d00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f24d07, 17 },
    { 0xb8124d08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80024d0d, 90 },
    { 0xfc24d0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x75024d11, 95 },
    { 0xd1c24d12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52624d1a, 33 },
    { 0x21b24d1b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7224d1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8224d22, 95 },
    { 0xad124d22, 95 },
    { 0x4a324d24, 78 },
    { 0x99a24d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c24d2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f24d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee924d34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6524d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd024d3e, 61 },
    { 0, 0 },
    { 0x84b24d40, 95 },
    { 0, 0 },
    { 0xe8e24d42, 99 },
    { 0, 0 },
    { 0x23b24d44, 66 },
    { 0x18024d45, 90 },
    { 0x17524d46, 23 },
    { 0x36b24d47, 33 },
    { 0, 0 },
    { 0x24024d49, 23 },
    { 0, 0 },
    { 0x2b324d4b, 61 },
    { 0xa724d4c, 78 },
    { 0x52924d4d, 95 },
    { 0x74024d4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1824d51, 59 },
    { 0, 0 },
    { 0x38124d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79024d58, 90 },
    { 0, 0 },
    { 0xcf24d5a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac24d60, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc24d67, 61 },
    { 0x13924d68, 33 },
    { 0, 0 },
    { 0x94424d6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa024d6f, 98 },
    { 0x22424d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea24d74, 61 },
    { 0x3f24d75, 76 },
    { 0x18324d76, 46 },
    { 0x6db24d77, 93 },
    { 0xf8524d77, 34 },
    { 0xf6624d78, 23 },
    { 0x19b24d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9824d7e, 33 },
    { 0x5ea24d7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c024d82, 56 },
    { 0, 0 },
    { 0x8bd24d84, 61 },
    { 0x93424d85, 55 },
    { 0, 0 },
    { 0xe7e24d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe824d8a, 93 },
    { 0x56c24d8b, 68 },
    { 0, 0 },
    { 0x3d924d8d, 87 },
    { 0x90d24d8e, 83 },
    { 0xf5b24d8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa624d93, 90 },
    { 0x74724d93, 78 },
    { 0x3da24d95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed24d9a, 91 },
    { 0x92824d9b, 33 },
    { 0x40c24d9c, 33 },
    { 0xb1124d9c, 33 },
    { 0xe6824d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c24da1, 33 },
    { 0x81824da2, 33 },
    { 0x3524da3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59824da6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db24da9, 90 },
    { 0x7a724daa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75424dad, 88 },
    { 0xb2724dad, 23 },
    { 0x77d24dae, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7024db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b624dbc, 40 },
    { 0x98124dbc, 23 },
    { 0x5ea24dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a24dc1, 23 },
    { 0x72824dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed124dc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c24dc8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41024dcc, 55 },
    { 0, 0 },
    { 0x21e24dce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec324dd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed24dd7, 61 },
    { 0x11d24dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3024ddc, 61 },
    { 0xd9c24ddc, 46 },
    { 0, 0 },
    { 0xce224ddf, 33 },
    { 0x20c24de0, 23 },
    { 0xa7824de0, 23 },
    { 0x3e324de0, 76 },
    { 0x7b024de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d24de8, 53 },
    { 0x97d24de8, 55 },
    { 0x6a24dea, 76 },
    { 0x9ed24deb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f24def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca524df2, 76 },
    { 0x96f24df3, 23 },
    { 0xf0e24df4, 66 },
    { 0x24924df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c124e05, 67 },
    { 0x20d24e05, 43 },
    { 0x3bf24e07, 55 },
    { 0x42424e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85924e0c, 90 },
    { 0xe6924e0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e24e10, 90 },
    { 0, 0 },
    { 0x3e24e12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b24e15, 33 },
    { 0, 0 },
    { 0x78324e17, 76 },
    { 0, 0 },
    { 0x50224e19, 46 },
    { 0, 0 },
    { 0x48724e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e24e23, 99 },
    { 0xf0324e24, 43 },
    { 0x4b924e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a224e32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99824e36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1824e39, 43 },
    { 0, 0 },
    { 0x71024e3b, 61 },
    { 0, 0 },
    { 0xc5224e3d, 55 },
    { 0x34824e3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d24e41, 52 },
    { 0x21c24e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a24e46, 34 },
    { 0, 0 },
    { 0xd8524e48, 61 },
    { 0, 0 },
    { 0x5b424e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46924e4f, 61 },
    { 0, 0 },
    { 0xd4024e51, 100 },
    { 0xe3524e51, 77 },
    { 0xc8a24e53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39524e5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x45024e5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc624e62, 98 },
    { 0x82b24e63, 95 },
    { 0, 0 },
    { 0xf3824e65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2824e6c, 98 },
    { 0x4dd24e6c, 91 },
    { 0xfbe24e6e, 23 },
    { 0x67924e6e, 38 },
    { 0xece24e70, 90 },
    { 0x5a24e71, 93 },
    { 0x2b124e6e, 98 },
    { 0xfb224e73, 23 },
    { 0x3a624e74, 84 },
    { 0x4324e6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c24e7b, 23 },
    { 0xf6824e7b, 90 },
    { 0, 0 },
    { 0xbf824e7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0824e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b24e86, 55 },
    { 0, 0 },
    { 0x56724e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9224e8d, 87 },
    { 0xbd124e8e, 23 },
    { 0, 0 },
    { 0x20224e90, 84 },
    { 0x6ae24e90, 95 },
    { 0x2bb24e91, 61 },
    { 0x91724e93, 87 },
    { 0x26a24e94, 100 },
    { 0, 0 },
    { 0x99324e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53924e9b, 61 },
    { 0x47d24e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5124ea3, 98 },
    { 0xf1424ea4, 91 },
    { 0, 0 },
    { 0x5524ea6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf24ead, 61 },
    { 0, 0 },
    { 0x5c124eaf, 76 },
    { 0, 0 },
    { 0x63a24eb1, 52 },
    { 0, 0 },
    { 0xaeb24eb3, 25 },
    { 0, 0 },
    { 0x54824eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58324ec0, 98 },
    { 0x3ad24ec1, 61 },
    { 0x5d724ec2, 23 },
    { 0x72324ec2, 17 },
    { 0xde324ec0, 99 },
    { 0, 0 },
    { 0xf0b24ec6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xec224ec9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d24ed0, 91 },
    { 0x34b24ed1, 90 },
    { 0x3824ed2, 90 },
    { 0x47324ed2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ed8, 61 },
    { 0xeb024ed9, 95 },
    { 0, 0 },
    { 0x18b24edb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd24ede, 33 },
    { 0, 0 },
    { 0x2c824ee0, 93 },
    { 0x48a24ee1, 62 },
    { 0xa4924ee2, 93 },
    { 0x4c224ee2, 76 },
    { 0, 0 },
    { 0xa0d24ee5, 78 },
    { 0x2c324ee5, 33 },
    { 0x12b24ee7, 23 },
    { 0x5a124ee8, 98 },
    { 0x36b24ee7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa24eed, 81 },
    { 0xcfb24eee, 95 },
    { 0xd0124eef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de24efa, 99 },
    { 0x76e24efa, 100 },
    { 0, 0 },
    { 0xb7024efd, 43 },
    { 0x60e24efe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f24f01, 98 },
    { 0x97624f01, 55 },
    { 0, 0 },
    { 0x49124f04, 76 },
    { 0x24924f05, 91 },
    { 0xbcb24f06, 90 },
    { 0x84624f07, 98 },
    { 0x6f324f07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f24f12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe224f15, 95 },
    { 0, 0 },
    { 0x89224f17, 23 },
    { 0x5f224f18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b24f1b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad24f24, 91 },
    { 0xf0524f25, 33 },
    { 0x82524f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a124f2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab24f34, 98 },
    { 0xffc24f35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8924f38, 95 },
    { 0, 0 },
    { 0xef524f3a, 33 },
    { 0x39f24f3b, 33 },
    { 0xc1224f3c, 33 },
    { 0, 0 },
    { 0x4a24f3e, 33 },
    { 0xd5b24f3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee424f44, 95 },
    { 0, 0 },
    { 0x7bc24f46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b24f4a, 33 },
    { 0x7924f4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f524f4f, 34 },
    { 0x7b724f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1824f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c24f59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5124f5c, 100 },
    { 0x11d24f5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724f61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a124f64, 95 },
    { 0xdfe24f65, 23 },
    { 0x80e24f66, 99 },
    { 0x3224f67, 98 },
    { 0, 0 },
    { 0x44524f69, 61 },
    { 0x12c24f6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc24f6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0124f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd24f77, 69 },
    { 0x79924f78, 34 },
    { 0x51424f79, 36 },
    { 0xf0a24f7a, 61 },
    { 0x10924f7a, 61 },
    { 0xe8624f7c, 23 },
    { 0xdf124f7c, 61 },
    { 0xfba24f7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e024f81, 61 },
    { 0xf224f81, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x95224f85, 91 },
    { 0xbae24f85, 93 },
    { 0, 0 },
    { 0xc7224f88, 43 },
    { 0xee024f89, 98 },
    { 0x6f424f8a, 61 },
    { 0xa024f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd624f91, 28 },
    { 0, 0 },
    { 0x31c24f93, 90 },
    { 0, 0 },
    { 0x89c24f95, 84 },
    { 0, 0 },
    { 0x20524f97, 61 },
    { 0x86a24f98, 23 },
    { 0xef124f99, 90 },
    { 0x4c024f99, 67 },
    { 0xa2a24f9a, 43 },
    { 0, 0 },
    { 0x9cd24f9d, 84 },
    { 0xe0f24f9e, 86 },
    { 0x97e24f9f, 90 },
    { 0xf9e24f9d, 91 },
    { 0, 0 },
    { 0x18a24fa2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99524fa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f24faf, 55 },
    { 0x45024faf, 88 },
    { 0x20a24fb0, 98 },
    { 0xe5c24fb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc024fb6, 95 },
    { 0, 0 },
    { 0x50124fb8, 43 },
    { 0x51c24fb9, 90 },
    { 0x36224fba, 61 },
    { 0, 0 },
    { 0x2a24fbc, 33 },
    { 0x52a24fbd, 81 },
    { 0x5a024fbd, 61 },
    { 0, 0 },
    { 0xae124fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2224fc5, 46 },
    { 0xc1924fc5, 95 },
    { 0, 0 },
    { 0x76c24fc8, 94 },
    { 0, 0 },
    { 0xcb224fca, 78 },
    { 0x37724fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e24fd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d24fd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1024fd8, 66 },
    { 0x30a24fd8, 95 },
    { 0xf5a24fda, 35 },
    { 0xe5624fda, 33 },
    { 0xdd524fdc, 23 },
    { 0x37524fdd, 98 },
    { 0x4224fdd, 68 },
    { 0x5e024fdf, 33 },
    { 0x7c24fe0, 87 },
    { 0xffe24fd8, 36 },
    { 0x86e24fde, 93 },
    { 0, 0 },
    { 0x43524fe4, 52 },
    { 0xa3824fe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4824feb, 61 },
    { 0x9024fec, 95 },
    { 0x48b24feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef224ff2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e624ff9, 90 },
    { 0, 0 },
    { 0xa3724ffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a624ffe, 90 },
    { 0xf1024fff, 61 },
    { 0x2524fff, 33 },
    { 0xcb924fff, 98 },
    { 0, 0 },
    { 0x1ef25003, 66 },
    { 0xf0425003, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38925008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b2500b, 33 },
    { 0xd802500c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf425015, 55 },
    { 0x6e525015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf102501a, 95 },
    { 0, 0 },
    { 0xe952501c, 8 },
    { 0x6f32501c, 61 },
    { 0x59f2501d, 23 },
    { 0x7392501f, 76 },
    { 0x9232501d, 23 },
    { 0x30125021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c525024, 65 },
    { 0x6fa25025, 23 },
    { 0xdd625026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da2502b, 82 },
    { 0, 0 },
    { 0x2982502d, 87 },
    { 0xd4c2502e, 80 },
    { 0, 0 },
    { 0x37b25030, 33 },
    { 0xb4025030, 46 },
    { 0xa3725030, 90 },
    { 0x60425033, 61 },
    { 0xc3e25033, 33 },
    { 0xe3525035, 91 },
    { 0xeb825034, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa592503a, 55 },
    { 0xfce2503b, 61 },
    { 0, 0 },
    { 0xb0e2503d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad425045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13425049, 17 },
    { 0x50a2504a, 91 },
    { 0x6892504a, 61 },
    { 0xb3c2504c, 33 },
    { 0xbd32504c, 87 },
    { 0, 0 },
    { 0xc192504f, 101 },
    { 0xe225050, 33 },
    { 0x82d25051, 33 },
    { 0x66425052, 34 },
    { 0xd6a25050, 33 },
    { 0, 0 },
    { 0x82725055, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb62505d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2625064, 90 },
    { 0xaa325064, 46 },
    { 0x51425066, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef72506b, 76 },
    { 0x3a02506b, 17 },
    { 0x9322506d, 95 },
    { 0x7032506d, 55 },
    { 0x3fe2506f, 90 },
    { 0xc4125070, 23 },
    { 0, 0 },
    { 0xc1125072, 33 },
    { 0x72f25073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1725076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67925079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252507c, 90 },
    { 0, 0 },
    { 0x71f2507e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38425081, 90 },
    { 0x9d625082, 33 },
    { 0x8d125082, 55 },
    { 0xc3525082, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb102508f, 55 },
    { 0, 0 },
    { 0xd1725091, 91 },
    { 0x3e225091, 95 },
    { 0x4ac25093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb72509a, 90 },
    { 0xd342509b, 90 },
    { 0, 0 },
    { 0x89e2509d, 61 },
    { 0xcf42509d, 43 },
    { 0x7332509f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e250a2, 95 },
    { 0xff0250a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982250a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454250ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd250b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd250b4, 23 },
    { 0x5fd250b4, 84 },
    { 0xc6b250b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c250bb, 33 },
    { 0xf4c250bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0250c3, 89 },
    { 0x90f250c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740250c8, 82 },
    { 0x66c250c8, 52 },
    { 0, 0 },
    { 0x20f250cb, 43 },
    { 0, 0 },
    { 0x81a250cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e250d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299250d4, 23 },
    { 0xb9b250d5, 23 },
    { 0x2d9250d4, 98 },
    { 0, 0 },
    { 0xf6e250d8, 80 },
    { 0, 0 },
    { 0x8c1250da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c250dd, 55 },
    { 0x69a250de, 98 },
    { 0, 0 },
    { 0xc64250e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f250e3, 55 },
    { 0, 0 },
    { 0x5a6250e5, 34 },
    { 0x236250e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19250e9, 23 },
    { 0x544250ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4250f2, 91 },
    { 0, 0 },
    { 0x262250f4, 95 },
    { 0x87a250f5, 95 },
    { 0xd6f250f6, 95 },
    { 0xdd8250f7, 23 },
    { 0x1a3250f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x484250fb, 87 },
    { 0, 0 },
    { 0xd68250fd, 34 },
    { 0x992250fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1825111, 98 },
    { 0, 0 },
    { 0x72f25113, 36 },
    { 0, 0 },
    { 0x8b525115, 93 },
    { 0x88125116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15225122, 43 },
    { 0xcef25123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb925126, 90 },
    { 0, 0 },
    { 0x93625128, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7332512e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xef725131, 61 },
    { 0, 0 },
    { 0x95825133, 46 },
    { 0, 0 },
    { 0x9f225135, 95 },
    { 0x4e025136, 81 },
    { 0, 0 },
    { 0xc9f25138, 76 },
    { 0, 0 },
    { 0x1b42513a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002513d, 95 },
    { 0xd9a2513e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c25144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952514c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9225153, 76 },
    { 0xa0725154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e625157, 23 },
    { 0xa7525157, 23 },
    { 0xede25159, 55 },
    { 0x2602515a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52515e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d25164, 95 },
    { 0x53125164, 95 },
    { 0x64825166, 33 },
    { 0x97f25167, 23 },
    { 0xf1125168, 17 },
    { 0x90625169, 43 },
    { 0x91625164, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2516d, 36 },
    { 0x1192516e, 88 },
    { 0x1032516e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd925172, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c25175, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d25179, 98 },
    { 0x14425179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c32517f, 68 },
    { 0, 0 },
    { 0x58f25181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19225185, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a25189, 66 },
    { 0x1522518a, 91 },
    { 0xf1a2518b, 67 },
    { 0x8102518a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782518f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7925194, 91 },
    { 0, 0 },
    { 0xe2225196, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d2519b, 55 },
    { 0xb332519c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7251a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db251a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8251b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d251ba, 33 },
    { 0x8ea251ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0251c2, 43 },
    { 0, 0 },
    { 0xdda251c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d251cc, 95 },
    { 0x83d251cc, 95 },
    { 0xcd1251ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d251d1, 33 },
    { 0xee9251d1, 34 },
    { 0, 0 },
    { 0x30251d4, 90 },
    { 0x9d7251d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x119251d8, 55 },
    { 0, 0 },
    { 0xe6a251da, 93 },
    { 0xe00251db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x624251de, 33 },
    { 0xbf3251df, 90 },
    { 0x595251e0, 98 },
    { 0xf6251e0, 91 },
    { 0xbc251df, 95 },
    { 0x49d251e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e251e6, 84 },
    { 0x746251e7, 99 },
    { 0x3e6251e7, 88 },
    { 0x6d7251e6, 33 },
    { 0xe19251ea, 23 },
    { 0x15d251ea, 76 },
    { 0, 0 },
    { 0xf36251ed, 60 },
    { 0x301251ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2251f2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2251f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d25203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82520f, 75 },
    { 0, 0 },
    { 0x1ca25211, 98 },
    { 0xabc25211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df25215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a25218, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62521c, 46 },
    { 0x8ff2521d, 90 },
    { 0, 0 },
    { 0xdb82521f, 90 },
    { 0, 0 },
    { 0x3025221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20825224, 34 },
    { 0xab25225, 33 },
    { 0xe1a25225, 61 },
    { 0xfcc25227, 90 },
    { 0x6725228, 78 },
    { 0, 0 },
    { 0x6812522a, 61 },
    { 0x9d2522b, 95 },
    { 0xa592522c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022522f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd2523d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a25241, 33 },
    { 0, 0 },
    { 0x2cb25243, 61 },
    { 0, 0 },
    { 0x5eb25245, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2524a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b025253, 98 },
    { 0, 0 },
    { 0x56b25255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68425259, 34 },
    { 0x7222525a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd25263, 90 },
    { 0xe825263, 98 },
    { 0x3f025263, 61 },
    { 0, 0 },
    { 0xee225267, 61 },
    { 0xbc025268, 99 },
    { 0xb3125267, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322526f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b525273, 34 },
    { 0, 0 },
    { 0xc5925275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62527e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd25282, 77 },
    { 0, 0 },
    { 0xa1525284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf2528a, 23 },
    { 0x7a12528b, 78 },
    { 0, 0 },
    { 0x69c2528d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0925295, 23 },
    { 0, 0 },
    { 0xebe25297, 95 },
    { 0x60425297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b2529d, 95 },
    { 0x3c92529e, 77 },
    { 0x7fe2529d, 75 },
    { 0x31f252a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12252a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c252aa, 90 },
    { 0x349252aa, 95 },
    { 0xef4252ac, 34 },
    { 0x706252ad, 61 },
    { 0x81252ae, 76 },
    { 0x58252af, 17 },
    { 0x498252ae, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28252b9, 33 },
    { 0, 0 },
    { 0xd7d252bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88252bf, 34 },
    { 0xd7b252c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33252c3, 61 },
    { 0x1e3252c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a252cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a252cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448252d4, 61 },
    { 0x8e5252d5, 87 },
    { 0x2ec252d5, 90 },
    { 0x834252d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968252df, 90 },
    { 0xb7f252df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xede252e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7252ec, 55 },
    { 0xa74252ec, 91 },
    { 0x9ec252ee, 77 },
    { 0xaae252ef, 33 },
    { 0, 0 },
    { 0x8d3252f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a252f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b252fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2225300, 95 },
    { 0x91225301, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e25304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe125307, 69 },
    { 0x6ee25308, 23 },
    { 0, 0 },
    { 0x5542530a, 81 },
    { 0, 0 },
    { 0xb082530c, 95 },
    { 0, 0 },
    { 0x14a2530e, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d2531b, 100 },
    { 0, 0 },
    { 0x9c82531d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b25321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d25325, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa12532a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5072532e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c25331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4125335, 36 },
    { 0x4d825336, 68 },
    { 0x18225337, 95 },
    { 0, 0 },
    { 0xfff25339, 90 },
    { 0x1882533a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2533e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d925341, 87 },
    { 0x25f25341, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc25347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f2534d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4725351, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d825356, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de2535d, 90 },
    { 0, 0 },
    { 0x7072535f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f625362, 75 },
    { 0, 0 },
    { 0x8c025364, 98 },
    { 0x62025365, 87 },
    { 0, 0 },
    { 0x18425367, 78 },
    { 0xd8725367, 40 },
    { 0, 0 },
    { 0x9c72536a, 43 },
    { 0, 0 },
    { 0x2932536c, 81 },
    { 0, 0 },
    { 0x4f22536e, 90 },
    { 0x2df2536f, 78 },
    { 0xe282536f, 40 },
    { 0x30925371, 46 },
    { 0x96c25372, 78 },
    { 0xf3c25371, 90 },
    { 0xc1d25374, 34 },
    { 0x6242536f, 23 },
    { 0x4af25375, 76 },
    { 0x4b625377, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c25382, 81 },
    { 0x36525382, 46 },
    { 0x4cb25383, 95 },
    { 0x52425385, 84 },
    { 0, 0 },
    { 0xb8125387, 95 },
    { 0x3a925387, 23 },
    { 0x4cd25388, 88 },
    { 0xacb25388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32538d, 69 },
    { 0x182538e, 90 },
    { 0, 0 },
    { 0x87425390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f425394, 75 },
    { 0x86025394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d25398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7942539c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4652539f, 76 },
    { 0, 0 },
    { 0x55d253a1, 93 },
    { 0x581253a1, 90 },
    { 0x579253a3, 61 },
    { 0x9a9253a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a253a7, 33 },
    { 0xff253a8, 61 },
    { 0x730253a9, 76 },
    { 0, 0 },
    { 0xd79253ab, 61 },
    { 0x328253ac, 33 },
    { 0x767253ad, 76 },
    { 0x7b6253ae, 90 },
    { 0, 0 },
    { 0x34d253b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f253b5, 75 },
    { 0xbe3253b6, 61 },
    { 0xda2253b7, 64 },
    { 0x41253b8, 23 },
    { 0x59b253b9, 84 },
    { 0x835253b9, 69 },
    { 0xd9e253ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c253c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307253cd, 52 },
    { 0, 0 },
    { 0xdf7253cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70253d6, 85 },
    { 0, 0 },
    { 0xa74253d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8253db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3253df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa253e5, 100 },
    { 0x1c1253e6, 33 },
    { 0, 0 },
    { 0x54b253e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d253ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71253f1, 90 },
    { 0x8a253f1, 61 },
    { 0xc54253f2, 66 },
    { 0x42d253f1, 78 },
    { 0x1f253f5, 99 },
    { 0x8c253f4, 80 },
    { 0xa9e253f7, 90 },
    { 0x22f253f8, 99 },
    { 0, 0 },
    { 0xd10253fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca253ff, 33 },
    { 0x42d253ff, 67 },
    { 0xb5025401, 61 },
    { 0xa0a25402, 100 },
    { 0xa825402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c25409, 99 },
    { 0, 0 },
    { 0x3282540b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe822540e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f125411, 33 },
    { 0xb6b25412, 23 },
    { 0xee225412, 77 },
    { 0x82825414, 77 },
    { 0x21b25414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f25418, 76 },
    { 0, 0 },
    { 0xc242541a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc25420, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4025426, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2542d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8425432, 68 },
    { 0x71d25433, 43 },
    { 0, 0 },
    { 0x9225435, 61 },
    { 0, 0 },
    { 0x26f25437, 88 },
    { 0x6bf25438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62543b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2543f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e25447, 90 },
    { 0, 0 },
    { 0x52625449, 78 },
    { 0, 0 },
    { 0x5152544b, 95 },
    { 0x3f32544c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd325450, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0125454, 95 },
    { 0, 0 },
    { 0x42025456, 43 },
    { 0, 0 },
    { 0xd5125458, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c2545c, 95 },
    { 0, 0 },
    { 0xfea2545e, 85 },
    { 0x7d52545e, 99 },
    { 0xedd2545f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025465, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52546d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20425472, 61 },
    { 0x2a625473, 99 },
    { 0, 0 },
    { 0x7d225475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44825478, 90 },
    { 0, 0 },
    { 0xab72547a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5925480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a125483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac25486, 52 },
    { 0, 0 },
    { 0x28625488, 76 },
    { 0, 0 },
    { 0xac52548a, 95 },
    { 0xcd82548a, 69 },
    { 0x32e2548c, 85 },
    { 0x5442548d, 95 },
    { 0, 0 },
    { 0xd672548f, 43 },
    { 0xe8125490, 95 },
    { 0xff02548f, 61 },
    { 0, 0 },
    { 0x4a425493, 90 },
    { 0x1ac25493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a225499, 33 },
    { 0x21a2549a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4254a2, 23 },
    { 0xabf254a3, 46 },
    { 0x58f254a4, 84 },
    { 0x17e254a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8254a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57254ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4254b1, 61 },
    { 0, 0 },
    { 0x51d254b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f254bb, 90 },
    { 0xbca254bc, 76 },
    { 0xd6b254bd, 61 },
    { 0xa47254bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91254c2, 33 },
    { 0, 0 },
    { 0x86f254c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416254ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b254d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x463254d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563254d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306254de, 46 },
    { 0x2d3254df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc254e5, 38 },
    { 0x5b4254e6, 33 },
    { 0xc73254e6, 34 },
    { 0, 0 },
    { 0x4fa254e9, 61 },
    { 0xe87254ea, 45 },
    { 0x29254eb, 33 },
    { 0xa35254ec, 33 },
    { 0x95b254ed, 61 },
    { 0x1bf254ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2254f3, 90 },
    { 0x179254f4, 60 },
    { 0x61c254f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8254fc, 78 },
    { 0, 0 },
    { 0x752254fe, 61 },
    { 0x17a254ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a025502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b125509, 52 },
    { 0, 0 },
    { 0xc1f2550b, 98 },
    { 0x5b22550b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed925514, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b25517, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e2551c, 55 },
    { 0, 0 },
    { 0xa522551e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a725526, 34 },
    { 0, 0 },
    { 0x5af25528, 23 },
    { 0xbc25529, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf042552d, 55 },
    { 0xe122552e, 33 },
    { 0xde02552d, 95 },
    { 0x36825530, 87 },
    { 0x8ea25531, 91 },
    { 0xb2f2552e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a425535, 90 },
    { 0x9a725535, 98 },
    { 0, 0 },
    { 0xd8725538, 90 },
    { 0x4ed25539, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0225541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a25546, 61 },
    { 0xc3125546, 97 },
    { 0x71325548, 33 },
    { 0x1f25549, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f02554e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d25553, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a25556, 97 },
    { 0x7a725556, 61 },
    { 0x1be25558, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5772555b, 98 },
    { 0x2232555c, 76 },
    { 0, 0 },
    { 0xfac2555e, 78 },
    { 0xc732555f, 19 },
    { 0xa2725560, 33 },
    { 0x7eb2555f, 33 },
    { 0xd3725561, 95 },
    { 0xfdd25560, 81 },
    { 0x44f25564, 98 },
    { 0xdf025565, 23 },
    { 0x99125566, 33 },
    { 0, 0 },
    { 0xe8425568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372556e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f25573, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a2557c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2557f, 33 },
    { 0, 0 },
    { 0x8f825581, 61 },
    { 0, 0 },
    { 0x16a25583, 83 },
    { 0xa7325584, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c25587, 91 },
    { 0, 0 },
    { 0x8e925589, 34 },
    { 0x21125589, 95 },
    { 0x8352558b, 61 },
    { 0xcab2558b, 23 },
    { 0xd4a2558d, 34 },
    { 0xa522558e, 85 },
    { 0x5db2558f, 34 },
    { 0xa382558b, 97 },
    { 0x592558e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39025596, 61 },
    { 0, 0 },
    { 0x9d925598, 88 },
    { 0xae525599, 23 },
    { 0x302559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea255a2, 77 },
    { 0xfb0255a3, 98 },
    { 0, 0 },
    { 0x8b8255a5, 90 },
    { 0xa3255a5, 12 },
    { 0xc3e255a7, 95 },
    { 0xb8f255a8, 46 },
    { 0xec6255a7, 34 },
    { 0x5c9255aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7255ae, 98 },
    { 0xd8b255af, 98 },
    { 0xb29255af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0255b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828255bd, 33 },
    { 0x544255be, 95 },
    { 0xa8e255bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc255c5, 99 },
    { 0, 0 },
    { 0x42f255c7, 55 },
    { 0, 0 },
    { 0x5ae255c9, 90 },
    { 0, 0 },
    { 0xde6255cb, 61 },
    { 0, 0 },
    { 0x32e255cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a255d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95255e2, 81 },
    { 0xd5a255e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2255e6, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9255ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298255f7, 95 },
    { 0x879255f8, 77 },
    { 0x971255f8, 98 },
    { 0x725255f9, 61 },
    { 0x334255fb, 78 },
    { 0x6f5255f7, 55 },
    { 0xf7f255fd, 100 },
    { 0x309255fe, 61 },
    { 0x8d6255ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28025603, 33 },
    { 0x6b225604, 98 },
    { 0, 0 },
    { 0xf425606, 65 },
    { 0xc825607, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212560b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5625612, 61 },
    { 0, 0 },
    { 0xf525614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68325624, 76 },
    { 0x61025625, 61 },
    { 0x82225625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd112562a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2562d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c25630, 76 },
    { 0x14225630, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9425637, 33 },
    { 0, 0 },
    { 0x17225639, 33 },
    { 0, 0 },
    { 0xccb2563b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8252563f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b25642, 43 },
    { 0xd7125643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb925648, 93 },
    { 0xd5c25648, 100 },
    { 0x3f625648, 91 },
    { 0x34625648, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402564e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd725652, 33 },
    { 0, 0 },
    { 0xe4325654, 90 },
    { 0xe5b25655, 66 },
    { 0xf5325656, 33 },
    { 0x28325656, 99 },
    { 0xd725656, 55 },
    { 0x1e725658, 78 },
    { 0x77b25657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2565d, 61 },
    { 0x392565e, 98 },
    { 0x5882565f, 85 },
    { 0x63d2565d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71525663, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad92566a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92425671, 90 },
    { 0x6b625672, 33 },
    { 0xfb125673, 91 },
    { 0, 0 },
    { 0xc4125675, 90 },
    { 0x6ea25676, 69 },
    { 0, 0 },
    { 0x7c225678, 17 },
    { 0xe8f25679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3725681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d25686, 95 },
    { 0xcb825686, 61 },
    { 0x6fe25688, 61 },
    { 0x17e25689, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb2568f, 95 },
    { 0, 0 },
    { 0xbb125691, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad25695, 66 },
    { 0xfb025695, 90 },
    { 0x6fc25696, 61 },
    { 0, 0 },
    { 0x21725699, 33 },
    { 0xadc2569a, 90 },
    { 0x6d22569a, 90 },
    { 0, 0 },
    { 0x4f82569d, 33 },
    { 0xf1d2569d, 76 },
    { 0xdb62569d, 34 },
    { 0x6ee256a0, 33 },
    { 0xf90256a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7256a4, 88 },
    { 0, 0 },
    { 0x4de256a6, 95 },
    { 0x792256a7, 81 },
    { 0xc7b256a7, 52 },
    { 0, 0 },
    { 0x3d5256aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e256b0, 95 },
    { 0, 0 },
    { 0x404256b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679256bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd256bf, 81 },
    { 0, 0 },
    { 0x48256c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f256c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b256cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d256d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0256d7, 76 },
    { 0x5cf256d7, 34 },
    { 0xce5256d7, 95 },
    { 0xf00256da, 76 },
    { 0x3e4256da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d256de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa256e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b256ee, 76 },
    { 0x16a256ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803256f4, 33 },
    { 0, 0 },
    { 0xd11256f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6256fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x233256ff, 61 },
    { 0xfb525700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b525703, 46 },
    { 0x21e25704, 36 },
    { 0xcdf25704, 90 },
    { 0x65e25704, 43 },
    { 0xfbe25705, 99 },
    { 0xb5525708, 33 },
    { 0, 0 },
    { 0xb1b2570a, 34 },
    { 0x7222570b, 90 },
    { 0x6872570c, 98 },
    { 0xd3e2570c, 90 },
    { 0xbd2570c, 95 },
    { 0x3512570d, 61 },
    { 0x6672570c, 34 },
    { 0x8b525711, 81 },
    { 0x2d62570e, 52 },
    { 0xf5b2570e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be25719, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2571f, 90 },
    { 0, 0 },
    { 0x4b425721, 98 },
    { 0x9ef25722, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f25725, 76 },
    { 0xabc25726, 95 },
    { 0, 0 },
    { 0xd3425728, 61 },
    { 0x23b25728, 95 },
    { 0x9cb25729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922572d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e425730, 23 },
    { 0, 0 },
    { 0xe8b25732, 34 },
    { 0x97b25732, 43 },
    { 0x63f25734, 61 },
    { 0xbde25734, 39 },
    { 0xe5525736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a225739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e92573c, 49 },
    { 0, 0 },
    { 0x22c2573e, 98 },
    { 0x5eb2573f, 55 },
    { 0x64b2573f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f325743, 34 },
    { 0x7e025744, 34 },
    { 0x61f25745, 68 },
    { 0xeec25746, 61 },
    { 0xd6b25743, 43 },
    { 0x4d625748, 90 },
    { 0xed725744, 85 },
    { 0x8c225745, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a32574e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25756, 100 },
    { 0xd6c25757, 55 },
    { 0xfe625758, 82 },
    { 0x58325759, 90 },
    { 0x9fc2575a, 23 },
    { 0x2b525758, 83 },
    { 0, 0 },
    { 0x7ed2575d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f725763, 23 },
    { 0, 0 },
    { 0xc0925765, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78625768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf652576c, 91 },
    { 0xb962576c, 33 },
    { 0xbbb2576c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20925771, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2577a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc25782, 88 },
    { 0xb7d25783, 67 },
    { 0xe6025784, 98 },
    { 0x77025783, 64 },
    { 0x5df25786, 42 },
    { 0xc7e25787, 33 },
    { 0x50b25787, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e2578e, 66 },
    { 0, 0 },
    { 0x74d25790, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18125797, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b22579d, 33 },
    { 0, 0 },
    { 0xbbe2579f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7257a5, 82 },
    { 0x259257a6, 98 },
    { 0x490257a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9257ab, 91 },
    { 0xc97257ac, 99 },
    { 0xfcf257ac, 46 },
    { 0x8a1257ab, 33 },
    { 0x62a257af, 98 },
    { 0xad6257af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2257b8, 33 },
    { 0x9d9257b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x569257bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986257c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33257c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x780257c7, 56 },
    { 0xfcf257c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a257d3, 76 },
    { 0x53c257d4, 52 },
    { 0x83f257d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x402257d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0257df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987257e5, 100 },
    { 0, 0 },
    { 0xb70257e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x588257ea, 33 },
    { 0x730257eb, 34 },
    { 0xe6a257ec, 33 },
    { 0, 0 },
    { 0xd84257ee, 75 },
    { 0x23b257ef, 85 },
    { 0x1b257f0, 23 },
    { 0, 0 },
    { 0x3e4257f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x433257f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d325800, 90 },
    { 0x41225801, 80 },
    { 0xe7525802, 77 },
    { 0x50f25802, 91 },
    { 0, 0 },
    { 0x2d825805, 23 },
    { 0, 0 },
    { 0xb8e25807, 52 },
    { 0, 0 },
    { 0xb4c25809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92580f, 33 },
    { 0, 0 },
    { 0xd8d25811, 55 },
    { 0, 0 },
    { 0x5cb25813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222581a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f2581d, 55 },
    { 0x3c2581e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef725828, 55 },
    { 0, 0 },
    { 0xb462582a, 90 },
    { 0x782582b, 90 },
    { 0x5b52582b, 93 },
    { 0x9812582b, 95 },
    { 0xafb2582e, 90 },
    { 0, 0 },
    { 0xd3525830, 90 },
    { 0, 0 },
    { 0x1bd25832, 33 },
    { 0x55325833, 84 },
    { 0x39925834, 33 },
    { 0xe6d25835, 76 },
    { 0x1a425836, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a625839, 23 },
    { 0, 0 },
    { 0x8032583b, 34 },
    { 0x3c42583c, 84 },
    { 0x3e22583d, 61 },
    { 0x1692583d, 88 },
    { 0x6622583b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb125843, 33 },
    { 0x4c625844, 23 },
    { 0x2525844, 95 },
    { 0x1be25844, 33 },
    { 0xf3425847, 98 },
    { 0, 0 },
    { 0xe0e25849, 78 },
    { 0x1692584a, 23 },
    { 0xd682584a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73325850, 65 },
    { 0, 0 },
    { 0x9c525852, 96 },
    { 0xb8925853, 93 },
    { 0x6c25854, 90 },
    { 0x6fe25855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a25859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9812585c, 98 },
    { 0x5312585d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db25869, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2586d, 23 },
    { 0x6e92586e, 98 },
    { 0x3772586f, 84 },
    { 0xb9b25870, 95 },
    { 0xb8d2586e, 34 },
    { 0x5982586f, 93 },
    { 0x1c625872, 88 },
    { 0x98425870, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c925877, 61 },
    { 0x40f25878, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2587b, 98 },
    { 0, 0 },
    { 0xb282587d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4725880, 98 },
    { 0, 0 },
    { 0xa1525882, 78 },
    { 0xef725883, 36 },
    { 0x70825884, 76 },
    { 0xb0625885, 33 },
    { 0xf6c25886, 98 },
    { 0, 0 },
    { 0x73c25888, 90 },
    { 0x8f625889, 95 },
    { 0, 0 },
    { 0xc72588b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2588f, 33 },
    { 0, 0 },
    { 0xe1625891, 90 },
    { 0x3b25892, 95 },
    { 0x2c925893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72225896, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f32589a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d2589d, 38 },
    { 0x84f2589d, 98 },
    { 0, 0 },
    { 0xc75258a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36258a4, 90 },
    { 0x2f3258a5, 67 },
    { 0x1da258a6, 55 },
    { 0x7d0258a5, 100 },
    { 0xc7a258a8, 46 },
    { 0x979258a5, 76 },
    { 0x7e0258aa, 95 },
    { 0x3c9258ab, 84 },
    { 0, 0 },
    { 0x325258ad, 43 },
    { 0, 0 },
    { 0xc90258af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82258b7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b258bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88258be, 33 },
    { 0, 0 },
    { 0xc96258c0, 95 },
    { 0, 0 },
    { 0xa1f258c2, 23 },
    { 0xdf3258c2, 34 },
    { 0x768258c4, 23 },
    { 0x365258c5, 33 },
    { 0xda8258c5, 33 },
    { 0, 0 },
    { 0x468258c8, 23 },
    { 0x259258c9, 90 },
    { 0, 0 },
    { 0x97258cb, 33 },
    { 0xb03258cb, 23 },
    { 0x623258cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e258d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f258d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db258da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757258e4, 88 },
    { 0x44e258e5, 23 },
    { 0x78b258e6, 90 },
    { 0, 0 },
    { 0xe9258e8, 77 },
    { 0xc74258e8, 98 },
    { 0, 0 },
    { 0x535258eb, 33 },
    { 0xa9b258ec, 46 },
    { 0, 0 },
    { 0xd9b258ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646258fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2425902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac925905, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b2590a, 97 },
    { 0xe092590a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41125911, 76 },
    { 0x23125911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf25915, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5522591e, 66 },
    { 0x5532591f, 61 },
    { 0, 0 },
    { 0xdeb25921, 43 },
    { 0x88325922, 78 },
    { 0x81a25923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b25928, 61 },
    { 0x7ce25928, 98 },
    { 0x76c2592a, 90 },
    { 0, 0 },
    { 0xb6f2592c, 46 },
    { 0xd0e2592c, 33 },
    { 0x4a82592e, 90 },
    { 0, 0 },
    { 0x7c25930, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e25941, 46 },
    { 0x90b25941, 95 },
    { 0x1bf25943, 61 },
    { 0xbf225944, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c225948, 33 },
    { 0, 0 },
    { 0x3872594a, 90 },
    { 0x6d52594b, 33 },
    { 0xd502594c, 90 },
    { 0x25d2594c, 65 },
    { 0x8a62594b, 43 },
    { 0xdee2594e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc25954, 89 },
    { 0x8aa25955, 90 },
    { 0x9e525955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67225959, 34 },
    { 0x4e02595a, 90 },
    { 0xaf2595a, 33 },
    { 0, 0 },
    { 0x1d12595d, 61 },
    { 0x73c2595e, 61 },
    { 0, 0 },
    { 0x1825960, 65 },
    { 0x40625961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d22596b, 33 },
    { 0x4e22596c, 95 },
    { 0xe422596d, 17 },
    { 0x30b2596e, 34 },
    { 0x2b32596c, 43 },
    { 0x5582596b, 87 },
    { 0x25525971, 98 },
    { 0xc1a25972, 93 },
    { 0x3ff25973, 95 },
    { 0x4402596f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31625977, 100 },
    { 0xddf25978, 61 },
    { 0, 0 },
    { 0x56c2597a, 90 },
    { 0, 0 },
    { 0xfb32597c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f25980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1325983, 34 },
    { 0, 0 },
    { 0x3f325985, 33 },
    { 0, 0 },
    { 0xd4025987, 55 },
    { 0xe4c25988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf32598b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa452598f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd25998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e2599b, 90 },
    { 0, 0 },
    { 0x2ac2599d, 23 },
    { 0xc552599e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f259a4, 61 },
    { 0xd7e259a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00259a8, 55 },
    { 0x69f259a9, 55 },
    { 0, 0 },
    { 0xcbe259ab, 61 },
    { 0xe6b259ab, 90 },
    { 0x59d259ad, 52 },
    { 0x4c8259ae, 90 },
    { 0x8e9259ae, 66 },
    { 0xfc6259af, 98 },
    { 0, 0 },
    { 0x806259b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3259b5, 33 },
    { 0x270259b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f259b9, 34 },
    { 0x70a259b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63259c4, 88 },
    { 0, 0 },
    { 0x45f259c6, 90 },
    { 0, 0 },
    { 0x346259c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a259cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee259cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d259d2, 99 },
    { 0xd19259d3, 84 },
    { 0, 0 },
    { 0x7e5259d5, 33 },
    { 0, 0 },
    { 0xaca259d7, 33 },
    { 0xde259d8, 69 },
    { 0x62c259d9, 69 },
    { 0, 0 },
    { 0xba0259db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1259e0, 33 },
    { 0xb17259e1, 76 },
    { 0xb62259e0, 85 },
    { 0x80e259e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed259e8, 55 },
    { 0x8cc259e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac259ef, 93 },
    { 0, 0 },
    { 0x331259f1, 61 },
    { 0, 0 },
    { 0x82c259f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab259f6, 95 },
    { 0x439259f6, 66 },
    { 0x320259f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8725a03, 46 },
    { 0x96425a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e25a0b, 94 },
    { 0x56725a0c, 93 },
    { 0xa1625a0d, 76 },
    { 0x1c825a0e, 33 },
    { 0, 0 },
    { 0xa5225a10, 67 },
    { 0, 0 },
    { 0x79325a12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e25a19, 85 },
    { 0xc2c25a1a, 98 },
    { 0x9be25a1a, 90 },
    { 0x38425a1a, 95 },
    { 0xfd625a1d, 85 },
    { 0, 0 },
    { 0xe1625a1f, 95 },
    { 0x66b25a20, 43 },
    { 0x7cd25a21, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0225a25, 33 },
    { 0, 0 },
    { 0x71f25a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0825a2a, 61 },
    { 0x35425a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1825a32, 85 },
    { 0, 0 },
    { 0x23125a34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11525a3c, 61 },
    { 0x9cf25a3c, 76 },
    { 0x72a25a3e, 90 },
    { 0x84625a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e25a42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25a47, 43 },
    { 0, 0 },
    { 0x52725a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff25a4e, 90 },
    { 0x24325a4e, 33 },
    { 0xbc625a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6625a56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad25a5b, 90 },
    { 0, 0 },
    { 0xc7b25a5d, 66 },
    { 0x38825a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9625a62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57625a65, 95 },
    { 0x2fd25a65, 55 },
    { 0, 0 },
    { 0xf0825a68, 68 },
    { 0, 0 },
    { 0xee225a6a, 40 },
    { 0x99925a6b, 84 },
    { 0, 0 },
    { 0x76525a6d, 61 },
    { 0x96a25a6e, 61 },
    { 0, 0 },
    { 0x1125a70, 99 },
    { 0, 0 },
    { 0xd2d25a72, 34 },
    { 0xbe325a72, 33 },
    { 0xe3225a74, 46 },
    { 0xd5125a75, 36 },
    { 0x7d725a75, 61 },
    { 0x63125a77, 65 },
    { 0xc8425a77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xee625a7b, 23 },
    { 0xe1825a7c, 46 },
    { 0, 0 },
    { 0x99625a7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea725a81, 95 },
    { 0xc4725a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa25a85, 17 },
    { 0, 0 },
    { 0x4625a87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a8a, 33 },
    { 0x8fe25a8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17025a92, 33 },
    { 0, 0 },
    { 0x3b525a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d25a99, 76 },
    { 0, 0 },
    { 0xd4e25a9b, 43 },
    { 0, 0 },
    { 0x27b25a9d, 23 },
    { 0x7a25a9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba25aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d25aa4, 85 },
    { 0x76225aa5, 33 },
    { 0x31525aa5, 90 },
    { 0xdc725aa7, 98 },
    { 0x96125aa6, 90 },
    { 0x67c25aa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e725aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df25ab0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x53825ab3, 90 },
    { 0x2be25ab4, 91 },
    { 0xc1625ab5, 66 },
    { 0xefe25ab6, 93 },
    { 0x1c425ab7, 90 },
    { 0xc3625ab8, 61 },
    { 0xb9a25ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f325abe, 61 },
    { 0, 0 },
    { 0xf5e25ac0, 98 },
    { 0, 0 },
    { 0x57325ac2, 95 },
    { 0, 0 },
    { 0xa9025ac4, 68 },
    { 0xe1225ac5, 65 },
    { 0x99725ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba25aca, 33 },
    { 0xb1d25aca, 90 },
    { 0, 0 },
    { 0x2b425acd, 55 },
    { 0, 0 },
    { 0xf0d25acf, 99 },
    { 0xb3f25acf, 38 },
    { 0, 0 },
    { 0x51f25ad2, 52 },
    { 0, 0 },
    { 0xb8225ad4, 87 },
    { 0xb8825ad5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91125ad9, 65 },
    { 0x5c125ada, 23 },
    { 0xc3e25adb, 92 },
    { 0x6a225adc, 95 },
    { 0, 0 },
    { 0x9e125ade, 53 },
    { 0x10a25adf, 68 },
    { 0x38325ae0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31325ae4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37725ae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf625aee, 91 },
    { 0x9cb25aef, 84 },
    { 0x91f25aef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b325af3, 52 },
    { 0x6a125af4, 90 },
    { 0x3ab25af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe625af8, 85 },
    { 0, 0 },
    { 0xbc325afa, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf25afe, 76 },
    { 0xfeb25aff, 34 },
    { 0x3025b00, 68 },
    { 0, 0 },
    { 0x56e25b02, 95 },
    { 0x40125b02, 98 },
    { 0, 0 },
    { 0x33025b05, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf25b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac825b0c, 76 },
    { 0, 0 },
    { 0xbde25b0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc25b12, 61 },
    { 0xbb025b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e25b17, 23 },
    { 0x27625b18, 23 },
    { 0x2c525b18, 99 },
    { 0xeca25b17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55125b1f, 97 },
    { 0xfea25b1f, 77 },
    { 0xaff25b1f, 33 },
    { 0xc2725b22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59425b2b, 55 },
    { 0xf5c25b2b, 90 },
    { 0xd3e25b2d, 95 },
    { 0x76d25b2d, 67 },
    { 0, 0 },
    { 0x10c25b30, 90 },
    { 0, 0 },
    { 0xcde25b32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70025b39, 34 },
    { 0xa3c25b39, 84 },
    { 0xfda25b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5325b40, 61 },
    { 0xe3225b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb25b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c25b4d, 33 },
    { 0xa3825b4e, 55 },
    { 0xeb925b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85125b52, 93 },
    { 0xd5825b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda525b59, 98 },
    { 0x61e25b5a, 55 },
    { 0, 0 },
    { 0xcc925b5c, 23 },
    { 0xcb425b5c, 98 },
    { 0x9d525b5e, 65 },
    { 0, 0 },
    { 0xefb25b60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8825b67, 23 },
    { 0xb1725b67, 38 },
    { 0x45c25b68, 33 },
    { 0, 0 },
    { 0x2bb25b6b, 33 },
    { 0x72b25b6b, 91 },
    { 0x2ed25b6d, 61 },
    { 0, 0 },
    { 0xfe25b6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d25b77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec625b7e, 96 },
    { 0xb6225b7f, 90 },
    { 0x51125b80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49925b85, 33 },
    { 0, 0 },
    { 0x33f25b87, 78 },
    { 0, 0 },
    { 0x49825b89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f525b8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c25b91, 98 },
    { 0x4bd25b91, 23 },
    { 0, 0 },
    { 0x17d25b94, 61 },
    { 0xb25b95, 77 },
    { 0x64225b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58225b9b, 90 },
    { 0xdfb25b9c, 77 },
    { 0x56625b9c, 78 },
    { 0x1bd25b9e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76325ba2, 100 },
    { 0xf0b25ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f925ba8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf25bad, 76 },
    { 0x53325bad, 83 },
    { 0xb0525bae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a425bb3, 61 },
    { 0, 0 },
    { 0x5fa25bb5, 90 },
    { 0, 0 },
    { 0xe6025bb7, 98 },
    { 0, 0 },
    { 0x6ab25bb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7625bbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3025bc1, 81 },
    { 0x925bc1, 23 },
    { 0xbfa25bc1, 23 },
    { 0xde525bc1, 81 },
    { 0x87f25bc3, 61 },
    { 0, 0 },
    { 0xd225bc7, 95 },
    { 0x9ec25bc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f325bce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x68925bd1, 55 },
    { 0xe5025bd1, 76 },
    { 0x4a025bd3, 34 },
    { 0xcf925bd2, 40 },
    { 0xd5525bd2, 23 },
    { 0xd9925bd4, 65 },
    { 0x8f325bd7, 77 },
    { 0xe3325bd2, 23 },
    { 0xa5525bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d425beb, 23 },
    { 0xb5325bec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0825bef, 55 },
    { 0x52225bf0, 53 },
    { 0xc225bf1, 33 },
    { 0x6ca25bf2, 52 },
    { 0x5c725bf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f25bfb, 43 },
    { 0, 0 },
    { 0x97825bfd, 46 },
    { 0, 0 },
    { 0xff725bff, 90 },
    { 0, 0 },
    { 0xa6c25c01, 61 },
    { 0xb0625c02, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a25c05, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66225c0a, 98 },
    { 0, 0 },
    { 0x18e25c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e25c11, 95 },
    { 0xc5025c11, 67 },
    { 0x87c25c13, 55 },
    { 0xa7d25c12, 90 },
    { 0x97425c15, 66 },
    { 0x8a625c13, 100 },
    { 0xc7025c13, 76 },
    { 0, 0 },
    { 0xa2a25c19, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb625c1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8025c1f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27525c23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff825c26, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da25c2b, 23 },
    { 0, 0 },
    { 0xb0f25c2d, 55 },
    { 0, 0 },
    { 0x22c25c2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87825c38, 66 },
    { 0, 0 },
    { 0xd1325c3a, 98 },
    { 0, 0 },
    { 0xa1225c3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa25c40, 90 },
    { 0xd3025c41, 75 },
    { 0x87525c42, 61 },
    { 0x5325c43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e25c46, 98 },
    { 0, 0 },
    { 0x76a25c48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31825c4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30125c50, 55 },
    { 0x7d425c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b525c56, 76 },
    { 0x8e325c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a225c5a, 83 },
    { 0, 0 },
    { 0x9d025c5c, 77 },
    { 0, 0 },
    { 0x80225c5e, 91 },
    { 0xd6a25c5f, 61 },
    { 0, 0 },
    { 0x1c025c61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab925c67, 93 },
    { 0x70425c68, 90 },
    { 0x90025c69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb225c71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f25c74, 23 },
    { 0xaf425c75, 33 },
    { 0x62a25c76, 90 },
    { 0xc6a25c76, 95 },
    { 0, 0 },
    { 0xbe25c79, 24 },
    { 0xfa325c7a, 100 },
    { 0, 0 },
    { 0x1d725c7c, 98 },
    { 0x20225c7d, 84 },
    { 0xeef25c7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d25c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb425c8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76725c91, 99 },
    { 0xd0b25c92, 76 },
    { 0, 0 },
    { 0xcab25c94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59325c9a, 61 },
    { 0x4e725c9a, 61 },
    { 0xb25c9c, 98 },
    { 0x1eb25c9c, 98 },
    { 0, 0 },
    { 0x10e25c9f, 33 },
    { 0xcad25ca0, 52 },
    { 0xe0a25ca0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c25ca6, 85 },
    { 0, 0 },
    { 0xb0225ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad25cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe25cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c25cc1, 46 },
    { 0, 0 },
    { 0x41f25cc3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4725cc7, 91 },
    { 0x4125cc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf225ccc, 95 },
    { 0x32025ccc, 23 },
    { 0xbb625ccd, 23 },
    { 0, 0 },
    { 0xaee25cd0, 55 },
    { 0x97925cd1, 64 },
    { 0, 0 },
    { 0xeae25cd3, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9525cd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde25cda, 33 },
    { 0xea725cda, 95 },
    { 0, 0 },
    { 0x36625cdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16925ce0, 95 },
    { 0x8dd25ce1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe625ce4, 33 },
    { 0, 0 },
    { 0x1e425ce6, 93 },
    { 0, 0 },
    { 0xbfe25ce8, 55 },
    { 0, 0 },
    { 0x25125cea, 95 },
    { 0x6a725ceb, 33 },
    { 0x36025ceb, 33 },
    { 0xba225cea, 90 },
    { 0x37b25cee, 90 },
    { 0x74325cee, 23 },
    { 0x5a425cf0, 93 },
    { 0xe225cf1, 90 },
    { 0x21925ceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5925cf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f25cfa, 81 },
    { 0xb25cfb, 46 },
    { 0xcd225cfb, 95 },
    { 0, 0 },
    { 0xd8925cfe, 93 },
    { 0, 0 },
    { 0x1825d00, 66 },
    { 0xa6025d01, 99 },
    { 0x3d325d02, 55 },
    { 0x9725d01, 33 },
    { 0x2bb25d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb25d08, 69 },
    { 0xc3325d09, 76 },
    { 0xf5425d08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4725d0f, 61 },
    { 0xbfb25d10, 46 },
    { 0x1d425d10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91125d14, 23 },
    { 0x80e25d14, 77 },
    { 0x4fd25d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94325d1a, 84 },
    { 0x3fb25d1a, 61 },
    { 0, 0 },
    { 0xd6f25d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d25d24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3125d27, 23 },
    { 0xb2b25d28, 66 },
    { 0, 0 },
    { 0x9b525d2a, 55 },
    { 0x9d025d2a, 40 },
    { 0xdda25d2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6025d30, 70 },
    { 0xc9b25d31, 95 },
    { 0, 0 },
    { 0xb5025d33, 90 },
    { 0x48025d33, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb25d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2925d3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa025d3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e25d47, 98 },
    { 0xf0f25d48, 33 },
    { 0x5c25d49, 40 },
    { 0, 0 },
    { 0x79925d4b, 98 },
    { 0, 0 },
    { 0x8425d4d, 55 },
    { 0x7d625d4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69725d51, 34 },
    { 0x88925d52, 75 },
    { 0x5225d53, 68 },
    { 0x71225d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f025d57, 76 },
    { 0x8f725d57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d25d5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a025d62, 95 },
    { 0x1db25d63, 100 },
    { 0x5af25d62, 33 },
    { 0x44725d62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30325d69, 98 },
    { 0xb725d6a, 33 },
    { 0x5825d6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef25d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa225d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b525d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24325d7d, 90 },
    { 0xe3325d7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c25d81, 67 },
    { 0x8b425d81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11025d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67525d88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a25d8d, 33 },
    { 0, 0 },
    { 0xa3e25d8f, 33 },
    { 0xb0a25d90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd25d98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8725d9c, 66 },
    { 0xc0425d9d, 98 },
    { 0x97225d9e, 61 },
    { 0x8db25d9d, 34 },
    { 0xa7625da0, 61 },
    { 0xeba25da0, 95 },
    { 0x44d25da2, 66 },
    { 0, 0 },
    { 0x27525da4, 95 },
    { 0x8e125da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2125daf, 87 },
    { 0, 0 },
    { 0x80625db1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x15425db4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b25db7, 95 },
    { 0x55825db8, 61 },
    { 0x9d925db7, 85 },
    { 0, 0 },
    { 0x40f25dbb, 43 },
    { 0x34f25dbc, 84 },
    { 0xf2025dbb, 43 },
    { 0x6ed25dbe, 90 },
    { 0x8a625dbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36825dc2, 93 },
    { 0x87625dc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc25dd5, 34 },
    { 0x5c825dd6, 98 },
    { 0x1c925dd6, 61 },
    { 0x51825dd8, 23 },
    { 0xad825dd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd25ddc, 95 },
    { 0xb6325ddd, 98 },
    { 0xc7425dde, 90 },
    { 0x70b25dde, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f825de3, 76 },
    { 0x75425de4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e25de9, 66 },
    { 0, 0 },
    { 0x80425deb, 33 },
    { 0xde625dec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a25def, 75 },
    { 0xe3325def, 91 },
    { 0x50025def, 90 },
    { 0xfb425df2, 84 },
    { 0xdbb25df3, 88 },
    { 0, 0 },
    { 0x7df25df5, 88 },
    { 0xf1025df6, 61 },
    { 0xe7025df5, 43 },
    { 0, 0 },
    { 0xf5225df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc25dfd, 33 },
    { 0x31d25dfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf425e06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80025e0c, 67 },
    { 0x2a025e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df25e11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2125e14, 98 },
    { 0x5e225e15, 76 },
    { 0, 0 },
    { 0x60425e17, 33 },
    { 0x16a25e18, 98 },
    { 0x49425e19, 88 },
    { 0x99825e18, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59625e1d, 90 },
    { 0x5cb25e1e, 61 },
    { 0x15c25e1e, 98 },
    { 0xdeb25e1d, 33 },
    { 0xad625e21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47425e24, 87 },
    { 0, 0 },
    { 0xac725e26, 98 },
    { 0x8f625e27, 33 },
    { 0, 0 },
    { 0xaac25e29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5425e2d, 98 },
    { 0xee525e2d, 61 },
    { 0x70925e2f, 76 },
    { 0x35625e2f, 90 },
    { 0x89b25e31, 23 },
    { 0, 0 },
    { 0x78725e33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e25e37, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2625e3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30725e42, 98 },
    { 0xcd225e43, 98 },
    { 0x4fe25e44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a25e4c, 95 },
    { 0x58a25e4d, 85 },
    { 0, 0 },
    { 0x25225e4f, 61 },
    { 0x42525e50, 69 },
    { 0, 0 },
    { 0x32125e52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c225e56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e425e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95525e5e, 33 },
    { 0xe2525e5e, 76 },
    { 0x2c625e5f, 90 },
    { 0x90125e60, 33 },
    { 0xa5125e62, 55 },
    { 0x58225e63, 98 },
    { 0x93c25e64, 77 },
    { 0xc9225e65, 33 },
    { 0xde825e66, 66 },
    { 0xa625e5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a25e6a, 75 },
    { 0x93625e6a, 43 },
    { 0x39b25e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68625e70, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x85325e73, 76 },
    { 0, 0 },
    { 0xac725e75, 85 },
    { 0x6ff25e75, 41 },
    { 0x94325e77, 40 },
    { 0, 0 },
    { 0xf4b25e79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa025e7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d25e80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b25e83, 90 },
    { 0x8a125e84, 90 },
    { 0x79c25e85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7525e89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e025e8c, 90 },
    { 0, 0 },
    { 0xab25e8e, 61 },
    { 0x84725e8f, 61 },
    { 0xc9325e90, 93 },
    { 0x5f025e8f, 80 },
    { 0x90e25e92, 78 },
    { 0x1025e93, 93 },
    { 0, 0 },
    { 0x85b25e95, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b725e99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a325e9d, 61 },
    { 0, 0 },
    { 0x4d025e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42825ea3, 23 },
    { 0x77f25ea4, 95 },
    { 0x46325ea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de25ea9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f25ead, 95 },
    { 0x12325ead, 33 },
    { 0xc6725eaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc25eb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef925eba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb25ebe, 85 },
    { 0xb7b25ebe, 61 },
    { 0, 0 },
    { 0xbb525ec1, 55 },
    { 0x2c425ec2, 23 },
    { 0x9f525ec3, 90 },
    { 0, 0 },
    { 0x9b025ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a25ec8, 90 },
    { 0x38625ec9, 90 },
    { 0xfab25eca, 66 },
    { 0x12e25ec8, 95 },
    { 0x1da25ecc, 98 },
    { 0xc9a25ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f725ed0, 90 },
    { 0xec825ed0, 78 },
    { 0xb2125ed2, 66 },
    { 0x27625ed3, 61 },
    { 0, 0 },
    { 0x86825ed5, 23 },
    { 0x50625ed6, 33 },
    { 0x70125ed7, 90 },
    { 0x5c225ed8, 76 },
    { 0x29225ed9, 66 },
    { 0x57d25ed7, 61 },
    { 0x88a25edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb25ee0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb25ee6, 77 },
    { 0x4cb25ee7, 34 },
    { 0xa4d25ee8, 93 },
    { 0xe5525ee8, 61 },
    { 0xeae25ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b25eef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa425ef2, 84 },
    { 0, 0 },
    { 0x32225ef4, 98 },
    { 0x7e125ef5, 61 },
    { 0x77925ef6, 61 },
    { 0x77625ef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea825efb, 61 },
    { 0, 0 },
    { 0x3a25efd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325f02, 85 },
    { 0, 0 },
    { 0x2d325f04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f625f09, 90 },
    { 0x92425f0a, 93 },
    { 0xf8025f0b, 98 },
    { 0, 0 },
    { 0x8fa25f0d, 90 },
    { 0, 0 },
    { 0x4cd25f0f, 90 },
    { 0x6c225f0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae25f13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c25f19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6225f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f725f23, 43 },
    { 0, 0 },
    { 0x2bb25f25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f25f29, 23 },
    { 0xc6425f2a, 90 },
    { 0x16d25f2b, 34 },
    { 0xf2625f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b25f32, 93 },
    { 0, 0 },
    { 0xaf225f34, 98 },
    { 0xde525f35, 33 },
    { 0, 0 },
    { 0xfdc25f37, 95 },
    { 0x94925f38, 83 },
    { 0x11e25f39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7425f3d, 98 },
    { 0xf5225f3e, 75 },
    { 0, 0 },
    { 0xc9a25f40, 93 },
    { 0x64925f41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85225f4b, 55 },
    { 0, 0 },
    { 0x3d625f4d, 88 },
    { 0x69d25f4d, 96 },
    { 0, 0 },
    { 0x82825f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a625f53, 90 },
    { 0x6325f54, 65 },
    { 0, 0 },
    { 0x78125f56, 55 },
    { 0x31025f56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf25f5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c25f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc25f61, 23 },
    { 0xb5f25f62, 95 },
    { 0x3be25f63, 90 },
    { 0xbbd25f64, 100 },
    { 0xb3225f61, 95 },
    { 0x4f125f63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37625f69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d325f6e, 90 },
    { 0xf1025f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99025f75, 35 },
    { 0xb2525f75, 61 },
    { 0x5f425f75, 90 },
    { 0, 0 },
    { 0x7425f79, 33 },
    { 0xa1b25f7a, 90 },
    { 0, 0 },
    { 0x9c325f7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60125f82, 69 },
    { 0, 0 },
    { 0xf7525f84, 33 },
    { 0, 0 },
    { 0xe8f25f86, 98 },
    { 0, 0 },
    { 0x41525f88, 76 },
    { 0x54625f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5125f92, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde25f96, 98 },
    { 0x93325f96, 34 },
    { 0, 0 },
    { 0x60625f99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea25f9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0625fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13725fa9, 77 },
    { 0x13c25faa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc25fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf625fb5, 36 },
    { 0, 0 },
    { 0x27425fb7, 98 },
    { 0x97f25fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf025fbd, 96 },
    { 0x28c25fbd, 61 },
    { 0xe6d25fbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f925fc4, 90 },
    { 0x30325fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5025fcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d25fd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71125fd5, 98 },
    { 0x7fe25fd6, 65 },
    { 0x33125fd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45925fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10625fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57525fe8, 90 },
    { 0, 0 },
    { 0x73725fea, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf25fee, 98 },
    { 0x36325fee, 61 },
    { 0x97725ff0, 95 },
    { 0, 0 },
    { 0xeef25ff2, 78 },
    { 0x1dd25ff3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b425ffa, 95 },
    { 0x12925ffb, 55 },
    { 0x93c25ffc, 34 },
    { 0x30c25ffd, 98 },
    { 0xe4225ffe, 78 },
    { 0, 0 },
    { 0x3ea26000, 34 },
    { 0, 0 },
    { 0xf0f26002, 90 },
    { 0x55826003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f526008, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222600c, 61 },
    { 0x3a42600c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10926014, 33 },
    { 0, 0 },
    { 0xcf026016, 95 },
    { 0, 0 },
    { 0x57726018, 61 },
    { 0, 0 },
    { 0x70d2601a, 40 },
    { 0xc012601b, 52 },
    { 0xe832601c, 61 },
    { 0xad72601c, 90 },
    { 0x4562601e, 68 },
    { 0x1452601f, 45 },
    { 0x5512601d, 98 },
    { 0, 0 },
    { 0x91e26022, 98 },
    { 0, 0 },
    { 0x49626024, 78 },
    { 0x20926025, 76 },
    { 0, 0 },
    { 0x89c26027, 75 },
    { 0xdbc26028, 67 },
    { 0x5b126027, 34 },
    { 0, 0 },
    { 0x5892602b, 52 },
    { 0xf9e2602b, 90 },
    { 0x75e2602c, 23 },
    { 0x12f2602e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac26031, 90 },
    { 0xb3026032, 90 },
    { 0x20926032, 76 },
    { 0xc0926032, 90 },
    { 0x5be26035, 98 },
    { 0xd2e26032, 85 },
    { 0, 0 },
    { 0x7926038, 33 },
    { 0x2c726039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62603e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe426041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d26044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d2604d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a526056, 23 },
    { 0x3ff26056, 90 },
    { 0x71926058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2605d, 53 },
    { 0, 0 },
    { 0x6ed2605f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2126063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd26068, 61 },
    { 0, 0 },
    { 0xec02606a, 100 },
    { 0x802606a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e72606f, 43 },
    { 0, 0 },
    { 0xad326071, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf926074, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c26078, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a2607d, 87 },
    { 0x862607d, 33 },
    { 0x8072607f, 33 },
    { 0xca626080, 95 },
    { 0, 0 },
    { 0x1ef26082, 33 },
    { 0xf3b26083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50626086, 90 },
    { 0x9bd26087, 76 },
    { 0, 0 },
    { 0xa8626089, 78 },
    { 0x6d726089, 98 },
    { 0, 0 },
    { 0x3bf2608c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc26096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda826099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc952609d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd260a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6260ad, 90 },
    { 0x48c260ad, 61 },
    { 0x429260ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7260b3, 61 },
    { 0x279260b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba260bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x287260c0, 90 },
    { 0x67c260c0, 78 },
    { 0, 0 },
    { 0xcd1260c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974260c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x857260ca, 76 },
    { 0x296260ca, 95 },
    { 0xb2c260cc, 98 },
    { 0x136260cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab260d3, 23 },
    { 0x9be260d4, 67 },
    { 0, 0 },
    { 0x797260d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc260da, 98 },
    { 0x6cf260db, 55 },
    { 0, 0 },
    { 0xd1e260dd, 61 },
    { 0x912260de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66260e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c260ea, 95 },
    { 0xfc7260eb, 51 },
    { 0x76260ec, 75 },
    { 0, 0 },
    { 0xf99260ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e260f1, 34 },
    { 0, 0 },
    { 0x417260f3, 82 },
    { 0xdf5260f4, 76 },
    { 0x3e0260f4, 90 },
    { 0x77b260f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba260fa, 33 },
    { 0, 0 },
    { 0xcb6260fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca526100, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41926109, 90 },
    { 0x78a2610a, 95 },
    { 0x3b02610b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022610e, 64 },
    { 0x6ba2610f, 78 },
    { 0x9bb26110, 76 },
    { 0, 0 },
    { 0xcc26112, 23 },
    { 0, 0 },
    { 0x25126114, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b26117, 90 },
    { 0x1026118, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f2611b, 76 },
    { 0x46d2611b, 61 },
    { 0x6882611d, 84 },
    { 0x1b52611b, 90 },
    { 0x8d82611f, 93 },
    { 0xc8f26120, 78 },
    { 0x41926121, 46 },
    { 0x5b326122, 98 },
    { 0xc762611c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6026127, 33 },
    { 0, 0 },
    { 0xf4826129, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2612d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b26131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb026139, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9226146, 99 },
    { 0x8dc26147, 77 },
    { 0x44826146, 61 },
    { 0xe1e26149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb232614c, 98 },
    { 0, 0 },
    { 0xd2614e, 23 },
    { 0, 0 },
    { 0xc926150, 46 },
    { 0, 0 },
    { 0xf1d26152, 76 },
    { 0x73126153, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9226158, 33 },
    { 0, 0 },
    { 0x9982615a, 23 },
    { 0x5a32615b, 34 },
    { 0xf152615c, 61 },
    { 0xea92615d, 33 },
    { 0x4dc2615e, 17 },
    { 0x7dd2615e, 75 },
    { 0xfe26160, 93 },
    { 0x60e26161, 91 },
    { 0xb326161, 55 },
    { 0x62226162, 23 },
    { 0x13d26164, 76 },
    { 0xf3726164, 66 },
    { 0xf932615b, 61 },
    { 0x89d26162, 55 },
    { 0xc2126168, 75 },
    { 0x44b26169, 90 },
    { 0xd9f26169, 33 },
    { 0xef02616b, 40 },
    { 0x2802616b, 87 },
    { 0x40726168, 23 },
    { 0x26167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb26174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72726179, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb112617e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f826181, 48 },
    { 0x9a326181, 84 },
    { 0xe5626183, 69 },
    { 0xee926184, 68 },
    { 0xb3c26184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226188, 33 },
    { 0xce826189, 33 },
    { 0x77f2618a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb002618f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b826199, 76 },
    { 0, 0 },
    { 0xbdc2619b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7812619f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf261a2, 87 },
    { 0x37d261a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb261a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48261a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f261ae, 23 },
    { 0xbf0261ae, 56 },
    { 0x18e261ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x577261b3, 33 },
    { 0, 0 },
    { 0xe66261b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd261be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8261c9, 76 },
    { 0, 0 },
    { 0x834261cb, 95 },
    { 0xa96261cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e261d1, 90 },
    { 0x2a5261d2, 90 },
    { 0, 0 },
    { 0x5fc261d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab261d7, 55 },
    { 0, 0 },
    { 0xa4a261d9, 36 },
    { 0x84261d9, 23 },
    { 0x89c261d9, 33 },
    { 0, 0 },
    { 0xe7f261dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0261e1, 95 },
    { 0, 0 },
    { 0xb5c261e3, 76 },
    { 0x687261e4, 90 },
    { 0, 0 },
    { 0x1d0261e6, 91 },
    { 0x687261e6, 77 },
    { 0x8b261e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b261ed, 23 },
    { 0x694261ed, 43 },
    { 0, 0 },
    { 0x964261f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb261f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f261f7, 90 },
    { 0xb19261f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a261fc, 87 },
    { 0x332261fd, 43 },
    { 0xb20261fd, 95 },
    { 0, 0 },
    { 0xb4026200, 98 },
    { 0xc0f26201, 33 },
    { 0x3eb26202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8326216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef26219, 98 },
    { 0xf6b2621a, 98 },
    { 0x68f2621b, 68 },
    { 0xbe62621b, 90 },
    { 0x102621d, 90 },
    { 0x61b2621e, 91 },
    { 0xeb42621e, 95 },
    { 0xf9f2621f, 90 },
    { 0xb8a2621e, 69 },
    { 0x94f26222, 76 },
    { 0x2ce26221, 23 },
    { 0, 0 },
    { 0xd3b26225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27326228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc2622c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26231, 33 },
    { 0x90426232, 34 },
    { 0xc0626233, 84 },
    { 0x5e526234, 33 },
    { 0xe4826235, 95 },
    { 0x6e226236, 23 },
    { 0, 0 },
    { 0x92a26238, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda52623b, 59 },
    { 0, 0 },
    { 0x7e52623d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3626243, 69 },
    { 0x48226243, 55 },
    { 0x61d26244, 98 },
    { 0x99126244, 98 },
    { 0, 0 },
    { 0x1e226248, 23 },
    { 0x39f26248, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952624c, 61 },
    { 0xc1b2624d, 61 },
    { 0x8702624e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64826253, 90 },
    { 0xb8426254, 23 },
    { 0xa5e26255, 98 },
    { 0x8f826255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c2625d, 98 },
    { 0, 0 },
    { 0x5ba2625f, 52 },
    { 0x94226260, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33726264, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226267, 61 },
    { 0x1b526268, 93 },
    { 0xef726269, 33 },
    { 0x2112626a, 34 },
    { 0x6aa2626a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2626f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x89526272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50326277, 80 },
    { 0x3926278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42627d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1126284, 33 },
    { 0x73926285, 90 },
    { 0x51526286, 98 },
    { 0x9ab26284, 90 },
    { 0x1e526288, 84 },
    { 0xfad26284, 23 },
    { 0, 0 },
    { 0x5282628b, 95 },
    { 0xa962628c, 34 },
    { 0xdd72628d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79026293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af26299, 36 },
    { 0x5212629a, 76 },
    { 0xa1b2629b, 90 },
    { 0x6e32629a, 87 },
    { 0, 0 },
    { 0xfbd2629e, 33 },
    { 0x5b12629f, 93 },
    { 0, 0 },
    { 0x612262a1, 51 },
    { 0, 0 },
    { 0x8e8262a3, 34 },
    { 0x7b3262a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb262aa, 55 },
    { 0x1f6262ab, 61 },
    { 0x2db262ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8262af, 87 },
    { 0x854262af, 87 },
    { 0x81e262b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e262b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67262b8, 33 },
    { 0x11d262b8, 23 },
    { 0x70c262b8, 23 },
    { 0x8c262bb, 40 },
    { 0x435262bc, 76 },
    { 0, 0 },
    { 0x649262be, 61 },
    { 0x419262be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1262c2, 75 },
    { 0xed2262c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3262c6, 23 },
    { 0xb29262c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28262cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10262cf, 48 },
    { 0x73d262d0, 83 },
    { 0, 0 },
    { 0xfdd262d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x839262d5, 61 },
    { 0xa7f262d6, 91 },
    { 0xf0f262d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55262db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f262de, 43 },
    { 0x6b0262df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6262e2, 90 },
    { 0, 0 },
    { 0x805262e4, 61 },
    { 0xbcf262e4, 90 },
    { 0, 0 },
    { 0xee2262e7, 77 },
    { 0x8f262e8, 95 },
    { 0, 0 },
    { 0x91d262ea, 52 },
    { 0xb59262eb, 85 },
    { 0x651262ec, 68 },
    { 0x393262ed, 98 },
    { 0x862262eb, 69 },
    { 0x233262ed, 61 },
    { 0x3cf262ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x754262f3, 33 },
    { 0, 0 },
    { 0xe25262f5, 76 },
    { 0x2b0262f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362262f9, 98 },
    { 0x1f6262fa, 33 },
    { 0xf68262fb, 93 },
    { 0x3bd262fb, 61 },
    { 0x226262fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75226300, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a826303, 33 },
    { 0, 0 },
    { 0xb9926305, 98 },
    { 0x99126305, 23 },
    { 0, 0 },
    { 0xf2726308, 98 },
    { 0xdda26308, 90 },
    { 0xfc62630a, 61 },
    { 0x24c26308, 23 },
    { 0xb202630a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3626310, 99 },
    { 0, 0 },
    { 0x6c726312, 90 },
    { 0, 0 },
    { 0xb2c26314, 75 },
    { 0x85326314, 84 },
    { 0x1ef26316, 61 },
    { 0x28b26317, 17 },
    { 0xd0b26316, 61 },
    { 0, 0 },
    { 0x3ca2631a, 33 },
    { 0x30c2631b, 76 },
    { 0xb932631c, 34 },
    { 0x3442631d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e26329, 76 },
    { 0x8ce2632a, 66 },
    { 0x56c2632b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd112632e, 23 },
    { 0, 0 },
    { 0xb3826330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca26334, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7826338, 68 },
    { 0x9a126339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa2633c, 61 },
    { 0x5a2633d, 76 },
    { 0xc7f2633d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91726341, 90 },
    { 0x9d526342, 91 },
    { 0, 0 },
    { 0x2fd26344, 61 },
    { 0x38b26345, 84 },
    { 0xb7226346, 90 },
    { 0xfb426347, 100 },
    { 0xa0a26344, 95 },
    { 0x39726349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2634c, 66 },
    { 0xf252634c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94226351, 76 },
    { 0xfc426352, 98 },
    { 0x5bb26353, 33 },
    { 0x19826353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c426359, 87 },
    { 0, 0 },
    { 0x8052635b, 100 },
    { 0x12a2635c, 87 },
    { 0, 0 },
    { 0xae72635e, 90 },
    { 0xc7c2635f, 85 },
    { 0xa9926360, 55 },
    { 0x18c26361, 33 },
    { 0xf1b26361, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112636a, 43 },
    { 0x6872636b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f62637e, 76 },
    { 0xd4e2637f, 91 },
    { 0, 0 },
    { 0x5eb26381, 95 },
    { 0x78926382, 98 },
    { 0x3c626382, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d52638d, 98 },
    { 0x18b2638e, 76 },
    { 0x6442638f, 23 },
    { 0, 0 },
    { 0x3a526391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86926394, 43 },
    { 0x7fe26395, 17 },
    { 0x8c726395, 94 },
    { 0, 0 },
    { 0xcf426398, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4263a5, 98 },
    { 0, 0 },
    { 0xb8b263a7, 68 },
    { 0x5dc263a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c263ac, 98 },
    { 0x5b263ad, 93 },
    { 0x6bd263ae, 98 },
    { 0x973263ac, 34 },
    { 0x47263b0, 43 },
    { 0x51d263b1, 95 },
    { 0xf56263b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e263b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff263bb, 84 },
    { 0, 0 },
    { 0x808263bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c263c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de263c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e263cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x949263cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35263d3, 33 },
    { 0xfa9263d3, 46 },
    { 0, 0 },
    { 0x384263d6, 90 },
    { 0x9f1263d7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb263dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d263e2, 90 },
    { 0x40e263e3, 33 },
    { 0xa25263e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da263e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x853263ea, 34 },
    { 0x583263eb, 35 },
    { 0x8e1263eb, 80 },
    { 0xf59263ed, 98 },
    { 0, 0 },
    { 0x327263ef, 33 },
    { 0x93263ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222263f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc263fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7726400, 76 },
    { 0, 0 },
    { 0xc7626402, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x23326405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21526408, 33 },
    { 0, 0 },
    { 0xca12640a, 98 },
    { 0x4982640a, 61 },
    { 0x3822640c, 23 },
    { 0x4192640b, 90 },
    { 0x8dd2640d, 23 },
    { 0xf3d2640f, 61 },
    { 0xbc26410, 33 },
    { 0xe7626411, 100 },
    { 0xdd126412, 23 },
    { 0x54d2640b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49326417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5742641c, 43 },
    { 0x7092641c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7826424, 55 },
    { 0xe126425, 52 },
    { 0, 0 },
    { 0x2af26427, 76 },
    { 0xbe426428, 90 },
    { 0x40626429, 61 },
    { 0xa2126428, 91 },
    { 0x3da26428, 90 },
    { 0xee32642c, 100 },
    { 0x36f2642c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef26430, 46 },
    { 0xfcb26430, 52 },
    { 0xd5a26431, 98 },
    { 0xff526433, 78 },
    { 0x4d426432, 99 },
    { 0xb9e26435, 56 },
    { 0x2c826432, 90 },
    { 0xcd526431, 46 },
    { 0x9a426430, 91 },
    { 0x85c26439, 98 },
    { 0xe9526437, 90 },
    { 0x2d42643b, 34 },
    { 0x46f26437, 43 },
    { 0x5402643d, 23 },
    { 0x5482643b, 95 },
    { 0, 0 },
    { 0x65f26440, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c126443, 84 },
    { 0xdd526444, 43 },
    { 0x36426445, 90 },
    { 0x9e426446, 43 },
    { 0xd7326444, 9 },
    { 0xf6e26446, 61 },
    { 0xd8a26449, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a02644d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73826451, 33 },
    { 0, 0 },
    { 0x1a926453, 65 },
    { 0xd4726454, 95 },
    { 0xc9e26453, 55 },
    { 0xbb826456, 73 },
    { 0x86f26455, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2645b, 88 },
    { 0, 0 },
    { 0xd8f2645d, 98 },
    { 0xfaf2645e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16226463, 33 },
    { 0xbb326464, 33 },
    { 0x84926465, 88 },
    { 0xc3f26464, 33 },
    { 0x3f726467, 61 },
    { 0x5a526468, 33 },
    { 0x7026466, 12 },
    { 0x8832646a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d62646d, 98 },
    { 0, 0 },
    { 0xe8f2646f, 61 },
    { 0x2dc26470, 61 },
    { 0x90c2646f, 75 },
    { 0x71626470, 33 },
    { 0x42b26473, 33 },
    { 0xe7526473, 61 },
    { 0xa6a26475, 83 },
    { 0xb2326476, 75 },
    { 0x3c426477, 67 },
    { 0xd5226478, 46 },
    { 0x2762646f, 95 },
    { 0, 0 },
    { 0x9b42647b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2647f, 84 },
    { 0x43c26480, 98 },
    { 0x3fd26480, 90 },
    { 0x36426482, 76 },
    { 0x73c26480, 93 },
    { 0x3cb26484, 61 },
    { 0xa4d26485, 52 },
    { 0xece26484, 66 },
    { 0xefc26487, 60 },
    { 0xf2f26488, 61 },
    { 0x41526487, 98 },
    { 0x62c26488, 91 },
    { 0xad626484, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb26490, 91 },
    { 0xb8026490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa26494, 46 },
    { 0x7b626495, 76 },
    { 0x35526494, 98 },
    { 0x45526497, 90 },
    { 0x74e26497, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e32649b, 61 },
    { 0x512649c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e2649f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7264a7, 46 },
    { 0x299264a8, 61 },
    { 0x5a5264a9, 99 },
    { 0, 0 },
    { 0x3f264ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638264bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f264c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d264cb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8264d0, 78 },
    { 0x2f6264d1, 33 },
    { 0x7c6264d0, 68 },
    { 0, 0 },
    { 0x58f264d4, 38 },
    { 0, 0 },
    { 0xa5e264d6, 43 },
    { 0, 0 },
    { 0xe3c264d8, 23 },
    { 0x4264d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770264e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7264ea, 61 },
    { 0xd91264eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b264ef, 23 },
    { 0xe10264f0, 90 },
    { 0x9c4264ef, 76 },
    { 0xaf8264f2, 100 },
    { 0xa9b264f1, 43 },
    { 0xc51264f4, 61 },
    { 0xca6264f5, 85 },
    { 0x98264ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8264fb, 90 },
    { 0xca5264fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d26500, 46 },
    { 0x95726501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd26506, 95 },
    { 0, 0 },
    { 0xae526508, 33 },
    { 0x42a26509, 87 },
    { 0x8f326508, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52650d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf26511, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9326516, 75 },
    { 0x72b26517, 78 },
    { 0, 0 },
    { 0x4ca26519, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae2651e, 95 },
    { 0x2e12651e, 55 },
    { 0xb2651f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6926524, 75 },
    { 0, 0 },
    { 0xbff26526, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x35626529, 33 },
    { 0, 0 },
    { 0xef22652b, 67 },
    { 0, 0 },
    { 0x34b2652d, 23 },
    { 0xe092652e, 93 },
    { 0xae82652e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c26533, 100 },
    { 0x88526534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10426539, 34 },
    { 0, 0 },
    { 0x1e22653b, 100 },
    { 0x8162653c, 95 },
    { 0xce52653d, 84 },
    { 0, 0 },
    { 0xa0f2653f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de26548, 40 },
    { 0x5d126549, 23 },
    { 0, 0 },
    { 0x3802654b, 33 },
    { 0, 0 },
    { 0xdaa2654d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4526551, 90 },
    { 0, 0 },
    { 0x80e26553, 34 },
    { 0xb8526554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6122655d, 61 },
    { 0x56e2655d, 34 },
    { 0x46a2655f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a26566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f26569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9262656c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64826570, 90 },
    { 0, 0 },
    { 0x8d726572, 33 },
    { 0, 0 },
    { 0xa5026574, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce26578, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb092657b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3226580, 98 },
    { 0xf2c26581, 90 },
    { 0x30826580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c726586, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8302658a, 90 },
    { 0x8742658b, 90 },
    { 0, 0 },
    { 0x9462658d, 66 },
    { 0xaed2658e, 34 },
    { 0x8ba2658f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa26593, 90 },
    { 0x2ed26593, 34 },
    { 0, 0 },
    { 0x76326596, 61 },
    { 0xac26597, 76 },
    { 0x90626598, 85 },
    { 0x7de26596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2659e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26265a1, 76 },
    { 0xaea265a2, 23 },
    { 0x3f8265a3, 33 },
    { 0, 0 },
    { 0x66d265a5, 34 },
    { 0x5e5265a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a265a9, 61 },
    { 0xd75265aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9265af, 23 },
    { 0x44b265b0, 90 },
    { 0xa66265b1, 61 },
    { 0x271265b2, 33 },
    { 0x9b9265b3, 61 },
    { 0xff3265b4, 76 },
    { 0xe67265b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94265b8, 98 },
    { 0x179265b8, 66 },
    { 0x46d265ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322265c2, 81 },
    { 0, 0 },
    { 0xdf265c4, 90 },
    { 0, 0 },
    { 0x5f8265c6, 98 },
    { 0x4a6265c6, 90 },
    { 0x53d265c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb265cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5265d9, 99 },
    { 0xc2c265d9, 34 },
    { 0, 0 },
    { 0xf10265dc, 95 },
    { 0x3a0265dd, 19 },
    { 0x444265dc, 90 },
    { 0, 0 },
    { 0xf5f265e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4265e6, 23 },
    { 0xd53265e6, 90 },
    { 0xd63265e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae265ed, 36 },
    { 0, 0 },
    { 0xd10265ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9265f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75265fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e26602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2326609, 46 },
    { 0x85e2660a, 65 },
    { 0x5012660b, 33 },
    { 0, 0 },
    { 0xf252660d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e26610, 61 },
    { 0x25626611, 95 },
    { 0xdf726611, 34 },
    { 0x2af26613, 33 },
    { 0x37d26613, 95 },
    { 0x9c126615, 76 },
    { 0xe4026616, 55 },
    { 0xe4626617, 68 },
    { 0x77b26612, 95 },
    { 0x6a826617, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26620, 90 },
    { 0xf3026620, 34 },
    { 0xc26621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3326625, 23 },
    { 0x8d426626, 91 },
    { 0x89126625, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5752662a, 65 },
    { 0x7d2662b, 76 },
    { 0, 0 },
    { 0x3db2662d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c26632, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa26635, 66 },
    { 0xa5f26635, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d42663a, 23 },
    { 0x49e2663b, 33 },
    { 0x98a2663c, 90 },
    { 0x9c32663b, 76 },
    { 0x41a2663e, 2 },
    { 0x34e2663e, 90 },
    { 0x6b92663f, 34 },
    { 0x63d26641, 61 },
    { 0xfc926642, 46 },
    { 0xc8326642, 93 },
    { 0x2c926644, 61 },
    { 0x14c26643, 66 },
    { 0xe7f26646, 40 },
    { 0x29926647, 33 },
    { 0x3dc26647, 90 },
    { 0xd6f26643, 43 },
    { 0x952664a, 34 },
    { 0xf3a2664b, 76 },
    { 0x342664c, 33 },
    { 0x7f92663a, 23 },
    { 0xfa02663f, 95 },
    { 0x13f2664a, 55 },
    { 0xf9f26650, 71 },
    { 0xb2526650, 90 },
    { 0x1f72664c, 23 },
    { 0xa572664c, 80 },
    { 0xfeb26654, 90 },
    { 0, 0 },
    { 0x3fc26656, 100 },
    { 0x22126657, 90 },
    { 0x60f26658, 98 },
    { 0, 0 },
    { 0xddf2665a, 75 },
    { 0, 0 },
    { 0x3e82665c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f526666, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a22666b, 85 },
    { 0xe1f2666c, 61 },
    { 0x7482666d, 85 },
    { 0xf4e2666e, 93 },
    { 0xfb2666e, 76 },
    { 0, 0 },
    { 0x1c26671, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c626680, 98 },
    { 0x8826681, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa26685, 87 },
    { 0, 0 },
    { 0x43826687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f12668a, 33 },
    { 0, 0 },
    { 0x6832668c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b826692, 100 },
    { 0x68726693, 98 },
    { 0, 0 },
    { 0x50e26695, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef52669a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362669e, 45 },
    { 0x20a2669f, 46 },
    { 0xfb0266a0, 81 },
    { 0x3db266a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x567266a4, 90 },
    { 0, 0 },
    { 0x211266a6, 55 },
    { 0, 0 },
    { 0x3a6266a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c266ab, 53 },
    { 0xa24266ab, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6266b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd266b5, 33 },
    { 0x6e2266b6, 55 },
    { 0, 0 },
    { 0x725266b8, 90 },
    { 0xac6266b9, 33 },
    { 0xfe1266ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9266bd, 93 },
    { 0xa8d266be, 23 },
    { 0x8b8266bf, 61 },
    { 0xdc3266c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a266d0, 76 },
    { 0xd1a266d1, 43 },
    { 0x44a266d2, 95 },
    { 0x252266d3, 33 },
    { 0x847266d4, 25 },
    { 0xdaa266d1, 23 },
    { 0x2c0266d2, 12 },
    { 0xc19266d0, 55 },
    { 0, 0 },
    { 0xb45266d9, 23 },
    { 0xb6a266da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453266de, 33 },
    { 0xd0c266de, 98 },
    { 0xa6b266e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x116266e3, 61 },
    { 0xb8a266e3, 88 },
    { 0x30f266e3, 99 },
    { 0x14a266e6, 33 },
    { 0x559266e7, 78 },
    { 0xa99266e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7266ed, 85 },
    { 0xea266ee, 76 },
    { 0x429266ee, 52 },
    { 0x94d266f0, 95 },
    { 0x6f6266f1, 90 },
    { 0, 0 },
    { 0x7e5266f3, 33 },
    { 0x382266f4, 98 },
    { 0x3f4266f4, 23 },
    { 0x7ec266f3, 23 },
    { 0x2c0266f7, 90 },
    { 0xceb266f8, 85 },
    { 0, 0 },
    { 0xb99266fa, 72 },
    { 0xc84266fb, 90 },
    { 0x727266fb, 61 },
    { 0xbd266fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f26702, 90 },
    { 0x90726703, 77 },
    { 0x26326704, 34 },
    { 0xf1c26702, 33 },
    { 0x9eb26703, 34 },
    { 0, 0 },
    { 0xe3b26708, 90 },
    { 0x92026709, 40 },
    { 0x3b926708, 34 },
    { 0xa42670a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb2670f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb126714, 43 },
    { 0xf9726715, 90 },
    { 0, 0 },
    { 0xa3a26717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c26721, 76 },
    { 0xc2f26721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76626726, 66 },
    { 0, 0 },
    { 0x6c926728, 23 },
    { 0x94126728, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c626732, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf226741, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a126744, 66 },
    { 0x31226745, 78 },
    { 0, 0 },
    { 0xcdf26747, 93 },
    { 0, 0 },
    { 0x69526749, 88 },
    { 0xf492674a, 77 },
    { 0x7a326749, 33 },
    { 0, 0 },
    { 0xeb22674d, 61 },
    { 0xcc32674e, 34 },
    { 0xfd72674f, 55 },
    { 0, 0 },
    { 0xe8b26751, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea726755, 33 },
    { 0, 0 },
    { 0x4b826757, 90 },
    { 0, 0 },
    { 0x7f426759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2675f, 23 },
    { 0xaf826760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e26768, 34 },
    { 0, 0 },
    { 0xac02676a, 90 },
    { 0x4a62676b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6852676f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b26773, 43 },
    { 0xbe126774, 90 },
    { 0x97426775, 61 },
    { 0xfb426776, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20526779, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a2677d, 61 },
    { 0x5d12677e, 91 },
    { 0x5c32677e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc52678c, 97 },
    { 0x6852678c, 99 },
    { 0, 0 },
    { 0xf3f2678f, 23 },
    { 0xb5926790, 90 },
    { 0x2d326790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74626794, 23 },
    { 0x69526795, 95 },
    { 0x94d26796, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a26799, 98 },
    { 0x78c2679a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982679e, 23 },
    { 0xb0a2679f, 84 },
    { 0, 0 },
    { 0xb0f267a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd267a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f267a9, 95 },
    { 0x563267aa, 76 },
    { 0x647267ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8267af, 96 },
    { 0, 0 },
    { 0xb4b267b1, 61 },
    { 0, 0 },
    { 0xb66267b3, 98 },
    { 0x922267b4, 64 },
    { 0xafa267b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e267bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db267be, 33 },
    { 0xd1d267bf, 34 },
    { 0, 0 },
    { 0xf2f267c1, 95 },
    { 0x7e5267c2, 76 },
    { 0x7a0267c1, 90 },
    { 0x3aa267c4, 34 },
    { 0x48e267c4, 90 },
    { 0x9aa267c4, 95 },
    { 0x445267c7, 55 },
    { 0xcbf267c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718267cd, 98 },
    { 0xef1267ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb267d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d267da, 98 },
    { 0x560267db, 76 },
    { 0x781267db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14267df, 95 },
    { 0, 0 },
    { 0x43e267e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f267e6, 23 },
    { 0x9b5267e7, 61 },
    { 0, 0 },
    { 0x11f267e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d267f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2267f7, 55 },
    { 0, 0 },
    { 0xd85267f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58267fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782680a, 95 },
    { 0xcab2680b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb226810, 88 },
    { 0, 0 },
    { 0x44d26812, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e126815, 98 },
    { 0x76126816, 78 },
    { 0x6dd26816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc192681d, 67 },
    { 0xf032681e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68026823, 74 },
    { 0x40626823, 55 },
    { 0, 0 },
    { 0xc6d26826, 98 },
    { 0, 0 },
    { 0x5b726828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a22682b, 99 },
    { 0xdcf2682c, 98 },
    { 0x38f2682c, 34 },
    { 0xeaa2682e, 61 },
    { 0xd112682c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba26832, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd026839, 33 },
    { 0xeb626839, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec2683d, 98 },
    { 0xd492683e, 17 },
    { 0, 0 },
    { 0xb9a26840, 93 },
    { 0xf6026841, 34 },
    { 0xb4926842, 33 },
    { 0xff626841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd926847, 90 },
    { 0xe6426848, 76 },
    { 0x64126849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed2684f, 34 },
    { 0, 0 },
    { 0x1bd26851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d526855, 33 },
    { 0, 0 },
    { 0x4f526857, 83 },
    { 0x4e826857, 98 },
    { 0x10426859, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692685e, 98 },
    { 0xb372685f, 67 },
    { 0, 0 },
    { 0x8be26861, 75 },
    { 0, 0 },
    { 0x31226863, 84 },
    { 0, 0 },
    { 0x7b026865, 98 },
    { 0x1ca26865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac726869, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f42686f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x82326872, 99 },
    { 0xc2c26873, 61 },
    { 0, 0 },
    { 0xa7426875, 90 },
    { 0x15d26875, 76 },
    { 0, 0 },
    { 0x85f26878, 55 },
    { 0xef526879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea2687d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad926889, 55 },
    { 0, 0 },
    { 0x1b12688b, 23 },
    { 0xfb92688c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e26890, 33 },
    { 0xb8726891, 23 },
    { 0x42726892, 33 },
    { 0, 0 },
    { 0x74226894, 93 },
    { 0xd9726895, 98 },
    { 0x85626896, 98 },
    { 0xce526896, 61 },
    { 0, 0 },
    { 0x3af26899, 90 },
    { 0, 0 },
    { 0x2742689b, 23 },
    { 0x8932689b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d268a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb268aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342268ae, 23 },
    { 0x87c268af, 98 },
    { 0, 0 },
    { 0xe76268b1, 90 },
    { 0x4e3268b2, 55 },
    { 0, 0 },
    { 0x376268b4, 33 },
    { 0, 0 },
    { 0xe7d268b6, 23 },
    { 0, 0 },
    { 0x49d268b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5268bc, 90 },
    { 0xf74268bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15268c5, 33 },
    { 0, 0 },
    { 0x2c6268c7, 23 },
    { 0xfdb268c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c268cb, 23 },
    { 0x767268cb, 43 },
    { 0, 0 },
    { 0x9ff268ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a268d1, 43 },
    { 0x39268d2, 67 },
    { 0xe8a268d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f268d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe268de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4268e6, 61 },
    { 0xa56268e6, 43 },
    { 0, 0 },
    { 0xb44268e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2268ed, 46 },
    { 0xcc6268ed, 61 },
    { 0xc6a268ef, 61 },
    { 0xa82268f0, 34 },
    { 0x9bf268f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c268f4, 34 },
    { 0, 0 },
    { 0x696268f6, 23 },
    { 0xe2d268f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3268fa, 23 },
    { 0x67b268fb, 95 },
    { 0x78c268fc, 95 },
    { 0x602268fd, 75 },
    { 0x6e9268fe, 94 },
    { 0x1e2268ff, 61 },
    { 0xdac268ff, 78 },
    { 0xe28268fe, 95 },
    { 0x33826902, 61 },
    { 0x42a26903, 23 },
    { 0x88126904, 84 },
    { 0x41826902, 61 },
    { 0x60126906, 69 },
    { 0x5cb26907, 95 },
    { 0x98426907, 98 },
    { 0x30f26909, 61 },
    { 0xb8826906, 84 },
    { 0, 0 },
    { 0x77b2690c, 66 },
    { 0, 0 },
    { 0xb942690e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a26911, 90 },
    { 0, 0 },
    { 0x2226913, 76 },
    { 0, 0 },
    { 0xc3f26915, 95 },
    { 0xf1026916, 46 },
    { 0x91526917, 34 },
    { 0x9c26917, 95 },
    { 0x9926919, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef82691e, 34 },
    { 0xcb42691f, 98 },
    { 0x9fa26920, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e526926, 87 },
    { 0xd4526927, 33 },
    { 0x6f526926, 75 },
    { 0, 0 },
    { 0x4922692a, 95 },
    { 0, 0 },
    { 0x2792692c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c26930, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb26935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77926938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d2693d, 96 },
    { 0x5982693e, 87 },
    { 0, 0 },
    { 0x92826940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b26943, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1626946, 33 },
    { 0x86226947, 61 },
    { 0, 0 },
    { 0xcf726949, 55 },
    { 0xd352694a, 33 },
    { 0, 0 },
    { 0xc362694c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x792694f, 33 },
    { 0x77526950, 98 },
    { 0, 0 },
    { 0x97326952, 93 },
    { 0x52d26953, 90 },
    { 0x24e26954, 56 },
    { 0x95526953, 33 },
    { 0x70626956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb26961, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1126964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f92696f, 90 },
    { 0xe4126970, 61 },
    { 0xf2026971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a26974, 61 },
    { 0x28d26975, 34 },
    { 0x2ed26974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9726979, 33 },
    { 0xdc326979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb62697e, 85 },
    { 0xa782697f, 23 },
    { 0x2be26980, 76 },
    { 0xf1426981, 43 },
    { 0x9692697e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf26985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7452698c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca26993, 55 },
    { 0, 0 },
    { 0x21226995, 33 },
    { 0x55226996, 95 },
    { 0xa1d26996, 83 },
    { 0xccb26998, 24 },
    { 0xcc926999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5062699d, 95 },
    { 0xa812699d, 95 },
    { 0xb6c2699f, 33 },
    { 0, 0 },
    { 0xcd7269a1, 100 },
    { 0, 0 },
    { 0xfac269a3, 90 },
    { 0, 0 },
    { 0xc33269a5, 23 },
    { 0x6a3269a6, 98 },
    { 0x24f269a7, 90 },
    { 0x29f269a8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c269b0, 73 },
    { 0x23269b1, 55 },
    { 0x99a269b2, 95 },
    { 0x18c269b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d269b6, 95 },
    { 0x9d9269b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332269bf, 23 },
    { 0x84b269bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f269c3, 61 },
    { 0x88f269c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88269c7, 80 },
    { 0x34f269c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8269cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65269d1, 95 },
    { 0x939269d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703269d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90269d8, 98 },
    { 0xe72269d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5269df, 33 },
    { 0, 0 },
    { 0xbb3269e1, 61 },
    { 0xa68269e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15269e6, 55 },
    { 0xafd269e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce269ec, 90 },
    { 0, 0 },
    { 0xf61269ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2269f3, 90 },
    { 0xc87269f3, 43 },
    { 0x8b7269f5, 99 },
    { 0xc21269f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7269fc, 95 },
    { 0xf22269fc, 34 },
    { 0, 0 },
    { 0x2b6269ff, 46 },
    { 0, 0 },
    { 0xdc26a01, 61 },
    { 0xd8226a02, 55 },
    { 0x37926a03, 90 },
    { 0, 0 },
    { 0x34f26a05, 100 },
    { 0xb5526a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0826a0b, 76 },
    { 0, 0 },
    { 0xbc426a0d, 90 },
    { 0, 0 },
    { 0x25626a0f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da26a13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a26a22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d26a29, 61 },
    { 0xec126a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d326a32, 90 },
    { 0, 0 },
    { 0x85026a34, 46 },
    { 0, 0 },
    { 0x25626a36, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d26a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f26a40, 81 },
    { 0xf0226a40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee26a47, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58826a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda926a50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab626a5a, 33 },
    { 0x9c226a5b, 90 },
    { 0x3b726a5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e826a60, 75 },
    { 0x9a926a61, 33 },
    { 0x28926a60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c26a65, 23 },
    { 0xba626a66, 34 },
    { 0x4b926a67, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83926a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e26a6f, 76 },
    { 0xa6c26a70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1626a75, 98 },
    { 0xe5026a75, 90 },
    { 0, 0 },
    { 0x43826a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47926a7c, 61 },
    { 0, 0 },
    { 0x91226a7e, 90 },
    { 0, 0 },
    { 0xba026a80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57226a83, 61 },
    { 0xcec26a84, 90 },
    { 0, 0 },
    { 0x98e26a86, 90 },
    { 0, 0 },
    { 0xe9026a88, 76 },
    { 0xe0d26a89, 95 },
    { 0x58326a8a, 61 },
    { 0xd7826a89, 43 },
    { 0xd3f26a8c, 61 },
    { 0xdee26a88, 90 },
    { 0xa5526a8c, 43 },
    { 0x1d226a89, 75 },
    { 0, 0 },
    { 0xe3b26a91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52226a94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea26a99, 85 },
    { 0xdb226a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee26aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd26aa6, 34 },
    { 0x46326aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe26aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a726aae, 91 },
    { 0, 0 },
    { 0xc1126ab0, 91 },
    { 0, 0 },
    { 0x67a26ab2, 76 },
    { 0x33f26ab3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf926ab7, 33 },
    { 0xc0426ab7, 99 },
    { 0, 0 },
    { 0x8ce26aba, 90 },
    { 0, 0 },
    { 0x4af26abc, 95 },
    { 0x37d26abd, 90 },
    { 0x15d26abe, 23 },
    { 0x3b926abd, 61 },
    { 0xbed26ac0, 78 },
    { 0x59126ac1, 95 },
    { 0xde826ac0, 40 },
    { 0x8f26ac3, 95 },
    { 0xee026ac4, 33 },
    { 0xf6426ac5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d726ace, 95 },
    { 0x3f726acf, 90 },
    { 0x22b26acf, 95 },
    { 0x5e826ad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55126ad5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d26ae8, 67 },
    { 0x9f426ae8, 33 },
    { 0xcfd26aea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27726aee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1226af1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d26af5, 69 },
    { 0x11c26af6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3126af9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46526afe, 81 },
    { 0, 0 },
    { 0x9e626b00, 66 },
    { 0x24e26b01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82226b04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa26b07, 29 },
    { 0x74b26b08, 78 },
    { 0x26226b09, 90 },
    { 0, 0 },
    { 0x73c26b0b, 85 },
    { 0, 0 },
    { 0x6fa26b0d, 90 },
    { 0x78826b0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d26b12, 76 },
    { 0, 0 },
    { 0xdd826b14, 52 },
    { 0, 0 },
    { 0xee026b16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74626b20, 78 },
    { 0x8026b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89026b24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb26b28, 78 },
    { 0x32626b28, 76 },
    { 0x56926b29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc26b2d, 77 },
    { 0, 0 },
    { 0x89f26b2f, 98 },
    { 0x2f826b30, 77 },
    { 0xb1d26b31, 90 },
    { 0x48b26b31, 100 },
    { 0x6dc26b32, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf526b37, 55 },
    { 0xfe926b38, 23 },
    { 0x49d26b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5626b40, 33 },
    { 0xac526b41, 95 },
    { 0, 0 },
    { 0x95226b43, 91 },
    { 0xd4826b44, 90 },
    { 0, 0 },
    { 0x11c26b46, 46 },
    { 0xeb026b46, 90 },
    { 0x23126b47, 46 },
    { 0x1c126b46, 98 },
    { 0x90626b4a, 61 },
    { 0xb6b26b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf026b4e, 95 },
    { 0x8726b4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed926b55, 52 },
    { 0x54b26b56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a026b59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c26b5d, 55 },
    { 0xa7526b5d, 33 },
    { 0x27f26b5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45026b68, 95 },
    { 0xec926b69, 55 },
    { 0, 0 },
    { 0xe5e26b6b, 52 },
    { 0x8926b6c, 90 },
    { 0x31c26b6d, 91 },
    { 0xf3026b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8426b74, 90 },
    { 0x60026b75, 90 },
    { 0x44126b75, 91 },
    { 0, 0 },
    { 0x8c26b78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82326b7d, 51 },
    { 0, 0 },
    { 0x55526b7f, 90 },
    { 0xdd626b80, 33 },
    { 0x93526b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3326b87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86026b8d, 33 },
    { 0, 0 },
    { 0x8f426b8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd26b97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb726b9a, 98 },
    { 0xaee26b9b, 76 },
    { 0, 0 },
    { 0x18c26b9d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31326ba1, 68 },
    { 0, 0 },
    { 0xee26ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a526ba8, 84 },
    { 0, 0 },
    { 0xfd926baa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6326bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d26bb2, 43 },
    { 0, 0 },
    { 0xff726bb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6626bbc, 43 },
    { 0, 0 },
    { 0xca726bbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52226bc2, 81 },
    { 0x1a026bc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17926bc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75126bcc, 98 },
    { 0xb7626bcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c126bd0, 76 },
    { 0xe4226bd1, 43 },
    { 0, 0 },
    { 0xc9c26bd3, 61 },
    { 0, 0 },
    { 0x26a26bd5, 46 },
    { 0xa1926bd6, 78 },
    { 0x80526bd6, 33 },
    { 0x6db26bd6, 33 },
    { 0x5f526bd6, 23 },
    { 0xb7426bd9, 33 },
    { 0xef426bd7, 84 },
    { 0x5f726bdc, 90 },
    { 0x27a26bdc, 52 },
    { 0x1ee26bde, 93 },
    { 0x4fa26bd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21026be2, 33 },
    { 0x7ce26be3, 76 },
    { 0xe0026be4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5826be9, 98 },
    { 0x19e26bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29626bed, 33 },
    { 0, 0 },
    { 0x8da26bef, 23 },
    { 0x4ac26bef, 34 },
    { 0x92a26bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea26bf8, 76 },
    { 0, 0 },
    { 0x9fc26bfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42926bfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a26c09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d426c0c, 55 },
    { 0xec226c0d, 56 },
    { 0, 0 },
    { 0xaf526c0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa26c16, 93 },
    { 0x86526c17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf826c1a, 61 },
    { 0x1ad26c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c21, 43 },
    { 0xadb26c22, 90 },
    { 0x94926c23, 90 },
    { 0xbe326c24, 23 },
    { 0x46326c22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30326c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac26c2c, 46 },
    { 0, 0 },
    { 0xc5826c2e, 98 },
    { 0xdfa26c2e, 99 },
    { 0, 0 },
    { 0xbb426c31, 77 },
    { 0xaf626c31, 66 },
    { 0x34926c33, 90 },
    { 0x3fc26c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a526c38, 90 },
    { 0x6c626c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9626c3f, 61 },
    { 0x58526c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d26c43, 90 },
    { 0, 0 },
    { 0x50226c45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc326c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9726c51, 99 },
    { 0x63b26c52, 55 },
    { 0x9fc26c53, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d26c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5626c59, 33 },
    { 0xeae26c59, 90 },
    { 0, 0 },
    { 0xd6d26c5c, 90 },
    { 0x8cd26c5d, 87 },
    { 0xa1726c5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa426c67, 48 },
    { 0, 0 },
    { 0x8e626c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f26c6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66626c73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826c77, 95 },
    { 0x5c626c78, 100 },
    { 0, 0 },
    { 0xb0126c7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d26c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a26c85, 61 },
    { 0, 0 },
    { 0x4cd26c87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4226c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af26c92, 61 },
    { 0, 0 },
    { 0x7ff26c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1926c9c, 95 },
    { 0x3e726c9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a726ca2, 90 },
    { 0xa3926ca3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc426ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22726cab, 66 },
    { 0x3fb26cac, 33 },
    { 0, 0 },
    { 0x4c426cae, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec726cb3, 76 },
    { 0xd3826cb4, 34 },
    { 0xb4c26cb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x53426cb8, 78 },
    { 0xcee26cb9, 23 },
    { 0xb0026cba, 87 },
    { 0xf2e26cb9, 77 },
    { 0x44e26cbb, 95 },
    { 0x20426cbc, 33 },
    { 0x2e26cbb, 61 },
    { 0x8cf26cbf, 34 },
    { 0x42326cb8, 98 },
    { 0x1f026cc1, 23 },
    { 0x78f26cc2, 85 },
    { 0xb4626cc3, 23 },
    { 0xc5326cc4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e26cc8, 43 },
    { 0xb6726cc9, 95 },
    { 0, 0 },
    { 0xa4c26ccb, 66 },
    { 0, 0 },
    { 0x4a26ccd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3826cd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2826cd5, 95 },
    { 0x3b26cd6, 51 },
    { 0xf3326cd7, 90 },
    { 0x7f126cd8, 87 },
    { 0, 0 },
    { 0x9bb26cda, 87 },
    { 0xbef26cda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea26ce3, 55 },
    { 0xb7d26ce4, 90 },
    { 0xb7826ce4, 90 },
    { 0x81c26ce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde226cec, 91 },
    { 0x18026ced, 75 },
    { 0x3426cee, 23 },
    { 0, 0 },
    { 0xbc726cf0, 91 },
    { 0, 0 },
    { 0x43926cf2, 60 },
    { 0x5a526cf3, 67 },
    { 0x31a26cf4, 91 },
    { 0x15f26cf5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91426cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf126d00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9726d04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c26d08, 87 },
    { 0xe9026d08, 99 },
    { 0xe5326d0a, 98 },
    { 0xeca26d0b, 61 },
    { 0, 0 },
    { 0x4f226d0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45626d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8626d15, 83 },
    { 0, 0 },
    { 0x8fb26d17, 91 },
    { 0xec626d17, 76 },
    { 0x58826d17, 98 },
    { 0xe9c26d1a, 55 },
    { 0x67426d17, 76 },
    { 0x66a26d1c, 90 },
    { 0xb9626d1a, 98 },
    { 0x1d726d1e, 95 },
    { 0x9c826d1f, 55 },
    { 0x4de26d1f, 84 },
    { 0x2a026d21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d226d24, 55 },
    { 0, 0 },
    { 0x4d226d26, 76 },
    { 0x15826d27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f26d2a, 33 },
    { 0, 0 },
    { 0xe2f26d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d26d2f, 78 },
    { 0x51e26d30, 98 },
    { 0xad326d30, 90 },
    { 0xe5726d32, 33 },
    { 0x2b326d32, 90 },
    { 0xe826d34, 66 },
    { 0, 0 },
    { 0x3b026d36, 98 },
    { 0x79c26d37, 23 },
    { 0xfef26d38, 33 },
    { 0x1b626d39, 67 },
    { 0x32b26d39, 61 },
    { 0xb4126d3b, 55 },
    { 0x7626d3a, 95 },
    { 0x1c026d3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19826d41, 98 },
    { 0x92f26d42, 98 },
    { 0x3fa26d43, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xea826d46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca026d4e, 23 },
    { 0x94b26d4f, 34 },
    { 0, 0 },
    { 0xf9a26d51, 98 },
    { 0x64b26d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50326d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc26d5b, 85 },
    { 0, 0 },
    { 0x61f26d5d, 91 },
    { 0x62426d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba26d64, 46 },
    { 0, 0 },
    { 0x53c26d66, 52 },
    { 0xe0226d66, 40 },
    { 0x86626d66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49226d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc226d72, 34 },
    { 0xcc926d73, 99 },
    { 0xb3a26d74, 55 },
    { 0x10326d74, 85 },
    { 0xf8126d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad26d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d726d80, 76 },
    { 0xff726d80, 59 },
    { 0x82426d82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96626d86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40926d8a, 61 },
    { 0xbc826d8b, 23 },
    { 0xd1426d8c, 66 },
    { 0xdbc26d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x30826d90, 33 },
    { 0x46426d91, 78 },
    { 0x8fd26d92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36926d9c, 90 },
    { 0, 0 },
    { 0xdcf26d9e, 33 },
    { 0xfa626d9e, 67 },
    { 0x5cc26da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d26da4, 95 },
    { 0x17626da5, 87 },
    { 0x48126da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226db6, 78 },
    { 0xa4326db6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c26dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c26dbf, 33 },
    { 0xe7926dc0, 95 },
    { 0x17326dbf, 90 },
    { 0xdc826dc2, 52 },
    { 0xe0126dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48826dca, 95 },
    { 0x45a26dcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b26dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4926dd2, 61 },
    { 0, 0 },
    { 0x4fd26dd4, 67 },
    { 0x22f26dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b26ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6626de0, 86 },
    { 0xc3e26de1, 99 },
    { 0x66626de0, 33 },
    { 0xdee26de3, 55 },
    { 0, 0 },
    { 0xf1526de5, 87 },
    { 0x23c26de5, 76 },
    { 0x75d26de7, 95 },
    { 0x81a26de6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75826dec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f126df7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4526dfd, 78 },
    { 0x36826dfe, 23 },
    { 0, 0 },
    { 0x76126e00, 87 },
    { 0x34026e00, 33 },
    { 0x8b126e02, 23 },
    { 0, 0 },
    { 0xc3126e04, 23 },
    { 0x97826e05, 100 },
    { 0, 0 },
    { 0x2bc26e07, 55 },
    { 0xd526e07, 95 },
    { 0, 0 },
    { 0xbd626e0a, 34 },
    { 0xdc526e0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b926e10, 33 },
    { 0x1cc26e10, 33 },
    { 0, 0 },
    { 0x7c226e13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac26e1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b26e1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d26e23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69126e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd226e2f, 33 },
    { 0, 0 },
    { 0x4f326e31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f26e38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8026e3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5626e44, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6726e49, 23 },
    { 0x1c026e49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a226e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f26e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f026e62, 33 },
    { 0xe8726e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa26e68, 61 },
    { 0, 0 },
    { 0x9726e6a, 52 },
    { 0x7bc26e6b, 77 },
    { 0x24b26e6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51626e6f, 98 },
    { 0xcbb26e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e26e79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb26e7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40826e80, 95 },
    { 0, 0 },
    { 0x4cb26e82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57726e85, 84 },
    { 0xcc626e86, 76 },
    { 0xdb426e87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b826e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97026e91, 34 },
    { 0xfb326e92, 85 },
    { 0, 0 },
    { 0x36d26e94, 34 },
    { 0x2a426e95, 61 },
    { 0x6bf26e96, 90 },
    { 0x11a26e97, 23 },
    { 0x5ee26e96, 87 },
    { 0xa3f26e96, 34 },
    { 0x63126e98, 61 },
    { 0xadc26e9b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f426ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d26ea6, 78 },
    { 0x89126ea7, 23 },
    { 0xce526ea8, 43 },
    { 0xa926ea6, 52 },
    { 0x8ff26ea8, 23 },
    { 0x2f026eab, 90 },
    { 0x4dc26ea9, 33 },
    { 0, 0 },
    { 0x34726eae, 34 },
    { 0x86726eaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63626eb5, 33 },
    { 0, 0 },
    { 0xac926eb7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b26ebc, 99 },
    { 0, 0 },
    { 0x58726ebe, 72 },
    { 0xd2326ebe, 46 },
    { 0x86e26ebf, 35 },
    { 0, 0 },
    { 0x9f226ec2, 17 },
    { 0, 0 },
    { 0x13d26ec4, 90 },
    { 0x56426ec5, 17 },
    { 0x94426ec5, 19 },
    { 0x80c26ec7, 68 },
    { 0x33526ec4, 17 },
    { 0xb9d26ec9, 33 },
    { 0x36526ec9, 98 },
    { 0xaa026ec9, 98 },
    { 0xe6d26ecc, 85 },
    { 0x41f26eca, 98 },
    { 0xea26ecc, 91 },
    { 0xd1326ecf, 88 },
    { 0, 0 },
    { 0xc9626ed1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e226ed7, 23 },
    { 0x37926ed8, 90 },
    { 0x91726ed9, 61 },
    { 0, 0 },
    { 0xe8b26edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec26ee6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb626eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60026eef, 98 },
    { 0x50526ef0, 67 },
    { 0x67a26ef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c26ef6, 33 },
    { 0xe8526ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe026efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66226f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b526f10, 52 },
    { 0x47c26f10, 34 },
    { 0xa4726f12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b26f15, 61 },
    { 0x75326f15, 33 },
    { 0xcdc26f16, 23 },
    { 0, 0 },
    { 0x55726f19, 90 },
    { 0x7e626f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1926f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74526f22, 55 },
    { 0, 0 },
    { 0x2c626f24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f26f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33626f2b, 69 },
    { 0xf6726f2c, 93 },
    { 0x96926f2d, 61 },
    { 0x32726f2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4126f31, 46 },
    { 0x69526f32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86526f36, 44 },
    { 0, 0 },
    { 0xf7d26f38, 93 },
    { 0xbe026f39, 34 },
    { 0, 0 },
    { 0x90b26f3b, 90 },
    { 0xd8226f3c, 33 },
    { 0x25b26f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd726f44, 95 },
    { 0x9ed26f45, 98 },
    { 0x87426f46, 90 },
    { 0, 0 },
    { 0xec426f48, 84 },
    { 0, 0 },
    { 0xf0f26f4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba126f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f26f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e26f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed26f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b26f5c, 66 },
    { 0xca126f5c, 76 },
    { 0, 0 },
    { 0xac526f5f, 98 },
    { 0x6ce26f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f26f69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f26f71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e26f76, 23 },
    { 0xfb26f77, 61 },
    { 0xb1426f78, 90 },
    { 0x5df26f79, 46 },
    { 0xe5126f7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f26f7f, 55 },
    { 0x1ae26f7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b26f86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a326f8e, 93 },
    { 0, 0 },
    { 0xba126f90, 61 },
    { 0x82c26f90, 93 },
    { 0, 0 },
    { 0xfab26f93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b26f97, 84 },
    { 0, 0 },
    { 0xf5c26f99, 98 },
    { 0, 0 },
    { 0x75126f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b26f9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e326fa7, 90 },
    { 0x5726fa8, 91 },
    { 0, 0 },
    { 0x4b826faa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40926fb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d26fb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d26fba, 85 },
    { 0xbbc26fba, 33 },
    { 0, 0 },
    { 0x6b26fbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51926fc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326fc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d26fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93626fca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a26fd1, 33 },
    { 0xcfe26fd1, 61 },
    { 0x78b26fd1, 66 },
    { 0, 0 },
    { 0x54d26fd5, 61 },
    { 0xbd326fd5, 98 },
    { 0xa0526fd6, 90 },
    { 0xa9726fd8, 100 },
    { 0xe0026fd8, 67 },
    { 0xc0e26fd9, 77 },
    { 0xe2726fdb, 33 },
    { 0x8c026fdc, 66 },
    { 0x30426fdd, 33 },
    { 0x7fc26fdc, 95 },
    { 0x57426fd8, 99 },
    { 0x68726fd7, 34 },
    { 0, 0 },
    { 0x5ae26fe2, 61 },
    { 0x4026fe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a26fe7, 90 },
    { 0, 0 },
    { 0xe6b26fe9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64926fee, 99 },
    { 0x4e126fee, 98 },
    { 0x45026ff0, 17 },
    { 0x37526ff1, 78 },
    { 0x67826ff1, 76 },
    { 0xc6226fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59026ffb, 51 },
    { 0x32026ffb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93527002, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2700a, 43 },
    { 0xdd92700b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb27011, 99 },
    { 0x5d227012, 23 },
    { 0, 0 },
    { 0x79c27014, 90 },
    { 0x50727014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc427019, 52 },
    { 0xa192701a, 61 },
    { 0x64a2701a, 17 },
    { 0x73d2701c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a27023, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d927028, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb2702d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30527031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e27035, 91 },
    { 0xb1c27036, 33 },
    { 0x38327037, 93 },
    { 0xf0627038, 90 },
    { 0, 0 },
    { 0xf0e2703a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad927040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71227043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb92704a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62704f, 33 },
    { 0, 0 },
    { 0x43d27051, 98 },
    { 0xf2027052, 46 },
    { 0, 0 },
    { 0x22127054, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a27058, 68 },
    { 0, 0 },
    { 0xf6d2705a, 61 },
    { 0, 0 },
    { 0xf982705c, 33 },
    { 0x5592705d, 61 },
    { 0x8572705e, 95 },
    { 0x5082705f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x47327062, 23 },
    { 0x89827063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29127068, 34 },
    { 0x17c27069, 98 },
    { 0x37a2706a, 33 },
    { 0x90d2706b, 98 },
    { 0x94627068, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff27074, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2707a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f2707d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65327082, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10327088, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472708d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca127093, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b27096, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae27099, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732709e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801270a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb270ab, 33 },
    { 0xc56270ac, 90 },
    { 0xc05270ad, 90 },
    { 0xcc7270ad, 33 },
    { 0, 0 },
    { 0xd91270b0, 76 },
    { 0, 0 },
    { 0x394270b2, 90 },
    { 0xe53270b3, 90 },
    { 0xe49270b4, 90 },
    { 0x37270b3, 90 },
    { 0x7ca270b4, 34 },
    { 0xc31270b7, 33 },
    { 0, 0 },
    { 0x79f270b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6270c4, 46 },
    { 0xdfa270c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0270cd, 61 },
    { 0x706270ce, 96 },
    { 0x62b270ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5270d5, 33 },
    { 0xf47270d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9270dc, 55 },
    { 0x6fe270dc, 46 },
    { 0x595270dd, 23 },
    { 0x558270df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93270eb, 52 },
    { 0x58e270eb, 90 },
    { 0x68f270eb, 33 },
    { 0, 0 },
    { 0x74c270ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b270f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8270f5, 33 },
    { 0x164270f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc270f9, 33 },
    { 0xc4c270f9, 43 },
    { 0xadd270fb, 17 },
    { 0, 0 },
    { 0x547270fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8327105, 95 },
    { 0xcc727105, 66 },
    { 0x47827107, 2 },
    { 0x1b027107, 34 },
    { 0xdf727105, 98 },
    { 0x16a2710a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf327111, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2327115, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2711d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58827123, 61 },
    { 0xc0c27123, 95 },
    { 0x79927125, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52712c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9627133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e27139, 66 },
    { 0x802713a, 52 },
    { 0x22327139, 61 },
    { 0x1982713c, 33 },
    { 0x5a72713d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1227140, 95 },
    { 0x60627141, 66 },
    { 0x2cb27142, 34 },
    { 0, 0 },
    { 0xf3c27144, 98 },
    { 0, 0 },
    { 0x4f127146, 34 },
    { 0, 0 },
    { 0x7a827148, 90 },
    { 0x3d527149, 87 },
    { 0x6c827148, 95 },
    { 0x6b42714b, 90 },
    { 0x4bd2714b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f27153, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a27156, 99 },
    { 0, 0 },
    { 0xb2527158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2715d, 80 },
    { 0x6d2715d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc927161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3972716a, 76 },
    { 0x7a22716b, 48 },
    { 0x69f2716c, 55 },
    { 0x2402716d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68027172, 61 },
    { 0, 0 },
    { 0x8be27174, 55 },
    { 0, 0 },
    { 0x47927176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532717a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd562717e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad27184, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27188, 55 },
    { 0xc1a27189, 68 },
    { 0x7827189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf27191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b27195, 85 },
    { 0x1c27196, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe62719f, 93 },
    { 0, 0 },
    { 0x6ae271a1, 33 },
    { 0x8c1271a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e271a6, 90 },
    { 0x672271a7, 33 },
    { 0x2ed271a6, 91 },
    { 0, 0 },
    { 0x928271aa, 33 },
    { 0xe51271aa, 43 },
    { 0, 0 },
    { 0xf04271ad, 95 },
    { 0xfb3271ad, 91 },
    { 0xfc0271af, 98 },
    { 0x895271ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4271b6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c271ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749271bf, 76 },
    { 0, 0 },
    { 0xacb271c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a271c4, 95 },
    { 0, 0 },
    { 0xfb5271c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c271cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37271d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4271d3, 100 },
    { 0x7b8271d4, 98 },
    { 0xb99271d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f271dc, 33 },
    { 0x1d2271dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f271e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77271e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc271ee, 95 },
    { 0x4ff271ef, 33 },
    { 0, 0 },
    { 0x2ab271f1, 91 },
    { 0, 0 },
    { 0xf27271f3, 98 },
    { 0xc7e271f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632271f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08271fd, 81 },
    { 0x285271fe, 61 },
    { 0x58271fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f27207, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac62720e, 66 },
    { 0xa402720f, 55 },
    { 0x39b27210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7227215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22721c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a027222, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44827228, 52 },
    { 0, 0 },
    { 0x49c2722a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f2722d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77727230, 33 },
    { 0x8d127231, 34 },
    { 0x9c927232, 98 },
    { 0x11827233, 95 },
    { 0xdd327234, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29627237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02723a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7232723d, 61 },
    { 0, 0 },
    { 0x29e2723f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e827243, 23 },
    { 0, 0 },
    { 0xd3827245, 95 },
    { 0x3c227246, 55 },
    { 0x38a27246, 98 },
    { 0, 0 },
    { 0x80427249, 61 },
    { 0xed2724a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb2724d, 100 },
    { 0, 0 },
    { 0x6382724f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e2725f, 98 },
    { 0, 0 },
    { 0x4d27261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9427265, 90 },
    { 0xac327265, 33 },
    { 0x68a27267, 55 },
    { 0x82827268, 33 },
    { 0xd7027269, 67 },
    { 0x3da2726a, 61 },
    { 0x9662726b, 98 },
    { 0xaf2726c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7427276, 61 },
    { 0x1cd27276, 46 },
    { 0xcef27277, 33 },
    { 0x17627279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582727e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e27286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba2728d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93627291, 33 },
    { 0, 0 },
    { 0xc427293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd27296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6427299, 88 },
    { 0, 0 },
    { 0x7772729b, 88 },
    { 0, 0 },
    { 0xf1e2729d, 23 },
    { 0x14e2729d, 46 },
    { 0x5c72729e, 52 },
    { 0x36e272a0, 87 },
    { 0, 0 },
    { 0x2d1272a2, 61 },
    { 0, 0 },
    { 0xe07272a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e272a9, 67 },
    { 0xfa3272aa, 55 },
    { 0x85a272ab, 23 },
    { 0x86f272aa, 17 },
    { 0x979272ad, 33 },
    { 0x58272ab, 33 },
    { 0x27a272af, 33 },
    { 0x5c3272b0, 61 },
    { 0xd48272b1, 40 },
    { 0xe45272aa, 92 },
    { 0, 0 },
    { 0x5ad272b4, 90 },
    { 0xee3272b5, 95 },
    { 0, 0 },
    { 0x3f9272b7, 89 },
    { 0xadd272b7, 78 },
    { 0xc80272b9, 33 },
    { 0x69f272ba, 98 },
    { 0xfc9272ba, 70 },
    { 0xed5272bc, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab272bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7272c5, 74 },
    { 0xf40272c6, 95 },
    { 0, 0 },
    { 0xc38272c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501272d1, 98 },
    { 0x292272d2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x524272d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4272d9, 33 },
    { 0x935272da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f272e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669272e8, 77 },
    { 0xb23272e8, 61 },
    { 0, 0 },
    { 0x97d272eb, 90 },
    { 0xea6272ec, 33 },
    { 0x205272ed, 52 },
    { 0x2fd272ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5272f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10272f4, 98 },
    { 0x675272f5, 98 },
    { 0x226272f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd272fe, 61 },
    { 0x66272ff, 33 },
    { 0xfb27300, 67 },
    { 0x72327301, 91 },
    { 0xa5427302, 33 },
    { 0x59427300, 98 },
    { 0x85b27304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43027307, 69 },
    { 0xd4b27307, 55 },
    { 0x68427309, 55 },
    { 0, 0 },
    { 0x67d2730b, 90 },
    { 0xe862730c, 33 },
    { 0x4f72730c, 43 },
    { 0x83c2730c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c27317, 67 },
    { 0xe6727318, 34 },
    { 0xa1227319, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d2731c, 87 },
    { 0x55d2731c, 52 },
    { 0xc3e2731c, 66 },
    { 0, 0 },
    { 0xfac27320, 85 },
    { 0x13827320, 95 },
    { 0x36127322, 95 },
    { 0x3d727323, 101 },
    { 0, 0 },
    { 0x2bd27325, 78 },
    { 0, 0 },
    { 0x44a27327, 33 },
    { 0x5ee27327, 23 },
    { 0x5d927329, 95 },
    { 0x9c02732a, 33 },
    { 0xab62732b, 34 },
    { 0x32127327, 97 },
    { 0, 0 },
    { 0x4722732e, 91 },
    { 0x1fb2732f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f527332, 91 },
    { 0, 0 },
    { 0x89027334, 65 },
    { 0, 0 },
    { 0xf8527336, 61 },
    { 0x5ca27337, 33 },
    { 0x13b27337, 75 },
    { 0xbfe27338, 43 },
    { 0x1052733a, 76 },
    { 0xb882733b, 93 },
    { 0x43e2733c, 92 },
    { 0x43d2733a, 95 },
    { 0xc382733e, 33 },
    { 0x91d2733e, 55 },
    { 0x9a227340, 76 },
    { 0xec027339, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8627344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5427347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332734b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf27354, 66 },
    { 0x51f27354, 91 },
    { 0xa6a27356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f2735c, 66 },
    { 0xa2a2735c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba327360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60627363, 90 },
    { 0xd0c27364, 95 },
    { 0xf4b27363, 85 },
    { 0, 0 },
    { 0x4b427367, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a27370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b27378, 95 },
    { 0xb0427379, 23 },
    { 0x7f42737a, 46 },
    { 0xfe52737b, 95 },
    { 0x6e2737c, 98 },
    { 0x3822737c, 61 },
    { 0, 0 },
    { 0xd102737f, 98 },
    { 0x78427380, 66 },
    { 0xd3b2737f, 67 },
    { 0, 0 },
    { 0xba127383, 98 },
    { 0x80a27384, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c27387, 61 },
    { 0x8e27388, 95 },
    { 0, 0 },
    { 0x2512738a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa2738e, 98 },
    { 0x2292738f, 61 },
    { 0, 0 },
    { 0x9127391, 91 },
    { 0x71d27392, 61 },
    { 0, 0 },
    { 0x4d427394, 33 },
    { 0x16d27394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9882739a, 91 },
    { 0xbb22739b, 61 },
    { 0xc7d2739c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212273a1, 61 },
    { 0x4bc273a2, 46 },
    { 0x461273a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c273a7, 95 },
    { 0, 0 },
    { 0x744273a9, 61 },
    { 0, 0 },
    { 0x5d1273ab, 75 },
    { 0, 0 },
    { 0x52a273ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7273b1, 23 },
    { 0xd91273b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82273b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23273bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x610273be, 95 },
    { 0xc13273bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd273c5, 90 },
    { 0x9ac273c5, 76 },
    { 0xdfe273c7, 90 },
    { 0, 0 },
    { 0x287273c9, 33 },
    { 0x11273ca, 90 },
    { 0xaa0273c9, 66 },
    { 0xbcb273cc, 100 },
    { 0, 0 },
    { 0x45a273ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf273d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f273d7, 61 },
    { 0x77f273d7, 67 },
    { 0, 0 },
    { 0x47f273da, 98 },
    { 0x3a6273da, 98 },
    { 0x3ae273dc, 93 },
    { 0x113273da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x632273e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9273e4, 95 },
    { 0x5c9273e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc273e9, 90 },
    { 0x219273e9, 38 },
    { 0xa46273ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c273ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32273f1, 68 },
    { 0xe41273f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9273f6, 23 },
    { 0x57e273f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b273fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd527401, 90 },
    { 0xc0f27402, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af27405, 76 },
    { 0xa4727406, 43 },
    { 0, 0 },
    { 0x21927408, 46 },
    { 0, 0 },
    { 0x30c2740a, 17 },
    { 0x3442740a, 98 },
    { 0x1d2740b, 90 },
    { 0xdf52740d, 34 },
    { 0x7fd2740c, 93 },
    { 0x3c32740a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9527418, 61 },
    { 0, 0 },
    { 0x7342741a, 33 },
    { 0xe952741b, 87 },
    { 0x9432741c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a27421, 93 },
    { 0x2c027422, 46 },
    { 0x78427422, 75 },
    { 0x7fe27424, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8627429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762742e, 95 },
    { 0xc052742f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6327434, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b27438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f02743d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b827441, 61 },
    { 0x59727442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0627445, 90 },
    { 0, 0 },
    { 0x81527447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2744b, 61 },
    { 0x4c92744b, 99 },
    { 0x7332744d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f27454, 85 },
    { 0x98527455, 33 },
    { 0x6627456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e32745f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8227469, 33 },
    { 0xd6927469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9632746e, 95 },
    { 0x4922746f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f27472, 93 },
    { 0xd1327472, 23 },
    { 0x6a727472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a27477, 92 },
    { 0x29f27478, 43 },
    { 0, 0 },
    { 0x38e2747a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12747d, 33 },
    { 0xf792747e, 61 },
    { 0xa9f2747f, 87 },
    { 0xa3a2747d, 90 },
    { 0x90e27481, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be27486, 23 },
    { 0x64c27487, 94 },
    { 0x3ae27488, 99 },
    { 0x9127486, 61 },
    { 0xd3b2748a, 19 },
    { 0x73f27486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c427496, 95 },
    { 0x39227497, 77 },
    { 0, 0 },
    { 0x45f27499, 61 },
    { 0x9452749a, 98 },
    { 0x6172749a, 61 },
    { 0x4f2749c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc274a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe274a8, 61 },
    { 0, 0 },
    { 0xadd274aa, 95 },
    { 0x80d274ab, 34 },
    { 0x17a274ac, 90 },
    { 0xb5274aa, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72274b0, 45 },
    { 0, 0 },
    { 0x7f4274b2, 95 },
    { 0, 0 },
    { 0x1eb274b4, 61 },
    { 0x6b2274b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb274bd, 76 },
    { 0, 0 },
    { 0x929274bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6274c3, 95 },
    { 0, 0 },
    { 0xf9f274c5, 61 },
    { 0x1d6274c6, 55 },
    { 0x133274c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac274cb, 85 },
    { 0x925274cb, 76 },
    { 0, 0 },
    { 0xd8a274ce, 61 },
    { 0x9bf274cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b274d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc274df, 23 },
    { 0x50a274e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x615274e3, 33 },
    { 0x5fe274e4, 61 },
    { 0x18f274e3, 61 },
    { 0x7b0274e5, 95 },
    { 0, 0 },
    { 0xa14274e8, 77 },
    { 0, 0 },
    { 0xe0c274ea, 90 },
    { 0, 0 },
    { 0xd63274ec, 68 },
    { 0, 0 },
    { 0x995274ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355274f4, 91 },
    { 0x7e0274f5, 17 },
    { 0, 0 },
    { 0x45b274f7, 33 },
    { 0x393274f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec27502, 33 },
    { 0xbf927502, 95 },
    { 0x48027504, 93 },
    { 0x73d27505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f27508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1427513, 91 },
    { 0x40c27514, 23 },
    { 0x72d27515, 61 },
    { 0x21b27516, 55 },
    { 0, 0 },
    { 0x24527518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2751b, 101 },
    { 0x61c2751c, 85 },
    { 0x7872751b, 15 },
    { 0xf602751e, 44 },
    { 0x3192751e, 43 },
    { 0x68627520, 33 },
    { 0x59f27520, 61 },
    { 0xce427522, 61 },
    { 0x7a927523, 33 },
    { 0x67a2751f, 61 },
    { 0x35027525, 95 },
    { 0x7c227525, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x53327529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2752d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc27531, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c27536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b42753b, 98 },
    { 0x8462753c, 55 },
    { 0xdbc2753d, 78 },
    { 0xee42753e, 90 },
    { 0, 0 },
    { 0xf4527540, 23 },
    { 0x3e427541, 76 },
    { 0x58c27540, 90 },
    { 0xbd727542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af27546, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c427549, 61 },
    { 0x4342754a, 84 },
    { 0xea2754b, 90 },
    { 0xc292754c, 33 },
    { 0xe4f2754d, 78 },
    { 0xf7c2754e, 90 },
    { 0xd4f2754d, 46 },
    { 0, 0 },
    { 0x9fc27551, 55 },
    { 0, 0 },
    { 0x96027553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff82755c, 61 },
    { 0, 0 },
    { 0x9b42755e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe27561, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f27567, 33 },
    { 0x38f27568, 90 },
    { 0, 0 },
    { 0x95d2756a, 61 },
    { 0, 0 },
    { 0x67d2756c, 23 },
    { 0xcbc2756d, 90 },
    { 0xa402756d, 23 },
    { 0, 0 },
    { 0xc1627570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e427573, 98 },
    { 0xa1027574, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3927577, 61 },
    { 0x80e27578, 34 },
    { 0xb4427577, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd2757e, 95 },
    { 0xbd82757f, 81 },
    { 0xb782757e, 76 },
    { 0x60d27580, 52 },
    { 0xa2627582, 66 },
    { 0x1b32757f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b227589, 33 },
    { 0x5352758a, 61 },
    { 0xb332758a, 55 },
    { 0x3ca2758c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91527590, 23 },
    { 0, 0 },
    { 0xb6c27592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee127596, 76 },
    { 0xe5827597, 23 },
    { 0xf2d27596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5275a0, 90 },
    { 0x2ea275a1, 61 },
    { 0, 0 },
    { 0x657275a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494275a9, 33 },
    { 0xccf275aa, 52 },
    { 0x4c0275ab, 55 },
    { 0xa05275ac, 98 },
    { 0xa41275ab, 17 },
    { 0x590275ad, 61 },
    { 0x498275af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f275b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95275bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d275bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4275c2, 33 },
    { 0, 0 },
    { 0x72a275c4, 100 },
    { 0xb8275c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x467275c8, 98 },
    { 0xb2d275c8, 33 },
    { 0xeb2275ca, 98 },
    { 0xbd5275cb, 68 },
    { 0x710275cb, 98 },
    { 0xc2f275cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77275d1, 99 },
    { 0x9a9275d2, 66 },
    { 0xe8c275d3, 61 },
    { 0x2e4275d3, 76 },
    { 0x512275d4, 46 },
    { 0x14f275d1, 99 },
    { 0x9dc275d7, 61 },
    { 0x422275d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be275e0, 91 },
    { 0x531275e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b275e7, 33 },
    { 0xeba275e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3275ef, 33 },
    { 0xdcd275ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x260275f3, 90 },
    { 0xd37275f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765275f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74275fb, 88 },
    { 0x25e275fb, 90 },
    { 0x850275fb, 76 },
    { 0xccc275fc, 66 },
    { 0xba4275ff, 61 },
    { 0x509275fd, 90 },
    { 0x81c275fd, 61 },
    { 0x67a275fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627608, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d2760b, 91 },
    { 0, 0 },
    { 0xe082760d, 43 },
    { 0x12f2760e, 33 },
    { 0x95f2760d, 90 },
    { 0, 0 },
    { 0x3da27611, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b27614, 61 },
    { 0xac727615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83827618, 76 },
    { 0, 0 },
    { 0xe622761a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2761d, 67 },
    { 0x3542761d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5727627, 90 },
    { 0xcd027628, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda32762f, 46 },
    { 0x18e27630, 43 },
    { 0x32e27630, 61 },
    { 0x2d727630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8627636, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a27642, 90 },
    { 0x90327643, 52 },
    { 0xdaf27643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f27647, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9712764a, 81 },
    { 0, 0 },
    { 0x8992764c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95127656, 66 },
    { 0, 0 },
    { 0x53a27658, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f627661, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0127666, 33 },
    { 0x14427667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912766c, 99 },
    { 0x69b2766c, 100 },
    { 0x60b2766e, 90 },
    { 0xc132766e, 95 },
    { 0x18227670, 95 },
    { 0, 0 },
    { 0x9b127672, 90 },
    { 0xc0127672, 93 },
    { 0xa1a27674, 98 },
    { 0x6827673, 98 },
    { 0, 0 },
    { 0xdf827677, 95 },
    { 0, 0 },
    { 0x2d027679, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2767c, 23 },
    { 0, 0 },
    { 0x8fe2767e, 76 },
    { 0, 0 },
    { 0x4f27680, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2527683, 90 },
    { 0x51927683, 69 },
    { 0xeb27685, 87 },
    { 0x94527683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a727689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9142768c, 76 },
    { 0x13b2768c, 55 },
    { 0, 0 },
    { 0x592768f, 83 },
    { 0xf0b27690, 33 },
    { 0, 0 },
    { 0x37927692, 23 },
    { 0, 0 },
    { 0x47527694, 98 },
    { 0x4d27694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa427699, 98 },
    { 0x6582769a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b32769e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf276a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf276a9, 23 },
    { 0, 0 },
    { 0xbdf276ab, 85 },
    { 0x1dc276ac, 77 },
    { 0xcb0276ad, 61 },
    { 0xafe276ad, 95 },
    { 0x93c276ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3276b3, 34 },
    { 0, 0 },
    { 0x462276b5, 90 },
    { 0, 0 },
    { 0x869276b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744276bb, 77 },
    { 0x3a0276bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39276c2, 33 },
    { 0, 0 },
    { 0x9b9276c4, 93 },
    { 0x161276c5, 76 },
    { 0xa7d276c5, 55 },
    { 0, 0 },
    { 0x3a276c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58276cc, 33 },
    { 0xab1276cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda276d5, 43 },
    { 0xb3d276d6, 91 },
    { 0x7ba276d7, 61 },
    { 0xa33276d8, 87 },
    { 0xa5276d7, 95 },
    { 0x260276da, 33 },
    { 0x980276da, 90 },
    { 0, 0 },
    { 0x89f276dd, 66 },
    { 0xe29276de, 90 },
    { 0x1df276df, 43 },
    { 0xf2b276e0, 95 },
    { 0, 0 },
    { 0xc67276e2, 87 },
    { 0, 0 },
    { 0x7ec276e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b276ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7276f2, 99 },
    { 0, 0 },
    { 0x955276f4, 55 },
    { 0x753276f5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f276f8, 90 },
    { 0x5fe276f9, 34 },
    { 0x380276fa, 17 },
    { 0xcdf276fb, 61 },
    { 0, 0 },
    { 0xb4276fd, 17 },
    { 0, 0 },
    { 0x862276ff, 90 },
    { 0, 0 },
    { 0x5ef27701, 85 },
    { 0, 0 },
    { 0xf3627703, 76 },
    { 0x66a27703, 75 },
    { 0, 0 },
    { 0xace27706, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc292770c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80927711, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf27714, 75 },
    { 0x5e327715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a2771b, 34 },
    { 0, 0 },
    { 0xab82771d, 61 },
    { 0x4a92771e, 90 },
    { 0x6812771f, 61 },
    { 0, 0 },
    { 0x71f27721, 33 },
    { 0x9f127721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f827727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca22772a, 67 },
    { 0x9b52772a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11127730, 76 },
    { 0xd6227731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f27734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9982773a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92773d, 66 },
    { 0xf462773e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f27744, 100 },
    { 0, 0 },
    { 0xf5727746, 88 },
    { 0xd0727747, 91 },
    { 0x31227747, 87 },
    { 0x8027748, 61 },
    { 0x60f2774a, 97 },
    { 0xb902774b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a2774e, 66 },
    { 0x8482774e, 75 },
    { 0, 0 },
    { 0xf1427751, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2227754, 61 },
    { 0x8a427754, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d27758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f02775d, 90 },
    { 0, 0 },
    { 0xd5d2775f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1927769, 93 },
    { 0xc882776a, 34 },
    { 0x44b2776b, 66 },
    { 0x95f2776a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce427772, 61 },
    { 0, 0 },
    { 0x4f827774, 98 },
    { 0x14027775, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752777a, 93 },
    { 0, 0 },
    { 0xbb72777c, 33 },
    { 0x3502777c, 23 },
    { 0x87f2777d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d27781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb27784, 17 },
    { 0x96e27785, 85 },
    { 0x9b727786, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac27789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072778c, 61 },
    { 0xf752778c, 33 },
    { 0, 0 },
    { 0xf462778f, 23 },
    { 0xbf72778f, 52 },
    { 0x9ab27791, 61 },
    { 0, 0 },
    { 0x81527793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d27796, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b27799, 55 },
    { 0xcd727799, 90 },
    { 0, 0 },
    { 0x84a2779c, 67 },
    { 0, 0 },
    { 0x20b2779e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83277a5, 61 },
    { 0, 0 },
    { 0xaf7277a7, 76 },
    { 0xd32277a7, 33 },
    { 0xef7277a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693277b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e277b8, 52 },
    { 0x82e277b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0277c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106277c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d277c9, 90 },
    { 0x39d277ca, 77 },
    { 0xee0277cb, 23 },
    { 0xdf9277cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6277cf, 95 },
    { 0x29277d0, 61 },
    { 0xe85277d1, 3 },
    { 0x646277d0, 99 },
    { 0x927277d3, 90 },
    { 0x8c277d4, 52 },
    { 0x1bb277d4, 34 },
    { 0xd1d277d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9277d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08277de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8277e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6277e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413277ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08277f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92277fe, 61 },
    { 0, 0 },
    { 0x94227800, 90 },
    { 0x32d27801, 93 },
    { 0x78427802, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe427809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72780f, 90 },
    { 0x55d27810, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d2781a, 24 },
    { 0x9112781b, 33 },
    { 0, 0 },
    { 0x6492781d, 76 },
    { 0x40f2781e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d27822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5427825, 33 },
    { 0, 0 },
    { 0x9d727827, 85 },
    { 0x57d27828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a62782c, 61 },
    { 0x5342782d, 36 },
    { 0x10d2782e, 95 },
    { 0xb782782e, 61 },
    { 0x87a2782d, 43 },
    { 0x3a72782e, 33 },
    { 0x87127832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3727835, 98 },
    { 0, 0 },
    { 0x98827837, 76 },
    { 0x2ba27837, 91 },
    { 0x6dc27837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2783e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f27843, 61 },
    { 0x53727843, 34 },
    { 0, 0 },
    { 0xba27846, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f27849, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6127850, 33 },
    { 0x1d927850, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37627855, 33 },
    { 0, 0 },
    { 0x9b827857, 76 },
    { 0x7ef27857, 61 },
    { 0x93927859, 73 },
    { 0x47027859, 90 },
    { 0x66a2785a, 23 },
    { 0x5bd2785c, 76 },
    { 0x39727859, 98 },
    { 0x66d2785e, 23 },
    { 0xd512785f, 90 },
    { 0x9d72785f, 61 },
    { 0x53b27861, 90 },
    { 0x42927861, 67 },
    { 0, 0 },
    { 0x2b227864, 75 },
    { 0, 0 },
    { 0xc6d27866, 55 },
    { 0x8427867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x482786a, 52 },
    { 0xf802786a, 55 },
    { 0, 0 },
    { 0xc032786d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e927873, 98 },
    { 0xa1527874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88427877, 23 },
    { 0x9e627877, 55 },
    { 0x8b327879, 91 },
    { 0xd192787a, 64 },
    { 0, 0 },
    { 0xc792787c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec27882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1327888, 33 },
    { 0x7fb27889, 55 },
    { 0xc362788a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb92788e, 90 },
    { 0x9c02788e, 90 },
    { 0x2c27890, 61 },
    { 0x47c27890, 76 },
    { 0xb027892, 23 },
    { 0xa3a27892, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb322789c, 95 },
    { 0x80a2789c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266278a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6278a8, 94 },
    { 0, 0 },
    { 0xda4278aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2278ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8278b7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8278bd, 85 },
    { 0xe29278be, 67 },
    { 0, 0 },
    { 0x6e278c0, 93 },
    { 0, 0 },
    { 0x3ef278c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228278c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650278ce, 55 },
    { 0x25a278ce, 68 },
    { 0xbfc278cf, 61 },
    { 0xd53278d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0278d6, 33 },
    { 0, 0 },
    { 0xc5a278d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c278dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6278e0, 90 },
    { 0x6d9278e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8278e5, 55 },
    { 0x448278e6, 95 },
    { 0x366278e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10278ea, 90 },
    { 0xbc8278eb, 85 },
    { 0x9c1278eb, 76 },
    { 0x233278ed, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38278f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41278f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f278f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00278fa, 98 },
    { 0x9ae278fb, 77 },
    { 0, 0 },
    { 0x44c278fd, 90 },
    { 0xb95278fe, 90 },
    { 0, 0 },
    { 0x83227900, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982790a, 49 },
    { 0x81e2790b, 61 },
    { 0xf0b2790b, 98 },
    { 0x3892790b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf27910, 23 },
    { 0, 0 },
    { 0xf6527912, 33 },
    { 0x26e27913, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb227916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea52791d, 90 },
    { 0, 0 },
    { 0x1952791f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77127926, 33 },
    { 0xb5427926, 78 },
    { 0xca627928, 76 },
    { 0x3f327926, 33 },
    { 0xf6c2792a, 61 },
    { 0xef82792b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92027930, 98 },
    { 0, 0 },
    { 0x7c27932, 95 },
    { 0x5f927933, 36 },
    { 0xb0327933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5327937, 90 },
    { 0xa8e27938, 61 },
    { 0x1b427939, 93 },
    { 0xc7b2793a, 61 },
    { 0, 0 },
    { 0xf082793c, 55 },
    { 0x9b12793d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28127949, 76 },
    { 0xf752794a, 23 },
    { 0xda12794b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f27951, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1027956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d2795b, 90 },
    { 0xa1f2795c, 23 },
    { 0, 0 },
    { 0x57f2795e, 76 },
    { 0, 0 },
    { 0x87527960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b627964, 76 },
    { 0x43927964, 83 },
    { 0x6627964, 90 },
    { 0x9ce27965, 33 },
    { 0xfe227964, 61 },
    { 0, 0 },
    { 0x34d2796a, 66 },
    { 0xdee2796b, 43 },
    { 0x5162796b, 98 },
    { 0, 0 },
    { 0x5792796e, 93 },
    { 0x88d2796e, 90 },
    { 0, 0 },
    { 0x50227971, 35 },
    { 0, 0 },
    { 0xb5027973, 77 },
    { 0, 0 },
    { 0x93f27975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4512797b, 90 },
    { 0x312797c, 23 },
    { 0xd602797d, 55 },
    { 0xc512797e, 90 },
    { 0x9ac2797f, 66 },
    { 0x9bb27980, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827984, 90 },
    { 0, 0 },
    { 0x36a27986, 87 },
    { 0x8ec27986, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b22798c, 64 },
    { 0, 0 },
    { 0xd042798e, 95 },
    { 0xa392798e, 90 },
    { 0x76627990, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1127995, 33 },
    { 0xa9727996, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2527999, 33 },
    { 0x1727999, 91 },
    { 0x7122799b, 100 },
    { 0xaa32799c, 61 },
    { 0xc332799c, 52 },
    { 0x8fe2799e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162279a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc279a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x176279ab, 17 },
    { 0, 0 },
    { 0x3ce279ad, 91 },
    { 0x617279ae, 90 },
    { 0, 0 },
    { 0x197279b0, 61 },
    { 0xa43279b0, 90 },
    { 0x6ce279b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b279b7, 76 },
    { 0, 0 },
    { 0x6c279b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0279be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98279c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4279c9, 65 },
    { 0x461279ca, 33 },
    { 0xa14279c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b279d2, 55 },
    { 0xf82279d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a279d9, 61 },
    { 0x400279d9, 55 },
    { 0, 0 },
    { 0xf58279dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2279df, 33 },
    { 0x9c5279df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71279ea, 61 },
    { 0x76a279eb, 90 },
    { 0x710279ec, 98 },
    { 0xdba279ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d279f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x229279f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3279fa, 76 },
    { 0x193279fa, 34 },
    { 0x4a4279fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85527a02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c27a05, 68 },
    { 0, 0 },
    { 0xc7a27a07, 80 },
    { 0xc9e27a07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6927a0d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa27a13, 34 },
    { 0, 0 },
    { 0x22127a15, 61 },
    { 0, 0 },
    { 0xa5627a17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e827a1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7227a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27a29, 90 },
    { 0x2b927a2a, 85 },
    { 0x2fc27a2a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec927a30, 92 },
    { 0, 0 },
    { 0xbea27a32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3327a3c, 66 },
    { 0, 0 },
    { 0x89727a3e, 98 },
    { 0, 0 },
    { 0x17d27a40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b227a44, 68 },
    { 0x35127a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427a4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce27a4d, 95 },
    { 0, 0 },
    { 0x24827a4f, 90 },
    { 0x7b227a50, 33 },
    { 0x8aa27a50, 23 },
    { 0, 0 },
    { 0xd8627a53, 98 },
    { 0, 0 },
    { 0xbad27a55, 78 },
    { 0xa6e27a56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45227a59, 76 },
    { 0x2f127a5a, 46 },
    { 0, 0 },
    { 0xd4527a5c, 98 },
    { 0, 0 },
    { 0x2bd27a5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83327a61, 33 },
    { 0xed527a61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c27a68, 43 },
    { 0xecd27a69, 33 },
    { 0x5fb27a69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b227a6d, 76 },
    { 0, 0 },
    { 0xd8027a6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e27a74, 33 },
    { 0x8ca27a74, 78 },
    { 0x70e27a76, 23 },
    { 0x1f627a77, 77 },
    { 0x9d627a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a27a7e, 90 },
    { 0xab527a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a527a88, 77 },
    { 0x7a827a89, 61 },
    { 0x9f427a88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc27a8e, 95 },
    { 0xd8c27a8e, 33 },
    { 0x33327a90, 98 },
    { 0x44627a8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f27a95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23927a98, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d27a9e, 61 },
    { 0xf6727a9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78727aa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d27aab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d27ab0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd927ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d427ac5, 55 },
    { 0xf6a27ac6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc27ac9, 90 },
    { 0x95927aca, 84 },
    { 0xff27ac9, 33 },
    { 0xdf727acc, 98 },
    { 0x45e27acd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c427ad0, 95 },
    { 0, 0 },
    { 0x93c27ad2, 43 },
    { 0xd0d27ad3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d327ad6, 61 },
    { 0, 0 },
    { 0x23927ad8, 78 },
    { 0x26727ad8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1827ade, 33 },
    { 0, 0 },
    { 0x3327ae0, 98 },
    { 0, 0 },
    { 0xf1527ae2, 66 },
    { 0x25927ae2, 80 },
    { 0x38f27ae2, 76 },
    { 0xea827ae5, 34 },
    { 0xf9827ae6, 23 },
    { 0xa8527ae3, 76 },
    { 0, 0 },
    { 0xd7a27ae9, 90 },
    { 0x73927ae9, 23 },
    { 0, 0 },
    { 0xee27aec, 34 },
    { 0, 0 },
    { 0x74027aee, 90 },
    { 0x14627aee, 90 },
    { 0xc5827aee, 97 },
    { 0x2be27aee, 38 },
    { 0, 0 },
    { 0xe9f27af3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2627b02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74727b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14827b0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1627b0f, 61 },
    { 0x11627b10, 95 },
    { 0x85f27b10, 76 },
    { 0x34527b0f, 95 },
    { 0, 0 },
    { 0x6827b14, 33 },
    { 0x7b627b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb227b20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0727b23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda227b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b27b35, 40 },
    { 0, 0 },
    { 0x31a27b37, 95 },
    { 0xaad27b38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6527b3d, 33 },
    { 0, 0 },
    { 0xf8827b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97527b42, 40 },
    { 0, 0 },
    { 0xab627b44, 67 },
    { 0, 0 },
    { 0xe8927b46, 98 },
    { 0x7d027b47, 93 },
    { 0x6c227b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b527b4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb27b54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb27b57, 32 },
    { 0, 0 },
    { 0xba527b59, 95 },
    { 0, 0 },
    { 0x43127b5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8827b5f, 78 },
    { 0x21627b60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c27b66, 84 },
    { 0xf7a27b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff27b6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47327b6e, 90 },
    { 0xc2c27b6f, 93 },
    { 0x6c727b6f, 95 },
    { 0, 0 },
    { 0xba227b72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8827b75, 95 },
    { 0x49127b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf927b7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a327b84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d27b87, 90 },
    { 0, 0 },
    { 0x99927b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b27b8e, 87 },
    { 0, 0 },
    { 0x33f27b90, 90 },
    { 0x83027b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9027b97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1827ba0, 91 },
    { 0, 0 },
    { 0x55127ba2, 95 },
    { 0xd0227ba3, 41 },
    { 0x88627ba3, 33 },
    { 0xe1027ba5, 61 },
    { 0xb8127ba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca027bab, 78 },
    { 0x37b27bab, 33 },
    { 0, 0 },
    { 0xa9a27bae, 85 },
    { 0, 0 },
    { 0xdcb27bb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e27bb3, 61 },
    { 0, 0 },
    { 0x92c27bb5, 91 },
    { 0xe3527bb6, 80 },
    { 0x89d27bb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x83527bba, 52 },
    { 0, 0 },
    { 0x93527bbc, 55 },
    { 0, 0 },
    { 0x10b27bbe, 33 },
    { 0xf4827bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8627bcc, 61 },
    { 0x627bcd, 99 },
    { 0x65827bce, 61 },
    { 0x28027bce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa27bd2, 75 },
    { 0x2827bd2, 19 },
    { 0, 0 },
    { 0x9cb27bd5, 23 },
    { 0x80327bd6, 98 },
    { 0xb6227bd7, 90 },
    { 0, 0 },
    { 0xe6c27bd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd427bde, 62 },
    { 0x32e27bde, 40 },
    { 0, 0 },
    { 0x6c827be1, 78 },
    { 0x11f27be2, 90 },
    { 0x9c27be3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c827bef, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6527bf2, 92 },
    { 0x33327bf3, 90 },
    { 0x17a27bf4, 81 },
    { 0xf1c27bf2, 46 },
    { 0xa3027bf6, 46 },
    { 0x7a827bf7, 46 },
    { 0, 0 },
    { 0x66b27bf9, 84 },
    { 0x5cd27bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde27bfe, 93 },
    { 0, 0 },
    { 0xadd27c00, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1627c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f327c0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e527c13, 90 },
    { 0, 0 },
    { 0x56927c15, 23 },
    { 0x20427c15, 76 },
    { 0xc4327c17, 77 },
    { 0x6b027c17, 95 },
    { 0xa0427c19, 95 },
    { 0x5a327c1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87127c1e, 97 },
    { 0, 0 },
    { 0xee327c20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b527c23, 76 },
    { 0, 0 },
    { 0xa9c27c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac927c28, 91 },
    { 0x26b27c29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34627c2d, 100 },
    { 0xb8727c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae327c31, 66 },
    { 0xc6527c32, 77 },
    { 0, 0 },
    { 0x6d527c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb27c3c, 55 },
    { 0x3b927c3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52127c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d27c45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab027c49, 76 },
    { 0x6e827c49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b27c4d, 33 },
    { 0, 0 },
    { 0xb3727c4f, 23 },
    { 0x47827c50, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96627c55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d27c5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd27c64, 90 },
    { 0xec227c64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96927c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a27c6b, 23 },
    { 0x65c27c6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d27c70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9327c73, 33 },
    { 0, 0 },
    { 0x89a27c75, 90 },
    { 0, 0 },
    { 0x2a327c77, 61 },
    { 0, 0 },
    { 0x88527c79, 5 },
    { 0x53327c7a, 77 },
    { 0xc7327c7a, 33 },
    { 0xb8d27c7c, 90 },
    { 0x6ee27c7c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x13527c80, 92 },
    { 0x7727c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae727c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd27c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d27c8d, 95 },
    { 0, 0 },
    { 0xd9927c8f, 98 },
    { 0x11527c90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c27c93, 98 },
    { 0, 0 },
    { 0x74e27c95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18527c9a, 93 },
    { 0x2b27c9a, 84 },
    { 0xe8e27c9c, 66 },
    { 0x3dc27c9a, 90 },
    { 0, 0 },
    { 0x1ec27c9f, 85 },
    { 0xf5127c9f, 99 },
    { 0x3aa27ca1, 95 },
    { 0x13827ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd27ca5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb27ca8, 100 },
    { 0, 0 },
    { 0x94027caa, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f27cad, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9327cb2, 90 },
    { 0x27cb3, 90 },
    { 0xcb627cb3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed27cb8, 76 },
    { 0x7e027cb9, 23 },
    { 0xd5f27cba, 33 },
    { 0xe7e27cba, 61 },
    { 0x54127cb9, 55 },
    { 0x39727cb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc127cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c27cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a627cc7, 76 },
    { 0x9c127cc8, 90 },
    { 0x35127cc8, 24 },
    { 0x8bb27cca, 23 },
    { 0x1d227ccb, 33 },
    { 0x66c27ccc, 55 },
    { 0x5d127ccd, 46 },
    { 0x89e27cce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c27cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59227cda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3727ce0, 95 },
    { 0x21227ce1, 33 },
    { 0x3f627ce2, 99 },
    { 0x51227ce0, 76 },
    { 0x2b727ce4, 90 },
    { 0x4f927ce4, 84 },
    { 0x89a27ce0, 80 },
    { 0x4f927ce6, 90 },
    { 0xc6327ce8, 33 },
    { 0, 0 },
    { 0xe0a27cea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac227ced, 98 },
    { 0x54527ced, 90 },
    { 0x5e727ced, 77 },
    { 0, 0 },
    { 0x23027cf1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf727cf4, 95 },
    { 0xa0f27cf5, 93 },
    { 0x4a227cf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb427cfa, 77 },
    { 0x89e27cfb, 95 },
    { 0xac927cfc, 98 },
    { 0, 0 },
    { 0xddd27cfe, 52 },
    { 0xa9927cff, 61 },
    { 0x99e27cff, 90 },
    { 0, 0 },
    { 0x64127d02, 95 },
    { 0xa5327d03, 90 },
    { 0xb3327d04, 43 },
    { 0x99627d05, 90 },
    { 0x40a27d04, 81 },
    { 0x9cf27d07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90127d0e, 61 },
    { 0, 0 },
    { 0xe2327d10, 98 },
    { 0, 0 },
    { 0xc5327d12, 61 },
    { 0x84327d13, 33 },
    { 0, 0 },
    { 0x7b027d15, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf27d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e427d1c, 98 },
    { 0xc9127d1d, 33 },
    { 0, 0 },
    { 0x54827d1f, 61 },
    { 0xdff27d1f, 33 },
    { 0x4d27d1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe527d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc27d29, 23 },
    { 0x5e827d2a, 36 },
    { 0x7fb27d2a, 61 },
    { 0xa7027d29, 58 },
    { 0x71e27d2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f27d32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16427d38, 33 },
    { 0xa5e27d38, 33 },
    { 0x9fc27d38, 95 },
    { 0xe3c27d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb327d3e, 76 },
    { 0xe3627d3e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3727d43, 68 },
    { 0xc2b27d44, 95 },
    { 0xb5827d45, 34 },
    { 0xec627d43, 98 },
    { 0x78d27d46, 90 },
    { 0, 0 },
    { 0xbff27d49, 61 },
    { 0xc6727d4a, 100 },
    { 0xd0a27d49, 65 },
    { 0x4e727d4c, 23 },
    { 0xed927d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b27d52, 90 },
    { 0, 0 },
    { 0x5fd27d54, 34 },
    { 0x19627d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff27d59, 61 },
    { 0xf1427d5a, 76 },
    { 0, 0 },
    { 0x84a27d5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f27d62, 100 },
    { 0x40c27d62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a927d66, 90 },
    { 0x72827d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe127d6e, 61 },
    { 0xe1027d6f, 90 },
    { 0x76227d70, 93 },
    { 0x7ed27d6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f27d78, 90 },
    { 0xa7d27d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d27d81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62527d85, 69 },
    { 0x8e027d85, 55 },
    { 0xf1827d86, 23 },
    { 0x37227d86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b27d8b, 61 },
    { 0xab527d8b, 93 },
    { 0x33727d8d, 92 },
    { 0x6eb27d8e, 23 },
    { 0xaaf27d8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b727d92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d027d9b, 61 },
    { 0x50127d9c, 87 },
    { 0x2bf27d9c, 33 },
    { 0x1e827d9d, 17 },
    { 0x3a427d9e, 34 },
    { 0x93127da0, 55 },
    { 0xcbf27d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50127da6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d27dac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1027db0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0327db5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7627db9, 75 },
    { 0, 0 },
    { 0xbe727dbb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21327dbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e27dc2, 34 },
    { 0, 0 },
    { 0xfbd27dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e27dce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a827dd1, 33 },
    { 0, 0 },
    { 0xf4527dd3, 90 },
    { 0, 0 },
    { 0xfc127dd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3027dd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2427ddc, 90 },
    { 0x2bc27ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30027de0, 84 },
    { 0x37427de1, 78 },
    { 0xfd527de0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83427deb, 99 },
    { 0x31227dec, 98 },
    { 0x28427ded, 52 },
    { 0x48327dec, 90 },
    { 0, 0 },
    { 0xb0e27df0, 61 },
    { 0x66c27df1, 46 },
    { 0, 0 },
    { 0xbbf27df3, 17 },
    { 0, 0 },
    { 0x3f27df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de27dfa, 91 },
    { 0, 0 },
    { 0x27127dfc, 23 },
    { 0x7a427dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5727e07, 61 },
    { 0xe7627e08, 98 },
    { 0, 0 },
    { 0x68d27e0a, 61 },
    { 0, 0 },
    { 0x47027e0c, 87 },
    { 0x64e27e0c, 33 },
    { 0xadd27e0c, 61 },
    { 0, 0 },
    { 0x50b27e10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c27e15, 78 },
    { 0, 0 },
    { 0x1d227e17, 23 },
    { 0x48e27e18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b27e1d, 95 },
    { 0xe6a27e1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x10827e21, 97 },
    { 0xb5827e21, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac27e29, 61 },
    { 0, 0 },
    { 0x6eb27e2b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b227e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a27e34, 98 },
    { 0, 0 },
    { 0x42227e36, 61 },
    { 0x70d27e37, 91 },
    { 0xa5727e37, 96 },
    { 0x75e27e38, 95 },
    { 0x29727e3a, 90 },
    { 0, 0 },
    { 0x6ed27e3c, 46 },
    { 0x6b327e3d, 66 },
    { 0x7c27e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b27e43, 88 },
    { 0, 0 },
    { 0x99527e45, 33 },
    { 0x87527e45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f927e4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7027e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67227e51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d27e54, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2727e57, 33 },
    { 0x85a27e57, 84 },
    { 0xfda27e59, 99 },
    { 0x9ce27e59, 96 },
    { 0xfed27e5b, 43 },
    { 0, 0 },
    { 0xfbb27e5d, 34 },
    { 0x24027e5e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c627e64, 98 },
    { 0x4fc27e65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98827e69, 43 },
    { 0x87c27e6a, 98 },
    { 0xa6827e6b, 33 },
    { 0x7927e6b, 98 },
    { 0xa0b27e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f27e71, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f27e77, 90 },
    { 0xb4727e78, 66 },
    { 0x4e027e77, 46 },
    { 0x20227e7a, 98 },
    { 0xf6927e7b, 90 },
    { 0xdfa27e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98227e80, 45 },
    { 0xc2e27e80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e527e8d, 33 },
    { 0x37227e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9427e91, 90 },
    { 0x85e27e92, 46 },
    { 0x8fa27e93, 19 },
    { 0, 0 },
    { 0xa1027e95, 88 },
    { 0x3d127e95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc227e99, 87 },
    { 0, 0 },
    { 0x11f27e9b, 98 },
    { 0, 0 },
    { 0x7527e9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8027ea0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f27ea4, 55 },
    { 0x6e527ea5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f27ea8, 70 },
    { 0x9d727ea9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef27eac, 90 },
    { 0xd127ead, 85 },
    { 0x7b627eac, 55 },
    { 0x8227eac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b27eb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27eb6, 99 },
    { 0, 0 },
    { 0xcd527eb8, 98 },
    { 0, 0 },
    { 0x33127eba, 23 },
    { 0xf5427ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64527ec0, 59 },
    { 0x94d27ec1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d827ec7, 99 },
    { 0x6e727ec8, 84 },
    { 0x4e227ec8, 76 },
    { 0x66f27ec8, 90 },
    { 0xb3927eca, 76 },
    { 0x2e127ecb, 98 },
    { 0, 0 },
    { 0x42927ece, 46 },
    { 0, 0 },
    { 0xfd827ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe27ed5, 61 },
    { 0, 0 },
    { 0x84227ed7, 61 },
    { 0x47727ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e27edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37427edf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d527ee6, 33 },
    { 0xd5727ee6, 100 },
    { 0, 0 },
    { 0x22627ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12927eed, 90 },
    { 0, 0 },
    { 0x4c527eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e27ef2, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e27ef7, 90 },
    { 0, 0 },
    { 0xd9c27ef9, 90 },
    { 0x67c27ef9, 33 },
    { 0, 0 },
    { 0x7a927efc, 34 },
    { 0x13627efd, 61 },
    { 0xcbb27efe, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x8927f01, 100 },
    { 0x14827f02, 55 },
    { 0x36d27f03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd927f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a27f0f, 98 },
    { 0, 0 },
    { 0xa7727f11, 65 },
    { 0xa7327f12, 95 },
    { 0xee627f11, 93 },
    { 0, 0 },
    { 0x48127f15, 85 },
    { 0x84d27f16, 33 },
    { 0, 0 },
    { 0x34927f18, 84 },
    { 0, 0 },
    { 0x10f27f1a, 55 },
    { 0, 0 },
    { 0x6d827f1c, 98 },
    { 0, 0 },
    { 0xa0927f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44427f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b27f29, 78 },
    { 0, 0 },
    { 0x3df27f2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53127f2f, 34 },
    { 0, 0 },
    { 0xdc327f31, 87 },
    { 0x7e627f31, 23 },
    { 0x8227f33, 33 },
    { 0xc4d27f34, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x88727f37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b27f3a, 76 },
    { 0x63a27f3b, 84 },
    { 0x47f27f3b, 98 },
    { 0xdd527f3d, 78 },
    { 0x54f27f3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d27f43, 98 },
    { 0x1b227f43, 94 },
    { 0x20727f43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd427f4c, 23 },
    { 0, 0 },
    { 0xc6d27f4e, 95 },
    { 0x5d827f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a27f53, 52 },
    { 0x16c27f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f27f57, 83 },
    { 0xaf227f57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a27f5d, 98 },
    { 0, 0 },
    { 0x73f27f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7927f63, 93 },
    { 0, 0 },
    { 0x9bc27f65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90727f68, 90 },
    { 0xc1027f69, 43 },
    { 0x6b727f6a, 46 },
    { 0x1bc27f6b, 65 },
    { 0xc4327f68, 88 },
    { 0xa8f27f6d, 76 },
    { 0x2d327f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d27f74, 23 },
    { 0xcad27f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a27f78, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae727f7d, 61 },
    { 0xd2327f7d, 76 },
    { 0x9d927f7f, 76 },
    { 0x8db27f7d, 34 },
    { 0x74b27f81, 76 },
    { 0x86127f81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a27f85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8727f8b, 33 },
    { 0x9b27f8b, 81 },
    { 0xe6327f8c, 90 },
    { 0x7b627f8c, 33 },
    { 0xf6d27f8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8327f95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d727f98, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e27f9b, 61 },
    { 0, 0 },
    { 0xa9c27f9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c327fa0, 33 },
    { 0, 0 },
    { 0x30c27fa2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb027fa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f27fae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d27fb4, 90 },
    { 0, 0 },
    { 0xcb27fb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a27fba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b927fbe, 100 },
    { 0, 0 },
    { 0xf2627fc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82027fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a827fca, 68 },
    { 0, 0 },
    { 0x9fb27fcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf27fd4, 55 },
    { 0, 0 },
    { 0xfa227fd6, 61 },
    { 0, 0 },
    { 0x3cf27fd8, 34 },
    { 0, 0 },
    { 0x47e27fda, 23 },
    { 0x43a27fdb, 93 },
    { 0x45627fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad27fe4, 55 },
    { 0xb7f27fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3827fe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb27feb, 98 },
    { 0x43c27fec, 98 },
    { 0, 0 },
    { 0x53d27fee, 98 },
    { 0x44c27fef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c27ff2, 61 },
    { 0, 0 },
    { 0x53827ff4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b28000, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb228006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c28009, 95 },
    { 0x7cf2800a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42800f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e928012, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde628017, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442801a, 90 },
    { 0xa2e2801b, 55 },
    { 0x2472801c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e28020, 77 },
    { 0xc3a28020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e28026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f928029, 75 },
    { 0xc9328029, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e28033, 98 },
    { 0x37128034, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e28037, 98 },
    { 0x44f28038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b2803c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b428043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c28049, 76 },
    { 0, 0 },
    { 0xf1d2804b, 52 },
    { 0xe602804c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822804f, 98 },
    { 0xd6328050, 95 },
    { 0xcdc2804f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac2805d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64228060, 93 },
    { 0x82128061, 61 },
    { 0xf5e28060, 85 },
    { 0x74828063, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2806d, 78 },
    { 0x5d2806e, 84 },
    { 0, 0 },
    { 0x59d28070, 90 },
    { 0xfb728071, 87 },
    { 0x1da28070, 61 },
    { 0x58e28073, 23 },
    { 0x62128074, 95 },
    { 0x42928075, 55 },
    { 0, 0 },
    { 0xe6f28077, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1512807c, 33 },
    { 0x39f2807c, 90 },
    { 0, 0 },
    { 0x4c22807f, 33 },
    { 0, 0 },
    { 0xdd428081, 91 },
    { 0, 0 },
    { 0xeb28083, 33 },
    { 0x62d28084, 63 },
    { 0xdf128085, 91 },
    { 0xd3728085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af28089, 76 },
    { 0, 0 },
    { 0x66e2808b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42928096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5342809e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69280a4, 98 },
    { 0x36e280a5, 90 },
    { 0xbb6280a5, 61 },
    { 0x249280a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce280ad, 78 },
    { 0x487280ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d280b1, 55 },
    { 0xd08280b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x482280b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29280b9, 78 },
    { 0, 0 },
    { 0xaa9280bb, 95 },
    { 0x7d1280bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187280c1, 55 },
    { 0, 0 },
    { 0x6e4280c3, 90 },
    { 0xde280c3, 61 },
    { 0, 0 },
    { 0x3ac280c6, 95 },
    { 0, 0 },
    { 0xb0280c8, 33 },
    { 0xef0280c8, 90 },
    { 0x5bb280c9, 98 },
    { 0x8e3280cb, 99 },
    { 0x245280cc, 84 },
    { 0xbe8280cc, 95 },
    { 0, 0 },
    { 0xf7e280cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08280d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655280d9, 98 },
    { 0x146280da, 95 },
    { 0, 0 },
    { 0x81b280dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5280e0, 95 },
    { 0, 0 },
    { 0xc96280e2, 90 },
    { 0x6d6280e2, 100 },
    { 0x8b9280e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1280ee, 98 },
    { 0x3fa280ef, 90 },
    { 0x1c9280ee, 23 },
    { 0, 0 },
    { 0xa92280f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf280f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11280fc, 93 },
    { 0, 0 },
    { 0x526280fe, 33 },
    { 0xbc7280ff, 76 },
    { 0xecc28100, 61 },
    { 0, 0 },
    { 0x19228102, 23 },
    { 0x15b28103, 83 },
    { 0xb8128103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec928109, 24 },
    { 0x7222810a, 23 },
    { 0x342810b, 91 },
    { 0x8e228109, 55 },
    { 0xd712810a, 90 },
    { 0x3202810d, 74 },
    { 0x24e2810f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb28115, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf128118, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac82811d, 93 },
    { 0, 0 },
    { 0x7f02811f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c328122, 85 },
    { 0, 0 },
    { 0xb3c28124, 33 },
    { 0, 0 },
    { 0x36e28126, 90 },
    { 0xf1228127, 61 },
    { 0x9a428126, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8792812c, 61 },
    { 0x4e22812c, 85 },
    { 0xb512812e, 88 },
    { 0xeb2812d, 33 },
    { 0x4542812c, 52 },
    { 0xd7728130, 61 },
    { 0x43228132, 23 },
    { 0, 0 },
    { 0x98c28134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46528137, 81 },
    { 0x13628138, 43 },
    { 0xf0928139, 66 },
    { 0x9b92813a, 46 },
    { 0, 0 },
    { 0xd422813c, 23 },
    { 0x8f12813d, 61 },
    { 0, 0 },
    { 0x15a2813f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b928144, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c32814b, 91 },
    { 0, 0 },
    { 0xa662814d, 33 },
    { 0x4ee2814d, 55 },
    { 0xf602814d, 43 },
    { 0x7b12814f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b28154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e28159, 77 },
    { 0xd32815a, 43 },
    { 0xc502815b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9628161, 43 },
    { 0x2ec28162, 61 },
    { 0x2ee28163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19428167, 90 },
    { 0xbba28168, 17 },
    { 0x92d28169, 33 },
    { 0x65d2816a, 55 },
    { 0x53528168, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56628173, 76 },
    { 0x2dc28173, 76 },
    { 0x23b28173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112817a, 95 },
    { 0xd8f2817b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e12817e, 17 },
    { 0, 0 },
    { 0xd4128180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7228183, 33 },
    { 0x12e28184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f28189, 53 },
    { 0, 0 },
    { 0x7a52818b, 93 },
    { 0, 0 },
    { 0x482818d, 98 },
    { 0x7902818e, 61 },
    { 0, 0 },
    { 0x17928190, 61 },
    { 0, 0 },
    { 0xf9528192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd2819a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf202819e, 95 },
    { 0x1972819f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd281a4, 33 },
    { 0xcd1281a5, 23 },
    { 0x41281a5, 98 },
    { 0x7d2281a7, 66 },
    { 0, 0 },
    { 0x8ee281a9, 93 },
    { 0x841281aa, 23 },
    { 0xc4c281a9, 33 },
    { 0x785281ab, 33 },
    { 0x541281a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628281b3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce281bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201281c0, 90 },
    { 0x239281c1, 93 },
    { 0xd4c281c2, 40 },
    { 0, 0 },
    { 0x385281c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc281ce, 91 },
    { 0x7e9281cf, 66 },
    { 0x3f2281d0, 90 },
    { 0, 0 },
    { 0x1be281d2, 99 },
    { 0x3db281d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed281d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba281dc, 43 },
    { 0xd0e281dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04281e8, 77 },
    { 0, 0 },
    { 0x178281ea, 55 },
    { 0xfc6281eb, 98 },
    { 0, 0 },
    { 0xeae281ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d281f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a281f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a281f9, 90 },
    { 0, 0 },
    { 0xbdc281fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d328202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef2820b, 17 },
    { 0x5202820b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8728211, 52 },
    { 0, 0 },
    { 0x37e28213, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b28218, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2072821c, 95 },
    { 0, 0 },
    { 0x962821e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc28222, 84 },
    { 0, 0 },
    { 0x84a28224, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2822c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e28230, 95 },
    { 0x4ca28231, 33 },
    { 0x48528230, 81 },
    { 0xc2328233, 98 },
    { 0x8f228233, 52 },
    { 0x9d728233, 58 },
    { 0xb2128236, 90 },
    { 0xfa928237, 90 },
    { 0x1c328232, 38 },
    { 0x7d728239, 30 },
    { 0, 0 },
    { 0x5f52823b, 90 },
    { 0x9ae2823c, 90 },
    { 0x6462823b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f28240, 17 },
    { 0, 0 },
    { 0x95b28242, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17528249, 90 },
    { 0x8e32824a, 34 },
    { 0x8b72824b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad2824f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c28252, 85 },
    { 0, 0 },
    { 0x94928254, 98 },
    { 0x48228255, 65 },
    { 0x85228256, 61 },
    { 0, 0 },
    { 0x4f628258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922825b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0328262, 75 },
    { 0x8e228263, 87 },
    { 0, 0 },
    { 0xafb28265, 90 },
    { 0x33028266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f2826b, 95 },
    { 0, 0 },
    { 0xc22826d, 90 },
    { 0xb6d2826e, 34 },
    { 0x62826f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc728273, 84 },
    { 0, 0 },
    { 0x58228275, 90 },
    { 0x36028276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca28279, 52 },
    { 0xbe42827a, 33 },
    { 0xa552827b, 76 },
    { 0, 0 },
    { 0x9792827d, 99 },
    { 0, 0 },
    { 0xa9e2827f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8128282, 17 },
    { 0, 0 },
    { 0x4d228284, 101 },
    { 0x5c28285, 61 },
    { 0, 0 },
    { 0xb9528287, 98 },
    { 0xff828287, 43 },
    { 0, 0 },
    { 0xd262828a, 33 },
    { 0xd882828b, 33 },
    { 0xbd22828b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd852828f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce628293, 90 },
    { 0x51728294, 34 },
    { 0xb0a28295, 90 },
    { 0, 0 },
    { 0x1c628297, 61 },
    { 0x36b28298, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32829f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb282a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132282a8, 66 },
    { 0x63d282a8, 55 },
    { 0xf60282a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d282ad, 95 },
    { 0xd06282ae, 98 },
    { 0xff1282af, 38 },
    { 0x187282b0, 67 },
    { 0, 0 },
    { 0xa7b282b2, 65 },
    { 0x20d282b2, 76 },
    { 0, 0 },
    { 0x739282b5, 36 },
    { 0xb7d282b5, 36 },
    { 0, 0 },
    { 0x3d9282b8, 90 },
    { 0x155282b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a282bd, 52 },
    { 0x8df282be, 60 },
    { 0, 0 },
    { 0x59e282c0, 90 },
    { 0xb47282c1, 61 },
    { 0x448282c1, 93 },
    { 0, 0 },
    { 0x98b282c4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe282c8, 100 },
    { 0xf2d282c8, 99 },
    { 0xea282c8, 17 },
    { 0, 0 },
    { 0x227282cc, 87 },
    { 0x191282cd, 93 },
    { 0xda7282cd, 95 },
    { 0, 0 },
    { 0xdf1282d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a282d5, 77 },
    { 0xace282d6, 90 },
    { 0x166282d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27282dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940282e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9282e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5282ea, 93 },
    { 0xb94282eb, 68 },
    { 0xbd8282eb, 61 },
    { 0xf67282ed, 83 },
    { 0, 0 },
    { 0x5a9282ef, 33 },
    { 0xfe6282f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc282f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63282f8, 61 },
    { 0, 0 },
    { 0x87c282fa, 23 },
    { 0x172282fb, 90 },
    { 0x91f282fc, 84 },
    { 0x539282fd, 33 },
    { 0, 0 },
    { 0x804282ff, 98 },
    { 0xa2728300, 98 },
    { 0x3de28301, 91 },
    { 0xde728302, 61 },
    { 0xfc7282ff, 95 },
    { 0x50b28304, 90 },
    { 0x7c228300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2830a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a32830d, 98 },
    { 0x7572830e, 90 },
    { 0x132830e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d928312, 100 },
    { 0x6dc28312, 98 },
    { 0xef028314, 98 },
    { 0, 0 },
    { 0xf0728316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be28319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582831f, 66 },
    { 0x51e2831f, 100 },
    { 0, 0 },
    { 0x27b28322, 23 },
    { 0x61228323, 93 },
    { 0, 0 },
    { 0x19328325, 61 },
    { 0x45b28326, 75 },
    { 0xd8c28327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd572832a, 23 },
    { 0x6342832a, 90 },
    { 0, 0 },
    { 0xd4c2832d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8928330, 90 },
    { 0xa4e28330, 81 },
    { 0, 0 },
    { 0x5628333, 56 },
    { 0x77128334, 95 },
    { 0x4df28335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9228338, 34 },
    { 0x8da28339, 55 },
    { 0xee12833a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f2833f, 95 },
    { 0xa022833f, 90 },
    { 0x22d2833f, 61 },
    { 0xe7b28342, 101 },
    { 0, 0 },
    { 0x75928344, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86828348, 33 },
    { 0xa2c28349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae2834c, 33 },
    { 0x8d92834d, 97 },
    { 0, 0 },
    { 0x6932834f, 98 },
    { 0, 0 },
    { 0x9ed28351, 76 },
    { 0x61628352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b92835c, 65 },
    { 0xa082835c, 43 },
    { 0x3202835e, 35 },
    { 0xe432835d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9228362, 92 },
    { 0xba728363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df28369, 55 },
    { 0xa8f28369, 84 },
    { 0x9922836b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3902836e, 61 },
    { 0, 0 },
    { 0xeff28370, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf28376, 76 },
    { 0x4828376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f2837b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628384, 33 },
    { 0, 0 },
    { 0xb2a28386, 95 },
    { 0x11a28387, 81 },
    { 0xbc628386, 38 },
    { 0x42c28389, 33 },
    { 0x6a728389, 88 },
    { 0x97e2838a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd2838f, 33 },
    { 0x8128390, 90 },
    { 0x6f92838f, 98 },
    { 0, 0 },
    { 0xb0528393, 90 },
    { 0x73928394, 33 },
    { 0x5b728395, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c28399, 91 },
    { 0x44e28399, 55 },
    { 0xd9b28399, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f283a0, 55 },
    { 0x4b283a1, 66 },
    { 0, 0 },
    { 0xbbf283a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91283a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f283b0, 56 },
    { 0x94d283b0, 61 },
    { 0, 0 },
    { 0x18a283b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f283b7, 61 },
    { 0, 0 },
    { 0xb9d283b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d283c0, 55 },
    { 0xaa4283c0, 52 },
    { 0, 0 },
    { 0xbdc283c3, 40 },
    { 0, 0 },
    { 0xec2283c5, 90 },
    { 0, 0 },
    { 0xcb2283c7, 98 },
    { 0x492283c8, 84 },
    { 0xcb1283c9, 46 },
    { 0x61f283ca, 77 },
    { 0xeb1283c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d283cf, 23 },
    { 0xc24283d0, 90 },
    { 0x11e283cf, 90 },
    { 0x51d283d2, 23 },
    { 0x9e3283d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4283df, 34 },
    { 0x8bc283df, 46 },
    { 0, 0 },
    { 0xc2283e2, 90 },
    { 0xecc283e2, 93 },
    { 0x376283e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6283ee, 90 },
    { 0, 0 },
    { 0x13e283f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf283f4, 68 },
    { 0, 0 },
    { 0xc8a283f6, 84 },
    { 0xe17283f7, 88 },
    { 0xd48283f8, 33 },
    { 0x702283f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f283fe, 66 },
    { 0x8d1283fe, 93 },
    { 0, 0 },
    { 0x33c28401, 95 },
    { 0x17528402, 90 },
    { 0xb7628402, 61 },
    { 0x7d28404, 99 },
    { 0xaaa28401, 23 },
    { 0x8d228406, 33 },
    { 0x12f28407, 76 },
    { 0xdf628408, 68 },
    { 0, 0 },
    { 0x3b2840a, 84 },
    { 0, 0 },
    { 0xa192840c, 23 },
    { 0x8a52840c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc28410, 61 },
    { 0xf6228411, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde328417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf142841c, 52 },
    { 0x9772841d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d28420, 34 },
    { 0xb4828421, 23 },
    { 0, 0 },
    { 0xe8728423, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86328426, 97 },
    { 0x14928427, 95 },
    { 0xe6828426, 98 },
    { 0x2ad28428, 75 },
    { 0xcb12842a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62842e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x70428431, 90 },
    { 0xa9928432, 78 },
    { 0x46d28431, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf128437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc682843a, 59 },
    { 0xa762843b, 95 },
    { 0xf4d2843b, 80 },
    { 0xb7e2843c, 46 },
    { 0xce82843d, 87 },
    { 0xd492843b, 95 },
    { 0, 0 },
    { 0xa9528441, 84 },
    { 0xca428442, 23 },
    { 0x1a028441, 91 },
    { 0xdf928444, 76 },
    { 0x7c928444, 95 },
    { 0x328445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d28449, 61 },
    { 0xc852844a, 66 },
    { 0, 0 },
    { 0xe0d2844c, 78 },
    { 0xd22844c, 90 },
    { 0x2b62844c, 17 },
    { 0x48d2844f, 36 },
    { 0x3ce28450, 90 },
    { 0x48328450, 90 },
    { 0x43a28451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272845a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed72845e, 100 },
    { 0x6582845f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2328464, 33 },
    { 0xbca28464, 95 },
    { 0x7428466, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9882846d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0328471, 23 },
    { 0xf7328471, 61 },
    { 0x73b28473, 23 },
    { 0x6f628473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2528478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2847b, 61 },
    { 0xa502847c, 33 },
    { 0xdd62847d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6328481, 98 },
    { 0x8e28482, 96 },
    { 0, 0 },
    { 0x9f528484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77128488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2848b, 80 },
    { 0xb772848b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59928490, 76 },
    { 0xbde28490, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2128498, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42849e, 95 },
    { 0x1882849e, 55 },
    { 0x77f284a0, 33 },
    { 0xad1284a1, 61 },
    { 0xe70284a2, 23 },
    { 0, 0 },
    { 0xf92284a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d284a7, 90 },
    { 0x751284a7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808284ac, 78 },
    { 0x717284ad, 61 },
    { 0xa3a284ae, 55 },
    { 0x4f5284ae, 23 },
    { 0, 0 },
    { 0xe4a284b1, 66 },
    { 0, 0 },
    { 0xaa284b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98284b7, 55 },
    { 0x2ff284b8, 33 },
    { 0x6e3284b9, 98 },
    { 0, 0 },
    { 0x17f284bb, 90 },
    { 0xc1c284bc, 98 },
    { 0, 0 },
    { 0xdc6284be, 90 },
    { 0x55284bf, 81 },
    { 0xf28284c0, 94 },
    { 0x752284c1, 95 },
    { 0x93284c2, 84 },
    { 0x4cf284c3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4284c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f284cc, 76 },
    { 0, 0 },
    { 0x5bf284ce, 33 },
    { 0xabe284ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea284d7, 100 },
    { 0xa86284d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e284db, 43 },
    { 0x5fc284db, 76 },
    { 0x9b2284dd, 43 },
    { 0xa68284de, 61 },
    { 0xb0b284de, 90 },
    { 0xa0b284de, 45 },
    { 0x27d284e0, 98 },
    { 0x5ef284df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4284e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e284ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x547284f1, 78 },
    { 0x8c1284f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07284f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980284ff, 90 },
    { 0x94228500, 33 },
    { 0x37f28501, 87 },
    { 0, 0 },
    { 0x3cb28503, 85 },
    { 0, 0 },
    { 0x68528505, 99 },
    { 0xb5528506, 90 },
    { 0x8e428507, 33 },
    { 0x2fd28507, 91 },
    { 0x52f28507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a52850c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe28510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c28518, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832851e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f28522, 90 },
    { 0x74428523, 90 },
    { 0, 0 },
    { 0xa828525, 69 },
    { 0x19b28525, 90 },
    { 0x86028525, 66 },
    { 0x29428528, 93 },
    { 0, 0 },
    { 0x58d2852a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3028531, 98 },
    { 0x5b228532, 90 },
    { 0x86e28532, 46 },
    { 0x5c428534, 85 },
    { 0xca228534, 90 },
    { 0x50228532, 61 },
    { 0x12f28537, 34 },
    { 0xf7c28536, 43 },
    { 0xef028539, 53 },
    { 0x48428539, 90 },
    { 0xb4a2853a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552853e, 27 },
    { 0x4b02853e, 33 },
    { 0, 0 },
    { 0xe7d28541, 87 },
    { 0x13528542, 90 },
    { 0, 0 },
    { 0xa3e28544, 34 },
    { 0xc2828544, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d628548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1942854b, 98 },
    { 0x4a32854b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7202854f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf128552, 61 },
    { 0x10028553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a028557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b28561, 87 },
    { 0xd0f28562, 98 },
    { 0, 0 },
    { 0x61828564, 98 },
    { 0, 0 },
    { 0x43628566, 76 },
    { 0, 0 },
    { 0x33928568, 77 },
    { 0xec28569, 23 },
    { 0xe462856a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2272856f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38928573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc428578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee2857b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f2857e, 95 },
    { 0, 0 },
    { 0x99d28580, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37428585, 33 },
    { 0xd4228586, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb528589, 78 },
    { 0x8272858a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472858d, 95 },
    { 0, 0 },
    { 0x6b2858f, 90 },
    { 0x10f2858f, 66 },
    { 0x1ab28591, 99 },
    { 0xecd28591, 95 },
    { 0, 0 },
    { 0x24228594, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8232859a, 90 },
    { 0x5492859b, 76 },
    { 0x3962859a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66285a0, 36 },
    { 0, 0 },
    { 0x411285a2, 91 },
    { 0xfa8285a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b285aa, 76 },
    { 0x9c6285ab, 95 },
    { 0x32a285ac, 61 },
    { 0x944285ac, 91 },
    { 0x847285ae, 98 },
    { 0x5ff285ac, 84 },
    { 0x80d285ad, 90 },
    { 0x33d285ad, 99 },
    { 0xfe4285b1, 95 },
    { 0xb49285b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3285b6, 91 },
    { 0, 0 },
    { 0xb93285b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb285bb, 95 },
    { 0xaab285bc, 95 },
    { 0, 0 },
    { 0x624285be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5285c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b285c8, 100 },
    { 0x3bf285c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23285cf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd285d3, 61 },
    { 0xa4d285d3, 49 },
    { 0, 0 },
    { 0x72285d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b285d9, 23 },
    { 0, 0 },
    { 0xcd8285db, 76 },
    { 0x91f285dc, 69 },
    { 0x204285dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca285e1, 87 },
    { 0x94f285e2, 61 },
    { 0x907285e2, 23 },
    { 0x285285e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0285e8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e285eb, 93 },
    { 0xd0285eb, 90 },
    { 0x2f6285ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab285f0, 55 },
    { 0x975285f0, 58 },
    { 0, 0 },
    { 0x870285f3, 33 },
    { 0x4b8285f4, 23 },
    { 0, 0 },
    { 0x33a285f6, 17 },
    { 0, 0 },
    { 0x2c285f8, 46 },
    { 0x9d6285f9, 90 },
    { 0x26e285fa, 61 },
    { 0, 0 },
    { 0xc72285fc, 81 },
    { 0x3c1285fd, 90 },
    { 0x55a285fe, 23 },
    { 0xfd9285ff, 98 },
    { 0x22828600, 61 },
    { 0xc2c28601, 95 },
    { 0x38f28601, 61 },
    { 0, 0 },
    { 0xffe28604, 32 },
    { 0, 0 },
    { 0x43d28606, 95 },
    { 0, 0 },
    { 0x87328608, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe2860b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72860e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c28611, 61 },
    { 0x85328612, 33 },
    { 0x34128613, 33 },
    { 0x9728614, 23 },
    { 0xe5f28615, 33 },
    { 0x86a28611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8312861e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca528621, 52 },
    { 0xf328622, 91 },
    { 0x9c628621, 84 },
    { 0x61028624, 46 },
    { 0, 0 },
    { 0xe4a28626, 95 },
    { 0, 0 },
    { 0x3c328628, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2862c, 90 },
    { 0xbd32862d, 90 },
    { 0x4d42862d, 61 },
    { 0, 0 },
    { 0x76c28630, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c28635, 86 },
    { 0xda28635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f28639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2863d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa128640, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f328643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb28646, 93 },
    { 0, 0 },
    { 0x6b128648, 91 },
    { 0xfd528649, 95 },
    { 0, 0 },
    { 0xa912864b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb392864f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2528652, 98 },
    { 0x5b28653, 52 },
    { 0x8cc28653, 98 },
    { 0xc5a28655, 55 },
    { 0xafd28655, 33 },
    { 0x70d28654, 93 },
    { 0, 0 },
    { 0x1ef28659, 95 },
    { 0, 0 },
    { 0xc6b2865b, 35 },
    { 0x7ca2865c, 84 },
    { 0, 0 },
    { 0x6e62865e, 36 },
    { 0, 0 },
    { 0x3c628660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e628665, 40 },
    { 0xdaa28666, 61 },
    { 0x84028667, 23 },
    { 0xa7128667, 23 },
    { 0x2928666, 90 },
    { 0, 0 },
    { 0x7412866b, 90 },
    { 0x55f2866b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7402866f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028674, 98 },
    { 0xa828675, 55 },
    { 0x91128676, 95 },
    { 0xc1328677, 23 },
    { 0xd4728677, 55 },
    { 0x16228679, 90 },
    { 0x3542867a, 95 },
    { 0x3762867b, 67 },
    { 0x752867c, 90 },
    { 0xc5428679, 33 },
    { 0x8c42867e, 90 },
    { 0x3852867b, 95 },
    { 0x89428680, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56528683, 23 },
    { 0xef728683, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f2868d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a828695, 23 },
    { 0x21328695, 33 },
    { 0x8c028697, 40 },
    { 0x1c728696, 61 },
    { 0xe3f28699, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1762869e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c286a1, 61 },
    { 0x3d3286a1, 34 },
    { 0, 0 },
    { 0xba2286a4, 90 },
    { 0, 0 },
    { 0x51a286a6, 55 },
    { 0x28d286a6, 95 },
    { 0, 0 },
    { 0x3af286a9, 33 },
    { 0x876286aa, 55 },
    { 0xce9286aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9286af, 93 },
    { 0, 0 },
    { 0x134286b1, 23 },
    { 0x4a3286b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d286b5, 46 },
    { 0xb63286b6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808286ba, 33 },
    { 0x48e286bb, 98 },
    { 0xa72286ba, 90 },
    { 0xb1d286bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776286c1, 55 },
    { 0, 0 },
    { 0x70a286c3, 33 },
    { 0x115286c3, 91 },
    { 0x846286c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b286cb, 95 },
    { 0x6ee286cc, 76 },
    { 0, 0 },
    { 0x39e286ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a286d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c286d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6286db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130286df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd286e2, 61 },
    { 0, 0 },
    { 0x39f286e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d286e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc286ea, 98 },
    { 0xd1286eb, 90 },
    { 0x9fd286ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a286f0, 61 },
    { 0xabc286f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e286f4, 33 },
    { 0x9fb286f5, 55 },
    { 0, 0 },
    { 0x350286f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1228701, 90 },
    { 0x94e28702, 91 },
    { 0x80628702, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb028706, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8972870f, 23 },
    { 0x5d828710, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45728714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc2871d, 76 },
    { 0, 0 },
    { 0x75d2871f, 33 },
    { 0x8cb2871f, 78 },
    { 0x8b228721, 90 },
    { 0xfe028721, 92 },
    { 0xe4b28723, 61 },
    { 0x17328723, 90 },
    { 0x79e28724, 78 },
    { 0x43c28726, 91 },
    { 0x1b428726, 90 },
    { 0x57828724, 90 },
    { 0x5f28729, 95 },
    { 0x1f128722, 55 },
    { 0, 0 },
    { 0xb822872c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e28736, 84 },
    { 0x7e28737, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee92873e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e28745, 52 },
    { 0x40528746, 95 },
    { 0, 0 },
    { 0xedb28748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2874c, 75 },
    { 0x2de2874d, 98 },
    { 0, 0 },
    { 0x7332874f, 34 },
    { 0x31928750, 33 },
    { 0xb8128751, 43 },
    { 0xa2e2874f, 95 },
    { 0, 0 },
    { 0x3ac28754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d528759, 61 },
    { 0x3102875a, 98 },
    { 0x1d32875b, 95 },
    { 0xc322875c, 95 },
    { 0xfad2875d, 78 },
    { 0x6152875d, 76 },
    { 0, 0 },
    { 0x93428760, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d2876a, 46 },
    { 0xd232876a, 90 },
    { 0x6762876c, 33 },
    { 0x71e2876c, 95 },
    { 0x3b32876c, 98 },
    { 0, 0 },
    { 0x5fa28770, 90 },
    { 0xb4928770, 23 },
    { 0x8828771, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32877b, 17 },
    { 0x17f2877c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29328781, 61 },
    { 0x9e528781, 33 },
    { 0, 0 },
    { 0x3d728784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a28788, 83 },
    { 0x5ee28788, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x752878c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b628792, 80 },
    { 0x10f28793, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8128797, 98 },
    { 0xf9028798, 95 },
    { 0, 0 },
    { 0xe872879a, 23 },
    { 0x9372879b, 90 },
    { 0xfbc2879c, 98 },
    { 0xcd02879d, 52 },
    { 0, 0 },
    { 0x7a92879f, 61 },
    { 0x23a287a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547287a6, 33 },
    { 0xa42287a7, 98 },
    { 0x332287a7, 66 },
    { 0, 0 },
    { 0x4a4287aa, 98 },
    { 0, 0 },
    { 0x3ff287ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf287af, 95 },
    { 0x11d287b0, 46 },
    { 0, 0 },
    { 0x9df287b2, 65 },
    { 0, 0 },
    { 0x671287b4, 90 },
    { 0x82a287b5, 33 },
    { 0x215287b4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x590287b9, 55 },
    { 0, 0 },
    { 0x3af287bb, 78 },
    { 0xab0287bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887287c0, 91 },
    { 0x985287c1, 43 },
    { 0xedf287c1, 76 },
    { 0x8c6287c2, 61 },
    { 0xaa3287c4, 61 },
    { 0x31287c3, 90 },
    { 0x31a287c5, 76 },
    { 0x387287c7, 90 },
    { 0xe8b287c8, 66 },
    { 0xc07287c9, 55 },
    { 0xa0a287ca, 98 },
    { 0x424287c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59287cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b287d5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e287dc, 23 },
    { 0xae5287dd, 92 },
    { 0x3287dc, 33 },
    { 0, 0 },
    { 0xd1287e0, 90 },
    { 0, 0 },
    { 0x1a4287e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0287e6, 95 },
    { 0x916287e6, 99 },
    { 0x85f287e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3287ed, 90 },
    { 0x371287ed, 61 },
    { 0, 0 },
    { 0x46b287f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac287f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389287f8, 88 },
    { 0x447287f9, 77 },
    { 0, 0 },
    { 0x8b8287fb, 17 },
    { 0, 0 },
    { 0xcbb287fd, 55 },
    { 0xcf2287fe, 93 },
    { 0, 0 },
    { 0x6d728800, 99 },
    { 0, 0 },
    { 0x95a28802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2880a, 90 },
    { 0xe702880b, 98 },
    { 0x6e22880c, 98 },
    { 0x7772880b, 95 },
    { 0x5352880d, 98 },
    { 0x77b2880f, 33 },
    { 0, 0 },
    { 0x74228811, 84 },
    { 0xfe128812, 81 },
    { 0x1ed28813, 90 },
    { 0x59e28814, 51 },
    { 0xb4328811, 98 },
    { 0x30328816, 33 },
    { 0, 0 },
    { 0x8f128818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5202881d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f28820, 100 },
    { 0x5c928820, 17 },
    { 0xbb28822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba28826, 55 },
    { 0xb928827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1482882c, 75 },
    { 0, 0 },
    { 0x62b2882e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e28834, 90 },
    { 0, 0 },
    { 0xd4728836, 93 },
    { 0xe3628836, 98 },
    { 0xe4428838, 95 },
    { 0x5e928836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2883e, 90 },
    { 0xe282883f, 43 },
    { 0xc7428840, 52 },
    { 0x4ad28840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b28845, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802884b, 55 },
    { 0, 0 },
    { 0x65a2884d, 84 },
    { 0x4b42884e, 23 },
    { 0, 0 },
    { 0xe6e28850, 100 },
    { 0x35128851, 95 },
    { 0xbd028852, 66 },
    { 0xd0528853, 87 },
    { 0xf2228854, 88 },
    { 0xc6528855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e2885a, 91 },
    { 0x6ec2885b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f02885e, 43 },
    { 0xc842885e, 43 },
    { 0xc292885e, 98 },
    { 0, 0 },
    { 0xa2d28862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab28866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d2886a, 46 },
    { 0x2ac2886b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6842886e, 67 },
    { 0x2ad2886f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8228872, 90 },
    { 0x44f28872, 33 },
    { 0, 0 },
    { 0xba028875, 98 },
    { 0x84528876, 84 },
    { 0x7c528877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf62887f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c828885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d28888, 90 },
    { 0x88e28889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e2888e, 66 },
    { 0, 0 },
    { 0xdfa28890, 90 },
    { 0x99328891, 100 },
    { 0, 0 },
    { 0x44428893, 81 },
    { 0x4a528894, 98 },
    { 0xc2228895, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8328898, 61 },
    { 0xd9c28899, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2889d, 98 },
    { 0x5432889d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9288a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd288a4, 98 },
    { 0, 0 },
    { 0x233288a6, 76 },
    { 0xc72288a7, 61 },
    { 0, 0 },
    { 0x41e288a9, 76 },
    { 0xfab288a9, 95 },
    { 0, 0 },
    { 0x4ea288ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724288b1, 90 },
    { 0x216288b2, 55 },
    { 0, 0 },
    { 0x821288b4, 40 },
    { 0x362288b4, 46 },
    { 0xedf288b5, 43 },
    { 0, 0 },
    { 0xf9d288b8, 23 },
    { 0xbb8288b8, 66 },
    { 0, 0 },
    { 0x286288bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c288bf, 90 },
    { 0x5e9288bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b288c4, 88 },
    { 0x568288c4, 55 },
    { 0x10c288c6, 43 },
    { 0x281288c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25288cc, 61 },
    { 0x335288cd, 17 },
    { 0xeb1288ce, 61 },
    { 0xa8a288cc, 34 },
    { 0x32e288d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681288d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531288da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5288dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9288e4, 90 },
    { 0x889288e5, 90 },
    { 0x586288e6, 23 },
    { 0xbe288e7, 52 },
    { 0xd9a288e5, 100 },
    { 0xc5b288e9, 76 },
    { 0xa72288ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a288ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679288f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x698288f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520288fe, 99 },
    { 0xc4d288fe, 52 },
    { 0x90228900, 23 },
    { 0x455288ff, 33 },
    { 0x7028902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a128906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7022890a, 95 },
    { 0xe9a2890a, 84 },
    { 0x59e2890c, 93 },
    { 0xa8d2890b, 88 },
    { 0xd0e2890c, 91 },
    { 0xab82890e, 23 },
    { 0x8aa2890e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c328918, 46 },
    { 0x99d28919, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2891c, 23 },
    { 0x44e2891c, 98 },
    { 0x58d2891e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e28922, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0428925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50928931, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b528937, 23 },
    { 0x3df28938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced28944, 23 },
    { 0xdeb28945, 20 },
    { 0, 0 },
    { 0xb4028947, 33 },
    { 0, 0 },
    { 0x52b28949, 65 },
    { 0x8de2894a, 87 },
    { 0x96b2894b, 61 },
    { 0x3602894a, 90 },
    { 0x992894d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28950, 46 },
    { 0xc3728951, 61 },
    { 0xa5128952, 88 },
    { 0, 0 },
    { 0xca828954, 55 },
    { 0x56828954, 23 },
    { 0x40428956, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2895e, 33 },
    { 0x1d82895f, 98 },
    { 0xedf28960, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d128965, 5 },
    { 0x5eb28966, 76 },
    { 0xe7128965, 98 },
    { 0xf1228968, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce028970, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d28974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1128978, 61 },
    { 0x5dd28978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2897c, 33 },
    { 0xfbb2897d, 17 },
    { 0xc962897e, 23 },
    { 0xae82897f, 23 },
    { 0, 0 },
    { 0x22328981, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5828985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec72898a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7028998, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02899f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x943289a2, 90 },
    { 0x8ac289a3, 93 },
    { 0x138289a4, 76 },
    { 0x1f4289a3, 61 },
    { 0, 0 },
    { 0x335289a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c289b0, 95 },
    { 0xc5e289b0, 33 },
    { 0x72a289b2, 23 },
    { 0x4c5289b3, 89 },
    { 0x361289b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6289b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31289bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404289c4, 43 },
    { 0xf16289c4, 33 },
    { 0, 0 },
    { 0xc8f289c7, 34 },
    { 0x85e289c8, 17 },
    { 0x380289c9, 55 },
    { 0x697289c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53289cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af289d2, 46 },
    { 0, 0 },
    { 0x3f1289d4, 33 },
    { 0, 0 },
    { 0x230289d6, 90 },
    { 0xac0289d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8289db, 68 },
    { 0xda2289db, 61 },
    { 0x1e7289dd, 61 },
    { 0x995289dd, 17 },
    { 0x3df289dd, 43 },
    { 0x7f4289e0, 81 },
    { 0xe9289e0, 23 },
    { 0xa61289e2, 78 },
    { 0xefe289e0, 52 },
    { 0x373289e4, 33 },
    { 0x6fd289e4, 66 },
    { 0, 0 },
    { 0x6d8289e7, 95 },
    { 0xbd8289e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x563289eb, 43 },
    { 0x93d289eb, 95 },
    { 0x72f289ed, 98 },
    { 0x212289ee, 23 },
    { 0xfe3289eb, 76 },
    { 0xb07289f0, 33 },
    { 0x2dd289f0, 23 },
    { 0, 0 },
    { 0x1c7289f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775289f7, 76 },
    { 0xfc289f8, 33 },
    { 0x5c4289f9, 76 },
    { 0x7b9289f9, 61 },
    { 0, 0 },
    { 0x8bc289fc, 87 },
    { 0xf36289fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f28a01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54028a06, 93 },
    { 0, 0 },
    { 0x8528a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf28a1a, 23 },
    { 0, 0 },
    { 0xca528a1c, 95 },
    { 0x78228a1d, 98 },
    { 0, 0 },
    { 0x6d528a1f, 23 },
    { 0x64528a20, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d628a23, 76 },
    { 0xd6d28a24, 91 },
    { 0xcfe28a24, 90 },
    { 0x72928a26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63428a2a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6828a2d, 38 },
    { 0x6d428a2e, 33 },
    { 0x49228a2f, 34 },
    { 0x9d928a2f, 76 },
    { 0xb628a31, 100 },
    { 0xfeb28a32, 46 },
    { 0xb8028a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba28a3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e28a3f, 100 },
    { 0x41928a3f, 98 },
    { 0x9c228a40, 33 },
    { 0xe7228a3f, 46 },
    { 0xc8728a3f, 33 },
    { 0, 0 },
    { 0xb5f28a45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf28a51, 87 },
    { 0xf3f28a52, 23 },
    { 0xe7528a53, 23 },
    { 0x15828a54, 65 },
    { 0, 0 },
    { 0xff28a56, 78 },
    { 0x48428a57, 78 },
    { 0xe728a58, 55 },
    { 0x19028a56, 90 },
    { 0, 0 },
    { 0xf6c28a5b, 87 },
    { 0x91828a5c, 61 },
    { 0xb5a28a5d, 98 },
    { 0, 0 },
    { 0xce928a5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d728a62, 46 },
    { 0x88428a62, 98 },
    { 0x64928a64, 91 },
    { 0x6a428a65, 84 },
    { 0x48128a65, 23 },
    { 0x5b628a62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62728a6b, 90 },
    { 0x51c28a6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb228a74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f28a7b, 99 },
    { 0x18428a7c, 46 },
    { 0x1f628a7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x48928a80, 61 },
    { 0, 0 },
    { 0xebd28a82, 76 },
    { 0, 0 },
    { 0x4c28a84, 52 },
    { 0x96d28a85, 90 },
    { 0x86c28a86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef28a8a, 34 },
    { 0xa0e28a8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d928a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b28a93, 98 },
    { 0xd228a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0828aa1, 93 },
    { 0, 0 },
    { 0x6bb28aa3, 100 },
    { 0x65c28aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d28aa7, 90 },
    { 0x7928aa8, 17 },
    { 0xc5c28aa9, 90 },
    { 0x7cb28aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac28aae, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f328ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e28ab5, 76 },
    { 0x2d928ab5, 98 },
    { 0x46228ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17528abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e28ac1, 91 },
    { 0xe7d28ac2, 90 },
    { 0x3af28ac1, 33 },
    { 0x88428ac4, 95 },
    { 0x23128ac5, 23 },
    { 0, 0 },
    { 0xa3128ac7, 90 },
    { 0xb0028ac7, 55 },
    { 0x4bb28ac8, 45 },
    { 0, 0 },
    { 0x3a228acb, 76 },
    { 0x50628acc, 78 },
    { 0x41c28acd, 68 },
    { 0x6ca28ace, 99 },
    { 0xe2528acf, 69 },
    { 0x60928ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25228ad3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19228ad6, 33 },
    { 0x6b528ad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e428ada, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e28adf, 95 },
    { 0xd5128ae0, 33 },
    { 0, 0 },
    { 0xe9f28ae2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71328ae7, 33 },
    { 0xdef28ae8, 43 },
    { 0xc0028ae9, 75 },
    { 0xbcb28aea, 93 },
    { 0xc2a28ae9, 55 },
    { 0, 0 },
    { 0xbba28aed, 95 },
    { 0xc2328aee, 90 },
    { 0x30128aed, 95 },
    { 0x74928aef, 76 },
    { 0xc9b28aed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54128af4, 43 },
    { 0x46e28af5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba28afa, 61 },
    { 0x50628afb, 76 },
    { 0x6f228afb, 80 },
    { 0xaa728afc, 61 },
    { 0, 0 },
    { 0xe2128aff, 95 },
    { 0xb3a28aff, 90 },
    { 0xea628b00, 76 },
    { 0xbca28b02, 88 },
    { 0xba928b03, 33 },
    { 0x1e028b03, 68 },
    { 0x4b328b05, 98 },
    { 0xb1f28b06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28b09, 23 },
    { 0, 0 },
    { 0x39528b0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e428b0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x38528b12, 95 },
    { 0x13b28b13, 78 },
    { 0x59528b12, 34 },
    { 0, 0 },
    { 0x35c28b16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a28b19, 23 },
    { 0, 0 },
    { 0x5da28b1b, 61 },
    { 0xf9028b1c, 76 },
    { 0xf1028b1c, 76 },
    { 0xa9728b1e, 33 },
    { 0xda28b1f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d28b24, 95 },
    { 0x3cd28b25, 76 },
    { 0x40d28b26, 76 },
    { 0x90d28b27, 98 },
    { 0xa8128b26, 46 },
    { 0xb9828b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f28b2e, 61 },
    { 0x89128b2f, 33 },
    { 0, 0 },
    { 0xd1528b31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d928b34, 98 },
    { 0x29428b35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b428b38, 98 },
    { 0xc9c28b39, 95 },
    { 0xb8828b38, 95 },
    { 0xfe328b3b, 33 },
    { 0, 0 },
    { 0x4c528b3d, 98 },
    { 0x2a228b3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d28b4c, 93 },
    { 0x12428b4d, 34 },
    { 0xf7a28b4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7428b51, 8 },
    { 0, 0 },
    { 0xdfd28b53, 65 },
    { 0x83428b53, 76 },
    { 0, 0 },
    { 0x2628b56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7128b5a, 100 },
    { 0x72128b5a, 91 },
    { 0x8e328b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f28b60, 33 },
    { 0xa3628b61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15728b6c, 76 },
    { 0, 0 },
    { 0x42128b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de28b72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46628b76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb28b7d, 68 },
    { 0x50228b7d, 61 },
    { 0x25b28b7f, 95 },
    { 0, 0 },
    { 0x27628b81, 78 },
    { 0x32528b81, 33 },
    { 0x4d928b83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce28b96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5428b99, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb28b9c, 15 },
    { 0xed028b9c, 95 },
    { 0, 0 },
    { 0x54328b9f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76428ba6, 23 },
    { 0x4ee28ba7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c128baf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce128bb3, 23 },
    { 0xf7e28bb4, 90 },
    { 0x1ee28bb4, 34 },
    { 0x73228bb6, 33 },
    { 0xcf228bb4, 91 },
    { 0xa9f28bb8, 90 },
    { 0x63328bb9, 84 },
    { 0x58828bba, 33 },
    { 0x78528bb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d128bc2, 90 },
    { 0x74028bc3, 43 },
    { 0, 0 },
    { 0x69f28bc5, 66 },
    { 0, 0 },
    { 0x2e828bc7, 90 },
    { 0xa0328bc8, 85 },
    { 0x36028bc8, 76 },
    { 0, 0 },
    { 0x43a28bcb, 76 },
    { 0, 0 },
    { 0x86028bcd, 95 },
    { 0xf7628bce, 85 },
    { 0x2ee28bcf, 33 },
    { 0x5e928bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc28bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b628bda, 76 },
    { 0xcd228bdb, 98 },
    { 0, 0 },
    { 0x52728bdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb228be0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d28be3, 23 },
    { 0x2b028be3, 43 },
    { 0xc2a28be4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a28be9, 91 },
    { 0, 0 },
    { 0x79828beb, 33 },
    { 0, 0 },
    { 0xc1128bed, 93 },
    { 0x93f28bee, 98 },
    { 0, 0 },
    { 0xd0528bf0, 23 },
    { 0x86e28bf1, 78 },
    { 0x52b28bf2, 23 },
    { 0xd5128bf2, 36 },
    { 0xb1428bf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef328bfb, 33 },
    { 0, 0 },
    { 0x6828bfd, 90 },
    { 0, 0 },
    { 0xe6128bff, 76 },
    { 0xa7e28bff, 61 },
    { 0, 0 },
    { 0x9fa28c02, 34 },
    { 0x8b828c02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d28c06, 23 },
    { 0xf0b28c07, 90 },
    { 0x70828c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0228c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea28c12, 93 },
    { 0x2e228c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59428c17, 88 },
    { 0x6e628c18, 76 },
    { 0x96728c18, 95 },
    { 0x59828c1a, 33 },
    { 0x8a528c18, 55 },
    { 0x98728c17, 55 },
    { 0xf0828c1d, 90 },
    { 0x38128c1d, 95 },
    { 0x5df28c1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab028c28, 100 },
    { 0x45828c29, 98 },
    { 0xe5528c29, 34 },
    { 0x75128c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e828c2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a28c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8028c3d, 61 },
    { 0x55728c3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc28c46, 90 },
    { 0x5fc28c47, 87 },
    { 0, 0 },
    { 0x63a28c49, 90 },
    { 0, 0 },
    { 0x64728c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d28c4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a28c53, 23 },
    { 0x42428c53, 61 },
    { 0xea428c53, 91 },
    { 0x7b28c56, 34 },
    { 0x51228c56, 64 },
    { 0x37028c58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad28c5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c728c61, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc828c65, 52 },
    { 0xc5728c66, 61 },
    { 0x3ea28c67, 90 },
    { 0x6428c66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec028c6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76828c70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d28c76, 68 },
    { 0, 0 },
    { 0x22528c78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c628c80, 90 },
    { 0x9d328c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd628c8c, 95 },
    { 0x58128c8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24328c91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3528c94, 91 },
    { 0, 0 },
    { 0x1ea28c96, 97 },
    { 0xd2c28c97, 82 },
    { 0xf8d28c96, 17 },
    { 0, 0 },
    { 0x22728c9a, 46 },
    { 0x3ba28c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98128c9f, 90 },
    { 0x74a28ca0, 34 },
    { 0x7dc28ca0, 100 },
    { 0x828c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2028ca5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc28ca8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba28caf, 90 },
    { 0xfec28cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d28cb4, 98 },
    { 0xdac28cb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1128cba, 23 },
    { 0x44f28cbb, 98 },
    { 0, 0 },
    { 0x60e28cbd, 55 },
    { 0, 0 },
    { 0xee428cbf, 99 },
    { 0x7e028cc0, 23 },
    { 0x7a928cc1, 90 },
    { 0x28d28cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab28ccf, 61 },
    { 0, 0 },
    { 0x5b128cd1, 62 },
    { 0x5b828cd1, 23 },
    { 0xbf428cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81828cdb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b928ce1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff428ce5, 90 },
    { 0xa9728ce5, 23 },
    { 0xa0f28ce5, 33 },
    { 0xd2828ce5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb28cee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f28cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38928cf5, 43 },
    { 0x7d228cf5, 65 },
    { 0x24e28cf6, 61 },
    { 0x4f028cf8, 99 },
    { 0x2a628cf9, 76 },
    { 0x90b28cf5, 95 },
    { 0xef328cf7, 93 },
    { 0x5c428cfc, 61 },
    { 0x78728cfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e528d03, 65 },
    { 0, 0 },
    { 0x3f228d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d28d08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed28d13, 33 },
    { 0xf6928d14, 100 },
    { 0x24128d14, 91 },
    { 0xfec28d16, 66 },
    { 0x7e728d16, 93 },
    { 0x6a828d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce28d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85728d21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd28d25, 65 },
    { 0, 0 },
    { 0xc1928d27, 77 },
    { 0x2ad28d28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b828d2c, 61 },
    { 0x3ab28d2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a28d32, 78 },
    { 0x9c928d32, 76 },
    { 0x9be28d34, 55 },
    { 0x58d28d33, 61 },
    { 0xa2328d33, 95 },
    { 0, 0 },
    { 0x3e828d38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b28d3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028d41, 52 },
    { 0, 0 },
    { 0x34528d43, 95 },
    { 0, 0 },
    { 0x16f28d45, 10 },
    { 0xd5728d46, 46 },
    { 0, 0 },
    { 0x85e28d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde28d4c, 46 },
    { 0x5a928d4d, 61 },
    { 0x93d28d4e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b28d53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d28d5c, 33 },
    { 0xbc28d5c, 33 },
    { 0, 0 },
    { 0x72f28d5f, 33 },
    { 0xea628d60, 83 },
    { 0x37928d61, 100 },
    { 0xbf628d5f, 96 },
    { 0xe1428d63, 46 },
    { 0, 0 },
    { 0xfe028d65, 96 },
    { 0x2c628d66, 68 },
    { 0, 0 },
    { 0x45428d68, 90 },
    { 0x8e028d68, 91 },
    { 0x86028d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc28d6e, 84 },
    { 0xcfc28d6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b28d78, 77 },
    { 0x24a28d79, 98 },
    { 0, 0 },
    { 0x3e828d7b, 76 },
    { 0, 0 },
    { 0xc2328d7d, 61 },
    { 0xfee28d7e, 69 },
    { 0xaf628d7f, 77 },
    { 0x7e28d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94528d84, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a28d8a, 66 },
    { 0x37f28d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa528d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c28da0, 55 },
    { 0, 0 },
    { 0x6c928da2, 90 },
    { 0x40228da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc028da7, 81 },
    { 0, 0 },
    { 0x59828da9, 33 },
    { 0, 0 },
    { 0x89628dab, 34 },
    { 0x78628dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e428db1, 61 },
    { 0x64828db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e628db6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd028dbc, 34 },
    { 0, 0 },
    { 0x7d128dbe, 66 },
    { 0x16628dbf, 68 },
    { 0x9ce28dc0, 61 },
    { 0xf8e28dc0, 90 },
    { 0xde28dc2, 87 },
    { 0x4ec28dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd828dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f328dcb, 23 },
    { 0x95f28dcb, 65 },
    { 0x9a928dcd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a28dd4, 98 },
    { 0, 0 },
    { 0xd4028dd6, 98 },
    { 0xe4128dd6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61328dda, 95 },
    { 0, 0 },
    { 0x8aa28ddc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67928de0, 33 },
    { 0, 0 },
    { 0xa7f28de2, 65 },
    { 0, 0 },
    { 0xac828de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6528de7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43128dee, 66 },
    { 0x3b128dee, 61 },
    { 0, 0 },
    { 0x71628df1, 98 },
    { 0, 0 },
    { 0x25828df3, 46 },
    { 0, 0 },
    { 0x9d628df5, 96 },
    { 0, 0 },
    { 0x1b728df7, 90 },
    { 0xeba28df7, 76 },
    { 0, 0 },
    { 0x6ce28dfa, 33 },
    { 0x81128dfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e28dff, 90 },
    { 0xeb828dff, 17 },
    { 0x1c728e01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45728e0d, 90 },
    { 0xd6f28e0d, 88 },
    { 0x8ff28e0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7628e14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x23428e17, 90 },
    { 0, 0 },
    { 0xed028e19, 100 },
    { 0, 0 },
    { 0x5d428e1b, 23 },
    { 0, 0 },
    { 0x90928e1d, 95 },
    { 0, 0 },
    { 0xdc428e1f, 90 },
    { 0x9a828e20, 84 },
    { 0, 0 },
    { 0xf0d28e22, 99 },
    { 0x76c28e23, 33 },
    { 0x8dc28e22, 33 },
    { 0xee928e24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee28e2c, 55 },
    { 0x53f28e2d, 88 },
    { 0x4928e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba28e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69828e38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6528e3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d28e43, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc28e47, 33 },
    { 0, 0 },
    { 0xb5928e49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd828e55, 76 },
    { 0x44928e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d28e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e28e5f, 40 },
    { 0x3c628e60, 96 },
    { 0x90228e61, 78 },
    { 0, 0 },
    { 0x2e728e63, 95 },
    { 0, 0 },
    { 0xc228e65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x31028e68, 98 },
    { 0x5328e68, 98 },
    { 0x14b28e68, 43 },
    { 0x83028e6b, 87 },
    { 0, 0 },
    { 0x20528e6d, 34 },
    { 0xb4728e6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c328e73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8828e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61728e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e28e89, 23 },
    { 0x86328e89, 61 },
    { 0x4e628e8a, 84 },
    { 0x3ac28e8c, 95 },
    { 0x1aa28e8d, 33 },
    { 0x55028e8e, 88 },
    { 0xff228e8f, 100 },
    { 0x85728e90, 95 },
    { 0xd2128e90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32928e95, 99 },
    { 0x10928e96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a428e9e, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x77328ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35628ea7, 61 },
    { 0x21e28ea8, 90 },
    { 0xc3028ea9, 83 },
    { 0xe2d28ea9, 55 },
    { 0, 0 },
    { 0xc1f28eac, 43 },
    { 0x37f28eac, 77 },
    { 0xb8428eae, 34 },
    { 0xde28eaf, 61 },
    { 0x8d128eac, 33 },
    { 0xe3228eb1, 33 },
    { 0x40b28eae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13828eb7, 61 },
    { 0, 0 },
    { 0x32f28eb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc28ebc, 90 },
    { 0xed028ebd, 43 },
    { 0x28a28ebe, 95 },
    { 0x26128ebf, 90 },
    { 0x25928ebd, 90 },
    { 0, 0 },
    { 0xf28ec2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95928ec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2128ec9, 90 },
    { 0, 0 },
    { 0xa2328ecb, 93 },
    { 0x7d528ecc, 84 },
    { 0x2d728ecd, 90 },
    { 0x12f28ecd, 34 },
    { 0x9ee28ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95628edd, 43 },
    { 0xa6828ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34428ee1, 55 },
    { 0x33128ee2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9728ee6, 90 },
    { 0x11b28ee7, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a28eec, 76 },
    { 0, 0 },
    { 0xfd228eee, 23 },
    { 0, 0 },
    { 0xebb28ef0, 85 },
    { 0x5c28ef1, 34 },
    { 0xfb928ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d28ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38028efc, 76 },
    { 0xd9928efc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f228f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e28f07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc28f0e, 61 },
    { 0xa1528f0f, 61 },
    { 0x88f28f10, 46 },
    { 0, 0 },
    { 0x19e28f12, 95 },
    { 0xe5928f12, 91 },
    { 0x3fa28f13, 98 },
    { 0, 0 },
    { 0x50a28f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a28f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11028f1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe28f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f28f23, 95 },
    { 0xa9628f24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42728f28, 34 },
    { 0xf2b28f29, 90 },
    { 0x20128f2a, 99 },
    { 0xba828f2a, 55 },
    { 0xca928f29, 95 },
    { 0x39328f2b, 90 },
    { 0xb2628f2d, 90 },
    { 0xeaa28f2f, 34 },
    { 0x56e28f30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99328f37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62728f3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b28f3f, 61 },
    { 0x20c28f40, 100 },
    { 0, 0 },
    { 0xc1428f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428f46, 33 },
    { 0, 0 },
    { 0x6e928f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad028f4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9028f59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11928f5d, 91 },
    { 0xfdf28f5e, 23 },
    { 0, 0 },
    { 0x85828f60, 78 },
    { 0, 0 },
    { 0xd8928f62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x63728f65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb28f6a, 100 },
    { 0, 0 },
    { 0x1d728f6c, 95 },
    { 0x39128f6d, 87 },
    { 0x56028f6c, 33 },
    { 0x79828f6c, 33 },
    { 0xd7b28f70, 92 },
    { 0x9b828f6d, 23 },
    { 0x38428f72, 98 },
    { 0x61c28f73, 98 },
    { 0x5e028f73, 95 },
    { 0x10128f6c, 90 },
    { 0x9ab28f76, 61 },
    { 0xe6128f77, 34 },
    { 0x64428f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f28f7b, 85 },
    { 0x84e28f7c, 23 },
    { 0x3e728f7d, 33 },
    { 0xe0828f7e, 90 },
    { 0x89928f7f, 93 },
    { 0x5f628f80, 34 },
    { 0x95428f7f, 95 },
    { 0xfad28f82, 61 },
    { 0x50c28f7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac28f89, 17 },
    { 0x94f28f89, 55 },
    { 0xfe628f8b, 38 },
    { 0, 0 },
    { 0x74928f8d, 61 },
    { 0x4a028f8e, 93 },
    { 0x81628f8e, 98 },
    { 0, 0 },
    { 0x7af28f91, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef428f9f, 24 },
    { 0xc1728fa0, 61 },
    { 0x83a28fa1, 33 },
    { 0xcea28fa2, 90 },
    { 0x44f28fa2, 95 },
    { 0xe5928fa2, 61 },
    { 0xed528fa4, 90 },
    { 0x49a28fa6, 33 },
    { 0x54328fa7, 91 },
    { 0x5528fa8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34928fab, 90 },
    { 0, 0 },
    { 0x49b28fad, 76 },
    { 0x49a28fae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48828fb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10328fb4, 52 },
    { 0xcca28fb5, 43 },
    { 0, 0 },
    { 0x9a928fb7, 80 },
    { 0xfdd28fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99728fbc, 61 },
    { 0, 0 },
    { 0x61d28fbe, 90 },
    { 0, 0 },
    { 0x9d928fc0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30328fc7, 23 },
    { 0xbba28fc8, 74 },
    { 0, 0 },
    { 0x6e228fca, 95 },
    { 0xa4f28fca, 33 },
    { 0xcc328fcc, 52 },
    { 0, 0 },
    { 0xa228fce, 90 },
    { 0xf8728fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee28fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c28fd8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef028fe5, 40 },
    { 0xa4928fe5, 98 },
    { 0xb8628fe5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x62028fea, 43 },
    { 0, 0 },
    { 0xe1a28fec, 76 },
    { 0x3e028fed, 61 },
    { 0, 0 },
    { 0x5fe28fef, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92828ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3228ffb, 95 },
    { 0, 0 },
    { 0x3ed28ffd, 76 },
    { 0, 0 },
    { 0xa6e28fff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd29008, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34829013, 33 },
    { 0, 0 },
    { 0x90c29015, 98 },
    { 0xf3329015, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512901b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc062901f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda129023, 90 },
    { 0x9d29024, 93 },
    { 0x6ec29024, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d29029, 98 },
    { 0xc342902a, 91 },
    { 0xf4229029, 93 },
    { 0x522902a, 67 },
    { 0xe8c2902d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99529032, 87 },
    { 0x1c229033, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba2903e, 67 },
    { 0, 0 },
    { 0x68229040, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28929044, 98 },
    { 0, 0 },
    { 0x48829046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852904e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5629051, 61 },
    { 0x8c329051, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe29055, 90 },
    { 0xc8029055, 61 },
    { 0x7a529057, 95 },
    { 0xaf429058, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f29060, 95 },
    { 0xea629061, 61 },
    { 0xeb529062, 95 },
    { 0x90329060, 33 },
    { 0x2ac29062, 95 },
    { 0, 0 },
    { 0xe0f29066, 76 },
    { 0x9e929067, 90 },
    { 0x89629068, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc32906f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a29077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa312907c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5282907f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f29084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5029087, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d72908b, 56 },
    { 0xbe82908c, 97 },
    { 0x4b72908d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec929091, 90 },
    { 0, 0 },
    { 0x18129093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f29096, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c2909a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322909d, 34 },
    { 0x3302909e, 33 },
    { 0, 0 },
    { 0x6c9290a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1290a8, 23 },
    { 0, 0 },
    { 0x44f290aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355290af, 95 },
    { 0x83c290af, 33 },
    { 0x8ab290b1, 33 },
    { 0x754290b0, 98 },
    { 0xde6290b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7290ba, 100 },
    { 0, 0 },
    { 0x4fc290bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc290c0, 40 },
    { 0x986290c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a290c7, 67 },
    { 0x361290c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769290cf, 43 },
    { 0xe78290cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538290d4, 53 },
    { 0xb9e290d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00290d9, 66 },
    { 0x4b2290d9, 90 },
    { 0x7c8290db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8290e0, 34 },
    { 0xd45290e1, 81 },
    { 0x36d290e2, 90 },
    { 0x818290e3, 23 },
    { 0xfc5290e4, 91 },
    { 0x63b290e5, 68 },
    { 0xb98290e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632290eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547290ef, 93 },
    { 0, 0 },
    { 0x141290f1, 66 },
    { 0, 0 },
    { 0x77e290f3, 23 },
    { 0x4d0290f4, 90 },
    { 0xed2290f5, 61 },
    { 0x858290f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0029103, 90 },
    { 0xe5929104, 23 },
    { 0xac129105, 61 },
    { 0x13129104, 34 },
    { 0x68d29107, 61 },
    { 0x17029108, 33 },
    { 0x4c029104, 95 },
    { 0xdfd29104, 95 },
    { 0x9f229107, 61 },
    { 0x7962910c, 61 },
    { 0x83c2910c, 98 },
    { 0x97c2910e, 76 },
    { 0xc712910d, 61 },
    { 0xf3229110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d29113, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7342911c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02911f, 95 },
    { 0x98729120, 55 },
    { 0x4f429121, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a29125, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85029129, 93 },
    { 0xc192912a, 90 },
    { 0x4022912b, 23 },
    { 0x16d2912a, 33 },
    { 0xca92912d, 52 },
    { 0, 0 },
    { 0x71f2912f, 33 },
    { 0xb3e29130, 17 },
    { 0x8bb29131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7629135, 52 },
    { 0xda229135, 33 },
    { 0xdb729135, 61 },
    { 0x3a729138, 34 },
    { 0xbe629137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f82913c, 33 },
    { 0x1cd2913d, 90 },
    { 0x7982913c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc229142, 61 },
    { 0x65229142, 61 },
    { 0, 0 },
    { 0x13929145, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e29149, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc29156, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec29161, 76 },
    { 0xbb329161, 76 },
    { 0xe29163, 33 },
    { 0xfc29164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd482916a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91329170, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x49829173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95529177, 61 },
    { 0xb6629178, 66 },
    { 0x56d29177, 34 },
    { 0x7232917a, 99 },
    { 0x8432917b, 61 },
    { 0x11a2917b, 90 },
    { 0x9e52917d, 95 },
    { 0x25f2917e, 90 },
    { 0xb372917f, 90 },
    { 0, 0 },
    { 0x3a629181, 90 },
    { 0x15029182, 76 },
    { 0xb8129182, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab29189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c429191, 55 },
    { 0xb3c29192, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f29198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d291a1, 33 },
    { 0xddb291a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f291a5, 91 },
    { 0, 0 },
    { 0x94f291a7, 77 },
    { 0, 0 },
    { 0xe71291a9, 34 },
    { 0x9b7291aa, 36 },
    { 0xa9e291ab, 98 },
    { 0x37f291ac, 93 },
    { 0x8f291a9, 84 },
    { 0, 0 },
    { 0x719291af, 99 },
    { 0x550291af, 43 },
    { 0, 0 },
    { 0x700291b2, 23 },
    { 0xbf8291b3, 76 },
    { 0x5c291b2, 76 },
    { 0x6a7291b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9291b9, 61 },
    { 0x362291b9, 55 },
    { 0x69c291bb, 76 },
    { 0x547291bb, 98 },
    { 0, 0 },
    { 0x3f6291be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2291c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9291ce, 98 },
    { 0x47b291ce, 87 },
    { 0xbee291d0, 23 },
    { 0xb6f291d0, 90 },
    { 0, 0 },
    { 0xd2a291d3, 98 },
    { 0x402291d3, 66 },
    { 0, 0 },
    { 0x622291d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba291d9, 91 },
    { 0x5e6291da, 66 },
    { 0xbe8291d9, 52 },
    { 0x2a5291dc, 95 },
    { 0x626291dc, 87 },
    { 0, 0 },
    { 0xd6e291df, 66 },
    { 0x67e291df, 100 },
    { 0xfe3291df, 81 },
    { 0x7c291e2, 76 },
    { 0xaf6291e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b291e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211291ed, 52 },
    { 0xdaf291ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe291f4, 98 },
    { 0x42c291f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3291fc, 55 },
    { 0xc5a291fd, 90 },
    { 0, 0 },
    { 0x354291ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd929205, 68 },
    { 0x6ce29206, 88 },
    { 0x85329207, 87 },
    { 0, 0 },
    { 0x6bd29209, 23 },
    { 0x1ce2920a, 55 },
    { 0x76c2920a, 55 },
    { 0, 0 },
    { 0xcdb2920d, 33 },
    { 0x6582920d, 85 },
    { 0, 0 },
    { 0x87129210, 34 },
    { 0xe1629210, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a29215, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b29218, 76 },
    { 0x27929218, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8029223, 61 },
    { 0x86129224, 61 },
    { 0x6c629225, 98 },
    { 0, 0 },
    { 0x19e29227, 87 },
    { 0x9d629227, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2922f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c729233, 33 },
    { 0xfca29234, 95 },
    { 0, 0 },
    { 0x3d429236, 95 },
    { 0, 0 },
    { 0xa5a29238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e2923b, 33 },
    { 0xe12923c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2923f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c29242, 23 },
    { 0x26329243, 95 },
    { 0xa0c29243, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15729248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5352924e, 68 },
    { 0, 0 },
    { 0xb2529250, 61 },
    { 0x21c29251, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xab829254, 65 },
    { 0xbd629255, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca2925a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662925e, 90 },
    { 0xaa72925e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2129262, 55 },
    { 0x17829263, 23 },
    { 0xff29262, 23 },
    { 0xd4b29264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f2926d, 45 },
    { 0x552926e, 33 },
    { 0x1822926f, 87 },
    { 0, 0 },
    { 0x6ef29271, 76 },
    { 0xbff29272, 95 },
    { 0, 0 },
    { 0x1b329274, 38 },
    { 0xfe029274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b29278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452927e, 98 },
    { 0x2b32927f, 77 },
    { 0x8922927f, 33 },
    { 0xc3629281, 23 },
    { 0x1b02927e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a029285, 96 },
    { 0x3fa29285, 100 },
    { 0xe5429286, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b2928c, 33 },
    { 0, 0 },
    { 0x9c62928e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d29291, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf29295, 93 },
    { 0x9b629296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9729299, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2929e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a292a2, 33 },
    { 0, 0 },
    { 0x326292a4, 87 },
    { 0xa2b292a5, 76 },
    { 0x45f292a5, 98 },
    { 0x42f292a6, 91 },
    { 0x3c3292a6, 23 },
    { 0xfae292a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1292ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d292af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b292b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x426292ba, 23 },
    { 0, 0 },
    { 0x587292bc, 76 },
    { 0x694292bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e292c6, 46 },
    { 0xecc292c7, 67 },
    { 0, 0 },
    { 0xdf292c9, 90 },
    { 0x1c0292ca, 33 },
    { 0x324292cb, 98 },
    { 0x7a9292cb, 90 },
    { 0x683292ca, 90 },
    { 0x704292ce, 55 },
    { 0x41b292ce, 76 },
    { 0x306292cf, 98 },
    { 0xb57292d1, 95 },
    { 0x53d292d1, 90 },
    { 0x782292d3, 40 },
    { 0, 0 },
    { 0xd74292d5, 33 },
    { 0x75c292d6, 99 },
    { 0, 0 },
    { 0x177292d8, 23 },
    { 0, 0 },
    { 0x8db292da, 33 },
    { 0xacd292db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b292df, 61 },
    { 0x3a2292df, 55 },
    { 0xf30292e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f292ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd292f4, 95 },
    { 0, 0 },
    { 0x5b8292f6, 55 },
    { 0xf91292f7, 34 },
    { 0xb11292f7, 98 },
    { 0x5c8292f9, 46 },
    { 0x3dd292f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0292ff, 55 },
    { 0xa80292ff, 98 },
    { 0xf4f29301, 95 },
    { 0xfa229302, 90 },
    { 0xb3629302, 95 },
    { 0x36229304, 61 },
    { 0xe2029302, 33 },
    { 0xa5229303, 76 },
    { 0x81829307, 61 },
    { 0x84629308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482930b, 43 },
    { 0x3662930c, 43 },
    { 0, 0 },
    { 0x2192930e, 90 },
    { 0, 0 },
    { 0x77029310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8729319, 100 },
    { 0xdff2931a, 99 },
    { 0x8da2931b, 90 },
    { 0xe282931b, 88 },
    { 0x59d2931d, 55 },
    { 0xbe2931c, 61 },
    { 0x8b22931f, 76 },
    { 0x5ab2931b, 55 },
    { 0xcf02931a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1929324, 61 },
    { 0xcab29324, 87 },
    { 0x85029326, 93 },
    { 0x97729327, 33 },
    { 0xfa229326, 66 },
    { 0xe9729329, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c2932d, 90 },
    { 0, 0 },
    { 0x512932f, 91 },
    { 0, 0 },
    { 0xa7d29331, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96229334, 95 },
    { 0, 0 },
    { 0x8c929336, 75 },
    { 0, 0 },
    { 0xdb129338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f129344, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e29353, 90 },
    { 0, 0 },
    { 0x48629355, 91 },
    { 0x3cc29355, 67 },
    { 0, 0 },
    { 0x30129358, 99 },
    { 0, 0 },
    { 0x5ee2935a, 90 },
    { 0xd4d2935b, 61 },
    { 0xed82935c, 98 },
    { 0x6d92935d, 90 },
    { 0xca22935e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae29367, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432936a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa29370, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a29374, 89 },
    { 0x79e29375, 61 },
    { 0x84f29376, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2937a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc429382, 84 },
    { 0xf6729383, 52 },
    { 0x59e29384, 90 },
    { 0xa2429385, 80 },
    { 0xa629383, 80 },
    { 0, 0 },
    { 0xd2629388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c629391, 61 },
    { 0xa2e29391, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56029398, 98 },
    { 0, 0 },
    { 0x9272939a, 75 },
    { 0x472939b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x242939e, 23 },
    { 0x93a2939f, 85 },
    { 0, 0 },
    { 0x8cf293a1, 90 },
    { 0, 0 },
    { 0xfbf293a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621293ac, 61 },
    { 0xf58293ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4293b8, 85 },
    { 0x861293b9, 77 },
    { 0x322293ba, 100 },
    { 0xf77293bb, 90 },
    { 0x910293ba, 33 },
    { 0, 0 },
    { 0xbf0293be, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3293c1, 61 },
    { 0, 0 },
    { 0xb7d293c3, 81 },
    { 0x413293c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592293c9, 61 },
    { 0, 0 },
    { 0x59a293cb, 43 },
    { 0xb48293cc, 33 },
    { 0x827293cc, 76 },
    { 0xbb7293ce, 78 },
    { 0xf36293cf, 33 },
    { 0x97f293d0, 90 },
    { 0x189293d1, 78 },
    { 0x22293d1, 43 },
    { 0xbe6293d2, 91 },
    { 0x3fe293d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7293d8, 61 },
    { 0xc34293d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643293df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54293e3, 90 },
    { 0xf56293e4, 33 },
    { 0x5a7293e4, 95 },
    { 0, 0 },
    { 0x57293e7, 66 },
    { 0, 0 },
    { 0x869293e9, 90 },
    { 0xa65293ea, 23 },
    { 0x34b293eb, 100 },
    { 0x867293ea, 95 },
    { 0, 0 },
    { 0x562293ee, 23 },
    { 0xc5c293ee, 56 },
    { 0, 0 },
    { 0x6c2293f1, 94 },
    { 0, 0 },
    { 0xc2e293f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d293f9, 68 },
    { 0, 0 },
    { 0x1bc293fb, 61 },
    { 0x716293fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb29406, 90 },
    { 0, 0 },
    { 0xfb329408, 79 },
    { 0, 0 },
    { 0xbf2940a, 45 },
    { 0x1022940b, 66 },
    { 0, 0 },
    { 0xd892940d, 33 },
    { 0xec02940d, 33 },
    { 0x6112940e, 61 },
    { 0xbcd29410, 85 },
    { 0xd4729411, 90 },
    { 0, 0 },
    { 0xd6e29413, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d29418, 66 },
    { 0x86829418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf372941c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d29422, 23 },
    { 0, 0 },
    { 0xdeb29424, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e2942a, 33 },
    { 0x4382942b, 90 },
    { 0x5dd2942c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c29430, 78 },
    { 0x78129431, 23 },
    { 0x3ef29432, 75 },
    { 0x81a29433, 76 },
    { 0xa2b29430, 33 },
    { 0xf5529433, 33 },
    { 0xf4d29433, 23 },
    { 0x2ed29431, 55 },
    { 0x9dc29438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc72943c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d29440, 34 },
    { 0x2ba29441, 90 },
    { 0, 0 },
    { 0xc4f29443, 51 },
    { 0xda29444, 76 },
    { 0, 0 },
    { 0x1cb29446, 23 },
    { 0x3af29447, 61 },
    { 0x7e629447, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b02944c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd902944f, 36 },
    { 0xac629450, 93 },
    { 0, 0 },
    { 0xd5a29452, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6629455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c2945b, 38 },
    { 0x2622945c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87929465, 95 },
    { 0, 0 },
    { 0x4529467, 93 },
    { 0x9a029467, 87 },
    { 0, 0 },
    { 0xc0a2946a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab12946d, 77 },
    { 0x9042946d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11029472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27129475, 33 },
    { 0xf1b29476, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622947a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8382947d, 100 },
    { 0x1a52947d, 98 },
    { 0x4f82947f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64129484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11229489, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf02948e, 33 },
    { 0x3872948e, 80 },
    { 0, 0 },
    { 0xac429491, 23 },
    { 0xae329492, 98 },
    { 0xdb529492, 98 },
    { 0x5cd29494, 81 },
    { 0x67c29493, 61 },
    { 0xb529496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d2949e, 97 },
    { 0, 0 },
    { 0xa8f294a0, 23 },
    { 0, 0 },
    { 0xe3d294a2, 77 },
    { 0xbdb294a3, 98 },
    { 0x800294a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8294a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f294ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7294b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df294b5, 46 },
    { 0, 0 },
    { 0x42a294b7, 87 },
    { 0x9b0294b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95294bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928294c1, 17 },
    { 0, 0 },
    { 0x6df294c3, 88 },
    { 0xb72294c4, 61 },
    { 0, 0 },
    { 0x80d294c6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x338294c9, 55 },
    { 0xdc6294ca, 52 },
    { 0x99a294cb, 23 },
    { 0, 0 },
    { 0xb8c294cd, 17 },
    { 0xecb294ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc294d2, 65 },
    { 0x48294d3, 61 },
    { 0, 0 },
    { 0x6a1294d5, 84 },
    { 0xe0e294d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64294dd, 95 },
    { 0x8e4294de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f294e2, 87 },
    { 0x4da294e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a294e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46294ef, 88 },
    { 0xa53294f0, 33 },
    { 0xcef294ef, 23 },
    { 0, 0 },
    { 0x814294f3, 66 },
    { 0xac4294f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15294f8, 88 },
    { 0xc5d294f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a294fd, 33 },
    { 0xb2d294fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91729501, 46 },
    { 0x5fb29501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f529507, 55 },
    { 0x6f229507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d2950b, 33 },
    { 0x2742950c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a29512, 33 },
    { 0x8a829513, 46 },
    { 0x50429512, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0929517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b12951f, 99 },
    { 0xcd729520, 90 },
    { 0x24429521, 61 },
    { 0x39129522, 69 },
    { 0xef629523, 33 },
    { 0, 0 },
    { 0xe3729525, 52 },
    { 0xb9f29526, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252952e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529535, 68 },
    { 0x35629536, 34 },
    { 0x7429537, 90 },
    { 0, 0 },
    { 0xdd729539, 98 },
    { 0x9ad29539, 90 },
    { 0x7d62953b, 33 },
    { 0, 0 },
    { 0xdf42953d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1329542, 95 },
    { 0xfbb29543, 61 },
    { 0x23229544, 95 },
    { 0xa2629542, 61 },
    { 0, 0 },
    { 0x21429547, 61 },
    { 0xe9029548, 90 },
    { 0x11929548, 85 },
    { 0x8129548, 23 },
    { 0x87729548, 33 },
    { 0, 0 },
    { 0xdbd2954d, 90 },
    { 0, 0 },
    { 0xf872954f, 98 },
    { 0xd2b29550, 77 },
    { 0x70129551, 98 },
    { 0x18929552, 67 },
    { 0x24f29552, 55 },
    { 0xb1829552, 64 },
    { 0x6af29552, 33 },
    { 0x80529556, 93 },
    { 0x26f29556, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7732955d, 95 },
    { 0xd5d2955e, 64 },
    { 0x90c2955e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a629564, 91 },
    { 0, 0 },
    { 0x2c329566, 95 },
    { 0x9ff29567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8062956a, 90 },
    { 0x81b2956b, 100 },
    { 0x5a32956b, 33 },
    { 0x51e2956c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82729571, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40729574, 95 },
    { 0x6d629574, 99 },
    { 0x7e029576, 75 },
    { 0x9cc29577, 99 },
    { 0x8fe29578, 33 },
    { 0xec129576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8342957d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29580, 23 },
    { 0x8be29581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb29587, 33 },
    { 0x8b729587, 76 },
    { 0, 0 },
    { 0xea12958a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71029590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98429594, 34 },
    { 0xb4629594, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb262959c, 38 },
    { 0x96a2959c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38295a1, 33 },
    { 0x7d9295a1, 75 },
    { 0, 0 },
    { 0xb72295a4, 33 },
    { 0xf5f295a5, 23 },
    { 0x12c295a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1295aa, 76 },
    { 0, 0 },
    { 0xbfa295ac, 23 },
    { 0, 0 },
    { 0x6ab295ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce295ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0295bd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x407295c0, 89 },
    { 0x226295c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1295c8, 90 },
    { 0x56e295c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e295cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x485295d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff295d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1295dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e295df, 55 },
    { 0x28a295e0, 17 },
    { 0x830295e1, 95 },
    { 0, 0 },
    { 0xa5a295e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03295ec, 90 },
    { 0xbd5295ed, 90 },
    { 0x66e295ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25295f1, 95 },
    { 0, 0 },
    { 0x826295f3, 61 },
    { 0, 0 },
    { 0xb08295f5, 33 },
    { 0xf98295f5, 33 },
    { 0x904295f5, 55 },
    { 0, 0 },
    { 0xda295f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae295fe, 76 },
    { 0xf21295ff, 99 },
    { 0, 0 },
    { 0xd3b29601, 46 },
    { 0x11a29602, 23 },
    { 0x4a729602, 91 },
    { 0x78b29604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62529609, 61 },
    { 0x1da29609, 23 },
    { 0x7482960b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b29611, 43 },
    { 0, 0 },
    { 0xc829613, 66 },
    { 0, 0 },
    { 0x5ed29615, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2029619, 23 },
    { 0, 0 },
    { 0x1212961b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42961f, 46 },
    { 0xd9829620, 93 },
    { 0x32e2961f, 61 },
    { 0x2db29622, 90 },
    { 0x6129620, 90 },
    { 0x43229621, 95 },
    { 0x4c029625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc429629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe472962d, 33 },
    { 0x26b2962e, 99 },
    { 0x6092962f, 95 },
    { 0xd0729630, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a929635, 55 },
    { 0, 0 },
    { 0x3ca29637, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f2963a, 33 },
    { 0, 0 },
    { 0x17b2963c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b2963f, 90 },
    { 0x1dc2963f, 98 },
    { 0, 0 },
    { 0x78829642, 86 },
    { 0xc3229642, 17 },
    { 0xe2e29643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65929648, 90 },
    { 0xd7529649, 61 },
    { 0x5d229649, 84 },
    { 0xd4b2964b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x352964e, 33 },
    { 0xbdc2964e, 95 },
    { 0x3d129650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e29654, 55 },
    { 0x94729655, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x31129658, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782965b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85729661, 68 },
    { 0x48a29662, 87 },
    { 0x25429663, 61 },
    { 0xd5e29661, 55 },
    { 0xd7d29665, 46 },
    { 0x96029666, 66 },
    { 0x38329666, 78 },
    { 0x9ca29668, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b29670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7229677, 61 },
    { 0x45629678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2967c, 97 },
    { 0x4d82967d, 36 },
    { 0, 0 },
    { 0x23e2967f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84529682, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6129686, 100 },
    { 0, 0 },
    { 0xb7529688, 43 },
    { 0, 0 },
    { 0x2372968a, 33 },
    { 0, 0 },
    { 0xec2968c, 98 },
    { 0, 0 },
    { 0xf9e2968e, 100 },
    { 0x4f02968f, 33 },
    { 0, 0 },
    { 0x42729691, 90 },
    { 0xa7e29691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f529695, 55 },
    { 0xc5229695, 91 },
    { 0xb9729697, 33 },
    { 0, 0 },
    { 0xfc429699, 98 },
    { 0xdfb29699, 78 },
    { 0x5392969b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5296a8, 43 },
    { 0xac6296a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90296b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f296b5, 95 },
    { 0x77d296b5, 23 },
    { 0x4c9296b7, 83 },
    { 0x52296b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267296bd, 43 },
    { 0x213296be, 95 },
    { 0xb8a296bf, 98 },
    { 0, 0 },
    { 0x522296c1, 55 },
    { 0xd1c296c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe296cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd296d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6296d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x619296da, 34 },
    { 0xdc4296da, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359296e5, 90 },
    { 0xbd8296e6, 99 },
    { 0x7d8296e7, 33 },
    { 0x793296e7, 76 },
    { 0, 0 },
    { 0xeba296ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3296ed, 95 },
    { 0xf74296ed, 40 },
    { 0xe78296ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a296f5, 61 },
    { 0, 0 },
    { 0x898296f7, 88 },
    { 0, 0 },
    { 0x26c296f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d296fc, 61 },
    { 0xc9296fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67329707, 90 },
    { 0x6a129707, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9332970e, 55 },
    { 0x3de2970f, 34 },
    { 0xb1a29710, 23 },
    { 0xd0a2970f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e29716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e29719, 33 },
    { 0x13429719, 90 },
    { 0x36e2971b, 90 },
    { 0xbba2971c, 55 },
    { 0x1702971d, 55 },
    { 0x5c62971e, 43 },
    { 0, 0 },
    { 0x3d529720, 66 },
    { 0, 0 },
    { 0x82629722, 91 },
    { 0, 0 },
    { 0x39e29724, 23 },
    { 0, 0 },
    { 0xd4129726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d2972a, 23 },
    { 0x2e22972b, 99 },
    { 0x3e42972a, 95 },
    { 0x3322972d, 98 },
    { 0xd542972d, 34 },
    { 0, 0 },
    { 0xe3f29730, 33 },
    { 0xd2029731, 97 },
    { 0x40629732, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b29735, 99 },
    { 0, 0 },
    { 0x56a29737, 98 },
    { 0, 0 },
    { 0xd0429739, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9852973c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf529746, 53 },
    { 0, 0 },
    { 0x99029748, 43 },
    { 0, 0 },
    { 0x3442974a, 61 },
    { 0, 0 },
    { 0x112974c, 23 },
    { 0x3312974c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4229754, 33 },
    { 0xae229754, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67629759, 90 },
    { 0xb8729759, 23 },
    { 0xbf82975b, 34 },
    { 0x5572975c, 61 },
    { 0x1ee2975c, 90 },
    { 0, 0 },
    { 0xd682975f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb429763, 91 },
    { 0xd5029763, 100 },
    { 0xfd629763, 85 },
    { 0x95329766, 61 },
    { 0x1b29766, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d32976d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41229770, 55 },
    { 0x2d129770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262977f, 95 },
    { 0, 0 },
    { 0x6cb29781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a29785, 61 },
    { 0xd2529785, 81 },
    { 0, 0 },
    { 0xc5229788, 33 },
    { 0x86929789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd29790, 99 },
    { 0, 0 },
    { 0x7c429792, 33 },
    { 0, 0 },
    { 0x9a829794, 91 },
    { 0x1ab29795, 33 },
    { 0, 0 },
    { 0xea629797, 55 },
    { 0x9de29797, 33 },
    { 0xeb229799, 95 },
    { 0x6eb2979a, 100 },
    { 0, 0 },
    { 0x85f2979c, 90 },
    { 0x6152979c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e297a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547297a9, 23 },
    { 0xeec297a9, 23 },
    { 0x141297a9, 90 },
    { 0x8c2297ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d297b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4297b5, 23 },
    { 0x5fa297b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede297ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888297bf, 98 },
    { 0x5e3297bf, 98 },
    { 0x7ef297c1, 23 },
    { 0xabb297c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4297c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53297cc, 23 },
    { 0x5f297cd, 55 },
    { 0x591297ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4297d1, 98 },
    { 0x368297d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23297d9, 98 },
    { 0x1be297d9, 51 },
    { 0x870297d9, 61 },
    { 0x83e297dc, 90 },
    { 0xf5d297dd, 33 },
    { 0x57b297de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f297e1, 98 },
    { 0x7bf297e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b297e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0297ed, 90 },
    { 0x66d297ee, 76 },
    { 0xa40297ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x199297f2, 76 },
    { 0xdaf297f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254297f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x268297fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83297fe, 90 },
    { 0, 0 },
    { 0xdc229800, 90 },
    { 0, 0 },
    { 0x69b29802, 92 },
    { 0xce829803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae529806, 76 },
    { 0x67229807, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f62980a, 61 },
    { 0, 0 },
    { 0x8212980c, 33 },
    { 0, 0 },
    { 0x1f32980e, 95 },
    { 0x1792980f, 53 },
    { 0x24029810, 23 },
    { 0xd429811, 88 },
    { 0, 0 },
    { 0x57b29813, 55 },
    { 0xa1f29814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c29817, 23 },
    { 0xf8229817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda92981d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc929820, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f29824, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242982c, 100 },
    { 0, 0 },
    { 0xa532982e, 90 },
    { 0, 0 },
    { 0x52129830, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c29834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9272983a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9282983d, 95 },
    { 0xd262983e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b29843, 61 },
    { 0, 0 },
    { 0x51e29845, 52 },
    { 0, 0 },
    { 0xbae29847, 98 },
    { 0, 0 },
    { 0x3c329849, 33 },
    { 0, 0 },
    { 0x2e42984b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa29850, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3929853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce529859, 46 },
    { 0xa9c2985a, 61 },
    { 0, 0 },
    { 0xfa22985c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb29861, 64 },
    { 0xc4a29861, 46 },
    { 0xb5829863, 98 },
    { 0x68629863, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e29867, 98 },
    { 0x5ad29868, 90 },
    { 0x5cd29868, 61 },
    { 0x69429868, 98 },
    { 0x4952986a, 76 },
    { 0x1782986a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce029870, 61 },
    { 0xf1229871, 98 },
    { 0x1ea29871, 98 },
    { 0xe0929873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d2987c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2029883, 65 },
    { 0xa1129884, 90 },
    { 0xd3f29885, 55 },
    { 0x5f629886, 61 },
    { 0, 0 },
    { 0x9329888, 90 },
    { 0x7bd29888, 98 },
    { 0x7d92988a, 23 },
    { 0xc5c2988b, 33 },
    { 0x3cd2988c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75929894, 91 },
    { 0xec029895, 46 },
    { 0xd4e29896, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8432989a, 92 },
    { 0xb412989b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf982989e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572298a2, 66 },
    { 0x643298a2, 98 },
    { 0xd85298a4, 33 },
    { 0x930298a5, 66 },
    { 0xcd3298a3, 55 },
    { 0xa2a298a7, 98 },
    { 0x579298a8, 91 },
    { 0xaac298a9, 98 },
    { 0x184298aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee298ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea298b6, 55 },
    { 0x8af298b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa298bd, 46 },
    { 0xe2298bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31298c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98298c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23298c8, 23 },
    { 0x51d298c9, 91 },
    { 0x16f298ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x201298cd, 95 },
    { 0xbd0298ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf298d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355298dd, 98 },
    { 0x177298de, 99 },
    { 0, 0 },
    { 0x488298e0, 98 },
    { 0x9e6298e0, 89 },
    { 0, 0 },
    { 0x875298e3, 75 },
    { 0x84e298e4, 33 },
    { 0x822298e5, 88 },
    { 0xf56298e3, 91 },
    { 0xecf298e7, 98 },
    { 0x771298e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2298f1, 97 },
    { 0, 0 },
    { 0x31e298f3, 43 },
    { 0x2b7298f4, 24 },
    { 0xe2d298f5, 66 },
    { 0x141298f5, 61 },
    { 0x71298f5, 91 },
    { 0xe5d298f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364298fc, 33 },
    { 0x44298fd, 46 },
    { 0x98b298fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61729907, 43 },
    { 0x86429907, 61 },
    { 0x70d29908, 100 },
    { 0, 0 },
    { 0xb5a2990b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce29912, 34 },
    { 0xf3829913, 90 },
    { 0x68629912, 87 },
    { 0, 0 },
    { 0x8e429916, 78 },
    { 0x94f29917, 95 },
    { 0, 0 },
    { 0x9f229919, 96 },
    { 0, 0 },
    { 0xd8b2991b, 90 },
    { 0, 0 },
    { 0xb692991d, 61 },
    { 0x5522991e, 33 },
    { 0xc522991e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d929925, 78 },
    { 0x99429926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab2992d, 23 },
    { 0, 0 },
    { 0xef2992f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a829932, 76 },
    { 0, 0 },
    { 0x69e29934, 90 },
    { 0xc6529935, 98 },
    { 0x1829934, 84 },
    { 0, 0 },
    { 0x98229938, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba92993c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f2993f, 46 },
    { 0, 0 },
    { 0xd9129941, 61 },
    { 0x65929942, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a29945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39729948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54829950, 85 },
    { 0x3d129950, 91 },
    { 0x10b29952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6229955, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb29958, 33 },
    { 0, 0 },
    { 0x43b2995a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df2995d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47429961, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d029967, 98 },
    { 0, 0 },
    { 0x4b629969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d2996c, 55 },
    { 0, 0 },
    { 0xfd72996e, 90 },
    { 0, 0 },
    { 0xf9c29970, 100 },
    { 0xf7929971, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e29978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2512997c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542997f, 68 },
    { 0x80a2997f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c29984, 61 },
    { 0, 0 },
    { 0x35729986, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d29989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1412998e, 85 },
    { 0x32e2998f, 76 },
    { 0xfe929990, 46 },
    { 0x69d2998e, 45 },
    { 0x7dd29992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d29998, 95 },
    { 0x25b29999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12999c, 23 },
    { 0xd2e2999d, 61 },
    { 0x4202999e, 34 },
    { 0x28c2999f, 33 },
    { 0xde82999c, 55 },
    { 0x8482999c, 55 },
    { 0xb61299a0, 33 },
    { 0x698299a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a299ab, 33 },
    { 0x283299ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33299b0, 90 },
    { 0x11b299b0, 33 },
    { 0, 0 },
    { 0x1d299b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683299b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x956299ba, 61 },
    { 0x1f8299bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab299be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d299c3, 90 },
    { 0, 0 },
    { 0xf54299c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125299c9, 33 },
    { 0, 0 },
    { 0x153299cb, 76 },
    { 0xe6299cc, 93 },
    { 0xd23299cb, 90 },
    { 0x268299cb, 77 },
    { 0, 0 },
    { 0xc38299d0, 100 },
    { 0x103299d0, 2 },
    { 0, 0 },
    { 0x756299d3, 98 },
    { 0x8d5299d4, 65 },
    { 0, 0 },
    { 0xb0e299d6, 61 },
    { 0, 0 },
    { 0x633299d8, 88 },
    { 0, 0 },
    { 0x5df299da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f299dd, 95 },
    { 0, 0 },
    { 0x37299df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901299e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da299ed, 23 },
    { 0xb1c299ed, 55 },
    { 0xaa2299ef, 55 },
    { 0x900299ef, 94 },
    { 0xa95299ef, 98 },
    { 0x942299ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376299f8, 98 },
    { 0, 0 },
    { 0x2bf299fa, 40 },
    { 0x952299fb, 95 },
    { 0, 0 },
    { 0x173299fd, 76 },
    { 0xa6c299fe, 95 },
    { 0x3f0299ff, 84 },
    { 0x32c299ff, 55 },
    { 0xcc4299fd, 87 },
    { 0, 0 },
    { 0xd2829a03, 88 },
    { 0xc0e29a03, 33 },
    { 0xf7129a05, 23 },
    { 0x9f429a03, 95 },
    { 0x93329a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x38729a0a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad29a0e, 87 },
    { 0x16c29a0f, 90 },
    { 0x96029a0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a929a13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d29a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70929a21, 55 },
    { 0, 0 },
    { 0x12429a23, 90 },
    { 0xb4129a23, 61 },
    { 0xb5129a25, 91 },
    { 0x94d29a25, 90 },
    { 0x4b529a26, 93 },
    { 0, 0 },
    { 0x63c29a29, 43 },
    { 0x89929a29, 61 },
    { 0xebb29a2b, 99 },
    { 0x83a29a2b, 88 },
    { 0, 0 },
    { 0x1bd29a2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22729a31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b29a36, 65 },
    { 0x99b29a36, 76 },
    { 0, 0 },
    { 0x36329a39, 98 },
    { 0xfa129a3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0229a46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12729a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83829a4e, 78 },
    { 0, 0 },
    { 0xc4b29a50, 33 },
    { 0, 0 },
    { 0xa6d29a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93029a55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc429a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f29a5b, 97 },
    { 0xf0129a5c, 62 },
    { 0xbbe29a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b29a64, 90 },
    { 0, 0 },
    { 0x24029a66, 33 },
    { 0xfe429a66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1629a70, 23 },
    { 0x18b29a70, 95 },
    { 0x64e29a70, 61 },
    { 0, 0 },
    { 0x7c929a74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1329a78, 17 },
    { 0xcf29a79, 90 },
    { 0x29f29a7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16329a83, 76 },
    { 0x90129a83, 90 },
    { 0x94c29a84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f629a8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4229a91, 99 },
    { 0xefe29a92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b29a96, 93 },
    { 0, 0 },
    { 0x21f29a98, 33 },
    { 0, 0 },
    { 0xfd29a9a, 90 },
    { 0x19029a9b, 78 },
    { 0x7429a9b, 93 },
    { 0x50229a9d, 61 },
    { 0x28229a9e, 90 },
    { 0xc3829a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2629aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82229aaa, 33 },
    { 0x54e29aaa, 90 },
    { 0xa29aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5229ab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce429abb, 98 },
    { 0, 0 },
    { 0xda829abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe529ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea829ac6, 90 },
    { 0, 0 },
    { 0xf1429ac8, 76 },
    { 0, 0 },
    { 0x9f929aca, 23 },
    { 0x329aca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9429ace, 88 },
    { 0x89b29acf, 90 },
    { 0x78d29ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52929ad4, 76 },
    { 0x17a29ad5, 61 },
    { 0x6b129ad5, 33 },
    { 0x70429ad7, 95 },
    { 0x37029ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c29adc, 75 },
    { 0x1429add, 23 },
    { 0x19829ade, 33 },
    { 0xbf229ade, 97 },
    { 0xf7529adf, 23 },
    { 0x98729add, 33 },
    { 0x21429adf, 34 },
    { 0x98729ae3, 33 },
    { 0x7f529ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab29aec, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b29af7, 87 },
    { 0xfa29af7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c29afc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5429b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde829b10, 98 },
    { 0, 0 },
    { 0x2c29b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe29b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e629b1b, 68 },
    { 0, 0 },
    { 0xa7629b1d, 90 },
    { 0x8929b1e, 68 },
    { 0x7fa29b1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x87129b22, 23 },
    { 0x28b29b22, 43 },
    { 0xa0b29b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc29b28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5429b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d29b2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af29b31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a29b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8829b3b, 80 },
    { 0x7c029b3c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9629b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de29b44, 98 },
    { 0x9ef29b45, 43 },
    { 0, 0 },
    { 0xa2c29b47, 77 },
    { 0, 0 },
    { 0x1b029b49, 77 },
    { 0, 0 },
    { 0x7e329b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb29b52, 90 },
    { 0xa729b52, 33 },
    { 0x7e529b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db29b58, 33 },
    { 0, 0 },
    { 0x70e29b5a, 23 },
    { 0, 0 },
    { 0x6f29b5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b29b5f, 34 },
    { 0x8f929b60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1529b65, 98 },
    { 0, 0 },
    { 0xf5929b67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f29b6c, 65 },
    { 0, 0 },
    { 0xa1e29b6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4329b71, 100 },
    { 0xee029b72, 98 },
    { 0x1329b71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d29b77, 33 },
    { 0x1e429b77, 90 },
    { 0xf6929b79, 55 },
    { 0xa29b7a, 100 },
    { 0xa8a29b7a, 93 },
    { 0xfa429b7c, 76 },
    { 0x78029b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac29b81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62229b86, 90 },
    { 0x1e129b87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64529b8a, 95 },
    { 0x8b129b8b, 61 },
    { 0, 0 },
    { 0xcab29b8d, 33 },
    { 0, 0 },
    { 0x16f29b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd29b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9229b99, 85 },
    { 0, 0 },
    { 0x79829b9b, 98 },
    { 0x25629b9c, 90 },
    { 0x7c229b9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf129ba0, 87 },
    { 0x77929ba1, 99 },
    { 0xadf29ba0, 90 },
    { 0x10a29ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40329ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24829bac, 87 },
    { 0x52429bad, 55 },
    { 0x96d29bae, 55 },
    { 0, 0 },
    { 0xee829bb0, 43 },
    { 0x6cf29bb1, 93 },
    { 0xe0829bb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d029bb5, 76 },
    { 0x98229bb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c29bba, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d229bbd, 93 },
    { 0xe4c29bbe, 90 },
    { 0x42829bbf, 61 },
    { 0, 0 },
    { 0x94829bc1, 34 },
    { 0xd8b29bc2, 23 },
    { 0xb5529bc2, 43 },
    { 0, 0 },
    { 0x7a929bc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d829bcc, 61 },
    { 0x14729bcd, 69 },
    { 0x3229bcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa29bd6, 66 },
    { 0x2e729bd7, 33 },
    { 0x43e29bd8, 85 },
    { 0, 0 },
    { 0x24029bda, 76 },
    { 0xac929bdb, 90 },
    { 0, 0 },
    { 0x18229bdd, 90 },
    { 0, 0 },
    { 0xab929bdf, 46 },
    { 0xcfc29be0, 91 },
    { 0, 0 },
    { 0xc2229be2, 61 },
    { 0x27e29be3, 61 },
    { 0x3c529be2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab29bec, 90 },
    { 0, 0 },
    { 0x64929bee, 77 },
    { 0x23829bee, 83 },
    { 0, 0 },
    { 0x9ef29bf1, 91 },
    { 0x29829bf1, 52 },
    { 0x7d929bf3, 34 },
    { 0x94629bf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a429c01, 61 },
    { 0xfa029c02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66429c05, 46 },
    { 0, 0 },
    { 0x88329c07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88529c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22629c14, 33 },
    { 0, 0 },
    { 0x91429c16, 90 },
    { 0xe3429c17, 95 },
    { 0x98f29c18, 66 },
    { 0xbdc29c18, 90 },
    { 0x4aa29c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8629c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21129c25, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x97129c28, 75 },
    { 0xaa329c28, 90 },
    { 0xf529c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b29c32, 81 },
    { 0x23f29c32, 90 },
    { 0xf7d29c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b829c3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc29c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a29c44, 34 },
    { 0, 0 },
    { 0xd0b29c46, 90 },
    { 0xe3929c47, 55 },
    { 0x66d29c48, 23 },
    { 0xbc29c49, 78 },
    { 0x27629c4a, 61 },
    { 0x34f29c4b, 90 },
    { 0x41d29c4c, 83 },
    { 0xf6829c4d, 76 },
    { 0x96e29c4e, 65 },
    { 0xbf329c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd29c52, 36 },
    { 0xc6e29c52, 43 },
    { 0, 0 },
    { 0x31429c55, 43 },
    { 0x12029c56, 98 },
    { 0xd4a29c57, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e29c5e, 90 },
    { 0x66e29c5f, 90 },
    { 0, 0 },
    { 0x9e729c61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7129c65, 61 },
    { 0x24229c65, 90 },
    { 0xfb629c67, 91 },
    { 0xd3629c68, 66 },
    { 0x10029c69, 95 },
    { 0xeb229c6a, 61 },
    { 0x28529c6b, 59 },
    { 0xe9029c6c, 43 },
    { 0xe1029c6c, 55 },
    { 0xc3829c6e, 98 },
    { 0x55129c6f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e29c77, 66 },
    { 0x29129c78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee29c7b, 61 },
    { 0, 0 },
    { 0x9629c7d, 64 },
    { 0x72b29c7d, 81 },
    { 0, 0 },
    { 0x55c29c80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a29c83, 55 },
    { 0x1fb29c83, 76 },
    { 0xf3c29c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e29c89, 61 },
    { 0, 0 },
    { 0x1f129c8b, 61 },
    { 0x7dc29c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc29c8f, 66 },
    { 0xf9f29c8f, 68 },
    { 0, 0 },
    { 0xf3e29c92, 33 },
    { 0x90229c92, 61 },
    { 0xa2a29c94, 90 },
    { 0x78429c95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c029c9b, 56 },
    { 0x36129c9b, 76 },
    { 0xf9929c9b, 61 },
    { 0, 0 },
    { 0x52e29c9f, 68 },
    { 0x2fb29ca0, 66 },
    { 0, 0 },
    { 0x6c029ca2, 55 },
    { 0x4ba29ca3, 98 },
    { 0xa6029ca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62829ca9, 25 },
    { 0, 0 },
    { 0xd2d29cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0929cb0, 33 },
    { 0, 0 },
    { 0xa6f29cb2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c29cb5, 61 },
    { 0xedd29cb6, 87 },
    { 0xd5b29cb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6429cbd, 91 },
    { 0, 0 },
    { 0x14529cbf, 33 },
    { 0, 0 },
    { 0xfee29cc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a29cc4, 67 },
    { 0xf4329cc4, 61 },
    { 0xc2529cc6, 95 },
    { 0, 0 },
    { 0x7b129cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe529ccc, 81 },
    { 0x3729ccd, 90 },
    { 0xd3c29cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac629cd4, 61 },
    { 0, 0 },
    { 0xb3029cd6, 43 },
    { 0x30229cd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64829cdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb29ce2, 90 },
    { 0xed329ce2, 95 },
    { 0x55129ce4, 98 },
    { 0x75c29ce4, 87 },
    { 0x27a29ce5, 77 },
    { 0, 0 },
    { 0x61d29ce8, 81 },
    { 0, 0 },
    { 0x68229cea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc29cee, 95 },
    { 0xd5129cee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2529cf6, 95 },
    { 0x9ac29cf7, 98 },
    { 0xbef29cf8, 66 },
    { 0x71729cf6, 90 },
    { 0, 0 },
    { 0xd9929cfb, 85 },
    { 0x1cd29cfc, 43 },
    { 0x22b29cfd, 23 },
    { 0x87229cfc, 66 },
    { 0xc0829cff, 100 },
    { 0x93729d00, 90 },
    { 0x87829cfe, 75 },
    { 0xad829d02, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d829d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d129d0c, 61 },
    { 0x3e629d0d, 91 },
    { 0x9b829d0e, 95 },
    { 0xc7e29d0f, 96 },
    { 0, 0 },
    { 0xd5e29d11, 34 },
    { 0x76c29d12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb29d15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c29d1c, 78 },
    { 0, 0 },
    { 0x3fd29d1e, 55 },
    { 0x89929d1f, 49 },
    { 0x8db29d1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf429d25, 88 },
    { 0xfdc29d26, 90 },
    { 0x68229d27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c429d2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c29d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9429d36, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c929d3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd29d3d, 96 },
    { 0xf6529d3e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6229d41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace29d44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60829d4d, 61 },
    { 0xa0229d4e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e929d55, 76 },
    { 0x39329d56, 76 },
    { 0, 0 },
    { 0x62b29d58, 90 },
    { 0, 0 },
    { 0x55f29d5a, 23 },
    { 0xaa429d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34829d5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6229d62, 61 },
    { 0, 0 },
    { 0x40c29d64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a129d6d, 46 },
    { 0xbae29d6d, 83 },
    { 0xf6429d6d, 33 },
    { 0xa1729d70, 55 },
    { 0x3b229d6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43329d74, 33 },
    { 0x25329d75, 23 },
    { 0xc4729d76, 46 },
    { 0, 0 },
    { 0x47029d78, 43 },
    { 0, 0 },
    { 0xdc029d7a, 67 },
    { 0x88829d7b, 84 },
    { 0x54529d7c, 93 },
    { 0, 0 },
    { 0x13f29d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f29d86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129d8d, 98 },
    { 0x40729d8d, 90 },
    { 0xe1929d8d, 98 },
    { 0x4fa29d90, 100 },
    { 0x2c029d90, 17 },
    { 0, 0 },
    { 0x2fa29d93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62129d96, 68 },
    { 0x43829d97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a29d9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff29da2, 23 },
    { 0x81e29da2, 61 },
    { 0x8cd29da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d29da8, 95 },
    { 0x19c29da9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c29daf, 90 },
    { 0x6ef29db0, 34 },
    { 0x8c029db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20729db8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9629dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc629dc0, 77 },
    { 0x64729dc1, 77 },
    { 0x14229dc0, 99 },
    { 0xe4f29dc3, 93 },
    { 0x95a29dc0, 61 },
    { 0xd2b29dc0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0429dc8, 55 },
    { 0x64029dc9, 33 },
    { 0, 0 },
    { 0x9d29dcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3029dce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41729dd1, 100 },
    { 0x76529dd2, 61 },
    { 0x7e329dd1, 85 },
    { 0x3ab29dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33629dd9, 85 },
    { 0xf9529dda, 61 },
    { 0xbe529dda, 23 },
    { 0x86229ddc, 23 },
    { 0, 0 },
    { 0x6fd29dde, 93 },
    { 0xedb29dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6029de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6029de5, 66 },
    { 0xf4129de5, 97 },
    { 0, 0 },
    { 0x4f029de8, 43 },
    { 0, 0 },
    { 0xb0a29dea, 90 },
    { 0xe7f29dea, 90 },
    { 0xd7629dec, 96 },
    { 0xfda29ded, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff29df1, 87 },
    { 0, 0 },
    { 0x4bd29df3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e929df7, 87 },
    { 0, 0 },
    { 0xd2029df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e29e02, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x40929e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61929e08, 55 },
    { 0x1ea29e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b29e14, 65 },
    { 0xa2329e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1729e19, 90 },
    { 0x43f29e1a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11429e1e, 61 },
    { 0x6e529e1f, 34 },
    { 0x9cb29e20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef629e28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc629e2b, 91 },
    { 0x86a29e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f29e31, 95 },
    { 0x7ed29e32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79929e3b, 97 },
    { 0x39629e3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff329e3f, 81 },
    { 0x96129e40, 69 },
    { 0x1029e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a29e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e529e49, 55 },
    { 0, 0 },
    { 0x7829e4b, 33 },
    { 0, 0 },
    { 0x55529e4d, 95 },
    { 0xbcf29e4e, 65 },
    { 0x64429e4f, 95 },
    { 0x98729e50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb629e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79229e59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56929e5c, 68 },
    { 0x53129e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54529e61, 46 },
    { 0, 0 },
    { 0x65529e63, 23 },
    { 0, 0 },
    { 0xfdd29e65, 17 },
    { 0x9be29e66, 61 },
    { 0xe6129e67, 90 },
    { 0x93529e68, 66 },
    { 0xf0629e67, 98 },
    { 0x7c829e6a, 90 },
    { 0xec929e6b, 34 },
    { 0, 0 },
    { 0x8b429e6d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93529e71, 90 },
    { 0, 0 },
    { 0x22a29e73, 90 },
    { 0, 0 },
    { 0xd2c29e75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6929e7a, 76 },
    { 0, 0 },
    { 0xb6c29e7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11629e7f, 76 },
    { 0, 0 },
    { 0xa5a29e81, 95 },
    { 0x41429e81, 98 },
    { 0x74a29e81, 95 },
    { 0, 0 },
    { 0xe6929e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20629e8d, 33 },
    { 0x94b29e8d, 95 },
    { 0xa4829e8f, 23 },
    { 0x58429e8d, 33 },
    { 0x97d29e91, 90 },
    { 0xe2d29e92, 98 },
    { 0xa6a29e91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee29e96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde629e9b, 33 },
    { 0x47129e9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11029e9f, 90 },
    { 0, 0 },
    { 0xeaa29ea1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd29ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9229ead, 98 },
    { 0xbd229eae, 61 },
    { 0x73129eae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab029eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e29eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e29ebd, 77 },
    { 0x75229ebd, 95 },
    { 0x9c129ebf, 95 },
    { 0x61929ec0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x83429ec3, 91 },
    { 0xc5229ec4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e29ec9, 53 },
    { 0, 0 },
    { 0x25329ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f29ed1, 33 },
    { 0xc4229ed2, 55 },
    { 0xfb829ed3, 66 },
    { 0, 0 },
    { 0x2ee29ed5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8329ed9, 23 },
    { 0, 0 },
    { 0xc2929edb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea29ee0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0929ee3, 95 },
    { 0x8c629ee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f29eea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3929ef2, 46 },
    { 0xcca29ef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5129ef8, 90 },
    { 0x3e529ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2029efc, 23 },
    { 0xd8429efc, 98 },
    { 0xb8f29efe, 95 },
    { 0x7a629eff, 85 },
    { 0xa1829f00, 34 },
    { 0x8df29eff, 76 },
    { 0x6d29f02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc29f05, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f29f0a, 61 },
    { 0, 0 },
    { 0xaf729f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea629f0f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34329f13, 92 },
    { 0, 0 },
    { 0xdeb29f15, 48 },
    { 0x65129f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e29f19, 90 },
    { 0x26d29f1a, 61 },
    { 0, 0 },
    { 0xa29f1c, 95 },
    { 0xa0d29f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac529f20, 48 },
    { 0xb7729f21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d29f25, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25329f29, 33 },
    { 0xf6129f2a, 61 },
    { 0xa5f29f2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0229f30, 85 },
    { 0x99c29f30, 90 },
    { 0x85c29f31, 98 },
    { 0xfa329f30, 52 },
    { 0xada29f34, 98 },
    { 0xef329f35, 90 },
    { 0xea329f34, 34 },
    { 0, 0 },
    { 0x2c929f38, 76 },
    { 0, 0 },
    { 0x15429f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff829f3d, 23 },
    { 0x2929f3e, 76 },
    { 0xfd129f3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f729f46, 34 },
    { 0xae929f46, 90 },
    { 0, 0 },
    { 0x55529f49, 84 },
    { 0x1bb29f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63029f50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97929f54, 95 },
    { 0, 0 },
    { 0xcb129f56, 95 },
    { 0x16e29f57, 34 },
    { 0, 0 },
    { 0x34929f59, 61 },
    { 0, 0 },
    { 0xa7729f5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96029f5f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e29f62, 95 },
    { 0, 0 },
    { 0xbcd29f64, 46 },
    { 0xfa329f65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40229f69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54929f6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db29f70, 61 },
    { 0xec029f71, 33 },
    { 0x10f29f72, 33 },
    { 0x8b829f70, 23 },
    { 0x8cf29f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5229f78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93229f7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e29f82, 99 },
    { 0, 0 },
    { 0x11729f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13429f8d, 66 },
    { 0, 0 },
    { 0x27629f8f, 98 },
    { 0xe5429f90, 99 },
    { 0x4e929f91, 61 },
    { 0xbce29f92, 93 },
    { 0x9eb29f90, 87 },
    { 0x43d29f93, 23 },
    { 0, 0 },
    { 0x19b29f96, 90 },
    { 0, 0 },
    { 0xb2929f98, 76 },
    { 0xe6329f98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c929fa1, 34 },
    { 0x7d429fa2, 100 },
    { 0x64d29fa3, 95 },
    { 0x78529fa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11229fa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a29fac, 78 },
    { 0x7a929fac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d429fb2, 33 },
    { 0, 0 },
    { 0x65329fb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba329fbb, 46 },
    { 0x2b629fbc, 33 },
    { 0x2ef29fbd, 76 },
    { 0x1c829fbe, 61 },
    { 0x24429fbf, 85 },
    { 0xa7129fbc, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5829fc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74629fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5529fcb, 43 },
    { 0, 0 },
    { 0xc8329fcd, 43 },
    { 0xb8729fcd, 34 },
    { 0xcda29fcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb129fd2, 98 },
    { 0x1c429fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d529fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60429fe4, 46 },
    { 0xc1b29fe5, 90 },
    { 0x9da29fe5, 98 },
    { 0xbf829fe5, 66 },
    { 0x75029fe7, 98 },
    { 0, 0 },
    { 0xffa29fea, 90 },
    { 0x4d129feb, 33 },
    { 0xfa529fec, 34 },
    { 0, 0 },
    { 0xf6829fee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c029ff2, 61 },
    { 0x55a29ff3, 61 },
    { 0xe8229ff3, 43 },
    { 0x69029ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb52a000, 55 },
    { 0, 0 },
    { 0x10c2a002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492a005, 23 },
    { 0x8952a006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2a00b, 76 },
    { 0xa552a00c, 61 },
    { 0, 0 },
    { 0x9392a00e, 90 },
    { 0, 0 },
    { 0x16e2a010, 61 },
    { 0x8882a011, 84 },
    { 0xfe22a010, 61 },
    { 0, 0 },
    { 0xd322a014, 90 },
    { 0x4c02a015, 34 },
    { 0x4bf2a016, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2a01e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba2a022, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb882a02b, 87 },
    { 0x6592a02c, 33 },
    { 0, 0 },
    { 0xcb72a02e, 93 },
    { 0, 0 },
    { 0xf172a030, 61 },
    { 0x3932a031, 84 },
    { 0xfc82a030, 33 },
    { 0x3bb2a031, 43 },
    { 0xb382a030, 23 },
    { 0, 0 },
    { 0xcb62a036, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2a03c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572a03f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a2a045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f2a048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb72a052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7202a055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d22a05c, 33 },
    { 0x27b2a05d, 98 },
    { 0, 0 },
    { 0x2a72a05f, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2a063, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf272a06b, 90 },
    { 0x4b32a06c, 90 },
    { 0x95d2a06b, 75 },
    { 0x5282a06e, 23 },
    { 0, 0 },
    { 0x8012a070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52a077, 93 },
    { 0x7472a078, 71 },
    { 0xd462a079, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2a07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a081, 88 },
    { 0x9ee2a082, 93 },
    { 0x2612a083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f2a087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7532a08a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1882a08d, 90 },
    { 0, 0 },
    { 0xbdd2a08f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3622a094, 61 },
    { 0x36c2a095, 85 },
    { 0, 0 },
    { 0xa7b2a097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2a09d, 99 },
    { 0x9332a09d, 67 },
    { 0, 0 },
    { 0x49f2a0a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4292a0a4, 61 },
    { 0xdeb2a0a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f2a0a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252a0b0, 100 },
    { 0x6b32a0b0, 3 },
    { 0x4812a0b2, 55 },
    { 0xcc02a0b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb02a0b9, 61 },
    { 0x5482a0ba, 90 },
    { 0x96a2a0b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572a0c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142a0c8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2a0cb, 88 },
    { 0x3882a0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422a0d3, 93 },
    { 0x1612a0d3, 95 },
    { 0, 0 },
    { 0xa9d2a0d6, 95 },
    { 0xeaa2a0d7, 61 },
    { 0xc6f2a0d7, 98 },
    { 0x87f2a0d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7002a0e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef2a0e4, 90 },
    { 0x10c2a0e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb2a0e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72a0ed, 34 },
    { 0x702a0ed, 17 },
    { 0, 0 },
    { 0xa0b2a0f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb392a0f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd72a0fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb82a102, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa652a106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a10c, 23 },
    { 0, 0 },
    { 0xb552a10e, 90 },
    { 0xc902a10e, 85 },
    { 0x5112a10e, 95 },
    { 0x95a2a111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5782a114, 93 },
    { 0x7142a115, 23 },
    { 0x4012a116, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362a11a, 77 },
    { 0x4382a11b, 95 },
    { 0x8842a11b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed2a121, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2a124, 61 },
    { 0x6f22a124, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42a128, 33 },
    { 0x8642a129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e2a130, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822a134, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7492a139, 95 },
    { 0x55d2a13a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2792a13e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d2a141, 87 },
    { 0, 0 },
    { 0x6702a143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f2a147, 95 },
    { 0xbf12a147, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2a14b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a14f, 33 },
    { 0x8be2a150, 23 },
    { 0xb032a14f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc872a155, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42a158, 55 },
    { 0x36d2a158, 67 },
    { 0x732a15a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2a161, 61 },
    { 0xf8b2a162, 52 },
    { 0x28d2a163, 93 },
    { 0x6a02a164, 84 },
    { 0x7052a165, 53 },
    { 0xd1f2a165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c2a16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692a16d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2a170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf2a174, 66 },
    { 0, 0 },
    { 0x2f62a176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6832a17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd2a17f, 46 },
    { 0x56d2a180, 99 },
    { 0x4e82a181, 78 },
    { 0xa4c2a181, 38 },
    { 0x4c32a181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2a186, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf02a18a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12a18f, 77 },
    { 0xe7e2a190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662a194, 90 },
    { 0x66f2a195, 66 },
    { 0, 0 },
    { 0x9282a197, 91 },
    { 0, 0 },
    { 0xaca2a199, 93 },
    { 0x46a2a19a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2a19d, 90 },
    { 0x7ff2a19d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf2a1a3, 66 },
    { 0x3ce2a1a4, 33 },
    { 0x4012a1a5, 76 },
    { 0x1e02a1a5, 98 },
    { 0x1922a1a7, 23 },
    { 0x17a2a1a8, 23 },
    { 0x8ce2a1a8, 76 },
    { 0x4452a1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762a1b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f2a1b4, 33 },
    { 0, 0 },
    { 0xaa22a1b6, 76 },
    { 0x7f02a1b7, 33 },
    { 0x7f52a1b8, 33 },
    { 0x3c52a1b9, 95 },
    { 0xf462a1ba, 81 },
    { 0xb7a2a1b8, 67 },
    { 0x8e42a1b8, 43 },
    { 0x4be2a1ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3072a1c1, 99 },
    { 0x9f22a1c2, 55 },
    { 0x58f2a1c3, 61 },
    { 0x72c2a1c4, 17 },
    { 0, 0 },
    { 0x53d2a1c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf2a1c9, 43 },
    { 0xea72a1ca, 61 },
    { 0x1912a1cb, 23 },
    { 0x192a1ca, 46 },
    { 0x7cd2a1cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62a1d4, 55 },
    { 0, 0 },
    { 0xd432a1d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe042a1d9, 64 },
    { 0, 0 },
    { 0x1582a1db, 33 },
    { 0x5e42a1db, 34 },
    { 0x27a2a1dd, 75 },
    { 0x91c2a1db, 61 },
    { 0x6c52a1df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2a1e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b2a1e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5812a1ec, 64 },
    { 0x6572a1ed, 33 },
    { 0x53f2a1ee, 95 },
    { 0x94d2a1ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602a1f6, 93 },
    { 0x6612a1f6, 23 },
    { 0, 0 },
    { 0x4942a1f9, 61 },
    { 0xb782a1fa, 33 },
    { 0, 0 },
    { 0xbb72a1fc, 95 },
    { 0xcd92a1fd, 90 },
    { 0xcfb2a1fc, 98 },
    { 0xd012a1fd, 43 },
    { 0, 0 },
    { 0xa042a201, 95 },
    { 0, 0 },
    { 0x6d02a203, 95 },
    { 0xe402a204, 53 },
    { 0x3d42a205, 61 },
    { 0x1632a205, 61 },
    { 0x5aa2a203, 55 },
    { 0x9932a208, 95 },
    { 0xa702a208, 91 },
    { 0xb152a20a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32a210, 33 },
    { 0xef52a211, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b2a216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5972a219, 100 },
    { 0, 0 },
    { 0xb432a21b, 90 },
    { 0, 0 },
    { 0x352a21d, 61 },
    { 0x27e2a21d, 90 },
    { 0xd542a21f, 76 },
    { 0x7132a220, 76 },
    { 0xd352a220, 66 },
    { 0, 0 },
    { 0x9f82a223, 95 },
    { 0xadf2a224, 91 },
    { 0xf772a224, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf2a228, 46 },
    { 0x4c32a229, 33 },
    { 0x6632a22a, 91 },
    { 0, 0 },
    { 0x592a22c, 23 },
    { 0, 0 },
    { 0x322a22e, 64 },
    { 0x5c72a22f, 34 },
    { 0xe7a2a230, 33 },
    { 0xf82a231, 61 },
    { 0x9c62a230, 67 },
    { 0x8052a230, 98 },
    { 0x6722a234, 23 },
    { 0xc972a22f, 28 },
    { 0x38f2a236, 90 },
    { 0x8b22a237, 98 },
    { 0x9732a236, 33 },
    { 0x1e2a239, 33 },
    { 0x2f22a23a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9192a23d, 100 },
    { 0, 0 },
    { 0x6052a23f, 55 },
    { 0x7de2a240, 34 },
    { 0x6ad2a241, 98 },
    { 0x7872a240, 23 },
    { 0x9cb2a243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9042a247, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22a24f, 66 },
    { 0xf9e2a250, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e52a255, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba2a25a, 36 },
    { 0x6ad2a25b, 46 },
    { 0x77e2a25c, 99 },
    { 0, 0 },
    { 0xba82a25e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c2a261, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf442a265, 84 },
    { 0x7722a266, 33 },
    { 0, 0 },
    { 0xb712a268, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692a26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7882a272, 33 },
    { 0, 0 },
    { 0x9502a274, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa2a278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22a280, 61 },
    { 0, 0 },
    { 0xa5c2a282, 98 },
    { 0, 0 },
    { 0xa5b2a284, 90 },
    { 0xf1d2a284, 55 },
    { 0x3852a286, 43 },
    { 0xe0f2a287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea2a28e, 61 },
    { 0x4392a28f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172a293, 90 },
    { 0xd472a294, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d2a297, 61 },
    { 0, 0 },
    { 0xea32a299, 98 },
    { 0, 0 },
    { 0x5222a29b, 98 },
    { 0x7ac2a29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd2a2a1, 95 },
    { 0, 0 },
    { 0x9b62a2a3, 95 },
    { 0x29b2a2a3, 83 },
    { 0x80d2a2a5, 61 },
    { 0x3b62a2a4, 66 },
    { 0x1b32a2a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3592a2ab, 81 },
    { 0x7812a2ac, 84 },
    { 0, 0 },
    { 0x222a2ae, 34 },
    { 0x50e2a2ae, 84 },
    { 0x1bb2a2ae, 98 },
    { 0x1eb2a2b1, 33 },
    { 0x5492a2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8002a2b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602a2b8, 81 },
    { 0xd3d2a2b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52a2bc, 88 },
    { 0, 0 },
    { 0x91b2a2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662a2c2, 46 },
    { 0, 0 },
    { 0x5372a2c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b32a2cb, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d42a2ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2602a2d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1272a2dd, 98 },
    { 0xc052a2de, 61 },
    { 0xfb42a2de, 33 },
    { 0x8272a2de, 23 },
    { 0x2b62a2df, 33 },
    { 0xce22a2e0, 87 },
    { 0x4cd2a2df, 46 },
    { 0x1682a2e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f52a2e8, 43 },
    { 0xa1c2a2e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942a2f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a2a2f7, 84 },
    { 0xd612a2f8, 33 },
    { 0x4cb2a2f7, 84 },
    { 0, 0 },
    { 0xd5d2a2fb, 67 },
    { 0x69f2a2fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb2a304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe022a308, 33 },
    { 0, 0 },
    { 0x4622a30a, 66 },
    { 0x4892a30b, 90 },
    { 0x5202a30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a2a310, 95 },
    { 0xc9f2a311, 67 },
    { 0x6cc2a312, 33 },
    { 0x8292a313, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd22a316, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe152a319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c32a31c, 78 },
    { 0xb4a2a31c, 41 },
    { 0xdef2a31e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1902a321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f2a324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d2a327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd032a32a, 95 },
    { 0, 0 },
    { 0xec32a32c, 34 },
    { 0, 0 },
    { 0xd312a32e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac72a331, 49 },
    { 0xa5f2a332, 66 },
    { 0xa262a333, 90 },
    { 0, 0 },
    { 0x5bc2a335, 33 },
    { 0, 0 },
    { 0xb862a337, 65 },
    { 0xbab2a338, 61 },
    { 0xa862a339, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8372a33e, 93 },
    { 0xe212a33f, 34 },
    { 0, 0 },
    { 0x5f72a341, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1182a349, 23 },
    { 0xb2e2a349, 98 },
    { 0x3962a349, 23 },
    { 0x1d62a34c, 67 },
    { 0xa4d2a34d, 43 },
    { 0xc332a34c, 61 },
    { 0x6ab2a349, 61 },
    { 0x7652a34e, 59 },
    { 0x70d2a351, 90 },
    { 0xb2e2a352, 23 },
    { 0x5b2a352, 13 },
    { 0xc182a352, 33 },
    { 0x6b32a353, 85 },
    { 0xa202a356, 46 },
    { 0xe0e2a357, 84 },
    { 0x5e2a353, 90 },
    { 0x6a92a355, 84 },
    { 0x29f2a356, 76 },
    { 0x88e2a355, 33 },
    { 0x95a2a34c, 61 },
    { 0xd082a35d, 81 },
    { 0xdcc2a35d, 98 },
    { 0x4972a35f, 33 },
    { 0xf5e2a352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5402a365, 23 },
    { 0, 0 },
    { 0xa8d2a367, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe612a36b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4752a378, 61 },
    { 0x9112a378, 95 },
    { 0xceb2a37a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4072a380, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb772a386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e2a389, 33 },
    { 0xad92a389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f02a392, 90 },
    { 0, 0 },
    { 0x4b82a394, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9512a397, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e72a39e, 61 },
    { 0x6952a39f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a72a3a5, 34 },
    { 0xb752a3a6, 90 },
    { 0xbac2a3a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982a3ad, 34 },
    { 0x5712a3ae, 95 },
    { 0xb122a3ad, 34 },
    { 0x7912a3b0, 90 },
    { 0x4ae2a3b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe422a3b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2a3bc, 61 },
    { 0xa962a3bd, 33 },
    { 0x8702a3be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c22a3c1, 61 },
    { 0, 0 },
    { 0x3d02a3c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922a3c7, 66 },
    { 0x252a3c8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa732a3cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c12a3d0, 90 },
    { 0, 0 },
    { 0x10c2a3d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab22a3d5, 91 },
    { 0x2002a3d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2a3dc, 68 },
    { 0, 0 },
    { 0xecc2a3de, 23 },
    { 0xf092a3de, 52 },
    { 0, 0 },
    { 0x64c2a3e1, 46 },
    { 0x61f2a3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222a3e6, 95 },
    { 0x2a82a3e7, 61 },
    { 0x4432a3e8, 33 },
    { 0x65c2a3e9, 33 },
    { 0x7662a3ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212a3ed, 43 },
    { 0x5f2a3ed, 77 },
    { 0, 0 },
    { 0x4dd2a3f0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c2a3f5, 46 },
    { 0x4252a3f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba2a3f9, 46 },
    { 0x18a2a3fa, 61 },
    { 0x8332a3fb, 90 },
    { 0x14d2a3fa, 100 },
    { 0x2972a3f9, 46 },
    { 0x152a3fb, 90 },
    { 0x6792a3ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4412a403, 90 },
    { 0x2532a404, 90 },
    { 0xfc52a405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a408, 98 },
    { 0x9bd2a409, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b72a411, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5112a415, 99 },
    { 0x8c92a416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2a41a, 84 },
    { 0xcd12a41b, 98 },
    { 0xcbe2a41b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b32a420, 46 },
    { 0x1532a421, 95 },
    { 0x61a2a421, 76 },
    { 0xf122a423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc92a42a, 90 },
    { 0x8e52a42b, 95 },
    { 0xd892a42c, 23 },
    { 0x8822a42d, 88 },
    { 0xf602a42e, 33 },
    { 0x5252a42e, 90 },
    { 0x57e2a430, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede2a436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b2a439, 70 },
    { 0x3752a43a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7052a43e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542a443, 61 },
    { 0x4df2a444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a448, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa102a44e, 61 },
    { 0, 0 },
    { 0x72e2a450, 78 },
    { 0x9122a451, 33 },
    { 0xadf2a452, 78 },
    { 0xb082a450, 61 },
    { 0x13b2a454, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2a457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2a45f, 84 },
    { 0, 0 },
    { 0xa582a461, 90 },
    { 0, 0 },
    { 0x7672a463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2a467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e22a473, 52 },
    { 0, 0 },
    { 0x50b2a475, 61 },
    { 0xc712a475, 95 },
    { 0x942a475, 90 },
    { 0x3232a478, 61 },
    { 0x5eb2a479, 67 },
    { 0x5cc2a47a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa142a47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2a486, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d2a489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072a48c, 93 },
    { 0xa792a48d, 85 },
    { 0xc6f2a48e, 90 },
    { 0xded2a48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3642a492, 33 },
    { 0, 0 },
    { 0x2452a494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c32a497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2a49a, 76 },
    { 0, 0 },
    { 0x2982a49c, 90 },
    { 0x43d2a49d, 33 },
    { 0xe562a49d, 90 },
    { 0x8932a49f, 92 },
    { 0x8d62a49c, 33 },
    { 0x52a4a1, 90 },
    { 0x5bc2a4a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b2a4a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a42a4b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd2a4b8, 99 },
    { 0xb252a4b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2262a4c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb02a4cb, 84 },
    { 0xf2c2a4cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a2a4d2, 95 },
    { 0, 0 },
    { 0x26e2a4d4, 69 },
    { 0x3bb2a4d4, 90 },
    { 0x9bf2a4d4, 33 },
    { 0x1222a4d7, 93 },
    { 0x392a4d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3532a4dd, 90 },
    { 0x81c2a4de, 100 },
    { 0xd602a4dd, 87 },
    { 0x9ee2a4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f82a4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9872a4e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc2a4ef, 95 },
    { 0xbf82a4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2a4f4, 33 },
    { 0x8eb2a4f5, 33 },
    { 0, 0 },
    { 0x2b02a4f7, 33 },
    { 0x7092a4f8, 62 },
    { 0x49b2a4f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf402a4fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d2a502, 95 },
    { 0, 0 },
    { 0xfa92a504, 90 },
    { 0, 0 },
    { 0x27c2a506, 100 },
    { 0x5cb2a506, 23 },
    { 0x6702a508, 90 },
    { 0, 0 },
    { 0x6bc2a50a, 76 },
    { 0x3152a50b, 85 },
    { 0x6822a50c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12a50f, 95 },
    { 0x97a2a50f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb92a514, 90 },
    { 0xd462a514, 34 },
    { 0x23c2a514, 61 },
    { 0, 0 },
    { 0x7602a518, 55 },
    { 0, 0 },
    { 0xfde2a51a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12a51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2a520, 61 },
    { 0x85c2a521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc12a527, 61 },
    { 0, 0 },
    { 0x1572a529, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc952a52f, 33 },
    { 0, 0 },
    { 0xb022a531, 61 },
    { 0, 0 },
    { 0x7f62a533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12a537, 61 },
    { 0, 0 },
    { 0xb22a539, 55 },
    { 0xa62a53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7592a53d, 43 },
    { 0, 0 },
    { 0xaea2a53f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8902a54e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2a553, 87 },
    { 0x84d2a554, 88 },
    { 0x652a553, 55 },
    { 0x96f2a556, 23 },
    { 0x9de2a557, 98 },
    { 0xf242a556, 91 },
    { 0xec32a556, 84 },
    { 0x9cb2a55a, 33 },
    { 0x362a557, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2a562, 95 },
    { 0x2842a562, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce52a566, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb2a569, 95 },
    { 0, 0 },
    { 0x7c22a56b, 62 },
    { 0xbae2a56c, 43 },
    { 0xa52a56c, 93 },
    { 0x57c2a56c, 55 },
    { 0, 0 },
    { 0xce02a570, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2a573, 68 },
    { 0x1742a574, 61 },
    { 0x6952a574, 61 },
    { 0x6032a573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4132a579, 61 },
    { 0xd2a57a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1432a580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8622a589, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a2a58e, 23 },
    { 0xd9c2a58e, 53 },
    { 0xf282a590, 77 },
    { 0x422a58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192a596, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2a59b, 33 },
    { 0xc0b2a59c, 87 },
    { 0xc682a59d, 87 },
    { 0, 0 },
    { 0x8102a59f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f2a5a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082a5a5, 91 },
    { 0x39e2a5a5, 76 },
    { 0x532a5a5, 34 },
    { 0, 0 },
    { 0xd822a5a9, 77 },
    { 0, 0 },
    { 0x91c2a5ab, 76 },
    { 0, 0 },
    { 0x2352a5ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c2a5b0, 78 },
    { 0xafb2a5b0, 90 },
    { 0x83a2a5b2, 33 },
    { 0xba42a5b3, 33 },
    { 0x9592a5b3, 77 },
    { 0x8612a5b0, 55 },
    { 0x67f2a5b6, 67 },
    { 0x1712a5b7, 61 },
    { 0, 0 },
    { 0x8362a5b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e2a5bd, 33 },
    { 0x1562a5be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92a5c3, 90 },
    { 0x8492a5c4, 91 },
    { 0xbec2a5c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf102a5c9, 87 },
    { 0x59f2a5ca, 91 },
    { 0x4962a5cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2a5d1, 95 },
    { 0x1e82a5d1, 90 },
    { 0x372a5d3, 51 },
    { 0x1092a5d4, 64 },
    { 0x3522a5d5, 61 },
    { 0x9642a5d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce12a5da, 98 },
    { 0xb882a5da, 55 },
    { 0x27b2a5dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962a5e2, 64 },
    { 0x9462a5e2, 61 },
    { 0, 0 },
    { 0x3bf2a5e5, 17 },
    { 0x9102a5e6, 61 },
    { 0xd712a5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7452a5ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa182a5f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02a5f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952a5f9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad2a602, 61 },
    { 0x1642a603, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1322a608, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f2a60b, 33 },
    { 0, 0 },
    { 0x14b2a60d, 61 },
    { 0xb922a60d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc2a614, 90 },
    { 0x8d2a615, 68 },
    { 0xd892a615, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc932a61c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f2a61f, 46 },
    { 0xf362a61f, 33 },
    { 0x5762a61f, 45 },
    { 0x1722a622, 23 },
    { 0xf0f2a623, 33 },
    { 0xecd2a622, 90 },
    { 0x3b52a625, 55 },
    { 0x7f2a624, 33 },
    { 0x1ac2a627, 76 },
    { 0xe232a626, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882a62b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2832a62f, 85 },
    { 0x5de2a62f, 33 },
    { 0xa472a631, 33 },
    { 0x2fd2a632, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f42a639, 33 },
    { 0xf432a639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd22a63d, 61 },
    { 0x5942a63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb2a644, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2a647, 95 },
    { 0x7b2a647, 98 },
    { 0xad72a649, 33 },
    { 0, 0 },
    { 0x70b2a64b, 90 },
    { 0x7792a64c, 98 },
    { 0xcb32a64d, 23 },
    { 0, 0 },
    { 0x4872a64f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2a652, 76 },
    { 0x5f72a653, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf872a65a, 88 },
    { 0, 0 },
    { 0x5d92a65c, 55 },
    { 0x5072a65d, 33 },
    { 0x4512a65e, 98 },
    { 0, 0 },
    { 0xb882a660, 61 },
    { 0, 0 },
    { 0x8292a662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2a66a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc302a671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb2a678, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2a67b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5482a67e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1102a681, 76 },
    { 0x7652a682, 95 },
    { 0xa452a681, 95 },
    { 0x5082a684, 43 },
    { 0x22a685, 61 },
    { 0x6052a684, 76 },
    { 0x2ac2a687, 23 },
    { 0xeea2a688, 97 },
    { 0x1ed2a686, 90 },
    { 0x6a72a687, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6272a68e, 90 },
    { 0xf92a68f, 99 },
    { 0, 0 },
    { 0xaa72a691, 61 },
    { 0x26b2a692, 43 },
    { 0x77c2a693, 95 },
    { 0x7eb2a692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2a697, 93 },
    { 0, 0 },
    { 0x7d72a699, 90 },
    { 0x7d82a69a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c2a6a6, 33 },
    { 0xc7a2a6a6, 93 },
    { 0, 0 },
    { 0xca82a6a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252a6ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc2a6b5, 23 },
    { 0, 0 },
    { 0x70e2a6b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2a6bb, 96 },
    { 0xf972a6bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6952a6bf, 23 },
    { 0xe3e2a6c0, 91 },
    { 0x95f2a6bf, 76 },
    { 0x6602a6c2, 61 },
    { 0xada2a6c2, 76 },
    { 0xd292a6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2562a6c8, 95 },
    { 0xfb82a6c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832a6cf, 76 },
    { 0x83c2a6d0, 55 },
    { 0x5042a6d1, 60 },
    { 0x2452a6d0, 98 },
    { 0x89f2a6d1, 33 },
    { 0xea82a6d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2a6d7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef2a6da, 76 },
    { 0x22c2a6db, 52 },
    { 0, 0 },
    { 0x3142a6dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1672a6e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e2a6e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582a6f0, 76 },
    { 0xba52a6f1, 98 },
    { 0xac2a6f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9782a6f7, 66 },
    { 0x48f2a6f8, 48 },
    { 0x57a2a6f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402a6fc, 98 },
    { 0x1662a6fd, 90 },
    { 0xa222a6fd, 95 },
    { 0x1b72a6ff, 84 },
    { 0x13d2a700, 46 },
    { 0x76e2a701, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2a708, 96 },
    { 0x2462a709, 90 },
    { 0, 0 },
    { 0x7982a70b, 61 },
    { 0xddc2a70b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a2a719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf42a71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7772a721, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2a725, 95 },
    { 0, 0 },
    { 0x7762a727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2662a72b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca2a72e, 95 },
    { 0xed72a72e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc2a735, 33 },
    { 0x3d72a736, 58 },
    { 0x2be2a735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12a73b, 95 },
    { 0xb592a73b, 52 },
    { 0, 0 },
    { 0xde42a73e, 43 },
    { 0, 0 },
    { 0x25e2a740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x912a743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a12a746, 90 },
    { 0, 0 },
    { 0x1d2a748, 33 },
    { 0, 0 },
    { 0x3eb2a74a, 92 },
    { 0xf3d2a74a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622a74f, 52 },
    { 0x2e92a74f, 33 },
    { 0x6a22a751, 68 },
    { 0xf302a752, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2a75d, 76 },
    { 0x2ac2a75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a2a762, 95 },
    { 0x8642a762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1832a766, 34 },
    { 0, 0 },
    { 0x5752a768, 98 },
    { 0, 0 },
    { 0x5572a76a, 98 },
    { 0, 0 },
    { 0x66c2a76c, 61 },
    { 0x802a76c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1712a772, 23 },
    { 0x9fe2a773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f2a778, 91 },
    { 0x8442a779, 93 },
    { 0x4b92a778, 95 },
    { 0xb902a77b, 95 },
    { 0x4682a77c, 95 },
    { 0xbf2a77d, 33 },
    { 0x9f72a77e, 92 },
    { 0x7332a77d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc832a782, 91 },
    { 0xfd52a783, 90 },
    { 0xb4c2a784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372a795, 64 },
    { 0xde02a795, 85 },
    { 0x1042a797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc432a79f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7042a7a5, 61 },
    { 0x2f02a7a6, 95 },
    { 0x9272a7a7, 33 },
    { 0xccb2a7a7, 90 },
    { 0, 0 },
    { 0x9df2a7aa, 85 },
    { 0x15c2a7ab, 90 },
    { 0x8ed2a7ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc532a7b6, 76 },
    { 0, 0 },
    { 0x8f12a7b8, 95 },
    { 0xfc52a7b9, 34 },
    { 0x7792a7ba, 90 },
    { 0x3ac2a7bb, 87 },
    { 0xd5d2a7ba, 61 },
    { 0xed2a7bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf62a7c7, 90 },
    { 0xded2a7c8, 33 },
    { 0xe532a7c8, 61 },
    { 0x85d2a7c8, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c32a7cf, 77 },
    { 0xce72a7cf, 99 },
    { 0x4462a7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb72a7d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e2a7d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af2a7da, 95 },
    { 0xde02a7db, 61 },
    { 0x542a7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc82a7e3, 61 },
    { 0, 0 },
    { 0x9732a7e5, 61 },
    { 0x5292a7e6, 100 },
    { 0xdea2a7e5, 23 },
    { 0x662a7e8, 81 },
    { 0, 0 },
    { 0x3852a7ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f2a7ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc042a7f0, 90 },
    { 0xaeb2a7f1, 23 },
    { 0x3e02a7f1, 98 },
    { 0, 0 },
    { 0xa5d2a7f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62a7f7, 23 },
    { 0xe972a7f7, 90 },
    { 0xa7e2a7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2a7fd, 76 },
    { 0xa3c2a7fe, 90 },
    { 0xcc12a7fd, 90 },
    { 0x5182a800, 33 },
    { 0, 0 },
    { 0x1592a802, 74 },
    { 0, 0 },
    { 0x7272a804, 46 },
    { 0x2892a805, 67 },
    { 0, 0 },
    { 0x3cd2a807, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c32a80b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7142a810, 33 },
    { 0xc7d2a810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf2a815, 90 },
    { 0x4d82a815, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b82a81a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f2a81f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b2a822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd2a825, 61 },
    { 0x3e12a825, 55 },
    { 0, 0 },
    { 0x7892a828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12a82d, 90 },
    { 0x35f2a82d, 31 },
    { 0x33b2a82f, 91 },
    { 0xce82a82f, 98 },
    { 0x4ef2a82f, 98 },
    { 0x2f22a830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa62a836, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942a839, 90 },
    { 0x60c2a83a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed2a840, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc002a843, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9002a847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e32a84a, 43 },
    { 0xa242a84b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae2a851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2a857, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d92a85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e2a860, 67 },
    { 0x46e2a860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312a864, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b02a867, 98 },
    { 0x4302a868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52a86b, 96 },
    { 0x9982a86b, 33 },
    { 0x6272a86d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb182a870, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a72a876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2a87b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8142a883, 85 },
    { 0x5492a884, 98 },
    { 0xb682a885, 61 },
    { 0xd462a884, 78 },
    { 0, 0 },
    { 0x1752a888, 76 },
    { 0, 0 },
    { 0x6b82a88a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e62a88f, 61 },
    { 0, 0 },
    { 0x9592a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2a896, 17 },
    { 0, 0 },
    { 0x4692a898, 98 },
    { 0x40b2a899, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32a89e, 66 },
    { 0xadf2a89f, 98 },
    { 0, 0 },
    { 0xd842a8a1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f2a8a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e2a8ad, 33 },
    { 0, 0 },
    { 0xd0d2a8af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12a8b3, 90 },
    { 0, 0 },
    { 0x9032a8b5, 98 },
    { 0xbab2a8b6, 78 },
    { 0x2142a8b7, 23 },
    { 0x5892a8b8, 46 },
    { 0, 0 },
    { 0x21f2a8ba, 23 },
    { 0, 0 },
    { 0xfd42a8bc, 33 },
    { 0x8362a8bd, 34 },
    { 0x9ec2a8be, 84 },
    { 0x34e2a8bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc32a8c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f2a8c5, 98 },
    { 0, 0 },
    { 0x2102a8c7, 61 },
    { 0, 0 },
    { 0x5432a8c9, 61 },
    { 0x5c02a8c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe42a8cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2a8d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f2a8d5, 34 },
    { 0xec62a8d6, 91 },
    { 0xe2a2a8d7, 90 },
    { 0x7b62a8d8, 34 },
    { 0x3902a8d8, 33 },
    { 0x95e2a8da, 55 },
    { 0xecc2a8db, 76 },
    { 0x2f32a8d9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8e1, 90 },
    { 0xe352a8e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442a8e8, 85 },
    { 0x6752a8e9, 78 },
    { 0, 0 },
    { 0x132a8eb, 23 },
    { 0x51b2a8eb, 90 },
    { 0xdec2a8ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422a8f0, 88 },
    { 0, 0 },
    { 0xdc52a8f2, 74 },
    { 0xa482a8f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f2a8f7, 52 },
    { 0xd682a8f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea2a8fe, 98 },
    { 0x2f12a8fe, 61 },
    { 0xb2a2a900, 61 },
    { 0, 0 },
    { 0xf3e2a902, 33 },
    { 0x9752a902, 23 },
    { 0, 0 },
    { 0xc842a905, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52a908, 90 },
    { 0x58f2a909, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe822a90c, 99 },
    { 0xde2a90d, 98 },
    { 0xdbe2a90e, 33 },
    { 0x6c72a90f, 52 },
    { 0, 0 },
    { 0xd3e2a911, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22a917, 33 },
    { 0xd802a918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd702a91d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef2a926, 68 },
    { 0xc622a927, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2a92c, 61 },
    { 0x6e22a92d, 99 },
    { 0x5ed2a92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8752a934, 61 },
    { 0xdf2a935, 98 },
    { 0x2a02a936, 33 },
    { 0x38b2a937, 98 },
    { 0xd452a938, 33 },
    { 0xece2a934, 33 },
    { 0xad72a937, 91 },
    { 0, 0 },
    { 0x4dc2a93c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c2a945, 90 },
    { 0xcc02a946, 80 },
    { 0x8d2a945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a94a, 43 },
    { 0xf742a94b, 90 },
    { 0x70a2a94a, 43 },
    { 0, 0 },
    { 0x6cb2a94e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e2a952, 61 },
    { 0xcf2a953, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe712a95c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6892a962, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8212a969, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b2a96f, 52 },
    { 0x8452a970, 38 },
    { 0xc7c2a970, 76 },
    { 0x5f32a970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf2a977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92a97b, 47 },
    { 0, 0 },
    { 0x2c02a97d, 95 },
    { 0, 0 },
    { 0x5682a97f, 95 },
    { 0x6812a980, 61 },
    { 0xbdd2a980, 33 },
    { 0x85c2a97f, 61 },
    { 0x78e2a983, 61 },
    { 0xdc72a983, 33 },
    { 0x45f2a985, 51 },
    { 0, 0 },
    { 0x3242a987, 98 },
    { 0xd9f2a988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a82a98b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2a98e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472a99a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f2a99d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f82a9a0, 76 },
    { 0xa782a9a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2a9a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d62a9ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7072a9bb, 61 },
    { 0x2f52a9bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6292a9c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f2a9cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d2a9d3, 33 },
    { 0x9152a9d4, 52 },
    { 0xeac2a9d4, 33 },
    { 0x8a02a9d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2a9d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe842a9dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd602a9e0, 33 },
    { 0x3d62a9e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf562a9e5, 61 },
    { 0x7cc2a9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992a9eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf432a9ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e72a9f6, 93 },
    { 0xe4c2a9f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d2a9fa, 84 },
    { 0, 0 },
    { 0xce82a9fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa282a9ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb2aa06, 43 },
    { 0, 0 },
    { 0xef32aa08, 76 },
    { 0x4972aa09, 38 },
    { 0xd222aa0a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd2aa0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a2aa12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2962aa15, 55 },
    { 0x6d52aa16, 98 },
    { 0xe0f2aa16, 95 },
    { 0, 0 },
    { 0xc762aa19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc2aa1d, 90 },
    { 0xb92aa1e, 90 },
    { 0xd782aa1d, 98 },
    { 0x8a02aa20, 88 },
    { 0x4672aa21, 55 },
    { 0x3802aa22, 46 },
    { 0x5fb2aa23, 98 },
    { 0xc352aa24, 52 },
    { 0, 0 },
    { 0x7442aa26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2aa2b, 55 },
    { 0x81e2aa2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca2aa33, 76 },
    { 0x76c2aa33, 43 },
    { 0x5e52aa35, 67 },
    { 0x2aa2aa35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2aa39, 95 },
    { 0x81f2aa3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2aa42, 98 },
    { 0x3da2aa43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2aa46, 85 },
    { 0x3122aa47, 90 },
    { 0x3912aa46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf92aa56, 76 },
    { 0x7be2aa57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf122aa5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d92aa5e, 76 },
    { 0, 0 },
    { 0xbc52aa60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f2aa63, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702aa67, 91 },
    { 0x1842aa68, 98 },
    { 0x31d2aa67, 52 },
    { 0x7382aa69, 99 },
    { 0, 0 },
    { 0xd602aa6c, 85 },
    { 0x6a22aa6d, 69 },
    { 0xb412aa6c, 23 },
    { 0x2fd2aa6f, 90 },
    { 0, 0 },
    { 0x9e52aa71, 84 },
    { 0x8dc2aa72, 61 },
    { 0x9722aa73, 61 },
    { 0x1ab2aa71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde92aa77, 23 },
    { 0, 0 },
    { 0x8b72aa79, 61 },
    { 0x6572aa7a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2aa81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802aa8c, 86 },
    { 0x80c2aa8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4842aa91, 33 },
    { 0xf3c2aa92, 61 },
    { 0, 0 },
    { 0xf5c2aa94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e82aa97, 95 },
    { 0x532aa98, 90 },
    { 0xc362aa99, 90 },
    { 0x1192aa9a, 23 },
    { 0x6b12aa9a, 76 },
    { 0x7a22aa9c, 61 },
    { 0x6e42aa9d, 90 },
    { 0x8562aa9b, 77 },
    { 0x6be2aa9f, 69 },
    { 0x7322aaa0, 23 },
    { 0xee82aa98, 33 },
    { 0xd3c2aaa2, 90 },
    { 0xbb02aaa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2aaa6, 61 },
    { 0x8482aaa7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd142aaab, 66 },
    { 0, 0 },
    { 0xa412aaad, 98 },
    { 0x72d2aaad, 98 },
    { 0x2362aaaf, 98 },
    { 0xf0e2aab0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c22aab3, 34 },
    { 0x6342aab4, 75 },
    { 0x2632aab4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc42aab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a22aabc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea2aabf, 61 },
    { 0, 0 },
    { 0x7a2aac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f2aac4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5822aac8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c2aacc, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842aad1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2aad5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8282aad8, 67 },
    { 0x71d2aad8, 23 },
    { 0, 0 },
    { 0xc662aadb, 95 },
    { 0, 0 },
    { 0x1962aadd, 46 },
    { 0, 0 },
    { 0x15a2aadf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a62aae2, 95 },
    { 0xe492aae2, 61 },
    { 0, 0 },
    { 0xa452aae5, 43 },
    { 0x1b2aae6, 61 },
    { 0x4bc2aae7, 98 },
    { 0xbeb2aae5, 34 },
    { 0xbae2aae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a2aaed, 33 },
    { 0, 0 },
    { 0x83b2aaef, 33 },
    { 0x9832aaf0, 55 },
    { 0, 0 },
    { 0xf752aaf2, 33 },
    { 0x7ec2aaf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d2aaf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8142aafb, 52 },
    { 0x332aafc, 61 },
    { 0x9d02aafd, 93 },
    { 0x4fc2aafd, 19 },
    { 0, 0 },
    { 0x7362ab00, 61 },
    { 0x522ab01, 34 },
    { 0xdb62ab02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d92ab08, 61 },
    { 0x7182ab09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2ab16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792ab1b, 61 },
    { 0xb932ab1b, 55 },
    { 0, 0 },
    { 0x33c2ab1e, 98 },
    { 0, 0 },
    { 0x9a02ab20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc62ab25, 34 },
    { 0xf692ab25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82ab2d, 61 },
    { 0x8c62ab2e, 90 },
    { 0, 0 },
    { 0xa912ab30, 34 },
    { 0x2872ab31, 33 },
    { 0xe202ab31, 90 },
    { 0x6b22ab33, 76 },
    { 0, 0 },
    { 0x6222ab35, 61 },
    { 0, 0 },
    { 0x172ab37, 61 },
    { 0x62b2ab37, 33 },
    { 0x1d2ab37, 33 },
    { 0x2f82ab3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a2ab42, 91 },
    { 0x952ab43, 99 },
    { 0, 0 },
    { 0xbee2ab45, 93 },
    { 0xcb92ab45, 76 },
    { 0, 0 },
    { 0x7252ab48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ab4f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c2ab52, 43 },
    { 0, 0 },
    { 0x1652ab54, 23 },
    { 0, 0 },
    { 0xc942ab56, 98 },
    { 0xccd2ab57, 23 },
    { 0xfe42ab58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6242ab5c, 90 },
    { 0, 0 },
    { 0x77f2ab5e, 33 },
    { 0xab92ab5e, 90 },
    { 0x29a2ab60, 90 },
    { 0x9222ab61, 61 },
    { 0, 0 },
    { 0x1d52ab63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2ab67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2ab6f, 90 },
    { 0x7fe2ab70, 43 },
    { 0xe22ab71, 77 },
    { 0x2fe2ab70, 43 },
    { 0xc502ab70, 93 },
    { 0xf312ab71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d2ab78, 95 },
    { 0xb342ab78, 98 },
    { 0, 0 },
    { 0xb452ab7b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a22ab7e, 33 },
    { 0x292ab7e, 33 },
    { 0xf922ab80, 95 },
    { 0x2002ab81, 67 },
    { 0xd522ab82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde2ab87, 33 },
    { 0x3892ab88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2ab8f, 55 },
    { 0x49d2ab90, 95 },
    { 0, 0 },
    { 0x4b52ab92, 55 },
    { 0, 0 },
    { 0xbe52ab94, 78 },
    { 0x96b2ab94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde22ab9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c52ab9e, 76 },
    { 0, 0 },
    { 0xaee2aba0, 55 },
    { 0x3472aba1, 91 },
    { 0, 0 },
    { 0x56b2aba3, 90 },
    { 0x3182aba3, 90 },
    { 0x6812aba5, 33 },
    { 0x7542aba5, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb32aba9, 90 },
    { 0x8a12aba9, 76 },
    { 0x3b02aba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a2abae, 90 },
    { 0, 0 },
    { 0xee82abb0, 66 },
    { 0x8e62abb1, 95 },
    { 0x14c2abb1, 33 },
    { 0xe4c2abb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2abb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb302abba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252abbd, 91 },
    { 0x3e02abbe, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b2abc1, 61 },
    { 0x4222abc2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e2abc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4252abcd, 33 },
    { 0, 0 },
    { 0x7302abcf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2abd4, 90 },
    { 0, 0 },
    { 0xa2abd6, 33 },
    { 0, 0 },
    { 0xee82abd8, 85 },
    { 0, 0 },
    { 0x5602abda, 34 },
    { 0x2002abda, 55 },
    { 0xfb12abdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c2abdf, 95 },
    { 0x1e62abe0, 33 },
    { 0, 0 },
    { 0xb3b2abe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7752abe7, 66 },
    { 0x76d2abe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d2abec, 90 },
    { 0x3822abed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902abf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c2abf4, 76 },
    { 0x38e2abf5, 34 },
    { 0x6ed2abf6, 85 },
    { 0xc1e2abf7, 77 },
    { 0, 0 },
    { 0xddb2abf9, 90 },
    { 0xbd52abf9, 100 },
    { 0x6e72abfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa362ac03, 90 },
    { 0, 0 },
    { 0x77b2ac05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552ac0d, 90 },
    { 0, 0 },
    { 0xf8d2ac0f, 90 },
    { 0xac32ac10, 87 },
    { 0, 0 },
    { 0x7c32ac12, 36 },
    { 0x91b2ac12, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2ac16, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9382ac1c, 33 },
    { 0, 0 },
    { 0x8202ac1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5372ac25, 76 },
    { 0xf062ac25, 90 },
    { 0, 0 },
    { 0x8af2ac28, 33 },
    { 0, 0 },
    { 0xa232ac2a, 55 },
    { 0, 0 },
    { 0xbb92ac2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f02ac32, 23 },
    { 0, 0 },
    { 0xa352ac34, 23 },
    { 0, 0 },
    { 0xa532ac36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb102ac3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f2ac3e, 90 },
    { 0, 0 },
    { 0x76b2ac40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d82ac45, 40 },
    { 0x79d2ac46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92ac4a, 90 },
    { 0x1392ac4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1542ac50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d52ac5a, 90 },
    { 0x8862ac5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b52ac5e, 61 },
    { 0, 0 },
    { 0xd5a2ac60, 33 },
    { 0, 0 },
    { 0x9e92ac62, 95 },
    { 0xc1c2ac62, 85 },
    { 0xb342ac64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2ac68, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2ac75, 99 },
    { 0x19a2ac75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c2ac7b, 87 },
    { 0x5812ac7c, 95 },
    { 0x6c82ac7d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d62ac82, 33 },
    { 0x3612ac82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe132ac86, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42ac8b, 77 },
    { 0, 0 },
    { 0x50e2ac8d, 98 },
    { 0, 0 },
    { 0xebc2ac8f, 78 },
    { 0x8002ac8f, 76 },
    { 0x3292ac91, 76 },
    { 0x9da2ac92, 88 },
    { 0x6772ac93, 51 },
    { 0x2902ac8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2362ac97, 76 },
    { 0xe022ac97, 65 },
    { 0, 0 },
    { 0xf5e2ac9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde2aca6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc32acaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc132acb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d2acb9, 43 },
    { 0x92e2acba, 76 },
    { 0x6b32acbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e2acbe, 34 },
    { 0xf202acbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572acc2, 65 },
    { 0x5f2acc2, 23 },
    { 0x8ad2acc4, 33 },
    { 0x82acc5, 33 },
    { 0x57b2acc4, 43 },
    { 0xa362acc6, 23 },
    { 0x46b2acc7, 76 },
    { 0x3282acc3, 90 },
    { 0, 0 },
    { 0x5152accb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc32accf, 43 },
    { 0x4672acd0, 66 },
    { 0xe632acd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a2acd4, 33 },
    { 0x17e2acd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e2acda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef2acdd, 61 },
    { 0xe532acde, 61 },
    { 0xf6a2acdf, 61 },
    { 0xdc62acde, 98 },
    { 0x4ed2ace1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b2ace4, 95 },
    { 0xf62ace5, 43 },
    { 0, 0 },
    { 0x6802ace7, 23 },
    { 0x892ace8, 55 },
    { 0xb7c2ace8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182acf1, 81 },
    { 0xa422acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2acfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac2ad04, 84 },
    { 0xc6f2ad05, 62 },
    { 0x7e02ad06, 33 },
    { 0, 0 },
    { 0x2722ad08, 66 },
    { 0x5b62ad09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2ad0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a62ad14, 55 },
    { 0x1052ad15, 95 },
    { 0xd592ad16, 61 },
    { 0, 0 },
    { 0xef22ad18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2ad22, 95 },
    { 0, 0 },
    { 0x1632ad24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1552ad29, 98 },
    { 0xb682ad2a, 90 },
    { 0, 0 },
    { 0xa332ad2c, 90 },
    { 0xe022ad2d, 98 },
    { 0x1262ad2e, 67 },
    { 0x9be2ad2f, 90 },
    { 0, 0 },
    { 0x802ad31, 98 },
    { 0xb292ad32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382ad36, 55 },
    { 0xf2d2ad37, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2ad3c, 76 },
    { 0, 0 },
    { 0x5eb2ad3e, 77 },
    { 0x4db2ad3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6182ad47, 52 },
    { 0xdb82ad48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e2ad4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f82ad50, 34 },
    { 0x2642ad51, 33 },
    { 0x2032ad50, 46 },
    { 0x27c2ad53, 23 },
    { 0xf982ad54, 100 },
    { 0xaf02ad55, 67 },
    { 0x5902ad55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f2ad5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322ad61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed2ad67, 43 },
    { 0x6b22ad67, 33 },
    { 0xe932ad69, 67 },
    { 0xc632ad6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4352ad6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d2ad74, 91 },
    { 0x882ad74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8232ad84, 90 },
    { 0x4cd2ad85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2ad8a, 99 },
    { 0, 0 },
    { 0x4e02ad8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d82ad8f, 99 },
    { 0x8782ad90, 77 },
    { 0x1422ad91, 68 },
    { 0x4992ad90, 34 },
    { 0, 0 },
    { 0xac22ad94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692ad9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe202ada2, 33 },
    { 0xbdd2ada3, 98 },
    { 0, 0 },
    { 0x2c12ada5, 61 },
    { 0x5ba2ada6, 95 },
    { 0, 0 },
    { 0xde12ada8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5002adac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92adb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d02adb5, 61 },
    { 0xd752adb5, 95 },
    { 0, 0 },
    { 0x94c2adb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22adbb, 98 },
    { 0x9fd2adbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef2adc5, 76 },
    { 0x6232adc5, 98 },
    { 0xe772adc7, 61 },
    { 0xce72adc7, 95 },
    { 0x6742adc9, 95 },
    { 0, 0 },
    { 0xb802adcb, 66 },
    { 0, 0 },
    { 0x3e2adcd, 90 },
    { 0xa8a2adce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2add3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa02add6, 76 },
    { 0xccc2add6, 55 },
    { 0x5012add8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa672addb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea52addf, 90 },
    { 0xc592ade0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1782ade5, 95 },
    { 0x7572ade5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a2ade9, 33 },
    { 0xe482ade9, 93 },
    { 0, 0 },
    { 0xc172adec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9402adef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42adf2, 60 },
    { 0xf602adf3, 33 },
    { 0x66b2adf4, 76 },
    { 0xa912adf3, 84 },
    { 0xa8f2adf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc672adf9, 61 },
    { 0, 0 },
    { 0x7ba2adfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6662ae01, 90 },
    { 0, 0 },
    { 0x7562ae03, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482ae0d, 95 },
    { 0x3ee2ae0e, 64 },
    { 0x7a92ae0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d2ae13, 90 },
    { 0x5ea2ae14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x532ae17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b02ae1b, 99 },
    { 0, 0 },
    { 0x7cc2ae1d, 33 },
    { 0xdc82ae1d, 90 },
    { 0x5ef2ae1f, 55 },
    { 0, 0 },
    { 0xeb42ae21, 43 },
    { 0xd5a2ae21, 61 },
    { 0xe0f2ae21, 43 },
    { 0, 0 },
    { 0xdd82ae25, 98 },
    { 0, 0 },
    { 0xf042ae27, 55 },
    { 0, 0 },
    { 0x83c2ae29, 23 },
    { 0, 0 },
    { 0x7962ae2b, 77 },
    { 0xa842ae2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642ae31, 78 },
    { 0xfda2ae31, 46 },
    { 0, 0 },
    { 0xc492ae34, 61 },
    { 0, 0 },
    { 0x83d2ae36, 90 },
    { 0, 0 },
    { 0x3662ae38, 52 },
    { 0xd812ae39, 61 },
    { 0x64e2ae39, 23 },
    { 0x3152ae3b, 55 },
    { 0x9b2ae3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e52ae42, 87 },
    { 0x8bd2ae43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ae4e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac2ae53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42ae56, 90 },
    { 0x1cf2ae57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa822ae5a, 34 },
    { 0, 0 },
    { 0x8ad2ae5c, 90 },
    { 0x9432ae5c, 95 },
    { 0x6d52ae5c, 90 },
    { 0xfab2ae5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5002ae68, 76 },
    { 0x66c2ae68, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a2ae70, 84 },
    { 0xa792ae70, 56 },
    { 0x51d2ae72, 53 },
    { 0xb522ae73, 55 },
    { 0x98e2ae72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ae78, 76 },
    { 0xedc2ae78, 98 },
    { 0x6e42ae78, 95 },
    { 0, 0 },
    { 0x3cc2ae7c, 90 },
    { 0xb612ae7c, 52 },
    { 0xba22ae7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832ae81, 85 },
    { 0x21c2ae82, 95 },
    { 0x54e2ae81, 66 },
    { 0xf652ae84, 90 },
    { 0x63a2ae83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c2ae88, 55 },
    { 0xc122ae89, 33 },
    { 0x3a42ae8a, 55 },
    { 0x2482ae8a, 61 },
    { 0x4ae2ae8c, 95 },
    { 0xd262ae8c, 61 },
    { 0x1c32ae8e, 99 },
    { 0x86b2ae8e, 99 },
    { 0x762ae90, 76 },
    { 0xa432ae90, 98 },
    { 0x6702ae92, 85 },
    { 0x3902ae93, 23 },
    { 0x4fd2ae93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc2ae9e, 40 },
    { 0x27d2ae9e, 33 },
    { 0x3b22aea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb232aea7, 90 },
    { 0x2a02aea8, 33 },
    { 0x3e82aea9, 81 },
    { 0, 0 },
    { 0x69d2aeab, 93 },
    { 0, 0 },
    { 0x7952aead, 23 },
    { 0, 0 },
    { 0x1bf2aeaf, 80 },
    { 0, 0 },
    { 0x882aeb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee72aeb4, 78 },
    { 0x8d2aeb5, 81 },
    { 0x8612aeb4, 23 },
    { 0xe982aeb4, 61 },
    { 0xd232aeb4, 34 },
    { 0xdff2aeb8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6952aebd, 100 },
    { 0xd6d2aebd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa062aec7, 67 },
    { 0x23a2aec7, 78 },
    { 0xb282aec9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2aecd, 61 },
    { 0xf62aece, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3112aed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2aed5, 95 },
    { 0, 0 },
    { 0xf6a2aed7, 100 },
    { 0x3f62aed8, 61 },
    { 0x95c2aed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22aee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f2aeeb, 23 },
    { 0x6122aeec, 90 },
    { 0x3982aeec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2aef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec32aefe, 33 },
    { 0x68d2aefe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6722af05, 36 },
    { 0, 0 },
    { 0xf432af07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2af0a, 59 },
    { 0, 0 },
    { 0x2712af0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52af11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf322af16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd2af1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b2af20, 95 },
    { 0, 0 },
    { 0xb702af22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c2af26, 90 },
    { 0xad42af26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d2af33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2af36, 76 },
    { 0x4462af37, 99 },
    { 0, 0 },
    { 0x4ee2af39, 33 },
    { 0x4ae2af3a, 61 },
    { 0xa652af3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec82af3f, 90 },
    { 0xdd82af3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x492af43, 90 },
    { 0xeeb2af44, 61 },
    { 0xb222af45, 98 },
    { 0, 0 },
    { 0x4112af47, 69 },
    { 0, 0 },
    { 0xda72af49, 61 },
    { 0xce2af4a, 61 },
    { 0x82af4a, 90 },
    { 0x9942af4c, 33 },
    { 0x7392af4d, 61 },
    { 0xcc02af4d, 95 },
    { 0xc562af4f, 55 },
    { 0x38f2af4e, 95 },
    { 0, 0 },
    { 0x9ae2af52, 84 },
    { 0xdec2af52, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02af59, 88 },
    { 0x6802af5a, 34 },
    { 0xf6a2af59, 76 },
    { 0x9392af5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9282af60, 90 },
    { 0xe182af61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2af67, 46 },
    { 0x1432af68, 88 },
    { 0, 0 },
    { 0x9042af6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde2af71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc2af75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a32af7a, 23 },
    { 0xa0a2af7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2202af7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb2af87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b82af8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b02af90, 33 },
    { 0x39f2af91, 84 },
    { 0xa5e2af92, 90 },
    { 0, 0 },
    { 0x752af94, 61 },
    { 0x69b2af95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02af9b, 34 },
    { 0, 0 },
    { 0xec42af9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2afa0, 78 },
    { 0, 0 },
    { 0xded2afa2, 66 },
    { 0, 0 },
    { 0x77b2afa4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb2afa7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd92afac, 61 },
    { 0x4662afad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2afb1, 33 },
    { 0x91d2afb2, 95 },
    { 0x2d22afb1, 98 },
    { 0x83d2afb4, 76 },
    { 0, 0 },
    { 0xb682afb6, 85 },
    { 0xf092afb7, 95 },
    { 0x1e02afb8, 46 },
    { 0xd572afb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e32afbc, 34 },
    { 0xf9f2afbc, 69 },
    { 0, 0 },
    { 0xc592afbf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2afc2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a82afc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd392afcb, 95 },
    { 0x8872afcc, 34 },
    { 0x7422afcd, 61 },
    { 0, 0 },
    { 0xff62afcf, 43 },
    { 0, 0 },
    { 0xdf12afd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b2afd4, 23 },
    { 0, 0 },
    { 0xcfe2afd6, 40 },
    { 0xa062afd7, 90 },
    { 0x932afd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b2afdd, 90 },
    { 0xc12afde, 90 },
    { 0x4ac2afdf, 52 },
    { 0x9602afde, 43 },
    { 0, 0 },
    { 0x5b42afe2, 76 },
    { 0, 0 },
    { 0xdb52afe4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e42afed, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc2aff0, 87 },
    { 0x58d2aff1, 33 },
    { 0xc2a2aff1, 43 },
    { 0x8332aff1, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2aff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd742affa, 91 },
    { 0, 0 },
    { 0xe632affc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e02b001, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172b008, 23 },
    { 0x20c2b008, 61 },
    { 0, 0 },
    { 0x7542b00b, 90 },
    { 0, 0 },
    { 0xd812b00d, 61 },
    { 0xd482b00d, 55 },
    { 0x3772b00f, 96 },
    { 0x7442b00d, 95 },
    { 0, 0 },
    { 0xe612b012, 76 },
    { 0x1c22b012, 90 },
    { 0x3e92b013, 43 },
    { 0xbee2b015, 61 },
    { 0, 0 },
    { 0xb422b017, 93 },
    { 0xad52b018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb32b01b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e2b01e, 95 },
    { 0x222b01e, 95 },
    { 0, 0 },
    { 0x6a12b021, 90 },
    { 0, 0 },
    { 0x59a2b023, 61 },
    { 0x6502b024, 33 },
    { 0xf252b024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e2b02d, 100 },
    { 0xc2e2b02d, 61 },
    { 0xcf42b02f, 61 },
    { 0x5542b030, 95 },
    { 0x7b72b030, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf92b036, 85 },
    { 0, 0 },
    { 0xa702b038, 87 },
    { 0x5002b038, 34 },
    { 0, 0 },
    { 0x6662b03b, 88 },
    { 0, 0 },
    { 0x5002b03d, 33 },
    { 0, 0 },
    { 0xca52b03f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2b044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2b04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9012b04e, 61 },
    { 0x1ff2b04f, 90 },
    { 0x5e52b050, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea2b059, 67 },
    { 0, 0 },
    { 0xdeb2b05b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb12b05e, 55 },
    { 0, 0 },
    { 0x8062b060, 83 },
    { 0xc572b060, 96 },
    { 0x7652b062, 98 },
    { 0x7a22b063, 46 },
    { 0x9332b062, 90 },
    { 0xea72b065, 34 },
    { 0x7202b066, 91 },
    { 0xcff2b067, 61 },
    { 0xf122b065, 33 },
    { 0xfcf2b067, 33 },
    { 0xf0e2b06a, 95 },
    { 0x6502b06b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c02b072, 90 },
    { 0, 0 },
    { 0xab22b074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c2b077, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b02b07c, 90 },
    { 0x4772b07d, 90 },
    { 0, 0 },
    { 0x7442b07f, 46 },
    { 0, 0 },
    { 0xaa42b081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9792b085, 90 },
    { 0x9602b085, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a2b089, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be2b08d, 90 },
    { 0, 0 },
    { 0x7e62b08f, 23 },
    { 0x7e02b08f, 23 },
    { 0x7652b091, 90 },
    { 0xe172b092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f2b095, 55 },
    { 0xae02b096, 61 },
    { 0x17c2b095, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72b09a, 55 },
    { 0x5612b09a, 43 },
    { 0x9ae2b09b, 95 },
    { 0x9432b09b, 68 },
    { 0x7fa2b09e, 98 },
    { 0xdec2b09f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252b0a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472b0a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5242b0ac, 81 },
    { 0x8c52b0ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc502b0b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d2b0b6, 34 },
    { 0xf422b0b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea82b0ba, 99 },
    { 0, 0 },
    { 0xe1b2b0bc, 81 },
    { 0, 0 },
    { 0x6fd2b0be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf52b0c1, 23 },
    { 0x1a62b0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd692b0cf, 90 },
    { 0, 0 },
    { 0x88f2b0d1, 61 },
    { 0x2822b0d1, 40 },
    { 0x7bd2b0d1, 98 },
    { 0, 0 },
    { 0x4282b0d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5902b0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e2b0de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f22b0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb572b0ee, 40 },
    { 0xcfc2b0ef, 98 },
    { 0x9292b0f0, 98 },
    { 0x62c2b0ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee2b0f4, 88 },
    { 0, 0 },
    { 0xe82b0f6, 76 },
    { 0, 0 },
    { 0x2882b0f8, 98 },
    { 0x3c62b0f8, 90 },
    { 0xdac2b0fa, 23 },
    { 0xd12b0fa, 95 },
    { 0, 0 },
    { 0xe292b0fd, 95 },
    { 0xdc92b0fe, 90 },
    { 0x2fb2b0fe, 95 },
    { 0x1792b100, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb42b106, 90 },
    { 0xbc42b107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc72b10c, 68 },
    { 0x16e2b10d, 17 },
    { 0, 0 },
    { 0x13c2b10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a02b116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2b11a, 90 },
    { 0, 0 },
    { 0x8422b11c, 78 },
    { 0, 0 },
    { 0x97a2b11e, 66 },
    { 0, 0 },
    { 0x592b120, 99 },
    { 0, 0 },
    { 0x7872b122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782b125, 68 },
    { 0x6fa2b126, 33 },
    { 0x4d62b127, 46 },
    { 0xe092b128, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692b12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b2b134, 93 },
    { 0x2782b135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c2b139, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7172b13d, 98 },
    { 0x5892b13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7922b142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2b147, 93 },
    { 0xc022b148, 33 },
    { 0x9692b147, 95 },
    { 0x4db2b14a, 85 },
    { 0x1d12b148, 66 },
    { 0x8d32b14a, 76 },
    { 0, 0 },
    { 0x2332b14e, 43 },
    { 0xbe32b14e, 34 },
    { 0, 0 },
    { 0x1232b151, 34 },
    { 0, 0 },
    { 0x4322b153, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c52b156, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd62b15e, 33 },
    { 0x2fe2b15e, 98 },
    { 0, 0 },
    { 0x6c92b161, 85 },
    { 0x4de2b161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f2b169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec2b16c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922b170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc162b175, 76 },
    { 0, 0 },
    { 0x3c12b177, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd832b17b, 90 },
    { 0x482b17b, 79 },
    { 0x7392b17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7712b182, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2b187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe022b18e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c2b191, 61 },
    { 0x96f2b192, 90 },
    { 0xad42b193, 55 },
    { 0, 0 },
    { 0x2012b195, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2b199, 90 },
    { 0x3b22b19a, 33 },
    { 0x6132b19b, 100 },
    { 0x5eb2b19b, 93 },
    { 0x4e22b19b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2b1a0, 78 },
    { 0x3142b1a1, 34 },
    { 0x6522b1a2, 61 },
    { 0xee22b1a2, 34 },
    { 0, 0 },
    { 0xd362b1a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f2b1ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde22b1ae, 99 },
    { 0x66e2b1af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c2b1b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe82b1b8, 99 },
    { 0xa462b1b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d2b1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042b1c0, 75 },
    { 0xb5c2b1c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1312b1c6, 81 },
    { 0, 0 },
    { 0x9522b1c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc2b1cc, 100 },
    { 0, 0 },
    { 0x4932b1ce, 34 },
    { 0xa092b1cf, 33 },
    { 0x92b1d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3252b1db, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5142b1df, 43 },
    { 0xb22b1e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2b1f2, 98 },
    { 0x4fb2b1f3, 61 },
    { 0x8f82b1f3, 90 },
    { 0x86b2b1f5, 98 },
    { 0xffb2b1f6, 85 },
    { 0x98d2b1f3, 31 },
    { 0x2432b1f6, 33 },
    { 0x1d72b1f9, 34 },
    { 0x5c2b1fa, 34 },
    { 0xc5b2b1f8, 34 },
    { 0x66b2b1f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2742b1ff, 55 },
    { 0, 0 },
    { 0x3072b201, 33 },
    { 0xef02b201, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c72b20d, 34 },
    { 0x6952b20d, 90 },
    { 0xa2d2b20f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6572b213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b218, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072b220, 98 },
    { 0, 0 },
    { 0x90a2b222, 98 },
    { 0x8ee2b222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a2b22a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b62b22d, 44 },
    { 0xda82b22e, 100 },
    { 0xc482b22f, 61 },
    { 0xb3b2b22e, 90 },
    { 0xb222b22f, 23 },
    { 0x7412b231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf2b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2b239, 88 },
    { 0, 0 },
    { 0xe642b23b, 98 },
    { 0xb602b23c, 55 },
    { 0x3922b23d, 93 },
    { 0x9422b23e, 23 },
    { 0x8b32b23e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5612b243, 90 },
    { 0, 0 },
    { 0xc382b245, 43 },
    { 0, 0 },
    { 0x4a62b247, 34 },
    { 0x2632b248, 93 },
    { 0, 0 },
    { 0x7ad2b24a, 95 },
    { 0, 0 },
    { 0xe462b24c, 33 },
    { 0x19d2b24c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed92b250, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6092b253, 33 },
    { 0xf682b253, 55 },
    { 0x6d92b254, 78 },
    { 0x77d2b256, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b25c, 88 },
    { 0xa232b25c, 90 },
    { 0xc4e2b25d, 98 },
    { 0, 0 },
    { 0xfdf2b260, 66 },
    { 0x3fe2b261, 33 },
    { 0x8232b260, 67 },
    { 0x98d2b263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2b266, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef2b26b, 43 },
    { 0xd9c2b26c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122b275, 100 },
    { 0x6e62b276, 61 },
    { 0xa9c2b277, 43 },
    { 0x8ae2b278, 55 },
    { 0xfe32b277, 33 },
    { 0x28d2b276, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe942b27d, 77 },
    { 0xd8b2b27e, 90 },
    { 0x4fb2b27f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6062b283, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xda32b286, 52 },
    { 0x44d2b286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362b28a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2b28d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02b291, 61 },
    { 0x2482b292, 95 },
    { 0x4e62b293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa472b298, 90 },
    { 0x6932b298, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322b29d, 98 },
    { 0xb9e2b29e, 81 },
    { 0x5f42b29f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a2b2a2, 91 },
    { 0x7142b2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe802b2ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf752b2b4, 46 },
    { 0xf472b2b5, 34 },
    { 0x6602b2b6, 95 },
    { 0, 0 },
    { 0xd7b2b2b8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa972b2bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd642b2c3, 93 },
    { 0xc312b2c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd42b2ca, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc2b2ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd2b2d5, 90 },
    { 0, 0 },
    { 0xe062b2d7, 55 },
    { 0xe5c2b2d8, 61 },
    { 0xbe32b2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb272b2e3, 33 },
    { 0, 0 },
    { 0xad52b2e5, 61 },
    { 0, 0 },
    { 0x6162b2e7, 66 },
    { 0xc502b2e7, 66 },
    { 0x2722b2e7, 33 },
    { 0x6922b2ea, 90 },
    { 0, 0 },
    { 0x6832b2ec, 61 },
    { 0, 0 },
    { 0x3b42b2ee, 61 },
    { 0, 0 },
    { 0x7b02b2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2b2f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca2b2fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02b2fd, 98 },
    { 0xcd62b2fe, 17 },
    { 0xb812b2ff, 95 },
    { 0, 0 },
    { 0xebc2b301, 85 },
    { 0, 0 },
    { 0x99d2b303, 100 },
    { 0, 0 },
    { 0x9272b305, 61 },
    { 0xece2b305, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2b30b, 75 },
    { 0, 0 },
    { 0x5a12b30d, 93 },
    { 0x9f2b30e, 43 },
    { 0x35e2b30f, 99 },
    { 0x102b310, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2b31a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf782b31d, 33 },
    { 0x442b31e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf682b321, 95 },
    { 0xd442b322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152b327, 40 },
    { 0x3362b328, 46 },
    { 0xcab2b327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd52b335, 90 },
    { 0xa6b2b336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2b339, 17 },
    { 0, 0 },
    { 0xd992b33b, 90 },
    { 0, 0 },
    { 0x5502b33d, 66 },
    { 0xa1b2b33d, 98 },
    { 0, 0 },
    { 0xd452b340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a12b343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3892b346, 90 },
    { 0xf532b347, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x562b34a, 33 },
    { 0, 0 },
    { 0xcf32b34c, 43 },
    { 0x3fe2b34d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72b352, 61 },
    { 0, 0 },
    { 0x3752b354, 90 },
    { 0x6c42b355, 88 },
    { 0x1112b356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a82b35b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd62b35e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452b363, 95 },
    { 0x17e2b364, 23 },
    { 0, 0 },
    { 0x8c22b366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc202b369, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba72b36f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa152b374, 33 },
    { 0x19a2b375, 98 },
    { 0x8b32b376, 90 },
    { 0x9152b374, 23 },
    { 0xfe22b376, 83 },
    { 0xced2b379, 85 },
    { 0x1d2b378, 98 },
    { 0x2b2b375, 52 },
    { 0xf052b37c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2b382, 95 },
    { 0x16b2b382, 93 },
    { 0, 0 },
    { 0x53a2b385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac2b388, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92b38b, 49 },
    { 0x72d2b38c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382b391, 33 },
    { 0, 0 },
    { 0x7a2b393, 61 },
    { 0x9692b394, 81 },
    { 0, 0 },
    { 0x942b396, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x292b399, 52 },
    { 0xdd62b39a, 17 },
    { 0x6c22b39a, 69 },
    { 0x9ff2b39c, 23 },
    { 0x7252b39d, 55 },
    { 0, 0 },
    { 0xdae2b39f, 90 },
    { 0x88b2b3a0, 90 },
    { 0x8612b3a1, 61 },
    { 0x2502b3a2, 90 },
    { 0x1d42b3a3, 23 },
    { 0, 0 },
    { 0x6c72b3a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb12b3a8, 88 },
    { 0, 0 },
    { 0xdc92b3aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd22b3ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d2b3b0, 68 },
    { 0xea62b3b1, 95 },
    { 0, 0 },
    { 0x7b82b3b3, 93 },
    { 0, 0 },
    { 0xf8b2b3b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e12b3c1, 66 },
    { 0x9582b3c2, 90 },
    { 0x6452b3c3, 95 },
    { 0x3b32b3c3, 78 },
    { 0xe282b3c2, 23 },
    { 0, 0 },
    { 0x3462b3c7, 33 },
    { 0xbc72b3c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c2b3cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172b3d0, 95 },
    { 0xcfe2b3d1, 93 },
    { 0x3a22b3d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e32b3d5, 88 },
    { 0, 0 },
    { 0x8702b3d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2b3de, 98 },
    { 0, 0 },
    { 0x182b3e0, 93 },
    { 0xa672b3e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782b3e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1332b3ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1812b3ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e72b3f0, 23 },
    { 0x5b22b3f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe562b3f4, 93 },
    { 0x2432b3f4, 33 },
    { 0x8462b3f6, 64 },
    { 0xae42b3f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2b3fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c2b3fd, 61 },
    { 0xfa02b3fe, 80 },
    { 0x7302b3ff, 17 },
    { 0x412b3fd, 43 },
    { 0, 0 },
    { 0x4b72b402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2b40a, 90 },
    { 0x68e2b40b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32b412, 23 },
    { 0x6442b413, 33 },
    { 0x7a2b414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be2b418, 23 },
    { 0x89a2b419, 76 },
    { 0xda02b41a, 46 },
    { 0x6ea2b41b, 76 },
    { 0x8a42b419, 61 },
    { 0xe602b41c, 90 },
    { 0, 0 },
    { 0xe8e2b41f, 55 },
    { 0, 0 },
    { 0x91f2b421, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc872b427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452b42a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f2b42e, 45 },
    { 0, 0 },
    { 0x8e42b430, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf02b433, 93 },
    { 0x3472b434, 91 },
    { 0xbe22b435, 100 },
    { 0x18e2b435, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b22b43e, 61 },
    { 0x2982b43f, 23 },
    { 0xd5e2b43e, 90 },
    { 0x5f92b441, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe082b444, 46 },
    { 0x212b444, 33 },
    { 0, 0 },
    { 0x2552b447, 36 },
    { 0xc432b447, 23 },
    { 0x9542b448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e42b44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2b451, 33 },
    { 0, 0 },
    { 0xa402b453, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2b456, 85 },
    { 0xf772b456, 23 },
    { 0, 0 },
    { 0xcf2b459, 95 },
    { 0x62b2b45a, 66 },
    { 0x2092b45a, 33 },
    { 0x19d2b45b, 76 },
    { 0xd412b45c, 95 },
    { 0, 0 },
    { 0xf4a2b45f, 99 },
    { 0, 0 },
    { 0xe42b461, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2092b466, 61 },
    { 0x7f12b466, 87 },
    { 0x7602b468, 93 },
    { 0xb372b469, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2b46e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292b471, 84 },
    { 0, 0 },
    { 0x8462b473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392b477, 61 },
    { 0x31d2b477, 61 },
    { 0, 0 },
    { 0xcd32b47a, 93 },
    { 0x8b52b47b, 34 },
    { 0, 0 },
    { 0xb0d2b47d, 98 },
    { 0x35f2b47e, 36 },
    { 0x70d2b47f, 87 },
    { 0x6642b480, 34 },
    { 0xd502b481, 33 },
    { 0x1712b482, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce02b485, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c2b48a, 46 },
    { 0x742b48b, 46 },
    { 0x2d62b48c, 52 },
    { 0x9112b48b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5092b492, 23 },
    { 0x3c82b492, 95 },
    { 0xddd2b492, 90 },
    { 0xca02b495, 65 },
    { 0x68f2b495, 95 },
    { 0, 0 },
    { 0x7802b498, 98 },
    { 0x2a02b499, 84 },
    { 0x2232b499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2b49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef2b4a1, 95 },
    { 0xa992b4a2, 93 },
    { 0, 0 },
    { 0x4a92b4a4, 100 },
    { 0x8492b4a5, 33 },
    { 0x7042b4a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d2b4aa, 23 },
    { 0xc0b2b4ab, 61 },
    { 0x4492b4ab, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d2b4b0, 95 },
    { 0xb192b4b1, 75 },
    { 0x79e2b4b2, 17 },
    { 0x902b4b3, 90 },
    { 0, 0 },
    { 0x48f2b4b5, 55 },
    { 0, 0 },
    { 0x9c72b4b7, 95 },
    { 0, 0 },
    { 0xa022b4b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa2b4bd, 77 },
    { 0x36f2b4be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2b4c1, 23 },
    { 0, 0 },
    { 0x3bc2b4c3, 34 },
    { 0x6952b4c3, 91 },
    { 0x15b2b4c3, 33 },
    { 0x3372b4c3, 61 },
    { 0xf3f2b4c7, 66 },
    { 0, 0 },
    { 0xf562b4c9, 33 },
    { 0x5592b4ca, 55 },
    { 0, 0 },
    { 0x8712b4cc, 61 },
    { 0xad22b4cc, 98 },
    { 0x6702b4ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf052b4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422b4d7, 23 },
    { 0x5f2b4d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2b4dd, 23 },
    { 0xb932b4de, 87 },
    { 0, 0 },
    { 0x87b2b4e0, 17 },
    { 0, 0 },
    { 0xffc2b4e2, 93 },
    { 0x21e2b4e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc972b4ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9632b4f6, 55 },
    { 0x4412b4f7, 66 },
    { 0xab52b4f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b12b4fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf22b4ff, 61 },
    { 0xddf2b500, 68 },
    { 0x762b500, 33 },
    { 0, 0 },
    { 0x6692b503, 95 },
    { 0x62a2b503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2b507, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9782b50c, 33 },
    { 0x6bc2b50d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba2b513, 61 },
    { 0x34b2b513, 98 },
    { 0xeed2b513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d2b519, 23 },
    { 0x6262b51a, 81 },
    { 0xddd2b51b, 53 },
    { 0x4a22b51c, 95 },
    { 0xce22b51c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b520, 90 },
    { 0x59e2b520, 23 },
    { 0x21a2b521, 90 },
    { 0xdba2b523, 38 },
    { 0xa782b524, 90 },
    { 0xfc12b524, 68 },
    { 0, 0 },
    { 0x3902b527, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6632b52a, 33 },
    { 0, 0 },
    { 0x7492b52c, 23 },
    { 0x4402b52d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf2b531, 33 },
    { 0, 0 },
    { 0x5dd2b533, 67 },
    { 0xafb2b534, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b2b539, 95 },
    { 0xf432b53a, 55 },
    { 0x9852b53a, 61 },
    { 0x3312b53b, 98 },
    { 0x9082b53a, 90 },
    { 0xf752b53a, 90 },
    { 0xb952b53f, 55 },
    { 0x5e62b540, 33 },
    { 0x9af2b541, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf272b548, 55 },
    { 0xd3b2b549, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b2b54c, 34 },
    { 0, 0 },
    { 0xb032b54e, 23 },
    { 0, 0 },
    { 0x23d2b550, 61 },
    { 0x7062b550, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442b558, 33 },
    { 0, 0 },
    { 0xdad2b55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4852b55d, 90 },
    { 0x57b2b55e, 46 },
    { 0x2f22b55f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302b563, 90 },
    { 0x23a2b564, 61 },
    { 0xffb2b564, 90 },
    { 0, 0 },
    { 0x4d92b567, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f2b56b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe542b572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb932b578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512b57c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2b58b, 77 },
    { 0x76a2b58b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452b58f, 34 },
    { 0x4432b58f, 98 },
    { 0x8922b591, 90 },
    { 0x75d2b591, 4 },
    { 0x5022b593, 23 },
    { 0x152b58f, 90 },
    { 0xf0a2b590, 61 },
    { 0xfa92b596, 95 },
    { 0xc7b2b591, 78 },
    { 0x3392b598, 85 },
    { 0x8702b599, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2b59d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d72b5a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1022b5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b5aa, 33 },
    { 0, 0 },
    { 0xb0d2b5ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc072b5af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e62b5b4, 33 },
    { 0, 0 },
    { 0x3b02b5b6, 90 },
    { 0x19e2b5b7, 78 },
    { 0x5742b5b7, 23 },
    { 0x4232b5b9, 93 },
    { 0xa772b5b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2b5bd, 95 },
    { 0, 0 },
    { 0x55b2b5bf, 23 },
    { 0, 0 },
    { 0xe3b2b5c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8362b5c5, 87 },
    { 0x7262b5c6, 95 },
    { 0xea02b5c5, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba2b5cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9872b5cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7342b5d2, 33 },
    { 0x7a92b5d3, 33 },
    { 0xd752b5d4, 33 },
    { 0x4f2b5d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4592b5d8, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b92b5dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf062b5e8, 33 },
    { 0xf612b5e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7692b5ed, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22b5f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc2b5f6, 78 },
    { 0xd172b5f7, 90 },
    { 0x4102b5f6, 98 },
    { 0xbe72b5f6, 95 },
    { 0x51f2b5f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2b5fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c2b600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6252b605, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812b611, 95 },
    { 0, 0 },
    { 0x8342b613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd32b616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf072b61c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a2b620, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e42b623, 33 },
    { 0x3dd2b624, 33 },
    { 0, 0 },
    { 0x5a62b626, 99 },
    { 0, 0 },
    { 0x11b2b628, 43 },
    { 0x2f52b629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8022b62c, 95 },
    { 0xd7d2b62d, 43 },
    { 0x30c2b62e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7622b631, 33 },
    { 0x6092b632, 33 },
    { 0x47c2b632, 76 },
    { 0xa012b631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2b63a, 23 },
    { 0xd072b63b, 23 },
    { 0x9062b63b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312b640, 61 },
    { 0x36f2b641, 90 },
    { 0x132b640, 43 },
    { 0xb332b643, 98 },
    { 0x4e82b643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e2b648, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3772b64e, 95 },
    { 0, 0 },
    { 0xa912b650, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2b653, 90 },
    { 0, 0 },
    { 0xb042b655, 77 },
    { 0xf612b655, 91 },
    { 0xeb2b656, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412b65c, 34 },
    { 0x2712b65d, 81 },
    { 0xf2a2b65d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f62b663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752b667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7032b681, 90 },
    { 0x6d2b681, 61 },
    { 0, 0 },
    { 0xe22b684, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa572b68a, 90 },
    { 0x2af2b68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552b68f, 85 },
    { 0xe852b690, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42b693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552b696, 23 },
    { 0xb9f2b697, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8002b6a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432b6a4, 33 },
    { 0x1cd2b6a5, 68 },
    { 0, 0 },
    { 0xdd32b6a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d2b6ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32b6ae, 66 },
    { 0xb312b6af, 65 },
    { 0xe552b6af, 52 },
    { 0x9682b6b1, 91 },
    { 0x4dd2b6b2, 93 },
    { 0x482b6b3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb272b6be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a2b6c3, 91 },
    { 0x6902b6c4, 95 },
    { 0x86f2b6c5, 33 },
    { 0, 0 },
    { 0x1e12b6c7, 95 },
    { 0x4e02b6c7, 90 },
    { 0, 0 },
    { 0x5452b6ca, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2b6cd, 38 },
    { 0xb7b2b6ce, 34 },
    { 0, 0 },
    { 0xd402b6d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572b6d3, 90 },
    { 0xc552b6d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8772b6d8, 98 },
    { 0x4c62b6d9, 91 },
    { 0xdca2b6da, 100 },
    { 0xaaa2b6db, 52 },
    { 0, 0 },
    { 0xaf02b6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe212b6e4, 52 },
    { 0x2952b6e5, 43 },
    { 0x562b6e6, 95 },
    { 0x3dd2b6e7, 90 },
    { 0xd7f2b6e7, 76 },
    { 0xa622b6e6, 99 },
    { 0x6562b6ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a42b6ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2b6f1, 100 },
    { 0x2382b6f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd52b6f9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7932b6fd, 55 },
    { 0x8462b6fd, 98 },
    { 0x1cf2b6ff, 23 },
    { 0x9a22b700, 55 },
    { 0xd452b700, 98 },
    { 0, 0 },
    { 0xe472b703, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe052b707, 61 },
    { 0xd1b2b708, 90 },
    { 0x5e02b707, 78 },
    { 0x54e2b708, 95 },
    { 0x2312b709, 90 },
    { 0xb912b70c, 98 },
    { 0, 0 },
    { 0x49b2b70e, 61 },
    { 0xe782b70e, 95 },
    { 0x6c62b70e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7522b713, 55 },
    { 0x7842b714, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6742b719, 33 },
    { 0, 0 },
    { 0xeaf2b71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d2b720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb342b724, 61 },
    { 0xf3f2b724, 100 },
    { 0x1d82b726, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a82b729, 95 },
    { 0xb5f2b72a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d2b734, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e2b739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532b744, 76 },
    { 0, 0 },
    { 0xeb42b746, 95 },
    { 0, 0 },
    { 0x4432b748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2b750, 76 },
    { 0x94f2b751, 61 },
    { 0x2a52b750, 61 },
    { 0x5a82b753, 33 },
    { 0x5ae2b754, 83 },
    { 0, 0 },
    { 0xb9a2b756, 66 },
    { 0xad92b757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7492b75a, 23 },
    { 0, 0 },
    { 0xcd02b75c, 69 },
    { 0x5b72b75c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6222b762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462b767, 93 },
    { 0xfce2b767, 38 },
    { 0, 0 },
    { 0xf52b76a, 95 },
    { 0xa1b2b76b, 76 },
    { 0x8342b76c, 78 },
    { 0, 0 },
    { 0x16a2b76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa2b776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12b77e, 34 },
    { 0x5c12b77e, 93 },
    { 0, 0 },
    { 0xbe62b781, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db2b785, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5192b78b, 33 },
    { 0x5882b78b, 55 },
    { 0xae42b78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa72b793, 33 },
    { 0x14e2b794, 90 },
    { 0, 0 },
    { 0x9152b796, 34 },
    { 0xbe42b797, 95 },
    { 0x8f12b797, 76 },
    { 0x53b2b796, 46 },
    { 0x2bf2b79a, 61 },
    { 0, 0 },
    { 0x79d2b79c, 90 },
    { 0x51d2b79d, 33 },
    { 0, 0 },
    { 0x2522b79f, 98 },
    { 0x7cf2b79f, 33 },
    { 0, 0 },
    { 0x5832b7a2, 87 },
    { 0x62b2b7a3, 98 },
    { 0x4572b7a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a42b7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2b7ac, 23 },
    { 0, 0 },
    { 0x54a2b7ae, 76 },
    { 0xee02b7af, 46 },
    { 0xda2b7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b7b3, 98 },
    { 0x1552b7b4, 46 },
    { 0x1d22b7b5, 17 },
    { 0x7c52b7b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2b7b9, 90 },
    { 0x4272b7ba, 40 },
    { 0, 0 },
    { 0x422b7bc, 43 },
    { 0, 0 },
    { 0x7b82b7be, 38 },
    { 0, 0 },
    { 0xb752b7c0, 34 },
    { 0x8d62b7c0, 23 },
    { 0x5cb2b7c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2b7c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2b7cb, 95 },
    { 0xcf02b7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc282b7cf, 90 },
    { 0, 0 },
    { 0xda62b7d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa612b7d4, 67 },
    { 0x82d2b7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd2b7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca2b7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4272b7e4, 90 },
    { 0xdf92b7e5, 68 },
    { 0xe7a2b7e6, 35 },
    { 0xa872b7e7, 34 },
    { 0xb582b7e8, 98 },
    { 0x42b2b7e4, 76 },
    { 0x64c2b7e8, 95 },
    { 0x9382b7ea, 85 },
    { 0x6122b7ec, 91 },
    { 0x12b2b7ed, 61 },
    { 0x58c2b7ee, 61 },
    { 0x53a2b7ee, 90 },
    { 0x8e82b7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb2b7f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1882b7f6, 34 },
    { 0xe8c2b7f7, 66 },
    { 0x44e2b7f6, 52 },
    { 0x3a52b7f7, 90 },
    { 0, 0 },
    { 0xd8d2b7fb, 61 },
    { 0x26c2b7fc, 95 },
    { 0x7612b7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c2b800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea2b806, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c62b80a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e2b80e, 75 },
    { 0xaff2b80e, 95 },
    { 0xc632b810, 61 },
    { 0xbd52b811, 23 },
    { 0, 0 },
    { 0x81f2b813, 99 },
    { 0xfd52b814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac82b81a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a42b81d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa102b820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad92b825, 46 },
    { 0, 0 },
    { 0xb722b827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2b82e, 91 },
    { 0, 0 },
    { 0xaee2b830, 76 },
    { 0x6262b830, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b2b836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2b839, 55 },
    { 0, 0 },
    { 0x1f2b83b, 76 },
    { 0x3cd2b83c, 23 },
    { 0, 0 },
    { 0xc812b83e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2b841, 95 },
    { 0x9372b842, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2b847, 90 },
    { 0x7552b847, 76 },
    { 0xc712b849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7142b84d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df2b851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf492b855, 95 },
    { 0x2772b856, 90 },
    { 0, 0 },
    { 0x6dc2b858, 100 },
    { 0x6bd2b859, 95 },
    { 0, 0 },
    { 0xbdd2b85b, 33 },
    { 0xf562b85c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5432b85f, 66 },
    { 0, 0 },
    { 0xc972b861, 84 },
    { 0x5752b861, 33 },
    { 0, 0 },
    { 0x8122b864, 65 },
    { 0, 0 },
    { 0xe582b866, 93 },
    { 0x32d2b867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2b86a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce2b86d, 23 },
    { 0xb7b2b86e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a2b872, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22b876, 23 },
    { 0x2712b877, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b2b87a, 61 },
    { 0, 0 },
    { 0xcab2b87c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702b880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf32b883, 33 },
    { 0xef22b884, 52 },
    { 0x31a2b885, 46 },
    { 0xd422b886, 100 },
    { 0xf042b885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb702b88a, 61 },
    { 0, 0 },
    { 0xcfd2b88c, 93 },
    { 0, 0 },
    { 0x72e2b88e, 91 },
    { 0x732b88f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d82b892, 61 },
    { 0x4b72b893, 95 },
    { 0x7072b892, 67 },
    { 0, 0 },
    { 0xd562b896, 33 },
    { 0, 0 },
    { 0x5422b898, 90 },
    { 0x6ab2b899, 55 },
    { 0xd682b898, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912b89d, 33 },
    { 0x85c2b89d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2b8a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9352b8af, 95 },
    { 0x5ab2b8b0, 66 },
    { 0x3c2b8af, 98 },
    { 0xc032b8b2, 33 },
    { 0, 0 },
    { 0xe372b8b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe862b8b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4972b8ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2b8c1, 61 },
    { 0, 0 },
    { 0x8872b8c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4512b8c6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772b8c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832b8cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b8d0, 76 },
    { 0, 0 },
    { 0xeb42b8d2, 84 },
    { 0xe8b2b8d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b8d9, 76 },
    { 0xc5d2b8da, 33 },
    { 0, 0 },
    { 0xc2e2b8dc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb042b8e0, 55 },
    { 0xec22b8e0, 87 },
    { 0xb352b8e0, 91 },
    { 0x4522b8e0, 90 },
    { 0x9502b8e0, 82 },
    { 0, 0 },
    { 0xde92b8e6, 90 },
    { 0xf372b8e6, 66 },
    { 0x2952b8e8, 95 },
    { 0xf802b8e9, 85 },
    { 0xa802b8e9, 91 },
    { 0, 0 },
    { 0x80e2b8ec, 61 },
    { 0, 0 },
    { 0x60d2b8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df2b8f1, 43 },
    { 0x1912b8f2, 76 },
    { 0x15e2b8f2, 90 },
    { 0x94d2b8f4, 34 },
    { 0x4672b8f4, 88 },
    { 0xb72b8f5, 90 },
    { 0xb3b2b8f6, 98 },
    { 0x6a42b8f7, 98 },
    { 0, 0 },
    { 0x8f92b8fa, 53 },
    { 0x3ee2b8fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52b8fe, 52 },
    { 0x2012b8ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c82b907, 77 },
    { 0, 0 },
    { 0xfe2b909, 98 },
    { 0x4192b90a, 90 },
    { 0, 0 },
    { 0x9102b90c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5662b915, 98 },
    { 0x2c82b916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f2b920, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552b923, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe152b927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1172b92a, 98 },
    { 0, 0 },
    { 0xc0d2b92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d2b92f, 90 },
    { 0, 0 },
    { 0x8782b931, 98 },
    { 0, 0 },
    { 0xfc02b933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd2b937, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca02b93a, 96 },
    { 0, 0 },
    { 0x4622b93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b52b940, 46 },
    { 0xc6b2b940, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2702b947, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b2b94a, 90 },
    { 0x6a42b94a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7862b94f, 33 },
    { 0, 0 },
    { 0xc432b951, 76 },
    { 0, 0 },
    { 0x44b2b953, 33 },
    { 0xcba2b953, 90 },
    { 0x3a42b954, 92 },
    { 0xbd02b956, 32 },
    { 0xee72b956, 23 },
    { 0x6332b958, 34 },
    { 0, 0 },
    { 0x6ab2b95a, 34 },
    { 0, 0 },
    { 0xc382b95c, 61 },
    { 0, 0 },
    { 0x2a72b95e, 93 },
    { 0x1832b95f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2b963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52b96a, 61 },
    { 0x30f2b96b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6032b96f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6732b974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee22b97b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce2b980, 69 },
    { 0xe9d2b981, 95 },
    { 0x8c32b982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d2b986, 95 },
    { 0, 0 },
    { 0x34f2b988, 33 },
    { 0x74b2b989, 61 },
    { 0x3822b98a, 33 },
    { 0xa342b98b, 95 },
    { 0xdeb2b98a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb692b990, 95 },
    { 0x1782b991, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b2b995, 61 },
    { 0x7d52b996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5252b99e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3612b9a2, 83 },
    { 0xfcd2b9a2, 90 },
    { 0x95b2b9a4, 76 },
    { 0xcd12b9a5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de2b9a8, 33 },
    { 0xe2b2b9a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd62b9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d2b9b5, 93 },
    { 0x3af2b9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d2b9ba, 90 },
    { 0x2972b9ba, 23 },
    { 0, 0 },
    { 0x2cd2b9bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a2b9c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe432b9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7002b9cc, 36 },
    { 0xc132b9cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc2b9d4, 67 },
    { 0, 0 },
    { 0xc8e2b9d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa62b9dc, 90 },
    { 0xde72b9dc, 74 },
    { 0xbe12b9de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc2b9e4, 95 },
    { 0, 0 },
    { 0x98e2b9e6, 61 },
    { 0x1fc2b9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62b9ea, 61 },
    { 0x3bc2b9ea, 61 },
    { 0x4e82b9ec, 98 },
    { 0x2342b9ec, 61 },
    { 0x8382b9ee, 93 },
    { 0xf412b9ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc72b9f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302b9f5, 43 },
    { 0, 0 },
    { 0x9b92b9f7, 34 },
    { 0, 0 },
    { 0xa0c2b9f9, 33 },
    { 0, 0 },
    { 0xb1a2b9fb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf2b9fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f62ba02, 55 },
    { 0xf9e2ba02, 33 },
    { 0x1cd2ba03, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e2ba0a, 75 },
    { 0, 0 },
    { 0xdfa2ba0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4852ba0f, 23 },
    { 0x5932ba0f, 90 },
    { 0x4902ba11, 90 },
    { 0x13b2ba11, 84 },
    { 0x29a2ba13, 95 },
    { 0xc92ba14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb2ba17, 33 },
    { 0xf0a2ba17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea2ba23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f2ba26, 84 },
    { 0x9882ba27, 90 },
    { 0x7212ba28, 91 },
    { 0x2ba2ba26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552ba31, 66 },
    { 0, 0 },
    { 0x6f92ba33, 61 },
    { 0x222ba34, 23 },
    { 0xf4e2ba35, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b2ba39, 23 },
    { 0x8e22ba39, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf162ba3e, 84 },
    { 0xe812ba3f, 83 },
    { 0x3022ba3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f2ba43, 90 },
    { 0x2822ba43, 98 },
    { 0x7732ba44, 90 },
    { 0x9052ba46, 99 },
    { 0xf9c2ba44, 95 },
    { 0x5722ba47, 80 },
    { 0xb2f2ba49, 90 },
    { 0xeb32ba4a, 66 },
    { 0xb1d2ba4b, 48 },
    { 0xb652ba47, 80 },
    { 0xd852ba47, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf882ba52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc742ba55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32ba5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e2ba63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe882ba66, 66 },
    { 0, 0 },
    { 0x4582ba68, 66 },
    { 0x1de2ba69, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc502ba6d, 91 },
    { 0x6742ba6e, 23 },
    { 0x632ba6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c2ba79, 90 },
    { 0xad2ba7a, 98 },
    { 0x4f82ba7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb22ba80, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62ba84, 33 },
    { 0x1e2ba85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982ba88, 99 },
    { 0x51e2ba89, 90 },
    { 0x6ca2ba8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c2ba8d, 84 },
    { 0xac12ba8e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2ba91, 76 },
    { 0, 0 },
    { 0xafd2ba93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982ba97, 96 },
    { 0x6d72ba98, 46 },
    { 0x8482ba99, 23 },
    { 0xc62ba9a, 90 },
    { 0xebb2ba97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d2ba9e, 93 },
    { 0xa232ba9f, 56 },
    { 0, 0 },
    { 0x4d82baa1, 90 },
    { 0x7ae2baa1, 78 },
    { 0, 0 },
    { 0xc932baa4, 34 },
    { 0xb572baa5, 61 },
    { 0x8b22baa5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752baab, 55 },
    { 0x3c02baab, 72 },
    { 0, 0 },
    { 0xc7a2baae, 90 },
    { 0, 0 },
    { 0xc382bab0, 90 },
    { 0xac32bab1, 87 },
    { 0, 0 },
    { 0x3162bab3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2262bab9, 98 },
    { 0, 0 },
    { 0x8d12babb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe272bac3, 90 },
    { 0x7b52bac4, 61 },
    { 0, 0 },
    { 0x3d2bac6, 33 },
    { 0x52e2bac7, 61 },
    { 0, 0 },
    { 0xe1b2bac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2bacd, 90 },
    { 0xd6e2bace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca2bad2, 95 },
    { 0x7a42bad3, 52 },
    { 0, 0 },
    { 0xf662bad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92badb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b2bae2, 34 },
    { 0xdd22bae3, 90 },
    { 0x3282bae3, 98 },
    { 0x6282bae4, 61 },
    { 0x5f02bae3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192baf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd062baf6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd32bafa, 76 },
    { 0xb3e2bafb, 78 },
    { 0x6dd2bafc, 90 },
    { 0, 0 },
    { 0x2882bafe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552bb02, 95 },
    { 0x27d2bb02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d52bb07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b12bb0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2bb13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed92bb17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7532bb1c, 66 },
    { 0, 0 },
    { 0x7622bb1e, 23 },
    { 0, 0 },
    { 0x7e72bb20, 98 },
    { 0x36d2bb21, 33 },
    { 0x81c2bb20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a32bb25, 33 },
    { 0x7052bb25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d2bb29, 91 },
    { 0x6342bb2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed62bb31, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442bb38, 34 },
    { 0x7db2bb38, 98 },
    { 0xc462bb38, 33 },
    { 0xc7a2bb3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb422bb43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72bb46, 98 },
    { 0xaba2bb47, 85 },
    { 0x88a2bb48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9412bb4f, 98 },
    { 0x3ef2bb4f, 90 },
    { 0x7652bb51, 100 },
    { 0xb652bb52, 95 },
    { 0xbaa2bb50, 98 },
    { 0x1d62bb52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d92bb57, 90 },
    { 0xf9d2bb58, 87 },
    { 0, 0 },
    { 0xdf12bb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412bb5f, 76 },
    { 0xf902bb60, 88 },
    { 0x15b2bb60, 98 },
    { 0x1ff2bb60, 23 },
    { 0, 0 },
    { 0x9952bb64, 61 },
    { 0x4c62bb64, 55 },
    { 0, 0 },
    { 0xb572bb67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x932bb6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782bb72, 43 },
    { 0xd7b2bb72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe482bb7c, 98 },
    { 0xd982bb7c, 90 },
    { 0xfa02bb7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2bb83, 66 },
    { 0xa712bb83, 17 },
    { 0x24b2bb83, 90 },
    { 0x5f62bb86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22bb8a, 95 },
    { 0x6e12bb8b, 55 },
    { 0, 0 },
    { 0x43c2bb8d, 34 },
    { 0, 0 },
    { 0x6222bb8f, 55 },
    { 0, 0 },
    { 0x29a2bb91, 90 },
    { 0xd772bb92, 33 },
    { 0xea32bb93, 61 },
    { 0x2f82bb94, 87 },
    { 0x2292bb94, 100 },
    { 0x6d52bb96, 34 },
    { 0xff42bb97, 61 },
    { 0xbbe2bb97, 100 },
    { 0xb272bb99, 61 },
    { 0x29f2bb99, 85 },
    { 0xeb22bb9b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2bba1, 78 },
    { 0x3562bba2, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9982bba6, 43 },
    { 0, 0 },
    { 0x6912bba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092bbab, 95 },
    { 0x9122bbab, 76 },
    { 0x5442bbab, 61 },
    { 0x89d2bbae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1722bbb2, 33 },
    { 0x51f2bbb3, 95 },
    { 0, 0 },
    { 0xe9e2bbb5, 98 },
    { 0x7022bbb5, 90 },
    { 0xf8c2bbb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde92bbbc, 33 },
    { 0xa1f2bbbd, 90 },
    { 0x2f82bbbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702bbc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5282bbca, 33 },
    { 0x59a2bbca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe2bbcf, 77 },
    { 0x8812bbd0, 23 },
    { 0, 0 },
    { 0xd182bbd2, 61 },
    { 0xda72bbd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a52bbd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa2bbd9, 81 },
    { 0x6472bbd9, 98 },
    { 0x61f2bbdb, 33 },
    { 0x50e2bbdc, 85 },
    { 0x1732bbdd, 61 },
    { 0xc6b2bbdc, 55 },
    { 0, 0 },
    { 0x9c32bbe0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a2bbe5, 33 },
    { 0, 0 },
    { 0x7d72bbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a2bbec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72bbef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3372bbf5, 61 },
    { 0xc672bbf6, 46 },
    { 0xecf2bbf5, 43 },
    { 0x5e42bbf5, 78 },
    { 0xf932bbf5, 93 },
    { 0x6342bbfa, 90 },
    { 0, 0 },
    { 0x1202bbfc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2bc00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1242bc0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62bc11, 33 },
    { 0xae52bc11, 91 },
    { 0x4c92bc12, 33 },
    { 0xa402bc14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc2bc18, 90 },
    { 0x34c2bc19, 76 },
    { 0xc562bc1a, 33 },
    { 0xe4e2bc1b, 34 },
    { 0, 0 },
    { 0xe542bc1d, 43 },
    { 0, 0 },
    { 0x5dc2bc1f, 61 },
    { 0xc012bc1f, 90 },
    { 0, 0 },
    { 0x2292bc22, 66 },
    { 0, 0 },
    { 0x5fc2bc24, 78 },
    { 0x8f72bc25, 98 },
    { 0xbba2bc25, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2bc2a, 61 },
    { 0, 0 },
    { 0x2282bc2c, 97 },
    { 0xfe12bc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb732bc36, 97 },
    { 0xf2b2bc37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc2bc3f, 98 },
    { 0x8782bc40, 78 },
    { 0xcb62bc41, 76 },
    { 0, 0 },
    { 0x5d12bc43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd2bc46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d72bc49, 33 },
    { 0xba62bc4a, 93 },
    { 0x6452bc4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2bc52, 93 },
    { 0xee22bc53, 78 },
    { 0, 0 },
    { 0xaa92bc55, 33 },
    { 0xcfb2bc56, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a02bc5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142bc5f, 76 },
    { 0x2702bc60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e2bc66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92bc6a, 95 },
    { 0, 0 },
    { 0x7042bc6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f2bc70, 61 },
    { 0x5ec2bc70, 95 },
    { 0, 0 },
    { 0x5712bc73, 68 },
    { 0, 0 },
    { 0x5b22bc75, 55 },
    { 0x4312bc76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2bc7a, 85 },
    { 0, 0 },
    { 0x97a2bc7c, 95 },
    { 0, 0 },
    { 0xc702bc7e, 61 },
    { 0, 0 },
    { 0x2312bc80, 61 },
    { 0x88b2bc80, 55 },
    { 0x3012bc82, 88 },
    { 0x7c62bc82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a2bc86, 33 },
    { 0x96b2bc87, 93 },
    { 0xd832bc87, 23 },
    { 0x7362bc87, 61 },
    { 0x7a42bc88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6622bc8e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e2bc91, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5562bc95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca2bc98, 33 },
    { 0x4282bc98, 61 },
    { 0xce62bc98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d2bc9d, 100 },
    { 0x2862bc9d, 98 },
    { 0x42b2bc9f, 78 },
    { 0xb712bc9f, 90 },
    { 0x83f2bc9f, 92 },
    { 0xb2c2bc9e, 33 },
    { 0x82a2bca3, 84 },
    { 0xd222bca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1522bca9, 55 },
    { 0, 0 },
    { 0xdd32bcab, 93 },
    { 0xaec2bcac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb012bcaf, 93 },
    { 0xf772bcb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2bcb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12bcb9, 76 },
    { 0xc522bcba, 98 },
    { 0, 0 },
    { 0xb9d2bcbc, 91 },
    { 0xab12bcbd, 100 },
    { 0xbef2bcbe, 98 },
    { 0x8e82bcbf, 61 },
    { 0x9d32bcbf, 95 },
    { 0x9d72bcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe32bcc6, 55 },
    { 0xaa72bcc6, 33 },
    { 0x8a12bcc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db2bccd, 90 },
    { 0, 0 },
    { 0x4822bccf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d2bcd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bcda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c2bcdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e2bce4, 90 },
    { 0x8dc2bce5, 23 },
    { 0xb5e2bce5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2bcf0, 85 },
    { 0x55d2bcf1, 98 },
    { 0x61e2bcf1, 33 },
    { 0xb5a2bcf1, 43 },
    { 0x56d2bcf1, 95 },
    { 0xc6d2bcf4, 40 },
    { 0xff52bcf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12bcfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb42bd01, 92 },
    { 0x37b2bd02, 55 },
    { 0xa2bd02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab02bd06, 90 },
    { 0, 0 },
    { 0x352bd08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2bd0c, 90 },
    { 0, 0 },
    { 0x98a2bd0e, 33 },
    { 0, 0 },
    { 0xc442bd10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3602bd13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082bd16, 90 },
    { 0x3fd2bd16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc302bd1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a32bd1e, 55 },
    { 0, 0 },
    { 0xf062bd20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2bd26, 90 },
    { 0, 0 },
    { 0x8f02bd28, 23 },
    { 0x2412bd29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b2bd2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2bd34, 99 },
    { 0xd4f2bd35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda42bd39, 55 },
    { 0x78e2bd3a, 17 },
    { 0x89f2bd3a, 61 },
    { 0x1cc2bd39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a2bd3f, 61 },
    { 0, 0 },
    { 0x6a42bd41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb062bd44, 98 },
    { 0xb42bd45, 98 },
    { 0xed52bd45, 75 },
    { 0, 0 },
    { 0x5f22bd48, 98 },
    { 0x4c52bd48, 93 },
    { 0x3a42bd4a, 23 },
    { 0xceb2bd4a, 95 },
    { 0xa442bd4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8882bd53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672bd57, 75 },
    { 0, 0 },
    { 0x98b2bd59, 90 },
    { 0xbfe2bd5a, 33 },
    { 0xbec2bd5b, 23 },
    { 0xc0e2bd5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292bd62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa72bd66, 90 },
    { 0xd732bd67, 95 },
    { 0x28b2bd67, 95 },
    { 0xd082bd67, 95 },
    { 0x7772bd6a, 91 },
    { 0, 0 },
    { 0x1f72bd6c, 90 },
    { 0xbe72bd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6082bd74, 61 },
    { 0xb462bd75, 76 },
    { 0x34f2bd76, 55 },
    { 0x3192bd77, 55 },
    { 0xe772bd77, 55 },
    { 0, 0 },
    { 0xc512bd7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c2bd7f, 51 },
    { 0, 0 },
    { 0x30d2bd81, 84 },
    { 0xa952bd82, 33 },
    { 0x9372bd81, 90 },
    { 0x7a12bd84, 87 },
    { 0x9bb2bd84, 48 },
    { 0x17e2bd86, 98 },
    { 0x9e2bd85, 90 },
    { 0xec62bd81, 93 },
    { 0xb992bd84, 55 },
    { 0, 0 },
    { 0xb522bd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a2bd8e, 23 },
    { 0, 0 },
    { 0x1f12bd90, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b2bd93, 61 },
    { 0, 0 },
    { 0x5e62bd95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4382bd9c, 99 },
    { 0xc2c2bd9c, 64 },
    { 0, 0 },
    { 0xa742bd9f, 95 },
    { 0x2c42bda0, 61 },
    { 0, 0 },
    { 0x8512bda2, 55 },
    { 0x5c72bda3, 38 },
    { 0x7d82bda2, 61 },
    { 0x77c2bda4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda92bda8, 53 },
    { 0, 0 },
    { 0x6262bdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f12bdaf, 80 },
    { 0x6862bdb0, 23 },
    { 0xc2e2bdb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e2bdb4, 90 },
    { 0, 0 },
    { 0xea2bdb6, 33 },
    { 0xe112bdb7, 98 },
    { 0x7142bdb8, 98 },
    { 0x38a2bdb8, 43 },
    { 0x1d62bdba, 95 },
    { 0x11e2bdb9, 98 },
    { 0xb2bdbb, 98 },
    { 0xa4c2bdbc, 43 },
    { 0, 0 },
    { 0x74e2bdbf, 81 },
    { 0x47d2bdbf, 90 },
    { 0xc402bdc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2bdc6, 87 },
    { 0x2762bdc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7112bdca, 17 },
    { 0x3dc2bdcb, 99 },
    { 0x3bc2bdcc, 98 },
    { 0x3b42bdcd, 98 },
    { 0x7992bdcd, 61 },
    { 0xdf92bdcd, 61 },
    { 0x7c72bdcd, 81 },
    { 0x53d2bdcb, 23 },
    { 0x4112bdd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6862bdd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd82bddd, 93 },
    { 0xbcd2bdde, 96 },
    { 0xa1c2bddf, 74 },
    { 0, 0 },
    { 0x26d2bde1, 90 },
    { 0x4692bde2, 90 },
    { 0x712bde2, 98 },
    { 0x5012bde3, 95 },
    { 0xcc02bde4, 49 },
    { 0x12b2bde6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca92bdea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2bded, 55 },
    { 0xd732bdee, 98 },
    { 0xc782bdef, 61 },
    { 0x3f82bdf0, 78 },
    { 0xd92bdf1, 95 },
    { 0xda72bdf2, 76 },
    { 0x55e2bdf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b32bdf6, 55 },
    { 0x7952bdf6, 83 },
    { 0xa872bdf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7582bdfd, 33 },
    { 0xcca2bdfe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd2be04, 52 },
    { 0, 0 },
    { 0xb752be06, 88 },
    { 0, 0 },
    { 0xe9e2be08, 33 },
    { 0, 0 },
    { 0x3862be0a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa052be0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2812be15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682be1b, 46 },
    { 0x8c02be1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b2be25, 83 },
    { 0x292be25, 90 },
    { 0xe192be27, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3432be2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92be2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa32be35, 33 },
    { 0xfe32be36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a2be3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982be3e, 33 },
    { 0, 0 },
    { 0xcf02be40, 90 },
    { 0x4872be41, 64 },
    { 0x18f2be40, 23 },
    { 0x5f2be43, 99 },
    { 0x4f82be44, 33 },
    { 0x1ba2be44, 66 },
    { 0x652be44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e2be4b, 91 },
    { 0x79f2be4b, 90 },
    { 0x4452be4d, 23 },
    { 0, 0 },
    { 0xdb72be4f, 33 },
    { 0xec32be4f, 90 },
    { 0xbff2be51, 98 },
    { 0x1df2be52, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef2be5c, 91 },
    { 0, 0 },
    { 0x1f42be5e, 76 },
    { 0xfaf2be5e, 61 },
    { 0xfb2be60, 66 },
    { 0xef92be61, 34 },
    { 0x1cf2be60, 61 },
    { 0, 0 },
    { 0x852be64, 61 },
    { 0, 0 },
    { 0x4332be66, 23 },
    { 0x1ed2be67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d2be6f, 46 },
    { 0, 0 },
    { 0x54e2be71, 67 },
    { 0, 0 },
    { 0xf8a2be73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832be76, 34 },
    { 0, 0 },
    { 0x5ea2be78, 55 },
    { 0xbeb2be78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb2be7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb72be7f, 78 },
    { 0, 0 },
    { 0xb3d2be81, 82 },
    { 0xbf12be82, 23 },
    { 0xd8f2be83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2be87, 95 },
    { 0x4582be88, 33 },
    { 0x6ca2be87, 90 },
    { 0x2372be89, 61 },
    { 0xcb2be87, 98 },
    { 0x5842be8c, 90 },
    { 0x6a02be88, 93 },
    { 0x8b12be8c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2be91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2be98, 91 },
    { 0x6932be98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a2bea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af2bea9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f2beac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8172beaf, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d2beb5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7852beb9, 33 },
    { 0, 0 },
    { 0x8732bebb, 81 },
    { 0, 0 },
    { 0xf532bebd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2492bec0, 75 },
    { 0, 0 },
    { 0xe022bec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd2beca, 33 },
    { 0, 0 },
    { 0x91b2becc, 23 },
    { 0x11d2becd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a2bed1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee2bed4, 17 },
    { 0xb442bed5, 90 },
    { 0xfae2bed6, 76 },
    { 0xa622bed7, 77 },
    { 0xdcc2bed5, 95 },
    { 0x3af2bed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc2bee0, 61 },
    { 0x2282bee1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1902bee5, 61 },
    { 0xe5e2bee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e2bee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12beef, 23 },
    { 0, 0 },
    { 0xce52bef1, 98 },
    { 0, 0 },
    { 0x7bb2bef3, 55 },
    { 0x79d2bef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f2bef7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd192befa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b2bf01, 61 },
    { 0xdf02bf01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2bf07, 90 },
    { 0x5952bf07, 95 },
    { 0x99d2bf09, 23 },
    { 0x54a2bf0a, 65 },
    { 0xecf2bf07, 98 },
    { 0x57e2bf0c, 84 },
    { 0x7d12bf0d, 66 },
    { 0xf5d2bf08, 33 },
    { 0, 0 },
    { 0xdfd2bf10, 87 },
    { 0xe2c2bf11, 34 },
    { 0, 0 },
    { 0xd442bf13, 90 },
    { 0xb872bf14, 90 },
    { 0x45e2bf15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2bf18, 53 },
    { 0x8482bf19, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2bf1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc02bf2d, 95 },
    { 0xf462bf2d, 33 },
    { 0xdb32bf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d2bf34, 98 },
    { 0xfcb2bf34, 61 },
    { 0xb962bf36, 90 },
    { 0xa4c2bf35, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe872bf3b, 80 },
    { 0x68b2bf3c, 87 },
    { 0x3cd2bf3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2bf41, 90 },
    { 0xd012bf42, 33 },
    { 0, 0 },
    { 0xdd72bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92bf47, 100 },
    { 0xdff2bf48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3212bf4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2bf51, 90 },
    { 0x7922bf52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8912bf56, 67 },
    { 0, 0 },
    { 0xa0e2bf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182bf5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe992bf63, 76 },
    { 0, 0 },
    { 0xaa92bf65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b2bf6c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3722bf6f, 76 },
    { 0x1612bf70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd662bf79, 95 },
    { 0xab72bf7a, 17 },
    { 0xbdb2bf7a, 95 },
    { 0x8ac2bf7c, 99 },
    { 0x50b2bf7a, 76 },
    { 0, 0 },
    { 0xe7e2bf7f, 95 },
    { 0, 0 },
    { 0xe9a2bf81, 33 },
    { 0x4482bf82, 99 },
    { 0xe1a2bf83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2bf87, 17 },
    { 0, 0 },
    { 0x9832bf89, 34 },
    { 0x23b2bf89, 76 },
    { 0x5f72bf8b, 90 },
    { 0x1822bf8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2072bf90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d2bf95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12bf98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad2bfa0, 98 },
    { 0x9d12bfa1, 95 },
    { 0x1bb2bfa1, 95 },
    { 0xf5c2bfa1, 33 },
    { 0xf462bfa4, 98 },
    { 0x75c2bfa4, 33 },
    { 0, 0 },
    { 0x9d42bfa7, 95 },
    { 0x9992bfa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb162bfab, 76 },
    { 0xc6e2bfab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb2bfb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2bfba, 33 },
    { 0xa882bfbb, 66 },
    { 0x2d52bfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32bfbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa492bfc3, 90 },
    { 0x5cd2bfc4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4312bfc9, 23 },
    { 0, 0 },
    { 0xb3d2bfcb, 84 },
    { 0, 0 },
    { 0xb792bfcd, 90 },
    { 0x75e2bfcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2bfd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5372bfd5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902bfd8, 98 },
    { 0, 0 },
    { 0xb1d2bfda, 68 },
    { 0xcdd2bfdb, 87 },
    { 0xfa92bfda, 66 },
    { 0, 0 },
    { 0x2872bfde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5632bfe3, 95 },
    { 0xcf2bfe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2bfee, 95 },
    { 0xb22bfef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd102bff5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b2bffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7702bffe, 99 },
    { 0xc8e2bfff, 66 },
    { 0, 0 },
    { 0x342c001, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a12c007, 81 },
    { 0, 0 },
    { 0xf0e2c009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a72c00d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c02c012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c2c016, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2c01d, 34 },
    { 0, 0 },
    { 0xc662c01f, 55 },
    { 0xf472c020, 61 },
    { 0xfec2c01f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b2c026, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22c029, 23 },
    { 0, 0 },
    { 0x1d12c02b, 17 },
    { 0xa0a2c02b, 98 },
    { 0xc8c2c02d, 33 },
    { 0, 0 },
    { 0x6d92c02f, 33 },
    { 0xd232c02f, 33 },
    { 0xae12c031, 95 },
    { 0xcca2c031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c03c, 90 },
    { 0, 0 },
    { 0xe2f2c03e, 78 },
    { 0xe1d2c03f, 23 },
    { 0x6b82c040, 23 },
    { 0x7332c041, 23 },
    { 0x45e2c042, 90 },
    { 0x1c2c043, 91 },
    { 0xd8d2c041, 23 },
    { 0x3022c045, 90 },
    { 0xc4c2c046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f2c04b, 61 },
    { 0, 0 },
    { 0x9cb2c04d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc52c052, 61 },
    { 0, 0 },
    { 0xcf12c054, 91 },
    { 0, 0 },
    { 0x4a2c056, 77 },
    { 0x93f2c057, 98 },
    { 0x80d2c058, 61 },
    { 0x6952c059, 98 },
    { 0xdc22c058, 81 },
    { 0x3382c05b, 83 },
    { 0xeeb2c05c, 95 },
    { 0xcd12c05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae2c064, 23 },
    { 0xc22c065, 43 },
    { 0xd2a2c065, 34 },
    { 0, 0 },
    { 0x6c02c068, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62c06c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9422c06f, 76 },
    { 0x2bb2c070, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8782c073, 90 },
    { 0x56a2c074, 91 },
    { 0, 0 },
    { 0xecd2c076, 77 },
    { 0xee62c077, 61 },
    { 0x7d42c078, 90 },
    { 0xfcd2c079, 23 },
    { 0xcb82c077, 98 },
    { 0x6012c07b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf672c080, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x832c083, 61 },
    { 0, 0 },
    { 0xf542c085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe22c08e, 61 },
    { 0x2d22c08f, 67 },
    { 0xf822c090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92c099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf872c09c, 33 },
    { 0, 0 },
    { 0x4022c09e, 23 },
    { 0x1eb2c09f, 95 },
    { 0, 0 },
    { 0xac82c0a1, 61 },
    { 0, 0 },
    { 0xf3e2c0a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402c0a9, 87 },
    { 0x7552c0aa, 61 },
    { 0x5d42c0ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc772c0b2, 78 },
    { 0x3c42c0b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee2c0ba, 90 },
    { 0, 0 },
    { 0x2002c0bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb52c0bf, 95 },
    { 0x2d62c0c0, 90 },
    { 0, 0 },
    { 0xcbe2c0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b2c0c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe192c0c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a22c0cd, 81 },
    { 0xf722c0ce, 90 },
    { 0, 0 },
    { 0x1092c0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb062c0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692c0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc2c0df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262c0e3, 96 },
    { 0x9762c0e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa352c0e8, 91 },
    { 0, 0 },
    { 0x4c02c0ea, 76 },
    { 0xf152c0eb, 76 },
    { 0x1b22c0eb, 43 },
    { 0xc822c0ea, 90 },
    { 0xf532c0ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd2c0f6, 61 },
    { 0x2342c0f7, 46 },
    { 0x25c2c0f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2c0ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9972c106, 98 },
    { 0, 0 },
    { 0x1fc2c108, 98 },
    { 0, 0 },
    { 0x8222c10a, 83 },
    { 0, 0 },
    { 0xed32c10c, 33 },
    { 0xb9e2c10d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe82c110, 61 },
    { 0xa032c111, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c118, 95 },
    { 0, 0 },
    { 0x9c82c11a, 95 },
    { 0xce42c11b, 93 },
    { 0xe832c11a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc792c121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832c125, 81 },
    { 0x38c2c126, 68 },
    { 0x10b2c126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c92c12c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b52c132, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2c135, 33 },
    { 0, 0 },
    { 0x7002c137, 90 },
    { 0, 0 },
    { 0xa392c139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c140, 23 },
    { 0x6682c141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672c144, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce2c147, 95 },
    { 0x7222c148, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa062c14b, 88 },
    { 0xd062c14c, 23 },
    { 0xb1e2c14d, 46 },
    { 0x5e12c14e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac2c152, 90 },
    { 0x9962c152, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b92c159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2c15d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c2c161, 33 },
    { 0xdbb2c162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6932c168, 98 },
    { 0xda22c169, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3882c173, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f42c179, 66 },
    { 0xddf2c17a, 68 },
    { 0x1b42c17a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9482c182, 23 },
    { 0, 0 },
    { 0x882c184, 95 },
    { 0x5ec2c185, 46 },
    { 0x4252c186, 43 },
    { 0xef62c184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2c18d, 98 },
    { 0x9c02c18d, 43 },
    { 0x2bf2c18f, 33 },
    { 0xdef2c190, 98 },
    { 0, 0 },
    { 0x212c192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2c197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2c1a2, 91 },
    { 0x3f2c1a2, 90 },
    { 0xf742c1a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9742c1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e2c1b6, 85 },
    { 0, 0 },
    { 0x1922c1b8, 88 },
    { 0xab62c1b9, 23 },
    { 0x71d2c1ba, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf252c1bd, 33 },
    { 0x9712c1be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf812c1c3, 90 },
    { 0xb312c1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2c1cd, 46 },
    { 0x5482c1ce, 55 },
    { 0x3542c1cf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc212c1d2, 52 },
    { 0, 0 },
    { 0x40f2c1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e2c1df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3802c1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd352c1ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4402c1ef, 87 },
    { 0x1532c1f0, 34 },
    { 0x4882c1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e2c1f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa62c1fa, 33 },
    { 0xcfd2c1fa, 98 },
    { 0x3ee2c1fc, 93 },
    { 0x8f42c1fd, 98 },
    { 0xf22c1fb, 95 },
    { 0x25d2c1fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d2c203, 34 },
    { 0xd3e2c203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52c208, 23 },
    { 0, 0 },
    { 0x1e62c20a, 33 },
    { 0x5fd2c20a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae2c212, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f2c216, 66 },
    { 0, 0 },
    { 0x40d2c218, 23 },
    { 0x722c219, 95 },
    { 0, 0 },
    { 0x2882c21b, 91 },
    { 0x7a12c21c, 90 },
    { 0x1962c21c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6452c221, 76 },
    { 0xbef2c222, 90 },
    { 0xeae2c223, 55 },
    { 0x40f2c224, 91 },
    { 0x20d2c225, 91 },
    { 0x872c226, 90 },
    { 0xb242c227, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452c22a, 90 },
    { 0, 0 },
    { 0x8242c22c, 90 },
    { 0x4c32c22c, 95 },
    { 0xa252c22c, 93 },
    { 0xa0e2c22f, 23 },
    { 0x3022c230, 78 },
    { 0xab12c22f, 69 },
    { 0xc3e2c22c, 33 },
    { 0x3552c233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2c237, 61 },
    { 0x9232c238, 53 },
    { 0, 0 },
    { 0xd252c23a, 61 },
    { 0x5b72c23a, 43 },
    { 0x5c62c23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2c243, 61 },
    { 0x2a52c244, 90 },
    { 0x5b22c245, 90 },
    { 0x4fa2c246, 53 },
    { 0x5e02c246, 55 },
    { 0xadd2c248, 90 },
    { 0xe642c244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2c24d, 43 },
    { 0x19b2c24d, 61 },
    { 0, 0 },
    { 0x8f92c250, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xea72c253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a22c258, 23 },
    { 0x9ea2c259, 80 },
    { 0xa2e2c25a, 95 },
    { 0xd352c259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2c25e, 93 },
    { 0x4f62c25e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf2c263, 90 },
    { 0xcfd2c264, 23 },
    { 0x6272c263, 55 },
    { 0x8502c266, 23 },
    { 0, 0 },
    { 0x4862c268, 65 },
    { 0, 0 },
    { 0xf322c26a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042c26e, 23 },
    { 0xb022c26e, 95 },
    { 0x3b72c26e, 98 },
    { 0xbb92c271, 52 },
    { 0x6fd2c272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f2c277, 98 },
    { 0x28d2c277, 90 },
    { 0xb482c279, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b2c27d, 98 },
    { 0x85c2c27e, 61 },
    { 0, 0 },
    { 0x8b02c280, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d2c284, 78 },
    { 0xaf62c285, 85 },
    { 0xbe22c286, 23 },
    { 0xb8f2c287, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf252c28c, 99 },
    { 0xfa92c28d, 61 },
    { 0xd5e2c28d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f2c291, 33 },
    { 0, 0 },
    { 0xdb82c293, 76 },
    { 0x72c294, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2c299, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa852c29c, 55 },
    { 0x61b2c29c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c2a0, 23 },
    { 0xf632c2a0, 85 },
    { 0x7752c2a0, 90 },
    { 0x85e2c2a1, 34 },
    { 0x61e2c2a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af2c2a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c2c2aa, 97 },
    { 0, 0 },
    { 0xbaa2c2ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4532c2af, 61 },
    { 0, 0 },
    { 0x9a2c2b1, 98 },
    { 0xa812c2b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a2c2b6, 23 },
    { 0, 0 },
    { 0xb202c2b8, 90 },
    { 0, 0 },
    { 0x8d62c2ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1182c2be, 27 },
    { 0x8e62c2bf, 61 },
    { 0x4cd2c2c0, 99 },
    { 0x95b2c2bf, 61 },
    { 0, 0 },
    { 0xa992c2c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22c2c7, 98 },
    { 0, 0 },
    { 0x7192c2c9, 78 },
    { 0, 0 },
    { 0xa8e2c2cb, 81 },
    { 0, 0 },
    { 0xa342c2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad2c2d0, 23 },
    { 0x6de2c2d1, 87 },
    { 0xf1f2c2d1, 34 },
    { 0x4ed2c2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2c2d8, 93 },
    { 0xdc02c2d9, 61 },
    { 0x3832c2da, 55 },
    { 0xdfc2c2db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee2c2e3, 69 },
    { 0xa022c2e4, 61 },
    { 0xf5f2c2e5, 61 },
    { 0, 0 },
    { 0x3a22c2e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc532c2eb, 61 },
    { 0x5f52c2ec, 68 },
    { 0, 0 },
    { 0x4552c2ee, 76 },
    { 0x5f52c2ef, 33 },
    { 0x3292c2ef, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562c2f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f52c2fb, 84 },
    { 0x1a12c2fc, 95 },
    { 0, 0 },
    { 0xd9d2c2fe, 23 },
    { 0x9ad2c2ff, 67 },
    { 0x5272c2ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592c303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa882c30a, 17 },
    { 0, 0 },
    { 0x59d2c30c, 95 },
    { 0x4162c30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec52c311, 61 },
    { 0x9822c312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec12c316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb782c31a, 23 },
    { 0, 0 },
    { 0x3622c31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd552c321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5292c326, 95 },
    { 0x3772c327, 66 },
    { 0x2ec2c328, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c32b, 76 },
    { 0x54c2c32b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf012c32f, 90 },
    { 0, 0 },
    { 0x4992c331, 90 },
    { 0, 0 },
    { 0xd122c333, 61 },
    { 0, 0 },
    { 0x57d2c335, 90 },
    { 0xf002c335, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a2c339, 55 },
    { 0x2f72c33a, 33 },
    { 0, 0 },
    { 0xf9f2c33c, 55 },
    { 0xfc32c33d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee2c349, 34 },
    { 0x2f02c349, 90 },
    { 0x8b52c34b, 85 },
    { 0xd272c34c, 76 },
    { 0, 0 },
    { 0x90e2c34e, 90 },
    { 0, 0 },
    { 0x2ae2c350, 61 },
    { 0, 0 },
    { 0xddf2c352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c2c355, 90 },
    { 0, 0 },
    { 0xa7b2c357, 98 },
    { 0, 0 },
    { 0xeef2c359, 90 },
    { 0xb8c2c35a, 34 },
    { 0, 0 },
    { 0x72e2c35c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb42c365, 33 },
    { 0x5f12c366, 61 },
    { 0, 0 },
    { 0x7502c368, 61 },
    { 0, 0 },
    { 0x22c36a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc2c375, 43 },
    { 0x97e2c376, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242c379, 90 },
    { 0x5632c37a, 35 },
    { 0, 0 },
    { 0xea62c37c, 87 },
    { 0, 0 },
    { 0xe2e2c37e, 84 },
    { 0xbbf2c37f, 95 },
    { 0, 0 },
    { 0xd1e2c381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f2c389, 90 },
    { 0xee52c38a, 93 },
    { 0xcfc2c38b, 98 },
    { 0, 0 },
    { 0x1402c38d, 46 },
    { 0x3a72c38d, 98 },
    { 0xe7c2c38d, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d2c392, 78 },
    { 0, 0 },
    { 0x8f2c394, 91 },
    { 0, 0 },
    { 0x6182c396, 61 },
    { 0xd462c396, 34 },
    { 0xacd2c398, 100 },
    { 0xcc22c397, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5582c39e, 66 },
    { 0, 0 },
    { 0xec2c3a0, 98 },
    { 0xc122c3a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b42c3a7, 61 },
    { 0x69d2c3a7, 33 },
    { 0x3052c3a8, 90 },
    { 0x77a2c3a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2c3ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd172c3b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae62c3b7, 90 },
    { 0xc9f2c3b8, 23 },
    { 0, 0 },
    { 0x9aa2c3ba, 34 },
    { 0x3762c3bb, 55 },
    { 0, 0 },
    { 0x65b2c3bd, 90 },
    { 0x6892c3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd2c3c1, 33 },
    { 0x6a42c3c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc792c3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac2c3cb, 61 },
    { 0x3222c3cb, 84 },
    { 0xf692c3cc, 23 },
    { 0xa502c3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed62c3d4, 95 },
    { 0xc082c3d5, 55 },
    { 0xc3b2c3d6, 77 },
    { 0, 0 },
    { 0x3f92c3d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4732c3df, 95 },
    { 0xa4d2c3e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1642c3e4, 85 },
    { 0, 0 },
    { 0x3782c3e6, 33 },
    { 0, 0 },
    { 0xcd22c3e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac22c3ec, 98 },
    { 0, 0 },
    { 0x5dc2c3ee, 23 },
    { 0x9d2c3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc52c3f7, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2c3fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92c404, 55 },
    { 0xcf52c404, 23 },
    { 0x9052c404, 33 },
    { 0xfcb2c407, 61 },
    { 0, 0 },
    { 0xbd32c409, 52 },
    { 0x5732c409, 34 },
    { 0x4732c40b, 90 },
    { 0x242c40c, 23 },
    { 0x5b02c40c, 55 },
    { 0, 0 },
    { 0x2b72c40f, 61 },
    { 0, 0 },
    { 0x4842c411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e22c414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2c41b, 93 },
    { 0, 0 },
    { 0xcce2c41d, 84 },
    { 0x7022c41d, 55 },
    { 0xcc02c41d, 34 },
    { 0x67f2c420, 95 },
    { 0xa082c41e, 23 },
    { 0xd1a2c420, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532c425, 78 },
    { 0xa242c426, 90 },
    { 0, 0 },
    { 0x4762c428, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab2c42d, 55 },
    { 0xd582c42e, 95 },
    { 0x14c2c42f, 17 },
    { 0, 0 },
    { 0xf312c431, 75 },
    { 0x8ad2c432, 90 },
    { 0, 0 },
    { 0x61d2c434, 98 },
    { 0x8e22c434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb2c438, 90 },
    { 0xa612c439, 33 },
    { 0, 0 },
    { 0xc5f2c43b, 90 },
    { 0, 0 },
    { 0xda42c43d, 61 },
    { 0x5ac2c43d, 33 },
    { 0xef62c43f, 90 },
    { 0, 0 },
    { 0x33d2c441, 40 },
    { 0x3c22c442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d2c44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432c44f, 90 },
    { 0x1b72c450, 87 },
    { 0xc632c451, 33 },
    { 0x7c82c452, 33 },
    { 0x3112c453, 64 },
    { 0xe152c450, 76 },
    { 0x3e72c455, 75 },
    { 0, 0 },
    { 0x3382c457, 33 },
    { 0, 0 },
    { 0x19f2c459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d2c45f, 90 },
    { 0xdac2c460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d02c464, 76 },
    { 0x3122c465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332c468, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5732c46d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc92c471, 61 },
    { 0xc9e2c472, 61 },
    { 0xacc2c473, 100 },
    { 0xe462c474, 77 },
    { 0, 0 },
    { 0xd682c476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca22c479, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc2c47d, 98 },
    { 0xd2d2c47e, 90 },
    { 0, 0 },
    { 0xf7e2c480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92c483, 91 },
    { 0x6e42c484, 90 },
    { 0x26f2c484, 43 },
    { 0, 0 },
    { 0x99b2c487, 93 },
    { 0x8f92c487, 98 },
    { 0, 0 },
    { 0x6a72c48a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942c48f, 97 },
    { 0x4b62c48f, 98 },
    { 0, 0 },
    { 0xee12c492, 33 },
    { 0, 0 },
    { 0x3792c494, 78 },
    { 0x4292c494, 94 },
    { 0xc3f2c495, 19 },
    { 0, 0 },
    { 0x1922c498, 75 },
    { 0x8952c499, 90 },
    { 0x9a52c49a, 88 },
    { 0x8c32c498, 76 },
    { 0xc482c498, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b52c49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2c4aa, 95 },
    { 0xfc32c4ab, 95 },
    { 0x7d82c4ac, 93 },
    { 0xe12c4ad, 33 },
    { 0, 0 },
    { 0x91b2c4af, 91 },
    { 0x7752c4af, 23 },
    { 0x4a92c4af, 33 },
    { 0x7b42c4b2, 90 },
    { 0, 0 },
    { 0xe8b2c4b4, 90 },
    { 0x6092c4b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7822c4bf, 33 },
    { 0x77c2c4bf, 90 },
    { 0x3072c4c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2c4c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12c4ca, 43 },
    { 0, 0 },
    { 0x7f52c4cc, 61 },
    { 0, 0 },
    { 0x99a2c4ce, 34 },
    { 0x7002c4cf, 23 },
    { 0, 0 },
    { 0x7e42c4d1, 61 },
    { 0, 0 },
    { 0x1cf2c4d3, 93 },
    { 0xf532c4d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c2c4d8, 17 },
    { 0xe5c2c4d8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad2c4dc, 95 },
    { 0xa392c4dc, 98 },
    { 0xea92c4dc, 84 },
    { 0x68a2c4df, 55 },
    { 0x34f2c4df, 76 },
    { 0xa02c4df, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12c4e6, 46 },
    { 0x75a2c4e7, 23 },
    { 0xef2c4e8, 98 },
    { 0xee82c4e9, 34 },
    { 0xb412c4e8, 33 },
    { 0, 0 },
    { 0x4642c4ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa822c4f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2c4f5, 90 },
    { 0xb22c4f5, 33 },
    { 0x16a2c4f7, 90 },
    { 0x35e2c4f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8382c4fc, 90 },
    { 0, 0 },
    { 0xc412c4fe, 61 },
    { 0xa8a2c4ff, 61 },
    { 0, 0 },
    { 0x55b2c501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172c506, 61 },
    { 0x2962c507, 68 },
    { 0x67b2c506, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72c50b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2c50f, 33 },
    { 0, 0 },
    { 0x892c511, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5322c515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d2c51d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b92c520, 61 },
    { 0, 0 },
    { 0x3242c522, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e2c525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582c529, 23 },
    { 0xea12c529, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302c52d, 61 },
    { 0x4332c52e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2c532, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe2c536, 99 },
    { 0, 0 },
    { 0xc112c538, 99 },
    { 0xb8b2c538, 76 },
    { 0x66a2c53a, 43 },
    { 0xea62c53b, 90 },
    { 0xdce2c539, 33 },
    { 0x4542c539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2c542, 95 },
    { 0xeac2c542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52c546, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b2c54b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e2c54e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2c553, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a22c556, 98 },
    { 0, 0 },
    { 0x3202c558, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad2c55c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d2c562, 61 },
    { 0x6f82c563, 84 },
    { 0x8a22c564, 93 },
    { 0x6292c564, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc052c56c, 46 },
    { 0, 0 },
    { 0x79b2c56e, 23 },
    { 0x2da2c56f, 95 },
    { 0x1502c56f, 84 },
    { 0, 0 },
    { 0x9b72c572, 90 },
    { 0xec12c572, 90 },
    { 0x3dd2c572, 88 },
    { 0xd222c575, 93 },
    { 0x7ef2c575, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722c580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7382c583, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d2c586, 90 },
    { 0xcdf2c587, 98 },
    { 0xf192c588, 45 },
    { 0x9592c588, 90 },
    { 0x6102c587, 76 },
    { 0x3f22c589, 43 },
    { 0, 0 },
    { 0xf452c58d, 65 },
    { 0, 0 },
    { 0xd3d2c58f, 55 },
    { 0x3352c58f, 33 },
    { 0xf7f2c591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2c599, 61 },
    { 0xb982c59a, 93 },
    { 0, 0 },
    { 0xfbf2c59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2c5a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac2c5a6, 98 },
    { 0x1d02c5a7, 76 },
    { 0x7c82c5a7, 55 },
    { 0x9372c5a7, 68 },
    { 0, 0 },
    { 0x7352c5ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b2c5b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3502c5b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492c5b8, 91 },
    { 0xec72c5b9, 61 },
    { 0x9272c5b9, 66 },
    { 0xa752c5ba, 68 },
    { 0x7972c5bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222c5c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d2c5c5, 23 },
    { 0, 0 },
    { 0x5c72c5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe582c5cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8932c5d0, 61 },
    { 0, 0 },
    { 0x4192c5d2, 61 },
    { 0x8bd2c5d2, 45 },
    { 0x6182c5d4, 23 },
    { 0x8ab2c5d5, 43 },
    { 0x34d2c5d2, 90 },
    { 0xd332c5d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52c5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1672c5e2, 95 },
    { 0, 0 },
    { 0xcf32c5e4, 90 },
    { 0x19e2c5e4, 33 },
    { 0x27b2c5e6, 84 },
    { 0x8f42c5e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce22c5ec, 66 },
    { 0xe3a2c5ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8162c5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb522c5f5, 84 },
    { 0x8d52c5f6, 93 },
    { 0x28a2c5f6, 98 },
    { 0, 0 },
    { 0x9432c5f9, 36 },
    { 0x62c2c5fa, 23 },
    { 0x152c5f9, 33 },
    { 0xe382c5fc, 88 },
    { 0x2932c5fd, 90 },
    { 0xe3e2c5fe, 90 },
    { 0xe7b2c5fc, 91 },
    { 0x9952c5ff, 90 },
    { 0x2fa2c601, 61 },
    { 0xf8a2c5fc, 55 },
    { 0x4b82c603, 33 },
    { 0xa6a2c603, 80 },
    { 0x2d2c605, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5732c608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f2c60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea22c60e, 23 },
    { 0, 0 },
    { 0xa662c610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4972c615, 95 },
    { 0, 0 },
    { 0xc312c617, 95 },
    { 0xb5a2c618, 61 },
    { 0x6b52c619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5262c61f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2c623, 33 },
    { 0, 0 },
    { 0xaf12c625, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2742c62c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a2c635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2752c639, 43 },
    { 0xf542c63a, 61 },
    { 0x182c63a, 76 },
    { 0xd782c63c, 55 },
    { 0x3f32c63a, 76 },
    { 0xcad2c63e, 34 },
    { 0x3872c63e, 96 },
    { 0x3f92c639, 33 },
    { 0x4e72c639, 33 },
    { 0xda82c642, 88 },
    { 0x3772c63d, 61 },
    { 0xb042c644, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c2c647, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b2c64b, 87 },
    { 0x4182c64b, 76 },
    { 0x1d22c64b, 23 },
    { 0, 0 },
    { 0xc702c64f, 61 },
    { 0xb342c64f, 33 },
    { 0x68e2c651, 61 },
    { 0x5292c652, 34 },
    { 0xac2c651, 34 },
    { 0x1a72c653, 75 },
    { 0x7492c653, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a72c65d, 76 },
    { 0, 0 },
    { 0xfab2c65f, 90 },
    { 0xac62c660, 76 },
    { 0x9ca2c65f, 93 },
    { 0, 0 },
    { 0x7e2c663, 43 },
    { 0x1d02c663, 98 },
    { 0x6a42c665, 100 },
    { 0xe1b2c665, 93 },
    { 0xd0b2c667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f2c66b, 66 },
    { 0, 0 },
    { 0x3a22c66d, 96 },
    { 0x1ea2c66e, 65 },
    { 0x72b2c66d, 98 },
    { 0x7c2c66e, 23 },
    { 0x3e72c671, 68 },
    { 0x47d2c672, 23 },
    { 0xf052c671, 23 },
    { 0, 0 },
    { 0xf462c675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022c681, 33 },
    { 0, 0 },
    { 0xcb62c683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab12c68b, 76 },
    { 0x1a12c68c, 90 },
    { 0x6732c68d, 72 },
    { 0x4502c68e, 90 },
    { 0x52e2c68d, 34 },
    { 0x10f2c68d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d2c69a, 100 },
    { 0x7aa2c69b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c12c69f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732c6a2, 98 },
    { 0xcf92c6a3, 55 },
    { 0x72d2c6a3, 90 },
    { 0x1e02c6a5, 43 },
    { 0xf9d2c6a6, 23 },
    { 0x4f52c6a7, 17 },
    { 0x9f52c6a8, 66 },
    { 0xb112c6a9, 33 },
    { 0, 0 },
    { 0xac82c6ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2432c6af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf2c6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f22c6ba, 99 },
    { 0x4d62c6ba, 78 },
    { 0xda52c6bc, 83 },
    { 0xb192c6bd, 95 },
    { 0x9752c6bc, 33 },
    { 0x6a72c6bf, 98 },
    { 0, 0 },
    { 0x2152c6c1, 90 },
    { 0xb0c2c6c1, 61 },
    { 0x51a2c6c3, 90 },
    { 0x4912c6c3, 33 },
    { 0x8c32c6c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee2c6cb, 23 },
    { 0, 0 },
    { 0xdee2c6cd, 55 },
    { 0xa42c6ce, 95 },
    { 0xea62c6cf, 84 },
    { 0, 0 },
    { 0xfc62c6d1, 23 },
    { 0, 0 },
    { 0x3912c6d3, 95 },
    { 0xd5d2c6d4, 52 },
    { 0, 0 },
    { 0x5962c6d6, 95 },
    { 0xeda2c6d7, 56 },
    { 0xe8d2c6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902c6de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42c6e3, 93 },
    { 0x7422c6e4, 98 },
    { 0x23f2c6e5, 90 },
    { 0x9372c6e5, 23 },
    { 0x7fe2c6e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e2c6ed, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2c6f0, 46 },
    { 0xbbf2c6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2c6f4, 34 },
    { 0xb2d2c6f5, 61 },
    { 0xa82c6f4, 61 },
    { 0, 0 },
    { 0x8952c6f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a02c6fb, 61 },
    { 0xfa02c6fb, 90 },
    { 0, 0 },
    { 0xef62c6fe, 88 },
    { 0, 0 },
    { 0x9682c700, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd92c707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402c70d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5122c711, 61 },
    { 0, 0 },
    { 0xbe2c713, 66 },
    { 0, 0 },
    { 0x5252c715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2c71a, 61 },
    { 0xb302c71b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772c71e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2c723, 90 },
    { 0, 0 },
    { 0x69e2c725, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262c729, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2862c72c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c731, 66 },
    { 0, 0 },
    { 0x2b2c733, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c2c736, 34 },
    { 0, 0 },
    { 0x8612c738, 98 },
    { 0x6a62c738, 23 },
    { 0x3cc2c73a, 23 },
    { 0xf8e2c73b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe202c73e, 77 },
    { 0xeca2c73f, 99 },
    { 0x4cb2c73e, 99 },
    { 0x6822c741, 23 },
    { 0, 0 },
    { 0x8e02c743, 33 },
    { 0x4dd2c744, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d52c747, 95 },
    { 0x17b2c748, 55 },
    { 0, 0 },
    { 0x5e2c74a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc2c74d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8432c750, 100 },
    { 0, 0 },
    { 0x58c2c752, 61 },
    { 0x6af2c753, 91 },
    { 0xa4d2c754, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d2c759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6332c75d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca82c764, 98 },
    { 0x3a02c765, 90 },
    { 0x3892c766, 55 },
    { 0xfd02c767, 75 },
    { 0x6ad2c764, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed22c76b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5982c76e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e2c771, 98 },
    { 0x8232c772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc462c777, 23 },
    { 0, 0 },
    { 0x582c779, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f2c77c, 90 },
    { 0, 0 },
    { 0x3192c77e, 23 },
    { 0xa8e2c77e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac2c782, 91 },
    { 0xdd22c783, 90 },
    { 0x6882c782, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b2c787, 99 },
    { 0x6112c788, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92c790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab2c797, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2c79c, 98 },
    { 0xb8d2c79d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72c7a0, 90 },
    { 0x1082c7a0, 76 },
    { 0xa8c2c7a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb12c7a6, 55 },
    { 0x8cd2c7a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a2c7aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba2c7ae, 100 },
    { 0, 0 },
    { 0x4602c7b0, 95 },
    { 0x1d82c7b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c7b5, 33 },
    { 0x69c2c7b5, 90 },
    { 0xc2c2c7b6, 33 },
    { 0x6732c7b8, 33 },
    { 0x83b2c7b9, 77 },
    { 0x6472c7b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd52c7c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb052c7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2c7cb, 23 },
    { 0, 0 },
    { 0xbfa2c7cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc832c7d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e22c7dd, 55 },
    { 0xb3a2c7de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2c7e3, 95 },
    { 0, 0 },
    { 0x75f2c7e5, 90 },
    { 0, 0 },
    { 0x71b2c7e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2c7ed, 56 },
    { 0, 0 },
    { 0x7d2c7ef, 33 },
    { 0x17f2c7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b2c7f6, 95 },
    { 0x8492c7f6, 81 },
    { 0xb4b2c7f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c2c7fe, 90 },
    { 0xaaf2c7ff, 97 },
    { 0xf512c7ff, 95 },
    { 0, 0 },
    { 0x79e2c802, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe552c806, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5192c80c, 98 },
    { 0x1d72c80d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692c812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb302c818, 93 },
    { 0x94b2c818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452c81f, 76 },
    { 0xbe92c820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b2c823, 64 },
    { 0x6f92c824, 98 },
    { 0x34a2c823, 98 },
    { 0, 0 },
    { 0xfda2c827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2c82a, 100 },
    { 0x5b12c82b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2c831, 87 },
    { 0, 0 },
    { 0xbca2c833, 98 },
    { 0x74f2c834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4692c838, 23 },
    { 0xe6c2c838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082c83c, 23 },
    { 0x5cb2c83d, 90 },
    { 0, 0 },
    { 0xee72c83f, 33 },
    { 0, 0 },
    { 0x3022c841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52c845, 76 },
    { 0, 0 },
    { 0x48f2c847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2c84b, 46 },
    { 0, 0 },
    { 0x4602c84d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2c852, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c2c855, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2c858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd242c85c, 98 },
    { 0x80e2c85d, 75 },
    { 0, 0 },
    { 0xbfe2c85f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922c862, 95 },
    { 0, 0 },
    { 0x8542c864, 87 },
    { 0xb592c864, 91 },
    { 0x9702c865, 61 },
    { 0, 0 },
    { 0xced2c868, 61 },
    { 0xd7d2c869, 87 },
    { 0xca2c868, 48 },
    { 0, 0 },
    { 0xe5d2c86c, 38 },
    { 0, 0 },
    { 0x16b2c86e, 38 },
    { 0, 0 },
    { 0x53e2c870, 56 },
    { 0x31c2c871, 90 },
    { 0x1672c872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9262c876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b2c87d, 23 },
    { 0x6462c87e, 69 },
    { 0xa82c87d, 40 },
    { 0xe982c880, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c2c885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1192c88c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce2c893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152c896, 61 },
    { 0x3882c897, 77 },
    { 0, 0 },
    { 0x4d92c899, 61 },
    { 0x7d22c89a, 90 },
    { 0xd1a2c899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea32c89f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3042c8a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692c8aa, 90 },
    { 0, 0 },
    { 0x3392c8ac, 34 },
    { 0, 0 },
    { 0x1852c8ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c2c8b2, 87 },
    { 0xca22c8b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3432c8b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2362c8bc, 99 },
    { 0, 0 },
    { 0x69d2c8be, 61 },
    { 0x7d32c8bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc2c8c2, 76 },
    { 0x4252c8c2, 93 },
    { 0xe52c8c2, 98 },
    { 0x8562c8c3, 90 },
    { 0x5362c8c5, 67 },
    { 0xa742c8c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca62c8ca, 100 },
    { 0x5452c8cb, 61 },
    { 0, 0 },
    { 0x4922c8cd, 87 },
    { 0x6702c8ce, 90 },
    { 0xa622c8cd, 90 },
    { 0x2732c8d0, 55 },
    { 0x4a2c8d1, 100 },
    { 0, 0 },
    { 0x6532c8d3, 81 },
    { 0xaa12c8d3, 43 },
    { 0, 0 },
    { 0xbfc2c8d6, 56 },
    { 0x2a12c8d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd12c8dc, 33 },
    { 0, 0 },
    { 0x4d12c8de, 99 },
    { 0xffe2c8de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e2c8e4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e2c8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe662c903, 46 },
    { 0xbc82c903, 33 },
    { 0xd492c903, 34 },
    { 0, 0 },
    { 0x2f42c907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82c90b, 98 },
    { 0x4d12c90c, 98 },
    { 0xf672c90d, 55 },
    { 0, 0 },
    { 0x8e72c90f, 55 },
    { 0x4632c90f, 33 },
    { 0x3ce2c911, 76 },
    { 0x7b92c912, 77 },
    { 0xf712c913, 23 },
    { 0x7872c914, 52 },
    { 0x7d2c915, 98 },
    { 0xdec2c916, 33 },
    { 0x55e2c917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa542c91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe052c923, 98 },
    { 0x3702c923, 34 },
    { 0xbc32c925, 55 },
    { 0xc892c924, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa312c929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2c92d, 33 },
    { 0xbea2c92e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2c932, 90 },
    { 0xb7d2c933, 84 },
    { 0, 0 },
    { 0xa932c935, 33 },
    { 0, 0 },
    { 0x4552c937, 33 },
    { 0x7b92c938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4492c93b, 76 },
    { 0x67d2c93c, 52 },
    { 0x4132c93d, 90 },
    { 0x25a2c93d, 91 },
    { 0xcd22c93f, 33 },
    { 0xd8a2c93f, 33 },
    { 0xca42c93f, 52 },
    { 0xcc52c940, 75 },
    { 0, 0 },
    { 0x17e2c944, 98 },
    { 0xf52c945, 77 },
    { 0xee32c946, 33 },
    { 0x5a82c945, 23 },
    { 0x1622c948, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952c94e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f2c951, 61 },
    { 0, 0 },
    { 0x8fe2c953, 55 },
    { 0, 0 },
    { 0x19d2c955, 90 },
    { 0x25e2c956, 100 },
    { 0x4072c956, 90 },
    { 0, 0 },
    { 0xb3d2c959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c2c95c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4892c961, 61 },
    { 0, 0 },
    { 0xfa2c963, 61 },
    { 0x8e2c964, 61 },
    { 0, 0 },
    { 0xe0b2c966, 67 },
    { 0x87e2c967, 55 },
    { 0, 0 },
    { 0x332c969, 55 },
    { 0xe762c969, 61 },
    { 0xad32c96b, 34 },
    { 0x77e2c96a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c52c972, 90 },
    { 0x6d82c973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5592c976, 95 },
    { 0xf52c976, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2c983, 93 },
    { 0x4902c984, 77 },
    { 0x2b22c985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb2c98a, 78 },
    { 0x72f2c98b, 33 },
    { 0x9162c98c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2c98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12c993, 95 },
    { 0, 0 },
    { 0xc2f2c995, 90 },
    { 0x5f02c995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222c999, 23 },
    { 0x7fe2c99a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa2c99e, 90 },
    { 0, 0 },
    { 0x4dc2c9a0, 33 },
    { 0x1742c9a1, 95 },
    { 0, 0 },
    { 0xda42c9a3, 34 },
    { 0, 0 },
    { 0x2192c9a5, 90 },
    { 0xb8c2c9a6, 91 },
    { 0, 0 },
    { 0x1de2c9a8, 23 },
    { 0xe6e2c9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe872c9af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2c9b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce62c9b5, 34 },
    { 0xab72c9b6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e2c9b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4132c9c1, 23 },
    { 0x3d72c9c2, 33 },
    { 0x6822c9c3, 76 },
    { 0xbfc2c9c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3362c9c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d02c9ca, 33 },
    { 0xab2c9ca, 95 },
    { 0x1012c9cc, 33 },
    { 0xd522c9cd, 23 },
    { 0x8f32c9ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c2c9d5, 33 },
    { 0, 0 },
    { 0xf192c9d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2c9dd, 23 },
    { 0x6892c9dd, 61 },
    { 0xeac2c9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d2c9e2, 61 },
    { 0x3f92c9e3, 83 },
    { 0x7582c9e2, 33 },
    { 0xca22c9e5, 98 },
    { 0xef42c9e6, 46 },
    { 0x52c2c9e2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe72c9ec, 98 },
    { 0xa7d2c9ec, 17 },
    { 0x6932c9ed, 34 },
    { 0x8522c9ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2c9f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd662c9fb, 97 },
    { 0, 0 },
    { 0x8142c9fd, 40 },
    { 0xc802c9fe, 61 },
    { 0x7072c9fd, 98 },
    { 0x3e42c9ff, 98 },
    { 0xa6a2ca01, 91 },
    { 0xe342ca01, 98 },
    { 0x6232ca03, 91 },
    { 0xf8d2ca04, 33 },
    { 0x9742ca04, 98 },
    { 0x4852ca06, 46 },
    { 0x7802ca07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe162ca14, 90 },
    { 0, 0 },
    { 0x73f2ca16, 90 },
    { 0xa122ca17, 61 },
    { 0xb0d2ca17, 33 },
    { 0x8152ca17, 33 },
    { 0x3e12ca18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32ca23, 61 },
    { 0x6f12ca24, 76 },
    { 0xa102ca25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb222ca28, 43 },
    { 0xe0f2ca29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a52ca2e, 61 },
    { 0, 0 },
    { 0xa302ca30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42ca35, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52ca39, 43 },
    { 0xe712ca39, 101 },
    { 0x58d2ca3b, 33 },
    { 0x53e2ca3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112ca40, 78 },
    { 0, 0 },
    { 0xa482ca42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac2ca45, 65 },
    { 0x1a12ca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x212ca49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca4c, 76 },
    { 0x7082ca4d, 34 },
    { 0xd42ca4e, 76 },
    { 0xa642ca4f, 61 },
    { 0, 0 },
    { 0x3582ca51, 61 },
    { 0, 0 },
    { 0x3122ca53, 93 },
    { 0x6f02ca53, 94 },
    { 0x6542ca55, 61 },
    { 0, 0 },
    { 0xf152ca57, 76 },
    { 0x33a2ca58, 90 },
    { 0x95d2ca57, 61 },
    { 0x8a42ca57, 61 },
    { 0x25e2ca5b, 90 },
    { 0x87e2ca5b, 81 },
    { 0x6ad2ca59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2ca60, 61 },
    { 0xfae2ca61, 33 },
    { 0xd432ca62, 95 },
    { 0, 0 },
    { 0xe302ca64, 91 },
    { 0, 0 },
    { 0xb252ca66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012ca6a, 61 },
    { 0, 0 },
    { 0x2022ca6c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e2ca6f, 33 },
    { 0, 0 },
    { 0x3a02ca71, 93 },
    { 0xf982ca72, 95 },
    { 0, 0 },
    { 0x6032ca74, 33 },
    { 0x6b12ca75, 98 },
    { 0x51e2ca76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ca7b, 84 },
    { 0, 0 },
    { 0x6532ca7d, 75 },
    { 0xbf62ca7e, 99 },
    { 0x9e12ca7f, 93 },
    { 0, 0 },
    { 0xd292ca81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e2ca8b, 61 },
    { 0x6b02ca8c, 80 },
    { 0xe422ca8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda2ca97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6752ca9e, 78 },
    { 0, 0 },
    { 0x57b2caa0, 43 },
    { 0x1f92caa1, 33 },
    { 0x7bd2caa1, 23 },
    { 0, 0 },
    { 0xc392caa4, 90 },
    { 0, 0 },
    { 0xa9a2caa6, 23 },
    { 0xae82caa7, 23 },
    { 0x2cd2caa8, 98 },
    { 0xd532caa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012caac, 23 },
    { 0, 0 },
    { 0xc642caae, 95 },
    { 0xf892caaf, 34 },
    { 0xdce2cab0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b2cab6, 98 },
    { 0, 0 },
    { 0xa692cab8, 98 },
    { 0, 0 },
    { 0xc4f2caba, 90 },
    { 0x4492cabb, 33 },
    { 0, 0 },
    { 0xfc82cabd, 90 },
    { 0x2132cabe, 91 },
    { 0xe472cabe, 61 },
    { 0, 0 },
    { 0xc3f2cac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb2cac4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b62caca, 95 },
    { 0x7362cacb, 43 },
    { 0x6b52cacc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3662cad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e2cadd, 90 },
    { 0x5a12cade, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72cae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3682caeb, 98 },
    { 0x4542caeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692caf0, 61 },
    { 0x2472caf1, 90 },
    { 0x8a2caf2, 66 },
    { 0xa8c2caf3, 93 },
    { 0, 0 },
    { 0xeb02caf5, 33 },
    { 0, 0 },
    { 0x7512caf7, 61 },
    { 0x7b82caf8, 78 },
    { 0x9972caf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd222cafe, 90 },
    { 0x5042cafe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52cb04, 46 },
    { 0, 0 },
    { 0x48e2cb06, 61 },
    { 0, 0 },
    { 0x62a2cb08, 52 },
    { 0x4b32cb09, 34 },
    { 0xc922cb09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a2cb0d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd082cb19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632cb22, 61 },
    { 0, 0 },
    { 0x9bf2cb24, 23 },
    { 0x48b2cb25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2cb2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad2cb2d, 33 },
    { 0xc832cb2e, 93 },
    { 0, 0 },
    { 0xc7a2cb30, 90 },
    { 0, 0 },
    { 0x6de2cb32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe2cb37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb2cb3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932cb48, 90 },
    { 0x8402cb49, 90 },
    { 0x4b22cb4a, 23 },
    { 0xc582cb4a, 90 },
    { 0x2092cb4c, 43 },
    { 0, 0 },
    { 0x8c92cb4e, 61 },
    { 0, 0 },
    { 0x3a92cb50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d2cb53, 90 },
    { 0xae62cb53, 61 },
    { 0xd732cb55, 52 },
    { 0xa872cb54, 98 },
    { 0xb412cb56, 91 },
    { 0xc842cb58, 87 },
    { 0x9732cb58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2cb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb22cb64, 61 },
    { 0xe862cb65, 61 },
    { 0x322cb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d2cb6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2cb77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f2cb7d, 95 },
    { 0, 0 },
    { 0x18e2cb7f, 90 },
    { 0xbae2cb80, 61 },
    { 0, 0 },
    { 0x13a2cb82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b42cb85, 23 },
    { 0, 0 },
    { 0x5302cb87, 85 },
    { 0x3e32cb88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262cb8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb742cb93, 78 },
    { 0xcf82cb94, 98 },
    { 0xfe82cb95, 100 },
    { 0xd7a2cb96, 98 },
    { 0x70f2cb97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x302cb9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1042cb9e, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f2cba3, 90 },
    { 0, 0 },
    { 0x3802cba5, 99 },
    { 0, 0 },
    { 0x18f2cba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8292cbab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6432cbaf, 90 },
    { 0xe212cbb0, 90 },
    { 0, 0 },
    { 0x3722cbb2, 36 },
    { 0x79c2cbb2, 61 },
    { 0x2562cbb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2cbb7, 61 },
    { 0x4c02cbb7, 100 },
    { 0x51e2cbb9, 23 },
    { 0xad12cbba, 88 },
    { 0x6862cbbb, 95 },
    { 0, 0 },
    { 0xb4d2cbbd, 90 },
    { 0, 0 },
    { 0x16c2cbbf, 34 },
    { 0x1822cbc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a2cbc6, 95 },
    { 0x1a62cbc7, 91 },
    { 0, 0 },
    { 0x80f2cbc9, 43 },
    { 0x41d2cbca, 93 },
    { 0xedd2cbcb, 33 },
    { 0xc582cbcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692cbd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7432cbd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2cbd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8422cbda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2032cbde, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2cbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2cbeb, 93 },
    { 0x6712cbec, 61 },
    { 0x5892cbed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f92cbf1, 43 },
    { 0, 0 },
    { 0x8c02cbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372cbf6, 33 },
    { 0x6a72cbf7, 52 },
    { 0xcd92cbf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f2cbfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9562cc01, 34 },
    { 0x5d82cc01, 90 },
    { 0, 0 },
    { 0xb7b2cc04, 98 },
    { 0x8b12cc05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f2cc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2cc0b, 84 },
    { 0, 0 },
    { 0x4b62cc0d, 77 },
    { 0x82b2cc0e, 61 },
    { 0, 0 },
    { 0xe5d2cc10, 23 },
    { 0, 0 },
    { 0x1bf2cc12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a2cc17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd852cc1a, 55 },
    { 0, 0 },
    { 0x9d72cc1c, 17 },
    { 0x4f2cc1d, 34 },
    { 0xfb52cc1c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2cc22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3222cc27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2672cc2b, 90 },
    { 0xd152cc2c, 90 },
    { 0x2e62cc2d, 78 },
    { 0xf3e2cc2c, 34 },
    { 0xe812cc2e, 90 },
    { 0x8292cc30, 34 },
    { 0x3552cc30, 85 },
    { 0x83c2cc2c, 91 },
    { 0x4532cc31, 19 },
    { 0, 0 },
    { 0xa8e2cc35, 23 },
    { 0xbae2cc35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2cc39, 23 },
    { 0, 0 },
    { 0x8aa2cc3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3952cc3e, 23 },
    { 0x6d2cc3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd362cc42, 98 },
    { 0x4e92cc43, 75 },
    { 0x96c2cc44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3082cc4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1462cc4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2cc51, 55 },
    { 0x9012cc52, 90 },
    { 0, 0 },
    { 0x7c82cc54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2cc58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e32cc5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb12cc62, 36 },
    { 0x17b2cc63, 61 },
    { 0x2412cc64, 90 },
    { 0, 0 },
    { 0xfeb2cc66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2cc6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d2cc6d, 78 },
    { 0, 0 },
    { 0x3552cc6f, 90 },
    { 0, 0 },
    { 0x79e2cc71, 43 },
    { 0x7dc2cc71, 61 },
    { 0, 0 },
    { 0x76b2cc74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf2cc78, 46 },
    { 0xcbc2cc78, 95 },
    { 0, 0 },
    { 0xd0e2cc7b, 61 },
    { 0, 0 },
    { 0x13f2cc7d, 90 },
    { 0xf392cc7e, 85 },
    { 0, 0 },
    { 0x6082cc80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e92cc84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e2cc8b, 88 },
    { 0xdd92cc8b, 96 },
    { 0x6c52cc8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2482cc95, 23 },
    { 0x3a22cc95, 98 },
    { 0xee32cc97, 90 },
    { 0xdf72cc98, 23 },
    { 0x4ef2cc98, 34 },
    { 0xda32cc9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd2cc9d, 23 },
    { 0x98f2cc9d, 85 },
    { 0, 0 },
    { 0x5d72cca0, 90 },
    { 0, 0 },
    { 0x8542cca2, 52 },
    { 0xd762cca3, 90 },
    { 0xaa82cca2, 23 },
    { 0x47d2cca2, 90 },
    { 0x6b22cca6, 91 },
    { 0x5a52cca3, 34 },
    { 0, 0 },
    { 0xb5e2cca9, 98 },
    { 0, 0 },
    { 0x7ef2ccab, 23 },
    { 0x3202ccac, 99 },
    { 0x5ad2ccad, 34 },
    { 0xde92ccae, 98 },
    { 0x3e42ccaf, 33 },
    { 0xc672ccad, 98 },
    { 0, 0 },
    { 0x99c2ccb2, 34 },
    { 0xe62ccb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2ccb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832ccb9, 90 },
    { 0x762ccba, 33 },
    { 0, 0 },
    { 0x3b62ccbc, 23 },
    { 0x7da2ccbd, 90 },
    { 0xc4e2ccbc, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x5872ccc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda62ccc4, 84 },
    { 0, 0 },
    { 0x66a2ccc6, 67 },
    { 0xb7d2ccc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d82ccca, 33 },
    { 0, 0 },
    { 0x7212cccc, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7952ccd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592ccd6, 66 },
    { 0, 0 },
    { 0xba62ccd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3302ccdc, 77 },
    { 0xd892ccdd, 61 },
    { 0, 0 },
    { 0x99c2ccdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a2cce2, 33 },
    { 0x1132cce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462cce8, 93 },
    { 0xdb32cce8, 33 },
    { 0xe92cce8, 33 },
    { 0xfa52cce9, 33 },
    { 0x3592ccec, 61 },
    { 0x7bc2cceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7152ccf1, 61 },
    { 0xcf12ccf2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02ccf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f2ccf9, 61 },
    { 0, 0 },
    { 0x6e22ccfb, 33 },
    { 0x7312ccfc, 87 },
    { 0x1d62ccfd, 90 },
    { 0x51a2ccfc, 19 },
    { 0, 0 },
    { 0xa012cd00, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22cd05, 55 },
    { 0, 0 },
    { 0x3ac2cd07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f2cd0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2cd1b, 90 },
    { 0, 0 },
    { 0x88d2cd1d, 95 },
    { 0xc6c2cd1e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2cd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c52cd25, 43 },
    { 0x2e32cd26, 33 },
    { 0xc9f2cd27, 99 },
    { 0, 0 },
    { 0xd942cd29, 61 },
    { 0xed22cd2a, 90 },
    { 0, 0 },
    { 0xd2e2cd2c, 76 },
    { 0xd962cd2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2cd30, 55 },
    { 0x1182cd31, 90 },
    { 0xc092cd31, 61 },
    { 0xc072cd33, 43 },
    { 0x7eb2cd30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982cd3b, 93 },
    { 0xaa82cd3c, 100 },
    { 0x7fe2cd3d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc962cd44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e2cd47, 91 },
    { 0x4a2cd48, 95 },
    { 0x3512cd49, 43 },
    { 0xedf2cd47, 78 },
    { 0xdf2cd4b, 66 },
    { 0xda42cd49, 33 },
    { 0, 0 },
    { 0x54f2cd4e, 100 },
    { 0, 0 },
    { 0x5532cd50, 98 },
    { 0, 0 },
    { 0x86c2cd52, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b2cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2cd5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22cd5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e2cd62, 23 },
    { 0x77a2cd62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca52cd6a, 61 },
    { 0, 0 },
    { 0x7312cd6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f02cd72, 76 },
    { 0, 0 },
    { 0x5b92cd74, 66 },
    { 0x90e2cd75, 90 },
    { 0x9902cd74, 99 },
    { 0xf0d2cd74, 33 },
    { 0x3c62cd74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd552cd80, 23 },
    { 0xb22cd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42cd84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb2cd89, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c2cd8e, 93 },
    { 0xe7c2cd8f, 43 },
    { 0x6652cd90, 90 },
    { 0, 0 },
    { 0xcb62cd92, 95 },
    { 0xc862cd93, 93 },
    { 0x47e2cd93, 23 },
    { 0, 0 },
    { 0x3182cd96, 80 },
    { 0, 0 },
    { 0x1ed2cd98, 90 },
    { 0x8862cd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592cd9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52cda0, 34 },
    { 0, 0 },
    { 0x7b52cda2, 55 },
    { 0, 0 },
    { 0x5462cda4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a2cda9, 34 },
    { 0x2672cdaa, 42 },
    { 0xb232cda9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb742cdae, 77 },
    { 0x6c32cdae, 33 },
    { 0x9ec2cdb0, 49 },
    { 0, 0 },
    { 0xda22cdb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed12cdb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2cdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a02cdbd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db2cdc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42cdc7, 90 },
    { 0x11d2cdc7, 90 },
    { 0, 0 },
    { 0xd702cdca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3502cdcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4532cdd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd962cdd9, 66 },
    { 0, 0 },
    { 0x55a2cddb, 65 },
    { 0xfc2cddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c82cde1, 98 },
    { 0, 0 },
    { 0xb9d2cde3, 33 },
    { 0xb242cde4, 34 },
    { 0, 0 },
    { 0x5442cde6, 78 },
    { 0x54e2cde7, 61 },
    { 0x542cde8, 100 },
    { 0x2212cde9, 81 },
    { 0x472cde9, 68 },
    { 0xb842cdea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d22cdee, 51 },
    { 0, 0 },
    { 0x85d2cdf0, 85 },
    { 0, 0 },
    { 0x7762cdf2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7592cdf5, 80 },
    { 0x9ed2cdf6, 76 },
    { 0xd602cdf6, 76 },
    { 0x5202cdf7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f2cdfe, 43 },
    { 0, 0 },
    { 0x4352ce00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ce0a, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2542ce0e, 85 },
    { 0x6c52ce0f, 61 },
    { 0xd6f2ce0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d2ce14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942ce18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d32ce1c, 61 },
    { 0x1f52ce1c, 23 },
    { 0xf792ce1e, 90 },
    { 0xda02ce1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e82ce25, 61 },
    { 0xff92ce26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf612ce2a, 78 },
    { 0, 0 },
    { 0x9592ce2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6472ce30, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e42ce35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2ce39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb862ce48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552ce4d, 76 },
    { 0x9e12ce4e, 76 },
    { 0, 0 },
    { 0xa032ce50, 52 },
    { 0x4162ce51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7562ce55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952ce58, 33 },
    { 0xd7e2ce59, 23 },
    { 0xdb12ce59, 61 },
    { 0x42d2ce5b, 33 },
    { 0x6dd2ce5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de2ce60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe562ce64, 95 },
    { 0x952ce65, 65 },
    { 0xbf92ce66, 61 },
    { 0xd602ce65, 43 },
    { 0xd5a2ce68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ce6b, 61 },
    { 0xd712ce6b, 90 },
    { 0, 0 },
    { 0x51c2ce6e, 90 },
    { 0x1d42ce6f, 98 },
    { 0x1d2ce70, 33 },
    { 0x35e2ce71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6392ce74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb02ce78, 99 },
    { 0, 0 },
    { 0x3ad2ce7a, 95 },
    { 0, 0 },
    { 0x70f2ce7c, 23 },
    { 0x812ce7c, 76 },
    { 0xb442ce7e, 61 },
    { 0x5292ce7e, 98 },
    { 0x9d42ce7c, 99 },
    { 0x53e2ce7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf712ce8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc132ce93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4212cea1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2cea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc2ceaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42cec0, 95 },
    { 0x5c92cec1, 90 },
    { 0x2ab2cec2, 98 },
    { 0x1482cec0, 66 },
    { 0x76a2cec4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f32cecb, 34 },
    { 0xe572cecc, 61 },
    { 0, 0 },
    { 0x6822cece, 33 },
    { 0x4a92cecf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642ced6, 98 },
    { 0, 0 },
    { 0x4ba2ced8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc42cedb, 95 },
    { 0x3272cedc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402cee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2cee3, 33 },
    { 0xf882cee3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b72cee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2ceea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4992ceed, 33 },
    { 0, 0 },
    { 0x3cb2ceef, 78 },
    { 0xd112ceef, 46 },
    { 0x2b62ceef, 95 },
    { 0xa652cef2, 78 },
    { 0x4ab2cef2, 99 },
    { 0, 0 },
    { 0x2082cef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c2cefd, 84 },
    { 0xc0d2cefd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2cf01, 77 },
    { 0xa2d2cf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8452cf07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7392cf0d, 90 },
    { 0xe1d2cf0e, 61 },
    { 0, 0 },
    { 0x1202cf10, 100 },
    { 0xcca2cf10, 43 },
    { 0x6e52cf12, 61 },
    { 0x7b92cf10, 100 },
    { 0x6432cf14, 43 },
    { 0x8642cf13, 90 },
    { 0xe402cf14, 66 },
    { 0x3682cf15, 81 },
    { 0x4502cf17, 61 },
    { 0x2462cf19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa212cf23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd42cf29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1102cf2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f2cf31, 61 },
    { 0xa632cf31, 33 },
    { 0x752cf33, 90 },
    { 0x86d2cf34, 95 },
    { 0xd322cf35, 23 },
    { 0xc5c2cf36, 98 },
    { 0xa8d2cf37, 78 },
    { 0xcd62cf32, 98 },
    { 0x6fd2cf39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242cf3c, 61 },
    { 0x9262cf3c, 95 },
    { 0x9312cf3e, 76 },
    { 0x13c2cf3d, 84 },
    { 0, 0 },
    { 0xcdb2cf41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a62cf45, 46 },
    { 0xcb92cf46, 33 },
    { 0x4382cf47, 93 },
    { 0, 0 },
    { 0x49e2cf49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a2cf4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902cf51, 98 },
    { 0x16a2cf51, 33 },
    { 0xb212cf51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92cf5c, 88 },
    { 0xf9e2cf5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d22cf60, 76 },
    { 0x3292cf61, 90 },
    { 0xafb2cf62, 90 },
    { 0x96a2cf63, 66 },
    { 0x662cf64, 91 },
    { 0x5ba2cf65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2cf6a, 34 },
    { 0xffa2cf6b, 33 },
    { 0, 0 },
    { 0xfbb2cf6d, 33 },
    { 0x9a92cf6d, 90 },
    { 0x21c2cf6d, 90 },
    { 0x4452cf70, 90 },
    { 0, 0 },
    { 0xb522cf72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2cf76, 61 },
    { 0, 0 },
    { 0x2c12cf78, 64 },
    { 0x4b92cf79, 64 },
    { 0, 0 },
    { 0xc202cf7b, 81 },
    { 0x13a2cf7c, 93 },
    { 0x2592cf7d, 33 },
    { 0xa772cf7d, 33 },
    { 0x2db2cf7d, 99 },
    { 0x89d2cf7d, 78 },
    { 0x7132cf7f, 61 },
    { 0xb572cf7c, 19 },
    { 0xc8f2cf83, 61 },
    { 0x9ed2cf83, 52 },
    { 0xf5b2cf85, 95 },
    { 0, 0 },
    { 0x3702cf87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f2cf8d, 34 },
    { 0xbba2cf8e, 43 },
    { 0x2542cf8f, 33 },
    { 0x5402cf90, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa2cf93, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52cf98, 93 },
    { 0x7e62cf99, 33 },
    { 0xc4a2cf9a, 38 },
    { 0x8402cf98, 61 },
    { 0xe652cf9c, 33 },
    { 0xc102cf9d, 36 },
    { 0x38e2cf9e, 67 },
    { 0x7d72cf9f, 93 },
    { 0x4102cfa0, 90 },
    { 0x9f2cfa1, 33 },
    { 0x4b82cf9b, 91 },
    { 0xdd82cfa3, 77 },
    { 0x5eb2cfa4, 33 },
    { 0xc212cfa3, 76 },
    { 0, 0 },
    { 0x3fc2cfa7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf12cfae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2cfb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6052cfbb, 98 },
    { 0x3562cfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e02cfc2, 90 },
    { 0x5dd2cfc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b32cfcb, 95 },
    { 0x75a2cfcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd22cfcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd862cfd3, 78 },
    { 0x3192cfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b2cfdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee2cfe5, 76 },
    { 0, 0 },
    { 0xddb2cfe7, 84 },
    { 0x7c92cfe8, 66 },
    { 0, 0 },
    { 0xae52cfea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9462cfee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362cff2, 90 },
    { 0xa392cff2, 55 },
    { 0xf312cff3, 95 },
    { 0, 0 },
    { 0xa8e2cff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942cffc, 85 },
    { 0xe1f2cffc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b92d001, 85 },
    { 0xb2d2d002, 90 },
    { 0xf142d001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da2d007, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42d00c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752d010, 90 },
    { 0xde72d010, 95 },
    { 0xc1e2d012, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea2d018, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c2d01b, 99 },
    { 0x3652d01c, 52 },
    { 0x7272d01b, 98 },
    { 0xe6b2d01e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a2d02d, 23 },
    { 0, 0 },
    { 0x4e72d02f, 33 },
    { 0x6c42d030, 95 },
    { 0xa472d030, 23 },
    { 0x56e2d032, 61 },
    { 0x25a2d033, 95 },
    { 0x91d2d033, 46 },
    { 0x26b2d032, 93 },
    { 0x1592d036, 95 },
    { 0xcc62d036, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc312d03b, 53 },
    { 0x1c2d03c, 90 },
    { 0x1132d03b, 17 },
    { 0, 0 },
    { 0x9a82d03f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582d042, 90 },
    { 0, 0 },
    { 0x4ae2d044, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3602d047, 91 },
    { 0xb432d048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2d04b, 90 },
    { 0xecc2d04b, 90 },
    { 0, 0 },
    { 0x6122d04e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e82d056, 81 },
    { 0, 0 },
    { 0xedb2d058, 61 },
    { 0, 0 },
    { 0x352d05a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d05e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2d061, 90 },
    { 0xa0b2d062, 90 },
    { 0x35a2d063, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2d06b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5892d071, 90 },
    { 0, 0 },
    { 0x3652d073, 95 },
    { 0x54b2d074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee2d07a, 33 },
    { 0xcaf2d07b, 61 },
    { 0x9cb2d07a, 76 },
    { 0x4452d07d, 98 },
    { 0x6f32d07a, 23 },
    { 0xb2f2d07f, 95 },
    { 0x21a2d07f, 61 },
    { 0x8bf2d07b, 33 },
    { 0, 0 },
    { 0xde62d083, 64 },
    { 0x2362d084, 90 },
    { 0x4bd2d085, 80 },
    { 0x8652d086, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xb532d089, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2142d08f, 55 },
    { 0x2232d08f, 34 },
    { 0x8582d091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782d095, 90 },
    { 0xeea2d095, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332d09c, 56 },
    { 0x90d2d09c, 98 },
    { 0x1742d09c, 90 },
    { 0x7292d09f, 23 },
    { 0x2172d0a0, 84 },
    { 0x6022d0a0, 95 },
    { 0x7682d0a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e52d0a5, 61 },
    { 0xd6b2d0a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e12d0ab, 95 },
    { 0, 0 },
    { 0x9902d0ad, 23 },
    { 0x9102d0ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed2d0b2, 75 },
    { 0x4292d0b2, 34 },
    { 0x8482d0b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb2d0b7, 67 },
    { 0x6962d0b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3052d0c0, 34 },
    { 0x6882d0c0, 90 },
    { 0x24a2d0c0, 33 },
    { 0x7982d0c3, 61 },
    { 0x9852d0c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922d0c7, 69 },
    { 0, 0 },
    { 0x89b2d0c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82d0d1, 38 },
    { 0, 0 },
    { 0x2962d0d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a2d0d6, 33 },
    { 0, 0 },
    { 0x23e2d0d8, 87 },
    { 0, 0 },
    { 0xe3d2d0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552d0de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e2d0e1, 17 },
    { 0, 0 },
    { 0x2872d0e3, 52 },
    { 0, 0 },
    { 0x23c2d0e5, 85 },
    { 0x7402d0e6, 33 },
    { 0xa262d0e6, 90 },
    { 0x1e62d0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce2d0ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca72d0f7, 95 },
    { 0, 0 },
    { 0xf672d0f9, 61 },
    { 0, 0 },
    { 0x2b82d0fb, 55 },
    { 0x9192d0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b42d0ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62d102, 84 },
    { 0, 0 },
    { 0x4ac2d104, 84 },
    { 0xbd32d104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be2d108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d12d10e, 98 },
    { 0, 0 },
    { 0x3222d110, 52 },
    { 0xf4a2d111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae2d117, 90 },
    { 0x5e52d118, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e2d11f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462d125, 90 },
    { 0x9fe2d126, 90 },
    { 0xc1a2d125, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa352d12a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe082d12f, 95 },
    { 0, 0 },
    { 0x80e2d131, 76 },
    { 0xeef2d132, 95 },
    { 0x382d132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332d137, 95 },
    { 0xfac2d138, 95 },
    { 0xfde2d138, 61 },
    { 0, 0 },
    { 0x2fa2d13b, 33 },
    { 0x38c2d13c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e22d140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f02d147, 65 },
    { 0xd3a2d147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec2d14b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9962d150, 23 },
    { 0x2fd2d151, 90 },
    { 0x2a52d152, 76 },
    { 0x2332d153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292d15a, 33 },
    { 0xeb32d15b, 93 },
    { 0xc7c2d15c, 90 },
    { 0x6662d15d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8052d161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee2d164, 68 },
    { 0x75c2d164, 85 },
    { 0x8372d164, 33 },
    { 0xa572d167, 52 },
    { 0xf012d167, 23 },
    { 0x6dc2d168, 23 },
    { 0xc492d169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c2d16d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8522d170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d2d173, 23 },
    { 0x7f2d173, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a2d177, 23 },
    { 0xa582d178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b92d17b, 87 },
    { 0, 0 },
    { 0xe722d17d, 93 },
    { 0x5452d17d, 99 },
    { 0x5c52d17f, 55 },
    { 0xfae2d180, 40 },
    { 0x6b12d17d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b2d184, 33 },
    { 0x7e62d185, 61 },
    { 0x5262d186, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2d191, 95 },
    { 0xd382d192, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd102d196, 17 },
    { 0xec92d196, 33 },
    { 0x5b42d198, 52 },
    { 0xcfc2d197, 55 },
    { 0x95f2d19a, 43 },
    { 0x1a82d199, 65 },
    { 0xe332d19c, 33 },
    { 0xa32d19d, 75 },
    { 0x4422d19a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc562d1a1, 61 },
    { 0x5422d1a1, 66 },
    { 0x9da2d1a3, 65 },
    { 0x4762d1a3, 43 },
    { 0x4242d1a5, 23 },
    { 0xec52d1a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf2d1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562d1b2, 90 },
    { 0x4f22d1b2, 95 },
    { 0, 0 },
    { 0x9562d1b5, 33 },
    { 0xaa12d1b6, 55 },
    { 0xd4e2d1b6, 33 },
    { 0x6242d1b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082d1bb, 95 },
    { 0x5b32d1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8992d1c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e2d1c4, 95 },
    { 0x56f2d1c4, 84 },
    { 0x2d1c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e2d1ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972d1d0, 90 },
    { 0xc6e2d1d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd472d1d7, 91 },
    { 0, 0 },
    { 0x5202d1d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f2d1dc, 33 },
    { 0x4d72d1dc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe932d1e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a2d1e3, 84 },
    { 0, 0 },
    { 0x852d1e5, 99 },
    { 0x99a2d1e6, 33 },
    { 0xc902d1e5, 33 },
    { 0x4b92d1e7, 91 },
    { 0xdb32d1e6, 76 },
    { 0xa7c2d1e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b52d1ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e72d1f7, 91 },
    { 0, 0 },
    { 0x682d1f9, 95 },
    { 0x66a2d1fa, 91 },
    { 0xc9a2d1fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2d205, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022d209, 33 },
    { 0xd6e2d209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d42d210, 95 },
    { 0, 0 },
    { 0x4242d212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2d215, 55 },
    { 0x3f72d216, 90 },
    { 0xe652d217, 43 },
    { 0x8902d217, 23 },
    { 0xf682d218, 95 },
    { 0xfd82d216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2d21f, 88 },
    { 0xa142d21f, 81 },
    { 0xf762d220, 33 },
    { 0, 0 },
    { 0x362d223, 33 },
    { 0xd2d224, 33 },
    { 0x7712d223, 55 },
    { 0x6802d226, 33 },
    { 0xf772d226, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252d22b, 67 },
    { 0xf332d22c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8772d230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb262d235, 97 },
    { 0x9cd2d235, 91 },
    { 0, 0 },
    { 0xf682d238, 23 },
    { 0, 0 },
    { 0x1e32d23a, 76 },
    { 0x942d23a, 33 },
    { 0, 0 },
    { 0x1d02d23d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b2d240, 95 },
    { 0xab82d241, 33 },
    { 0x2d02d242, 77 },
    { 0, 0 },
    { 0xad02d244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2d248, 90 },
    { 0x40f2d248, 98 },
    { 0xfb62d24a, 43 },
    { 0x2ec2d24b, 61 },
    { 0x2652d24c, 43 },
    { 0x8452d24c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb2d254, 90 },
    { 0, 0 },
    { 0x8802d256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c2d25a, 46 },
    { 0, 0 },
    { 0xdc52d25c, 90 },
    { 0x1e22d25c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b2d262, 95 },
    { 0x81e2d262, 98 },
    { 0, 0 },
    { 0x8712d265, 76 },
    { 0, 0 },
    { 0x4762d267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe2d26a, 46 },
    { 0xbfd2d26a, 90 },
    { 0, 0 },
    { 0xb902d26d, 33 },
    { 0x7ae2d26d, 61 },
    { 0, 0 },
    { 0x5132d270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a2d275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd2d279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52d27d, 90 },
    { 0, 0 },
    { 0xe9d2d27f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca52d282, 52 },
    { 0x7312d283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc272d287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb842d28b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d62d28e, 93 },
    { 0x4662d28f, 84 },
    { 0x5f2d290, 62 },
    { 0x4a32d290, 33 },
    { 0xd712d292, 61 },
    { 0x2062d290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f2d298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2d29c, 23 },
    { 0, 0 },
    { 0x95a2d29e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412d2a2, 91 },
    { 0xb12d2a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2d2a6, 23 },
    { 0xaad2d2a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b2d2aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d02d2ad, 90 },
    { 0x7572d2ae, 91 },
    { 0xe082d2af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df2d2b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322d2b6, 93 },
    { 0x8242d2b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22d2bb, 90 },
    { 0, 0 },
    { 0xed82d2bd, 84 },
    { 0, 0 },
    { 0xba52d2bf, 43 },
    { 0, 0 },
    { 0x99c2d2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a2d2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8872d2cf, 81 },
    { 0xc682d2d0, 43 },
    { 0x25e2d2cf, 61 },
    { 0xd282d2d0, 98 },
    { 0xba92d2d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002d2d7, 90 },
    { 0, 0 },
    { 0x1dd2d2d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2d2dc, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2d2e4, 76 },
    { 0x1992d2e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc802d2f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6712d2f5, 55 },
    { 0xfe32d2f5, 90 },
    { 0, 0 },
    { 0xd132d2f8, 85 },
    { 0x6602d2f9, 90 },
    { 0x26d2d2fa, 33 },
    { 0x92e2d2f9, 34 },
    { 0x70b2d2fc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c2d300, 75 },
    { 0x4b92d301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752d305, 90 },
    { 0x9e82d306, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de2d311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa372d314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca82d318, 95 },
    { 0x90f2d318, 98 },
    { 0, 0 },
    { 0xb4f2d31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a12d31f, 55 },
    { 0xa032d320, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa2d323, 87 },
    { 0x2f12d324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122d328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf42d32b, 75 },
    { 0xe392d32b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5152d332, 66 },
    { 0, 0 },
    { 0x4152d334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e92d337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf82d33a, 76 },
    { 0xcba2d33b, 90 },
    { 0x34f2d33b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1572d342, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322d345, 95 },
    { 0x4512d346, 76 },
    { 0, 0 },
    { 0xd292d348, 98 },
    { 0, 0 },
    { 0x6312d34a, 23 },
    { 0xf922d34a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1732d350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6992d354, 98 },
    { 0, 0 },
    { 0x9642d356, 43 },
    { 0, 0 },
    { 0x9bb2d358, 33 },
    { 0xf192d358, 23 },
    { 0xff72d358, 90 },
    { 0x4422d358, 98 },
    { 0x84e2d35c, 90 },
    { 0x5f22d35c, 75 },
    { 0x6d32d35e, 66 },
    { 0xe092d35f, 33 },
    { 0xd92d35f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b2d366, 87 },
    { 0x9422d366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c62d36b, 33 },
    { 0x7532d36b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c72d36f, 98 },
    { 0, 0 },
    { 0xae72d371, 76 },
    { 0x4342d372, 90 },
    { 0xa822d373, 33 },
    { 0x6142d372, 83 },
    { 0xfb72d372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e2d378, 91 },
    { 0x6252d379, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02d37c, 77 },
    { 0x4e82d37d, 98 },
    { 0xe652d37e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2d381, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5242d384, 90 },
    { 0, 0 },
    { 0x2a32d386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af2d389, 95 },
    { 0x2a12d38a, 93 },
    { 0x55d2d38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da2d397, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2d39a, 98 },
    { 0, 0 },
    { 0x1112d39c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2d3a1, 85 },
    { 0, 0 },
    { 0x9c92d3a3, 98 },
    { 0, 0 },
    { 0xdb52d3a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a2d3a9, 23 },
    { 0, 0 },
    { 0x2a12d3ab, 52 },
    { 0, 0 },
    { 0xfd62d3ad, 81 },
    { 0, 0 },
    { 0x4bc2d3af, 95 },
    { 0x13e2d3b0, 23 },
    { 0xe5c2d3b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192d3b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f22d3b7, 93 },
    { 0xb522d3b7, 98 },
    { 0xf682d3b8, 95 },
    { 0xeb2d3ba, 84 },
    { 0xb952d3bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee42d3be, 34 },
    { 0, 0 },
    { 0xb582d3c0, 43 },
    { 0xec52d3c0, 61 },
    { 0x8392d3c2, 98 },
    { 0, 0 },
    { 0x57f2d3c4, 61 },
    { 0x51d2d3c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf542d3c9, 43 },
    { 0, 0 },
    { 0xb2e2d3cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb212d3ce, 85 },
    { 0, 0 },
    { 0x7a32d3d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172d3d3, 99 },
    { 0x82e2d3d4, 90 },
    { 0x9f12d3d5, 76 },
    { 0xb402d3d3, 33 },
    { 0xf162d3d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7412d3da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd2d3dd, 23 },
    { 0, 0 },
    { 0xea02d3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92d3e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a2d3e9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f2d3ec, 68 },
    { 0x3132d3ec, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd2d3f2, 90 },
    { 0, 0 },
    { 0x3c32d3f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d3f7, 33 },
    { 0, 0 },
    { 0xab42d3f9, 74 },
    { 0x2b02d3fa, 85 },
    { 0x2152d3fa, 34 },
    { 0x23e2d3fc, 95 },
    { 0xc6a2d3fd, 61 },
    { 0x6142d3fa, 33 },
    { 0x50e2d3fb, 34 },
    { 0, 0 },
    { 0x7ba2d401, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d408, 76 },
    { 0x4422d408, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2d40e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812d411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b2d416, 76 },
    { 0x15f2d417, 84 },
    { 0xa612d417, 61 },
    { 0xbc72d417, 23 },
    { 0x9392d41a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae2d423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8132d426, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd2d42b, 61 },
    { 0x97d2d42b, 61 },
    { 0, 0 },
    { 0x3302d42e, 81 },
    { 0xaa12d42f, 17 },
    { 0x1642d42e, 23 },
    { 0xa4f2d430, 61 },
    { 0x8af2d432, 61 },
    { 0x2652d432, 98 },
    { 0xff82d42e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9532d438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f72d43b, 87 },
    { 0x1ed2d43b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2d440, 91 },
    { 0x56a2d441, 66 },
    { 0, 0 },
    { 0xbc82d443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2d449, 76 },
    { 0xa632d449, 34 },
    { 0xebb2d44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd962d452, 46 },
    { 0, 0 },
    { 0xd862d454, 90 },
    { 0, 0 },
    { 0x8632d456, 34 },
    { 0x932d457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d22d45a, 61 },
    { 0, 0 },
    { 0xdd2d45c, 33 },
    { 0, 0 },
    { 0xbe52d45e, 95 },
    { 0x11d2d45f, 34 },
    { 0, 0 },
    { 0x92a2d461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc2d465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252d468, 90 },
    { 0xec52d469, 90 },
    { 0, 0 },
    { 0xb442d46b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2d46f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462d474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b62d47a, 52 },
    { 0x2db2d47a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2d480, 23 },
    { 0x2962d481, 98 },
    { 0x1a52d482, 76 },
    { 0x8cc2d483, 90 },
    { 0xf972d484, 17 },
    { 0xc4d2d481, 61 },
    { 0xccb2d484, 100 },
    { 0x6a22d485, 23 },
    { 0x4c52d488, 43 },
    { 0, 0 },
    { 0xaca2d48a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2d490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba2d493, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8882d497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2d49b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022d49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df2d4a2, 34 },
    { 0, 0 },
    { 0x3632d4a4, 43 },
    { 0x9ee2d4a5, 55 },
    { 0x8042d4a6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72d4a9, 42 },
    { 0x3b82d4aa, 78 },
    { 0x152d4aa, 5 },
    { 0xb6b2d4ac, 23 },
    { 0, 0 },
    { 0x5a52d4ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212d4b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492d4b7, 55 },
    { 0x1c42d4b7, 95 },
    { 0x46e2d4b7, 88 },
    { 0x6da2d4ba, 93 },
    { 0xaf72d4bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2d4c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d12d4c3, 84 },
    { 0x7452d4c4, 95 },
    { 0, 0 },
    { 0x3202d4c6, 98 },
    { 0, 0 },
    { 0x9c72d4c8, 98 },
    { 0x14f2d4c8, 90 },
    { 0, 0 },
    { 0x8942d4cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae2d4ce, 68 },
    { 0xbe52d4ce, 76 },
    { 0x7a2d4d0, 76 },
    { 0, 0 },
    { 0x3252d4d2, 61 },
    { 0, 0 },
    { 0xd082d4d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e82d4d9, 99 },
    { 0x3a42d4d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22d4e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972d4e4, 98 },
    { 0xaa12d4e5, 101 },
    { 0x8c92d4e6, 33 },
    { 0xbad2d4e7, 90 },
    { 0x9a72d4e5, 61 },
    { 0xb492d4e9, 95 },
    { 0x55f2d4ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2d4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2d4f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f82d4fb, 90 },
    { 0xa312d4fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6512d50e, 95 },
    { 0x2dc2d50f, 61 },
    { 0x9c2d510, 61 },
    { 0, 0 },
    { 0xaff2d512, 55 },
    { 0, 0 },
    { 0x6ea2d514, 99 },
    { 0xc52d514, 95 },
    { 0, 0 },
    { 0x4482d517, 90 },
    { 0xaaf2d518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422d520, 33 },
    { 0x7752d521, 90 },
    { 0, 0 },
    { 0x4ed2d523, 95 },
    { 0xe852d524, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce2d52e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde82d532, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x5542d535, 77 },
    { 0x8ed2d536, 23 },
    { 0xdc12d537, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2d53a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c2d542, 33 },
    { 0x1b32d542, 34 },
    { 0, 0 },
    { 0x3782d545, 98 },
    { 0xde32d545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe762d549, 95 },
    { 0xf9b2d54a, 33 },
    { 0, 0 },
    { 0x3e32d54c, 67 },
    { 0xb112d54d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e12d554, 33 },
    { 0xc3a2d555, 100 },
    { 0xd022d556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992d559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb2d55d, 98 },
    { 0xf5a2d55d, 95 },
    { 0x4542d55d, 100 },
    { 0, 0 },
    { 0xf8d2d561, 61 },
    { 0x38f2d562, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b2d565, 77 },
    { 0x8ce2d566, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf52d569, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2d56f, 98 },
    { 0, 0 },
    { 0xca02d571, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d576, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1132d579, 93 },
    { 0xd9d2d57a, 33 },
    { 0xbd22d57b, 65 },
    { 0, 0 },
    { 0x3f82d57d, 61 },
    { 0xea72d57e, 61 },
    { 0, 0 },
    { 0x2912d580, 33 },
    { 0xe7d2d581, 61 },
    { 0xde2d582, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b2d58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa582d590, 94 },
    { 0, 0 },
    { 0xf12d592, 66 },
    { 0xd242d593, 90 },
    { 0xd622d594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc802d598, 61 },
    { 0xca2d599, 23 },
    { 0x1982d59a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce2d5a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2122d5a4, 95 },
    { 0, 0 },
    { 0x8be2d5a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302d5af, 93 },
    { 0, 0 },
    { 0x13e2d5b1, 69 },
    { 0, 0 },
    { 0xda02d5b3, 33 },
    { 0x2b32d5b3, 95 },
    { 0x2482d5b5, 52 },
    { 0xb162d5b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c2d5c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6772d5c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd392d5cb, 34 },
    { 0x1a42d5cb, 98 },
    { 0, 0 },
    { 0xb3c2d5ce, 23 },
    { 0x42d5cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5532d5d2, 90 },
    { 0xca22d5d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8362d5d6, 84 },
    { 0x1132d5d6, 95 },
    { 0, 0 },
    { 0xfe52d5d9, 46 },
    { 0xe912d5d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272d5e1, 87 },
    { 0xc6e2d5e2, 33 },
    { 0x9dc2d5e3, 95 },
    { 0x5f62d5e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c52d5e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e2d5ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1532d5f2, 61 },
    { 0x3652d5f3, 38 },
    { 0x66d2d5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852d5f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd182d5fb, 52 },
    { 0x80b2d5fc, 92 },
    { 0, 0 },
    { 0x6fb2d5fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca2d602, 90 },
    { 0, 0 },
    { 0x8ed2d604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d72d608, 90 },
    { 0x5d92d609, 93 },
    { 0, 0 },
    { 0x2562d60b, 34 },
    { 0, 0 },
    { 0x4df2d60d, 55 },
    { 0x5372d60d, 95 },
    { 0x5342d60e, 95 },
    { 0, 0 },
    { 0x46e2d611, 55 },
    { 0x1262d612, 100 },
    { 0x9e82d613, 33 },
    { 0xa212d613, 98 },
    { 0x83d2d615, 23 },
    { 0x6c62d616, 81 },
    { 0, 0 },
    { 0xfcc2d618, 61 },
    { 0x25b2d619, 100 },
    { 0x9e82d61a, 31 },
    { 0x3d2d618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa2d61f, 98 },
    { 0, 0 },
    { 0x12f2d621, 33 },
    { 0x47c2d622, 61 },
    { 0x6602d623, 98 },
    { 0x70a2d621, 23 },
    { 0xbcf2d625, 46 },
    { 0, 0 },
    { 0x2522d627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8082d62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d22d632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb422d63b, 66 },
    { 0xc9f2d63c, 90 },
    { 0x6b12d63c, 90 },
    { 0xc6f2d63c, 61 },
    { 0xd4b2d63d, 23 },
    { 0x5792d63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252d644, 61 },
    { 0xa292d645, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff72d648, 100 },
    { 0x6af2d648, 33 },
    { 0xa882d64a, 33 },
    { 0xd112d64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632d653, 99 },
    { 0x8522d653, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef2d657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c42d65d, 85 },
    { 0x5b92d65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e2d663, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952d669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a92d672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2d67a, 40 },
    { 0, 0 },
    { 0x4d02d67c, 87 },
    { 0x5592d67c, 100 },
    { 0xd9a2d67e, 87 },
    { 0x7bf2d67f, 87 },
    { 0xfd92d67f, 46 },
    { 0x1532d681, 95 },
    { 0x1662d67c, 67 },
    { 0xa2e2d683, 46 },
    { 0x6b72d67e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb62d68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462d68f, 43 },
    { 0, 0 },
    { 0x8932d691, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be2d695, 61 },
    { 0x2222d695, 66 },
    { 0, 0 },
    { 0xe3c2d698, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d69b, 61 },
    { 0, 0 },
    { 0x7102d69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc02d6a5, 34 },
    { 0, 0 },
    { 0x96c2d6a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2d6af, 66 },
    { 0, 0 },
    { 0xa6a2d6b1, 33 },
    { 0x6582d6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2632d6ba, 81 },
    { 0xc512d6ba, 95 },
    { 0, 0 },
    { 0x1362d6bd, 98 },
    { 0x632d6be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2d6cd, 46 },
    { 0x3d72d6ce, 90 },
    { 0, 0 },
    { 0xa132d6d0, 23 },
    { 0xbb72d6d0, 33 },
    { 0x99c2d6d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632d6d8, 76 },
    { 0, 0 },
    { 0x1dd2d6da, 23 },
    { 0, 0 },
    { 0x9ee2d6dc, 33 },
    { 0x17f2d6dc, 61 },
    { 0x9b12d6dc, 43 },
    { 0xdbd2d6df, 90 },
    { 0x99c2d6df, 90 },
    { 0xb8d2d6e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb472d6e5, 90 },
    { 0, 0 },
    { 0x9032d6e7, 99 },
    { 0x55a2d6e7, 98 },
    { 0xaa32d6e9, 23 },
    { 0x4df2d6ea, 33 },
    { 0x7fb2d6eb, 98 },
    { 0x6e2d6eb, 34 },
    { 0x2732d6e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe502d6f0, 85 },
    { 0x5a22d6f1, 61 },
    { 0, 0 },
    { 0x302d6f3, 33 },
    { 0xb602d6f4, 33 },
    { 0x5812d6f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402d6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db2d6fb, 61 },
    { 0xa1f2d6fc, 95 },
    { 0x69f2d6fd, 61 },
    { 0x9be2d6fe, 98 },
    { 0x5a72d6ff, 81 },
    { 0x3822d700, 98 },
    { 0x7c22d6fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae2d704, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e2d709, 76 },
    { 0x11a2d70a, 95 },
    { 0x44f2d70b, 77 },
    { 0xba02d70a, 98 },
    { 0xe2b2d709, 75 },
    { 0xe8e2d70e, 55 },
    { 0x3b92d70f, 33 },
    { 0xbba2d710, 91 },
    { 0xe282d711, 23 },
    { 0xbd22d711, 34 },
    { 0x3b32d713, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2d719, 65 },
    { 0, 0 },
    { 0x5c02d71b, 52 },
    { 0xd762d71b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd372d720, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d724, 90 },
    { 0x2862d725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe302d729, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc762d72d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62d730, 51 },
    { 0, 0 },
    { 0xa3c2d732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a2d736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2d73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2d740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2d749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa2d74d, 66 },
    { 0x57e2d74e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b2d751, 33 },
    { 0, 0 },
    { 0x1172d753, 34 },
    { 0xc532d753, 61 },
    { 0x5ee2d754, 95 },
    { 0, 0 },
    { 0x212d757, 23 },
    { 0xf172d758, 76 },
    { 0x8572d759, 23 },
    { 0xf2d2d759, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc2d764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa402d768, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc812d76e, 33 },
    { 0x5952d76f, 98 },
    { 0xa8d2d76f, 34 },
    { 0x7f62d76f, 46 },
    { 0x6e52d76f, 76 },
    { 0x4ed2d773, 76 },
    { 0xa322d774, 99 },
    { 0xd802d775, 61 },
    { 0xe082d776, 61 },
    { 0xc252d777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c62d77a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e2d782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x452d785, 90 },
    { 0x2ae2d785, 61 },
    { 0x212d787, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a2d78b, 98 },
    { 0x41e2d78b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf502d792, 77 },
    { 0xf1b2d792, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2d796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd572d79b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452d7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae2d7a6, 100 },
    { 0x6df2d7a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72d7aa, 90 },
    { 0, 0 },
    { 0x65a2d7ac, 91 },
    { 0xed12d7ad, 90 },
    { 0, 0 },
    { 0x2f92d7af, 61 },
    { 0xef72d7b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6972d7b4, 85 },
    { 0xe322d7b5, 40 },
    { 0, 0 },
    { 0xc922d7b7, 34 },
    { 0xdf72d7b8, 19 },
    { 0x2d42d7b8, 61 },
    { 0x43f2d7ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a2d7c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b2d7cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2d7d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532d7da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f2d7e1, 91 },
    { 0x7812d7e2, 95 },
    { 0, 0 },
    { 0xa412d7e4, 61 },
    { 0xdd2d7e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d62d7e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc572d7ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf52d7f1, 90 },
    { 0xee92d7f2, 23 },
    { 0x4f72d7f2, 98 },
    { 0x42c2d7f4, 90 },
    { 0, 0 },
    { 0x76a2d7f6, 76 },
    { 0, 0 },
    { 0x54a2d7f8, 78 },
    { 0x3d92d7f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3462d7fe, 90 },
    { 0xbb42d7ff, 23 },
    { 0, 0 },
    { 0x402d801, 81 },
    { 0, 0 },
    { 0xea2d803, 43 },
    { 0x2aa2d804, 34 },
    { 0x7302d804, 23 },
    { 0xbef2d806, 95 },
    { 0, 0 },
    { 0xa102d808, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2d80b, 46 },
    { 0xe7d2d80b, 91 },
    { 0, 0 },
    { 0xbf2d80e, 88 },
    { 0, 0 },
    { 0xa222d810, 91 },
    { 0xeb62d811, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232d817, 91 },
    { 0x6202d817, 55 },
    { 0x3a92d817, 17 },
    { 0xaa02d81a, 96 },
    { 0xb692d817, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e2d81e, 76 },
    { 0x6b92d81f, 61 },
    { 0x4332d820, 23 },
    { 0x3432d81e, 66 },
    { 0, 0 },
    { 0x63d2d823, 23 },
    { 0xdcb2d823, 52 },
    { 0xbac2d825, 43 },
    { 0xd7f2d826, 33 },
    { 0xc272d825, 95 },
    { 0, 0 },
    { 0x1302d829, 55 },
    { 0x3032d82a, 76 },
    { 0x842d82b, 23 },
    { 0xe452d82b, 61 },
    { 0x4a72d82d, 81 },
    { 0x32e2d82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e02d834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552d837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2d83e, 90 },
    { 0x9c02d83f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92d844, 78 },
    { 0, 0 },
    { 0x9302d846, 75 },
    { 0, 0 },
    { 0xee52d848, 55 },
    { 0, 0 },
    { 0x2e32d84a, 23 },
    { 0, 0 },
    { 0x2312d84c, 68 },
    { 0xd252d84d, 93 },
    { 0xf132d84d, 55 },
    { 0xa0d2d84f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882d853, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc912d858, 33 },
    { 0xa1c2d859, 61 },
    { 0xb002d859, 34 },
    { 0xaad2d85b, 90 },
    { 0xbcf2d85c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012d85f, 61 },
    { 0, 0 },
    { 0xaac2d861, 95 },
    { 0xb072d862, 61 },
    { 0xf92d861, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af2d868, 95 },
    { 0xba62d868, 87 },
    { 0x1472d86a, 98 },
    { 0x30d2d86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2d876, 95 },
    { 0, 0 },
    { 0xd2a2d878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6982d87c, 33 },
    { 0xa3b2d87c, 61 },
    { 0, 0 },
    { 0x652d87f, 95 },
    { 0x57f2d87f, 95 },
    { 0, 0 },
    { 0xd192d882, 93 },
    { 0x9a22d882, 76 },
    { 0xd252d884, 78 },
    { 0xf6f2d885, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2d888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c82d88f, 55 },
    { 0, 0 },
    { 0x1ed2d891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf212d894, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9932d89c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2d8a1, 95 },
    { 0, 0 },
    { 0x9fe2d8a3, 55 },
    { 0x7472d8a4, 61 },
    { 0xfb02d8a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212d8aa, 61 },
    { 0xeb02d8ab, 90 },
    { 0, 0 },
    { 0xab22d8ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b22d8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02d8b9, 33 },
    { 0, 0 },
    { 0x1ea2d8bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2d8c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6212d8c6, 61 },
    { 0x3632d8c6, 33 },
    { 0x5e42d8c8, 52 },
    { 0, 0 },
    { 0xc3c2d8ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92d8cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62d8d8, 98 },
    { 0xec52d8d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb2d8dc, 98 },
    { 0x1fd2d8dc, 33 },
    { 0, 0 },
    { 0x7632d8df, 33 },
    { 0, 0 },
    { 0x5ac2d8e1, 23 },
    { 0x3b2d8e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca32d8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5132d8f2, 76 },
    { 0x3042d8f3, 36 },
    { 0xfa72d8f4, 95 },
    { 0x74d2d8f5, 81 },
    { 0xd1d2d8f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf712d8ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8412d90c, 95 },
    { 0, 0 },
    { 0x2052d90e, 90 },
    { 0x6032d90f, 90 },
    { 0x6ed2d910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf82d913, 61 },
    { 0x49d2d914, 46 },
    { 0xc612d915, 61 },
    { 0xfdb2d916, 43 },
    { 0x42c2d917, 61 },
    { 0xc3c2d917, 33 },
    { 0xf052d919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab2d91c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec52d91f, 88 },
    { 0x1a32d91f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db2d923, 90 },
    { 0x2442d924, 75 },
    { 0x82f2d925, 17 },
    { 0xc552d925, 37 },
    { 0x2042d927, 23 },
    { 0xf882d927, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2d92b, 95 },
    { 0x262d92c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f72d92f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922d933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f2d937, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc2d93c, 90 },
    { 0xe912d93d, 95 },
    { 0x1002d93e, 90 },
    { 0x4ae2d93f, 34 },
    { 0x41e2d93f, 33 },
    { 0x23a2d93f, 76 },
    { 0x1ff2d941, 84 },
    { 0, 0 },
    { 0x4732d944, 46 },
    { 0, 0 },
    { 0xb222d946, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe192d94a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5542d94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4392d951, 84 },
    { 0xe322d951, 95 },
    { 0x48d2d952, 100 },
    { 0, 0 },
    { 0x43f2d955, 90 },
    { 0xb442d956, 78 },
    { 0xc172d956, 76 },
    { 0x2692d957, 33 },
    { 0x8452d959, 100 },
    { 0xd682d95a, 23 },
    { 0x2e2d95b, 33 },
    { 0x1c2d95c, 33 },
    { 0, 0 },
    { 0x5032d95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa2d96b, 43 },
    { 0x9c42d96c, 61 },
    { 0x7ec2d96d, 23 },
    { 0x7d2d96e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd2d97e, 98 },
    { 0xf842d97f, 66 },
    { 0x8af2d980, 76 },
    { 0x42f2d981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412d984, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52d987, 33 },
    { 0x97e2d987, 78 },
    { 0x16d2d987, 80 },
    { 0x12b2d98a, 95 },
    { 0xc472d98b, 66 },
    { 0x3f82d98b, 61 },
    { 0xfc12d98d, 23 },
    { 0x95e2d98e, 95 },
    { 0, 0 },
    { 0x52e2d990, 76 },
    { 0xfec2d990, 61 },
    { 0, 0 },
    { 0x8942d993, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42d999, 95 },
    { 0, 0 },
    { 0x23d2d99b, 61 },
    { 0, 0 },
    { 0xe062d99d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02d9a1, 85 },
    { 0x20b2d9a2, 33 },
    { 0xa082d9a3, 61 },
    { 0xb002d9a4, 99 },
    { 0xfcc2d9a3, 61 },
    { 0x9842d9a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8942d9a9, 90 },
    { 0x272d9aa, 33 },
    { 0xb562d9a9, 95 },
    { 0xf42d9ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be2d9af, 67 },
    { 0x8892d9b0, 77 },
    { 0x2452d9b1, 85 },
    { 0x4ed2d9b2, 95 },
    { 0x552d9b3, 23 },
    { 0xda72d9b3, 33 },
    { 0, 0 },
    { 0xe0f2d9b6, 85 },
    { 0, 0 },
    { 0xb592d9b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb2d9be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe392d9c8, 99 },
    { 0x74d2d9c9, 34 },
    { 0x9232d9ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7502d9d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2d9d5, 33 },
    { 0xfa62d9d5, 100 },
    { 0xa772d9d7, 95 },
    { 0x5e02d9d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022d9dc, 90 },
    { 0xee52d9dd, 87 },
    { 0xf9e2d9dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e2d9e3, 23 },
    { 0, 0 },
    { 0x1672d9e5, 61 },
    { 0, 0 },
    { 0x7572d9e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed2d9ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b2d9ef, 95 },
    { 0x1522d9f0, 98 },
    { 0xe2d9f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f2d9f4, 33 },
    { 0xf492d9f5, 98 },
    { 0x8eb2d9f6, 95 },
    { 0x2112d9f4, 55 },
    { 0x5622d9f8, 43 },
    { 0, 0 },
    { 0x2b2d9fa, 76 },
    { 0x8f22d9fa, 90 },
    { 0xb1c2d9fc, 76 },
    { 0x5e12d9fd, 99 },
    { 0x932d9fe, 61 },
    { 0xc812d9ff, 34 },
    { 0x3532d9fa, 23 },
    { 0x832d9fd, 67 },
    { 0, 0 },
    { 0xe642da03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a2da09, 33 },
    { 0x1252da0a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2da18, 68 },
    { 0xdce2da19, 55 },
    { 0xd142da18, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1862da21, 28 },
    { 0x26f2da21, 49 },
    { 0x33c2da23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf352da26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d92da2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a2da2d, 91 },
    { 0xcc42da2e, 98 },
    { 0x4432da2f, 61 },
    { 0, 0 },
    { 0x5da2da31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4892da37, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6442da3d, 67 },
    { 0x952da3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf2da41, 33 },
    { 0xbf52da42, 33 },
    { 0x35b2da41, 98 },
    { 0x45b2da44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a2da47, 98 },
    { 0xfac2da48, 55 },
    { 0xf702da49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6912da4c, 33 },
    { 0, 0 },
    { 0x312da4e, 76 },
    { 0x3632da4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d2da52, 98 },
    { 0, 0 },
    { 0xa0d2da54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c32da5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2da63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf62da6a, 61 },
    { 0xbef2da6b, 98 },
    { 0, 0 },
    { 0x3442da6d, 23 },
    { 0xc882da6e, 90 },
    { 0x2fd2da6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702da73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7842da76, 95 },
    { 0x66e2da77, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d12da7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb72da83, 23 },
    { 0xd3b2da84, 90 },
    { 0x4e82da83, 33 },
    { 0xfb82da85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee2da8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172da8d, 61 },
    { 0xd0a2da8e, 66 },
    { 0xb872da8f, 35 },
    { 0x6e82da90, 43 },
    { 0x5ec2da91, 34 },
    { 0x502da92, 40 },
    { 0x1db2da93, 61 },
    { 0x4df2da90, 61 },
    { 0xef2da91, 93 },
    { 0x6862da8e, 33 },
    { 0x79b2da91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6302daa1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa82daae, 33 },
    { 0, 0 },
    { 0xc9b2dab0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c2dab3, 91 },
    { 0x6252dab3, 23 },
    { 0xa682dab4, 23 },
    { 0x1a12dab6, 23 },
    { 0, 0 },
    { 0xe9d2dab8, 61 },
    { 0x4482dab9, 55 },
    { 0xc2d2dab8, 95 },
    { 0, 0 },
    { 0xee92dabc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x652dabf, 55 },
    { 0x24c2dac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382dac4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee2daca, 76 },
    { 0x8552dacb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da2dad0, 23 },
    { 0, 0 },
    { 0x8c22dad2, 84 },
    { 0x7e52dad3, 91 },
    { 0xad72dad4, 33 },
    { 0xea02dad4, 76 },
    { 0x582dad5, 98 },
    { 0xf6f2dad7, 95 },
    { 0xa0e2dad8, 90 },
    { 0xbf52dad8, 98 },
    { 0xe0a2dada, 95 },
    { 0xbb12dadb, 47 },
    { 0x4772dadc, 33 },
    { 0, 0 },
    { 0x6eb2dade, 43 },
    { 0x8672dade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d2dae3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe172daee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd482daf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe52daf8, 78 },
    { 0x6fa2daf9, 23 },
    { 0, 0 },
    { 0x7662dafb, 17 },
    { 0x87c2dafc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1332daff, 99 },
    { 0, 0 },
    { 0x4db2db01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552db07, 23 },
    { 0, 0 },
    { 0xb732db09, 88 },
    { 0x7a72db0a, 75 },
    { 0x5dd2db0b, 52 },
    { 0x5af2db0b, 95 },
    { 0, 0 },
    { 0x4532db0e, 39 },
    { 0, 0 },
    { 0xb6a2db10, 66 },
    { 0, 0 },
    { 0xb9d2db12, 66 },
    { 0, 0 },
    { 0x77b2db14, 83 },
    { 0xd3d2db15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6412db18, 90 },
    { 0x4ff2db19, 33 },
    { 0x5632db19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602db1d, 65 },
    { 0x3132db1e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa342db22, 43 },
    { 0, 0 },
    { 0xe522db24, 100 },
    { 0x5152db24, 99 },
    { 0xa82db26, 99 },
    { 0xf722db26, 76 },
    { 0, 0 },
    { 0x1f72db29, 98 },
    { 0x3252db2a, 61 },
    { 0x43c2db2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x982db2e, 23 },
    { 0xb802db2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e2db37, 98 },
    { 0x8382db38, 98 },
    { 0, 0 },
    { 0xb202db3a, 55 },
    { 0x9682db3b, 65 },
    { 0x4902db3b, 61 },
    { 0x60d2db3d, 76 },
    { 0, 0 },
    { 0x9cf2db3f, 87 },
    { 0xbcd2db3f, 61 },
    { 0x13e2db41, 24 },
    { 0xcd2db42, 100 },
    { 0x2ba2db42, 33 },
    { 0xf162db44, 23 },
    { 0x3fe2db43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d2db48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a2db4f, 61 },
    { 0xdd22db50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad52db54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42db59, 23 },
    { 0x7b72db5a, 33 },
    { 0x2082db59, 43 },
    { 0xd802db5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9532db5f, 95 },
    { 0, 0 },
    { 0x84e2db61, 33 },
    { 0, 0 },
    { 0xd502db63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2db67, 61 },
    { 0xfb2db68, 61 },
    { 0x5902db67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b22db6d, 95 },
    { 0xfe12db6e, 90 },
    { 0x1ab2db6f, 75 },
    { 0xd02db6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed02db77, 67 },
    { 0xdcb2db78, 98 },
    { 0, 0 },
    { 0x2c92db7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52db7d, 23 },
    { 0, 0 },
    { 0xf5f2db7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea2db82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2db86, 40 },
    { 0, 0 },
    { 0x4892db88, 91 },
    { 0, 0 },
    { 0xc802db8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3612db8f, 33 },
    { 0xdd12db90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa882db94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c52db99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c62db9c, 77 },
    { 0x4bf2db9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382dba2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f2dba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22dba9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc2dbad, 61 },
    { 0xbc12dbad, 90 },
    { 0x8b02dbaf, 33 },
    { 0x6b82dbb0, 66 },
    { 0xb5e2dbb1, 76 },
    { 0, 0 },
    { 0x8d42dbb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1162dbb7, 61 },
    { 0x7272dbb8, 34 },
    { 0x1902dbb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752dbbd, 34 },
    { 0x25f2dbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e2dbc2, 34 },
    { 0x6cc2dbc3, 23 },
    { 0xa8f2dbc4, 91 },
    { 0xa232dbc5, 84 },
    { 0xb342dbc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a2dbc9, 76 },
    { 0x6da2dbc9, 99 },
    { 0, 0 },
    { 0x1f22dbcc, 43 },
    { 0x5972dbcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf072dbd0, 95 },
    { 0xab02dbd1, 95 },
    { 0xe642dbd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d2dbd5, 90 },
    { 0x23d2dbd6, 61 },
    { 0x67e2dbd7, 98 },
    { 0x32a2dbd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7402dbdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb072dbdf, 34 },
    { 0x1622dbe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3912dbe5, 75 },
    { 0, 0 },
    { 0x12e2dbe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7642dbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4502dbf1, 98 },
    { 0, 0 },
    { 0xbbe2dbf3, 98 },
    { 0xaa2dbf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2dbf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6332dbff, 95 },
    { 0, 0 },
    { 0x4922dc01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2dc04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72dc07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d2dc0d, 95 },
    { 0x45c2dc0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc2dc16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7502dc21, 61 },
    { 0xfb62dc22, 76 },
    { 0xfe82dc22, 68 },
    { 0x7992dc22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52dc2c, 90 },
    { 0x9972dc2d, 76 },
    { 0xa8a2dc2e, 33 },
    { 0xe582dc2f, 76 },
    { 0x3592dc2f, 61 },
    { 0x3a32dc2e, 90 },
    { 0xd0a2dc32, 23 },
    { 0x8602dc32, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2dc37, 90 },
    { 0x43e2dc38, 98 },
    { 0xce22dc37, 90 },
    { 0x43e2dc3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82dc3e, 95 },
    { 0xc3e2dc3f, 87 },
    { 0, 0 },
    { 0x9fa2dc41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c2dc48, 100 },
    { 0x6952dc49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c22dc4d, 100 },
    { 0xa792dc4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02dc54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9172dc58, 33 },
    { 0xfa62dc59, 43 },
    { 0, 0 },
    { 0x7f22dc5b, 23 },
    { 0, 0 },
    { 0x5512dc5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7872dc6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c2dc6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2dc73, 61 },
    { 0, 0 },
    { 0xf7c2dc75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf082dc80, 33 },
    { 0x74c2dc81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd12dc88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52dc8b, 95 },
    { 0xd2b2dc8b, 90 },
    { 0xa982dc8d, 33 },
    { 0x8b42dc8e, 55 },
    { 0x9292dc8f, 75 },
    { 0x78e2dc8f, 98 },
    { 0, 0 },
    { 0x46c2dc92, 93 },
    { 0, 0 },
    { 0xe432dc94, 85 },
    { 0x23a2dc95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822dc99, 55 },
    { 0x1fd2dc9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2dc9e, 67 },
    { 0x56e2dc9e, 95 },
    { 0, 0 },
    { 0xac82dca1, 33 },
    { 0x8e72dca2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb662dca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2dcab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2dcae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a2dcb3, 64 },
    { 0xd302dcb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c2dcba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf2dcbe, 81 },
    { 0x68e2dcbf, 95 },
    { 0x3b42dcbf, 33 },
    { 0, 0 },
    { 0x10c2dcc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb2dcca, 33 },
    { 0x1592dccb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2dcce, 33 },
    { 0x9672dcce, 83 },
    { 0x5132dcd0, 93 },
    { 0xb572dcd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b12dcd8, 55 },
    { 0xdd2dcd8, 56 },
    { 0, 0 },
    { 0x85c2dcdb, 42 },
    { 0, 0 },
    { 0xf152dcdd, 98 },
    { 0, 0 },
    { 0xf482dcdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c12dce4, 85 },
    { 0xbcb2dce5, 90 },
    { 0x6f2dce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca2dceb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2dcf0, 61 },
    { 0x352dcf1, 78 },
    { 0, 0 },
    { 0x72f2dcf3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72dcfc, 95 },
    { 0x34d2dcfd, 34 },
    { 0x7b42dcfd, 95 },
    { 0xac2dcfc, 100 },
    { 0xcb2dd00, 61 },
    { 0, 0 },
    { 0xfba2dd02, 43 },
    { 0x9d22dd03, 83 },
    { 0xe1f2dd03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c2dd09, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd612dd0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2dd11, 98 },
    { 0, 0 },
    { 0xc4a2dd13, 33 },
    { 0xf482dd14, 55 },
    { 0x4cd2dd15, 90 },
    { 0xe672dd15, 66 },
    { 0xb542dd17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32dd1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb312dd22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1212dd25, 98 },
    { 0xe992dd26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2392dd2f, 90 },
    { 0x6ed2dd2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa2dd33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6372dd38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2dd3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4782dd42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f22dd45, 33 },
    { 0x9102dd46, 55 },
    { 0xdb52dd46, 91 },
    { 0xef72dd48, 34 },
    { 0, 0 },
    { 0xb162dd4a, 55 },
    { 0x5ec2dd4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac2dd4e, 98 },
    { 0x9252dd4f, 76 },
    { 0x5c62dd4f, 23 },
    { 0x5462dd50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2dd56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd02dd61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3182dd64, 55 },
    { 0x2312dd65, 95 },
    { 0x4112dd66, 100 },
    { 0xf352dd67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942dd6a, 76 },
    { 0xe212dd6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52dd75, 5 },
    { 0, 0 },
    { 0x70b2dd77, 46 },
    { 0, 0 },
    { 0x6ca2dd79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e2dd7d, 33 },
    { 0x9d22dd7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d2dd83, 90 },
    { 0, 0 },
    { 0xba72dd85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf342dd8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac2dd8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf592dd92, 88 },
    { 0x16d2dd93, 98 },
    { 0, 0 },
    { 0xdbf2dd95, 23 },
    { 0x3892dd96, 90 },
    { 0x2c32dd97, 95 },
    { 0x5702dd97, 85 },
    { 0xaf62dd98, 55 },
    { 0x4d92dd97, 100 },
    { 0, 0 },
    { 0x97f2dd9c, 64 },
    { 0x3312dd9d, 75 },
    { 0, 0 },
    { 0xcf92dd9f, 76 },
    { 0xe1d2dda0, 80 },
    { 0x5102dda1, 61 },
    { 0xfed2dd9f, 98 },
    { 0xbfb2dd9f, 17 },
    { 0xe752dda4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662dda7, 52 },
    { 0, 0 },
    { 0xecb2dda9, 90 },
    { 0, 0 },
    { 0x8c22ddab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082ddaf, 76 },
    { 0x76c2ddaf, 52 },
    { 0, 0 },
    { 0x6272ddb2, 99 },
    { 0x9342ddb2, 76 },
    { 0xd222ddb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062ddb7, 67 },
    { 0x6c92ddb8, 55 },
    { 0x7892ddb9, 33 },
    { 0x5d72ddba, 55 },
    { 0xe12ddba, 84 },
    { 0xcd62ddba, 95 },
    { 0, 0 },
    { 0xd352ddbe, 90 },
    { 0xd262ddbf, 90 },
    { 0xa1f2ddc0, 33 },
    { 0x5e72ddc1, 100 },
    { 0, 0 },
    { 0xd792ddc3, 98 },
    { 0x4e62ddc4, 43 },
    { 0x75b2ddc5, 46 },
    { 0xefd2ddc3, 91 },
    { 0x97b2ddc3, 61 },
    { 0x3062ddc8, 35 },
    { 0x9b62ddc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6422ddcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2ddd4, 56 },
    { 0xd4b2ddd5, 98 },
    { 0, 0 },
    { 0x8462ddd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452dddb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52dde0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc442ddea, 65 },
    { 0, 0 },
    { 0xd62ddec, 90 },
    { 0, 0 },
    { 0x8672ddee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ddf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a2ddf8, 98 },
    { 0x5372ddf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52ddfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532de01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f12de04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac92de08, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592de0b, 33 },
    { 0, 0 },
    { 0x82c2de0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2de11, 61 },
    { 0x7662de12, 55 },
    { 0x63a2de11, 76 },
    { 0x6232de14, 90 },
    { 0x3242de14, 90 },
    { 0x1cf2de15, 98 },
    { 0, 0 },
    { 0x48f2de18, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b62de21, 55 },
    { 0xeac2de21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd2de25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302de28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c2de31, 90 },
    { 0, 0 },
    { 0xd2a2de33, 95 },
    { 0, 0 },
    { 0x1a02de35, 90 },
    { 0x4632de35, 78 },
    { 0x9032de36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb132de3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b42de45, 46 },
    { 0xa6a2de46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a2de4f, 93 },
    { 0x8eb2de4f, 33 },
    { 0xb912de4f, 76 },
    { 0, 0 },
    { 0xb7d2de53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8522de57, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2de63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552de67, 33 },
    { 0xb852de68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf642de6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4902de70, 98 },
    { 0, 0 },
    { 0xcdc2de72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf82de75, 64 },
    { 0x9072de76, 55 },
    { 0x7dd2de77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe122de7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb2de80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62de84, 46 },
    { 0, 0 },
    { 0x8112de86, 34 },
    { 0xbce2de87, 43 },
    { 0xb7c2de88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa542de8d, 90 },
    { 0x23a2de8d, 33 },
    { 0x6032de8f, 33 },
    { 0xf562de90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf762de93, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xd682de96, 90 },
    { 0xdd2de97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c2dea2, 33 },
    { 0xc62dea3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e2dea6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8802deaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2deae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x412deb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b2deb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2792deb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd432debb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c2debe, 56 },
    { 0xfa02debf, 52 },
    { 0x5562dec0, 23 },
    { 0x1c92dec1, 61 },
    { 0, 0 },
    { 0x9f12dec3, 76 },
    { 0xb0d2dec4, 61 },
    { 0x8dc2dec3, 61 },
    { 0x46e2dec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb52deca, 90 },
    { 0x4cf2deca, 34 },
    { 0, 0 },
    { 0xab12decd, 90 },
    { 0, 0 },
    { 0x6e42decf, 90 },
    { 0xf542decf, 90 },
    { 0xc402ded0, 34 },
    { 0x3932ded2, 46 },
    { 0x9122decf, 51 },
    { 0xb442ded1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab2ded9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a32dedd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b2dee1, 90 },
    { 0xeb82dee2, 46 },
    { 0xaf72dee2, 66 },
    { 0x2ae2dee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32deeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc2deee, 67 },
    { 0x11c2deef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb222def7, 68 },
    { 0x51f2def8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac2defb, 61 },
    { 0xac22defc, 33 },
    { 0x2c82defd, 23 },
    { 0x3f92defe, 61 },
    { 0xeb12deff, 90 },
    { 0x5222df00, 23 },
    { 0, 0 },
    { 0xe52df02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982df05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba2df0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf912df13, 75 },
    { 0x7b92df14, 33 },
    { 0xeae2df15, 66 },
    { 0xb222df15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582df1b, 95 },
    { 0, 0 },
    { 0x9f32df1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c2df20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec32df24, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6142df27, 55 },
    { 0x19c2df28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e42df2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8472df2e, 98 },
    { 0x5332df2e, 34 },
    { 0x7f32df2e, 33 },
    { 0, 0 },
    { 0xd622df32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde42df36, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc2df3c, 51 },
    { 0, 0 },
    { 0x5552df3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xef12df41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962df46, 81 },
    { 0x31a2df46, 23 },
    { 0, 0 },
    { 0x8522df49, 66 },
    { 0, 0 },
    { 0x38c2df4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22df50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b32df53, 81 },
    { 0x17c2df54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2df57, 43 },
    { 0xc1b2df57, 61 },
    { 0x2da2df59, 61 },
    { 0x5f72df5a, 88 },
    { 0xb9d2df59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2df66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8012df69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22df6d, 55 },
    { 0xa622df6e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d2df72, 43 },
    { 0xf882df73, 90 },
    { 0x1042df72, 34 },
    { 0, 0 },
    { 0x50f2df76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2df79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1082df7c, 33 },
    { 0xa822df7d, 95 },
    { 0xc652df7d, 95 },
    { 0x7472df7f, 17 },
    { 0x2f62df7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12df83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e2df86, 98 },
    { 0, 0 },
    { 0x33f2df88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda2df8b, 90 },
    { 0x4cd2df8c, 33 },
    { 0x39c2df8c, 43 },
    { 0xd6b2df8e, 93 },
    { 0, 0 },
    { 0xcc02df90, 68 },
    { 0x1822df91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582df96, 88 },
    { 0xb9d2df96, 98 },
    { 0xfe12df98, 33 },
    { 0x76b2df99, 95 },
    { 0x3322df9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5712df9e, 61 },
    { 0x7082df9f, 90 },
    { 0xcfb2df9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22dfa6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa702dfad, 55 },
    { 0xe1c2dfae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d2dfb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7152dfbc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8132dfc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9892dfca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3762dfcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca2dfd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb312dfda, 65 },
    { 0, 0 },
    { 0xa472dfdc, 23 },
    { 0, 0 },
    { 0xb3f2dfde, 95 },
    { 0xa722dfdf, 99 },
    { 0xbf22dfe0, 61 },
    { 0x852dfdf, 90 },
    { 0x4e42dfde, 95 },
    { 0xc8c2dfe0, 95 },
    { 0xd752dfe4, 61 },
    { 0, 0 },
    { 0x4cb2dfe6, 23 },
    { 0xddc2dfe7, 99 },
    { 0xc972dfe8, 61 },
    { 0x9e32dfe7, 46 },
    { 0x3c22dfe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b2dfed, 93 },
    { 0x1872dfed, 66 },
    { 0x6632dfef, 100 },
    { 0xc892dff0, 95 },
    { 0x7e32dff1, 55 },
    { 0x7f92dfef, 95 },
    { 0xe302dff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa152dff6, 33 },
    { 0xf852dff7, 99 },
    { 0x4c82dff7, 98 },
    { 0xc782dff9, 46 },
    { 0x5fe2dffa, 78 },
    { 0x1e92dffb, 90 },
    { 0x1a22dffc, 43 },
    { 0xcee2dffd, 90 },
    { 0xa732dff8, 23 },
    { 0xf492dffb, 61 },
    { 0x6a22dffe, 76 },
    { 0, 0 },
    { 0xaa62e002, 88 },
    { 0x3762e002, 61 },
    { 0xa452e004, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812e007, 33 },
    { 0x6032e007, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a2e00c, 84 },
    { 0xf302e00d, 98 },
    { 0xa822e00c, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f2e012, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192e018, 23 },
    { 0, 0 },
    { 0xcc02e01a, 98 },
    { 0x3662e01b, 76 },
    { 0, 0 },
    { 0xdb92e01d, 69 },
    { 0xfbe2e01d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982e022, 93 },
    { 0, 0 },
    { 0x7602e024, 33 },
    { 0x2552e025, 61 },
    { 0xe2a2e024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442e029, 77 },
    { 0, 0 },
    { 0xfbb2e02b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652e02e, 46 },
    { 0xa532e02f, 95 },
    { 0xd752e02f, 90 },
    { 0x96e2e02f, 66 },
    { 0x1972e032, 23 },
    { 0x4cd2e033, 33 },
    { 0xf342e034, 33 },
    { 0, 0 },
    { 0x6662e036, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c32e039, 76 },
    { 0xe62e03a, 91 },
    { 0x9472e03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b2e03f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a2e042, 68 },
    { 0x1d92e043, 78 },
    { 0xa692e044, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6692e048, 95 },
    { 0xe532e049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652e04d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa2e055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3672e05b, 61 },
    { 0x9852e05b, 91 },
    { 0, 0 },
    { 0x41b2e05e, 98 },
    { 0x8e02e05e, 33 },
    { 0, 0 },
    { 0x4412e061, 17 },
    { 0, 0 },
    { 0x9522e063, 23 },
    { 0xfd92e064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532e06e, 91 },
    { 0xe2a2e06f, 55 },
    { 0x2cd2e06e, 90 },
    { 0x8172e06e, 76 },
    { 0xcec2e071, 33 },
    { 0, 0 },
    { 0x3712e074, 76 },
    { 0x302e074, 66 },
    { 0xf9e2e076, 90 },
    { 0xa192e077, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af2e07a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x752e07d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd422e081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7172e085, 33 },
    { 0x1af2e085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462e08c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a2e08f, 91 },
    { 0, 0 },
    { 0x2122e091, 46 },
    { 0x91c2e091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c02e096, 93 },
    { 0, 0 },
    { 0xa6c2e098, 95 },
    { 0x292e099, 33 },
    { 0, 0 },
    { 0x5502e09b, 85 },
    { 0x8d82e09c, 33 },
    { 0, 0 },
    { 0xdbd2e09e, 98 },
    { 0x8322e09e, 95 },
    { 0, 0 },
    { 0x3e92e0a1, 65 },
    { 0x46d2e0a2, 91 },
    { 0xd8b2e0a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa092e0a6, 55 },
    { 0xe8e2e0a7, 91 },
    { 0, 0 },
    { 0x72b2e0a9, 33 },
    { 0x77d2e0aa, 61 },
    { 0, 0 },
    { 0xf7c2e0ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02e0af, 17 },
    { 0x21d2e0b0, 95 },
    { 0x1652e0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1682e0b4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b02e0b7, 95 },
    { 0, 0 },
    { 0xd752e0b9, 98 },
    { 0x7f2e0ba, 64 },
    { 0x2a82e0ba, 91 },
    { 0xd672e0bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2e0c2, 90 },
    { 0xed32e0c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552e0c8, 98 },
    { 0, 0 },
    { 0x8b42e0ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e0ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b2e0d1, 78 },
    { 0x7f42e0d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2e0d6, 33 },
    { 0, 0 },
    { 0x4fe2e0d8, 75 },
    { 0, 0 },
    { 0x9c02e0da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4542e0df, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2e0e5, 66 },
    { 0, 0 },
    { 0x1022e0e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f2e0ec, 61 },
    { 0xaf12e0ec, 55 },
    { 0, 0 },
    { 0xff82e0ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd2e0f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e0fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9122e0fe, 67 },
    { 0xb662e0ff, 46 },
    { 0x2d52e100, 91 },
    { 0x7ca2e101, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6882e104, 39 },
    { 0x7992e104, 34 },
    { 0x8c72e106, 46 },
    { 0x5902e107, 95 },
    { 0x69e2e106, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e2e10e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5522e114, 98 },
    { 0xbc2e114, 75 },
    { 0, 0 },
    { 0x5f22e117, 55 },
    { 0xd9d2e118, 61 },
    { 0x1212e118, 33 },
    { 0, 0 },
    { 0xb622e11b, 55 },
    { 0, 0 },
    { 0x2fc2e11d, 55 },
    { 0xda22e11d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b2e123, 36 },
    { 0x47e2e124, 81 },
    { 0x7842e125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6462e12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122e135, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9392e13a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf612e13d, 76 },
    { 0, 0 },
    { 0x3c52e13f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc372e144, 66 },
    { 0xa7a2e145, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc722e14b, 75 },
    { 0xcd22e14b, 98 },
    { 0x9dc2e14d, 61 },
    { 0x73e2e14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22e155, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6972e160, 17 },
    { 0xab32e161, 34 },
    { 0x112e161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe752e16c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2232e172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2e179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba2e17d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442e180, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3262e18a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332e18e, 28 },
    { 0xcb22e18f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3962e192, 33 },
    { 0, 0 },
    { 0x9c42e194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8122e198, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe42e19b, 61 },
    { 0, 0 },
    { 0x2a22e19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea32e1a2, 33 },
    { 0, 0 },
    { 0xd4d2e1a4, 81 },
    { 0xba42e1a4, 84 },
    { 0xc3f2e1a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5042e1ae, 55 },
    { 0xcf32e1af, 90 },
    { 0xf522e1b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6682e1b3, 99 },
    { 0xffc2e1b4, 46 },
    { 0x542e1b5, 95 },
    { 0x1b42e1b6, 43 },
    { 0, 0 },
    { 0x38b2e1b8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e1bd, 90 },
    { 0x3af2e1be, 61 },
    { 0xa962e1bf, 100 },
    { 0x7b22e1bf, 61 },
    { 0x1772e1c1, 23 },
    { 0x4c62e1bd, 95 },
    { 0x7a92e1c0, 46 },
    { 0, 0 },
    { 0xa9b2e1c5, 90 },
    { 0xd852e1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe992e1cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc2e1cf, 43 },
    { 0, 0 },
    { 0x86f2e1d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8832e1d7, 46 },
    { 0, 0 },
    { 0x34e2e1d9, 78 },
    { 0x57f2e1da, 95 },
    { 0, 0 },
    { 0x8f2e1dc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d2e1e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12e1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e72e1ea, 95 },
    { 0x8462e1eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2e1f1, 61 },
    { 0x8922e1f1, 33 },
    { 0, 0 },
    { 0x2762e1f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1012e1f9, 90 },
    { 0, 0 },
    { 0x4532e1fb, 98 },
    { 0, 0 },
    { 0x4b22e1fd, 90 },
    { 0xe632e1fe, 61 },
    { 0x33c2e1ff, 84 },
    { 0x1432e1ff, 93 },
    { 0, 0 },
    { 0xbab2e202, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5052e207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2e20a, 61 },
    { 0x61e2e20a, 87 },
    { 0x7f82e20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8152e20f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592e218, 33 },
    { 0, 0 },
    { 0xf42e21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2e21f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2e229, 97 },
    { 0x6f32e229, 67 },
    { 0x6b92e22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1172e22f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a02e232, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd2e236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e23d, 23 },
    { 0xbb42e23d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382e243, 90 },
    { 0xa752e244, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e22e249, 52 },
    { 0, 0 },
    { 0x9102e24b, 90 },
    { 0xcb92e24c, 55 },
    { 0x2f32e24c, 88 },
    { 0x9512e24e, 95 },
    { 0xf42e24e, 90 },
    { 0xf732e24f, 33 },
    { 0xe9a2e24f, 43 },
    { 0xed82e252, 77 },
    { 0x8312e253, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442e25a, 23 },
    { 0x27d2e25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4572e25e, 90 },
    { 0x5fa2e25f, 75 },
    { 0x5462e25f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c62e264, 46 },
    { 0xb92e265, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3062e269, 23 },
    { 0xa662e26a, 23 },
    { 0x8b2e269, 96 },
    { 0, 0 },
    { 0x3632e26d, 53 },
    { 0x88c2e26e, 23 },
    { 0x5c22e26f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e273, 99 },
    { 0xe8e2e273, 84 },
    { 0, 0 },
    { 0x5d82e276, 33 },
    { 0xa22e277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7082e27a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b2e280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e289, 61 },
    { 0x31a2e28a, 90 },
    { 0, 0 },
    { 0xf902e28c, 93 },
    { 0x6182e28d, 95 },
    { 0xb772e28d, 84 },
    { 0x28c2e28f, 70 },
    { 0xde52e290, 91 },
    { 0x1e82e291, 55 },
    { 0xed92e290, 93 },
    { 0x9972e293, 98 },
    { 0x4992e294, 33 },
    { 0x5772e295, 95 },
    { 0xa62e292, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1092e29d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd92e2a5, 61 },
    { 0xd542e2a6, 61 },
    { 0xe3c2e2a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d2e2aa, 71 },
    { 0xcf02e2aa, 90 },
    { 0x4cb2e2aa, 87 },
    { 0xed12e2ad, 100 },
    { 0, 0 },
    { 0x90b2e2af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92e2b2, 52 },
    { 0x15e2e2b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2092e2b8, 91 },
    { 0xdf32e2b8, 43 },
    { 0x6d2e2ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4582e2bf, 90 },
    { 0, 0 },
    { 0x9222e2c1, 98 },
    { 0, 0 },
    { 0xaf42e2c3, 43 },
    { 0, 0 },
    { 0x8412e2c5, 98 },
    { 0xca92e2c6, 99 },
    { 0, 0 },
    { 0xe422e2c8, 43 },
    { 0x9342e2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd202e2cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc552e2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2e2d6, 70 },
    { 0, 0 },
    { 0x9812e2d8, 87 },
    { 0xd92e2d9, 75 },
    { 0x7432e2da, 61 },
    { 0x8e92e2db, 77 },
    { 0x26c2e2d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8032e2df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b2e2e2, 90 },
    { 0x2dd2e2e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2e2e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc42e2e9, 90 },
    { 0, 0 },
    { 0x7be2e2eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7042e2ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd2e2fa, 61 },
    { 0x9f32e2fb, 61 },
    { 0x452e2fa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82e2ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9342e304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd492e307, 85 },
    { 0xe752e308, 61 },
    { 0x2272e308, 33 },
    { 0xeaa2e309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f2e30d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2e313, 95 },
    { 0xd602e313, 95 },
    { 0xc852e314, 95 },
    { 0, 0 },
    { 0xe662e317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812e31c, 77 },
    { 0xf892e31d, 76 },
    { 0xe492e31d, 25 },
    { 0x9022e31f, 93 },
    { 0xab42e320, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab2e326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa092e32d, 99 },
    { 0, 0 },
    { 0xcde2e32f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d2e332, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc822e336, 95 },
    { 0xb2f2e336, 55 },
    { 0x7542e337, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5922e33c, 34 },
    { 0xa2e2e33d, 61 },
    { 0x1772e33e, 98 },
    { 0, 0 },
    { 0x1022e340, 95 },
    { 0xdb82e341, 93 },
    { 0xf7c2e342, 33 },
    { 0, 0 },
    { 0x7ef2e344, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa12e351, 90 },
    { 0x3c92e352, 33 },
    { 0x2382e353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2e356, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922e359, 95 },
    { 0x7232e35a, 99 },
    { 0x96a2e35b, 66 },
    { 0xf132e35a, 98 },
    { 0, 0 },
    { 0xc492e35e, 90 },
    { 0xaf32e35e, 34 },
    { 0, 0 },
    { 0xcf02e361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a2e365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42e368, 100 },
    { 0x8772e368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a2e36c, 76 },
    { 0x72e2e36d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea72e371, 95 },
    { 0xd4a2e371, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe392e377, 23 },
    { 0xd1c2e378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f2e37b, 55 },
    { 0, 0 },
    { 0x8972e37d, 93 },
    { 0x95f2e37e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8032e382, 59 },
    { 0x4fb2e382, 23 },
    { 0, 0 },
    { 0x44f2e385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2102e388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e2e392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec2e396, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef2e39b, 90 },
    { 0xe5d2e39c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb622e39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8322e3a4, 76 },
    { 0x2112e3a5, 24 },
    { 0xf6c2e3a5, 55 },
    { 0x9e32e3a7, 75 },
    { 0, 0 },
    { 0x6992e3a9, 90 },
    { 0x85d2e3aa, 33 },
    { 0xcb22e3aa, 23 },
    { 0xf502e3ab, 55 },
    { 0x8402e3ad, 33 },
    { 0xad72e3ac, 99 },
    { 0x7362e3af, 55 },
    { 0x3692e3ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf192e3bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4002e3c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba2e3c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf232e3c8, 95 },
    { 0, 0 },
    { 0xd2b2e3ca, 34 },
    { 0xb452e3ca, 90 },
    { 0x1572e3cc, 98 },
    { 0xde02e3cd, 23 },
    { 0x47d2e3cc, 61 },
    { 0xbb02e3cf, 23 },
    { 0x2c2e3d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb2e3d3, 33 },
    { 0x49d2e3d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252e3de, 95 },
    { 0x55e2e3df, 91 },
    { 0x9202e3e0, 95 },
    { 0xf942e3df, 52 },
    { 0x7922e3df, 33 },
    { 0, 0 },
    { 0x6512e3e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f2e3f0, 81 },
    { 0x9282e3f1, 33 },
    { 0, 0 },
    { 0x4522e3f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2e3f9, 91 },
    { 0xc982e3f9, 66 },
    { 0xe132e3fb, 95 },
    { 0, 0 },
    { 0x7542e3fd, 90 },
    { 0x3f32e3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4522e405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4012e408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42e40f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6722e414, 76 },
    { 0, 0 },
    { 0x7cc2e416, 98 },
    { 0x9182e416, 75 },
    { 0xb182e418, 53 },
    { 0, 0 },
    { 0x26c2e41a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8972e41d, 83 },
    { 0xeeb2e41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62e423, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb172e426, 33 },
    { 0x1882e426, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd42e42a, 33 },
    { 0x9c12e42b, 90 },
    { 0x4782e42c, 33 },
    { 0x53f2e42d, 78 },
    { 0x69f2e42e, 76 },
    { 0x5482e42a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e433, 66 },
    { 0xa902e433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e437, 93 },
    { 0, 0 },
    { 0xfd52e439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6342e443, 95 },
    { 0xb762e443, 66 },
    { 0xd8b2e445, 98 },
    { 0xa412e444, 33 },
    { 0xc122e446, 34 },
    { 0x5c02e443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222e44b, 46 },
    { 0xba82e44c, 75 },
    { 0xb682e44c, 34 },
    { 0xd2a2e44e, 90 },
    { 0xe2f2e44f, 90 },
    { 0x2132e44f, 33 },
    { 0xdb72e450, 85 },
    { 0x2212e452, 23 },
    { 0xe1e2e452, 23 },
    { 0xcaf2e44e, 23 },
    { 0x9a2e453, 95 },
    { 0x2912e451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e461, 23 },
    { 0x5372e462, 90 },
    { 0xcfd2e463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b12e466, 76 },
    { 0x37d2e466, 61 },
    { 0x52a2e468, 46 },
    { 0, 0 },
    { 0x5732e46a, 95 },
    { 0, 0 },
    { 0xec32e46c, 91 },
    { 0xab82e46c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3182e470, 98 },
    { 0, 0 },
    { 0x70b2e472, 61 },
    { 0, 0 },
    { 0x2ed2e474, 34 },
    { 0xf532e474, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb192e478, 98 },
    { 0xdf92e479, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72e480, 98 },
    { 0xf972e481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502e488, 95 },
    { 0x1222e488, 95 },
    { 0x8bb2e48a, 87 },
    { 0x45c2e48b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632e493, 90 },
    { 0, 0 },
    { 0xbe52e495, 61 },
    { 0, 0 },
    { 0xaf62e497, 98 },
    { 0, 0 },
    { 0x6e12e499, 53 },
    { 0, 0 },
    { 0x15e2e49b, 96 },
    { 0x53f2e49b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee72e4a0, 55 },
    { 0, 0 },
    { 0xba52e4a2, 98 },
    { 0, 0 },
    { 0xdfa2e4a4, 61 },
    { 0x9322e4a4, 90 },
    { 0, 0 },
    { 0x9e12e4a7, 98 },
    { 0x41d2e4a8, 98 },
    { 0xab62e4a9, 33 },
    { 0xf6d2e4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c12e4ae, 91 },
    { 0, 0 },
    { 0x4b92e4b0, 95 },
    { 0x2982e4b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf482e4b6, 33 },
    { 0x9f02e4b7, 76 },
    { 0, 0 },
    { 0x8a32e4b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9642e4bf, 98 },
    { 0, 0 },
    { 0x9172e4c1, 23 },
    { 0x91d2e4c2, 90 },
    { 0x2a52e4c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a72e4c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2e4ce, 99 },
    { 0, 0 },
    { 0x2242e4d0, 46 },
    { 0x93f2e4d0, 33 },
    { 0x3a72e4d0, 33 },
    { 0x2fc2e4d0, 95 },
    { 0xc3d2e4d2, 66 },
    { 0x1aa2e4d1, 95 },
    { 0xf322e4d6, 55 },
    { 0x4fa2e4d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272e4dc, 90 },
    { 0, 0 },
    { 0xd2c2e4de, 78 },
    { 0xb7c2e4df, 98 },
    { 0x27f2e4e0, 28 },
    { 0xc512e4e0, 61 },
    { 0xb7e2e4e0, 87 },
    { 0x4eb2e4e2, 98 },
    { 0x442e4e4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf172e4e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb2e4ea, 61 },
    { 0x32e2e4eb, 37 },
    { 0xb932e4ea, 34 },
    { 0xa172e4ec, 68 },
    { 0, 0 },
    { 0xde52e4ef, 78 },
    { 0x63f2e4f0, 76 },
    { 0xa9d2e4f1, 76 },
    { 0x5252e4f2, 98 },
    { 0xec62e4f3, 93 },
    { 0x7c82e4f2, 33 },
    { 0x3432e4f4, 76 },
    { 0xfb82e4f6, 34 },
    { 0x6c92e4f6, 33 },
    { 0xc72e4f6, 33 },
    { 0x2b12e4f9, 61 },
    { 0x14d2e4f9, 100 },
    { 0x11a2e4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d2e4fe, 43 },
    { 0, 0 },
    { 0x5d92e500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062e503, 85 },
    { 0x7112e503, 95 },
    { 0, 0 },
    { 0x65a2e506, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad82e50e, 90 },
    { 0, 0 },
    { 0xeb02e510, 33 },
    { 0, 0 },
    { 0x1552e512, 43 },
    { 0, 0 },
    { 0x5982e514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1432e51b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e51e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce22e522, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92e52b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf2e52f, 90 },
    { 0, 0 },
    { 0x8f22e531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a2e535, 33 },
    { 0xc7b2e536, 61 },
    { 0xbc82e536, 90 },
    { 0xff22e537, 55 },
    { 0xc6e2e539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1262e53d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f02e549, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa392e54d, 55 },
    { 0x9712e54d, 67 },
    { 0xf3d2e54f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2e556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b2e559, 67 },
    { 0x9392e55a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b32e55e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f2e564, 17 },
    { 0x8602e565, 87 },
    { 0xfa12e566, 69 },
    { 0x3752e564, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332e56a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd2e56d, 90 },
    { 0, 0 },
    { 0xc572e56f, 90 },
    { 0x2cc2e570, 33 },
    { 0, 0 },
    { 0x4562e572, 91 },
    { 0x2e42e573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882e576, 99 },
    { 0x6db2e576, 55 },
    { 0x4f32e578, 99 },
    { 0xb642e579, 93 },
    { 0xf462e579, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f2e585, 33 },
    { 0, 0 },
    { 0xf1c2e587, 33 },
    { 0xbc82e588, 84 },
    { 0xd8e2e589, 23 },
    { 0, 0 },
    { 0xd602e58b, 93 },
    { 0x5a92e58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52e58f, 61 },
    { 0, 0 },
    { 0x2912e591, 76 },
    { 0x1fc2e592, 88 },
    { 0, 0 },
    { 0xaa62e594, 61 },
    { 0x86d2e594, 34 },
    { 0x74d2e595, 90 },
    { 0, 0 },
    { 0x6c02e598, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e22e5a1, 76 },
    { 0xafd2e5a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad2e5a5, 33 },
    { 0, 0 },
    { 0x8042e5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2e5ab, 66 },
    { 0x2fc2e5ac, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb02e5b2, 90 },
    { 0x2da2e5b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92e5b8, 91 },
    { 0x33b2e5b9, 87 },
    { 0x7ff2e5b8, 100 },
    { 0xb992e5bb, 85 },
    { 0xd8e2e5bc, 66 },
    { 0xfd12e5bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4912e5c3, 98 },
    { 0, 0 },
    { 0xa842e5c5, 76 },
    { 0xf2b2e5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed02e5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e5cf, 61 },
    { 0, 0 },
    { 0x97b2e5d1, 95 },
    { 0xcca2e5d2, 67 },
    { 0, 0 },
    { 0x2a62e5d4, 66 },
    { 0, 0 },
    { 0x4f32e5d6, 33 },
    { 0x2222e5d7, 90 },
    { 0, 0 },
    { 0x2bb2e5d9, 76 },
    { 0x88c2e5da, 98 },
    { 0x6232e5d9, 99 },
    { 0xd5c2e5dc, 90 },
    { 0xdb62e5dd, 90 },
    { 0xb112e5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82e5e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e5e7, 93 },
    { 0x1e42e5e8, 55 },
    { 0xa312e5e8, 61 },
    { 0, 0 },
    { 0x9fc2e5eb, 91 },
    { 0xfa62e5ec, 34 },
    { 0, 0 },
    { 0xae72e5ee, 67 },
    { 0xb752e5ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92e5f4, 85 },
    { 0xfaf2e5f5, 52 },
    { 0x5412e5f5, 67 },
    { 0xd292e5f6, 33 },
    { 0xa5f2e5f8, 33 },
    { 0xf712e5f9, 95 },
    { 0xf52e5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52e5fe, 95 },
    { 0xf932e5ff, 61 },
    { 0x7302e600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa122e60a, 90 },
    { 0x9d42e60b, 95 },
    { 0x4732e60c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb02e60f, 98 },
    { 0x86e2e610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a92e615, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42e61a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c22e61d, 55 },
    { 0xa032e61d, 98 },
    { 0x2322e61d, 52 },
    { 0xe502e620, 33 },
    { 0xce82e621, 23 },
    { 0xee32e622, 33 },
    { 0xa812e623, 23 },
    { 0xa512e623, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002e627, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3732e62c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b2e630, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9792e634, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222e637, 40 },
    { 0, 0 },
    { 0x2692e639, 34 },
    { 0xb72e63a, 34 },
    { 0, 0 },
    { 0x3ca2e63c, 95 },
    { 0x9b42e63d, 90 },
    { 0x2622e63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f2e641, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2282e645, 67 },
    { 0xf032e646, 81 },
    { 0x45e2e645, 76 },
    { 0x1022e647, 43 },
    { 0x2b72e647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf912e64c, 80 },
    { 0xf012e64d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632e650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362e653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c2e65a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a2e65e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e42e662, 90 },
    { 0x98f2e663, 23 },
    { 0x5882e663, 61 },
    { 0xa202e665, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8472e669, 17 },
    { 0x1a92e66a, 33 },
    { 0xe6b2e66b, 46 },
    { 0xb162e66b, 67 },
    { 0, 0 },
    { 0x7c62e66e, 98 },
    { 0x7132e66f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2e674, 98 },
    { 0x8782e675, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c2e678, 43 },
    { 0xff32e679, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7102e67c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff2e680, 93 },
    { 0x74a2e681, 85 },
    { 0x97b2e682, 91 },
    { 0x5bc2e681, 61 },
    { 0x1e52e683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922e687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e2e68b, 81 },
    { 0x60c2e68b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2e691, 34 },
    { 0x3ed2e691, 46 },
    { 0x31f2e693, 61 },
    { 0xbb72e694, 33 },
    { 0x39a2e695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302e6a0, 61 },
    { 0x372e6a0, 90 },
    { 0x6a22e6a0, 90 },
    { 0, 0 },
    { 0x5632e6a4, 78 },
    { 0xc092e6a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9572e6aa, 81 },
    { 0xdc62e6ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8412e6b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72e6bb, 52 },
    { 0x8542e6bc, 90 },
    { 0x6502e6bd, 56 },
    { 0x9d42e6be, 33 },
    { 0xfed2e6be, 78 },
    { 0x82e2e6bd, 93 },
    { 0xf602e6be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b2e6c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32e6c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2e6cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1962e6d0, 78 },
    { 0, 0 },
    { 0x18e2e6d2, 99 },
    { 0, 0 },
    { 0xb272e6d4, 98 },
    { 0xe9a2e6d4, 98 },
    { 0, 0 },
    { 0x7272e6d7, 95 },
    { 0, 0 },
    { 0xa7c2e6d9, 95 },
    { 0x7232e6da, 90 },
    { 0, 0 },
    { 0x2392e6dc, 34 },
    { 0x2d42e6dd, 100 },
    { 0, 0 },
    { 0xd8c2e6df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f2e6e6, 67 },
    { 0xbd62e6e7, 76 },
    { 0xe5a2e6e8, 23 },
    { 0x3d2e6e7, 23 },
    { 0x3392e6e7, 76 },
    { 0x7512e6e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd82e6ee, 90 },
    { 0, 0 },
    { 0xc9b2e6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e2e6f3, 81 },
    { 0xc912e6f3, 75 },
    { 0x2192e6f5, 55 },
    { 0xd9b2e6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c32e6ff, 95 },
    { 0x4b2e700, 33 },
    { 0xf142e701, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2762e704, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c2e708, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2e70b, 78 },
    { 0, 0 },
    { 0x8dd2e70d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9602e712, 99 },
    { 0x6bf2e713, 34 },
    { 0x9a52e714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa242e718, 90 },
    { 0xf2e719, 98 },
    { 0xd7e2e719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d2e71d, 68 },
    { 0xc122e71e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582e726, 61 },
    { 0, 0 },
    { 0x8802e728, 55 },
    { 0xd8c2e729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292e72d, 93 },
    { 0xa832e72e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a52e732, 98 },
    { 0x3bf2e733, 55 },
    { 0x4e82e732, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc372e73a, 34 },
    { 0, 0 },
    { 0x4992e73c, 87 },
    { 0x7a12e73d, 84 },
    { 0x9732e73d, 87 },
    { 0x3fd2e73f, 77 },
    { 0, 0 },
    { 0x83e2e741, 67 },
    { 0xe342e742, 98 },
    { 0, 0 },
    { 0xdd32e744, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462e747, 61 },
    { 0xdb92e747, 76 },
    { 0, 0 },
    { 0x29c2e74a, 48 },
    { 0x2092e74a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2e74e, 43 },
    { 0x8442e74e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe622e753, 36 },
    { 0xc242e754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42e758, 34 },
    { 0xc342e759, 33 },
    { 0xfdf2e758, 98 },
    { 0, 0 },
    { 0xe542e75c, 83 },
    { 0x2562e75d, 55 },
    { 0, 0 },
    { 0xe1f2e75f, 82 },
    { 0xece2e760, 61 },
    { 0x28c2e760, 8 },
    { 0x87e2e760, 46 },
    { 0x5cf2e763, 23 },
    { 0x5202e764, 43 },
    { 0xd152e762, 33 },
    { 0xae52e763, 90 },
    { 0x7c42e767, 85 },
    { 0x5632e768, 17 },
    { 0xe6a2e769, 95 },
    { 0x6782e76a, 42 },
    { 0x97c2e76a, 81 },
    { 0x3e12e75f, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a72e776, 98 },
    { 0xb4e2e776, 61 },
    { 0x7082e778, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2e77c, 23 },
    { 0x792e77d, 34 },
    { 0, 0 },
    { 0xd262e77f, 76 },
    { 0x4a62e77f, 90 },
    { 0x62e781, 95 },
    { 0xb7d2e782, 53 },
    { 0x9d2e783, 99 },
    { 0, 0 },
    { 0xf4e2e785, 87 },
    { 0x252e786, 33 },
    { 0x2072e787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002e78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be2e78d, 51 },
    { 0x5e92e78e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c72e792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772e796, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e79d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa2e7a3, 98 },
    { 0x6112e7a4, 23 },
    { 0x17a2e7a5, 95 },
    { 0x8cd2e7a5, 33 },
    { 0x7732e7a7, 52 },
    { 0xeac2e7a8, 93 },
    { 0, 0 },
    { 0x4702e7aa, 95 },
    { 0xfe72e7ab, 77 },
    { 0x4692e7aa, 66 },
    { 0x6bd2e7ad, 68 },
    { 0, 0 },
    { 0xc0f2e7af, 95 },
    { 0x7352e7b0, 33 },
    { 0, 0 },
    { 0x462e7b2, 61 },
    { 0xe572e7b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7182e7b6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b22e7b9, 93 },
    { 0x7512e7b9, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xe962e7bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402e7c0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e7c3, 98 },
    { 0x5552e7c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b2e7ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7742e7cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda2e7d1, 90 },
    { 0xc032e7d2, 52 },
    { 0x6022e7d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9662e7d8, 95 },
    { 0xb0e2e7d9, 33 },
    { 0x4602e7d9, 76 },
    { 0x2ca2e7d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4092e7e1, 93 },
    { 0x1bb2e7e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4122e7ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9132e7f2, 90 },
    { 0x3f2e7f2, 95 },
    { 0x16b2e7f4, 90 },
    { 0, 0 },
    { 0x21b2e7f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2e7fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a02e7ff, 61 },
    { 0x19c2e800, 61 },
    { 0x4652e801, 91 },
    { 0x5242e801, 34 },
    { 0xd8a2e803, 95 },
    { 0xbc02e804, 66 },
    { 0xf702e804, 52 },
    { 0x69a2e804, 95 },
    { 0xe02e807, 23 },
    { 0, 0 },
    { 0xa1b2e809, 46 },
    { 0x9642e80a, 88 },
    { 0x49e2e80b, 23 },
    { 0xaa72e80b, 98 },
    { 0, 0 },
    { 0xa092e80e, 61 },
    { 0xbd42e80f, 98 },
    { 0xa1f2e80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7252e81c, 76 },
    { 0, 0 },
    { 0xb972e81e, 33 },
    { 0xe4e2e81f, 47 },
    { 0x5c22e820, 91 },
    { 0xd8f2e820, 43 },
    { 0x9f82e822, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d62e825, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce12e829, 90 },
    { 0x2f42e82a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff2e82d, 23 },
    { 0x7782e82e, 66 },
    { 0, 0 },
    { 0x9a32e830, 52 },
    { 0, 0 },
    { 0xab2e832, 84 },
    { 0, 0 },
    { 0x28f2e834, 34 },
    { 0, 0 },
    { 0x1f72e836, 87 },
    { 0x1c52e837, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2e83f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1992e851, 91 },
    { 0x4e12e851, 93 },
    { 0x1fc2e852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd2e85d, 33 },
    { 0, 0 },
    { 0xd982e85f, 23 },
    { 0xbde2e85f, 93 },
    { 0x90a2e860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4672e865, 61 },
    { 0xf2d2e866, 61 },
    { 0x5e02e867, 98 },
    { 0x5b92e868, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f22e86b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d12e871, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52e875, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92e878, 91 },
    { 0xcfe2e879, 95 },
    { 0xdc22e87a, 23 },
    { 0, 0 },
    { 0x6212e87c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c42e883, 55 },
    { 0, 0 },
    { 0x57a2e885, 65 },
    { 0x6e72e886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b2e889, 90 },
    { 0, 0 },
    { 0x5c2e88b, 46 },
    { 0xca92e88c, 95 },
    { 0xeb52e88d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32e890, 80 },
    { 0x37c2e891, 90 },
    { 0xd7b2e890, 46 },
    { 0xa482e890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda2e896, 90 },
    { 0x52c2e897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22e89c, 66 },
    { 0, 0 },
    { 0x6be2e89e, 55 },
    { 0x7512e89e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c2e8aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f2e8b1, 93 },
    { 0xb4b2e8b2, 55 },
    { 0x5b42e8b2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72e8b9, 51 },
    { 0x1982e8ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2e8bd, 90 },
    { 0x7dd2e8be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c12e8c2, 52 },
    { 0x9792e8c3, 85 },
    { 0x22a2e8c2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32e8ca, 85 },
    { 0xadc2e8cb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2e8d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd2e8dd, 40 },
    { 0xf792e8de, 98 },
    { 0x3a32e8df, 33 },
    { 0xb922e8df, 33 },
    { 0, 0 },
    { 0xebc2e8e2, 43 },
    { 0xc482e8e3, 90 },
    { 0x1522e8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2e8ed, 61 },
    { 0xe992e8ed, 93 },
    { 0, 0 },
    { 0x1b12e8f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf122e8f4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d2e8f8, 33 },
    { 0, 0 },
    { 0x1932e8fa, 55 },
    { 0xdd92e8fb, 23 },
    { 0xe2b2e8fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a22e900, 23 },
    { 0, 0 },
    { 0x22d2e902, 33 },
    { 0xdab2e902, 61 },
    { 0x2332e904, 23 },
    { 0xa842e905, 95 },
    { 0x7b42e906, 23 },
    { 0x4892e904, 90 },
    { 0, 0 },
    { 0x9a52e909, 43 },
    { 0, 0 },
    { 0xc802e90b, 91 },
    { 0x98d2e90c, 90 },
    { 0xf212e90b, 100 },
    { 0, 0 },
    { 0x7472e90f, 98 },
    { 0x4ba2e910, 98 },
    { 0xcda2e911, 85 },
    { 0xbfe2e912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92e915, 77 },
    { 0, 0 },
    { 0xf9b2e917, 66 },
    { 0x42d2e917, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2e924, 100 },
    { 0x2e925, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2e92a, 33 },
    { 0, 0 },
    { 0xe2d2e92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2e930, 90 },
    { 0x60c2e931, 76 },
    { 0x1422e930, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652e935, 55 },
    { 0x2c32e936, 55 },
    { 0, 0 },
    { 0x49e2e938, 90 },
    { 0xa532e939, 76 },
    { 0, 0 },
    { 0xeaf2e93b, 78 },
    { 0xc02e93c, 23 },
    { 0x2dd2e93d, 52 },
    { 0x72a2e93e, 23 },
    { 0x4b92e93e, 55 },
    { 0xe932e940, 46 },
    { 0xcaf2e941, 35 },
    { 0, 0 },
    { 0xa412e943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2e948, 90 },
    { 0x922e949, 33 },
    { 0x7ed2e94a, 33 },
    { 0xf722e949, 95 },
    { 0, 0 },
    { 0xd912e94d, 55 },
    { 0x1d82e94e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8952e951, 87 },
    { 0xfa02e951, 90 },
    { 0x3b2e952, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982e957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab2e95e, 78 },
    { 0xe3c2e95e, 55 },
    { 0x1252e960, 98 },
    { 0x2fa2e961, 23 },
    { 0x3672e962, 98 },
    { 0x5102e962, 23 },
    { 0, 0 },
    { 0x52e965, 95 },
    { 0xd402e965, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e969, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d72e972, 90 },
    { 0x9cf2e972, 61 },
    { 0x5a82e974, 55 },
    { 0x83a2e975, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x162e978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9542e97d, 98 },
    { 0x2692e97e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002e983, 33 },
    { 0, 0 },
    { 0xa7f2e985, 95 },
    { 0x6182e985, 23 },
    { 0xd8c2e987, 55 },
    { 0x7732e988, 43 },
    { 0xa6b2e986, 90 },
    { 0x6232e98a, 34 },
    { 0x4d52e98b, 33 },
    { 0x6c22e98c, 66 },
    { 0, 0 },
    { 0x40f2e98e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932e991, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2e995, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd32e99b, 33 },
    { 0xa12e99c, 67 },
    { 0xf2e99b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422e9a0, 78 },
    { 0xede2e9a0, 61 },
    { 0, 0 },
    { 0x842e9a3, 61 },
    { 0x5de2e9a4, 84 },
    { 0x91a2e9a5, 43 },
    { 0x6962e9a5, 93 },
    { 0x7d12e9a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c2e9ae, 93 },
    { 0, 0 },
    { 0x99b2e9b0, 61 },
    { 0x32a2e9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e2e9b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7242e9b7, 95 },
    { 0x8532e9b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2e9bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372e9be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8792e9c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f92e9c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b2e9cf, 76 },
    { 0x7942e9cf, 55 },
    { 0xb122e9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa932e9d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc2e9dc, 78 },
    { 0x7512e9dd, 46 },
    { 0x5192e9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef2e9e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42e9e7, 76 },
    { 0, 0 },
    { 0xa352e9e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2172e9ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62e9f3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc62e9fc, 90 },
    { 0, 0 },
    { 0x10c2e9fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf02ea02, 90 },
    { 0x3ac2ea03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c2ea0d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc2ea13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b2ea16, 34 },
    { 0xcf02ea17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9162ea21, 61 },
    { 0x5162ea21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6992ea28, 90 },
    { 0xc982ea28, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d2ea2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6642ea37, 81 },
    { 0x12b2ea37, 95 },
    { 0xb7c2ea39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f12ea42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f2ea45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd32ea48, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2ea4c, 90 },
    { 0x5d2ea4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f52ea52, 95 },
    { 0x1592ea53, 61 },
    { 0x1c92ea54, 23 },
    { 0x3f52ea53, 99 },
    { 0x4982ea56, 55 },
    { 0, 0 },
    { 0x5902ea58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462ea5b, 95 },
    { 0x8e62ea5c, 23 },
    { 0x8c32ea5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e2ea60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4352ea68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3542ea6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92ea6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32ea73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd2ea77, 66 },
    { 0x1b82ea78, 90 },
    { 0x68e2ea79, 46 },
    { 0, 0 },
    { 0x5992ea7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ea7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3422ea81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22ea89, 33 },
    { 0x5842ea8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52ea8d, 93 },
    { 0, 0 },
    { 0xd5e2ea8f, 90 },
    { 0xf22ea90, 93 },
    { 0xd8b2ea91, 61 },
    { 0, 0 },
    { 0xdd62ea93, 80 },
    { 0x5dc2ea94, 55 },
    { 0x80c2ea94, 76 },
    { 0xcb52ea96, 98 },
    { 0x5f52ea97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa42eaa7, 61 },
    { 0x9f02eaa8, 91 },
    { 0x8422eaa7, 87 },
    { 0x12eaa7, 88 },
    { 0x6d02eaa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a32eaae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852eab1, 76 },
    { 0x1d42eab2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe422eab5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad2eaba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc2eac1, 65 },
    { 0x54e2eac2, 67 },
    { 0xaa12eac2, 85 },
    { 0x1ca2eac4, 51 },
    { 0, 0 },
    { 0x3582eac6, 75 },
    { 0xbf72eac6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902eacb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec82eacf, 32 },
    { 0xe4e2ead0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf142ead3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72ead6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b2eadc, 98 },
    { 0x52c2eadc, 33 },
    { 0x9da2eadd, 98 },
    { 0xa82eadf, 95 },
    { 0, 0 },
    { 0xda52eae1, 61 },
    { 0, 0 },
    { 0xbb42eae3, 87 },
    { 0x1c52eae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4842eae9, 90 },
    { 0x12eaea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a72eaed, 23 },
    { 0x1212eaed, 46 },
    { 0, 0 },
    { 0x5042eaf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b02eaf3, 98 },
    { 0x3e2eaf3, 33 },
    { 0xe702eaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8962eafc, 69 },
    { 0x7de2eafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde72eb07, 23 },
    { 0x7d42eb08, 88 },
    { 0x2052eb09, 91 },
    { 0x7042eb0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c2eb0d, 83 },
    { 0x4dd2eb0e, 90 },
    { 0xb572eb0e, 33 },
    { 0xcc72eb10, 23 },
    { 0x8872eb11, 76 },
    { 0x6362eb12, 95 },
    { 0x4142eb11, 85 },
    { 0xdb42eb14, 23 },
    { 0xf422eb12, 90 },
    { 0xfcc2eb16, 23 },
    { 0xe392eb17, 61 },
    { 0x472eb15, 98 },
    { 0xc162eb13, 98 },
    { 0xa782eb18, 33 },
    { 0x682eb10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda2eb1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe2eb27, 90 },
    { 0xc532eb28, 34 },
    { 0x1532eb29, 68 },
    { 0x2f92eb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf032eb2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1222eb30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de2eb34, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2382eb38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2eb42, 61 },
    { 0x4532eb42, 33 },
    { 0x3312eb44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d62eb47, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2eb4a, 23 },
    { 0x5ae2eb4b, 98 },
    { 0x9fd2eb4a, 61 },
    { 0xd6e2eb4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2eb51, 56 },
    { 0x4982eb51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe2eb57, 23 },
    { 0xb352eb57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee2eb5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632eb60, 84 },
    { 0xc2c2eb61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42eb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd432eb74, 98 },
    { 0x2812eb75, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882eb78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x212eb7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842eb81, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd712eb89, 87 },
    { 0xb902eb89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb2eb8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a2eb92, 90 },
    { 0, 0 },
    { 0x5db2eb94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec2eb9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362eb9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc202eba1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a2eba6, 93 },
    { 0x8b92eba7, 98 },
    { 0x19f2eba8, 65 },
    { 0x1382eba8, 78 },
    { 0x8232eba8, 98 },
    { 0x27d2ebaa, 95 },
    { 0, 0 },
    { 0x26b2ebad, 91 },
    { 0x8aa2ebae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe2ebb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb32ebb6, 33 },
    { 0x5742ebb7, 90 },
    { 0xfc82ebb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b2ebc4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832ebc7, 84 },
    { 0x3322ebc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2ebcc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b2ebd1, 23 },
    { 0, 0 },
    { 0xa482ebd3, 46 },
    { 0x47e2ebd4, 87 },
    { 0x4572ebd5, 23 },
    { 0x7a2ebd4, 23 },
    { 0x3a52ebd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f2ebde, 33 },
    { 0x1e82ebdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb2ebe5, 91 },
    { 0x9ac2ebe6, 90 },
    { 0xb52ebe6, 61 },
    { 0x50d2ebe8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882ebeb, 67 },
    { 0x53d2ebec, 33 },
    { 0x3882ebec, 55 },
    { 0x8392ebee, 98 },
    { 0xe822ebee, 8 },
    { 0xcc12ebef, 55 },
    { 0xe62ebee, 84 },
    { 0x20e2ebf2, 55 },
    { 0x6352ebf3, 23 },
    { 0x81a2ebf4, 93 },
    { 0x1ac2ebf5, 61 },
    { 0xdfc2ebf4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf2ebf9, 52 },
    { 0x3bd2ebf9, 33 },
    { 0xdf82ebfb, 90 },
    { 0xa732ebfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2ec01, 98 },
    { 0, 0 },
    { 0x9572ec03, 98 },
    { 0, 0 },
    { 0x4892ec05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42ec09, 91 },
    { 0x8932ec09, 61 },
    { 0, 0 },
    { 0x8662ec0c, 93 },
    { 0, 0 },
    { 0xad22ec0e, 84 },
    { 0xbfb2ec0f, 96 },
    { 0xfb32ec0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc02ec15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef2ec18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1062ec1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d2ec23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb552ec27, 33 },
    { 0, 0 },
    { 0xb02ec29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d2ec2d, 90 },
    { 0, 0 },
    { 0x43a2ec2f, 46 },
    { 0xa082ec30, 33 },
    { 0x5552ec31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee2ec36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c72ec39, 68 },
    { 0, 0 },
    { 0x85c2ec3b, 34 },
    { 0, 0 },
    { 0x4422ec3d, 68 },
    { 0x512ec3e, 76 },
    { 0xb022ec3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e2ec45, 98 },
    { 0, 0 },
    { 0xf9d2ec47, 33 },
    { 0xa382ec48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d2ec4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2ec56, 100 },
    { 0, 0 },
    { 0x2c42ec58, 33 },
    { 0x54c2ec58, 98 },
    { 0xe0b2ec5a, 90 },
    { 0xd062ec58, 95 },
    { 0, 0 },
    { 0xa982ec5d, 100 },
    { 0x3f2ec5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632ec61, 33 },
    { 0xde72ec62, 92 },
    { 0xdf52ec61, 95 },
    { 0xfce2ec64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b2ec67, 51 },
    { 0xb1f2ec67, 95 },
    { 0x5bf2ec69, 23 },
    { 0x7862ec6a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4312ec6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f2ec71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe02ec76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692ec79, 91 },
    { 0x3902ec79, 4 },
    { 0x8252ec7b, 61 },
    { 0x9cb2ec7c, 33 },
    { 0x6cf2ec7d, 95 },
    { 0xc182ec7e, 85 },
    { 0x8c52ec7f, 61 },
    { 0x1cc2ec7e, 76 },
    { 0x9672ec81, 95 },
    { 0xd312ec7a, 59 },
    { 0, 0 },
    { 0x1eb2ec84, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c92ec88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf742ec8b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb802ec90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3722ec95, 55 },
    { 0xeb02ec96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc662eca0, 23 },
    { 0x62eca1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2eca4, 95 },
    { 0xbd12eca4, 33 },
    { 0xa7b2eca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa652ecac, 43 },
    { 0, 0 },
    { 0x75e2ecae, 23 },
    { 0xd802ecaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d82ecb4, 93 },
    { 0xa202ecb5, 95 },
    { 0xb972ecb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3412ecb9, 85 },
    { 0, 0 },
    { 0x8a2ecbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd642ecbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb2ecc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2ecca, 65 },
    { 0, 0 },
    { 0x7c72eccc, 55 },
    { 0xcba2eccc, 87 },
    { 0xddb2ecce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2ecd4, 76 },
    { 0, 0 },
    { 0x5112ecd6, 76 },
    { 0, 0 },
    { 0x3bd2ecd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7812ecdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082ece5, 23 },
    { 0, 0 },
    { 0xa592ece7, 61 },
    { 0xdeb2ece8, 90 },
    { 0x8fc2ece8, 90 },
    { 0x7fe2ece8, 99 },
    { 0xaa2ece9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72ecf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62ecf4, 33 },
    { 0x6b2ecf5, 87 },
    { 0, 0 },
    { 0xb222ecf7, 95 },
    { 0xad32ecf8, 95 },
    { 0, 0 },
    { 0x4de2ecfa, 95 },
    { 0xf092ecfa, 93 },
    { 0, 0 },
    { 0x3bd2ecfd, 34 },
    { 0x7ee2ecfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d82ed02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272ed05, 90 },
    { 0x7d52ed06, 33 },
    { 0, 0 },
    { 0xc622ed08, 78 },
    { 0x7172ed09, 43 },
    { 0xfe22ed0a, 33 },
    { 0xcad2ed0b, 95 },
    { 0x4652ed0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7882ed10, 43 },
    { 0xda82ed11, 87 },
    { 0x8f52ed12, 75 },
    { 0x98d2ed11, 43 },
    { 0, 0 },
    { 0x8b72ed15, 90 },
    { 0xc572ed16, 90 },
    { 0x1e42ed17, 99 },
    { 0x6092ed16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c02ed1c, 87 },
    { 0, 0 },
    { 0xafb2ed1e, 61 },
    { 0, 0 },
    { 0xe1d2ed20, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ed24, 77 },
    { 0, 0 },
    { 0x532ed26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c2ed30, 100 },
    { 0, 0 },
    { 0x2132ed32, 90 },
    { 0xb5c2ed32, 98 },
    { 0, 0 },
    { 0x4eb2ed35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc32ed38, 97 },
    { 0x4342ed39, 61 },
    { 0, 0 },
    { 0xf952ed3b, 43 },
    { 0x95b2ed3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc12ed3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead2ed4d, 98 },
    { 0x2632ed4d, 98 },
    { 0x1e92ed4f, 101 },
    { 0xcf2ed50, 19 },
    { 0, 0 },
    { 0x9b72ed52, 95 },
    { 0x3932ed53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2ed57, 23 },
    { 0, 0 },
    { 0xa9f2ed59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3832ed5c, 85 },
    { 0, 0 },
    { 0xfc22ed5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2ed62, 33 },
    { 0x9a82ed62, 33 },
    { 0, 0 },
    { 0xa152ed65, 33 },
    { 0xe22ed66, 23 },
    { 0xceb2ed67, 55 },
    { 0xeed2ed68, 60 },
    { 0x6ff2ed69, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b82ed6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2ed73, 98 },
    { 0, 0 },
    { 0x1952ed75, 76 },
    { 0xee82ed75, 61 },
    { 0xfcc2ed77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c2ed7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f2ed83, 98 },
    { 0x7902ed84, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092ed90, 61 },
    { 0xfad2ed90, 34 },
    { 0, 0 },
    { 0x77c2ed93, 93 },
    { 0xb522ed94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f2ed9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x152ed9f, 61 },
    { 0, 0 },
    { 0x2822eda1, 55 },
    { 0, 0 },
    { 0x6752eda3, 33 },
    { 0x5102eda3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2edaa, 33 },
    { 0x7102edaa, 76 },
    { 0x9bd2edaa, 69 },
    { 0xd962edad, 90 },
    { 0x46c2edae, 66 },
    { 0, 0 },
    { 0xa2d2edb0, 95 },
    { 0x16e2edb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02edb7, 65 },
    { 0, 0 },
    { 0x7102edb9, 76 },
    { 0x27b2edb9, 61 },
    { 0x45f2edbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade2edbf, 33 },
    { 0x3822edbf, 33 },
    { 0x6742edc1, 77 },
    { 0x5b92edbf, 52 },
    { 0, 0 },
    { 0xd4d2edc4, 33 },
    { 0xa3c2edc5, 43 },
    { 0xb7e2edc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b2edca, 88 },
    { 0x9d02edcb, 87 },
    { 0, 0 },
    { 0x6402edcd, 20 },
    { 0x402edcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf542edd1, 33 },
    { 0x14f2edd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f82edd7, 46 },
    { 0, 0 },
    { 0xe3b2edd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f62eddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4162ede1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512ede6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4242edeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32edf1, 61 },
    { 0, 0 },
    { 0x4d82edf3, 76 },
    { 0, 0 },
    { 0xaa2edf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f2edf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172edfd, 23 },
    { 0x52f2edfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f32ee02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2ee05, 67 },
    { 0x78a2ee06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b2ee09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ee0e, 61 },
    { 0x16a2ee0f, 90 },
    { 0x39d2ee10, 34 },
    { 0x5e32ee11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2ee14, 76 },
    { 0x77b2ee15, 76 },
    { 0x382ee16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb452ee19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d52ee1e, 76 },
    { 0xae82ee1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b2ee26, 33 },
    { 0, 0 },
    { 0x8622ee28, 33 },
    { 0, 0 },
    { 0x6652ee2a, 78 },
    { 0, 0 },
    { 0x8042ee2c, 78 },
    { 0x6662ee2d, 61 },
    { 0, 0 },
    { 0x7822ee2f, 90 },
    { 0, 0 },
    { 0xe9a2ee31, 43 },
    { 0, 0 },
    { 0xf762ee33, 33 },
    { 0, 0 },
    { 0xc8c2ee35, 34 },
    { 0x27c2ee36, 87 },
    { 0x79a2ee37, 88 },
    { 0, 0 },
    { 0x4a42ee39, 90 },
    { 0x4092ee39, 90 },
    { 0, 0 },
    { 0x7fd2ee3c, 31 },
    { 0xf002ee3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb22ee40, 95 },
    { 0, 0 },
    { 0xce12ee42, 23 },
    { 0x2e92ee43, 76 },
    { 0xaf82ee43, 87 },
    { 0x1072ee45, 95 },
    { 0xec52ee45, 61 },
    { 0x1e12ee43, 33 },
    { 0xaeb2ee43, 43 },
    { 0x9a72ee49, 90 },
    { 0x6a52ee49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4822ee50, 95 },
    { 0x2f52ee51, 64 },
    { 0, 0 },
    { 0x3022ee53, 85 },
    { 0x5042ee54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32ee5a, 90 },
    { 0x1292ee5b, 84 },
    { 0xc0e2ee5c, 87 },
    { 0xec72ee5d, 99 },
    { 0, 0 },
    { 0x1792ee5f, 23 },
    { 0x5752ee60, 61 },
    { 0x84f2ee60, 55 },
    { 0x7342ee61, 66 },
    { 0, 0 },
    { 0xa4f2ee64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3472ee68, 23 },
    { 0x2c22ee69, 90 },
    { 0xdfa2ee69, 87 },
    { 0x7f72ee69, 33 },
    { 0x1f62ee69, 75 },
    { 0xd232ee6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf802ee74, 17 },
    { 0x5d62ee74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd2ee83, 98 },
    { 0, 0 },
    { 0x3ed2ee85, 87 },
    { 0xcc52ee85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6962ee9e, 90 },
    { 0x85e2ee9e, 23 },
    { 0x3de2eea0, 98 },
    { 0x19a2eea1, 33 },
    { 0xbf22eea2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982eea6, 90 },
    { 0, 0 },
    { 0xad92eea8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd282eeab, 34 },
    { 0xced2eeac, 90 },
    { 0x7be2eead, 87 },
    { 0, 0 },
    { 0xd112eeaf, 95 },
    { 0, 0 },
    { 0x6e82eeb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72eebb, 90 },
    { 0, 0 },
    { 0xb612eebd, 90 },
    { 0x6fe2eebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912eec2, 87 },
    { 0xf2f2eec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542eecb, 23 },
    { 0x22d2eecc, 61 },
    { 0x1f32eecd, 90 },
    { 0x7602eece, 33 },
    { 0x27b2eece, 33 },
    { 0x26b2eece, 68 },
    { 0xc842eed0, 100 },
    { 0x1622eecc, 90 },
    { 0xd742eed2, 95 },
    { 0xbac2eecd, 93 },
    { 0, 0 },
    { 0xf4a2eed6, 25 },
    { 0, 0 },
    { 0xd9a2eed8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62eee2, 67 },
    { 0xb752eee3, 98 },
    { 0, 0 },
    { 0xb322eee5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa322eeea, 61 },
    { 0x2972eeea, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d02eef2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52eef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8482eef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2eefe, 90 },
    { 0, 0 },
    { 0x3f2ef00, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2ef04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f2ef08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f2ef0e, 61 },
    { 0x9ac2ef0f, 95 },
    { 0, 0 },
    { 0x2ed2ef11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b2ef18, 76 },
    { 0xa972ef19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3682ef1f, 91 },
    { 0, 0 },
    { 0xcf72ef21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd92ef2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2ef30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72ef33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4772ef36, 23 },
    { 0x1962ef37, 23 },
    { 0x4e22ef37, 94 },
    { 0x4c12ef38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482ef3c, 81 },
    { 0, 0 },
    { 0xd972ef3e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e42ef42, 17 },
    { 0x9812ef43, 90 },
    { 0x7202ef44, 34 },
    { 0, 0 },
    { 0x2222ef46, 98 },
    { 0x93f2ef47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf22ef4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52ef4e, 87 },
    { 0xdb2ef4f, 98 },
    { 0, 0 },
    { 0x4522ef51, 94 },
    { 0x8ce2ef52, 33 },
    { 0, 0 },
    { 0xe332ef54, 100 },
    { 0xe082ef55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a2ef58, 90 },
    { 0x7952ef58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ef5c, 88 },
    { 0x9882ef5d, 85 },
    { 0x9b02ef5e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc2ef68, 85 },
    { 0xfcd2ef68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3142ef6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc2ef6f, 68 },
    { 0xcea2ef6f, 33 },
    { 0xf7a2ef71, 90 },
    { 0x20f2ef71, 23 },
    { 0, 0 },
    { 0x1ea2ef74, 98 },
    { 0, 0 },
    { 0xa772ef76, 33 },
    { 0, 0 },
    { 0xa342ef78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7312ef7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa782ef7e, 81 },
    { 0xf2d2ef7f, 98 },
    { 0x4c32ef80, 98 },
    { 0xb872ef80, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2ef84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b2ef8a, 91 },
    { 0, 0 },
    { 0x4452ef8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e2ef90, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec2ef93, 98 },
    { 0, 0 },
    { 0xc822ef95, 74 },
    { 0, 0 },
    { 0x3aa2ef97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d82ef9d, 77 },
    { 0xbe62ef9e, 61 },
    { 0, 0 },
    { 0xaaa2efa0, 76 },
    { 0, 0 },
    { 0xa202efa2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732efa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2efa8, 95 },
    { 0x2322efa9, 33 },
    { 0x5e42efaa, 90 },
    { 0x1a42efa9, 98 },
    { 0x92d2efab, 95 },
    { 0xd8f2efad, 93 },
    { 0, 0 },
    { 0xd782efaf, 88 },
    { 0xe502efb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842efb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd2efbc, 91 },
    { 0xdd02efbd, 33 },
    { 0x8e22efbe, 98 },
    { 0xf2d2efbf, 23 },
    { 0xe542efbc, 95 },
    { 0xd422efc1, 23 },
    { 0x3c02efbc, 61 },
    { 0x1a22efc3, 61 },
    { 0, 0 },
    { 0x24a2efc5, 33 },
    { 0xf0b2efc5, 61 },
    { 0x2122efc7, 90 },
    { 0x5912efc8, 95 },
    { 0x3262efc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee92efcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd752efd0, 55 },
    { 0x2592efd1, 95 },
    { 0x1652efd2, 78 },
    { 0xbec2efd1, 61 },
    { 0xd042efd4, 61 },
    { 0xc42efd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6152efe0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82efe3, 33 },
    { 0x4272efe4, 93 },
    { 0x1462efe3, 90 },
    { 0x4fd2efe6, 90 },
    { 0xf2a2efe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452efea, 98 },
    { 0, 0 },
    { 0x9bf2efec, 55 },
    { 0x31d2efec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62eff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc2eff9, 95 },
    { 0, 0 },
    { 0x6f42effb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2f000, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d2f004, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf802f00d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda62f011, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf602f014, 93 },
    { 0, 0 },
    { 0xb482f016, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082f019, 91 },
    { 0xc5f2f019, 85 },
    { 0x88c2f01a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde12f01e, 88 },
    { 0x2e12f01f, 52 },
    { 0x1b52f020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa032f023, 33 },
    { 0xdef2f024, 52 },
    { 0x9c22f024, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922f028, 77 },
    { 0x13b2f028, 23 },
    { 0x2f72f02a, 80 },
    { 0xc8c2f02b, 98 },
    { 0x302f02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8622f039, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a2f03e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72f042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962f04f, 87 },
    { 0xb9b2f04f, 85 },
    { 0, 0 },
    { 0xa0b2f052, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb2f056, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2f059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1322f05f, 95 },
    { 0x88b2f060, 98 },
    { 0, 0 },
    { 0xd202f062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe2f068, 98 },
    { 0xe742f069, 84 },
    { 0xde42f06a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa22f06d, 66 },
    { 0x2b92f06e, 98 },
    { 0xf0f2f06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2f07e, 33 },
    { 0x65f2f07f, 80 },
    { 0, 0 },
    { 0xaa72f081, 23 },
    { 0xf152f081, 61 },
    { 0, 0 },
    { 0x6662f084, 61 },
    { 0, 0 },
    { 0xde82f086, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6252f08b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22f090, 60 },
    { 0x6602f091, 95 },
    { 0, 0 },
    { 0x4172f093, 33 },
    { 0x78a2f094, 87 },
    { 0, 0 },
    { 0xb9c2f096, 95 },
    { 0x9ef2f097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2f0a0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a2f0a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f12f0ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2f0af, 75 },
    { 0xfaf2f0b0, 99 },
    { 0xdfa2f0b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb22f0b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2f0bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9002f0c1, 46 },
    { 0x4a22f0c2, 17 },
    { 0, 0 },
    { 0x8502f0c4, 34 },
    { 0, 0 },
    { 0x6382f0c6, 76 },
    { 0xdca2f0c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc152f0cc, 95 },
    { 0xb922f0cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe2f0d0, 95 },
    { 0xae12f0d1, 33 },
    { 0x6542f0d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb32f0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e32f0d8, 77 },
    { 0, 0 },
    { 0xb082f0da, 61 },
    { 0x6292f0da, 61 },
    { 0xeb62f0db, 90 },
    { 0xb482f0dd, 23 },
    { 0xe2f2f0de, 76 },
    { 0x8e52f0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652f0e3, 95 },
    { 0x4e62f0e3, 77 },
    { 0xd972f0e5, 76 },
    { 0, 0 },
    { 0x19c2f0e7, 61 },
    { 0x34b2f0e8, 55 },
    { 0x2102f0e7, 33 },
    { 0, 0 },
    { 0x21e2f0eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab2f0f0, 46 },
    { 0x4de2f0f1, 23 },
    { 0x5ab2f0f1, 95 },
    { 0x2842f0f1, 37 },
    { 0x8412f0f4, 91 },
    { 0xa632f0f4, 83 },
    { 0, 0 },
    { 0xc002f0f7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2f0fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae2f0fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2f101, 61 },
    { 0x23e2f102, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2f106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c2f10c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2f110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde12f114, 46 },
    { 0xf452f115, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92f11b, 75 },
    { 0, 0 },
    { 0x7fc2f11d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7782f124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872f129, 65 },
    { 0x9f92f129, 77 },
    { 0, 0 },
    { 0xc732f12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472f12f, 93 },
    { 0xf712f130, 66 },
    { 0x9ad2f131, 95 },
    { 0xa842f132, 95 },
    { 0, 0 },
    { 0xd0e2f134, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf2f13d, 90 },
    { 0x4962f13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72f141, 90 },
    { 0x8c02f141, 33 },
    { 0xf7a2f143, 46 },
    { 0xe112f142, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962f14a, 98 },
    { 0xabc2f14a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2432f150, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed2f154, 61 },
    { 0xd7f2f154, 99 },
    { 0xd592f155, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9142f15e, 61 },
    { 0xf192f15f, 23 },
    { 0x9592f15e, 78 },
    { 0xc2e2f161, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf62f166, 61 },
    { 0x1cb2f167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622f16b, 90 },
    { 0, 0 },
    { 0x7682f16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6202f171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2882f178, 87 },
    { 0, 0 },
    { 0xf2f2f17a, 17 },
    { 0xa4b2f17b, 98 },
    { 0xb0b2f17b, 61 },
    { 0x8082f17b, 61 },
    { 0x4742f17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8282f18e, 78 },
    { 0x5382f18f, 90 },
    { 0x7aa2f18e, 55 },
    { 0, 0 },
    { 0x25a2f192, 90 },
    { 0, 0 },
    { 0xceb2f194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2562f198, 23 },
    { 0, 0 },
    { 0xcf92f19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf672f1a1, 90 },
    { 0x4eb2f1a2, 90 },
    { 0x77e2f1a2, 61 },
    { 0, 0 },
    { 0x35e2f1a5, 76 },
    { 0x5b72f1a6, 76 },
    { 0, 0 },
    { 0x5ac2f1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf312f1ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402f1b2, 61 },
    { 0x9e22f1b3, 34 },
    { 0x1072f1b4, 91 },
    { 0x5982f1b5, 76 },
    { 0, 0 },
    { 0x1e22f1b7, 33 },
    { 0x8e42f1b7, 61 },
    { 0xe5a2f1b9, 95 },
    { 0xef32f1ba, 52 },
    { 0xd922f1bb, 61 },
    { 0x96a2f1bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f1bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2f1c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce2f1cb, 76 },
    { 0xa7a2f1cb, 33 },
    { 0xde92f1cb, 90 },
    { 0xd302f1cb, 90 },
    { 0xaa52f1cb, 86 },
    { 0x31c2f1cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1722f1d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7192f1d8, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb2f1df, 61 },
    { 0xe4c2f1e0, 46 },
    { 0x7862f1e0, 67 },
    { 0x5022f1e2, 90 },
    { 0x25b2f1e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3632f1e7, 61 },
    { 0x35a2f1e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab2f1eb, 90 },
    { 0, 0 },
    { 0x2c62f1ed, 61 },
    { 0x51a2f1ee, 100 },
    { 0x8e62f1ef, 61 },
    { 0xa052f1f0, 55 },
    { 0x632f1f0, 52 },
    { 0x7752f1f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d72f1f7, 67 },
    { 0, 0 },
    { 0xda2f1f9, 61 },
    { 0, 0 },
    { 0xb412f1fb, 98 },
    { 0, 0 },
    { 0xe6e2f1fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff2f201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f205, 33 },
    { 0, 0 },
    { 0xf602f207, 33 },
    { 0, 0 },
    { 0x90d2f209, 84 },
    { 0xaae2f209, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa2f211, 34 },
    { 0x41d2f212, 33 },
    { 0xab62f212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f21a, 55 },
    { 0, 0 },
    { 0x5672f21c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x762f21f, 65 },
    { 0x48e2f220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062f225, 90 },
    { 0x1962f225, 87 },
    { 0xaf32f227, 76 },
    { 0xc2c2f228, 90 },
    { 0x4172f229, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192f22e, 90 },
    { 0, 0 },
    { 0x4702f230, 90 },
    { 0x43f2f231, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172f235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac2f238, 99 },
    { 0x9f2f239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2f23d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af2f240, 78 },
    { 0x1d22f241, 98 },
    { 0x8742f242, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22f248, 95 },
    { 0xf5e2f248, 61 },
    { 0x9b22f248, 98 },
    { 0xe0b2f249, 61 },
    { 0x4f52f248, 76 },
    { 0x59d2f24d, 65 },
    { 0xe892f24e, 55 },
    { 0x46c2f24f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382f252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9322f255, 61 },
    { 0xdc72f256, 98 },
    { 0xf9e2f256, 66 },
    { 0, 0 },
    { 0xc562f259, 43 },
    { 0, 0 },
    { 0x29a2f25b, 90 },
    { 0, 0 },
    { 0xc522f25d, 90 },
    { 0x99f2f25e, 66 },
    { 0xa812f25f, 33 },
    { 0, 0 },
    { 0x6fa2f261, 98 },
    { 0, 0 },
    { 0x80b2f263, 23 },
    { 0x39e2f263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e52f268, 90 },
    { 0xbf42f269, 90 },
    { 0x372f26a, 61 },
    { 0xe8f2f268, 66 },
    { 0xdd62f26a, 33 },
    { 0xf4a2f26d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262f272, 46 },
    { 0x7132f273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e2f27a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8022f27f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502f285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe912f289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6802f28d, 52 },
    { 0, 0 },
    { 0x7862f28f, 17 },
    { 0, 0 },
    { 0x9af2f291, 61 },
    { 0xca72f292, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2f295, 33 },
    { 0x76a2f296, 61 },
    { 0x8582f297, 87 },
    { 0xd032f296, 90 },
    { 0, 0 },
    { 0x3792f29a, 69 },
    { 0xe6e2f29b, 23 },
    { 0x362f29b, 70 },
    { 0x4be2f29c, 33 },
    { 0xe182f29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362f2a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf02f2af, 33 },
    { 0, 0 },
    { 0xcfd2f2b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92f2b4, 61 },
    { 0, 0 },
    { 0x9582f2b6, 34 },
    { 0, 0 },
    { 0x7372f2b8, 68 },
    { 0x412f2b8, 90 },
    { 0x54a2f2b8, 33 },
    { 0x5152f2b9, 55 },
    { 0x2fe2f2bc, 76 },
    { 0xb4b2f2bd, 99 },
    { 0xe462f2bd, 33 },
    { 0x80a2f2b8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc672f2c2, 76 },
    { 0, 0 },
    { 0x8a52f2c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4492f2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe42f2d0, 90 },
    { 0xb182f2d1, 33 },
    { 0xbfc2f2d2, 34 },
    { 0, 0 },
    { 0x2162f2d4, 95 },
    { 0x2802f2d5, 46 },
    { 0xcbb2f2d5, 23 },
    { 0xa8f2f2d7, 98 },
    { 0x9fb2f2d5, 33 },
    { 0, 0 },
    { 0xa6e2f2da, 61 },
    { 0xc842f2db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e22f2e2, 98 },
    { 0xcd82f2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9192f2e8, 33 },
    { 0xd6e2f2e9, 91 },
    { 0x4ef2f2e8, 52 },
    { 0x9692f2eb, 65 },
    { 0xebd2f2ec, 53 },
    { 0, 0 },
    { 0xd6e2f2ee, 33 },
    { 0, 0 },
    { 0xba72f2f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442f2f8, 61 },
    { 0xa6c2f2f8, 76 },
    { 0x58e2f2fa, 76 },
    { 0x8a22f2fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2f2ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d82f303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc72f308, 90 },
    { 0xd2f309, 88 },
    { 0, 0 },
    { 0x3bb2f30b, 61 },
    { 0x7662f30b, 33 },
    { 0x812f30d, 76 },
    { 0xe352f30d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f311, 99 },
    { 0x3152f312, 90 },
    { 0x12c2f313, 78 },
    { 0xa1b2f313, 98 },
    { 0xb9d2f313, 55 },
    { 0x9b12f314, 61 },
    { 0x7e62f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde2f31a, 98 },
    { 0, 0 },
    { 0x4782f31c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2f320, 23 },
    { 0x2c22f320, 100 },
    { 0x2732f320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2f326, 95 },
    { 0xfa42f327, 83 },
    { 0x8672f328, 75 },
    { 0xa402f329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2f32d, 76 },
    { 0xe6f2f32e, 100 },
    { 0x75b2f32d, 90 },
    { 0xda72f32d, 36 },
    { 0xc7f2f331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592f338, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362f33d, 90 },
    { 0xbb02f33e, 96 },
    { 0x1d2f33e, 95 },
    { 0x23c2f340, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c2f345, 55 },
    { 0x34d2f346, 55 },
    { 0x3b92f347, 23 },
    { 0, 0 },
    { 0xf142f349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d2f34c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf112f350, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e12f355, 61 },
    { 0xea2f356, 33 },
    { 0x74c2f356, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4032f365, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e2f369, 34 },
    { 0xeeb2f369, 33 },
    { 0x33f2f369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c2f36f, 76 },
    { 0xd1b2f36f, 23 },
    { 0, 0 },
    { 0x1be2f372, 34 },
    { 0, 0 },
    { 0xbef2f374, 46 },
    { 0, 0 },
    { 0x4952f376, 60 },
    { 0x8e62f377, 33 },
    { 0x4382f376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc852f37b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6772f37e, 33 },
    { 0, 0 },
    { 0x4a02f380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e52f387, 83 },
    { 0, 0 },
    { 0xb722f389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f52f38c, 90 },
    { 0, 0 },
    { 0x9ee2f38e, 33 },
    { 0x6e02f38f, 90 },
    { 0xff02f38f, 33 },
    { 0xe262f38e, 55 },
    { 0x2722f391, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82f399, 76 },
    { 0, 0 },
    { 0x7b92f39b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362f39e, 33 },
    { 0xaab2f39f, 95 },
    { 0x4f62f3a0, 90 },
    { 0x9cb2f3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a2f3a9, 90 },
    { 0, 0 },
    { 0x302f3ab, 76 },
    { 0xf12f3ac, 55 },
    { 0x67c2f3ad, 90 },
    { 0x1002f3ad, 66 },
    { 0x54c2f3af, 46 },
    { 0x68a2f3b0, 90 },
    { 0xdb42f3b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192f3b4, 60 },
    { 0, 0 },
    { 0x5ac2f3b6, 46 },
    { 0x5f32f3b7, 23 },
    { 0x9832f3b8, 49 },
    { 0xff12f3b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f3bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92f3c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922f3c8, 46 },
    { 0x1482f3c9, 55 },
    { 0, 0 },
    { 0xefe2f3cb, 17 },
    { 0, 0 },
    { 0xe312f3cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2f3d4, 91 },
    { 0x13f2f3d4, 61 },
    { 0, 0 },
    { 0x2242f3d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b2f3da, 80 },
    { 0x9cc2f3da, 76 },
    { 0x19e2f3db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f62f3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912f3ed, 43 },
    { 0, 0 },
    { 0x79a2f3ef, 91 },
    { 0xae72f3ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b2f3f4, 52 },
    { 0x4d32f3f5, 76 },
    { 0xa12f3f6, 38 },
    { 0, 0 },
    { 0x2b72f3f8, 33 },
    { 0, 0 },
    { 0x5942f3fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f2f3fe, 99 },
    { 0xe132f3ff, 90 },
    { 0x94f2f3fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72f404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2f408, 61 },
    { 0, 0 },
    { 0xc1d2f40a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f413, 95 },
    { 0x1c82f413, 34 },
    { 0, 0 },
    { 0x8342f416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc2f41a, 96 },
    { 0xf7b2f41b, 33 },
    { 0x8c2f41c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4452f420, 61 },
    { 0x6222f421, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4862f426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2f429, 84 },
    { 0xee92f42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa972f42e, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f12f434, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3062f43d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b2f440, 78 },
    { 0, 0 },
    { 0x98c2f442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa932f447, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2f44d, 87 },
    { 0, 0 },
    { 0x5cc2f44f, 84 },
    { 0, 0 },
    { 0x2c72f451, 64 },
    { 0, 0 },
    { 0xf402f453, 95 },
    { 0x7d12f454, 55 },
    { 0x40a2f455, 82 },
    { 0x6142f456, 90 },
    { 0xafe2f457, 61 },
    { 0x50f2f458, 23 },
    { 0x55c2f459, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b2f461, 90 },
    { 0x55e2f462, 90 },
    { 0xc092f463, 23 },
    { 0x5a12f463, 81 },
    { 0x2bb2f463, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82f46d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a2f472, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982f47b, 78 },
    { 0x29e2f47c, 78 },
    { 0, 0 },
    { 0x8352f47e, 84 },
    { 0xc742f47f, 91 },
    { 0xf7a2f47e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822f487, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc2f48f, 65 },
    { 0x1282f490, 80 },
    { 0xb352f491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2f498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2f49c, 52 },
    { 0xb252f49c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d62f4a3, 33 },
    { 0, 0 },
    { 0xc142f4a5, 33 },
    { 0xd132f4a6, 33 },
    { 0x4482f4a7, 66 },
    { 0x7f82f4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a2f4ac, 95 },
    { 0x27e2f4ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762f4b1, 95 },
    { 0x4482f4b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a2f4b6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da2f4be, 95 },
    { 0, 0 },
    { 0xef32f4c0, 61 },
    { 0, 0 },
    { 0xcc92f4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2f4d2, 77 },
    { 0, 0 },
    { 0xda62f4d4, 61 },
    { 0xa882f4d5, 43 },
    { 0xa362f4d6, 98 },
    { 0xdd02f4d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e72f4df, 43 },
    { 0x8f52f4e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae32f4e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2102f4e9, 23 },
    { 0xe582f4e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd562f4f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd42f4f5, 100 },
    { 0x42e2f4f5, 76 },
    { 0xbc92f4f5, 90 },
    { 0xcdd2f4f8, 95 },
    { 0, 0 },
    { 0xbb22f4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2f4fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa002f502, 76 },
    { 0x7f02f503, 84 },
    { 0x7b62f504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb2f509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182f50f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62f512, 34 },
    { 0x2df2f513, 76 },
    { 0xa0e2f513, 61 },
    { 0xd812f515, 40 },
    { 0x6972f515, 43 },
    { 0x7f22f516, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02f51d, 61 },
    { 0x1a82f51e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8782f521, 78 },
    { 0, 0 },
    { 0x84d2f523, 23 },
    { 0xa6f2f523, 33 },
    { 0x4a02f525, 98 },
    { 0, 0 },
    { 0x99e2f527, 98 },
    { 0x9422f528, 87 },
    { 0xa8f2f529, 88 },
    { 0xd232f527, 23 },
    { 0, 0 },
    { 0x4362f52c, 90 },
    { 0x8ec2f52d, 90 },
    { 0xb3f2f52c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd92f534, 90 },
    { 0, 0 },
    { 0xedb2f536, 95 },
    { 0, 0 },
    { 0xdb62f538, 33 },
    { 0, 0 },
    { 0x5e12f53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b2f53d, 61 },
    { 0, 0 },
    { 0xbd52f53f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6792f542, 90 },
    { 0, 0 },
    { 0x84a2f544, 23 },
    { 0x4192f545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3162f54a, 61 },
    { 0xcc22f54a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402f54e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e2f552, 90 },
    { 0xb312f553, 46 },
    { 0xf7d2f554, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad82f557, 76 },
    { 0xdcd2f558, 61 },
    { 0, 0 },
    { 0x74c2f55a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a52f55e, 90 },
    { 0xef82f55e, 34 },
    { 0, 0 },
    { 0xd3c2f561, 33 },
    { 0, 0 },
    { 0xd902f563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa2f568, 90 },
    { 0xfd42f569, 93 },
    { 0x7ef2f568, 76 },
    { 0xbbd2f56b, 84 },
    { 0x61e2f56c, 87 },
    { 0x9252f56d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062f572, 34 },
    { 0, 0 },
    { 0xc4e2f574, 33 },
    { 0x9f42f574, 93 },
    { 0x77a2f574, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9182f57b, 23 },
    { 0xd482f57b, 34 },
    { 0xbd32f57b, 61 },
    { 0x8352f57e, 90 },
    { 0xdc22f57e, 55 },
    { 0, 0 },
    { 0x8152f581, 8 },
    { 0xb9d2f582, 33 },
    { 0xfc52f583, 91 },
    { 0, 0 },
    { 0x2472f585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f2f588, 40 },
    { 0x27d2f588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692f58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f597, 33 },
    { 0xa892f598, 98 },
    { 0x3582f599, 33 },
    { 0xb3c2f59a, 23 },
    { 0xf5b2f59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92f5a0, 61 },
    { 0xc792f5a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942f5a5, 3 },
    { 0, 0 },
    { 0xa9d2f5a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f5ad, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd42f5b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b62f5b8, 93 },
    { 0, 0 },
    { 0xc422f5ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef72f5c4, 33 },
    { 0, 0 },
    { 0xfae2f5c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d2f5ca, 76 },
    { 0x6112f5cb, 52 },
    { 0x6212f5ca, 90 },
    { 0, 0 },
    { 0x33e2f5ce, 98 },
    { 0, 0 },
    { 0x5502f5d0, 66 },
    { 0x5e02f5d0, 33 },
    { 0, 0 },
    { 0x3482f5d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a42f5d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322f5db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d62f5e2, 33 },
    { 0, 0 },
    { 0xf7d2f5e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4482f5eb, 34 },
    { 0, 0 },
    { 0x7d92f5ed, 23 },
    { 0, 0 },
    { 0x5502f5ef, 23 },
    { 0x1072f5f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b2f5f3, 93 },
    { 0xa882f5f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f5fa, 78 },
    { 0x84e2f5fa, 76 },
    { 0x1302f5fc, 90 },
    { 0xae12f5fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2012f600, 99 },
    { 0xd9e2f601, 95 },
    { 0x8c52f602, 93 },
    { 0, 0 },
    { 0xd852f604, 33 },
    { 0x9602f605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4802f60b, 90 },
    { 0, 0 },
    { 0xec12f60d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf2f614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502f617, 34 },
    { 0, 0 },
    { 0x5752f619, 33 },
    { 0, 0 },
    { 0xb112f61b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e72f61e, 43 },
    { 0, 0 },
    { 0xee02f620, 33 },
    { 0xac82f620, 34 },
    { 0xbbd2f622, 98 },
    { 0, 0 },
    { 0xa0c2f624, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a72f628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc222f62b, 90 },
    { 0xfcd2f62c, 61 },
    { 0x1f32f62d, 66 },
    { 0, 0 },
    { 0xa432f62f, 61 },
    { 0xb982f62f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee32f634, 61 },
    { 0, 0 },
    { 0x14c2f636, 98 },
    { 0xc0f2f637, 61 },
    { 0xa722f637, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a2f63d, 76 },
    { 0, 0 },
    { 0x62c2f63f, 95 },
    { 0x7212f640, 61 },
    { 0, 0 },
    { 0xf942f642, 98 },
    { 0, 0 },
    { 0x2f62f644, 90 },
    { 0, 0 },
    { 0x9c22f646, 65 },
    { 0x2f2f647, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2f64a, 94 },
    { 0x5cb2f64b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe772f64e, 66 },
    { 0x6e82f64f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382f656, 61 },
    { 0x1eb2f657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b2f65a, 87 },
    { 0x7b22f65a, 95 },
    { 0, 0 },
    { 0x632f65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa2f660, 90 },
    { 0xf262f660, 23 },
    { 0xc722f662, 33 },
    { 0, 0 },
    { 0xbf52f664, 84 },
    { 0, 0 },
    { 0x1e12f666, 40 },
    { 0x9972f667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b2f66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb632f673, 46 },
    { 0x8aa2f673, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb512f67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b2f67f, 77 },
    { 0xab72f680, 40 },
    { 0x6f42f67f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca32f687, 100 },
    { 0xd432f688, 46 },
    { 0x92e2f689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad2f693, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f697, 90 },
    { 0xd8c2f698, 23 },
    { 0xe442f699, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42f6a0, 33 },
    { 0x2002f6a1, 66 },
    { 0x5e2f6a2, 88 },
    { 0xbb02f6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc82f6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f2f6b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4702f6b6, 61 },
    { 0x6e52f6b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3462f6ba, 46 },
    { 0x4e42f6ba, 34 },
    { 0x8a32f6bc, 33 },
    { 0xdc82f6ba, 33 },
    { 0x39a2f6be, 90 },
    { 0x7e12f6bf, 99 },
    { 0x9372f6bd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22f6c9, 66 },
    { 0x12a2f6c9, 61 },
    { 0x2c52f6cb, 33 },
    { 0xcfa2f6c9, 99 },
    { 0, 0 },
    { 0x78d2f6ce, 95 },
    { 0, 0 },
    { 0xdc82f6d0, 95 },
    { 0x9212f6d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x262f6d4, 95 },
    { 0xda42f6d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5592f6db, 85 },
    { 0xb982f6dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2f6e3, 95 },
    { 0, 0 },
    { 0xffa2f6e5, 5 },
    { 0, 0 },
    { 0xaea2f6e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f6ee, 90 },
    { 0x7ca2f6ef, 23 },
    { 0x40b2f6f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792f6f3, 77 },
    { 0x6d32f6f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2f6fc, 98 },
    { 0, 0 },
    { 0xcb12f6fe, 61 },
    { 0, 0 },
    { 0xf4a2f700, 85 },
    { 0xdb32f701, 61 },
    { 0x4a52f702, 93 },
    { 0xc72f701, 61 },
    { 0, 0 },
    { 0xfb2f705, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532f70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba2f70d, 88 },
    { 0x4db2f70e, 90 },
    { 0xf312f70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272f718, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf2f71d, 90 },
    { 0x272f71e, 90 },
    { 0x852f71f, 80 },
    { 0x8652f71f, 95 },
    { 0xb682f71d, 98 },
    { 0, 0 },
    { 0x6d12f723, 90 },
    { 0xc222f724, 33 },
    { 0xc442f723, 46 },
    { 0x35c2f726, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92f729, 91 },
    { 0, 0 },
    { 0xd622f72b, 90 },
    { 0x2512f72c, 90 },
    { 0xb3d2f72d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b62f730, 81 },
    { 0x6702f731, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b2f73b, 33 },
    { 0x1a72f73c, 23 },
    { 0, 0 },
    { 0x4a62f73e, 33 },
    { 0x35b2f73e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1992f742, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f2f745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32f748, 61 },
    { 0xe312f749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb442f750, 55 },
    { 0, 0 },
    { 0x3142f752, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512f756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c2f75a, 84 },
    { 0xe6a2f75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c12f760, 61 },
    { 0, 0 },
    { 0x3e12f762, 98 },
    { 0xe762f763, 33 },
    { 0, 0 },
    { 0x5852f765, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1352f768, 61 },
    { 0xa7a2f769, 98 },
    { 0xcd02f769, 61 },
    { 0, 0 },
    { 0xbed2f76c, 33 },
    { 0xe5a2f76d, 43 },
    { 0x27f2f76d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e2f772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2f777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2f77c, 90 },
    { 0, 0 },
    { 0x7372f77e, 90 },
    { 0xc0e2f77f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2f782, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52f785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2f78a, 95 },
    { 0x2d62f78b, 98 },
    { 0xcc72f78c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b2f794, 76 },
    { 0x2a2f795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d2f79a, 68 },
    { 0x4292f79b, 81 },
    { 0, 0 },
    { 0x5642f79d, 99 },
    { 0x72a2f79e, 98 },
    { 0x39d2f79f, 90 },
    { 0xdce2f7a0, 84 },
    { 0x17a2f79d, 55 },
    { 0x80c2f79d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2f7a7, 95 },
    { 0x50c2f7a7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6102f7ab, 88 },
    { 0xa402f7ac, 61 },
    { 0xd12f7ac, 93 },
    { 0xebd2f7ae, 23 },
    { 0x6e82f7ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32f7b3, 61 },
    { 0x2f7b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c2f7bb, 61 },
    { 0xff2f7bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf42f7c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2f7c5, 23 },
    { 0x3412f7c6, 98 },
    { 0x8412f7c7, 23 },
    { 0x36b2f7c7, 98 },
    { 0x1d52f7c9, 61 },
    { 0xb252f7c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba2f7d0, 95 },
    { 0xf22f7d0, 87 },
    { 0, 0 },
    { 0x602f7d3, 61 },
    { 0x5692f7d4, 100 },
    { 0x7972f7d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912f7d8, 100 },
    { 0xd402f7d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd52f7de, 33 },
    { 0x8422f7de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2f7e2, 76 },
    { 0xb1d2f7e2, 34 },
    { 0x5652f7e3, 55 },
    { 0, 0 },
    { 0x4562f7e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f82f7ec, 66 },
    { 0x9fa2f7ed, 96 },
    { 0xd382f7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9712f7f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2142f7f8, 98 },
    { 0, 0 },
    { 0xb2a2f7fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362f7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b2f801, 61 },
    { 0x3de2f802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b2f806, 90 },
    { 0x8872f807, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6612f80b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b2f814, 67 },
    { 0, 0 },
    { 0x2142f816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b2f825, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef62f82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2f830, 52 },
    { 0x90a2f831, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e92f834, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e2f83a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c22f83f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2232f845, 23 },
    { 0xb6e2f846, 17 },
    { 0x7472f847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e52f84f, 43 },
    { 0, 0 },
    { 0xf482f851, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff2f85e, 52 },
    { 0, 0 },
    { 0x13e2f860, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f2f866, 90 },
    { 0x1e12f867, 61 },
    { 0xbdd2f867, 34 },
    { 0x1b12f869, 23 },
    { 0x3022f867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6602f86e, 85 },
    { 0xf022f86f, 52 },
    { 0, 0 },
    { 0xaa92f871, 40 },
    { 0, 0 },
    { 0xf8c2f873, 99 },
    { 0xba72f873, 52 },
    { 0x5522f875, 34 },
    { 0x6fc2f876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb322f87a, 85 },
    { 0x2d72f87b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac2f87e, 33 },
    { 0x2da2f87f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe2f883, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112f888, 68 },
    { 0, 0 },
    { 0x9382f88a, 95 },
    { 0, 0 },
    { 0x12f88c, 68 },
    { 0, 0 },
    { 0xada2f88e, 61 },
    { 0x7a22f88f, 34 },
    { 0x2402f88e, 23 },
    { 0x2be2f88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d2f895, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc102f898, 34 },
    { 0, 0 },
    { 0x5822f89a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212f89f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e2f8a8, 33 },
    { 0, 0 },
    { 0xe372f8aa, 61 },
    { 0xec2f8ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc42f8ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f2f8b2, 55 },
    { 0x3dd2f8b3, 90 },
    { 0x5012f8b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf92f8b8, 61 },
    { 0, 0 },
    { 0x6a32f8ba, 67 },
    { 0, 0 },
    { 0xbb2f8bc, 77 },
    { 0x2952f8bd, 98 },
    { 0xe4e2f8be, 43 },
    { 0x8cf2f8bf, 84 },
    { 0xaa32f8bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52f8c3, 23 },
    { 0x4f52f8c3, 23 },
    { 0, 0 },
    { 0x4842f8c6, 61 },
    { 0x87b2f8c6, 75 },
    { 0x76f2f8c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b2f8cb, 33 },
    { 0, 0 },
    { 0x61b2f8cd, 90 },
    { 0, 0 },
    { 0xd412f8cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a2f8d2, 90 },
    { 0xc1d2f8d3, 90 },
    { 0, 0 },
    { 0xffb2f8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe582f8d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d82f8df, 90 },
    { 0x63d2f8e0, 23 },
    { 0, 0 },
    { 0xd392f8e2, 52 },
    { 0, 0 },
    { 0x6fb2f8e4, 68 },
    { 0x562f8e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b12f8ed, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2f8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1232f8f7, 80 },
    { 0x5d82f8f8, 95 },
    { 0, 0 },
    { 0x4bb2f8fa, 66 },
    { 0x3892f8fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9492f8fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762f904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a2f909, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552f910, 95 },
    { 0x2582f911, 90 },
    { 0, 0 },
    { 0x4e02f913, 61 },
    { 0xd442f914, 93 },
    { 0x4642f913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752f91d, 90 },
    { 0, 0 },
    { 0xa5c2f91f, 95 },
    { 0, 0 },
    { 0x6222f921, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b82f925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7652f92b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7852f931, 67 },
    { 0, 0 },
    { 0xd442f933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042f93d, 85 },
    { 0, 0 },
    { 0xf3a2f93f, 83 },
    { 0xe752f93f, 46 },
    { 0xbfa2f941, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2f948, 90 },
    { 0, 0 },
    { 0x42a2f94a, 55 },
    { 0, 0 },
    { 0xa022f94c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2f94f, 52 },
    { 0xbe82f950, 43 },
    { 0x2802f951, 90 },
    { 0x4692f951, 46 },
    { 0xa82f950, 76 },
    { 0x78d2f954, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf22f958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a92f95b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d2f95e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962f961, 43 },
    { 0, 0 },
    { 0xff32f963, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2f966, 95 },
    { 0, 0 },
    { 0xdc82f968, 61 },
    { 0x3e12f969, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf812f96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62f972, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d22f978, 77 },
    { 0x5b32f979, 33 },
    { 0xa9e2f97a, 93 },
    { 0x5fe2f97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02f982, 95 },
    { 0x71c2f982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f82f987, 95 },
    { 0, 0 },
    { 0xa4b2f989, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2f98c, 90 },
    { 0xde02f98d, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82f990, 33 },
    { 0xee22f990, 98 },
    { 0, 0 },
    { 0x972f993, 93 },
    { 0xa8c2f994, 90 },
    { 0x9682f995, 95 },
    { 0xf402f995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4632f99f, 34 },
    { 0x67b2f9a0, 33 },
    { 0x612f9a1, 90 },
    { 0x68a2f99f, 17 },
    { 0x52b2f9a3, 61 },
    { 0x26b2f9a4, 46 },
    { 0xf3e2f9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc2f9aa, 87 },
    { 0xf6e2f9ab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b22f9b4, 87 },
    { 0x7872f9b4, 91 },
    { 0, 0 },
    { 0xf852f9b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1032f9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb442f9c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d2f9c7, 90 },
    { 0x2a92f9c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd2f9cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2f9d1, 90 },
    { 0xc02f9d2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2f9d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532f9d9, 85 },
    { 0xec92f9da, 40 },
    { 0x3182f9d9, 90 },
    { 0xfa32f9dc, 23 },
    { 0, 0 },
    { 0xd9c2f9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b2f9e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2f9e7, 55 },
    { 0xd352f9e8, 90 },
    { 0xdd52f9e8, 99 },
    { 0x3292f9e7, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2f9ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932f9f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c2f9f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c2fa05, 46 },
    { 0x8042fa06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c82fa09, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c22fa0d, 66 },
    { 0x7172fa0e, 61 },
    { 0xc0a2fa0e, 98 },
    { 0xbc52fa0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd2fa15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d2fa1f, 34 },
    { 0x3a62fa20, 77 },
    { 0xf1e2fa21, 52 },
    { 0, 0 },
    { 0x4242fa23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5322fa26, 90 },
    { 0xd62fa27, 61 },
    { 0, 0 },
    { 0x9fe2fa29, 66 },
    { 0x32fa29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372fa32, 61 },
    { 0x1f12fa33, 23 },
    { 0x8462fa32, 99 },
    { 0x2282fa35, 23 },
    { 0x872fa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092fa3a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e2fa3d, 98 },
    { 0x8a62fa3d, 61 },
    { 0x3382fa3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2fa44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8802fa4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b52fa4f, 76 },
    { 0xa452fa50, 43 },
    { 0x81b2fa50, 55 },
    { 0x59f2fa52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2fa55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe752fa59, 33 },
    { 0, 0 },
    { 0xfa82fa5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb452fa5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f32fa63, 43 },
    { 0xe512fa64, 98 },
    { 0xfdb2fa65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152fa6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752fa6f, 76 },
    { 0, 0 },
    { 0x81a2fa71, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2fa77, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92fa83, 33 },
    { 0x45d2fa84, 100 },
    { 0, 0 },
    { 0x85e2fa86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f32fa8f, 95 },
    { 0xb842fa90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe2fa97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c2fa9a, 33 },
    { 0x96f2fa9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e42fa9e, 61 },
    { 0x3992fa9f, 90 },
    { 0x46d2fa9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c52faa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2faa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9362faac, 55 },
    { 0, 0 },
    { 0x9e22faae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa742fab2, 90 },
    { 0, 0 },
    { 0xefb2fab4, 81 },
    { 0, 0 },
    { 0xbc12fab6, 61 },
    { 0x1622fab6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9492faba, 98 },
    { 0, 0 },
    { 0x4272fabc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3842fac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6282fac6, 43 },
    { 0x4da2fac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2facb, 66 },
    { 0x7c92facc, 33 },
    { 0, 0 },
    { 0x8d12face, 46 },
    { 0xf262facf, 99 },
    { 0x9982facf, 55 },
    { 0xd802fad1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1542fad4, 76 },
    { 0, 0 },
    { 0x5982fad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852fadb, 95 },
    { 0, 0 },
    { 0x4152fadd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a32fae1, 81 },
    { 0x9792fae2, 95 },
    { 0xef92fae3, 98 },
    { 0x83d2fae4, 84 },
    { 0, 0 },
    { 0xb102fae6, 23 },
    { 0, 0 },
    { 0x252fae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de2faec, 98 },
    { 0xebb2faed, 76 },
    { 0xb62faed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a2faf2, 90 },
    { 0, 0 },
    { 0x1cc2faf4, 90 },
    { 0xa282faf4, 98 },
    { 0, 0 },
    { 0x43c2faf7, 91 },
    { 0, 0 },
    { 0x36d2faf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72fafc, 55 },
    { 0, 0 },
    { 0x3d52fafe, 23 },
    { 0, 0 },
    { 0xd632fb00, 81 },
    { 0, 0 },
    { 0x3a2fb02, 43 },
    { 0x31b2fb03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc2fb10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2fb14, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fb18, 95 },
    { 0, 0 },
    { 0xc502fb1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982fb1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d92fb22, 17 },
    { 0x9362fb23, 84 },
    { 0x8582fb22, 98 },
    { 0x6bd2fb25, 87 },
    { 0x9702fb24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa2fb2a, 66 },
    { 0x7502fb2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa612fb32, 95 },
    { 0, 0 },
    { 0xede2fb34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa002fb38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2002fb3c, 46 },
    { 0, 0 },
    { 0x5a72fb3e, 95 },
    { 0, 0 },
    { 0x85d2fb40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2fb4c, 80 },
    { 0xe5f2fb4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de2fb52, 98 },
    { 0, 0 },
    { 0x9012fb54, 34 },
    { 0x2262fb55, 61 },
    { 0x8f92fb55, 43 },
    { 0, 0 },
    { 0x64f2fb58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2fb5b, 46 },
    { 0x32a2fb5c, 69 },
    { 0xa852fb5c, 84 },
    { 0xd1d2fb5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4682fb61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb352fb66, 77 },
    { 0x2ca2fb67, 34 },
    { 0, 0 },
    { 0xb372fb69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8742fb70, 90 },
    { 0x8652fb71, 23 },
    { 0, 0 },
    { 0xcaf2fb73, 84 },
    { 0x7a2fb73, 93 },
    { 0xb732fb75, 61 },
    { 0, 0 },
    { 0x9122fb77, 76 },
    { 0x3cb2fb78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e12fb7e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb72fb86, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2fb89, 61 },
    { 0x4ed2fb8a, 33 },
    { 0x2b2fb8a, 87 },
    { 0, 0 },
    { 0x3c72fb8d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd922fb90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3632fb93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2fb98, 84 },
    { 0, 0 },
    { 0x1512fb9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc252fb9d, 23 },
    { 0x9382fb9e, 91 },
    { 0x7aa2fb9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392fba2, 23 },
    { 0x8fc2fba3, 90 },
    { 0x2d82fba4, 87 },
    { 0xdae2fba5, 46 },
    { 0xa762fba4, 90 },
    { 0x76b2fba7, 33 },
    { 0x8b82fba4, 78 },
    { 0xe922fba5, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc12fbb2, 95 },
    { 0, 0 },
    { 0x83e2fbb4, 90 },
    { 0x8d52fbb5, 93 },
    { 0x172fbb6, 38 },
    { 0, 0 },
    { 0x7e52fbb8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882fbbb, 98 },
    { 0, 0 },
    { 0xc62fbbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692fbc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1622fbc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e2fbca, 95 },
    { 0xa892fbcb, 55 },
    { 0xc012fbca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752fbd2, 95 },
    { 0xaf22fbd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd2fbd6, 23 },
    { 0xa4d2fbd6, 87 },
    { 0x6e32fbd8, 98 },
    { 0x9be2fbd9, 90 },
    { 0x682fbd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec2fbde, 43 },
    { 0, 0 },
    { 0xec2fbe0, 75 },
    { 0xf62fbe1, 55 },
    { 0, 0 },
    { 0xad52fbe3, 69 },
    { 0xe802fbe3, 98 },
    { 0x8612fbe5, 95 },
    { 0, 0 },
    { 0xad92fbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc62fbeb, 55 },
    { 0x99e2fbec, 98 },
    { 0xdde2fbec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1612fbf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442fbf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7462fbfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9862fc02, 85 },
    { 0xd772fc02, 55 },
    { 0xf902fc04, 90 },
    { 0x3172fc05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d2fc0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6092fc11, 12 },
    { 0xa552fc12, 93 },
    { 0x7ed2fc13, 95 },
    { 0x68f2fc13, 33 },
    { 0xc6c2fc15, 23 },
    { 0, 0 },
    { 0xf142fc17, 61 },
    { 0, 0 },
    { 0x5c62fc19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02fc1c, 55 },
    { 0, 0 },
    { 0x15d2fc1e, 46 },
    { 0, 0 },
    { 0xe1f2fc20, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb2fc23, 77 },
    { 0xdd72fc24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe2fc2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3322fc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172fc31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912fc39, 34 },
    { 0x762fc3a, 85 },
    { 0xa62fc3b, 95 },
    { 0xa772fc3a, 61 },
    { 0x452fc3b, 67 },
    { 0x1a22fc3e, 55 },
    { 0, 0 },
    { 0xd852fc40, 23 },
    { 0x5de2fc41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9502fc46, 61 },
    { 0, 0 },
    { 0x5422fc48, 61 },
    { 0xa932fc48, 90 },
    { 0x9f82fc4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622fc4f, 34 },
    { 0x5da2fc4f, 33 },
    { 0x96f2fc4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc2fc55, 90 },
    { 0x14d2fc56, 61 },
    { 0x2492fc57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2fc5a, 56 },
    { 0xe632fc5b, 84 },
    { 0xe852fc5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2712fc60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1642fc66, 90 },
    { 0, 0 },
    { 0x28e2fc68, 76 },
    { 0x6792fc69, 65 },
    { 0, 0 },
    { 0xe102fc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c2fc78, 33 },
    { 0x80f2fc79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082fc7c, 90 },
    { 0, 0 },
    { 0xe8b2fc7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a2fc82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802fc87, 66 },
    { 0x92d2fc88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e2fc8b, 61 },
    { 0, 0 },
    { 0xa162fc8d, 95 },
    { 0, 0 },
    { 0x1ae2fc8f, 90 },
    { 0, 0 },
    { 0xbb92fc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152fc94, 62 },
    { 0x4e2fc94, 66 },
    { 0xd3b2fc96, 43 },
    { 0xae72fc94, 91 },
    { 0xb52fc98, 33 },
    { 0xa052fc94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb362fc9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d22fc9f, 23 },
    { 0, 0 },
    { 0x3152fca1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1812fca6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4482fcaa, 100 },
    { 0x52a2fcab, 23 },
    { 0, 0 },
    { 0x9182fcad, 90 },
    { 0xf602fcad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222fcb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd2fcba, 61 },
    { 0, 0 },
    { 0xe612fcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392fcbf, 81 },
    { 0, 0 },
    { 0xfe82fcc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2fcc4, 51 },
    { 0x2f32fcc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442fcc9, 33 },
    { 0, 0 },
    { 0xa8d2fccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512fcd0, 61 },
    { 0x8342fcd0, 33 },
    { 0xaff2fcd1, 33 },
    { 0x6392fcd3, 61 },
    { 0xe5c2fcd4, 67 },
    { 0x6952fcd4, 33 },
    { 0xf172fcd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf2fcda, 76 },
    { 0xcfe2fcda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1132fcdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3362fce4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52fcea, 76 },
    { 0xa4c2fceb, 75 },
    { 0xf92fcec, 61 },
    { 0xd6a2fcec, 61 },
    { 0xe882fcee, 91 },
    { 0x8bf2fcef, 95 },
    { 0x5322fcea, 61 },
    { 0, 0 },
    { 0xfde2fcf2, 34 },
    { 0x88c2fcf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a92fcf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72fcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2fd00, 61 },
    { 0, 0 },
    { 0xe6f2fd02, 55 },
    { 0, 0 },
    { 0xc5b2fd04, 76 },
    { 0xb062fd05, 33 },
    { 0x2752fd06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6132fd1b, 98 },
    { 0x2962fd1b, 100 },
    { 0x6932fd1c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602fd20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc82fd23, 90 },
    { 0, 0 },
    { 0x95a2fd25, 61 },
    { 0, 0 },
    { 0xf242fd27, 46 },
    { 0xe3f2fd27, 34 },
    { 0, 0 },
    { 0x6f72fd2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152fd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e2fd30, 33 },
    { 0x7d52fd31, 43 },
    { 0xc72fd32, 76 },
    { 0xfde2fd33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c2fd3a, 65 },
    { 0, 0 },
    { 0xdb2fd3c, 95 },
    { 0x5532fd3d, 34 },
    { 0xb172fd3e, 88 },
    { 0xf592fd3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe72fd44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d2fd48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc2fd4b, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2962fd4e, 69 },
    { 0, 0 },
    { 0x2d22fd50, 97 },
    { 0, 0 },
    { 0xbd42fd52, 98 },
    { 0x4622fd53, 98 },
    { 0xd632fd54, 38 },
    { 0x5ae2fd54, 100 },
    { 0xfd2fd55, 33 },
    { 0xef12fd56, 23 },
    { 0, 0 },
    { 0x6d22fd59, 17 },
    { 0xdb62fd59, 84 },
    { 0xb302fd59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a2fd62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e2fd68, 84 },
    { 0xe292fd68, 90 },
    { 0x2942fd68, 95 },
    { 0x2182fd68, 93 },
    { 0xa5d2fd6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192fd6f, 76 },
    { 0x21d2fd6f, 46 },
    { 0x9402fd71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b2fd74, 90 },
    { 0x9cf2fd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b92fd78, 33 },
    { 0x1ed2fd78, 95 },
    { 0x79e2fd7a, 95 },
    { 0xe232fd7b, 78 },
    { 0xdf12fd7c, 78 },
    { 0xa272fd7d, 90 },
    { 0xd372fd7c, 76 },
    { 0xdf82fd78, 43 },
    { 0x96c2fd80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2fd85, 46 },
    { 0x8842fd85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4192fd89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f32fd8e, 38 },
    { 0, 0 },
    { 0x40f2fd90, 98 },
    { 0xc462fd90, 76 },
    { 0x9712fd92, 90 },
    { 0xb62fd93, 74 },
    { 0, 0 },
    { 0x5b72fd95, 43 },
    { 0x2f82fd95, 95 },
    { 0, 0 },
    { 0x9d22fd98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2fd9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722fda1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12fda4, 95 },
    { 0x3502fda4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe22fdaa, 46 },
    { 0x88b2fdaa, 67 },
    { 0x4892fdac, 23 },
    { 0x9102fdaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff72fdb1, 76 },
    { 0x72fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682fdb5, 67 },
    { 0xdce2fdb6, 87 },
    { 0xbfe2fdb5, 98 },
    { 0x1e22fdb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d2fdbb, 95 },
    { 0x9f62fdbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb052fdbf, 61 },
    { 0, 0 },
    { 0x4392fdc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72fdc4, 23 },
    { 0x1392fdc5, 83 },
    { 0xec72fdc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772fdd0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea2fdd5, 33 },
    { 0, 0 },
    { 0xaae2fdd7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572fdda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2fde5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da2fded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72fdf0, 33 },
    { 0, 0 },
    { 0x6ee2fdf2, 34 },
    { 0x1ea2fdf3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf772fdf6, 88 },
    { 0x1792fdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2fe00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d2fe03, 77 },
    { 0x31a2fe04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c2fe07, 90 },
    { 0xcda2fe08, 43 },
    { 0x97c2fe08, 43 },
    { 0xa2c2fe08, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca2fe0f, 67 },
    { 0x242fe10, 95 },
    { 0x10a2fe11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad62fe15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f72fe1d, 98 },
    { 0x55d2fe1d, 33 },
    { 0xacf2fe1f, 33 },
    { 0xfcf2fe20, 95 },
    { 0xc1a2fe1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5062fe24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc2fe2a, 90 },
    { 0x6b92fe2b, 69 },
    { 0xe7a2fe2b, 99 },
    { 0x9d12fe2d, 33 },
    { 0, 0 },
    { 0x4e42fe2f, 95 },
    { 0, 0 },
    { 0x7712fe31, 43 },
    { 0x7472fe32, 90 },
    { 0, 0 },
    { 0x7c52fe34, 23 },
    { 0, 0 },
    { 0x2312fe36, 33 },
    { 0x19b2fe37, 33 },
    { 0x1632fe37, 90 },
    { 0xf022fe39, 33 },
    { 0x5ea2fe37, 90 },
    { 0, 0 },
    { 0x7082fe3c, 90 },
    { 0xe4b2fe3c, 23 },
    { 0x3c32fe3e, 87 },
    { 0xdfc2fe3f, 91 },
    { 0x2d12fe3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2fe43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4502fe48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b2fe4d, 95 },
    { 0, 0 },
    { 0xd072fe4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d72fe53, 23 },
    { 0xe1b2fe53, 33 },
    { 0x3c22fe55, 95 },
    { 0x8282fe55, 52 },
    { 0x2a42fe54, 23 },
    { 0xa792fe58, 52 },
    { 0x2d02fe57, 23 },
    { 0x9042fe54, 100 },
    { 0, 0 },
    { 0xe0c2fe5c, 33 },
    { 0, 0 },
    { 0x5fe2fe5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d2fe61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82fe67, 90 },
    { 0x9412fe67, 43 },
    { 0, 0 },
    { 0x7b12fe6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e02fe6f, 76 },
    { 0, 0 },
    { 0x5982fe71, 98 },
    { 0, 0 },
    { 0x4562fe73, 99 },
    { 0x7672fe73, 90 },
    { 0x24a2fe74, 98 },
    { 0, 0 },
    { 0xe412fe77, 98 },
    { 0x91d2fe78, 98 },
    { 0x2f32fe79, 66 },
    { 0xd782fe79, 96 },
    { 0, 0 },
    { 0x7972fe7c, 93 },
    { 0, 0 },
    { 0x1562fe7e, 95 },
    { 0xf792fe7f, 90 },
    { 0xa5d2fe7f, 98 },
    { 0xd3e2fe80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba12fe84, 90 },
    { 0, 0 },
    { 0xc82fe86, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd72fe8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7752fe95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe2fe98, 43 },
    { 0x912fe99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52fe9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2fea1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2fea4, 61 },
    { 0, 0 },
    { 0x3a82fea6, 66 },
    { 0xc102fea7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e2feab, 95 },
    { 0xc0f2feab, 61 },
    { 0xb992fead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee2feb2, 91 },
    { 0xd782feb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902feba, 76 },
    { 0xec02feba, 90 },
    { 0x3992feba, 90 },
    { 0xc502febd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd2fed0, 91 },
    { 0x78a2fed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6752fed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72fed9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5782fedc, 55 },
    { 0, 0 },
    { 0xa1d2fede, 61 },
    { 0xdbe2fedf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d72fee2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042fee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd2feea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff12fef2, 33 },
    { 0xe2d2fef3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c2fef7, 66 },
    { 0xff32fef8, 46 },
    { 0x16e2fef7, 33 },
    { 0xfa12fef8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af2fefd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922ff00, 87 },
    { 0x95b2ff01, 77 },
    { 0xeef2ff01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6232ff08, 33 },
    { 0, 0 },
    { 0x1a72ff0a, 33 },
    { 0xdf02ff0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92ff11, 17 },
    { 0xcf72ff12, 76 },
    { 0x1f02ff13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2642ff18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e12ff1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2ff1f, 43 },
    { 0x4b2ff1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce2ff26, 43 },
    { 0, 0 },
    { 0xd452ff28, 55 },
    { 0, 0 },
    { 0xc152ff2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e2ff2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2ff30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292ff34, 33 },
    { 0x52a2ff35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed2ff38, 95 },
    { 0, 0 },
    { 0x8ed2ff3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792ff3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4362ff42, 93 },
    { 0, 0 },
    { 0x33a2ff44, 33 },
    { 0xd732ff45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc2ff4b, 87 },
    { 0, 0 },
    { 0xa692ff4d, 90 },
    { 0xb362ff4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5872ff51, 90 },
    { 0, 0 },
    { 0x4032ff53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f2ff58, 83 },
    { 0x9402ff58, 100 },
    { 0xd452ff5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce2ff64, 55 },
    { 0xc1f2ff65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22ff6b, 68 },
    { 0x7a22ff6b, 61 },
    { 0xa312ff6d, 90 },
    { 0x44e2ff6e, 95 },
    { 0x86c2ff6f, 91 },
    { 0x1762ff6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe612ff77, 90 },
    { 0, 0 },
    { 0xfe2ff79, 46 },
    { 0xd72ff7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f12ff81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2ff84, 98 },
    { 0, 0 },
    { 0xcab2ff86, 91 },
    { 0, 0 },
    { 0xc0e2ff88, 90 },
    { 0x1162ff89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62ff90, 100 },
    { 0xe812ff90, 33 },
    { 0x3f52ff92, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d2ff95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ff98, 61 },
    { 0, 0 },
    { 0xca32ff9a, 87 },
    { 0x3682ff9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed2ffa2, 61 },
    { 0x70c2ffa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2ffa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7422ffab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab82ffb5, 38 },
    { 0x2d52ffb5, 61 },
    { 0x2332ffb5, 90 },
    { 0xf492ffb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b2ffbe, 46 },
    { 0, 0 },
    { 0x6412ffc0, 93 },
    { 0xdc82ffc1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f92ffc7, 61 },
    { 0xade2ffc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32ffd6, 80 },
    { 0, 0 },
    { 0x4ad2ffd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3652ffde, 55 },
    { 0x3f42ffdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c2ffe4, 23 },
    { 0xf6d2ffe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b2ffed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32fff5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c42fffa, 77 },
    { 0xf492fffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7782fffe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca230004, 84 },
    { 0x67c30005, 90 },
    { 0x90030006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40230009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e3000d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f30011, 61 },
    { 0x86630012, 33 },
    { 0xbcb30013, 46 },
    { 0, 0 },
    { 0x33330015, 98 },
    { 0x8f130016, 33 },
    { 0x50530017, 99 },
    { 0, 0 },
    { 0xae530019, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65230021, 26 },
    { 0xf3330022, 55 },
    { 0xa2e30023, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf73002d, 90 },
    { 0x7dd3002d, 23 },
    { 0x7543002f, 95 },
    { 0xd5930030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea30037, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2213004d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff30050, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b30054, 76 },
    { 0x4ca30055, 81 },
    { 0xe5a30056, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b530059, 90 },
    { 0x1723005a, 55 },
    { 0x2543005a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4130060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce30063, 23 },
    { 0, 0 },
    { 0x65330065, 33 },
    { 0x42a30066, 76 },
    { 0x94e30067, 34 },
    { 0x4e130066, 75 },
    { 0x70030068, 47 },
    { 0xa0c30067, 17 },
    { 0xc933006b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a530072, 23 },
    { 0, 0 },
    { 0xaf330074, 90 },
    { 0x2a030075, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0030078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c3007b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac30081, 33 },
    { 0x7cd30082, 98 },
    { 0, 0 },
    { 0x77130084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e230097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778300a0, 78 },
    { 0x81300a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x420300a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774300ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7300af, 33 },
    { 0xb81300b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d300b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c300bc, 90 },
    { 0x906300bd, 33 },
    { 0xf6e300bc, 90 },
    { 0x7d300bc, 90 },
    { 0xbc2300c0, 46 },
    { 0x8b1300bd, 98 },
    { 0xbcc300c2, 93 },
    { 0xcfc300c3, 100 },
    { 0x48f300c4, 33 },
    { 0x4ee300c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a300ca, 93 },
    { 0xd1f300cb, 76 },
    { 0xa4d300cb, 33 },
    { 0x97e300cd, 23 },
    { 0xa4a300ca, 93 },
    { 0xbdd300ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5300d8, 40 },
    { 0xb67300d9, 90 },
    { 0x50c300da, 76 },
    { 0x7b5300d9, 90 },
    { 0, 0 },
    { 0xc2f300dd, 96 },
    { 0, 0 },
    { 0x782300df, 69 },
    { 0, 0 },
    { 0x489300e1, 90 },
    { 0x509300e2, 61 },
    { 0, 0 },
    { 0xe9b300e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x917300e7, 85 },
    { 0, 0 },
    { 0x7e9300e9, 90 },
    { 0, 0 },
    { 0x413300eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9300ee, 98 },
    { 0, 0 },
    { 0x677300f0, 84 },
    { 0xb28300f1, 23 },
    { 0, 0 },
    { 0xb4f300f3, 85 },
    { 0, 0 },
    { 0xff5300f5, 98 },
    { 0xe4300f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22300fb, 90 },
    { 0, 0 },
    { 0x6f8300fd, 93 },
    { 0x4a4300fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e30102, 95 },
    { 0xe9f30103, 64 },
    { 0x13130104, 88 },
    { 0x8c330105, 43 },
    { 0x89b30106, 67 },
    { 0xfeb30107, 55 },
    { 0x79630106, 38 },
    { 0x69130109, 90 },
    { 0xfa030105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e30110, 33 },
    { 0, 0 },
    { 0x32f30112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1530115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23011d, 23 },
    { 0xb453011d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83430128, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9693012c, 55 },
    { 0, 0 },
    { 0x4f3012e, 61 },
    { 0, 0 },
    { 0xd1830130, 76 },
    { 0, 0 },
    { 0xb5930132, 66 },
    { 0x84130132, 76 },
    { 0xbc730134, 95 },
    { 0x32730135, 90 },
    { 0x1ed30135, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30139, 76 },
    { 0xaee3013a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5953013e, 23 },
    { 0x8583013f, 55 },
    { 0, 0 },
    { 0x45230141, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67030145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc30148, 55 },
    { 0x1d930148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b3014c, 86 },
    { 0xa753014c, 98 },
    { 0x2323014c, 33 },
    { 0xef33014f, 55 },
    { 0x3a13014c, 90 },
    { 0xee230151, 33 },
    { 0, 0 },
    { 0x50030153, 65 },
    { 0xd9130154, 95 },
    { 0xfc330155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be30158, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4883015e, 33 },
    { 0x3913015f, 77 },
    { 0x3f630160, 23 },
    { 0x1233015f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8230168, 23 },
    { 0, 0 },
    { 0x7773016a, 33 },
    { 0, 0 },
    { 0x6af3016c, 90 },
    { 0xd373016d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a30175, 78 },
    { 0xeee30175, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca30179, 33 },
    { 0, 0 },
    { 0xb903017b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a230183, 61 },
    { 0x44230184, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2730188, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d3018b, 98 },
    { 0x8083018c, 77 },
    { 0xa163018d, 95 },
    { 0x63f3018d, 98 },
    { 0xfad3018f, 34 },
    { 0x83c30190, 81 },
    { 0x8763018f, 98 },
    { 0x8b73018f, 98 },
    { 0xf8c30193, 46 },
    { 0xbae30191, 46 },
    { 0x6463018b, 33 },
    { 0x8283018f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x36630199, 46 },
    { 0x2453019a, 33 },
    { 0x6f930199, 23 },
    { 0x8893019c, 67 },
    { 0x66f3019d, 95 },
    { 0x6f93019b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777301a4, 93 },
    { 0, 0 },
    { 0xf3c301a6, 70 },
    { 0x807301a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca301aa, 34 },
    { 0x1b1301aa, 76 },
    { 0xde0301ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x745301af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a301b2, 60 },
    { 0xb0d301b3, 34 },
    { 0, 0 },
    { 0x5b301b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7301c1, 99 },
    { 0, 0 },
    { 0x431301c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc301c9, 23 },
    { 0xf3a301ca, 76 },
    { 0xdcf301cb, 90 },
    { 0xae1301cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00301d1, 43 },
    { 0x55b301d2, 68 },
    { 0x8fb301d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e301d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2301dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4301e3, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26301e7, 17 },
    { 0, 0 },
    { 0x4a2301e9, 66 },
    { 0, 0 },
    { 0x8e9301eb, 23 },
    { 0xe5f301ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b301ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7301f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x497301f6, 85 },
    { 0x566301f6, 23 },
    { 0x2f4301f8, 85 },
    { 0, 0 },
    { 0xca1301fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7230202, 95 },
    { 0, 0 },
    { 0xdac30204, 93 },
    { 0, 0 },
    { 0x72730206, 17 },
    { 0x20430207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3020a, 65 },
    { 0, 0 },
    { 0xe863020c, 61 },
    { 0x16f3020c, 33 },
    { 0xe103020e, 23 },
    { 0, 0 },
    { 0xc330210, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b730216, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb163021b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd030223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97930229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef3022c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1930233, 76 },
    { 0xbe530234, 33 },
    { 0, 0 },
    { 0x6d230236, 91 },
    { 0x6ba30237, 90 },
    { 0, 0 },
    { 0xb8630239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3933023c, 55 },
    { 0x9a3023d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2230242, 76 },
    { 0x39530242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b730246, 46 },
    { 0x77730247, 90 },
    { 0x45130246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe3024c, 98 },
    { 0, 0 },
    { 0x48f3024e, 96 },
    { 0xa3c3024f, 90 },
    { 0, 0 },
    { 0x84630251, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f30256, 33 },
    { 0, 0 },
    { 0x45d30258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa903025e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9130261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30330268, 85 },
    { 0, 0 },
    { 0xe8b3026a, 80 },
    { 0, 0 },
    { 0xa8a3026c, 98 },
    { 0xe63026d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b30276, 33 },
    { 0, 0 },
    { 0xaa530278, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3913027c, 23 },
    { 0x7bd3027d, 61 },
    { 0, 0 },
    { 0x3123027f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e30284, 98 },
    { 0x98530285, 95 },
    { 0xf5e30285, 46 },
    { 0x95230287, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33028a, 90 },
    { 0xccf3028b, 17 },
    { 0, 0 },
    { 0x2b83028d, 55 },
    { 0x6c23028e, 88 },
    { 0x90f3028d, 65 },
    { 0x6553028d, 90 },
    { 0, 0 },
    { 0x72b30292, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98630297, 33 },
    { 0x56f30298, 90 },
    { 0, 0 },
    { 0xd803029a, 76 },
    { 0x8df3029b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46302a1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8302a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1302ab, 90 },
    { 0, 0 },
    { 0x676302ad, 34 },
    { 0x90a302ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af302b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8302ba, 90 },
    { 0, 0 },
    { 0x540302bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750302c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab302c7, 99 },
    { 0x6f9302c7, 43 },
    { 0, 0 },
    { 0x6a7302ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be302d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a302d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc302db, 90 },
    { 0, 0 },
    { 0x24f302dd, 78 },
    { 0x7c5302de, 46 },
    { 0x352302dd, 76 },
    { 0, 0 },
    { 0x56302e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7302e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141302e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4302ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11302f1, 33 },
    { 0x39302f1, 88 },
    { 0x66d302f1, 33 },
    { 0, 0 },
    { 0x8f6302f5, 87 },
    { 0xc0d302f5, 90 },
    { 0x7a6302f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e302fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1130300, 95 },
    { 0, 0 },
    { 0x70b30302, 23 },
    { 0xfd030303, 100 },
    { 0, 0 },
    { 0xab30305, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d30309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7713030c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66930312, 66 },
    { 0, 0 },
    { 0xdd430314, 33 },
    { 0xd6330314, 90 },
    { 0, 0 },
    { 0xbdf30317, 95 },
    { 0xaff30317, 90 },
    { 0xcdf30319, 85 },
    { 0x30230319, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9030322, 34 },
    { 0x7a230323, 90 },
    { 0x16330322, 66 },
    { 0, 0 },
    { 0x99c30326, 98 },
    { 0x4930327, 67 },
    { 0x17530327, 61 },
    { 0x86930327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2630330, 95 },
    { 0, 0 },
    { 0xc9630332, 34 },
    { 0xa7830333, 34 },
    { 0x75130334, 90 },
    { 0xca730334, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25430338, 33 },
    { 0x4fc30338, 33 },
    { 0xe1c3033a, 36 },
    { 0x5953033b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf3033e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2330343, 98 },
    { 0x49e30344, 90 },
    { 0x70930345, 98 },
    { 0, 0 },
    { 0xd7d30347, 90 },
    { 0, 0 },
    { 0x6cc30349, 78 },
    { 0, 0 },
    { 0xd63034b, 61 },
    { 0x7603034b, 98 },
    { 0xdf3034b, 100 },
    { 0xac83034e, 36 },
    { 0x2283034f, 98 },
    { 0x7493034d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38430354, 95 },
    { 0x71130355, 43 },
    { 0x72130356, 98 },
    { 0x75230354, 96 },
    { 0xc2d30358, 95 },
    { 0x78630359, 91 },
    { 0x58f3035a, 90 },
    { 0, 0 },
    { 0x7de3035c, 75 },
    { 0, 0 },
    { 0x4033035e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d730364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b3036d, 23 },
    { 0xc833036e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b730371, 55 },
    { 0, 0 },
    { 0xdbd30373, 95 },
    { 0x4c130374, 33 },
    { 0, 0 },
    { 0xd5b30376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d3037a, 87 },
    { 0, 0 },
    { 0xec53037c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a130382, 61 },
    { 0xdf30383, 81 },
    { 0x92e30384, 98 },
    { 0x6ce30385, 33 },
    { 0xf5d30385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3038b, 61 },
    { 0, 0 },
    { 0x3543038d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b030390, 61 },
    { 0x52c30390, 90 },
    { 0x18330392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1930397, 61 },
    { 0x34330397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19303a0, 90 },
    { 0x2e1303a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9303a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c303a8, 96 },
    { 0, 0 },
    { 0xf2c303aa, 23 },
    { 0x426303aa, 33 },
    { 0xd34303ac, 52 },
    { 0x46d303ad, 33 },
    { 0xa59303ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c303b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e303ba, 98 },
    { 0xe78303ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e303be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed303c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462303c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d303cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2303d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42303d6, 75 },
    { 0, 0 },
    { 0xe42303d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xece303db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c303de, 33 },
    { 0x71a303de, 33 },
    { 0x7b9303e0, 33 },
    { 0xf29303e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b303e7, 61 },
    { 0x27303e8, 95 },
    { 0xf25303e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d303ee, 23 },
    { 0x54b303ef, 81 },
    { 0xd3303f0, 33 },
    { 0, 0 },
    { 0x9aa303f2, 67 },
    { 0x737303f3, 34 },
    { 0x760303f2, 61 },
    { 0x230303f5, 84 },
    { 0xf04303f6, 43 },
    { 0x106303f7, 33 },
    { 0xd28303f3, 85 },
    { 0x40d303f5, 46 },
    { 0x97d303fa, 93 },
    { 0x98303f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce30404, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40930407, 66 },
    { 0x35a30408, 66 },
    { 0, 0 },
    { 0xcbf3040a, 55 },
    { 0x3813040b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3813040e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e30414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d30417, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf63041b, 61 },
    { 0x7713041c, 75 },
    { 0x1bf3041d, 61 },
    { 0xfcd3041e, 90 },
    { 0x1b3041f, 95 },
    { 0xf7f30420, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67830425, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68230429, 33 },
    { 0x6a43042a, 19 },
    { 0xed73042b, 100 },
    { 0xb873042a, 98 },
    { 0xd523042d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc30437, 52 },
    { 0xb7e30438, 49 },
    { 0x11d30439, 33 },
    { 0, 0 },
    { 0x9eb3043b, 90 },
    { 0xd843043c, 99 },
    { 0, 0 },
    { 0x9d93043e, 78 },
    { 0x9763043e, 90 },
    { 0, 0 },
    { 0x6dc30441, 100 },
    { 0x6c230442, 90 },
    { 0, 0 },
    { 0xa9930444, 61 },
    { 0, 0 },
    { 0x22c30446, 90 },
    { 0xf9130446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3903044e, 98 },
    { 0xb7d3044f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd230457, 61 },
    { 0xe1a30457, 91 },
    { 0x11330459, 90 },
    { 0xca73045a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663045e, 34 },
    { 0xe1e3045f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32630463, 33 },
    { 0xb7030463, 33 },
    { 0xa8e30463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf30468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a3046d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac30474, 33 },
    { 0xf0c30474, 61 },
    { 0x3e230476, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a30483, 90 },
    { 0x26430484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1330487, 85 },
    { 0x5b630488, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc963048e, 61 },
    { 0x8f3048f, 90 },
    { 0x39530490, 17 },
    { 0, 0 },
    { 0x3b730492, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96630496, 90 },
    { 0, 0 },
    { 0x19830498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816304a1, 61 },
    { 0xd31304a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c304ac, 23 },
    { 0x54a304ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6304b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5304b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b304b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c304bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b304c0, 61 },
    { 0x1c3304c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b304c4, 88 },
    { 0x9eb304c4, 95 },
    { 0x677304c6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8304c9, 23 },
    { 0xe55304ca, 46 },
    { 0xfb9304c9, 33 },
    { 0x8a0304cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c304cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3304d2, 90 },
    { 0xedb304d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96304d7, 46 },
    { 0xf23304d8, 61 },
    { 0, 0 },
    { 0xcd1304da, 33 },
    { 0x37c304db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab304e3, 75 },
    { 0, 0 },
    { 0x4ec304e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228304ea, 95 },
    { 0, 0 },
    { 0x1b1304ec, 93 },
    { 0x349304ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3304f2, 90 },
    { 0x6f8304f3, 55 },
    { 0xe0f304f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd304f8, 61 },
    { 0x25c304f9, 61 },
    { 0x120304fa, 95 },
    { 0, 0 },
    { 0x650304fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e30501, 40 },
    { 0x24730502, 61 },
    { 0x81e30503, 76 },
    { 0xb4030504, 76 },
    { 0xb0530505, 61 },
    { 0xac730506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2130509, 90 },
    { 0x9313050a, 55 },
    { 0x6e630509, 55 },
    { 0x32a3050c, 33 },
    { 0x6de3050d, 90 },
    { 0, 0 },
    { 0x5893050f, 98 },
    { 0xe1230510, 100 },
    { 0xe730511, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d33051d, 76 },
    { 0xf63051e, 76 },
    { 0xa4b3051f, 93 },
    { 0x7043051f, 93 },
    { 0xeef30521, 52 },
    { 0x74d30521, 98 },
    { 0x9c73051f, 61 },
    { 0x8cc30524, 69 },
    { 0x5f730525, 23 },
    { 0x89430526, 34 },
    { 0xe6830525, 85 },
    { 0x59130526, 98 },
    { 0xf9030522, 23 },
    { 0xb5c3051e, 95 },
    { 0, 0 },
    { 0x9243052c, 34 },
    { 0x2823052d, 61 },
    { 0xfc3052e, 23 },
    { 0x3e83052d, 85 },
    { 0xcc03052d, 69 },
    { 0xcc930531, 95 },
    { 0, 0 },
    { 0x8130533, 61 },
    { 0, 0 },
    { 0xf0130535, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65830539, 61 },
    { 0xbd130539, 33 },
    { 0x5b3053b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34130541, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2130544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa3054d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3530552, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90630556, 33 },
    { 0x45b30557, 55 },
    { 0, 0 },
    { 0xfda30559, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a30560, 93 },
    { 0x48d30560, 33 },
    { 0xae030561, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc30569, 23 },
    { 0x1a30569, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82830573, 91 },
    { 0xb9830574, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce30579, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2443057c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b3057f, 95 },
    { 0x5593057f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80630584, 95 },
    { 0x13630585, 95 },
    { 0x15e30586, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9730589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf593058d, 76 },
    { 0x6c73058e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2630592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f3059c, 43 },
    { 0xbd43059c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22305a0, 68 },
    { 0xb94305a0, 61 },
    { 0xa20305a1, 98 },
    { 0, 0 },
    { 0x63e305a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c305a7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d305aa, 23 },
    { 0x541305ab, 56 },
    { 0x3f1305aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5305af, 98 },
    { 0, 0 },
    { 0xd64305b1, 90 },
    { 0, 0 },
    { 0xce7305b3, 87 },
    { 0x7d0305b4, 99 },
    { 0x3ca305b4, 43 },
    { 0, 0 },
    { 0x357305b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28305bb, 61 },
    { 0, 0 },
    { 0x4fe305bd, 55 },
    { 0x3ac305be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511305c6, 98 },
    { 0, 0 },
    { 0x6f3305c8, 75 },
    { 0x64a305c9, 43 },
    { 0x38d305ca, 61 },
    { 0x4ff305c8, 61 },
    { 0x575305c9, 33 },
    { 0xbc6305c8, 55 },
    { 0x44d305ce, 61 },
    { 0x335305cf, 85 },
    { 0, 0 },
    { 0x565305d1, 76 },
    { 0x51f305d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98305da, 23 },
    { 0x5ee305da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b305e2, 98 },
    { 0, 0 },
    { 0xf40305e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x175305e7, 90 },
    { 0, 0 },
    { 0x378305e9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e305ec, 90 },
    { 0x93b305ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1305f2, 81 },
    { 0x26d305f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f305f7, 98 },
    { 0x53a305f7, 23 },
    { 0, 0 },
    { 0x940305fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7530600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc3060b, 95 },
    { 0xb933060b, 61 },
    { 0, 0 },
    { 0x3643060e, 61 },
    { 0, 0 },
    { 0xa6630610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e30616, 88 },
    { 0xab730617, 95 },
    { 0x2ae30616, 87 },
    { 0x99130616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a3061f, 98 },
    { 0xad93061f, 61 },
    { 0xb1230621, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b30624, 33 },
    { 0xdcb30624, 34 },
    { 0x29330625, 76 },
    { 0x38730627, 85 },
    { 0xcba30625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8973062b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db3062e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30930632, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb130635, 55 },
    { 0x5c830635, 90 },
    { 0, 0 },
    { 0x5d330638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4983063c, 95 },
    { 0x24e3063c, 69 },
    { 0x7dd3063c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f130641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd430644, 61 },
    { 0, 0 },
    { 0x94030646, 80 },
    { 0, 0 },
    { 0xf5e30648, 99 },
    { 0x9b630649, 90 },
    { 0x7bb3064a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda63064d, 90 },
    { 0xf0a3064e, 90 },
    { 0xd113064d, 87 },
    { 0xc0d30650, 55 },
    { 0x42f30651, 55 },
    { 0xc1430652, 55 },
    { 0xbce30653, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c30656, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad230659, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d3065d, 95 },
    { 0x34f3065e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15130665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3066a, 61 },
    { 0xf83066a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3066f, 90 },
    { 0xee63066f, 90 },
    { 0x1b930671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa930675, 34 },
    { 0x49230675, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb83067e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9830681, 33 },
    { 0x9f630682, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c930687, 46 },
    { 0x4330688, 33 },
    { 0x61e30689, 76 },
    { 0x61530689, 91 },
    { 0x1fe30687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e130693, 4 },
    { 0xef630694, 90 },
    { 0x1f730695, 90 },
    { 0x2d930696, 46 },
    { 0xa1730696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd33069a, 68 },
    { 0, 0 },
    { 0x9993069c, 76 },
    { 0, 0 },
    { 0xfc73069e, 61 },
    { 0xa223069f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9306a2, 90 },
    { 0x5d306a3, 84 },
    { 0xce6306a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771306a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc306af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a306b3, 75 },
    { 0x6d9306b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500306ba, 65 },
    { 0x228306bb, 66 },
    { 0x288306ba, 69 },
    { 0x372306bd, 90 },
    { 0x14d306bc, 52 },
    { 0x1ba306bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2306c2, 91 },
    { 0, 0 },
    { 0x77a306c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439306ca, 84 },
    { 0, 0 },
    { 0xa8a306cc, 33 },
    { 0x3f8306cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5306d4, 91 },
    { 0xf5e306d5, 91 },
    { 0x7b7306d5, 43 },
    { 0xa74306d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6306da, 46 },
    { 0xac6306db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63306de, 33 },
    { 0xe4f306df, 98 },
    { 0x947306de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb306e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8306ef, 61 },
    { 0xe0d306f0, 33 },
    { 0x1d7306f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1306f6, 76 },
    { 0x416306f6, 90 },
    { 0, 0 },
    { 0xad3306f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4306fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab30701, 61 },
    { 0, 0 },
    { 0xf1b30703, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb630706, 99 },
    { 0x29e30707, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd243070c, 46 },
    { 0, 0 },
    { 0x4373070e, 99 },
    { 0x3a73070f, 94 },
    { 0xcfb30710, 99 },
    { 0xbd53070e, 84 },
    { 0x45530712, 40 },
    { 0x8d73070f, 95 },
    { 0x62130713, 95 },
    { 0, 0 },
    { 0x85c30716, 46 },
    { 0x4a930717, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673071e, 52 },
    { 0xea3071e, 90 },
    { 0x3c53071e, 33 },
    { 0xa0430720, 43 },
    { 0x85c30722, 76 },
    { 0xaaf30722, 33 },
    { 0xff130724, 78 },
    { 0, 0 },
    { 0xc5630726, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9383072f, 33 },
    { 0xc0d3072f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28430734, 90 },
    { 0x95230734, 98 },
    { 0xeee30736, 100 },
    { 0xb0c30734, 34 },
    { 0xd9730736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1813073f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b30742, 23 },
    { 0xda30743, 78 },
    { 0x56f30744, 100 },
    { 0x1aa30745, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43074a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633074d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b930750, 61 },
    { 0, 0 },
    { 0xe3c30752, 55 },
    { 0x28330753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee30757, 84 },
    { 0x54330758, 56 },
    { 0x2d930757, 90 },
    { 0, 0 },
    { 0xa5b3075b, 87 },
    { 0xbdd3075c, 98 },
    { 0x5e83075c, 100 },
    { 0, 0 },
    { 0x9393075f, 67 },
    { 0, 0 },
    { 0xed830761, 55 },
    { 0x12530761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3076d, 81 },
    { 0x5e73076e, 67 },
    { 0x8ac3076e, 76 },
    { 0, 0 },
    { 0x88c30771, 93 },
    { 0, 0 },
    { 0x50d30773, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62830776, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf3077e, 69 },
    { 0x2423077f, 100 },
    { 0xc6e30780, 23 },
    { 0xa3b30781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a30784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc3078d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5230791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1307a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x738307a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda307aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0307b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67307ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28307bf, 23 },
    { 0x50f307c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e307c8, 33 },
    { 0x488307c9, 33 },
    { 0, 0 },
    { 0x47c307cb, 61 },
    { 0xb3e307cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145307d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d307d3, 33 },
    { 0x2e1307d4, 65 },
    { 0, 0 },
    { 0x52b307d6, 90 },
    { 0x51f307d7, 90 },
    { 0, 0 },
    { 0x7ff307d9, 76 },
    { 0, 0 },
    { 0xeff307db, 67 },
    { 0x218307dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf307e0, 77 },
    { 0x188307e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86307e4, 33 },
    { 0x5ab307e4, 55 },
    { 0, 0 },
    { 0xfd6307e7, 76 },
    { 0x79b307e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2307eb, 90 },
    { 0xf24307ec, 61 },
    { 0, 0 },
    { 0x9c3307ee, 100 },
    { 0xddc307ef, 33 },
    { 0x580307f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78307f7, 76 },
    { 0x245307f8, 61 },
    { 0, 0 },
    { 0x49e307fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae330800, 91 },
    { 0x84530801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7930807, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833080c, 61 },
    { 0x8493080d, 88 },
    { 0x74b3080d, 76 },
    { 0, 0 },
    { 0x27c30810, 61 },
    { 0x2d730810, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d30815, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673081b, 90 },
    { 0xfff3081c, 98 },
    { 0x5433081d, 93 },
    { 0xe6a3081e, 98 },
    { 0xab13081d, 76 },
    { 0x18730820, 61 },
    { 0xcbe30821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d30829, 55 },
    { 0x6243082a, 95 },
    { 0xe483082b, 84 },
    { 0x9dc3082c, 91 },
    { 0x5b83082c, 98 },
    { 0x8813082b, 55 },
    { 0, 0 },
    { 0x43730830, 69 },
    { 0xb7b30831, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b30838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f30843, 98 },
    { 0x91530844, 93 },
    { 0, 0 },
    { 0xf5130846, 46 },
    { 0xb5a30847, 55 },
    { 0x19530848, 88 },
    { 0, 0 },
    { 0x9243084a, 61 },
    { 0x9683084b, 61 },
    { 0x6df3084b, 98 },
    { 0x2a73084a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0330851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5630856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5073085b, 61 },
    { 0, 0 },
    { 0x7f73085d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6230865, 23 },
    { 0xadb30865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f30872, 61 },
    { 0x24830872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xde630876, 90 },
    { 0x98530877, 61 },
    { 0x8b030877, 23 },
    { 0xbcd30879, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf83087d, 95 },
    { 0xa643087d, 43 },
    { 0, 0 },
    { 0x1c130880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82430883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c730886, 98 },
    { 0x4b230887, 66 },
    { 0, 0 },
    { 0xc2630889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3088f, 76 },
    { 0x7f23088f, 61 },
    { 0, 0 },
    { 0xb8f30892, 33 },
    { 0x10730893, 61 },
    { 0xf2130893, 34 },
    { 0x5be30895, 75 },
    { 0, 0 },
    { 0x7a330897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa783089a, 43 },
    { 0, 0 },
    { 0x1523089c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae308a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437308a7, 88 },
    { 0xa07308a8, 95 },
    { 0x620308a9, 36 },
    { 0x750308a7, 95 },
    { 0x7f2308a9, 100 },
    { 0xd30308ac, 99 },
    { 0xbd308ad, 61 },
    { 0xd70308ae, 68 },
    { 0xcfa308ad, 55 },
    { 0, 0 },
    { 0xe1c308b1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x420308b4, 61 },
    { 0x54c308b5, 90 },
    { 0x82d308b6, 19 },
    { 0x22a308b7, 61 },
    { 0xc05308b5, 99 },
    { 0x658308b9, 90 },
    { 0x8c308ba, 95 },
    { 0x102308bb, 61 },
    { 0x41f308bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0308c0, 66 },
    { 0x900308c1, 95 },
    { 0, 0 },
    { 0x333308c3, 90 },
    { 0x536308c4, 78 },
    { 0xc36308c5, 98 },
    { 0x651308c6, 55 },
    { 0xcdb308c7, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x867308ca, 90 },
    { 0xb46308cb, 46 },
    { 0xbbc308cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f308d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b308d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94308db, 33 },
    { 0xe3a308dc, 90 },
    { 0xaa1308dd, 46 },
    { 0xd3d308de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37308e1, 100 },
    { 0xf19308e2, 67 },
    { 0xbed308e3, 100 },
    { 0x89a308e4, 33 },
    { 0x367308e1, 95 },
    { 0xdd7308e6, 90 },
    { 0xc06308e7, 33 },
    { 0x719308e7, 34 },
    { 0x67a308e7, 43 },
    { 0xa2c308ea, 33 },
    { 0x47d308ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24308f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x897308f3, 33 },
    { 0xf2e308f4, 77 },
    { 0xca6308f4, 46 },
    { 0x65e308f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e308fa, 99 },
    { 0, 0 },
    { 0x96c308fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11308ff, 98 },
    { 0, 0 },
    { 0xe5330901, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7930907, 76 },
    { 0xdc130907, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933090c, 76 },
    { 0xa2c3090d, 40 },
    { 0, 0 },
    { 0x2ba3090f, 66 },
    { 0, 0 },
    { 0xe330911, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba430914, 23 },
    { 0xc8530915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af30919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3091c, 23 },
    { 0x30c3091d, 90 },
    { 0x89f3091e, 43 },
    { 0, 0 },
    { 0x10e30920, 95 },
    { 0x3c330920, 95 },
    { 0xb1c30922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36930927, 23 },
    { 0xb4330928, 95 },
    { 0x93030929, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2413092c, 98 },
    { 0x23c3092d, 90 },
    { 0x28d3092e, 78 },
    { 0xf7b3092f, 95 },
    { 0xe1e3092e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e230934, 34 },
    { 0x96e30935, 33 },
    { 0x7a530934, 90 },
    { 0, 0 },
    { 0xe2d30938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23093b, 95 },
    { 0x6f13093c, 55 },
    { 0x54a3093c, 95 },
    { 0x5d3093e, 98 },
    { 0x90e3093e, 55 },
    { 0xb013093c, 46 },
    { 0x12b30941, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d30944, 23 },
    { 0xd2830945, 99 },
    { 0x61f30946, 100 },
    { 0xd2630946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf43094a, 90 },
    { 0xf173094b, 23 },
    { 0xab13094b, 33 },
    { 0x5b3094d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f530952, 99 },
    { 0x18630953, 33 },
    { 0xd4d30953, 84 },
    { 0xf230954, 23 },
    { 0, 0 },
    { 0x11e30957, 78 },
    { 0xa6130958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a73095b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793095e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb30968, 85 },
    { 0, 0 },
    { 0xa2e3096a, 33 },
    { 0, 0 },
    { 0xdb43096c, 55 },
    { 0xee63096d, 95 },
    { 0x9f13096c, 81 },
    { 0, 0 },
    { 0xcaa30970, 34 },
    { 0, 0 },
    { 0xad930972, 23 },
    { 0, 0 },
    { 0xaea30974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a130979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e30983, 93 },
    { 0x5be30984, 46 },
    { 0x3e30985, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb23098a, 99 },
    { 0x8683098b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c230991, 67 },
    { 0xd6530992, 95 },
    { 0xc0030993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab30996, 98 },
    { 0, 0 },
    { 0x9cd30998, 66 },
    { 0x69030999, 46 },
    { 0x38d30998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab309a9, 90 },
    { 0, 0 },
    { 0x3e309ab, 61 },
    { 0x7c1309ab, 95 },
    { 0xec309ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9309b3, 86 },
    { 0, 0 },
    { 0xc9e309b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be309b8, 98 },
    { 0, 0 },
    { 0xc98309ba, 76 },
    { 0, 0 },
    { 0x714309bc, 95 },
    { 0x83e309bd, 98 },
    { 0, 0 },
    { 0x18a309bf, 43 },
    { 0xee2309c0, 55 },
    { 0, 0 },
    { 0x7f8309c2, 33 },
    { 0xb68309c2, 76 },
    { 0x8f6309c4, 66 },
    { 0x257309c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19309c9, 95 },
    { 0, 0 },
    { 0x18309cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949309d0, 75 },
    { 0x98309d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a309d4, 75 },
    { 0xb2e309d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x325309d8, 52 },
    { 0, 0 },
    { 0xbde309da, 55 },
    { 0x950309db, 65 },
    { 0, 0 },
    { 0xb1a309dd, 61 },
    { 0xf39309dd, 33 },
    { 0, 0 },
    { 0x3ad309e0, 33 },
    { 0x690309e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c309f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c309f5, 43 },
    { 0, 0 },
    { 0xd1a309f7, 55 },
    { 0x4b5309f8, 69 },
    { 0xf3f309f9, 61 },
    { 0x241309f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c309ff, 98 },
    { 0x9a1309ff, 98 },
    { 0, 0 },
    { 0x73730a02, 78 },
    { 0xaf830a02, 95 },
    { 0x46a30a04, 23 },
    { 0, 0 },
    { 0xafd30a06, 67 },
    { 0, 0 },
    { 0xa7930a08, 23 },
    { 0x50730a09, 33 },
    { 0x10030a09, 33 },
    { 0xbf330a0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7830a0f, 95 },
    { 0xaa730a10, 23 },
    { 0x1d430a0f, 98 },
    { 0x70830a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba30a15, 43 },
    { 0, 0 },
    { 0x5d30a17, 85 },
    { 0x66730a17, 96 },
    { 0, 0 },
    { 0x19430a1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df30a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76830a28, 23 },
    { 0, 0 },
    { 0x33930a2a, 61 },
    { 0, 0 },
    { 0x6a130a2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7530a2f, 33 },
    { 0, 0 },
    { 0x84b30a31, 52 },
    { 0x65530a32, 23 },
    { 0xdae30a33, 33 },
    { 0xec130a34, 100 },
    { 0xce130a35, 43 },
    { 0xa4e30a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd30a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd730a3f, 98 },
    { 0, 0 },
    { 0x9b830a41, 88 },
    { 0, 0 },
    { 0xf9430a43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e630a4a, 68 },
    { 0x9fc30a4a, 22 },
    { 0xcf430a4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b30a52, 52 },
    { 0, 0 },
    { 0x77130a54, 33 },
    { 0xa0530a54, 87 },
    { 0xe7f30a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f30a59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c30a5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c130a62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38630a6a, 76 },
    { 0x6dc30a6b, 69 },
    { 0, 0 },
    { 0x4d230a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d30a72, 98 },
    { 0, 0 },
    { 0xe7230a74, 33 },
    { 0x44930a75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed30a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda430a87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86130a8b, 90 },
    { 0x82c30a8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b030a93, 23 },
    { 0xfe430a94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b030a9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be30aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba30aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb30aab, 75 },
    { 0x2ea30aac, 90 },
    { 0x64e30aad, 93 },
    { 0xe6530aae, 33 },
    { 0, 0 },
    { 0x4cf30ab0, 81 },
    { 0, 0 },
    { 0x67e30ab2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf30ab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60630ac0, 95 },
    { 0, 0 },
    { 0xbfe30ac2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94030ac7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e030aca, 46 },
    { 0x58a30aca, 95 },
    { 0, 0 },
    { 0x67730acd, 98 },
    { 0, 0 },
    { 0x37d30acf, 90 },
    { 0x60830ad0, 76 },
    { 0x18530ad1, 90 },
    { 0xf9930ad0, 34 },
    { 0, 0 },
    { 0x25630ad4, 95 },
    { 0x64930ad5, 78 },
    { 0, 0 },
    { 0x9d130ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa30ade, 95 },
    { 0, 0 },
    { 0xb9430ae0, 33 },
    { 0x8d730ae1, 99 },
    { 0, 0 },
    { 0x20630ae3, 23 },
    { 0xa1430ae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc830aea, 93 },
    { 0x5be30aeb, 98 },
    { 0xc8530aec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x96330aef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1030af2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e30af7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a430afc, 33 },
    { 0xc7030afd, 33 },
    { 0, 0 },
    { 0x53130aff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc30b02, 33 },
    { 0, 0 },
    { 0xa4430b04, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c30b08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c930b0e, 23 },
    { 0x11130b0f, 87 },
    { 0x73730b10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83730b14, 98 },
    { 0, 0 },
    { 0x62930b16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32630b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19130b1e, 85 },
    { 0, 0 },
    { 0x4b630b20, 61 },
    { 0xede30b21, 98 },
    { 0xdf30b22, 66 },
    { 0xa6a30b21, 78 },
    { 0x74730b24, 33 },
    { 0x75630b21, 23 },
    { 0xf5630b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc30b2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa30b34, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b30b3f, 84 },
    { 0x74e30b40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b630b46, 46 },
    { 0x9db30b47, 33 },
    { 0xab230b48, 61 },
    { 0x64730b49, 76 },
    { 0, 0 },
    { 0xa1330b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97930b56, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac30b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c30b60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98630b65, 33 },
    { 0xb2030b66, 90 },
    { 0x25b30b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f30b6b, 23 },
    { 0x6c230b6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4630b6f, 88 },
    { 0x65330b6f, 87 },
    { 0x9c430b71, 76 },
    { 0, 0 },
    { 0xd9a30b73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d630b77, 75 },
    { 0x40930b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91730b7f, 90 },
    { 0xe3730b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed30b84, 84 },
    { 0xbea30b84, 95 },
    { 0, 0 },
    { 0x830b87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a830b8d, 52 },
    { 0xd6f30b8e, 77 },
    { 0xbd530b8f, 95 },
    { 0, 0 },
    { 0x5e330b91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75030b94, 61 },
    { 0xc6430b95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42030b9c, 90 },
    { 0x77c30b9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb630ba0, 66 },
    { 0xe9e30ba0, 65 },
    { 0x4b230ba2, 95 },
    { 0x33f30ba2, 34 },
    { 0x75030ba3, 90 },
    { 0, 0 },
    { 0x3ee30ba6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c930baa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2230bae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc30bb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d30bb5, 90 },
    { 0x10a30bb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c30bba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e30bc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a630bc8, 45 },
    { 0xd9830bc8, 43 },
    { 0x6530bca, 100 },
    { 0xe1d30bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da30bd0, 76 },
    { 0, 0 },
    { 0x4b930bd2, 84 },
    { 0x8ff30bd3, 61 },
    { 0x49a30bd3, 33 },
    { 0x94330bd4, 98 },
    { 0x63230bd6, 23 },
    { 0x42b30bd7, 98 },
    { 0, 0 },
    { 0xeac30bd9, 55 },
    { 0, 0 },
    { 0x1130bdb, 90 },
    { 0x90530bdc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xde30bdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9330be3, 36 },
    { 0x5bc30be4, 95 },
    { 0x7b630be4, 95 },
    { 0xb30be3, 98 },
    { 0x61330be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa130beb, 78 },
    { 0x63030bec, 33 },
    { 0x91430bed, 90 },
    { 0xac630bed, 90 },
    { 0x82130bef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68930bf5, 23 },
    { 0, 0 },
    { 0xae430bf7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea930bfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa30c04, 94 },
    { 0xc0f30c04, 76 },
    { 0x33f30c04, 98 },
    { 0xc8a30c05, 93 },
    { 0x13330c08, 81 },
    { 0x17030c09, 66 },
    { 0xca230c0a, 33 },
    { 0x7bb30c0a, 33 },
    { 0, 0 },
    { 0xe9230c0d, 65 },
    { 0xafc30c0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c230c1a, 52 },
    { 0, 0 },
    { 0x4530c1c, 78 },
    { 0x18430c1d, 34 },
    { 0xe9a30c1e, 65 },
    { 0xc2e30c1f, 93 },
    { 0x2b730c1e, 96 },
    { 0xec430c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c230c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8330c2c, 61 },
    { 0x33730c2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f430c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea30c3f, 93 },
    { 0, 0 },
    { 0x17b30c41, 66 },
    { 0, 0 },
    { 0x18830c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4230c4a, 61 },
    { 0, 0 },
    { 0x59b30c4c, 95 },
    { 0xcc630c4d, 100 },
    { 0xef930c4e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x16630c51, 95 },
    { 0, 0 },
    { 0xaee30c53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91630c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee930c59, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56130c5d, 90 },
    { 0, 0 },
    { 0x7c830c5f, 95 },
    { 0xf2830c5f, 75 },
    { 0, 0 },
    { 0xc3530c62, 23 },
    { 0x5a330c62, 78 },
    { 0xbc430c64, 17 },
    { 0x1e830c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16630c69, 33 },
    { 0, 0 },
    { 0xb3130c6b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e30c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c30c75, 33 },
    { 0, 0 },
    { 0xd9e30c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc30c7b, 98 },
    { 0x43630c7c, 77 },
    { 0, 0 },
    { 0xf6830c7e, 23 },
    { 0x95730c7f, 43 },
    { 0, 0 },
    { 0x1df30c81, 33 },
    { 0, 0 },
    { 0x65530c83, 61 },
    { 0x33230c84, 100 },
    { 0x21c30c85, 23 },
    { 0x87a30c83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc930c8a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb30c8d, 100 },
    { 0xa9830c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f030c91, 78 },
    { 0, 0 },
    { 0xc3b30c93, 23 },
    { 0, 0 },
    { 0x17e30c95, 93 },
    { 0xb1d30c96, 95 },
    { 0, 0 },
    { 0xf5030c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e30c9d, 91 },
    { 0xeeb30c9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb30ca1, 100 },
    { 0, 0 },
    { 0x37630ca3, 95 },
    { 0xbe30ca4, 33 },
    { 0x5e130ca5, 9 },
    { 0x80d30ca6, 40 },
    { 0xd3a30ca3, 90 },
    { 0x8e830ca8, 98 },
    { 0xd4030ca4, 61 },
    { 0x4a130ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf630caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87430cba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6930cbd, 33 },
    { 0, 0 },
    { 0x78430cbf, 23 },
    { 0x9f030cc0, 95 },
    { 0xb6c30cc1, 33 },
    { 0x72930cc2, 46 },
    { 0x8ae30cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59030cc9, 23 },
    { 0xb0330cca, 76 },
    { 0x2130cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b30cd0, 85 },
    { 0, 0 },
    { 0x4b730cd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c30cd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10330cdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1930ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64730ce6, 40 },
    { 0, 0 },
    { 0xa8e30ce8, 33 },
    { 0x33430ce8, 46 },
    { 0x85330ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71130cee, 66 },
    { 0, 0 },
    { 0xab730cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26830cf6, 61 },
    { 0x12130cf7, 43 },
    { 0xbf630cf8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f730cfb, 76 },
    { 0, 0 },
    { 0x2a730cfd, 95 },
    { 0, 0 },
    { 0x7dc30cff, 61 },
    { 0x1f330d00, 76 },
    { 0x50d30d00, 95 },
    { 0, 0 },
    { 0xce830d03, 33 },
    { 0x4b30d04, 90 },
    { 0x5b330d04, 33 },
    { 0x3a130d06, 33 },
    { 0xc9030d04, 90 },
    { 0xe9b30d08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa30d0b, 95 },
    { 0xd4b30d0b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc30d12, 23 },
    { 0xc3f30d12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35830d17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d230d21, 87 },
    { 0xdf230d22, 36 },
    { 0x23430d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62530d28, 90 },
    { 0x6030d29, 90 },
    { 0xdb230d2a, 78 },
    { 0xe230d2b, 46 },
    { 0x64a30d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f30d30, 98 },
    { 0, 0 },
    { 0xcc630d32, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11730d36, 23 },
    { 0x35930d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e630d3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39530d46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a30d4a, 75 },
    { 0x3aa30d4a, 33 },
    { 0x66e30d4a, 46 },
    { 0x56330d4d, 88 },
    { 0x8fb30d4d, 90 },
    { 0x35830d4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84130d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf30d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b30d5b, 17 },
    { 0x6c730d5c, 95 },
    { 0xc7a30d5d, 98 },
    { 0x7c130d5e, 69 },
    { 0xc130d5e, 91 },
    { 0, 0 },
    { 0x73e30d61, 91 },
    { 0, 0 },
    { 0xf0830d63, 100 },
    { 0x54530d64, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70530d69, 84 },
    { 0x55430d69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac30d6d, 87 },
    { 0, 0 },
    { 0xe5c30d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59330d73, 99 },
    { 0xd9c30d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff30d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c30d80, 66 },
    { 0xb3c30d81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa30d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3430d8d, 76 },
    { 0x39530d8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8530d93, 84 },
    { 0, 0 },
    { 0x98130d95, 33 },
    { 0xfd30d95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d30d9a, 90 },
    { 0, 0 },
    { 0x38930d9c, 99 },
    { 0, 0 },
    { 0x11230d9e, 84 },
    { 0xbff30d9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e30da2, 34 },
    { 0xcae30da3, 23 },
    { 0, 0 },
    { 0xdb630da5, 77 },
    { 0x44c30da5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e30da9, 40 },
    { 0, 0 },
    { 0x56130dab, 55 },
    { 0x27930dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19530db0, 33 },
    { 0, 0 },
    { 0x29a30db2, 98 },
    { 0x3230db3, 95 },
    { 0x86330db4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3330dba, 91 },
    { 0, 0 },
    { 0x10430dbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30730dbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc630dc7, 33 },
    { 0xbd430dc8, 34 },
    { 0x3230dc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd30dd0, 90 },
    { 0, 0 },
    { 0xd8930dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa30dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f30dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff30de2, 95 },
    { 0x10230de3, 76 },
    { 0x95530de4, 23 },
    { 0xc0f30de3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be30deb, 98 },
    { 0x1f830dec, 90 },
    { 0x6330ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7930df0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9430df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee30dfb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd30dfe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8130e03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e30e07, 90 },
    { 0x4ff30e08, 76 },
    { 0x99330e07, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74630e0d, 61 },
    { 0x12130e0e, 67 },
    { 0x6e530e0f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9830e12, 55 },
    { 0x14530e12, 85 },
    { 0xc3530e14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25330e1c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x28930e1f, 33 },
    { 0x55530e20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75430e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa30e27, 61 },
    { 0xe2830e28, 90 },
    { 0x40a30e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa30e2d, 90 },
    { 0x27030e2d, 33 },
    { 0, 0 },
    { 0x7c530e30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b530e3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8430e42, 98 },
    { 0xbc930e43, 90 },
    { 0x76c30e42, 95 },
    { 0xf8830e43, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d30e4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc30e4f, 76 },
    { 0, 0 },
    { 0xe6e30e51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f430e54, 77 },
    { 0x3cb30e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad630e58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd30e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb30e60, 33 },
    { 0, 0 },
    { 0x9e630e62, 99 },
    { 0, 0 },
    { 0x9d930e64, 76 },
    { 0x12f30e64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5130e6a, 91 },
    { 0, 0 },
    { 0xa0b30e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb30e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2830e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4930e7e, 76 },
    { 0xe4b30e7e, 76 },
    { 0x69530e7e, 93 },
    { 0x12830e81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1230e85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85930e8a, 98 },
    { 0, 0 },
    { 0x17f30e8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e30e93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd30e96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x23830e99, 95 },
    { 0x7d130e99, 90 },
    { 0, 0 },
    { 0xe0430e9c, 77 },
    { 0, 0 },
    { 0x3330e9e, 23 },
    { 0xcea30e9f, 76 },
    { 0xc7d30ea0, 100 },
    { 0xea230ea0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c30ea4, 23 },
    { 0x45d30ea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2430ea8, 90 },
    { 0x4f30ea9, 46 },
    { 0, 0 },
    { 0xcb030eab, 100 },
    { 0x1a30eab, 95 },
    { 0x5ab30ead, 90 },
    { 0xade30ead, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d30eb6, 33 },
    { 0, 0 },
    { 0xba130eb8, 78 },
    { 0xdd330eb9, 93 },
    { 0x3e630eb9, 98 },
    { 0x2eb30ebb, 55 },
    { 0xe1830ebc, 23 },
    { 0x2430ebd, 98 },
    { 0x4e330ebe, 34 },
    { 0, 0 },
    { 0x5d730ec0, 61 },
    { 0x5230ec0, 67 },
    { 0, 0 },
    { 0x6ec30ec3, 90 },
    { 0x81130ec4, 90 },
    { 0x4cc30ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6630ec9, 95 },
    { 0xfb30ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd30ece, 76 },
    { 0x12930ece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d30ed4, 98 },
    { 0, 0 },
    { 0x32c30ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc30edb, 33 },
    { 0, 0 },
    { 0xe7230edd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2930ee5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1230ee8, 66 },
    { 0, 0 },
    { 0xdda30eea, 43 },
    { 0xf5030eea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c30ef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31430ef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf30efb, 98 },
    { 0xaab30efb, 76 },
    { 0, 0 },
    { 0x3c230efe, 56 },
    { 0x97030eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f30f04, 97 },
    { 0xee130f05, 94 },
    { 0, 0 },
    { 0x49b30f07, 91 },
    { 0xadc30f07, 90 },
    { 0, 0 },
    { 0xa4830f0a, 55 },
    { 0xec630f0a, 66 },
    { 0, 0 },
    { 0xc7730f0d, 78 },
    { 0x71030f0e, 61 },
    { 0x19630f0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae630f15, 95 },
    { 0, 0 },
    { 0x2b930f17, 46 },
    { 0x54130f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb130f1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90330f20, 23 },
    { 0, 0 },
    { 0x89930f22, 55 },
    { 0x73430f23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c330f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2930f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c30f2e, 33 },
    { 0x6f30f2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x15730f32, 43 },
    { 0x14b30f33, 95 },
    { 0, 0 },
    { 0x59430f35, 95 },
    { 0xde30f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16230f40, 95 },
    { 0x49830f41, 78 },
    { 0, 0 },
    { 0x37830f43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e30f46, 98 },
    { 0x49d30f46, 91 },
    { 0xb1330f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2930f4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5930f4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c30f51, 90 },
    { 0x23630f51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba930f59, 81 },
    { 0, 0 },
    { 0x12330f5b, 95 },
    { 0, 0 },
    { 0xd3130f5d, 23 },
    { 0, 0 },
    { 0xd0330f5f, 40 },
    { 0x9c930f60, 98 },
    { 0xfc430f61, 98 },
    { 0xf1f30f60, 76 },
    { 0x60930f63, 90 },
    { 0x4d30f63, 46 },
    { 0xa5230f65, 61 },
    { 0x1030f60, 60 },
    { 0x8230f67, 95 },
    { 0x79630f62, 23 },
    { 0, 0 },
    { 0xf9930f6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1530f70, 98 },
    { 0xe2430f70, 61 },
    { 0x7ce30f70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46430f7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b30f7d, 98 },
    { 0x3ee30f7e, 95 },
    { 0xf4e30f7f, 33 },
    { 0x4a30f80, 81 },
    { 0xf1930f7d, 90 },
    { 0x58d30f82, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630f88, 83 },
    { 0x4630f89, 98 },
    { 0x5d230f89, 52 },
    { 0x40930f8b, 90 },
    { 0, 0 },
    { 0xb0330f8d, 90 },
    { 0xde230f8e, 93 },
    { 0xa0930f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a430f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7430f95, 52 },
    { 0x3c330f95, 43 },
    { 0x4b30f97, 95 },
    { 0xd8b30f97, 78 },
    { 0xebe30f98, 50 },
    { 0xc2630f9a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9530fa0, 43 },
    { 0xd5e30fa0, 76 },
    { 0, 0 },
    { 0x73130fa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6530fa6, 98 },
    { 0xe8d30fa7, 76 },
    { 0x4530fa6, 73 },
    { 0x74c30fa7, 65 },
    { 0x26630faa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a30fb2, 23 },
    { 0, 0 },
    { 0x8c730fb4, 95 },
    { 0, 0 },
    { 0xd1930fb6, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b30fbb, 90 },
    { 0x31630fbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b30fbf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71430fc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d30fcc, 61 },
    { 0, 0 },
    { 0xb8e30fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5230fd2, 34 },
    { 0x95230fd3, 76 },
    { 0x2f730fd4, 95 },
    { 0xe7330fd5, 23 },
    { 0xca330fd5, 66 },
    { 0x3530fd7, 61 },
    { 0, 0 },
    { 0x19130fd9, 34 },
    { 0x43630fda, 90 },
    { 0xa5430fda, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3630fe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a30fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef30fea, 23 },
    { 0x14b30fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad30ff1, 76 },
    { 0x85130ff1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc30ff6, 55 },
    { 0, 0 },
    { 0x7a030ff8, 95 },
    { 0, 0 },
    { 0x2f730ffa, 81 },
    { 0x17430ffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c031001, 61 },
    { 0x28531001, 48 },
    { 0x9ea31003, 90 },
    { 0, 0 },
    { 0x78531005, 33 },
    { 0xe331006, 23 },
    { 0, 0 },
    { 0x6ed31008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a3100b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5353100f, 64 },
    { 0xf0031010, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc31015, 90 },
    { 0xa7931016, 77 },
    { 0x25831017, 61 },
    { 0, 0 },
    { 0x91031019, 98 },
    { 0xa7431019, 98 },
    { 0xefe31019, 90 },
    { 0x3033101c, 34 },
    { 0x1473101a, 61 },
    { 0xc0a3101d, 90 },
    { 0x21c3101f, 33 },
    { 0xc7a31020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a31025, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea31028, 99 },
    { 0x2ae31029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3102c, 90 },
    { 0, 0 },
    { 0x183102e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9031031, 76 },
    { 0xc5631032, 90 },
    { 0xd3831031, 76 },
    { 0x4a231033, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13231038, 100 },
    { 0, 0 },
    { 0x1213103a, 34 },
    { 0x75f3103b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39531041, 46 },
    { 0x1831041, 33 },
    { 0, 0 },
    { 0xba931044, 65 },
    { 0xf7731045, 90 },
    { 0x57931045, 61 },
    { 0xed531047, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f3104a, 98 },
    { 0x4693104a, 83 },
    { 0x92d3104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d31051, 95 },
    { 0, 0 },
    { 0xcef31053, 75 },
    { 0, 0 },
    { 0xadf31055, 84 },
    { 0x23831056, 90 },
    { 0x2131057, 90 },
    { 0, 0 },
    { 0xd8331059, 76 },
    { 0xf5b3105a, 88 },
    { 0x99f3105b, 99 },
    { 0x76b3105a, 95 },
    { 0x9383105b, 33 },
    { 0, 0 },
    { 0x2db3105f, 17 },
    { 0xb1831060, 95 },
    { 0xb583105f, 76 },
    { 0x73931062, 23 },
    { 0x76a31061, 90 },
    { 0x5e531060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3106a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3583106d, 84 },
    { 0x6af3106e, 90 },
    { 0, 0 },
    { 0x7f931070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b31074, 87 },
    { 0x95531074, 45 },
    { 0, 0 },
    { 0x4a31077, 40 },
    { 0xbc31078, 90 },
    { 0x51b31079, 95 },
    { 0xadd3107a, 98 },
    { 0x4ee31079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5031080, 23 },
    { 0, 0 },
    { 0xe1b31082, 98 },
    { 0x4e531083, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c31086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4153108c, 77 },
    { 0x5f93108d, 91 },
    { 0x7813108e, 91 },
    { 0x9663108f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff31095, 95 },
    { 0xb1a31096, 64 },
    { 0x931097, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3109c, 98 },
    { 0x9443109d, 100 },
    { 0, 0 },
    { 0x623109f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55310a7, 43 },
    { 0xa5b310a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea310ad, 78 },
    { 0x5a0310ad, 33 },
    { 0, 0 },
    { 0x8ba310b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb310b4, 90 },
    { 0x72e310b5, 61 },
    { 0, 0 },
    { 0xa41310b7, 100 },
    { 0x270310b8, 52 },
    { 0x591310b9, 90 },
    { 0, 0 },
    { 0x7ce310bb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60310be, 55 },
    { 0x812310be, 61 },
    { 0x126310c0, 67 },
    { 0xb310c0, 90 },
    { 0, 0 },
    { 0x967310c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c310c6, 61 },
    { 0x48f310c7, 33 },
    { 0x206310c8, 90 },
    { 0x599310c8, 68 },
    { 0xc97310ca, 46 },
    { 0x754310cb, 52 },
    { 0, 0 },
    { 0x3e2310cd, 78 },
    { 0x410310ce, 64 },
    { 0x649310cf, 55 },
    { 0xd55310d0, 56 },
    { 0xb47310cd, 98 },
    { 0, 0 },
    { 0x6f0310d3, 93 },
    { 0, 0 },
    { 0xddc310d5, 69 },
    { 0, 0 },
    { 0xef1310d7, 55 },
    { 0x52310d8, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e310db, 97 },
    { 0x562310dc, 98 },
    { 0x521310dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6310e1, 17 },
    { 0x57c310e2, 33 },
    { 0x6e0310e2, 78 },
    { 0, 0 },
    { 0xfd6310e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e310e9, 100 },
    { 0, 0 },
    { 0x417310eb, 61 },
    { 0x2c4310ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419310f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc310fa, 34 },
    { 0x76d310fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f310fe, 46 },
    { 0x756310ff, 61 },
    { 0, 0 },
    { 0xe331101, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e431105, 46 },
    { 0x7d131106, 98 },
    { 0, 0 },
    { 0x2a531108, 78 },
    { 0x1e231109, 95 },
    { 0xa323110a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853110d, 23 },
    { 0x19d3110d, 97 },
    { 0xf4a3110d, 47 },
    { 0xbb531110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf31115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11431118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f3111b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f31125, 90 },
    { 0, 0 },
    { 0x1a631127, 90 },
    { 0xa8831128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a3112d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0031132, 95 },
    { 0x33431133, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad631136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e31139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde73113d, 43 },
    { 0, 0 },
    { 0x2603113f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb31143, 90 },
    { 0x6ab31143, 95 },
    { 0x9f931144, 99 },
    { 0xcf631146, 76 },
    { 0xcee31147, 90 },
    { 0, 0 },
    { 0x6ba31149, 34 },
    { 0x6b83114a, 76 },
    { 0, 0 },
    { 0x67d3114c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55331155, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff731159, 95 },
    { 0x8ee31159, 90 },
    { 0, 0 },
    { 0xc883115c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023115f, 61 },
    { 0, 0 },
    { 0xdc831161, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33431165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83116d, 67 },
    { 0x5963116e, 61 },
    { 0x9b03116e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f731174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5031178, 98 },
    { 0x6ba31179, 17 },
    { 0x50931179, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc663117d, 23 },
    { 0xcad3117e, 80 },
    { 0xd8e3117f, 61 },
    { 0xa6e3117f, 61 },
    { 0xf663117f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b31184, 98 },
    { 0x3f931184, 95 },
    { 0xef531185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca31189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f3118d, 34 },
    { 0x9513118e, 55 },
    { 0, 0 },
    { 0xed631190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb31193, 33 },
    { 0x73531194, 23 },
    { 0x51931195, 99 },
    { 0x51631193, 52 },
    { 0x5bc31197, 98 },
    { 0x3c731198, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3119f, 90 },
    { 0x3b5311a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c311a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d311ac, 76 },
    { 0xb66311ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d311b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a311b6, 55 },
    { 0, 0 },
    { 0x19e311b8, 90 },
    { 0xeff311b9, 33 },
    { 0, 0 },
    { 0x49a311bb, 55 },
    { 0, 0 },
    { 0x1b2311bd, 87 },
    { 0x26d311be, 76 },
    { 0x5f0311bf, 33 },
    { 0x58c311be, 61 },
    { 0x35c311c0, 23 },
    { 0xa16311c0, 55 },
    { 0xde8311bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53311c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2311cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c311d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d311d4, 76 },
    { 0x19f311d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551311dc, 33 },
    { 0, 0 },
    { 0x503311de, 33 },
    { 0x17f311df, 33 },
    { 0xf1f311e0, 90 },
    { 0xaa1311e1, 85 },
    { 0x1e6311e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5311e5, 95 },
    { 0, 0 },
    { 0x254311e7, 33 },
    { 0xca6311e7, 33 },
    { 0, 0 },
    { 0x2ca311ea, 34 },
    { 0, 0 },
    { 0x973311ec, 43 },
    { 0xdc7311ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc311f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39311f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df311fd, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0331201, 55 },
    { 0x631202, 90 },
    { 0, 0 },
    { 0x3b631204, 81 },
    { 0, 0 },
    { 0xc8531206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a31209, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d431210, 90 },
    { 0x1ac31211, 34 },
    { 0x4f331211, 93 },
    { 0, 0 },
    { 0xa0231214, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a31217, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf3121b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123121f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x53531222, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63831225, 34 },
    { 0x9a231225, 34 },
    { 0x96231225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8953122a, 34 },
    { 0xea63122b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3122e, 33 },
    { 0x8853122f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f31235, 95 },
    { 0, 0 },
    { 0x59331237, 90 },
    { 0x2131238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3123d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f931247, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3124d, 23 },
    { 0x2543124e, 61 },
    { 0xb093124e, 90 },
    { 0xdcf3124f, 95 },
    { 0, 0 },
    { 0x6e531252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3923125c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f3125f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23331265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6531269, 23 },
    { 0x42c3126a, 60 },
    { 0, 0 },
    { 0x2e43126c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa31270, 33 },
    { 0x21231271, 46 },
    { 0x44931271, 90 },
    { 0, 0 },
    { 0xfcf31274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e3127f, 65 },
    { 0xa0631280, 98 },
    { 0, 0 },
    { 0x81131282, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d31285, 77 },
    { 0, 0 },
    { 0xa1a31287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93128b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30231292, 33 },
    { 0xebc31293, 52 },
    { 0, 0 },
    { 0x9b231295, 90 },
    { 0xce531295, 66 },
    { 0xc2d31295, 90 },
    { 0xbee31298, 75 },
    { 0x1e631298, 81 },
    { 0, 0 },
    { 0x9c33129b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3129e, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef312a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810312a6, 85 },
    { 0, 0 },
    { 0xccd312a8, 90 },
    { 0, 0 },
    { 0x777312aa, 90 },
    { 0, 0 },
    { 0x92b312ac, 61 },
    { 0xf9b312ad, 61 },
    { 0xc5d312ae, 61 },
    { 0, 0 },
    { 0x2c4312b0, 33 },
    { 0x821312b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b312b6, 87 },
    { 0x4d5312b7, 73 },
    { 0x55312b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0312bf, 55 },
    { 0xa84312bf, 100 },
    { 0x4ba312c1, 61 },
    { 0xbb9312c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6312c6, 90 },
    { 0, 0 },
    { 0x925312c8, 95 },
    { 0xb25312c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86312d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849312da, 43 },
    { 0x930312da, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd312de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d312e1, 93 },
    { 0, 0 },
    { 0x38a312e3, 23 },
    { 0xfaf312e3, 43 },
    { 0x341312e5, 87 },
    { 0xe63312e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912312eb, 80 },
    { 0x902312ec, 53 },
    { 0xfc3312eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234312f1, 61 },
    { 0x6e2312f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2312f6, 23 },
    { 0, 0 },
    { 0x252312f8, 95 },
    { 0, 0 },
    { 0x980312fa, 61 },
    { 0xf36312fb, 61 },
    { 0x63f312fb, 95 },
    { 0xa16312fd, 95 },
    { 0x3f8312fe, 84 },
    { 0x512312ff, 61 },
    { 0x70331300, 84 },
    { 0x4f131300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a31304, 90 },
    { 0x1ae31305, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53031310, 33 },
    { 0xd7431311, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91931315, 95 },
    { 0x6f031315, 90 },
    { 0, 0 },
    { 0x89e31318, 55 },
    { 0x45f31319, 77 },
    { 0, 0 },
    { 0xd203131b, 61 },
    { 0xe343131c, 93 },
    { 0x6303131d, 69 },
    { 0x8673131c, 61 },
    { 0x3483131f, 95 },
    { 0xdff3131f, 23 },
    { 0x8b931320, 90 },
    { 0x4513131f, 64 },
    { 0x8c13131e, 95 },
    { 0, 0 },
    { 0x6a031325, 78 },
    { 0, 0 },
    { 0xd8731327, 90 },
    { 0, 0 },
    { 0x32731329, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee3132e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56131331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd931336, 88 },
    { 0x83831336, 85 },
    { 0xdcd31338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53133b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d31340, 33 },
    { 0x48031340, 55 },
    { 0xd0331342, 23 },
    { 0x89931340, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e31348, 33 },
    { 0, 0 },
    { 0x62c3134a, 92 },
    { 0x1c93134a, 23 },
    { 0x6bd3134c, 52 },
    { 0xab23134d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b31354, 42 },
    { 0xe5c31355, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423135a, 34 },
    { 0x1193135b, 33 },
    { 0, 0 },
    { 0xd4a3135d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98531361, 80 },
    { 0x64131361, 100 },
    { 0xcb031363, 85 },
    { 0, 0 },
    { 0x7d31365, 93 },
    { 0, 0 },
    { 0x7e131367, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33136a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e31370, 61 },
    { 0x6e131370, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f931374, 66 },
    { 0x74731375, 95 },
    { 0x93d31375, 55 },
    { 0x1b231377, 34 },
    { 0xeaf31377, 97 },
    { 0, 0 },
    { 0x1be3137a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b53137f, 66 },
    { 0, 0 },
    { 0x11431381, 98 },
    { 0x51931381, 33 },
    { 0, 0 },
    { 0x93d31384, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463138d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22531396, 95 },
    { 0x89831397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413139a, 34 },
    { 0, 0 },
    { 0x3c93139c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573139f, 60 },
    { 0, 0 },
    { 0xbd9313a1, 67 },
    { 0x8f4313a2, 65 },
    { 0x9b3313a3, 55 },
    { 0xdfb313a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2313af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28313b3, 90 },
    { 0x313313b3, 66 },
    { 0x415313b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1313bd, 55 },
    { 0xc4b313be, 33 },
    { 0, 0 },
    { 0x944313c0, 95 },
    { 0, 0 },
    { 0xeaa313c2, 90 },
    { 0, 0 },
    { 0xe80313c4, 90 },
    { 0xd67313c5, 95 },
    { 0x6fb313c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x557313c9, 76 },
    { 0x74e313ca, 55 },
    { 0xc60313ca, 76 },
    { 0x2c6313ca, 95 },
    { 0xa22313cd, 61 },
    { 0xd07313cb, 75 },
    { 0xdca313cc, 33 },
    { 0xf63313ca, 24 },
    { 0x7bc313cb, 61 },
    { 0x240313cb, 17 },
    { 0, 0 },
    { 0x2af313d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de313d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88313db, 99 },
    { 0x78f313db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a313e3, 95 },
    { 0x9c8313e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48313eb, 76 },
    { 0xdc313eb, 90 },
    { 0x8b0313ed, 23 },
    { 0xe73313ee, 92 },
    { 0xdb9313eb, 84 },
    { 0x404313f0, 43 },
    { 0x67c313f1, 23 },
    { 0, 0 },
    { 0x88f313f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d313fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x906313fe, 95 },
    { 0x81f313ff, 98 },
    { 0xb48313fe, 34 },
    { 0x4b9313fe, 47 },
    { 0xb331402, 23 },
    { 0x1a031403, 98 },
    { 0x79431402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96731410, 23 },
    { 0, 0 },
    { 0x1c831412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47231416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe423141e, 67 },
    { 0x4693141f, 46 },
    { 0x87f3141f, 61 },
    { 0, 0 },
    { 0xa3031422, 91 },
    { 0xd5331422, 90 },
    { 0x87731424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc831428, 61 },
    { 0x63e31429, 43 },
    { 0xa0631429, 90 },
    { 0x36431428, 36 },
    { 0x21f31428, 87 },
    { 0x9803142d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e31432, 61 },
    { 0x7b131433, 76 },
    { 0x25531433, 23 },
    { 0xaff31435, 33 },
    { 0x75231436, 34 },
    { 0xb7331437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c31440, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43031445, 65 },
    { 0x49831445, 90 },
    { 0, 0 },
    { 0xbd431448, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3144b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5593144e, 95 },
    { 0x1373144f, 91 },
    { 0xba231450, 61 },
    { 0x5431451, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9473145e, 33 },
    { 0x1963145f, 88 },
    { 0x5413145f, 44 },
    { 0xaea31460, 23 },
    { 0xf3d3145f, 23 },
    { 0x72d31462, 33 },
    { 0, 0 },
    { 0x4dd31465, 99 },
    { 0, 0 },
    { 0xa1c31467, 90 },
    { 0, 0 },
    { 0x6f231469, 61 },
    { 0, 0 },
    { 0x43e3146b, 43 },
    { 0x9453146b, 43 },
    { 0, 0 },
    { 0x223146e, 61 },
    { 0x7a13146f, 53 },
    { 0xd103146f, 78 },
    { 0, 0 },
    { 0x331472, 100 },
    { 0x39a31473, 33 },
    { 0xde931473, 34 },
    { 0, 0 },
    { 0xead31476, 84 },
    { 0xcb731477, 67 },
    { 0xf1e31478, 43 },
    { 0x68f31479, 75 },
    { 0xa7d3147a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a43147e, 76 },
    { 0xb533147f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f31482, 95 },
    { 0x51831482, 66 },
    { 0x48431484, 34 },
    { 0x4bd31485, 91 },
    { 0x9da31486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc73148b, 61 },
    { 0x12f3148c, 58 },
    { 0xf843148d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a31490, 33 },
    { 0, 0 },
    { 0x39731492, 66 },
    { 0, 0 },
    { 0x15831494, 61 },
    { 0, 0 },
    { 0xce931496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd603149a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc3149f, 61 },
    { 0xc4314a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d314a6, 61 },
    { 0x884314a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac314ad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde314b1, 43 },
    { 0x6fb314b1, 34 },
    { 0xb27314b1, 90 },
    { 0xc3d314b4, 76 },
    { 0xf0e314b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7314bd, 46 },
    { 0x4ad314be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5314c3, 65 },
    { 0xc14314c4, 23 },
    { 0x98314c5, 55 },
    { 0x967314c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e314cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x857314ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1314d1, 61 },
    { 0x92b314d1, 23 },
    { 0, 0 },
    { 0x881314d4, 46 },
    { 0xe17314d5, 90 },
    { 0xa12314d6, 40 },
    { 0x340314d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e314dd, 23 },
    { 0xac3314dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a314e3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3314e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788314eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4314ef, 33 },
    { 0x78f314ef, 34 },
    { 0x993314f1, 55 },
    { 0xe65314f2, 68 },
    { 0x633314f2, 91 },
    { 0x5cb314f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca314f8, 98 },
    { 0, 0 },
    { 0x542314fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb314fd, 61 },
    { 0x647314fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd731503, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a31507, 99 },
    { 0x4d331507, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c63150b, 98 },
    { 0xab53150c, 90 },
    { 0x4e83150b, 66 },
    { 0xff53150b, 33 },
    { 0x8e93150c, 36 },
    { 0xa3531510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e531513, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103151e, 84 },
    { 0x1483151e, 100 },
    { 0, 0 },
    { 0xec631521, 76 },
    { 0xb2731522, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8231525, 23 },
    { 0, 0 },
    { 0xa2731527, 76 },
    { 0x87731528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb13152d, 95 },
    { 0, 0 },
    { 0xc4a3152f, 61 },
    { 0, 0 },
    { 0x14e31531, 75 },
    { 0xb1b31532, 84 },
    { 0x8f531533, 90 },
    { 0x7d531531, 69 },
    { 0xc8f31532, 76 },
    { 0, 0 },
    { 0x91031537, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3153a, 84 },
    { 0x8623153b, 92 },
    { 0xdaa3153b, 61 },
    { 0xcec3153c, 93 },
    { 0, 0 },
    { 0xe043153f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19131548, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x303154b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9253154e, 90 },
    { 0x2b83154e, 67 },
    { 0x80431550, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10131557, 90 },
    { 0xee231557, 65 },
    { 0xc0031557, 64 },
    { 0x68d3155a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf3155d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e31561, 23 },
    { 0x31c31561, 92 },
    { 0x4aa31562, 90 },
    { 0xa31561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f31568, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d3156c, 34 },
    { 0xf893156c, 84 },
    { 0x6443156d, 78 },
    { 0x32e3156f, 95 },
    { 0x2913156c, 90 },
    { 0xee13156c, 90 },
    { 0x2a531571, 90 },
    { 0x91931573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb31577, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3157a, 75 },
    { 0x73e3157b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3157e, 100 },
    { 0xced3157e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e31585, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de31588, 33 },
    { 0x90f31589, 24 },
    { 0xee731589, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03158d, 33 },
    { 0, 0 },
    { 0xeb3158f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1631593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b31599, 76 },
    { 0, 0 },
    { 0xeb93159b, 90 },
    { 0, 0 },
    { 0x1b73159d, 36 },
    { 0, 0 },
    { 0xe923159f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d315a3, 95 },
    { 0, 0 },
    { 0x4ef315a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d315a9, 61 },
    { 0x6f3315aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa315af, 84 },
    { 0, 0 },
    { 0x8a5315b1, 98 },
    { 0xb50315b2, 90 },
    { 0x854315b1, 55 },
    { 0x99d315b4, 84 },
    { 0xaf2315b5, 61 },
    { 0x935315b1, 33 },
    { 0x13d315b7, 81 },
    { 0xba7315b8, 98 },
    { 0xce0315b9, 75 },
    { 0xd38315b9, 43 },
    { 0x4a1315b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5315c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e315c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a315c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3315ca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef315d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c315d9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7315df, 68 },
    { 0x429315e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7315e3, 76 },
    { 0x887315e4, 43 },
    { 0xd37315e5, 33 },
    { 0xc08315e6, 75 },
    { 0x338315e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d315ea, 91 },
    { 0, 0 },
    { 0x5d8315ec, 90 },
    { 0x889315ec, 95 },
    { 0xa6b315ec, 95 },
    { 0xe3315ef, 77 },
    { 0x8ce315f0, 98 },
    { 0x374315f1, 52 },
    { 0x173315f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7315fa, 23 },
    { 0xc21315fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd931600, 61 },
    { 0x5e731601, 95 },
    { 0x5ad31602, 34 },
    { 0, 0 },
    { 0xf2e31604, 83 },
    { 0, 0 },
    { 0xec831606, 76 },
    { 0xe5b31607, 93 },
    { 0xd4d31608, 23 },
    { 0xedc31609, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853160f, 91 },
    { 0xf973160f, 98 },
    { 0, 0 },
    { 0x74c31612, 77 },
    { 0xd0531612, 98 },
    { 0xace31612, 33 },
    { 0, 0 },
    { 0x1f331616, 75 },
    { 0, 0 },
    { 0xe9131618, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8753161c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93731620, 23 },
    { 0, 0 },
    { 0x68231622, 33 },
    { 0, 0 },
    { 0x3de31624, 55 },
    { 0x3e131625, 33 },
    { 0x18131626, 95 },
    { 0x831627, 61 },
    { 0, 0 },
    { 0xb0931629, 96 },
    { 0, 0 },
    { 0x2783162b, 90 },
    { 0x4ad3162c, 61 },
    { 0xa783162d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb31630, 33 },
    { 0xbbd31631, 33 },
    { 0x96b31632, 85 },
    { 0xddb31633, 95 },
    { 0x67d31633, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4231637, 55 },
    { 0, 0 },
    { 0xe1c31639, 88 },
    { 0x6a93163a, 90 },
    { 0xb7d3163b, 33 },
    { 0xe8d3163c, 90 },
    { 0x7cd3163c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc731640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e031643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d31646, 23 },
    { 0xff731646, 95 },
    { 0x3bb31646, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343164c, 76 },
    { 0x5763164d, 86 },
    { 0xd33164c, 98 },
    { 0xbfe3164f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0831652, 76 },
    { 0x9831653, 33 },
    { 0x1c231654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9231658, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c431660, 98 },
    { 0, 0 },
    { 0x98c31662, 61 },
    { 0x71731663, 23 },
    { 0xa6831663, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2293166a, 95 },
    { 0x16c3166b, 33 },
    { 0x75e3166c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7383166f, 61 },
    { 0xced3166f, 33 },
    { 0, 0 },
    { 0xa7b31672, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c031678, 61 },
    { 0, 0 },
    { 0xf663167a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe513167d, 98 },
    { 0xdd23167e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de31687, 90 },
    { 0xf0b31687, 98 },
    { 0x19131687, 75 },
    { 0xbfa3168a, 55 },
    { 0x2103168b, 90 },
    { 0x7493168c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf631691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa731698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3733169b, 95 },
    { 0xb2a3169b, 98 },
    { 0, 0 },
    { 0x5e03169e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650316a2, 48 },
    { 0, 0 },
    { 0xea2316a4, 68 },
    { 0, 0 },
    { 0x1af316a6, 33 },
    { 0x7f9316a7, 98 },
    { 0xce9316a7, 61 },
    { 0, 0 },
    { 0x5a5316aa, 33 },
    { 0xc79316ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9316b0, 61 },
    { 0, 0 },
    { 0x54a316b2, 77 },
    { 0xf1c316b2, 52 },
    { 0x711316b2, 34 },
    { 0xda9316b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96316bd, 93 },
    { 0, 0 },
    { 0x5c0316bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3316c5, 91 },
    { 0xa99316c6, 61 },
    { 0x203316c7, 61 },
    { 0xb26316c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62316cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4316d1, 93 },
    { 0, 0 },
    { 0x7bc316d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f316db, 46 },
    { 0x936316db, 61 },
    { 0, 0 },
    { 0x3bc316de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55316e1, 95 },
    { 0, 0 },
    { 0x560316e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa316e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa316ea, 90 },
    { 0x880316eb, 33 },
    { 0x421316eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f316f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4316f4, 90 },
    { 0, 0 },
    { 0xa49316f6, 46 },
    { 0xdef316f6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5316fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee031700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a31707, 84 },
    { 0x2ba31708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3170c, 90 },
    { 0xcb3170d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad31712, 95 },
    { 0x4ba31713, 95 },
    { 0, 0 },
    { 0x91d31715, 52 },
    { 0x70431716, 78 },
    { 0x6c231717, 81 },
    { 0, 0 },
    { 0x41b31719, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7383171c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b531721, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8631724, 91 },
    { 0, 0 },
    { 0xcfa31726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d31729, 34 },
    { 0x6533172a, 84 },
    { 0xeb3172b, 96 },
    { 0, 0 },
    { 0x4723172d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4231731, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d431734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a93173e, 61 },
    { 0xf083173e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6131742, 91 },
    { 0xabc31743, 95 },
    { 0x72931744, 43 },
    { 0xb3631745, 90 },
    { 0xd3a31743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f531749, 61 },
    { 0, 0 },
    { 0xb1e3174b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd633174f, 90 },
    { 0xd3931750, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22031754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c31758, 43 },
    { 0, 0 },
    { 0xf873175a, 96 },
    { 0x69d3175a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3175f, 76 },
    { 0x34731760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f431765, 95 },
    { 0xdbc31766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93176a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2343176d, 43 },
    { 0xb1e3176e, 95 },
    { 0xde33176e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49531772, 33 },
    { 0x36931772, 61 },
    { 0x59431774, 17 },
    { 0xbc731775, 46 },
    { 0xf2a31775, 77 },
    { 0xd2231776, 61 },
    { 0xe2b31773, 95 },
    { 0x9f431779, 92 },
    { 0x35b31779, 66 },
    { 0, 0 },
    { 0x7c03177c, 66 },
    { 0xc2d3177c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b31780, 88 },
    { 0xb5b31780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9031785, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b31789, 61 },
    { 0x3573178a, 98 },
    { 0x87d3178b, 90 },
    { 0x6ad3178c, 33 },
    { 0x5463178d, 98 },
    { 0x2223178e, 95 },
    { 0xbe43178e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d31795, 90 },
    { 0x33d31796, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3623179c, 95 },
    { 0x2323179c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8317a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26317a5, 90 },
    { 0, 0 },
    { 0x401317a7, 61 },
    { 0x24c317a8, 78 },
    { 0xffd317a8, 23 },
    { 0x503317aa, 81 },
    { 0xaa6317aa, 95 },
    { 0xf0e317a9, 61 },
    { 0x934317aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b317b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5317b4, 99 },
    { 0xfb2317b4, 98 },
    { 0xac2317b4, 90 },
    { 0xbfd317b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5317be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61317c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1317cc, 46 },
    { 0x215317cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470317d1, 84 },
    { 0xa23317d2, 55 },
    { 0x436317d3, 90 },
    { 0xb70317d4, 76 },
    { 0xac317d2, 33 },
    { 0xffa317d1, 90 },
    { 0xf95317d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f317da, 98 },
    { 0x911317db, 55 },
    { 0xfec317dc, 76 },
    { 0x193317dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab317e1, 87 },
    { 0xf0f317e1, 98 },
    { 0xda9317e2, 46 },
    { 0x51317e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95317ef, 92 },
    { 0, 0 },
    { 0x4ab317f1, 33 },
    { 0x311317f2, 90 },
    { 0x8eb317f3, 98 },
    { 0x2f4317f3, 55 },
    { 0xd67317f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb317fb, 66 },
    { 0, 0 },
    { 0x1a7317fd, 61 },
    { 0, 0 },
    { 0x442317ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a31803, 61 },
    { 0x71e31804, 90 },
    { 0xa5231804, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b3180e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c31811, 95 },
    { 0, 0 },
    { 0x50731813, 66 },
    { 0xd6431814, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb831818, 34 },
    { 0x96831819, 76 },
    { 0xf0d3181a, 61 },
    { 0xc9f31818, 95 },
    { 0x9a33181a, 87 },
    { 0x6543181d, 99 },
    { 0, 0 },
    { 0x33b3181f, 53 },
    { 0x8e23181f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee31823, 66 },
    { 0, 0 },
    { 0xfba31825, 23 },
    { 0xdf331826, 66 },
    { 0x8c931827, 90 },
    { 0x2c231828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8573182d, 99 },
    { 0x6a13182e, 98 },
    { 0x543182d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4731833, 33 },
    { 0x90331834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a31838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b3183b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e3183e, 77 },
    { 0x8213183f, 90 },
    { 0xf5831840, 61 },
    { 0xb713183f, 33 },
    { 0x2e831842, 76 },
    { 0x4bc31843, 46 },
    { 0x54931843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf31849, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a3184d, 61 },
    { 0xc1c3184e, 55 },
    { 0xf373184f, 95 },
    { 0x3bd3184e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be31856, 88 },
    { 0, 0 },
    { 0xf7a31858, 43 },
    { 0xcbf31859, 33 },
    { 0x4b31859, 61 },
    { 0xfb83185a, 100 },
    { 0x7de3185c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc31862, 46 },
    { 0x7d731863, 84 },
    { 0xd3131862, 55 },
    { 0xd4831863, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea831869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf3186d, 33 },
    { 0xd0f3186d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf531871, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d31879, 61 },
    { 0x9ec31879, 76 },
    { 0xdc33187a, 48 },
    { 0x96d3187c, 9 },
    { 0x9303187d, 81 },
    { 0x8673187d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f031884, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3188b, 90 },
    { 0, 0 },
    { 0xf3c3188d, 90 },
    { 0xc4e3188d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71431891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3189c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f318a0, 91 },
    { 0x7a2318a0, 68 },
    { 0, 0 },
    { 0x35f318a3, 87 },
    { 0, 0 },
    { 0x8f318a5, 76 },
    { 0xcef318a5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef318a9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3318ad, 90 },
    { 0xa8318ae, 23 },
    { 0x669318af, 34 },
    { 0x354318b0, 33 },
    { 0x9a4318b1, 61 },
    { 0xf6d318af, 76 },
    { 0, 0 },
    { 0x24e318b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72318b7, 90 },
    { 0, 0 },
    { 0xeba318b9, 61 },
    { 0, 0 },
    { 0x8f3318bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c318bf, 33 },
    { 0x38b318c0, 33 },
    { 0x71f318bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e318c7, 95 },
    { 0xc33318c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c318cb, 61 },
    { 0xf1d318cc, 33 },
    { 0, 0 },
    { 0x7e7318ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565318d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5318d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6318da, 33 },
    { 0, 0 },
    { 0x882318dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1318df, 61 },
    { 0, 0 },
    { 0x434318e1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3318e5, 33 },
    { 0, 0 },
    { 0x477318e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632318ee, 33 },
    { 0x90a318ee, 23 },
    { 0xa7a318f0, 55 },
    { 0x1d4318f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8318f9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x455318fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f31900, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38831905, 98 },
    { 0x11431906, 95 },
    { 0, 0 },
    { 0x56731908, 33 },
    { 0x82131908, 33 },
    { 0xdd431908, 84 },
    { 0x25b3190b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35331917, 55 },
    { 0x7e131918, 90 },
    { 0xfbc31917, 85 },
    { 0, 0 },
    { 0xe6e3191b, 61 },
    { 0xe253191c, 90 },
    { 0x6033191c, 90 },
    { 0x5ec3191e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431923, 76 },
    { 0x61d31923, 61 },
    { 0, 0 },
    { 0x79331926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d31939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3193d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95631946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe473194a, 81 },
    { 0xb383194a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7231950, 84 },
    { 0, 0 },
    { 0x6ba31952, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98131956, 90 },
    { 0xa1a31956, 76 },
    { 0x37831956, 61 },
    { 0xf4331957, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36131961, 61 },
    { 0, 0 },
    { 0x50931963, 98 },
    { 0x4c831964, 46 },
    { 0, 0 },
    { 0x1ed31966, 52 },
    { 0, 0 },
    { 0xce431968, 68 },
    { 0x37a31969, 76 },
    { 0x40f31969, 33 },
    { 0, 0 },
    { 0xb293196c, 33 },
    { 0x60c3196d, 90 },
    { 0xdb63196e, 93 },
    { 0, 0 },
    { 0x1cf31970, 43 },
    { 0xc231971, 85 },
    { 0xb0931972, 46 },
    { 0x1f331972, 93 },
    { 0x64831974, 33 },
    { 0x98931974, 23 },
    { 0xc4831974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c3197a, 34 },
    { 0, 0 },
    { 0xb003197c, 55 },
    { 0x4213197c, 23 },
    { 0x61f3197c, 61 },
    { 0x9cd3197e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14731985, 93 },
    { 0, 0 },
    { 0xafc31987, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa283198f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5231994, 61 },
    { 0, 0 },
    { 0x34831996, 76 },
    { 0x1a31997, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3199d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba319a0, 40 },
    { 0x9b6319a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2319a4, 85 },
    { 0, 0 },
    { 0x606319a6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144319aa, 78 },
    { 0xc7c319aa, 93 },
    { 0x54a319ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9319b4, 33 },
    { 0x64c319b5, 33 },
    { 0, 0 },
    { 0x495319b7, 85 },
    { 0, 0 },
    { 0x9c2319b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1319bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2319c3, 75 },
    { 0xaf9319c3, 90 },
    { 0, 0 },
    { 0x9bd319c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a319ca, 90 },
    { 0, 0 },
    { 0x97f319cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde319d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254319d5, 52 },
    { 0x807319d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e319de, 40 },
    { 0xd7c319df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13319e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc319e7, 90 },
    { 0x8fe319e8, 61 },
    { 0x12d319e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb319ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9319f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19319f5, 61 },
    { 0, 0 },
    { 0x18b319f7, 66 },
    { 0x603319f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x359319fb, 78 },
    { 0x51319fb, 23 },
    { 0xd52319fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e731a01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x17431a04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e531a07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37331a0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20831a0d, 33 },
    { 0x8a431a0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4531a11, 90 },
    { 0, 0 },
    { 0x8731a13, 68 },
    { 0xb7931a14, 46 },
    { 0xb6731a14, 61 },
    { 0x98431a13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd31a21, 55 },
    { 0xc3931a22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59431a25, 90 },
    { 0x66031a25, 23 },
    { 0xed531a27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7631a2c, 33 },
    { 0x9f031a2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd31a30, 61 },
    { 0x37131a30, 24 },
    { 0xe8431a31, 61 },
    { 0x77331a33, 43 },
    { 0xcda31a34, 46 },
    { 0xb4931a35, 98 },
    { 0xd8431a36, 34 },
    { 0xf5631a33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9431a3a, 52 },
    { 0x5a531a3a, 95 },
    { 0x27331a3c, 55 },
    { 0x1f631a3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f31a41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae131a45, 61 },
    { 0x36e31a45, 33 },
    { 0x49531a46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22131a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd431a5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e31a60, 91 },
    { 0xe9f31a61, 90 },
    { 0, 0 },
    { 0x80831a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18431a6b, 46 },
    { 0x1a231a6b, 90 },
    { 0, 0 },
    { 0x6ed31a6e, 91 },
    { 0x53331a6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9031a74, 43 },
    { 0xbb231a75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1931a7d, 76 },
    { 0xd0f31a7d, 34 },
    { 0x12331a7f, 34 },
    { 0xbe431a7f, 98 },
    { 0xda031a81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d331a85, 32 },
    { 0, 0 },
    { 0x8d531a87, 96 },
    { 0x61b31a87, 33 },
    { 0, 0 },
    { 0xfef31a8a, 61 },
    { 0x26731a8b, 51 },
    { 0x4a031a8c, 52 },
    { 0xcb31a8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5631a90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca31a93, 52 },
    { 0x51c31a94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2131a97, 90 },
    { 0, 0 },
    { 0x71b31a99, 84 },
    { 0, 0 },
    { 0xe6831a9b, 90 },
    { 0, 0 },
    { 0x2b331a9d, 33 },
    { 0, 0 },
    { 0xcfb31a9f, 99 },
    { 0x89031a9f, 90 },
    { 0x30731aa1, 66 },
    { 0x6631aa1, 61 },
    { 0x57e31aa3, 95 },
    { 0x63931aa3, 66 },
    { 0xaa731aa0, 91 },
    { 0x5c031aa0, 34 },
    { 0, 0 },
    { 0x65231aa8, 33 },
    { 0x5b131aa8, 90 },
    { 0xd0b31aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96131aad, 95 },
    { 0, 0 },
    { 0xaf731aaf, 23 },
    { 0x12331aaf, 61 },
    { 0x1a331ab0, 43 },
    { 0x9f331aaf, 38 },
    { 0xd5731ab2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba31ab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d431aba, 23 },
    { 0x54c31abb, 52 },
    { 0x98631abc, 76 },
    { 0x72931abb, 76 },
    { 0, 0 },
    { 0xf5331abf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f31ac6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86831ac9, 90 },
    { 0, 0 },
    { 0x6f31acb, 33 },
    { 0, 0 },
    { 0x8fb31acd, 95 },
    { 0x76931ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef31ad1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83231ad5, 55 },
    { 0x52031ad5, 33 },
    { 0x7b631ad6, 43 },
    { 0xebf31ad8, 61 },
    { 0x7e31ad9, 98 },
    { 0x2d531ada, 66 },
    { 0x70c31adb, 91 },
    { 0, 0 },
    { 0xf531add, 23 },
    { 0xfda31ade, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b31ae4, 83 },
    { 0xfd531ae5, 76 },
    { 0x91a31ae6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f831aeb, 90 },
    { 0, 0 },
    { 0x6ef31aed, 95 },
    { 0, 0 },
    { 0xfe631aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac31af6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c31af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5431aff, 43 },
    { 0xf9e31b00, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec31b03, 61 },
    { 0xc1d31b03, 23 },
    { 0, 0 },
    { 0x94331b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c231b09, 65 },
    { 0x19a31b0a, 95 },
    { 0x5131b0b, 90 },
    { 0x67631b0c, 66 },
    { 0x1dd31b0c, 98 },
    { 0xe1931b09, 23 },
    { 0, 0 },
    { 0x2c131b10, 34 },
    { 0x5dd31b10, 90 },
    { 0xe2931b12, 81 },
    { 0x9a831b11, 69 },
    { 0xc0431b14, 66 },
    { 0xe4531b13, 90 },
    { 0xd0231b16, 33 },
    { 0xaeb31b10, 33 },
    { 0x9f031b14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21031b1b, 66 },
    { 0xef231b1b, 76 },
    { 0xf1e31b1c, 90 },
    { 0x85c31b1e, 23 },
    { 0x55531b1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2331b22, 33 },
    { 0x93c31b22, 90 },
    { 0x2ae31b24, 52 },
    { 0, 0 },
    { 0xe1431b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b31b29, 81 },
    { 0xef531b2a, 95 },
    { 0xba231b2b, 87 },
    { 0xd6831b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc731b2f, 87 },
    { 0x51231b30, 98 },
    { 0x9d531b2f, 90 },
    { 0xaca31b32, 52 },
    { 0xeae31b33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f31b3b, 55 },
    { 0, 0 },
    { 0x6bc31b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3631b42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84031b4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd731b4e, 90 },
    { 0, 0 },
    { 0xbc831b50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23831b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31b59, 85 },
    { 0x8d031b5a, 99 },
    { 0x35d31b5b, 95 },
    { 0x84231b5a, 43 },
    { 0xfaa31b5d, 90 },
    { 0, 0 },
    { 0xbbe31b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f31b64, 52 },
    { 0, 0 },
    { 0x85731b66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45531b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3931b6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd631b76, 61 },
    { 0x30031b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6531b82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe31b85, 33 },
    { 0x9d931b86, 55 },
    { 0xa331b87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6631b8b, 78 },
    { 0x7d631b8b, 33 },
    { 0, 0 },
    { 0xb3531b8e, 52 },
    { 0x8eb31b8f, 90 },
    { 0x38631b8e, 61 },
    { 0xea931b91, 52 },
    { 0xecc31b92, 98 },
    { 0x5ae31b92, 95 },
    { 0x98531b94, 33 },
    { 0xaac31b94, 61 },
    { 0xe8631b94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1431ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf531ba7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x24231baa, 95 },
    { 0, 0 },
    { 0x4e431bac, 90 },
    { 0x12131bad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab31bb1, 33 },
    { 0, 0 },
    { 0xfcb31bb3, 100 },
    { 0, 0 },
    { 0x97731bb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f31bb8, 33 },
    { 0x65b31bb9, 55 },
    { 0x90531bba, 68 },
    { 0, 0 },
    { 0x13731bbc, 98 },
    { 0, 0 },
    { 0x43a31bbe, 90 },
    { 0x4e31bbf, 90 },
    { 0xf1831bbe, 90 },
    { 0xa3131bc1, 23 },
    { 0xf0831bc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38631bcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a31bd7, 81 },
    { 0x95a31bd8, 76 },
    { 0x93031bd9, 98 },
    { 0xb0231bd7, 34 },
    { 0x1f431bdb, 98 },
    { 0xaef31bdc, 98 },
    { 0x84d31bdd, 23 },
    { 0xad931bdc, 33 },
    { 0x48c31bdb, 76 },
    { 0xf931be0, 93 },
    { 0xf8831be0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb31be4, 90 },
    { 0, 0 },
    { 0x4f931be6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef31bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c31bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa31bff, 38 },
    { 0xc2e31bff, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f531c03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f31c09, 84 },
    { 0xc3631c0a, 95 },
    { 0x36b31c0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd31c0e, 98 },
    { 0x71f31c0e, 100 },
    { 0x2e231c0e, 95 },
    { 0x1ba31c10, 66 },
    { 0, 0 },
    { 0xce031c13, 99 },
    { 0x2a331c14, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb331c1a, 99 },
    { 0xe2031c1b, 23 },
    { 0x5de31c1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5431c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c31c25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e31c29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e731c2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea31c31, 90 },
    { 0xff831c32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b31c38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7931c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56531c42, 46 },
    { 0, 0 },
    { 0x91831c44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94831c51, 99 },
    { 0x6c431c52, 98 },
    { 0x1dc31c53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x58531c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31c62, 90 },
    { 0x62231c63, 95 },
    { 0x74031c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37031c68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d31c6e, 43 },
    { 0, 0 },
    { 0x88b31c70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e31c75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9831c78, 99 },
    { 0x6a331c78, 23 },
    { 0x32a31c7a, 23 },
    { 0xc4731c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f431c7e, 90 },
    { 0x28831c7f, 34 },
    { 0x2f431c7f, 55 },
    { 0xcfe31c81, 66 },
    { 0x67d31c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2631c85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931c8a, 43 },
    { 0, 0 },
    { 0x10831c8c, 98 },
    { 0xdb231c8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f31c93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b231c9f, 33 },
    { 0x25131ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b931ca3, 66 },
    { 0xc31ca4, 95 },
    { 0xed031ca3, 93 },
    { 0x3f431ca5, 33 },
    { 0, 0 },
    { 0xe3631ca8, 64 },
    { 0, 0 },
    { 0xba31caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52031cb1, 78 },
    { 0, 0 },
    { 0x8a231cb3, 34 },
    { 0x86631cb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d31cb9, 90 },
    { 0, 0 },
    { 0x28331cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf231cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e631cc7, 33 },
    { 0x84931cc7, 98 },
    { 0x2dc31cc8, 34 },
    { 0xbd31cca, 90 },
    { 0xf831ccb, 99 },
    { 0x49731ccb, 43 },
    { 0xfac31cc8, 17 },
    { 0x98431cce, 95 },
    { 0x5431ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde31cd5, 68 },
    { 0x7ba31cd6, 33 },
    { 0, 0 },
    { 0xa3631cd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c31cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d31ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab31ce7, 23 },
    { 0, 0 },
    { 0x6b31ce9, 98 },
    { 0x61231cea, 33 },
    { 0x7ca31cea, 91 },
    { 0x95631cec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef31cf2, 36 },
    { 0xf1231cf3, 61 },
    { 0, 0 },
    { 0xa2e31cf5, 99 },
    { 0x9a231cf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38931cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0131cfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a631d05, 98 },
    { 0x3d831d06, 90 },
    { 0x94331d06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad31d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16031d0e, 98 },
    { 0, 0 },
    { 0xe4c31d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa031d1e, 67 },
    { 0, 0 },
    { 0x79d31d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3931d23, 61 },
    { 0x4ac31d24, 95 },
    { 0, 0 },
    { 0x8eb31d26, 78 },
    { 0, 0 },
    { 0xf4931d28, 90 },
    { 0x95431d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12231d31, 78 },
    { 0xc3e31d31, 90 },
    { 0xbc431d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed31d3a, 34 },
    { 0x85131d3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d31d3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53531d42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8131d45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e31d4b, 85 },
    { 0x8cd31d4c, 90 },
    { 0x7cd31d4b, 95 },
    { 0, 0 },
    { 0x48231d4f, 90 },
    { 0xfd531d50, 46 },
    { 0x60831d4f, 33 },
    { 0x74831d51, 91 },
    { 0x3ac31d53, 33 },
    { 0x8de31d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b31d57, 68 },
    { 0x7e931d58, 99 },
    { 0x64d31d59, 61 },
    { 0xeb131d5a, 55 },
    { 0x62e31d5b, 95 },
    { 0x77e31d5c, 55 },
    { 0x81231d5d, 90 },
    { 0xed731d5e, 61 },
    { 0, 0 },
    { 0x37231d60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80431d68, 90 },
    { 0xf5931d68, 90 },
    { 0x8e631d6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b031d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931d72, 77 },
    { 0, 0 },
    { 0xd9131d74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c431d77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a31d7a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c31d7d, 23 },
    { 0, 0 },
    { 0x7231d7f, 23 },
    { 0x3b331d7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9731d84, 34 },
    { 0x34331d84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a731d88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99331d8b, 34 },
    { 0, 0 },
    { 0x5a831d8d, 33 },
    { 0xb8931d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7431d91, 23 },
    { 0, 0 },
    { 0x6b031d93, 90 },
    { 0x10531d93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7031d98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd31d9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe131d9e, 90 },
    { 0x77e31d9e, 33 },
    { 0xb5731da0, 95 },
    { 0x90731da0, 99 },
    { 0x9a231da2, 93 },
    { 0x8e331da3, 98 },
    { 0xf3031da1, 76 },
    { 0x40d31da3, 61 },
    { 0, 0 },
    { 0x17631da7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f931db0, 95 },
    { 0, 0 },
    { 0x56f31db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d131db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a31db9, 77 },
    { 0, 0 },
    { 0x65331dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29331dbe, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da31dc2, 24 },
    { 0x1a731dc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a531dc8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87631dd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6031dd3, 98 },
    { 0xd8a31dd4, 33 },
    { 0xbcc31dd5, 95 },
    { 0x3931dd3, 51 },
    { 0xcfb31dd7, 55 },
    { 0, 0 },
    { 0x3c931dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0131de3, 46 },
    { 0, 0 },
    { 0x1c231de5, 55 },
    { 0x91831de6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c31de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7031df3, 52 },
    { 0x3a931df3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a31df9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd931dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e31e02, 55 },
    { 0x3f731e03, 55 },
    { 0xd9531e02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a31e08, 93 },
    { 0, 0 },
    { 0xf5b31e0a, 61 },
    { 0x61e31e0b, 61 },
    { 0xd9931e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7131e12, 99 },
    { 0x73c31e12, 34 },
    { 0x79631e12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d931e19, 17 },
    { 0x33331e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f31e1d, 90 },
    { 0x85931e1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1831e21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f131e25, 61 },
    { 0, 0 },
    { 0xc3431e27, 84 },
    { 0xa4131e27, 69 },
    { 0, 0 },
    { 0x11131e2a, 90 },
    { 0x75531e2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd31e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67731e32, 46 },
    { 0xa1731e33, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91731e40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c31e43, 34 },
    { 0x88131e44, 76 },
    { 0, 0 },
    { 0xdb231e46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b31e4a, 95 },
    { 0x78731e4a, 98 },
    { 0xc2131e4c, 90 },
    { 0xedf31e4c, 89 },
    { 0x9a331e4e, 46 },
    { 0xde031e4e, 66 },
    { 0xb4a31e4f, 61 },
    { 0x3bc31e4e, 66 },
    { 0, 0 },
    { 0xc9d31e53, 33 },
    { 0x10231e54, 98 },
    { 0, 0 },
    { 0x5b131e56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7231e5c, 95 },
    { 0x10331e5d, 61 },
    { 0xb31e5e, 17 },
    { 0xa6e31e5f, 99 },
    { 0x8cc31e5f, 55 },
    { 0xec831e5d, 90 },
    { 0xf2331e62, 95 },
    { 0x82231e62, 76 },
    { 0x85c31e61, 66 },
    { 0x80c31e65, 86 },
    { 0x4dd31e60, 88 },
    { 0xe2031e5f, 36 },
    { 0xa2031e60, 90 },
    { 0xd2b31e69, 95 },
    { 0x47d31e6a, 98 },
    { 0xc8e31e6b, 99 },
    { 0x5b131e5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16231e6f, 90 },
    { 0x76e31e70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2831e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2731e7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62931e7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a31e85, 43 },
    { 0x2cf31e86, 33 },
    { 0xd1d31e87, 76 },
    { 0, 0 },
    { 0x3a431e89, 33 },
    { 0x91531e89, 85 },
    { 0x20731e8b, 91 },
    { 0x6aa31e8b, 43 },
    { 0xd3a31e8d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70531e98, 35 },
    { 0x38431e99, 76 },
    { 0, 0 },
    { 0xf4e31e9b, 66 },
    { 0x70c31e9c, 33 },
    { 0, 0 },
    { 0xdb431e9e, 90 },
    { 0, 0 },
    { 0xf4631ea0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29631ea4, 33 },
    { 0x30631ea4, 98 },
    { 0, 0 },
    { 0xcc031ea7, 43 },
    { 0xa8231ea8, 33 },
    { 0, 0 },
    { 0x67c31eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31731eae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e31eb3, 78 },
    { 0xd2131eb3, 61 },
    { 0x7ff31eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87231eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d631ebc, 77 },
    { 0, 0 },
    { 0xdf431ebe, 33 },
    { 0x4d731ebf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31ec5, 55 },
    { 0, 0 },
    { 0xbe931ec7, 55 },
    { 0x77831ec7, 33 },
    { 0, 0 },
    { 0x78c31eca, 66 },
    { 0, 0 },
    { 0x7a31ecc, 55 },
    { 0x7b031ecc, 90 },
    { 0, 0 },
    { 0x2ce31ecf, 90 },
    { 0xd2a31ed0, 46 },
    { 0, 0 },
    { 0x7fe31ed2, 33 },
    { 0, 0 },
    { 0x84a31ed4, 61 },
    { 0, 0 },
    { 0x68931ed6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca31ed9, 34 },
    { 0xb9c31eda, 61 },
    { 0xec131eda, 69 },
    { 0xbc531eda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d231ee3, 95 },
    { 0x98531ee4, 61 },
    { 0x56531ee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea731eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69631eef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc131ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a31ef8, 90 },
    { 0xd5031ef8, 93 },
    { 0x2c431efa, 33 },
    { 0xd1a31efb, 46 },
    { 0xadd31efb, 55 },
    { 0xe7631efa, 90 },
    { 0xb0131efb, 98 },
    { 0x70131eff, 61 },
    { 0xbce31efe, 23 },
    { 0x97d31eff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca31f05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af31f0f, 93 },
    { 0, 0 },
    { 0x9cc31f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef31f16, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf731f1b, 55 },
    { 0x50b31f1c, 90 },
    { 0, 0 },
    { 0xcc831f1e, 34 },
    { 0, 0 },
    { 0x1f431f20, 87 },
    { 0x27431f21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb831f24, 90 },
    { 0xf6b31f25, 90 },
    { 0xe4e31f26, 84 },
    { 0x66f31f26, 34 },
    { 0x39231f27, 61 },
    { 0x18731f26, 95 },
    { 0x27d31f25, 90 },
    { 0xf5031f2b, 95 },
    { 0, 0 },
    { 0xa3a31f2d, 66 },
    { 0x12231f2e, 46 },
    { 0x56b31f2f, 95 },
    { 0, 0 },
    { 0x44f31f31, 90 },
    { 0xdd531f31, 85 },
    { 0xdd031f32, 100 },
    { 0x38c31f31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca31f37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c231f3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b31f40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e31f43, 90 },
    { 0xc9b31f44, 23 },
    { 0xc4531f43, 98 },
    { 0, 0 },
    { 0x31c31f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22331f4a, 77 },
    { 0xbbc31f4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f31f52, 76 },
    { 0x36031f53, 88 },
    { 0xf7231f54, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e631f5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f31f64, 98 },
    { 0xc9b31f65, 51 },
    { 0, 0 },
    { 0x36b31f67, 84 },
    { 0x21e31f67, 76 },
    { 0x2a931f69, 81 },
    { 0xac331f6a, 61 },
    { 0x7ce31f6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d31f6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b231f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72631f75, 90 },
    { 0x20431f75, 61 },
    { 0xca831f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b31f7a, 83 },
    { 0xf4e31f7a, 90 },
    { 0xb8731f7c, 36 },
    { 0x66531f7a, 90 },
    { 0, 0 },
    { 0x81d31f7f, 90 },
    { 0x34031f80, 23 },
    { 0x37731f7f, 33 },
    { 0xccb31f82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d31f85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d631f88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d231f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62031f8f, 17 },
    { 0xa6131f90, 61 },
    { 0xa7731f91, 100 },
    { 0x60131f8f, 90 },
    { 0x42531f93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1131f9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c31faa, 33 },
    { 0xb2d31fab, 61 },
    { 0x13231fab, 61 },
    { 0, 0 },
    { 0x7b431fae, 61 },
    { 0xfef31faf, 61 },
    { 0x48e31fae, 95 },
    { 0, 0 },
    { 0x85731fb2, 43 },
    { 0, 0 },
    { 0x4d931fb4, 25 },
    { 0x34831fb5, 98 },
    { 0x99f31fb6, 46 },
    { 0, 0 },
    { 0xb5031fb8, 74 },
    { 0, 0 },
    { 0xefd31fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f31fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae431fc1, 65 },
    { 0x3af31fc1, 46 },
    { 0xf4731fc2, 46 },
    { 0x92a31fc4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b31fc7, 68 },
    { 0x4cf31fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d31fcc, 34 },
    { 0, 0 },
    { 0xa1031fce, 61 },
    { 0xaa131fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf831fd2, 93 },
    { 0xac431fd3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9731fd6, 91 },
    { 0xd3431fd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c31fde, 46 },
    { 0xeb431fde, 100 },
    { 0xf1c31fe0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61431fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b831fe7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab31ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29631ff7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b31ffa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f431ffd, 98 },
    { 0x85731ffe, 55 },
    { 0x8bf31ffd, 61 },
    { 0x68232000, 100 },
    { 0xba331ffd, 61 },
    { 0, 0 },
    { 0xb5332003, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed32009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63200c, 77 },
    { 0, 0 },
    { 0xd0a3200e, 98 },
    { 0x783200f, 95 },
    { 0x29432010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e32013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f32018, 61 },
    { 0, 0 },
    { 0xb2b3201a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38832020, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a332024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be32030, 33 },
    { 0, 0 },
    { 0x7bf32032, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71632036, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb703203a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3203d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c32045, 33 },
    { 0x2ef32045, 61 },
    { 0x4cf32045, 96 },
    { 0xe3f32047, 33 },
    { 0x83132045, 76 },
    { 0, 0 },
    { 0xd643204b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82232054, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41632059, 97 },
    { 0x5783205a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3205d, 52 },
    { 0x2d43205d, 95 },
    { 0xc963205e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc43206d, 34 },
    { 0, 0 },
    { 0x2f03206f, 36 },
    { 0xedd32070, 40 },
    { 0xba032071, 55 },
    { 0x92332072, 65 },
    { 0x6bd32072, 93 },
    { 0x5d932070, 85 },
    { 0, 0 },
    { 0x4e732076, 90 },
    { 0x7f432077, 23 },
    { 0xa432078, 90 },
    { 0x16032078, 100 },
    { 0, 0 },
    { 0xe213207b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf03207f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f32086, 90 },
    { 0x1f532087, 98 },
    { 0x42d32087, 78 },
    { 0, 0 },
    { 0x25a3208a, 90 },
    { 0, 0 },
    { 0x7f03208c, 23 },
    { 0x5503208d, 33 },
    { 0x6343208e, 33 },
    { 0x3dd3208f, 40 },
    { 0x9993208e, 99 },
    { 0, 0 },
    { 0x2f732092, 23 },
    { 0x2ee32093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7732096, 33 },
    { 0xb4232097, 55 },
    { 0x69332097, 95 },
    { 0x89a32099, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985320a2, 67 },
    { 0, 0 },
    { 0x902320a4, 76 },
    { 0x3b9320a5, 34 },
    { 0, 0 },
    { 0xef7320a7, 98 },
    { 0xa44320a7, 98 },
    { 0xa35320a9, 34 },
    { 0xa0320aa, 85 },
    { 0x4ed320a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7320ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2320b9, 99 },
    { 0x525320ba, 55 },
    { 0xc5320bb, 95 },
    { 0xd3e320bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863320c0, 90 },
    { 0, 0 },
    { 0x92f320c2, 23 },
    { 0xbf0320c3, 95 },
    { 0, 0 },
    { 0x92f320c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e320c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403320d2, 95 },
    { 0xdd1320d2, 90 },
    { 0x3bf320d2, 61 },
    { 0, 0 },
    { 0x3e4320d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247320dc, 78 },
    { 0, 0 },
    { 0x860320de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf320e3, 52 },
    { 0, 0 },
    { 0x97b320e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45320e9, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25320ec, 61 },
    { 0, 0 },
    { 0x167320ee, 52 },
    { 0, 0 },
    { 0x786320f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9320f5, 33 },
    { 0, 0 },
    { 0xba3320f7, 61 },
    { 0x227320f8, 23 },
    { 0, 0 },
    { 0x8db320fa, 80 },
    { 0xb53320fb, 55 },
    { 0, 0 },
    { 0x536320fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7232100, 90 },
    { 0x10732101, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f32105, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e32109, 98 },
    { 0, 0 },
    { 0xf033210b, 76 },
    { 0xbbb3210c, 49 },
    { 0, 0 },
    { 0x9cd3210e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c32116, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c3211d, 77 },
    { 0x1b73211d, 61 },
    { 0xdc83211e, 90 },
    { 0x1cc3211d, 90 },
    { 0xc8a32121, 90 },
    { 0, 0 },
    { 0xdc232123, 95 },
    { 0xa4f32124, 99 },
    { 0x34932125, 46 },
    { 0x77b32126, 66 },
    { 0xd1632125, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff3212a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8283212e, 66 },
    { 0x2653212f, 61 },
    { 0x29a3212e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32134, 76 },
    { 0x63f32134, 90 },
    { 0xcd332136, 66 },
    { 0xa7432136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13213a, 90 },
    { 0x5133213b, 78 },
    { 0x6a83213c, 36 },
    { 0x3c73213d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a332141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e32145, 33 },
    { 0, 0 },
    { 0x35532147, 76 },
    { 0x95132148, 33 },
    { 0, 0 },
    { 0x54d3214a, 46 },
    { 0, 0 },
    { 0xcc33214c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde03214f, 69 },
    { 0, 0 },
    { 0xb32151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b32155, 95 },
    { 0x9332156, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a3215c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c32161, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21632165, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e832168, 61 },
    { 0xb1532168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43216e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd32172, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b32179, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa3217c, 61 },
    { 0, 0 },
    { 0x443217e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87532186, 61 },
    { 0xa0b32187, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc103218a, 98 },
    { 0, 0 },
    { 0x8c93218c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c03218f, 93 },
    { 0x56232190, 90 },
    { 0x77a32191, 66 },
    { 0x13832192, 90 },
    { 0, 0 },
    { 0xa5732194, 81 },
    { 0x81032195, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f632199, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3219e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399321af, 76 },
    { 0x40321b0, 33 },
    { 0xa8d321b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x475321b4, 90 },
    { 0x5d4321b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2321bb, 76 },
    { 0xb9d321bc, 17 },
    { 0xb9321bc, 99 },
    { 0xf69321bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86321c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695321c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e321c9, 88 },
    { 0x76e321c9, 77 },
    { 0x3ee321ca, 95 },
    { 0x23f321cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686321d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210321da, 95 },
    { 0xc3c321db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9321e4, 98 },
    { 0x919321e5, 61 },
    { 0, 0 },
    { 0xd7321e7, 23 },
    { 0x707321e7, 74 },
    { 0, 0 },
    { 0x5b321ea, 19 },
    { 0x72e321eb, 52 },
    { 0xbc0321ea, 90 },
    { 0x75f321ec, 61 },
    { 0x533321eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847321f2, 76 },
    { 0xb75321f2, 61 },
    { 0x49321f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e321f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c32201, 100 },
    { 0xd4932202, 23 },
    { 0, 0 },
    { 0xf4132204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8632207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c3220a, 93 },
    { 0x1103220b, 88 },
    { 0, 0 },
    { 0x92b3220d, 98 },
    { 0, 0 },
    { 0xc6a3220f, 61 },
    { 0xab332210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2332216, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a32220, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda032229, 23 },
    { 0, 0 },
    { 0xa003222b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673222e, 23 },
    { 0x6a33222f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b832232, 97 },
    { 0x9ad32232, 77 },
    { 0xa232232, 34 },
    { 0xf1b32235, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29132238, 76 },
    { 0, 0 },
    { 0xc8f3223a, 66 },
    { 0xc743223b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf532241, 61 },
    { 0, 0 },
    { 0x24332243, 90 },
    { 0xab32244, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa3224b, 95 },
    { 0, 0 },
    { 0x143224d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a32250, 17 },
    { 0x87c32251, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5032254, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e232259, 33 },
    { 0x91a3225a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa093225d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f232262, 23 },
    { 0x25d32262, 43 },
    { 0x80832264, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c03226b, 34 },
    { 0x47a3226c, 61 },
    { 0, 0 },
    { 0x7523226e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73832271, 77 },
    { 0xcce32271, 61 },
    { 0x5d32272, 61 },
    { 0x14b32274, 78 },
    { 0x1fe32272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50132278, 23 },
    { 0, 0 },
    { 0xaf13227a, 46 },
    { 0x5693227b, 33 },
    { 0x9a03227c, 98 },
    { 0xf8f3227d, 23 },
    { 0xf1d3227c, 95 },
    { 0x7863227c, 46 },
    { 0xf2e3227f, 33 },
    { 0xd5d32281, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce632284, 46 },
    { 0, 0 },
    { 0x91e32286, 61 },
    { 0, 0 },
    { 0x2c132288, 88 },
    { 0x8bf32289, 95 },
    { 0x4043228a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc803228d, 95 },
    { 0, 0 },
    { 0x6cd3228f, 98 },
    { 0x8f23228f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd32295, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c832298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb322a3, 76 },
    { 0x76a322a4, 46 },
    { 0x8bd322a5, 98 },
    { 0xf51322a3, 85 },
    { 0x7c4322a5, 75 },
    { 0x67d322a3, 95 },
    { 0xf2a322a7, 100 },
    { 0x889322a4, 33 },
    { 0x50b322ab, 78 },
    { 0x4f6322ac, 95 },
    { 0xdb1322a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a322b0, 34 },
    { 0xfa2322b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f322b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x577322b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a322bb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b322cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75322d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b322d9, 33 },
    { 0x4d5322d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46322e1, 76 },
    { 0xfef322e2, 98 },
    { 0x2b322e2, 61 },
    { 0xd71322e4, 95 },
    { 0, 0 },
    { 0xcb0322e6, 33 },
    { 0x41c322e7, 84 },
    { 0xe25322e6, 90 },
    { 0xe9d322e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97322f3, 52 },
    { 0x470322f3, 85 },
    { 0xb3e322f5, 66 },
    { 0xe0a322f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23322f9, 91 },
    { 0xd62322fa, 98 },
    { 0x69b322fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44322ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f32304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a932308, 93 },
    { 0, 0 },
    { 0x7623230a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43230d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f732311, 67 },
    { 0, 0 },
    { 0x6d332313, 76 },
    { 0x14b32314, 58 },
    { 0, 0 },
    { 0xe2232316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef32319, 99 },
    { 0xd43231a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e43231d, 33 },
    { 0x2e93231e, 55 },
    { 0, 0 },
    { 0xd8332320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b32324, 95 },
    { 0xb7532324, 66 },
    { 0x7d132326, 55 },
    { 0x66532327, 95 },
    { 0x79b32327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3232e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f332331, 66 },
    { 0x22832332, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983233e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd332341, 23 },
    { 0xc7532342, 95 },
    { 0x48b32341, 98 },
    { 0xbf132344, 76 },
    { 0x7da32344, 91 },
    { 0, 0 },
    { 0x31632347, 43 },
    { 0x6c932348, 78 },
    { 0xff132349, 55 },
    { 0xce432349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7432351, 33 },
    { 0xdd432351, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f432358, 7 },
    { 0, 0 },
    { 0x3b13235a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f3235e, 100 },
    { 0x9ba3235f, 52 },
    { 0x57932360, 98 },
    { 0xbcd32360, 96 },
    { 0xbba3235f, 90 },
    { 0, 0 },
    { 0x7a032364, 90 },
    { 0, 0 },
    { 0x10532366, 68 },
    { 0xdeb32366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d3236a, 68 },
    { 0xf193236a, 100 },
    { 0x44a3236a, 95 },
    { 0xbdb3236c, 98 },
    { 0x4d3236e, 91 },
    { 0xf123236b, 78 },
    { 0xa1f32370, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x64932373, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a232378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f3237b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f32383, 43 },
    { 0xa1232384, 33 },
    { 0x9fa32385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa393238b, 61 },
    { 0, 0 },
    { 0xfa23238d, 23 },
    { 0xf5d3238e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6132393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c32397, 91 },
    { 0xdb332398, 90 },
    { 0x93c32399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3239d, 95 },
    { 0x45a3239d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250323a4, 48 },
    { 0, 0 },
    { 0x737323a6, 85 },
    { 0xb39323a7, 43 },
    { 0x245323a8, 98 },
    { 0x213323a9, 43 },
    { 0x2b6323aa, 67 },
    { 0, 0 },
    { 0xdf5323ac, 61 },
    { 0x566323ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1323b8, 83 },
    { 0xb24323b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49323be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b323c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x946323c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec323ce, 80 },
    { 0x6dc323cf, 61 },
    { 0xdaa323ce, 23 },
    { 0x2ce323d1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7323d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2323d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8323e4, 91 },
    { 0xb81323e5, 61 },
    { 0xa65323e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb323ef, 87 },
    { 0x635323ef, 33 },
    { 0xc09323ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad323f5, 55 },
    { 0, 0 },
    { 0x4f5323f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5323fc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7323ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c32403, 81 },
    { 0xa5132403, 61 },
    { 0xf0432405, 46 },
    { 0, 0 },
    { 0x10032407, 34 },
    { 0, 0 },
    { 0xa1332409, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63532411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c32416, 95 },
    { 0x12032416, 99 },
    { 0x11332418, 33 },
    { 0x73b32419, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d3241f, 23 },
    { 0x7ae32420, 90 },
    { 0x36032420, 91 },
    { 0, 0 },
    { 0x98732423, 33 },
    { 0xeb632424, 85 },
    { 0x99e32425, 98 },
    { 0, 0 },
    { 0xbba32427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3242c, 93 },
    { 0x8903242d, 77 },
    { 0xb943242e, 23 },
    { 0, 0 },
    { 0xcf132430, 88 },
    { 0, 0 },
    { 0x9ad32432, 77 },
    { 0x76432433, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb32436, 95 },
    { 0, 0 },
    { 0x21332438, 33 },
    { 0x33832438, 23 },
    { 0x5e232438, 97 },
    { 0xbe03243b, 61 },
    { 0x3432439, 75 },
    { 0x21832439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce32449, 40 },
    { 0x4f53244a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e732454, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe83245a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc32460, 61 },
    { 0xd0932461, 91 },
    { 0, 0 },
    { 0x3bc32463, 78 },
    { 0xa7832463, 55 },
    { 0, 0 },
    { 0x35e32466, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1673246e, 90 },
    { 0x29e3246f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9233247a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c3247e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38832484, 33 },
    { 0xe3d32485, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d32489, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c23248c, 46 },
    { 0xa163248d, 46 },
    { 0x9ff3248e, 98 },
    { 0xcab3248f, 87 },
    { 0x5bb3248c, 98 },
    { 0x18332491, 90 },
    { 0x5e832492, 77 },
    { 0, 0 },
    { 0x3dc32494, 95 },
    { 0x98632495, 67 },
    { 0xa6332496, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a432499, 99 },
    { 0xad93249a, 95 },
    { 0xe3d3249b, 90 },
    { 0, 0 },
    { 0x8b23249d, 93 },
    { 0x10f3249e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b324a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262324a5, 61 },
    { 0x58c324a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615324ab, 85 },
    { 0x79324ac, 17 },
    { 0, 0 },
    { 0x6c3324ae, 76 },
    { 0x76324af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65324b3, 33 },
    { 0x413324b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55324b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3324bc, 33 },
    { 0, 0 },
    { 0xabe324be, 33 },
    { 0x70a324bf, 23 },
    { 0xb61324c0, 61 },
    { 0, 0 },
    { 0xcd8324c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971324c6, 66 },
    { 0xd7324c6, 90 },
    { 0, 0 },
    { 0xa88324c9, 17 },
    { 0x1e2324ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b324ce, 43 },
    { 0x141324cf, 98 },
    { 0xb31324cf, 95 },
    { 0x6e5324d1, 61 },
    { 0xd87324d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888324d5, 61 },
    { 0x747324d5, 98 },
    { 0xcb324d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d324db, 95 },
    { 0, 0 },
    { 0x6d8324dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab324e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f324e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d324e7, 76 },
    { 0, 0 },
    { 0x459324e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03324ed, 46 },
    { 0x404324ee, 43 },
    { 0x50a324ef, 33 },
    { 0x7cf324f0, 91 },
    { 0x4c4324f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a324f9, 99 },
    { 0x479324fa, 46 },
    { 0x324324fb, 55 },
    { 0x6ab324fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e32500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7232503, 52 },
    { 0x1c32504, 91 },
    { 0xc6132505, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0232508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe013250e, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b32512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c32523, 61 },
    { 0x23132524, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13252b, 31 },
    { 0x7743252c, 38 },
    { 0x5e53252c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d32538, 66 },
    { 0, 0 },
    { 0x6ab3253a, 76 },
    { 0, 0 },
    { 0xde23253c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39932540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e32543, 65 },
    { 0x4bc32543, 55 },
    { 0x20f32545, 90 },
    { 0xa7732544, 61 },
    { 0x1df32547, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3254a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf123254f, 91 },
    { 0xaf132550, 95 },
    { 0, 0 },
    { 0xec032552, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa343255a, 90 },
    { 0, 0 },
    { 0x2033255c, 33 },
    { 0xcd03255c, 34 },
    { 0xd013255e, 34 },
    { 0, 0 },
    { 0x5d132560, 90 },
    { 0xb2832560, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa583256c, 76 },
    { 0x91b3256d, 90 },
    { 0x39a3256c, 56 },
    { 0x5183256f, 23 },
    { 0x9c53256f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b32579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223257f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43732588, 76 },
    { 0x26c32589, 95 },
    { 0x5503258a, 91 },
    { 0xca73258b, 66 },
    { 0x56832589, 95 },
    { 0x42f3258d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f32591, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc32594, 46 },
    { 0x1f132595, 99 },
    { 0x55832596, 98 },
    { 0xf132596, 33 },
    { 0x70332596, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a3259b, 95 },
    { 0x2663259c, 17 },
    { 0xb943259d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86325a1, 34 },
    { 0, 0 },
    { 0x2af325a3, 95 },
    { 0x8d6325a4, 90 },
    { 0, 0 },
    { 0x2ee325a6, 23 },
    { 0, 0 },
    { 0xd5c325a8, 33 },
    { 0xca1325a8, 90 },
    { 0, 0 },
    { 0x410325ab, 66 },
    { 0x114325ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615325b1, 95 },
    { 0x99a325b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94325b6, 90 },
    { 0xcca325b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3325ba, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51325c0, 95 },
    { 0xdb325c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd325c4, 61 },
    { 0, 0 },
    { 0x1b5325c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x340325c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67325cc, 90 },
    { 0x48e325cd, 99 },
    { 0xef1325ce, 34 },
    { 0, 0 },
    { 0x259325d0, 90 },
    { 0, 0 },
    { 0x5d7325d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264325d9, 61 },
    { 0x63325da, 85 },
    { 0x810325da, 34 },
    { 0x9be325dc, 23 },
    { 0xfca325dd, 43 },
    { 0x735325de, 93 },
    { 0xd45325dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff325e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88325ea, 52 },
    { 0x19c325ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0325f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33325fb, 91 },
    { 0xaf7325fb, 95 },
    { 0x749325fd, 61 },
    { 0xa16325fc, 61 },
    { 0x9e7325ff, 90 },
    { 0x3ad325ff, 33 },
    { 0xbdf32601, 61 },
    { 0xeba32602, 88 },
    { 0xa7832602, 33 },
    { 0x9c632604, 46 },
    { 0xac532605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a832608, 77 },
    { 0x54132609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa632615, 23 },
    { 0xbf132616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d632619, 76 },
    { 0, 0 },
    { 0x91b3261b, 55 },
    { 0x51c3261c, 77 },
    { 0x5c33261d, 23 },
    { 0, 0 },
    { 0x2473261f, 76 },
    { 0x2f32620, 23 },
    { 0, 0 },
    { 0x71232622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c432628, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d3262e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc32631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533263c, 61 },
    { 0, 0 },
    { 0x7aa3263e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c32645, 52 },
    { 0x9232645, 93 },
    { 0x11932647, 100 },
    { 0xe7e32648, 61 },
    { 0x6ad32649, 61 },
    { 0, 0 },
    { 0x1ab3264b, 98 },
    { 0xf163264c, 85 },
    { 0x4503264d, 91 },
    { 0x53d3264b, 33 },
    { 0x65f3264f, 95 },
    { 0, 0 },
    { 0xc7732651, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5732658, 23 },
    { 0xfd832659, 98 },
    { 0x1b932659, 67 },
    { 0xe4532658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f32661, 98 },
    { 0xcba32661, 55 },
    { 0xc7b32661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee232673, 66 },
    { 0x43732674, 61 },
    { 0xdb532675, 23 },
    { 0xecb32676, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5432679, 90 },
    { 0, 0 },
    { 0x8183267b, 76 },
    { 0xa603267b, 90 },
    { 0x9473267d, 61 },
    { 0xa383267e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4832683, 33 },
    { 0x7da32684, 77 },
    { 0x48732683, 33 },
    { 0xb8d32686, 90 },
    { 0xd3932686, 76 },
    { 0x6cb32688, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa3268c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a3268f, 46 },
    { 0x103268f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a32695, 76 },
    { 0, 0 },
    { 0x4bd32697, 90 },
    { 0xe7632698, 100 },
    { 0x1c832697, 55 },
    { 0x74a3269a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c326a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c326a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329326a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba326ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee326b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7326b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7326ba, 55 },
    { 0xd13326bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af326be, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe326c4, 88 },
    { 0, 0 },
    { 0xb9d326c6, 100 },
    { 0xc40326c6, 23 },
    { 0x8ab326c8, 97 },
    { 0, 0 },
    { 0x3fe326ca, 66 },
    { 0, 0 },
    { 0xd79326cc, 55 },
    { 0xb68326cc, 46 },
    { 0x73f326ce, 34 },
    { 0xf5b326cf, 66 },
    { 0x2e8326d0, 98 },
    { 0x5a3326d0, 99 },
    { 0x592326d2, 91 },
    { 0xb0e326d1, 66 },
    { 0xd1326d4, 43 },
    { 0x7ec326d1, 33 },
    { 0xbbf326d1, 75 },
    { 0xcc0326d7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8326dd, 83 },
    { 0x52d326de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76326e6, 85 },
    { 0xa2c326e6, 75 },
    { 0x58d326e8, 77 },
    { 0x451326e6, 36 },
    { 0, 0 },
    { 0x828326eb, 91 },
    { 0x6f6326ec, 90 },
    { 0xfdb326ed, 43 },
    { 0x45e326ed, 76 },
    { 0x5cd326ed, 84 },
    { 0, 0 },
    { 0x186326f1, 90 },
    { 0xa40326f2, 33 },
    { 0x4df326f2, 98 },
    { 0x7bf326f4, 61 },
    { 0xc97326f5, 61 },
    { 0x7ba326f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82326fc, 94 },
    { 0xe52326fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59432701, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29232709, 43 },
    { 0x1f03270a, 33 },
    { 0x5643270a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb93270f, 23 },
    { 0x3663270f, 43 },
    { 0x24e32711, 90 },
    { 0x72d32712, 90 },
    { 0xdf532713, 90 },
    { 0x71032713, 87 },
    { 0x9ad32713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3763271c, 33 },
    { 0x853271d, 98 },
    { 0xda53271d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9323272a, 76 },
    { 0x4d63272a, 34 },
    { 0x9a93272c, 90 },
    { 0x5243272a, 55 },
    { 0xb4e3272d, 90 },
    { 0, 0 },
    { 0x6f432730, 33 },
    { 0xe9332730, 17 },
    { 0x7e32732, 90 },
    { 0x6d932732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f63273a, 95 },
    { 0xa213273b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03273e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65832741, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91732746, 65 },
    { 0x55a32747, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf32750, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e732755, 33 },
    { 0x4e932756, 23 },
    { 0, 0 },
    { 0xe132758, 34 },
    { 0x14c32759, 61 },
    { 0x5773275a, 34 },
    { 0xb2b3275b, 77 },
    { 0x1623275c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3275f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24132763, 61 },
    { 0, 0 },
    { 0xe4f32765, 90 },
    { 0xa7c32766, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb3276b, 33 },
    { 0, 0 },
    { 0xf23276d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b32773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b3277d, 90 },
    { 0, 0 },
    { 0xab03277f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc3278b, 46 },
    { 0xbcf3278c, 76 },
    { 0x4473278d, 61 },
    { 0x3b73278e, 61 },
    { 0x8043278c, 98 },
    { 0x4b63278d, 93 },
    { 0xf7032791, 91 },
    { 0x52f32792, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c132795, 23 },
    { 0x5d332796, 95 },
    { 0xdaa32797, 33 },
    { 0, 0 },
    { 0x76d32799, 34 },
    { 0x13b32799, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xec73279d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d327a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613327a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914327ae, 85 },
    { 0, 0 },
    { 0x765327b0, 52 },
    { 0xe2b327b0, 85 },
    { 0, 0 },
    { 0xd88327b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x910327b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f327b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea327bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f327c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898327c7, 98 },
    { 0, 0 },
    { 0x96327c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf327ce, 95 },
    { 0xa4b327ce, 100 },
    { 0x89d327cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e327d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8327da, 34 },
    { 0x216327db, 52 },
    { 0xc70327dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x496327df, 90 },
    { 0, 0 },
    { 0xe6d327e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd327e4, 81 },
    { 0x211327e5, 93 },
    { 0xd02327e5, 33 },
    { 0x351327e7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f327ea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x706327ed, 90 },
    { 0x5b3327ee, 78 },
    { 0x8f3327ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea327f2, 61 },
    { 0, 0 },
    { 0x992327f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60327fd, 95 },
    { 0xd1e327fe, 77 },
    { 0x45c327fe, 90 },
    { 0xd3832800, 33 },
    { 0x6a932801, 46 },
    { 0x51832801, 93 },
    { 0xc5532803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3280e, 52 },
    { 0xbe43280f, 61 },
    { 0x64232810, 61 },
    { 0xd2b3280e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e32816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67232819, 100 },
    { 0x7c03281a, 46 },
    { 0x1e63281a, 85 },
    { 0x5f23281c, 23 },
    { 0, 0 },
    { 0x5553281e, 98 },
    { 0xedc3281f, 90 },
    { 0x5013281e, 66 },
    { 0xd0732821, 93 },
    { 0xf0a32822, 90 },
    { 0x7713281f, 76 },
    { 0xf932820, 87 },
    { 0, 0 },
    { 0x4432826, 23 },
    { 0xf0532827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5713282b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e3282f, 98 },
    { 0x80a32830, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9632833, 90 },
    { 0, 0 },
    { 0x42232835, 33 },
    { 0x97132836, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d3283e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61432845, 88 },
    { 0x2ca32845, 55 },
    { 0xd4d32847, 34 },
    { 0, 0 },
    { 0xe1032849, 78 },
    { 0x9193284a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963284f, 56 },
    { 0x4d32850, 61 },
    { 0, 0 },
    { 0x53132852, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce932856, 85 },
    { 0x1032856, 90 },
    { 0x2d632858, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c3285e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d632861, 65 },
    { 0, 0 },
    { 0x25632863, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a32866, 99 },
    { 0x59a32867, 90 },
    { 0xe8a32868, 75 },
    { 0xd7a32867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e43286c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf832872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c432875, 76 },
    { 0x12332875, 23 },
    { 0x7ae32875, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1423287a, 61 },
    { 0x7fd3287b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef53287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f432882, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe32887, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe003288d, 33 },
    { 0x2bf3288e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e732899, 95 },
    { 0xdc032899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe93289e, 95 },
    { 0x6c33289f, 90 },
    { 0, 0 },
    { 0xbf2328a1, 34 },
    { 0x1e9328a2, 61 },
    { 0xf07328a3, 33 },
    { 0, 0 },
    { 0x4cc328a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf328a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72328ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501328b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4328b8, 98 },
    { 0, 0 },
    { 0xe78328ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03328bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897328c1, 91 },
    { 0, 0 },
    { 0xea9328c3, 23 },
    { 0x216328c4, 76 },
    { 0x95b328c4, 95 },
    { 0, 0 },
    { 0x9ea328c7, 96 },
    { 0x98f328c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x798328cb, 23 },
    { 0, 0 },
    { 0xc38328cd, 93 },
    { 0x4b8328ce, 91 },
    { 0xe91328cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f328d2, 17 },
    { 0x45328d3, 33 },
    { 0, 0 },
    { 0xa54328d5, 75 },
    { 0, 0 },
    { 0xfd5328d7, 98 },
    { 0, 0 },
    { 0x3d0328d9, 61 },
    { 0xe87328d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba328dd, 84 },
    { 0x318328de, 90 },
    { 0, 0 },
    { 0xc57328e0, 61 },
    { 0xa01328e1, 98 },
    { 0x6c3328e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa328e6, 61 },
    { 0xd9b328e7, 23 },
    { 0, 0 },
    { 0x7e9328e9, 99 },
    { 0xa91328e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87328f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20328fa, 98 },
    { 0x7af328fb, 66 },
    { 0xad0328fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb32903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b32909, 91 },
    { 0xad33290a, 52 },
    { 0, 0 },
    { 0x4c73290c, 87 },
    { 0xb3f3290c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a32913, 99 },
    { 0x7c932914, 93 },
    { 0x22332915, 77 },
    { 0xa7932916, 23 },
    { 0x1a332917, 98 },
    { 0, 0 },
    { 0xd4432919, 55 },
    { 0x6b332919, 61 },
    { 0, 0 },
    { 0xc63291c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1032920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c32923, 46 },
    { 0x29f32923, 84 },
    { 0x33a32924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0032929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f3292c, 34 },
    { 0, 0 },
    { 0x2113292e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5832931, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19132936, 90 },
    { 0, 0 },
    { 0x53232938, 90 },
    { 0xa3032939, 33 },
    { 0xc6832939, 95 },
    { 0, 0 },
    { 0x5813293c, 91 },
    { 0, 0 },
    { 0x8ac3293e, 95 },
    { 0x3713293e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe332945, 87 },
    { 0xb5f32946, 93 },
    { 0xcb132947, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85032953, 39 },
    { 0xb5632954, 38 },
    { 0x7aa32955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3663295e, 90 },
    { 0x4e93295e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14132968, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f3296b, 81 },
    { 0, 0 },
    { 0x6f53296d, 23 },
    { 0xee3296d, 90 },
    { 0, 0 },
    { 0xc5d32970, 34 },
    { 0, 0 },
    { 0x66232972, 23 },
    { 0xea632973, 99 },
    { 0x2db32972, 34 },
    { 0xa2532975, 99 },
    { 0x6a532976, 88 },
    { 0x59832977, 95 },
    { 0, 0 },
    { 0x80132979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a53297c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be32988, 86 },
    { 0xb3332989, 90 },
    { 0x2d03298a, 95 },
    { 0x3d832989, 55 },
    { 0xa013298c, 52 },
    { 0x1683298b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce32990, 23 },
    { 0xfee32991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a32996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84832999, 34 },
    { 0xd493299a, 98 },
    { 0, 0 },
    { 0x1de3299c, 90 },
    { 0x92d3299d, 34 },
    { 0xad53299d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c329a7, 61 },
    { 0x7a4329a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8329ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa329b2, 90 },
    { 0, 0 },
    { 0x83b329b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb329b7, 76 },
    { 0xa08329b8, 84 },
    { 0xb1b329b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8329bc, 43 },
    { 0x116329bd, 91 },
    { 0xfba329bd, 55 },
    { 0xcc6329bf, 90 },
    { 0x85e329c0, 71 },
    { 0xfe329c1, 88 },
    { 0x87b329c2, 33 },
    { 0, 0 },
    { 0xc6a329c4, 91 },
    { 0xe8a329c4, 95 },
    { 0xc10329c6, 23 },
    { 0xa87329c6, 100 },
    { 0, 0 },
    { 0x78f329c9, 66 },
    { 0xebc329ca, 33 },
    { 0x1f1329ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b329ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb329d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38329d5, 100 },
    { 0, 0 },
    { 0xe55329d7, 46 },
    { 0x719329d7, 90 },
    { 0, 0 },
    { 0xefc329da, 46 },
    { 0x428329da, 95 },
    { 0, 0 },
    { 0x133329dd, 91 },
    { 0x1d4329de, 77 },
    { 0xf61329df, 98 },
    { 0xce3329df, 91 },
    { 0x2d3329de, 95 },
    { 0xef2329e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66329ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b329f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab329fb, 61 },
    { 0x378329fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e432a05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7132a08, 61 },
    { 0x9732a09, 23 },
    { 0xe9232a0a, 33 },
    { 0x85a32a08, 61 },
    { 0, 0 },
    { 0xbc332a0d, 78 },
    { 0, 0 },
    { 0x3d932a0f, 90 },
    { 0x6cf32a10, 95 },
    { 0xf6b32a11, 88 },
    { 0, 0 },
    { 0x98b32a13, 33 },
    { 0x3b532a13, 76 },
    { 0x1a432a14, 84 },
    { 0xfa132a15, 55 },
    { 0xd4b32a17, 23 },
    { 0xe3f32a16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde232a1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b32a23, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e332a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49132a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a532a2f, 98 },
    { 0xe9132a30, 95 },
    { 0xccb32a31, 52 },
    { 0, 0 },
    { 0xc9632a33, 34 },
    { 0xaba32a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94232a37, 46 },
    { 0xb7732a37, 61 },
    { 0x44732a39, 95 },
    { 0, 0 },
    { 0xfaf32a3b, 90 },
    { 0x61c32a3b, 99 },
    { 0xfa132a3d, 95 },
    { 0xa2732a3e, 91 },
    { 0x15532a3f, 61 },
    { 0xb7532a3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59732a43, 23 },
    { 0xff832a43, 95 },
    { 0x55432a45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e532a48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3632a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f32a4f, 76 },
    { 0, 0 },
    { 0xfb832a51, 33 },
    { 0, 0 },
    { 0xc4e32a53, 90 },
    { 0xf6032a53, 95 },
    { 0x74132a54, 76 },
    { 0, 0 },
    { 0xfb632a57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc32a5c, 23 },
    { 0, 0 },
    { 0x52432a5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1132a63, 66 },
    { 0x24332a63, 36 },
    { 0xff132a63, 100 },
    { 0x432a66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52432a6a, 61 },
    { 0, 0 },
    { 0xf032a6c, 78 },
    { 0x49732a6d, 87 },
    { 0x33632a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332a73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d32a7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc32a7e, 95 },
    { 0x50a32a7f, 55 },
    { 0x7ce32a7e, 23 },
    { 0, 0 },
    { 0x61f32a82, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa732a8a, 99 },
    { 0x93d32a8b, 93 },
    { 0xb132a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e32a91, 90 },
    { 0xcc232a91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab32a95, 90 },
    { 0x7dd32a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5732a99, 23 },
    { 0x25d32a99, 43 },
    { 0, 0 },
    { 0x19e32a9c, 23 },
    { 0xc6732a9d, 93 },
    { 0x92832a9e, 52 },
    { 0xb2d32a9e, 98 },
    { 0, 0 },
    { 0x8a732aa1, 19 },
    { 0xd0e32aa1, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55132aa8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb832aae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4432ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6932abe, 98 },
    { 0, 0 },
    { 0x4a632ac0, 98 },
    { 0x8f632ac0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc432ac6, 80 },
    { 0x68632ac7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2432aca, 90 },
    { 0, 0 },
    { 0x71532acc, 61 },
    { 0x81d32acc, 66 },
    { 0, 0 },
    { 0x64932acf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x62332ad2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef332ad8, 52 },
    { 0x51f32ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b32ae1, 98 },
    { 0, 0 },
    { 0x3b332ae3, 46 },
    { 0x8e532ae3, 90 },
    { 0xc5932ae5, 87 },
    { 0x15c32ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea32aed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b932af2, 61 },
    { 0, 0 },
    { 0x67232af4, 98 },
    { 0, 0 },
    { 0x36332af6, 43 },
    { 0x90a32af7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28132afc, 55 },
    { 0, 0 },
    { 0xb9432afe, 90 },
    { 0xf5b32afe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56832b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90932b08, 91 },
    { 0x4ea32b09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xada32b0c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f32b17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce332b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6832b24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e32b28, 88 },
    { 0x2df32b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7732b2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1732b30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d32b34, 93 },
    { 0, 0 },
    { 0x69c32b36, 90 },
    { 0xa6d32b37, 61 },
    { 0, 0 },
    { 0x8a732b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4332b3d, 33 },
    { 0x67232b3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e32b47, 97 },
    { 0, 0 },
    { 0xab032b49, 23 },
    { 0, 0 },
    { 0xa3432b4b, 33 },
    { 0xa8f32b4c, 90 },
    { 0xbbf32b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3832b50, 61 },
    { 0, 0 },
    { 0xd7e32b52, 76 },
    { 0xf9b32b53, 61 },
    { 0xe6232b54, 23 },
    { 0xef832b54, 23 },
    { 0, 0 },
    { 0x54232b57, 61 },
    { 0xea732b58, 55 },
    { 0xa3232b57, 34 },
    { 0xa8f32b5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83832b60, 55 },
    { 0x23432b61, 93 },
    { 0, 0 },
    { 0xe5732b63, 93 },
    { 0, 0 },
    { 0x22032b65, 76 },
    { 0, 0 },
    { 0x9dc32b67, 55 },
    { 0x99132b68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x29332b6b, 98 },
    { 0x65e32b6c, 67 },
    { 0x29f32b6b, 33 },
    { 0, 0 },
    { 0x1fc32b6f, 46 },
    { 0x53f32b70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f432b73, 93 },
    { 0x4b832b74, 98 },
    { 0x18d32b75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab732b78, 33 },
    { 0, 0 },
    { 0xfe732b7a, 87 },
    { 0xb4832b7a, 76 },
    { 0x35832b7c, 85 },
    { 0x32632b7d, 100 },
    { 0x75b32b7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f32b83, 90 },
    { 0, 0 },
    { 0xadc32b85, 23 },
    { 0, 0 },
    { 0xd4c32b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0832b95, 33 },
    { 0xc1d32b95, 23 },
    { 0, 0 },
    { 0x1e532b98, 27 },
    { 0x66932b98, 34 },
    { 0x5d632b9a, 76 },
    { 0, 0 },
    { 0x2a232b9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xca032b9f, 84 },
    { 0x78532ba0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36432ba7, 61 },
    { 0x63f32ba8, 100 },
    { 0, 0 },
    { 0x9d632baa, 33 },
    { 0x6d432baa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e32bb2, 76 },
    { 0xa4032bb3, 90 },
    { 0, 0 },
    { 0x7e932bb5, 33 },
    { 0x62732bb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d32bbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b32bbf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4032bc2, 95 },
    { 0, 0 },
    { 0xcd432bc4, 52 },
    { 0x7e632bc5, 90 },
    { 0xa7b32bc4, 90 },
    { 0, 0 },
    { 0xa7d32bc8, 23 },
    { 0xf832bc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d32bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b32bd2, 76 },
    { 0xad532bd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef32bd6, 87 },
    { 0, 0 },
    { 0x96b32bd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94932bdd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22132be2, 75 },
    { 0xb9232be2, 95 },
    { 0, 0 },
    { 0x4cf32be5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c32bea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a32bee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17132bf2, 61 },
    { 0xd5d32bf3, 98 },
    { 0x6ed32bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d32bf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b432c01, 33 },
    { 0x52732c02, 33 },
    { 0, 0 },
    { 0x8f032c04, 66 },
    { 0xfb232c05, 90 },
    { 0x6c732c05, 33 },
    { 0, 0 },
    { 0x60132c08, 76 },
    { 0, 0 },
    { 0x7bd32c0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332c16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d32c1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda932c1f, 67 },
    { 0x65232c1f, 98 },
    { 0x64632c21, 95 },
    { 0xf32c22, 98 },
    { 0x52832c23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a32c2c, 61 },
    { 0, 0 },
    { 0xdd32c2e, 98 },
    { 0xc7432c2f, 61 },
    { 0xe7032c2f, 90 },
    { 0xeab32c31, 90 },
    { 0xc8032c31, 98 },
    { 0x4a132c31, 78 },
    { 0xf3132c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84332c37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8032c3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3032c3e, 43 },
    { 0, 0 },
    { 0xfaf32c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f632c43, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5032c4d, 23 },
    { 0, 0 },
    { 0x87732c4f, 88 },
    { 0x65332c50, 90 },
    { 0, 0 },
    { 0x69b32c52, 90 },
    { 0x5af32c53, 95 },
    { 0xd2432c53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f32c5e, 46 },
    { 0xd4232c5f, 55 },
    { 0, 0 },
    { 0xa8732c61, 98 },
    { 0x58032c62, 93 },
    { 0x5cf32c62, 33 },
    { 0x50e32c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e32c70, 61 },
    { 0xde132c71, 66 },
    { 0x25332c72, 90 },
    { 0x9a332c73, 61 },
    { 0xf0132c74, 93 },
    { 0, 0 },
    { 0x57b32c76, 61 },
    { 0, 0 },
    { 0xd9532c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a32c7c, 88 },
    { 0x4b032c7c, 90 },
    { 0, 0 },
    { 0x89232c7f, 99 },
    { 0, 0 },
    { 0x4ae32c81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a32c84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8832c8d, 33 },
    { 0xe3a32c8e, 61 },
    { 0x5c932c8f, 98 },
    { 0xfbb32c90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b32c93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d32c96, 91 },
    { 0x9b832c97, 46 },
    { 0x7832c98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xead32c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5632c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d32ca2, 95 },
    { 0, 0 },
    { 0x55a32ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff32ca7, 100 },
    { 0xba32ca8, 95 },
    { 0, 0 },
    { 0x5da32caa, 77 },
    { 0xa5032caa, 101 },
    { 0x55432cac, 51 },
    { 0x91132cad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e32cb1, 95 },
    { 0, 0 },
    { 0x75532cb3, 33 },
    { 0x57132cb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d032cb8, 59 },
    { 0x36332cb8, 95 },
    { 0xb4232cba, 98 },
    { 0xca032cb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c32cc5, 33 },
    { 0xc6032cc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6532cc9, 75 },
    { 0x13432cc9, 95 },
    { 0, 0 },
    { 0xcba32ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32632cd6, 93 },
    { 0xb0432cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda432cdd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a932ce1, 87 },
    { 0x2de32ce2, 23 },
    { 0xb5832ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a532ce8, 100 },
    { 0x34a32ce8, 55 },
    { 0xf5e32cea, 81 },
    { 0, 0 },
    { 0x72332cec, 23 },
    { 0, 0 },
    { 0x19d32cee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c32cf5, 33 },
    { 0x53c32cf6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f232cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f732cff, 90 },
    { 0, 0 },
    { 0x46432d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff32d04, 90 },
    { 0xf0f32d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb532d08, 75 },
    { 0x2ce32d09, 23 },
    { 0x5b932d0a, 52 },
    { 0x64832d0b, 66 },
    { 0x5fa32d0b, 90 },
    { 0x8632d0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20232d10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf232d13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4432d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a332d1c, 67 },
    { 0, 0 },
    { 0x71032d1e, 98 },
    { 0xe1032d1f, 33 },
    { 0, 0 },
    { 0xb0832d21, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0532d28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f32d2b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80632d32, 78 },
    { 0x79432d33, 43 },
    { 0xde332d34, 61 },
    { 0x6fb32d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6832d39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa32d3d, 81 },
    { 0xb7532d3e, 98 },
    { 0, 0 },
    { 0xe4b32d40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92132d43, 33 },
    { 0x7cf32d44, 91 },
    { 0xc7232d45, 98 },
    { 0x43732d46, 46 },
    { 0, 0 },
    { 0xa632d48, 61 },
    { 0, 0 },
    { 0x1de32d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d932d4d, 33 },
    { 0xec932d4e, 61 },
    { 0x5fe32d4d, 33 },
    { 0xb7132d50, 61 },
    { 0, 0 },
    { 0x7ad32d52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15232d55, 88 },
    { 0, 0 },
    { 0xbf132d57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce32d5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c32d5f, 33 },
    { 0xf7132d60, 68 },
    { 0xe332d5f, 36 },
    { 0x44e32d60, 91 },
    { 0, 0 },
    { 0x61b32d64, 23 },
    { 0x89d32d64, 95 },
    { 0x37c32d64, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb832d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf532d6e, 61 },
    { 0x37532d6f, 90 },
    { 0xbe532d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a332d74, 99 },
    { 0xca432d75, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d32d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff32d7c, 52 },
    { 0xc4932d7c, 98 },
    { 0x9bd32d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8132d82, 81 },
    { 0, 0 },
    { 0x2fc32d84, 34 },
    { 0x11732d85, 33 },
    { 0xa6432d86, 43 },
    { 0x14732d87, 93 },
    { 0, 0 },
    { 0xf6132d89, 98 },
    { 0, 0 },
    { 0x8aa32d8b, 90 },
    { 0x3e732d8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62932d90, 55 },
    { 0x16132d91, 43 },
    { 0xbc432d92, 95 },
    { 0x91732d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2132d9c, 23 },
    { 0x69d32d9c, 76 },
    { 0xdef32d9e, 98 },
    { 0x35a32d9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54932da2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5132da8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a32dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95732db3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b32db7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5832dc2, 98 },
    { 0, 0 },
    { 0x86832dc4, 17 },
    { 0x77d32dc5, 80 },
    { 0xbf32dc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2732dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e32dd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5932dd6, 95 },
    { 0, 0 },
    { 0xfcc32dd8, 90 },
    { 0xd7732dd8, 17 },
    { 0xa5332dda, 76 },
    { 0x25a32ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10432ddf, 90 },
    { 0xe2d32de0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85232de6, 36 },
    { 0x68132de7, 98 },
    { 0xcaa32de7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6732deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52132dee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7432df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe832df5, 95 },
    { 0x37f32df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b432dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc32e01, 81 },
    { 0xf4c32e02, 90 },
    { 0, 0 },
    { 0x8bf32e04, 90 },
    { 0xcd532e05, 93 },
    { 0xeb532e04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e832e0f, 61 },
    { 0, 0 },
    { 0xb3632e11, 90 },
    { 0x7f232e11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25632e19, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4932e1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b832e20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43632e27, 33 },
    { 0x18732e27, 61 },
    { 0xc8f32e28, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4532e2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba932e30, 90 },
    { 0, 0 },
    { 0x8ca32e32, 76 },
    { 0x92a32e32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x81832e36, 93 },
    { 0x34632e36, 90 },
    { 0x64732e38, 23 },
    { 0x6e232e37, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22432e3c, 90 },
    { 0x35b32e3d, 43 },
    { 0x50c32e3d, 90 },
    { 0x9f032e3f, 33 },
    { 0xb3632e3e, 95 },
    { 0xdce32e3f, 48 },
    { 0, 0 },
    { 0x5e532e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11332e49, 33 },
    { 0, 0 },
    { 0xdf332e4b, 93 },
    { 0xe9c32e4c, 33 },
    { 0x1d932e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b32e52, 85 },
    { 0xde132e53, 98 },
    { 0xa9832e53, 90 },
    { 0xeed32e55, 84 },
    { 0x21032e56, 93 },
    { 0, 0 },
    { 0x9e432e58, 55 },
    { 0xcd932e59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f32e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90232e69, 61 },
    { 0, 0 },
    { 0x8c032e6b, 66 },
    { 0, 0 },
    { 0x8e32e6d, 33 },
    { 0xb6132e6e, 90 },
    { 0xcff32e6f, 81 },
    { 0x20832e6e, 93 },
    { 0xda932e70, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c32e77, 40 },
    { 0x49232e77, 33 },
    { 0, 0 },
    { 0xd5732e7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7732e7e, 33 },
    { 0x23832e7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77232e82, 55 },
    { 0x68932e83, 76 },
    { 0x1ab32e84, 98 },
    { 0x96a32e83, 90 },
    { 0xb8932e86, 66 },
    { 0x7632e84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e32e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6532e91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36032e95, 90 },
    { 0x2632e96, 52 },
    { 0x29b32e97, 90 },
    { 0xef732e98, 33 },
    { 0, 0 },
    { 0x86432e9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a32e9e, 43 },
    { 0x96632e9e, 43 },
    { 0x39e32e9f, 91 },
    { 0x1bb32ea1, 23 },
    { 0xaa532ea1, 76 },
    { 0x96232ea3, 34 },
    { 0, 0 },
    { 0x6e232ea5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c32eaa, 33 },
    { 0x5d032eab, 34 },
    { 0x5cc32eac, 43 },
    { 0xe8d32ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab332eb3, 33 },
    { 0x39d32eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd32eb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5132ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1732ebf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff32ec5, 91 },
    { 0xa5932ec5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47832ecc, 90 },
    { 0x7fa32ecc, 98 },
    { 0x8ea32ece, 33 },
    { 0xa0e32ecf, 90 },
    { 0xbd632ecd, 61 },
    { 0xe7e32ed1, 23 },
    { 0x93a32ed2, 95 },
    { 0x64e32ecd, 78 },
    { 0x88e32ed4, 33 },
    { 0xe2032ed5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x11932ed8, 90 },
    { 0xf4932ed9, 23 },
    { 0x50032ed9, 33 },
    { 0x31a32ed8, 46 },
    { 0xa8a32edc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead32ee0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4132ee7, 95 },
    { 0x6af32ee8, 90 },
    { 0x7f032ee9, 33 },
    { 0, 0 },
    { 0x5a632eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8132ef5, 52 },
    { 0x81b32ef5, 90 },
    { 0, 0 },
    { 0x27a32ef8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47632efb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d32efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa632f02, 100 },
    { 0xca132f03, 93 },
    { 0xb3032f04, 81 },
    { 0x25832f05, 77 },
    { 0x8bf32f06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c32f09, 36 },
    { 0, 0 },
    { 0x12032f0b, 76 },
    { 0x9e832f0c, 90 },
    { 0x47832f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc432f11, 61 },
    { 0x4c832f12, 46 },
    { 0x2a932f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba332f24, 90 },
    { 0xee532f25, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f032f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46832f2d, 93 },
    { 0xd1132f2e, 23 },
    { 0xce32f2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6932f35, 33 },
    { 0x33532f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed32f3b, 23 },
    { 0xc6f32f3c, 61 },
    { 0x59b32f3b, 90 },
    { 0xa8332f3b, 95 },
    { 0, 0 },
    { 0x5a332f40, 99 },
    { 0, 0 },
    { 0xd0332f42, 61 },
    { 0x53d32f42, 61 },
    { 0, 0 },
    { 0x88632f45, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e632f4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e32f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7032f57, 98 },
    { 0x6e232f58, 66 },
    { 0, 0 },
    { 0xef432f5a, 95 },
    { 0x25832f5b, 42 },
    { 0, 0 },
    { 0xbe532f5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c32f66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a32f6c, 43 },
    { 0x21f32f6d, 76 },
    { 0xf3b32f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e32f7a, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58632f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc432f81, 87 },
    { 0xfbe32f82, 98 },
    { 0, 0 },
    { 0x82332f84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97332f88, 33 },
    { 0xccc32f89, 76 },
    { 0, 0 },
    { 0x20932f8b, 90 },
    { 0, 0 },
    { 0x2a232f8d, 91 },
    { 0, 0 },
    { 0xbd932f8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78532f96, 90 },
    { 0, 0 },
    { 0xe8332f98, 23 },
    { 0xba132f99, 33 },
    { 0, 0 },
    { 0x59a32f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c032fa2, 90 },
    { 0xe0932fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53432fa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75832fac, 98 },
    { 0xc9f32fac, 61 },
    { 0xa6f32fae, 81 },
    { 0xf6732faf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d32fb3, 67 },
    { 0x16b32fb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f32fc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc9, 62 },
    { 0xbc432fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b32fce, 90 },
    { 0x60d32fcf, 77 },
    { 0x9ff32fcf, 95 },
    { 0x50a32fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb32fd4, 98 },
    { 0, 0 },
    { 0x9f432fd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92032fe0, 53 },
    { 0, 0 },
    { 0x92c32fe2, 98 },
    { 0, 0 },
    { 0x93932fe4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5932fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b32fed, 61 },
    { 0x28532fee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b32ff3, 98 },
    { 0, 0 },
    { 0x65632ff5, 23 },
    { 0x9b32ff6, 55 },
    { 0, 0 },
    { 0x59332ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67632ffb, 90 },
    { 0x3ff32ffc, 40 },
    { 0x61432ffd, 76 },
    { 0x6ac32ffe, 33 },
    { 0x75932fff, 33 },
    { 0xd5032ffb, 98 },
    { 0, 0 },
    { 0x49633002, 55 },
    { 0xac233003, 66 },
    { 0, 0 },
    { 0x83233005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac633008, 93 },
    { 0, 0 },
    { 0xc653300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3300e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1833016, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993301c, 55 },
    { 0, 0 },
    { 0xcc23301e, 55 },
    { 0, 0 },
    { 0x5cc33020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e433025, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53302e, 76 },
    { 0x4103302e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb33036, 87 },
    { 0x34e33037, 34 },
    { 0, 0 },
    { 0x90f33039, 80 },
    { 0x4763303a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc583304d, 66 },
    { 0x2293304e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d733052, 17 },
    { 0x9f033053, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70133056, 84 },
    { 0x2f033057, 90 },
    { 0xa3333058, 98 },
    { 0, 0 },
    { 0x72f3305a, 95 },
    { 0xae3305b, 99 },
    { 0, 0 },
    { 0x3ac3305d, 23 },
    { 0, 0 },
    { 0x5a3305f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a33075, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca33079, 61 },
    { 0xe9533079, 61 },
    { 0xc9d3307b, 61 },
    { 0x8ed3307a, 66 },
    { 0xf123307d, 55 },
    { 0x8853307d, 33 },
    { 0x2da3307f, 61 },
    { 0, 0 },
    { 0x75c33081, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4323308c, 17 },
    { 0, 0 },
    { 0xb363308e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4793309b, 90 },
    { 0x3493309c, 95 },
    { 0xce83309b, 33 },
    { 0, 0 },
    { 0x7483309f, 100 },
    { 0xa61330a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e330a5, 33 },
    { 0, 0 },
    { 0x829330a7, 77 },
    { 0x67b330a7, 33 },
    { 0x360330a9, 98 },
    { 0, 0 },
    { 0x6c7330ab, 88 },
    { 0xbba330ac, 90 },
    { 0xd1a330ad, 90 },
    { 0x2cf330ae, 62 },
    { 0xed4330ae, 90 },
    { 0xf12330ad, 61 },
    { 0x163330ab, 76 },
    { 0x83a330b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e330b5, 43 },
    { 0x24c330b6, 33 },
    { 0x9d1330b7, 55 },
    { 0x1e0330b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c330c0, 95 },
    { 0x9bd330c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3330c4, 76 },
    { 0x7cd330c5, 99 },
    { 0x80e330c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791330cb, 95 },
    { 0x3c330cb, 34 },
    { 0x1be330cd, 94 },
    { 0xbf6330cc, 99 },
    { 0x296330cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35330d5, 90 },
    { 0x7a8330d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f330e0, 33 },
    { 0xb30330e1, 33 },
    { 0xe5f330e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f330e5, 61 },
    { 0x45e330e6, 90 },
    { 0xee2330e6, 91 },
    { 0x35330e6, 84 },
    { 0x8a0330e9, 87 },
    { 0x211330e9, 52 },
    { 0x163330eb, 23 },
    { 0x99c330e9, 99 },
    { 0xc00330ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6330f0, 23 },
    { 0, 0 },
    { 0xa9c330f2, 53 },
    { 0xb4a330f3, 33 },
    { 0, 0 },
    { 0xa1d330f5, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753330fa, 96 },
    { 0, 0 },
    { 0xdeb330fc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3330ff, 90 },
    { 0xf2733100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53833104, 76 },
    { 0xa3333105, 76 },
    { 0x35c33106, 55 },
    { 0xffb33105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83310a, 76 },
    { 0xbea3310b, 61 },
    { 0x1453310a, 90 },
    { 0x6513310c, 61 },
    { 0x3df3310a, 34 },
    { 0xac63310f, 98 },
    { 0, 0 },
    { 0xec533111, 43 },
    { 0xb0433112, 61 },
    { 0xb8b33113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92533118, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9893311b, 76 },
    { 0, 0 },
    { 0xd133311d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42133123, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f033127, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63312e, 68 },
    { 0x13f3312f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef33134, 75 },
    { 0xba633135, 33 },
    { 0xf5533136, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0133139, 61 },
    { 0xa5833139, 76 },
    { 0x1383313a, 58 },
    { 0x8573313b, 98 },
    { 0x83f3313d, 33 },
    { 0xace3313d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3833141, 33 },
    { 0xc433142, 55 },
    { 0xbb633143, 23 },
    { 0x63a33143, 23 },
    { 0xfeb33145, 33 },
    { 0, 0 },
    { 0x3f833147, 53 },
    { 0x97133148, 90 },
    { 0xd2533149, 64 },
    { 0, 0 },
    { 0xff3314b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf23314f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33153, 77 },
    { 0x53733153, 91 },
    { 0x7ac33155, 61 },
    { 0x5533156, 33 },
    { 0, 0 },
    { 0xac533158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf3315b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb683315e, 23 },
    { 0x66f3315f, 90 },
    { 0, 0 },
    { 0x4ab33161, 98 },
    { 0x8a033162, 95 },
    { 0x6ae33162, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce733167, 90 },
    { 0x94b33168, 98 },
    { 0x1c533169, 98 },
    { 0xa1e33168, 61 },
    { 0, 0 },
    { 0xf5b3316c, 98 },
    { 0x8133316c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7633171, 90 },
    { 0, 0 },
    { 0x45533173, 76 },
    { 0xc3e33173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e933177, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3317f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de33187, 97 },
    { 0x84c33188, 66 },
    { 0xdf133189, 95 },
    { 0, 0 },
    { 0x7d83318b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f33195, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c233199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a73319e, 23 },
    { 0xca13319f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728331a4, 49 },
    { 0x93f331a5, 90 },
    { 0x807331a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52331a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40331ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x264331b0, 33 },
    { 0, 0 },
    { 0x3a5331b2, 97 },
    { 0, 0 },
    { 0xe5e331b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7331b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34331bb, 61 },
    { 0x19331bb, 67 },
    { 0, 0 },
    { 0xee3331be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430331c2, 61 },
    { 0x7f8331c2, 76 },
    { 0x3d8331c2, 98 },
    { 0xa3a331c2, 87 },
    { 0x8d331c6, 43 },
    { 0xdc5331c4, 100 },
    { 0xa64331c8, 46 },
    { 0x64e331c8, 98 },
    { 0x8aa331c8, 34 },
    { 0, 0 },
    { 0xa7d331cc, 93 },
    { 0, 0 },
    { 0xfed331ce, 98 },
    { 0, 0 },
    { 0x5dd331d0, 23 },
    { 0, 0 },
    { 0x27f331d2, 38 },
    { 0, 0 },
    { 0x95d331d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856331d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c331dc, 90 },
    { 0x315331dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7331e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4331e7, 91 },
    { 0, 0 },
    { 0xe14331e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983331ed, 56 },
    { 0x83c331ee, 43 },
    { 0x4f1331ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80331f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9331fa, 88 },
    { 0, 0 },
    { 0x997331fc, 98 },
    { 0, 0 },
    { 0xc32331fe, 87 },
    { 0xc26331ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb733205, 68 },
    { 0, 0 },
    { 0xab333207, 90 },
    { 0x29033208, 87 },
    { 0xce233209, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8633320d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c233212, 40 },
    { 0x1de33213, 90 },
    { 0xaf133212, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b233218, 61 },
    { 0xab233219, 85 },
    { 0x4c133218, 100 },
    { 0x8de3321b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a53321f, 87 },
    { 0x13a33220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14733223, 81 },
    { 0x85f33224, 99 },
    { 0x50833225, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe093322a, 90 },
    { 0x9223322a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3322e, 33 },
    { 0x3c23322f, 43 },
    { 0x4e233230, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33233, 33 },
    { 0, 0 },
    { 0x7bb33235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8273323c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b33240, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a33245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4133324b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3324e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db33251, 95 },
    { 0xe9333251, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23325a, 43 },
    { 0x4413325a, 38 },
    { 0xc613325a, 91 },
    { 0xbb53325d, 95 },
    { 0, 0 },
    { 0x9a63325f, 61 },
    { 0, 0 },
    { 0x75533261, 90 },
    { 0, 0 },
    { 0x9e333263, 93 },
    { 0x6d433263, 76 },
    { 0x2f633263, 55 },
    { 0, 0 },
    { 0x9e933267, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf583326b, 23 },
    { 0x5eb3326c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3326f, 33 },
    { 0xcfe33270, 99 },
    { 0xc8233271, 52 },
    { 0x3053326f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19833275, 81 },
    { 0x40d33275, 61 },
    { 0x6cd33277, 95 },
    { 0xbd433276, 90 },
    { 0, 0 },
    { 0x3113327a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5673327e, 98 },
    { 0x8813327e, 88 },
    { 0x9f93327e, 55 },
    { 0x64133281, 33 },
    { 0x68c33281, 55 },
    { 0, 0 },
    { 0xe4d33284, 33 },
    { 0, 0 },
    { 0xab733286, 91 },
    { 0, 0 },
    { 0x5cf33288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403328d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f933294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33298, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa73329b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40332a1, 98 },
    { 0xba1332a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a332aa, 95 },
    { 0x4f9332aa, 98 },
    { 0x5a9332ac, 91 },
    { 0, 0 },
    { 0xd09332ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d332bd, 98 },
    { 0x8a4332bd, 66 },
    { 0, 0 },
    { 0xa7a332c0, 98 },
    { 0x58e332c0, 90 },
    { 0xe7c332c1, 64 },
    { 0x677332c3, 43 },
    { 0xbca332c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170332ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f332da, 46 },
    { 0x653332db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39332e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de332ed, 55 },
    { 0x6b0332ee, 90 },
    { 0, 0 },
    { 0x8e9332f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4332f5, 23 },
    { 0xf59332f6, 95 },
    { 0x43c332f7, 77 },
    { 0x53d332f6, 91 },
    { 0x425332f9, 66 },
    { 0xf49332fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5833300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b33307, 100 },
    { 0, 0 },
    { 0xf1c33309, 61 },
    { 0xcf3330a, 90 },
    { 0x513330b, 90 },
    { 0x70e3330c, 99 },
    { 0x183330d, 90 },
    { 0x87a3330e, 65 },
    { 0x4103330f, 61 },
    { 0, 0 },
    { 0xe4a33311, 61 },
    { 0, 0 },
    { 0xb1f33313, 91 },
    { 0xc9a33314, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19333318, 65 },
    { 0xee333318, 23 },
    { 0xd213331a, 61 },
    { 0xd833331b, 87 },
    { 0x5a533318, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba3331f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef133323, 98 },
    { 0x26a33323, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd3332b, 98 },
    { 0x6e63332b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c433331, 61 },
    { 0x79e33332, 90 },
    { 0, 0 },
    { 0xf1833334, 76 },
    { 0, 0 },
    { 0x46533336, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x29233339, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093333c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3643333f, 61 },
    { 0xd1733340, 76 },
    { 0x4ee33341, 90 },
    { 0x56a33342, 52 },
    { 0x2d033343, 56 },
    { 0, 0 },
    { 0xaa533345, 76 },
    { 0xcc33346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe3334e, 55 },
    { 0x3df3334f, 67 },
    { 0x4833334e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9933355, 68 },
    { 0x10833356, 33 },
    { 0, 0 },
    { 0x2ae33358, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7453335b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f3335e, 36 },
    { 0x3943335f, 23 },
    { 0, 0 },
    { 0x11433361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e33365, 33 },
    { 0x8b533365, 95 },
    { 0xe8c33366, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a3336d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3123337a, 99 },
    { 0, 0 },
    { 0xd343337c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3333382, 90 },
    { 0x37633382, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6033386, 23 },
    { 0, 0 },
    { 0x7be33388, 33 },
    { 0, 0 },
    { 0x2ae3338a, 67 },
    { 0, 0 },
    { 0x8783338c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e33396, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee3339a, 100 },
    { 0, 0 },
    { 0x923339c, 61 },
    { 0x34b3339d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b333a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db333a5, 23 },
    { 0x88333a6, 33 },
    { 0xe6333a7, 76 },
    { 0x611333a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e333ab, 61 },
    { 0xfe4333ab, 33 },
    { 0x814333ab, 23 },
    { 0xbe5333ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x584333b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218333b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x419333b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197333bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9333c1, 55 },
    { 0x443333c2, 61 },
    { 0x638333c3, 90 },
    { 0xef1333c2, 99 },
    { 0xaab333c2, 96 },
    { 0xb6d333c6, 55 },
    { 0xe34333c7, 93 },
    { 0x76a333c7, 34 },
    { 0xdbc333c8, 55 },
    { 0xe40333c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7333ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3333d4, 77 },
    { 0, 0 },
    { 0x326333d6, 33 },
    { 0x623333d6, 98 },
    { 0xecd333d6, 95 },
    { 0x93f333d8, 23 },
    { 0, 0 },
    { 0xfa2333db, 23 },
    { 0x2b2333db, 90 },
    { 0x377333dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45333e0, 23 },
    { 0xea1333e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333e6, 55 },
    { 0x613333e6, 76 },
    { 0xcca333e7, 98 },
    { 0xca0333e9, 61 },
    { 0x1b4333e6, 48 },
    { 0, 0 },
    { 0xc58333ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d333ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea333f2, 80 },
    { 0x3d6333f2, 61 },
    { 0, 0 },
    { 0x86b333f5, 33 },
    { 0xb2a333f6, 43 },
    { 0xd38333f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1333fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b333ff, 100 },
    { 0, 0 },
    { 0x13733401, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x43433404, 66 },
    { 0, 0 },
    { 0x77c33406, 23 },
    { 0, 0 },
    { 0xf0d33408, 98 },
    { 0, 0 },
    { 0xeeb3340a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a3340d, 23 },
    { 0x89f3340e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x40333411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff633414, 46 },
    { 0xa5a33414, 23 },
    { 0x48c33416, 99 },
    { 0x11d33416, 33 },
    { 0, 0 },
    { 0xe8933419, 90 },
    { 0x5d73341a, 33 },
    { 0x4f53341b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb373341e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba33423, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2893342c, 90 },
    { 0x2e63342d, 99 },
    { 0xdb23342c, 33 },
    { 0xf593342f, 95 },
    { 0x5973342c, 95 },
    { 0xed233431, 99 },
    { 0xd9a33431, 65 },
    { 0xf1133432, 90 },
    { 0, 0 },
    { 0x8a533435, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613343b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533343f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c33442, 61 },
    { 0, 0 },
    { 0xb7e33444, 46 },
    { 0, 0 },
    { 0xf9a33446, 76 },
    { 0xd9033447, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c43344e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc33451, 90 },
    { 0xf9a33452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d033459, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803345e, 61 },
    { 0, 0 },
    { 0xbec33460, 78 },
    { 0x96133460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a33464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f633468, 90 },
    { 0x1c033469, 55 },
    { 0xbde33468, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603346d, 90 },
    { 0x2233346e, 93 },
    { 0x7773346e, 43 },
    { 0x8963346e, 43 },
    { 0xb9933471, 33 },
    { 0x38b33472, 23 },
    { 0x24233472, 67 },
    { 0x45e33470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65533478, 23 },
    { 0xa7333479, 95 },
    { 0x6ad3347a, 34 },
    { 0xf8a3347b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb153347f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0033482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada33486, 33 },
    { 0, 0 },
    { 0xff33488, 76 },
    { 0, 0 },
    { 0x32e3348a, 78 },
    { 0x4ae3348a, 61 },
    { 0x9793348c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d33493, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x78433496, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5733499, 33 },
    { 0, 0 },
    { 0xf273349b, 95 },
    { 0, 0 },
    { 0x8003349d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x550334a0, 90 },
    { 0, 0 },
    { 0x7ec334a2, 52 },
    { 0x738334a3, 76 },
    { 0x5bb334a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a334ab, 61 },
    { 0x993334ac, 76 },
    { 0xc8c334ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x837334b0, 34 },
    { 0xd1f334b0, 98 },
    { 0xfbf334b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e334b7, 91 },
    { 0, 0 },
    { 0x307334b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3334be, 90 },
    { 0x3e2334bf, 95 },
    { 0xe41334c0, 33 },
    { 0xbf7334bf, 90 },
    { 0x9e7334be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d334c7, 98 },
    { 0, 0 },
    { 0xa3d334c9, 23 },
    { 0xa9c334ca, 61 },
    { 0x1f5334cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f334cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46334d3, 93 },
    { 0, 0 },
    { 0xec3334d5, 76 },
    { 0, 0 },
    { 0xc9b334d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8334df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1334e2, 78 },
    { 0, 0 },
    { 0xb1a334e4, 91 },
    { 0x5e334e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88334e8, 75 },
    { 0, 0 },
    { 0x932334ea, 95 },
    { 0x551334eb, 66 },
    { 0, 0 },
    { 0x5b2334ed, 76 },
    { 0x3cc334ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04334f4, 34 },
    { 0, 0 },
    { 0xe65334f6, 93 },
    { 0, 0 },
    { 0xd20334f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835334fc, 33 },
    { 0, 0 },
    { 0xb1f334fe, 35 },
    { 0x54334ff, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x44433502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a033505, 33 },
    { 0, 0 },
    { 0x91b33507, 90 },
    { 0x63733508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3350c, 69 },
    { 0x723350c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd33516, 90 },
    { 0, 0 },
    { 0xa6133518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa453351b, 55 },
    { 0, 0 },
    { 0xeef3351d, 46 },
    { 0xdde3351d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88333521, 55 },
    { 0, 0 },
    { 0x3b133523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda33526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c3352d, 59 },
    { 0x8a3352e, 95 },
    { 0xabd3352f, 52 },
    { 0x9f733530, 87 },
    { 0x6c533531, 100 },
    { 0x35333530, 98 },
    { 0x9c133533, 23 },
    { 0x62f33531, 23 },
    { 0xc8533535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab733539, 61 },
    { 0xd513353a, 34 },
    { 0xb7d3353b, 43 },
    { 0xd203353c, 85 },
    { 0x613353d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6833540, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1133545, 78 },
    { 0, 0 },
    { 0xfa833547, 61 },
    { 0x8333547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3354e, 81 },
    { 0xca33354f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c33555, 33 },
    { 0x28233556, 33 },
    { 0x48233557, 81 },
    { 0x59233558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f23355b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde3355e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a3356b, 55 },
    { 0x3313356b, 100 },
    { 0, 0 },
    { 0xfea3356e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70033576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f633579, 17 },
    { 0x31933579, 99 },
    { 0x48933579, 98 },
    { 0, 0 },
    { 0xec83357d, 78 },
    { 0xc613357e, 61 },
    { 0xf573357d, 55 },
    { 0xa803357d, 33 },
    { 0x5b933581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9333587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843358c, 95 },
    { 0x4933358d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85033591, 95 },
    { 0xf1333591, 65 },
    { 0, 0 },
    { 0x6ed33594, 34 },
    { 0x43e33595, 85 },
    { 0x53c33595, 33 },
    { 0xbf633595, 98 },
    { 0x4fa33598, 34 },
    { 0x41333594, 33 },
    { 0xa8a33596, 90 },
    { 0, 0 },
    { 0x5a73359c, 61 },
    { 0x9cf3359c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194335a1, 98 },
    { 0xafb335a2, 33 },
    { 0, 0 },
    { 0x38c335a4, 61 },
    { 0x174335a5, 43 },
    { 0x839335a6, 46 },
    { 0x4d2335a7, 61 },
    { 0xbc7335a7, 33 },
    { 0x404335a4, 98 },
    { 0xf75335a5, 43 },
    { 0x917335ab, 33 },
    { 0, 0 },
    { 0x3335ad, 95 },
    { 0, 0 },
    { 0x65f335af, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4335b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97335b8, 95 },
    { 0x8df335b8, 39 },
    { 0x80d335ba, 33 },
    { 0xe20335b8, 61 },
    { 0xf05335bc, 33 },
    { 0, 0 },
    { 0xf19335be, 55 },
    { 0, 0 },
    { 0x9d6335c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5335c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76335cd, 23 },
    { 0x947335ce, 33 },
    { 0, 0 },
    { 0xc20335d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26335d3, 90 },
    { 0, 0 },
    { 0x22f335d5, 95 },
    { 0x98a335d6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb335da, 23 },
    { 0x303335db, 90 },
    { 0xa4b335dc, 67 },
    { 0xd6f335dd, 99 },
    { 0, 0 },
    { 0xd2b335df, 23 },
    { 0x216335df, 61 },
    { 0, 0 },
    { 0xf8e335e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0335e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978335eb, 17 },
    { 0xcc9335ec, 93 },
    { 0xa8c335ed, 95 },
    { 0xab1335ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d335f1, 33 },
    { 0, 0 },
    { 0x12b335f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x952335f6, 33 },
    { 0x24f335f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3335fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73933601, 76 },
    { 0xd733601, 90 },
    { 0x89b33603, 95 },
    { 0, 0 },
    { 0x68833605, 33 },
    { 0x87733606, 91 },
    { 0xe6d33607, 98 },
    { 0xce133606, 90 },
    { 0, 0 },
    { 0x50b3360a, 33 },
    { 0x8473360b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66133613, 34 },
    { 0x60f33613, 43 },
    { 0x19c33615, 46 },
    { 0x25433616, 43 },
    { 0, 0 },
    { 0x69f33618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc73361b, 85 },
    { 0x8193361b, 33 },
    { 0x2ea3361d, 33 },
    { 0x3943361e, 90 },
    { 0x86a3361f, 95 },
    { 0x72b33620, 78 },
    { 0xdcf3361c, 23 },
    { 0x6a23361c, 23 },
    { 0, 0 },
    { 0xb6833624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a33628, 23 },
    { 0xf933628, 95 },
    { 0xc163362a, 88 },
    { 0xdcc3362a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33636, 95 },
    { 0, 0 },
    { 0x5ab33638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4103363b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0433642, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc33645, 55 },
    { 0xd333646, 98 },
    { 0, 0 },
    { 0xbab33648, 64 },
    { 0x37c33649, 23 },
    { 0x73033648, 81 },
    { 0, 0 },
    { 0x5f73364c, 98 },
    { 0xfd23364d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9233650, 66 },
    { 0x20d33651, 85 },
    { 0xa3633650, 95 },
    { 0x6d533652, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f933657, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e33365b, 98 },
    { 0, 0 },
    { 0x6e23365d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60133669, 95 },
    { 0x11e3366a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9063366e, 93 },
    { 0x6d53366f, 61 },
    { 0xf8b3366f, 46 },
    { 0xba73366f, 61 },
    { 0, 0 },
    { 0xde433673, 33 },
    { 0x2f533674, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e13367a, 80 },
    { 0x4f53367b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed3367e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e33685, 98 },
    { 0xbae33686, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab533689, 84 },
    { 0x45b33689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb23368d, 33 },
    { 0xee63368d, 81 },
    { 0x9e53368d, 33 },
    { 0, 0 },
    { 0x2f533691, 95 },
    { 0, 0 },
    { 0x20633693, 61 },
    { 0xf333693, 99 },
    { 0xe8933695, 55 },
    { 0x7b633695, 95 },
    { 0x33733696, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13336a0, 23 },
    { 0x3fd336a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9336a5, 92 },
    { 0xdb3336a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b336a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65336ad, 98 },
    { 0xcaf336ad, 43 },
    { 0, 0 },
    { 0x1af336b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b336b3, 23 },
    { 0xcc8336b4, 34 },
    { 0x5d1336b4, 55 },
    { 0xff9336b6, 91 },
    { 0x8ac336b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a336c3, 95 },
    { 0x8d7336c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea336c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x856336cc, 23 },
    { 0x27f336cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e336d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781336db, 81 },
    { 0, 0 },
    { 0x69a336dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141336e1, 55 },
    { 0, 0 },
    { 0x3b1336e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae336e8, 95 },
    { 0xf0f336e9, 66 },
    { 0x7aa336ea, 81 },
    { 0x9bb336eb, 90 },
    { 0, 0 },
    { 0x1a2336ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a336f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78336fc, 87 },
    { 0x4c9336fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c33701, 90 },
    { 0, 0 },
    { 0xf3233703, 98 },
    { 0xce333704, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af33707, 95 },
    { 0x7c933708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d3370c, 43 },
    { 0x4ca3370d, 91 },
    { 0xe723370e, 61 },
    { 0x6c13370d, 93 },
    { 0, 0 },
    { 0x33933711, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af33718, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c3371b, 66 },
    { 0, 0 },
    { 0xbd53371d, 99 },
    { 0, 0 },
    { 0x8c43371f, 46 },
    { 0xac933720, 95 },
    { 0xcf033721, 88 },
    { 0xbb633722, 23 },
    { 0xd8e33720, 33 },
    { 0x8c733722, 33 },
    { 0xbeb33724, 52 },
    { 0x32233726, 90 },
    { 0, 0 },
    { 0x68b33728, 52 },
    { 0x82d33728, 34 },
    { 0x2e93372a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf3372d, 46 },
    { 0, 0 },
    { 0x273372f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3373a, 43 },
    { 0x2a53373a, 43 },
    { 0x9b53373c, 8 },
    { 0, 0 },
    { 0x2a43373e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94933741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae33744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d73374a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33374d, 61 },
    { 0x4193374d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a33754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8933757, 98 },
    { 0xfae33757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9823375e, 33 },
    { 0xae83375f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e33763, 23 },
    { 0x60633763, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb33767, 61 },
    { 0xd6a33768, 33 },
    { 0xb9133769, 78 },
    { 0xa383376a, 98 },
    { 0x7913376a, 95 },
    { 0xd93376b, 33 },
    { 0, 0 },
    { 0xb833376e, 95 },
    { 0x1053376f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d33775, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc013377a, 34 },
    { 0x52e3377a, 55 },
    { 0, 0 },
    { 0xf183377d, 46 },
    { 0xb4b3377d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b33781, 76 },
    { 0x94933781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15433788, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea73378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6133791, 90 },
    { 0, 0 },
    { 0xf9133793, 61 },
    { 0, 0 },
    { 0x2a533795, 23 },
    { 0xf2e33795, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e733799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c337a6, 88 },
    { 0x6ed337a6, 67 },
    { 0x7df337a8, 98 },
    { 0xdfd337a9, 90 },
    { 0x806337aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470337b1, 90 },
    { 0, 0 },
    { 0xbe4337b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40337b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42337bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee337c4, 43 },
    { 0, 0 },
    { 0x596337c6, 90 },
    { 0xe0b337c7, 76 },
    { 0xd06337c6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362337d0, 23 },
    { 0, 0 },
    { 0xc3b337d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c337d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef337dc, 40 },
    { 0xe5e337dc, 95 },
    { 0x1f8337dd, 34 },
    { 0x781337df, 98 },
    { 0x5b337e0, 93 },
    { 0x27e337df, 23 },
    { 0x750337e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d337e5, 23 },
    { 0x88d337e5, 46 },
    { 0x1f8337e7, 23 },
    { 0xfba337e8, 95 },
    { 0x45e337e6, 96 },
    { 0, 0 },
    { 0xead337eb, 33 },
    { 0xf00337ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b337ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250337f3, 23 },
    { 0, 0 },
    { 0xdb3337f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c337ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f33802, 61 },
    { 0x5d333802, 34 },
    { 0, 0 },
    { 0xfad33805, 93 },
    { 0, 0 },
    { 0xc3033807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf493380c, 87 },
    { 0x4223380d, 33 },
    { 0x9af3380d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c33381b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b3381f, 66 },
    { 0x69033820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c33824, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc33827, 90 },
    { 0xc4a33828, 43 },
    { 0xd6533829, 23 },
    { 0, 0 },
    { 0x74f3382b, 61 },
    { 0, 0 },
    { 0xf383382d, 95 },
    { 0, 0 },
    { 0x9b03382f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2293383a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3383d, 90 },
    { 0x6e53383d, 23 },
    { 0x12f3383f, 46 },
    { 0xcc633840, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c33843, 61 },
    { 0xa6e33844, 99 },
    { 0x36f33844, 33 },
    { 0, 0 },
    { 0xe1333847, 98 },
    { 0, 0 },
    { 0x4d733849, 90 },
    { 0xfc233849, 90 },
    { 0xb3033849, 34 },
    { 0xd6f33849, 34 },
    { 0xf633384d, 68 },
    { 0x8923384b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88433855, 23 },
    { 0, 0 },
    { 0x33433857, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13385f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99433864, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f33867, 90 },
    { 0xef833867, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca3386c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8533870, 91 },
    { 0x4af33870, 90 },
    { 0, 0 },
    { 0x12c33873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0633878, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x443387b, 80 },
    { 0, 0 },
    { 0xb1e3387d, 84 },
    { 0x3813387e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf733887, 33 },
    { 0xbac33887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f33890, 23 },
    { 0, 0 },
    { 0xa0133892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f333895, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3389a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1338a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13338a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46338aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7338ad, 33 },
    { 0, 0 },
    { 0xbf8338af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11338b4, 98 },
    { 0x517338b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2338b8, 43 },
    { 0, 0 },
    { 0x88a338ba, 98 },
    { 0x51c338bb, 78 },
    { 0xfae338ba, 90 },
    { 0xa7d338bc, 23 },
    { 0, 0 },
    { 0x5d6338bf, 67 },
    { 0xf5f338c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8338c5, 61 },
    { 0xc5a338c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c338ca, 95 },
    { 0x21f338cb, 34 },
    { 0xffe338cc, 95 },
    { 0, 0 },
    { 0x284338ce, 99 },
    { 0, 0 },
    { 0xb19338d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14338d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d338d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f338db, 95 },
    { 0xfa2338dc, 95 },
    { 0xe51338db, 34 },
    { 0x2ac338dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a338e3, 91 },
    { 0x99338e3, 33 },
    { 0x369338e5, 90 },
    { 0x106338e4, 52 },
    { 0xfc2338e5, 33 },
    { 0x13338e5, 67 },
    { 0x546338e9, 46 },
    { 0xcea338e9, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34338ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201338f2, 61 },
    { 0, 0 },
    { 0x6ba338f4, 95 },
    { 0xcd1338f4, 98 },
    { 0xe6a338f6, 95 },
    { 0xa5d338f6, 33 },
    { 0, 0 },
    { 0xf6b338f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126338ff, 55 },
    { 0xff233900, 43 },
    { 0xd7b33900, 91 },
    { 0x74333902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe633908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3390d, 83 },
    { 0x4803390e, 23 },
    { 0x5343390f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33915, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd833919, 43 },
    { 0, 0 },
    { 0x64d3391b, 76 },
    { 0xffa3391c, 91 },
    { 0, 0 },
    { 0x5b03391e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533921, 90 },
    { 0, 0 },
    { 0x43e33923, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4333926, 95 },
    { 0xa7433926, 33 },
    { 0xf1033928, 66 },
    { 0xdd033929, 99 },
    { 0xb073392a, 100 },
    { 0, 0 },
    { 0x8613392c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad033931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b33934, 95 },
    { 0x5aa33934, 23 },
    { 0, 0 },
    { 0xd9733937, 46 },
    { 0xa6b33938, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53393f, 97 },
    { 0x6f833940, 99 },
    { 0x8cc33941, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12333946, 43 },
    { 0, 0 },
    { 0xe0033948, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf3394b, 33 },
    { 0, 0 },
    { 0x3f3394d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7833951, 43 },
    { 0, 0 },
    { 0x50a33953, 76 },
    { 0x8d433953, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53133957, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db3395c, 98 },
    { 0x583395d, 98 },
    { 0xff3395c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94533964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c633967, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a3396b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3396f, 33 },
    { 0x1c733970, 75 },
    { 0x9d033971, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c833974, 46 },
    { 0, 0 },
    { 0x2fe33976, 34 },
    { 0xd3433976, 8 },
    { 0, 0 },
    { 0x55c33979, 90 },
    { 0, 0 },
    { 0xd6c3397b, 55 },
    { 0xd923397b, 90 },
    { 0xed73397c, 17 },
    { 0x4983397e, 98 },
    { 0, 0 },
    { 0xfff33980, 77 },
    { 0x89c33980, 98 },
    { 0xedb33982, 23 },
    { 0, 0 },
    { 0x84233984, 67 },
    { 0x61833985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22333988, 90 },
    { 0, 0 },
    { 0xfde3398a, 66 },
    { 0xdaf3398b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a3398f, 35 },
    { 0x1c53398f, 98 },
    { 0xabc33990, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd33994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82633999, 98 },
    { 0xe6033999, 66 },
    { 0xcaf3399b, 33 },
    { 0x7b93399c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace339a2, 61 },
    { 0xa35339a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x868339a6, 61 },
    { 0, 0 },
    { 0x793339a8, 55 },
    { 0x858339a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319339af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3339b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c339b5, 98 },
    { 0xb07339b6, 46 },
    { 0x147339b7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90339ba, 78 },
    { 0, 0 },
    { 0x5a9339bc, 95 },
    { 0, 0 },
    { 0xd43339be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7339c1, 76 },
    { 0, 0 },
    { 0xc5b339c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739339ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb339d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0339d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c339dd, 98 },
    { 0xd39339de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a339e2, 95 },
    { 0xc07339e3, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f339e6, 95 },
    { 0x46e339e7, 34 },
    { 0x829339e8, 69 },
    { 0x721339e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2339ee, 95 },
    { 0, 0 },
    { 0x50339f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae339f4, 87 },
    { 0x599339f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f339f9, 98 },
    { 0, 0 },
    { 0x810339fb, 23 },
    { 0xf83339fb, 66 },
    { 0x9a8339fb, 90 },
    { 0xec3339fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2933a05, 90 },
    { 0xb3633a06, 97 },
    { 0, 0 },
    { 0x82a33a08, 91 },
    { 0, 0 },
    { 0xb3633a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99733a0f, 23 },
    { 0xa3f33a10, 84 },
    { 0xc3e33a10, 99 },
    { 0x19233a12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc33a1b, 61 },
    { 0x54c33a1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f633a21, 90 },
    { 0x77a33a22, 78 },
    { 0x87433a22, 52 },
    { 0, 0 },
    { 0x6e433a25, 90 },
    { 0xf1f33a26, 61 },
    { 0x85433a27, 34 },
    { 0x64733a26, 61 },
    { 0, 0 },
    { 0x4fc33a2a, 91 },
    { 0xfb633a2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f533a34, 66 },
    { 0, 0 },
    { 0x36a33a36, 90 },
    { 0x21733a37, 98 },
    { 0xba533a37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a33a3b, 33 },
    { 0xafc33a3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b33a3f, 33 },
    { 0x93933a40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39333a45, 33 },
    { 0x18733a46, 33 },
    { 0x54e33a47, 33 },
    { 0, 0 },
    { 0xc6d33a49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d233a4c, 66 },
    { 0xba733a4d, 23 },
    { 0x25a33a4c, 33 },
    { 0x7ea33a4e, 52 },
    { 0x9b033a4e, 75 },
    { 0x91a33a51, 61 },
    { 0, 0 },
    { 0xd5033a53, 55 },
    { 0, 0 },
    { 0x59733a55, 98 },
    { 0x6c33a55, 90 },
    { 0xfd233a56, 33 },
    { 0, 0 },
    { 0xee333a59, 90 },
    { 0xeb933a5a, 93 },
    { 0xcd433a5a, 33 },
    { 0x4a233a5c, 55 },
    { 0xeac33a59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb733a60, 23 },
    { 0x25f33a60, 95 },
    { 0x5a133a62, 98 },
    { 0x9e633a63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69633a69, 98 },
    { 0xe1b33a6a, 98 },
    { 0, 0 },
    { 0xc9533a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5933a6f, 33 },
    { 0xe7833a70, 43 },
    { 0, 0 },
    { 0x7133a72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef33a77, 76 },
    { 0x43533a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c33a7b, 95 },
    { 0, 0 },
    { 0x8be33a7d, 69 },
    { 0x18f33a7e, 34 },
    { 0x40233a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd33a83, 69 },
    { 0xe3233a84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf633a8a, 17 },
    { 0xdad33a8b, 52 },
    { 0xcf533a8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5733a91, 95 },
    { 0x4a633a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea133a96, 76 },
    { 0xaee33a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef933aa1, 98 },
    { 0x12d33aa1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3433aa6, 90 },
    { 0x46f33aa6, 33 },
    { 0x26233aa6, 90 },
    { 0x27833aa6, 43 },
    { 0xfa933aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa33aae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c33ab1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee33ab4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e733ab7, 61 },
    { 0xe2933ab7, 23 },
    { 0x64a33ab9, 85 },
    { 0, 0 },
    { 0xa8033abb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3333ac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a333aca, 95 },
    { 0, 0 },
    { 0x3a133acc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38533ad0, 67 },
    { 0x94d33ad1, 94 },
    { 0x48033ad2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9433adb, 23 },
    { 0, 0 },
    { 0x21f33add, 76 },
    { 0, 0 },
    { 0x3a333adf, 23 },
    { 0, 0 },
    { 0x9433ae1, 61 },
    { 0x7f733ae2, 75 },
    { 0, 0 },
    { 0x3ea33ae4, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x12933ae7, 23 },
    { 0x77633ae8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff33aeb, 85 },
    { 0x7c533aec, 17 },
    { 0xc5c33aed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f33af0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e633af7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed333aff, 87 },
    { 0xa8233b00, 67 },
    { 0x83d33b00, 66 },
    { 0x9d133b00, 23 },
    { 0x2c433b03, 95 },
    { 0x76b33b04, 24 },
    { 0xc3d33b05, 99 },
    { 0x86233b05, 33 },
    { 0, 0 },
    { 0x40733b08, 39 },
    { 0x1f33b08, 91 },
    { 0xc733b09, 34 },
    { 0xfce33b0b, 61 },
    { 0xbda33b0c, 66 },
    { 0x20a33b0d, 23 },
    { 0, 0 },
    { 0x89533b0f, 78 },
    { 0, 0 },
    { 0xcca33b11, 88 },
    { 0xa7933b11, 90 },
    { 0xcda33b13, 55 },
    { 0x8aa33b14, 91 },
    { 0xb8833b12, 95 },
    { 0x20333b13, 66 },
    { 0xad333b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74633b1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d33b1d, 33 },
    { 0, 0 },
    { 0xa6133b1f, 33 },
    { 0x1fd33b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3433b23, 95 },
    { 0x99f33b24, 78 },
    { 0, 0 },
    { 0x9bd33b26, 33 },
    { 0, 0 },
    { 0x2d733b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c633b2e, 98 },
    { 0, 0 },
    { 0x13e33b30, 90 },
    { 0x55333b30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d433b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b45, 23 },
    { 0x10833b46, 33 },
    { 0xbfe33b46, 23 },
    { 0, 0 },
    { 0xba433b49, 98 },
    { 0, 0 },
    { 0xd33b4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b33b4e, 61 },
    { 0x86f33b4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc33b53, 95 },
    { 0x67533b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d533b59, 93 },
    { 0, 0 },
    { 0xbdb33b5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71433b5f, 8 },
    { 0, 0 },
    { 0x633b61, 76 },
    { 0, 0 },
    { 0xe9233b63, 67 },
    { 0xd1433b64, 95 },
    { 0x8b733b65, 33 },
    { 0x76433b65, 33 },
    { 0x8a33b67, 98 },
    { 0x20f33b68, 95 },
    { 0, 0 },
    { 0xef933b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9833b6d, 61 },
    { 0x3aa33b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d633b73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d33b76, 85 },
    { 0, 0 },
    { 0x87833b78, 68 },
    { 0x23933b78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec133b7d, 33 },
    { 0x17f33b7e, 90 },
    { 0x81033b7e, 61 },
    { 0xffc33b7d, 33 },
    { 0, 0 },
    { 0xfce33b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58733b87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb33b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f33b93, 90 },
    { 0x68033b93, 95 },
    { 0, 0 },
    { 0x51e33b96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e33b9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d33ba0, 95 },
    { 0x17433ba1, 90 },
    { 0xb9833ba0, 93 },
    { 0xd2933ba1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d33ba9, 43 },
    { 0, 0 },
    { 0x34333bab, 87 },
    { 0, 0 },
    { 0xb3233bad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33bb0, 23 },
    { 0xc833bb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf633bbc, 34 },
    { 0, 0 },
    { 0xf0a33bbe, 87 },
    { 0x8e233bbf, 66 },
    { 0, 0 },
    { 0x40f33bc1, 95 },
    { 0, 0 },
    { 0x8ea33bc3, 46 },
    { 0xd5f33bc4, 33 },
    { 0, 0 },
    { 0x2fc33bc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b333bca, 93 },
    { 0, 0 },
    { 0x4d633bcc, 90 },
    { 0x59433bcd, 98 },
    { 0, 0 },
    { 0x95433bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66833bd5, 100 },
    { 0xa7f33bd6, 93 },
    { 0x41133bd7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca133bda, 95 },
    { 0xc1633bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46033bde, 61 },
    { 0xf833bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6633be2, 99 },
    { 0x73b33be3, 76 },
    { 0x35233be4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47433beb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9433bee, 81 },
    { 0x3733bee, 95 },
    { 0xb633bee, 90 },
    { 0, 0 },
    { 0xe9833bf2, 95 },
    { 0, 0 },
    { 0x50f33bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e33bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6833c08, 85 },
    { 0xbf633c09, 34 },
    { 0x28233c0a, 93 },
    { 0x78c33c09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98033c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a33c15, 85 },
    { 0x31533c15, 81 },
    { 0xaa933c17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9233c1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda333c24, 76 },
    { 0x1a433c25, 90 },
    { 0x5b433c26, 34 },
    { 0x46c33c27, 90 },
    { 0x85133c25, 46 },
    { 0xb9933c26, 81 },
    { 0xae633c2a, 61 },
    { 0x96733c2a, 84 },
    { 0x95933c2c, 46 },
    { 0xaae33c2c, 98 },
    { 0xc2d33c2e, 90 },
    { 0x99a33c2e, 34 },
    { 0xeb633c2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf733c33, 23 },
    { 0x55433c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3233c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb133c3c, 95 },
    { 0x1933c3c, 61 },
    { 0xe3433c3e, 33 },
    { 0x9f733c3e, 46 },
    { 0xa6933c40, 84 },
    { 0xe0833c40, 34 },
    { 0xfe433c41, 78 },
    { 0x26d33c3e, 98 },
    { 0xdb433c3e, 33 },
    { 0x9a33c45, 90 },
    { 0x9a333c3f, 99 },
    { 0, 0 },
    { 0x10b33c48, 95 },
    { 0, 0 },
    { 0x8f33c4a, 61 },
    { 0x4c633c4b, 77 },
    { 0x4833c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67533c54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b33c5a, 23 },
    { 0x7b633c5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9833c62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d33c67, 55 },
    { 0x57133c67, 98 },
    { 0x99d33c69, 33 },
    { 0x13a33c6a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1433c6e, 85 },
    { 0, 0 },
    { 0x28b33c70, 33 },
    { 0x3c833c71, 52 },
    { 0xb733c71, 76 },
    { 0xef733c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a33c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4033c80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5533c83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33c8f, 40 },
    { 0xd5533c90, 33 },
    { 0x50c33c8f, 90 },
    { 0, 0 },
    { 0xeca33c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3833c96, 90 },
    { 0, 0 },
    { 0xf4233c98, 66 },
    { 0, 0 },
    { 0x18533c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23633c9d, 98 },
    { 0xa0133c9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c33ca2, 61 },
    { 0xe5933ca2, 76 },
    { 0xdc233ca3, 99 },
    { 0xd3933ca5, 95 },
    { 0x38e33ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe33cac, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x12633caf, 84 },
    { 0xc9333cb0, 33 },
    { 0x32f33cb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b33cb5, 23 },
    { 0xd2a33cb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e333cb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac933cbe, 95 },
    { 0, 0 },
    { 0xdd833cc0, 99 },
    { 0x94133cc1, 98 },
    { 0xe4733cc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3333cc9, 34 },
    { 0xf4333cca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65833cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b33cd1, 98 },
    { 0, 0 },
    { 0x9a233cd3, 100 },
    { 0x53033cd4, 95 },
    { 0xd433cd3, 34 },
    { 0xc533cd6, 90 },
    { 0x44233cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d33cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd33cdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63633ce2, 61 },
    { 0xaae33ce2, 100 },
    { 0, 0 },
    { 0x63233ce5, 76 },
    { 0x33d33ce5, 98 },
    { 0, 0 },
    { 0x6fd33ce8, 40 },
    { 0x3a833ce9, 87 },
    { 0xbaf33cea, 61 },
    { 0xc5a33ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14233cee, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2733cf1, 100 },
    { 0, 0 },
    { 0xfdf33cf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb633cf8, 66 },
    { 0x74733cf9, 33 },
    { 0xf2733cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4133cfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a533d00, 33 },
    { 0x54533d01, 99 },
    { 0x48f33d01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0733d07, 68 },
    { 0x46c33d07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77333d0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf33d0f, 81 },
    { 0, 0 },
    { 0x84733d11, 61 },
    { 0x86f33d12, 23 },
    { 0, 0 },
    { 0x78033d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba33d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8733d1d, 33 },
    { 0x83233d1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f833d22, 33 },
    { 0, 0 },
    { 0x52a33d24, 88 },
    { 0x8e033d25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86033d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c33d30, 51 },
    { 0xf2d33d31, 76 },
    { 0, 0 },
    { 0x86b33d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee233d36, 61 },
    { 0x88533d37, 77 },
    { 0x32333d38, 46 },
    { 0x8c733d39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b33d3f, 34 },
    { 0xdc633d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91533d44, 90 },
    { 0, 0 },
    { 0x26233d46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5233d4b, 33 },
    { 0xbb133d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833d52, 52 },
    { 0, 0 },
    { 0xcad33d54, 91 },
    { 0xe8633d55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38933d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a933d5c, 93 },
    { 0xb0833d5d, 33 },
    { 0x29d33d5e, 76 },
    { 0xf933d5e, 43 },
    { 0x13433d5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc533d65, 61 },
    { 0x42d33d66, 33 },
    { 0xdcc33d66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2133d6f, 95 },
    { 0, 0 },
    { 0x5b333d71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6633d74, 61 },
    { 0xecb33d75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b33d7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49233d7f, 93 },
    { 0xcb733d80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e33d84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b33d87, 43 },
    { 0x47a33d88, 33 },
    { 0x7c933d89, 33 },
    { 0, 0 },
    { 0x9a33d8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb233d8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x63433d91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49933d95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff133d99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51633d9d, 23 },
    { 0, 0 },
    { 0xdca33d9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd33da5, 61 },
    { 0xc8c33da6, 55 },
    { 0xe2733da6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda33dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e33db2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9933db6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf33dbe, 33 },
    { 0x69833dbe, 66 },
    { 0x2a833dc0, 93 },
    { 0xa5933dbf, 77 },
    { 0x33d33dc2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7033dc5, 93 },
    { 0xceb33dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d633dc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c33dcd, 68 },
    { 0x93e33dce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9633dd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8333dd4, 77 },
    { 0, 0 },
    { 0xcd633dd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f033ddd, 46 },
    { 0x2033ddd, 94 },
    { 0, 0 },
    { 0xb1133de0, 66 },
    { 0x90633de1, 33 },
    { 0x65e33de2, 12 },
    { 0, 0 },
    { 0x88133de4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e033dee, 99 },
    { 0xb3a33dee, 85 },
    { 0xb3933df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a733df4, 90 },
    { 0x76933df5, 90 },
    { 0xe4433df6, 92 },
    { 0, 0 },
    { 0x6c433df8, 98 },
    { 0, 0 },
    { 0x66633dfa, 88 },
    { 0x50633dfb, 75 },
    { 0x77033dfc, 90 },
    { 0x8133dfc, 90 },
    { 0x3dd33dfe, 33 },
    { 0xf333dfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14533e02, 85 },
    { 0, 0 },
    { 0xa9d33e04, 98 },
    { 0x4ec33e05, 84 },
    { 0x3b633e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42033e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81033e0f, 23 },
    { 0x19533e10, 33 },
    { 0x31733e10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda33e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7533e1c, 98 },
    { 0, 0 },
    { 0x67833e1e, 61 },
    { 0, 0 },
    { 0x11133e20, 76 },
    { 0, 0 },
    { 0x59d33e22, 55 },
    { 0, 0 },
    { 0x3dc33e24, 90 },
    { 0x8f533e24, 40 },
    { 0xcbd33e25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6433e29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d333e2d, 55 },
    { 0, 0 },
    { 0x76c33e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69833e33, 78 },
    { 0x6af33e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33e37, 34 },
    { 0x41633e38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d233e3b, 98 },
    { 0, 0 },
    { 0xcdf33e3d, 87 },
    { 0, 0 },
    { 0x5b233e3f, 61 },
    { 0xb4a33e40, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8633e43, 90 },
    { 0, 0 },
    { 0xd2e33e45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x64133e48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30933e4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad33e50, 77 },
    { 0x38a33e51, 33 },
    { 0x9a033e52, 34 },
    { 0x99f33e53, 77 },
    { 0, 0 },
    { 0xbdf33e55, 90 },
    { 0xfd533e56, 52 },
    { 0, 0 },
    { 0x7233e58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6233e5e, 53 },
    { 0x3f933e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db33e65, 76 },
    { 0x9db33e66, 76 },
    { 0xfc933e66, 95 },
    { 0x81f33e68, 55 },
    { 0x51233e67, 76 },
    { 0x28933e6a, 61 },
    { 0x3d33e6a, 33 },
    { 0x86a33e6a, 81 },
    { 0x7ef33e6a, 98 },
    { 0xe2533e6e, 23 },
    { 0, 0 },
    { 0xd8433e70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0933e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a833e77, 90 },
    { 0xd633e78, 23 },
    { 0, 0 },
    { 0xdf033e7a, 23 },
    { 0, 0 },
    { 0x44433e7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf833e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d533e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91633e90, 98 },
    { 0, 0 },
    { 0xe833e92, 90 },
    { 0xe533e92, 38 },
    { 0, 0 },
    { 0x24c33e95, 34 },
    { 0x4d533e95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d33e9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd33ea4, 100 },
    { 0x9e733ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33233eaf, 76 },
    { 0xdac33eb0, 35 },
    { 0x5ce33eb1, 55 },
    { 0xff33eb1, 46 },
    { 0, 0 },
    { 0xa4a33eb4, 90 },
    { 0xa7f33eb5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74433eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd33ebe, 95 },
    { 0x3ca33ebf, 55 },
    { 0x81833ec0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14133ec6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c833ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c933ecf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf33ed2, 52 },
    { 0xfe533ed2, 33 },
    { 0x7ce33ed4, 95 },
    { 0x20233ed3, 98 },
    { 0, 0 },
    { 0x22633ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b733edb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc633ee1, 90 },
    { 0x9c33ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2633ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80033eec, 84 },
    { 0, 0 },
    { 0x56533eee, 23 },
    { 0, 0 },
    { 0x3dd33ef0, 98 },
    { 0xdc33ef1, 99 },
    { 0x15133ef2, 98 },
    { 0, 0 },
    { 0x40d33ef4, 23 },
    { 0, 0 },
    { 0x34733ef6, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x35133ef9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45933efd, 33 },
    { 0, 0 },
    { 0x29733eff, 33 },
    { 0x34133f00, 98 },
    { 0x93c33f01, 61 },
    { 0x7fe33f00, 48 },
    { 0x3cf33f03, 61 },
    { 0x4f733f04, 61 },
    { 0x4ec33f04, 76 },
    { 0x3be33f06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed33f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39333f11, 61 },
    { 0x38233f12, 33 },
    { 0x12b33f13, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef33f16, 95 },
    { 0, 0 },
    { 0xbf133f18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b33f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8833f23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d33f26, 90 },
    { 0xbe33f26, 90 },
    { 0, 0 },
    { 0x2f033f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb033f2c, 99 },
    { 0, 0 },
    { 0xda033f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a33f34, 76 },
    { 0xa2033f34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf33f3a, 90 },
    { 0xaf333f3a, 43 },
    { 0xa0133f3c, 95 },
    { 0xde033f3d, 33 },
    { 0x8333f3e, 34 },
    { 0x10a33f3e, 85 },
    { 0xebd33f40, 100 },
    { 0, 0 },
    { 0xa1733f42, 46 },
    { 0xd1433f43, 76 },
    { 0x32733f44, 99 },
    { 0x1a033f44, 61 },
    { 0xc7233f45, 95 },
    { 0xabe33f42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0233f4a, 90 },
    { 0, 0 },
    { 0x5a233f4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2833f51, 75 },
    { 0x8b633f52, 43 },
    { 0x6b633f53, 95 },
    { 0x4d133f51, 100 },
    { 0x77333f55, 33 },
    { 0x2b333f56, 34 },
    { 0x6aa33f56, 98 },
    { 0x6b833f56, 98 },
    { 0, 0 },
    { 0x6ad33f5a, 95 },
    { 0x96933f5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0433f5f, 68 },
    { 0x11b33f5f, 95 },
    { 0, 0 },
    { 0xfd033f62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed33f65, 17 },
    { 0xa8633f65, 92 },
    { 0xf3033f67, 95 },
    { 0x4ad33f68, 87 },
    { 0xc2233f69, 90 },
    { 0x51333f69, 55 },
    { 0x3ad33f6a, 90 },
    { 0xdfb33f6b, 95 },
    { 0x5f633f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f33f76, 66 },
    { 0x23533f77, 61 },
    { 0xe5133f78, 34 },
    { 0xef233f78, 90 },
    { 0x40333f77, 95 },
    { 0xc6733f7b, 98 },
    { 0, 0 },
    { 0x1cf33f7d, 33 },
    { 0x6533f7e, 46 },
    { 0xcb333f7f, 90 },
    { 0x69733f80, 76 },
    { 0, 0 },
    { 0xed633f82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff533f85, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3833f88, 90 },
    { 0xd7033f89, 46 },
    { 0x75c33f8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96133f8e, 61 },
    { 0x6df33f8e, 90 },
    { 0x3f633f8e, 90 },
    { 0x47633f91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9633f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd33fa0, 76 },
    { 0x42b33fa1, 98 },
    { 0, 0 },
    { 0x83733fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e33fa7, 84 },
    { 0xb0f33fa8, 68 },
    { 0xde333fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e733fad, 90 },
    { 0x27933fad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6633fb1, 90 },
    { 0, 0 },
    { 0x51133fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4633fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f33fbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36733fc2, 33 },
    { 0xd6a33fc3, 98 },
    { 0, 0 },
    { 0xaad33fc5, 98 },
    { 0, 0 },
    { 0x74033fc7, 33 },
    { 0, 0 },
    { 0x8ea33fc9, 61 },
    { 0xad533fc9, 88 },
    { 0x28a33fcb, 55 },
    { 0x2cc33fcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e33fd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd33fd3, 99 },
    { 0, 0 },
    { 0xd8733fd5, 98 },
    { 0, 0 },
    { 0x4c433fd7, 98 },
    { 0, 0 },
    { 0x60d33fd9, 61 },
    { 0, 0 },
    { 0xf333fdb, 17 },
    { 0x74333fdb, 61 },
    { 0xa1d33fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59133fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70533fea, 93 },
    { 0xf4633feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e33ff0, 76 },
    { 0xca233ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33ffb, 61 },
    { 0, 0 },
    { 0xd6833ffd, 90 },
    { 0, 0 },
    { 0x8a433fff, 95 },
    { 0, 0 },
    { 0x16534001, 61 },
    { 0x60634002, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75134007, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e3400b, 36 },
    { 0x91e3400c, 17 },
    { 0x85f3400d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d34010, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa34016, 68 },
    { 0, 0 },
    { 0x33334018, 98 },
    { 0x77334019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54134024, 61 },
    { 0xf0134025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f534028, 52 },
    { 0x67634029, 67 },
    { 0xb2534029, 90 },
    { 0x5863402b, 94 },
    { 0x4bc3402c, 33 },
    { 0x4b23402d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5034030, 90 },
    { 0xc8534030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a34034, 33 },
    { 0, 0 },
    { 0x3034036, 61 },
    { 0, 0 },
    { 0xc2b34038, 33 },
    { 0x27d34038, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6253403e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4434045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d3404a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3404e, 43 },
    { 0, 0 },
    { 0x45934050, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e34053, 95 },
    { 0xa7534054, 90 },
    { 0x16e34055, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f3405a, 55 },
    { 0xd043405a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b34060, 68 },
    { 0x21234061, 80 },
    { 0xe6f34061, 52 },
    { 0x27f34060, 33 },
    { 0, 0 },
    { 0xcba34065, 61 },
    { 0xb0c34066, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3406b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e3406f, 38 },
    { 0x31a34070, 90 },
    { 0x73c34071, 34 },
    { 0x57934072, 99 },
    { 0xc5634073, 33 },
    { 0x6c134074, 77 },
    { 0x77334074, 61 },
    { 0xd9a34070, 76 },
    { 0xb6934073, 96 },
    { 0xa6f34071, 43 },
    { 0x64334079, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af3407d, 38 },
    { 0x8323407d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98734084, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8034091, 55 },
    { 0xe7d34092, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103340a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f340a8, 90 },
    { 0x1c0340a9, 23 },
    { 0x446340a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94340ae, 77 },
    { 0x1fa340ae, 55 },
    { 0xa98340b0, 98 },
    { 0xd4d340b1, 23 },
    { 0x95c340b1, 61 },
    { 0, 0 },
    { 0x4aa340b4, 90 },
    { 0x865340b5, 93 },
    { 0xd5e340b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c340bb, 76 },
    { 0xe3a340bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74340bf, 98 },
    { 0xca8340c0, 95 },
    { 0xcc3340c0, 61 },
    { 0x499340c2, 90 },
    { 0x98e340bf, 23 },
    { 0xefe340c4, 23 },
    { 0x472340c5, 90 },
    { 0xae3340c6, 90 },
    { 0x452340c0, 95 },
    { 0x73f340c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9340cb, 90 },
    { 0x64d340cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f340d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5340d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff340da, 90 },
    { 0x3dc340db, 23 },
    { 0xf6c340db, 90 },
    { 0x44d340dd, 98 },
    { 0x9bd340de, 23 },
    { 0, 0 },
    { 0x432340e0, 61 },
    { 0x25f340e1, 46 },
    { 0x62b340e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9340e6, 76 },
    { 0, 0 },
    { 0x24e340e8, 75 },
    { 0, 0 },
    { 0xf03340ea, 90 },
    { 0x462340eb, 23 },
    { 0xdd6340ec, 100 },
    { 0xd29340ed, 66 },
    { 0, 0 },
    { 0xa09340ef, 46 },
    { 0x7e340ef, 98 },
    { 0x5db340f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6340f5, 34 },
    { 0xd6340f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5340fa, 61 },
    { 0, 0 },
    { 0x751340fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e340ff, 40 },
    { 0x7234100, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61334104, 66 },
    { 0, 0 },
    { 0xc4d34106, 87 },
    { 0xa7534107, 23 },
    { 0xc1d34108, 33 },
    { 0, 0 },
    { 0xb5b3410a, 91 },
    { 0, 0 },
    { 0x17d3410c, 33 },
    { 0x8833410c, 95 },
    { 0xc3d3410e, 61 },
    { 0x7853410f, 23 },
    { 0x3ed3410f, 43 },
    { 0xd3434111, 80 },
    { 0, 0 },
    { 0x3534113, 34 },
    { 0, 0 },
    { 0xe6034115, 61 },
    { 0xb9a34116, 33 },
    { 0x75934115, 98 },
    { 0x2a234118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63411b, 33 },
    { 0, 0 },
    { 0x1973411d, 90 },
    { 0, 0 },
    { 0x8c93411f, 99 },
    { 0x4a63411f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2033412c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb034130, 78 },
    { 0x35234131, 95 },
    { 0, 0 },
    { 0x34434133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85334137, 23 },
    { 0x6d434138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2683413f, 81 },
    { 0x80d34140, 90 },
    { 0x39d3413f, 23 },
    { 0x2573413f, 33 },
    { 0xab334143, 95 },
    { 0xe9934143, 43 },
    { 0xbad34140, 61 },
    { 0x4ff34146, 66 },
    { 0xa9234141, 98 },
    { 0, 0 },
    { 0xb8d34149, 61 },
    { 0xe953414a, 94 },
    { 0x4d33414b, 85 },
    { 0, 0 },
    { 0x28d3414d, 46 },
    { 0xa083414d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10534153, 33 },
    { 0x9dd34153, 90 },
    { 0xbfa34155, 90 },
    { 0x9c734156, 55 },
    { 0xb8934156, 61 },
    { 0xb4d34158, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb3415d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34160, 95 },
    { 0x64d34161, 90 },
    { 0x39234162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2534165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653416d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7634177, 95 },
    { 0x33f34178, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3417d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e134180, 33 },
    { 0, 0 },
    { 0x7034182, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42634186, 23 },
    { 0xd4734186, 23 },
    { 0x2b534186, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb523418b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd883418e, 98 },
    { 0xbfb3418f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f34192, 91 },
    { 0x26d34192, 33 },
    { 0x8a534193, 90 },
    { 0xec634195, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e634198, 84 },
    { 0, 0 },
    { 0x1743419a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9341a0, 33 },
    { 0xf1e341a1, 83 },
    { 0x579341a1, 76 },
    { 0x9a9341a3, 75 },
    { 0x98e341a2, 76 },
    { 0xe04341a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867341ab, 55 },
    { 0, 0 },
    { 0xa8a341ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7341b2, 33 },
    { 0, 0 },
    { 0x52c341b4, 78 },
    { 0x44b341b5, 93 },
    { 0, 0 },
    { 0xc61341b7, 87 },
    { 0, 0 },
    { 0xedf341b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab341bf, 61 },
    { 0x8e0341bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a341c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389341ce, 43 },
    { 0, 0 },
    { 0xbc7341d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee341d4, 76 },
    { 0, 0 },
    { 0x38c341d6, 55 },
    { 0, 0 },
    { 0xae3341d8, 33 },
    { 0, 0 },
    { 0x641341da, 76 },
    { 0x546341db, 68 },
    { 0x337341dc, 90 },
    { 0x6b1341dc, 55 },
    { 0x2e6341dd, 76 },
    { 0x520341df, 23 },
    { 0x94341e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23341e4, 43 },
    { 0x76c341e4, 55 },
    { 0xb53341e5, 99 },
    { 0xdc5341e5, 100 },
    { 0, 0 },
    { 0xa5f341e9, 95 },
    { 0x4e341ea, 52 },
    { 0x59d341eb, 61 },
    { 0, 0 },
    { 0x6d5341ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e341f2, 90 },
    { 0x777341f3, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439341f8, 95 },
    { 0, 0 },
    { 0x50c341fa, 96 },
    { 0x2dd341fb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6034200, 66 },
    { 0, 0 },
    { 0xd2034202, 61 },
    { 0, 0 },
    { 0x8bf34204, 46 },
    { 0, 0 },
    { 0x6a634206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e3420a, 85 },
    { 0x6d43420a, 42 },
    { 0, 0 },
    { 0x3433420d, 76 },
    { 0x3ec3420e, 33 },
    { 0, 0 },
    { 0x3ce34210, 68 },
    { 0x56434211, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd934217, 33 },
    { 0x8cb34217, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543421e, 95 },
    { 0x2da3421f, 76 },
    { 0xc9834220, 100 },
    { 0x7103421e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf3422b, 61 },
    { 0xca53422c, 98 },
    { 0xacd3422d, 88 },
    { 0x85e3422c, 90 },
    { 0x1d03422d, 33 },
    { 0x5d3422d, 23 },
    { 0x4763422b, 100 },
    { 0xf2234232, 33 },
    { 0xd0434230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f53423e, 90 },
    { 0, 0 },
    { 0xe7634240, 98 },
    { 0xe4334240, 33 },
    { 0xba334240, 95 },
    { 0xd2c34243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1334247, 84 },
    { 0xa1334248, 61 },
    { 0x7c734249, 85 },
    { 0, 0 },
    { 0xaa63424b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c34250, 88 },
    { 0xe1534251, 93 },
    { 0x76634252, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39534256, 91 },
    { 0x5ad34256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093425d, 55 },
    { 0, 0 },
    { 0x16b3425f, 90 },
    { 0xa203425f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9134266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3426d, 77 },
    { 0x9c3426e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d134271, 95 },
    { 0x32f34272, 66 },
    { 0x99c34273, 67 },
    { 0x5a234273, 87 },
    { 0xb334273, 85 },
    { 0xb4f34274, 61 },
    { 0, 0 },
    { 0x5ff34278, 33 },
    { 0x93334278, 23 },
    { 0x3e34278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd3427d, 93 },
    { 0x5463427e, 68 },
    { 0xec23427e, 23 },
    { 0xd183427f, 76 },
    { 0, 0 },
    { 0x4f234282, 61 },
    { 0, 0 },
    { 0x6d134284, 23 },
    { 0xe1d34285, 33 },
    { 0x6fe34286, 90 },
    { 0x1cb34287, 76 },
    { 0x31a34287, 98 },
    { 0xb9434287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2334293, 36 },
    { 0, 0 },
    { 0xf3834295, 78 },
    { 0xd0234296, 23 },
    { 0x49c34296, 90 },
    { 0xab34297, 91 },
    { 0x2f34299, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504342a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d342af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a342b4, 90 },
    { 0xda8342b4, 61 },
    { 0x457342b5, 96 },
    { 0xd3a342b4, 98 },
    { 0, 0 },
    { 0x905342b9, 100 },
    { 0xf07342ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4342be, 95 },
    { 0, 0 },
    { 0xf2c342c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981342c5, 33 },
    { 0, 0 },
    { 0xd77342c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709342cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x481342cf, 98 },
    { 0x85a342d0, 23 },
    { 0x337342d0, 43 },
    { 0x6ca342d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71342da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52342e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03342ea, 95 },
    { 0x3af342eb, 90 },
    { 0, 0 },
    { 0x820342ed, 55 },
    { 0, 0 },
    { 0x86b342ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10342f2, 95 },
    { 0, 0 },
    { 0x9bd342f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e342f9, 84 },
    { 0x132342fa, 61 },
    { 0, 0 },
    { 0x5aa342fc, 43 },
    { 0x5ff342fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x63534300, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f34303, 81 },
    { 0xc9f34304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a3430b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963430e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62234311, 33 },
    { 0x9b534312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e3431c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a34325, 98 },
    { 0x9db34326, 99 },
    { 0x60534326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943432b, 66 },
    { 0, 0 },
    { 0x7ad3432d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec34332, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80434336, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6333433a, 90 },
    { 0x84c3433b, 34 },
    { 0, 0 },
    { 0x7f3433d, 95 },
    { 0x28f3433e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a34344, 61 },
    { 0x60434344, 81 },
    { 0x77834346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a3434c, 66 },
    { 0xe163434c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234353, 66 },
    { 0x4fc34353, 98 },
    { 0xddc34355, 23 },
    { 0x75334355, 99 },
    { 0x16834355, 90 },
    { 0x92834358, 23 },
    { 0xac34359, 87 },
    { 0x6c43435a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193435e, 76 },
    { 0x84d3435f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf853436a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b34375, 34 },
    { 0xc1d34376, 61 },
    { 0, 0 },
    { 0xba734378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b83437e, 95 },
    { 0xf9f3437f, 33 },
    { 0x6813437e, 33 },
    { 0xd9034381, 90 },
    { 0x26134382, 61 },
    { 0x5e34383, 23 },
    { 0x87a34384, 55 },
    { 0, 0 },
    { 0xc4234386, 95 },
    { 0x78a34387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x413438a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783438d, 98 },
    { 0xde33438d, 33 },
    { 0, 0 },
    { 0xd0f34390, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e34397, 34 },
    { 0x2ac34398, 90 },
    { 0, 0 },
    { 0x6b33439a, 98 },
    { 0xb9f3439b, 98 },
    { 0xc283439c, 84 },
    { 0, 0 },
    { 0x8a33439e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89343a3, 43 },
    { 0xfd4343a4, 88 },
    { 0xffb343a4, 76 },
    { 0x52d343a4, 55 },
    { 0x68f343a7, 33 },
    { 0x557343a7, 90 },
    { 0xd4343a4, 98 },
    { 0xbb343aa, 100 },
    { 0x13d343ab, 23 },
    { 0xd15343ac, 66 },
    { 0x48a343aa, 95 },
    { 0, 0 },
    { 0xd8343af, 90 },
    { 0, 0 },
    { 0xffb343b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981343be, 33 },
    { 0xe58343bf, 61 },
    { 0xa5e343c0, 98 },
    { 0x74343be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a343c9, 34 },
    { 0xc4b343ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc343cd, 33 },
    { 0, 0 },
    { 0x488343cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850343d3, 17 },
    { 0x7be343d4, 23 },
    { 0x5af343d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d343db, 90 },
    { 0xbe1343dc, 87 },
    { 0, 0 },
    { 0xb7d343de, 95 },
    { 0, 0 },
    { 0xb80343e0, 90 },
    { 0x3fd343e0, 95 },
    { 0x924343e1, 90 },
    { 0xc26343e2, 23 },
    { 0x899343e4, 66 },
    { 0, 0 },
    { 0x60c343e6, 88 },
    { 0xda9343e7, 33 },
    { 0xc60343e8, 61 },
    { 0xddc343e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444343ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad343f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422343f9, 61 },
    { 0x1cb343fa, 88 },
    { 0x8be343f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d34400, 33 },
    { 0x92734400, 90 },
    { 0x3b334402, 33 },
    { 0x5ee34403, 34 },
    { 0x2a234404, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8443440b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1853440e, 23 },
    { 0, 0 },
    { 0x1dc34410, 91 },
    { 0xc6c34411, 95 },
    { 0xc0734411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11834415, 55 },
    { 0, 0 },
    { 0x43234417, 98 },
    { 0x8f834418, 33 },
    { 0x9dd34419, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3441d, 97 },
    { 0, 0 },
    { 0x1193441f, 38 },
    { 0xc0f34420, 55 },
    { 0x86534421, 98 },
    { 0x4ae34421, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8634425, 91 },
    { 0x81c34425, 98 },
    { 0x9cb34427, 23 },
    { 0x1ef34426, 95 },
    { 0, 0 },
    { 0xec53442a, 98 },
    { 0, 0 },
    { 0xb293442c, 40 },
    { 0x1333442d, 85 },
    { 0x4203442e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a734433, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c934436, 33 },
    { 0xbb334437, 34 },
    { 0x75034438, 55 },
    { 0, 0 },
    { 0x89d3443a, 33 },
    { 0, 0 },
    { 0x70e3443c, 61 },
    { 0, 0 },
    { 0xde83443e, 98 },
    { 0x50a3443e, 90 },
    { 0x9df3443e, 93 },
    { 0xd7b3443f, 90 },
    { 0x22434442, 95 },
    { 0, 0 },
    { 0xd0734444, 43 },
    { 0x1f834445, 95 },
    { 0x61534446, 91 },
    { 0x13434447, 33 },
    { 0xc9534448, 90 },
    { 0x7ed34448, 90 },
    { 0xcb934445, 90 },
    { 0x5b53444a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f34451, 36 },
    { 0xfb34452, 61 },
    { 0xd2834453, 43 },
    { 0x9534453, 98 },
    { 0x18d34455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb3445c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6083445f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe34463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b334466, 43 },
    { 0, 0 },
    { 0x81c34468, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd3446d, 76 },
    { 0xf533446e, 84 },
    { 0, 0 },
    { 0xc1134470, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa34473, 33 },
    { 0x2b434474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff34479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433447c, 43 },
    { 0xd543447d, 61 },
    { 0x8383447c, 77 },
    { 0xfb33447f, 93 },
    { 0xf2e34480, 84 },
    { 0xfc934481, 61 },
    { 0, 0 },
    { 0x4af34483, 52 },
    { 0, 0 },
    { 0x60134485, 52 },
    { 0xf9a34485, 90 },
    { 0x11c34486, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be3448a, 93 },
    { 0, 0 },
    { 0xf793448c, 91 },
    { 0x5ce3448d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be34490, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc734496, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f3449c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459344a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42344a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x346344a8, 90 },
    { 0, 0 },
    { 0xadb344aa, 61 },
    { 0, 0 },
    { 0xc21344ac, 61 },
    { 0, 0 },
    { 0x6b6344ae, 46 },
    { 0x4fd344af, 81 },
    { 0xe67344af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b344b3, 78 },
    { 0xc7344b3, 78 },
    { 0xee0344b4, 95 },
    { 0xd8344b6, 52 },
    { 0xc5f344b6, 90 },
    { 0xdb8344b8, 33 },
    { 0xf47344b9, 61 },
    { 0, 0 },
    { 0x401344bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704344c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72344c9, 34 },
    { 0x71d344ca, 23 },
    { 0xd13344ca, 87 },
    { 0xc8d344ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90344d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5344d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb344df, 61 },
    { 0x1a1344e0, 23 },
    { 0xfa3344e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3344e4, 46 },
    { 0, 0 },
    { 0xfe1344e6, 23 },
    { 0, 0 },
    { 0x5cd344e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d344ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c344f2, 90 },
    { 0xfd6344f3, 34 },
    { 0xa51344f4, 33 },
    { 0xef6344f3, 34 },
    { 0x63f344f4, 90 },
    { 0xca5344f7, 98 },
    { 0xb91344f8, 61 },
    { 0x7c9344f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d344fc, 33 },
    { 0xaa3344fd, 61 },
    { 0xe37344fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba34504, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a3450a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b34512, 34 },
    { 0xc4b34513, 36 },
    { 0xd4234514, 98 },
    { 0x60f34515, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e34518, 95 },
    { 0x6c334519, 95 },
    { 0xfb434518, 95 },
    { 0xd6a3451b, 33 },
    { 0x7093451c, 77 },
    { 0xf3a3451a, 91 },
    { 0x20134518, 90 },
    { 0xb1734519, 43 },
    { 0x39a3451f, 98 },
    { 0x2534521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62534526, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64434529, 61 },
    { 0xae13452a, 76 },
    { 0x6e334529, 93 },
    { 0x40e3452c, 98 },
    { 0, 0 },
    { 0x28b3452e, 84 },
    { 0x3c13452f, 84 },
    { 0, 0 },
    { 0xa6e34531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5734538, 55 },
    { 0x2534539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3453c, 64 },
    { 0x983453c, 99 },
    { 0xa6d3453d, 76 },
    { 0, 0 },
    { 0xafe34540, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f34545, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c13454c, 88 },
    { 0xfe33454d, 33 },
    { 0xe773454d, 85 },
    { 0xb733454c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff234553, 43 },
    { 0x6bd34554, 46 },
    { 0x2de34555, 69 },
    { 0x6bb34556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c3455c, 34 },
    { 0x6e3455d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4134567, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243456b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6534570, 90 },
    { 0x34234571, 33 },
    { 0x82934572, 55 },
    { 0x3034573, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a34576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e3457a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313457d, 90 },
    { 0x23e3457e, 55 },
    { 0x7223457f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f34584, 93 },
    { 0x48b34585, 95 },
    { 0x3534586, 34 },
    { 0, 0 },
    { 0x67c34588, 33 },
    { 0x82534588, 34 },
    { 0, 0 },
    { 0x96e3458b, 87 },
    { 0xeba3458c, 90 },
    { 0, 0 },
    { 0x9ed3458e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x19034591, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a34595, 48 },
    { 0, 0 },
    { 0x1b834597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713459b, 46 },
    { 0x8843459c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29345a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8345a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1345a7, 90 },
    { 0x42f345a8, 61 },
    { 0x224345a8, 90 },
    { 0x66d345aa, 33 },
    { 0, 0 },
    { 0x6c4345ac, 61 },
    { 0x910345ad, 93 },
    { 0xee9345ac, 43 },
    { 0x931345af, 46 },
    { 0xd19345ae, 95 },
    { 0x5dc345af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b345b4, 90 },
    { 0x289345b5, 52 },
    { 0, 0 },
    { 0x8f8345b7, 46 },
    { 0x70a345b8, 90 },
    { 0xd3e345b7, 33 },
    { 0x946345ba, 90 },
    { 0xfc4345bb, 90 },
    { 0, 0 },
    { 0xd47345bd, 80 },
    { 0x9bf345be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20345c8, 95 },
    { 0xad7345c9, 69 },
    { 0x7ab345ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7345ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a345d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x854345dc, 19 },
    { 0, 0 },
    { 0x20f345de, 55 },
    { 0, 0 },
    { 0x44a345e0, 91 },
    { 0, 0 },
    { 0x6a9345e2, 90 },
    { 0, 0 },
    { 0xbce345e4, 33 },
    { 0x651345e5, 58 },
    { 0xf62345e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3345eb, 61 },
    { 0x625345eb, 61 },
    { 0xbbf345ed, 99 },
    { 0x84f345eb, 33 },
    { 0xb75345eb, 61 },
    { 0x4b0345f0, 48 },
    { 0x60b345ee, 90 },
    { 0x904345f2, 23 },
    { 0xe97345f3, 46 },
    { 0xce4345f3, 78 },
    { 0x131345f5, 90 },
    { 0xd7c345f6, 90 },
    { 0x562345f7, 33 },
    { 0x892345f6, 90 },
    { 0x236345f8, 95 },
    { 0x98b345f5, 78 },
    { 0x848345eb, 43 },
    { 0, 0 },
    { 0xf0e345fd, 34 },
    { 0x615345fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12334601, 90 },
    { 0, 0 },
    { 0x62434603, 95 },
    { 0x7134604, 91 },
    { 0xe0d34605, 33 },
    { 0, 0 },
    { 0x59734607, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd3460f, 33 },
    { 0, 0 },
    { 0x35734611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e34614, 95 },
    { 0, 0 },
    { 0x5d334616, 52 },
    { 0x98e34617, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a3461d, 90 },
    { 0xbac3461d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4234625, 90 },
    { 0x8ad34626, 100 },
    { 0, 0 },
    { 0x19034628, 55 },
    { 0, 0 },
    { 0xeb3462a, 78 },
    { 0xcff3462b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83462e, 55 },
    { 0x15f3462f, 98 },
    { 0xb253462f, 43 },
    { 0x43d3462f, 46 },
    { 0xee134632, 33 },
    { 0xa0534633, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a734639, 98 },
    { 0xcca3463a, 95 },
    { 0xcad3463b, 66 },
    { 0x1f234639, 91 },
    { 0x3f63463a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b34641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83464a, 33 },
    { 0x3313464b, 33 },
    { 0x8e13464c, 66 },
    { 0xa43464c, 98 },
    { 0xb123464e, 23 },
    { 0xb003464f, 34 },
    { 0, 0 },
    { 0x98834651, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b34656, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec3465a, 93 },
    { 0x53465b, 61 },
    { 0, 0 },
    { 0xec43465d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c934660, 68 },
    { 0, 0 },
    { 0x14c34662, 98 },
    { 0, 0 },
    { 0x42b34664, 33 },
    { 0xca534664, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd34669, 23 },
    { 0x6ef3466a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763466e, 95 },
    { 0xcc53466e, 72 },
    { 0x32d3466f, 85 },
    { 0, 0 },
    { 0x23c34672, 33 },
    { 0xbe834673, 61 },
    { 0xb734673, 99 },
    { 0x5b934674, 98 },
    { 0x6ff34674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d3467c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd34683, 98 },
    { 0x69d34684, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53234688, 98 },
    { 0xbe334688, 95 },
    { 0xd323468a, 99 },
    { 0x6283468b, 78 },
    { 0x23634689, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d434690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35234693, 61 },
    { 0xb1534694, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223469a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783469f, 88 },
    { 0xe76346a0, 98 },
    { 0xca5346a1, 93 },
    { 0, 0 },
    { 0xe9c346a3, 76 },
    { 0x1d2346a4, 36 },
    { 0xe08346a5, 68 },
    { 0x890346a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61346a9, 61 },
    { 0x3ae346aa, 23 },
    { 0, 0 },
    { 0x1bb346ac, 33 },
    { 0, 0 },
    { 0xded346ae, 61 },
    { 0x8f8346af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01346b4, 90 },
    { 0, 0 },
    { 0xefe346b6, 90 },
    { 0x212346b6, 55 },
    { 0x2fa346b6, 23 },
    { 0xbb7346b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4346bc, 55 },
    { 0xc29346bc, 90 },
    { 0xc55346bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13346c1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x927346c4, 90 },
    { 0x9de346c5, 61 },
    { 0xcfc346c6, 98 },
    { 0, 0 },
    { 0x926346c8, 43 },
    { 0, 0 },
    { 0x389346ca, 33 },
    { 0xdb4346ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb346d0, 52 },
    { 0, 0 },
    { 0xf15346d2, 46 },
    { 0xd4f346d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21346d8, 33 },
    { 0x5a2346d8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1346dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464346e3, 95 },
    { 0xc3d346e4, 55 },
    { 0, 0 },
    { 0x687346e6, 93 },
    { 0, 0 },
    { 0x1ca346e8, 76 },
    { 0, 0 },
    { 0x226346ea, 90 },
    { 0x7b9346eb, 91 },
    { 0xcc4346ec, 77 },
    { 0, 0 },
    { 0xbd7346ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596346f5, 98 },
    { 0x9fc346f6, 95 },
    { 0x112346f5, 40 },
    { 0xbce346f8, 76 },
    { 0x106346f8, 65 },
    { 0x5ea346fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d346fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78134704, 47 },
    { 0x8c434704, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab34708, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c3470d, 90 },
    { 0x9683470e, 55 },
    { 0xb9e3470f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa334713, 52 },
    { 0x1ae34713, 95 },
    { 0x25534715, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90534719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc3471d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff34721, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b34724, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a934728, 23 },
    { 0x53c34729, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233472d, 61 },
    { 0xe53472e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e934731, 61 },
    { 0x47234732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed34735, 78 },
    { 0x71534736, 38 },
    { 0xe6534736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa13473c, 95 },
    { 0x5fc3473c, 90 },
    { 0x2e13473c, 90 },
    { 0, 0 },
    { 0x19a34740, 93 },
    { 0xdd634741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5834744, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f034748, 17 },
    { 0, 0 },
    { 0xc923474a, 85 },
    { 0, 0 },
    { 0x36f3474c, 93 },
    { 0x6e63474d, 95 },
    { 0x91b3474e, 98 },
    { 0x38b3474e, 55 },
    { 0xb2734750, 90 },
    { 0x93c34751, 33 },
    { 0x65a34752, 98 },
    { 0xfa03474c, 91 },
    { 0x6f934754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda534759, 95 },
    { 0x19934759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213475e, 66 },
    { 0x5663475f, 23 },
    { 0x5b34760, 76 },
    { 0xd6c34760, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f34764, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f134769, 92 },
    { 0, 0 },
    { 0xcf93476b, 34 },
    { 0x7a23476b, 90 },
    { 0xb63476d, 55 },
    { 0x16d3476e, 55 },
    { 0xda73476e, 33 },
    { 0xdd43476e, 76 },
    { 0xc2a34771, 33 },
    { 0, 0 },
    { 0x2ad34773, 100 },
    { 0x83d34774, 90 },
    { 0x23e34774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb83477d, 33 },
    { 0x2913477e, 43 },
    { 0, 0 },
    { 0xbc334780, 90 },
    { 0x82634781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea534784, 100 },
    { 0xb6834784, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2334789, 78 },
    { 0, 0 },
    { 0x593478b, 61 },
    { 0, 0 },
    { 0xda13478d, 75 },
    { 0x2da3478e, 69 },
    { 0xc7d3478f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b334792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de34797, 43 },
    { 0, 0 },
    { 0xdf734799, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6347a7, 43 },
    { 0, 0 },
    { 0x7fc347a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4347b0, 98 },
    { 0, 0 },
    { 0xb08347b2, 33 },
    { 0x258347b3, 55 },
    { 0, 0 },
    { 0x39b347b5, 55 },
    { 0x3f3347b6, 90 },
    { 0x393347b7, 80 },
    { 0x52f347b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50347bc, 98 },
    { 0x8d5347bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1347c4, 91 },
    { 0x895347c5, 33 },
    { 0, 0 },
    { 0xd3b347c7, 55 },
    { 0x365347c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51347cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39347d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x389347da, 23 },
    { 0xf91347db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27347df, 61 },
    { 0, 0 },
    { 0xf21347e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa347e6, 46 },
    { 0x67f347e6, 100 },
    { 0x9af347e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615347ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3347f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81434800, 95 },
    { 0xbb034801, 61 },
    { 0xa1334802, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31334805, 61 },
    { 0xc7734806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80734809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc3480d, 87 },
    { 0x35c3480d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b34813, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b34816, 33 },
    { 0x93c34817, 98 },
    { 0xc9334818, 65 },
    { 0x84d34816, 61 },
    { 0, 0 },
    { 0xea93481b, 33 },
    { 0x7ec3481c, 78 },
    { 0, 0 },
    { 0x48b3481e, 98 },
    { 0x91d3481e, 55 },
    { 0, 0 },
    { 0xae834821, 96 },
    { 0, 0 },
    { 0x1e234823, 90 },
    { 0x1bf34823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5634830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d34835, 23 },
    { 0, 0 },
    { 0x44f34837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd34842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2534845, 95 },
    { 0x3d134846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda834851, 33 },
    { 0xd0534852, 23 },
    { 0xe7534853, 90 },
    { 0x6f34854, 97 },
    { 0x70734855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92934859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f134865, 17 },
    { 0xebd34866, 95 },
    { 0xd4f34866, 69 },
    { 0xeb734865, 36 },
    { 0x91034869, 38 },
    { 0x9843486a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d334872, 95 },
    { 0, 0 },
    { 0x62034874, 33 },
    { 0x68a34874, 61 },
    { 0x4f634875, 95 },
    { 0x5a134877, 95 },
    { 0, 0 },
    { 0x1b734879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9703487d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c34880, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7834893, 33 },
    { 0x64334894, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd23489e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc348a1, 40 },
    { 0xf5b348a2, 63 },
    { 0x6c3348a3, 98 },
    { 0xd95348a4, 23 },
    { 0x668348a3, 61 },
    { 0, 0 },
    { 0x125348a7, 87 },
    { 0xbb8348a7, 95 },
    { 0xd30348a9, 61 },
    { 0x76d348aa, 94 },
    { 0xcb9348ab, 90 },
    { 0x968348aa, 23 },
    { 0x6c7348ad, 55 },
    { 0x1e7348ae, 78 },
    { 0, 0 },
    { 0xaa8348b0, 55 },
    { 0x900348b1, 61 },
    { 0, 0 },
    { 0x1f9348b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea348b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463348ba, 93 },
    { 0x5e4348bb, 99 },
    { 0x363348bc, 40 },
    { 0x498348bd, 76 },
    { 0x29d348bd, 23 },
    { 0xbe4348bf, 33 },
    { 0x458348bd, 23 },
    { 0x2e0348bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2348c5, 98 },
    { 0xb8d348c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850348cb, 95 },
    { 0, 0 },
    { 0xde1348cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb348d2, 90 },
    { 0x9d2348d2, 61 },
    { 0xfc2348d2, 98 },
    { 0xc32348d5, 76 },
    { 0xfce348d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7348da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x988348dd, 69 },
    { 0, 0 },
    { 0x589348df, 81 },
    { 0, 0 },
    { 0x55f348e1, 90 },
    { 0, 0 },
    { 0xa74348e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36348e6, 76 },
    { 0x1e0348e7, 77 },
    { 0x320348e6, 90 },
    { 0x5b4348e9, 23 },
    { 0x6b0348e9, 98 },
    { 0xf6e348eb, 90 },
    { 0x2f8348ec, 93 },
    { 0x20e348ed, 23 },
    { 0x2c0348ec, 85 },
    { 0xb8a348e7, 55 },
    { 0xaa2348ec, 98 },
    { 0xba2348f1, 98 },
    { 0x92d348f1, 98 },
    { 0, 0 },
    { 0x603348f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e348fc, 95 },
    { 0xf22348fc, 98 },
    { 0xe02348fd, 100 },
    { 0, 0 },
    { 0x9be34900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc243490b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c3490f, 17 },
    { 0, 0 },
    { 0x3f734911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a234915, 33 },
    { 0, 0 },
    { 0x9734917, 98 },
    { 0x1df34918, 66 },
    { 0, 0 },
    { 0xb783491a, 61 },
    { 0xd2a3491b, 84 },
    { 0x66d3491b, 95 },
    { 0xcff3491d, 43 },
    { 0x9a3491d, 56 },
    { 0x1523491f, 76 },
    { 0xe113491d, 55 },
    { 0x6323491c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b34924, 90 },
    { 0xd4034925, 90 },
    { 0x82b34924, 93 },
    { 0xa9734924, 61 },
    { 0, 0 },
    { 0x98134929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f13492c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3492f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a34932, 43 },
    { 0xb8034933, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26234939, 90 },
    { 0xccc3493a, 43 },
    { 0x7f33493b, 46 },
    { 0xf5b3493a, 61 },
    { 0x7e33493d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad34945, 90 },
    { 0xa8634945, 100 },
    { 0x1a934947, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb193494a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf013494e, 98 },
    { 0, 0 },
    { 0xaf734950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66734957, 91 },
    { 0, 0 },
    { 0xcfd34959, 95 },
    { 0xc5834959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f3495d, 34 },
    { 0x4a3495d, 90 },
    { 0xc353495f, 43 },
    { 0x7cd3495f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d34963, 46 },
    { 0x4c334963, 76 },
    { 0x74c34963, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0734968, 61 },
    { 0xeb434969, 95 },
    { 0xee53496a, 76 },
    { 0, 0 },
    { 0x4a93496c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec34970, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8034973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43034978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5034980, 61 },
    { 0, 0 },
    { 0x73b34982, 36 },
    { 0, 0 },
    { 0x83534984, 95 },
    { 0x5d534985, 88 },
    { 0, 0 },
    { 0xb7534987, 98 },
    { 0xa1234988, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d3498b, 90 },
    { 0x8053498c, 76 },
    { 0xdba3498d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee34991, 55 },
    { 0, 0 },
    { 0x8934993, 99 },
    { 0x13234994, 98 },
    { 0xd0934995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33834998, 61 },
    { 0xb9934999, 88 },
    { 0xeb534999, 61 },
    { 0, 0 },
    { 0xc393499c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81349a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e349a8, 90 },
    { 0xc2e349a8, 68 },
    { 0x222349aa, 23 },
    { 0x5d7349a8, 90 },
    { 0x936349ac, 81 },
    { 0xde9349ad, 38 },
    { 0xb7a349ad, 90 },
    { 0x9ba349af, 34 },
    { 0xa28349ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16349b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455349bd, 61 },
    { 0x136349be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80349c1, 23 },
    { 0xd8d349c2, 90 },
    { 0x329349c3, 98 },
    { 0x6b5349c3, 66 },
    { 0xe5c349c3, 61 },
    { 0xdbc349c3, 78 },
    { 0x29b349c5, 96 },
    { 0x31e349c8, 33 },
    { 0x68d349c9, 61 },
    { 0x1d7349ca, 34 },
    { 0xac349cb, 33 },
    { 0x1349cc, 43 },
    { 0x6bb349cd, 64 },
    { 0, 0 },
    { 0x76b349cf, 23 },
    { 0xb73349d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7349d3, 61 },
    { 0, 0 },
    { 0x26c349d5, 90 },
    { 0x21b349d6, 33 },
    { 0xd43349d5, 98 },
    { 0x6ba349d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d349dc, 33 },
    { 0xe72349dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139349e4, 90 },
    { 0x48f349e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84349e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552349ef, 95 },
    { 0, 0 },
    { 0xb80349f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc349f4, 23 },
    { 0x176349f5, 99 },
    { 0x89c349f4, 76 },
    { 0x181349f5, 61 },
    { 0, 0 },
    { 0x86349f9, 52 },
    { 0x7d5349fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27434a01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b34a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b534a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b334a11, 93 },
    { 0x87d34a12, 90 },
    { 0, 0 },
    { 0xf7d34a14, 95 },
    { 0xeb834a14, 33 },
    { 0x41b34a14, 90 },
    { 0x534a17, 61 },
    { 0x83a34a14, 98 },
    { 0xf5534a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6034a1e, 61 },
    { 0x4b134a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c34a23, 34 },
    { 0xb9c34a23, 95 },
    { 0x7d134a25, 33 },
    { 0x64034a26, 69 },
    { 0, 0 },
    { 0xeeb34a28, 34 },
    { 0xabc34a29, 76 },
    { 0, 0 },
    { 0xe1e34a2b, 93 },
    { 0x68734a2b, 95 },
    { 0x21034a2d, 55 },
    { 0, 0 },
    { 0xa0a34a2f, 52 },
    { 0, 0 },
    { 0x3a134a31, 68 },
    { 0, 0 },
    { 0x31034a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb34a38, 98 },
    { 0x96134a39, 88 },
    { 0x97734a39, 61 },
    { 0, 0 },
    { 0x4fa34a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22534a43, 61 },
    { 0x1d034a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd34a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34a4a, 90 },
    { 0x24634a4a, 87 },
    { 0xa034a4c, 66 },
    { 0x7f034a4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d34a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e34a5b, 33 },
    { 0, 0 },
    { 0x48b34a5d, 66 },
    { 0x89034a5e, 84 },
    { 0xce34a5f, 100 },
    { 0x74a34a5e, 95 },
    { 0x6ac34a61, 90 },
    { 0, 0 },
    { 0x40434a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb034a6e, 55 },
    { 0x8de34a6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2534a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc34a76, 90 },
    { 0, 0 },
    { 0x9c534a78, 96 },
    { 0, 0 },
    { 0xecd34a7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff34a7f, 76 },
    { 0, 0 },
    { 0xad534a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f034a84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e334a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d34a8e, 23 },
    { 0x65134a8f, 55 },
    { 0x5ab34a90, 90 },
    { 0x26734a90, 90 },
    { 0xbac34a92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b134a98, 90 },
    { 0x25b34a98, 95 },
    { 0xf7034a9a, 80 },
    { 0x89c34a99, 61 },
    { 0x87b34a9c, 100 },
    { 0xe534a9b, 95 },
    { 0x54634a9c, 76 },
    { 0xf9934a9f, 90 },
    { 0x58634a9a, 95 },
    { 0, 0 },
    { 0x86834aa2, 40 },
    { 0x27f34aa3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a734aa6, 78 },
    { 0, 0 },
    { 0xfd734aa8, 93 },
    { 0, 0 },
    { 0xf0c34aaa, 90 },
    { 0x4fe34aab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b234aae, 100 },
    { 0x4ff34aae, 98 },
    { 0xa1e34ab0, 23 },
    { 0, 0 },
    { 0x1b734ab2, 75 },
    { 0, 0 },
    { 0x72c34ab4, 46 },
    { 0x71034ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee34ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba34abe, 100 },
    { 0x17834abf, 61 },
    { 0, 0 },
    { 0xad534ac1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a34ac4, 90 },
    { 0x2d34ac5, 93 },
    { 0x94434ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb34ac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77834ace, 90 },
    { 0x7a434acf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50834ad7, 76 },
    { 0, 0 },
    { 0x22c34ad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13034adc, 33 },
    { 0xf5434add, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c34ae2, 61 },
    { 0, 0 },
    { 0x83834ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4334ae7, 98 },
    { 0, 0 },
    { 0xfb734ae9, 99 },
    { 0xbaa34aea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15534af7, 66 },
    { 0xfb334af7, 93 },
    { 0x8b234af9, 61 },
    { 0x91734af8, 76 },
    { 0, 0 },
    { 0x13c34afc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e34aff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f434b02, 46 },
    { 0, 0 },
    { 0xda434b04, 61 },
    { 0, 0 },
    { 0x2d734b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d934b0b, 93 },
    { 0xe4c34b0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a034b10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b934b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5134b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad734b22, 43 },
    { 0, 0 },
    { 0x74434b24, 33 },
    { 0x2cd34b25, 93 },
    { 0x95834b26, 19 },
    { 0xdc734b24, 46 },
    { 0x7a834b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf834b2d, 43 },
    { 0xdb234b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c534b38, 43 },
    { 0xc5734b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c34b3c, 33 },
    { 0, 0 },
    { 0xbe634b3e, 90 },
    { 0, 0 },
    { 0xf8a34b40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30234b4a, 23 },
    { 0x54934b4b, 78 },
    { 0xefe34b4a, 52 },
    { 0, 0 },
    { 0xf234b4e, 67 },
    { 0xa6234b4e, 95 },
    { 0x89c34b4e, 34 },
    { 0x8d34b51, 100 },
    { 0x81834b52, 65 },
    { 0, 0 },
    { 0x1e034b54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1234b59, 23 },
    { 0x85134b5a, 66 },
    { 0x9ef34b5a, 90 },
    { 0x17134b5c, 98 },
    { 0x6634b59, 90 },
    { 0xbd534b5e, 95 },
    { 0, 0 },
    { 0x38634b60, 85 },
    { 0, 0 },
    { 0x83134b62, 46 },
    { 0, 0 },
    { 0xdd934b64, 61 },
    { 0x5f434b65, 23 },
    { 0x16134b66, 98 },
    { 0xbe534b67, 90 },
    { 0x44d34b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5434b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff34b72, 98 },
    { 0xc6134b72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2534b7f, 93 },
    { 0, 0 },
    { 0x17f34b81, 97 },
    { 0x4e34b82, 88 },
    { 0, 0 },
    { 0xf0834b84, 61 },
    { 0xa3034b85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd834b88, 61 },
    { 0xc1534b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd934b8e, 80 },
    { 0, 0 },
    { 0x4f34b90, 98 },
    { 0x8d634b91, 34 },
    { 0xb6934b90, 19 },
    { 0x5d134b93, 33 },
    { 0x72734b94, 93 },
    { 0xebd34b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3034b99, 90 },
    { 0xc7934b9a, 33 },
    { 0, 0 },
    { 0x6b834b9c, 90 },
    { 0, 0 },
    { 0xb1e34b9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81034ba3, 100 },
    { 0xb4b34ba3, 100 },
    { 0x83634ba5, 98 },
    { 0x1cc34ba6, 77 },
    { 0x87734ba7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b734bac, 66 },
    { 0xc1d34bad, 90 },
    { 0x6d334bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf34bb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c934bba, 61 },
    { 0xee134bbb, 95 },
    { 0x48934bbc, 76 },
    { 0x5ff34bbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3334bc1, 61 },
    { 0x46b34bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2734bc9, 98 },
    { 0x86134bca, 81 },
    { 0xd2334bc9, 93 },
    { 0, 0 },
    { 0x52f34bcd, 98 },
    { 0x8a434bce, 33 },
    { 0, 0 },
    { 0x68134bd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90534bd3, 95 },
    { 0, 0 },
    { 0xd3a34bd5, 33 },
    { 0x7e234bd6, 23 },
    { 0, 0 },
    { 0x7cb34bd8, 61 },
    { 0xbc734bd9, 99 },
    { 0, 0 },
    { 0x7b034bdb, 98 },
    { 0, 0 },
    { 0x69e34bdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e34be5, 38 },
    { 0, 0 },
    { 0x94734be7, 23 },
    { 0, 0 },
    { 0xb4934be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa034bf0, 55 },
    { 0xa234bf1, 90 },
    { 0xc3d34bf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a534bf8, 23 },
    { 0xacb34bf8, 90 },
    { 0x99334bf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe34c02, 87 },
    { 0x14134c03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee434c06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55434c09, 81 },
    { 0xcba34c0a, 90 },
    { 0x22834c0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b34c0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1234c18, 43 },
    { 0x45d34c18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1634c1c, 85 },
    { 0x3e234c1d, 90 },
    { 0x7ad34c1e, 55 },
    { 0x89334c1c, 34 },
    { 0xb0b34c1f, 55 },
    { 0xc3b34c1f, 98 },
    { 0xd8334c22, 61 },
    { 0x6ec34c21, 61 },
    { 0, 0 },
    { 0xe3434c25, 90 },
    { 0, 0 },
    { 0x6134c27, 33 },
    { 0xefc34c28, 65 },
    { 0x3e634c27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf734c2c, 55 },
    { 0xd0f34c2c, 61 },
    { 0xda334c2e, 95 },
    { 0, 0 },
    { 0x9de34c30, 95 },
    { 0x82534c30, 43 },
    { 0xf5934c31, 95 },
    { 0x98a34c33, 66 },
    { 0x71334c34, 23 },
    { 0xd5034c33, 5 },
    { 0, 0 },
    { 0x30734c37, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5134c3a, 61 },
    { 0, 0 },
    { 0x91134c3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da34c3f, 46 },
    { 0x7e134c40, 55 },
    { 0xb2d34c40, 38 },
    { 0xf334c3f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba34c47, 55 },
    { 0xe1e34c48, 23 },
    { 0xcd34c49, 52 },
    { 0x3fc34c49, 61 },
    { 0x72734c49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5334c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2034c53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35634c58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba734c5b, 46 },
    { 0, 0 },
    { 0xb9434c5d, 55 },
    { 0x38c34c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68834c66, 99 },
    { 0, 0 },
    { 0xaf134c68, 33 },
    { 0x99434c69, 90 },
    { 0x54a34c6a, 33 },
    { 0, 0 },
    { 0xa6c34c6c, 93 },
    { 0x3fe34c6d, 61 },
    { 0x1c534c6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd34c73, 90 },
    { 0x50f34c73, 34 },
    { 0, 0 },
    { 0xec334c76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17034c80, 98 },
    { 0, 0 },
    { 0xd3134c82, 90 },
    { 0x1f634c83, 90 },
    { 0x68234c82, 75 },
    { 0x26034c85, 95 },
    { 0xd0f34c86, 76 },
    { 0x64434c86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a34c91, 90 },
    { 0xa7934c92, 61 },
    { 0x60534c91, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed34c96, 90 },
    { 0x1c34c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51334c9c, 91 },
    { 0xf3934c9c, 34 },
    { 0x3d234c9e, 95 },
    { 0x59334c9f, 33 },
    { 0, 0 },
    { 0xcd34ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa834ca6, 46 },
    { 0xe8834ca7, 23 },
    { 0x82834ca8, 89 },
    { 0xc7234ca9, 90 },
    { 0, 0 },
    { 0xcfb34cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a34cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4734cb3, 33 },
    { 0xe8d34cb4, 61 },
    { 0x39a34cb4, 36 },
    { 0x8d434cb6, 76 },
    { 0x38234cb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a034cbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed734cc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26634cc6, 61 },
    { 0x3934cc7, 90 },
    { 0, 0 },
    { 0x24734cc9, 98 },
    { 0, 0 },
    { 0x1fd34ccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6434cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd34cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11034cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b34cda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c34cdd, 67 },
    { 0, 0 },
    { 0x84d34cdf, 76 },
    { 0x6b934ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51134ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3434ce9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52534cec, 93 },
    { 0x4e034cec, 90 },
    { 0, 0 },
    { 0x92734cef, 23 },
    { 0xda834cf0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d34cf3, 90 },
    { 0xd6234cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b34cf8, 23 },
    { 0, 0 },
    { 0xa3f34cfa, 76 },
    { 0x2a34cfb, 98 },
    { 0xc6834cfc, 78 },
    { 0x55c34cfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc34d07, 61 },
    { 0xd534d07, 34 },
    { 0, 0 },
    { 0x5ab34d0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d334d0d, 90 },
    { 0xf6c34d0e, 99 },
    { 0xbd534d0d, 98 },
    { 0x4d434d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33134d15, 90 },
    { 0xe3834d16, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad34d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec34d1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed034d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e34d2d, 43 },
    { 0, 0 },
    { 0xf4b34d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a34d33, 46 },
    { 0x58434d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d34d39, 98 },
    { 0xf0834d3a, 23 },
    { 0xbc334d3b, 95 },
    { 0xf9934d3c, 43 },
    { 0xb1534d3b, 38 },
    { 0x7534d3e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034d41, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb734d4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70734d4f, 76 },
    { 0, 0 },
    { 0xa34d51, 61 },
    { 0x90834d51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d34d57, 43 },
    { 0, 0 },
    { 0xfb34d59, 95 },
    { 0x46434d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64634d5f, 95 },
    { 0x6f934d60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc834d64, 61 },
    { 0xc1b34d65, 66 },
    { 0, 0 },
    { 0x2e934d67, 90 },
    { 0x45434d67, 55 },
    { 0, 0 },
    { 0xd834d6a, 98 },
    { 0, 0 },
    { 0x46c34d6c, 46 },
    { 0x42234d6d, 55 },
    { 0x78b34d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21934d72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80834d84, 90 },
    { 0x6d434d85, 33 },
    { 0x9e134d85, 95 },
    { 0x88334d87, 90 },
    { 0x8ae34d87, 95 },
    { 0x3a034d89, 78 },
    { 0xae634d88, 23 },
    { 0, 0 },
    { 0x90a34d8c, 66 },
    { 0x5a634d8d, 95 },
    { 0x5d234d8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e34d92, 65 },
    { 0, 0 },
    { 0x1e134d94, 95 },
    { 0x9f334d94, 23 },
    { 0x3ff34d96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x64534d99, 33 },
    { 0, 0 },
    { 0x73134d9b, 55 },
    { 0x5ec34d9b, 90 },
    { 0, 0 },
    { 0xcac34d9e, 33 },
    { 0x32a34d9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b34da5, 34 },
    { 0x1e634da6, 55 },
    { 0, 0 },
    { 0x2bb34da8, 90 },
    { 0x6c334da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f34dae, 95 },
    { 0, 0 },
    { 0x1f734db0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e434db5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86634dba, 93 },
    { 0x46434dbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e34dbf, 33 },
    { 0, 0 },
    { 0xc3534dc1, 61 },
    { 0, 0 },
    { 0x75e34dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d34dcb, 95 },
    { 0x3b34dcb, 55 },
    { 0x4c834dcc, 38 },
    { 0x7ea34dce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee34dd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6934dd4, 33 },
    { 0, 0 },
    { 0xd0d34dd6, 75 },
    { 0x68a34dd7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77734dde, 84 },
    { 0x80c34ddf, 90 },
    { 0xfc534de0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b34de7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a34dea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb34df1, 61 },
    { 0xd2a34df2, 98 },
    { 0xe0e34df3, 23 },
    { 0xb3034df4, 91 },
    { 0xd3934df5, 61 },
    { 0xb1334df6, 33 },
    { 0, 0 },
    { 0x8c534df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34dfb, 23 },
    { 0xe2134dfb, 90 },
    { 0xb1434dfc, 98 },
    { 0x9e34dfb, 91 },
    { 0x48d34dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa34e02, 78 },
    { 0, 0 },
    { 0x38c34e04, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98534e0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59434e17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f34e1d, 76 },
    { 0x2f634e1d, 23 },
    { 0x5a134e1d, 33 },
    { 0xa6234e1d, 61 },
    { 0x7334e21, 76 },
    { 0xd9a34e20, 98 },
    { 0xcb334e23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc34e26, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df34e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16534e34, 55 },
    { 0, 0 },
    { 0xc7e34e36, 99 },
    { 0xae434e36, 43 },
    { 0x82b34e36, 17 },
    { 0xa9c34e38, 23 },
    { 0xa0234e3a, 66 },
    { 0, 0 },
    { 0x6c934e3c, 94 },
    { 0x57434e3c, 91 },
    { 0xc9f34e3e, 34 },
    { 0x5f834e3e, 33 },
    { 0x9b934e3e, 70 },
    { 0xfd834e40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a534e44, 69 },
    { 0, 0 },
    { 0x14834e46, 91 },
    { 0x1c934e47, 55 },
    { 0, 0 },
    { 0xa4534e49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a34e4e, 97 },
    { 0xcaf34e4f, 17 },
    { 0xe9b34e4f, 61 },
    { 0x8d834e51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f34e54, 90 },
    { 0x8cd34e55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d34e62, 80 },
    { 0, 0 },
    { 0x70f34e64, 99 },
    { 0xcfc34e64, 77 },
    { 0x91734e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a434e6c, 98 },
    { 0xeb234e6d, 23 },
    { 0x7e334e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77934e73, 98 },
    { 0xfdd34e73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc34e79, 33 },
    { 0xe2034e79, 98 },
    { 0x54634e79, 23 },
    { 0x12534e7c, 46 },
    { 0x20234e7d, 76 },
    { 0x63934e7c, 33 },
    { 0xfa834e7c, 77 },
    { 0x4cc34e80, 55 },
    { 0x82134e7a, 90 },
    { 0x7f934e7a, 76 },
    { 0xefa34e82, 85 },
    { 0x7f34e84, 61 },
    { 0x5a034e84, 43 },
    { 0xc8334e86, 43 },
    { 0xd7734e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf34e8a, 93 },
    { 0xcd034e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18334e8e, 87 },
    { 0x18734e8f, 90 },
    { 0x9bb34e90, 90 },
    { 0x52f34e8e, 90 },
    { 0xfac34e91, 95 },
    { 0, 0 },
    { 0x49634e94, 61 },
    { 0xb1634e94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16234e98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf634e9f, 61 },
    { 0x95934ea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2634ea4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x49634ea7, 90 },
    { 0x3a734ea8, 61 },
    { 0x47234ea9, 87 },
    { 0, 0 },
    { 0xdfa34eab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c34eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d734eb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc34eb9, 90 },
    { 0, 0 },
    { 0x6fb34ebb, 43 },
    { 0, 0 },
    { 0x1ba34ebd, 81 },
    { 0xb5334ebe, 65 },
    { 0x79d34ebf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34ec3, 68 },
    { 0xcdd34ec4, 33 },
    { 0xc6d34ec3, 34 },
    { 0x92e34ec6, 62 },
    { 0, 0 },
    { 0x96f34ec8, 66 },
    { 0x2da34ec9, 46 },
    { 0x99334ec9, 61 },
    { 0x6b334ecb, 90 },
    { 0, 0 },
    { 0x2c734ecd, 61 },
    { 0x8c434ecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c034ed5, 100 },
    { 0x1b434ed5, 100 },
    { 0x4a34ed6, 98 },
    { 0x44a34ed8, 76 },
    { 0xa8d34ed8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7634edc, 34 },
    { 0xf0234edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34ee0, 33 },
    { 0xe4f34ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98634ee4, 61 },
    { 0x71334ee5, 61 },
    { 0x14e34ee6, 33 },
    { 0x7aa34ee4, 23 },
    { 0xdae34ee8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4434eec, 23 },
    { 0, 0 },
    { 0x6d234eee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da34ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a34ef9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a034efc, 33 },
    { 0, 0 },
    { 0xede34efe, 33 },
    { 0x51034efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac634f05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xab334f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11734f0d, 76 },
    { 0xf5134f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54734f11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a234f14, 98 },
    { 0x6fe34f15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3634f18, 61 },
    { 0x5dd34f19, 61 },
    { 0, 0 },
    { 0x7a234f1b, 90 },
    { 0x61334f1c, 91 },
    { 0xd6334f1d, 33 },
    { 0x79334f1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x19734f21, 33 },
    { 0, 0 },
    { 0x34f23, 76 },
    { 0x17934f23, 76 },
    { 0, 0 },
    { 0xa9e34f26, 95 },
    { 0, 0 },
    { 0x4ef34f28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f34f2b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6734f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e434f38, 90 },
    { 0x9e234f38, 98 },
    { 0x5bf34f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba034f3d, 76 },
    { 0x69c34f3e, 91 },
    { 0x22534f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9734f43, 33 },
    { 0x4b134f44, 43 },
    { 0x66134f43, 98 },
    { 0x86f34f46, 33 },
    { 0x68e34f44, 76 },
    { 0, 0 },
    { 0x6ab34f49, 98 },
    { 0xf234f49, 76 },
    { 0x28134f4b, 77 },
    { 0x90a34f4b, 55 },
    { 0x44e34f4d, 90 },
    { 0xd0b34f4e, 95 },
    { 0x31c34f4f, 55 },
    { 0x7ff34f50, 66 },
    { 0x3ff34f51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50434f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b434f59, 61 },
    { 0, 0 },
    { 0xfee34f5b, 46 },
    { 0xb3734f5c, 23 },
    { 0, 0 },
    { 0x1a034f5e, 78 },
    { 0x32634f5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea34f62, 61 },
    { 0xcb934f63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e34f67, 52 },
    { 0x3bb34f68, 33 },
    { 0x56434f67, 52 },
    { 0xaed34f6a, 33 },
    { 0xda434f6a, 76 },
    { 0xe34f6c, 98 },
    { 0xcbc34f6a, 61 },
    { 0x79034f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c34f75, 55 },
    { 0x46934f76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec234f7e, 33 },
    { 0x6dc34f7e, 90 },
    { 0, 0 },
    { 0xee334f81, 61 },
    { 0xa5734f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d34f8a, 93 },
    { 0xd0e34f8a, 61 },
    { 0x80334f8c, 61 },
    { 0, 0 },
    { 0xfe034f8e, 90 },
    { 0x34934f8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae34f94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6734f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc134fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db34fa8, 33 },
    { 0xb8034fa8, 75 },
    { 0x2d334faa, 42 },
    { 0x1ad34faa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec34fb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e034fbe, 33 },
    { 0x5d834fbf, 87 },
    { 0x6e534fbf, 33 },
    { 0x6f434fbf, 90 },
    { 0, 0 },
    { 0x6e134fc3, 23 },
    { 0, 0 },
    { 0x5ad34fc5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37034fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50934fd1, 33 },
    { 0x3b34fd2, 33 },
    { 0, 0 },
    { 0xa9834fd4, 90 },
    { 0, 0 },
    { 0xf7634fd6, 87 },
    { 0, 0 },
    { 0xd6b34fd8, 98 },
    { 0xa9e34fd8, 43 },
    { 0, 0 },
    { 0x4f234fdb, 55 },
    { 0, 0 },
    { 0x16734fdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1134fe1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f234fe6, 34 },
    { 0x4ac34fe7, 23 },
    { 0x8e134fe7, 96 },
    { 0x93f34fe9, 55 },
    { 0, 0 },
    { 0xaf534feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c34ff3, 99 },
    { 0xc4434ff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82634ff9, 52 },
    { 0xfc34ff9, 100 },
    { 0, 0 },
    { 0xab034ffc, 23 },
    { 0x8a34ffd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf735001, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d35005, 34 },
    { 0xcc235006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a35009, 90 },
    { 0x49f3500a, 76 },
    { 0xfc03500b, 34 },
    { 0, 0 },
    { 0xeb83500d, 43 },
    { 0x4553500e, 61 },
    { 0x5e43500f, 52 },
    { 0x22e3500e, 93 },
    { 0, 0 },
    { 0xb1c35012, 40 },
    { 0xea335013, 85 },
    { 0x4735012, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe35018, 95 },
    { 0x25f35019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda635020, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9735029, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2473502e, 85 },
    { 0x1623502f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14835032, 61 },
    { 0, 0 },
    { 0x43f35034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34935037, 33 },
    { 0, 0 },
    { 0xf2035039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a35040, 76 },
    { 0, 0 },
    { 0x81935042, 34 },
    { 0, 0 },
    { 0x48c35044, 90 },
    { 0x94b35044, 23 },
    { 0x16335045, 33 },
    { 0xa5735045, 23 },
    { 0x47d35045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2203504d, 84 },
    { 0xf9a3504e, 61 },
    { 0x94f3504f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e335052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb35056, 55 },
    { 0xc9235056, 61 },
    { 0, 0 },
    { 0x21435059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3505e, 98 },
    { 0x5253505f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8335064, 33 },
    { 0x6ed35065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a35069, 97 },
    { 0xecc35069, 98 },
    { 0x89a35069, 90 },
    { 0x4093506c, 76 },
    { 0, 0 },
    { 0xc2a3506e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c935074, 23 },
    { 0x61d35075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47835078, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c3507d, 69 },
    { 0xd9a3507e, 76 },
    { 0, 0 },
    { 0xb1635080, 90 },
    { 0x86435081, 23 },
    { 0, 0 },
    { 0xba335083, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3335086, 33 },
    { 0xe7235087, 33 },
    { 0xf7035088, 87 },
    { 0xa7535089, 76 },
    { 0x19b35089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e23508f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3535092, 46 },
    { 0x54735093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac435097, 46 },
    { 0x55235098, 55 },
    { 0, 0 },
    { 0x7c23509a, 34 },
    { 0x5ac3509a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623509e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72350a5, 61 },
    { 0, 0 },
    { 0xdc7350a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c350ac, 90 },
    { 0xfa7350ad, 77 },
    { 0x8da350ae, 43 },
    { 0xeb1350af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b350b2, 90 },
    { 0, 0 },
    { 0x88a350b4, 40 },
    { 0xa5c350b5, 34 },
    { 0x73f350b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34350b9, 33 },
    { 0xbd0350b9, 61 },
    { 0xd08350b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f350bf, 46 },
    { 0x291350bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27350ca, 75 },
    { 0x65d350cb, 66 },
    { 0, 0 },
    { 0xc28350cd, 76 },
    { 0xd0d350ce, 43 },
    { 0x1e0350ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f350d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa350d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x898350da, 83 },
    { 0xd9f350db, 78 },
    { 0xe65350dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b350e0, 90 },
    { 0x94f350e1, 61 },
    { 0x5c9350e0, 98 },
    { 0x355350e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95350e8, 52 },
    { 0xeec350e9, 66 },
    { 0x6b5350e9, 100 },
    { 0xa40350ea, 98 },
    { 0x86350ea, 23 },
    { 0x5ed350ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c350f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b350f6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e350fa, 47 },
    { 0x86f350fa, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x415350fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e635102, 95 },
    { 0, 0 },
    { 0x31a35104, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76135107, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae535110, 90 },
    { 0x11335110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81035114, 61 },
    { 0, 0 },
    { 0xc9535116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e3511b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d35120, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b35124, 90 },
    { 0x4bc35125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d53512c, 33 },
    { 0x46e3512d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6535133, 90 },
    { 0x86b35134, 90 },
    { 0x6435134, 61 },
    { 0xceb35135, 46 },
    { 0x42c35137, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13513a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd13513d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56535141, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d3514c, 43 },
    { 0x13e3514d, 100 },
    { 0xf193514e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c35151, 33 },
    { 0, 0 },
    { 0xab735153, 93 },
    { 0x29435154, 55 },
    { 0, 0 },
    { 0xd7635156, 93 },
    { 0xd3e35157, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963515f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d935163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58435168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e3516e, 95 },
    { 0xbe63516f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2535174, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b23517b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3517e, 23 },
    { 0xa63517e, 83 },
    { 0x4ad35180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e35183, 61 },
    { 0x98635184, 43 },
    { 0xd5735185, 52 },
    { 0x77635186, 85 },
    { 0x54835183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63518a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99435191, 61 },
    { 0xcce35191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc35196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864351a7, 55 },
    { 0xbbb351a8, 93 },
    { 0, 0 },
    { 0x6c7351aa, 61 },
    { 0, 0 },
    { 0xf7e351ac, 98 },
    { 0, 0 },
    { 0x2e8351ae, 90 },
    { 0, 0 },
    { 0x4eb351b0, 66 },
    { 0x503351b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33351b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df351bc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8351c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de351c8, 34 },
    { 0, 0 },
    { 0xaa8351ca, 98 },
    { 0x80f351ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999351cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5351d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a351df, 76 },
    { 0x3c4351e0, 19 },
    { 0, 0 },
    { 0xd3e351e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17351ea, 95 },
    { 0xa2d351ea, 61 },
    { 0x698351ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a351ef, 61 },
    { 0x209351f0, 90 },
    { 0, 0 },
    { 0x6e5351f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a351f8, 61 },
    { 0xb29351f9, 46 },
    { 0, 0 },
    { 0x325351fb, 99 },
    { 0x1c0351fc, 23 },
    { 0xf8f351fd, 33 },
    { 0xc92351fe, 78 },
    { 0xab8351ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb35203, 61 },
    { 0x76335203, 33 },
    { 0, 0 },
    { 0xbe835206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a35209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3520c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76635210, 23 },
    { 0x75b35211, 90 },
    { 0xdf335212, 90 },
    { 0, 0 },
    { 0xc4c35214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32035218, 64 },
    { 0xf4d35219, 55 },
    { 0x1573521a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3521d, 23 },
    { 0xf203521e, 98 },
    { 0x6e33521f, 93 },
    { 0, 0 },
    { 0x37f35221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed435224, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1735229, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd3522c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293522f, 61 },
    { 0xeb235230, 23 },
    { 0x88835231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd135238, 95 },
    { 0, 0 },
    { 0xdaa3523a, 94 },
    { 0xd663523b, 33 },
    { 0xbef3523c, 23 },
    { 0xbb53523a, 93 },
    { 0xe7c3523a, 23 },
    { 0, 0 },
    { 0xb9935240, 66 },
    { 0, 0 },
    { 0xa5135242, 34 },
    { 0, 0 },
    { 0x89835244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18235247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3524b, 98 },
    { 0, 0 },
    { 0xc9e3524d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d35253, 90 },
    { 0xeb035253, 85 },
    { 0, 0 },
    { 0x6ad35256, 33 },
    { 0, 0 },
    { 0x86c35258, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc3525c, 33 },
    { 0x6f23525d, 43 },
    { 0xb133525c, 76 },
    { 0, 0 },
    { 0xbc235260, 78 },
    { 0x2a35261, 95 },
    { 0x6c235260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf35265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de35268, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3526d, 23 },
    { 0xad63526e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b235271, 95 },
    { 0x2b735272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca35275, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e435279, 61 },
    { 0, 0 },
    { 0xb9d3527b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3723527f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c35285, 95 },
    { 0, 0 },
    { 0xcc935287, 98 },
    { 0x34a35288, 43 },
    { 0x13335289, 90 },
    { 0x50535289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f35296, 95 },
    { 0, 0 },
    { 0xc7235298, 61 },
    { 0x9e735299, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3423529d, 95 },
    { 0xcc13529e, 90 },
    { 0xf113529f, 76 },
    { 0, 0 },
    { 0xd36352a1, 87 },
    { 0x1d4352a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240352a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437352ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c352b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e352b6, 64 },
    { 0x2a6352b7, 90 },
    { 0, 0 },
    { 0xfb8352b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4352bd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478352c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309352c6, 23 },
    { 0xf8d352c6, 98 },
    { 0, 0 },
    { 0xc07352c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62352cc, 61 },
    { 0, 0 },
    { 0xa01352ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e352d1, 34 },
    { 0xdbb352d2, 34 },
    { 0xdd0352d2, 61 },
    { 0, 0 },
    { 0xf66352d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198352d8, 98 },
    { 0xa3f352d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c352dc, 52 },
    { 0x9e6352dc, 91 },
    { 0x3c8352dd, 98 },
    { 0, 0 },
    { 0xa31352e0, 90 },
    { 0x535352e1, 83 },
    { 0, 0 },
    { 0xc1d352e3, 78 },
    { 0, 0 },
    { 0x142352e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b352ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b352ef, 90 },
    { 0x4f7352ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8352f3, 95 },
    { 0x211352f3, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa352f9, 90 },
    { 0x6d8352fa, 82 },
    { 0x5cf352fb, 95 },
    { 0x6df352f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a235300, 93 },
    { 0x11c35300, 99 },
    { 0, 0 },
    { 0x3ee35303, 90 },
    { 0x1ac35304, 61 },
    { 0, 0 },
    { 0xa0e35306, 90 },
    { 0xeca35307, 52 },
    { 0x22135307, 95 },
    { 0x6d135308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3530c, 52 },
    { 0, 0 },
    { 0x1ce3530e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee35316, 95 },
    { 0x86735316, 100 },
    { 0, 0 },
    { 0x91c35319, 33 },
    { 0, 0 },
    { 0x1b83531b, 61 },
    { 0xf123531c, 52 },
    { 0xa573531d, 43 },
    { 0x4dc3531e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9835322, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe535327, 85 },
    { 0x8f035328, 98 },
    { 0x2dc35329, 90 },
    { 0x5035329, 90 },
    { 0x1a13532b, 91 },
    { 0x9a53532c, 66 },
    { 0xa5c3532b, 23 },
    { 0x4363532c, 90 },
    { 0xcda3532f, 33 },
    { 0xc0d35329, 55 },
    { 0x3d635331, 9 },
    { 0, 0 },
    { 0x43035333, 76 },
    { 0xf3235333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b35338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153533b, 97 },
    { 0xffe3533c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e35342, 90 },
    { 0x7f535343, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae35348, 95 },
    { 0x1ab35349, 98 },
    { 0x9943534a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7733534d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4535352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea35358, 61 },
    { 0x42535359, 58 },
    { 0, 0 },
    { 0x1a53535b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333535e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd35364, 43 },
    { 0x3d335364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e3536b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c435371, 66 },
    { 0x78535372, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a35375, 93 },
    { 0, 0 },
    { 0xdae35377, 51 },
    { 0xeee35378, 78 },
    { 0, 0 },
    { 0xd753537a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3537f, 61 },
    { 0x5a73537f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab35389, 65 },
    { 0xb9235389, 99 },
    { 0, 0 },
    { 0x6bb3538c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b35390, 91 },
    { 0xac735391, 90 },
    { 0x5f235392, 17 },
    { 0x1fd35392, 23 },
    { 0xf8535394, 37 },
    { 0, 0 },
    { 0x30d35396, 23 },
    { 0x4ec35397, 90 },
    { 0, 0 },
    { 0x6635399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3353a6, 84 },
    { 0x322353a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142353ab, 75 },
    { 0x519353ac, 43 },
    { 0, 0 },
    { 0xf34353ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13353b8, 61 },
    { 0x6c3353b9, 95 },
    { 0xf8b353ba, 80 },
    { 0x9b3353b9, 61 },
    { 0xd5c353b8, 98 },
    { 0x770353bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f353c1, 33 },
    { 0, 0 },
    { 0xf89353c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f353c8, 90 },
    { 0x1c0353c8, 90 },
    { 0xa60353c8, 95 },
    { 0, 0 },
    { 0x1fb353cc, 90 },
    { 0, 0 },
    { 0xca353ce, 55 },
    { 0, 0 },
    { 0xcc3353d0, 95 },
    { 0x7a6353d0, 77 },
    { 0x670353d1, 40 },
    { 0xa92353d0, 90 },
    { 0x8f8353d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56353d7, 98 },
    { 0x4df353d7, 78 },
    { 0x608353d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724353de, 98 },
    { 0x8b2353de, 66 },
    { 0xea6353e0, 98 },
    { 0xd82353e1, 67 },
    { 0x747353e1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291353eb, 17 },
    { 0x840353ec, 76 },
    { 0x198353ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35353f4, 80 },
    { 0, 0 },
    { 0xec6353f6, 46 },
    { 0x79b353f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c353fa, 58 },
    { 0x13c353fb, 23 },
    { 0x7c7353fb, 87 },
    { 0x6353fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2635401, 23 },
    { 0xc6f35402, 84 },
    { 0x81635401, 67 },
    { 0x51635404, 56 },
    { 0xc4d35405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc63540a, 95 },
    { 0xee73540b, 61 },
    { 0x3873540b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3540f, 98 },
    { 0x61235410, 90 },
    { 0x7f835411, 98 },
    { 0xd8f35410, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef35418, 61 },
    { 0xbdc35419, 33 },
    { 0xb483541a, 84 },
    { 0xb143541b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf3541e, 55 },
    { 0, 0 },
    { 0x5b335420, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97735423, 98 },
    { 0xd5f35424, 98 },
    { 0x12c35425, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa683542a, 33 },
    { 0xdb63542b, 84 },
    { 0x793542b, 90 },
    { 0x7f13542d, 90 },
    { 0xdeb3542e, 34 },
    { 0x3f33542f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca835432, 98 },
    { 0, 0 },
    { 0xae535434, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f35437, 76 },
    { 0x3a135438, 33 },
    { 0, 0 },
    { 0x203543a, 17 },
    { 0xb6d3543b, 33 },
    { 0x58d3543a, 95 },
    { 0x7893543d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70535445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe603544e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa435453, 64 },
    { 0x9ff35454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1903545c, 38 },
    { 0x5203545c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2335460, 59 },
    { 0, 0 },
    { 0x47d35462, 66 },
    { 0xcac35463, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba43546e, 50 },
    { 0xa013546e, 90 },
    { 0x5583546f, 76 },
    { 0x2563546f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a35475, 77 },
    { 0x81735476, 99 },
    { 0xde735475, 33 },
    { 0x4a535477, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e3547c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc3547f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535486, 33 },
    { 0x48435487, 64 },
    { 0x20c35488, 91 },
    { 0xf0435486, 67 },
    { 0x8113548a, 95 },
    { 0x33d3548b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f035495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a735498, 33 },
    { 0, 0 },
    { 0xc3f3549a, 55 },
    { 0x1d53549a, 33 },
    { 0, 0 },
    { 0xe0e3549d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5354a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d354a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f354ae, 61 },
    { 0x213354af, 61 },
    { 0x7d9354b0, 95 },
    { 0xa58354b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d354b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8354ba, 84 },
    { 0xbea354ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x166354be, 8 },
    { 0x741354bf, 98 },
    { 0xbeb354c0, 91 },
    { 0, 0 },
    { 0xef8354c2, 23 },
    { 0xf31354c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43354c9, 75 },
    { 0x4b354ca, 66 },
    { 0, 0 },
    { 0x7d5354cc, 33 },
    { 0x42a354cc, 95 },
    { 0, 0 },
    { 0x16c354cf, 98 },
    { 0, 0 },
    { 0x787354d1, 98 },
    { 0, 0 },
    { 0xed6354d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852354d7, 46 },
    { 0xf8c354d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9354dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530354e2, 87 },
    { 0x755354e3, 90 },
    { 0x9b7354e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1354e9, 76 },
    { 0x918354e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5354ee, 87 },
    { 0x5c4354ee, 33 },
    { 0xac6354ef, 99 },
    { 0x24c354ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b354f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3354f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a135507, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b43550b, 98 },
    { 0, 0 },
    { 0x9953550d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e35511, 33 },
    { 0x55c35511, 90 },
    { 0x1d135512, 33 },
    { 0x79035514, 33 },
    { 0x4f135514, 55 },
    { 0x68a35512, 95 },
    { 0, 0 },
    { 0x25e35518, 90 },
    { 0x23c35519, 90 },
    { 0, 0 },
    { 0x72d3551b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72735521, 100 },
    { 0xf6735521, 61 },
    { 0x54f35523, 90 },
    { 0x3ef35524, 78 },
    { 0x27835521, 95 },
    { 0x98e35522, 90 },
    { 0x94c35527, 33 },
    { 0xe9035527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd653552c, 61 },
    { 0x5493552d, 34 },
    { 0x4053552e, 95 },
    { 0, 0 },
    { 0x45b35530, 34 },
    { 0x7ef35531, 77 },
    { 0xc3b35532, 90 },
    { 0x64c35533, 33 },
    { 0, 0 },
    { 0x87a35535, 76 },
    { 0, 0 },
    { 0x38235537, 90 },
    { 0x82235538, 95 },
    { 0x82d35537, 95 },
    { 0xc663553a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b23553f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd135542, 90 },
    { 0, 0 },
    { 0xfee35544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74935549, 87 },
    { 0x8703554a, 98 },
    { 0x55f3554b, 98 },
    { 0x2443554b, 76 },
    { 0x6023554c, 61 },
    { 0, 0 },
    { 0x5293554f, 66 },
    { 0, 0 },
    { 0x7735551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3555a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab3555d, 90 },
    { 0xaab3555e, 76 },
    { 0xd133555f, 33 },
    { 0, 0 },
    { 0xbfb35561, 34 },
    { 0, 0 },
    { 0xba235563, 43 },
    { 0, 0 },
    { 0x5a935565, 98 },
    { 0, 0 },
    { 0xd735567, 23 },
    { 0xdc435568, 76 },
    { 0x69935569, 43 },
    { 0x6ca3556a, 33 },
    { 0, 0 },
    { 0xf633556c, 83 },
    { 0x1623556d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3835574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8603557c, 33 },
    { 0x213557d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e35580, 61 },
    { 0x77535581, 34 },
    { 0, 0 },
    { 0x2835583, 95 },
    { 0, 0 },
    { 0x7d535585, 61 },
    { 0x65c35586, 76 },
    { 0x66d35586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8235591, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d35595, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc003559e, 90 },
    { 0xc223559f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451355a5, 88 },
    { 0, 0 },
    { 0x4d7355a7, 61 },
    { 0x4fa355a8, 61 },
    { 0x91c355a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5355af, 44 },
    { 0, 0 },
    { 0xa2e355b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x179355b4, 95 },
    { 0x1dc355b5, 81 },
    { 0x192355b6, 33 },
    { 0x6f6355b6, 67 },
    { 0, 0 },
    { 0x885355b9, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e355be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17355c3, 95 },
    { 0xcdf355c4, 66 },
    { 0x1ca355c5, 93 },
    { 0x1dc355c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562355ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4355d1, 76 },
    { 0xd9355d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e355d8, 100 },
    { 0x49355d8, 33 },
    { 0x2e1355da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1355dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305355e2, 33 },
    { 0, 0 },
    { 0xb79355e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3355e7, 67 },
    { 0x3c0355e8, 76 },
    { 0xaf9355e9, 65 },
    { 0x859355ea, 65 },
    { 0, 0 },
    { 0x65d355ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d355f5, 95 },
    { 0, 0 },
    { 0x7a6355f7, 37 },
    { 0x58f355f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d355fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9235603, 55 },
    { 0xc7135603, 90 },
    { 0xc9335603, 55 },
    { 0x33e35606, 43 },
    { 0, 0 },
    { 0xe5435608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb63560c, 33 },
    { 0x1893560d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47935610, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd035614, 66 },
    { 0x12035615, 99 },
    { 0x6d635616, 98 },
    { 0x13735617, 33 },
    { 0x3a635618, 61 },
    { 0x92335617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d43561c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e35622, 91 },
    { 0x3d035623, 33 },
    { 0xd0a35624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d35629, 76 },
    { 0x31135629, 33 },
    { 0x5113562b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba03562e, 61 },
    { 0, 0 },
    { 0xc1535630, 95 },
    { 0, 0 },
    { 0xae535632, 60 },
    { 0x73235632, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x14435636, 23 },
    { 0x3f435637, 95 },
    { 0x39e35638, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363563c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d35640, 43 },
    { 0xb8035641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c35647, 84 },
    { 0xe6635648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3263564b, 55 },
    { 0xf1d3564c, 69 },
    { 0, 0 },
    { 0xc1d3564e, 66 },
    { 0x3f23564e, 55 },
    { 0xa7b35650, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9535658, 48 },
    { 0xa2535659, 90 },
    { 0x9f63565a, 33 },
    { 0x3323565b, 43 },
    { 0x9a53565b, 98 },
    { 0x4193565c, 97 },
    { 0xa0a3565e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b35661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5243566a, 98 },
    { 0x37f3566b, 33 },
    { 0xe603566c, 78 },
    { 0x9813566b, 34 },
    { 0xddb3566b, 98 },
    { 0x8593566f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f35673, 61 },
    { 0, 0 },
    { 0xe7435675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc335679, 93 },
    { 0xa373567a, 34 },
    { 0x21c3567b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c35682, 33 },
    { 0, 0 },
    { 0xef635684, 33 },
    { 0, 0 },
    { 0xd9a35686, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3703568a, 95 },
    { 0x7a33568b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae35692, 61 },
    { 0xacb35693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49735696, 90 },
    { 0, 0 },
    { 0x7db35698, 34 },
    { 0, 0 },
    { 0xe3b3569a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4203569e, 33 },
    { 0xc3b3569f, 99 },
    { 0, 0 },
    { 0x42d356a1, 43 },
    { 0xd7356a1, 34 },
    { 0x55e356a3, 46 },
    { 0xa3f356a1, 84 },
    { 0, 0 },
    { 0x9d356a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2356b3, 77 },
    { 0, 0 },
    { 0xe43356b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1356bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f356bf, 95 },
    { 0xea7356c0, 23 },
    { 0x59c356c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c356d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x330356d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b356d8, 90 },
    { 0, 0 },
    { 0xcfb356da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0356dd, 33 },
    { 0xab8356de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee356e1, 34 },
    { 0x6ce356e1, 33 },
    { 0x600356e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd356e7, 34 },
    { 0x268356e7, 2 },
    { 0xbe0356e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a356ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x597356ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78356f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6356ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e35702, 90 },
    { 0x9d035703, 23 },
    { 0xaba35704, 90 },
    { 0, 0 },
    { 0x69c35706, 98 },
    { 0xd5035707, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa3570c, 34 },
    { 0xa643570d, 64 },
    { 0x3753570e, 90 },
    { 0x9f63570e, 88 },
    { 0x8dc3570e, 95 },
    { 0xf6f3570e, 90 },
    { 0x21735711, 90 },
    { 0x5a35713, 84 },
    { 0x39735713, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535718, 94 },
    { 0xd3a35718, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce73571c, 78 },
    { 0xb143571c, 99 },
    { 0, 0 },
    { 0xdd63571f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9035724, 90 },
    { 0x1ef35725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30435729, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13572c, 33 },
    { 0xf783572c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35732, 33 },
    { 0, 0 },
    { 0xaa835734, 55 },
    { 0, 0 },
    { 0xe7535736, 61 },
    { 0x6a035737, 85 },
    { 0x15435738, 77 },
    { 0x81435739, 98 },
    { 0, 0 },
    { 0x25d3573b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc3573f, 90 },
    { 0x66135740, 95 },
    { 0x36935741, 52 },
    { 0xf2a35742, 61 },
    { 0, 0 },
    { 0x2ac35744, 90 },
    { 0x23b35744, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3574a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c3574d, 76 },
    { 0xca13574e, 36 },
    { 0x6953574e, 23 },
    { 0xd723574f, 90 },
    { 0x50c3574f, 76 },
    { 0xecd35752, 85 },
    { 0xcca35753, 43 },
    { 0, 0 },
    { 0x6035755, 78 },
    { 0xd0835756, 33 },
    { 0, 0 },
    { 0x23f35758, 55 },
    { 0, 0 },
    { 0x1163575a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb813575e, 80 },
    { 0x9843575f, 66 },
    { 0x7843575e, 33 },
    { 0x593575f, 55 },
    { 0xa773575f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14135767, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b3576e, 46 },
    { 0xd293576e, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c35772, 90 },
    { 0, 0 },
    { 0xbde35774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37435778, 98 },
    { 0, 0 },
    { 0x61f3577a, 95 },
    { 0x5423577b, 34 },
    { 0x2cb3577a, 43 },
    { 0x9103577d, 43 },
    { 0, 0 },
    { 0x3473577f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21135782, 33 },
    { 0, 0 },
    { 0xb9a35784, 90 },
    { 0x4da35785, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x32235788, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c3578b, 90 },
    { 0, 0 },
    { 0x2943578d, 66 },
    { 0, 0 },
    { 0x4903578f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc435792, 46 },
    { 0xafa35793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb35799, 55 },
    { 0, 0 },
    { 0x8763579b, 95 },
    { 0xd233579c, 33 },
    { 0, 0 },
    { 0x67a3579e, 98 },
    { 0xfa33579e, 84 },
    { 0x1ca357a0, 33 },
    { 0xdb4357a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f357a5, 38 },
    { 0xcb2357a6, 61 },
    { 0, 0 },
    { 0x48d357a8, 78 },
    { 0x50c357a9, 34 },
    { 0x2ad357aa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf357af, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf357b3, 90 },
    { 0xb66357b3, 61 },
    { 0x785357b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3357ba, 34 },
    { 0, 0 },
    { 0xa0f357bc, 23 },
    { 0, 0 },
    { 0x2f2357be, 76 },
    { 0, 0 },
    { 0x23d357c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b357c6, 76 },
    { 0x36c357c7, 47 },
    { 0x6c357c7, 90 },
    { 0, 0 },
    { 0x351357ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93357cf, 23 },
    { 0x5d3357d0, 98 },
    { 0xa42357d0, 23 },
    { 0xa78357d0, 98 },
    { 0, 0 },
    { 0xf7e357d4, 75 },
    { 0, 0 },
    { 0xac0357d6, 23 },
    { 0x8a2357d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707357db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98357e0, 74 },
    { 0xd7c357e1, 66 },
    { 0x3c4357e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24357e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55357e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f357f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7357f6, 78 },
    { 0x904357f7, 76 },
    { 0, 0 },
    { 0x91a357f9, 43 },
    { 0x8c5357f9, 93 },
    { 0x94c357f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c835800, 61 },
    { 0x82335801, 40 },
    { 0x3d635802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf35805, 33 },
    { 0x25e35805, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa35812, 95 },
    { 0, 0 },
    { 0xebe35814, 90 },
    { 0xe2135815, 89 },
    { 0x57735816, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93581a, 90 },
    { 0, 0 },
    { 0x8943581c, 33 },
    { 0x9ef3581d, 76 },
    { 0x4ea3581e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88035823, 33 },
    { 0xdad35824, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74335829, 78 },
    { 0x29e3582a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5113582e, 91 },
    { 0x9603582f, 33 },
    { 0xafd35830, 55 },
    { 0xbbe35830, 90 },
    { 0x16135832, 8 },
    { 0x81135832, 23 },
    { 0xbd935834, 78 },
    { 0x9c335833, 33 },
    { 0x9be3582f, 90 },
    { 0x1435837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc173583b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec83583e, 90 },
    { 0xe8f3583f, 100 },
    { 0x2243583e, 98 },
    { 0xde3583f, 90 },
    { 0xe6b35842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50735847, 43 },
    { 0xb6735848, 98 },
    { 0, 0 },
    { 0x20a3584a, 91 },
    { 0x2c73584a, 33 },
    { 0xaf3584a, 76 },
    { 0, 0 },
    { 0x753584e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2535851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16035854, 61 },
    { 0x60435854, 33 },
    { 0, 0 },
    { 0xc1c35857, 53 },
    { 0, 0 },
    { 0xb9535859, 34 },
    { 0xec735859, 93 },
    { 0, 0 },
    { 0xc3e3585c, 33 },
    { 0x5503585c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd735868, 23 },
    { 0, 0 },
    { 0x6353586a, 43 },
    { 0, 0 },
    { 0xfae3586c, 61 },
    { 0xbf23586d, 95 },
    { 0x1d93586d, 61 },
    { 0x60c3586d, 61 },
    { 0, 0 },
    { 0xb6e35871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb35876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a635879, 61 },
    { 0xd7f3587a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74835880, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9035884, 90 },
    { 0x96835884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183588f, 76 },
    { 0x63c3588f, 23 },
    { 0xd2535891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed135897, 100 },
    { 0x19335898, 90 },
    { 0x60d35899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413589c, 80 },
    { 0, 0 },
    { 0x7643589e, 90 },
    { 0x77f3589f, 55 },
    { 0xb77358a0, 61 },
    { 0x997358a0, 93 },
    { 0x86a358a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x30358a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81358ad, 90 },
    { 0x112358ae, 93 },
    { 0x65a358ad, 85 },
    { 0, 0 },
    { 0xb19358b1, 46 },
    { 0x5fe358b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec358b6, 67 },
    { 0, 0 },
    { 0xdbf358b8, 82 },
    { 0x380358b9, 90 },
    { 0x83c358ba, 33 },
    { 0x9c0358ba, 90 },
    { 0x733358bc, 34 },
    { 0, 0 },
    { 0xb26358be, 93 },
    { 0xc09358bf, 75 },
    { 0, 0 },
    { 0x7cf358c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6358c7, 17 },
    { 0x8af358c7, 76 },
    { 0xae1358c9, 61 },
    { 0x659358c8, 93 },
    { 0xece358cb, 61 },
    { 0x45358c8, 23 },
    { 0, 0 },
    { 0xc43358ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904358d2, 95 },
    { 0x70358d3, 91 },
    { 0x60f358d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4358d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c358da, 43 },
    { 0xa65358db, 67 },
    { 0x540358dc, 34 },
    { 0x28b358dd, 43 },
    { 0x207358dd, 90 },
    { 0, 0 },
    { 0xf21358e0, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x939358e3, 61 },
    { 0x306358e4, 33 },
    { 0x40f358e4, 23 },
    { 0x318358e4, 52 },
    { 0x30f358e4, 90 },
    { 0x6b2358e8, 90 },
    { 0x144358e9, 95 },
    { 0x711358ea, 33 },
    { 0x5a358e8, 81 },
    { 0x161358ec, 33 },
    { 0x9f5358ec, 76 },
    { 0xac7358ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785358f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4358f9, 81 },
    { 0x792358f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1358ff, 95 },
    { 0x421358ff, 90 },
    { 0x11835900, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6635904, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a3590a, 61 },
    { 0, 0 },
    { 0xe3c3590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8423590f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4535916, 61 },
    { 0x41f35917, 61 },
    { 0xb2335918, 74 },
    { 0x9ba35919, 98 },
    { 0x7df35919, 61 },
    { 0x893591b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28535924, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b35928, 66 },
    { 0xa1d35929, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf3592c, 77 },
    { 0x2ad3592d, 87 },
    { 0x4b03592c, 23 },
    { 0, 0 },
    { 0x62035930, 90 },
    { 0x49435931, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad35936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2435939, 33 },
    { 0, 0 },
    { 0xdc73593b, 61 },
    { 0, 0 },
    { 0xe103593d, 61 },
    { 0, 0 },
    { 0x8643593f, 90 },
    { 0, 0 },
    { 0xf7a35941, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e35945, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5535949, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf043594c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68935952, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c35956, 93 },
    { 0x75b35957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3595d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b35961, 76 },
    { 0x50f35962, 93 },
    { 0xc2835963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5235967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d83596c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0835974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62635979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c3597d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd35981, 61 },
    { 0, 0 },
    { 0x49935983, 77 },
    { 0, 0 },
    { 0x6e135985, 61 },
    { 0x92835986, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa135991, 87 },
    { 0, 0 },
    { 0x12c35993, 84 },
    { 0xfd35994, 52 },
    { 0x2ed35995, 95 },
    { 0x81035995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7635999, 87 },
    { 0xa003599a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa283599d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9359a1, 95 },
    { 0xdd8359a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229359a8, 33 },
    { 0x9ea359a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371359af, 61 },
    { 0x289359af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac359b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa359bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f359bf, 33 },
    { 0x230359c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85359c3, 34 },
    { 0x6ec359c4, 90 },
    { 0x3c3359c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x992359c8, 81 },
    { 0xf7c359c8, 55 },
    { 0xef5359ca, 93 },
    { 0xd50359ca, 98 },
    { 0x666359cc, 81 },
    { 0xb3a359cc, 87 },
    { 0x7aa359ce, 33 },
    { 0, 0 },
    { 0x910359d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7359d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x161359d6, 90 },
    { 0x597359d7, 17 },
    { 0, 0 },
    { 0x29359d9, 91 },
    { 0, 0 },
    { 0xef8359db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f359e0, 98 },
    { 0x64a359e1, 61 },
    { 0xc62359e0, 88 },
    { 0x1e0359e3, 17 },
    { 0xc7359e3, 23 },
    { 0xea2359e4, 99 },
    { 0x424359e6, 61 },
    { 0, 0 },
    { 0xf2c359e8, 34 },
    { 0xfa9359e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc359ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63359f3, 74 },
    { 0xc85359f4, 95 },
    { 0x828359f4, 33 },
    { 0, 0 },
    { 0x26f359f7, 90 },
    { 0, 0 },
    { 0x275359f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8359ff, 34 },
    { 0, 0 },
    { 0xc4635a01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58435a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e35a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f135a19, 90 },
    { 0xcc435a1a, 78 },
    { 0xfa435a19, 53 },
    { 0x5b635a1c, 100 },
    { 0x34035a1d, 78 },
    { 0x39635a1d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db35a21, 76 },
    { 0x8b935a22, 90 },
    { 0, 0 },
    { 0xd5c35a24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b035a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d335a30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8435a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42035a3f, 90 },
    { 0x61535a3f, 66 },
    { 0x6a135a41, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d635a47, 81 },
    { 0x1ef35a48, 94 },
    { 0x1bf35a48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd35a4c, 23 },
    { 0xb7835a4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71935a53, 2 },
    { 0x4d835a53, 95 },
    { 0, 0 },
    { 0x7735a56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8335a5d, 98 },
    { 0x9035a5d, 23 },
    { 0xbd135a5f, 90 },
    { 0x94735a60, 95 },
    { 0x24e35a5e, 33 },
    { 0xcea35a62, 95 },
    { 0, 0 },
    { 0x60935a64, 95 },
    { 0x4c435a64, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea35a68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34735a76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c35a79, 46 },
    { 0xb6b35a7a, 93 },
    { 0xada35a7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd35a7e, 23 },
    { 0, 0 },
    { 0xefb35a80, 34 },
    { 0, 0 },
    { 0xbd035a82, 90 },
    { 0x80b35a83, 66 },
    { 0xca835a84, 33 },
    { 0x77d35a84, 99 },
    { 0x78a35a85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9635a8a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83735a8d, 61 },
    { 0xebc35a8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f35a91, 98 },
    { 0, 0 },
    { 0xe135a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9535a97, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb635a9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1035aa1, 90 },
    { 0, 0 },
    { 0x57d35aa3, 76 },
    { 0x50335aa4, 90 },
    { 0, 0 },
    { 0x9d635aa6, 55 },
    { 0x40335aa7, 95 },
    { 0x9b935aa7, 55 },
    { 0xf8d35aa9, 23 },
    { 0x69935aa8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d335ab0, 95 },
    { 0, 0 },
    { 0x52f35ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8835ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb35abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe35ac4, 98 },
    { 0x7bc35ac4, 65 },
    { 0xa5935ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f135ac9, 93 },
    { 0x84235ac9, 33 },
    { 0xced35aca, 95 },
    { 0x16435ac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc35ad4, 56 },
    { 0x2fb35ad4, 23 },
    { 0xc7035ad6, 98 },
    { 0, 0 },
    { 0xdb635ad8, 23 },
    { 0x50e35ad8, 55 },
    { 0xa0835ada, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b35add, 46 },
    { 0xacb35add, 66 },
    { 0xddf35ade, 76 },
    { 0xd2535ae0, 95 },
    { 0xe3135ae1, 93 },
    { 0xb5635ae2, 85 },
    { 0xb1c35ade, 76 },
    { 0xd7d35ae4, 23 },
    { 0x84235ae1, 66 },
    { 0x49135ade, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x60835ae9, 84 },
    { 0xee735aea, 84 },
    { 0, 0 },
    { 0x8d235aec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b35af0, 33 },
    { 0x14835af1, 33 },
    { 0x2f135af1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d35af6, 61 },
    { 0x76b35af6, 61 },
    { 0x2e435af8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c35afc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb35aff, 69 },
    { 0, 0 },
    { 0x8bd35b01, 55 },
    { 0xebe35b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xda335b05, 23 },
    { 0x7d235b06, 23 },
    { 0xdd835b07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc535b0b, 90 },
    { 0, 0 },
    { 0x3ef35b0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94135b12, 78 },
    { 0xbb435b13, 76 },
    { 0x53c35b13, 23 },
    { 0x5ac35b13, 46 },
    { 0x42935b16, 87 },
    { 0xd6c35b16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28135b1b, 80 },
    { 0, 0 },
    { 0xb6235b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d035b23, 61 },
    { 0x3d835b24, 90 },
    { 0xc7735b23, 23 },
    { 0, 0 },
    { 0xa3e35b27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf935b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d835b2e, 33 },
    { 0x30a35b2e, 87 },
    { 0x87c35b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba735b37, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96535b3a, 61 },
    { 0x7e135b3a, 43 },
    { 0, 0 },
    { 0xfc135b3d, 23 },
    { 0, 0 },
    { 0x36e35b3f, 55 },
    { 0xf1f35b3f, 33 },
    { 0, 0 },
    { 0x75835b42, 61 },
    { 0x4535b43, 33 },
    { 0xf8035b44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d35b48, 33 },
    { 0xc3435b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa35b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3235b52, 98 },
    { 0x5e635b53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac35b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb335b5c, 78 },
    { 0xec035b5c, 95 },
    { 0x50135b5e, 43 },
    { 0x6c835b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28835b65, 90 },
    { 0, 0 },
    { 0x55235b67, 77 },
    { 0xa9235b68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a35b6c, 34 },
    { 0, 0 },
    { 0x7dc35b6e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a35b77, 90 },
    { 0, 0 },
    { 0xd0635b79, 33 },
    { 0x9e835b79, 67 },
    { 0xf2435b79, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b35b7f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b35b83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d035b87, 76 },
    { 0, 0 },
    { 0xad235b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed735b8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83135b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b35b9d, 95 },
    { 0, 0 },
    { 0xb1d35b9f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f35ba6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b35ba9, 84 },
    { 0x4dd35ba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd35bb2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e35bb5, 95 },
    { 0xf2c35bb6, 23 },
    { 0x79f35bb7, 43 },
    { 0x85a35bb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9835bc1, 87 },
    { 0x10b35bc1, 61 },
    { 0x81f35bc3, 33 },
    { 0, 0 },
    { 0x56435bc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad535bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c35bd0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b35bd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb635bd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55235bdf, 90 },
    { 0x88735bdf, 55 },
    { 0x5e135be0, 84 },
    { 0x85f35bdf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6235bea, 33 },
    { 0x5ac35beb, 76 },
    { 0x77735beb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e935bf5, 33 },
    { 0x47a35bf6, 23 },
    { 0, 0 },
    { 0x71a35bf8, 61 },
    { 0, 0 },
    { 0x23a35bfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9635bfe, 55 },
    { 0, 0 },
    { 0x48935c00, 90 },
    { 0, 0 },
    { 0x7c935c02, 84 },
    { 0xe4c35c03, 76 },
    { 0xf635c02, 23 },
    { 0x1b335c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c35c09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82635c0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x25635c12, 77 },
    { 0x4e535c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf535c17, 46 },
    { 0x12135c18, 23 },
    { 0, 0 },
    { 0x3de35c1a, 23 },
    { 0, 0 },
    { 0xc035c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e135c20, 23 },
    { 0, 0 },
    { 0x5e335c22, 33 },
    { 0x2a635c23, 76 },
    { 0x56235c24, 61 },
    { 0xe3235c25, 46 },
    { 0xc435c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7135c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44135c2d, 90 },
    { 0x65735c2e, 95 },
    { 0xa2035c2d, 34 },
    { 0xa3735c2d, 52 },
    { 0x33f35c31, 66 },
    { 0xde035c32, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce35c35, 78 },
    { 0x5b435c36, 34 },
    { 0, 0 },
    { 0x17e35c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa35c3c, 34 },
    { 0xd8f35c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f35c41, 33 },
    { 0x1df35c42, 77 },
    { 0, 0 },
    { 0x38335c44, 61 },
    { 0xc8a35c45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff35c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e35c55, 33 },
    { 0x90635c56, 99 },
    { 0x9835c57, 61 },
    { 0x35435c58, 95 },
    { 0, 0 },
    { 0xc4635c5a, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38835c61, 90 },
    { 0x21c35c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e35c6e, 90 },
    { 0x3ec35c6f, 90 },
    { 0x63435c70, 40 },
    { 0xbde35c70, 76 },
    { 0xeef35c72, 17 },
    { 0, 0 },
    { 0x50835c74, 100 },
    { 0xdce35c75, 65 },
    { 0x63535c75, 34 },
    { 0, 0 },
    { 0x4e135c78, 23 },
    { 0x18335c79, 87 },
    { 0, 0 },
    { 0x86f35c7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f835c7e, 46 },
    { 0x9235c7f, 67 },
    { 0, 0 },
    { 0x74c35c81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced35c8a, 61 },
    { 0x3be35c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f35c8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a35c91, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a35c95, 66 },
    { 0x96535c96, 43 },
    { 0x8e935c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c35c9a, 34 },
    { 0xf0d35c9b, 90 },
    { 0x3a135c9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a35ca3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1235ca6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c35ca9, 76 },
    { 0, 0 },
    { 0x8b235cab, 33 },
    { 0, 0 },
    { 0xa635cad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c335cb2, 78 },
    { 0, 0 },
    { 0x12235cb4, 91 },
    { 0x25635cb5, 95 },
    { 0, 0 },
    { 0x51835cb7, 34 },
    { 0xf6a35cb8, 76 },
    { 0x94035cb9, 33 },
    { 0x5a435cb9, 88 },
    { 0x8ba35cbb, 88 },
    { 0x18435cbc, 93 },
    { 0, 0 },
    { 0x8fe35cbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a35cc4, 91 },
    { 0xc7c35cc4, 33 },
    { 0xbc335cc5, 95 },
    { 0xf7935cc7, 95 },
    { 0x36d35cc8, 66 },
    { 0x7635cc9, 33 },
    { 0x12f35cc5, 43 },
    { 0xe0e35cc6, 95 },
    { 0x7c835ccc, 34 },
    { 0xa8e35ccd, 17 },
    { 0x96f35cce, 76 },
    { 0x69735cc5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b235cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c35cd8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5135cdc, 98 },
    { 0xf7a35cdd, 95 },
    { 0x35935cde, 87 },
    { 0x65f35cde, 67 },
    { 0x2e535cdd, 78 },
    { 0xa8f35cdd, 33 },
    { 0xae835ce2, 90 },
    { 0x72535ce3, 99 },
    { 0, 0 },
    { 0xa8435ce5, 95 },
    { 0x9ed35ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd35ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a35cf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6635cf5, 93 },
    { 0, 0 },
    { 0xbbc35cf7, 33 },
    { 0xa235cf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f35cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea35d03, 98 },
    { 0x3f935d04, 61 },
    { 0xf3f35d05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e35d08, 33 },
    { 0x66f35d08, 43 },
    { 0, 0 },
    { 0x63d35d0b, 43 },
    { 0, 0 },
    { 0xf5d35d0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a35d11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c035d14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44435d18, 61 },
    { 0xf4a35d18, 98 },
    { 0x29f35d18, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57735d22, 65 },
    { 0, 0 },
    { 0x20135d24, 95 },
    { 0x26635d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef35d28, 76 },
    { 0x6ab35d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96335d2e, 34 },
    { 0x1f635d2e, 91 },
    { 0x81c35d2e, 91 },
    { 0, 0 },
    { 0xc8b35d32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e35d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f35d3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e35d42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa35d45, 100 },
    { 0xcd035d46, 23 },
    { 0x23835d47, 43 },
    { 0, 0 },
    { 0xe9835d49, 90 },
    { 0x92135d49, 95 },
    { 0x9a435d49, 84 },
    { 0, 0 },
    { 0x9bf35d4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a35d52, 95 },
    { 0xbe535d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61335d57, 76 },
    { 0xd9e35d57, 93 },
    { 0x1a435d57, 90 },
    { 0x50a35d58, 88 },
    { 0, 0 },
    { 0x72135d5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62335d5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61335d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc35d6c, 76 },
    { 0x9ba35d6d, 34 },
    { 0xa8135d6c, 61 },
    { 0xb3535d6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76135d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2835d77, 90 },
    { 0xf9935d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34735d80, 33 },
    { 0xb8335d81, 55 },
    { 0x51135d82, 76 },
    { 0x3b035d82, 100 },
    { 0, 0 },
    { 0x6e635d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b35d89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a35d8e, 98 },
    { 0, 0 },
    { 0xcee35d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab35d93, 90 },
    { 0xb7f35d93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd35d97, 98 },
    { 0x60935d97, 90 },
    { 0xf8035d99, 98 },
    { 0x6a435d9a, 68 },
    { 0x90d35d9a, 23 },
    { 0, 0 },
    { 0x2ee35d9d, 95 },
    { 0xdf635d9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f35da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0035dab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62935daf, 99 },
    { 0xcf635daf, 98 },
    { 0x43d35db0, 90 },
    { 0xcb35db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da35db5, 92 },
    { 0x43435db5, 90 },
    { 0xe1f35db5, 95 },
    { 0, 0 },
    { 0x14235db9, 90 },
    { 0x6a035dba, 33 },
    { 0x8a235dba, 90 },
    { 0x5f535dbc, 75 },
    { 0xb9535dbd, 68 },
    { 0x8eb35dbe, 78 },
    { 0xb2c35dbf, 55 },
    { 0, 0 },
    { 0x1aa35dc1, 52 },
    { 0x29835dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9035dc5, 95 },
    { 0xb135dc6, 33 },
    { 0xf1735dc5, 33 },
    { 0, 0 },
    { 0x7ec35dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52735dcc, 34 },
    { 0xd7035dcd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92735dd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf35ddb, 33 },
    { 0x43135ddc, 33 },
    { 0, 0 },
    { 0xc0335dde, 33 },
    { 0x39735ddf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48635de2, 23 },
    { 0xa9635de2, 95 },
    { 0xe8235de4, 67 },
    { 0xad335de2, 95 },
    { 0, 0 },
    { 0x7b435de7, 87 },
    { 0x4b735de8, 84 },
    { 0x90835de9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c035df2, 68 },
    { 0x51535df3, 67 },
    { 0x13135df2, 61 },
    { 0x64a35df5, 33 },
    { 0x5b935df6, 98 },
    { 0x16e35df7, 94 },
    { 0x55235df7, 33 },
    { 0xc7035df8, 90 },
    { 0x8c635dfa, 78 },
    { 0x78235dfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc35e00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f35e06, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34135e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd35e10, 55 },
    { 0x55535e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa35e14, 90 },
    { 0xa6d35e15, 92 },
    { 0x4bd35e15, 33 },
    { 0, 0 },
    { 0xd1b35e18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd235e1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c735e23, 61 },
    { 0x1d835e23, 83 },
    { 0xab935e25, 98 },
    { 0xcd735e23, 23 },
    { 0x51335e24, 81 },
    { 0x98235e25, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf35e30, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f135e38, 66 },
    { 0x30a35e39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd335e3c, 33 },
    { 0, 0 },
    { 0x49835e3e, 33 },
    { 0x5b935e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a035e49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f035e4c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed35e50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88735e5a, 33 },
    { 0x49c35e5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1535e5e, 95 },
    { 0, 0 },
    { 0x95d35e60, 90 },
    { 0, 0 },
    { 0xe6935e62, 82 },
    { 0xf5f35e63, 95 },
    { 0xd1035e64, 98 },
    { 0x47535e65, 90 },
    { 0x98435e63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13535e6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9335e6f, 52 },
    { 0x4e35e70, 43 },
    { 0xed035e6f, 61 },
    { 0xd8835e72, 61 },
    { 0x53335e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f035e76, 90 },
    { 0, 0 },
    { 0x26735e78, 55 },
    { 0x5d035e78, 61 },
    { 0x85c35e79, 61 },
    { 0x98735e7b, 90 },
    { 0xe0f35e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c35e83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b135e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d35e8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f35e91, 33 },
    { 0, 0 },
    { 0x8b35e93, 55 },
    { 0x16a35e94, 99 },
    { 0, 0 },
    { 0x9d135e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7635e9e, 52 },
    { 0x6c535e9f, 17 },
    { 0x39535ea0, 81 },
    { 0x63335ea1, 85 },
    { 0x13a35ea2, 77 },
    { 0xf2135e9e, 34 },
    { 0x20b35ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0435ea7, 98 },
    { 0xf6f35ea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2435eab, 46 },
    { 0xca735eac, 99 },
    { 0x53b35ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65235eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd35eb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e35eb7, 87 },
    { 0, 0 },
    { 0x5db35eb9, 95 },
    { 0xd9a35eba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd435ebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435ec1, 97 },
    { 0xfdf35ec2, 61 },
    { 0, 0 },
    { 0xe6335ec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf35ec9, 33 },
    { 0x62835ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64635ecd, 55 },
    { 0, 0 },
    { 0xb2635ecf, 33 },
    { 0xf635ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93535ed3, 33 },
    { 0x1e235ed4, 98 },
    { 0x7b835ed5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e535ed9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea35ede, 93 },
    { 0xd9335edf, 95 },
    { 0x76835ede, 95 },
    { 0, 0 },
    { 0x75a35ee2, 89 },
    { 0x41135ee2, 87 },
    { 0x21a35ee4, 61 },
    { 0x65b35ee3, 66 },
    { 0x4bb35ee4, 93 },
    { 0xfc635ee7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b235ef1, 34 },
    { 0x1dd35ef1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45035ef9, 98 },
    { 0x1bc35ef9, 85 },
    { 0xf4035efb, 61 },
    { 0x15635efc, 90 },
    { 0x9aa35efd, 98 },
    { 0xd4935efd, 93 },
    { 0x6a635efd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab035f03, 91 },
    { 0x7e435f04, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8535f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d35f0b, 23 },
    { 0, 0 },
    { 0xe2c35f0d, 95 },
    { 0x3eb35f0e, 76 },
    { 0xbcf35f0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d335f12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3735f15, 34 },
    { 0x21c35f15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b35f19, 92 },
    { 0x69835f19, 98 },
    { 0xe8535f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97035f1e, 90 },
    { 0x6a835f1f, 99 },
    { 0x335f20, 98 },
    { 0x95535f21, 78 },
    { 0xe8b35f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5835f25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3235f2a, 61 },
    { 0xef535f2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535f2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1335f32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48735f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc535f3c, 90 },
    { 0xf6335f3d, 95 },
    { 0, 0 },
    { 0x51435f3f, 65 },
    { 0, 0 },
    { 0x54735f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b35f49, 61 },
    { 0xf4235f49, 61 },
    { 0, 0 },
    { 0x51e35f4c, 99 },
    { 0x27735f4d, 33 },
    { 0, 0 },
    { 0xcf335f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71635f53, 23 },
    { 0x2fd35f53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9035f58, 98 },
    { 0xc35f59, 98 },
    { 0x9f935f58, 61 },
    { 0x51f35f5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9235f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa235f64, 34 },
    { 0xd5135f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf35f69, 91 },
    { 0x43635f6a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a35f73, 61 },
    { 0x80c35f73, 80 },
    { 0x5f735f73, 64 },
    { 0, 0 },
    { 0x5a235f77, 68 },
    { 0x3ba35f78, 23 },
    { 0xa2235f79, 34 },
    { 0x93435f77, 34 },
    { 0, 0 },
    { 0x2a035f7c, 98 },
    { 0xafe35f7d, 90 },
    { 0x45e35f7e, 77 },
    { 0xdd335f7f, 90 },
    { 0x83335f80, 93 },
    { 0x9dc35f81, 90 },
    { 0, 0 },
    { 0x6b535f83, 84 },
    { 0x1235f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e35f87, 40 },
    { 0x98e35f87, 76 },
    { 0x39535f87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70135f8f, 33 },
    { 0xfac35f8f, 34 },
    { 0x46035f91, 90 },
    { 0x6cf35f92, 95 },
    { 0x48935f92, 98 },
    { 0x54135f94, 61 },
    { 0, 0 },
    { 0xf1135f96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b835f9a, 95 },
    { 0xd1f35f9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82835fa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a035fa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b35fac, 95 },
    { 0x5fa35fad, 61 },
    { 0, 0 },
    { 0x25135faf, 95 },
    { 0x34c35fb0, 87 },
    { 0, 0 },
    { 0xfc635fb2, 61 },
    { 0xcfc35fb2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40735fbb, 65 },
    { 0x74e35fbb, 33 },
    { 0, 0 },
    { 0x4c535fbe, 87 },
    { 0xd2735fbf, 23 },
    { 0xa2135fc0, 5 },
    { 0, 0 },
    { 0x4b735fc2, 23 },
    { 0x8cd35fc3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a35fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c035fce, 64 },
    { 0x23235fcf, 33 },
    { 0x3c435fd0, 61 },
    { 0xe2f35fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35835fd4, 23 },
    { 0x97835fd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38635fdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd35fde, 76 },
    { 0x7cf35fde, 95 },
    { 0xbf135fe0, 33 },
    { 0, 0 },
    { 0x4d535fe2, 91 },
    { 0, 0 },
    { 0x26035fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53535fee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b935ff1, 83 },
    { 0x6fe35ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b35ff8, 33 },
    { 0x87e35ff8, 43 },
    { 0, 0 },
    { 0xad435ffb, 99 },
    { 0xf2b35ffc, 40 },
    { 0xf135ffd, 52 },
    { 0xd6435ffe, 90 },
    { 0x3f535fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5036004, 59 },
    { 0xb5736005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec36008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b3600c, 91 },
    { 0, 0 },
    { 0x4223600e, 76 },
    { 0xbb73600f, 87 },
    { 0x3ef36010, 66 },
    { 0x21c3600e, 34 },
    { 0x2a736012, 33 },
    { 0xd8c36012, 77 },
    { 0xa6536011, 84 },
    { 0, 0 },
    { 0x51536016, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9036019, 61 },
    { 0, 0 },
    { 0x2f93601b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf136021, 46 },
    { 0x58236022, 64 },
    { 0xd0d36021, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8493602a, 46 },
    { 0x2c53602b, 87 },
    { 0x4253602a, 95 },
    { 0x5793602d, 33 },
    { 0, 0 },
    { 0x48c3602f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b036035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f3603a, 43 },
    { 0xa5b3603b, 52 },
    { 0xe8a3603b, 99 },
    { 0, 0 },
    { 0xa583603e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e36041, 43 },
    { 0x6c836041, 90 },
    { 0, 0 },
    { 0x33936044, 95 },
    { 0, 0 },
    { 0x30a36046, 77 },
    { 0, 0 },
    { 0x34736048, 43 },
    { 0xc6c36049, 78 },
    { 0xc0936049, 33 },
    { 0, 0 },
    { 0x6943604c, 34 },
    { 0xe4c3604d, 95 },
    { 0, 0 },
    { 0x6813604f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b36052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f36055, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5936059, 38 },
    { 0, 0 },
    { 0xcee3605b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d236062, 95 },
    { 0x48736063, 90 },
    { 0, 0 },
    { 0x4bb36065, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69336069, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53606d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd36070, 33 },
    { 0x4c336071, 23 },
    { 0xc3b36072, 33 },
    { 0, 0 },
    { 0x40836074, 33 },
    { 0xfc236074, 33 },
    { 0x47236075, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3607b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b3607f, 84 },
    { 0x99136080, 17 },
    { 0x3da3607f, 33 },
    { 0x82a36080, 23 },
    { 0xb4536083, 67 },
    { 0, 0 },
    { 0x2ab36085, 84 },
    { 0xe2a36086, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb136093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc36096, 43 },
    { 0x76e36097, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999360a0, 61 },
    { 0xd1e360a1, 100 },
    { 0xb57360a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e360a9, 81 },
    { 0, 0 },
    { 0x2de360ab, 55 },
    { 0, 0 },
    { 0xeb1360ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837360b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x491360b9, 46 },
    { 0xec360ba, 46 },
    { 0xc34360bb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506360bf, 78 },
    { 0x2bf360c0, 95 },
    { 0x18d360c1, 85 },
    { 0, 0 },
    { 0x810360c3, 90 },
    { 0, 0 },
    { 0xdf1360c5, 77 },
    { 0, 0 },
    { 0xd4c360c7, 46 },
    { 0x2c7360c7, 17 },
    { 0xa25360c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x828360cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c360d4, 55 },
    { 0, 0 },
    { 0xf0d360d6, 33 },
    { 0, 0 },
    { 0xaf360d8, 95 },
    { 0x1c3360d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b360dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c360e2, 61 },
    { 0, 0 },
    { 0x223360e4, 55 },
    { 0xf2c360e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x259360e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2360ef, 90 },
    { 0xfe3360f0, 61 },
    { 0, 0 },
    { 0xe58360f2, 76 },
    { 0x634360f3, 75 },
    { 0, 0 },
    { 0x956360f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98360f9, 61 },
    { 0x20a360f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8360fd, 55 },
    { 0xe9a360fe, 98 },
    { 0, 0 },
    { 0xdac36100, 98 },
    { 0xf3e36101, 34 },
    { 0xc4336101, 76 },
    { 0x1b036103, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x98636106, 33 },
    { 0x1c036107, 55 },
    { 0, 0 },
    { 0x26f36109, 77 },
    { 0x27b3610a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee36110, 55 },
    { 0x50136110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8636114, 34 },
    { 0, 0 },
    { 0x4c436116, 68 },
    { 0x8a136116, 61 },
    { 0, 0 },
    { 0xbe636119, 95 },
    { 0, 0 },
    { 0xb1e3611b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d03611e, 88 },
    { 0xbdd3611f, 80 },
    { 0xc6b36120, 43 },
    { 0xb103611f, 33 },
    { 0xcfd36122, 90 },
    { 0x39936121, 91 },
    { 0x103611f, 61 },
    { 0x54436120, 99 },
    { 0x9d336123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab93612b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb53612f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236132, 90 },
    { 0, 0 },
    { 0xfa836134, 95 },
    { 0, 0 },
    { 0xbc436136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33613b, 61 },
    { 0xf763613b, 100 },
    { 0x9c73613c, 99 },
    { 0, 0 },
    { 0xbbc3613f, 24 },
    { 0xdaf36140, 76 },
    { 0x93036140, 98 },
    { 0xb6e36141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9213614e, 90 },
    { 0, 0 },
    { 0xbb636150, 84 },
    { 0, 0 },
    { 0x23a36152, 33 },
    { 0, 0 },
    { 0xba436154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3615b, 55 },
    { 0x3e73615c, 93 },
    { 0x5f53615d, 100 },
    { 0x5b83615e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7036161, 55 },
    { 0, 0 },
    { 0x4cf36163, 34 },
    { 0xcbb36163, 23 },
    { 0x11036164, 77 },
    { 0xcff36166, 90 },
    { 0x53b36164, 90 },
    { 0x92b36164, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf663616d, 98 },
    { 0xd7c3616d, 52 },
    { 0x3333616f, 61 },
    { 0xf3036170, 78 },
    { 0xcde36171, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac36174, 23 },
    { 0, 0 },
    { 0x5e136176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd3617b, 40 },
    { 0xaf63617c, 33 },
    { 0, 0 },
    { 0x5f93617e, 33 },
    { 0, 0 },
    { 0x9bd36180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e36183, 61 },
    { 0, 0 },
    { 0xddf36185, 23 },
    { 0xc9b36186, 33 },
    { 0x73f36187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013618a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1013618e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd36191, 70 },
    { 0x8da36192, 33 },
    { 0xda336191, 67 },
    { 0x58936194, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c3619d, 46 },
    { 0x8a73619d, 55 },
    { 0xfe53619f, 99 },
    { 0x31c3619f, 95 },
    { 0xed13619e, 43 },
    { 0x95a361a2, 90 },
    { 0, 0 },
    { 0xb10361a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08361ab, 61 },
    { 0x8cb361ac, 68 },
    { 0xade361ad, 61 },
    { 0x14361ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43361b1, 52 },
    { 0xb08361b2, 76 },
    { 0x655361b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x946361b6, 76 },
    { 0x833361b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3361bb, 33 },
    { 0, 0 },
    { 0x7dc361bd, 61 },
    { 0x762361be, 95 },
    { 0, 0 },
    { 0xee1361c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3361c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89361cc, 27 },
    { 0, 0 },
    { 0x13b361ce, 90 },
    { 0, 0 },
    { 0xbcf361d0, 95 },
    { 0xb0b361d1, 77 },
    { 0x454361d2, 90 },
    { 0x299361d1, 34 },
    { 0, 0 },
    { 0x5da361d5, 46 },
    { 0, 0 },
    { 0x525361d7, 23 },
    { 0xae7361d8, 90 },
    { 0x71b361d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d361dd, 61 },
    { 0x911361de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1361e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0361e5, 100 },
    { 0, 0 },
    { 0x8af361e7, 95 },
    { 0x859361e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28361f4, 90 },
    { 0xd4e361f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113361f9, 95 },
    { 0xcd8361fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a36206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43620d, 90 },
    { 0, 0 },
    { 0x9993620f, 98 },
    { 0, 0 },
    { 0xbbb36211, 95 },
    { 0xf3436212, 23 },
    { 0x9a336212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c36216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1013621d, 77 },
    { 0, 0 },
    { 0xe773621f, 61 },
    { 0x26936220, 61 },
    { 0xc5836221, 77 },
    { 0x37d36221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d3622c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8933622f, 95 },
    { 0, 0 },
    { 0xdd836231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82236236, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd36239, 23 },
    { 0, 0 },
    { 0xbfb3623b, 77 },
    { 0, 0 },
    { 0xd753623d, 90 },
    { 0, 0 },
    { 0x5f13623f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78836242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b36246, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b3624b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde36251, 40 },
    { 0x61136252, 76 },
    { 0x89136253, 55 },
    { 0x8cd36251, 34 },
    { 0x24d36255, 91 },
    { 0, 0 },
    { 0x7f336257, 61 },
    { 0xca536258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e3625c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5636261, 90 },
    { 0x26d36261, 98 },
    { 0xd4736263, 49 },
    { 0, 0 },
    { 0x6df36265, 99 },
    { 0x4f536265, 33 },
    { 0xf5836265, 87 },
    { 0x31636268, 23 },
    { 0x30536269, 78 },
    { 0xc5436266, 43 },
    { 0, 0 },
    { 0xc9f3626c, 92 },
    { 0x313626c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c36272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4336275, 76 },
    { 0, 0 },
    { 0x73536277, 95 },
    { 0x89936277, 90 },
    { 0x5d36279, 33 },
    { 0xf8d36279, 90 },
    { 0xce23627b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef336286, 91 },
    { 0xf4136286, 56 },
    { 0x3036288, 40 },
    { 0x9e236289, 95 },
    { 0xafa36289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac3628d, 93 },
    { 0, 0 },
    { 0xfdd3628f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d236294, 33 },
    { 0, 0 },
    { 0x86836296, 23 },
    { 0x61636297, 33 },
    { 0xa1536296, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3603629b, 90 },
    { 0, 0 },
    { 0x6a73629d, 98 },
    { 0x3d3629e, 90 },
    { 0xd1f3629e, 98 },
    { 0xe85362a0, 33 },
    { 0x3be362a0, 52 },
    { 0xe2362a0, 34 },
    { 0x8f5362a3, 95 },
    { 0xab6362a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca362a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d362b2, 33 },
    { 0xb20362b2, 98 },
    { 0xe91362b3, 66 },
    { 0x15a362b5, 34 },
    { 0x510362b2, 38 },
    { 0x4a0362b5, 90 },
    { 0, 0 },
    { 0x608362b9, 56 },
    { 0xee8362b9, 95 },
    { 0x4f9362bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09362be, 23 },
    { 0, 0 },
    { 0xc22362c0, 100 },
    { 0, 0 },
    { 0x5c2362c2, 88 },
    { 0, 0 },
    { 0x98f362c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32362c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22362ca, 33 },
    { 0xd16362cb, 81 },
    { 0xe25362cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x716362cf, 95 },
    { 0, 0 },
    { 0x18c362d1, 23 },
    { 0xd9362d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33362d9, 52 },
    { 0, 0 },
    { 0xe17362db, 53 },
    { 0, 0 },
    { 0x6a362dd, 78 },
    { 0, 0 },
    { 0x5a7362df, 61 },
    { 0, 0 },
    { 0xfef362e1, 46 },
    { 0xc2362e2, 91 },
    { 0x723362e3, 33 },
    { 0x65a362e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e362ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7362ef, 3 },
    { 0, 0 },
    { 0xb14362f1, 88 },
    { 0x294362f1, 78 },
    { 0xbb1362f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe362f6, 61 },
    { 0x8b3362f7, 34 },
    { 0x234362f7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc362fc, 61 },
    { 0, 0 },
    { 0xb65362fe, 95 },
    { 0x91f362ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57636302, 33 },
    { 0xb4936303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3630b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f13630e, 85 },
    { 0x87b3630f, 98 },
    { 0xb343630e, 31 },
    { 0xcb536311, 95 },
    { 0xf623630f, 33 },
    { 0, 0 },
    { 0xba636314, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd253631c, 95 },
    { 0xd9f3631d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec36321, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97436328, 88 },
    { 0x9cf36329, 90 },
    { 0x1c63632a, 61 },
    { 0x44036328, 61 },
    { 0x93c3632a, 75 },
    { 0, 0 },
    { 0xebe3632e, 66 },
    { 0x52f3632e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3936335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e36340, 43 },
    { 0x12a36340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0536344, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3433634e, 23 },
    { 0x7df3634f, 46 },
    { 0x73d3634e, 99 },
    { 0, 0 },
    { 0x72336352, 90 },
    { 0xf136353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9136360, 90 },
    { 0, 0 },
    { 0x2e736362, 84 },
    { 0x90c36363, 33 },
    { 0x2f436364, 90 },
    { 0xa6236362, 33 },
    { 0, 0 },
    { 0xa0636367, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3636a, 33 },
    { 0xf413636b, 34 },
    { 0xb463636c, 90 },
    { 0x49d3636c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9636371, 98 },
    { 0, 0 },
    { 0x5b36373, 90 },
    { 0x5c536374, 52 },
    { 0x54f36375, 55 },
    { 0x73536375, 33 },
    { 0xcfb36377, 61 },
    { 0x1be36374, 87 },
    { 0x35336378, 92 },
    { 0xecb36373, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903637f, 61 },
    { 0xcb436380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65936389, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7203638f, 98 },
    { 0x71b36390, 61 },
    { 0xb4d36390, 61 },
    { 0x9a536390, 55 },
    { 0, 0 },
    { 0xf6036394, 68 },
    { 0x31336394, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3639a, 95 },
    { 0x6583639b, 76 },
    { 0x8c93639c, 77 },
    { 0x7fe3639d, 90 },
    { 0x8053639e, 34 },
    { 0, 0 },
    { 0xa01363a0, 95 },
    { 0x68363a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4363a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b363a9, 90 },
    { 0xd48363aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11363b0, 68 },
    { 0xb8363b1, 87 },
    { 0, 0 },
    { 0x442363b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3363b6, 61 },
    { 0x7a7363b6, 8 },
    { 0x989363b6, 95 },
    { 0, 0 },
    { 0xb56363ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed363bf, 33 },
    { 0xcf9363bf, 80 },
    { 0x664363c1, 98 },
    { 0, 0 },
    { 0x1e3363c3, 68 },
    { 0xa7a363c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1363c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8363d0, 23 },
    { 0x675363d1, 90 },
    { 0, 0 },
    { 0xda363d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x156363d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875363e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b363e3, 98 },
    { 0xa9f363e4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745363ea, 93 },
    { 0x4ea363eb, 33 },
    { 0x13c363eb, 46 },
    { 0xce4363eb, 95 },
    { 0x31a363ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9363f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff363f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9363fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590363ff, 93 },
    { 0x6d2363ff, 90 },
    { 0xa9a36401, 93 },
    { 0xd1936402, 61 },
    { 0xfc436401, 61 },
    { 0x27636402, 76 },
    { 0xc4b36405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b636418, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2353641b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8013641f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x39836422, 23 },
    { 0xf7136423, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e36426, 76 },
    { 0x5cd36427, 90 },
    { 0x34a36428, 98 },
    { 0x1cb36427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03642d, 98 },
    { 0x3e3642e, 61 },
    { 0, 0 },
    { 0x4b736430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb236436, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86836439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153643d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40536442, 34 },
    { 0x5136443, 85 },
    { 0xd0f36444, 95 },
    { 0x5c136445, 85 },
    { 0x51936445, 100 },
    { 0xa9b36443, 76 },
    { 0xc4c36448, 36 },
    { 0xbc236445, 99 },
    { 0x2013644a, 33 },
    { 0x53a3644b, 93 },
    { 0x2763644c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17736451, 95 },
    { 0x7e636452, 76 },
    { 0x5e936452, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d436456, 33 },
    { 0x71736457, 95 },
    { 0, 0 },
    { 0x45936459, 23 },
    { 0xf6b3645a, 78 },
    { 0xe183645b, 84 },
    { 0x9793645b, 91 },
    { 0, 0 },
    { 0x9bb3645e, 98 },
    { 0x17b3645f, 77 },
    { 0x49a36460, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5436463, 61 },
    { 0x8dc36464, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d3646a, 46 },
    { 0xeb3646b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a336471, 90 },
    { 0x1636472, 43 },
    { 0xd0936471, 90 },
    { 0, 0 },
    { 0x6c436475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5936480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52836489, 33 },
    { 0xe113648a, 65 },
    { 0xb8736489, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1836490, 38 },
    { 0x45536490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4553649b, 95 },
    { 0xbcf3649c, 100 },
    { 0, 0 },
    { 0x2473649e, 90 },
    { 0x6f3649f, 91 },
    { 0, 0 },
    { 0x450364a1, 76 },
    { 0x382364a1, 23 },
    { 0, 0 },
    { 0x6e7364a4, 66 },
    { 0x541364a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf364a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x367364ab, 95 },
    { 0, 0 },
    { 0x41c364ad, 76 },
    { 0x50a364ae, 77 },
    { 0x379364af, 55 },
    { 0xa62364af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65364b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb364b7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x189364ba, 76 },
    { 0xda6364ba, 95 },
    { 0x489364bc, 33 },
    { 0, 0 },
    { 0x1db364be, 76 },
    { 0, 0 },
    { 0xed0364c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361364c3, 84 },
    { 0xa8364c4, 85 },
    { 0x615364c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4364c8, 76 },
    { 0x817364c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa364cd, 100 },
    { 0xce0364cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3364d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604364d5, 33 },
    { 0, 0 },
    { 0xa1e364d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x438364da, 76 },
    { 0x695364db, 61 },
    { 0x27c364dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6364e0, 87 },
    { 0, 0 },
    { 0x594364e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7364e7, 98 },
    { 0x8a7364e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5364eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda364ee, 61 },
    { 0, 0 },
    { 0xaeb364f0, 90 },
    { 0x9a5364f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9364fb, 61 },
    { 0x932364fc, 98 },
    { 0x490364fd, 34 },
    { 0xd41364fc, 23 },
    { 0x8fb364ff, 33 },
    { 0x20d36500, 90 },
    { 0x8536501, 55 },
    { 0xca136502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc36507, 76 },
    { 0, 0 },
    { 0x9f136509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3650c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3650f, 61 },
    { 0, 0 },
    { 0x92d36511, 34 },
    { 0x76936511, 61 },
    { 0xc2436513, 23 },
    { 0x91136514, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2936519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd3651f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb36523, 43 },
    { 0xe2c36524, 23 },
    { 0, 0 },
    { 0x1fb36526, 23 },
    { 0x6f936527, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363652a, 67 },
    { 0, 0 },
    { 0x5a53652c, 87 },
    { 0xef03652d, 33 },
    { 0x2433652e, 88 },
    { 0, 0 },
    { 0x6f936530, 90 },
    { 0xe5f36530, 96 },
    { 0xa1436532, 33 },
    { 0, 0 },
    { 0x62736534, 22 },
    { 0x17f36535, 78 },
    { 0x92436536, 90 },
    { 0x38336536, 33 },
    { 0xd4536535, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4253653b, 33 },
    { 0x2813653c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2433653f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd36543, 98 },
    { 0x32a36544, 90 },
    { 0x85a36544, 65 },
    { 0x63536546, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb36550, 76 },
    { 0xd5a36550, 95 },
    { 0x23536550, 33 },
    { 0x52c36553, 23 },
    { 0x59e36554, 90 },
    { 0xa436555, 77 },
    { 0x56836553, 23 },
    { 0x16636557, 34 },
    { 0x42f36551, 85 },
    { 0, 0 },
    { 0x4303655a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3136562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d3656f, 61 },
    { 0x6ec36570, 67 },
    { 0, 0 },
    { 0xe2e36572, 98 },
    { 0x6aa36573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9236576, 95 },
    { 0x38136577, 76 },
    { 0xf6c36577, 76 },
    { 0x50136579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e336581, 99 },
    { 0, 0 },
    { 0x36536583, 99 },
    { 0xcf436583, 55 },
    { 0xb0336585, 33 },
    { 0, 0 },
    { 0x17c36587, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453658b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53658e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe636596, 98 },
    { 0, 0 },
    { 0xe3236598, 33 },
    { 0x9a936599, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e43659e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66365a1, 90 },
    { 0xaa2365a1, 33 },
    { 0, 0 },
    { 0xe93365a4, 93 },
    { 0x36f365a5, 78 },
    { 0xb4a365a4, 23 },
    { 0xddc365a5, 23 },
    { 0, 0 },
    { 0x2e4365a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81365af, 33 },
    { 0x17a365b0, 100 },
    { 0xa4c365b1, 55 },
    { 0x46e365b2, 90 },
    { 0xe7365b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55365b6, 98 },
    { 0, 0 },
    { 0xab7365b8, 97 },
    { 0, 0 },
    { 0x725365ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7365bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b365c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7365c6, 90 },
    { 0, 0 },
    { 0x269365c8, 68 },
    { 0, 0 },
    { 0x43a365ca, 46 },
    { 0xd4365ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x428365ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02365d2, 61 },
    { 0x38d365d3, 85 },
    { 0x83c365d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671365d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97365e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad365e3, 33 },
    { 0x690365e4, 76 },
    { 0x1f0365e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede365e9, 61 },
    { 0x67f365e9, 90 },
    { 0, 0 },
    { 0x4e3365ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79365ef, 29 },
    { 0xd4d365f0, 61 },
    { 0x8e8365f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f365f5, 33 },
    { 0, 0 },
    { 0x496365f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6365ff, 99 },
    { 0x6a2365ff, 76 },
    { 0xf6a36600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f436606, 34 },
    { 0x7a936607, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3660a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee3660d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80136615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc53661b, 98 },
    { 0x21a3661b, 98 },
    { 0xb253661d, 98 },
    { 0x9d53661e, 55 },
    { 0, 0 },
    { 0x3bc36620, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd536627, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4823662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753662e, 85 },
    { 0x1f13662f, 90 },
    { 0x28d3662e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe833663c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b3663f, 98 },
    { 0x4b53663f, 61 },
    { 0x7f636641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c036645, 34 },
    { 0x3336646, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb036649, 90 },
    { 0x2853664a, 33 },
    { 0x52b36649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99736651, 78 },
    { 0, 0 },
    { 0xa2a36653, 52 },
    { 0x98c36654, 90 },
    { 0, 0 },
    { 0x58a36656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b236659, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3665c, 90 },
    { 0xe4d3665d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2636664, 93 },
    { 0xa5d36665, 65 },
    { 0xf5136666, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f3666a, 78 },
    { 0xb303666b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a36672, 33 },
    { 0x29336673, 40 },
    { 0x8336674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473667c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1933667f, 98 },
    { 0x5c136680, 65 },
    { 0x9d336681, 98 },
    { 0, 0 },
    { 0xea36683, 93 },
    { 0, 0 },
    { 0x15d36685, 33 },
    { 0xdd636686, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab36689, 23 },
    { 0x42136689, 98 },
    { 0, 0 },
    { 0xbd83668c, 90 },
    { 0, 0 },
    { 0x9813668e, 61 },
    { 0xf833668e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9636692, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e36696, 23 },
    { 0, 0 },
    { 0x51136698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3669c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3713669f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a366a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780366a9, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6366ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e366af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2366b3, 34 },
    { 0xe7c366b3, 61 },
    { 0x9ca366b5, 33 },
    { 0x274366b3, 90 },
    { 0x194366b7, 75 },
    { 0, 0 },
    { 0xcb6366b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68366be, 90 },
    { 0x725366bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc366c3, 33 },
    { 0xcb0366c4, 76 },
    { 0xe4c366c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543366d0, 23 },
    { 0x48f366d1, 33 },
    { 0xbd9366d2, 90 },
    { 0x2be366d3, 67 },
    { 0x9ca366d4, 76 },
    { 0x1d0366d5, 93 },
    { 0, 0 },
    { 0x5ec366d7, 61 },
    { 0, 0 },
    { 0xe60366d9, 52 },
    { 0, 0 },
    { 0x2e7366db, 95 },
    { 0xf53366dc, 98 },
    { 0, 0 },
    { 0xaa0366de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32366e2, 96 },
    { 0x71366e3, 33 },
    { 0xe4f366e4, 93 },
    { 0, 0 },
    { 0x933366e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0366e9, 66 },
    { 0xf0a366ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8366ed, 95 },
    { 0, 0 },
    { 0xdcd366ef, 98 },
    { 0, 0 },
    { 0xa35366f1, 33 },
    { 0x5c3366f2, 93 },
    { 0xc98366f2, 77 },
    { 0x2e7366f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7366f8, 81 },
    { 0x94c366f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x692366fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8366ff, 91 },
    { 0xe7f36700, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45736704, 99 },
    { 0x39d36704, 46 },
    { 0xbe36704, 69 },
    { 0xf936707, 87 },
    { 0xd2136708, 61 },
    { 0x29136705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1123670c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3936714, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8236717, 90 },
    { 0xccd36717, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6043671d, 33 },
    { 0x2f3671e, 76 },
    { 0x2703671f, 87 },
    { 0x30836720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d36726, 61 },
    { 0x1b936727, 98 },
    { 0, 0 },
    { 0x81236729, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf673672d, 61 },
    { 0, 0 },
    { 0xa7f3672f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f636736, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c3673a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85736740, 55 },
    { 0x1ea36741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a636748, 33 },
    { 0x25336749, 90 },
    { 0x6453674a, 46 },
    { 0, 0 },
    { 0x4533674c, 90 },
    { 0xe623674c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90436750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c36753, 23 },
    { 0xaf136754, 98 },
    { 0x17436755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d36758, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483675c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae636760, 90 },
    { 0, 0 },
    { 0x94f36762, 36 },
    { 0, 0 },
    { 0x13a36764, 33 },
    { 0xd7936765, 85 },
    { 0xd1236765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03676a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5536779, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9773677e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d3678c, 23 },
    { 0xf123678d, 67 },
    { 0x14d3678e, 68 },
    { 0x28f3678e, 61 },
    { 0x77436790, 33 },
    { 0x7973678e, 95 },
    { 0xcbf3678d, 61 },
    { 0, 0 },
    { 0x7f836794, 43 },
    { 0xe936794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40836798, 43 },
    { 0xb2e36799, 95 },
    { 0xc053679a, 55 },
    { 0x9473679a, 61 },
    { 0xeb33679b, 85 },
    { 0, 0 },
    { 0x5e33679e, 34 },
    { 0x623679e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38367a4, 66 },
    { 0x8dd367a5, 55 },
    { 0x5f8367a6, 87 },
    { 0, 0 },
    { 0x5a1367a8, 33 },
    { 0, 0 },
    { 0xd6e367aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1367b0, 69 },
    { 0x5c3367b1, 33 },
    { 0, 0 },
    { 0xdfc367b3, 23 },
    { 0xfb5367b3, 33 },
    { 0x616367b5, 33 },
    { 0, 0 },
    { 0x72c367b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21367c0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x874367c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502367d0, 55 },
    { 0x476367d0, 46 },
    { 0, 0 },
    { 0x12b367d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5367d7, 53 },
    { 0xf46367d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e367dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62367e2, 52 },
    { 0x9bf367e3, 33 },
    { 0xafc367e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6367e8, 33 },
    { 0, 0 },
    { 0xea6367ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db367ee, 90 },
    { 0, 0 },
    { 0x82b367f0, 98 },
    { 0x202367f1, 98 },
    { 0xaed367f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf367fa, 61 },
    { 0, 0 },
    { 0x5b9367fc, 33 },
    { 0xbf9367fd, 90 },
    { 0xc91367fe, 95 },
    { 0x81e367ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9436802, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45336806, 100 },
    { 0xcd136807, 98 },
    { 0xa6a36807, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6636812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d36817, 98 },
    { 0x51236818, 34 },
    { 0x59436819, 90 },
    { 0xc8f3681a, 91 },
    { 0xa3b3681b, 33 },
    { 0xbf53681a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb836821, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a936827, 95 },
    { 0xbf336828, 46 },
    { 0x14536828, 100 },
    { 0xf383682a, 23 },
    { 0, 0 },
    { 0xcbc3682c, 95 },
    { 0x8963682d, 52 },
    { 0x4103682d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9236834, 61 },
    { 0, 0 },
    { 0xa1836836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe533683e, 33 },
    { 0x9e43683e, 76 },
    { 0x3cd36840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54036845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15236853, 50 },
    { 0xb1936854, 90 },
    { 0x3f636855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f36858, 65 },
    { 0x59136858, 98 },
    { 0xf1f3685a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3536864, 55 },
    { 0x1af36864, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a3686e, 84 },
    { 0, 0 },
    { 0x71436870, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d36873, 61 },
    { 0, 0 },
    { 0xb5036875, 91 },
    { 0xc6636876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee3687a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3687d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b36882, 98 },
    { 0, 0 },
    { 0x5aa36884, 63 },
    { 0x11636884, 90 },
    { 0, 0 },
    { 0x11e36887, 95 },
    { 0, 0 },
    { 0x3d36889, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373688d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32736892, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa536899, 52 },
    { 0xd113689a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c93689e, 98 },
    { 0x7cc3689e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e368a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x910368a5, 98 },
    { 0x698368a6, 76 },
    { 0, 0 },
    { 0x153368a8, 90 },
    { 0x9bb368a9, 55 },
    { 0xb7b368a9, 90 },
    { 0x5ff368aa, 77 },
    { 0x86a368ab, 90 },
    { 0x8d368ad, 66 },
    { 0xd23368aa, 76 },
    { 0xef1368af, 34 },
    { 0x9c7368b0, 55 },
    { 0xc61368af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e368b4, 87 },
    { 0xe5e368b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34368bb, 23 },
    { 0xbe368bc, 78 },
    { 0xf90368bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x742368c0, 33 },
    { 0xa2d368c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd368c7, 99 },
    { 0x22f368c7, 40 },
    { 0xf7e368c9, 23 },
    { 0x657368ca, 55 },
    { 0xfb1368cb, 81 },
    { 0xd03368c9, 98 },
    { 0x904368cc, 98 },
    { 0xd06368ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f368d3, 90 },
    { 0, 0 },
    { 0xa80368d5, 61 },
    { 0x686368d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2368d9, 77 },
    { 0, 0 },
    { 0x4ee368db, 33 },
    { 0, 0 },
    { 0x84f368dd, 93 },
    { 0, 0 },
    { 0xbc1368df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x986368e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec368e6, 77 },
    { 0, 0 },
    { 0x431368e8, 55 },
    { 0x32b368e9, 53 },
    { 0x710368e8, 61 },
    { 0xfc2368eb, 52 },
    { 0x8e3368ec, 100 },
    { 0x9f1368eb, 23 },
    { 0, 0 },
    { 0x6b5368ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f368f3, 78 },
    { 0xae4368f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x677368f7, 61 },
    { 0x47f368f8, 76 },
    { 0xb1e368f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742368fd, 86 },
    { 0, 0 },
    { 0x8f8368ff, 43 },
    { 0xc2b36900, 90 },
    { 0xa9a368ff, 98 },
    { 0xfd736902, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc36906, 34 },
    { 0xf2a36907, 93 },
    { 0x7f636908, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f36911, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c36915, 61 },
    { 0xb8136916, 85 },
    { 0xad736917, 23 },
    { 0xb6136916, 43 },
    { 0x25536919, 98 },
    { 0xfdd3691a, 75 },
    { 0x8353691b, 33 },
    { 0xe4736917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa936927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473692a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f3692d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe136933, 46 },
    { 0x75936933, 23 },
    { 0xd0636935, 43 },
    { 0x51836936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b236939, 76 },
    { 0, 0 },
    { 0x2d33693b, 98 },
    { 0xe6f3693c, 33 },
    { 0, 0 },
    { 0xebc3693e, 92 },
    { 0xbf43693e, 76 },
    { 0xae236940, 87 },
    { 0x97736941, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e036945, 43 },
    { 0xeb636945, 90 },
    { 0x40c36947, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b43694a, 87 },
    { 0xda33694b, 61 },
    { 0x2e53694b, 55 },
    { 0x7663694c, 88 },
    { 0xd6f3694e, 95 },
    { 0xcd13694a, 90 },
    { 0xcaf3694d, 61 },
    { 0x90e36950, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e36956, 55 },
    { 0, 0 },
    { 0x94f36958, 66 },
    { 0x73336959, 90 },
    { 0, 0 },
    { 0xa53695b, 33 },
    { 0, 0 },
    { 0xde43695d, 53 },
    { 0xb283695d, 98 },
    { 0x81f3695f, 98 },
    { 0x8643695f, 99 },
    { 0x84e36961, 91 },
    { 0x6b836962, 76 },
    { 0x37436961, 76 },
    { 0xe2b36961, 93 },
    { 0x9493695d, 67 },
    { 0, 0 },
    { 0xdb036967, 33 },
    { 0xac236968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3696e, 90 },
    { 0xbe33696f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a936972, 90 },
    { 0xea836972, 61 },
    { 0x95136972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee36978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2903697e, 43 },
    { 0x9a43697f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c3698a, 33 },
    { 0, 0 },
    { 0xf253698c, 95 },
    { 0x81d3698d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4636990, 84 },
    { 0x87836990, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff36994, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe436999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce53699c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb213699f, 43 },
    { 0xaf6369a0, 43 },
    { 0, 0 },
    { 0xa90369a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f369a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e369ae, 98 },
    { 0xdb3369af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c369b3, 90 },
    { 0xe68369b4, 23 },
    { 0x892369b5, 95 },
    { 0xf6369b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b369b9, 90 },
    { 0x88d369ba, 76 },
    { 0xad0369bb, 61 },
    { 0x8e7369bc, 90 },
    { 0xaaa369ba, 90 },
    { 0xa8f369bc, 33 },
    { 0x1c2369bd, 98 },
    { 0xdc9369bf, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61369c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c369ca, 100 },
    { 0x8de369ca, 90 },
    { 0xff0369cb, 90 },
    { 0x35c369cb, 34 },
    { 0x55c369ca, 76 },
    { 0xff5369cf, 55 },
    { 0x490369ca, 98 },
    { 0xba1369d1, 43 },
    { 0, 0 },
    { 0xf85369d3, 91 },
    { 0, 0 },
    { 0xa5b369d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265369dc, 61 },
    { 0, 0 },
    { 0x940369de, 90 },
    { 0x677369df, 23 },
    { 0x6d5369df, 77 },
    { 0xb66369e1, 95 },
    { 0x512369de, 22 },
    { 0, 0 },
    { 0x5a9369e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6369e9, 61 },
    { 0x6b8369e9, 52 },
    { 0, 0 },
    { 0x1a2369ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac369f5, 61 },
    { 0xaaf369f5, 78 },
    { 0xd9c369f7, 90 },
    { 0xa6b369f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7369fb, 78 },
    { 0x13c369fc, 95 },
    { 0xdba369fd, 98 },
    { 0x929369fe, 91 },
    { 0x84b369fd, 91 },
    { 0x26936a00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27836a05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e736a08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba36a0d, 55 },
    { 0x7c536a0e, 33 },
    { 0, 0 },
    { 0xd1636a10, 95 },
    { 0xe9e36a11, 61 },
    { 0x35836a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84836a16, 90 },
    { 0x48e36a17, 98 },
    { 0, 0 },
    { 0x79436a19, 90 },
    { 0x33436a19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b36a22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36936a26, 43 },
    { 0, 0 },
    { 0x86e36a28, 98 },
    { 0xaa636a29, 99 },
    { 0x32936a29, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb36a2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3836a32, 52 },
    { 0x37536a33, 76 },
    { 0xe0736a32, 90 },
    { 0xefc36a34, 93 },
    { 0x3dd36a34, 23 },
    { 0x54736a35, 23 },
    { 0xa9736a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71136a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82536a40, 90 },
    { 0xab736a41, 90 },
    { 0xf7436a40, 61 },
    { 0xcdc36a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14936a46, 98 },
    { 0xcb36a47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe036a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea36a4f, 65 },
    { 0xcb336a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8136a54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9336a58, 61 },
    { 0x93936a58, 43 },
    { 0x92336a5a, 33 },
    { 0x49336a5a, 33 },
    { 0xb1336a59, 58 },
    { 0xbe436a5d, 84 },
    { 0xd9536a5d, 90 },
    { 0x9f036a5e, 34 },
    { 0xddd36a60, 95 },
    { 0, 0 },
    { 0x85a36a62, 87 },
    { 0x8ef36a62, 61 },
    { 0x76136a62, 76 },
    { 0x4cd36a64, 67 },
    { 0xd3636a66, 46 },
    { 0x48036a64, 95 },
    { 0x30136a68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a36a6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7536a75, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xde436a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de36a80, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36a86, 61 },
    { 0, 0 },
    { 0x3f036a88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89736a8c, 93 },
    { 0x7fb36a8d, 61 },
    { 0xf2c36a8e, 80 },
    { 0x3d936a8e, 90 },
    { 0xaa036a8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c36a93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b736a97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb736a9c, 95 },
    { 0, 0 },
    { 0xf6636a9e, 33 },
    { 0, 0 },
    { 0x97136aa0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67836aa3, 95 },
    { 0xbb736aa4, 90 },
    { 0x82a36aa5, 61 },
    { 0, 0 },
    { 0x5ca36aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65836ab5, 23 },
    { 0x79c36ab6, 87 },
    { 0x3ee36ab7, 98 },
    { 0x8c836ab8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd036abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b36ac4, 33 },
    { 0x38a36ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e536aca, 46 },
    { 0x2fd36acb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77536ace, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2136ad5, 33 },
    { 0xce336ad5, 95 },
    { 0xe3436ad6, 99 },
    { 0x72336ad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7236adb, 95 },
    { 0, 0 },
    { 0x52836add, 98 },
    { 0, 0 },
    { 0x5bd36adf, 93 },
    { 0x7a836adf, 34 },
    { 0x68536ae0, 33 },
    { 0x8a136ae1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8436ae5, 23 },
    { 0x36f36ae6, 95 },
    { 0x12d36ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71336aeb, 39 },
    { 0xa2336aec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad36af0, 61 },
    { 0x29036af1, 90 },
    { 0x84636af2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4536af6, 66 },
    { 0xc0936af6, 23 },
    { 0xe2436af7, 95 },
    { 0x89036af9, 95 },
    { 0x85d36af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a336afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d536b00, 36 },
    { 0, 0 },
    { 0x56a36b02, 83 },
    { 0x2336b03, 61 },
    { 0xf2a36b04, 85 },
    { 0xb9436b04, 90 },
    { 0x2ef36b06, 46 },
    { 0x3c536b04, 61 },
    { 0x3ee36b02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa236b0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42036b10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc36b13, 66 },
    { 0, 0 },
    { 0xdab36b15, 95 },
    { 0, 0 },
    { 0x1a936b17, 98 },
    { 0xb336b18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72236b20, 23 },
    { 0, 0 },
    { 0x2e036b22, 55 },
    { 0, 0 },
    { 0x36136b24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a36b27, 52 },
    { 0x48736b27, 52 },
    { 0x95936b27, 90 },
    { 0xdd036b2a, 90 },
    { 0, 0 },
    { 0xcac36b2c, 90 },
    { 0x7d36b2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b30, 88 },
    { 0x34236b30, 98 },
    { 0x35436b32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7836b36, 90 },
    { 0x6f636b37, 34 },
    { 0x3f36b37, 87 },
    { 0xf0436b38, 17 },
    { 0x41536b37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78036b43, 76 },
    { 0xc8836b43, 78 },
    { 0xb5736b45, 99 },
    { 0xfe736b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e36b4c, 98 },
    { 0, 0 },
    { 0x90536b4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9336b53, 99 },
    { 0x1b336b54, 90 },
    { 0, 0 },
    { 0x1ae36b56, 78 },
    { 0x6de36b57, 55 },
    { 0x24f36b56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x51436b5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b36b5f, 17 },
    { 0, 0 },
    { 0xab136b61, 65 },
    { 0x4536b62, 90 },
    { 0x41736b61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d36b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f36b74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce36b77, 95 },
    { 0, 0 },
    { 0xe536b79, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f36b82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf836b85, 55 },
    { 0xc9c36b85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c36b8b, 61 },
    { 0x97b36b8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b36b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e336b94, 55 },
    { 0, 0 },
    { 0x59d36b96, 61 },
    { 0xe2136b96, 98 },
    { 0, 0 },
    { 0x8f36b99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30936b9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe936ba1, 55 },
    { 0x3c936ba1, 90 },
    { 0, 0 },
    { 0xdbc36ba4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76236ba8, 78 },
    { 0x64236ba9, 90 },
    { 0xa0636baa, 98 },
    { 0xa0d36ba8, 98 },
    { 0x90d36bac, 95 },
    { 0x76336bad, 91 },
    { 0x21e36bae, 76 },
    { 0x42b36bad, 90 },
    { 0xe6636ba8, 43 },
    { 0, 0 },
    { 0xf2336bb2, 23 },
    { 0xb8136bb3, 77 },
    { 0x8a936bb4, 23 },
    { 0x21b36bb5, 66 },
    { 0x40636bb6, 34 },
    { 0xbcf36bb7, 23 },
    { 0, 0 },
    { 0x6e136bb9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa36bbd, 76 },
    { 0, 0 },
    { 0x39d36bbf, 46 },
    { 0, 0 },
    { 0xa6d36bc1, 95 },
    { 0, 0 },
    { 0x7aa36bc3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba36bc7, 55 },
    { 0, 0 },
    { 0xf0a36bc9, 92 },
    { 0xea136bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f36bcd, 17 },
    { 0xdde36bce, 98 },
    { 0xcff36bcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1236bd9, 95 },
    { 0, 0 },
    { 0x90036bdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c336be1, 98 },
    { 0xcdc36be2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d736be8, 42 },
    { 0x64b36be9, 65 },
    { 0xfa336bea, 76 },
    { 0x2236bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b36bef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1636bf8, 19 },
    { 0, 0 },
    { 0x81836bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37836bfe, 19 },
    { 0xceb36bff, 61 },
    { 0x1a936bff, 85 },
    { 0xe7936c01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a636c07, 98 },
    { 0xfee36c07, 46 },
    { 0x64c36c09, 78 },
    { 0xeac36c09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b36c10, 55 },
    { 0, 0 },
    { 0x2e036c12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236c18, 43 },
    { 0xcdd36c19, 95 },
    { 0x58736c1a, 23 },
    { 0xae136c1b, 34 },
    { 0, 0 },
    { 0x5d736c1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19036c20, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd36c23, 76 },
    { 0xde536c24, 34 },
    { 0x1bf36c25, 95 },
    { 0x3a336c25, 90 },
    { 0xf6836c23, 95 },
    { 0x4f936c28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a36c2f, 99 },
    { 0x50636c30, 93 },
    { 0xcca36c2f, 100 },
    { 0xcf536c32, 66 },
    { 0, 0 },
    { 0x37636c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c136c37, 98 },
    { 0xeb636c38, 90 },
    { 0, 0 },
    { 0x79436c3a, 87 },
    { 0xcd136c3b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47236c3f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c736c45, 33 },
    { 0x48d36c46, 61 },
    { 0xc7936c45, 23 },
    { 0, 0 },
    { 0x42036c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e36c4d, 76 },
    { 0, 0 },
    { 0xc3036c4f, 95 },
    { 0xeb036c50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e936c55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e236c58, 84 },
    { 0xb936c59, 90 },
    { 0xbf636c5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30136c5d, 61 },
    { 0x1f236c5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236c61, 33 },
    { 0x45036c62, 33 },
    { 0xd6036c63, 33 },
    { 0x4c636c63, 61 },
    { 0xb6f36c65, 95 },
    { 0, 0 },
    { 0x2aa36c67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9036c6f, 98 },
    { 0x95f36c70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c36c77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54236c7b, 93 },
    { 0xc0f36c7b, 90 },
    { 0x49836c7d, 33 },
    { 0, 0 },
    { 0x8d136c7f, 61 },
    { 0xe3836c80, 33 },
    { 0x93a36c81, 66 },
    { 0x8de36c81, 90 },
    { 0x44936c80, 61 },
    { 0, 0 },
    { 0x16936c85, 90 },
    { 0xf1e36c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d736c8b, 66 },
    { 0, 0 },
    { 0x53d36c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b836c90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e536c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81036c99, 90 },
    { 0xd9b36c99, 34 },
    { 0xdfd36c9a, 95 },
    { 0xb6036c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4536ca4, 23 },
    { 0, 0 },
    { 0x66536ca6, 95 },
    { 0x9e636ca6, 90 },
    { 0xb7b36ca8, 98 },
    { 0x1436ca7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64136cae, 93 },
    { 0xa3536caf, 23 },
    { 0xaef36cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc36cb3, 90 },
    { 0, 0 },
    { 0xd4136cb5, 33 },
    { 0x87536cb6, 46 },
    { 0xc4436cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9436cbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d636cbe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91136cc3, 61 },
    { 0x78236cc4, 95 },
    { 0xda736cc5, 33 },
    { 0x80536cc6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc36ccc, 90 },
    { 0xea136ccd, 81 },
    { 0x64836cce, 87 },
    { 0x2f636ccd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e36cdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e36ce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63936ce8, 33 },
    { 0, 0 },
    { 0xef536cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91336ced, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb36d01, 90 },
    { 0x51836d02, 99 },
    { 0x87c36d03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5436d09, 85 },
    { 0, 0 },
    { 0x73f36d0b, 55 },
    { 0x2e936d0c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd336d12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84536d15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c36d18, 61 },
    { 0x48136d18, 84 },
    { 0xd536d1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34936d23, 78 },
    { 0x91d36d24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e36d29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48936d2d, 61 },
    { 0x73f36d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42236d32, 46 },
    { 0, 0 },
    { 0xccb36d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17936d3a, 90 },
    { 0, 0 },
    { 0xf9736d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86736d44, 43 },
    { 0xe5136d44, 98 },
    { 0x49736d46, 85 },
    { 0xe4336d44, 98 },
    { 0x99c36d47, 90 },
    { 0x3f036d49, 85 },
    { 0x62036d4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c636d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41136d5f, 17 },
    { 0x9d736d60, 61 },
    { 0x58636d61, 52 },
    { 0, 0 },
    { 0xcd336d63, 88 },
    { 0x8a736d63, 85 },
    { 0x5f336d63, 55 },
    { 0, 0 },
    { 0xddf36d67, 46 },
    { 0x90b36d67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39836d6c, 90 },
    { 0x28636d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4036d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d36d75, 61 },
    { 0, 0 },
    { 0xd1436d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6236d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc36d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93136d80, 90 },
    { 0, 0 },
    { 0xde936d82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x88336d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2036d89, 95 },
    { 0, 0 },
    { 0xe4c36d8b, 78 },
    { 0x7d336d8c, 75 },
    { 0, 0 },
    { 0x83336d8e, 60 },
    { 0xa2236d8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c436d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd36d99, 55 },
    { 0xc4536d9a, 17 },
    { 0, 0 },
    { 0xf9636d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a336d9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d036da3, 78 },
    { 0x96f36da4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96636da7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14736dab, 84 },
    { 0xc9136dac, 23 },
    { 0, 0 },
    { 0xba236dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad036db2, 33 },
    { 0xed636db2, 61 },
    { 0x6236db2, 23 },
    { 0, 0 },
    { 0xc3436db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e36db9, 43 },
    { 0x38836dba, 87 },
    { 0x34436dbb, 69 },
    { 0xd3b36dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c036dc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d336dc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef36dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac36dd2, 90 },
    { 0xda736dd3, 94 },
    { 0xb3236dd3, 23 },
    { 0x3fd36dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea36de2, 90 },
    { 0xc6236de2, 40 },
    { 0xbff36de2, 98 },
    { 0x3dc36de3, 61 },
    { 0xe9336de3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6036de9, 61 },
    { 0xc3436dea, 34 },
    { 0, 0 },
    { 0xf2036dec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a36df4, 33 },
    { 0x83a36df5, 78 },
    { 0xeb436df6, 23 },
    { 0x36e36df6, 33 },
    { 0xb9036df8, 23 },
    { 0, 0 },
    { 0xdfe36dfa, 2 },
    { 0, 0 },
    { 0xffd36dfc, 23 },
    { 0, 0 },
    { 0xabd36dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b36e04, 65 },
    { 0x52a36e05, 45 },
    { 0, 0 },
    { 0xf0136e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e36e0a, 34 },
    { 0x1b536e0b, 91 },
    { 0, 0 },
    { 0xea036e0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d036e13, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32836e24, 55 },
    { 0, 0 },
    { 0xf7e36e26, 91 },
    { 0xa3036e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf936e2a, 76 },
    { 0x7db36e2b, 46 },
    { 0x70236e2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47236e31, 66 },
    { 0, 0 },
    { 0x8a236e33, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a36e3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e836e3e, 90 },
    { 0, 0 },
    { 0x10c36e40, 23 },
    { 0xc9936e41, 88 },
    { 0xda136e42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0636e46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94536e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7836e50, 55 },
    { 0, 0 },
    { 0x10236e52, 91 },
    { 0, 0 },
    { 0x40036e54, 66 },
    { 0xd136e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa236e5b, 33 },
    { 0xdfc36e5c, 55 },
    { 0xbbb36e5d, 98 },
    { 0xce736e5b, 55 },
    { 0x92a36e5f, 93 },
    { 0xd6836e5f, 87 },
    { 0x84d36e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d36e66, 98 },
    { 0x93436e67, 23 },
    { 0x4ff36e68, 34 },
    { 0, 0 },
    { 0xd2036e6a, 96 },
    { 0, 0 },
    { 0x64b36e6c, 25 },
    { 0xf3a36e6d, 61 },
    { 0, 0 },
    { 0x92236e6f, 23 },
    { 0x45836e6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88836e73, 95 },
    { 0xcca36e74, 90 },
    { 0xbad36e73, 95 },
    { 0xd0336e76, 61 },
    { 0x74136e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17936e82, 76 },
    { 0x41836e83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e36e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136e8d, 98 },
    { 0xd0d36e8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f36e91, 23 },
    { 0xca436e91, 98 },
    { 0, 0 },
    { 0xa4736e94, 81 },
    { 0, 0 },
    { 0x2e236e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50636e9d, 88 },
    { 0x9b636e9e, 90 },
    { 0x5ee36e9f, 100 },
    { 0xe9136e9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30936ea6, 91 },
    { 0x56836ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e736eaa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc36eb0, 91 },
    { 0xb936eb0, 87 },
    { 0xca436eb1, 46 },
    { 0xafc36eb3, 98 },
    { 0x6fd36eb3, 90 },
    { 0x49f36eb5, 23 },
    { 0xd4636eb5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe436eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9536ebc, 24 },
    { 0x9c136ebd, 46 },
    { 0x8cc36ebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b36ec8, 23 },
    { 0xb0136ec8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99836ed0, 61 },
    { 0x16136ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5636edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d136ee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1036ee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a36eea, 23 },
    { 0, 0 },
    { 0x9ce36eec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x74736eef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0536ef3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10936ef8, 80 },
    { 0x4ff36ef8, 33 },
    { 0xce136ef9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d36efd, 76 },
    { 0xbf836efd, 33 },
    { 0x7df36eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b36f03, 61 },
    { 0, 0 },
    { 0x6c336f05, 95 },
    { 0x37f36f06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136f09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25736f0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f736f0f, 93 },
    { 0, 0 },
    { 0x6e536f11, 90 },
    { 0xdc536f12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9436f19, 34 },
    { 0xf8836f19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83136f1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d436f21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d536f2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe36f32, 17 },
    { 0x4a236f33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad436f36, 90 },
    { 0xbda36f37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf36f3d, 87 },
    { 0, 0 },
    { 0xea436f3f, 33 },
    { 0x68236f3f, 66 },
    { 0x66836f40, 23 },
    { 0xc6b36f42, 95 },
    { 0x42a36f41, 33 },
    { 0x68536f43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32636f49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee236f52, 34 },
    { 0x59136f53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c36f57, 95 },
    { 0xe0e36f57, 33 },
    { 0x99236f59, 61 },
    { 0x6b336f57, 80 },
    { 0xeab36f5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c36f62, 33 },
    { 0x36836f63, 61 },
    { 0, 0 },
    { 0xd5536f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf236f68, 90 },
    { 0xf1636f69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81136f71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf36f76, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99936f7a, 23 },
    { 0xd8836f7b, 90 },
    { 0, 0 },
    { 0x8ce36f7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e36f80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c36f84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e36f87, 95 },
    { 0xfda36f87, 61 },
    { 0x35e36f88, 40 },
    { 0x9ca36f87, 52 },
    { 0x5b236f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x57336f8e, 90 },
    { 0xa0e36f8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8736f94, 98 },
    { 0x4e636f95, 23 },
    { 0, 0 },
    { 0x9db36f97, 23 },
    { 0x66736f98, 34 },
    { 0xeb36f99, 23 },
    { 0x6ad36f9a, 66 },
    { 0x14a36f9b, 98 },
    { 0, 0 },
    { 0x8cc36f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d36fa1, 98 },
    { 0xd2836fa2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x78036fa5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc436fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a36faf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e336fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d36fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59536fbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35636fc2, 17 },
    { 0, 0 },
    { 0x97e36fc4, 23 },
    { 0x5836fc4, 90 },
    { 0x3b836fc4, 52 },
    { 0, 0 },
    { 0x12036fc8, 61 },
    { 0x71036fc9, 61 },
    { 0xa536fc9, 23 },
    { 0x2b636fcb, 46 },
    { 0, 0 },
    { 0x46436fcd, 23 },
    { 0, 0 },
    { 0xa5736fcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec36fd5, 23 },
    { 0xd2e36fd5, 34 },
    { 0x8ea36fd5, 67 },
    { 0, 0 },
    { 0x1bc36fd9, 17 },
    { 0x2ea36fda, 85 },
    { 0x83e36fd9, 95 },
    { 0, 0 },
    { 0x31836fdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a36fe1, 90 },
    { 0x52736fe2, 76 },
    { 0x6cd36fe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de36fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d36fee, 90 },
    { 0xa8436fef, 23 },
    { 0, 0 },
    { 0xbba36ff1, 93 },
    { 0x33c36ff2, 33 },
    { 0, 0 },
    { 0xa5c36ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec36ff8, 23 },
    { 0x22b36ff9, 66 },
    { 0x1b236ff8, 95 },
    { 0xa7c36ffb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec37004, 55 },
    { 0xd537005, 91 },
    { 0xe7437006, 98 },
    { 0x7837007, 90 },
    { 0x26537008, 94 },
    { 0xd9e37009, 46 },
    { 0xb2b3700a, 98 },
    { 0xe8537004, 76 },
    { 0xd9a3700c, 87 },
    { 0x39a3700d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50137011, 99 },
    { 0, 0 },
    { 0xc6137013, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3701c, 87 },
    { 0x5913701d, 52 },
    { 0x5873701d, 95 },
    { 0x8683701f, 23 },
    { 0x1e237020, 61 },
    { 0x9bd3701f, 34 },
    { 0, 0 },
    { 0x3fd37023, 98 },
    { 0x53437024, 98 },
    { 0x27f37025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee3702a, 33 },
    { 0, 0 },
    { 0xcb3702c, 76 },
    { 0x7543702c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e937032, 95 },
    { 0xeea37032, 66 },
    { 0x8c437034, 34 },
    { 0, 0 },
    { 0xcc737036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b37043, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d37048, 91 },
    { 0, 0 },
    { 0xfda3704a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17437052, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc337058, 33 },
    { 0x88237058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3705e, 17 },
    { 0, 0 },
    { 0x27a37060, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa37063, 87 },
    { 0xc437064, 23 },
    { 0x2b737064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b37069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f3706c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d37072, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be37076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8737079, 33 },
    { 0, 0 },
    { 0xf8e3707b, 23 },
    { 0x603707b, 33 },
    { 0xebf3707b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c37081, 76 },
    { 0, 0 },
    { 0x2fd37083, 23 },
    { 0xc9f37084, 75 },
    { 0, 0 },
    { 0x30037086, 90 },
    { 0xaa037087, 76 },
    { 0x3b937088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023708b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b37090, 66 },
    { 0x85d37091, 84 },
    { 0xdb537091, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f537096, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd03709f, 90 },
    { 0, 0 },
    { 0x928370a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174370a5, 95 },
    { 0x6f4370a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3370aa, 90 },
    { 0, 0 },
    { 0x3d4370ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x599370af, 78 },
    { 0, 0 },
    { 0xf45370b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48370bb, 61 },
    { 0x7e370bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4370c1, 95 },
    { 0xd19370c2, 34 },
    { 0xe6f370c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79370cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df370d2, 98 },
    { 0x99a370d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2370d6, 34 },
    { 0xc41370d7, 76 },
    { 0xa65370d7, 61 },
    { 0x303370d9, 61 },
    { 0x13f370d8, 43 },
    { 0xed8370db, 23 },
    { 0, 0 },
    { 0xf11370dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e370e1, 33 },
    { 0x898370e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d370e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05370f2, 55 },
    { 0x6fd370f3, 87 },
    { 0xd76370f2, 95 },
    { 0, 0 },
    { 0x231370f6, 91 },
    { 0, 0 },
    { 0x38370f8, 76 },
    { 0, 0 },
    { 0xefd370fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f370ff, 66 },
    { 0xe3c370ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf37104, 75 },
    { 0x72137105, 34 },
    { 0xd4f37106, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda37110, 98 },
    { 0x78f37111, 85 },
    { 0xfd837112, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343711b, 61 },
    { 0, 0 },
    { 0x4b83711d, 61 },
    { 0xe043711e, 97 },
    { 0xe323711f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c37123, 61 },
    { 0xc9637123, 34 },
    { 0x31837125, 98 },
    { 0xc637126, 61 },
    { 0xca37126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb3712a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3712d, 55 },
    { 0x8323712e, 95 },
    { 0x9a83712f, 43 },
    { 0x7d83712f, 46 },
    { 0, 0 },
    { 0x48e37132, 69 },
    { 0xf0737133, 95 },
    { 0xd7537134, 33 },
    { 0x1fa37135, 77 },
    { 0x8ae37132, 93 },
    { 0x56f37136, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b83713d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1337148, 66 },
    { 0xe6237149, 80 },
    { 0x25037149, 53 },
    { 0x18e3714b, 90 },
    { 0x1113714b, 90 },
    { 0x7c37148, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7337150, 90 },
    { 0x12e37150, 61 },
    { 0xf6437151, 90 },
    { 0, 0 },
    { 0x6137154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d537157, 22 },
    { 0x5a137158, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283715c, 33 },
    { 0, 0 },
    { 0xb493715e, 95 },
    { 0xb1e3715e, 90 },
    { 0xa653715f, 33 },
    { 0x15f37161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3837164, 33 },
    { 0, 0 },
    { 0x80437166, 100 },
    { 0, 0 },
    { 0xb8737168, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0237170, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89737174, 93 },
    { 0xe2337174, 98 },
    { 0x55137176, 61 },
    { 0x65937176, 90 },
    { 0xeb337177, 99 },
    { 0, 0 },
    { 0x2333717a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e23717e, 33 },
    { 0xb453717e, 23 },
    { 0x87e3717f, 38 },
    { 0x74237181, 90 },
    { 0, 0 },
    { 0x74437183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0737188, 95 },
    { 0x64537188, 78 },
    { 0xf8137189, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2583718e, 55 },
    { 0xd653718f, 90 },
    { 0x41d3718f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa337195, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4637198, 95 },
    { 0x6e337199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a3719d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e371a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb371a4, 33 },
    { 0, 0 },
    { 0xca3371a6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483371ab, 90 },
    { 0, 0 },
    { 0x293371ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b371b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x966371b3, 98 },
    { 0xff8371b3, 87 },
    { 0x84d371b5, 95 },
    { 0x11b371b6, 95 },
    { 0xafc371b4, 90 },
    { 0, 0 },
    { 0x5c8371b9, 90 },
    { 0xd8d371ba, 93 },
    { 0x6cd371ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca371be, 34 },
    { 0x9e5371bf, 95 },
    { 0xc49371bf, 95 },
    { 0xef4371c0, 75 },
    { 0x1c2371c2, 55 },
    { 0x8fa371c1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1371c6, 99 },
    { 0xbfa371c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce371cb, 95 },
    { 0, 0 },
    { 0x4c7371cd, 33 },
    { 0x182371ce, 23 },
    { 0x2ab371cd, 33 },
    { 0xde0371cd, 100 },
    { 0x285371cf, 33 },
    { 0x243371d2, 90 },
    { 0x8b1371cf, 91 },
    { 0x924371d4, 43 },
    { 0x423371d5, 98 },
    { 0, 0 },
    { 0x59a371d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6371de, 23 },
    { 0x8df371de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca371e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a371ed, 98 },
    { 0xec3371ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616371f2, 87 },
    { 0x98371f2, 90 },
    { 0, 0 },
    { 0xec2371f5, 61 },
    { 0, 0 },
    { 0x5d2371f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a371fa, 95 },
    { 0xff371fb, 52 },
    { 0x3b371fc, 46 },
    { 0xae8371fd, 90 },
    { 0xcc371fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3637204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3720b, 55 },
    { 0x2fb3720c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313720f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf137216, 90 },
    { 0, 0 },
    { 0xdbb37218, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe443721b, 98 },
    { 0xa733721c, 90 },
    { 0xedb3721c, 61 },
    { 0x1b93721b, 84 },
    { 0x6a73721d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8137224, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2637227, 66 },
    { 0x7f637227, 33 },
    { 0, 0 },
    { 0x6013722a, 93 },
    { 0xa723722b, 61 },
    { 0x69c3722b, 95 },
    { 0xcc43722c, 90 },
    { 0, 0 },
    { 0x7d03722f, 84 },
    { 0, 0 },
    { 0x85e37231, 90 },
    { 0x28e37232, 23 },
    { 0xebe37232, 61 },
    { 0x4737233, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037237, 87 },
    { 0x84737237, 76 },
    { 0xb3937239, 36 },
    { 0, 0 },
    { 0x5a33723b, 90 },
    { 0x1683723c, 61 },
    { 0xa783723b, 23 },
    { 0x1e93723e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20437241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1137244, 76 },
    { 0, 0 },
    { 0x8d137246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf63724a, 71 },
    { 0x65d3724b, 95 },
    { 0, 0 },
    { 0xc903724d, 87 },
    { 0, 0 },
    { 0x24f3724f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6737261, 91 },
    { 0xdf937261, 61 },
    { 0, 0 },
    { 0x7cf37264, 76 },
    { 0, 0 },
    { 0xa2a37266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb837269, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e23726e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2337271, 23 },
    { 0, 0 },
    { 0x63637273, 55 },
    { 0, 0 },
    { 0xae237275, 88 },
    { 0, 0 },
    { 0x637277, 98 },
    { 0, 0 },
    { 0xc0237279, 34 },
    { 0xf703727a, 90 },
    { 0x62637279, 23 },
    { 0, 0 },
    { 0x3a43727d, 34 },
    { 0, 0 },
    { 0x72f3727f, 76 },
    { 0xf6a3727f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7237283, 77 },
    { 0x32837284, 98 },
    { 0xaae37284, 90 },
    { 0xac037286, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503728d, 98 },
    { 0xb183728e, 95 },
    { 0xfec3728f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc837293, 98 },
    { 0xf8137294, 78 },
    { 0x2e37294, 61 },
    { 0xc537296, 88 },
    { 0x47737294, 23 },
    { 0x21e37297, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3729d, 23 },
    { 0, 0 },
    { 0x8fb3729f, 90 },
    { 0x172372a0, 93 },
    { 0, 0 },
    { 0x532372a2, 65 },
    { 0xe12372a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6372a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99372aa, 91 },
    { 0x75b372ab, 98 },
    { 0x3e4372ac, 100 },
    { 0x628372aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3372b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0372b3, 66 },
    { 0x9d4372b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4372c0, 33 },
    { 0, 0 },
    { 0x10e372c2, 61 },
    { 0xf6a372c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x949372c6, 61 },
    { 0, 0 },
    { 0xbae372c8, 61 },
    { 0x520372c8, 33 },
    { 0, 0 },
    { 0xa4e372cb, 93 },
    { 0, 0 },
    { 0xbaf372cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84372d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad372db, 76 },
    { 0xcb372dc, 98 },
    { 0x6b3372dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0372e8, 76 },
    { 0, 0 },
    { 0xdf372ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d372ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2372f2, 33 },
    { 0, 0 },
    { 0xc20372f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c372f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318372fe, 67 },
    { 0xc91372ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b37314, 52 },
    { 0x2a037314, 98 },
    { 0, 0 },
    { 0xbc37317, 39 },
    { 0, 0 },
    { 0xee637319, 93 },
    { 0x74437319, 93 },
    { 0, 0 },
    { 0xdea3731c, 55 },
    { 0xff13731d, 78 },
    { 0x5ec3731e, 33 },
    { 0x8ac3731e, 76 },
    { 0x4737320, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88337325, 23 },
    { 0, 0 },
    { 0xa4c37327, 85 },
    { 0x31537328, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373732c, 90 },
    { 0x7b93732d, 75 },
    { 0x1293732e, 23 },
    { 0x8e53732f, 90 },
    { 0x59e37330, 98 },
    { 0x473732e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d237334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e537337, 83 },
    { 0, 0 },
    { 0xe9837339, 55 },
    { 0, 0 },
    { 0xb3b3733b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a37340, 99 },
    { 0x60e37341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23037349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef3734d, 46 },
    { 0xb623734e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b37351, 61 },
    { 0x55b37351, 55 },
    { 0xd537352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d837357, 97 },
    { 0x3d737358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd183735c, 90 },
    { 0x3133735d, 46 },
    { 0x7a23735e, 55 },
    { 0x4373735f, 75 },
    { 0x8d13735f, 98 },
    { 0, 0 },
    { 0x4a137362, 92 },
    { 0xac337362, 66 },
    { 0, 0 },
    { 0x6e637365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3736c, 98 },
    { 0x7383736d, 33 },
    { 0x37b3736e, 98 },
    { 0xbce3736c, 76 },
    { 0, 0 },
    { 0x78b37371, 33 },
    { 0x79137371, 95 },
    { 0x9c437373, 90 },
    { 0xc6637371, 95 },
    { 0x1c937374, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1173737a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb537380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79637387, 61 },
    { 0xdcc37387, 93 },
    { 0xe5237389, 98 },
    { 0xb3937389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93837391, 33 },
    { 0xc2337391, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01373a3, 23 },
    { 0, 0 },
    { 0xb4d373a5, 77 },
    { 0x7e7373a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844373aa, 93 },
    { 0xe1e373ab, 67 },
    { 0, 0 },
    { 0x424373ad, 90 },
    { 0, 0 },
    { 0xb6f373af, 76 },
    { 0x734373b0, 85 },
    { 0x3f373af, 81 },
    { 0, 0 },
    { 0xb6f373b3, 55 },
    { 0, 0 },
    { 0x91d373b5, 90 },
    { 0xad3373b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c373c2, 34 },
    { 0xc30373c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d373c8, 73 },
    { 0xc87373c8, 98 },
    { 0x357373c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560373ce, 90 },
    { 0xcde373cf, 76 },
    { 0, 0 },
    { 0xc60373d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f373d5, 33 },
    { 0, 0 },
    { 0x915373d7, 14 },
    { 0, 0 },
    { 0x232373d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7373e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428373e9, 23 },
    { 0, 0 },
    { 0x8e2373eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12373f7, 90 },
    { 0, 0 },
    { 0x3a0373f9, 55 },
    { 0xaf1373f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d637402, 76 },
    { 0x55a37402, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e37406, 76 },
    { 0xb4637406, 60 },
    { 0x7337408, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d3740c, 95 },
    { 0x3483740c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5737411, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a37415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d3741a, 78 },
    { 0xf783741b, 80 },
    { 0x4ba3741c, 100 },
    { 0, 0 },
    { 0x1183741e, 53 },
    { 0x8df3741f, 46 },
    { 0xc5e3741f, 88 },
    { 0xc9337421, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2903742d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24037438, 90 },
    { 0xa5e37438, 90 },
    { 0x6fd3743a, 55 },
    { 0x8483743b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553743f, 95 },
    { 0xbf537440, 95 },
    { 0xb6337440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91837445, 61 },
    { 0xa4b37445, 95 },
    { 0xbbf37445, 43 },
    { 0x69137448, 98 },
    { 0x31b37449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e03744c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45437454, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69437457, 23 },
    { 0xfc937458, 76 },
    { 0xd6037459, 75 },
    { 0xe8a37457, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2913745e, 88 },
    { 0xc743745f, 55 },
    { 0, 0 },
    { 0x86037461, 66 },
    { 0xba437462, 76 },
    { 0xf5e37463, 86 },
    { 0, 0 },
    { 0x29e37465, 33 },
    { 0, 0 },
    { 0x83437467, 23 },
    { 0, 0 },
    { 0xe7937469, 44 },
    { 0, 0 },
    { 0xdfe3746b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd913746e, 90 },
    { 0x2a73746f, 23 },
    { 0xe513746e, 61 },
    { 0x9a63746e, 90 },
    { 0xc5237472, 55 },
    { 0x4e837472, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb3747f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce737482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6337485, 99 },
    { 0, 0 },
    { 0xc2837487, 90 },
    { 0xf6837488, 99 },
    { 0x9037488, 98 },
    { 0x9543748a, 90 },
    { 0x8f83748a, 98 },
    { 0x44d3748c, 61 },
    { 0x1a33748d, 76 },
    { 0xe4f3748e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b37497, 34 },
    { 0x4bf37498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313749b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44374a0, 33 },
    { 0x48b374a1, 76 },
    { 0, 0 },
    { 0x6fd374a3, 43 },
    { 0xdab374a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875374a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80374ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5374b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d374b5, 23 },
    { 0, 0 },
    { 0x2cc374b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b374bd, 55 },
    { 0xc16374be, 34 },
    { 0xe14374be, 52 },
    { 0x6f3374bf, 98 },
    { 0x834374c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29374c7, 81 },
    { 0x9c6374c8, 93 },
    { 0x2b9374c9, 90 },
    { 0x141374c9, 90 },
    { 0x93a374c7, 95 },
    { 0, 0 },
    { 0x74f374cd, 65 },
    { 0xc75374ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147374d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662374db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3374e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431374e9, 78 },
    { 0xdf5374e9, 33 },
    { 0, 0 },
    { 0x40b374ec, 90 },
    { 0xb20374ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba374f0, 90 },
    { 0x416374f1, 33 },
    { 0xde0374f2, 23 },
    { 0, 0 },
    { 0xc19374f4, 85 },
    { 0xdaa374f4, 90 },
    { 0, 0 },
    { 0x7e2374f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1374fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79437503, 33 },
    { 0x63137504, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b37509, 55 },
    { 0x1273750a, 46 },
    { 0x3e93750b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d37511, 23 },
    { 0x44137512, 55 },
    { 0xa7137512, 33 },
    { 0x2bd37512, 90 },
    { 0xafa37513, 90 },
    { 0x9a837511, 70 },
    { 0x1df37511, 33 },
    { 0x82537518, 90 },
    { 0x32937518, 67 },
    { 0xc6f37512, 33 },
    { 0x6ff3751b, 77 },
    { 0xcc53751c, 61 },
    { 0x2f83751a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95237522, 101 },
    { 0, 0 },
    { 0xcb037524, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d23752d, 90 },
    { 0xc2c3752e, 90 },
    { 0x19f3752f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b37535, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd283753e, 33 },
    { 0x6573753f, 61 },
    { 0, 0 },
    { 0x1f437541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc537544, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2937549, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3754e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d37552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea23755a, 95 },
    { 0x94c3755a, 84 },
    { 0x3263755c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd37560, 90 },
    { 0x4237561, 81 },
    { 0x5f437562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3737566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8737569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85037570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e37575, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f37578, 34 },
    { 0, 0 },
    { 0x9bc3757a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b437582, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca37588, 33 },
    { 0xfee37589, 93 },
    { 0x7e03758a, 76 },
    { 0xe943758a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe253758e, 90 },
    { 0, 0 },
    { 0xdb737590, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb37593, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac3759c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1375a4, 34 },
    { 0xc1c375a5, 99 },
    { 0, 0 },
    { 0xfe5375a7, 33 },
    { 0, 0 },
    { 0xb6d375a9, 23 },
    { 0x7a3375aa, 33 },
    { 0, 0 },
    { 0xe31375ac, 66 },
    { 0x342375ac, 98 },
    { 0x5375ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90375b2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae375b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a375bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3375cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff375d5, 90 },
    { 0x46f375d6, 80 },
    { 0x243375d5, 90 },
    { 0x181375d8, 98 },
    { 0x131375d9, 61 },
    { 0x730375da, 90 },
    { 0xc1f375d8, 43 },
    { 0x751375d9, 61 },
    { 0x4b3375dd, 98 },
    { 0xa02375de, 33 },
    { 0x6c9375df, 38 },
    { 0xcf375e0, 90 },
    { 0xa50375dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1375e5, 98 },
    { 0x915375e5, 80 },
    { 0xf49375e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5375ec, 95 },
    { 0x461375ec, 77 },
    { 0xba7375ee, 90 },
    { 0x725375ef, 68 },
    { 0xf76375ee, 78 },
    { 0x65c375f1, 66 },
    { 0x7b7375f0, 33 },
    { 0xcb4375f0, 76 },
    { 0x1a9375ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f375f8, 76 },
    { 0, 0 },
    { 0x4db375fa, 91 },
    { 0x9d8375fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63037601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3760c, 48 },
    { 0xb153760d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71437614, 23 },
    { 0xae237615, 90 },
    { 0x76d37616, 77 },
    { 0, 0 },
    { 0x5ec37618, 90 },
    { 0, 0 },
    { 0x1d53761a, 23 },
    { 0xf5f3761b, 98 },
    { 0, 0 },
    { 0xe663761d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3137620, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81137628, 61 },
    { 0x47d37629, 101 },
    { 0, 0 },
    { 0xfff3762b, 34 },
    { 0, 0 },
    { 0xfa63762d, 95 },
    { 0x2b43762e, 61 },
    { 0, 0 },
    { 0x24a37630, 61 },
    { 0xc7137630, 23 },
    { 0xd5737632, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c37635, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e37639, 90 },
    { 0xb0b3763a, 76 },
    { 0xd933763a, 23 },
    { 0x4a13763b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e37649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e3764e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3437651, 90 },
    { 0x86537651, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f37655, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49237660, 23 },
    { 0, 0 },
    { 0x71c37662, 95 },
    { 0, 0 },
    { 0xfc737664, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b3766a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7873766d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95737674, 66 },
    { 0x48b37675, 90 },
    { 0x6ba37676, 95 },
    { 0, 0 },
    { 0xafa37678, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf37680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc037686, 61 },
    { 0, 0 },
    { 0x92637688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3768f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a37693, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37696, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e37699, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3769d, 90 },
    { 0xc6f3769e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe376a1, 95 },
    { 0x4e7376a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951376a7, 61 },
    { 0x4cd376a7, 23 },
    { 0x309376a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940376ad, 66 },
    { 0, 0 },
    { 0x304376af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b376b7, 98 },
    { 0, 0 },
    { 0x45e376b9, 76 },
    { 0x43376ba, 46 },
    { 0x290376b9, 98 },
    { 0, 0 },
    { 0xf0a376bd, 87 },
    { 0x669376be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9376c4, 93 },
    { 0, 0 },
    { 0x27c376c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8376ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7376cf, 53 },
    { 0, 0 },
    { 0x837376d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x173376d4, 77 },
    { 0x49d376d5, 95 },
    { 0x6f376d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea376d9, 85 },
    { 0, 0 },
    { 0xed8376db, 46 },
    { 0x67c376dc, 64 },
    { 0, 0 },
    { 0xcff376de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d376e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c376e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4376ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x188376f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e376f8, 90 },
    { 0, 0 },
    { 0x22f376fa, 34 },
    { 0xb8e376fa, 90 },
    { 0x987376fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0376ff, 55 },
    { 0, 0 },
    { 0x2f737701, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4937705, 90 },
    { 0xb1837706, 51 },
    { 0x1e737707, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3770b, 33 },
    { 0, 0 },
    { 0x4b23770d, 33 },
    { 0xec13770d, 61 },
    { 0, 0 },
    { 0xa4a37710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba037719, 90 },
    { 0, 0 },
    { 0xa2b3771b, 93 },
    { 0x32e3771c, 76 },
    { 0, 0 },
    { 0x1593771e, 98 },
    { 0x2753771e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a37724, 76 },
    { 0, 0 },
    { 0xaea37726, 75 },
    { 0x6bf37726, 90 },
    { 0x3ee37728, 23 },
    { 0, 0 },
    { 0x6b43772a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac37730, 91 },
    { 0x6c737731, 83 },
    { 0, 0 },
    { 0x55a37733, 77 },
    { 0x4c037733, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf837738, 55 },
    { 0x4a537739, 24 },
    { 0xe6d3773a, 98 },
    { 0xa7237738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8537741, 43 },
    { 0, 0 },
    { 0xff537743, 87 },
    { 0xf1937744, 90 },
    { 0, 0 },
    { 0x52237746, 33 },
    { 0, 0 },
    { 0x32337748, 61 },
    { 0x29e37749, 34 },
    { 0x2d437749, 33 },
    { 0xb6737748, 33 },
    { 0, 0 },
    { 0xe893774d, 75 },
    { 0xa413774e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71437753, 78 },
    { 0xab037754, 90 },
    { 0x4a037755, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d537758, 33 },
    { 0xb9637759, 81 },
    { 0x96237759, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d3775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80637763, 95 },
    { 0, 0 },
    { 0x7dd37765, 33 },
    { 0xc5237765, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3776a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21037770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2937773, 87 },
    { 0x65037774, 90 },
    { 0, 0 },
    { 0x5d537776, 90 },
    { 0xf7c37777, 61 },
    { 0xfd437778, 95 },
    { 0xb8e37778, 55 },
    { 0x93b3777a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7313777d, 95 },
    { 0xa703777e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f637782, 98 },
    { 0, 0 },
    { 0xe3237784, 76 },
    { 0x33137785, 85 },
    { 0, 0 },
    { 0x8b237787, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343778f, 90 },
    { 0x2e737790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65337794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a37797, 43 },
    { 0, 0 },
    { 0xf6637799, 61 },
    { 0x9443779a, 61 },
    { 0x48037799, 85 },
    { 0xb913779a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e3779f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef377a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a377a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585377ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85377b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x254377b6, 93 },
    { 0xd33377b7, 61 },
    { 0, 0 },
    { 0xee2377b9, 90 },
    { 0x7df377ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91377bd, 52 },
    { 0xcd1377be, 33 },
    { 0, 0 },
    { 0x902377c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f377c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0377cc, 93 },
    { 0, 0 },
    { 0xca0377ce, 66 },
    { 0x7cd377cf, 52 },
    { 0x26e377d0, 33 },
    { 0x5de377cf, 52 },
    { 0x50d377d2, 90 },
    { 0x368377d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60377d6, 95 },
    { 0, 0 },
    { 0x25b377d8, 61 },
    { 0x8a2377d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7377e3, 100 },
    { 0x5a9377e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89377ea, 61 },
    { 0, 0 },
    { 0x4db377ec, 95 },
    { 0xdc9377ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22377f1, 23 },
    { 0x942377f2, 90 },
    { 0x50377f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x407377f6, 23 },
    { 0x24d377f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x250377fa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366377ff, 88 },
    { 0x6ed37800, 33 },
    { 0, 0 },
    { 0xd0c37802, 76 },
    { 0, 0 },
    { 0x78337804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75237808, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763780b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c3780f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd537812, 33 },
    { 0, 0 },
    { 0xaec37814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693781a, 43 },
    { 0x37c3781a, 95 },
    { 0xb303781b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f337820, 90 },
    { 0xc3e37821, 75 },
    { 0xae037822, 98 },
    { 0x33937823, 23 },
    { 0xd6437824, 95 },
    { 0x9af37824, 33 },
    { 0xb8a37823, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2537829, 67 },
    { 0, 0 },
    { 0x623782b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb943782f, 68 },
    { 0x20a3782f, 95 },
    { 0xf0037831, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac37835, 95 },
    { 0x55337836, 91 },
    { 0, 0 },
    { 0xfab37838, 58 },
    { 0x52e37839, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843783e, 91 },
    { 0x6143783f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0337843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d37846, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba37849, 98 },
    { 0x44037849, 43 },
    { 0, 0 },
    { 0xc833784c, 76 },
    { 0xbfd3784d, 64 },
    { 0, 0 },
    { 0x3213784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd737856, 23 },
    { 0x13737857, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd3785a, 33 },
    { 0, 0 },
    { 0x4aa3785c, 98 },
    { 0, 0 },
    { 0x5ee3785e, 90 },
    { 0, 0 },
    { 0xd1437860, 90 },
    { 0x76337860, 95 },
    { 0xb8537862, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf737867, 85 },
    { 0, 0 },
    { 0xa7837869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3786e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b37876, 61 },
    { 0x62137877, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9713787b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3787e, 67 },
    { 0, 0 },
    { 0xc2c37880, 23 },
    { 0, 0 },
    { 0x6df37882, 98 },
    { 0xaf537883, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7037886, 33 },
    { 0xfea37886, 90 },
    { 0x42d37887, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be3788e, 52 },
    { 0xe23788f, 81 },
    { 0x6353788e, 90 },
    { 0xd5737891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b37897, 98 },
    { 0x3fd37898, 33 },
    { 0xc5e37899, 84 },
    { 0xbdd37899, 95 },
    { 0x1b13789b, 83 },
    { 0xedd3789b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc663789f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7378a4, 61 },
    { 0xd00378a5, 23 },
    { 0, 0 },
    { 0xc4e378a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf378b0, 61 },
    { 0xfe9378b0, 43 },
    { 0xfa6378b2, 95 },
    { 0, 0 },
    { 0x287378b4, 33 },
    { 0, 0 },
    { 0x4e8378b6, 61 },
    { 0, 0 },
    { 0xae5378b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x358378bb, 98 },
    { 0, 0 },
    { 0x72c378bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219378c2, 91 },
    { 0, 0 },
    { 0xca4378c4, 87 },
    { 0, 0 },
    { 0xd72378c6, 93 },
    { 0x9d378c7, 90 },
    { 0, 0 },
    { 0x914378c9, 82 },
    { 0x259378c9, 90 },
    { 0x3d9378cb, 99 },
    { 0xa9a378c9, 23 },
    { 0x5e1378c9, 85 },
    { 0x6ec378ce, 90 },
    { 0x6f5378cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35378d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb378d7, 61 },
    { 0x96378d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624378de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77378e4, 75 },
    { 0x388378e5, 35 },
    { 0x215378e6, 23 },
    { 0xeb0378e7, 78 },
    { 0x645378e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227378ec, 52 },
    { 0, 0 },
    { 0x5f1378ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe378f1, 33 },
    { 0x5a7378f1, 95 },
    { 0xaf8378f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a378f9, 23 },
    { 0, 0 },
    { 0x153378fb, 23 },
    { 0x67a378fb, 95 },
    { 0x240378fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17737903, 99 },
    { 0x1ee37904, 98 },
    { 0, 0 },
    { 0xd4c37906, 69 },
    { 0, 0 },
    { 0x30c37908, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad37910, 90 },
    { 0x7f37911, 66 },
    { 0x6d437912, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c537916, 90 },
    { 0xd6b37917, 88 },
    { 0xd4b37918, 34 },
    { 0xc1637916, 90 },
    { 0, 0 },
    { 0x4aa3791b, 58 },
    { 0x663791b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b137920, 98 },
    { 0, 0 },
    { 0x18a37922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f37925, 90 },
    { 0, 0 },
    { 0x69f37927, 98 },
    { 0xbc837928, 33 },
    { 0xf1037929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa123792c, 95 },
    { 0, 0 },
    { 0x5de3792e, 91 },
    { 0, 0 },
    { 0xf7837930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a37934, 66 },
    { 0, 0 },
    { 0xe1737936, 23 },
    { 0, 0 },
    { 0x5d37938, 90 },
    { 0xb0d37939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f13793d, 43 },
    { 0xfaf3793e, 90 },
    { 0, 0 },
    { 0x93837940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9737943, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b37947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3683794d, 95 },
    { 0x6293794e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce37951, 36 },
    { 0xc837951, 76 },
    { 0xd0e37953, 95 },
    { 0x20b37952, 33 },
    { 0xb2837951, 23 },
    { 0x3ed37956, 76 },
    { 0x9e037952, 61 },
    { 0, 0 },
    { 0x4f237959, 90 },
    { 0, 0 },
    { 0xec73795b, 33 },
    { 0x7433795c, 100 },
    { 0x7f03795d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb037960, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc537966, 34 },
    { 0x69e37966, 90 },
    { 0x27137968, 80 },
    { 0xea337966, 55 },
    { 0x72c3796a, 95 },
    { 0, 0 },
    { 0x553796c, 91 },
    { 0, 0 },
    { 0xa0b3796e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d37971, 90 },
    { 0x13837972, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55037976, 95 },
    { 0xb3f37976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde13797b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae63797e, 90 },
    { 0xde53797f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60937986, 76 },
    { 0x46237987, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb3798e, 33 },
    { 0x6523798f, 95 },
    { 0x23b37990, 33 },
    { 0x5e73798e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f337994, 99 },
    { 0, 0 },
    { 0x7f837996, 90 },
    { 0, 0 },
    { 0xcde37998, 95 },
    { 0, 0 },
    { 0xaf43799a, 52 },
    { 0x6893799b, 95 },
    { 0x96e3799c, 67 },
    { 0xee83799d, 87 },
    { 0xaf33799a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0379a2, 23 },
    { 0, 0 },
    { 0x5bd379a4, 100 },
    { 0x655379a5, 87 },
    { 0x918379a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df379aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c379ad, 90 },
    { 0, 0 },
    { 0x815379af, 33 },
    { 0x1b7379b0, 99 },
    { 0x589379b0, 17 },
    { 0xe57379af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c379b6, 55 },
    { 0, 0 },
    { 0xe29379b8, 53 },
    { 0, 0 },
    { 0x53e379ba, 17 },
    { 0x6da379ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46379be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321379c8, 93 },
    { 0xeb5379c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21379cc, 95 },
    { 0x111379cd, 61 },
    { 0xbf4379ce, 97 },
    { 0x664379ce, 95 },
    { 0x44d379cd, 96 },
    { 0x441379d1, 90 },
    { 0, 0 },
    { 0xc43379d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6379da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290379de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27379e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221379e6, 90 },
    { 0x869379e6, 76 },
    { 0x1a5379e8, 98 },
    { 0, 0 },
    { 0x609379ea, 33 },
    { 0x1ad379ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770379f2, 88 },
    { 0x5e0379f2, 43 },
    { 0x618379f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0379f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef37a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea37a07, 98 },
    { 0, 0 },
    { 0x26a37a09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537a0c, 95 },
    { 0, 0 },
    { 0x72137a0e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81737a12, 61 },
    { 0x43237a13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c37a17, 43 },
    { 0, 0 },
    { 0x29d37a19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2737a1c, 81 },
    { 0, 0 },
    { 0x46737a1e, 72 },
    { 0, 0 },
    { 0xc8537a20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e537a26, 95 },
    { 0x50837a27, 55 },
    { 0xe2b37a28, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d37a2b, 33 },
    { 0, 0 },
    { 0x36237a2d, 23 },
    { 0, 0 },
    { 0x4337a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23437a36, 61 },
    { 0, 0 },
    { 0x20137a38, 76 },
    { 0x2537a39, 23 },
    { 0x97937a3a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9637a3d, 61 },
    { 0, 0 },
    { 0x71037a3f, 78 },
    { 0, 0 },
    { 0x59437a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62437a45, 90 },
    { 0x69c37a45, 55 },
    { 0x77937a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef837a4d, 98 },
    { 0xbb937a4e, 61 },
    { 0x74537a4d, 55 },
    { 0, 0 },
    { 0xb2637a51, 76 },
    { 0xe537a51, 55 },
    { 0x97637a52, 78 },
    { 0xb9a37a54, 33 },
    { 0x3d737a54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22837a5b, 76 },
    { 0, 0 },
    { 0xc5b37a5d, 90 },
    { 0, 0 },
    { 0x77737a5f, 34 },
    { 0x3f337a60, 90 },
    { 0x6937a61, 95 },
    { 0x9ae37a62, 33 },
    { 0xcd137a5f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c737a66, 90 },
    { 0xecb37a66, 55 },
    { 0x14337a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7537a6c, 87 },
    { 0xd3237a6d, 98 },
    { 0xe0537a6e, 91 },
    { 0, 0 },
    { 0x14337a70, 90 },
    { 0x6b537a71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19237a78, 98 },
    { 0, 0 },
    { 0xdf837a7a, 98 },
    { 0x4a37a7b, 46 },
    { 0, 0 },
    { 0x19a37a7d, 98 },
    { 0xc4e37a7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77137a81, 33 },
    { 0, 0 },
    { 0x24637a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51637a88, 98 },
    { 0, 0 },
    { 0xf1037a8a, 61 },
    { 0, 0 },
    { 0xc2e37a8c, 46 },
    { 0x46237a8c, 90 },
    { 0xe1c37a8c, 23 },
    { 0xe2e37a8f, 90 },
    { 0xe8737a90, 93 },
    { 0, 0 },
    { 0x2b37a92, 93 },
    { 0, 0 },
    { 0x3ea37a94, 91 },
    { 0x5c37a95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0137a9c, 23 },
    { 0x38537a9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3037aa6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b37aa9, 61 },
    { 0, 0 },
    { 0xcd237aab, 81 },
    { 0xf9137aac, 78 },
    { 0xea637aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1537ab0, 23 },
    { 0xb4337ab1, 84 },
    { 0, 0 },
    { 0xb3a37ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4537ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0637abc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97737ac1, 92 },
    { 0xc1437ac2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db37ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58737ac8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x98637acb, 33 },
    { 0xdd437acc, 95 },
    { 0x78437acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa37ad1, 98 },
    { 0x54637ad2, 34 },
    { 0x5e437ad3, 76 },
    { 0x44537ad2, 95 },
    { 0xe2437ad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e237ad9, 93 },
    { 0x39c37ad9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2237ade, 46 },
    { 0x69837adf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12137ae5, 61 },
    { 0xbf537ae6, 34 },
    { 0xed737ae5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe37aeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76937af2, 67 },
    { 0xcd637af3, 61 },
    { 0, 0 },
    { 0x10037af5, 23 },
    { 0, 0 },
    { 0xed037af7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e837afa, 90 },
    { 0x3dc37afa, 34 },
    { 0, 0 },
    { 0x83637afd, 33 },
    { 0x63637afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e37b01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f437b04, 61 },
    { 0, 0 },
    { 0x8f37b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64037b0d, 33 },
    { 0x7ce37b0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a337b1a, 90 },
    { 0, 0 },
    { 0xad337b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b37b1f, 43 },
    { 0, 0 },
    { 0x30937b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35537b28, 61 },
    { 0xc4637b29, 76 },
    { 0xa9937b2a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e37b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b37b34, 95 },
    { 0xc2037b34, 23 },
    { 0, 0 },
    { 0x47d37b37, 95 },
    { 0xe9637b38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d37b3b, 78 },
    { 0x3b437b3c, 23 },
    { 0xc2237b3d, 23 },
    { 0x6de37b3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c937b41, 66 },
    { 0, 0 },
    { 0xee37b43, 96 },
    { 0, 0 },
    { 0x95837b45, 58 },
    { 0x32237b45, 23 },
    { 0xb8937b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3637b4c, 90 },
    { 0, 0 },
    { 0x4ab37b4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5037b52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75137b58, 61 },
    { 0xcdd37b58, 87 },
    { 0, 0 },
    { 0x3c137b5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59837b68, 95 },
    { 0, 0 },
    { 0x83d37b6a, 98 },
    { 0x21137b6b, 61 },
    { 0, 0 },
    { 0x26a37b6d, 61 },
    { 0xa7b37b6e, 55 },
    { 0x3f037b6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac37b77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e737b7b, 53 },
    { 0, 0 },
    { 0x39137b7d, 46 },
    { 0xf9437b7e, 33 },
    { 0, 0 },
    { 0x137b80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f37b83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be37b87, 98 },
    { 0xffd37b88, 100 },
    { 0x21337b88, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95137b93, 66 },
    { 0, 0 },
    { 0xd737b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea137b9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe37b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd37ba7, 98 },
    { 0x5d037ba7, 98 },
    { 0xbd537ba9, 99 },
    { 0xc8737baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c537bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d437bb5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a37bbb, 88 },
    { 0, 0 },
    { 0x5c437bbd, 98 },
    { 0xcf137bbe, 84 },
    { 0xb9337bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437bc2, 33 },
    { 0x30b37bc3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b37bcb, 95 },
    { 0x3f337bcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9337bd1, 84 },
    { 0xa7f37bd2, 56 },
    { 0xec537bd1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7837bd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc137be4, 33 },
    { 0x7d337be4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef37bec, 69 },
    { 0x8c337bed, 23 },
    { 0xb4f37bed, 90 },
    { 0, 0 },
    { 0xc4b37bf0, 90 },
    { 0, 0 },
    { 0xe0b37bf2, 66 },
    { 0x85b37bf3, 65 },
    { 0x92237bf3, 55 },
    { 0x81b37bf2, 90 },
    { 0, 0 },
    { 0x11b37bf7, 17 },
    { 0, 0 },
    { 0x9d537bf9, 90 },
    { 0xc5d37bf9, 90 },
    { 0, 0 },
    { 0x95c37bfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b37c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c237c05, 46 },
    { 0, 0 },
    { 0x8f237c07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c637c0a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7937c0f, 33 },
    { 0xa9f37c10, 33 },
    { 0, 0 },
    { 0x40237c12, 61 },
    { 0xbdf37c13, 88 },
    { 0x36d37c12, 33 },
    { 0, 0 },
    { 0x29537c16, 23 },
    { 0, 0 },
    { 0xac337c18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed37c1b, 61 },
    { 0x26737c1c, 90 },
    { 0xd2837c1d, 33 },
    { 0xf2737c1d, 100 },
    { 0x5be37c1f, 99 },
    { 0x58c37c1d, 99 },
    { 0xa6037c1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b937c25, 98 },
    { 0xa8137c26, 90 },
    { 0xa8937c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5437c2b, 90 },
    { 0x5737c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d37c34, 55 },
    { 0, 0 },
    { 0x79c37c36, 23 },
    { 0xd3637c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c37c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b837c42, 95 },
    { 0x1cf37c43, 78 },
    { 0xcd837c43, 98 },
    { 0x7a637c43, 23 },
    { 0x8fd37c45, 98 },
    { 0, 0 },
    { 0x8437c48, 55 },
    { 0x2d837c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a37c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90237c5b, 67 },
    { 0x98837c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28837c60, 98 },
    { 0, 0 },
    { 0x1a037c62, 33 },
    { 0, 0 },
    { 0xbd37c64, 90 },
    { 0xd4437c65, 55 },
    { 0x19b37c64, 43 },
    { 0x98237c65, 76 },
    { 0, 0 },
    { 0xbda37c69, 98 },
    { 0x2db37c6a, 55 },
    { 0, 0 },
    { 0x44037c6c, 43 },
    { 0xf4737c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7337c70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a037c78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc37c7b, 95 },
    { 0xec037c7c, 33 },
    { 0x49f37c7c, 76 },
    { 0, 0 },
    { 0xad437c7f, 84 },
    { 0x6fe37c80, 61 },
    { 0x12a37c7f, 93 },
    { 0x3fe37c82, 17 },
    { 0xae237c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a37c88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77637c8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5937c8f, 90 },
    { 0, 0 },
    { 0x52637c91, 90 },
    { 0x15737c92, 99 },
    { 0xe4237c92, 84 },
    { 0, 0 },
    { 0xeb337c95, 23 },
    { 0xf1e37c96, 43 },
    { 0xace37c96, 95 },
    { 0x1de37c98, 55 },
    { 0xefc37c98, 67 },
    { 0x50337c99, 4 },
    { 0x33537c98, 95 },
    { 0x92337c9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b37c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef937ca4, 55 },
    { 0x30c37ca5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x29637ca8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b137cad, 23 },
    { 0xf2237cae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56937cb3, 33 },
    { 0xda037cb4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b37cb9, 66 },
    { 0xf2f37cb9, 95 },
    { 0x32237cbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd137cbf, 66 },
    { 0xafa37cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40337cc8, 95 },
    { 0x28037cc9, 76 },
    { 0x12e37cc9, 75 },
    { 0x7be37cc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b737cce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd737cd1, 33 },
    { 0x81237cd2, 23 },
    { 0, 0 },
    { 0xd5c37cd4, 93 },
    { 0xa7937cd5, 67 },
    { 0, 0 },
    { 0xec937cd7, 33 },
    { 0x6c237cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d37ce0, 90 },
    { 0x3be37ce0, 95 },
    { 0x4f937ce2, 93 },
    { 0x537ce0, 34 },
    { 0x83b37ce4, 99 },
    { 0, 0 },
    { 0x20837ce6, 90 },
    { 0x3037ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81337cee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f937cf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2737cf4, 43 },
    { 0xf0137cf5, 23 },
    { 0x53237cf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1237cfb, 98 },
    { 0xb3b37cfb, 67 },
    { 0x9b37cfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5337d02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8037d08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc37d0e, 90 },
    { 0, 0 },
    { 0xa8837d10, 90 },
    { 0xe4137d10, 90 },
    { 0xc8437d11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4737d15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc837d1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42037d20, 61 },
    { 0x6e837d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e37d24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc937d27, 55 },
    { 0x7b237d28, 23 },
    { 0x1637d29, 99 },
    { 0x40137d2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66537d2e, 95 },
    { 0x61b37d2f, 23 },
    { 0x13f37d2e, 61 },
    { 0xb8337d31, 93 },
    { 0x89737d32, 38 },
    { 0xfbc37d30, 66 },
    { 0x65837d34, 98 },
    { 0x26d37d34, 85 },
    { 0x9537d36, 88 },
    { 0xd0137d34, 98 },
    { 0x81737d38, 23 },
    { 0x93c37d34, 90 },
    { 0, 0 },
    { 0x1b837d3b, 98 },
    { 0x44d37d3c, 61 },
    { 0, 0 },
    { 0x80337d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea37d44, 33 },
    { 0xb8d37d45, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b37d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e237d4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc37d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac37d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc37d5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69737d66, 76 },
    { 0x77737d67, 33 },
    { 0xa0b37d68, 96 },
    { 0xb5b37d66, 66 },
    { 0, 0 },
    { 0xae837d6b, 90 },
    { 0, 0 },
    { 0xdaa37d6d, 94 },
    { 0, 0 },
    { 0xd1337d6f, 98 },
    { 0x56537d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb37d75, 98 },
    { 0xd6e37d76, 33 },
    { 0, 0 },
    { 0x65a37d78, 81 },
    { 0xbec37d79, 90 },
    { 0, 0 },
    { 0x88237d7b, 100 },
    { 0xa0737d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b37d80, 43 },
    { 0x9e37d81, 61 },
    { 0x74037d82, 23 },
    { 0xdbb37d81, 47 },
    { 0, 0 },
    { 0x84d37d85, 90 },
    { 0xab637d85, 76 },
    { 0, 0 },
    { 0x35f37d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8437d8e, 99 },
    { 0x7f737d8f, 46 },
    { 0x5c637d90, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50237da3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e337da8, 23 },
    { 0x66537da8, 90 },
    { 0xef937da9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9137db1, 84 },
    { 0x5c837db2, 52 },
    { 0x8ca37db3, 53 },
    { 0x62137db2, 91 },
    { 0x95937db5, 95 },
    { 0x8e637db6, 90 },
    { 0xaa237db5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf037dbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa37dc2, 61 },
    { 0x5ff37dc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f37dc9, 66 },
    { 0x13c37dca, 78 },
    { 0xac237dc9, 91 },
    { 0x87a37dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f37dcf, 33 },
    { 0xe4837dd0, 46 },
    { 0x64c37dcf, 23 },
    { 0xe6337dd2, 23 },
    { 0x2dc37dd3, 33 },
    { 0x33337dd4, 43 },
    { 0x70d37dd4, 66 },
    { 0x11a37dd2, 75 },
    { 0x4cf37dd3, 55 },
    { 0, 0 },
    { 0x73337dd9, 66 },
    { 0x92d37dd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25537ddd, 33 },
    { 0x90437dde, 33 },
    { 0, 0 },
    { 0x1b837de0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59637de4, 34 },
    { 0, 0 },
    { 0x48b37de6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6937dee, 66 },
    { 0, 0 },
    { 0xa1a37df0, 95 },
    { 0xe5537df1, 98 },
    { 0x9fe37df2, 34 },
    { 0x83f37df3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d037df6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be37dfa, 61 },
    { 0, 0 },
    { 0x20c37dfc, 90 },
    { 0x2f637dfd, 90 },
    { 0xcf137dfc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf37e04, 85 },
    { 0xb1537e04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb937e08, 33 },
    { 0, 0 },
    { 0x77d37e0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d37e10, 98 },
    { 0x2af37e11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe37e15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c837e1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b37e1e, 33 },
    { 0x2f837e1f, 23 },
    { 0xaa537e1f, 76 },
    { 0x3e137e1f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc237e27, 27 },
    { 0x4cc37e27, 90 },
    { 0x7e537e29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16337e2d, 90 },
    { 0xa0a37e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b37e31, 89 },
    { 0x30f37e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f37e35, 91 },
    { 0x4a137e35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b137e3a, 65 },
    { 0x5bd37e3a, 90 },
    { 0x2ed37e3c, 33 },
    { 0x5cf37e3c, 88 },
    { 0x35c37e3a, 91 },
    { 0x3f337e3d, 61 },
    { 0xfe237e3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c37e43, 87 },
    { 0, 0 },
    { 0xab637e45, 90 },
    { 0xbf637e45, 23 },
    { 0xacd37e45, 55 },
    { 0x9af37e48, 95 },
    { 0x7ec37e49, 23 },
    { 0, 0 },
    { 0xd1837e4b, 95 },
    { 0, 0 },
    { 0x26237e4d, 33 },
    { 0x93737e4e, 46 },
    { 0xfba37e4d, 17 },
    { 0x50e37e50, 61 },
    { 0x4f37e4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa37e54, 85 },
    { 0x5e737e55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x54337e58, 90 },
    { 0, 0 },
    { 0x6dd37e5a, 61 },
    { 0, 0 },
    { 0x6e637e5c, 66 },
    { 0, 0 },
    { 0x93237e5e, 23 },
    { 0x21237e5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44237e65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d237e6c, 64 },
    { 0xc1f37e6d, 33 },
    { 0x50937e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30837e77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f37e7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e37e7f, 23 },
    { 0xbcf37e80, 90 },
    { 0xdc137e81, 98 },
    { 0xbb437e81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xded37e85, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab37e88, 93 },
    { 0x12737e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57537e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1037e9b, 33 },
    { 0xd0437e9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae437ea3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8237ea7, 38 },
    { 0, 0 },
    { 0x6fe37ea9, 23 },
    { 0, 0 },
    { 0xac537eab, 95 },
    { 0, 0 },
    { 0xbdc37ead, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x89437eb0, 55 },
    { 0, 0 },
    { 0x9d337eb2, 87 },
    { 0, 0 },
    { 0x1b37eb4, 87 },
    { 0xc9a37eb5, 61 },
    { 0x7f437eb6, 55 },
    { 0x6a37eb6, 34 },
    { 0xcbd37eb8, 98 },
    { 0x86237eb7, 46 },
    { 0x25737eb4, 91 },
    { 0x58b37eb4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f37ec6, 43 },
    { 0, 0 },
    { 0x3b237ec8, 46 },
    { 0x9f537ec8, 61 },
    { 0x7a737ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9237ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c137ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23537ed5, 34 },
    { 0, 0 },
    { 0xf5d37ed7, 46 },
    { 0x5a637ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92437edd, 61 },
    { 0x2b437edd, 95 },
    { 0x7b537edf, 55 },
    { 0, 0 },
    { 0x6f37ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb037ee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d537eeb, 95 },
    { 0xb8b37eeb, 33 },
    { 0xe4d37eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74937ef3, 55 },
    { 0xc337ef4, 76 },
    { 0, 0 },
    { 0xa7337ef6, 33 },
    { 0x92a37ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf837efb, 78 },
    { 0x8a037efc, 95 },
    { 0, 0 },
    { 0xa3f37efe, 33 },
    { 0x13137eff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb137f02, 90 },
    { 0x58a37f02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58037f0d, 76 },
    { 0, 0 },
    { 0xb4037f0f, 17 },
    { 0x70637f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab37f14, 33 },
    { 0x57137f15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad37f1d, 93 },
    { 0xaa637f1d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d937f24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95337f2a, 66 },
    { 0xc137f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f37f2e, 80 },
    { 0x47d37f2f, 61 },
    { 0x49337f30, 93 },
    { 0xed337f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f37f35, 95 },
    { 0x66337f36, 23 },
    { 0x21c37f37, 66 },
    { 0x37f37f36, 98 },
    { 0x49137f39, 98 },
    { 0x2d437f3a, 66 },
    { 0x15b37f3a, 34 },
    { 0xcc537f3c, 23 },
    { 0xc1437f3a, 33 },
    { 0xff837f3a, 95 },
    { 0xe7e37f3f, 98 },
    { 0x7e337f3e, 33 },
    { 0x6ed37f41, 33 },
    { 0x78937f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6737f45, 23 },
    { 0xcfd37f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c37f4c, 95 },
    { 0, 0 },
    { 0x91437f4e, 66 },
    { 0x90037f4e, 95 },
    { 0xd037f4e, 61 },
    { 0xa2d37f51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc37f5a, 69 },
    { 0x8d337f5b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29037f64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e837f6a, 43 },
    { 0x4d437f6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x61937f6e, 17 },
    { 0x50537f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc737f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7037f7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2637f82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d037f88, 23 },
    { 0, 0 },
    { 0x6b37f8a, 23 },
    { 0x87337f8b, 99 },
    { 0x6cb37f8b, 95 },
    { 0x1e137f8d, 77 },
    { 0xc4b37f8c, 98 },
    { 0xf8037f8f, 34 },
    { 0, 0 },
    { 0xc4737f91, 67 },
    { 0x9d637f91, 95 },
    { 0xc6437f93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c37f99, 76 },
    { 0, 0 },
    { 0x72f37f9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5337f9e, 34 },
    { 0, 0 },
    { 0xffb37fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51837fa7, 90 },
    { 0, 0 },
    { 0x54237fa9, 66 },
    { 0x6c137faa, 33 },
    { 0x85037fab, 87 },
    { 0xbe637fab, 93 },
    { 0xe7237fad, 95 },
    { 0xcf37fae, 23 },
    { 0x9fd37faf, 67 },
    { 0x67a37fb0, 66 },
    { 0xa1c37fb0, 95 },
    { 0x2e37fae, 51 },
    { 0, 0 },
    { 0x97b37fb4, 90 },
    { 0xf4f37fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e37fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32237fbe, 98 },
    { 0x41237fbf, 46 },
    { 0x60a37fc0, 95 },
    { 0, 0 },
    { 0xd8337fc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4237fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c237fca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a737fcd, 98 },
    { 0x8bd37fce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a37fd4, 77 },
    { 0x91237fd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b37fdc, 95 },
    { 0x43b37fdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a37fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a37fe5, 98 },
    { 0, 0 },
    { 0x64d37fe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x68237fea, 90 },
    { 0xf6a37feb, 88 },
    { 0x6db37fea, 33 },
    { 0x51337fed, 85 },
    { 0x64037fed, 85 },
    { 0, 0 },
    { 0x50437ff0, 95 },
    { 0xde637ff1, 23 },
    { 0xb1837ff1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32137ff5, 43 },
    { 0x62f37ff6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0237ff9, 33 },
    { 0xbb637ffa, 90 },
    { 0xb9337ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438004, 34 },
    { 0x16138004, 23 },
    { 0, 0 },
    { 0x99338007, 93 },
    { 0x9b338008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe723800b, 76 },
    { 0x9943800b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a23800f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7338012, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x52138015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25938018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72738020, 66 },
    { 0x95e38021, 99 },
    { 0xf4438020, 98 },
    { 0x88a38023, 34 },
    { 0, 0 },
    { 0x23b38025, 61 },
    { 0x27638026, 98 },
    { 0x73d38027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3802c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87538034, 88 },
    { 0x81838035, 69 },
    { 0x37438036, 95 },
    { 0xba038034, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3803f, 78 },
    { 0xab938040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8138043, 33 },
    { 0, 0 },
    { 0xe1138045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed38048, 90 },
    { 0, 0 },
    { 0x9123804a, 76 },
    { 0x6c43804a, 90 },
    { 0, 0 },
    { 0x2833804d, 81 },
    { 0x26f3804e, 61 },
    { 0, 0 },
    { 0x23b38050, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e138054, 33 },
    { 0x54a38055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3805a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a38063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d38066, 46 },
    { 0x3bd38066, 76 },
    { 0x3a538066, 52 },
    { 0, 0 },
    { 0x64c3806a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14738073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b738078, 90 },
    { 0x67538079, 86 },
    { 0x2173807a, 33 },
    { 0x46b38078, 84 },
    { 0x4773807c, 34 },
    { 0x1173807a, 76 },
    { 0, 0 },
    { 0x7593807f, 90 },
    { 0xb7938080, 23 },
    { 0x68d38081, 90 },
    { 0x7ea38082, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d38088, 61 },
    { 0x85838088, 100 },
    { 0x7783808a, 98 },
    { 0x21d3808b, 33 },
    { 0, 0 },
    { 0xcc33808d, 95 },
    { 0x5e63808e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1138091, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838095, 69 },
    { 0, 0 },
    { 0x1c338097, 33 },
    { 0, 0 },
    { 0xcdf38099, 67 },
    { 0x9b33809a, 98 },
    { 0x79b38099, 98 },
    { 0x6863809c, 34 },
    { 0x7db3809c, 76 },
    { 0, 0 },
    { 0x673809f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8380a2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e380a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0380aa, 85 },
    { 0, 0 },
    { 0x59380ac, 61 },
    { 0x733380ad, 33 },
    { 0xe8b380ac, 33 },
    { 0xa12380af, 90 },
    { 0x70e380af, 98 },
    { 0x165380ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a380b6, 84 },
    { 0, 0 },
    { 0xf4380b8, 61 },
    { 0x429380b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e380bc, 66 },
    { 0x597380bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x688380c0, 61 },
    { 0x4a5380c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37380c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b380cc, 76 },
    { 0x998380cd, 85 },
    { 0, 0 },
    { 0x609380cf, 17 },
    { 0xa4b380cf, 33 },
    { 0, 0 },
    { 0x9e6380d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88380d6, 55 },
    { 0xa87380d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450380de, 95 },
    { 0, 0 },
    { 0x2a1380e0, 93 },
    { 0, 0 },
    { 0xced380e2, 95 },
    { 0x364380e3, 93 },
    { 0xf8d380e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5380e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73380ed, 76 },
    { 0xe8d380ee, 90 },
    { 0x203380ef, 99 },
    { 0xc41380f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x923380f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c380fa, 23 },
    { 0, 0 },
    { 0x3ab380fc, 76 },
    { 0, 0 },
    { 0xeac380fe, 98 },
    { 0xc00380fe, 33 },
    { 0x8dc38100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6938104, 17 },
    { 0xfbf38105, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df38110, 90 },
    { 0, 0 },
    { 0xb1538112, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed38115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71338119, 37 },
    { 0, 0 },
    { 0xa63811b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd38123, 75 },
    { 0xf9a38124, 33 },
    { 0, 0 },
    { 0xc2e38126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b33812a, 40 },
    { 0x4cd3812b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4953812e, 90 },
    { 0, 0 },
    { 0xe9738130, 52 },
    { 0xa5c38130, 65 },
    { 0, 0 },
    { 0xd738133, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f23813a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e38140, 66 },
    { 0, 0 },
    { 0xdb138142, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f638145, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33338148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7703814d, 33 },
    { 0xbc63814e, 33 },
    { 0x6303814f, 90 },
    { 0xeb3814f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893815d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f38165, 95 },
    { 0x9d38166, 98 },
    { 0x67e38165, 87 },
    { 0x9d638168, 95 },
    { 0x2ec38168, 67 },
    { 0, 0 },
    { 0x9703816b, 43 },
    { 0xfb83816b, 33 },
    { 0, 0 },
    { 0x3d53816e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b38175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4083817a, 23 },
    { 0, 0 },
    { 0x8213817c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e38180, 23 },
    { 0x65338180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd133818a, 46 },
    { 0x6483818a, 61 },
    { 0x1103818b, 39 },
    { 0x7273818d, 61 },
    { 0xb1f3818d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32438195, 33 },
    { 0xa0b38196, 43 },
    { 0x2ae38197, 61 },
    { 0x88d38198, 90 },
    { 0x5b938199, 52 },
    { 0x9b738198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a13819f, 68 },
    { 0x622381a0, 46 },
    { 0x96a381a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x494381a4, 33 },
    { 0xc4f381a4, 90 },
    { 0x1f6381a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8381a9, 98 },
    { 0, 0 },
    { 0xc1b381ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6381b3, 61 },
    { 0xdfa381b3, 77 },
    { 0, 0 },
    { 0x751381b6, 90 },
    { 0x4df381b7, 33 },
    { 0x448381b7, 33 },
    { 0x66e381b6, 90 },
    { 0x598381b8, 90 },
    { 0xc06381bb, 52 },
    { 0x4f4381bc, 90 },
    { 0xb05381bd, 33 },
    { 0x5c381be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f381c3, 33 },
    { 0x8b0381c3, 34 },
    { 0x8b381c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59381cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c381ce, 36 },
    { 0x4a5381cf, 66 },
    { 0x412381cf, 95 },
    { 0xeee381d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489381d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5381dd, 95 },
    { 0x333381dd, 48 },
    { 0, 0 },
    { 0xcc3381e0, 17 },
    { 0xe06381e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780381e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852381ee, 23 },
    { 0xf7c381ee, 43 },
    { 0x478381ef, 33 },
    { 0xe7381f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43381f5, 99 },
    { 0xa1a381f6, 61 },
    { 0x898381f7, 90 },
    { 0xa4b381f8, 90 },
    { 0xaee381f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831381fe, 43 },
    { 0xb4b381ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba538203, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb238206, 93 },
    { 0, 0 },
    { 0x9b438208, 46 },
    { 0x2e238208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd3820c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd38210, 33 },
    { 0x85e38210, 90 },
    { 0, 0 },
    { 0x3fb38213, 38 },
    { 0xbcd38214, 90 },
    { 0x74538215, 61 },
    { 0x4d738216, 43 },
    { 0xaee38216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3821b, 33 },
    { 0x7853821c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf038220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d38223, 34 },
    { 0x49038223, 23 },
    { 0x12838223, 98 },
    { 0x2b938224, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713822c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92738238, 71 },
    { 0x16838239, 61 },
    { 0x9253823a, 77 },
    { 0xe438239, 40 },
    { 0xba03823c, 61 },
    { 0x7bc3823d, 61 },
    { 0xe4c38239, 33 },
    { 0xd023823f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e638243, 91 },
    { 0x7c238244, 34 },
    { 0x31438243, 23 },
    { 0x40638244, 90 },
    { 0x13138246, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38238251, 93 },
    { 0, 0 },
    { 0x88438253, 23 },
    { 0x3ae38253, 99 },
    { 0xd6538255, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb738258, 66 },
    { 0, 0 },
    { 0x1423825a, 64 },
    { 0xf543825b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f3825e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9238261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c638267, 34 },
    { 0xb7438268, 76 },
    { 0, 0 },
    { 0xfef3826a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x933826d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x48038270, 61 },
    { 0, 0 },
    { 0xd1c38272, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3827b, 98 },
    { 0x66d3827c, 34 },
    { 0x20d3827b, 33 },
    { 0, 0 },
    { 0xddb3827f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64438283, 61 },
    { 0xfe438284, 33 },
    { 0x7ae38284, 33 },
    { 0x8ea38286, 80 },
    { 0x4ae38286, 90 },
    { 0x20138288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4033828d, 66 },
    { 0x6f63828d, 98 },
    { 0x14f3828d, 61 },
    { 0x38d38290, 98 },
    { 0x96d3828f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f38294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d538299, 95 },
    { 0, 0 },
    { 0x1d63829b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2193829f, 85 },
    { 0xcb23829f, 66 },
    { 0xbe3382a1, 84 },
    { 0, 0 },
    { 0x753382a3, 55 },
    { 0x677382a4, 33 },
    { 0x94c382a5, 84 },
    { 0, 0 },
    { 0xbb6382a7, 76 },
    { 0x4e9382a8, 34 },
    { 0x754382a7, 91 },
    { 0x814382a9, 61 },
    { 0x5ad382ab, 90 },
    { 0x59f382ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8382af, 88 },
    { 0x7c382b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8382b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3382b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b382ba, 95 },
    { 0x50d382ba, 78 },
    { 0xe382bc, 99 },
    { 0, 0 },
    { 0x56e382be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48382c7, 23 },
    { 0xdc4382c7, 61 },
    { 0xbd5382c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4382cc, 91 },
    { 0x61382cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x502382d0, 90 },
    { 0, 0 },
    { 0xa48382d2, 90 },
    { 0x872382d3, 90 },
    { 0xe20382d3, 90 },
    { 0, 0 },
    { 0x257382d6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d382da, 88 },
    { 0xee2382db, 37 },
    { 0xc51382da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25382e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x545382e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b382e7, 87 },
    { 0, 0 },
    { 0x5cc382e9, 99 },
    { 0xee5382ea, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9382ed, 69 },
    { 0, 0 },
    { 0xa3e382ef, 100 },
    { 0x4cc382f0, 98 },
    { 0, 0 },
    { 0xe3a382f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7382f5, 23 },
    { 0, 0 },
    { 0x904382f7, 61 },
    { 0x659382f7, 34 },
    { 0x68382f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec382fc, 33 },
    { 0x76e382fc, 95 },
    { 0x475382fe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74238303, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a38307, 23 },
    { 0xd238308, 93 },
    { 0xa4338309, 87 },
    { 0x6fd38309, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb38312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a738315, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f38319, 46 },
    { 0, 0 },
    { 0x603831b, 90 },
    { 0, 0 },
    { 0xe903831d, 23 },
    { 0x5103831e, 19 },
    { 0x8513831f, 98 },
    { 0x86038320, 61 },
    { 0, 0 },
    { 0x98238322, 98 },
    { 0x3e138323, 93 },
    { 0x5f038322, 66 },
    { 0xb0b38324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083832a, 52 },
    { 0, 0 },
    { 0x2073832c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1038330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9838333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae38338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d23833c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63238341, 90 },
    { 0, 0 },
    { 0x22538343, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4238348, 37 },
    { 0x88c38349, 98 },
    { 0, 0 },
    { 0xaf03834b, 98 },
    { 0, 0 },
    { 0x4283834d, 90 },
    { 0x8193834e, 76 },
    { 0x2853834d, 95 },
    { 0, 0 },
    { 0x82438351, 90 },
    { 0xce38352, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b38355, 23 },
    { 0xc2b38356, 95 },
    { 0x1da38356, 23 },
    { 0x38838355, 90 },
    { 0, 0 },
    { 0xcdd3835a, 52 },
    { 0x4553835b, 98 },
    { 0x2083835c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c3835f, 66 },
    { 0xf9d38360, 95 },
    { 0xa8938361, 91 },
    { 0, 0 },
    { 0x6538363, 98 },
    { 0xd8a38364, 61 },
    { 0x82e38365, 33 },
    { 0xa0d38366, 90 },
    { 0x10138363, 40 },
    { 0xb0638368, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd3836b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b83836e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd38374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c938377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b3837b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f13837e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b938382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc538385, 33 },
    { 0xdf438386, 17 },
    { 0x8ff38387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53838d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c38391, 92 },
    { 0x6bd38392, 98 },
    { 0x1fa38393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d38398, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623839e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01383a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a383a5, 76 },
    { 0x1f6383a6, 46 },
    { 0x713383a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483383ab, 87 },
    { 0xccc383ac, 76 },
    { 0, 0 },
    { 0xaac383ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2383b3, 23 },
    { 0xce3383b3, 76 },
    { 0xc85383b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177383ba, 61 },
    { 0xef0383bb, 98 },
    { 0x6ba383bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x905383bf, 76 },
    { 0x95e383c0, 61 },
    { 0x220383c1, 87 },
    { 0xfd5383c0, 55 },
    { 0x793383c3, 61 },
    { 0xc43383c4, 88 },
    { 0xc71383c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c383cd, 33 },
    { 0, 0 },
    { 0x16f383cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa383d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39383d8, 95 },
    { 0xb69383d9, 61 },
    { 0, 0 },
    { 0x616383db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5383df, 33 },
    { 0xd8b383e0, 98 },
    { 0xf2c383e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a383e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e383ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3383f1, 76 },
    { 0xa62383f2, 98 },
    { 0xb5a383f2, 23 },
    { 0x269383f4, 98 },
    { 0xbfa383f5, 33 },
    { 0xdd5383f6, 33 },
    { 0x7c6383f6, 93 },
    { 0xa3a383f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd383fb, 91 },
    { 0, 0 },
    { 0xfd2383fd, 78 },
    { 0x6bb383fd, 99 },
    { 0x6ab383ff, 23 },
    { 0, 0 },
    { 0x3d138401, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4038409, 93 },
    { 0x8da3840a, 98 },
    { 0x6823840a, 90 },
    { 0, 0 },
    { 0x633840d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b338410, 98 },
    { 0x5db38410, 61 },
    { 0xc9638412, 33 },
    { 0xb938413, 61 },
    { 0x27038414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423841e, 95 },
    { 0x36a3841e, 90 },
    { 0, 0 },
    { 0x7a338421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c38427, 66 },
    { 0x37b38428, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33842b, 34 },
    { 0xe1a3842c, 78 },
    { 0xd353842d, 23 },
    { 0xb7c3842c, 98 },
    { 0, 0 },
    { 0x9ed38430, 23 },
    { 0, 0 },
    { 0x69338432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da38436, 64 },
    { 0, 0 },
    { 0xdc438438, 81 },
    { 0xe3838439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12838440, 23 },
    { 0x7cc38441, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec038446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9138449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23844d, 81 },
    { 0x4753844e, 90 },
    { 0xf373844f, 85 },
    { 0x2b83844e, 33 },
    { 0x4713844f, 53 },
    { 0x8ce3844f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70638457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43845a, 98 },
    { 0x1f33845a, 95 },
    { 0xd9f3845b, 61 },
    { 0x1dc3845d, 61 },
    { 0xab53845c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa538468, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f438473, 52 },
    { 0xd1538474, 90 },
    { 0xcea38473, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113847c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb613847f, 23 },
    { 0xa2238480, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc38483, 80 },
    { 0xaec38483, 43 },
    { 0xc6e38485, 23 },
    { 0x68038486, 61 },
    { 0x88e38487, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb973848a, 55 },
    { 0x68d3848b, 90 },
    { 0x9de3848c, 33 },
    { 0x4103848d, 98 },
    { 0, 0 },
    { 0x6a13848f, 76 },
    { 0xc8038490, 76 },
    { 0x738491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49938494, 87 },
    { 0x39738495, 76 },
    { 0x2ca38495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8583849a, 95 },
    { 0x8873849b, 42 },
    { 0xe953849b, 33 },
    { 0x8653849d, 81 },
    { 0, 0 },
    { 0x44a3849f, 90 },
    { 0xda1384a0, 81 },
    { 0x374384a1, 90 },
    { 0x1e6384a2, 46 },
    { 0xcba384a0, 67 },
    { 0xd8f384a4, 100 },
    { 0xc9b384a2, 61 },
    { 0, 0 },
    { 0x7f6384a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d384aa, 76 },
    { 0x653384aa, 23 },
    { 0x592384ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8384b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3384b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4384bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a384ca, 34 },
    { 0, 0 },
    { 0x7e3384cc, 61 },
    { 0xd88384cc, 76 },
    { 0, 0 },
    { 0xe5c384cf, 33 },
    { 0xc8f384d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba384d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503384d7, 78 },
    { 0x7fa384d8, 98 },
    { 0, 0 },
    { 0x6ae384da, 39 },
    { 0xc2b384db, 33 },
    { 0xbf0384da, 77 },
    { 0x5fc384db, 55 },
    { 0x25384de, 88 },
    { 0xf67384df, 23 },
    { 0x642384dc, 95 },
    { 0xaa5384df, 95 },
    { 0, 0 },
    { 0xd11384e3, 61 },
    { 0x556384e3, 98 },
    { 0x1cf384e5, 68 },
    { 0xf0a384e5, 61 },
    { 0x541384e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b384ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48384ee, 23 },
    { 0, 0 },
    { 0x3bc384f0, 95 },
    { 0, 0 },
    { 0x96e384f2, 61 },
    { 0x595384f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c384fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef38504, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca38507, 33 },
    { 0x9c738508, 55 },
    { 0x53438508, 76 },
    { 0x6be38509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8423850e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1638514, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a38518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3851c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa138523, 43 },
    { 0xd3f38524, 23 },
    { 0, 0 },
    { 0x6ca38526, 90 },
    { 0xe338527, 61 },
    { 0, 0 },
    { 0xd4938529, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f93852f, 91 },
    { 0x61138530, 84 },
    { 0x3f238531, 23 },
    { 0x8e238532, 23 },
    { 0, 0 },
    { 0xf8838534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f53853d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4838540, 55 },
    { 0x35038541, 100 },
    { 0xb8338541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd038545, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27538548, 68 },
    { 0x22d38548, 55 },
    { 0, 0 },
    { 0x4383854b, 23 },
    { 0, 0 },
    { 0xbcd3854d, 55 },
    { 0x9553854e, 95 },
    { 0, 0 },
    { 0xf3338550, 81 },
    { 0, 0 },
    { 0xc1838552, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b038556, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x80138559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa633855c, 61 },
    { 0x78e3855d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83038561, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae438566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d3856e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e38572, 33 },
    { 0xc4638573, 77 },
    { 0x31c38573, 61 },
    { 0x25438574, 98 },
    { 0xb9b38574, 20 },
    { 0xa6538577, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803857a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033857e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7438584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x50138587, 91 },
    { 0xa6738588, 33 },
    { 0xcd738589, 61 },
    { 0xf313858a, 90 },
    { 0, 0 },
    { 0x33858c, 61 },
    { 0x10c3858d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d438590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a38594, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb073859b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3859e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68385a5, 91 },
    { 0xb56385a6, 95 },
    { 0, 0 },
    { 0x80f385a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66385ab, 23 },
    { 0xecb385ab, 75 },
    { 0xd4a385ad, 34 },
    { 0x639385ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae385b1, 75 },
    { 0x91c385b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1385b5, 90 },
    { 0, 0 },
    { 0xaa1385b7, 95 },
    { 0xb01385b8, 95 },
    { 0x7c385b9, 91 },
    { 0x7e0385b7, 55 },
    { 0x152385ba, 61 },
    { 0x366385b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f385c2, 90 },
    { 0xb15385c2, 46 },
    { 0, 0 },
    { 0xd9e385c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d385d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x153385d4, 34 },
    { 0xaac385d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a385dc, 76 },
    { 0x782385dd, 93 },
    { 0xde0385dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556385e9, 33 },
    { 0xd0c385ea, 66 },
    { 0, 0 },
    { 0x47e385ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66385ef, 61 },
    { 0x118385f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c385f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x766385fc, 52 },
    { 0x62c385fd, 34 },
    { 0x415385fc, 46 },
    { 0, 0 },
    { 0xd2c38600, 99 },
    { 0, 0 },
    { 0x95f38602, 100 },
    { 0xf9c38603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f63860a, 93 },
    { 0xb483860a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d3860f, 55 },
    { 0xb9638610, 23 },
    { 0xac838611, 91 },
    { 0, 0 },
    { 0xab738613, 23 },
    { 0x74f38614, 55 },
    { 0x49c38614, 23 },
    { 0x8e038616, 46 },
    { 0xd2038617, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993861a, 90 },
    { 0, 0 },
    { 0x1373861c, 98 },
    { 0x55c3861d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a38620, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae538626, 76 },
    { 0x11738627, 43 },
    { 0x24c38628, 90 },
    { 0xb8438629, 90 },
    { 0x52a3862a, 33 },
    { 0x5a438629, 90 },
    { 0, 0 },
    { 0xae33862d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b38632, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc238637, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda23863d, 90 },
    { 0x5e3863d, 34 },
    { 0x6b23863e, 98 },
    { 0, 0 },
    { 0xca438641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f538647, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673864a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3864d, 93 },
    { 0xadf3864e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0038656, 43 },
    { 0x5f38657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33865d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc238663, 55 },
    { 0x1d038664, 81 },
    { 0x8da38665, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11938669, 93 },
    { 0, 0 },
    { 0xfed3866b, 93 },
    { 0xf163866c, 93 },
    { 0xd173866d, 61 },
    { 0, 0 },
    { 0xa333866f, 98 },
    { 0xef938670, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2238675, 23 },
    { 0x81338676, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be38685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d838688, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b3868d, 23 },
    { 0x5543868e, 81 },
    { 0xde23868f, 76 },
    { 0x1693868f, 34 },
    { 0x6103868e, 61 },
    { 0x20c38692, 55 },
    { 0, 0 },
    { 0x90338694, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc38697, 61 },
    { 0xe5e38697, 90 },
    { 0, 0 },
    { 0xcfd3869a, 33 },
    { 0, 0 },
    { 0xb5b3869c, 56 },
    { 0, 0 },
    { 0xd7d3869e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704386a4, 78 },
    { 0x37b386a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1386a9, 78 },
    { 0, 0 },
    { 0xa92386ab, 33 },
    { 0x80f386ac, 34 },
    { 0, 0 },
    { 0x75f386ae, 69 },
    { 0xbd1386ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66386b4, 85 },
    { 0, 0 },
    { 0xa0e386b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192386bb, 38 },
    { 0xbbf386bc, 95 },
    { 0x845386bd, 23 },
    { 0x1c2386be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159386c2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753386ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54386d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9386d5, 33 },
    { 0, 0 },
    { 0x8c0386d7, 33 },
    { 0x8f5386d8, 90 },
    { 0, 0 },
    { 0x303386da, 90 },
    { 0, 0 },
    { 0xa07386dc, 33 },
    { 0xde2386dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f386e0, 95 },
    { 0xd73386e1, 19 },
    { 0, 0 },
    { 0x18d386e3, 52 },
    { 0x4a9386e4, 61 },
    { 0x2f386e4, 76 },
    { 0xcbf386e6, 95 },
    { 0x3d386e7, 90 },
    { 0x75a386e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd386ed, 95 },
    { 0, 0 },
    { 0xced386ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337386f3, 33 },
    { 0x5fc386f3, 77 },
    { 0x915386f4, 46 },
    { 0x8e1386f5, 90 },
    { 0x8cb386f7, 33 },
    { 0xd7b386f5, 95 },
    { 0x7e6386f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c38707, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3870a, 33 },
    { 0x133870b, 98 },
    { 0x3b63870c, 33 },
    { 0x24b3870c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37138710, 33 },
    { 0xc7338711, 46 },
    { 0x5c138711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c238718, 93 },
    { 0x74238718, 38 },
    { 0x5193871a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413871e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd038724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d38727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf13872b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3738733, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b38737, 81 },
    { 0xbe538738, 84 },
    { 0x3bf38738, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9483873c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c38744, 84 },
    { 0x4f038745, 95 },
    { 0x58f38746, 61 },
    { 0xba538745, 84 },
    { 0x3b438747, 34 },
    { 0, 0 },
    { 0x4143874a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f3874e, 55 },
    { 0x9393874e, 43 },
    { 0x4d03874e, 55 },
    { 0xfc3874e, 55 },
    { 0x7f338752, 84 },
    { 0x4d838752, 55 },
    { 0x87938753, 65 },
    { 0, 0 },
    { 0x25738756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a38764, 46 },
    { 0x66538764, 92 },
    { 0xfe538764, 98 },
    { 0x2a038766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e3876d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf38773, 90 },
    { 0xaf738774, 61 },
    { 0x26038775, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5338779, 96 },
    { 0x4493877a, 88 },
    { 0xf923877b, 90 },
    { 0x2de3877c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83938783, 95 },
    { 0, 0 },
    { 0xe3238785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86938789, 87 },
    { 0x4b43878a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1453878d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed38792, 99 },
    { 0, 0 },
    { 0xdff38794, 84 },
    { 0xfb838795, 55 },
    { 0x9c438796, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe38799, 90 },
    { 0, 0 },
    { 0x9823879b, 56 },
    { 0x4643879b, 46 },
    { 0xe0d3879c, 22 },
    { 0x1b43879e, 77 },
    { 0xa653879f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848387a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48387a8, 61 },
    { 0x150387a8, 33 },
    { 0xa6d387a8, 76 },
    { 0x1fe387ab, 55 },
    { 0x22f387aa, 61 },
    { 0xf48387aa, 90 },
    { 0xa8387ae, 98 },
    { 0x50f387ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6387b3, 98 },
    { 0x675387b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x975387b7, 91 },
    { 0x4387b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa387bb, 95 },
    { 0xbb6387bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513387c1, 90 },
    { 0, 0 },
    { 0x62d387c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f387ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0387d0, 17 },
    { 0x569387d0, 34 },
    { 0xaa8387d2, 34 },
    { 0x588387d2, 98 },
    { 0xa1387d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e387d8, 61 },
    { 0xbb4387d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6387df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06387e2, 99 },
    { 0xd0e387e3, 61 },
    { 0xcac387e3, 95 },
    { 0x5fc387e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae387e9, 55 },
    { 0xe53387e9, 95 },
    { 0x9b2387ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae387ee, 19 },
    { 0x9f387ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba387f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e387f7, 90 },
    { 0, 0 },
    { 0x23e387f9, 90 },
    { 0x3fa387fa, 86 },
    { 0, 0 },
    { 0xee3387fc, 95 },
    { 0xc73387fd, 46 },
    { 0x460387fd, 5 },
    { 0x866387ff, 77 },
    { 0, 0 },
    { 0x50438801, 90 },
    { 0x8e338801, 23 },
    { 0x4e38803, 87 },
    { 0x55438804, 87 },
    { 0xef738801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe3880b, 59 },
    { 0x2073880b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d38811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c38815, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9438818, 90 },
    { 0x91838819, 52 },
    { 0, 0 },
    { 0x7273881b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6038823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb693882a, 90 },
    { 0x4b53882b, 34 },
    { 0, 0 },
    { 0x3553882d, 34 },
    { 0x65d3882e, 76 },
    { 0xc3e3882e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d38832, 53 },
    { 0xc6538833, 33 },
    { 0x9a538834, 55 },
    { 0x6d338835, 55 },
    { 0, 0 },
    { 0x37a38837, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843883b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52938841, 61 },
    { 0x47e38842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5538845, 46 },
    { 0x34138845, 93 },
    { 0, 0 },
    { 0x93638848, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b3884b, 100 },
    { 0xf473884b, 95 },
    { 0x4a33884d, 23 },
    { 0xa793884b, 34 },
    { 0, 0 },
    { 0x7a338850, 90 },
    { 0, 0 },
    { 0x3538852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3885c, 33 },
    { 0xf7c3885c, 33 },
    { 0, 0 },
    { 0x97d3885f, 42 },
    { 0xd9238860, 61 },
    { 0x9ee38860, 34 },
    { 0x37e38862, 34 },
    { 0xe9938863, 95 },
    { 0x3213885f, 90 },
    { 0x78338865, 81 },
    { 0x14638865, 61 },
    { 0xc7738867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf353886a, 99 },
    { 0xbb83886b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d93886f, 67 },
    { 0x6e438870, 100 },
    { 0x58438871, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32638874, 98 },
    { 0x91e38875, 33 },
    { 0, 0 },
    { 0xcd638877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb413887c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba338884, 88 },
    { 0x59b38885, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99138888, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf923888b, 61 },
    { 0x203888b, 55 },
    { 0xc63888d, 90 },
    { 0xbca3888d, 70 },
    { 0xa423888f, 85 },
    { 0x62438890, 76 },
    { 0, 0 },
    { 0x2c638892, 23 },
    { 0x5df38893, 78 },
    { 0xf138893, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5723889e, 67 },
    { 0xbcd3889f, 33 },
    { 0, 0 },
    { 0xfb9388a1, 61 },
    { 0x1388a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f388aa, 93 },
    { 0, 0 },
    { 0x333388ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3388b0, 61 },
    { 0x7b5388b0, 85 },
    { 0x5e4388b1, 98 },
    { 0, 0 },
    { 0x59388b4, 87 },
    { 0xdeb388b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x985388b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43388bc, 52 },
    { 0xac2388bc, 61 },
    { 0xf61388be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed388c3, 75 },
    { 0xc56388c4, 46 },
    { 0x4f7388c5, 98 },
    { 0x40c388c5, 90 },
    { 0, 0 },
    { 0xe6b388c8, 76 },
    { 0xe14388c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6388cc, 55 },
    { 0, 0 },
    { 0xdbd388ce, 33 },
    { 0x44c388ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd388d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc388d7, 61 },
    { 0x7c388d8, 23 },
    { 0x375388d8, 88 },
    { 0, 0 },
    { 0x64c388db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef388e0, 99 },
    { 0, 0 },
    { 0xbd9388e2, 93 },
    { 0x7aa388e2, 95 },
    { 0xdc5388e4, 33 },
    { 0x3ae388e3, 34 },
    { 0xe44388e6, 90 },
    { 0x4a2388e7, 33 },
    { 0x7d1388e7, 33 },
    { 0x585388e8, 55 },
    { 0xf1b388e8, 67 },
    { 0xe0c388ea, 33 },
    { 0, 0 },
    { 0x94c388ed, 98 },
    { 0, 0 },
    { 0xccc388ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18388f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4238902, 75 },
    { 0xd6e38903, 95 },
    { 0x1ab38904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f38907, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c23890c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81138910, 78 },
    { 0, 0 },
    { 0x36338912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98038917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b83891a, 55 },
    { 0x7703891a, 33 },
    { 0x64e3891c, 34 },
    { 0x1e03891d, 76 },
    { 0x1da3891e, 34 },
    { 0x2cf3891f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d38924, 90 },
    { 0x24c38925, 46 },
    { 0x80c38925, 96 },
    { 0x3c638925, 33 },
    { 0x83038924, 61 },
    { 0x39138929, 23 },
    { 0x86738927, 61 },
    { 0xafa38928, 55 },
    { 0xa173892c, 87 },
    { 0x31b3892c, 77 },
    { 0xf1f3892d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28638932, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c038939, 98 },
    { 0x67d38939, 55 },
    { 0x1073893a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9538942, 95 },
    { 0x91138942, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc38948, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f38953, 17 },
    { 0x4c438954, 90 },
    { 0xd2338954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11838958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa543895f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6738964, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c93896a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283896f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8738975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533897a, 23 },
    { 0, 0 },
    { 0xc5f3897c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad238981, 55 },
    { 0xc5638982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838986, 78 },
    { 0, 0 },
    { 0xaff38988, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba3898e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de38991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e338994, 90 },
    { 0x6e238995, 76 },
    { 0x2938996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec3899f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6389a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb389ab, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569389b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65389b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15389c3, 93 },
    { 0x158389c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7389c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1389cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc389d3, 91 },
    { 0xb24389d4, 91 },
    { 0x249389d5, 23 },
    { 0x3a6389d6, 46 },
    { 0x507389d7, 90 },
    { 0xf8c389d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901389dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6389ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f389f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b389f5, 77 },
    { 0, 0 },
    { 0xb8c389f7, 95 },
    { 0, 0 },
    { 0x8b7389f9, 40 },
    { 0, 0 },
    { 0xd4e389fb, 98 },
    { 0, 0 },
    { 0xa62389fd, 87 },
    { 0x890389fe, 98 },
    { 0x4ca389ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe38a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9838a07, 66 },
    { 0, 0 },
    { 0x85e38a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe438a11, 55 },
    { 0x81738a12, 33 },
    { 0x44d38a13, 93 },
    { 0x7b838a11, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3538a1b, 33 },
    { 0x3c38a1b, 76 },
    { 0, 0 },
    { 0xa3b38a1e, 52 },
    { 0, 0 },
    { 0xc1c38a20, 91 },
    { 0, 0 },
    { 0xd7038a22, 100 },
    { 0x19838a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d38a26, 61 },
    { 0, 0 },
    { 0x7ef38a28, 100 },
    { 0xf6338a29, 3 },
    { 0xeda38a2a, 23 },
    { 0xa4138a2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19238a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47138a33, 61 },
    { 0xe0c38a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76538a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b338a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b338a45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b38a48, 98 },
    { 0xf8a38a49, 81 },
    { 0, 0 },
    { 0x65a38a4b, 23 },
    { 0, 0 },
    { 0xc3538a4d, 87 },
    { 0, 0 },
    { 0x51238a4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f538a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8838a5c, 46 },
    { 0x84f38a5d, 34 },
    { 0, 0 },
    { 0xa7738a5f, 33 },
    { 0x2ab38a5f, 33 },
    { 0, 0 },
    { 0x37038a62, 33 },
    { 0, 0 },
    { 0xa738a64, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9338a68, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b738a72, 46 },
    { 0x88038a72, 90 },
    { 0, 0 },
    { 0x99838a75, 90 },
    { 0x61338a76, 43 },
    { 0xaa738a75, 33 },
    { 0xf5638a77, 61 },
    { 0xdb538a76, 99 },
    { 0, 0 },
    { 0x94438a7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad638a83, 61 },
    { 0, 0 },
    { 0x37838a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94438a88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a138a8b, 33 },
    { 0x26c38a8c, 95 },
    { 0xd0b38a8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d38a91, 98 },
    { 0x35838a92, 93 },
    { 0xe5c38a93, 98 },
    { 0x29938a94, 23 },
    { 0xb8538a92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f38a9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5738aa3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x18838aa6, 61 },
    { 0, 0 },
    { 0xaad38aa8, 46 },
    { 0, 0 },
    { 0xbf238aaa, 90 },
    { 0x6138aab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84338aaf, 55 },
    { 0x1a138ab0, 33 },
    { 0x64638ab1, 23 },
    { 0xb0638ab2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff338ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a038aba, 61 },
    { 0x92838abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e938abf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc838aca, 98 },
    { 0, 0 },
    { 0x80238acc, 90 },
    { 0xabc38acd, 95 },
    { 0x3e938acc, 98 },
    { 0, 0 },
    { 0xb8e38ad0, 61 },
    { 0xc7d38ad0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff38ad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c38adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e38ade, 85 },
    { 0x1c038adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53238ae2, 84 },
    { 0x53d38ae3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c38ae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a38aec, 81 },
    { 0, 0 },
    { 0xc7d38aee, 101 },
    { 0x8bd38aee, 92 },
    { 0x4cf38af0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15338af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c738af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da38afc, 87 },
    { 0xfed38afc, 87 },
    { 0x35b38afe, 67 },
    { 0x18638aff, 90 },
    { 0, 0 },
    { 0x57538b01, 34 },
    { 0x5dd38b02, 91 },
    { 0xde038b03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f438b06, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8438b09, 66 },
    { 0x37038b09, 53 },
    { 0xdb138b0b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac38b10, 33 },
    { 0xf6f38b11, 34 },
    { 0xf7038b12, 76 },
    { 0, 0 },
    { 0x40138b14, 61 },
    { 0, 0 },
    { 0x87138b16, 76 },
    { 0, 0 },
    { 0x14938b18, 34 },
    { 0, 0 },
    { 0x7ab38b1a, 66 },
    { 0, 0 },
    { 0xb0238b1c, 95 },
    { 0x51f38b1d, 98 },
    { 0xdc938b1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee138b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c38b25, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe738b2d, 33 },
    { 0x4d338b2e, 87 },
    { 0x3a238b2e, 90 },
    { 0, 0 },
    { 0x65b38b31, 33 },
    { 0xee838b32, 43 },
    { 0xc8b38b33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4738b38, 95 },
    { 0xd8e38b39, 90 },
    { 0x7b938b38, 67 },
    { 0xd2338b3b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf838b3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x38738b42, 99 },
    { 0x54f38b43, 47 },
    { 0xb8938b44, 23 },
    { 0x28138b44, 55 },
    { 0, 0 },
    { 0xcd238b47, 85 },
    { 0xabf38b47, 13 },
    { 0, 0 },
    { 0xdb338b4a, 76 },
    { 0x38238b4a, 55 },
    { 0xf7e38b4a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab38b4f, 90 },
    { 0, 0 },
    { 0x38538b51, 98 },
    { 0x7f638b51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88938b56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e838b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce938b64, 55 },
    { 0x37238b65, 98 },
    { 0xde638b64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c38b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1138b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb438b72, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x10838b75, 33 },
    { 0x53d38b76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40638b7b, 43 },
    { 0x24f38b7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e838b7f, 90 },
    { 0x1a738b80, 90 },
    { 0xfb738b80, 91 },
    { 0xe8138b80, 80 },
    { 0xee538b81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4538b86, 90 },
    { 0x90b38b87, 34 },
    { 0x5ce38b87, 34 },
    { 0x17938b87, 95 },
    { 0x7e38b86, 95 },
    { 0xdab38b8b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b38b9b, 33 },
    { 0x9a638b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69838ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a38ba4, 95 },
    { 0x92538ba4, 55 },
    { 0, 0 },
    { 0x7138ba7, 61 },
    { 0, 0 },
    { 0x68438ba9, 34 },
    { 0x72538ba9, 90 },
    { 0x7d038bab, 87 },
    { 0x4e838baa, 33 },
    { 0, 0 },
    { 0xc2b38bae, 100 },
    { 0x23438bae, 95 },
    { 0xca338bb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b438bb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3538bbc, 98 },
    { 0, 0 },
    { 0x9d838bbe, 91 },
    { 0x10938bbf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638bc6, 17 },
    { 0x11138bc7, 100 },
    { 0x6d38bc8, 95 },
    { 0xd9138bc9, 34 },
    { 0x7bb38bc8, 90 },
    { 0x7e038bc7, 90 },
    { 0xf6c38bcc, 90 },
    { 0, 0 },
    { 0x4df38bce, 23 },
    { 0, 0 },
    { 0xd9338bd0, 67 },
    { 0x24538bd0, 98 },
    { 0, 0 },
    { 0x6b638bd3, 90 },
    { 0x99038bd4, 90 },
    { 0, 0 },
    { 0x57938bd6, 34 },
    { 0, 0 },
    { 0x33438bd8, 87 },
    { 0xfe638bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe338bdd, 87 },
    { 0x1c738bde, 98 },
    { 0x2cc38bdd, 76 },
    { 0x22538be0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f638be3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd38beb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9638bef, 33 },
    { 0, 0 },
    { 0x19138bf1, 101 },
    { 0x3eb38bf1, 33 },
    { 0x59d38bf3, 61 },
    { 0xb8a38bf2, 55 },
    { 0xb4b38bf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b38bf9, 23 },
    { 0x3ea38bf9, 52 },
    { 0xeae38bfa, 46 },
    { 0, 0 },
    { 0x75e38bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc38c05, 98 },
    { 0xb5938c06, 61 },
    { 0xf9038c06, 61 },
    { 0x9fa38c08, 61 },
    { 0x99538c09, 34 },
    { 0x5ea38c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c38c0e, 76 },
    { 0x21738c0f, 34 },
    { 0xd4738c0f, 90 },
    { 0xef638c11, 2 },
    { 0x1338c12, 95 },
    { 0x32738c13, 90 },
    { 0xa6d38c13, 90 },
    { 0xb4638c15, 87 },
    { 0x31138c0f, 98 },
    { 0, 0 },
    { 0x35738c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96338c1f, 33 },
    { 0xa7538c20, 90 },
    { 0x8d938c1f, 94 },
    { 0x59238c1f, 76 },
    { 0x29138c23, 90 },
    { 0, 0 },
    { 0xaee38c25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db38c28, 90 },
    { 0xf7d38c28, 98 },
    { 0xf38c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a838c2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d38c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a238c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8638c3b, 17 },
    { 0xea738c3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b38c49, 33 },
    { 0, 0 },
    { 0x2b338c4b, 81 },
    { 0x6df38c4c, 33 },
    { 0x33438c4d, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5538c51, 88 },
    { 0x5e038c51, 90 },
    { 0, 0 },
    { 0x9c38c54, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8438c57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c838c5b, 61 },
    { 0x31e38c5b, 90 },
    { 0x87338c5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19938c62, 85 },
    { 0xe3938c63, 95 },
    { 0, 0 },
    { 0x14838c65, 90 },
    { 0x53f38c65, 33 },
    { 0x1f738c65, 33 },
    { 0xb6038c65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd38c6e, 52 },
    { 0xd4d38c6f, 99 },
    { 0xf6838c70, 90 },
    { 0xf2c38c6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce838c74, 98 },
    { 0x6d238c74, 56 },
    { 0, 0 },
    { 0xc1c38c77, 33 },
    { 0xe8b38c78, 61 },
    { 0x16d38c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0938c85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3438c88, 87 },
    { 0, 0 },
    { 0xf7738c8a, 84 },
    { 0x1f838c8b, 80 },
    { 0xc938c8c, 23 },
    { 0x5bf38c8d, 76 },
    { 0x5c938c8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538c91, 91 },
    { 0, 0 },
    { 0xadf38c93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe38c97, 17 },
    { 0x93538c98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d038c9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a38ca1, 98 },
    { 0xa7538ca1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c338ca7, 33 },
    { 0x65438ca7, 76 },
    { 0x5af38ca9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48038caf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87038cb5, 61 },
    { 0xbf738cb5, 94 },
    { 0x1a338cb7, 93 },
    { 0x71e38cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b38cbf, 33 },
    { 0xa4238cbf, 33 },
    { 0xe6b38cbf, 76 },
    { 0x87738cc0, 95 },
    { 0, 0 },
    { 0x7cf38cc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d038cc8, 90 },
    { 0x9b738cc9, 95 },
    { 0xdfa38cca, 90 },
    { 0x75138cc9, 95 },
    { 0, 0 },
    { 0xc3b38ccd, 43 },
    { 0xd9c38ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef38cd5, 46 },
    { 0xb6138cd6, 23 },
    { 0x78238cd7, 23 },
    { 0x10e38cd8, 43 },
    { 0x87338cd9, 33 },
    { 0x46438cd8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d38cdd, 61 },
    { 0, 0 },
    { 0x2ec38cdf, 33 },
    { 0x5c938cdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd38ce8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab38cf1, 17 },
    { 0xcc938cf1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7838cf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea38cf8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63438cff, 90 },
    { 0x7c638d00, 61 },
    { 0xa2e38d01, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df38d04, 90 },
    { 0, 0 },
    { 0x64a38d06, 56 },
    { 0xd1a38d07, 46 },
    { 0, 0 },
    { 0x4fa38d09, 84 },
    { 0, 0 },
    { 0x5ed38d0b, 61 },
    { 0, 0 },
    { 0xa5a38d0d, 95 },
    { 0, 0 },
    { 0xf6538d0f, 61 },
    { 0x99b38d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13438d17, 92 },
    { 0, 0 },
    { 0x77b38d19, 85 },
    { 0, 0 },
    { 0xb3a38d1b, 87 },
    { 0, 0 },
    { 0x95138d1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e538d20, 76 },
    { 0, 0 },
    { 0xc8c38d22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1838d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe38d31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38d35, 61 },
    { 0x5d38d36, 23 },
    { 0x3ec38d37, 98 },
    { 0x68038d37, 90 },
    { 0x29638d39, 61 },
    { 0, 0 },
    { 0x73438d3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f38d3e, 33 },
    { 0x58e38d3f, 40 },
    { 0, 0 },
    { 0x5cd38d41, 90 },
    { 0x7ef38d41, 34 },
    { 0xc9338d43, 76 },
    { 0, 0 },
    { 0xe3438d45, 80 },
    { 0x2a838d46, 90 },
    { 0, 0 },
    { 0xff538d48, 33 },
    { 0, 0 },
    { 0x8ab38d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e38d4d, 33 },
    { 0x49138d4d, 90 },
    { 0x4b738d4d, 90 },
    { 0, 0 },
    { 0x5838d51, 33 },
    { 0, 0 },
    { 0x4938d53, 76 },
    { 0, 0 },
    { 0x17d38d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b38d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53538d5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d38d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c438d6a, 33 },
    { 0xa4f38d6b, 17 },
    { 0x3b438d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d38d73, 95 },
    { 0x76f38d74, 43 },
    { 0xa1a38d75, 90 },
    { 0, 0 },
    { 0x46638d77, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50538d7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e338d82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96238d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f038d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b38d95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71538d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1838d9d, 52 },
    { 0, 0 },
    { 0xa3838d9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb38dae, 90 },
    { 0xca838daf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d438db6, 43 },
    { 0x87238db7, 68 },
    { 0x5138db8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x18838dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de38dc6, 67 },
    { 0x16438dc7, 55 },
    { 0x58e38dc7, 98 },
    { 0x45738dc6, 55 },
    { 0x9f538dca, 61 },
    { 0x93d38dc9, 23 },
    { 0, 0 },
    { 0xe1038dcd, 33 },
    { 0x5d438dce, 88 },
    { 0, 0 },
    { 0x7f138dd0, 33 },
    { 0x67438dd0, 90 },
    { 0xd3338dd0, 33 },
    { 0x25a38dd3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d38dd6, 17 },
    { 0xb3538dd7, 44 },
    { 0, 0 },
    { 0x97f38dd9, 76 },
    { 0xf3538dda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b38dde, 23 },
    { 0xd8e38dde, 76 },
    { 0x4c338ddf, 61 },
    { 0xeba38ddf, 23 },
    { 0, 0 },
    { 0xa5238de3, 43 },
    { 0x9fe38de3, 76 },
    { 0, 0 },
    { 0x93238de6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b38df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c738df6, 61 },
    { 0xbd038df7, 61 },
    { 0xafd38df7, 90 },
    { 0x7a938df8, 40 },
    { 0xef138dfa, 33 },
    { 0xa6138dfb, 34 },
    { 0x96738dfa, 34 },
    { 0x9ee38dfb, 76 },
    { 0x7b738dfe, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96538e03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb438e0e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3738e11, 90 },
    { 0, 0 },
    { 0xd7038e13, 23 },
    { 0xb2138e13, 90 },
    { 0x438e15, 33 },
    { 0x23e38e16, 33 },
    { 0x2dc38e16, 33 },
    { 0x1c938e13, 95 },
    { 0x16f38e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d938e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f238e21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea38e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4838e2b, 90 },
    { 0xb7338e2c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6738e30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac38e35, 76 },
    { 0x14738e36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a538e3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e38e44, 46 },
    { 0, 0 },
    { 0x16b38e46, 93 },
    { 0xe0d38e47, 76 },
    { 0x1e38e48, 23 },
    { 0x74338e48, 23 },
    { 0xf3c38e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d38e4d, 98 },
    { 0x6c038e4e, 61 },
    { 0, 0 },
    { 0x5ed38e50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b838e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e38e5a, 78 },
    { 0x10338e5a, 84 },
    { 0xa3138e5c, 76 },
    { 0x2138e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca38e62, 33 },
    { 0xc3938e63, 23 },
    { 0xff438e64, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab238e68, 90 },
    { 0x86238e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4838e6d, 80 },
    { 0, 0 },
    { 0xefa38e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b638e7c, 33 },
    { 0x88138e7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8938e80, 95 },
    { 0xe6b38e80, 23 },
    { 0x61338e82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c638e87, 90 },
    { 0, 0 },
    { 0x92238e89, 34 },
    { 0, 0 },
    { 0x10438e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd38e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28138e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b38e9e, 98 },
    { 0x2e238e9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f38ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78138ea9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b38eac, 95 },
    { 0x8e38eac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee38eb6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b38ebc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e938ebf, 91 },
    { 0, 0 },
    { 0xfa038ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83738ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2238ec9, 36 },
    { 0, 0 },
    { 0x99438ecb, 61 },
    { 0x8ac38ecc, 90 },
    { 0xc1638ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d38ed0, 23 },
    { 0x36d38ed1, 90 },
    { 0x21c38ed0, 55 },
    { 0xf8138ed0, 43 },
    { 0, 0 },
    { 0xb7838ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c038eda, 55 },
    { 0, 0 },
    { 0x9d938edc, 81 },
    { 0x15b38edd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a38ee2, 87 },
    { 0x85938ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c38eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b38ef2, 61 },
    { 0xbe38ef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac38ef7, 66 },
    { 0x61538ef7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c438efd, 95 },
    { 0, 0 },
    { 0xa1c38eff, 77 },
    { 0, 0 },
    { 0x7c838f01, 61 },
    { 0x25a38f02, 98 },
    { 0, 0 },
    { 0x82538f04, 17 },
    { 0x5d38f04, 75 },
    { 0xfa838f06, 98 },
    { 0x7a538f07, 93 },
    { 0xa5e38f08, 33 },
    { 0xad338f09, 52 },
    { 0xe5238f07, 55 },
    { 0x20f38f0b, 84 },
    { 0xf3e38f04, 46 },
    { 0x46838f0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46738f14, 65 },
    { 0x238f15, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f138f18, 55 },
    { 0xa5938f19, 46 },
    { 0xb5b38f1a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c38f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8738f25, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad138f2a, 98 },
    { 0, 0 },
    { 0xa5738f2c, 90 },
    { 0x7d038f2d, 60 },
    { 0, 0 },
    { 0x54538f2f, 78 },
    { 0xf2838f30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19738f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda838f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9338f44, 90 },
    { 0, 0 },
    { 0x71838f46, 55 },
    { 0x46a38f47, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b938f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e38f4f, 33 },
    { 0xef638f50, 76 },
    { 0x8f738f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d38f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17438f57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae38f60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438f67, 90 },
    { 0x12238f68, 49 },
    { 0x52638f68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b138f6d, 33 },
    { 0, 0 },
    { 0xaba38f6f, 98 },
    { 0, 0 },
    { 0x1138f71, 43 },
    { 0x26038f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d638f7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d38f85, 33 },
    { 0xeb138f85, 98 },
    { 0x1c238f86, 66 },
    { 0xb8d38f88, 61 },
    { 0x82c38f89, 80 },
    { 0, 0 },
    { 0xb1c38f8b, 98 },
    { 0xd0f38f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c738f98, 98 },
    { 0, 0 },
    { 0xe1938f9a, 98 },
    { 0x64538f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e38f9f, 98 },
    { 0, 0 },
    { 0xa4e38fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d38fa4, 61 },
    { 0xf4938fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f38fab, 90 },
    { 0xad138fab, 33 },
    { 0xa6438fac, 98 },
    { 0xca738fab, 90 },
    { 0x41638fad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd138fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c38fb8, 93 },
    { 0, 0 },
    { 0x64838fba, 61 },
    { 0, 0 },
    { 0x82a38fbc, 55 },
    { 0x15338fbd, 95 },
    { 0xad438fbd, 90 },
    { 0, 0 },
    { 0x76138fc0, 90 },
    { 0, 0 },
    { 0x11b38fc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc638fc5, 90 },
    { 0x19738fc6, 95 },
    { 0, 0 },
    { 0x47e38fc8, 95 },
    { 0xe4438fc9, 61 },
    { 0, 0 },
    { 0x83538fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e38fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d738fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38938fdd, 90 },
    { 0x2a538fdd, 34 },
    { 0x3cb38fdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed338fe3, 100 },
    { 0xf6338fe3, 85 },
    { 0x85838fe5, 80 },
    { 0x6f438fe5, 98 },
    { 0xcac38fe7, 66 },
    { 0x63138fe5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c38ff0, 66 },
    { 0, 0 },
    { 0xe8b38ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce38ff5, 84 },
    { 0xa5438ff6, 98 },
    { 0xf9238ff7, 87 },
    { 0x78538ff8, 90 },
    { 0x9de38ff9, 67 },
    { 0xa1538ffa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb38ffe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20939003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d39006, 55 },
    { 0xed739007, 76 },
    { 0xf939008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c33900c, 33 },
    { 0, 0 },
    { 0x20f3900e, 52 },
    { 0x1de3900f, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x74539012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68639015, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b39019, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb3901e, 75 },
    { 0, 0 },
    { 0xd6039020, 43 },
    { 0x80f39021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f39024, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e3902b, 87 },
    { 0, 0 },
    { 0x4413902d, 33 },
    { 0xde63902e, 90 },
    { 0xc393902f, 90 },
    { 0, 0 },
    { 0x7c739031, 33 },
    { 0x58139032, 46 },
    { 0x2d639032, 76 },
    { 0x29739032, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d039037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe263903a, 46 },
    { 0, 0 },
    { 0x6c93903c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed53903f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d39042, 84 },
    { 0, 0 },
    { 0xff39044, 34 },
    { 0x85e39045, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c39048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a3904f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19839057, 55 },
    { 0xf7339058, 55 },
    { 0x4e239059, 33 },
    { 0, 0 },
    { 0xb8e3905b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9793905e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97039061, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e39066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e39069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13906c, 55 },
    { 0xea63906d, 23 },
    { 0xfd73906d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca39073, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d39076, 43 },
    { 0x42c39076, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8593907a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd783907d, 61 },
    { 0xe4a3907e, 95 },
    { 0x3153907e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc039084, 53 },
    { 0xdf339085, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e39089, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff739090, 61 },
    { 0, 0 },
    { 0xfd439092, 85 },
    { 0x8e739093, 23 },
    { 0x8ca39094, 23 },
    { 0xbc439095, 90 },
    { 0x4639096, 76 },
    { 0xa1639096, 33 },
    { 0xd0439096, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81390a0, 61 },
    { 0x302390a1, 90 },
    { 0x77d390a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572390a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3390ad, 90 },
    { 0xb13390ae, 78 },
    { 0xfb0390af, 23 },
    { 0x7ec390b0, 98 },
    { 0xcb2390af, 90 },
    { 0x3f8390af, 33 },
    { 0xe9b390b3, 33 },
    { 0x1be390b4, 33 },
    { 0, 0 },
    { 0x435390b6, 55 },
    { 0x122390b6, 81 },
    { 0x236390b8, 52 },
    { 0, 0 },
    { 0x27c390ba, 98 },
    { 0xc1f390bb, 61 },
    { 0x6d1390bc, 52 },
    { 0xdd390bd, 91 },
    { 0xb48390bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa390c4, 61 },
    { 0x97e390c5, 84 },
    { 0xbca390c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94390d1, 61 },
    { 0, 0 },
    { 0xf73390d3, 61 },
    { 0, 0 },
    { 0xc53390d5, 55 },
    { 0xecb390d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e390db, 23 },
    { 0x743390dc, 34 },
    { 0x137390db, 93 },
    { 0x3b4390de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92390e2, 93 },
    { 0x952390e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca390e7, 33 },
    { 0x76d390e7, 52 },
    { 0x8b390e9, 90 },
    { 0xd390ea, 55 },
    { 0, 0 },
    { 0xbf8390ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c390ef, 90 },
    { 0x421390ef, 33 },
    { 0x70f390ef, 33 },
    { 0xa1e390f1, 98 },
    { 0, 0 },
    { 0x47c390f4, 66 },
    { 0, 0 },
    { 0xd0d390f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53390fa, 61 },
    { 0x2db390fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539100, 61 },
    { 0xbeb39100, 33 },
    { 0x65239102, 34 },
    { 0, 0 },
    { 0x5a139104, 90 },
    { 0, 0 },
    { 0xd2739106, 33 },
    { 0x18739107, 67 },
    { 0, 0 },
    { 0x3b939109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8739111, 95 },
    { 0, 0 },
    { 0xcfb39113, 33 },
    { 0, 0 },
    { 0xd0439115, 33 },
    { 0x7fb39116, 33 },
    { 0x43339117, 43 },
    { 0x8be39118, 85 },
    { 0x95c39118, 55 },
    { 0xffe39119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3823911e, 33 },
    { 0xd633911e, 90 },
    { 0, 0 },
    { 0x1cb39121, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1039126, 90 },
    { 0xb2939127, 95 },
    { 0x40439127, 34 },
    { 0x1e139129, 61 },
    { 0x1233912a, 95 },
    { 0x8aa3912b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9543912e, 48 },
    { 0, 0 },
    { 0x93a39130, 98 },
    { 0x78839131, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d39136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23539139, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7823913e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98839141, 95 },
    { 0x3939141, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e39145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee339151, 76 },
    { 0, 0 },
    { 0x6e039153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d39156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22239159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23915c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba439163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83916b, 33 },
    { 0x203916c, 33 },
    { 0xc383916b, 33 },
    { 0x3ca3916e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1939174, 98 },
    { 0x9f239175, 76 },
    { 0, 0 },
    { 0x2c939177, 23 },
    { 0xec739178, 43 },
    { 0x75b39179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e13917e, 40 },
    { 0x9743917f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26439189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xade3918c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96339191, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x66739194, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28539199, 30 },
    { 0x6183919a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74391a4, 46 },
    { 0, 0 },
    { 0xe55391a6, 52 },
    { 0x75b391a6, 33 },
    { 0x302391a7, 55 },
    { 0x4b5391a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d391ad, 96 },
    { 0x14391ae, 90 },
    { 0x191391ad, 90 },
    { 0xb87391af, 55 },
    { 0, 0 },
    { 0xf8391b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe391b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9391bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02391c0, 40 },
    { 0xea2391c0, 90 },
    { 0xa8391c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca391c5, 33 },
    { 0x5ee391c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91391cc, 95 },
    { 0x80e391cd, 90 },
    { 0xb56391cd, 98 },
    { 0x62f391cf, 61 },
    { 0x74d391cf, 61 },
    { 0x1cf391d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8391d7, 77 },
    { 0xfc391d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e391de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52391e3, 95 },
    { 0xcca391e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4391e7, 33 },
    { 0x7a8391e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf391eb, 33 },
    { 0x836391eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf391ef, 46 },
    { 0xb5e391f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d391f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3391fe, 85 },
    { 0xe38391fe, 61 },
    { 0x3e939200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e739209, 23 },
    { 0x44f3920a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c63920f, 34 },
    { 0xb593920f, 65 },
    { 0x67439210, 33 },
    { 0, 0 },
    { 0x73139213, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4739216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3439219, 95 },
    { 0xdbf3921a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333921e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49939221, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24039228, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff83922b, 99 },
    { 0x6a73922c, 55 },
    { 0xd113922d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80739230, 33 },
    { 0xbe139231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8363923b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b39240, 46 },
    { 0xbeb39241, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d039244, 77 },
    { 0x95039245, 98 },
    { 0x5539246, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17839251, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1039254, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3925a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd3925d, 64 },
    { 0x71c3925e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c39262, 95 },
    { 0, 0 },
    { 0x1ac39264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16539269, 61 },
    { 0xe13926a, 90 },
    { 0x3c139269, 76 },
    { 0x3d13926a, 90 },
    { 0x3943926d, 55 },
    { 0xe73926d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11439271, 95 },
    { 0x2a339272, 90 },
    { 0, 0 },
    { 0x89139274, 61 },
    { 0x8be39275, 90 },
    { 0x6739276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32339281, 23 },
    { 0x56139282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c39287, 90 },
    { 0x69639288, 78 },
    { 0x3b239288, 66 },
    { 0x7de39288, 90 },
    { 0, 0 },
    { 0xa3b3928c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a39293, 100 },
    { 0xa7539294, 96 },
    { 0x9139295, 95 },
    { 0, 0 },
    { 0x28939297, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa223929b, 90 },
    { 0x1f53929b, 23 },
    { 0, 0 },
    { 0x1f13929e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa392a4, 33 },
    { 0, 0 },
    { 0x5c392a6, 95 },
    { 0x359392a7, 76 },
    { 0xd30392a7, 95 },
    { 0xed3392a9, 23 },
    { 0x12d392a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc392b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956392b7, 97 },
    { 0x4ec392b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a392bb, 38 },
    { 0x921392bc, 76 },
    { 0xfc0392bc, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49392c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369392cc, 99 },
    { 0x276392cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709392d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9392d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b392da, 40 },
    { 0, 0 },
    { 0x574392dc, 78 },
    { 0xf74392dd, 98 },
    { 0, 0 },
    { 0xd2c392df, 98 },
    { 0x2a6392df, 90 },
    { 0, 0 },
    { 0x112392e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0392e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d392e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4392ed, 84 },
    { 0, 0 },
    { 0x153392ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x940392f2, 43 },
    { 0xd0a392f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89392fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab39300, 90 },
    { 0x29839300, 33 },
    { 0x64a39300, 90 },
    { 0, 0 },
    { 0x88239304, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b39308, 85 },
    { 0, 0 },
    { 0x2fe3930a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d13930f, 98 },
    { 0, 0 },
    { 0x54339311, 55 },
    { 0x38539312, 43 },
    { 0x41039313, 66 },
    { 0, 0 },
    { 0x15e39315, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd239318, 66 },
    { 0x8ef39319, 90 },
    { 0x7d039318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc39321, 99 },
    { 0xc1639322, 52 },
    { 0xf9439323, 23 },
    { 0x33239324, 46 },
    { 0xca39324, 94 },
    { 0, 0 },
    { 0xa7339327, 23 },
    { 0x93539328, 33 },
    { 0xb6439328, 90 },
    { 0x70139329, 95 },
    { 0, 0 },
    { 0xb2a3932c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd239336, 78 },
    { 0x82639337, 55 },
    { 0x60639338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5273933c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c39340, 76 },
    { 0x7ff39341, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf3934a, 90 },
    { 0x4f43934b, 23 },
    { 0xef03934c, 98 },
    { 0x76d3934d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81939354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f3935f, 46 },
    { 0x72339360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15039366, 90 },
    { 0x6d439367, 23 },
    { 0, 0 },
    { 0x45739369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c03936c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c39372, 61 },
    { 0xc0139373, 95 },
    { 0x66639374, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e739377, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf39380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139384, 33 },
    { 0xf9139385, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2939388, 80 },
    { 0xfa739389, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5523938d, 87 },
    { 0xec03938e, 46 },
    { 0x3853938f, 97 },
    { 0x84f3938e, 43 },
    { 0x1a139390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b039395, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063939a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb243939d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82393a0, 90 },
    { 0, 0 },
    { 0x9b3393a2, 33 },
    { 0x6ae393a3, 90 },
    { 0x545393a4, 66 },
    { 0x854393a4, 90 },
    { 0x9e2393a5, 87 },
    { 0x1c3393a7, 95 },
    { 0x366393a7, 34 },
    { 0, 0 },
    { 0x13c393aa, 61 },
    { 0x911393ab, 76 },
    { 0x208393ac, 23 },
    { 0x84f393ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e393b0, 23 },
    { 0x83f393b0, 88 },
    { 0, 0 },
    { 0xcf5393b3, 76 },
    { 0, 0 },
    { 0xf65393b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c393b8, 66 },
    { 0x8a1393b9, 61 },
    { 0xfb5393b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96393c3, 33 },
    { 0, 0 },
    { 0x676393c5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce393c9, 90 },
    { 0, 0 },
    { 0xfb9393cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a393d6, 61 },
    { 0, 0 },
    { 0x8be393d8, 23 },
    { 0x803393d9, 76 },
    { 0x5f7393da, 33 },
    { 0xdb3393db, 43 },
    { 0, 0 },
    { 0xa3e393dd, 90 },
    { 0xb6e393de, 55 },
    { 0x715393dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef393e6, 95 },
    { 0, 0 },
    { 0x201393e8, 55 },
    { 0, 0 },
    { 0xce4393ea, 95 },
    { 0xef5393eb, 90 },
    { 0x57f393ec, 83 },
    { 0x932393ed, 98 },
    { 0x523393ee, 90 },
    { 0, 0 },
    { 0x194393f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11393f5, 90 },
    { 0xdf9393f6, 23 },
    { 0xfc7393f7, 95 },
    { 0xe6393f7, 99 },
    { 0, 0 },
    { 0xd91393fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6393fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6639402, 90 },
    { 0x61c39402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc39407, 100 },
    { 0x69e39408, 90 },
    { 0x7e439409, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c3940f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xac639412, 100 },
    { 0, 0 },
    { 0x9b339414, 95 },
    { 0x39e39414, 100 },
    { 0x82e39416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b3941c, 66 },
    { 0x22f3941d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81839421, 90 },
    { 0x79939421, 55 },
    { 0xd439422, 66 },
    { 0x1bd39424, 46 },
    { 0x15d39425, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc93942c, 60 },
    { 0x86a3942c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d639431, 84 },
    { 0x89339431, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f039435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a39439, 52 },
    { 0x2803943a, 46 },
    { 0x7c63943b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e3943f, 55 },
    { 0x4b03943f, 90 },
    { 0xfad39441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44739445, 61 },
    { 0xfbb39446, 90 },
    { 0, 0 },
    { 0xf0439448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3944b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3944f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5239452, 43 },
    { 0xf4a39452, 95 },
    { 0x98239453, 100 },
    { 0xcf539455, 61 },
    { 0x3f039455, 46 },
    { 0, 0 },
    { 0x98b39458, 23 },
    { 0xad439459, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f3945f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd39462, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4739468, 91 },
    { 0x5d739468, 55 },
    { 0x7ee3946a, 33 },
    { 0x1d3946b, 46 },
    { 0x80a39468, 61 },
    { 0xe103946d, 46 },
    { 0xc4a3946d, 95 },
    { 0x8fd3946d, 69 },
    { 0, 0 },
    { 0x67739471, 55 },
    { 0x94b39472, 46 },
    { 0x77839471, 53 },
    { 0x21939474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc339478, 95 },
    { 0, 0 },
    { 0xd4e3947a, 76 },
    { 0xcfd3947b, 98 },
    { 0x87c3947c, 33 },
    { 0x2923947d, 36 },
    { 0xbf23947e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46839481, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4339487, 98 },
    { 0x4dc39488, 33 },
    { 0, 0 },
    { 0xa073948a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453948f, 61 },
    { 0, 0 },
    { 0x6a439491, 87 },
    { 0xa7639491, 95 },
    { 0x22e39491, 33 },
    { 0xbee39494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e3949c, 61 },
    { 0, 0 },
    { 0x2943949e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297394a3, 55 },
    { 0xbcd394a4, 95 },
    { 0, 0 },
    { 0xf8e394a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0394ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x416394ae, 83 },
    { 0x231394af, 90 },
    { 0, 0 },
    { 0x80d394b1, 90 },
    { 0xd44394b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4394b7, 77 },
    { 0xf0c394b8, 23 },
    { 0xc6f394b9, 100 },
    { 0x5d3394b9, 98 },
    { 0x506394bb, 95 },
    { 0xcc0394b9, 46 },
    { 0xc89394bd, 95 },
    { 0x33d394ba, 65 },
    { 0xe27394bf, 61 },
    { 0x9fa394c0, 23 },
    { 0x1ea394c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717394c5, 75 },
    { 0x778394c6, 61 },
    { 0xcaf394c6, 23 },
    { 0xa8e394c8, 33 },
    { 0, 0 },
    { 0x1f8394ca, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a394cf, 61 },
    { 0xbd7394d0, 76 },
    { 0, 0 },
    { 0xe72394d2, 60 },
    { 0xd3b394d3, 84 },
    { 0, 0 },
    { 0x7ae394d5, 61 },
    { 0xcfa394d6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5394d9, 41 },
    { 0, 0 },
    { 0x9fe394db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76394e0, 95 },
    { 0xb94394e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05394e6, 90 },
    { 0x5dc394e6, 52 },
    { 0x401394e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2394f1, 93 },
    { 0xe8c394f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd394f7, 33 },
    { 0xc7b394f8, 23 },
    { 0x351394f9, 52 },
    { 0xa3a394fa, 33 },
    { 0x742394fb, 67 },
    { 0xa05394fa, 33 },
    { 0xe50394fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f39502, 77 },
    { 0, 0 },
    { 0x4ba39504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b539507, 31 },
    { 0x4f939507, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8143950c, 90 },
    { 0xa553950d, 23 },
    { 0x5ca3950e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16639512, 99 },
    { 0x5f339513, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb39518, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f139520, 61 },
    { 0, 0 },
    { 0x75f39522, 90 },
    { 0xc8c39522, 95 },
    { 0xaa39524, 90 },
    { 0x37939523, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5733952a, 90 },
    { 0x1123952b, 99 },
    { 0xe8b3952c, 23 },
    { 0xba53952a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e39531, 84 },
    { 0xd2039532, 34 },
    { 0, 0 },
    { 0x54139534, 33 },
    { 0x51439534, 98 },
    { 0x7d639536, 80 },
    { 0x1239535, 30 },
    { 0x91539538, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23953d, 76 },
    { 0xbee3953e, 90 },
    { 0xf483953f, 95 },
    { 0xf9c3953f, 99 },
    { 0x7a139541, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e839548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec3954b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603954f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e39553, 66 },
    { 0xc7639554, 61 },
    { 0xed239554, 98 },
    { 0x40b39553, 98 },
    { 0x17d39555, 77 },
    { 0, 0 },
    { 0xb7439559, 98 },
    { 0, 0 },
    { 0x3433955b, 99 },
    { 0x27b3955c, 69 },
    { 0x1973955d, 33 },
    { 0x1153955d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9639561, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc39564, 99 },
    { 0xdf039564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84939568, 90 },
    { 0x6c039569, 33 },
    { 0, 0 },
    { 0x8bc3956b, 33 },
    { 0x8893956c, 62 },
    { 0x7583956c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac39576, 17 },
    { 0, 0 },
    { 0x50939578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe3957b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13957e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5303958b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53959a, 33 },
    { 0x9883959b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f63959f, 74 },
    { 0x7fb395a0, 66 },
    { 0x8de395a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e395a6, 84 },
    { 0xd8b395a7, 34 },
    { 0x799395a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7395ac, 90 },
    { 0, 0 },
    { 0xea5395ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f395b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb395ba, 98 },
    { 0xdac395ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2395bf, 99 },
    { 0x763395c0, 95 },
    { 0x26d395c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7395c7, 88 },
    { 0x814395c7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280395cc, 84 },
    { 0, 0 },
    { 0xaf4395ce, 81 },
    { 0xcec395cf, 96 },
    { 0, 0 },
    { 0x5c6395d1, 95 },
    { 0xc34395d2, 88 },
    { 0x113395d2, 52 },
    { 0x476395d2, 55 },
    { 0x625395d1, 23 },
    { 0, 0 },
    { 0xe42395d7, 88 },
    { 0xd1d395d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f395df, 55 },
    { 0x440395e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a395e4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd395eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5395f0, 43 },
    { 0x3d395f0, 61 },
    { 0, 0 },
    { 0xda9395f3, 66 },
    { 0, 0 },
    { 0x33a395f5, 90 },
    { 0, 0 },
    { 0xaa0395f7, 61 },
    { 0x667395f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd339600, 95 },
    { 0xf5339600, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab39605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac63960a, 23 },
    { 0, 0 },
    { 0x21b3960c, 66 },
    { 0, 0 },
    { 0x7e73960e, 95 },
    { 0xd043960f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c39615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133961c, 34 },
    { 0xd6f3961d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21439621, 17 },
    { 0, 0 },
    { 0x4039623, 43 },
    { 0x9af39623, 33 },
    { 0x45039624, 95 },
    { 0xa6639623, 23 },
    { 0x99139627, 99 },
    { 0x95c39625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e63962e, 55 },
    { 0, 0 },
    { 0x29439630, 84 },
    { 0xe6439630, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb39636, 98 },
    { 0xa1e39636, 23 },
    { 0xfd839638, 23 },
    { 0, 0 },
    { 0x5cd3963a, 23 },
    { 0, 0 },
    { 0x5573963c, 93 },
    { 0, 0 },
    { 0x6df3963e, 55 },
    { 0, 0 },
    { 0x3fe39640, 99 },
    { 0xc7a39640, 33 },
    { 0x4f439642, 34 },
    { 0x5339643, 55 },
    { 0xc7b39644, 90 },
    { 0xa0b39645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50639651, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c39658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf693965e, 61 },
    { 0x5563965e, 93 },
    { 0x4dd39660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c39665, 34 },
    { 0xace39665, 91 },
    { 0x14939665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53966b, 81 },
    { 0x5353966b, 61 },
    { 0, 0 },
    { 0xfc03966e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9639672, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b39676, 90 },
    { 0xa4e39677, 61 },
    { 0x63739678, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63967f, 90 },
    { 0x9053967f, 98 },
    { 0, 0 },
    { 0x38e39682, 34 },
    { 0x4df39683, 55 },
    { 0xe9c39684, 99 },
    { 0, 0 },
    { 0xde339686, 61 },
    { 0x61339686, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a3968b, 46 },
    { 0xed23968b, 90 },
    { 0, 0 },
    { 0xf243968e, 99 },
    { 0x59c3968f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37239695, 76 },
    { 0x32339696, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3969c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee3969f, 85 },
    { 0xe073969f, 95 },
    { 0x15a396a1, 79 },
    { 0x584396a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e396ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260396b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c396b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x575396bc, 46 },
    { 0xe9a396bd, 98 },
    { 0x29e396be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78396c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad396c8, 46 },
    { 0x15396c9, 61 },
    { 0, 0 },
    { 0x6e396cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c396cf, 100 },
    { 0x32f396d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962396d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22396d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45396e0, 76 },
    { 0xfa396e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a396e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e396ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0396f3, 87 },
    { 0, 0 },
    { 0xb5e396f5, 76 },
    { 0, 0 },
    { 0xbb1396f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec396fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7396fe, 95 },
    { 0x9b396ff, 61 },
    { 0xd2839700, 98 },
    { 0x15a396fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad639705, 33 },
    { 0xcc839706, 95 },
    { 0x26d39707, 55 },
    { 0x5bc39708, 66 },
    { 0x28739705, 46 },
    { 0xb8e39705, 55 },
    { 0xff83970b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea39713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f3971a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf39726, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8483972f, 61 },
    { 0xf9f39730, 46 },
    { 0xa2a3972f, 95 },
    { 0x1e739732, 95 },
    { 0x7df39733, 76 },
    { 0, 0 },
    { 0x57239735, 46 },
    { 0x90e39736, 90 },
    { 0, 0 },
    { 0xf0739738, 95 },
    { 0, 0 },
    { 0x4ef3973a, 34 },
    { 0x9e63973b, 91 },
    { 0xfa93973b, 46 },
    { 0, 0 },
    { 0xe9e3973e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2539742, 23 },
    { 0, 0 },
    { 0x17839744, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3839749, 34 },
    { 0, 0 },
    { 0x8023974b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2363974f, 33 },
    { 0xe6c39750, 61 },
    { 0xb4a39751, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43975c, 91 },
    { 0x3a43975c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1739761, 85 },
    { 0, 0 },
    { 0x57839763, 61 },
    { 0xcbf39764, 76 },
    { 0xd6a39765, 46 },
    { 0x18c39765, 91 },
    { 0x1a39766, 33 },
    { 0x47939767, 95 },
    { 0xdf139765, 76 },
    { 0x5713976a, 90 },
    { 0x4213976a, 23 },
    { 0xba23976c, 40 },
    { 0x5c839767, 95 },
    { 0x2663976e, 12 },
    { 0x7743976f, 90 },
    { 0xda439769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5239773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e3977b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8439780, 76 },
    { 0xbc139781, 33 },
    { 0xeeb39780, 95 },
    { 0x81739783, 43 },
    { 0xca439784, 90 },
    { 0xca339784, 98 },
    { 0, 0 },
    { 0x19439787, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc3978a, 61 },
    { 0xf443978a, 38 },
    { 0xae63978c, 55 },
    { 0, 0 },
    { 0x2843978e, 90 },
    { 0xa703978f, 90 },
    { 0, 0 },
    { 0x57a39791, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b39794, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1703979e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01397a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a397a5, 98 },
    { 0xeae397a6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x714397a9, 95 },
    { 0, 0 },
    { 0xec1397ab, 67 },
    { 0xa13397ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e397b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x594397b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1397ba, 55 },
    { 0, 0 },
    { 0xef9397bc, 33 },
    { 0, 0 },
    { 0xfd1397be, 90 },
    { 0x9c397bf, 33 },
    { 0xcc2397be, 55 },
    { 0, 0 },
    { 0xe18397c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d397c6, 90 },
    { 0x1d6397c7, 95 },
    { 0x77e397c8, 99 },
    { 0x79397c9, 33 },
    { 0, 0 },
    { 0x4f397cb, 95 },
    { 0, 0 },
    { 0x865397cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3397d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1397d9, 46 },
    { 0x2b5397da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66397e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4397ec, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb397f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989397f7, 52 },
    { 0x3ee397f7, 90 },
    { 0x15c397f9, 69 },
    { 0xe3d397fa, 95 },
    { 0xd4e397fb, 34 },
    { 0x6e6397fc, 83 },
    { 0x4de397fb, 93 },
    { 0xcf2397fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3539803, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e3980a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe939812, 90 },
    { 0x57f39813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe739816, 100 },
    { 0x58739817, 33 },
    { 0xc0339818, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5883981d, 98 },
    { 0xfda3981e, 85 },
    { 0, 0 },
    { 0x51d39820, 23 },
    { 0xe2339820, 37 },
    { 0xeb439821, 78 },
    { 0x58439821, 55 },
    { 0x85839820, 33 },
    { 0xa5239822, 33 },
    { 0xde39826, 52 },
    { 0x69139823, 90 },
    { 0, 0 },
    { 0xfb439829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13982c, 99 },
    { 0, 0 },
    { 0xc803982e, 83 },
    { 0x1d3982e, 76 },
    { 0x59b39830, 68 },
    { 0xd023982f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73039836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae39839, 76 },
    { 0x4133983a, 77 },
    { 0x2bd39839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c3983e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9739841, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed239845, 61 },
    { 0x9139846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4439852, 66 },
    { 0x21d39853, 68 },
    { 0xbb339853, 93 },
    { 0x47739855, 68 },
    { 0xa8439855, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3985a, 33 },
    { 0x4a03985b, 98 },
    { 0, 0 },
    { 0x1613985d, 61 },
    { 0, 0 },
    { 0x9a13985f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f539864, 38 },
    { 0, 0 },
    { 0xb1939866, 33 },
    { 0x68d39867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243986b, 19 },
    { 0xc413986c, 33 },
    { 0x9273986c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe239874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c239877, 61 },
    { 0x57939877, 95 },
    { 0x29539879, 40 },
    { 0xabe39877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53987d, 66 },
    { 0x7b93987e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x65239881, 76 },
    { 0x7a839881, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45739889, 90 },
    { 0x8b93988a, 55 },
    { 0xc3039889, 87 },
    { 0, 0 },
    { 0x5b93988d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb39892, 46 },
    { 0x94739893, 61 },
    { 0x74239894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7739897, 64 },
    { 0xd8e39898, 23 },
    { 0, 0 },
    { 0x5a23989a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf3989f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507398a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927398ad, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x701398b0, 98 },
    { 0x1d1398b0, 91 },
    { 0xa7398b2, 78 },
    { 0x84b398b2, 90 },
    { 0xc35398b4, 43 },
    { 0x457398b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1398bd, 90 },
    { 0, 0 },
    { 0xfdb398bf, 73 },
    { 0x357398c0, 52 },
    { 0, 0 },
    { 0xa92398c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1398c6, 98 },
    { 0x18e398c7, 55 },
    { 0x487398c8, 98 },
    { 0xcc2398c7, 78 },
    { 0x6ed398ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd398d0, 23 },
    { 0x401398d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120398d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e398db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f398de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6398e1, 90 },
    { 0x713398e2, 87 },
    { 0, 0 },
    { 0xec3398e4, 90 },
    { 0, 0 },
    { 0x44c398e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09398e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb398ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9398f1, 33 },
    { 0x2de398f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260398f6, 33 },
    { 0x823398f7, 23 },
    { 0, 0 },
    { 0x8f1398f9, 33 },
    { 0, 0 },
    { 0x2b3398fb, 43 },
    { 0xde0398fc, 33 },
    { 0x364398fd, 17 },
    { 0xf15398fd, 33 },
    { 0xf2398ff, 23 },
    { 0xf8939900, 61 },
    { 0xa6e398fe, 90 },
    { 0x78398fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62239905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d639909, 77 },
    { 0xfa13990a, 55 },
    { 0x69339909, 98 },
    { 0, 0 },
    { 0xdfe3990d, 23 },
    { 0x5be3990d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b39914, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43991b, 91 },
    { 0, 0 },
    { 0x1c23991d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f239922, 78 },
    { 0, 0 },
    { 0xd9d39924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a539929, 90 },
    { 0, 0 },
    { 0xdcd3992b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde839931, 46 },
    { 0xe0139932, 61 },
    { 0, 0 },
    { 0xf6139934, 61 },
    { 0xfbf39935, 61 },
    { 0xd1f39936, 95 },
    { 0x75d39935, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f3993f, 98 },
    { 0xe13993f, 17 },
    { 0x4683993f, 33 },
    { 0x12c39942, 98 },
    { 0x9043993f, 61 },
    { 0xaa139944, 23 },
    { 0, 0 },
    { 0xe0339946, 98 },
    { 0x3e839947, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f639950, 67 },
    { 0xfc639951, 34 },
    { 0, 0 },
    { 0x6ed39953, 83 },
    { 0xfe339953, 98 },
    { 0xa0939955, 55 },
    { 0, 0 },
    { 0x2c39957, 95 },
    { 0x5039958, 69 },
    { 0, 0 },
    { 0x7de3995a, 93 },
    { 0x2a23995b, 33 },
    { 0, 0 },
    { 0x49c3995d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19039960, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b3996b, 76 },
    { 0xffa3996b, 55 },
    { 0x273996d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca39978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333997e, 98 },
    { 0x9783997f, 55 },
    { 0x29839980, 100 },
    { 0x62b39980, 91 },
    { 0x38b39982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13939989, 61 },
    { 0, 0 },
    { 0x5183998b, 90 },
    { 0x5673998b, 95 },
    { 0xc433998d, 87 },
    { 0, 0 },
    { 0x493998f, 93 },
    { 0xab039990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd639993, 55 },
    { 0, 0 },
    { 0x9ee39995, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f39998, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc399a0, 99 },
    { 0x62a399a1, 87 },
    { 0x1ad399a0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af399a6, 61 },
    { 0, 0 },
    { 0x693399a8, 40 },
    { 0x1399a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09399b0, 43 },
    { 0x47d399b1, 100 },
    { 0x842399b2, 90 },
    { 0x678399b3, 43 },
    { 0xffd399b2, 44 },
    { 0, 0 },
    { 0x8ee399b6, 58 },
    { 0, 0 },
    { 0x1ca399b8, 17 },
    { 0x863399b8, 66 },
    { 0, 0 },
    { 0x5eb399bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814399bf, 95 },
    { 0x579399bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5399c8, 61 },
    { 0, 0 },
    { 0x68c399ca, 78 },
    { 0x8cb399cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897399d1, 33 },
    { 0xf30399d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69399d5, 96 },
    { 0x92f399d6, 99 },
    { 0xcc3399d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202399dd, 91 },
    { 0x4e4399de, 98 },
    { 0x35e399df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f399eb, 90 },
    { 0, 0 },
    { 0xbf5399ed, 59 },
    { 0xc99399ee, 93 },
    { 0x16f399ef, 98 },
    { 0x53399f0, 61 },
    { 0, 0 },
    { 0x91c399f2, 33 },
    { 0xc72399f3, 23 },
    { 0x129399f3, 61 },
    { 0xb41399f2, 90 },
    { 0xe45399f6, 33 },
    { 0xb9f399f7, 61 },
    { 0, 0 },
    { 0xe91399f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1399fc, 75 },
    { 0xe99399fd, 33 },
    { 0, 0 },
    { 0xb18399ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7239a05, 61 },
    { 0, 0 },
    { 0x2d239a07, 55 },
    { 0xf8b39a07, 46 },
    { 0x61439a09, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f039a0f, 90 },
    { 0, 0 },
    { 0xc1039a11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e39a1a, 100 },
    { 0, 0 },
    { 0x7df39a1c, 93 },
    { 0, 0 },
    { 0x1f539a1e, 55 },
    { 0x51639a1f, 100 },
    { 0xd3a39a1e, 31 },
    { 0xdbe39a21, 55 },
    { 0xc6b39a22, 34 },
    { 0xaee39a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b39a26, 100 },
    { 0xad739a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60439a2d, 100 },
    { 0x1b939a2e, 90 },
    { 0x60e39a2f, 61 },
    { 0xec239a2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd039a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71839a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd39a45, 90 },
    { 0x95c39a46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c39a4b, 43 },
    { 0, 0 },
    { 0xefc39a4d, 80 },
    { 0xda639a4e, 33 },
    { 0xa0639a4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75239a56, 76 },
    { 0xe3239a57, 90 },
    { 0xdf239a56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c39a5c, 98 },
    { 0x2cb39a5c, 90 },
    { 0xb1739a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f739a62, 33 },
    { 0x1d239a63, 46 },
    { 0xb4a39a64, 61 },
    { 0, 0 },
    { 0x5ff39a66, 90 },
    { 0x63239a67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed239a6c, 95 },
    { 0x80639a6c, 43 },
    { 0x13e39a6c, 95 },
    { 0xf7e39a6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d739a75, 98 },
    { 0x9db39a75, 46 },
    { 0x9a39a77, 98 },
    { 0xf8939a77, 33 },
    { 0x38439a79, 78 },
    { 0xc3439a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70639a83, 100 },
    { 0x2a339a83, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93439a8a, 40 },
    { 0, 0 },
    { 0xe6239a8c, 98 },
    { 0, 0 },
    { 0xf6d39a8e, 84 },
    { 0, 0 },
    { 0xbc339a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd39a94, 33 },
    { 0xe2939a95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c39a99, 34 },
    { 0, 0 },
    { 0x1c639a9b, 90 },
    { 0xea139a9c, 68 },
    { 0, 0 },
    { 0xc0439a9e, 95 },
    { 0, 0 },
    { 0xcb139aa0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc839aa3, 90 },
    { 0x3ae39aa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee339aaa, 61 },
    { 0, 0 },
    { 0x6bb39aac, 61 },
    { 0xa7e39aad, 43 },
    { 0x8539aad, 95 },
    { 0x73b39aac, 99 },
    { 0xdc739aaf, 90 },
    { 0x5b939ab0, 61 },
    { 0x1439ab2, 90 },
    { 0xd6239ab3, 85 },
    { 0x59739ab4, 90 },
    { 0, 0 },
    { 0x62239ab6, 87 },
    { 0, 0 },
    { 0x9ab39ab8, 38 },
    { 0x26839ab9, 78 },
    { 0x76439aba, 40 },
    { 0xb8039abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f839abe, 97 },
    { 0x6fc39abf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d339acf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9039ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c39adf, 61 },
    { 0, 0 },
    { 0xc7c39ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13739ae5, 55 },
    { 0, 0 },
    { 0x6439ae7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25139af4, 98 },
    { 0xbc439af4, 90 },
    { 0xf0c39af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb39afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c739b05, 95 },
    { 0xc8039b06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d39b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5839b0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3039b10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45239b13, 90 },
    { 0xec39b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0239b19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7839b1e, 33 },
    { 0, 0 },
    { 0x11439b20, 95 },
    { 0, 0 },
    { 0x2e139b22, 90 },
    { 0, 0 },
    { 0xd6039b24, 46 },
    { 0, 0 },
    { 0x1b739b26, 85 },
    { 0x45b39b27, 90 },
    { 0, 0 },
    { 0xe639b29, 33 },
    { 0xc5d39b2a, 46 },
    { 0x65539b2b, 91 },
    { 0x54239b2b, 23 },
    { 0x84a39b2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a939b32, 87 },
    { 0x4d939b33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92439b36, 95 },
    { 0x57539b37, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a39b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6239b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba39b44, 61 },
    { 0xa1839b45, 90 },
    { 0x5e039b46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39b4c, 87 },
    { 0x17139b4d, 95 },
    { 0xe6339b4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a39b53, 90 },
    { 0, 0 },
    { 0x25439b55, 23 },
    { 0x5da39b55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe39b59, 34 },
    { 0x44139b5a, 61 },
    { 0x81639b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b39b68, 66 },
    { 0x59739b69, 46 },
    { 0x96939b6a, 23 },
    { 0x71039b6b, 43 },
    { 0x7e139b68, 33 },
    { 0x70b39b6d, 33 },
    { 0x19739b6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92039b75, 61 },
    { 0x88139b76, 33 },
    { 0, 0 },
    { 0xefa39b78, 90 },
    { 0x39539b79, 61 },
    { 0xe5b39b7a, 75 },
    { 0x60339b7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc039b7e, 90 },
    { 0xe6739b7f, 69 },
    { 0x60b39b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46239b83, 90 },
    { 0xf3a39b84, 98 },
    { 0x7da39b85, 64 },
    { 0x95e39b85, 55 },
    { 0x75539b87, 33 },
    { 0xef839b85, 61 },
    { 0x4ae39b88, 61 },
    { 0x38239b8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c039b8d, 43 },
    { 0xf1b39b8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f839b91, 61 },
    { 0x11f39b92, 33 },
    { 0xaa239b93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f539b98, 87 },
    { 0xe3939b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59139b9d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a39ba2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d39ba5, 34 },
    { 0x8d339ba5, 95 },
    { 0x17239ba7, 66 },
    { 0, 0 },
    { 0x87939ba9, 90 },
    { 0, 0 },
    { 0x75339bab, 52 },
    { 0x9b539bab, 93 },
    { 0xd0a39bab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae739bb0, 43 },
    { 0x56c39bb1, 90 },
    { 0x38839bb0, 84 },
    { 0xdad39bb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d439bb7, 55 },
    { 0x12639bb8, 55 },
    { 0x5d639bb9, 98 },
    { 0x5439bba, 23 },
    { 0, 0 },
    { 0x50139bbc, 95 },
    { 0, 0 },
    { 0x42f39bbe, 90 },
    { 0x84c39bbe, 43 },
    { 0x62139bc0, 77 },
    { 0x1539bbf, 90 },
    { 0x8439bc2, 46 },
    { 0x7bb39bc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2539bcd, 90 },
    { 0x2ce39bcd, 77 },
    { 0x7a339bcd, 61 },
    { 0x48039bd0, 33 },
    { 0xfe439bd1, 90 },
    { 0x16039bd2, 61 },
    { 0xc6039bd3, 55 },
    { 0x71a39bd2, 96 },
    { 0, 0 },
    { 0xa39bd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb39bde, 98 },
    { 0x92039bdf, 65 },
    { 0, 0 },
    { 0xa1e39be1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c39be5, 23 },
    { 0, 0 },
    { 0x3b139be7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e39bea, 46 },
    { 0xf8739bea, 67 },
    { 0x69239bec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b39bf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf039bf4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7839bf7, 83 },
    { 0xe6f39bf7, 90 },
    { 0xe3439bf9, 33 },
    { 0x7d939bfa, 43 },
    { 0x48d39bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94839c00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f39c06, 33 },
    { 0x44c39c07, 52 },
    { 0x9ea39c06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe39c0e, 99 },
    { 0, 0 },
    { 0x34039c10, 78 },
    { 0x73939c10, 90 },
    { 0x7d139c12, 34 },
    { 0x72c39c13, 95 },
    { 0x5d39c11, 43 },
    { 0xedf39c12, 90 },
    { 0xd0139c10, 69 },
    { 0, 0 },
    { 0xff739c18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8739c27, 95 },
    { 0x30b39c28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7539c2c, 61 },
    { 0x3b839c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c39c32, 90 },
    { 0, 0 },
    { 0xead39c34, 43 },
    { 0, 0 },
    { 0xb9539c36, 55 },
    { 0, 0 },
    { 0x82139c38, 34 },
    { 0xa3639c39, 43 },
    { 0x33139c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e39c3e, 75 },
    { 0x39b39c3f, 55 },
    { 0x11939c40, 90 },
    { 0xfb239c41, 43 },
    { 0x6339c41, 33 },
    { 0x62839c43, 23 },
    { 0xc9239c43, 76 },
    { 0x30739c40, 61 },
    { 0xf8439c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77839c49, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c39c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae39c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb39c53, 78 },
    { 0xa0739c54, 61 },
    { 0x9a739c55, 33 },
    { 0x42339c56, 81 },
    { 0, 0 },
    { 0x26c39c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde039c5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71339c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49539c6b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb39c70, 81 },
    { 0xa5739c71, 55 },
    { 0xcc39c72, 33 },
    { 0xa9c39c70, 90 },
    { 0, 0 },
    { 0x3eb39c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c139c79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d39c82, 99 },
    { 0x1e339c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26739c88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97639c8c, 90 },
    { 0xbbf39c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6039c91, 46 },
    { 0xd0939c92, 95 },
    { 0x76339c93, 90 },
    { 0xce039c93, 96 },
    { 0x3439c94, 98 },
    { 0x22d39c96, 90 },
    { 0xe3039c97, 55 },
    { 0x4c239c91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e39c9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c39c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf39ca1, 100 },
    { 0xd0839ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11339ca5, 90 },
    { 0, 0 },
    { 0x60739ca7, 66 },
    { 0xc939ca8, 91 },
    { 0xc8139ca9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c639cac, 17 },
    { 0xe4d39cad, 61 },
    { 0, 0 },
    { 0x59e39caf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96239cb3, 83 },
    { 0x37139cb3, 17 },
    { 0, 0 },
    { 0x84139cb6, 23 },
    { 0x3b839cb7, 33 },
    { 0xb0739cb8, 87 },
    { 0x9ba39cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x45839cc2, 87 },
    { 0, 0 },
    { 0x46339cc4, 23 },
    { 0, 0 },
    { 0x85b39cc6, 95 },
    { 0x13e39cc7, 52 },
    { 0x25639cc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d39ccf, 98 },
    { 0x32f39cd0, 93 },
    { 0xc4139cd1, 40 },
    { 0xdb839cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f39cd6, 68 },
    { 0xc0039cd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c339cda, 93 },
    { 0xa6139cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39cdf, 90 },
    { 0x84439ce0, 99 },
    { 0, 0 },
    { 0x11039ce2, 90 },
    { 0xd4c39ce3, 76 },
    { 0xc0139ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a39ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54239cec, 72 },
    { 0xdbe39ced, 66 },
    { 0xd7439cee, 98 },
    { 0xaa639cee, 94 },
    { 0xf7839cf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139cf3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e939cfb, 76 },
    { 0, 0 },
    { 0x36439cfd, 43 },
    { 0x3d239cfe, 90 },
    { 0x70839cfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7139d04, 90 },
    { 0x79039d05, 61 },
    { 0x5d739d04, 66 },
    { 0xbe639d07, 69 },
    { 0, 0 },
    { 0xc6239d09, 61 },
    { 0xe4939d09, 43 },
    { 0x88639d0b, 90 },
    { 0xcfe39d0c, 61 },
    { 0xe1439d0a, 76 },
    { 0x23b39d0e, 90 },
    { 0xfb539d0f, 33 },
    { 0x5e939d0f, 43 },
    { 0xbbd39d11, 90 },
    { 0xf2539d0d, 17 },
    { 0xa2839d0e, 90 },
    { 0xe3b39d14, 95 },
    { 0x27f39d15, 87 },
    { 0x13939d0f, 46 },
    { 0x27139d17, 55 },
    { 0xa9a39d0f, 34 },
    { 0x7f839d19, 23 },
    { 0x3f839d11, 76 },
    { 0xd4f39d1b, 98 },
    { 0x63a39d1b, 33 },
    { 0x5d239d1d, 98 },
    { 0xff039d0d, 61 },
    { 0xb2739d1f, 93 },
    { 0xb4739d20, 33 },
    { 0xe5b39d21, 23 },
    { 0x6f939d0d, 43 },
    { 0x64d39d23, 34 },
    { 0, 0 },
    { 0x7d339d25, 66 },
    { 0xd4b39d26, 55 },
    { 0, 0 },
    { 0xbcf39d28, 88 },
    { 0xdf539d28, 34 },
    { 0xedd39d2a, 52 },
    { 0x61f39d2b, 53 },
    { 0x45239d2c, 90 },
    { 0, 0 },
    { 0x96a39d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c39d35, 55 },
    { 0, 0 },
    { 0x65a39d37, 85 },
    { 0x5c439d38, 101 },
    { 0, 0 },
    { 0x72e39d3a, 76 },
    { 0, 0 },
    { 0x87039d3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca39d3f, 33 },
    { 0x2f139d40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4639d43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe39d47, 76 },
    { 0x21039d48, 66 },
    { 0xb5239d48, 46 },
    { 0x35739d47, 23 },
    { 0xa8439d4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5739d51, 90 },
    { 0x5d39d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61739d55, 19 },
    { 0xf3639d56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d39d5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27239d62, 78 },
    { 0x99a39d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41739d69, 52 },
    { 0x50539d69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x44239d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb639d77, 34 },
    { 0, 0 },
    { 0x83239d79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc39d88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x99339d8b, 53 },
    { 0, 0 },
    { 0xb6a39d8d, 91 },
    { 0x51f39d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2339d92, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f39d96, 93 },
    { 0xf4539d97, 78 },
    { 0x20a39d98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b39d9e, 76 },
    { 0x55d39d9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76239da8, 33 },
    { 0xcdc39da9, 19 },
    { 0xcd139da9, 37 },
    { 0xd6e39dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9739db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a39db8, 84 },
    { 0, 0 },
    { 0x7f439dba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e39dbd, 76 },
    { 0xccf39dbd, 90 },
    { 0xdb439dbf, 33 },
    { 0x87539dc0, 81 },
    { 0, 0 },
    { 0x54739dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef639dc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2839dcb, 99 },
    { 0x12839dcc, 98 },
    { 0x46239dcc, 33 },
    { 0xf5b39dce, 98 },
    { 0xdb139dcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d39dd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba39dd9, 98 },
    { 0, 0 },
    { 0x5e639ddb, 76 },
    { 0x62239ddb, 43 },
    { 0x7f739ddb, 33 },
    { 0, 0 },
    { 0xfdd39ddf, 23 },
    { 0x5ff39de0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f939deb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97539def, 46 },
    { 0xd5139df0, 90 },
    { 0xe1739def, 23 },
    { 0, 0 },
    { 0x639df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2039dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed839dfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77439e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee439e05, 23 },
    { 0, 0 },
    { 0x33e39e07, 80 },
    { 0, 0 },
    { 0x62539e09, 57 },
    { 0xad839e0a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8339e0f, 61 },
    { 0xc039e10, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa39e13, 23 },
    { 0, 0 },
    { 0x3af39e15, 76 },
    { 0, 0 },
    { 0x23f39e17, 95 },
    { 0, 0 },
    { 0xc4939e19, 78 },
    { 0x19b39e1a, 90 },
    { 0xa4039e1b, 75 },
    { 0xf9339e1c, 76 },
    { 0x14039e1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd539e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf539e23, 78 },
    { 0xa5039e24, 23 },
    { 0xef639e25, 33 },
    { 0, 0 },
    { 0x10839e27, 93 },
    { 0x3ae39e28, 34 },
    { 0, 0 },
    { 0x45239e2a, 90 },
    { 0x42239e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4639e34, 90 },
    { 0x36b39e35, 61 },
    { 0x8a139e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e39e39, 84 },
    { 0x86f39e3a, 95 },
    { 0x49739e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73439e3f, 98 },
    { 0x2c439e40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2039e43, 55 },
    { 0x93339e44, 33 },
    { 0xc7839e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a39e49, 55 },
    { 0x9bf39e49, 33 },
    { 0x81339e4b, 55 },
    { 0xaab39e4c, 77 },
    { 0, 0 },
    { 0xb3139e4e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa39e51, 23 },
    { 0xa9639e52, 34 },
    { 0x97f39e53, 65 },
    { 0, 0 },
    { 0x6239e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80339e5d, 90 },
    { 0x2a39e5d, 23 },
    { 0x6d239e5f, 90 },
    { 0x3a639e60, 61 },
    { 0x21939e5e, 23 },
    { 0, 0 },
    { 0x16839e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84539e6b, 66 },
    { 0xae239e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0139e70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14539e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a39e85, 33 },
    { 0x83a39e86, 76 },
    { 0, 0 },
    { 0x68939e88, 46 },
    { 0x17b39e89, 90 },
    { 0x3a639e89, 95 },
    { 0xd0639e8b, 95 },
    { 0x7b939e8c, 90 },
    { 0x28539e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b39e90, 95 },
    { 0x25e39e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39e97, 40 },
    { 0x8ef39e98, 66 },
    { 0xc9e39e97, 90 },
    { 0, 0 },
    { 0x51039e9b, 90 },
    { 0xab739e9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2439ea2, 52 },
    { 0, 0 },
    { 0x20839ea4, 90 },
    { 0, 0 },
    { 0xf5a39ea6, 52 },
    { 0, 0 },
    { 0x21739ea8, 90 },
    { 0x6a739ea9, 61 },
    { 0x52f39eaa, 90 },
    { 0x54839ea9, 43 },
    { 0xa8439ea9, 90 },
    { 0, 0 },
    { 0xfa639eae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd39eb4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca39eb9, 46 },
    { 0x36239eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c39ec9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c39ecd, 61 },
    { 0xd5939ece, 55 },
    { 0xab139ecd, 76 },
    { 0xfee39ecd, 90 },
    { 0xcb939ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80739ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd39eda, 90 },
    { 0xd0c39edb, 23 },
    { 0, 0 },
    { 0x71539edd, 90 },
    { 0x5ea39edd, 77 },
    { 0x6d739ede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5639ee2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f39ee6, 91 },
    { 0, 0 },
    { 0x61239ee8, 96 },
    { 0x6b739ee8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a39eee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b839ef1, 61 },
    { 0, 0 },
    { 0x37139ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd39ef8, 68 },
    { 0xca139ef8, 76 },
    { 0xb6139efa, 100 },
    { 0xe9c39efa, 98 },
    { 0x50b39efc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9239f01, 81 },
    { 0, 0 },
    { 0xca439f03, 23 },
    { 0, 0 },
    { 0x44939f05, 61 },
    { 0xc1b39f06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xff539f09, 61 },
    { 0x5d539f0a, 66 },
    { 0x12039f0a, 55 },
    { 0x3ce39f0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50439f10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6239f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6139f18, 68 },
    { 0x7339f19, 81 },
    { 0x11839f1a, 23 },
    { 0x4be39f19, 34 },
    { 0x34e39f1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa39f20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939f2a, 93 },
    { 0xdaa39f2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd39f2e, 90 },
    { 0, 0 },
    { 0x97a39f30, 33 },
    { 0xc8539f31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3239f36, 36 },
    { 0xfd739f37, 99 },
    { 0, 0 },
    { 0x29939f39, 33 },
    { 0x25839f39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0939f3f, 91 },
    { 0xc8f39f3f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b39f4b, 98 },
    { 0, 0 },
    { 0xf2b39f4d, 34 },
    { 0xd7b39f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54139f59, 99 },
    { 0xe1739f5a, 34 },
    { 0x1f39f59, 76 },
    { 0, 0 },
    { 0xf5039f5d, 84 },
    { 0, 0 },
    { 0xcef39f5f, 98 },
    { 0x35139f60, 99 },
    { 0, 0 },
    { 0x39d39f62, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8139f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb39f69, 68 },
    { 0, 0 },
    { 0x57039f6b, 43 },
    { 0xc8539f6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62839f71, 64 },
    { 0xafe39f72, 76 },
    { 0xf1239f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a39f76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f39f7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a39f7d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9239f80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a39f86, 78 },
    { 0x23d39f87, 81 },
    { 0x8ff39f86, 98 },
    { 0xd6e39f88, 55 },
    { 0x63739f87, 95 },
    { 0xbab39f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9439f8e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb839f91, 95 },
    { 0x2d339f91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed39f97, 23 },
    { 0x73e39f98, 90 },
    { 0xcb839f97, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1939f9c, 23 },
    { 0x85739f9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf39fa0, 55 },
    { 0x2a939fa1, 90 },
    { 0x9af39fa2, 95 },
    { 0x3c439fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5839fa9, 33 },
    { 0xa9539faa, 90 },
    { 0x8ed39fab, 76 },
    { 0xd3939fac, 61 },
    { 0x7fd39fad, 55 },
    { 0xb6639fae, 99 },
    { 0xfa939fab, 43 },
    { 0xdba39fb0, 61 },
    { 0, 0 },
    { 0x69039fb2, 60 },
    { 0, 0 },
    { 0xc4739fb4, 90 },
    { 0, 0 },
    { 0x9eb39fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d439fbe, 23 },
    { 0xa6939fbf, 55 },
    { 0x12039fc0, 90 },
    { 0x96b39fbf, 95 },
    { 0, 0 },
    { 0xd3339fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0439fc7, 93 },
    { 0x8de39fc8, 93 },
    { 0x49c39fc9, 90 },
    { 0x1939fca, 66 },
    { 0xa6e39fcb, 90 },
    { 0x26c39fc9, 33 },
    { 0xf2239fcc, 85 },
    { 0, 0 },
    { 0x46639fcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3139fd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74939fdd, 55 },
    { 0, 0 },
    { 0x6e239fdf, 46 },
    { 0x68339fe0, 90 },
    { 0x7339fe1, 90 },
    { 0x15c39fe1, 90 },
    { 0xf8839fe3, 98 },
    { 0, 0 },
    { 0xd539fe5, 93 },
    { 0, 0 },
    { 0xc7139fe7, 55 },
    { 0, 0 },
    { 0x2f139fe9, 95 },
    { 0x3ae39fea, 55 },
    { 0x3a439feb, 80 },
    { 0x37a39fea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe39ff3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6239ff6, 61 },
    { 0, 0 },
    { 0xde839ff8, 61 },
    { 0x26f39ff9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ffc, 65 },
    { 0xc7439ffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133a002, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe213a006, 43 },
    { 0x3483a007, 61 },
    { 0x6e03a008, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553a00b, 33 },
    { 0x8fa3a00c, 94 },
    { 0xf113a00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed3a011, 84 },
    { 0x9c63a012, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f3a015, 61 },
    { 0xc0c3a016, 80 },
    { 0, 0 },
    { 0x2303a018, 99 },
    { 0xa983a019, 95 },
    { 0, 0 },
    { 0x2f03a01b, 23 },
    { 0, 0 },
    { 0x9e13a01d, 98 },
    { 0, 0 },
    { 0xcd03a01f, 76 },
    { 0x5fb3a020, 23 },
    { 0xea03a020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4373a027, 90 },
    { 0x7383a027, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb43a02f, 98 },
    { 0xb953a02f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403a035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df3a038, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a33a03c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd23a041, 84 },
    { 0xd983a042, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3a048, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd3a04c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa913a04f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a3a055, 55 },
    { 0x48a3a056, 90 },
    { 0x43a3a056, 61 },
    { 0x14c3a055, 98 },
    { 0, 0 },
    { 0xeb73a05a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd523a05d, 100 },
    { 0x7cc3a05d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a3a061, 43 },
    { 0xb423a061, 101 },
    { 0, 0 },
    { 0x72c3a064, 95 },
    { 0xe843a064, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b3a06b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3743a070, 91 },
    { 0, 0 },
    { 0x33e3a072, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9673a078, 75 },
    { 0x5723a079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623a07e, 33 },
    { 0x1783a07f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac3a082, 95 },
    { 0xf663a083, 87 },
    { 0x2663a083, 95 },
    { 0xef63a083, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7013a088, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b3a08b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3a08f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab3a093, 98 },
    { 0x8e13a094, 98 },
    { 0xd013a095, 85 },
    { 0xd533a096, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3a09a, 85 },
    { 0, 0 },
    { 0xc73a09c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1303a0a0, 46 },
    { 0x5e93a0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e93a0a5, 95 },
    { 0xf53a0a6, 33 },
    { 0x71b3a0a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c3a0ad, 90 },
    { 0x9673a0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f3a0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d93a0b5, 33 },
    { 0x8a3a0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2423a0bb, 90 },
    { 0xb693a0bc, 34 },
    { 0xd923a0bd, 77 },
    { 0x2dc3a0be, 72 },
    { 0x8503a0be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873a0c2, 33 },
    { 0x853a0c3, 95 },
    { 0, 0 },
    { 0x7383a0c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d23a0ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b3a0ce, 52 },
    { 0xdab3a0cf, 55 },
    { 0, 0 },
    { 0x3fc3a0d1, 90 },
    { 0x9ac3a0d2, 43 },
    { 0, 0 },
    { 0x20d3a0d4, 90 },
    { 0, 0 },
    { 0x2603a0d6, 55 },
    { 0, 0 },
    { 0x6833a0d8, 33 },
    { 0x8403a0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3a0dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf53a0e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493a0e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf133a0eb, 55 },
    { 0, 0 },
    { 0xf6c3a0ed, 78 },
    { 0xf813a0ed, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4293a0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13a0f5, 78 },
    { 0, 0 },
    { 0xb7b3a0f7, 67 },
    { 0xc23a0f7, 61 },
    { 0xc753a0f9, 23 },
    { 0, 0 },
    { 0x1113a0fb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1693a0fe, 98 },
    { 0, 0 },
    { 0x1c93a100, 85 },
    { 0xd6b3a100, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d3a104, 98 },
    { 0, 0 },
    { 0x21a3a106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc3a10a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3a115, 93 },
    { 0x7b3a116, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c33a11d, 33 },
    { 0xeb63a11e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83a125, 96 },
    { 0x7d13a126, 90 },
    { 0, 0 },
    { 0xc53a128, 61 },
    { 0, 0 },
    { 0xf103a12a, 26 },
    { 0xb383a12b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a131, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43a136, 78 },
    { 0x7033a136, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc953a13a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d73a13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e3a142, 68 },
    { 0x6ab3a143, 43 },
    { 0x9483a144, 55 },
    { 0x4163a143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a14c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3473a14f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc593a156, 55 },
    { 0x9a83a157, 83 },
    { 0x2b3a158, 88 },
    { 0x73a157, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9663a15f, 38 },
    { 0x8833a160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f3a164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a63a16a, 55 },
    { 0x1df3a16b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a171, 38 },
    { 0xbf13a172, 99 },
    { 0x6773a173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983a177, 90 },
    { 0x2673a178, 61 },
    { 0x6053a177, 46 },
    { 0, 0 },
    { 0x4143a17b, 61 },
    { 0, 0 },
    { 0x8d63a17d, 23 },
    { 0x2e33a17d, 95 },
    { 0, 0 },
    { 0x1e93a180, 61 },
    { 0xda83a181, 95 },
    { 0xf263a182, 95 },
    { 0x58d3a183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a18b, 95 },
    { 0xc5c3a18b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef3a191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323a197, 98 },
    { 0x6a13a197, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac3a19b, 91 },
    { 0x70a3a19c, 87 },
    { 0xfa83a19d, 78 },
    { 0xf2a3a19e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa3a1a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf063a1a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc73a1b6, 85 },
    { 0x9b43a1b7, 55 },
    { 0xb4f3a1b8, 52 },
    { 0x1633a1b8, 98 },
    { 0x47e3a1ba, 52 },
    { 0, 0 },
    { 0x5a53a1bc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c3a1bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3a1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d83a1c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb193a1cc, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3353a1d4, 34 },
    { 0x2ee3a1d4, 95 },
    { 0, 0 },
    { 0xe0f3a1d7, 33 },
    { 0x9c3a1d7, 66 },
    { 0, 0 },
    { 0xf263a1da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc3a1df, 98 },
    { 0xd273a1e0, 90 },
    { 0, 0 },
    { 0x1a53a1e2, 99 },
    { 0x1b93a1e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc3a1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe933a1eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc163a1ee, 34 },
    { 0xaf53a1ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe813a1f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a1f6, 46 },
    { 0, 0 },
    { 0xca33a1f8, 93 },
    { 0xa383a1f9, 76 },
    { 0xd893a1fa, 95 },
    { 0x98e3a1fb, 100 },
    { 0x8a13a1fc, 76 },
    { 0x73d3a1fb, 33 },
    { 0xb6c3a1fe, 95 },
    { 0xd1d3a1ff, 90 },
    { 0, 0 },
    { 0x7a93a201, 90 },
    { 0, 0 },
    { 0x4183a203, 33 },
    { 0xfe53a204, 91 },
    { 0, 0 },
    { 0x5573a206, 90 },
    { 0x9663a207, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b3a20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3a214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e3a21e, 95 },
    { 0x33c3a21f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e3a223, 90 },
    { 0xf2b3a223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8463a227, 46 },
    { 0x1e83a227, 98 },
    { 0x2b23a227, 91 },
    { 0, 0 },
    { 0x6303a22b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763a234, 33 },
    { 0x8333a235, 45 },
    { 0x79b3a236, 23 },
    { 0, 0 },
    { 0x84d3a238, 95 },
    { 0, 0 },
    { 0xc933a23a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3a23e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d3a242, 61 },
    { 0x73c3a243, 98 },
    { 0xd373a244, 55 },
    { 0x6ec3a244, 90 },
    { 0x8613a243, 75 },
    { 0xf523a245, 33 },
    { 0x1783a244, 23 },
    { 0, 0 },
    { 0xc0d3a24a, 100 },
    { 0xad3a24b, 90 },
    { 0, 0 },
    { 0x54b3a24d, 93 },
    { 0x7b3a24d, 23 },
    { 0xf6a3a24d, 43 },
    { 0x26e3a250, 61 },
    { 0xb723a251, 78 },
    { 0, 0 },
    { 0x5a3a253, 78 },
    { 0, 0 },
    { 0x5863a255, 55 },
    { 0, 0 },
    { 0x7a3a257, 33 },
    { 0, 0 },
    { 0x9243a259, 98 },
    { 0x2f93a25a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03a261, 98 },
    { 0xb6c3a261, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3a26d, 98 },
    { 0x71e3a26e, 81 },
    { 0xe2e3a26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd433a275, 90 },
    { 0, 0 },
    { 0x3d23a277, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a27c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc93a281, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9613a285, 33 },
    { 0x613a286, 33 },
    { 0xbfd3a285, 17 },
    { 0xce13a288, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef73a28e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7993a293, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa3a297, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63a29b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a2a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8443a2a5, 76 },
    { 0, 0 },
    { 0xd6a3a2a7, 33 },
    { 0x95a3a2a8, 90 },
    { 0xb873a2a9, 87 },
    { 0x2bb3a2aa, 98 },
    { 0x5873a2a9, 98 },
    { 0x3183a2ac, 23 },
    { 0x5873a2ad, 33 },
    { 0xe1a3a2ac, 55 },
    { 0x68f3a2ac, 68 },
    { 0, 0 },
    { 0xc593a2b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213a2b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e83a2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b73a2c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7633a2c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9273a2d1, 43 },
    { 0x3663a2d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013a2d5, 61 },
    { 0x3003a2d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813a2dc, 85 },
    { 0xa503a2dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe893a2e1, 90 },
    { 0x8723a2e1, 98 },
    { 0, 0 },
    { 0x7f63a2e4, 43 },
    { 0x6523a2e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a2ef, 91 },
    { 0xd043a2f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483a2fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103a304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a307, 23 },
    { 0xbda3a307, 52 },
    { 0xf713a307, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca3a30c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7583a311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6153a314, 91 },
    { 0x4f03a315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb283a319, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323a327, 90 },
    { 0x2773a327, 90 },
    { 0xe2b3a329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563a32c, 61 },
    { 0, 0 },
    { 0x57f3a32e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3a331, 75 },
    { 0x1e63a332, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b3a335, 46 },
    { 0, 0 },
    { 0x2cb3a337, 98 },
    { 0xb653a338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da3a33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a345, 61 },
    { 0xf4b3a346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe773a34a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f3a34d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed43a350, 65 },
    { 0xbe33a350, 61 },
    { 0x35c3a352, 81 },
    { 0x89a3a353, 76 },
    { 0x28a3a354, 98 },
    { 0xd43a355, 76 },
    { 0x3f23a355, 23 },
    { 0, 0 },
    { 0xe733a358, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb3a35e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a361, 95 },
    { 0x6df3a362, 66 },
    { 0, 0 },
    { 0x38a3a364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf553a367, 90 },
    { 0x793a368, 98 },
    { 0x86c3a369, 93 },
    { 0, 0 },
    { 0xd213a36b, 66 },
    { 0x6b83a36c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833a372, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb993a376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a3a379, 76 },
    { 0, 0 },
    { 0xf283a37b, 90 },
    { 0xbef3a37c, 95 },
    { 0x7c03a37b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf473a384, 33 },
    { 0xf83a385, 78 },
    { 0x6913a386, 90 },
    { 0x1f53a387, 23 },
    { 0x4183a385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83a38c, 76 },
    { 0x14a3a38d, 80 },
    { 0x3c83a38c, 95 },
    { 0xca03a38c, 69 },
    { 0x8e3a390, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b3a395, 95 },
    { 0xa6c3a396, 61 },
    { 0x6d73a397, 33 },
    { 0x9023a396, 90 },
    { 0x6fa3a399, 98 },
    { 0, 0 },
    { 0x2b13a39b, 99 },
    { 0, 0 },
    { 0x1873a39d, 87 },
    { 0x1013a39e, 61 },
    { 0x743a39e, 90 },
    { 0x19f3a39f, 90 },
    { 0xf933a3a1, 66 },
    { 0x6143a3a1, 43 },
    { 0x6ff3a3a3, 55 },
    { 0xd343a3a3, 67 },
    { 0x1543a39d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe303a3a8, 43 },
    { 0, 0 },
    { 0xc583a3aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763a3ae, 76 },
    { 0x2fb3a3ae, 61 },
    { 0x3653a3b0, 23 },
    { 0xaa83a3b0, 90 },
    { 0xc33a3b2, 66 },
    { 0x9283a3b2, 46 },
    { 0xf4b3a3af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9643a3b8, 33 },
    { 0xd053a3b9, 33 },
    { 0xa9a3a3ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3a3bf, 40 },
    { 0x2353a3c0, 88 },
    { 0x3bf3a3c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13a3c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd953a3c8, 95 },
    { 0x4e13a3c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e3a3d0, 43 },
    { 0x6893a3d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3a3d7, 100 },
    { 0, 0 },
    { 0x77b3a3d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd3a3de, 100 },
    { 0x8993a3df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e3a3e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb3a3ed, 99 },
    { 0x3c43a3ee, 78 },
    { 0x3693a3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf3a3f2, 95 },
    { 0x6dd3a3f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b03a3f8, 85 },
    { 0x9ca3a3f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83a3fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a401, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a404, 87 },
    { 0, 0 },
    { 0x4143a406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73a40e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33a411, 76 },
    { 0x7733a412, 65 },
    { 0xc353a411, 61 },
    { 0x5393a414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae3a418, 95 },
    { 0xf1a3a419, 55 },
    { 0xdf13a418, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6113a41d, 23 },
    { 0x57b3a41d, 75 },
    { 0, 0 },
    { 0xf393a420, 66 },
    { 0xdea3a420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff3a429, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93a42f, 84 },
    { 0x1dc3a42f, 93 },
    { 0xb8b3a42f, 90 },
    { 0xaf13a42f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f3a436, 95 },
    { 0, 0 },
    { 0xc813a438, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2693a440, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53a444, 55 },
    { 0, 0 },
    { 0xf6a3a446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de3a44c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3a44f, 34 },
    { 0, 0 },
    { 0xea43a451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe613a454, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd3a457, 90 },
    { 0, 0 },
    { 0x60d3a459, 90 },
    { 0xe083a45a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273a462, 100 },
    { 0x5e23a463, 91 },
    { 0x91b3a463, 95 },
    { 0x6bd3a465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a3a468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b43a46c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc263a475, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f3a479, 61 },
    { 0, 0 },
    { 0x4993a47b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd3a47e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3a483, 90 },
    { 0, 0 },
    { 0xac83a485, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce3a48b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133a498, 23 },
    { 0x7893a499, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d53a49c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5183a4a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5733a4aa, 76 },
    { 0xbf13a4ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6643a4ae, 65 },
    { 0x7893a4af, 95 },
    { 0xb6e3a4b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1183a4b3, 33 },
    { 0xa1e3a4b4, 23 },
    { 0xfd33a4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9093a4b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec43a4cb, 33 },
    { 0, 0 },
    { 0x8a83a4cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133a4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd33a4d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f73a4d7, 95 },
    { 0x4e3a4d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa013a4dc, 65 },
    { 0x63a4dd, 66 },
    { 0x15d3a4dd, 33 },
    { 0xef23a4df, 85 },
    { 0xca33a4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b3a4e3, 36 },
    { 0xd443a4e4, 95 },
    { 0x8823a4e5, 90 },
    { 0xf903a4e4, 98 },
    { 0x84d3a4e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb423a4ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53a4f0, 91 },
    { 0xb153a4f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c83a4f4, 98 },
    { 0x97e3a4f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea3a4fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893a500, 95 },
    { 0, 0 },
    { 0xb6a3a502, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc473a50c, 95 },
    { 0x7c43a50d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e3a511, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593a514, 33 },
    { 0xfb83a514, 77 },
    { 0xbee3a516, 98 },
    { 0x8bd3a517, 66 },
    { 0x3c13a518, 90 },
    { 0x3733a515, 90 },
    { 0, 0 },
    { 0x7d73a51b, 33 },
    { 0, 0 },
    { 0x8a93a51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3a520, 33 },
    { 0x2293a520, 85 },
    { 0, 0 },
    { 0x4543a523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e3a527, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3363a533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca3a538, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a3a544, 61 },
    { 0xee83a544, 61 },
    { 0, 0 },
    { 0x4d53a547, 98 },
    { 0, 0 },
    { 0x11a3a549, 61 },
    { 0xcc83a54a, 76 },
    { 0x58a3a54b, 98 },
    { 0xcc43a54c, 87 },
    { 0xd703a54d, 43 },
    { 0x39a3a54a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa3a551, 90 },
    { 0x3ee3a551, 33 },
    { 0x7ef3a553, 78 },
    { 0xc793a553, 85 },
    { 0, 0 },
    { 0xf353a556, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e3a55e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6493a565, 95 },
    { 0x1c23a566, 95 },
    { 0x9193a567, 95 },
    { 0x4823a568, 33 },
    { 0xb233a566, 52 },
    { 0x663a56a, 85 },
    { 0xbd03a56b, 55 },
    { 0x3473a56c, 90 },
    { 0x1b3a568, 98 },
    { 0xf603a56a, 23 },
    { 0x5123a566, 56 },
    { 0x2e93a570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd193a574, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd743a579, 23 },
    { 0, 0 },
    { 0x5ff3a57b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba3a57e, 33 },
    { 0x1a73a57e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4533a584, 33 },
    { 0, 0 },
    { 0x6483a586, 98 },
    { 0xcba3a586, 33 },
    { 0x5863a588, 78 },
    { 0xa003a588, 98 },
    { 0x5c3a589, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf3a592, 78 },
    { 0x4523a592, 98 },
    { 0, 0 },
    { 0x3a43a595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3a599, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9593a5a0, 5 },
    { 0, 0 },
    { 0xc803a5a2, 43 },
    { 0x9983a5a2, 33 },
    { 0x4603a5a4, 90 },
    { 0x4ae3a5a2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad3a5a9, 90 },
    { 0x8ab3a5aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc3a5b2, 90 },
    { 0, 0 },
    { 0xafa3a5b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6263a5b9, 23 },
    { 0x693a5b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3a5bf, 23 },
    { 0x5af3a5c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd03a5c3, 88 },
    { 0, 0 },
    { 0xe413a5c5, 33 },
    { 0x903a5c6, 65 },
    { 0, 0 },
    { 0x2303a5c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a3a5ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663a5d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03a5d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3a5d9, 23 },
    { 0, 0 },
    { 0xbfe3a5db, 34 },
    { 0xecb3a5dc, 90 },
    { 0xba13a5dd, 80 },
    { 0, 0 },
    { 0xa743a5df, 66 },
    { 0xdc73a5e0, 98 },
    { 0xd83a5e1, 76 },
    { 0x62c3a5e2, 91 },
    { 0x64b3a5e0, 46 },
    { 0x6f3a5e4, 43 },
    { 0xa9b3a5e4, 33 },
    { 0x693a5e6, 33 },
    { 0x7033a5e6, 33 },
    { 0, 0 },
    { 0x2e93a5e9, 76 },
    { 0x93b3a5e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a3a5ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013a5f5, 69 },
    { 0xbcc3a5f5, 98 },
    { 0xa353a5f7, 61 },
    { 0, 0 },
    { 0x1933a5f9, 61 },
    { 0x8d23a5f9, 61 },
    { 0x2763a5fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f03a5ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb63a603, 98 },
    { 0, 0 },
    { 0x3b43a605, 23 },
    { 0, 0 },
    { 0x1083a607, 95 },
    { 0xb813a608, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9983a615, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a53a61b, 61 },
    { 0, 0 },
    { 0x7d13a61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4603a622, 23 },
    { 0xb013a622, 61 },
    { 0xd463a622, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e3a629, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc3a62d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3a633, 23 },
    { 0, 0 },
    { 0x6583a635, 100 },
    { 0xa4d3a636, 91 },
    { 0xacd3a635, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda53a63e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d93a646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf153a649, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9033a64e, 55 },
    { 0x8383a64e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a3a652, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b3a657, 58 },
    { 0, 0 },
    { 0xe393a659, 98 },
    { 0xa5c3a65a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a93a65f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8483a663, 98 },
    { 0, 0 },
    { 0xe303a665, 78 },
    { 0x77b3a666, 17 },
    { 0x5153a665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953a66a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3a671, 66 },
    { 0xd143a672, 34 },
    { 0x6d03a673, 61 },
    { 0x1e03a671, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf3a682, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf743a687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa63a68a, 33 },
    { 0x1763a68a, 43 },
    { 0x1613a68c, 84 },
    { 0xcb93a68c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e3a690, 95 },
    { 0xe183a690, 33 },
    { 0xcc33a692, 61 },
    { 0x3183a693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb953a697, 99 },
    { 0xc303a698, 98 },
    { 0xaf03a698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb93a6a1, 23 },
    { 0, 0 },
    { 0x8c23a6a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f3a6a6, 52 },
    { 0xdf93a6a7, 36 },
    { 0, 0 },
    { 0x2733a6a9, 61 },
    { 0, 0 },
    { 0xd1c3a6ab, 90 },
    { 0x8663a6ac, 33 },
    { 0xf8e3a6ad, 90 },
    { 0xead3a6ac, 76 },
    { 0xb903a6af, 78 },
    { 0xd023a6af, 33 },
    { 0x14a3a6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3a6b4, 46 },
    { 0x2883a6b5, 49 },
    { 0x73d3a6b4, 43 },
    { 0x5de3a6b6, 33 },
    { 0xfe03a6b5, 61 },
    { 0x7b03a6b8, 90 },
    { 0x82a3a6ba, 55 },
    { 0, 0 },
    { 0xca53a6bc, 83 },
    { 0xe453a6bd, 84 },
    { 0x8403a6bd, 66 },
    { 0x3cc3a6bf, 33 },
    { 0x94b3a6bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3a6c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e3a6c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873a6cc, 76 },
    { 0x1323a6cd, 34 },
    { 0x8453a6ce, 98 },
    { 0, 0 },
    { 0x39f3a6d0, 33 },
    { 0xf033a6d1, 98 },
    { 0, 0 },
    { 0x6343a6d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3a6d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e3a6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003a6e3, 98 },
    { 0xe063a6e4, 61 },
    { 0xdec3a6e5, 91 },
    { 0x3be3a6e6, 93 },
    { 0, 0 },
    { 0xff33a6e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9073a6ec, 33 },
    { 0x7f3a6ed, 43 },
    { 0xb933a6ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043a6f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e43a6f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a3a6fd, 98 },
    { 0, 0 },
    { 0x4963a6ff, 98 },
    { 0xe393a6ff, 61 },
    { 0xb3b3a700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053a705, 33 },
    { 0xe923a705, 76 },
    { 0x1f43a707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3a70a, 69 },
    { 0x18d3a70a, 93 },
    { 0xe4a3a70b, 98 },
    { 0x9863a70d, 61 },
    { 0, 0 },
    { 0xf063a70f, 90 },
    { 0, 0 },
    { 0x8e13a711, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3a71b, 55 },
    { 0xb953a71c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a71f, 23 },
    { 0x93a3a71f, 95 },
    { 0x5783a720, 43 },
    { 0x60e3a722, 90 },
    { 0x40a3a71f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3a726, 6 },
    { 0x7b63a727, 67 },
    { 0xfa73a728, 87 },
    { 0, 0 },
    { 0xce83a72a, 61 },
    { 0, 0 },
    { 0x55e3a72c, 61 },
    { 0xd6f3a72c, 33 },
    { 0, 0 },
    { 0x1723a72f, 76 },
    { 0xf013a72f, 46 },
    { 0x6123a730, 61 },
    { 0, 0 },
    { 0x79e3a733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243a737, 90 },
    { 0x6463a738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a73c, 90 },
    { 0x3013a73d, 52 },
    { 0xda43a73d, 94 },
    { 0x2d13a73c, 61 },
    { 0x2363a740, 93 },
    { 0xf0c3a73c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b3a744, 95 },
    { 0xea23a745, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6403a74a, 93 },
    { 0, 0 },
    { 0x2d13a74c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3a755, 61 },
    { 0xc23a756, 33 },
    { 0xacb3a757, 55 },
    { 0xdc33a755, 34 },
    { 0xeb03a759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b3a75d, 33 },
    { 0, 0 },
    { 0xf1f3a75f, 55 },
    { 0x22c3a760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9763a766, 85 },
    { 0, 0 },
    { 0xf113a768, 55 },
    { 0x9743a769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d3a773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3a777, 95 },
    { 0x4ac3a778, 68 },
    { 0, 0 },
    { 0xc153a77a, 43 },
    { 0, 0 },
    { 0x3553a77c, 23 },
    { 0, 0 },
    { 0x2cf3a77e, 90 },
    { 0, 0 },
    { 0xd93a780, 98 },
    { 0, 0 },
    { 0x3e13a782, 78 },
    { 0xdfc3a783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c83a786, 33 },
    { 0, 0 },
    { 0x7463a788, 33 },
    { 0xd423a789, 61 },
    { 0x61f3a789, 96 },
    { 0x64c3a789, 69 },
    { 0, 0 },
    { 0x7e93a78d, 95 },
    { 0, 0 },
    { 0x9f43a78f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a794, 33 },
    { 0, 0 },
    { 0xc423a796, 78 },
    { 0, 0 },
    { 0xef83a798, 33 },
    { 0xd753a799, 61 },
    { 0x41a3a799, 33 },
    { 0x3013a79b, 90 },
    { 0xfb43a79b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5973a79f, 100 },
    { 0xa373a79f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc723a7a7, 43 },
    { 0x1813a7a8, 84 },
    { 0x4e43a7a8, 87 },
    { 0x5453a7aa, 52 },
    { 0, 0 },
    { 0xa413a7ac, 90 },
    { 0x7de3a7ac, 90 },
    { 0x1083a7ad, 93 },
    { 0x3883a7af, 52 },
    { 0xac53a7ac, 52 },
    { 0xc383a7b0, 85 },
    { 0x6563a7b2, 61 },
    { 0xf8d3a7b3, 61 },
    { 0x3703a7b3, 76 },
    { 0x5b93a7b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3a7be, 84 },
    { 0, 0 },
    { 0x1093a7c0, 88 },
    { 0xfd53a7c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad3a7c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8003a7d2, 90 },
    { 0x73d3a7d2, 33 },
    { 0x8693a7d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3863a7dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc493a7df, 9 },
    { 0x66b3a7e0, 90 },
    { 0xdf03a7e1, 46 },
    { 0xff13a7e2, 77 },
    { 0xc0c3a7e2, 55 },
    { 0x2433a7e0, 98 },
    { 0xad03a7e5, 52 },
    { 0x1d23a7e6, 98 },
    { 0x80f3a7e5, 90 },
    { 0x943a7e8, 76 },
    { 0x3173a7e9, 43 },
    { 0x7e63a7e5, 90 },
    { 0x91e3a7eb, 43 },
    { 0x1363a7ea, 55 },
    { 0x2093a7e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3a7f1, 61 },
    { 0xe7e3a7f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde33a7f8, 93 },
    { 0x37b3a7f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2083a7fe, 43 },
    { 0x2993a7ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633a811, 98 },
    { 0, 0 },
    { 0x283a813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3a817, 76 },
    { 0x3c83a818, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73a81c, 33 },
    { 0x2a03a81d, 99 },
    { 0xd113a81e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3a825, 23 },
    { 0, 0 },
    { 0x8d73a827, 61 },
    { 0xc2f3a827, 76 },
    { 0x3e93a829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83a82c, 23 },
    { 0xba53a82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd903a839, 46 },
    { 0x3b63a83a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b3a841, 67 },
    { 0, 0 },
    { 0xf6e3a843, 23 },
    { 0xc6f3a844, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xac63a847, 93 },
    { 0x7af3a847, 99 },
    { 0, 0 },
    { 0x1cd3a84a, 33 },
    { 0, 0 },
    { 0xc73a84c, 100 },
    { 0xda73a84c, 33 },
    { 0xe33a84e, 61 },
    { 0xfca3a84d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313a857, 90 },
    { 0, 0 },
    { 0x75b3a859, 61 },
    { 0, 0 },
    { 0x2c93a85b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e3a85e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb83a861, 90 },
    { 0, 0 },
    { 0xaaa3a863, 95 },
    { 0, 0 },
    { 0xa573a865, 96 },
    { 0x2bf3a865, 23 },
    { 0x6693a867, 95 },
    { 0, 0 },
    { 0x2d03a869, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783a86c, 90 },
    { 0, 0 },
    { 0xb933a86e, 76 },
    { 0x1f53a86f, 36 },
    { 0x99f3a870, 98 },
    { 0xeea3a871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513a877, 23 },
    { 0x6143a878, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713a87c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f13a87f, 90 },
    { 0xed83a880, 100 },
    { 0x6583a87f, 95 },
    { 0xf643a882, 81 },
    { 0x5953a882, 85 },
    { 0xb9e3a884, 55 },
    { 0x29a3a885, 90 },
    { 0xa983a886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5963a889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8623a88d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3a891, 100 },
    { 0x65f3a892, 28 },
    { 0, 0 },
    { 0x75c3a894, 61 },
    { 0xebf3a894, 90 },
    { 0, 0 },
    { 0x9f83a897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead3a89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b3a8a1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83a8a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6213a8af, 33 },
    { 0xfa33a8b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f3a8b3, 23 },
    { 0xca43a8b4, 90 },
    { 0, 0 },
    { 0x39d3a8b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff83a8c0, 33 },
    { 0x23c3a8c1, 90 },
    { 0x9863a8c1, 61 },
    { 0xa0e3a8c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9153a8c6, 43 },
    { 0, 0 },
    { 0xf0e3a8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd3a8cc, 91 },
    { 0xb243a8cd, 81 },
    { 0xc093a8cc, 33 },
    { 0xdae3a8cf, 34 },
    { 0x2213a8cc, 17 },
    { 0xa1b3a8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c3a8db, 95 },
    { 0x1c93a8db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee3a8e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3a8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793a8ec, 85 },
    { 0, 0 },
    { 0xc433a8ee, 86 },
    { 0xbfd3a8ef, 43 },
    { 0xbf23a8f0, 98 },
    { 0xf53a8ef, 33 },
    { 0xba23a8ef, 23 },
    { 0, 0 },
    { 0x1dd3a8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23a8f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3a8fc, 61 },
    { 0, 0 },
    { 0x2c63a8fe, 80 },
    { 0x5393a8ff, 98 },
    { 0xc4d3a8fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e3a904, 90 },
    { 0x3d3a904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d3a908, 61 },
    { 0xb613a909, 80 },
    { 0x6cb3a909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053a915, 23 },
    { 0xec03a915, 23 },
    { 0x2fa3a917, 90 },
    { 0xb393a918, 94 },
    { 0xc3f3a917, 100 },
    { 0xb0a3a91a, 61 },
    { 0xa083a91b, 90 },
    { 0xa013a919, 46 },
    { 0x7173a915, 95 },
    { 0x1b33a91e, 76 },
    { 0x5a23a917, 98 },
    { 0, 0 },
    { 0x4d73a921, 90 },
    { 0, 0 },
    { 0xe653a923, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd3a926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5993a929, 95 },
    { 0xa863a92a, 78 },
    { 0xa2c3a92a, 90 },
    { 0, 0 },
    { 0x4bd3a92d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x773a930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c33a93c, 76 },
    { 0x99c3a93d, 95 },
    { 0x8ec3a93d, 98 },
    { 0xddc3a93f, 76 },
    { 0, 0 },
    { 0x7e73a941, 90 },
    { 0x9773a942, 33 },
    { 0x1c03a943, 61 },
    { 0x5fd3a943, 34 },
    { 0x2a03a945, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3a954, 95 },
    { 0xd93a954, 83 },
    { 0, 0 },
    { 0x73a3a957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a3a95d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383a963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c3a968, 76 },
    { 0x2443a969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f3a96f, 23 },
    { 0xea33a96f, 100 },
    { 0x90b3a971, 85 },
    { 0xd523a970, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7503a976, 98 },
    { 0x5153a977, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c3a97d, 61 },
    { 0x41f3a97e, 90 },
    { 0xee13a97f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb023a982, 33 },
    { 0x5da3a982, 99 },
    { 0x5253a982, 61 },
    { 0x6aa3a983, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3a98a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd973a98d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6983a995, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc933a999, 61 },
    { 0, 0 },
    { 0x1b13a99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa193a99e, 33 },
    { 0, 0 },
    { 0xba93a9a0, 55 },
    { 0x253a9a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3213a9a4, 77 },
    { 0x19c3a9a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53a9a8, 23 },
    { 0, 0 },
    { 0x25b3a9aa, 95 },
    { 0xbc53a9ab, 87 },
    { 0, 0 },
    { 0x14c3a9ad, 90 },
    { 0, 0 },
    { 0xff43a9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa613a9b2, 55 },
    { 0xc953a9b3, 17 },
    { 0x97d3a9b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f53a9b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc3a9ba, 33 },
    { 0xd83a9ba, 55 },
    { 0, 0 },
    { 0xcae3a9bd, 98 },
    { 0, 0 },
    { 0x6923a9bf, 55 },
    { 0x6b63a9c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a33a9c6, 90 },
    { 0xe7d3a9c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a13a9ce, 40 },
    { 0, 0 },
    { 0xe3c3a9d0, 33 },
    { 0x2ef3a9d0, 23 },
    { 0xb73a9d2, 61 },
    { 0, 0 },
    { 0x6c33a9d4, 55 },
    { 0x1723a9d5, 76 },
    { 0, 0 },
    { 0x7a63a9d7, 23 },
    { 0x6223a9d8, 93 },
    { 0x6f73a9d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533a9dd, 61 },
    { 0xb13a9dd, 90 },
    { 0xec63a9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23a9e3, 52 },
    { 0, 0 },
    { 0x1ed3a9e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3a9e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e3a9ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab3a9ef, 23 },
    { 0xc553a9ef, 95 },
    { 0xcd43a9f0, 85 },
    { 0x2bc3a9f0, 93 },
    { 0, 0 },
    { 0x4103a9f4, 90 },
    { 0, 0 },
    { 0xd3f3a9f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e3a9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753aa01, 36 },
    { 0x45e3aa01, 33 },
    { 0, 0 },
    { 0x48e3aa04, 33 },
    { 0xf423aa05, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7653aa09, 78 },
    { 0x6db3aa0a, 33 },
    { 0, 0 },
    { 0xa8d3aa0c, 91 },
    { 0x4ea3aa0d, 55 },
    { 0xbf63aa0e, 98 },
    { 0, 0 },
    { 0xd1e3aa10, 90 },
    { 0, 0 },
    { 0xd703aa12, 68 },
    { 0x4583aa13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83aa17, 61 },
    { 0, 0 },
    { 0x6a33aa19, 90 },
    { 0xccd3aa1a, 33 },
    { 0x4d43aa19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d63aa1f, 55 },
    { 0xd813aa20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e3aa23, 33 },
    { 0x3c33aa24, 76 },
    { 0x5433aa23, 90 },
    { 0x72a3aa26, 34 },
    { 0x8a83aa27, 61 },
    { 0x62a3aa28, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253aa31, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113aa36, 88 },
    { 0, 0 },
    { 0x24a3aa38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483aa3b, 90 },
    { 0x4503aa3c, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd3aa3f, 91 },
    { 0, 0 },
    { 0x7ca3aa41, 23 },
    { 0xf993aa41, 23 },
    { 0, 0 },
    { 0x33f3aa44, 90 },
    { 0x7493aa45, 95 },
    { 0xb9c3aa46, 33 },
    { 0xc6b3aa47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3323aa4a, 34 },
    { 0xcb03aa4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c23aa50, 55 },
    { 0x7ec3aa51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d3aa56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953aa5a, 76 },
    { 0x33a3aa5b, 90 },
    { 0, 0 },
    { 0x97c3aa5d, 93 },
    { 0xc533aa5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693aa61, 33 },
    { 0x2be3aa61, 90 },
    { 0x62d3aa63, 34 },
    { 0x3c53aa64, 65 },
    { 0xdac3aa61, 91 },
    { 0x8d33aa65, 77 },
    { 0, 0 },
    { 0x12a3aa68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853aa6b, 98 },
    { 0xdfa3aa6c, 43 },
    { 0x5a43aa6d, 77 },
    { 0x1c53aa6e, 98 },
    { 0x7373aa6f, 43 },
    { 0xc253aa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa933aa73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843aa77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3aa7d, 87 },
    { 0x3a93aa7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f23aa8c, 46 },
    { 0, 0 },
    { 0xb953aa8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3633aa93, 91 },
    { 0, 0 },
    { 0x4463aa95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f13aa99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b3aa9e, 85 },
    { 0, 0 },
    { 0x7d43aaa0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5763aaa3, 90 },
    { 0x1203aaa4, 68 },
    { 0xadb3aaa5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b3aaa9, 100 },
    { 0x8e13aaaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03aaae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8453aab2, 67 },
    { 0x323aab3, 44 },
    { 0x2673aab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f3aab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e3aaba, 61 },
    { 0x3383aaba, 33 },
    { 0xdc13aaba, 55 },
    { 0, 0 },
    { 0x2783aabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc553aac2, 55 },
    { 0, 0 },
    { 0x6383aac4, 76 },
    { 0x6113aac5, 90 },
    { 0xf143aac4, 90 },
    { 0, 0 },
    { 0xde33aac8, 42 },
    { 0, 0 },
    { 0xd0d3aaca, 78 },
    { 0x8783aacb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d3aad7, 52 },
    { 0x6083aad7, 46 },
    { 0, 0 },
    { 0x293aada, 34 },
    { 0x54b3aadb, 101 },
    { 0x63a3aadc, 34 },
    { 0, 0 },
    { 0xa0f3aade, 61 },
    { 0, 0 },
    { 0x4dc3aae0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe13aae5, 78 },
    { 0, 0 },
    { 0xa763aae7, 95 },
    { 0, 0 },
    { 0xa2b3aae9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc3aaed, 33 },
    { 0x2553aaee, 90 },
    { 0x70f3aaee, 98 },
    { 0, 0 },
    { 0x4173aaf1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb3aaf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5123aafa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f3ab01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9143ab04, 61 },
    { 0, 0 },
    { 0xd0e3ab06, 43 },
    { 0xac3ab07, 33 },
    { 0xeb43ab06, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3ab0b, 34 },
    { 0, 0 },
    { 0xf2f3ab0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3ab14, 55 },
    { 0x2d23ab15, 23 },
    { 0xbb3ab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa823ab19, 52 },
    { 0, 0 },
    { 0x6963ab1b, 64 },
    { 0x9463ab1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9413ab20, 95 },
    { 0x76b3ab21, 61 },
    { 0, 0 },
    { 0xd143ab23, 90 },
    { 0xbfc3ab24, 90 },
    { 0x75e3ab25, 78 },
    { 0x1953ab26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c3ab31, 61 },
    { 0x7973ab32, 55 },
    { 0xf13ab31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa3ab38, 55 },
    { 0xbcf3ab39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3ab3c, 55 },
    { 0, 0 },
    { 0x2d13ab3e, 61 },
    { 0x60c3ab3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543ab42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5613ab48, 93 },
    { 0, 0 },
    { 0xead3ab4a, 90 },
    { 0, 0 },
    { 0x8293ab4c, 98 },
    { 0, 0 },
    { 0xcff3ab4e, 52 },
    { 0x2dd3ab4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c3ab52, 98 },
    { 0x74d3ab53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8943ab58, 23 },
    { 0x8403ab58, 95 },
    { 0x3493ab58, 33 },
    { 0xa513ab5b, 99 },
    { 0x10d3ab5c, 23 },
    { 0x4b03ab5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc163ab65, 87 },
    { 0, 0 },
    { 0xcf23ab67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd923ab71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d3ab75, 23 },
    { 0xec23ab75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3ab79, 34 },
    { 0x9fc3ab7a, 93 },
    { 0xce53ab7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3ab7f, 46 },
    { 0x7ef3ab7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093ab85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1473ab88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f93ab8b, 61 },
    { 0, 0 },
    { 0xd393ab8d, 43 },
    { 0xd563ab8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93ab91, 90 },
    { 0x8dc3ab91, 23 },
    { 0xded3ab93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173ab98, 61 },
    { 0, 0 },
    { 0x8153ab9a, 33 },
    { 0xfa13ab9b, 88 },
    { 0xc8d3ab9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9913ab9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa993aba2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53aba9, 93 },
    { 0xb03abaa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633abad, 23 },
    { 0xeea3abad, 99 },
    { 0x66d3abae, 34 },
    { 0xcf93abb0, 61 },
    { 0xdf63abb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3abb6, 76 },
    { 0x68f3abb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243abbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e3abbf, 90 },
    { 0x9483abc0, 90 },
    { 0x35c3abc0, 81 },
    { 0xf3a3abc1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3abc5, 95 },
    { 0, 0 },
    { 0x1293abc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a63abcc, 95 },
    { 0xc0f3abcc, 61 },
    { 0, 0 },
    { 0xdde3abcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a3abd3, 33 },
    { 0x8f03abd3, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff3abd7, 95 },
    { 0x3113abd7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a3abdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513abe0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c53abe8, 52 },
    { 0x9603abe9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443abee, 88 },
    { 0x6c93abee, 85 },
    { 0, 0 },
    { 0xd313abf1, 61 },
    { 0x7803abf1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c53abf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293abfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33ac00, 23 },
    { 0x9503ac01, 23 },
    { 0x95b3ac00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea43ac06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe543ac0c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3943ac13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3ac16, 90 },
    { 0, 0 },
    { 0x61f3ac18, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53ac1d, 61 },
    { 0x78f3ac1e, 90 },
    { 0x1903ac1f, 99 },
    { 0x1cd3ac1e, 90 },
    { 0, 0 },
    { 0x5173ac22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f3ac26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3ac29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263ac2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3ac33, 61 },
    { 0x2283ac34, 33 },
    { 0, 0 },
    { 0x81a3ac36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3ac39, 34 },
    { 0xe2c3ac3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3ac3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e3ac41, 33 },
    { 0x4843ac41, 34 },
    { 0xc2f3ac41, 98 },
    { 0, 0 },
    { 0x3303ac45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf13ac4b, 78 },
    { 0, 0 },
    { 0xc9f3ac4d, 99 },
    { 0x7563ac4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6933ac51, 76 },
    { 0xf0c3ac52, 33 },
    { 0x38f3ac53, 95 },
    { 0xd0c3ac52, 55 },
    { 0x413ac55, 53 },
    { 0x43c3ac52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab3ac59, 95 },
    { 0xb8a3ac5a, 96 },
    { 0x1f23ac5b, 68 },
    { 0xcc03ac5b, 76 },
    { 0x7853ac5d, 46 },
    { 0x2aa3ac5d, 80 },
    { 0x7873ac5d, 91 },
    { 0x4a03ac5e, 98 },
    { 0x613ac5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83ac64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f73ac67, 33 },
    { 0x4373ac67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d43ac6d, 95 },
    { 0xe493ac6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca3ac71, 23 },
    { 0x9283ac71, 61 },
    { 0x6d93ac71, 77 },
    { 0x9163ac74, 33 },
    { 0x7cc3ac75, 17 },
    { 0xf4a3ac75, 90 },
    { 0x2853ac72, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf3ac84, 98 },
    { 0, 0 },
    { 0x3413ac86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c3ac8c, 85 },
    { 0xb323ac8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23ac95, 84 },
    { 0, 0 },
    { 0x3f63ac97, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3ac9a, 78 },
    { 0, 0 },
    { 0xd503ac9c, 81 },
    { 0x703ac9c, 95 },
    { 0x5c53ac9e, 34 },
    { 0x32c3ac9f, 95 },
    { 0xe273aca0, 36 },
    { 0xf593aca0, 34 },
    { 0xbbd3aca2, 67 },
    { 0xabc3aca1, 55 },
    { 0x1873aca4, 33 },
    { 0xbc03aca4, 98 },
    { 0x9d83aca5, 55 },
    { 0xc093aca7, 43 },
    { 0x76f3aca8, 99 },
    { 0x2893aca9, 61 },
    { 0xa313aca9, 61 },
    { 0xfeb3acab, 99 },
    { 0x65a3acab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd573acb3, 90 },
    { 0, 0 },
    { 0x6023acb5, 23 },
    { 0xa2e3acb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3acbc, 61 },
    { 0xd113acbd, 61 },
    { 0x4753acbe, 67 },
    { 0x8fd3acbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3acc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc3acc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3983acc8, 85 },
    { 0xb033acc8, 33 },
    { 0x9f33acc8, 61 },
    { 0xe1b3acc8, 90 },
    { 0, 0 },
    { 0x5503accd, 61 },
    { 0x9ec3accd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f3acd1, 55 },
    { 0xcf13acd1, 23 },
    { 0, 0 },
    { 0xb9e3acd4, 93 },
    { 0x1ab3acd4, 98 },
    { 0xa4c3acd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3acda, 98 },
    { 0x4c03acdb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c3ace3, 33 },
    { 0, 0 },
    { 0xcb93ace5, 33 },
    { 0, 0 },
    { 0xece3ace7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc613aced, 90 },
    { 0x9273acee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93acf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3acfa, 38 },
    { 0x5353acfa, 34 },
    { 0x7a13acfb, 23 },
    { 0x4263acfd, 78 },
    { 0xbc63acfe, 67 },
    { 0x4a3acff, 55 },
    { 0x1f43acfa, 61 },
    { 0, 0 },
    { 0xb743ad02, 46 },
    { 0xe283ad03, 67 },
    { 0, 0 },
    { 0x1843ad05, 98 },
    { 0, 0 },
    { 0x4cb3ad07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee3ad0b, 98 },
    { 0, 0 },
    { 0x17b3ad0d, 34 },
    { 0xede3ad0d, 46 },
    { 0xa433ad0f, 33 },
    { 0x2e03ad0d, 76 },
    { 0x5243ad10, 93 },
    { 0x91b3ad12, 33 },
    { 0x5563ad0e, 87 },
    { 0, 0 },
    { 0xae93ad15, 61 },
    { 0x3e13ad16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d3ad19, 76 },
    { 0x50a3ad19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf013ad1d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963ad20, 43 },
    { 0x7bc3ad21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b93ad25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2263ad2c, 85 },
    { 0x8603ad2c, 90 },
    { 0x7553ad2e, 23 },
    { 0x4773ad2c, 100 },
    { 0, 0 },
    { 0x3d63ad31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e3ad36, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef3ad3a, 43 },
    { 0x9573ad3b, 93 },
    { 0xebb3ad3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493ad40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3ad46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed3ad4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3ad50, 76 },
    { 0x8fa3ad51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2963ad55, 93 },
    { 0x6ce3ad55, 76 },
    { 0xea23ad57, 85 },
    { 0x4003ad57, 76 },
    { 0x30e3ad59, 98 },
    { 0x19e3ad59, 90 },
    { 0, 0 },
    { 0xff13ad5c, 76 },
    { 0xd693ad5d, 43 },
    { 0xfeb3ad5d, 43 },
    { 0xb133ad5f, 90 },
    { 0x31d3ad5d, 76 },
    { 0, 0 },
    { 0xaf63ad62, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa143ad66, 99 },
    { 0x563ad67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33ad6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd403ad6d, 98 },
    { 0x97c3ad6e, 95 },
    { 0xee53ad6f, 90 },
    { 0xddb3ad6f, 23 },
    { 0xb083ad71, 95 },
    { 0x18d3ad6f, 23 },
    { 0x4f23ad73, 33 },
    { 0xd483ad74, 75 },
    { 0x76e3ad6e, 61 },
    { 0x8553ad76, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a3ad7b, 90 },
    { 0, 0 },
    { 0x82a3ad7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7303ad81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9373ad85, 95 },
    { 0xc7f3ad86, 23 },
    { 0x7d03ad87, 52 },
    { 0xd7f3ad86, 61 },
    { 0x9813ad86, 84 },
    { 0xbcc3ad8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b93ad91, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c3ad94, 33 },
    { 0xe6a3ad95, 91 },
    { 0xb5b3ad95, 84 },
    { 0xcd83ad95, 90 },
    { 0xb83ad98, 99 },
    { 0x4633ad98, 94 },
    { 0x24b3ad98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a13ada4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa3adac, 34 },
    { 0x3ca3adad, 33 },
    { 0x103adad, 93 },
    { 0xcdb3adaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b3adb3, 98 },
    { 0xe703adb4, 76 },
    { 0, 0 },
    { 0x51d3adb6, 99 },
    { 0x1103adb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743adbd, 88 },
    { 0x42b3adbd, 91 },
    { 0, 0 },
    { 0x3c83adc0, 83 },
    { 0xef93adc1, 52 },
    { 0x74e3adc2, 33 },
    { 0xc8b3adc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe273adc7, 80 },
    { 0x2093adc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf93adcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb3add1, 95 },
    { 0x63c3add2, 61 },
    { 0xced3add3, 61 },
    { 0, 0 },
    { 0x1f83add5, 61 },
    { 0, 0 },
    { 0x9673add7, 33 },
    { 0xb403add8, 33 },
    { 0, 0 },
    { 0xac93adda, 98 },
    { 0xb393addb, 61 },
    { 0xd953addb, 33 },
    { 0xc023addb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe113ade1, 46 },
    { 0xb3e3ade2, 95 },
    { 0, 0 },
    { 0x3a33ade4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a23ade7, 23 },
    { 0x86b3ade8, 61 },
    { 0, 0 },
    { 0x6963adea, 90 },
    { 0xb5b3adeb, 98 },
    { 0, 0 },
    { 0xc9b3aded, 23 },
    { 0, 0 },
    { 0xc383adef, 23 },
    { 0, 0 },
    { 0x3933adf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa863adfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f3adff, 34 },
    { 0x32f3ae00, 61 },
    { 0x9fc3ae01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb573ae05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d63ae08, 84 },
    { 0x13e3ae09, 46 },
    { 0x2823ae08, 34 },
    { 0x8b83ae09, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff33ae15, 34 },
    { 0, 0 },
    { 0x44a3ae17, 61 },
    { 0, 0 },
    { 0x12a3ae19, 61 },
    { 0xafc3ae1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1663ae1d, 98 },
    { 0xed23ae1e, 64 },
    { 0x4003ae1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b83ae22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa453ae26, 95 },
    { 0xc373ae27, 12 },
    { 0xf063ae27, 76 },
    { 0xa93ae26, 87 },
    { 0xba13ae2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5123ae2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2583ae34, 55 },
    { 0xbad3ae34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603ae38, 52 },
    { 0xaac3ae38, 85 },
    { 0, 0 },
    { 0xae23ae3b, 23 },
    { 0xa6e3ae3c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd443ae42, 46 },
    { 0x3093ae43, 95 },
    { 0, 0 },
    { 0xe833ae45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b3ae4b, 33 },
    { 0x4423ae4c, 34 },
    { 0xf273ae4d, 93 },
    { 0x4333ae4c, 61 },
    { 0xd383ae4e, 43 },
    { 0x8f3ae50, 68 },
    { 0xa633ae51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc3ae57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873ae5b, 61 },
    { 0, 0 },
    { 0x4a3ae5d, 52 },
    { 0xe8d3ae5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a3ae62, 76 },
    { 0x9813ae62, 53 },
    { 0x6343ae62, 33 },
    { 0x5393ae62, 90 },
    { 0xef73ae65, 66 },
    { 0x2f03ae67, 100 },
    { 0x2633ae68, 87 },
    { 0x1af3ae69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f3ae6f, 34 },
    { 0, 0 },
    { 0x3803ae71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe43ae74, 68 },
    { 0x29d3ae74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c3ae78, 33 },
    { 0xfb83ae78, 61 },
    { 0, 0 },
    { 0x4fb3ae7b, 76 },
    { 0x8163ae7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff73ae80, 90 },
    { 0x34b3ae81, 95 },
    { 0x8fb3ae82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb033ae89, 33 },
    { 0x5623ae8a, 23 },
    { 0x973ae8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103ae8f, 61 },
    { 0xcc3ae90, 95 },
    { 0x7b33ae91, 33 },
    { 0x85b3ae92, 82 },
    { 0x40d3ae93, 33 },
    { 0, 0 },
    { 0x2013ae95, 36 },
    { 0, 0 },
    { 0x9723ae97, 95 },
    { 0, 0 },
    { 0x8b43ae99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e33ae9f, 33 },
    { 0, 0 },
    { 0xc3aea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1823aea6, 61 },
    { 0x2d93aea7, 78 },
    { 0xce3aea8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab3aeaf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d03aeb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf283aebd, 33 },
    { 0, 0 },
    { 0x57c3aebf, 90 },
    { 0, 0 },
    { 0x263aec1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c13aec5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f3aecb, 23 },
    { 0xd5a3aecb, 98 },
    { 0x4803aecd, 75 },
    { 0xaf63aece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2663aed2, 55 },
    { 0, 0 },
    { 0x8f83aed4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba83aed8, 61 },
    { 0x2093aed9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f3aedf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b33aee2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e3aee6, 65 },
    { 0x853aee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6573aeeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e3aef3, 17 },
    { 0xe223aef3, 95 },
    { 0x7e03aef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c3aefc, 33 },
    { 0x8dd3aefd, 90 },
    { 0xa4e3aefd, 66 },
    { 0xeb83aeff, 85 },
    { 0xc893af00, 69 },
    { 0xce93af01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03af06, 43 },
    { 0xdbb3af07, 43 },
    { 0x4383af07, 97 },
    { 0xd3b3af09, 33 },
    { 0x53f3af0a, 33 },
    { 0, 0 },
    { 0x52a3af0c, 95 },
    { 0x7cf3af0d, 61 },
    { 0, 0 },
    { 0x8153af0f, 43 },
    { 0xd383af10, 98 },
    { 0x2103af11, 100 },
    { 0xe73af12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f83af1d, 91 },
    { 0x52d3af1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9913af21, 61 },
    { 0x3dd3af21, 93 },
    { 0x4c23af23, 36 },
    { 0, 0 },
    { 0x3e93af25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e3af28, 34 },
    { 0xf563af29, 90 },
    { 0x3433af2a, 75 },
    { 0, 0 },
    { 0x3b53af2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7293af31, 100 },
    { 0, 0 },
    { 0x4063af33, 75 },
    { 0x9353af34, 61 },
    { 0x1263af33, 61 },
    { 0xe653af36, 87 },
    { 0x7be3af37, 90 },
    { 0xbb63af38, 34 },
    { 0x58e3af36, 61 },
    { 0x8723af3a, 77 },
    { 0, 0 },
    { 0x5703af3c, 98 },
    { 0x8f53af3d, 95 },
    { 0xbfa3af3e, 90 },
    { 0, 0 },
    { 0x9523af40, 98 },
    { 0x4d93af41, 67 },
    { 0xb9b3af41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f73af45, 23 },
    { 0x9933af46, 98 },
    { 0, 0 },
    { 0x6b03af48, 76 },
    { 0x54f3af49, 23 },
    { 0x8c63af48, 80 },
    { 0x8893af4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593af4e, 90 },
    { 0xc8c3af4e, 95 },
    { 0x6c73af4f, 33 },
    { 0x6953af51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603af54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d3af59, 95 },
    { 0xd873af5a, 96 },
    { 0, 0 },
    { 0x82b3af5c, 23 },
    { 0x7fa3af5d, 84 },
    { 0xa803af5e, 67 },
    { 0x34e3af5f, 23 },
    { 0xefa3af60, 76 },
    { 0xc7f3af61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f3af66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c3af69, 95 },
    { 0x1953af69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd73af6f, 43 },
    { 0xd953af70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc573af73, 23 },
    { 0, 0 },
    { 0x6823af75, 55 },
    { 0, 0 },
    { 0xf3c3af77, 55 },
    { 0x4143af78, 95 },
    { 0x1143af77, 98 },
    { 0, 0 },
    { 0x783af7b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3af80, 90 },
    { 0xccf3af81, 66 },
    { 0x45f3af82, 52 },
    { 0x1593af80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3af88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf083af98, 32 },
    { 0x4ec3af98, 23 },
    { 0x6ff3af9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3af9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7073afa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf503afac, 76 },
    { 0xb9b3afad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9193afb0, 77 },
    { 0, 0 },
    { 0xa0f3afb2, 98 },
    { 0, 0 },
    { 0x20e3afb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3afb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3603afbb, 95 },
    { 0, 0 },
    { 0x8663afbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3afc3, 55 },
    { 0, 0 },
    { 0xde83afc5, 33 },
    { 0, 0 },
    { 0xd763afc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073afcc, 33 },
    { 0, 0 },
    { 0xb4d3afce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113afd1, 90 },
    { 0x6bc3afd1, 61 },
    { 0, 0 },
    { 0x2263afd4, 91 },
    { 0xa433afd5, 90 },
    { 0x4023afd6, 33 },
    { 0x4483afd7, 61 },
    { 0xa5d3afd8, 61 },
    { 0x46d3afd4, 23 },
    { 0x1f23afda, 68 },
    { 0, 0 },
    { 0xdfa3afdc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5133afdf, 23 },
    { 0, 0 },
    { 0x9123afe1, 79 },
    { 0x19a3afe2, 43 },
    { 0xc2b3afe3, 23 },
    { 0x5823afe4, 34 },
    { 0xc8a3afe1, 98 },
    { 0xab33afe6, 90 },
    { 0x69a3afe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2803afea, 33 },
    { 0, 0 },
    { 0x5ab3afec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83afef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3aff3, 95 },
    { 0xf273aff3, 61 },
    { 0x6213aff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e3aff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f3affb, 62 },
    { 0, 0 },
    { 0xc2d3affd, 23 },
    { 0, 0 },
    { 0xac53afff, 43 },
    { 0, 0 },
    { 0x3e73b001, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893b005, 34 },
    { 0x14a3b006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063b009, 34 },
    { 0xf5a3b00a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123b00d, 78 },
    { 0xcca3b00d, 34 },
    { 0xde13b00d, 61 },
    { 0xd293b010, 23 },
    { 0x9463b00f, 34 },
    { 0xf0a3b011, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d3b017, 43 },
    { 0x1d73b018, 90 },
    { 0x15a3b019, 23 },
    { 0x19e3b017, 90 },
    { 0, 0 },
    { 0xa423b01c, 95 },
    { 0x2a13b01d, 61 },
    { 0x533b01e, 61 },
    { 0xb783b01f, 61 },
    { 0x23b020, 93 },
    { 0x6243b021, 90 },
    { 0x27b3b01e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f3b025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363b02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e53b02f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb23b034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9703b038, 90 },
    { 0, 0 },
    { 0x7173b03a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef03b040, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e3b044, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b04a, 33 },
    { 0, 0 },
    { 0x8b73b04c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9773b052, 43 },
    { 0, 0 },
    { 0x63a3b054, 23 },
    { 0, 0 },
    { 0x4be3b056, 68 },
    { 0x2173b057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223b05e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc353b062, 33 },
    { 0x43e3b063, 90 },
    { 0, 0 },
    { 0x703b065, 91 },
    { 0, 0 },
    { 0xae3b067, 61 },
    { 0x4513b068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd713b070, 38 },
    { 0x1d33b071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613b075, 90 },
    { 0x8d03b076, 95 },
    { 0xfb53b075, 98 },
    { 0xef3b078, 84 },
    { 0xbbd3b078, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc23b07c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8853b082, 68 },
    { 0x9e53b083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc253b08e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e43b093, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f43b098, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c3b09d, 55 },
    { 0, 0 },
    { 0xcf53b09f, 99 },
    { 0x3ba3b0a0, 33 },
    { 0x11f3b0a0, 46 },
    { 0x1843b0a2, 43 },
    { 0x2133b0a3, 76 },
    { 0, 0 },
    { 0x1a3b0a5, 23 },
    { 0xea83b0a6, 90 },
    { 0xb853b0a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903b0ab, 85 },
    { 0x8893b0ab, 98 },
    { 0x443b0ad, 33 },
    { 0x9cc3b0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b0b1, 95 },
    { 0x20e3b0b2, 90 },
    { 0x69f3b0b1, 78 },
    { 0x37e3b0b1, 91 },
    { 0xb493b0b5, 90 },
    { 0x6073b0b4, 55 },
    { 0xc053b0b6, 34 },
    { 0xc123b0b5, 67 },
    { 0xa6d3b0b5, 33 },
    { 0x5003b0ba, 95 },
    { 0xfd83b0bb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33b0c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3b0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa463b0cb, 52 },
    { 0xfa13b0cc, 76 },
    { 0, 0 },
    { 0x22b3b0ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b0d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3b0d4, 46 },
    { 0x3a93b0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253b0da, 61 },
    { 0x6863b0db, 96 },
    { 0, 0 },
    { 0xa433b0dd, 33 },
    { 0, 0 },
    { 0xc383b0df, 99 },
    { 0x4dd3b0e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1073b0e4, 36 },
    { 0xd783b0e5, 61 },
    { 0xcaa3b0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d3b0ed, 85 },
    { 0x82a3b0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793b0f1, 43 },
    { 0x8f13b0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823b0f8, 98 },
    { 0, 0 },
    { 0x6e33b0fa, 67 },
    { 0x4c43b0fb, 90 },
    { 0, 0 },
    { 0xc343b0fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653b100, 95 },
    { 0x9ad3b101, 85 },
    { 0xa2f3b102, 98 },
    { 0x1c43b103, 43 },
    { 0x5d73b104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf53b109, 33 },
    { 0xad63b10a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f73b10d, 77 },
    { 0x6303b10e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5773b118, 90 },
    { 0xbaa3b118, 100 },
    { 0x6143b11a, 98 },
    { 0x53e3b11a, 23 },
    { 0x5db3b11a, 85 },
    { 0x5463b11d, 93 },
    { 0x4533b11e, 90 },
    { 0x2383b11c, 55 },
    { 0, 0 },
    { 0x47e3b121, 83 },
    { 0x3623b122, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede3b126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa433b129, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363b12c, 46 },
    { 0, 0 },
    { 0xe433b12e, 33 },
    { 0x8f13b12f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b3b135, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b139, 95 },
    { 0, 0 },
    { 0xe0e3b13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b13e, 67 },
    { 0x5cb3b13e, 33 },
    { 0x3a93b140, 34 },
    { 0x3dc3b141, 95 },
    { 0x4f93b140, 99 },
    { 0, 0 },
    { 0x4a03b144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2103b14b, 84 },
    { 0, 0 },
    { 0xe943b14d, 98 },
    { 0x49b3b14d, 90 },
    { 0xb983b14e, 55 },
    { 0x80d3b150, 88 },
    { 0x7f83b150, 61 },
    { 0x4b23b152, 34 },
    { 0x6923b152, 33 },
    { 0xaa33b152, 33 },
    { 0x7293b152, 95 },
    { 0, 0 },
    { 0x7113b157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983b15a, 67 },
    { 0xb8d3b15b, 98 },
    { 0x8283b15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b165, 91 },
    { 0, 0 },
    { 0xbf13b167, 90 },
    { 0xb293b167, 88 },
    { 0xcc63b169, 55 },
    { 0x4f43b16a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa73b16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7103b174, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa63b178, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c33b17d, 61 },
    { 0xbe33b17e, 90 },
    { 0x7103b17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243b185, 23 },
    { 0xb823b186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd63b18a, 90 },
    { 0, 0 },
    { 0x67c3b18c, 98 },
    { 0x5f43b18d, 61 },
    { 0x7f03b18e, 52 },
    { 0xa5b3b18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4323b193, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b73b1a0, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd73b1a4, 85 },
    { 0, 0 },
    { 0x2e23b1a6, 90 },
    { 0x1233b1a7, 33 },
    { 0xb1b3b1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e3b1ab, 66 },
    { 0, 0 },
    { 0x84d3b1ad, 93 },
    { 0, 0 },
    { 0xc803b1af, 98 },
    { 0, 0 },
    { 0x69c3b1b1, 95 },
    { 0xa833b1b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee3b1b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3b1b9, 87 },
    { 0x2c43b1ba, 23 },
    { 0xb13b1b9, 93 },
    { 0xb2e3b1ba, 34 },
    { 0x80b3b1bb, 15 },
    { 0xc993b1be, 90 },
    { 0xeb13b1bf, 61 },
    { 0xc903b1bf, 66 },
    { 0xa3b1c1, 85 },
    { 0x933b1c2, 95 },
    { 0, 0 },
    { 0xe773b1c4, 95 },
    { 0x9823b1c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc793b1cd, 52 },
    { 0x5973b1cd, 46 },
    { 0x6ca3b1cf, 33 },
    { 0, 0 },
    { 0xbff3b1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033b1d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e3b1da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b3b1df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3b1e2, 88 },
    { 0x1c53b1e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa573b1e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533b1ea, 49 },
    { 0, 0 },
    { 0x9fc3b1ec, 93 },
    { 0, 0 },
    { 0x7a33b1ee, 95 },
    { 0x5973b1ef, 90 },
    { 0x9c03b1f0, 77 },
    { 0, 0 },
    { 0xce93b1f2, 90 },
    { 0x1d73b1f2, 23 },
    { 0x9a13b1f2, 33 },
    { 0, 0 },
    { 0xa933b1f6, 95 },
    { 0x4f83b1f6, 90 },
    { 0xf1c3b1f8, 95 },
    { 0, 0 },
    { 0xef93b1fa, 99 },
    { 0x8683b1fa, 23 },
    { 0x9d63b1fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb33b1ff, 33 },
    { 0, 0 },
    { 0xc6d3b201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc3b204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf723b207, 88 },
    { 0xef73b208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce3b20b, 90 },
    { 0x9993b20b, 85 },
    { 0x2be3b20b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b23b213, 61 },
    { 0x8113b214, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b3b218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473b21d, 61 },
    { 0x1f23b21e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643b224, 33 },
    { 0, 0 },
    { 0x8673b226, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43b22f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd03b23b, 33 },
    { 0xeaa3b23c, 23 },
    { 0xa7a3b23d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb3b241, 76 },
    { 0xe4a3b241, 90 },
    { 0, 0 },
    { 0x33b244, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3b249, 55 },
    { 0, 0 },
    { 0xc373b24b, 43 },
    { 0x7863b24c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3b24f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3b255, 61 },
    { 0xa383b256, 90 },
    { 0x2be3b257, 33 },
    { 0x6123b255, 55 },
    { 0xdb03b256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3b25d, 23 },
    { 0xd703b25e, 23 },
    { 0xbd23b25f, 46 },
    { 0xa833b260, 95 },
    { 0x8043b261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d3b264, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e3b267, 98 },
    { 0x1b23b267, 93 },
    { 0x96d3b269, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced3b270, 41 },
    { 0xa883b270, 95 },
    { 0x2cc3b271, 23 },
    { 0x2bf3b273, 90 },
    { 0x6df3b270, 96 },
    { 0, 0 },
    { 0xb473b276, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff3b27b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa843b283, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9293b287, 33 },
    { 0, 0 },
    { 0x5403b289, 43 },
    { 0, 0 },
    { 0x1973b28b, 33 },
    { 0, 0 },
    { 0xfc43b28d, 90 },
    { 0, 0 },
    { 0x1353b28f, 90 },
    { 0, 0 },
    { 0x1573b291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103b295, 23 },
    { 0x4283b296, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad83b29a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7473b29f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53b2a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca3b2a7, 33 },
    { 0, 0 },
    { 0x3173b2a9, 82 },
    { 0, 0 },
    { 0xd1d3b2ab, 90 },
    { 0, 0 },
    { 0x6093b2ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa553b2b1, 98 },
    { 0x7753b2b2, 33 },
    { 0x9f03b2b3, 90 },
    { 0xb223b2b3, 66 },
    { 0, 0 },
    { 0x35b3b2b6, 99 },
    { 0x4b63b2b7, 87 },
    { 0x9ef3b2b8, 43 },
    { 0x3223b2b8, 66 },
    { 0, 0 },
    { 0x9603b2bb, 90 },
    { 0, 0 },
    { 0xc343b2bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103b2c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3b2c3, 88 },
    { 0x35a3b2c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c3b2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc3b2d3, 90 },
    { 0xf193b2d4, 66 },
    { 0x3043b2d5, 56 },
    { 0, 0 },
    { 0x8de3b2d7, 33 },
    { 0x13d3b2d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe273b2e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e93b2ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb683b2ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd3b2f2, 81 },
    { 0x12e3b2f2, 56 },
    { 0x1123b2f4, 36 },
    { 0, 0 },
    { 0xe3b3b2f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533b300, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53b308, 33 },
    { 0xea33b309, 91 },
    { 0x6253b309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a3b30e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef63b311, 96 },
    { 0, 0 },
    { 0xbf33b313, 23 },
    { 0x7d23b314, 33 },
    { 0, 0 },
    { 0xf913b316, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5613b31b, 34 },
    { 0, 0 },
    { 0xdf3b31d, 46 },
    { 0x97e3b31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda3b322, 99 },
    { 0, 0 },
    { 0xfe13b324, 93 },
    { 0, 0 },
    { 0xc733b326, 33 },
    { 0x8e23b327, 55 },
    { 0x49f3b327, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf3b32c, 33 },
    { 0xac93b32d, 77 },
    { 0x59f3b32e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9853b336, 90 },
    { 0x6573b337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed3b33a, 78 },
    { 0, 0 },
    { 0x7df3b33c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053b33f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843b343, 85 },
    { 0x7043b344, 98 },
    { 0x793b345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33b349, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b3b34c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3b34f, 61 },
    { 0x7d53b350, 90 },
    { 0xb563b34f, 34 },
    { 0, 0 },
    { 0xb4e3b353, 98 },
    { 0xf163b353, 98 },
    { 0xfb13b355, 90 },
    { 0, 0 },
    { 0x4783b357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b361, 61 },
    { 0x5003b361, 61 },
    { 0, 0 },
    { 0xc683b364, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b368, 55 },
    { 0, 0 },
    { 0x2bc3b36a, 98 },
    { 0xc683b36b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b36e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3b374, 34 },
    { 0xfda3b374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7873b37b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3b381, 81 },
    { 0x4353b381, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43b385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553b38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a3b391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb3b394, 62 },
    { 0xbbb3b395, 98 },
    { 0, 0 },
    { 0x613b397, 37 },
    { 0xeeb3b398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef3b39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3b39e, 23 },
    { 0x2323b39f, 76 },
    { 0x33f3b39f, 37 },
    { 0x3253b3a1, 66 },
    { 0xb63b39f, 34 },
    { 0xa163b3a3, 33 },
    { 0xcb3b3a4, 100 },
    { 0xc0b3b3a3, 61 },
    { 0x8233b3a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3b3ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523b3b5, 61 },
    { 0x43b3b6, 56 },
    { 0x1673b3b7, 55 },
    { 0x7c23b3b7, 84 },
    { 0x5103b3b9, 65 },
    { 0x87a3b3ba, 66 },
    { 0, 0 },
    { 0xf063b3bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93b3bf, 90 },
    { 0, 0 },
    { 0x24d3b3c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273b3c5, 95 },
    { 0, 0 },
    { 0x5643b3c7, 95 },
    { 0, 0 },
    { 0xfa53b3c9, 33 },
    { 0, 0 },
    { 0x5da3b3cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d03b3cf, 93 },
    { 0x90a3b3cf, 99 },
    { 0, 0 },
    { 0xa03b3d2, 52 },
    { 0x57f3b3d3, 78 },
    { 0xc7d3b3d4, 33 },
    { 0x2e13b3d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad63b3db, 90 },
    { 0x5fd3b3dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec53b3df, 23 },
    { 0xe3f3b3e0, 68 },
    { 0xa163b3e0, 46 },
    { 0, 0 },
    { 0x4093b3e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f83b3e6, 61 },
    { 0, 0 },
    { 0xb673b3e8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f3b3ef, 90 },
    { 0xc03b3f0, 95 },
    { 0x9f3b3f0, 75 },
    { 0xc4a3b3f2, 55 },
    { 0x8363b3f3, 77 },
    { 0xaf33b3f4, 93 },
    { 0x8eb3b3f4, 90 },
    { 0xb593b3f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9313b3f9, 33 },
    { 0x8c33b3f9, 33 },
    { 0, 0 },
    { 0x1033b3fc, 57 },
    { 0x9303b3fd, 33 },
    { 0x3b93b3fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b402, 76 },
    { 0xa8d3b403, 76 },
    { 0, 0 },
    { 0x9563b405, 33 },
    { 0, 0 },
    { 0x4b63b407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b40a, 98 },
    { 0x58c3b40b, 43 },
    { 0, 0 },
    { 0xa8d3b40d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3b41c, 98 },
    { 0x93a3b41c, 90 },
    { 0x1da3b41e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad3b422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f3b426, 33 },
    { 0x3e23b427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e3b42e, 27 },
    { 0, 0 },
    { 0xc843b430, 23 },
    { 0x2b23b431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf723b437, 67 },
    { 0x72b3b437, 90 },
    { 0, 0 },
    { 0x7f83b43a, 98 },
    { 0xb323b43b, 78 },
    { 0x19f3b43b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xa353b43f, 90 },
    { 0, 0 },
    { 0xef23b441, 76 },
    { 0xb393b442, 90 },
    { 0xd763b443, 86 },
    { 0, 0 },
    { 0xe773b445, 87 },
    { 0x5143b446, 91 },
    { 0x9dc3b446, 61 },
    { 0xd8f3b446, 85 },
    { 0xcc23b446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13b44c, 76 },
    { 0x4793b44c, 33 },
    { 0x3c73b44e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443b452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5273b456, 90 },
    { 0x21a3b457, 90 },
    { 0xf0e3b457, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d3b45c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3b467, 83 },
    { 0xc803b468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3b46b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e3b471, 91 },
    { 0x5733b472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3b475, 23 },
    { 0xded3b476, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d63b47a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b3b47e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc353b481, 90 },
    { 0, 0 },
    { 0xd333b483, 76 },
    { 0, 0 },
    { 0xf603b485, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753b488, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf083b48d, 33 },
    { 0x8ab3b48d, 90 },
    { 0x2283b48e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb3b493, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb3b498, 52 },
    { 0xc0f3b499, 95 },
    { 0xc2c3b49a, 90 },
    { 0, 0 },
    { 0x6343b49c, 91 },
    { 0xd123b49c, 33 },
    { 0x5833b49e, 32 },
    { 0x35f3b49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763b4a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543b4b1, 61 },
    { 0x6953b4b2, 55 },
    { 0, 0 },
    { 0x3353b4b4, 98 },
    { 0xc923b4b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e3b4b8, 90 },
    { 0x2e13b4b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab33b4bc, 76 },
    { 0, 0 },
    { 0x493b4be, 33 },
    { 0xad23b4bf, 67 },
    { 0x53c3b4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8383b4c3, 33 },
    { 0x3dd3b4c4, 95 },
    { 0, 0 },
    { 0x7d93b4c6, 23 },
    { 0xa233b4c7, 98 },
    { 0x1c93b4c6, 98 },
    { 0x3b83b4c6, 23 },
    { 0x7b03b4c9, 61 },
    { 0xcb23b4cb, 76 },
    { 0x1513b4cc, 61 },
    { 0xe053b4cc, 87 },
    { 0x57d3b4ce, 33 },
    { 0x2103b4c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1343b4d2, 73 },
    { 0xccc3b4d2, 33 },
    { 0xef53b4d4, 23 },
    { 0, 0 },
    { 0x46f3b4d6, 76 },
    { 0, 0 },
    { 0xac03b4d8, 76 },
    { 0, 0 },
    { 0xdc03b4da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3b4dd, 61 },
    { 0x82a3b4de, 61 },
    { 0x5e53b4df, 76 },
    { 0x9d33b4e0, 95 },
    { 0x33b3b4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753b4e4, 61 },
    { 0x3783b4e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b3b4ee, 36 },
    { 0x70f3b4ee, 34 },
    { 0xe803b4f0, 40 },
    { 0, 0 },
    { 0xf203b4f2, 90 },
    { 0, 0 },
    { 0x3053b4f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1273b4f7, 99 },
    { 0x7a03b4f8, 23 },
    { 0x4a3b4f7, 55 },
    { 0, 0 },
    { 0x83d3b4fb, 23 },
    { 0x3f53b4fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e3b4ff, 55 },
    { 0x2f83b4ff, 52 },
    { 0x2333b501, 95 },
    { 0x1203b501, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83b509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653b50d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4123b513, 36 },
    { 0xd5a3b513, 87 },
    { 0, 0 },
    { 0x973b516, 66 },
    { 0xe7c3b516, 52 },
    { 0, 0 },
    { 0xbc93b519, 34 },
    { 0x7a33b519, 90 },
    { 0xeb03b51b, 55 },
    { 0xa133b51a, 95 },
    { 0xd343b51d, 76 },
    { 0, 0 },
    { 0xe9f3b51f, 77 },
    { 0x92f3b51f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc33b523, 33 },
    { 0xff03b524, 61 },
    { 0x5473b525, 90 },
    { 0, 0 },
    { 0x5db3b527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3b52a, 23 },
    { 0x493b52b, 33 },
    { 0x85e3b52c, 33 },
    { 0xb833b52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3b530, 85 },
    { 0, 0 },
    { 0x4d73b532, 81 },
    { 0x1fa3b533, 66 },
    { 0xe813b534, 98 },
    { 0xb9d3b535, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee3b53e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa03b541, 23 },
    { 0, 0 },
    { 0x3c13b543, 95 },
    { 0x9a03b543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f3b547, 33 },
    { 0xf563b548, 33 },
    { 0xed13b549, 76 },
    { 0, 0 },
    { 0xc903b54b, 95 },
    { 0x1653b54b, 33 },
    { 0, 0 },
    { 0x29a3b54e, 23 },
    { 0x9353b54f, 55 },
    { 0x86a3b54e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd123b555, 77 },
    { 0xab03b555, 33 },
    { 0x303b555, 95 },
    { 0xf193b558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3b55f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b03b562, 93 },
    { 0x1b03b563, 90 },
    { 0, 0 },
    { 0x44c3b565, 52 },
    { 0xd563b565, 90 },
    { 0, 0 },
    { 0x3913b568, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413b573, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3b576, 33 },
    { 0, 0 },
    { 0x4913b578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3b57d, 9 },
    { 0xb403b57e, 55 },
    { 0x8623b57e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423b58b, 43 },
    { 0, 0 },
    { 0x253b58d, 93 },
    { 0xfaa3b58d, 55 },
    { 0xf343b58e, 33 },
    { 0xe0f3b590, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc693b594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673b59c, 81 },
    { 0xdc53b59d, 76 },
    { 0x2f63b59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe963b5a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3b5ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113b5af, 76 },
    { 0xd523b5af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8473b5b5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d33b5b8, 91 },
    { 0x91f3b5b9, 34 },
    { 0xda63b5b9, 61 },
    { 0, 0 },
    { 0x79f3b5bc, 46 },
    { 0x86a3b5bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac3b5cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753b5ce, 91 },
    { 0x7ae3b5ce, 98 },
    { 0x473b5d0, 61 },
    { 0x2d33b5d0, 90 },
    { 0x4bb3b5d2, 33 },
    { 0x21b3b5d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa503b5d9, 90 },
    { 0, 0 },
    { 0xe3f3b5db, 77 },
    { 0x4a53b5dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a93b5df, 99 },
    { 0x7be3b5e0, 90 },
    { 0x3553b5e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553b5e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc523b5ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb143b5f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d3b5f5, 76 },
    { 0xce93b5f5, 99 },
    { 0, 0 },
    { 0x53a3b5f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3b5fd, 34 },
    { 0x8313b5fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8413b602, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f3b605, 34 },
    { 0, 0 },
    { 0x5d93b607, 95 },
    { 0x7543b608, 52 },
    { 0xa063b607, 55 },
    { 0xbb83b60a, 66 },
    { 0, 0 },
    { 0xa5e3b60c, 23 },
    { 0x85e3b60d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd033b616, 61 },
    { 0xbd23b617, 90 },
    { 0x49f3b618, 99 },
    { 0xe3d3b619, 99 },
    { 0xcb23b61a, 78 },
    { 0x2543b61b, 61 },
    { 0xa13b619, 61 },
    { 0x7b23b61c, 93 },
    { 0xb1e3b61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a23b625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7173b62a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af3b630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff3b635, 23 },
    { 0, 0 },
    { 0x8513b637, 95 },
    { 0x80e3b638, 85 },
    { 0, 0 },
    { 0x2e3b63a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713b63e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83b643, 33 },
    { 0xb593b643, 23 },
    { 0xc303b645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a3b64a, 95 },
    { 0, 0 },
    { 0xa5a3b64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe93b650, 98 },
    { 0x5313b651, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5933b654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e3b657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c3b65b, 90 },
    { 0x7ff3b65c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503b660, 23 },
    { 0x9b93b660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb793b669, 88 },
    { 0, 0 },
    { 0x1043b66b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53b66f, 43 },
    { 0x1303b670, 33 },
    { 0x76a3b671, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae3b67b, 98 },
    { 0xf6c3b67c, 75 },
    { 0xec03b67d, 23 },
    { 0x7393b67b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573b681, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa953b685, 34 },
    { 0, 0 },
    { 0x9763b687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc953b68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b43b690, 95 },
    { 0, 0 },
    { 0x1a83b692, 66 },
    { 0x6cd3b693, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6043b696, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6143b69a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e3b69d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b6a0, 34 },
    { 0, 0 },
    { 0xa913b6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f3b6a7, 95 },
    { 0xd9e3b6a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113b6ac, 67 },
    { 0x9553b6ad, 34 },
    { 0xd9b3b6ae, 95 },
    { 0, 0 },
    { 0x333b6b0, 33 },
    { 0, 0 },
    { 0x7183b6b2, 17 },
    { 0, 0 },
    { 0x6963b6b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e3b6b7, 88 },
    { 0, 0 },
    { 0xda63b6b9, 33 },
    { 0x95c3b6ba, 90 },
    { 0, 0 },
    { 0x1503b6bc, 33 },
    { 0xa1c3b6bd, 43 },
    { 0x363b6bd, 98 },
    { 0xb913b6bf, 23 },
    { 0x1cb3b6bf, 61 },
    { 0x9e43b6bc, 98 },
    { 0xcfb3b6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533b6c7, 90 },
    { 0, 0 },
    { 0xcce3b6c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec3b6cf, 95 },
    { 0, 0 },
    { 0xa9b3b6d1, 69 },
    { 0x71d3b6d2, 98 },
    { 0x97e3b6d3, 34 },
    { 0xfdf3b6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83b6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3b6e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b83b6e6, 67 },
    { 0x9d33b6e7, 66 },
    { 0xe163b6e7, 66 },
    { 0xae03b6e9, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2213b6ed, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93b6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe493b6f3, 23 },
    { 0x2373b6f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be3b6ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3b704, 61 },
    { 0, 0 },
    { 0xa9e3b706, 90 },
    { 0xb33b707, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa3b70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb853b712, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb913b716, 91 },
    { 0x35c3b717, 76 },
    { 0x16c3b718, 61 },
    { 0x66d3b719, 33 },
    { 0, 0 },
    { 0x84d3b71b, 90 },
    { 0xcff3b71c, 100 },
    { 0x953b71d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53b720, 99 },
    { 0, 0 },
    { 0x4f63b722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba3b727, 43 },
    { 0x59e3b728, 85 },
    { 0x4c23b729, 90 },
    { 0xa463b728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3413b733, 33 },
    { 0xaae3b734, 55 },
    { 0xedf3b735, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d3b738, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae43b73f, 68 },
    { 0x5a43b740, 43 },
    { 0x8fa3b741, 55 },
    { 0, 0 },
    { 0x2693b743, 61 },
    { 0xa173b744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a3b747, 101 },
    { 0xe0c3b748, 84 },
    { 0, 0 },
    { 0xdc23b74a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd93b74d, 65 },
    { 0, 0 },
    { 0x2863b74f, 17 },
    { 0, 0 },
    { 0xcc13b751, 77 },
    { 0, 0 },
    { 0x44d3b753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad3b756, 52 },
    { 0xcc03b756, 23 },
    { 0x9943b757, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c73b75b, 34 },
    { 0x48e3b75c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543b75f, 66 },
    { 0xe223b760, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83b76b, 23 },
    { 0x3bb3b76c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643b774, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d3b77b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f43b780, 100 },
    { 0x633b781, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b784, 67 },
    { 0xda03b785, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83b789, 98 },
    { 0xc563b78a, 95 },
    { 0x4123b789, 33 },
    { 0x6ba3b78c, 98 },
    { 0x53a3b78d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b3b790, 34 },
    { 0xf53b791, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e3b796, 98 },
    { 0, 0 },
    { 0x9093b798, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x8033b79b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d3b7a2, 55 },
    { 0xa723b7a3, 33 },
    { 0xcc63b7a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb33b7a7, 61 },
    { 0x82c3b7a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e3b7ab, 95 },
    { 0xec63b7ac, 46 },
    { 0xea63b7ac, 98 },
    { 0xcb3b7ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403b7b1, 43 },
    { 0xcfd3b7b2, 95 },
    { 0x22c3b7b2, 66 },
    { 0, 0 },
    { 0xbce3b7b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d53b7b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3b7bf, 93 },
    { 0x2943b7bf, 33 },
    { 0xdbf3b7c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c3b7c6, 78 },
    { 0x5133b7c7, 85 },
    { 0x4df3b7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa813b7cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d3b7d0, 33 },
    { 0, 0 },
    { 0xb6c3b7d2, 76 },
    { 0x3493b7d2, 23 },
    { 0, 0 },
    { 0x4923b7d5, 33 },
    { 0, 0 },
    { 0xf9f3b7d7, 61 },
    { 0, 0 },
    { 0xc753b7d9, 34 },
    { 0, 0 },
    { 0xdf63b7db, 52 },
    { 0x52a3b7db, 33 },
    { 0x8af3b7dd, 85 },
    { 0xf063b7de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3b7e2, 93 },
    { 0, 0 },
    { 0xb1c3b7e4, 99 },
    { 0xfbe3b7e5, 90 },
    { 0x6393b7e6, 33 },
    { 0xc643b7e7, 67 },
    { 0x1913b7e6, 90 },
    { 0xbb03b7e9, 93 },
    { 0x8073b7ea, 23 },
    { 0xbe93b7ea, 61 },
    { 0, 0 },
    { 0x4163b7ed, 61 },
    { 0, 0 },
    { 0x193b7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9013b7f4, 84 },
    { 0x1fe3b7f5, 90 },
    { 0, 0 },
    { 0x6323b7f7, 43 },
    { 0x2303b7f8, 61 },
    { 0, 0 },
    { 0xb7b3b7fa, 43 },
    { 0x34b3b7fa, 33 },
    { 0x4ab3b7fc, 43 },
    { 0xe3b7fa, 93 },
    { 0x4e23b7fa, 61 },
    { 0xf5b3b7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe953b802, 55 },
    { 0, 0 },
    { 0x79a3b804, 91 },
    { 0x7833b805, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b3b80a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33b810, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73b817, 85 },
    { 0x73a3b818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b3b81b, 93 },
    { 0x8843b81c, 92 },
    { 0xefd3b81d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7043b823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae33b828, 76 },
    { 0x8f63b829, 99 },
    { 0x7be3b82a, 36 },
    { 0x4743b82b, 93 },
    { 0xd223b82a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c03b830, 91 },
    { 0xf393b831, 91 },
    { 0, 0 },
    { 0x8cc3b833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1873b837, 10 },
    { 0xdff3b838, 33 },
    { 0, 0 },
    { 0x4c83b83a, 91 },
    { 0xb133b83b, 95 },
    { 0x8403b83c, 83 },
    { 0x23b3b83c, 59 },
    { 0x9eb3b83c, 34 },
    { 0xa3f3b83f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713b846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563b849, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3b84c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c3b84f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3893b85d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a3b862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3b866, 90 },
    { 0x1d3b867, 33 },
    { 0x2ad3b866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e3b86e, 33 },
    { 0x9ba3b86e, 93 },
    { 0x6bd3b870, 99 },
    { 0xdfa3b871, 90 },
    { 0x3483b86e, 61 },
    { 0xad53b871, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93b879, 85 },
    { 0, 0 },
    { 0x9873b87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573b87e, 100 },
    { 0, 0 },
    { 0x8ce3b880, 95 },
    { 0x6423b880, 91 },
    { 0, 0 },
    { 0x2543b883, 99 },
    { 0x1ab3b884, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba93b88c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a3b892, 95 },
    { 0x533b892, 33 },
    { 0x4c23b894, 95 },
    { 0, 0 },
    { 0x5cb3b896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf533b89d, 99 },
    { 0, 0 },
    { 0x19f3b89f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3b8a2, 61 },
    { 0x14b3b8a2, 46 },
    { 0, 0 },
    { 0x27c3b8a5, 90 },
    { 0x8ef3b8a6, 34 },
    { 0, 0 },
    { 0x3b73b8a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f23b8ae, 34 },
    { 0xbb63b8af, 23 },
    { 0, 0 },
    { 0x9f53b8b1, 90 },
    { 0x25d3b8b1, 95 },
    { 0xebb3b8b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e3b8b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073b8bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d3b8c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf023b8cd, 85 },
    { 0x2a83b8ce, 46 },
    { 0x4e3b8cf, 90 },
    { 0x3043b8d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2723b8d5, 66 },
    { 0xe833b8d6, 43 },
    { 0x50c3b8d7, 61 },
    { 0x7ba3b8d7, 90 },
    { 0xe673b8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313b8dc, 76 },
    { 0, 0 },
    { 0xf873b8de, 66 },
    { 0xc793b8df, 95 },
    { 0x4203b8e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3b8e7, 98 },
    { 0x8313b8e8, 52 },
    { 0x3683b8e9, 66 },
    { 0xbc53b8e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3b8f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83b8f8, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333b8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb563b908, 43 },
    { 0, 0 },
    { 0x33c3b90a, 59 },
    { 0xe913b90b, 40 },
    { 0x3b3b90b, 93 },
    { 0, 0 },
    { 0xa873b90e, 90 },
    { 0xa1e3b90f, 78 },
    { 0xf983b910, 61 },
    { 0x4d63b910, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23b916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf463b91c, 95 },
    { 0x36d3b91c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9153b920, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d3b926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf53b92d, 98 },
    { 0xaf83b92e, 61 },
    { 0x83e3b92f, 100 },
    { 0x9d23b92d, 98 },
    { 0x9193b931, 61 },
    { 0xa323b932, 64 },
    { 0xc073b933, 33 },
    { 0xbde3b933, 61 },
    { 0xb413b935, 61 },
    { 0x9223b934, 46 },
    { 0x5363b937, 95 },
    { 0xbb3b937, 61 },
    { 0x2213b933, 90 },
    { 0xdcd3b93a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c43b93e, 61 },
    { 0xd153b93f, 18 },
    { 0, 0 },
    { 0x22c3b941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323b944, 85 },
    { 0x9043b944, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd103b948, 87 },
    { 0xc253b949, 46 },
    { 0x4303b94a, 93 },
    { 0xaa13b94b, 76 },
    { 0xd6a3b94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf563b94f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3b952, 33 },
    { 0, 0 },
    { 0x4a63b954, 90 },
    { 0, 0 },
    { 0xf963b956, 93 },
    { 0xf833b956, 90 },
    { 0, 0 },
    { 0x6233b959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc3b961, 65 },
    { 0x7d63b961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b3b96c, 67 },
    { 0xb733b96d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db3b975, 23 },
    { 0, 0 },
    { 0xdb53b977, 90 },
    { 0xad03b978, 17 },
    { 0x24f3b979, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5383b97c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d3b97f, 90 },
    { 0x9e73b980, 95 },
    { 0x63a3b981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a63b987, 95 },
    { 0x2853b988, 23 },
    { 0, 0 },
    { 0x7043b98a, 33 },
    { 0x65c3b98b, 55 },
    { 0x9de3b98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093b993, 46 },
    { 0x5903b993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec63b99a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b03b9a2, 100 },
    { 0x6053b9a3, 43 },
    { 0, 0 },
    { 0x6373b9a5, 90 },
    { 0x6d33b9a6, 55 },
    { 0xd1e3b9a7, 33 },
    { 0xb5f3b9a5, 90 },
    { 0x68c3b9a9, 34 },
    { 0, 0 },
    { 0x63b3b9ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d3b9ae, 61 },
    { 0xa1e3b9af, 55 },
    { 0x6203b9ae, 34 },
    { 0, 0 },
    { 0x82f3b9b2, 87 },
    { 0x2a03b9b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e63b9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63b9bd, 97 },
    { 0x5643b9bd, 33 },
    { 0xb5a3b9bf, 98 },
    { 0xb513b9c0, 43 },
    { 0xc263b9c1, 66 },
    { 0x7cd3b9bf, 23 },
    { 0, 0 },
    { 0x52f3b9c4, 46 },
    { 0x303b9c4, 87 },
    { 0x6533b9c4, 33 },
    { 0xae23b9c5, 33 },
    { 0xc293b9c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2333b9cf, 84 },
    { 0, 0 },
    { 0x74e3b9d1, 17 },
    { 0x18e3b9d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6313b9d6, 95 },
    { 0x5bd3b9d7, 61 },
    { 0xf2c3b9d8, 43 },
    { 0xf8d3b9d9, 95 },
    { 0x5213b9da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6673b9e1, 99 },
    { 0x80b3b9e1, 90 },
    { 0x8b73b9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a3b9e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e3b9e9, 93 },
    { 0x3623b9ea, 46 },
    { 0x3283b9eb, 46 },
    { 0xa023b9ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3763b9f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde3b9fa, 81 },
    { 0x7863b9fa, 33 },
    { 0x68c3b9fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa353b9ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f03ba07, 90 },
    { 0xc633ba07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33ba0b, 33 },
    { 0xa93ba0c, 98 },
    { 0xfa33ba0c, 46 },
    { 0x6d03ba0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a3ba12, 84 },
    { 0xf2a3ba12, 98 },
    { 0x62d3ba14, 66 },
    { 0xa263ba15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2973ba1b, 61 },
    { 0xe2e3ba1b, 33 },
    { 0, 0 },
    { 0xacc3ba1e, 98 },
    { 0x2293ba1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb3ba28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f3ba2c, 98 },
    { 0, 0 },
    { 0xbef3ba2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c33ba34, 61 },
    { 0x2ba3ba35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d3ba3f, 85 },
    { 0x543ba3f, 95 },
    { 0x6fc3ba40, 55 },
    { 0, 0 },
    { 0xe053ba43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903ba47, 33 },
    { 0xa983ba48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d3ba4e, 50 },
    { 0xc463ba4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6453ba53, 61 },
    { 0, 0 },
    { 0xa5b3ba55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53ba60, 90 },
    { 0x1e43ba61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493ba64, 33 },
    { 0xb93ba64, 61 },
    { 0x9943ba66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9813ba69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7933ba6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e3ba75, 90 },
    { 0x6623ba76, 95 },
    { 0xa1e3ba77, 46 },
    { 0xf923ba77, 43 },
    { 0x7203ba79, 98 },
    { 0xb923ba79, 98 },
    { 0x7b93ba7b, 67 },
    { 0x823ba7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b3ba81, 91 },
    { 0xa633ba81, 34 },
    { 0, 0 },
    { 0x3113ba84, 23 },
    { 0xfa03ba85, 90 },
    { 0xa7a3ba86, 61 },
    { 0x3403ba87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa3ba8a, 33 },
    { 0, 0 },
    { 0x3293ba8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc3ba90, 61 },
    { 0x3a43ba90, 19 },
    { 0, 0 },
    { 0x49b3ba93, 61 },
    { 0xa833ba94, 34 },
    { 0, 0 },
    { 0xaa03ba96, 23 },
    { 0xd583ba96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b3ba9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473ba9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b63baa5, 33 },
    { 0xe203baa6, 61 },
    { 0x8673baa7, 90 },
    { 0xf193baa8, 78 },
    { 0, 0 },
    { 0x8ef3baaa, 98 },
    { 0x1603baab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa3baaf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173bab7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d3babb, 90 },
    { 0x2bf3babb, 95 },
    { 0x8403babd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9653bac1, 55 },
    { 0x4f23bac1, 90 },
    { 0, 0 },
    { 0x5763bac4, 85 },
    { 0x5b13bac5, 91 },
    { 0x2dd3bac6, 85 },
    { 0x61c3bac6, 61 },
    { 0x4743bac6, 78 },
    { 0x2d53bac4, 93 },
    { 0xf43baca, 33 },
    { 0x5ed3bac5, 76 },
    { 0, 0 },
    { 0x1ae3bacd, 100 },
    { 0xb1f3bacd, 90 },
    { 0xe4c3bacd, 34 },
    { 0x6ff3bad0, 33 },
    { 0, 0 },
    { 0xbd93bad2, 33 },
    { 0, 0 },
    { 0xa3b3bad4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4003bad9, 55 },
    { 0x3793bada, 90 },
    { 0, 0 },
    { 0x1013badc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc13badf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3bae2, 84 },
    { 0x8f43bae3, 23 },
    { 0xcd13bae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5713baea, 61 },
    { 0xe833baeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383baee, 46 },
    { 0, 0 },
    { 0xd103baf0, 98 },
    { 0xe303baf1, 61 },
    { 0, 0 },
    { 0xfdc3baf3, 98 },
    { 0x8823baf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6153baf7, 61 },
    { 0xd8b3baf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93bb05, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23bb08, 33 },
    { 0, 0 },
    { 0xa483bb0a, 98 },
    { 0, 0 },
    { 0xfa53bb0c, 33 },
    { 0xfc23bb0d, 98 },
    { 0, 0 },
    { 0xccc3bb0f, 46 },
    { 0, 0 },
    { 0x9173bb11, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b3bb17, 33 },
    { 0xbda3bb18, 33 },
    { 0x6c43bb19, 99 },
    { 0x1523bb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963bb20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223bb29, 23 },
    { 0xf253bb29, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93bb30, 98 },
    { 0xd363bb31, 93 },
    { 0x5883bb32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63bb36, 90 },
    { 0x9913bb37, 38 },
    { 0x5593bb37, 64 },
    { 0xb783bb39, 95 },
    { 0x1da3bb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe93bb3e, 46 },
    { 0xa7a3bb3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843bb44, 34 },
    { 0x8d43bb45, 23 },
    { 0xd7d3bb46, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a3bb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d3bb4c, 81 },
    { 0x3743bb4d, 23 },
    { 0xa093bb4c, 24 },
    { 0x77b3bb4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d33bb55, 85 },
    { 0x7313bb56, 76 },
    { 0x68b3bb57, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae93bb5c, 98 },
    { 0x17b3bb5d, 33 },
    { 0x5893bb5e, 90 },
    { 0x6c13bb5f, 99 },
    { 0, 0 },
    { 0x6b83bb61, 93 },
    { 0xa743bb62, 99 },
    { 0, 0 },
    { 0x98b3bb64, 95 },
    { 0xb653bb65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a3bb6c, 23 },
    { 0xec13bb6d, 51 },
    { 0xb313bb6e, 69 },
    { 0x533bb6f, 61 },
    { 0x2913bb6f, 23 },
    { 0xc073bb71, 66 },
    { 0x3923bb72, 90 },
    { 0xb283bb71, 99 },
    { 0x44a3bb6e, 61 },
    { 0x9b03bb75, 66 },
    { 0, 0 },
    { 0x4663bb77, 90 },
    { 0x89f3bb78, 87 },
    { 0xf8a3bb79, 33 },
    { 0x1e83bb7a, 90 },
    { 0x51d3bb7b, 97 },
    { 0, 0 },
    { 0x8073bb7d, 77 },
    { 0x77d3bb7d, 90 },
    { 0x8253bb7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5173bb82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad63bb88, 23 },
    { 0x7743bb89, 61 },
    { 0x2293bb8a, 76 },
    { 0x7143bb89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3bb90, 34 },
    { 0x2fb3bb91, 77 },
    { 0x3f43bb92, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6753bb96, 93 },
    { 0xb063bb97, 70 },
    { 0x8313bb98, 95 },
    { 0x9d53bb98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca3bb9c, 77 },
    { 0, 0 },
    { 0x98f3bb9e, 36 },
    { 0, 0 },
    { 0xd353bba0, 84 },
    { 0x6653bba1, 66 },
    { 0, 0 },
    { 0xb1d3bba3, 40 },
    { 0, 0 },
    { 0x3683bba5, 98 },
    { 0xd9f3bba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f33bbaa, 12 },
    { 0, 0 },
    { 0x1f43bbac, 99 },
    { 0x1a93bbad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e3bbb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad3bbbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7473bbc6, 80 },
    { 0xe4b3bbc7, 66 },
    { 0xb2e3bbc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e63bbce, 95 },
    { 0x38b3bbcf, 90 },
    { 0, 0 },
    { 0x4f73bbd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1403bbd5, 76 },
    { 0xf063bbd6, 23 },
    { 0x7953bbd6, 76 },
    { 0x8473bbd8, 83 },
    { 0x103bbd9, 66 },
    { 0, 0 },
    { 0x3803bbdb, 43 },
    { 0xa83bbdc, 98 },
    { 0x8e23bbdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8103bbe4, 33 },
    { 0xff3bbe4, 77 },
    { 0x2d93bbe6, 61 },
    { 0xab93bbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263bbee, 90 },
    { 0x9283bbef, 91 },
    { 0x1c53bbee, 55 },
    { 0x9e13bbef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5423bbf7, 46 },
    { 0, 0 },
    { 0x9893bbf9, 88 },
    { 0x95c3bbfa, 46 },
    { 0x6fa3bbfa, 91 },
    { 0, 0 },
    { 0xc863bbfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe633bc00, 33 },
    { 0x50b3bc01, 90 },
    { 0, 0 },
    { 0xedb3bc03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f3bc06, 98 },
    { 0x19b3bc07, 87 },
    { 0x3c3bc08, 98 },
    { 0x1f73bc09, 90 },
    { 0, 0 },
    { 0x4783bc0b, 95 },
    { 0, 0 },
    { 0x33bc0d, 95 },
    { 0xc773bc0e, 98 },
    { 0, 0 },
    { 0x6403bc10, 61 },
    { 0, 0 },
    { 0x6d53bc12, 36 },
    { 0x27f3bc13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf3bc16, 43 },
    { 0x733bc17, 23 },
    { 0x6f83bc18, 61 },
    { 0xecf3bc18, 99 },
    { 0, 0 },
    { 0x3a33bc1b, 95 },
    { 0x8e23bc1c, 95 },
    { 0xaae3bc1d, 19 },
    { 0x1f33bc1e, 76 },
    { 0x3863bc1d, 98 },
    { 0x57f3bc20, 55 },
    { 0x2853bc1c, 90 },
    { 0x613bc1e, 55 },
    { 0xda83bc1d, 34 },
    { 0x9fa3bc1b, 61 },
    { 0xb1b3bc25, 95 },
    { 0x9153bc26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3bc2b, 98 },
    { 0, 0 },
    { 0xcf13bc2d, 90 },
    { 0x2e83bc2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453bc31, 33 },
    { 0xff33bc32, 61 },
    { 0x89d3bc32, 61 },
    { 0x2643bc31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3bc38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x163bc3b, 33 },
    { 0xbd23bc3b, 98 },
    { 0xbb53bc3b, 23 },
    { 0x5c53bc3e, 75 },
    { 0xf293bc3f, 64 },
    { 0x9ac3bc3e, 61 },
    { 0x93d3bc41, 84 },
    { 0, 0 },
    { 0x5cb3bc43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663bc47, 98 },
    { 0x1143bc48, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173bc4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1273bc52, 76 },
    { 0xace3bc53, 33 },
    { 0x1053bc54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4943bc57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc023bc5e, 84 },
    { 0, 0 },
    { 0xa833bc60, 43 },
    { 0, 0 },
    { 0xf143bc62, 90 },
    { 0xce33bc63, 66 },
    { 0, 0 },
    { 0xf3c3bc65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b3bc6d, 33 },
    { 0xe3d3bc6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb463bc71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e3bc76, 46 },
    { 0xe003bc76, 75 },
    { 0x5f83bc78, 87 },
    { 0xc943bc79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33bc7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f3bc82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc473bc86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2853bc8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f63bc8d, 38 },
    { 0x4b43bc8e, 99 },
    { 0x48f3bc8f, 33 },
    { 0x45e3bc90, 90 },
    { 0x8c13bc8f, 76 },
    { 0x7d3bc92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a43bc95, 93 },
    { 0xb793bc96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3bc9a, 33 },
    { 0x5dd3bc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe803bc9e, 90 },
    { 0, 0 },
    { 0x4583bca0, 33 },
    { 0xd123bca1, 84 },
    { 0x1b33bca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9973bca6, 67 },
    { 0x5643bca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53bcaa, 90 },
    { 0x2833bcaa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963bcb1, 90 },
    { 0x79e3bcb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93bcb7, 52 },
    { 0, 0 },
    { 0x5af3bcb9, 61 },
    { 0, 0 },
    { 0xb473bcbb, 33 },
    { 0xab53bcbb, 23 },
    { 0, 0 },
    { 0x32e3bcbe, 46 },
    { 0xffb3bcbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe363bcc2, 84 },
    { 0x81a3bcc3, 65 },
    { 0xf023bcc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1383bcca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c3bccf, 33 },
    { 0x9603bcd0, 90 },
    { 0, 0 },
    { 0x6883bcd2, 78 },
    { 0xa1b3bcd3, 76 },
    { 0xee33bcd4, 52 },
    { 0, 0 },
    { 0xfb83bcd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f3bcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba03bce0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703bce5, 23 },
    { 0xd253bce5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde23bcee, 91 },
    { 0xcbe3bcef, 91 },
    { 0xf3bcf0, 98 },
    { 0x9503bcf0, 61 },
    { 0x2a03bcf1, 46 },
    { 0x2b23bcf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3bcfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf763bd03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783bd08, 90 },
    { 0xc013bd09, 33 },
    { 0x3e93bd0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3843bd0d, 90 },
    { 0x7b33bd0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f3bd11, 99 },
    { 0x5c83bd11, 95 },
    { 0, 0 },
    { 0x3363bd14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53bd17, 55 },
    { 0xc933bd18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23bd1c, 33 },
    { 0xbf63bd1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed73bd20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3bd25, 90 },
    { 0xc7a3bd25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3bd2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b53bd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7913bd33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x453bd36, 65 },
    { 0xf003bd37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533bd3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3bd3e, 77 },
    { 0xe3bd3e, 90 },
    { 0x413bd40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1003bd47, 43 },
    { 0x8433bd48, 61 },
    { 0xc243bd48, 55 },
    { 0x6c3bd4a, 40 },
    { 0x9a3bd4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f3bd56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff3bd5d, 33 },
    { 0x7a63bd5e, 33 },
    { 0x2f73bd5f, 23 },
    { 0x7d3bd60, 95 },
    { 0xce3bd61, 34 },
    { 0x3583bd5e, 77 },
    { 0, 0 },
    { 0x88d3bd64, 33 },
    { 0, 0 },
    { 0xa0b3bd66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53bd6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a83bd70, 61 },
    { 0xfaf3bd70, 85 },
    { 0x5ea3bd72, 61 },
    { 0x32e3bd72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a3bd77, 33 },
    { 0, 0 },
    { 0x66b3bd79, 33 },
    { 0x1cb3bd79, 90 },
    { 0x9b33bd7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb823bd7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3bd81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783bd84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf03bd8a, 98 },
    { 0, 0 },
    { 0x9de3bd8c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5083bd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec3bd94, 23 },
    { 0, 0 },
    { 0x7863bd96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff13bd99, 61 },
    { 0, 0 },
    { 0x9d63bd9b, 23 },
    { 0x37c3bd9b, 95 },
    { 0, 0 },
    { 0xc063bd9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bda2, 90 },
    { 0x123bda3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac3bda9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5013bdaf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d3bdb4, 33 },
    { 0x4113bdb5, 33 },
    { 0xa7e3bdb4, 90 },
    { 0xb303bdb7, 91 },
    { 0x9213bdb8, 95 },
    { 0, 0 },
    { 0x61e3bdba, 87 },
    { 0, 0 },
    { 0x3753bdbc, 61 },
    { 0x7613bdbc, 76 },
    { 0xc33bdbe, 95 },
    { 0, 0 },
    { 0xbed3bdc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae3bdc6, 93 },
    { 0, 0 },
    { 0xdee3bdc8, 90 },
    { 0x1923bdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb3bdcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe073bdd2, 76 },
    { 0, 0 },
    { 0x21b3bdd4, 90 },
    { 0, 0 },
    { 0xfec3bdd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493bddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc3bde1, 95 },
    { 0, 0 },
    { 0xbe83bde3, 34 },
    { 0, 0 },
    { 0xdab3bde5, 52 },
    { 0, 0 },
    { 0xb813bde7, 98 },
    { 0x6643bde7, 95 },
    { 0xbe3bde8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3bded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b3bdf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7063bdf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3bdfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c3bdfe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb23be03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3be0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3be13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f3be21, 81 },
    { 0xe923be21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a23be25, 61 },
    { 0x1ae3be25, 90 },
    { 0x2a03be25, 61 },
    { 0x4833be28, 33 },
    { 0x6273be29, 52 },
    { 0xc73be2a, 23 },
    { 0xdd3be2b, 23 },
    { 0x77a3be26, 99 },
    { 0xebe3be26, 67 },
    { 0x91d3be26, 90 },
    { 0x1f43be2f, 90 },
    { 0xe223be2e, 76 },
    { 0, 0 },
    { 0xd63be32, 23 },
    { 0x2333be33, 80 },
    { 0x9083be34, 81 },
    { 0x4623be33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb43be38, 95 },
    { 0, 0 },
    { 0xa83be3a, 33 },
    { 0xca33be3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b3be3f, 80 },
    { 0x45e3be3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3be44, 23 },
    { 0x9983be45, 95 },
    { 0, 0 },
    { 0x4603be47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653be4c, 76 },
    { 0xbe3be4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3be52, 98 },
    { 0, 0 },
    { 0x63a3be54, 90 },
    { 0x22e3be55, 61 },
    { 0xbd53be55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc943be5f, 69 },
    { 0x3e73be60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c63be63, 81 },
    { 0xb283be64, 67 },
    { 0x6a93be63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb3be68, 95 },
    { 0x20b3be69, 90 },
    { 0x5993be69, 46 },
    { 0, 0 },
    { 0xa633be6c, 33 },
    { 0xca83be6d, 33 },
    { 0xca93be6d, 95 },
    { 0xafd3be6c, 87 },
    { 0x8763be70, 90 },
    { 0xb4a3be6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4973be75, 46 },
    { 0xc563be75, 33 },
    { 0xccd3be76, 94 },
    { 0, 0 },
    { 0xa0e3be79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7743be7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6613be80, 64 },
    { 0x46e3be81, 61 },
    { 0x5f53be82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a3be86, 78 },
    { 0x8193be87, 55 },
    { 0x3683be86, 77 },
    { 0x50a3be89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793be8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a3be91, 55 },
    { 0, 0 },
    { 0x75a3be93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae3be97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3be9f, 34 },
    { 0x483bea0, 90 },
    { 0xa4c3bea1, 23 },
    { 0x3713bea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca3bea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3053beaa, 76 },
    { 0, 0 },
    { 0x53f3beac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a3beaf, 81 },
    { 0x1fb3beaf, 95 },
    { 0, 0 },
    { 0x1cd3beb2, 69 },
    { 0x77b3beb3, 34 },
    { 0, 0 },
    { 0xa613beb5, 90 },
    { 0x1603beb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5823bebc, 76 },
    { 0xf913bebc, 61 },
    { 0xcff3bebe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763bec2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3833bec6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463bec9, 95 },
    { 0xf2f3beca, 61 },
    { 0x83e3beca, 33 },
    { 0x5c23becb, 95 },
    { 0xc443becd, 83 },
    { 0x1de3bece, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a3bed1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b3bed4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33bedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf83bede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed3bee3, 90 },
    { 0x1823bee4, 76 },
    { 0x3ff3bee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d63beea, 85 },
    { 0, 0 },
    { 0x58b3beec, 61 },
    { 0, 0 },
    { 0x85c3beee, 76 },
    { 0xcd73beef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43bef5, 95 },
    { 0x4153bef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1753bef9, 61 },
    { 0x95d3befa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4793beff, 85 },
    { 0x5113bf00, 90 },
    { 0x1343beff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023bf0a, 61 },
    { 0, 0 },
    { 0x4a73bf0c, 99 },
    { 0x1f33bf0c, 46 },
    { 0x7523bf0c, 65 },
    { 0x11c3bf0c, 43 },
    { 0x6b53bf0d, 61 },
    { 0x54c3bf11, 61 },
    { 0x6153bf0c, 90 },
    { 0xe173bf0c, 66 },
    { 0, 0 },
    { 0x2d13bf15, 52 },
    { 0x3293bf16, 98 },
    { 0, 0 },
    { 0xeb53bf18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103bf1b, 95 },
    { 0xe3e3bf1b, 61 },
    { 0x7903bf1c, 75 },
    { 0, 0 },
    { 0x7803bf1f, 61 },
    { 0xd3b3bf20, 76 },
    { 0xd413bf20, 66 },
    { 0x5483bf22, 83 },
    { 0xad23bf22, 55 },
    { 0xc33bf23, 90 },
    { 0x6b83bf22, 66 },
    { 0x8ab3bf26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc23bf2b, 65 },
    { 0x9273bf2b, 95 },
    { 0, 0 },
    { 0x3ce3bf2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3bf37, 61 },
    { 0xd853bf38, 93 },
    { 0x53bf38, 34 },
    { 0x63a3bf3a, 33 },
    { 0, 0 },
    { 0xaa83bf3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab3bf3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f63bf43, 55 },
    { 0xb583bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313bf48, 61 },
    { 0xaff3bf49, 61 },
    { 0xccf3bf4a, 33 },
    { 0xa7b3bf4b, 46 },
    { 0x473bf48, 90 },
    { 0x703bf4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3bf54, 56 },
    { 0x4223bf55, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e3bf59, 61 },
    { 0xae73bf5a, 84 },
    { 0xf733bf5a, 81 },
    { 0xf433bf5c, 33 },
    { 0, 0 },
    { 0x3f53bf5e, 46 },
    { 0x29d3bf5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec83bf63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b3bf68, 84 },
    { 0x2b43bf69, 98 },
    { 0x9ff3bf69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad3bf6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac3bf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc063bf76, 95 },
    { 0x3af3bf76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3553bf7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003bf85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe063bf8d, 61 },
    { 0x8633bf8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3bf92, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e3bf97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa3bfa4, 98 },
    { 0, 0 },
    { 0x3033bfa6, 98 },
    { 0, 0 },
    { 0xe1f3bfa8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc413bfab, 95 },
    { 0x88a3bfab, 33 },
    { 0x2b23bfab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc3bfb0, 90 },
    { 0xe703bfb1, 33 },
    { 0, 0 },
    { 0x4f33bfb3, 98 },
    { 0x22c3bfb4, 67 },
    { 0xe7b3bfb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc563bfc1, 61 },
    { 0x2563bfc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5573bfc6, 95 },
    { 0, 0 },
    { 0x9793bfc8, 23 },
    { 0x90e3bfc8, 33 },
    { 0x4153bfc8, 76 },
    { 0x703bfc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3bfce, 61 },
    { 0xaeb3bfcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc183bfd2, 33 },
    { 0xd1e3bfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f13bfdd, 77 },
    { 0x1903bfdd, 90 },
    { 0x6b63bfdd, 97 },
    { 0x2233bfe0, 78 },
    { 0x9c63bfe1, 95 },
    { 0x1e03bfe0, 61 },
    { 0x36d3bfe3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae83bfe7, 33 },
    { 0, 0 },
    { 0x52b3bfe9, 98 },
    { 0x8b53bfea, 98 },
    { 0x7d83bfeb, 95 },
    { 0xce13bfeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f3bfef, 93 },
    { 0x77e3bff0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b3bff3, 94 },
    { 0, 0 },
    { 0x98d3bff5, 23 },
    { 0x6a03bff6, 42 },
    { 0, 0 },
    { 0xa413bff8, 98 },
    { 0xb7b3bff8, 33 },
    { 0xa7f3bffa, 66 },
    { 0x8543bffb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb3c000, 95 },
    { 0x4093c001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf643c006, 61 },
    { 0, 0 },
    { 0xc693c008, 52 },
    { 0xe7c3c008, 33 },
    { 0xfb33c00a, 23 },
    { 0x4ba3c00a, 33 },
    { 0xb923c008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2623c011, 68 },
    { 0x95e3c012, 55 },
    { 0, 0 },
    { 0x4643c014, 33 },
    { 0xb303c014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3543c01e, 71 },
    { 0, 0 },
    { 0xb03c020, 85 },
    { 0xdd03c021, 95 },
    { 0x42f3c020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d3c031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3643c035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f23c038, 55 },
    { 0xe453c039, 23 },
    { 0, 0 },
    { 0x1233c03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe3c046, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393c04d, 46 },
    { 0x523c04e, 33 },
    { 0, 0 },
    { 0x5b93c050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea73c053, 76 },
    { 0, 0 },
    { 0x8f3c055, 97 },
    { 0x18e3c056, 33 },
    { 0xa9c3c056, 33 },
    { 0x8d03c055, 23 },
    { 0x5bb3c057, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a3c05c, 78 },
    { 0x4183c05d, 95 },
    { 0xd5a3c05c, 76 },
    { 0xeb3c05d, 99 },
    { 0xb503c060, 23 },
    { 0xd2a3c05f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c067, 90 },
    { 0, 0 },
    { 0xb363c069, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43c06d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf3c071, 23 },
    { 0x56a3c072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663c075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe603c079, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3c07e, 95 },
    { 0x26d3c07e, 98 },
    { 0x1f63c080, 23 },
    { 0xb003c081, 65 },
    { 0x53c3c081, 33 },
    { 0x6213c083, 43 },
    { 0xf93c082, 95 },
    { 0x7c73c082, 61 },
    { 0x7743c080, 61 },
    { 0xf973c087, 84 },
    { 0, 0 },
    { 0xb383c089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa3c08e, 66 },
    { 0, 0 },
    { 0xe803c090, 90 },
    { 0x61d3c090, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae83c095, 17 },
    { 0x4e23c095, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3c099, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3c09c, 55 },
    { 0xd663c09d, 75 },
    { 0x10d3c09e, 81 },
    { 0x9b63c09e, 90 },
    { 0, 0 },
    { 0x3a53c0a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f3c0a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e3c0a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7823c0ab, 23 },
    { 0xf723c0ab, 23 },
    { 0x3f13c0ab, 66 },
    { 0x7363c0ae, 90 },
    { 0x2c43c0af, 81 },
    { 0x7de3c0b0, 76 },
    { 0xdf53c0ad, 93 },
    { 0, 0 },
    { 0x3963c0b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b3c0b9, 76 },
    { 0, 0 },
    { 0xee93c0bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333c0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983c0c1, 76 },
    { 0x4233c0c1, 33 },
    { 0xc1c3c0c2, 46 },
    { 0xf273c0c4, 33 },
    { 0xcfc3c0c5, 98 },
    { 0x4e83c0c6, 55 },
    { 0x4633c0c7, 77 },
    { 0x643c0c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3c0cc, 23 },
    { 0xc6e3c0cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2483c0d2, 75 },
    { 0xca53c0d3, 93 },
    { 0x9833c0d4, 98 },
    { 0xff43c0d5, 90 },
    { 0xd5f3c0d6, 90 },
    { 0xa9d3c0d6, 91 },
    { 0xef03c0d8, 61 },
    { 0xf853c0d9, 33 },
    { 0xdb03c0d9, 61 },
    { 0x2613c0db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd343c0e9, 43 },
    { 0x4b03c0ea, 33 },
    { 0xc73c0eb, 95 },
    { 0x3e83c0ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7333c0ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc83c0f3, 95 },
    { 0xdd33c0f3, 33 },
    { 0, 0 },
    { 0x9423c0f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f3c0fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9353c101, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed83c106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2603c10a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633c112, 55 },
    { 0xac33c113, 99 },
    { 0, 0 },
    { 0x2943c115, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe403c118, 90 },
    { 0xc0e3c119, 98 },
    { 0xa163c119, 93 },
    { 0, 0 },
    { 0xffe3c11c, 93 },
    { 0, 0 },
    { 0x1d13c11e, 45 },
    { 0x483c11f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563c126, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee3c12d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc723c13a, 95 },
    { 0, 0 },
    { 0xb763c13c, 68 },
    { 0x2c63c13d, 76 },
    { 0x5723c13d, 91 },
    { 0xd03c13c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a3c144, 91 },
    { 0x6f13c145, 43 },
    { 0xe3b3c144, 100 },
    { 0x5663c144, 90 },
    { 0xea23c148, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c3c14c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83c150, 93 },
    { 0x3f13c150, 66 },
    { 0x9183c150, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb3c15b, 100 },
    { 0x6003c15c, 87 },
    { 0x9673c15c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b3c162, 98 },
    { 0, 0 },
    { 0x8753c164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a93c16f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe3c172, 55 },
    { 0xb7d3c172, 23 },
    { 0x5413c173, 23 },
    { 0xb343c174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4423c179, 95 },
    { 0x283c179, 66 },
    { 0x70d3c17b, 61 },
    { 0xe483c17c, 96 },
    { 0, 0 },
    { 0xe9d3c17e, 61 },
    { 0x5d13c17f, 23 },
    { 0xa543c17f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3c183, 98 },
    { 0x69d3c184, 61 },
    { 0, 0 },
    { 0x2393c186, 23 },
    { 0xfc13c187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a3c18b, 33 },
    { 0xaed3c18c, 61 },
    { 0, 0 },
    { 0x3d73c18e, 76 },
    { 0x5233c18f, 93 },
    { 0, 0 },
    { 0xea23c191, 93 },
    { 0x25b3c192, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3963c196, 99 },
    { 0, 0 },
    { 0xc5c3c198, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3c19b, 90 },
    { 0xa813c19c, 61 },
    { 0xdc13c19d, 34 },
    { 0x4753c19b, 61 },
    { 0xc133c19f, 43 },
    { 0, 0 },
    { 0xa703c1a1, 76 },
    { 0x5993c1a2, 90 },
    { 0, 0 },
    { 0x6f43c1a4, 43 },
    { 0, 0 },
    { 0x1a03c1a6, 90 },
    { 0xd3b3c1a7, 33 },
    { 0xddc3c1a7, 91 },
    { 0, 0 },
    { 0x5c13c1aa, 92 },
    { 0xad43c1aa, 67 },
    { 0, 0 },
    { 0xb553c1ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83c1b0, 91 },
    { 0xef93c1b1, 77 },
    { 0xba83c1b0, 93 },
    { 0xa043c1b3, 34 },
    { 0xe773c1b3, 95 },
    { 0xdfe3c1b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c3c1bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c1c5, 61 },
    { 0x2513c1c6, 46 },
    { 0xb323c1c6, 98 },
    { 0x95e3c1c6, 95 },
    { 0, 0 },
    { 0xd333c1ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9793c1cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43c1d2, 61 },
    { 0xc613c1d3, 78 },
    { 0x7813c1d4, 52 },
    { 0xc893c1d5, 23 },
    { 0xd4f3c1d6, 99 },
    { 0xe053c1d7, 61 },
    { 0x693c1d8, 76 },
    { 0, 0 },
    { 0x8323c1da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf23c1dd, 66 },
    { 0, 0 },
    { 0x1993c1df, 98 },
    { 0x31f3c1df, 61 },
    { 0, 0 },
    { 0x6e23c1e2, 23 },
    { 0xdd13c1e3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b3c1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab3c1ec, 98 },
    { 0x14e3c1ed, 23 },
    { 0x3ee3c1ee, 23 },
    { 0x1f63c1ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea83c1f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5523c1f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9233c1f9, 76 },
    { 0, 0 },
    { 0x75b3c1fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3c203, 33 },
    { 0xed33c204, 38 },
    { 0x10a3c205, 90 },
    { 0xf843c206, 88 },
    { 0x39e3c207, 90 },
    { 0, 0 },
    { 0x2843c209, 98 },
    { 0xd6b3c20a, 66 },
    { 0x79f3c20b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb963c213, 95 },
    { 0x73b3c214, 61 },
    { 0x37b3c215, 23 },
    { 0, 0 },
    { 0xcfe3c217, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d3c21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf183c223, 34 },
    { 0x5b23c224, 68 },
    { 0, 0 },
    { 0xc583c226, 23 },
    { 0, 0 },
    { 0x31a3c228, 33 },
    { 0, 0 },
    { 0xae03c22a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403c22d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c83c230, 34 },
    { 0xfda3c230, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9323c235, 90 },
    { 0x3e13c236, 40 },
    { 0, 0 },
    { 0x31f3c238, 95 },
    { 0, 0 },
    { 0xf683c23a, 23 },
    { 0, 0 },
    { 0x8283c23c, 61 },
    { 0xcbb3c23d, 76 },
    { 0x9ad3c23d, 34 },
    { 0xb383c23f, 88 },
    { 0xd5c3c240, 33 },
    { 0x1bc3c241, 55 },
    { 0xa863c242, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d3c245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3c24e, 77 },
    { 0x61a3c24f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d03c254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d3c25a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc93c25f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313c267, 95 },
    { 0x74b3c267, 33 },
    { 0x20e3c269, 55 },
    { 0x633c26a, 61 },
    { 0x82c3c26b, 33 },
    { 0x8333c26a, 95 },
    { 0x2c23c26b, 96 },
    { 0x2ba3c269, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd3c276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e3c27e, 97 },
    { 0xfad3c27f, 76 },
    { 0x95e3c280, 61 },
    { 0xcd83c27e, 33 },
    { 0x78f3c282, 23 },
    { 0xac33c283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3163c286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173c289, 98 },
    { 0xa093c28a, 93 },
    { 0x7e73c28a, 90 },
    { 0, 0 },
    { 0x5de3c28d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f3c295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b3c298, 90 },
    { 0xe853c299, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d53c29c, 96 },
    { 0x71d3c29d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda3c2a0, 33 },
    { 0xe533c2a1, 90 },
    { 0x16d3c2a2, 55 },
    { 0xcf13c2a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d3c2a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93c2ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533c2b2, 80 },
    { 0x74a3c2b3, 83 },
    { 0xfd83c2b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3353c2bc, 98 },
    { 0x5923c2bd, 101 },
    { 0x3683c2bd, 98 },
    { 0x37b3c2bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf33c2c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa3c2cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb03c2d4, 55 },
    { 0x1113c2d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e33c2dc, 78 },
    { 0x9b33c2dd, 84 },
    { 0x17d3c2dc, 43 },
    { 0x2fe3c2dd, 55 },
    { 0xd2e3c2e0, 23 },
    { 0xcf83c2e1, 61 },
    { 0x56b3c2e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee3c2ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23c2ed, 23 },
    { 0x4ed3c2ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53c2fa, 65 },
    { 0, 0 },
    { 0x3823c2fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6853c30a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf03c30e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483c313, 95 },
    { 0, 0 },
    { 0x6d03c315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f3c31d, 34 },
    { 0x2983c31e, 91 },
    { 0x7233c31f, 52 },
    { 0xec13c31e, 95 },
    { 0xe073c321, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1733c32b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e73c32e, 46 },
    { 0x7543c32e, 43 },
    { 0, 0 },
    { 0x30c3c331, 43 },
    { 0, 0 },
    { 0x5643c333, 52 },
    { 0xcec3c333, 46 },
    { 0xf523c335, 90 },
    { 0, 0 },
    { 0x22b3c337, 34 },
    { 0x2f63c338, 33 },
    { 0x59b3c338, 87 },
    { 0x12f3c33a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa203c33d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23c342, 61 },
    { 0x4213c343, 84 },
    { 0xa4b3c344, 22 },
    { 0x8d23c345, 33 },
    { 0xa0e3c346, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3c34a, 61 },
    { 0xc273c34b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2333c34e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8503c352, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223c357, 52 },
    { 0xb113c358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3c35b, 93 },
    { 0x8b53c35b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e3c361, 33 },
    { 0x1ab3c362, 34 },
    { 0, 0 },
    { 0x5c53c364, 99 },
    { 0xaef3c364, 90 },
    { 0x8583c366, 98 },
    { 0xfeb3c367, 90 },
    { 0x6ae3c368, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd293c36b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c36e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd83c375, 66 },
    { 0xf983c376, 66 },
    { 0x3fa3c375, 98 },
    { 0, 0 },
    { 0x9cb3c379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63c37c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e3c37f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3c382, 78 },
    { 0, 0 },
    { 0x2d73c384, 61 },
    { 0x9233c385, 46 },
    { 0x9ef3c384, 95 },
    { 0, 0 },
    { 0x6583c388, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d73c38b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6773c38f, 90 },
    { 0x1f73c390, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe353c396, 23 },
    { 0xc163c396, 76 },
    { 0x3b83c398, 46 },
    { 0xfb73c399, 46 },
    { 0xe53c398, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893c3ab, 23 },
    { 0, 0 },
    { 0x33c3ad, 95 },
    { 0x673c3ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3c3b1, 88 },
    { 0xb963c3b2, 43 },
    { 0x4883c3b2, 100 },
    { 0x8273c3b4, 77 },
    { 0xf7e3c3b4, 33 },
    { 0x3363c3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd3c3ba, 98 },
    { 0x12f3c3bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3c3c0, 87 },
    { 0, 0 },
    { 0x47a3c3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a13c3c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223c3cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa333c3d1, 96 },
    { 0x9bf3c3d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c3d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4813c3df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073c3ee, 90 },
    { 0x6fb3c3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033c3f3, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b3c3f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb183c3fd, 95 },
    { 0xfdb3c3fe, 52 },
    { 0x2053c3ff, 66 },
    { 0xabd3c400, 33 },
    { 0, 0 },
    { 0x6263c402, 33 },
    { 0x60d3c403, 61 },
    { 0, 0 },
    { 0xbd3c405, 55 },
    { 0xc973c406, 46 },
    { 0x1603c407, 52 },
    { 0x1df3c407, 43 },
    { 0x83d3c409, 33 },
    { 0x39c3c409, 99 },
    { 0xcbc3c40b, 23 },
    { 0xf043c40c, 33 },
    { 0x9aa3c40c, 67 },
    { 0x8193c40e, 67 },
    { 0xba23c40f, 98 },
    { 0x1393c408, 43 },
    { 0x53c3c40a, 55 },
    { 0x5f63c412, 90 },
    { 0, 0 },
    { 0xa943c414, 61 },
    { 0xcf03c414, 76 },
    { 0xad43c415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23c419, 23 },
    { 0xfc73c41a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403c424, 98 },
    { 0x44e3c425, 90 },
    { 0x3023c426, 33 },
    { 0x483c427, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f3c42b, 81 },
    { 0x7e93c42c, 90 },
    { 0xfdf3c42d, 33 },
    { 0xd513c42c, 23 },
    { 0x46e3c42f, 34 },
    { 0x813c42e, 87 },
    { 0xc493c42f, 95 },
    { 0x4e03c432, 76 },
    { 0x3d13c432, 34 },
    { 0xe843c434, 77 },
    { 0x40f3c434, 43 },
    { 0xa323c42e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93c43b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf03c444, 55 },
    { 0xb523c445, 91 },
    { 0, 0 },
    { 0x13c3c447, 33 },
    { 0xdbe3c448, 76 },
    { 0x46b3c449, 84 },
    { 0x9b33c449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c3c44f, 33 },
    { 0x263c450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563c453, 95 },
    { 0xe0d3c453, 61 },
    { 0, 0 },
    { 0xd9e3c456, 98 },
    { 0xb1a3c457, 98 },
    { 0xe1e3c457, 91 },
    { 0, 0 },
    { 0x8e43c45a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3503c45e, 99 },
    { 0, 0 },
    { 0xa423c460, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c465, 46 },
    { 0x3653c465, 61 },
    { 0xd233c467, 95 },
    { 0xcf13c468, 23 },
    { 0x673c468, 33 },
    { 0, 0 },
    { 0x2d33c46b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf583c46e, 66 },
    { 0x6863c46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883c472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d3c476, 93 },
    { 0x21f3c477, 55 },
    { 0x8d63c477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8923c47c, 61 },
    { 0, 0 },
    { 0x6d03c47e, 43 },
    { 0xc5e3c47f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb3c486, 98 },
    { 0x3c83c487, 46 },
    { 0, 0 },
    { 0xa093c489, 33 },
    { 0x60d3c48a, 100 },
    { 0x8c33c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe353c490, 33 },
    { 0x6313c490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3303c498, 33 },
    { 0, 0 },
    { 0xacd3c49a, 67 },
    { 0xa063c49a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93c4a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3c4a4, 95 },
    { 0xba43c4a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5853c4a9, 46 },
    { 0, 0 },
    { 0xef83c4ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a3c4af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb3c4b2, 61 },
    { 0xa083c4b3, 34 },
    { 0x1023c4b3, 61 },
    { 0, 0 },
    { 0x71e3c4b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c23c4b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53c4bc, 23 },
    { 0x61d3c4bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813c4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b3c4c5, 95 },
    { 0, 0 },
    { 0x27b3c4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa3c4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3c4d2, 33 },
    { 0, 0 },
    { 0x5a03c4d4, 100 },
    { 0x70a3c4d5, 90 },
    { 0, 0 },
    { 0xddc3c4d7, 84 },
    { 0, 0 },
    { 0xbbf3c4d9, 90 },
    { 0x2833c4da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3c4dd, 98 },
    { 0xd263c4dd, 85 },
    { 0xc4b3c4df, 33 },
    { 0x3f83c4df, 100 },
    { 0xa253c4de, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b3c4e5, 61 },
    { 0xb3d3c4e5, 33 },
    { 0, 0 },
    { 0xf4d3c4e8, 76 },
    { 0xde33c4e9, 33 },
    { 0x92c3c4e9, 61 },
    { 0x65b3c4ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb003c4f2, 95 },
    { 0x26b3c4f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c53c4f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3c4fb, 76 },
    { 0x1d23c4fb, 98 },
    { 0xf883c4fd, 90 },
    { 0, 0 },
    { 0x4023c4ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe783c502, 52 },
    { 0x71b3c502, 91 },
    { 0x9773c504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b3c50c, 91 },
    { 0, 0 },
    { 0xc113c50e, 33 },
    { 0xdd03c50e, 95 },
    { 0x8ae3c510, 93 },
    { 0x6203c510, 90 },
    { 0x50a3c50e, 61 },
    { 0xc943c512, 76 },
    { 0xc323c514, 61 },
    { 0x25b3c513, 93 },
    { 0, 0 },
    { 0xe4c3c517, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c3c51a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23c51d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c3c523, 69 },
    { 0x9ef3c524, 66 },
    { 0x5583c524, 61 },
    { 0xdbb3c525, 98 },
    { 0x2553c525, 95 },
    { 0x91a3c527, 35 },
    { 0x1493c529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6333c535, 34 },
    { 0, 0 },
    { 0x70c3c537, 33 },
    { 0, 0 },
    { 0x3753c539, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee03c53f, 91 },
    { 0, 0 },
    { 0x44d3c541, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3683c547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33c550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e3c555, 55 },
    { 0x2a83c556, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3c55a, 95 },
    { 0x7633c55b, 91 },
    { 0x9cb3c55a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d3c560, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa973c564, 65 },
    { 0x5b73c564, 95 },
    { 0xc003c565, 76 },
    { 0x8743c566, 66 },
    { 0x9f83c564, 100 },
    { 0x3db3c569, 95 },
    { 0x8c03c56a, 64 },
    { 0x3a43c564, 90 },
    { 0x47b3c564, 93 },
    { 0, 0 },
    { 0xeb13c56e, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a63c573, 76 },
    { 0xeba3c574, 68 },
    { 0x8113c575, 76 },
    { 0x9773c575, 34 },
    { 0xcf63c575, 90 },
    { 0xb713c578, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c43c57b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3c57e, 33 },
    { 0xf373c57f, 23 },
    { 0x6053c57e, 76 },
    { 0x67c3c57f, 75 },
    { 0x7753c582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b33c585, 98 },
    { 0x1f03c586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9143c58b, 95 },
    { 0xc693c58c, 61 },
    { 0x1bd3c58d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e43c593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63c599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b73c59c, 95 },
    { 0x9bf3c59d, 61 },
    { 0xeae3c59d, 43 },
    { 0, 0 },
    { 0x7cc3c5a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a3c5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7523c5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f3c5b2, 23 },
    { 0xb0a3c5b3, 69 },
    { 0x1a33c5b3, 43 },
    { 0, 0 },
    { 0x7b83c5b6, 35 },
    { 0, 0 },
    { 0x7783c5b8, 33 },
    { 0xb8b3c5b9, 98 },
    { 0, 0 },
    { 0x9013c5bb, 61 },
    { 0x2583c5bc, 33 },
    { 0xfc43c5bd, 100 },
    { 0xdcf3c5bd, 68 },
    { 0xf733c5bf, 78 },
    { 0x5c63c5bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873c5c4, 23 },
    { 0x71b3c5c5, 76 },
    { 0, 0 },
    { 0xea53c5c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6883c5cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc433c5d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe903c5d7, 33 },
    { 0x6583c5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1363c5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa513c5e7, 76 },
    { 0xb2d3c5e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333c5eb, 77 },
    { 0, 0 },
    { 0xe0f3c5ed, 33 },
    { 0xa6b3c5ee, 78 },
    { 0x52e3c5ef, 78 },
    { 0xd6c3c5ef, 33 },
    { 0x7153c5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3c5fe, 55 },
    { 0, 0 },
    { 0xa663c600, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f3c606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac3c609, 33 },
    { 0x9533c609, 90 },
    { 0xe803c609, 98 },
    { 0x3ab3c60c, 66 },
    { 0xd3d3c60d, 61 },
    { 0x9ad3c60a, 61 },
    { 0x5383c609, 95 },
    { 0xf2a3c610, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd63c614, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013c618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3c61f, 46 },
    { 0xfb53c620, 65 },
    { 0x43c3c621, 90 },
    { 0x2683c620, 23 },
    { 0x7a63c61f, 33 },
    { 0x9fc3c624, 34 },
    { 0x6903c61f, 91 },
    { 0xcec3c624, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa563c62b, 90 },
    { 0, 0 },
    { 0xad23c62d, 90 },
    { 0x4d33c62e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d93c633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac3c636, 90 },
    { 0xa5a3c636, 34 },
    { 0, 0 },
    { 0xb2f3c639, 78 },
    { 0x24d3c63a, 99 },
    { 0, 0 },
    { 0xc583c63c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c644, 55 },
    { 0xd153c645, 23 },
    { 0, 0 },
    { 0x8633c647, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc83c64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3c64e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e3c658, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f3c65b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263c65f, 33 },
    { 0x7243c660, 52 },
    { 0x7333c661, 12 },
    { 0x22e3c662, 23 },
    { 0xa983c663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933c667, 33 },
    { 0x3823c668, 90 },
    { 0, 0 },
    { 0xc8a3c66a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed3c66d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b3c670, 95 },
    { 0x3903c671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd3c674, 90 },
    { 0x3693c675, 23 },
    { 0x9293c676, 77 },
    { 0x9e63c675, 95 },
    { 0x9a03c674, 76 },
    { 0x95d3c674, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd063c67d, 33 },
    { 0x9603c67e, 33 },
    { 0xf603c67d, 95 },
    { 0xb783c680, 33 },
    { 0, 0 },
    { 0xe8a3c682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373c686, 95 },
    { 0, 0 },
    { 0x6363c688, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9693c68c, 93 },
    { 0x18f3c68c, 99 },
    { 0, 0 },
    { 0x76a3c68f, 61 },
    { 0x41c3c68f, 43 },
    { 0xed13c691, 61 },
    { 0xd883c692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633c69a, 52 },
    { 0x4c33c69b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d3c69e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3c6a2, 90 },
    { 0xfd13c6a3, 33 },
    { 0x11e3c6a4, 33 },
    { 0x943c6a5, 33 },
    { 0, 0 },
    { 0x7873c6a7, 61 },
    { 0, 0 },
    { 0xb423c6a9, 33 },
    { 0x3173c6aa, 99 },
    { 0, 0 },
    { 0x6083c6ac, 76 },
    { 0x1e93c6ad, 60 },
    { 0x4033c6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad43c6b2, 78 },
    { 0xeb33c6b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93c6b9, 61 },
    { 0x5a33c6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2423c6bd, 34 },
    { 0xb763c6bd, 43 },
    { 0, 0 },
    { 0x3c53c6c0, 23 },
    { 0, 0 },
    { 0xb213c6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43c6c7, 43 },
    { 0xa883c6c8, 66 },
    { 0x7b03c6c9, 90 },
    { 0xbf3c6c9, 65 },
    { 0x8163c6cb, 98 },
    { 0x5983c6c7, 34 },
    { 0x85e3c6cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb453c6d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a3c6d6, 95 },
    { 0xa4a3c6d7, 98 },
    { 0xa173c6d6, 93 },
    { 0x3933c6d8, 90 },
    { 0xbfa3c6da, 33 },
    { 0x55f3c6d9, 99 },
    { 0x9973c6d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743c6e5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3c6e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a23c6f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb23c6fa, 23 },
    { 0x7ae3c6fb, 52 },
    { 0xa303c6fc, 55 },
    { 0x8b03c6fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec3c700, 90 },
    { 0xb8f3c701, 34 },
    { 0xd2e3c701, 33 },
    { 0x11b3c700, 98 },
    { 0xd5b3c704, 100 },
    { 0x6393c704, 95 },
    { 0x4073c704, 90 },
    { 0x3493c704, 98 },
    { 0x46b3c708, 61 },
    { 0xa483c706, 95 },
    { 0xf583c709, 76 },
    { 0, 0 },
    { 0x9c73c70c, 90 },
    { 0x57c3c70c, 43 },
    { 0xb103c70e, 38 },
    { 0x59c3c70e, 76 },
    { 0xfc93c710, 98 },
    { 0x5d33c70e, 33 },
    { 0x4283c70c, 61 },
    { 0x90a3c70e, 33 },
    { 0, 0 },
    { 0x72f3c715, 34 },
    { 0, 0 },
    { 0xa43c717, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf653c71c, 98 },
    { 0, 0 },
    { 0xf933c71e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d3c724, 23 },
    { 0xda83c725, 33 },
    { 0, 0 },
    { 0xed63c727, 55 },
    { 0xf423c727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783c734, 61 },
    { 0, 0 },
    { 0xfb23c736, 23 },
    { 0x43a3c736, 23 },
    { 0x32e3c737, 66 },
    { 0x773c737, 90 },
    { 0x5d43c73a, 33 },
    { 0, 0 },
    { 0x2173c73c, 43 },
    { 0xb343c73c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3c740, 95 },
    { 0x20e3c741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013c746, 61 },
    { 0xae3c747, 61 },
    { 0x6b43c747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3c74b, 34 },
    { 0xfe43c74c, 95 },
    { 0x7a13c74d, 43 },
    { 0x6603c74b, 95 },
    { 0xac83c74f, 93 },
    { 0xccf3c750, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3c755, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3c758, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e23c75c, 90 },
    { 0x4413c75d, 90 },
    { 0xc223c75c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a93c763, 61 },
    { 0xd323c764, 17 },
    { 0xfd73c765, 65 },
    { 0xab03c766, 100 },
    { 0x33f3c766, 33 },
    { 0, 0 },
    { 0xc803c769, 98 },
    { 0x57b3c769, 98 },
    { 0, 0 },
    { 0x36f3c76c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573c76f, 33 },
    { 0x9013c770, 47 },
    { 0, 0 },
    { 0x9fd3c772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3c776, 17 },
    { 0, 0 },
    { 0xd013c778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b3c781, 33 },
    { 0, 0 },
    { 0xf153c783, 98 },
    { 0x1993c784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe323c787, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b3c78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe993c793, 33 },
    { 0xbcf3c794, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3c797, 49 },
    { 0, 0 },
    { 0x3843c799, 55 },
    { 0xd123c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443c79e, 85 },
    { 0x48e3c79f, 46 },
    { 0xc473c7a0, 95 },
    { 0x523c7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933c7a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb003c7ac, 76 },
    { 0xb943c7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe033c7b2, 85 },
    { 0xf9a3c7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3463c7b6, 23 },
    { 0xe1e3c7b6, 34 },
    { 0x6cc3c7b8, 78 },
    { 0x1d53c7b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7493c7bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793c7c3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a33c7c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3c7c9, 99 },
    { 0x8e23c7ca, 61 },
    { 0xef93c7cb, 46 },
    { 0xa203c7cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e3c7d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1813c7d5, 15 },
    { 0xe053c7d6, 97 },
    { 0, 0 },
    { 0x7333c7d8, 95 },
    { 0x1303c7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd623c7df, 67 },
    { 0, 0 },
    { 0x8953c7e1, 66 },
    { 0x2493c7e2, 95 },
    { 0, 0 },
    { 0x28e3c7e4, 76 },
    { 0xbb3c7e5, 95 },
    { 0xd633c7e6, 33 },
    { 0x1263c7e4, 55 },
    { 0xd4f3c7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9943c7ec, 91 },
    { 0x5b33c7ed, 85 },
    { 0xcf3c7ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae3c7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe563c7f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8113c7fe, 33 },
    { 0, 0 },
    { 0x7ef3c800, 90 },
    { 0xffb3c801, 91 },
    { 0xb933c802, 98 },
    { 0x4b63c803, 55 },
    { 0xd823c804, 33 },
    { 0, 0 },
    { 0xe053c806, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663c80a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3c814, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad3c817, 78 },
    { 0x4c63c818, 61 },
    { 0, 0 },
    { 0x73b3c81a, 75 },
    { 0, 0 },
    { 0x2f53c81c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4833c821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9213c825, 23 },
    { 0x5673c826, 90 },
    { 0, 0 },
    { 0x5653c828, 90 },
    { 0xdcb3c829, 33 },
    { 0x90a3c828, 34 },
    { 0x3e13c82b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe463c82e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3c833, 76 },
    { 0x2ed3c833, 78 },
    { 0x9983c835, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993c838, 78 },
    { 0xb553c839, 61 },
    { 0xa723c838, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c3c841, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b3c84d, 95 },
    { 0x7953c84e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3c852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313c858, 76 },
    { 0x8c63c858, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3c861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3803c864, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3c86a, 100 },
    { 0, 0 },
    { 0x3e53c86c, 88 },
    { 0x43a3c86d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa303c870, 90 },
    { 0x37c3c871, 77 },
    { 0x193c870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103c876, 95 },
    { 0, 0 },
    { 0x7a03c878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b93c87e, 95 },
    { 0, 0 },
    { 0x4083c880, 90 },
    { 0, 0 },
    { 0xc2d3c882, 33 },
    { 0, 0 },
    { 0x7af3c884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f73c888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f3c88b, 98 },
    { 0, 0 },
    { 0xc003c88d, 23 },
    { 0x8ab3c88e, 21 },
    { 0xc5c3c88f, 100 },
    { 0xb8b3c890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe783c894, 90 },
    { 0xd1e3c895, 101 },
    { 0x6f93c896, 34 },
    { 0x933c897, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd283c89a, 78 },
    { 0xb783c89a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b3c8a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf923c8a6, 23 },
    { 0xd9f3c8a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413c8b3, 33 },
    { 0, 0 },
    { 0xd263c8b5, 23 },
    { 0x9b33c8b6, 61 },
    { 0xd5b3c8b7, 90 },
    { 0x7703c8b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783c8bc, 23 },
    { 0x513c8bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a3c8c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3c8cc, 59 },
    { 0, 0 },
    { 0x6e03c8ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe493c8d3, 23 },
    { 0x2ad3c8d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc3c8e1, 99 },
    { 0xf793c8e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3c8e5, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xd583c8e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3c8ee, 55 },
    { 0x8853c8ef, 98 },
    { 0, 0 },
    { 0x2823c8f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c8f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4273c8f7, 90 },
    { 0x1423c8f7, 76 },
    { 0x2d23c8f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603c902, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df3c906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c90b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd893c90e, 68 },
    { 0x3a73c90f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213c914, 90 },
    { 0, 0 },
    { 0x4ce3c916, 90 },
    { 0x14e3c916, 61 },
    { 0xdaa3c918, 83 },
    { 0x9d63c919, 43 },
    { 0, 0 },
    { 0x193c91b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb3c91f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93c922, 55 },
    { 0, 0 },
    { 0x26a3c924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43c927, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e23c930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883c939, 19 },
    { 0x8113c939, 33 },
    { 0, 0 },
    { 0x6533c93c, 98 },
    { 0xed03c93c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd73c946, 90 },
    { 0, 0 },
    { 0xa973c948, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e3c94c, 77 },
    { 0xc2c3c94d, 88 },
    { 0x24e3c94c, 68 },
    { 0xa7a3c94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd3c954, 68 },
    { 0x5943c955, 93 },
    { 0x7cf3c956, 84 },
    { 0x1823c954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd3c95b, 17 },
    { 0xdb83c95c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3c964, 98 },
    { 0x7563c965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb953c96a, 90 },
    { 0x9633c96b, 46 },
    { 0x79e3c96c, 52 },
    { 0x7003c96d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1033c970, 61 },
    { 0xab73c970, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea63c974, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5083c978, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3c97b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa3c97f, 90 },
    { 0x6933c980, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf933c984, 76 },
    { 0xa6a3c985, 17 },
    { 0x3283c986, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3c98e, 81 },
    { 0, 0 },
    { 0xc1f3c990, 95 },
    { 0x1fc3c990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3c994, 52 },
    { 0, 0 },
    { 0xc793c996, 90 },
    { 0xd913c997, 98 },
    { 0x9d83c998, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33c99b, 77 },
    { 0, 0 },
    { 0x3893c99d, 98 },
    { 0x2603c99e, 90 },
    { 0xa903c99f, 95 },
    { 0, 0 },
    { 0x19f3c9a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c73c9aa, 33 },
    { 0x31f3c9ab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433c9ae, 98 },
    { 0x4d63c9af, 17 },
    { 0x3d3c9af, 91 },
    { 0xd53c9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad3c9b5, 40 },
    { 0, 0 },
    { 0xa713c9b7, 36 },
    { 0xba53c9b8, 90 },
    { 0x9c83c9b8, 85 },
    { 0x1a23c9b9, 90 },
    { 0, 0 },
    { 0x2263c9bc, 61 },
    { 0, 0 },
    { 0x53e3c9be, 85 },
    { 0xeb43c9be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33c9c6, 95 },
    { 0xd193c9c6, 80 },
    { 0x3d53c9c6, 43 },
    { 0x94b3c9c7, 34 },
    { 0xcc93c9ca, 34 },
    { 0x9f63c9cb, 67 },
    { 0x3e43c9cc, 90 },
    { 0, 0 },
    { 0xd03c9ce, 23 },
    { 0xe343c9cf, 84 },
    { 0xdaf3c9d0, 35 },
    { 0x2633c9d1, 23 },
    { 0x8183c9cf, 90 },
    { 0x2aa3c9d3, 95 },
    { 0, 0 },
    { 0xc343c9d5, 95 },
    { 0xdcb3c9d6, 95 },
    { 0x2b43c9d7, 90 },
    { 0x2a33c9d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533c9e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63c9ed, 76 },
    { 0, 0 },
    { 0xf3e3c9ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b53c9f3, 98 },
    { 0x2e23c9f4, 98 },
    { 0xe423c9f4, 100 },
    { 0x35a3c9f6, 61 },
    { 0xc873c9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae23c9ff, 95 },
    { 0, 0 },
    { 0x62e3ca01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3ca05, 95 },
    { 0xa5e3ca06, 23 },
    { 0x4853ca06, 61 },
    { 0xce03ca08, 84 },
    { 0xcb23ca06, 65 },
    { 0x5a33ca06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53ca0e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1613ca14, 98 },
    { 0x1c63ca14, 98 },
    { 0x7733ca16, 46 },
    { 0xa573ca15, 23 },
    { 0x3f33ca16, 61 },
    { 0x12b3ca19, 52 },
    { 0, 0 },
    { 0xfd53ca1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893ca21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6803ca24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43ca2e, 33 },
    { 0, 0 },
    { 0x78a3ca30, 46 },
    { 0x5073ca31, 23 },
    { 0xb8d3ca32, 44 },
    { 0x3913ca32, 23 },
    { 0xa8f3ca32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d3ca3c, 92 },
    { 0, 0 },
    { 0xe943ca3e, 58 },
    { 0x7b43ca3f, 87 },
    { 0, 0 },
    { 0x8673ca41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953ca47, 23 },
    { 0x2c73ca48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c3ca4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113ca50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3ca53, 69 },
    { 0, 0 },
    { 0xe813ca55, 55 },
    { 0, 0 },
    { 0x4663ca57, 40 },
    { 0xd3b3ca58, 98 },
    { 0x3343ca57, 85 },
    { 0, 0 },
    { 0xbe53ca5b, 61 },
    { 0x2863ca5c, 61 },
    { 0xe053ca5c, 33 },
    { 0x98d3ca5d, 90 },
    { 0x9643ca5f, 66 },
    { 0x24d3ca5f, 90 },
    { 0, 0 },
    { 0x17c3ca62, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e03ca65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83ca68, 90 },
    { 0xde43ca69, 98 },
    { 0xbdf3ca6a, 61 },
    { 0x9553ca6b, 23 },
    { 0x8133ca6b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f3ca6f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e23ca73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3153ca76, 76 },
    { 0xde73ca77, 99 },
    { 0x7143ca78, 17 },
    { 0xa743ca79, 90 },
    { 0, 0 },
    { 0x70b3ca7b, 91 },
    { 0xfe63ca7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6433ca7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153ca8c, 20 },
    { 0, 0 },
    { 0x3523ca8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de3ca92, 23 },
    { 0xbb93ca92, 55 },
    { 0, 0 },
    { 0x353ca95, 35 },
    { 0x213ca95, 90 },
    { 0x55e3ca95, 55 },
    { 0xaa3ca98, 66 },
    { 0x8c43ca99, 95 },
    { 0x7ba3ca9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad53caa2, 23 },
    { 0xefa3caa3, 76 },
    { 0, 0 },
    { 0x4953caa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1493caa8, 65 },
    { 0, 0 },
    { 0x8ed3caaa, 46 },
    { 0x26a3caab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3cab0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3cab4, 33 },
    { 0xa563cab5, 32 },
    { 0, 0 },
    { 0x30f3cab7, 98 },
    { 0, 0 },
    { 0x4ec3cab9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1113cabe, 98 },
    { 0x5f03cabf, 90 },
    { 0xb273cabe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf683cac8, 17 },
    { 0, 0 },
    { 0x3623caca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3cacd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293cad0, 61 },
    { 0x15e3cad1, 43 },
    { 0, 0 },
    { 0x4ed3cad3, 65 },
    { 0x9343cad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed3cad7, 33 },
    { 0, 0 },
    { 0x49e3cad9, 61 },
    { 0, 0 },
    { 0x3ee3cadb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc3cade, 98 },
    { 0x3cb3cade, 76 },
    { 0xced3cae0, 97 },
    { 0xe1c3cae1, 66 },
    { 0x4643cae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3cae6, 61 },
    { 0x1e43cae7, 58 },
    { 0xa03cae8, 90 },
    { 0xc183cae9, 83 },
    { 0xd723caea, 43 },
    { 0, 0 },
    { 0xc7f3caec, 46 },
    { 0x7ff3caed, 23 },
    { 0x8f43caec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713caf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc703caf7, 98 },
    { 0, 0 },
    { 0xd6b3caf9, 23 },
    { 0xf363cafa, 93 },
    { 0, 0 },
    { 0xb3d3cafc, 55 },
    { 0, 0 },
    { 0x2f83cafe, 92 },
    { 0x4bf3cafe, 99 },
    { 0x483cb00, 36 },
    { 0, 0 },
    { 0x3ec3cb02, 33 },
    { 0xf673cb03, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc983cb08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5373cb0c, 78 },
    { 0x85b3cb0d, 96 },
    { 0xb1f3cb0d, 61 },
    { 0xb323cb0e, 95 },
    { 0xb83cb10, 93 },
    { 0x3d03cb11, 99 },
    { 0xb13cb10, 90 },
    { 0x1253cb13, 49 },
    { 0xa573cb14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c3cb1b, 90 },
    { 0x293cb1c, 98 },
    { 0x37d3cb1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd53cb20, 23 },
    { 0, 0 },
    { 0x6653cb22, 23 },
    { 0, 0 },
    { 0x8793cb24, 76 },
    { 0x7fa3cb25, 90 },
    { 0, 0 },
    { 0x9c43cb27, 95 },
    { 0, 0 },
    { 0x5433cb29, 67 },
    { 0xae13cb2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73cb2e, 100 },
    { 0x8f33cb2e, 91 },
    { 0, 0 },
    { 0x903cb31, 23 },
    { 0, 0 },
    { 0xe483cb33, 24 },
    { 0x5b3cb34, 76 },
    { 0xf763cb35, 23 },
    { 0x6953cb36, 33 },
    { 0xe43cb37, 101 },
    { 0x98b3cb37, 90 },
    { 0, 0 },
    { 0x89f3cb3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3cb3d, 75 },
    { 0, 0 },
    { 0xf2e3cb3f, 90 },
    { 0xa6b3cb40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d3cb44, 91 },
    { 0xb0d3cb45, 85 },
    { 0xfb93cb45, 90 },
    { 0, 0 },
    { 0xa803cb48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e33cb4b, 34 },
    { 0xaf23cb4b, 35 },
    { 0x9a63cb4d, 99 },
    { 0xb873cb4e, 90 },
    { 0x63cb4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3cb53, 76 },
    { 0, 0 },
    { 0x92a3cb55, 33 },
    { 0, 0 },
    { 0xf8e3cb57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3cb5a, 90 },
    { 0x7a23cb5a, 43 },
    { 0xd8a3cb5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f3cb5f, 34 },
    { 0x4e83cb5f, 83 },
    { 0x3a63cb61, 33 },
    { 0, 0 },
    { 0xd4f3cb63, 33 },
    { 0x5dd3cb64, 66 },
    { 0, 0 },
    { 0xfeb3cb66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093cb69, 61 },
    { 0, 0 },
    { 0x84a3cb6b, 88 },
    { 0xb4a3cb6c, 61 },
    { 0xf1f3cb6d, 61 },
    { 0x2d13cb6d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8353cb76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a23cb7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c53cb82, 90 },
    { 0x6873cb83, 90 },
    { 0x7e43cb83, 24 },
    { 0, 0 },
    { 0x10d3cb86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f3cb89, 61 },
    { 0xd813cb8a, 34 },
    { 0x8223cb8b, 84 },
    { 0x4333cb8c, 99 },
    { 0xe133cb8d, 99 },
    { 0xab73cb8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3cb93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a3cb97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc43cb9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba53cb9e, 17 },
    { 0xa883cb9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7993cba3, 90 },
    { 0x9463cba3, 95 },
    { 0xc943cba5, 76 },
    { 0, 0 },
    { 0xa2c3cba7, 68 },
    { 0, 0 },
    { 0x78a3cba9, 95 },
    { 0x6623cbaa, 34 },
    { 0xbf93cbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43cbae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553cbb1, 87 },
    { 0xc7b3cbb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3783cbb5, 33 },
    { 0, 0 },
    { 0x5b83cbb7, 95 },
    { 0, 0 },
    { 0x74e3cbb9, 87 },
    { 0, 0 },
    { 0x4063cbbb, 81 },
    { 0x6123cbbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b3cbc0, 66 },
    { 0, 0 },
    { 0x5b33cbc2, 90 },
    { 0xabc3cbc3, 90 },
    { 0xc8c3cbc3, 61 },
    { 0x7d53cbc3, 61 },
    { 0x7563cbc6, 52 },
    { 0xe3a3cbc7, 23 },
    { 0xcaf3cbc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e3cbcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2183cbd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3cbd7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb73cbdd, 61 },
    { 0xdf53cbde, 90 },
    { 0x43b3cbdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83cbe3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3cbe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533cbe9, 61 },
    { 0xd8f3cbea, 64 },
    { 0xfc93cbea, 23 },
    { 0xdc83cbec, 33 },
    { 0x66f3cbea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b3cbf1, 61 },
    { 0x51a3cbf2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x963cbf5, 90 },
    { 0x6953cbf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da3cbfb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe713cbfe, 90 },
    { 0, 0 },
    { 0x3443cc00, 100 },
    { 0xa093cc01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be3cc0c, 23 },
    { 0x6fd3cc0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb03cc1f, 95 },
    { 0x8eb3cc20, 60 },
    { 0x7c43cc1f, 17 },
    { 0xa683cc21, 33 },
    { 0, 0 },
    { 0xf2f3cc24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc3cc29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc913cc2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e3cc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d33cc37, 76 },
    { 0x9333cc37, 61 },
    { 0xac53cc39, 95 },
    { 0, 0 },
    { 0x46a3cc3b, 95 },
    { 0, 0 },
    { 0x3153cc3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc513cc42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc223cc48, 23 },
    { 0x34e3cc49, 33 },
    { 0xe523cc49, 23 },
    { 0, 0 },
    { 0xc33cc4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5903cc51, 98 },
    { 0x2953cc52, 17 },
    { 0x2083cc53, 90 },
    { 0x4363cc53, 95 },
    { 0xb8c3cc55, 90 },
    { 0x5453cc56, 68 },
    { 0xce03cc51, 99 },
    { 0x2713cc58, 93 },
    { 0xa4e3cc56, 84 },
    { 0, 0 },
    { 0x6723cc5b, 99 },
    { 0x8de3cc5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf93cc5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe083cc63, 88 },
    { 0x4493cc64, 78 },
    { 0xc883cc65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b93cc6a, 90 },
    { 0xbb03cc6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa533cc72, 56 },
    { 0xc993cc73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3cc76, 88 },
    { 0x54e3cc77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333cc7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3cc8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03cc93, 33 },
    { 0x58b3cc93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6203cc97, 9 },
    { 0x26d3cc98, 98 },
    { 0xeb63cc97, 83 },
    { 0xb4b3cc99, 95 },
    { 0x1f3cc98, 55 },
    { 0x753cc9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f3cc9f, 76 },
    { 0xe5a3cca0, 33 },
    { 0x2f33cc9f, 33 },
    { 0xf0a3cca0, 66 },
    { 0x5993cca3, 95 },
    { 0x9823cca4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3ccae, 61 },
    { 0, 0 },
    { 0x1bb3ccb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d93ccb5, 84 },
    { 0xbec3ccb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb3ccbc, 33 },
    { 0, 0 },
    { 0x44a3ccbe, 65 },
    { 0, 0 },
    { 0xce93ccc0, 90 },
    { 0x9c43ccc1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d93ccc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3ccc9, 90 },
    { 0xb943ccca, 61 },
    { 0x5763cccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4423cccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c3ccd5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe503ccd9, 77 },
    { 0x5f3ccda, 23 },
    { 0x3253ccda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf63cce3, 55 },
    { 0xf113cce4, 43 },
    { 0xed93cce5, 46 },
    { 0xf6a3cce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2933cce9, 61 },
    { 0, 0 },
    { 0x7d53cceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb833ccee, 33 },
    { 0, 0 },
    { 0x7773ccf0, 22 },
    { 0x1083ccf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553ccf7, 23 },
    { 0x1ec3ccf8, 66 },
    { 0, 0 },
    { 0xdb43ccfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3ccfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d3cd00, 90 },
    { 0, 0 },
    { 0xa753cd02, 90 },
    { 0xf693cd03, 33 },
    { 0, 0 },
    { 0x33b3cd05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3cd0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f83cd11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb73cd14, 87 },
    { 0xd4e3cd15, 23 },
    { 0, 0 },
    { 0xbeb3cd17, 33 },
    { 0x40e3cd18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93cd1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb83cd21, 47 },
    { 0x2c23cd21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7363cd25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7003cd28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6103cd2c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6843cd2f, 33 },
    { 0, 0 },
    { 0x4bc3cd31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf3cd34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca3cd37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf13cd3b, 90 },
    { 0x90c3cd3b, 90 },
    { 0x28c3cd3d, 76 },
    { 0x7d3cd3e, 66 },
    { 0x1a73cd3b, 33 },
    { 0x2f03cd40, 33 },
    { 0, 0 },
    { 0x61f3cd42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e43cd48, 61 },
    { 0x9b43cd48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba3cd4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3cd55, 23 },
    { 0, 0 },
    { 0x67d3cd57, 34 },
    { 0x73cd57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd973cd5d, 33 },
    { 0x9aa3cd5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd623cd61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43cd67, 90 },
    { 0xb4d3cd67, 34 },
    { 0xb823cd69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf733cd6d, 98 },
    { 0x8c53cd6d, 61 },
    { 0x3ca3cd6f, 76 },
    { 0x3353cd70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe643cd74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb253cd79, 90 },
    { 0x8bb3cd7a, 98 },
    { 0xd543cd79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3cd7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273cd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a03cd85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703cd8b, 23 },
    { 0, 0 },
    { 0x3fb3cd8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333cd90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753cd99, 61 },
    { 0, 0 },
    { 0xac63cd9b, 66 },
    { 0, 0 },
    { 0x3e73cd9d, 88 },
    { 0, 0 },
    { 0x11a3cd9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5043cda2, 24 },
    { 0, 0 },
    { 0xeca3cda4, 90 },
    { 0x713cda5, 55 },
    { 0xe6a3cda5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3cdab, 95 },
    { 0, 0 },
    { 0x5493cdad, 97 },
    { 0, 0 },
    { 0xb123cdaf, 61 },
    { 0, 0 },
    { 0xbaf3cdb1, 90 },
    { 0, 0 },
    { 0xb0c3cdb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483cdb6, 23 },
    { 0x6c93cdb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea63cdbf, 23 },
    { 0xf2a3cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3cdc4, 95 },
    { 0x3943cdc5, 91 },
    { 0x6ee3cdc4, 87 },
    { 0, 0 },
    { 0xccd3cdc8, 33 },
    { 0x2763cdc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c3cdce, 78 },
    { 0xc153cdcf, 90 },
    { 0xd3d3cdd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb3cdd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43cdd9, 33 },
    { 0x1593cdda, 98 },
    { 0xbf43cddb, 95 },
    { 0xa2a3cddb, 61 },
    { 0, 0 },
    { 0xab73cdde, 61 },
    { 0xe143cddf, 65 },
    { 0, 0 },
    { 0x3ca3cde1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c3cde7, 61 },
    { 0x7713cde8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253cdeb, 33 },
    { 0xe583cdec, 90 },
    { 0xede3cdeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf23cdf1, 99 },
    { 0xeb03cdf2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e03cdf5, 95 },
    { 0xe453cdf6, 49 },
    { 0, 0 },
    { 0x8453cdf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d03cdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5413ce00, 80 },
    { 0x283ce01, 33 },
    { 0, 0 },
    { 0x1d53ce03, 61 },
    { 0, 0 },
    { 0x9433ce05, 61 },
    { 0xfd53ce06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ce0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd223ce16, 34 },
    { 0, 0 },
    { 0x38b3ce18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed33ce1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223ce22, 52 },
    { 0xef23ce22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e63ce26, 66 },
    { 0xfdb3ce27, 33 },
    { 0xe793ce27, 85 },
    { 0x73d3ce29, 38 },
    { 0, 0 },
    { 0x20c3ce2b, 98 },
    { 0x8be3ce2c, 98 },
    { 0, 0 },
    { 0xee83ce2e, 95 },
    { 0xa8c3ce2e, 95 },
    { 0, 0 },
    { 0x7a3ce31, 90 },
    { 0xf083ce31, 68 },
    { 0x83d3ce33, 34 },
    { 0, 0 },
    { 0xfe43ce35, 23 },
    { 0xa933ce35, 84 },
    { 0xe613ce36, 90 },
    { 0x2763ce38, 98 },
    { 0x1383ce39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383ce3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e93ce3f, 55 },
    { 0xa123ce40, 67 },
    { 0x8443ce41, 23 },
    { 0x9e63ce40, 95 },
    { 0xe513ce3f, 23 },
    { 0x4803ce3f, 61 },
    { 0, 0 },
    { 0x1f53ce46, 61 },
    { 0x37e3ce47, 61 },
    { 0x5283ce48, 67 },
    { 0x8163ce46, 95 },
    { 0, 0 },
    { 0xfac3ce4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd73ce51, 88 },
    { 0xe283ce52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd993ce5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213ce60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df3ce63, 61 },
    { 0, 0 },
    { 0x7af3ce65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43ce69, 84 },
    { 0xf8e3ce6a, 76 },
    { 0, 0 },
    { 0xb8b3ce6c, 33 },
    { 0xe453ce6d, 78 },
    { 0xecb3ce6d, 46 },
    { 0xd893ce6f, 88 },
    { 0x4a33ce6e, 61 },
    { 0xe443ce6f, 33 },
    { 0x5353ce72, 90 },
    { 0x6fd3ce73, 55 },
    { 0xbdc3ce74, 55 },
    { 0, 0 },
    { 0xcc3ce76, 78 },
    { 0xd553ce76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa3ce7b, 76 },
    { 0xb873ce7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993ce85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb363ce8c, 33 },
    { 0xd613ce8d, 100 },
    { 0xb683ce8e, 55 },
    { 0, 0 },
    { 0xf13ce90, 33 },
    { 0, 0 },
    { 0xbc63ce92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3ce9a, 34 },
    { 0x5dd3ce9b, 33 },
    { 0xa393ce9c, 81 },
    { 0x5683ce9b, 23 },
    { 0x2503ce9e, 61 },
    { 0xf843ce9f, 84 },
    { 0xa613ce9f, 90 },
    { 0xf0b3cea1, 66 },
    { 0xe803cea0, 96 },
    { 0, 0 },
    { 0x1e43cea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5783ceab, 98 },
    { 0, 0 },
    { 0xb7c3cead, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73ceb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23ceb5, 93 },
    { 0x9d43ceb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153ceb9, 95 },
    { 0x9e53ceb9, 55 },
    { 0xfd73cebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4853cebe, 85 },
    { 0xbc03cebf, 99 },
    { 0x5363cebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7813cec3, 34 },
    { 0, 0 },
    { 0xf7f3cec5, 98 },
    { 0, 0 },
    { 0x6c3cec7, 23 },
    { 0x8d33cec8, 43 },
    { 0x4a43cec8, 23 },
    { 0xe643ceca, 84 },
    { 0x8143cec8, 55 },
    { 0, 0 },
    { 0xbd03cecd, 61 },
    { 0xf083cecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf23ced4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f3ced7, 63 },
    { 0x3473ced8, 98 },
    { 0, 0 },
    { 0x4633ceda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa063cee0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8663cee3, 46 },
    { 0x19c3cee3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a93cee7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7763ceec, 34 },
    { 0, 0 },
    { 0x5213ceee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813cef5, 61 },
    { 0, 0 },
    { 0xcab3cef7, 55 },
    { 0xc8f3cef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab3cefb, 95 },
    { 0x65f3cefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153ceff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a83cf03, 77 },
    { 0, 0 },
    { 0x7b13cf05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd3cf09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc3cf0c, 33 },
    { 0x31c3cf0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9393cf12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf423cf18, 95 },
    { 0, 0 },
    { 0x93cf1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd3cf21, 61 },
    { 0xc483cf21, 90 },
    { 0x68a3cf22, 88 },
    { 0x66b3cf24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe623cf27, 61 },
    { 0xcb23cf27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e3cf2b, 38 },
    { 0x38a3cf2c, 66 },
    { 0xb0c3cf2c, 33 },
    { 0xa333cf2e, 76 },
    { 0, 0 },
    { 0xf8a3cf30, 91 },
    { 0x2e3cf31, 66 },
    { 0x77c3cf32, 46 },
    { 0xbc53cf31, 91 },
    { 0x8343cf33, 94 },
    { 0xa8f3cf32, 33 },
    { 0x7043cf36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b3cf39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c73cf3d, 90 },
    { 0, 0 },
    { 0x6823cf3f, 91 },
    { 0x42c3cf40, 90 },
    { 0x9a13cf3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3cf46, 90 },
    { 0, 0 },
    { 0x6c53cf48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d3cf4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde73cf52, 55 },
    { 0, 0 },
    { 0x4a33cf54, 95 },
    { 0x9713cf55, 98 },
    { 0xfa23cf56, 76 },
    { 0xab3cf56, 87 },
    { 0, 0 },
    { 0x8633cf59, 51 },
    { 0, 0 },
    { 0xafb3cf5b, 52 },
    { 0x57e3cf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c3cf61, 46 },
    { 0x6373cf61, 36 },
    { 0x3803cf63, 99 },
    { 0xeac3cf63, 93 },
    { 0xed93cf65, 66 },
    { 0xe0b3cf65, 80 },
    { 0, 0 },
    { 0xbae3cf68, 33 },
    { 0, 0 },
    { 0x8ad3cf6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83cf74, 90 },
    { 0x5e73cf75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d3cf78, 40 },
    { 0xc063cf79, 33 },
    { 0xfbe3cf79, 33 },
    { 0xa373cf78, 33 },
    { 0, 0 },
    { 0xd313cf7d, 68 },
    { 0x493cf7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793cf81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4893cf86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913cf8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe123cf91, 69 },
    { 0, 0 },
    { 0x9a33cf93, 99 },
    { 0xd283cf93, 90 },
    { 0xd293cf95, 33 },
    { 0, 0 },
    { 0x9ef3cf97, 23 },
    { 0xf703cf98, 95 },
    { 0, 0 },
    { 0xf143cf9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3cf9d, 90 },
    { 0xe1e3cf9e, 93 },
    { 0x83f3cf9f, 43 },
    { 0x2133cf9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7813cfa3, 90 },
    { 0x7a03cfa3, 95 },
    { 0x8ba3cfa4, 90 },
    { 0xa523cfa6, 34 },
    { 0xffb3cfa6, 76 },
    { 0x6733cfa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3cfad, 95 },
    { 0x77f3cfae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b3cfb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6833cfb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa173cfbe, 88 },
    { 0, 0 },
    { 0xc893cfc0, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba43cfc4, 95 },
    { 0xac73cfc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903cfc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff3cfcb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d3cfcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f3cfd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3cfda, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3cfdf, 33 },
    { 0x72d3cfdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43cfe7, 66 },
    { 0x3973cfe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9843cfeb, 51 },
    { 0x1033cfeb, 98 },
    { 0x93cfed, 83 },
    { 0x56e3cfed, 66 },
    { 0xca13cfef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673cff2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4043cff6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e3cffc, 43 },
    { 0xbd83cffc, 33 },
    { 0x35a3cffc, 100 },
    { 0x6ad3cfff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5213d003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b83d007, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da3d00b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7013d010, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393d013, 34 },
    { 0, 0 },
    { 0x9693d015, 77 },
    { 0, 0 },
    { 0xe483d017, 61 },
    { 0, 0 },
    { 0x1243d019, 88 },
    { 0xc443d01a, 45 },
    { 0, 0 },
    { 0xfb63d01c, 43 },
    { 0xe013d01c, 77 },
    { 0xca13d01e, 61 },
    { 0, 0 },
    { 0xb273d020, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee3d024, 33 },
    { 0, 0 },
    { 0xf203d026, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3d029, 98 },
    { 0x3363d02a, 40 },
    { 0, 0 },
    { 0xfde3d02c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba3d030, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23d034, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9223d037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63d03a, 76 },
    { 0, 0 },
    { 0xbfe3d03c, 55 },
    { 0xa3f3d03d, 93 },
    { 0x6b23d03d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543d041, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3d045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53d04b, 90 },
    { 0, 0 },
    { 0x64f3d04d, 34 },
    { 0xba53d04e, 98 },
    { 0x1133d04f, 55 },
    { 0xcb63d04f, 93 },
    { 0, 0 },
    { 0xe43d052, 23 },
    { 0x2503d053, 87 },
    { 0x6aa3d054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3d058, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d05e, 77 },
    { 0xb3a3d05f, 98 },
    { 0, 0 },
    { 0x7d13d061, 90 },
    { 0xd8d3d062, 34 },
    { 0xcc43d062, 46 },
    { 0, 0 },
    { 0x1d03d065, 34 },
    { 0x14b3d065, 23 },
    { 0x3ff3d067, 90 },
    { 0x9b3d068, 55 },
    { 0x2d33d069, 23 },
    { 0, 0 },
    { 0x4753d06b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2573d06f, 98 },
    { 0x5773d06f, 43 },
    { 0xca13d06f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8273d07b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe433d082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4663d08a, 33 },
    { 0x2473d08b, 95 },
    { 0x2c73d08c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc63d08f, 23 },
    { 0x6bc3d090, 33 },
    { 0xe883d091, 95 },
    { 0x1a23d092, 90 },
    { 0xcb13d093, 61 },
    { 0x9e73d090, 90 },
    { 0, 0 },
    { 0xfa13d096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3d099, 23 },
    { 0x8e53d09a, 65 },
    { 0xa0a3d099, 67 },
    { 0, 0 },
    { 0xf8a3d09d, 33 },
    { 0, 0 },
    { 0xab03d09f, 75 },
    { 0x1af3d0a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3d0a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43d0ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f3d0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3573d0b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8853d0bf, 90 },
    { 0xe193d0c0, 88 },
    { 0x803d0bf, 77 },
    { 0x8803d0c0, 33 },
    { 0x7a83d0bf, 23 },
    { 0x37f3d0c2, 98 },
    { 0x8843d0c2, 85 },
    { 0x45f3d0c6, 61 },
    { 0, 0 },
    { 0x8333d0c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83d0cb, 95 },
    { 0, 0 },
    { 0xfb3d0cd, 43 },
    { 0, 0 },
    { 0x7263d0cf, 52 },
    { 0x6b23d0d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a83d0d3, 90 },
    { 0xfb63d0d3, 33 },
    { 0x44a3d0d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a3d0d8, 33 },
    { 0xc513d0d8, 23 },
    { 0, 0 },
    { 0x34c3d0db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb93d0de, 78 },
    { 0, 0 },
    { 0x3ba3d0e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23d0e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4183d0e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc3d0ee, 88 },
    { 0, 0 },
    { 0x3b33d0f0, 33 },
    { 0x2633d0f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea3d0f4, 43 },
    { 0x66d3d0f5, 61 },
    { 0x9fd3d0f5, 90 },
    { 0x98d3d0f6, 47 },
    { 0x36e3d0f6, 91 },
    { 0x843d0f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd323d0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf003d100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e13d104, 46 },
    { 0x5393d104, 91 },
    { 0, 0 },
    { 0x93a3d107, 85 },
    { 0xf693d107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3d10c, 100 },
    { 0x4043d10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e93d111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c3d114, 88 },
    { 0, 0 },
    { 0x2733d116, 55 },
    { 0x7193d117, 55 },
    { 0xa753d118, 23 },
    { 0xf863d118, 71 },
    { 0xdbe3d117, 33 },
    { 0xa5c3d116, 90 },
    { 0, 0 },
    { 0xeb83d11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3d121, 78 },
    { 0x8ec3d122, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd873d125, 66 },
    { 0xe863d126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793d12c, 61 },
    { 0xc2e3d12d, 93 },
    { 0x8263d12d, 61 },
    { 0, 0 },
    { 0x7de3d130, 34 },
    { 0xb5d3d131, 99 },
    { 0, 0 },
    { 0x4f3d133, 76 },
    { 0, 0 },
    { 0xb783d135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3d138, 95 },
    { 0xcd43d139, 34 },
    { 0xa443d13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf3d142, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a03d145, 95 },
    { 0, 0 },
    { 0x9493d147, 98 },
    { 0xf023d147, 33 },
    { 0xdcc3d149, 23 },
    { 0x5be3d148, 100 },
    { 0, 0 },
    { 0x9183d14c, 23 },
    { 0, 0 },
    { 0xbf13d14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3993d151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec3d154, 98 },
    { 0, 0 },
    { 0x8763d156, 90 },
    { 0, 0 },
    { 0x55b3d158, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8163d15f, 98 },
    { 0x2323d15f, 98 },
    { 0x4f83d15f, 33 },
    { 0xda23d15f, 98 },
    { 0x1b93d163, 55 },
    { 0x32f3d164, 66 },
    { 0, 0 },
    { 0x7db3d166, 76 },
    { 0x91e3d167, 95 },
    { 0, 0 },
    { 0xe2f3d169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9513d16f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3d173, 95 },
    { 0, 0 },
    { 0x9563d175, 81 },
    { 0x7413d175, 100 },
    { 0x7f33d177, 93 },
    { 0x3843d175, 90 },
    { 0x8423d179, 95 },
    { 0, 0 },
    { 0xcf73d17b, 23 },
    { 0xc933d17c, 23 },
    { 0x32c3d17c, 98 },
    { 0x9b3d17d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e3d182, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb353d186, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3d190, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af3d194, 61 },
    { 0, 0 },
    { 0xf953d196, 35 },
    { 0x8e83d197, 95 },
    { 0, 0 },
    { 0xccd3d199, 87 },
    { 0xde03d19a, 76 },
    { 0x6d33d199, 43 },
    { 0x8c13d19c, 33 },
    { 0x29f3d19c, 88 },
    { 0xa4e3d19d, 95 },
    { 0x1883d19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b3d1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9603d1ab, 90 },
    { 0xf513d1ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23d1b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa63d1bf, 98 },
    { 0xcd13d1c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503d1c3, 77 },
    { 0, 0 },
    { 0xfa63d1c5, 87 },
    { 0xd783d1c6, 100 },
    { 0, 0 },
    { 0x3d03d1c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5393d1ce, 33 },
    { 0xda23d1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3d1d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43d1d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a3d1db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad3d1de, 90 },
    { 0xcec3d1df, 33 },
    { 0x9943d1e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6613d1e3, 93 },
    { 0xbb93d1e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb3d1e7, 52 },
    { 0x7363d1e8, 76 },
    { 0, 0 },
    { 0xa453d1ea, 95 },
    { 0, 0 },
    { 0x8263d1ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3d1f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef33d1f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd253d1f7, 86 },
    { 0x4643d1f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc3d1fd, 23 },
    { 0, 0 },
    { 0xc283d1ff, 34 },
    { 0, 0 },
    { 0x2e83d201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903d20b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33d20e, 55 },
    { 0xfad3d20f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee3d212, 91 },
    { 0x6523d212, 98 },
    { 0, 0 },
    { 0x5a73d215, 100 },
    { 0xf2a3d216, 61 },
    { 0x9ba3d216, 67 },
    { 0xb803d218, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3d220, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3d225, 83 },
    { 0, 0 },
    { 0x1813d227, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133d22f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53d232, 66 },
    { 0, 0 },
    { 0x4f93d234, 34 },
    { 0, 0 },
    { 0xc603d236, 33 },
    { 0, 0 },
    { 0xbf23d238, 68 },
    { 0, 0 },
    { 0x65d3d23a, 33 },
    { 0, 0 },
    { 0xd203d23c, 98 },
    { 0xdf03d23d, 76 },
    { 0x7333d23e, 85 },
    { 0x3f13d23e, 95 },
    { 0x76c3d240, 95 },
    { 0, 0 },
    { 0xc9e3d242, 23 },
    { 0, 0 },
    { 0x5643d244, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf33d249, 91 },
    { 0x2cf3d24a, 23 },
    { 0xf423d24b, 97 },
    { 0x8f33d24c, 46 },
    { 0xd8c3d24d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf3d250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd713d254, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53d257, 95 },
    { 0x21b3d258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7193d25b, 98 },
    { 0xf8c3d25b, 23 },
    { 0xba3d25b, 43 },
    { 0x8923d25e, 78 },
    { 0xb6b3d25f, 43 },
    { 0xb443d260, 61 },
    { 0x1cd3d261, 43 },
    { 0x9303d25f, 95 },
    { 0xf123d263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3d266, 87 },
    { 0xfbe3d267, 90 },
    { 0x91f3d268, 90 },
    { 0x47b3d268, 98 },
    { 0x5843d26a, 95 },
    { 0x95c3d268, 90 },
    { 0x2f63d26b, 90 },
    { 0x7033d26d, 43 },
    { 0xa993d26e, 23 },
    { 0x7f33d26b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483d278, 22 },
    { 0x25f3d278, 66 },
    { 0xa793d278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33d27f, 23 },
    { 0x36c3d280, 23 },
    { 0x4d73d280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d287, 34 },
    { 0x8e23d288, 61 },
    { 0x2ce3d289, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3d291, 95 },
    { 0, 0 },
    { 0xeff3d293, 33 },
    { 0, 0 },
    { 0x7ec3d295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f3d299, 76 },
    { 0x2b93d299, 90 },
    { 0xf1a3d29b, 20 },
    { 0xb853d29c, 23 },
    { 0xd13d299, 33 },
    { 0xa4f3d29e, 90 },
    { 0xb23d29f, 90 },
    { 0x90c3d29f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f33d2a4, 76 },
    { 0x4143d2a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9523d2ab, 34 },
    { 0x3d2ac, 34 },
    { 0xa243d2ab, 95 },
    { 0x74d3d2ae, 99 },
    { 0xcb73d2af, 23 },
    { 0x5ea3d2b0, 46 },
    { 0x3a53d2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xada3d2b4, 46 },
    { 0x9a33d2b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3d2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb03d2c1, 55 },
    { 0, 0 },
    { 0x9603d2c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3063d2d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d2d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc263d2da, 23 },
    { 0, 0 },
    { 0x62e3d2dc, 95 },
    { 0x78c3d2dd, 77 },
    { 0, 0 },
    { 0xa263d2df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f3d2e4, 99 },
    { 0, 0 },
    { 0x71d3d2e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903d2ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6743d2f0, 33 },
    { 0x3203d2f1, 33 },
    { 0, 0 },
    { 0x1c83d2f3, 90 },
    { 0x1633d2f4, 88 },
    { 0x3c83d2f4, 23 },
    { 0x8dc3d2f6, 95 },
    { 0xa153d2f6, 23 },
    { 0xad03d2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef3d2fd, 66 },
    { 0xa393d2fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3d301, 43 },
    { 0, 0 },
    { 0x2f93d303, 89 },
    { 0, 0 },
    { 0xf6b3d305, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a83d30d, 90 },
    { 0xe8b3d30e, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9513d312, 23 },
    { 0x4813d313, 47 },
    { 0x4533d313, 95 },
    { 0, 0 },
    { 0x41a3d316, 23 },
    { 0, 0 },
    { 0xb133d318, 95 },
    { 0xced3d318, 33 },
    { 0xf593d31a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3d321, 33 },
    { 0xa5d3d322, 90 },
    { 0x9ef3d323, 61 },
    { 0, 0 },
    { 0xb333d325, 75 },
    { 0x6d23d326, 9 },
    { 0x7e63d325, 61 },
    { 0xae43d328, 87 },
    { 0x5e3d327, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a43d334, 99 },
    { 0, 0 },
    { 0x1513d336, 99 },
    { 0xc83d336, 98 },
    { 0x6f3d338, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e3d33e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1693d343, 61 },
    { 0xa763d343, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3d349, 66 },
    { 0, 0 },
    { 0xbfa3d34b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3d350, 90 },
    { 0xb83d350, 95 },
    { 0x9ed3d352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9723d356, 98 },
    { 0x1fb3d357, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e3d35c, 91 },
    { 0xfb13d35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3d363, 34 },
    { 0, 0 },
    { 0x8983d365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23d36a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e3d36e, 55 },
    { 0x1cb3d36f, 61 },
    { 0x70b3d36f, 33 },
    { 0x99c3d36f, 90 },
    { 0x93c3d372, 100 },
    { 0x3a43d372, 90 },
    { 0, 0 },
    { 0x88b3d375, 95 },
    { 0x2bf3d376, 55 },
    { 0, 0 },
    { 0xbef3d378, 90 },
    { 0, 0 },
    { 0x7143d37a, 90 },
    { 0xdbe3d37b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e3d380, 98 },
    { 0, 0 },
    { 0xaf13d382, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb053d386, 33 },
    { 0xe683d387, 55 },
    { 0, 0 },
    { 0x243d389, 85 },
    { 0, 0 },
    { 0xef3d38b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043d38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3d39d, 77 },
    { 0xe573d39d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c3d3a5, 33 },
    { 0x4253d3a6, 33 },
    { 0x8fb3d3a6, 43 },
    { 0x7293d3a8, 101 },
    { 0xf2e3d3a6, 75 },
    { 0, 0 },
    { 0xe443d3ab, 22 },
    { 0, 0 },
    { 0x2463d3ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8683d3b1, 77 },
    { 0, 0 },
    { 0xbb03d3b3, 90 },
    { 0x33a3d3b4, 34 },
    { 0, 0 },
    { 0x7d73d3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d63d3b9, 78 },
    { 0, 0 },
    { 0x1e43d3bb, 90 },
    { 0xbda3d3bc, 55 },
    { 0xfc03d3bd, 61 },
    { 0xe453d3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4033d3c3, 66 },
    { 0, 0 },
    { 0x9953d3c5, 23 },
    { 0x8dd3d3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c53d3cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa3d3ce, 55 },
    { 0x2583d3ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13d3d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf3d3de, 90 },
    { 0, 0 },
    { 0x9b13d3e0, 95 },
    { 0, 0 },
    { 0xbcf3d3e2, 81 },
    { 0x8fa3d3e3, 53 },
    { 0x70b3d3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc3d3e7, 61 },
    { 0xf693d3e7, 33 },
    { 0, 0 },
    { 0x7f3d3ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa983d3ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3d3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe13d3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623d3fb, 91 },
    { 0x2423d3fb, 90 },
    { 0, 0 },
    { 0x3013d3fe, 76 },
    { 0x3e13d3ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f73d402, 90 },
    { 0x2a63d402, 17 },
    { 0xc4b3d402, 93 },
    { 0xcec3d402, 98 },
    { 0xff03d406, 75 },
    { 0xa7c3d406, 46 },
    { 0x2163d408, 46 },
    { 0x6c33d408, 90 },
    { 0xf4a3d406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f3d40e, 98 },
    { 0x9923d40e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf303d413, 90 },
    { 0x3793d414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d417, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e3d41a, 95 },
    { 0x5d03d41b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663d41f, 81 },
    { 0xba43d41f, 99 },
    { 0xdc03d421, 67 },
    { 0xfad3d41f, 87 },
    { 0xd9b3d422, 98 },
    { 0xc403d422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc3d427, 34 },
    { 0x8ca3d428, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d3d42d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b3d433, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8813d437, 91 },
    { 0x3e33d438, 23 },
    { 0, 0 },
    { 0x5353d43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e73d43d, 66 },
    { 0x8723d43e, 33 },
    { 0xcd43d43f, 61 },
    { 0xba23d440, 33 },
    { 0xb143d441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc053d446, 52 },
    { 0xf743d446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d3d454, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d3d457, 98 },
    { 0, 0 },
    { 0xe063d459, 90 },
    { 0x7c73d45a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5463d45d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3d467, 43 },
    { 0, 0 },
    { 0x86d3d469, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd633d46c, 90 },
    { 0xb643d46d, 40 },
    { 0xa113d46e, 55 },
    { 0xd003d46f, 77 },
    { 0x4623d470, 95 },
    { 0xfd3d470, 76 },
    { 0x61d3d470, 33 },
    { 0, 0 },
    { 0xe193d474, 90 },
    { 0xc013d475, 98 },
    { 0xe7b3d476, 33 },
    { 0xfd13d477, 77 },
    { 0x1e83d475, 61 },
    { 0xbe33d477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf433d47d, 17 },
    { 0x1d43d47d, 95 },
    { 0x1123d47f, 66 },
    { 0, 0 },
    { 0xdf53d481, 52 },
    { 0, 0 },
    { 0xd6c3d483, 33 },
    { 0x8123d483, 33 },
    { 0x5773d485, 78 },
    { 0x4973d486, 52 },
    { 0x1463d487, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed13d48a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c3d48e, 87 },
    { 0xe943d48f, 76 },
    { 0xf063d48e, 90 },
    { 0x2613d490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe523d496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d13d49a, 40 },
    { 0x5d83d49b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace3d4a1, 55 },
    { 0, 0 },
    { 0x8963d4a3, 76 },
    { 0xc633d4a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed93d4a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d3d4ab, 93 },
    { 0x3923d4ac, 90 },
    { 0x1f83d4ab, 85 },
    { 0xe053d4ad, 61 },
    { 0x8d53d4ae, 61 },
    { 0xc3d3d4ab, 93 },
    { 0x8ab3d4b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb3d4b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973d4b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43d4bf, 98 },
    { 0xd273d4c0, 66 },
    { 0x8cb3d4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce3d4c6, 98 },
    { 0x9e53d4c6, 95 },
    { 0x43d4c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8893d4cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b13d4d1, 33 },
    { 0xa9a3d4d2, 61 },
    { 0, 0 },
    { 0xa333d4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe883d4d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe33d4de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843d4ec, 91 },
    { 0xc933d4ed, 55 },
    { 0x5ec3d4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7723d4f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d3d4f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3153d4fa, 95 },
    { 0, 0 },
    { 0x4773d4fc, 61 },
    { 0x6ec3d4fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a53d500, 81 },
    { 0, 0 },
    { 0x4e93d502, 61 },
    { 0xa5b3d503, 33 },
    { 0x4263d504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053d508, 23 },
    { 0x5b13d508, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xced3d50c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8453d50f, 98 },
    { 0x88e3d510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493d513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb23d51e, 90 },
    { 0x3f3d51f, 90 },
    { 0x5c83d51f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9053d525, 98 },
    { 0x5a23d525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c53d529, 91 },
    { 0, 0 },
    { 0x45a3d52b, 95 },
    { 0x77b3d52c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893d530, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d23d539, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773d53c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb363d53f, 34 },
    { 0, 0 },
    { 0xca43d541, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4593d544, 33 },
    { 0xeaf3d545, 90 },
    { 0, 0 },
    { 0x1c83d547, 61 },
    { 0, 0 },
    { 0x6763d549, 56 },
    { 0xd1e3d54a, 61 },
    { 0x4d73d549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4773d550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3303d554, 78 },
    { 0xfaf3d555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c53d561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c3d564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc73d568, 98 },
    { 0, 0 },
    { 0x5f33d56a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63d56e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb3d571, 23 },
    { 0xc993d572, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd903d578, 76 },
    { 0, 0 },
    { 0x67a3d57a, 34 },
    { 0x5763d57b, 98 },
    { 0, 0 },
    { 0x14e3d57d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d3d580, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1253d585, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc493d58c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3d594, 33 },
    { 0x4ab3d595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa983d598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad03d59b, 95 },
    { 0, 0 },
    { 0x74a3d59d, 52 },
    { 0x1c03d59d, 33 },
    { 0x6773d59f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d3d5a2, 23 },
    { 0x5f63d5a3, 98 },
    { 0x363d5a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23d5a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43d5ac, 78 },
    { 0, 0 },
    { 0x5be3d5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde63d5b3, 95 },
    { 0xdf83d5b4, 52 },
    { 0x7643d5b3, 33 },
    { 0x6d3d5b6, 61 },
    { 0x3773d5b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd3d5be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a3d5c6, 76 },
    { 0, 0 },
    { 0xe363d5c8, 87 },
    { 0xe113d5c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3d5cd, 17 },
    { 0, 0 },
    { 0x9d33d5cf, 33 },
    { 0xe993d5d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d03d5d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf863d5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa803d5db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6323d5df, 67 },
    { 0xc853d5df, 90 },
    { 0x1873d5e0, 95 },
    { 0xf293d5e2, 90 },
    { 0xe873d5e0, 95 },
    { 0xcb83d5e0, 98 },
    { 0xe303d5e5, 33 },
    { 0xc7a3d5e2, 23 },
    { 0xeb73d5e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b3d5eb, 23 },
    { 0, 0 },
    { 0xb8f3d5ed, 90 },
    { 0xc143d5ee, 61 },
    { 0x4fb3d5ee, 23 },
    { 0x5923d5ee, 85 },
    { 0xdcf3d5f1, 42 },
    { 0x69e3d5ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4383d5f5, 34 },
    { 0x73a3d5f6, 98 },
    { 0x9d43d5f5, 98 },
    { 0x5aa3d5f6, 34 },
    { 0x46f3d5f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813d5fe, 81 },
    { 0x1ed3d5ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3d602, 33 },
    { 0xc3f3d603, 33 },
    { 0x5483d604, 84 },
    { 0xcfb3d605, 76 },
    { 0xeb53d606, 90 },
    { 0x97e3d606, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5183d60f, 90 },
    { 0, 0 },
    { 0xac53d611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d43d620, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7693d626, 33 },
    { 0x7e93d626, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc543d62a, 34 },
    { 0x48a3d62a, 96 },
    { 0, 0 },
    { 0x6623d62d, 98 },
    { 0xd8c3d62d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183d632, 90 },
    { 0xc563d633, 76 },
    { 0x3683d634, 33 },
    { 0x3e63d635, 98 },
    { 0x78e3d633, 98 },
    { 0x5453d637, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c43d63c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c73d63f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b33d646, 87 },
    { 0, 0 },
    { 0x92e3d648, 80 },
    { 0x4133d649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3d64f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b3d652, 98 },
    { 0, 0 },
    { 0x55c3d654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3d66b, 90 },
    { 0x5703d66b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f3d670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e3d677, 76 },
    { 0xc373d677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb3d67b, 33 },
    { 0x8b03d67c, 23 },
    { 0x99d3d67d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d3d680, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2873d686, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f3d68c, 100 },
    { 0, 0 },
    { 0x3e73d68e, 52 },
    { 0x983d68f, 95 },
    { 0x26d3d690, 76 },
    { 0x5063d68f, 93 },
    { 0, 0 },
    { 0xdad3d693, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5773d699, 55 },
    { 0, 0 },
    { 0xf3d69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3d69e, 95 },
    { 0x27f3d69f, 95 },
    { 0xac33d69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb843d6a6, 43 },
    { 0x6183d6a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e3d6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623d6ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43d6b0, 34 },
    { 0, 0 },
    { 0x75f3d6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3393d6b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad93d6bf, 90 },
    { 0xb0a3d6c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3d6c3, 100 },
    { 0x8433d6c4, 95 },
    { 0xca03d6c5, 95 },
    { 0xb3d3d6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673d6ca, 43 },
    { 0x2073d6cb, 23 },
    { 0x6f43d6cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073d6d0, 31 },
    { 0, 0 },
    { 0x6603d6d2, 90 },
    { 0, 0 },
    { 0xbbe3d6d4, 61 },
    { 0xbb93d6d4, 46 },
    { 0x6113d6d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7633d6d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53d6df, 23 },
    { 0x3293d6e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc3d6eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc3d6ee, 78 },
    { 0x5893d6ef, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea33d6f5, 67 },
    { 0x94b3d6f5, 90 },
    { 0xfa23d6f7, 87 },
    { 0x2553d6f6, 93 },
    { 0x75a3d6f9, 85 },
    { 0x5583d6fa, 88 },
    { 0xc183d6fb, 95 },
    { 0, 0 },
    { 0x4273d6fd, 61 },
    { 0, 0 },
    { 0x54d3d6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813d70a, 76 },
    { 0x23d70b, 55 },
    { 0x1803d70c, 78 },
    { 0x61a3d70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba93d711, 34 },
    { 0xd603d712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93d716, 95 },
    { 0xf603d717, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f83d71a, 67 },
    { 0, 0 },
    { 0x71d3d71c, 90 },
    { 0x27a3d71c, 23 },
    { 0xa683d71c, 90 },
    { 0, 0 },
    { 0x5dc3d720, 98 },
    { 0x20d3d721, 43 },
    { 0x54e3d722, 33 },
    { 0x103d723, 46 },
    { 0, 0 },
    { 0xc0e3d725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773d72a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b23d72d, 23 },
    { 0, 0 },
    { 0x9c93d72f, 33 },
    { 0x56c3d730, 33 },
    { 0x5023d731, 33 },
    { 0xd303d730, 76 },
    { 0xec53d732, 93 },
    { 0x7e83d734, 55 },
    { 0, 0 },
    { 0x8dc3d736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe093d739, 23 },
    { 0x5f3d73a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd463d740, 87 },
    { 0, 0 },
    { 0x6883d742, 61 },
    { 0, 0 },
    { 0x94f3d744, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233d748, 36 },
    { 0, 0 },
    { 0xc9e3d74a, 33 },
    { 0x6183d74b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad13d74e, 90 },
    { 0, 0 },
    { 0xa4c3d750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23d757, 67 },
    { 0, 0 },
    { 0x3353d759, 76 },
    { 0x993d75a, 98 },
    { 0xefb3d75b, 23 },
    { 0x90b3d75a, 23 },
    { 0, 0 },
    { 0x1513d75e, 33 },
    { 0xfa83d75e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee73d763, 80 },
    { 0, 0 },
    { 0x1873d765, 61 },
    { 0xab73d766, 40 },
    { 0x96a3d767, 98 },
    { 0, 0 },
    { 0x71a3d769, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3d76d, 52 },
    { 0x95b3d76d, 95 },
    { 0, 0 },
    { 0xb793d770, 84 },
    { 0x9573d771, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8123d77d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3d781, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d33d787, 67 },
    { 0, 0 },
    { 0x8e13d789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef3d78d, 68 },
    { 0xc673d78d, 95 },
    { 0, 0 },
    { 0x77c3d790, 52 },
    { 0xd123d790, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3d797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5403d79a, 98 },
    { 0x1af3d79b, 68 },
    { 0x61b3d79b, 33 },
    { 0, 0 },
    { 0xfdb3d79e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e3d7a5, 33 },
    { 0x20f3d7a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2113d7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33d7b9, 65 },
    { 0xc243d7b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc83d7bf, 81 },
    { 0x6973d7c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5543d7c3, 98 },
    { 0xcf13d7c3, 95 },
    { 0, 0 },
    { 0xa913d7c6, 61 },
    { 0, 0 },
    { 0x9c23d7c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333d7cc, 98 },
    { 0, 0 },
    { 0xdc3d7ce, 33 },
    { 0x64a3d7ce, 46 },
    { 0xf803d7ce, 93 },
    { 0, 0 },
    { 0xb193d7d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3d7d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3d7da, 66 },
    { 0, 0 },
    { 0xa0b3d7dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6683d7e0, 84 },
    { 0x2313d7e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3d7e5, 55 },
    { 0x5d93d7e6, 38 },
    { 0x3143d7e5, 17 },
    { 0x2263d7e8, 61 },
    { 0x68e3d7e9, 90 },
    { 0x1b43d7e9, 46 },
    { 0x9c53d7e7, 87 },
    { 0x5923d7ec, 61 },
    { 0x30e3d7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e73d7f2, 68 },
    { 0xb4a3d7f2, 33 },
    { 0, 0 },
    { 0xee13d7f5, 95 },
    { 0, 0 },
    { 0x20f3d7f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d7fa, 90 },
    { 0x7373d7fb, 90 },
    { 0xc573d7fa, 93 },
    { 0x88b3d7fd, 55 },
    { 0x6783d7fb, 53 },
    { 0xbd63d7fc, 78 },
    { 0, 0 },
    { 0xe653d801, 61 },
    { 0x1363d802, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa733d806, 33 },
    { 0, 0 },
    { 0x2ab3d808, 69 },
    { 0xec53d809, 76 },
    { 0, 0 },
    { 0xc233d80b, 95 },
    { 0xa663d80c, 61 },
    { 0xa403d80d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553d813, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa93d816, 33 },
    { 0x4863d817, 93 },
    { 0xc523d818, 33 },
    { 0xdcc3d819, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd873d820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b93d823, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3d827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313d82d, 61 },
    { 0x8123d82d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3d838, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd743d83b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573d83e, 66 },
    { 0xb623d83e, 95 },
    { 0xfc53d840, 90 },
    { 0x4fb3d840, 23 },
    { 0xbbf3d842, 33 },
    { 0x6543d841, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb3d854, 67 },
    { 0xd713d855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53d858, 61 },
    { 0x3363d859, 61 },
    { 0, 0 },
    { 0x3a3d85b, 98 },
    { 0xec93d85b, 33 },
    { 0x9dd3d85c, 38 },
    { 0x5103d85e, 93 },
    { 0xbcf3d85f, 75 },
    { 0x7e03d860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6833d868, 99 },
    { 0, 0 },
    { 0x79a3d86a, 43 },
    { 0xa153d86a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc03d86f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe923d872, 98 },
    { 0x6773d873, 61 },
    { 0x1233d872, 90 },
    { 0xebd3d875, 87 },
    { 0, 0 },
    { 0xbfa3d877, 98 },
    { 0x2d33d878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe483d87d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953d880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3d884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7263d888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd3d88f, 99 },
    { 0, 0 },
    { 0xf253d891, 23 },
    { 0, 0 },
    { 0xf773d893, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f3d89a, 66 },
    { 0, 0 },
    { 0x6eb3d89c, 33 },
    { 0xdfb3d89c, 23 },
    { 0, 0 },
    { 0xe1b3d89f, 52 },
    { 0, 0 },
    { 0xd283d8a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb3d8a4, 90 },
    { 0x9583d8a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3d8a8, 95 },
    { 0x61a3d8a9, 76 },
    { 0, 0 },
    { 0x7203d8ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb3d8b1, 91 },
    { 0xf553d8b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe433d8ba, 91 },
    { 0x42d3d8bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13d8be, 33 },
    { 0x1293d8bf, 61 },
    { 0xdd93d8bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3d8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d3d8c9, 33 },
    { 0, 0 },
    { 0x8383d8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e3d8d0, 52 },
    { 0xf5f3d8d0, 88 },
    { 0x2783d8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3d8d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7233d8db, 95 },
    { 0, 0 },
    { 0x93b3d8dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe3d8e2, 90 },
    { 0xe6a3d8e3, 23 },
    { 0x30c3d8e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3d8e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f3d8eb, 90 },
    { 0x46a3d8eb, 61 },
    { 0, 0 },
    { 0xc4c3d8ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9953d8f1, 23 },
    { 0x8893d8f1, 99 },
    { 0, 0 },
    { 0xd2c3d8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f3d8f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d8fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4843d906, 91 },
    { 0x3673d906, 99 },
    { 0x9fb3d907, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf3d90b, 33 },
    { 0x4cf3d90c, 76 },
    { 0x9ad3d90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e93d910, 34 },
    { 0xc4b3d910, 97 },
    { 0, 0 },
    { 0x9163d913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa3d917, 100 },
    { 0x1fb3d917, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8613d91b, 23 },
    { 0, 0 },
    { 0xe153d91d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7093d921, 36 },
    { 0x6163d921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe03d926, 61 },
    { 0, 0 },
    { 0x77d3d928, 98 },
    { 0x4ee3d929, 61 },
    { 0x1d33d92a, 78 },
    { 0x2ae3d929, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d3d92e, 90 },
    { 0x4763d92f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f3d933, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef3d937, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5803d93a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae3d93e, 81 },
    { 0xb833d93f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5053d943, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e3d94c, 61 },
    { 0x29b3d94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf563d951, 84 },
    { 0, 0 },
    { 0x2183d953, 8 },
    { 0x7ab3d954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d95b, 95 },
    { 0x20a3d95c, 33 },
    { 0xaea3d95d, 23 },
    { 0xf923d95e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983d962, 61 },
    { 0xa023d962, 61 },
    { 0xa803d963, 61 },
    { 0xf383d965, 93 },
    { 0xd453d966, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893d96a, 23 },
    { 0xabe3d96b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7523d96e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893d972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e3d975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb3d97a, 61 },
    { 0x1493d97b, 90 },
    { 0, 0 },
    { 0x19a3d97d, 46 },
    { 0, 0 },
    { 0xfaa3d97f, 95 },
    { 0xd753d97f, 33 },
    { 0, 0 },
    { 0x9d33d982, 95 },
    { 0x7233d982, 33 },
    { 0xb753d984, 33 },
    { 0, 0 },
    { 0x6a23d986, 33 },
    { 0, 0 },
    { 0x6833d988, 43 },
    { 0, 0 },
    { 0x20a3d98a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6493d98d, 90 },
    { 0x8f13d98e, 61 },
    { 0x6d23d98f, 98 },
    { 0xeae3d990, 90 },
    { 0xcfd3d991, 33 },
    { 0x6ca3d992, 33 },
    { 0x7c33d993, 55 },
    { 0x55d3d994, 81 },
    { 0x2f23d995, 95 },
    { 0, 0 },
    { 0xfb83d997, 77 },
    { 0x1f23d998, 61 },
    { 0x4303d997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d3d99d, 98 },
    { 0, 0 },
    { 0xd243d99f, 76 },
    { 0, 0 },
    { 0x9c93d9a1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6193d9a9, 95 },
    { 0xc793d9aa, 68 },
    { 0x5313d9ab, 76 },
    { 0, 0 },
    { 0x5ac3d9ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a63d9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43d9b5, 33 },
    { 0, 0 },
    { 0x6be3d9b7, 33 },
    { 0, 0 },
    { 0xf713d9b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3283d9c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763d9c5, 23 },
    { 0xd893d9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163d9cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc593d9d1, 80 },
    { 0x6f3d9d2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a53d9d9, 48 },
    { 0, 0 },
    { 0xa103d9db, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3d9e0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd483d9e3, 12 },
    { 0x77b3d9e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x693d9e7, 77 },
    { 0xc243d9e7, 34 },
    { 0x1673d9e9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e3d9ec, 91 },
    { 0x8f23d9ed, 56 },
    { 0, 0 },
    { 0x2f83d9ef, 55 },
    { 0x1a23d9f0, 95 },
    { 0, 0 },
    { 0x9ca3d9f2, 33 },
    { 0, 0 },
    { 0xce3d9f4, 76 },
    { 0, 0 },
    { 0xf853d9f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c73d9f9, 34 },
    { 0x17a3d9fa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5743d9fd, 61 },
    { 0x513d9fe, 61 },
    { 0xeb03d9ff, 66 },
    { 0x7083d9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1183da05, 90 },
    { 0xa963da06, 90 },
    { 0x9253da07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a3da0a, 43 },
    { 0, 0 },
    { 0x2c03da0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533da0f, 76 },
    { 0, 0 },
    { 0x5133da11, 90 },
    { 0x8d93da11, 43 },
    { 0xea53da13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f3da18, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c3da21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a3da29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9523da2c, 55 },
    { 0x1fc3da2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a53da34, 40 },
    { 0x7dc3da35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2843da38, 98 },
    { 0, 0 },
    { 0x59b3da3a, 87 },
    { 0x3de3da3a, 77 },
    { 0x6e63da3a, 90 },
    { 0x84b3da3b, 95 },
    { 0x4393da3b, 81 },
    { 0xe8e3da3b, 87 },
    { 0x87d3da3c, 55 },
    { 0xf2a3da3d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff3da44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe053da48, 61 },
    { 0xae63da49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3da4d, 92 },
    { 0, 0 },
    { 0xf3da4f, 23 },
    { 0x1473da4f, 95 },
    { 0x93a3da51, 75 },
    { 0x4ac3da52, 66 },
    { 0, 0 },
    { 0x36b3da54, 55 },
    { 0, 0 },
    { 0x3a33da56, 33 },
    { 0xac83da56, 96 },
    { 0, 0 },
    { 0x96b3da59, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753da5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123da62, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573da67, 90 },
    { 0xfe33da68, 61 },
    { 0x2503da67, 93 },
    { 0, 0 },
    { 0xb0b3da6b, 90 },
    { 0, 0 },
    { 0x6203da6d, 23 },
    { 0xa493da6e, 17 },
    { 0, 0 },
    { 0x44a3da70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7803da73, 61 },
    { 0x40f3da74, 75 },
    { 0x8833da74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b63da7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf263da7d, 90 },
    { 0, 0 },
    { 0x1233da7f, 96 },
    { 0, 0 },
    { 0xe023da81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793da84, 61 },
    { 0x90c3da84, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe3da88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a3da8e, 78 },
    { 0x9dd3da8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3da93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce3da97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf33da9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f3daa3, 68 },
    { 0xa253daa4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3daa7, 98 },
    { 0, 0 },
    { 0xf993daa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8093daaf, 61 },
    { 0x2cc3dab0, 23 },
    { 0x133dab0, 33 },
    { 0x5cc3dab2, 90 },
    { 0xa9d3dab2, 98 },
    { 0, 0 },
    { 0x6d83dab5, 95 },
    { 0xce13dab5, 91 },
    { 0x9243dab7, 95 },
    { 0x4643dab5, 61 },
    { 0xc123dab9, 61 },
    { 0xbb83dab9, 95 },
    { 0x56f3dabb, 46 },
    { 0x9a43dabc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023dac4, 23 },
    { 0xaa13dac5, 90 },
    { 0x38f3dac6, 78 },
    { 0x6d83dac7, 46 },
    { 0xf813dac6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93dacd, 100 },
    { 0xf413dace, 90 },
    { 0x3353dace, 95 },
    { 0x7e63dacf, 90 },
    { 0x6b63dad0, 61 },
    { 0, 0 },
    { 0x4f93dad3, 87 },
    { 0x4a3dad3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe593dad8, 91 },
    { 0x6a33dad9, 43 },
    { 0x57c3dada, 76 },
    { 0x4373dadb, 33 },
    { 0, 0 },
    { 0xb083dadd, 61 },
    { 0, 0 },
    { 0xc3b3dadf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13dae2, 55 },
    { 0x7963dae3, 61 },
    { 0xa073dae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3563dae9, 52 },
    { 0, 0 },
    { 0xc4e3daeb, 55 },
    { 0xca3daec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3daf2, 98 },
    { 0xfc93daf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453daf6, 98 },
    { 0x9e53daf7, 52 },
    { 0x3363daf7, 81 },
    { 0x2893daf9, 33 },
    { 0xf8a3daf7, 78 },
    { 0, 0 },
    { 0x5493dafc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae3db00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5383db07, 61 },
    { 0xd003db08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e43db0b, 43 },
    { 0, 0 },
    { 0xf033db0d, 87 },
    { 0, 0 },
    { 0x9ff3db0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93db12, 100 },
    { 0, 0 },
    { 0x5b83db14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993db17, 43 },
    { 0x6ed3db18, 76 },
    { 0, 0 },
    { 0x37d3db1a, 90 },
    { 0, 0 },
    { 0x4293db1c, 55 },
    { 0xee3db1d, 84 },
    { 0, 0 },
    { 0x8843db1f, 85 },
    { 0xa173db20, 91 },
    { 0x21f3db21, 46 },
    { 0xb183db22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c83db26, 55 },
    { 0x5ca3db27, 50 },
    { 0xff73db28, 41 },
    { 0, 0 },
    { 0xa683db2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc773db2e, 95 },
    { 0xecb3db2f, 95 },
    { 0, 0 },
    { 0xcd53db31, 46 },
    { 0x76a3db32, 90 },
    { 0, 0 },
    { 0xbcf3db34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb203db37, 95 },
    { 0, 0 },
    { 0xfca3db39, 95 },
    { 0x6103db3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc413db40, 61 },
    { 0xa6f3db41, 52 },
    { 0x1e43db42, 99 },
    { 0x9883db43, 23 },
    { 0xfbf3db44, 95 },
    { 0, 0 },
    { 0x8393db46, 76 },
    { 0x4b93db47, 68 },
    { 0x8cf3db46, 55 },
    { 0xb883db47, 100 },
    { 0, 0 },
    { 0xb6c3db4b, 98 },
    { 0, 0 },
    { 0x6373db4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd3db52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9673db59, 46 },
    { 0x9c83db5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c93db5e, 17 },
    { 0xe103db5f, 90 },
    { 0x2643db5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3db66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b03db6c, 84 },
    { 0x8613db6d, 95 },
    { 0xe623db6e, 88 },
    { 0x983db6f, 23 },
    { 0, 0 },
    { 0xb6b3db71, 98 },
    { 0xc4c3db71, 100 },
    { 0xaf43db73, 94 },
    { 0xe933db74, 90 },
    { 0x37f3db75, 96 },
    { 0x1e73db71, 34 },
    { 0x4d83db77, 90 },
    { 0x2463db76, 17 },
    { 0xca43db76, 61 },
    { 0xca3db79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b23db81, 98 },
    { 0x90d3db81, 43 },
    { 0xcd93db82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6653db86, 95 },
    { 0, 0 },
    { 0x39a3db88, 61 },
    { 0x7393db89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf943db8c, 96 },
    { 0x9b13db8d, 46 },
    { 0x2d53db8e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4283db94, 90 },
    { 0xafb3db94, 95 },
    { 0xf5e3db95, 98 },
    { 0x4b23db95, 95 },
    { 0x3d53db98, 81 },
    { 0x1453db98, 61 },
    { 0, 0 },
    { 0x1bc3db9b, 93 },
    { 0, 0 },
    { 0xb423db9d, 90 },
    { 0x9b13db9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b3dba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1413dbac, 84 },
    { 0x96e3dbad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563dbb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9183dbb3, 90 },
    { 0x4dd3dbb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63dbb7, 95 },
    { 0, 0 },
    { 0xc913dbb9, 90 },
    { 0, 0 },
    { 0x2f73dbbb, 61 },
    { 0xf0c3dbbb, 95 },
    { 0x6e43dbbc, 100 },
    { 0x6e33dbbb, 95 },
    { 0x3c3dbbf, 61 },
    { 0xb823dbbb, 38 },
    { 0xa453dbc1, 87 },
    { 0x2273dbc1, 90 },
    { 0x28d3dbc2, 90 },
    { 0x9d73dbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d3dbcd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3dbd1, 55 },
    { 0x883dbd2, 33 },
    { 0x2f03dbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7153dbda, 95 },
    { 0xf193dbdb, 91 },
    { 0x4e83dbdc, 23 },
    { 0x6023dbdd, 100 },
    { 0xc9b3dbda, 78 },
    { 0, 0 },
    { 0x8073dbe0, 43 },
    { 0, 0 },
    { 0x63f3dbe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3753dbe5, 66 },
    { 0xcbf3dbe6, 43 },
    { 0, 0 },
    { 0xb953dbe8, 61 },
    { 0xfeb3dbe9, 43 },
    { 0x2b63dbea, 95 },
    { 0x3d23dbeb, 83 },
    { 0x2b93dbea, 90 },
    { 0x47c3dbea, 34 },
    { 0xacb3dbee, 23 },
    { 0x4613dbe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b63dbf2, 34 },
    { 0x5853dbf3, 46 },
    { 0x31a3dbf4, 95 },
    { 0xbe73dbf4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b3dbf8, 77 },
    { 0x68d3dbf9, 52 },
    { 0x3793dbf9, 91 },
    { 0xade3dbfa, 33 },
    { 0x7a23dbf9, 34 },
    { 0, 0 },
    { 0x9843dbfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493dc05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd93dc0b, 75 },
    { 0, 0 },
    { 0xb2c3dc0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf93dc10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3dc15, 33 },
    { 0x2f43dc15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3dc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c3dc1c, 95 },
    { 0xbda3dc1c, 91 },
    { 0, 0 },
    { 0xe4d3dc1f, 93 },
    { 0, 0 },
    { 0x7023dc21, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc303dc27, 34 },
    { 0x74c3dc28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773dc33, 76 },
    { 0xa153dc34, 33 },
    { 0xc793dc33, 34 },
    { 0xb73dc36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe443dc39, 46 },
    { 0, 0 },
    { 0x7c13dc3b, 46 },
    { 0x25c3dc3c, 33 },
    { 0x9013dc3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e53dc4c, 93 },
    { 0, 0 },
    { 0xf9e3dc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93dc54, 81 },
    { 0x2943dc55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023dc59, 83 },
    { 0x1313dc5a, 23 },
    { 0x4e3dc5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3dc66, 33 },
    { 0, 0 },
    { 0xa9f3dc68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa3dc6b, 98 },
    { 0x33c3dc6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba53dc6f, 95 },
    { 0, 0 },
    { 0x8b23dc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3dc75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c33dc7b, 61 },
    { 0x2253dc7c, 95 },
    { 0x31d3dc7c, 55 },
    { 0x1ca3dc7e, 66 },
    { 0, 0 },
    { 0x58c3dc80, 90 },
    { 0x4963dc81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc63dc84, 61 },
    { 0x9203dc85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e3dc88, 56 },
    { 0x2903dc89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff93dc9a, 75 },
    { 0xbf53dc9b, 98 },
    { 0x7fd3dc9b, 91 },
    { 0x6cf3dc9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53dca0, 95 },
    { 0, 0 },
    { 0x8333dca2, 23 },
    { 0x47f3dca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83dcb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f3dcba, 34 },
    { 0xff73dcba, 33 },
    { 0x7753dcbc, 23 },
    { 0x5013dcbc, 95 },
    { 0x82d3dcbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b3dcc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33dcc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93dcc8, 33 },
    { 0, 0 },
    { 0x1c23dcca, 77 },
    { 0xc0d3dcca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc373dcce, 34 },
    { 0, 0 },
    { 0x8da3dcd0, 33 },
    { 0x1413dcd1, 61 },
    { 0, 0 },
    { 0xb9b3dcd3, 34 },
    { 0x1023dcd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f3dcd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83dcda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3dce7, 85 },
    { 0x4cd3dce8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3dceb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6563dcee, 98 },
    { 0, 0 },
    { 0xb4d3dcf0, 95 },
    { 0, 0 },
    { 0x1323dcf2, 23 },
    { 0xe2a3dcf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3dcf7, 46 },
    { 0, 0 },
    { 0xfec3dcf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243dcfd, 90 },
    { 0xc5f3dcfd, 98 },
    { 0xa263dcff, 66 },
    { 0x7c13dcff, 23 },
    { 0x31e3dcfe, 61 },
    { 0, 0 },
    { 0x2043dd03, 90 },
    { 0x8983dd04, 33 },
    { 0, 0 },
    { 0x66e3dd06, 76 },
    { 0xb083dd06, 23 },
    { 0xda3dd08, 33 },
    { 0x4fd3dd06, 23 },
    { 0, 0 },
    { 0x8213dd0b, 86 },
    { 0, 0 },
    { 0xe43dd0d, 80 },
    { 0x7bc3dd0e, 77 },
    { 0x5643dd0e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc763dd13, 76 },
    { 0, 0 },
    { 0x7c43dd15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc3dd18, 86 },
    { 0xa153dd19, 33 },
    { 0, 0 },
    { 0xa623dd1b, 12 },
    { 0xaa93dd1c, 98 },
    { 0x5203dd1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1233dd25, 75 },
    { 0x1183dd26, 43 },
    { 0xe6f3dd26, 98 },
    { 0xf673dd28, 81 },
    { 0xf243dd26, 98 },
    { 0, 0 },
    { 0x6433dd2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3dd32, 17 },
    { 0xe663dd32, 90 },
    { 0, 0 },
    { 0x2303dd35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c3dd3f, 95 },
    { 0x1a63dd40, 90 },
    { 0xd9c3dd41, 84 },
    { 0, 0 },
    { 0xa643dd43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7243dd48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd53dd4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb3dd56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a3dd5a, 90 },
    { 0, 0 },
    { 0xb173dd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c3dd5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3dd65, 68 },
    { 0x2943dd66, 33 },
    { 0xfd53dd67, 55 },
    { 0xe543dd67, 90 },
    { 0xccf3dd69, 76 },
    { 0x8db3dd66, 91 },
    { 0, 0 },
    { 0x6113dd6c, 90 },
    { 0, 0 },
    { 0x1dc3dd6e, 33 },
    { 0x1be3dd6f, 90 },
    { 0x2c03dd6f, 78 },
    { 0x4e43dd71, 55 },
    { 0x86f3dd72, 43 },
    { 0x5ff3dd6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8993dd7c, 61 },
    { 0xa3e3dd7d, 33 },
    { 0xce43dd7d, 68 },
    { 0x663dd7e, 78 },
    { 0x9693dd7e, 46 },
    { 0x7073dd7d, 61 },
    { 0, 0 },
    { 0x4dd3dd83, 61 },
    { 0x2c73dd83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f3dd87, 55 },
    { 0x5433dd88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3dd8e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2513dd94, 98 },
    { 0, 0 },
    { 0xfa03dd96, 87 },
    { 0x1303dd97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3dd9d, 61 },
    { 0, 0 },
    { 0xbae3dd9f, 93 },
    { 0x6f63dda0, 33 },
    { 0, 0 },
    { 0x36c3dda2, 97 },
    { 0x8a53dda3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3523ddbc, 55 },
    { 0x5903ddbd, 91 },
    { 0x753ddbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe3ddc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6653ddc6, 61 },
    { 0, 0 },
    { 0x4a73ddc8, 61 },
    { 0xb703ddc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8283ddd0, 23 },
    { 0x4bc3ddd1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23ddd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc113ddd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb3dddc, 55 },
    { 0xb763dddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc913dde0, 33 },
    { 0xee63dde1, 84 },
    { 0x2f43dde2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3dde5, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf13ddea, 55 },
    { 0x3ac3ddea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa403ddef, 90 },
    { 0x88a3ddef, 98 },
    { 0x7213ddef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf283ddf4, 75 },
    { 0xaeb3ddf4, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b03ddf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3ddfd, 97 },
    { 0xdbc3ddfd, 76 },
    { 0xddf3ddfe, 98 },
    { 0xaf23ddfe, 23 },
    { 0, 0 },
    { 0x7663de02, 77 },
    { 0xf973de03, 90 },
    { 0xce53de03, 33 },
    { 0x6db3de05, 61 },
    { 0xb113de06, 23 },
    { 0xf3c3de07, 43 },
    { 0x1483de05, 67 },
    { 0x2bb3de09, 90 },
    { 0, 0 },
    { 0x663de0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3de0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e3de19, 61 },
    { 0x4b3de19, 33 },
    { 0x12b3de19, 17 },
    { 0, 0 },
    { 0xe6f3de1d, 81 },
    { 0, 0 },
    { 0xb2b3de1f, 78 },
    { 0, 0 },
    { 0x1003de21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3de26, 55 },
    { 0x5543de27, 61 },
    { 0, 0 },
    { 0x6ae3de29, 33 },
    { 0xbf33de2a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa103de2e, 23 },
    { 0x7173de2f, 61 },
    { 0x18e3de30, 90 },
    { 0xc703de2e, 55 },
    { 0xd3e3de32, 99 },
    { 0x3403de33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d43de37, 90 },
    { 0xad63de38, 33 },
    { 0x8dd3de39, 76 },
    { 0, 0 },
    { 0x5023de3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983de42, 33 },
    { 0, 0 },
    { 0xc13de44, 95 },
    { 0, 0 },
    { 0x16f3de46, 33 },
    { 0, 0 },
    { 0xf3e3de48, 99 },
    { 0xa743de49, 98 },
    { 0x3fe3de4a, 95 },
    { 0x83d3de4b, 98 },
    { 0x9393de4c, 43 },
    { 0x4c63de4c, 23 },
    { 0xc773de4a, 23 },
    { 0, 0 },
    { 0xb633de50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093de55, 90 },
    { 0xff53de56, 90 },
    { 0x4d53de55, 61 },
    { 0, 0 },
    { 0xea23de59, 61 },
    { 0x44d3de5a, 33 },
    { 0xc2e3de5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b23de5e, 76 },
    { 0x10c3de5e, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813de67, 90 },
    { 0xd813de68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3de6b, 67 },
    { 0x25e3de6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe453de6f, 91 },
    { 0, 0 },
    { 0xe883de71, 33 },
    { 0, 0 },
    { 0x2183de73, 84 },
    { 0x8fb3de74, 81 },
    { 0, 0 },
    { 0x3b83de76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b3de7e, 100 },
    { 0x88d3de7e, 33 },
    { 0x3f63de7e, 95 },
    { 0x1043de81, 95 },
    { 0xb2f3de7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae3de91, 95 },
    { 0, 0 },
    { 0x43de93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3de96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa783de99, 23 },
    { 0x7af3de9a, 34 },
    { 0, 0 },
    { 0xc7b3de9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5483dea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca3dea9, 78 },
    { 0, 0 },
    { 0xb5e3deab, 33 },
    { 0xa23deac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3deb3, 95 },
    { 0, 0 },
    { 0xece3deb5, 98 },
    { 0, 0 },
    { 0xdd3deb7, 46 },
    { 0x6b53deb8, 76 },
    { 0x8bf3deb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae43debe, 52 },
    { 0xbe03debe, 76 },
    { 0xf233dec0, 33 },
    { 0x80f3dec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c3dec8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb63ded1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33ded4, 98 },
    { 0, 0 },
    { 0x3903ded6, 17 },
    { 0x24b3ded7, 88 },
    { 0x1c13ded7, 90 },
    { 0x4233ded8, 88 },
    { 0xb3b3ded9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e63dee2, 56 },
    { 0x9c73dee2, 76 },
    { 0xf903dee2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f43deef, 61 },
    { 0xc6d3def0, 55 },
    { 0, 0 },
    { 0x3413def2, 100 },
    { 0, 0 },
    { 0xa2c3def4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e93def7, 52 },
    { 0xd573def7, 33 },
    { 0xa933def8, 87 },
    { 0xd303defa, 43 },
    { 0, 0 },
    { 0x4643defc, 78 },
    { 0xb783defd, 55 },
    { 0xc463defd, 33 },
    { 0x68b3deff, 33 },
    { 0x2113defe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc943df05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e33df08, 90 },
    { 0xa8c3df08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb923df0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1593df0f, 75 },
    { 0x3f53df10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b43df14, 98 },
    { 0x3763df15, 93 },
    { 0x9a63df15, 81 },
    { 0x43f3df15, 98 },
    { 0x2173df17, 34 },
    { 0x4e03df19, 23 },
    { 0xeba3df1a, 23 },
    { 0, 0 },
    { 0x9b43df1c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e43df20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc443df28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483df2b, 43 },
    { 0x33df2c, 65 },
    { 0x7f3df2b, 90 },
    { 0, 0 },
    { 0x7033df2f, 93 },
    { 0, 0 },
    { 0x6043df31, 80 },
    { 0x4e23df32, 88 },
    { 0xdcd3df32, 98 },
    { 0xcb33df33, 43 },
    { 0x8b83df35, 61 },
    { 0xd723df36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e3df45, 76 },
    { 0x47d3df45, 66 },
    { 0, 0 },
    { 0x8133df48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d63df4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e3df4e, 98 },
    { 0x55b3df4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23df55, 52 },
    { 0x963df55, 34 },
    { 0xcd93df57, 66 },
    { 0, 0 },
    { 0xc83df59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f3df65, 61 },
    { 0x333df66, 76 },
    { 0xacc3df67, 95 },
    { 0xb543df68, 98 },
    { 0, 0 },
    { 0x9f53df6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3df6f, 90 },
    { 0x86b3df6f, 33 },
    { 0, 0 },
    { 0x2dd3df72, 98 },
    { 0, 0 },
    { 0x8443df74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc833df83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f3df87, 33 },
    { 0, 0 },
    { 0x1303df89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963df8e, 33 },
    { 0xadf3df8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963df93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b3df97, 99 },
    { 0, 0 },
    { 0xad33df99, 90 },
    { 0x3e43df9a, 95 },
    { 0, 0 },
    { 0x53e3df9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb53dfa2, 46 },
    { 0xf4a3dfa3, 43 },
    { 0, 0 },
    { 0x5a83dfa5, 66 },
    { 0, 0 },
    { 0x2b53dfa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb613dfaa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3dfb1, 33 },
    { 0, 0 },
    { 0xe2f3dfb3, 78 },
    { 0xf713dfb4, 95 },
    { 0x7053dfb3, 98 },
    { 0x2133dfb6, 33 },
    { 0x3ec3dfb7, 61 },
    { 0x423dfb7, 23 },
    { 0x68a3dfb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4333dfbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b3dfc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2013dfc5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1723dfca, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e3dfcf, 90 },
    { 0xa183dfd0, 61 },
    { 0, 0 },
    { 0x3863dfd2, 61 },
    { 0xc983dfd2, 91 },
    { 0x7e13dfd2, 95 },
    { 0x4f73dfd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe213dfd8, 33 },
    { 0, 0 },
    { 0x8a13dfda, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2193dfe0, 23 },
    { 0x2f63dfe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe63dfe5, 55 },
    { 0x4263dfe6, 33 },
    { 0xf173dfe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383dfee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3dff5, 90 },
    { 0xb2f3dff6, 52 },
    { 0x5ef3dff5, 93 },
    { 0x24f3dff8, 78 },
    { 0, 0 },
    { 0x5083dffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713e007, 65 },
    { 0xc573e008, 95 },
    { 0, 0 },
    { 0xb373e00a, 61 },
    { 0xa4d3e00b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b73e00f, 55 },
    { 0, 0 },
    { 0x42b3e011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b3e015, 97 },
    { 0, 0 },
    { 0x99b3e017, 61 },
    { 0x7d33e018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c63e01c, 55 },
    { 0, 0 },
    { 0x76f3e01e, 43 },
    { 0x1d53e01f, 60 },
    { 0x183e01f, 34 },
    { 0x4d3e021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313e025, 23 },
    { 0x2b33e026, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3e02b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa773e032, 65 },
    { 0x5ae3e032, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd23e039, 33 },
    { 0x9253e039, 23 },
    { 0x13b3e03b, 97 },
    { 0xfb13e03a, 93 },
    { 0x6053e03c, 99 },
    { 0xec83e03e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d3e041, 17 },
    { 0x8dc3e042, 23 },
    { 0xd33e043, 95 },
    { 0x7cd3e044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb293e048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb23e04c, 90 },
    { 0x5fc3e04c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c73e050, 52 },
    { 0, 0 },
    { 0xc153e052, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403e059, 76 },
    { 0x46d3e05a, 23 },
    { 0x2aa3e05b, 61 },
    { 0, 0 },
    { 0x6f73e05d, 33 },
    { 0xdd63e05e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c3e061, 98 },
    { 0, 0 },
    { 0x79e3e063, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03e066, 98 },
    { 0, 0 },
    { 0x4e73e068, 33 },
    { 0x1433e069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa73e06e, 78 },
    { 0x2073e06f, 95 },
    { 0, 0 },
    { 0x8443e071, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f33e074, 90 },
    { 0, 0 },
    { 0x98c3e076, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe523e07a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2183e07f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x613e082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf93e086, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3e08e, 23 },
    { 0x5fa3e08e, 33 },
    { 0xb03e08f, 88 },
    { 0x203e091, 98 },
    { 0x5953e090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe73e095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa493e098, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5833e09c, 61 },
    { 0x9c93e09d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153e0a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813e0a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec3e0a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b73e0b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e0bd, 33 },
    { 0xb2e3e0be, 23 },
    { 0x9253e0bf, 46 },
    { 0xdc43e0bf, 98 },
    { 0x33c3e0bf, 90 },
    { 0x5f13e0c2, 43 },
    { 0xda43e0bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f3e0ca, 95 },
    { 0xa633e0cb, 95 },
    { 0, 0 },
    { 0xa9c3e0cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3e0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd583e0d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a3e0da, 34 },
    { 0x29b3e0db, 52 },
    { 0x2373e0dc, 61 },
    { 0x3eb3e0dd, 91 },
    { 0x9c3e0de, 95 },
    { 0xdb03e0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013e0e4, 90 },
    { 0x20d3e0e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b3e0e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc63e0eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9853e0f1, 90 },
    { 0x39d3e0f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d3e0fb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9033e0ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b3e103, 61 },
    { 0x6613e103, 33 },
    { 0xc373e105, 95 },
    { 0xffa3e106, 93 },
    { 0x8363e104, 23 },
    { 0xb0a3e108, 76 },
    { 0xabd3e108, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103e10d, 33 },
    { 0, 0 },
    { 0x9533e10f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd203e113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33e116, 66 },
    { 0xc1a3e117, 76 },
    { 0xa183e118, 46 },
    { 0x1b83e119, 33 },
    { 0, 0 },
    { 0xc533e11b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853e124, 76 },
    { 0x37f3e125, 23 },
    { 0xeb23e126, 34 },
    { 0x1743e126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e3e12b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43e12e, 99 },
    { 0x8b3e12e, 95 },
    { 0, 0 },
    { 0x8963e131, 93 },
    { 0x8453e132, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333e139, 68 },
    { 0xe143e13a, 55 },
    { 0x8463e13a, 34 },
    { 0x3483e13c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903e140, 90 },
    { 0x7b43e140, 61 },
    { 0x5843e141, 51 },
    { 0, 0 },
    { 0xb3d3e144, 61 },
    { 0xc7f3e145, 99 },
    { 0, 0 },
    { 0xbc3e147, 91 },
    { 0x6603e148, 95 },
    { 0x26d3e149, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3633e14d, 51 },
    { 0x3773e14d, 61 },
    { 0xbe83e14d, 93 },
    { 0x1213e14d, 23 },
    { 0, 0 },
    { 0xbb43e152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1513e155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8193e158, 99 },
    { 0x633e159, 90 },
    { 0xe273e159, 98 },
    { 0xac3e158, 43 },
    { 0x5c93e15c, 98 },
    { 0xa143e15d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f3e161, 17 },
    { 0, 0 },
    { 0x2f93e163, 90 },
    { 0x9e73e164, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3e167, 61 },
    { 0xf553e168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e16b, 98 },
    { 0xe003e16c, 78 },
    { 0, 0 },
    { 0xead3e16e, 96 },
    { 0, 0 },
    { 0x4ef3e170, 90 },
    { 0, 0 },
    { 0x18a3e172, 23 },
    { 0, 0 },
    { 0x6713e174, 23 },
    { 0, 0 },
    { 0xc253e176, 57 },
    { 0xc423e177, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3e181, 91 },
    { 0xdf23e181, 36 },
    { 0, 0 },
    { 0x5583e184, 33 },
    { 0, 0 },
    { 0xdaf3e186, 95 },
    { 0xe253e186, 61 },
    { 0, 0 },
    { 0x6fe3e189, 61 },
    { 0x7a93e18a, 66 },
    { 0, 0 },
    { 0x47f3e18c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3e19c, 98 },
    { 0x4963e19d, 95 },
    { 0xf7a3e19d, 33 },
    { 0x89a3e19e, 36 },
    { 0x2993e1a0, 55 },
    { 0xa083e1a1, 93 },
    { 0, 0 },
    { 0xcbb3e1a3, 33 },
    { 0x8693e1a4, 55 },
    { 0x2a23e1a3, 23 },
    { 0x3b73e1a4, 95 },
    { 0x6393e1a7, 23 },
    { 0xadf3e1a8, 23 },
    { 0, 0 },
    { 0x2a33e1aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8813e1ae, 46 },
    { 0, 0 },
    { 0x1f03e1b0, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1453e1b9, 76 },
    { 0xd6e3e1ba, 95 },
    { 0, 0 },
    { 0x57d3e1bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb3e1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf3e1c9, 95 },
    { 0x9833e1ca, 93 },
    { 0xb7f3e1c9, 33 },
    { 0x58d3e1cc, 98 },
    { 0x73c3e1cd, 77 },
    { 0x22f3e1ce, 33 },
    { 0xf033e1cb, 90 },
    { 0x3333e1cb, 95 },
    { 0x9d33e1ce, 33 },
    { 0, 0 },
    { 0x7c83e1d3, 61 },
    { 0x8db3e1d4, 93 },
    { 0x5cf3e1d5, 55 },
    { 0, 0 },
    { 0x3713e1d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e3e1da, 98 },
    { 0x9c43e1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c93e1de, 61 },
    { 0, 0 },
    { 0x5d43e1e0, 33 },
    { 0x2723e1e1, 33 },
    { 0xdde3e1e2, 90 },
    { 0x6883e1e3, 55 },
    { 0xaf93e1e3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e1ea, 34 },
    { 0x46d3e1ea, 90 },
    { 0x2e13e1ea, 33 },
    { 0, 0 },
    { 0x5623e1ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e23e1f1, 61 },
    { 0x5d33e1f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d3e1f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933e1fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d3e201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053e20a, 90 },
    { 0x1e3e20b, 90 },
    { 0, 0 },
    { 0x94c3e20d, 65 },
    { 0, 0 },
    { 0x5373e20f, 33 },
    { 0xa383e210, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d3e215, 55 },
    { 0x7f63e216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663e219, 17 },
    { 0x5f23e21a, 33 },
    { 0x7283e21b, 77 },
    { 0, 0 },
    { 0x6ac3e21d, 78 },
    { 0, 0 },
    { 0xa8c3e21f, 46 },
    { 0x3783e21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a3e224, 97 },
    { 0, 0 },
    { 0x3593e226, 95 },
    { 0, 0 },
    { 0x32d3e228, 76 },
    { 0, 0 },
    { 0xc9c3e22a, 90 },
    { 0x5fd3e22a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4463e22e, 61 },
    { 0x97a3e22f, 93 },
    { 0xbc93e22e, 61 },
    { 0, 0 },
    { 0x3053e232, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3e23b, 23 },
    { 0, 0 },
    { 0x7cf3e23d, 86 },
    { 0xea13e23e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b3e242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3e24f, 76 },
    { 0, 0 },
    { 0xad63e251, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403e254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a73e25f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c3e266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5443e269, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273e26d, 98 },
    { 0, 0 },
    { 0x56c3e26f, 76 },
    { 0xd6e3e270, 76 },
    { 0x9293e271, 52 },
    { 0x7bd3e270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3e275, 69 },
    { 0xc13e275, 100 },
    { 0x1833e277, 84 },
    { 0xfea3e278, 90 },
    { 0x2493e279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d3e27f, 95 },
    { 0x69e3e280, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c3e285, 98 },
    { 0xb703e286, 95 },
    { 0x53d3e286, 95 },
    { 0x1d63e286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf33e28d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83e291, 33 },
    { 0x8683e291, 43 },
    { 0, 0 },
    { 0xa023e294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf953e298, 88 },
    { 0x1ae3e299, 88 },
    { 0xb443e299, 38 },
    { 0xdaf3e299, 95 },
    { 0xe403e29c, 95 },
    { 0xa633e29d, 81 },
    { 0, 0 },
    { 0x8c73e29f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa443e2a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5553e2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6263e2b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3e2b5, 99 },
    { 0xf9c3e2b5, 33 },
    { 0xb2a3e2b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc3e2bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b93e2c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93e2c5, 98 },
    { 0x9fb3e2c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3e2cd, 66 },
    { 0, 0 },
    { 0x9513e2cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb3e2d2, 90 },
    { 0x76b3e2d3, 53 },
    { 0xfc03e2d4, 90 },
    { 0xd3e3e2d4, 95 },
    { 0x8743e2d2, 98 },
    { 0, 0 },
    { 0x9de3e2d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593e2e2, 90 },
    { 0, 0 },
    { 0x6213e2e4, 85 },
    { 0x4213e2e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3e2e9, 90 },
    { 0xf863e2e9, 23 },
    { 0, 0 },
    { 0xdfb3e2ec, 66 },
    { 0x68a3e2ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793e2f1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d3e2f7, 100 },
    { 0x9883e2f7, 23 },
    { 0xc803e2f9, 76 },
    { 0xc933e2fa, 61 },
    { 0x3d63e2f8, 99 },
    { 0xfbb3e2f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd643e308, 19 },
    { 0x36a3e309, 55 },
    { 0xc33e309, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3e310, 33 },
    { 0x5b23e310, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b63e315, 90 },
    { 0x7c33e316, 46 },
    { 0, 0 },
    { 0x4633e318, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83e31b, 34 },
    { 0x85f3e31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab3e322, 76 },
    { 0x2ff3e323, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c3e326, 66 },
    { 0xe983e327, 46 },
    { 0x9223e326, 61 },
    { 0x9ad3e326, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c3e32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c73e334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb063e33f, 90 },
    { 0, 0 },
    { 0x72f3e341, 91 },
    { 0x763e341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f63e34a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6913e34d, 90 },
    { 0, 0 },
    { 0x3dc3e34f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3e352, 65 },
    { 0xbfa3e353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973e357, 76 },
    { 0xf7b3e358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8343e363, 91 },
    { 0x26d3e364, 90 },
    { 0x9113e363, 90 },
    { 0x7f73e363, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313e36a, 93 },
    { 0xe03e36b, 90 },
    { 0, 0 },
    { 0x9e83e36d, 88 },
    { 0xd933e36e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e371, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2313e377, 34 },
    { 0, 0 },
    { 0x6c73e379, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd363e37d, 55 },
    { 0xca83e37d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f3e383, 46 },
    { 0xb6b3e384, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3e387, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff53e38c, 68 },
    { 0, 0 },
    { 0xf3f3e38e, 76 },
    { 0xc2b3e38f, 91 },
    { 0xf433e38f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793e393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c13e398, 90 },
    { 0x2e33e399, 43 },
    { 0xed43e39a, 88 },
    { 0x11e3e399, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea3e39e, 33 },
    { 0, 0 },
    { 0x3993e3a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe913e3a4, 84 },
    { 0x2c03e3a5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa73e3ac, 23 },
    { 0x603e3ac, 61 },
    { 0, 0 },
    { 0xf6e3e3af, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3e3b2, 52 },
    { 0x7fc3e3b3, 33 },
    { 0, 0 },
    { 0xf283e3b5, 65 },
    { 0, 0 },
    { 0x4a53e3b7, 87 },
    { 0x8df3e3b8, 23 },
    { 0xc633e3b7, 33 },
    { 0x2c23e3ba, 61 },
    { 0x823e3b9, 23 },
    { 0, 0 },
    { 0xfde3e3bd, 55 },
    { 0xa5b3e3be, 84 },
    { 0xc2f3e3be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec3e3c6, 84 },
    { 0xd5e3e3c6, 33 },
    { 0x4663e3c8, 61 },
    { 0xd9c3e3c8, 34 },
    { 0x1413e3ca, 61 },
    { 0xfe83e3cb, 68 },
    { 0x6b03e3cc, 33 },
    { 0xec53e3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6183e3d3, 98 },
    { 0xcb33e3d4, 55 },
    { 0x8f73e3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b3e3d9, 100 },
    { 0xa6c3e3da, 40 },
    { 0xe483e3db, 17 },
    { 0, 0 },
    { 0x56a3e3dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4773e3e0, 76 },
    { 0xc3d3e3e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a3e3e5, 69 },
    { 0x8cb3e3e6, 33 },
    { 0x4743e3e7, 33 },
    { 0x8703e3e8, 77 },
    { 0xf973e3e8, 84 },
    { 0, 0 },
    { 0x6c03e3eb, 55 },
    { 0, 0 },
    { 0xb1f3e3ed, 80 },
    { 0x6d53e3ed, 23 },
    { 0x4863e3ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4643e3f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e33e3f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3e3f8, 33 },
    { 0, 0 },
    { 0x2203e3fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43e3fe, 98 },
    { 0xc223e3ff, 87 },
    { 0xe163e3ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a3e403, 95 },
    { 0, 0 },
    { 0xaeb3e405, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1253e40d, 61 },
    { 0xd403e40e, 90 },
    { 0, 0 },
    { 0xc703e410, 66 },
    { 0x8673e411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3e418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3e420, 99 },
    { 0x1543e420, 67 },
    { 0xb353e420, 95 },
    { 0x8d43e423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af3e428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf33e42c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b53e431, 23 },
    { 0x3043e432, 68 },
    { 0xb4c3e432, 61 },
    { 0x59c3e433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d3e43d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773e444, 51 },
    { 0x6103e445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3e44b, 78 },
    { 0x2393e44b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb873e451, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b33e455, 33 },
    { 0, 0 },
    { 0xf1d3e457, 93 },
    { 0, 0 },
    { 0x3513e459, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743e45c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7233e45f, 95 },
    { 0x15b3e460, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63e464, 40 },
    { 0, 0 },
    { 0xc4a3e466, 90 },
    { 0x7ab3e466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d3e46b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263e46e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f3e474, 78 },
    { 0x3a13e475, 33 },
    { 0x1613e475, 43 },
    { 0x1283e477, 90 },
    { 0x763e477, 65 },
    { 0xfcd3e475, 90 },
    { 0, 0 },
    { 0xa223e47b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb43e487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb143e48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3e496, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13e49a, 90 },
    { 0x4293e49a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623e4a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7043e4a9, 76 },
    { 0x5b83e4a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e4b4, 98 },
    { 0, 0 },
    { 0xc0c3e4b6, 23 },
    { 0x6ea3e4b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c3e4bb, 43 },
    { 0xa873e4bc, 90 },
    { 0, 0 },
    { 0x8543e4be, 88 },
    { 0x6fe3e4bf, 85 },
    { 0, 0 },
    { 0x5d23e4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5603e4c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673e4c9, 23 },
    { 0x4c13e4ca, 90 },
    { 0x7763e4cb, 99 },
    { 0x4d3e4cc, 93 },
    { 0xb2e3e4cd, 80 },
    { 0x5323e4ce, 55 },
    { 0x6733e4cf, 33 },
    { 0xca33e4d0, 90 },
    { 0x4fb3e4cc, 93 },
    { 0x703e4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e3e4da, 33 },
    { 0xbe83e4db, 55 },
    { 0x92f3e4db, 95 },
    { 0, 0 },
    { 0xfdd3e4de, 95 },
    { 0x11f3e4df, 95 },
    { 0x4693e4e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df3e4e4, 99 },
    { 0x9f33e4e5, 33 },
    { 0xdf23e4e6, 98 },
    { 0xc673e4e4, 85 },
    { 0, 0 },
    { 0x4663e4e9, 95 },
    { 0xd773e4ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce43e4f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e33e4f8, 33 },
    { 0x5663e4f9, 23 },
    { 0xe2d3e4f8, 100 },
    { 0x3d83e4fb, 23 },
    { 0, 0 },
    { 0xcfd3e4fd, 34 },
    { 0x9a3e4fe, 66 },
    { 0xab73e4fe, 43 },
    { 0, 0 },
    { 0x6ec3e501, 87 },
    { 0, 0 },
    { 0x3393e503, 90 },
    { 0x2de3e503, 76 },
    { 0xab03e505, 98 },
    { 0x7f53e506, 75 },
    { 0xb2d3e503, 43 },
    { 0xa13e508, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8013e50b, 52 },
    { 0xd1d3e50c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e3e511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff3e514, 85 },
    { 0x8813e515, 90 },
    { 0xb743e516, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023e51e, 95 },
    { 0x1fe3e51f, 78 },
    { 0, 0 },
    { 0x12a3e521, 76 },
    { 0xb3e3e521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e43e527, 84 },
    { 0xebd3e527, 23 },
    { 0xb9a3e529, 23 },
    { 0x833e52a, 90 },
    { 0x9f83e52b, 76 },
    { 0xbd03e52c, 64 },
    { 0xb493e52d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf803e530, 95 },
    { 0xf1e3e530, 90 },
    { 0x38c3e531, 95 },
    { 0x2a03e533, 76 },
    { 0, 0 },
    { 0x66a3e535, 23 },
    { 0x1e33e536, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c3e539, 85 },
    { 0x1d03e53a, 90 },
    { 0x1a93e53a, 90 },
    { 0x7313e539, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b3e53f, 61 },
    { 0xad63e540, 99 },
    { 0xd413e541, 98 },
    { 0x9f33e542, 95 },
    { 0x7553e543, 43 },
    { 0x2373e543, 23 },
    { 0x40a3e544, 76 },
    { 0xad03e546, 66 },
    { 0xfd03e546, 55 },
    { 0x29a3e53f, 84 },
    { 0x1683e549, 47 },
    { 0xe653e545, 55 },
    { 0, 0 },
    { 0x4853e54c, 46 },
    { 0xf833e54d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6673e551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a43e554, 77 },
    { 0x4033e555, 33 },
    { 0xd643e556, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1613e559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f3e55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc573e55f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33e56c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa933e573, 55 },
    { 0x9773e574, 92 },
    { 0x763e574, 23 },
    { 0x2eb3e576, 34 },
    { 0x5043e577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93e57c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e3e583, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf433e58b, 78 },
    { 0xe533e58c, 76 },
    { 0xc03e58b, 76 },
    { 0x8d83e58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993e596, 99 },
    { 0x8b53e596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d83e59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e59d, 33 },
    { 0xc393e59e, 64 },
    { 0x4f33e59f, 76 },
    { 0xcfa3e5a0, 33 },
    { 0xe63e5a1, 66 },
    { 0xadb3e5a2, 61 },
    { 0x88c3e5a3, 69 },
    { 0xb563e5a4, 23 },
    { 0x393e5a5, 90 },
    { 0x48a3e5a6, 90 },
    { 0, 0 },
    { 0x3883e5a8, 90 },
    { 0, 0 },
    { 0xf6d3e5aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673e5ad, 95 },
    { 0, 0 },
    { 0x8943e5af, 98 },
    { 0, 0 },
    { 0xf4a3e5b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3e5b6, 83 },
    { 0, 0 },
    { 0xba73e5b8, 99 },
    { 0xa43e5b9, 95 },
    { 0, 0 },
    { 0x8f93e5bb, 90 },
    { 0, 0 },
    { 0x3083e5bd, 78 },
    { 0xd253e5be, 23 },
    { 0x3d13e5be, 52 },
    { 0, 0 },
    { 0xae13e5c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3e5c8, 66 },
    { 0xf8c3e5c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593e5cd, 61 },
    { 0xbb33e5ce, 76 },
    { 0, 0 },
    { 0xc563e5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3e5d4, 45 },
    { 0x4ae3e5d5, 61 },
    { 0, 0 },
    { 0xb053e5d7, 55 },
    { 0xdfc3e5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee3e5dc, 23 },
    { 0x6913e5dc, 99 },
    { 0, 0 },
    { 0x77a3e5df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e3e5e4, 81 },
    { 0x7033e5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2983e5eb, 43 },
    { 0x4c13e5ec, 52 },
    { 0x43b3e5ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e5f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a3e5f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3e5fc, 61 },
    { 0, 0 },
    { 0xdea3e5fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb043e602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e3e608, 99 },
    { 0, 0 },
    { 0x7273e60a, 33 },
    { 0x94f3e60b, 34 },
    { 0, 0 },
    { 0xaeb3e60d, 65 },
    { 0x4933e60e, 48 },
    { 0x5f13e60f, 83 },
    { 0xf043e60d, 76 },
    { 0x1ba3e611, 90 },
    { 0x5d03e612, 77 },
    { 0x59a3e613, 43 },
    { 0x27f3e60f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1333e619, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53e61c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f3e61f, 93 },
    { 0x8863e620, 90 },
    { 0x2a83e620, 81 },
    { 0xd1d3e622, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b3e625, 98 },
    { 0xb1d3e626, 23 },
    { 0x5113e627, 61 },
    { 0xf653e628, 55 },
    { 0xbca3e629, 61 },
    { 0x63b3e629, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7303e62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153e632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba3e635, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xda03e638, 68 },
    { 0xb7a3e639, 38 },
    { 0xae73e63a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e63d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f13e642, 76 },
    { 0xe83e642, 68 },
    { 0x9c33e644, 23 },
    { 0xb993e643, 99 },
    { 0x7a43e646, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe733e649, 76 },
    { 0x3af3e649, 76 },
    { 0x3b93e64b, 76 },
    { 0xb9a3e64a, 98 },
    { 0x3e93e64d, 61 },
    { 0, 0 },
    { 0xbf73e64f, 98 },
    { 0x6d73e650, 55 },
    { 0x9603e651, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553e654, 34 },
    { 0x4033e655, 95 },
    { 0xeb83e654, 87 },
    { 0x2c03e657, 95 },
    { 0, 0 },
    { 0xc03e659, 55 },
    { 0xa8f3e65a, 23 },
    { 0x6933e65b, 34 },
    { 0, 0 },
    { 0x5cf3e65d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b93e661, 93 },
    { 0xaff3e662, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xee33e665, 17 },
    { 0xae93e666, 80 },
    { 0x68c3e666, 33 },
    { 0, 0 },
    { 0x6c43e669, 90 },
    { 0x3c73e66a, 93 },
    { 0x4593e66a, 33 },
    { 0x4b3e66c, 61 },
    { 0, 0 },
    { 0x1ab3e66e, 76 },
    { 0xffc3e66f, 98 },
    { 0x3c23e670, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc743e675, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d3e67e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e3e685, 76 },
    { 0x6b23e686, 23 },
    { 0x4073e687, 98 },
    { 0x9e73e688, 90 },
    { 0x9cb3e688, 61 },
    { 0x87d3e688, 23 },
    { 0x78c3e687, 95 },
    { 0xd923e68c, 46 },
    { 0, 0 },
    { 0x3723e68e, 61 },
    { 0x2353e68f, 23 },
    { 0xe9d3e68f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa3e698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3483e69d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb933e6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c3e6a3, 90 },
    { 0x83a3e6a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253e6a8, 95 },
    { 0xdba3e6a9, 23 },
    { 0xc313e6aa, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa3e6ae, 91 },
    { 0, 0 },
    { 0xc713e6b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc3e6b4, 90 },
    { 0xf083e6b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f3e6b8, 95 },
    { 0, 0 },
    { 0xcb23e6ba, 66 },
    { 0x8be3e6bb, 51 },
    { 0x2013e6ba, 90 },
    { 0, 0 },
    { 0x83f3e6be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5693e6c1, 90 },
    { 0, 0 },
    { 0x8433e6c3, 38 },
    { 0x3e33e6c4, 52 },
    { 0x7713e6c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d43e6c8, 64 },
    { 0xd1a3e6c9, 67 },
    { 0x1073e6c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe223e6d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333e6d8, 61 },
    { 0x59a3e6d9, 76 },
    { 0x37d3e6d9, 91 },
    { 0, 0 },
    { 0xef63e6dc, 52 },
    { 0x7d63e6dd, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3e6e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2443e6e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f43e6eb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x5953e6ee, 81 },
    { 0xe823e6ee, 23 },
    { 0xb1e3e6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3e6f4, 90 },
    { 0x2253e6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc883e6fa, 26 },
    { 0, 0 },
    { 0x18b3e6fc, 33 },
    { 0xf93e6fc, 95 },
    { 0xcfc3e6fe, 68 },
    { 0xa123e6ff, 61 },
    { 0, 0 },
    { 0xdbc3e701, 93 },
    { 0x8203e702, 93 },
    { 0, 0 },
    { 0xecc3e704, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3e707, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e70c, 65 },
    { 0x57e3e70d, 90 },
    { 0xc283e70d, 61 },
    { 0, 0 },
    { 0x4d23e710, 90 },
    { 0x3583e711, 67 },
    { 0x6783e711, 34 },
    { 0xfe03e713, 23 },
    { 0x6853e714, 95 },
    { 0xc703e715, 68 },
    { 0x2753e715, 98 },
    { 0xddf3e716, 46 },
    { 0xb813e715, 76 },
    { 0x1503e717, 17 },
    { 0x2ed3e718, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c73e724, 66 },
    { 0xe0b3e725, 33 },
    { 0x9823e726, 61 },
    { 0x1863e727, 99 },
    { 0xced3e726, 95 },
    { 0x2153e729, 52 },
    { 0xd9a3e724, 66 },
    { 0x2513e72a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8863e731, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483e734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb73e737, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b3e73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b03e73f, 98 },
    { 0x4463e740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb553e747, 52 },
    { 0, 0 },
    { 0x7ae3e749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2553e74d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3e751, 90 },
    { 0xda23e752, 55 },
    { 0x5393e753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083e75a, 34 },
    { 0x6ab3e75b, 23 },
    { 0x6ec3e75b, 33 },
    { 0x3773e75c, 34 },
    { 0xdaa3e75e, 61 },
    { 0x1293e75f, 93 },
    { 0xd4b3e760, 46 },
    { 0x2e73e760, 61 },
    { 0, 0 },
    { 0xf1f3e763, 55 },
    { 0x3263e763, 43 },
    { 0, 0 },
    { 0x1933e766, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d03e76a, 95 },
    { 0, 0 },
    { 0x1243e76c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c3e772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13e776, 90 },
    { 0xf553e776, 99 },
    { 0x1963e776, 33 },
    { 0xd0b3e779, 23 },
    { 0x2823e77a, 85 },
    { 0xe7b3e778, 55 },
    { 0x96e3e778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53e77f, 61 },
    { 0, 0 },
    { 0xbb63e781, 38 },
    { 0, 0 },
    { 0xbfb3e783, 95 },
    { 0x7e23e784, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463e78c, 98 },
    { 0x1073e78d, 76 },
    { 0x8a13e78e, 98 },
    { 0x2693e78c, 99 },
    { 0xa683e790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f3e794, 90 },
    { 0xcb93e795, 27 },
    { 0x2243e796, 87 },
    { 0x5c23e796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3e79c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e33e7a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a23e7a3, 66 },
    { 0, 0 },
    { 0x9883e7a5, 46 },
    { 0, 0 },
    { 0xfb13e7a7, 38 },
    { 0xffc3e7a8, 67 },
    { 0xd393e7a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff3e7b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9543e7b4, 43 },
    { 0x8473e7b4, 95 },
    { 0x12e3e7b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f3e7c4, 34 },
    { 0x1903e7c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83e7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5013e7d7, 33 },
    { 0, 0 },
    { 0xb33e7d9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc33e7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf03e7df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3e7e2, 90 },
    { 0x433e7e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3e7ee, 90 },
    { 0x6053e7ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453e7f4, 80 },
    { 0x2bd3e7f5, 76 },
    { 0, 0 },
    { 0xc33e7f7, 61 },
    { 0xf7e3e7f7, 33 },
    { 0xf143e7f9, 93 },
    { 0xb3b3e7fa, 34 },
    { 0xdb13e7fb, 23 },
    { 0xd063e7fc, 90 },
    { 0x3f43e7fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8233e803, 23 },
    { 0x81e3e804, 61 },
    { 0, 0 },
    { 0x9fc3e806, 85 },
    { 0, 0 },
    { 0xf513e808, 43 },
    { 0x2803e808, 91 },
    { 0xae73e80a, 90 },
    { 0xbf73e808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4363e80f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f23e813, 23 },
    { 0xf53e813, 61 },
    { 0xe703e815, 34 },
    { 0xcea3e815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3e819, 23 },
    { 0x3d43e81a, 95 },
    { 0x71c3e81b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a43e81e, 33 },
    { 0x71c3e81e, 78 },
    { 0, 0 },
    { 0x783e821, 90 },
    { 0x71f3e821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c3e827, 90 },
    { 0, 0 },
    { 0x9413e829, 23 },
    { 0x1fd3e829, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23e82e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3e83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e845, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33e84b, 98 },
    { 0x7163e84c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9943e850, 90 },
    { 0x51e3e850, 84 },
    { 0, 0 },
    { 0x9f33e853, 33 },
    { 0xa0a3e854, 91 },
    { 0, 0 },
    { 0xa243e856, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d3e85b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd603e860, 76 },
    { 0xb2c3e861, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef63e865, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd3e869, 43 },
    { 0, 0 },
    { 0x8813e86b, 84 },
    { 0xbb53e86c, 55 },
    { 0x6ae3e86c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b3e870, 66 },
    { 0x8863e871, 78 },
    { 0x4823e871, 98 },
    { 0xddd3e873, 95 },
    { 0x6133e873, 75 },
    { 0xf13e873, 34 },
    { 0x9623e874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e3e87b, 90 },
    { 0x25c3e87c, 43 },
    { 0xcbc3e87c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1443e885, 52 },
    { 0x5df3e885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713e889, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c73e88d, 33 },
    { 0x6653e88d, 50 },
    { 0xb823e88f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b13e895, 46 },
    { 0x39b3e896, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33e899, 61 },
    { 0xbbe3e89a, 38 },
    { 0, 0 },
    { 0xf43e89c, 95 },
    { 0x5533e89c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d53e8a2, 34 },
    { 0x2823e8a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023e8a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd83e8a9, 38 },
    { 0x7d03e8aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63e8ad, 90 },
    { 0x62d3e8ad, 96 },
    { 0, 0 },
    { 0x9993e8b0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c3e8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae13e8b6, 17 },
    { 0x7df3e8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c3e8bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8503e8c6, 55 },
    { 0xdaa3e8c6, 67 },
    { 0xe963e8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e8d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013e8de, 47 },
    { 0xdc53e8df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23e8e4, 99 },
    { 0x8bc3e8e5, 23 },
    { 0xf6a3e8e4, 33 },
    { 0xe373e8e7, 23 },
    { 0, 0 },
    { 0x69b3e8e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43e8ec, 90 },
    { 0, 0 },
    { 0xeb53e8ee, 99 },
    { 0x35e3e8ef, 23 },
    { 0x6473e8ef, 90 },
    { 0, 0 },
    { 0x7543e8f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8223e8f8, 98 },
    { 0x9413e8f8, 76 },
    { 0xc073e8fa, 66 },
    { 0xdac3e8fa, 95 },
    { 0x5f63e8fb, 98 },
    { 0xcd83e8f9, 66 },
    { 0xc213e8fe, 33 },
    { 0, 0 },
    { 0xecf3e900, 61 },
    { 0xa313e901, 88 },
    { 0x9bd3e902, 76 },
    { 0xdeb3e903, 55 },
    { 0x8ff3e901, 56 },
    { 0xfdf3e903, 67 },
    { 0xdd03e902, 85 },
    { 0x8e03e901, 93 },
    { 0, 0 },
    { 0xe143e909, 98 },
    { 0, 0 },
    { 0x15e3e90b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb13e90f, 61 },
    { 0, 0 },
    { 0xfaa3e911, 46 },
    { 0xafb3e911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f3e91c, 90 },
    { 0x2fd3e91d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5943e920, 95 },
    { 0xe873e920, 61 },
    { 0xe083e922, 61 },
    { 0, 0 },
    { 0x5a23e924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3e928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe203e92f, 46 },
    { 0x8e93e930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3e934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43e938, 90 },
    { 0x6663e938, 98 },
    { 0x53c3e93a, 33 },
    { 0, 0 },
    { 0x2dc3e93c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff3e941, 76 },
    { 0x38b3e941, 80 },
    { 0xe543e941, 95 },
    { 0, 0 },
    { 0xf413e945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3e948, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa3e94d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e63e952, 56 },
    { 0x1ea3e952, 34 },
    { 0x65a3e954, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c3e958, 87 },
    { 0xc153e958, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc3e95c, 67 },
    { 0, 0 },
    { 0xf3a3e95e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a3e964, 95 },
    { 0, 0 },
    { 0x49d3e966, 90 },
    { 0x783e966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d3e96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3e96f, 33 },
    { 0xb343e96f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423e974, 17 },
    { 0xffa3e975, 67 },
    { 0xd943e974, 55 },
    { 0, 0 },
    { 0x8623e978, 95 },
    { 0x78f3e978, 98 },
    { 0x1413e97a, 91 },
    { 0xd373e97b, 90 },
    { 0x2523e97c, 55 },
    { 0, 0 },
    { 0x7253e97e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc3e981, 68 },
    { 0x9283e982, 61 },
    { 0x9aa3e981, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e98b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e990, 48 },
    { 0, 0 },
    { 0xe3e992, 98 },
    { 0, 0 },
    { 0x3b63e994, 55 },
    { 0x7b63e995, 52 },
    { 0xee43e994, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa573e9a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea3e9a8, 98 },
    { 0, 0 },
    { 0x89e3e9aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3e9b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f3e9b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb443e9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db3e9c0, 95 },
    { 0x95a3e9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc3e9c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83e9ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e9ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543e9db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003e9e4, 23 },
    { 0x5cf3e9e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33e9ea, 90 },
    { 0, 0 },
    { 0x5663e9ec, 78 },
    { 0x253e9ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3e9f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b3e9f3, 90 },
    { 0x2e3e9f4, 85 },
    { 0x60c3e9f4, 90 },
    { 0x8503e9f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43ea04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f53ea08, 61 },
    { 0x41f3ea09, 100 },
    { 0xa5e3ea0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa3ea0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4853ea11, 66 },
    { 0x5cc3ea12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03ea16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4763ea19, 78 },
    { 0x7b73ea1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5863ea24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe713ea28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3ea2b, 61 },
    { 0xbd63ea2c, 68 },
    { 0x12d3ea2d, 66 },
    { 0xa463ea2c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7103ea32, 78 },
    { 0, 0 },
    { 0x2823ea34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4483ea38, 23 },
    { 0x4cb3ea39, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3ea41, 33 },
    { 0x4db3ea42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c03ea46, 34 },
    { 0x4ad3ea46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead3ea4b, 78 },
    { 0x5c93ea4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a73ea4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc43ea52, 48 },
    { 0xb893ea52, 43 },
    { 0x5c63ea54, 43 },
    { 0, 0 },
    { 0x4013ea56, 98 },
    { 0xa833ea57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a3ea5c, 93 },
    { 0x6fc3ea5d, 78 },
    { 0xc843ea5e, 23 },
    { 0x4753ea5f, 46 },
    { 0xbd13ea5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3ea63, 33 },
    { 0, 0 },
    { 0x24a3ea65, 44 },
    { 0x5583ea66, 78 },
    { 0, 0 },
    { 0xb8f3ea68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4803ea71, 99 },
    { 0x5193ea72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3ea76, 90 },
    { 0, 0 },
    { 0x3d3ea78, 40 },
    { 0, 0 },
    { 0xe33ea7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec23ea7e, 95 },
    { 0xc9a3ea7e, 90 },
    { 0, 0 },
    { 0x3f83ea81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af3ea85, 87 },
    { 0x5333ea85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb3ea8b, 61 },
    { 0xcb33ea8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3ea95, 95 },
    { 0x9923ea95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc3ea9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f3eaa0, 99 },
    { 0x69f3eaa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd43eaa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903eaab, 23 },
    { 0xf763eaac, 98 },
    { 0x3ff3eaad, 90 },
    { 0xc323eaae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73eab1, 55 },
    { 0x5d33eab2, 43 },
    { 0xa9d3eab3, 95 },
    { 0xeb13eab4, 66 },
    { 0xfd33eab5, 61 },
    { 0, 0 },
    { 0x9063eab7, 33 },
    { 0x293eab8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43eabb, 23 },
    { 0xaf03eabc, 76 },
    { 0xd9f3eabb, 98 },
    { 0x43f3eabd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa13eac3, 61 },
    { 0x513eac4, 61 },
    { 0x1f33eac3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e3eac9, 90 },
    { 0xe003eaca, 61 },
    { 0, 0 },
    { 0x1ad3eacc, 95 },
    { 0, 0 },
    { 0xfd23eace, 34 },
    { 0, 0 },
    { 0x2583ead0, 61 },
    { 0xba23ead0, 98 },
    { 0xbed3ead2, 34 },
    { 0x8143ead3, 95 },
    { 0xaad3ead4, 90 },
    { 0, 0 },
    { 0xbd03ead6, 98 },
    { 0xce63ead6, 33 },
    { 0xd843ead6, 33 },
    { 0x6743ead8, 46 },
    { 0, 0 },
    { 0x3e63eadb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c3eae3, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x1033eae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc33eaed, 84 },
    { 0x5353eaee, 55 },
    { 0xace3eaef, 33 },
    { 0x8863eaee, 77 },
    { 0x6b53eaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e43eaf5, 95 },
    { 0xb3a3eaf5, 90 },
    { 0xbd73eaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353eafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a03eb01, 61 },
    { 0x1963eb01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53eb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b3eb0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3eb13, 33 },
    { 0xf243eb14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce3eb1f, 33 },
    { 0, 0 },
    { 0x7af3eb21, 76 },
    { 0x86c3eb22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4173eb25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3eb2c, 98 },
    { 0, 0 },
    { 0xa593eb2e, 95 },
    { 0x9f63eb2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f3eb32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba3eb35, 33 },
    { 0, 0 },
    { 0x193eb37, 33 },
    { 0x7843eb38, 69 },
    { 0x4d63eb37, 46 },
    { 0xc663eb38, 78 },
    { 0xddf3eb3b, 61 },
    { 0, 0 },
    { 0x4b43eb3d, 43 },
    { 0x7e43eb3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8683eb41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3eb46, 66 },
    { 0x7283eb46, 100 },
    { 0x1e33eb48, 91 },
    { 0, 0 },
    { 0x54d3eb4a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883eb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03eb56, 23 },
    { 0x8673eb56, 61 },
    { 0x6e43eb58, 61 },
    { 0xb2d3eb58, 33 },
    { 0x7663eb5a, 23 },
    { 0x8863eb5b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743eb5e, 95 },
    { 0xfdd3eb5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f3eb64, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3eb68, 85 },
    { 0, 0 },
    { 0xac73eb6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3eb71, 76 },
    { 0x6f13eb71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3eb75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4203eb79, 90 },
    { 0xb993eb7a, 55 },
    { 0x20f3eb7b, 100 },
    { 0x84e3eb7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b3eb84, 98 },
    { 0, 0 },
    { 0xa863eb86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2033eb8a, 55 },
    { 0, 0 },
    { 0x8b83eb8c, 98 },
    { 0xf523eb8d, 67 },
    { 0x15f3eb8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e13eb95, 90 },
    { 0, 0 },
    { 0x3223eb97, 90 },
    { 0, 0 },
    { 0x5063eb99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3eb9c, 23 },
    { 0x44a3eb9d, 95 },
    { 0x66c3eb9e, 90 },
    { 0x2693eb9e, 33 },
    { 0xe0b3eba0, 78 },
    { 0xb9b3eba0, 95 },
    { 0xce33eba0, 99 },
    { 0, 0 },
    { 0x65d3eba4, 95 },
    { 0x5423eba5, 95 },
    { 0x7143eba4, 98 },
    { 0x4e73eba7, 61 },
    { 0x80a3eba8, 85 },
    { 0x4113eba9, 23 },
    { 0, 0 },
    { 0x7b73ebab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ebae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3423ebb1, 68 },
    { 0x4bf3ebb2, 33 },
    { 0xad23ebb2, 90 },
    { 0xb083ebb4, 85 },
    { 0x4763ebb2, 55 },
    { 0, 0 },
    { 0x27c3ebb7, 94 },
    { 0x4d53ebb8, 90 },
    { 0x4e53ebb8, 98 },
    { 0x6193ebba, 43 },
    { 0, 0 },
    { 0xd7b3ebbc, 55 },
    { 0xec43ebbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb833ebc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d3ebcf, 78 },
    { 0xdda3ebcf, 90 },
    { 0, 0 },
    { 0xc6e3ebd2, 95 },
    { 0x9ec3ebd3, 94 },
    { 0, 0 },
    { 0x80d3ebd5, 88 },
    { 0, 0 },
    { 0xc13ebd7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73ebdc, 93 },
    { 0x1f63ebdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d3ebe0, 33 },
    { 0x9263ebe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93ebe6, 78 },
    { 0, 0 },
    { 0x5863ebe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe533ebee, 98 },
    { 0x76b3ebee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653ebf4, 61 },
    { 0xc503ebf5, 52 },
    { 0x5cd3ebf6, 100 },
    { 0xe193ebf7, 61 },
    { 0x9613ebf7, 98 },
    { 0xb243ebf9, 61 },
    { 0x2193ebfa, 93 },
    { 0xed03ebfa, 33 },
    { 0x6203ebf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf703ec04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xece3ec07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb513ec0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553ec13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5553ec19, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3143ec21, 90 },
    { 0x8f13ec21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3383ec26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb53ec29, 90 },
    { 0, 0 },
    { 0x5993ec2b, 66 },
    { 0x30f3ec2c, 95 },
    { 0xb023ec2c, 34 },
    { 0x2fd3ec2d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b3ec35, 98 },
    { 0x6163ec36, 34 },
    { 0x1e03ec37, 101 },
    { 0xecd3ec38, 95 },
    { 0, 0 },
    { 0x8783ec3a, 65 },
    { 0x7be3ec3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa403ec3f, 23 },
    { 0xa503ec40, 86 },
    { 0x77a3ec41, 23 },
    { 0, 0 },
    { 0xf6d3ec43, 90 },
    { 0xc203ec44, 33 },
    { 0xac03ec45, 95 },
    { 0, 0 },
    { 0xdef3ec47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3083ec4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d3ec57, 95 },
    { 0xdf73ec57, 34 },
    { 0x7d93ec59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e93ec64, 90 },
    { 0x4bf3ec65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b3ec6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b83ec72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd93ec76, 91 },
    { 0, 0 },
    { 0x3133ec78, 90 },
    { 0, 0 },
    { 0x2983ec7a, 61 },
    { 0xdd23ec7b, 98 },
    { 0, 0 },
    { 0x8343ec7d, 36 },
    { 0x9ad3ec7e, 33 },
    { 0, 0 },
    { 0x9563ec80, 61 },
    { 0x80f3ec81, 78 },
    { 0x1d83ec80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a83ec87, 20 },
    { 0xbb13ec88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd3ec8e, 52 },
    { 0xa43ec8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3ec93, 80 },
    { 0xc8a3ec93, 12 },
    { 0x4143ec95, 76 },
    { 0, 0 },
    { 0x8dd3ec97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b93ec9e, 100 },
    { 0x4dd3ec9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3eca2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83ecab, 76 },
    { 0xcab3ecab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4353ecb1, 90 },
    { 0, 0 },
    { 0x2bb3ecb3, 90 },
    { 0x76b3ecb4, 33 },
    { 0xf393ecb5, 100 },
    { 0xa633ecb5, 61 },
    { 0x6b43ecb7, 33 },
    { 0xbf83ecb7, 77 },
    { 0x7373ecb5, 66 },
    { 0x7823ecb7, 93 },
    { 0x6e33ecba, 93 },
    { 0, 0 },
    { 0x85e3ecbd, 78 },
    { 0x60c3ecbe, 88 },
    { 0x3a3ecbf, 90 },
    { 0x86a3ecc0, 76 },
    { 0x7363ecc1, 87 },
    { 0x43c3ecc2, 90 },
    { 0x70a3ecc3, 61 },
    { 0x8f83ecc1, 33 },
    { 0, 0 },
    { 0xf483ecc6, 90 },
    { 0x1503ecc7, 87 },
    { 0x483ecc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9353ecd1, 98 },
    { 0xb43ecd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa833ecd5, 76 },
    { 0x1893ecd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a3ecde, 23 },
    { 0x4d53ecdf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3ece3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf23ece9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc3ecef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3193ecf6, 90 },
    { 0x5923ecf6, 96 },
    { 0, 0 },
    { 0x8e63ecf9, 33 },
    { 0x3ecf9, 61 },
    { 0, 0 },
    { 0xfd23ecfc, 78 },
    { 0xd2b3ecfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f3ed02, 33 },
    { 0xba63ed02, 95 },
    { 0, 0 },
    { 0xd3ed05, 36 },
    { 0xd513ed06, 23 },
    { 0, 0 },
    { 0x74a3ed08, 90 },
    { 0x2e83ed08, 38 },
    { 0, 0 },
    { 0x1143ed0b, 95 },
    { 0, 0 },
    { 0x1893ed0d, 95 },
    { 0x2d93ed0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff3ed14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3ed1b, 98 },
    { 0, 0 },
    { 0x3d23ed1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c3ed22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd853ed25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8933ed2b, 23 },
    { 0x86e3ed2b, 34 },
    { 0x9de3ed2b, 23 },
    { 0x7453ed2e, 61 },
    { 0xea3ed2f, 87 },
    { 0x3433ed2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3563ed3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a23ed3e, 91 },
    { 0x5c03ed3f, 95 },
    { 0x2da3ed40, 99 },
    { 0, 0 },
    { 0x5c23ed42, 76 },
    { 0x1093ed43, 61 },
    { 0, 0 },
    { 0x7b13ed45, 95 },
    { 0, 0 },
    { 0x2d53ed47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b3ed4b, 76 },
    { 0x1343ed4c, 98 },
    { 0xbe13ed4c, 99 },
    { 0xd103ed4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf383ed52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2313ed55, 34 },
    { 0x5f93ed55, 90 },
    { 0x8e53ed57, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e93ed5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3ed63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef13ed71, 85 },
    { 0, 0 },
    { 0xed13ed73, 90 },
    { 0, 0 },
    { 0x4dc3ed75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd403ed79, 34 },
    { 0x6483ed79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d3ed7d, 90 },
    { 0x76e3ed7e, 61 },
    { 0x8c93ed7e, 23 },
    { 0x2403ed80, 85 },
    { 0, 0 },
    { 0xdef3ed82, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd3ed87, 61 },
    { 0x6823ed88, 61 },
    { 0x3c13ed89, 98 },
    { 0xc343ed89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7083ed8d, 52 },
    { 0x4e73ed8e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3ed97, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e3ed9c, 95 },
    { 0, 0 },
    { 0x123ed9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb063eda2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7273eda5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab3eda8, 77 },
    { 0x75f3eda8, 23 },
    { 0, 0 },
    { 0xc333edab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe3edaf, 67 },
    { 0xcaf3edb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1093edb3, 90 },
    { 0, 0 },
    { 0x6fd3edb5, 95 },
    { 0, 0 },
    { 0x81b3edb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603edba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c3edc0, 45 },
    { 0, 0 },
    { 0xc943edc2, 56 },
    { 0, 0 },
    { 0xea73edc4, 98 },
    { 0, 0 },
    { 0x24e3edc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba3edc9, 95 },
    { 0x7db3edca, 87 },
    { 0xec23edcb, 93 },
    { 0x2d13edca, 48 },
    { 0xa0f3edcd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb93edd2, 37 },
    { 0x1943edd3, 34 },
    { 0xc933edd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3eddb, 91 },
    { 0, 0 },
    { 0xdb33eddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473ede6, 33 },
    { 0xad43ede6, 33 },
    { 0x8693ede8, 55 },
    { 0x4a53ede8, 93 },
    { 0xe083ede8, 75 },
    { 0x7183ede7, 33 },
    { 0x4843edec, 96 },
    { 0x8403edec, 34 },
    { 0x1ac3eded, 33 },
    { 0x9593edef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc693edf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f3edf8, 95 },
    { 0xb213edf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3edfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7963ee01, 61 },
    { 0, 0 },
    { 0x5773ee03, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc613ee07, 61 },
    { 0x9c13ee07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb33ee0b, 93 },
    { 0x3673ee0b, 90 },
    { 0x2ed3ee0d, 90 },
    { 0, 0 },
    { 0x99b3ee0f, 88 },
    { 0, 0 },
    { 0x1203ee11, 90 },
    { 0xb9d3ee11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be3ee15, 55 },
    { 0, 0 },
    { 0x7f13ee17, 60 },
    { 0xfb73ee17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83ee1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e3ee24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda43ee29, 91 },
    { 0xf233ee2a, 55 },
    { 0x8733ee2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd143ee30, 46 },
    { 0x7cb3ee30, 55 },
    { 0, 0 },
    { 0x3233ee33, 90 },
    { 0xcb3ee34, 95 },
    { 0x1483ee34, 76 },
    { 0, 0 },
    { 0xda93ee37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5373ee3f, 90 },
    { 0xb383ee40, 95 },
    { 0x323ee40, 78 },
    { 0xca53ee42, 90 },
    { 0x1173ee41, 95 },
    { 0x4923ee40, 52 },
    { 0, 0 },
    { 0x3e63ee46, 61 },
    { 0xc9c3ee46, 100 },
    { 0x4f73ee48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5903ee4d, 68 },
    { 0, 0 },
    { 0x80c3ee4f, 92 },
    { 0, 0 },
    { 0x84a3ee51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993ee55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5653ee58, 33 },
    { 0x9333ee58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e3ee62, 87 },
    { 0x213ee63, 58 },
    { 0x9603ee62, 95 },
    { 0, 0 },
    { 0x5dc3ee66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb323ee69, 90 },
    { 0x8463ee69, 98 },
    { 0x853ee6a, 61 },
    { 0x4e53ee6a, 43 },
    { 0, 0 },
    { 0x9733ee6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393ee75, 33 },
    { 0x4a93ee76, 23 },
    { 0x57f3ee77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883ee7c, 52 },
    { 0x6a23ee7c, 33 },
    { 0xd8b3ee7e, 98 },
    { 0x3af3ee7f, 95 },
    { 0, 0 },
    { 0xe0c3ee81, 95 },
    { 0, 0 },
    { 0x1423ee83, 46 },
    { 0x4c33ee83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d3ee8a, 98 },
    { 0x8023ee8b, 43 },
    { 0xa983ee8c, 61 },
    { 0xeaa3ee8d, 84 },
    { 0xdae3ee8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc843ee94, 61 },
    { 0, 0 },
    { 0x1f73ee96, 98 },
    { 0, 0 },
    { 0xef23ee98, 33 },
    { 0x2563ee99, 33 },
    { 0xbac3ee99, 66 },
    { 0x2643ee9a, 12 },
    { 0, 0 },
    { 0x5cb3ee9d, 90 },
    { 0x2e33ee9e, 98 },
    { 0xf93ee9d, 90 },
    { 0xf0e3ee9f, 90 },
    { 0, 0 },
    { 0x5073eea2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e73eea5, 80 },
    { 0x9253eea5, 55 },
    { 0, 0 },
    { 0xe4f3eea8, 98 },
    { 0x1633eea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3eeb8, 84 },
    { 0x5f83eeb9, 95 },
    { 0xc3c3eeba, 98 },
    { 0xc2f3eeba, 78 },
    { 0x6113eebc, 77 },
    { 0, 0 },
    { 0xb603eebe, 46 },
    { 0, 0 },
    { 0xcff3eec0, 95 },
    { 0, 0 },
    { 0x54f3eec2, 34 },
    { 0x2443eec2, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3eec6, 88 },
    { 0, 0 },
    { 0x9353eec8, 76 },
    { 0xbb83eec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d3eecf, 61 },
    { 0x4143eecf, 33 },
    { 0xfa3eed0, 95 },
    { 0x1953eed2, 91 },
    { 0x9743eed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3eed6, 99 },
    { 0x6403eed7, 90 },
    { 0, 0 },
    { 0x9853eed9, 90 },
    { 0x473eeda, 76 },
    { 0xb033eedb, 46 },
    { 0xb2d3eedb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e13eedf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc43eee5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d73eee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb3eef1, 61 },
    { 0, 0 },
    { 0xd763eef3, 76 },
    { 0x3713eef3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd223eef9, 98 },
    { 0x7063eefa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883eefd, 34 },
    { 0x7353eefe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f13ef0a, 23 },
    { 0, 0 },
    { 0x1fa3ef0c, 55 },
    { 0xcb83ef0d, 52 },
    { 0, 0 },
    { 0xd903ef0f, 61 },
    { 0x62d3ef0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c53ef13, 78 },
    { 0, 0 },
    { 0x3fb3ef15, 9 },
    { 0xfa43ef16, 98 },
    { 0xdca3ef15, 95 },
    { 0, 0 },
    { 0x15d3ef19, 98 },
    { 0x44c3ef1a, 93 },
    { 0xa373ef1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf3ef25, 93 },
    { 0xf943ef26, 85 },
    { 0, 0 },
    { 0xdd83ef28, 65 },
    { 0x7103ef28, 76 },
    { 0, 0 },
    { 0x4403ef2b, 78 },
    { 0xa803ef2b, 52 },
    { 0, 0 },
    { 0xafb3ef2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2573ef32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103ef35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7653ef38, 33 },
    { 0x7f03ef38, 77 },
    { 0xdd73ef3a, 61 },
    { 0, 0 },
    { 0x5623ef3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb3ef42, 61 },
    { 0, 0 },
    { 0xd493ef44, 61 },
    { 0x14a3ef44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2653ef4c, 33 },
    { 0x1bf3ef4c, 90 },
    { 0xd143ef4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd3ef55, 55 },
    { 0x82f3ef56, 61 },
    { 0xa133ef55, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3ef5a, 90 },
    { 0, 0 },
    { 0x6353ef5c, 17 },
    { 0x7b3ef5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd3ef65, 23 },
    { 0, 0 },
    { 0x1e83ef67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a93ef6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae3ef6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d3ef78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413ef7b, 90 },
    { 0x4e63ef7c, 76 },
    { 0, 0 },
    { 0xdab3ef7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ef83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc73ef87, 78 },
    { 0, 0 },
    { 0x15d3ef89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5623ef8d, 90 },
    { 0, 0 },
    { 0xaf93ef8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb03ef92, 80 },
    { 0x7d43ef93, 68 },
    { 0, 0 },
    { 0x3a43ef95, 99 },
    { 0x2383ef96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff3ef9e, 42 },
    { 0x8653ef9f, 61 },
    { 0x99c3ef9f, 95 },
    { 0xcf13efa1, 90 },
    { 0x34a3ef9f, 23 },
    { 0x9363efa2, 23 },
    { 0xa9f3efa1, 33 },
    { 0x7603efa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63efa9, 76 },
    { 0, 0 },
    { 0xaee3efab, 43 },
    { 0xbc83efab, 67 },
    { 0xc4c3efad, 95 },
    { 0, 0 },
    { 0x33c3efaf, 100 },
    { 0, 0 },
    { 0x2083efb1, 33 },
    { 0xfd83efb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4053efb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab3efb9, 95 },
    { 0x33e3efba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe33efc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8943efc8, 46 },
    { 0xc333efc9, 61 },
    { 0xcd23efca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4603efd1, 33 },
    { 0xc5e3efd2, 93 },
    { 0x2bd3efd3, 95 },
    { 0xe823efd1, 76 },
    { 0xbc3efd3, 55 },
    { 0x92d3efd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a43efd9, 99 },
    { 0, 0 },
    { 0x5203efdb, 93 },
    { 0x2143efdc, 90 },
    { 0, 0 },
    { 0x7543efde, 76 },
    { 0, 0 },
    { 0x8033efe0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb973efe3, 66 },
    { 0, 0 },
    { 0xedd3efe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc63efea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993efee, 46 },
    { 0x62a3efef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc03effb, 55 },
    { 0xbca3effb, 67 },
    { 0x4bc3effd, 23 },
    { 0, 0 },
    { 0x5643efff, 61 },
    { 0x8ef3f000, 23 },
    { 0xc8f3efff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba3f007, 46 },
    { 0x53a3f008, 33 },
    { 0xb5a3f007, 90 },
    { 0xd2c3f008, 24 },
    { 0xa913f00b, 99 },
    { 0x5253f00b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f016, 66 },
    { 0x38b3f016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b13f01b, 88 },
    { 0x5363f01c, 85 },
    { 0xe53f01d, 87 },
    { 0xa593f01e, 76 },
    { 0xaed3f01b, 69 },
    { 0xc2e3f01e, 90 },
    { 0xc23f01d, 95 },
    { 0xb6c3f022, 95 },
    { 0x6693f023, 61 },
    { 0xd5e3f024, 92 },
    { 0x7423f024, 95 },
    { 0xa313f026, 95 },
    { 0x393f027, 98 },
    { 0xb873f01c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3f02b, 52 },
    { 0xcea3f02c, 88 },
    { 0xd8c3f02c, 95 },
    { 0, 0 },
    { 0x4a23f02f, 90 },
    { 0x2ed3f030, 61 },
    { 0, 0 },
    { 0x3cb3f032, 77 },
    { 0x93b3f033, 61 },
    { 0x7243f033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43f038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce3f03c, 90 },
    { 0x8b23f03d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a63f040, 90 },
    { 0x4b73f041, 34 },
    { 0xa6f3f041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d3f046, 23 },
    { 0x4ef3f046, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243f04b, 99 },
    { 0, 0 },
    { 0x1223f04d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e3f052, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x9653f055, 76 },
    { 0xe433f056, 94 },
    { 0xc4f3f055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9863f05b, 33 },
    { 0xb2e3f05b, 61 },
    { 0, 0 },
    { 0xcbb3f05e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x673f061, 34 },
    { 0, 0 },
    { 0xbb33f063, 90 },
    { 0, 0 },
    { 0xa3a3f065, 85 },
    { 0x7743f066, 80 },
    { 0x7653f067, 53 },
    { 0, 0 },
    { 0xff83f069, 23 },
    { 0xb723f06a, 51 },
    { 0x6573f069, 24 },
    { 0, 0 },
    { 0xcf43f06d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f3f070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf3f073, 93 },
    { 0x46e3f073, 33 },
    { 0, 0 },
    { 0x5463f076, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a3f079, 23 },
    { 0, 0 },
    { 0xede3f07b, 93 },
    { 0x5d13f07c, 75 },
    { 0x4823f07c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a23f084, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3f08f, 61 },
    { 0x7dd3f090, 61 },
    { 0, 0 },
    { 0x1c23f092, 33 },
    { 0x3f73f093, 90 },
    { 0x8b3f093, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a3f098, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3f09b, 93 },
    { 0, 0 },
    { 0x27d3f09d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c63f0a3, 61 },
    { 0, 0 },
    { 0xd63f0a5, 27 },
    { 0x8e73f0a5, 33 },
    { 0x14b3f0a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3f0aa, 75 },
    { 0x6793f0ab, 61 },
    { 0x3073f0ac, 67 },
    { 0xdb83f0ad, 66 },
    { 0xb423f0ae, 46 },
    { 0xf323f0ad, 85 },
    { 0xe1d3f0ae, 95 },
    { 0x3a3f0b1, 90 },
    { 0xd193f0b1, 90 },
    { 0, 0 },
    { 0xba33f0b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183f0ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3f0bd, 61 },
    { 0x673f0be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb33f0c7, 81 },
    { 0, 0 },
    { 0x8413f0c9, 66 },
    { 0x603f0ca, 46 },
    { 0x94a3f0cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd153f0ce, 81 },
    { 0x7493f0cf, 34 },
    { 0x4413f0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6563f0d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e33f0db, 88 },
    { 0, 0 },
    { 0x77a3f0dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a3f0e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f0e9, 34 },
    { 0, 0 },
    { 0xd7a3f0eb, 34 },
    { 0xa843f0eb, 95 },
    { 0xb833f0ed, 90 },
    { 0xb4b3f0ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d23f0f7, 90 },
    { 0, 0 },
    { 0xe023f0f9, 95 },
    { 0, 0 },
    { 0xf603f0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf3f0ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb123f108, 61 },
    { 0, 0 },
    { 0x63f10a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3f10d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2543f118, 76 },
    { 0xd7f3f119, 56 },
    { 0x5e3f119, 34 },
    { 0xed63f11b, 90 },
    { 0x90f3f119, 90 },
    { 0xa523f119, 55 },
    { 0xd193f11a, 46 },
    { 0, 0 },
    { 0xd943f120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d3f124, 76 },
    { 0, 0 },
    { 0xfd3f126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe103f129, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83f12d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d3f130, 75 },
    { 0x1b93f131, 95 },
    { 0, 0 },
    { 0xa473f133, 90 },
    { 0x8063f134, 61 },
    { 0xb2e3f135, 46 },
    { 0, 0 },
    { 0x2b23f137, 78 },
    { 0xff33f137, 55 },
    { 0, 0 },
    { 0xbeb3f13a, 98 },
    { 0x52a3f13a, 98 },
    { 0x77a3f13c, 66 },
    { 0x123f13b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea13f142, 93 },
    { 0xc73f142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e03f147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083f14e, 90 },
    { 0x1913f14f, 95 },
    { 0xcf63f150, 76 },
    { 0, 0 },
    { 0xae63f152, 90 },
    { 0, 0 },
    { 0x2d83f154, 52 },
    { 0x70a3f155, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3f158, 76 },
    { 0x1bf3f158, 85 },
    { 0xed53f159, 91 },
    { 0, 0 },
    { 0xb863f15c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873f160, 36 },
    { 0x343f161, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d3f16b, 23 },
    { 0xc3b3f16c, 76 },
    { 0, 0 },
    { 0x47a3f16e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723f173, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6863f177, 23 },
    { 0x4003f178, 90 },
    { 0xbe33f179, 61 },
    { 0, 0 },
    { 0x5d83f17b, 85 },
    { 0xd0e3f17c, 17 },
    { 0, 0 },
    { 0x99e3f17e, 17 },
    { 0x95d3f17f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c3f182, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc73f18d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a3f190, 61 },
    { 0xed43f191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a83f194, 90 },
    { 0x88d3f194, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf823f198, 77 },
    { 0xf973f199, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623f19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d3f1a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5483f1a6, 55 },
    { 0, 0 },
    { 0x7c13f1a8, 100 },
    { 0x1ae3f1a9, 87 },
    { 0xd213f1aa, 34 },
    { 0xf7c3f1a9, 90 },
    { 0x8033f1ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3f1b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323f1b4, 33 },
    { 0x8f73f1b4, 81 },
    { 0x893f1b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf3f1b9, 99 },
    { 0xc1a3f1ba, 23 },
    { 0x4d43f1bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda33f1be, 76 },
    { 0x8f33f1bf, 95 },
    { 0xb753f1c0, 23 },
    { 0x4723f1c1, 66 },
    { 0xf913f1be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f3f1c6, 43 },
    { 0x2433f1c7, 68 },
    { 0xd8c3f1c7, 98 },
    { 0, 0 },
    { 0x20f3f1ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93f1cd, 61 },
    { 0xcd33f1cd, 33 },
    { 0x7313f1ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d3f1d4, 87 },
    { 0x40e3f1d5, 90 },
    { 0x8013f1d6, 43 },
    { 0x1a93f1d7, 98 },
    { 0, 0 },
    { 0xd523f1d9, 61 },
    { 0xa813f1d9, 90 },
    { 0, 0 },
    { 0x7633f1dc, 23 },
    { 0xcc3f1dd, 66 },
    { 0x803f1de, 78 },
    { 0x8313f1df, 33 },
    { 0xabe3f1de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d3f1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5893f1ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e33f1f1, 33 },
    { 0x8203f1f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd203f1f7, 76 },
    { 0x873f1f8, 34 },
    { 0x53a3f1f8, 61 },
    { 0, 0 },
    { 0x8c43f1fb, 76 },
    { 0x6643f1fc, 90 },
    { 0xef83f1fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb93f203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf23f206, 76 },
    { 0xff3f207, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3f20a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3f20f, 45 },
    { 0, 0 },
    { 0x99d3f211, 34 },
    { 0, 0 },
    { 0x4ae3f213, 34 },
    { 0xb493f213, 93 },
    { 0x9ac3f213, 81 },
    { 0x7073f215, 98 },
    { 0x72d3f214, 43 },
    { 0x1393f218, 95 },
    { 0xc473f218, 90 },
    { 0xefd3f215, 90 },
    { 0x45e3f21b, 97 },
    { 0xb273f21c, 34 },
    { 0, 0 },
    { 0x8223f21e, 98 },
    { 0xc813f21e, 34 },
    { 0x9203f220, 92 },
    { 0x5df3f221, 33 },
    { 0xfe73f222, 61 },
    { 0xc113f21e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43f229, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253f231, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe33f235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003f238, 98 },
    { 0xe793f238, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13f23e, 61 },
    { 0, 0 },
    { 0x4573f240, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3f245, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643f248, 78 },
    { 0x7c53f248, 90 },
    { 0xd063f24a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023f250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db3f255, 76 },
    { 0x93f256, 23 },
    { 0, 0 },
    { 0x70c3f258, 61 },
    { 0xca33f259, 95 },
    { 0x7bc3f25a, 61 },
    { 0, 0 },
    { 0xe923f25c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053f261, 23 },
    { 0x563f261, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3f265, 76 },
    { 0xbac3f265, 23 },
    { 0x5343f267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b53f270, 95 },
    { 0, 0 },
    { 0x4cf3f272, 69 },
    { 0xa523f273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa23f278, 61 },
    { 0, 0 },
    { 0x25b3f27a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2883f27d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7193f280, 90 },
    { 0xa893f280, 88 },
    { 0xe953f282, 96 },
    { 0x7893f282, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc793f28b, 34 },
    { 0, 0 },
    { 0xdec3f28d, 95 },
    { 0xcf73f28e, 61 },
    { 0x54d3f28f, 34 },
    { 0xfde3f28e, 61 },
    { 0x8143f291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f299, 23 },
    { 0x63f29a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc253f2a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1753f2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde93f2b2, 46 },
    { 0xe783f2b3, 85 },
    { 0x2273f2b4, 33 },
    { 0x11c3f2b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab93f2be, 24 },
    { 0, 0 },
    { 0xc593f2c0, 90 },
    { 0x8053f2c1, 76 },
    { 0xa383f2c2, 17 },
    { 0xb9f3f2c3, 95 },
    { 0xe8e3f2c1, 23 },
    { 0x11f3f2c5, 75 },
    { 0x2473f2c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d3f2ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d13f2d6, 61 },
    { 0x15d3f2d6, 14 },
    { 0x52d3f2d8, 91 },
    { 0x8b73f2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3f2dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a3f2e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d3f2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e3f2f6, 33 },
    { 0x6563f2f6, 61 },
    { 0x2b03f2f8, 95 },
    { 0x6833f2f9, 76 },
    { 0x35e3f2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae3f2fd, 93 },
    { 0x7cf3f2fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c03f302, 93 },
    { 0x10b3f302, 95 },
    { 0, 0 },
    { 0xdaf3f305, 23 },
    { 0, 0 },
    { 0xeb53f307, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3f30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8053f30d, 55 },
    { 0x36b3f30e, 90 },
    { 0xd123f30e, 33 },
    { 0xbb53f310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b3f31b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3f322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa3f328, 87 },
    { 0, 0 },
    { 0x4173f32a, 90 },
    { 0, 0 },
    { 0xfac3f32c, 34 },
    { 0, 0 },
    { 0xb7d3f32e, 93 },
    { 0xe203f32f, 8 },
    { 0x5443f330, 33 },
    { 0x6623f32e, 33 },
    { 0x84b3f330, 33 },
    { 0, 0 },
    { 0x733f334, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd943f337, 76 },
    { 0x1613f338, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9733f33f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf83f344, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103f34e, 66 },
    { 0xfd23f34e, 90 },
    { 0x94c3f350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523f353, 78 },
    { 0x94e3f354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1483f358, 98 },
    { 0x7c33f358, 93 },
    { 0x94f3f35a, 23 },
    { 0xddf3f35a, 85 },
    { 0xbab3f35c, 33 },
    { 0x1e53f35c, 95 },
    { 0x5663f35c, 33 },
    { 0xe243f35f, 90 },
    { 0xb313f35b, 76 },
    { 0x6c03f361, 33 },
    { 0x8f63f35c, 61 },
    { 0, 0 },
    { 0xc443f364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5923f367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e3f36a, 61 },
    { 0x8293f36b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903f36f, 33 },
    { 0, 0 },
    { 0x33a3f371, 34 },
    { 0xa683f371, 65 },
    { 0x79b3f373, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2063f379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f63f37c, 33 },
    { 0, 0 },
    { 0xa3f37e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3f381, 52 },
    { 0x3673f381, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c63f385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd3f38b, 64 },
    { 0x5743f38b, 61 },
    { 0x24f3f38d, 90 },
    { 0xb6c3f38e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a73f392, 33 },
    { 0xb413f393, 100 },
    { 0xa3f393, 76 },
    { 0x9ab3f393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a43f39c, 9 },
    { 0x90e3f39d, 43 },
    { 0xead3f39e, 80 },
    { 0x7ba3f39f, 61 },
    { 0xae73f39e, 66 },
    { 0x60f3f39d, 76 },
    { 0x89c3f39e, 33 },
    { 0xf693f39d, 98 },
    { 0, 0 },
    { 0x1403f3a5, 55 },
    { 0xdb13f3a5, 98 },
    { 0x52a3f3a7, 76 },
    { 0x843f3a8, 93 },
    { 0xe6f3f3a9, 55 },
    { 0xc093f3a9, 23 },
    { 0x6713f3ab, 98 },
    { 0xedc3f3ac, 75 },
    { 0x5b03f3ad, 23 },
    { 0xbd63f3ad, 85 },
    { 0, 0 },
    { 0x2e73f3b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3f3ba, 66 },
    { 0xd073f3ba, 77 },
    { 0x1553f3bb, 90 },
    { 0x4143f3ba, 61 },
    { 0x5243f3be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc383f3c5, 98 },
    { 0x1953f3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e3f3cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3f3d0, 33 },
    { 0x2f93f3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf3f3d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d13f3d7, 84 },
    { 0, 0 },
    { 0x2993f3d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb903f3de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f3e2, 43 },
    { 0, 0 },
    { 0x2043f3e4, 90 },
    { 0, 0 },
    { 0xf803f3e6, 98 },
    { 0x76c3f3e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb873f3ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8633f3f4, 88 },
    { 0xc1b3f3f4, 95 },
    { 0xa023f3f6, 95 },
    { 0x7473f3f7, 76 },
    { 0, 0 },
    { 0x92e3f3f9, 95 },
    { 0x333f3fa, 100 },
    { 0x55d3f3fb, 46 },
    { 0xd053f3fb, 76 },
    { 0xcef3f3fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb3f402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb283f405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a53f40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed23f419, 90 },
    { 0x2683f41a, 98 },
    { 0x1693f41b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3f41e, 23 },
    { 0xfe03f41f, 34 },
    { 0xd7f3f41f, 77 },
    { 0x4523f41f, 90 },
    { 0x6e23f422, 23 },
    { 0, 0 },
    { 0xe183f424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663f42a, 99 },
    { 0x4fe3f42b, 90 },
    { 0x7a03f42c, 70 },
    { 0, 0 },
    { 0x4c63f42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b13f435, 87 },
    { 0x933f436, 98 },
    { 0, 0 },
    { 0xbe73f438, 90 },
    { 0x5e3f439, 55 },
    { 0x1563f43a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c3f43d, 90 },
    { 0xdd23f43e, 33 },
    { 0x5053f43f, 95 },
    { 0xcf43f440, 90 },
    { 0, 0 },
    { 0x4b13f442, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de3f445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4513f458, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c3f45d, 23 },
    { 0, 0 },
    { 0x43f45f, 69 },
    { 0xd103f460, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a3f466, 99 },
    { 0x7773f467, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a3f46a, 100 },
    { 0x7043f46b, 65 },
    { 0x22a3f46c, 61 },
    { 0xf973f46c, 87 },
    { 0x4ce3f46b, 55 },
    { 0xcf83f46b, 90 },
    { 0x22b3f46b, 90 },
    { 0x90d3f46b, 98 },
    { 0, 0 },
    { 0xf3e3f473, 65 },
    { 0xa683f473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf533f479, 33 },
    { 0xbd23f47a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x483f47d, 33 },
    { 0x2933f47d, 95 },
    { 0xe443f47f, 93 },
    { 0, 0 },
    { 0xef93f481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c83f489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a3f48c, 33 },
    { 0, 0 },
    { 0x77d3f48e, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a3f493, 33 },
    { 0x2c73f494, 33 },
    { 0x8b53f495, 61 },
    { 0x4643f495, 23 },
    { 0x55c3f494, 55 },
    { 0x5053f498, 34 },
    { 0x33c3f499, 90 },
    { 0xd4f3f49a, 93 },
    { 0x8d73f49a, 90 },
    { 0xe43f49c, 61 },
    { 0x2a83f49c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e3f4a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c3f4a3, 92 },
    { 0xb343f4a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f4ae, 55 },
    { 0x3f83f4af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e03f4b4, 67 },
    { 0xc033f4b4, 23 },
    { 0, 0 },
    { 0x8da3f4b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513f4bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553f4bf, 61 },
    { 0x5a83f4c0, 33 },
    { 0x6db3f4c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803f4c9, 98 },
    { 0, 0 },
    { 0xc1e3f4cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73f4cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7313f4d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x723f4d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f3f4db, 61 },
    { 0xb6a3f4db, 95 },
    { 0xef43f4db, 33 },
    { 0x2a83f4de, 99 },
    { 0x87c3f4db, 61 },
    { 0x79e3f4db, 46 },
    { 0x9d63f4dd, 17 },
    { 0x7da3f4e2, 61 },
    { 0x2ef3f4e0, 33 },
    { 0x5763f4e4, 76 },
    { 0x7d43f4e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963f4ed, 55 },
    { 0x5d13f4ed, 33 },
    { 0x3d13f4ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433f4fb, 76 },
    { 0xecd3f4fb, 100 },
    { 0, 0 },
    { 0x59a3f4fe, 90 },
    { 0x293f4fe, 23 },
    { 0x3a23f4ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f3f503, 55 },
    { 0x1e13f504, 66 },
    { 0, 0 },
    { 0xf813f506, 84 },
    { 0x3033f507, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2383f50a, 90 },
    { 0x6333f50b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3f50e, 52 },
    { 0x9643f50e, 33 },
    { 0xeba3f510, 33 },
    { 0xc113f510, 95 },
    { 0xfb83f512, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e53f517, 66 },
    { 0x5713f518, 33 },
    { 0x2bf3f518, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5593f51c, 55 },
    { 0, 0 },
    { 0xa6a3f51e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c43f523, 23 },
    { 0xf753f524, 87 },
    { 0xbe13f525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663f528, 33 },
    { 0, 0 },
    { 0x1b63f52a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6953f52d, 43 },
    { 0xb553f52d, 34 },
    { 0xfb53f52d, 77 },
    { 0x7ed3f52e, 98 },
    { 0xeed3f531, 61 },
    { 0xb63f532, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb713f535, 91 },
    { 0, 0 },
    { 0xbd53f537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb093f53b, 98 },
    { 0xe633f53b, 55 },
    { 0x8d53f53d, 23 },
    { 0x7663f53e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793f543, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca3f546, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc463f54a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43f54e, 66 },
    { 0xa0d3f54f, 98 },
    { 0, 0 },
    { 0x91c3f551, 55 },
    { 0x8793f552, 95 },
    { 0xbd93f553, 90 },
    { 0xb1b3f553, 90 },
    { 0x2693f555, 33 },
    { 0x3963f551, 33 },
    { 0xbf23f552, 90 },
    { 0xf843f552, 98 },
    { 0x823f559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3f55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff93f562, 34 },
    { 0xd603f562, 66 },
    { 0xb593f564, 36 },
    { 0x6a13f565, 34 },
    { 0x6643f565, 98 },
    { 0x6273f567, 33 },
    { 0x5ea3f568, 33 },
    { 0xd853f568, 43 },
    { 0x31d3f56a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3f576, 33 },
    { 0xee53f576, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc903f57b, 36 },
    { 0, 0 },
    { 0x56a3f57d, 52 },
    { 0xf353f57d, 98 },
    { 0x68e3f57e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b3f583, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453f588, 61 },
    { 0x7363f589, 77 },
    { 0x58f3f58a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3f591, 76 },
    { 0x2723f592, 23 },
    { 0xa863f593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa373f596, 97 },
    { 0xb03f596, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa343f59c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e03f5a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5a5, 80 },
    { 0x8e3f5a6, 33 },
    { 0x203f5a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3f5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f3f5b2, 34 },
    { 0, 0 },
    { 0x6ae3f5b4, 33 },
    { 0x2e23f5b4, 90 },
    { 0, 0 },
    { 0x4603f5b7, 98 },
    { 0, 0 },
    { 0xf6b3f5b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f03f5bf, 46 },
    { 0xe313f5bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba3f5c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe983f5c6, 67 },
    { 0, 0 },
    { 0x17c3f5c8, 95 },
    { 0, 0 },
    { 0x7b03f5ca, 65 },
    { 0xe793f5cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6463f5d0, 75 },
    { 0x6ff3f5d1, 76 },
    { 0x9713f5d2, 61 },
    { 0x50d3f5d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043f5d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c63f5dc, 95 },
    { 0x6373f5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163f5e4, 95 },
    { 0x5a13f5e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8583f5ee, 61 },
    { 0, 0 },
    { 0x6ef3f5f0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f5f3, 77 },
    { 0x2f63f5f4, 95 },
    { 0xea83f5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa293f601, 2 },
    { 0, 0 },
    { 0xf0d3f603, 33 },
    { 0, 0 },
    { 0x6613f605, 90 },
    { 0xe493f605, 98 },
    { 0x2ff3f606, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b3f60b, 90 },
    { 0, 0 },
    { 0xba83f60d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813f610, 23 },
    { 0, 0 },
    { 0xd563f612, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb83f617, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa373f61a, 61 },
    { 0xabd3f61b, 76 },
    { 0xf583f61b, 87 },
    { 0, 0 },
    { 0xa363f61e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8603f621, 76 },
    { 0x4333f622, 33 },
    { 0xbd83f623, 77 },
    { 0x3263f623, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f3f62f, 81 },
    { 0, 0 },
    { 0x473f631, 90 },
    { 0x9413f632, 33 },
    { 0x58c3f632, 90 },
    { 0x4133f633, 43 },
    { 0x19f3f635, 46 },
    { 0x9113f631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53f63a, 90 },
    { 0, 0 },
    { 0x9a03f63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3f63f, 85 },
    { 0, 0 },
    { 0xbd73f641, 84 },
    { 0x9973f642, 95 },
    { 0, 0 },
    { 0xa343f644, 55 },
    { 0xba93f645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5753f648, 52 },
    { 0, 0 },
    { 0x6753f64a, 61 },
    { 0x943f64b, 33 },
    { 0, 0 },
    { 0x9d93f64d, 98 },
    { 0xdec3f64d, 33 },
    { 0x42a3f64f, 33 },
    { 0xdd83f650, 23 },
    { 0xe8f3f650, 46 },
    { 0xadd3f652, 90 },
    { 0x4fb3f652, 95 },
    { 0x5f93f654, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb3f657, 91 },
    { 0xa393f657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3f65d, 23 },
    { 0, 0 },
    { 0xc913f65f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1423f66a, 99 },
    { 0xdbc3f66b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc03f670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3f67c, 67 },
    { 0x4413f67c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783f682, 90 },
    { 0, 0 },
    { 0x6ad3f684, 93 },
    { 0xf173f685, 61 },
    { 0xc6e3f685, 90 },
    { 0xd623f685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee3f68c, 23 },
    { 0xd393f68d, 33 },
    { 0x3753f68c, 34 },
    { 0xa523f68f, 43 },
    { 0xcd73f690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e3f693, 33 },
    { 0, 0 },
    { 0x8ea3f695, 90 },
    { 0xa853f696, 34 },
    { 0x57f3f696, 76 },
    { 0xc123f698, 76 },
    { 0, 0 },
    { 0x8e53f69a, 100 },
    { 0x82f3f69a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb3f69e, 98 },
    { 0x80d3f69f, 90 },
    { 0, 0 },
    { 0x5143f6a1, 90 },
    { 0x5d53f6a2, 90 },
    { 0, 0 },
    { 0xaee3f6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9313f6a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f6ae, 76 },
    { 0x6153f6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e3f6b4, 95 },
    { 0x623f6b5, 65 },
    { 0x4153f6b6, 76 },
    { 0xbd33f6b7, 61 },
    { 0x16f3f6b8, 76 },
    { 0xd183f6b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a3f6bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a3f6c0, 87 },
    { 0, 0 },
    { 0xf683f6c2, 61 },
    { 0x7913f6c3, 40 },
    { 0x7373f6c3, 46 },
    { 0xc7b3f6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df3f6d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d3f6d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393f6dd, 66 },
    { 0x1a73f6de, 98 },
    { 0xff83f6df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e33f6e2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d3f6e9, 75 },
    { 0x5c23f6ea, 98 },
    { 0x5323f6e9, 84 },
    { 0x6d93f6e9, 43 },
    { 0x3693f6ed, 55 },
    { 0, 0 },
    { 0x9ff3f6ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4403f6f3, 46 },
    { 0x1303f6f4, 58 },
    { 0x7aa3f6f5, 68 },
    { 0x1113f6f6, 51 },
    { 0xf4c3f6f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33f6fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23f6ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3093f704, 98 },
    { 0, 0 },
    { 0xdfe3f706, 33 },
    { 0x9cd3f706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763f70b, 61 },
    { 0, 0 },
    { 0x2ba3f70d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea3f711, 23 },
    { 0xa463f711, 75 },
    { 0xa033f713, 33 },
    { 0, 0 },
    { 0x9333f715, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13f71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b3f71e, 81 },
    { 0xaf33f71f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be3f722, 23 },
    { 0x6073f722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff3f726, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13f72c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf633f731, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe173f736, 98 },
    { 0xbe93f737, 52 },
    { 0x4663f738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713f73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa3f743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973f747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb803f74a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f3f74e, 98 },
    { 0x87e3f74f, 61 },
    { 0, 0 },
    { 0x363f751, 43 },
    { 0, 0 },
    { 0x4493f753, 84 },
    { 0x4363f754, 61 },
    { 0, 0 },
    { 0xaf23f756, 95 },
    { 0, 0 },
    { 0x9403f758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe993f75d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca3f762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4253f767, 56 },
    { 0x7193f768, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa973f76b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8093f76f, 98 },
    { 0x4753f76f, 77 },
    { 0x5e83f771, 90 },
    { 0x6f03f772, 88 },
    { 0x2d43f771, 98 },
    { 0x4873f774, 76 },
    { 0xd063f775, 95 },
    { 0x3da3f776, 98 },
    { 0xe3a3f775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e23f77c, 34 },
    { 0xe673f77d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3783f780, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5543f784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493f789, 61 },
    { 0, 0 },
    { 0xfb53f78b, 43 },
    { 0xe9b3f78c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c3f78f, 43 },
    { 0x5953f790, 84 },
    { 0, 0 },
    { 0x8b43f792, 77 },
    { 0xf563f793, 74 },
    { 0xa153f792, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323f798, 95 },
    { 0xc6c3f798, 23 },
    { 0x9c63f79a, 90 },
    { 0x3973f798, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c3f79f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83f7af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb3f7b2, 98 },
    { 0x56a3f7b2, 66 },
    { 0x8913f7b4, 33 },
    { 0, 0 },
    { 0x9493f7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33f7ba, 90 },
    { 0x3e03f7ba, 34 },
    { 0x7c83f7ba, 90 },
    { 0x2963f7bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3f7c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3133f7c5, 90 },
    { 0, 0 },
    { 0xbbd3f7c7, 98 },
    { 0x8433f7c7, 66 },
    { 0x3c53f7c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93f7ce, 90 },
    { 0x2533f7ce, 98 },
    { 0x7c3f7d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3f7d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c93f7d8, 33 },
    { 0xa653f7d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113f7dd, 87 },
    { 0xc53f7de, 77 },
    { 0x5c43f7df, 100 },
    { 0, 0 },
    { 0x7933f7e1, 33 },
    { 0, 0 },
    { 0x88c3f7e3, 100 },
    { 0x9e93f7e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3f7e8, 23 },
    { 0x2153f7e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac33f7ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d93f7f1, 61 },
    { 0x1d33f7f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83f7f6, 23 },
    { 0xd73f7f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe3f7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c3f801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f3f804, 23 },
    { 0xd233f805, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe513f808, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb3f80c, 55 },
    { 0x30d3f80d, 61 },
    { 0x8173f80e, 43 },
    { 0x4823f80f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b63f818, 61 },
    { 0x9db3f819, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4273f81d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4673f821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb373f825, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c3f829, 95 },
    { 0x7a03f82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983f838, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3f83e, 33 },
    { 0xb0a3f83f, 76 },
    { 0xca23f83f, 90 },
    { 0xa8a3f83f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93f845, 53 },
    { 0xd1a3f846, 23 },
    { 0x9f53f847, 95 },
    { 0xcde3f846, 100 },
    { 0x9f73f846, 95 },
    { 0x5d03f84a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa203f853, 34 },
    { 0, 0 },
    { 0x4da3f855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af3f85d, 100 },
    { 0x2a13f85e, 39 },
    { 0x8b3f85f, 93 },
    { 0x90c3f85e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee33f863, 88 },
    { 0, 0 },
    { 0x4a73f865, 76 },
    { 0, 0 },
    { 0xf693f867, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4493f86a, 23 },
    { 0x1c13f86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd63f870, 90 },
    { 0, 0 },
    { 0x2fa3f872, 91 },
    { 0x4fd3f873, 55 },
    { 0x3b13f872, 66 },
    { 0, 0 },
    { 0xc283f876, 61 },
    { 0x4603f877, 81 },
    { 0, 0 },
    { 0x4a93f879, 90 },
    { 0x76b3f879, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b3f880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3f883, 87 },
    { 0x4b03f884, 98 },
    { 0, 0 },
    { 0xfcc3f886, 93 },
    { 0, 0 },
    { 0xdc83f888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7033f88b, 90 },
    { 0x3cb3f88b, 55 },
    { 0x2053f88d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993f890, 84 },
    { 0x623f890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423f895, 98 },
    { 0x1ca3f896, 33 },
    { 0xb5a3f897, 61 },
    { 0, 0 },
    { 0x1f23f899, 33 },
    { 0, 0 },
    { 0x1043f89b, 33 },
    { 0, 0 },
    { 0x6ab3f89d, 23 },
    { 0xde03f89e, 95 },
    { 0xe53f89e, 33 },
    { 0x1ac3f8a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8033f8a3, 33 },
    { 0xa5c3f8a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1503f8a8, 90 },
    { 0xc153f8a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade3f8ae, 67 },
    { 0xa733f8af, 90 },
    { 0xbe33f8b0, 34 },
    { 0x17e3f8ae, 75 },
    { 0x9143f8b2, 43 },
    { 0x60f3f8b3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e3f8b6, 33 },
    { 0, 0 },
    { 0x75f3f8b8, 77 },
    { 0x4e33f8b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3f8bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d33f8c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b3f8c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3f8ca, 91 },
    { 0x6f53f8ca, 52 },
    { 0xa8c3f8cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93f8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23f8d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf613f8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3f8dc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3473f8e4, 46 },
    { 0xca33f8e5, 55 },
    { 0xfe63f8e5, 78 },
    { 0x5223f8e6, 23 },
    { 0x4d43f8e8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3f8ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3f8f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd3f8f5, 61 },
    { 0xc793f8f6, 33 },
    { 0, 0 },
    { 0xba03f8f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3f8fb, 61 },
    { 0, 0 },
    { 0xc203f8fd, 90 },
    { 0, 0 },
    { 0x3953f8ff, 85 },
    { 0x6643f900, 61 },
    { 0x6f03f8ff, 23 },
    { 0xd3a3f902, 34 },
    { 0x23d3f903, 46 },
    { 0x1bd3f902, 95 },
    { 0xaf93f905, 61 },
    { 0, 0 },
    { 0xd363f907, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523f90a, 33 },
    { 0x5ef3f90b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713f910, 95 },
    { 0xb2a3f911, 98 },
    { 0, 0 },
    { 0x80e3f913, 98 },
    { 0x463f913, 99 },
    { 0, 0 },
    { 0x953f916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe863f91a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e63f91e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde3f921, 55 },
    { 0x7ac3f921, 33 },
    { 0xa973f922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d73f926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc033f92a, 95 },
    { 0, 0 },
    { 0xa493f92c, 34 },
    { 0, 0 },
    { 0xaea3f92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63f936, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233f93e, 81 },
    { 0xe123f93f, 98 },
    { 0xc283f93f, 87 },
    { 0x1d3f940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b3f948, 33 },
    { 0x2123f949, 98 },
    { 0, 0 },
    { 0x5a63f94b, 23 },
    { 0x7703f94c, 75 },
    { 0x29f3f94d, 61 },
    { 0xb623f94e, 90 },
    { 0x5253f94f, 76 },
    { 0xfce3f950, 90 },
    { 0xdfc3f950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d63f954, 33 },
    { 0x7e43f955, 90 },
    { 0, 0 },
    { 0xa5f3f957, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4483f95b, 65 },
    { 0, 0 },
    { 0x9123f95d, 84 },
    { 0xb733f95e, 93 },
    { 0x9723f95f, 87 },
    { 0xa6a3f960, 90 },
    { 0xb883f961, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d43f965, 61 },
    { 0, 0 },
    { 0x9fb3f967, 95 },
    { 0, 0 },
    { 0x69e3f969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4333f96d, 98 },
    { 0xe973f96d, 61 },
    { 0, 0 },
    { 0xd393f970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b3f973, 99 },
    { 0xe5d3f973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3f97e, 43 },
    { 0x7d23f97f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2933f986, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b93f989, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a3f992, 98 },
    { 0x31c3f992, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e73f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f3f99c, 34 },
    { 0xef53f99d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63f9a0, 66 },
    { 0x9c03f9a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d13f9a4, 87 },
    { 0x2bd3f9a5, 55 },
    { 0x42d3f9a6, 33 },
    { 0xd833f9a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093f9aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c3f9ad, 19 },
    { 0, 0 },
    { 0x3893f9af, 61 },
    { 0, 0 },
    { 0xb3d3f9b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3f9b5, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff3f9bb, 67 },
    { 0x4013f9bb, 23 },
    { 0, 0 },
    { 0xc183f9be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a73f9c2, 90 },
    { 0xd533f9c3, 95 },
    { 0, 0 },
    { 0xa2a3f9c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5113f9cb, 90 },
    { 0x6ee3f9cc, 91 },
    { 0xbfa3f9cd, 95 },
    { 0x5623f9ce, 76 },
    { 0x3353f9cf, 23 },
    { 0x3853f9d0, 98 },
    { 0xe2b3f9cd, 67 },
    { 0, 0 },
    { 0x33f9d3, 33 },
    { 0, 0 },
    { 0xbb93f9d5, 87 },
    { 0xb2d3f9d6, 33 },
    { 0x3b13f9d6, 85 },
    { 0xf7a3f9d7, 95 },
    { 0xdbe3f9d7, 23 },
    { 0x27b3f9da, 95 },
    { 0x3373f9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93f9e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf63f9e9, 23 },
    { 0xd463f9ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313f9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f9f8, 43 },
    { 0x8af3f9f8, 61 },
    { 0, 0 },
    { 0xe313f9fb, 95 },
    { 0xef03f9fc, 23 },
    { 0, 0 },
    { 0x9913f9fe, 33 },
    { 0xeaf3f9ff, 23 },
    { 0x3743fa00, 95 },
    { 0x1a43fa00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe3fa06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe043fa0c, 91 },
    { 0x64b3fa0d, 65 },
    { 0x7123fa0d, 33 },
    { 0x3a13fa0f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb903fa12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6683fa17, 87 },
    { 0x6583fa18, 61 },
    { 0xed03fa18, 55 },
    { 0xee63fa1a, 90 },
    { 0xd83fa1b, 42 },
    { 0x9553fa1c, 76 },
    { 0, 0 },
    { 0x2e13fa1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe563fa22, 87 },
    { 0x2373fa22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c33fa27, 17 },
    { 0xf753fa27, 55 },
    { 0x663fa29, 67 },
    { 0xd4b3fa2a, 81 },
    { 0, 0 },
    { 0xd833fa2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533fa32, 61 },
    { 0xf683fa32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e53fa39, 83 },
    { 0x1263fa3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33fa3e, 61 },
    { 0x8583fa3f, 98 },
    { 0xd043fa3e, 61 },
    { 0xf263fa41, 51 },
    { 0x5ee3fa42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53fa46, 33 },
    { 0xd143fa46, 61 },
    { 0xf5a3fa48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f3fa4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b53fa54, 90 },
    { 0xeaf3fa55, 23 },
    { 0xdb73fa56, 95 },
    { 0xb5b3fa56, 76 },
    { 0x7543fa58, 95 },
    { 0x94a3fa57, 95 },
    { 0x4083fa5a, 61 },
    { 0, 0 },
    { 0x32b3fa5c, 33 },
    { 0, 0 },
    { 0xc383fa5e, 23 },
    { 0xb563fa5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a93fa62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd33fa66, 90 },
    { 0xcb43fa66, 101 },
    { 0xe03fa67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf663fa6e, 23 },
    { 0, 0 },
    { 0xb703fa70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1063fa78, 66 },
    { 0xc93fa79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b3fa7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53fa85, 78 },
    { 0xc1e3fa86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e83fa89, 95 },
    { 0x4b03fa89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da3fa8f, 78 },
    { 0x2cf3fa8f, 61 },
    { 0, 0 },
    { 0xd733fa92, 46 },
    { 0, 0 },
    { 0x1e73fa94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f3fa97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d3fa9b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c3faa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593faa9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b33fab3, 99 },
    { 0x6253fab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d3fab7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3fabb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe3fabe, 34 },
    { 0xc833fabe, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec53fac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1043facb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5893face, 61 },
    { 0xf913facf, 33 },
    { 0xbc53fad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x433fad3, 33 },
    { 0xb2b3fad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b3fadb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f93fade, 23 },
    { 0, 0 },
    { 0x1ac3fae0, 61 },
    { 0, 0 },
    { 0xa143fae2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8043fae5, 95 },
    { 0, 0 },
    { 0x28c3fae7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc3faf0, 43 },
    { 0x8b23faf1, 98 },
    { 0x1a03faf2, 56 },
    { 0, 0 },
    { 0x5253faf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef3fafa, 90 },
    { 0x38c3fafa, 98 },
    { 0x6b83fafb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab83faff, 83 },
    { 0, 0 },
    { 0xadf3fb01, 98 },
    { 0xeb63fb02, 33 },
    { 0xf313fb02, 23 },
    { 0x71e3fb01, 95 },
    { 0, 0 },
    { 0x2d23fb06, 76 },
    { 0xfa13fb07, 90 },
    { 0, 0 },
    { 0x2993fb09, 95 },
    { 0, 0 },
    { 0xab93fb0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d3fb0f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a3fb1d, 90 },
    { 0xd583fb1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9253fb22, 76 },
    { 0, 0 },
    { 0x1323fb24, 100 },
    { 0, 0 },
    { 0xe753fb26, 95 },
    { 0xb483fb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3fb2f, 61 },
    { 0x6653fb30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a3fb33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43fb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7913fb3c, 95 },
    { 0x8f83fb3c, 87 },
    { 0x9053fb3e, 33 },
    { 0, 0 },
    { 0x1d83fb40, 93 },
    { 0x7e93fb40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3213fb44, 91 },
    { 0x7423fb44, 93 },
    { 0x9483fb44, 78 },
    { 0xb023fb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f3fb4a, 67 },
    { 0x5483fb4b, 84 },
    { 0, 0 },
    { 0xda23fb4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce3fb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243fb56, 17 },
    { 0, 0 },
    { 0xcc03fb58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded3fb5c, 33 },
    { 0x79d3fb5c, 95 },
    { 0x1c13fb5d, 84 },
    { 0, 0 },
    { 0xc423fb60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e13fb63, 61 },
    { 0, 0 },
    { 0xe23fb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb353fb6c, 52 },
    { 0xf773fb6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9443fb72, 95 },
    { 0x5753fb73, 88 },
    { 0xc653fb74, 76 },
    { 0xaa33fb73, 46 },
    { 0, 0 },
    { 0x7e63fb77, 90 },
    { 0x9d93fb78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3fb7e, 90 },
    { 0x6e53fb7f, 90 },
    { 0, 0 },
    { 0x4723fb81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3fb90, 90 },
    { 0x6c63fb91, 100 },
    { 0x8123fb91, 95 },
    { 0, 0 },
    { 0x3223fb94, 33 },
    { 0x3953fb95, 33 },
    { 0xc2f3fb96, 40 },
    { 0x5ef3fb97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a3fb9b, 55 },
    { 0, 0 },
    { 0xff03fb9d, 90 },
    { 0xbc53fb9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973fba1, 93 },
    { 0x39c3fba2, 33 },
    { 0xf053fba3, 61 },
    { 0xac23fba2, 61 },
    { 0xbe43fba3, 88 },
    { 0x97b3fba6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43fbab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc3fbb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323fbb9, 90 },
    { 0x8993fbb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc933fbbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3fbc4, 98 },
    { 0xee73fbc5, 93 },
    { 0xe3e3fbc4, 95 },
    { 0x20e3fbc7, 61 },
    { 0x82b3fbc8, 34 },
    { 0x96e3fbc9, 23 },
    { 0xae93fbca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b3fbcd, 90 },
    { 0, 0 },
    { 0x5873fbcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a53fbd2, 55 },
    { 0xa553fbd3, 93 },
    { 0x5653fbd4, 90 },
    { 0x1333fbd5, 61 },
    { 0x6ac3fbd4, 34 },
    { 0x7913fbd7, 53 },
    { 0xab93fbd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2303fbdf, 90 },
    { 0x93c3fbe0, 85 },
    { 0xa2a3fbe0, 90 },
    { 0x4ac3fbe2, 33 },
    { 0xdd73fbe3, 93 },
    { 0x22e3fbe2, 93 },
    { 0x3633fbe4, 95 },
    { 0x87e3fbe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063fbea, 90 },
    { 0x45c3fbea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe923fbf1, 61 },
    { 0x6e33fbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed93fbfc, 87 },
    { 0xaf43fbfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d3fc01, 23 },
    { 0xa6a3fc02, 95 },
    { 0x18d3fc03, 61 },
    { 0x77f3fc04, 90 },
    { 0xbdb3fc05, 91 },
    { 0x6053fc01, 34 },
    { 0x51c3fc02, 76 },
    { 0, 0 },
    { 0xdf83fc09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb073fc0e, 17 },
    { 0x5023fc0f, 55 },
    { 0xf73fc10, 100 },
    { 0x1883fc0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb3fc15, 33 },
    { 0xb6f3fc16, 93 },
    { 0x4e13fc17, 77 },
    { 0x1243fc17, 95 },
    { 0xc2c3fc15, 76 },
    { 0, 0 },
    { 0xf7f3fc1b, 98 },
    { 0, 0 },
    { 0x5a03fc1d, 67 },
    { 0xcf53fc1d, 98 },
    { 0x7e3fc1e, 23 },
    { 0xa3fc1d, 76 },
    { 0, 0 },
    { 0xa163fc22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee53fc28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe253fc2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1863fc2e, 91 },
    { 0x5033fc2f, 61 },
    { 0x7053fc2f, 55 },
    { 0xd903fc2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b3fc37, 90 },
    { 0, 0 },
    { 0x7173fc39, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3803fc49, 36 },
    { 0, 0 },
    { 0xe423fc4b, 46 },
    { 0x5ba3fc4b, 95 },
    { 0, 0 },
    { 0x19d3fc4e, 61 },
    { 0x22d3fc4f, 65 },
    { 0x6193fc50, 33 },
    { 0xfa03fc51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3fc55, 61 },
    { 0x27d3fc56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1333fc5a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c3fc5f, 90 },
    { 0x8703fc5f, 34 },
    { 0x80a3fc5f, 24 },
    { 0xf5e3fc62, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d3fc68, 99 },
    { 0x4373fc69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe3fc72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab3fc77, 90 },
    { 0x67c3fc77, 76 },
    { 0x15c3fc77, 95 },
    { 0x6b03fc79, 61 },
    { 0, 0 },
    { 0x2e73fc7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd893fc7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3fc82, 23 },
    { 0xd0b3fc82, 85 },
    { 0x9313fc83, 95 },
    { 0xdb33fc85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3fc8e, 96 },
    { 0xb9e3fc8f, 99 },
    { 0, 0 },
    { 0x4543fc91, 90 },
    { 0xade3fc91, 66 },
    { 0x1ce3fc91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed03fc9a, 33 },
    { 0xa3e3fc9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a3fc9f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b13fca3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d3fca8, 76 },
    { 0, 0 },
    { 0xcef3fcaa, 55 },
    { 0, 0 },
    { 0xd243fcac, 78 },
    { 0xdf3fcad, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e23fcb2, 76 },
    { 0, 0 },
    { 0xa103fcb4, 90 },
    { 0xe273fcb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2723fcbc, 33 },
    { 0, 0 },
    { 0x7143fcbe, 33 },
    { 0, 0 },
    { 0x85c3fcc0, 75 },
    { 0xbe13fcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903fcc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c3fcc8, 95 },
    { 0xed13fcc9, 91 },
    { 0x653fcc9, 61 },
    { 0xa8a3fcca, 34 },
    { 0xd273fcc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1573fcd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473fcd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad73fcdb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3fce0, 90 },
    { 0xbe93fce1, 78 },
    { 0, 0 },
    { 0xee3fce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf13fceb, 66 },
    { 0x1a53fcec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93fcf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b3fcf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323fcff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3fd03, 61 },
    { 0xc2e3fd04, 33 },
    { 0x4ba3fd04, 33 },
    { 0x1293fd06, 55 },
    { 0x7103fd07, 76 },
    { 0xbd3fd07, 23 },
    { 0, 0 },
    { 0xdba3fd0a, 98 },
    { 0xd3a3fd0b, 34 },
    { 0, 0 },
    { 0xd053fd0d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703fd10, 98 },
    { 0x5fa3fd11, 43 },
    { 0, 0 },
    { 0x79a3fd13, 95 },
    { 0xffe3fd14, 55 },
    { 0xfeb3fd14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7513fd19, 33 },
    { 0xda43fd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a3fd1d, 66 },
    { 0x30f3fd1d, 90 },
    { 0, 0 },
    { 0x5c43fd20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233fd23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd73fd31, 34 },
    { 0, 0 },
    { 0x42f3fd33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d3fd3b, 98 },
    { 0x2353fd3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3fd43, 61 },
    { 0x5523fd43, 34 },
    { 0x5d53fd45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1773fd4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f3fd4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf3fd63, 85 },
    { 0x4ba3fd64, 68 },
    { 0x6263fd65, 33 },
    { 0xfcf3fd66, 68 },
    { 0xa033fd66, 23 },
    { 0x5393fd63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a03fd6b, 66 },
    { 0xea73fd6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a3fd6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a93fd72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf913fd75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd173fd78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d3fd7f, 23 },
    { 0x84f3fd80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x503fd83, 90 },
    { 0x223fd83, 95 },
    { 0x2803fd85, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7403fd8a, 76 },
    { 0, 0 },
    { 0x2513fd8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad3fd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e23fd94, 46 },
    { 0x2b33fd94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43fd9d, 61 },
    { 0xd053fd9d, 55 },
    { 0, 0 },
    { 0xd043fda0, 68 },
    { 0, 0 },
    { 0xedd3fda2, 55 },
    { 0, 0 },
    { 0x963fda4, 66 },
    { 0, 0 },
    { 0x3293fda6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa903fdac, 37 },
    { 0x5773fdad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac3fdb1, 33 },
    { 0, 0 },
    { 0xa533fdb3, 99 },
    { 0xbef3fdb4, 34 },
    { 0xf013fdb5, 100 },
    { 0xb373fdb5, 95 },
    { 0x2c03fdb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb73fdba, 75 },
    { 0x793fdba, 61 },
    { 0, 0 },
    { 0xeee3fdbd, 66 },
    { 0xa493fdbe, 33 },
    { 0, 0 },
    { 0x5273fdc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9973fdc6, 23 },
    { 0, 0 },
    { 0xb853fdc8, 95 },
    { 0xad83fdc9, 90 },
    { 0x3f53fdc9, 95 },
    { 0x1b33fdcb, 95 },
    { 0xff53fdcc, 55 },
    { 0x9fc3fdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab23fdd2, 33 },
    { 0x61b3fdd3, 34 },
    { 0xfac3fdd4, 99 },
    { 0, 0 },
    { 0x5b83fdd6, 61 },
    { 0xd523fdd7, 66 },
    { 0xba13fdd8, 98 },
    { 0x55b3fdd9, 61 },
    { 0, 0 },
    { 0xaaa3fddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3fde0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd553fde4, 23 },
    { 0x61c3fde5, 33 },
    { 0x63c3fde6, 75 },
    { 0x6e63fde6, 34 },
    { 0xd333fde5, 90 },
    { 0, 0 },
    { 0x2503fdea, 33 },
    { 0x9183fdea, 33 },
    { 0x7573fdeb, 17 },
    { 0x8c43fded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23fdf0, 33 },
    { 0, 0 },
    { 0xec03fdf2, 61 },
    { 0x75e3fdf3, 87 },
    { 0xadd3fdf4, 61 },
    { 0, 0 },
    { 0xccc3fdf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db3fdf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe223fdfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc3fe03, 60 },
    { 0x8833fe03, 98 },
    { 0, 0 },
    { 0x81e3fe06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5043fe13, 61 },
    { 0, 0 },
    { 0xd453fe15, 91 },
    { 0, 0 },
    { 0x2af3fe17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d23fe1b, 52 },
    { 0x1d43fe1c, 98 },
    { 0x3a03fe1d, 95 },
    { 0x8e33fe1e, 55 },
    { 0xe0e3fe1c, 98 },
    { 0xbd23fe1f, 33 },
    { 0x9e33fe1e, 81 },
    { 0x7bb3fe1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d93fe25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d3fe2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc043fe2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3fe35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f3fe39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2203fe3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc53fe42, 55 },
    { 0x2923fe43, 56 },
    { 0x1b3fe44, 91 },
    { 0xf63fe43, 90 },
    { 0, 0 },
    { 0x85e3fe47, 35 },
    { 0x9003fe48, 43 },
    { 0xa3c3fe48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b53fe50, 46 },
    { 0xfb83fe51, 90 },
    { 0x2ea3fe50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b63fe58, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c3fe5c, 17 },
    { 0x5273fe5d, 95 },
    { 0x80e3fe5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf63fe62, 61 },
    { 0xfd3fe63, 91 },
    { 0xd633fe63, 33 },
    { 0x98d3fe65, 95 },
    { 0x29b3fe66, 66 },
    { 0x3fb3fe66, 33 },
    { 0x1e53fe68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c3fe74, 61 },
    { 0, 0 },
    { 0xa3c3fe76, 61 },
    { 0xf473fe77, 33 },
    { 0xfc73fe78, 23 },
    { 0x7713fe77, 33 },
    { 0xff13fe76, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b3fe7f, 61 },
    { 0x2643fe80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf903fe84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf903fe87, 91 },
    { 0, 0 },
    { 0xc873fe89, 33 },
    { 0xa73fe8a, 98 },
    { 0x5383fe8b, 90 },
    { 0x2e43fe89, 77 },
    { 0xc0b3fe8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd3fe93, 95 },
    { 0, 0 },
    { 0xacc3fe95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a3fe9f, 33 },
    { 0, 0 },
    { 0x89a3fea1, 61 },
    { 0x6733fea2, 34 },
    { 0x4bc3fea3, 33 },
    { 0, 0 },
    { 0x23a3fea5, 68 },
    { 0xb183fea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3feaa, 33 },
    { 0xf163feaa, 90 },
    { 0x1573feac, 66 },
    { 0x1f63fead, 43 },
    { 0, 0 },
    { 0x7b3feaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b43feb5, 85 },
    { 0xe883feb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7023febc, 98 },
    { 0xb1d3febc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3fec1, 61 },
    { 0xec33fec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb933fecb, 46 },
    { 0, 0 },
    { 0xcfb3fecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb3fed0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93fed5, 91 },
    { 0xf5f3fed5, 46 },
    { 0x7fb3fed7, 69 },
    { 0x1323fed6, 98 },
    { 0, 0 },
    { 0xd2f3feda, 81 },
    { 0x6413feda, 55 },
    { 0x18b3fedb, 98 },
    { 0xcc93fedd, 61 },
    { 0xd3b3fedc, 85 },
    { 0x5063fede, 61 },
    { 0x2983fee0, 18 },
    { 0x5843fedd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb3fee6, 23 },
    { 0, 0 },
    { 0x8833fee8, 34 },
    { 0xdb63fee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3feed, 100 },
    { 0xba93feee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3fef9, 68 },
    { 0x92b3fef9, 98 },
    { 0x7ce3fefb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f3ff00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9063ff10, 61 },
    { 0, 0 },
    { 0x6a83ff12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3ff17, 76 },
    { 0xec43ff18, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503ff1b, 90 },
    { 0x63e3ff1c, 90 },
    { 0x3b03ff1b, 93 },
    { 0x8443ff1e, 38 },
    { 0xf833ff1b, 90 },
    { 0xdb53ff20, 61 },
    { 0x34a3ff20, 33 },
    { 0x78f3ff22, 33 },
    { 0x4d73ff23, 23 },
    { 0xe403ff20, 95 },
    { 0xe333ff24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13ff28, 93 },
    { 0, 0 },
    { 0xf23ff2a, 33 },
    { 0x6773ff2b, 66 },
    { 0x5633ff2a, 47 },
    { 0x6f03ff2c, 33 },
    { 0xeaf3ff2e, 90 },
    { 0x4d23ff2d, 55 },
    { 0, 0 },
    { 0x15e3ff31, 76 },
    { 0xb383ff32, 33 },
    { 0xf123ff33, 23 },
    { 0xb923ff34, 67 },
    { 0xf493ff34, 90 },
    { 0, 0 },
    { 0x93d3ff37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3ff3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073ff42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab63ff45, 90 },
    { 0x9d83ff45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe43ff4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b3ff58, 23 },
    { 0, 0 },
    { 0x5963ff5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8163ff5d, 33 },
    { 0, 0 },
    { 0xbc03ff5f, 43 },
    { 0xf2f3ff60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a63ff64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63ff6d, 90 },
    { 0, 0 },
    { 0x4d63ff6f, 34 },
    { 0xa893ff6f, 90 },
    { 0, 0 },
    { 0xa6d3ff72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813ff77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f93ff7b, 91 },
    { 0xeaf3ff7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133ff87, 98 },
    { 0x67a3ff88, 95 },
    { 0x3b03ff88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b3ff8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823ff95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593ff9a, 55 },
    { 0, 0 },
    { 0x5ec3ff9c, 60 },
    { 0xbc23ff9d, 40 },
    { 0x55f3ff9c, 23 },
    { 0x84f3ff9d, 90 },
    { 0x5c73ffa0, 61 },
    { 0, 0 },
    { 0x16e3ffa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e43ffae, 61 },
    { 0x6193ffaf, 23 },
    { 0, 0 },
    { 0xfab3ffb1, 56 },
    { 0x71f3ffb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1653ffb7, 23 },
    { 0xe2a3ffb8, 85 },
    { 0x2843ffb7, 34 },
    { 0x4943ffb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b3ffc4, 23 },
    { 0xfb83ffc5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc463ffca, 61 },
    { 0xc713ffcb, 95 },
    { 0xd053ffcc, 100 },
    { 0, 0 },
    { 0xa353ffce, 90 },
    { 0, 0 },
    { 0xb3ffd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c3ffdc, 95 },
    { 0, 0 },
    { 0x6f83ffde, 33 },
    { 0x9cb3ffdf, 98 },
    { 0x5f43ffe0, 90 },
    { 0x4513ffe1, 78 },
    { 0xe323ffde, 76 },
    { 0x893ffe0, 95 },
    { 0x9e83ffe4, 95 },
    { 0xa1b3ffde, 93 },
    { 0xe6b3ffe6, 33 },
    { 0xb533ffe7, 68 },
    { 0xdc3ffe7, 90 },
    { 0, 0 },
    { 0xe223ffea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f63ffed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e63fff1, 90 },
    { 0xa8a3fff1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf573fffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a43fffd, 67 },
    { 0x3403fffe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a40001, 33 },
    { 0xfc40002, 95 },
    { 0x3a40003, 99 },
    { 0, 0 },
    { 0x9da40005, 87 },
    { 0x2cf40006, 33 },
    { 0x17e40007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a740013, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a40018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634001b, 46 },
    { 0x4864001c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d40021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c40026, 95 },
    { 0xc5240027, 98 },
    { 0x8a640026, 76 },
    { 0xf8440028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede40030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b40033, 98 },
    { 0x81240033, 98 },
    { 0x3de40035, 85 },
    { 0x92040036, 33 },
    { 0xb4740036, 95 },
    { 0xdc140038, 93 },
    { 0xd6240035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b4003e, 61 },
    { 0, 0 },
    { 0xad240040, 88 },
    { 0x9ff40041, 46 },
    { 0xe3040041, 52 },
    { 0, 0 },
    { 0xad140044, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe40047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5514004b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74004f, 66 },
    { 0, 0 },
    { 0x7c340051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764005b, 61 },
    { 0x7d74005c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad40062, 61 },
    { 0x96140063, 93 },
    { 0x6f740063, 23 },
    { 0x3da40065, 90 },
    { 0x2be40062, 35 },
    { 0x61140067, 61 },
    { 0x3db40067, 34 },
    { 0x99640069, 85 },
    { 0xf6340067, 67 },
    { 0x6604006b, 76 },
    { 0xada4006b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf40073, 33 },
    { 0, 0 },
    { 0x1640075, 77 },
    { 0x6040075, 55 },
    { 0xabc40075, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d4007b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d240082, 99 },
    { 0x7d340083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5740086, 61 },
    { 0, 0 },
    { 0x76d40088, 81 },
    { 0x14f40088, 66 },
    { 0, 0 },
    { 0xf064008b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d40092, 90 },
    { 0x96e40092, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2740096, 34 },
    { 0xf0e40096, 51 },
    { 0xea540098, 23 },
    { 0, 0 },
    { 0x1b94009a, 95 },
    { 0, 0 },
    { 0x9964009c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa4009f, 90 },
    { 0x734400a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f400aa, 98 },
    { 0xa9e400ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e400b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15400ba, 93 },
    { 0, 0 },
    { 0xb59400bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f400c5, 95 },
    { 0, 0 },
    { 0xf20400c7, 23 },
    { 0x224400c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2400cf, 84 },
    { 0x46400d0, 95 },
    { 0x550400d1, 90 },
    { 0, 0 },
    { 0xdcb400d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791400d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8400e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a400ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d400f0, 84 },
    { 0x648400f0, 23 },
    { 0x2e1400f2, 23 },
    { 0x668400f0, 33 },
    { 0xdfe400f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87400f8, 61 },
    { 0, 0 },
    { 0x58a400fa, 40 },
    { 0xc25400fb, 85 },
    { 0x890400fc, 93 },
    { 0x813400fd, 23 },
    { 0x1fd400fe, 61 },
    { 0, 0 },
    { 0xe1140100, 55 },
    { 0xff140100, 34 },
    { 0x3d440102, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20040109, 98 },
    { 0, 0 },
    { 0x4124010b, 75 },
    { 0x2e24010b, 98 },
    { 0x8ca4010d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9640110, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30840114, 90 },
    { 0xd3b40114, 33 },
    { 0xfd940115, 95 },
    { 0x1b440116, 34 },
    { 0, 0 },
    { 0x44140119, 46 },
    { 0xa9e4011a, 33 },
    { 0x79f4011b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e40120, 23 },
    { 0x82e40121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4840124, 33 },
    { 0, 0 },
    { 0xa2b40126, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d40130, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d40134, 95 },
    { 0, 0 },
    { 0x41240136, 61 },
    { 0, 0 },
    { 0x14840138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44013f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0440142, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46540149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa014014d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8940154, 33 },
    { 0xc0140154, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab40159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea40161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e740168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d40171, 46 },
    { 0x70640172, 96 },
    { 0x3840173, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2840179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1244017c, 34 },
    { 0x384017c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf840185, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e4018b, 61 },
    { 0xd824018c, 91 },
    { 0x3114018d, 61 },
    { 0x1db4018e, 90 },
    { 0x8cf4018f, 90 },
    { 0x4934018f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f140193, 43 },
    { 0xbd640194, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe614019b, 23 },
    { 0, 0 },
    { 0x99d4019d, 66 },
    { 0x6e04019d, 23 },
    { 0x2304019d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5401ac, 95 },
    { 0xf8e401ad, 33 },
    { 0xd5a401ae, 43 },
    { 0x53e401ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6401b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac401b6, 33 },
    { 0x1b9401b7, 87 },
    { 0x6e4401b7, 100 },
    { 0xb77401b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be401c1, 93 },
    { 0, 0 },
    { 0xc13401c3, 76 },
    { 0xb96401c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb401c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x287401cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f401d1, 76 },
    { 0, 0 },
    { 0xb58401d3, 33 },
    { 0x14b401d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937401dc, 76 },
    { 0, 0 },
    { 0xf3e401de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f401e1, 33 },
    { 0x3b8401e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb401e8, 95 },
    { 0, 0 },
    { 0xf58401ea, 90 },
    { 0x193401eb, 99 },
    { 0x5ab401ec, 95 },
    { 0x370401ec, 90 },
    { 0x6f3401ee, 95 },
    { 0xf41401ef, 19 },
    { 0xeb6401ef, 33 },
    { 0x322401f1, 90 },
    { 0x722401ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4401f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4401fa, 23 },
    { 0, 0 },
    { 0xb66401fc, 12 },
    { 0xe5d401fd, 93 },
    { 0xabe401fd, 23 },
    { 0xfd0401ff, 100 },
    { 0, 0 },
    { 0x1d440201, 76 },
    { 0x29840202, 23 },
    { 0x11440203, 90 },
    { 0x71140204, 100 },
    { 0xe0a40204, 55 },
    { 0x43c40203, 76 },
    { 0x8d340205, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4020d, 46 },
    { 0x6eb4020e, 99 },
    { 0, 0 },
    { 0x7d040210, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c40214, 84 },
    { 0, 0 },
    { 0x24a40216, 98 },
    { 0xf3e40217, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed4021c, 43 },
    { 0x11d4021d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b40220, 61 },
    { 0xb8c40221, 67 },
    { 0x8fa40221, 33 },
    { 0x4d240222, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d40226, 98 },
    { 0, 0 },
    { 0x1e340228, 90 },
    { 0x58f40229, 90 },
    { 0x78040228, 78 },
    { 0xe8d4022a, 78 },
    { 0, 0 },
    { 0xd294022d, 95 },
    { 0x7b4022e, 95 },
    { 0, 0 },
    { 0xde840230, 98 },
    { 0x68a40231, 66 },
    { 0xa3f40231, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b40236, 93 },
    { 0, 0 },
    { 0xfa840238, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e84023e, 66 },
    { 0, 0 },
    { 0xca640240, 95 },
    { 0x3b640240, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6540246, 77 },
    { 0x93340246, 55 },
    { 0xee840247, 95 },
    { 0xaff40249, 90 },
    { 0x2db4024a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34024e, 90 },
    { 0x5ab4024f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7740252, 90 },
    { 0x5b140252, 33 },
    { 0x90140254, 84 },
    { 0xa240253, 95 },
    { 0, 0 },
    { 0x2d440257, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa944025a, 90 },
    { 0, 0 },
    { 0x50c4025c, 87 },
    { 0, 0 },
    { 0x3a14025e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48640267, 88 },
    { 0x48c40267, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d4026b, 87 },
    { 0, 0 },
    { 0x97c4026d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d40270, 90 },
    { 0x52040271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df40275, 98 },
    { 0xd4540276, 85 },
    { 0, 0 },
    { 0x1a140278, 55 },
    { 0, 0 },
    { 0xbd64027a, 61 },
    { 0x6204027b, 95 },
    { 0x6484027a, 75 },
    { 0x7794027d, 46 },
    { 0x3c84027a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca40282, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c40285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb4028a, 84 },
    { 0x17e4028b, 90 },
    { 0x66f4028b, 95 },
    { 0x41f4028d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f40290, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e40294, 91 },
    { 0x9cf40295, 90 },
    { 0xc8340295, 47 },
    { 0x2e640297, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de4029b, 90 },
    { 0x97b4029b, 43 },
    { 0x5f14029c, 90 },
    { 0, 0 },
    { 0x214029f, 43 },
    { 0x516402a0, 90 },
    { 0x2d3402a1, 90 },
    { 0x4d44029f, 76 },
    { 0x1c4402a3, 90 },
    { 0xdfe402a4, 76 },
    { 0x30b4029f, 33 },
    { 0, 0 },
    { 0x5c2402a7, 95 },
    { 0xa16402a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x334402ab, 98 },
    { 0, 0 },
    { 0x408402ad, 95 },
    { 0x66c402ae, 81 },
    { 0x263402ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a402b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433402b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e402b9, 93 },
    { 0x157402ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e402be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb402c3, 100 },
    { 0xe0402c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca402c7, 33 },
    { 0x403402c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a402cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90402d1, 81 },
    { 0xbc6402d2, 76 },
    { 0x9e2402d2, 76 },
    { 0x67e402d1, 90 },
    { 0x17402d5, 66 },
    { 0xcae402d5, 46 },
    { 0xd20402d6, 100 },
    { 0, 0 },
    { 0xa58402d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc402dc, 23 },
    { 0xcb2402dd, 65 },
    { 0, 0 },
    { 0x697402df, 88 },
    { 0x524402df, 78 },
    { 0xa6b402e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f402e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff402e8, 78 },
    { 0x99a402e9, 23 },
    { 0xe0e402ea, 91 },
    { 0x182402eb, 78 },
    { 0x354402ea, 43 },
    { 0xf7a402ea, 36 },
    { 0xb21402ee, 78 },
    { 0x300402ef, 33 },
    { 0xf9a402eb, 61 },
    { 0x459402ef, 90 },
    { 0x1f2402f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f402f5, 98 },
    { 0, 0 },
    { 0x112402f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba402fc, 31 },
    { 0x5f5402fc, 91 },
    { 0x634402fe, 23 },
    { 0x9ff402fe, 95 },
    { 0x1fc40300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x65140303, 61 },
    { 0x97940304, 33 },
    { 0x55740304, 93 },
    { 0, 0 },
    { 0xdf440307, 23 },
    { 0xbb140307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb794030b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39140310, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9740313, 76 },
    { 0x32b40314, 95 },
    { 0, 0 },
    { 0x1ea40316, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7840319, 67 },
    { 0x3b84031a, 78 },
    { 0x2924031b, 90 },
    { 0x5474031c, 93 },
    { 0xbf64031c, 76 },
    { 0xa264031c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6340323, 76 },
    { 0xa8540324, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed40327, 23 },
    { 0x1740328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14032c, 33 },
    { 0x7b4032c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc40330, 90 },
    { 0x87940330, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e40336, 76 },
    { 0, 0 },
    { 0xb1a40338, 84 },
    { 0x68040339, 17 },
    { 0xf5e40338, 55 },
    { 0xb2b4033b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69140340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3440344, 77 },
    { 0x7f640345, 78 },
    { 0, 0 },
    { 0x21f40347, 23 },
    { 0, 0 },
    { 0xe6940349, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014034e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59140355, 90 },
    { 0, 0 },
    { 0xc9840357, 33 },
    { 0xe5f40358, 33 },
    { 0x2e840359, 17 },
    { 0x7514035a, 90 },
    { 0x35a40358, 98 },
    { 0, 0 },
    { 0xbbf4035d, 76 },
    { 0xa704035e, 33 },
    { 0xc6d4035f, 95 },
    { 0, 0 },
    { 0xf5240361, 95 },
    { 0xb240361, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1584036f, 23 },
    { 0xf1340370, 90 },
    { 0x71e40371, 23 },
    { 0xab540372, 90 },
    { 0x1f640373, 74 },
    { 0xb5340373, 90 },
    { 0x9f140374, 68 },
    { 0x96640376, 100 },
    { 0x94d40377, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5174037a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04037e, 98 },
    { 0, 0 },
    { 0x10240380, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d740384, 90 },
    { 0xf3640385, 34 },
    { 0x25f40386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18840389, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af40396, 95 },
    { 0xcb40396, 87 },
    { 0xacb40398, 33 },
    { 0, 0 },
    { 0xeb64039a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c34039d, 23 },
    { 0x1e84039e, 8 },
    { 0x8344039e, 99 },
    { 0, 0 },
    { 0x48e403a1, 44 },
    { 0, 0 },
    { 0x86a403a3, 33 },
    { 0, 0 },
    { 0x101403a5, 90 },
    { 0x9c8403a6, 34 },
    { 0x5d5403a7, 95 },
    { 0xffa403a8, 33 },
    { 0xed1403a8, 84 },
    { 0x35c403a6, 46 },
    { 0, 0 },
    { 0x75e403ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87403b0, 33 },
    { 0xf57403b0, 76 },
    { 0x942403b0, 24 },
    { 0x4a7403b3, 76 },
    { 0, 0 },
    { 0xb70403b5, 37 },
    { 0x3e3403b6, 88 },
    { 0x5c5403b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8403bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637403c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538403c4, 55 },
    { 0x4db403c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6403c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd403cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8403cf, 46 },
    { 0, 0 },
    { 0xdfd403d1, 61 },
    { 0x935403d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0403d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77403de, 91 },
    { 0x2c7403de, 33 },
    { 0xcc8403df, 93 },
    { 0xd70403df, 43 },
    { 0xa7403df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c403e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67403f0, 23 },
    { 0xacb403f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab403f4, 64 },
    { 0xcdc403f5, 61 },
    { 0x4ba403f4, 93 },
    { 0xc7b403f4, 33 },
    { 0, 0 },
    { 0x360403f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1403fd, 91 },
    { 0x31d403fe, 61 },
    { 0, 0 },
    { 0xd9e40400, 88 },
    { 0x1940401, 76 },
    { 0x47d40402, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e40405, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7004040b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1754040e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83140416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc4041c, 90 },
    { 0, 0 },
    { 0xddb4041e, 23 },
    { 0x8314041f, 98 },
    { 0xf6f4041f, 67 },
    { 0x3b740420, 46 },
    { 0xdd84041e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d940429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b04042e, 43 },
    { 0x3514042f, 52 },
    { 0x62240430, 33 },
    { 0x4eb40430, 95 },
    { 0x6fc40432, 55 },
    { 0x8a140432, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa724043a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d40440, 95 },
    { 0xe6b40441, 58 },
    { 0x35140441, 23 },
    { 0, 0 },
    { 0x89240444, 36 },
    { 0xebe40445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63640452, 33 },
    { 0x4f640453, 90 },
    { 0x8d140454, 98 },
    { 0x1da40455, 94 },
    { 0x80b40456, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4045f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e40463, 90 },
    { 0x59840463, 90 },
    { 0, 0 },
    { 0x91240466, 95 },
    { 0x8e240466, 43 },
    { 0xabf40468, 66 },
    { 0, 0 },
    { 0xe0b4046a, 61 },
    { 0, 0 },
    { 0x9e84046c, 55 },
    { 0xec94046c, 98 },
    { 0xaa74046e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d40473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc24047d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f40480, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7540488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc174048c, 33 },
    { 0xe374048d, 52 },
    { 0x3784048d, 95 },
    { 0xe3c4048f, 100 },
    { 0xd134048e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a40497, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e4049c, 81 },
    { 0xab04049d, 61 },
    { 0x2b44049c, 33 },
    { 0x8d4049c, 76 },
    { 0xff4404a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0404a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7404a8, 77 },
    { 0, 0 },
    { 0xa46404aa, 61 },
    { 0x89e404ab, 40 },
    { 0x93f404ac, 95 },
    { 0, 0 },
    { 0x4cb404ae, 34 },
    { 0, 0 },
    { 0x84d404b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0404b5, 23 },
    { 0x108404b5, 55 },
    { 0x17a404b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e404bc, 90 },
    { 0, 0 },
    { 0xd04404be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x519404c1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54404c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff404ca, 66 },
    { 0, 0 },
    { 0x9d4404cc, 95 },
    { 0xd1f404cd, 33 },
    { 0x93f404ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5404df, 34 },
    { 0, 0 },
    { 0xfe1404e1, 98 },
    { 0, 0 },
    { 0x423404e3, 90 },
    { 0x9dd404e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x651404e7, 90 },
    { 0, 0 },
    { 0x201404e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727404f5, 93 },
    { 0xa4f404f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26404fa, 33 },
    { 0x6d2404fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x227404fe, 90 },
    { 0x2cf404fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e40502, 33 },
    { 0x5d840503, 81 },
    { 0xfaf40503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d440507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a4050c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214050f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9140512, 24 },
    { 0x1b740513, 90 },
    { 0x34c40513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45140517, 76 },
    { 0xc5040518, 55 },
    { 0x1dd40518, 99 },
    { 0x79d4051a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3140520, 55 },
    { 0x2f40520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70040525, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5254052c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e40531, 43 },
    { 0, 0 },
    { 0xc3d40533, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474053a, 85 },
    { 0xfc14053b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e4053e, 61 },
    { 0x9084053e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b40543, 98 },
    { 0x85840544, 46 },
    { 0xc540545, 57 },
    { 0x31740546, 34 },
    { 0, 0 },
    { 0xc0a40548, 86 },
    { 0xe3940549, 100 },
    { 0, 0 },
    { 0x3284054b, 93 },
    { 0x2f74054c, 98 },
    { 0xdda4054d, 64 },
    { 0x14e4054d, 17 },
    { 0, 0 },
    { 0xda340550, 76 },
    { 0, 0 },
    { 0x2e40552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5540557, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f4055b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d4055e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4240565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae4056a, 90 },
    { 0x1134056b, 23 },
    { 0xd4d4056c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb124056f, 76 },
    { 0x41240570, 33 },
    { 0xc8a40571, 61 },
    { 0xc7940571, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d40575, 33 },
    { 0x33240576, 91 },
    { 0, 0 },
    { 0x9b040578, 33 },
    { 0, 0 },
    { 0xd174057a, 65 },
    { 0x27f4057b, 61 },
    { 0, 0 },
    { 0xacf4057d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe140583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4240586, 23 },
    { 0, 0 },
    { 0xd9940588, 99 },
    { 0x6ec40589, 33 },
    { 0, 0 },
    { 0xca4058b, 95 },
    { 0xf6f4058c, 23 },
    { 0xd6b4058b, 46 },
    { 0x6034058e, 90 },
    { 0x3de4058e, 76 },
    { 0x88c40590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22040594, 43 },
    { 0x9c540595, 66 },
    { 0, 0 },
    { 0x7b340597, 90 },
    { 0x7bd40598, 8 },
    { 0x60040599, 98 },
    { 0xad740598, 84 },
    { 0xdba4059b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8405a0, 33 },
    { 0, 0 },
    { 0x503405a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3405a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608405ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375405b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b405b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1405bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b405c0, 95 },
    { 0x76c405c1, 61 },
    { 0, 0 },
    { 0x408405c3, 93 },
    { 0x3a8405c3, 65 },
    { 0xe61405c5, 61 },
    { 0x266405c4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07405cb, 87 },
    { 0x173405cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb405d4, 90 },
    { 0xd89405d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a405e3, 100 },
    { 0xa38405e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7405e8, 90 },
    { 0xf1b405e9, 88 },
    { 0xeb6405ea, 66 },
    { 0xea9405eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e405ee, 90 },
    { 0xe28405ee, 33 },
    { 0, 0 },
    { 0x1c6405f1, 33 },
    { 0xd79405f2, 67 },
    { 0, 0 },
    { 0x380405f4, 95 },
    { 0, 0 },
    { 0x58d405f6, 93 },
    { 0, 0 },
    { 0xb30405f8, 23 },
    { 0x595405f9, 34 },
    { 0x456405f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8405ff, 33 },
    { 0x70c40600, 33 },
    { 0x81540601, 61 },
    { 0x2b040601, 91 },
    { 0xdd540603, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d40609, 95 },
    { 0, 0 },
    { 0xf844060b, 33 },
    { 0xd24060c, 69 },
    { 0x6ab4060d, 84 },
    { 0x2034060e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f140614, 90 },
    { 0x97f40615, 98 },
    { 0x8c940616, 33 },
    { 0, 0 },
    { 0xcbc40618, 61 },
    { 0x10640619, 34 },
    { 0, 0 },
    { 0x324061b, 76 },
    { 0xeec4061c, 33 },
    { 0xcf44061c, 85 },
    { 0, 0 },
    { 0x3f34061f, 88 },
    { 0, 0 },
    { 0x30840621, 34 },
    { 0x26b40622, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d40628, 33 },
    { 0xd340629, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4062d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40631, 90 },
    { 0xffd40632, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1140639, 23 },
    { 0xc7c40639, 46 },
    { 0x3dc4063a, 43 },
    { 0xc4040639, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b40641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e94064a, 55 },
    { 0, 0 },
    { 0xdad4064c, 98 },
    { 0x9294064c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce4065a, 84 },
    { 0xb14065b, 90 },
    { 0, 0 },
    { 0x2d64065d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1940662, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd40667, 90 },
    { 0, 0 },
    { 0x7f540669, 55 },
    { 0, 0 },
    { 0x5a14066b, 88 },
    { 0xa044066b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf684066f, 85 },
    { 0, 0 },
    { 0xd8640671, 98 },
    { 0x52640671, 46 },
    { 0x4d240671, 93 },
    { 0x42540674, 61 },
    { 0x1c740675, 100 },
    { 0x93e40676, 98 },
    { 0xe240671, 42 },
    { 0x2df40678, 51 },
    { 0xa7740679, 61 },
    { 0x12540677, 90 },
    { 0x9ee4067b, 98 },
    { 0x7f74067b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8364067f, 66 },
    { 0, 0 },
    { 0x45040681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41140685, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2340688, 99 },
    { 0, 0 },
    { 0xa424068a, 87 },
    { 0xc454068a, 99 },
    { 0x1304068a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a40695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd94069a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90406a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26406a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb406a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6406aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb406af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0406b4, 90 },
    { 0, 0 },
    { 0x6c8406b6, 68 },
    { 0, 0 },
    { 0xb0a406b8, 69 },
    { 0x1b0406b9, 85 },
    { 0xf3d406ba, 55 },
    { 0x5e5406bb, 23 },
    { 0, 0 },
    { 0x8ea406bd, 34 },
    { 0x61406be, 33 },
    { 0x8c9406be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6406c2, 23 },
    { 0x924406c3, 61 },
    { 0x45b406c2, 100 },
    { 0x811406c5, 90 },
    { 0xaa8406c6, 83 },
    { 0x768406c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x506406ca, 46 },
    { 0, 0 },
    { 0x8dc406cc, 90 },
    { 0, 0 },
    { 0x620406ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123406d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7406d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0406dd, 55 },
    { 0x801406de, 61 },
    { 0xd63406df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca406e7, 33 },
    { 0x731406e7, 95 },
    { 0xb19406e8, 95 },
    { 0xec2406ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b406ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914406fb, 90 },
    { 0x4d8406fb, 98 },
    { 0x96406fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf040700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b4070a, 77 },
    { 0xd104070b, 93 },
    { 0xf054070c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540712, 43 },
    { 0xc2640713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4071a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f240721, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91540726, 61 },
    { 0x40740727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3704072b, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90340730, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32040738, 84 },
    { 0, 0 },
    { 0x2fd4073a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2194073d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40940743, 52 },
    { 0xd1f40744, 77 },
    { 0x5c940744, 85 },
    { 0xd0640746, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4074c, 93 },
    { 0x6e24074c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a940752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a140757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4075c, 65 },
    { 0x9f74075d, 81 },
    { 0, 0 },
    { 0x434075f, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c40763, 76 },
    { 0x42c40764, 23 },
    { 0, 0 },
    { 0xf8940766, 23 },
    { 0x77640767, 68 },
    { 0xadb40768, 64 },
    { 0xb1c40769, 76 },
    { 0x9b84076a, 90 },
    { 0xb1e4076a, 55 },
    { 0xa4d4076b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f34076f, 98 },
    { 0x22540770, 80 },
    { 0x4884076f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4240774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec40777, 77 },
    { 0xc0f40777, 61 },
    { 0, 0 },
    { 0xa0b4077a, 90 },
    { 0x9024077b, 80 },
    { 0x7c04077c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42440784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc424078b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x974078e, 90 },
    { 0xac74078f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e40792, 33 },
    { 0, 0 },
    { 0xfc340794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a54079a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f4079d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc407a1, 68 },
    { 0, 0 },
    { 0x9a2407a3, 67 },
    { 0, 0 },
    { 0xa9a407a5, 33 },
    { 0x211407a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804407aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b407ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31407b1, 19 },
    { 0xe32407b2, 98 },
    { 0xc62407b3, 93 },
    { 0, 0 },
    { 0x9a7407b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd407c0, 33 },
    { 0, 0 },
    { 0x6d8407c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c407c5, 33 },
    { 0x4a7407c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d407cb, 61 },
    { 0x24a407cc, 67 },
    { 0x9a0407cd, 64 },
    { 0x664407cc, 99 },
    { 0xa8e407cf, 66 },
    { 0, 0 },
    { 0xb75407d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c407d4, 33 },
    { 0xe3407d5, 33 },
    { 0x8bd407d4, 98 },
    { 0, 0 },
    { 0x1a9407d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60407de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x349407e1, 81 },
    { 0x69b407e1, 33 },
    { 0, 0 },
    { 0xc1407e4, 33 },
    { 0xebb407e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448407ea, 55 },
    { 0x631407eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d407ef, 90 },
    { 0xff1407ef, 33 },
    { 0x34407ef, 90 },
    { 0xc64407f2, 55 },
    { 0xec2407f2, 76 },
    { 0x3cb407f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de407f7, 34 },
    { 0x6e7407f7, 33 },
    { 0x97c407f9, 87 },
    { 0x613407f8, 87 },
    { 0x1b6407f7, 95 },
    { 0x26407f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91407ff, 95 },
    { 0xe5840800, 33 },
    { 0xa1040800, 33 },
    { 0, 0 },
    { 0x71040803, 33 },
    { 0xe4540804, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb14080a, 87 },
    { 0xa4f4080b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b4080f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2840812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30340815, 95 },
    { 0x34840816, 46 },
    { 0, 0 },
    { 0xd8540818, 61 },
    { 0, 0 },
    { 0x64f4081a, 61 },
    { 0xc244081b, 90 },
    { 0, 0 },
    { 0x5634081d, 76 },
    { 0, 0 },
    { 0x58f4081f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee740822, 95 },
    { 0, 0 },
    { 0x4a940824, 95 },
    { 0x5b140825, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4082b, 66 },
    { 0xc44082c, 94 },
    { 0x54f4082c, 61 },
    { 0, 0 },
    { 0x2b24082f, 87 },
    { 0x7774082f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e40837, 61 },
    { 0, 0 },
    { 0xc4c40839, 81 },
    { 0x96c40839, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4083d, 90 },
    { 0x8424083e, 33 },
    { 0x1834083f, 61 },
    { 0x59840840, 90 },
    { 0x5f940840, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1240844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d140847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7634084c, 78 },
    { 0xe874084d, 98 },
    { 0x1244084e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36740853, 33 },
    { 0x18e40854, 99 },
    { 0x73240853, 76 },
    { 0, 0 },
    { 0x47040857, 38 },
    { 0, 0 },
    { 0x27540859, 98 },
    { 0xc194085a, 33 },
    { 0xec34085b, 90 },
    { 0x63e4085b, 98 },
    { 0xc194085d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c40862, 33 },
    { 0, 0 },
    { 0xa1940864, 52 },
    { 0xe4540865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14086b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a540871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c40875, 61 },
    { 0x9c940875, 98 },
    { 0x10a40877, 66 },
    { 0xbd140877, 17 },
    { 0x2c840879, 95 },
    { 0x2c640877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204087d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e440881, 33 },
    { 0x2d40882, 33 },
    { 0x73e40881, 95 },
    { 0, 0 },
    { 0x9d40885, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d40889, 95 },
    { 0, 0 },
    { 0x22e4088b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2140891, 100 },
    { 0, 0 },
    { 0x52e40893, 43 },
    { 0x65640893, 34 },
    { 0x4f340894, 95 },
    { 0x70240896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff4089c, 43 },
    { 0xdc74089d, 61 },
    { 0xeb84089e, 34 },
    { 0x3d84089f, 40 },
    { 0x1c4408a0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b408a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8408a9, 33 },
    { 0, 0 },
    { 0xcce408ab, 90 },
    { 0, 0 },
    { 0x248408ad, 40 },
    { 0, 0 },
    { 0xbf1408af, 90 },
    { 0, 0 },
    { 0xb6d408b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5408b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f408bc, 99 },
    { 0xb22408bc, 33 },
    { 0xd11408bd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7408c6, 52 },
    { 0x119408c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1408ce, 96 },
    { 0x92f408ce, 98 },
    { 0x139408d0, 84 },
    { 0xf32408d0, 36 },
    { 0x2f9408d0, 33 },
    { 0xc37408d0, 23 },
    { 0x122408d0, 55 },
    { 0xc18408d1, 90 },
    { 0x8a3408d5, 95 },
    { 0xb63408d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x702408da, 91 },
    { 0x6c3408da, 100 },
    { 0x5e6408dc, 52 },
    { 0, 0 },
    { 0x316408de, 33 },
    { 0xf74408de, 67 },
    { 0, 0 },
    { 0x312408e1, 52 },
    { 0xc94408e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc408e5, 93 },
    { 0x822408e5, 90 },
    { 0, 0 },
    { 0x106408e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e408ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994408f4, 90 },
    { 0x4e2408f5, 34 },
    { 0x4d408f5, 23 },
    { 0xb37408f6, 34 },
    { 0x7e7408f8, 76 },
    { 0x579408f8, 33 },
    { 0x76b408f6, 90 },
    { 0x67b408fb, 91 },
    { 0x3b9408fc, 61 },
    { 0x4ec408f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e40900, 95 },
    { 0xc0440901, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1140906, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d40909, 95 },
    { 0xdc140909, 69 },
    { 0xafa4090b, 76 },
    { 0x4b64090c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x104090f, 33 },
    { 0x25440910, 87 },
    { 0xf6c40911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2140915, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4091f, 81 },
    { 0, 0 },
    { 0xc840921, 61 },
    { 0x38440922, 33 },
    { 0x31c40923, 80 },
    { 0x9d040924, 95 },
    { 0x36c40924, 90 },
    { 0x6b440926, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e4092b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264092e, 23 },
    { 0, 0 },
    { 0x33440930, 94 },
    { 0, 0 },
    { 0x26840932, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab340939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4093f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b940946, 34 },
    { 0x61340947, 76 },
    { 0, 0 },
    { 0x22340949, 46 },
    { 0, 0 },
    { 0xea4094b, 98 },
    { 0x1a04094b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a240950, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd40959, 93 },
    { 0x6824095a, 55 },
    { 0xf7640959, 34 },
    { 0xa704095b, 99 },
    { 0x9f44095a, 76 },
    { 0x8d64095e, 87 },
    { 0x2e64095b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90040966, 90 },
    { 0, 0 },
    { 0xeed40968, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda84096f, 33 },
    { 0xee140970, 80 },
    { 0x24d40970, 33 },
    { 0xb2a40972, 76 },
    { 0x4b740973, 52 },
    { 0x9c140972, 78 },
    { 0, 0 },
    { 0x1b640976, 61 },
    { 0, 0 },
    { 0xe5040978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf4097c, 95 },
    { 0x5b04097d, 98 },
    { 0, 0 },
    { 0xb854097f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36340983, 77 },
    { 0xf9340984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954098c, 95 },
    { 0x73d4098d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d40993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e640996, 76 },
    { 0, 0 },
    { 0x40940998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f54099c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x994099f, 33 },
    { 0, 0 },
    { 0xe2c409a1, 90 },
    { 0x82f409a1, 87 },
    { 0x610409a3, 85 },
    { 0xbb8409a4, 43 },
    { 0x1f9409a5, 76 },
    { 0, 0 },
    { 0x103409a7, 33 },
    { 0x678409a8, 90 },
    { 0, 0 },
    { 0x433409aa, 61 },
    { 0xc19409ab, 66 },
    { 0, 0 },
    { 0x4fa409ad, 95 },
    { 0x4e6409ae, 23 },
    { 0x26f409ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776409b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c409bd, 55 },
    { 0x4ea409bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365409c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30409c5, 61 },
    { 0xf27409c6, 84 },
    { 0xa46409c5, 97 },
    { 0xa0e409c7, 84 },
    { 0, 0 },
    { 0x681409ca, 94 },
    { 0, 0 },
    { 0x82409cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94409d1, 98 },
    { 0xd9e409d2, 90 },
    { 0xa05409d1, 68 },
    { 0, 0 },
    { 0x984409d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x945409d8, 34 },
    { 0, 0 },
    { 0x815409da, 53 },
    { 0xc5c409da, 85 },
    { 0x6c409dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49409df, 52 },
    { 0xeaf409df, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b409e8, 90 },
    { 0xd0d409e9, 33 },
    { 0x232409ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3409ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01409f5, 23 },
    { 0, 0 },
    { 0xecb409f7, 93 },
    { 0, 0 },
    { 0xe60409f9, 61 },
    { 0xb69409fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62409ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d40a02, 33 },
    { 0x3db40a02, 34 },
    { 0x10540a02, 61 },
    { 0xe7140a05, 90 },
    { 0, 0 },
    { 0x8d140a07, 100 },
    { 0, 0 },
    { 0x35040a09, 98 },
    { 0x58940a0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44540a0d, 95 },
    { 0x72640a0e, 76 },
    { 0xa7840a0f, 90 },
    { 0x84540a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36640a14, 67 },
    { 0xab140a14, 76 },
    { 0xf8c40a16, 96 },
    { 0x69540a16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e40a1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3940a21, 90 },
    { 0, 0 },
    { 0x61f40a23, 61 },
    { 0x2ff40a24, 93 },
    { 0, 0 },
    { 0xa6c40a26, 88 },
    { 0xa4b40a26, 76 },
    { 0, 0 },
    { 0xf0040a29, 43 },
    { 0x7040a2a, 93 },
    { 0xe3e40a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41240a38, 55 },
    { 0, 0 },
    { 0xc2840a3a, 99 },
    { 0x2c740a3b, 87 },
    { 0xf3340a3c, 64 },
    { 0x93b40a3d, 76 },
    { 0x78540a3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a040a42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc040a4d, 90 },
    { 0x70440a4e, 76 },
    { 0x13c40a4f, 76 },
    { 0x99340a4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca40a58, 90 },
    { 0xcab40a59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2840a5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64740a63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f040a67, 34 },
    { 0xcf740a68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c840a70, 90 },
    { 0, 0 },
    { 0x1c240a72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440a75, 33 },
    { 0, 0 },
    { 0x8a140a77, 43 },
    { 0, 0 },
    { 0xd3640a79, 43 },
    { 0x57f40a7a, 33 },
    { 0x8f640a79, 67 },
    { 0, 0 },
    { 0x18540a7d, 90 },
    { 0xb6940a7e, 98 },
    { 0x8c840a7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c40a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e640a88, 81 },
    { 0x48f40a89, 91 },
    { 0, 0 },
    { 0x92240a8b, 33 },
    { 0x1f740a8b, 76 },
    { 0xd040a8c, 90 },
    { 0x20b40a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74640a92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50240a96, 55 },
    { 0xb3f40a97, 53 },
    { 0, 0 },
    { 0xc6540a99, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a40a9e, 85 },
    { 0x42740a9f, 59 },
    { 0xece40aa0, 34 },
    { 0xf9940aa1, 90 },
    { 0x8d240aa2, 52 },
    { 0x39840aa3, 95 },
    { 0xae240aa4, 33 },
    { 0xae340aa4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94640aa8, 90 },
    { 0xe3f40aa8, 84 },
    { 0xd1b40aa9, 98 },
    { 0x59f40aa8, 61 },
    { 0, 0 },
    { 0x2a740aad, 45 },
    { 0, 0 },
    { 0x6a140aaf, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340ab2, 61 },
    { 0x81240ab3, 61 },
    { 0, 0 },
    { 0x13840ab5, 67 },
    { 0xf3740ab5, 23 },
    { 0, 0 },
    { 0x45a40ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70240abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3940abe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4040ac1, 55 },
    { 0xada40ac2, 75 },
    { 0, 0 },
    { 0x71040ac4, 23 },
    { 0xbef40ac5, 61 },
    { 0xa6540ac4, 78 },
    { 0xcd440ac6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c540aca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25540acf, 34 },
    { 0xe8640acf, 43 },
    { 0, 0 },
    { 0x25240ad2, 61 },
    { 0x13640ad3, 55 },
    { 0x61d40ad4, 53 },
    { 0xbdd40ad4, 90 },
    { 0x94b40ad6, 61 },
    { 0x97040ad7, 85 },
    { 0, 0 },
    { 0x7cb40ad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa740adc, 36 },
    { 0xe740add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a40ae9, 43 },
    { 0, 0 },
    { 0xfd040aeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c40aee, 98 },
    { 0, 0 },
    { 0x8b40af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b40af4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f40af9, 65 },
    { 0xe7440afa, 93 },
    { 0x2fc40af9, 55 },
    { 0, 0 },
    { 0x26b40afd, 100 },
    { 0x12c40afe, 33 },
    { 0xbb640aff, 61 },
    { 0, 0 },
    { 0x4b940b01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e940b06, 93 },
    { 0x1b40b07, 95 },
    { 0xb0a40b07, 46 },
    { 0, 0 },
    { 0xfc840b0a, 90 },
    { 0, 0 },
    { 0x3ff40b0c, 33 },
    { 0x37240b0c, 95 },
    { 0xdd40b0c, 55 },
    { 0xb0740b0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f40b14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea40b17, 61 },
    { 0xd2140b18, 67 },
    { 0x46540b19, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92940b20, 17 },
    { 0xee540b21, 66 },
    { 0, 0 },
    { 0xfb240b23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66140b27, 90 },
    { 0, 0 },
    { 0xc6440b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb40b2e, 98 },
    { 0x4a740b2e, 77 },
    { 0, 0 },
    { 0x5240b31, 76 },
    { 0x5de40b32, 98 },
    { 0xf8a40b32, 43 },
    { 0xb5a40b32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd140b37, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f40b42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67740b4d, 90 },
    { 0x98540b4e, 33 },
    { 0xb40b4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a640b55, 33 },
    { 0, 0 },
    { 0x1640b57, 34 },
    { 0x5e840b58, 61 },
    { 0, 0 },
    { 0x65b40b5a, 33 },
    { 0xfdb40b5a, 55 },
    { 0x8b340b5b, 98 },
    { 0x67d40b5d, 23 },
    { 0xb8440b5d, 55 },
    { 0x46140b5f, 23 },
    { 0xf1740b5d, 61 },
    { 0x1d240b61, 33 },
    { 0xf1440b5f, 90 },
    { 0xf2540b5e, 90 },
    { 0, 0 },
    { 0x12140b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e40b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd40b72, 33 },
    { 0, 0 },
    { 0xa0240b74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f40b79, 68 },
    { 0xf440b7a, 90 },
    { 0, 0 },
    { 0x4d440b7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e540b84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37240b92, 23 },
    { 0xad740b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e40b99, 76 },
    { 0x57940b99, 95 },
    { 0xea740b9b, 95 },
    { 0xbc240b9c, 95 },
    { 0xbe040b9d, 33 },
    { 0xebb40b9e, 34 },
    { 0x78640b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81240ba2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540baa, 61 },
    { 0x67940bab, 88 },
    { 0xa9940bac, 61 },
    { 0x21e40bad, 61 },
    { 0x1fc40bae, 98 },
    { 0x17340bae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d40bb4, 61 },
    { 0x75b40bb4, 23 },
    { 0xe2240bb6, 61 },
    { 0x35a40bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73440bba, 61 },
    { 0x9940bbb, 95 },
    { 0xd6940bbc, 33 },
    { 0xf2540bbd, 33 },
    { 0x5a840bbb, 23 },
    { 0xbef40bbf, 33 },
    { 0x8d540bbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb40bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf840bc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb40bd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c640bd9, 66 },
    { 0x79a40bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a740bdd, 66 },
    { 0, 0 },
    { 0x2c640bdf, 95 },
    { 0xa0540bdf, 90 },
    { 0x8ca40be1, 93 },
    { 0x8f340be1, 52 },
    { 0x8e40be3, 90 },
    { 0xd2c40be4, 34 },
    { 0x9d240be4, 90 },
    { 0x88640be1, 96 },
    { 0x10440be6, 55 },
    { 0, 0 },
    { 0xc4940be9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78840bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed40bf1, 66 },
    { 0x63340bf1, 87 },
    { 0, 0 },
    { 0xdbc40bf4, 23 },
    { 0x9ea40bf5, 33 },
    { 0, 0 },
    { 0x42440bf7, 34 },
    { 0, 0 },
    { 0xf3840bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91540bfd, 23 },
    { 0x1ef40bfe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f40c05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340c08, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e240c10, 66 },
    { 0x9d040c11, 61 },
    { 0xa7740c10, 61 },
    { 0xda140c13, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93140c18, 99 },
    { 0x8c640c19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8240c1d, 17 },
    { 0xe540c1e, 98 },
    { 0x34a40c1f, 81 },
    { 0x80240c20, 90 },
    { 0, 0 },
    { 0x7d340c22, 100 },
    { 0, 0 },
    { 0xfd940c24, 33 },
    { 0, 0 },
    { 0xffc40c26, 98 },
    { 0x62840c26, 85 },
    { 0, 0 },
    { 0x33e40c29, 98 },
    { 0, 0 },
    { 0x40540c2b, 77 },
    { 0x54440c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50040c2f, 80 },
    { 0x1e140c30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x53440c33, 34 },
    { 0x7bd40c34, 52 },
    { 0xc6040c35, 90 },
    { 0xfcd40c36, 90 },
    { 0x83f40c34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef340c3a, 61 },
    { 0x53540c3a, 61 },
    { 0x1d40c3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e40c3f, 93 },
    { 0x57a40c40, 78 },
    { 0, 0 },
    { 0xb2a40c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde40c4a, 95 },
    { 0x3a640c4b, 83 },
    { 0xcec40c4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea40c4f, 84 },
    { 0x4cd40c50, 90 },
    { 0x8c640c51, 98 },
    { 0x47940c51, 55 },
    { 0xd1d40c53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c40c56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7840c5c, 99 },
    { 0xfcd40c5c, 61 },
    { 0x69440c5d, 33 },
    { 0xf2040c5e, 95 },
    { 0x17040c5d, 76 },
    { 0, 0 },
    { 0xdc240c62, 42 },
    { 0x59940c63, 76 },
    { 0xb2040c64, 56 },
    { 0x30740c63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc440c6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e40c72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b740c76, 99 },
    { 0, 0 },
    { 0x15740c78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f140c81, 87 },
    { 0xd6840c82, 83 },
    { 0xdc740c82, 67 },
    { 0xc8740c83, 23 },
    { 0xbf740c85, 87 },
    { 0xac340c86, 83 },
    { 0x10f40c85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f240c8f, 90 },
    { 0x27e40c90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d40c93, 78 },
    { 0x32740c93, 95 },
    { 0x4c440c95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55740ca0, 66 },
    { 0x14f40ca1, 90 },
    { 0, 0 },
    { 0x1ab40ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d40caa, 23 },
    { 0, 0 },
    { 0x31840cac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b40caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f840cb3, 33 },
    { 0xa4040cb3, 95 },
    { 0x97740cb3, 34 },
    { 0xd4240cb5, 76 },
    { 0xe0f40cb7, 33 },
    { 0xe140cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd40cbc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b40cbf, 34 },
    { 0, 0 },
    { 0x46340cc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e40cc7, 94 },
    { 0xf3740cc8, 98 },
    { 0x63740cc9, 90 },
    { 0x33f40cc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa440cce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40040cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1440cd6, 23 },
    { 0x59040cd7, 99 },
    { 0xf3d40cd7, 98 },
    { 0, 0 },
    { 0x20140cda, 67 },
    { 0x38e40cda, 76 },
    { 0x7640cda, 76 },
    { 0x89740cdd, 61 },
    { 0xd8f40cda, 90 },
    { 0xbcc40cdf, 98 },
    { 0x74440ce0, 33 },
    { 0xb4040cdf, 61 },
    { 0, 0 },
    { 0x6a440ce3, 66 },
    { 0x2a140ce3, 61 },
    { 0x6940ce3, 33 },
    { 0x75b40ce3, 34 },
    { 0, 0 },
    { 0x1e140ce8, 95 },
    { 0xa6a40ce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd40cef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda040cfa, 61 },
    { 0x6bf40cfb, 43 },
    { 0x61440cfc, 23 },
    { 0xc9f40cfc, 99 },
    { 0xe6140cfe, 91 },
    { 0xcef40cfb, 23 },
    { 0xf2640d00, 33 },
    { 0, 0 },
    { 0x7e940d02, 84 },
    { 0xe6040d02, 61 },
    { 0x77140d04, 90 },
    { 0xe3140d05, 87 },
    { 0x10740d02, 61 },
    { 0x71040d06, 95 },
    { 0xb7940d08, 34 },
    { 0, 0 },
    { 0xee40d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840d0f, 23 },
    { 0x1ea40d10, 46 },
    { 0x83f40d11, 76 },
    { 0, 0 },
    { 0xded40d13, 95 },
    { 0, 0 },
    { 0x140d15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd40d18, 93 },
    { 0xb8040d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee140d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37740d28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a540d2e, 60 },
    { 0x92440d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb40d35, 55 },
    { 0, 0 },
    { 0xd9740d37, 99 },
    { 0x3d40d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b540d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d40d41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49640d44, 33 },
    { 0, 0 },
    { 0xd3640d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb440d51, 76 },
    { 0x70a40d51, 33 },
    { 0x61840d51, 87 },
    { 0x29840d52, 90 },
    { 0xecd40d55, 23 },
    { 0xa7640d56, 90 },
    { 0x2ad40d56, 38 },
    { 0, 0 },
    { 0x4eb40d59, 61 },
    { 0x1b340d5a, 100 },
    { 0x3f540d5b, 55 },
    { 0x10640d5b, 33 },
    { 0xe6f40d5c, 23 },
    { 0xaf340d5e, 76 },
    { 0xb0c40d5f, 34 },
    { 0, 0 },
    { 0xc5e40d61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5640d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6040d67, 66 },
    { 0x28540d67, 55 },
    { 0x1c40d68, 61 },
    { 0x76340d68, 76 },
    { 0x59840d68, 93 },
    { 0x5f840d6c, 19 },
    { 0xb740d67, 93 },
    { 0xef940d6e, 77 },
    { 0xe4840d6a, 43 },
    { 0x7a040d70, 76 },
    { 0xa5340d71, 99 },
    { 0xa1b40d71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a340d76, 34 },
    { 0, 0 },
    { 0xb8340d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f740d7c, 46 },
    { 0xb5c40d7d, 66 },
    { 0xdf640d7e, 23 },
    { 0x86940d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d240d85, 78 },
    { 0, 0 },
    { 0x58f40d87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b140d8d, 33 },
    { 0, 0 },
    { 0xdd940d8f, 90 },
    { 0, 0 },
    { 0xc5d40d91, 23 },
    { 0xa1140d92, 100 },
    { 0xc1040d93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe40d9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da40da0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc740da4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d40db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d40db3, 34 },
    { 0x8cd40db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe40db9, 33 },
    { 0xbd140dba, 77 },
    { 0xdc540dbb, 76 },
    { 0, 0 },
    { 0x36840dbd, 98 },
    { 0x61f40dbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f640dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd40dc5, 61 },
    { 0x4fd40dc5, 87 },
    { 0x74840dc7, 61 },
    { 0, 0 },
    { 0xc5740dc9, 76 },
    { 0x43440dca, 61 },
    { 0, 0 },
    { 0xbfa40dcc, 33 },
    { 0x68240dcc, 76 },
    { 0x28c40dcc, 90 },
    { 0, 0 },
    { 0x85b40dd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5440dd3, 76 },
    { 0x86d40dd3, 61 },
    { 0xcf140dd5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5040dda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6640dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e40de6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc940de9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d040ded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10240df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb40df3, 100 },
    { 0xb2d40df3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0640df9, 61 },
    { 0, 0 },
    { 0xd2f40dfb, 95 },
    { 0x59240dfc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd940dff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c140e09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c140e0c, 55 },
    { 0, 0 },
    { 0xd2040e0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5240e11, 34 },
    { 0x59540e12, 50 },
    { 0x54e40e11, 100 },
    { 0x7ce40e14, 90 },
    { 0x82c40e15, 84 },
    { 0xfcf40e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f340e19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b40e1c, 95 },
    { 0xd6740e1c, 23 },
    { 0xce40e1e, 76 },
    { 0, 0 },
    { 0x6a440e20, 90 },
    { 0x6ea40e20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee440e25, 76 },
    { 0xbad40e26, 61 },
    { 0, 0 },
    { 0xcf140e28, 61 },
    { 0x15840e28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f40e2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67540e37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1340e3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38440e3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e40e45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9640e48, 33 },
    { 0x19e40e49, 82 },
    { 0xd2e40e4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae940e4d, 33 },
    { 0xcf040e4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d40e51, 33 },
    { 0x6440e51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89340e55, 98 },
    { 0, 0 },
    { 0x82c40e57, 23 },
    { 0x47740e57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1340e5b, 93 },
    { 0xd2b40e5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af40e61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31240e64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36340e67, 23 },
    { 0xf0640e67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11240e6d, 76 },
    { 0, 0 },
    { 0xca340e6f, 81 },
    { 0x3240e6f, 33 },
    { 0xd2a40e6f, 98 },
    { 0x87e40e72, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1440e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9740e7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d40e88, 75 },
    { 0x16640e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88840e8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e40e90, 34 },
    { 0x6640e90, 98 },
    { 0, 0 },
    { 0xcb40e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0440e98, 43 },
    { 0, 0 },
    { 0xebf40e9a, 68 },
    { 0xef040e9a, 93 },
    { 0xcbd40e9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88540e9f, 55 },
    { 0x92f40e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb40ea4, 95 },
    { 0xb6840ea5, 98 },
    { 0, 0 },
    { 0x65640ea7, 23 },
    { 0, 0 },
    { 0x1ee40ea9, 56 },
    { 0x3440ea9, 93 },
    { 0x1c040eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4840eb2, 34 },
    { 0xeea40eb3, 66 },
    { 0x22440eb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88540ec0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2040ec4, 84 },
    { 0xd2e40ec4, 33 },
    { 0x8bf40ec5, 90 },
    { 0, 0 },
    { 0x83740ec8, 22 },
    { 0, 0 },
    { 0x69140eca, 69 },
    { 0x4ec40eca, 34 },
    { 0, 0 },
    { 0xf1140ecd, 55 },
    { 0xb3840ece, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f840ed1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e40ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c240edc, 17 },
    { 0xe2140edd, 98 },
    { 0, 0 },
    { 0xe1c40edf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b40ee4, 98 },
    { 0x1dd40ee5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee40ee9, 87 },
    { 0xf0040eea, 34 },
    { 0x53f40eeb, 76 },
    { 0x38440ee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f40ef0, 90 },
    { 0x5c440ef1, 78 },
    { 0x72240ef1, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e140ef5, 87 },
    { 0x58040ef6, 51 },
    { 0x44040ef5, 61 },
    { 0xd9d40ef8, 90 },
    { 0, 0 },
    { 0x35c40efa, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb40efe, 95 },
    { 0x56540efe, 90 },
    { 0, 0 },
    { 0x51940f01, 55 },
    { 0x28440f02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b240f06, 100 },
    { 0, 0 },
    { 0xe9040f08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe440f0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6340f10, 90 },
    { 0x2e840f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa40f14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50240f17, 95 },
    { 0, 0 },
    { 0x76c40f19, 95 },
    { 0x57240f1a, 33 },
    { 0, 0 },
    { 0x3dd40f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340f24, 95 },
    { 0x21e40f25, 66 },
    { 0xf6940f26, 69 },
    { 0, 0 },
    { 0x89140f28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d040f34, 23 },
    { 0xb3940f34, 38 },
    { 0x3a940f36, 90 },
    { 0xa6c40f37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc940f40, 90 },
    { 0x3c40f40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c40f45, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x59340f48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf40f4b, 93 },
    { 0xd3a40f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c240f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53440f56, 66 },
    { 0x7db40f57, 61 },
    { 0xb4e40f56, 83 },
    { 0, 0 },
    { 0xd4640f5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54340f6e, 52 },
    { 0xc2740f6e, 91 },
    { 0x88d40f6e, 90 },
    { 0x32b40f70, 90 },
    { 0xef340f71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d40f7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4140f7d, 61 },
    { 0, 0 },
    { 0x64740f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d40f84, 61 },
    { 0xc7540f85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66140f8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e40f8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe40f91, 56 },
    { 0x6a640f91, 95 },
    { 0x1fb40f93, 42 },
    { 0x13e40f94, 90 },
    { 0xb3a40f95, 34 },
    { 0xca040f95, 95 },
    { 0, 0 },
    { 0xea640f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6240f9b, 61 },
    { 0x9dd40f9c, 91 },
    { 0x82f40f9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a40fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b40fa4, 90 },
    { 0x3b940fa4, 95 },
    { 0, 0 },
    { 0x66e40fa7, 67 },
    { 0x53940fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e40fab, 52 },
    { 0x69a40fac, 38 },
    { 0x25940fad, 61 },
    { 0xcaf40fac, 33 },
    { 0xfc740fac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xec840fb2, 33 },
    { 0x98a40fb3, 17 },
    { 0, 0 },
    { 0xa540fb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf640fb8, 98 },
    { 0, 0 },
    { 0x31540fba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2540fbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e40fc2, 98 },
    { 0x20040fc3, 76 },
    { 0xb7840fc4, 33 },
    { 0x97140fc5, 23 },
    { 0x68e40fc5, 88 },
    { 0xd2c40fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab340fd8, 90 },
    { 0x40740fd9, 61 },
    { 0x28b40fd9, 61 },
    { 0x5c40fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8740fde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74640fe2, 55 },
    { 0x88840fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69340fe7, 61 },
    { 0xce440fe8, 95 },
    { 0x7f040fe9, 33 },
    { 0x22c40fea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb940fef, 61 },
    { 0, 0 },
    { 0xd5440ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17340ff4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d40ffa, 43 },
    { 0x16440ffb, 77 },
    { 0xe340ffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a40fff, 76 },
    { 0x91441000, 23 },
    { 0x17741001, 98 },
    { 0, 0 },
    { 0x3241003, 33 },
    { 0x3d041004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf41007, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1564100a, 98 },
    { 0xd434100a, 87 },
    { 0x7764100b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e41010, 23 },
    { 0x14e41011, 61 },
    { 0xce841010, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed741016, 98 },
    { 0, 0 },
    { 0xd9241018, 98 },
    { 0xf1441019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4101c, 98 },
    { 0x66c4101c, 85 },
    { 0xbe94101e, 78 },
    { 0xb234101f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af41024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d4102b, 33 },
    { 0x8a44102c, 76 },
    { 0, 0 },
    { 0x9b94102e, 67 },
    { 0xbda4102e, 33 },
    { 0x82c4102e, 33 },
    { 0x20f41031, 90 },
    { 0xfb141032, 49 },
    { 0x8db4102f, 98 },
    { 0x13141034, 61 },
    { 0x9e641035, 98 },
    { 0x2bd41036, 95 },
    { 0x31841034, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f4103a, 100 },
    { 0xb594103a, 93 },
    { 0, 0 },
    { 0xc9c4103d, 55 },
    { 0xbaa4103e, 61 },
    { 0xe564103f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9741044, 43 },
    { 0xc7d41045, 78 },
    { 0, 0 },
    { 0x6be41047, 55 },
    { 0, 0 },
    { 0x75141049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214104e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8641056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68641059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94105d, 95 },
    { 0, 0 },
    { 0x97e4105f, 61 },
    { 0, 0 },
    { 0x8a41061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41064, 98 },
    { 0xd6e41065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef4106f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b41072, 61 },
    { 0x77741073, 90 },
    { 0x98741074, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254107b, 90 },
    { 0x8a84107b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24441080, 90 },
    { 0x6fb41081, 73 },
    { 0x18b41081, 46 },
    { 0x39a41083, 98 },
    { 0, 0 },
    { 0xc9941085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79541089, 90 },
    { 0x1784108a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa4108f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2441098, 78 },
    { 0xd8841099, 34 },
    { 0xe6841099, 98 },
    { 0xde241099, 90 },
    { 0xa5c4109c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1704109f, 100 },
    { 0, 0 },
    { 0xcd0410a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122410a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09410a8, 23 },
    { 0x7e5410a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc410b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956410b5, 33 },
    { 0xe3410b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec410b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89410bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a410c3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2410c6, 23 },
    { 0, 0 },
    { 0x4a8410c8, 55 },
    { 0x384410c9, 98 },
    { 0, 0 },
    { 0x1f0410cb, 98 },
    { 0x3c9410cc, 95 },
    { 0xd9d410cb, 33 },
    { 0xd36410cd, 100 },
    { 0x735410cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec410d3, 33 },
    { 0, 0 },
    { 0x96410d5, 61 },
    { 0xa45410d6, 33 },
    { 0x97d410d7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1410da, 86 },
    { 0, 0 },
    { 0xcfe410dc, 98 },
    { 0, 0 },
    { 0xb15410de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979410ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a410f4, 91 },
    { 0x199410f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf410fa, 33 },
    { 0xc8b410fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a410fe, 90 },
    { 0xa1a410ff, 66 },
    { 0x18041100, 98 },
    { 0x9aa41100, 23 },
    { 0x32841100, 90 },
    { 0x9d041103, 76 },
    { 0, 0 },
    { 0x7f941105, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5541108, 33 },
    { 0xe5a41108, 90 },
    { 0, 0 },
    { 0x40a4110b, 100 },
    { 0xfb74110b, 34 },
    { 0xb274110d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e41111, 95 },
    { 0, 0 },
    { 0x4d141113, 76 },
    { 0x42141114, 95 },
    { 0xc8c41115, 23 },
    { 0x67c41115, 61 },
    { 0x6c641117, 43 },
    { 0xc4e41117, 95 },
    { 0xf5b41116, 33 },
    { 0xb2941116, 61 },
    { 0x74d41117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2704111e, 23 },
    { 0x5ce4111f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e41122, 90 },
    { 0x9ea41122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10341127, 90 },
    { 0xc1241128, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f4112e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c41133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd41137, 33 },
    { 0xb2141138, 66 },
    { 0, 0 },
    { 0xc4f4113a, 9 },
    { 0x3944113a, 61 },
    { 0x6f74113b, 90 },
    { 0xa484113a, 93 },
    { 0x1d34113c, 33 },
    { 0xb324113f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb41143, 75 },
    { 0x3fa41143, 52 },
    { 0xe4141145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4114a, 61 },
    { 0xf544114b, 98 },
    { 0x4a14114c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d54114f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb241158, 90 },
    { 0x7ce41159, 98 },
    { 0xe5d41159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef4115d, 33 },
    { 0x54f4115e, 23 },
    { 0x404115e, 84 },
    { 0xc0941160, 34 },
    { 0x904115f, 34 },
    { 0x98c41162, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d41172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a41175, 98 },
    { 0x6b441176, 98 },
    { 0xe5f41177, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f24117c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2341180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e41183, 95 },
    { 0, 0 },
    { 0x141185, 95 },
    { 0, 0 },
    { 0x6dc41187, 76 },
    { 0x62a41188, 98 },
    { 0x61c41189, 61 },
    { 0x2ba4118a, 90 },
    { 0x3b44118a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb4118e, 23 },
    { 0, 0 },
    { 0x51741190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1504119e, 68 },
    { 0x3794119e, 90 },
    { 0, 0 },
    { 0xf91411a1, 98 },
    { 0x719411a2, 65 },
    { 0x9be411a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35411a7, 95 },
    { 0xde411a8, 98 },
    { 0xcb5411a9, 61 },
    { 0, 0 },
    { 0x9c3411ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead411af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a411b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd411c2, 95 },
    { 0x38411c3, 84 },
    { 0x464411c3, 91 },
    { 0xff9411c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b411cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f411ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58411d4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2411d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7411da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5411de, 33 },
    { 0x4a4411df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef411e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c411e7, 61 },
    { 0x41e411e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f411eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x332411ee, 66 },
    { 0, 0 },
    { 0xaaa411f0, 98 },
    { 0x92b411f1, 81 },
    { 0, 0 },
    { 0xf60411f3, 95 },
    { 0, 0 },
    { 0xbd2411f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f411f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63841202, 76 },
    { 0xc9641202, 98 },
    { 0xd3b41202, 90 },
    { 0x6f941202, 23 },
    { 0x52441202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79441210, 90 },
    { 0x77141211, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1041214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b41218, 98 },
    { 0x13741218, 90 },
    { 0x8824121a, 98 },
    { 0xab841219, 95 },
    { 0xa0441218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb041220, 90 },
    { 0xa9041221, 98 },
    { 0xab641222, 99 },
    { 0x8a441222, 90 },
    { 0xaad41224, 77 },
    { 0x5aa41224, 33 },
    { 0x5d041226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7341229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34122e, 90 },
    { 0x3c54122f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1441232, 90 },
    { 0, 0 },
    { 0x83f41234, 23 },
    { 0xdd441234, 61 },
    { 0x6aa41235, 95 },
    { 0xc7e41237, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464123d, 66 },
    { 0, 0 },
    { 0xff04123f, 90 },
    { 0xc8a41240, 61 },
    { 0x5fd41241, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2641247, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c4124f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a841254, 90 },
    { 0x88141254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64125c, 33 },
    { 0x3fd4125c, 46 },
    { 0xacd4125e, 33 },
    { 0xed14125c, 98 },
    { 0x11a41260, 33 },
    { 0xae641260, 53 },
    { 0xc6c41262, 66 },
    { 0xa8741263, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1541266, 33 },
    { 0xde041267, 61 },
    { 0x69441268, 95 },
    { 0xc7b41269, 55 },
    { 0x8ba4126a, 61 },
    { 0x7014126b, 87 },
    { 0x2fd4126a, 95 },
    { 0x4e34126d, 95 },
    { 0xdcc41268, 90 },
    { 0x6ef4126b, 95 },
    { 0xdbd41268, 87 },
    { 0xcd441271, 34 },
    { 0, 0 },
    { 0xc9541273, 52 },
    { 0xa9f41274, 91 },
    { 0x68141275, 33 },
    { 0x8d741273, 100 },
    { 0xb7841277, 85 },
    { 0, 0 },
    { 0xe8e41279, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5354127e, 95 },
    { 0, 0 },
    { 0xef341280, 90 },
    { 0x9d941281, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d541288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c4128c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764128f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60941293, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e41296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf241299, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xba24129c, 61 },
    { 0x3654129d, 61 },
    { 0xf414129e, 43 },
    { 0, 0 },
    { 0x22a412a0, 66 },
    { 0, 0 },
    { 0xaea412a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549412a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x235412a9, 90 },
    { 0xbeb412a9, 33 },
    { 0x7a1412ab, 91 },
    { 0, 0 },
    { 0x8412ad, 95 },
    { 0x528412ad, 52 },
    { 0, 0 },
    { 0xca1412b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f412b9, 90 },
    { 0, 0 },
    { 0xa4e412bb, 46 },
    { 0x822412bb, 65 },
    { 0xa1d412bd, 90 },
    { 0x380412bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xab412c1, 55 },
    { 0, 0 },
    { 0x531412c3, 78 },
    { 0xfde412c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf412c7, 90 },
    { 0x311412c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c412cb, 23 },
    { 0x62a412cc, 90 },
    { 0xcc8412cd, 95 },
    { 0, 0 },
    { 0x2ea412cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b412d3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923412d7, 90 },
    { 0, 0 },
    { 0x74d412d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6412e0, 90 },
    { 0xb5c412e0, 98 },
    { 0x5e2412e2, 98 },
    { 0xde4412e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5412f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58412f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x74412f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d41302, 44 },
    { 0x4e641303, 55 },
    { 0xfc341304, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c41307, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324130c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf4130f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a41313, 34 },
    { 0, 0 },
    { 0xf7e41315, 100 },
    { 0, 0 },
    { 0x3b641317, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b84131a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc4131f, 23 },
    { 0x61d41320, 33 },
    { 0xf8341321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54341325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7614132a, 98 },
    { 0, 0 },
    { 0xa14132c, 98 },
    { 0xec84132d, 23 },
    { 0x6394132e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9341333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe941336, 43 },
    { 0xf8041337, 33 },
    { 0x2ba41338, 93 },
    { 0x4ac41339, 90 },
    { 0xeb84133a, 76 },
    { 0xbdb41339, 70 },
    { 0x28d41337, 33 },
    { 0x3a541339, 55 },
    { 0xb454133e, 61 },
    { 0x4ed41337, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb4134a, 40 },
    { 0xe64134b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4134f, 61 },
    { 0x1204134f, 23 },
    { 0x5b41350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c4135c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe41361, 78 },
    { 0, 0 },
    { 0xb3541363, 93 },
    { 0xbc241364, 55 },
    { 0x86541365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b41369, 43 },
    { 0x9c54136a, 101 },
    { 0x26a4136b, 61 },
    { 0x9744136a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c41373, 93 },
    { 0x28f41374, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7144137a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a94137d, 90 },
    { 0xb684137e, 61 },
    { 0, 0 },
    { 0x7ad41380, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x78741383, 93 },
    { 0x80b41383, 88 },
    { 0xf3641385, 88 },
    { 0x13041386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad4138c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2324138f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff41392, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5541395, 95 },
    { 0xccf41395, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534139c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130413a0, 91 },
    { 0x99e413a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709413a6, 55 },
    { 0x21d413a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe413b2, 61 },
    { 0xbfb413b3, 33 },
    { 0x962413b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da413b7, 52 },
    { 0xdb0413b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5413bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b413c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf413c5, 90 },
    { 0x5a9413c6, 55 },
    { 0, 0 },
    { 0x851413c8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8413cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x477413d0, 53 },
    { 0x70f413d1, 90 },
    { 0, 0 },
    { 0x8cd413d3, 90 },
    { 0xfb1413d4, 98 },
    { 0xa71413d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753413db, 85 },
    { 0xad3413dc, 98 },
    { 0xb5e413dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b413e1, 84 },
    { 0xee0413e2, 90 },
    { 0x9ba413e3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c413eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc413f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7413f6, 91 },
    { 0, 0 },
    { 0x581413f8, 23 },
    { 0, 0 },
    { 0x2c6413fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87413fd, 76 },
    { 0x87a413fd, 99 },
    { 0xd54413ff, 81 },
    { 0xda941400, 61 },
    { 0x52e41400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d041407, 90 },
    { 0, 0 },
    { 0x73c41409, 90 },
    { 0x1674140a, 23 },
    { 0x3b74140a, 55 },
    { 0, 0 },
    { 0x3a44140d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd41410, 65 },
    { 0xade41410, 90 },
    { 0xcd141412, 44 },
    { 0x1a941413, 23 },
    { 0, 0 },
    { 0x68041415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404141f, 68 },
    { 0, 0 },
    { 0xd1c41421, 90 },
    { 0, 0 },
    { 0xcdd41423, 90 },
    { 0x97741423, 75 },
    { 0, 0 },
    { 0xc9341426, 87 },
    { 0xea041427, 94 },
    { 0xdcc41427, 43 },
    { 0, 0 },
    { 0x66e4142a, 43 },
    { 0, 0 },
    { 0x4904142c, 76 },
    { 0, 0 },
    { 0x43f4142e, 23 },
    { 0xa474142e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d441434, 66 },
    { 0xb4c41435, 90 },
    { 0x2a541436, 52 },
    { 0, 0 },
    { 0x51541438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d4143d, 77 },
    { 0x6b74143d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9541442, 81 },
    { 0, 0 },
    { 0xd7541444, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9441449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4144e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x52641451, 61 },
    { 0x76b41451, 34 },
    { 0x1a641452, 98 },
    { 0xdf441454, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xed741457, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4145c, 95 },
    { 0x2bf4145c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6841461, 33 },
    { 0x7ea41462, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67341467, 23 },
    { 0, 0 },
    { 0x27041469, 90 },
    { 0x3d94146a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4146e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6341475, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f4147b, 81 },
    { 0x3074147b, 95 },
    { 0x1234147d, 23 },
    { 0x9df4147d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c41485, 60 },
    { 0x86741486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d4148b, 23 },
    { 0xde24148b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64148f, 84 },
    { 0x194148f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c41493, 98 },
    { 0x84941494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574149c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4149f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d414a3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14414a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb414ac, 34 },
    { 0x6d3414ad, 61 },
    { 0, 0 },
    { 0xda1414af, 90 },
    { 0x328414af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f414b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78414bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616414c4, 61 },
    { 0x92f414c5, 26 },
    { 0x587414c6, 52 },
    { 0x83d414c6, 34 },
    { 0xeb1414c7, 61 },
    { 0, 0 },
    { 0xd2d414ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed414cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c414d1, 33 },
    { 0xd82414d2, 90 },
    { 0xdba414d3, 85 },
    { 0xf0d414d4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac414d8, 76 },
    { 0, 0 },
    { 0x78e414da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf414dd, 78 },
    { 0xb14414dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1414e1, 98 },
    { 0xe39414e1, 19 },
    { 0x911414e3, 100 },
    { 0x7e414e2, 88 },
    { 0x27414e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621414ef, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac414f4, 84 },
    { 0xffe414f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233414fb, 98 },
    { 0x931414fc, 76 },
    { 0x495414fd, 35 },
    { 0x997414fc, 93 },
    { 0xed0414ff, 98 },
    { 0x48b41500, 66 },
    { 0x69341500, 95 },
    { 0x9a8414fc, 95 },
    { 0, 0 },
    { 0xbfc41504, 61 },
    { 0xb4c41504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb641509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x824150c, 61 },
    { 0xe544150d, 33 },
    { 0xb164150e, 87 },
    { 0x3374150f, 34 },
    { 0xf424150e, 34 },
    { 0x87641510, 66 },
    { 0x4d041510, 75 },
    { 0x6a241513, 23 },
    { 0, 0 },
    { 0xba141515, 85 },
    { 0xff841516, 88 },
    { 0x99941516, 34 },
    { 0x5db41518, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11241520, 100 },
    { 0, 0 },
    { 0xc6241522, 33 },
    { 0xf3041523, 61 },
    { 0x43e41522, 46 },
    { 0x58241522, 61 },
    { 0x9c041526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb4152d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74341531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8341534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d41539, 95 },
    { 0xfa4153a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb204153d, 90 },
    { 0, 0 },
    { 0x74153f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d141545, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc41549, 93 },
    { 0x19f4154a, 77 },
    { 0x8064154a, 23 },
    { 0xbcf4154c, 23 },
    { 0x84e4154d, 90 },
    { 0x6d84154d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a841552, 93 },
    { 0x55841553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84155d, 95 },
    { 0x8b64155e, 90 },
    { 0x4604155f, 90 },
    { 0, 0 },
    { 0x1f141561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1641566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e4156a, 78 },
    { 0x8bd4156b, 90 },
    { 0x3304156c, 61 },
    { 0, 0 },
    { 0xc6e4156e, 23 },
    { 0, 0 },
    { 0xd3a41570, 61 },
    { 0xc8d41571, 90 },
    { 0x4ae41571, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d41578, 61 },
    { 0x4fd41579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2294157c, 46 },
    { 0, 0 },
    { 0xaf44157e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a41581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c41585, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a4158a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3484158e, 48 },
    { 0x7ed4158f, 46 },
    { 0xb8f4158f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace41594, 61 },
    { 0x89341595, 23 },
    { 0x5e041596, 61 },
    { 0x73341597, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4159c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994159f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548415a6, 98 },
    { 0x92a415a7, 55 },
    { 0x37f415a8, 67 },
    { 0xf97415a8, 67 },
    { 0xff4415a7, 95 },
    { 0x816415a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d415bc, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138415c5, 34 },
    { 0xbb8415c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd415c9, 96 },
    { 0x6c415c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac415cd, 33 },
    { 0, 0 },
    { 0xdb7415cf, 95 },
    { 0, 0 },
    { 0x377415d1, 33 },
    { 0, 0 },
    { 0xd93415d3, 61 },
    { 0x317415d3, 88 },
    { 0xc6c415d5, 87 },
    { 0x81b415d6, 61 },
    { 0x6f4415d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674415db, 23 },
    { 0x943415db, 52 },
    { 0, 0 },
    { 0xd5415de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a415e4, 90 },
    { 0x7f415e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2415e8, 33 },
    { 0xd93415e8, 43 },
    { 0, 0 },
    { 0x3ee415eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73415ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22415f2, 83 },
    { 0, 0 },
    { 0xc5f415f4, 33 },
    { 0xf00415f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21415fd, 46 },
    { 0x3d8415fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f441602, 70 },
    { 0xd9e41603, 33 },
    { 0, 0 },
    { 0xced41605, 95 },
    { 0x7e441605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe41609, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c4160e, 90 },
    { 0x44d4160e, 99 },
    { 0xcc04160f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f441613, 91 },
    { 0, 0 },
    { 0xb941615, 95 },
    { 0x7f641616, 33 },
    { 0x3ad41616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4161e, 88 },
    { 0x52c4161f, 66 },
    { 0x3eb41620, 98 },
    { 0xd154161f, 55 },
    { 0x9304161f, 98 },
    { 0, 0 },
    { 0x5b041624, 93 },
    { 0x4b541624, 66 },
    { 0xd3441626, 17 },
    { 0xfde41625, 98 },
    { 0xd4041628, 53 },
    { 0x5c941628, 78 },
    { 0x61341629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b41631, 56 },
    { 0xcd41632, 90 },
    { 0x1f841633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf41636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1541646, 61 },
    { 0xf4741647, 98 },
    { 0xd8541648, 84 },
    { 0xf9b41646, 52 },
    { 0, 0 },
    { 0x38c4164b, 64 },
    { 0x6b4164c, 78 },
    { 0xfa44164b, 33 },
    { 0, 0 },
    { 0x4644164f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c41654, 23 },
    { 0x2b741655, 67 },
    { 0x39141654, 52 },
    { 0xea41656, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864165a, 61 },
    { 0x3204165b, 33 },
    { 0x34e4165c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d41661, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99441666, 90 },
    { 0xadf41666, 61 },
    { 0, 0 },
    { 0xf0441669, 23 },
    { 0xfa841669, 61 },
    { 0xc14166b, 93 },
    { 0, 0 },
    { 0xdb4166d, 95 },
    { 0x4e14166d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f041672, 90 },
    { 0x2f241673, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24167a, 98 },
    { 0xa3e4167b, 33 },
    { 0x7804167a, 90 },
    { 0x4e24167d, 46 },
    { 0x2674167c, 90 },
    { 0xeca4167e, 78 },
    { 0x2ba41680, 61 },
    { 0, 0 },
    { 0x52541682, 95 },
    { 0, 0 },
    { 0xe4941684, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d54168b, 90 },
    { 0x9ae4168c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb4169a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c84169d, 95 },
    { 0x1f4169e, 83 },
    { 0xfd14169f, 33 },
    { 0xfee416a0, 88 },
    { 0x92c416a0, 98 },
    { 0xe54416a1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6416a5, 85 },
    { 0xa89416a5, 55 },
    { 0xb91416a7, 43 },
    { 0, 0 },
    { 0x18f416a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x487416ac, 36 },
    { 0x12c416ad, 61 },
    { 0x8d9416ae, 76 },
    { 0xc1f416af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b416b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d416be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x221416c1, 95 },
    { 0x50e416c2, 43 },
    { 0x8d7416c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f416c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a416cd, 23 },
    { 0x24416ce, 33 },
    { 0, 0 },
    { 0x714416d0, 34 },
    { 0xcb416d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8416d5, 43 },
    { 0xc43416d6, 23 },
    { 0x631416d7, 93 },
    { 0x152416d6, 95 },
    { 0x339416d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d416dd, 46 },
    { 0x36b416de, 46 },
    { 0xad2416de, 34 },
    { 0, 0 },
    { 0x274416e1, 55 },
    { 0x6fa416e2, 93 },
    { 0, 0 },
    { 0xfdf416e4, 90 },
    { 0, 0 },
    { 0xde1416e6, 95 },
    { 0x654416e7, 87 },
    { 0xc05416e8, 33 },
    { 0x85c416e7, 23 },
    { 0, 0 },
    { 0x15c416eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23416ee, 93 },
    { 0x46f416ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5416f4, 74 },
    { 0, 0 },
    { 0x5e2416f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf416f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x156416fc, 34 },
    { 0x97416fd, 34 },
    { 0xfc9416fe, 90 },
    { 0xc48416fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d41702, 78 },
    { 0x41341703, 77 },
    { 0, 0 },
    { 0x8a941705, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c4170b, 93 },
    { 0x2b04170b, 61 },
    { 0x2fb4170d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66841711, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6041714, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc4171b, 61 },
    { 0xa394171b, 95 },
    { 0xfb14171b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51541721, 90 },
    { 0x91641721, 90 },
    { 0x4ec41722, 43 },
    { 0xefb41723, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b4172b, 98 },
    { 0, 0 },
    { 0x54172d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee941733, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88441736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd41739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc4173c, 48 },
    { 0x28e4173d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf541744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3241749, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3141751, 78 },
    { 0xcf141751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae941756, 79 },
    { 0x66941757, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd964175b, 23 },
    { 0xdab4175b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93641761, 98 },
    { 0x62141762, 90 },
    { 0x4641763, 99 },
    { 0, 0 },
    { 0x67a41765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc4176a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b41771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b41777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74177d, 86 },
    { 0x6c64177d, 90 },
    { 0, 0 },
    { 0x1f541780, 98 },
    { 0, 0 },
    { 0x9b641782, 61 },
    { 0, 0 },
    { 0x74241784, 98 },
    { 0x54041785, 77 },
    { 0xdec41786, 95 },
    { 0x16a41787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c4178e, 75 },
    { 0, 0 },
    { 0xe6241790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a41798, 43 },
    { 0xd9d41799, 95 },
    { 0xe974179a, 95 },
    { 0x1014179a, 90 },
    { 0xa5f4179c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5394179f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7417a2, 95 },
    { 0, 0 },
    { 0xbee417a4, 61 },
    { 0, 0 },
    { 0xe95417a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587417aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a417ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b417b2, 90 },
    { 0xca5417b3, 98 },
    { 0xa7c417b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9417b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187417bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e417c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c417c3, 23 },
    { 0xc56417c4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb417c7, 34 },
    { 0xe2d417c7, 43 },
    { 0x1d8417c9, 66 },
    { 0xaf3417c8, 93 },
    { 0, 0 },
    { 0xe46417cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9417d0, 33 },
    { 0x81c417d1, 59 },
    { 0, 0 },
    { 0xe72417d3, 90 },
    { 0x86e417d4, 76 },
    { 0x55a417d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4417d9, 61 },
    { 0x154417da, 90 },
    { 0xfe8417d9, 33 },
    { 0x182417dc, 66 },
    { 0xc42417dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2417e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x667417e5, 23 },
    { 0xa09417e6, 95 },
    { 0xd08417e7, 23 },
    { 0xe9417e8, 61 },
    { 0x8d3417e9, 33 },
    { 0x7d7417ea, 81 },
    { 0xa31417eb, 90 },
    { 0xb0417ea, 98 },
    { 0x952417e7, 90 },
    { 0x2f3417ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99417f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6417f8, 90 },
    { 0, 0 },
    { 0x76c417fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553417fe, 52 },
    { 0xf03417ff, 76 },
    { 0x376417fe, 61 },
    { 0xb1d41800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27441807, 88 },
    { 0xec041807, 90 },
    { 0x27341809, 55 },
    { 0xbd04180a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264180f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a941813, 100 },
    { 0xecc41814, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324181a, 61 },
    { 0xc1f4181b, 23 },
    { 0, 0 },
    { 0x6644181d, 98 },
    { 0x9684181e, 90 },
    { 0x7e64181e, 90 },
    { 0x3f341820, 55 },
    { 0x88a41821, 84 },
    { 0xe5941821, 33 },
    { 0, 0 },
    { 0x4ef41824, 78 },
    { 0x47941825, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f4182c, 67 },
    { 0x1734182d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7441830, 33 },
    { 0x34741830, 23 },
    { 0xebc41832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4541836, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4183b, 36 },
    { 0xf844183c, 95 },
    { 0, 0 },
    { 0xa694183e, 98 },
    { 0x8a64183f, 61 },
    { 0x844183f, 33 },
    { 0xb1d4183f, 43 },
    { 0x86341842, 61 },
    { 0, 0 },
    { 0xab441844, 95 },
    { 0x70541845, 23 },
    { 0x12341846, 33 },
    { 0x32941847, 95 },
    { 0x2a441845, 90 },
    { 0, 0 },
    { 0xb884184a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32841851, 83 },
    { 0xe9f41852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7954185f, 90 },
    { 0, 0 },
    { 0xb3941861, 84 },
    { 0x37941861, 90 },
    { 0, 0 },
    { 0x57141864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71241868, 43 },
    { 0xfb941869, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4186c, 33 },
    { 0xe154186c, 95 },
    { 0, 0 },
    { 0xf64186f, 81 },
    { 0x8ed4186f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd741873, 78 },
    { 0xc1f41874, 88 },
    { 0, 0 },
    { 0x7a341876, 90 },
    { 0xb2641877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464187a, 98 },
    { 0xfd54187a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f64187f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc41884, 97 },
    { 0, 0 },
    { 0x95e41886, 75 },
    { 0xd7d41887, 68 },
    { 0xe3f41888, 100 },
    { 0x6bb41888, 98 },
    { 0x1264188a, 90 },
    { 0x4a041889, 14 },
    { 0, 0 },
    { 0xfa64188d, 90 },
    { 0xbae4188e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b941892, 34 },
    { 0x6da41893, 99 },
    { 0xffc41894, 90 },
    { 0xe8941895, 95 },
    { 0xb1841895, 61 },
    { 0x53a41897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72418a1, 33 },
    { 0x9fa418a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864418a6, 90 },
    { 0x2db418a7, 33 },
    { 0x739418a6, 90 },
    { 0xb94418a9, 87 },
    { 0xda7418a6, 33 },
    { 0xf65418aa, 43 },
    { 0x45d418a7, 33 },
    { 0x103418ad, 46 },
    { 0xcb418ad, 33 },
    { 0xbff418af, 61 },
    { 0x746418ad, 40 },
    { 0xab3418b0, 61 },
    { 0x80c418b1, 69 },
    { 0, 0 },
    { 0xddd418b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe418bd, 33 },
    { 0x821418be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280418c3, 34 },
    { 0xd59418c4, 33 },
    { 0xcdc418c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3418c8, 33 },
    { 0x7d4418c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352418cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b418d8, 95 },
    { 0x207418d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29418e0, 23 },
    { 0x97f418e1, 84 },
    { 0x974418e2, 90 },
    { 0xccb418e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b418e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7418ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b418f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2418f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8418fa, 33 },
    { 0xb8418fb, 56 },
    { 0x479418fb, 98 },
    { 0x32a418fb, 95 },
    { 0x729418fe, 33 },
    { 0x2d5418ff, 98 },
    { 0x9fe41900, 98 },
    { 0xb72418fe, 33 },
    { 0x4a941902, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58341908, 33 },
    { 0x8a041909, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43141912, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45341915, 95 },
    { 0xa0f41915, 83 },
    { 0x29041917, 98 },
    { 0, 0 },
    { 0x74a41919, 90 },
    { 0, 0 },
    { 0x5d04191b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04191e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48341923, 76 },
    { 0xf2041924, 61 },
    { 0, 0 },
    { 0x93141926, 33 },
    { 0x6641927, 23 },
    { 0, 0 },
    { 0x75441929, 76 },
    { 0, 0 },
    { 0x6d74192b, 68 },
    { 0xb874192c, 85 },
    { 0x89c4192c, 90 },
    { 0x3ae4192e, 95 },
    { 0x1b4192f, 76 },
    { 0x84b41930, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc41935, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9341940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774194c, 95 },
    { 0x1814194d, 76 },
    { 0, 0 },
    { 0xacf4194f, 38 },
    { 0x46741950, 87 },
    { 0, 0 },
    { 0xfaf41952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a641955, 90 },
    { 0, 0 },
    { 0xa0341957, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824195d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2941962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d4196e, 23 },
    { 0, 0 },
    { 0xb6241970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d341973, 61 },
    { 0, 0 },
    { 0xaf941975, 40 },
    { 0x37941975, 52 },
    { 0x17d41977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b4197d, 90 },
    { 0x7204197d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43841986, 61 },
    { 0x8cb41986, 76 },
    { 0, 0 },
    { 0xec441989, 90 },
    { 0xa2b4198a, 61 },
    { 0xe9c4198b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2514198e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f41991, 99 },
    { 0, 0 },
    { 0x14241993, 33 },
    { 0xa0b41994, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c4199d, 91 },
    { 0xb944199d, 75 },
    { 0, 0 },
    { 0xb6c419a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35419ab, 98 },
    { 0x2d5419ac, 90 },
    { 0x421419ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb419b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b419b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9419bd, 90 },
    { 0xdac419be, 78 },
    { 0x487419bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76419c2, 61 },
    { 0xe6a419c2, 34 },
    { 0x20419c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78419cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd419ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef419d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c419d8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c419db, 90 },
    { 0, 0 },
    { 0x669419dd, 90 },
    { 0, 0 },
    { 0x38419df, 33 },
    { 0x4f8419e0, 98 },
    { 0x59a419e0, 33 },
    { 0x93c419e0, 61 },
    { 0, 0 },
    { 0x4b2419e4, 33 },
    { 0xbbf419e5, 23 },
    { 0xa1f419e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a419ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a419f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca419f4, 87 },
    { 0, 0 },
    { 0xb05419f6, 40 },
    { 0x5f419f7, 95 },
    { 0x434419f8, 34 },
    { 0xe11419f9, 34 },
    { 0xe2419fa, 23 },
    { 0xbd0419fb, 46 },
    { 0x275419fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57541a08, 33 },
    { 0x80a41a08, 61 },
    { 0, 0 },
    { 0x78d41a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea241a0e, 98 },
    { 0xb8e41a0f, 61 },
    { 0x60d41a0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60541a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be41a1c, 43 },
    { 0x9a341a1d, 98 },
    { 0xab041a1e, 76 },
    { 0, 0 },
    { 0xb1d41a20, 20 },
    { 0x25c41a20, 90 },
    { 0xa6f41a20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc41a27, 66 },
    { 0x2141a28, 61 },
    { 0, 0 },
    { 0x2a741a2a, 85 },
    { 0, 0 },
    { 0x94041a2c, 55 },
    { 0xf4d41a2d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24741a32, 76 },
    { 0x65641a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41a3c, 55 },
    { 0x46641a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb041a41, 80 },
    { 0x1641a42, 46 },
    { 0, 0 },
    { 0x13a41a44, 52 },
    { 0xe8a41a44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a341a49, 33 },
    { 0, 0 },
    { 0xb1341a4b, 90 },
    { 0xd3041a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48241a4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e041a56, 61 },
    { 0xe041a56, 33 },
    { 0x59141a57, 93 },
    { 0xd2c41a59, 100 },
    { 0xb4a41a58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2441a61, 90 },
    { 0xca241a61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c141a65, 33 },
    { 0, 0 },
    { 0x2cb41a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c41a6a, 98 },
    { 0x9041a6a, 61 },
    { 0xe1041a6a, 87 },
    { 0, 0 },
    { 0xbda41a6e, 90 },
    { 0xe3441a6e, 90 },
    { 0x57141a6e, 100 },
    { 0x7f341a70, 24 },
    { 0xaec41a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba341a75, 61 },
    { 0x3941a76, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6941a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba41a7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7141a7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4141a82, 46 },
    { 0x91841a83, 46 },
    { 0, 0 },
    { 0xbd341a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44341a8b, 90 },
    { 0x2b341a8b, 66 },
    { 0x6b941a8d, 23 },
    { 0x85b41a8e, 90 },
    { 0, 0 },
    { 0x88c41a90, 55 },
    { 0xa3441a91, 78 },
    { 0, 0 },
    { 0x99041a93, 90 },
    { 0x3ca41a94, 95 },
    { 0x10941a95, 93 },
    { 0, 0 },
    { 0xdeb41a97, 34 },
    { 0x5ab41a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83641a9c, 90 },
    { 0xba41a9c, 94 },
    { 0xd8741a9c, 33 },
    { 0x36441a9d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e41aa2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63441aaa, 33 },
    { 0xc0f41aab, 85 },
    { 0x4741aaa, 77 },
    { 0xe3741aaa, 76 },
    { 0xcc41aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041ab1, 55 },
    { 0x71a41ab2, 66 },
    { 0, 0 },
    { 0x2ef41ab4, 85 },
    { 0x78a41ab5, 46 },
    { 0x15b41ab6, 95 },
    { 0, 0 },
    { 0x28441ab8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc441abb, 43 },
    { 0, 0 },
    { 0xe2041abd, 55 },
    { 0x2e941abe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52941ac2, 90 },
    { 0x5f641ac2, 84 },
    { 0x9bb41ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d41ac7, 55 },
    { 0x37941ac8, 99 },
    { 0x1a041ac9, 95 },
    { 0x44941ac8, 90 },
    { 0xc0241acb, 95 },
    { 0x73241acb, 97 },
    { 0x99e41acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7541ad0, 66 },
    { 0xbc741ad0, 95 },
    { 0xde041ad0, 68 },
    { 0x35241ad0, 55 },
    { 0, 0 },
    { 0x3db41ad5, 95 },
    { 0xd6941ad6, 90 },
    { 0x30e41ad7, 33 },
    { 0x1fd41ad7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd741add, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b41ae4, 99 },
    { 0x15441ae4, 33 },
    { 0x6fc41ae6, 65 },
    { 0, 0 },
    { 0xef941ae8, 90 },
    { 0, 0 },
    { 0xa0041aea, 33 },
    { 0xfae41aeb, 33 },
    { 0x60341aec, 95 },
    { 0x8441aeb, 34 },
    { 0x65641aee, 17 },
    { 0x40e41aed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa641af6, 90 },
    { 0, 0 },
    { 0x5ca41af8, 33 },
    { 0x72f41af9, 52 },
    { 0, 0 },
    { 0x94d41afb, 87 },
    { 0, 0 },
    { 0x1e441afd, 61 },
    { 0x37a41afe, 6 },
    { 0xc8e41afd, 95 },
    { 0xba941b00, 55 },
    { 0x6d741b01, 23 },
    { 0, 0 },
    { 0x1c441b03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd41b07, 85 },
    { 0x90c41b08, 90 },
    { 0, 0 },
    { 0x43a41b0a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea041b1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a41b1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2841b29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30341b2c, 23 },
    { 0, 0 },
    { 0x61841b2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e741b35, 61 },
    { 0x6e441b35, 90 },
    { 0, 0 },
    { 0x32141b38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b41b3b, 33 },
    { 0x4ec41b3b, 90 },
    { 0x5d841b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a41b40, 61 },
    { 0, 0 },
    { 0xd4c41b42, 98 },
    { 0xa7241b43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb41b47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5441b4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c541b50, 76 },
    { 0, 0 },
    { 0x6f341b52, 76 },
    { 0xf8241b53, 95 },
    { 0xb5541b54, 90 },
    { 0xa2641b52, 98 },
    { 0x8d341b54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27641b5e, 61 },
    { 0, 0 },
    { 0x5b041b60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5841b69, 90 },
    { 0x4af41b6a, 80 },
    { 0xea241b69, 66 },
    { 0xd7c41b6b, 98 },
    { 0, 0 },
    { 0x12141b6e, 43 },
    { 0xe7241b6e, 40 },
    { 0, 0 },
    { 0x6ad41b71, 33 },
    { 0xf6041b72, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc41b79, 87 },
    { 0, 0 },
    { 0xdf741b7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b041b83, 33 },
    { 0, 0 },
    { 0x9db41b85, 93 },
    { 0x27141b85, 90 },
    { 0, 0 },
    { 0xc0a41b88, 78 },
    { 0xa1e41b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca441b8d, 34 },
    { 0x2b41b8e, 90 },
    { 0xf5641b8f, 33 },
    { 0xac941b8e, 85 },
    { 0xb7541b91, 38 },
    { 0, 0 },
    { 0x85c41b93, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c41b97, 90 },
    { 0xf7c41b98, 55 },
    { 0xa5941b99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a41b9c, 61 },
    { 0, 0 },
    { 0x2dc41b9e, 61 },
    { 0xf0841b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d41ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52441ba8, 76 },
    { 0x9141ba9, 33 },
    { 0x69e41baa, 52 },
    { 0, 0 },
    { 0x1a41bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e41bb0, 93 },
    { 0, 0 },
    { 0x2b741bb2, 33 },
    { 0x6ba41bb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d141bb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c41bbb, 76 },
    { 0x82141bbc, 55 },
    { 0xf6541bbd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f641bc2, 53 },
    { 0, 0 },
    { 0x5be41bc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a41bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e041bd1, 95 },
    { 0x20a41bd1, 23 },
    { 0x7c841bd3, 46 },
    { 0, 0 },
    { 0x7e341bd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f41bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae41bdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c41be4, 98 },
    { 0x68c41be4, 84 },
    { 0x38741be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf41be9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d41bec, 34 },
    { 0, 0 },
    { 0x6bf41bee, 99 },
    { 0, 0 },
    { 0xb9441bf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19741bf9, 90 },
    { 0x4e241bfa, 61 },
    { 0xa0d41bfa, 92 },
    { 0x7e241bfb, 90 },
    { 0x47e41bfd, 61 },
    { 0x18a41bfa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6141c02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c841c06, 61 },
    { 0xdc341c07, 100 },
    { 0, 0 },
    { 0xea741c09, 55 },
    { 0x78641c0a, 76 },
    { 0x55e41c09, 33 },
    { 0xd5641c0c, 76 },
    { 0, 0 },
    { 0xb541c0e, 61 },
    { 0, 0 },
    { 0x3c841c10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44941c13, 90 },
    { 0x2741c14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c41c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde341c1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee41c24, 66 },
    { 0x5be41c25, 93 },
    { 0x3bb41c25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40941c29, 90 },
    { 0x31841c2a, 98 },
    { 0xcdd41c2a, 90 },
    { 0xc1a41c2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd41c2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43141c36, 55 },
    { 0xea741c37, 66 },
    { 0, 0 },
    { 0xae141c39, 78 },
    { 0, 0 },
    { 0xcbe41c3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7441c40, 43 },
    { 0xfa241c40, 76 },
    { 0x89041c42, 98 },
    { 0xbd041c43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde041c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec941c4a, 66 },
    { 0xc7d41c4a, 33 },
    { 0x14241c4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e41c53, 33 },
    { 0x89041c53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e41c5d, 96 },
    { 0x4b241c5d, 61 },
    { 0x7f641c5f, 85 },
    { 0xe8a41c5f, 46 },
    { 0x5af41c5d, 95 },
    { 0xeae41c62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3941c66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b41c69, 23 },
    { 0xb8d41c6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e41c75, 55 },
    { 0xee241c75, 34 },
    { 0, 0 },
    { 0x41741c78, 95 },
    { 0x1d741c79, 46 },
    { 0, 0 },
    { 0xa7e41c7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac41c81, 95 },
    { 0x51e41c81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf141c87, 53 },
    { 0xffe41c88, 66 },
    { 0x61f41c88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d141c93, 90 },
    { 0x20b41c94, 90 },
    { 0x31c41c95, 33 },
    { 0x2bc41c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f741c99, 95 },
    { 0, 0 },
    { 0xf741c9b, 90 },
    { 0xaaa41c9c, 55 },
    { 0x22a41c9d, 23 },
    { 0x29341c9d, 67 },
    { 0x21741c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d41ca5, 98 },
    { 0x25341ca5, 91 },
    { 0xdb441ca7, 56 },
    { 0xbb641ca8, 98 },
    { 0x68741ca8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec41caf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61141cb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2341cb8, 61 },
    { 0x8741cb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a41cbc, 87 },
    { 0, 0 },
    { 0xe4b41cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd841cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3841cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e41cc8, 76 },
    { 0xaca41cc9, 99 },
    { 0xf5741cc9, 23 },
    { 0x81141ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e41cd0, 93 },
    { 0xd1b41cd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c41cd5, 80 },
    { 0x60741cd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6141cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f41cdf, 33 },
    { 0x56d41ce0, 75 },
    { 0, 0 },
    { 0x6ef41ce2, 43 },
    { 0, 0 },
    { 0xdd441ce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6241ce8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba41cec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x26941cef, 23 },
    { 0xe4441cf0, 43 },
    { 0xafa41cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb641cf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27041cff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94141d07, 23 },
    { 0x53341d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60441d0c, 91 },
    { 0x8f441d0d, 33 },
    { 0xcb241d0c, 33 },
    { 0x7ef41d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d41d12, 61 },
    { 0xbe241d13, 33 },
    { 0x4db41d14, 66 },
    { 0, 0 },
    { 0x72941d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9541d1b, 69 },
    { 0xd2641d1c, 43 },
    { 0x4b441d1b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b941d23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66841d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd41d2c, 85 },
    { 0x97f41d2c, 90 },
    { 0, 0 },
    { 0x63841d2f, 34 },
    { 0, 0 },
    { 0x71341d31, 87 },
    { 0x8ad41d32, 55 },
    { 0xf5141d33, 88 },
    { 0x25d41d34, 61 },
    { 0xa6741d35, 23 },
    { 0, 0 },
    { 0x2c841d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca241d3a, 33 },
    { 0, 0 },
    { 0xead41d3c, 90 },
    { 0, 0 },
    { 0x7c441d3e, 64 },
    { 0xa4241d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc41d43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3141d4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0441d56, 95 },
    { 0xbe541d57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7141d5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad41d5e, 61 },
    { 0xce641d5e, 92 },
    { 0, 0 },
    { 0x1f141d61, 76 },
    { 0, 0 },
    { 0xc5141d63, 95 },
    { 0, 0 },
    { 0x6d041d65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91341d69, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62441d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e41d71, 65 },
    { 0xeac41d71, 33 },
    { 0, 0 },
    { 0xb3e41d74, 90 },
    { 0x98141d75, 95 },
    { 0xb3f41d75, 33 },
    { 0, 0 },
    { 0x95741d78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0641d7c, 90 },
    { 0x5a741d7d, 36 },
    { 0x76541d7e, 33 },
    { 0xaae41d7e, 81 },
    { 0xe3841d80, 98 },
    { 0x1c641d81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0041d8b, 55 },
    { 0x57b41d8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae41d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db41d94, 98 },
    { 0xfa941d95, 90 },
    { 0xd2441d96, 34 },
    { 0x50141d97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4241d9e, 33 },
    { 0xe6a41d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84541da3, 67 },
    { 0x39e41da4, 78 },
    { 0xeba41da4, 95 },
    { 0x67841da6, 87 },
    { 0xc1441da7, 81 },
    { 0x88241da8, 23 },
    { 0x56941da9, 23 },
    { 0xeb641daa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c41db1, 61 },
    { 0x94741db2, 23 },
    { 0x29f41db3, 23 },
    { 0x5241db4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d641db7, 61 },
    { 0xdf441db7, 34 },
    { 0x5ff41db9, 55 },
    { 0x8c241dba, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4241dbd, 66 },
    { 0, 0 },
    { 0xdc41dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37241dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97041dc5, 23 },
    { 0xcca41dc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c41dc9, 90 },
    { 0x2cd41dca, 46 },
    { 0, 0 },
    { 0x9f041dcc, 98 },
    { 0x86941dcd, 97 },
    { 0, 0 },
    { 0xa2441dcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9941dd7, 90 },
    { 0x9ef41dd8, 61 },
    { 0, 0 },
    { 0x7941dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d541de2, 90 },
    { 0, 0 },
    { 0xe1e41de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e41de9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec41dee, 90 },
    { 0x5f41dee, 90 },
    { 0x8b841df0, 95 },
    { 0x2ef41df0, 68 },
    { 0xe241df2, 34 },
    { 0x94541df0, 33 },
    { 0x64641df4, 34 },
    { 0x1fe41df4, 100 },
    { 0x4a741df6, 90 },
    { 0x5d341df5, 87 },
    { 0x18941df1, 100 },
    { 0xba241df9, 90 },
    { 0xc4a41dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c41dfe, 61 },
    { 0, 0 },
    { 0xa7041e00, 23 },
    { 0x7f441e00, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b41e04, 76 },
    { 0xbf941e05, 90 },
    { 0x5d141e05, 66 },
    { 0xcc341e07, 90 },
    { 0, 0 },
    { 0x8b241e09, 43 },
    { 0xd6f41e0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd41e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841e12, 95 },
    { 0x68441e13, 66 },
    { 0xb2f41e14, 87 },
    { 0x3e941e15, 52 },
    { 0xc6941e16, 61 },
    { 0x89141e15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee41e1c, 33 },
    { 0, 0 },
    { 0xce341e1e, 61 },
    { 0x3341e1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a41e25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39841e29, 76 },
    { 0xe6441e2a, 61 },
    { 0, 0 },
    { 0xe1741e2c, 23 },
    { 0, 0 },
    { 0x26041e2e, 33 },
    { 0xbae41e2f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63841e36, 98 },
    { 0, 0 },
    { 0x46041e38, 76 },
    { 0xca841e39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be41e42, 61 },
    { 0xad441e42, 46 },
    { 0xc9041e44, 46 },
    { 0x1f641e44, 90 },
    { 0x69341e46, 76 },
    { 0, 0 },
    { 0x11341e48, 43 },
    { 0, 0 },
    { 0xd9741e4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf241e4e, 93 },
    { 0, 0 },
    { 0xa6b41e50, 33 },
    { 0, 0 },
    { 0x3de41e52, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e41e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e41e5d, 34 },
    { 0, 0 },
    { 0xa0241e5f, 78 },
    { 0x33e41e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf641e63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32441e6b, 81 },
    { 0x7c341e6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8541e71, 61 },
    { 0xd9941e71, 99 },
    { 0x35941e73, 61 },
    { 0x4ba41e74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d441e77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3341e7b, 23 },
    { 0x1bf41e7c, 56 },
    { 0x25441e7b, 87 },
    { 0x1d841e7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34841e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70141e85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd541e88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b41e8c, 95 },
    { 0, 0 },
    { 0xf9c41e8e, 24 },
    { 0x28941e8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9341e92, 95 },
    { 0xcc941e92, 81 },
    { 0, 0 },
    { 0x26741e95, 61 },
    { 0, 0 },
    { 0x6c541e97, 43 },
    { 0, 0 },
    { 0xd9441e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e41ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76641ea5, 100 },
    { 0, 0 },
    { 0xfbf41ea7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac441eae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a41eb4, 34 },
    { 0xd8641eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab41eba, 33 },
    { 0x8ef41ebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d741ec0, 80 },
    { 0xb141ec1, 90 },
    { 0xf741ec1, 43 },
    { 0x17c41ec1, 95 },
    { 0xee541ec4, 61 },
    { 0, 0 },
    { 0x30841ec6, 84 },
    { 0x78f41ec7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf41ecc, 78 },
    { 0, 0 },
    { 0x70941ece, 55 },
    { 0x4b441ecf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f41ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d41edf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41ee2, 90 },
    { 0, 0 },
    { 0xc2c41ee4, 33 },
    { 0, 0 },
    { 0x77f41ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c41eea, 98 },
    { 0xbb41eeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13741ef1, 55 },
    { 0xa3c41ef2, 78 },
    { 0xf5441ef1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x28841ef6, 46 },
    { 0, 0 },
    { 0xa7d41ef8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45041efe, 98 },
    { 0, 0 },
    { 0xaa41f00, 90 },
    { 0xd1141f01, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f441f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0841f08, 76 },
    { 0x11b41f08, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8341f0c, 76 },
    { 0xf4741f0d, 90 },
    { 0x13d41f0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec41f16, 23 },
    { 0xe2041f17, 90 },
    { 0, 0 },
    { 0x4f741f19, 52 },
    { 0x2f941f1a, 43 },
    { 0xf9241f19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71841f20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd341f24, 95 },
    { 0x1f741f25, 34 },
    { 0xd0141f24, 94 },
    { 0x28541f27, 34 },
    { 0x7ec41f24, 46 },
    { 0x45041f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e841f2c, 33 },
    { 0x99841f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0641f30, 12 },
    { 0, 0 },
    { 0x79341f32, 84 },
    { 0x56241f33, 98 },
    { 0xe2941f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76541f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a41f3c, 98 },
    { 0x73e41f3d, 87 },
    { 0, 0 },
    { 0x5dc41f3f, 55 },
    { 0x62a41f40, 98 },
    { 0x5f641f3f, 95 },
    { 0xddb41f3f, 76 },
    { 0x9e041f41, 46 },
    { 0x1041f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541f4b, 33 },
    { 0, 0 },
    { 0x8641f4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a41f51, 56 },
    { 0x9a941f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f41f56, 95 },
    { 0xdd641f57, 99 },
    { 0x78341f57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a41f5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x63241f62, 33 },
    { 0, 0 },
    { 0x1e841f64, 100 },
    { 0xafa41f64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe741f6e, 90 },
    { 0x81e41f6e, 33 },
    { 0x10b41f6e, 76 },
    { 0x94a41f71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b41f78, 81 },
    { 0, 0 },
    { 0x37041f7a, 100 },
    { 0xb3341f7a, 76 },
    { 0, 0 },
    { 0xe1f41f7d, 71 },
    { 0x7ca41f7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53041f81, 98 },
    { 0xe3c41f82, 98 },
    { 0xee341f81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e141f87, 55 },
    { 0x17e41f87, 55 },
    { 0xfa641f89, 66 },
    { 0xe9b41f87, 33 },
    { 0x72a41f8b, 29 },
    { 0x26941f8c, 34 },
    { 0xf7941f8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae41f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a41f99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b41f9d, 85 },
    { 0xa8741f9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b41fa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e41fa4, 98 },
    { 0xdd741fa5, 98 },
    { 0x8c641fa6, 55 },
    { 0, 0 },
    { 0x49c41fa8, 98 },
    { 0, 0 },
    { 0x7a641faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b641fae, 90 },
    { 0, 0 },
    { 0x2ed41fb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5141fb3, 23 },
    { 0, 0 },
    { 0xbcf41fb5, 95 },
    { 0x39f41fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe341fbc, 52 },
    { 0x6f641fbd, 43 },
    { 0, 0 },
    { 0x38041fbf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b41fc2, 88 },
    { 0xe3c41fc2, 23 },
    { 0xc7641fc4, 95 },
    { 0x2be41fc5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27341fca, 55 },
    { 0x16e41fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce541fd2, 61 },
    { 0, 0 },
    { 0xcb41fd4, 90 },
    { 0xe8341fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72841fda, 33 },
    { 0xb1a41fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b941fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea441fe3, 95 },
    { 0x7a541fe4, 34 },
    { 0x2fd41fe5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59341fe9, 43 },
    { 0xfe141fe9, 98 },
    { 0x2941fe9, 67 },
    { 0x51141fec, 46 },
    { 0xe8341fea, 100 },
    { 0xb1b41fee, 100 },
    { 0x28d41fef, 53 },
    { 0x67a41fee, 95 },
    { 0x8a841ff1, 87 },
    { 0xbd641ff2, 85 },
    { 0x23041ff2, 88 },
    { 0x36941ff0, 75 },
    { 0xec141ff5, 76 },
    { 0x8f941fed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba41ffe, 23 },
    { 0xa2841ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd42003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e42008, 76 },
    { 0x48342009, 98 },
    { 0x4c74200a, 61 },
    { 0x4aa4200a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2824200e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa42017, 95 },
    { 0x3a242017, 95 },
    { 0x81942017, 33 },
    { 0xd6a4201a, 33 },
    { 0x20f4201b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4201e, 90 },
    { 0xf7d4201f, 91 },
    { 0x5c84201e, 77 },
    { 0xbce42020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e42025, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b42028, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54202b, 33 },
    { 0x6fa4202c, 87 },
    { 0x8154202d, 61 },
    { 0x3694202d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae42036, 99 },
    { 0x45d42037, 33 },
    { 0, 0 },
    { 0x29542039, 61 },
    { 0x3424203a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa4203e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9774204b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3374204e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c842051, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd642056, 33 },
    { 0, 0 },
    { 0x39942058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e4205b, 90 },
    { 0xf554205c, 17 },
    { 0x1254205c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa742060, 52 },
    { 0xa2642061, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43542067, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4206d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf942071, 61 },
    { 0x84d42071, 23 },
    { 0x93742072, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c742078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8914207b, 76 },
    { 0xd884207c, 87 },
    { 0xfb74207b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf42081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f42084, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d42087, 33 },
    { 0, 0 },
    { 0x22b42089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43442091, 55 },
    { 0x60742092, 68 },
    { 0, 0 },
    { 0x3b842094, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea42097, 81 },
    { 0xb8242098, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84209b, 52 },
    { 0x9b14209c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604420a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78420a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1420aa, 61 },
    { 0x2b9420ab, 66 },
    { 0xc04420ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4420b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815420bf, 91 },
    { 0xba5420c0, 33 },
    { 0x819420c1, 23 },
    { 0x29b420c2, 98 },
    { 0xa82420c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed420c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489420cc, 67 },
    { 0xa96420cc, 34 },
    { 0x8fc420cc, 78 },
    { 0x1f4420cf, 61 },
    { 0, 0 },
    { 0xd9c420d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd420d5, 34 },
    { 0x812420d6, 61 },
    { 0x3e420d7, 90 },
    { 0x4c7420d7, 90 },
    { 0x58a420d9, 43 },
    { 0x7bb420d8, 61 },
    { 0xf07420d6, 90 },
    { 0x559420dc, 98 },
    { 0x991420dd, 23 },
    { 0xb47420de, 55 },
    { 0, 0 },
    { 0x933420e0, 23 },
    { 0xa10420e1, 90 },
    { 0xe35420e1, 33 },
    { 0x641420e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x730420e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3420e9, 33 },
    { 0x98f420ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997420ee, 61 },
    { 0xc31420ee, 61 },
    { 0x318420f0, 61 },
    { 0x8ad420f1, 23 },
    { 0, 0 },
    { 0xaae420f3, 23 },
    { 0xaed420f3, 33 },
    { 0, 0 },
    { 0x54d420f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31420fe, 66 },
    { 0, 0 },
    { 0x34042100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa42103, 52 },
    { 0x89642103, 23 },
    { 0x64942105, 34 },
    { 0x14a42103, 23 },
    { 0xa7842107, 90 },
    { 0xef42107, 100 },
    { 0xf5e42108, 95 },
    { 0, 0 },
    { 0x5894210b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd44210f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce42117, 55 },
    { 0xf042117, 23 },
    { 0xb0c42117, 67 },
    { 0x5314211a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5242122, 61 },
    { 0x4642123, 61 },
    { 0xd1642123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142129, 55 },
    { 0, 0 },
    { 0x2444212b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e142131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf542137, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae94213b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa114213f, 90 },
    { 0x494213f, 33 },
    { 0x4ee42141, 93 },
    { 0xd542140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e42145, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72242149, 85 },
    { 0xc414214a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f4214d, 90 },
    { 0xfb94214e, 88 },
    { 0x7434214f, 99 },
    { 0x75d4214e, 75 },
    { 0xb904214d, 61 },
    { 0x23242152, 93 },
    { 0xe3f42150, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2142157, 90 },
    { 0, 0 },
    { 0x81542159, 33 },
    { 0xd0c4215a, 61 },
    { 0xfe74215b, 98 },
    { 0xd6f4215a, 90 },
    { 0, 0 },
    { 0x38a4215e, 55 },
    { 0xcaf4215f, 65 },
    { 0xac842160, 55 },
    { 0x7144215f, 55 },
    { 0x3ac42162, 61 },
    { 0xcd42160, 99 },
    { 0x98e42164, 55 },
    { 0xeb542165, 23 },
    { 0x60742166, 98 },
    { 0xf7b42167, 61 },
    { 0x7f042161, 23 },
    { 0x30f42169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb54216c, 34 },
    { 0, 0 },
    { 0x4354216e, 90 },
    { 0, 0 },
    { 0x19c42170, 61 },
    { 0x73742170, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed42178, 90 },
    { 0xc5a42178, 43 },
    { 0xe0942178, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f4217e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c542181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb42184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef42187, 49 },
    { 0xa0742188, 77 },
    { 0xc9642188, 90 },
    { 0, 0 },
    { 0xca34218b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44218f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51442192, 55 },
    { 0xb7a42193, 55 },
    { 0, 0 },
    { 0xaef42195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11842199, 90 },
    { 0xfd54219a, 90 },
    { 0x6fc4219b, 90 },
    { 0xd784219c, 33 },
    { 0x7444219d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153421a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54421a6, 61 },
    { 0x104421a7, 98 },
    { 0x5fc421a6, 98 },
    { 0xe0e421a9, 80 },
    { 0xe37421aa, 90 },
    { 0x730421a7, 93 },
    { 0x752421a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4421b0, 98 },
    { 0, 0 },
    { 0xd19421b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d421b6, 43 },
    { 0, 0 },
    { 0x439421b8, 93 },
    { 0, 0 },
    { 0xfe4421ba, 46 },
    { 0x25b421ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63421be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397421c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0421ca, 91 },
    { 0xce6421cb, 80 },
    { 0x82c421ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e421cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc421d2, 61 },
    { 0, 0 },
    { 0xe82421d4, 33 },
    { 0x39a421d4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c421e0, 98 },
    { 0xd06421e1, 61 },
    { 0xe95421e2, 66 },
    { 0xc7c421e0, 95 },
    { 0x4e9421e4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01421f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db421f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25421ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb42203, 55 },
    { 0x86b42204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd04220b, 23 },
    { 0x5bd4220b, 91 },
    { 0xff14220b, 23 },
    { 0x3194220e, 90 },
    { 0x6ed4220f, 23 },
    { 0, 0 },
    { 0xa0642211, 90 },
    { 0x15842212, 23 },
    { 0, 0 },
    { 0x2e442214, 61 },
    { 0x4ce42214, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0842219, 61 },
    { 0, 0 },
    { 0xe474221b, 100 },
    { 0x1b14221c, 33 },
    { 0x4944221d, 17 },
    { 0xe7c4221e, 84 },
    { 0x4444221e, 95 },
    { 0, 0 },
    { 0xabb42221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4242224, 61 },
    { 0xbf842225, 34 },
    { 0xc6a42226, 61 },
    { 0xc4e42227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d44222d, 76 },
    { 0x2e54222d, 90 },
    { 0xf334222f, 52 },
    { 0xfa142230, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8842234, 88 },
    { 0xdc42235, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2894223c, 61 },
    { 0xe304223d, 90 },
    { 0xc3d4223d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51642242, 90 },
    { 0x4cc42243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b742246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764224b, 90 },
    { 0x2ff4224b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3864224f, 90 },
    { 0, 0 },
    { 0xeea42251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80842256, 93 },
    { 0x7de42257, 67 },
    { 0x68542257, 33 },
    { 0xd0b42259, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4225f, 93 },
    { 0x9434225f, 93 },
    { 0x30542261, 90 },
    { 0xbd542261, 33 },
    { 0x31f42263, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e84226d, 93 },
    { 0x33b4226d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6942272, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe42275, 91 },
    { 0xa9d42275, 23 },
    { 0x7642277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb4227a, 61 },
    { 0, 0 },
    { 0x9be4227c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3242281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a42285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e44228d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85542291, 23 },
    { 0, 0 },
    { 0x1e542293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf42296, 90 },
    { 0xd6b42296, 55 },
    { 0, 0 },
    { 0x73942299, 76 },
    { 0x2c742299, 95 },
    { 0xe624229b, 80 },
    { 0x294229b, 34 },
    { 0, 0 },
    { 0xb794229e, 99 },
    { 0, 0 },
    { 0xcee422a0, 15 },
    { 0xc69422a1, 94 },
    { 0, 0 },
    { 0xe59422a3, 95 },
    { 0, 0 },
    { 0xba422a5, 23 },
    { 0x175422a6, 76 },
    { 0x2d1422a7, 34 },
    { 0xd47422a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab422ac, 52 },
    { 0x203422ad, 74 },
    { 0xf77422ae, 23 },
    { 0x18422af, 34 },
    { 0x6d6422af, 95 },
    { 0, 0 },
    { 0xd5c422b2, 98 },
    { 0xf50422b2, 33 },
    { 0x2d1422b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b422b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67422bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1422c1, 99 },
    { 0xc73422c2, 15 },
    { 0, 0 },
    { 0x526422c4, 95 },
    { 0, 0 },
    { 0x913422c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171422c9, 46 },
    { 0xeba422ca, 33 },
    { 0x4422cb, 33 },
    { 0, 0 },
    { 0x7bb422cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0422d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28422d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad422db, 55 },
    { 0x236422dc, 23 },
    { 0xd2e422dd, 61 },
    { 0xb4f422dd, 67 },
    { 0x761422df, 61 },
    { 0x8c4422dd, 61 },
    { 0xe6f422e0, 94 },
    { 0xb14422e2, 61 },
    { 0, 0 },
    { 0x836422e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c422e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79422ea, 90 },
    { 0x8b422eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1422f6, 95 },
    { 0xf0e422f7, 55 },
    { 0x702422f6, 33 },
    { 0x69d422f9, 55 },
    { 0, 0 },
    { 0xf19422fb, 90 },
    { 0, 0 },
    { 0x925422fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f42304, 40 },
    { 0, 0 },
    { 0xc4742306, 43 },
    { 0, 0 },
    { 0x26f42308, 23 },
    { 0x81f42309, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f4230c, 76 },
    { 0x9ad4230d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35242311, 67 },
    { 0xa1542312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17542318, 91 },
    { 0xa2142318, 100 },
    { 0x4994231a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384231e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d42321, 66 },
    { 0x1f342321, 90 },
    { 0, 0 },
    { 0x3f842324, 75 },
    { 0xc0b42325, 76 },
    { 0xd3a42325, 33 },
    { 0x2e942327, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874232b, 33 },
    { 0x5f94232b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90442331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50942335, 76 },
    { 0x7d542336, 33 },
    { 0x57842336, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d54233b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a42341, 61 },
    { 0, 0 },
    { 0x8db42343, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68342347, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8854234c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe294234f, 61 },
    { 0x4ea42350, 77 },
    { 0, 0 },
    { 0x89a42352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e442355, 76 },
    { 0x9d942356, 23 },
    { 0, 0 },
    { 0x43742358, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc42360, 80 },
    { 0xa9042360, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4236a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa942370, 34 },
    { 0, 0 },
    { 0x78a42372, 76 },
    { 0, 0 },
    { 0xb2242374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf42377, 76 },
    { 0, 0 },
    { 0xc6e42379, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba4237d, 53 },
    { 0x1444237e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc542388, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf034238b, 98 },
    { 0, 0 },
    { 0x6c34238d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd42393, 61 },
    { 0x24842394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda42398, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d4239b, 76 },
    { 0x4f94239c, 81 },
    { 0xd8a4239d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d423a0, 76 },
    { 0xd8e423a1, 65 },
    { 0x4ff423a2, 33 },
    { 0, 0 },
    { 0xe00423a4, 33 },
    { 0x96423a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35423aa, 90 },
    { 0, 0 },
    { 0x7c5423ac, 95 },
    { 0, 0 },
    { 0x524423ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee423b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a423b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79423bc, 78 },
    { 0xc1423bd, 34 },
    { 0x65a423be, 38 },
    { 0xd35423bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1423c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x218423cc, 76 },
    { 0, 0 },
    { 0x775423ce, 55 },
    { 0x5b5423ce, 98 },
    { 0x2ed423ce, 95 },
    { 0xdbf423d1, 90 },
    { 0xbdf423d2, 76 },
    { 0x243423d1, 23 },
    { 0xb2e423d2, 61 },
    { 0xdd9423d5, 55 },
    { 0x67c423d6, 76 },
    { 0xaef423d5, 90 },
    { 0x985423d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d423db, 91 },
    { 0x709423db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x978423df, 23 },
    { 0xc71423df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db423e6, 76 },
    { 0xce5423e7, 40 },
    { 0, 0 },
    { 0xbfe423e9, 90 },
    { 0x5cf423e9, 33 },
    { 0, 0 },
    { 0x168423ec, 99 },
    { 0xf09423ed, 23 },
    { 0x9f5423ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3423f3, 66 },
    { 0x7d9423f3, 33 },
    { 0xa63423f5, 98 },
    { 0x1ef423f6, 33 },
    { 0x6f0423f6, 23 },
    { 0x26a423f4, 93 },
    { 0x464423f9, 33 },
    { 0x21e423f9, 90 },
    { 0xe1f423f6, 98 },
    { 0x702423fc, 75 },
    { 0xeef423fc, 43 },
    { 0, 0 },
    { 0xa5b423ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f042405, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b742409, 95 },
    { 0xc924240a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384240f, 78 },
    { 0xbdd42410, 95 },
    { 0x4c142410, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61842417, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f24241b, 36 },
    { 0, 0 },
    { 0xa3d4241d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21742420, 67 },
    { 0, 0 },
    { 0x7a942422, 76 },
    { 0xd0842422, 58 },
    { 0x61f42424, 90 },
    { 0x28e42424, 33 },
    { 0xc6b42423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6042429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd4242c, 61 },
    { 0xc594242c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1342432, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92142435, 61 },
    { 0, 0 },
    { 0xaf042437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b94243e, 93 },
    { 0, 0 },
    { 0x22642440, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96042443, 33 },
    { 0xf2a42444, 81 },
    { 0, 0 },
    { 0x21c42446, 33 },
    { 0xf3742447, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb014244f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b42452, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e42456, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce4245a, 43 },
    { 0xb374245a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf42462, 61 },
    { 0xe3c42462, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b42466, 99 },
    { 0, 0 },
    { 0x4da42468, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294246c, 33 },
    { 0xae54246d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba42477, 87 },
    { 0xad142478, 91 },
    { 0, 0 },
    { 0x5614247a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94247e, 67 },
    { 0, 0 },
    { 0x29542480, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f842489, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6394248d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4642491, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe42497, 98 },
    { 0x10642498, 95 },
    { 0, 0 },
    { 0x954249a, 33 },
    { 0x6674249b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee4249e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88424a1, 80 },
    { 0x883424a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x812424a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a424a9, 95 },
    { 0xd31424aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc424ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709424b1, 40 },
    { 0, 0 },
    { 0xeca424b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a424b7, 43 },
    { 0xd8d424b8, 33 },
    { 0xdf9424b9, 43 },
    { 0x479424b9, 66 },
    { 0x7c7424b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x729424be, 55 },
    { 0xbc424bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x581424c2, 55 },
    { 0, 0 },
    { 0x2c8424c4, 99 },
    { 0, 0 },
    { 0xd76424c6, 52 },
    { 0x7d1424c6, 23 },
    { 0xf34424c8, 76 },
    { 0, 0 },
    { 0x88e424ca, 92 },
    { 0x7c4424cb, 76 },
    { 0, 0 },
    { 0xe28424cd, 23 },
    { 0, 0 },
    { 0x443424cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb424d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef424d6, 67 },
    { 0xf0424d6, 53 },
    { 0x824424d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5424db, 75 },
    { 0, 0 },
    { 0x1fd424dd, 90 },
    { 0x9f6424de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa424e2, 98 },
    { 0, 0 },
    { 0x531424e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8424ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x525424ed, 23 },
    { 0x95b424ee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9424f1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333424f5, 61 },
    { 0xc9b424f5, 77 },
    { 0x3e8424f7, 95 },
    { 0x981424f8, 43 },
    { 0x51f424f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3424fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9424ff, 90 },
    { 0, 0 },
    { 0xeb942501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb42505, 95 },
    { 0, 0 },
    { 0xf4642507, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d4250b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10142510, 33 },
    { 0x3ec42511, 98 },
    { 0, 0 },
    { 0x7f42513, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9342517, 87 },
    { 0xf3642517, 95 },
    { 0x17242519, 76 },
    { 0x8a64251a, 99 },
    { 0x84b4251a, 56 },
    { 0, 0 },
    { 0x5734251d, 55 },
    { 0, 0 },
    { 0x2034251f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49542522, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a442529, 98 },
    { 0, 0 },
    { 0xcd04252b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9342534, 33 },
    { 0x12842535, 67 },
    { 0xb2042536, 93 },
    { 0xad442537, 61 },
    { 0, 0 },
    { 0xf742539, 90 },
    { 0, 0 },
    { 0x3c94253b, 87 },
    { 0x1434253c, 85 },
    { 0xb2c4253d, 55 },
    { 0xe454253b, 95 },
    { 0x1b94253d, 81 },
    { 0xf164253f, 90 },
    { 0x28742541, 100 },
    { 0xac742541, 76 },
    { 0x1e342542, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40142546, 33 },
    { 0, 0 },
    { 0x15542548, 90 },
    { 0xf1b42548, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec14254d, 95 },
    { 0, 0 },
    { 0xcab4254f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3842552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784255c, 23 },
    { 0x9964255d, 82 },
    { 0x7134255e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b42565, 90 },
    { 0xcf842566, 83 },
    { 0x5f842567, 40 },
    { 0, 0 },
    { 0x10b42569, 23 },
    { 0, 0 },
    { 0x7d94256b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7114256e, 33 },
    { 0xa204256e, 67 },
    { 0x90142570, 95 },
    { 0xce842570, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed842574, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38242578, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4244257c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9342582, 90 },
    { 0xe4f42583, 43 },
    { 0x8e042584, 61 },
    { 0xa0142585, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c542588, 22 },
    { 0, 0 },
    { 0x1d4258a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634258f, 33 },
    { 0x8b042590, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2204259b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa4259e, 53 },
    { 0x53b4259e, 90 },
    { 0x871425a0, 61 },
    { 0xd9d425a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05425a6, 43 },
    { 0, 0 },
    { 0xab1425a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7425ac, 95 },
    { 0x590425ad, 87 },
    { 0x68a425ad, 95 },
    { 0xe08425af, 85 },
    { 0x1b6425b0, 66 },
    { 0x256425b1, 61 },
    { 0x79b425b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed425b6, 23 },
    { 0, 0 },
    { 0xe4c425b8, 66 },
    { 0, 0 },
    { 0x3af425ba, 95 },
    { 0x513425bb, 52 },
    { 0x651425bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc425bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72425c4, 93 },
    { 0, 0 },
    { 0x7a6425c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0425ca, 46 },
    { 0x5c7425ca, 95 },
    { 0x436425cb, 90 },
    { 0xaf8425cc, 90 },
    { 0xf01425cc, 95 },
    { 0, 0 },
    { 0xe95425d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x857425d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6425dd, 61 },
    { 0xe5b425dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81425e3, 61 },
    { 0, 0 },
    { 0xd6b425e5, 90 },
    { 0, 0 },
    { 0x7b4425e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83425ed, 90 },
    { 0, 0 },
    { 0x541425ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8425f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x370425f8, 61 },
    { 0x704425f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea425fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d42601, 84 },
    { 0xc2142602, 40 },
    { 0xc9142601, 90 },
    { 0, 0 },
    { 0x4d242605, 95 },
    { 0x4d342606, 66 },
    { 0x65b42606, 43 },
    { 0xf3a42605, 87 },
    { 0xa6d42609, 68 },
    { 0x4374260a, 90 },
    { 0x68742609, 34 },
    { 0x154260c, 61 },
    { 0x74242608, 76 },
    { 0x50142608, 55 },
    { 0, 0 },
    { 0xa3142610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22142615, 90 },
    { 0xefa42616, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e42619, 61 },
    { 0xc4942619, 90 },
    { 0x6de4261a, 33 },
    { 0x96d4261c, 98 },
    { 0, 0 },
    { 0xea64261e, 95 },
    { 0, 0 },
    { 0xf9642620, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb942623, 61 },
    { 0xa3e42624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf514262f, 43 },
    { 0x69d42630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb142634, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c54263a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef4263d, 61 },
    { 0xa034263e, 100 },
    { 0x9524263e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d42644, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6524264a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e42654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6142658, 76 },
    { 0, 0 },
    { 0x3134265a, 65 },
    { 0xee54265a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e842661, 38 },
    { 0x20442662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93042667, 95 },
    { 0x36c42667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24266c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4266f, 55 },
    { 0xabc42670, 34 },
    { 0, 0 },
    { 0x94042672, 95 },
    { 0x10f42673, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc434267b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9294267e, 23 },
    { 0, 0 },
    { 0x40842680, 78 },
    { 0xd9042681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e42685, 98 },
    { 0x5aa42686, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e4268d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c742692, 81 },
    { 0x35642693, 61 },
    { 0x7d342694, 95 },
    { 0xe8e42693, 75 },
    { 0xd3e42696, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d42699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4269e, 99 },
    { 0xa464269f, 52 },
    { 0x5f04269f, 95 },
    { 0xd9426a1, 98 },
    { 0xc74426a2, 34 },
    { 0, 0 },
    { 0xf66426a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f426ab, 33 },
    { 0x9ee426ab, 76 },
    { 0, 0 },
    { 0xaa1426ae, 78 },
    { 0, 0 },
    { 0x4e3426b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8426b3, 90 },
    { 0xe93426b4, 67 },
    { 0xf57426b5, 76 },
    { 0x9af426b6, 98 },
    { 0x98b426b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9426c7, 61 },
    { 0xd45426c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74426cf, 78 },
    { 0xcce426cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993426d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920426d9, 23 },
    { 0x1a9426da, 61 },
    { 0xcdc426d9, 84 },
    { 0x1cb426db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d426df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45426e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51426e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6426e9, 61 },
    { 0xea8426ea, 23 },
    { 0xc5b426e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73426ee, 66 },
    { 0x215426ef, 91 },
    { 0xb4a426f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6426f9, 98 },
    { 0xb27426fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41426fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c42701, 90 },
    { 0xffe42701, 90 },
    { 0x69642703, 23 },
    { 0, 0 },
    { 0x42742705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234270a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854270d, 60 },
    { 0x4b94270e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e42715, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b84271c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d42720, 90 },
    { 0x3f42721, 85 },
    { 0x46942720, 23 },
    { 0, 0 },
    { 0x88042724, 90 },
    { 0xff742724, 61 },
    { 0, 0 },
    { 0x7042727, 75 },
    { 0xb342728, 95 },
    { 0x26c42728, 61 },
    { 0x5004272a, 90 },
    { 0x574272a, 61 },
    { 0, 0 },
    { 0x3e94272d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d42735, 23 },
    { 0, 0 },
    { 0x9c42737, 93 },
    { 0xc6f42737, 90 },
    { 0x13242739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a4273c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8364273f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35042744, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b642748, 93 },
    { 0x7f242749, 90 },
    { 0xe24274a, 90 },
    { 0xf664274a, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6714274f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7242756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8274275e, 55 },
    { 0x83b4275f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39342767, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c4276c, 76 },
    { 0x6944276c, 34 },
    { 0x8dd4276e, 33 },
    { 0x1884276f, 23 },
    { 0, 0 },
    { 0xd3942771, 95 },
    { 0, 0 },
    { 0xd0742773, 33 },
    { 0x87a42774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48642784, 23 },
    { 0, 0 },
    { 0xf6e42786, 90 },
    { 0, 0 },
    { 0xf1242788, 33 },
    { 0xb7a42789, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5524278e, 90 },
    { 0x1104278f, 90 },
    { 0, 0 },
    { 0xd3642791, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4279b, 82 },
    { 0, 0 },
    { 0x1534279d, 61 },
    { 0x8b74279e, 76 },
    { 0, 0 },
    { 0x290427a0, 90 },
    { 0xba3427a1, 61 },
    { 0x3dd427a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5427a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3427b0, 40 },
    { 0, 0 },
    { 0xe4c427b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d427b7, 23 },
    { 0xa6e427b7, 98 },
    { 0, 0 },
    { 0x7a0427ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db427c0, 90 },
    { 0, 0 },
    { 0x37427c2, 90 },
    { 0, 0 },
    { 0xa70427c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5427c9, 78 },
    { 0x839427c9, 90 },
    { 0x20e427cb, 33 },
    { 0, 0 },
    { 0xa02427cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90427d0, 76 },
    { 0, 0 },
    { 0x6e3427d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953427da, 46 },
    { 0xf9a427da, 90 },
    { 0, 0 },
    { 0xb71427dd, 33 },
    { 0, 0 },
    { 0x786427df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3427e5, 66 },
    { 0x8e7427e6, 95 },
    { 0x3be427e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653427eb, 61 },
    { 0x556427eb, 61 },
    { 0x71c427ec, 23 },
    { 0xe57427ec, 95 },
    { 0xbf5427ef, 95 },
    { 0xead427ef, 33 },
    { 0x63427ee, 55 },
    { 0, 0 },
    { 0xf49427f3, 61 },
    { 0x368427f4, 76 },
    { 0, 0 },
    { 0xf92427f6, 61 },
    { 0x6ef427f6, 91 },
    { 0, 0 },
    { 0x7cd427f9, 95 },
    { 0xb78427f9, 95 },
    { 0, 0 },
    { 0xa88427fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b42801, 95 },
    { 0, 0 },
    { 0x1cb42803, 34 },
    { 0x95b42804, 33 },
    { 0x18442805, 90 },
    { 0, 0 },
    { 0x40e42807, 98 },
    { 0xabd42808, 93 },
    { 0x98342809, 23 },
    { 0xa504280a, 83 },
    { 0, 0 },
    { 0x91b4280c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b942810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd42817, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e54281d, 66 },
    { 0xe624281e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c42822, 33 },
    { 0, 0 },
    { 0xbda42824, 76 },
    { 0xcc42825, 100 },
    { 0, 0 },
    { 0xb5542827, 99 },
    { 0, 0 },
    { 0x9b542829, 55 },
    { 0x10d42829, 34 },
    { 0x56042829, 23 },
    { 0, 0 },
    { 0xb174282d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a442833, 93 },
    { 0, 0 },
    { 0x23e42835, 76 },
    { 0, 0 },
    { 0xf7e42837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a4283a, 90 },
    { 0x5f4283b, 76 },
    { 0, 0 },
    { 0x8c44283d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c242843, 23 },
    { 0x2b442844, 84 },
    { 0x1de42843, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37042849, 55 },
    { 0xd344284a, 95 },
    { 0x6564284b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda42851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9842856, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774285b, 33 },
    { 0x2dc4285c, 61 },
    { 0, 0 },
    { 0x79c4285e, 34 },
    { 0x79b4285f, 56 },
    { 0xadd4285e, 61 },
    { 0x25542860, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d342866, 33 },
    { 0, 0 },
    { 0x14942868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4286d, 33 },
    { 0xaab4286e, 84 },
    { 0, 0 },
    { 0xd4d42870, 40 },
    { 0, 0 },
    { 0x1642872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a4287e, 77 },
    { 0x8cd4287e, 91 },
    { 0xa454287f, 90 },
    { 0xe7442881, 40 },
    { 0x39442882, 95 },
    { 0, 0 },
    { 0xeab42884, 95 },
    { 0xf42885, 90 },
    { 0x3d42886, 83 },
    { 0x40a42887, 61 },
    { 0xa8b42887, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4288b, 90 },
    { 0, 0 },
    { 0xd6c4288d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff42891, 61 },
    { 0xb3b42892, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1842895, 33 },
    { 0, 0 },
    { 0xf9e42897, 95 },
    { 0x77042897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4289b, 66 },
    { 0xa424289c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6428a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0428a3, 33 },
    { 0x576428a3, 97 },
    { 0x254428a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57428ac, 84 },
    { 0xaa0428ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a428b4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3428b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8428ba, 66 },
    { 0, 0 },
    { 0x124428bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87428c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e428c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x919428c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36428ca, 33 },
    { 0, 0 },
    { 0x76a428cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22428d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7428d8, 56 },
    { 0x62e428d9, 95 },
    { 0, 0 },
    { 0x6e5428db, 88 },
    { 0xf9b428db, 52 },
    { 0x8e0428dd, 77 },
    { 0x8cd428dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983428e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414428e9, 90 },
    { 0xb03428ea, 98 },
    { 0, 0 },
    { 0x93428ec, 87 },
    { 0x3af428ec, 76 },
    { 0x41f428ee, 77 },
    { 0xe8428ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17428fc, 43 },
    { 0x65a428fd, 55 },
    { 0x31d428fc, 95 },
    { 0x54b428ff, 33 },
    { 0x8a5428fe, 55 },
    { 0xfba42901, 91 },
    { 0xef642901, 100 },
    { 0xa8242903, 98 },
    { 0xc742901, 33 },
    { 0, 0 },
    { 0x9a042906, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e94290a, 84 },
    { 0xe8c4290b, 95 },
    { 0xfb54290c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b4291d, 90 },
    { 0x7d4291e, 85 },
    { 0, 0 },
    { 0x96442920, 77 },
    { 0, 0 },
    { 0xcfc42922, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51442926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3804292a, 23 },
    { 0xf4d4292b, 81 },
    { 0xa284292a, 96 },
    { 0, 0 },
    { 0x2074292e, 75 },
    { 0xc194292f, 98 },
    { 0x3a24292f, 98 },
    { 0x4ea42930, 52 },
    { 0x4b44292f, 33 },
    { 0x98242933, 33 },
    { 0x90a42934, 75 },
    { 0xe3142935, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9954293a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14293d, 85 },
    { 0x38f4293e, 93 },
    { 0xdf74293f, 77 },
    { 0x8eb42940, 75 },
    { 0x63442941, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8442944, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7542948, 95 },
    { 0, 0 },
    { 0x8ef4294a, 99 },
    { 0, 0 },
    { 0xddd4294c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc344294f, 46 },
    { 0x4964294f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81442953, 84 },
    { 0x66a42953, 90 },
    { 0xfbc42955, 93 },
    { 0x18442956, 33 },
    { 0, 0 },
    { 0x2f242958, 80 },
    { 0x80a42959, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc824295d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f842963, 23 },
    { 0x37542964, 76 },
    { 0xecc42965, 76 },
    { 0x4d842965, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf024296c, 23 },
    { 0, 0 },
    { 0xfb04296e, 90 },
    { 0, 0 },
    { 0x93b42970, 90 },
    { 0xd8542971, 55 },
    { 0xcc42970, 51 },
    { 0xb6d42973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff42979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df4297e, 61 },
    { 0x8344297e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa042982, 76 },
    { 0x34342983, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e242986, 68 },
    { 0x44f42987, 95 },
    { 0x1f442986, 43 },
    { 0x7c42989, 95 },
    { 0x5164298a, 98 },
    { 0x1354298b, 99 },
    { 0x67a4298c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95842990, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2844299a, 90 },
    { 0xc974299b, 55 },
    { 0, 0 },
    { 0x8044299d, 23 },
    { 0x2234299e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01429a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67429a4, 55 },
    { 0, 0 },
    { 0xd71429a6, 33 },
    { 0xbf0429a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa429af, 95 },
    { 0, 0 },
    { 0x5ae429b1, 55 },
    { 0xef5429b2, 100 },
    { 0, 0 },
    { 0x2f6429b4, 69 },
    { 0xc96429b4, 76 },
    { 0x7f3429b6, 23 },
    { 0x9e3429b7, 34 },
    { 0x900429b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde429be, 87 },
    { 0x162429be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef429c4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf429c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c429cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152429d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942429d6, 87 },
    { 0x74f429d7, 76 },
    { 0xe02429d6, 55 },
    { 0x3b4429d8, 76 },
    { 0x1b5429d8, 87 },
    { 0xe9b429db, 99 },
    { 0xf98429dc, 100 },
    { 0x227429d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x328429e0, 84 },
    { 0, 0 },
    { 0x95429e2, 33 },
    { 0x9ed429e2, 52 },
    { 0, 0 },
    { 0x5de429e5, 23 },
    { 0x49c429e6, 17 },
    { 0xc01429e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x710429ea, 66 },
    { 0x295429eb, 95 },
    { 0, 0 },
    { 0x42f429ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321429f1, 98 },
    { 0xb6a429f2, 33 },
    { 0xc3f429f2, 96 },
    { 0x98c429f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8429f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9429fc, 90 },
    { 0x15a429fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c442a02, 90 },
    { 0, 0 },
    { 0x68742a04, 77 },
    { 0x1c542a05, 98 },
    { 0xec342a06, 95 },
    { 0x6142a07, 55 },
    { 0xafa42a06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b42a0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38042a10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c642a13, 23 },
    { 0, 0 },
    { 0x83742a15, 99 },
    { 0x6a742a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46742a1b, 33 },
    { 0, 0 },
    { 0x8c042a1d, 78 },
    { 0xc1042a1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7442a21, 33 },
    { 0x11442a22, 55 },
    { 0, 0 },
    { 0xdc642a24, 61 },
    { 0x43e42a25, 77 },
    { 0, 0 },
    { 0xfcd42a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8942a2b, 65 },
    { 0x27242a2b, 95 },
    { 0x37342a2b, 43 },
    { 0x6b142a2d, 95 },
    { 0xf6242a2d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4342a33, 46 },
    { 0x38742a33, 61 },
    { 0x5b742a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a42a39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe242a3c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6742a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8242a45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f42a4b, 67 },
    { 0, 0 },
    { 0xd5042a4d, 76 },
    { 0x6bf42a4e, 93 },
    { 0x9e942a4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe742a52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b42a57, 95 },
    { 0, 0 },
    { 0xa9342a59, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f42a60, 95 },
    { 0x9242a61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b942a66, 66 },
    { 0, 0 },
    { 0xbbe42a68, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63042a6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c42a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0842a72, 43 },
    { 0xcad42a72, 33 },
    { 0x65642a74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac342a78, 98 },
    { 0xa0a42a79, 70 },
    { 0xdfb42a7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a42a7d, 88 },
    { 0x43f42a7d, 23 },
    { 0x30b42a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc42a82, 55 },
    { 0x6c742a83, 34 },
    { 0xa6942a84, 33 },
    { 0, 0 },
    { 0xd6942a86, 33 },
    { 0, 0 },
    { 0x82242a88, 95 },
    { 0, 0 },
    { 0xac242a8a, 61 },
    { 0x52942a8b, 55 },
    { 0x32b42a8b, 91 },
    { 0x5d642a8b, 85 },
    { 0xfd642a8e, 61 },
    { 0x82642a8f, 98 },
    { 0, 0 },
    { 0xcd942a91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f242a98, 36 },
    { 0, 0 },
    { 0xc6742a9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2842a9e, 66 },
    { 0, 0 },
    { 0x3fb42aa0, 98 },
    { 0x5642aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f42aa6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe242aa9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042aac, 61 },
    { 0x94542aad, 81 },
    { 0x61b42aac, 95 },
    { 0xab142aac, 43 },
    { 0x9c142aaf, 43 },
    { 0x44242ab0, 83 },
    { 0x67d42ab2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f542ab7, 91 },
    { 0x9242ab8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0542ac0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f442ac3, 98 },
    { 0, 0 },
    { 0xb5642ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e42aca, 66 },
    { 0, 0 },
    { 0xb6542acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e842acf, 98 },
    { 0, 0 },
    { 0xa0d42ad1, 17 },
    { 0x14442ad2, 67 },
    { 0xf7242ad2, 95 },
    { 0x9e442ad3, 90 },
    { 0xae542ad2, 61 },
    { 0, 0 },
    { 0x24842ad7, 95 },
    { 0xe0742ad7, 90 },
    { 0xaa042ad9, 33 },
    { 0x32b42ad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e942adf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc142ae6, 33 },
    { 0, 0 },
    { 0x5c942ae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f42aeb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042aee, 93 },
    { 0, 0 },
    { 0xa1142af0, 98 },
    { 0, 0 },
    { 0xfac42af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5542afb, 61 },
    { 0, 0 },
    { 0x84442afd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20642b01, 77 },
    { 0, 0 },
    { 0xa2542b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63442b06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66842b0b, 33 },
    { 0, 0 },
    { 0x16042b0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6842b13, 91 },
    { 0xc8542b14, 98 },
    { 0xaf042b13, 33 },
    { 0xf2d42b16, 76 },
    { 0xc1a42b13, 34 },
    { 0xb8442b13, 90 },
    { 0xe9d42b19, 100 },
    { 0x6f242b1a, 23 },
    { 0x7a42b1b, 34 },
    { 0xf5142b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48742b1f, 23 },
    { 0x49242b20, 33 },
    { 0xe6942b21, 65 },
    { 0x61842b22, 90 },
    { 0xc3342b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c42b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85742b2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5542b32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c42b39, 95 },
    { 0x9cf42b3a, 67 },
    { 0xf8542b3b, 76 },
    { 0x2742b3c, 33 },
    { 0xfa42b3b, 90 },
    { 0xafb42b3b, 33 },
    { 0xb1142b3c, 90 },
    { 0x5bb42b39, 61 },
    { 0xfa42b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86042b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d42b4f, 90 },
    { 0x7c442b50, 98 },
    { 0xb2742b51, 55 },
    { 0x5ca42b50, 34 },
    { 0xb4342b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f942b57, 85 },
    { 0, 0 },
    { 0xe5d42b59, 55 },
    { 0, 0 },
    { 0x74342b5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee42b5f, 90 },
    { 0x99442b5f, 93 },
    { 0, 0 },
    { 0xb8342b62, 98 },
    { 0x50242b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c342b67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21142b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e42b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042b76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a42b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc42b90, 23 },
    { 0, 0 },
    { 0x97642b92, 33 },
    { 0xf1842b93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15942b97, 65 },
    { 0, 0 },
    { 0x4ca42b99, 61 },
    { 0x9f342b9a, 98 },
    { 0xc2242b9b, 93 },
    { 0xbad42b9b, 93 },
    { 0, 0 },
    { 0x4ca42b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa242ba1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e842ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e42baa, 96 },
    { 0x9e542bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b42bbb, 90 },
    { 0x7d742bbc, 23 },
    { 0xb2142bbb, 55 },
    { 0x61142bbe, 36 },
    { 0xc2a42bbd, 76 },
    { 0x4a242bbd, 78 },
    { 0x4ab42bc1, 65 },
    { 0x8e542bc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5842bc9, 23 },
    { 0, 0 },
    { 0x62342bcb, 52 },
    { 0, 0 },
    { 0xbab42bcd, 91 },
    { 0x38b42bce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c642bd2, 33 },
    { 0x3b042bd2, 61 },
    { 0xde842bd4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e42bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a542bde, 34 },
    { 0x6b742bdf, 43 },
    { 0x72942be0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37642be5, 77 },
    { 0, 0 },
    { 0x62a42be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b42bea, 90 },
    { 0x5a142beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f42bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1042bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9842bff, 97 },
    { 0x43542c00, 90 },
    { 0x9e342bff, 95 },
    { 0x48942c01, 68 },
    { 0xe3b42c03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd542c06, 34 },
    { 0x1f342c07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e42c0d, 84 },
    { 0x3fa42c0e, 17 },
    { 0x70542c0f, 91 },
    { 0xaaf42c0e, 33 },
    { 0x64a42c11, 76 },
    { 0x9b542c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda242c1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82842c20, 46 },
    { 0x59e42c20, 61 },
    { 0xc4642c22, 61 },
    { 0x7f842c23, 43 },
    { 0, 0 },
    { 0x27842c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a342c2c, 90 },
    { 0xbb142c2c, 93 },
    { 0x91942c2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da42c32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19542c36, 43 },
    { 0x36242c37, 34 },
    { 0xd1442c37, 55 },
    { 0xecb42c37, 61 },
    { 0xc8142c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78642c45, 98 },
    { 0xa9942c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f842c4e, 33 },
    { 0, 0 },
    { 0x88d42c50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e42c53, 43 },
    { 0xd3542c54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x85742c57, 55 },
    { 0x47442c58, 61 },
    { 0x8fe42c59, 100 },
    { 0xb0642c5a, 98 },
    { 0x1e542c5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26042c5e, 33 },
    { 0x6d342c5f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xed342c62, 98 },
    { 0, 0 },
    { 0xaf842c64, 90 },
    { 0x72542c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a42c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f142c6e, 76 },
    { 0, 0 },
    { 0xdb842c70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52642c77, 98 },
    { 0xea042c78, 40 },
    { 0x3de42c79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33542c80, 43 },
    { 0x28c42c81, 91 },
    { 0x68742c82, 99 },
    { 0x2a742c82, 77 },
    { 0, 0 },
    { 0x9c242c85, 95 },
    { 0x2ce42c86, 33 },
    { 0x7ba42c86, 17 },
    { 0xed142c87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe42c8d, 81 },
    { 0xcb342c8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe242c92, 98 },
    { 0, 0 },
    { 0xddf42c94, 33 },
    { 0, 0 },
    { 0x67042c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca42c9d, 68 },
    { 0xf8c42c9d, 34 },
    { 0xfb442c9f, 61 },
    { 0xd7442ca0, 90 },
    { 0, 0 },
    { 0x97a42ca2, 76 },
    { 0xb5e42ca3, 90 },
    { 0xb0542ca4, 61 },
    { 0xb542ca3, 43 },
    { 0xd6442ca6, 76 },
    { 0x99042ca7, 33 },
    { 0xa4a42ca8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a042cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b42cb1, 61 },
    { 0x8e42cb2, 66 },
    { 0x64c42cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa442cb6, 64 },
    { 0x5cf42cb7, 34 },
    { 0xfe342cb8, 95 },
    { 0x68b42cb7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9242cbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a42cc8, 68 },
    { 0, 0 },
    { 0x9d042cca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f842cd1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e42cd6, 61 },
    { 0x2a242cd7, 23 },
    { 0x11f42cd8, 23 },
    { 0x2642cd8, 61 },
    { 0x8f42cda, 90 },
    { 0x3aa42cdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1342cdf, 95 },
    { 0, 0 },
    { 0x1ab42ce1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x29242ce4, 23 },
    { 0x27d42ce5, 23 },
    { 0xeb842ce5, 91 },
    { 0x4542ce7, 91 },
    { 0x1c642ce8, 23 },
    { 0xd6542ce9, 95 },
    { 0x1b042cea, 98 },
    { 0x24e42ceb, 90 },
    { 0x39c42ce8, 34 },
    { 0xeea42ce7, 61 },
    { 0x3ee42cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc842cf3, 95 },
    { 0xe2c42cf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0542cfd, 69 },
    { 0xdfa42cfe, 90 },
    { 0x48942cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c142d03, 90 },
    { 0, 0 },
    { 0x2f642d05, 23 },
    { 0, 0 },
    { 0x66342d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85542d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26142d0d, 56 },
    { 0x43742d0d, 90 },
    { 0, 0 },
    { 0xfdd42d10, 33 },
    { 0x2b942d11, 76 },
    { 0xaae42d12, 55 },
    { 0, 0 },
    { 0x76b42d14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c42d1b, 76 },
    { 0x14742d1c, 76 },
    { 0xc9542d1d, 90 },
    { 0x5b642d1c, 46 },
    { 0x8c942d1f, 99 },
    { 0x84642d20, 76 },
    { 0x70c42d21, 23 },
    { 0, 0 },
    { 0x20342d23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68242d29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa42d2d, 76 },
    { 0x66b42d2d, 98 },
    { 0x57c42d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a942d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5742d37, 43 },
    { 0xb5d42d38, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a842d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e42d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e42d4d, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d42d50, 90 },
    { 0x4942d51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e42d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab942d5f, 35 },
    { 0, 0 },
    { 0xbb942d61, 76 },
    { 0, 0 },
    { 0x9f942d63, 100 },
    { 0x40e42d64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d42d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9642d6f, 67 },
    { 0x11542d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d42d76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d42d79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff42d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58542d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80442d8a, 66 },
    { 0xd4d42d8a, 61 },
    { 0x3a142d8c, 93 },
    { 0x96242d8a, 61 },
    { 0, 0 },
    { 0xb0242d8f, 95 },
    { 0xd3142d90, 101 },
    { 0x38e42d91, 52 },
    { 0, 0 },
    { 0x7f42d93, 17 },
    { 0xc442d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c42d9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7242d9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef42da1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642da6, 85 },
    { 0xbcd42da7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32042db0, 77 },
    { 0, 0 },
    { 0x81642db2, 99 },
    { 0xb3b42db3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51542db8, 90 },
    { 0x86742db8, 90 },
    { 0xfaa42dba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d42dbd, 98 },
    { 0x5a942dbe, 61 },
    { 0xb7b42dbf, 33 },
    { 0x79742dc0, 23 },
    { 0, 0 },
    { 0x95d42dc2, 78 },
    { 0x12c42dc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd142dca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac42dce, 97 },
    { 0xee342dce, 33 },
    { 0x34142dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe42dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e42dd9, 33 },
    { 0x25942dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5942de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb342de9, 61 },
    { 0, 0 },
    { 0xb9842deb, 76 },
    { 0x1e42deb, 90 },
    { 0x27642ded, 90 },
    { 0xbcf42dee, 23 },
    { 0x13342def, 90 },
    { 0x13c42def, 85 },
    { 0x32942def, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac42df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c42df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb42dfe, 77 },
    { 0xb1d42dff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59942e06, 100 },
    { 0x1642e07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2242e0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed42e0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a42e14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94642e17, 46 },
    { 0x83a42e18, 43 },
    { 0xbc442e19, 67 },
    { 0x1e942e19, 23 },
    { 0x53542e1b, 95 },
    { 0x69442e1b, 34 },
    { 0xd42e1c, 61 },
    { 0x3c642e1e, 33 },
    { 0x1ea42e1e, 76 },
    { 0xdb642e1e, 84 },
    { 0xce542e21, 33 },
    { 0x7242e1b, 55 },
    { 0x8e442e23, 98 },
    { 0xfd342e20, 56 },
    { 0xea442e25, 55 },
    { 0x34542e1c, 43 },
    { 0, 0 },
    { 0x22e42e28, 23 },
    { 0x18d42e29, 95 },
    { 0xa6942e2a, 61 },
    { 0xdc942e2a, 95 },
    { 0x45042e2c, 90 },
    { 0, 0 },
    { 0x8e42e2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x25542e31, 33 },
    { 0x16742e32, 95 },
    { 0x7b942e32, 93 },
    { 0x7f342e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab42e37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2842e3a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f142e40, 90 },
    { 0xdc542e41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2842e44, 90 },
    { 0, 0 },
    { 0x75642e46, 78 },
    { 0, 0 },
    { 0xc1c42e48, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43642e4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21042e4f, 98 },
    { 0, 0 },
    { 0x73642e51, 91 },
    { 0xa2142e51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b42e55, 91 },
    { 0x91e42e56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5142e59, 90 },
    { 0x1242e59, 65 },
    { 0x38f42e5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b42e5e, 95 },
    { 0x3b742e5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242e63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39042e66, 99 },
    { 0x64e42e66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e42e6d, 17 },
    { 0x5b442e6e, 81 },
    { 0x2cb42e6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17842e76, 91 },
    { 0, 0 },
    { 0x20f42e78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c42e7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca242e81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de42e86, 76 },
    { 0, 0 },
    { 0xd2942e88, 55 },
    { 0xd0242e88, 76 },
    { 0x6dd42e8a, 61 },
    { 0xf1542e89, 76 },
    { 0x7c042e8c, 91 },
    { 0x55942e89, 17 },
    { 0xdc42e8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e342e96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1842e9b, 95 },
    { 0x52142e9c, 84 },
    { 0x2a142e9c, 78 },
    { 0xb7b42e9e, 64 },
    { 0xdd442e9f, 65 },
    { 0xa9442ea0, 95 },
    { 0, 0 },
    { 0x4e242ea2, 61 },
    { 0xd3c42ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3742eac, 52 },
    { 0x41542eac, 53 },
    { 0x4c542eae, 61 },
    { 0x8b342eae, 23 },
    { 0x7bf42eac, 85 },
    { 0, 0 },
    { 0x7bc42eb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75142eba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a42ebe, 76 },
    { 0xa1842ebf, 23 },
    { 0x98d42ec0, 98 },
    { 0x93542ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57042ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16442ec8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a442ed0, 61 },
    { 0, 0 },
    { 0x79942ed2, 23 },
    { 0x15542ed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97742eda, 76 },
    { 0xd2242edb, 90 },
    { 0xd7842edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd242edf, 87 },
    { 0, 0 },
    { 0x8a842ee1, 46 },
    { 0, 0 },
    { 0x6ae42ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84942ee6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c42eea, 98 },
    { 0x9af42eea, 98 },
    { 0x50542eec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa742ef3, 81 },
    { 0x13442ef3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58542efa, 95 },
    { 0x80c42efa, 90 },
    { 0x8e342efc, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43442f01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45842f08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63542f0c, 98 },
    { 0, 0 },
    { 0x4c242f0e, 33 },
    { 0xd5842f0f, 88 },
    { 0x54e42f10, 61 },
    { 0xd4a42f11, 23 },
    { 0x32c42f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd342f16, 66 },
    { 0, 0 },
    { 0x5742f18, 90 },
    { 0x58842f18, 33 },
    { 0x33142f19, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f242f1f, 33 },
    { 0xbe442f20, 90 },
    { 0x542f1f, 55 },
    { 0, 0 },
    { 0xb9542f23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f42f26, 93 },
    { 0, 0 },
    { 0xb2d42f28, 90 },
    { 0xe3242f28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68342f2d, 23 },
    { 0, 0 },
    { 0x51242f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57942f35, 61 },
    { 0x13c42f35, 31 },
    { 0xd642f37, 33 },
    { 0x6ec42f38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b42f3c, 23 },
    { 0xbca42f3d, 100 },
    { 0x20e42f3c, 98 },
    { 0, 0 },
    { 0xe4542f40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb42f43, 95 },
    { 0x76442f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff42f49, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e142f4e, 52 },
    { 0x9e42f4f, 33 },
    { 0x20a42f4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14142f56, 95 },
    { 0x1be42f57, 90 },
    { 0x5cc42f57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f742f60, 75 },
    { 0x9c842f61, 67 },
    { 0x85a42f62, 77 },
    { 0x34e42f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93742f67, 23 },
    { 0x72842f68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c42f6f, 33 },
    { 0xd2642f70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f42f77, 76 },
    { 0, 0 },
    { 0x26b42f79, 88 },
    { 0, 0 },
    { 0xbf842f7b, 90 },
    { 0xffd42f7c, 95 },
    { 0xc6442f7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a242f80, 90 },
    { 0xb9142f80, 88 },
    { 0x93a42f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3942f87, 82 },
    { 0x6d942f87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df42f91, 81 },
    { 0x6fc42f91, 58 },
    { 0x72642f93, 65 },
    { 0, 0 },
    { 0xa2042f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1042f9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44842f9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2142fa6, 33 },
    { 0x87442fa7, 90 },
    { 0xcd42fa7, 85 },
    { 0, 0 },
    { 0xec42faa, 43 },
    { 0xf1742fab, 33 },
    { 0x8e942fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d42faf, 96 },
    { 0x5fd42faf, 33 },
    { 0xf9e42fb1, 23 },
    { 0xc9042fb2, 46 },
    { 0xb3a42fb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e42fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf42fbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3842fbe, 90 },
    { 0xa3f42fbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f42fc2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f442fc5, 23 },
    { 0xa6b42fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e42fd0, 92 },
    { 0x14242fd1, 95 },
    { 0x55042fd2, 78 },
    { 0x43e42fd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59742fd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66942fda, 95 },
    { 0, 0 },
    { 0x3bf42fdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd542fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd42fe5, 76 },
    { 0, 0 },
    { 0xa5a42fe7, 90 },
    { 0, 0 },
    { 0x2f142fe9, 76 },
    { 0, 0 },
    { 0xa4d42feb, 46 },
    { 0, 0 },
    { 0xd1b42fed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e42ff2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11442ff9, 33 },
    { 0x20142ffa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90042fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c443002, 76 },
    { 0x1143002, 33 },
    { 0, 0 },
    { 0xca943005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd143011, 78 },
    { 0x6e843012, 23 },
    { 0x56d43012, 90 },
    { 0x76843014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa43019, 90 },
    { 0xa364301a, 90 },
    { 0x6464301a, 43 },
    { 0, 0 },
    { 0x1ab4301d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c043021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb43028, 55 },
    { 0x20643028, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554302d, 61 },
    { 0x8ac4302d, 90 },
    { 0x7874302f, 95 },
    { 0x4d243030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d43037, 66 },
    { 0, 0 },
    { 0x99343039, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d4303d, 55 },
    { 0, 0 },
    { 0xff04303f, 61 },
    { 0, 0 },
    { 0x8b743041, 90 },
    { 0, 0 },
    { 0xa7a43043, 91 },
    { 0x59a43044, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85143048, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d4304e, 61 },
    { 0xf904304e, 23 },
    { 0, 0 },
    { 0xe3f43051, 67 },
    { 0xd2343051, 42 },
    { 0x48143053, 90 },
    { 0x48343051, 33 },
    { 0x71e43055, 68 },
    { 0x55c43054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b143059, 90 },
    { 0x75d43059, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50943063, 55 },
    { 0x44d43063, 80 },
    { 0x61f43065, 61 },
    { 0x43a43064, 90 },
    { 0xcde43067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b4306a, 43 },
    { 0x9bf4306a, 33 },
    { 0, 0 },
    { 0xbe44306d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77243070, 98 },
    { 0, 0 },
    { 0xcbd43072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e43078, 23 },
    { 0xf1443079, 55 },
    { 0x1784307a, 95 },
    { 0, 0 },
    { 0x6f04307c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43080, 91 },
    { 0xaa343081, 95 },
    { 0xcf343082, 95 },
    { 0x12943083, 43 },
    { 0, 0 },
    { 0xbb043085, 98 },
    { 0, 0 },
    { 0x3743087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf74308e, 76 },
    { 0xc024308f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e43094, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e43097, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc4309a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df430a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43430aa, 87 },
    { 0x7c2430aa, 55 },
    { 0, 0 },
    { 0x211430ad, 23 },
    { 0x727430ae, 61 },
    { 0, 0 },
    { 0x9ab430b0, 76 },
    { 0x53a430b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x669430b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19430b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784430bb, 69 },
    { 0x496430bc, 40 },
    { 0xd88430bd, 61 },
    { 0x3b430be, 91 },
    { 0xcf8430bc, 34 },
    { 0, 0 },
    { 0x2d5430c1, 81 },
    { 0x1b6430c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5430c7, 90 },
    { 0x18b430c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687430cc, 52 },
    { 0x50b430cc, 61 },
    { 0, 0 },
    { 0xa51430cf, 88 },
    { 0xced430d0, 23 },
    { 0x8ee430d1, 76 },
    { 0x2cf430cf, 90 },
    { 0xcc2430d2, 55 },
    { 0x20f430d4, 33 },
    { 0x424430d5, 33 },
    { 0x185430d5, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2430dc, 98 },
    { 0, 0 },
    { 0x20e430de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc430e6, 67 },
    { 0xe36430e7, 33 },
    { 0, 0 },
    { 0x7c5430e9, 61 },
    { 0x1e0430ea, 61 },
    { 0xcf5430eb, 95 },
    { 0x788430ec, 52 },
    { 0, 0 },
    { 0x257430ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd430f1, 53 },
    { 0, 0 },
    { 0xcb4430f3, 93 },
    { 0, 0 },
    { 0x1aa430f5, 96 },
    { 0, 0 },
    { 0xed430f7, 84 },
    { 0xa78430f8, 66 },
    { 0, 0 },
    { 0xa99430fa, 95 },
    { 0x26e430fb, 98 },
    { 0x8b8430fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c343100, 43 },
    { 0x51f43101, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb43104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee43107, 34 },
    { 0xe6743108, 43 },
    { 0xca443107, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c4310e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf943113, 33 },
    { 0x6ea43113, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd443117, 76 },
    { 0, 0 },
    { 0xe343119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6974311d, 52 },
    { 0x8794311d, 34 },
    { 0xa0a4311e, 90 },
    { 0x7a543120, 98 },
    { 0xc6c43120, 61 },
    { 0x64a43120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d43127, 95 },
    { 0x72b43128, 81 },
    { 0x23343129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e74312d, 98 },
    { 0, 0 },
    { 0x16a4312f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0743132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb543137, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4313b, 95 },
    { 0, 0 },
    { 0xb3a4313d, 33 },
    { 0xfaf4313e, 61 },
    { 0, 0 },
    { 0x53743140, 80 },
    { 0, 0 },
    { 0x7cf43142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9843145, 90 },
    { 0x85343146, 90 },
    { 0x7e043147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64314c, 33 },
    { 0, 0 },
    { 0xfff4314e, 95 },
    { 0x1f64314e, 33 },
    { 0x6c54314f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c43158, 84 },
    { 0, 0 },
    { 0x7034315a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4315d, 46 },
    { 0, 0 },
    { 0xae64315f, 34 },
    { 0x8aa43160, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc843163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab43167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a4316b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3343171, 55 },
    { 0x1ff43172, 23 },
    { 0xf9b43173, 61 },
    { 0xda243174, 83 },
    { 0xb1d43175, 76 },
    { 0x69443172, 95 },
    { 0xb3443175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e4317a, 69 },
    { 0x1f04317b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe344317f, 91 },
    { 0, 0 },
    { 0x3d243181, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e43186, 61 },
    { 0, 0 },
    { 0xb3943188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5864318b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90643196, 90 },
    { 0xfc943197, 90 },
    { 0xef043198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5964319b, 61 },
    { 0x8094319c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b431a2, 91 },
    { 0xff0431a3, 76 },
    { 0, 0 },
    { 0x7bf431a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2431ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c431b2, 23 },
    { 0xd07431b2, 96 },
    { 0x240431b4, 93 },
    { 0x908431b5, 33 },
    { 0xf54431b6, 90 },
    { 0x8c3431b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c431bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x775431be, 95 },
    { 0x35431bf, 33 },
    { 0x49a431c0, 61 },
    { 0, 0 },
    { 0x321431c2, 23 },
    { 0x6e5431c2, 84 },
    { 0, 0 },
    { 0xf0a431c5, 85 },
    { 0x634431c6, 46 },
    { 0xba9431c7, 96 },
    { 0, 0 },
    { 0x784431c9, 88 },
    { 0xcb2431c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330431ce, 34 },
    { 0x748431cf, 99 },
    { 0xf69431cf, 34 },
    { 0x301431cf, 95 },
    { 0x252431d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e431d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f431dc, 43 },
    { 0x318431dd, 90 },
    { 0xa6a431dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b431e4, 23 },
    { 0x7f431e5, 61 },
    { 0xbd5431e6, 93 },
    { 0, 0 },
    { 0x927431e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721431ec, 61 },
    { 0xefe431ec, 34 },
    { 0, 0 },
    { 0xbaf431ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8431fa, 93 },
    { 0xa41431fa, 99 },
    { 0xb84431fc, 33 },
    { 0xdd8431fc, 66 },
    { 0, 0 },
    { 0x978431ff, 95 },
    { 0x83543200, 33 },
    { 0x26f43201, 61 },
    { 0, 0 },
    { 0x9e143203, 95 },
    { 0, 0 },
    { 0x9bb43205, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48143209, 33 },
    { 0x7924320a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d74320e, 90 },
    { 0x1984320e, 85 },
    { 0xde943210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61643216, 55 },
    { 0x7a343217, 90 },
    { 0x62a43218, 61 },
    { 0x65043219, 55 },
    { 0, 0 },
    { 0xf9c4321b, 98 },
    { 0, 0 },
    { 0xfcf4321d, 96 },
    { 0x4fc4321d, 34 },
    { 0x5774321f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9443223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac143228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474322d, 61 },
    { 0x89e4322d, 95 },
    { 0, 0 },
    { 0x92443230, 100 },
    { 0xdd843231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb243239, 90 },
    { 0x8f44323a, 23 },
    { 0x6954323a, 34 },
    { 0, 0 },
    { 0xfcc4323d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc43240, 98 },
    { 0, 0 },
    { 0xfaa43242, 61 },
    { 0, 0 },
    { 0xad143244, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a64324f, 52 },
    { 0, 0 },
    { 0x28443251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b43257, 66 },
    { 0, 0 },
    { 0xc6c43259, 76 },
    { 0, 0 },
    { 0x6b74325b, 11 },
    { 0, 0 },
    { 0xd6a4325d, 33 },
    { 0x1d84325e, 98 },
    { 0x1f34325f, 98 },
    { 0xcbf43260, 33 },
    { 0xb9143261, 99 },
    { 0xec343260, 98 },
    { 0xe1e43263, 78 },
    { 0, 0 },
    { 0x2dd43265, 23 },
    { 0xedc43266, 55 },
    { 0x80843267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b4326f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f43273, 55 },
    { 0x65143274, 80 },
    { 0, 0 },
    { 0x52843276, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f443280, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49043286, 34 },
    { 0xb9043286, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa4328a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9094328d, 90 },
    { 0, 0 },
    { 0xb914328f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41743292, 43 },
    { 0, 0 },
    { 0x81743294, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d24329e, 76 },
    { 0x2ba4329f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13432a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1432a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f432ab, 98 },
    { 0, 0 },
    { 0x6e6432ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x140432b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x973432b3, 90 },
    { 0, 0 },
    { 0x9e1432b5, 93 },
    { 0, 0 },
    { 0x87432b7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca432bd, 77 },
    { 0x392432be, 55 },
    { 0x287432be, 33 },
    { 0, 0 },
    { 0x930432c1, 61 },
    { 0x261432c2, 17 },
    { 0, 0 },
    { 0x7c9432c4, 76 },
    { 0x840432c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23432c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a432cf, 75 },
    { 0x5b2432d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b432d3, 66 },
    { 0, 0 },
    { 0xa5e432d5, 33 },
    { 0x900432d5, 43 },
    { 0x5c432d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff432df, 95 },
    { 0x867432e0, 61 },
    { 0, 0 },
    { 0xa85432e2, 61 },
    { 0, 0 },
    { 0x93f432e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f432e7, 35 },
    { 0xdd2432e8, 55 },
    { 0x819432e9, 34 },
    { 0x337432ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11432ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121432f1, 95 },
    { 0x2d7432f1, 43 },
    { 0xf5a432f3, 61 },
    { 0x4f1432f3, 85 },
    { 0xa1432f5, 43 },
    { 0xcf5432f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632432f9, 95 },
    { 0, 0 },
    { 0x664432fb, 23 },
    { 0x2f3432fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e43304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32043307, 70 },
    { 0, 0 },
    { 0x5e743309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc264330d, 76 },
    { 0xcde4330d, 55 },
    { 0, 0 },
    { 0xec943310, 76 },
    { 0xb2543310, 43 },
    { 0, 0 },
    { 0x4e243313, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be43318, 78 },
    { 0x72943319, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc84331c, 90 },
    { 0, 0 },
    { 0xce84331e, 90 },
    { 0, 0 },
    { 0x46343320, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d843325, 52 },
    { 0, 0 },
    { 0xc0743327, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f4332c, 61 },
    { 0, 0 },
    { 0xdb74332e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69943337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e24333c, 93 },
    { 0x8cd4333d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64343342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc043353, 98 },
    { 0, 0 },
    { 0xdea43355, 43 },
    { 0xb5143356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf4335a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f4335d, 65 },
    { 0, 0 },
    { 0xdc34335f, 98 },
    { 0x4d243360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e443377, 34 },
    { 0x5ff43378, 90 },
    { 0, 0 },
    { 0xfb44337a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97943380, 33 },
    { 0xdfb43380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e24338a, 61 },
    { 0, 0 },
    { 0x1424338c, 46 },
    { 0x3e74338d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7843393, 23 },
    { 0xb6c43394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134339d, 61 },
    { 0xd964339e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb433a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d433a5, 34 },
    { 0, 0 },
    { 0x29a433a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6433ad, 90 },
    { 0x427433ae, 98 },
    { 0x27d433ad, 23 },
    { 0xce0433b0, 61 },
    { 0x8f3433b1, 61 },
    { 0x718433b2, 90 },
    { 0, 0 },
    { 0x263433b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x231433b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b433bd, 80 },
    { 0, 0 },
    { 0xa37433bf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6433c3, 61 },
    { 0x810433c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9433c8, 33 },
    { 0xbff433c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118433cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b433d0, 98 },
    { 0x9ee433d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811433d5, 75 },
    { 0xf6c433d6, 46 },
    { 0, 0 },
    { 0xc10433d8, 61 },
    { 0x66b433d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd433dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e433e2, 34 },
    { 0xa43433e2, 100 },
    { 0x8d6433e4, 88 },
    { 0x437433e2, 98 },
    { 0x61c433e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1433eb, 23 },
    { 0xfff433ec, 95 },
    { 0x97d433ec, 55 },
    { 0x94f433ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8433f1, 23 },
    { 0, 0 },
    { 0xb24433f3, 55 },
    { 0xdc6433f4, 61 },
    { 0, 0 },
    { 0x74433f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x356433f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x737433fc, 23 },
    { 0xd28433fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd43402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7843405, 61 },
    { 0x84e43406, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f43409, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c143412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9843417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e43423, 75 },
    { 0x1bb43423, 23 },
    { 0x1de43423, 90 },
    { 0, 0 },
    { 0xc1b43427, 95 },
    { 0xcaf43428, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf584342e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba43431, 95 },
    { 0, 0 },
    { 0xdc443433, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5364343c, 33 },
    { 0xc924343d, 90 },
    { 0x344343e, 81 },
    { 0x1684343c, 33 },
    { 0x2624343f, 23 },
    { 0, 0 },
    { 0xbbf43442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c043446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64344b, 93 },
    { 0xae14344c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec43453, 90 },
    { 0x3d343454, 93 },
    { 0x1f843455, 91 },
    { 0x80943453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2343459, 61 },
    { 0, 0 },
    { 0x2994345b, 90 },
    { 0, 0 },
    { 0xd984345d, 95 },
    { 0x3214345d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e43468, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe324346f, 33 },
    { 0x7b34346f, 95 },
    { 0x8e143471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf043477, 75 },
    { 0xf4f43478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc14347b, 34 },
    { 0, 0 },
    { 0x2d24347d, 33 },
    { 0x9d04347e, 55 },
    { 0xda64347f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe43482, 23 },
    { 0xf5943483, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d943486, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74443489, 33 },
    { 0, 0 },
    { 0x2624348b, 90 },
    { 0x1544348c, 90 },
    { 0x274348c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64843492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3094349f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e434a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120434a6, 61 },
    { 0x235434a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x581434aa, 90 },
    { 0xc4d434ab, 33 },
    { 0, 0 },
    { 0xf90434ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61434b2, 100 },
    { 0x5c7434b2, 55 },
    { 0, 0 },
    { 0xdc3434b5, 98 },
    { 0x9fd434b6, 90 },
    { 0xe83434b7, 95 },
    { 0, 0 },
    { 0xc2d434b9, 33 },
    { 0x473434b9, 78 },
    { 0, 0 },
    { 0xe67434bc, 34 },
    { 0x3fc434bd, 87 },
    { 0x8c9434bc, 100 },
    { 0xa43434bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770434ce, 61 },
    { 0x7c4434ce, 43 },
    { 0x1b6434d0, 34 },
    { 0x464434d1, 90 },
    { 0xeae434cf, 98 },
    { 0, 0 },
    { 0xd6d434d4, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a434d7, 99 },
    { 0x599434d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805434dc, 90 },
    { 0xa2a434dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182434e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13434f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37434fe, 61 },
    { 0x87f434fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63143503, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea43508, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4350b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc43513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744351f, 90 },
    { 0x2dd43520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48643529, 78 },
    { 0xc474352a, 93 },
    { 0x3dc4352b, 33 },
    { 0xf7b4352b, 98 },
    { 0xdaa4352b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e543530, 98 },
    { 0xf6343530, 33 },
    { 0x9dc43532, 85 },
    { 0x2be43532, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc343536, 33 },
    { 0xa1b43537, 61 },
    { 0xfef43536, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39943542, 33 },
    { 0x34a43542, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc43546, 76 },
    { 0, 0 },
    { 0x8c543548, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6404354f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xba343552, 43 },
    { 0x97043553, 90 },
    { 0, 0 },
    { 0x45b43555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534355d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc443562, 17 },
    { 0, 0 },
    { 0x4c343564, 84 },
    { 0xf5343565, 23 },
    { 0xda143566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd143577, 33 },
    { 0x5dd43578, 33 },
    { 0x73543578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b4357c, 91 },
    { 0xbfe4357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b43582, 98 },
    { 0x41b43583, 33 },
    { 0, 0 },
    { 0x8a343585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8443589, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604358f, 90 },
    { 0xc6c43590, 23 },
    { 0xeaf43591, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab943595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf4359c, 99 },
    { 0, 0 },
    { 0x4ec4359e, 34 },
    { 0xe014359f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421435a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8435ab, 85 },
    { 0x648435ac, 61 },
    { 0x174435ac, 95 },
    { 0x4db435ae, 80 },
    { 0x874435af, 91 },
    { 0, 0 },
    { 0xf52435b1, 23 },
    { 0x500435b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d435b5, 46 },
    { 0, 0 },
    { 0x63a435b7, 77 },
    { 0x4dd435b8, 33 },
    { 0, 0 },
    { 0xef5435ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06435bf, 23 },
    { 0x982435c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5435c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6435c6, 61 },
    { 0, 0 },
    { 0xbf2435c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4435cd, 95 },
    { 0, 0 },
    { 0x188435cf, 98 },
    { 0xc4a435d0, 33 },
    { 0, 0 },
    { 0xe8c435d2, 61 },
    { 0xd74435d3, 84 },
    { 0x74b435d4, 95 },
    { 0x104435d4, 98 },
    { 0x204435d6, 33 },
    { 0, 0 },
    { 0x9ce435d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e435db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99435de, 96 },
    { 0xc96435de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375435e3, 90 },
    { 0, 0 },
    { 0x7a3435e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6435ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6435f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435f4, 46 },
    { 0, 0 },
    { 0x108435f6, 98 },
    { 0, 0 },
    { 0xfa2435f8, 93 },
    { 0xc61435f9, 85 },
    { 0, 0 },
    { 0xed1435fb, 17 },
    { 0x233435fc, 33 },
    { 0, 0 },
    { 0xf5b435fe, 23 },
    { 0x7b7435fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e43602, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c43606, 67 },
    { 0x29443607, 23 },
    { 0x36243608, 90 },
    { 0, 0 },
    { 0xf014360a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe374360e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf374361c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214361f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f43622, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e243628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f4362c, 23 },
    { 0x19b4362c, 76 },
    { 0x3c84362d, 98 },
    { 0, 0 },
    { 0xd4643630, 55 },
    { 0, 0 },
    { 0x50443632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee943636, 88 },
    { 0x8ca43636, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d14363c, 90 },
    { 0xf5d4363d, 95 },
    { 0xb7e4363e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24743644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1443647, 55 },
    { 0xad643648, 76 },
    { 0x22243649, 47 },
    { 0xa934364a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5143650, 34 },
    { 0xba743651, 34 },
    { 0xeb043652, 23 },
    { 0xcd643653, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243656, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd43659, 76 },
    { 0x4bf4365a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a4365e, 33 },
    { 0, 0 },
    { 0x18343660, 55 },
    { 0, 0 },
    { 0x66a43662, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643666, 99 },
    { 0xa843667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca4366a, 33 },
    { 0, 0 },
    { 0xe534366c, 61 },
    { 0xdae4366d, 87 },
    { 0xcae4366d, 61 },
    { 0x3944366c, 90 },
    { 0xa343670, 90 },
    { 0xb8143670, 43 },
    { 0x3f943672, 76 },
    { 0x76d43673, 93 },
    { 0x92d43673, 84 },
    { 0xa5043673, 23 },
    { 0, 0 },
    { 0x83143677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b4367b, 33 },
    { 0x55b4367b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a94367f, 99 },
    { 0xbb4367f, 61 },
    { 0xd4943681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5143687, 87 },
    { 0x78443688, 61 },
    { 0xf7143689, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37643691, 61 },
    { 0, 0 },
    { 0x72243693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab843696, 90 },
    { 0x33643696, 43 },
    { 0x62d43698, 34 },
    { 0, 0 },
    { 0xbae4369a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240436a1, 61 },
    { 0x492436a2, 61 },
    { 0x332436a3, 23 },
    { 0x498436a3, 33 },
    { 0, 0 },
    { 0x4e436a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1436a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb436ad, 95 },
    { 0, 0 },
    { 0x18d436af, 23 },
    { 0, 0 },
    { 0xe2436b1, 61 },
    { 0x7a9436b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5436b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e436b9, 76 },
    { 0xa1436b9, 95 },
    { 0, 0 },
    { 0x351436bc, 98 },
    { 0, 0 },
    { 0x28e436be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd436c6, 81 },
    { 0xb5b436c7, 76 },
    { 0x49f436c6, 90 },
    { 0x86436c9, 61 },
    { 0, 0 },
    { 0x288436cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a436d1, 33 },
    { 0xdbb436d1, 34 },
    { 0, 0 },
    { 0xe42436d4, 90 },
    { 0xb32436d5, 66 },
    { 0, 0 },
    { 0x10d436d7, 55 },
    { 0xb8b436d8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2436db, 61 },
    { 0x17b436dc, 85 },
    { 0x61d436dc, 33 },
    { 0, 0 },
    { 0x5b8436df, 95 },
    { 0x8c7436e0, 90 },
    { 0x2b2436e1, 33 },
    { 0x946436e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a436e6, 87 },
    { 0x4ca436e6, 90 },
    { 0xc63436e6, 43 },
    { 0x352436e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e436f4, 77 },
    { 0, 0 },
    { 0xc9c436f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31436f9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b436ff, 61 },
    { 0x834436ff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c43705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b64370a, 95 },
    { 0x27c4370b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564370f, 78 },
    { 0x7d143710, 95 },
    { 0, 0 },
    { 0x14643712, 98 },
    { 0xf6743712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d43717, 90 },
    { 0xb9d43718, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d4371d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b43722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b43728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91943735, 23 },
    { 0xc7543736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10043741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f43749, 61 },
    { 0, 0 },
    { 0x7424374b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c43750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73443754, 61 },
    { 0, 0 },
    { 0xe9443756, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf4375a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54375d, 77 },
    { 0x46c4375e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d43762, 95 },
    { 0, 0 },
    { 0x1ac43764, 98 },
    { 0x8c943764, 61 },
    { 0xeef43766, 33 },
    { 0x7e243765, 90 },
    { 0, 0 },
    { 0xf8943769, 65 },
    { 0, 0 },
    { 0x6014376b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c64376f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17043773, 88 },
    { 0, 0 },
    { 0x8fd43775, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4377a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243782, 95 },
    { 0, 0 },
    { 0xdc943784, 95 },
    { 0xda543784, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6544378f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d43792, 98 },
    { 0x31d43793, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d43797, 33 },
    { 0x1d143798, 95 },
    { 0x26643798, 23 },
    { 0, 0 },
    { 0x2f84379b, 98 },
    { 0xa094379c, 97 },
    { 0, 0 },
    { 0x7504379e, 46 },
    { 0x6aa4379e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9437a6, 90 },
    { 0xb3437a7, 52 },
    { 0x2da437a8, 93 },
    { 0x42d437a8, 98 },
    { 0, 0 },
    { 0x6f8437ab, 90 },
    { 0, 0 },
    { 0x8db437ad, 61 },
    { 0xea3437ae, 90 },
    { 0, 0 },
    { 0xbdd437b0, 90 },
    { 0x860437b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89437b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45437b8, 55 },
    { 0x20437b9, 90 },
    { 0x955437ba, 23 },
    { 0x8d4437bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a437be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46437c6, 90 },
    { 0, 0 },
    { 0xc70437c8, 93 },
    { 0x887437c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe437cc, 76 },
    { 0, 0 },
    { 0xb96437ce, 48 },
    { 0xcf2437ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad437d2, 52 },
    { 0x5f0437d2, 90 },
    { 0x479437d4, 98 },
    { 0, 0 },
    { 0xfa8437d6, 23 },
    { 0x3f3437d7, 33 },
    { 0xd43437d6, 33 },
    { 0xd92437d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b437dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d437e0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0437e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd437ec, 24 },
    { 0x3e437ed, 43 },
    { 0xf34437ed, 98 },
    { 0xc27437ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b437f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93437fc, 61 },
    { 0, 0 },
    { 0xa9f437fe, 81 },
    { 0x986437fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa043808, 93 },
    { 0x7f943808, 33 },
    { 0x40c43808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa474380d, 95 },
    { 0xe154380e, 78 },
    { 0x7b04380e, 90 },
    { 0x78543810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1043813, 33 },
    { 0x26d43814, 90 },
    { 0x63543813, 17 },
    { 0x57a43816, 61 },
    { 0x36143814, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f24381c, 95 },
    { 0, 0 },
    { 0x2594381e, 51 },
    { 0x4da4381f, 90 },
    { 0, 0 },
    { 0x3d343821, 95 },
    { 0x8a943822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4382a, 95 },
    { 0x4cf4382b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b43836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf4383a, 61 },
    { 0x7774383b, 98 },
    { 0x6894383a, 76 },
    { 0xad34383d, 33 },
    { 0xad04383e, 98 },
    { 0x2c44383f, 33 },
    { 0x51243840, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a143843, 87 },
    { 0, 0 },
    { 0x52b43845, 90 },
    { 0x96543845, 65 },
    { 0xfbb43847, 101 },
    { 0x3ef43847, 76 },
    { 0xaad43849, 43 },
    { 0, 0 },
    { 0x3174384b, 46 },
    { 0x9514384c, 95 },
    { 0, 0 },
    { 0x6fe4384e, 23 },
    { 0xcd4384e, 55 },
    { 0xb164384e, 78 },
    { 0xb3e43851, 23 },
    { 0xb6643852, 90 },
    { 0x4e743852, 34 },
    { 0xfef43853, 55 },
    { 0xd0043852, 95 },
    { 0xd643856, 66 },
    { 0, 0 },
    { 0x3bc43858, 98 },
    { 0xf6343859, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4804385c, 98 },
    { 0xa1d4385d, 55 },
    { 0, 0 },
    { 0x28b4385f, 33 },
    { 0, 0 },
    { 0x77443861, 55 },
    { 0, 0 },
    { 0x91b43863, 81 },
    { 0, 0 },
    { 0x47943865, 56 },
    { 0x18943865, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1143870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0643875, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a943879, 43 },
    { 0x4f343879, 99 },
    { 0xaf24387a, 46 },
    { 0, 0 },
    { 0x8164387d, 55 },
    { 0x5e84387e, 98 },
    { 0x8414387f, 86 },
    { 0x71a43880, 96 },
    { 0x24e43881, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d43885, 66 },
    { 0xd2f43885, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de43889, 90 },
    { 0, 0 },
    { 0x3b24388b, 61 },
    { 0, 0 },
    { 0x3924388d, 90 },
    { 0x3734388d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b743892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd043897, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d4389b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8438a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63438a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7438ae, 61 },
    { 0xdde438af, 55 },
    { 0, 0 },
    { 0xa96438b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82438b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f438bc, 96 },
    { 0x9e9438bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb438c0, 68 },
    { 0xb67438c1, 23 },
    { 0x201438c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184438c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b438cd, 55 },
    { 0x3aa438ce, 90 },
    { 0xb8438cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404438d2, 98 },
    { 0x864438d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e438d6, 55 },
    { 0xb36438d6, 33 },
    { 0xd33438d8, 43 },
    { 0x7d5438d8, 93 },
    { 0xb51438d8, 61 },
    { 0xac8438db, 23 },
    { 0x702438dc, 33 },
    { 0xbdd438dc, 61 },
    { 0, 0 },
    { 0xd14438df, 76 },
    { 0xaea438e0, 76 },
    { 0, 0 },
    { 0x6a2438e2, 87 },
    { 0xa59438e3, 43 },
    { 0xbc8438e2, 23 },
    { 0, 0 },
    { 0x7f4438e6, 52 },
    { 0, 0 },
    { 0xe3438e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e438ec, 43 },
    { 0xa90438ed, 95 },
    { 0xb64438ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97438f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59438f8, 91 },
    { 0x29a438f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d438fc, 90 },
    { 0xbfe438fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e543902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52043905, 98 },
    { 0x73343905, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884390f, 93 },
    { 0x26b4390f, 98 },
    { 0xb5f4390f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e343916, 33 },
    { 0, 0 },
    { 0x23643918, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc634391d, 95 },
    { 0x3d54391e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd43921, 61 },
    { 0x35f43922, 96 },
    { 0x38b43922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb943928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d4392c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7543937, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4393b, 98 },
    { 0xcc34393b, 61 },
    { 0xfe04393d, 95 },
    { 0, 0 },
    { 0x7e24393f, 66 },
    { 0x8554393f, 76 },
    { 0xa7943941, 46 },
    { 0xfb043941, 95 },
    { 0xd1f43943, 34 },
    { 0, 0 },
    { 0xc4443945, 33 },
    { 0x15243946, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34394c, 23 },
    { 0xe084394c, 98 },
    { 0xf004394c, 3 },
    { 0, 0 },
    { 0x46943950, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa943953, 93 },
    { 0x6243953, 93 },
    { 0xe6d43955, 61 },
    { 0xa6843953, 43 },
    { 0xd9543957, 93 },
    { 0x72643958, 56 },
    { 0, 0 },
    { 0x8d94395a, 76 },
    { 0xf344395b, 90 },
    { 0xbc94395b, 99 },
    { 0xb354395b, 55 },
    { 0xd44395d, 76 },
    { 0x1774395d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee43964, 98 },
    { 0x3c243964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8443968, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3884397b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4397f, 98 },
    { 0xb544397f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99143989, 55 },
    { 0xc3543989, 61 },
    { 0x2ae43989, 53 },
    { 0x4464398c, 91 },
    { 0xd2a4398c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e43996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a4399d, 99 },
    { 0, 0 },
    { 0xd354399f, 23 },
    { 0, 0 },
    { 0xf12439a1, 95 },
    { 0xd95439a2, 98 },
    { 0, 0 },
    { 0xcff439a4, 46 },
    { 0xd8c439a5, 34 },
    { 0x263439a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820439ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5439af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04439b2, 55 },
    { 0xdab439b3, 23 },
    { 0xfd7439b2, 55 },
    { 0x20d439b5, 95 },
    { 0xb83439b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418439be, 93 },
    { 0xa08439bf, 98 },
    { 0xaf2439c0, 87 },
    { 0x2a8439c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef439c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be439cb, 33 },
    { 0x3a4439cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858439d4, 95 },
    { 0x20a439d5, 90 },
    { 0x273439d4, 61 },
    { 0x5a0439d7, 99 },
    { 0x26f439d8, 87 },
    { 0x82c439d7, 76 },
    { 0x241439da, 67 },
    { 0, 0 },
    { 0xbdd439dc, 90 },
    { 0xf7e439dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440439e1, 33 },
    { 0x9d0439e2, 61 },
    { 0x40d439e1, 98 },
    { 0x429439e4, 83 },
    { 0x933439e4, 55 },
    { 0xc2b439e5, 77 },
    { 0xde6439e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f439ef, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174439f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1439f9, 55 },
    { 0x216439fa, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x426439fd, 33 },
    { 0x680439fe, 76 },
    { 0x5e439fd, 93 },
    { 0x9db43a00, 90 },
    { 0x2243a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7543a06, 90 },
    { 0x9ef43a07, 34 },
    { 0x62043a07, 34 },
    { 0x63543a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e43a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d043a12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50643a1c, 23 },
    { 0xed543a1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3143a22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e543a25, 98 },
    { 0xf3b43a26, 34 },
    { 0xd0c43a26, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d43a2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5043a36, 33 },
    { 0x66943a36, 90 },
    { 0x23f43a38, 95 },
    { 0xb8343a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19343a3d, 98 },
    { 0x4e43a3e, 61 },
    { 0x3f443a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c43a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4943a4a, 95 },
    { 0, 0 },
    { 0xd7a43a4c, 61 },
    { 0xb2543a4d, 34 },
    { 0xe2743a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7443a56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc43a5d, 55 },
    { 0, 0 },
    { 0x26543a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89743a64, 17 },
    { 0x40e43a65, 76 },
    { 0x7143a65, 95 },
    { 0xd8843a66, 33 },
    { 0x62143a68, 98 },
    { 0xc8443a69, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46043a71, 52 },
    { 0, 0 },
    { 0xa3a43a73, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d43a7c, 90 },
    { 0, 0 },
    { 0x9da43a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd43a83, 90 },
    { 0, 0 },
    { 0x51843a85, 95 },
    { 0x1fd43a85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb43a89, 65 },
    { 0, 0 },
    { 0xca943a8b, 69 },
    { 0x40343a8c, 34 },
    { 0xe2043a8b, 98 },
    { 0x90343a8c, 97 },
    { 0x56b43a8c, 90 },
    { 0xc7a43a90, 40 },
    { 0x5e843a91, 99 },
    { 0xe3843a92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f43a9d, 90 },
    { 0x99c43a9e, 61 },
    { 0, 0 },
    { 0x89a43aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac143aa3, 95 },
    { 0xe1a43aa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96243aab, 61 },
    { 0, 0 },
    { 0x75b43aad, 52 },
    { 0xe6943aae, 23 },
    { 0x64b43aaf, 95 },
    { 0x84343aae, 65 },
    { 0xb4543ab1, 61 },
    { 0x56743ab2, 33 },
    { 0xee443aad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e43ab7, 98 },
    { 0, 0 },
    { 0x16643ab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8343abd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f43ac1, 33 },
    { 0, 0 },
    { 0x59143ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8343ac6, 40 },
    { 0xbf343ac7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99043acc, 93 },
    { 0, 0 },
    { 0x4ed43ace, 33 },
    { 0x54543acf, 66 },
    { 0x7d043ad0, 85 },
    { 0x99b43ad1, 85 },
    { 0xb2443ad0, 68 },
    { 0x83443ad2, 67 },
    { 0, 0 },
    { 0x19c43ad5, 90 },
    { 0x9ce43ad5, 90 },
    { 0, 0 },
    { 0xaec43ad8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4143adb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9243ae0, 84 },
    { 0, 0 },
    { 0xd4643ae2, 33 },
    { 0x8e143ae3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343aeb, 84 },
    { 0, 0 },
    { 0x44143aed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e743af0, 98 },
    { 0x2e43af1, 55 },
    { 0, 0 },
    { 0xcb443af3, 23 },
    { 0xdf43af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5743af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74043afd, 84 },
    { 0, 0 },
    { 0xe4743aff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb543b02, 87 },
    { 0x5fb43b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef43b06, 55 },
    { 0x5dd43b07, 52 },
    { 0xf8543b07, 34 },
    { 0x85343b06, 66 },
    { 0xc4e43b08, 33 },
    { 0xbba43b07, 98 },
    { 0x2ec43b0c, 55 },
    { 0x1a543b0d, 33 },
    { 0, 0 },
    { 0xe7943b0f, 95 },
    { 0xa9143b10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d143b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23343b1a, 90 },
    { 0x3c943b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda243b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a43b25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a243b28, 97 },
    { 0x6bf43b28, 95 },
    { 0xfb743b28, 76 },
    { 0xe2743b2b, 81 },
    { 0xdf143b2a, 90 },
    { 0xe7543b2d, 100 },
    { 0xf7f43b2e, 34 },
    { 0x5b643b2f, 91 },
    { 0xe5643b2f, 100 },
    { 0xa7e43b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d43b36, 95 },
    { 0xbc343b36, 61 },
    { 0, 0 },
    { 0x61b43b39, 90 },
    { 0, 0 },
    { 0x3b943b3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9643b41, 95 },
    { 0x91743b42, 34 },
    { 0x62043b42, 90 },
    { 0x33143b44, 66 },
    { 0x79043b41, 61 },
    { 0xf3343b46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9943b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4743b4e, 23 },
    { 0x90e43b4e, 91 },
    { 0xd4343b4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81443b54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca43b58, 61 },
    { 0x7be43b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f43b5f, 90 },
    { 0x86643b5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b743b65, 67 },
    { 0, 0 },
    { 0x15943b67, 23 },
    { 0x25c43b68, 43 },
    { 0x51543b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85943b6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f43b7a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4543b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a43b8a, 90 },
    { 0x83743b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb43b91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb443b99, 66 },
    { 0x8dd43b9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96543b9d, 90 },
    { 0, 0 },
    { 0xd9643b9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d43ba4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5843ba8, 20 },
    { 0x27843ba9, 23 },
    { 0x91e43baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e43bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c043bb3, 40 },
    { 0, 0 },
    { 0x43343bb5, 87 },
    { 0, 0 },
    { 0x3e443bb7, 55 },
    { 0x74343bb7, 46 },
    { 0xa2e43bb9, 87 },
    { 0xc2543bb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73243bbf, 98 },
    { 0x2ea43bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc443bc3, 33 },
    { 0x41b43bc3, 23 },
    { 0x7f743bc4, 90 },
    { 0, 0 },
    { 0xe3c43bc7, 61 },
    { 0x22043bc8, 34 },
    { 0x1ff43bc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57543bcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1543bcf, 98 },
    { 0x91e43bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a243bd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55143bd7, 46 },
    { 0, 0 },
    { 0xad243bd9, 33 },
    { 0x19b43bda, 61 },
    { 0xd5643bdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343be0, 76 },
    { 0xb7043be0, 91 },
    { 0x16343be1, 33 },
    { 0x42e43be3, 34 },
    { 0, 0 },
    { 0xd7e43be5, 61 },
    { 0xeae43be6, 33 },
    { 0xa3443be6, 75 },
    { 0x8d43be6, 66 },
    { 0x81843be9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81443bee, 90 },
    { 0x36443bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28543bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab543bfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac043c00, 78 },
    { 0xd0943c00, 95 },
    { 0x61c43c00, 90 },
    { 0x47c43c02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf43c06, 66 },
    { 0, 0 },
    { 0x97f43c08, 87 },
    { 0, 0 },
    { 0xcfa43c0a, 61 },
    { 0x5e143c0b, 68 },
    { 0x27b43c0c, 98 },
    { 0xbe643c0b, 90 },
    { 0x8e043c0d, 33 },
    { 0xad43c0f, 46 },
    { 0x9f143c0c, 77 },
    { 0x62143c11, 23 },
    { 0xd7043c12, 55 },
    { 0x80043c13, 90 },
    { 0xf843c13, 34 },
    { 0, 0 },
    { 0x4b643c16, 67 },
    { 0xe7a43c17, 95 },
    { 0xe1343c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f43c1d, 90 },
    { 0x62243c1e, 52 },
    { 0xfbe43c1d, 93 },
    { 0, 0 },
    { 0x76443c21, 33 },
    { 0, 0 },
    { 0x7bb43c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e843c29, 33 },
    { 0xb3143c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b043c2f, 90 },
    { 0, 0 },
    { 0x53543c31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc43c35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x33243c38, 61 },
    { 0, 0 },
    { 0x1b843c3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7643c3f, 78 },
    { 0xe5343c40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80143c49, 33 },
    { 0x19443c4a, 91 },
    { 0, 0 },
    { 0x38e43c4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db43c4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52643c54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c43c5b, 46 },
    { 0xeb643c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5343c60, 23 },
    { 0x76343c61, 90 },
    { 0xb6e43c62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e43c65, 46 },
    { 0xc0d43c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd243c6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a643c70, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1443c75, 61 },
    { 0, 0 },
    { 0x66243c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a43c7b, 52 },
    { 0x5ef43c7c, 90 },
    { 0, 0 },
    { 0x5a043c7e, 46 },
    { 0, 0 },
    { 0x40743c80, 90 },
    { 0xf0843c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde43c91, 91 },
    { 0xf4043c92, 61 },
    { 0x6a843c93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec43c97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b43c9a, 78 },
    { 0x9e443c9a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b43ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df43ca7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a43caf, 46 },
    { 0, 0 },
    { 0xd6543cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa43cba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a943cc0, 87 },
    { 0x1ee43cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4143cc7, 23 },
    { 0x8243cc7, 46 },
    { 0xdb143cc8, 61 },
    { 0, 0 },
    { 0x31c43ccb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff343cd0, 46 },
    { 0, 0 },
    { 0x1ee43cd2, 33 },
    { 0, 0 },
    { 0x46843cd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b43cda, 40 },
    { 0x26443cdb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7543cdf, 61 },
    { 0x44c43cdf, 43 },
    { 0x50a43ce1, 43 },
    { 0xac443ce2, 66 },
    { 0x4bb43ce3, 61 },
    { 0xcb143ce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1643cea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x36543ced, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc343cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d443cf7, 33 },
    { 0, 0 },
    { 0x73943cf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f43cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc43d00, 98 },
    { 0xba543d00, 46 },
    { 0, 0 },
    { 0xbdf43d03, 100 },
    { 0x31543d04, 33 },
    { 0x23943d05, 68 },
    { 0xf9743d04, 61 },
    { 0xd2243d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de43d0c, 98 },
    { 0, 0 },
    { 0xb543d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc43d14, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e43d19, 55 },
    { 0xff743d1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a443d1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56243d21, 23 },
    { 0xba543d22, 33 },
    { 0, 0 },
    { 0x45043d24, 95 },
    { 0, 0 },
    { 0x6e743d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc743d2a, 98 },
    { 0xf8143d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a43d2f, 24 },
    { 0x5643d30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b843d33, 78 },
    { 0xfa543d33, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69543d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643d40, 43 },
    { 0, 0 },
    { 0x64f43d42, 98 },
    { 0xae643d43, 101 },
    { 0x9ab43d43, 90 },
    { 0x2b243d43, 78 },
    { 0x69c43d44, 98 },
    { 0, 0 },
    { 0x32843d48, 85 },
    { 0, 0 },
    { 0x40c43d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8343d4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x71943d50, 34 },
    { 0xcc643d50, 33 },
    { 0x1ac43d52, 61 },
    { 0x78543d52, 55 },
    { 0xf943d53, 65 },
    { 0, 0 },
    { 0xaac43d56, 67 },
    { 0x64c43d56, 23 },
    { 0xae743d58, 61 },
    { 0x8c43d59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a43d60, 88 },
    { 0x21f43d61, 23 },
    { 0x11043d62, 61 },
    { 0x59243d63, 93 },
    { 0x5bd43d63, 23 },
    { 0x9ad43d60, 34 },
    { 0xade43d66, 61 },
    { 0x7a143d67, 101 },
    { 0xcd943d68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1243d6c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31743d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f43d75, 55 },
    { 0, 0 },
    { 0xc6743d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c43d7e, 55 },
    { 0x1fd43d7f, 68 },
    { 0x9cc43d7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd43d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c343d89, 34 },
    { 0xac443d8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c43d8d, 87 },
    { 0xb9643d8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed643d99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e943d9d, 46 },
    { 0, 0 },
    { 0xdca43d9f, 33 },
    { 0xbd443da0, 67 },
    { 0x10e43da1, 98 },
    { 0xf8543da2, 76 },
    { 0x2ca43da2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5543da6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x66343da9, 33 },
    { 0x3fe43da9, 58 },
    { 0x59a43dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc43daf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce943db5, 95 },
    { 0xb5e43db5, 98 },
    { 0, 0 },
    { 0xaa843db8, 61 },
    { 0, 0 },
    { 0xd1c43dba, 98 },
    { 0xda043dbb, 81 },
    { 0, 0 },
    { 0xfe543dbd, 33 },
    { 0xbe143dbe, 23 },
    { 0, 0 },
    { 0x14743dc0, 95 },
    { 0x28b43dc1, 23 },
    { 0, 0 },
    { 0xb7543dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc43dcc, 93 },
    { 0x34843dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73843dd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3543dd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x22443ddc, 95 },
    { 0xc4e43ddd, 95 },
    { 0, 0 },
    { 0xe8743ddf, 95 },
    { 0, 0 },
    { 0xfa343de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf243de5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16143def, 55 },
    { 0x95743df0, 95 },
    { 0x7ac43def, 90 },
    { 0xef43df2, 90 },
    { 0, 0 },
    { 0x1a843df4, 34 },
    { 0, 0 },
    { 0xdb743df6, 90 },
    { 0, 0 },
    { 0x51243df8, 92 },
    { 0x7b443df9, 90 },
    { 0x91943dfa, 99 },
    { 0, 0 },
    { 0xbab43dfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31643dff, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x53243e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a443e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4543e0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c43e12, 34 },
    { 0, 0 },
    { 0xda943e14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa043e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb43e22, 17 },
    { 0x65143e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd443e28, 91 },
    { 0, 0 },
    { 0xe9043e2a, 90 },
    { 0xa5943e2b, 98 },
    { 0xf0043e2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf943e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf143e34, 61 },
    { 0xa0d43e34, 96 },
    { 0, 0 },
    { 0xadc43e37, 90 },
    { 0xa0543e38, 61 },
    { 0x24743e39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c43e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f43e3f, 101 },
    { 0, 0 },
    { 0x40043e41, 23 },
    { 0x7fc43e42, 61 },
    { 0x5c443e43, 87 },
    { 0xe0843e41, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c043e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e43e54, 33 },
    { 0x1b943e54, 46 },
    { 0x3d943e56, 34 },
    { 0x8cf43e57, 43 },
    { 0, 0 },
    { 0xf4c43e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc843e60, 98 },
    { 0x4fb43e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b643e64, 98 },
    { 0xa7643e65, 46 },
    { 0x36143e65, 61 },
    { 0x45443e67, 61 },
    { 0, 0 },
    { 0xb4543e69, 68 },
    { 0, 0 },
    { 0x33643e6b, 46 },
    { 0, 0 },
    { 0xda043e6d, 90 },
    { 0x79043e6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d043e76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb243e79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62143e7d, 84 },
    { 0, 0 },
    { 0x94043e7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5343e82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1243e85, 98 },
    { 0x38743e86, 90 },
    { 0, 0 },
    { 0x98c43e88, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c43e8d, 61 },
    { 0xf3b43e8e, 85 },
    { 0xaa43e8e, 33 },
    { 0xba343e8f, 61 },
    { 0x5c543e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b43e97, 98 },
    { 0x89043e98, 77 },
    { 0, 0 },
    { 0xc0b43e9a, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b43e9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa43ea3, 98 },
    { 0xe4e43ea3, 90 },
    { 0, 0 },
    { 0xd1843ea6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c343eaa, 90 },
    { 0x75443eab, 91 },
    { 0, 0 },
    { 0x75543ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68643eb0, 95 },
    { 0, 0 },
    { 0xc7343eb2, 61 },
    { 0x5f443eb2, 78 },
    { 0, 0 },
    { 0xeaa43eb5, 99 },
    { 0, 0 },
    { 0xd3743eb7, 33 },
    { 0, 0 },
    { 0xdc843eb9, 93 },
    { 0x6c743eba, 76 },
    { 0x1a943ebb, 91 },
    { 0x57643ebc, 23 },
    { 0x27243ebd, 90 },
    { 0, 0 },
    { 0x4ed43ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f43ec4, 76 },
    { 0, 0 },
    { 0xb43ec6, 90 },
    { 0x1ba43ec6, 90 },
    { 0xac943ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf43ecb, 90 },
    { 0, 0 },
    { 0xea43ecd, 78 },
    { 0xb4443ece, 85 },
    { 0xd9943ecd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70943ed3, 33 },
    { 0x76443ed3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70443ed9, 43 },
    { 0, 0 },
    { 0x75343edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8743ee0, 99 },
    { 0xd7643ee0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7343eec, 78 },
    { 0x52843eed, 23 },
    { 0xb0343eee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1443ef3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a443efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d843f02, 33 },
    { 0, 0 },
    { 0x96e43f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11443f07, 33 },
    { 0xf843f07, 90 },
    { 0xe5d43f09, 34 },
    { 0x9b443f08, 46 },
    { 0, 0 },
    { 0xd0343f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76743f10, 76 },
    { 0x88643f11, 98 },
    { 0x96343f12, 85 },
    { 0xccd43f13, 61 },
    { 0xf0c43f12, 61 },
    { 0xd3b43f13, 34 },
    { 0x31543f16, 98 },
    { 0x27143f17, 55 },
    { 0x28843f18, 100 },
    { 0xced43f18, 66 },
    { 0x8043f13, 96 },
    { 0, 0 },
    { 0x77243f1c, 98 },
    { 0, 0 },
    { 0x28f43f1e, 61 },
    { 0x48143f1f, 90 },
    { 0xad143f1f, 98 },
    { 0xdf43f1e, 98 },
    { 0, 0 },
    { 0xdcf43f23, 61 },
    { 0x75143f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf843f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe543f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47343f33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50343f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff43f3d, 68 },
    { 0xece43f3d, 90 },
    { 0, 0 },
    { 0x6e143f40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6343f43, 90 },
    { 0x7e43f43, 17 },
    { 0xce543f45, 90 },
    { 0x41543f43, 98 },
    { 0xb5f43f47, 61 },
    { 0x57d43f48, 61 },
    { 0xad743f48, 83 },
    { 0xc2843f43, 33 },
    { 0xcdf43f48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba43f51, 78 },
    { 0x36143f52, 90 },
    { 0, 0 },
    { 0xb7c43f54, 95 },
    { 0x35443f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e43f58, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a43f5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a943f60, 95 },
    { 0x58c43f60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343f65, 33 },
    { 0, 0 },
    { 0x67343f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70943f6e, 34 },
    { 0xd4f43f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd443f77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87843f84, 34 },
    { 0, 0 },
    { 0x65843f86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10643f8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad143f8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e43f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5243f95, 67 },
    { 0x10643f96, 33 },
    { 0x74243f97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71343fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a43fa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d43fa8, 67 },
    { 0xcf643fa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f43fae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf243fb3, 61 },
    { 0xed143fb4, 52 },
    { 0, 0 },
    { 0xd2843fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c843fbd, 98 },
    { 0x7c43fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1143fc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99243fc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e43fce, 23 },
    { 0, 0 },
    { 0x81e43fd0, 76 },
    { 0xcae43fd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e43fd6, 95 },
    { 0x15b43fd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2843fda, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e43fdd, 52 },
    { 0xac343fdd, 90 },
    { 0xef443fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15943fe3, 95 },
    { 0xbfc43fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d43fe7, 46 },
    { 0, 0 },
    { 0x84443fe9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98243fee, 33 },
    { 0xe7543fef, 46 },
    { 0, 0 },
    { 0x82b43ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x28143ff4, 84 },
    { 0x26943ff5, 40 },
    { 0xfc643ff4, 98 },
    { 0, 0 },
    { 0x68843ff8, 33 },
    { 0, 0 },
    { 0x40343ffa, 23 },
    { 0x99443ffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e44002, 90 },
    { 0xbb944003, 52 },
    { 0, 0 },
    { 0x76f44005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51944008, 84 },
    { 0x33144009, 90 },
    { 0x5e04400a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa634400d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d44010, 90 },
    { 0, 0 },
    { 0x98044012, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764401a, 98 },
    { 0x42e4401b, 76 },
    { 0, 0 },
    { 0x76b4401d, 43 },
    { 0x2e84401e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d244021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd44025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac4402b, 88 },
    { 0xdce4402c, 95 },
    { 0xf714402b, 66 },
    { 0x85c4402e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb44033, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac4403c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e44043, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f744046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d4404a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba14404d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19644053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b844057, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f84405b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed444060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944069, 33 },
    { 0xd624406a, 77 },
    { 0x224406b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2844074, 99 },
    { 0xe8544075, 95 },
    { 0x33544075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde244079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4408c, 90 },
    { 0x69a4408d, 98 },
    { 0x7bb4408d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1944099, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345440a2, 33 },
    { 0x8fc440a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50440a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff440ac, 90 },
    { 0, 0 },
    { 0x171440ae, 34 },
    { 0x752440af, 85 },
    { 0, 0 },
    { 0x62440b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08440b5, 90 },
    { 0, 0 },
    { 0xf5a440b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59440ba, 34 },
    { 0xe0c440ba, 90 },
    { 0xd10440bc, 90 },
    { 0x24c440bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x447440c0, 61 },
    { 0, 0 },
    { 0x7c7440c2, 61 },
    { 0xe1f440c3, 93 },
    { 0xcb1440c2, 81 },
    { 0, 0 },
    { 0xdf8440c6, 43 },
    { 0xd79440c6, 61 },
    { 0xd67440c7, 93 },
    { 0x45a440c6, 93 },
    { 0, 0 },
    { 0xe88440cb, 43 },
    { 0x5c8440cb, 90 },
    { 0xddd440cd, 87 },
    { 0x53f440ce, 23 },
    { 0xf75440cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849440d8, 61 },
    { 0x474440d9, 61 },
    { 0x382440da, 61 },
    { 0xe8f440db, 61 },
    { 0xff2440dc, 90 },
    { 0x45f440dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4440e1, 65 },
    { 0, 0 },
    { 0x814440e3, 33 },
    { 0, 0 },
    { 0xb9440e5, 76 },
    { 0xbc0440e6, 91 },
    { 0, 0 },
    { 0xca3440e8, 61 },
    { 0xcb6440e9, 76 },
    { 0xe03440e8, 7 },
    { 0xff4440eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e440ee, 81 },
    { 0xdc7440ef, 95 },
    { 0, 0 },
    { 0xf9f440f1, 76 },
    { 0xd63440f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x464440f5, 81 },
    { 0, 0 },
    { 0x42440f7, 52 },
    { 0xc24440f7, 98 },
    { 0, 0 },
    { 0xf5c440fa, 90 },
    { 0xe09440fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7440ff, 52 },
    { 0, 0 },
    { 0xd0444101, 78 },
    { 0x6b644101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f544105, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc44110, 61 },
    { 0, 0 },
    { 0x7a844112, 90 },
    { 0, 0 },
    { 0xcb144114, 33 },
    { 0xdeb44115, 84 },
    { 0, 0 },
    { 0xe4144117, 33 },
    { 0, 0 },
    { 0x77144119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58744125, 34 },
    { 0, 0 },
    { 0xa3244127, 23 },
    { 0xb8744128, 61 },
    { 0xf4444127, 43 },
    { 0x7c944127, 66 },
    { 0xa694412b, 23 },
    { 0x4334412c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8944131, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c44134, 31 },
    { 0, 0 },
    { 0xff744136, 90 },
    { 0x8e544137, 98 },
    { 0x3c544137, 33 },
    { 0, 0 },
    { 0xcbe4413a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4144149, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e24414c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd344150, 61 },
    { 0, 0 },
    { 0xdaa44152, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c344158, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc4415c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40644165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16044168, 23 },
    { 0, 0 },
    { 0xead4416a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f44170, 53 },
    { 0xbd844171, 90 },
    { 0, 0 },
    { 0xc6844173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50144176, 43 },
    { 0x21f44177, 76 },
    { 0xc7c44178, 90 },
    { 0x59244179, 55 },
    { 0xb1944178, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xda24417d, 88 },
    { 0xa1c4417e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44244181, 33 },
    { 0, 0 },
    { 0x3be44183, 33 },
    { 0, 0 },
    { 0xd2c44185, 88 },
    { 0xd2844185, 34 },
    { 0x99044185, 55 },
    { 0x13044187, 95 },
    { 0x1944188, 52 },
    { 0xbe344188, 40 },
    { 0xd824418a, 95 },
    { 0xe5c44188, 98 },
    { 0xd3444186, 84 },
    { 0x4b34418e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76744198, 33 },
    { 0x1c844198, 90 },
    { 0x51144199, 43 },
    { 0x39744199, 43 },
    { 0x8a34419c, 99 },
    { 0, 0 },
    { 0x9834419e, 76 },
    { 0xfcf4419e, 23 },
    { 0, 0 },
    { 0xe39441a1, 90 },
    { 0xb84441a2, 95 },
    { 0x632441a1, 95 },
    { 0xe9f441a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c441a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2441ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec441b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f441bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be441c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8441c6, 98 },
    { 0x4c441c6, 76 },
    { 0x8fa441c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b441ce, 98 },
    { 0, 0 },
    { 0xf7c441d0, 43 },
    { 0, 0 },
    { 0x5ff441d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545441d8, 69 },
    { 0x27a441d9, 23 },
    { 0xdb8441d9, 90 },
    { 0xa6f441d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d441e0, 77 },
    { 0xc0d441e1, 66 },
    { 0x49e441e2, 95 },
    { 0x5b3441e1, 91 },
    { 0x190441e2, 99 },
    { 0x4a7441e4, 61 },
    { 0x12441e2, 90 },
    { 0, 0 },
    { 0xa5e441e8, 78 },
    { 0x6f7441e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400441f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a441f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0441f9, 61 },
    { 0x55d441f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f44203, 98 },
    { 0x81244204, 55 },
    { 0xabe44204, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d44209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334420f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe44212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b744215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d4421c, 34 },
    { 0x6634421c, 98 },
    { 0x6324421c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82244225, 43 },
    { 0, 0 },
    { 0xff944227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e94422a, 61 },
    { 0xe424422a, 34 },
    { 0, 0 },
    { 0xfbe4422d, 98 },
    { 0x1ab4422e, 33 },
    { 0xf544422f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5244233, 55 },
    { 0x17444234, 88 },
    { 0x2d644234, 55 },
    { 0xddf44234, 33 },
    { 0x1e244233, 23 },
    { 0x17344236, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c24423b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb44241, 23 },
    { 0x74e44242, 81 },
    { 0xbd944242, 90 },
    { 0x8144244, 34 },
    { 0x37644245, 34 },
    { 0x29044243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25344249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7944252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4044425d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e44260, 33 },
    { 0x62244261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e44266, 85 },
    { 0x67844267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da4426c, 55 },
    { 0, 0 },
    { 0x63f4426e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e14427d, 91 },
    { 0x6244427e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6444281, 88 },
    { 0xa7244281, 76 },
    { 0, 0 },
    { 0x1ec44284, 23 },
    { 0xc644285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074428a, 76 },
    { 0xe3d4428a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a34428f, 61 },
    { 0, 0 },
    { 0x21644291, 17 },
    { 0xcfa44292, 23 },
    { 0x4ef44293, 76 },
    { 0x78f44292, 55 },
    { 0x51b44294, 61 },
    { 0x3ca44294, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e44299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3442a0, 93 },
    { 0x810442a1, 55 },
    { 0, 0 },
    { 0xd9f442a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x167442a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b442ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f442b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7442c3, 78 },
    { 0, 0 },
    { 0x820442c5, 38 },
    { 0x6dd442c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a442c9, 23 },
    { 0x1d7442c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71442d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657442d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8442dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab442df, 76 },
    { 0, 0 },
    { 0x930442e1, 23 },
    { 0xe2a442e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c442e5, 90 },
    { 0xb81442e6, 95 },
    { 0x6b3442e7, 88 },
    { 0x941442e8, 43 },
    { 0x7ee442e9, 61 },
    { 0x180442ea, 90 },
    { 0, 0 },
    { 0x382442ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570442f0, 95 },
    { 0, 0 },
    { 0xec2442f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8442f7, 98 },
    { 0, 0 },
    { 0xf9a442f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04442fd, 34 },
    { 0x766442fe, 90 },
    { 0xe4b442fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80544308, 33 },
    { 0, 0 },
    { 0x59d4430a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8544430d, 61 },
    { 0, 0 },
    { 0x94a4430f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x22844312, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b44317, 84 },
    { 0x6c144317, 3 },
    { 0x23044319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d4431c, 66 },
    { 0, 0 },
    { 0x3e34431e, 55 },
    { 0xa544431e, 23 },
    { 0, 0 },
    { 0x7be44321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26844325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4432a, 93 },
    { 0xab34432a, 95 },
    { 0xb864432c, 76 },
    { 0x4af4432a, 65 },
    { 0xa304432e, 90 },
    { 0x8ef4432f, 99 },
    { 0x4b744330, 17 },
    { 0x53a4432e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e44335, 62 },
    { 0x85244335, 98 },
    { 0xcf944337, 55 },
    { 0xf8544337, 98 },
    { 0, 0 },
    { 0x1e24433a, 90 },
    { 0x2e84433a, 33 },
    { 0x5844433a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c044344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a844347, 92 },
    { 0x43d44348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee4434e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d44351, 87 },
    { 0x73544352, 80 },
    { 0, 0 },
    { 0x17e44354, 61 },
    { 0x83844355, 95 },
    { 0x97444354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4435a, 43 },
    { 0x9d74435a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf944435f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd444366, 95 },
    { 0x11d44366, 90 },
    { 0, 0 },
    { 0xef944369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5394436f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb44437b, 100 },
    { 0xe34437b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb814437f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f444386, 65 },
    { 0, 0 },
    { 0x21244388, 95 },
    { 0x48944388, 33 },
    { 0xd6244389, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c44438d, 46 },
    { 0x5734438e, 95 },
    { 0xcf44438f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8244395, 90 },
    { 0, 0 },
    { 0xc3744397, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4439c, 76 },
    { 0, 0 },
    { 0xeff4439e, 33 },
    { 0xd474439f, 90 },
    { 0x194439e, 52 },
    { 0, 0 },
    { 0x3a5443a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d443a6, 95 },
    { 0x5e0443a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb443ab, 98 },
    { 0x826443ac, 99 },
    { 0, 0 },
    { 0x43a443ae, 61 },
    { 0x8fe443af, 65 },
    { 0x86e443b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19443b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f443b8, 76 },
    { 0x473443b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45443bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9443bf, 95 },
    { 0xf76443c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x873443c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603443c8, 61 },
    { 0xdf6443c9, 33 },
    { 0xf1e443ca, 95 },
    { 0x558443cb, 33 },
    { 0x222443cc, 90 },
    { 0x3c0443cb, 98 },
    { 0xca4443c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80443d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127443dc, 84 },
    { 0, 0 },
    { 0x479443de, 61 },
    { 0, 0 },
    { 0x975443e0, 93 },
    { 0x8e1443e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6443e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1443ea, 90 },
    { 0xdc3443eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50443ee, 33 },
    { 0xdf0443ee, 33 },
    { 0xc73443f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5443f4, 51 },
    { 0xf93443f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f443fd, 33 },
    { 0xc12443fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e644403, 33 },
    { 0xe3644403, 77 },
    { 0x37444405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d944408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd44414, 33 },
    { 0, 0 },
    { 0x3ba44416, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95744419, 76 },
    { 0, 0 },
    { 0x7fd4441b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c44420, 78 },
    { 0x64144421, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde44426, 55 },
    { 0x8a644427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce4442a, 95 },
    { 0, 0 },
    { 0x3164442c, 76 },
    { 0xaee4442d, 95 },
    { 0xbe34442e, 98 },
    { 0x7cf4442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54244433, 33 },
    { 0xd3c44434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c44438, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27744442, 100 },
    { 0xdc444443, 33 },
    { 0xf5e44444, 43 },
    { 0xef244444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee844448, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27044453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f44459, 33 },
    { 0x23a4445a, 100 },
    { 0, 0 },
    { 0xde44445c, 90 },
    { 0, 0 },
    { 0xb7b4445e, 36 },
    { 0x4d04445f, 98 },
    { 0, 0 },
    { 0xa3544461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd144464, 90 },
    { 0xf2d44465, 90 },
    { 0x5b644466, 91 },
    { 0x26c44467, 61 },
    { 0, 0 },
    { 0x41d44469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4446d, 98 },
    { 0xb244446e, 90 },
    { 0x7ac4446e, 99 },
    { 0xd44446e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d44476, 33 },
    { 0xe4c44477, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea64447b, 43 },
    { 0x36e4447b, 55 },
    { 0x6104447d, 95 },
    { 0xa04447c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3544484, 17 },
    { 0, 0 },
    { 0x50844486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3244489, 23 },
    { 0xc574448a, 46 },
    { 0xb34448b, 61 },
    { 0xe0c4448b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa414448f, 91 },
    { 0x2a44490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b44497, 90 },
    { 0x7144498, 33 },
    { 0xb3344499, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4449f, 52 },
    { 0xa2d444a0, 87 },
    { 0x1ed444a1, 78 },
    { 0x204444a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81444a5, 76 },
    { 0xb3e444a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859444ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790444af, 55 },
    { 0xec444af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30444b6, 90 },
    { 0xe39444b7, 90 },
    { 0x26b444b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d444bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7444c3, 61 },
    { 0xfad444c4, 55 },
    { 0xb88444c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0444c8, 51 },
    { 0x22e444c9, 78 },
    { 0x609444ca, 90 },
    { 0, 0 },
    { 0x21f444cc, 99 },
    { 0x5f0444cd, 33 },
    { 0xb3d444ce, 87 },
    { 0x8ef444ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x432444d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d444da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91444de, 98 },
    { 0, 0 },
    { 0x57d444e0, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db444e3, 33 },
    { 0x222444e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763444e9, 100 },
    { 0x7e4444ea, 95 },
    { 0, 0 },
    { 0xcb0444ec, 69 },
    { 0x4a444ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd444fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0444fe, 64 },
    { 0xea5444ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe4450e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88644514, 23 },
    { 0, 0 },
    { 0x89844516, 75 },
    { 0, 0 },
    { 0x41144518, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91844520, 33 },
    { 0, 0 },
    { 0x90444522, 61 },
    { 0, 0 },
    { 0x90144524, 33 },
    { 0xf4b44525, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc644529, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294452d, 76 },
    { 0x2db4452d, 95 },
    { 0x97c4452f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea44534, 66 },
    { 0x6cb44535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d044539, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4453f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c44543, 93 },
    { 0xaf344544, 61 },
    { 0xec544544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7114454a, 99 },
    { 0, 0 },
    { 0xe964454c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb54454f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45144559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f644563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7744566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74456a, 84 },
    { 0x3f94456b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac34456f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f244574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9644577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6724457b, 75 },
    { 0, 0 },
    { 0xb964457d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9644582, 46 },
    { 0, 0 },
    { 0xc8344584, 61 },
    { 0x6fd44584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a44588, 34 },
    { 0x34344589, 88 },
    { 0, 0 },
    { 0x14c4458b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1304458f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a44592, 61 },
    { 0x49f44593, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b344596, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b24459e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f445a1, 99 },
    { 0, 0 },
    { 0xa85445a3, 61 },
    { 0x9d5445a4, 55 },
    { 0, 0 },
    { 0x119445a6, 78 },
    { 0, 0 },
    { 0x884445a8, 90 },
    { 0x7cc445a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5445ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25445b2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a445b9, 90 },
    { 0, 0 },
    { 0xe56445bb, 61 },
    { 0, 0 },
    { 0x171445bd, 95 },
    { 0xca2445be, 91 },
    { 0xd3c445be, 65 },
    { 0, 0 },
    { 0xb00445c1, 90 },
    { 0xb28445c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407445ca, 61 },
    { 0, 0 },
    { 0x37c445cc, 91 },
    { 0x1fc445cd, 78 },
    { 0, 0 },
    { 0x694445cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f445d3, 23 },
    { 0xd7d445d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771445d9, 61 },
    { 0x7bc445da, 90 },
    { 0, 0 },
    { 0xd20445dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697445e0, 98 },
    { 0xf98445e1, 98 },
    { 0x572445e1, 98 },
    { 0x3cb445e3, 76 },
    { 0xfa9445e4, 61 },
    { 0x465445e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x870445e8, 94 },
    { 0, 0 },
    { 0x762445ea, 55 },
    { 0, 0 },
    { 0x136445ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495445f3, 47 },
    { 0x9ea445f4, 90 },
    { 0xfd9445f5, 84 },
    { 0, 0 },
    { 0x33e445f7, 65 },
    { 0xe75445f8, 83 },
    { 0x82445f9, 76 },
    { 0xe57445fa, 93 },
    { 0x10f445fb, 76 },
    { 0xbd8445fc, 98 },
    { 0x8cc445f8, 43 },
    { 0x77d445fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a844602, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f244606, 34 },
    { 0xe7644607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74460a, 90 },
    { 0, 0 },
    { 0x65e4460c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9344611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964461b, 33 },
    { 0, 0 },
    { 0x8cb4461d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54844620, 93 },
    { 0xb5544620, 43 },
    { 0xed444622, 34 },
    { 0, 0 },
    { 0xf7044624, 50 },
    { 0xd7244625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a744628, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea444630, 90 },
    { 0x5d244631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e344636, 76 },
    { 0xafb44637, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c4463a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x714463d, 23 },
    { 0, 0 },
    { 0x4d4463f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4044642, 43 },
    { 0x28244643, 90 },
    { 0xee544644, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b44647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6244650, 98 },
    { 0x59d44651, 61 },
    { 0xf5144652, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f44659, 81 },
    { 0, 0 },
    { 0x25e4465b, 93 },
    { 0x9024465b, 84 },
    { 0, 0 },
    { 0x2f14465e, 76 },
    { 0x55d4465f, 90 },
    { 0x8a044660, 43 },
    { 0, 0 },
    { 0x2de44662, 34 },
    { 0, 0 },
    { 0x43f44664, 61 },
    { 0x1da44665, 88 },
    { 0xc1044666, 84 },
    { 0, 0 },
    { 0x56544668, 43 },
    { 0x42744669, 23 },
    { 0xce24466a, 98 },
    { 0x8da4466a, 76 },
    { 0x2184466c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c944670, 38 },
    { 0, 0 },
    { 0x86444672, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73244675, 23 },
    { 0x4bf44676, 76 },
    { 0, 0 },
    { 0x18b44678, 90 },
    { 0x35044678, 93 },
    { 0, 0 },
    { 0x9584467b, 90 },
    { 0x81a4467c, 95 },
    { 0x3534467b, 98 },
    { 0, 0 },
    { 0x43f4467f, 55 },
    { 0x62444680, 90 },
    { 0x3e844680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c44687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904468a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab44690, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf064469b, 95 },
    { 0, 0 },
    { 0x5144469d, 95 },
    { 0xa414469d, 90 },
    { 0x544469e, 90 },
    { 0x8a34469e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6446a3, 77 },
    { 0xaf8446a3, 43 },
    { 0xcdb446a5, 80 },
    { 0x567446a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb446a9, 84 },
    { 0x80b446aa, 33 },
    { 0, 0 },
    { 0x8a0446ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56446b1, 98 },
    { 0xad4446b2, 94 },
    { 0, 0 },
    { 0x579446b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e446ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd446be, 93 },
    { 0, 0 },
    { 0x100446c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b446c4, 76 },
    { 0xd5d446c5, 33 },
    { 0x8f0446c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72446ce, 43 },
    { 0x44b446cf, 78 },
    { 0xc18446cf, 61 },
    { 0x9f7446d1, 95 },
    { 0x656446ce, 98 },
    { 0x4b9446d3, 55 },
    { 0xe16446d1, 95 },
    { 0x168446cf, 61 },
    { 0, 0 },
    { 0x77446d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931446dd, 34 },
    { 0, 0 },
    { 0x97a446df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7446e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a446e5, 17 },
    { 0x63c446e6, 61 },
    { 0x1e7446e7, 76 },
    { 0x714446e8, 75 },
    { 0xde4446e9, 68 },
    { 0, 0 },
    { 0x125446eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f446f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10446f5, 33 },
    { 0, 0 },
    { 0xe05446f7, 33 },
    { 0xbfd446f7, 33 },
    { 0xbc3446f9, 95 },
    { 0, 0 },
    { 0x5c5446fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c446fe, 23 },
    { 0xaed446ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a44704, 90 },
    { 0x57644705, 85 },
    { 0, 0 },
    { 0x91844707, 55 },
    { 0, 0 },
    { 0x86244709, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b64470f, 23 },
    { 0xdd944710, 55 },
    { 0x8764470f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef44718, 78 },
    { 0, 0 },
    { 0xa5a4471a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454471d, 17 },
    { 0xc8b4471e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c44725, 76 },
    { 0xe4044726, 23 },
    { 0x68544727, 33 },
    { 0x4444728, 90 },
    { 0xa3d44729, 43 },
    { 0xe2744727, 52 },
    { 0x5d844729, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4472f, 23 },
    { 0, 0 },
    { 0xfc944731, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98244735, 76 },
    { 0xdf944735, 90 },
    { 0, 0 },
    { 0xed444738, 61 },
    { 0xf2444739, 34 },
    { 0x6804473a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c24473d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8944741, 88 },
    { 0x40444741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b44745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294474b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944757, 66 },
    { 0xe0344757, 23 },
    { 0, 0 },
    { 0x1204475a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c44763, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86144768, 33 },
    { 0xec244769, 34 },
    { 0x63844769, 61 },
    { 0x60b4476b, 31 },
    { 0xdd04476b, 90 },
    { 0xf954476b, 61 },
    { 0xe754476b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db44773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb44777, 100 },
    { 0xa2c44777, 80 },
    { 0xa9544778, 23 },
    { 0xe644777, 33 },
    { 0x9f94477b, 34 },
    { 0x6004477a, 95 },
    { 0xc524477d, 90 },
    { 0x78b4477c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68244781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a144784, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa314478e, 61 },
    { 0, 0 },
    { 0x83e44790, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad344794, 55 },
    { 0x5ef44795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68044799, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd674479c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2447a4, 90 },
    { 0x1b2447a5, 33 },
    { 0x877447a6, 90 },
    { 0x88e447a6, 33 },
    { 0x9d6447a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c447ac, 61 },
    { 0x913447ac, 75 },
    { 0, 0 },
    { 0x2b447af, 93 },
    { 0x4b7447af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af447b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f447c2, 23 },
    { 0, 0 },
    { 0xe54447c4, 98 },
    { 0xf9f447c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea447c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c447cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd447d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452447d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547447e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde447e8, 52 },
    { 0, 0 },
    { 0xa01447ea, 98 },
    { 0, 0 },
    { 0xd5a447ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559447f1, 100 },
    { 0x1df447f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1447f5, 55 },
    { 0xc6c447f6, 90 },
    { 0xfd7447f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc447fa, 90 },
    { 0x4d4447fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98444800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd444803, 88 },
    { 0xbc644804, 95 },
    { 0xe9a44805, 46 },
    { 0xf144804, 66 },
    { 0x52d44807, 98 },
    { 0, 0 },
    { 0xf9144809, 61 },
    { 0x9b544809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a144812, 90 },
    { 0x61c44813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41344818, 98 },
    { 0x29a44819, 34 },
    { 0x2c04481a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd44824, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34482a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6514482e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade44834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4604483a, 33 },
    { 0, 0 },
    { 0xbef4483c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da44840, 76 },
    { 0, 0 },
    { 0xdb044842, 33 },
    { 0xa4144843, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e44847, 81 },
    { 0xeaa44847, 55 },
    { 0x1b644849, 23 },
    { 0xc6b44849, 95 },
    { 0xe2b4484a, 33 },
    { 0xca4484a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6574484f, 98 },
    { 0xaee44850, 85 },
    { 0xc6144851, 90 },
    { 0xfce4484f, 69 },
    { 0x804484f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804485b, 23 },
    { 0x9cc4485c, 98 },
    { 0x4894485c, 61 },
    { 0xd1b4485e, 91 },
    { 0, 0 },
    { 0xf1d44860, 90 },
    { 0, 0 },
    { 0xe6444862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94486b, 23 },
    { 0x9c54486c, 98 },
    { 0x3754486d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c544872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae44881, 33 },
    { 0x88044882, 23 },
    { 0x34b44883, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6644886, 95 },
    { 0x4bd44887, 61 },
    { 0xdb44888, 55 },
    { 0xcb844887, 90 },
    { 0, 0 },
    { 0x17b4488b, 98 },
    { 0xeac4488c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11844890, 90 },
    { 0x49244891, 33 },
    { 0, 0 },
    { 0x54d44893, 76 },
    { 0x51c44894, 95 },
    { 0xf6a44894, 88 },
    { 0xbba44893, 98 },
    { 0x15e44896, 96 },
    { 0x1f344896, 43 },
    { 0xbfe44899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf394489e, 96 },
    { 0xb24489e, 40 },
    { 0x1f74489f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64448a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x691448a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec448af, 33 },
    { 0x5e4448af, 80 },
    { 0, 0 },
    { 0x85e448b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24448b6, 43 },
    { 0x7fe448b7, 55 },
    { 0xc50448b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644448c1, 90 },
    { 0, 0 },
    { 0xd3c448c3, 76 },
    { 0, 0 },
    { 0xce0448c5, 96 },
    { 0, 0 },
    { 0xee5448c7, 61 },
    { 0, 0 },
    { 0xfb1448c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x688448cc, 52 },
    { 0, 0 },
    { 0xad448ce, 100 },
    { 0, 0 },
    { 0xf4b448d0, 98 },
    { 0x62c448d1, 61 },
    { 0, 0 },
    { 0xf6d448d3, 81 },
    { 0, 0 },
    { 0x695448d5, 99 },
    { 0xbc4448d5, 78 },
    { 0, 0 },
    { 0x658448d8, 46 },
    { 0xa65448d9, 90 },
    { 0x638448da, 64 },
    { 0, 0 },
    { 0x148448dc, 67 },
    { 0xcab448dc, 61 },
    { 0, 0 },
    { 0x75b448df, 68 },
    { 0xbc0448e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8448e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39448e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b448ea, 61 },
    { 0, 0 },
    { 0x82448ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd448ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80448f4, 87 },
    { 0, 0 },
    { 0xbd5448f6, 61 },
    { 0xf0f448f7, 52 },
    { 0x624448f6, 90 },
    { 0x761448f8, 67 },
    { 0xf84448fa, 33 },
    { 0xee0448fb, 95 },
    { 0, 0 },
    { 0x50d448fd, 61 },
    { 0xeee448fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b44903, 46 },
    { 0, 0 },
    { 0x16a44905, 84 },
    { 0xa6b44906, 33 },
    { 0x8cb44905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb4490a, 90 },
    { 0, 0 },
    { 0x64490c, 61 },
    { 0x5cc4490d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7144911, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48044914, 61 },
    { 0x26d44914, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c04491b, 90 },
    { 0xdba4491b, 98 },
    { 0, 0 },
    { 0x8454491e, 33 },
    { 0x6ca4491f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58744925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c4492a, 87 },
    { 0, 0 },
    { 0x5bc4492c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73244930, 69 },
    { 0x2bb44931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86744939, 61 },
    { 0x57d44939, 46 },
    { 0xdd04493b, 61 },
    { 0xc4493c, 98 },
    { 0x4e64493d, 93 },
    { 0xb9d4493d, 40 },
    { 0x85d4493f, 61 },
    { 0x7e344939, 100 },
    { 0x46244941, 55 },
    { 0x80944942, 78 },
    { 0x13044942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4494c, 90 },
    { 0x174494d, 52 },
    { 0x764494e, 64 },
    { 0x6414494f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd44953, 46 },
    { 0, 0 },
    { 0x72644955, 34 },
    { 0xe3844956, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd4495a, 90 },
    { 0x8d64495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864495e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5644961, 33 },
    { 0x7b44962, 61 },
    { 0x94744963, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844496a, 77 },
    { 0, 0 },
    { 0x90a4496c, 65 },
    { 0x2734496d, 90 },
    { 0x8424496d, 90 },
    { 0x85e4496f, 55 },
    { 0x64a4496f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e944976, 53 },
    { 0, 0 },
    { 0x7e844978, 95 },
    { 0x3d944978, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12944983, 85 },
    { 0x36f44984, 90 },
    { 0x23d44985, 99 },
    { 0x68044986, 95 },
    { 0x4bb44987, 55 },
    { 0xa3a44988, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa974498c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf524498f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14499a, 66 },
    { 0x9854499b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e4499e, 46 },
    { 0, 0 },
    { 0x5a6449a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881449a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5449af, 93 },
    { 0x97d449b0, 33 },
    { 0xfba449b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637449b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19449ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xded449bd, 90 },
    { 0x94b449bd, 93 },
    { 0xfb1449bd, 76 },
    { 0xc99449c0, 90 },
    { 0xb80449c0, 61 },
    { 0x74d449bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x907449c5, 76 },
    { 0, 0 },
    { 0x621449c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea449ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63449d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95449d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4449d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c449db, 33 },
    { 0x7e6449dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78449df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4449e7, 90 },
    { 0x92d449e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268449ed, 98 },
    { 0x44449ee, 91 },
    { 0xdb3449ef, 82 },
    { 0x2be449f0, 40 },
    { 0x98f449f1, 98 },
    { 0x946449f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213449f6, 98 },
    { 0x576449f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x137449fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8449ff, 61 },
    { 0x9c744a00, 43 },
    { 0, 0 },
    { 0x21f44a02, 98 },
    { 0x45c44a02, 61 },
    { 0, 0 },
    { 0x7e344a05, 77 },
    { 0x38044a06, 76 },
    { 0, 0 },
    { 0xf2c44a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2544a10, 17 },
    { 0, 0 },
    { 0xeda44a12, 61 },
    { 0, 0 },
    { 0x53044a14, 33 },
    { 0xac144a14, 91 },
    { 0x8e44a16, 99 },
    { 0x33244a17, 68 },
    { 0x25144a18, 33 },
    { 0xeed44a19, 46 },
    { 0, 0 },
    { 0xdc44a1b, 90 },
    { 0xcfd44a1b, 90 },
    { 0xed44a1c, 40 },
    { 0, 0 },
    { 0xb1844a1f, 33 },
    { 0x78944a20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2944a25, 81 },
    { 0xf7144a25, 23 },
    { 0, 0 },
    { 0xf1944a28, 33 },
    { 0, 0 },
    { 0xc4744a2a, 33 },
    { 0xf3544a2b, 46 },
    { 0x40744a2c, 34 },
    { 0x2a444a2c, 61 },
    { 0, 0 },
    { 0x87344a2f, 76 },
    { 0xf0c44a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f044a34, 91 },
    { 0x1c944a35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53244a3b, 76 },
    { 0xd6144a3c, 61 },
    { 0x2b344a3d, 23 },
    { 0xbf644a3e, 33 },
    { 0x96544a3e, 98 },
    { 0x15d44a40, 55 },
    { 0, 0 },
    { 0xa3544a42, 90 },
    { 0x2ef44a42, 90 },
    { 0, 0 },
    { 0x26a44a45, 67 },
    { 0xa1344a45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35944a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1044a52, 33 },
    { 0, 0 },
    { 0x74a44a54, 23 },
    { 0, 0 },
    { 0x59644a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff44a5a, 34 },
    { 0x18344a5b, 23 },
    { 0xe8444a5c, 93 },
    { 0, 0 },
    { 0xe1b44a5e, 90 },
    { 0, 0 },
    { 0x81b44a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41444a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d44a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f344a6e, 90 },
    { 0x8b44a6f, 61 },
    { 0x62744a6f, 98 },
    { 0x21944a6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f144a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e44a81, 33 },
    { 0x8c244a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab544a86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc044a8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed144a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd44a90, 34 },
    { 0, 0 },
    { 0xae44a92, 33 },
    { 0xd9f44a93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3844a96, 85 },
    { 0x8f944a97, 91 },
    { 0x64d44a98, 46 },
    { 0xdf544a97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b44a9d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85144aa2, 84 },
    { 0xfca44aa3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e44aa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae144aab, 61 },
    { 0xbb344aac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c44ab6, 98 },
    { 0, 0 },
    { 0xcd444ab8, 85 },
    { 0x89a44ab9, 23 },
    { 0x2da44aba, 76 },
    { 0x2e44abb, 55 },
    { 0, 0 },
    { 0xbca44abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d544ac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58444ac5, 95 },
    { 0xf0f44ac5, 55 },
    { 0, 0 },
    { 0x31144ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b344acb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf844ad0, 61 },
    { 0xa5f44ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea544ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36544adb, 90 },
    { 0x81044adb, 98 },
    { 0x30c44add, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a44ae8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba44aec, 61 },
    { 0, 0 },
    { 0x2dd44aee, 61 },
    { 0x6fe44aef, 93 },
    { 0, 0 },
    { 0x64a44af1, 97 },
    { 0x2c144af2, 52 },
    { 0x6f944af3, 84 },
    { 0xc3744af3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9744afb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa44b00, 85 },
    { 0, 0 },
    { 0x85444b02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee644b0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e944b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e44b16, 76 },
    { 0xfa544b17, 65 },
    { 0xbfc44b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a44b1d, 98 },
    { 0x3af44b1d, 95 },
    { 0x16644b1f, 68 },
    { 0x4da44b1f, 17 },
    { 0x5cd44b20, 19 },
    { 0xf944b22, 55 },
    { 0x8a644b1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa44b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc44b2d, 34 },
    { 0x8af44b2e, 61 },
    { 0x5eb44b2f, 33 },
    { 0xcde44b30, 92 },
    { 0xf1244b30, 61 },
    { 0xc2244b32, 33 },
    { 0x32644b33, 23 },
    { 0xdd044b2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c44b38, 46 },
    { 0, 0 },
    { 0x71b44b3a, 46 },
    { 0xd844b3b, 90 },
    { 0x75144b3b, 33 },
    { 0xa344b3c, 43 },
    { 0x18f44b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e44b41, 98 },
    { 0x80644b42, 90 },
    { 0xdb044b43, 90 },
    { 0x36144b43, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf044b47, 33 },
    { 0xd1944b48, 52 },
    { 0x38344b47, 85 },
    { 0x24d44b4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf344b4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d44b50, 76 },
    { 0xcde44b50, 99 },
    { 0xd6d44b52, 98 },
    { 0x69344b51, 55 },
    { 0xdb744b54, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28044b59, 33 },
    { 0, 0 },
    { 0x74544b5b, 95 },
    { 0x2e44b5b, 61 },
    { 0, 0 },
    { 0x7a944b5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad44b61, 87 },
    { 0xbc144b62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2644b65, 68 },
    { 0x2c244b66, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9544b69, 55 },
    { 0, 0 },
    { 0xe9244b6b, 23 },
    { 0x9f344b6b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x22844b6f, 91 },
    { 0x5ba44b70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59244b74, 52 },
    { 0xba344b75, 95 },
    { 0, 0 },
    { 0xadb44b77, 90 },
    { 0x7dc44b78, 61 },
    { 0x82244b78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb44b7e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a44b84, 90 },
    { 0xf5344b85, 39 },
    { 0x50744b86, 43 },
    { 0, 0 },
    { 0x71c44b88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf844b93, 33 },
    { 0x70a44b94, 75 },
    { 0x9ce44b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e44b99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe44b9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a44ba2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f044ba7, 95 },
    { 0xec144ba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b44bac, 90 },
    { 0, 0 },
    { 0xa3844bae, 78 },
    { 0x5d144baf, 33 },
    { 0x70a44bb0, 91 },
    { 0x92944bb0, 91 },
    { 0xf344bb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9244bb6, 98 },
    { 0, 0 },
    { 0x87e44bb8, 78 },
    { 0xa0944bb8, 55 },
    { 0xd7a44bb9, 65 },
    { 0xd4044bbb, 55 },
    { 0xa1044bbc, 98 },
    { 0xfa44bbd, 68 },
    { 0x53544bbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd744bc4, 52 },
    { 0, 0 },
    { 0x58144bc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c44bc9, 93 },
    { 0, 0 },
    { 0x4dc44bcb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2744bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8344bd5, 88 },
    { 0xfea44bd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c44bda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5044be0, 98 },
    { 0, 0 },
    { 0x10c44be2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d744be5, 33 },
    { 0xb1e44be6, 91 },
    { 0x7a944be7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad644bef, 23 },
    { 0xfc544bef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9644bf4, 85 },
    { 0, 0 },
    { 0x81f44bf6, 52 },
    { 0xcd544bf7, 98 },
    { 0x32344bf8, 90 },
    { 0x39344bf6, 90 },
    { 0x2d244bf6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae44bfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b244c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c544c03, 34 },
    { 0x68f44c04, 33 },
    { 0, 0 },
    { 0x2cb44c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa44c0b, 23 },
    { 0x9d844c0c, 90 },
    { 0x57d44c0d, 81 },
    { 0x1a344c0e, 33 },
    { 0x64644c0c, 17 },
    { 0xe5544c0f, 33 },
    { 0x35f44c11, 95 },
    { 0x27144c11, 83 },
    { 0xb2c44c13, 33 },
    { 0xe0a44c0e, 93 },
    { 0xe4344c15, 90 },
    { 0x77e44c16, 33 },
    { 0xb4444c16, 90 },
    { 0xaa344c17, 61 },
    { 0x9bc44c19, 100 },
    { 0x94644c1a, 95 },
    { 0xd1f44c18, 33 },
    { 0, 0 },
    { 0xb3e44c1d, 61 },
    { 0x1fb44c1e, 23 },
    { 0, 0 },
    { 0x71e44c20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf44c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba44c29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6744c2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d44c31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b544c34, 90 },
    { 0x39644c35, 95 },
    { 0x7fd44c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed44c3d, 23 },
    { 0xcd644c3e, 23 },
    { 0xd5144c3d, 95 },
    { 0, 0 },
    { 0x1744c41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d044c44, 55 },
    { 0xd4344c45, 66 },
    { 0, 0 },
    { 0xfae44c47, 46 },
    { 0x77144c48, 78 },
    { 0xa6944c49, 61 },
    { 0x1044c4a, 61 },
    { 0xa1b44c4a, 76 },
    { 0, 0 },
    { 0xbc44c4d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f944c51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a44c55, 76 },
    { 0, 0 },
    { 0x19044c57, 98 },
    { 0x1b244c58, 95 },
    { 0x60944c59, 33 },
    { 0xa844c5a, 90 },
    { 0, 0 },
    { 0xd9d44c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf844c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0944c63, 98 },
    { 0x36244c64, 34 },
    { 0x5e144c64, 95 },
    { 0x33744c66, 34 },
    { 0xbd944c67, 95 },
    { 0x35844c68, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a44c70, 61 },
    { 0x67744c71, 69 },
    { 0xb1d44c71, 46 },
    { 0x5d144c72, 34 },
    { 0xabb44c74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944c7c, 90 },
    { 0x53044c7d, 46 },
    { 0, 0 },
    { 0x82544c7f, 23 },
    { 0x74c44c80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab44c84, 90 },
    { 0xb8344c85, 90 },
    { 0xd6f44c85, 25 },
    { 0xa6644c87, 90 },
    { 0, 0 },
    { 0x98844c89, 61 },
    { 0, 0 },
    { 0x39344c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d44c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a44c94, 55 },
    { 0x97f44c94, 61 },
    { 0, 0 },
    { 0x7bb44c97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c944c9c, 65 },
    { 0x9bc44c9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c344ca0, 85 },
    { 0xc1d44ca1, 23 },
    { 0xf7844ca1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51544ca7, 90 },
    { 0xf44ca8, 99 },
    { 0x5a844ca9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0144caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b444cb4, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b044cb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77344cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad044cc2, 90 },
    { 0x34744cc3, 55 },
    { 0x1ef44cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a144cca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a44ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde544cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd44cd8, 95 },
    { 0, 0 },
    { 0xac944cda, 55 },
    { 0x54844cdb, 23 },
    { 0xbc544cdc, 23 },
    { 0xe2c44cdc, 98 },
    { 0, 0 },
    { 0xd2644cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b144ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb44ce7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d44cec, 95 },
    { 0xc9944ced, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f44cf0, 90 },
    { 0x78e44cf1, 90 },
    { 0xfde44cf2, 75 },
    { 0x5a44cf3, 98 },
    { 0, 0 },
    { 0xeec44cf5, 98 },
    { 0, 0 },
    { 0xe1644cf7, 61 },
    { 0x87044cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e44cfb, 61 },
    { 0xf3944cfc, 76 },
    { 0xd2644cfc, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80844d07, 85 },
    { 0, 0 },
    { 0x97544d09, 61 },
    { 0, 0 },
    { 0x1a844d0b, 17 },
    { 0xf1c44d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75244d11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc244d14, 61 },
    { 0, 0 },
    { 0x4bd44d16, 84 },
    { 0xaa644d16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f844d1b, 76 },
    { 0x40d44d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab544d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28244d23, 95 },
    { 0x1bd44d23, 90 },
    { 0x53e44d25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d44d2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1244d2d, 61 },
    { 0, 0 },
    { 0x27e44d2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78944d32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd44d36, 52 },
    { 0xaef44d36, 75 },
    { 0xa8044d38, 84 },
    { 0x40244d39, 100 },
    { 0x33044d39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x38044d3d, 84 },
    { 0xde144d3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c844d41, 33 },
    { 0x98144d42, 33 },
    { 0x2f344d43, 98 },
    { 0x1b844d43, 98 },
    { 0xf3c44d45, 98 },
    { 0, 0 },
    { 0xd8344d47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac544d4b, 33 },
    { 0x20744d4c, 90 },
    { 0, 0 },
    { 0x5ed44d4e, 66 },
    { 0xd4c44d4f, 95 },
    { 0x17c44d50, 61 },
    { 0xd1544d51, 98 },
    { 0x15644d52, 29 },
    { 0x2e644d53, 55 },
    { 0x22944d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12944d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a44d5a, 55 },
    { 0, 0 },
    { 0x51f44d5c, 23 },
    { 0x70444d5d, 95 },
    { 0xd9644d5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f844d68, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad44d70, 55 },
    { 0xcb644d71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb744d79, 76 },
    { 0x44344d7a, 34 },
    { 0x33644d7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee44d7e, 95 },
    { 0x46c44d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94444d85, 90 },
    { 0x87f44d86, 78 },
    { 0x31844d87, 90 },
    { 0, 0 },
    { 0xb5a44d89, 90 },
    { 0x29044d8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd44d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43444d91, 61 },
    { 0x4e244d92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34044d95, 90 },
    { 0x2f944d96, 52 },
    { 0xee544d96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5644d9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe144da5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc444da9, 75 },
    { 0x65c44daa, 61 },
    { 0xe5644dab, 93 },
    { 0x4cf44dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f344daf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39444db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f44dbb, 100 },
    { 0, 0 },
    { 0x74644dbd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51144dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d044dcc, 93 },
    { 0, 0 },
    { 0x6ba44dce, 33 },
    { 0xa2944dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59644dd2, 61 },
    { 0xcca44dd2, 90 },
    { 0, 0 },
    { 0xc3b44dd5, 90 },
    { 0x6cd44dd6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c44dd9, 61 },
    { 0, 0 },
    { 0x6ce44ddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6444ddf, 36 },
    { 0, 0 },
    { 0x9d444de1, 98 },
    { 0xea944de2, 17 },
    { 0xf2144de3, 23 },
    { 0x48f44de2, 90 },
    { 0x35044de2, 90 },
    { 0xb3344de2, 95 },
    { 0, 0 },
    { 0xba144de8, 61 },
    { 0xeef44de9, 92 },
    { 0, 0 },
    { 0x1d44deb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc44dee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d644df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e744df5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6744df8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa144dfb, 55 },
    { 0, 0 },
    { 0x3fb44dfd, 84 },
    { 0, 0 },
    { 0x33344dff, 80 },
    { 0x2c144e00, 90 },
    { 0xdcb44e01, 78 },
    { 0x7ca44e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb44e05, 23 },
    { 0x20244e05, 90 },
    { 0xb2944e07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c044e0b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b44e0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa244e15, 91 },
    { 0xdc144e15, 23 },
    { 0x6df44e16, 61 },
    { 0xd44e18, 90 },
    { 0xd9144e19, 76 },
    { 0xbfa44e15, 53 },
    { 0, 0 },
    { 0xec544e1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63944e21, 34 },
    { 0x83c44e22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c844e25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c44e29, 55 },
    { 0x77744e29, 33 },
    { 0x98644e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81044e36, 23 },
    { 0x8ea44e37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4344e3b, 77 },
    { 0xfb644e3b, 61 },
    { 0x25144e3d, 34 },
    { 0, 0 },
    { 0x9af44e3f, 66 },
    { 0x7444e40, 23 },
    { 0xd4144e41, 55 },
    { 0xcfb44e40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40144e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b744e4c, 33 },
    { 0x2ac44e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22944e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c844e58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc044e5b, 36 },
    { 0x83144e5c, 88 },
    { 0x52f44e5d, 61 },
    { 0, 0 },
    { 0xa2344e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0144e63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f844e69, 80 },
    { 0xb144e69, 33 },
    { 0x64a44e6b, 87 },
    { 0x37344e6b, 55 },
    { 0xe5544e6a, 33 },
    { 0x59344e6d, 33 },
    { 0x78b44e6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef644e76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f44e7d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1244e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5144e88, 49 },
    { 0, 0 },
    { 0x60b44e8a, 52 },
    { 0, 0 },
    { 0xabd44e8c, 100 },
    { 0x8e044e8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac44e91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49344e95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044e98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf44e9b, 91 },
    { 0, 0 },
    { 0xa0544e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61444ea2, 90 },
    { 0, 0 },
    { 0xcf444ea4, 23 },
    { 0x1f744ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15844eb0, 98 },
    { 0, 0 },
    { 0xf3944eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da44eb6, 67 },
    { 0x11f44eb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e44ec0, 76 },
    { 0, 0 },
    { 0x22744ec2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c44ec7, 93 },
    { 0, 0 },
    { 0x53844ec9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d044ece, 46 },
    { 0, 0 },
    { 0xf5244ed0, 95 },
    { 0, 0 },
    { 0x29344ed2, 66 },
    { 0xf9e44ed2, 90 },
    { 0xe4b44ed2, 100 },
    { 0x94244ed5, 76 },
    { 0x6744ed5, 53 },
    { 0xcc644ed6, 98 },
    { 0xc5844ed4, 90 },
    { 0xc0a44ed9, 55 },
    { 0x9e044eda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f44edd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc344ee1, 90 },
    { 0xb0344ee2, 90 },
    { 0xa2e44ee2, 90 },
    { 0xdf44ee3, 95 },
    { 0x9fb44ee3, 34 },
    { 0x60944ee6, 84 },
    { 0x94d44ee4, 46 },
    { 0xfd44ee8, 90 },
    { 0x9c144ee9, 43 },
    { 0x5ee44eea, 66 },
    { 0xc7944eea, 81 },
    { 0, 0 },
    { 0x6ec44eed, 95 },
    { 0xf2444eee, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4744ef6, 23 },
    { 0xcaf44ef7, 100 },
    { 0xfd44ef7, 33 },
    { 0, 0 },
    { 0x7c344efa, 52 },
    { 0x42b44efb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c44f00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7944f06, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d244f09, 67 },
    { 0xbc044f0a, 46 },
    { 0x21244f0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef544f0e, 98 },
    { 0xe0644f0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87244f14, 95 },
    { 0x37344f15, 97 },
    { 0x95144f16, 95 },
    { 0, 0 },
    { 0xab44f18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1144f1f, 95 },
    { 0, 0 },
    { 0x8a044f21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f44f24, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9144f29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83044f2c, 99 },
    { 0xda044f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b244f32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a44f36, 76 },
    { 0x34944f37, 96 },
    { 0xe6644f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76644f3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af44f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38944f46, 66 },
    { 0x12c44f47, 99 },
    { 0x21b44f48, 66 },
    { 0x3df44f47, 34 },
    { 0, 0 },
    { 0xc4144f4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea344f51, 34 },
    { 0, 0 },
    { 0x86844f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe144f57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b44f5a, 96 },
    { 0x93b44f5b, 23 },
    { 0, 0 },
    { 0xcb644f5d, 61 },
    { 0, 0 },
    { 0xd7044f5f, 23 },
    { 0x67344f5f, 76 },
    { 0, 0 },
    { 0x61f44f62, 66 },
    { 0, 0 },
    { 0x8c244f64, 98 },
    { 0xb7844f64, 46 },
    { 0x1c744f66, 23 },
    { 0xecd44f66, 55 },
    { 0x79844f66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec44f6b, 56 },
    { 0xf4844f6c, 90 },
    { 0x62044f6b, 76 },
    { 0, 0 },
    { 0xc4444f6f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x92344f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f44f75, 23 },
    { 0xe5c44f75, 90 },
    { 0, 0 },
    { 0xaff44f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa44f7c, 90 },
    { 0, 0 },
    { 0x19444f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45544f82, 61 },
    { 0x33044f83, 90 },
    { 0x30244f84, 66 },
    { 0x41444f83, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a444f89, 84 },
    { 0x99244f8a, 61 },
    { 0x62c44f8b, 55 },
    { 0x68444f8c, 61 },
    { 0xcf644f8d, 91 },
    { 0xc9c44f8c, 98 },
    { 0x44644f8e, 66 },
    { 0xd4544f8e, 95 },
    { 0xa3c44f91, 46 },
    { 0, 0 },
    { 0x18344f93, 87 },
    { 0x32144f94, 76 },
    { 0x4044f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8944f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50544f9f, 78 },
    { 0xee444f9f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a44fa3, 90 },
    { 0x7ff44fa4, 33 },
    { 0x12744fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc444fad, 33 },
    { 0xcc144fae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad44fb2, 98 },
    { 0, 0 },
    { 0x24644fb4, 61 },
    { 0x86044fb4, 95 },
    { 0, 0 },
    { 0x22b44fb7, 52 },
    { 0x1d744fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f44fbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1844fc5, 90 },
    { 0, 0 },
    { 0xb4044fc7, 90 },
    { 0x70b44fc8, 17 },
    { 0x13544fc7, 33 },
    { 0x26344fc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f444fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c44fd7, 95 },
    { 0x62f44fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5544fdd, 33 },
    { 0xf9444fde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb344fe1, 61 },
    { 0, 0 },
    { 0x39544fe3, 90 },
    { 0x11544fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8544fe7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba44fea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf644fed, 65 },
    { 0, 0 },
    { 0x29244fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f44ff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18844ff8, 33 },
    { 0, 0 },
    { 0x9af44ffa, 26 },
    { 0x5fb44ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15745003, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef45006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a54500d, 61 },
    { 0xc124500e, 23 },
    { 0x7514500f, 76 },
    { 0x5344500e, 61 },
    { 0x5845010, 33 },
    { 0x6d94500e, 23 },
    { 0xdc54500d, 55 },
    { 0x1b245014, 75 },
    { 0xd445015, 80 },
    { 0x4e945016, 98 },
    { 0x40545017, 82 },
    { 0x3c45010, 67 },
    { 0x77645019, 34 },
    { 0xfee4501a, 93 },
    { 0xc3b4501b, 95 },
    { 0x6b74501c, 46 },
    { 0x78f4501c, 90 },
    { 0xe974501e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b45022, 98 },
    { 0, 0 },
    { 0xb3e45024, 59 },
    { 0x88445025, 98 },
    { 0, 0 },
    { 0x5845027, 76 },
    { 0x6d245027, 100 },
    { 0x6c345027, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4502e, 33 },
    { 0xb404502f, 90 },
    { 0x9e745030, 98 },
    { 0x8e345030, 66 },
    { 0, 0 },
    { 0xde445033, 55 },
    { 0, 0 },
    { 0x2e645035, 23 },
    { 0x46045035, 95 },
    { 0xaf745036, 90 },
    { 0, 0 },
    { 0xd2a45039, 90 },
    { 0x5af4503a, 33 },
    { 0xa4545039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb45041, 23 },
    { 0x6e145042, 61 },
    { 0xa2745043, 52 },
    { 0xe2045044, 23 },
    { 0xaa245041, 78 },
    { 0xb0845045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344504a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54504e, 23 },
    { 0, 0 },
    { 0x5d045050, 61 },
    { 0x1745051, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2245054, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f845057, 98 },
    { 0x7b45058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22845060, 55 },
    { 0, 0 },
    { 0xe2345062, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f45066, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e4506a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84506d, 33 },
    { 0, 0 },
    { 0x5114506f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2745073, 78 },
    { 0x7ce45073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1845079, 95 },
    { 0x6bd4507a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54507f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25845082, 96 },
    { 0xdcc45083, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87145086, 55 },
    { 0xb7345087, 40 },
    { 0x2b645088, 90 },
    { 0, 0 },
    { 0xf264508a, 61 },
    { 0xe024508b, 43 },
    { 0, 0 },
    { 0x5b54508d, 78 },
    { 0x78f4508e, 43 },
    { 0xfcb4508f, 61 },
    { 0xaf45090, 33 },
    { 0, 0 },
    { 0xf9745092, 95 },
    { 0x32845093, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba45099, 98 },
    { 0x5de45099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d04509d, 91 },
    { 0x80b4509e, 61 },
    { 0, 0 },
    { 0xe13450a0, 52 },
    { 0x1f3450a1, 91 },
    { 0x94450a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46450a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec450ad, 97 },
    { 0x1ed450ad, 23 },
    { 0, 0 },
    { 0x77450b0, 23 },
    { 0xbb6450b1, 52 },
    { 0x15e450b2, 96 },
    { 0x273450b2, 95 },
    { 0x980450b4, 91 },
    { 0x325450b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d450bd, 52 },
    { 0x657450be, 61 },
    { 0x1c9450bf, 76 },
    { 0x4f3450c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b450c3, 68 },
    { 0, 0 },
    { 0x1a5450c5, 33 },
    { 0x852450c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0450ca, 99 },
    { 0, 0 },
    { 0x451450cc, 85 },
    { 0x551450cd, 84 },
    { 0xb0f450ce, 53 },
    { 0xae8450cd, 34 },
    { 0x414450d0, 87 },
    { 0xc32450d1, 98 },
    { 0x3fd450ce, 91 },
    { 0xfa5450d3, 34 },
    { 0, 0 },
    { 0xb90450d5, 66 },
    { 0, 0 },
    { 0x84b450d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233450dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b450e4, 55 },
    { 0x6b4450e5, 68 },
    { 0xd88450e6, 35 },
    { 0x401450e6, 68 },
    { 0xb43450e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7450eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x809450ee, 90 },
    { 0, 0 },
    { 0x2c1450f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe450f6, 23 },
    { 0, 0 },
    { 0xa62450f8, 90 },
    { 0, 0 },
    { 0x8ee450fa, 87 },
    { 0x63a450fb, 33 },
    { 0, 0 },
    { 0xccb450fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d745103, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xca745106, 43 },
    { 0x74c45107, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc4510b, 43 },
    { 0x6664510b, 95 },
    { 0, 0 },
    { 0x2ad4510e, 75 },
    { 0x6954510e, 61 },
    { 0xf5c45110, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3345114, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea45120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee45125, 33 },
    { 0, 0 },
    { 0x94745127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2834512e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x20445131, 61 },
    { 0x5ae45131, 90 },
    { 0xa7745133, 90 },
    { 0x56345132, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24445138, 23 },
    { 0x1fe45139, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b4513d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4045140, 61 },
    { 0, 0 },
    { 0x5f845142, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e045148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1784514c, 90 },
    { 0, 0 },
    { 0xa6f4514e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a45158, 33 },
    { 0, 0 },
    { 0xa404515a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4515d, 52 },
    { 0, 0 },
    { 0x2624515f, 55 },
    { 0x3514515f, 95 },
    { 0xa4e45161, 95 },
    { 0x3c945162, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d45166, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454516d, 98 },
    { 0xdd24516d, 61 },
    { 0x75b4516f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a945172, 98 },
    { 0x34045173, 93 },
    { 0x22545173, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d45183, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd45188, 55 },
    { 0x59c45189, 43 },
    { 0x66c4518a, 99 },
    { 0x5594518b, 90 },
    { 0x41c4518b, 43 },
    { 0x3384518a, 91 },
    { 0xe594518e, 61 },
    { 0, 0 },
    { 0x47445190, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc45193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea45198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b4519d, 95 },
    { 0x7234519d, 90 },
    { 0x73b4519d, 55 },
    { 0, 0 },
    { 0xc77451a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x373451a4, 76 },
    { 0x75d451a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667451aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921451b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa451bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385451c6, 98 },
    { 0, 0 },
    { 0x8e8451c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7451cc, 77 },
    { 0, 0 },
    { 0x3f7451ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d451d3, 98 },
    { 0xfb5451d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc451d8, 88 },
    { 0x5d451d9, 33 },
    { 0x3b2451d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f451df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2451ea, 90 },
    { 0x6fd451ea, 90 },
    { 0x346451ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x499451ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8451f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555451f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8451fc, 33 },
    { 0x1ac451fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d45200, 61 },
    { 0, 0 },
    { 0x8c745202, 33 },
    { 0xdc645203, 55 },
    { 0xb2d45204, 92 },
    { 0xff845205, 85 },
    { 0x77445206, 34 },
    { 0x5ff45206, 76 },
    { 0x55245207, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b4520c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a45215, 95 },
    { 0x84445216, 98 },
    { 0xba845217, 23 },
    { 0, 0 },
    { 0xdec45219, 90 },
    { 0, 0 },
    { 0x1d54521b, 88 },
    { 0x5d14521c, 46 },
    { 0, 0 },
    { 0xe214521e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x47445221, 84 },
    { 0x4d345222, 93 },
    { 0x97d45222, 61 },
    { 0x66845224, 46 },
    { 0xb4245225, 46 },
    { 0, 0 },
    { 0xe3045227, 87 },
    { 0x99345227, 98 },
    { 0, 0 },
    { 0x53e4522a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8864522f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a45233, 98 },
    { 0x3a545233, 90 },
    { 0x54745234, 90 },
    { 0x9e145236, 91 },
    { 0xa8545237, 33 },
    { 0x3145237, 27 },
    { 0x7d745234, 98 },
    { 0, 0 },
    { 0xfce4523b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54523f, 33 },
    { 0x21c45240, 55 },
    { 0x30845241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a545246, 66 },
    { 0x33845246, 34 },
    { 0, 0 },
    { 0xa0445249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4524d, 85 },
    { 0, 0 },
    { 0x72c4524f, 84 },
    { 0, 0 },
    { 0xcc845251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c45258, 66 },
    { 0x2c245258, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc45262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4945269, 61 },
    { 0x2704526a, 61 },
    { 0x1cd45269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32045274, 46 },
    { 0, 0 },
    { 0xa7b45276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf84527b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b145282, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90045285, 98 },
    { 0x62045285, 61 },
    { 0, 0 },
    { 0x6845288, 33 },
    { 0x21d45289, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f4528c, 68 },
    { 0x96f4528d, 61 },
    { 0xcf34528c, 90 },
    { 0, 0 },
    { 0x58845290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88452a1, 43 },
    { 0xf2452a2, 95 },
    { 0x2e4452a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860452ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf452b2, 61 },
    { 0, 0 },
    { 0x105452b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02452b7, 78 },
    { 0xb58452b8, 95 },
    { 0xcf1452b8, 66 },
    { 0xbc8452ba, 23 },
    { 0xec0452bb, 55 },
    { 0x119452bc, 87 },
    { 0xa0e452bd, 95 },
    { 0xdf2452bb, 73 },
    { 0xa2e452bf, 33 },
    { 0xda8452c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5452c3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a452c6, 90 },
    { 0x113452c6, 87 },
    { 0, 0 },
    { 0x78f452c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798452cd, 52 },
    { 0xd8d452ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc452d6, 55 },
    { 0, 0 },
    { 0xfc6452d8, 23 },
    { 0xf6e452d9, 90 },
    { 0x94b452d9, 98 },
    { 0x788452d9, 98 },
    { 0x53e452d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb452df, 23 },
    { 0xb43452e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf452e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70452ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df452ed, 90 },
    { 0x1f452ee, 90 },
    { 0xdfb452ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x400452f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44452ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b445306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c44530a, 98 },
    { 0x9c04530b, 23 },
    { 0, 0 },
    { 0x9804530d, 17 },
    { 0x75a4530e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e45311, 68 },
    { 0x2eb45311, 87 },
    { 0xad45311, 17 },
    { 0xc0e45314, 61 },
    { 0xc1e45313, 17 },
    { 0, 0 },
    { 0x35145317, 95 },
    { 0xd6945318, 23 },
    { 0x46b45319, 95 },
    { 0, 0 },
    { 0x4934531b, 46 },
    { 0xbe04531c, 61 },
    { 0xda14531d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d045321, 33 },
    { 0, 0 },
    { 0x3d945323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a45329, 67 },
    { 0, 0 },
    { 0x8074532b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3745331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f945334, 34 },
    { 0, 0 },
    { 0xf1f45336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4533d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6745341, 66 },
    { 0x92545341, 68 },
    { 0, 0 },
    { 0x37e45344, 95 },
    { 0, 0 },
    { 0x3ce45346, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd964534a, 33 },
    { 0x67c4534a, 23 },
    { 0x934534c, 95 },
    { 0x3374534b, 75 },
    { 0x2854534e, 98 },
    { 0, 0 },
    { 0x2a45350, 93 },
    { 0xc7245351, 55 },
    { 0x8ce45352, 76 },
    { 0x2e545352, 76 },
    { 0x87145354, 33 },
    { 0xd4e45353, 84 },
    { 0, 0 },
    { 0x3b945357, 81 },
    { 0xc9f45358, 33 },
    { 0x1b745358, 61 },
    { 0, 0 },
    { 0x7774535b, 33 },
    { 0x89a4535c, 95 },
    { 0xe24535d, 61 },
    { 0x2cf4535d, 95 },
    { 0x2cb4535c, 90 },
    { 0, 0 },
    { 0x9fc45361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b845364, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f45367, 61 },
    { 0xb6445368, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3234536d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a45370, 33 },
    { 0xba545371, 90 },
    { 0x24145371, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73545375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee4537b, 76 },
    { 0, 0 },
    { 0x6694537d, 61 },
    { 0x3ac4537e, 33 },
    { 0x8a04537d, 61 },
    { 0xa604537f, 90 },
    { 0x2df45380, 90 },
    { 0x32b4537d, 95 },
    { 0xd6d45383, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2445388, 76 },
    { 0x2c845389, 37 },
    { 0x8054538a, 23 },
    { 0, 0 },
    { 0x31a4538c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc545394, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e4539a, 90 },
    { 0xcb54539b, 65 },
    { 0xb694539c, 91 },
    { 0xb4539b, 100 },
    { 0, 0 },
    { 0x4214539f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb453a3, 85 },
    { 0, 0 },
    { 0x4c9453a5, 34 },
    { 0, 0 },
    { 0x4d7453a7, 78 },
    { 0x3ae453a8, 90 },
    { 0x280453a9, 43 },
    { 0xee9453a7, 95 },
    { 0xe2c453ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c453b1, 33 },
    { 0, 0 },
    { 0xc10453b3, 98 },
    { 0, 0 },
    { 0x48453b5, 61 },
    { 0x842453b6, 55 },
    { 0, 0 },
    { 0xc65453b8, 90 },
    { 0, 0 },
    { 0x1f8453ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b453be, 95 },
    { 0x49453bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1453c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9453cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30453cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa453d7, 52 },
    { 0x6a9453d8, 95 },
    { 0xa24453d9, 66 },
    { 0, 0 },
    { 0xf9f453db, 33 },
    { 0xd6b453db, 48 },
    { 0x206453dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x784453e0, 55 },
    { 0xb59453e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec453e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768453e8, 93 },
    { 0x796453e9, 61 },
    { 0x211453ea, 66 },
    { 0x432453eb, 90 },
    { 0xe24453ec, 66 },
    { 0xf86453ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09453f1, 93 },
    { 0xce2453f2, 97 },
    { 0xc45453f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf453f7, 93 },
    { 0xc3d453f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff45400, 55 },
    { 0, 0 },
    { 0x66645402, 61 },
    { 0x46045403, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace45408, 98 },
    { 0xa3a45408, 43 },
    { 0xd274540a, 93 },
    { 0, 0 },
    { 0x3a24540c, 85 },
    { 0xe0f4540d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89745410, 75 },
    { 0x81145410, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x85945414, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a45419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a4541c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99845420, 34 },
    { 0xd5845421, 98 },
    { 0x41645422, 23 },
    { 0x72345423, 95 },
    { 0x15445424, 95 },
    { 0x68e45424, 43 },
    { 0x2f645426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x88345429, 91 },
    { 0, 0 },
    { 0x1c74542b, 98 },
    { 0x8b24542b, 33 },
    { 0x64c4542d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd45430, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda045438, 91 },
    { 0x9945439, 34 },
    { 0xed24543a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c4543d, 95 },
    { 0xbc84543e, 52 },
    { 0x9124543f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3145442, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae4544b, 46 },
    { 0x2bf4544c, 94 },
    { 0, 0 },
    { 0x8cf4544e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25345454, 95 },
    { 0xfb45455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c845458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf4545c, 91 },
    { 0, 0 },
    { 0x7604545e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d545461, 85 },
    { 0x68945462, 88 },
    { 0x76745461, 33 },
    { 0, 0 },
    { 0x5ff45465, 90 },
    { 0x33f45465, 43 },
    { 0xee945465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4546a, 87 },
    { 0, 0 },
    { 0xb754546c, 77 },
    { 0x8cc4546c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a45470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23645473, 76 },
    { 0, 0 },
    { 0xcac45475, 43 },
    { 0xc8745476, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa045479, 87 },
    { 0x7d45479, 33 },
    { 0x80e4547b, 55 },
    { 0xa5845479, 90 },
    { 0x48f4547d, 61 },
    { 0x6944547e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b245483, 81 },
    { 0x55745484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3845488, 66 },
    { 0x18f45488, 34 },
    { 0, 0 },
    { 0x2dd4548b, 91 },
    { 0, 0 },
    { 0x3b24548d, 33 },
    { 0x65b4548e, 95 },
    { 0x3314548d, 98 },
    { 0xff345490, 95 },
    { 0, 0 },
    { 0xc8d45492, 17 },
    { 0, 0 },
    { 0xc1445494, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab45497, 55 },
    { 0x51045497, 33 },
    { 0x42a45499, 93 },
    { 0x5494549a, 90 },
    { 0x4bf4549b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb414549f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8454a7, 78 },
    { 0x178454a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d454ae, 84 },
    { 0xfb8454af, 78 },
    { 0xb46454af, 61 },
    { 0xfa6454b1, 61 },
    { 0x940454b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2454b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e454b8, 43 },
    { 0x9ff454b8, 61 },
    { 0, 0 },
    { 0xe01454bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54454c0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43454c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353454cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4454ce, 61 },
    { 0xe72454cf, 33 },
    { 0, 0 },
    { 0x61e454d1, 23 },
    { 0x861454d2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d454d6, 99 },
    { 0, 0 },
    { 0x5ae454d8, 23 },
    { 0x45d454d8, 77 },
    { 0x74c454d8, 33 },
    { 0xc3d454db, 23 },
    { 0x29b454d8, 34 },
    { 0x818454dd, 90 },
    { 0x445454de, 61 },
    { 0xdf7454d8, 95 },
    { 0x5b8454da, 66 },
    { 0, 0 },
    { 0x230454e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a454e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96454f4, 95 },
    { 0x5d4454f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84454fb, 98 },
    { 0x91e454fc, 100 },
    { 0xffe454fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a345500, 61 },
    { 0x6d245501, 33 },
    { 0x11545501, 34 },
    { 0, 0 },
    { 0x5a445504, 100 },
    { 0xe3545504, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074550e, 66 },
    { 0, 0 },
    { 0x78e45510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd045513, 69 },
    { 0xef045514, 23 },
    { 0x5b545515, 35 },
    { 0, 0 },
    { 0xfc245517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a74551b, 56 },
    { 0xcab4551c, 95 },
    { 0x7524551d, 68 },
    { 0x9c64551e, 85 },
    { 0xed24551e, 61 },
    { 0xd424551e, 90 },
    { 0x7e54551e, 87 },
    { 0xe1045522, 95 },
    { 0x4df45523, 33 },
    { 0x75545523, 33 },
    { 0xd3c45525, 23 },
    { 0x46245523, 33 },
    { 0, 0 },
    { 0x94f45528, 23 },
    { 0, 0 },
    { 0xaf4552a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6224552e, 90 },
    { 0xa924552f, 91 },
    { 0x27045530, 90 },
    { 0xba94552f, 90 },
    { 0xe6045532, 62 },
    { 0x1f945533, 61 },
    { 0, 0 },
    { 0x3e245535, 78 },
    { 0x9d745536, 33 },
    { 0x33545536, 34 },
    { 0, 0 },
    { 0x7ae45539, 95 },
    { 0x7c74553a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a45540, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd45543, 76 },
    { 0xca845544, 98 },
    { 0x1b45544, 52 },
    { 0xd2345544, 76 },
    { 0x53545546, 91 },
    { 0xa1545547, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544554b, 23 },
    { 0, 0 },
    { 0x3934554d, 61 },
    { 0, 0 },
    { 0x26b4554f, 87 },
    { 0x6924554f, 98 },
    { 0x5bf45551, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xded45554, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c45557, 55 },
    { 0xd3345558, 90 },
    { 0xbf45558, 88 },
    { 0, 0 },
    { 0x77a4555b, 95 },
    { 0xc454555c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4555f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc45565, 99 },
    { 0x8b445566, 97 },
    { 0, 0 },
    { 0xdf345568, 95 },
    { 0xcc545568, 95 },
    { 0x63b4556a, 33 },
    { 0x5c74556a, 76 },
    { 0x1624556b, 100 },
    { 0xb064556a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78445578, 43 },
    { 0x75845579, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04557d, 76 },
    { 0x19b4557e, 80 },
    { 0x1674557f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb45582, 76 },
    { 0, 0 },
    { 0xe8045584, 33 },
    { 0, 0 },
    { 0xdc545586, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6214558d, 33 },
    { 0, 0 },
    { 0xd024558f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1045592, 48 },
    { 0x6fe45593, 83 },
    { 0x73a45594, 99 },
    { 0x8c145595, 90 },
    { 0x5c345596, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55945599, 23 },
    { 0x4634559a, 23 },
    { 0x7044559a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4559e, 23 },
    { 0xb824559f, 33 },
    { 0, 0 },
    { 0x9ed455a1, 55 },
    { 0x9b6455a2, 60 },
    { 0xe0f455a2, 67 },
    { 0xefa455a4, 90 },
    { 0x7be455a2, 99 },
    { 0x8f8455a6, 33 },
    { 0x829455a6, 98 },
    { 0x52455a8, 98 },
    { 0x6f5455a8, 98 },
    { 0xe14455a7, 33 },
    { 0xd84455a7, 33 },
    { 0xe4b455ac, 55 },
    { 0x561455ad, 53 },
    { 0x753455ae, 55 },
    { 0x452455ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d455b4, 68 },
    { 0x7fb455b5, 61 },
    { 0x67f455b4, 34 },
    { 0, 0 },
    { 0x335455b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994455bd, 61 },
    { 0, 0 },
    { 0x8455bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f455c5, 91 },
    { 0, 0 },
    { 0xe5d455c7, 46 },
    { 0x342455c8, 33 },
    { 0x793455c8, 43 },
    { 0, 0 },
    { 0x4bb455cb, 95 },
    { 0x20c455cc, 98 },
    { 0xe09455cd, 91 },
    { 0x4f2455ce, 95 },
    { 0x6fe455cf, 61 },
    { 0x417455cd, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe455d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe455dc, 43 },
    { 0, 0 },
    { 0x27455de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f455e2, 90 },
    { 0, 0 },
    { 0x667455e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26455ef, 58 },
    { 0x29455ef, 76 },
    { 0, 0 },
    { 0x187455f2, 46 },
    { 0x784455f3, 84 },
    { 0x6ec455f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11455f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b455fb, 88 },
    { 0xaf7455fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd945603, 43 },
    { 0, 0 },
    { 0x88545605, 91 },
    { 0, 0 },
    { 0xef445607, 43 },
    { 0x63545608, 88 },
    { 0xed345607, 51 },
    { 0xf4c4560a, 34 },
    { 0x5e74560b, 93 },
    { 0, 0 },
    { 0x714560d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b45612, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87545617, 76 },
    { 0xf1f45618, 61 },
    { 0xe5545619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f04561d, 61 },
    { 0xfa14561e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c45621, 33 },
    { 0xe0645622, 98 },
    { 0, 0 },
    { 0x43345624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51645627, 23 },
    { 0, 0 },
    { 0x99745629, 61 },
    { 0x77c4562a, 23 },
    { 0xc4a4562b, 23 },
    { 0x1945629, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x114562f, 33 },
    { 0xdcf4562f, 91 },
    { 0x44745631, 55 },
    { 0x89645630, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d45635, 98 },
    { 0, 0 },
    { 0x6ce45637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b4563d, 100 },
    { 0x5ba4563e, 43 },
    { 0x3bd4563e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0645648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda64564f, 52 },
    { 0xc9b4564f, 33 },
    { 0xd745651, 3 },
    { 0x2a545652, 23 },
    { 0x32845652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5245656, 61 },
    { 0, 0 },
    { 0x74745658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7324565b, 33 },
    { 0, 0 },
    { 0x5ba4565d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc145660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc745665, 88 },
    { 0xaa145665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb444566b, 33 },
    { 0x4904566c, 76 },
    { 0, 0 },
    { 0x5334566e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92745671, 61 },
    { 0xe4545672, 90 },
    { 0, 0 },
    { 0x20845674, 33 },
    { 0, 0 },
    { 0x80545676, 95 },
    { 0xac145676, 99 },
    { 0, 0 },
    { 0xd8345679, 95 },
    { 0xb9345679, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e645684, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134568a, 43 },
    { 0x15f4568b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa294568f, 98 },
    { 0, 0 },
    { 0xc4845691, 66 },
    { 0x4e945692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd545695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534569a, 23 },
    { 0, 0 },
    { 0x5234569c, 33 },
    { 0xa304569d, 23 },
    { 0xdeb4569c, 90 },
    { 0x23c4569d, 95 },
    { 0xfbf4569c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80456a7, 77 },
    { 0, 0 },
    { 0x5ed456a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x767456ac, 94 },
    { 0xc5f456ad, 34 },
    { 0x953456ad, 95 },
    { 0xb9d456ad, 98 },
    { 0xb01456b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x129456b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b456b8, 66 },
    { 0xf1a456b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83456bc, 61 },
    { 0x55d456bd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4456c2, 55 },
    { 0, 0 },
    { 0xd58456c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77456c7, 34 },
    { 0x30a456c8, 91 },
    { 0, 0 },
    { 0xb98456ca, 76 },
    { 0x679456cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be456d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01456d4, 33 },
    { 0x881456d5, 46 },
    { 0x267456d5, 95 },
    { 0x4ff456d7, 93 },
    { 0x447456d8, 61 },
    { 0xa83456d8, 46 },
    { 0xd22456d7, 97 },
    { 0xdc4456db, 93 },
    { 0, 0 },
    { 0x587456dd, 23 },
    { 0, 0 },
    { 0xfd6456df, 76 },
    { 0x4b9456e0, 61 },
    { 0x2b5456e0, 76 },
    { 0x75d456e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x181456e5, 33 },
    { 0xbf7456e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c456eb, 99 },
    { 0, 0 },
    { 0x7e7456ed, 76 },
    { 0, 0 },
    { 0x1b6456ef, 33 },
    { 0x668456ef, 98 },
    { 0, 0 },
    { 0x179456f2, 52 },
    { 0, 0 },
    { 0xdb456f4, 33 },
    { 0x371456f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x304456f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x275456fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14456ff, 33 },
    { 0, 0 },
    { 0x90545701, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x85045704, 100 },
    { 0xe45705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46045708, 36 },
    { 0x66545708, 53 },
    { 0xf6f4570a, 75 },
    { 0x2404570a, 23 },
    { 0xf564570c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb45712, 61 },
    { 0xe2345713, 78 },
    { 0, 0 },
    { 0x9ed45715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e45718, 33 },
    { 0, 0 },
    { 0x7594571a, 95 },
    { 0x4bc4571a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3104571e, 80 },
    { 0, 0 },
    { 0x10b45720, 78 },
    { 0xc2145721, 98 },
    { 0, 0 },
    { 0xf2745723, 78 },
    { 0x5c745723, 23 },
    { 0xb6945724, 33 },
    { 0, 0 },
    { 0x1d245727, 90 },
    { 0, 0 },
    { 0x3145729, 87 },
    { 0xe7745729, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a45730, 85 },
    { 0x97b45731, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8045736, 93 },
    { 0, 0 },
    { 0x43645738, 98 },
    { 0x59f45739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4573c, 61 },
    { 0, 0 },
    { 0x10e4573e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36545743, 91 },
    { 0x6a945743, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca45747, 61 },
    { 0x4af45748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13045753, 68 },
    { 0x93045754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd104575a, 61 },
    { 0x1c34575b, 95 },
    { 0xe604575c, 78 },
    { 0, 0 },
    { 0xcc64575e, 67 },
    { 0, 0 },
    { 0xcbb45760, 98 },
    { 0x2b345760, 34 },
    { 0x5ed45761, 98 },
    { 0xeb745763, 98 },
    { 0xe3045763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d45767, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e04576a, 85 },
    { 0xe3d4576b, 23 },
    { 0, 0 },
    { 0xab24576d, 33 },
    { 0x8f54576e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e45772, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd45776, 80 },
    { 0, 0 },
    { 0x99145778, 98 },
    { 0xcf145779, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394577c, 95 },
    { 0xb094577d, 61 },
    { 0x1b04577e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3045783, 77 },
    { 0, 0 },
    { 0xd0145785, 34 },
    { 0, 0 },
    { 0x41045787, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba345796, 43 },
    { 0x8dd45796, 33 },
    { 0x48c45798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56457a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952457ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb457af, 43 },
    { 0xdde457b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da457b5, 99 },
    { 0, 0 },
    { 0x472457b7, 84 },
    { 0x190457b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28457bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e457c0, 87 },
    { 0x11a457c1, 34 },
    { 0x22c457c2, 90 },
    { 0xe5b457c3, 67 },
    { 0x9a6457c4, 55 },
    { 0, 0 },
    { 0x38b457c6, 97 },
    { 0x6ea457c7, 33 },
    { 0x4cc457c8, 66 },
    { 0, 0 },
    { 0x134457ca, 34 },
    { 0xedc457cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b457d1, 98 },
    { 0xaa9457d2, 61 },
    { 0x429457d2, 87 },
    { 0, 0 },
    { 0x36d457d5, 33 },
    { 0x474457d6, 93 },
    { 0x473457d5, 99 },
    { 0x1f1457d8, 55 },
    { 0x483457d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297457de, 33 },
    { 0, 0 },
    { 0x7e3457e0, 90 },
    { 0xd18457e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e457ed, 61 },
    { 0xbde457ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87945803, 55 },
    { 0x96045804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd245807, 98 },
    { 0x5c845807, 90 },
    { 0x12945809, 88 },
    { 0xe2a4580a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe14580d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c045812, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39945817, 90 },
    { 0x46845818, 66 },
    { 0xd4945817, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4581d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45824, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9145827, 35 },
    { 0x80845828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304582b, 67 },
    { 0, 0 },
    { 0x4cf4582d, 23 },
    { 0x2e04582e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc045833, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c54583d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa45842, 34 },
    { 0, 0 },
    { 0x2e445844, 65 },
    { 0, 0 },
    { 0x5d145846, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f945849, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd4584e, 61 },
    { 0xdbd4584e, 61 },
    { 0, 0 },
    { 0xf2a45851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d45856, 91 },
    { 0xafb45856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f4585b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784585e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52745864, 100 },
    { 0, 0 },
    { 0x55c45866, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae4586f, 23 },
    { 0x6a745870, 34 },
    { 0, 0 },
    { 0x5eb45872, 23 },
    { 0xcbd45873, 61 },
    { 0xffa45873, 85 },
    { 0xece45875, 90 },
    { 0x2df45875, 61 },
    { 0xa7045873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca645880, 59 },
    { 0, 0 },
    { 0x8dd45882, 61 },
    { 0xc8845882, 93 },
    { 0xbc845884, 93 },
    { 0, 0 },
    { 0x16845886, 43 },
    { 0xee045887, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a45894, 33 },
    { 0x39145895, 95 },
    { 0x2ae45895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a4589a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd458a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d458a3, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb458aa, 76 },
    { 0xcbc458ab, 85 },
    { 0x4eb458ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9458b1, 83 },
    { 0x8f1458b2, 90 },
    { 0x9a2458b3, 90 },
    { 0x547458b4, 55 },
    { 0x3dd458b3, 33 },
    { 0x16f458b6, 83 },
    { 0x1e5458b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4458bb, 92 },
    { 0x14f458bc, 46 },
    { 0xbcc458bd, 95 },
    { 0x2e9458bb, 65 },
    { 0xbe458bb, 55 },
    { 0xa94458c0, 55 },
    { 0xdbd458c1, 33 },
    { 0xb20458c2, 45 },
    { 0x1c5458c3, 67 },
    { 0, 0 },
    { 0x9aa458c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f458c8, 61 },
    { 0, 0 },
    { 0x7b4458ca, 34 },
    { 0, 0 },
    { 0x937458cc, 33 },
    { 0xeef458cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1458d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6458d5, 59 },
    { 0xd9c458d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e458db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc458e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e458e6, 34 },
    { 0x10a458e7, 61 },
    { 0x3c1458e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1458eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae458ee, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46458f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93458f5, 81 },
    { 0, 0 },
    { 0x116458f7, 68 },
    { 0x80f458f8, 4 },
    { 0x304458f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa45900, 76 },
    { 0, 0 },
    { 0x8245902, 91 },
    { 0x9d745903, 87 },
    { 0xfee45904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f45907, 33 },
    { 0, 0 },
    { 0x2b145909, 100 },
    { 0x8e04590a, 61 },
    { 0x2f4590a, 95 },
    { 0, 0 },
    { 0xfcd4590d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea045913, 90 },
    { 0x66845914, 23 },
    { 0xd6245915, 99 },
    { 0x14a45916, 33 },
    { 0x4d345915, 23 },
    { 0x84e45917, 23 },
    { 0x53a45917, 34 },
    { 0x18645919, 76 },
    { 0, 0 },
    { 0x5854591c, 61 },
    { 0x66a4591d, 17 },
    { 0x9dd4591d, 90 },
    { 0x3da4591f, 90 },
    { 0x97145920, 43 },
    { 0xbed45920, 90 },
    { 0xe8a45922, 100 },
    { 0x17145923, 98 },
    { 0xce54591f, 93 },
    { 0x43545921, 55 },
    { 0x9e04591d, 46 },
    { 0x40c4591f, 55 },
    { 0x57545928, 43 },
    { 0, 0 },
    { 0x43b4592a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe4592d, 43 },
    { 0xc714592e, 34 },
    { 0x6cd4592f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad445935, 90 },
    { 0x3db45936, 23 },
    { 0xeca45937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c44593e, 76 },
    { 0x9cd4593f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c45943, 46 },
    { 0x5ff45944, 61 },
    { 0xec245945, 87 },
    { 0x93845946, 98 },
    { 0xb1b45947, 98 },
    { 0, 0 },
    { 0xd8345949, 33 },
    { 0, 0 },
    { 0x8854594b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2424594e, 80 },
    { 0x69d4594f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f45952, 90 },
    { 0, 0 },
    { 0xb8545954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c45957, 95 },
    { 0x5fc45958, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd45961, 76 },
    { 0, 0 },
    { 0xacb45963, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904596d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b945973, 78 },
    { 0x56245974, 64 },
    { 0, 0 },
    { 0x8a445976, 95 },
    { 0x93645977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274597a, 77 },
    { 0, 0 },
    { 0x8114597c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5445987, 100 },
    { 0x24145988, 100 },
    { 0x2e545987, 65 },
    { 0, 0 },
    { 0xa424598b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c4598f, 90 },
    { 0xf7845990, 88 },
    { 0x6b045990, 60 },
    { 0x49045992, 61 },
    { 0xba745991, 99 },
    { 0, 0 },
    { 0xa6245995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1745999, 77 },
    { 0x6e04599a, 61 },
    { 0, 0 },
    { 0x7fe4599c, 98 },
    { 0xa054599d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae459a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb459a3, 76 },
    { 0, 0 },
    { 0x98e459a5, 90 },
    { 0x62e459a5, 97 },
    { 0xaff459a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758459b1, 40 },
    { 0xbe2459b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4459b6, 43 },
    { 0, 0 },
    { 0xf56459b8, 90 },
    { 0, 0 },
    { 0xaaf459ba, 78 },
    { 0, 0 },
    { 0xaa5459bc, 90 },
    { 0, 0 },
    { 0x918459be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d459c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa459c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f459c7, 17 },
    { 0x542459c7, 43 },
    { 0x7a8459c9, 43 },
    { 0xfd0459c7, 75 },
    { 0x988459cb, 33 },
    { 0x321459cc, 90 },
    { 0, 0 },
    { 0xf33459ce, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248459d4, 93 },
    { 0, 0 },
    { 0x44f459d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab459db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b459e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972459ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc459ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8459f4, 61 },
    { 0xe54459f5, 84 },
    { 0x69a459f6, 95 },
    { 0, 0 },
    { 0xe0c459f8, 76 },
    { 0x82a459f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657459fd, 61 },
    { 0, 0 },
    { 0x635459ff, 90 },
    { 0xd5f45a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b45a07, 55 },
    { 0x62945a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc845a0f, 91 },
    { 0x5d645a0f, 33 },
    { 0xe6745a0f, 49 },
    { 0x6a645a12, 43 },
    { 0xb6745a13, 95 },
    { 0xb8d45a14, 33 },
    { 0x26045a14, 76 },
    { 0x97945a15, 95 },
    { 0x78d45a12, 90 },
    { 0xabe45a18, 43 },
    { 0xe6245a19, 90 },
    { 0, 0 },
    { 0x33045a1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77745a1f, 61 },
    { 0xd0e45a20, 95 },
    { 0xeea45a21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b45a25, 95 },
    { 0, 0 },
    { 0xb7f45a27, 23 },
    { 0x6cb45a28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8245a2b, 17 },
    { 0, 0 },
    { 0x94e45a2d, 90 },
    { 0x95f45a2e, 100 },
    { 0xff45a2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb945a33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f445a37, 33 },
    { 0xb0545a38, 24 },
    { 0x7c45a38, 75 },
    { 0x97a45a3a, 100 },
    { 0x11a45a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed945a41, 23 },
    { 0xc2745a41, 77 },
    { 0x61a45a42, 95 },
    { 0xcd145a41, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25345a49, 33 },
    { 0x4e945a49, 93 },
    { 0, 0 },
    { 0x9fa45a4c, 98 },
    { 0xc2245a4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d45a50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc45a58, 93 },
    { 0, 0 },
    { 0x82d45a5a, 76 },
    { 0x4a245a5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd145a5f, 43 },
    { 0x69845a5f, 76 },
    { 0x6d745a60, 43 },
    { 0x52245a60, 23 },
    { 0x92f45a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47145a66, 95 },
    { 0xa045a67, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c845a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be45a79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc45a7f, 87 },
    { 0x5b145a80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b45a83, 84 },
    { 0, 0 },
    { 0x74145a85, 61 },
    { 0x9045a85, 33 },
    { 0x48845a87, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x69045a8a, 76 },
    { 0x91845a8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c45a8f, 90 },
    { 0, 0 },
    { 0x33945a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c045a95, 33 },
    { 0x25c45a95, 90 },
    { 0x19745a97, 95 },
    { 0xa7445a98, 100 },
    { 0x11245a98, 98 },
    { 0x68045a99, 43 },
    { 0, 0 },
    { 0x80b45a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e45a9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11745aa5, 98 },
    { 0x88545aa6, 95 },
    { 0xde745aa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2345aaa, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c45aae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39245ab5, 100 },
    { 0x12245ab5, 90 },
    { 0, 0 },
    { 0x9f45ab8, 55 },
    { 0x4c045ab8, 98 },
    { 0, 0 },
    { 0x52345abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84345abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d45ac5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f245ac8, 33 },
    { 0x52a45ac8, 91 },
    { 0x9d245aca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce45ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4645ad4, 98 },
    { 0xe5845ad5, 23 },
    { 0xca045ad6, 61 },
    { 0, 0 },
    { 0xe1145ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf45ae1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb145ae4, 88 },
    { 0, 0 },
    { 0xd0d45ae6, 98 },
    { 0x6c345ae7, 90 },
    { 0x4a845ae7, 23 },
    { 0x4a945ae6, 95 },
    { 0x67945aea, 46 },
    { 0, 0 },
    { 0xf0f45aec, 55 },
    { 0xfb545aed, 90 },
    { 0xf2b45aee, 33 },
    { 0, 0 },
    { 0x13945af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac945af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c45af7, 65 },
    { 0x1a845af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df45b04, 61 },
    { 0x96f45b04, 90 },
    { 0, 0 },
    { 0x49645b07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48545b0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b545b0d, 55 },
    { 0x48145b0e, 90 },
    { 0x52b45b0f, 43 },
    { 0x58645b0f, 23 },
    { 0x8c245b11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b45b14, 95 },
    { 0, 0 },
    { 0xedd45b16, 90 },
    { 0, 0 },
    { 0x3a545b18, 99 },
    { 0x5b045b18, 52 },
    { 0x30b45b1a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c45b1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a145b23, 34 },
    { 0x94645b24, 55 },
    { 0, 0 },
    { 0xe1545b26, 98 },
    { 0x21745b27, 38 },
    { 0, 0 },
    { 0x13445b29, 90 },
    { 0, 0 },
    { 0xcc645b2b, 98 },
    { 0x5d545b2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b45b2f, 33 },
    { 0, 0 },
    { 0x3b745b31, 61 },
    { 0, 0 },
    { 0x4af45b33, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf45b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0145b3f, 78 },
    { 0x8de45b3f, 91 },
    { 0x86f45b41, 98 },
    { 0xeee45b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2445b47, 93 },
    { 0, 0 },
    { 0x58645b49, 43 },
    { 0x3d945b4a, 66 },
    { 0x55b45b4a, 55 },
    { 0x4cc45b4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9645b52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e745b55, 99 },
    { 0xcaf45b55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f745b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18045b61, 90 },
    { 0x71345b61, 90 },
    { 0x91a45b62, 67 },
    { 0x1bc45b62, 33 },
    { 0x86f45b65, 23 },
    { 0x51b45b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d45b6a, 95 },
    { 0xbe045b6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b145b6e, 33 },
    { 0x33b45b6f, 90 },
    { 0x54445b6f, 66 },
    { 0, 0 },
    { 0x5e845b72, 23 },
    { 0x20545b73, 90 },
    { 0, 0 },
    { 0xc3b45b75, 23 },
    { 0xa5e45b76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e45b7f, 61 },
    { 0x65c45b80, 33 },
    { 0xc6745b81, 98 },
    { 0, 0 },
    { 0x1c345b83, 95 },
    { 0xe8745b84, 90 },
    { 0x7a045b83, 90 },
    { 0x9f445b86, 33 },
    { 0x86645b87, 98 },
    { 0xdff45b88, 95 },
    { 0xf4745b85, 33 },
    { 0x34d45b89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d345b8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d45b94, 84 },
    { 0, 0 },
    { 0xc6645b96, 91 },
    { 0xfca45b97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e45ba0, 75 },
    { 0x47b45ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40945ba5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a45ba9, 90 },
    { 0, 0 },
    { 0x4b445bab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed745bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a645bb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c45bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2045bbb, 33 },
    { 0xd4145bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38845bc1, 85 },
    { 0, 0 },
    { 0x69b45bc3, 33 },
    { 0, 0 },
    { 0x8ae45bc5, 76 },
    { 0x56d45bc6, 61 },
    { 0x3fa45bc7, 95 },
    { 0, 0 },
    { 0x82a45bc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b45bcc, 68 },
    { 0xe4145bcd, 33 },
    { 0xca245bce, 98 },
    { 0xff45bcf, 17 },
    { 0x46045bd0, 23 },
    { 0, 0 },
    { 0x60a45bd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a145bd8, 61 },
    { 0x6c845bd9, 33 },
    { 0xdf945bd9, 68 },
    { 0xdf645bd8, 95 },
    { 0xcd745bda, 98 },
    { 0x1df45bdd, 22 },
    { 0x3eb45bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75945be3, 95 },
    { 0xc0745be4, 98 },
    { 0xbcd45be4, 61 },
    { 0x9bc45be6, 61 },
    { 0xd0145be4, 23 },
    { 0x14345be8, 23 },
    { 0, 0 },
    { 0xa5445bea, 95 },
    { 0x49745beb, 85 },
    { 0, 0 },
    { 0x89c45bed, 51 },
    { 0x25b45bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c845bf4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24845bfb, 61 },
    { 0, 0 },
    { 0x4f245bfd, 90 },
    { 0, 0 },
    { 0x5a445bff, 33 },
    { 0x41145c00, 100 },
    { 0xddd45c01, 90 },
    { 0, 0 },
    { 0x7f045c03, 23 },
    { 0, 0 },
    { 0xa8545c05, 43 },
    { 0x8f745c06, 76 },
    { 0xbbd45c07, 23 },
    { 0x45245c08, 84 },
    { 0xcd445c05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54345c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d45c10, 81 },
    { 0x4af45c11, 61 },
    { 0x59745c10, 33 },
    { 0xe8945c12, 76 },
    { 0x7c945c11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b45c17, 61 },
    { 0, 0 },
    { 0xeb645c19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab45c1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c1f, 61 },
    { 0, 0 },
    { 0x6545c21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74545c24, 90 },
    { 0, 0 },
    { 0x92b45c26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d45c32, 55 },
    { 0xfc845c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f45c36, 76 },
    { 0x50a45c37, 61 },
    { 0xac245c37, 100 },
    { 0, 0 },
    { 0x94745c3a, 55 },
    { 0xe8745c3b, 33 },
    { 0x1fe45c3b, 55 },
    { 0x11245c3a, 98 },
    { 0x22845c3e, 33 },
    { 0xe2045c3b, 90 },
    { 0x8a445c40, 75 },
    { 0x56a45c41, 61 },
    { 0x97145c3c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d45c47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c45c4c, 90 },
    { 0x9a245c4d, 98 },
    { 0xbc145c4e, 98 },
    { 0x8845c4c, 43 },
    { 0, 0 },
    { 0xde45c51, 95 },
    { 0xc7b45c51, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43645c56, 67 },
    { 0, 0 },
    { 0xf8645c58, 77 },
    { 0xe3c45c58, 55 },
    { 0xab445c58, 93 },
    { 0x78345c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae45c60, 98 },
    { 0, 0 },
    { 0x6ab45c62, 43 },
    { 0x25945c63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f45c66, 61 },
    { 0, 0 },
    { 0xa8245c68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf45c6b, 95 },
    { 0x33245c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d45c6f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93645c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a645c7a, 61 },
    { 0x5fc45c7b, 99 },
    { 0x79845c7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec45c7f, 55 },
    { 0x4b045c80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e45c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45245c89, 98 },
    { 0xea745c8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9745c8f, 33 },
    { 0x86045c8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f645c99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc345c9c, 99 },
    { 0xa8245c9c, 90 },
    { 0x7b945c9e, 95 },
    { 0, 0 },
    { 0xbba45ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6845ca8, 95 },
    { 0x22545ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf45cac, 34 },
    { 0xc4345cad, 76 },
    { 0, 0 },
    { 0x43745caf, 99 },
    { 0xb1445caf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c945cb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe45cbb, 23 },
    { 0x44045cbb, 61 },
    { 0xeaf45cbc, 93 },
    { 0xe4f45cbe, 84 },
    { 0, 0 },
    { 0x8a245cc0, 95 },
    { 0, 0 },
    { 0xd2545cc2, 99 },
    { 0, 0 },
    { 0xc0345cc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f45ccc, 33 },
    { 0xdf45ccd, 67 },
    { 0xb4245ccd, 87 },
    { 0x72845cce, 40 },
    { 0x94345cd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0045cd4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a645cdb, 23 },
    { 0x98745cdc, 98 },
    { 0xfa445cdd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a45cea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60345ced, 66 },
    { 0xc4f45cee, 77 },
    { 0, 0 },
    { 0x55245cf0, 78 },
    { 0, 0 },
    { 0x3d445cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42445cf5, 91 },
    { 0, 0 },
    { 0x65f45cf7, 98 },
    { 0, 0 },
    { 0x6d745cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87345d05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba545d08, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59345d11, 88 },
    { 0x29545d11, 33 },
    { 0x3e245d11, 93 },
    { 0xa7f45d14, 98 },
    { 0xfc545d11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd45d19, 88 },
    { 0xdc345d19, 98 },
    { 0xd9045d1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be45d25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b45d29, 61 },
    { 0x15845d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b45d2d, 66 },
    { 0x30545d2d, 55 },
    { 0xad045d2f, 87 },
    { 0, 0 },
    { 0x8d745d31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e645d35, 78 },
    { 0xd8845d35, 90 },
    { 0xe2a45d35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca245d3a, 55 },
    { 0xa545d3b, 91 },
    { 0x91645d3c, 55 },
    { 0xf445d3b, 95 },
    { 0x8f045d3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd145d41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f345d45, 85 },
    { 0x22845d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d945d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f845d58, 33 },
    { 0x11445d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b45d5d, 95 },
    { 0x5ad45d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb445d62, 93 },
    { 0, 0 },
    { 0x62c45d64, 33 },
    { 0x23245d65, 23 },
    { 0, 0 },
    { 0x63745d67, 52 },
    { 0x27145d68, 66 },
    { 0x82345d67, 69 },
    { 0xb445d67, 33 },
    { 0x15945d6b, 97 },
    { 0, 0 },
    { 0xb2f45d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b45d7b, 23 },
    { 0, 0 },
    { 0x46245d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f145d81, 98 },
    { 0, 0 },
    { 0xc9945d83, 33 },
    { 0xf2b45d83, 61 },
    { 0x29945d85, 34 },
    { 0xf9945d85, 95 },
    { 0xebe45d87, 76 },
    { 0x2b845d88, 91 },
    { 0x16a45d89, 33 },
    { 0x79b45d8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a45d8d, 90 },
    { 0, 0 },
    { 0x59945d8f, 98 },
    { 0x4c945d8f, 61 },
    { 0x23745d90, 61 },
    { 0, 0 },
    { 0xd1c45d93, 98 },
    { 0, 0 },
    { 0x19745d95, 91 },
    { 0xd5b45d96, 61 },
    { 0xe1745d96, 85 },
    { 0xb1345d97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a945d9f, 87 },
    { 0x38c45da0, 33 },
    { 0xeeb45da0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e45da5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84045da9, 52 },
    { 0xd5845daa, 90 },
    { 0xbc645dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90045dae, 95 },
    { 0xff645daf, 100 },
    { 0x4f245db0, 61 },
    { 0x7c145daf, 55 },
    { 0xae145db2, 100 },
    { 0x9345db3, 33 },
    { 0, 0 },
    { 0x48245db5, 85 },
    { 0x41d45db5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25645dba, 23 },
    { 0, 0 },
    { 0x39845dbc, 52 },
    { 0x73045dbd, 43 },
    { 0x44345dbe, 23 },
    { 0x2945dbe, 78 },
    { 0xdc445dc0, 34 },
    { 0x10845dc1, 75 },
    { 0xf345dc2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20845dc6, 100 },
    { 0xdb245dc6, 95 },
    { 0xeed45dc7, 23 },
    { 0, 0 },
    { 0xc2845dca, 98 },
    { 0x41a45dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e45dd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2845dd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac045ddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25045de1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16645de4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f945de9, 95 },
    { 0x39f45de9, 76 },
    { 0x89545deb, 82 },
    { 0, 0 },
    { 0xbc045ded, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8545df7, 90 },
    { 0x88045df8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d45dfc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15145dff, 95 },
    { 0, 0 },
    { 0x58f45e01, 46 },
    { 0, 0 },
    { 0x80d45e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce445e07, 61 },
    { 0x1445e08, 98 },
    { 0xe9d45e09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a45e0e, 100 },
    { 0xe7345e0f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de45e15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8845e23, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e545e27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d345e2b, 43 },
    { 0x7b345e2c, 43 },
    { 0x9d245e2d, 23 },
    { 0xf8945e2d, 69 },
    { 0x7d345e2c, 23 },
    { 0, 0 },
    { 0xc4b45e31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44145e39, 61 },
    { 0, 0 },
    { 0xb9345e3b, 95 },
    { 0, 0 },
    { 0x33245e3d, 33 },
    { 0x2e445e3e, 52 },
    { 0x10745e3e, 61 },
    { 0, 0 },
    { 0xc4945e41, 67 },
    { 0, 0 },
    { 0xde545e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9045e4d, 33 },
    { 0x84745e4e, 33 },
    { 0x7145e4f, 43 },
    { 0x94045e4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a345e53, 95 },
    { 0x6f45e54, 55 },
    { 0x3ea45e54, 98 },
    { 0, 0 },
    { 0xc7d45e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda345e5d, 46 },
    { 0x5d945e5d, 95 },
    { 0, 0 },
    { 0xf1945e60, 61 },
    { 0xe6045e61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a845e69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c45e71, 85 },
    { 0x4a345e71, 38 },
    { 0x59a45e73, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x26445e76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4345e79, 66 },
    { 0xa4345e7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a245e7d, 90 },
    { 0x60c45e7e, 61 },
    { 0, 0 },
    { 0xf7c45e80, 23 },
    { 0xf8045e81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d445e86, 76 },
    { 0xf1b45e87, 81 },
    { 0x78045e88, 90 },
    { 0x4cd45e89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21945e8d, 76 },
    { 0x5ed45e8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36445e92, 33 },
    { 0xf4d45e93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7245e98, 33 },
    { 0x3b545e99, 98 },
    { 0x8c945e98, 34 },
    { 0, 0 },
    { 0x4c745e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9245e9f, 34 },
    { 0x82d45e9f, 98 },
    { 0x84545ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc645ea5, 33 },
    { 0, 0 },
    { 0x7ff45ea7, 95 },
    { 0, 0 },
    { 0xf2945ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71045eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee945eb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a45eba, 33 },
    { 0xec145ebb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d45ebe, 33 },
    { 0, 0 },
    { 0x79a45ec0, 61 },
    { 0, 0 },
    { 0xcc345ec2, 61 },
    { 0xb8445ec3, 76 },
    { 0x68d45ec3, 67 },
    { 0xb3745ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9745ecd, 99 },
    { 0, 0 },
    { 0x1ec45ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73145ed2, 60 },
    { 0xcde45ed3, 93 },
    { 0x28145ed4, 55 },
    { 0, 0 },
    { 0x40845ed6, 95 },
    { 0x51f45ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f45eda, 93 },
    { 0, 0 },
    { 0x1c445edc, 61 },
    { 0xb7845edd, 61 },
    { 0xd8c45ede, 95 },
    { 0xad845edf, 61 },
    { 0xa1445ede, 33 },
    { 0, 0 },
    { 0x2a745ee2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac445ee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c45eeb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a245eee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2845ef2, 61 },
    { 0x6e845ef3, 98 },
    { 0x5e45ef3, 23 },
    { 0, 0 },
    { 0x7ee45ef6, 61 },
    { 0, 0 },
    { 0xc8e45ef8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b45efd, 33 },
    { 0x50b45efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a845f01, 96 },
    { 0xe7745f02, 90 },
    { 0xcef45f03, 23 },
    { 0x38b45f02, 61 },
    { 0xe2045f04, 93 },
    { 0, 0 },
    { 0x52145f07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f45f0a, 33 },
    { 0x36c45f0b, 76 },
    { 0x9c645f0b, 93 },
    { 0x7b245f0d, 76 },
    { 0x85d45f0a, 90 },
    { 0xbb045f0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e45f14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f45f18, 100 },
    { 0x81f45f18, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef45f21, 55 },
    { 0, 0 },
    { 0xf4345f23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8745f26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e45f29, 90 },
    { 0x44645f2a, 90 },
    { 0xb5945f2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46745f35, 95 },
    { 0, 0 },
    { 0x1e245f37, 91 },
    { 0xf3545f38, 61 },
    { 0, 0 },
    { 0x31145f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f45f3f, 23 },
    { 0xb7945f40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe945f43, 17 },
    { 0, 0 },
    { 0x43345f45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd945f48, 95 },
    { 0, 0 },
    { 0x73245f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d45f50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81845f53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae845f57, 61 },
    { 0x37645f58, 33 },
    { 0x9c45f59, 95 },
    { 0x4f145f58, 23 },
    { 0xfbe45f57, 90 },
    { 0x2fe45f5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b345f62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c245f66, 61 },
    { 0x7b345f66, 34 },
    { 0x7d445f67, 77 },
    { 0xf6d45f66, 91 },
    { 0xd4045f67, 34 },
    { 0, 0 },
    { 0x5cd45f6c, 90 },
    { 0x40545f6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38045f70, 19 },
    { 0, 0 },
    { 0xaa845f72, 61 },
    { 0x8a545f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec45f77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50745f7e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f745f81, 23 },
    { 0x8c345f82, 76 },
    { 0x1dd45f83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b45f86, 93 },
    { 0xf3a45f87, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d945f8a, 61 },
    { 0x15345f8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb645f90, 76 },
    { 0xe2345f91, 55 },
    { 0x47145f90, 55 },
    { 0x8ab45f92, 92 },
    { 0, 0 },
    { 0x32c45f95, 90 },
    { 0, 0 },
    { 0x54645f97, 33 },
    { 0x7b645f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c745f9c, 84 },
    { 0xe6e45f9d, 64 },
    { 0x3f545f9c, 61 },
    { 0, 0 },
    { 0x70545fa0, 95 },
    { 0xa8145fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2645fa7, 66 },
    { 0x79045fa8, 95 },
    { 0x18745fa7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4145fac, 33 },
    { 0xd5945fad, 87 },
    { 0, 0 },
    { 0xbc245faf, 33 },
    { 0x6fe45fb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde45fb5, 17 },
    { 0x1d945fb5, 90 },
    { 0, 0 },
    { 0x64045fb8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8945fbd, 67 },
    { 0xbf745fbd, 43 },
    { 0xaa145fbe, 23 },
    { 0, 0 },
    { 0xfa545fc1, 85 },
    { 0x6a445fc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7445fc7, 98 },
    { 0x36245fc8, 78 },
    { 0, 0 },
    { 0xfb645fca, 46 },
    { 0x8de45fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f45fd1, 95 },
    { 0, 0 },
    { 0x6d945fd3, 90 },
    { 0xb3f45fd4, 98 },
    { 0, 0 },
    { 0x79f45fd6, 23 },
    { 0x19a45fd7, 76 },
    { 0, 0 },
    { 0x30845fd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e45fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb645fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3945fe9, 95 },
    { 0, 0 },
    { 0x26045feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26745ff1, 88 },
    { 0x2b945ff2, 90 },
    { 0, 0 },
    { 0x24445ff4, 55 },
    { 0, 0 },
    { 0x90245ff6, 90 },
    { 0x68a45ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae45fff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85646006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a4600a, 76 },
    { 0, 0 },
    { 0x8fa4600c, 84 },
    { 0x5e4600d, 61 },
    { 0, 0 },
    { 0x6894600f, 98 },
    { 0x6bc46010, 98 },
    { 0x43946011, 23 },
    { 0, 0 },
    { 0x11b46013, 33 },
    { 0x12346014, 90 },
    { 0x16046015, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41646019, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d4601e, 23 },
    { 0xe64601f, 98 },
    { 0xe2746020, 95 },
    { 0xb7a46020, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de46024, 52 },
    { 0, 0 },
    { 0x82f46026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35646029, 61 },
    { 0, 0 },
    { 0xf664602b, 46 },
    { 0x8234602c, 90 },
    { 0x5384602d, 55 },
    { 0x4034602b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf346036, 99 },
    { 0x35946037, 85 },
    { 0xa3146038, 78 },
    { 0xd8f46039, 93 },
    { 0x4e646039, 95 },
    { 0, 0 },
    { 0xd4d4603c, 61 },
    { 0, 0 },
    { 0x6964603e, 85 },
    { 0x1694603e, 33 },
    { 0x63d46040, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3546049, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e46050, 95 },
    { 0, 0 },
    { 0x5d746052, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5446055, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5546058, 76 },
    { 0x6746059, 23 },
    { 0x6604605a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e046065, 90 },
    { 0x95a46065, 90 },
    { 0, 0 },
    { 0xe3246068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5454606b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c46070, 61 },
    { 0xcc846070, 17 },
    { 0, 0 },
    { 0x9be46073, 83 },
    { 0xa4e46074, 33 },
    { 0xb8e46075, 34 },
    { 0x8f346076, 34 },
    { 0, 0 },
    { 0xbe546078, 66 },
    { 0, 0 },
    { 0xcb4607a, 43 },
    { 0xd284607b, 53 },
    { 0xd074607c, 76 },
    { 0x3474607d, 95 },
    { 0x1544607d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f46081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd46089, 61 },
    { 0x3cd46089, 43 },
    { 0xe814608b, 95 },
    { 0, 0 },
    { 0xfc4608d, 46 },
    { 0xcec4608e, 90 },
    { 0xb394608d, 40 },
    { 0x8284608f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18046096, 98 },
    { 0, 0 },
    { 0xd0946098, 93 },
    { 0xe2d46098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334609d, 93 },
    { 0, 0 },
    { 0x61f4609f, 76 },
    { 0, 0 },
    { 0x5f8460a1, 46 },
    { 0x94e460a2, 61 },
    { 0xbb1460a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4460a8, 33 },
    { 0x3d0460a8, 23 },
    { 0x206460a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0460b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82460bb, 66 },
    { 0, 0 },
    { 0x5d7460bd, 50 },
    { 0xdec460be, 67 },
    { 0x192460bf, 69 },
    { 0, 0 },
    { 0xb89460c1, 90 },
    { 0x4d6460c2, 76 },
    { 0x28460c3, 34 },
    { 0x8e7460c1, 46 },
    { 0xa16460c3, 95 },
    { 0xe1c460c6, 95 },
    { 0xf9d460c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9460ce, 98 },
    { 0xc74460cf, 76 },
    { 0, 0 },
    { 0x73d460d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b460d6, 34 },
    { 0xd7f460d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0460dc, 98 },
    { 0x1d2460dd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad460e0, 78 },
    { 0xcc6460e0, 76 },
    { 0x4ef460e2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416460e6, 33 },
    { 0, 0 },
    { 0xced460e8, 95 },
    { 0x17e460e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x573460ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9460f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58460f5, 90 },
    { 0x94a460f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523460fa, 90 },
    { 0x5b5460fa, 33 },
    { 0, 0 },
    { 0x44e460fd, 55 },
    { 0, 0 },
    { 0xe49460ff, 23 },
    { 0, 0 },
    { 0x50546101, 98 },
    { 0x96646102, 95 },
    { 0, 0 },
    { 0x25746104, 33 },
    { 0x5ef46105, 46 },
    { 0, 0 },
    { 0x46546107, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a46115, 34 },
    { 0xfdb46116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c46119, 33 },
    { 0, 0 },
    { 0x5504611b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646127, 78 },
    { 0x32c46128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5934612e, 23 },
    { 0, 0 },
    { 0x46046130, 52 },
    { 0x28746130, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c46137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c4613a, 83 },
    { 0x8614613b, 33 },
    { 0x33b4613c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b4613f, 51 },
    { 0x8474613f, 76 },
    { 0x50746141, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2364614d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46246154, 91 },
    { 0, 0 },
    { 0x9c646156, 91 },
    { 0x13546157, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e4615c, 55 },
    { 0, 0 },
    { 0x4b74615e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6546161, 65 },
    { 0xb4446162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b46165, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a46168, 61 },
    { 0xeeb46169, 78 },
    { 0, 0 },
    { 0xf674616b, 78 },
    { 0xfe24616b, 34 },
    { 0x6324616b, 51 },
    { 0x8574616e, 98 },
    { 0x7934616c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a46174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3474617d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a46182, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4846186, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f46189, 61 },
    { 0, 0 },
    { 0xa6c4618b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4618f, 95 },
    { 0xe9a46190, 61 },
    { 0x35146190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb46196, 23 },
    { 0x24746197, 99 },
    { 0xaed46196, 100 },
    { 0xdb146199, 61 },
    { 0, 0 },
    { 0xfd64619b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30461a1, 78 },
    { 0xa83461a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372461a9, 88 },
    { 0x410461a9, 55 },
    { 0xbca461ab, 92 },
    { 0x9b3461ab, 33 },
    { 0x639461ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68461b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf461b5, 61 },
    { 0xcd6461b6, 23 },
    { 0x668461b6, 61 },
    { 0x301461b8, 34 },
    { 0x610461b9, 55 },
    { 0x4f3461ba, 96 },
    { 0xb3c461b8, 77 },
    { 0x41e461bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72461bf, 33 },
    { 0, 0 },
    { 0xe68461c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f461c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d461c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7461ce, 98 },
    { 0, 0 },
    { 0xac5461d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509461d8, 75 },
    { 0x62d461d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0461dc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9461e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3461ed, 33 },
    { 0xcc5461ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35461f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a461f5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404461f9, 46 },
    { 0, 0 },
    { 0x1cf461fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97946200, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d946205, 90 },
    { 0x53a46206, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xec246209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794620c, 98 },
    { 0, 0 },
    { 0xe0b4620e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32546213, 33 },
    { 0x9a846214, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7846217, 23 },
    { 0, 0 },
    { 0xe5946219, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f46221, 52 },
    { 0x65346222, 90 },
    { 0x68746223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246227, 33 },
    { 0, 0 },
    { 0xe9f46229, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7946231, 95 },
    { 0, 0 },
    { 0x59d46233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f46238, 61 },
    { 0x67246239, 98 },
    { 0x3b44623a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6146240, 61 },
    { 0, 0 },
    { 0xb7946242, 64 },
    { 0, 0 },
    { 0xbbe46244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee46249, 33 },
    { 0x9764624a, 90 },
    { 0xbc44624a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec4624f, 87 },
    { 0, 0 },
    { 0xd7646251, 98 },
    { 0x48346252, 94 },
    { 0xd6446253, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf446256, 55 },
    { 0x8c246257, 90 },
    { 0xb6c46258, 101 },
    { 0xc4446258, 78 },
    { 0x2604625a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7714625f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70446266, 33 },
    { 0x46d46266, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb104626a, 90 },
    { 0x29a4626a, 90 },
    { 0x394626c, 34 },
    { 0xa6e4626d, 55 },
    { 0x91b4626a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd146271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac46274, 98 },
    { 0x75146275, 95 },
    { 0x54346276, 61 },
    { 0, 0 },
    { 0x1d946278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64627c, 95 },
    { 0xf314627d, 77 },
    { 0xc974627e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa46281, 33 },
    { 0xfab46281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c346285, 90 },
    { 0x97646286, 95 },
    { 0xa3b46287, 95 },
    { 0, 0 },
    { 0xff746289, 84 },
    { 0xbb84628a, 33 },
    { 0x5414628a, 98 },
    { 0x55f4628a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c46294, 61 },
    { 0, 0 },
    { 0x2ba46296, 46 },
    { 0xdff46297, 90 },
    { 0xe1b46296, 93 },
    { 0x61e46299, 51 },
    { 0, 0 },
    { 0x1514629b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f84629f, 23 },
    { 0x7bf4629f, 46 },
    { 0, 0 },
    { 0x65e462a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49462a5, 90 },
    { 0, 0 },
    { 0x35462a7, 85 },
    { 0xc29462a8, 33 },
    { 0x8f0462a9, 76 },
    { 0x1f462aa, 23 },
    { 0xa55462aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496462b0, 90 },
    { 0, 0 },
    { 0x8c6462b2, 46 },
    { 0xe1d462b3, 23 },
    { 0, 0 },
    { 0xf5c462b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b462b8, 95 },
    { 0x519462b9, 52 },
    { 0xb6e462b9, 17 },
    { 0xe46462b9, 23 },
    { 0xa57462bc, 23 },
    { 0xe03462bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b462c0, 85 },
    { 0x52d462c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745462c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e462cb, 46 },
    { 0x830462cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e462d0, 34 },
    { 0x9c9462d1, 84 },
    { 0x8d3462d0, 46 },
    { 0x2dd462d3, 33 },
    { 0x3fa462d2, 98 },
    { 0x2a7462d4, 84 },
    { 0, 0 },
    { 0xa70462d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce462da, 99 },
    { 0, 0 },
    { 0x5e7462dc, 98 },
    { 0xae1462dc, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5462e0, 98 },
    { 0x951462e1, 77 },
    { 0xc42462e2, 43 },
    { 0x318462e2, 23 },
    { 0, 0 },
    { 0x3a1462e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0462e9, 95 },
    { 0x834462ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30462f1, 90 },
    { 0x335462f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02462fc, 55 },
    { 0x2d2462fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a46301, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f46304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4630d, 98 },
    { 0x3fe4630e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd46312, 33 },
    { 0x7be46313, 55 },
    { 0, 0 },
    { 0x7d846315, 100 },
    { 0x29f46316, 90 },
    { 0, 0 },
    { 0x1e46318, 52 },
    { 0x3f846318, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d46322, 33 },
    { 0xe9946323, 55 },
    { 0x92246324, 90 },
    { 0, 0 },
    { 0xc6e46326, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd746329, 46 },
    { 0x65f4632a, 33 },
    { 0x8b34632b, 90 },
    { 0x34e4632c, 95 },
    { 0, 0 },
    { 0xae24632e, 61 },
    { 0x1884632e, 98 },
    { 0x5d146330, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28446334, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f46338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7346346, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b04634a, 95 },
    { 0, 0 },
    { 0xea04634c, 91 },
    { 0x1c34634d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4046352, 90 },
    { 0, 0 },
    { 0x64f46354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e46358, 90 },
    { 0xc346359, 61 },
    { 0x6614635a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc54635d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84146360, 95 },
    { 0x25746360, 93 },
    { 0xa5a46362, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61546366, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e74636c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d46373, 95 },
    { 0x81346374, 43 },
    { 0xc5f46375, 34 },
    { 0xbb846376, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94637a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c46389, 93 },
    { 0x3244638a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd4638d, 17 },
    { 0x6e54638e, 55 },
    { 0x55f4638d, 76 },
    { 0x23d4638f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9246393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c46397, 23 },
    { 0, 0 },
    { 0x28746399, 100 },
    { 0xf9a4639a, 90 },
    { 0x7ce46399, 33 },
    { 0xd494639b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c74639f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x287463a2, 91 },
    { 0, 0 },
    { 0x5d6463a4, 88 },
    { 0xc97463a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d463a8, 90 },
    { 0x779463a9, 90 },
    { 0, 0 },
    { 0x746463ab, 91 },
    { 0x889463ab, 95 },
    { 0x5f8463ac, 23 },
    { 0, 0 },
    { 0xc7f463af, 78 },
    { 0x87e463af, 90 },
    { 0xe39463b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96463b5, 95 },
    { 0, 0 },
    { 0x18c463b7, 67 },
    { 0x2cb463b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1463bb, 95 },
    { 0, 0 },
    { 0xaa8463bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c463c6, 43 },
    { 0x940463c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91463cc, 90 },
    { 0x25d463cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a463d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4463d6, 93 },
    { 0, 0 },
    { 0x5df463d8, 86 },
    { 0x8463d8, 61 },
    { 0, 0 },
    { 0x13e463db, 98 },
    { 0x308463dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4463ed, 99 },
    { 0, 0 },
    { 0x9a4463ef, 33 },
    { 0, 0 },
    { 0x847463f1, 52 },
    { 0x4c2463f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a463f7, 84 },
    { 0x5ce463f7, 93 },
    { 0x16b463f9, 23 },
    { 0x701463f9, 76 },
    { 0x646463f8, 67 },
    { 0xfab463fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37846404, 94 },
    { 0xd5546404, 90 },
    { 0x38e46406, 61 },
    { 0xed46407, 93 },
    { 0, 0 },
    { 0x12c46409, 61 },
    { 0x1f046409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4640d, 88 },
    { 0, 0 },
    { 0x65a4640f, 34 },
    { 0xa044640f, 75 },
    { 0, 0 },
    { 0x12546412, 34 },
    { 0x35346413, 61 },
    { 0x1b746414, 33 },
    { 0x63f46415, 68 },
    { 0x46e46416, 61 },
    { 0x57f46417, 77 },
    { 0, 0 },
    { 0x3e146419, 23 },
    { 0x84b4641a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9794641d, 23 },
    { 0x2634641e, 100 },
    { 0x3bc4641f, 76 },
    { 0xf7946420, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c46425, 33 },
    { 0x83746426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b04642a, 91 },
    { 0, 0 },
    { 0xc6b4642c, 46 },
    { 0x9154642c, 33 },
    { 0x8114642e, 91 },
    { 0x3d04642f, 33 },
    { 0x9b84642d, 90 },
    { 0, 0 },
    { 0xb1946432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1294643b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d646444, 87 },
    { 0xff446445, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc46448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4644d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c46452, 61 },
    { 0x53446452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65746459, 55 },
    { 0xf4346459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4645d, 33 },
    { 0xefb4645d, 33 },
    { 0x64f4645f, 98 },
    { 0x4fe46460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4946464, 60 },
    { 0xcf046464, 33 },
    { 0, 0 },
    { 0x2cf46467, 33 },
    { 0x21c46468, 88 },
    { 0xa8346468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa04646d, 76 },
    { 0x75d4646e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd46475, 61 },
    { 0x6e646475, 55 },
    { 0x3c846477, 46 },
    { 0xde146478, 61 },
    { 0xbbc46478, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa184647c, 76 },
    { 0x54a4647d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e46481, 90 },
    { 0x96a46482, 61 },
    { 0x8bd46482, 93 },
    { 0, 0 },
    { 0x1cd46485, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c4648a, 90 },
    { 0xc694648b, 61 },
    { 0, 0 },
    { 0x3114648d, 99 },
    { 0x7ac4648e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c46493, 95 },
    { 0xe4e46494, 33 },
    { 0x98b46494, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca4649f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29464a4, 43 },
    { 0x5d1464a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x746464a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x544464ab, 95 },
    { 0, 0 },
    { 0xe1c464ad, 61 },
    { 0xdf5464ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a464b3, 33 },
    { 0xb48464b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a464c3, 55 },
    { 0x718464c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737464cc, 95 },
    { 0, 0 },
    { 0x1ab464ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07464d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3464d8, 76 },
    { 0, 0 },
    { 0xeec464da, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6464dd, 86 },
    { 0xb3a464dd, 92 },
    { 0x414464dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a464e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23464e6, 46 },
    { 0x4fe464e6, 34 },
    { 0, 0 },
    { 0xeb464e9, 33 },
    { 0xab4464e9, 69 },
    { 0, 0 },
    { 0xdf1464ec, 90 },
    { 0, 0 },
    { 0xb0d464ee, 98 },
    { 0xbe8464ef, 81 },
    { 0x378464f0, 55 },
    { 0x153464ee, 33 },
    { 0, 0 },
    { 0xbde464f3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb464f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc464f9, 46 },
    { 0x9eb464f9, 33 },
    { 0x83c464fb, 61 },
    { 0xdc9464fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4146500, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb846503, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea946508, 61 },
    { 0x37346509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd84650d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c146510, 23 },
    { 0xbd446511, 34 },
    { 0x68946512, 90 },
    { 0x68146512, 90 },
    { 0x18e46514, 61 },
    { 0x71246515, 33 },
    { 0, 0 },
    { 0x69d46517, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad4651a, 61 },
    { 0, 0 },
    { 0x1234651c, 95 },
    { 0, 0 },
    { 0x3a54651e, 77 },
    { 0x8744651f, 9 },
    { 0x3764651e, 33 },
    { 0xeb64651f, 76 },
    { 0xcba46522, 68 },
    { 0x23946523, 34 },
    { 0x65a46522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f46528, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0746540, 76 },
    { 0, 0 },
    { 0xb7b46542, 98 },
    { 0xd4546543, 46 },
    { 0xa0046544, 46 },
    { 0xb9246542, 61 },
    { 0x18e46544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4684654d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba46555, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5146559, 98 },
    { 0xfb44655a, 98 },
    { 0xdae4655b, 90 },
    { 0x7a44655c, 95 },
    { 0, 0 },
    { 0xaa64655e, 61 },
    { 0xec64655f, 95 },
    { 0, 0 },
    { 0xf9d46561, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4656c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc324656f, 95 },
    { 0, 0 },
    { 0x12f46571, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c46577, 77 },
    { 0xe8046578, 84 },
    { 0xc2d46578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a4657d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd146586, 61 },
    { 0x12346587, 90 },
    { 0, 0 },
    { 0x2a946589, 34 },
    { 0xeb34658a, 66 },
    { 0xc5a46589, 90 },
    { 0, 0 },
    { 0xcc54658d, 52 },
    { 0x6744658d, 34 },
    { 0xab94658d, 91 },
    { 0xef646590, 77 },
    { 0x67a46590, 61 },
    { 0x86a46591, 95 },
    { 0xc746593, 90 },
    { 0xa1b46593, 33 },
    { 0x4f046595, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a4659d, 76 },
    { 0xa374659d, 98 },
    { 0x96b4659f, 43 },
    { 0xa89465a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3465a8, 23 },
    { 0x1df465a8, 61 },
    { 0x3b2465a8, 90 },
    { 0x589465ab, 34 },
    { 0x6b6465a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c465b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5465b8, 66 },
    { 0, 0 },
    { 0x65f465ba, 90 },
    { 0, 0 },
    { 0xc5f465bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd465c2, 61 },
    { 0x3a465c3, 23 },
    { 0x429465c3, 87 },
    { 0, 0 },
    { 0xa81465c6, 52 },
    { 0x205465c6, 61 },
    { 0x8a1465c8, 84 },
    { 0, 0 },
    { 0xc43465ca, 95 },
    { 0, 0 },
    { 0xe0d465cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d465cf, 90 },
    { 0x450465cf, 90 },
    { 0xdd3465d1, 68 },
    { 0x1d465d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c465da, 61 },
    { 0, 0 },
    { 0xa7e465dc, 23 },
    { 0, 0 },
    { 0x70d465de, 98 },
    { 0xbf8465df, 93 },
    { 0x472465e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71465e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x518465e6, 81 },
    { 0, 0 },
    { 0xc9b465e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0465ee, 90 },
    { 0, 0 },
    { 0x3bf465f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f465f4, 23 },
    { 0x74f465f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68465fc, 76 },
    { 0, 0 },
    { 0x8b1465fe, 76 },
    { 0, 0 },
    { 0xae446600, 61 },
    { 0x72546601, 64 },
    { 0xcf046602, 55 },
    { 0, 0 },
    { 0x3ff46604, 87 },
    { 0xa9b46604, 61 },
    { 0x91b46604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf346609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d24660d, 22 },
    { 0xb684660e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5946612, 98 },
    { 0xa3b46613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c54661b, 33 },
    { 0, 0 },
    { 0x6384661d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46146621, 23 },
    { 0x42546622, 90 },
    { 0xbcf46623, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab446629, 95 },
    { 0xdde4662a, 61 },
    { 0, 0 },
    { 0xfc94662c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8446630, 95 },
    { 0x30246631, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a24663a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa824663e, 98 },
    { 0, 0 },
    { 0x6546640, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9846647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf246654, 82 },
    { 0xead46655, 95 },
    { 0x6b746655, 33 },
    { 0x2246655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa814665e, 61 },
    { 0, 0 },
    { 0xb8846660, 55 },
    { 0x99046660, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56646668, 55 },
    { 0x5c946669, 52 },
    { 0x8a846669, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc834666d, 95 },
    { 0x7ba4666e, 98 },
    { 0, 0 },
    { 0x9546670, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d46673, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e46676, 90 },
    { 0x38046677, 76 },
    { 0, 0 },
    { 0xa2546679, 90 },
    { 0x1934667a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4667f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6746685, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944668b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf46691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa546694, 87 },
    { 0xcae46694, 8 },
    { 0, 0 },
    { 0x80e46697, 93 },
    { 0, 0 },
    { 0xc3e46699, 12 },
    { 0x86446699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e4669f, 90 },
    { 0, 0 },
    { 0x619466a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x978466a4, 95 },
    { 0, 0 },
    { 0x9a2466a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735466b3, 67 },
    { 0xf25466b4, 95 },
    { 0xb31466b5, 95 },
    { 0x1ac466b6, 61 },
    { 0x697466b7, 43 },
    { 0x804466b6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d466c2, 55 },
    { 0xda6466c2, 23 },
    { 0x927466c4, 33 },
    { 0x461466c4, 33 },
    { 0x85f466c6, 52 },
    { 0, 0 },
    { 0xd64466c8, 68 },
    { 0, 0 },
    { 0x294466ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95466ce, 61 },
    { 0x4fb466ce, 88 },
    { 0x2e8466d0, 90 },
    { 0, 0 },
    { 0x7d1466d2, 33 },
    { 0x1bd466d2, 98 },
    { 0x831466d4, 76 },
    { 0xa9a466d5, 33 },
    { 0x1e2466d6, 98 },
    { 0x706466d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc466e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10466e9, 43 },
    { 0, 0 },
    { 0xf02466eb, 55 },
    { 0, 0 },
    { 0x488466ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab466f1, 61 },
    { 0x8e4466f1, 61 },
    { 0, 0 },
    { 0x990466f4, 43 },
    { 0, 0 },
    { 0x1be466f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c466fa, 95 },
    { 0xbd3466fa, 90 },
    { 0, 0 },
    { 0xec466fd, 64 },
    { 0x3c5466fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a46702, 90 },
    { 0xa9a46703, 33 },
    { 0, 0 },
    { 0xa3e46705, 75 },
    { 0x8cc46706, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2384670a, 61 },
    { 0, 0 },
    { 0x3574670c, 56 },
    { 0x5644670d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e46711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4671b, 75 },
    { 0xd414671c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4671f, 55 },
    { 0x8654671f, 99 },
    { 0x69246721, 98 },
    { 0x16a46720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b046726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a46729, 33 },
    { 0x2a546729, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad46732, 98 },
    { 0x1a346733, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574673b, 95 },
    { 0, 0 },
    { 0xaaa4673d, 61 },
    { 0xd124673d, 76 },
    { 0xf4f4673f, 76 },
    { 0xa4a46740, 87 },
    { 0x7bf4673d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5346744, 34 },
    { 0xa5746744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc446748, 98 },
    { 0x88d46748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a4674e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb46751, 43 },
    { 0, 0 },
    { 0x21746753, 52 },
    { 0, 0 },
    { 0xbfd46755, 23 },
    { 0x53646755, 61 },
    { 0x79246757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84675b, 95 },
    { 0x3d24675b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9346760, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x14346763, 90 },
    { 0xf7d46763, 100 },
    { 0x9b746765, 77 },
    { 0x34346766, 33 },
    { 0xb8846765, 90 },
    { 0, 0 },
    { 0xe6446769, 23 },
    { 0, 0 },
    { 0xe034676b, 95 },
    { 0x2324676b, 61 },
    { 0x6f44676c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d46774, 23 },
    { 0x7c846774, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e4677a, 23 },
    { 0x4d14677a, 75 },
    { 0x2a44677c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9824677f, 55 },
    { 0xc194677f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a746784, 40 },
    { 0xde646784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f4678a, 33 },
    { 0xb904678a, 90 },
    { 0xb134678c, 76 },
    { 0xfb44678b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c46790, 55 },
    { 0x56e46791, 76 },
    { 0x4a346792, 76 },
    { 0xc3246793, 52 },
    { 0x41646794, 90 },
    { 0xfda46795, 66 },
    { 0x60546794, 61 },
    { 0x53d46797, 100 },
    { 0xa5f46798, 77 },
    { 0x7e046798, 33 },
    { 0, 0 },
    { 0x95b4679b, 76 },
    { 0x20b4679c, 99 },
    { 0xb024679d, 23 },
    { 0x794679e, 61 },
    { 0xb0f4679f, 98 },
    { 0xa954679f, 76 },
    { 0x7db4679b, 43 },
    { 0xe24679c, 46 },
    { 0xd3f467a3, 78 },
    { 0x233467a4, 43 },
    { 0, 0 },
    { 0xa40467a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9467a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61467b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a467b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe467b7, 33 },
    { 0, 0 },
    { 0x456467b9, 77 },
    { 0x501467ba, 34 },
    { 0, 0 },
    { 0xe80467bc, 43 },
    { 0xb2c467bd, 43 },
    { 0xbb1467be, 23 },
    { 0, 0 },
    { 0xf63467c0, 90 },
    { 0, 0 },
    { 0xaf6467c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c467c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6467ca, 23 },
    { 0xd84467cb, 23 },
    { 0x14c467ca, 66 },
    { 0xff3467cd, 61 },
    { 0x3c0467ce, 33 },
    { 0x406467cf, 80 },
    { 0xbeb467ce, 43 },
    { 0, 0 },
    { 0x592467d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4467d5, 99 },
    { 0x5bb467d6, 93 },
    { 0xd09467d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df467df, 88 },
    { 0x82c467df, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf467e4, 48 },
    { 0, 0 },
    { 0x507467e6, 75 },
    { 0xf99467e6, 77 },
    { 0x435467e8, 77 },
    { 0, 0 },
    { 0xf7c467ea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x484467ed, 55 },
    { 0, 0 },
    { 0x45467ef, 98 },
    { 0x1f6467ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd467f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1467fa, 99 },
    { 0x623467fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a46800, 76 },
    { 0x6b246800, 33 },
    { 0x14546800, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95546805, 100 },
    { 0xc4e46806, 61 },
    { 0xdb46807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x104680a, 61 },
    { 0x8234680b, 95 },
    { 0x5ce4680c, 33 },
    { 0x4d34680d, 100 },
    { 0x7ff4680e, 23 },
    { 0x73d4680d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df46812, 55 },
    { 0x66a46813, 95 },
    { 0xba146812, 95 },
    { 0xd2e46815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f946818, 33 },
    { 0xcaf46819, 80 },
    { 0xc1c4681a, 33 },
    { 0x2b4681b, 93 },
    { 0x5314681c, 95 },
    { 0, 0 },
    { 0x4404681e, 23 },
    { 0xdd24681f, 99 },
    { 0xfc54681e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4346826, 23 },
    { 0, 0 },
    { 0x84d46828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4334682e, 85 },
    { 0x2704682f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13346833, 66 },
    { 0, 0 },
    { 0x22f46835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65746839, 52 },
    { 0x1146839, 75 },
    { 0x2cf4683b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4683e, 85 },
    { 0xfee4683f, 33 },
    { 0xf2346840, 90 },
    { 0xfc246841, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1724684d, 87 },
    { 0x2c44684d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd546852, 23 },
    { 0, 0 },
    { 0x1a246854, 83 },
    { 0x6fa46854, 34 },
    { 0xea446856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b46859, 33 },
    { 0x6ea46859, 66 },
    { 0xa514685a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8764685f, 98 },
    { 0, 0 },
    { 0x3c546861, 87 },
    { 0, 0 },
    { 0x7046863, 68 },
    { 0x87c46864, 61 },
    { 0x7f346865, 87 },
    { 0x30846866, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f84686a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4686f, 46 },
    { 0xa1346870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f346873, 100 },
    { 0x8de46874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe94687b, 34 },
    { 0x9614687b, 76 },
    { 0x3584687d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54746883, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28146888, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9344688d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe246894, 84 },
    { 0, 0 },
    { 0x86746896, 87 },
    { 0, 0 },
    { 0x16f46898, 23 },
    { 0, 0 },
    { 0x1dd4689a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14689f, 23 },
    { 0xa73468a0, 23 },
    { 0xd5468a0, 87 },
    { 0x96f468a0, 83 },
    { 0x29d4689f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d468a6, 33 },
    { 0xabd468a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5468aa, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4468ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c468b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22468b4, 76 },
    { 0x8ca468b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc468bc, 98 },
    { 0, 0 },
    { 0xb62468be, 61 },
    { 0xb76468bf, 85 },
    { 0x1c9468be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b468c9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42468cd, 88 },
    { 0xb88468ce, 95 },
    { 0x46d468cd, 67 },
    { 0xca3468d0, 90 },
    { 0xd45468d1, 55 },
    { 0xae4468cf, 93 },
    { 0, 0 },
    { 0x1c468d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249468d9, 33 },
    { 0, 0 },
    { 0x138468db, 98 },
    { 0x8f5468dc, 90 },
    { 0x5ec468dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04468e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x147468e3, 95 },
    { 0x56b468e3, 61 },
    { 0, 0 },
    { 0x2a468e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b468e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905468ed, 98 },
    { 0xb45468ee, 100 },
    { 0x7cf468ee, 23 },
    { 0x66a468f0, 76 },
    { 0x2d7468ee, 76 },
    { 0xeb9468f2, 90 },
    { 0x352468f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6468f6, 100 },
    { 0, 0 },
    { 0xd7d468f8, 90 },
    { 0, 0 },
    { 0x1c4468fa, 98 },
    { 0xb63468fb, 33 },
    { 0x56c468fc, 61 },
    { 0x6d8468fc, 66 },
    { 0xc33468fd, 43 },
    { 0x7ff468ff, 61 },
    { 0x3f646900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c46903, 52 },
    { 0x7d746903, 98 },
    { 0x19546905, 23 },
    { 0x7d946904, 33 },
    { 0, 0 },
    { 0xe0446908, 55 },
    { 0xae446909, 33 },
    { 0x2904690a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14690d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73546912, 90 },
    { 0xf4c46912, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e46919, 78 },
    { 0xa894691a, 23 },
    { 0x8d74691b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c246920, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43346923, 68 },
    { 0x91146924, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e46929, 90 },
    { 0x3fc4692a, 55 },
    { 0x3df4692b, 61 },
    { 0x2b44692c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1046931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d46934, 61 },
    { 0x25546934, 90 },
    { 0xe5446934, 52 },
    { 0x5bb46934, 43 },
    { 0xfe146938, 90 },
    { 0x74746936, 90 },
    { 0x8e4693a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde546940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35146943, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b46948, 33 },
    { 0, 0 },
    { 0xff4694a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6094694e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7146951, 61 },
    { 0x89746952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93646957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4695a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf134695d, 53 },
    { 0xdf44695e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d46963, 85 },
    { 0x11f46964, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e74696d, 61 },
    { 0xac74696d, 23 },
    { 0x48b4696f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec46972, 46 },
    { 0x32f46972, 61 },
    { 0x92d46973, 33 },
    { 0xdeb46975, 23 },
    { 0xb7046974, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8164697a, 55 },
    { 0xb5a4697b, 98 },
    { 0, 0 },
    { 0x4364697d, 23 },
    { 0, 0 },
    { 0xcf34697f, 84 },
    { 0xa2946980, 33 },
    { 0xbc546980, 23 },
    { 0x8d46981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6246988, 76 },
    { 0xb7046989, 66 },
    { 0x3ec46989, 64 },
    { 0x7ac4698b, 33 },
    { 0x34d4698c, 90 },
    { 0x45b4698d, 55 },
    { 0x5cc4698d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf46994, 99 },
    { 0xa4146995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f46999, 95 },
    { 0, 0 },
    { 0x7274699b, 23 },
    { 0x234699c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127469a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41469a5, 90 },
    { 0x8e1469a6, 91 },
    { 0xbca469a6, 61 },
    { 0x5a5469a7, 76 },
    { 0xce6469a9, 33 },
    { 0xbd0469a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5469ad, 90 },
    { 0, 0 },
    { 0x776469af, 34 },
    { 0x854469af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79469b3, 90 },
    { 0, 0 },
    { 0x412469b5, 33 },
    { 0, 0 },
    { 0x82d469b7, 61 },
    { 0xc66469b7, 61 },
    { 0x956469b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155469be, 61 },
    { 0xe65469bf, 98 },
    { 0x626469c0, 66 },
    { 0, 0 },
    { 0x3ed469c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63469c7, 33 },
    { 0, 0 },
    { 0x4df469c9, 33 },
    { 0x3ba469ca, 76 },
    { 0, 0 },
    { 0x32f469cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b469d0, 23 },
    { 0xefd469d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df469d6, 90 },
    { 0x11f469d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e469da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67469de, 61 },
    { 0, 0 },
    { 0x459469e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c469e6, 90 },
    { 0x91b469e7, 76 },
    { 0, 0 },
    { 0xc81469e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405469f0, 61 },
    { 0, 0 },
    { 0xf0a469f2, 93 },
    { 0xa0a469f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee469f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19469fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91646a03, 90 },
    { 0xabe46a04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1246a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35246a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98646a1b, 85 },
    { 0xd346a1b, 95 },
    { 0, 0 },
    { 0xd4046a1e, 83 },
    { 0x19a46a1f, 90 },
    { 0x8fc46a1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5546a24, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x80146a27, 17 },
    { 0, 0 },
    { 0xbef46a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa746a31, 33 },
    { 0xadc46a31, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76046a38, 76 },
    { 0x86146a39, 90 },
    { 0xb6e46a38, 65 },
    { 0xb746a38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66146a3f, 76 },
    { 0, 0 },
    { 0x32046a41, 68 },
    { 0xaaf46a42, 67 },
    { 0x57c46a43, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c846a46, 90 },
    { 0x86646a46, 98 },
    { 0, 0 },
    { 0xe9146a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc246a4c, 90 },
    { 0x9346a4d, 98 },
    { 0, 0 },
    { 0xced46a4f, 46 },
    { 0xc146a50, 93 },
    { 0, 0 },
    { 0xc4b46a52, 55 },
    { 0, 0 },
    { 0xff046a54, 76 },
    { 0, 0 },
    { 0x74546a56, 90 },
    { 0x22846a57, 91 },
    { 0xb9646a57, 95 },
    { 0xf0346a59, 61 },
    { 0x4f446a59, 95 },
    { 0x65046a59, 95 },
    { 0xd7546a57, 23 },
    { 0xfcd46a56, 85 },
    { 0xaa346a5e, 95 },
    { 0x6c846a5a, 93 },
    { 0xd1046a5f, 90 },
    { 0xd8946a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d346a64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28146a68, 90 },
    { 0, 0 },
    { 0x86d46a6a, 78 },
    { 0x32346a6a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c46a70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce546a73, 93 },
    { 0, 0 },
    { 0xa8346a75, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7946a80, 33 },
    { 0x10246a80, 75 },
    { 0xb346a82, 78 },
    { 0x43246a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8546a88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20346a8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96246a95, 85 },
    { 0x9ec46a96, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f46a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3746a9d, 93 },
    { 0xac446a9d, 61 },
    { 0x81e46a9f, 90 },
    { 0, 0 },
    { 0x9c946aa1, 90 },
    { 0xb3846aa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20346aa5, 34 },
    { 0x6246aa5, 52 },
    { 0, 0 },
    { 0x7a946aa8, 23 },
    { 0, 0 },
    { 0x8fc46aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6046ab6, 33 },
    { 0xde046ab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f346abc, 84 },
    { 0xb7546abc, 55 },
    { 0, 0 },
    { 0x89546abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e46ac3, 66 },
    { 0xe9d46ac4, 91 },
    { 0, 0 },
    { 0xf1546ac6, 95 },
    { 0x23746ac7, 52 },
    { 0xec046ac8, 46 },
    { 0xddb46ac6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b046acc, 87 },
    { 0xbab46acd, 91 },
    { 0x65b46acc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45346ad4, 90 },
    { 0x42946ad4, 33 },
    { 0, 0 },
    { 0x30f46ad7, 33 },
    { 0x53546ad7, 90 },
    { 0x60346ad9, 99 },
    { 0x8fd46ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8246add, 66 },
    { 0x64546ade, 34 },
    { 0xa6646adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45146ae2, 33 },
    { 0x49246ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3546ae8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26346aed, 33 },
    { 0xf7246aee, 43 },
    { 0, 0 },
    { 0xab346af0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec46af5, 46 },
    { 0xd6846af5, 34 },
    { 0xe7746af7, 98 },
    { 0xbf346af8, 90 },
    { 0xc5c46af5, 55 },
    { 0, 0 },
    { 0x6d346afb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca346aff, 90 },
    { 0xb3a46b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f46b05, 93 },
    { 0x6a246b06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec46b0a, 34 },
    { 0xde446b0b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c146b0f, 98 },
    { 0, 0 },
    { 0x4146b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df46b15, 55 },
    { 0, 0 },
    { 0x7db46b17, 87 },
    { 0, 0 },
    { 0x28e46b19, 76 },
    { 0xf9b46b19, 64 },
    { 0x1e646b1b, 23 },
    { 0xdf746b1c, 33 },
    { 0x34b46b1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd46b20, 53 },
    { 0, 0 },
    { 0xd846b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47046b26, 93 },
    { 0xe1a46b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f46b2f, 90 },
    { 0x78c46b30, 34 },
    { 0, 0 },
    { 0x44346b32, 90 },
    { 0xd0746b33, 95 },
    { 0xe446b34, 33 },
    { 0x52a46b33, 61 },
    { 0xfdc46b35, 61 },
    { 0x3446b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e446b3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9646b45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c46b49, 76 },
    { 0xb3446b4a, 84 },
    { 0x72546b4a, 61 },
    { 0xb5b46b4b, 95 },
    { 0x84346b4a, 23 },
    { 0xcef46b4e, 33 },
    { 0x15246b4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d46b52, 95 },
    { 0x1f46b52, 55 },
    { 0, 0 },
    { 0x75d46b55, 61 },
    { 0x58a46b56, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f46b59, 55 },
    { 0, 0 },
    { 0xe6146b5b, 90 },
    { 0x7e46b5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad46b69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e46b6c, 93 },
    { 0, 0 },
    { 0xd2446b6e, 77 },
    { 0xb9d46b6e, 55 },
    { 0, 0 },
    { 0xfea46b71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae046b75, 33 },
    { 0x95c46b75, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf46b7d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d46b84, 90 },
    { 0xc5d46b85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2146b88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64246b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7546b8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59746b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846b98, 34 },
    { 0x9d646b99, 99 },
    { 0xa1646b99, 23 },
    { 0x1c646b9b, 23 },
    { 0x9fc46b9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0446b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c46ba4, 78 },
    { 0x86b46ba5, 98 },
    { 0, 0 },
    { 0x5b546ba7, 98 },
    { 0x3a146ba8, 34 },
    { 0xb2846ba9, 34 },
    { 0xcf946baa, 90 },
    { 0xdd846baa, 33 },
    { 0x64f46bab, 76 },
    { 0x35646baa, 61 },
    { 0x2f546bae, 61 },
    { 0xa6846baf, 80 },
    { 0x62f46bb0, 95 },
    { 0x41946bb1, 61 },
    { 0x8e46bae, 100 },
    { 0x5e746bab, 90 },
    { 0xa1546bb4, 84 },
    { 0xc4446bb4, 33 },
    { 0x8f346bb4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32146bbb, 70 },
    { 0x58446bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5046bbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede46bc4, 55 },
    { 0, 0 },
    { 0x19c46bc6, 66 },
    { 0, 0 },
    { 0x33846bc8, 23 },
    { 0x91946bc8, 55 },
    { 0xde546bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff46bcd, 93 },
    { 0x28246bce, 66 },
    { 0x47646bce, 33 },
    { 0x4a346bcd, 33 },
    { 0, 0 },
    { 0xb1446bd2, 46 },
    { 0x28e46bd3, 98 },
    { 0x7d246bd2, 90 },
    { 0, 0 },
    { 0x57646bd6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f246bda, 84 },
    { 0x47646bda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38246be0, 23 },
    { 0x52b46be0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c46beb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e46bee, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2046bf1, 33 },
    { 0x2dc46bf2, 33 },
    { 0, 0 },
    { 0xf1f46bf4, 76 },
    { 0, 0 },
    { 0x83346bf6, 95 },
    { 0, 0 },
    { 0xac946bf8, 33 },
    { 0xf2f46bf9, 33 },
    { 0, 0 },
    { 0x4246bfb, 61 },
    { 0x90b46bfc, 33 },
    { 0x1046bfc, 23 },
    { 0xbe046bfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e46c08, 90 },
    { 0, 0 },
    { 0xfca46c0a, 65 },
    { 0x54946c0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb46c0e, 81 },
    { 0x46a46c0f, 84 },
    { 0x58246c10, 83 },
    { 0xa3f46c11, 66 },
    { 0x55f46c10, 23 },
    { 0x95046c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e46c17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e346c1d, 95 },
    { 0xea446c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b746c23, 95 },
    { 0xe3346c24, 23 },
    { 0x1f746c23, 64 },
    { 0xfb746c26, 69 },
    { 0x18246c27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb46c2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18746c2f, 66 },
    { 0x28d46c30, 99 },
    { 0x8f346c31, 23 },
    { 0xb7646c32, 90 },
    { 0x4a646c33, 23 },
    { 0x7a546c34, 55 },
    { 0xd5e46c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e46c3e, 76 },
    { 0, 0 },
    { 0x46446c40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88946c45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b846c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa246c50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146c53, 52 },
    { 0, 0 },
    { 0xc3546c55, 61 },
    { 0, 0 },
    { 0xe8446c57, 61 },
    { 0, 0 },
    { 0x57c46c59, 23 },
    { 0x1e146c59, 76 },
    { 0x25146c59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be46c60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e546c63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37646c67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f46c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22146c74, 90 },
    { 0xfaa46c75, 95 },
    { 0xe3e46c76, 90 },
    { 0, 0 },
    { 0xb3646c78, 90 },
    { 0x81c46c78, 76 },
    { 0xd7c46c78, 55 },
    { 0x5bc46c7b, 90 },
    { 0xe5a46c78, 65 },
    { 0, 0 },
    { 0x8c646c7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f46c84, 98 },
    { 0x4ee46c85, 76 },
    { 0xb3046c86, 34 },
    { 0xa3b46c86, 90 },
    { 0x37c46c88, 61 },
    { 0xc0146c86, 76 },
    { 0x47246c89, 61 },
    { 0xbc246c8b, 66 },
    { 0x7ad46c86, 23 },
    { 0, 0 },
    { 0x6ee46c8e, 61 },
    { 0, 0 },
    { 0x57746c90, 95 },
    { 0, 0 },
    { 0x9e146c92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85546c95, 98 },
    { 0x59646c96, 43 },
    { 0x62046c97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c446c9b, 66 },
    { 0, 0 },
    { 0x98946c9d, 100 },
    { 0x64846c9e, 99 },
    { 0x20a46c9e, 55 },
    { 0, 0 },
    { 0xbb046ca1, 90 },
    { 0xd7346ca2, 78 },
    { 0, 0 },
    { 0xf5446ca4, 43 },
    { 0xaea46ca4, 36 },
    { 0x89b46ca6, 95 },
    { 0x97646ca7, 76 },
    { 0xb3146ca8, 99 },
    { 0x1b246ca6, 90 },
    { 0x7e046ca6, 23 },
    { 0x4dd46cab, 84 },
    { 0x33246cab, 67 },
    { 0x4a146cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd446cb2, 90 },
    { 0xeb946cb3, 53 },
    { 0x4b246cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d46cb9, 95 },
    { 0x55d46cb9, 46 },
    { 0x4b146cb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x18046cbe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7946cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40946cc6, 90 },
    { 0x13846cc7, 98 },
    { 0x5d546cc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4846cce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96346cd4, 23 },
    { 0x36146cd4, 52 },
    { 0x3a546cd6, 33 },
    { 0, 0 },
    { 0xdf046cd8, 91 },
    { 0x2d046cd9, 81 },
    { 0x19a46cda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a46ce0, 33 },
    { 0xdd946ce0, 61 },
    { 0, 0 },
    { 0x90646ce3, 55 },
    { 0x33246ce4, 55 },
    { 0, 0 },
    { 0xdb046ce6, 95 },
    { 0x6bb46ce7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d46cea, 33 },
    { 0, 0 },
    { 0x37846cec, 23 },
    { 0x53146ced, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x78646cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3246cf6, 88 },
    { 0, 0 },
    { 0x59e46cf8, 90 },
    { 0, 0 },
    { 0xf4346cfa, 75 },
    { 0x1f146cfb, 98 },
    { 0, 0 },
    { 0x38446cfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57746d03, 99 },
    { 0, 0 },
    { 0x30746d05, 46 },
    { 0, 0 },
    { 0xbf546d07, 61 },
    { 0xc4846d07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce46d0b, 95 },
    { 0x2b146d0b, 68 },
    { 0x63d46d0b, 33 },
    { 0xa5b46d0e, 17 },
    { 0x69f46d0b, 43 },
    { 0x71f46d10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb046d1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1546d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x79646d23, 66 },
    { 0, 0 },
    { 0xfcc46d25, 23 },
    { 0xb9946d26, 95 },
    { 0xd8d46d27, 38 },
    { 0x88b46d25, 43 },
    { 0x7f846d29, 36 },
    { 0x22e46d2a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5246d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25846d31, 46 },
    { 0, 0 },
    { 0xc0646d33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d046d39, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb46d3e, 23 },
    { 0x27f46d3f, 85 },
    { 0xf4a46d40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af46d49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58946d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf46d56, 100 },
    { 0, 0 },
    { 0xdf846d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30846d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85046d68, 93 },
    { 0x52e46d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4946d77, 66 },
    { 0xe246d78, 90 },
    { 0x7c846d79, 98 },
    { 0, 0 },
    { 0x6146d7b, 99 },
    { 0, 0 },
    { 0x38446d7d, 76 },
    { 0xbe946d7d, 33 },
    { 0, 0 },
    { 0x3d246d80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e946d8d, 46 },
    { 0x8da46d8e, 84 },
    { 0xf0546d8e, 95 },
    { 0xcfa46d90, 55 },
    { 0x81e46d90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f46d95, 83 },
    { 0x6d846d96, 76 },
    { 0x4ee46d96, 34 },
    { 0x73f46d98, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d46d9b, 23 },
    { 0xe4a46d9c, 23 },
    { 0, 0 },
    { 0xb7146d9e, 23 },
    { 0x22c46d9e, 61 },
    { 0, 0 },
    { 0xb0446da1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88646da8, 95 },
    { 0x3ae46da8, 90 },
    { 0x49b46daa, 95 },
    { 0xb6546daa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba46dae, 69 },
    { 0xdb946dae, 91 },
    { 0, 0 },
    { 0x4e146db1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7946db4, 56 },
    { 0, 0 },
    { 0x73246db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47346dbb, 93 },
    { 0x21046dbc, 55 },
    { 0xa3c46dbc, 90 },
    { 0xd8946dbe, 33 },
    { 0x1d046dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646dc4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be46dc7, 90 },
    { 0, 0 },
    { 0x8e346dc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7746dcf, 95 },
    { 0, 0 },
    { 0xa8746dd1, 68 },
    { 0, 0 },
    { 0x25d46dd3, 90 },
    { 0, 0 },
    { 0xe6546dd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4146dd8, 98 },
    { 0xe5846dd9, 76 },
    { 0xb5e46dd8, 76 },
    { 0xadc46dd8, 90 },
    { 0xca046dd9, 96 },
    { 0xc9046ddd, 95 },
    { 0xa6c46dde, 33 },
    { 0x36f46ddb, 43 },
    { 0, 0 },
    { 0xc4c46de1, 76 },
    { 0x24f46de2, 55 },
    { 0x28c46de3, 98 },
    { 0x3d946de2, 23 },
    { 0x3146de2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91746de9, 84 },
    { 0xf3146de9, 98 },
    { 0xaf046deb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db46def, 34 },
    { 0x53146def, 98 },
    { 0xcca46df0, 43 },
    { 0xfd746df0, 23 },
    { 0x82d46df1, 90 },
    { 0x92646df4, 95 },
    { 0x87d46df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a46df8, 100 },
    { 0xdfe46df8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef846dfc, 52 },
    { 0x3aa46dfd, 33 },
    { 0xfe46dfe, 85 },
    { 0x11246dff, 77 },
    { 0xf9c46e00, 61 },
    { 0, 0 },
    { 0xdbc46e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f46e0c, 99 },
    { 0xb5d46e0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9646e1c, 76 },
    { 0xe1546e1d, 98 },
    { 0, 0 },
    { 0x27946e1f, 46 },
    { 0x42146e20, 66 },
    { 0xd6846e1f, 61 },
    { 0, 0 },
    { 0xf7f46e23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40146e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2846e2c, 90 },
    { 0x22846e2d, 33 },
    { 0, 0 },
    { 0xe2646e2f, 95 },
    { 0x77946e30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd46e39, 46 },
    { 0xbe646e39, 90 },
    { 0x9d946e3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f46e43, 67 },
    { 0, 0 },
    { 0x49946e45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a946e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3646e51, 90 },
    { 0x8d246e51, 43 },
    { 0xaaf46e53, 84 },
    { 0, 0 },
    { 0x28246e55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2946e5a, 90 },
    { 0, 0 },
    { 0x5bc46e5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b946e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b46e63, 73 },
    { 0xc1946e63, 23 },
    { 0x30a46e65, 23 },
    { 0xb646e65, 33 },
    { 0x3a246e67, 52 },
    { 0xf4946e68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32646e79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22146e80, 55 },
    { 0xcbf46e81, 46 },
    { 0x11146e82, 90 },
    { 0xb5846e83, 90 },
    { 0x3b046e83, 23 },
    { 0x86946e81, 33 },
    { 0xc1f46e80, 23 },
    { 0, 0 },
    { 0x59d46e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42346e93, 33 },
    { 0, 0 },
    { 0xe246e95, 99 },
    { 0, 0 },
    { 0xbf646e97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf46e9a, 91 },
    { 0xe8346e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e46ea6, 76 },
    { 0x2c146ea7, 76 },
    { 0xd0e46ea6, 98 },
    { 0, 0 },
    { 0x50846eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce646eb4, 34 },
    { 0xc8646eb5, 33 },
    { 0x85146eb6, 23 },
    { 0x3d446eb6, 88 },
    { 0x7a46eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12746ebb, 98 },
    { 0xff946ebc, 34 },
    { 0, 0 },
    { 0x56446ebe, 55 },
    { 0, 0 },
    { 0x55e46ec0, 90 },
    { 0, 0 },
    { 0x96f46ec2, 34 },
    { 0, 0 },
    { 0xe0346ec4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac46ec8, 90 },
    { 0x21446ec9, 75 },
    { 0x2cd46eca, 90 },
    { 0, 0 },
    { 0x2f246ecc, 90 },
    { 0xa4046ecd, 34 },
    { 0x88f46ecc, 33 },
    { 0x21646ecf, 61 },
    { 0xb2446ece, 90 },
    { 0x4a46ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47946ed6, 90 },
    { 0x30646ed7, 34 },
    { 0x78546ed7, 66 },
    { 0, 0 },
    { 0xb5a46eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5246edf, 67 },
    { 0, 0 },
    { 0xe3b46ee1, 33 },
    { 0, 0 },
    { 0x82846ee3, 93 },
    { 0x5046ee3, 90 },
    { 0xba246ee5, 33 },
    { 0, 0 },
    { 0x22146ee7, 67 },
    { 0, 0 },
    { 0xb7446ee9, 95 },
    { 0x97a46ee9, 55 },
    { 0x2bf46eeb, 55 },
    { 0xbbf46eec, 55 },
    { 0xf7146ee9, 34 },
    { 0x10646eee, 38 },
    { 0x8fe46ee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60946ef6, 90 },
    { 0xf5046ef7, 67 },
    { 0, 0 },
    { 0xa8046ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc446f01, 98 },
    { 0xef646f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c46f05, 61 },
    { 0, 0 },
    { 0xb7246f07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58946f0d, 90 },
    { 0x5c246f0e, 33 },
    { 0x46d46f0e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x58546f12, 91 },
    { 0x18d46f12, 55 },
    { 0x1c146f14, 98 },
    { 0x2de46f15, 95 },
    { 0x2c346f13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f46f19, 46 },
    { 0xcdf46f19, 33 },
    { 0xf0646f1a, 33 },
    { 0x59846f1c, 98 },
    { 0xeb746f1d, 61 },
    { 0xb8e46f1e, 76 },
    { 0xb5046f1e, 23 },
    { 0xfe246f20, 98 },
    { 0xf946f20, 33 },
    { 0x71246f22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44946f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8346f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14446f30, 33 },
    { 0, 0 },
    { 0xb5f46f32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45246f3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d46f3e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e46f42, 67 },
    { 0x82046f43, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e46f49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50046f4d, 95 },
    { 0, 0 },
    { 0x14a46f4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25446f52, 23 },
    { 0x5d946f53, 84 },
    { 0xc5046f54, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72546f59, 90 },
    { 0x83946f5a, 23 },
    { 0x19446f5a, 61 },
    { 0x88846f5b, 55 },
    { 0x2c346f5d, 61 },
    { 0, 0 },
    { 0x13846f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db46f62, 95 },
    { 0x56b46f63, 77 },
    { 0x1fc46f64, 46 },
    { 0, 0 },
    { 0x17446f66, 90 },
    { 0xd4946f67, 90 },
    { 0xcaf46f68, 91 },
    { 0x93a46f67, 52 },
    { 0x23646f6a, 23 },
    { 0xb2a46f6b, 45 },
    { 0, 0 },
    { 0xdc046f6d, 52 },
    { 0x78246f6d, 95 },
    { 0xa2b46f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb46f78, 33 },
    { 0xfdf46f79, 61 },
    { 0xdaa46f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d46f7f, 78 },
    { 0xaaa46f80, 76 },
    { 0xaf146f81, 99 },
    { 0, 0 },
    { 0x9146f83, 90 },
    { 0xbfc46f83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1346f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61046f8c, 90 },
    { 0, 0 },
    { 0x11246f8e, 23 },
    { 0, 0 },
    { 0x8ba46f90, 61 },
    { 0x87f46f90, 100 },
    { 0xee046f92, 33 },
    { 0xac846f92, 87 },
    { 0xb4046f93, 81 },
    { 0xee046f93, 33 },
    { 0xf1746f96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2546f9d, 23 },
    { 0x1646f9e, 23 },
    { 0x16d46f9e, 95 },
    { 0, 0 },
    { 0x5e146fa1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d46fa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87846fae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b46fb1, 33 },
    { 0x7ba46fb1, 98 },
    { 0x22c46fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a46fb9, 33 },
    { 0x2ed46fb9, 84 },
    { 0x60746fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba046fbf, 55 },
    { 0, 0 },
    { 0xd1a46fc1, 52 },
    { 0x74046fc2, 61 },
    { 0xca446fc2, 34 },
    { 0, 0 },
    { 0xdd046fc5, 95 },
    { 0xc8f46fc5, 25 },
    { 0, 0 },
    { 0x8a146fc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf046fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646fd1, 93 },
    { 0, 0 },
    { 0x3e446fd3, 36 },
    { 0x13146fd4, 38 },
    { 0x1c346fd5, 43 },
    { 0xf9d46fd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda746fdf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95846fe3, 66 },
    { 0x5ba46fe3, 76 },
    { 0xdf346fe5, 34 },
    { 0xb4346fe6, 95 },
    { 0xd8c46fe6, 23 },
    { 0xebd46fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98146feb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be46ff0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf46ff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b46ff8, 95 },
    { 0xa4246ff9, 61 },
    { 0xbb546ffa, 40 },
    { 0x78d46ffb, 33 },
    { 0xc9446ffb, 68 },
    { 0, 0 },
    { 0x6c046ffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f47001, 76 },
    { 0x2747002, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea47007, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30747010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a47015, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf47018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284701c, 66 },
    { 0x2314701c, 33 },
    { 0x8d34701e, 90 },
    { 0xf3f4701f, 23 },
    { 0, 0 },
    { 0x84947021, 46 },
    { 0xa7047022, 98 },
    { 0, 0 },
    { 0x51e47024, 90 },
    { 0x22c47025, 34 },
    { 0x2be47026, 33 },
    { 0x28d47027, 85 },
    { 0x16247026, 61 },
    { 0xcb947025, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a47032, 61 },
    { 0, 0 },
    { 0x32847034, 95 },
    { 0xe947035, 84 },
    { 0x46447036, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b4703b, 61 },
    { 0x17f4703c, 98 },
    { 0x3bb4703d, 61 },
    { 0x48c4703d, 78 },
    { 0xe204703d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d247044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7747049, 34 },
    { 0x8ee4704a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a47053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63247058, 23 },
    { 0x3f847059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d34705c, 84 },
    { 0, 0 },
    { 0x9cf4705e, 33 },
    { 0xfb04705f, 90 },
    { 0x347060, 23 },
    { 0, 0 },
    { 0x61647062, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef47065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd4706d, 84 },
    { 0xf264706e, 93 },
    { 0, 0 },
    { 0x93747070, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd547076, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e64707b, 77 },
    { 0, 0 },
    { 0x3f94707d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68147080, 61 },
    { 0x69f47080, 76 },
    { 0xad847082, 61 },
    { 0x73c47083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847087, 61 },
    { 0x71347088, 85 },
    { 0xbbe47089, 55 },
    { 0xa4147088, 23 },
    { 0x25d47089, 23 },
    { 0x1b44708c, 23 },
    { 0, 0 },
    { 0x8d14708e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf47093, 75 },
    { 0xea47094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a4709c, 33 },
    { 0x96e4709c, 36 },
    { 0, 0 },
    { 0xe974709f, 100 },
    { 0xd5b470a0, 76 },
    { 0xcc3470a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307470a9, 46 },
    { 0, 0 },
    { 0x430470ab, 46 },
    { 0x113470ab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5470b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8470c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9470c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e470cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb470d0, 76 },
    { 0xdb4470d1, 23 },
    { 0x177470d2, 23 },
    { 0x944470d1, 91 },
    { 0x385470d3, 98 },
    { 0x91d470d1, 43 },
    { 0x8d6470d6, 76 },
    { 0x81c470d6, 76 },
    { 0x5ea470d8, 76 },
    { 0xea4470d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43470e5, 88 },
    { 0x635470e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x425470e9, 61 },
    { 0x21d470e9, 55 },
    { 0x240470eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d470f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13747100, 77 },
    { 0xa2847101, 95 },
    { 0, 0 },
    { 0xa2147103, 34 },
    { 0x3147104, 23 },
    { 0, 0 },
    { 0x73447106, 98 },
    { 0, 0 },
    { 0x34047108, 34 },
    { 0x99b47109, 23 },
    { 0xd0e4710a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b4710e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67347111, 48 },
    { 0x7c847112, 33 },
    { 0x43e47111, 90 },
    { 0x6ae47114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc47117, 75 },
    { 0, 0 },
    { 0xfb547119, 76 },
    { 0, 0 },
    { 0xc044711b, 34 },
    { 0, 0 },
    { 0xb664711d, 98 },
    { 0x5a14711e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13547123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4947132, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa47139, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3854713e, 95 },
    { 0, 0 },
    { 0x81747140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf147143, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a447149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4714c, 90 },
    { 0xac94714d, 23 },
    { 0xf754714e, 77 },
    { 0x7e54714f, 33 },
    { 0, 0 },
    { 0xea047151, 46 },
    { 0x6c147151, 96 },
    { 0, 0 },
    { 0x2a847154, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a84715d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0647162, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae47166, 90 },
    { 0x50c47167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc774716b, 52 },
    { 0x9854716c, 95 },
    { 0x4da4716c, 90 },
    { 0x4b44716e, 61 },
    { 0xf854716e, 67 },
    { 0x1184716b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6447177, 99 },
    { 0x17b47178, 76 },
    { 0xe5b47177, 34 },
    { 0xaf947179, 76 },
    { 0xdd47179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044717e, 34 },
    { 0xe244717f, 66 },
    { 0xc124717f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70747186, 61 },
    { 0, 0 },
    { 0xdf647188, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe124718b, 88 },
    { 0x77d4718c, 90 },
    { 0x344718c, 90 },
    { 0x4624718c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f47193, 93 },
    { 0xcc747194, 94 },
    { 0x3b547194, 84 },
    { 0x75d47196, 90 },
    { 0x18647197, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2114719a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454719f, 52 },
    { 0xc744719f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51471a6, 90 },
    { 0xf6a471a6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94471aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be471b7, 93 },
    { 0, 0 },
    { 0x8ae471b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x544471bc, 81 },
    { 0xb23471bc, 80 },
    { 0xdb9471bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa471c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702471c8, 33 },
    { 0x52471c9, 90 },
    { 0x92d471ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x336471cd, 95 },
    { 0, 0 },
    { 0xc21471cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b471de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53471e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b471ea, 76 },
    { 0, 0 },
    { 0x327471ec, 90 },
    { 0x9a471ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2471f0, 85 },
    { 0, 0 },
    { 0x305471f2, 100 },
    { 0x91c471f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11471f6, 55 },
    { 0x655471f7, 77 },
    { 0xd66471f7, 90 },
    { 0xb70471f9, 90 },
    { 0xac2471f9, 97 },
    { 0xe7e471fb, 65 },
    { 0xac2471fb, 90 },
    { 0x430471fd, 61 },
    { 0x93d471fb, 98 },
    { 0x48f471fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e47202, 55 },
    { 0xcc047202, 93 },
    { 0, 0 },
    { 0x60d47205, 43 },
    { 0xc5a47206, 88 },
    { 0xe6947207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14720b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71947211, 87 },
    { 0, 0 },
    { 0xce547213, 90 },
    { 0x79547214, 33 },
    { 0xac947215, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb747218, 61 },
    { 0xc4947219, 81 },
    { 0, 0 },
    { 0x3b64721b, 61 },
    { 0x1654721c, 61 },
    { 0, 0 },
    { 0x8f94721e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e647221, 93 },
    { 0xd1e47221, 95 },
    { 0x93247223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864722a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c64722e, 84 },
    { 0xeda4722f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0247232, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f47238, 90 },
    { 0x31147239, 33 },
    { 0xc3b47239, 61 },
    { 0x7e147238, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d647241, 69 },
    { 0xe3447241, 37 },
    { 0x6d447243, 77 },
    { 0x5447244, 55 },
    { 0x7447244, 33 },
    { 0x27847246, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4724a, 98 },
    { 0x60a4724a, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90547252, 23 },
    { 0xa9147252, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b47256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63547259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd24725c, 40 },
    { 0x1664725d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c547260, 61 },
    { 0xf8e47261, 98 },
    { 0xbc947262, 76 },
    { 0, 0 },
    { 0x76f47264, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6684726f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca47274, 55 },
    { 0xff47275, 98 },
    { 0, 0 },
    { 0xc7e47277, 67 },
    { 0xfd347278, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc64727c, 46 },
    { 0x7cd4727d, 90 },
    { 0xbbc4727d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e847283, 96 },
    { 0, 0 },
    { 0x3fb47285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb14728b, 17 },
    { 0x8594728c, 90 },
    { 0x5514728d, 33 },
    { 0x6e74728e, 95 },
    { 0xc4d4728f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1047296, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074729e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a472a8, 33 },
    { 0, 0 },
    { 0x56d472aa, 55 },
    { 0, 0 },
    { 0x3f0472ac, 43 },
    { 0, 0 },
    { 0x86b472ae, 98 },
    { 0x3ba472af, 31 },
    { 0x125472b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b472b6, 78 },
    { 0xcde472b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a472bb, 52 },
    { 0xe6c472bb, 90 },
    { 0, 0 },
    { 0x3d1472be, 91 },
    { 0xdf5472be, 34 },
    { 0x890472c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61472c6, 90 },
    { 0xa6f472c6, 96 },
    { 0xe3d472c8, 23 },
    { 0xcf6472c9, 93 },
    { 0xcfd472ca, 23 },
    { 0xf35472c6, 76 },
    { 0x1fa472cc, 34 },
    { 0, 0 },
    { 0xa35472ce, 90 },
    { 0x84b472cf, 95 },
    { 0, 0 },
    { 0xadd472d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd472db, 69 },
    { 0x8a6472dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41472e0, 33 },
    { 0x2bd472e0, 91 },
    { 0xbe1472e2, 33 },
    { 0, 0 },
    { 0x2e472e4, 23 },
    { 0x912472e5, 33 },
    { 0x8f2472e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49147300, 23 },
    { 0, 0 },
    { 0x14747302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6147309, 55 },
    { 0xcae47309, 40 },
    { 0xaeb4730a, 23 },
    { 0x56a4730c, 43 },
    { 0x124730d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a347310, 98 },
    { 0xcdb47310, 61 },
    { 0x9ed47312, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63847317, 52 },
    { 0, 0 },
    { 0xe4147319, 76 },
    { 0, 0 },
    { 0x3e24731b, 67 },
    { 0x4364731b, 55 },
    { 0xc964731d, 43 },
    { 0, 0 },
    { 0x2f14731f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21247323, 55 },
    { 0x32547324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b47327, 68 },
    { 0xeb447328, 95 },
    { 0x30947328, 61 },
    { 0xa1947329, 98 },
    { 0, 0 },
    { 0x9ee4732c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79147331, 84 },
    { 0xdc047332, 90 },
    { 0x21447333, 33 },
    { 0xca047333, 23 },
    { 0, 0 },
    { 0x4b047336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14733a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944733d, 43 },
    { 0x8f64733d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16747345, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c147349, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6954734d, 23 },
    { 0x59b4734e, 23 },
    { 0x4904734e, 34 },
    { 0xa924734f, 90 },
    { 0x75047351, 87 },
    { 0xbd147352, 33 },
    { 0x8ff47353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c4735d, 33 },
    { 0, 0 },
    { 0x5bb4735f, 59 },
    { 0x23147360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc347363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a147367, 23 },
    { 0x5747367, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4544736c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1047376, 23 },
    { 0x3e447377, 61 },
    { 0, 0 },
    { 0xa5447379, 61 },
    { 0xd164737a, 82 },
    { 0xd7b4737b, 61 },
    { 0, 0 },
    { 0xf4737d, 95 },
    { 0xdf14737e, 33 },
    { 0, 0 },
    { 0x92447380, 46 },
    { 0xf9547381, 95 },
    { 0xb0847382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc747386, 76 },
    { 0x2b247386, 93 },
    { 0xdeb47388, 98 },
    { 0x90247389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2964738d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e47391, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c47396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1304739a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5473a0, 93 },
    { 0, 0 },
    { 0x640473a2, 65 },
    { 0x955473a2, 61 },
    { 0xa4473a2, 33 },
    { 0, 0 },
    { 0x1e473a6, 75 },
    { 0x31f473a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb473ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545473b0, 40 },
    { 0xdcb473b1, 68 },
    { 0, 0 },
    { 0x2f6473b3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf473c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a473c6, 23 },
    { 0x92d473c7, 90 },
    { 0x84e473c8, 95 },
    { 0xcb5473c9, 46 },
    { 0xc4d473ca, 99 },
    { 0x4ed473ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7473ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508473d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3473d5, 23 },
    { 0, 0 },
    { 0xc473d7, 90 },
    { 0, 0 },
    { 0xa94473d9, 95 },
    { 0xff2473da, 98 },
    { 0x2ed473db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53473e7, 61 },
    { 0x19a473e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412473ef, 34 },
    { 0, 0 },
    { 0x706473f1, 84 },
    { 0, 0 },
    { 0x805473f3, 95 },
    { 0x852473f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340473f8, 78 },
    { 0x401473f9, 98 },
    { 0xce2473f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea47402, 55 },
    { 0x80f47403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e047406, 90 },
    { 0, 0 },
    { 0x44347408, 90 },
    { 0x81c47409, 87 },
    { 0xbb647408, 43 },
    { 0xe054740b, 95 },
    { 0x6f34740c, 36 },
    { 0x5d34740d, 77 },
    { 0x6964740e, 23 },
    { 0x1b44740f, 23 },
    { 0x51447410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d947419, 84 },
    { 0xb674741a, 74 },
    { 0x44c47419, 76 },
    { 0xec14741c, 76 },
    { 0x90f4741d, 85 },
    { 0xfdd4741c, 90 },
    { 0x7474741f, 61 },
    { 0x87547420, 84 },
    { 0xb464741e, 95 },
    { 0x3724741c, 43 },
    { 0x4ef4741b, 44 },
    { 0x64547424, 40 },
    { 0, 0 },
    { 0x31347426, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4144742a, 23 },
    { 0x1044742a, 55 },
    { 0, 0 },
    { 0x7334742d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc47431, 55 },
    { 0x39247431, 55 },
    { 0, 0 },
    { 0x7b747434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea47437, 88 },
    { 0x30847438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4743b, 93 },
    { 0x5624743b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b47441, 87 },
    { 0x2b347442, 33 },
    { 0x44047443, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb4744a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad4744f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a47456, 23 },
    { 0, 0 },
    { 0xaba47458, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d4745b, 96 },
    { 0xe2f4745c, 76 },
    { 0xa7e4745c, 61 },
    { 0, 0 },
    { 0x8a24745f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8147462, 98 },
    { 0x17847463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18247466, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134746a, 61 },
    { 0xc544746b, 61 },
    { 0xcf24746b, 100 },
    { 0, 0 },
    { 0xcdd4746e, 98 },
    { 0xcbd4746e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x62247472, 99 },
    { 0x94947473, 98 },
    { 0, 0 },
    { 0xd6447475, 68 },
    { 0, 0 },
    { 0x71f47477, 78 },
    { 0x4447478, 98 },
    { 0x5e047479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904747d, 76 },
    { 0xdb54747e, 23 },
    { 0xfef4747d, 61 },
    { 0x78947480, 98 },
    { 0, 0 },
    { 0x83c47482, 68 },
    { 0, 0 },
    { 0xb5e47484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c47489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d54748c, 68 },
    { 0x3ff4748c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8547491, 33 },
    { 0x92047492, 61 },
    { 0x8b447492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d47496, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec47499, 46 },
    { 0, 0 },
    { 0xf074749b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6474a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5474a8, 90 },
    { 0xa13474a9, 90 },
    { 0, 0 },
    { 0x9a6474ab, 100 },
    { 0, 0 },
    { 0xfe1474ad, 38 },
    { 0xe26474ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7474b4, 34 },
    { 0x759474b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1474bd, 24 },
    { 0x621474be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758474c3, 76 },
    { 0, 0 },
    { 0xec0474c5, 98 },
    { 0, 0 },
    { 0xc51474c7, 95 },
    { 0xda474c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88474cd, 90 },
    { 0, 0 },
    { 0x258474cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3474d3, 91 },
    { 0x41d474d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458474da, 33 },
    { 0, 0 },
    { 0x27474dc, 90 },
    { 0, 0 },
    { 0xb7474de, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x32474e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b474e5, 33 },
    { 0x4c474e6, 95 },
    { 0xce4474e7, 95 },
    { 0xeed474e8, 90 },
    { 0x4f3474e8, 33 },
    { 0x7b474ea, 78 },
    { 0xb10474eb, 33 },
    { 0x3a5474e7, 43 },
    { 0x47c474e5, 61 },
    { 0xab7474ee, 40 },
    { 0x884474ef, 43 },
    { 0x6f5474e8, 90 },
    { 0x91c474f1, 33 },
    { 0xa54474f1, 90 },
    { 0x9fc474f3, 93 },
    { 0xb10474f4, 36 },
    { 0x81a474f5, 90 },
    { 0x4df474f6, 61 },
    { 0xfb3474f5, 90 },
    { 0x177474f8, 93 },
    { 0xbda474f2, 90 },
    { 0x2b2474e7, 67 },
    { 0, 0 },
    { 0x923474fc, 78 },
    { 0x2ea474fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc47500, 55 },
    { 0xb047500, 33 },
    { 0xd3447502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f47505, 90 },
    { 0, 0 },
    { 0x72e47507, 80 },
    { 0x7a647508, 33 },
    { 0x6b547509, 68 },
    { 0, 0 },
    { 0x9da4750b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e847510, 81 },
    { 0x66247510, 52 },
    { 0, 0 },
    { 0x26747513, 33 },
    { 0, 0 },
    { 0x6b47515, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb447518, 88 },
    { 0x18647518, 90 },
    { 0xa9a47518, 61 },
    { 0x18c47519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be4751f, 98 },
    { 0xf0247520, 55 },
    { 0, 0 },
    { 0xaee47522, 76 },
    { 0xdbe47522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b047528, 99 },
    { 0, 0 },
    { 0x4cd4752a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394752e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f447532, 90 },
    { 0x18b47533, 46 },
    { 0, 0 },
    { 0x62847535, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf234753a, 33 },
    { 0, 0 },
    { 0xc824753c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a47540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a47543, 90 },
    { 0xeb147543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b47548, 76 },
    { 0, 0 },
    { 0x7ef4754a, 23 },
    { 0x3a34754b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94754e, 52 },
    { 0xe704754f, 91 },
    { 0, 0 },
    { 0xd9747551, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49947554, 46 },
    { 0, 0 },
    { 0x80347556, 23 },
    { 0, 0 },
    { 0xddf47558, 17 },
    { 0xcda47559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4755c, 61 },
    { 0xd514755d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47566, 33 },
    { 0x82447567, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1574756c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b47576, 76 },
    { 0x12147576, 55 },
    { 0xa5847578, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a4757b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47347583, 95 },
    { 0x38447584, 98 },
    { 0xc7047585, 34 },
    { 0xccc47586, 90 },
    { 0x1ca47586, 33 },
    { 0x84c47588, 90 },
    { 0x9cf47589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5784758c, 95 },
    { 0, 0 },
    { 0x3784758e, 85 },
    { 0xe424758f, 98 },
    { 0x2ac47590, 90 },
    { 0xc9147591, 23 },
    { 0x1ed47591, 61 },
    { 0xd947590, 95 },
    { 0, 0 },
    { 0x8447595, 37 },
    { 0, 0 },
    { 0xac547597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3654759d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e475a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d475a6, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5475a9, 93 },
    { 0xa5f475aa, 43 },
    { 0x4d8475ab, 90 },
    { 0x95a475ab, 95 },
    { 0xffc475ab, 98 },
    { 0x102475ae, 95 },
    { 0xeba475af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0475b6, 52 },
    { 0x92d475b6, 95 },
    { 0x16c475b8, 46 },
    { 0x153475b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x809475bc, 95 },
    { 0, 0 },
    { 0x287475be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80475c1, 90 },
    { 0x1ad475c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x936475c5, 23 },
    { 0x61c475c6, 33 },
    { 0xb6f475c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc475ca, 66 },
    { 0xfdd475cb, 52 },
    { 0x746475cc, 23 },
    { 0x8d4475ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67475d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc475d4, 90 },
    { 0x8e8475d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699475db, 90 },
    { 0, 0 },
    { 0xfe5475dd, 90 },
    { 0x74b475de, 33 },
    { 0, 0 },
    { 0xe73475e0, 98 },
    { 0x137475e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c475e4, 34 },
    { 0xab6475e5, 98 },
    { 0x411475e6, 66 },
    { 0xb08475e7, 76 },
    { 0x8a9475e8, 95 },
    { 0xeb7475e7, 61 },
    { 0x7f1475e7, 34 },
    { 0xf58475eb, 95 },
    { 0x846475ec, 33 },
    { 0, 0 },
    { 0xd1f475ee, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x596475f1, 95 },
    { 0x736475f2, 75 },
    { 0xb02475f3, 36 },
    { 0x39475f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e475fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2547604, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa47607, 34 },
    { 0x17047608, 76 },
    { 0, 0 },
    { 0xeb4760a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac47611, 98 },
    { 0, 0 },
    { 0x5d047613, 43 },
    { 0x2db47614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4761a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb14761d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae47624, 23 },
    { 0x31b47625, 33 },
    { 0, 0 },
    { 0x6dc47627, 95 },
    { 0x6e547627, 33 },
    { 0x47747629, 90 },
    { 0xde54762a, 75 },
    { 0, 0 },
    { 0x8714762c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92947630, 76 },
    { 0, 0 },
    { 0x9c147632, 76 },
    { 0x64647633, 76 },
    { 0x43247634, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b4763a, 95 },
    { 0xad24763a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04763f, 81 },
    { 0xe5c47640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba047643, 91 },
    { 0x18d47644, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb4764a, 55 },
    { 0, 0 },
    { 0xb414764c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a47650, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45347653, 78 },
    { 0, 0 },
    { 0xa9247655, 23 },
    { 0x12047655, 100 },
    { 0, 0 },
    { 0x9e547658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf244765b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8647663, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754766b, 69 },
    { 0x45c4766b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a4766f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4447673, 34 },
    { 0x97747673, 95 },
    { 0, 0 },
    { 0x34a47676, 61 },
    { 0x30e47677, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b4767a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894767e, 46 },
    { 0xe34767f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d47682, 61 },
    { 0, 0 },
    { 0xba347684, 66 },
    { 0x24047684, 33 },
    { 0x92247685, 76 },
    { 0x84d47686, 90 },
    { 0x29147688, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014768b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41247691, 52 },
    { 0x7ca47692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47697, 61 },
    { 0xa4447698, 33 },
    { 0xd7f47699, 85 },
    { 0xc884769a, 33 },
    { 0x7fe4769b, 99 },
    { 0xeb74769c, 78 },
    { 0xb2f4769d, 95 },
    { 0x5264769e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4476a2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0476ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b476af, 33 },
    { 0x168476b0, 76 },
    { 0x8f6476b1, 81 },
    { 0xf67476b0, 67 },
    { 0xf6c476b1, 33 },
    { 0x879476b4, 43 },
    { 0, 0 },
    { 0xdfe476b6, 98 },
    { 0x96a476b6, 61 },
    { 0, 0 },
    { 0x8c8476b9, 93 },
    { 0xbcb476ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44476c1, 67 },
    { 0x69d476c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600476c6, 76 },
    { 0xfde476c7, 90 },
    { 0, 0 },
    { 0x8f5476c9, 95 },
    { 0x26b476ca, 98 },
    { 0x878476ca, 34 },
    { 0x80476cc, 95 },
    { 0x4c6476cd, 98 },
    { 0xc67476cc, 17 },
    { 0x841476cf, 87 },
    { 0, 0 },
    { 0x5be476d1, 81 },
    { 0xb82476d2, 33 },
    { 0x6b4476d1, 43 },
    { 0, 0 },
    { 0x156476d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87476d8, 76 },
    { 0, 0 },
    { 0x74d476da, 66 },
    { 0, 0 },
    { 0x23a476dc, 93 },
    { 0, 0 },
    { 0x332476de, 28 },
    { 0x981476de, 33 },
    { 0x8b9476de, 96 },
    { 0xd7476df, 67 },
    { 0, 0 },
    { 0x86b476e3, 33 },
    { 0x790476e4, 90 },
    { 0x3b1476e5, 23 },
    { 0xde9476e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6476ea, 76 },
    { 0x271476ea, 90 },
    { 0x7ec476ec, 99 },
    { 0xc63476eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749476f7, 98 },
    { 0x9f9476f8, 75 },
    { 0xbdb476f9, 99 },
    { 0xfcf476fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4476fd, 95 },
    { 0, 0 },
    { 0x6bc476ff, 98 },
    { 0xedb47700, 33 },
    { 0xdd047700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c47706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a44770c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc4770f, 98 },
    { 0x5ae47710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc47718, 61 },
    { 0x41547718, 23 },
    { 0, 0 },
    { 0x8f4771b, 66 },
    { 0, 0 },
    { 0xbfc4771d, 91 },
    { 0, 0 },
    { 0x8854771f, 46 },
    { 0xe8947720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a47726, 90 },
    { 0xdc447727, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4772b, 98 },
    { 0x17d4772b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2847730, 61 },
    { 0xf3d47731, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a47734, 33 },
    { 0xd1647735, 93 },
    { 0, 0 },
    { 0x49b47737, 46 },
    { 0xc747738, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb94773c, 33 },
    { 0x9b84773d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c247743, 73 },
    { 0, 0 },
    { 0xa9247745, 98 },
    { 0xba547746, 61 },
    { 0, 0 },
    { 0x52b47748, 61 },
    { 0x73c47749, 91 },
    { 0x2c547749, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db47752, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc447756, 81 },
    { 0, 0 },
    { 0xe2e47758, 95 },
    { 0xc8647758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6244775e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f47765, 33 },
    { 0, 0 },
    { 0xbde47767, 78 },
    { 0xb4b47767, 95 },
    { 0xbb747768, 85 },
    { 0xc5f47767, 33 },
    { 0xa254776b, 99 },
    { 0x4a747767, 23 },
    { 0x50c4776d, 33 },
    { 0, 0 },
    { 0xa5a4776f, 90 },
    { 0xf4547770, 93 },
    { 0x2ff47770, 96 },
    { 0x65147770, 61 },
    { 0x57347771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f147779, 23 },
    { 0, 0 },
    { 0x1fc4777b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b347783, 33 },
    { 0x13147784, 23 },
    { 0x17347784, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x27047788, 33 },
    { 0, 0 },
    { 0x8764778a, 33 },
    { 0x4614778b, 90 },
    { 0xa9d4778b, 90 },
    { 0xa0e4778c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1147790, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8947794, 98 },
    { 0, 0 },
    { 0x5dd47796, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f4779b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3477a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5477a8, 90 },
    { 0xebe477a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59477ac, 90 },
    { 0xa3d477ac, 76 },
    { 0, 0 },
    { 0x71b477af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89477b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8477bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2477bf, 43 },
    { 0, 0 },
    { 0xad9477c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28477c5, 33 },
    { 0xb7c477c6, 88 },
    { 0x65e477c7, 87 },
    { 0x6f4477c7, 66 },
    { 0x29477c8, 23 },
    { 0x31d477ca, 23 },
    { 0x903477ca, 17 },
    { 0x53b477cb, 33 },
    { 0xc34477cd, 65 },
    { 0x576477c6, 46 },
    { 0, 0 },
    { 0xb9f477d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed477d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a477de, 76 },
    { 0xfe0477df, 100 },
    { 0, 0 },
    { 0x78d477e1, 90 },
    { 0x4a477e2, 65 },
    { 0x125477e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233477e7, 95 },
    { 0x482477e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f477ec, 33 },
    { 0x7f9477ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4477f1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3477f4, 90 },
    { 0xeb3477f5, 52 },
    { 0xa5b477f6, 67 },
    { 0xa02477f7, 90 },
    { 0xc06477f7, 23 },
    { 0x3ba477f9, 99 },
    { 0x5f8477fa, 100 },
    { 0xae0477fa, 23 },
    { 0xc50477fc, 46 },
    { 0xe66477fb, 99 },
    { 0x673477fe, 85 },
    { 0xf9e477fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58747806, 99 },
    { 0xed847806, 98 },
    { 0xea947806, 46 },
    { 0x5dd47809, 34 },
    { 0x45a47809, 52 },
    { 0xefb4780b, 90 },
    { 0, 0 },
    { 0x1d64780d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e47814, 61 },
    { 0x55547815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1847819, 87 },
    { 0x954781a, 61 },
    { 0xfcf4781a, 95 },
    { 0x46c47819, 55 },
    { 0, 0 },
    { 0x7dc4781e, 65 },
    { 0x8d14781f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a47822, 91 },
    { 0x15f47823, 52 },
    { 0xb3c47824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23747829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd304782f, 34 },
    { 0xcd947830, 95 },
    { 0x7e047831, 85 },
    { 0x22547831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb47836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4783c, 90 },
    { 0, 0 },
    { 0x6564783e, 40 },
    { 0, 0 },
    { 0x49d47840, 76 },
    { 0xe8847841, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b47848, 90 },
    { 0xb5847849, 55 },
    { 0x9dd4784a, 96 },
    { 0xe334784b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca47853, 95 },
    { 0, 0 },
    { 0x56347855, 69 },
    { 0, 0 },
    { 0x7947857, 55 },
    { 0xb5947858, 87 },
    { 0, 0 },
    { 0x924785a, 33 },
    { 0x5384785a, 95 },
    { 0, 0 },
    { 0x8194785d, 52 },
    { 0, 0 },
    { 0xa204785f, 33 },
    { 0, 0 },
    { 0xa7b47861, 98 },
    { 0xa9247862, 67 },
    { 0, 0 },
    { 0x89947864, 76 },
    { 0xa9547865, 33 },
    { 0x1a247866, 95 },
    { 0x93e47867, 90 },
    { 0x31b47864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74786b, 43 },
    { 0x7d84786c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9647871, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664787e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48147883, 23 },
    { 0x6ab47884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d4788a, 46 },
    { 0xdc4788b, 33 },
    { 0x7ab4788c, 91 },
    { 0x6d4788c, 95 },
    { 0x1744788d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f47892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43047895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64147899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf844789d, 34 },
    { 0xedb4789e, 90 },
    { 0x1ee4789f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x543478a2, 100 },
    { 0xe38478a3, 95 },
    { 0, 0 },
    { 0x812478a5, 61 },
    { 0xa4a478a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb478a9, 95 },
    { 0x6e478aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51478ad, 88 },
    { 0xfed478ad, 61 },
    { 0x498478af, 84 },
    { 0x716478ae, 52 },
    { 0, 0 },
    { 0xea0478b2, 90 },
    { 0x1fe478b2, 76 },
    { 0x847478b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1478b8, 81 },
    { 0xc08478b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a478c2, 93 },
    { 0xb70478c3, 52 },
    { 0xa1b478c3, 23 },
    { 0x3f9478c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723478cf, 90 },
    { 0x4ee478cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754478d4, 94 },
    { 0, 0 },
    { 0xf72478d6, 90 },
    { 0xdcb478d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd478da, 76 },
    { 0x7f1478db, 95 },
    { 0x5df478db, 90 },
    { 0x478db, 76 },
    { 0x37478dc, 76 },
    { 0x90c478da, 23 },
    { 0, 0 },
    { 0x664478e1, 52 },
    { 0, 0 },
    { 0x1d8478e3, 98 },
    { 0x26f478e4, 98 },
    { 0x181478e4, 98 },
    { 0x1bc478e6, 46 },
    { 0, 0 },
    { 0x63478e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee478f3, 76 },
    { 0xd40478f4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x500478f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d478fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53347900, 61 },
    { 0x30947900, 61 },
    { 0x4d647900, 95 },
    { 0, 0 },
    { 0x4cb47904, 76 },
    { 0xc0b47904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2554790a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab14790d, 43 },
    { 0, 0 },
    { 0x62d4790f, 33 },
    { 0x3184790f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88447916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe47919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb4791c, 78 },
    { 0x9d24791d, 90 },
    { 0, 0 },
    { 0xb7a4791f, 17 },
    { 0x6f547920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51247923, 78 },
    { 0, 0 },
    { 0x11e47925, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074792d, 61 },
    { 0x2c74792e, 76 },
    { 0, 0 },
    { 0x5fc47930, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8247934, 33 },
    { 0x94747934, 38 },
    { 0, 0 },
    { 0x19b47937, 23 },
    { 0, 0 },
    { 0x93a47939, 33 },
    { 0x43d4793a, 85 },
    { 0xfd04793b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094793f, 23 },
    { 0x80747940, 95 },
    { 0, 0 },
    { 0x36047942, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11247947, 61 },
    { 0, 0 },
    { 0x47b47949, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9904794d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b47950, 76 },
    { 0, 0 },
    { 0xc2f47952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8147956, 34 },
    { 0x11c47957, 33 },
    { 0x2d247957, 90 },
    { 0xd6647959, 76 },
    { 0, 0 },
    { 0xcb24795b, 61 },
    { 0x32a4795c, 55 },
    { 0xcf94795d, 55 },
    { 0x4794795e, 33 },
    { 0xa914795f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8047963, 76 },
    { 0xb4447964, 61 },
    { 0xdf847965, 38 },
    { 0x87947963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72547969, 46 },
    { 0x2004796a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8447970, 43 },
    { 0xcb547971, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8847974, 33 },
    { 0x63847975, 95 },
    { 0x90747976, 88 },
    { 0, 0 },
    { 0xd6847978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d4797e, 88 },
    { 0, 0 },
    { 0x67047980, 90 },
    { 0, 0 },
    { 0x9bd47982, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6147987, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c44798a, 43 },
    { 0, 0 },
    { 0x434798c, 100 },
    { 0x5ea4798d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f647993, 46 },
    { 0x65747994, 85 },
    { 0xb2847995, 55 },
    { 0x9c347996, 87 },
    { 0x6e647993, 23 },
    { 0, 0 },
    { 0x44f47999, 23 },
    { 0x7e047999, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4799f, 61 },
    { 0x459479a0, 23 },
    { 0x4f6479a0, 90 },
    { 0, 0 },
    { 0x5ae479a3, 34 },
    { 0x4e479a3, 17 },
    { 0xeef479a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6479a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154479ae, 95 },
    { 0x921479af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec479b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d479b9, 90 },
    { 0xfe9479b9, 95 },
    { 0x34d479b9, 33 },
    { 0x33b479bb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34479bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f479c6, 91 },
    { 0xe3479c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b479cb, 77 },
    { 0x80a479cc, 33 },
    { 0x85479cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d479d6, 90 },
    { 0x16a479d7, 90 },
    { 0xfca479d8, 53 },
    { 0x37479d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f479e1, 77 },
    { 0, 0 },
    { 0xd89479e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b479e6, 90 },
    { 0xd9a479e7, 33 },
    { 0x595479e8, 98 },
    { 0xb51479e8, 98 },
    { 0xec7479ea, 76 },
    { 0xd08479eb, 33 },
    { 0x750479eb, 23 },
    { 0xb54479ed, 76 },
    { 0x350479ed, 90 },
    { 0xe98479ea, 90 },
    { 0x8fd479e6, 90 },
    { 0xe5c479f1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00479f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c479fb, 55 },
    { 0xb27479fb, 55 },
    { 0, 0 },
    { 0x7db479fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d847a02, 75 },
    { 0x78447a02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff147a0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb47a0f, 46 },
    { 0xd5347a10, 90 },
    { 0x4b147a11, 55 },
    { 0x70247a12, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4447a15, 77 },
    { 0, 0 },
    { 0xf3247a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e047a21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca247a25, 34 },
    { 0, 0 },
    { 0x76947a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14947a2a, 95 },
    { 0xbb947a2b, 76 },
    { 0x51247a2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c47a2f, 33 },
    { 0, 0 },
    { 0xc5747a31, 33 },
    { 0x9df47a31, 61 },
    { 0xcb347a33, 90 },
    { 0xd7a47a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b47a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53047a41, 52 },
    { 0x74947a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3847a49, 100 },
    { 0x3b747a49, 90 },
    { 0x5aa47a4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2847a52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20947a55, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x25447a58, 17 },
    { 0, 0 },
    { 0xfbd47a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36047a62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1947a66, 95 },
    { 0, 0 },
    { 0x35e47a68, 90 },
    { 0, 0 },
    { 0x66347a6a, 77 },
    { 0x35d47a6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e47a74, 33 },
    { 0xb1847a75, 87 },
    { 0x18247a75, 95 },
    { 0x4f647a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b47a7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f47a82, 90 },
    { 0, 0 },
    { 0x48247a84, 48 },
    { 0x92247a85, 36 },
    { 0x8247a86, 52 },
    { 0x66547a86, 33 },
    { 0x96247a85, 95 },
    { 0xefb47a89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47a8d, 96 },
    { 0xc8b47a8e, 75 },
    { 0xca547a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70347a94, 34 },
    { 0x8f047a95, 77 },
    { 0x5ab47a95, 66 },
    { 0x5bd47a97, 98 },
    { 0, 0 },
    { 0x59747a99, 90 },
    { 0x1db47a9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47aa0, 17 },
    { 0, 0 },
    { 0x78447aa2, 40 },
    { 0, 0 },
    { 0x77247aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e47aab, 33 },
    { 0xd9347aab, 96 },
    { 0, 0 },
    { 0x82347aae, 33 },
    { 0xd0e47aaf, 91 },
    { 0, 0 },
    { 0x7bb47ab1, 78 },
    { 0xe6d47ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a47ab9, 23 },
    { 0x5f847aba, 69 },
    { 0xe7e47abb, 98 },
    { 0xe6c47abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad47ac5, 55 },
    { 0, 0 },
    { 0x68847ac7, 55 },
    { 0x1f847ac7, 67 },
    { 0xfb047ac8, 46 },
    { 0x4ab47aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4347ace, 90 },
    { 0xc9e47acf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b47ad2, 23 },
    { 0xa7e47ad2, 85 },
    { 0x6ab47ad4, 95 },
    { 0x77447ad4, 43 },
    { 0, 0 },
    { 0xa0f47ad7, 76 },
    { 0x86547ad8, 98 },
    { 0x87d47ad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe947adc, 68 },
    { 0xda847add, 95 },
    { 0x4cc47ade, 69 },
    { 0, 0 },
    { 0xcb747ae0, 46 },
    { 0xeb847ae0, 95 },
    { 0x7bb47ae0, 61 },
    { 0xc6947ae3, 95 },
    { 0x70747ae3, 23 },
    { 0x3d947ae1, 33 },
    { 0x71147ae4, 33 },
    { 0x3fb47ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea347aea, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc47af0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c47af5, 55 },
    { 0x27e47af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847af9, 23 },
    { 0x5d47afa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3747b00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd447b04, 95 },
    { 0x83047b05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea47b0a, 76 },
    { 0, 0 },
    { 0x54447b0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf47b10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba47b14, 76 },
    { 0x17647b15, 76 },
    { 0x3ab47b16, 100 },
    { 0xd6947b15, 23 },
    { 0x5b247b15, 23 },
    { 0x6c47b19, 90 },
    { 0xf9d47b1a, 46 },
    { 0xdf747b19, 66 },
    { 0x7d447b16, 34 },
    { 0xada47b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e47b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f47b30, 95 },
    { 0x79e47b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d47b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea47b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a47b40, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c047b44, 47 },
    { 0xd5e47b45, 33 },
    { 0xa5e47b45, 57 },
    { 0xb2047b47, 95 },
    { 0, 0 },
    { 0x90747b49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac47b4c, 95 },
    { 0x82047b4c, 95 },
    { 0x2b147b4e, 83 },
    { 0x95347b4f, 55 },
    { 0xd2a47b4f, 95 },
    { 0, 0 },
    { 0x67b47b52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1947b56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d847b59, 23 },
    { 0x19147b5a, 98 },
    { 0xf3a47b59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3247b60, 76 },
    { 0, 0 },
    { 0x11247b62, 61 },
    { 0, 0 },
    { 0xc3e47b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0647b6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6947b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf847b79, 91 },
    { 0xdf247b7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547b7e, 55 },
    { 0, 0 },
    { 0xeef47b80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a47b8d, 33 },
    { 0xe2847b8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef47b91, 90 },
    { 0x77747b91, 46 },
    { 0x24847b92, 55 },
    { 0xee147b94, 77 },
    { 0xa7447b95, 77 },
    { 0xe5d47b94, 43 },
    { 0xd1b47b93, 90 },
    { 0x39547b93, 55 },
    { 0, 0 },
    { 0x6b547b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6647b9f, 80 },
    { 0x78547ba0, 77 },
    { 0x2c047ba0, 76 },
    { 0xa5247ba2, 36 },
    { 0x11847ba3, 90 },
    { 0x20747ba4, 87 },
    { 0x70a47ba1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24647ba9, 98 },
    { 0, 0 },
    { 0xb2a47bab, 69 },
    { 0, 0 },
    { 0xf1a47bad, 95 },
    { 0x85147bad, 90 },
    { 0xf8247bae, 37 },
    { 0, 0 },
    { 0xed447bb1, 61 },
    { 0x7e047bb2, 55 },
    { 0xc1a47bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62647bb7, 95 },
    { 0, 0 },
    { 0xee847bb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c47bbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1547bc6, 67 },
    { 0xa5f47bc7, 90 },
    { 0, 0 },
    { 0x74a47bc9, 93 },
    { 0x3d447bca, 17 },
    { 0x10947bca, 98 },
    { 0x3d247bcb, 84 },
    { 0xd9a47bcd, 33 },
    { 0x7e547bcd, 98 },
    { 0xef747bcf, 68 },
    { 0x57347bd0, 88 },
    { 0x24c47bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4147bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65447bd9, 55 },
    { 0x8bb47bda, 43 },
    { 0x7bc47bdb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee47bdf, 90 },
    { 0xd447be0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7147be3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b47be6, 33 },
    { 0, 0 },
    { 0x15247be8, 79 },
    { 0x1ea47be9, 76 },
    { 0x89547bea, 61 },
    { 0xf0a47beb, 33 },
    { 0x15247be9, 90 },
    { 0x4ec47bed, 85 },
    { 0, 0 },
    { 0xad447bef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0947bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d547bfc, 38 },
    { 0x92a47bfc, 55 },
    { 0xdde47bfe, 33 },
    { 0x8e547bfe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1247c06, 55 },
    { 0x98d47c07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c47c0e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7947c11, 33 },
    { 0, 0 },
    { 0x1c47c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c47c18, 34 },
    { 0xd5947c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0847c1f, 23 },
    { 0x64f47c1f, 98 },
    { 0xe8b47c21, 52 },
    { 0x48647c22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f47c28, 66 },
    { 0x9e147c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b747c2f, 98 },
    { 0xd4a47c30, 98 },
    { 0xafe47c2f, 90 },
    { 0, 0 },
    { 0x8c47c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec747c36, 95 },
    { 0, 0 },
    { 0x15c47c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b47c40, 38 },
    { 0xbed47c40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42847c48, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae47c4b, 95 },
    { 0x1b247c4c, 65 },
    { 0xf8247c4c, 66 },
    { 0xaa247c4e, 90 },
    { 0x50947c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c47c59, 84 },
    { 0, 0 },
    { 0x9cc47c5b, 23 },
    { 0x5f947c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be47c5f, 96 },
    { 0xec47c5f, 34 },
    { 0xf5747c60, 61 },
    { 0, 0 },
    { 0xef447c63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49247c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4647c69, 98 },
    { 0xfbb47c6a, 95 },
    { 0x62647c6b, 43 },
    { 0x76747c6a, 43 },
    { 0, 0 },
    { 0x36747c6e, 61 },
    { 0xdb547c6f, 34 },
    { 0x94747c6e, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30247c78, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8047c7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94347c81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d47c86, 61 },
    { 0x13347c87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b47c8e, 33 },
    { 0xe0647c8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b047c92, 95 },
    { 0xc6047c93, 88 },
    { 0, 0 },
    { 0xc3147c95, 90 },
    { 0x1b147c95, 85 },
    { 0xa4947c95, 61 },
    { 0x26247c98, 34 },
    { 0, 0 },
    { 0x21547c9a, 98 },
    { 0, 0 },
    { 0xef747c9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a647ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b47ca5, 78 },
    { 0xb9747ca6, 98 },
    { 0xec947ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86447caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62147cad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc47cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac47cb7, 98 },
    { 0x67d47cb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba47cbd, 77 },
    { 0x98947cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a47cc9, 98 },
    { 0xa8b47cca, 90 },
    { 0x38b47cca, 34 },
    { 0, 0 },
    { 0xcc447ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c47cd1, 67 },
    { 0x42b47cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4547cd6, 95 },
    { 0x8c347cd6, 55 },
    { 0, 0 },
    { 0x5b147cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c47cde, 90 },
    { 0x27047cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2647ce8, 23 },
    { 0, 0 },
    { 0xe0047cea, 46 },
    { 0x8f747ceb, 98 },
    { 0xf7847cec, 33 },
    { 0x8647cec, 98 },
    { 0x50847cea, 61 },
    { 0xf4947cee, 66 },
    { 0x65847ceb, 64 },
    { 0xbea47cef, 90 },
    { 0, 0 },
    { 0xaf047cf3, 40 },
    { 0x3c447cf3, 52 },
    { 0, 0 },
    { 0xf9847cf6, 55 },
    { 0, 0 },
    { 0x56d47cf8, 60 },
    { 0x50447cf9, 95 },
    { 0x63747cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7347cfd, 90 },
    { 0, 0 },
    { 0x74c47cff, 90 },
    { 0x69a47d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86247d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0847d0b, 90 },
    { 0x64b47d0c, 77 },
    { 0, 0 },
    { 0x51c47d0e, 23 },
    { 0x9f147d0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46647d17, 76 },
    { 0x4ec47d18, 33 },
    { 0xc2247d19, 33 },
    { 0xaff47d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d47d21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c47d25, 33 },
    { 0x4a647d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2747d2c, 98 },
    { 0xd1e47d2d, 34 },
    { 0, 0 },
    { 0x4f047d2f, 93 },
    { 0x84347d2f, 33 },
    { 0x7fc47d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a47d35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c847d38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3647d42, 96 },
    { 0xa9847d43, 95 },
    { 0x1a647d44, 64 },
    { 0, 0 },
    { 0x28a47d46, 23 },
    { 0, 0 },
    { 0x42e47d48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d47d4b, 61 },
    { 0xaed47d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9947d53, 91 },
    { 0x1e947d54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67147d5c, 90 },
    { 0x30047d5d, 90 },
    { 0x4ac47d5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e47d61, 85 },
    { 0xce747d62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a47d65, 76 },
    { 0xbb947d65, 87 },
    { 0x6c447d67, 81 },
    { 0xa2447d67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17247d6b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e47d6f, 61 },
    { 0, 0 },
    { 0xbed47d71, 55 },
    { 0x70047d72, 46 },
    { 0x70947d72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b547d77, 33 },
    { 0x24f47d77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf947d7d, 95 },
    { 0x59247d7e, 87 },
    { 0x47b47d7f, 90 },
    { 0x89747d80, 34 },
    { 0, 0 },
    { 0x24347d82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f47d87, 98 },
    { 0x97f47d87, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7847d8f, 98 },
    { 0, 0 },
    { 0xee247d91, 95 },
    { 0x7dd47d92, 55 },
    { 0x1d47d93, 61 },
    { 0xb8d47d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af47d97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda47d9c, 61 },
    { 0x73347d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f47da1, 98 },
    { 0, 0 },
    { 0x2347da3, 76 },
    { 0, 0 },
    { 0xc1747da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44947dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59447db6, 46 },
    { 0, 0 },
    { 0x39547db8, 84 },
    { 0xf2e47db9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e347dbe, 76 },
    { 0xc9947dbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7447dc2, 23 },
    { 0x16647dc2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d47dc8, 98 },
    { 0x4347dc9, 98 },
    { 0x14747dca, 98 },
    { 0x24347dc9, 91 },
    { 0xbfc47dcc, 81 },
    { 0xfba47dcd, 90 },
    { 0, 0 },
    { 0xc4247dcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb947dd2, 98 },
    { 0x12f47dd2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc847dd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81247ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0147dde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0447de1, 61 },
    { 0, 0 },
    { 0xe2947de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e47dea, 91 },
    { 0x9c047dea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1747dee, 100 },
    { 0, 0 },
    { 0xe2247df0, 66 },
    { 0xb1e47df1, 81 },
    { 0x2f047df0, 33 },
    { 0x9e647df1, 46 },
    { 0x1e347df0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b847df7, 76 },
    { 0, 0 },
    { 0x7e647df9, 68 },
    { 0x48947dfa, 90 },
    { 0x8e047dfb, 100 },
    { 0x4e347dfc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f047e02, 95 },
    { 0, 0 },
    { 0x1d047e04, 23 },
    { 0xa7e47e05, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e547e0b, 98 },
    { 0xca147e0c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff47e10, 33 },
    { 0, 0 },
    { 0x89447e12, 55 },
    { 0x78747e13, 90 },
    { 0xb4247e14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7347e17, 68 },
    { 0xec447e17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c47e1b, 61 },
    { 0x9c947e1c, 23 },
    { 0xf6c47e1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c847e21, 43 },
    { 0x9f247e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c47e25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a47e28, 66 },
    { 0x7da47e28, 90 },
    { 0x3c947e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34447e32, 23 },
    { 0xfd547e33, 33 },
    { 0xf9f47e32, 84 },
    { 0xc0a47e34, 66 },
    { 0xa4147e36, 55 },
    { 0xc8747e37, 61 },
    { 0x8b147e35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1147e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b247e45, 61 },
    { 0xff447e46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b047e49, 95 },
    { 0x5c647e49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa447e55, 80 },
    { 0x9dc47e56, 97 },
    { 0, 0 },
    { 0x86547e58, 55 },
    { 0, 0 },
    { 0x2a247e5a, 33 },
    { 0x5e247e5b, 90 },
    { 0xcd647e5b, 43 },
    { 0xbb147e5a, 95 },
    { 0xb1147e5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11047e61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa47e64, 95 },
    { 0x9b247e65, 46 },
    { 0x44847e66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be47e6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b47e6f, 76 },
    { 0, 0 },
    { 0x36e47e71, 81 },
    { 0xf5f47e72, 33 },
    { 0xae147e71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36047e76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35147e7b, 93 },
    { 0xb47e7b, 98 },
    { 0xc6447e7c, 33 },
    { 0, 0 },
    { 0x2ff47e7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2147e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3247e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67547e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd347e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91347e90, 33 },
    { 0x3c847e91, 96 },
    { 0x11147e90, 90 },
    { 0xe6a47e93, 81 },
    { 0x88f47e93, 75 },
    { 0x7f347e94, 90 },
    { 0x7ba47e90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f47e9a, 88 },
    { 0x69247e9b, 31 },
    { 0x8c847e9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e947ea0, 95 },
    { 0x86547ea1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69147ea5, 76 },
    { 0x43747ea6, 23 },
    { 0, 0 },
    { 0x2f47ea8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef47eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7647eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8347ebb, 78 },
    { 0x55447ebc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb47ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15847ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af47eca, 91 },
    { 0x87547ecb, 93 },
    { 0, 0 },
    { 0x1e147ecd, 78 },
    { 0xbe247ece, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d047ed3, 99 },
    { 0, 0 },
    { 0x9bf47ed5, 95 },
    { 0, 0 },
    { 0x11d47ed7, 43 },
    { 0x94647ed8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db47edf, 69 },
    { 0x6ab47edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2547ee3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8547ee7, 55 },
    { 0x77347ee8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a47eeb, 46 },
    { 0, 0 },
    { 0xd2047eed, 88 },
    { 0xbe247eee, 85 },
    { 0, 0 },
    { 0xd4c47ef0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa47ef5, 80 },
    { 0xfc47ef6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28147efc, 61 },
    { 0x31d47efc, 43 },
    { 0x38547efe, 31 },
    { 0x2c347eff, 34 },
    { 0x56d47efe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547f06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b247f0a, 33 },
    { 0xd8c47f0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68847f0e, 43 },
    { 0xf2147f0e, 90 },
    { 0, 0 },
    { 0xf7047f11, 92 },
    { 0x29747f11, 69 },
    { 0, 0 },
    { 0x63347f14, 95 },
    { 0x19b47f15, 95 },
    { 0x6947f15, 98 },
    { 0x25647f15, 33 },
    { 0x35847f15, 95 },
    { 0, 0 },
    { 0x1e647f1a, 52 },
    { 0xa7947f1a, 90 },
    { 0, 0 },
    { 0x34747f1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f47f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f47f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c47f2d, 55 },
    { 0, 0 },
    { 0xc6e47f2f, 76 },
    { 0, 0 },
    { 0xa6947f31, 98 },
    { 0xff047f31, 90 },
    { 0, 0 },
    { 0xfb847f34, 95 },
    { 0x7847f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4047f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca47f43, 61 },
    { 0x73a47f44, 52 },
    { 0xac347f45, 90 },
    { 0xc2a47f46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80547f4b, 98 },
    { 0, 0 },
    { 0xe47f4d, 33 },
    { 0, 0 },
    { 0x16b47f4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b47f52, 55 },
    { 0x12747f53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d347f57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a47f5e, 68 },
    { 0, 0 },
    { 0xc5747f60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58047f64, 98 },
    { 0, 0 },
    { 0xf3147f66, 33 },
    { 0x9d47f67, 55 },
    { 0xae447f68, 98 },
    { 0x99b47f69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96747f72, 55 },
    { 0x25947f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f347f76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e47f7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6947f81, 23 },
    { 0x1d547f82, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21947f85, 66 },
    { 0xd3747f85, 93 },
    { 0xcc647f87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12347f8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b47f90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa47f9b, 95 },
    { 0xf7d47f9c, 87 },
    { 0, 0 },
    { 0x4ba47f9e, 55 },
    { 0x5247f9e, 93 },
    { 0, 0 },
    { 0x6bd47fa1, 80 },
    { 0xf347fa2, 90 },
    { 0x79b47fa3, 98 },
    { 0, 0 },
    { 0x7147fa5, 81 },
    { 0xaa047fa6, 95 },
    { 0xf1847fa6, 23 },
    { 0x92247fa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6947faf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b47fb3, 40 },
    { 0, 0 },
    { 0x6c747fb5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e47fbb, 98 },
    { 0x8c47fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb47fc0, 85 },
    { 0, 0 },
    { 0x13c47fc2, 90 },
    { 0xd6247fc2, 87 },
    { 0x34347fc4, 76 },
    { 0x7ef47fc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7547fd1, 43 },
    { 0xe1847fd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c47fd9, 95 },
    { 0xdef47fd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21047fdd, 23 },
    { 0xbdc47fde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e47fe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc747fe8, 43 },
    { 0x54147fe8, 52 },
    { 0, 0 },
    { 0xcb947feb, 46 },
    { 0x18547fec, 61 },
    { 0x8f047fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c347ff1, 90 },
    { 0x89647ff2, 90 },
    { 0xd2847ff3, 90 },
    { 0, 0 },
    { 0xba547ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a47ff8, 90 },
    { 0xd2547ff8, 61 },
    { 0xc9c47ffa, 23 },
    { 0, 0 },
    { 0xc9847ffc, 34 },
    { 0xd8147ffd, 40 },
    { 0xa8d47ffe, 34 },
    { 0x6e947ffd, 61 },
    { 0x51447ffc, 55 },
    { 0x1b448001, 95 },
    { 0x99547ffd, 33 },
    { 0, 0 },
    { 0x46248004, 34 },
    { 0xf2048004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc48009, 55 },
    { 0, 0 },
    { 0xba64800b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e148012, 84 },
    { 0x80148012, 17 },
    { 0xc7348013, 33 },
    { 0x46448014, 61 },
    { 0xaaa48015, 91 },
    { 0xc1348017, 93 },
    { 0, 0 },
    { 0x84148019, 61 },
    { 0x93a4801a, 90 },
    { 0x7434801a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb48023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2848027, 43 },
    { 0, 0 },
    { 0x40448029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84802c, 76 },
    { 0, 0 },
    { 0xfcd4802e, 98 },
    { 0x1094802f, 76 },
    { 0x3d248030, 33 },
    { 0xec948031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79248035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69948039, 90 },
    { 0x8894803a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe4803d, 91 },
    { 0x5084803e, 78 },
    { 0x4f84803f, 76 },
    { 0x18648040, 98 },
    { 0x71548041, 55 },
    { 0x9324803e, 33 },
    { 0xb9548043, 34 },
    { 0x67148043, 34 },
    { 0x20f48045, 43 },
    { 0xb9c4803e, 61 },
    { 0, 0 },
    { 0xa8748048, 90 },
    { 0x80748049, 61 },
    { 0x8cb4804a, 55 },
    { 0x7384804b, 33 },
    { 0x9cd4804c, 85 },
    { 0xc194804b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d248050, 95 },
    { 0xdc748051, 95 },
    { 0x6c848052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c948055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0848058, 90 },
    { 0x36b48058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734805e, 66 },
    { 0xdec4805e, 90 },
    { 0, 0 },
    { 0xbcb48061, 95 },
    { 0xeb848062, 55 },
    { 0, 0 },
    { 0x86c48064, 95 },
    { 0, 0 },
    { 0xa0f48066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b4806b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e4806f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da48073, 91 },
    { 0x21f48074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca48078, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a48080, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef48084, 90 },
    { 0x73948085, 67 },
    { 0xcb648084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2084808c, 95 },
    { 0xf804808d, 90 },
    { 0x6574808e, 95 },
    { 0xbe54808f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f48093, 95 },
    { 0, 0 },
    { 0xb7748095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa48098, 90 },
    { 0xc948098, 33 },
    { 0xe134809a, 76 },
    { 0, 0 },
    { 0x31f4809c, 61 },
    { 0, 0 },
    { 0x1674809e, 33 },
    { 0x3db4809f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c480a3, 33 },
    { 0x662480a4, 23 },
    { 0xe09480a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34480a8, 33 },
    { 0x14b480a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2480ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x657480b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4480b4, 61 },
    { 0, 0 },
    { 0x87d480b6, 82 },
    { 0xdc5480b7, 98 },
    { 0x94480b8, 43 },
    { 0xbb4480b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0480bc, 80 },
    { 0, 0 },
    { 0x803480be, 61 },
    { 0, 0 },
    { 0x162480c0, 61 },
    { 0xa8f480c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5480c7, 33 },
    { 0, 0 },
    { 0xc3d480c9, 46 },
    { 0, 0 },
    { 0x1d1480cb, 78 },
    { 0xa51480cb, 45 },
    { 0xeff480cc, 77 },
    { 0x9aa480ce, 90 },
    { 0, 0 },
    { 0x285480d0, 96 },
    { 0, 0 },
    { 0x1c2480d2, 23 },
    { 0, 0 },
    { 0x69d480d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x423480d7, 66 },
    { 0xf9e480d7, 65 },
    { 0x803480d9, 48 },
    { 0, 0 },
    { 0x79c480db, 95 },
    { 0, 0 },
    { 0xd51480dd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x303480e0, 90 },
    { 0xd89480e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10480e4, 95 },
    { 0xd11480e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a480e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e480ec, 23 },
    { 0xb8e480ed, 98 },
    { 0x145480ee, 90 },
    { 0, 0 },
    { 0x8ce480f0, 93 },
    { 0xa06480f1, 36 },
    { 0x744480f2, 100 },
    { 0xb8e480f3, 95 },
    { 0, 0 },
    { 0x110480f5, 88 },
    { 0x5d2480f6, 90 },
    { 0x323480f6, 98 },
    { 0, 0 },
    { 0x400480f9, 90 },
    { 0xf35480fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2480ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45348102, 34 },
    { 0x29648103, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d848110, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b48118, 76 },
    { 0x82248119, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a4811c, 23 },
    { 0, 0 },
    { 0xd124811e, 90 },
    { 0xe224811e, 23 },
    { 0, 0 },
    { 0x8dc48121, 61 },
    { 0x71348122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda148126, 85 },
    { 0, 0 },
    { 0xfc948128, 66 },
    { 0xff648129, 90 },
    { 0x5584812a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c4812d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c48132, 61 },
    { 0x9bb48133, 99 },
    { 0, 0 },
    { 0xa0048135, 85 },
    { 0x83148135, 43 },
    { 0, 0 },
    { 0xcd148138, 91 },
    { 0xdd48138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d84813d, 40 },
    { 0, 0 },
    { 0x1664813f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb48143, 61 },
    { 0xd6848144, 55 },
    { 0x63748145, 76 },
    { 0x91f48146, 33 },
    { 0xaa048145, 90 },
    { 0x12e48146, 61 },
    { 0x5fa48149, 90 },
    { 0x71a4814a, 23 },
    { 0x18b48146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f04814e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d48151, 80 },
    { 0xfe348151, 33 },
    { 0xff248153, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b48158, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab4815b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78348161, 90 },
    { 0x81848162, 23 },
    { 0x96748163, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f4816a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f64816d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61348174, 66 },
    { 0xff848175, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48179, 90 },
    { 0x2d548179, 33 },
    { 0x2264817b, 76 },
    { 0x3ed4817b, 90 },
    { 0xd164817d, 98 },
    { 0xd044817e, 95 },
    { 0xe954817e, 23 },
    { 0, 0 },
    { 0xec448181, 90 },
    { 0, 0 },
    { 0x59248183, 33 },
    { 0xfb848184, 61 },
    { 0x8dd48185, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d748188, 61 },
    { 0xadb48189, 90 },
    { 0xc014818a, 61 },
    { 0x72748189, 90 },
    { 0xff4818b, 76 },
    { 0x66f4818c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f348191, 96 },
    { 0xc4148191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58348197, 61 },
    { 0xb5d48197, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4819e, 84 },
    { 0x814819f, 93 },
    { 0x8ac4819f, 90 },
    { 0, 0 },
    { 0x7df481a2, 23 },
    { 0x247481a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967481a8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3481ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc481b3, 61 },
    { 0x5fb481b4, 33 },
    { 0x8bc481b5, 90 },
    { 0xb3e481b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862481ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x774481bd, 90 },
    { 0, 0 },
    { 0x627481bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd481c5, 80 },
    { 0x1c9481c6, 98 },
    { 0x97d481c7, 77 },
    { 0x459481c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768481cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2481d3, 33 },
    { 0, 0 },
    { 0x7ac481d5, 90 },
    { 0x1c9481d6, 98 },
    { 0xce3481d7, 51 },
    { 0x72e481d7, 33 },
    { 0xa77481d9, 33 },
    { 0xebd481d6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109481de, 68 },
    { 0x730481df, 100 },
    { 0x600481e0, 90 },
    { 0x513481e1, 95 },
    { 0x2cc481e2, 34 },
    { 0xcef481e1, 61 },
    { 0, 0 },
    { 0x306481e5, 90 },
    { 0x9e1481e6, 17 },
    { 0x4b1481e7, 95 },
    { 0x9d9481e6, 55 },
    { 0x647481e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841481f0, 43 },
    { 0xdab481f0, 90 },
    { 0, 0 },
    { 0x372481f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x280481f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136481fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1481ff, 97 },
    { 0xa5d48200, 95 },
    { 0xa848200, 33 },
    { 0, 0 },
    { 0x33948203, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a34820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e24820f, 91 },
    { 0x46248210, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf24821a, 61 },
    { 0, 0 },
    { 0xd584821c, 95 },
    { 0x4a04821c, 33 },
    { 0x3754821e, 43 },
    { 0xce14821f, 95 },
    { 0, 0 },
    { 0x52148221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf848227, 69 },
    { 0xd2648227, 90 },
    { 0x10d48227, 76 },
    { 0x60048227, 34 },
    { 0, 0 },
    { 0xb624822c, 23 },
    { 0x9ae4822d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e648230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba48234, 55 },
    { 0, 0 },
    { 0x2b548236, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be48241, 90 },
    { 0xb1a48242, 84 },
    { 0xf1a48243, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35948247, 23 },
    { 0x58848247, 46 },
    { 0, 0 },
    { 0x3aa4824a, 33 },
    { 0, 0 },
    { 0x21d4824c, 17 },
    { 0xc6e4824d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6548250, 34 },
    { 0, 0 },
    { 0xc9b48252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf48255, 84 },
    { 0x53748255, 93 },
    { 0, 0 },
    { 0x1ba48258, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4825c, 81 },
    { 0xc134825d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18648263, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92548267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab4826c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e4826f, 75 },
    { 0xb8948270, 36 },
    { 0x1b448270, 90 },
    { 0x53348272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b48278, 84 },
    { 0x46048279, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4827e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d348281, 64 },
    { 0xc6a48281, 55 },
    { 0x3dd48282, 34 },
    { 0xcd348284, 66 },
    { 0x45d48282, 91 },
    { 0xc4f48286, 33 },
    { 0x8fc48287, 33 },
    { 0x3ea48288, 94 },
    { 0x48e48289, 98 },
    { 0xb2c48286, 90 },
    { 0, 0 },
    { 0xe3c4828c, 61 },
    { 0, 0 },
    { 0xd594828e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d48291, 43 },
    { 0x5b548291, 95 },
    { 0, 0 },
    { 0x31648294, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48297, 66 },
    { 0x57f48298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf482a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e482a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84482a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86482ad, 87 },
    { 0, 0 },
    { 0x4e7482af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752482b4, 52 },
    { 0x156482b5, 95 },
    { 0xe66482b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d482b9, 23 },
    { 0x4f5482b9, 34 },
    { 0x513482bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2482bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483482c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4482ca, 84 },
    { 0, 0 },
    { 0xece482cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x861482cf, 93 },
    { 0, 0 },
    { 0x77c482d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea482d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0482d9, 46 },
    { 0xca5482d9, 34 },
    { 0, 0 },
    { 0xa4b482dc, 88 },
    { 0x620482dd, 80 },
    { 0, 0 },
    { 0x907482df, 52 },
    { 0xace482df, 38 },
    { 0xc0a482e1, 93 },
    { 0xb63482e1, 55 },
    { 0xfca482e3, 61 },
    { 0, 0 },
    { 0x216482e5, 34 },
    { 0, 0 },
    { 0x5bf482e7, 66 },
    { 0x606482e8, 91 },
    { 0x837482e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba482ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x463482ef, 69 },
    { 0xfa2482ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49482f6, 23 },
    { 0, 0 },
    { 0x530482f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23482fc, 90 },
    { 0, 0 },
    { 0x83b482fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a48305, 23 },
    { 0x26e48305, 61 },
    { 0x42848305, 99 },
    { 0xd5b48308, 46 },
    { 0x46a48309, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4830c, 64 },
    { 0xa554830d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a48311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24831c, 76 },
    { 0x5b34831d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e48321, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc248327, 37 },
    { 0x82448327, 33 },
    { 0xcb348329, 34 },
    { 0x8cf4832a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7874832f, 43 },
    { 0x1fc48330, 33 },
    { 0xe2548331, 95 },
    { 0xe6348330, 61 },
    { 0x45048333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46148338, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e04833c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e48340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e48346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed048349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa74834c, 76 },
    { 0, 0 },
    { 0xda14834e, 76 },
    { 0x89c4834e, 33 },
    { 0xf648350, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b648353, 90 },
    { 0, 0 },
    { 0x6fc48355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce348358, 90 },
    { 0x62e48359, 22 },
    { 0x18f4835a, 85 },
    { 0x9404835a, 66 },
    { 0x63c48358, 34 },
    { 0x9254835b, 95 },
    { 0xe3d4835a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae648364, 61 },
    { 0x7a048364, 75 },
    { 0, 0 },
    { 0x9ec48367, 34 },
    { 0xa4348368, 98 },
    { 0x53248368, 95 },
    { 0, 0 },
    { 0x3674836b, 64 },
    { 0, 0 },
    { 0xfe04836d, 84 },
    { 0xe474836e, 61 },
    { 0xaab4836f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f48377, 61 },
    { 0x6b748377, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa4837d, 95 },
    { 0xc1e4837d, 69 },
    { 0x7464837f, 90 },
    { 0x92c4837f, 55 },
    { 0x9ac48381, 33 },
    { 0, 0 },
    { 0x68248383, 100 },
    { 0xc8a48384, 93 },
    { 0xd0948385, 95 },
    { 0x6d748386, 98 },
    { 0xb9648387, 67 },
    { 0x6d348388, 66 },
    { 0x7df48387, 90 },
    { 0x2d64838a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a4838d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef948395, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4674839c, 95 },
    { 0x6944839d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2483a1, 61 },
    { 0, 0 },
    { 0x892483a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6483a7, 98 },
    { 0xf79483a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc483b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1483b9, 98 },
    { 0x461483ba, 76 },
    { 0, 0 },
    { 0x716483bc, 66 },
    { 0x62c483bc, 95 },
    { 0x41f483bd, 91 },
    { 0, 0 },
    { 0x5ea483c0, 33 },
    { 0, 0 },
    { 0x5df483c2, 91 },
    { 0x7b7483c3, 23 },
    { 0x31a483c4, 81 },
    { 0x38483c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3483c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x302483cc, 99 },
    { 0x3f9483cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6483d5, 78 },
    { 0xb61483d5, 66 },
    { 0xb6483d6, 33 },
    { 0xd78483d6, 75 },
    { 0xee0483d9, 84 },
    { 0x8f8483da, 23 },
    { 0x725483d5, 76 },
    { 0x482483dc, 61 },
    { 0x29e483dd, 23 },
    { 0xa92483de, 77 },
    { 0x426483de, 67 },
    { 0, 0 },
    { 0x41d483e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6483e7, 90 },
    { 0xde3483e8, 61 },
    { 0x55d483e9, 55 },
    { 0, 0 },
    { 0x818483eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8483f0, 66 },
    { 0xf8483f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce483f4, 61 },
    { 0xdaf483f4, 61 },
    { 0x556483f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1483fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd483ff, 33 },
    { 0, 0 },
    { 0xd1048401, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80748405, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8594840a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c4840d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d548410, 81 },
    { 0, 0 },
    { 0xfd648412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e348419, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae4841e, 34 },
    { 0x9df4841e, 95 },
    { 0x9d848420, 87 },
    { 0xad748420, 33 },
    { 0x11548421, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2548428, 91 },
    { 0, 0 },
    { 0x8484842a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc704842d, 33 },
    { 0, 0 },
    { 0xc304842f, 98 },
    { 0, 0 },
    { 0x33048431, 46 },
    { 0, 0 },
    { 0xf4048433, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a48436, 98 },
    { 0xa5748436, 90 },
    { 0xe4b48437, 90 },
    { 0xb9048438, 46 },
    { 0, 0 },
    { 0xb9b4843b, 61 },
    { 0xf1b4843c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2448446, 61 },
    { 0x9fd48447, 92 },
    { 0x30c48448, 91 },
    { 0xe7648446, 34 },
    { 0xa3e4844a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf748450, 90 },
    { 0xc6748451, 97 },
    { 0, 0 },
    { 0x1f548453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23248459, 23 },
    { 0xdc54845a, 33 },
    { 0, 0 },
    { 0xd5d4845c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e48460, 23 },
    { 0x23248461, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d48464, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94846f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef48476, 23 },
    { 0, 0 },
    { 0xf8648478, 85 },
    { 0x25448479, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb4847c, 90 },
    { 0x5604847d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d4848a, 53 },
    { 0xe384848b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e48494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f48499, 90 },
    { 0x96d4849a, 43 },
    { 0xbe24849b, 33 },
    { 0xa514849b, 34 },
    { 0x4bb48499, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d484a1, 99 },
    { 0xeb6484a1, 95 },
    { 0x355484a2, 33 },
    { 0x93c484a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397484ac, 68 },
    { 0xbab484ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4484b0, 65 },
    { 0x639484b1, 98 },
    { 0x631484b2, 87 },
    { 0, 0 },
    { 0xdbb484b4, 23 },
    { 0x3e5484b5, 99 },
    { 0x8c4484b6, 90 },
    { 0x9ae484b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x790484ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89484be, 55 },
    { 0x9d1484be, 36 },
    { 0, 0 },
    { 0x44484c1, 98 },
    { 0, 0 },
    { 0x283484c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91484c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986484cc, 57 },
    { 0x29a484cc, 43 },
    { 0x507484ce, 61 },
    { 0xecf484cf, 61 },
    { 0x518484cf, 90 },
    { 0x6e0484d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5484da, 76 },
    { 0, 0 },
    { 0x948484dc, 98 },
    { 0, 0 },
    { 0x665484de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x661484e1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de484ea, 76 },
    { 0x661484ea, 76 },
    { 0x1ca484ea, 23 },
    { 0x52c484eb, 90 },
    { 0x51f484ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74484f6, 76 },
    { 0, 0 },
    { 0x494484f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3484fc, 52 },
    { 0x506484fd, 61 },
    { 0, 0 },
    { 0x869484ff, 43 },
    { 0x11e48500, 81 },
    { 0, 0 },
    { 0xad148502, 42 },
    { 0, 0 },
    { 0xc8148504, 23 },
    { 0, 0 },
    { 0xb5a48506, 33 },
    { 0x65048506, 61 },
    { 0x20048508, 34 },
    { 0, 0 },
    { 0x2b34850a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e648512, 46 },
    { 0x46648513, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb034851c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b4851f, 61 },
    { 0, 0 },
    { 0xc1f48521, 90 },
    { 0x1ba48521, 84 },
    { 0x71e48523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff48528, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b4852d, 90 },
    { 0, 0 },
    { 0x4544852f, 88 },
    { 0x95548530, 52 },
    { 0xd1848531, 100 },
    { 0x5d48531, 90 },
    { 0, 0 },
    { 0xd3848534, 98 },
    { 0x59348535, 98 },
    { 0, 0 },
    { 0xa5048537, 53 },
    { 0xddb48538, 90 },
    { 0x11a48539, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4814853e, 53 },
    { 0xcf54853f, 77 },
    { 0, 0 },
    { 0x8fc48541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92148546, 99 },
    { 0xecf48547, 33 },
    { 0xc2048547, 23 },
    { 0, 0 },
    { 0xe134854a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe134854d, 99 },
    { 0xc4b4854e, 76 },
    { 0x8354854f, 95 },
    { 0x43948550, 55 },
    { 0x2f848550, 95 },
    { 0x1514854f, 34 },
    { 0x34d4854d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a48557, 68 },
    { 0xe1048557, 90 },
    { 0, 0 },
    { 0xa914855a, 55 },
    { 0xe584855b, 61 },
    { 0x4204855c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65748561, 67 },
    { 0, 0 },
    { 0xc6948563, 90 },
    { 0x5e548564, 61 },
    { 0x1e448565, 66 },
    { 0x5e748566, 90 },
    { 0xfc448567, 52 },
    { 0x68448568, 34 },
    { 0x15748569, 61 },
    { 0xb7a4856a, 61 },
    { 0x19f48567, 34 },
    { 0xc6d4856c, 61 },
    { 0xa6b4856d, 61 },
    { 0x7734856e, 98 },
    { 0xa394856f, 23 },
    { 0xf2c48569, 33 },
    { 0, 0 },
    { 0x67548572, 23 },
    { 0xb5048573, 90 },
    { 0x47e48574, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93748577, 43 },
    { 0, 0 },
    { 0xca848579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4794857f, 87 },
    { 0xa114857f, 91 },
    { 0x5d648581, 23 },
    { 0x81348582, 33 },
    { 0xafe48582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4858a, 34 },
    { 0xf034858b, 100 },
    { 0x45d4858a, 17 },
    { 0, 0 },
    { 0x8024858e, 91 },
    { 0xda94858f, 38 },
    { 0xd1548590, 33 },
    { 0x6b948590, 78 },
    { 0x64d48590, 46 },
    { 0x9cf4858f, 87 },
    { 0x40c48594, 90 },
    { 0xb1e4858f, 98 },
    { 0xacc48591, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4859a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4485a0, 61 },
    { 0xc5c485a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee485a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4485a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad485af, 61 },
    { 0, 0 },
    { 0xcfb485b1, 34 },
    { 0x50e485b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a485b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd485ba, 26 },
    { 0xd22485ba, 23 },
    { 0, 0 },
    { 0x13b485bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e485c0, 43 },
    { 0x388485c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda485c7, 34 },
    { 0xd0c485c8, 23 },
    { 0xc89485c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc485ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7485d4, 56 },
    { 0xe0485d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53485d9, 90 },
    { 0, 0 },
    { 0xd81485db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b485e1, 23 },
    { 0xd7d485e2, 95 },
    { 0x550485e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32485e6, 90 },
    { 0xb74485e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c485ec, 93 },
    { 0x1aa485ed, 76 },
    { 0, 0 },
    { 0x7e485ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9485f8, 90 },
    { 0xb26485f9, 90 },
    { 0x46b485f9, 61 },
    { 0xc3d485fa, 61 },
    { 0, 0 },
    { 0x9f9485fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d348601, 66 },
    { 0, 0 },
    { 0xeeb48603, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4860c, 55 },
    { 0x24f4860c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a648612, 78 },
    { 0x98548613, 98 },
    { 0xb9448614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67648617, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324861a, 98 },
    { 0, 0 },
    { 0x3484861c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc44861f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c48624, 90 },
    { 0, 0 },
    { 0xd9248626, 61 },
    { 0x35d48627, 95 },
    { 0, 0 },
    { 0xd6248629, 90 },
    { 0x31a4862a, 93 },
    { 0xbdc4862a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf594862e, 33 },
    { 0x4ee4862f, 90 },
    { 0xb948630, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xce348633, 65 },
    { 0, 0 },
    { 0xa2748635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e48639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0348640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8934864d, 98 },
    { 0x93f4864d, 95 },
    { 0, 0 },
    { 0xa9848650, 33 },
    { 0xaf548651, 61 },
    { 0, 0 },
    { 0x3c148653, 46 },
    { 0, 0 },
    { 0x94e48655, 90 },
    { 0x24448656, 77 },
    { 0xd4248656, 34 },
    { 0x1df48658, 23 },
    { 0x2e848659, 98 },
    { 0x93f48657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3154865d, 76 },
    { 0x9214865d, 33 },
    { 0xa0d4865d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6348662, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da48666, 33 },
    { 0, 0 },
    { 0xd8c48668, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5964866c, 33 },
    { 0, 0 },
    { 0x1d44866e, 95 },
    { 0, 0 },
    { 0xfe248670, 85 },
    { 0x18948671, 98 },
    { 0x37b48672, 99 },
    { 0x9ea48672, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8248677, 95 },
    { 0x91448677, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6444867b, 5 },
    { 0x47c4867b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e48681, 95 },
    { 0x18248682, 99 },
    { 0, 0 },
    { 0xc0c48684, 95 },
    { 0x17c48685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd48688, 61 },
    { 0, 0 },
    { 0x514868a, 33 },
    { 0x93e4868a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35748695, 90 },
    { 0, 0 },
    { 0x9e848697, 33 },
    { 0x8b248697, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe024869d, 55 },
    { 0xaff4869e, 23 },
    { 0x7784869f, 52 },
    { 0, 0 },
    { 0x709486a1, 98 },
    { 0x39c486a1, 90 },
    { 0xcd2486a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855486ab, 76 },
    { 0, 0 },
    { 0xc4c486ad, 66 },
    { 0x7f9486ae, 95 },
    { 0, 0 },
    { 0x41f486b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa486b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b486b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb486ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x480486bd, 61 },
    { 0xa5e486be, 23 },
    { 0xb2b486bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38486c3, 78 },
    { 0xc24486c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce486c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84486cd, 23 },
    { 0x8a6486cd, 33 },
    { 0xda486ce, 95 },
    { 0, 0 },
    { 0xea1486d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90486d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b486d7, 84 },
    { 0xbc486d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b486db, 87 },
    { 0x5ca486dc, 90 },
    { 0xbec486dc, 88 },
    { 0xcc486db, 98 },
    { 0xe40486dc, 36 },
    { 0x568486e0, 23 },
    { 0, 0 },
    { 0x1d9486e2, 61 },
    { 0x8d5486e3, 38 },
    { 0x75b486e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x677486e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb486eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40486f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x886486f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea486fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c486ff, 87 },
    { 0xfd48700, 84 },
    { 0x21348701, 95 },
    { 0x8c248701, 33 },
    { 0, 0 },
    { 0xae548704, 33 },
    { 0, 0 },
    { 0x6aa48706, 46 },
    { 0x20248707, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454870c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948712, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4548719, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa204871f, 95 },
    { 0x8d148720, 90 },
    { 0, 0 },
    { 0xc5d48722, 61 },
    { 0, 0 },
    { 0x32b48724, 55 },
    { 0x2fb48724, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df4872a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44872e, 95 },
    { 0x6714872f, 43 },
    { 0x1af4872e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff48738, 95 },
    { 0x7fd48738, 81 },
    { 0, 0 },
    { 0xa4b4873b, 33 },
    { 0, 0 },
    { 0x7e4873d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d548744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b48749, 76 },
    { 0x73248749, 90 },
    { 0, 0 },
    { 0x5714874c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab48754, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3648757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b4875b, 34 },
    { 0, 0 },
    { 0xbea4875d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a48760, 40 },
    { 0, 0 },
    { 0x20748762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed448765, 23 },
    { 0x43948766, 78 },
    { 0xf2448767, 95 },
    { 0xd7148768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb994876c, 55 },
    { 0xf3c4876d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d48770, 67 },
    { 0x44c48771, 33 },
    { 0xe5948770, 34 },
    { 0x40d48773, 90 },
    { 0xc6e48770, 99 },
    { 0x31a48775, 61 },
    { 0x67e48776, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18948779, 33 },
    { 0xd484877a, 98 },
    { 0x7014877b, 61 },
    { 0x20d4877c, 95 },
    { 0x8514877d, 93 },
    { 0x20d4877d, 90 },
    { 0x5354877e, 95 },
    { 0, 0 },
    { 0xd048781, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38748788, 95 },
    { 0xf3748788, 55 },
    { 0x23b4878a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5434878e, 76 },
    { 0x6bd4878f, 91 },
    { 0x2bf4878e, 33 },
    { 0x5b148791, 33 },
    { 0xbba48790, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x548795, 87 },
    { 0, 0 },
    { 0x6c248797, 100 },
    { 0x37148798, 98 },
    { 0, 0 },
    { 0xaf24879a, 55 },
    { 0, 0 },
    { 0x6374879c, 61 },
    { 0xd814879d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9487a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73487a6, 98 },
    { 0xae3487a7, 43 },
    { 0x547487a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc487ad, 76 },
    { 0, 0 },
    { 0x3bd487af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372487b5, 95 },
    { 0xeff487b6, 23 },
    { 0x76c487b6, 76 },
    { 0x7c2487b8, 90 },
    { 0xa4e487b9, 95 },
    { 0x3d8487b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56487be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2487c5, 33 },
    { 0xe51487c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89487c9, 90 },
    { 0x414487ca, 34 },
    { 0x3ca487ca, 95 },
    { 0x8f8487cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654487d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d487dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f487e1, 77 },
    { 0xa61487e2, 17 },
    { 0, 0 },
    { 0x9b6487e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d487e8, 90 },
    { 0, 0 },
    { 0x2ae487ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877487f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x385487f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x763487f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f487fc, 34 },
    { 0xff9487fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7148803, 37 },
    { 0xeb248804, 33 },
    { 0, 0 },
    { 0x13548806, 90 },
    { 0, 0 },
    { 0x35d48808, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4880d, 90 },
    { 0x9854880e, 90 },
    { 0x7494880f, 33 },
    { 0, 0 },
    { 0x78d48811, 55 },
    { 0, 0 },
    { 0xf1448813, 23 },
    { 0x90a48814, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95348818, 98 },
    { 0, 0 },
    { 0x7ee4881a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb48823, 90 },
    { 0x64b48824, 95 },
    { 0xdb948823, 93 },
    { 0x26248826, 46 },
    { 0, 0 },
    { 0x3ac48828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f84882c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f648830, 34 },
    { 0x5e148831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11448834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde948837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034883b, 98 },
    { 0x8514883c, 55 },
    { 0x5ab4883c, 90 },
    { 0x1c74883c, 90 },
    { 0x5eb4883c, 90 },
    { 0x99a48840, 61 },
    { 0xc5048841, 90 },
    { 0xf9548840, 95 },
    { 0x28148843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e48847, 98 },
    { 0x86748848, 95 },
    { 0x4cf48849, 33 },
    { 0x49748848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f4884d, 34 },
    { 0, 0 },
    { 0xdc24884f, 33 },
    { 0, 0 },
    { 0x2348851, 33 },
    { 0xbcc48852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73748859, 34 },
    { 0, 0 },
    { 0x5bc4885b, 95 },
    { 0xcbc4885c, 75 },
    { 0x7ec4885d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73548861, 98 },
    { 0x53248862, 55 },
    { 0x3fc48861, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f48868, 33 },
    { 0x31f48868, 33 },
    { 0xb1748869, 33 },
    { 0xc5748869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb48874, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda848878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4748880, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e748883, 61 },
    { 0x13c48884, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6604888a, 23 },
    { 0x4bf4888b, 55 },
    { 0xb824888a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd48893, 33 },
    { 0, 0 },
    { 0x10248895, 33 },
    { 0xc7d48896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f048899, 55 },
    { 0x9ef4889a, 87 },
    { 0x6484889b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35488a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff488a8, 91 },
    { 0, 0 },
    { 0x716488aa, 52 },
    { 0xe4c488aa, 61 },
    { 0xc62488ac, 33 },
    { 0x12c488ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380488b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3488bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd488c0, 90 },
    { 0x74b488c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3488c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0488ce, 95 },
    { 0, 0 },
    { 0xb4a488d0, 90 },
    { 0, 0 },
    { 0x7d7488d2, 90 },
    { 0x3e9488d3, 66 },
    { 0x45a488d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x982488d7, 43 },
    { 0x216488d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e488db, 66 },
    { 0, 0 },
    { 0xc2b488dd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6488e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464488e5, 100 },
    { 0x66b488e6, 98 },
    { 0, 0 },
    { 0xbf3488e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989488ec, 98 },
    { 0, 0 },
    { 0x43a488ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x925488f1, 90 },
    { 0x172488f2, 52 },
    { 0, 0 },
    { 0x129488f4, 90 },
    { 0, 0 },
    { 0x3aa488f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498488fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9488fe, 77 },
    { 0x3f4488ff, 98 },
    { 0x4b148900, 90 },
    { 0, 0 },
    { 0x7f148902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e48906, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d48909, 98 },
    { 0x1414890a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24890f, 95 },
    { 0, 0 },
    { 0xd1348911, 43 },
    { 0, 0 },
    { 0x9c48913, 76 },
    { 0, 0 },
    { 0x95948915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1534891a, 90 },
    { 0, 0 },
    { 0x2c54891c, 43 },
    { 0, 0 },
    { 0xcf4891e, 95 },
    { 0xf904891e, 69 },
    { 0xd7e48920, 33 },
    { 0xa8d4891f, 33 },
    { 0xcaa4891f, 34 },
    { 0, 0 },
    { 0x2148924, 34 },
    { 0, 0 },
    { 0x66648926, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2048929, 100 },
    { 0, 0 },
    { 0x4284892b, 95 },
    { 0xbdb4892c, 61 },
    { 0, 0 },
    { 0xc4892e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf48931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e148936, 34 },
    { 0, 0 },
    { 0x90d48938, 98 },
    { 0x21548938, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa094893f, 93 },
    { 0, 0 },
    { 0xf48941, 33 },
    { 0x99148942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a148945, 90 },
    { 0x96848946, 33 },
    { 0xce048946, 23 },
    { 0x42148946, 95 },
    { 0x7b348949, 76 },
    { 0xc0b4894a, 61 },
    { 0x8084894b, 90 },
    { 0x6204894c, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c348950, 72 },
    { 0x23948951, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef48958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade4895f, 88 },
    { 0x8b48960, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c448965, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d48968, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d04896b, 33 },
    { 0, 0 },
    { 0x68a4896d, 99 },
    { 0x5df4896e, 65 },
    { 0xf6e4896f, 76 },
    { 0xcc48970, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c48976, 93 },
    { 0, 0 },
    { 0xdfc48978, 91 },
    { 0xf5048978, 90 },
    { 0x70e48979, 85 },
    { 0xa94897a, 90 },
    { 0xdd648978, 23 },
    { 0xbf248979, 101 },
    { 0x7484897d, 33 },
    { 0xb94897f, 98 },
    { 0, 0 },
    { 0xefa48981, 77 },
    { 0xfdb48982, 76 },
    { 0xba948982, 34 },
    { 0xda848984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f948988, 55 },
    { 0, 0 },
    { 0x1dc4898a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4898e, 90 },
    { 0x9484898f, 83 },
    { 0xb6d4898e, 93 },
    { 0x1eb48991, 61 },
    { 0x7b348992, 95 },
    { 0x64c48993, 90 },
    { 0x99048994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c48997, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d4899e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x633489a1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5489a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0489a7, 76 },
    { 0, 0 },
    { 0x3e0489a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea489ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f489b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x105489b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7489bf, 23 },
    { 0xe34489bf, 61 },
    { 0x482489c1, 23 },
    { 0xe5d489c2, 61 },
    { 0, 0 },
    { 0xee8489c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e489ca, 84 },
    { 0, 0 },
    { 0x3a4489cc, 61 },
    { 0x3de489cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8489d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79489d3, 98 },
    { 0x191489d4, 78 },
    { 0, 0 },
    { 0xfab489d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29489d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343489dd, 95 },
    { 0x3fe489de, 66 },
    { 0xda0489df, 67 },
    { 0xfbc489de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849489e4, 99 },
    { 0x4a0489e5, 52 },
    { 0xc9c489e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6489ec, 90 },
    { 0xcb4489ed, 88 },
    { 0xe85489ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35489f1, 61 },
    { 0, 0 },
    { 0x1dc489f3, 76 },
    { 0xbf489f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf489f9, 76 },
    { 0xfdc489fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4489fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd248a01, 23 },
    { 0x4d648a02, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45848a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe48a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d48a10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd48a18, 98 },
    { 0x81648a18, 90 },
    { 0xfc248a19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648a1d, 90 },
    { 0xf2848a1d, 95 },
    { 0, 0 },
    { 0xe6248a20, 59 },
    { 0x8b948a21, 76 },
    { 0, 0 },
    { 0x92d48a23, 61 },
    { 0x74348a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84748a27, 98 },
    { 0x94148a28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7448a2b, 34 },
    { 0, 0 },
    { 0xa1a48a2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c48a36, 66 },
    { 0xbf948a37, 95 },
    { 0x94d48a38, 95 },
    { 0, 0 },
    { 0x6648a3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x14448a3d, 98 },
    { 0, 0 },
    { 0x97148a3f, 23 },
    { 0x83548a40, 76 },
    { 0, 0 },
    { 0xa2948a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d48a4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f248a4f, 43 },
    { 0, 0 },
    { 0xc4a48a51, 23 },
    { 0, 0 },
    { 0xaa648a53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a48a58, 34 },
    { 0x3fd48a59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b48a62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a048a66, 61 },
    { 0x5a748a67, 68 },
    { 0x70d48a67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10648a6d, 95 },
    { 0, 0 },
    { 0x87e48a6f, 34 },
    { 0x77648a70, 23 },
    { 0x44248a71, 17 },
    { 0x1c548a71, 23 },
    { 0x7ac48a6f, 85 },
    { 0xd8348a74, 81 },
    { 0x67e48a75, 90 },
    { 0x8c148a76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd148a7b, 76 },
    { 0xe2748a7b, 55 },
    { 0x46d48a7c, 90 },
    { 0xb4248a7d, 93 },
    { 0xfa448a7d, 33 },
    { 0, 0 },
    { 0xe3548a81, 53 },
    { 0x47e48a81, 34 },
    { 0xb2a48a83, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6148a89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11748a8d, 47 },
    { 0x6ad48a8d, 91 },
    { 0x4f748a8f, 52 },
    { 0x7a748a8f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e48a96, 61 },
    { 0x60b48a97, 33 },
    { 0x9da48a96, 55 },
    { 0x99b48a99, 34 },
    { 0x6d48a9a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf248aa2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d48aa6, 87 },
    { 0, 0 },
    { 0x48048aa8, 43 },
    { 0xe2448aa9, 23 },
    { 0x41848aaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66648aae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748ab1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64648ab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9948ac0, 33 },
    { 0x93f48ac1, 55 },
    { 0x35448ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b48ac5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12848ac9, 95 },
    { 0xe8248aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce48ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11348ad1, 34 },
    { 0, 0 },
    { 0xcf48ad3, 76 },
    { 0, 0 },
    { 0x3d748ad5, 34 },
    { 0, 0 },
    { 0x1e848ad7, 46 },
    { 0, 0 },
    { 0xd1348ad9, 90 },
    { 0xe0748ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4248ae1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4448ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28748ae9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d48af1, 76 },
    { 0x18f48af2, 36 },
    { 0x19048af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48248af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb48afe, 95 },
    { 0, 0 },
    { 0xf2148b00, 98 },
    { 0x55f48b01, 98 },
    { 0, 0 },
    { 0x1b348b03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64348b06, 77 },
    { 0xecd48b06, 43 },
    { 0xa7a48b08, 33 },
    { 0, 0 },
    { 0x35d48b0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af48b0f, 61 },
    { 0xcda48b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d48b15, 34 },
    { 0x6f48b15, 33 },
    { 0x7b548b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648b1a, 40 },
    { 0, 0 },
    { 0xa0148b1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d48b2a, 98 },
    { 0xcfe48b2b, 67 },
    { 0, 0 },
    { 0x9f48b2d, 99 },
    { 0xfe48b2d, 66 },
    { 0x73f48b2f, 76 },
    { 0x12a48b2f, 43 },
    { 0x8cb48b31, 33 },
    { 0x21a48b32, 90 },
    { 0x69f48b33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb248b36, 66 },
    { 0, 0 },
    { 0xe9248b38, 98 },
    { 0x5a148b38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e748b3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4948b43, 33 },
    { 0xd6548b44, 90 },
    { 0x4b48b45, 98 },
    { 0x54b48b45, 33 },
    { 0, 0 },
    { 0xd8e48b48, 33 },
    { 0, 0 },
    { 0xcf348b4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a48b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b648b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f48b59, 40 },
    { 0x9d748b59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b48b5d, 33 },
    { 0, 0 },
    { 0x5d948b5f, 76 },
    { 0x8a448b60, 90 },
    { 0x5f748b61, 61 },
    { 0x49448b62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0148b65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b748b68, 33 },
    { 0xc4f48b69, 90 },
    { 0, 0 },
    { 0x69248b6b, 55 },
    { 0, 0 },
    { 0x748b6d, 61 },
    { 0x81448b6d, 8 },
    { 0x1b848b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f48b76, 61 },
    { 0xb6648b77, 61 },
    { 0x3c748b77, 77 },
    { 0x24a48b78, 67 },
    { 0x62948b7a, 95 },
    { 0x59548b7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48b81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef448b86, 61 },
    { 0xf5948b86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c48b8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d48b91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd648b98, 33 },
    { 0x22548b98, 33 },
    { 0xdea48b9a, 95 },
    { 0x9e548b9b, 77 },
    { 0xfd48b9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb48b9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67048ba2, 80 },
    { 0x3cb48ba3, 77 },
    { 0xb3e48ba4, 99 },
    { 0x7f648ba5, 76 },
    { 0x71048ba6, 98 },
    { 0x13248ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb648bab, 90 },
    { 0, 0 },
    { 0x48148bad, 42 },
    { 0x2d148bae, 90 },
    { 0xf0748bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc048bb4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48bb8, 95 },
    { 0, 0 },
    { 0x3e748bba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51148bbe, 77 },
    { 0xfd848bbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f48bc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec248bc8, 23 },
    { 0, 0 },
    { 0xdbe48bca, 95 },
    { 0, 0 },
    { 0x3a348bcc, 61 },
    { 0x99e48bcd, 61 },
    { 0xd7b48bce, 78 },
    { 0x9d448bce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc48bd9, 43 },
    { 0, 0 },
    { 0xf7348bdb, 78 },
    { 0x97448bdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348be3, 91 },
    { 0x1c448be3, 83 },
    { 0, 0 },
    { 0x12348be6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3448be9, 23 },
    { 0xe248bea, 23 },
    { 0x35748beb, 34 },
    { 0x78748beb, 23 },
    { 0x71a48bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1848bf1, 43 },
    { 0xa1748bf1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59248bf8, 55 },
    { 0x7e148bf9, 23 },
    { 0x72848bf9, 43 },
    { 0xfcb48bfb, 90 },
    { 0xdc948bf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0748c01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf148c05, 77 },
    { 0x36c48c06, 84 },
    { 0xe9c48c07, 61 },
    { 0xde748c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a248c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5948c13, 24 },
    { 0xf2848c14, 61 },
    { 0x7e48c13, 43 },
    { 0x29648c14, 61 },
    { 0, 0 },
    { 0xeda48c18, 95 },
    { 0xdc48c18, 34 },
    { 0x22d48c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9348c1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26948c26, 33 },
    { 0x2d648c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d48c2b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14148c2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0948c32, 93 },
    { 0xa6b48c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2148c37, 61 },
    { 0xce348c38, 61 },
    { 0x96248c39, 43 },
    { 0x81c48c3a, 95 },
    { 0x49448c3b, 90 },
    { 0, 0 },
    { 0xdd848c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46748c40, 23 },
    { 0x67548c40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda148c48, 95 },
    { 0, 0 },
    { 0x64b48c4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97248c51, 46 },
    { 0x9ae48c52, 46 },
    { 0, 0 },
    { 0x8048c54, 77 },
    { 0, 0 },
    { 0x1f048c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b48c5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb648c61, 99 },
    { 0, 0 },
    { 0xb9848c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa848c6a, 23 },
    { 0xc1448c6b, 46 },
    { 0x46e48c6b, 90 },
    { 0x20948c6a, 93 },
    { 0, 0 },
    { 0xab648c6f, 98 },
    { 0xe548c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e48c77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb748c87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a48c8a, 95 },
    { 0x49d48c8b, 98 },
    { 0xbd648c8c, 95 },
    { 0xabd48c8d, 90 },
    { 0, 0 },
    { 0x80748c8f, 98 },
    { 0xbb548c8f, 90 },
    { 0x8d448c91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c48c95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46348ca2, 23 },
    { 0, 0 },
    { 0x8f348ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85048ca8, 23 },
    { 0, 0 },
    { 0x6d548caa, 76 },
    { 0, 0 },
    { 0xaf448cac, 90 },
    { 0, 0 },
    { 0xdd148cae, 61 },
    { 0x6f148cae, 40 },
    { 0, 0 },
    { 0xede48cb1, 81 },
    { 0xc0948cb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd248cb5, 17 },
    { 0x94748cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b48cb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb848cbe, 66 },
    { 0xc3b48cbf, 76 },
    { 0xe4b48cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a48cc5, 93 },
    { 0x6b748cc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d48cc9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2948cce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18548cd6, 23 },
    { 0, 0 },
    { 0x19b48cd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0248cdb, 95 },
    { 0x5ad48cdc, 77 },
    { 0x36548cdc, 95 },
    { 0xf5e48cde, 43 },
    { 0x2b948cdc, 85 },
    { 0x99b48cde, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x10348ce3, 88 },
    { 0x10c48ce4, 34 },
    { 0x78e48ce5, 81 },
    { 0, 0 },
    { 0x8d248ce7, 78 },
    { 0, 0 },
    { 0x66d48ce9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61948cee, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa48cf2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x84848cf5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d48cfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c48d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f48d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc48d06, 97 },
    { 0, 0 },
    { 0x28548d08, 90 },
    { 0, 0 },
    { 0x82548d0a, 33 },
    { 0, 0 },
    { 0xa6948d0c, 33 },
    { 0, 0 },
    { 0x13648d0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a48d11, 90 },
    { 0xc8c48d11, 33 },
    { 0, 0 },
    { 0xa1648d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69348d19, 90 },
    { 0, 0 },
    { 0x83948d1b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2648d21, 91 },
    { 0xa5a48d22, 81 },
    { 0x97048d22, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58848d28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d48d2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50648d2f, 55 },
    { 0xc5f48d2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff448d33, 76 },
    { 0xbf948d34, 85 },
    { 0, 0 },
    { 0x12448d36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1348d3e, 34 },
    { 0, 0 },
    { 0x3a948d40, 36 },
    { 0x89348d41, 90 },
    { 0xf1548d40, 98 },
    { 0x13f48d43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99148d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37348d4c, 81 },
    { 0x6248d4d, 87 },
    { 0x8a548d4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d748d51, 55 },
    { 0, 0 },
    { 0x85948d53, 23 },
    { 0, 0 },
    { 0x79d48d55, 90 },
    { 0, 0 },
    { 0xb9748d57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e48d5a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a148d5d, 33 },
    { 0x99848d5e, 33 },
    { 0xf5a48d5d, 90 },
    { 0xb5e48d5f, 61 },
    { 0, 0 },
    { 0x3d548d62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b248d67, 33 },
    { 0x10048d68, 55 },
    { 0, 0 },
    { 0x46848d6a, 61 },
    { 0xe0048d6b, 23 },
    { 0x90548d6b, 46 },
    { 0, 0 },
    { 0xdd948d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb48d72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff648d76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e48d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44548d7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18348d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f48d83, 85 },
    { 0x8d848d84, 67 },
    { 0, 0 },
    { 0x4dc48d86, 34 },
    { 0x24448d87, 77 },
    { 0x29048d88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3848d8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c48d91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f148d98, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23448da0, 55 },
    { 0xec548da0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57448da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40448da8, 34 },
    { 0xd0448da8, 65 },
    { 0x84748daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa248db7, 33 },
    { 0, 0 },
    { 0xa7848db9, 98 },
    { 0, 0 },
    { 0x15c48dbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a48dbf, 61 },
    { 0x63548dc0, 33 },
    { 0x64048dc1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648dc4, 75 },
    { 0x36248dc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b48dc8, 36 },
    { 0x35c48dc8, 33 },
    { 0xf48dca, 95 },
    { 0x24148dc8, 98 },
    { 0x38048dcb, 98 },
    { 0x85448dcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8548dd0, 33 },
    { 0x54f48dd1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc48dd4, 90 },
    { 0xf4248dd5, 23 },
    { 0x33948dd5, 61 },
    { 0x83748dd7, 61 },
    { 0xc1848dd8, 61 },
    { 0xda648dd6, 23 },
    { 0x41548dd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f48ddf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74548de3, 23 },
    { 0x1c448de3, 85 },
    { 0, 0 },
    { 0xf1a48de6, 95 },
    { 0, 0 },
    { 0x88948de8, 95 },
    { 0x8eb48de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca48dec, 90 },
    { 0, 0 },
    { 0x9ab48dee, 61 },
    { 0x52548def, 95 },
    { 0x13d48dee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75548df6, 43 },
    { 0x2648df7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf448dfd, 98 },
    { 0, 0 },
    { 0x13748dff, 90 },
    { 0x9a348e00, 67 },
    { 0x46848e01, 99 },
    { 0x7d248e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d48e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd48e0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab548e12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48548e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14248e1d, 81 },
    { 0, 0 },
    { 0x4d048e1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d48e22, 94 },
    { 0x23348e22, 55 },
    { 0, 0 },
    { 0x28a48e25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d48e2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d848e30, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b348e34, 66 },
    { 0, 0 },
    { 0x2a348e36, 33 },
    { 0xa8848e37, 33 },
    { 0x78e48e37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b48e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53648e40, 61 },
    { 0x3fa48e41, 64 },
    { 0x55d48e40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c48e46, 95 },
    { 0x7ea48e47, 61 },
    { 0xf0d48e47, 61 },
    { 0x2b848e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a848e50, 23 },
    { 0x38b48e50, 34 },
    { 0, 0 },
    { 0x1f948e53, 93 },
    { 0, 0 },
    { 0xb448e55, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81748e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b48e5c, 33 },
    { 0, 0 },
    { 0x2fc48e5e, 34 },
    { 0, 0 },
    { 0x1848e60, 43 },
    { 0xf6c48e60, 85 },
    { 0x50348e62, 90 },
    { 0xd2e48e63, 61 },
    { 0x92648e60, 66 },
    { 0xdca48e65, 34 },
    { 0x44348e62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a848e6b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19348e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d48e7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64848e80, 76 },
    { 0x2b548e81, 23 },
    { 0xac748e81, 61 },
    { 0, 0 },
    { 0x2cd48e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff448e88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd48e8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd848e91, 23 },
    { 0x1ea48e92, 61 },
    { 0x58048e93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b48e9c, 33 },
    { 0, 0 },
    { 0x5048e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe948eaa, 98 },
    { 0xac48eab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5548eb4, 23 },
    { 0xcf48eb5, 61 },
    { 0xa6548eb6, 85 },
    { 0xeda48eb7, 90 },
    { 0x4e48eb6, 61 },
    { 0x6bd48eb7, 95 },
    { 0x37048eb4, 23 },
    { 0, 0 },
    { 0xb4c48ebc, 91 },
    { 0x6e548ebd, 17 },
    { 0, 0 },
    { 0x74b48ebf, 90 },
    { 0, 0 },
    { 0x2e748ec1, 95 },
    { 0, 0 },
    { 0x8a448ec3, 23 },
    { 0xa8248ec3, 84 },
    { 0xc0448ec5, 100 },
    { 0, 0 },
    { 0x6a48ec7, 33 },
    { 0, 0 },
    { 0x6ba48ec9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab348ecd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d448ed1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3148ed6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36848edd, 34 },
    { 0x19648ede, 23 },
    { 0x7e748edf, 66 },
    { 0x3fd48ede, 23 },
    { 0, 0 },
    { 0xf1848ee2, 43 },
    { 0x43848ee2, 76 },
    { 0x3d148ee4, 90 },
    { 0xc4748ee5, 61 },
    { 0xe6b48ee6, 17 },
    { 0xc9a48ee6, 23 },
    { 0x99448ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b48eeb, 17 },
    { 0xa0848eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd48eef, 23 },
    { 0xe448ef0, 78 },
    { 0xcd248ef0, 23 },
    { 0, 0 },
    { 0x10b48ef3, 38 },
    { 0x1af48ef4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88648ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33148efd, 23 },
    { 0x7f548efd, 33 },
    { 0x9c48eff, 43 },
    { 0x4a348eff, 90 },
    { 0xe0948f01, 23 },
    { 0xc1f48eff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8248f05, 61 },
    { 0, 0 },
    { 0xab248f07, 33 },
    { 0x4f148f08, 23 },
    { 0xef48f09, 90 },
    { 0, 0 },
    { 0x12e48f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43748f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc748f14, 23 },
    { 0x22948f15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a48f1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7348f21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e948f27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df48f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c48f2f, 34 },
    { 0x94048f2f, 33 },
    { 0xe2448f2f, 46 },
    { 0x7f648f31, 67 },
    { 0xcc748f2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d548f37, 36 },
    { 0xc5e48f38, 33 },
    { 0x2d148f39, 61 },
    { 0xbe248f3a, 76 },
    { 0xc1148f37, 93 },
    { 0x14f48f3c, 33 },
    { 0xb4748f39, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe48f40, 90 },
    { 0x50548f40, 23 },
    { 0, 0 },
    { 0x1e948f43, 95 },
    { 0, 0 },
    { 0x9148f45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b148f4f, 98 },
    { 0xdfd48f4f, 76 },
    { 0xe5348f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab48f56, 43 },
    { 0x34b48f57, 61 },
    { 0xe3148f58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a48f5b, 90 },
    { 0, 0 },
    { 0xcaa48f5d, 67 },
    { 0xce248f5e, 84 },
    { 0x30948f5f, 80 },
    { 0xda648f60, 90 },
    { 0, 0 },
    { 0x3eb48f62, 98 },
    { 0x3048f62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d48f66, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27048f6b, 90 },
    { 0xcd748f6b, 90 },
    { 0xe1948f6d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a48f71, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648f74, 90 },
    { 0x63c48f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d48f78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348f7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15748f83, 93 },
    { 0x62e48f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12748f87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb848f8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72648f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348f92, 33 },
    { 0x7648f93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f48f96, 77 },
    { 0, 0 },
    { 0x71a48f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71548fa1, 69 },
    { 0, 0 },
    { 0x22248fa3, 46 },
    { 0xeec48fa3, 77 },
    { 0, 0 },
    { 0xbac48fa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39648faf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4048fb2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd148fb5, 61 },
    { 0, 0 },
    { 0xd0648fb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xded48fba, 66 },
    { 0x2f348fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b48fc1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61348fca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f48fcd, 43 },
    { 0, 0 },
    { 0xaa048fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a148fd3, 55 },
    { 0, 0 },
    { 0x80c48fd5, 46 },
    { 0x51848fd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f948fe0, 87 },
    { 0x6c748fe1, 98 },
    { 0x16b48fe2, 98 },
    { 0x39448fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62548feb, 23 },
    { 0x87148fec, 33 },
    { 0x75248feb, 61 },
    { 0, 0 },
    { 0x48e48fef, 100 },
    { 0xd6448ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f448ff5, 33 },
    { 0x26848ff6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd48ffe, 97 },
    { 0xa9248ffe, 68 },
    { 0x26e48ffe, 12 },
    { 0xfe549001, 98 },
    { 0x3d149002, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4900a, 90 },
    { 0x2394900a, 90 },
    { 0xc5d4900a, 95 },
    { 0x5244900a, 90 },
    { 0, 0 },
    { 0xe594900f, 84 },
    { 0x2a34900f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37849013, 95 },
    { 0, 0 },
    { 0x5549015, 87 },
    { 0xa2c49016, 33 },
    { 0xee549017, 17 },
    { 0x12f49018, 61 },
    { 0xbf649018, 43 },
    { 0xbe14901a, 38 },
    { 0xa1b49019, 90 },
    { 0, 0 },
    { 0xc614901d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e949020, 99 },
    { 0, 0 },
    { 0x8b249022, 23 },
    { 0, 0 },
    { 0xd7f49024, 23 },
    { 0x5d949024, 55 },
    { 0xa8d49024, 98 },
    { 0, 0 },
    { 0x64449028, 76 },
    { 0x27549029, 100 },
    { 0x52b4902a, 23 },
    { 0x7d74902a, 76 },
    { 0x13449029, 33 },
    { 0x20449029, 84 },
    { 0x5214902b, 76 },
    { 0xebd4902a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e149037, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb204903a, 88 },
    { 0xb34903b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21049040, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31349043, 95 },
    { 0x93249044, 61 },
    { 0x38a49045, 80 },
    { 0x4cc49046, 33 },
    { 0x4f49046, 55 },
    { 0xf3149046, 23 },
    { 0xb6549045, 55 },
    { 0xf7b4904a, 23 },
    { 0x97f49045, 81 },
    { 0xafb49049, 100 },
    { 0xa3549046, 85 },
    { 0x80a4904e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d49052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c64905a, 77 },
    { 0x2b44905b, 65 },
    { 0x5084905c, 61 },
    { 0xbaf4905a, 69 },
    { 0x3704905e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf949061, 46 },
    { 0xe3549061, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa749065, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce049068, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c4906c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45449070, 61 },
    { 0x44249071, 83 },
    { 0xfd249071, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4907a, 67 },
    { 0x4ab4907a, 87 },
    { 0x39e4907c, 99 },
    { 0, 0 },
    { 0x86e4907e, 34 },
    { 0xba4907f, 66 },
    { 0x5bb4907e, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a49084, 95 },
    { 0x5aa49085, 36 },
    { 0x5ed49086, 52 },
    { 0xa3c49087, 90 },
    { 0x33049088, 95 },
    { 0x28049089, 88 },
    { 0x3b449088, 97 },
    { 0x6e4908a, 38 },
    { 0x5f24908b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d549090, 93 },
    { 0, 0 },
    { 0xf0249092, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67249095, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64909a, 95 },
    { 0xcae4909b, 61 },
    { 0xc754909a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64909f, 61 },
    { 0xb4f4909f, 55 },
    { 0xac7490a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9490a7, 98 },
    { 0, 0 },
    { 0x95e490a9, 76 },
    { 0xb3b490a9, 61 },
    { 0, 0 },
    { 0x3ee490ac, 99 },
    { 0, 0 },
    { 0x64b490ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8490b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95490b9, 76 },
    { 0x16a490ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76490bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf490c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba490ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd490cd, 95 },
    { 0xb15490ce, 43 },
    { 0x2fa490cf, 46 },
    { 0xeac490d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72490d4, 93 },
    { 0x3ba490d5, 87 },
    { 0x723490d6, 40 },
    { 0, 0 },
    { 0x5db490d8, 55 },
    { 0x3aa490d9, 23 },
    { 0xeb3490d9, 93 },
    { 0xe28490d9, 23 },
    { 0xc80490dc, 90 },
    { 0xb56490dd, 61 },
    { 0xec0490dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca490e5, 33 },
    { 0xca0490e6, 90 },
    { 0xfe490e7, 55 },
    { 0xc39490e8, 94 },
    { 0xa89490e6, 33 },
    { 0, 0 },
    { 0xbbd490eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x475490ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f490f5, 93 },
    { 0xb64490f5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x145490f9, 78 },
    { 0xfc5490fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f49103, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd49106, 52 },
    { 0x80149106, 23 },
    { 0x7ee49107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4910d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6449113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f49117, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984911a, 67 },
    { 0x7ce4911b, 93 },
    { 0x8b84911a, 61 },
    { 0, 0 },
    { 0xf034911e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b149124, 87 },
    { 0, 0 },
    { 0x9649126, 23 },
    { 0x77849127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd814912a, 76 },
    { 0x4004912b, 95 },
    { 0xb124912a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4874912f, 88 },
    { 0, 0 },
    { 0xec049131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc49138, 90 },
    { 0x64049139, 69 },
    { 0x71c49139, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa044913d, 85 },
    { 0, 0 },
    { 0x3374913f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb49144, 33 },
    { 0, 0 },
    { 0x20549146, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d249150, 93 },
    { 0x8bc49151, 90 },
    { 0x8a49152, 61 },
    { 0x20849150, 90 },
    { 0xe5249151, 23 },
    { 0xf2a49152, 76 },
    { 0xc7149156, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa254915c, 98 },
    { 0x64b4915d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff349160, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7149167, 46 },
    { 0x44e49168, 61 },
    { 0, 0 },
    { 0x7bb4916a, 95 },
    { 0x86f4916b, 87 },
    { 0x2d84916b, 98 },
    { 0xdfe4916d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac49172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a149176, 34 },
    { 0xc3f49177, 99 },
    { 0, 0 },
    { 0x9b49179, 90 },
    { 0xc6c4917a, 81 },
    { 0x8da49179, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1549180, 33 },
    { 0, 0 },
    { 0xffb49182, 75 },
    { 0, 0 },
    { 0x92c49184, 95 },
    { 0xc0149185, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf314918a, 23 },
    { 0, 0 },
    { 0xa504918c, 23 },
    { 0xebe4918c, 66 },
    { 0x11c4918c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc49191, 23 },
    { 0x72e49191, 64 },
    { 0, 0 },
    { 0xe7749194, 65 },
    { 0xd549195, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574919a, 23 },
    { 0xda24919b, 43 },
    { 0xee74919a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff491a4, 34 },
    { 0xaa3491a5, 66 },
    { 0xc8a491a6, 98 },
    { 0xb74491a6, 61 },
    { 0x745491a7, 95 },
    { 0xfbe491a7, 95 },
    { 0xa91491aa, 67 },
    { 0x6e491ab, 90 },
    { 0, 0 },
    { 0x893491ad, 95 },
    { 0x243491ad, 87 },
    { 0x167491af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60491b2, 61 },
    { 0xfdc491b2, 61 },
    { 0x1e3491b3, 80 },
    { 0, 0 },
    { 0x308491b6, 98 },
    { 0, 0 },
    { 0x8df491b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a491bc, 68 },
    { 0xb01491bc, 33 },
    { 0xd6f491bd, 34 },
    { 0, 0 },
    { 0x499491c0, 23 },
    { 0x878491c1, 61 },
    { 0x433491c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x853491c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61491c8, 98 },
    { 0x2d4491c9, 33 },
    { 0xc6b491c8, 95 },
    { 0xf99491c9, 90 },
    { 0xeee491c8, 64 },
    { 0xed1491ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb491d3, 98 },
    { 0x570491d4, 98 },
    { 0x671491d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5491e5, 75 },
    { 0xa3491e6, 90 },
    { 0x28c491e7, 95 },
    { 0xb95491e6, 43 },
    { 0x8c0491e9, 83 },
    { 0xb0b491ea, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x196491ed, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49491f2, 90 },
    { 0, 0 },
    { 0x4bd491f4, 90 },
    { 0, 0 },
    { 0x38a491f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7491fa, 33 },
    { 0, 0 },
    { 0x3b2491fc, 33 },
    { 0x58b491fd, 93 },
    { 0, 0 },
    { 0xd22491ff, 33 },
    { 0xe9049200, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d449203, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x83949206, 61 },
    { 0, 0 },
    { 0x79349208, 80 },
    { 0, 0 },
    { 0x7294920a, 77 },
    { 0xa134920b, 77 },
    { 0xe954920b, 85 },
    { 0xf564920c, 93 },
    { 0xac74920e, 61 },
    { 0x4d54920e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6849212, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f49216, 93 },
    { 0, 0 },
    { 0xf9649218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724921b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d49220, 93 },
    { 0x80f49220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149225, 43 },
    { 0xfc249226, 88 },
    { 0x6049225, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e749230, 23 },
    { 0x92d49230, 33 },
    { 0xe2549231, 90 },
    { 0, 0 },
    { 0xc3149234, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc49239, 95 },
    { 0, 0 },
    { 0x74b4923b, 98 },
    { 0, 0 },
    { 0x28b4923d, 76 },
    { 0, 0 },
    { 0x7f44923f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab49242, 33 },
    { 0xf9749243, 33 },
    { 0x63e49244, 23 },
    { 0, 0 },
    { 0x67749246, 76 },
    { 0x91d49247, 33 },
    { 0x69549247, 61 },
    { 0xa0749246, 34 },
    { 0xa3e4924a, 61 },
    { 0x2ca49248, 98 },
    { 0x82a49246, 66 },
    { 0xe964924c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb49252, 90 },
    { 0x7d449253, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb49256, 61 },
    { 0, 0 },
    { 0x8b249258, 99 },
    { 0x14649259, 97 },
    { 0, 0 },
    { 0x3a54925b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f049260, 23 },
    { 0, 0 },
    { 0x34f49262, 34 },
    { 0, 0 },
    { 0xb8c49264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c649267, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f4926a, 67 },
    { 0x57f4926a, 46 },
    { 0xa2c4926a, 61 },
    { 0x7b64926d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea49271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f49274, 23 },
    { 0, 0 },
    { 0xdac49276, 98 },
    { 0, 0 },
    { 0x24049278, 33 },
    { 0x26949278, 99 },
    { 0xf3d49279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c4927d, 33 },
    { 0xe664927e, 17 },
    { 0x1744927e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18149288, 33 },
    { 0x65849289, 90 },
    { 0x3004928a, 99 },
    { 0x15e4928b, 90 },
    { 0xab54928c, 61 },
    { 0x52449289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e49292, 33 },
    { 0, 0 },
    { 0xddf49294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094929b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e492a0, 70 },
    { 0x55a492a1, 69 },
    { 0x1c6492a2, 33 },
    { 0xe4e492a3, 61 },
    { 0xfe6492a2, 90 },
    { 0, 0 },
    { 0x985492a6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0492b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9492b4, 34 },
    { 0x3df492b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x944492b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40492bf, 98 },
    { 0xd5e492c0, 95 },
    { 0, 0 },
    { 0x35d492c2, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80492c5, 95 },
    { 0xb0c492c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92492c9, 90 },
    { 0x834492c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8492cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e492d4, 46 },
    { 0x9f8492d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce492d8, 55 },
    { 0x228492d9, 81 },
    { 0xec492d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b492e1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99492e5, 17 },
    { 0xfd8492e6, 90 },
    { 0xf58492e7, 75 },
    { 0xc90492e8, 68 },
    { 0xfa0492e7, 68 },
    { 0x634492e7, 61 },
    { 0x871492eb, 95 },
    { 0xc56492e9, 78 },
    { 0xf02492ec, 66 },
    { 0x8b7492ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6492f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128492fa, 88 },
    { 0x70492fb, 78 },
    { 0xc0f492fc, 85 },
    { 0, 0 },
    { 0xa51492fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7649302, 91 },
    { 0xd8b49303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8549308, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84930d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f49315, 52 },
    { 0, 0 },
    { 0x9e349317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd344931a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4931d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d349322, 61 },
    { 0, 0 },
    { 0x66149324, 76 },
    { 0xf2b49325, 95 },
    { 0xab349326, 90 },
    { 0xf9649327, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e4932a, 91 },
    { 0xfbf4932b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b4932e, 90 },
    { 0xee74932f, 23 },
    { 0x5449330, 46 },
    { 0xe0449331, 90 },
    { 0x49749332, 52 },
    { 0x1e049332, 61 },
    { 0x5f49334, 98 },
    { 0xb4749333, 38 },
    { 0x8e649336, 90 },
    { 0x5549337, 65 },
    { 0xe149338, 23 },
    { 0x1f649339, 90 },
    { 0xba74933a, 61 },
    { 0x9c14933b, 33 },
    { 0xc3f4933a, 76 },
    { 0, 0 },
    { 0x4334933e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4449349, 90 },
    { 0, 0 },
    { 0xe5c4934b, 98 },
    { 0x8914934b, 98 },
    { 0, 0 },
    { 0xf244934e, 100 },
    { 0, 0 },
    { 0x7dd49350, 76 },
    { 0, 0 },
    { 0xc5849352, 33 },
    { 0, 0 },
    { 0x78b49354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf49357, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4935a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf649363, 61 },
    { 0xbdf49364, 95 },
    { 0xf6049363, 33 },
    { 0x9249363, 98 },
    { 0, 0 },
    { 0x44949368, 90 },
    { 0x3e849369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754936e, 33 },
    { 0x24a4936e, 80 },
    { 0xa214936f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d49374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1949378, 52 },
    { 0x38949379, 76 },
    { 0x5c649379, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949380, 33 },
    { 0, 0 },
    { 0x2b949382, 23 },
    { 0, 0 },
    { 0xed49384, 90 },
    { 0xe9749385, 33 },
    { 0x95449386, 98 },
    { 0x31549387, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74938e, 90 },
    { 0x174938e, 90 },
    { 0xf1f4938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2149398, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5324939b, 43 },
    { 0xaeb4939c, 90 },
    { 0x96b4939d, 98 },
    { 0x1164939e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503493a2, 95 },
    { 0, 0 },
    { 0x553493a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c493a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a493ad, 98 },
    { 0xbc6493ae, 23 },
    { 0x6f5493ad, 61 },
    { 0x659493af, 61 },
    { 0, 0 },
    { 0x2e0493b2, 43 },
    { 0, 0 },
    { 0xe493b4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47493b8, 33 },
    { 0x4af493b9, 93 },
    { 0x890493ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98493bd, 47 },
    { 0, 0 },
    { 0x57493bf, 61 },
    { 0, 0 },
    { 0xd8c493c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0493c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8493ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d493cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad493d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9493d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68493e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9493e5, 53 },
    { 0x418493e6, 34 },
    { 0, 0 },
    { 0xd06493e8, 69 },
    { 0xe53493e9, 99 },
    { 0xdc9493e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5493ef, 95 },
    { 0x1e4493f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19493f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6493f7, 90 },
    { 0xbfb493f8, 66 },
    { 0x9c0493f7, 91 },
    { 0x596493fa, 66 },
    { 0xac3493fb, 84 },
    { 0x33b493fc, 61 },
    { 0x603493fd, 93 },
    { 0xd0c493fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4249401, 95 },
    { 0, 0 },
    { 0x7f649403, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5749407, 98 },
    { 0xe6649407, 98 },
    { 0x27149409, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6449410, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4941b, 87 },
    { 0x1104941b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d849424, 66 },
    { 0x8d449425, 91 },
    { 0, 0 },
    { 0xd8f49427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b54942f, 34 },
    { 0xa4e49430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f49433, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc149436, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e849440, 43 },
    { 0xa5d49441, 61 },
    { 0x72649442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e049446, 95 },
    { 0xab749447, 39 },
    { 0xf8349446, 33 },
    { 0xf249449, 66 },
    { 0x5f049449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2449450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf49454, 76 },
    { 0, 0 },
    { 0x9eb49456, 34 },
    { 0x7a149457, 76 },
    { 0x7c949458, 61 },
    { 0x19549459, 90 },
    { 0x8c849459, 61 },
    { 0, 0 },
    { 0xb74945c, 33 },
    { 0x73d4945d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd349460, 61 },
    { 0x96a49461, 33 },
    { 0x22049462, 34 },
    { 0, 0 },
    { 0x2d49464, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c4946a, 66 },
    { 0x6764946a, 46 },
    { 0x3fd4946a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd49470, 33 },
    { 0x23a49471, 90 },
    { 0x87149472, 99 },
    { 0x60249473, 33 },
    { 0xc1749471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd49478, 97 },
    { 0, 0 },
    { 0xbc94947a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x714947d, 90 },
    { 0x6094947d, 90 },
    { 0xe7b4947f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca49483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af49488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604948c, 61 },
    { 0xf454948d, 91 },
    { 0, 0 },
    { 0x804948f, 65 },
    { 0xab24948f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad49499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a64949c, 95 },
    { 0, 0 },
    { 0x7a74949e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e494aa, 52 },
    { 0xe6c494ab, 98 },
    { 0x732494aa, 33 },
    { 0xfdb494ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703494b4, 98 },
    { 0x3f0494b5, 77 },
    { 0, 0 },
    { 0xca8494b7, 61 },
    { 0x2e3494b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b494bb, 96 },
    { 0xaad494bc, 43 },
    { 0x537494bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466494c1, 90 },
    { 0, 0 },
    { 0x67b494c3, 61 },
    { 0xe1d494c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183494c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd494cb, 98 },
    { 0, 0 },
    { 0xf92494cd, 52 },
    { 0x1f3494ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b494d2, 76 },
    { 0x5a3494d3, 34 },
    { 0, 0 },
    { 0x51b494d5, 95 },
    { 0xa494d6, 33 },
    { 0xdd5494d7, 33 },
    { 0xca8494d7, 43 },
    { 0x370494d9, 100 },
    { 0xe82494d9, 87 },
    { 0x34a494db, 91 },
    { 0x8de494d9, 90 },
    { 0, 0 },
    { 0xe47494de, 91 },
    { 0x7494de, 95 },
    { 0x307494e0, 81 },
    { 0x476494e0, 66 },
    { 0x4d2494de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44494e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8494e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e494ec, 23 },
    { 0, 0 },
    { 0xa40494ee, 65 },
    { 0x34a494ef, 23 },
    { 0x63c494ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d494f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b494fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb494ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2549502, 52 },
    { 0x53c49502, 46 },
    { 0, 0 },
    { 0xb3949505, 17 },
    { 0x1449506, 91 },
    { 0, 0 },
    { 0xdcd49508, 43 },
    { 0, 0 },
    { 0x6d44950a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3749515, 23 },
    { 0, 0 },
    { 0x76549517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb394951c, 68 },
    { 0x8ab4951c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d49527, 68 },
    { 0xaa549528, 66 },
    { 0x74349528, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4144952c, 85 },
    { 0xba54952d, 95 },
    { 0, 0 },
    { 0x3f84952f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x18049532, 33 },
    { 0x4eb49533, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a249538, 93 },
    { 0xabe49539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64953d, 23 },
    { 0xe54953e, 90 },
    { 0, 0 },
    { 0x54049540, 66 },
    { 0x64049541, 77 },
    { 0xad749542, 95 },
    { 0xa3f49543, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7049552, 95 },
    { 0x10249553, 77 },
    { 0, 0 },
    { 0xd1349555, 23 },
    { 0, 0 },
    { 0xa0049557, 100 },
    { 0x1aa49558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554955c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024955f, 23 },
    { 0x26d49560, 93 },
    { 0, 0 },
    { 0x86249562, 65 },
    { 0xd5c49563, 98 },
    { 0xe9f49564, 66 },
    { 0x5b349562, 33 },
    { 0xc2249562, 99 },
    { 0, 0 },
    { 0x45a49568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4956f, 90 },
    { 0, 0 },
    { 0x3ed49571, 95 },
    { 0x21e49572, 23 },
    { 0, 0 },
    { 0x87749574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef49578, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf284957e, 95 },
    { 0xd344957f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d49582, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd49587, 96 },
    { 0x49c49587, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf104958b, 33 },
    { 0xbb64958c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e49590, 95 },
    { 0x40f49591, 55 },
    { 0xeec49591, 23 },
    { 0x1ca49593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e64959a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc714959f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4495a4, 33 },
    { 0, 0 },
    { 0x594495a6, 93 },
    { 0xb32495a7, 81 },
    { 0, 0 },
    { 0x582495a9, 90 },
    { 0x668495a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a495ad, 61 },
    { 0x856495ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c495b5, 61 },
    { 0x375495b6, 95 },
    { 0, 0 },
    { 0x837495b8, 90 },
    { 0x39495b9, 23 },
    { 0x668495ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10495bd, 90 },
    { 0, 0 },
    { 0x7d1495bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf495c2, 76 },
    { 0x51b495c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5495c7, 91 },
    { 0xecb495c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76495cb, 98 },
    { 0x4a0495cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2495d1, 65 },
    { 0x194495d2, 53 },
    { 0x82f495d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2495e0, 84 },
    { 0, 0 },
    { 0xc81495e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9495ea, 98 },
    { 0x434495eb, 76 },
    { 0x21495ec, 99 },
    { 0xd76495ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9495f0, 90 },
    { 0x834495f0, 33 },
    { 0xef7495f0, 84 },
    { 0x861495f3, 85 },
    { 0xa36495f2, 23 },
    { 0, 0 },
    { 0xa9e495f6, 33 },
    { 0, 0 },
    { 0xb9f495f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e495fc, 55 },
    { 0x2de495fd, 98 },
    { 0x35f495fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x62649601, 93 },
    { 0xba349601, 95 },
    { 0x5c149602, 90 },
    { 0xb3749604, 61 },
    { 0x66449604, 61 },
    { 0x94949601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394960b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee4960e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38649613, 33 },
    { 0x64c49614, 97 },
    { 0, 0 },
    { 0x74a49616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc704961a, 33 },
    { 0, 0 },
    { 0x57e4961c, 87 },
    { 0, 0 },
    { 0xf494961e, 95 },
    { 0, 0 },
    { 0xc6a49620, 23 },
    { 0x32d49620, 97 },
    { 0x15b49622, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d549629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844962f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d749634, 61 },
    { 0x31949635, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59549639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b4963c, 90 },
    { 0x36c4963d, 87 },
    { 0x5d04963e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce849646, 76 },
    { 0xe5649647, 90 },
    { 0xa1149648, 33 },
    { 0x21449649, 93 },
    { 0x4154964a, 94 },
    { 0xb2d49649, 83 },
    { 0x6c14964c, 98 },
    { 0xfc34964d, 52 },
    { 0, 0 },
    { 0xf4b4964f, 98 },
    { 0, 0 },
    { 0x8ea49651, 93 },
    { 0xfad49652, 61 },
    { 0x22449653, 23 },
    { 0x91449653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97949657, 95 },
    { 0x1e149658, 90 },
    { 0xab949659, 90 },
    { 0xd4149657, 61 },
    { 0xad749657, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94965f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d849662, 23 },
    { 0xcb49662, 98 },
    { 0, 0 },
    { 0x8a49665, 90 },
    { 0x5a349666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd4966c, 45 },
    { 0x3314966d, 90 },
    { 0, 0 },
    { 0x7d64966f, 34 },
    { 0, 0 },
    { 0x69c49671, 84 },
    { 0x78b49672, 34 },
    { 0x2ff49673, 98 },
    { 0x6e049671, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e4967a, 91 },
    { 0x6c64967b, 23 },
    { 0xd9e4967a, 33 },
    { 0x9764967d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70149681, 78 },
    { 0x19849681, 98 },
    { 0, 0 },
    { 0x43a49684, 33 },
    { 0, 0 },
    { 0xa8349686, 33 },
    { 0xdfd49687, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794968c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3094968f, 78 },
    { 0x7484968f, 95 },
    { 0xf8849690, 90 },
    { 0x4c54968f, 64 },
    { 0x91c49693, 33 },
    { 0xef749694, 90 },
    { 0x36749690, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4249698, 61 },
    { 0x2a549699, 90 },
    { 0xc984969a, 61 },
    { 0x1ff4969b, 98 },
    { 0x4b149698, 91 },
    { 0x1794969d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15496a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa496a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x365496ab, 61 },
    { 0x5de496ac, 52 },
    { 0x1a8496ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340496bf, 100 },
    { 0, 0 },
    { 0x5a1496c1, 23 },
    { 0, 0 },
    { 0x78e496c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66496c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36496ca, 99 },
    { 0x68d496cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e496d0, 90 },
    { 0x5e9496d1, 17 },
    { 0xd71496d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84496d6, 52 },
    { 0x246496d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72496db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a496de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56496e3, 23 },
    { 0, 0 },
    { 0xf60496e5, 23 },
    { 0x1a2496e6, 90 },
    { 0xa0e496e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32496eb, 81 },
    { 0x65c496ec, 100 },
    { 0x655496eb, 61 },
    { 0x790496eb, 55 },
    { 0, 0 },
    { 0x96496f0, 77 },
    { 0xe69496f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a496f6, 61 },
    { 0xdfe496f7, 43 },
    { 0, 0 },
    { 0x7ab496f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692496fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c49701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b49704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b14970b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf49711, 46 },
    { 0x60e49711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e49716, 23 },
    { 0x64c49717, 93 },
    { 0xf8149718, 88 },
    { 0x66a49717, 33 },
    { 0xfb64971a, 46 },
    { 0x2214971b, 80 },
    { 0x724971c, 23 },
    { 0x294971a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d49724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5149729, 67 },
    { 0, 0 },
    { 0xfda4972b, 46 },
    { 0x6894972c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4972f, 91 },
    { 0, 0 },
    { 0xb8649731, 95 },
    { 0xaa949732, 87 },
    { 0x5749733, 87 },
    { 0x8ff49731, 61 },
    { 0xa249732, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8649738, 52 },
    { 0, 0 },
    { 0xb904973a, 17 },
    { 0xc204973a, 98 },
    { 0, 0 },
    { 0x4a4973d, 90 },
    { 0x9bd4973e, 61 },
    { 0, 0 },
    { 0xbcc49740, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40349745, 95 },
    { 0x64e49746, 90 },
    { 0, 0 },
    { 0x37149748, 95 },
    { 0, 0 },
    { 0x3524974a, 23 },
    { 0, 0 },
    { 0xf0f4974c, 46 },
    { 0, 0 },
    { 0x51c4974e, 34 },
    { 0, 0 },
    { 0x85049750, 61 },
    { 0, 0 },
    { 0x1fa49752, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea549759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe844975c, 33 },
    { 0, 0 },
    { 0x74d4975e, 12 },
    { 0xa1e4975f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8849762, 61 },
    { 0, 0 },
    { 0x13449764, 52 },
    { 0x6a049764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf534976c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45149772, 90 },
    { 0x1bc49773, 61 },
    { 0x82e49774, 61 },
    { 0xca049775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14977c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa49780, 33 },
    { 0x95249781, 78 },
    { 0, 0 },
    { 0xc1f49783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c249786, 33 },
    { 0xb3d49787, 90 },
    { 0xcf049787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d49792, 95 },
    { 0xb6049792, 61 },
    { 0xa2649794, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e49798, 85 },
    { 0xa0e49799, 90 },
    { 0x32749798, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1984979e, 76 },
    { 0, 0 },
    { 0x38f497a0, 90 },
    { 0x2e497a1, 33 },
    { 0xfcb497a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429497a7, 95 },
    { 0xaed497a8, 33 },
    { 0, 0 },
    { 0x570497aa, 33 },
    { 0x29c497ab, 99 },
    { 0xa80497aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa497b1, 65 },
    { 0, 0 },
    { 0x6e2497b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153497b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7497bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da497c4, 90 },
    { 0x82d497c5, 85 },
    { 0x137497c6, 34 },
    { 0x965497c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd497d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef497d7, 55 },
    { 0x91d497d8, 99 },
    { 0x43c497d9, 46 },
    { 0x6ad497d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63497e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6497e3, 33 },
    { 0, 0 },
    { 0x1b497e5, 23 },
    { 0, 0 },
    { 0x729497e7, 76 },
    { 0, 0 },
    { 0xc9497e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6497ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3497f1, 87 },
    { 0, 0 },
    { 0xd1a497f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104497fb, 78 },
    { 0x1af497fc, 61 },
    { 0x958497fd, 98 },
    { 0x677497fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa49802, 90 },
    { 0xcab49803, 99 },
    { 0, 0 },
    { 0xe6749805, 33 },
    { 0x8e449806, 98 },
    { 0x73549807, 55 },
    { 0xcfb49808, 33 },
    { 0x89b49805, 23 },
    { 0, 0 },
    { 0x1fe4980b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824980f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9449815, 23 },
    { 0xcdf49815, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28249820, 67 },
    { 0xb9549820, 23 },
    { 0xe1249821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28049826, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1949829, 91 },
    { 0x71e49829, 95 },
    { 0xdd74982b, 77 },
    { 0xc7649829, 80 },
    { 0x8414982d, 55 },
    { 0x9084982d, 34 },
    { 0xcfc4982d, 95 },
    { 0x5404982d, 43 },
    { 0xe8549829, 98 },
    { 0xf0e49832, 81 },
    { 0xcb049832, 23 },
    { 0x7894982b, 33 },
    { 0xc154982b, 76 },
    { 0x7dd4982d, 33 },
    { 0x59b4982f, 90 },
    { 0, 0 },
    { 0xe3449839, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b84983c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49843, 90 },
    { 0xf8f49844, 34 },
    { 0xc1d49845, 78 },
    { 0x70949845, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4984b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f149852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f64985b, 91 },
    { 0x3d94985c, 52 },
    { 0x7744985d, 98 },
    { 0x8784985d, 23 },
    { 0, 0 },
    { 0xa9c49860, 98 },
    { 0xb949861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b249868, 61 },
    { 0x49149869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2904986d, 33 },
    { 0, 0 },
    { 0x7064986f, 38 },
    { 0x91e49870, 61 },
    { 0x1e649871, 61 },
    { 0, 0 },
    { 0x5b849873, 98 },
    { 0, 0 },
    { 0xd0c49875, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4987a, 98 },
    { 0, 0 },
    { 0xd854987c, 61 },
    { 0x76d4987d, 34 },
    { 0x3ff4987e, 61 },
    { 0xc04987d, 99 },
    { 0x81249880, 33 },
    { 0xda149881, 84 },
    { 0x52d49882, 52 },
    { 0x45649883, 27 },
    { 0x7304987e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1949889, 61 },
    { 0, 0 },
    { 0x9b4988b, 76 },
    { 0, 0 },
    { 0x1044988d, 38 },
    { 0x8d94988d, 66 },
    { 0x19a4988f, 91 },
    { 0xc0149890, 77 },
    { 0xcd449890, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49894, 61 },
    { 0x73049895, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef49898, 95 },
    { 0x8d049899, 23 },
    { 0x1e24989a, 33 },
    { 0xdea49899, 34 },
    { 0xa4989b, 95 },
    { 0x1164989d, 55 },
    { 0, 0 },
    { 0xe894989f, 98 },
    { 0xf3a498a0, 85 },
    { 0xd4a498a0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb498a5, 76 },
    { 0, 0 },
    { 0x71a498a7, 43 },
    { 0, 0 },
    { 0x737498a9, 33 },
    { 0, 0 },
    { 0xa63498ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda498b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4498b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d498be, 46 },
    { 0x3fe498bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655498c5, 69 },
    { 0, 0 },
    { 0xb50498c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949498d0, 95 },
    { 0x55f498d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6498d4, 99 },
    { 0, 0 },
    { 0xa55498d6, 90 },
    { 0xbd2498d7, 33 },
    { 0xa38498d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40498db, 34 },
    { 0x27a498db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x891498df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a498e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74498e8, 23 },
    { 0x9b6498e9, 80 },
    { 0, 0 },
    { 0x4b4498eb, 91 },
    { 0x841498ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366498f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50498f4, 87 },
    { 0xe5d498f5, 100 },
    { 0x907498f5, 55 },
    { 0xc5498f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35749900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14549906, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85149909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d4990c, 33 },
    { 0x4604990d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52349911, 46 },
    { 0x76e49912, 33 },
    { 0, 0 },
    { 0xbfc49914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c49918, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a49920, 33 },
    { 0xa1749920, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c49926, 95 },
    { 0xc8849927, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb74992a, 33 },
    { 0, 0 },
    { 0x61d4992c, 43 },
    { 0, 0 },
    { 0xb044992e, 23 },
    { 0x5b34992e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d49933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4993d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3049941, 61 },
    { 0x22b49942, 76 },
    { 0x88749943, 31 },
    { 0x7ab49944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a49947, 43 },
    { 0xf9549948, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b4994d, 91 },
    { 0x57f4994e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9749951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a49955, 33 },
    { 0xb249956, 55 },
    { 0xb3849957, 98 },
    { 0xc9849956, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14995d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d49960, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64549963, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x65449966, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4249969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af4996f, 52 },
    { 0, 0 },
    { 0x86749971, 66 },
    { 0, 0 },
    { 0x75049973, 85 },
    { 0x54349973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9224997a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7149980, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d49983, 91 },
    { 0xe9b49984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b849987, 61 },
    { 0x74c49988, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b44998c, 61 },
    { 0x4244998c, 34 },
    { 0xe54998d, 85 },
    { 0x5d14998f, 93 },
    { 0xa834998c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed49993, 65 },
    { 0x8e249994, 98 },
    { 0xf2d49993, 76 },
    { 0x60749996, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8614999a, 61 },
    { 0xdff4999b, 76 },
    { 0xf1d4999c, 90 },
    { 0, 0 },
    { 0xfce4999e, 52 },
    { 0x7cb4999e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939499a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff499ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e499b1, 100 },
    { 0x64f499b1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602499b6, 33 },
    { 0x1b7499b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9499be, 61 },
    { 0, 0 },
    { 0xe3a499c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f499c7, 55 },
    { 0x3d0499c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd499cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135499cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a499d7, 43 },
    { 0x58b499d8, 96 },
    { 0x5c499d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04499de, 61 },
    { 0, 0 },
    { 0x1a499e0, 91 },
    { 0x32f499e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c499e6, 55 },
    { 0, 0 },
    { 0x344499e8, 61 },
    { 0xd0e499e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68499ed, 95 },
    { 0x1c5499ee, 99 },
    { 0, 0 },
    { 0xdf1499f0, 95 },
    { 0, 0 },
    { 0xe76499f2, 90 },
    { 0, 0 },
    { 0x7e4499f4, 61 },
    { 0, 0 },
    { 0xce4499f6, 55 },
    { 0x72c499f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51499fb, 98 },
    { 0xaf3499fb, 61 },
    { 0x3e8499fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649a01, 2 },
    { 0, 0 },
    { 0x16c49a03, 23 },
    { 0x67649a04, 55 },
    { 0xb9749a05, 46 },
    { 0x2bb49a06, 90 },
    { 0x53549a07, 90 },
    { 0xf2949a04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b149a0c, 98 },
    { 0, 0 },
    { 0xdc649a0e, 76 },
    { 0xb6c49a0e, 33 },
    { 0x7c349a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f649a14, 81 },
    { 0, 0 },
    { 0xaba49a16, 31 },
    { 0x5d749a16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b449a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a449a21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75249a26, 98 },
    { 0x2d349a27, 93 },
    { 0xad849a28, 95 },
    { 0x6e549a29, 100 },
    { 0, 0 },
    { 0x68f49a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b49a2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc849a39, 68 },
    { 0x76049a39, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf049a3e, 98 },
    { 0xa0d49a3f, 90 },
    { 0x16e49a40, 78 },
    { 0, 0 },
    { 0x3749a42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f49a47, 61 },
    { 0x48f49a48, 99 },
    { 0, 0 },
    { 0xb9049a4a, 98 },
    { 0x3ad49a4b, 36 },
    { 0x3d249a4b, 33 },
    { 0xb4549a4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b49a51, 95 },
    { 0, 0 },
    { 0x19749a53, 66 },
    { 0x62949a54, 95 },
    { 0x68949a55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4149a58, 9 },
    { 0xfb649a59, 95 },
    { 0x68249a5a, 23 },
    { 0x9c149a58, 98 },
    { 0, 0 },
    { 0x1ac49a5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c49a61, 90 },
    { 0xdf449a62, 76 },
    { 0x7749a63, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3149a69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d49a6e, 43 },
    { 0x52e49a6f, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a449a73, 90 },
    { 0x16349a74, 95 },
    { 0x9a349a74, 93 },
    { 0xa9949a74, 84 },
    { 0, 0 },
    { 0xcae49a78, 55 },
    { 0x3da49a79, 64 },
    { 0xd7749a78, 81 },
    { 0x50949a79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a80, 98 },
    { 0, 0 },
    { 0x59549a82, 95 },
    { 0x8b749a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b49a87, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75049a8f, 33 },
    { 0x93649a90, 61 },
    { 0x7af49a91, 55 },
    { 0xf2149a92, 34 },
    { 0xd6949a93, 68 },
    { 0x26a49a94, 38 },
    { 0x14a49a90, 93 },
    { 0xd8549a96, 99 },
    { 0xc8249a95, 34 },
    { 0x50c49a97, 90 },
    { 0xdf849a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c49a9e, 61 },
    { 0, 0 },
    { 0x78949aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a49aa3, 90 },
    { 0, 0 },
    { 0xb0d49aa5, 91 },
    { 0xd5549aa6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x50149aa9, 76 },
    { 0x6a249aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e049aae, 34 },
    { 0, 0 },
    { 0x2e949ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e49ab3, 95 },
    { 0x5b549ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a49aba, 67 },
    { 0xb2249abb, 99 },
    { 0x7349abc, 61 },
    { 0xd6b49abc, 90 },
    { 0x4749abe, 97 },
    { 0, 0 },
    { 0xfcc49ac0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f49ac7, 33 },
    { 0x5e849ac8, 33 },
    { 0xadb49ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e49ad2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a249ad8, 34 },
    { 0xd9849ad8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca649ae2, 76 },
    { 0, 0 },
    { 0xd7549ae4, 61 },
    { 0, 0 },
    { 0xd7c49ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28549ae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19849af3, 85 },
    { 0x60349af3, 98 },
    { 0xddf49af5, 17 },
    { 0xb2749af5, 95 },
    { 0xbd249af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b49afe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7449b08, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb49b0c, 33 },
    { 0x15e49b0c, 91 },
    { 0xe5d49b0e, 90 },
    { 0x73849b0e, 91 },
    { 0xbc749b10, 55 },
    { 0x2ef49b11, 33 },
    { 0x58f49b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37849b15, 76 },
    { 0x1ae49b16, 55 },
    { 0, 0 },
    { 0x91749b18, 99 },
    { 0xe7549b19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5949b1d, 95 },
    { 0xf2a49b1e, 90 },
    { 0, 0 },
    { 0x58849b20, 76 },
    { 0x26d49b20, 55 },
    { 0, 0 },
    { 0x7bf49b23, 65 },
    { 0x4d449b24, 23 },
    { 0x98e49b24, 33 },
    { 0x26649b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19649b29, 100 },
    { 0xd5249b2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f449b2d, 87 },
    { 0, 0 },
    { 0x4ac49b2f, 52 },
    { 0xa3549b30, 98 },
    { 0xadc49b2f, 23 },
    { 0xd2c49b2f, 98 },
    { 0xd3649b33, 68 },
    { 0x5e949b31, 43 },
    { 0, 0 },
    { 0xec249b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83049b39, 98 },
    { 0x92649b3a, 90 },
    { 0x36349b3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc849b40, 90 },
    { 0x5ec49b40, 46 },
    { 0x2249b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a49b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62349b49, 78 },
    { 0x5ee49b4a, 95 },
    { 0x4449b49, 87 },
    { 0xf1449b4b, 22 },
    { 0x6d249b4d, 77 },
    { 0x8d949b4e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9449b54, 33 },
    { 0, 0 },
    { 0x66f49b56, 55 },
    { 0x3df49b57, 90 },
    { 0x2c249b57, 98 },
    { 0x4e49b56, 33 },
    { 0x1cf49b57, 91 },
    { 0x7e649b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83649b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1049b64, 61 },
    { 0x1b749b64, 98 },
    { 0x1ce49b65, 23 },
    { 0xeda49b67, 43 },
    { 0x94949b68, 90 },
    { 0x4a349b66, 23 },
    { 0, 0 },
    { 0xdeb49b6b, 90 },
    { 0, 0 },
    { 0x84f49b6d, 67 },
    { 0, 0 },
    { 0x59849b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f149b74, 99 },
    { 0, 0 },
    { 0x70e49b76, 77 },
    { 0, 0 },
    { 0xd049b78, 98 },
    { 0x1e649b78, 87 },
    { 0, 0 },
    { 0xdd49b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd49b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a749b84, 98 },
    { 0, 0 },
    { 0x92649b86, 98 },
    { 0x15949b87, 33 },
    { 0x22049b88, 23 },
    { 0xfd049b89, 90 },
    { 0x95e49b89, 33 },
    { 0xc8e49b8a, 23 },
    { 0x7a349b87, 61 },
    { 0x14e49b8d, 36 },
    { 0x9d949b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d49b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5249b99, 78 },
    { 0x18849b9a, 84 },
    { 0x3a349b9b, 76 },
    { 0x3e249b9c, 67 },
    { 0x99d49b9a, 98 },
    { 0, 0 },
    { 0x4e349b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3049bae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66149bb5, 46 },
    { 0x20349bb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a49bba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f49bbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e349bc0, 43 },
    { 0x41049bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1649bcb, 43 },
    { 0xb1849bcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57349bcf, 93 },
    { 0x14a49bcf, 33 },
    { 0x8649bd1, 95 },
    { 0x8a849bd2, 61 },
    { 0x3bb49bd3, 90 },
    { 0x4ea49bd1, 61 },
    { 0, 0 },
    { 0x449bd6, 61 },
    { 0xf8e49bd6, 91 },
    { 0x61b49bd6, 61 },
    { 0, 0 },
    { 0xfdb49bda, 36 },
    { 0, 0 },
    { 0xaf249bdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea949be1, 90 },
    { 0, 0 },
    { 0x2c849be3, 67 },
    { 0xd3349be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55849be7, 46 },
    { 0xa6f49be8, 33 },
    { 0xa9a49be7, 66 },
    { 0x54949bea, 61 },
    { 0, 0 },
    { 0x54949bec, 17 },
    { 0x5ff49bed, 87 },
    { 0xd249bec, 34 },
    { 0x2fd49bed, 17 },
    { 0, 0 },
    { 0xea749bf1, 65 },
    { 0xfc549bf1, 23 },
    { 0x1e749bf1, 34 },
    { 0x5fd49bf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a49bf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71049bfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4249bff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac49c02, 100 },
    { 0xd6649c03, 76 },
    { 0, 0 },
    { 0xbdf49c05, 76 },
    { 0xdaa49c06, 23 },
    { 0x5f449c07, 95 },
    { 0, 0 },
    { 0x9e249c09, 84 },
    { 0xea549c09, 84 },
    { 0xd8549c0b, 33 },
    { 0xbff49c0a, 76 },
    { 0, 0 },
    { 0xb749c0e, 61 },
    { 0x85349c0e, 90 },
    { 0x449c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35849c13, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3349c16, 98 },
    { 0xbf049c16, 34 },
    { 0, 0 },
    { 0xab549c19, 78 },
    { 0x3cb49c19, 23 },
    { 0x61d49c1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d49c1f, 76 },
    { 0x5ba49c20, 100 },
    { 0xddf49c21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75949c29, 46 },
    { 0x75c49c2a, 90 },
    { 0, 0 },
    { 0x3d949c2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5249c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f149c36, 33 },
    { 0xe9849c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87649c3b, 55 },
    { 0, 0 },
    { 0xc4949c3d, 33 },
    { 0xc1949c3e, 90 },
    { 0x9849c3e, 80 },
    { 0x13e49c40, 77 },
    { 0x33749c41, 66 },
    { 0xceb49c42, 53 },
    { 0x6c549c43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf49c47, 33 },
    { 0x3b49c47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6849c4b, 90 },
    { 0x45f49c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a749c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73449c56, 23 },
    { 0x65949c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec249c5a, 18 },
    { 0, 0 },
    { 0xa9749c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6249c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e49c6c, 33 },
    { 0xf3349c6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a49c74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b49c7b, 98 },
    { 0x94e49c7b, 90 },
    { 0x2e549c7d, 90 },
    { 0, 0 },
    { 0x6e749c7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24149c82, 43 },
    { 0, 0 },
    { 0xe5249c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0449c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e49c8c, 95 },
    { 0x3f349c8c, 46 },
    { 0, 0 },
    { 0x99049c8f, 98 },
    { 0x98049c90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc049c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa449c97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8549c9a, 100 },
    { 0xf3849c9b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28249ca0, 55 },
    { 0xc9a49ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba49ca4, 98 },
    { 0xbb649ca4, 61 },
    { 0x2fb49ca5, 90 },
    { 0x7dd49ca4, 23 },
    { 0x37d49ca7, 46 },
    { 0xc3849ca9, 75 },
    { 0xf9b49caa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f49caf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd49cb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d49cb6, 90 },
    { 0xaaf49cb7, 90 },
    { 0x2df49cb8, 23 },
    { 0xbb049cb9, 61 },
    { 0x52c49cba, 33 },
    { 0xe0b49cbb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d649cc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e49cc6, 33 },
    { 0, 0 },
    { 0xa1149cc8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x86349ccb, 100 },
    { 0, 0 },
    { 0x48f49ccd, 40 },
    { 0x6c649cce, 90 },
    { 0x68649ccf, 42 },
    { 0xec949cd0, 46 },
    { 0x36649cd0, 55 },
    { 0x57549cd1, 76 },
    { 0x59349cd1, 90 },
    { 0, 0 },
    { 0xbdd49cd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0049cd8, 78 },
    { 0x46e49cd9, 55 },
    { 0, 0 },
    { 0x1c549cdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88649cef, 8 },
    { 0x15e49cf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c449cf4, 84 },
    { 0x92b49cf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92149cfb, 95 },
    { 0x2e049cfc, 33 },
    { 0x59e49cfd, 76 },
    { 0x449cfc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b549d03, 55 },
    { 0xcf349d04, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb149d09, 34 },
    { 0, 0 },
    { 0xa2649d0b, 23 },
    { 0x7b149d0b, 95 },
    { 0xd7949d0d, 48 },
    { 0x9b449d0e, 61 },
    { 0x91d49d0f, 56 },
    { 0, 0 },
    { 0xd4f49d11, 66 },
    { 0xd9949d12, 61 },
    { 0xc6249d11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9249d20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7149d23, 90 },
    { 0, 0 },
    { 0x5f149d25, 90 },
    { 0x28949d26, 23 },
    { 0x3a149d27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0349d2b, 33 },
    { 0x66049d2b, 23 },
    { 0x26649d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96849d35, 95 },
    { 0, 0 },
    { 0xc3749d37, 43 },
    { 0xef349d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb49d40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5249d43, 90 },
    { 0xf3849d44, 98 },
    { 0xff349d44, 95 },
    { 0xb0249d46, 83 },
    { 0x1ff49d47, 98 },
    { 0x6a449d48, 33 },
    { 0x91549d47, 96 },
    { 0x5a549d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31649d51, 91 },
    { 0, 0 },
    { 0x54949d53, 90 },
    { 0x96349d53, 95 },
    { 0x6e149d55, 61 },
    { 0x33049d56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4149d5a, 77 },
    { 0, 0 },
    { 0x1be49d5c, 90 },
    { 0x26149d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7649d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9949d65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9649d69, 90 },
    { 0, 0 },
    { 0x9dd49d6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a549d71, 61 },
    { 0, 0 },
    { 0x9fa49d73, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c49d76, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65649d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f949d81, 82 },
    { 0x5dc49d82, 43 },
    { 0, 0 },
    { 0xad49d84, 85 },
    { 0x99449d85, 55 },
    { 0, 0 },
    { 0xdb049d87, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd349d8c, 92 },
    { 0xbf049d8d, 95 },
    { 0x30f49d8d, 90 },
    { 0x1e349d8f, 23 },
    { 0xa0849d8f, 88 },
    { 0xcec49d8c, 61 },
    { 0x95949d92, 76 },
    { 0, 0 },
    { 0x2c549d94, 87 },
    { 0xfc149d95, 90 },
    { 0x16b49d96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb49d9a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e49d9f, 78 },
    { 0, 0 },
    { 0xe9b49da1, 33 },
    { 0xf9949da2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16049da6, 61 },
    { 0x3ba49da7, 95 },
    { 0x80349da8, 78 },
    { 0x3349da9, 88 },
    { 0x95149daa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e49db1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7949db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73849dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4549dc0, 33 },
    { 0, 0 },
    { 0x3db49dc2, 61 },
    { 0xb7e49dc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc949dc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a49dc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf49dd0, 55 },
    { 0, 0 },
    { 0x44549dd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81349dd9, 95 },
    { 0, 0 },
    { 0x21049ddb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b49de0, 34 },
    { 0xf6b49de0, 61 },
    { 0x9d849de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf49de7, 66 },
    { 0, 0 },
    { 0x73249de9, 91 },
    { 0, 0 },
    { 0xb9d49deb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed49def, 67 },
    { 0, 0 },
    { 0xf0149df1, 93 },
    { 0, 0 },
    { 0x40f49df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d49df6, 23 },
    { 0x1f749df6, 33 },
    { 0, 0 },
    { 0xaaa49df9, 61 },
    { 0x12649dfa, 66 },
    { 0xbd649dfb, 61 },
    { 0, 0 },
    { 0x9ed49dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd149e00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba49e04, 78 },
    { 0x16649e05, 98 },
    { 0x86a49e06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc49e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5449e0e, 55 },
    { 0, 0 },
    { 0x35249e10, 66 },
    { 0, 0 },
    { 0x55049e12, 61 },
    { 0x62b49e12, 98 },
    { 0xef649e14, 90 },
    { 0x9a549e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7649e1a, 37 },
    { 0x17849e1b, 90 },
    { 0, 0 },
    { 0x22949e1d, 67 },
    { 0, 0 },
    { 0x7aa49e1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b049e25, 90 },
    { 0xc7e49e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b49e29, 68 },
    { 0x89b49e2a, 61 },
    { 0x85649e2a, 77 },
    { 0, 0 },
    { 0xae549e2d, 61 },
    { 0xb5749e2e, 90 },
    { 0x61949e2e, 61 },
    { 0x42049e2e, 98 },
    { 0x3a949e2d, 33 },
    { 0x9ae49e30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a49e37, 61 },
    { 0x9c249e38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549e3b, 94 },
    { 0xa4f49e3b, 98 },
    { 0x90e49e3d, 90 },
    { 0, 0 },
    { 0xdda49e3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d49e42, 43 },
    { 0xc4749e42, 90 },
    { 0x66649e43, 61 },
    { 0xf6449e43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd49e49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049e54, 87 },
    { 0x8ce49e54, 52 },
    { 0xb0449e55, 33 },
    { 0x46b49e57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78049e5b, 76 },
    { 0, 0 },
    { 0x5549e5d, 95 },
    { 0, 0 },
    { 0x7b749e5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a49e63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2749e68, 43 },
    { 0x34949e69, 90 },
    { 0xab49e6a, 98 },
    { 0x7ae49e68, 61 },
    { 0x9a349e6c, 96 },
    { 0x93f49e69, 38 },
    { 0x60d49e6d, 99 },
    { 0, 0 },
    { 0xa3549e70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd049e79, 46 },
    { 0xc6849e7a, 33 },
    { 0x71649e79, 98 },
    { 0x42c49e7c, 85 },
    { 0xd8849e7d, 28 },
    { 0x18749e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a49e84, 46 },
    { 0xcbb49e84, 46 },
    { 0, 0 },
    { 0x3bd49e87, 93 },
    { 0, 0 },
    { 0x88849e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f749e8f, 55 },
    { 0, 0 },
    { 0xcc149e91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b049e95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb649e98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac49e9b, 46 },
    { 0xdc349e9c, 95 },
    { 0xaec49e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d049ea0, 33 },
    { 0x3d49ea0, 95 },
    { 0xce249ea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9149ea7, 68 },
    { 0x6a849ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab749eab, 90 },
    { 0xc2549eac, 90 },
    { 0xb4d49eac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1849eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b49eb4, 98 },
    { 0x3bb49eb5, 34 },
    { 0x86049eb6, 95 },
    { 0xd3c49eb6, 43 },
    { 0xd7849eb8, 76 },
    { 0x21549eb7, 34 },
    { 0, 0 },
    { 0xf8049ebb, 33 },
    { 0x79949ebc, 85 },
    { 0xf6849ebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47149ec4, 55 },
    { 0xa2049ec5, 47 },
    { 0xc8149ec6, 90 },
    { 0xf1c49ec6, 98 },
    { 0x50d49ec8, 17 },
    { 0x82149ec8, 61 },
    { 0x78f49ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3149ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb49ed1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e49ed4, 33 },
    { 0xa9d49ed5, 34 },
    { 0xf4c49ed4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649edf, 96 },
    { 0x75f49ee0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a49ee5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d49ee8, 61 },
    { 0xb1249ee8, 98 },
    { 0xabe49eea, 34 },
    { 0xe6849eea, 100 },
    { 0x84649eec, 98 },
    { 0x29b49eed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95249ef7, 55 },
    { 0, 0 },
    { 0xd2649ef9, 33 },
    { 0xb7149efa, 33 },
    { 0xb6249efb, 17 },
    { 0x70849efc, 33 },
    { 0x20e49efd, 88 },
    { 0, 0 },
    { 0xc5049eff, 91 },
    { 0xaac49f00, 52 },
    { 0xd5949f01, 33 },
    { 0xbfb49f02, 46 },
    { 0xd1c49f02, 90 },
    { 0x33349f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549f0a, 61 },
    { 0xec449f0a, 95 },
    { 0x55749f0c, 98 },
    { 0x3cd49f0d, 90 },
    { 0x4b049f0b, 62 },
    { 0x3fd49f0b, 90 },
    { 0xabd49f10, 34 },
    { 0x6e849f0f, 34 },
    { 0x9e649f0b, 90 },
    { 0x18349f13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b549f17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f49f1b, 88 },
    { 0x88949f1c, 90 },
    { 0, 0 },
    { 0x61049f1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a49f22, 55 },
    { 0xb8649f23, 81 },
    { 0, 0 },
    { 0x3af49f25, 90 },
    { 0xfa849f26, 33 },
    { 0, 0 },
    { 0x42c49f28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70949f30, 84 },
    { 0, 0 },
    { 0x31349f32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c49f37, 61 },
    { 0x14d49f38, 90 },
    { 0x43049f39, 78 },
    { 0x78049f39, 23 },
    { 0x7ad49f3a, 34 },
    { 0, 0 },
    { 0x58549f3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5349f42, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f049f46, 33 },
    { 0xe8249f46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb449f4c, 38 },
    { 0xc249f4c, 95 },
    { 0xa1d49f4e, 90 },
    { 0x9c549f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19449f52, 95 },
    { 0x8f849f53, 33 },
    { 0xe0549f52, 84 },
    { 0, 0 },
    { 0x90c49f56, 34 },
    { 0x82549f57, 33 },
    { 0x61349f56, 34 },
    { 0x5d749f58, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14749f5d, 46 },
    { 0x8f649f5d, 81 },
    { 0xd7449f5f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5249f64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66249f67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38749f6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d649f70, 23 },
    { 0, 0 },
    { 0xd2149f72, 98 },
    { 0xafc49f72, 46 },
    { 0xd2749f73, 23 },
    { 0x44c49f75, 87 },
    { 0x64949f75, 90 },
    { 0x36649f75, 34 },
    { 0xe6649f78, 23 },
    { 0x7a749f78, 36 },
    { 0xe7049f7a, 51 },
    { 0xd149f75, 98 },
    { 0x9c249f72, 55 },
    { 0xf7549f7a, 90 },
    { 0xbb549f7e, 34 },
    { 0, 0 },
    { 0x6af49f80, 95 },
    { 0xf7449f81, 23 },
    { 0, 0 },
    { 0xd8149f83, 76 },
    { 0xce49f84, 90 },
    { 0x11e49f84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a249f88, 100 },
    { 0xe8649f88, 34 },
    { 0x35249f8a, 23 },
    { 0x75049f8b, 98 },
    { 0x94a49f89, 98 },
    { 0, 0 },
    { 0x8d249f8e, 33 },
    { 0x97249f8f, 66 },
    { 0x3fc49f8f, 95 },
    { 0x68349f91, 23 },
    { 0xed249f91, 90 },
    { 0x6e349f92, 98 },
    { 0x4ae49f94, 61 },
    { 0xd5a49f95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6949f9c, 90 },
    { 0x15949f9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5949fa0, 61 },
    { 0x40e49fa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be49fa4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc49fac, 40 },
    { 0x38c49fad, 90 },
    { 0x19349fae, 98 },
    { 0xa6b49fae, 33 },
    { 0x12e49fb0, 98 },
    { 0xdf349fb0, 76 },
    { 0, 0 },
    { 0x84949fb3, 18 },
    { 0xa6c49fb4, 80 },
    { 0, 0 },
    { 0x26349fb6, 76 },
    { 0x95849fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32549fba, 61 },
    { 0x7e549fbb, 81 },
    { 0x7c449fbb, 23 },
    { 0x3e549fbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a49fc0, 90 },
    { 0xc7b49fc0, 46 },
    { 0x89b49fc1, 34 },
    { 0x47549fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83649fc9, 61 },
    { 0xe0f49fca, 93 },
    { 0xa2049fca, 76 },
    { 0xa1a49fcc, 78 },
    { 0, 0 },
    { 0x5d449fce, 34 },
    { 0x3cf49fce, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f49fd4, 33 },
    { 0x77849fd5, 90 },
    { 0x9749fd6, 98 },
    { 0x8b049fd6, 95 },
    { 0x5aa49fd7, 46 },
    { 0, 0 },
    { 0x77b49fda, 93 },
    { 0x88749fdb, 90 },
    { 0x21849fda, 76 },
    { 0x6ba49fdb, 55 },
    { 0xa7549fde, 34 },
    { 0, 0 },
    { 0xaa049fe0, 61 },
    { 0x62449fe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46949fe5, 85 },
    { 0, 0 },
    { 0x6dc49fe7, 61 },
    { 0xdfc49fe7, 33 },
    { 0x69549fe7, 98 },
    { 0x2949fe8, 76 },
    { 0x50149feb, 55 },
    { 0x14749fea, 95 },
    { 0, 0 },
    { 0x9449fee, 33 },
    { 0xad949fef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f49ff5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9949ffa, 33 },
    { 0, 0 },
    { 0xd0349ffc, 61 },
    { 0x4ad49ffd, 34 },
    { 0, 0 },
    { 0xec949fff, 95 },
    { 0x5fa4a000, 76 },
    { 0xa2c49fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b04a00c, 53 },
    { 0x76d4a00d, 90 },
    { 0x50f4a00d, 61 },
    { 0x3614a00f, 66 },
    { 0xa5a4a00f, 98 },
    { 0, 0 },
    { 0xa814a012, 53 },
    { 0, 0 },
    { 0xe924a014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb54a017, 61 },
    { 0x5984a017, 33 },
    { 0, 0 },
    { 0xb504a01a, 43 },
    { 0x4964a01b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e4a01e, 33 },
    { 0x1584a01e, 52 },
    { 0xe7a4a020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd694a024, 52 },
    { 0x21d4a025, 93 },
    { 0x4884a026, 65 },
    { 0, 0 },
    { 0xc5d4a028, 34 },
    { 0x90e4a028, 76 },
    { 0xc904a028, 23 },
    { 0, 0 },
    { 0xe924a02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4a033, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed4a038, 98 },
    { 0, 0 },
    { 0x3ad4a03a, 91 },
    { 0, 0 },
    { 0x174a03c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94a040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a44a043, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x774a046, 23 },
    { 0x6a94a047, 43 },
    { 0, 0 },
    { 0x1a4a049, 75 },
    { 0x8e64a04a, 68 },
    { 0x7fa4a04b, 34 },
    { 0xe554a04c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574a04f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f04a052, 93 },
    { 0x9344a053, 23 },
    { 0xddd4a054, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9314a057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4a060, 66 },
    { 0xd44a060, 34 },
    { 0x4104a061, 90 },
    { 0x82d4a063, 90 },
    { 0x8714a064, 90 },
    { 0, 0 },
    { 0xdfb4a066, 43 },
    { 0x1214a067, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c54a06f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf764a072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14a081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb374a084, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac64a08a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764a08d, 93 },
    { 0xa814a08d, 33 },
    { 0xa5a4a08f, 67 },
    { 0, 0 },
    { 0xf0c4a091, 53 },
    { 0, 0 },
    { 0x60c4a093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d34a098, 90 },
    { 0, 0 },
    { 0x2604a09a, 61 },
    { 0x2434a09a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b4a09f, 93 },
    { 0x1244a0a0, 33 },
    { 0x6734a09f, 90 },
    { 0, 0 },
    { 0xc4c4a0a3, 98 },
    { 0xa144a0a4, 90 },
    { 0xc704a0a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f4a0b7, 34 },
    { 0x3e54a0b7, 17 },
    { 0, 0 },
    { 0xc624a0ba, 95 },
    { 0x2264a0bb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354a0c1, 76 },
    { 0x11b4a0c2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1904a0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9484a0cb, 46 },
    { 0x2064a0cc, 95 },
    { 0x1f14a0cc, 90 },
    { 0xb394a0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4a0d4, 88 },
    { 0x2144a0d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d4a0dc, 23 },
    { 0, 0 },
    { 0x10c4a0de, 38 },
    { 0x5654a0df, 90 },
    { 0x784a0de, 48 },
    { 0x72c4a0e1, 90 },
    { 0, 0 },
    { 0x4714a0e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4a0e6, 52 },
    { 0x42c4a0e7, 91 },
    { 0x47b4a0e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd494a0ef, 34 },
    { 0xc84a0f0, 33 },
    { 0x4344a0f1, 98 },
    { 0xb9e4a0f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6024a0f5, 98 },
    { 0xd414a0f6, 98 },
    { 0, 0 },
    { 0x3744a0f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb4a0fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34a106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84a10b, 88 },
    { 0xbba4a10b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc24a111, 76 },
    { 0x1ec4a111, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec74a117, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04a11c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1744a122, 95 },
    { 0, 0 },
    { 0x3da4a124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794a12b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8004a12f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4a132, 98 },
    { 0x1704a132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac4a137, 91 },
    { 0, 0 },
    { 0xe854a139, 46 },
    { 0x5c94a139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b74a13d, 75 },
    { 0, 0 },
    { 0x6554a13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a4a142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba84a145, 53 },
    { 0xe0b4a146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9944a152, 34 },
    { 0x4f54a152, 90 },
    { 0, 0 },
    { 0xdfd4a155, 95 },
    { 0x9fb4a156, 61 },
    { 0, 0 },
    { 0x53e4a158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f4a15c, 95 },
    { 0xc094a15d, 93 },
    { 0xde4a15d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee54a163, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd744a168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd64a16e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3034a173, 23 },
    { 0, 0 },
    { 0xf984a175, 81 },
    { 0x7334a176, 52 },
    { 0x5d04a177, 64 },
    { 0x4924a177, 55 },
    { 0x1f74a176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf4a17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd684a182, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4a185, 90 },
    { 0x7ba4a186, 91 },
    { 0xc724a186, 33 },
    { 0x55f4a188, 93 },
    { 0x95a4a189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb494a18d, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xad84a190, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4a198, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f4a19f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc214a1a2, 55 },
    { 0, 0 },
    { 0xb04a1a4, 95 },
    { 0x6fe4a1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654a1b3, 93 },
    { 0x1c14a1b3, 55 },
    { 0x98d4a1b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f4a1ba, 52 },
    { 0xb444a1bb, 90 },
    { 0x1434a1bb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd754a1bf, 52 },
    { 0x4124a1c0, 95 },
    { 0x5344a1c0, 33 },
    { 0xb6e4a1c2, 67 },
    { 0xe264a1c0, 84 },
    { 0xd354a1c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe4a1c8, 76 },
    { 0x6294a1c9, 76 },
    { 0xdfc4a1c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a54a1cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db4a1d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b4a1d5, 84 },
    { 0xe734a1d6, 64 },
    { 0, 0 },
    { 0x6384a1d8, 46 },
    { 0, 0 },
    { 0x8154a1da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394a1dd, 84 },
    { 0x794a1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1264a1e2, 90 },
    { 0, 0 },
    { 0xd914a1e4, 77 },
    { 0x48a4a1e5, 61 },
    { 0x5ce4a1e6, 55 },
    { 0x3854a1e7, 33 },
    { 0x7f54a1e6, 90 },
    { 0x2ea4a1e9, 55 },
    { 0x28e4a1e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4a1ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c4a1f5, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4a1fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c34a201, 87 },
    { 0xe54a201, 85 },
    { 0, 0 },
    { 0x98d4a204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e54a20a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4a213, 23 },
    { 0xa1c4a213, 33 },
    { 0, 0 },
    { 0x7b04a216, 56 },
    { 0x76d4a217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8114a21d, 77 },
    { 0x2ad4a21e, 46 },
    { 0x6924a21f, 34 },
    { 0x5104a21f, 34 },
    { 0x4c84a21e, 55 },
    { 0x8974a222, 61 },
    { 0x5764a223, 95 },
    { 0x574a224, 52 },
    { 0x8f94a222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c4a228, 61 },
    { 0x5c84a228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f14a22d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1314a231, 34 },
    { 0x134a232, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554a235, 23 },
    { 0xb4d4a235, 61 },
    { 0, 0 },
    { 0x1144a238, 87 },
    { 0x2654a239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc064a23d, 66 },
    { 0, 0 },
    { 0xb744a23f, 89 },
    { 0xee44a240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2574a243, 64 },
    { 0, 0 },
    { 0x2e54a245, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474a248, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94a24b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1364a24f, 99 },
    { 0xd0a4a24f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74a255, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec4a260, 90 },
    { 0x9eb4a260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a265, 23 },
    { 0xbc14a266, 90 },
    { 0x46b4a266, 76 },
    { 0xa094a265, 90 },
    { 0xa544a269, 46 },
    { 0xf904a269, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5084a26f, 76 },
    { 0x72c4a270, 61 },
    { 0, 0 },
    { 0x5324a272, 43 },
    { 0, 0 },
    { 0xf614a274, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8154a278, 90 },
    { 0xf444a279, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c24a27d, 90 },
    { 0x19a4a27d, 33 },
    { 0x3064a27f, 90 },
    { 0x6494a280, 23 },
    { 0xc804a281, 66 },
    { 0x8804a282, 84 },
    { 0x8a74a283, 33 },
    { 0x3124a284, 93 },
    { 0xd534a285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d4a288, 101 },
    { 0xa7d4a289, 95 },
    { 0x7a34a288, 81 },
    { 0x5134a28b, 95 },
    { 0x8ba4a28b, 34 },
    { 0xafd4a28c, 34 },
    { 0x9234a28e, 93 },
    { 0x80f4a28f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824a292, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf304a296, 90 },
    { 0, 0 },
    { 0x1414a298, 91 },
    { 0, 0 },
    { 0x5ca4a29a, 78 },
    { 0xf234a29a, 78 },
    { 0, 0 },
    { 0x11c4a29d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074a2a1, 33 },
    { 0, 0 },
    { 0xfc74a2a3, 95 },
    { 0x9394a2a4, 52 },
    { 0xd1f4a2a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14a2aa, 61 },
    { 0x83d4a2aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a4a2b4, 33 },
    { 0x6e4a2b4, 52 },
    { 0x28c4a2b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa44a2b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7804a2be, 85 },
    { 0x9984a2bf, 100 },
    { 0x4e94a2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e4a2c7, 34 },
    { 0xdc94a2c8, 34 },
    { 0x36b4a2c8, 98 },
    { 0xaca4a2c8, 98 },
    { 0, 0 },
    { 0x884a2cc, 61 },
    { 0xffc4a2cd, 23 },
    { 0xd884a2ce, 33 },
    { 0x7864a2cd, 23 },
    { 0x434a2cd, 33 },
    { 0xb014a2d0, 33 },
    { 0xd184a2d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x364a2d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf94a2d8, 93 },
    { 0xbea4a2d9, 96 },
    { 0, 0 },
    { 0x2214a2db, 51 },
    { 0xb934a2db, 33 },
    { 0x6744a2dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2654a2e1, 90 },
    { 0x9dc4a2e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2304a2e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4a2ee, 85 },
    { 0, 0 },
    { 0x67a4a2f0, 98 },
    { 0x1404a2f0, 34 },
    { 0x1f34a2f2, 95 },
    { 0x9484a2f3, 33 },
    { 0, 0 },
    { 0x8834a2f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c64a2f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f14a2ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf564a305, 95 },
    { 0, 0 },
    { 0xd124a307, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a30a, 35 },
    { 0, 0 },
    { 0x50f4a30c, 33 },
    { 0x5b54a30d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9804a310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4a314, 76 },
    { 0xae24a315, 33 },
    { 0xa924a316, 55 },
    { 0xfc64a317, 98 },
    { 0x19a4a318, 90 },
    { 0xd034a318, 33 },
    { 0xa224a31a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea4a31d, 90 },
    { 0xb894a31e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74a324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4a327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e24a32d, 87 },
    { 0, 0 },
    { 0x2664a32f, 98 },
    { 0x2844a330, 33 },
    { 0x2db4a331, 66 },
    { 0xd744a331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2914a336, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c4a339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e4a33d, 92 },
    { 0xe2e4a33e, 85 },
    { 0x8ee4a33f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4a347, 67 },
    { 0, 0 },
    { 0x2ac4a349, 95 },
    { 0x6514a34a, 55 },
    { 0, 0 },
    { 0xbf24a34c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4a350, 78 },
    { 0x2764a351, 67 },
    { 0xd854a350, 98 },
    { 0xb504a353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f4a357, 90 },
    { 0x6394a358, 88 },
    { 0x41b4a358, 90 },
    { 0x3a04a35a, 34 },
    { 0x55f4a35b, 76 },
    { 0xa734a358, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2714a360, 43 },
    { 0x9d34a361, 61 },
    { 0x6d64a360, 76 },
    { 0x3504a362, 81 },
    { 0x1ab4a360, 33 },
    { 0xf444a365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d54a36d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04a371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a4a37a, 33 },
    { 0x3584a37b, 93 },
    { 0xc2a4a37a, 17 },
    { 0xea04a37d, 61 },
    { 0, 0 },
    { 0x9d4a37f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614a383, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034a38a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae4a38d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24a396, 93 },
    { 0, 0 },
    { 0x5b84a398, 51 },
    { 0xdc24a399, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc644a39d, 76 },
    { 0xa8f4a39e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a3a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d4a3a4, 43 },
    { 0xd174a3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4a3ac, 23 },
    { 0x9c84a3ad, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5204a3b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6244a3b9, 95 },
    { 0x8c14a3ba, 33 },
    { 0x8c84a3ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6414a3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e94a3c3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4a3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a3cb, 61 },
    { 0, 0 },
    { 0x3614a3cd, 78 },
    { 0, 0 },
    { 0x1cb4a3cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a3db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454a3de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c4a3e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3074a3eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9304a3ef, 98 },
    { 0xfec4a3f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa4a3f3, 90 },
    { 0x7a64a3f4, 99 },
    { 0xe414a3f5, 33 },
    { 0, 0 },
    { 0xeb4a3f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb4a3fa, 78 },
    { 0, 0 },
    { 0x3fb4a3fc, 33 },
    { 0x2044a3fc, 95 },
    { 0x6e14a3fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384a402, 46 },
    { 0x9f34a402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4a40b, 85 },
    { 0x65e4a40b, 33 },
    { 0x6004a40c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f4a414, 77 },
    { 0x9564a414, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc364a421, 33 },
    { 0x77f4a421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4874a425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14a432, 95 },
    { 0xcc74a433, 95 },
    { 0, 0 },
    { 0x3c34a435, 100 },
    { 0x3a44a436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da4a43b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c4a440, 90 },
    { 0, 0 },
    { 0xc794a442, 46 },
    { 0xaee4a443, 61 },
    { 0, 0 },
    { 0x9e64a445, 61 },
    { 0x65f4a445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4a44a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b24a44d, 98 },
    { 0x21c4a44d, 99 },
    { 0, 0 },
    { 0x7c54a450, 61 },
    { 0x5e24a450, 95 },
    { 0x2684a452, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe64a458, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde54a45b, 98 },
    { 0x27c4a45c, 55 },
    { 0, 0 },
    { 0x2e74a45e, 100 },
    { 0x5a34a45f, 98 },
    { 0x3374a460, 33 },
    { 0x6704a461, 61 },
    { 0, 0 },
    { 0xbcc4a463, 52 },
    { 0x4934a464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff84a467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3124a471, 98 },
    { 0, 0 },
    { 0x1c44a473, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f4a477, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2954a47a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc34a47f, 75 },
    { 0x6394a480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9604a483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4a48a, 17 },
    { 0x3d54a48a, 90 },
    { 0x99d4a48a, 90 },
    { 0xb084a48d, 46 },
    { 0x9614a48d, 61 },
    { 0x34c4a48f, 52 },
    { 0x5174a48f, 100 },
    { 0x7c94a48e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce4a498, 77 },
    { 0x64e4a498, 17 },
    { 0x794a498, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd64a4a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca4a4a3, 33 },
    { 0, 0 },
    { 0x85f4a4a5, 95 },
    { 0, 0 },
    { 0xf3e4a4a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34a4af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa614a4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b84a4b7, 61 },
    { 0x7004a4b7, 23 },
    { 0, 0 },
    { 0x34a4a4ba, 66 },
    { 0xd4a4a4bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84a4ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9014a4d5, 55 },
    { 0x75c4a4d6, 46 },
    { 0, 0 },
    { 0x8ce4a4d8, 66 },
    { 0, 0 },
    { 0xea14a4da, 90 },
    { 0, 0 },
    { 0x8bd4a4dc, 53 },
    { 0, 0 },
    { 0x1104a4de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4884a4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6114a4f0, 61 },
    { 0xa1f4a4f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d4a4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054a4f7, 98 },
    { 0xc44a4f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7424a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb174a4fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b44a502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa4a509, 78 },
    { 0xd4a50a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34a50d, 98 },
    { 0x6314a50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5044a512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1504a51c, 33 },
    { 0x1c64a51c, 101 },
    { 0x6c94a51e, 23 },
    { 0x9874a51f, 17 },
    { 0xef34a51c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9604a523, 33 },
    { 0, 0 },
    { 0x4f44a525, 61 },
    { 0x4564a526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf694a52a, 90 },
    { 0x2154a52a, 67 },
    { 0xecc4a52a, 90 },
    { 0xa014a52d, 98 },
    { 0, 0 },
    { 0xa224a52f, 95 },
    { 0xf184a52f, 76 },
    { 0, 0 },
    { 0xa1d4a532, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb14a535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e34a539, 95 },
    { 0, 0 },
    { 0x76b4a53b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f4a53f, 90 },
    { 0x8214a540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde64a544, 76 },
    { 0, 0 },
    { 0x56d4a546, 95 },
    { 0x90b4a547, 88 },
    { 0x9944a546, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f4a54e, 98 },
    { 0x3f4a54f, 66 },
    { 0x6ac4a550, 70 },
    { 0xddd4a551, 76 },
    { 0x7da4a54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4a558, 93 },
    { 0, 0 },
    { 0x4234a55a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b94a55f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b4a567, 23 },
    { 0x1884a567, 95 },
    { 0x7bf4a569, 100 },
    { 0x7bd4a568, 33 },
    { 0xa954a56a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a56e, 91 },
    { 0x5d54a56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8524a572, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac4a575, 33 },
    { 0x2b54a576, 46 },
    { 0x8844a575, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa4a57b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154a57e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc74a581, 98 },
    { 0xb214a582, 80 },
    { 0x90b4a583, 50 },
    { 0x70a4a584, 100 },
    { 0xf8c4a585, 95 },
    { 0xa774a584, 98 },
    { 0, 0 },
    { 0xebf4a588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d4a58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04a593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3684a59b, 61 },
    { 0xac74a59c, 34 },
    { 0x2bc4a59c, 90 },
    { 0xaec4a59b, 84 },
    { 0x4614a59f, 61 },
    { 0xedf4a5a0, 23 },
    { 0, 0 },
    { 0xbb4a5a2, 90 },
    { 0x1484a5a3, 100 },
    { 0x6c54a5a3, 88 },
    { 0x1a84a5a2, 61 },
    { 0x9b04a5a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a44a5aa, 90 },
    { 0, 0 },
    { 0x6c74a5ac, 33 },
    { 0xe164a5ac, 33 },
    { 0x8984a5ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c34a5b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344a5b6, 33 },
    { 0x7074a5b7, 90 },
    { 0x1ce4a5b7, 95 },
    { 0x7f24a5b8, 66 },
    { 0x2b04a5ba, 95 },
    { 0x37b4a5ba, 65 },
    { 0x9ba4a5bb, 95 },
    { 0x93a4a5bd, 90 },
    { 0x7364a5be, 36 },
    { 0xa9b4a5bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4a5c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e4a5cb, 46 },
    { 0x5bb4a5cc, 61 },
    { 0xca24a5cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a64a5d0, 43 },
    { 0, 0 },
    { 0x4ca4a5d2, 98 },
    { 0x1e94a5d3, 55 },
    { 0xa184a5d3, 84 },
    { 0xc254a5d4, 61 },
    { 0x2714a5d4, 43 },
    { 0x6664a5d7, 91 },
    { 0xac04a5d8, 92 },
    { 0x67e4a5d7, 66 },
    { 0x5294a5d5, 78 },
    { 0x5274a5d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4a5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7484a5eb, 33 },
    { 0x6f34a5ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f4a5f3, 34 },
    { 0x7ee4a5f3, 61 },
    { 0xf144a5f3, 75 },
    { 0x9a04a5f6, 95 },
    { 0, 0 },
    { 0xf1c4a5f8, 23 },
    { 0xf544a5f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44a5fe, 61 },
    { 0x604a5ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e4a607, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f4a60b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4a60e, 61 },
    { 0x9274a60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f4a61d, 23 },
    { 0x1144a61e, 61 },
    { 0, 0 },
    { 0xe304a620, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314a626, 84 },
    { 0xf534a627, 90 },
    { 0, 0 },
    { 0x6124a629, 23 },
    { 0, 0 },
    { 0x6e94a62b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf824a632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb024a635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d4a638, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a63d, 99 },
    { 0x89a4a63d, 43 },
    { 0, 0 },
    { 0xf074a640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc94a644, 76 },
    { 0, 0 },
    { 0x90f4a646, 43 },
    { 0x30c4a647, 93 },
    { 0x99a4a646, 95 },
    { 0xc344a646, 91 },
    { 0xf1e4a64a, 66 },
    { 0xff74a64a, 65 },
    { 0xe1a4a64c, 61 },
    { 0xf6c4a64d, 95 },
    { 0x5724a64c, 23 },
    { 0x4664a64b, 55 },
    { 0x57c4a650, 78 },
    { 0x58e4a651, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f4a655, 76 },
    { 0x3f94a656, 61 },
    { 0xc364a656, 33 },
    { 0, 0 },
    { 0x84b4a659, 90 },
    { 0x4fe4a65a, 23 },
    { 0xac04a65b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa064a660, 46 },
    { 0xf4a660, 95 },
    { 0x6f94a662, 93 },
    { 0xfa54a661, 46 },
    { 0, 0 },
    { 0xc1f4a665, 78 },
    { 0, 0 },
    { 0x8a94a667, 90 },
    { 0x78d4a668, 23 },
    { 0, 0 },
    { 0xf24a66a, 92 },
    { 0, 0 },
    { 0x824a66c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a84a674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4a678, 34 },
    { 0x9d34a679, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154a67d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4a686, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc4a689, 23 },
    { 0xc794a68a, 61 },
    { 0x774a68b, 23 },
    { 0, 0 },
    { 0xbac4a68d, 87 },
    { 0xc754a68e, 43 },
    { 0x8e74a68f, 76 },
    { 0, 0 },
    { 0x7854a691, 98 },
    { 0, 0 },
    { 0x3714a693, 23 },
    { 0, 0 },
    { 0xa384a695, 61 },
    { 0, 0 },
    { 0x21b4a697, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94a6a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f64a6ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f4a6b2, 90 },
    { 0xb324a6b3, 91 },
    { 0xee64a6b4, 34 },
    { 0xb904a6b2, 17 },
    { 0xa154a6b6, 66 },
    { 0xcb14a6b6, 99 },
    { 0xc084a6b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4204a6bc, 34 },
    { 0, 0 },
    { 0xb554a6be, 61 },
    { 0xd0f4a6bf, 67 },
    { 0xe644a6bf, 61 },
    { 0x6ef4a6c1, 33 },
    { 0xd8b4a6c1, 95 },
    { 0xed74a6c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de4a6c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434a6c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa4a6cc, 93 },
    { 0xaa94a6cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9504a6d0, 78 },
    { 0x3d94a6d1, 90 },
    { 0xcec4a6d1, 93 },
    { 0xf6e4a6d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a6d8, 76 },
    { 0xd914a6d9, 98 },
    { 0, 0 },
    { 0xbc64a6db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4a6e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54a6ea, 90 },
    { 0, 0 },
    { 0x8974a6ec, 23 },
    { 0x5d44a6ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564a6f2, 98 },
    { 0, 0 },
    { 0x27d4a6f4, 55 },
    { 0x83c4a6f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f44a6fd, 90 },
    { 0x2264a6fe, 43 },
    { 0xf34a6ff, 90 },
    { 0xc824a6ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe4a703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed4a709, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4a710, 34 },
    { 0xb7c4a711, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a71a, 55 },
    { 0x60c4a71b, 98 },
    { 0x4aa4a71c, 61 },
    { 0x8074a71d, 40 },
    { 0, 0 },
    { 0xc314a71f, 23 },
    { 0x4f84a71f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a4a728, 84 },
    { 0, 0 },
    { 0x5f64a72a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384a72f, 52 },
    { 0x4224a730, 95 },
    { 0xdd24a731, 33 },
    { 0x9944a732, 78 },
    { 0xe934a732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd4a737, 90 },
    { 0, 0 },
    { 0xd374a739, 93 },
    { 0, 0 },
    { 0x1604a73b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9964a73e, 55 },
    { 0x1f84a73e, 23 },
    { 0xf3d4a73f, 90 },
    { 0, 0 },
    { 0xece4a742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda44a745, 90 },
    { 0, 0 },
    { 0xe824a747, 61 },
    { 0, 0 },
    { 0x9214a749, 65 },
    { 0x60b4a749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4a74e, 61 },
    { 0x4544a74f, 23 },
    { 0xfa84a750, 23 },
    { 0xe824a74f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb574a754, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d4a75d, 23 },
    { 0xe6b4a75e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074a762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94a766, 60 },
    { 0xa34a767, 90 },
    { 0xa594a768, 95 },
    { 0, 0 },
    { 0x53c4a76a, 90 },
    { 0xa3b4a76a, 93 },
    { 0, 0 },
    { 0xc1f4a76d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4a770, 43 },
    { 0, 0 },
    { 0x5df4a772, 33 },
    { 0x7d44a772, 95 },
    { 0xc9f4a772, 43 },
    { 0x5664a774, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64a77b, 61 },
    { 0, 0 },
    { 0x4274a77d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4a785, 77 },
    { 0, 0 },
    { 0xf404a787, 55 },
    { 0x6974a788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4a78b, 64 },
    { 0x9dc4a78b, 23 },
    { 0, 0 },
    { 0xea4a78e, 23 },
    { 0xec64a78e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7334a792, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b4a795, 48 },
    { 0, 0 },
    { 0x51e4a797, 76 },
    { 0x43b4a798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614a79b, 43 },
    { 0x7114a79c, 90 },
    { 0x3304a79c, 43 },
    { 0xaec4a79c, 43 },
    { 0xb244a79f, 46 },
    { 0xa2c4a79e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1114a7a8, 98 },
    { 0, 0 },
    { 0x1954a7aa, 67 },
    { 0, 0 },
    { 0xe524a7ac, 95 },
    { 0, 0 },
    { 0x3a84a7ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d4a7b2, 84 },
    { 0x4054a7b2, 33 },
    { 0x5a14a7b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae34a7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694a7c0, 46 },
    { 0x9874a7c0, 90 },
    { 0x1d34a7c2, 61 },
    { 0xe914a7c0, 56 },
    { 0xfd44a7c4, 46 },
    { 0xaa84a7c5, 33 },
    { 0x7374a7c5, 98 },
    { 0x44a4a7c7, 98 },
    { 0xb784a7c7, 33 },
    { 0x8954a7c3, 33 },
    { 0, 0 },
    { 0x9184a7cb, 52 },
    { 0xebd4a7cc, 55 },
    { 0xa4a7cc, 77 },
    { 0x8a34a7ce, 17 },
    { 0xc8c4a7ce, 76 },
    { 0xe694a7cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4974a7d5, 33 },
    { 0xc234a7d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc94a7e4, 36 },
    { 0x2594a7e4, 99 },
    { 0x4b64a7e6, 76 },
    { 0x8824a7e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134a7ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a4a7f1, 55 },
    { 0x89f4a7f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa724a7f6, 98 },
    { 0x4344a7f6, 81 },
    { 0xb334a7f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc254a7fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e4a802, 76 },
    { 0, 0 },
    { 0x7884a804, 98 },
    { 0, 0 },
    { 0x2da4a806, 61 },
    { 0x71b4a807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354a80c, 98 },
    { 0, 0 },
    { 0xad04a80e, 33 },
    { 0, 0 },
    { 0x89e4a810, 98 },
    { 0, 0 },
    { 0xce04a812, 95 },
    { 0x6d04a813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e4a81c, 55 },
    { 0x2b14a81d, 33 },
    { 0x5da4a81e, 90 },
    { 0, 0 },
    { 0x7fc4a820, 90 },
    { 0x8c94a821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc484a825, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4a829, 23 },
    { 0x5c64a82a, 61 },
    { 0, 0 },
    { 0xf124a82c, 78 },
    { 0x8924a82c, 98 },
    { 0x5634a82c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc494a832, 76 },
    { 0xece4a832, 43 },
    { 0x53f4a833, 90 },
    { 0x27f4a835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4a83b, 96 },
    { 0x1674a83c, 98 },
    { 0x7524a83c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a4a841, 52 },
    { 0x814a842, 78 },
    { 0x1024a843, 61 },
    { 0xd7f4a843, 55 },
    { 0xbbe4a844, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4a84b, 95 },
    { 0, 0 },
    { 0x5d04a84d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc844a855, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04a858, 33 },
    { 0, 0 },
    { 0x3c54a85a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9124a864, 84 },
    { 0x53a4a865, 84 },
    { 0x9764a866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b4a86a, 23 },
    { 0x9824a86a, 33 },
    { 0, 0 },
    { 0x4114a86d, 43 },
    { 0x28d4a86e, 76 },
    { 0, 0 },
    { 0xfd4a870, 90 },
    { 0x91c4a870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd4a877, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef84a87b, 61 },
    { 0x1c84a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a94a886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae4a88a, 61 },
    { 0x9e24a88b, 65 },
    { 0xb8f4a88a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054a891, 78 },
    { 0, 0 },
    { 0xcf84a893, 85 },
    { 0x3aa4a894, 34 },
    { 0, 0 },
    { 0x1d34a896, 52 },
    { 0x19b4a896, 17 },
    { 0, 0 },
    { 0xa7c4a899, 61 },
    { 0x3d64a899, 98 },
    { 0x9084a89b, 90 },
    { 0x8e34a89c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7224a89f, 34 },
    { 0x1564a8a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c64a8a3, 77 },
    { 0x4224a8a4, 61 },
    { 0x124a8a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b4a8a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4a8ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a54a8af, 95 },
    { 0, 0 },
    { 0x3ef4a8b1, 75 },
    { 0xcd34a8b1, 61 },
    { 0x6ca4a8b3, 100 },
    { 0x8574a8b4, 95 },
    { 0, 0 },
    { 0x29d4a8b6, 95 },
    { 0x8ee4a8b7, 95 },
    { 0xca04a8b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f64a8bd, 33 },
    { 0x6d84a8be, 61 },
    { 0, 0 },
    { 0x7c94a8c0, 95 },
    { 0xa7a4a8c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f4a8c4, 91 },
    { 0x4bb4a8c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224a8cc, 90 },
    { 0xce34a8cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3004a8d0, 90 },
    { 0x41a4a8d1, 100 },
    { 0x9df4a8d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3084a8d6, 76 },
    { 0, 0 },
    { 0xc8f4a8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5034a8db, 98 },
    { 0xe7f4a8dc, 98 },
    { 0x4a94a8dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a8e1, 33 },
    { 0x4354a8e1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954a8e5, 95 },
    { 0x3984a8e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa4a8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b4a8ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b04a8f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf84a8f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854a8f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594a8fc, 61 },
    { 0, 0 },
    { 0xe434a8fe, 75 },
    { 0, 0 },
    { 0xf3a4a900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564a903, 84 },
    { 0xe604a904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754a909, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4a90c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf194a90f, 90 },
    { 0, 0 },
    { 0x4de4a911, 33 },
    { 0x614a912, 76 },
    { 0, 0 },
    { 0x6604a914, 55 },
    { 0x90d4a915, 98 },
    { 0, 0 },
    { 0x4a64a917, 78 },
    { 0xd564a918, 33 },
    { 0x5b74a919, 91 },
    { 0xc034a91a, 90 },
    { 0x3934a91a, 43 },
    { 0xb614a91c, 23 },
    { 0x5674a91c, 93 },
    { 0x80f4a91e, 33 },
    { 0xaac4a91d, 43 },
    { 0xdb04a920, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a92d, 93 },
    { 0, 0 },
    { 0x2464a92f, 76 },
    { 0x5844a930, 33 },
    { 0xfe54a930, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7094a934, 33 },
    { 0, 0 },
    { 0xa734a936, 98 },
    { 0xc4f4a937, 90 },
    { 0xcc84a938, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf4a93b, 68 },
    { 0, 0 },
    { 0x1ff4a93d, 76 },
    { 0x7c64a93e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4a943, 90 },
    { 0x4f14a943, 90 },
    { 0x534a945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf084a94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf984a94e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054a951, 46 },
    { 0x6534a952, 33 },
    { 0xd4a951, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af4a95b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf74a95e, 66 },
    { 0x1b04a95f, 24 },
    { 0x6274a960, 98 },
    { 0x44d4a960, 85 },
    { 0x2c44a962, 76 },
    { 0xccc4a962, 84 },
    { 0, 0 },
    { 0xb504a965, 98 },
    { 0x7d04a965, 98 },
    { 0xd8f4a967, 23 },
    { 0xa384a968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8174a970, 34 },
    { 0xd6c4a971, 34 },
    { 0x1354a972, 91 },
    { 0xa814a972, 88 },
    { 0xb2b4a974, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca4a978, 33 },
    { 0xe1a4a979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f94a97f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b44a982, 98 },
    { 0, 0 },
    { 0x4ac4a984, 17 },
    { 0xf7b4a984, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b4a98a, 52 },
    { 0, 0 },
    { 0xe964a98c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df4a991, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c4a997, 98 },
    { 0xed4a998, 90 },
    { 0xd814a999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a34a99e, 90 },
    { 0, 0 },
    { 0x8074a9a0, 98 },
    { 0x6914a9a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca14a9a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f4a9a9, 55 },
    { 0x2d4a9a9, 33 },
    { 0xf844a9ab, 66 },
    { 0xbd64a9ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4a9af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4a9b3, 93 },
    { 0, 0 },
    { 0x5a54a9b5, 33 },
    { 0x79d4a9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b4a9bb, 61 },
    { 0x34c4a9bc, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed04a9c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde4a9c5, 40 },
    { 0x1d34a9c6, 93 },
    { 0x7824a9c7, 95 },
    { 0x374a9c7, 96 },
    { 0, 0 },
    { 0x36a4a9ca, 61 },
    { 0, 0 },
    { 0xd554a9cc, 98 },
    { 0, 0 },
    { 0xfc64a9ce, 33 },
    { 0x6d54a9cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf554a9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf204a9d7, 33 },
    { 0x8504a9d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b34a9db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d74a9e2, 33 },
    { 0xc644a9e3, 90 },
    { 0xfd4a9e4, 46 },
    { 0, 0 },
    { 0x90e4a9e6, 90 },
    { 0, 0 },
    { 0x2dc4a9e8, 76 },
    { 0xcb94a9e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a9f0, 90 },
    { 0xb504a9f1, 67 },
    { 0x75a4a9f2, 90 },
    { 0xc484a9f2, 33 },
    { 0x1ad4a9f0, 95 },
    { 0, 0 },
    { 0xe914a9f6, 95 },
    { 0x4b4a9f7, 23 },
    { 0x294a9f8, 33 },
    { 0, 0 },
    { 0xdc34a9fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4aa02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4aa05, 23 },
    { 0xcba4aa06, 76 },
    { 0x394aa07, 61 },
    { 0xcc64aa08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1584aa0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c4aa10, 34 },
    { 0x7564aa11, 61 },
    { 0x8454aa10, 61 },
    { 0x4574aa12, 33 },
    { 0x5c04aa14, 90 },
    { 0x5504aa15, 55 },
    { 0x7bd4aa16, 43 },
    { 0, 0 },
    { 0x4104aa18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab4aa1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e4aa26, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d4aa2b, 66 },
    { 0x8b14aa2c, 23 },
    { 0x8954aa2d, 52 },
    { 0x184aa2e, 95 },
    { 0xbdc4aa2f, 90 },
    { 0xfe4aa2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4aa33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a74aa37, 55 },
    { 0xae24aa38, 34 },
    { 0xc8b4aa38, 33 },
    { 0x7174aa3a, 90 },
    { 0x8744aa3b, 81 },
    { 0xbe54aa3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4aa43, 33 },
    { 0xa5d4aa44, 90 },
    { 0x77e4aa45, 33 },
    { 0xef24aa46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e64aa4e, 23 },
    { 0x8514aa4f, 90 },
    { 0x3784aa50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9154aa54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4aa5e, 90 },
    { 0, 0 },
    { 0x7ad4aa60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf854aa63, 98 },
    { 0x3824aa64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac4aa68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb24aa6b, 98 },
    { 0xbcf4aa6c, 55 },
    { 0, 0 },
    { 0x65f4aa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab4aa71, 61 },
    { 0, 0 },
    { 0x3c14aa73, 23 },
    { 0xd1f4aa74, 76 },
    { 0x5f04aa73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f44aa84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e4aa8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4aa91, 60 },
    { 0x3494aa92, 55 },
    { 0x5f44aa92, 55 },
    { 0xa514aa93, 90 },
    { 0xc5a4aa95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb4aa98, 34 },
    { 0, 0 },
    { 0xc114aa9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94aa9f, 61 },
    { 0, 0 },
    { 0x384aaa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104aaa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574aaad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b4aab4, 67 },
    { 0x9f64aab5, 52 },
    { 0x83e4aab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4aab9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4aabc, 95 },
    { 0xc3d4aabd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4aac0, 33 },
    { 0xd6a4aac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974aace, 93 },
    { 0, 0 },
    { 0xc8d4aad0, 98 },
    { 0x5154aad1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf644aad6, 23 },
    { 0xa2c4aad7, 67 },
    { 0xff34aad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754aadc, 90 },
    { 0, 0 },
    { 0x2aa4aade, 61 },
    { 0x2634aadf, 68 },
    { 0xfc4aade, 95 },
    { 0x71e4aae1, 61 },
    { 0xd884aae2, 84 },
    { 0xe754aae3, 66 },
    { 0x5d4aae3, 76 },
    { 0xd294aadf, 81 },
    { 0, 0 },
    { 0xbb54aae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924aaeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc54aaf0, 90 },
    { 0, 0 },
    { 0x75f4aaf2, 23 },
    { 0x6d4aaf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4aaf6, 46 },
    { 0xa334aaf7, 66 },
    { 0xce64aaf8, 33 },
    { 0x1554aaf9, 90 },
    { 0xdc94aaf9, 52 },
    { 0x6934aaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe4aafe, 61 },
    { 0x6684aaff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc984ab04, 84 },
    { 0, 0 },
    { 0xe074ab06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4ab09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f54ab0c, 43 },
    { 0x3734ab0d, 100 },
    { 0x2e74ab0e, 33 },
    { 0xc14ab0e, 90 },
    { 0x5274ab10, 100 },
    { 0x7aa4ab0f, 78 },
    { 0x3eb4ab12, 61 },
    { 0x5c44ab13, 34 },
    { 0xdf4ab13, 85 },
    { 0xd5b4ab15, 85 },
    { 0xb8f4ab0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8544ab19, 40 },
    { 0, 0 },
    { 0x20a4ab1b, 34 },
    { 0xbc04ab1b, 33 },
    { 0xee14ab1d, 61 },
    { 0x614ab1e, 87 },
    { 0x14d4ab1f, 88 },
    { 0x99a4ab20, 95 },
    { 0x174ab1c, 95 },
    { 0x8ee4ab22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354ab25, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d4ab28, 66 },
    { 0x93a4ab29, 91 },
    { 0x98e4ab28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b4ab2f, 52 },
    { 0xbd44ab2f, 43 },
    { 0x6654ab31, 33 },
    { 0x7e4ab2f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe4ab3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf834ab3e, 64 },
    { 0xc9e4ab3e, 61 },
    { 0, 0 },
    { 0x6d34ab41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff4ab44, 61 },
    { 0x6c54ab45, 98 },
    { 0x5f84ab44, 91 },
    { 0x5ba4ab47, 14 },
    { 0xc0e4ab48, 90 },
    { 0, 0 },
    { 0xf4b4ab4a, 100 },
    { 0xb9b4ab4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3404ab4e, 93 },
    { 0x1894ab4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a4ab53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014ab5c, 61 },
    { 0x1184ab5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de4ab60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa4ab66, 90 },
    { 0x5ea4ab66, 46 },
    { 0xcfb4ab68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f4ab6b, 33 },
    { 0xe054ab6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7254ab72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe4ab77, 76 },
    { 0x3d54ab78, 76 },
    { 0xc894ab79, 81 },
    { 0xfe4ab7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654ab83, 33 },
    { 0x6d64ab84, 95 },
    { 0, 0 },
    { 0xe0f4ab86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce4ab91, 61 },
    { 0x6744ab92, 98 },
    { 0x3974ab93, 23 },
    { 0xc664ab92, 23 },
    { 0x30c4ab91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c4ab9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa394aba0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4aba5, 46 },
    { 0, 0 },
    { 0xfb34aba7, 90 },
    { 0x87b4aba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854abab, 61 },
    { 0xd0b4abac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34abaf, 76 },
    { 0x6ab4abb0, 98 },
    { 0, 0 },
    { 0x8594abb2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd954abb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564abbb, 90 },
    { 0, 0 },
    { 0x7cc4abbd, 33 },
    { 0, 0 },
    { 0xfa04abbf, 23 },
    { 0x5524abc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4abc7, 91 },
    { 0x4654abc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d04abcb, 52 },
    { 0x8824abcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b4abd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a4abd3, 95 },
    { 0xd104abd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd204abd7, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0x3254abda, 61 },
    { 0, 0 },
    { 0xfa34abdc, 90 },
    { 0xbe34abdd, 55 },
    { 0x15d4abde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e4abe1, 98 },
    { 0x84b4abe2, 23 },
    { 0x54d4abe2, 95 },
    { 0x6cd4abe4, 88 },
    { 0x9534abe4, 61 },
    { 0, 0 },
    { 0xa414abe7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc4abeb, 88 },
    { 0xf0c4abeb, 93 },
    { 0xa674abed, 61 },
    { 0x3df4abec, 34 },
    { 0xef54abef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d4abf2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4714abf9, 55 },
    { 0x56a4abfa, 55 },
    { 0xa374abfb, 90 },
    { 0x27b4abfc, 43 },
    { 0, 0 },
    { 0x6a4abfe, 23 },
    { 0x4ca4abff, 95 },
    { 0x9184abff, 95 },
    { 0x5a94abfe, 95 },
    { 0, 0 },
    { 0x5734ac03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a04ac06, 34 },
    { 0xf9f4ac07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64ac10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4ac13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x804ac16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd4ac19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4ac1c, 98 },
    { 0x2134ac1c, 90 },
    { 0, 0 },
    { 0x314ac1f, 93 },
    { 0xbbd4ac20, 100 },
    { 0x1d84ac21, 33 },
    { 0x41d4ac1f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd944ac25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd4ac29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa4ac2e, 68 },
    { 0xf5c4ac2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ac34, 90 },
    { 0x8a34ac34, 61 },
    { 0, 0 },
    { 0x874ac37, 98 },
    { 0, 0 },
    { 0xb654ac39, 90 },
    { 0x8284ac3a, 23 },
    { 0x964ac3b, 61 },
    { 0x1d74ac3a, 95 },
    { 0x8f94ac3c, 52 },
    { 0xb4b4ac3e, 43 },
    { 0x4d64ac3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ac4a, 23 },
    { 0, 0 },
    { 0xede4ac4c, 90 },
    { 0x3a4ac4d, 61 },
    { 0xed74ac4e, 78 },
    { 0xffa4ac4d, 90 },
    { 0x4184ac50, 98 },
    { 0, 0 },
    { 0xb304ac52, 87 },
    { 0x28e4ac52, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd004ac56, 90 },
    { 0, 0 },
    { 0xb8c4ac58, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064ac5b, 55 },
    { 0, 0 },
    { 0x9944ac5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee64ac61, 95 },
    { 0x7b24ac62, 44 },
    { 0x6be4ac63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af4ac66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c4ac69, 90 },
    { 0xe424ac6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc4ac6e, 84 },
    { 0xfbc4ac6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd4ac72, 81 },
    { 0x1a34ac73, 98 },
    { 0xd34ac74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4ac77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd594ac7c, 34 },
    { 0xf84ac7d, 76 },
    { 0x15b4ac7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7194ac81, 84 },
    { 0x9d14ac81, 55 },
    { 0xa7c4ac83, 76 },
    { 0xc994ac83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7834ac89, 33 },
    { 0, 0 },
    { 0x21c4ac8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc4ac91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8674ac97, 95 },
    { 0, 0 },
    { 0x9034ac99, 98 },
    { 0, 0 },
    { 0x7814ac9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074aca0, 46 },
    { 0x9f54aca1, 90 },
    { 0x31c4aca2, 61 },
    { 0, 0 },
    { 0xb014aca4, 98 },
    { 0xef84aca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x484aca8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e4acab, 98 },
    { 0x8564acac, 61 },
    { 0x9624acad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a4acb1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b4acb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5994acb9, 95 },
    { 0xfa14acba, 98 },
    { 0x4fd4acbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914acbf, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e4acc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa854acc9, 40 },
    { 0xec44acca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc34accd, 76 },
    { 0x6924accd, 98 },
    { 0, 0 },
    { 0x8e54acd0, 55 },
    { 0, 0 },
    { 0xce4acd2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a4acd9, 23 },
    { 0, 0 },
    { 0x7e24acdb, 98 },
    { 0x2794acdc, 68 },
    { 0x4d54acdd, 65 },
    { 0xdc54acdc, 95 },
    { 0xc734acdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe114ace3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614ace8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c4aceb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc444acef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f4acf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb724acf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c4acf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4acfe, 95 },
    { 0, 0 },
    { 0xa94ad00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf834ad04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ad08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2424ad0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944ad0f, 23 },
    { 0x4a64ad0f, 23 },
    { 0xa7a4ad11, 90 },
    { 0xcc94ad12, 23 },
    { 0, 0 },
    { 0xa044ad14, 46 },
    { 0, 0 },
    { 0x6884ad16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b4ad19, 23 },
    { 0, 0 },
    { 0xb734ad1b, 33 },
    { 0, 0 },
    { 0xbfa4ad1d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb514ad22, 61 },
    { 0x1574ad23, 77 },
    { 0x5ab4ad22, 90 },
    { 0x3d24ad22, 33 },
    { 0xbcb4ad26, 33 },
    { 0, 0 },
    { 0x1cc4ad28, 93 },
    { 0x2784ad29, 61 },
    { 0xd664ad2a, 33 },
    { 0x9114ad28, 93 },
    { 0xcd84ad2c, 90 },
    { 0, 0 },
    { 0xdb44ad2e, 34 },
    { 0x42f4ad2f, 23 },
    { 0xb7e4ad2f, 100 },
    { 0, 0 },
    { 0x7a24ad32, 61 },
    { 0xc7e4ad32, 98 },
    { 0xd5d4ad34, 98 },
    { 0xe364ad35, 61 },
    { 0x7cc4ad33, 43 },
    { 0x1284ad35, 55 },
    { 0x7a34ad35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2734ad3b, 31 },
    { 0xe404ad3c, 23 },
    { 0xfec4ad3d, 90 },
    { 0x8d44ad3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3824ad42, 17 },
    { 0xbdc4ad42, 23 },
    { 0x2da4ad44, 95 },
    { 0x4b84ad45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb124ad4e, 46 },
    { 0x5194ad4e, 46 },
    { 0x63c4ad50, 90 },
    { 0xcff4ad50, 69 },
    { 0xf024ad52, 90 },
    { 0x6794ad51, 90 },
    { 0x7444ad54, 81 },
    { 0, 0 },
    { 0x86e4ad56, 95 },
    { 0x82f4ad57, 98 },
    { 0xbe84ad58, 100 },
    { 0, 0 },
    { 0x9824ad5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d4ad63, 93 },
    { 0xf04ad64, 90 },
    { 0xaec4ad65, 61 },
    { 0x9024ad66, 99 },
    { 0, 0 },
    { 0xff4ad68, 90 },
    { 0x3864ad69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a04ad6d, 66 },
    { 0x90a4ad6d, 33 },
    { 0x1a64ad6e, 19 },
    { 0x7754ad70, 88 },
    { 0xc1c4ad70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ad74, 91 },
    { 0, 0 },
    { 0xcd84ad76, 95 },
    { 0, 0 },
    { 0x80c4ad78, 17 },
    { 0xc414ad79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774ad7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe644ad81, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b4ad8d, 65 },
    { 0x3e4ad8e, 23 },
    { 0, 0 },
    { 0xfca4ad90, 93 },
    { 0xcfb4ad91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a04ad95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6654ad9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4ada5, 46 },
    { 0x744ada5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8474adaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7094adae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d4adb2, 33 },
    { 0x9d44adb3, 33 },
    { 0x1f64adb4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4adb7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5074adba, 61 },
    { 0x1434adba, 61 },
    { 0x2fe4adbb, 95 },
    { 0, 0 },
    { 0xac4adbe, 99 },
    { 0x7304adbf, 33 },
    { 0xf054adbe, 90 },
    { 0x5ab4adc1, 61 },
    { 0x1784adc2, 93 },
    { 0x6d14adc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e4adc6, 87 },
    { 0x6974adc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d04adca, 76 },
    { 0x68d4adca, 61 },
    { 0, 0 },
    { 0x63a4adcd, 84 },
    { 0, 0 },
    { 0x6544adcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2934add7, 23 },
    { 0x7c94add7, 23 },
    { 0x9cf4add8, 34 },
    { 0x7fb4add8, 43 },
    { 0xd684addb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ade3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2374ade9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4adec, 33 },
    { 0, 0 },
    { 0x5944adee, 48 },
    { 0x2754adee, 98 },
    { 0, 0 },
    { 0x28e4adf1, 23 },
    { 0x4b64adf1, 67 },
    { 0xf4e4adf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5064adf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa084adfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb974ae00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4ae04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8234ae08, 33 },
    { 0xcfb4ae09, 93 },
    { 0x7ed4ae0a, 55 },
    { 0, 0 },
    { 0xfa94ae0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4244ae15, 98 },
    { 0xc5b4ae16, 23 },
    { 0x3ce4ae17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d4ae1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb4ae1d, 46 },
    { 0x90c4ae1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764ae29, 88 },
    { 0xe7d4ae29, 95 },
    { 0, 0 },
    { 0x41e4ae2c, 100 },
    { 0x9894ae2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94ae37, 61 },
    { 0xd84ae38, 61 },
    { 0xb74ae39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7914ae3d, 34 },
    { 0x85e4ae3e, 84 },
    { 0, 0 },
    { 0xe3e4ae40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6154ae49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9054ae4c, 90 },
    { 0x9464ae4d, 98 },
    { 0, 0 },
    { 0x9804ae4f, 33 },
    { 0x67a4ae4f, 76 },
    { 0xbf54ae4f, 95 },
    { 0x7784ae51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e94ae5a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694ae5d, 85 },
    { 0, 0 },
    { 0x6b94ae5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2984ae64, 61 },
    { 0x3d24ae65, 81 },
    { 0, 0 },
    { 0xf0d4ae67, 65 },
    { 0xcbc4ae68, 61 },
    { 0x7964ae69, 98 },
    { 0x5d74ae69, 33 },
    { 0, 0 },
    { 0xa124ae6c, 55 },
    { 0x8c84ae6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8884ae75, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c4ae78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d4ae7d, 94 },
    { 0xeda4ae7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4ae81, 66 },
    { 0xcd24ae82, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b4ae86, 78 },
    { 0x3684ae87, 90 },
    { 0, 0 },
    { 0x72c4ae89, 52 },
    { 0x1d14ae89, 33 },
    { 0, 0 },
    { 0x5e74ae8c, 98 },
    { 0xd124ae8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ae95, 90 },
    { 0, 0 },
    { 0xa934ae97, 75 },
    { 0x2144ae98, 61 },
    { 0x21c4ae98, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5014ae9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974aea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8024aea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f64aead, 33 },
    { 0, 0 },
    { 0x9714aeaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac4aeb5, 23 },
    { 0xd104aeb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5834aeb9, 61 },
    { 0, 0 },
    { 0xce44aebb, 23 },
    { 0x6134aebc, 40 },
    { 0x9774aebd, 90 },
    { 0xa24aebe, 43 },
    { 0xb3e4aebc, 87 },
    { 0x1184aebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e04aeca, 93 },
    { 0x23f4aecb, 47 },
    { 0xc9e4aecb, 11 },
    { 0, 0 },
    { 0xfac4aece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d94aed3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064aed8, 93 },
    { 0xc674aed8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc874aedc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5334aee4, 77 },
    { 0xf2a4aee5, 43 },
    { 0xa6e4aee6, 23 },
    { 0x1b4aee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9814aef3, 33 },
    { 0, 0 },
    { 0xeb84aef5, 87 },
    { 0x6e64aef6, 43 },
    { 0x33f4aef7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8594aefb, 76 },
    { 0xb4e4aefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa324af03, 23 },
    { 0x5044af03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34af07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a4af0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4af10, 66 },
    { 0, 0 },
    { 0x59e4af12, 52 },
    { 0xfe94af13, 23 },
    { 0, 0 },
    { 0xd9e4af15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9634af19, 61 },
    { 0, 0 },
    { 0x2dd4af1b, 90 },
    { 0xf484af1c, 90 },
    { 0x9eb4af1c, 93 },
    { 0xb664af1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1944af24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e4af27, 98 },
    { 0x34af27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d4af2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044af30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574af35, 95 },
    { 0, 0 },
    { 0xa224af37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf54af3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e44af3f, 55 },
    { 0xe3a4af3f, 66 },
    { 0xdba4af40, 33 },
    { 0xc134af42, 98 },
    { 0xe464af42, 34 },
    { 0x3774af44, 81 },
    { 0, 0 },
    { 0xdc14af46, 95 },
    { 0, 0 },
    { 0x5af4af48, 23 },
    { 0x2a24af49, 46 },
    { 0xe24af4a, 69 },
    { 0x11f4af4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad64af50, 93 },
    { 0xb884af50, 98 },
    { 0xa874af52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1894af59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf464af61, 95 },
    { 0, 0 },
    { 0x1b14af63, 34 },
    { 0xc0d4af64, 90 },
    { 0x1cc4af64, 95 },
    { 0x7144af66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4af69, 78 },
    { 0xb894af6a, 76 },
    { 0x9294af69, 34 },
    { 0x6754af6a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04af70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d4af74, 95 },
    { 0xfa44af75, 25 },
    { 0x3204af76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4af79, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94af7d, 61 },
    { 0, 0 },
    { 0xcb84af7f, 90 },
    { 0xfac4af7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494af84, 61 },
    { 0x6064af84, 33 },
    { 0xef04af86, 92 },
    { 0x794af86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94af8a, 33 },
    { 0xc984af8b, 61 },
    { 0xf814af8c, 23 },
    { 0xfe04af8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64af92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3254af9b, 61 },
    { 0xc294af9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724afa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4afa4, 66 },
    { 0xe0d4afa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944afa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c4afab, 61 },
    { 0x2494afac, 43 },
    { 0x98d4afab, 93 },
    { 0x3934afab, 99 },
    { 0x1264afaf, 61 },
    { 0xf464afb0, 90 },
    { 0xa8c4afb1, 33 },
    { 0x6094afb2, 61 },
    { 0x4a84afb2, 33 },
    { 0, 0 },
    { 0xca24afb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d54afc0, 100 },
    { 0x4a04afc1, 33 },
    { 0xa444afc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744afc9, 33 },
    { 0, 0 },
    { 0x6324afcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4afd2, 92 },
    { 0x8334afd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94afd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4afdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e4afe0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d54afe6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5574afe9, 88 },
    { 0, 0 },
    { 0x5ce4afeb, 47 },
    { 0x1444afec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e4afef, 66 },
    { 0x58c4afef, 98 },
    { 0x46f4aff1, 90 },
    { 0xbfe4aff2, 43 },
    { 0x18f4aff3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d64aff9, 95 },
    { 0xbc54affa, 33 },
    { 0x71f4affa, 75 },
    { 0x46b4affa, 90 },
    { 0xd154aff9, 98 },
    { 0x5954affc, 95 },
    { 0xd524aff9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a4b002, 55 },
    { 0x1904b003, 95 },
    { 0, 0 },
    { 0xcfd4b005, 95 },
    { 0, 0 },
    { 0x3fd4b007, 71 },
    { 0x1814b008, 90 },
    { 0x9444b009, 87 },
    { 0xf984b009, 55 },
    { 0x68f4b00b, 61 },
    { 0xc9e4b00c, 98 },
    { 0x4aa4b00c, 33 },
    { 0xf7c4b00e, 91 },
    { 0, 0 },
    { 0x8814b010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94b015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de4b01b, 46 },
    { 0x6e14b01c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7754b01f, 78 },
    { 0xb164b020, 46 },
    { 0, 0 },
    { 0x3c14b022, 61 },
    { 0, 0 },
    { 0xeb4b024, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd434b027, 23 },
    { 0, 0 },
    { 0xcc34b029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c4b02c, 34 },
    { 0xc5c4b02d, 95 },
    { 0x9a24b02e, 90 },
    { 0xb604b02f, 84 },
    { 0x9144b02e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f94b035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f74b03c, 76 },
    { 0x2874b03d, 33 },
    { 0x8134b03d, 100 },
    { 0x6424b03f, 33 },
    { 0xf6d4b03c, 95 },
    { 0xf044b03d, 40 },
    { 0xe4b042, 61 },
    { 0xf554b043, 61 },
    { 0x3e24b041, 90 },
    { 0x3bf4b045, 100 },
    { 0x73b4b046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3414b04a, 95 },
    { 0xff84b04b, 93 },
    { 0xa44b04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf724b052, 23 },
    { 0xfd34b053, 98 },
    { 0x7b64b054, 66 },
    { 0x57b4b055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c4b058, 95 },
    { 0x5ee4b058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c04b05d, 53 },
    { 0, 0 },
    { 0x25e4b05f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa214b064, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e54b06a, 88 },
    { 0x8c84b06b, 50 },
    { 0xdf34b06c, 99 },
    { 0, 0 },
    { 0xca4b06e, 34 },
    { 0, 0 },
    { 0xfeb4b070, 76 },
    { 0, 0 },
    { 0xe6e4b072, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd74b07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a24b082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4b086, 40 },
    { 0x8c94b087, 90 },
    { 0, 0 },
    { 0x95b4b089, 90 },
    { 0, 0 },
    { 0xe424b08b, 95 },
    { 0xc0a4b08c, 93 },
    { 0xe124b08d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f94b090, 67 },
    { 0, 0 },
    { 0xaad4b092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd514b095, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b099, 55 },
    { 0x7a64b09a, 91 },
    { 0, 0 },
    { 0xd2b4b09c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef4b0a0, 17 },
    { 0x3674b0a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff4b0a7, 98 },
    { 0x8494b0a8, 95 },
    { 0, 0 },
    { 0xe794b0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa34b0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44b0b1, 33 },
    { 0xc6a4b0b2, 34 },
    { 0x904b0b3, 98 },
    { 0, 0 },
    { 0xbba4b0b5, 55 },
    { 0x9364b0b6, 67 },
    { 0, 0 },
    { 0xd4b4b0b8, 77 },
    { 0x4d44b0b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb914b0bc, 67 },
    { 0x4684b0bd, 85 },
    { 0xda14b0bd, 55 },
    { 0xd3e4b0bf, 90 },
    { 0, 0 },
    { 0x1704b0c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7984b0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4b0cd, 90 },
    { 0xb454b0cd, 76 },
    { 0x2204b0ce, 90 },
    { 0xcf44b0d0, 46 },
    { 0, 0 },
    { 0x70d4b0d2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94b0d5, 76 },
    { 0x5c94b0d6, 78 },
    { 0x54a4b0d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4b0da, 95 },
    { 0xc564b0da, 91 },
    { 0x14c4b0dc, 33 },
    { 0xe634b0dc, 99 },
    { 0x4124b0de, 46 },
    { 0, 0 },
    { 0xa8e4b0e0, 65 },
    { 0x9c84b0e1, 66 },
    { 0xdeb4b0e0, 55 },
    { 0x2874b0e0, 95 },
    { 0x7ef4b0e3, 90 },
    { 0x77d4b0e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c94b0e8, 46 },
    { 0xc0d4b0e8, 66 },
    { 0x3be4b0ea, 43 },
    { 0xeee4b0e8, 61 },
    { 0x7ac4b0ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf094b0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe104b0f9, 55 },
    { 0, 0 },
    { 0x394b0fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd4b100, 66 },
    { 0xe564b101, 90 },
    { 0xc004b100, 55 },
    { 0x8664b103, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3534b107, 12 },
    { 0, 0 },
    { 0x5534b109, 23 },
    { 0, 0 },
    { 0x314b10b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794b111, 98 },
    { 0, 0 },
    { 0x1aa4b113, 33 },
    { 0xcce4b114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b94b117, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb354b11b, 98 },
    { 0x6174b11b, 23 },
    { 0x5eb4b11b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1594b121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf44b125, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd4b128, 34 },
    { 0xa2c4b128, 85 },
    { 0xe194b12a, 52 },
    { 0x1f44b12a, 33 },
    { 0xec64b12a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7434b12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b132, 76 },
    { 0x3354b132, 76 },
    { 0, 0 },
    { 0x6284b135, 85 },
    { 0x6704b136, 43 },
    { 0xfa24b137, 52 },
    { 0x514b138, 82 },
    { 0xd764b139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4b13c, 55 },
    { 0xd424b13d, 90 },
    { 0xaba4b13e, 61 },
    { 0x6424b13f, 101 },
    { 0xef84b140, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8684b148, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc04b14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc4b154, 23 },
    { 0, 0 },
    { 0xcb84b156, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3624b15b, 85 },
    { 0xbcd4b15c, 23 },
    { 0, 0 },
    { 0xe8b4b15e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda4b167, 76 },
    { 0x34b167, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9644b176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9054b179, 78 },
    { 0x2fc4b17a, 67 },
    { 0, 0 },
    { 0x2c14b17c, 33 },
    { 0x34f4b17d, 93 },
    { 0, 0 },
    { 0x4964b17f, 78 },
    { 0, 0 },
    { 0x8564b181, 98 },
    { 0xb974b181, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xb274b185, 100 },
    { 0xa544b186, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9714b189, 76 },
    { 0xf774b18a, 23 },
    { 0x8464b189, 100 },
    { 0xd7d4b18c, 23 },
    { 0xe6b4b18c, 75 },
    { 0xc1f4b18e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4b193, 61 },
    { 0, 0 },
    { 0xab64b195, 90 },
    { 0x704b196, 61 },
    { 0xc1b4b197, 98 },
    { 0x2984b198, 55 },
    { 0, 0 },
    { 0x6524b19a, 65 },
    { 0x3304b19b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e34b19e, 90 },
    { 0x5e84b19e, 61 },
    { 0x3324b1a0, 90 },
    { 0x964b1a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe34b1ab, 34 },
    { 0xcd54b1ac, 33 },
    { 0x3644b1ac, 98 },
    { 0, 0 },
    { 0xf6d4b1af, 55 },
    { 0xdc24b1b0, 92 },
    { 0x8f84b1b0, 100 },
    { 0x5b94b1b0, 23 },
    { 0xe6f4b1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204b1b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034b1bd, 90 },
    { 0x7754b1bd, 95 },
    { 0x1784b1bd, 93 },
    { 0xe754b1c0, 90 },
    { 0xaeb4b1c1, 78 },
    { 0x7b74b1c2, 23 },
    { 0xa404b1be, 83 },
    { 0x6274b1bd, 23 },
    { 0x1e64b1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae4b1cc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e4b1d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d34b1db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44b1de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034b1e5, 90 },
    { 0x3654b1e6, 33 },
    { 0, 0 },
    { 0x9f94b1e8, 75 },
    { 0xcb24b1e9, 23 },
    { 0xdd54b1e9, 98 },
    { 0x6694b1eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024b1f7, 61 },
    { 0, 0 },
    { 0xac14b1f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4b1fc, 23 },
    { 0xd274b1fc, 52 },
    { 0x2514b1fe, 76 },
    { 0xbca4b1fe, 76 },
    { 0x3354b1ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4b203, 43 },
    { 0xade4b204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc884b207, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9904b20a, 90 },
    { 0, 0 },
    { 0xe134b20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b20f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5424b215, 61 },
    { 0xab04b216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b4b21b, 66 },
    { 0x4624b21c, 67 },
    { 0x5af4b21d, 84 },
    { 0x73c4b21d, 49 },
    { 0xbf4b21e, 90 },
    { 0x1eb4b220, 55 },
    { 0x9a44b21f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda74b225, 46 },
    { 0, 0 },
    { 0x6fc4b227, 95 },
    { 0xa404b228, 52 },
    { 0x6b84b228, 23 },
    { 0x7544b22a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d4b231, 53 },
    { 0, 0 },
    { 0xf304b233, 23 },
    { 0x7e74b234, 23 },
    { 0, 0 },
    { 0xf834b236, 61 },
    { 0x6c64b236, 17 },
    { 0, 0 },
    { 0x2164b239, 33 },
    { 0, 0 },
    { 0xf894b23b, 90 },
    { 0x18e4b23c, 84 },
    { 0xf1a4b23c, 55 },
    { 0xf694b23d, 98 },
    { 0, 0 },
    { 0xaf84b240, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec64b243, 34 },
    { 0x8d74b244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9614b24b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b04b24e, 90 },
    { 0x62a4b24f, 40 },
    { 0, 0 },
    { 0x1264b251, 95 },
    { 0x9f34b252, 61 },
    { 0xd4f4b253, 76 },
    { 0, 0 },
    { 0x51f4b255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d4b258, 43 },
    { 0xc4a4b258, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a4b25c, 33 },
    { 0x6624b25d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3634b266, 34 },
    { 0x7c34b267, 90 },
    { 0, 0 },
    { 0x1804b269, 31 },
    { 0, 0 },
    { 0x3584b26b, 90 },
    { 0x5c74b26c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf94b270, 90 },
    { 0x2294b271, 98 },
    { 0x3c4b272, 90 },
    { 0x3824b271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e4b27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe4b283, 99 },
    { 0, 0 },
    { 0x9c84b285, 34 },
    { 0, 0 },
    { 0x9724b287, 33 },
    { 0x6354b287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1944b28b, 33 },
    { 0xec74b28c, 33 },
    { 0x7804b28d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c4b294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7584b29e, 84 },
    { 0x1624b29f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9144b2a5, 66 },
    { 0x16d4b2a5, 23 },
    { 0xb124b2a7, 33 },
    { 0xde54b2a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4b2ac, 90 },
    { 0xbd04b2ad, 90 },
    { 0xf7f4b2ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7674b2b2, 90 },
    { 0xd324b2b3, 84 },
    { 0x5e14b2b4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d74b2bc, 61 },
    { 0xcff4b2bc, 85 },
    { 0x6cf4b2be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad4b2c2, 69 },
    { 0x75e4b2c3, 85 },
    { 0, 0 },
    { 0x784b2c5, 91 },
    { 0xec24b2c5, 98 },
    { 0xcb64b2c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d74b2cb, 90 },
    { 0xe474b2cc, 33 },
    { 0xd524b2cc, 43 },
    { 0, 0 },
    { 0x2c34b2cf, 93 },
    { 0x42b4b2d0, 33 },
    { 0, 0 },
    { 0x1324b2d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xea54b2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb194b2df, 73 },
    { 0x8b94b2df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b2e4, 87 },
    { 0xa184b2e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f54b2e8, 85 },
    { 0x4574b2e8, 98 },
    { 0x93e4b2ea, 98 },
    { 0, 0 },
    { 0xca24b2ec, 98 },
    { 0x224b2ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c94b2f2, 95 },
    { 0xc304b2f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f04b300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b84b303, 76 },
    { 0xd234b304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34b307, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2624b30c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4b314, 95 },
    { 0xf054b315, 33 },
    { 0x2eb4b314, 93 },
    { 0x8e54b317, 98 },
    { 0xdab4b318, 100 },
    { 0xcdd4b319, 91 },
    { 0, 0 },
    { 0xb804b31b, 23 },
    { 0x67d4b31b, 34 },
    { 0, 0 },
    { 0x5294b31e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4b326, 61 },
    { 0xfe44b327, 98 },
    { 0xc674b328, 98 },
    { 0x2d94b328, 76 },
    { 0x64b4b32a, 91 },
    { 0x2c74b32b, 100 },
    { 0xa164b329, 76 },
    { 0x90a4b32d, 90 },
    { 0x59f4b32c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2634b339, 61 },
    { 0x1514b33a, 76 },
    { 0x1354b33a, 48 },
    { 0x4534b33c, 36 },
    { 0xacd4b33c, 33 },
    { 0xda34b33c, 33 },
    { 0, 0 },
    { 0x8de4b340, 90 },
    { 0xcfe4b341, 33 },
    { 0, 0 },
    { 0x9574b343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a4b34d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f4b350, 88 },
    { 0x6734b350, 90 },
    { 0xb864b352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e4b358, 23 },
    { 0, 0 },
    { 0xd4d4b35a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f4b35e, 55 },
    { 0x3c64b35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x574b362, 23 },
    { 0, 0 },
    { 0xc3d4b364, 91 },
    { 0x27d4b365, 90 },
    { 0x40e4b365, 33 },
    { 0, 0 },
    { 0x8724b368, 61 },
    { 0x8d84b369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd524b36c, 61 },
    { 0x58f4b36d, 61 },
    { 0xbd74b36e, 78 },
    { 0x6e34b36d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc634b37b, 99 },
    { 0, 0 },
    { 0x534b37d, 23 },
    { 0, 0 },
    { 0xa544b37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b384, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde4b387, 61 },
    { 0xaca4b387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa714b38c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2894b390, 33 },
    { 0x8fa4b391, 84 },
    { 0, 0 },
    { 0x62d4b393, 40 },
    { 0xc894b394, 61 },
    { 0, 0 },
    { 0xbf14b396, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4b39d, 91 },
    { 0x1e14b39e, 33 },
    { 0, 0 },
    { 0x8f44b3a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9494b3a6, 77 },
    { 0x244b3a6, 55 },
    { 0xb1f4b3a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4b3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e4b3af, 98 },
    { 0x36d4b3b0, 90 },
    { 0x8684b3b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214b3b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb34b3bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4b3bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4654b3c6, 52 },
    { 0, 0 },
    { 0xac84b3c8, 34 },
    { 0, 0 },
    { 0xd694b3ca, 100 },
    { 0x69b4b3cb, 90 },
    { 0xfcc4b3cc, 59 },
    { 0x3784b3cd, 33 },
    { 0x27b4b3ce, 98 },
    { 0xaf64b3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c4b3d3, 84 },
    { 0x45a4b3d4, 23 },
    { 0x8774b3d5, 95 },
    { 0x24f4b3d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884b3d9, 87 },
    { 0, 0 },
    { 0xcd04b3db, 61 },
    { 0x31e4b3dc, 56 },
    { 0x84e4b3dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d4b3e4, 61 },
    { 0, 0 },
    { 0xbb24b3e6, 80 },
    { 0, 0 },
    { 0xa264b3e8, 61 },
    { 0, 0 },
    { 0x5d64b3ea, 43 },
    { 0, 0 },
    { 0xdfa4b3ec, 33 },
    { 0xab64b3ed, 95 },
    { 0xf534b3ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa654b3f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a64b3f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x804b3f9, 33 },
    { 0xdc14b3f9, 61 },
    { 0xfed4b3fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54b401, 61 },
    { 0xd434b402, 88 },
    { 0x1e44b403, 61 },
    { 0x5994b401, 23 },
    { 0x6924b405, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d44b40a, 98 },
    { 0x35d4b40b, 33 },
    { 0, 0 },
    { 0x8454b40d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd4b410, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f4b413, 93 },
    { 0x4ee4b414, 76 },
    { 0x1914b415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d4b419, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e4b41f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f4b422, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8784b425, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd724b42a, 23 },
    { 0, 0 },
    { 0x10b4b42c, 46 },
    { 0x5f24b42c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da4b432, 77 },
    { 0, 0 },
    { 0xfc64b434, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74b43c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24b43f, 34 },
    { 0, 0 },
    { 0x7734b441, 23 },
    { 0x39d4b442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064b448, 99 },
    { 0x2464b448, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc604b44c, 55 },
    { 0, 0 },
    { 0x7074b44e, 61 },
    { 0x1a44b44f, 23 },
    { 0xb674b450, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b4b456, 61 },
    { 0xe294b457, 36 },
    { 0x8724b458, 23 },
    { 0xa494b457, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54b460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4b465, 91 },
    { 0xbb24b466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5584b46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814b472, 33 },
    { 0, 0 },
    { 0x6e04b474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f4b478, 100 },
    { 0xad04b479, 84 },
    { 0x6934b479, 93 },
    { 0x58a4b478, 23 },
    { 0xec4b47b, 61 },
    { 0x9134b47d, 46 },
    { 0, 0 },
    { 0x94a4b47f, 33 },
    { 0x7714b47f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94b483, 66 },
    { 0x93c4b483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2224b489, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb644b48c, 43 },
    { 0x5f44b48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944b491, 33 },
    { 0x2dd4b492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2014b495, 99 },
    { 0x9f24b496, 65 },
    { 0, 0 },
    { 0x5be4b498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34b49f, 61 },
    { 0x84c4b4a0, 34 },
    { 0x18e4b4a1, 33 },
    { 0x3684b49f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554b4a8, 100 },
    { 0, 0 },
    { 0x96c4b4aa, 76 },
    { 0xad84b4aa, 61 },
    { 0x5ca4b4ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb24b4b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca54b4b5, 17 },
    { 0x84c4b4b5, 98 },
    { 0x19c4b4b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd74b4bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54b4c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494b4c5, 90 },
    { 0x5774b4c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd34b4cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2584b4d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9154b4d4, 43 },
    { 0x3774b4d5, 52 },
    { 0xfd54b4d6, 34 },
    { 0x8e74b4d7, 33 },
    { 0x9a64b4d8, 33 },
    { 0x5b44b4d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5474b4df, 77 },
    { 0x45f4b4df, 52 },
    { 0, 0 },
    { 0x44b4b4e2, 61 },
    { 0x9364b4e2, 81 },
    { 0xadc4b4e4, 95 },
    { 0xacc4b4e4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8194b4ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4b4ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7234b4f2, 76 },
    { 0x4f54b4f3, 91 },
    { 0, 0 },
    { 0x32f4b4f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1084b4f9, 23 },
    { 0x6914b4fa, 52 },
    { 0xf5c4b4fb, 77 },
    { 0xd884b4fc, 93 },
    { 0xad34b4fd, 61 },
    { 0xaf84b4fb, 76 },
    { 0x8814b4fe, 32 },
    { 0, 0 },
    { 0x4434b501, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3104b507, 88 },
    { 0xc914b508, 23 },
    { 0x2144b507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24b50c, 95 },
    { 0xfac4b50c, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a4b511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb4b514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34b517, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b4b51c, 33 },
    { 0x60f4b51d, 61 },
    { 0x3ce4b51c, 84 },
    { 0, 0 },
    { 0xa0a4b520, 78 },
    { 0xa064b521, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674b524, 90 },
    { 0xe184b525, 65 },
    { 0x15f4b525, 33 },
    { 0x294b525, 67 },
    { 0x644b528, 46 },
    { 0xf4f4b529, 90 },
    { 0, 0 },
    { 0x954b52b, 60 },
    { 0, 0 },
    { 0x23f4b52d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4964b531, 98 },
    { 0x5684b532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e94b535, 95 },
    { 0x5c74b535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a14b53a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b34b53e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b541, 52 },
    { 0xff34b541, 90 },
    { 0x16b4b542, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564b546, 44 },
    { 0x3054b547, 46 },
    { 0xc654b548, 77 },
    { 0xe2c4b546, 93 },
    { 0, 0 },
    { 0xc2c4b54b, 98 },
    { 0x3aa4b54c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334b551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e4b555, 61 },
    { 0, 0 },
    { 0x4484b557, 93 },
    { 0x7744b557, 66 },
    { 0x1974b559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8624b569, 93 },
    { 0, 0 },
    { 0x64c4b56b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d4b571, 85 },
    { 0, 0 },
    { 0x5ad4b573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9024b579, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab4b57e, 34 },
    { 0, 0 },
    { 0xccd4b580, 61 },
    { 0x98e4b580, 46 },
    { 0xc834b582, 20 },
    { 0xbb64b582, 33 },
    { 0xf794b584, 47 },
    { 0, 0 },
    { 0x21e4b586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa4b58b, 66 },
    { 0, 0 },
    { 0xc704b58d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8374b594, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b4b598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd544b5a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a64b5ad, 55 },
    { 0x5a84b5ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44b5b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734b5b5, 90 },
    { 0xf4d4b5b5, 33 },
    { 0, 0 },
    { 0xe9e4b5b8, 95 },
    { 0x7e34b5b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4b5c1, 90 },
    { 0xbeb4b5c2, 33 },
    { 0, 0 },
    { 0xd064b5c4, 81 },
    { 0x9bd4b5c4, 55 },
    { 0, 0 },
    { 0xe6d4b5c7, 98 },
    { 0, 0 },
    { 0x4b74b5c9, 90 },
    { 0x8ee4b5c9, 75 },
    { 0, 0 },
    { 0x7484b5cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab4b5d1, 98 },
    { 0x52d4b5d2, 99 },
    { 0xfc14b5d3, 91 },
    { 0xe54b5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c74b5d9, 90 },
    { 0xe264b5d9, 95 },
    { 0xcca4b5db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc324b5e5, 52 },
    { 0x7244b5e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74b5ea, 61 },
    { 0, 0 },
    { 0x71d4b5ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34b5f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da4b5fb, 95 },
    { 0x29d4b5fb, 33 },
    { 0x42d4b5fd, 61 },
    { 0x5064b5fb, 90 },
    { 0x68c4b5fd, 90 },
    { 0, 0 },
    { 0x6e74b601, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd334b607, 98 },
    { 0x1404b608, 93 },
    { 0x62c4b609, 91 },
    { 0x1d34b60a, 23 },
    { 0x1984b607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff34b60f, 33 },
    { 0, 0 },
    { 0x844b611, 87 },
    { 0xf194b612, 95 },
    { 0, 0 },
    { 0xd264b614, 85 },
    { 0xcce4b615, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c64b618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194b620, 98 },
    { 0, 0 },
    { 0xbee4b622, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674b626, 91 },
    { 0xbe44b626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc684b62c, 19 },
    { 0x5414b62d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7054b630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8764b635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5314b638, 81 },
    { 0x4354b639, 98 },
    { 0xf944b63a, 90 },
    { 0x68c4b639, 33 },
    { 0x3814b63c, 61 },
    { 0x1a4b63d, 61 },
    { 0x9804b639, 46 },
    { 0x88e4b639, 66 },
    { 0xe634b63d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4b644, 98 },
    { 0, 0 },
    { 0x7a14b646, 95 },
    { 0x9434b646, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca4b64d, 56 },
    { 0xe194b64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94b651, 78 },
    { 0, 0 },
    { 0xda4b653, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b4b656, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f4b659, 90 },
    { 0x3a54b65a, 34 },
    { 0, 0 },
    { 0x3214b65c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4b661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe654b665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4b66e, 91 },
    { 0xa594b66f, 98 },
    { 0xfca4b670, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64b676, 76 },
    { 0, 0 },
    { 0xa2e4b678, 98 },
    { 0, 0 },
    { 0x82e4b67a, 23 },
    { 0, 0 },
    { 0xf544b67c, 23 },
    { 0x1b14b67c, 95 },
    { 0xf104b67e, 66 },
    { 0, 0 },
    { 0xf594b680, 34 },
    { 0x2664b681, 90 },
    { 0xf414b681, 98 },
    { 0, 0 },
    { 0xc114b684, 90 },
    { 0xd004b684, 76 },
    { 0x57c4b686, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf924b689, 33 },
    { 0, 0 },
    { 0x7d04b68b, 85 },
    { 0xa3e4b68b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba4b68f, 90 },
    { 0x5064b68f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b4b695, 76 },
    { 0xf6d4b695, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e4b69a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4b69f, 78 },
    { 0x9264b6a0, 91 },
    { 0x3a54b6a0, 61 },
    { 0, 0 },
    { 0x71b4b6a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd4b6aa, 95 },
    { 0, 0 },
    { 0x3404b6ac, 33 },
    { 0, 0 },
    { 0x4504b6ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9514b6b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94b6b4, 61 },
    { 0x25f4b6b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd4b6b8, 93 },
    { 0x4ee4b6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd074b6be, 90 },
    { 0, 0 },
    { 0x7854b6c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f4b6c8, 90 },
    { 0x1004b6c9, 67 },
    { 0x3194b6ca, 90 },
    { 0, 0 },
    { 0x9544b6cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b6d4, 65 },
    { 0, 0 },
    { 0x8ef4b6d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa014b6d9, 94 },
    { 0xb8c4b6da, 95 },
    { 0xed24b6da, 55 },
    { 0x5014b6d9, 95 },
    { 0xb5c4b6dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d4b6e3, 76 },
    { 0xb604b6e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9534b6e8, 61 },
    { 0, 0 },
    { 0x6ab4b6ea, 98 },
    { 0, 0 },
    { 0xbdc4b6ec, 33 },
    { 0, 0 },
    { 0x7934b6ee, 61 },
    { 0xde34b6ef, 90 },
    { 0x4cf4b6f0, 69 },
    { 0x1bb4b6f1, 95 },
    { 0x47a4b6f0, 98 },
    { 0x1094b6f3, 34 },
    { 0x3d54b6ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4734b6fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f4b6fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5004b703, 95 },
    { 0, 0 },
    { 0xdf94b705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc404b708, 93 },
    { 0, 0 },
    { 0xbdc4b70a, 23 },
    { 0, 0 },
    { 0x54c4b70c, 99 },
    { 0x8084b70d, 98 },
    { 0, 0 },
    { 0x3784b70f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e4b712, 90 },
    { 0x1314b713, 88 },
    { 0, 0 },
    { 0xe894b715, 17 },
    { 0xfb74b716, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14b719, 98 },
    { 0, 0 },
    { 0xaa34b71b, 81 },
    { 0, 0 },
    { 0x7594b71d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e4b721, 93 },
    { 0, 0 },
    { 0x13e4b723, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f4b729, 60 },
    { 0, 0 },
    { 0xc7e4b72b, 95 },
    { 0x824b72b, 90 },
    { 0, 0 },
    { 0x27e4b72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d4b731, 46 },
    { 0, 0 },
    { 0xe2a4b733, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7194b738, 93 },
    { 0xa9d4b739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2884b73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294b742, 33 },
    { 0xeb54b742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa334b746, 34 },
    { 0, 0 },
    { 0x71c4b748, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3884b74c, 90 },
    { 0, 0 },
    { 0xef24b74e, 34 },
    { 0x2d44b74f, 95 },
    { 0, 0 },
    { 0x8d64b751, 77 },
    { 0xc3f4b752, 90 },
    { 0x6a04b753, 90 },
    { 0x9154b751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4094b75a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54b75d, 33 },
    { 0xd164b75d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b764, 61 },
    { 0x7684b765, 90 },
    { 0, 0 },
    { 0x36f4b767, 33 },
    { 0xf244b768, 23 },
    { 0x6374b768, 61 },
    { 0, 0 },
    { 0xa904b76b, 85 },
    { 0x8e74b76c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd584b76f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214b778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d4b77c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234b780, 65 },
    { 0, 0 },
    { 0xe544b782, 61 },
    { 0x8c74b783, 23 },
    { 0x7674b784, 90 },
    { 0, 0 },
    { 0x1534b786, 23 },
    { 0, 0 },
    { 0x30b4b788, 61 },
    { 0xa34b789, 76 },
    { 0x3ca4b78a, 23 },
    { 0x93e4b78b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b44b790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b793, 34 },
    { 0x2334b793, 98 },
    { 0xbdd4b795, 95 },
    { 0xb024b796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa474b79a, 91 },
    { 0, 0 },
    { 0x8d4b79c, 90 },
    { 0, 0 },
    { 0xf6c4b79e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4b7a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4b7a7, 93 },
    { 0x3094b7a8, 75 },
    { 0xd304b7a9, 87 },
    { 0x3bd4b7aa, 98 },
    { 0, 0 },
    { 0x2d4b7ac, 34 },
    { 0xc364b7ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4b7b0, 34 },
    { 0, 0 },
    { 0xaa14b7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa824b7b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade4b7ba, 23 },
    { 0x33a4b7ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e4b7be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14b7c1, 23 },
    { 0, 0 },
    { 0xd14b7c3, 23 },
    { 0, 0 },
    { 0x4c84b7c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7054b7c8, 95 },
    { 0, 0 },
    { 0x94e4b7ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b4b7ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xef84b7d1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4b7d6, 95 },
    { 0x8254b7d6, 33 },
    { 0x5574b7d6, 23 },
    { 0x1784b7d9, 33 },
    { 0x51d4b7da, 55 },
    { 0x2544b7db, 66 },
    { 0x5804b7dc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94b7df, 55 },
    { 0x9e4b7e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b4b7e4, 23 },
    { 0x25b4b7e5, 66 },
    { 0x13c4b7e4, 98 },
    { 0x5044b7e7, 95 },
    { 0xbbb4b7e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b84b7eb, 61 },
    { 0, 0 },
    { 0x7d94b7ed, 95 },
    { 0x94f4b7ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f4b7f4, 33 },
    { 0x5b44b7f5, 61 },
    { 0x3e94b7f4, 95 },
    { 0xe0e4b7f6, 34 },
    { 0, 0 },
    { 0xec84b7f9, 40 },
    { 0xb74b7fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab4b7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad4b803, 55 },
    { 0xbac4b804, 95 },
    { 0x6204b803, 76 },
    { 0x92f4b804, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad4b810, 93 },
    { 0, 0 },
    { 0x7f54b812, 23 },
    { 0xb8b4b813, 88 },
    { 0x8de4b813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6934b818, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c4b81d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca64b820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044b82b, 84 },
    { 0xfe54b82c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf14b831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1554b835, 95 },
    { 0xe034b836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064b839, 23 },
    { 0xf3d4b839, 90 },
    { 0xad54b83b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534b83f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee4b842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f4b846, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe714b84b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4b84f, 81 },
    { 0, 0 },
    { 0x7664b851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4b854, 90 },
    { 0xbb84b855, 17 },
    { 0x714b856, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa534b859, 52 },
    { 0xb534b85a, 98 },
    { 0x20d4b85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394b85f, 33 },
    { 0xdc84b85f, 55 },
    { 0x27b4b860, 34 },
    { 0x2fb4b860, 61 },
    { 0xf8f4b863, 95 },
    { 0xafc4b863, 65 },
    { 0, 0 },
    { 0x52f4b866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d4b86a, 61 },
    { 0, 0 },
    { 0x73e4b86c, 76 },
    { 0xc8b4b86d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b870, 34 },
    { 0xb8a4b871, 95 },
    { 0xd714b870, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6424b875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4b878, 93 },
    { 0xf254b879, 99 },
    { 0x8304b879, 75 },
    { 0x9934b87b, 69 },
    { 0x73f4b87b, 61 },
    { 0xc754b87c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4b880, 33 },
    { 0xe424b881, 87 },
    { 0x34d4b882, 100 },
    { 0x694b882, 97 },
    { 0xd9a4b884, 85 },
    { 0xc7f4b885, 61 },
    { 0xd604b886, 40 },
    { 0xda34b886, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a4b88a, 52 },
    { 0x6e34b88b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1114b890, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14b898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb4b89e, 61 },
    { 0, 0 },
    { 0xbbe4b8a0, 33 },
    { 0, 0 },
    { 0x51f4b8a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xece4b8a5, 90 },
    { 0x9b24b8a6, 96 },
    { 0x8f44b8a5, 95 },
    { 0x2124b8a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9384b8ac, 61 },
    { 0x25e4b8ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb14b8b3, 85 },
    { 0, 0 },
    { 0xea94b8b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4b8ba, 85 },
    { 0x814b8ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7584b8c3, 23 },
    { 0x37e4b8c4, 61 },
    { 0x3ad4b8c3, 43 },
    { 0xae4b8c6, 98 },
    { 0, 0 },
    { 0x4c54b8c8, 52 },
    { 0, 0 },
    { 0xcfc4b8ca, 101 },
    { 0x8a94b8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8ce, 43 },
    { 0xd2a4b8cf, 100 },
    { 0, 0 },
    { 0xd274b8d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1224b8d5, 43 },
    { 0x7614b8d6, 95 },
    { 0xa634b8d7, 90 },
    { 0x45a4b8d8, 90 },
    { 0x8c74b8d9, 43 },
    { 0x7294b8d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a34b8dd, 76 },
    { 0x8214b8dd, 90 },
    { 0x4a34b8df, 81 },
    { 0x6bf4b8e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc414b8e9, 95 },
    { 0x1a24b8ea, 23 },
    { 0x3bd4b8eb, 23 },
    { 0xcf04b8ec, 34 },
    { 0x3764b8ed, 87 },
    { 0x8a34b8eb, 61 },
    { 0xda74b8ec, 23 },
    { 0x1474b8f0, 91 },
    { 0xf6f4b8ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d4b8f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2184b8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd4b8ff, 61 },
    { 0x1fe4b900, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34b903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4b906, 43 },
    { 0x37f4b907, 85 },
    { 0xf5f4b906, 99 },
    { 0xc154b907, 90 },
    { 0x68f4b906, 91 },
    { 0xc6e4b90b, 100 },
    { 0x3e04b90c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c44b90f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b912, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c4b91a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d04b91d, 76 },
    { 0, 0 },
    { 0x7b44b91f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864b927, 98 },
    { 0x4174b928, 94 },
    { 0xa7b4b929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c14b92f, 43 },
    { 0xf8c4b930, 88 },
    { 0x7d44b931, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7324b934, 66 },
    { 0xc94b934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b4b93a, 100 },
    { 0x3514b93b, 90 },
    { 0x7ed4b93c, 90 },
    { 0, 0 },
    { 0xbfe4b93e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234b942, 61 },
    { 0x26e4b943, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534b947, 23 },
    { 0xaac4b947, 91 },
    { 0, 0 },
    { 0xf74b94a, 17 },
    { 0, 0 },
    { 0xf6b4b94c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb874b950, 43 },
    { 0x9344b950, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe934b956, 76 },
    { 0x6b74b956, 40 },
    { 0, 0 },
    { 0xc164b959, 98 },
    { 0, 0 },
    { 0x9bc4b95b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4b95e, 61 },
    { 0, 0 },
    { 0xac44b960, 98 },
    { 0x9ea4b960, 55 },
    { 0, 0 },
    { 0x95b4b963, 91 },
    { 0x7454b964, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4b969, 78 },
    { 0, 0 },
    { 0x4614b96b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534b96e, 34 },
    { 0xe434b96f, 55 },
    { 0x6b04b96e, 36 },
    { 0x2134b971, 76 },
    { 0xe1b4b972, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4984b976, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a4b97c, 84 },
    { 0, 0 },
    { 0xd1f4b97e, 76 },
    { 0, 0 },
    { 0xfd84b980, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b4b983, 55 },
    { 0, 0 },
    { 0x85b4b985, 93 },
    { 0x6344b986, 33 },
    { 0x9494b987, 66 },
    { 0xf764b986, 65 },
    { 0xee24b989, 66 },
    { 0xa794b989, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e4b991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b994, 76 },
    { 0xdac4b995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c4b99d, 33 },
    { 0x5444b99d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f34b9a3, 55 },
    { 0, 0 },
    { 0x5814b9a5, 76 },
    { 0xd964b9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b4b9aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44b9ad, 46 },
    { 0x2954b9ad, 52 },
    { 0x38e4b9ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc4b9b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b9bc, 23 },
    { 0xe914b9bd, 33 },
    { 0x6be4b9be, 88 },
    { 0xeff4b9bf, 90 },
    { 0xcc84b9c0, 78 },
    { 0x49b4b9bf, 69 },
    { 0xb754b9c2, 43 },
    { 0x8994b9c3, 23 },
    { 0xe4d4b9c4, 80 },
    { 0xa54b9be, 55 },
    { 0, 0 },
    { 0x1714b9c7, 40 },
    { 0x8274b9c7, 23 },
    { 0, 0 },
    { 0x25d4b9ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f4b9cd, 76 },
    { 0x7ec4b9ce, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a44b9dd, 78 },
    { 0xe854b9de, 33 },
    { 0x7c24b9df, 68 },
    { 0xdae4b9dd, 33 },
    { 0x9744b9e1, 46 },
    { 0, 0 },
    { 0x43d4b9e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94b9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e94b9eb, 87 },
    { 0, 0 },
    { 0x8854b9ed, 23 },
    { 0x96d4b9ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74b9f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b9fa, 90 },
    { 0x5fc4b9fa, 91 },
    { 0xdc64b9fc, 90 },
    { 0xe454b9fa, 43 },
    { 0xc344b9fe, 33 },
    { 0x8684b9ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb144ba03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344ba07, 90 },
    { 0, 0 },
    { 0xd804ba09, 95 },
    { 0xcec4ba0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4ba0e, 66 },
    { 0, 0 },
    { 0x5154ba10, 34 },
    { 0x8eb4ba11, 46 },
    { 0x1804ba12, 23 },
    { 0xa5d4ba12, 75 },
    { 0x1fb4ba14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14ba1d, 95 },
    { 0x33d4ba1d, 85 },
    { 0xdd74ba1f, 98 },
    { 0, 0 },
    { 0xaa74ba21, 23 },
    { 0, 0 },
    { 0x53c4ba23, 23 },
    { 0xb984ba24, 90 },
    { 0xa064ba23, 78 },
    { 0xb7f4ba26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4ba2b, 46 },
    { 0xf8c4ba2b, 98 },
    { 0x8004ba2d, 33 },
    { 0x5d14ba2e, 95 },
    { 0, 0 },
    { 0xb774ba30, 43 },
    { 0, 0 },
    { 0x9004ba32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6294ba38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7444ba3f, 23 },
    { 0, 0 },
    { 0x11d4ba41, 95 },
    { 0x5874ba42, 95 },
    { 0x42b4ba43, 90 },
    { 0, 0 },
    { 0xea14ba45, 98 },
    { 0x87b4ba46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c54ba4c, 55 },
    { 0, 0 },
    { 0x8124ba4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a4ba51, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94ba57, 55 },
    { 0x7ff4ba58, 46 },
    { 0x6b64ba59, 100 },
    { 0x5ce4ba58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874ba5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b4ba62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24ba67, 33 },
    { 0, 0 },
    { 0xd3c4ba69, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa374ba71, 90 },
    { 0xafe4ba72, 61 },
    { 0xa164ba73, 95 },
    { 0xeb14ba72, 95 },
    { 0xd1b4ba72, 90 },
    { 0xf54ba76, 67 },
    { 0x8f64ba77, 99 },
    { 0x6cf4ba71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae4ba7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944ba7f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494ba87, 95 },
    { 0x5714ba88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc4ba8e, 87 },
    { 0x4f54ba8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ba92, 61 },
    { 0x2f44ba92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8114ba96, 44 },
    { 0x7964ba97, 95 },
    { 0x4844ba98, 23 },
    { 0x6d4ba99, 84 },
    { 0x1844ba96, 90 },
    { 0xeab4ba9a, 98 },
    { 0x2584ba9c, 33 },
    { 0, 0 },
    { 0xb904ba9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a94baa4, 61 },
    { 0, 0 },
    { 0xce14baa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1284baa9, 90 },
    { 0x88a4baaa, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf34bab8, 43 },
    { 0x50a4bab9, 12 },
    { 0xb454baba, 95 },
    { 0xf9c4baba, 61 },
    { 0x60f4babb, 61 },
    { 0x7a74babc, 61 },
    { 0xd854baba, 43 },
    { 0x46e4babf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2484bac2, 46 },
    { 0xce24bac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4bac8, 98 },
    { 0, 0 },
    { 0x6244baca, 76 },
    { 0x4c44bacb, 98 },
    { 0, 0 },
    { 0x1264bacd, 55 },
    { 0x2f44bacd, 46 },
    { 0x9694bacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe4bad5, 95 },
    { 0, 0 },
    { 0x79f4bad7, 33 },
    { 0xb424bad8, 98 },
    { 0xc5f4bad7, 33 },
    { 0xf4f4bad8, 91 },
    { 0xe2d4badb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94bade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4bae3, 90 },
    { 0x6564bae4, 93 },
    { 0x6594bae5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4baeb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324baee, 90 },
    { 0xc9d4baee, 88 },
    { 0x1d34baee, 52 },
    { 0xfea4baf1, 66 },
    { 0xb34baef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd134baf7, 81 },
    { 0, 0 },
    { 0xd444baf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694bb00, 52 },
    { 0, 0 },
    { 0x77a4bb02, 61 },
    { 0xbb14bb03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4224bb06, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd404bb09, 33 },
    { 0x9994bb0a, 33 },
    { 0x8554bb0b, 76 },
    { 0xbac4bb0b, 98 },
    { 0x3094bb0c, 90 },
    { 0x2104bb0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4bb11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac34bb14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b44bb19, 74 },
    { 0xfa64bb1a, 55 },
    { 0x1724bb1b, 61 },
    { 0, 0 },
    { 0x95a4bb1d, 66 },
    { 0xd804bb1e, 68 },
    { 0, 0 },
    { 0xee44bb20, 69 },
    { 0xd254bb20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc74bb29, 90 },
    { 0x5b4bb2a, 90 },
    { 0xefa4bb2b, 98 },
    { 0xfb24bb29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4bb30, 98 },
    { 0, 0 },
    { 0x38a4bb32, 33 },
    { 0x87c4bb33, 75 },
    { 0xd94bb32, 65 },
    { 0, 0 },
    { 0xde34bb36, 33 },
    { 0x7f64bb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4bb3d, 33 },
    { 0xace4bb3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1814bb41, 33 },
    { 0x7774bb41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f34bb47, 98 },
    { 0, 0 },
    { 0xfa94bb49, 46 },
    { 0x9454bb4a, 98 },
    { 0, 0 },
    { 0xc4b4bb4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b4bb50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc44bb57, 84 },
    { 0xa8d4bb58, 24 },
    { 0xf764bb58, 100 },
    { 0, 0 },
    { 0xfda4bb5b, 98 },
    { 0xe284bb5b, 43 },
    { 0xdab4bb5d, 33 },
    { 0x5444bb5e, 61 },
    { 0, 0 },
    { 0xbb84bb60, 90 },
    { 0x2984bb60, 90 },
    { 0x5864bb61, 90 },
    { 0x674bb63, 95 },
    { 0x9e54bb61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b4bb67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee44bb6a, 46 },
    { 0x3004bb6b, 61 },
    { 0x36f4bb6c, 66 },
    { 0, 0 },
    { 0xda34bb6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94bb73, 95 },
    { 0, 0 },
    { 0x8484bb75, 98 },
    { 0, 0 },
    { 0xc604bb77, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a44bb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5924bb81, 61 },
    { 0x7734bb81, 23 },
    { 0xc164bb83, 23 },
    { 0xfb34bb82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d4bb8a, 84 },
    { 0, 0 },
    { 0x6614bb8c, 76 },
    { 0xe7f4bb8d, 34 },
    { 0xc114bb8e, 52 },
    { 0xd834bb8e, 33 },
    { 0xd5c4bb8c, 98 },
    { 0xee24bb8c, 90 },
    { 0, 0 },
    { 0x1244bb93, 34 },
    { 0, 0 },
    { 0x2734bb95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694bb99, 99 },
    { 0x5b94bb99, 98 },
    { 0x3964bb99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4bb9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d4bba3, 98 },
    { 0x3df4bba3, 68 },
    { 0xc4a4bba3, 90 },
    { 0x4c4bba6, 61 },
    { 0x6e74bba7, 34 },
    { 0x5054bba7, 23 },
    { 0x3c04bba9, 78 },
    { 0xc6b4bba9, 61 },
    { 0xbf74bbab, 95 },
    { 0x6fb4bba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa4bbaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f4bbb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4bbb6, 91 },
    { 0x4ff4bbb7, 61 },
    { 0, 0 },
    { 0xca54bbb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74bbc6, 61 },
    { 0, 0 },
    { 0xa2f4bbc8, 52 },
    { 0, 0 },
    { 0x91c4bbca, 61 },
    { 0, 0 },
    { 0x8d84bbcc, 98 },
    { 0xd564bbcd, 95 },
    { 0, 0 },
    { 0x9144bbcf, 90 },
    { 0, 0 },
    { 0x17e4bbd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b4bbd6, 98 },
    { 0xa0c4bbd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064bbe6, 34 },
    { 0xe24bbe6, 90 },
    { 0x8704bbe8, 93 },
    { 0xea14bbe8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b24bbf0, 61 },
    { 0xcaa4bbf1, 55 },
    { 0xaa94bbf2, 95 },
    { 0xb944bbf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb4bbf8, 81 },
    { 0x45b4bbf9, 42 },
    { 0xa474bbf9, 91 },
    { 0x54d4bbf9, 23 },
    { 0x604bbfc, 98 },
    { 0x3394bbfd, 34 },
    { 0x2374bbfe, 46 },
    { 0x9af4bbfc, 90 },
    { 0xcc84bc00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474bc06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7644bc0a, 66 },
    { 0xb434bc0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda4bc10, 87 },
    { 0x7cd4bc11, 99 },
    { 0x84a4bc12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd014bc15, 85 },
    { 0xac54bc15, 95 },
    { 0x2134bc17, 90 },
    { 0x7134bc15, 17 },
    { 0x4514bc16, 90 },
    { 0x52f4bc1a, 90 },
    { 0x61d4bc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5234bc1f, 90 },
    { 0, 0 },
    { 0x5264bc21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4bc26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e4bc29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4bc2c, 35 },
    { 0, 0 },
    { 0x4f14bc2e, 61 },
    { 0xe364bc2f, 8 },
    { 0x6b44bc30, 34 },
    { 0x4ce4bc31, 34 },
    { 0, 0 },
    { 0x8bc4bc33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa544bc3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174bc3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b4bc42, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f4bc46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8214bc53, 99 },
    { 0, 0 },
    { 0x7754bc55, 89 },
    { 0x14d4bc56, 23 },
    { 0xd684bc57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c4bc5d, 82 },
    { 0x9e74bc5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6354bc62, 76 },
    { 0, 0 },
    { 0xc7c4bc64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4bc68, 93 },
    { 0x6934bc68, 55 },
    { 0xbc34bc69, 61 },
    { 0xc144bc6b, 34 },
    { 0, 0 },
    { 0x98b4bc6d, 95 },
    { 0x7104bc6e, 87 },
    { 0, 0 },
    { 0x5434bc70, 96 },
    { 0, 0 },
    { 0xe334bc72, 61 },
    { 0x55e4bc73, 46 },
    { 0x6764bc73, 61 },
    { 0x3ad4bc75, 90 },
    { 0xfa34bc76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a54bc79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7374bc7c, 97 },
    { 0xef84bc7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e44bc83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a34bc88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f4bc8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3174bc91, 69 },
    { 0xe214bc92, 98 },
    { 0xc34bc92, 33 },
    { 0x43b4bc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4bc99, 90 },
    { 0x1954bc99, 95 },
    { 0x2204bc9b, 67 },
    { 0xdaf4bc9b, 23 },
    { 0x3cb4bc9d, 78 },
    { 0x9084bc9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4bca4, 95 },
    { 0x874bca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd204bca9, 66 },
    { 0x93d4bcaa, 23 },
    { 0x64c4bcab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d24bcb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4bcb3, 80 },
    { 0x3014bcb4, 100 },
    { 0, 0 },
    { 0x9074bcb6, 23 },
    { 0x3104bcb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e04bcbc, 23 },
    { 0, 0 },
    { 0xea94bcbe, 68 },
    { 0x2fe4bcbf, 61 },
    { 0x4764bcc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4bcd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad4bcd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa114bcdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc34bce1, 33 },
    { 0x7bb4bce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf024bce5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6134bcea, 87 },
    { 0xc764bceb, 67 },
    { 0x7a94bcec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf64bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54bcf3, 90 },
    { 0, 0 },
    { 0xdee4bcf5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff4bcf8, 98 },
    { 0x70b4bcf8, 43 },
    { 0, 0 },
    { 0x6254bcfb, 95 },
    { 0, 0 },
    { 0xfa54bcfd, 95 },
    { 0xce54bcfe, 90 },
    { 0x47b4bcff, 75 },
    { 0xc9f4bd00, 55 },
    { 0x75e4bd00, 23 },
    { 0xf434bcfe, 23 },
    { 0xbb34bcff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4bd06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334bd0b, 98 },
    { 0x6674bd0b, 95 },
    { 0, 0 },
    { 0x3844bd0e, 61 },
    { 0x50c4bd0f, 90 },
    { 0x6d44bd0e, 23 },
    { 0, 0 },
    { 0xd244bd12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd4bd18, 95 },
    { 0, 0 },
    { 0xa994bd1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44bd1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e14bd20, 32 },
    { 0, 0 },
    { 0xeab4bd22, 55 },
    { 0x7664bd23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b04bd27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe284bd2c, 90 },
    { 0xdfe4bd2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb384bd34, 55 },
    { 0, 0 },
    { 0x7554bd36, 93 },
    { 0xc84bd36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xef54bd3a, 90 },
    { 0x2bf4bd3b, 98 },
    { 0x9954bd3b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf4bd3f, 23 },
    { 0x25a4bd3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3714bd51, 61 },
    { 0, 0 },
    { 0x72c4bd53, 90 },
    { 0, 0 },
    { 0xef04bd55, 100 },
    { 0xad84bd56, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c84bd5a, 84 },
    { 0xa434bd5a, 95 },
    { 0xa874bd5c, 61 },
    { 0xf014bd5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8334bd66, 93 },
    { 0x6784bd67, 66 },
    { 0xae34bd68, 23 },
    { 0xcc94bd69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe364bd70, 95 },
    { 0xfc84bd70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5084bd75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a44bd78, 33 },
    { 0xba14bd79, 23 },
    { 0xb094bd7a, 65 },
    { 0x9e54bd79, 98 },
    { 0, 0 },
    { 0x6e24bd7d, 99 },
    { 0x1d14bd7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc4bd8b, 85 },
    { 0xcba4bd8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9384bd8f, 36 },
    { 0, 0 },
    { 0xede4bd91, 98 },
    { 0xcae4bd92, 55 },
    { 0x6994bd93, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc914bd97, 90 },
    { 0x37d4bd97, 61 },
    { 0, 0 },
    { 0xfb24bd9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e64bda2, 98 },
    { 0xfad4bda3, 80 },
    { 0, 0 },
    { 0x7d84bda5, 24 },
    { 0, 0 },
    { 0xb914bda7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa514bdaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c4bdad, 76 },
    { 0, 0 },
    { 0xf3e4bdaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b4bdb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334bdb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d04bdbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44bdc0, 90 },
    { 0xf6d4bdc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704bdc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4bdca, 84 },
    { 0, 0 },
    { 0xe3a4bdcc, 23 },
    { 0x6f94bdcd, 61 },
    { 0xa804bdce, 55 },
    { 0x8c14bdcf, 78 },
    { 0x8154bdcf, 23 },
    { 0xd7a4bdd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6914bdd5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d4bdda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd234bddf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3534bde4, 61 },
    { 0x3e44bde5, 98 },
    { 0, 0 },
    { 0xa7c4bde7, 23 },
    { 0xab94bde8, 90 },
    { 0xb1d4bde9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d24bdec, 95 },
    { 0xacd4bded, 76 },
    { 0x41d4bdee, 76 },
    { 0xd964bded, 95 },
    { 0x8704bdf0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf564bdf4, 67 },
    { 0x2e64bdf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614bdf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c4bdfd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54be0b, 46 },
    { 0, 0 },
    { 0x5104be0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd4be11, 33 },
    { 0, 0 },
    { 0x4354be13, 90 },
    { 0, 0 },
    { 0x5154be15, 33 },
    { 0x4b04be16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d4be1c, 90 },
    { 0x93b4be1c, 93 },
    { 0x1e24be1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4be25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f4be2b, 95 },
    { 0x3254be2c, 23 },
    { 0, 0 },
    { 0x51d4be2e, 34 },
    { 0xd424be2f, 34 },
    { 0x874be30, 90 },
    { 0, 0 },
    { 0x9844be32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354be36, 87 },
    { 0x89e4be36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab14be40, 61 },
    { 0xdd64be41, 33 },
    { 0x6e54be40, 90 },
    { 0xd4e4be43, 90 },
    { 0xc4f4be44, 55 },
    { 0x3714be45, 46 },
    { 0x1774be45, 95 },
    { 0x9e44be47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc074be4c, 34 },
    { 0x5c4be4d, 67 },
    { 0, 0 },
    { 0xc634be4f, 76 },
    { 0x4444be50, 61 },
    { 0x3204be51, 90 },
    { 0x4514be52, 33 },
    { 0x7fa4be52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb664be57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf614be5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14be62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4be66, 83 },
    { 0x5704be67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504be6a, 97 },
    { 0x8d04be6b, 66 },
    { 0x8d4be6c, 84 },
    { 0x68d4be6b, 61 },
    { 0x36d4be6e, 90 },
    { 0, 0 },
    { 0xb0e4be70, 61 },
    { 0x86d4be71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c4be74, 23 },
    { 0x4ed4be75, 77 },
    { 0x4a34be75, 66 },
    { 0x3c44be77, 61 },
    { 0x5a24be77, 33 },
    { 0xe104be77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804be7d, 90 },
    { 0xd5a4be7d, 46 },
    { 0xd2b4be7f, 61 },
    { 0x7a84be7f, 87 },
    { 0x2b94be81, 77 },
    { 0xfb54be7f, 87 },
    { 0xe9c4be83, 61 },
    { 0x93b4be84, 34 },
    { 0xa8e4be7d, 98 },
    { 0x1a14be81, 95 },
    { 0x3674be87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9754be90, 76 },
    { 0x2794be91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e24be97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc734bea0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a14bea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864beae, 61 },
    { 0, 0 },
    { 0x2fc4beb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7234bebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc4bec0, 61 },
    { 0xce24bec1, 33 },
    { 0, 0 },
    { 0x8e34bec3, 90 },
    { 0x4384bec4, 90 },
    { 0x5f04bec5, 87 },
    { 0, 0 },
    { 0x9924bec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b4becc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24becf, 23 },
    { 0x9404becf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e74bed4, 33 },
    { 0, 0 },
    { 0xd824bed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5324bedb, 76 },
    { 0xbdd4bedb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14bee6, 88 },
    { 0xd604bee7, 100 },
    { 0xb974bee7, 33 },
    { 0x4cb4bee9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b4bef0, 33 },
    { 0, 0 },
    { 0xc084bef2, 99 },
    { 0, 0 },
    { 0x5cf4bef4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa064beff, 76 },
    { 0xeb34bf00, 23 },
    { 0, 0 },
    { 0xebf4bf02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa204bf08, 95 },
    { 0xf4bf09, 95 },
    { 0x8864bf0a, 98 },
    { 0xabe4bf09, 46 },
    { 0, 0 },
    { 0x8ef4bf0d, 76 },
    { 0xd074bf0d, 61 },
    { 0, 0 },
    { 0x1284bf10, 95 },
    { 0x91a4bf10, 27 },
    { 0, 0 },
    { 0x8ac4bf13, 95 },
    { 0x2b64bf14, 61 },
    { 0, 0 },
    { 0xbfe4bf16, 61 },
    { 0xa24bf17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e64bf1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc474bf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564bf29, 91 },
    { 0x2c14bf2a, 93 },
    { 0, 0 },
    { 0xe194bf2c, 33 },
    { 0x8644bf2d, 76 },
    { 0x4dc4bf2d, 76 },
    { 0x4c34bf2d, 61 },
    { 0xd9d4bf2c, 87 },
    { 0x604bf31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f4bf35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794bf38, 46 },
    { 0xfc14bf38, 55 },
    { 0, 0 },
    { 0x85c4bf3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1344bf42, 34 },
    { 0, 0 },
    { 0xc924bf44, 33 },
    { 0xf434bf44, 85 },
    { 0, 0 },
    { 0x1334bf47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3624bf4a, 52 },
    { 0x5914bf4b, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c4bf51, 33 },
    { 0x2e54bf51, 98 },
    { 0, 0 },
    { 0xa104bf54, 99 },
    { 0, 0 },
    { 0x3ab4bf56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274bf59, 55 },
    { 0x204bf59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7784bf5d, 43 },
    { 0x5864bf5e, 92 },
    { 0xa414bf5f, 95 },
    { 0x75e4bf5f, 76 },
    { 0x7104bf5f, 61 },
    { 0xb7b4bf62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa594bf6a, 81 },
    { 0, 0 },
    { 0x3d44bf6c, 95 },
    { 0, 0 },
    { 0x59f4bf6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654bf74, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4bf77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484bf7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd974bf81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe484bf90, 98 },
    { 0, 0 },
    { 0x19c4bf92, 99 },
    { 0xe944bf93, 93 },
    { 0, 0 },
    { 0xaa74bf95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a4bf9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b4bf9e, 93 },
    { 0, 0 },
    { 0xebf4bfa0, 98 },
    { 0xe014bfa1, 23 },
    { 0xb7d4bfa2, 80 },
    { 0xcc54bfa0, 77 },
    { 0x11c4bfa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4bfa8, 33 },
    { 0, 0 },
    { 0x9154bfaa, 93 },
    { 0, 0 },
    { 0xc5d4bfac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c34bfaf, 91 },
    { 0x1df4bfaf, 61 },
    { 0x1b74bfb1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714bfb5, 78 },
    { 0x8ed4bfb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba04bfbb, 34 },
    { 0x7844bfbb, 55 },
    { 0x5d24bfbd, 67 },
    { 0x54b4bfbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce4bfc3, 66 },
    { 0xbe94bfc4, 91 },
    { 0, 0 },
    { 0xe284bfc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f4bfcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8774bfd0, 34 },
    { 0, 0 },
    { 0x92b4bfd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34bfd5, 76 },
    { 0, 0 },
    { 0x454bfd7, 33 },
    { 0x21a4bfd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4bfdd, 43 },
    { 0x9744bfdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb454bfe8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904bfec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9324bff1, 33 },
    { 0x77a4bff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe954bff6, 87 },
    { 0xe154bff7, 98 },
    { 0x23b4bff8, 90 },
    { 0x5624bff8, 98 },
    { 0, 0 },
    { 0xc354bffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe094c000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d14c004, 90 },
    { 0x5424c005, 90 },
    { 0x6cf4c006, 91 },
    { 0xe364c006, 98 },
    { 0, 0 },
    { 0xe204c009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda4c00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb044c019, 61 },
    { 0xa554c01a, 43 },
    { 0x22d4c01b, 93 },
    { 0x6a14c01c, 85 },
    { 0x70e4c01b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e54c021, 95 },
    { 0x1fb4c022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c94c02a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf14c034, 23 },
    { 0, 0 },
    { 0x59c4c036, 46 },
    { 0xf214c036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d4c03c, 61 },
    { 0x81f4c03c, 95 },
    { 0x7e64c03e, 34 },
    { 0x38c4c03f, 33 },
    { 0xc5a4c03f, 88 },
    { 0x9e54c041, 61 },
    { 0, 0 },
    { 0xe394c043, 43 },
    { 0xe8b4c044, 95 },
    { 0xbb24c044, 76 },
    { 0, 0 },
    { 0xfb24c047, 61 },
    { 0x5f04c047, 61 },
    { 0, 0 },
    { 0x1ee4c04a, 99 },
    { 0xf134c04b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc4c04e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac74c057, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b34c05b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb014c05f, 90 },
    { 0xaf54c060, 33 },
    { 0x6fa4c060, 90 },
    { 0xef64c060, 23 },
    { 0x9c44c063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba14c069, 46 },
    { 0, 0 },
    { 0x5424c06b, 91 },
    { 0xa2e4c06b, 46 },
    { 0x7154c06d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4c070, 93 },
    { 0, 0 },
    { 0xe614c072, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce4c077, 90 },
    { 0, 0 },
    { 0x9044c079, 78 },
    { 0x4b34c07a, 76 },
    { 0, 0 },
    { 0x7684c07c, 98 },
    { 0, 0 },
    { 0xc614c07e, 90 },
    { 0x5044c07f, 33 },
    { 0x4ed4c07f, 33 },
    { 0, 0 },
    { 0xc64c082, 90 },
    { 0x77b4c083, 43 },
    { 0x4724c084, 90 },
    { 0, 0 },
    { 0x9f14c086, 55 },
    { 0x4d54c086, 34 },
    { 0xaa74c088, 55 },
    { 0x99f4c088, 17 },
    { 0xa594c089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe84c095, 55 },
    { 0x69b4c096, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f4c09a, 64 },
    { 0xfa04c09b, 98 },
    { 0x6124c09b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce4c09f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604c0a5, 90 },
    { 0x434c0a5, 23 },
    { 0xaec4c0a7, 94 },
    { 0x3754c0a5, 90 },
    { 0, 0 },
    { 0x5ca4c0aa, 90 },
    { 0xb674c0ab, 90 },
    { 0, 0 },
    { 0x7924c0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d4c0b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc044c0b4, 93 },
    { 0, 0 },
    { 0x7fa4c0b6, 84 },
    { 0xc974c0b6, 90 },
    { 0x5c04c0b6, 93 },
    { 0, 0 },
    { 0x9534c0ba, 33 },
    { 0x2de4c0bb, 95 },
    { 0x1674c0bb, 43 },
    { 0xf484c0ba, 33 },
    { 0, 0 },
    { 0x80c4c0bf, 23 },
    { 0, 0 },
    { 0x40b4c0c1, 66 },
    { 0xec44c0c2, 98 },
    { 0xdb94c0c3, 23 },
    { 0x8684c0c4, 23 },
    { 0x3a04c0c5, 23 },
    { 0x7494c0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c4c0ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc84c0d7, 98 },
    { 0xc294c0d8, 55 },
    { 0xcc44c0d7, 3 },
    { 0, 0 },
    { 0xcea4c0db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a24c0e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd4c0e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5374c0ea, 61 },
    { 0, 0 },
    { 0xaaa4c0ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb14c0f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd44c0f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb74c0f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4c103, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff4c10e, 69 },
    { 0xf7a4c10e, 78 },
    { 0x67d4c10f, 43 },
    { 0x92c4c111, 93 },
    { 0, 0 },
    { 0xa004c113, 90 },
    { 0, 0 },
    { 0x5b4c115, 61 },
    { 0xe3a4c116, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774c119, 23 },
    { 0, 0 },
    { 0x2104c11b, 91 },
    { 0x7f94c11b, 43 },
    { 0x89a4c11b, 23 },
    { 0, 0 },
    { 0x7bd4c11f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a24c123, 90 },
    { 0x1f14c124, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4c127, 90 },
    { 0x58e4c128, 93 },
    { 0xbd64c129, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a74c132, 76 },
    { 0xf074c133, 90 },
    { 0x7494c133, 98 },
    { 0x1184c134, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4814c13a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4c13d, 90 },
    { 0x30f4c13e, 68 },
    { 0x8ee4c13e, 20 },
    { 0x374c140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf704c143, 43 },
    { 0x4b34c143, 91 },
    { 0x4e34c145, 44 },
    { 0x5b64c146, 55 },
    { 0x3ba4c143, 75 },
    { 0, 0 },
    { 0x42d4c149, 33 },
    { 0, 0 },
    { 0x3e34c14b, 23 },
    { 0, 0 },
    { 0x8224c14d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a4c152, 90 },
    { 0, 0 },
    { 0x4414c154, 33 },
    { 0, 0 },
    { 0x7f84c156, 33 },
    { 0x3ec4c157, 33 },
    { 0x5024c157, 55 },
    { 0x2744c158, 33 },
    { 0x3374c15a, 95 },
    { 0x4284c159, 95 },
    { 0, 0 },
    { 0x2694c15d, 55 },
    { 0xf3e4c15e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04c161, 66 },
    { 0xe654c161, 23 },
    { 0, 0 },
    { 0x37e4c164, 65 },
    { 0, 0 },
    { 0x5cd4c166, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd74c169, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6124c170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c175, 90 },
    { 0, 0 },
    { 0x46e4c177, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94c17c, 84 },
    { 0, 0 },
    { 0x96d4c17e, 95 },
    { 0x5584c17f, 95 },
    { 0x3f4c180, 85 },
    { 0, 0 },
    { 0x4c24c182, 93 },
    { 0, 0 },
    { 0x9334c184, 77 },
    { 0x2574c185, 2 },
    { 0x7a44c186, 90 },
    { 0xc114c185, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384c18b, 95 },
    { 0x3784c18c, 61 },
    { 0, 0 },
    { 0x3404c18e, 46 },
    { 0x2904c18e, 95 },
    { 0x22b4c190, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb4c196, 23 },
    { 0x74e4c196, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4c1a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a4c1a6, 34 },
    { 0x73d4c1a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5304c1ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d4c1b1, 76 },
    { 0, 0 },
    { 0x6344c1b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d94c1b7, 90 },
    { 0x6024c1b8, 23 },
    { 0, 0 },
    { 0x1754c1ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874c1bf, 67 },
    { 0xcb24c1c0, 23 },
    { 0x6ee4c1c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b94c1cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4764c1d0, 36 },
    { 0xd794c1d0, 43 },
    { 0xa424c1d2, 23 },
    { 0x5624c1d3, 95 },
    { 0x4034c1d4, 95 },
    { 0x7114c1d5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6654c1d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c1dc, 98 },
    { 0, 0 },
    { 0x7b84c1de, 78 },
    { 0x1d54c1df, 93 },
    { 0x45f4c1de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a4c1e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d4c1e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb54c1ec, 61 },
    { 0x6e24c1ed, 91 },
    { 0x7384c1ed, 61 },
    { 0, 0 },
    { 0xb0f4c1f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d4c1f5, 90 },
    { 0x92e4c1f6, 67 },
    { 0x1394c1f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe24c1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f4c1fd, 98 },
    { 0xec04c1fe, 95 },
    { 0xbb34c1ff, 33 },
    { 0xca44c200, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24c205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c44c20a, 98 },
    { 0x2284c20b, 55 },
    { 0, 0 },
    { 0x1ee4c20d, 55 },
    { 0x7ac4c20e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f4c213, 90 },
    { 0x9274c214, 78 },
    { 0xda24c215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074c21e, 55 },
    { 0xfa94c21f, 46 },
    { 0x5eb4c21e, 61 },
    { 0x3514c221, 61 },
    { 0x85e4c222, 91 },
    { 0xf214c223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1094c229, 76 },
    { 0x5d94c229, 96 },
    { 0xbd54c22a, 90 },
    { 0xe4d4c22a, 90 },
    { 0xcf94c22d, 66 },
    { 0xd374c22e, 95 },
    { 0x3d34c22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4284c232, 33 },
    { 0xc0a4c232, 95 },
    { 0x34c4c234, 91 },
    { 0xce34c234, 90 },
    { 0x5e14c233, 95 },
    { 0xc904c237, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd954c23b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de4c23e, 93 },
    { 0xa474c23f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714c244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534c247, 33 },
    { 0xa274c248, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4c24b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4c24f, 90 },
    { 0, 0 },
    { 0xb4c251, 90 },
    { 0, 0 },
    { 0xa1d4c253, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04c256, 23 },
    { 0xc524c257, 90 },
    { 0x4cc4c258, 75 },
    { 0xd4c257, 90 },
    { 0xaea4c256, 33 },
    { 0, 0 },
    { 0x5624c25c, 90 },
    { 0xe544c25d, 33 },
    { 0x6244c25e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf44c267, 33 },
    { 0, 0 },
    { 0x8ce4c269, 76 },
    { 0, 0 },
    { 0x2af4c26b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774c277, 34 },
    { 0xda54c278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec04c27f, 90 },
    { 0, 0 },
    { 0xc324c281, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea44c289, 98 },
    { 0x73f4c28a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4c28f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe474c293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3004c296, 33 },
    { 0x5a54c296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04c29f, 93 },
    { 0, 0 },
    { 0x8414c2a1, 61 },
    { 0xba84c2a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494c2a7, 38 },
    { 0, 0 },
    { 0x70f4c2a9, 46 },
    { 0x6174c2a9, 99 },
    { 0, 0 },
    { 0x8de4c2ac, 33 },
    { 0, 0 },
    { 0x7a14c2ae, 33 },
    { 0, 0 },
    { 0xc184c2b0, 90 },
    { 0, 0 },
    { 0x4054c2b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5864c2bb, 95 },
    { 0x6c84c2bc, 76 },
    { 0, 0 },
    { 0xbc04c2be, 77 },
    { 0x9784c2bf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf04c2c5, 100 },
    { 0x5c64c2c5, 23 },
    { 0xb7e4c2c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2544c2cb, 93 },
    { 0xb6a4c2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd534c2cf, 66 },
    { 0x5d04c2d0, 61 },
    { 0, 0 },
    { 0x8f24c2d2, 90 },
    { 0x53f4c2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9974c2d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x944c2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4c2dc, 23 },
    { 0x2e34c2dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8634c2e1, 23 },
    { 0, 0 },
    { 0x3674c2e3, 34 },
    { 0x3794c2e4, 33 },
    { 0x9f54c2e3, 84 },
    { 0xce4c2e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb4c2e9, 23 },
    { 0xc974c2ea, 34 },
    { 0x9944c2eb, 77 },
    { 0xcac4c2ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334c2f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf4c2f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d4c2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c4c2fe, 61 },
    { 0x9794c2ff, 61 },
    { 0xcef4c300, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e94c305, 90 },
    { 0x5804c306, 98 },
    { 0x2904c307, 55 },
    { 0, 0 },
    { 0xcb24c309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c84c311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f94c315, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa084c31c, 95 },
    { 0xc7d4c31d, 90 },
    { 0x9394c31e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c4c321, 68 },
    { 0x6b44c322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc4c326, 40 },
    { 0xcf44c326, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6514c32a, 34 },
    { 0x2d14c32a, 76 },
    { 0xe484c32c, 33 },
    { 0x9cc4c32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5724c331, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4c334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3414c337, 33 },
    { 0x1814c338, 78 },
    { 0x2594c338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc54c344, 90 },
    { 0xbc34c345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4c348, 76 },
    { 0x69d4c349, 61 },
    { 0xec64c34a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd274c34f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4c359, 55 },
    { 0, 0 },
    { 0x6ae4c35b, 95 },
    { 0, 0 },
    { 0x48b4c35d, 90 },
    { 0x6964c35e, 90 },
    { 0x9234c35e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574c364, 100 },
    { 0, 0 },
    { 0xd4f4c366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14c36b, 98 },
    { 0xf714c36c, 61 },
    { 0x4e44c36c, 90 },
    { 0, 0 },
    { 0xe8e4c36f, 84 },
    { 0xb174c370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c377, 52 },
    { 0x1214c378, 34 },
    { 0x1954c378, 76 },
    { 0xd044c37a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd294c37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb804c381, 33 },
    { 0xf534c382, 61 },
    { 0, 0 },
    { 0x92e4c384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a4c387, 43 },
    { 0, 0 },
    { 0x2ac4c389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c4c38d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb4c391, 46 },
    { 0x23e4c392, 75 },
    { 0x3854c393, 84 },
    { 0, 0 },
    { 0x9ea4c395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe4c399, 93 },
    { 0, 0 },
    { 0x1184c39b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14c3a6, 43 },
    { 0, 0 },
    { 0xe5f4c3a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf44c3ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb4c3b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74c3b4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b4c3b8, 33 },
    { 0x61b4c3b8, 75 },
    { 0x5be4c3ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5444c3bf, 98 },
    { 0xdbf4c3c0, 33 },
    { 0xf344c3bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7864c3c4, 61 },
    { 0x674c3c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a34c3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974c3cf, 55 },
    { 0, 0 },
    { 0xe444c3d1, 36 },
    { 0, 0 },
    { 0x8294c3d3, 61 },
    { 0x1054c3d4, 90 },
    { 0, 0 },
    { 0x9814c3d6, 91 },
    { 0, 0 },
    { 0x1b14c3d8, 76 },
    { 0x7804c3d8, 46 },
    { 0x86f4c3da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9474c3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5264c3ed, 95 },
    { 0xe654c3ed, 95 },
    { 0xfa54c3ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab4c3f6, 90 },
    { 0, 0 },
    { 0x54c4c3f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd74c3fc, 90 },
    { 0, 0 },
    { 0x3d94c3fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe764c403, 93 },
    { 0xe7e4c404, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324c408, 91 },
    { 0xc434c409, 23 },
    { 0xf584c40a, 23 },
    { 0, 0 },
    { 0x3c24c40c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4444c411, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e64c414, 36 },
    { 0xc5e4c414, 33 },
    { 0x82c4c416, 90 },
    { 0x9af4c415, 95 },
    { 0, 0 },
    { 0x6f54c419, 87 },
    { 0, 0 },
    { 0x2054c41b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b4c41e, 78 },
    { 0x2e4c41f, 23 },
    { 0, 0 },
    { 0xc024c421, 76 },
    { 0xc014c422, 33 },
    { 0, 0 },
    { 0x2e4c424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe4c42a, 66 },
    { 0x4c74c42b, 34 },
    { 0xc8e4c42a, 88 },
    { 0x3b44c42b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf4c431, 87 },
    { 0, 0 },
    { 0x2154c433, 34 },
    { 0xb764c433, 90 },
    { 0, 0 },
    { 0x2be4c436, 61 },
    { 0, 0 },
    { 0xd2b4c438, 92 },
    { 0x1554c439, 76 },
    { 0xd994c43a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf004c43d, 61 },
    { 0x2674c43e, 61 },
    { 0x6984c43f, 91 },
    { 0, 0 },
    { 0x7774c441, 98 },
    { 0x4f44c441, 95 },
    { 0, 0 },
    { 0x8634c444, 46 },
    { 0, 0 },
    { 0xac4c446, 78 },
    { 0xf8a4c447, 98 },
    { 0x7cf4c448, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd684c44e, 84 },
    { 0xac4c44e, 23 },
    { 0, 0 },
    { 0xda04c451, 85 },
    { 0, 0 },
    { 0x3ab4c453, 76 },
    { 0x17f4c454, 98 },
    { 0xddf4c454, 98 },
    { 0xf744c456, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b4c459, 43 },
    { 0x50a4c459, 43 },
    { 0, 0 },
    { 0x3c64c45c, 46 },
    { 0x65c4c45d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f4c464, 85 },
    { 0, 0 },
    { 0xfa94c466, 98 },
    { 0x53f4c466, 34 },
    { 0x6c84c466, 98 },
    { 0x7be4c469, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb454c46f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384c479, 90 },
    { 0x6804c479, 93 },
    { 0x4d74c47a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3904c482, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba4c485, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f4c48c, 55 },
    { 0, 0 },
    { 0xfd04c48e, 90 },
    { 0x6754c48e, 75 },
    { 0xc6c4c48f, 90 },
    { 0xa9a4c491, 23 },
    { 0, 0 },
    { 0xda14c493, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04c498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4c49c, 61 },
    { 0x2e54c49d, 34 },
    { 0, 0 },
    { 0x8d34c49f, 33 },
    { 0, 0 },
    { 0x9074c4a1, 61 },
    { 0, 0 },
    { 0xd244c4a3, 61 },
    { 0xd204c4a4, 17 },
    { 0, 0 },
    { 0xd6f4c4a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea74c4ab, 98 },
    { 0x1ec4c4ac, 53 },
    { 0, 0 },
    { 0x8614c4ae, 55 },
    { 0, 0 },
    { 0x9f24c4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3514c4b5, 91 },
    { 0x1d34c4b6, 80 },
    { 0x64f4c4b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde4c4bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b04c4bf, 61 },
    { 0xb84c4c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c4c6, 55 },
    { 0x2114c4c7, 61 },
    { 0x49e4c4c6, 61 },
    { 0x4f54c4c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e4c4d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4c4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b4c4dd, 43 },
    { 0xbc4c4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c84c4e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x804c4e5, 90 },
    { 0x2664c4e6, 33 },
    { 0xd2e4c4e7, 61 },
    { 0xfce4c4e8, 93 },
    { 0x9fc4c4e8, 85 },
    { 0, 0 },
    { 0xc964c4eb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c4c4ef, 90 },
    { 0xc404c4ef, 61 },
    { 0x4bc4c4f1, 100 },
    { 0x9ca4c4ef, 77 },
    { 0x3184c4ef, 95 },
    { 0xb8a4c4f4, 33 },
    { 0, 0 },
    { 0xf0c4c4f6, 76 },
    { 0x4ba4c4f6, 100 },
    { 0, 0 },
    { 0xf544c4f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c4fd, 88 },
    { 0, 0 },
    { 0xe274c4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4c505, 98 },
    { 0x7b74c506, 84 },
    { 0x324c506, 95 },
    { 0x7914c506, 61 },
    { 0xccc4c505, 95 },
    { 0x6c84c508, 98 },
    { 0, 0 },
    { 0xb3a4c50c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1274c50f, 34 },
    { 0x9e64c510, 23 },
    { 0xd6a4c511, 46 },
    { 0xfae4c510, 23 },
    { 0x63c4c513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf754c519, 76 },
    { 0xd1a4c519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df4c51f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac24c524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da4c528, 61 },
    { 0, 0 },
    { 0x3f84c52a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4c52d, 61 },
    { 0, 0 },
    { 0x1f24c52f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54c532, 33 },
    { 0x8a14c533, 98 },
    { 0xf0b4c534, 23 },
    { 0x24a4c535, 61 },
    { 0x2354c536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab04c53a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c4c53d, 90 },
    { 0, 0 },
    { 0x7bd4c53f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b4c542, 23 },
    { 0x4814c543, 95 },
    { 0, 0 },
    { 0x9854c545, 98 },
    { 0x3ea4c546, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd34c54b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434c550, 61 },
    { 0, 0 },
    { 0x2114c552, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd34c55b, 95 },
    { 0x2f44c55c, 85 },
    { 0xbd44c55d, 40 },
    { 0x9cb4c55d, 61 },
    { 0xdba4c55d, 52 },
    { 0x4364c55e, 93 },
    { 0x624c561, 76 },
    { 0xff54c55d, 95 },
    { 0xac94c55b, 23 },
    { 0xfa34c564, 95 },
    { 0x2d4c560, 90 },
    { 0x8294c55c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4c56b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2394c56e, 55 },
    { 0xa24c56f, 33 },
    { 0x3534c56e, 98 },
    { 0x5ef4c56f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1054c574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74c579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d4c57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb784c580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4c583, 61 },
    { 0, 0 },
    { 0x94e4c585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe404c593, 85 },
    { 0xa504c594, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e4c597, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154c59c, 33 },
    { 0x8d24c59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3764c5a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f4c5a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04c5a8, 59 },
    { 0x1774c5a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a14c5ac, 96 },
    { 0x68f4c5ad, 34 },
    { 0xe844c5ad, 33 },
    { 0xc654c5ac, 33 },
    { 0x1dc4c5b0, 48 },
    { 0xc974c5b1, 76 },
    { 0x7224c5af, 99 },
    { 0xcd54c5b3, 93 },
    { 0x57f4c5b4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xac74c5b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604c5bb, 55 },
    { 0, 0 },
    { 0x5584c5bd, 76 },
    { 0, 0 },
    { 0x20c4c5bf, 34 },
    { 0xabd4c5bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef04c5c3, 93 },
    { 0x6f74c5c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6704c5cd, 66 },
    { 0xb474c5cd, 93 },
    { 0x81f4c5cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7534c5d2, 95 },
    { 0x2c14c5d2, 76 },
    { 0xeac4c5d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d94c5d8, 101 },
    { 0, 0 },
    { 0xe8e4c5da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2614c5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144c5e3, 46 },
    { 0, 0 },
    { 0xa184c5e5, 66 },
    { 0x6094c5e6, 90 },
    { 0, 0 },
    { 0x1d64c5e8, 46 },
    { 0x9164c5e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34c5ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a74c5f3, 33 },
    { 0xe784c5f3, 65 },
    { 0, 0 },
    { 0x9f94c5f6, 95 },
    { 0xb8b4c5f7, 55 },
    { 0x7c74c5f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d94c5fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14c601, 23 },
    { 0x5b34c602, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5874c606, 98 },
    { 0x6d24c607, 98 },
    { 0x4194c608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff4c60c, 61 },
    { 0xfa44c60d, 61 },
    { 0xac74c60e, 46 },
    { 0xe434c60f, 45 },
    { 0x4314c60e, 61 },
    { 0x7574c60e, 88 },
    { 0, 0 },
    { 0x75e4c613, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4c616, 33 },
    { 0xe854c617, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384c61b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd04c620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2174c62a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b4c62d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae04c635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde14c63d, 61 },
    { 0x1e54c63e, 94 },
    { 0xefc4c63f, 43 },
    { 0xb254c63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a54c645, 91 },
    { 0x5c94c646, 90 },
    { 0xd9a4c647, 90 },
    { 0x814c647, 90 },
    { 0xa154c647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2054c64e, 23 },
    { 0x58c4c64f, 93 },
    { 0x7af4c64e, 23 },
    { 0xab84c651, 46 },
    { 0, 0 },
    { 0xc524c653, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c4c659, 95 },
    { 0x9814c65a, 33 },
    { 0x1434c65b, 23 },
    { 0xbb54c659, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4c663, 90 },
    { 0, 0 },
    { 0x9174c665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74c668, 33 },
    { 0, 0 },
    { 0x5a04c66a, 34 },
    { 0xc644c66b, 67 },
    { 0x4e74c66c, 60 },
    { 0, 0 },
    { 0xedb4c66e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb4c672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa344c677, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a4c67a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44c67e, 61 },
    { 0, 0 },
    { 0x1a94c680, 33 },
    { 0x8784c681, 95 },
    { 0, 0 },
    { 0x6544c683, 93 },
    { 0xcd34c683, 33 },
    { 0x5754c685, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f94c689, 55 },
    { 0, 0 },
    { 0x224c68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb604c694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4c697, 85 },
    { 0, 0 },
    { 0x3a94c699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa44c69e, 43 },
    { 0x5774c69f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd4c6a6, 40 },
    { 0x8b34c6a7, 99 },
    { 0x6594c6a7, 23 },
    { 0xd024c6a9, 61 },
    { 0xa554c6a9, 98 },
    { 0xa524c6ab, 52 },
    { 0xa824c6ac, 68 },
    { 0x6734c6ac, 91 },
    { 0, 0 },
    { 0xceb4c6af, 61 },
    { 0x40d4c6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c6b3, 95 },
    { 0xae34c6b4, 90 },
    { 0x4824c6b5, 90 },
    { 0x38e4c6b6, 55 },
    { 0x5424c6b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2674c6ba, 90 },
    { 0x6c14c6bb, 66 },
    { 0x9194c6bb, 55 },
    { 0, 0 },
    { 0x1a14c6be, 95 },
    { 0x8d04c6be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb514c6c5, 33 },
    { 0, 0 },
    { 0x1f74c6c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7824c6cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c4c6ce, 69 },
    { 0x6874c6cf, 55 },
    { 0x37e4c6cf, 33 },
    { 0x55d4c6d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4c6d5, 46 },
    { 0x39b4c6d6, 90 },
    { 0, 0 },
    { 0xc4b4c6d8, 95 },
    { 0x88a4c6d8, 91 },
    { 0xa434c6d8, 100 },
    { 0x3244c6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb094c6e0, 100 },
    { 0, 0 },
    { 0xfa44c6e2, 61 },
    { 0xcc24c6e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a64c6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd4c6ec, 46 },
    { 0x4bc4c6ec, 76 },
    { 0, 0 },
    { 0x55b4c6ef, 61 },
    { 0xfa94c6f0, 87 },
    { 0x6894c6f1, 68 },
    { 0xe014c6f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8434c6fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde4c6fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14c701, 90 },
    { 0, 0 },
    { 0x3754c703, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4c708, 66 },
    { 0x7124c709, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664c70e, 98 },
    { 0, 0 },
    { 0xa84c710, 36 },
    { 0, 0 },
    { 0x7114c712, 61 },
    { 0, 0 },
    { 0xaf74c714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd4c71a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354c71e, 33 },
    { 0x1584c71f, 33 },
    { 0, 0 },
    { 0xb4e4c721, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d4c725, 33 },
    { 0xad74c725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf694c72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c4c72d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e4c733, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd724c739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8804c73d, 81 },
    { 0x3ec4c73d, 98 },
    { 0x1054c73f, 61 },
    { 0x5ec4c73f, 61 },
    { 0x5014c73f, 91 },
    { 0, 0 },
    { 0x6094c743, 33 },
    { 0x5e54c744, 61 },
    { 0xbaa4c744, 100 },
    { 0x34c745, 91 },
    { 0, 0 },
    { 0xdb4c748, 76 },
    { 0xe754c749, 87 },
    { 0xc104c74a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a94c755, 61 },
    { 0x4b24c756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3344c75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9954c763, 33 },
    { 0x5504c764, 44 },
    { 0xef64c763, 98 },
    { 0x7a84c766, 55 },
    { 0x8bc4c766, 33 },
    { 0xed84c763, 23 },
    { 0xedb4c768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4c76c, 96 },
    { 0, 0 },
    { 0x2534c76e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4c773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd4c776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914c779, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb484c77d, 66 },
    { 0x63d4c77d, 95 },
    { 0x2864c77f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a4c78a, 78 },
    { 0xa5c4c78b, 95 },
    { 0xbe54c78b, 76 },
    { 0xfa4c78d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e4c791, 33 },
    { 0x1d54c792, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4c797, 90 },
    { 0x7da4c798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4c7a1, 53 },
    { 0xe8e4c7a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad4c7ab, 77 },
    { 0x58d4c7ab, 98 },
    { 0, 0 },
    { 0x1384c7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24c7b3, 88 },
    { 0x5c4c7b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f84c7b7, 90 },
    { 0x1234c7b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d4c7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb4c7bf, 23 },
    { 0x4524c7c0, 67 },
    { 0, 0 },
    { 0x7344c7c2, 46 },
    { 0xaa24c7c3, 68 },
    { 0xcff4c7c2, 90 },
    { 0x1ff4c7c3, 61 },
    { 0x4cc4c7c3, 43 },
    { 0x6e74c7c7, 61 },
    { 0xd84c7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e4c7cb, 85 },
    { 0x92b4c7cc, 23 },
    { 0x1314c7cd, 75 },
    { 0x24a4c7cd, 98 },
    { 0x5b14c7cf, 34 },
    { 0, 0 },
    { 0xa1b4c7d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a14c7d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494c7dd, 23 },
    { 0, 0 },
    { 0x74c7df, 76 },
    { 0x7434c7e0, 76 },
    { 0, 0 },
    { 0x5234c7e2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff4c7e5, 66 },
    { 0xbb44c7e5, 98 },
    { 0xe704c7e7, 76 },
    { 0x6f24c7e8, 90 },
    { 0x1484c7e9, 61 },
    { 0x2864c7ea, 90 },
    { 0x1884c7e9, 34 },
    { 0x7954c7ec, 84 },
    { 0x4164c7e8, 67 },
    { 0x7c44c7ed, 33 },
    { 0xc3e4c7ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb4c7f4, 61 },
    { 0, 0 },
    { 0xe274c7f6, 95 },
    { 0x4dd4c7f7, 33 },
    { 0, 0 },
    { 0xdf44c7f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5224c7fc, 55 },
    { 0xfe34c7fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974c80c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774c80f, 33 },
    { 0x7934c810, 76 },
    { 0xab64c80f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054c818, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5714c81c, 67 },
    { 0x2b64c81d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f44c821, 33 },
    { 0x15b4c822, 81 },
    { 0xf74c822, 33 },
    { 0x3024c822, 61 },
    { 0x9e34c821, 77 },
    { 0x59b4c826, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8584c82a, 23 },
    { 0x15e4c82b, 23 },
    { 0x4e84c82c, 33 },
    { 0x6154c82d, 61 },
    { 0x2be4c82e, 33 },
    { 0xa464c82d, 76 },
    { 0xc294c82b, 61 },
    { 0x5834c831, 74 },
    { 0xfea4c82e, 55 },
    { 0x61c4c833, 68 },
    { 0x53b4c834, 85 },
    { 0xac84c833, 61 },
    { 0xa894c82b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb4c83b, 99 },
    { 0, 0 },
    { 0xb7b4c83d, 90 },
    { 0xfcf4c83d, 78 },
    { 0xc074c83e, 95 },
    { 0xd784c840, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134c845, 61 },
    { 0, 0 },
    { 0xaea4c847, 90 },
    { 0, 0 },
    { 0xeca4c849, 95 },
    { 0x5d84c849, 76 },
    { 0xe484c84b, 56 },
    { 0xfeb4c84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d4c852, 61 },
    { 0xb614c853, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc004c85a, 98 },
    { 0xf0a4c85b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4c85e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a4c861, 87 },
    { 0xc354c862, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d4c865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c872, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5594c875, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc14c87c, 43 },
    { 0xd6e4c87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed4c880, 61 },
    { 0x2de4c881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744c886, 33 },
    { 0, 0 },
    { 0x5f34c888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb154c88f, 90 },
    { 0, 0 },
    { 0xbee4c891, 33 },
    { 0x2ac4c892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4c8a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc4c8a5, 61 },
    { 0xd54c8a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb54c8aa, 91 },
    { 0, 0 },
    { 0xa714c8ac, 17 },
    { 0x7d44c8ad, 98 },
    { 0xf014c8ae, 90 },
    { 0x7384c8ae, 90 },
    { 0x7964c8af, 90 },
    { 0xf454c8b1, 76 },
    { 0x5624c8b2, 85 },
    { 0x5d34c8b3, 33 },
    { 0xac74c8b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb424c8c2, 43 },
    { 0x2774c8c3, 61 },
    { 0x2d14c8c4, 87 },
    { 0, 0 },
    { 0x3fb4c8c6, 90 },
    { 0xa1c4c8c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac4c8cb, 33 },
    { 0, 0 },
    { 0x65c4c8cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4c8d0, 23 },
    { 0xb364c8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34c8d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2804c8df, 76 },
    { 0x2b34c8df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee4c8e3, 98 },
    { 0, 0 },
    { 0x2f74c8e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6274c8e9, 90 },
    { 0xb8f4c8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2974c8ed, 99 },
    { 0, 0 },
    { 0xcc44c8ef, 88 },
    { 0xaa74c8ef, 61 },
    { 0xce14c8f1, 90 },
    { 0x274c8f2, 34 },
    { 0x374c8f3, 87 },
    { 0x4b94c8f4, 17 },
    { 0xa784c8f5, 90 },
    { 0x90e4c8f5, 90 },
    { 0x2064c8f7, 96 },
    { 0, 0 },
    { 0x7204c8f9, 23 },
    { 0x13d4c8fa, 61 },
    { 0x1f84c8fb, 90 },
    { 0xe064c8f9, 99 },
    { 0x93f4c8fd, 33 },
    { 0x8874c8fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1644c905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c4c908, 46 },
    { 0, 0 },
    { 0x4944c90a, 52 },
    { 0, 0 },
    { 0x2734c90c, 90 },
    { 0, 0 },
    { 0xf04c90e, 90 },
    { 0, 0 },
    { 0xbbd4c910, 98 },
    { 0x2fb4c911, 90 },
    { 0, 0 },
    { 0x7834c913, 55 },
    { 0xd994c914, 23 },
    { 0x8d94c915, 90 },
    { 0, 0 },
    { 0x94c917, 80 },
    { 0, 0 },
    { 0xc7d4c919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6854c91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac14c91f, 33 },
    { 0xef34c91f, 64 },
    { 0xd1f4c921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf734c926, 90 },
    { 0x7374c926, 40 },
    { 0x8ff4c928, 85 },
    { 0x244c928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8484c92c, 33 },
    { 0x63e4c92d, 95 },
    { 0x2354c92e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74c933, 33 },
    { 0xc094c934, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf4c937, 33 },
    { 0, 0 },
    { 0xa934c939, 52 },
    { 0x50b4c939, 61 },
    { 0x8864c93b, 33 },
    { 0x63e4c93c, 55 },
    { 0xb7e4c93d, 95 },
    { 0xaa74c93a, 95 },
    { 0xf324c93b, 90 },
    { 0x7464c93b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554c947, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94c952, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134c955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74c95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc4c961, 61 },
    { 0x41d4c962, 61 },
    { 0xa134c961, 90 },
    { 0x95a4c964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1764c967, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee94c96f, 34 },
    { 0xac94c970, 23 },
    { 0x3a64c971, 90 },
    { 0xfd04c972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d24c97a, 76 },
    { 0, 0 },
    { 0x2684c97c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244c97f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f34c986, 68 },
    { 0x20c4c986, 95 },
    { 0x6794c988, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e54c98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74c98f, 66 },
    { 0, 0 },
    { 0x5ec4c991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4374c998, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a44c99b, 90 },
    { 0, 0 },
    { 0xd14c99d, 87 },
    { 0, 0 },
    { 0x44d4c99f, 55 },
    { 0xc154c9a0, 66 },
    { 0x1514c9a1, 95 },
    { 0x9634c9a1, 97 },
    { 0x2b84c9a1, 23 },
    { 0x2864c9a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4c9a7, 34 },
    { 0x65a4c9a8, 68 },
    { 0x9d34c9a9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe304c9ad, 93 },
    { 0, 0 },
    { 0x6304c9af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04c9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3314c9b7, 61 },
    { 0x8d04c9b7, 38 },
    { 0x5f64c9b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba4c9be, 55 },
    { 0x28e4c9bf, 91 },
    { 0, 0 },
    { 0xdff4c9c1, 98 },
    { 0, 0 },
    { 0xa3c4c9c3, 43 },
    { 0xd2e4c9c4, 23 },
    { 0x7294c9c4, 90 },
    { 0x59e4c9c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec4c9c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed34c9cc, 23 },
    { 0x1c54c9cd, 99 },
    { 0x1404c9cd, 95 },
    { 0x74b4c9cd, 93 },
    { 0x9664c9d0, 98 },
    { 0xc0e4c9d1, 100 },
    { 0x7e74c9d2, 78 },
    { 0x1cf4c9d2, 95 },
    { 0xf4b4c9d4, 46 },
    { 0x21a4c9d2, 90 },
    { 0xe5b4c9d6, 61 },
    { 0x4474c9d6, 95 },
    { 0xd5b4c9d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd534c9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb324c9e4, 91 },
    { 0x8ae4c9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d64c9ed, 61 },
    { 0x4954c9ee, 33 },
    { 0xb7a4c9ef, 78 },
    { 0x8884c9ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c14c9f5, 33 },
    { 0x6274c9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e4c9f9, 43 },
    { 0xe784c9fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad94ca02, 90 },
    { 0x4634ca03, 34 },
    { 0, 0 },
    { 0x3834ca05, 95 },
    { 0xbf44ca06, 61 },
    { 0x1fb4ca07, 46 },
    { 0x8564ca05, 98 },
    { 0, 0 },
    { 0xbf64ca0a, 61 },
    { 0xbe64ca0b, 88 },
    { 0xa854ca0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14ca11, 95 },
    { 0x1964ca11, 61 },
    { 0xe334ca13, 55 },
    { 0x5b4ca13, 33 },
    { 0x7964ca11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1664ca21, 98 },
    { 0x63e4ca21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4ca29, 90 },
    { 0x2164ca2a, 76 },
    { 0xa664ca2b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa24ca2e, 46 },
    { 0x8254ca2f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6434ca3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e4ca40, 88 },
    { 0xaa84ca41, 66 },
    { 0x33e4ca42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304ca49, 98 },
    { 0, 0 },
    { 0xa14ca4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4ca53, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b4ca59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f4ca5d, 61 },
    { 0x77b4ca5e, 78 },
    { 0x53c4ca5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b4ca62, 76 },
    { 0, 0 },
    { 0x5804ca64, 66 },
    { 0x2024ca65, 92 },
    { 0xdfb4ca65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4ca6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7534ca72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8554ca76, 23 },
    { 0, 0 },
    { 0x8aa4ca78, 95 },
    { 0xae84ca78, 17 },
    { 0x50a4ca7a, 75 },
    { 0x8464ca7b, 55 },
    { 0, 0 },
    { 0x2e04ca7d, 74 },
    { 0xc424ca7e, 61 },
    { 0, 0 },
    { 0x88d4ca80, 34 },
    { 0x4b84ca80, 61 },
    { 0x6a54ca80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4ca8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed4ca8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4ca95, 84 },
    { 0, 0 },
    { 0xf274ca97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1134ca9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844caa2, 23 },
    { 0x25d4caa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a04caa7, 90 },
    { 0, 0 },
    { 0xd6d4caa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04caad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d64cab1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34cab4, 33 },
    { 0x3834cab5, 61 },
    { 0xfda4cab6, 78 },
    { 0, 0 },
    { 0x1764cab8, 23 },
    { 0, 0 },
    { 0x194caba, 98 },
    { 0x9844caba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134cabe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d4cac1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf74cac6, 95 },
    { 0x6464cac7, 66 },
    { 0x9f24cac7, 95 },
    { 0x8764cac6, 99 },
    { 0, 0 },
    { 0x8854cacb, 23 },
    { 0x5724cacc, 61 },
    { 0xc04cacc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724cad2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9974cad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb964cadc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6214cadf, 65 },
    { 0x65d4cae0, 76 },
    { 0x4fd4cae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3934caef, 95 },
    { 0xb444caf0, 95 },
    { 0x2b24caef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf44caf6, 99 },
    { 0xc354caf7, 37 },
    { 0, 0 },
    { 0xfe64caf9, 91 },
    { 0xf914cafa, 68 },
    { 0xae84cafb, 33 },
    { 0, 0 },
    { 0xf6b4cafd, 91 },
    { 0xb234cafe, 43 },
    { 0xa194caff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5974cb02, 99 },
    { 0xbf64cb03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb394cb07, 95 },
    { 0x8014cb08, 33 },
    { 0xc754cb08, 61 },
    { 0x4224cb07, 84 },
    { 0, 0 },
    { 0x34d4cb0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324cb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf014cb16, 23 },
    { 0x7414cb17, 33 },
    { 0xdca4cb18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4cb1e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf34cb26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc4cb29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f04cb32, 55 },
    { 0, 0 },
    { 0xaae4cb34, 95 },
    { 0, 0 },
    { 0xb104cb36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94cb44, 99 },
    { 0xa1e4cb44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d4cb49, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb54cb57, 90 },
    { 0xb574cb58, 61 },
    { 0x38d4cb57, 33 },
    { 0xb954cb59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d34cb5d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa904cb67, 43 },
    { 0, 0 },
    { 0x9a84cb69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x394cb6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b4cb70, 91 },
    { 0, 0 },
    { 0x66c4cb72, 33 },
    { 0, 0 },
    { 0x384cb74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d4cb7a, 61 },
    { 0, 0 },
    { 0x9ec4cb7c, 98 },
    { 0x4ea4cb7d, 98 },
    { 0, 0 },
    { 0x1a24cb7f, 90 },
    { 0x22d4cb80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec64cb84, 76 },
    { 0x9c24cb85, 90 },
    { 0xacf4cb86, 34 },
    { 0xc4c4cb84, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9834cb91, 95 },
    { 0x1354cb92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb894cb95, 98 },
    { 0, 0 },
    { 0xc8a4cb97, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb4cb9c, 95 },
    { 0, 0 },
    { 0x56d4cb9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9774cba2, 97 },
    { 0, 0 },
    { 0xf984cba4, 33 },
    { 0xba24cba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d4cbae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7484cbb1, 34 },
    { 0x8f94cbb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cbb5, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb34cbba, 90 },
    { 0x9f94cbba, 98 },
    { 0xb574cbbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5454cbc2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2484cbc9, 95 },
    { 0xff94cbc9, 33 },
    { 0, 0 },
    { 0xc134cbcc, 95 },
    { 0, 0 },
    { 0xbac4cbce, 66 },
    { 0, 0 },
    { 0xf64cbd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324cbd3, 85 },
    { 0, 0 },
    { 0xe84cbd5, 98 },
    { 0x1bf4cbd6, 61 },
    { 0xc234cbd7, 90 },
    { 0x40c4cbd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7804cbdc, 87 },
    { 0x49f4cbdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4cbe1, 91 },
    { 0x5a84cbe1, 33 },
    { 0x8144cbe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a94cbe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f14cbeb, 33 },
    { 0xcc14cbec, 33 },
    { 0, 0 },
    { 0x8164cbee, 100 },
    { 0x5754cbef, 98 },
    { 0x8904cbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee4cbf3, 52 },
    { 0, 0 },
    { 0xafd4cbf5, 34 },
    { 0, 0 },
    { 0x7304cbf7, 61 },
    { 0x8424cbf8, 55 },
    { 0x64c4cbf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df4cbfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164cbff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8554cc06, 90 },
    { 0x92f4cc07, 33 },
    { 0x48f4cc08, 90 },
    { 0x6334cc08, 67 },
    { 0x3fc4cc0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf4cc0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d4cc13, 64 },
    { 0x9d34cc14, 93 },
    { 0xcf64cc15, 33 },
    { 0x6764cc16, 23 },
    { 0x91b4cc17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9494cc1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc44cc21, 90 },
    { 0xc114cc21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1464cc25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4cc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284cc2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5844cc35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9784cc38, 61 },
    { 0x1854cc39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc394cc3d, 98 },
    { 0x4c44cc3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b4cc41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494cc4b, 33 },
    { 0x43e4cc4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f4cc51, 96 },
    { 0x3794cc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b64cc56, 23 },
    { 0, 0 },
    { 0xf864cc58, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f4cc66, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc194cc6e, 76 },
    { 0x15a4cc6e, 90 },
    { 0xee94cc70, 34 },
    { 0, 0 },
    { 0xbe74cc72, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b4cc75, 85 },
    { 0xe404cc76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f24cc79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf164cc7f, 95 },
    { 0, 0 },
    { 0x1d44cc81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb4cc88, 95 },
    { 0x7004cc88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a4cc8e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e4cc92, 98 },
    { 0xd114cc93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe424cc98, 100 },
    { 0xd3c4cc99, 75 },
    { 0x6fc4cc99, 85 },
    { 0x6ac4cc99, 61 },
    { 0xa994cc9c, 34 },
    { 0x1364cc99, 98 },
    { 0, 0 },
    { 0xa154cc9f, 98 },
    { 0, 0 },
    { 0xd9a4cca1, 87 },
    { 0x7834cca1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534ccb4, 66 },
    { 0x9624ccb5, 95 },
    { 0x8cf4ccb6, 90 },
    { 0, 0 },
    { 0x70d4ccb8, 76 },
    { 0x7a74ccb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de4ccbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf4ccc0, 98 },
    { 0x494ccc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054ccc6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e4cccd, 100 },
    { 0, 0 },
    { 0x6c04cccf, 33 },
    { 0xaed4ccd0, 90 },
    { 0x3464ccd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb384ccd5, 61 },
    { 0, 0 },
    { 0x6a94ccd7, 52 },
    { 0x9254ccd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d4cce5, 68 },
    { 0x4c94cce5, 33 },
    { 0x6814cce5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ccea, 88 },
    { 0x9af4cceb, 76 },
    { 0x6cf4ccec, 61 },
    { 0xe334cced, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d4ccf2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d4ccfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4cd01, 43 },
    { 0x5ad4cd02, 98 },
    { 0xa724cd02, 43 },
    { 0x3ae4cd04, 95 },
    { 0xb4c4cd05, 34 },
    { 0, 0 },
    { 0xdb94cd07, 61 },
    { 0, 0 },
    { 0x1a4cd09, 91 },
    { 0xf814cd09, 100 },
    { 0x1d84cd0b, 23 },
    { 0x4cc4cd0c, 98 },
    { 0xa44cd0c, 90 },
    { 0x2e24cd0e, 33 },
    { 0xc94cd0f, 33 },
    { 0x5fc4cd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe524cd14, 90 },
    { 0, 0 },
    { 0x1de4cd16, 76 },
    { 0xd54cd16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd884cd1a, 91 },
    { 0, 0 },
    { 0xe884cd1c, 93 },
    { 0x3a14cd1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24cd22, 98 },
    { 0xbc44cd22, 33 },
    { 0x42b4cd23, 93 },
    { 0x5264cd25, 95 },
    { 0, 0 },
    { 0xadc4cd27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2374cd2a, 33 },
    { 0x7164cd2b, 33 },
    { 0, 0 },
    { 0x8a34cd2d, 87 },
    { 0xc264cd2e, 33 },
    { 0xc054cd2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4cd35, 91 },
    { 0xd184cd36, 95 },
    { 0xb84cd36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde84cd3b, 23 },
    { 0, 0 },
    { 0xe1f4cd3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5594cd46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4cd49, 90 },
    { 0xded4cd4a, 90 },
    { 0, 0 },
    { 0x9264cd4c, 76 },
    { 0, 0 },
    { 0xb544cd4e, 90 },
    { 0, 0 },
    { 0x7724cd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2924cd5b, 95 },
    { 0xf584cd5c, 90 },
    { 0x93a4cd5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b4cd60, 90 },
    { 0x3064cd60, 34 },
    { 0x5c94cd62, 90 },
    { 0x9184cd62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee34cd66, 33 },
    { 0, 0 },
    { 0xa924cd68, 87 },
    { 0xdf94cd68, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c4cd70, 33 },
    { 0x24a4cd71, 66 },
    { 0, 0 },
    { 0xfe64cd73, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d4cd78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54cd7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf84cd80, 46 },
    { 0x4884cd81, 55 },
    { 0x4e14cd81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a4cd8b, 17 },
    { 0x4074cd8c, 61 },
    { 0xa144cd8d, 96 },
    { 0xcce4cd8b, 55 },
    { 0xb4c4cd8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84cd96, 90 },
    { 0x76f4cd97, 90 },
    { 0xbb54cd97, 100 },
    { 0x5404cd99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd84cd9d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e44cda1, 67 },
    { 0, 0 },
    { 0x7da4cda3, 33 },
    { 0x3834cda4, 95 },
    { 0x6c44cda5, 88 },
    { 0xe6c4cda6, 34 },
    { 0x8734cda7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4cdad, 23 },
    { 0, 0 },
    { 0xa294cdaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c4cdb2, 95 },
    { 0xc734cdb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5874cdb6, 23 },
    { 0xd794cdb7, 90 },
    { 0, 0 },
    { 0x23e4cdb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4cdbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba4cdc5, 95 },
    { 0, 0 },
    { 0x9ff4cdc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84cdcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4584cdd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae24cdd8, 93 },
    { 0xce94cdd8, 40 },
    { 0xc4b4cdda, 90 },
    { 0xcc44cddb, 61 },
    { 0, 0 },
    { 0x94c4cddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4cde1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9014cde8, 95 },
    { 0x34e4cde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4cdee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44cdfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb64ce00, 33 },
    { 0, 0 },
    { 0x84c4ce02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb74ce06, 93 },
    { 0x6b34ce06, 34 },
    { 0, 0 },
    { 0x8374ce09, 90 },
    { 0x9534ce0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a4ce0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf604ce14, 46 },
    { 0, 0 },
    { 0xadb4ce16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6954ce1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7244ce22, 98 },
    { 0xcb64ce23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b4ce26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc4ce29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4ce31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ce35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74ce38, 46 },
    { 0xd454ce38, 61 },
    { 0x5844ce39, 98 },
    { 0x5dd4ce3b, 95 },
    { 0, 0 },
    { 0xda74ce3d, 90 },
    { 0xdcc4ce3e, 76 },
    { 0, 0 },
    { 0x60e4ce40, 93 },
    { 0xb6f4ce41, 34 },
    { 0x93b4ce41, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134ce47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec4ce4d, 90 },
    { 0x75f4ce4e, 91 },
    { 0x3334ce4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ce52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4ce57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea4ce5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd4ce65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf104ce6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd694ce6d, 98 },
    { 0xbde4ce6e, 95 },
    { 0, 0 },
    { 0x3f44ce70, 61 },
    { 0x23f4ce71, 61 },
    { 0, 0 },
    { 0xe064ce73, 90 },
    { 0x3894ce73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4ce78, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5954ce7d, 95 },
    { 0x9214ce7d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4ce82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2564ce85, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a4ce89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec4ce8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714ce90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b4ce94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4444ce97, 98 },
    { 0xfb04ce98, 23 },
    { 0, 0 },
    { 0xb424ce9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874ce9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e4cea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664cea5, 61 },
    { 0xeb34cea5, 93 },
    { 0, 0 },
    { 0xaa24cea8, 77 },
    { 0xdba4cea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f4ceaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ceb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad44ceb6, 95 },
    { 0, 0 },
    { 0x744ceb8, 90 },
    { 0x92c4ceb9, 92 },
    { 0, 0 },
    { 0x8f84cebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864cebe, 84 },
    { 0x2f94cebf, 80 },
    { 0xd5d4cebf, 76 },
    { 0x1b24cebe, 55 },
    { 0x4e54cec1, 93 },
    { 0x414cec0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf674cec9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf094cecd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af4ced0, 52 },
    { 0xbe84ced0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354ced4, 95 },
    { 0xb6f4ced5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704cedb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd4cee1, 95 },
    { 0x94b4cee1, 23 },
    { 0, 0 },
    { 0x32c4cee4, 76 },
    { 0, 0 },
    { 0x5c74cee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3814ceeb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3334ceef, 33 },
    { 0, 0 },
    { 0x6954cef1, 55 },
    { 0, 0 },
    { 0xde4cef3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7424cef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504cefa, 90 },
    { 0x10e4cefb, 98 },
    { 0x1544cefa, 61 },
    { 0xbd94cefd, 76 },
    { 0x94e4cefe, 90 },
    { 0, 0 },
    { 0x7b54cf00, 43 },
    { 0xa854cf00, 93 },
    { 0, 0 },
    { 0x41c4cf03, 91 },
    { 0x5f44cf03, 96 },
    { 0x4654cf03, 55 },
    { 0x9974cf06, 90 },
    { 0xe6c4cf07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714cf13, 76 },
    { 0, 0 },
    { 0xc654cf15, 68 },
    { 0xb404cf15, 95 },
    { 0xd234cf16, 61 },
    { 0x8114cf15, 95 },
    { 0xc794cf18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14cf1e, 88 },
    { 0x2ab4cf1e, 90 },
    { 0x1654cf20, 61 },
    { 0x5784cf21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2964cf25, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f4cf29, 90 },
    { 0, 0 },
    { 0x7c34cf2b, 61 },
    { 0x4c4cf2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e4cf2f, 55 },
    { 0xa8f4cf30, 76 },
    { 0x93c4cf31, 98 },
    { 0xe184cf32, 90 },
    { 0xdac4cf33, 90 },
    { 0x3824cf33, 91 },
    { 0x2814cf30, 95 },
    { 0x1c04cf36, 98 },
    { 0xa074cf37, 93 },
    { 0xdff4cf34, 90 },
    { 0x5dc4cf39, 95 },
    { 0x5f54cf3a, 33 },
    { 0, 0 },
    { 0xdd54cf3c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8414cf3f, 61 },
    { 0xd404cf3f, 90 },
    { 0xd024cf41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf4cf45, 33 },
    { 0xc4d4cf46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf474cf4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa684cf4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4cf52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4cf55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c14cf58, 90 },
    { 0, 0 },
    { 0x48a4cf5a, 90 },
    { 0xd804cf5a, 76 },
    { 0x3844cf5c, 88 },
    { 0, 0 },
    { 0xf3e4cf5e, 88 },
    { 0x4234cf5f, 87 },
    { 0, 0 },
    { 0xa224cf61, 61 },
    { 0xabc4cf62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4cf69, 46 },
    { 0x6b4cf6a, 99 },
    { 0x7e14cf6b, 61 },
    { 0, 0 },
    { 0xb84cf6d, 34 },
    { 0, 0 },
    { 0x5654cf6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d54cf75, 76 },
    { 0, 0 },
    { 0x1364cf77, 55 },
    { 0x39f4cf78, 76 },
    { 0, 0 },
    { 0xb0a4cf7a, 17 },
    { 0xcdc4cf7b, 46 },
    { 0xea14cf7b, 52 },
    { 0xf584cf7a, 33 },
    { 0x5494cf7e, 23 },
    { 0, 0 },
    { 0x42e4cf80, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc074cf83, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4504cf87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4cf8d, 75 },
    { 0x9b64cf8e, 66 },
    { 0, 0 },
    { 0x2674cf90, 98 },
    { 0xc224cf91, 77 },
    { 0x9694cf91, 95 },
    { 0xe824cf93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e04cf97, 33 },
    { 0x7f24cf98, 52 },
    { 0x1fb4cf99, 61 },
    { 0x2354cf98, 61 },
    { 0, 0 },
    { 0xb74cf9c, 52 },
    { 0x9514cf9d, 90 },
    { 0xf304cf9e, 98 },
    { 0x9474cf9c, 90 },
    { 0x4be4cf9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe304cfab, 95 },
    { 0, 0 },
    { 0xc5c4cfad, 33 },
    { 0x87b4cfad, 34 },
    { 0x9764cfaf, 55 },
    { 0x36b4cfb0, 61 },
    { 0x4284cfb0, 65 },
    { 0, 0 },
    { 0xb584cfb3, 55 },
    { 0, 0 },
    { 0x3d64cfb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b4cfbe, 23 },
    { 0x9604cfbf, 34 },
    { 0x5f44cfc0, 98 },
    { 0x3e74cfc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd4cfc6, 95 },
    { 0xf554cfc7, 55 },
    { 0xe154cfc8, 93 },
    { 0x6f04cfc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3234cfd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044cfd3, 23 },
    { 0x2a84cfd3, 55 },
    { 0xa1a4cfd5, 95 },
    { 0xa084cfd5, 76 },
    { 0xa654cfd7, 55 },
    { 0x9594cfd8, 23 },
    { 0xa394cfd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054cfde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84cfe1, 90 },
    { 0, 0 },
    { 0xc724cfe3, 46 },
    { 0xe474cfe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9304cfe9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94cfec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc4cff1, 90 },
    { 0, 0 },
    { 0x39b4cff3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a4cff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f64cffc, 61 },
    { 0xeaf4cffd, 65 },
    { 0xe074cffc, 76 },
    { 0x54cfff, 88 },
    { 0xa0c4cfff, 90 },
    { 0x2284d000, 95 },
    { 0x5164d000, 33 },
    { 0xb124d003, 55 },
    { 0, 0 },
    { 0x6cb4d005, 90 },
    { 0x5fd4d006, 33 },
    { 0, 0 },
    { 0x4f64d008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe694d00b, 91 },
    { 0, 0 },
    { 0x1744d00d, 75 },
    { 0x9eb4d00d, 100 },
    { 0, 0 },
    { 0x1294d010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb184d016, 98 },
    { 0, 0 },
    { 0x28d4d018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe254d01d, 81 },
    { 0xde44d01e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284d021, 33 },
    { 0xa104d022, 67 },
    { 0, 0 },
    { 0x7a74d024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b34d02a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14d031, 98 },
    { 0x8714d032, 36 },
    { 0, 0 },
    { 0xc5c4d034, 42 },
    { 0, 0 },
    { 0xd74d036, 90 },
    { 0xa944d037, 23 },
    { 0, 0 },
    { 0xc364d039, 76 },
    { 0x5c04d039, 23 },
    { 0, 0 },
    { 0x8214d03c, 75 },
    { 0xd134d03d, 23 },
    { 0xf9c4d03c, 23 },
    { 0, 0 },
    { 0xc654d040, 23 },
    { 0xbb94d040, 90 },
    { 0xed64d042, 61 },
    { 0x9c04d042, 76 },
    { 0x5bf4d040, 34 },
    { 0, 0 },
    { 0x1754d046, 34 },
    { 0xc264d046, 33 },
    { 0x4c74d047, 98 },
    { 0x3a4d049, 87 },
    { 0x78d4d049, 95 },
    { 0x7ca4d047, 90 },
    { 0xf454d04c, 95 },
    { 0, 0 },
    { 0xb8b4d04e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564d052, 98 },
    { 0x7954d053, 40 },
    { 0, 0 },
    { 0xade4d055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4d05b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc74d060, 91 },
    { 0, 0 },
    { 0x7a94d062, 33 },
    { 0, 0 },
    { 0x5a54d064, 61 },
    { 0x8eb4d065, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa704d068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff54d06c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb94d072, 43 },
    { 0xfec4d072, 23 },
    { 0x4ea4d074, 81 },
    { 0, 0 },
    { 0x3824d076, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6684d079, 23 },
    { 0x3ac4d07a, 33 },
    { 0xec14d07b, 100 },
    { 0xe424d07a, 95 },
    { 0, 0 },
    { 0xaa94d07e, 23 },
    { 0, 0 },
    { 0x2d74d080, 68 },
    { 0x9164d081, 40 },
    { 0, 0 },
    { 0x50f4d083, 61 },
    { 0xfe34d084, 76 },
    { 0x7384d084, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4d08a, 61 },
    { 0x2cf4d08a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8714d092, 66 },
    { 0, 0 },
    { 0x4b4d094, 33 },
    { 0x8534d095, 95 },
    { 0x12d4d095, 85 },
    { 0xd3f4d096, 78 },
    { 0xb754d094, 23 },
    { 0x714d095, 61 },
    { 0x35d4d095, 33 },
    { 0x87a4d09b, 76 },
    { 0x1c24d09c, 98 },
    { 0x2354d09d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324d0a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e4d0a6, 55 },
    { 0, 0 },
    { 0xae74d0a8, 93 },
    { 0x334d0a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e4d0ad, 90 },
    { 0x2704d0ad, 90 },
    { 0x6dc4d0ae, 33 },
    { 0x5894d0b0, 61 },
    { 0x4284d0b1, 55 },
    { 0xef04d0b0, 23 },
    { 0x5b44d0b3, 81 },
    { 0xfa94d0b1, 75 },
    { 0, 0 },
    { 0xea4d0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74d0ba, 98 },
    { 0x51b4d0bb, 34 },
    { 0x9934d0bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c4d0bf, 91 },
    { 0, 0 },
    { 0x29b4d0c1, 55 },
    { 0, 0 },
    { 0x5cd4d0c3, 34 },
    { 0x244d0c4, 90 },
    { 0xe474d0c5, 55 },
    { 0x33d4d0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4d0ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d0cd, 55 },
    { 0x9684d0cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2704d0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf34d0d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204d0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774d0dd, 98 },
    { 0, 0 },
    { 0xf744d0df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc4d0e2, 33 },
    { 0, 0 },
    { 0xc3e4d0e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44d0e7, 33 },
    { 0x3084d0e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc754d0ef, 23 },
    { 0x7584d0f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d0f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe4d0f9, 33 },
    { 0, 0 },
    { 0xed54d0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd084d0fe, 90 },
    { 0xc754d0ff, 61 },
    { 0, 0 },
    { 0xc8d4d101, 33 },
    { 0xc244d102, 55 },
    { 0x7b14d103, 90 },
    { 0, 0 },
    { 0xac44d105, 76 },
    { 0, 0 },
    { 0x7364d107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94d10f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f4d112, 90 },
    { 0x4024d113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b4d116, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814d119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f4d121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d4d127, 90 },
    { 0, 0 },
    { 0x354d129, 33 },
    { 0x99c4d12a, 80 },
    { 0xf394d12a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1224d131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074d136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba34d13a, 33 },
    { 0xb9d4d13b, 61 },
    { 0xd364d13b, 90 },
    { 0xfcf4d13b, 23 },
    { 0x2b4d13e, 90 },
    { 0x3064d13f, 78 },
    { 0xc334d13f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7464d144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7694d156, 23 },
    { 0xc014d157, 33 },
    { 0x8b4d157, 33 },
    { 0xc374d159, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb84d15c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a4d161, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4d164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd454d167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7434d16c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c4d172, 33 },
    { 0x2904d173, 61 },
    { 0x5cc4d174, 33 },
    { 0, 0 },
    { 0xb914d176, 95 },
    { 0x8534d177, 95 },
    { 0xd9d4d178, 23 },
    { 0xb8b4d178, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b44d17e, 90 },
    { 0x94d17f, 90 },
    { 0xf1e4d180, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704d184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c54d18b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa384d18e, 95 },
    { 0x73c4d18f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024d192, 76 },
    { 0x4904d193, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1254d196, 33 },
    { 0xa1d4d197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d19b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354d19e, 33 },
    { 0, 0 },
    { 0x88f4d1a0, 55 },
    { 0x97f4d1a1, 76 },
    { 0x6fc4d1a2, 85 },
    { 0x7f84d1a3, 98 },
    { 0x2594d1a4, 90 },
    { 0, 0 },
    { 0xd914d1a6, 76 },
    { 0x9b64d1a7, 88 },
    { 0x62e4d1a8, 96 },
    { 0x2464d1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794d1af, 90 },
    { 0xaa4d1af, 61 },
    { 0xc714d1b1, 76 },
    { 0x7954d1b2, 55 },
    { 0xe5a4d1b1, 43 },
    { 0x3334d1b4, 61 },
    { 0, 0 },
    { 0x6df4d1b6, 75 },
    { 0, 0 },
    { 0x1db4d1b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344d1bc, 33 },
    { 0, 0 },
    { 0x3504d1be, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x3284d1c1, 23 },
    { 0, 0 },
    { 0x1a14d1c3, 77 },
    { 0, 0 },
    { 0xed04d1c5, 33 },
    { 0, 0 },
    { 0x2c14d1c7, 100 },
    { 0x67b4d1c8, 85 },
    { 0x2064d1c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984d1cf, 85 },
    { 0x2ca4d1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f4d1d3, 98 },
    { 0, 0 },
    { 0x95a4d1d5, 23 },
    { 0, 0 },
    { 0x8124d1d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab4d1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e4d1e8, 23 },
    { 0x7224d1e8, 98 },
    { 0x2c94d1ea, 23 },
    { 0, 0 },
    { 0xc804d1ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a4d1f0, 61 },
    { 0, 0 },
    { 0x2024d1f2, 99 },
    { 0, 0 },
    { 0x6ef4d1f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44d1f8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee4d1fc, 78 },
    { 0x714d1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7274d200, 95 },
    { 0x6d44d200, 66 },
    { 0xe064d202, 66 },
    { 0xd524d200, 23 },
    { 0x1c64d204, 99 },
    { 0x9b24d205, 95 },
    { 0xcbd4d205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e44d20a, 61 },
    { 0xf774d20b, 100 },
    { 0x2424d20c, 76 },
    { 0x8094d20c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5704d218, 36 },
    { 0xcde4d219, 43 },
    { 0xf6b4d21a, 90 },
    { 0x6404d218, 98 },
    { 0x28f4d21c, 33 },
    { 0x3ce4d21c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7034d220, 98 },
    { 0x1a74d221, 87 },
    { 0, 0 },
    { 0x1e04d223, 95 },
    { 0xca14d224, 76 },
    { 0x5824d225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe584d228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c74d22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4d22e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4424d231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df4d234, 77 },
    { 0xfe54d234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7684d23c, 90 },
    { 0xc924d23d, 87 },
    { 0x8494d23e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794d242, 93 },
    { 0xbf54d243, 90 },
    { 0x6264d244, 43 },
    { 0xc914d245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14d24a, 33 },
    { 0xb074d24a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb14d24e, 98 },
    { 0x78a4d24e, 23 },
    { 0, 0 },
    { 0x67c4d251, 61 },
    { 0x3194d251, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d24d257, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e84d25f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d269, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa244d26d, 33 },
    { 0xf394d26d, 33 },
    { 0, 0 },
    { 0x1d64d270, 61 },
    { 0x51c4d270, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b4d275, 98 },
    { 0x754d276, 90 },
    { 0x9ff4d275, 95 },
    { 0, 0 },
    { 0x30f4d279, 90 },
    { 0xfa74d279, 95 },
    { 0xcaa4d27b, 61 },
    { 0x48c4d27c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da4d280, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8454d283, 95 },
    { 0, 0 },
    { 0xcb34d285, 33 },
    { 0xc184d286, 34 },
    { 0xab34d287, 33 },
    { 0xc764d288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6154d28c, 45 },
    { 0x1554d28d, 67 },
    { 0xe1a4d28d, 98 },
    { 0, 0 },
    { 0x1594d290, 100 },
    { 0x5de4d291, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4d298, 34 },
    { 0x9d24d299, 61 },
    { 0, 0 },
    { 0xd4e4d29b, 98 },
    { 0x8bd4d29c, 95 },
    { 0, 0 },
    { 0x6534d29e, 98 },
    { 0x2864d29e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c44d2a4, 33 },
    { 0, 0 },
    { 0x3ca4d2a6, 23 },
    { 0x3bb4d2a7, 55 },
    { 0, 0 },
    { 0x2c54d2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b4d2ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc574d2b0, 17 },
    { 0, 0 },
    { 0x8fd4d2b2, 100 },
    { 0x7f94d2b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x754d2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5994d2b9, 33 },
    { 0x86a4d2ba, 98 },
    { 0, 0 },
    { 0x2454d2bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d2bf, 61 },
    { 0x95e4d2c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4d2c7, 98 },
    { 0x3cd4d2c8, 101 },
    { 0x74c4d2c9, 81 },
    { 0xc394d2c8, 93 },
    { 0x49e4d2c9, 61 },
    { 0xcf24d2c7, 52 },
    { 0x7104d2cd, 93 },
    { 0, 0 },
    { 0x8f14d2cf, 81 },
    { 0x35d4d2cf, 76 },
    { 0, 0 },
    { 0x7f84d2d2, 90 },
    { 0x51d4d2d3, 46 },
    { 0xc564d2d3, 98 },
    { 0x32b4d2d5, 33 },
    { 0xe514d2d4, 100 },
    { 0x41c4d2d6, 61 },
    { 0xc64d2d7, 68 },
    { 0, 0 },
    { 0x3614d2da, 81 },
    { 0, 0 },
    { 0x1e44d2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d2df, 33 },
    { 0x80b4d2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b4d2e7, 100 },
    { 0x2dc4d2e8, 43 },
    { 0x7f94d2e7, 23 },
    { 0x2e04d2e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4d2ed, 84 },
    { 0, 0 },
    { 0x4d24d2ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe4d2f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4d302, 43 },
    { 0x70a4d302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604d30d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x574d310, 98 },
    { 0, 0 },
    { 0xdf24d312, 52 },
    { 0x7aa4d312, 90 },
    { 0, 0 },
    { 0x9544d315, 33 },
    { 0x9094d316, 61 },
    { 0x7b4d316, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9864d31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c74d31f, 23 },
    { 0, 0 },
    { 0xc594d321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4d324, 61 },
    { 0, 0 },
    { 0xc3f4d326, 90 },
    { 0x53e4d327, 95 },
    { 0xbd64d328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec84d32b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3814d32f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b74d333, 23 },
    { 0xd6c4d333, 33 },
    { 0x8924d335, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd4d33b, 95 },
    { 0x46b4d33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe854d340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6264d343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6884d348, 23 },
    { 0x8154d348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb494d34e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf4d353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14d359, 77 },
    { 0, 0 },
    { 0x7064d35b, 98 },
    { 0x3874d35b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd004d360, 91 },
    { 0, 0 },
    { 0xea24d362, 91 },
    { 0, 0 },
    { 0xcfa4d364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb104d372, 81 },
    { 0x1544d373, 98 },
    { 0xfe74d374, 88 },
    { 0xdae4d375, 90 },
    { 0x7634d376, 76 },
    { 0, 0 },
    { 0x56d4d378, 23 },
    { 0xbb74d379, 34 },
    { 0x37e4d37a, 23 },
    { 0xd3e4d37a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d37f, 98 },
    { 0x2334d37f, 90 },
    { 0, 0 },
    { 0xa2d4d382, 98 },
    { 0x8614d383, 33 },
    { 0x9634d384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394d38a, 87 },
    { 0, 0 },
    { 0x7804d38c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9914d393, 98 },
    { 0xef34d394, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64d398, 80 },
    { 0x24c4d399, 52 },
    { 0xcfe4d399, 66 },
    { 0xd124d399, 67 },
    { 0xedb4d398, 33 },
    { 0, 0 },
    { 0x5554d39e, 67 },
    { 0, 0 },
    { 0x40b4d3a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb874d3a4, 90 },
    { 0xdcb4d3a5, 23 },
    { 0x9214d3a5, 90 },
    { 0x32c4d3a4, 33 },
    { 0x15e4d3a8, 90 },
    { 0xce24d3a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b4d3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4954d3b1, 23 },
    { 0xc444d3b2, 98 },
    { 0x65e4d3b3, 78 },
    { 0xf864d3b1, 95 },
    { 0x1774d3b5, 33 },
    { 0, 0 },
    { 0x6254d3b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f74d3bb, 55 },
    { 0x9324d3bc, 100 },
    { 0, 0 },
    { 0x80c4d3be, 93 },
    { 0, 0 },
    { 0x6624d3c0, 95 },
    { 0xa634d3c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee4d3c7, 43 },
    { 0x4694d3c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134d3ce, 61 },
    { 0, 0 },
    { 0x66c4d3d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9874d3d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc4d3db, 99 },
    { 0, 0 },
    { 0xefd4d3dd, 99 },
    { 0x1374d3de, 93 },
    { 0xd084d3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9474d3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94d3e5, 78 },
    { 0x3814d3e6, 61 },
    { 0, 0 },
    { 0x8014d3e8, 95 },
    { 0xc054d3e9, 38 },
    { 0x6a24d3ea, 76 },
    { 0x8454d3eb, 17 },
    { 0xf944d3ec, 81 },
    { 0x19b4d3ec, 95 },
    { 0xfb44d3ed, 95 },
    { 0xecb4d3ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e4d3f3, 98 },
    { 0, 0 },
    { 0x98f4d3f5, 66 },
    { 0x8504d3f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf84d3f9, 84 },
    { 0x58b4d3f9, 61 },
    { 0, 0 },
    { 0x9094d3fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6414d3ff, 46 },
    { 0xf4d4d400, 90 },
    { 0, 0 },
    { 0x6824d402, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb054d408, 61 },
    { 0xee14d409, 78 },
    { 0x97b4d408, 90 },
    { 0, 0 },
    { 0x1c64d40c, 23 },
    { 0, 0 },
    { 0x4e44d40e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3924d412, 95 },
    { 0, 0 },
    { 0x3184d414, 76 },
    { 0, 0 },
    { 0xe884d416, 76 },
    { 0, 0 },
    { 0xf664d418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f44d41e, 33 },
    { 0x57c4d41f, 44 },
    { 0, 0 },
    { 0x7ed4d421, 61 },
    { 0xcbe4d422, 33 },
    { 0xdbf4d423, 81 },
    { 0xa4d424, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704d429, 90 },
    { 0x43b4d42a, 98 },
    { 0x8ed4d42b, 91 },
    { 0xc514d42c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4d42f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea4d432, 83 },
    { 0x7c4d432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f4d438, 98 },
    { 0, 0 },
    { 0xd3e4d43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2784d442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4d445, 98 },
    { 0x6c14d446, 90 },
    { 0, 0 },
    { 0xb664d448, 66 },
    { 0x7144d448, 68 },
    { 0x4284d44a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a04d44f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974d453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4974d457, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d4d465, 23 },
    { 0x7d24d466, 61 },
    { 0x9904d467, 98 },
    { 0xdb74d465, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1644d46c, 55 },
    { 0x2e94d46d, 95 },
    { 0x41e4d46d, 33 },
    { 0x13b4d46e, 61 },
    { 0xe884d470, 90 },
    { 0xf0c4d470, 61 },
    { 0xbdb4d472, 95 },
    { 0, 0 },
    { 0x3f64d474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74d477, 83 },
    { 0x1624d478, 52 },
    { 0x5c04d479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd4d47c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354d481, 23 },
    { 0x6574d482, 77 },
    { 0x53e4d483, 61 },
    { 0x6a54d483, 76 },
    { 0x5964d485, 33 },
    { 0x1684d482, 95 },
    { 0x7a84d487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d84d48b, 33 },
    { 0xd6e4d48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764d48f, 98 },
    { 0x1ff4d490, 81 },
    { 0x58d4d491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa54d498, 33 },
    { 0x3c24d499, 95 },
    { 0xb394d498, 66 },
    { 0x8a04d49a, 33 },
    { 0x5bd4d49c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244d4a1, 61 },
    { 0xe384d4a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1294d4a5, 61 },
    { 0xb084d4a5, 76 },
    { 0, 0 },
    { 0x1bf4d4a8, 61 },
    { 0x2754d4a8, 76 },
    { 0x9354d4aa, 33 },
    { 0x4ca4d4ab, 78 },
    { 0x2324d4aa, 34 },
    { 0x7994d4ad, 99 },
    { 0x6ba4d4ae, 90 },
    { 0xebd4d4af, 66 },
    { 0, 0 },
    { 0x4dc4d4b1, 93 },
    { 0x6fc4d4b2, 65 },
    { 0xe114d4b1, 34 },
    { 0xe2e4d4b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a4d4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc844d4ba, 99 },
    { 0, 0 },
    { 0x6134d4bc, 23 },
    { 0x5af4d4bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb4d4c6, 46 },
    { 0x8d74d4c6, 75 },
    { 0x8e04d4c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a4d4cc, 95 },
    { 0x6fc4d4cd, 97 },
    { 0x4974d4ce, 52 },
    { 0xa354d4cd, 33 },
    { 0x3864d4ce, 43 },
    { 0x4d54d4d0, 61 },
    { 0xcb74d4d2, 98 },
    { 0x1734d4d3, 46 },
    { 0xe5b4d4d2, 90 },
    { 0x2934d4d2, 34 },
    { 0xe3a4d4d0, 61 },
    { 0xbed4d4ce, 98 },
    { 0xf5c4d4d3, 98 },
    { 0xc084d4d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5214d4dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354d4e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94d4e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf4d4e7, 90 },
    { 0xd894d4e7, 52 },
    { 0xc664d4e9, 90 },
    { 0, 0 },
    { 0x72a4d4eb, 95 },
    { 0x89e4d4ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554d4ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4d4f4, 81 },
    { 0, 0 },
    { 0x8ff4d4f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4d4fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa884d4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a4d502, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb4d505, 87 },
    { 0, 0 },
    { 0x8644d507, 68 },
    { 0x5a84d508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a44d50c, 90 },
    { 0, 0 },
    { 0xa4a4d50e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f4d513, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794d518, 33 },
    { 0, 0 },
    { 0x38f4d51a, 95 },
    { 0x1e4d51b, 95 },
    { 0x8a14d51c, 33 },
    { 0xecf4d51b, 33 },
    { 0, 0 },
    { 0xc94d51f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b04d524, 95 },
    { 0xc1e4d525, 23 },
    { 0xfdf4d525, 95 },
    { 0xdc34d527, 23 },
    { 0xedd4d527, 34 },
    { 0x6654d529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd4d52e, 23 },
    { 0x3494d52f, 87 },
    { 0x12a4d52f, 34 },
    { 0x3684d531, 98 },
    { 0, 0 },
    { 0x9cc4d533, 33 },
    { 0x19e4d534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d4d537, 61 },
    { 0, 0 },
    { 0x924d539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c4d540, 90 },
    { 0xa2a4d541, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe34d545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa4d551, 100 },
    { 0x9b64d552, 33 },
    { 0x4fd4d553, 55 },
    { 0x6164d554, 52 },
    { 0xbc34d554, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc4d55d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134d560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1004d563, 33 },
    { 0xe484d564, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc944d56c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4d570, 88 },
    { 0, 0 },
    { 0xdc04d572, 91 },
    { 0x4474d573, 99 },
    { 0x1634d574, 61 },
    { 0, 0 },
    { 0x4884d576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa414d57a, 88 },
    { 0, 0 },
    { 0x3e74d57c, 61 },
    { 0x4b34d57d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc44d583, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac4d589, 93 },
    { 0xa1e4d58a, 99 },
    { 0x84a4d58b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4d58e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa424d592, 90 },
    { 0xfea4d593, 100 },
    { 0x2524d594, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb94d59d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef4d5a0, 33 },
    { 0x4544d5a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3594d5a7, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4d5aa, 76 },
    { 0, 0 },
    { 0xd9f4d5ac, 23 },
    { 0xd554d5ad, 95 },
    { 0, 0 },
    { 0xa944d5af, 23 },
    { 0xf244d5af, 46 },
    { 0x3964d5af, 23 },
    { 0x2584d5b2, 87 },
    { 0x3ea4d5b3, 98 },
    { 0xf214d5b2, 98 },
    { 0xb334d5b3, 95 },
    { 0xe2c4d5b3, 43 },
    { 0xd484d5b7, 61 },
    { 0x3e4d5b8, 43 },
    { 0xf564d5b9, 61 },
    { 0, 0 },
    { 0x4154d5bb, 66 },
    { 0xe124d5bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd434d5c3, 55 },
    { 0xdb44d5c3, 23 },
    { 0xa424d5c5, 33 },
    { 0x50c4d5c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e4d5cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb74d5d0, 100 },
    { 0, 0 },
    { 0xb7c4d5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x274d5d5, 33 },
    { 0xa834d5d5, 61 },
    { 0x394d5d7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4d5dc, 33 },
    { 0, 0 },
    { 0x25e4d5de, 33 },
    { 0x6a44d5de, 76 },
    { 0x4064d5e0, 23 },
    { 0x1ca4d5e0, 100 },
    { 0, 0 },
    { 0x7d34d5e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04d5e7, 85 },
    { 0, 0 },
    { 0xeb74d5e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb684d5f1, 61 },
    { 0, 0 },
    { 0x9de4d5f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde94d5f8, 90 },
    { 0, 0 },
    { 0x5784d5fa, 90 },
    { 0xbcf4d5fa, 61 },
    { 0, 0 },
    { 0xaa64d5fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec4d606, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5104d609, 95 },
    { 0x2ed4d60a, 55 },
    { 0xc14d609, 93 },
    { 0xe214d60c, 78 },
    { 0x6154d60c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac44d619, 90 },
    { 0, 0 },
    { 0x1104d61b, 23 },
    { 0, 0 },
    { 0x5544d61d, 90 },
    { 0x8624d61e, 68 },
    { 0xd194d61d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6304d622, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b14d627, 61 },
    { 0xa144d627, 43 },
    { 0xe924d629, 8 },
    { 0x8c4d628, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b24d62d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4d632, 33 },
    { 0x2d84d633, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf174d637, 85 },
    { 0, 0 },
    { 0xbb24d639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4d642, 90 },
    { 0, 0 },
    { 0xc8e4d644, 98 },
    { 0, 0 },
    { 0x6264d646, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a4d64a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a84d64d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb434d655, 95 },
    { 0xd6e4d656, 75 },
    { 0x3844d657, 90 },
    { 0x3d54d658, 66 },
    { 0x2fb4d657, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d660, 90 },
    { 0, 0 },
    { 0x6884d662, 66 },
    { 0xb54d663, 95 },
    { 0, 0 },
    { 0xff04d665, 34 },
    { 0, 0 },
    { 0x6864d667, 76 },
    { 0, 0 },
    { 0x49c4d669, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xca54d66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344d671, 99 },
    { 0xae34d671, 98 },
    { 0xd4f4d673, 61 },
    { 0x6554d674, 61 },
    { 0xfa44d675, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a34d679, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4d67e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554d681, 33 },
    { 0, 0 },
    { 0xe224d683, 33 },
    { 0xda14d683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794d687, 76 },
    { 0, 0 },
    { 0xf514d689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a24d68f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4d694, 76 },
    { 0xb684d695, 61 },
    { 0xe194d695, 98 },
    { 0, 0 },
    { 0xf634d698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d6a1, 88 },
    { 0, 0 },
    { 0x1ea4d6a3, 23 },
    { 0, 0 },
    { 0x4224d6a5, 23 },
    { 0, 0 },
    { 0x8604d6a7, 98 },
    { 0x96a4d6a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4d6ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a54d6b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4d6b6, 95 },
    { 0x72e4d6b7, 46 },
    { 0x6364d6b8, 34 },
    { 0x6974d6b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2174d6bc, 23 },
    { 0x9134d6bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb584d6c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4494d6ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2134d6cd, 33 },
    { 0, 0 },
    { 0xf54d6cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04d6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd064d6e3, 61 },
    { 0xf494d6e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e04d6ea, 65 },
    { 0x82e4d6eb, 90 },
    { 0xc3f4d6ea, 61 },
    { 0x424d6ed, 81 },
    { 0xf094d6ed, 98 },
    { 0x90c4d6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d6f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4d6f6, 55 },
    { 0xf434d6f7, 55 },
    { 0xa7b4d6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4d6fb, 93 },
    { 0, 0 },
    { 0x60f4d6fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4d709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4d70c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44d712, 61 },
    { 0x8584d713, 52 },
    { 0, 0 },
    { 0xa084d715, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74d71d, 23 },
    { 0, 0 },
    { 0x9a64d71f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac4d723, 90 },
    { 0x27a4d724, 76 },
    { 0x9a44d725, 100 },
    { 0x71d4d725, 91 },
    { 0xf404d723, 95 },
    { 0x4e44d725, 23 },
    { 0xaf54d729, 23 },
    { 0xcef4d72a, 100 },
    { 0xc314d729, 90 },
    { 0xeb04d72a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d731, 33 },
    { 0, 0 },
    { 0x8284d733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf4d736, 66 },
    { 0, 0 },
    { 0xe144d738, 33 },
    { 0, 0 },
    { 0xa274d73a, 76 },
    { 0x67e4d73a, 100 },
    { 0x5df4d73a, 76 },
    { 0x50d4d73d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x8714d740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9724d744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc874d749, 91 },
    { 0xce74d749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8144d74e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4d751, 88 },
    { 0, 0 },
    { 0x3cc4d753, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4994d757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd584d75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b24d764, 93 },
    { 0x8174d764, 90 },
    { 0, 0 },
    { 0xe674d767, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9124d76c, 55 },
    { 0xef34d76c, 61 },
    { 0, 0 },
    { 0x65f4d76f, 23 },
    { 0xf034d76f, 40 },
    { 0xabc4d76f, 78 },
    { 0xdfc4d770, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04d778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f4d77e, 78 },
    { 0x6384d77e, 17 },
    { 0xccb4d780, 84 },
    { 0x2404d780, 90 },
    { 0x53b4d782, 98 },
    { 0x9bc4d783, 100 },
    { 0xf144d784, 100 },
    { 0xa844d77f, 66 },
    { 0xdca4d785, 23 },
    { 0x6a94d783, 46 },
    { 0x35d4d788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4d792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6234d795, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824d799, 93 },
    { 0xa694d799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14d79e, 83 },
    { 0x474d79f, 91 },
    { 0xdf34d79f, 66 },
    { 0x4584d7a1, 33 },
    { 0x8404d79e, 76 },
    { 0x834d7a3, 84 },
    { 0, 0 },
    { 0x16c4d7a5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4d7a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d7ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914d7b6, 99 },
    { 0x1554d7b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04d7ba, 100 },
    { 0xb154d7ba, 33 },
    { 0xaf34d7bc, 97 },
    { 0xabc4d7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4704d7c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24d7c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db4d7c6, 78 },
    { 0, 0 },
    { 0xcbd4d7c8, 95 },
    { 0, 0 },
    { 0x96b4d7ca, 46 },
    { 0x2c74d7cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e54d7cf, 81 },
    { 0, 0 },
    { 0xf9c4d7d1, 46 },
    { 0x33a4d7d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b4d7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014d7e1, 98 },
    { 0, 0 },
    { 0xcf64d7e3, 33 },
    { 0x3f4d7e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xea54d7e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d4d7ea, 98 },
    { 0, 0 },
    { 0xc764d7ec, 52 },
    { 0x2e04d7ed, 90 },
    { 0x5394d7ee, 33 },
    { 0x3f14d7ed, 90 },
    { 0, 0 },
    { 0x6994d7f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d7f4, 90 },
    { 0xc14d7f5, 99 },
    { 0xa414d7f6, 98 },
    { 0x9fa4d7f4, 61 },
    { 0, 0 },
    { 0xe5a4d7f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb4d7ff, 95 },
    { 0xf1b4d800, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24d803, 33 },
    { 0x6d74d804, 61 },
    { 0x1f14d805, 95 },
    { 0, 0 },
    { 0x2034d807, 55 },
    { 0xe5d4d808, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7074d811, 95 },
    { 0xae34d812, 90 },
    { 0x4fa4d813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174d816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44d81c, 65 },
    { 0x8cb4d81c, 66 },
    { 0, 0 },
    { 0x2824d81f, 80 },
    { 0, 0 },
    { 0x15a4d821, 98 },
    { 0, 0 },
    { 0x6324d823, 98 },
    { 0xefc4d823, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8274d827, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af4d82a, 84 },
    { 0, 0 },
    { 0x2244d82c, 98 },
    { 0x15b4d82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c44d831, 66 },
    { 0xc424d832, 66 },
    { 0xb454d831, 95 },
    { 0x7594d831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4d838, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe394d83b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c84d842, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f4d849, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f54d84d, 98 },
    { 0x9ba4d84e, 61 },
    { 0x7884d84f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d74d855, 95 },
    { 0xc0c4d856, 69 },
    { 0x55e4d857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e4d85b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b4d85f, 78 },
    { 0xa5e4d860, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d94d864, 61 },
    { 0x3614d864, 100 },
    { 0xb1e4d864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f74d86c, 17 },
    { 0x1aa4d86d, 98 },
    { 0x9244d86d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b54d872, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234d877, 46 },
    { 0, 0 },
    { 0xb264d879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a4d87f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c4d887, 90 },
    { 0xdbe4d887, 34 },
    { 0, 0 },
    { 0xa754d88a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec4d88d, 93 },
    { 0x5004d88d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134d891, 99 },
    { 0xafe4d891, 76 },
    { 0xfeb4d891, 96 },
    { 0xaba4d894, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d898, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4d89c, 98 },
    { 0x5be4d89d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e4d8a1, 61 },
    { 0xc5e4d8a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f64d8a6, 55 },
    { 0xe94d8a7, 90 },
    { 0, 0 },
    { 0x4ce4d8a9, 34 },
    { 0x88b4d8aa, 100 },
    { 0xe894d8ab, 99 },
    { 0, 0 },
    { 0xa904d8ad, 95 },
    { 0x87c4d8ae, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d4d8b3, 61 },
    { 0, 0 },
    { 0xe6e4d8b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94d8bb, 55 },
    { 0, 0 },
    { 0xd1a4d8bd, 46 },
    { 0xd44d8be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a4d8c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4d8d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4d8dc, 23 },
    { 0x67d4d8dd, 23 },
    { 0x1404d8de, 90 },
    { 0, 0 },
    { 0xda24d8e0, 90 },
    { 0x3eb4d8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b4d8e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944d8ec, 55 },
    { 0, 0 },
    { 0x7694d8ee, 61 },
    { 0, 0 },
    { 0xcc34d8f0, 23 },
    { 0, 0 },
    { 0xa924d8f2, 33 },
    { 0, 0 },
    { 0x6614d8f4, 33 },
    { 0x68c4d8f4, 78 },
    { 0xb474d8f5, 76 },
    { 0xb274d8f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4d8fb, 90 },
    { 0x62d4d8fc, 91 },
    { 0, 0 },
    { 0xb224d8fe, 33 },
    { 0xa74d8ff, 55 },
    { 0xad84d900, 52 },
    { 0x1004d8ff, 38 },
    { 0, 0 },
    { 0x94d4d903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e4d908, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1404d90e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7874d911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4d916, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c74d919, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6564d91d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc4d923, 90 },
    { 0xe7e4d924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d927, 70 },
    { 0x5e24d928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc774d930, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944d933, 61 },
    { 0x3dc4d934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9824d938, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd4d941, 98 },
    { 0xb294d942, 80 },
    { 0x4324d942, 33 },
    { 0xc644d944, 33 },
    { 0x6964d943, 33 },
    { 0xe374d946, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4d950, 55 },
    { 0, 0 },
    { 0x4084d952, 90 },
    { 0xbe74d952, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf204d959, 33 },
    { 0xbee4d959, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7864d960, 98 },
    { 0, 0 },
    { 0xb0b4d962, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4d966, 61 },
    { 0x6444d967, 55 },
    { 0, 0 },
    { 0xf8c4d969, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd4d973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c4d976, 55 },
    { 0x514d977, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54d981, 61 },
    { 0, 0 },
    { 0x26b4d983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe424d98d, 61 },
    { 0x6ca4d98d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774d991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb4d998, 50 },
    { 0, 0 },
    { 0xd0d4d99a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e4d99d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4d9a4, 99 },
    { 0, 0 },
    { 0x2394d9a6, 23 },
    { 0x97b4d9a6, 84 },
    { 0xc264d9a7, 33 },
    { 0xcd84d9a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc4d9ac, 90 },
    { 0x7fa4d9ad, 18 },
    { 0x3ef4d9ae, 99 },
    { 0x9e74d9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7414d9bc, 23 },
    { 0, 0 },
    { 0x5814d9be, 90 },
    { 0xee54d9be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5444d9c7, 90 },
    { 0, 0 },
    { 0xa9f4d9c9, 80 },
    { 0x4744d9c9, 91 },
    { 0x2f4d9ca, 93 },
    { 0x8a74d9cc, 95 },
    { 0xbe54d9cd, 76 },
    { 0x5c04d9cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff4d9d1, 23 },
    { 0, 0 },
    { 0xeb04d9d3, 33 },
    { 0, 0 },
    { 0xa594d9d5, 43 },
    { 0x274d9d6, 61 },
    { 0x6be4d9d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984d9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a54d9e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964d9e4, 55 },
    { 0, 0 },
    { 0xdb34d9e6, 69 },
    { 0x6194d9e7, 85 },
    { 0xb054d9e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6104d9f2, 98 },
    { 0, 0 },
    { 0xb174d9f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c44d9fa, 91 },
    { 0x1264d9fa, 34 },
    { 0x6fc4d9fb, 68 },
    { 0x8714d9fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f74da00, 61 },
    { 0x85f4da01, 33 },
    { 0xa344da02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb824da06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894da0d, 33 },
    { 0x2b04da0d, 95 },
    { 0xdcb4da0f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb224da12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da15, 100 },
    { 0, 0 },
    { 0x28c4da17, 33 },
    { 0x944da18, 91 },
    { 0xe3d4da19, 90 },
    { 0xe2c4da1a, 95 },
    { 0x1844da17, 90 },
    { 0x9604da1c, 99 },
    { 0xae24da1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a44da23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb4da27, 90 },
    { 0, 0 },
    { 0x5004da29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f4da2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc494da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b44da37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b4da3d, 98 },
    { 0x3fc4da3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c4da44, 22 },
    { 0xa0d4da45, 65 },
    { 0xf704da46, 33 },
    { 0xc394da47, 33 },
    { 0x354da47, 33 },
    { 0, 0 },
    { 0xc6f4da4a, 61 },
    { 0, 0 },
    { 0x3984da4c, 55 },
    { 0xd554da4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d4da50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f4da53, 95 },
    { 0x2374da54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5714da5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf4da5d, 93 },
    { 0, 0 },
    { 0x15c4da5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc4da64, 100 },
    { 0x8a44da64, 76 },
    { 0, 0 },
    { 0x2824da67, 23 },
    { 0x7164da68, 46 },
    { 0x7654da69, 43 },
    { 0x8354da68, 85 },
    { 0xf224da6b, 55 },
    { 0x1714da6b, 61 },
    { 0x9734da6b, 87 },
    { 0x29a4da6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4da71, 90 },
    { 0x8e44da72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4384da7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4da82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b4da86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9834da89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534da8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64da92, 67 },
    { 0x6084da92, 90 },
    { 0x2b04da94, 75 },
    { 0x6314da95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a94da9b, 61 },
    { 0x6eb4da9c, 34 },
    { 0x69f4da9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4daaa, 90 },
    { 0x1834daab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd4daaf, 33 },
    { 0xcce4dab0, 61 },
    { 0, 0 },
    { 0xa64dab2, 95 },
    { 0, 0 },
    { 0xc084dab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2334dab8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf294dabd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e4dac1, 55 },
    { 0x14c4dac1, 61 },
    { 0xbbd4dac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad64dac6, 52 },
    { 0x7384dac6, 34 },
    { 0, 0 },
    { 0xd714dac9, 55 },
    { 0xd2d4dac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94dacd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4dad0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb904dad3, 68 },
    { 0xf24dad4, 98 },
    { 0x5c94dad4, 23 },
    { 0xe414dad6, 34 },
    { 0x684dad7, 99 },
    { 0x9c14dad8, 65 },
    { 0x4b44dad9, 55 },
    { 0x55f4dada, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd764dadd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014dae2, 46 },
    { 0x5af4dae2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c4dae7, 23 },
    { 0xf644dae8, 90 },
    { 0x6c94dae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a4daec, 55 },
    { 0x11f4daed, 96 },
    { 0x4804daed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b4daf2, 84 },
    { 0xa6e4daf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7204daf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4dafd, 95 },
    { 0x3784dafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb764db01, 90 },
    { 0, 0 },
    { 0xbf04db03, 90 },
    { 0x6084db04, 95 },
    { 0, 0 },
    { 0xbc54db06, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4db0a, 23 },
    { 0x2894db0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf914db11, 34 },
    { 0, 0 },
    { 0x1574db13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c4db16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54db19, 34 },
    { 0xd174db1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94db1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb724db22, 90 },
    { 0xcb44db23, 98 },
    { 0x75e4db24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d94db27, 95 },
    { 0xa294db28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf244db32, 98 },
    { 0, 0 },
    { 0x1744db34, 33 },
    { 0x83b4db35, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd384db3b, 83 },
    { 0xd254db3c, 85 },
    { 0x6d24db3c, 34 },
    { 0xeef4db3d, 23 },
    { 0xa534db3c, 90 },
    { 0x6914db40, 98 },
    { 0xe604db3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e4db51, 87 },
    { 0x1014db51, 93 },
    { 0x1a64db51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb74db5b, 33 },
    { 0x2b34db5c, 95 },
    { 0, 0 },
    { 0x1bb4db5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d4db67, 33 },
    { 0x12c4db68, 17 },
    { 0xf314db69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4db6d, 90 },
    { 0xedf4db6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf84db73, 98 },
    { 0xb124db74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b4db78, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db87, 23 },
    { 0, 0 },
    { 0x5c34db89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b4db91, 91 },
    { 0x6614db91, 34 },
    { 0x914db93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9714db97, 33 },
    { 0x64d4db97, 88 },
    { 0x4d94db99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d4db9c, 100 },
    { 0x7ba4db9c, 23 },
    { 0xb1d4db9e, 61 },
    { 0x2ed4db9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354dba5, 90 },
    { 0x3384dba6, 90 },
    { 0x5d34dba7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934dbab, 93 },
    { 0x4a54dbac, 90 },
    { 0, 0 },
    { 0x70f4dbae, 66 },
    { 0x4654dbaf, 85 },
    { 0xcc84dbb0, 66 },
    { 0x7d4dbb1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4dbb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4dbb9, 76 },
    { 0xb5f4dbba, 100 },
    { 0x2124dbbb, 98 },
    { 0xf824dbb9, 95 },
    { 0xa504dbbb, 23 },
    { 0, 0 },
    { 0x6d54dbbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7604dbc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a74dbc7, 43 },
    { 0x94a4dbc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce4dbcc, 52 },
    { 0x3fe4dbcd, 23 },
    { 0, 0 },
    { 0x2374dbcf, 61 },
    { 0x72b4dbd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e4dbd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4dbde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4594dbe2, 61 },
    { 0x65a4dbe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c4dbeb, 61 },
    { 0x2e04dbec, 98 },
    { 0xfaf4dbed, 78 },
    { 0xa764dbed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e4dbf9, 61 },
    { 0, 0 },
    { 0x7224dbfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44dc02, 76 },
    { 0x4304dc03, 19 },
    { 0xf34dc04, 46 },
    { 0xc974dc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c64dc09, 90 },
    { 0, 0 },
    { 0x3514dc0b, 98 },
    { 0x8bd4dc0c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee04dc15, 23 },
    { 0, 0 },
    { 0xdd44dc17, 61 },
    { 0, 0 },
    { 0xc054dc19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d4dc1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14dc22, 33 },
    { 0, 0 },
    { 0x4404dc24, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e4dc29, 78 },
    { 0x76f4dc29, 97 },
    { 0x1264dc2a, 100 },
    { 0x1b04dc2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c4dc30, 68 },
    { 0, 0 },
    { 0x86f4dc32, 75 },
    { 0x4584dc33, 34 },
    { 0, 0 },
    { 0xb514dc35, 98 },
    { 0x5644dc36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4194dc3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84dc3e, 33 },
    { 0, 0 },
    { 0x23a4dc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d4dc43, 69 },
    { 0x42f4dc44, 98 },
    { 0, 0 },
    { 0xe624dc46, 61 },
    { 0x2984dc47, 33 },
    { 0x2684dc48, 46 },
    { 0, 0 },
    { 0x7534dc4a, 34 },
    { 0x3d24dc4a, 98 },
    { 0, 0 },
    { 0xe1f4dc4d, 95 },
    { 0, 0 },
    { 0x16a4dc4f, 100 },
    { 0, 0 },
    { 0xe884dc51, 43 },
    { 0xcc04dc52, 76 },
    { 0, 0 },
    { 0x1754dc54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda24dc57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde64dc5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe614dc60, 76 },
    { 0, 0 },
    { 0x2d74dc62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6104dc65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4dc6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec4dc6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc4dc72, 67 },
    { 0xc364dc72, 98 },
    { 0x5874dc72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace4dc77, 43 },
    { 0x3884dc78, 98 },
    { 0x89e4dc78, 33 },
    { 0, 0 },
    { 0xace4dc7b, 61 },
    { 0, 0 },
    { 0xa304dc7d, 46 },
    { 0, 0 },
    { 0xad64dc7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a84dc82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4dc86, 76 },
    { 0, 0 },
    { 0x3324dc88, 78 },
    { 0xe8f4dc88, 95 },
    { 0, 0 },
    { 0x14c4dc8b, 98 },
    { 0x2aa4dc8b, 61 },
    { 0x9874dc8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4dc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a4dc98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a4dc9f, 95 },
    { 0xab74dca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc504dca4, 93 },
    { 0x6204dca5, 67 },
    { 0xe74dca4, 61 },
    { 0, 0 },
    { 0xba94dca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb764dcab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8514dcae, 33 },
    { 0, 0 },
    { 0x7084dcb0, 61 },
    { 0, 0 },
    { 0x3fd4dcb2, 23 },
    { 0x9de4dcb2, 97 },
    { 0x59e4dcb4, 33 },
    { 0x9144dcb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014dcb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4964dcbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe444dcc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64dccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc294dccf, 33 },
    { 0x4794dcd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae84dcd5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2914dcda, 90 },
    { 0, 0 },
    { 0xcf44dcdc, 92 },
    { 0x46b4dcdd, 33 },
    { 0, 0 },
    { 0x1c24dcdf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944dce2, 87 },
    { 0xc674dce3, 98 },
    { 0xa4dce4, 75 },
    { 0xbac4dce3, 61 },
    { 0xba54dce4, 76 },
    { 0x5474dce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4824dcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b4dcf1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c4dcf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae44dcf8, 23 },
    { 0, 0 },
    { 0x62e4dcfa, 66 },
    { 0, 0 },
    { 0xad44dcfc, 68 },
    { 0x4d84dcfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5064dd01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c4dd06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1474dd09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8754dd0d, 84 },
    { 0x6574dd0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d4dd13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74dd17, 33 },
    { 0xf7d4dd18, 93 },
    { 0, 0 },
    { 0x84f4dd1a, 95 },
    { 0xc0d4dd1b, 76 },
    { 0xf5d4dd1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e04dd1f, 76 },
    { 0x2fa4dd1f, 76 },
    { 0xf964dd21, 76 },
    { 0x9284dd22, 98 },
    { 0, 0 },
    { 0x7104dd24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef4dd27, 33 },
    { 0xf294dd28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4dd2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c84dd31, 85 },
    { 0xf7d4dd32, 85 },
    { 0, 0 },
    { 0xf4c4dd34, 95 },
    { 0xce14dd34, 33 },
    { 0x1044dd36, 61 },
    { 0xbfc4dd36, 76 },
    { 0x36d4dd38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa4dd3c, 33 },
    { 0xb864dd3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e4dd40, 55 },
    { 0x77f4dd41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae94dd47, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b4dd4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e4dd4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4dd5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc934dd5f, 98 },
    { 0x7444dd5f, 23 },
    { 0xd444dd61, 23 },
    { 0x3d44dd62, 55 },
    { 0, 0 },
    { 0x62b4dd64, 23 },
    { 0xb2d4dd64, 90 },
    { 0x5364dd66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe294dd6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3934dd72, 61 },
    { 0xd7c4dd73, 33 },
    { 0, 0 },
    { 0x57b4dd75, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc24dd7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc4dd7f, 98 },
    { 0x90c4dd80, 67 },
    { 0, 0 },
    { 0x9f14dd82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f4dd8c, 90 },
    { 0xc924dd8d, 36 },
    { 0, 0 },
    { 0x5704dd8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4914dd94, 95 },
    { 0, 0 },
    { 0x2164dd96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8744dd9a, 46 },
    { 0x5db4dd9a, 99 },
    { 0x7794dd9c, 90 },
    { 0x3e14dd9c, 98 },
    { 0x25f4dd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d4dda2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c4dda5, 33 },
    { 0, 0 },
    { 0x93a4dda7, 61 },
    { 0, 0 },
    { 0xcd24dda9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2114ddb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc94ddb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4ddba, 90 },
    { 0xb3f4ddbb, 76 },
    { 0x75f4ddbb, 36 },
    { 0x3644ddbd, 98 },
    { 0xd7e4ddbe, 87 },
    { 0xa444ddbf, 33 },
    { 0xd934ddbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4ddc4, 33 },
    { 0x5b74ddc4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8744ddca, 34 },
    { 0, 0 },
    { 0xb444ddcc, 90 },
    { 0x3da4ddcd, 99 },
    { 0x784ddcd, 98 },
    { 0x3fb4ddcf, 88 },
    { 0, 0 },
    { 0xf1d4ddd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c94ddd7, 98 },
    { 0x9454ddd7, 93 },
    { 0x9164ddd9, 34 },
    { 0, 0 },
    { 0x4d04dddb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc034dde2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd254dde5, 95 },
    { 0x7ce4dde5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94ddeb, 61 },
    { 0xe4d4ddec, 90 },
    { 0xd3d4dded, 33 },
    { 0x5fe4ddee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54ddf7, 98 },
    { 0x5244ddf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b4de01, 78 },
    { 0, 0 },
    { 0xc834de03, 100 },
    { 0x60a4de04, 23 },
    { 0x3504de05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be4de0b, 23 },
    { 0, 0 },
    { 0xd934de0d, 55 },
    { 0xd4b4de0e, 100 },
    { 0xd5b4de0e, 23 },
    { 0xd464de0f, 95 },
    { 0xa424de11, 52 },
    { 0xfda4de12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724de15, 95 },
    { 0, 0 },
    { 0xb574de17, 61 },
    { 0, 0 },
    { 0x8634de19, 33 },
    { 0x6f44de1a, 76 },
    { 0xed24de1a, 61 },
    { 0xb624de1c, 90 },
    { 0xb2f4de1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f4de28, 68 },
    { 0, 0 },
    { 0xdd44de2a, 98 },
    { 0x9db4de2b, 33 },
    { 0x90f4de2b, 40 },
    { 0, 0 },
    { 0xc564de2e, 23 },
    { 0x2d14de2e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf4de35, 65 },
    { 0xd74de35, 23 },
    { 0, 0 },
    { 0xe354de38, 99 },
    { 0x84f4de38, 33 },
    { 0xb8b4de3a, 93 },
    { 0x7eb4de3b, 52 },
    { 0x2ee4de3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4de41, 88 },
    { 0xf4d4de42, 90 },
    { 0xf574de43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c4de48, 98 },
    { 0xe474de48, 23 },
    { 0, 0 },
    { 0xdb14de4b, 66 },
    { 0, 0 },
    { 0x9e84de4d, 90 },
    { 0, 0 },
    { 0x65b4de4f, 95 },
    { 0, 0 },
    { 0xbf4de51, 93 },
    { 0xa9a4de51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84de55, 46 },
    { 0x1e84de55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe674de59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb4de5e, 95 },
    { 0, 0 },
    { 0x35a4de60, 90 },
    { 0, 0 },
    { 0x7104de62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74de65, 23 },
    { 0x2ea4de65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b34de6a, 91 },
    { 0x5b64de6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3634de6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4de73, 95 },
    { 0xe114de74, 76 },
    { 0xc2a4de75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384de79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5694de80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3904de85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9844de8d, 83 },
    { 0, 0 },
    { 0x2af4de8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab4de93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074de96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54de99, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7884de9c, 55 },
    { 0xd974de9d, 85 },
    { 0x8ad4de9c, 95 },
    { 0xecc4de9e, 67 },
    { 0xbdc4dea0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34dea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2284dea6, 95 },
    { 0, 0 },
    { 0x2bd4dea8, 87 },
    { 0xbbf4dea9, 33 },
    { 0x7c94deaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc264deae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd414deb4, 83 },
    { 0, 0 },
    { 0xb4b4deb6, 61 },
    { 0, 0 },
    { 0x6b74deb8, 66 },
    { 0, 0 },
    { 0x13c4deba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa394debf, 55 },
    { 0x3af4debf, 91 },
    { 0xb744dec1, 52 },
    { 0x1734dec2, 84 },
    { 0xc714dec2, 23 },
    { 0x2464dec3, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4decd, 90 },
    { 0xef34dece, 90 },
    { 0x8214decd, 99 },
    { 0xe754decf, 23 },
    { 0, 0 },
    { 0x1714ded2, 92 },
    { 0x43e4ded3, 87 },
    { 0x6074ded3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae4ded8, 46 },
    { 0x354ded8, 90 },
    { 0xc034deda, 61 },
    { 0x7434dedb, 33 },
    { 0, 0 },
    { 0xb284dedd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894dee9, 46 },
    { 0x9564deea, 46 },
    { 0, 0 },
    { 0x1274deec, 61 },
    { 0, 0 },
    { 0x44a4deee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c74def1, 69 },
    { 0, 0 },
    { 0x6f64def3, 33 },
    { 0xbc04def4, 33 },
    { 0, 0 },
    { 0x1194def6, 77 },
    { 0x1844def6, 61 },
    { 0x32a4def8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe4defe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a4df04, 90 },
    { 0x7434df05, 33 },
    { 0xf94df06, 33 },
    { 0xa914df06, 55 },
    { 0x8a4df08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54df10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e4df16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa184df19, 34 },
    { 0xbca4df1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe624df24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854df2d, 98 },
    { 0x1074df2e, 23 },
    { 0xe964df2f, 65 },
    { 0, 0 },
    { 0xf354df31, 90 },
    { 0, 0 },
    { 0x50f4df33, 76 },
    { 0x7d14df34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5954df37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd384df3b, 90 },
    { 0x5d34df3c, 33 },
    { 0, 0 },
    { 0x11e4df3e, 23 },
    { 0xb834df3f, 76 },
    { 0xf2a4df40, 78 },
    { 0x1724df41, 23 },
    { 0x9a94df3f, 33 },
    { 0x5794df43, 91 },
    { 0xa544df44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14df47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24df4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4df53, 90 },
    { 0x2b94df54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9034df58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf674df5d, 95 },
    { 0, 0 },
    { 0x9ea4df5f, 61 },
    { 0x4b34df60, 61 },
    { 0x4e54df5f, 55 },
    { 0x5cf4df62, 61 },
    { 0xebf4df61, 87 },
    { 0xcac4df5f, 90 },
    { 0, 0 },
    { 0x75f4df66, 33 },
    { 0xd644df67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774df6b, 61 },
    { 0x30a4df6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844df6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc4df72, 33 },
    { 0xc594df73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e74df77, 88 },
    { 0xbaa4df78, 91 },
    { 0xed74df78, 17 },
    { 0, 0 },
    { 0x4e4df7b, 23 },
    { 0x204df7c, 66 },
    { 0xa74df7d, 33 },
    { 0x2214df7e, 90 },
    { 0x7c74df7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4df82, 78 },
    { 0xe214df82, 98 },
    { 0x8a74df83, 98 },
    { 0x2e44df83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba4df8a, 33 },
    { 0, 0 },
    { 0x27b4df8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2594df94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd624df99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb4df9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed54dfa9, 98 },
    { 0x6df4dfaa, 98 },
    { 0, 0 },
    { 0xf7b4dfac, 98 },
    { 0, 0 },
    { 0xa1a4dfae, 90 },
    { 0xbe34dfaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354dfb3, 61 },
    { 0xdf84dfb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4dfb8, 100 },
    { 0xb404dfb9, 52 },
    { 0x67a4dfba, 91 },
    { 0xa6e4dfbb, 61 },
    { 0xc8a4dfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8694dfbf, 61 },
    { 0x64b4dfbf, 55 },
    { 0, 0 },
    { 0xbf04dfc2, 33 },
    { 0, 0 },
    { 0x1b4dfc4, 61 },
    { 0xaaf4dfc5, 93 },
    { 0x82b4dfc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc124dfcb, 34 },
    { 0xac34dfcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda04dfd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf824dfdb, 90 },
    { 0x5b44dfdb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8024dfe2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3044dfe8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6324dfef, 95 },
    { 0, 0 },
    { 0x4464dff1, 98 },
    { 0x2c4dff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa384dff6, 33 },
    { 0x48e4dff7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f4dffa, 34 },
    { 0, 0 },
    { 0xa704dffc, 98 },
    { 0, 0 },
    { 0xdb24dffe, 98 },
    { 0x7234dffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d14e003, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf84e007, 61 },
    { 0x3c54e008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4e00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3584e00f, 98 },
    { 0xee54e010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa984e014, 23 },
    { 0, 0 },
    { 0xd724e016, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd154e019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc4e01d, 76 },
    { 0x9224e01e, 66 },
    { 0x42e4e01f, 98 },
    { 0xbee4e020, 90 },
    { 0x9c04e01d, 65 },
    { 0xf464e022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854e026, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x614e029, 23 },
    { 0xd524e029, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b4e030, 61 },
    { 0xe7a4e031, 61 },
    { 0x1d54e032, 55 },
    { 0, 0 },
    { 0x1964e034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e54e039, 94 },
    { 0x3c84e03a, 90 },
    { 0x5e44e03b, 52 },
    { 0x2814e03c, 76 },
    { 0xe4c4e03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4e040, 66 },
    { 0x5904e040, 23 },
    { 0, 0 },
    { 0xfcd4e043, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4e04d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4e050, 90 },
    { 0xe5e4e050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf924e055, 61 },
    { 0xfe54e056, 33 },
    { 0xc1b4e057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa494e05c, 33 },
    { 0x3544e05d, 61 },
    { 0x9654e05d, 95 },
    { 0xa684e05f, 61 },
    { 0xd774e05f, 78 },
    { 0x5b24e05f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694e064, 90 },
    { 0xa004e065, 23 },
    { 0xd794e066, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce4e06a, 55 },
    { 0, 0 },
    { 0x3db4e06c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c34e073, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4e078, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b4e07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc014e084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f4e087, 61 },
    { 0x44f4e088, 77 },
    { 0x26e4e088, 90 },
    { 0, 0 },
    { 0x77e4e08b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe44e094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b4e098, 61 },
    { 0, 0 },
    { 0xf164e09a, 84 },
    { 0x7f04e09b, 61 },
    { 0, 0 },
    { 0x9634e09d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944e0a2, 33 },
    { 0x2474e0a3, 77 },
    { 0xa294e0a2, 23 },
    { 0x3214e0a2, 61 },
    { 0, 0 },
    { 0x4094e0a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654e0b4, 23 },
    { 0, 0 },
    { 0xd624e0b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4e0bc, 67 },
    { 0xd094e0bd, 85 },
    { 0x22b4e0be, 46 },
    { 0, 0 },
    { 0x54a4e0c0, 61 },
    { 0xcb04e0c1, 34 },
    { 0xc04e0c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b34e0c5, 90 },
    { 0x94e4e0c6, 91 },
    { 0x6ea4e0c5, 93 },
    { 0, 0 },
    { 0x2ed4e0c9, 23 },
    { 0x6c84e0ca, 55 },
    { 0xdbb4e0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4e0cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854e0d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a4e0d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4e0db, 95 },
    { 0, 0 },
    { 0x47f4e0dd, 23 },
    { 0xd04e0de, 90 },
    { 0x10d4e0df, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c04e0e6, 23 },
    { 0xf0b4e0e7, 55 },
    { 0x5594e0e6, 99 },
    { 0xb8a4e0e6, 87 },
    { 0xebc4e0e9, 55 },
    { 0xa654e0e8, 61 },
    { 0xa074e0ec, 95 },
    { 0xef44e0ed, 34 },
    { 0x6594e0e6, 46 },
    { 0xdce4e0ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c4e0f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2344e0f8, 33 },
    { 0x4b04e0f9, 100 },
    { 0xde04e0fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4e108, 95 },
    { 0x3904e109, 91 },
    { 0x9bb4e108, 95 },
    { 0xdd24e10a, 78 },
    { 0x7134e10a, 33 },
    { 0, 0 },
    { 0x59a4e10e, 90 },
    { 0x28b4e10f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484e117, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b84e11f, 23 },
    { 0x7f54e120, 98 },
    { 0x4b54e121, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb484e125, 33 },
    { 0, 0 },
    { 0xcc14e127, 90 },
    { 0, 0 },
    { 0x7a54e129, 81 },
    { 0, 0 },
    { 0xecb4e12b, 84 },
    { 0x9d14e12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84e12f, 82 },
    { 0xd604e130, 61 },
    { 0, 0 },
    { 0x90f4e132, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14e136, 17 },
    { 0, 0 },
    { 0xddc4e138, 99 },
    { 0x1984e139, 61 },
    { 0, 0 },
    { 0xd144e13b, 93 },
    { 0x7c64e13c, 23 },
    { 0, 0 },
    { 0xa0b4e13e, 93 },
    { 0x6b44e13f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb94e142, 90 },
    { 0xa704e142, 23 },
    { 0xc704e144, 90 },
    { 0x2ec4e144, 61 },
    { 0xb684e142, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4e14b, 61 },
    { 0xd0a4e14c, 55 },
    { 0x1aa4e14d, 98 },
    { 0xad14e14b, 90 },
    { 0x22c4e14f, 55 },
    { 0x3b44e150, 36 },
    { 0xdd24e151, 43 },
    { 0xf094e152, 95 },
    { 0, 0 },
    { 0x7da4e154, 33 },
    { 0x82e4e154, 33 },
    { 0x2554e156, 90 },
    { 0, 0 },
    { 0x9774e158, 76 },
    { 0xf7f4e159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b4e15f, 33 },
    { 0xbcd4e160, 33 },
    { 0, 0 },
    { 0x4364e162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4e16b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d64e172, 95 },
    { 0x5104e173, 95 },
    { 0x1d34e172, 90 },
    { 0, 0 },
    { 0x4164e176, 66 },
    { 0xcef4e177, 33 },
    { 0x7e04e178, 90 },
    { 0xb374e178, 61 },
    { 0, 0 },
    { 0x6cf4e17b, 23 },
    { 0x1cd4e17c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a04e17f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4024e182, 23 },
    { 0x24f4e182, 61 },
    { 0x2e74e184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b94e18a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb214e18d, 90 },
    { 0, 0 },
    { 0x45e4e18f, 90 },
    { 0, 0 },
    { 0x10f4e191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a4e19c, 67 },
    { 0x53e4e19d, 90 },
    { 0, 0 },
    { 0xfd44e19f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe24e1a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa4e1a5, 90 },
    { 0x52c4e1a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1854e1ae, 87 },
    { 0, 0 },
    { 0xefd4e1b0, 99 },
    { 0x1224e1b1, 90 },
    { 0x5054e1b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d4e1b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784e1bf, 33 },
    { 0x8b4e1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff4e1c5, 33 },
    { 0x24d4e1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3674e1cb, 75 },
    { 0xcc54e1cb, 95 },
    { 0x4684e1cb, 38 },
    { 0x7034e1cc, 98 },
    { 0x48e4e1ce, 56 },
    { 0x5624e1d0, 91 },
    { 0xb8b4e1cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284e1d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f4e1dc, 90 },
    { 0xeb54e1dd, 98 },
    { 0x8dc4e1dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4e1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f54e1e5, 33 },
    { 0xa7a4e1e5, 76 },
    { 0xff74e1e7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9964e1eb, 88 },
    { 0x8154e1ec, 61 },
    { 0x1c54e1ec, 90 },
    { 0x5354e1eb, 33 },
    { 0xbd44e1ee, 94 },
    { 0, 0 },
    { 0xf64e1f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed4e1fe, 90 },
    { 0, 0 },
    { 0x3b44e200, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d64e203, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd214e207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d04e20a, 85 },
    { 0, 0 },
    { 0x9204e20c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a24e20f, 76 },
    { 0xdd64e20f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a04e21b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4e21f, 46 },
    { 0, 0 },
    { 0xf7c4e221, 61 },
    { 0xd064e221, 91 },
    { 0x32d4e223, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f4e227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4e22b, 46 },
    { 0xa114e22c, 46 },
    { 0, 0 },
    { 0x9ab4e22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf4e233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea34e239, 33 },
    { 0x7664e239, 76 },
    { 0x1874e239, 91 },
    { 0x7654e23c, 95 },
    { 0x71d4e23a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8134e242, 85 },
    { 0, 0 },
    { 0xe584e244, 95 },
    { 0xe704e245, 46 },
    { 0x1a4e245, 90 },
    { 0, 0 },
    { 0x8404e248, 33 },
    { 0, 0 },
    { 0x8f64e24a, 76 },
    { 0xa284e24b, 81 },
    { 0x3b94e24c, 55 },
    { 0xd5d4e24b, 95 },
    { 0xa874e24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64e253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a4e257, 90 },
    { 0x2fd4e258, 89 },
    { 0x3b24e258, 61 },
    { 0x5954e258, 99 },
    { 0xca4e25b, 33 },
    { 0xb1c4e258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa154e260, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a4e26f, 43 },
    { 0x71b4e270, 33 },
    { 0xf564e271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd14e274, 46 },
    { 0, 0 },
    { 0x3d4e276, 61 },
    { 0xbd04e276, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4e27c, 78 },
    { 0xd634e27d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6164e283, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5104e28a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4e28f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc4e292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd4e295, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f54e29b, 34 },
    { 0x1d34e29c, 55 },
    { 0x1674e29c, 95 },
    { 0x53a4e29d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca24e2a3, 80 },
    { 0x84a4e2a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf54e2aa, 90 },
    { 0xdb4e2aa, 55 },
    { 0x4954e2ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7474e2ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd4e2c2, 33 },
    { 0x3194e2c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294e2cb, 61 },
    { 0x7e44e2cc, 76 },
    { 0x38c4e2cc, 98 },
    { 0, 0 },
    { 0x3b4e2cf, 64 },
    { 0, 0 },
    { 0xf384e2d1, 61 },
    { 0, 0 },
    { 0xfd94e2d3, 34 },
    { 0x124e2d4, 33 },
    { 0x85e4e2d3, 98 },
    { 0xd2e4e2d6, 89 },
    { 0xb7d4e2d6, 95 },
    { 0xfcb4e2d8, 77 },
    { 0x6b04e2d9, 76 },
    { 0xb5d4e2d9, 61 },
    { 0, 0 },
    { 0x1b24e2dc, 99 },
    { 0x1424e2dd, 76 },
    { 0x4f64e2dc, 100 },
    { 0x9e74e2df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c4e2e3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1264e2e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe094e2ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a4e2f1, 43 },
    { 0x3da4e2f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec4e2f5, 95 },
    { 0, 0 },
    { 0x1e04e2f7, 34 },
    { 0, 0 },
    { 0x2284e2f9, 90 },
    { 0x95e4e2f9, 66 },
    { 0xe2e4e2fb, 55 },
    { 0x9324e2fa, 95 },
    { 0x1404e2fa, 90 },
    { 0x6c94e2f9, 23 },
    { 0xbae4e2fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94e306, 90 },
    { 0xb5b4e307, 55 },
    { 0xe3b4e307, 23 },
    { 0xfda4e308, 68 },
    { 0x6bc4e307, 90 },
    { 0, 0 },
    { 0x8754e30c, 95 },
    { 0x64e4e30c, 23 },
    { 0xf824e30d, 95 },
    { 0x7384e30f, 66 },
    { 0x4284e30e, 33 },
    { 0x2f74e30f, 81 },
    { 0xcaf4e30d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c34e317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a14e31a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4e321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf474e324, 100 },
    { 0x5324e325, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a4e32a, 76 },
    { 0xab04e32b, 33 },
    { 0, 0 },
    { 0x8694e32d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3344e331, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6724e337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e94e33a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174e33f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4e343, 77 },
    { 0x3cc4e344, 90 },
    { 0x9334e344, 66 },
    { 0x6924e346, 17 },
    { 0, 0 },
    { 0xd534e348, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34e34f, 33 },
    { 0xa624e350, 32 },
    { 0xb384e350, 76 },
    { 0xcd54e350, 23 },
    { 0x9a14e352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4214e35c, 17 },
    { 0x34e4e35d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94e361, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b4e364, 90 },
    { 0x74d4e365, 77 },
    { 0x4284e365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4e36a, 90 },
    { 0, 0 },
    { 0x5354e36c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4e371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8654e375, 90 },
    { 0x77a4e376, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce4e37b, 78 },
    { 0xd534e37c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6064e388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed4e392, 19 },
    { 0x784e393, 61 },
    { 0x7614e392, 75 },
    { 0xd994e394, 76 },
    { 0x1df4e396, 61 },
    { 0x64e397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc014e39d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1314e3a0, 61 },
    { 0x6ca4e3a1, 95 },
    { 0x8064e3a2, 23 },
    { 0x20a4e3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e3a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1904e3ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5894e3b4, 90 },
    { 0, 0 },
    { 0x58f4e3b6, 76 },
    { 0xde44e3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b84e3bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524e3c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db4e3c5, 55 },
    { 0x8734e3c6, 34 },
    { 0x24c4e3c6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e4e3ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4924e3d3, 84 },
    { 0x7414e3d4, 90 },
    { 0, 0 },
    { 0x19e4e3d6, 46 },
    { 0, 0 },
    { 0x5cc4e3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34e3dd, 90 },
    { 0, 0 },
    { 0x7544e3df, 46 },
    { 0xdc24e3df, 61 },
    { 0, 0 },
    { 0xc8d4e3e2, 23 },
    { 0, 0 },
    { 0xaf24e3e4, 98 },
    { 0xc574e3e5, 90 },
    { 0x5b84e3e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334e3eb, 99 },
    { 0, 0 },
    { 0xd484e3ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c4e3f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca4e3f6, 33 },
    { 0, 0 },
    { 0xafb4e3f8, 98 },
    { 0x14e4e3f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e401, 95 },
    { 0x14e4e402, 99 },
    { 0xc784e403, 61 },
    { 0xc404e401, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab4e408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe64e40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7924e412, 61 },
    { 0x9094e413, 97 },
    { 0x49b4e413, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba4e419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2994e41c, 46 },
    { 0xad04e41c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e420, 77 },
    { 0x96c4e420, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e4e425, 84 },
    { 0x4464e425, 33 },
    { 0xd394e426, 90 },
    { 0x56b4e428, 99 },
    { 0xaf24e427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4e433, 23 },
    { 0x4a84e434, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9274e439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44e440, 98 },
    { 0x4fc4e440, 98 },
    { 0x8d14e441, 55 },
    { 0, 0 },
    { 0xbf44e444, 90 },
    { 0, 0 },
    { 0x3944e446, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4484e44e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a4e453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454e456, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e459, 66 },
    { 0xdb64e45a, 23 },
    { 0x4d64e45a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e460, 95 },
    { 0xd434e461, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa674e473, 23 },
    { 0, 0 },
    { 0x5d14e475, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e4e478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4e47b, 76 },
    { 0xe044e47b, 90 },
    { 0x9294e47d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b74e482, 46 },
    { 0, 0 },
    { 0xb9f4e484, 33 },
    { 0x4764e485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4e488, 23 },
    { 0xcd4e489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d4e48d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df4e493, 90 },
    { 0, 0 },
    { 0x5014e495, 90 },
    { 0x7c94e496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1184e499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4e49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9654e4a8, 98 },
    { 0x38a4e4a9, 23 },
    { 0, 0 },
    { 0xec4e4ab, 61 },
    { 0xd384e4ac, 33 },
    { 0, 0 },
    { 0xc544e4ae, 33 },
    { 0xcd84e4af, 67 },
    { 0x9874e4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac4e4b4, 34 },
    { 0x2664e4b4, 33 },
    { 0x9b14e4b6, 93 },
    { 0x16d4e4b6, 33 },
    { 0x2bf4e4b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614e4bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4604e4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f4e4c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd834e4c7, 90 },
    { 0, 0 },
    { 0x3774e4c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e4e4cd, 34 },
    { 0xbfe4e4ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed64e4d5, 17 },
    { 0x6574e4d6, 90 },
    { 0xa4d4e4d5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db4e4dd, 78 },
    { 0x6744e4de, 95 },
    { 0xefa4e4df, 90 },
    { 0x28b4e4e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4e4e5, 95 },
    { 0, 0 },
    { 0xdd04e4e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734e4ea, 23 },
    { 0xfa44e4ea, 98 },
    { 0, 0 },
    { 0xd124e4ed, 95 },
    { 0x1f4e4ed, 98 },
    { 0x4bf4e4ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4e4f5, 90 },
    { 0x1ee4e4f6, 100 },
    { 0x9324e4f7, 61 },
    { 0x14c4e4f8, 61 },
    { 0xb434e4f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd4e501, 76 },
    { 0xe7b4e501, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a14e505, 91 },
    { 0x634e506, 61 },
    { 0xaa24e507, 81 },
    { 0x4ac4e505, 93 },
    { 0x9ba4e505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f74e50d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5254e515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5634e518, 76 },
    { 0xbbd4e519, 78 },
    { 0x5d64e51a, 81 },
    { 0, 0 },
    { 0xbd54e51c, 55 },
    { 0x50d4e51d, 77 },
    { 0x9414e51d, 76 },
    { 0, 0 },
    { 0x44a4e520, 33 },
    { 0, 0 },
    { 0x7454e522, 61 },
    { 0xbd14e523, 96 },
    { 0x6ad4e524, 99 },
    { 0x1c94e522, 78 },
    { 0x9844e526, 96 },
    { 0xcb44e526, 98 },
    { 0x9904e525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd4e52b, 90 },
    { 0x5864e52c, 88 },
    { 0x85d4e52c, 91 },
    { 0x6be4e52d, 90 },
    { 0xbc34e52f, 34 },
    { 0x6614e52f, 98 },
    { 0, 0 },
    { 0x4eb4e532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd54e535, 55 },
    { 0x7034e536, 88 },
    { 0x7c64e537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d4e53b, 90 },
    { 0xcc34e53c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec4e53f, 76 },
    { 0x2a44e53f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d4e543, 61 },
    { 0x50b4e544, 61 },
    { 0, 0 },
    { 0x124e546, 98 },
    { 0x2ea4e547, 69 },
    { 0, 0 },
    { 0x2b34e549, 90 },
    { 0, 0 },
    { 0x25e4e54b, 92 },
    { 0xf9a4e54b, 46 },
    { 0xb704e54d, 90 },
    { 0, 0 },
    { 0x3574e54f, 93 },
    { 0xd114e550, 95 },
    { 0x2114e551, 93 },
    { 0x8344e552, 34 },
    { 0xbaf4e551, 95 },
    { 0x5974e554, 34 },
    { 0x5714e553, 84 },
    { 0xba04e556, 34 },
    { 0xf664e556, 33 },
    { 0, 0 },
    { 0xafd4e559, 66 },
    { 0x24e559, 77 },
    { 0x7ae4e55a, 98 },
    { 0x1c04e55c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24e561, 77 },
    { 0, 0 },
    { 0x6854e563, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914e56c, 84 },
    { 0x10a4e56d, 23 },
    { 0xf164e56e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d4e574, 98 },
    { 0x3424e575, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b4e578, 23 },
    { 0, 0 },
    { 0x12b4e57a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf384e57e, 23 },
    { 0xb304e57f, 61 },
    { 0x3534e57f, 33 },
    { 0xa5e4e581, 90 },
    { 0, 0 },
    { 0x1a84e583, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34e58f, 90 },
    { 0x2154e58f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594e595, 90 },
    { 0xca74e596, 76 },
    { 0xe094e597, 91 },
    { 0, 0 },
    { 0xa844e599, 23 },
    { 0xd824e59a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804e59f, 90 },
    { 0, 0 },
    { 0x53c4e5a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6704e5ae, 66 },
    { 0, 0 },
    { 0x5324e5b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e5b3, 91 },
    { 0x31d4e5b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4e5b8, 23 },
    { 0x6924e5b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14e5bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a4e5c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff4e5c5, 46 },
    { 0x6944e5c6, 90 },
    { 0, 0 },
    { 0x8614e5c8, 91 },
    { 0x69b4e5c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf94e5ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a4e5d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5644e5d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4e5da, 84 },
    { 0x5f54e5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5984e5df, 88 },
    { 0x2f14e5e0, 33 },
    { 0, 0 },
    { 0xe264e5e2, 97 },
    { 0, 0 },
    { 0xb194e5e4, 68 },
    { 0xf634e5e5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e14e5eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3364e5f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa544e5f7, 90 },
    { 0, 0 },
    { 0x9514e5f9, 66 },
    { 0, 0 },
    { 0x5d44e5fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c4e600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04e603, 90 },
    { 0, 0 },
    { 0xbb34e605, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa984e608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb4e60b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4e60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4e612, 93 },
    { 0x3eb4e613, 34 },
    { 0x2e14e614, 93 },
    { 0x6584e615, 33 },
    { 0, 0 },
    { 0x7a94e617, 100 },
    { 0x3924e618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c54e620, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa4e624, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f4e62a, 40 },
    { 0x14c4e62a, 61 },
    { 0, 0 },
    { 0x5c94e62d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744e63a, 61 },
    { 0x854e63a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d54e641, 97 },
    { 0, 0 },
    { 0x6344e643, 95 },
    { 0, 0 },
    { 0x96a4e645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b4e648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc204e64b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef34e651, 98 },
    { 0, 0 },
    { 0x8d24e653, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9654e665, 93 },
    { 0, 0 },
    { 0xc9d4e667, 78 },
    { 0x2be4e668, 55 },
    { 0, 0 },
    { 0x6894e66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc444e66f, 84 },
    { 0xe5a4e66f, 33 },
    { 0x1a4e670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf64e678, 99 },
    { 0xaba4e679, 46 },
    { 0xb04e67a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd884e67e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454e68d, 98 },
    { 0, 0 },
    { 0x79d4e68f, 90 },
    { 0, 0 },
    { 0xdaa4e691, 90 },
    { 0, 0 },
    { 0x24a4e693, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24e696, 78 },
    { 0xae14e697, 36 },
    { 0x6fb4e698, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4e6a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04e6a9, 98 },
    { 0, 0 },
    { 0xb864e6ab, 33 },
    { 0x1ed4e6ac, 23 },
    { 0x22c4e6ac, 23 },
    { 0xfc84e6ae, 90 },
    { 0, 0 },
    { 0x764e6b0, 66 },
    { 0x6174e6b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef4e6b6, 95 },
    { 0x2564e6b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544e6bb, 90 },
    { 0, 0 },
    { 0xe454e6bd, 88 },
    { 0xfd44e6bd, 46 },
    { 0x64f4e6bd, 61 },
    { 0, 0 },
    { 0xd074e6c1, 98 },
    { 0, 0 },
    { 0x98d4e6c3, 67 },
    { 0x7ae4e6c4, 23 },
    { 0x9de4e6c3, 81 },
    { 0x87e4e6c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4414e6cb, 95 },
    { 0xd374e6cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c4e6d3, 76 },
    { 0x2494e6d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b4e6dd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24e6e0, 40 },
    { 0xfdc4e6e0, 98 },
    { 0xa2b4e6e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc84e6e7, 33 },
    { 0xdc54e6e7, 33 },
    { 0x98d4e6e9, 52 },
    { 0x8924e6e8, 65 },
    { 0x8604e6e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64e6ef, 66 },
    { 0, 0 },
    { 0xaeb4e6f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9184e6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e6fc, 95 },
    { 0, 0 },
    { 0x6654e6fe, 23 },
    { 0, 0 },
    { 0x7c44e700, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f64e703, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2384e70d, 61 },
    { 0, 0 },
    { 0x5804e70f, 95 },
    { 0x1894e710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a4e713, 98 },
    { 0xfd04e714, 33 },
    { 0x78a4e713, 23 },
    { 0x76c4e716, 33 },
    { 0x2064e717, 98 },
    { 0, 0 },
    { 0x4164e719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694e71e, 46 },
    { 0x4c44e71f, 95 },
    { 0xbc34e71e, 98 },
    { 0xdf94e71f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da4e727, 33 },
    { 0x21e4e728, 90 },
    { 0xf1c4e728, 87 },
    { 0x97e4e72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2434e733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb034e738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24e73e, 66 },
    { 0, 0 },
    { 0xfc44e740, 66 },
    { 0x2904e740, 89 },
    { 0, 0 },
    { 0x58a4e743, 43 },
    { 0xe744e744, 33 },
    { 0x63d4e745, 61 },
    { 0xeaa4e746, 23 },
    { 0xec64e743, 61 },
    { 0x1f94e744, 95 },
    { 0x9374e749, 87 },
    { 0x5174e748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac4e74d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf54e753, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed4e757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4e75b, 33 },
    { 0xf844e75b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe474e75f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee4e762, 34 },
    { 0, 0 },
    { 0xc9a4e764, 98 },
    { 0x7c44e764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884e76a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf64e76d, 95 },
    { 0x1784e76e, 33 },
    { 0, 0 },
    { 0x7fc4e770, 100 },
    { 0x8544e770, 34 },
    { 0, 0 },
    { 0x38e4e773, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8394e77a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74e77f, 90 },
    { 0x34e780, 93 },
    { 0x9164e781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa04e78c, 34 },
    { 0, 0 },
    { 0xede4e78e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd454e794, 85 },
    { 0, 0 },
    { 0xb474e796, 87 },
    { 0xdf94e797, 23 },
    { 0x8ed4e798, 99 },
    { 0x1d14e797, 33 },
    { 0xc7f4e799, 95 },
    { 0x8694e798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f04e79e, 93 },
    { 0xcd44e79e, 95 },
    { 0x4914e7a0, 33 },
    { 0xc7c4e7a1, 23 },
    { 0x14f4e7a2, 43 },
    { 0, 0 },
    { 0x7af4e7a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3494e7a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344e7af, 90 },
    { 0xa5d4e7af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c74e7b4, 95 },
    { 0, 0 },
    { 0x6084e7b6, 99 },
    { 0xb7f4e7b6, 34 },
    { 0xb054e7b8, 55 },
    { 0, 0 },
    { 0xf474e7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8694e7bd, 55 },
    { 0x8084e7be, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4e7ca, 46 },
    { 0, 0 },
    { 0x4aa4e7cc, 52 },
    { 0xdae4e7cd, 95 },
    { 0xf4b4e7ce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214e7d5, 61 },
    { 0x7a84e7d6, 92 },
    { 0xa884e7d7, 61 },
    { 0x6294e7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac4e7de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4e7e1, 36 },
    { 0xba4e7e2, 95 },
    { 0x5d4e7e1, 17 },
    { 0, 0 },
    { 0xef84e7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404e7f1, 55 },
    { 0x1b74e7f2, 91 },
    { 0x7e34e7f3, 61 },
    { 0x50c4e7f4, 95 },
    { 0x58f4e7f5, 84 },
    { 0x2254e7f6, 95 },
    { 0x64f4e7f7, 23 },
    { 0x3214e7f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984e7fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe854e806, 33 },
    { 0, 0 },
    { 0x8ec4e808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd4e80e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf4e817, 55 },
    { 0xa004e817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4e81b, 33 },
    { 0x8f54e81c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a34e81f, 90 },
    { 0x2744e820, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d44e824, 52 },
    { 0x214e825, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054e829, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1824e82d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794e830, 46 },
    { 0xd614e831, 95 },
    { 0x57f4e832, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f4e837, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c44e83f, 76 },
    { 0, 0 },
    { 0x1ff4e841, 90 },
    { 0xc054e841, 23 },
    { 0x9404e842, 87 },
    { 0xecf4e844, 30 },
    { 0, 0 },
    { 0xf844e846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa104e849, 95 },
    { 0x65c4e849, 76 },
    { 0, 0 },
    { 0xda64e84c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14e858, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3874e85e, 68 },
    { 0x6484e85e, 95 },
    { 0, 0 },
    { 0x1b04e861, 90 },
    { 0xad84e862, 100 },
    { 0, 0 },
    { 0x7c94e864, 33 },
    { 0, 0 },
    { 0x3c14e866, 69 },
    { 0x254e867, 67 },
    { 0, 0 },
    { 0xbf74e869, 93 },
    { 0x88d4e86a, 90 },
    { 0xad34e86b, 33 },
    { 0xb314e86c, 95 },
    { 0xd4e86c, 98 },
    { 0xeb54e86e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4e873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1884e877, 43 },
    { 0xf834e878, 24 },
    { 0x4934e878, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554e87c, 75 },
    { 0x34e4e87d, 91 },
    { 0xa454e87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884e883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654e886, 33 },
    { 0x16e4e887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74e894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4e8a1, 95 },
    { 0x2d4e8a1, 76 },
    { 0x1ad4e8a2, 95 },
    { 0xf2a4e8a4, 88 },
    { 0xa204e8a5, 90 },
    { 0, 0 },
    { 0x2704e8a7, 95 },
    { 0x1ff4e8a8, 81 },
    { 0x2714e8a9, 91 },
    { 0xae84e8aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b24e8ae, 61 },
    { 0, 0 },
    { 0x5934e8b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb984e8bd, 76 },
    { 0, 0 },
    { 0x2f14e8bf, 76 },
    { 0, 0 },
    { 0xb4d4e8c1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6144e8c6, 93 },
    { 0, 0 },
    { 0x9e04e8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea4e8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f4e8d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d14e8d3, 90 },
    { 0x3414e8d3, 61 },
    { 0, 0 },
    { 0xab74e8d6, 23 },
    { 0x8de4e8d7, 95 },
    { 0xbd64e8d7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4e8db, 78 },
    { 0xa2b4e8db, 23 },
    { 0x8384e8dd, 91 },
    { 0, 0 },
    { 0xaaf4e8df, 23 },
    { 0x59e4e8e0, 61 },
    { 0xfd84e8e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca4e8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f4e8e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b64e8ed, 23 },
    { 0, 0 },
    { 0x2df4e8ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d34e8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc714e8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454e8fb, 90 },
    { 0, 0 },
    { 0xcde4e8fd, 34 },
    { 0x4b24e8fe, 33 },
    { 0x3ce4e8ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac4e904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b24e90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea4e910, 61 },
    { 0x7e44e910, 81 },
    { 0x5f4e912, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa554e915, 66 },
    { 0xdca4e916, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5164e919, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d44e91c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c84e929, 23 },
    { 0, 0 },
    { 0x8d64e92b, 23 },
    { 0x4bf4e92c, 84 },
    { 0x1204e92d, 53 },
    { 0xd9a4e92d, 40 },
    { 0x9694e92e, 33 },
    { 0x73d4e92e, 33 },
    { 0x9db4e931, 33 },
    { 0xdc14e932, 53 },
    { 0xf64e932, 90 },
    { 0xc954e934, 34 },
    { 0x94c4e934, 67 },
    { 0x4154e92c, 23 },
    { 0xe6c4e932, 90 },
    { 0xaa74e92f, 90 },
    { 0, 0 },
    { 0xef04e93a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe414e93f, 92 },
    { 0x6094e940, 34 },
    { 0, 0 },
    { 0x41e4e942, 46 },
    { 0x4734e943, 83 },
    { 0xe684e944, 78 },
    { 0x14d4e943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c4e948, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e4e94f, 23 },
    { 0xda74e950, 82 },
    { 0xf34e951, 93 },
    { 0xe814e951, 23 },
    { 0xe254e953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5184e956, 52 },
    { 0xe094e957, 61 },
    { 0xe894e958, 33 },
    { 0xcef4e959, 81 },
    { 0xfaa4e95a, 65 },
    { 0xd3e4e95b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea4e962, 78 },
    { 0, 0 },
    { 0x6b74e964, 98 },
    { 0, 0 },
    { 0x9634e966, 33 },
    { 0x4194e967, 23 },
    { 0xfc04e966, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb44e96b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b4e973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc484e977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa864e97a, 98 },
    { 0x504e97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb04e980, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a64e984, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4e987, 33 },
    { 0x8ff4e988, 43 },
    { 0, 0 },
    { 0xfdd4e98a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4e98e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1384e993, 55 },
    { 0xbae4e994, 90 },
    { 0x9014e995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd304e998, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804e99d, 99 },
    { 0xb84e99e, 95 },
    { 0, 0 },
    { 0x7d64e9a0, 93 },
    { 0xf0e4e9a1, 43 },
    { 0x10e4e9a2, 90 },
    { 0x9264e9a2, 34 },
    { 0x5ed4e9a0, 99 },
    { 0xe734e9a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x774e9a8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b4e9ab, 67 },
    { 0x16c4e9ab, 61 },
    { 0x5894e9ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2514e9b0, 90 },
    { 0x7794e9b1, 99 },
    { 0, 0 },
    { 0x7fa4e9b3, 43 },
    { 0, 0 },
    { 0xcda4e9b5, 85 },
    { 0x8464e9b6, 76 },
    { 0xc254e9b5, 76 },
    { 0xcfb4e9b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa74e9ca, 95 },
    { 0, 0 },
    { 0xe404e9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8034e9d0, 33 },
    { 0xcc14e9d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4e9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac14e9da, 95 },
    { 0x384e9da, 33 },
    { 0xc584e9dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e44e9df, 85 },
    { 0, 0 },
    { 0x37b4e9e1, 23 },
    { 0xc274e9e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3154e9eb, 95 },
    { 0xf224e9ec, 23 },
    { 0, 0 },
    { 0x6fa4e9ee, 33 },
    { 0x41d4e9ef, 34 },
    { 0x3964e9f0, 33 },
    { 0x5b64e9f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd554e9f4, 90 },
    { 0xd714e9f4, 23 },
    { 0xaa24e9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4ea02, 90 },
    { 0xb7d4ea02, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c4ea08, 61 },
    { 0x1774ea09, 75 },
    { 0x2a54ea08, 67 },
    { 0x8fa4ea09, 43 },
    { 0x3f94ea0c, 78 },
    { 0, 0 },
    { 0x5764ea0e, 55 },
    { 0xea94ea0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344ea15, 23 },
    { 0x3164ea16, 61 },
    { 0, 0 },
    { 0x5704ea18, 56 },
    { 0x8d4ea19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7834ea1c, 55 },
    { 0xaf64ea1d, 33 },
    { 0, 0 },
    { 0xac24ea1f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f4ea23, 36 },
    { 0x99a4ea23, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134ea27, 90 },
    { 0, 0 },
    { 0xc594ea29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b44ea2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b84ea32, 61 },
    { 0x2c94ea33, 90 },
    { 0xdbc4ea32, 98 },
    { 0xec74ea35, 61 },
    { 0, 0 },
    { 0x1c64ea37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874ea44, 95 },
    { 0xaa34ea45, 95 },
    { 0x30c4ea46, 87 },
    { 0xf554ea47, 46 },
    { 0x2ca4ea48, 98 },
    { 0xfcd4ea49, 80 },
    { 0, 0 },
    { 0x85b4ea4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e4ea52, 33 },
    { 0x9254ea53, 61 },
    { 0, 0 },
    { 0x9a04ea55, 23 },
    { 0x7204ea55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4724ea5b, 43 },
    { 0x84e4ea5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f4ea60, 90 },
    { 0xe024ea61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c4ea6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4ea6d, 17 },
    { 0, 0 },
    { 0x15d4ea6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc004ea72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534ea79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f64ea7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94ea83, 23 },
    { 0x7af4ea84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564ea8c, 23 },
    { 0, 0 },
    { 0xe004ea8e, 68 },
    { 0x21a4ea8f, 34 },
    { 0x5f24ea8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3754ea95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b64ea9e, 55 },
    { 0x76f4ea9f, 95 },
    { 0, 0 },
    { 0x9ba4eaa1, 76 },
    { 0x8f94eaa2, 55 },
    { 0, 0 },
    { 0x7864eaa4, 61 },
    { 0xe1b4eaa5, 90 },
    { 0x544eaa6, 61 },
    { 0x6264eaa7, 61 },
    { 0x1434eaa8, 98 },
    { 0x3e54eaa7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f4eaac, 23 },
    { 0xd804eaac, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814eab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d94eab9, 76 },
    { 0, 0 },
    { 0x1a54eabb, 34 },
    { 0, 0 },
    { 0x4584eabd, 76 },
    { 0xff64eabe, 34 },
    { 0xc494eabf, 61 },
    { 0, 0 },
    { 0xf884eac1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354eac5, 33 },
    { 0xf194eac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb4eac9, 61 },
    { 0x3d4eaca, 95 },
    { 0x64e4eacb, 100 },
    { 0x4784eacb, 90 },
    { 0xaea4eacd, 61 },
    { 0x3484eace, 95 },
    { 0xf9c4eace, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf4ead2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4ead6, 98 },
    { 0, 0 },
    { 0x3a14ead8, 76 },
    { 0x3e04ead9, 61 },
    { 0x9404eada, 46 },
    { 0x8724ead9, 95 },
    { 0xe34ead9, 90 },
    { 0x5724eadd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa64eae0, 90 },
    { 0xff4eae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8454eaf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654eaf8, 66 },
    { 0, 0 },
    { 0x6ab4eafa, 76 },
    { 0x2c24eafb, 23 },
    { 0, 0 },
    { 0x69a4eafd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4eb03, 95 },
    { 0x3504eb04, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad14eb0c, 87 },
    { 0, 0 },
    { 0x3b94eb0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f4eb11, 23 },
    { 0, 0 },
    { 0x5774eb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd294eb16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f54eb19, 76 },
    { 0x5564eb1a, 34 },
    { 0x8914eb1a, 90 },
    { 0xbac4eb1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb4eb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee4eb2e, 87 },
    { 0x57c4eb2f, 23 },
    { 0x3134eb2f, 61 },
    { 0x2194eb2f, 92 },
    { 0, 0 },
    { 0xc1b4eb33, 35 },
    { 0x4484eb33, 66 },
    { 0xa834eb34, 100 },
    { 0xa364eb34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874eb39, 64 },
    { 0xcd4eb3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6184eb3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c4eb44, 85 },
    { 0, 0 },
    { 0x81c4eb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974eb4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e4eb51, 77 },
    { 0x5a94eb51, 95 },
    { 0xbf4eb53, 23 },
    { 0, 0 },
    { 0xf094eb55, 23 },
    { 0x23e4eb56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f84eb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344eb61, 95 },
    { 0, 0 },
    { 0xc914eb63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4eb67, 61 },
    { 0xa514eb68, 98 },
    { 0xd334eb69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4eb6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d4eb71, 33 },
    { 0xf6c4eb72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb64eb78, 91 },
    { 0xfb4eb79, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a4eb86, 61 },
    { 0xabc4eb86, 23 },
    { 0x50a4eb88, 98 },
    { 0xba74eb87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca4eb93, 61 },
    { 0xc1d4eb94, 75 },
    { 0x9734eb95, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4eb9b, 90 },
    { 0, 0 },
    { 0xd924eb9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef34eba2, 46 },
    { 0x2a14eba3, 95 },
    { 0, 0 },
    { 0xca94eba5, 33 },
    { 0x7aa4eba5, 43 },
    { 0xf0e4eba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4ebab, 33 },
    { 0x5b4ebac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4ebb2, 93 },
    { 0xe964ebb3, 100 },
    { 0, 0 },
    { 0xb4c4ebb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354ebbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a4ebc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9724ebc6, 98 },
    { 0, 0 },
    { 0xf444ebc8, 78 },
    { 0, 0 },
    { 0xffd4ebca, 34 },
    { 0xa434ebca, 34 },
    { 0x25a4ebcc, 17 },
    { 0, 0 },
    { 0x9b84ebce, 90 },
    { 0x8554ebcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa224ebd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d4ebdd, 33 },
    { 0x8424ebde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b4ebe2, 17 },
    { 0xe7a4ebe3, 90 },
    { 0x62c4ebe3, 33 },
    { 0x9fb4ebe3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc4ebe9, 95 },
    { 0xe574ebea, 84 },
    { 0x5054ebe9, 46 },
    { 0xefd4ebea, 33 },
    { 0xdb34ebec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed4ebf1, 33 },
    { 0x5474ebf1, 90 },
    { 0, 0 },
    { 0xbdf4ebf4, 98 },
    { 0xb084ebf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a24ebfa, 52 },
    { 0x3eb4ebfb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744ebfe, 23 },
    { 0xe1f4ebff, 91 },
    { 0xcd14ec00, 52 },
    { 0, 0 },
    { 0x6fc4ec02, 61 },
    { 0x26e4ec03, 98 },
    { 0xd0d4ec03, 76 },
    { 0x3b64ec05, 93 },
    { 0xea64ec06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4ec0a, 95 },
    { 0, 0 },
    { 0xc524ec0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134ec0f, 46 },
    { 0x3a44ec0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c4ec14, 90 },
    { 0, 0 },
    { 0x334ec16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524ec19, 61 },
    { 0x7814ec1a, 43 },
    { 0xa7a4ec1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a54ec1e, 90 },
    { 0xb2e4ec1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84ec23, 90 },
    { 0xd794ec24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ec28, 95 },
    { 0, 0 },
    { 0x5974ec2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6464ec30, 97 },
    { 0, 0 },
    { 0x2784ec32, 98 },
    { 0xd2b4ec32, 23 },
    { 0x88d4ec32, 12 },
    { 0x3e34ec32, 34 },
    { 0x5844ec36, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe374ec3d, 90 },
    { 0xccc4ec3d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044ec42, 95 },
    { 0x2a84ec43, 92 },
    { 0xaa54ec43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074ec4a, 76 },
    { 0xcc34ec4b, 95 },
    { 0, 0 },
    { 0x7f04ec4d, 66 },
    { 0x24b4ec4e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4ec51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b4ec57, 67 },
    { 0x29f4ec58, 23 },
    { 0xa414ec59, 43 },
    { 0, 0 },
    { 0x9374ec5b, 95 },
    { 0, 0 },
    { 0xd6e4ec5d, 91 },
    { 0, 0 },
    { 0xf1e4ec5f, 66 },
    { 0x95e4ec5f, 76 },
    { 0xc5c4ec5f, 34 },
    { 0, 0 },
    { 0x8774ec63, 23 },
    { 0x4f14ec64, 43 },
    { 0, 0 },
    { 0x9b34ec66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad4ec6b, 99 },
    { 0, 0 },
    { 0x9a84ec6d, 46 },
    { 0xf764ec6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384ec72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4ec75, 98 },
    { 0, 0 },
    { 0x4194ec77, 98 },
    { 0, 0 },
    { 0xa04ec79, 55 },
    { 0x4e84ec7a, 91 },
    { 0x9fc4ec7b, 76 },
    { 0xa544ec79, 33 },
    { 0x9234ec7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4ec81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ec84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a4ec8a, 98 },
    { 0x37c4ec8b, 43 },
    { 0x72c4ec8b, 90 },
    { 0, 0 },
    { 0x50d4ec8e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab4ec95, 90 },
    { 0x5784ec96, 52 },
    { 0xec14ec97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814ec9a, 56 },
    { 0x7054ec9a, 98 },
    { 0x9f64ec9a, 93 },
    { 0xe994ec9d, 98 },
    { 0xeac4ec9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae64eca2, 90 },
    { 0x2314eca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb94eca8, 33 },
    { 0x98d4eca9, 87 },
    { 0x76f4eca9, 68 },
    { 0x4374eca8, 98 },
    { 0xb024ecac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1474ecb2, 90 },
    { 0xe2b4ecb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d4ecb6, 61 },
    { 0x7144ecb6, 33 },
    { 0xfb94ecb8, 90 },
    { 0x6424ecb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2054ecbe, 90 },
    { 0xfc94ecbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd64ecc6, 95 },
    { 0xbbb4ecc6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764ecca, 65 },
    { 0x7704eccb, 90 },
    { 0xeb44eccc, 77 },
    { 0xcea4ecca, 46 },
    { 0, 0 },
    { 0x53b4eccf, 100 },
    { 0, 0 },
    { 0xc094ecd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454ecd5, 34 },
    { 0xb694ecd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4ecd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc194ecdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04ece5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84ece8, 61 },
    { 0xcfd4ece8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe4ecec, 52 },
    { 0, 0 },
    { 0x4344ecee, 100 },
    { 0x47f4ecef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a4ecf4, 84 },
    { 0x1994ecf5, 33 },
    { 0xf734ecf5, 90 },
    { 0x3d24ecf5, 33 },
    { 0x7a54ecf8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4ecfb, 95 },
    { 0x24e4ecfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca4ed01, 17 },
    { 0xa804ed02, 90 },
    { 0x4a4ed03, 95 },
    { 0xe544ed04, 61 },
    { 0x1594ed03, 40 },
    { 0, 0 },
    { 0xfeb4ed07, 23 },
    { 0xd044ed08, 55 },
    { 0xbe44ed09, 95 },
    { 0x46d4ed0a, 78 },
    { 0xe864ed0b, 84 },
    { 0xab54ed0c, 38 },
    { 0, 0 },
    { 0xf6e4ed0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe54ed11, 98 },
    { 0xd5e4ed12, 88 },
    { 0x3cd4ed13, 98 },
    { 0, 0 },
    { 0xdaf4ed15, 85 },
    { 0, 0 },
    { 0xef74ed17, 95 },
    { 0, 0 },
    { 0xd864ed19, 46 },
    { 0x7be4ed1a, 23 },
    { 0x68d4ed1b, 95 },
    { 0, 0 },
    { 0x2364ed1d, 90 },
    { 0x5f74ed1d, 90 },
    { 0x90c4ed1d, 33 },
    { 0, 0 },
    { 0x7484ed21, 76 },
    { 0, 0 },
    { 0xfec4ed23, 33 },
    { 0x6594ed23, 98 },
    { 0, 0 },
    { 0x9464ed26, 61 },
    { 0xf294ed26, 33 },
    { 0, 0 },
    { 0x80f4ed29, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd124ed2c, 90 },
    { 0xdaa4ed2d, 61 },
    { 0, 0 },
    { 0xbe74ed2f, 84 },
    { 0, 0 },
    { 0xd74ed31, 95 },
    { 0, 0 },
    { 0x84a4ed33, 33 },
    { 0x2d54ed34, 23 },
    { 0xfad4ed34, 90 },
    { 0x3744ed36, 90 },
    { 0, 0 },
    { 0x4a04ed38, 23 },
    { 0x7794ed39, 87 },
    { 0x84f4ed38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d44ed41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74ed44, 95 },
    { 0, 0 },
    { 0x8714ed46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644ed4a, 34 },
    { 0, 0 },
    { 0x1084ed4c, 95 },
    { 0x1034ed4c, 100 },
    { 0x93c4ed4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34ed52, 61 },
    { 0x5ea4ed53, 33 },
    { 0x8534ed54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8054ed57, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc4ed60, 81 },
    { 0, 0 },
    { 0x3f04ed62, 23 },
    { 0xf844ed63, 87 },
    { 0, 0 },
    { 0x9204ed65, 95 },
    { 0x7b44ed66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f4ed69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e64ed6c, 88 },
    { 0x594ed6c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9924ed70, 78 },
    { 0x204ed71, 95 },
    { 0x4684ed72, 61 },
    { 0, 0 },
    { 0xe844ed74, 61 },
    { 0, 0 },
    { 0x3824ed76, 88 },
    { 0x204ed77, 23 },
    { 0, 0 },
    { 0x1df4ed79, 95 },
    { 0x6a34ed79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7714ed82, 85 },
    { 0x75c4ed83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ed86, 95 },
    { 0x1e04ed87, 46 },
    { 0x2324ed88, 43 },
    { 0x414ed88, 61 },
    { 0x9a34ed8a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94ed8e, 90 },
    { 0x1c94ed8f, 61 },
    { 0x224ed90, 87 },
    { 0x4b74ed91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4ed96, 90 },
    { 0, 0 },
    { 0xec94ed98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5414eda1, 61 },
    { 0, 0 },
    { 0xa914eda3, 91 },
    { 0x29b4eda4, 43 },
    { 0xa4e4eda3, 48 },
    { 0x5cd4eda6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x244eda9, 90 },
    { 0x2ba4edaa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e94edb2, 33 },
    { 0x14edb3, 98 },
    { 0xebd4edb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e64edb7, 84 },
    { 0x9c74edb8, 22 },
    { 0xa8d4edb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4edc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db4edc6, 55 },
    { 0, 0 },
    { 0xc784edc8, 61 },
    { 0xf64edc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a54edcd, 76 },
    { 0, 0 },
    { 0xe864edcf, 61 },
    { 0xbca4edcf, 68 },
    { 0, 0 },
    { 0x53a4edd2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e74edd5, 81 },
    { 0x8164edd6, 33 },
    { 0xe614edd6, 90 },
    { 0x3fc4edd8, 61 },
    { 0x6fc4edd5, 23 },
    { 0, 0 },
    { 0x6a34eddb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4edde, 99 },
    { 0x74c4eddf, 33 },
    { 0xe774ede0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74ede9, 76 },
    { 0, 0 },
    { 0xc304edeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f24edf2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4edf5, 90 },
    { 0xfe94edf5, 90 },
    { 0x2404edf7, 93 },
    { 0, 0 },
    { 0x434edf9, 68 },
    { 0x634edfa, 23 },
    { 0xe074edfb, 43 },
    { 0xbd54edf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4ee01, 90 },
    { 0x1a54ee02, 90 },
    { 0x214ee02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd794ee08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a04ee0b, 61 },
    { 0x7cb4ee0b, 90 },
    { 0xbcf4ee0d, 84 },
    { 0x7074ee0e, 61 },
    { 0x5774ee0e, 93 },
    { 0, 0 },
    { 0x5ee4ee11, 99 },
    { 0, 0 },
    { 0xf654ee13, 34 },
    { 0x97c4ee14, 93 },
    { 0x9bf4ee15, 61 },
    { 0, 0 },
    { 0xa234ee17, 61 },
    { 0xc5f4ee18, 61 },
    { 0xaff4ee19, 33 },
    { 0x474ee17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f4ee23, 50 },
    { 0x1194ee24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164ee29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b4ee2d, 98 },
    { 0xce84ee2e, 99 },
    { 0, 0 },
    { 0x2954ee30, 91 },
    { 0xde74ee31, 90 },
    { 0xf3c4ee32, 90 },
    { 0xca44ee33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54ee37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4ee3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604ee3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4ee42, 88 },
    { 0x3284ee42, 98 },
    { 0x75a4ee44, 81 },
    { 0x884ee43, 23 },
    { 0, 0 },
    { 0x38d4ee47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74ee51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd514ee56, 40 },
    { 0x1a84ee57, 33 },
    { 0xb9d4ee58, 61 },
    { 0xed24ee58, 76 },
    { 0, 0 },
    { 0x4eb4ee5b, 95 },
    { 0x8644ee5c, 80 },
    { 0xfda4ee5d, 23 },
    { 0xb9a4ee5b, 61 },
    { 0, 0 },
    { 0xfb44ee60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2644ee63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4ee6b, 90 },
    { 0x84e4ee6b, 90 },
    { 0, 0 },
    { 0x7904ee6e, 33 },
    { 0xf734ee6e, 33 },
    { 0x2884ee70, 33 },
    { 0, 0 },
    { 0xe414ee72, 33 },
    { 0xe424ee72, 90 },
    { 0x9834ee73, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb04ee78, 61 },
    { 0, 0 },
    { 0xe074ee7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e4ee7d, 61 },
    { 0x2a64ee7d, 43 },
    { 0, 0 },
    { 0x2a74ee80, 84 },
    { 0x80f4ee81, 90 },
    { 0x3954ee80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ee86, 55 },
    { 0, 0 },
    { 0x2d44ee88, 33 },
    { 0, 0 },
    { 0xa964ee8a, 80 },
    { 0x4df4ee8b, 90 },
    { 0, 0 },
    { 0x3834ee8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14ee91, 83 },
    { 0, 0 },
    { 0x36e4ee93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ee98, 98 },
    { 0x7c14ee99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf544ee9c, 33 },
    { 0xa644ee9c, 95 },
    { 0, 0 },
    { 0x1674ee9f, 23 },
    { 0x2894ee9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd84eeaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064eeb1, 24 },
    { 0x7d84eeb2, 46 },
    { 0x1f14eeb3, 91 },
    { 0, 0 },
    { 0x4904eeb5, 74 },
    { 0xc9d4eeb6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea34eebb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed4eebe, 98 },
    { 0xbe14eebf, 46 },
    { 0xd774eec0, 90 },
    { 0xaec4eec1, 90 },
    { 0x6b74eec2, 33 },
    { 0x41c4eec3, 77 },
    { 0x86b4eec0, 100 },
    { 0, 0 },
    { 0x6134eec6, 84 },
    { 0x5974eec6, 90 },
    { 0xfd44eec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d4eecb, 33 },
    { 0xfe34eecc, 34 },
    { 0, 0 },
    { 0x9574eece, 43 },
    { 0x3c64eece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84eed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f4eed6, 23 },
    { 0x9584eed7, 95 },
    { 0, 0 },
    { 0xe6b4eed9, 55 },
    { 0x1004eeda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4eedd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4eee6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1414eeea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1624eef1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf664eef4, 90 },
    { 0x8ed4eef5, 99 },
    { 0x3b34eef6, 92 },
    { 0, 0 },
    { 0x5164eef8, 100 },
    { 0x5f54eef8, 44 },
    { 0, 0 },
    { 0xba74eefb, 33 },
    { 0x2084eefc, 46 },
    { 0x1284eefc, 33 },
    { 0x5a4eefc, 33 },
    { 0x37e4eeff, 33 },
    { 0x3aa4eefe, 93 },
    { 0xe594eefb, 98 },
    { 0, 0 },
    { 0x8994ef03, 61 },
    { 0, 0 },
    { 0x5fe4ef05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb4ef0a, 61 },
    { 0xa9c4ef0b, 33 },
    { 0x8464ef0c, 23 },
    { 0x2f44ef0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xece4ef10, 95 },
    { 0xba34ef11, 95 },
    { 0xc444ef12, 33 },
    { 0x43a4ef13, 85 },
    { 0xcd44ef11, 23 },
    { 0xcc64ef15, 90 },
    { 0x1024ef15, 84 },
    { 0xe384ef17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e4ef1a, 23 },
    { 0x1d84ef1a, 40 },
    { 0, 0 },
    { 0xb814ef1d, 23 },
    { 0xf794ef1e, 33 },
    { 0, 0 },
    { 0x5f34ef20, 98 },
    { 0, 0 },
    { 0x7f74ef22, 85 },
    { 0xd374ef23, 95 },
    { 0xdd54ef24, 33 },
    { 0x1464ef25, 76 },
    { 0x77a4ef26, 87 },
    { 0x16d4ef27, 34 },
    { 0, 0 },
    { 0x4ab4ef29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce64ef2d, 61 },
    { 0x5f64ef2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb234ef34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a4ef38, 38 },
    { 0xba64ef38, 95 },
    { 0x1824ef3a, 95 },
    { 0xd1a4ef38, 61 },
    { 0x1ee4ef3a, 46 },
    { 0xddb4ef3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa4ef46, 66 },
    { 0xf334ef47, 33 },
    { 0xf5d4ef47, 23 },
    { 0x39e4ef47, 33 },
    { 0x9cc4ef49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4ef4f, 23 },
    { 0x4af4ef50, 87 },
    { 0, 0 },
    { 0x25d4ef52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc84ef56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e54ef59, 76 },
    { 0xe324ef5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa364ef5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf294ef61, 61 },
    { 0, 0 },
    { 0x40a4ef63, 90 },
    { 0x6a24ef64, 91 },
    { 0xefa4ef64, 33 },
    { 0xf6b4ef66, 61 },
    { 0x4984ef64, 100 },
    { 0x31d4ef65, 67 },
    { 0, 0 },
    { 0xa2c4ef6a, 99 },
    { 0xe914ef6b, 66 },
    { 0xc064ef6c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264ef6f, 98 },
    { 0x8ba4ef70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4ef7b, 98 },
    { 0x1f44ef7c, 90 },
    { 0xb3c4ef7b, 61 },
    { 0x5c4ef7e, 90 },
    { 0x2c74ef7f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4464ef83, 95 },
    { 0x414ef83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa934ef89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4ef8f, 23 },
    { 0, 0 },
    { 0x1f34ef91, 23 },
    { 0x90e4ef91, 81 },
    { 0x13e4ef92, 34 },
    { 0, 0 },
    { 0xc4b4ef95, 100 },
    { 0xdb44ef96, 23 },
    { 0x1d64ef97, 55 },
    { 0, 0 },
    { 0x6364ef99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8084ef9d, 23 },
    { 0, 0 },
    { 0xd4a4ef9f, 98 },
    { 0, 0 },
    { 0x6b94efa1, 90 },
    { 0xb8f4efa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4efa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8514efa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3984efab, 76 },
    { 0, 0 },
    { 0xe5d4efad, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8644efb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f4efb7, 61 },
    { 0, 0 },
    { 0xd154efb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794efc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974efc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874efc8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4084efcc, 23 },
    { 0xeae4efcd, 23 },
    { 0xcde4efcd, 23 },
    { 0xe5f4efcf, 76 },
    { 0xcd44efcf, 84 },
    { 0x6304efd0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94efd4, 23 },
    { 0xf564efd5, 81 },
    { 0xd284efd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94efda, 61 },
    { 0x36b4efdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9994efde, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4efe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e4efe6, 33 },
    { 0xd6b4efe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c04efea, 78 },
    { 0, 0 },
    { 0x3004efec, 43 },
    { 0x804efec, 61 },
    { 0x2884efee, 52 },
    { 0xd234efee, 46 },
    { 0, 0 },
    { 0x2754eff1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e4eff4, 95 },
    { 0xc8e4eff4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd064eff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4effe, 98 },
    { 0x8b54effe, 98 },
    { 0x6bc4effe, 90 },
    { 0xbc34efff, 55 },
    { 0x39a4f002, 33 },
    { 0x48d4f000, 95 },
    { 0, 0 },
    { 0x3324f005, 61 },
    { 0, 0 },
    { 0xcd04f007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd94f00c, 46 },
    { 0, 0 },
    { 0x4264f00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2304f016, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134f01a, 95 },
    { 0, 0 },
    { 0x12f4f01c, 61 },
    { 0, 0 },
    { 0xd514f01e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554f023, 95 },
    { 0xc8c4f024, 90 },
    { 0x5e04f024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84f032, 65 },
    { 0xc1f4f033, 23 },
    { 0, 0 },
    { 0xdd64f035, 93 },
    { 0xccd4f035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc384f039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc954f03d, 90 },
    { 0x4ba4f03e, 85 },
    { 0xe564f03f, 33 },
    { 0x78f4f040, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb4f043, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04f048, 65 },
    { 0x5d54f049, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4f050, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814f054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f4f059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde04f05d, 90 },
    { 0x7b14f05d, 98 },
    { 0x3224f05e, 90 },
    { 0x9874f05f, 90 },
    { 0, 0 },
    { 0xb3a4f062, 52 },
    { 0x4194f063, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9864f067, 34 },
    { 0xca54f068, 61 },
    { 0x8eb4f068, 95 },
    { 0x99a4f06a, 34 },
    { 0x77b4f06b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b34f07c, 61 },
    { 0xe8e4f07c, 61 },
    { 0x2534f07e, 43 },
    { 0, 0 },
    { 0x41b4f080, 33 },
    { 0x2834f081, 85 },
    { 0x9424f081, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884f08b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab4f092, 91 },
    { 0xf94f093, 99 },
    { 0x7b4f092, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c84f098, 33 },
    { 0xeb74f099, 65 },
    { 0, 0 },
    { 0xfb84f09b, 55 },
    { 0, 0 },
    { 0x4ba4f09d, 87 },
    { 0, 0 },
    { 0xd084f09f, 66 },
    { 0x4c64f0a0, 43 },
    { 0xc424f0a1, 34 },
    { 0xe624f0a0, 88 },
    { 0x7de4f0a0, 33 },
    { 0xc3b4f0a4, 23 },
    { 0xaf84f0a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a4f0a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134f0ad, 55 },
    { 0x3774f0ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724f0b3, 87 },
    { 0xed84f0b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf194f0b8, 33 },
    { 0, 0 },
    { 0x224f0ba, 34 },
    { 0, 0 },
    { 0xb334f0bc, 81 },
    { 0xcc44f0bd, 55 },
    { 0, 0 },
    { 0x4504f0bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd94f0c8, 33 },
    { 0xcc44f0c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc4f0d0, 40 },
    { 0x2204f0d1, 33 },
    { 0, 0 },
    { 0xcac4f0d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb4f0d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7594f0dc, 90 },
    { 0xe604f0dc, 100 },
    { 0xd614f0de, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f0e3, 34 },
    { 0, 0 },
    { 0x4464f0e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd464f0e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9424f0ec, 98 },
    { 0, 0 },
    { 0xa4b4f0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc274f0f2, 95 },
    { 0xd1d4f0f3, 61 },
    { 0, 0 },
    { 0xa2a4f0f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2824f0fb, 23 },
    { 0x5bf4f0fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb54f102, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e4f106, 78 },
    { 0, 0 },
    { 0xb704f108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c4f10d, 33 },
    { 0, 0 },
    { 0xefc4f10f, 90 },
    { 0xbb24f110, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf414f117, 75 },
    { 0, 0 },
    { 0x16e4f119, 76 },
    { 0xbd24f119, 33 },
    { 0x47c4f11a, 90 },
    { 0, 0 },
    { 0x1c74f11d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf14f120, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa464f123, 98 },
    { 0x6794f123, 99 },
    { 0, 0 },
    { 0x4ae4f126, 33 },
    { 0x46d4f127, 66 },
    { 0, 0 },
    { 0x3754f129, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd4f12f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c4f135, 90 },
    { 0x7bd4f136, 91 },
    { 0, 0 },
    { 0xae74f138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c4f13b, 90 },
    { 0, 0 },
    { 0x59f4f13d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec74f141, 52 },
    { 0, 0 },
    { 0xefc4f143, 98 },
    { 0x7204f143, 90 },
    { 0xde74f144, 98 },
    { 0x5674f146, 100 },
    { 0xfda4f146, 17 },
    { 0x8f04f147, 90 },
    { 0xac04f149, 78 },
    { 0x7ee4f149, 56 },
    { 0x394f146, 95 },
    { 0, 0 },
    { 0xd414f14d, 91 },
    { 0x6dd4f14e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94f151, 85 },
    { 0x7764f152, 43 },
    { 0, 0 },
    { 0x1a64f154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e4f15a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4f161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff4f167, 77 },
    { 0x83f4f167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be4f16c, 76 },
    { 0x5424f16d, 98 },
    { 0xd0e4f16e, 52 },
    { 0x6044f16e, 34 },
    { 0x7704f16c, 23 },
    { 0x4ab4f171, 76 },
    { 0xcea4f171, 33 },
    { 0x69a4f16f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a4f178, 95 },
    { 0x37c4f178, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8394f17c, 98 },
    { 0x63e4f17d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04f184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab4f18a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca4f18e, 76 },
    { 0x95a4f18e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94f192, 80 },
    { 0x1574f193, 53 },
    { 0, 0 },
    { 0x75b4f195, 55 },
    { 0xb4d4f195, 17 },
    { 0x6c34f197, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7364f19c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e4f1a4, 95 },
    { 0x7564f1a5, 90 },
    { 0, 0 },
    { 0x4f64f1a7, 43 },
    { 0, 0 },
    { 0xd444f1a9, 46 },
    { 0, 0 },
    { 0x85f4f1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f24f1ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924f1b6, 33 },
    { 0xf0c4f1b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd4f1c3, 34 },
    { 0xdc24f1c4, 98 },
    { 0xeae4f1c5, 61 },
    { 0x934f1c4, 23 },
    { 0x2a84f1c7, 33 },
    { 0xf754f1c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7384f1cb, 40 },
    { 0x25f4f1cc, 90 },
    { 0xf5f4f1cd, 76 },
    { 0x9324f1cd, 95 },
    { 0x1954f1cd, 76 },
    { 0x4c54f1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9934f1d3, 66 },
    { 0x2fd4f1d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f14f1dd, 61 },
    { 0xdae4f1de, 90 },
    { 0, 0 },
    { 0x3f44f1e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864f1e6, 95 },
    { 0, 0 },
    { 0xe304f1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4f1eb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc034f1ef, 90 },
    { 0x1734f1f0, 55 },
    { 0, 0 },
    { 0x27f4f1f2, 52 },
    { 0x9024f1f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554f1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa54f1fd, 76 },
    { 0x1284f1fd, 33 },
    { 0xfa54f1ff, 33 },
    { 0xd1a4f200, 87 },
    { 0x6aa4f201, 33 },
    { 0xd654f202, 23 },
    { 0x5654f202, 61 },
    { 0x3974f204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4894f207, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524f20c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7024f211, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794f218, 90 },
    { 0x6fe4f219, 98 },
    { 0x2ba4f21a, 67 },
    { 0x48b4f219, 33 },
    { 0x1194f21c, 61 },
    { 0, 0 },
    { 0x35d4f21e, 88 },
    { 0x4f14f21e, 23 },
    { 0, 0 },
    { 0x35d4f221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d34f22b, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f232, 90 },
    { 0xff24f233, 34 },
    { 0x5ed4f234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb4f237, 90 },
    { 0x99c4f238, 61 },
    { 0, 0 },
    { 0xa534f23a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4f23e, 65 },
    { 0, 0 },
    { 0xd6c4f240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4f246, 34 },
    { 0xfc14f247, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474f24a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd4f24f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4f252, 52 },
    { 0x9e64f253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2774f256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24f25a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f260, 90 },
    { 0xbdf4f261, 23 },
    { 0x23d4f262, 66 },
    { 0x9c54f260, 34 },
    { 0xfbd4f264, 33 },
    { 0, 0 },
    { 0xff24f266, 23 },
    { 0, 0 },
    { 0x5ed4f268, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1154f26d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb4f273, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5044f278, 91 },
    { 0x3bb4f279, 91 },
    { 0x1754f27a, 43 },
    { 0xf774f27b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a4f284, 65 },
    { 0x2304f284, 98 },
    { 0x84b4f285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4934f28c, 53 },
    { 0, 0 },
    { 0x4a94f28e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d34f295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde74f299, 98 },
    { 0x5764f299, 76 },
    { 0x30b4f29b, 95 },
    { 0, 0 },
    { 0x434f29d, 90 },
    { 0xa054f29e, 33 },
    { 0, 0 },
    { 0x35d4f2a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64f2a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4f2b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be4f2b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4904f2be, 33 },
    { 0xac04f2bf, 90 },
    { 0xa224f2c0, 23 },
    { 0xceb4f2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f4f2d2, 33 },
    { 0xc974f2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2144f2d7, 61 },
    { 0, 0 },
    { 0x5e34f2d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe994f2e1, 33 },
    { 0, 0 },
    { 0x3814f2e3, 61 },
    { 0x5804f2e4, 90 },
    { 0x43e4f2e5, 77 },
    { 0, 0 },
    { 0xff74f2e7, 90 },
    { 0x13a4f2e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb4f2eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394f2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f2f6, 34 },
    { 0, 0 },
    { 0xf584f2f8, 23 },
    { 0x5284f2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea4f2fc, 33 },
    { 0xfce4f2fd, 76 },
    { 0x7284f2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f4f301, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4f305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f30d, 34 },
    { 0x2834f30e, 91 },
    { 0x3e54f30e, 99 },
    { 0x99f4f30e, 64 },
    { 0xf534f311, 90 },
    { 0xf364f311, 55 },
    { 0xb2c4f311, 43 },
    { 0, 0 },
    { 0x6504f315, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd44f31a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd24f31d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b4f328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d34f32b, 95 },
    { 0x3674f32c, 99 },
    { 0xd804f32d, 88 },
    { 0x5c74f32e, 65 },
    { 0, 0 },
    { 0xf8c4f330, 33 },
    { 0xd724f330, 98 },
    { 0, 0 },
    { 0x4aa4f333, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b4f338, 55 },
    { 0x89c4f339, 43 },
    { 0x6454f33a, 95 },
    { 0x4eb4f33a, 90 },
    { 0xd84f33a, 34 },
    { 0xeb44f33d, 24 },
    { 0, 0 },
    { 0xdfa4f33f, 43 },
    { 0x4e44f340, 93 },
    { 0x2d4f33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb4f346, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf984f34b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f34f, 46 },
    { 0xd2d4f350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6604f353, 23 },
    { 0, 0 },
    { 0x5174f355, 90 },
    { 0x3a94f356, 68 },
    { 0x5564f356, 75 },
    { 0x32f4f357, 90 },
    { 0x8934f358, 43 },
    { 0xe014f355, 98 },
    { 0x4274f356, 91 },
    { 0x7d94f35c, 69 },
    { 0, 0 },
    { 0x38f4f35e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b4f361, 77 },
    { 0xeec4f361, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e4f365, 83 },
    { 0xa464f365, 55 },
    { 0, 0 },
    { 0x22c4f368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de4f36b, 93 },
    { 0, 0 },
    { 0x65e4f36d, 34 },
    { 0x3414f36e, 76 },
    { 0x3044f36d, 33 },
    { 0, 0 },
    { 0x5534f371, 37 },
    { 0x2ed4f372, 91 },
    { 0x4144f372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc434f378, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a4f37c, 78 },
    { 0xfbe4f37d, 76 },
    { 0xd564f37e, 52 },
    { 0x7264f37d, 55 },
    { 0, 0 },
    { 0x244f381, 76 },
    { 0x9c74f382, 61 },
    { 0x6724f383, 87 },
    { 0x9124f384, 95 },
    { 0xbd44f382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d4f38a, 97 },
    { 0x5b54f38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4f391, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c64f39e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d04f3a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a54f3a4, 61 },
    { 0x9724f3a4, 90 },
    { 0xaa74f3a6, 95 },
    { 0, 0 },
    { 0x2624f3a8, 22 },
    { 0x7684f3a8, 98 },
    { 0x7c34f3aa, 90 },
    { 0x1bc4f3aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34f3af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04f3b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4f3b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14f3ba, 9 },
    { 0xffc4f3ba, 66 },
    { 0x1834f3bc, 90 },
    { 0x57c4f3bc, 93 },
    { 0xa444f3bd, 98 },
    { 0x1e34f3bc, 33 },
    { 0xe284f3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6614f3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6224f3c6, 90 },
    { 0x93f4f3c7, 33 },
    { 0xc544f3c8, 61 },
    { 0xebf4f3c9, 90 },
    { 0x9774f3ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d14f3cd, 43 },
    { 0xb264f3ce, 61 },
    { 0x9b14f3cf, 95 },
    { 0xfc94f3d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44f3d3, 99 },
    { 0x88f4f3d4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244f3d7, 88 },
    { 0, 0 },
    { 0xd204f3d9, 34 },
    { 0x6e64f3d9, 98 },
    { 0x8c84f3db, 17 },
    { 0x47a4f3dc, 98 },
    { 0, 0 },
    { 0x4e64f3de, 33 },
    { 0xdb74f3df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714f3e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed74f3e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf584f3e9, 91 },
    { 0xd554f3ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b4f3f1, 61 },
    { 0x6d04f3f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4f3f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4f3f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a64f3ff, 93 },
    { 0x6b04f400, 65 },
    { 0x3934f401, 33 },
    { 0xe114f401, 98 },
    { 0x2c64f403, 87 },
    { 0xe344f404, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb94f408, 77 },
    { 0, 0 },
    { 0x81d4f40a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b4f415, 23 },
    { 0x21b4f416, 23 },
    { 0x6fa4f417, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f4f41a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3684f41d, 91 },
    { 0, 0 },
    { 0xa384f41f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf994f423, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e4f427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284f42a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54f42d, 34 },
    { 0x63b4f42e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9234f433, 34 },
    { 0, 0 },
    { 0xa7d4f435, 76 },
    { 0xcde4f436, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e4f441, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9704f444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8254f44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e64f457, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a4f45c, 55 },
    { 0xdf94f45d, 43 },
    { 0x2184f45e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a4f465, 33 },
    { 0xfe24f466, 33 },
    { 0, 0 },
    { 0x6114f468, 33 },
    { 0x9714f469, 55 },
    { 0xf924f46a, 88 },
    { 0xac44f468, 55 },
    { 0xac34f469, 90 },
    { 0xace4f46d, 61 },
    { 0x3c94f468, 85 },
    { 0, 0 },
    { 0x3ae4f470, 23 },
    { 0x95d4f470, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f34f479, 100 },
    { 0xc314f47a, 61 },
    { 0xad84f47a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4384f47f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc814f484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f74f488, 88 },
    { 0xc914f488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5524f490, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xff94f493, 61 },
    { 0x1c24f494, 33 },
    { 0, 0 },
    { 0xfbb4f496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2604f499, 90 },
    { 0x7644f49a, 90 },
    { 0xc9b4f49b, 43 },
    { 0, 0 },
    { 0x4364f49d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1664f4a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e44f4a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd4f4ac, 90 },
    { 0xfa4f4ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64f4b0, 99 },
    { 0xa994f4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf34f4b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4f4be, 33 },
    { 0xc3a4f4be, 90 },
    { 0x4b24f4c0, 61 },
    { 0xe2e4f4c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4f4c8, 55 },
    { 0x9d24f4c8, 84 },
    { 0xf224f4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4f4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e44f4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa914f4e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354f4e6, 67 },
    { 0, 0 },
    { 0xf904f4e8, 98 },
    { 0, 0 },
    { 0x8464f4ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734f4f2, 33 },
    { 0, 0 },
    { 0x89b4f4f4, 61 },
    { 0x1274f4f5, 33 },
    { 0x2144f4f6, 93 },
    { 0xeb94f4f7, 90 },
    { 0x9634f4f8, 55 },
    { 0xfd34f4f9, 97 },
    { 0x2cd4f4fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4f509, 43 },
    { 0x3fa4f50a, 76 },
    { 0x4c94f50b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca24f511, 90 },
    { 0, 0 },
    { 0xb14f513, 93 },
    { 0, 0 },
    { 0xb894f515, 23 },
    { 0x6984f515, 33 },
    { 0, 0 },
    { 0x43e4f518, 95 },
    { 0xc784f519, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524f521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424f524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84f529, 90 },
    { 0, 0 },
    { 0x9064f52b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944f533, 61 },
    { 0x2c94f534, 90 },
    { 0x29d4f535, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504f538, 61 },
    { 0, 0 },
    { 0xd364f53a, 91 },
    { 0, 0 },
    { 0xc444f53c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14f544, 90 },
    { 0x4744f545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c4f548, 76 },
    { 0x9224f549, 33 },
    { 0, 0 },
    { 0x3ff4f54b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc414f54f, 61 },
    { 0, 0 },
    { 0x4244f551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b94f554, 33 },
    { 0xe2a4f554, 90 },
    { 0x9b94f555, 32 },
    { 0xd404f557, 76 },
    { 0x6854f558, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604f55b, 99 },
    { 0, 0 },
    { 0x61f4f55d, 34 },
    { 0, 0 },
    { 0x2114f55f, 99 },
    { 0, 0 },
    { 0x6804f561, 76 },
    { 0x2274f561, 34 },
    { 0x904f563, 76 },
    { 0x14c4f562, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf704f567, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f54f56c, 61 },
    { 0x3c94f56c, 33 },
    { 0, 0 },
    { 0xa374f56f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324f578, 75 },
    { 0, 0 },
    { 0x9974f57a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d84f57e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404f583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb694f586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e4f58f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe914f597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074f59b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9264f5a2, 76 },
    { 0, 0 },
    { 0x4bc4f5a4, 45 },
    { 0xa444f5a5, 100 },
    { 0xdef4f5a5, 66 },
    { 0x6514f5a7, 61 },
    { 0, 0 },
    { 0x8e4f5a9, 90 },
    { 0xc444f5aa, 33 },
    { 0x3724f5ab, 90 },
    { 0, 0 },
    { 0x6f64f5ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c4f5b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1374f5b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa674f5b6, 76 },
    { 0x7794f5b6, 65 },
    { 0, 0 },
    { 0x6614f5b9, 33 },
    { 0xea4f5b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f5be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304f5c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7674f5c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb4f5c9, 61 },
    { 0x81e4f5ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74f5d1, 46 },
    { 0xe384f5d2, 33 },
    { 0xb9b4f5d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f4f5d6, 95 },
    { 0x7d74f5d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb4f5da, 90 },
    { 0, 0 },
    { 0xd474f5dc, 100 },
    { 0, 0 },
    { 0x1144f5de, 52 },
    { 0, 0 },
    { 0x6c24f5e0, 52 },
    { 0x4184f5e1, 98 },
    { 0xc194f5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd224f5e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc664f5eb, 81 },
    { 0xbbd4f5ec, 72 },
    { 0, 0 },
    { 0xe124f5ee, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe104f5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4f5f5, 98 },
    { 0, 0 },
    { 0xa5e4f5f7, 78 },
    { 0x9514f5f8, 90 },
    { 0xa8a4f5f7, 95 },
    { 0xa354f5f8, 66 },
    { 0, 0 },
    { 0xb904f5fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f4f604, 84 },
    { 0xcbe4f605, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d94f60c, 95 },
    { 0x8df4f60d, 46 },
    { 0, 0 },
    { 0x4694f60f, 76 },
    { 0xb2a4f610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7044f614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d44f618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc64f61f, 34 },
    { 0x2984f61f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b54f623, 52 },
    { 0x9e04f623, 52 },
    { 0x5e4f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4f628, 68 },
    { 0, 0 },
    { 0xccd4f62a, 85 },
    { 0x5e4f62b, 55 },
    { 0xdd24f62c, 68 },
    { 0x64b4f62d, 40 },
    { 0xa354f62b, 95 },
    { 0x8044f62f, 23 },
    { 0xa7c4f630, 61 },
    { 0, 0 },
    { 0x4c04f632, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e4f635, 93 },
    { 0x8614f635, 61 },
    { 0, 0 },
    { 0xfb54f638, 98 },
    { 0x42a4f638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3324f642, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f646, 99 },
    { 0x8324f647, 85 },
    { 0x5ae4f648, 61 },
    { 0x1414f647, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf164f652, 99 },
    { 0, 0 },
    { 0x7f24f654, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864f65a, 98 },
    { 0xcc94f65b, 97 },
    { 0xc7a4f65a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f661, 33 },
    { 0, 0 },
    { 0xd994f663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f4f66b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44f66e, 68 },
    { 0, 0 },
    { 0xfa4f670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f04f679, 46 },
    { 0, 0 },
    { 0xfea4f67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb4f67e, 95 },
    { 0x9cd4f67e, 95 },
    { 0, 0 },
    { 0x2724f681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794f687, 98 },
    { 0xc24f687, 33 },
    { 0, 0 },
    { 0x844f68a, 90 },
    { 0xf564f68a, 75 },
    { 0x67e4f68c, 90 },
    { 0xb544f68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5434f690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe574f693, 33 },
    { 0xf0a4f694, 90 },
    { 0, 0 },
    { 0xd074f696, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4f69a, 36 },
    { 0, 0 },
    { 0x81a4f69c, 61 },
    { 0xc5e4f69c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce54f6a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa124f6a6, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34f6aa, 67 },
    { 0, 0 },
    { 0xc824f6ac, 34 },
    { 0x96e4f6ac, 34 },
    { 0xeb94f6ac, 17 },
    { 0x5614f6af, 98 },
    { 0xf234f6ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4f6b4, 61 },
    { 0x7314f6b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f4f6b9, 99 },
    { 0x7984f6ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434f6be, 33 },
    { 0, 0 },
    { 0x63c4f6c0, 66 },
    { 0xd714f6c0, 90 },
    { 0xee24f6c0, 53 },
    { 0, 0 },
    { 0x9784f6c4, 98 },
    { 0, 0 },
    { 0x1b94f6c6, 98 },
    { 0xad54f6c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd24f6cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4f6cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b24f6d2, 46 },
    { 0xa764f6d3, 61 },
    { 0, 0 },
    { 0x7e4f6d5, 34 },
    { 0xf414f6d5, 98 },
    { 0, 0 },
    { 0x1804f6d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf534f6db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x254f6de, 46 },
    { 0x4874f6df, 95 },
    { 0x7f74f6df, 76 },
    { 0x664f6e0, 46 },
    { 0x8074f6e2, 52 },
    { 0x70a4f6de, 43 },
    { 0x72a4f6e4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf544f6eb, 90 },
    { 0xb914f6ec, 90 },
    { 0, 0 },
    { 0x8e54f6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354f6f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd4f6f6, 95 },
    { 0xc54f6f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf144f6fb, 98 },
    { 0xa944f6fc, 43 },
    { 0xd754f6fd, 88 },
    { 0x40f4f6fd, 46 },
    { 0xdc34f6fb, 61 },
    { 0, 0 },
    { 0x7774f701, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4f70e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4f711, 61 },
    { 0, 0 },
    { 0xb984f713, 56 },
    { 0, 0 },
    { 0x3094f715, 88 },
    { 0xe844f715, 34 },
    { 0x75e4f717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af4f71b, 52 },
    { 0xb904f71b, 34 },
    { 0x43f4f71c, 55 },
    { 0xf994f71e, 61 },
    { 0x7554f71e, 67 },
    { 0x4f4f71d, 95 },
    { 0xe94f721, 33 },
    { 0, 0 },
    { 0x6234f723, 33 },
    { 0x7124f724, 61 },
    { 0x1fa4f725, 95 },
    { 0x5da4f724, 93 },
    { 0x3824f723, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654f72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab44f731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a4f736, 90 },
    { 0xb274f737, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4f73c, 91 },
    { 0x7494f73c, 61 },
    { 0x5704f73d, 98 },
    { 0x3314f73f, 43 },
    { 0x9254f73f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f4f747, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4f74d, 33 },
    { 0xe604f74d, 40 },
    { 0, 0 },
    { 0xb1a4f750, 84 },
    { 0x8cc4f751, 95 },
    { 0, 0 },
    { 0xee4f753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c4f756, 84 },
    { 0, 0 },
    { 0x1714f758, 61 },
    { 0x9ee4f759, 76 },
    { 0x9654f75a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24f75d, 98 },
    { 0, 0 },
    { 0xe1c4f75f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4f767, 95 },
    { 0, 0 },
    { 0xb8b4f769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c4f76f, 90 },
    { 0x90e4f76f, 95 },
    { 0, 0 },
    { 0x3d54f772, 43 },
    { 0xcab4f773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4f778, 80 },
    { 0x1f14f778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f4f780, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe064f785, 33 },
    { 0x1624f786, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3064f789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f4f78c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f4f790, 90 },
    { 0x5774f791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f794, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e4f79c, 34 },
    { 0x1e84f79c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f7a3, 100 },
    { 0xbbf4f7a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc4f7a7, 90 },
    { 0, 0 },
    { 0xb44f7a9, 34 },
    { 0x3fc4f7aa, 34 },
    { 0, 0 },
    { 0xd7f4f7ac, 43 },
    { 0x4414f7ac, 90 },
    { 0xe2c4f7ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c64f7b2, 46 },
    { 0x164f7b2, 98 },
    { 0xa4d4f7b2, 76 },
    { 0x7fd4f7b2, 90 },
    { 0x98d4f7b6, 52 },
    { 0x6d4f7b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6124f7bb, 36 },
    { 0x4d24f7bb, 90 },
    { 0x2eb4f7bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9554f7c0, 98 },
    { 0xd914f7c1, 23 },
    { 0xa184f7c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6284f7c5, 17 },
    { 0xbc4f7c6, 23 },
    { 0x23e4f7c6, 77 },
    { 0x7bb4f7c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb824f7cf, 88 },
    { 0xb664f7cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5664f7d3, 23 },
    { 0xd034f7d3, 55 },
    { 0x5474f7d5, 87 },
    { 0x73b4f7d5, 33 },
    { 0xaf84f7d6, 94 },
    { 0x49f4f7d4, 90 },
    { 0xb2a4f7d6, 33 },
    { 0xd1b4f7d4, 76 },
    { 0, 0 },
    { 0x71e4f7dc, 100 },
    { 0xbd64f7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb554f7e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424f7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6484f7ed, 98 },
    { 0xdc84f7ee, 78 },
    { 0, 0 },
    { 0x94b4f7f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c4f7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8134f7fc, 90 },
    { 0x1934f7fc, 90 },
    { 0x89d4f7fe, 98 },
    { 0x694f7fe, 95 },
    { 0x6144f7ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db4f807, 34 },
    { 0x6f4f808, 87 },
    { 0, 0 },
    { 0x5474f80a, 33 },
    { 0, 0 },
    { 0x244f80c, 33 },
    { 0xfa24f80d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04f811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e04f814, 8 },
    { 0, 0 },
    { 0xc454f816, 76 },
    { 0x8654f817, 33 },
    { 0x7de4f817, 65 },
    { 0xa9e4f818, 93 },
    { 0xb4a4f81a, 43 },
    { 0, 0 },
    { 0xea84f81c, 77 },
    { 0, 0 },
    { 0xbbe4f81e, 61 },
    { 0xf224f81e, 98 },
    { 0x9d34f81e, 98 },
    { 0, 0 },
    { 0x3134f822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b24f828, 75 },
    { 0, 0 },
    { 0x81f4f82a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634f82f, 61 },
    { 0x6534f830, 23 },
    { 0xf2d4f831, 19 },
    { 0x1264f831, 78 },
    { 0x9824f833, 61 },
    { 0x5264f831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef4f838, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4f83d, 88 },
    { 0x55f4f83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa144f841, 90 },
    { 0xe754f842, 52 },
    { 0x55d4f843, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x704f846, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4f849, 87 },
    { 0x98a4f84a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8914f84d, 40 },
    { 0x274f84e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e4f852, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4f85c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d4f85f, 78 },
    { 0x58f4f860, 85 },
    { 0x1044f861, 61 },
    { 0, 0 },
    { 0x3194f863, 55 },
    { 0x2e04f863, 61 },
    { 0xa734f865, 95 },
    { 0x8744f866, 99 },
    { 0x9d64f863, 33 },
    { 0, 0 },
    { 0xc564f869, 100 },
    { 0, 0 },
    { 0x6624f86b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f54f86e, 90 },
    { 0xcc34f86f, 90 },
    { 0x604f870, 98 },
    { 0xdf34f86f, 61 },
    { 0x5714f870, 75 },
    { 0x4484f873, 99 },
    { 0x4d24f874, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f87d, 68 },
    { 0xabf4f87d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4f888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d4f892, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4f895, 76 },
    { 0x5214f895, 61 },
    { 0x9ac4f897, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd574f89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34f8a0, 98 },
    { 0xb7f4f8a1, 91 },
    { 0x1544f8a2, 98 },
    { 0, 0 },
    { 0xd1d4f8a4, 46 },
    { 0xd7b4f8a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754f8a9, 61 },
    { 0x5464f8a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b4f8ad, 100 },
    { 0, 0 },
    { 0x45e4f8af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5144f8b5, 98 },
    { 0xddd4f8b6, 23 },
    { 0xcf14f8b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1244f8bc, 76 },
    { 0x3434f8bd, 17 },
    { 0xd164f8bc, 95 },
    { 0, 0 },
    { 0x3324f8c0, 23 },
    { 0, 0 },
    { 0xf1f4f8c2, 46 },
    { 0x4214f8c3, 90 },
    { 0xb064f8c4, 40 },
    { 0xb394f8c5, 43 },
    { 0, 0 },
    { 0x6e74f8c7, 55 },
    { 0xae04f8c7, 34 },
    { 0, 0 },
    { 0xf94f8ca, 61 },
    { 0xeaa4f8cb, 38 },
    { 0xfe94f8cc, 61 },
    { 0x4614f8cd, 43 },
    { 0x1ae4f8ca, 23 },
    { 0x4dc4f8cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74f8d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74f8d8, 78 },
    { 0xa964f8d9, 55 },
    { 0x6614f8d8, 90 },
    { 0, 0 },
    { 0x4e04f8dc, 23 },
    { 0, 0 },
    { 0x75e4f8de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944f8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x774f8e5, 38 },
    { 0xdb94f8e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894f8e9, 33 },
    { 0xe914f8ea, 33 },
    { 0xe964f8eb, 98 },
    { 0, 0 },
    { 0x5da4f8ed, 93 },
    { 0, 0 },
    { 0x4e34f8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf124f8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324f8f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f8fb, 75 },
    { 0x734f8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c4f901, 100 },
    { 0, 0 },
    { 0xcbf4f903, 34 },
    { 0x1bd4f903, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c4f90a, 61 },
    { 0, 0 },
    { 0xe404f90c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4f910, 43 },
    { 0xfc34f910, 61 },
    { 0x2e04f910, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954f91b, 23 },
    { 0x76f4f91c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054f925, 23 },
    { 0x4cb4f926, 34 },
    { 0xcac4f927, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94f92c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4774f936, 43 },
    { 0xe954f937, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54f93a, 46 },
    { 0x6234f93a, 91 },
    { 0x69c4f93a, 55 },
    { 0xdb94f93d, 90 },
    { 0x26a4f93d, 90 },
    { 0x9ec4f93f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4f942, 38 },
    { 0x9754f943, 33 },
    { 0x6164f944, 34 },
    { 0, 0 },
    { 0x5214f946, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4f94a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f34f94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb34f950, 84 },
    { 0x2694f951, 61 },
    { 0xbd94f951, 33 },
    { 0x2484f951, 61 },
    { 0xdae4f954, 93 },
    { 0xca14f955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b84f958, 34 },
    { 0x7ea4f958, 76 },
    { 0x254f958, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf404f960, 78 },
    { 0x9e84f961, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e4f970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074f974, 23 },
    { 0x9504f975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea4f97a, 95 },
    { 0, 0 },
    { 0xb524f97c, 95 },
    { 0x3504f97d, 23 },
    { 0x2c84f97d, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b34f987, 55 },
    { 0, 0 },
    { 0x73a4f989, 61 },
    { 0xe4a4f98a, 90 },
    { 0x92a4f98a, 36 },
    { 0x4cc4f98c, 98 },
    { 0xb954f98d, 90 },
    { 0xe274f98e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b84f991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f4f995, 52 },
    { 0, 0 },
    { 0x51b4f997, 43 },
    { 0, 0 },
    { 0x7004f999, 33 },
    { 0, 0 },
    { 0xb94f99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5234f9a1, 56 },
    { 0xb1b4f9a2, 23 },
    { 0x3514f9a1, 98 },
    { 0xc0d4f9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f4f9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4f9ac, 76 },
    { 0xa384f9ac, 95 },
    { 0x5524f9ad, 34 },
    { 0x93c4f9af, 23 },
    { 0x89e4f9ad, 98 },
    { 0x29f4f9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb784f9b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924f9ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1974f9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7374f9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d4f9c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4f9cb, 98 },
    { 0x8e24f9cc, 33 },
    { 0xd644f9cd, 43 },
    { 0x3f14f9ce, 90 },
    { 0, 0 },
    { 0x4924f9d0, 90 },
    { 0xeb24f9d0, 90 },
    { 0, 0 },
    { 0xd4b4f9d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee24f9d6, 61 },
    { 0xaf34f9d7, 95 },
    { 0x8d14f9d8, 33 },
    { 0xfc04f9d6, 34 },
    { 0, 0 },
    { 0xd7b4f9db, 61 },
    { 0, 0 },
    { 0x1e14f9dd, 43 },
    { 0, 0 },
    { 0x1f74f9df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb54f9e2, 76 },
    { 0xade4f9e3, 61 },
    { 0x6914f9e4, 61 },
    { 0, 0 },
    { 0x8634f9e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d74f9ea, 100 },
    { 0, 0 },
    { 0xc054f9ec, 90 },
    { 0x1564f9ed, 23 },
    { 0xa0c4f9ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394f9f1, 67 },
    { 0x3574f9f1, 95 },
    { 0x254f9f3, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa594f9f8, 90 },
    { 0x2054f9f8, 33 },
    { 0xb6b4f9f8, 55 },
    { 0x94f9fb, 61 },
    { 0x9314f9fc, 33 },
    { 0x5754f9fb, 34 },
    { 0xd9b4f9f9, 92 },
    { 0, 0 },
    { 0x54a4fa00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6674fa04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b4fa07, 34 },
    { 0, 0 },
    { 0x6d84fa09, 95 },
    { 0xfef4fa0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e4fa0e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d4fa12, 61 },
    { 0xe804fa13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b4fa16, 33 },
    { 0x93d4fa17, 23 },
    { 0xf784fa17, 61 },
    { 0x3bc4fa19, 95 },
    { 0x48e4fa1a, 76 },
    { 0xa304fa1b, 61 },
    { 0, 0 },
    { 0xbb34fa1d, 33 },
    { 0x10e4fa1e, 66 },
    { 0xa594fa1f, 87 },
    { 0, 0 },
    { 0x3e04fa21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca74fa24, 88 },
    { 0x5ec4fa25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d04fa2b, 78 },
    { 0, 0 },
    { 0x334fa2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d24fa30, 100 },
    { 0, 0 },
    { 0x74d4fa32, 95 },
    { 0, 0 },
    { 0xa574fa34, 100 },
    { 0x23c4fa35, 61 },
    { 0xda94fa36, 76 },
    { 0xe964fa37, 23 },
    { 0, 0 },
    { 0x89c4fa39, 23 },
    { 0x28c4fa3a, 33 },
    { 0, 0 },
    { 0xcf94fa3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4fa3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a14fa42, 34 },
    { 0x8644fa42, 33 },
    { 0, 0 },
    { 0xa6a4fa45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc4fa48, 100 },
    { 0, 0 },
    { 0x7ea4fa4a, 66 },
    { 0, 0 },
    { 0x2044fa4c, 97 },
    { 0xd774fa4d, 76 },
    { 0xe504fa4d, 90 },
    { 0xfe64fa4f, 77 },
    { 0x3db4fa50, 23 },
    { 0xfee4fa4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2494fa57, 61 },
    { 0, 0 },
    { 0x6e94fa59, 23 },
    { 0, 0 },
    { 0x8024fa5b, 61 },
    { 0xcf54fa5b, 61 },
    { 0x9134fa5d, 61 },
    { 0x504fa5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044fa61, 33 },
    { 0, 0 },
    { 0x78a4fa63, 75 },
    { 0, 0 },
    { 0x8d54fa65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa744fa69, 37 },
    { 0, 0 },
    { 0xad04fa6b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d4fa78, 81 },
    { 0xf084fa79, 90 },
    { 0x19c4fa79, 61 },
    { 0xac4fa79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94fa7e, 90 },
    { 0xc204fa7f, 23 },
    { 0xcf94fa7e, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34fa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c4fa8e, 61 },
    { 0, 0 },
    { 0x2ea4fa90, 46 },
    { 0x2a64fa91, 33 },
    { 0x6ad4fa92, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994fa9a, 95 },
    { 0x1054fa9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14fa9e, 98 },
    { 0, 0 },
    { 0x3764faa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9934faa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864faa9, 90 },
    { 0, 0 },
    { 0x4c44faab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb404fab0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2124fab8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4864fabc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca64fabf, 61 },
    { 0, 0 },
    { 0xa1b4fac1, 43 },
    { 0xb6f4fac1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b4fac7, 43 },
    { 0, 0 },
    { 0x8f34fac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde44fad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4fad8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774fadc, 55 },
    { 0x33f4fadd, 99 },
    { 0x8c34fade, 90 },
    { 0xc8f4fadd, 90 },
    { 0x24c4fae0, 81 },
    { 0x84c4fadd, 93 },
    { 0x40c4fadd, 98 },
    { 0, 0 },
    { 0x2b74fae4, 90 },
    { 0x7494fae4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc4fae8, 33 },
    { 0x8ae4fae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf44faee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4faf2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b74faf6, 64 },
    { 0x6094faf7, 61 },
    { 0x57b4faf8, 76 },
    { 0, 0 },
    { 0x5fc4fafa, 33 },
    { 0x2d64fafb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5824fb00, 61 },
    { 0x96a4fb01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b74fb04, 76 },
    { 0xe884fb04, 23 },
    { 0x1f44fb06, 23 },
    { 0xccc4fb05, 23 },
    { 0x8d34fb04, 93 },
    { 0, 0 },
    { 0x78c4fb0a, 67 },
    { 0, 0 },
    { 0x8144fb0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f4fb0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c4fb12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b4fb17, 23 },
    { 0, 0 },
    { 0x97c4fb19, 84 },
    { 0x91e4fb1a, 33 },
    { 0xa994fb19, 95 },
    { 0xf854fb1c, 90 },
    { 0xd484fb1d, 90 },
    { 0xab74fb19, 33 },
    { 0xe4c4fb1f, 66 },
    { 0x2a54fb1c, 100 },
    { 0x1ac4fb21, 90 },
    { 0, 0 },
    { 0x8714fb23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe444fb26, 23 },
    { 0, 0 },
    { 0x1bd4fb28, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb4fb33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b4fb3a, 93 },
    { 0xfb04fb3b, 55 },
    { 0, 0 },
    { 0x4474fb3d, 85 },
    { 0x7654fb3e, 66 },
    { 0x3414fb3e, 90 },
    { 0, 0 },
    { 0xba14fb41, 17 },
    { 0, 0 },
    { 0x45c4fb43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054fb46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf054fb4e, 46 },
    { 0xe694fb4f, 55 },
    { 0, 0 },
    { 0xce4fb51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954fb54, 23 },
    { 0xa114fb54, 98 },
    { 0x7b94fb55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134fb59, 75 },
    { 0x2ba4fb5a, 95 },
    { 0xf824fb5a, 83 },
    { 0x8e54fb5c, 55 },
    { 0x5b34fb5c, 81 },
    { 0xf714fb5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e84fb62, 90 },
    { 0x4da4fb63, 90 },
    { 0x51e4fb63, 87 },
    { 0x604fb65, 93 },
    { 0x39a4fb66, 98 },
    { 0xa404fb66, 98 },
    { 0x8694fb68, 93 },
    { 0, 0 },
    { 0x8c84fb6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4fb6e, 55 },
    { 0x5874fb6e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf4fb72, 87 },
    { 0xe364fb72, 23 },
    { 0x6c64fb72, 33 },
    { 0, 0 },
    { 0x6244fb76, 76 },
    { 0xd184fb77, 61 },
    { 0xa664fb77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7634fb7c, 46 },
    { 0xecd4fb7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae4fb81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9194fb8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f4fb9a, 99 },
    { 0xc214fb9b, 34 },
    { 0, 0 },
    { 0xf334fb9d, 77 },
    { 0xd654fb9e, 78 },
    { 0, 0 },
    { 0xac24fba0, 23 },
    { 0x1934fba1, 98 },
    { 0x2ba4fba0, 33 },
    { 0x8ca4fba3, 67 },
    { 0x9714fba4, 98 },
    { 0, 0 },
    { 0x7574fba6, 43 },
    { 0xcfe4fba7, 43 },
    { 0xf0e4fba8, 76 },
    { 0xc694fba9, 98 },
    { 0x7a4fbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc4fbae, 90 },
    { 0, 0 },
    { 0x32e4fbb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5204fbb3, 67 },
    { 0x1574fbb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a4fbb7, 33 },
    { 0, 0 },
    { 0xa6f4fbb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd84fbc6, 76 },
    { 0, 0 },
    { 0x9b74fbc8, 75 },
    { 0xb0c4fbc9, 90 },
    { 0xf324fbca, 95 },
    { 0x1fa4fbcb, 61 },
    { 0xa404fbc9, 95 },
    { 0, 0 },
    { 0x4154fbce, 99 },
    { 0x37d4fbcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e4fbda, 33 },
    { 0, 0 },
    { 0x424fbdc, 88 },
    { 0xf04fbdd, 34 },
    { 0, 0 },
    { 0xdbc4fbdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4fbe2, 99 },
    { 0x2924fbe2, 43 },
    { 0, 0 },
    { 0x3994fbe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104fbed, 61 },
    { 0x2964fbee, 99 },
    { 0, 0 },
    { 0x9fb4fbf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb44fbf3, 94 },
    { 0x3d74fbf4, 33 },
    { 0xc334fbf5, 33 },
    { 0x7e44fbf6, 98 },
    { 0x5d44fbf7, 34 },
    { 0, 0 },
    { 0x5ba4fbf9, 33 },
    { 0xfca4fbf9, 23 },
    { 0x3c94fbfb, 67 },
    { 0, 0 },
    { 0x74fbfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b4fc01, 52 },
    { 0x2d24fc02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24fc08, 34 },
    { 0x7b84fc09, 88 },
    { 0x5e4fc0a, 95 },
    { 0xcb84fc0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c54fc0f, 61 },
    { 0x1674fc10, 55 },
    { 0x9a94fc0f, 33 },
    { 0x90e4fc0f, 84 },
    { 0xd414fc0f, 52 },
    { 0xe744fc14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de4fc19, 93 },
    { 0xd1b4fc1a, 90 },
    { 0, 0 },
    { 0xf604fc1c, 66 },
    { 0, 0 },
    { 0x9cd4fc1e, 33 },
    { 0, 0 },
    { 0xd6e4fc20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe04fc24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84fc2b, 76 },
    { 0, 0 },
    { 0xcb94fc2d, 98 },
    { 0x24fc2e, 95 },
    { 0x56f4fc2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4fc32, 99 },
    { 0x14f4fc33, 98 },
    { 0x97e4fc34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fc39, 46 },
    { 0, 0 },
    { 0x67c4fc3b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24fc3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe004fc42, 85 },
    { 0, 0 },
    { 0xf994fc44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2014fc48, 84 },
    { 0x5044fc48, 61 },
    { 0xc6b4fc4a, 52 },
    { 0x1f04fc4b, 76 },
    { 0xe914fc4b, 23 },
    { 0x3a14fc4d, 90 },
    { 0x7554fc4e, 43 },
    { 0xb694fc4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb4fc55, 33 },
    { 0xb874fc56, 61 },
    { 0, 0 },
    { 0xd0e4fc58, 55 },
    { 0x3194fc59, 61 },
    { 0x3b34fc5a, 61 },
    { 0x57e4fc5b, 51 },
    { 0x33b4fc59, 46 },
    { 0xe634fc5d, 88 },
    { 0, 0 },
    { 0xc474fc5f, 43 },
    { 0x3f04fc60, 65 },
    { 0x4954fc61, 76 },
    { 0xece4fc60, 84 },
    { 0xc8d4fc63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea4fc66, 55 },
    { 0xa1d4fc67, 95 },
    { 0, 0 },
    { 0x2ef4fc69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a4fc6c, 61 },
    { 0x9964fc6d, 43 },
    { 0xc404fc6e, 76 },
    { 0x12c4fc6f, 42 },
    { 0, 0 },
    { 0x7654fc71, 90 },
    { 0xe734fc72, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db4fc76, 33 },
    { 0x49d4fc77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d94fc7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab44fc81, 61 },
    { 0x6254fc82, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a14fc85, 46 },
    { 0xe934fc85, 46 },
    { 0x42e4fc87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604fc8d, 46 },
    { 0x654fc8e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf684fc91, 36 },
    { 0x6a24fc92, 76 },
    { 0xd5d4fc92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354fc97, 46 },
    { 0x4dc4fc97, 90 },
    { 0x31f4fc99, 99 },
    { 0x99d4fc98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6774fc9e, 77 },
    { 0xb1f4fc9f, 67 },
    { 0x87a4fca0, 85 },
    { 0, 0 },
    { 0x9c54fca2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8444fca8, 61 },
    { 0x1424fca9, 61 },
    { 0x85b4fcaa, 98 },
    { 0x20b4fca9, 35 },
    { 0xf074fca9, 33 },
    { 0xf0c4fca8, 85 },
    { 0xa644fcae, 85 },
    { 0xec54fcae, 76 },
    { 0, 0 },
    { 0xca14fcb1, 76 },
    { 0xdfe4fcb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34fcb5, 95 },
    { 0x5c14fcb6, 43 },
    { 0x80a4fcb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc354fcbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b4fcc3, 90 },
    { 0, 0 },
    { 0xf74fcc5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc514fcce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34fcd1, 99 },
    { 0x33c4fcd1, 34 },
    { 0x69b4fcd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24fcd6, 90 },
    { 0, 0 },
    { 0xf44fcd8, 76 },
    { 0xbd64fcd9, 76 },
    { 0x1944fcd8, 55 },
    { 0x8244fcdb, 33 },
    { 0xa524fcd9, 100 },
    { 0xe3a4fcd8, 90 },
    { 0x2fa4fcde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4fce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5174fce4, 33 },
    { 0x8164fce5, 34 },
    { 0xc984fce5, 98 },
    { 0xdd4fce7, 98 },
    { 0xbff4fce8, 33 },
    { 0xeea4fce9, 90 },
    { 0, 0 },
    { 0x7634fceb, 85 },
    { 0x4bb4fcec, 23 },
    { 0x5344fced, 87 },
    { 0x4d04fced, 83 },
    { 0xab04fcef, 55 },
    { 0, 0 },
    { 0x5964fcf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4404fcf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e4fcfa, 91 },
    { 0, 0 },
    { 0x81e4fcfc, 92 },
    { 0, 0 },
    { 0x9d84fcfe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc14fd02, 34 },
    { 0x8784fd02, 90 },
    { 0x7c94fd04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214fd07, 61 },
    { 0xb5f4fd08, 34 },
    { 0x75e4fd09, 90 },
    { 0xd1e4fd08, 93 },
    { 0x7af4fd0b, 43 },
    { 0x6554fd0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d4fd10, 91 },
    { 0xa4a4fd11, 87 },
    { 0xaa4fd12, 87 },
    { 0xafc4fd12, 81 },
    { 0x4294fd13, 33 },
    { 0x76d4fd11, 66 },
    { 0xc0c4fd13, 81 },
    { 0x52e4fd17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494fd1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe394fd1f, 94 },
    { 0x4b54fd20, 96 },
    { 0x1fd4fd20, 43 },
    { 0xd1c4fd22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4174fd26, 52 },
    { 0, 0 },
    { 0x2bf4fd28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104fd2f, 23 },
    { 0x734fd30, 61 },
    { 0x1354fd2f, 95 },
    { 0x9f34fd32, 33 },
    { 0, 0 },
    { 0xdd64fd34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d4fd38, 100 },
    { 0x61d4fd38, 90 },
    { 0x7e34fd3a, 90 },
    { 0x6f24fd3b, 23 },
    { 0xcbc4fd39, 99 },
    { 0xa6e4fd38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4fd40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744fd48, 81 },
    { 0, 0 },
    { 0x6344fd4a, 46 },
    { 0x1a44fd4b, 23 },
    { 0x9564fd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db4fd51, 55 },
    { 0, 0 },
    { 0x2814fd53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e4fd56, 23 },
    { 0xa594fd56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4fd5a, 43 },
    { 0x7444fd5b, 90 },
    { 0xef44fd5b, 76 },
    { 0, 0 },
    { 0x5454fd5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f4fd61, 89 },
    { 0x9bd4fd62, 33 },
    { 0, 0 },
    { 0x7b14fd64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb974fd6b, 90 },
    { 0x52c4fd6b, 91 },
    { 0x8fa4fd6d, 34 },
    { 0x7184fd6e, 99 },
    { 0x2854fd6e, 55 },
    { 0x3314fd70, 60 },
    { 0xe614fd70, 23 },
    { 0xa1d4fd72, 77 },
    { 0x64b4fd72, 95 },
    { 0x4b04fd72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e4fd78, 66 },
    { 0, 0 },
    { 0x4ab4fd7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa4fd7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af4fd82, 75 },
    { 0x50b4fd82, 98 },
    { 0x74fd84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c4fd8b, 23 },
    { 0xc194fd8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24fd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb4fd94, 90 },
    { 0xc744fd95, 92 },
    { 0, 0 },
    { 0xdef4fd97, 43 },
    { 0x7534fd97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e4fd9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4fda7, 90 },
    { 0, 0 },
    { 0xdf44fda9, 23 },
    { 0x6a74fdaa, 87 },
    { 0xd8e4fdab, 46 },
    { 0xab84fdac, 82 },
    { 0x1004fdad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf74fdb2, 33 },
    { 0, 0 },
    { 0x3774fdb4, 95 },
    { 0x4634fdb5, 43 },
    { 0xc084fdb5, 90 },
    { 0, 0 },
    { 0x59d4fdb8, 34 },
    { 0xfa4fdb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4fdbd, 33 },
    { 0xc094fdbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a4fdc8, 34 },
    { 0xa1f4fdc9, 55 },
    { 0x5884fdc9, 88 },
    { 0xd494fdc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a94fdce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4fdd7, 23 },
    { 0xee54fdd8, 95 },
    { 0, 0 },
    { 0x9b64fdda, 76 },
    { 0x7cf4fddb, 80 },
    { 0x9d54fddc, 66 },
    { 0, 0 },
    { 0xb954fdde, 33 },
    { 0, 0 },
    { 0xed4fde0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed34fde6, 46 },
    { 0xed44fde6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d4fdf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b4fdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2314fdfb, 55 },
    { 0, 0 },
    { 0xd914fdfd, 98 },
    { 0x46b4fdfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9524fe03, 88 },
    { 0, 0 },
    { 0x9b04fe05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba4fe0c, 33 },
    { 0x13f4fe0d, 90 },
    { 0x2c94fe0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d54fe11, 55 },
    { 0, 0 },
    { 0x6e64fe13, 90 },
    { 0xb794fe14, 53 },
    { 0xc2b4fe13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e84fe20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354fe2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c4fe2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b24fe32, 61 },
    { 0xea54fe33, 34 },
    { 0, 0 },
    { 0xa254fe35, 61 },
    { 0x1764fe36, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1514fe39, 69 },
    { 0x19c4fe3a, 43 },
    { 0x5714fe3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe414fe3e, 76 },
    { 0xd054fe3f, 34 },
    { 0x57a4fe3f, 95 },
    { 0, 0 },
    { 0x1e74fe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4fe48, 92 },
    { 0xcd24fe48, 90 },
    { 0xd7f4fe48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4214fe4f, 55 },
    { 0x4974fe50, 23 },
    { 0x4344fe50, 33 },
    { 0x44d4fe50, 76 },
    { 0, 0 },
    { 0xe034fe54, 85 },
    { 0x3a4fe54, 61 },
    { 0x1534fe56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1124fe5b, 90 },
    { 0, 0 },
    { 0x4664fe5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054fe63, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7404fe66, 91 },
    { 0x5be4fe66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724fe6b, 43 },
    { 0x1a74fe6c, 34 },
    { 0xc8c4fe6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb174fe75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2884fe7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf154fe82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d4fe85, 61 },
    { 0xaeb4fe86, 95 },
    { 0, 0 },
    { 0x94fe88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c4fe96, 93 },
    { 0x4314fe96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e4fe9d, 85 },
    { 0xedb4fe9e, 98 },
    { 0x3bf4fe9f, 61 },
    { 0x1914fe9f, 23 },
    { 0x26c4fe9f, 98 },
    { 0x4074fea2, 33 },
    { 0xffc4fea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d4fea6, 95 },
    { 0, 0 },
    { 0xeac4fea8, 95 },
    { 0, 0 },
    { 0xc464feaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014feaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f4feb8, 87 },
    { 0x2f34feb9, 43 },
    { 0, 0 },
    { 0x5894febb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f4fec4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d4fec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf014fecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa4fed1, 100 },
    { 0x19d4fed2, 94 },
    { 0x2b84fed3, 46 },
    { 0x7fd4fed4, 59 },
    { 0x164fed5, 61 },
    { 0xa384fed6, 93 },
    { 0x3a54fed5, 93 },
    { 0xd224fed2, 33 },
    { 0x8764fed1, 17 },
    { 0x9a14fed5, 43 },
    { 0x4684fedb, 98 },
    { 0x8354fedc, 76 },
    { 0xe404fedb, 99 },
    { 0x4014fede, 34 },
    { 0x5bc4fed1, 61 },
    { 0, 0 },
    { 0x7e84fee1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea74fee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5544feeb, 84 },
    { 0, 0 },
    { 0x8e24feed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6964fef3, 23 },
    { 0xf0b4fef4, 55 },
    { 0x9ab4fef5, 67 },
    { 0xb274fef5, 97 },
    { 0x9f64fef4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad74ff02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe604ff06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf4ff09, 95 },
    { 0xd2b4ff0a, 61 },
    { 0x8814ff09, 52 },
    { 0x7724ff0c, 90 },
    { 0, 0 },
    { 0xde04ff0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd984ff13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854ff16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944ff1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6854ff1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554ff20, 90 },
    { 0x2824ff21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd144ff24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74ff2d, 85 },
    { 0xfda4ff2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d4ff32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6274ff35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d4ff3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d14ff44, 23 },
    { 0, 0 },
    { 0xa104ff46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b4ff49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4ff51, 55 },
    { 0x3414ff52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024ff57, 61 },
    { 0, 0 },
    { 0xa494ff59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf414ff5f, 33 },
    { 0x50c4ff60, 61 },
    { 0xc9b4ff60, 43 },
    { 0xba24ff62, 90 },
    { 0, 0 },
    { 0xbad4ff64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4954ff68, 23 },
    { 0, 0 },
    { 0x7264ff6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994ff70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4ff73, 33 },
    { 0x4824ff74, 24 },
    { 0xe544ff74, 90 },
    { 0xfcb4ff73, 61 },
    { 0x40d4ff77, 43 },
    { 0x2a4ff78, 98 },
    { 0xe3a4ff73, 67 },
    { 0, 0 },
    { 0xb4b4ff7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc4ff7e, 75 },
    { 0x8ba4ff7f, 61 },
    { 0x3544ff80, 95 },
    { 0x2ba4ff81, 52 },
    { 0xc5b4ff7f, 43 },
    { 0xeb94ff83, 61 },
    { 0x8dd4ff84, 67 },
    { 0xb414ff83, 34 },
    { 0x1654ff86, 85 },
    { 0xf464ff81, 85 },
    { 0xe544ff83, 55 },
    { 0x63d4ff89, 90 },
    { 0x8de4ff8a, 95 },
    { 0xd104ff89, 98 },
    { 0, 0 },
    { 0x85b4ff8d, 90 },
    { 0, 0 },
    { 0x8484ff8f, 90 },
    { 0xbad4ff8f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a4ff93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf274ff96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae4ff99, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4ff9e, 99 },
    { 0xdc84ff9f, 95 },
    { 0x3b34ffa0, 68 },
    { 0xbfb4ffa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c4ffa4, 98 },
    { 0xf034ffa5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc4ffa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f24ffaf, 23 },
    { 0, 0 },
    { 0xd9f4ffb1, 42 },
    { 0x554ffb2, 90 },
    { 0, 0 },
    { 0x91b4ffb4, 61 },
    { 0x2404ffb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9184ffbe, 55 },
    { 0x8d14ffbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a54ffc7, 23 },
    { 0x1724ffc7, 33 },
    { 0x6794ffc8, 61 },
    { 0x4904ffca, 90 },
    { 0xb614ffca, 69 },
    { 0, 0 },
    { 0x6644ffcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de4ffd0, 90 },
    { 0, 0 },
    { 0x9e4ffd2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3444ffd5, 56 },
    { 0, 0 },
    { 0xb174ffd7, 23 },
    { 0, 0 },
    { 0x4c84ffd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b4ffdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344ffdf, 90 },
    { 0x9a44ffdf, 23 },
    { 0, 0 },
    { 0xa2b4ffe2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324ffe8, 76 },
    { 0x99b4ffe9, 90 },
    { 0x8d34ffea, 90 },
    { 0x8db4ffe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c4ffee, 61 },
    { 0x4e34ffef, 23 },
    { 0x20b4fff0, 46 },
    { 0xbff4fff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf4fff8, 23 },
    { 0xa684fff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf594fffe, 95 },
    { 0x1284fffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f50002, 33 },
    { 0xa7550003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee50007, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f5000d, 78 },
    { 0x5665000e, 90 },
    { 0xe3b5000e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc650012, 95 },
    { 0x88b50013, 90 },
    { 0xf6d50014, 61 },
    { 0, 0 },
    { 0xb2e50016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32250021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e50024, 78 },
    { 0, 0 },
    { 0xfe950026, 76 },
    { 0x60450027, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69950033, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68550037, 84 },
    { 0, 0 },
    { 0x1b250039, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xef25003c, 90 },
    { 0xff15003c, 90 },
    { 0x22f5003d, 33 },
    { 0, 0 },
    { 0xcaa50040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32450045, 64 },
    { 0x72d50046, 98 },
    { 0, 0 },
    { 0x20250048, 95 },
    { 0xd0e50049, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5665004f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe155005a, 33 },
    { 0x5065005b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5005f, 95 },
    { 0x60a5005f, 40 },
    { 0, 0 },
    { 0xa1950062, 99 },
    { 0x80f50063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5450068, 43 },
    { 0, 0 },
    { 0x3365006a, 23 },
    { 0x1e45006b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a50071, 93 },
    { 0xcf850071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca350075, 65 },
    { 0xa0250075, 90 },
    { 0x29a50077, 78 },
    { 0xd9950075, 33 },
    { 0x28250077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea5007c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa250083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d50087, 84 },
    { 0x70c50087, 61 },
    { 0xbc550087, 76 },
    { 0xa3c5008a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe50092, 33 },
    { 0xe8550093, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595009a, 67 },
    { 0x4ac5009b, 84 },
    { 0, 0 },
    { 0xe8d5009d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515500a1, 23 },
    { 0, 0 },
    { 0x47f500a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00500a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d500a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be500ae, 61 },
    { 0, 0 },
    { 0xc0500b0, 17 },
    { 0xc30500b1, 98 },
    { 0xe5c500b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0500b9, 33 },
    { 0, 0 },
    { 0x7e500bb, 23 },
    { 0x90a500bc, 76 },
    { 0xfbe500bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c500c0, 52 },
    { 0x409500c1, 23 },
    { 0xee2500c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8500c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15500ca, 87 },
    { 0x633500ca, 23 },
    { 0xb78500cc, 84 },
    { 0xa88500cb, 90 },
    { 0xdc2500ce, 61 },
    { 0x60e500cc, 93 },
    { 0x7cf500d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84500d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4500d7, 33 },
    { 0, 0 },
    { 0x96a500d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d500dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9500e0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2500e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e500eb, 95 },
    { 0x21500ec, 52 },
    { 0x5b6500ed, 33 },
    { 0, 0 },
    { 0xb03500ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f500f7, 90 },
    { 0xfc0500f8, 90 },
    { 0xda2500f8, 33 },
    { 0x3af500f8, 95 },
    { 0x7af500f8, 85 },
    { 0x43c500fc, 33 },
    { 0xc14500f9, 68 },
    { 0, 0 },
    { 0x48b500ff, 95 },
    { 0xb450100, 90 },
    { 0, 0 },
    { 0x4e450102, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8950109, 43 },
    { 0, 0 },
    { 0x5415010b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c950116, 90 },
    { 0xc6750117, 95 },
    { 0x67950116, 33 },
    { 0x97650119, 76 },
    { 0xaea50119, 34 },
    { 0x4f65011b, 23 },
    { 0x87050119, 98 },
    { 0xa9c5011a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e50120, 61 },
    { 0xd850121, 23 },
    { 0x4d850120, 34 },
    { 0, 0 },
    { 0x29f50124, 46 },
    { 0, 0 },
    { 0x21450126, 93 },
    { 0, 0 },
    { 0x76350128, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9515012d, 33 },
    { 0xf135012d, 23 },
    { 0, 0 },
    { 0xf1a50130, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98850135, 66 },
    { 0x83b50136, 69 },
    { 0x90e50136, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c95013a, 100 },
    { 0x4af5013b, 67 },
    { 0x2615013a, 43 },
    { 0x2495013b, 76 },
    { 0x9435013b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65450142, 43 },
    { 0x53250143, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe50147, 33 },
    { 0x1da50147, 90 },
    { 0xd1150149, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5014f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe50152, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8250155, 90 },
    { 0x27950156, 98 },
    { 0, 0 },
    { 0x64b50158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15015f, 80 },
    { 0xfdd50160, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee50164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5016a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd50170, 61 },
    { 0x79d50171, 90 },
    { 0x94950172, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f50175, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f5017c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac50180, 55 },
    { 0x46550180, 43 },
    { 0, 0 },
    { 0xeef50183, 33 },
    { 0, 0 },
    { 0xa4050185, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a50189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e65018c, 43 },
    { 0x5a65018d, 61 },
    { 0, 0 },
    { 0xb585018f, 23 },
    { 0, 0 },
    { 0x9e850191, 82 },
    { 0x50650192, 90 },
    { 0, 0 },
    { 0xcfb50194, 68 },
    { 0xf2750194, 76 },
    { 0x90950196, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a5019c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563501a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x368501a3, 75 },
    { 0x656501a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x259501a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf501aa, 78 },
    { 0x3fc501ab, 33 },
    { 0x729501ac, 34 },
    { 0x373501ad, 90 },
    { 0x2b8501ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5501b1, 81 },
    { 0, 0 },
    { 0xf6c501b3, 33 },
    { 0x59a501b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1501c0, 66 },
    { 0xa9e501c1, 93 },
    { 0x86e501c2, 61 },
    { 0, 0 },
    { 0xb65501c4, 95 },
    { 0xe69501c5, 91 },
    { 0x85b501c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e501ca, 95 },
    { 0x5c3501ca, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0501ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db501d2, 93 },
    { 0x6f501d2, 90 },
    { 0x8c6501d3, 61 },
    { 0xf6f501d2, 33 },
    { 0, 0 },
    { 0x852501d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee501db, 95 },
    { 0xc5c501dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e501df, 90 },
    { 0x9f6501e0, 90 },
    { 0x974501e1, 95 },
    { 0x8cf501e2, 34 },
    { 0xb22501df, 23 },
    { 0, 0 },
    { 0x90d501e5, 90 },
    { 0xa8e501e5, 55 },
    { 0x79d501e5, 17 },
    { 0xa8a501e6, 87 },
    { 0x2e3501e9, 87 },
    { 0, 0 },
    { 0x5b4501eb, 65 },
    { 0xb0f501ec, 23 },
    { 0xf3501ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae501f0, 98 },
    { 0, 0 },
    { 0x270501f2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b501fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff50201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d50207, 61 },
    { 0x42b50208, 90 },
    { 0x4fc50207, 23 },
    { 0xf5f50208, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a45020d, 53 },
    { 0x4415020d, 78 },
    { 0, 0 },
    { 0x2d150210, 90 },
    { 0x84650211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6975021f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7350225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c750229, 93 },
    { 0xb2b5022a, 61 },
    { 0xb665022a, 61 },
    { 0x5022b, 52 },
    { 0x9a15022a, 19 },
    { 0, 0 },
    { 0x5655022f, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd50234, 55 },
    { 0, 0 },
    { 0x90a50236, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5023b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5023f, 76 },
    { 0, 0 },
    { 0x6ef50241, 55 },
    { 0, 0 },
    { 0x39d50243, 97 },
    { 0, 0 },
    { 0x83950245, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c50248, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c50253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f50257, 95 },
    { 0xb0e50257, 61 },
    { 0x6cc50259, 90 },
    { 0x10f5025a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515026a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf15026d, 95 },
    { 0xdbc5026e, 61 },
    { 0, 0 },
    { 0x45950270, 67 },
    { 0, 0 },
    { 0xf3350272, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8350277, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf575027a, 90 },
    { 0x775027b, 95 },
    { 0x8ce5027c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1455027f, 34 },
    { 0x9ab50280, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8050285, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x63650288, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc50293, 90 },
    { 0xc5550293, 77 },
    { 0, 0 },
    { 0xde750296, 33 },
    { 0, 0 },
    { 0x1cd50298, 90 },
    { 0xab350299, 90 },
    { 0, 0 },
    { 0x915029b, 23 },
    { 0x98b5029c, 61 },
    { 0x4585029d, 23 },
    { 0x5ef5029b, 95 },
    { 0, 0 },
    { 0x1ff502a0, 33 },
    { 0xa75502a1, 85 },
    { 0x9fd502a1, 75 },
    { 0xb19502a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d502a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2502ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df502b2, 80 },
    { 0x340502b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69502b8, 34 },
    { 0, 0 },
    { 0x98b502ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd502c1, 23 },
    { 0, 0 },
    { 0xd2502c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304502c7, 90 },
    { 0xbb2502c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f502cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d502cf, 61 },
    { 0x1c7502d0, 65 },
    { 0x27e502d1, 90 },
    { 0x67c502d2, 90 },
    { 0xb6e502d3, 40 },
    { 0, 0 },
    { 0xab6502d5, 23 },
    { 0x95502d6, 61 },
    { 0x1b6502d5, 33 },
    { 0x6a1502d8, 95 },
    { 0x966502d9, 33 },
    { 0, 0 },
    { 0x105502db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f502de, 98 },
    { 0xa4b502df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01502e4, 85 },
    { 0xbdf502e4, 33 },
    { 0x77b502e6, 56 },
    { 0x4df502e7, 99 },
    { 0xf6f502e8, 46 },
    { 0xbc4502e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13502ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2502f4, 68 },
    { 0xeac502f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e502f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1502fb, 98 },
    { 0xfe7502fb, 94 },
    { 0, 0 },
    { 0x8f5502fe, 91 },
    { 0xabd502ff, 61 },
    { 0, 0 },
    { 0x26b50301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49650305, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a50309, 76 },
    { 0xde050309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f5030f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38950316, 98 },
    { 0x65150316, 91 },
    { 0x7b950316, 76 },
    { 0, 0 },
    { 0x2cf5031a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b65031e, 81 },
    { 0, 0 },
    { 0xf0750320, 66 },
    { 0xd2650321, 98 },
    { 0, 0 },
    { 0xae550323, 90 },
    { 0xaea50324, 34 },
    { 0, 0 },
    { 0xa8050326, 98 },
    { 0xcfa50326, 17 },
    { 0x72150328, 90 },
    { 0x7ca50329, 33 },
    { 0x8ef5032a, 23 },
    { 0x31050326, 98 },
    { 0x5325032c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e50331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96150337, 98 },
    { 0x5a050337, 61 },
    { 0x75350339, 91 },
    { 0xa1450339, 23 },
    { 0, 0 },
    { 0x8a5033c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75033f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e50343, 33 },
    { 0xfb850344, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b50348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e5034c, 33 },
    { 0x27f5034d, 95 },
    { 0x7e85034e, 90 },
    { 0x9ab5034e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed50352, 95 },
    { 0x67850352, 23 },
    { 0x68050353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5445035e, 76 },
    { 0, 0 },
    { 0x43450360, 23 },
    { 0xfb450360, 95 },
    { 0x55c50361, 76 },
    { 0xfcc50362, 61 },
    { 0, 0 },
    { 0xea850365, 23 },
    { 0, 0 },
    { 0xd7b50367, 84 },
    { 0x61b50368, 78 },
    { 0x6ea50369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5036d, 90 },
    { 0, 0 },
    { 0x5f85036f, 88 },
    { 0, 0 },
    { 0xa850371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e50375, 93 },
    { 0xc4c50376, 98 },
    { 0xb4a50376, 90 },
    { 0xf050378, 76 },
    { 0x1d250375, 76 },
    { 0xa245037a, 55 },
    { 0x61050377, 33 },
    { 0x2b95037a, 55 },
    { 0, 0 },
    { 0xe565037e, 61 },
    { 0, 0 },
    { 0xfc050380, 99 },
    { 0xd2450380, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92250384, 23 },
    { 0x62450385, 95 },
    { 0xcfb50386, 93 },
    { 0xbd850387, 61 },
    { 0xc1950385, 46 },
    { 0x4ef50384, 61 },
    { 0x25f5038a, 67 },
    { 0, 0 },
    { 0x76b5038c, 66 },
    { 0x5645038d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b250392, 68 },
    { 0, 0 },
    { 0x55850394, 52 },
    { 0, 0 },
    { 0xf5350396, 67 },
    { 0xc9350397, 95 },
    { 0xb5750398, 33 },
    { 0xe4550399, 53 },
    { 0, 0 },
    { 0x98d5039b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c95039f, 33 },
    { 0x9e6503a0, 98 },
    { 0x560503a0, 55 },
    { 0xdfc503a2, 84 },
    { 0x9e4503a2, 61 },
    { 0x4c9503a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d503b1, 90 },
    { 0x3f2503b2, 98 },
    { 0xce4503b1, 61 },
    { 0x91d503b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c503b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac503bf, 90 },
    { 0, 0 },
    { 0x682503c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc503c5, 61 },
    { 0x4c9503c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14503d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92503d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0503dc, 76 },
    { 0, 0 },
    { 0x3e3503de, 91 },
    { 0x38d503de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94503e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86503ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da503f7, 93 },
    { 0xfbe503f8, 55 },
    { 0x6fa503f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de503fc, 66 },
    { 0x1e8503fd, 91 },
    { 0xaad503fd, 38 },
    { 0x31d503fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c50402, 98 },
    { 0x5ee50402, 23 },
    { 0xe9050403, 22 },
    { 0xc050405, 34 },
    { 0x14f50406, 84 },
    { 0xe9e50407, 78 },
    { 0x64a50407, 78 },
    { 0x1f050407, 23 },
    { 0x66650406, 52 },
    { 0, 0 },
    { 0x1e75040c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6650414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd50417, 33 },
    { 0xc7450418, 76 },
    { 0xbcd50417, 99 },
    { 0x2f650419, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46050421, 23 },
    { 0xe6350422, 23 },
    { 0xa650421, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f50429, 76 },
    { 0x39b5042a, 90 },
    { 0x6735042b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5042f, 83 },
    { 0, 0 },
    { 0x86450431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b50434, 61 },
    { 0, 0 },
    { 0x53650436, 23 },
    { 0, 0 },
    { 0xb9950438, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa005043f, 53 },
    { 0, 0 },
    { 0x66150441, 38 },
    { 0xba850441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c950445, 33 },
    { 0x14350446, 90 },
    { 0xc1750445, 90 },
    { 0, 0 },
    { 0xf7b50449, 33 },
    { 0xae450449, 90 },
    { 0x3325044a, 98 },
    { 0, 0 },
    { 0xa785044d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb850451, 78 },
    { 0xd1550452, 23 },
    { 0x90250453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5045a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965045d, 46 },
    { 0x98f5045e, 88 },
    { 0x4e05045f, 90 },
    { 0xa3550460, 90 },
    { 0xfaa5045e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac050467, 23 },
    { 0xac250468, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1075046c, 99 },
    { 0, 0 },
    { 0xf245046e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc250478, 95 },
    { 0xc8550479, 17 },
    { 0, 0 },
    { 0x5835047b, 23 },
    { 0xf8a5047c, 91 },
    { 0x7f45047d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e50484, 33 },
    { 0x34050484, 98 },
    { 0, 0 },
    { 0xc8150487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf605048d, 36 },
    { 0xba35048d, 68 },
    { 0xd1c5048f, 67 },
    { 0x7c15048d, 23 },
    { 0, 0 },
    { 0x8ec50492, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9950497, 55 },
    { 0x9b950498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5155049d, 33 },
    { 0x17b5049e, 61 },
    { 0x3915049f, 95 },
    { 0, 0 },
    { 0x6cb504a1, 95 },
    { 0x649504a1, 61 },
    { 0xa8a504a2, 43 },
    { 0, 0 },
    { 0x522504a5, 90 },
    { 0xe1b504a6, 23 },
    { 0x4d504a7, 23 },
    { 0x292504a6, 84 },
    { 0x807504a9, 84 },
    { 0x576504a7, 91 },
    { 0x3cf504a9, 90 },
    { 0x8cc504a8, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f504b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610504b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1504bc, 76 },
    { 0x30a504bd, 61 },
    { 0xd11504bd, 61 },
    { 0x5e2504bd, 90 },
    { 0x46504bf, 93 },
    { 0x8ac504be, 98 },
    { 0x74504bd, 90 },
    { 0x82504c3, 98 },
    { 0x6fd504c1, 55 },
    { 0, 0 },
    { 0x23d504c6, 91 },
    { 0x8b3504c7, 90 },
    { 0x450504c8, 61 },
    { 0, 0 },
    { 0x9ab504ca, 43 },
    { 0x31504ca, 75 },
    { 0, 0 },
    { 0xe29504cd, 46 },
    { 0x67504cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0504d2, 61 },
    { 0x245504d3, 93 },
    { 0, 0 },
    { 0x415504d5, 98 },
    { 0, 0 },
    { 0x8ce504d7, 55 },
    { 0x84504d8, 55 },
    { 0x591504d7, 98 },
    { 0xc88504da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd504df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39504e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61504e8, 98 },
    { 0x6ce504e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6504f3, 46 },
    { 0xaa8504f3, 98 },
    { 0x6bd504f5, 77 },
    { 0x23a504f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7504fa, 55 },
    { 0x1a0504fa, 95 },
    { 0xe0c504fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c550500, 23 },
    { 0x92950501, 33 },
    { 0xca950502, 80 },
    { 0, 0 },
    { 0xdc650504, 55 },
    { 0, 0 },
    { 0x31c50506, 66 },
    { 0x5b650507, 90 },
    { 0, 0 },
    { 0x3c850509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca5050c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d85050f, 95 },
    { 0x15c5050f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52050515, 17 },
    { 0, 0 },
    { 0x3d650517, 46 },
    { 0x53b50517, 55 },
    { 0x90a50519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5051e, 52 },
    { 0x9415051f, 90 },
    { 0, 0 },
    { 0xafb50521, 33 },
    { 0, 0 },
    { 0xc2950523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f650527, 43 },
    { 0x93150527, 34 },
    { 0, 0 },
    { 0x3735052a, 33 },
    { 0, 0 },
    { 0x7d25052c, 33 },
    { 0xb265052c, 88 },
    { 0x3f65052c, 67 },
    { 0xa2f5052e, 98 },
    { 0x4b450530, 78 },
    { 0x52350531, 61 },
    { 0x94950532, 17 },
    { 0x76850532, 90 },
    { 0x57750534, 87 },
    { 0x20a50532, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47250539, 90 },
    { 0, 0 },
    { 0xbaf5053b, 61 },
    { 0x1c85053c, 55 },
    { 0xf455053d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90750541, 67 },
    { 0x71c50542, 87 },
    { 0, 0 },
    { 0xa0c50544, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c50548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa635054c, 90 },
    { 0x7ce5054d, 90 },
    { 0xfc35054d, 90 },
    { 0x9015054f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35055a, 67 },
    { 0x8e15055b, 43 },
    { 0xeb05055c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850561, 61 },
    { 0, 0 },
    { 0x22e50563, 91 },
    { 0x1c750563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae450568, 46 },
    { 0, 0 },
    { 0xe105056a, 55 },
    { 0, 0 },
    { 0xc035056c, 83 },
    { 0x1175056d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac150571, 43 },
    { 0xd1750572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a50575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa085057a, 98 },
    { 0x45d5057b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4850581, 33 },
    { 0x22050581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77050589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d5058f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b550593, 95 },
    { 0x73b50594, 48 },
    { 0x23750594, 90 },
    { 0x47b50596, 87 },
    { 0xb1850594, 24 },
    { 0xdfb50598, 33 },
    { 0x26150598, 87 },
    { 0, 0 },
    { 0x7925059b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e505a0, 100 },
    { 0, 0 },
    { 0x5a6505a2, 34 },
    { 0, 0 },
    { 0x7d7505a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41505ab, 33 },
    { 0, 0 },
    { 0xe0a505ad, 65 },
    { 0, 0 },
    { 0x333505af, 95 },
    { 0xf8e505af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6505b4, 93 },
    { 0x6e6505b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa505bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413505c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a505c4, 95 },
    { 0x3fb505c5, 33 },
    { 0xd86505c6, 77 },
    { 0, 0 },
    { 0x753505c8, 98 },
    { 0x9a4505c8, 33 },
    { 0xfe1505c8, 95 },
    { 0, 0 },
    { 0x322505cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2505cf, 33 },
    { 0xdb4505cf, 66 },
    { 0xbe6505d1, 68 },
    { 0x8a3505d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298505d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1505da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x814505dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8505e3, 76 },
    { 0x42a505e3, 85 },
    { 0x552505e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x314505e8, 61 },
    { 0x4a7505e8, 33 },
    { 0x154505e9, 98 },
    { 0xa5f505e8, 61 },
    { 0x4bb505ec, 40 },
    { 0x740505ec, 90 },
    { 0x4d9505ee, 90 },
    { 0x4bc505ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d505f2, 90 },
    { 0, 0 },
    { 0xaf1505f4, 75 },
    { 0x5f505f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf505fa, 98 },
    { 0, 0 },
    { 0x60c505fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050601, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c50608, 85 },
    { 0x5e850608, 90 },
    { 0, 0 },
    { 0x4875060b, 76 },
    { 0x735060c, 87 },
    { 0, 0 },
    { 0xbb35060e, 61 },
    { 0x6cf5060e, 33 },
    { 0, 0 },
    { 0xc0450611, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe850617, 88 },
    { 0, 0 },
    { 0x43250619, 23 },
    { 0xe875061a, 77 },
    { 0x6d75061b, 90 },
    { 0, 0 },
    { 0x5e45061d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f650623, 61 },
    { 0, 0 },
    { 0xc0c50625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30850628, 23 },
    { 0, 0 },
    { 0x7f15062a, 81 },
    { 0, 0 },
    { 0x4855062c, 95 },
    { 0, 0 },
    { 0x63a5062e, 98 },
    { 0, 0 },
    { 0x80950630, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a550633, 61 },
    { 0x58250634, 95 },
    { 0xb9950635, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d50638, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c5063c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd925063f, 61 },
    { 0x85950640, 23 },
    { 0x36e50641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d50646, 98 },
    { 0x4ba50646, 95 },
    { 0xe550648, 95 },
    { 0x14b50648, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575064e, 52 },
    { 0, 0 },
    { 0xfe850650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a650653, 55 },
    { 0x3d650654, 33 },
    { 0xd9b50655, 65 },
    { 0x48c50656, 95 },
    { 0xb4850657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf5065b, 17 },
    { 0x29f5065c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5350663, 61 },
    { 0, 0 },
    { 0x20b50665, 46 },
    { 0x50150665, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a75066a, 33 },
    { 0, 0 },
    { 0xe115066c, 98 },
    { 0xdad5066d, 84 },
    { 0x3b65066d, 46 },
    { 0x67a5066d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1850673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23150676, 98 },
    { 0x3e250677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8765067a, 23 },
    { 0, 0 },
    { 0xf2c5067c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115067f, 93 },
    { 0x7b65067f, 33 },
    { 0xea150681, 85 },
    { 0x4c55067f, 95 },
    { 0x4ea50681, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4650688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5068e, 66 },
    { 0x9685068f, 61 },
    { 0x83a5068f, 97 },
    { 0x7650690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b50694, 46 },
    { 0, 0 },
    { 0xd9150696, 95 },
    { 0x3450697, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35069a, 55 },
    { 0x7d45069a, 23 },
    { 0xec45069a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754506a7, 17 },
    { 0x940506a8, 33 },
    { 0, 0 },
    { 0x17f506aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd506b2, 23 },
    { 0, 0 },
    { 0x147506b4, 90 },
    { 0xf52506b5, 90 },
    { 0x651506b4, 34 },
    { 0x2f3506b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x720506ba, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907506be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93506c3, 43 },
    { 0x888506c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac506c7, 46 },
    { 0x780506c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94506cb, 85 },
    { 0x664506cc, 87 },
    { 0, 0 },
    { 0x7f3506ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6506d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1506d5, 99 },
    { 0x9e1506d5, 17 },
    { 0xeb7506d7, 33 },
    { 0, 0 },
    { 0xd0d506d9, 61 },
    { 0xecb506da, 66 },
    { 0xfa4506db, 76 },
    { 0, 0 },
    { 0x545506dd, 61 },
    { 0xaf6506de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f506e2, 23 },
    { 0, 0 },
    { 0x59c506e4, 61 },
    { 0x7ed506e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2506ed, 90 },
    { 0x826506ee, 23 },
    { 0x58c506ed, 81 },
    { 0x57f506ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213506f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4506fc, 23 },
    { 0xd4506fd, 33 },
    { 0x684506fe, 43 },
    { 0, 0 },
    { 0x7b650700, 98 },
    { 0, 0 },
    { 0xbf250702, 61 },
    { 0, 0 },
    { 0xad450704, 95 },
    { 0, 0 },
    { 0xf2e50706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745070c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515070f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3350712, 33 },
    { 0x8d450713, 43 },
    { 0xd6d50712, 95 },
    { 0x43150715, 90 },
    { 0x72b50715, 61 },
    { 0x58550715, 46 },
    { 0xff750718, 34 },
    { 0x19950719, 55 },
    { 0x20b50719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a50720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd50727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3095072a, 90 },
    { 0xda85072b, 81 },
    { 0xf695072a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x105072f, 61 },
    { 0, 0 },
    { 0x1a150731, 46 },
    { 0, 0 },
    { 0x29450733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85073d, 61 },
    { 0xade5073e, 68 },
    { 0x3225073f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe850743, 34 },
    { 0x51850744, 95 },
    { 0xc0f50745, 76 },
    { 0x94050746, 33 },
    { 0xf0a50747, 81 },
    { 0, 0 },
    { 0x90850749, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e5074e, 90 },
    { 0x3df5074f, 23 },
    { 0x9e95074f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80450755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d850759, 77 },
    { 0x83450759, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a850764, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x92350767, 55 },
    { 0, 0 },
    { 0x9f350769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba5076f, 23 },
    { 0xf1150770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63050774, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c850777, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9715077a, 80 },
    { 0xe625077a, 34 },
    { 0, 0 },
    { 0x89e5077d, 43 },
    { 0, 0 },
    { 0xb345077f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f50783, 90 },
    { 0x8d50784, 52 },
    { 0, 0 },
    { 0xa4750786, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x10050789, 76 },
    { 0, 0 },
    { 0x12b5078b, 65 },
    { 0xe885078c, 76 },
    { 0x6795078d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x32350790, 88 },
    { 0, 0 },
    { 0x65e50792, 61 },
    { 0xc1250793, 98 },
    { 0x99f50794, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce50798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc385079f, 61 },
    { 0, 0 },
    { 0xfb5507a1, 77 },
    { 0x4eb507a1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x290507a5, 90 },
    { 0x4a8507a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x742507a9, 95 },
    { 0x1aa507aa, 33 },
    { 0x8e7507ab, 98 },
    { 0, 0 },
    { 0xcfe507ad, 43 },
    { 0x44d507ad, 93 },
    { 0x1e0507af, 85 },
    { 0xf19507b0, 93 },
    { 0x564507af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32507b4, 88 },
    { 0, 0 },
    { 0xa75507b6, 61 },
    { 0xfd0507b7, 23 },
    { 0, 0 },
    { 0x3f2507b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22507bc, 55 },
    { 0, 0 },
    { 0x100507be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf507c2, 61 },
    { 0x986507c2, 84 },
    { 0xd8e507c4, 23 },
    { 0x64e507c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x266507c8, 43 },
    { 0, 0 },
    { 0xc10507ca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169507ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6507d2, 34 },
    { 0xe04507d2, 34 },
    { 0, 0 },
    { 0xadf507d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b507d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3507db, 84 },
    { 0xc96507dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c507e9, 52 },
    { 0x604507ea, 98 },
    { 0, 0 },
    { 0x866507ec, 95 },
    { 0xb88507ed, 61 },
    { 0x272507ee, 76 },
    { 0, 0 },
    { 0xf5c507f0, 61 },
    { 0, 0 },
    { 0x54b507f2, 75 },
    { 0, 0 },
    { 0x4b0507f4, 90 },
    { 0xa01507f5, 95 },
    { 0, 0 },
    { 0xb3d507f7, 61 },
    { 0, 0 },
    { 0x578507f9, 67 },
    { 0x759507fa, 90 },
    { 0xef6507f9, 34 },
    { 0x91a507fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f450800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4150803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde50808, 90 },
    { 0, 0 },
    { 0x1a15080a, 90 },
    { 0, 0 },
    { 0x8095080c, 95 },
    { 0, 0 },
    { 0xc795080e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b50813, 34 },
    { 0x70b50814, 33 },
    { 0xf3e50815, 33 },
    { 0x52c50816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9345081d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab50821, 43 },
    { 0x32d50822, 90 },
    { 0x91050823, 100 },
    { 0x17450821, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d50827, 85 },
    { 0x7b950828, 75 },
    { 0, 0 },
    { 0xdfd5082a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a050831, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0950834, 48 },
    { 0xb4f50835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3665083e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30750842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c850848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa685084c, 46 },
    { 0, 0 },
    { 0xc865084e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b50851, 61 },
    { 0xdd250852, 61 },
    { 0x51c50853, 23 },
    { 0x73450853, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f50859, 100 },
    { 0x81b50859, 17 },
    { 0xda25085b, 90 },
    { 0x185085c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe5085f, 61 },
    { 0x35050860, 17 },
    { 0x3f50860, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31550864, 77 },
    { 0, 0 },
    { 0x8e650866, 95 },
    { 0, 0 },
    { 0x86050868, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b5086b, 98 },
    { 0x4685086b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd5086f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e50875, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a5087a, 90 },
    { 0, 0 },
    { 0xf785087c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31750880, 90 },
    { 0, 0 },
    { 0x41c50882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c50885, 76 },
    { 0, 0 },
    { 0xa3550887, 57 },
    { 0xe1250888, 84 },
    { 0, 0 },
    { 0xa865088a, 33 },
    { 0x7315088a, 46 },
    { 0x7c15088a, 95 },
    { 0x9b85088b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c50894, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe575089b, 43 },
    { 0x4455089b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671508a0, 91 },
    { 0x553508a1, 95 },
    { 0x30e508a1, 33 },
    { 0xe7d508a3, 61 },
    { 0x119508a4, 76 },
    { 0, 0 },
    { 0xc82508a6, 61 },
    { 0xb1f508a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b508ab, 98 },
    { 0xfa3508ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48508af, 90 },
    { 0x9fc508b0, 61 },
    { 0x4c508b1, 99 },
    { 0, 0 },
    { 0xb21508b3, 90 },
    { 0, 0 },
    { 0x7d3508b5, 23 },
    { 0, 0 },
    { 0xb93508b7, 76 },
    { 0, 0 },
    { 0x844508b9, 98 },
    { 0, 0 },
    { 0x29b508bb, 55 },
    { 0, 0 },
    { 0x8b5508bd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e508c0, 95 },
    { 0xd8a508c1, 80 },
    { 0x539508c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c508c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731508c9, 34 },
    { 0xd508ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1508ce, 81 },
    { 0xb40508cf, 52 },
    { 0xbe0508d0, 69 },
    { 0x2a5508cf, 76 },
    { 0xa3a508d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331508d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321508db, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x641508de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85508e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1508e9, 61 },
    { 0xcf1508e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759508f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4508f9, 90 },
    { 0x597508fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58950900, 98 },
    { 0xe5b50901, 52 },
    { 0xf5550900, 55 },
    { 0xe4850903, 95 },
    { 0x97950904, 90 },
    { 0, 0 },
    { 0xa8250906, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98650909, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc50915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25050918, 90 },
    { 0x4ce50919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905091c, 34 },
    { 0x33e5091d, 87 },
    { 0x34c5091e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f550921, 33 },
    { 0xffb50921, 55 },
    { 0xe5d50923, 95 },
    { 0x32850924, 34 },
    { 0x7e150925, 61 },
    { 0xae50926, 46 },
    { 0xf2a50927, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35092f, 91 },
    { 0x1075092f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26750933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6750938, 43 },
    { 0x3ba50938, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05093d, 55 },
    { 0, 0 },
    { 0x5605093f, 69 },
    { 0x5a55093f, 90 },
    { 0x39550940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c50945, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77650948, 91 },
    { 0xe6e50948, 17 },
    { 0x41c5094a, 95 },
    { 0x7f650948, 61 },
    { 0xca85094c, 90 },
    { 0xa905094d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5950952, 61 },
    { 0xa3750952, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e50956, 23 },
    { 0x47d50957, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c5095a, 95 },
    { 0, 0 },
    { 0xfa35095c, 100 },
    { 0, 0 },
    { 0xeec5095e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba50961, 46 },
    { 0x38e50961, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c50966, 61 },
    { 0xa4750966, 46 },
    { 0x45b50967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2005096c, 33 },
    { 0xce45096d, 23 },
    { 0xa325096c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e850971, 90 },
    { 0x99850971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d750979, 61 },
    { 0xa865097a, 33 },
    { 0x765097b, 66 },
    { 0x49e5097c, 68 },
    { 0x8985097a, 90 },
    { 0xfa05097e, 90 },
    { 0xd5097e, 61 },
    { 0xb650980, 80 },
    { 0x5b650981, 33 },
    { 0xbe15097f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f50986, 23 },
    { 0xfb50986, 91 },
    { 0x54950988, 66 },
    { 0, 0 },
    { 0xec35098a, 52 },
    { 0xeea5098b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5098e, 77 },
    { 0, 0 },
    { 0x52050990, 76 },
    { 0, 0 },
    { 0xcae50992, 90 },
    { 0xd0450992, 34 },
    { 0, 0 },
    { 0xf9550995, 98 },
    { 0x30f50995, 67 },
    { 0x9a350995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f5099c, 85 },
    { 0x7315099d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4509a0, 95 },
    { 0, 0 },
    { 0x62d509a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57509a6, 33 },
    { 0x34a509a6, 23 },
    { 0x1cc509a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98509af, 88 },
    { 0xede509b0, 90 },
    { 0x6d5509b1, 52 },
    { 0x58509b2, 52 },
    { 0x488509b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab509ba, 91 },
    { 0, 0 },
    { 0x19b509bc, 61 },
    { 0, 0 },
    { 0x727509be, 90 },
    { 0x52d509be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0509c3, 55 },
    { 0, 0 },
    { 0x62b509c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5509cc, 90 },
    { 0, 0 },
    { 0x21509ce, 93 },
    { 0x161509cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f509d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416509d9, 61 },
    { 0xd04509da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db509e0, 90 },
    { 0x8f8509e1, 23 },
    { 0x49c509e0, 61 },
    { 0, 0 },
    { 0x264509e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae509e9, 23 },
    { 0xe94509ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f509ed, 43 },
    { 0x2f4509ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f509f1, 95 },
    { 0x3ce509f2, 55 },
    { 0xe1e509f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382509fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c50a01, 33 },
    { 0, 0 },
    { 0xb8a50a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81550a08, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e50a0b, 61 },
    { 0x16250a0b, 55 },
    { 0x55550a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31850a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c450a14, 23 },
    { 0x4b550a14, 61 },
    { 0xce150a14, 61 },
    { 0x25a50a14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff50a1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a50a20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd050a23, 43 },
    { 0, 0 },
    { 0x85550a25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40750a28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db50a35, 76 },
    { 0xc9650a35, 77 },
    { 0xeb650a36, 95 },
    { 0x93550a38, 91 },
    { 0, 0 },
    { 0x250a3a, 95 },
    { 0x52850a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb450a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58850a49, 98 },
    { 0xd4950a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea50a4d, 23 },
    { 0, 0 },
    { 0x56350a4f, 76 },
    { 0x85150a50, 76 },
    { 0x24e50a51, 23 },
    { 0x96950a51, 33 },
    { 0xaaf50a50, 61 },
    { 0x38f50a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab50a5f, 23 },
    { 0xc1750a5f, 55 },
    { 0, 0 },
    { 0x41250a62, 93 },
    { 0x32050a63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2050a67, 94 },
    { 0x10650a68, 76 },
    { 0xfd850a69, 43 },
    { 0xcc350a6a, 98 },
    { 0xaf350a6a, 90 },
    { 0, 0 },
    { 0x3c550a6d, 34 },
    { 0, 0 },
    { 0x5f250a6f, 34 },
    { 0x94a50a6f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d50a76, 98 },
    { 0x4f850a76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2450a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1150a7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f750a84, 23 },
    { 0x29850a85, 90 },
    { 0xbda50a85, 23 },
    { 0x9d150a85, 61 },
    { 0xf4650a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc50a8e, 55 },
    { 0, 0 },
    { 0x7550a90, 83 },
    { 0x41a50a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33550a96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8250aa0, 61 },
    { 0, 0 },
    { 0x66850aa2, 61 },
    { 0, 0 },
    { 0x69350aa4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x90350aa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd50aaa, 53 },
    { 0, 0 },
    { 0x4a450aac, 43 },
    { 0, 0 },
    { 0x95250aae, 61 },
    { 0, 0 },
    { 0x3d750ab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b850ab3, 90 },
    { 0xf7950ab4, 93 },
    { 0, 0 },
    { 0xdfc50ab6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e50aba, 34 },
    { 0xff450abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32150abf, 76 },
    { 0x46f50ac0, 90 },
    { 0, 0 },
    { 0x15450ac2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d50ac8, 33 },
    { 0, 0 },
    { 0xf4850aca, 98 },
    { 0x15650acb, 90 },
    { 0x45e50acc, 87 },
    { 0, 0 },
    { 0x57550ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1650ad1, 99 },
    { 0, 0 },
    { 0xc7050ad3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de50ad7, 78 },
    { 0xa9550ad8, 99 },
    { 0, 0 },
    { 0x67d50ada, 33 },
    { 0xaf250adb, 100 },
    { 0x6ae50adc, 95 },
    { 0xa7050add, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d50ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35950ae4, 23 },
    { 0, 0 },
    { 0xc9350ae6, 65 },
    { 0xf1350ae6, 90 },
    { 0x40750ae6, 98 },
    { 0xce250ae9, 95 },
    { 0, 0 },
    { 0xf7350aeb, 55 },
    { 0xbbf50aec, 53 },
    { 0xe2550aed, 46 },
    { 0x51950aec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76050af2, 90 },
    { 0x66e50af2, 61 },
    { 0, 0 },
    { 0xe6850af5, 78 },
    { 0x3f550af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee250b02, 43 },
    { 0, 0 },
    { 0x85350b04, 78 },
    { 0x16450b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17050b0a, 88 },
    { 0x93550b0b, 23 },
    { 0x72d50b0c, 90 },
    { 0x98c50b0a, 90 },
    { 0x57b50b0e, 61 },
    { 0x14750b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f50b1c, 93 },
    { 0, 0 },
    { 0x65f50b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5050b26, 81 },
    { 0, 0 },
    { 0xdb350b28, 90 },
    { 0x8ee50b28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86750b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf450b32, 99 },
    { 0x9ab50b33, 23 },
    { 0xc5d50b33, 67 },
    { 0xcad50b35, 78 },
    { 0x24850b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2650b3a, 95 },
    { 0x19750b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a50b42, 66 },
    { 0x5c450b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb350b49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40450b4d, 65 },
    { 0x83150b4e, 61 },
    { 0x74550b4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa550b53, 90 },
    { 0, 0 },
    { 0x78550b55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e50b5b, 52 },
    { 0, 0 },
    { 0xb8c50b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2650b61, 98 },
    { 0x55c50b61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c450b65, 23 },
    { 0, 0 },
    { 0xe6a50b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac150b72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a50b76, 98 },
    { 0, 0 },
    { 0x6e150b78, 98 },
    { 0xb8d50b79, 85 },
    { 0x25e50b7a, 99 },
    { 0x23850b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4450b7e, 23 },
    { 0xd7550b7f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d50b86, 97 },
    { 0xf5050b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f50b8a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb50b90, 88 },
    { 0x32550b90, 93 },
    { 0, 0 },
    { 0xaab50b93, 61 },
    { 0, 0 },
    { 0x1a950b95, 67 },
    { 0x2550b96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b550b9a, 84 },
    { 0, 0 },
    { 0xf2250b9c, 84 },
    { 0x2ad50b9c, 95 },
    { 0xe0550b9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a850ba4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75250ba9, 52 },
    { 0x28450baa, 33 },
    { 0, 0 },
    { 0x8d450bac, 78 },
    { 0x50850bad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4550bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe50bba, 23 },
    { 0xbb450bbb, 95 },
    { 0x9f650bbc, 72 },
    { 0xd1d50bbc, 77 },
    { 0x6cd50bbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64450bc2, 96 },
    { 0, 0 },
    { 0xcf950bc4, 40 },
    { 0x10d50bc5, 66 },
    { 0, 0 },
    { 0x32e50bc7, 91 },
    { 0xdb150bc8, 55 },
    { 0x44850bc9, 33 },
    { 0x18350bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa450bd7, 90 },
    { 0xcc350bd8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d450bdc, 88 },
    { 0, 0 },
    { 0x78a50bde, 61 },
    { 0, 0 },
    { 0x18a50be0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x75850be3, 33 },
    { 0, 0 },
    { 0xc4c50be5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32550bed, 100 },
    { 0xbcc50bed, 90 },
    { 0x25950bef, 95 },
    { 0x18150bef, 33 },
    { 0x4850bf1, 99 },
    { 0x71a50bf2, 23 },
    { 0x88a50bf3, 98 },
    { 0x92850bed, 55 },
    { 0xd7b50bf4, 52 },
    { 0, 0 },
    { 0xee750bf7, 98 },
    { 0xa9d50bf8, 98 },
    { 0xbd450bf9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f350bfc, 23 },
    { 0x7f050bfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2750c00, 52 },
    { 0xeb50c01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78650c0c, 90 },
    { 0, 0 },
    { 0x96d50c0e, 95 },
    { 0, 0 },
    { 0x70150c10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68850c16, 33 },
    { 0xc1450c17, 77 },
    { 0, 0 },
    { 0xf0350c19, 61 },
    { 0xdb750c1a, 47 },
    { 0, 0 },
    { 0x48250c1c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa50c25, 53 },
    { 0x7ae50c26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb850c29, 76 },
    { 0, 0 },
    { 0x87250c2b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82350c2e, 46 },
    { 0x80c50c2f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a50c32, 100 },
    { 0x58c50c32, 33 },
    { 0xb1150c33, 61 },
    { 0, 0 },
    { 0x29c50c36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a050c3a, 34 },
    { 0, 0 },
    { 0xf5650c3c, 33 },
    { 0xb7450c3c, 33 },
    { 0x5e250c3c, 55 },
    { 0xcee50c3d, 23 },
    { 0x18650c40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6050c44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96950c48, 23 },
    { 0, 0 },
    { 0xf0f50c4a, 34 },
    { 0x67350c4a, 81 },
    { 0x22850c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26550c53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c50c5a, 61 },
    { 0, 0 },
    { 0x2a550c5c, 56 },
    { 0x1b950c5d, 66 },
    { 0, 0 },
    { 0x95d50c5f, 95 },
    { 0xf6750c60, 23 },
    { 0xbfa50c60, 98 },
    { 0x62150c60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c50c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45350c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77250c70, 23 },
    { 0x86b50c70, 76 },
    { 0xe2e50c72, 33 },
    { 0, 0 },
    { 0x89650c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2650c79, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf50c7c, 76 },
    { 0xa2f50c7c, 33 },
    { 0, 0 },
    { 0x4d950c7f, 100 },
    { 0, 0 },
    { 0x52b50c81, 61 },
    { 0, 0 },
    { 0x41550c83, 23 },
    { 0xbe750c83, 98 },
    { 0, 0 },
    { 0x64250c86, 90 },
    { 0, 0 },
    { 0xd6e50c88, 95 },
    { 0xf2850c88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea450c8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a250c91, 99 },
    { 0x22c50c91, 46 },
    { 0xdbe50c91, 95 },
    { 0x3e450c92, 33 },
    { 0, 0 },
    { 0xe6450c96, 76 },
    { 0x9bd50c97, 46 },
    { 0x27a50c96, 34 },
    { 0x39a50c97, 76 },
    { 0xc1c50c98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a50ca1, 90 },
    { 0, 0 },
    { 0xa6e50ca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f50ca6, 68 },
    { 0x7b250ca7, 91 },
    { 0, 0 },
    { 0xf1650ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc950cac, 46 },
    { 0x72050cad, 33 },
    { 0x73a50cac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad50cb3, 76 },
    { 0xee850cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f50cb7, 67 },
    { 0x2ea50cb8, 76 },
    { 0, 0 },
    { 0x79b50cba, 52 },
    { 0xe0e50cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb650cc1, 95 },
    { 0, 0 },
    { 0x40250cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53750cc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae50ccf, 43 },
    { 0xcc550cd0, 76 },
    { 0, 0 },
    { 0x4db50cd2, 77 },
    { 0xe1450cd3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d50cd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef50cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb50ce2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f50ce6, 66 },
    { 0x66c50ce6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98650cf0, 33 },
    { 0xb1450cf1, 61 },
    { 0xdcd50cf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b450cf5, 80 },
    { 0x9ff50cf6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be50cf9, 34 },
    { 0x52f50cfa, 99 },
    { 0, 0 },
    { 0x55350cfc, 23 },
    { 0, 0 },
    { 0xdbc50cfe, 98 },
    { 0x57050cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c850d03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77750d07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2350d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac50d17, 87 },
    { 0xd5c50d18, 82 },
    { 0x65150d19, 52 },
    { 0x7c050d19, 23 },
    { 0x85750d1b, 76 },
    { 0, 0 },
    { 0x5df50d1d, 82 },
    { 0xe5850d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38650d22, 95 },
    { 0, 0 },
    { 0xe8950d24, 98 },
    { 0x11950d25, 40 },
    { 0xe4050d26, 55 },
    { 0, 0 },
    { 0xf650d28, 90 },
    { 0x41050d29, 90 },
    { 0x18c50d2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda150d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf650d33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac50d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39550d41, 98 },
    { 0, 0 },
    { 0x91c50d43, 33 },
    { 0, 0 },
    { 0x89f50d45, 23 },
    { 0x66250d46, 17 },
    { 0xd0150d46, 98 },
    { 0xc2350d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de50d4e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b50d53, 92 },
    { 0x39850d53, 90 },
    { 0xc8550d53, 34 },
    { 0x1cd50d55, 61 },
    { 0x22c50d57, 100 },
    { 0xb6f50d57, 90 },
    { 0, 0 },
    { 0x30850d5a, 87 },
    { 0x4a50d5b, 43 },
    { 0x26850d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28750d63, 12 },
    { 0x4f850d64, 93 },
    { 0x78150d65, 23 },
    { 0xf2650d65, 61 },
    { 0xc5c50d63, 52 },
    { 0x55850d68, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc50d6b, 33 },
    { 0xf1b50d6b, 90 },
    { 0xe7250d6d, 23 },
    { 0xf3750d6e, 98 },
    { 0x57f50d6f, 34 },
    { 0x8aa50d6f, 87 },
    { 0x2050d71, 77 },
    { 0x40150d72, 98 },
    { 0x95750d6d, 52 },
    { 0x6e750d74, 76 },
    { 0x97d50d75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9650d78, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e50d7b, 90 },
    { 0x8cb50d7c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b50d7f, 98 },
    { 0x98a50d80, 61 },
    { 0xa3750d81, 76 },
    { 0x5d650d80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1150d88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de50d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50050d90, 95 },
    { 0, 0 },
    { 0x9d150d92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50050d96, 95 },
    { 0xf1050d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be50d9a, 52 },
    { 0, 0 },
    { 0x94d50d9c, 23 },
    { 0x69a50d9d, 95 },
    { 0xbdd50d9d, 61 },
    { 0x1150d9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97750da4, 36 },
    { 0, 0 },
    { 0xb4350da6, 53 },
    { 0x50da6, 34 },
    { 0, 0 },
    { 0x1a50da9, 98 },
    { 0, 0 },
    { 0x23850dab, 90 },
    { 0xcba50dab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e50db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda250db9, 93 },
    { 0, 0 },
    { 0xd0450dbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b950dc1, 23 },
    { 0x7ab50dc1, 33 },
    { 0xa50dc1, 52 },
    { 0x10d50dc4, 99 },
    { 0x38c50dc5, 93 },
    { 0x3ee50dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc50dce, 90 },
    { 0, 0 },
    { 0xf4950dd0, 95 },
    { 0xca950dd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9250dd4, 100 },
    { 0, 0 },
    { 0x2d550dd6, 61 },
    { 0xc6850dd7, 23 },
    { 0x18b50dd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d50ddb, 80 },
    { 0x17750ddb, 43 },
    { 0, 0 },
    { 0xef050dde, 93 },
    { 0, 0 },
    { 0x82850de0, 33 },
    { 0, 0 },
    { 0x8d650de2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb50de6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93550dea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4050ded, 84 },
    { 0, 0 },
    { 0xc7650def, 60 },
    { 0xd550def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c50df3, 46 },
    { 0xee250df4, 55 },
    { 0xecc50df5, 81 },
    { 0, 0 },
    { 0x90d50df7, 33 },
    { 0x64250df8, 72 },
    { 0xbd650df7, 95 },
    { 0x53750dfa, 61 },
    { 0xf3350dfa, 23 },
    { 0x6c550dfa, 34 },
    { 0xa5050dfa, 33 },
    { 0, 0 },
    { 0x4a150dff, 84 },
    { 0, 0 },
    { 0xf9950e01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e50e0a, 100 },
    { 0, 0 },
    { 0xb4b50e0c, 55 },
    { 0, 0 },
    { 0xa1150e0e, 33 },
    { 0xc3950e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35750e12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92550e16, 61 },
    { 0x4b750e17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea50e1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f50e1d, 38 },
    { 0xb0950e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a50e21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7150e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd50e28, 61 },
    { 0xa1e50e29, 80 },
    { 0x6c50e2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f50e2d, 78 },
    { 0, 0 },
    { 0x94250e2f, 90 },
    { 0x25250e2f, 33 },
    { 0x67750e31, 43 },
    { 0xdc950e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c50e3b, 46 },
    { 0x2af50e3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc50e42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x93350e45, 98 },
    { 0xa6e50e46, 46 },
    { 0x73150e46, 33 },
    { 0xd5b50e46, 55 },
    { 0xead50e49, 76 },
    { 0x88750e46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f750e50, 98 },
    { 0x5ba50e51, 99 },
    { 0x5450e51, 95 },
    { 0, 0 },
    { 0x6a850e54, 23 },
    { 0xe0a50e55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf350e59, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e050e62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a750e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c950e69, 67 },
    { 0xbed50e6a, 61 },
    { 0xe3a50e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2650e73, 90 },
    { 0, 0 },
    { 0x72750e75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f350e7a, 93 },
    { 0x79c50e7b, 65 },
    { 0x1ca50e7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b950e7f, 34 },
    { 0xa0250e80, 92 },
    { 0x9a750e81, 90 },
    { 0x60850e82, 33 },
    { 0xafb50e83, 76 },
    { 0xd6150e83, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a050e87, 33 },
    { 0xbf150e87, 55 },
    { 0xf4750e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc950e8c, 90 },
    { 0x63150e8d, 90 },
    { 0xc2150e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6650e93, 75 },
    { 0x33d50e94, 68 },
    { 0xfd450e93, 98 },
    { 0x7fe50e94, 90 },
    { 0x1df50e97, 75 },
    { 0, 0 },
    { 0x1d150e99, 83 },
    { 0, 0 },
    { 0x19050e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e750ea5, 99 },
    { 0x4450ea6, 36 },
    { 0xcc650ea7, 90 },
    { 0, 0 },
    { 0x23850ea9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b50eb2, 90 },
    { 0, 0 },
    { 0xd3d50eb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a50eb8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c650ebb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5650ec0, 90 },
    { 0x2d50ec0, 95 },
    { 0x98650ec1, 33 },
    { 0, 0 },
    { 0x21450ec4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce150ec9, 61 },
    { 0x88850eca, 76 },
    { 0x78c50ecb, 95 },
    { 0, 0 },
    { 0xf8550ecd, 78 },
    { 0, 0 },
    { 0xc8050ecf, 33 },
    { 0x8e350ed0, 90 },
    { 0xaf350ecf, 61 },
    { 0, 0 },
    { 0x60c50ed3, 98 },
    { 0, 0 },
    { 0x63550ed5, 55 },
    { 0xe2850ed5, 66 },
    { 0xb8750ed7, 55 },
    { 0, 0 },
    { 0xef750ed9, 23 },
    { 0, 0 },
    { 0x3c450edb, 43 },
    { 0, 0 },
    { 0x95150edd, 93 },
    { 0, 0 },
    { 0xbd450edf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c350ee3, 33 },
    { 0xfb750ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a650eee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be50ef1, 95 },
    { 0, 0 },
    { 0xb850ef3, 95 },
    { 0xdfc50ef3, 98 },
    { 0xd5f50ef3, 33 },
    { 0xbab50ef4, 76 },
    { 0xda950ef7, 90 },
    { 0, 0 },
    { 0xb1a50ef9, 61 },
    { 0, 0 },
    { 0x12350efb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d150eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e350f04, 91 },
    { 0, 0 },
    { 0x1ac50f06, 95 },
    { 0xc3350f06, 61 },
    { 0x3e650f08, 95 },
    { 0xdef50f09, 33 },
    { 0x35650f0a, 23 },
    { 0x74950f0b, 95 },
    { 0, 0 },
    { 0xdef50f0d, 67 },
    { 0x9f50f0e, 61 },
    { 0x32050f0e, 76 },
    { 0xf4250f0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b50f17, 98 },
    { 0xf6850f18, 91 },
    { 0x7ad50f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650f1f, 61 },
    { 0, 0 },
    { 0x5b50f21, 69 },
    { 0xe1050f22, 93 },
    { 0, 0 },
    { 0xf4450f24, 43 },
    { 0, 0 },
    { 0xfcc50f26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b850f29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f50f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26950f35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a50f39, 95 },
    { 0x4a50f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34550f3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c50f43, 98 },
    { 0xe5250f44, 85 },
    { 0x98f50f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec50f48, 55 },
    { 0x30f50f49, 98 },
    { 0, 0 },
    { 0x37750f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8250f4f, 100 },
    { 0xe0850f50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4650f57, 33 },
    { 0xe7e50f58, 65 },
    { 0xf9750f59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91550f62, 87 },
    { 0xa3250f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e50f6d, 78 },
    { 0x81550f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f250f75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd50f78, 33 },
    { 0x77c50f79, 61 },
    { 0x3a550f7a, 23 },
    { 0xd6450f7a, 69 },
    { 0x42c50f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b50f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d50f88, 43 },
    { 0, 0 },
    { 0x36450f8a, 61 },
    { 0x71450f8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5550f94, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35850f97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0450f9e, 91 },
    { 0xc1c50f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12150fa2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb150fa5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0750fa8, 98 },
    { 0x10850fa8, 87 },
    { 0xb1450faa, 61 },
    { 0xa2050fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb950fae, 61 },
    { 0xd4b50faf, 90 },
    { 0xb7c50faf, 46 },
    { 0x59450fb1, 52 },
    { 0, 0 },
    { 0x7e450fb3, 90 },
    { 0, 0 },
    { 0xbdf50fb5, 93 },
    { 0xa2450fb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e950fbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab650fca, 46 },
    { 0, 0 },
    { 0xab050fcc, 33 },
    { 0x8b350fcd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1850fd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e50fd6, 90 },
    { 0xb3150fd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e650fda, 33 },
    { 0x52d50fda, 76 },
    { 0xdb650fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8450fdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80950fe7, 98 },
    { 0x6c750fe8, 76 },
    { 0x14d50fe9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72850ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df50ff6, 99 },
    { 0xde850ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e950ffc, 33 },
    { 0xeb450ffd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da51000, 34 },
    { 0x87051001, 90 },
    { 0x5f051002, 61 },
    { 0x8cf51002, 92 },
    { 0xc8551004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc25100c, 61 },
    { 0xf205100d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf51010, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc351015, 99 },
    { 0, 0 },
    { 0x48551017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb5101d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef51022, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575102a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e5102f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c51034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5103b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91351040, 90 },
    { 0x1051040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd5104d, 95 },
    { 0x59d5104d, 55 },
    { 0x6a5104f, 87 },
    { 0xbaf51050, 78 },
    { 0xf6c51050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d51057, 67 },
    { 0x15d51058, 34 },
    { 0x68c51059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c5105c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5851060, 95 },
    { 0xdbe51061, 61 },
    { 0xafd51062, 95 },
    { 0xeb651061, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa251068, 98 },
    { 0, 0 },
    { 0xd335106a, 90 },
    { 0xc675106b, 57 },
    { 0xa195106a, 61 },
    { 0x7b05106d, 90 },
    { 0x7e25106e, 34 },
    { 0, 0 },
    { 0xebe51070, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45551073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44951079, 23 },
    { 0x9495107a, 77 },
    { 0, 0 },
    { 0xdb55107c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9251085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe5108c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6151090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58951093, 33 },
    { 0x63951094, 95 },
    { 0xa251095, 87 },
    { 0x44251096, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b5109d, 98 },
    { 0x81c5109e, 98 },
    { 0, 0 },
    { 0xd03510a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f510a8, 33 },
    { 0x3d4510a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98510ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a510b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822510b9, 93 },
    { 0x835510ba, 23 },
    { 0x921510bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4510be, 61 },
    { 0x8db510be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8510cc, 83 },
    { 0x8a2510cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b510d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e510d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753510e0, 98 },
    { 0x516510e1, 99 },
    { 0x85b510e2, 76 },
    { 0, 0 },
    { 0x6ac510e4, 84 },
    { 0x343510e5, 61 },
    { 0x267510e5, 33 },
    { 0xb70510e5, 33 },
    { 0x75510e8, 95 },
    { 0x2ef510e5, 95 },
    { 0x4a3510e7, 77 },
    { 0x9cc510eb, 95 },
    { 0xb13510eb, 90 },
    { 0x7bc510eb, 33 },
    { 0xd3f510e7, 98 },
    { 0, 0 },
    { 0x4e4510f0, 61 },
    { 0x208510f0, 99 },
    { 0, 0 },
    { 0xea8510f3, 95 },
    { 0x45510f4, 90 },
    { 0, 0 },
    { 0x855510f6, 55 },
    { 0x3c7510f7, 61 },
    { 0x833510f8, 61 },
    { 0, 0 },
    { 0x5ff510fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf510ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db51103, 87 },
    { 0, 0 },
    { 0xdb951105, 23 },
    { 0x93451105, 95 },
    { 0xe6e51106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc245110d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251111, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9551114, 61 },
    { 0xfae51114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee551118, 98 },
    { 0x3de51119, 93 },
    { 0x16551119, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5111d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38851122, 55 },
    { 0, 0 },
    { 0x84651124, 90 },
    { 0xa2051125, 55 },
    { 0xd9c51124, 93 },
    { 0x77851127, 23 },
    { 0x7ad51127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5005112f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38851140, 90 },
    { 0x15d51141, 52 },
    { 0x2b951140, 80 },
    { 0x7ff51143, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51147, 55 },
    { 0x2d751148, 90 },
    { 0x12551147, 61 },
    { 0x4e651149, 95 },
    { 0xfeb51149, 90 },
    { 0, 0 },
    { 0x4065114d, 99 },
    { 0xf65114e, 78 },
    { 0xc35114d, 33 },
    { 0, 0 },
    { 0x41951151, 87 },
    { 0xbc651152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb51155, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5115f, 23 },
    { 0x5f451160, 33 },
    { 0x37c51161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1751166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715116a, 34 },
    { 0, 0 },
    { 0x1975116c, 98 },
    { 0xe945116d, 98 },
    { 0x5745116e, 90 },
    { 0x1d25116f, 76 },
    { 0, 0 },
    { 0x9351171, 93 },
    { 0x45d51172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd51176, 55 },
    { 0x55c51177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf351182, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45051186, 95 },
    { 0xacb51187, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5118d, 99 },
    { 0xffb5118e, 52 },
    { 0x1015118f, 23 },
    { 0, 0 },
    { 0xf4351191, 61 },
    { 0x1c951192, 90 },
    { 0, 0 },
    { 0xb3151194, 23 },
    { 0, 0 },
    { 0x3bb51196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5119b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7235119f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439511a8, 93 },
    { 0xb9511a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d511ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0511bf, 90 },
    { 0, 0 },
    { 0x1c9511c1, 99 },
    { 0, 0 },
    { 0x787511c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb511c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x364511ca, 90 },
    { 0x6c3511ca, 52 },
    { 0x87511cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd511d0, 77 },
    { 0x6fe511d0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401511d9, 55 },
    { 0x4db511da, 46 },
    { 0x5d511d9, 17 },
    { 0x62b511dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8511e0, 66 },
    { 0, 0 },
    { 0x524511e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a511e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c511ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906511ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b511f6, 99 },
    { 0x266511f6, 66 },
    { 0xbc3511f6, 95 },
    { 0, 0 },
    { 0x385511fa, 66 },
    { 0xba7511fa, 76 },
    { 0xc44511fc, 65 },
    { 0, 0 },
    { 0xb6e511fe, 76 },
    { 0, 0 },
    { 0x11551200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c851203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a651208, 43 },
    { 0xfde51209, 77 },
    { 0x45151209, 23 },
    { 0xe905120b, 95 },
    { 0, 0 },
    { 0xed05120d, 33 },
    { 0xd095120d, 75 },
    { 0x56e5120f, 61 },
    { 0, 0 },
    { 0x12851211, 85 },
    { 0x76451212, 55 },
    { 0x64151213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d251219, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15121d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea451223, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe51227, 66 },
    { 0xda751227, 38 },
    { 0x88451227, 23 },
    { 0xac15122a, 80 },
    { 0x7055122b, 90 },
    { 0, 0 },
    { 0x30c5122d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1351231, 61 },
    { 0xd7e51231, 91 },
    { 0xce851231, 95 },
    { 0xc7651233, 55 },
    { 0xe7e51235, 33 },
    { 0x48451236, 90 },
    { 0x4c851235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2185123e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0151244, 61 },
    { 0x9851245, 52 },
    { 0xafc51244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x65951249, 55 },
    { 0, 0 },
    { 0x8a35124b, 90 },
    { 0xac55124b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b5124f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b51252, 77 },
    { 0xae251253, 90 },
    { 0xd051254, 33 },
    { 0x97351254, 55 },
    { 0, 0 },
    { 0x79851257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515125b, 76 },
    { 0x1115125c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51265, 33 },
    { 0, 0 },
    { 0xa651267, 34 },
    { 0x7da51267, 84 },
    { 0x37b51268, 34 },
    { 0xe225126a, 33 },
    { 0x3f51269, 33 },
    { 0x9a65126c, 76 },
    { 0xb215126d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72051270, 68 },
    { 0x9c451271, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1151277, 47 },
    { 0, 0 },
    { 0xc1a51279, 61 },
    { 0, 0 },
    { 0x62a5127b, 23 },
    { 0, 0 },
    { 0xf375127d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13651284, 33 },
    { 0, 0 },
    { 0x90151286, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91851289, 91 },
    { 0xdc851289, 61 },
    { 0x435128b, 68 },
    { 0xe7b5128b, 90 },
    { 0xa415128d, 90 },
    { 0, 0 },
    { 0x6175128f, 33 },
    { 0xea951290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d51299, 33 },
    { 0x2695129a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6225129d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11512a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43512a7, 90 },
    { 0x76d512a8, 91 },
    { 0xd01512a7, 78 },
    { 0xb75512a7, 33 },
    { 0xc03512a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213512af, 84 },
    { 0, 0 },
    { 0x81b512b1, 90 },
    { 0xb95512b2, 23 },
    { 0, 0 },
    { 0x5e8512b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3512b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x235512bc, 69 },
    { 0x8a2512bd, 33 },
    { 0xf1f512bd, 95 },
    { 0x327512bf, 95 },
    { 0, 0 },
    { 0xa58512c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08512c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61512cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7512d0, 81 },
    { 0, 0 },
    { 0x99e512d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca512d6, 90 },
    { 0, 0 },
    { 0x250512d8, 23 },
    { 0, 0 },
    { 0x442512da, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c512e0, 76 },
    { 0, 0 },
    { 0x938512e2, 17 },
    { 0, 0 },
    { 0x211512e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80512ea, 61 },
    { 0, 0 },
    { 0x506512ec, 98 },
    { 0x28f512ed, 43 },
    { 0x583512ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db512f5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6512f9, 90 },
    { 0xda512f9, 95 },
    { 0x612512fb, 95 },
    { 0x86b512fc, 48 },
    { 0xa63512fd, 91 },
    { 0x502512fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b251302, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c51307, 23 },
    { 0, 0 },
    { 0xed751309, 90 },
    { 0x6e35130a, 90 },
    { 0xb6f5130a, 90 },
    { 0x4855130c, 78 },
    { 0, 0 },
    { 0x2b25130e, 93 },
    { 0, 0 },
    { 0x30551310, 77 },
    { 0x86151311, 100 },
    { 0, 0 },
    { 0xdd451313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d51318, 33 },
    { 0xdc551319, 78 },
    { 0x4385131a, 90 },
    { 0xfd551318, 55 },
    { 0xe9f5131c, 90 },
    { 0x4915131c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c851325, 77 },
    { 0x5e851325, 76 },
    { 0x5f151327, 76 },
    { 0x55851327, 98 },
    { 0x5ab51325, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b75132d, 95 },
    { 0xc6d5132e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3751332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e51336, 96 },
    { 0x97651336, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8415133c, 90 },
    { 0, 0 },
    { 0xb765133e, 85 },
    { 0xaee5133f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a51343, 98 },
    { 0x41051344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e25134d, 61 },
    { 0, 0 },
    { 0x1665134f, 100 },
    { 0x7a451350, 90 },
    { 0xbd85134f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e51355, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625135a, 23 },
    { 0, 0 },
    { 0x8085135c, 33 },
    { 0x8665135c, 33 },
    { 0, 0 },
    { 0xa555135f, 87 },
    { 0x17a51360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98451369, 98 },
    { 0, 0 },
    { 0x41b5136b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e51375, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025137c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f551384, 61 },
    { 0, 0 },
    { 0x80951386, 90 },
    { 0x89751386, 90 },
    { 0xc1051387, 34 },
    { 0x1e051389, 46 },
    { 0, 0 },
    { 0xce85138b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e5138e, 90 },
    { 0x7475138e, 95 },
    { 0x36f51390, 34 },
    { 0x4de5138f, 66 },
    { 0xc1e51392, 33 },
    { 0xae851393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30451397, 55 },
    { 0xe0d51398, 90 },
    { 0x36951399, 34 },
    { 0x4c95139a, 88 },
    { 0xa125139b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f55139e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea513a1, 55 },
    { 0x198513a2, 84 },
    { 0xbf4513a3, 76 },
    { 0x446513a2, 76 },
    { 0xfec513a1, 43 },
    { 0, 0 },
    { 0xc21513a7, 34 },
    { 0x199513a8, 61 },
    { 0xe27513a9, 33 },
    { 0, 0 },
    { 0xb5c513ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f513ae, 81 },
    { 0x49513ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12513b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c513be, 87 },
    { 0, 0 },
    { 0x881513c0, 23 },
    { 0xbbb513c0, 67 },
    { 0xaa1513c0, 33 },
    { 0x302513c3, 77 },
    { 0x2cf513c1, 55 },
    { 0xfa1513c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05513cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610513d0, 56 },
    { 0x5b6513d1, 90 },
    { 0, 0 },
    { 0xf4a513d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20513d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5513df, 23 },
    { 0xdca513e0, 23 },
    { 0, 0 },
    { 0xf49513e2, 40 },
    { 0xb0a513e3, 65 },
    { 0x604513e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5513e8, 17 },
    { 0x33513e9, 23 },
    { 0, 0 },
    { 0xa46513eb, 55 },
    { 0, 0 },
    { 0xb2c513ed, 43 },
    { 0, 0 },
    { 0x216513ef, 90 },
    { 0x2e7513f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f513f4, 33 },
    { 0, 0 },
    { 0x490513f6, 95 },
    { 0x862513f7, 33 },
    { 0xd92513f7, 98 },
    { 0x26f513f9, 84 },
    { 0, 0 },
    { 0xbeb513fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac651400, 33 },
    { 0x11d51401, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1615140b, 91 },
    { 0, 0 },
    { 0x8db5140d, 43 },
    { 0x69b5140e, 33 },
    { 0x73f5140e, 43 },
    { 0xb9f5140d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f751413, 90 },
    { 0x3e251414, 33 },
    { 0xc5051413, 76 },
    { 0x49451416, 90 },
    { 0xa4151417, 76 },
    { 0xcfe51418, 66 },
    { 0xa7851413, 66 },
    { 0x1515141a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f51420, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92751425, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c51428, 98 },
    { 0xc9951429, 95 },
    { 0, 0 },
    { 0x6eb5142b, 23 },
    { 0x56a5142c, 33 },
    { 0, 0 },
    { 0x2bb5142e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20351436, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a51439, 23 },
    { 0, 0 },
    { 0xa125143b, 76 },
    { 0x5435143c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60951440, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef51444, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f51449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f5144e, 51 },
    { 0xa415144e, 76 },
    { 0xfa751450, 99 },
    { 0xe1a5144f, 81 },
    { 0, 0 },
    { 0xfbc51453, 67 },
    { 0x68151454, 100 },
    { 0xd1e51455, 33 },
    { 0x35151456, 61 },
    { 0x37b51457, 33 },
    { 0xb3351458, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695145c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8785145f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59351462, 88 },
    { 0x95c51463, 43 },
    { 0x78851462, 23 },
    { 0xd0a51463, 34 },
    { 0xbc051462, 98 },
    { 0, 0 },
    { 0x91051468, 90 },
    { 0xf3b51469, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a251472, 99 },
    { 0, 0 },
    { 0xad151474, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb045147c, 88 },
    { 0x8f05147d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85251480, 90 },
    { 0x57851481, 95 },
    { 0, 0 },
    { 0x8a551483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d951487, 33 },
    { 0x75c51488, 23 },
    { 0x47751489, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef5148e, 68 },
    { 0, 0 },
    { 0xf6051490, 88 },
    { 0x25151490, 90 },
    { 0x8e551490, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab51495, 23 },
    { 0x49451496, 23 },
    { 0x78451497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee5149a, 90 },
    { 0x8f25149a, 93 },
    { 0x3075149a, 98 },
    { 0xb5b5149d, 33 },
    { 0x21c5149d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee514a3, 61 },
    { 0x482514a4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3514a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x302514ab, 52 },
    { 0x1a8514ac, 78 },
    { 0xd84514ab, 34 },
    { 0xa37514ae, 98 },
    { 0xd40514ad, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x162514b2, 55 },
    { 0x59d514b3, 90 },
    { 0x112514b3, 33 },
    { 0xda4514b5, 100 },
    { 0, 0 },
    { 0x999514b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac514ba, 61 },
    { 0, 0 },
    { 0xbb1514bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e514bf, 61 },
    { 0x431514bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf514c6, 36 },
    { 0x844514c7, 90 },
    { 0, 0 },
    { 0x778514c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8514d0, 98 },
    { 0x534514d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9514d5, 99 },
    { 0x52e514d6, 23 },
    { 0x7fb514d7, 77 },
    { 0xa80514d6, 93 },
    { 0xbd7514d9, 95 },
    { 0x77a514da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759514e4, 88 },
    { 0xe60514e5, 61 },
    { 0xef0514e6, 55 },
    { 0, 0 },
    { 0x9d2514e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a514f2, 87 },
    { 0, 0 },
    { 0x759514f4, 98 },
    { 0x6ed514f5, 61 },
    { 0x469514f5, 95 },
    { 0x76e514f5, 61 },
    { 0x204514f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8514fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f51504, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b95150a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5551516, 100 },
    { 0x1f451516, 95 },
    { 0x74a51518, 98 },
    { 0x73c51517, 95 },
    { 0x88a51518, 61 },
    { 0x22a5151a, 55 },
    { 0x285151c, 93 },
    { 0x6445151d, 23 },
    { 0, 0 },
    { 0xba35151f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5152c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd5152f, 99 },
    { 0x7e251530, 90 },
    { 0, 0 },
    { 0x84551532, 23 },
    { 0xaf51532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f51538, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4385153c, 61 },
    { 0xdb75153c, 61 },
    { 0xf555153c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f51543, 66 },
    { 0xb7e51544, 78 },
    { 0x9f151544, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b751549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd135154d, 93 },
    { 0, 0 },
    { 0xc9d5154f, 33 },
    { 0, 0 },
    { 0x90551551, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0751562, 90 },
    { 0, 0 },
    { 0xd3251564, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f51567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5156b, 84 },
    { 0xf145156c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f951573, 95 },
    { 0, 0 },
    { 0x42051575, 93 },
    { 0, 0 },
    { 0xee351577, 61 },
    { 0xdfe51578, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5157b, 100 },
    { 0x45e5157c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31751580, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98851584, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd51587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b051590, 76 },
    { 0, 0 },
    { 0xfef51592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada51596, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b15159e, 33 },
    { 0, 0 },
    { 0x8fc515a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2515a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f515a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca515a9, 91 },
    { 0x59a515aa, 100 },
    { 0x21e515ab, 33 },
    { 0, 0 },
    { 0xd10515ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c515b1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb515b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215515bc, 76 },
    { 0, 0 },
    { 0xb4f515be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd515c2, 33 },
    { 0xf9d515c2, 98 },
    { 0xd32515c4, 46 },
    { 0xbe0515c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6515ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69515d4, 33 },
    { 0xc75515d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305515e2, 95 },
    { 0, 0 },
    { 0x527515e4, 80 },
    { 0xb92515e5, 23 },
    { 0, 0 },
    { 0xb2a515e7, 91 },
    { 0, 0 },
    { 0x4ed515e9, 55 },
    { 0xc18515ea, 23 },
    { 0x78e515ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8515ee, 55 },
    { 0x20f515ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1515f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76851605, 95 },
    { 0, 0 },
    { 0xe9751607, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb95160e, 76 },
    { 0xb65160f, 55 },
    { 0x2a15160e, 23 },
    { 0, 0 },
    { 0x751612, 90 },
    { 0, 0 },
    { 0x7dc51614, 55 },
    { 0xeeb51615, 61 },
    { 0x7d151615, 85 },
    { 0xb0851617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6765161a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85161e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b151621, 98 },
    { 0xb9f51622, 33 },
    { 0x39b51623, 43 },
    { 0x87751621, 38 },
    { 0xe4c51625, 33 },
    { 0x53051621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b51630, 33 },
    { 0x32351631, 76 },
    { 0x39751631, 75 },
    { 0x35a51632, 52 },
    { 0x10351634, 77 },
    { 0xcb051635, 34 },
    { 0x59351635, 80 },
    { 0xc2651635, 61 },
    { 0x47051630, 33 },
    { 0xebd51630, 56 },
    { 0xdaa5163a, 90 },
    { 0, 0 },
    { 0xbf45163c, 98 },
    { 0x7ee5163d, 98 },
    { 0, 0 },
    { 0x68e5163f, 90 },
    { 0x3b65163f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d51645, 64 },
    { 0xc6251645, 91 },
    { 0x3ef51647, 100 },
    { 0x42c51648, 84 },
    { 0xcf551648, 100 },
    { 0, 0 },
    { 0x995164b, 98 },
    { 0, 0 },
    { 0xd5e5164d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72651650, 43 },
    { 0xe2b51651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51656, 52 },
    { 0x5c351657, 95 },
    { 0xad551658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535165b, 38 },
    { 0xeb55165c, 52 },
    { 0xc1a5165c, 85 },
    { 0, 0 },
    { 0x715165f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d51662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55751665, 23 },
    { 0x20051666, 23 },
    { 0, 0 },
    { 0xf5851668, 61 },
    { 0x5d651668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a5166c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb51675, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab751680, 90 },
    { 0xf0151681, 23 },
    { 0xead51682, 52 },
    { 0xa2c51680, 65 },
    { 0, 0 },
    { 0xad751685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5168b, 91 },
    { 0x2785168c, 76 },
    { 0, 0 },
    { 0xb315168e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41051695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1305169f, 76 },
    { 0x45e516a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc516a8, 91 },
    { 0x1e6516a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e516ae, 81 },
    { 0, 0 },
    { 0x4c2516b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0516b3, 95 },
    { 0x92516b4, 90 },
    { 0x7bd516b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f516bb, 95 },
    { 0, 0 },
    { 0x58c516bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243516c5, 96 },
    { 0xed2516c6, 90 },
    { 0x25d516c5, 43 },
    { 0x85c516c6, 90 },
    { 0xae4516c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f516cc, 90 },
    { 0, 0 },
    { 0x8a0516ce, 40 },
    { 0xce7516cf, 61 },
    { 0, 0 },
    { 0x2da516d1, 84 },
    { 0, 0 },
    { 0xb75516d3, 87 },
    { 0x32c516d4, 23 },
    { 0x88f516d5, 61 },
    { 0, 0 },
    { 0x9d6516d7, 55 },
    { 0x4b1516d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f516dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305516e7, 67 },
    { 0x609516e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3516ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd516f5, 99 },
    { 0xa98516f6, 52 },
    { 0, 0 },
    { 0x61e516f8, 23 },
    { 0xe9b516f8, 61 },
    { 0x913516fa, 33 },
    { 0x871516f9, 66 },
    { 0x4516fc, 46 },
    { 0xb85516fd, 85 },
    { 0, 0 },
    { 0x7fe516ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec51702, 55 },
    { 0, 0 },
    { 0x57f51704, 46 },
    { 0x98e51705, 95 },
    { 0, 0 },
    { 0x3da51707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe15170b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da5170f, 84 },
    { 0x6e95170f, 75 },
    { 0, 0 },
    { 0x4f251712, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e51715, 90 },
    { 0xa0151716, 78 },
    { 0x79251716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5171a, 98 },
    { 0x7c5171b, 90 },
    { 0xf25171a, 76 },
    { 0xc015171d, 66 },
    { 0x4cf5171e, 78 },
    { 0xa225171d, 33 },
    { 0x59d5171b, 90 },
    { 0xabc51720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff51724, 46 },
    { 0x52651725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d51728, 66 },
    { 0, 0 },
    { 0x77f5172a, 23 },
    { 0x9ff5172a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f51738, 76 },
    { 0, 0 },
    { 0xd5b5173a, 70 },
    { 0xe865173a, 76 },
    { 0x6855173b, 23 },
    { 0x6bf5173d, 100 },
    { 0xcfc5173e, 100 },
    { 0, 0 },
    { 0xc6d51740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15551747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5195174a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3751751, 78 },
    { 0x8e651752, 46 },
    { 0x3af51751, 52 },
    { 0xaba51754, 46 },
    { 0xe751754, 75 },
    { 0, 0 },
    { 0xd2351757, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5175c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb51760, 33 },
    { 0xf2d51761, 98 },
    { 0x12951762, 98 },
    { 0x22751762, 55 },
    { 0x7c451764, 77 },
    { 0x40951761, 88 },
    { 0x70651766, 87 },
    { 0xaec51766, 100 },
    { 0x7ac51763, 75 },
    { 0, 0 },
    { 0xb055176a, 23 },
    { 0x9625176b, 43 },
    { 0x27b5176c, 90 },
    { 0x5f85176d, 85 },
    { 0x2af5176e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6451771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea051774, 98 },
    { 0x22e51774, 98 },
    { 0x1aa51774, 95 },
    { 0x2d151776, 65 },
    { 0x50851778, 66 },
    { 0xdff51779, 98 },
    { 0, 0 },
    { 0x1065177b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805177f, 61 },
    { 0xc4151780, 98 },
    { 0x3a751781, 91 },
    { 0, 0 },
    { 0x1c451783, 98 },
    { 0x8c051783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f251789, 61 },
    { 0xa3a5178a, 90 },
    { 0, 0 },
    { 0xd365178c, 95 },
    { 0x3455178d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60951795, 90 },
    { 0x4d851796, 99 },
    { 0, 0 },
    { 0x15651798, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4205179b, 99 },
    { 0xfb45179c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94517a1, 85 },
    { 0xacd517a1, 67 },
    { 0xddb517a1, 33 },
    { 0x7d2517a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613517ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350517b0, 33 },
    { 0x9d8517b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209517b6, 55 },
    { 0, 0 },
    { 0x84f517b8, 61 },
    { 0x660517b9, 52 },
    { 0xffe517ba, 95 },
    { 0x2ad517bb, 61 },
    { 0, 0 },
    { 0x126517bd, 23 },
    { 0x3d4517be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94517c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481517c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec517ca, 55 },
    { 0, 0 },
    { 0x9bb517cc, 95 },
    { 0xb60517cc, 84 },
    { 0xc2e517ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x611517d1, 93 },
    { 0x507517d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7517d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4517dd, 61 },
    { 0xdea517de, 98 },
    { 0x256517de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206517e7, 98 },
    { 0x520517e8, 78 },
    { 0x2d517e9, 55 },
    { 0x5db517ea, 68 },
    { 0x2fa517e7, 33 },
    { 0xed3517ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811517f0, 90 },
    { 0, 0 },
    { 0x92e517f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705517f6, 33 },
    { 0xb88517f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a517fb, 23 },
    { 0x597517fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x116517ff, 100 },
    { 0xf9251800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b251803, 100 },
    { 0x47051803, 87 },
    { 0x2e51804, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad51809, 95 },
    { 0xebf5180a, 46 },
    { 0x9455180a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73751813, 34 },
    { 0, 0 },
    { 0xc4351815, 75 },
    { 0x9ed51816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45181b, 81 },
    { 0, 0 },
    { 0x78c5181d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5851820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5551823, 78 },
    { 0xf8951823, 66 },
    { 0x52151825, 88 },
    { 0xbe351823, 85 },
    { 0xc6c51827, 99 },
    { 0xad051826, 33 },
    { 0x50451824, 95 },
    { 0xea51827, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9605182d, 66 },
    { 0xc7e5182e, 43 },
    { 0x84c5182e, 98 },
    { 0x8b451830, 23 },
    { 0xa4d5182d, 90 },
    { 0, 0 },
    { 0x8cc51833, 90 },
    { 0, 0 },
    { 0x6c51835, 90 },
    { 0x1a851835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50151840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62151844, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6751848, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695184d, 33 },
    { 0xe145184d, 96 },
    { 0xb485184f, 33 },
    { 0, 0 },
    { 0x1c651851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c251854, 46 },
    { 0, 0 },
    { 0xe5951856, 43 },
    { 0xc5551857, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7735185c, 90 },
    { 0, 0 },
    { 0x7345185e, 90 },
    { 0xd7c5185f, 94 },
    { 0, 0 },
    { 0x91e51861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8151867, 61 },
    { 0x83b51868, 34 },
    { 0x3d51868, 95 },
    { 0x9ba5186a, 18 },
    { 0, 0 },
    { 0x435186c, 81 },
    { 0x2cd5186d, 34 },
    { 0, 0 },
    { 0x3ee5186f, 76 },
    { 0x37051870, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5151876, 46 },
    { 0, 0 },
    { 0xcd651878, 90 },
    { 0xefd51879, 90 },
    { 0x7295187a, 76 },
    { 0xd885187b, 81 },
    { 0x87f5187b, 90 },
    { 0xa8d5187a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85751880, 84 },
    { 0xc9351880, 99 },
    { 0xf7951881, 55 },
    { 0xe4d51883, 100 },
    { 0x44e51884, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef51889, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba75188c, 33 },
    { 0xac55188d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad51897, 78 },
    { 0x53551898, 46 },
    { 0xd8b51899, 61 },
    { 0, 0 },
    { 0xb215189b, 46 },
    { 0xdfc5189c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873518a3, 94 },
    { 0x82f518a3, 95 },
    { 0x198518a5, 95 },
    { 0x5b5518a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268518ab, 23 },
    { 0, 0 },
    { 0x1b8518ad, 84 },
    { 0xf17518ae, 78 },
    { 0xb61518ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73518b3, 95 },
    { 0xe7c518b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e518b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766518bb, 61 },
    { 0, 0 },
    { 0x71f518bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4518c7, 98 },
    { 0, 0 },
    { 0x9b6518c9, 33 },
    { 0, 0 },
    { 0x1af518cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0518d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x753518d6, 98 },
    { 0, 0 },
    { 0x5ba518d8, 75 },
    { 0, 0 },
    { 0x9b3518da, 17 },
    { 0xa0d518db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2518de, 23 },
    { 0, 0 },
    { 0xbb2518e0, 90 },
    { 0, 0 },
    { 0x2af518e2, 34 },
    { 0x1b9518e2, 38 },
    { 0, 0 },
    { 0xa9b518e5, 95 },
    { 0x43f518e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9518e9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f518ec, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235518f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181518fa, 68 },
    { 0, 0 },
    { 0xfdc518fc, 98 },
    { 0, 0 },
    { 0xd54518fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e51908, 98 },
    { 0x46351909, 88 },
    { 0, 0 },
    { 0x6045190b, 55 },
    { 0x4f5190c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3265190f, 81 },
    { 0x1435190f, 61 },
    { 0x46851911, 77 },
    { 0x9b35190f, 90 },
    { 0x49451913, 98 },
    { 0x1a951912, 75 },
    { 0xbee51915, 90 },
    { 0x4d451915, 23 },
    { 0xe6351917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25191d, 55 },
    { 0x8895191e, 87 },
    { 0, 0 },
    { 0x13751920, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf951923, 23 },
    { 0x11951923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185192a, 23 },
    { 0x70a5192b, 90 },
    { 0x3c15192c, 85 },
    { 0x4d35192b, 98 },
    { 0x7745192b, 33 },
    { 0x1f65192f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd351933, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1351937, 34 },
    { 0x7bc51938, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05193f, 33 },
    { 0xb0f51940, 90 },
    { 0xc9251941, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7251946, 34 },
    { 0x94c51946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6555194a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea5194f, 34 },
    { 0, 0 },
    { 0x3c851951, 95 },
    { 0x3d751952, 91 },
    { 0x26c51951, 55 },
    { 0x14351952, 34 },
    { 0x48e51951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d51959, 93 },
    { 0xe5951959, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb885195d, 91 },
    { 0x4db5195d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc51964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99951967, 95 },
    { 0xbc151968, 61 },
    { 0x54551969, 33 },
    { 0x8455196a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5196d, 90 },
    { 0x2255196d, 95 },
    { 0xa085196f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3451976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e5197c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5198a, 38 },
    { 0, 0 },
    { 0xc605198c, 90 },
    { 0x3005198d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69351992, 98 },
    { 0x50d51993, 98 },
    { 0xc9b51994, 33 },
    { 0x3aa51995, 90 },
    { 0xd9e51996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd45199f, 33 },
    { 0xb2e519a0, 93 },
    { 0x92519a1, 34 },
    { 0xc54519a1, 43 },
    { 0x375519a1, 75 },
    { 0, 0 },
    { 0xeb2519a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8519ac, 87 },
    { 0x39f519ac, 98 },
    { 0x216519ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x418519b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd519b8, 33 },
    { 0, 0 },
    { 0x81f519ba, 98 },
    { 0, 0 },
    { 0x784519bc, 90 },
    { 0x209519bd, 100 },
    { 0, 0 },
    { 0x16f519bf, 33 },
    { 0x858519bf, 98 },
    { 0xf8d519bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95519c4, 100 },
    { 0x58c519c4, 34 },
    { 0xe9a519c5, 67 },
    { 0, 0 },
    { 0x38f519c8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9519cf, 23 },
    { 0x752519d0, 66 },
    { 0x900519d1, 89 },
    { 0xa38519d2, 23 },
    { 0xbaa519d0, 95 },
    { 0x3a6519d4, 95 },
    { 0xb0f519d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2519dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e519e0, 98 },
    { 0x17b519e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a519e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49519ed, 88 },
    { 0, 0 },
    { 0xe8d519ef, 33 },
    { 0, 0 },
    { 0x5a519f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9519f4, 98 },
    { 0x325519f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83519f8, 65 },
    { 0x923519f8, 23 },
    { 0x4fb519f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c519fd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9751a01, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc51a06, 91 },
    { 0x48151a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67751a0b, 78 },
    { 0xf5951a0b, 76 },
    { 0x72a51a0b, 95 },
    { 0, 0 },
    { 0xc51a0f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8951a15, 33 },
    { 0xfe851a15, 90 },
    { 0x4a051a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb51a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f451a1d, 98 },
    { 0x76451a1e, 23 },
    { 0x46851a1e, 78 },
    { 0x92851a20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec651a24, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a851a29, 46 },
    { 0xbec51a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a851a30, 76 },
    { 0x3d251a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12551a35, 46 },
    { 0, 0 },
    { 0x1b451a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca51a3e, 83 },
    { 0xaf651a3f, 90 },
    { 0x5f351a3f, 87 },
    { 0, 0 },
    { 0x5e851a42, 95 },
    { 0, 0 },
    { 0xae851a44, 98 },
    { 0xdf651a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f51a48, 61 },
    { 0x2a951a49, 93 },
    { 0x93d51a48, 33 },
    { 0, 0 },
    { 0xa4651a4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf51a50, 61 },
    { 0, 0 },
    { 0x76651a52, 33 },
    { 0x1d751a53, 95 },
    { 0x85551a53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f51a57, 100 },
    { 0x5ff51a57, 33 },
    { 0, 0 },
    { 0xbee51a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0951a5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb851a63, 61 },
    { 0x5551a64, 76 },
    { 0xeac51a65, 85 },
    { 0x2d51a66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b751a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b51a71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa251a77, 77 },
    { 0, 0 },
    { 0xd8851a79, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d051a7d, 98 },
    { 0xf4051a7e, 90 },
    { 0x9de51a7e, 23 },
    { 0xdc451a80, 76 },
    { 0xad251a81, 87 },
    { 0x73c51a82, 67 },
    { 0xdc451a83, 98 },
    { 0xfa151a84, 91 },
    { 0xea551a84, 6 },
    { 0x69e51a7d, 61 },
    { 0xdef51a87, 43 },
    { 0x86d51a88, 43 },
    { 0x18351a88, 33 },
    { 0x22551a86, 55 },
    { 0, 0 },
    { 0x20551a8c, 90 },
    { 0, 0 },
    { 0x91251a8e, 90 },
    { 0xaf851a8e, 78 },
    { 0x62e51a90, 46 },
    { 0x83d51a8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f51a94, 61 },
    { 0x74451a95, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e51a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e851a9d, 23 },
    { 0xee851a9e, 95 },
    { 0x18151a9f, 77 },
    { 0, 0 },
    { 0x70851aa1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33051aa4, 52 },
    { 0, 0 },
    { 0xf5f51aa6, 66 },
    { 0, 0 },
    { 0x13c51aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c51aab, 76 },
    { 0x7ee51aab, 55 },
    { 0x14551aad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e51ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b351ab8, 61 },
    { 0x1bd51ab9, 78 },
    { 0x78251ab9, 91 },
    { 0x5e751ab8, 33 },
    { 0x37951abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d51abf, 90 },
    { 0x93951ac0, 34 },
    { 0, 0 },
    { 0x53a51ac2, 33 },
    { 0xcfd51ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b51ac8, 76 },
    { 0xed51ac9, 76 },
    { 0x72151aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a51ace, 38 },
    { 0x45c51acf, 90 },
    { 0xab851acf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda51ad5, 48 },
    { 0, 0 },
    { 0x22051ad7, 55 },
    { 0x4d51ad7, 76 },
    { 0, 0 },
    { 0x3e951ada, 61 },
    { 0xd0151adb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be51ade, 98 },
    { 0, 0 },
    { 0x5d051ae0, 76 },
    { 0xbdc51ae0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc51ae5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65951aea, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe051aee, 76 },
    { 0x9c351aee, 95 },
    { 0, 0 },
    { 0x8d451af1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f51af5, 98 },
    { 0, 0 },
    { 0xf5751af7, 76 },
    { 0xfa651af7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a851afd, 85 },
    { 0xd8e51afe, 67 },
    { 0x83051aff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f51b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45051b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea551b0a, 90 },
    { 0, 0 },
    { 0x8b51b0c, 23 },
    { 0x69751b0d, 99 },
    { 0xbf351b0e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51b11, 33 },
    { 0x8c251b12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e51b16, 61 },
    { 0x3e351b17, 66 },
    { 0, 0 },
    { 0xac451b19, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68651b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f51b20, 93 },
    { 0xfbc51b21, 98 },
    { 0xb6051b22, 90 },
    { 0, 0 },
    { 0x14e51b24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92151b2a, 34 },
    { 0x65b51b2a, 95 },
    { 0xf3b51b2c, 38 },
    { 0x79e51b2d, 61 },
    { 0x77051b2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c51b3a, 33 },
    { 0x20651b3b, 33 },
    { 0x68651b3c, 98 },
    { 0x82051b3d, 90 },
    { 0xabc51b3e, 23 },
    { 0x3fe51b3e, 99 },
    { 0xfb451b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e51b48, 95 },
    { 0xecf51b49, 46 },
    { 0x11e51b4a, 80 },
    { 0xa5551b4b, 89 },
    { 0xc4651b48, 95 },
    { 0x19851b4d, 61 },
    { 0x3ea51b4d, 95 },
    { 0xd6451b4f, 33 },
    { 0x11951b4c, 55 },
    { 0, 0 },
    { 0x9051b52, 98 },
    { 0, 0 },
    { 0x92651b54, 46 },
    { 0x3bd51b55, 66 },
    { 0x80551b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27951b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5651b60, 52 },
    { 0x53251b61, 33 },
    { 0x5f751b62, 33 },
    { 0x42c51b60, 87 },
    { 0xaaa51b63, 33 },
    { 0x1a851b65, 76 },
    { 0, 0 },
    { 0x7fd51b67, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b51b6b, 23 },
    { 0xb8151b6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16451b6f, 55 },
    { 0, 0 },
    { 0xde251b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a251b75, 33 },
    { 0x33e51b76, 17 },
    { 0, 0 },
    { 0xb3c51b78, 100 },
    { 0xc2051b79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24151b7e, 61 },
    { 0, 0 },
    { 0xe5e51b80, 98 },
    { 0x8ef51b81, 43 },
    { 0xe3551b82, 23 },
    { 0xf7c51b83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab751b88, 33 },
    { 0, 0 },
    { 0xe2651b8a, 100 },
    { 0xeca51b8b, 61 },
    { 0x91351b8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f51b8f, 61 },
    { 0, 0 },
    { 0x14151b91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d51b94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac51b9a, 95 },
    { 0xd8a51b9b, 95 },
    { 0x20e51b9b, 90 },
    { 0x8651b9b, 90 },
    { 0x86151b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18151ba1, 33 },
    { 0x96151ba2, 33 },
    { 0xcf51ba3, 33 },
    { 0, 0 },
    { 0xf5a51ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b051ba8, 33 },
    { 0, 0 },
    { 0x24751baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75951bad, 33 },
    { 0x6a851bae, 33 },
    { 0x39551baf, 95 },
    { 0x2e751bb0, 76 },
    { 0xcd451bb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c51bb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68251bc1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851bc9, 61 },
    { 0xe1651bca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17151bcf, 90 },
    { 0xb1b51bcf, 33 },
    { 0, 0 },
    { 0x9ea51bd2, 95 },
    { 0, 0 },
    { 0x5e251bd4, 47 },
    { 0x90e51bd5, 40 },
    { 0x5ab51bd4, 61 },
    { 0xb651bd7, 91 },
    { 0x28151bd8, 100 },
    { 0xb5e51bd9, 61 },
    { 0xd9e51bd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e51be3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b51be6, 55 },
    { 0x94c51be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca751bec, 34 },
    { 0, 0 },
    { 0x12151bee, 91 },
    { 0x1cc51bef, 23 },
    { 0x63c51bf0, 52 },
    { 0x38451bef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa151bf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf51bfb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0751bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7551c05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37351c09, 40 },
    { 0, 0 },
    { 0xe1f51c0b, 90 },
    { 0xbc651c0b, 61 },
    { 0, 0 },
    { 0x4ce51c0e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46951c14, 78 },
    { 0x52d51c15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe51c18, 55 },
    { 0x88451c19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x22651c1c, 77 },
    { 0x8b451c1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51c25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1851c28, 34 },
    { 0xca151c28, 76 },
    { 0x2e851c28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66551c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa251c3b, 87 },
    { 0x33451c3c, 61 },
    { 0xc7f51c3c, 23 },
    { 0x77c51c3b, 61 },
    { 0x2bf51c3f, 95 },
    { 0x89751c3d, 33 },
    { 0x7f751c41, 85 },
    { 0xc7551c42, 40 },
    { 0xacc51c3c, 33 },
    { 0x21d51c44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e51c49, 99 },
    { 0, 0 },
    { 0x28d51c4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64251c51, 43 },
    { 0x7dc51c51, 98 },
    { 0xd1751c52, 85 },
    { 0x54f51c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3651c57, 23 },
    { 0, 0 },
    { 0x3c451c59, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab051c5d, 43 },
    { 0, 0 },
    { 0x40851c5f, 34 },
    { 0, 0 },
    { 0xc1e51c61, 92 },
    { 0x4a451c62, 55 },
    { 0, 0 },
    { 0x16851c64, 99 },
    { 0xe6c51c65, 87 },
    { 0xdaa51c66, 52 },
    { 0x84451c67, 76 },
    { 0x7851c65, 76 },
    { 0x38e51c67, 43 },
    { 0x21751c6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6251c71, 90 },
    { 0xa1951c72, 61 },
    { 0xc4651c72, 90 },
    { 0x7d451c74, 76 },
    { 0x5fb51c75, 83 },
    { 0, 0 },
    { 0x7e351c77, 34 },
    { 0, 0 },
    { 0x62751c79, 95 },
    { 0xff51c7a, 34 },
    { 0xfad51c7b, 98 },
    { 0xbe051c7c, 43 },
    { 0xa3251c7d, 55 },
    { 0x5d251c7a, 95 },
    { 0xff251c7f, 90 },
    { 0x40551c7f, 80 },
    { 0xa9e51c7d, 55 },
    { 0xdcf51c82, 33 },
    { 0x8b351c83, 55 },
    { 0xd1c51c82, 33 },
    { 0x96e51c83, 55 },
    { 0x9f551c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a051c89, 98 },
    { 0x7a251c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d551c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54351c9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac651ca6, 61 },
    { 0x52451ca7, 87 },
    { 0xb1051ca6, 90 },
    { 0x57851ca8, 95 },
    { 0xda651ca8, 66 },
    { 0xa8e51ca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73251cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa51cb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c51cb9, 23 },
    { 0x81651cba, 61 },
    { 0x43351cbb, 95 },
    { 0, 0 },
    { 0x64351cbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1351cc1, 61 },
    { 0x20551cc2, 34 },
    { 0x5951cc3, 55 },
    { 0, 0 },
    { 0x59d51cc5, 99 },
    { 0x51151cc6, 90 },
    { 0, 0 },
    { 0x68751cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c51ccc, 99 },
    { 0x91b51ccd, 23 },
    { 0x19b51ccc, 34 },
    { 0, 0 },
    { 0xb0951cd0, 55 },
    { 0xbe351cd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99351cd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe851cdd, 95 },
    { 0xb8f51cde, 87 },
    { 0x68351cdf, 33 },
    { 0xcc151cdd, 66 },
    { 0x7de51ce1, 33 },
    { 0x3ee51cdd, 96 },
    { 0xa3a51ce3, 87 },
    { 0xb8451ce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51ce7, 98 },
    { 0x17051ce8, 66 },
    { 0xbe151ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb851cec, 55 },
    { 0, 0 },
    { 0x5e451cee, 33 },
    { 0x64751cef, 95 },
    { 0x54451cf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11151cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2751cfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c51cff, 46 },
    { 0, 0 },
    { 0xe4f51d01, 98 },
    { 0x6cf51d02, 90 },
    { 0xc7f51d03, 81 },
    { 0x3f151d03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4651d08, 69 },
    { 0x14551d09, 81 },
    { 0x48951d08, 98 },
    { 0x67a51d0a, 61 },
    { 0x87751d0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b351d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4651d16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x36951d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed51d1e, 81 },
    { 0x3eb51d1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d051d27, 61 },
    { 0, 0 },
    { 0xae451d29, 23 },
    { 0x21051d2a, 43 },
    { 0x48e51d29, 87 },
    { 0x3d151d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f51d2f, 23 },
    { 0, 0 },
    { 0x5e451d31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5751d36, 78 },
    { 0xa6051d37, 76 },
    { 0, 0 },
    { 0xa9751d39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44051d41, 76 },
    { 0x9d851d41, 75 },
    { 0, 0 },
    { 0xc6a51d44, 51 },
    { 0x7d51d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55051d49, 67 },
    { 0x40b51d49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca951d4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c51d50, 76 },
    { 0xb5651d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21551d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc51d5f, 55 },
    { 0, 0 },
    { 0x7e151d61, 61 },
    { 0x5e751d61, 90 },
    { 0x47b51d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851d6c, 33 },
    { 0xa6a51d6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c51d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c51d78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74051d7b, 34 },
    { 0x16d51d7c, 61 },
    { 0xede51d7d, 91 },
    { 0, 0 },
    { 0x82b51d7f, 23 },
    { 0, 0 },
    { 0xe7f51d81, 95 },
    { 0, 0 },
    { 0x4651d83, 88 },
    { 0, 0 },
    { 0xa2851d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba851d8a, 95 },
    { 0x1f151d8b, 85 },
    { 0xa9151d8c, 52 },
    { 0x6e551d8c, 23 },
    { 0xf1251d8e, 23 },
    { 0x1b651d8a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e651d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd51d9f, 66 },
    { 0, 0 },
    { 0x49c51da1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7451da4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab51da7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b251daa, 61 },
    { 0x44b51dab, 23 },
    { 0x6b851dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a51db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81151db9, 61 },
    { 0xfee51dba, 33 },
    { 0x7a851dba, 98 },
    { 0x6f051db9, 98 },
    { 0x70951dba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b351dc5, 34 },
    { 0xf51dc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec51dce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9051dd4, 34 },
    { 0x75b51dd5, 61 },
    { 0x43351dd6, 46 },
    { 0xa1c51dd7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0451de0, 100 },
    { 0xc9851de1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5551de7, 55 },
    { 0x3a651de8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90151dec, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d351def, 93 },
    { 0x2d751df0, 90 },
    { 0x68451df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7651df4, 23 },
    { 0, 0 },
    { 0x7151df6, 95 },
    { 0, 0 },
    { 0x15051df8, 66 },
    { 0x69851df9, 99 },
    { 0xef351df9, 87 },
    { 0x7ab51dfb, 61 },
    { 0, 0 },
    { 0xfe551dfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7651e00, 93 },
    { 0, 0 },
    { 0xcbf51e02, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf251e09, 98 },
    { 0xb6c51e0a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e51e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e751e1a, 69 },
    { 0, 0 },
    { 0x6e251e1c, 46 },
    { 0, 0 },
    { 0x60551e1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4151e22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec51e26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5751e2e, 23 },
    { 0, 0 },
    { 0x8eb51e30, 23 },
    { 0x8f151e31, 55 },
    { 0xde351e32, 88 },
    { 0xfb951e31, 43 },
    { 0xd7851e34, 95 },
    { 0xcd251e33, 100 },
    { 0, 0 },
    { 0xb7751e37, 23 },
    { 0x8ea51e38, 90 },
    { 0xc6251e39, 61 },
    { 0x8a851e39, 90 },
    { 0x70651e3a, 23 },
    { 0x72151e3b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0851e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38251e4c, 90 },
    { 0x35e51e4c, 100 },
    { 0, 0 },
    { 0xa0a51e4f, 78 },
    { 0, 0 },
    { 0xc0551e51, 90 },
    { 0x1b851e51, 61 },
    { 0x9e151e53, 61 },
    { 0, 0 },
    { 0xe2151e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7451e59, 61 },
    { 0x63f51e5a, 95 },
    { 0xb151e5a, 84 },
    { 0, 0 },
    { 0x91f51e5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2251e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55151e66, 43 },
    { 0x49e51e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94951e6c, 90 },
    { 0, 0 },
    { 0x6c351e6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc451e71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19951e75, 17 },
    { 0xfeb51e76, 33 },
    { 0x51d51e77, 76 },
    { 0xd1b51e78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b51e7b, 95 },
    { 0xd0151e7b, 33 },
    { 0, 0 },
    { 0xfe851e7e, 33 },
    { 0, 0 },
    { 0xe8f51e80, 33 },
    { 0xd0d51e80, 43 },
    { 0, 0 },
    { 0x37d51e83, 23 },
    { 0x2a451e84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57151e8a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb451e99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7251e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93751ea5, 95 },
    { 0x19051ea6, 68 },
    { 0xba151ea5, 55 },
    { 0x7f751ea8, 61 },
    { 0x6751ea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10451eb2, 78 },
    { 0x9ab51eb3, 61 },
    { 0x2ed51eb4, 91 },
    { 0, 0 },
    { 0x29351eb6, 61 },
    { 0, 0 },
    { 0xad551eb8, 46 },
    { 0x27f51eb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba51ec1, 95 },
    { 0, 0 },
    { 0xce651ec3, 90 },
    { 0, 0 },
    { 0x851ec5, 33 },
    { 0x24c51ec6, 66 },
    { 0x7e51ec7, 98 },
    { 0x6bf51ec7, 90 },
    { 0x7e651ec9, 95 },
    { 0x6b751eca, 98 },
    { 0xb5751ec9, 90 },
    { 0, 0 },
    { 0x70551ecd, 33 },
    { 0x3b851ece, 46 },
    { 0x55351ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc751ed4, 90 },
    { 0x87551ed4, 33 },
    { 0, 0 },
    { 0xd2e51ed7, 78 },
    { 0xbcf51ed8, 90 },
    { 0x21651ed9, 95 },
    { 0x7c251ed9, 43 },
    { 0x77051ed9, 36 },
    { 0x8b051edc, 98 },
    { 0xd1251edd, 55 },
    { 0x97851ede, 61 },
    { 0, 0 },
    { 0x24051ee0, 23 },
    { 0, 0 },
    { 0x19c51ee2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec51ee5, 61 },
    { 0x9b051ee6, 90 },
    { 0x50451ee7, 17 },
    { 0xa4351ee8, 98 },
    { 0, 0 },
    { 0x84351eea, 55 },
    { 0x59f51eeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14551eef, 23 },
    { 0xd7151ef0, 98 },
    { 0xa5d51ef0, 43 },
    { 0x4cf51ef2, 52 },
    { 0xb251ef3, 55 },
    { 0xc5751eef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89551ef9, 33 },
    { 0x89c51efa, 33 },
    { 0xbd751efb, 66 },
    { 0xd9d51efc, 23 },
    { 0x56a51efd, 52 },
    { 0xbe751efd, 23 },
    { 0xc951eff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd51f06, 23 },
    { 0, 0 },
    { 0xb1851f08, 90 },
    { 0, 0 },
    { 0x9ef51f0a, 88 },
    { 0xe7e51f0b, 69 },
    { 0x9651f0b, 90 },
    { 0x6b251f0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b51f10, 66 },
    { 0xdfe51f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e351f14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e751f17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac51f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851f1e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51f22, 93 },
    { 0xfb451f22, 23 },
    { 0x89551f24, 61 },
    { 0x6e51f25, 90 },
    { 0x5fe51f25, 33 },
    { 0x32651f26, 33 },
    { 0, 0 },
    { 0x18851f29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90151f2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5551f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0751f3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e51f42, 61 },
    { 0, 0 },
    { 0x69151f44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50551f4a, 33 },
    { 0, 0 },
    { 0x54a51f4c, 61 },
    { 0, 0 },
    { 0x7a551f4e, 33 },
    { 0x6f51f4e, 52 },
    { 0x9c251f50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0051f58, 23 },
    { 0, 0 },
    { 0x5df51f5a, 33 },
    { 0xf0351f5b, 33 },
    { 0xb3f51f5b, 33 },
    { 0x71151f5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb451f60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e51f63, 76 },
    { 0x55951f64, 85 },
    { 0x27b51f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c51f68, 33 },
    { 0xb7851f68, 55 },
    { 0x17e51f6a, 61 },
    { 0x93151f6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b751f70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4551f74, 34 },
    { 0xa4851f75, 43 },
    { 0, 0 },
    { 0x49e51f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d51f7e, 77 },
    { 0x9a551f7f, 33 },
    { 0x69f51f7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35451f84, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a51f89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0951f8f, 33 },
    { 0, 0 },
    { 0x44f51f91, 90 },
    { 0x8e651f92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8051f9e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25851fa2, 23 },
    { 0x2f951fa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c851faa, 61 },
    { 0x33351faa, 33 },
    { 0, 0 },
    { 0xcc751fad, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a051fb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f151fbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6351fbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5051fc2, 98 },
    { 0, 0 },
    { 0xe8751fc4, 43 },
    { 0, 0 },
    { 0x20d51fc6, 77 },
    { 0, 0 },
    { 0x8e451fc8, 99 },
    { 0x1b551fc9, 93 },
    { 0x86151fc8, 23 },
    { 0, 0 },
    { 0xc3351fcc, 61 },
    { 0, 0 },
    { 0xcf451fce, 55 },
    { 0x97751fcf, 33 },
    { 0, 0 },
    { 0xb0e51fd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da51fd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48451fd7, 98 },
    { 0, 0 },
    { 0xb1f51fd9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xce651fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49951fdf, 85 },
    { 0, 0 },
    { 0xf5151fe1, 61 },
    { 0x16f51fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd751fe5, 23 },
    { 0x4b051fe5, 90 },
    { 0xa1651fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96651fea, 90 },
    { 0x6e251feb, 89 },
    { 0x6c751fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57851ff0, 33 },
    { 0x8a651ff1, 75 },
    { 0, 0 },
    { 0x72c51ff3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95051ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd651ff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8851ffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf52000, 43 },
    { 0x35452001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b52004, 55 },
    { 0xb5152005, 90 },
    { 0x6f552005, 95 },
    { 0x3152005, 84 },
    { 0xc8452008, 90 },
    { 0xdff52009, 98 },
    { 0xfd65200a, 97 },
    { 0x9d05200b, 90 },
    { 0xb875200c, 80 },
    { 0x6ce5200b, 95 },
    { 0x6595200d, 91 },
    { 0, 0 },
    { 0x66952010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a52015, 99 },
    { 0xbb352016, 52 },
    { 0x3e52015, 90 },
    { 0x2dc52018, 93 },
    { 0xa8052017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4015201d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe52020, 95 },
    { 0xd6d52021, 33 },
    { 0x4ac52022, 61 },
    { 0xbdd52023, 84 },
    { 0x13552022, 95 },
    { 0, 0 },
    { 0xea752026, 66 },
    { 0xa9e52026, 61 },
    { 0xa8d52028, 33 },
    { 0xd8052028, 61 },
    { 0x70b52029, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f55202e, 34 },
    { 0x1b75202f, 87 },
    { 0xba552030, 38 },
    { 0x935202f, 93 },
    { 0x14f5202e, 90 },
    { 0xc9b52032, 61 },
    { 0x8d752034, 40 },
    { 0xcf652035, 61 },
    { 0xa9352036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5203b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6615203e, 61 },
    { 0xdfe5203e, 84 },
    { 0xf7452040, 95 },
    { 0x69952040, 41 },
    { 0x5dd52042, 55 },
    { 0xf5752043, 52 },
    { 0x91a52043, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d5204f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53352056, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5152059, 100 },
    { 0x6135205a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74852060, 61 },
    { 0x1ab52060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe752067, 78 },
    { 0x29e52068, 23 },
    { 0x7fa52067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605206e, 61 },
    { 0x9215206f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a52073, 67 },
    { 0, 0 },
    { 0x8b952075, 43 },
    { 0x20c52076, 76 },
    { 0x6a952075, 97 },
    { 0x28f52077, 34 },
    { 0x37852079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea52082, 87 },
    { 0xe5a52083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67052086, 61 },
    { 0x27952087, 61 },
    { 0xd3a52088, 23 },
    { 0xa9c52089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb852091, 46 },
    { 0x2b352091, 95 },
    { 0xdcc52093, 33 },
    { 0xcc152094, 90 },
    { 0xdc752095, 95 },
    { 0xa5252091, 61 },
    { 0, 0 },
    { 0x9b752098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d45209d, 90 },
    { 0xdc55209e, 33 },
    { 0x4565209d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6520a3, 34 },
    { 0x5a3520a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1520a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x911520ab, 78 },
    { 0xb17520ac, 95 },
    { 0x8f6520ad, 65 },
    { 0x21c520ae, 23 },
    { 0xf58520ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6520b2, 98 },
    { 0, 0 },
    { 0x63520b4, 98 },
    { 0xd7520b4, 76 },
    { 0xb95520b6, 98 },
    { 0xef4520b7, 91 },
    { 0x4b520b4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13520bc, 76 },
    { 0x3c3520bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657520c1, 83 },
    { 0xe4e520c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83520c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571520cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e520d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x539520d7, 61 },
    { 0x66b520d7, 93 },
    { 0xfc6520d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d520e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f520e5, 85 },
    { 0, 0 },
    { 0xa3a520e7, 76 },
    { 0xb66520e8, 23 },
    { 0x261520e7, 95 },
    { 0xbfa520ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e520ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f520f4, 46 },
    { 0x90520f5, 66 },
    { 0x36f520f6, 95 },
    { 0xf90520f5, 76 },
    { 0, 0 },
    { 0x1ef520f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c520fd, 33 },
    { 0x45e520fd, 52 },
    { 0xdd5520fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4152102, 99 },
    { 0x7052103, 23 },
    { 0x20c52103, 90 },
    { 0x96452105, 93 },
    { 0x56c52106, 84 },
    { 0x10a52107, 76 },
    { 0x3e052102, 95 },
    { 0xaa352109, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5210c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5252113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61452117, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5211a, 66 },
    { 0xb1c5211a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2195211f, 23 },
    { 0xc6052120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d52126, 55 },
    { 0x67152127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e55212a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe895212d, 95 },
    { 0x4a05212e, 85 },
    { 0xc0e5212e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc552134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952139, 43 },
    { 0, 0 },
    { 0x3fa5213b, 55 },
    { 0, 0 },
    { 0x1b55213d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb52140, 95 },
    { 0x83452141, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f52146, 33 },
    { 0xa1452147, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8115214a, 43 },
    { 0x4a05214b, 87 },
    { 0xf1e5214c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd52150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68252153, 51 },
    { 0, 0 },
    { 0x7452155, 99 },
    { 0x4d052155, 23 },
    { 0xfa952157, 84 },
    { 0, 0 },
    { 0x7652159, 33 },
    { 0, 0 },
    { 0x7615215b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73352160, 34 },
    { 0xc7652161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d52164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e52167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435216c, 98 },
    { 0x7a75216c, 33 },
    { 0x2d5216e, 90 },
    { 0x6635216f, 33 },
    { 0x67752170, 34 },
    { 0x63d52171, 99 },
    { 0x2b55216f, 61 },
    { 0x3ee52173, 33 },
    { 0xfd752174, 90 },
    { 0x8a52173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195217f, 61 },
    { 0x84352180, 90 },
    { 0, 0 },
    { 0x53352182, 23 },
    { 0x8b452183, 33 },
    { 0xa1152182, 33 },
    { 0x54b52183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc052188, 55 },
    { 0, 0 },
    { 0x57f5218a, 93 },
    { 0x48d5218b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9365218e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa5219f, 90 },
    { 0, 0 },
    { 0xc3c521a1, 23 },
    { 0, 0 },
    { 0x7e7521a3, 76 },
    { 0xb04521a4, 90 },
    { 0x4e7521a4, 99 },
    { 0, 0 },
    { 0x621521a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a521ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x175521ae, 55 },
    { 0x8e1521ae, 95 },
    { 0xb0f521af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af521b9, 61 },
    { 0, 0 },
    { 0x682521bb, 64 },
    { 0, 0 },
    { 0xd2a521bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9521c2, 33 },
    { 0xb42521c2, 98 },
    { 0x740521c4, 98 },
    { 0x177521c5, 90 },
    { 0xda7521c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3521cd, 61 },
    { 0xc53521cd, 34 },
    { 0, 0 },
    { 0x7c2521d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea521d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4521d9, 61 },
    { 0xf2521da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12521e2, 64 },
    { 0xc48521e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28521e6, 61 },
    { 0xa47521e7, 23 },
    { 0, 0 },
    { 0xc8f521e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14521ec, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7521f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6521f4, 96 },
    { 0x541521f5, 98 },
    { 0x54e521f5, 55 },
    { 0xe45521f7, 90 },
    { 0xc8d521f8, 33 },
    { 0xab2521f5, 100 },
    { 0xa0c521fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca52201, 95 },
    { 0x17452202, 33 },
    { 0x1f52203, 33 },
    { 0x4f252202, 23 },
    { 0x60452205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed852209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc935220e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa052212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305221c, 83 },
    { 0xfd15221d, 93 },
    { 0xad25221e, 61 },
    { 0x2f45221e, 33 },
    { 0x62152220, 55 },
    { 0, 0 },
    { 0xe7e52222, 90 },
    { 0, 0 },
    { 0xc1052224, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f52228, 95 },
    { 0x5b852228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d5222c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe52235, 55 },
    { 0xa2e52235, 33 },
    { 0x26652237, 91 },
    { 0xddc52236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf65223d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e52246, 95 },
    { 0x54452247, 98 },
    { 0, 0 },
    { 0x63852249, 68 },
    { 0x74f52249, 90 },
    { 0xde5224b, 61 },
    { 0xb005224c, 61 },
    { 0x1ca5224b, 33 },
    { 0xa8f5224d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d52252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d52255, 17 },
    { 0x18b52256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa465225d, 66 },
    { 0x6435225d, 90 },
    { 0xf4b5225f, 93 },
    { 0x65952260, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f5226d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf465227b, 33 },
    { 0, 0 },
    { 0x1a5227d, 90 },
    { 0xd8a5227e, 53 },
    { 0x435227f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7652282, 55 },
    { 0, 0 },
    { 0x5c852284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d65228a, 84 },
    { 0x5cd5228b, 98 },
    { 0xa0a5228c, 76 },
    { 0xeee5228d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xac852290, 52 },
    { 0x5b752290, 100 },
    { 0, 0 },
    { 0x40952293, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98552296, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa235229c, 95 },
    { 0, 0 },
    { 0x1e45229e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68522a1, 34 },
    { 0xa5c522a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b522a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7522ad, 90 },
    { 0, 0 },
    { 0xa1a522af, 33 },
    { 0, 0 },
    { 0xeab522b1, 33 },
    { 0, 0 },
    { 0x733522b3, 65 },
    { 0xe4b522b3, 75 },
    { 0x39f522b5, 91 },
    { 0, 0 },
    { 0xf02522b7, 61 },
    { 0xc8a522b7, 95 },
    { 0xba2522b9, 95 },
    { 0x2cf522b7, 90 },
    { 0x852522b7, 84 },
    { 0xea522bc, 46 },
    { 0x651522bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283522c1, 46 },
    { 0x4b2522c1, 90 },
    { 0, 0 },
    { 0x420522c4, 33 },
    { 0, 0 },
    { 0xfbe522c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a522cc, 61 },
    { 0, 0 },
    { 0xad3522ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x401522d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03522d5, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db522da, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d522dd, 76 },
    { 0x1ee522de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912522e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a522e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce522ea, 90 },
    { 0xc0a522eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda522ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57522f1, 87 },
    { 0x9f5522f1, 52 },
    { 0x58b522f3, 90 },
    { 0x17d522f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c522f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f522fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf05230c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca352314, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c52319, 78 },
    { 0, 0 },
    { 0x8c45231b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35231e, 93 },
    { 0xb6f5231f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc652323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e5232c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde05232f, 98 },
    { 0xd1152330, 55 },
    { 0, 0 },
    { 0x25d52332, 96 },
    { 0, 0 },
    { 0x65652334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6252338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f5233b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb652340, 69 },
    { 0, 0 },
    { 0xff852342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd352345, 76 },
    { 0, 0 },
    { 0xc1652347, 90 },
    { 0xac952347, 33 },
    { 0x21f52349, 33 },
    { 0x50a52349, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c5234d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4352353, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4652357, 90 },
    { 0, 0 },
    { 0x5f152359, 90 },
    { 0x89552359, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6135235d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9452363, 61 },
    { 0, 0 },
    { 0x2c152365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae5236f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac52377, 65 },
    { 0x18152378, 88 },
    { 0xa5652379, 33 },
    { 0x72a52377, 98 },
    { 0x68e52377, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca52380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2852384, 87 },
    { 0, 0 },
    { 0x1052386, 52 },
    { 0xf3a52387, 52 },
    { 0, 0 },
    { 0xdff52389, 69 },
    { 0, 0 },
    { 0x35e5238b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4545238f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e52393, 61 },
    { 0, 0 },
    { 0x9bb52395, 95 },
    { 0xd7c52396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a523a0, 76 },
    { 0x1c0523a1, 90 },
    { 0x473523a2, 23 },
    { 0x6c4523a0, 90 },
    { 0x800523a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65523a7, 76 },
    { 0x190523a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x508523ab, 90 },
    { 0x4da523ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193523b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945523b8, 90 },
    { 0x434523b9, 90 },
    { 0, 0 },
    { 0x166523bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471523c0, 95 },
    { 0, 0 },
    { 0x345523c2, 34 },
    { 0xff2523c3, 90 },
    { 0x4e9523c3, 93 },
    { 0xb39523c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5523c9, 93 },
    { 0x639523ca, 61 },
    { 0x688523ca, 61 },
    { 0x444523ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x748523cf, 98 },
    { 0xfd0523d0, 61 },
    { 0, 0 },
    { 0x279523d2, 23 },
    { 0x6e5523d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31523d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb523de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873523e5, 90 },
    { 0, 0 },
    { 0xa64523e7, 91 },
    { 0x89d523e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0523ee, 66 },
    { 0x228523ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156523f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324523fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c52400, 61 },
    { 0, 0 },
    { 0x1d952402, 76 },
    { 0x24752403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29952411, 95 },
    { 0xa1352412, 93 },
    { 0x95b52413, 69 },
    { 0x8b652413, 47 },
    { 0xb6552412, 34 },
    { 0x2ff52416, 53 },
    { 0xfcf52416, 52 },
    { 0xe9d52414, 61 },
    { 0xa2d52419, 75 },
    { 0xc2c52419, 76 },
    { 0xbcf5241b, 90 },
    { 0, 0 },
    { 0x9a25241d, 93 },
    { 0, 0 },
    { 0x8645241f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7352422, 43 },
    { 0x23952423, 90 },
    { 0x2a52422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5242f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad52433, 33 },
    { 0xf3252434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5243a, 90 },
    { 0, 0 },
    { 0xcca5243c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5052445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33252448, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef5244c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda52456, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd5245d, 33 },
    { 0xba15245d, 55 },
    { 0xa6c5245d, 55 },
    { 0, 0 },
    { 0x15752461, 95 },
    { 0x9de52461, 34 },
    { 0xb052463, 100 },
    { 0xc0752464, 61 },
    { 0, 0 },
    { 0xd5052466, 56 },
    { 0xe9b52466, 90 },
    { 0xa9752468, 90 },
    { 0x81052468, 98 },
    { 0xae252468, 99 },
    { 0x7df52469, 90 },
    { 0xf2052468, 23 },
    { 0, 0 },
    { 0x3425246e, 76 },
    { 0xfa35246f, 76 },
    { 0x5665246f, 95 },
    { 0x28852471, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed052474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5652477, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b95247d, 98 },
    { 0, 0 },
    { 0xe595247f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45752482, 23 },
    { 0, 0 },
    { 0x45652484, 33 },
    { 0x13e52485, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405248a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf52491, 95 },
    { 0x8ef52491, 90 },
    { 0, 0 },
    { 0x16552494, 90 },
    { 0, 0 },
    { 0x83452496, 76 },
    { 0xc6052497, 100 },
    { 0xeba52497, 23 },
    { 0, 0 },
    { 0x12b5249a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9855249f, 93 },
    { 0, 0 },
    { 0x4fe524a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1524a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af524ae, 43 },
    { 0, 0 },
    { 0xb10524b0, 76 },
    { 0xbc4524b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82524b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9524bd, 90 },
    { 0x11e524bd, 99 },
    { 0x3c0524bf, 93 },
    { 0xea0524bf, 100 },
    { 0, 0 },
    { 0x85a524c2, 95 },
    { 0xa27524c3, 90 },
    { 0xa5b524c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2524ca, 33 },
    { 0xc4f524ca, 33 },
    { 0x3524ca, 90 },
    { 0x17e524cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e524d2, 23 },
    { 0, 0 },
    { 0xef8524d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d524d7, 81 },
    { 0x7bb524d8, 55 },
    { 0x87c524d9, 90 },
    { 0x66e524d9, 17 },
    { 0x5e5524db, 77 },
    { 0, 0 },
    { 0x55a524dd, 98 },
    { 0x2e5524dd, 40 },
    { 0, 0 },
    { 0xf18524e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64524e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df524ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4524f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1524f6, 76 },
    { 0, 0 },
    { 0x24f524f8, 98 },
    { 0, 0 },
    { 0xa35524fa, 95 },
    { 0xf75524fb, 33 },
    { 0xff0524fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f52500, 23 },
    { 0x7cd52501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c52505, 95 },
    { 0, 0 },
    { 0xd9452507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d5250b, 95 },
    { 0xbb85250c, 68 },
    { 0x3d55250c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645251a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd315251d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17852523, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2935252a, 34 },
    { 0, 0 },
    { 0x8a65252c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7552532, 77 },
    { 0xd6e52533, 55 },
    { 0, 0 },
    { 0x19352535, 91 },
    { 0x4b952535, 93 },
    { 0, 0 },
    { 0xf1952538, 55 },
    { 0x96552539, 46 },
    { 0xf915253a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc895253e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c552542, 85 },
    { 0xa452542, 90 },
    { 0xba552543, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9652549, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf352550, 61 },
    { 0xeba52551, 90 },
    { 0, 0 },
    { 0x9bc52553, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48652556, 95 },
    { 0, 0 },
    { 0x19652558, 34 },
    { 0x59652559, 90 },
    { 0x3b55255a, 66 },
    { 0x2ce5255b, 61 },
    { 0x70d5255a, 76 },
    { 0x7425255d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f52561, 98 },
    { 0xd9352562, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58652568, 95 },
    { 0x10952568, 88 },
    { 0x13f52569, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15256e, 60 },
    { 0, 0 },
    { 0xc2952570, 90 },
    { 0x49652571, 90 },
    { 0x98052572, 90 },
    { 0x78852573, 33 },
    { 0xe7152572, 23 },
    { 0x50752573, 46 },
    { 0xcf952576, 75 },
    { 0x51852574, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d5257a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84952580, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6452583, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f52587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b5258a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d352591, 55 },
    { 0x69152592, 98 },
    { 0x5c352593, 98 },
    { 0x15d52591, 90 },
    { 0xe1152595, 88 },
    { 0xf9e52595, 98 },
    { 0, 0 },
    { 0x26952598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b5259d, 98 },
    { 0, 0 },
    { 0xd9b5259f, 77 },
    { 0x1cf525a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01525a3, 98 },
    { 0, 0 },
    { 0xd97525a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849525a9, 33 },
    { 0xceb525aa, 90 },
    { 0x6c6525aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0525ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a525b4, 61 },
    { 0x319525b4, 77 },
    { 0xb7a525b6, 61 },
    { 0, 0 },
    { 0xe10525b8, 93 },
    { 0, 0 },
    { 0xe01525ba, 33 },
    { 0, 0 },
    { 0xdb6525bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e525c1, 90 },
    { 0x6bb525c2, 90 },
    { 0x71525c3, 61 },
    { 0xaf0525c4, 55 },
    { 0xb11525c4, 78 },
    { 0xa3f525c6, 85 },
    { 0xca7525c7, 61 },
    { 0x64f525c1, 98 },
    { 0, 0 },
    { 0x77e525ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385525d1, 23 },
    { 0, 0 },
    { 0xf2e525d3, 76 },
    { 0, 0 },
    { 0xc9525d5, 33 },
    { 0, 0 },
    { 0xc73525d7, 99 },
    { 0x98b525d8, 52 },
    { 0x373525d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc525dd, 95 },
    { 0, 0 },
    { 0x956525df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd525e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879525e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985525ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0525f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4525f5, 52 },
    { 0x76525f6, 61 },
    { 0, 0 },
    { 0xa4f525f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f525ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c352604, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc5260a, 95 },
    { 0xdd55260b, 97 },
    { 0x2475260b, 61 },
    { 0x6705260a, 23 },
    { 0x5d05260e, 100 },
    { 0xba15260b, 82 },
    { 0, 0 },
    { 0x1f852611, 76 },
    { 0x2a452611, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a752616, 90 },
    { 0xd552617, 61 },
    { 0x51152617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165261c, 66 },
    { 0x2275261d, 91 },
    { 0x5dc5261e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f352624, 76 },
    { 0xc5952625, 61 },
    { 0, 0 },
    { 0x43352627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535262d, 33 },
    { 0x4715262e, 98 },
    { 0xcd95262f, 95 },
    { 0xcaf52630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a352636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a52639, 76 },
    { 0x9da5263a, 99 },
    { 0x8505263a, 33 },
    { 0x10f52639, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11352641, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8452647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae5264b, 31 },
    { 0x1a65264c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d52650, 23 },
    { 0x39752651, 66 },
    { 0, 0 },
    { 0xc8152653, 43 },
    { 0xc8052654, 43 },
    { 0, 0 },
    { 0x81452656, 55 },
    { 0, 0 },
    { 0x26d52658, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5265f, 85 },
    { 0x10f52660, 61 },
    { 0xe525265f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa852667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5315266d, 33 },
    { 0xbd45266e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab52673, 55 },
    { 0, 0 },
    { 0xaa052675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d5267b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5267e, 61 },
    { 0x3f75267f, 23 },
    { 0x9375267f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a52683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93652689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45452692, 95 },
    { 0x57552693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a52698, 95 },
    { 0x15e52698, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d65269d, 84 },
    { 0xe45269e, 61 },
    { 0, 0 },
    { 0xb74526a0, 34 },
    { 0x4c8526a1, 95 },
    { 0x5fe526a2, 98 },
    { 0x148526a1, 33 },
    { 0x58e526a4, 98 },
    { 0x937526a2, 22 },
    { 0x9ef526a6, 52 },
    { 0x636526a6, 90 },
    { 0, 0 },
    { 0xb00526a9, 61 },
    { 0x4d526aa, 55 },
    { 0, 0 },
    { 0xdc3526ac, 87 },
    { 0xed2526ad, 85 },
    { 0x8d0526ad, 55 },
    { 0, 0 },
    { 0x6c8526b0, 90 },
    { 0x1d526b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219526b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9526bd, 95 },
    { 0xe0a526be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332526c4, 95 },
    { 0xaef526c5, 98 },
    { 0, 0 },
    { 0x214526c7, 88 },
    { 0, 0 },
    { 0xa07526c9, 46 },
    { 0xad2526c9, 23 },
    { 0x504526cb, 23 },
    { 0x6bd526cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b526d2, 87 },
    { 0xe01526d3, 23 },
    { 0, 0 },
    { 0xf2c526d5, 90 },
    { 0xbe3526d6, 61 },
    { 0x29c526d6, 43 },
    { 0x7c1526d8, 72 },
    { 0xf73526d6, 34 },
    { 0xc1d526da, 78 },
    { 0x50b526db, 74 },
    { 0x684526db, 61 },
    { 0xb90526dd, 61 },
    { 0, 0 },
    { 0x6f7526df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6526e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6526e6, 76 },
    { 0x29b526e6, 84 },
    { 0x57e526e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c526ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7526f3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a452705, 33 },
    { 0, 0 },
    { 0x9ce52707, 95 },
    { 0, 0 },
    { 0x93452709, 67 },
    { 0xedf52709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75270e, 90 },
    { 0xfbb5270f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc352715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a052724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e552727, 91 },
    { 0xc9a52728, 55 },
    { 0x23452729, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475272c, 87 },
    { 0xc0b5272c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca52734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55652738, 69 },
    { 0, 0 },
    { 0x8415273a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e852740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34352744, 80 },
    { 0x41b52745, 90 },
    { 0x73652746, 76 },
    { 0x2b852747, 77 },
    { 0x33c52748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b5274f, 78 },
    { 0, 0 },
    { 0xc1452751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d52755, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f52758, 23 },
    { 0xfce52758, 95 },
    { 0xe7b5275a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5275d, 33 },
    { 0x90c5275e, 33 },
    { 0xfd55275e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c52762, 90 },
    { 0x77152763, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b52768, 98 },
    { 0xba052768, 95 },
    { 0x7745276a, 93 },
    { 0, 0 },
    { 0xe4a5276c, 95 },
    { 0x5f35276d, 66 },
    { 0, 0 },
    { 0x8835276f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6452775, 61 },
    { 0x84152775, 93 },
    { 0x5e952776, 98 },
    { 0x94552776, 55 },
    { 0, 0 },
    { 0xdd75277a, 100 },
    { 0xe8f5277a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d5277f, 85 },
    { 0, 0 },
    { 0x2ec52781, 87 },
    { 0xdeb52782, 66 },
    { 0xabc52783, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa52788, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855278b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5278f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a352799, 93 },
    { 0xe3d52799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e527a1, 95 },
    { 0xf18527a2, 87 },
    { 0x937527a2, 55 },
    { 0x800527a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a527a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c527ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b527b1, 61 },
    { 0, 0 },
    { 0x8e6527b3, 95 },
    { 0, 0 },
    { 0xc07527b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde527b9, 78 },
    { 0x922527ba, 33 },
    { 0x97b527bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1527c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04527ca, 90 },
    { 0, 0 },
    { 0x781527cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24527cf, 90 },
    { 0xbfb527d0, 23 },
    { 0, 0 },
    { 0x91e527d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3527d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0527db, 23 },
    { 0, 0 },
    { 0xfc527dd, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4527e0, 23 },
    { 0, 0 },
    { 0xe15527e2, 46 },
    { 0xe07527e2, 42 },
    { 0xfca527e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6527f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f527f6, 100 },
    { 0x72d527f6, 23 },
    { 0xcfa527f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57527fb, 84 },
    { 0xe24527fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38527ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52804, 17 },
    { 0, 0 },
    { 0x55152806, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b5280a, 90 },
    { 0, 0 },
    { 0x7ef5280c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc55280f, 80 },
    { 0x4dc52810, 95 },
    { 0xb7a52811, 90 },
    { 0x54152810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5281a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5281e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6152827, 33 },
    { 0xaa352827, 33 },
    { 0x1a552827, 61 },
    { 0x98d5282a, 93 },
    { 0xcaf52829, 90 },
    { 0x3335282c, 33 },
    { 0, 0 },
    { 0x26c5282e, 99 },
    { 0, 0 },
    { 0x6bc52830, 66 },
    { 0xc4252831, 95 },
    { 0xde952832, 34 },
    { 0x61c52830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c52836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd5283a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52842, 75 },
    { 0, 0 },
    { 0xf5652844, 90 },
    { 0x91052845, 33 },
    { 0xec52846, 34 },
    { 0x15952847, 61 },
    { 0x5f152848, 56 },
    { 0xccf52846, 76 },
    { 0x7785284a, 100 },
    { 0xb1652847, 33 },
    { 0x12652844, 75 },
    { 0xc8b52848, 93 },
    { 0x3fc52844, 81 },
    { 0x5425284f, 91 },
    { 0x9ed52850, 52 },
    { 0xd0252851, 91 },
    { 0xa3e52852, 76 },
    { 0x51f52852, 33 },
    { 0x43c5284a, 90 },
    { 0, 0 },
    { 0xc4c52856, 93 },
    { 0xc1252857, 83 },
    { 0x6f552858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65285b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5285f, 76 },
    { 0xd985285f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4152865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80452868, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa052876, 91 },
    { 0xf6d52877, 80 },
    { 0x45852878, 61 },
    { 0x52552877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc52880, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x25252883, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c452889, 33 },
    { 0xe3a5288a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf45288e, 33 },
    { 0x2d65288f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa52893, 34 },
    { 0xa6452894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52899, 90 },
    { 0xb7b5289a, 23 },
    { 0x50452899, 61 },
    { 0x9175289c, 98 },
    { 0x9b25289c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41528a5, 65 },
    { 0x6de528a5, 66 },
    { 0x8cf528a5, 34 },
    { 0x68f528a6, 23 },
    { 0xf47528a9, 91 },
    { 0xa51528aa, 46 },
    { 0x9a8528a8, 55 },
    { 0, 0 },
    { 0xee4528ad, 90 },
    { 0xab6528ae, 90 },
    { 0, 0 },
    { 0x5cb528b0, 61 },
    { 0x4fe528b1, 33 },
    { 0x520528b1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3528ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462528be, 78 },
    { 0, 0 },
    { 0x45528c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50528c4, 61 },
    { 0, 0 },
    { 0x1c528c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82528cc, 83 },
    { 0xed8528cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8528d0, 23 },
    { 0xaf7528d1, 80 },
    { 0x254528d1, 95 },
    { 0, 0 },
    { 0x552528d4, 75 },
    { 0x6ac528d5, 53 },
    { 0xfea528d5, 23 },
    { 0xab6528d7, 98 },
    { 0x83e528d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09528e1, 90 },
    { 0, 0 },
    { 0xc90528e3, 61 },
    { 0xf59528e4, 90 },
    { 0x289528e4, 23 },
    { 0xed8528e6, 90 },
    { 0x8bf528e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db528ec, 100 },
    { 0, 0 },
    { 0x318528ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4528f3, 84 },
    { 0, 0 },
    { 0xb26528f5, 80 },
    { 0x48a528f6, 55 },
    { 0xdf9528f6, 90 },
    { 0x384528f8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f528fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd652901, 55 },
    { 0x60352902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e052908, 36 },
    { 0, 0 },
    { 0x7e05290a, 23 },
    { 0, 0 },
    { 0xa325290c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d352913, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad52919, 34 },
    { 0xd265291a, 34 },
    { 0x64b5291b, 34 },
    { 0, 0 },
    { 0x58f5291d, 43 },
    { 0xec75291e, 33 },
    { 0, 0 },
    { 0xefd52920, 98 },
    { 0x1252920, 17 },
    { 0x6cf52922, 33 },
    { 0xd8552922, 75 },
    { 0xfa152924, 43 },
    { 0xf9e52925, 98 },
    { 0xe6452926, 46 },
    { 0x9c52925, 61 },
    { 0x92152922, 95 },
    { 0x68752929, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa485292f, 90 },
    { 0x8215292f, 55 },
    { 0xb7d52930, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe52934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085293d, 46 },
    { 0x30d5293d, 69 },
    { 0x3ce5293f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38852946, 76 },
    { 0xc8652947, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf5294b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6915294e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7552958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5295b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb5295e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec852961, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad52965, 99 },
    { 0x5c652965, 95 },
    { 0, 0 },
    { 0xe0152968, 33 },
    { 0x60252968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f52973, 90 },
    { 0xc2f52974, 87 },
    { 0xa5252974, 33 },
    { 0xa3052976, 46 },
    { 0, 0 },
    { 0xb8f52978, 93 },
    { 0, 0 },
    { 0x3335297a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98752981, 98 },
    { 0, 0 },
    { 0xeb452983, 61 },
    { 0xbe252983, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x21852987, 78 },
    { 0x37352988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe552994, 33 },
    { 0xdb452995, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb52998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde5299b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31529a2, 23 },
    { 0, 0 },
    { 0xf9c529a4, 46 },
    { 0xff4529a5, 33 },
    { 0xf1e529a6, 99 },
    { 0xc45529a7, 95 },
    { 0xae8529a8, 61 },
    { 0xe79529a7, 95 },
    { 0xf16529aa, 99 },
    { 0x29e529a8, 87 },
    { 0xe87529a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0529b3, 84 },
    { 0x82c529b4, 33 },
    { 0x897529b5, 61 },
    { 0x6b529b6, 33 },
    { 0xd37529b6, 55 },
    { 0x9b9529b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x383529bb, 33 },
    { 0xff9529bc, 34 },
    { 0x1b6529bd, 23 },
    { 0xa64529be, 61 },
    { 0x916529bb, 99 },
    { 0x14a529c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x179529c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6529c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f529d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50529d7, 61 },
    { 0x758529d8, 98 },
    { 0xd76529d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7529df, 43 },
    { 0, 0 },
    { 0xc28529e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75529e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a529e7, 33 },
    { 0, 0 },
    { 0x7fa529e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a529ed, 90 },
    { 0x516529ee, 88 },
    { 0, 0 },
    { 0x29f529f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36529f8, 90 },
    { 0x539529f9, 88 },
    { 0xb49529fa, 90 },
    { 0xca529fb, 90 },
    { 0x759529fc, 95 },
    { 0xf15529fc, 90 },
    { 0, 0 },
    { 0xb04529ff, 76 },
    { 0xcc752a00, 23 },
    { 0x6cc52a01, 46 },
    { 0x40b52a02, 23 },
    { 0xaf52a01, 12 },
    { 0x69d52a04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a652a08, 33 },
    { 0xbe252a09, 90 },
    { 0, 0 },
    { 0xed652a0b, 98 },
    { 0x95c52a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f52a10, 98 },
    { 0x26552a10, 31 },
    { 0x14452a12, 95 },
    { 0, 0 },
    { 0xdcd52a14, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c52a17, 69 },
    { 0xdac52a18, 90 },
    { 0x23f52a19, 33 },
    { 0xeb152a1a, 23 },
    { 0xad052a17, 17 },
    { 0xad852a1c, 55 },
    { 0xcd52a1d, 95 },
    { 0x5aa52a19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f52a21, 87 },
    { 0xb5f52a22, 95 },
    { 0xe0452a23, 34 },
    { 0xde052a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e52a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452a2c, 93 },
    { 0x68e52a2d, 33 },
    { 0, 0 },
    { 0x4fb52a2f, 46 },
    { 0x74552a30, 33 },
    { 0, 0 },
    { 0x8b52a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb652a35, 69 },
    { 0x97952a36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c52a3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9952a45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb552a48, 23 },
    { 0xd1252a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e52a4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac952a51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47452a54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49852a5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34252a63, 76 },
    { 0, 0 },
    { 0xe9452a65, 33 },
    { 0x79c52a65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57052a6a, 97 },
    { 0, 0 },
    { 0x6c052a6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d52a70, 96 },
    { 0xae852a70, 33 },
    { 0x50352a72, 95 },
    { 0, 0 },
    { 0x73c52a74, 67 },
    { 0xb2552a75, 95 },
    { 0, 0 },
    { 0x13e52a77, 98 },
    { 0xfdb52a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea52a80, 61 },
    { 0x11052a80, 95 },
    { 0x3f552a82, 61 },
    { 0, 0 },
    { 0x5ae52a84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af52a88, 65 },
    { 0, 0 },
    { 0x89b52a8a, 90 },
    { 0, 0 },
    { 0x43a52a8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde352a93, 85 },
    { 0, 0 },
    { 0xff352a95, 90 },
    { 0x1c252a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd752a99, 69 },
    { 0, 0 },
    { 0x74452a9b, 43 },
    { 0xbb52a9b, 61 },
    { 0xaa52a9d, 100 },
    { 0, 0 },
    { 0x50f52a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c252aa7, 80 },
    { 0xcfc52aa8, 43 },
    { 0x39452aa8, 38 },
    { 0xc3052aa8, 55 },
    { 0x11c52aab, 33 },
    { 0xa4652aab, 90 },
    { 0, 0 },
    { 0xe0952aae, 95 },
    { 0x8f552aae, 34 },
    { 0, 0 },
    { 0xd1852ab1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3952ab4, 23 },
    { 0, 0 },
    { 0xfaf52ab6, 87 },
    { 0x1d152ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd52ac4, 66 },
    { 0xa2352ac5, 90 },
    { 0x54d52ac5, 99 },
    { 0xe2952ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e052aca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb552acd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a52ad3, 23 },
    { 0x3f152ad4, 90 },
    { 0x2ab52ad3, 76 },
    { 0xfc952ad6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3152adb, 33 },
    { 0xc6352adb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb052ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d452ae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf52ae9, 88 },
    { 0x84152aea, 53 },
    { 0xa552aea, 95 },
    { 0xaa152ae9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc852af1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff752af7, 90 },
    { 0xad552af8, 61 },
    { 0xe6252af7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb352afd, 46 },
    { 0x4e652afd, 96 },
    { 0x53952afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe652b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50952b07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d52b10, 84 },
    { 0x6f252b10, 23 },
    { 0, 0 },
    { 0xa1d52b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8552b18, 88 },
    { 0xc6452b18, 61 },
    { 0xfe552b1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27252b1d, 76 },
    { 0x13552b1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee52b21, 78 },
    { 0xee352b21, 43 },
    { 0, 0 },
    { 0x81152b24, 33 },
    { 0xe1952b25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b52b28, 95 },
    { 0xecb52b28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40852b2c, 34 },
    { 0x1ce52b2c, 34 },
    { 0xd9452b2d, 33 },
    { 0x40352b2d, 66 },
    { 0x49752b2f, 46 },
    { 0x3f852b31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4652b34, 23 },
    { 0xc3c52b34, 76 },
    { 0xf3652b36, 55 },
    { 0xc7252b37, 55 },
    { 0, 0 },
    { 0x26752b39, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0352b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27352b42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4052b49, 23 },
    { 0x88352b4a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee52b4f, 33 },
    { 0, 0 },
    { 0xcd352b51, 61 },
    { 0x28152b52, 98 },
    { 0x18652b53, 58 },
    { 0x7fb52b54, 33 },
    { 0x17f52b55, 90 },
    { 0xba952b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37952b59, 55 },
    { 0, 0 },
    { 0x65b52b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac52b64, 52 },
    { 0, 0 },
    { 0xba852b66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f852b6c, 90 },
    { 0x8c852b6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46252b71, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada52b77, 95 },
    { 0x3d452b78, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a952b7b, 23 },
    { 0x63752b7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc152b80, 33 },
    { 0xae952b81, 90 },
    { 0x47052b82, 17 },
    { 0xabf52b81, 23 },
    { 0x2cb52b84, 98 },
    { 0x54452b81, 34 },
    { 0xc6552b86, 52 },
    { 0x38852b87, 66 },
    { 0x76752b86, 33 },
    { 0x32052b87, 96 },
    { 0xa7852b8a, 43 },
    { 0xcb952b8b, 75 },
    { 0xf8452b82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46252b94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7052b97, 61 },
    { 0x7dc52b97, 55 },
    { 0x6d452b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8452b9d, 55 },
    { 0xbb152b9e, 84 },
    { 0, 0 },
    { 0x40752ba0, 90 },
    { 0xddd52ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad052ba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac52ba9, 61 },
    { 0, 0 },
    { 0x64e52bab, 33 },
    { 0x3b752bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a52baf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf752bb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa952bba, 64 },
    { 0, 0 },
    { 0x1bc52bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7552bc3, 34 },
    { 0x33752bc4, 93 },
    { 0, 0 },
    { 0xe3a52bc6, 46 },
    { 0, 0 },
    { 0x33652bc8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e52bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31952bd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452bd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4552bda, 100 },
    { 0xa5052bdb, 76 },
    { 0, 0 },
    { 0x2c52bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21452be2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f52be7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d352bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85852bf5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92552bfb, 91 },
    { 0x6f452bfc, 61 },
    { 0x8cc52bfc, 19 },
    { 0xdaa52bfe, 33 },
    { 0, 0 },
    { 0xf9e52c00, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3252c0a, 36 },
    { 0x22452c0b, 93 },
    { 0xbe452c0a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x53852c0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa52c1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a52c1f, 34 },
    { 0, 0 },
    { 0xe6652c21, 90 },
    { 0, 0 },
    { 0xb2552c23, 90 },
    { 0xce652c24, 95 },
    { 0x59d52c24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a452c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf952c3f, 90 },
    { 0, 0 },
    { 0xd4052c41, 66 },
    { 0xb7052c42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11652c48, 33 },
    { 0xd8752c49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0852c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1452c56, 90 },
    { 0xf8e52c57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae52c5c, 43 },
    { 0x28a52c5d, 78 },
    { 0x37252c5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db52c61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28452c69, 93 },
    { 0, 0 },
    { 0x70852c6b, 99 },
    { 0, 0 },
    { 0x25352c6d, 68 },
    { 0xa252c6e, 76 },
    { 0x33152c6f, 67 },
    { 0xfd452c6e, 33 },
    { 0x54f52c71, 55 },
    { 0xcd352c72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea52c76, 95 },
    { 0x89552c77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc52c7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1352c7f, 85 },
    { 0xe3e52c80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c952c86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9452c89, 33 },
    { 0x5af52c8a, 34 },
    { 0x6e152c8a, 66 },
    { 0x21052c8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae52c8f, 23 },
    { 0x69452c90, 98 },
    { 0xf8a52c8f, 61 },
    { 0x4e52c92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f52c9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a52ca1, 90 },
    { 0, 0 },
    { 0x8e552ca3, 33 },
    { 0, 0 },
    { 0x9c452ca5, 75 },
    { 0xeb152ca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b52ca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e52cac, 55 },
    { 0, 0 },
    { 0xf2a52cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66552cb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0552cb5, 33 },
    { 0x3ad52cb5, 23 },
    { 0, 0 },
    { 0xdc652cb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5752cc1, 91 },
    { 0xe3c52cc1, 93 },
    { 0, 0 },
    { 0x97652cc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e552cce, 93 },
    { 0xc752ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca52cd5, 76 },
    { 0x6ef52cd5, 61 },
    { 0xaa152cd7, 95 },
    { 0xbd452cd6, 67 },
    { 0, 0 },
    { 0xf0e52cda, 99 },
    { 0x4fb52cdb, 77 },
    { 0, 0 },
    { 0x20152cdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c652ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552ce8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea52ced, 68 },
    { 0x7d52cee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36352cf2, 66 },
    { 0, 0 },
    { 0x8c252cf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48552cf9, 90 },
    { 0, 0 },
    { 0xcb252cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d352cff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7752d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c52d09, 95 },
    { 0xa2852d0a, 33 },
    { 0, 0 },
    { 0x85952d0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc52d0f, 33 },
    { 0x2bc52d10, 81 },
    { 0x6a952d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24352d15, 64 },
    { 0xf6752d15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73552d19, 98 },
    { 0x2fc52d19, 90 },
    { 0, 0 },
    { 0xefd52d1c, 33 },
    { 0x63a52d1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8552d20, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51752d26, 34 },
    { 0x36052d27, 95 },
    { 0x9b252d27, 95 },
    { 0x5af52d26, 61 },
    { 0xed252d2a, 90 },
    { 0xd0452d2b, 46 },
    { 0x6fe52d2c, 91 },
    { 0x36f52d2c, 76 },
    { 0, 0 },
    { 0xd4f52d2f, 100 },
    { 0, 0 },
    { 0xf9652d31, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x11952d34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b52d3d, 92 },
    { 0x2fb52d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4052d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd52d4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb052d52, 33 },
    { 0x9f852d52, 24 },
    { 0, 0 },
    { 0x76f52d55, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1652d5b, 33 },
    { 0xa9c52d5c, 98 },
    { 0x3d652d5c, 43 },
    { 0x7df52d5d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa252d63, 33 },
    { 0, 0 },
    { 0x45452d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e52d6a, 90 },
    { 0x8f52d6a, 61 },
    { 0xa1952d6b, 90 },
    { 0, 0 },
    { 0x78252d6e, 95 },
    { 0x17052d6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1052d77, 61 },
    { 0x30552d78, 98 },
    { 0x6a652d79, 33 },
    { 0x6b52d79, 23 },
    { 0x8cd52d7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb52d7f, 90 },
    { 0x60652d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a252d88, 87 },
    { 0x5be52d89, 33 },
    { 0, 0 },
    { 0xae152d8b, 38 },
    { 0x7a852d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb52d8f, 78 },
    { 0x41352d90, 96 },
    { 0xe8852d8f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0152d98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3852d9e, 33 },
    { 0xa1752d9f, 90 },
    { 0x71352da0, 43 },
    { 0xe4b52da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12352da7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8052daa, 52 },
    { 0xe6552daa, 98 },
    { 0, 0 },
    { 0x68752dad, 33 },
    { 0, 0 },
    { 0xc8552daf, 33 },
    { 0x20f52daf, 55 },
    { 0xedb52db1, 77 },
    { 0x5b152db2, 90 },
    { 0xb9152db2, 46 },
    { 0xca452db2, 43 },
    { 0x5e252db3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b52db8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42452dbf, 90 },
    { 0, 0 },
    { 0x1a952dc1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b52dc5, 33 },
    { 0x3fd52dc6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94552dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2452dd1, 90 },
    { 0xf5552dd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64452dd5, 55 },
    { 0x1cf52dd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2252dd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b52de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb52de3, 34 },
    { 0x2a152de4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae52de8, 66 },
    { 0x1bf52de9, 98 },
    { 0x7df52de9, 33 },
    { 0x43752deb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e52dee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c52df1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a52df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e52df9, 95 },
    { 0xbbe52df9, 61 },
    { 0, 0 },
    { 0x70f52dfc, 55 },
    { 0x80352dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e52e02, 90 },
    { 0xb752e02, 75 },
    { 0xeac52e02, 95 },
    { 0x6e552e05, 43 },
    { 0x72b52e06, 61 },
    { 0x41852e02, 90 },
    { 0xfce52e08, 34 },
    { 0, 0 },
    { 0x26252e0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24552e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc452e15, 61 },
    { 0, 0 },
    { 0xa4452e17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3052e1f, 90 },
    { 0x86652e20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a52e23, 90 },
    { 0xeff52e24, 91 },
    { 0x95852e25, 95 },
    { 0xc6252e26, 17 },
    { 0xf6f52e27, 87 },
    { 0x74f52e28, 55 },
    { 0, 0 },
    { 0x2652e2a, 33 },
    { 0x61552e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74252e2e, 95 },
    { 0x5452e2f, 23 },
    { 0xb7152e30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59752e33, 33 },
    { 0, 0 },
    { 0x3e052e35, 95 },
    { 0, 0 },
    { 0x79a52e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21152e3b, 33 },
    { 0x39452e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab752e3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57352e42, 75 },
    { 0, 0 },
    { 0x72252e44, 88 },
    { 0x84252e45, 85 },
    { 0, 0 },
    { 0x752e47, 33 },
    { 0xb1052e48, 61 },
    { 0x16852e48, 66 },
    { 0x51352e4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23152e51, 23 },
    { 0x85452e52, 52 },
    { 0x94c52e51, 90 },
    { 0, 0 },
    { 0x40052e55, 17 },
    { 0xa9c52e55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55052e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c852e62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad52e68, 95 },
    { 0, 0 },
    { 0x2bc52e6a, 61 },
    { 0x24552e6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c52e6e, 61 },
    { 0, 0 },
    { 0xcc52e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac52e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f52e79, 61 },
    { 0x9bb52e7a, 93 },
    { 0, 0 },
    { 0xfc452e7c, 23 },
    { 0x23b52e7d, 33 },
    { 0x2a52e7e, 33 },
    { 0x6d952e7e, 90 },
    { 0xa8552e80, 90 },
    { 0, 0 },
    { 0x51452e82, 34 },
    { 0, 0 },
    { 0x9352e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52e8a, 91 },
    { 0xd9a52e8a, 61 },
    { 0xe4052e8c, 100 },
    { 0x3f052e8c, 90 },
    { 0x53c52e8b, 76 },
    { 0xa7c52e8f, 61 },
    { 0, 0 },
    { 0x3db52e91, 55 },
    { 0, 0 },
    { 0x38e52e93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd52e97, 100 },
    { 0xefb52e98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3952e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa352ea6, 68 },
    { 0, 0 },
    { 0x4752ea8, 95 },
    { 0, 0 },
    { 0xee952eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e752ead, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a452eb0, 95 },
    { 0x10052eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f952eb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b52eb8, 95 },
    { 0xd1852eb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152ebd, 99 },
    { 0x7c752ebe, 33 },
    { 0, 0 },
    { 0x78652ec0, 33 },
    { 0x6dd52ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38052ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b52ec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be52ecd, 94 },
    { 0, 0 },
    { 0x48052ecf, 33 },
    { 0, 0 },
    { 0xb1c52ed1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd52ed4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c52ed7, 61 },
    { 0, 0 },
    { 0xe8152ed9, 93 },
    { 0x88d52ed9, 66 },
    { 0, 0 },
    { 0xfa652edc, 61 },
    { 0, 0 },
    { 0x76452ede, 32 },
    { 0xcb452ede, 33 },
    { 0xc4452ee0, 38 },
    { 0x2f852ee0, 96 },
    { 0x9b252ee2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c52ee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a452eec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd52eef, 23 },
    { 0x8f752ef0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c52ef4, 23 },
    { 0xece52ef5, 85 },
    { 0xe752ef4, 34 },
    { 0, 0 },
    { 0xcdb52ef8, 23 },
    { 0, 0 },
    { 0x3e952efa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec952f01, 34 },
    { 0xbd252f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1052f07, 90 },
    { 0, 0 },
    { 0x70052f09, 23 },
    { 0, 0 },
    { 0x73f52f0b, 77 },
    { 0, 0 },
    { 0x35352f0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9952f15, 76 },
    { 0x41f52f16, 46 },
    { 0, 0 },
    { 0x20f52f18, 76 },
    { 0xb552f18, 95 },
    { 0, 0 },
    { 0xc3452f1b, 34 },
    { 0, 0 },
    { 0xa3652f1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90052f25, 98 },
    { 0x7aa52f26, 23 },
    { 0, 0 },
    { 0x3852f28, 61 },
    { 0x94f52f28, 34 },
    { 0, 0 },
    { 0xddf52f2b, 76 },
    { 0, 0 },
    { 0xce652f2d, 43 },
    { 0x68e52f2e, 95 },
    { 0x1e652f2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4352f33, 46 },
    { 0x21b52f33, 33 },
    { 0x3ba52f33, 23 },
    { 0, 0 },
    { 0x22452f37, 52 },
    { 0, 0 },
    { 0xa5d52f39, 55 },
    { 0xe0852f3a, 82 },
    { 0x56e52f39, 68 },
    { 0x26052f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a52f3f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e052f42, 95 },
    { 0x70952f43, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce52f49, 95 },
    { 0x3f952f49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec52f55, 98 },
    { 0x84e52f56, 83 },
    { 0xea752f57, 23 },
    { 0x10452f56, 95 },
    { 0, 0 },
    { 0xaf052f5a, 81 },
    { 0, 0 },
    { 0x97952f5c, 76 },
    { 0xd8152f5d, 93 },
    { 0, 0 },
    { 0x2b152f5f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0852f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10852f6a, 99 },
    { 0, 0 },
    { 0xfe052f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9752f71, 23 },
    { 0x74b52f72, 93 },
    { 0x20c52f73, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b52f76, 87 },
    { 0, 0 },
    { 0x4a852f78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a52f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf52f7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef52f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21652f85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15652f8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f98, 95 },
    { 0, 0 },
    { 0xeb652f9a, 33 },
    { 0x42852f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb52fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c52fa4, 34 },
    { 0x11f52fa4, 52 },
    { 0x85452fa6, 61 },
    { 0x34052fa6, 95 },
    { 0xe3c52fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79652fae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d052fb3, 33 },
    { 0x21c52fb4, 61 },
    { 0, 0 },
    { 0x5d52fb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5952fbb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb052fc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x44452fc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d752fc8, 90 },
    { 0xcb752fc9, 90 },
    { 0x25552fca, 93 },
    { 0x65f52fcb, 99 },
    { 0x16752fca, 33 },
    { 0x65f52fc9, 23 },
    { 0xd5f52fce, 76 },
    { 0x8e852fcd, 90 },
    { 0xac452fc8, 61 },
    { 0x2ae52fd1, 93 },
    { 0, 0 },
    { 0x7fb52fd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c452fd6, 66 },
    { 0, 0 },
    { 0xa7352fd8, 95 },
    { 0x94552fd8, 90 },
    { 0x37652fda, 61 },
    { 0, 0 },
    { 0x76952fdc, 98 },
    { 0x1ae52fdd, 76 },
    { 0xe3f52fde, 91 },
    { 0, 0 },
    { 0xbf052fe0, 95 },
    { 0, 0 },
    { 0x7f452fe2, 84 },
    { 0, 0 },
    { 0xba52fe4, 38 },
    { 0xc3952fe4, 61 },
    { 0x71552fe6, 52 },
    { 0, 0 },
    { 0x1d752fe8, 61 },
    { 0x8ba52fe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c52fef, 33 },
    { 0xeaf52ff0, 99 },
    { 0x81752fef, 98 },
    { 0x72652ff2, 91 },
    { 0xb4352ff2, 87 },
    { 0x48a52ff4, 78 },
    { 0x7b652ff5, 90 },
    { 0xb7b52ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b52ff9, 78 },
    { 0x1252ffa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d53002, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0453005, 34 },
    { 0x15f53006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad75300b, 23 },
    { 0, 0 },
    { 0x6d15300d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf53016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5301a, 76 },
    { 0, 0 },
    { 0x5995301c, 90 },
    { 0xd3f5301d, 46 },
    { 0x2af5301c, 98 },
    { 0, 0 },
    { 0x5bf53020, 98 },
    { 0x4053021, 98 },
    { 0xd4d53022, 61 },
    { 0x36353022, 90 },
    { 0x10a53024, 33 },
    { 0xbc253021, 90 },
    { 0x77953026, 99 },
    { 0x12053020, 100 },
    { 0, 0 },
    { 0x8e953029, 98 },
    { 0, 0 },
    { 0x6355302b, 55 },
    { 0, 0 },
    { 0x1b45302d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12753032, 93 },
    { 0xc0053033, 88 },
    { 0x7ec53032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4905303b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70453041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a853044, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce75304c, 76 },
    { 0, 0 },
    { 0x3ac5304e, 99 },
    { 0x5005304f, 77 },
    { 0xf8653050, 99 },
    { 0xe245304e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d53055, 100 },
    { 0x67153055, 88 },
    { 0x59953056, 33 },
    { 0, 0 },
    { 0x67653059, 98 },
    { 0xc4a5305a, 33 },
    { 0x4bd5305b, 33 },
    { 0x2005305c, 97 },
    { 0x2415305c, 33 },
    { 0xb235305d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53064, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf53067, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975306d, 98 },
    { 0x9ff5306e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de53071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b053074, 66 },
    { 0x1ea53074, 61 },
    { 0x97353075, 90 },
    { 0, 0 },
    { 0x54453078, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f45307f, 23 },
    { 0x7ee53080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe53087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f5308a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d653095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795309a, 91 },
    { 0xd615309b, 91 },
    { 0, 0 },
    { 0xb945309d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae530a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c530a9, 43 },
    { 0xe07530aa, 52 },
    { 0xed6530aa, 95 },
    { 0x8be530ac, 95 },
    { 0x6e1530ac, 23 },
    { 0x883530ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b530b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257530bb, 95 },
    { 0x984530bc, 34 },
    { 0xbe9530bd, 34 },
    { 0xfdc530be, 61 },
    { 0x35b530be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457530c3, 23 },
    { 0xdd530c3, 23 },
    { 0x37b530c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x208530c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709530cd, 90 },
    { 0x91d530cd, 33 },
    { 0, 0 },
    { 0x9b8530d0, 90 },
    { 0xc1a530d1, 77 },
    { 0x685530d1, 76 },
    { 0xf06530d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e530d6, 61 },
    { 0xc39530d7, 23 },
    { 0x9c0530d7, 34 },
    { 0xed8530d9, 33 },
    { 0xd7530da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283530de, 46 },
    { 0xaa5530df, 61 },
    { 0x9e530e0, 25 },
    { 0x9b9530e0, 65 },
    { 0xf2d530e2, 23 },
    { 0x8e3530e3, 51 },
    { 0x6fa530e3, 33 },
    { 0x6ad530e5, 95 },
    { 0x115530e5, 90 },
    { 0x1a3530e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57530ee, 98 },
    { 0x372530ef, 90 },
    { 0x26b530ef, 23 },
    { 0xb88530ef, 67 },
    { 0xf66530ef, 34 },
    { 0xa68530ef, 93 },
    { 0x1aa530f4, 90 },
    { 0x629530f5, 23 },
    { 0, 0 },
    { 0x97b530f7, 50 },
    { 0x460530f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc53100, 55 },
    { 0x82853100, 23 },
    { 0xb1d53102, 77 },
    { 0x78153103, 100 },
    { 0x67753103, 77 },
    { 0x9753105, 55 },
    { 0x16353104, 80 },
    { 0x1153102, 48 },
    { 0x26d53108, 52 },
    { 0xc2653109, 55 },
    { 0xb9a53108, 98 },
    { 0xa715310b, 95 },
    { 0xa3153108, 62 },
    { 0xacb53101, 99 },
    { 0x2655310e, 96 },
    { 0, 0 },
    { 0x20b53110, 83 },
    { 0xdcf53111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab65311a, 34 },
    { 0, 0 },
    { 0xba45311c, 34 },
    { 0x8ab5311c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2653122, 43 },
    { 0x63b53123, 33 },
    { 0x9b653123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da53128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b15312c, 33 },
    { 0x5c05312d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae53135, 33 },
    { 0xfdb53136, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de53139, 95 },
    { 0xccf53139, 100 },
    { 0x7b5313b, 33 },
    { 0x1ed5313c, 81 },
    { 0x1b253139, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22653141, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c53149, 34 },
    { 0, 0 },
    { 0x6055314b, 23 },
    { 0, 0 },
    { 0x87e5314d, 65 },
    { 0xa6f5314e, 46 },
    { 0x69c5314d, 33 },
    { 0x3c25314e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53157, 93 },
    { 0x40d53158, 90 },
    { 0xdb053157, 91 },
    { 0xf7d5315a, 95 },
    { 0x4f53157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e5315e, 52 },
    { 0xfa85315f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45316f, 91 },
    { 0xa5f53170, 88 },
    { 0xa9953170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a753176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d5317b, 61 },
    { 0, 0 },
    { 0xe195317d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71053188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5275318c, 55 },
    { 0xd465318d, 23 },
    { 0x43b5318c, 87 },
    { 0x1f45318f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e853196, 43 },
    { 0, 0 },
    { 0xd9d53198, 95 },
    { 0, 0 },
    { 0x5b35319a, 95 },
    { 0, 0 },
    { 0x4be5319c, 33 },
    { 0xec65319c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2531a3, 33 },
    { 0x116531a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb531a7, 90 },
    { 0x94a531a8, 61 },
    { 0xcd7531a9, 23 },
    { 0x5ef531a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c531ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f531b3, 76 },
    { 0x695531b3, 68 },
    { 0xee3531b5, 61 },
    { 0xd1a531b6, 98 },
    { 0x913531b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e531ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419531bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f531c3, 43 },
    { 0xf74531c3, 95 },
    { 0xd38531c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3531cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39531d0, 33 },
    { 0xae4531d1, 46 },
    { 0x3f5531d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0531d8, 61 },
    { 0xcff531d9, 43 },
    { 0x4ec531da, 68 },
    { 0x24a531db, 23 },
    { 0xd69531dc, 90 },
    { 0x824531d8, 98 },
    { 0, 0 },
    { 0xe79531df, 43 },
    { 0, 0 },
    { 0x3be531e1, 33 },
    { 0, 0 },
    { 0x701531e3, 98 },
    { 0x958531e3, 98 },
    { 0xb58531e5, 99 },
    { 0, 0 },
    { 0x8b1531e7, 90 },
    { 0x7f1531e8, 34 },
    { 0xd9d531e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d531ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21531ef, 98 },
    { 0, 0 },
    { 0xd0e531f1, 23 },
    { 0, 0 },
    { 0xbfe531f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a531f8, 99 },
    { 0x2da531f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7531ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15953209, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5320d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x47853210, 46 },
    { 0x7cb53210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94953217, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c55321a, 98 },
    { 0x39c5321a, 76 },
    { 0, 0 },
    { 0x2aa5321d, 46 },
    { 0x3395321e, 23 },
    { 0x955321e, 98 },
    { 0x32e53220, 34 },
    { 0xce753221, 85 },
    { 0xdce5321e, 95 },
    { 0x6685321e, 98 },
    { 0x54653220, 46 },
    { 0x28253224, 23 },
    { 0xf4453226, 75 },
    { 0xaa553227, 33 },
    { 0x72253228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4295322d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0053230, 31 },
    { 0x90d53231, 95 },
    { 0, 0 },
    { 0x81f53233, 78 },
    { 0xfe53234, 61 },
    { 0xd3f53235, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b53239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c35323c, 98 },
    { 0x8ef5323d, 36 },
    { 0xca35323c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd53242, 33 },
    { 0x2bd53242, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a253249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde15324f, 23 },
    { 0, 0 },
    { 0xc253251, 55 },
    { 0, 0 },
    { 0xf3353253, 43 },
    { 0, 0 },
    { 0x19e53255, 23 },
    { 0x39a53256, 85 },
    { 0xcbc53257, 77 },
    { 0xb5b53258, 23 },
    { 0xfe653257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5325d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc353262, 46 },
    { 0x4a753263, 23 },
    { 0x36453264, 46 },
    { 0x1a753264, 23 },
    { 0xfca53265, 98 },
    { 0xbd953266, 61 },
    { 0x26653268, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5326c, 33 },
    { 0, 0 },
    { 0x1525326e, 64 },
    { 0x5d25326f, 95 },
    { 0, 0 },
    { 0x4c353271, 90 },
    { 0x33953271, 93 },
    { 0x69253273, 43 },
    { 0xf8853274, 33 },
    { 0xf2053275, 46 },
    { 0x69653274, 24 },
    { 0xd6f53273, 33 },
    { 0x90f53278, 76 },
    { 0x553279, 61 },
    { 0x1fe53272, 76 },
    { 0, 0 },
    { 0x2095327c, 17 },
    { 0xb995327d, 33 },
    { 0, 0 },
    { 0xb3a5327f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e53282, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b53285, 67 },
    { 0x26b53285, 88 },
    { 0x2c853287, 95 },
    { 0x9553288, 100 },
    { 0x4f553289, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce453291, 100 },
    { 0, 0 },
    { 0x62753293, 23 },
    { 0x62353294, 34 },
    { 0x4d053295, 33 },
    { 0x2a53295, 76 },
    { 0x2fe53297, 76 },
    { 0x21a53298, 33 },
    { 0, 0 },
    { 0x2665329a, 91 },
    { 0xb335329b, 61 },
    { 0x5ea5329a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55329f, 33 },
    { 0x82f5329f, 55 },
    { 0x696532a1, 38 },
    { 0, 0 },
    { 0x5f2532a3, 90 },
    { 0xddb532a3, 33 },
    { 0, 0 },
    { 0xbe0532a6, 98 },
    { 0xceb532a7, 90 },
    { 0x86a532a8, 46 },
    { 0x746532a9, 83 },
    { 0x359532a8, 84 },
    { 0x386532ab, 76 },
    { 0, 0 },
    { 0x26c532ad, 43 },
    { 0, 0 },
    { 0x66f532af, 69 },
    { 0, 0 },
    { 0x7d9532b1, 93 },
    { 0, 0 },
    { 0x123532b3, 90 },
    { 0, 0 },
    { 0x39d532b5, 33 },
    { 0, 0 },
    { 0x739532b7, 33 },
    { 0x37532b8, 78 },
    { 0, 0 },
    { 0xf3f532ba, 98 },
    { 0x358532bb, 46 },
    { 0, 0 },
    { 0xd75532bd, 61 },
    { 0xdcf532be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60532c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed532c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218532d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7532d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff532db, 33 },
    { 0x64d532dc, 46 },
    { 0x676532dd, 91 },
    { 0x729532db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5532e2, 66 },
    { 0x60e532e3, 93 },
    { 0xb86532e4, 33 },
    { 0xde8532e5, 98 },
    { 0x60f532e4, 34 },
    { 0, 0 },
    { 0x671532e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x108532eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a532ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6532f2, 95 },
    { 0xd8c532f3, 67 },
    { 0xc12532f3, 61 },
    { 0xba1532f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75953304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e153309, 55 },
    { 0x57f5330a, 78 },
    { 0x7c65330b, 77 },
    { 0x72253309, 98 },
    { 0xf6c5330d, 33 },
    { 0, 0 },
    { 0xf7d5330f, 98 },
    { 0x4ef53310, 95 },
    { 0, 0 },
    { 0x1ea53312, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16153315, 93 },
    { 0xda553316, 98 },
    { 0, 0 },
    { 0xd0053318, 33 },
    { 0, 0 },
    { 0x9cc5331a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d53321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52553324, 33 },
    { 0x8b53325, 81 },
    { 0x92253326, 91 },
    { 0x31e53327, 90 },
    { 0x95353328, 34 },
    { 0x31353328, 95 },
    { 0xb0c5332a, 78 },
    { 0x9305332b, 61 },
    { 0x4ad5332c, 87 },
    { 0x9bf5332d, 84 },
    { 0, 0 },
    { 0x23a5332f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee53338, 51 },
    { 0x36153339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa5333d, 17 },
    { 0xad05333e, 34 },
    { 0, 0 },
    { 0xe7053340, 98 },
    { 0xe5153340, 93 },
    { 0xe9153341, 76 },
    { 0xb4a53343, 33 },
    { 0xa8f53342, 23 },
    { 0xfb253343, 36 },
    { 0x6e53345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda53351, 17 },
    { 0x9ab53352, 90 },
    { 0xb7653352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b5335f, 55 },
    { 0, 0 },
    { 0xa2453361, 23 },
    { 0x19f53362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48953365, 33 },
    { 0, 0 },
    { 0x2e453367, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4653370, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd053374, 98 },
    { 0x64c53375, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96753378, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd275337b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105337f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea5338e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea53393, 68 },
    { 0, 0 },
    { 0xba853395, 33 },
    { 0x26d53396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a53399, 90 },
    { 0xb0e53399, 90 },
    { 0xcb553399, 61 },
    { 0x5875339b, 43 },
    { 0xc8b5339b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb533a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be533ad, 96 },
    { 0x4e4533ae, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4533b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405533bc, 23 },
    { 0, 0 },
    { 0xa8c533be, 76 },
    { 0, 0 },
    { 0xe23533c0, 61 },
    { 0, 0 },
    { 0xd8e533c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744533c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5533ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8533ce, 52 },
    { 0, 0 },
    { 0x777533d0, 43 },
    { 0x6c1533d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9533d4, 68 },
    { 0, 0 },
    { 0xde2533d6, 23 },
    { 0x42e533d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17533db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3533e1, 95 },
    { 0x957533e2, 61 },
    { 0x3a7533e1, 76 },
    { 0, 0 },
    { 0xd53533e5, 100 },
    { 0x7af533e6, 68 },
    { 0x452533e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1533ec, 61 },
    { 0x714533ed, 95 },
    { 0x5d9533ed, 90 },
    { 0x948533ed, 23 },
    { 0, 0 },
    { 0x783533f1, 61 },
    { 0xdfd533f2, 46 },
    { 0x5d2533f3, 23 },
    { 0x3a5533f4, 23 },
    { 0xd0a533f5, 61 },
    { 0x30533f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f533f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca533fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4753402, 84 },
    { 0xf4d53402, 23 },
    { 0, 0 },
    { 0x8d53405, 78 },
    { 0, 0 },
    { 0x50d53407, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25340a, 23 },
    { 0, 0 },
    { 0x4025340c, 56 },
    { 0x1fb5340c, 76 },
    { 0x1905340d, 17 },
    { 0, 0 },
    { 0x80b53410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c953415, 93 },
    { 0x15053416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b53419, 85 },
    { 0x9d55341a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5341e, 46 },
    { 0x80d5341f, 90 },
    { 0xc6353420, 4 },
    { 0x8e553421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa53426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215342b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a5342f, 40 },
    { 0, 0 },
    { 0x27953431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa653434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5153439, 68 },
    { 0xdd95343a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505343f, 61 },
    { 0xc6c53440, 95 },
    { 0xf0653440, 95 },
    { 0xfa253441, 95 },
    { 0x16c53441, 90 },
    { 0xcf053444, 23 },
    { 0x3d53445, 98 },
    { 0x3c553446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f53449, 84 },
    { 0, 0 },
    { 0x9915344b, 81 },
    { 0xa075344c, 55 },
    { 0, 0 },
    { 0x185344e, 98 },
    { 0x8c65344e, 46 },
    { 0x86253450, 95 },
    { 0xa2953450, 90 },
    { 0xd1f53451, 98 },
    { 0, 0 },
    { 0x653454, 34 },
    { 0xe4e53455, 76 },
    { 0, 0 },
    { 0x95e53457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953468, 61 },
    { 0, 0 },
    { 0xef35346a, 23 },
    { 0x5555346a, 90 },
    { 0, 0 },
    { 0xc9b5346d, 61 },
    { 0x1a35346e, 33 },
    { 0x2fb5346f, 46 },
    { 0x48b5346f, 98 },
    { 0xbfa53470, 85 },
    { 0, 0 },
    { 0xd2b53473, 23 },
    { 0xb8b53474, 90 },
    { 0, 0 },
    { 0x9353476, 33 },
    { 0x40b53476, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe75347a, 67 },
    { 0xd515347b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5653481, 23 },
    { 0x85953482, 76 },
    { 0x39753483, 90 },
    { 0x66b53483, 46 },
    { 0xf1353483, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65053488, 98 },
    { 0xa3f53489, 76 },
    { 0xa285348a, 55 },
    { 0x1dc5348b, 52 },
    { 0xb8f5348b, 93 },
    { 0xf05348d, 90 },
    { 0x2f5348d, 43 },
    { 0xd205348a, 98 },
    { 0xd675348d, 61 },
    { 0xf1f53491, 98 },
    { 0x34d53489, 95 },
    { 0xc553488, 61 },
    { 0, 0 },
    { 0x76953495, 93 },
    { 0, 0 },
    { 0xc5d53497, 98 },
    { 0x11153498, 69 },
    { 0x1f953498, 98 },
    { 0xf4c53498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5349f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa534a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881534a7, 33 },
    { 0x281534a7, 90 },
    { 0x3c5534a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5534b1, 43 },
    { 0x7d3534b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92534b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865534c2, 98 },
    { 0x2df534c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e534c6, 46 },
    { 0x4ed534c7, 33 },
    { 0x394534c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1534cc, 84 },
    { 0, 0 },
    { 0x640534ce, 55 },
    { 0, 0 },
    { 0x3e9534d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b534d3, 43 },
    { 0xbfb534d4, 76 },
    { 0x6c4534d5, 78 },
    { 0, 0 },
    { 0x644534d7, 85 },
    { 0x2f5534d7, 67 },
    { 0xab6534d9, 98 },
    { 0x84b534da, 36 },
    { 0, 0 },
    { 0x743534dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7534e0, 23 },
    { 0x6b1534e0, 34 },
    { 0xb6e534e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191534e6, 76 },
    { 0x3d0534e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8534f2, 36 },
    { 0xb0e534f3, 61 },
    { 0x86c534f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74534fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c53502, 96 },
    { 0, 0 },
    { 0x36053504, 78 },
    { 0x94953505, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305350a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525350d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b53510, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7853519, 93 },
    { 0xb8a53519, 33 },
    { 0xf5c53519, 69 },
    { 0, 0 },
    { 0x4e95351d, 91 },
    { 0x4b45351e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4753521, 90 },
    { 0x79a53521, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0553527, 61 },
    { 0x36d53528, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5525352b, 33 },
    { 0x2005352b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53653531, 90 },
    { 0x9ea53532, 90 },
    { 0xd0c53531, 76 },
    { 0, 0 },
    { 0xe0b53535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f53539, 98 },
    { 0x9755353a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe485353d, 77 },
    { 0xafb5353e, 90 },
    { 0xb235353d, 23 },
    { 0xb0053540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3853543, 66 },
    { 0x13e53544, 88 },
    { 0x97753544, 95 },
    { 0x28a53546, 61 },
    { 0xb6c53546, 98 },
    { 0x6fa53545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb425354b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25553553, 90 },
    { 0, 0 },
    { 0x2ab53555, 33 },
    { 0x1353556, 98 },
    { 0x1c853557, 15 },
    { 0xd5953556, 95 },
    { 0xdcb53559, 90 },
    { 0xa453559, 46 },
    { 0x9525355a, 34 },
    { 0xd0853559, 61 },
    { 0xc225355d, 95 },
    { 0xa2e5355e, 90 },
    { 0x825355f, 52 },
    { 0x20153560, 33 },
    { 0x75b5355f, 33 },
    { 0x3f553560, 61 },
    { 0x40c5355d, 90 },
    { 0xe3b53564, 55 },
    { 0x6c5355f, 91 },
    { 0xe253566, 90 },
    { 0x8d153567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1945356b, 90 },
    { 0, 0 },
    { 0x1115356d, 61 },
    { 0x3e85356e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x51453571, 33 },
    { 0x9d153572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c353579, 23 },
    { 0x64253579, 23 },
    { 0xbc75357b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e5357e, 68 },
    { 0x93a5357f, 95 },
    { 0x85953580, 90 },
    { 0, 0 },
    { 0x23053582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d153585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d53589, 23 },
    { 0x1c053589, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd53590, 55 },
    { 0, 0 },
    { 0x42553592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x39953595, 76 },
    { 0xa7c53595, 93 },
    { 0, 0 },
    { 0x84153598, 98 },
    { 0xea953599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7955359d, 23 },
    { 0xf215359e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9535a4, 95 },
    { 0, 0 },
    { 0xc97535a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45535ab, 43 },
    { 0xc7e535ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb535b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76535b6, 78 },
    { 0, 0 },
    { 0x173535b8, 33 },
    { 0x860535b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf535be, 33 },
    { 0, 0 },
    { 0xa6a535c0, 55 },
    { 0, 0 },
    { 0xb71535c2, 66 },
    { 0, 0 },
    { 0x35f535c4, 77 },
    { 0x1e6535c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f535c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958535cc, 90 },
    { 0xef2535cc, 93 },
    { 0x26a535ce, 98 },
    { 0xbdb535cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a535d3, 55 },
    { 0, 0 },
    { 0xaec535d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9535dd, 23 },
    { 0, 0 },
    { 0x423535df, 36 },
    { 0x8a1535e0, 52 },
    { 0xc61535e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36535e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3535e7, 55 },
    { 0xafd535e8, 61 },
    { 0, 0 },
    { 0x576535ea, 34 },
    { 0, 0 },
    { 0x7fc535ec, 91 },
    { 0, 0 },
    { 0xff4535ee, 34 },
    { 0x937535ef, 90 },
    { 0x1eb535ee, 46 },
    { 0xe12535f1, 95 },
    { 0x42d535ee, 66 },
    { 0xa50535f3, 55 },
    { 0, 0 },
    { 0xd04535f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc535f9, 23 },
    { 0x3b2535fa, 81 },
    { 0xa41535fa, 90 },
    { 0, 0 },
    { 0x4e7535fd, 40 },
    { 0xa16535fe, 78 },
    { 0x30b535ff, 55 },
    { 0x15753600, 95 },
    { 0x9da53601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29553604, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86653607, 90 },
    { 0x45453608, 33 },
    { 0, 0 },
    { 0x1235360a, 90 },
    { 0xa9b5360b, 43 },
    { 0x9265360a, 87 },
    { 0x16f5360d, 98 },
    { 0xb795360c, 36 },
    { 0x8cc5360f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e253613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55253616, 65 },
    { 0x9c353616, 33 },
    { 0xd853618, 52 },
    { 0x6ed53619, 78 },
    { 0xe295361a, 77 },
    { 0x96d5361b, 55 },
    { 0xfa35361c, 90 },
    { 0xdcc53617, 23 },
    { 0x3ab5361e, 76 },
    { 0xd855361f, 61 },
    { 0x6935361e, 23 },
    { 0x4025361f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653625, 46 },
    { 0x42053626, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34153629, 76 },
    { 0x165362a, 33 },
    { 0xca5362b, 34 },
    { 0x8815362c, 33 },
    { 0xcfc5362d, 98 },
    { 0x2ee5362c, 90 },
    { 0, 0 },
    { 0xc3f53630, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60653634, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575363a, 90 },
    { 0xbb05363a, 95 },
    { 0xb3c5363a, 33 },
    { 0x9a35363d, 55 },
    { 0, 0 },
    { 0x8765363f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7153643, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d53647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d75364b, 100 },
    { 0, 0 },
    { 0x7345364d, 43 },
    { 0x47c5364e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f653651, 67 },
    { 0x74353652, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91953657, 98 },
    { 0xcb753658, 46 },
    { 0x42e53657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c5365c, 95 },
    { 0x2795365d, 76 },
    { 0, 0 },
    { 0xe365365f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80953663, 76 },
    { 0x7be53664, 56 },
    { 0xbad53665, 98 },
    { 0x83553664, 34 },
    { 0xdbe53664, 52 },
    { 0x67f53668, 85 },
    { 0x26f53669, 95 },
    { 0xc9053666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b5366f, 96 },
    { 0x2b253670, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c53674, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc925367b, 38 },
    { 0x7fe5367c, 81 },
    { 0, 0 },
    { 0xcdb5367e, 61 },
    { 0, 0 },
    { 0xbf453680, 90 },
    { 0, 0 },
    { 0x71353682, 91 },
    { 0xa5d53683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d953686, 98 },
    { 0xb2553687, 98 },
    { 0, 0 },
    { 0x8f453689, 88 },
    { 0xa9553689, 61 },
    { 0x9a35368b, 100 },
    { 0xeae5368c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad753693, 95 },
    { 0, 0 },
    { 0xbe353695, 33 },
    { 0xb7953696, 23 },
    { 0x8e153697, 61 },
    { 0, 0 },
    { 0x11e53699, 23 },
    { 0, 0 },
    { 0x5945369b, 23 },
    { 0xe485369c, 98 },
    { 0xcbe5369b, 87 },
    { 0x8fd5369e, 33 },
    { 0, 0 },
    { 0x995536a0, 98 },
    { 0x4b536a1, 61 },
    { 0xcc8536a1, 46 },
    { 0x699536a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1536a7, 33 },
    { 0x754536a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15536ab, 76 },
    { 0, 0 },
    { 0x85b536ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766536b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb536b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x165536ba, 76 },
    { 0x692536bb, 98 },
    { 0x204536bc, 33 },
    { 0, 0 },
    { 0xcb3536be, 98 },
    { 0x325536be, 33 },
    { 0, 0 },
    { 0xf53536c1, 61 },
    { 0xe62536c2, 55 },
    { 0xb6f536c3, 91 },
    { 0x9f7536c1, 98 },
    { 0x71536c2, 55 },
    { 0x50c536c4, 43 },
    { 0xd69536c3, 100 },
    { 0x7e5536c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d536cc, 23 },
    { 0, 0 },
    { 0xb9e536ce, 34 },
    { 0xa0f536ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28536d5, 68 },
    { 0xda1536d6, 100 },
    { 0, 0 },
    { 0x254536d8, 52 },
    { 0x3a6536d9, 98 },
    { 0x40d536da, 90 },
    { 0xcab536d9, 61 },
    { 0xfc9536dc, 17 },
    { 0xc07536d9, 61 },
    { 0, 0 },
    { 0x6cc536df, 61 },
    { 0xc8c536e0, 77 },
    { 0x8a536e1, 58 },
    { 0x836536e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x976536e5, 95 },
    { 0x5bc536e5, 84 },
    { 0x442536e7, 61 },
    { 0xac4536e5, 98 },
    { 0, 0 },
    { 0x13b536ea, 55 },
    { 0x462536eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4536f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3536f6, 90 },
    { 0, 0 },
    { 0x58b536f8, 33 },
    { 0x3536f8, 90 },
    { 0xbef536fa, 99 },
    { 0, 0 },
    { 0x711536fc, 61 },
    { 0x33d536fc, 91 },
    { 0, 0 },
    { 0x835536ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e153703, 98 },
    { 0, 0 },
    { 0xaad53705, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b53708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67053710, 66 },
    { 0x34053711, 61 },
    { 0x1a253712, 23 },
    { 0xaa953713, 77 },
    { 0x15e53712, 76 },
    { 0x28b53713, 90 },
    { 0xcfb53714, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5371e, 90 },
    { 0xdf95371f, 55 },
    { 0x1ef53720, 98 },
    { 0x3253721, 23 },
    { 0xbd53720, 33 },
    { 0x4bb53723, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73253729, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5372c, 95 },
    { 0x39e5372d, 61 },
    { 0x415372e, 43 },
    { 0, 0 },
    { 0xd2a53730, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b53733, 91 },
    { 0x25153733, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e53737, 33 },
    { 0xfa253738, 34 },
    { 0x7e953738, 88 },
    { 0xa7753739, 98 },
    { 0x80653738, 90 },
    { 0, 0 },
    { 0xb6e5373d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66853742, 75 },
    { 0x42e53743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee5374b, 61 },
    { 0, 0 },
    { 0x92c5374d, 43 },
    { 0xbba5374e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd53754, 46 },
    { 0, 0 },
    { 0xa4e53756, 90 },
    { 0x7f453757, 55 },
    { 0x76953756, 95 },
    { 0x30753759, 90 },
    { 0x5f35375a, 55 },
    { 0xabe5375a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a53763, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37853774, 33 },
    { 0x96a53775, 95 },
    { 0x99853775, 90 },
    { 0x9a153777, 72 },
    { 0x36c53775, 55 },
    { 0x90153775, 95 },
    { 0xb1a5377a, 61 },
    { 0xea35377a, 82 },
    { 0x8d5377b, 55 },
    { 0x56253779, 90 },
    { 0, 0 },
    { 0x1b85377f, 90 },
    { 0xa8f5377f, 33 },
    { 0xac553781, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67553785, 77 },
    { 0x75553786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165378b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5378e, 34 },
    { 0x5de5378f, 46 },
    { 0x2465378e, 33 },
    { 0x14653791, 34 },
    { 0xa735378e, 85 },
    { 0, 0 },
    { 0x50b53794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8753797, 34 },
    { 0xb6053798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5379c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d537a0, 67 },
    { 0, 0 },
    { 0x9cb537a2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875537a8, 77 },
    { 0x921537a9, 23 },
    { 0xb3537a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa537af, 91 },
    { 0, 0 },
    { 0xdb2537b1, 61 },
    { 0xc79537b2, 98 },
    { 0x430537b2, 100 },
    { 0x7bf537b2, 90 },
    { 0x4ad537b3, 93 },
    { 0x6ea537b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f537b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b537bc, 90 },
    { 0x8b1537bd, 33 },
    { 0x44e537bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0537c1, 69 },
    { 0x431537c1, 98 },
    { 0, 0 },
    { 0x2f3537c4, 61 },
    { 0x285537c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b537c9, 67 },
    { 0xe02537c9, 98 },
    { 0xb35537cb, 23 },
    { 0, 0 },
    { 0xeac537cd, 61 },
    { 0x4b4537cd, 61 },
    { 0x910537cf, 90 },
    { 0, 0 },
    { 0xe34537d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd537d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31537da, 34 },
    { 0, 0 },
    { 0x2e5537dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8537e0, 33 },
    { 0xd8f537e1, 61 },
    { 0x86d537e2, 52 },
    { 0xe21537e3, 52 },
    { 0x60c537e4, 88 },
    { 0x553537e4, 33 },
    { 0xce8537e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682537eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x174537ee, 98 },
    { 0, 0 },
    { 0x550537f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464537fa, 78 },
    { 0x9ee537fb, 43 },
    { 0, 0 },
    { 0x1d537fd, 95 },
    { 0xf85537fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef53803, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b5380a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7785380d, 23 },
    { 0, 0 },
    { 0x9e65380f, 55 },
    { 0x65253810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61653814, 61 },
    { 0, 0 },
    { 0x14753816, 98 },
    { 0x62e53817, 90 },
    { 0, 0 },
    { 0xee753819, 34 },
    { 0xf1e5381a, 88 },
    { 0xd6c53819, 87 },
    { 0xce55381c, 33 },
    { 0xf5381d, 98 },
    { 0x5265381e, 33 },
    { 0x9405381d, 91 },
    { 0x7753820, 66 },
    { 0x51c53821, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8453826, 33 },
    { 0x87f53826, 98 },
    { 0xd9e53827, 90 },
    { 0x48f53829, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e25382c, 87 },
    { 0, 0 },
    { 0xc35382e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b53832, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9305383b, 99 },
    { 0x54c5383b, 33 },
    { 0xd965383b, 55 },
    { 0xe895383e, 84 },
    { 0xbf35383f, 88 },
    { 0, 0 },
    { 0x23653841, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c53845, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c5384b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a65384e, 76 },
    { 0x66c5384f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb53852, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d253855, 34 },
    { 0, 0 },
    { 0x58c53857, 34 },
    { 0, 0 },
    { 0xd3153859, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5385c, 90 },
    { 0xcc75385d, 33 },
    { 0x6a65385e, 98 },
    { 0x1c25385f, 93 },
    { 0x79e53860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9553864, 68 },
    { 0, 0 },
    { 0xcc253866, 93 },
    { 0x6dd53867, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e5386c, 66 },
    { 0x5ae5386d, 80 },
    { 0x2635386d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82653871, 33 },
    { 0x6a653871, 91 },
    { 0x1a853871, 94 },
    { 0x9b753874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93953877, 76 },
    { 0xe4853878, 61 },
    { 0xb5553878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f5387d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8753882, 61 },
    { 0, 0 },
    { 0x8e53884, 61 },
    { 0x9b153884, 38 },
    { 0xcda53886, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b5388b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e253893, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c53898, 33 },
    { 0xb9a53898, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d5389c, 33 },
    { 0, 0 },
    { 0xf6b5389e, 65 },
    { 0, 0 },
    { 0xabd538a0, 55 },
    { 0x7bf538a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15538ab, 84 },
    { 0x333538ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da538af, 66 },
    { 0xd5a538b0, 85 },
    { 0x715538af, 98 },
    { 0xc46538b1, 81 },
    { 0xa60538b2, 76 },
    { 0x592538b4, 34 },
    { 0x36e538b5, 95 },
    { 0, 0 },
    { 0x347538b7, 95 },
    { 0x359538b8, 61 },
    { 0xe1f538b7, 69 },
    { 0, 0 },
    { 0xb0f538bb, 52 },
    { 0, 0 },
    { 0x827538bd, 68 },
    { 0xa15538be, 98 },
    { 0x48e538bd, 33 },
    { 0x4ea538c0, 100 },
    { 0xb78538c1, 96 },
    { 0x6aa538c2, 85 },
    { 0xf57538bd, 96 },
    { 0, 0 },
    { 0x496538c5, 61 },
    { 0x6a6538c6, 78 },
    { 0xf82538c7, 91 },
    { 0x4ed538c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d538ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93538d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7538db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b538df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1538e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249538eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7538f8, 61 },
    { 0, 0 },
    { 0x596538fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c538fd, 39 },
    { 0x138538fd, 91 },
    { 0xed4538ff, 61 },
    { 0xdf053900, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2653903, 66 },
    { 0, 0 },
    { 0x65753905, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf653908, 53 },
    { 0, 0 },
    { 0x4905390a, 17 },
    { 0x6895390b, 100 },
    { 0xefd5390c, 90 },
    { 0x6b55390d, 33 },
    { 0x6185390d, 95 },
    { 0, 0 },
    { 0x62653910, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29353914, 66 },
    { 0, 0 },
    { 0xfa653916, 78 },
    { 0x5a853916, 33 },
    { 0xc1553917, 98 },
    { 0x72b53919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05391d, 43 },
    { 0, 0 },
    { 0xcb25391f, 64 },
    { 0xf8e53920, 33 },
    { 0x8d453920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea53924, 90 },
    { 0xc453925, 34 },
    { 0x90453926, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165392f, 99 },
    { 0, 0 },
    { 0x1cc53931, 43 },
    { 0, 0 },
    { 0xb4d53933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365393b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2285393e, 33 },
    { 0x5db5393f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5053942, 33 },
    { 0, 0 },
    { 0x5b553944, 33 },
    { 0x43053945, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35394d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d53958, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd5395c, 87 },
    { 0xf305395d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91453964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3305396c, 91 },
    { 0xacf5396d, 43 },
    { 0xc865396e, 93 },
    { 0xc25396f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5753978, 90 },
    { 0, 0 },
    { 0x9ff5397a, 61 },
    { 0x5f5397b, 34 },
    { 0x4ff5397c, 90 },
    { 0x1845397a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd53981, 17 },
    { 0, 0 },
    { 0x9d153983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e53988, 77 },
    { 0, 0 },
    { 0x61a5398a, 33 },
    { 0x2605398a, 52 },
    { 0xa915398a, 90 },
    { 0x9385398a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c53991, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af53996, 91 },
    { 0x99553997, 81 },
    { 0, 0 },
    { 0x57953999, 22 },
    { 0x29753999, 90 },
    { 0xa435399b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e65399f, 33 },
    { 0, 0 },
    { 0x68f539a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d539a7, 93 },
    { 0xe82539a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745539ad, 93 },
    { 0, 0 },
    { 0xc72539af, 98 },
    { 0x8c7539b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c539b3, 90 },
    { 0x7a4539b4, 98 },
    { 0xa1c539b4, 88 },
    { 0x397539b4, 85 },
    { 0x8ca539b5, 43 },
    { 0xe54539b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c539bf, 66 },
    { 0, 0 },
    { 0x1eb539c1, 98 },
    { 0x69e539c1, 98 },
    { 0x637539c2, 61 },
    { 0, 0 },
    { 0x771539c5, 61 },
    { 0xc44539c6, 23 },
    { 0x171539c7, 34 },
    { 0xc94539c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97539cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e539d2, 33 },
    { 0x30e539d2, 55 },
    { 0x1b3539d4, 90 },
    { 0x9ca539d5, 83 },
    { 0x935539d6, 55 },
    { 0x735539d2, 87 },
    { 0xe9b539d8, 61 },
    { 0x214539d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c539e0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a539e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19539ea, 99 },
    { 0, 0 },
    { 0x2b0539ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed539f1, 88 },
    { 0xed1539f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0539f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05539fb, 99 },
    { 0, 0 },
    { 0xfcc539fd, 61 },
    { 0xba539fd, 23 },
    { 0x1e7539fd, 93 },
    { 0x10a53a00, 52 },
    { 0x24753a00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c53a04, 84 },
    { 0xa1e53a05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c353a08, 88 },
    { 0x1f053a08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b53a0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86753a10, 88 },
    { 0, 0 },
    { 0x3c653a12, 43 },
    { 0x9ff53a12, 66 },
    { 0xbd153a12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253a1c, 90 },
    { 0, 0 },
    { 0x9953a1e, 78 },
    { 0xe0153a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2053a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb53a27, 88 },
    { 0xd5f53a27, 34 },
    { 0, 0 },
    { 0xe8553a2a, 87 },
    { 0, 0 },
    { 0x9c453a2c, 33 },
    { 0xdbd53a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17653a31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e653a35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97753a38, 90 },
    { 0xc7453a39, 98 },
    { 0x34d53a38, 23 },
    { 0, 0 },
    { 0xaa353a3c, 91 },
    { 0xec253a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7753a40, 80 },
    { 0xa8c53a41, 100 },
    { 0, 0 },
    { 0xb0a53a43, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19953a46, 90 },
    { 0x2e553a47, 95 },
    { 0, 0 },
    { 0xc5f53a49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72853a4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b53a53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2653a56, 61 },
    { 0x14c53a57, 85 },
    { 0x6b153a58, 95 },
    { 0xe6253a57, 99 },
    { 0, 0 },
    { 0x65c53a5b, 98 },
    { 0x3c953a5b, 90 },
    { 0xb7053a5c, 17 },
    { 0xc5153a5e, 90 },
    { 0xa6653a5c, 91 },
    { 0x33353a60, 61 },
    { 0x66e53a61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40553a68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb53a6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f53a70, 92 },
    { 0, 0 },
    { 0x95e53a72, 61 },
    { 0x47553a73, 98 },
    { 0xd0953a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe553a79, 66 },
    { 0xfb353a7a, 90 },
    { 0xde353a7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f53a7f, 76 },
    { 0x92153a80, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe53a85, 87 },
    { 0x16b53a85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e53a89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf353a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8253a9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27253a9d, 90 },
    { 0xa7b53a9e, 33 },
    { 0x84d53a9f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453aa6, 90 },
    { 0, 0 },
    { 0xfd753aa8, 55 },
    { 0xea253aa9, 84 },
    { 0xbf553aaa, 98 },
    { 0xf5f53aab, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d353aae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc553ab1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a53abb, 61 },
    { 0xc9e53abb, 43 },
    { 0xe8053abd, 77 },
    { 0x40f53abb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53853ac8, 95 },
    { 0xfbb53ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e53acf, 43 },
    { 0xd2253ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f753ad4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54353ad9, 95 },
    { 0xd8753ada, 23 },
    { 0, 0 },
    { 0xbd853adc, 33 },
    { 0x8c253add, 52 },
    { 0, 0 },
    { 0xdf53adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d53ae3, 84 },
    { 0, 0 },
    { 0x27353ae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8153aec, 67 },
    { 0x2f453aec, 91 },
    { 0x34253aee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc53af1, 24 },
    { 0x4fa53af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd53af6, 66 },
    { 0xbf253af7, 61 },
    { 0x38b53af8, 90 },
    { 0x7d53af6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb553afe, 55 },
    { 0x98c53aff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32853b08, 38 },
    { 0x88053b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47853b0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd53b0f, 98 },
    { 0, 0 },
    { 0xbe453b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053b14, 61 },
    { 0x60c53b15, 78 },
    { 0x4cf53b16, 66 },
    { 0x56d53b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef553b1f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39253b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e253b29, 95 },
    { 0xaa853b2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da53b2e, 61 },
    { 0xe6253b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82353b32, 90 },
    { 0xbd753b33, 43 },
    { 0xb7553b34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c253b37, 55 },
    { 0xfce53b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b53b3f, 84 },
    { 0, 0 },
    { 0x9c753b41, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd53b4a, 61 },
    { 0xd5553b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3253b4e, 100 },
    { 0, 0 },
    { 0xe0153b50, 90 },
    { 0xd6453b50, 61 },
    { 0, 0 },
    { 0xda353b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b353b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41453b5d, 95 },
    { 0xd5053b5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51453b61, 46 },
    { 0, 0 },
    { 0x47553b63, 41 },
    { 0xf8953b64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15653b6b, 100 },
    { 0x58753b6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e53b70, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92853b75, 91 },
    { 0x5bb53b75, 55 },
    { 0x88353b76, 61 },
    { 0, 0 },
    { 0x46853b79, 95 },
    { 0, 0 },
    { 0x4de53b7b, 43 },
    { 0, 0 },
    { 0x2d853b7d, 61 },
    { 0xa1d53b7d, 90 },
    { 0x7c753b7e, 93 },
    { 0xa253b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58953b84, 33 },
    { 0, 0 },
    { 0xd5953b86, 99 },
    { 0xaed53b86, 98 },
    { 0x63653b88, 61 },
    { 0, 0 },
    { 0x78353b8a, 90 },
    { 0x53353b8b, 66 },
    { 0, 0 },
    { 0xb2c53b8d, 95 },
    { 0xbb453b8d, 100 },
    { 0x11c53b8f, 88 },
    { 0, 0 },
    { 0xe1253b91, 95 },
    { 0x82b53b92, 98 },
    { 0xd6f53b92, 98 },
    { 0xfb453b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9453b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a53b9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db53ba1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x44953ba4, 93 },
    { 0x48d53ba5, 33 },
    { 0, 0 },
    { 0x33e53ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b653bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f53bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e753bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a53bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf53bc3, 67 },
    { 0, 0 },
    { 0x41b53bc5, 90 },
    { 0xd0253bc6, 76 },
    { 0x23253bc7, 61 },
    { 0x88853bc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a553bcd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a53bd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36553bd8, 33 },
    { 0, 0 },
    { 0x92553bda, 98 },
    { 0x6f953bda, 52 },
    { 0xc2153bdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e53be0, 52 },
    { 0, 0 },
    { 0x9a653be2, 23 },
    { 0xc8453be2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57153be7, 93 },
    { 0x61f53be7, 78 },
    { 0x69e53be9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc53bed, 98 },
    { 0x54453bed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb853bf3, 55 },
    { 0x87853bf3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22453bfc, 76 },
    { 0xd2453bfd, 61 },
    { 0, 0 },
    { 0x18c53bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3753c03, 98 },
    { 0xf8c53c04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4453c07, 61 },
    { 0x83553c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb653c0b, 98 },
    { 0x28053c0b, 95 },
    { 0xf8453c0c, 98 },
    { 0xf1f53c0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953c21, 61 },
    { 0x7ca53c21, 55 },
    { 0x64953c23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e53c26, 33 },
    { 0x97853c26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30053c2a, 98 },
    { 0xd7353c2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c453c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68453c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90853c3b, 34 },
    { 0xd1753c3c, 61 },
    { 0xd553c3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c53c43, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc53c4c, 77 },
    { 0x45c53c4d, 33 },
    { 0xf7f53c4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053c53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f53c58, 90 },
    { 0x94d53c58, 90 },
    { 0x73353c5a, 65 },
    { 0, 0 },
    { 0x90753c5c, 100 },
    { 0, 0 },
    { 0x28753c5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe053c64, 88 },
    { 0, 0 },
    { 0x59553c66, 98 },
    { 0x65d53c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd53c6b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8653c76, 61 },
    { 0xaab53c76, 76 },
    { 0x9a53c76, 90 },
    { 0xfbb53c78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6153c80, 33 },
    { 0xdc953c80, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb453c86, 78 },
    { 0xe6353c87, 61 },
    { 0xa5a53c86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1453c8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d953c90, 68 },
    { 0x6d253c90, 61 },
    { 0x49553c90, 91 },
    { 0x253c93, 35 },
    { 0xda853c94, 88 },
    { 0xd5653c92, 66 },
    { 0, 0 },
    { 0xdb53c97, 61 },
    { 0, 0 },
    { 0xf7753c99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd853c9d, 84 },
    { 0x76453c9e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x93853ca1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4553ca6, 98 },
    { 0xd9853ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6753cab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57853caf, 43 },
    { 0x95753cb0, 61 },
    { 0, 0 },
    { 0x40253cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c53cb7, 76 },
    { 0x9e553cb7, 61 },
    { 0x75753cb9, 76 },
    { 0x46853cba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd53cbd, 98 },
    { 0, 0 },
    { 0x9b53cbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a53cc3, 97 },
    { 0x44453cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14853cc7, 78 },
    { 0x9e053cc8, 61 },
    { 0x35653cc9, 23 },
    { 0xe5c53cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86353ccd, 33 },
    { 0, 0 },
    { 0xe1953ccf, 66 },
    { 0x79e53cd0, 84 },
    { 0, 0 },
    { 0xf7253cd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3753cd7, 23 },
    { 0, 0 },
    { 0x5a453cd9, 33 },
    { 0x75c53cda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef53ce0, 34 },
    { 0, 0 },
    { 0x7d653ce2, 99 },
    { 0x8de53ce3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca53ce9, 78 },
    { 0x4c253ce9, 90 },
    { 0x2fc53ceb, 90 },
    { 0x4c753cec, 76 },
    { 0xb3753ced, 34 },
    { 0, 0 },
    { 0xf9753cef, 34 },
    { 0, 0 },
    { 0x8a753cf1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86953cf7, 55 },
    { 0x8a653cf8, 93 },
    { 0xca53cf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4353d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f53d0d, 83 },
    { 0x9ab53d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a253d15, 95 },
    { 0xefa53d16, 90 },
    { 0x1e553d17, 43 },
    { 0x77353d18, 23 },
    { 0xfba53d19, 61 },
    { 0, 0 },
    { 0xf4b53d1b, 23 },
    { 0x8ee53d1b, 61 },
    { 0, 0 },
    { 0x5e853d1e, 33 },
    { 0, 0 },
    { 0x90b53d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc653d26, 33 },
    { 0xea553d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71453d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b253d32, 98 },
    { 0xc9953d32, 85 },
    { 0xde153d32, 76 },
    { 0, 0 },
    { 0x36d53d36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef53d3a, 23 },
    { 0, 0 },
    { 0xb8a53d3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b53d3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52853d44, 61 },
    { 0x7dd53d45, 66 },
    { 0xb153d45, 61 },
    { 0, 0 },
    { 0x53b53d48, 23 },
    { 0x62353d48, 23 },
    { 0xa8b53d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17553d4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16053d55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab53d58, 80 },
    { 0xc1353d59, 49 },
    { 0, 0 },
    { 0xbfe53d5b, 67 },
    { 0xbfa53d5c, 76 },
    { 0xac453d5b, 75 },
    { 0, 0 },
    { 0xcdc53d5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb53d62, 99 },
    { 0x32553d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66653d68, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b653d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97653d70, 23 },
    { 0xffb53d70, 33 },
    { 0x40a53d71, 34 },
    { 0, 0 },
    { 0x37553d74, 99 },
    { 0, 0 },
    { 0x5a553d76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0453d79, 46 },
    { 0x50a53d7a, 85 },
    { 0x9d253d7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49753d82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d53d89, 76 },
    { 0, 0 },
    { 0x54e53d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7953d90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c53d93, 90 },
    { 0x49653d94, 76 },
    { 0, 0 },
    { 0x48753d96, 65 },
    { 0x36953d96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1653d9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca853d9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b53da4, 72 },
    { 0x7f253da5, 98 },
    { 0xb6053da6, 95 },
    { 0xdcf53da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f53dab, 33 },
    { 0xa4f53dac, 90 },
    { 0xfeb53dac, 33 },
    { 0x90353dac, 33 },
    { 0xf0053dad, 78 },
    { 0xb4253daf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57153db4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17453dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98353dbf, 33 },
    { 0x19553dbf, 23 },
    { 0xca453dc1, 90 },
    { 0xd7e53dc2, 95 },
    { 0x20453dc1, 84 },
    { 0x80253dc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e53dc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b653dcb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99953dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c53dd2, 98 },
    { 0, 0 },
    { 0xcb953dd4, 23 },
    { 0x20653dd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50353ddb, 76 },
    { 0xa9953ddb, 99 },
    { 0x11353ddd, 100 },
    { 0x87753dde, 55 },
    { 0x5be53ddf, 61 },
    { 0, 0 },
    { 0x4e53de1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d53de4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c153de9, 67 },
    { 0x3c253dea, 55 },
    { 0, 0 },
    { 0xaca53dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f653df7, 84 },
    { 0xd7453df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b53dfb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2553e00, 33 },
    { 0x5f53e01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x69253e04, 90 },
    { 0xfbd53e05, 95 },
    { 0x27d53e06, 61 },
    { 0x10d53e06, 33 },
    { 0x25e53e06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c053e0f, 100 },
    { 0xdf153e10, 90 },
    { 0xcb053e11, 33 },
    { 0x31c53e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e53e18, 55 },
    { 0x4a053e18, 55 },
    { 0x6a153e1a, 33 },
    { 0x8af53e1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d353e1f, 90 },
    { 0xd5c53e20, 55 },
    { 0x41253e21, 17 },
    { 0xede53e22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58053e26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4053e2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab53e30, 52 },
    { 0x30353e31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf53e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba53e3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d653e40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc253e47, 33 },
    { 0x51453e48, 75 },
    { 0x25553e48, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71853e4e, 78 },
    { 0xffd53e4f, 95 },
    { 0, 0 },
    { 0xd0e53e51, 95 },
    { 0x8a453e52, 78 },
    { 0x1953e51, 23 },
    { 0xd9a53e52, 61 },
    { 0x38453e51, 23 },
    { 0x85e53e56, 17 },
    { 0xe9e53e55, 98 },
    { 0, 0 },
    { 0xf3053e59, 61 },
    { 0, 0 },
    { 0xd9553e5b, 34 },
    { 0xd5053e5c, 61 },
    { 0x24853e5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9953e63, 61 },
    { 0, 0 },
    { 0xc953e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc53e6a, 77 },
    { 0xf2453e6b, 95 },
    { 0x23b53e6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93853e6f, 95 },
    { 0, 0 },
    { 0xd4e53e71, 95 },
    { 0, 0 },
    { 0x95853e73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5853e76, 98 },
    { 0x5ef53e77, 23 },
    { 0xd1653e77, 98 },
    { 0, 0 },
    { 0x3f153e7a, 46 },
    { 0x45853e7b, 23 },
    { 0, 0 },
    { 0xf9853e7d, 99 },
    { 0x29753e7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b53e81, 55 },
    { 0, 0 },
    { 0x62a53e83, 95 },
    { 0xcbd53e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b453e8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b053e8f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66553e92, 75 },
    { 0xa853e93, 61 },
    { 0x7e153e93, 90 },
    { 0x3a753e95, 93 },
    { 0xa753e92, 99 },
    { 0, 0 },
    { 0x39a53e98, 55 },
    { 0, 0 },
    { 0x44453e9a, 23 },
    { 0xbaa53e9b, 99 },
    { 0x3e453e9c, 68 },
    { 0xbf153e9b, 98 },
    { 0x45553e9e, 61 },
    { 0x87b53e9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053ea3, 46 },
    { 0x34f53ea3, 99 },
    { 0xe4953ea5, 100 },
    { 0x3bf53ea6, 34 },
    { 0xcfc53ea7, 23 },
    { 0x17553ea6, 93 },
    { 0xe4953ea7, 84 },
    { 0xd7853eaa, 98 },
    { 0x1cc53eaa, 91 },
    { 0xe3b53eac, 95 },
    { 0x83c53eab, 55 },
    { 0, 0 },
    { 0xa6153eaf, 23 },
    { 0, 0 },
    { 0x8d753eb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48153eb7, 95 },
    { 0x64c53eb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14153ebc, 23 },
    { 0xe5d53ebc, 90 },
    { 0x69553ebd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a53ec4, 87 },
    { 0, 0 },
    { 0x26e53ec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f553ec9, 23 },
    { 0x63453eca, 61 },
    { 0, 0 },
    { 0x53953ecc, 55 },
    { 0xeb53ecc, 33 },
    { 0x99853ece, 87 },
    { 0xfeb53ecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af53ed7, 43 },
    { 0xd8f53ed7, 90 },
    { 0x63c53ed8, 61 },
    { 0, 0 },
    { 0x33353edb, 61 },
    { 0, 0 },
    { 0x48253edd, 61 },
    { 0x1bd53ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16653ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b253eec, 77 },
    { 0x99353eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2453ef0, 98 },
    { 0xf1053ef0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a53ef8, 61 },
    { 0, 0 },
    { 0x5f053efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f453efd, 24 },
    { 0, 0 },
    { 0x97e53eff, 23 },
    { 0xc0a53eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e453f03, 33 },
    { 0x4a153f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xade53f07, 23 },
    { 0xde353f08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4053f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1253f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c653f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa553f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4553f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53653f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb153f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e53f2d, 33 },
    { 0, 0 },
    { 0x6d953f2f, 61 },
    { 0x2153f30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd053f34, 91 },
    { 0x6cd53f35, 98 },
    { 0x14953f35, 99 },
    { 0, 0 },
    { 0xc253f38, 61 },
    { 0x9d553f39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d53f3c, 90 },
    { 0xa2c53f3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87153f41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83853f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x55653f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a53f51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc53f58, 90 },
    { 0, 0 },
    { 0x2f653f5a, 56 },
    { 0x3c853f5b, 61 },
    { 0xada53f5b, 33 },
    { 0, 0 },
    { 0x7c653f5e, 90 },
    { 0x1e653f5f, 90 },
    { 0xb7353f60, 33 },
    { 0x43a53f5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3053f65, 34 },
    { 0x19a53f66, 91 },
    { 0xe3653f67, 55 },
    { 0x16053f68, 34 },
    { 0, 0 },
    { 0x7853f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d753f6e, 92 },
    { 0xf6253f6e, 59 },
    { 0xfc553f6f, 90 },
    { 0x5c553f71, 61 },
    { 0x74953f71, 78 },
    { 0xccf53f72, 61 },
    { 0x7ca53f74, 43 },
    { 0x4353f74, 91 },
    { 0x27953f75, 95 },
    { 0xae953f74, 33 },
    { 0x61e53f70, 90 },
    { 0x2d053f6e, 95 },
    { 0x78753f7a, 61 },
    { 0x90453f7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d953f84, 93 },
    { 0, 0 },
    { 0xe6253f86, 64 },
    { 0x6aa53f87, 95 },
    { 0x9f753f86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb53f8c, 46 },
    { 0x51b53f8c, 61 },
    { 0x74f53f8e, 76 },
    { 0, 0 },
    { 0xde953f90, 33 },
    { 0, 0 },
    { 0x94853f92, 23 },
    { 0x62653f92, 95 },
    { 0xe8b53f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb53f97, 43 },
    { 0x78c53f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de53f9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d53f9e, 100 },
    { 0x2fc53f9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x43353fa2, 61 },
    { 0xcab53fa3, 90 },
    { 0, 0 },
    { 0x88e53fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a53fa8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8553fae, 76 },
    { 0xbc453fae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a53fb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7653fc5, 8 },
    { 0, 0 },
    { 0xcc553fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17953fcf, 61 },
    { 0xaa853fcf, 38 },
    { 0xc3053fd1, 90 },
    { 0x2d453fcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22253fd6, 61 },
    { 0x9ef53fd7, 91 },
    { 0x94b53fd8, 61 },
    { 0xa5853fd9, 33 },
    { 0x69a53fda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e53fdd, 95 },
    { 0x92453fde, 52 },
    { 0, 0 },
    { 0x6a153fe0, 23 },
    { 0, 0 },
    { 0xa6953fe2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1253feb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e653ff1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b53ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6553fff, 61 },
    { 0x1553fff, 98 },
    { 0x1d354001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c25400f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f85401d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a5402b, 88 },
    { 0x3ae5402c, 84 },
    { 0x7055402d, 86 },
    { 0xea45402e, 61 },
    { 0xc0f5402b, 84 },
    { 0x7895402c, 52 },
    { 0x97054030, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e54036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5403b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b554040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554047, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef95404c, 85 },
    { 0x3735404d, 55 },
    { 0x91c5404e, 40 },
    { 0x39c5404e, 61 },
    { 0, 0 },
    { 0x3b954051, 77 },
    { 0x62354051, 33 },
    { 0xb4654053, 46 },
    { 0x6dd54054, 85 },
    { 0x26654051, 43 },
    { 0, 0 },
    { 0x4f854057, 68 },
    { 0x36f54057, 33 },
    { 0x3e154059, 46 },
    { 0, 0 },
    { 0xe8a5405b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f754065, 61 },
    { 0x76554065, 91 },
    { 0, 0 },
    { 0x67054068, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6895406c, 90 },
    { 0xdc65406d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1354071, 75 },
    { 0xa2754071, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb154078, 97 },
    { 0, 0 },
    { 0x25a5407a, 98 },
    { 0xac55407b, 66 },
    { 0x2505407c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3775407f, 83 },
    { 0x3a54080, 52 },
    { 0x99b54081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66254086, 90 },
    { 0, 0 },
    { 0x8554088, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe905408c, 90 },
    { 0, 0 },
    { 0x2075408e, 46 },
    { 0, 0 },
    { 0x6eb54090, 93 },
    { 0xed054091, 61 },
    { 0xb3654092, 76 },
    { 0, 0 },
    { 0x8b54094, 34 },
    { 0x5e954095, 90 },
    { 0, 0 },
    { 0xf4154097, 46 },
    { 0xdf654098, 90 },
    { 0, 0 },
    { 0xaab5409a, 34 },
    { 0xebf5409b, 83 },
    { 0x2705409c, 55 },
    { 0x4b95409c, 91 },
    { 0xa125409e, 90 },
    { 0x9b25409e, 33 },
    { 0x1285409e, 53 },
    { 0x9635409e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x852540a4, 99 },
    { 0x3a5540a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d540ae, 75 },
    { 0xf7d540af, 90 },
    { 0x95f540b0, 98 },
    { 0x7f8540b1, 98 },
    { 0x5f9540b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b540b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce540bc, 90 },
    { 0xb74540bc, 84 },
    { 0x72b540bd, 91 },
    { 0xd52540bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619540c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a540ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x417540cd, 90 },
    { 0xd37540ce, 99 },
    { 0, 0 },
    { 0xaa540d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1540d5, 23 },
    { 0xf7540d6, 76 },
    { 0xfda540d7, 90 },
    { 0xcd9540d8, 61 },
    { 0, 0 },
    { 0x841540da, 55 },
    { 0x302540da, 98 },
    { 0xcf0540db, 56 },
    { 0x27b540dd, 90 },
    { 0xb27540de, 75 },
    { 0x1d4540df, 33 },
    { 0xa90540df, 33 },
    { 0, 0 },
    { 0xea3540e2, 90 },
    { 0x239540e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746540e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c540f3, 23 },
    { 0x59c540f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b540fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b654105, 33 },
    { 0xd8e54105, 93 },
    { 0xb8754107, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc645410e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34654111, 66 },
    { 0x5a354111, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb54116, 95 },
    { 0x36454117, 78 },
    { 0x61b54116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25411d, 76 },
    { 0xa925411e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b54128, 33 },
    { 0, 0 },
    { 0x11e5412a, 61 },
    { 0x9cd5412b, 91 },
    { 0x6015412c, 87 },
    { 0xd7f5412d, 84 },
    { 0x8685412e, 90 },
    { 0, 0 },
    { 0xd2454130, 90 },
    { 0xc8254131, 43 },
    { 0x72354132, 77 },
    { 0x47854133, 33 },
    { 0x72c54134, 95 },
    { 0xa8754135, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb54138, 90 },
    { 0, 0 },
    { 0xab5413a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63054141, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef854145, 55 },
    { 0x5e354146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a5414a, 52 },
    { 0, 0 },
    { 0xbc15414c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce85414f, 23 },
    { 0, 0 },
    { 0x60754151, 23 },
    { 0xe6854152, 34 },
    { 0xe8054153, 90 },
    { 0x94a54154, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73354158, 61 },
    { 0x1dd54159, 87 },
    { 0x65c5415a, 33 },
    { 0x585415a, 67 },
    { 0, 0 },
    { 0x2385415d, 84 },
    { 0, 0 },
    { 0x9ff5415f, 61 },
    { 0xbfc54160, 61 },
    { 0xd7c5415f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6354164, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a54167, 55 },
    { 0x3fb54167, 55 },
    { 0xe7154169, 75 },
    { 0x25a5416a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4854175, 95 },
    { 0x6154176, 23 },
    { 0xa8c54177, 34 },
    { 0xacb54178, 33 },
    { 0xdd854179, 46 },
    { 0x8a054176, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3395417e, 87 },
    { 0xfcd5417e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d854184, 90 },
    { 0, 0 },
    { 0xd8d54186, 95 },
    { 0xa9154187, 98 },
    { 0xc1154188, 61 },
    { 0, 0 },
    { 0x2ec5418a, 93 },
    { 0xd1a5418b, 43 },
    { 0x6645418b, 34 },
    { 0x9425418d, 61 },
    { 0x2775418b, 33 },
    { 0x1645418f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b254192, 33 },
    { 0xe2054193, 55 },
    { 0, 0 },
    { 0xa8a54195, 95 },
    { 0x5e354196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2875419b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d5419f, 94 },
    { 0, 0 },
    { 0x3c5541a1, 33 },
    { 0x733541a1, 67 },
    { 0xd84541a2, 33 },
    { 0x2a541a1, 90 },
    { 0xac0541a5, 61 },
    { 0x8db541a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0541ab, 23 },
    { 0, 0 },
    { 0xb6541ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2541b1, 61 },
    { 0x56a541b1, 23 },
    { 0x953541b3, 91 },
    { 0xab7541b3, 23 },
    { 0x298541b4, 95 },
    { 0xe2a541b6, 66 },
    { 0x4dd541b6, 64 },
    { 0xd34541b8, 83 },
    { 0, 0 },
    { 0x343541ba, 46 },
    { 0xbfb541bb, 78 },
    { 0x9ec541ba, 33 },
    { 0x198541bd, 76 },
    { 0, 0 },
    { 0x380541bf, 34 },
    { 0x721541bf, 55 },
    { 0x9e7541c1, 33 },
    { 0x8bd541c2, 93 },
    { 0xcf2541c3, 90 },
    { 0xee7541c3, 90 },
    { 0x17f541c0, 17 },
    { 0xc2c541c5, 34 },
    { 0x8dc541c4, 76 },
    { 0x23b541c8, 40 },
    { 0x261541c9, 95 },
    { 0x37e541ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e541cd, 90 },
    { 0xb82541cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334541d8, 83 },
    { 0, 0 },
    { 0x109541da, 61 },
    { 0, 0 },
    { 0xb1b541dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x725541df, 33 },
    { 0, 0 },
    { 0x881541e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf541e9, 17 },
    { 0, 0 },
    { 0x937541eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239541f1, 98 },
    { 0x17a541f1, 33 },
    { 0, 0 },
    { 0x62c541f4, 77 },
    { 0x76f541f5, 90 },
    { 0xe5f541f5, 76 },
    { 0x9da541f7, 87 },
    { 0xef6541f4, 75 },
    { 0x1a1541f9, 90 },
    { 0, 0 },
    { 0xed6541fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a541ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a54202, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe685420c, 85 },
    { 0xe785420d, 98 },
    { 0, 0 },
    { 0x83c5420f, 61 },
    { 0x6765420f, 98 },
    { 0xc225420f, 98 },
    { 0x8fb54212, 33 },
    { 0xd0754213, 33 },
    { 0x3a054213, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7554217, 91 },
    { 0, 0 },
    { 0x7d654219, 76 },
    { 0xa9254219, 23 },
    { 0x79754219, 61 },
    { 0xe845421c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde454220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37954229, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c5422c, 95 },
    { 0x1285422c, 90 },
    { 0xb6d5422e, 76 },
    { 0xe205422d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d654232, 95 },
    { 0, 0 },
    { 0x5bf54234, 52 },
    { 0, 0 },
    { 0x73054236, 76 },
    { 0, 0 },
    { 0xc354238, 55 },
    { 0, 0 },
    { 0xce35423a, 100 },
    { 0x5065423b, 56 },
    { 0x8d25423b, 91 },
    { 0, 0 },
    { 0xa6c5423e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3154241, 90 },
    { 0x14254241, 90 },
    { 0, 0 },
    { 0xf3254244, 76 },
    { 0, 0 },
    { 0x5054246, 84 },
    { 0xb8954247, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4585424b, 66 },
    { 0xc4d5424b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b54253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24154256, 88 },
    { 0, 0 },
    { 0xf6954258, 46 },
    { 0, 0 },
    { 0xd015425a, 98 },
    { 0, 0 },
    { 0xf265425c, 36 },
    { 0x37c5425c, 90 },
    { 0x98e5425c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c054265, 98 },
    { 0x8a54266, 61 },
    { 0xb8554267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1935426b, 46 },
    { 0x8a5426b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76854273, 69 },
    { 0x3a954274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f454277, 76 },
    { 0xa854277, 98 },
    { 0x9f154279, 40 },
    { 0, 0 },
    { 0xfdd5427b, 23 },
    { 0, 0 },
    { 0xf875427d, 66 },
    { 0xd3b5427e, 61 },
    { 0, 0 },
    { 0x15e54280, 43 },
    { 0x30354280, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a754285, 87 },
    { 0xa9f54286, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d54289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab754290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7854293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c54297, 55 },
    { 0, 0 },
    { 0x89454299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3605429e, 98 },
    { 0, 0 },
    { 0x39b542a0, 98 },
    { 0xff4542a1, 46 },
    { 0, 0 },
    { 0xf0c542a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684542a7, 61 },
    { 0, 0 },
    { 0xfc2542a9, 87 },
    { 0x670542a9, 95 },
    { 0x8a9542ab, 95 },
    { 0, 0 },
    { 0x61542ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9542b5, 95 },
    { 0xcbd542b6, 61 },
    { 0xc77542b7, 34 },
    { 0xd86542b8, 88 },
    { 0xdc5542b9, 66 },
    { 0xce2542ba, 76 },
    { 0xa96542bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0542c5, 100 },
    { 0, 0 },
    { 0xf67542c7, 90 },
    { 0, 0 },
    { 0xd19542c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x235542cc, 33 },
    { 0xe62542cc, 61 },
    { 0x8fd542ce, 61 },
    { 0x86542cf, 77 },
    { 0xcdb542cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17542d4, 90 },
    { 0xfb2542d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920542db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x494542de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a542e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76542ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c542f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca542f9, 66 },
    { 0, 0 },
    { 0x618542fb, 90 },
    { 0xeda542fb, 61 },
    { 0, 0 },
    { 0xf0f542fe, 81 },
    { 0xe5b542ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6654306, 76 },
    { 0x64a54306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe345430a, 33 },
    { 0x4485430a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea54310, 61 },
    { 0, 0 },
    { 0x48454312, 90 },
    { 0xb1854313, 43 },
    { 0, 0 },
    { 0x93854315, 23 },
    { 0x22f54316, 87 },
    { 0x2b54317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b5431d, 33 },
    { 0, 0 },
    { 0x9965431f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1754329, 66 },
    { 0, 0 },
    { 0xcf05432b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9535432e, 33 },
    { 0xcb25432f, 81 },
    { 0, 0 },
    { 0x7454331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd5433a, 93 },
    { 0x23e5433b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96454341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b54344, 76 },
    { 0, 0 },
    { 0x20254346, 95 },
    { 0, 0 },
    { 0x39954348, 61 },
    { 0xb4d54349, 98 },
    { 0xf035434a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca65434f, 77 },
    { 0x28b5434f, 33 },
    { 0, 0 },
    { 0xef454352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f654355, 78 },
    { 0x5c854355, 23 },
    { 0x57254355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c54363, 90 },
    { 0, 0 },
    { 0x55e54365, 46 },
    { 0x3c154366, 55 },
    { 0xf0454367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae95436c, 52 },
    { 0x4985436d, 80 },
    { 0xcb25436e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb25437b, 85 },
    { 0x3db5437c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda54386, 55 },
    { 0x9b354387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb5438a, 23 },
    { 0, 0 },
    { 0x85c5438c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90454391, 33 },
    { 0x4b754391, 95 },
    { 0x6ae54393, 46 },
    { 0x9b854394, 95 },
    { 0xb3d54395, 76 },
    { 0x18354396, 88 },
    { 0xbd554397, 98 },
    { 0x58a54396, 34 },
    { 0x4154399, 98 },
    { 0, 0 },
    { 0x5ae5439b, 84 },
    { 0xe5d5439c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c543a3, 23 },
    { 0x376543a4, 91 },
    { 0x157543a5, 43 },
    { 0x12c543a5, 40 },
    { 0xb9c543a7, 95 },
    { 0xcd1543a7, 90 },
    { 0xe88543a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292543ad, 91 },
    { 0x873543ae, 43 },
    { 0x803543af, 61 },
    { 0x1de543ae, 33 },
    { 0x696543ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec543b7, 100 },
    { 0x8d7543b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde543bb, 90 },
    { 0x2d3543bc, 33 },
    { 0x7ee543bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7543c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba543c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447543cb, 49 },
    { 0x9e5543cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1543cf, 61 },
    { 0, 0 },
    { 0x604543d1, 23 },
    { 0x5dd543d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91543d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a543dd, 76 },
    { 0, 0 },
    { 0x450543df, 90 },
    { 0, 0 },
    { 0x4da543e1, 90 },
    { 0xde8543e2, 76 },
    { 0xc19543e2, 93 },
    { 0x3e6543e1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de543e7, 98 },
    { 0, 0 },
    { 0x398543e9, 47 },
    { 0x6ad543e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89543ef, 76 },
    { 0, 0 },
    { 0x2d7543f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3543f4, 34 },
    { 0xc85543f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89543fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x735543fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c54403, 23 },
    { 0, 0 },
    { 0xd8454405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a54409, 61 },
    { 0xea354409, 33 },
    { 0x3715440b, 98 },
    { 0x5545440c, 95 },
    { 0x3825440c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5654410, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d5441e, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d454421, 23 },
    { 0xc354422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c754425, 75 },
    { 0xe9154426, 33 },
    { 0x3f054425, 38 },
    { 0x6fb54426, 98 },
    { 0x85254426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2115442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70854435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46254438, 90 },
    { 0, 0 },
    { 0x8545443a, 64 },
    { 0, 0 },
    { 0x56b5443c, 4 },
    { 0xc665443d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87854440, 87 },
    { 0xee854441, 52 },
    { 0xb9e54441, 33 },
    { 0x33a54441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5254446, 95 },
    { 0x5a654447, 75 },
    { 0x3c454448, 90 },
    { 0xa3454448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4305444c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa54450, 33 },
    { 0x51754450, 90 },
    { 0x4a754450, 12 },
    { 0, 0 },
    { 0x8dd54454, 53 },
    { 0, 0 },
    { 0x92b54456, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a54459, 48 },
    { 0x2325445a, 38 },
    { 0x7a154459, 90 },
    { 0xe195445c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e54462, 61 },
    { 0, 0 },
    { 0xc3c54464, 90 },
    { 0x5a654464, 61 },
    { 0, 0 },
    { 0xb54467, 23 },
    { 0xda354468, 43 },
    { 0xb2954467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5446e, 12 },
    { 0xe485446f, 76 },
    { 0, 0 },
    { 0x87c54471, 23 },
    { 0xd4254472, 76 },
    { 0xf8754473, 69 },
    { 0x61554474, 95 },
    { 0x60354471, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4655447a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5447f, 23 },
    { 0xc1d5447f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c54486, 38 },
    { 0, 0 },
    { 0x47154488, 33 },
    { 0xdf154489, 52 },
    { 0xee45448a, 85 },
    { 0x7b454489, 93 },
    { 0x3705448c, 46 },
    { 0x46d54488, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x78854490, 98 },
    { 0, 0 },
    { 0xc2c54492, 91 },
    { 0x3da54493, 91 },
    { 0xc8754492, 100 },
    { 0, 0 },
    { 0x62754496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a544a0, 61 },
    { 0xae7544a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66544a6, 90 },
    { 0xc5d544a7, 91 },
    { 0x90b544a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8544ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9544af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0544b4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d544b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5544bd, 38 },
    { 0x6ef544be, 65 },
    { 0xda4544bf, 76 },
    { 0xf5a544bd, 61 },
    { 0xc93544bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684544c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24544cd, 90 },
    { 0xf5e544ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47544d1, 85 },
    { 0x5c6544d2, 88 },
    { 0xf38544d3, 67 },
    { 0x2d6544d4, 53 },
    { 0xd07544d1, 23 },
    { 0x402544d4, 82 },
    { 0xc04544d3, 68 },
    { 0x564544d8, 95 },
    { 0xab0544d8, 33 },
    { 0x3f6544d8, 33 },
    { 0x52e544db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22544df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d544e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd544eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d544ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03544f3, 67 },
    { 0, 0 },
    { 0x147544f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d544fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba54500, 76 },
    { 0x43354501, 55 },
    { 0x80b54502, 90 },
    { 0x79454502, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6254506, 90 },
    { 0x60654506, 98 },
    { 0x99654508, 81 },
    { 0x76f54509, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d5450c, 90 },
    { 0xd4d5450d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f54515, 99 },
    { 0, 0 },
    { 0xe6354517, 95 },
    { 0x67c54518, 99 },
    { 0, 0 },
    { 0x3545451a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf454521, 33 },
    { 0x5aa54522, 66 },
    { 0xeaf54523, 90 },
    { 0, 0 },
    { 0xad654525, 80 },
    { 0x8be54526, 90 },
    { 0, 0 },
    { 0x74954528, 99 },
    { 0xf0754528, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac554534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b5453c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14654544, 61 },
    { 0, 0 },
    { 0xeab54546, 33 },
    { 0x98b54547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e5454a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde954554, 34 },
    { 0, 0 },
    { 0x67754556, 90 },
    { 0x7e254557, 90 },
    { 0, 0 },
    { 0x69654559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c5455c, 61 },
    { 0x7135455c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354564, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f54568, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f954570, 61 },
    { 0, 0 },
    { 0xee954572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96954577, 61 },
    { 0, 0 },
    { 0x85b54579, 76 },
    { 0x5d35457a, 90 },
    { 0x13a5457b, 52 },
    { 0xfcc5457c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5954582, 93 },
    { 0x54c54582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d5458c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99654590, 33 },
    { 0xa6654591, 33 },
    { 0x16f54591, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf45459a, 98 },
    { 0x7f55459b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711545a4, 76 },
    { 0, 0 },
    { 0x2e1545a6, 76 },
    { 0x505545a6, 33 },
    { 0x41e545a8, 68 },
    { 0x347545a9, 33 },
    { 0, 0 },
    { 0x18f545ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563545b2, 90 },
    { 0x115545b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5545bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e545c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86545d0, 33 },
    { 0x846545d1, 61 },
    { 0, 0 },
    { 0x6ed545d3, 23 },
    { 0, 0 },
    { 0x12c545d5, 93 },
    { 0x570545d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243545da, 33 },
    { 0x9ec545db, 33 },
    { 0xf0f545dc, 76 },
    { 0x4f545db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea545e3, 90 },
    { 0xcc6545e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c545e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37545ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0545f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f545f5, 33 },
    { 0xe07545f5, 95 },
    { 0x6d9545f7, 95 },
    { 0x1e2545f8, 33 },
    { 0xbf4545f6, 33 },
    { 0xfa6545fa, 23 },
    { 0x315545fb, 91 },
    { 0x545545fc, 33 },
    { 0xc01545fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e54601, 76 },
    { 0xd8a54601, 17 },
    { 0x82c54602, 92 },
    { 0x2d254601, 61 },
    { 0xcca54602, 43 },
    { 0, 0 },
    { 0xc7e54607, 67 },
    { 0xd2d54608, 33 },
    { 0x13754608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b854610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a54618, 91 },
    { 0, 0 },
    { 0x1505461a, 23 },
    { 0, 0 },
    { 0x5b25461c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe54621, 33 },
    { 0, 0 },
    { 0xc9754623, 78 },
    { 0xcdd54623, 52 },
    { 0, 0 },
    { 0x74454626, 61 },
    { 0xb8754627, 95 },
    { 0, 0 },
    { 0x5c354629, 85 },
    { 0x8ae5462a, 95 },
    { 0xf985462b, 33 },
    { 0x9f55462c, 95 },
    { 0x8345462d, 33 },
    { 0x12f5462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef54632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e254635, 2 },
    { 0, 0 },
    { 0xdaa54637, 90 },
    { 0, 0 },
    { 0xcb154639, 23 },
    { 0, 0 },
    { 0x8e05463b, 81 },
    { 0, 0 },
    { 0x18d5463d, 23 },
    { 0, 0 },
    { 0xffb5463f, 23 },
    { 0x2c154640, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854645, 90 },
    { 0x70154645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90354649, 33 },
    { 0xc5d5464a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5464d, 66 },
    { 0x2b35464d, 56 },
    { 0xec25464d, 95 },
    { 0x48c5464e, 33 },
    { 0x97e5464e, 23 },
    { 0, 0 },
    { 0xe5e54653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b35465d, 33 },
    { 0x8645465e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6154663, 98 },
    { 0, 0 },
    { 0xeae54665, 95 },
    { 0x2e754665, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575466c, 77 },
    { 0x89f5466d, 61 },
    { 0xbc5466c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a54672, 61 },
    { 0, 0 },
    { 0xde554674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd54678, 33 },
    { 0, 0 },
    { 0xd2b5467a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de5467d, 90 },
    { 0x75c5467e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac75468b, 61 },
    { 0x465468c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80254695, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b54698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d5469c, 91 },
    { 0, 0 },
    { 0x2245469e, 76 },
    { 0xc6e5469e, 46 },
    { 0xbb4546a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee546a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552546ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939546b2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0546b5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x280546b8, 34 },
    { 0x720546b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4546bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0546c9, 23 },
    { 0xdc8546ca, 64 },
    { 0xe2b546ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2546cf, 88 },
    { 0x4b0546d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6546d4, 23 },
    { 0x216546d5, 33 },
    { 0x4b3546d5, 95 },
    { 0x7ee546d7, 95 },
    { 0x24a546d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x192546db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41546de, 77 },
    { 0xe4a546df, 34 },
    { 0x953546de, 61 },
    { 0x6a2546de, 98 },
    { 0xe8546e2, 76 },
    { 0x84e546e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78546e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167546ee, 40 },
    { 0, 0 },
    { 0x505546f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x710546f3, 55 },
    { 0, 0 },
    { 0xffc546f5, 76 },
    { 0, 0 },
    { 0x8a0546f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828546fb, 34 },
    { 0, 0 },
    { 0x2c0546fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc54700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed54708, 52 },
    { 0, 0 },
    { 0x5ee5470a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95954713, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x77254716, 90 },
    { 0x6fb54716, 91 },
    { 0xecf54718, 61 },
    { 0xf8d54718, 61 },
    { 0, 0 },
    { 0x76b5471b, 75 },
    { 0x3965471b, 43 },
    { 0, 0 },
    { 0x6395471e, 90 },
    { 0x23a5471e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1954726, 55 },
    { 0, 0 },
    { 0x85654728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355472b, 98 },
    { 0x56b5472c, 33 },
    { 0, 0 },
    { 0xca45472e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5954735, 23 },
    { 0xfb554736, 76 },
    { 0xcc554737, 61 },
    { 0xdb054738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a5473b, 90 },
    { 0x4db5473c, 100 },
    { 0, 0 },
    { 0x1ac5473e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c654743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a54746, 98 },
    { 0, 0 },
    { 0x41c54748, 98 },
    { 0x82b54749, 23 },
    { 0xd95474a, 78 },
    { 0x8254749, 33 },
    { 0, 0 },
    { 0xca75474d, 66 },
    { 0, 0 },
    { 0x3ad5474f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6854752, 23 },
    { 0x4ad54753, 56 },
    { 0xb9854752, 33 },
    { 0xb6a54755, 34 },
    { 0x8dc54756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345475c, 99 },
    { 0xe995475c, 67 },
    { 0x87f5475c, 61 },
    { 0, 0 },
    { 0x4e654760, 55 },
    { 0, 0 },
    { 0x5cd54762, 67 },
    { 0x9de54763, 46 },
    { 0xcfd54762, 76 },
    { 0x7c354762, 93 },
    { 0x41754764, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc75476a, 90 },
    { 0x2a35476b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86954772, 90 },
    { 0x6cf54773, 96 },
    { 0x30954772, 98 },
    { 0x12854775, 33 },
    { 0, 0 },
    { 0x8da54777, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5477b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b35478b, 95 },
    { 0xb335478c, 66 },
    { 0x19f5478c, 61 },
    { 0x2235478d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8154793, 61 },
    { 0xb3f54793, 33 },
    { 0, 0 },
    { 0x5f654796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b354799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef5479c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d5479f, 98 },
    { 0x8c85479f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x284547a3, 91 },
    { 0xd3e547a4, 98 },
    { 0, 0 },
    { 0x418547a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d547b1, 67 },
    { 0, 0 },
    { 0x10f547b3, 61 },
    { 0xb8c547b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5547b7, 95 },
    { 0xc71547b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e547bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1547c2, 91 },
    { 0xaa5547c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6547c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a547c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd547cd, 95 },
    { 0x5f2547ce, 23 },
    { 0, 0 },
    { 0xd14547d0, 43 },
    { 0, 0 },
    { 0x19e547d2, 55 },
    { 0xb76547d3, 90 },
    { 0xadb547d4, 46 },
    { 0x2e4547d5, 84 },
    { 0x9bd547d6, 33 },
    { 0, 0 },
    { 0xb60547d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b547e2, 61 },
    { 0x8ab547e3, 55 },
    { 0x5b9547e4, 40 },
    { 0xe8547e2, 95 },
    { 0xaf5547e6, 88 },
    { 0x62c547e5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x215547ea, 61 },
    { 0xb63547ea, 33 },
    { 0x7f547ec, 90 },
    { 0x8c7547ed, 90 },
    { 0, 0 },
    { 0x103547ef, 55 },
    { 0, 0 },
    { 0x6d9547f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2547f7, 61 },
    { 0, 0 },
    { 0xaa8547f9, 76 },
    { 0, 0 },
    { 0xdb1547fb, 33 },
    { 0x58c547fc, 61 },
    { 0x746547fd, 87 },
    { 0, 0 },
    { 0x214547ff, 74 },
    { 0, 0 },
    { 0x25954801, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b54804, 90 },
    { 0, 0 },
    { 0x5bd54806, 61 },
    { 0x96454807, 33 },
    { 0xa7554807, 90 },
    { 0x89054809, 98 },
    { 0, 0 },
    { 0xabe5480b, 90 },
    { 0xd365480c, 98 },
    { 0x9d15480c, 40 },
    { 0xa6c5480e, 65 },
    { 0x30e5480f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f54818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15481b, 90 },
    { 0xa345481c, 76 },
    { 0xfce5481d, 95 },
    { 0xe365481c, 98 },
    { 0xce25481f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2254822, 33 },
    { 0, 0 },
    { 0xedb54824, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c54827, 90 },
    { 0x95154827, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45482b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6854830, 46 },
    { 0x59b54831, 33 },
    { 0xf354832, 87 },
    { 0x38854830, 84 },
    { 0, 0 },
    { 0x92754835, 76 },
    { 0, 0 },
    { 0x4db54837, 66 },
    { 0x26854837, 33 },
    { 0x37454839, 91 },
    { 0xe2954837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c5483d, 93 },
    { 0xe6e5483e, 99 },
    { 0x8c65483f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x91654842, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f5484a, 38 },
    { 0x7e25484a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815484e, 95 },
    { 0x8e25484e, 33 },
    { 0, 0 },
    { 0x62e54851, 67 },
    { 0xfd754851, 78 },
    { 0, 0 },
    { 0x16154854, 95 },
    { 0x39254854, 95 },
    { 0xd9854856, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495485b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2255485e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c054865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a54869, 23 },
    { 0x5695486a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d15486d, 23 },
    { 0, 0 },
    { 0xc405486f, 90 },
    { 0xc4854870, 99 },
    { 0, 0 },
    { 0x17154872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd754879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e75487c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5487f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c254883, 90 },
    { 0, 0 },
    { 0x97f54885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd25488a, 95 },
    { 0x9de5488b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055488f, 98 },
    { 0x6b254890, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92454898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e5489d, 66 },
    { 0x7ff5489d, 55 },
    { 0xf4c5489f, 81 },
    { 0x7f5548a0, 61 },
    { 0xa6e548a1, 33 },
    { 0x570548a2, 76 },
    { 0x780548a1, 43 },
    { 0x16f548a3, 33 },
    { 0xf23548a5, 67 },
    { 0x7d3548a3, 24 },
    { 0x899548a7, 85 },
    { 0x871548a8, 33 },
    { 0x4b7548a9, 55 },
    { 0x49b548a9, 61 },
    { 0xfa4548ab, 98 },
    { 0, 0 },
    { 0xa5d548ad, 68 },
    { 0xf06548ae, 55 },
    { 0x6d3548ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a548b7, 99 },
    { 0xab1548b7, 76 },
    { 0x27f548b9, 66 },
    { 0x88f548ba, 61 },
    { 0x25b548bb, 65 },
    { 0xf4c548bb, 34 },
    { 0, 0 },
    { 0xaec548be, 66 },
    { 0x27b548be, 95 },
    { 0x90e548be, 23 },
    { 0x6d8548c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f548c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56548d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8548d3, 85 },
    { 0x645548d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb548db, 34 },
    { 0xb2d548dc, 43 },
    { 0xbf8548db, 95 },
    { 0xa2e548de, 95 },
    { 0, 0 },
    { 0x475548e0, 33 },
    { 0xef548e1, 43 },
    { 0xdd2548e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec548e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x837548ea, 67 },
    { 0x4f4548eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x30548f3, 98 },
    { 0, 0 },
    { 0x545548f5, 90 },
    { 0x5f0548f5, 99 },
    { 0, 0 },
    { 0x399548f8, 99 },
    { 0x2ff548f9, 17 },
    { 0, 0 },
    { 0x64548fb, 85 },
    { 0x6548fc, 76 },
    { 0x3b8548fc, 93 },
    { 0xee5548fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab54902, 95 },
    { 0, 0 },
    { 0x7d754904, 91 },
    { 0x89154904, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5490a, 61 },
    { 0xc2b5490a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc5490f, 98 },
    { 0xef54910, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61254915, 95 },
    { 0x1cd54915, 43 },
    { 0xa1554917, 55 },
    { 0, 0 },
    { 0x1c854919, 95 },
    { 0, 0 },
    { 0x9175491b, 98 },
    { 0, 0 },
    { 0x1bc5491d, 52 },
    { 0xa025491d, 61 },
    { 0xe715491d, 33 },
    { 0x93c54920, 85 },
    { 0, 0 },
    { 0xf9154922, 90 },
    { 0xc3554923, 80 },
    { 0xad54923, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225492b, 98 },
    { 0xfa35492b, 34 },
    { 0xd895492d, 43 },
    { 0, 0 },
    { 0x7ab5492f, 95 },
    { 0xb125492f, 90 },
    { 0xe8054931, 98 },
    { 0xd9154932, 90 },
    { 0x4da54932, 90 },
    { 0x8c654932, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b554939, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd54948, 65 },
    { 0x19854948, 75 },
    { 0, 0 },
    { 0xeed5494b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d254950, 23 },
    { 0x4a054951, 76 },
    { 0, 0 },
    { 0x55354953, 33 },
    { 0xb6054954, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a54958, 84 },
    { 0x18354959, 23 },
    { 0x46154958, 33 },
    { 0xa585495b, 61 },
    { 0x25e5495c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4775495f, 76 },
    { 0x6c45495f, 66 },
    { 0x3a654960, 23 },
    { 0x81554962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64254965, 90 },
    { 0, 0 },
    { 0x24d54967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95496b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d054970, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35497c, 95 },
    { 0xa865497d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58754980, 52 },
    { 0, 0 },
    { 0xaaf54982, 33 },
    { 0xd8554983, 98 },
    { 0x90a54983, 55 },
    { 0xf0554985, 88 },
    { 0x16f54983, 61 },
    { 0, 0 },
    { 0x87454988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f25498b, 33 },
    { 0x6b35498c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c54993, 98 },
    { 0x52f54994, 46 },
    { 0x3b754993, 87 },
    { 0, 0 },
    { 0xd4e54997, 98 },
    { 0, 0 },
    { 0x2f54999, 23 },
    { 0, 0 },
    { 0x49c5499b, 55 },
    { 0x7db5499c, 23 },
    { 0xef35499b, 55 },
    { 0, 0 },
    { 0xd35499f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf549a2, 95 },
    { 0, 0 },
    { 0x469549a4, 33 },
    { 0x8c549a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae549a8, 23 },
    { 0, 0 },
    { 0x64c549aa, 20 },
    { 0xbe549aa, 90 },
    { 0x8fa549ab, 90 },
    { 0xfc0549ad, 55 },
    { 0xbaf549ad, 76 },
    { 0x144549af, 90 },
    { 0, 0 },
    { 0x6e9549b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc549b9, 33 },
    { 0, 0 },
    { 0xe1c549bb, 33 },
    { 0, 0 },
    { 0x596549bd, 76 },
    { 0xc5b549be, 23 },
    { 0x689549be, 74 },
    { 0x23e549c0, 55 },
    { 0xe5e549bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771549c5, 43 },
    { 0, 0 },
    { 0x1c5549c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813549cc, 34 },
    { 0x710549cc, 98 },
    { 0x867549ce, 90 },
    { 0, 0 },
    { 0xbc6549d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1549d9, 23 },
    { 0xba8549da, 76 },
    { 0x727549db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e549ea, 34 },
    { 0xe8a549eb, 33 },
    { 0xc25549ec, 98 },
    { 0xa17549ed, 33 },
    { 0xb92549ed, 33 },
    { 0x6f6549ee, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x649549f2, 23 },
    { 0xcae549f3, 98 },
    { 0xc51549f2, 76 },
    { 0x4ba549f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x120549f8, 93 },
    { 0xbc2549f8, 76 },
    { 0, 0 },
    { 0x631549fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54549ff, 95 },
    { 0x89e54a00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1354a05, 90 },
    { 0xbc454a05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97054a09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf54a0d, 85 },
    { 0xf1e54a0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x97254a11, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e354a14, 84 },
    { 0, 0 },
    { 0xb1154a16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x34554a19, 55 },
    { 0xee754a1a, 46 },
    { 0x25354a1b, 33 },
    { 0x5ae54a1b, 95 },
    { 0x90d54a1d, 99 },
    { 0xc4454a1e, 56 },
    { 0x96754a1e, 33 },
    { 0x9154a20, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56454a25, 90 },
    { 0x26154a26, 33 },
    { 0x31a54a26, 90 },
    { 0xe0b54a28, 23 },
    { 0x9f554a26, 61 },
    { 0x4554a26, 78 },
    { 0x59354a2b, 55 },
    { 0xd5554a2c, 61 },
    { 0xf9154a2d, 90 },
    { 0x30f54a2e, 55 },
    { 0xed054a2e, 61 },
    { 0, 0 },
    { 0xab254a31, 61 },
    { 0x7c654a31, 95 },
    { 0, 0 },
    { 0x2a354a34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa54a37, 61 },
    { 0xbe754a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e54a3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x36754a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f54a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb154a4e, 43 },
    { 0xea154a4f, 61 },
    { 0x73f54a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72654a54, 23 },
    { 0xebd54a55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4654a61, 76 },
    { 0x7b354a62, 94 },
    { 0x29754a62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b54a66, 34 },
    { 0xb6354a66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1954a6c, 52 },
    { 0x5f554a6d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6554a70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c854a73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2754a7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c54a7e, 98 },
    { 0, 0 },
    { 0x51654a80, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1354a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab54a8d, 61 },
    { 0xda054a8e, 90 },
    { 0xef654a8f, 96 },
    { 0, 0 },
    { 0x3ac54a91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39754a98, 90 },
    { 0xa6154a99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0054a9e, 61 },
    { 0x8c654a9f, 52 },
    { 0x93054a9f, 61 },
    { 0x37354aa1, 76 },
    { 0x76f54aa1, 61 },
    { 0xe8f54aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe54aaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a854aad, 33 },
    { 0x41b54aad, 99 },
    { 0xe0854aad, 90 },
    { 0, 0 },
    { 0xf8d54ab1, 61 },
    { 0x6b354ab2, 87 },
    { 0, 0 },
    { 0x1e254ab4, 90 },
    { 0x38e54ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c54aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44654abd, 55 },
    { 0xcaf54abd, 76 },
    { 0xa6754abd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1154ac4, 61 },
    { 0x15c54ac5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82754ad0, 75 },
    { 0x76e54ad1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b54ad5, 76 },
    { 0x61554ad5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa54ad9, 98 },
    { 0x22d54ada, 90 },
    { 0xa9054adb, 83 },
    { 0x37454adc, 61 },
    { 0xd2f54add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb54ae7, 85 },
    { 0xb6554ae8, 33 },
    { 0x91154ae7, 52 },
    { 0x3ad54aea, 66 },
    { 0x8a954aeb, 61 },
    { 0x2b354aec, 23 },
    { 0xb0754aed, 19 },
    { 0x48854ae7, 23 },
    { 0x4b954aef, 43 },
    { 0x12354ae8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e54afa, 33 },
    { 0x21854afa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5454aff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b054b05, 76 },
    { 0xcb754b06, 55 },
    { 0x74c54b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94554b12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e54b15, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a454b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a54b1f, 65 },
    { 0, 0 },
    { 0x4ca54b21, 87 },
    { 0xe7354b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e854b2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3254b2e, 74 },
    { 0xceb54b2f, 23 },
    { 0x65c54b30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb54b33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81754b38, 78 },
    { 0x92954b39, 95 },
    { 0xecc54b38, 90 },
    { 0xa7054b38, 33 },
    { 0xa2c54b38, 95 },
    { 0, 0 },
    { 0x11354b3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f254b49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa54b50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a54b55, 98 },
    { 0, 0 },
    { 0x3e54b57, 98 },
    { 0, 0 },
    { 0xe0d54b59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef54b5c, 61 },
    { 0x11054b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c254b60, 95 },
    { 0, 0 },
    { 0x6b254b62, 77 },
    { 0, 0 },
    { 0x7b554b64, 33 },
    { 0, 0 },
    { 0x88154b66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe54b6b, 33 },
    { 0x8e54b6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35554b71, 90 },
    { 0x80854b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b054b77, 43 },
    { 0, 0 },
    { 0x53d54b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b54b7e, 76 },
    { 0, 0 },
    { 0xac454b80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f54b83, 67 },
    { 0x12654b84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc54b8c, 34 },
    { 0x12154b8d, 34 },
    { 0x24c54b8d, 43 },
    { 0xfd854b8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41054b95, 61 },
    { 0, 0 },
    { 0x91254b97, 98 },
    { 0, 0 },
    { 0xd3554b99, 61 },
    { 0xc9454b9a, 66 },
    { 0x1d54b9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d54b9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c854ba6, 93 },
    { 0xe2b54ba7, 33 },
    { 0xe1a54ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90254bad, 90 },
    { 0, 0 },
    { 0x1f154baf, 87 },
    { 0xd0154bb0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c54bb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb54bba, 90 },
    { 0x4f054bbb, 80 },
    { 0, 0 },
    { 0x57654bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354bc7, 93 },
    { 0xa9654bc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4054bcb, 65 },
    { 0x12e54bcb, 23 },
    { 0x76354bcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b54bda, 61 },
    { 0, 0 },
    { 0xc4e54bdc, 90 },
    { 0, 0 },
    { 0x70254bde, 95 },
    { 0, 0 },
    { 0x4c954be0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d54be3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e54bec, 61 },
    { 0x40b54bec, 61 },
    { 0xb6654bec, 55 },
    { 0, 0 },
    { 0x354bf0, 61 },
    { 0x52c54bf0, 78 },
    { 0, 0 },
    { 0xb054bf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb954bfa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c54bfd, 76 },
    { 0, 0 },
    { 0x5de54bff, 61 },
    { 0, 0 },
    { 0x80554c01, 98 },
    { 0, 0 },
    { 0xbbc54c03, 100 },
    { 0xaa354c04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54c0b, 91 },
    { 0xb2b54c0c, 43 },
    { 0x90254c0d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5954c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87354c14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2454c1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45554c1e, 60 },
    { 0xed154c1f, 76 },
    { 0xa8354c1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb54c28, 80 },
    { 0x4f754c28, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19654c2d, 33 },
    { 0x77854c2d, 33 },
    { 0, 0 },
    { 0x74f54c30, 33 },
    { 0, 0 },
    { 0x7f954c32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6354c35, 76 },
    { 0, 0 },
    { 0x43354c37, 95 },
    { 0x81154c38, 77 },
    { 0x6b954c39, 61 },
    { 0xac754c39, 90 },
    { 0x2a54c3b, 61 },
    { 0x6ea54c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4154c3f, 95 },
    { 0x74f54c40, 98 },
    { 0x62654c40, 91 },
    { 0xe3654c40, 90 },
    { 0x3854c40, 55 },
    { 0x1fc54c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac854c4b, 90 },
    { 0xa7554c4c, 95 },
    { 0x6a254c4d, 98 },
    { 0x5c154c4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b54c51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc54c54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf54c5b, 99 },
    { 0xc554c5c, 52 },
    { 0xa2454c5c, 98 },
    { 0, 0 },
    { 0x94054c5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23554c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b54c6e, 90 },
    { 0, 0 },
    { 0x70354c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d054c75, 23 },
    { 0x5eb54c76, 79 },
    { 0, 0 },
    { 0x46c54c78, 23 },
    { 0, 0 },
    { 0x27754c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a854c80, 17 },
    { 0x6fa54c81, 33 },
    { 0, 0 },
    { 0x59154c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20954c8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30854c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c54c92, 33 },
    { 0xacd54c93, 90 },
    { 0xe9154c94, 61 },
    { 0x1f554c95, 23 },
    { 0, 0 },
    { 0x2ef54c97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd54c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac54ca0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x20254ca3, 76 },
    { 0x62154ca3, 33 },
    { 0, 0 },
    { 0xacc54ca6, 96 },
    { 0x15b54ca7, 33 },
    { 0xe6054ca7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5854cad, 68 },
    { 0, 0 },
    { 0xefd54caf, 33 },
    { 0xfb854cb0, 61 },
    { 0x4a654cb1, 98 },
    { 0x6f454caf, 90 },
    { 0xeda54cb3, 61 },
    { 0x3a454cb3, 23 },
    { 0, 0 },
    { 0x51054cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f054cba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e54cbd, 76 },
    { 0x96054cbd, 33 },
    { 0x3a854cbe, 46 },
    { 0xc2154cbf, 93 },
    { 0x4ca54cbf, 77 },
    { 0xd6b54cc2, 85 },
    { 0, 0 },
    { 0x63254cc4, 33 },
    { 0xd0354cc5, 78 },
    { 0, 0 },
    { 0x6de54cc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33454cce, 90 },
    { 0x55854ccf, 33 },
    { 0x3654cd0, 68 },
    { 0x2f854cd1, 33 },
    { 0x9a854cd0, 93 },
    { 0x96554cd3, 76 },
    { 0x5b354cd4, 95 },
    { 0, 0 },
    { 0x5f954cd6, 33 },
    { 0x6954cd7, 61 },
    { 0x73754cd8, 90 },
    { 0x21954cd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e54cdd, 98 },
    { 0x39f54cdd, 37 },
    { 0x49e54cdd, 43 },
    { 0x7ea54ce0, 97 },
    { 0x79b54ce1, 90 },
    { 0, 0 },
    { 0x47a54ce3, 46 },
    { 0xeb254ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99354ce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb554ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a954cf1, 98 },
    { 0, 0 },
    { 0x31654cf3, 33 },
    { 0x70c54cf4, 37 },
    { 0, 0 },
    { 0x9cb54cf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff54cfb, 61 },
    { 0x43f54cfc, 23 },
    { 0x50554cfd, 23 },
    { 0xb9554cfc, 77 },
    { 0, 0 },
    { 0x6af54d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d54d08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa54d0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0454d0e, 98 },
    { 0x24254d0f, 87 },
    { 0x99054d10, 61 },
    { 0, 0 },
    { 0xc7554d12, 33 },
    { 0, 0 },
    { 0xf2154d14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e254d17, 33 },
    { 0x8c354d18, 33 },
    { 0x69e54d18, 87 },
    { 0, 0 },
    { 0xd1754d1b, 99 },
    { 0xb6e54d1c, 94 },
    { 0x3e154d1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d354d20, 56 },
    { 0xded54d20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3954d25, 66 },
    { 0xaf454d25, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff54d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f54d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64454d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4154d38, 33 },
    { 0xa9054d39, 76 },
    { 0xea254d3a, 51 },
    { 0x24754d3b, 23 },
    { 0, 0 },
    { 0x6b154d3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f554d40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c54d45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc54d48, 68 },
    { 0xec154d48, 51 },
    { 0xc7154d4a, 90 },
    { 0x9d954d4b, 91 },
    { 0x22c54d4c, 23 },
    { 0, 0 },
    { 0x65d54d4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7154d51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9554d54, 98 },
    { 0xe0154d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40154d5a, 98 },
    { 0, 0 },
    { 0xebd54d5c, 98 },
    { 0, 0 },
    { 0xef554d5e, 61 },
    { 0xc6254d5f, 95 },
    { 0, 0 },
    { 0x53054d61, 64 },
    { 0, 0 },
    { 0x46654d63, 33 },
    { 0x86354d64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7954d67, 23 },
    { 0, 0 },
    { 0xf0754d69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc54d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e54d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39554d7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a854d82, 78 },
    { 0xb8354d83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe454d86, 95 },
    { 0x8a754d87, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e54d8a, 95 },
    { 0x6e554d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da54d91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6954d95, 33 },
    { 0xc5854d96, 43 },
    { 0x2da54d97, 90 },
    { 0, 0 },
    { 0xfb854d99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a054d9c, 95 },
    { 0xbf354d9c, 78 },
    { 0x5e954d9e, 67 },
    { 0x65554d9f, 98 },
    { 0x7a554d9e, 33 },
    { 0, 0 },
    { 0x18354da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c54da5, 61 },
    { 0xeb54da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54dab, 95 },
    { 0, 0 },
    { 0x56354dad, 55 },
    { 0x62f54dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7154db1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c254dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a54dc2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1854dcc, 66 },
    { 0, 0 },
    { 0xe7254dce, 66 },
    { 0xe2454dce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3154dd4, 88 },
    { 0xb3d54dd5, 61 },
    { 0x4cc54dd5, 76 },
    { 0, 0 },
    { 0x9d754dd8, 96 },
    { 0, 0 },
    { 0x17a54dda, 98 },
    { 0xa7c54dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a354ddf, 17 },
    { 0x9f54de0, 95 },
    { 0x7fd54de1, 34 },
    { 0x4ed54de2, 88 },
    { 0x46854de0, 55 },
    { 0xc8954de1, 95 },
    { 0xdb354de5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3054dec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94854df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1054df4, 76 },
    { 0, 0 },
    { 0xef954df6, 23 },
    { 0, 0 },
    { 0x1d754df8, 55 },
    { 0x2dd54df8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a54dfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d54e04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7654e07, 66 },
    { 0xff854e07, 65 },
    { 0x7a754e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64254e0e, 61 },
    { 0x5d654e0f, 33 },
    { 0x55654e0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d454e15, 43 },
    { 0, 0 },
    { 0x51754e17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd54e1b, 33 },
    { 0xc1454e1c, 75 },
    { 0x90754e1d, 61 },
    { 0x8c254e1e, 96 },
    { 0x13754e1f, 43 },
    { 0x1de54e1f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd54e27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49854e2b, 23 },
    { 0, 0 },
    { 0xeb854e2d, 23 },
    { 0x80354e2d, 98 },
    { 0x41e54e2f, 43 },
    { 0x6754e30, 98 },
    { 0x3754e31, 90 },
    { 0x30954e32, 46 },
    { 0x9c354e30, 84 },
    { 0x5e754e34, 43 },
    { 0x76954e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76754e3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x30754e3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62054e41, 33 },
    { 0, 0 },
    { 0x9b854e43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce54e46, 66 },
    { 0x80454e47, 61 },
    { 0, 0 },
    { 0x9ea54e49, 76 },
    { 0xf754e49, 36 },
    { 0x1e554e49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2354e51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x85854e54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e54e57, 95 },
    { 0x47454e58, 23 },
    { 0x19c54e58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39754e65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4854e68, 87 },
    { 0, 0 },
    { 0x24854e6a, 91 },
    { 0, 0 },
    { 0xdeb54e6c, 90 },
    { 0x8f354e6d, 90 },
    { 0x69654e6e, 61 },
    { 0x64354e6f, 84 },
    { 0x35354e6f, 91 },
    { 0x49454e71, 85 },
    { 0xbb354e72, 80 },
    { 0x3d854e72, 90 },
    { 0x88e54e74, 23 },
    { 0xe2354e74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da54e7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99754e7f, 90 },
    { 0xbe754e80, 93 },
    { 0x1ec54e80, 75 },
    { 0xa5f54e80, 85 },
    { 0xcc954e83, 33 },
    { 0x72654e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89554e8f, 61 },
    { 0xc9954e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81154e96, 33 },
    { 0, 0 },
    { 0xad754e98, 46 },
    { 0, 0 },
    { 0xfc754e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0654e9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54554ea4, 34 },
    { 0x54254ea5, 23 },
    { 0x6ec54ea6, 84 },
    { 0xafe54ea7, 81 },
    { 0x53254ea6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a54ead, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb454eb6, 33 },
    { 0, 0 },
    { 0xbea54eb8, 46 },
    { 0x6d954eb9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354ebc, 61 },
    { 0xe9f54ebd, 76 },
    { 0xbf854ebe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f54ec2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49554ec8, 23 },
    { 0x43954ec9, 46 },
    { 0x4eb54ec9, 43 },
    { 0, 0 },
    { 0x46854ecc, 65 },
    { 0x79f54ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c54ed0, 90 },
    { 0, 0 },
    { 0x83f54ed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa54ed8, 90 },
    { 0xb8954ed9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1254edf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c754ee4, 23 },
    { 0xda54ee5, 100 },
    { 0x16754ee5, 95 },
    { 0, 0 },
    { 0x6ca54ee8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b54eec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c854ef1, 90 },
    { 0x90354ef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a54ef7, 98 },
    { 0, 0 },
    { 0xfbe54ef9, 43 },
    { 0, 0 },
    { 0x17f54efb, 90 },
    { 0x4a454efc, 100 },
    { 0x18054efd, 90 },
    { 0xbd954efc, 90 },
    { 0xe6b54eff, 85 },
    { 0x61e54f00, 61 },
    { 0x7de54efd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd54f07, 23 },
    { 0xe6954f07, 43 },
    { 0x99754f09, 95 },
    { 0, 0 },
    { 0x3a254f0b, 95 },
    { 0x3ca54f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a354f10, 88 },
    { 0xe9b54f10, 43 },
    { 0, 0 },
    { 0x5f854f13, 95 },
    { 0x6c054f13, 95 },
    { 0x6ff54f13, 93 },
    { 0xe4a54f14, 90 },
    { 0x72e54f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd554f1e, 99 },
    { 0x4ec54f1f, 90 },
    { 0xbb154f20, 61 },
    { 0, 0 },
    { 0xc6954f22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64354f28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95554f2c, 98 },
    { 0xc4854f2d, 95 },
    { 0x34a54f2e, 82 },
    { 0x4c254f2f, 17 },
    { 0xcae54f30, 76 },
    { 0xa1a54f2f, 90 },
    { 0x85954f32, 55 },
    { 0xa8654f2d, 34 },
    { 0xa2454f34, 55 },
    { 0xa2f54f35, 23 },
    { 0xcba54f35, 77 },
    { 0x10f54f37, 55 },
    { 0x25754f38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae54f3c, 98 },
    { 0, 0 },
    { 0x44b54f3e, 61 },
    { 0x62f54f3e, 61 },
    { 0, 0 },
    { 0x2d954f41, 98 },
    { 0x5aa54f41, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde54f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16354f53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a54f5d, 76 },
    { 0x50754f5e, 95 },
    { 0x64b54f5f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x99154f62, 95 },
    { 0xc3c54f63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f54f67, 37 },
    { 0xf3754f68, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43954f6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c54f72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8654f7a, 69 },
    { 0x5054f7a, 34 },
    { 0xbac54f7a, 76 },
    { 0x97654f7b, 34 },
    { 0x9c954f7e, 61 },
    { 0x4cb54f7e, 98 },
    { 0, 0 },
    { 0xd0354f81, 23 },
    { 0, 0 },
    { 0x68154f83, 61 },
    { 0x7ae54f84, 75 },
    { 0, 0 },
    { 0x88954f86, 90 },
    { 0x73a54f87, 98 },
    { 0xd9754f88, 34 },
    { 0xcbf54f88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22154f8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d54f92, 61 },
    { 0x96554f92, 98 },
    { 0xa6754f94, 91 },
    { 0xda254f95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b54f9c, 55 },
    { 0x4c154f9d, 98 },
    { 0x97454f9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea54fa3, 23 },
    { 0xdb954fa4, 23 },
    { 0xdec54fa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a754fa8, 81 },
    { 0x86954fa9, 69 },
    { 0xe1e54fa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c54fae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a54fb2, 91 },
    { 0x8b154fb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47954fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee854fc0, 90 },
    { 0xb9554fc1, 90 },
    { 0, 0 },
    { 0x1e554fc3, 90 },
    { 0, 0 },
    { 0x6e654fc5, 97 },
    { 0xece54fc5, 66 },
    { 0x67b54fc5, 95 },
    { 0, 0 },
    { 0xb3454fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x42354fcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61054fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d854fd6, 90 },
    { 0, 0 },
    { 0x1c754fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab54fdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25854fdf, 33 },
    { 0, 0 },
    { 0x40754fe1, 61 },
    { 0x85654fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x37454fe5, 76 },
    { 0xaeb54fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d154feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b54fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52954ff4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49354ff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d854ffa, 33 },
    { 0xda454ffb, 95 },
    { 0xad254ffa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a55001, 23 },
    { 0x34b55002, 40 },
    { 0, 0 },
    { 0x37455004, 23 },
    { 0, 0 },
    { 0xf8055006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd85500a, 33 },
    { 0xd0c5500b, 90 },
    { 0xef25500c, 33 },
    { 0x5135500d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0855013, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c355016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435501a, 23 },
    { 0xae15501b, 23 },
    { 0, 0 },
    { 0x7a35501d, 66 },
    { 0, 0 },
    { 0xbb15501f, 77 },
    { 0xb55020, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb55026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5502b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43055031, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32055036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5503b, 87 },
    { 0xc3b5503c, 98 },
    { 0x8fc5503d, 61 },
    { 0, 0 },
    { 0xa835503f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c055042, 90 },
    { 0, 0 },
    { 0x13c55044, 100 },
    { 0xe2455045, 98 },
    { 0, 0 },
    { 0x34355047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a55051, 23 },
    { 0x24b55051, 95 },
    { 0, 0 },
    { 0xda55054, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b55058, 17 },
    { 0x79055059, 33 },
    { 0xc7355058, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c5505f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be55062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40955067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5506e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d855071, 90 },
    { 0xa4a55071, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f55077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa55081, 84 },
    { 0, 0 },
    { 0x77c55083, 52 },
    { 0x2fa55084, 23 },
    { 0x2f155085, 95 },
    { 0x16955084, 76 },
    { 0x21b55086, 61 },
    { 0x7955088, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2155508b, 33 },
    { 0, 0 },
    { 0x46b5508d, 49 },
    { 0x6b75508e, 25 },
    { 0x2db5508f, 33 },
    { 0x5d55090, 98 },
    { 0xa6255090, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97855097, 91 },
    { 0x2d955098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5509c, 46 },
    { 0xfd05509c, 46 },
    { 0x92e5509e, 23 },
    { 0, 0 },
    { 0xa89550a0, 90 },
    { 0x221550a1, 33 },
    { 0xe70550a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3550a9, 55 },
    { 0, 0 },
    { 0xcdc550ab, 95 },
    { 0, 0 },
    { 0x8a3550ad, 66 },
    { 0xa84550ae, 46 },
    { 0, 0 },
    { 0x8db550b0, 76 },
    { 0x7ad550b0, 33 },
    { 0x338550b2, 23 },
    { 0, 0 },
    { 0xb47550b4, 55 },
    { 0x72e550b5, 95 },
    { 0xf77550b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d550bc, 90 },
    { 0xd88550bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c550c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5550c3, 90 },
    { 0, 0 },
    { 0xc8550c5, 98 },
    { 0x21b550c5, 95 },
    { 0, 0 },
    { 0x243550c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26550cb, 100 },
    { 0, 0 },
    { 0x313550cd, 98 },
    { 0x305550ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9550d1, 65 },
    { 0x5fd550d2, 23 },
    { 0x686550d3, 66 },
    { 0x94d550d3, 61 },
    { 0xbc550d5, 95 },
    { 0xda1550d6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d550d9, 76 },
    { 0x234550da, 100 },
    { 0x209550db, 23 },
    { 0x67e550da, 93 },
    { 0xb8f550db, 23 },
    { 0x584550d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17550e4, 68 },
    { 0, 0 },
    { 0x197550e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe550ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e550f5, 33 },
    { 0x5f550f6, 48 },
    { 0x37a550f6, 17 },
    { 0x541550f8, 95 },
    { 0x966550f9, 23 },
    { 0x160550f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x630550fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5855101, 95 },
    { 0x8ee55102, 61 },
    { 0, 0 },
    { 0x19355104, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f5510a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51355110, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b55113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6655511c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46755123, 98 },
    { 0x86c55124, 61 },
    { 0, 0 },
    { 0x59055126, 95 },
    { 0, 0 },
    { 0x82e55128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5512b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce55130, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24355137, 90 },
    { 0x5d455137, 33 },
    { 0x60055138, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7325513c, 33 },
    { 0, 0 },
    { 0x3145513e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75755147, 90 },
    { 0x46655148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235514b, 93 },
    { 0x1c15514c, 98 },
    { 0x6585514c, 4 },
    { 0xde25514e, 61 },
    { 0, 0 },
    { 0xe9055150, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a55156, 90 },
    { 0xeb655157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa425515b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535515e, 90 },
    { 0xa2e5515e, 78 },
    { 0xfae5515f, 94 },
    { 0x81055161, 100 },
    { 0, 0 },
    { 0x94955163, 33 },
    { 0, 0 },
    { 0xb4d55165, 98 },
    { 0x28855166, 46 },
    { 0xa1355167, 75 },
    { 0x94455165, 95 },
    { 0xfdb55165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65516d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa55171, 93 },
    { 0, 0 },
    { 0x27b55173, 24 },
    { 0x35e55173, 43 },
    { 0, 0 },
    { 0xb5555176, 61 },
    { 0, 0 },
    { 0x25d55178, 76 },
    { 0x37f55179, 98 },
    { 0xbd05517a, 90 },
    { 0xe605517b, 33 },
    { 0, 0 },
    { 0xb635517d, 95 },
    { 0x2c05517e, 61 },
    { 0xd615517f, 93 },
    { 0x70655180, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb855183, 55 },
    { 0x3555184, 90 },
    { 0, 0 },
    { 0x95a55186, 61 },
    { 0x69855187, 90 },
    { 0x35c55187, 84 },
    { 0xfb355189, 90 },
    { 0x5865518a, 33 },
    { 0x5ce5518b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf485518e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de55192, 65 },
    { 0xe8555193, 78 },
    { 0x61455194, 34 },
    { 0x71355194, 33 },
    { 0x28a55196, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc5519b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95519f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20551a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4551a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720551ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c551af, 52 },
    { 0x9a7551af, 46 },
    { 0xd78551b1, 90 },
    { 0x373551b0, 56 },
    { 0x63b551b3, 92 },
    { 0x329551b4, 33 },
    { 0x502551b5, 95 },
    { 0, 0 },
    { 0x47e551b7, 58 },
    { 0x700551b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a551bd, 34 },
    { 0x923551be, 33 },
    { 0xfd2551bf, 23 },
    { 0xe78551c0, 100 },
    { 0x4ec551c1, 85 },
    { 0, 0 },
    { 0xa3e551c3, 34 },
    { 0, 0 },
    { 0xc27551c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b551c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4551ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815551d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc551d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e551da, 78 },
    { 0xf1e551db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac551de, 95 },
    { 0x3c6551df, 34 },
    { 0xa1551e0, 33 },
    { 0x4a5551df, 90 },
    { 0, 0 },
    { 0xcb4551e3, 33 },
    { 0xf3e551e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c551e8, 98 },
    { 0x43f551e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d551ec, 23 },
    { 0x192551ec, 95 },
    { 0x1fe551ee, 77 },
    { 0xd33551ef, 34 },
    { 0, 0 },
    { 0x2b0551f1, 77 },
    { 0, 0 },
    { 0x55e551f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da551f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff55202, 33 },
    { 0xdba55202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6655209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe55520d, 93 },
    { 0x3755520e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5855214, 98 },
    { 0xb6055215, 90 },
    { 0xa9355216, 90 },
    { 0x2b855217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5521a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c45521e, 90 },
    { 0xb315521f, 98 },
    { 0xf845521f, 93 },
    { 0xd7655221, 34 },
    { 0x9cc55221, 90 },
    { 0x14855222, 90 },
    { 0xd6955224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50455228, 33 },
    { 0x77955229, 93 },
    { 0x31c5522a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe55230, 90 },
    { 0x4d055231, 90 },
    { 0x4055231, 61 },
    { 0, 0 },
    { 0xa1155234, 95 },
    { 0, 0 },
    { 0x1bc55236, 93 },
    { 0x68855237, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9145523c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455246, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac55249, 33 },
    { 0x4de5524a, 93 },
    { 0x8375524a, 100 },
    { 0x6fa5524c, 78 },
    { 0x7555524a, 90 },
    { 0x62d5524e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52555251, 23 },
    { 0xd0655251, 61 },
    { 0xc9055253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97255259, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e55260, 43 },
    { 0, 0 },
    { 0x43b55262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd55268, 84 },
    { 0, 0 },
    { 0x4185526a, 90 },
    { 0xee95526b, 81 },
    { 0x5175526b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd955274, 43 },
    { 0xc4e55275, 23 },
    { 0, 0 },
    { 0xbc255277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b5527a, 17 },
    { 0x1cf5527a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a455287, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda65528b, 90 },
    { 0x5b15528c, 61 },
    { 0xe7e5528d, 98 },
    { 0x8c45528d, 90 },
    { 0xf075528f, 46 },
    { 0x5b255290, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de55293, 34 },
    { 0, 0 },
    { 0x96655295, 90 },
    { 0, 0 },
    { 0x92455297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85529b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd185529e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3552a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2552aa, 17 },
    { 0x1c5552ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a552af, 69 },
    { 0, 0 },
    { 0x523552b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f552b4, 56 },
    { 0xf65552b5, 33 },
    { 0x32c552b6, 52 },
    { 0x5a3552b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27552bc, 93 },
    { 0x24f552bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a552c3, 78 },
    { 0xe0a552c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822552cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x136552d0, 61 },
    { 0, 0 },
    { 0x235552d2, 52 },
    { 0, 0 },
    { 0x2c7552d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54552d7, 43 },
    { 0x7ba552d8, 95 },
    { 0xfb2552d8, 43 },
    { 0x720552d7, 34 },
    { 0, 0 },
    { 0x9f1552dc, 55 },
    { 0, 0 },
    { 0xd38552de, 98 },
    { 0xf52552de, 83 },
    { 0xc78552e0, 85 },
    { 0, 0 },
    { 0x695552e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb552e6, 61 },
    { 0x5e9552e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1552eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x942552ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b552f1, 92 },
    { 0, 0 },
    { 0xf7a552f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5552f7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a55308, 61 },
    { 0x1f455309, 93 },
    { 0x57a55309, 43 },
    { 0xdc155308, 75 },
    { 0x3675530c, 87 },
    { 0xd135530d, 61 },
    { 0x285530b, 98 },
    { 0x2f45530f, 98 },
    { 0x12f55310, 23 },
    { 0x6f155311, 61 },
    { 0xc0255312, 85 },
    { 0, 0 },
    { 0xf1c55314, 90 },
    { 0xf5955314, 22 },
    { 0x28f55316, 98 },
    { 0, 0 },
    { 0xa1f55318, 90 },
    { 0xfda55318, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24155321, 33 },
    { 0x57655322, 76 },
    { 0x45b55322, 98 },
    { 0xfea55324, 53 },
    { 0xbf055324, 98 },
    { 0xf6955326, 66 },
    { 0xfda55327, 90 },
    { 0x47b55321, 55 },
    { 0xd455329, 61 },
    { 0xbb35532a, 46 },
    { 0xb255328, 90 },
    { 0x8fc55328, 46 },
    { 0, 0 },
    { 0x7625532e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf155335, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56455339, 34 },
    { 0, 0 },
    { 0xe0f5533b, 95 },
    { 0, 0 },
    { 0x3eb5533d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f155340, 34 },
    { 0xff155341, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7555346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f55349, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1855352, 33 },
    { 0, 0 },
    { 0xcb355354, 23 },
    { 0x4c155354, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73755358, 78 },
    { 0x6ec55359, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad955361, 98 },
    { 0x83f55361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa155368, 33 },
    { 0x12455368, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc55376, 90 },
    { 0xc8555377, 100 },
    { 0x9c655376, 23 },
    { 0x77655378, 98 },
    { 0x1405537a, 87 },
    { 0xe5d5537a, 98 },
    { 0, 0 },
    { 0x2dc5537d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5955381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c55384, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c55387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe245538b, 65 },
    { 0x3765538c, 43 },
    { 0x9a05538c, 76 },
    { 0x1255538e, 61 },
    { 0xece5538f, 85 },
    { 0xdcc5538f, 76 },
    { 0x32b5538b, 55 },
    { 0, 0 },
    { 0xb3255393, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3155396, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e55399, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7135539d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0553a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0553a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20553a9, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07553b1, 33 },
    { 0xfd7553b2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5553b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8553b8, 40 },
    { 0x381553b9, 90 },
    { 0, 0 },
    { 0x9553bb, 95 },
    { 0, 0 },
    { 0x176553bd, 91 },
    { 0x945553bd, 34 },
    { 0xc1a553bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c553c2, 23 },
    { 0x199553c3, 90 },
    { 0xff5553c3, 33 },
    { 0x154553c4, 67 },
    { 0, 0 },
    { 0xabb553c7, 98 },
    { 0, 0 },
    { 0xd32553c9, 61 },
    { 0, 0 },
    { 0xc35553cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a553d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8553d5, 95 },
    { 0, 0 },
    { 0x233553d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee553da, 76 },
    { 0x7f4553da, 95 },
    { 0x4dc553db, 99 },
    { 0xaca553dd, 76 },
    { 0xd3e553de, 33 },
    { 0x50f553dd, 95 },
    { 0xbc4553dc, 92 },
    { 0xedf553e1, 93 },
    { 0x152553e2, 100 },
    { 0x4f9553df, 33 },
    { 0x8fc553e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f553f1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d553f4, 84 },
    { 0x10c553f5, 61 },
    { 0, 0 },
    { 0xfd4553f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b55402, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23655405, 33 },
    { 0, 0 },
    { 0xbd155407, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5540b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5555412, 33 },
    { 0, 0 },
    { 0x79d55414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5355418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5541b, 93 },
    { 0, 0 },
    { 0xc2c5541d, 36 },
    { 0, 0 },
    { 0x3be5541f, 23 },
    { 0x63955420, 88 },
    { 0x58655420, 33 },
    { 0, 0 },
    { 0x29955423, 52 },
    { 0x74f55424, 95 },
    { 0xaf355425, 23 },
    { 0xa0b55426, 17 },
    { 0x2fc55425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a5542a, 61 },
    { 0xe0b5542a, 19 },
    { 0xe75542c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd55433, 99 },
    { 0x97255434, 87 },
    { 0x24d55435, 61 },
    { 0, 0 },
    { 0xdfc55437, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e5543a, 93 },
    { 0x9635543b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f855440, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f55445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c55448, 34 },
    { 0xdf255448, 33 },
    { 0x7135544a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe5544e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a55455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6955459, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee55464, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f255467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c5546a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c155473, 33 },
    { 0x43755474, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa5547a, 76 },
    { 0xddb5547a, 90 },
    { 0xc255547c, 34 },
    { 0x6185547d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8355485, 78 },
    { 0, 0 },
    { 0xa9a55487, 61 },
    { 0x77f55488, 23 },
    { 0xf1d55489, 36 },
    { 0xf0f5548a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735548d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73555495, 90 },
    { 0, 0 },
    { 0xdaa55497, 34 },
    { 0, 0 },
    { 0x47355499, 90 },
    { 0, 0 },
    { 0x4725549b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d554b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3554b8, 33 },
    { 0xcd0554b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219554bd, 55 },
    { 0, 0 },
    { 0x45b554bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb554c4, 23 },
    { 0, 0 },
    { 0x33554c6, 34 },
    { 0xb1b554c7, 33 },
    { 0x8b0554c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f554cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588554d0, 55 },
    { 0x85e554d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8554d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0554dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441554e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e554e6, 52 },
    { 0x1ad554e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568554eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c554f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0554f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00554f7, 61 },
    { 0x61a554f8, 95 },
    { 0x66a554f9, 98 },
    { 0x4a7554f8, 23 },
    { 0xaa4554fa, 61 },
    { 0, 0 },
    { 0x3a5554fd, 46 },
    { 0x173554fe, 98 },
    { 0xf04554fe, 90 },
    { 0, 0 },
    { 0x2cc55501, 38 },
    { 0xa9555502, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f755505, 24 },
    { 0x20c55506, 33 },
    { 0xd2f55507, 34 },
    { 0x62f55505, 33 },
    { 0, 0 },
    { 0x1c45550a, 90 },
    { 0x9a25550b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63655511, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae355518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae5551c, 88 },
    { 0, 0 },
    { 0xceb5551e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd55525, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab55529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb025552d, 95 },
    { 0x6025552d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53855532, 34 },
    { 0, 0 },
    { 0xe7155534, 34 },
    { 0xca955535, 78 },
    { 0x6bc55536, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb55539, 88 },
    { 0x32c5553a, 90 },
    { 0xac65553a, 90 },
    { 0xbe75553a, 95 },
    { 0x3155553c, 98 },
    { 0xc6d5553a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41855543, 90 },
    { 0x35555544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5855548, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf65554c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f55555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755555b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a5555f, 90 },
    { 0xbcc55560, 95 },
    { 0x5625555f, 23 },
    { 0xe65555f, 61 },
    { 0x9b155562, 84 },
    { 0xd2a5555f, 90 },
    { 0, 0 },
    { 0x41755566, 34 },
    { 0, 0 },
    { 0x8b555568, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be55570, 95 },
    { 0, 0 },
    { 0x3c655572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455557e, 33 },
    { 0x8a5557f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44455588, 90 },
    { 0x35255588, 23 },
    { 0xb935558a, 93 },
    { 0x5b955589, 98 },
    { 0, 0 },
    { 0x1db5558d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea55596, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e55599, 68 },
    { 0, 0 },
    { 0x8e15559b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b5559e, 95 },
    { 0xaa5559f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761555a3, 61 },
    { 0xb11555a4, 90 },
    { 0x5a0555a3, 53 },
    { 0x2ea555a6, 76 },
    { 0xd07555a5, 90 },
    { 0, 0 },
    { 0x3f8555a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324555ad, 91 },
    { 0, 0 },
    { 0xb9c555af, 34 },
    { 0xd4f555b0, 78 },
    { 0x556555b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb555b9, 52 },
    { 0, 0 },
    { 0xdd555bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f555be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf555c2, 52 },
    { 0x79f555c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6555c9, 90 },
    { 0x3b0555c9, 33 },
    { 0x9c1555ca, 17 },
    { 0x797555cc, 95 },
    { 0, 0 },
    { 0xf13555ce, 61 },
    { 0, 0 },
    { 0x3f1555d0, 76 },
    { 0x815555d1, 90 },
    { 0, 0 },
    { 0xe69555d3, 90 },
    { 0xfd8555d4, 46 },
    { 0x508555d4, 61 },
    { 0x7b6555d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10555da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff555df, 90 },
    { 0x354555df, 90 },
    { 0x11d555e1, 95 },
    { 0xe4e555e2, 90 },
    { 0xff7555e3, 90 },
    { 0x2a4555e4, 98 },
    { 0x9b7555e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa555e8, 95 },
    { 0x3cf555e9, 98 },
    { 0, 0 },
    { 0xf30555eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888555f6, 66 },
    { 0x818555f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d555fb, 23 },
    { 0xd41555fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9555ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b55602, 76 },
    { 0xd9d55602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25055606, 98 },
    { 0xe6e55607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865560b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28755610, 23 },
    { 0xfe855611, 33 },
    { 0x6a055612, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa55615, 80 },
    { 0xb8355616, 61 },
    { 0xfaf55617, 90 },
    { 0xc8555618, 90 },
    { 0, 0 },
    { 0x98e5561a, 33 },
    { 0x45a5561b, 23 },
    { 0x44d5561b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc955620, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45562c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff55631, 69 },
    { 0, 0 },
    { 0xb3655633, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55637, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5563a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa5563d, 23 },
    { 0, 0 },
    { 0x5f5563f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc265564a, 68 },
    { 0xaba5564b, 99 },
    { 0xe5f5564c, 90 },
    { 0x9f55564d, 55 },
    { 0xa065564b, 23 },
    { 0xb235564a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b155655, 98 },
    { 0x48655656, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a55565b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96755662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a55665, 23 },
    { 0x37d55665, 46 },
    { 0xaf555667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e5566f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4255674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85567b, 66 },
    { 0xee85567c, 55 },
    { 0x7d75567d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc755680, 76 },
    { 0x66055680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6355684, 98 },
    { 0x6f255685, 76 },
    { 0, 0 },
    { 0x93755687, 33 },
    { 0xad855688, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a5568b, 78 },
    { 0, 0 },
    { 0x455568d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c55690, 43 },
    { 0, 0 },
    { 0x20055692, 23 },
    { 0xc1055693, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55698, 34 },
    { 0x7f455698, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f556a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x999556a6, 98 },
    { 0xbdf556a7, 61 },
    { 0xbc7556a8, 55 },
    { 0xcfa556a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea556ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83556b1, 76 },
    { 0xf46556b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d556b5, 66 },
    { 0, 0 },
    { 0x161556b7, 36 },
    { 0, 0 },
    { 0x9d556b9, 61 },
    { 0x933556b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30556bd, 46 },
    { 0xd42556be, 90 },
    { 0x686556bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee556c2, 98 },
    { 0, 0 },
    { 0x6f5556c4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd556cc, 23 },
    { 0xee5556cc, 81 },
    { 0, 0 },
    { 0x665556cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a556d3, 95 },
    { 0, 0 },
    { 0x7ec556d5, 90 },
    { 0x2da556d6, 84 },
    { 0x791556d7, 90 },
    { 0xdda556d8, 87 },
    { 0x69e556d9, 61 },
    { 0xa4c556da, 90 },
    { 0, 0 },
    { 0xf5c556dc, 23 },
    { 0x5d556dd, 56 },
    { 0x4db556de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5556e3, 100 },
    { 0, 0 },
    { 0x275556e5, 90 },
    { 0x9d4556e5, 43 },
    { 0x488556e7, 52 },
    { 0, 0 },
    { 0x391556e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc556ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43556ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6556f9, 98 },
    { 0x2d1556f9, 98 },
    { 0x9e6556fb, 90 },
    { 0, 0 },
    { 0x28c556fd, 61 },
    { 0x8c1556fd, 61 },
    { 0xde0556ff, 90 },
    { 0x17355700, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4555704, 99 },
    { 0x67355705, 46 },
    { 0x8c155705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f45570b, 66 },
    { 0x3ad5570b, 55 },
    { 0xa565570d, 38 },
    { 0x39b5570e, 93 },
    { 0xe7c5570f, 98 },
    { 0xc0055710, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf55714, 34 },
    { 0x4ed55715, 48 },
    { 0, 0 },
    { 0xea455717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085571c, 77 },
    { 0x9c85571c, 23 },
    { 0xba85571e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xea455721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f55724, 43 },
    { 0x1d755725, 92 },
    { 0xabc55726, 33 },
    { 0xc0f55727, 66 },
    { 0, 0 },
    { 0xed755729, 17 },
    { 0x94d5572a, 66 },
    { 0, 0 },
    { 0xde35572c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6355731, 33 },
    { 0xd3f55732, 61 },
    { 0x46455733, 46 },
    { 0xbba55734, 52 },
    { 0xc0555735, 98 },
    { 0x5d055735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40055739, 33 },
    { 0, 0 },
    { 0xc6d5573b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5355740, 49 },
    { 0xace55740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf055744, 90 },
    { 0, 0 },
    { 0x9aa55746, 95 },
    { 0xaf455747, 90 },
    { 0, 0 },
    { 0x59b55749, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3155574f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71555752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045575a, 34 },
    { 0xc235575b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095575e, 33 },
    { 0x4955575f, 90 },
    { 0x71c55760, 95 },
    { 0xf365575e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af55768, 23 },
    { 0, 0 },
    { 0x8665576a, 75 },
    { 0, 0 },
    { 0xfec5576c, 23 },
    { 0, 0 },
    { 0x6ac5576e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d55778, 98 },
    { 0, 0 },
    { 0xeaf5577a, 67 },
    { 0xef35577a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f5577e, 44 },
    { 0x7205577e, 33 },
    { 0xbbb55780, 61 },
    { 0, 0 },
    { 0xd5f55782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8555785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb955789, 61 },
    { 0x2175578a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5578d, 23 },
    { 0xf4a5578e, 33 },
    { 0x8a55578f, 91 },
    { 0x9715578f, 23 },
    { 0x73f55790, 33 },
    { 0, 0 },
    { 0x8f955793, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b755796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35579c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee557a7, 88 },
    { 0xbac557a8, 90 },
    { 0, 0 },
    { 0x62d557aa, 95 },
    { 0xca2557ab, 100 },
    { 0x4d6557ac, 61 },
    { 0, 0 },
    { 0xbd5557ae, 98 },
    { 0xda1557ae, 8 },
    { 0x383557b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217557b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6557b9, 65 },
    { 0x7d5557ba, 75 },
    { 0, 0 },
    { 0xe0c557bc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a557bf, 33 },
    { 0x36c557bf, 90 },
    { 0x47b557c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164557c6, 93 },
    { 0, 0 },
    { 0xafe557c8, 76 },
    { 0, 0 },
    { 0xab3557ca, 99 },
    { 0, 0 },
    { 0xd8b557cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14557d2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x900557d5, 91 },
    { 0x68a557d6, 87 },
    { 0x1fd557d7, 98 },
    { 0xaf1557d8, 52 },
    { 0x8f8557d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5557dc, 84 },
    { 0xd6a557dd, 42 },
    { 0xa06557dc, 75 },
    { 0xb7f557df, 81 },
    { 0x36f557e0, 34 },
    { 0x248557df, 23 },
    { 0, 0 },
    { 0x281557e3, 34 },
    { 0, 0 },
    { 0xdee557e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e557ea, 43 },
    { 0, 0 },
    { 0x183557ec, 52 },
    { 0x322557ed, 61 },
    { 0x1a6557ee, 46 },
    { 0, 0 },
    { 0xbb8557f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a557f9, 33 },
    { 0x368557f9, 43 },
    { 0, 0 },
    { 0xfe6557fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92855807, 33 },
    { 0xecd55808, 55 },
    { 0x60555809, 66 },
    { 0xf365580a, 76 },
    { 0x73555808, 46 },
    { 0x93e55809, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0455811, 85 },
    { 0, 0 },
    { 0xb9a55813, 95 },
    { 0x60155814, 61 },
    { 0xf9655815, 84 },
    { 0, 0 },
    { 0xa8755817, 93 },
    { 0x26455818, 46 },
    { 0x36955819, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f55821, 90 },
    { 0, 0 },
    { 0xd0c55823, 75 },
    { 0xb8a55823, 46 },
    { 0x97855824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76355829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5582f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68155835, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f55839, 77 },
    { 0xab155839, 46 },
    { 0x87e5583b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f5583e, 98 },
    { 0x78c5583f, 90 },
    { 0, 0 },
    { 0x3755841, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5584b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c5584e, 46 },
    { 0x6c75584f, 23 },
    { 0, 0 },
    { 0x10655851, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955854, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec65585a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c5585e, 52 },
    { 0x3035585f, 90 },
    { 0xa6655860, 33 },
    { 0x89b55861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a55864, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9925586c, 60 },
    { 0x3105586d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9555875, 23 },
    { 0x93155876, 61 },
    { 0x6fb55877, 98 },
    { 0xdfe55878, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565587e, 90 },
    { 0, 0 },
    { 0xb8855880, 90 },
    { 0x12a55881, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12755886, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc5588a, 65 },
    { 0xd7f5588b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d55890, 76 },
    { 0, 0 },
    { 0x8c555892, 77 },
    { 0, 0 },
    { 0x50155894, 34 },
    { 0x80255895, 33 },
    { 0x97e55896, 69 },
    { 0xfad55896, 46 },
    { 0xa4f55896, 98 },
    { 0x21b55895, 90 },
    { 0x19f5589a, 65 },
    { 0x5815589a, 90 },
    { 0x4685589b, 55 },
    { 0x9e15589c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96558a6, 51 },
    { 0x959558a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f558aa, 61 },
    { 0x908558ab, 93 },
    { 0, 0 },
    { 0xf0d558ad, 93 },
    { 0, 0 },
    { 0xc50558af, 34 },
    { 0, 0 },
    { 0xddd558b1, 67 },
    { 0x6ce558b2, 98 },
    { 0xee4558b3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648558ba, 95 },
    { 0x53558ba, 85 },
    { 0xc0f558bc, 34 },
    { 0x324558ba, 95 },
    { 0x2bd558be, 34 },
    { 0x9cb558bf, 95 },
    { 0xd1a558bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1558c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52558cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34558d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61558d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x467558d7, 61 },
    { 0, 0 },
    { 0x87f558d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308558e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x595558e6, 98 },
    { 0x548558e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0558ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564558f8, 55 },
    { 0x161558f9, 99 },
    { 0x947558f9, 61 },
    { 0x4558fb, 90 },
    { 0x9a8558fc, 34 },
    { 0x6b9558fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92955900, 98 },
    { 0x89955901, 33 },
    { 0xa3f55902, 90 },
    { 0x87d55900, 88 },
    { 0x6ed55903, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c55907, 90 },
    { 0, 0 },
    { 0x4a855909, 95 },
    { 0, 0 },
    { 0xe465590b, 23 },
    { 0x46c5590c, 64 },
    { 0, 0 },
    { 0x77c5590e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef55917, 75 },
    { 0x32955918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475591b, 95 },
    { 0x64a5591c, 36 },
    { 0xa05591c, 76 },
    { 0xe615591d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd555922, 76 },
    { 0x2e855923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1955926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025592a, 98 },
    { 0, 0 },
    { 0x70d5592c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36155935, 67 },
    { 0, 0 },
    { 0x41c55937, 38 },
    { 0x3a55938, 61 },
    { 0, 0 },
    { 0x6895593a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d95593f, 76 },
    { 0, 0 },
    { 0x20e55941, 66 },
    { 0xb3355942, 95 },
    { 0, 0 },
    { 0x51855944, 77 },
    { 0, 0 },
    { 0x8d455946, 95 },
    { 0x27455947, 85 },
    { 0x9655948, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b55952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2955956, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5595d, 100 },
    { 0x4865595d, 55 },
    { 0xac5595f, 95 },
    { 0xeba55960, 55 },
    { 0x9ed55961, 95 },
    { 0x7f655962, 91 },
    { 0x2215595f, 93 },
    { 0xc0955964, 91 },
    { 0xd6355965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a5596d, 99 },
    { 0xe6a5596d, 33 },
    { 0x48e5596d, 55 },
    { 0, 0 },
    { 0x52a55971, 52 },
    { 0, 0 },
    { 0x31655973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42555976, 85 },
    { 0xe7055977, 23 },
    { 0, 0 },
    { 0x4455979, 98 },
    { 0x4f75597a, 61 },
    { 0x78155979, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b55981, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d555984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90555987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625598a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5598d, 61 },
    { 0x3e75598e, 95 },
    { 0x3865598e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67455992, 61 },
    { 0, 0 },
    { 0xd8355994, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f655997, 98 },
    { 0x65e55998, 98 },
    { 0x63755999, 90 },
    { 0, 0 },
    { 0x3ab5599b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085599e, 48 },
    { 0x74f5599f, 90 },
    { 0xd705599f, 84 },
    { 0xa1d5599f, 61 },
    { 0, 0 },
    { 0xbd9559a3, 61 },
    { 0x16559a4, 76 },
    { 0xfec559a5, 98 },
    { 0xfc6559a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263559aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392559af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6559b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877559b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2559bb, 95 },
    { 0, 0 },
    { 0x810559bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64559c6, 95 },
    { 0x67d559c7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a559cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208559d0, 76 },
    { 0x30b559d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78559d4, 95 },
    { 0, 0 },
    { 0x66a559d6, 23 },
    { 0, 0 },
    { 0x75c559d8, 77 },
    { 0x7df559d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720559dc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372559e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c559e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c559f0, 76 },
    { 0x7dd559f1, 95 },
    { 0xe86559f1, 55 },
    { 0xb73559f3, 43 },
    { 0xa00559f4, 33 },
    { 0, 0 },
    { 0x695559f6, 85 },
    { 0xf6f559f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1559fa, 90 },
    { 0x83f559fb, 92 },
    { 0xa46559fc, 23 },
    { 0xa86559fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad55a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c55a03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c755a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a55a09, 78 },
    { 0, 0 },
    { 0x78c55a0b, 61 },
    { 0xda155a0b, 17 },
    { 0x61655a0b, 90 },
    { 0xafc55a0b, 46 },
    { 0x72d55a0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa55a12, 23 },
    { 0, 0 },
    { 0x1e755a14, 23 },
    { 0xe4655a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3355a19, 81 },
    { 0x7c655a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69255a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e55a24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb055a28, 88 },
    { 0xcf355a29, 68 },
    { 0, 0 },
    { 0x6f655a2b, 90 },
    { 0, 0 },
    { 0x60355a2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55a30, 95 },
    { 0x9f155a30, 96 },
    { 0, 0 },
    { 0xb6655a33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e755a36, 87 },
    { 0xa9055a37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac55a3e, 34 },
    { 0xed55a3e, 90 },
    { 0, 0 },
    { 0x3855a41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a55a46, 9 },
    { 0, 0 },
    { 0xf3355a48, 99 },
    { 0xb7455a49, 98 },
    { 0x6c555a4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9455a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf55a52, 33 },
    { 0x96b55a53, 91 },
    { 0, 0 },
    { 0x2655a55, 91 },
    { 0xd655a56, 34 },
    { 0x90355a57, 98 },
    { 0x3bf55a58, 91 },
    { 0x12055a59, 87 },
    { 0xdc955a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c155a60, 90 },
    { 0x34355a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e55a67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94555a6c, 99 },
    { 0xf5655a6c, 90 },
    { 0xc9255a6e, 61 },
    { 0, 0 },
    { 0x49b55a70, 23 },
    { 0x55e55a71, 34 },
    { 0x3df55a71, 55 },
    { 0xac555a73, 76 },
    { 0x69b55a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda55a77, 76 },
    { 0x54655a78, 33 },
    { 0x67855a77, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16055a80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa55a85, 61 },
    { 0x5f55a86, 69 },
    { 0, 0 },
    { 0x88a55a88, 61 },
    { 0, 0 },
    { 0x17055a8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb055a8d, 66 },
    { 0xb5c55a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d955a92, 33 },
    { 0, 0 },
    { 0x355a94, 100 },
    { 0xbf955a94, 95 },
    { 0x1a955a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a355a99, 65 },
    { 0x47c55a99, 56 },
    { 0, 0 },
    { 0x2cb55a9c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055aa0, 90 },
    { 0x2da55aa1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a55aa6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c555aa9, 61 },
    { 0, 0 },
    { 0xf8c55aab, 95 },
    { 0x48955aac, 23 },
    { 0, 0 },
    { 0x3fe55aae, 61 },
    { 0x72555aaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64655ab2, 52 },
    { 0, 0 },
    { 0xd7655ab4, 66 },
    { 0, 0 },
    { 0x54655ab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c55aba, 93 },
    { 0, 0 },
    { 0x99755abc, 91 },
    { 0x70055abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df55ac0, 23 },
    { 0xd4255ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0855ac6, 98 },
    { 0x90355ac7, 76 },
    { 0, 0 },
    { 0x82f55ac9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c55ad0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36455ad5, 98 },
    { 0xa155ad6, 95 },
    { 0xe3255ad5, 55 },
    { 0x71a55ad7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3255adb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea55ade, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2955ae4, 17 },
    { 0x70655ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97855aeb, 46 },
    { 0, 0 },
    { 0x33b55aed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79455af1, 88 },
    { 0x59d55af1, 56 },
    { 0x88e55af3, 61 },
    { 0xd2655af2, 75 },
    { 0x4c855af5, 87 },
    { 0xff955af3, 95 },
    { 0xa8f55af7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3855afb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44055afe, 88 },
    { 0x9fc55aff, 33 },
    { 0x4555afe, 93 },
    { 0, 0 },
    { 0x2ba55b02, 33 },
    { 0x9b755b03, 91 },
    { 0x5a455b04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce955b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d55b0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8055b1b, 28 },
    { 0xed755b1c, 33 },
    { 0xaf055b1c, 90 },
    { 0, 0 },
    { 0x57255b1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35455b23, 61 },
    { 0x92155b24, 99 },
    { 0, 0 },
    { 0xf55b26, 90 },
    { 0, 0 },
    { 0x1d755b28, 90 },
    { 0xb5255b29, 40 },
    { 0x48a55b2a, 23 },
    { 0x32b55b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57455b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b55b35, 46 },
    { 0x4c455b35, 90 },
    { 0, 0 },
    { 0x49255b38, 95 },
    { 0, 0 },
    { 0x25255b3a, 52 },
    { 0xb9855b3b, 85 },
    { 0xa0555b3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd755b40, 100 },
    { 0, 0 },
    { 0xb355b42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf55b4a, 61 },
    { 0xb6955b4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e55b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255b59, 90 },
    { 0x2b055b59, 93 },
    { 0xcb055b5a, 85 },
    { 0, 0 },
    { 0x11b55b5d, 34 },
    { 0x36255b5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41655b61, 43 },
    { 0xf1b55b62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb55b65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a155b68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8355b6b, 61 },
    { 0, 0 },
    { 0xf2355b6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb55b71, 98 },
    { 0xe1f55b71, 23 },
    { 0x71355b72, 76 },
    { 0, 0 },
    { 0x2c355b75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b255b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e55b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3455b8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb55b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a55b97, 46 },
    { 0x56155b98, 46 },
    { 0x86355b98, 55 },
    { 0, 0 },
    { 0xf3255b9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2255b9e, 93 },
    { 0, 0 },
    { 0xdc655ba0, 23 },
    { 0x7af55ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb55ba5, 19 },
    { 0, 0 },
    { 0xc4e55ba7, 33 },
    { 0xf2655ba7, 38 },
    { 0x85455ba7, 95 },
    { 0, 0 },
    { 0x59355bab, 34 },
    { 0, 0 },
    { 0x23a55bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed755bb2, 80 },
    { 0, 0 },
    { 0x2e955bb4, 90 },
    { 0x3f755bb5, 33 },
    { 0xc4655bb6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x21255bb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea55bbc, 98 },
    { 0x16d55bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac55bc0, 98 },
    { 0xdf455bc1, 95 },
    { 0xaf555bc1, 33 },
    { 0xf8555bc0, 33 },
    { 0, 0 },
    { 0x79c55bc5, 99 },
    { 0, 0 },
    { 0x8a555bc7, 55 },
    { 0, 0 },
    { 0x34b55bc9, 95 },
    { 0, 0 },
    { 0x9355bcb, 90 },
    { 0x7c055bcc, 33 },
    { 0, 0 },
    { 0x5b255bce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc455bd6, 23 },
    { 0xde055bd7, 81 },
    { 0x6d855bd8, 91 },
    { 0x9bb55bd9, 87 },
    { 0x97755bda, 70 },
    { 0x7f55bdb, 95 },
    { 0x82a55bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56555be5, 23 },
    { 0xc8d55be5, 66 },
    { 0x61955be7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb255beb, 66 },
    { 0x47955beb, 61 },
    { 0xeee55bec, 33 },
    { 0, 0 },
    { 0x2a155bef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d55bf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64855bf5, 33 },
    { 0x3f455bf6, 61 },
    { 0x26f55bf5, 33 },
    { 0xbe955bf5, 95 },
    { 0x4d155bf9, 59 },
    { 0x32355bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec55bfd, 61 },
    { 0x2e855bfe, 61 },
    { 0x655bff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9655c02, 87 },
    { 0xed555c02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59955c06, 88 },
    { 0, 0 },
    { 0xc0b55c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2355c0b, 43 },
    { 0x49255c0b, 95 },
    { 0x60b55c0b, 33 },
    { 0x66055c0b, 75 },
    { 0x85e55c0c, 52 },
    { 0x2a455c10, 46 },
    { 0x2055c11, 76 },
    { 0xbd055c12, 46 },
    { 0x87f55c13, 90 },
    { 0xe1c55c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc55c17, 93 },
    { 0xac55c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48955c1d, 93 },
    { 0xb6355c1e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x28955c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50455c24, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c55c2a, 90 },
    { 0xc6455c2a, 23 },
    { 0, 0 },
    { 0xf2255c2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97555c32, 23 },
    { 0xe5355c33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89355c39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc955c3c, 33 },
    { 0x4cb55c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e55c45, 90 },
    { 0xf3f55c46, 90 },
    { 0x26755c47, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6355c4d, 33 },
    { 0x7cb55c4e, 90 },
    { 0, 0 },
    { 0xb1555c50, 36 },
    { 0xbaa55c51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55655c62, 23 },
    { 0xcd055c62, 90 },
    { 0xaea55c64, 33 },
    { 0xda455c65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42355c6c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab55c75, 76 },
    { 0xe2555c76, 61 },
    { 0xd9f55c77, 33 },
    { 0x2ef55c78, 91 },
    { 0x20955c79, 55 },
    { 0, 0 },
    { 0x59355c7b, 61 },
    { 0, 0 },
    { 0x67555c7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55c82, 56 },
    { 0x86455c82, 90 },
    { 0, 0 },
    { 0x37855c85, 91 },
    { 0x6ad55c86, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd55c8c, 95 },
    { 0x3da55c8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055c90, 93 },
    { 0xd6555c91, 90 },
    { 0, 0 },
    { 0xc9955c93, 75 },
    { 0x5fb55c94, 95 },
    { 0, 0 },
    { 0xef655c96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa55c99, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85455c9c, 90 },
    { 0xbbc55c9d, 85 },
    { 0x93c55c9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e55ca1, 61 },
    { 0x1b155ca2, 33 },
    { 0x3af55ca3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa855caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d55cae, 51 },
    { 0x4b855caf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37355cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47255cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55cbf, 17 },
    { 0, 0 },
    { 0x36e55cc1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55cc6, 90 },
    { 0x4d855cc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f655cd1, 90 },
    { 0xdf55cd2, 90 },
    { 0x84855cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc55cd7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20755cdb, 33 },
    { 0x9e555cdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f55cec, 78 },
    { 0, 0 },
    { 0x48055cee, 93 },
    { 0xdb255cee, 95 },
    { 0x39455cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6455cfa, 90 },
    { 0x66a55cfb, 61 },
    { 0xb6955cfa, 98 },
    { 0x3f655cfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26055d01, 61 },
    { 0x6c155d02, 76 },
    { 0xf9a55d02, 83 },
    { 0x5bb55d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76855d0c, 91 },
    { 0xb2355d0c, 61 },
    { 0x56255d0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf855d11, 90 },
    { 0, 0 },
    { 0x37d55d13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955d1f, 55 },
    { 0, 0 },
    { 0xf6955d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a55d26, 66 },
    { 0x84a55d26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff55d2b, 33 },
    { 0xb055d2c, 98 },
    { 0, 0 },
    { 0x46455d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac55d32, 61 },
    { 0xbf055d33, 93 },
    { 0xbac55d34, 61 },
    { 0xb7055d35, 76 },
    { 0, 0 },
    { 0x67255d37, 52 },
    { 0x5df55d38, 55 },
    { 0xd7055d39, 34 },
    { 0, 0 },
    { 0x89455d3b, 90 },
    { 0xbc555d3c, 90 },
    { 0x8a455d3d, 33 },
    { 0xda455d3c, 90 },
    { 0x9f255d3c, 61 },
    { 0x6f855d3b, 94 },
    { 0xc6d55d3c, 76 },
    { 0xd4d55d42, 90 },
    { 0x6d255d43, 61 },
    { 0, 0 },
    { 0x30455d45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2755d4a, 95 },
    { 0x86055d4a, 78 },
    { 0, 0 },
    { 0xd7855d4d, 61 },
    { 0, 0 },
    { 0xde255d4f, 95 },
    { 0xc0f55d4f, 23 },
    { 0xf2e55d50, 23 },
    { 0x9e655d51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce55d55, 36 },
    { 0x5d255d56, 61 },
    { 0x98a55d57, 43 },
    { 0, 0 },
    { 0x8255d59, 95 },
    { 0, 0 },
    { 0x1355d5b, 90 },
    { 0xc9355d5c, 90 },
    { 0x92955d5d, 61 },
    { 0x7b055d5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055d61, 43 },
    { 0, 0 },
    { 0x3da55d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43855d66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3855d6b, 33 },
    { 0x31f55d6c, 81 },
    { 0x4fe55d6d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d55d70, 43 },
    { 0xa5955d71, 67 },
    { 0x1cf55d72, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32555d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd455d79, 67 },
    { 0xf7255d7a, 33 },
    { 0, 0 },
    { 0x95f55d7c, 77 },
    { 0x8ab55d7d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78055d81, 17 },
    { 0xf2855d82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5555d85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2355d8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20555d91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c55d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48855d97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb755da4, 90 },
    { 0x2155da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1255daa, 77 },
    { 0xaec55dab, 33 },
    { 0, 0 },
    { 0xf7a55dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f255db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe55dba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0155dc1, 43 },
    { 0x53d55dc2, 90 },
    { 0xe3b55dc1, 61 },
    { 0x12b55dc4, 95 },
    { 0x70f55dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd55dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb55dcd, 55 },
    { 0, 0 },
    { 0x70155dcf, 66 },
    { 0x5a755dd0, 76 },
    { 0xf0255dd0, 98 },
    { 0, 0 },
    { 0x26055dd3, 56 },
    { 0xe7555dd4, 93 },
    { 0xe8a55dd3, 34 },
    { 0, 0 },
    { 0x4df55dd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba55ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec455de0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b455de3, 53 },
    { 0xffe55de4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc55de7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50355df5, 43 },
    { 0x7ef55df6, 77 },
    { 0x29255df6, 23 },
    { 0x83b55df8, 61 },
    { 0x6fe55df7, 98 },
    { 0x96055dfa, 9 },
    { 0, 0 },
    { 0xc855dfc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd55dff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e455e0b, 95 },
    { 0x20355e0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c55e14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d355e18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a55e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34255e2e, 61 },
    { 0xbf155e2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd55e32, 98 },
    { 0xb6b55e33, 90 },
    { 0xa4355e33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17755e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54355e3b, 61 },
    { 0, 0 },
    { 0xf9555e3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd355e45, 61 },
    { 0xe3255e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb55e4b, 23 },
    { 0x4d955e4b, 98 },
    { 0x80c55e4d, 38 },
    { 0x55e55e4e, 90 },
    { 0x24555e4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e55e57, 67 },
    { 0x51655e57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1355e5c, 19 },
    { 0x4e155e5d, 33 },
    { 0x17155e5d, 98 },
    { 0xd6e55e5f, 95 },
    { 0x13255e5f, 95 },
    { 0xc2155e5f, 98 },
    { 0xd3355e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9855e68, 98 },
    { 0xb7755e69, 90 },
    { 0xe0255e68, 55 },
    { 0xdd055e6b, 53 },
    { 0, 0 },
    { 0x7f555e6d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d55e72, 46 },
    { 0x27655e73, 40 },
    { 0xadb55e74, 77 },
    { 0x3ff55e73, 95 },
    { 0x62d55e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c955e79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34855e80, 76 },
    { 0x8ef55e81, 98 },
    { 0x47155e82, 90 },
    { 0xff155e83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c255e87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee55e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49055e91, 33 },
    { 0x59f55e91, 48 },
    { 0xdb955e92, 75 },
    { 0xef255e94, 95 },
    { 0xe6955e95, 95 },
    { 0x4c355e95, 23 },
    { 0x3d855e95, 99 },
    { 0x28755e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63155ea0, 99 },
    { 0, 0 },
    { 0x16555ea2, 43 },
    { 0, 0 },
    { 0xbbb55ea4, 93 },
    { 0xed355ea5, 43 },
    { 0x87e55ea4, 23 },
    { 0x9c255ea7, 34 },
    { 0x98d55ea7, 52 },
    { 0xb955ea9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f55ead, 98 },
    { 0xd8355eae, 33 },
    { 0x62f55eae, 95 },
    { 0xd4655eb0, 98 },
    { 0xf9155ead, 29 },
    { 0x9a755eb2, 61 },
    { 0, 0 },
    { 0xc7f55eb4, 46 },
    { 0, 0 },
    { 0xee155eb6, 98 },
    { 0xc0d55eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26255eba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd55ec4, 61 },
    { 0x35555ec4, 23 },
    { 0xd8055ec5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a55ecf, 33 },
    { 0, 0 },
    { 0x24c55ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c55ed5, 95 },
    { 0x4755ed5, 17 },
    { 0x32a55ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc55edd, 95 },
    { 0, 0 },
    { 0x7a655edf, 90 },
    { 0xcc555ee0, 61 },
    { 0xb5655ee1, 95 },
    { 0x96255ee2, 95 },
    { 0, 0 },
    { 0x43955ee4, 99 },
    { 0x13755ee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a55ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4555eeb, 95 },
    { 0, 0 },
    { 0xbf55eed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa55ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd855ef7, 55 },
    { 0xf8655ef8, 65 },
    { 0x35a55ef7, 38 },
    { 0xcf355efa, 99 },
    { 0, 0 },
    { 0x5d55efc, 33 },
    { 0x8cc55efd, 33 },
    { 0xfd755efe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f55f06, 100 },
    { 0xe0955f06, 90 },
    { 0x3e055f08, 55 },
    { 0xc8d55f09, 23 },
    { 0x3ee55f07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955f10, 99 },
    { 0x92055f11, 33 },
    { 0, 0 },
    { 0xdaa55f13, 88 },
    { 0xe1555f13, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb55f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad55f1a, 23 },
    { 0xbe055f1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b855f21, 84 },
    { 0x8f655f22, 78 },
    { 0xd6855f22, 67 },
    { 0xb8f55f23, 91 },
    { 0x35855f25, 95 },
    { 0, 0 },
    { 0x57c55f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e55f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f55f31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f55f3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b55f41, 91 },
    { 0, 0 },
    { 0xd8655f43, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4755f4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa55f4d, 33 },
    { 0xc6055f4d, 43 },
    { 0x26355f4f, 48 },
    { 0x10255f4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72455f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce55f56, 43 },
    { 0x9055f57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72255f5d, 98 },
    { 0xca55f5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455f63, 87 },
    { 0, 0 },
    { 0xa9255f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68855f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb955f6d, 61 },
    { 0xd5455f6e, 95 },
    { 0xb7955f6f, 84 },
    { 0, 0 },
    { 0x1955f71, 48 },
    { 0x4d955f72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b455f75, 95 },
    { 0x22455f76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15755f7c, 19 },
    { 0xa8c55f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a355f81, 61 },
    { 0x24055f82, 61 },
    { 0x62855f83, 95 },
    { 0xca455f82, 43 },
    { 0xd1455f85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23755f89, 61 },
    { 0, 0 },
    { 0x1ea55f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f55f8e, 61 },
    { 0x77055f8f, 33 },
    { 0, 0 },
    { 0xf0e55f91, 23 },
    { 0xc1455f91, 85 },
    { 0, 0 },
    { 0x1ad55f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce755f97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda55f9a, 18 },
    { 0, 0 },
    { 0xb4055f9c, 23 },
    { 0, 0 },
    { 0x78855f9e, 76 },
    { 0x1a355f9f, 69 },
    { 0x4b355fa0, 93 },
    { 0, 0 },
    { 0x41755fa2, 36 },
    { 0x59b55fa3, 95 },
    { 0xa2b55fa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1555faf, 23 },
    { 0xaf555fb0, 33 },
    { 0x52f55faf, 75 },
    { 0xf8255fb2, 90 },
    { 0xbdb55fb3, 95 },
    { 0xdf055fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d55fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7655fbf, 23 },
    { 0xa5955fc0, 95 },
    { 0x51f55fc0, 92 },
    { 0, 0 },
    { 0x3b555fc3, 46 },
    { 0x5ce55fc4, 78 },
    { 0, 0 },
    { 0x83455fc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81755fcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b55fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb055fe3, 33 },
    { 0xa9555fe4, 90 },
    { 0xbdb55fe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63155fee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16755ff4, 66 },
    { 0xac355ff4, 90 },
    { 0, 0 },
    { 0xfc455ff7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab55ffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4756001, 23 },
    { 0x8ff56001, 100 },
    { 0x2356002, 33 },
    { 0xb5656004, 84 },
    { 0x73656005, 34 },
    { 0, 0 },
    { 0xbc456007, 98 },
    { 0xc0856007, 34 },
    { 0xfb756009, 93 },
    { 0xc2e5600a, 76 },
    { 0xc135600b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6775600e, 33 },
    { 0xa645600e, 33 },
    { 0xd6256010, 90 },
    { 0, 0 },
    { 0x81156012, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c56016, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb495601c, 90 },
    { 0xdd15601d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58356020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d56027, 61 },
    { 0x69556028, 2 },
    { 0x26656029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925602c, 76 },
    { 0, 0 },
    { 0xa445602e, 23 },
    { 0x1865602f, 52 },
    { 0xf1556030, 90 },
    { 0x5465602e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda656038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3956041, 23 },
    { 0x31256041, 90 },
    { 0x1d056043, 17 },
    { 0xb2056042, 90 },
    { 0x41c56045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e256049, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c15604c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc56055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e56060, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556063, 61 },
    { 0x32256064, 87 },
    { 0xf4056065, 90 },
    { 0xbe656065, 24 },
    { 0x85556067, 61 },
    { 0x6db56068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe515606c, 23 },
    { 0x8df5606d, 61 },
    { 0, 0 },
    { 0x61b5606f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x94556072, 90 },
    { 0, 0 },
    { 0xa7956074, 93 },
    { 0xaf556074, 91 },
    { 0x2fe56076, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5607b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5607f, 93 },
    { 0, 0 },
    { 0x68956081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5956084, 33 },
    { 0xa9256085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f56089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b5608e, 61 },
    { 0xf35608e, 33 },
    { 0, 0 },
    { 0xfd856091, 55 },
    { 0x97456092, 61 },
    { 0, 0 },
    { 0xd1656094, 90 },
    { 0x95956095, 90 },
    { 0x9c656096, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5609f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64560a3, 23 },
    { 0xf86560a4, 38 },
    { 0xf23560a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67560ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x77560af, 66 },
    { 0x93b560af, 85 },
    { 0xb8560b1, 90 },
    { 0x803560af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f560b6, 23 },
    { 0x9b2560b7, 43 },
    { 0xb34560b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e560bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5560c5, 67 },
    { 0x141560c6, 61 },
    { 0x584560c7, 33 },
    { 0xf95560c8, 66 },
    { 0x7af560c5, 69 },
    { 0xeda560ca, 61 },
    { 0xaed560c9, 17 },
    { 0, 0 },
    { 0x8d1560cd, 23 },
    { 0x865560ce, 55 },
    { 0x7cb560cf, 98 },
    { 0x916560ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22560da, 66 },
    { 0xf56560db, 61 },
    { 0x9af560da, 66 },
    { 0x66c560dc, 90 },
    { 0xd0560de, 23 },
    { 0xd29560df, 78 },
    { 0xb42560e0, 98 },
    { 0x2c8560e1, 23 },
    { 0x96f560e2, 23 },
    { 0x1f9560e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19560e7, 61 },
    { 0x165560e7, 95 },
    { 0x8c2560e7, 84 },
    { 0, 0 },
    { 0x30b560eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x936560ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7560f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c560fb, 84 },
    { 0x80c560fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a56100, 100 },
    { 0xb656100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc956105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c556109, 33 },
    { 0, 0 },
    { 0x2235610b, 85 },
    { 0xc85610b, 23 },
    { 0x4c25610d, 65 },
    { 0, 0 },
    { 0x5605610f, 98 },
    { 0xf1056110, 90 },
    { 0, 0 },
    { 0xea56112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8356117, 55 },
    { 0xa5c56118, 98 },
    { 0xe7856119, 23 },
    { 0xfb856119, 95 },
    { 0x9555611b, 33 },
    { 0xe705611c, 43 },
    { 0, 0 },
    { 0xa665611e, 61 },
    { 0x2fb5611f, 23 },
    { 0xbb356120, 52 },
    { 0x4915611e, 61 },
    { 0x93656122, 67 },
    { 0x52356122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f456126, 76 },
    { 0xab56126, 99 },
    { 0x56b56128, 85 },
    { 0xb2756128, 90 },
    { 0, 0 },
    { 0x37b5612b, 98 },
    { 0x7b15612c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5612f, 61 },
    { 0, 0 },
    { 0x3f456131, 91 },
    { 0, 0 },
    { 0xb3e56133, 23 },
    { 0x8a256133, 64 },
    { 0, 0 },
    { 0x45756136, 66 },
    { 0, 0 },
    { 0xad456138, 23 },
    { 0x74c56138, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f05613d, 76 },
    { 0x52c5613d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f56144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93256147, 49 },
    { 0x5cf56148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a75614c, 55 },
    { 0x3665614d, 91 },
    { 0xb15614e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60956154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd656159, 33 },
    { 0, 0 },
    { 0xff65615b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55615f, 90 },
    { 0, 0 },
    { 0xdac56161, 90 },
    { 0, 0 },
    { 0xe3656163, 76 },
    { 0x35f56164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12156167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615616d, 55 },
    { 0xe3b5616d, 95 },
    { 0x2015616d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd56172, 43 },
    { 0, 0 },
    { 0xb7156174, 17 },
    { 0x92a56175, 33 },
    { 0x35656176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9556179, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2056180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f56189, 90 },
    { 0xd9f5618a, 61 },
    { 0x2ef5618b, 93 },
    { 0, 0 },
    { 0x6915618d, 93 },
    { 0x4f45618e, 81 },
    { 0x57b5618e, 61 },
    { 0x33156190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e56196, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5619a, 95 },
    { 0xcf5619b, 88 },
    { 0x24d5619a, 61 },
    { 0x4125619d, 55 },
    { 0x7105619a, 95 },
    { 0xd9f5619f, 84 },
    { 0, 0 },
    { 0x7e7561a1, 52 },
    { 0, 0 },
    { 0x3c1561a3, 61 },
    { 0, 0 },
    { 0x20561a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5561a9, 95 },
    { 0x9c8561a9, 90 },
    { 0xef5561ab, 34 },
    { 0x3e561ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af561b3, 68 },
    { 0, 0 },
    { 0x401561b5, 34 },
    { 0x859561b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23561ba, 76 },
    { 0, 0 },
    { 0x489561bc, 91 },
    { 0xf07561bc, 93 },
    { 0x9e8561bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566561c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a561c5, 33 },
    { 0, 0 },
    { 0x398561c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367561d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3561d8, 76 },
    { 0, 0 },
    { 0x915561da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75561e0, 75 },
    { 0x624561e0, 90 },
    { 0x6ab561e1, 76 },
    { 0, 0 },
    { 0x413561e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x337561e7, 83 },
    { 0xe13561e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2561ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf561ef, 98 },
    { 0, 0 },
    { 0x89b561f1, 61 },
    { 0x35d561f2, 68 },
    { 0x673561f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9561f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f561fa, 98 },
    { 0x866561fb, 33 },
    { 0x318561fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c56201, 98 },
    { 0x76d56202, 33 },
    { 0x95056202, 55 },
    { 0x8f456204, 84 },
    { 0, 0 },
    { 0x8b356206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985620d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f56210, 23 },
    { 0, 0 },
    { 0x2fd56212, 23 },
    { 0xc856212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f5621f, 90 },
    { 0xd0156220, 98 },
    { 0x2dd56221, 65 },
    { 0x4b656221, 43 },
    { 0x4b656223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83256226, 98 },
    { 0, 0 },
    { 0xf0656228, 87 },
    { 0x9756229, 61 },
    { 0x1d656229, 75 },
    { 0x4c65622b, 95 },
    { 0xe0456229, 33 },
    { 0xa4056228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55656233, 95 },
    { 0xa6056234, 52 },
    { 0x36f56235, 95 },
    { 0xdd56235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b456239, 33 },
    { 0, 0 },
    { 0xcf15623b, 85 },
    { 0, 0 },
    { 0xe835623d, 43 },
    { 0x7e35623e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3256241, 61 },
    { 0x7e656242, 33 },
    { 0, 0 },
    { 0xa9156244, 23 },
    { 0x30b56245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87456248, 90 },
    { 0xd4c56249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91056250, 69 },
    { 0x84c56251, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a56256, 61 },
    { 0xb7156256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de56260, 81 },
    { 0, 0 },
    { 0x72856262, 100 },
    { 0xa3456263, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91156269, 23 },
    { 0xc615626a, 23 },
    { 0xb25626a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc635626f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d56272, 95 },
    { 0xe2056272, 34 },
    { 0x33c56272, 95 },
    { 0x58e56275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d56278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525627f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d56285, 69 },
    { 0x5356286, 90 },
    { 0, 0 },
    { 0xff456288, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x645628b, 90 },
    { 0, 0 },
    { 0x6ce5628d, 93 },
    { 0x6885628e, 23 },
    { 0x69f5628f, 61 },
    { 0xdf05628f, 43 },
    { 0x2c85628d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81156297, 91 },
    { 0xc3d56297, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf175629f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f562a3, 81 },
    { 0x98562a4, 59 },
    { 0x4ec562a5, 80 },
    { 0x4e7562a5, 95 },
    { 0xb88562a7, 33 },
    { 0, 0 },
    { 0x26d562a9, 100 },
    { 0, 0 },
    { 0xdc2562ab, 81 },
    { 0xf50562ab, 90 },
    { 0x919562ad, 43 },
    { 0, 0 },
    { 0x1ea562af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a562b3, 47 },
    { 0x92b562b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5562b9, 34 },
    { 0x348562b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3562bd, 81 },
    { 0, 0 },
    { 0x3f8562bf, 13 },
    { 0x41a562c0, 76 },
    { 0x35562c1, 99 },
    { 0x128562c2, 99 },
    { 0x3b0562c3, 68 },
    { 0xb06562c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a562c9, 78 },
    { 0x872562c9, 85 },
    { 0, 0 },
    { 0xc87562cc, 59 },
    { 0x5ac562cc, 66 },
    { 0x308562ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a562d2, 95 },
    { 0, 0 },
    { 0x35d562d4, 98 },
    { 0x456562d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4562dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776562e1, 42 },
    { 0x377562e2, 98 },
    { 0xc4e562e2, 88 },
    { 0xc4a562e2, 100 },
    { 0xf21562e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753562e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e562ee, 61 },
    { 0x863562ef, 33 },
    { 0, 0 },
    { 0x75b562f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff562f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18562f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6562fc, 87 },
    { 0x66f562fd, 85 },
    { 0xff8562fe, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5056301, 66 },
    { 0xc6056301, 90 },
    { 0x1b656303, 65 },
    { 0, 0 },
    { 0x23a56305, 46 },
    { 0, 0 },
    { 0xfe56307, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c05630a, 78 },
    { 0xc995630b, 90 },
    { 0x5a65630a, 33 },
    { 0xc745630b, 90 },
    { 0x885630e, 33 },
    { 0x4605630f, 17 },
    { 0, 0 },
    { 0xa8856311, 55 },
    { 0x22f56312, 99 },
    { 0x16956313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5175631a, 100 },
    { 0, 0 },
    { 0xbab5631c, 99 },
    { 0, 0 },
    { 0xe3e5631e, 84 },
    { 0x5f65631f, 81 },
    { 0, 0 },
    { 0xd5156321, 17 },
    { 0xbc256322, 23 },
    { 0xb3e56323, 33 },
    { 0, 0 },
    { 0x2a656325, 97 },
    { 0x17156326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f5632b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05632e, 66 },
    { 0x2915632f, 87 },
    { 0x24156330, 88 },
    { 0x27e56331, 78 },
    { 0x3fd56332, 33 },
    { 0x4f356331, 23 },
    { 0, 0 },
    { 0xda156335, 90 },
    { 0x4e456335, 91 },
    { 0x19c56337, 100 },
    { 0x26f56338, 90 },
    { 0x9f56339, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165633d, 23 },
    { 0, 0 },
    { 0xbe85633f, 33 },
    { 0xfa956340, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63156344, 90 },
    { 0, 0 },
    { 0x74f56346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9655634a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a55634d, 23 },
    { 0x1c35634d, 95 },
    { 0xa25634f, 69 },
    { 0x4cf5634e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd56353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2856357, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f35635a, 33 },
    { 0, 0 },
    { 0xf915635c, 90 },
    { 0x6535635d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a56360, 61 },
    { 0, 0 },
    { 0x47656362, 23 },
    { 0x9f56363, 90 },
    { 0x4ba56364, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff56367, 100 },
    { 0x2d056368, 76 },
    { 0xee356369, 66 },
    { 0xa6d56368, 51 },
    { 0xf225636b, 95 },
    { 0x99f5636a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5636f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74156377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc435637a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb56381, 23 },
    { 0xf5556382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed56385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e256388, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25638b, 33 },
    { 0xd445638c, 43 },
    { 0x2425638d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf256395, 99 },
    { 0x62956396, 95 },
    { 0x5b256397, 85 },
    { 0x6b856395, 90 },
    { 0, 0 },
    { 0xe5e5639a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb45639d, 98 },
    { 0x6915639e, 91 },
    { 0x4245639d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x997563a2, 36 },
    { 0x5bb563a3, 99 },
    { 0x50563a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294563af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35563b3, 76 },
    { 0, 0 },
    { 0x616563b5, 95 },
    { 0xa06563b6, 23 },
    { 0, 0 },
    { 0x43b563b8, 33 },
    { 0x668563b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35563bf, 43 },
    { 0xa2563c0, 78 },
    { 0xb91563c1, 33 },
    { 0xf24563c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x232563c5, 95 },
    { 0xa28563c5, 75 },
    { 0xa14563c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395563d2, 23 },
    { 0xbda563d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df563d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60563dc, 61 },
    { 0x88e563dd, 46 },
    { 0x282563dd, 46 },
    { 0x4d6563de, 61 },
    { 0xd25563e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1563e3, 90 },
    { 0, 0 },
    { 0x809563e5, 34 },
    { 0x535563e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae563eb, 33 },
    { 0, 0 },
    { 0x7a563ed, 95 },
    { 0xa7563ee, 95 },
    { 0x9e563ef, 76 },
    { 0x488563f0, 61 },
    { 0xe8d563f0, 95 },
    { 0xbd1563f2, 100 },
    { 0x55b563f3, 87 },
    { 0xe8a563f3, 65 },
    { 0xff4563f5, 100 },
    { 0xb0f563f4, 18 },
    { 0xb52563f6, 99 },
    { 0x79c563f8, 98 },
    { 0, 0 },
    { 0xc1c563fa, 33 },
    { 0xfbb563fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911563ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa656403, 100 },
    { 0xfd056404, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2156409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015640d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89956411, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f956414, 87 },
    { 0x8056415, 98 },
    { 0x60e56416, 98 },
    { 0xf2056417, 61 },
    { 0, 0 },
    { 0x60b56419, 90 },
    { 0xcbd5641a, 98 },
    { 0x4b956419, 98 },
    { 0x58f5641a, 34 },
    { 0x9575641c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f56420, 61 },
    { 0x9c356420, 34 },
    { 0xa1c56420, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c5642d, 91 },
    { 0xc125642e, 67 },
    { 0xc075642f, 90 },
    { 0xd6756430, 61 },
    { 0x26e5642d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a56435, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8165643e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee56443, 96 },
    { 0xe8f56444, 68 },
    { 0, 0 },
    { 0x71b56446, 76 },
    { 0xfd956447, 55 },
    { 0xb9256448, 33 },
    { 0xdff56447, 90 },
    { 0, 0 },
    { 0x6235644b, 33 },
    { 0x8475644c, 34 },
    { 0xc715644c, 84 },
    { 0xee95644e, 55 },
    { 0x3335644b, 23 },
    { 0xf0656450, 99 },
    { 0xbc056451, 98 },
    { 0xdd56451, 33 },
    { 0xb0d56453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8456459, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735645c, 61 },
    { 0xe9e5645d, 61 },
    { 0x4045645c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56462, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7456465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8856468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5646b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d5646e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2356471, 46 },
    { 0xc1356472, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37856478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca5647f, 76 },
    { 0, 0 },
    { 0xe8256481, 23 },
    { 0, 0 },
    { 0x5c156483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d156487, 17 },
    { 0x6f656487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825648b, 34 },
    { 0xbca5648c, 61 },
    { 0xd9d5648d, 17 },
    { 0xe1a5648e, 76 },
    { 0, 0 },
    { 0xd5956490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e5649c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd564a1, 90 },
    { 0xd63564a2, 61 },
    { 0x441564a3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x867564a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131564b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08564b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x677564bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8564be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156564c2, 90 },
    { 0, 0 },
    { 0x548564c4, 95 },
    { 0, 0 },
    { 0x7ee564c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb564ca, 23 },
    { 0x17f564cb, 81 },
    { 0x3ed564cc, 61 },
    { 0xfc3564cd, 98 },
    { 0x424564ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896564d9, 61 },
    { 0, 0 },
    { 0x4d4564db, 100 },
    { 0, 0 },
    { 0x370564dd, 100 },
    { 0, 0 },
    { 0x165564df, 95 },
    { 0, 0 },
    { 0xae8564e1, 65 },
    { 0xac5564e2, 98 },
    { 0, 0 },
    { 0x21b564e4, 98 },
    { 0x584564e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e564e9, 98 },
    { 0x394564ea, 85 },
    { 0xed9564ea, 90 },
    { 0x8ee564eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558564fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66356500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72056504, 33 },
    { 0, 0 },
    { 0x2b56506, 90 },
    { 0x5b56506, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295650a, 23 },
    { 0, 0 },
    { 0xb3d5650c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b56510, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3756513, 61 },
    { 0x1b256514, 23 },
    { 0xe2756515, 61 },
    { 0xa5056516, 91 },
    { 0x7a056515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae5651b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe555651e, 55 },
    { 0, 0 },
    { 0x3d356520, 91 },
    { 0, 0 },
    { 0x6c056522, 34 },
    { 0, 0 },
    { 0x20756524, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba56528, 98 },
    { 0xc1656528, 80 },
    { 0x2d456529, 55 },
    { 0, 0 },
    { 0x8d15652c, 23 },
    { 0x8425652d, 76 },
    { 0xee65652c, 90 },
    { 0, 0 },
    { 0xcd256530, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e56535, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd56539, 91 },
    { 0x7ea5653a, 91 },
    { 0x6375653b, 61 },
    { 0x8f65653c, 61 },
    { 0x3575653d, 93 },
    { 0x3565653e, 90 },
    { 0xb415653f, 23 },
    { 0x9d5653f, 34 },
    { 0xbb056541, 33 },
    { 0xec35653b, 101 },
    { 0, 0 },
    { 0x7d156544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff56549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a5654d, 46 },
    { 0, 0 },
    { 0x8d95654f, 34 },
    { 0xc9556550, 90 },
    { 0x2d356550, 17 },
    { 0, 0 },
    { 0xac956553, 100 },
    { 0xa2c56554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee56557, 46 },
    { 0xaa56557, 87 },
    { 0x21f56559, 33 },
    { 0x3a35655a, 61 },
    { 0, 0 },
    { 0x1075655c, 76 },
    { 0x8585655d, 69 },
    { 0x7ea5655c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a56562, 76 },
    { 0x42956563, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a56568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4075656d, 87 },
    { 0xedb5656d, 66 },
    { 0x6915656f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67956573, 61 },
    { 0xc7056574, 55 },
    { 0x27356574, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15657d, 34 },
    { 0x1f5657e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf856583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4856586, 90 },
    { 0x1e556586, 34 },
    { 0, 0 },
    { 0xc056589, 33 },
    { 0, 0 },
    { 0x33a5658b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a5658e, 61 },
    { 0x5cc5658f, 91 },
    { 0, 0 },
    { 0x1d456591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c56596, 98 },
    { 0x74456596, 99 },
    { 0, 0 },
    { 0x6de56599, 61 },
    { 0, 0 },
    { 0xc6a5659b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8565a1, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136565a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238565ac, 95 },
    { 0x9d565ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301565b6, 55 },
    { 0, 0 },
    { 0xac4565b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d565bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da565bf, 33 },
    { 0x8cb565c0, 90 },
    { 0, 0 },
    { 0xa3e565c2, 61 },
    { 0, 0 },
    { 0xce8565c4, 85 },
    { 0xd3b565c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda565c8, 88 },
    { 0x85565c8, 93 },
    { 0x29565c9, 95 },
    { 0, 0 },
    { 0x294565cc, 61 },
    { 0x174565cd, 77 },
    { 0xb8c565cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde565d3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba565e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80565e6, 34 },
    { 0xdfb565e6, 33 },
    { 0x3f7565e7, 76 },
    { 0x3ac565e9, 33 },
    { 0x24b565ea, 91 },
    { 0x96d565e7, 65 },
    { 0xd97565ec, 61 },
    { 0, 0 },
    { 0x672565ee, 68 },
    { 0xf2d565ef, 90 },
    { 0, 0 },
    { 0xd0d565f1, 43 },
    { 0xcd5565f2, 91 },
    { 0xd7d565f3, 98 },
    { 0x158565f3, 96 },
    { 0xce4565f5, 37 },
    { 0xa8f565f6, 33 },
    { 0, 0 },
    { 0xfec565f8, 33 },
    { 0x756565f9, 33 },
    { 0x8a0565fa, 95 },
    { 0, 0 },
    { 0xa56565fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c356606, 61 },
    { 0, 0 },
    { 0xf6056608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b5660d, 100 },
    { 0, 0 },
    { 0x94a5660f, 81 },
    { 0xa055660f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd456614, 66 },
    { 0x2e56615, 33 },
    { 0x5de56616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d05661c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe56622, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf956626, 98 },
    { 0xc9256627, 33 },
    { 0xee856628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f25662d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1556631, 23 },
    { 0x7fe56632, 99 },
    { 0, 0 },
    { 0x72356634, 76 },
    { 0xfe356634, 33 },
    { 0xd1e56636, 61 },
    { 0x90a56635, 100 },
    { 0, 0 },
    { 0xc3a56639, 90 },
    { 0x4f95663a, 23 },
    { 0x1f85663a, 87 },
    { 0xd8a56639, 61 },
    { 0x3e25663d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a56640, 61 },
    { 0x52656641, 93 },
    { 0x75f56642, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98056646, 61 },
    { 0xc0c56646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7445664d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd456651, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2356654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6556658, 43 },
    { 0x4556659, 98 },
    { 0xd7b5665a, 66 },
    { 0xf275665b, 76 },
    { 0x4b55665c, 99 },
    { 0, 0 },
    { 0xdee5665e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f56662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e356665, 19 },
    { 0x7a456666, 52 },
    { 0xdda56666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65666a, 33 },
    { 0x7a65666b, 55 },
    { 0x76b5666c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba05666f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d56672, 90 },
    { 0x96656672, 23 },
    { 0x92e56674, 81 },
    { 0x15256675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e256678, 52 },
    { 0x7ae56678, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c5667c, 61 },
    { 0x1015667c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a56684, 74 },
    { 0xc856685, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a5668c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c56690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13566a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d566a5, 33 },
    { 0x859566a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7566aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x968566ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff566b0, 23 },
    { 0x8b7566b1, 87 },
    { 0xac5566b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4566b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77566bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1566c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6566c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d566c6, 99 },
    { 0x793566c6, 46 },
    { 0x3d8566c7, 87 },
    { 0, 0 },
    { 0x4d8566ca, 83 },
    { 0x7f4566ca, 34 },
    { 0, 0 },
    { 0x456566cd, 93 },
    { 0x7f6566ce, 33 },
    { 0, 0 },
    { 0x352566d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3566d7, 90 },
    { 0x44566d7, 98 },
    { 0x8b3566d8, 84 },
    { 0x436566da, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92566e2, 66 },
    { 0x51c566e3, 93 },
    { 0x32f566e4, 66 },
    { 0xcca566e3, 33 },
    { 0xd95566e3, 33 },
    { 0x9f3566e7, 93 },
    { 0xfe3566e7, 100 },
    { 0, 0 },
    { 0x822566ea, 90 },
    { 0x684566eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44566f3, 23 },
    { 0x596566f4, 61 },
    { 0, 0 },
    { 0xadf566f6, 87 },
    { 0, 0 },
    { 0x519566f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4566fe, 76 },
    { 0x104566ff, 88 },
    { 0x6d756700, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0056706, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5670b, 90 },
    { 0xe265670c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc235670f, 100 },
    { 0, 0 },
    { 0xc4356711, 100 },
    { 0xfe356711, 61 },
    { 0x92356712, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b56718, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65671c, 95 },
    { 0, 0 },
    { 0xe695671e, 33 },
    { 0xaba5671f, 34 },
    { 0x63156720, 98 },
    { 0xe6f56720, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a656724, 65 },
    { 0, 0 },
    { 0xa2456726, 90 },
    { 0x36f56727, 100 },
    { 0, 0 },
    { 0xe0556729, 55 },
    { 0xd1656729, 65 },
    { 0xe825672b, 90 },
    { 0x6e55672c, 46 },
    { 0x6735672c, 98 },
    { 0x3dd5672c, 61 },
    { 0xf5556729, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c56735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1685673b, 91 },
    { 0xbcf5673b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9475673f, 81 },
    { 0x9355673f, 34 },
    { 0, 0 },
    { 0xc4f56742, 46 },
    { 0x3f456743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a956749, 76 },
    { 0x1e656749, 33 },
    { 0x6985674b, 55 },
    { 0, 0 },
    { 0xaf85674d, 33 },
    { 0x8dd5674d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22056752, 90 },
    { 0, 0 },
    { 0xebc56754, 93 },
    { 0, 0 },
    { 0xe6a56756, 76 },
    { 0, 0 },
    { 0xa8856758, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c85675b, 95 },
    { 0x9dd5675c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b56763, 100 },
    { 0, 0 },
    { 0x89356765, 93 },
    { 0xf5e56766, 23 },
    { 0x70e56766, 34 },
    { 0x5c956767, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625676b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085676f, 33 },
    { 0x3965676f, 66 },
    { 0x3dc56770, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40156774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7156779, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5065677f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed56782, 61 },
    { 0xf7b56783, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc95678a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05678e, 52 },
    { 0, 0 },
    { 0xfde56790, 76 },
    { 0x37656790, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e56795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce456799, 90 },
    { 0, 0 },
    { 0xec15679b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd735679f, 91 },
    { 0x37f567a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x646567a3, 23 },
    { 0xf74567a4, 99 },
    { 0, 0 },
    { 0xc0e567a6, 69 },
    { 0, 0 },
    { 0x194567a8, 90 },
    { 0, 0 },
    { 0x59c567aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd567ae, 33 },
    { 0x6a7567af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff567b4, 98 },
    { 0x4bd567b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd567b9, 55 },
    { 0xd4c567ba, 46 },
    { 0x12b567bb, 96 },
    { 0x640567bc, 98 },
    { 0xd34567bd, 46 },
    { 0xc20567be, 90 },
    { 0x1d8567bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab567c6, 64 },
    { 0, 0 },
    { 0x201567c8, 90 },
    { 0, 0 },
    { 0x143567ca, 23 },
    { 0xb44567ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577567d0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa567d6, 52 },
    { 0, 0 },
    { 0xcda567d8, 95 },
    { 0xacd567d9, 84 },
    { 0x39d567da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db567df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x346567e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70567e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda567e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7567eb, 88 },
    { 0x176567ec, 75 },
    { 0xcc9567ed, 98 },
    { 0xfca567ed, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2567f1, 77 },
    { 0xd70567f2, 90 },
    { 0x7fe567f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e567f6, 76 },
    { 0xd67567f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10567fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2556800, 95 },
    { 0x39e56800, 93 },
    { 0, 0 },
    { 0xa4b56803, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f56806, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5680a, 90 },
    { 0x88a5680b, 76 },
    { 0, 0 },
    { 0x4295680d, 95 },
    { 0x83c5680e, 90 },
    { 0, 0 },
    { 0x87656810, 84 },
    { 0x4e656811, 53 },
    { 0x22256811, 61 },
    { 0xc6756813, 34 },
    { 0xdee56814, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af56817, 40 },
    { 0x92756818, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5681b, 93 },
    { 0x20e5681c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc56821, 55 },
    { 0x60956822, 33 },
    { 0x4e556822, 90 },
    { 0x4b56822, 76 },
    { 0x39e56822, 67 },
    { 0xee956826, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x90256829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa935682c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72256833, 76 },
    { 0x59856833, 95 },
    { 0, 0 },
    { 0x12e56836, 46 },
    { 0, 0 },
    { 0x3bd56838, 90 },
    { 0, 0 },
    { 0x7e5683a, 90 },
    { 0xc4e5683b, 98 },
    { 0xd855683c, 99 },
    { 0x2565683c, 95 },
    { 0x9d95683c, 100 },
    { 0x73b5683f, 84 },
    { 0, 0 },
    { 0xf4856841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b56844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52456848, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585684d, 33 },
    { 0, 0 },
    { 0x2505684f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce256854, 66 },
    { 0x94a56855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44556859, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a56861, 93 },
    { 0, 0 },
    { 0xdc256863, 87 },
    { 0x20756864, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff56868, 88 },
    { 0x6056869, 76 },
    { 0, 0 },
    { 0x9245686b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16156871, 23 },
    { 0x1c856871, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce056875, 68 },
    { 0xbef56876, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5687b, 95 },
    { 0xa7d5687c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3856882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e256885, 55 },
    { 0x2c456886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f5688c, 66 },
    { 0, 0 },
    { 0x4695688e, 98 },
    { 0, 0 },
    { 0x30356890, 33 },
    { 0x3a956891, 98 },
    { 0x81c56891, 90 },
    { 0x3ec56893, 90 },
    { 0x78056894, 55 },
    { 0x5d656892, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05689a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b05689d, 51 },
    { 0, 0 },
    { 0xc335689f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf568a4, 100 },
    { 0, 0 },
    { 0xb11568a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5568a9, 84 },
    { 0x59d568aa, 80 },
    { 0x8b568aa, 23 },
    { 0xdd5568aa, 33 },
    { 0x304568ad, 95 },
    { 0, 0 },
    { 0xce8568af, 100 },
    { 0x9e8568b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e568ba, 81 },
    { 0xcd1568bb, 94 },
    { 0x772568bc, 95 },
    { 0xe91568bc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f568c0, 95 },
    { 0, 0 },
    { 0xac9568c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48568c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9568c8, 98 },
    { 0xf08568c9, 61 },
    { 0xcfd568ca, 91 },
    { 0x9b1568cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de568d2, 34 },
    { 0x1a5568d3, 90 },
    { 0x359568d4, 90 },
    { 0, 0 },
    { 0x287568d6, 94 },
    { 0xf73568d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67568da, 76 },
    { 0xa16568db, 77 },
    { 0xfc5568dc, 95 },
    { 0x81f568dd, 87 },
    { 0x5c9568dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb568e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3568e6, 95 },
    { 0x604568e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238568ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e568f3, 34 },
    { 0xd2f568f4, 81 },
    { 0xf98568f4, 43 },
    { 0xb0c568f5, 23 },
    { 0x2b9568f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea568fb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x239568fe, 61 },
    { 0x835568ff, 67 },
    { 0, 0 },
    { 0x85456901, 95 },
    { 0x7eb56901, 90 },
    { 0, 0 },
    { 0x79656904, 76 },
    { 0x80056904, 90 },
    { 0x88856906, 95 },
    { 0xa956906, 61 },
    { 0xc4956908, 55 },
    { 0xa1856909, 90 },
    { 0xa8d56909, 17 },
    { 0xf0b5690b, 98 },
    { 0x84f5690c, 17 },
    { 0x705690d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d56912, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee56915, 90 },
    { 0x2b256915, 34 },
    { 0xd7156917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad35691f, 33 },
    { 0x67756920, 33 },
    { 0xe3856920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb56926, 95 },
    { 0x1ec56927, 93 },
    { 0x9b256928, 95 },
    { 0x1c656927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5475692d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305693a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5425693e, 23 },
    { 0xc05693e, 55 },
    { 0, 0 },
    { 0xf7456941, 52 },
    { 0, 0 },
    { 0x86f56943, 23 },
    { 0, 0 },
    { 0xacd56945, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad5694e, 46 },
    { 0xc7b5694f, 23 },
    { 0xd6156950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d56954, 61 },
    { 0, 0 },
    { 0x74d56956, 76 },
    { 0x5e956957, 98 },
    { 0xd6b56958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac65695d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce56967, 64 },
    { 0x5c556968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff56971, 84 },
    { 0xe6556972, 23 },
    { 0xff656971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda156977, 23 },
    { 0x50a56977, 23 },
    { 0x21b56977, 33 },
    { 0x96556978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e5697f, 93 },
    { 0x36f56980, 45 },
    { 0xfcc56981, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f56987, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef5698d, 33 },
    { 0xc045698d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0256994, 77 },
    { 0x69056994, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e05699f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600569a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8569a8, 61 },
    { 0x4f4569a9, 95 },
    { 0x3fe569a8, 23 },
    { 0x627569ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d569b1, 90 },
    { 0x3c1569b2, 90 },
    { 0, 0 },
    { 0x72e569b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f569b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x960569ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf569be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c569c1, 46 },
    { 0x8cb569c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413569cd, 23 },
    { 0xfa9569ce, 55 },
    { 0xd7569cd, 33 },
    { 0x23f569d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5569d4, 90 },
    { 0x617569d4, 98 },
    { 0x816569d6, 67 },
    { 0xb08569d6, 76 },
    { 0x591569d8, 93 },
    { 0xd8d569d9, 61 },
    { 0x1d1569d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b569dd, 61 },
    { 0, 0 },
    { 0xae4569df, 75 },
    { 0x159569e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f569e4, 33 },
    { 0, 0 },
    { 0x900569e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05569e9, 22 },
    { 0xc83569e9, 76 },
    { 0, 0 },
    { 0x520569ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45569ef, 55 },
    { 0, 0 },
    { 0xe51569f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x772569f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da569f7, 76 },
    { 0, 0 },
    { 0x520569f9, 30 },
    { 0x186569fa, 61 },
    { 0x6c1569fb, 91 },
    { 0xd2569fc, 98 },
    { 0xff2569fd, 23 },
    { 0x964569fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45256a08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd56a0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd156a11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7256a14, 95 },
    { 0, 0 },
    { 0xbad56a16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90956a1a, 66 },
    { 0x71556a1b, 95 },
    { 0x6c456a1a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e256a20, 96 },
    { 0x3cb56a21, 100 },
    { 0xc6256a22, 55 },
    { 0x28a56a23, 8 },
    { 0x30156a24, 84 },
    { 0xeb056a24, 53 },
    { 0xebf56a26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f656a29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1156a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3956a30, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e56a33, 34 },
    { 0, 0 },
    { 0x54e56a35, 95 },
    { 0xa8256a36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a56a39, 33 },
    { 0xf8f56a39, 98 },
    { 0x3eb56a3b, 33 },
    { 0xbc956a3b, 52 },
    { 0xf6256a3d, 33 },
    { 0x356a3c, 33 },
    { 0xac156a3f, 95 },
    { 0xa256a3b, 23 },
    { 0, 0 },
    { 0x57956a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61856a45, 23 },
    { 0, 0 },
    { 0xf6156a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f56a4c, 90 },
    { 0xfd556a4d, 55 },
    { 0x61356a4e, 90 },
    { 0xd6356a4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc56a55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a56a59, 46 },
    { 0xc1656a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a56a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c56a62, 61 },
    { 0, 0 },
    { 0x56d56a64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e56a67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b956a6d, 40 },
    { 0x53a56a6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3256a79, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f56a7c, 33 },
    { 0x19756a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43156a83, 90 },
    { 0x7a456a84, 95 },
    { 0x7f356a85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8256a88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c56a8e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd156a94, 43 },
    { 0x99556a95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a656a9b, 76 },
    { 0, 0 },
    { 0x80956a9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7056aa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69556aaa, 61 },
    { 0x59856aab, 46 },
    { 0x89a56aac, 61 },
    { 0xe2956aad, 55 },
    { 0x3a556aae, 43 },
    { 0x83d56aae, 90 },
    { 0x7f456ab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa56ab8, 98 },
    { 0xaed56ab9, 33 },
    { 0, 0 },
    { 0x48b56abb, 90 },
    { 0xbdc56abc, 52 },
    { 0x2f156abc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3056ac2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70756ac7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb256aca, 68 },
    { 0x1ab56aca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e56ace, 90 },
    { 0x3556acf, 69 },
    { 0xf3556acf, 46 },
    { 0xa0056ad1, 34 },
    { 0xd2f56ad1, 98 },
    { 0xe0856ad1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68256ad8, 81 },
    { 0x11e56ad9, 85 },
    { 0xcce56ada, 96 },
    { 0xe0656ad9, 76 },
    { 0x87256ad8, 95 },
    { 0x33e56add, 33 },
    { 0, 0 },
    { 0xd3556adf, 97 },
    { 0xcdf56adf, 93 },
    { 0, 0 },
    { 0x4e056ae2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe056af0, 93 },
    { 0xd8356af1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf356af6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f56afd, 91 },
    { 0x2ff56afd, 55 },
    { 0x50e56aff, 43 },
    { 0, 0 },
    { 0xea456b01, 46 },
    { 0x71356b01, 61 },
    { 0xabb56b03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf656b06, 55 },
    { 0xa1556b07, 99 },
    { 0x38456b07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d56b0d, 23 },
    { 0xf3356b0e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86156b11, 61 },
    { 0, 0 },
    { 0x4a356b13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1456b18, 90 },
    { 0x4dd56b19, 53 },
    { 0xde156b19, 99 },
    { 0x1f556b1b, 95 },
    { 0x9ab56b1b, 33 },
    { 0, 0 },
    { 0x6f256b1e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6256b22, 46 },
    { 0xbb356b23, 66 },
    { 0x78c56b24, 96 },
    { 0xd0f56b25, 33 },
    { 0x68756b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce56b2b, 61 },
    { 0, 0 },
    { 0xdb356b2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d56b34, 55 },
    { 0x5eb56b35, 23 },
    { 0, 0 },
    { 0xf8456b37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db56b3b, 34 },
    { 0xd9156b3c, 56 },
    { 0x23656b3d, 23 },
    { 0xf5456b3e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x36056b41, 61 },
    { 0, 0 },
    { 0x26256b43, 67 },
    { 0xc9b56b44, 68 },
    { 0xc356b44, 61 },
    { 0xf2756b43, 90 },
    { 0xa1156b45, 65 },
    { 0x5c456b48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d256b4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a56b50, 98 },
    { 0xf2e56b51, 85 },
    { 0, 0 },
    { 0x8d756b53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3856b57, 23 },
    { 0x4ed56b58, 90 },
    { 0x2a356b58, 76 },
    { 0, 0 },
    { 0xa9a56b5b, 23 },
    { 0x20256b5c, 65 },
    { 0x60f56b5b, 90 },
    { 0x44f56b5b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a56b64, 34 },
    { 0xe7756b65, 61 },
    { 0, 0 },
    { 0xc5556b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe556b6f, 53 },
    { 0, 0 },
    { 0xc6e56b71, 52 },
    { 0x29956b71, 33 },
    { 0x37656b71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf56b78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc856b82, 23 },
    { 0x59756b83, 61 },
    { 0xdbe56b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f56b87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd156b8a, 52 },
    { 0xa0956b8b, 33 },
    { 0x9c256b8c, 85 },
    { 0xc5656b8d, 98 },
    { 0, 0 },
    { 0xe7056b8f, 23 },
    { 0, 0 },
    { 0x10956b91, 76 },
    { 0x4e56b92, 61 },
    { 0x5e956b93, 90 },
    { 0, 0 },
    { 0xe9456b95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55856b9b, 75 },
    { 0x1be56b9c, 66 },
    { 0xc4656b9c, 93 },
    { 0xb556b9e, 90 },
    { 0xf256b9e, 99 },
    { 0, 0 },
    { 0x66b56ba1, 78 },
    { 0, 0 },
    { 0xb0b56ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37856ba6, 95 },
    { 0, 0 },
    { 0xa756ba8, 95 },
    { 0x15056ba9, 98 },
    { 0x4556baa, 33 },
    { 0xd2756bab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58256bb5, 90 },
    { 0x8c156bb6, 17 },
    { 0xe8f56bb5, 100 },
    { 0x8cb56bb5, 95 },
    { 0x19556bb5, 93 },
    { 0xc6a56bba, 61 },
    { 0, 0 },
    { 0x8c756bbc, 92 },
    { 0x73f56bbd, 23 },
    { 0x44456bbe, 85 },
    { 0xb6856bbf, 90 },
    { 0, 0 },
    { 0x9b256bc1, 34 },
    { 0, 0 },
    { 0x7c656bc3, 50 },
    { 0x13856bc3, 34 },
    { 0xc5556bc5, 84 },
    { 0x15c56bc3, 99 },
    { 0xad356bc7, 33 },
    { 0xf8a56bc4, 61 },
    { 0xc8356bc9, 66 },
    { 0x94856bca, 90 },
    { 0x21056bc3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x82756bce, 95 },
    { 0, 0 },
    { 0xeb856bd0, 90 },
    { 0x5c356bd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f156bd6, 64 },
    { 0xb8056bd7, 61 },
    { 0x18856bd8, 23 },
    { 0, 0 },
    { 0xd8356bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a56be2, 43 },
    { 0x1a956be3, 55 },
    { 0, 0 },
    { 0x17a56be5, 90 },
    { 0x67956be5, 52 },
    { 0x3ab56be6, 96 },
    { 0x23d56be5, 23 },
    { 0x87f56be9, 98 },
    { 0x5b656bea, 90 },
    { 0xee756bea, 90 },
    { 0, 0 },
    { 0x36b56bed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab56bf2, 55 },
    { 0xd6056bf3, 84 },
    { 0xd4956bf2, 90 },
    { 0x7af56bf5, 39 },
    { 0x66456bf6, 78 },
    { 0x74d56bf7, 90 },
    { 0x9e656bf8, 68 },
    { 0x81956bf9, 99 },
    { 0, 0 },
    { 0x3f756bfb, 61 },
    { 0, 0 },
    { 0xfb156bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe56c01, 87 },
    { 0x86856c01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe156c06, 43 },
    { 0x65e56c06, 76 },
    { 0xde956c07, 33 },
    { 0x28556c09, 33 },
    { 0x94b56c0a, 53 },
    { 0x53c56c06, 95 },
    { 0x27c56c06, 33 },
    { 0x57f56c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b56c10, 69 },
    { 0xe2656c11, 61 },
    { 0xb0d56c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18956c1b, 55 },
    { 0, 0 },
    { 0x7c856c1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c56c21, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b56c24, 33 },
    { 0x71a56c25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef56c2b, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df56c2f, 66 },
    { 0xc4456c30, 73 },
    { 0x56556c31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a56c36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde56c3a, 52 },
    { 0x81756c3b, 78 },
    { 0x69e56c3b, 90 },
    { 0xf1f56c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f56c41, 87 },
    { 0xe8e56c41, 69 },
    { 0x1cf56c41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a56c48, 85 },
    { 0x8e356c49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee456c4d, 98 },
    { 0, 0 },
    { 0xa2256c4f, 91 },
    { 0x2ef56c50, 23 },
    { 0xbdd56c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe56c56, 55 },
    { 0x8c856c56, 61 },
    { 0x7d156c57, 61 },
    { 0, 0 },
    { 0x38a56c5a, 46 },
    { 0xf8d56c5b, 43 },
    { 0, 0 },
    { 0x6a456c5d, 46 },
    { 0x73456c5e, 99 },
    { 0xc4f56c5f, 65 },
    { 0, 0 },
    { 0xe5b56c61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba856c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b56c6f, 53 },
    { 0, 0 },
    { 0x8ec56c71, 91 },
    { 0, 0 },
    { 0x79856c73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6856c76, 43 },
    { 0x35856c77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c556c7f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea156c83, 95 },
    { 0, 0 },
    { 0xf3356c85, 23 },
    { 0x91f56c86, 66 },
    { 0x4e656c87, 33 },
    { 0x56656c86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5456c8b, 33 },
    { 0xdf256c8c, 52 },
    { 0x2aa56c8c, 55 },
    { 0x8d256c8e, 65 },
    { 0, 0 },
    { 0x20156c90, 95 },
    { 0, 0 },
    { 0x18f56c92, 76 },
    { 0xb9856c93, 68 },
    { 0x8fc56c93, 98 },
    { 0xa4d56c94, 36 },
    { 0xcd956c94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e756c99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e56c9d, 95 },
    { 0, 0 },
    { 0x8a456c9f, 55 },
    { 0x2ac56ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f56ca5, 93 },
    { 0, 0 },
    { 0x9ff56ca7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb56caa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea56cb1, 90 },
    { 0x8fe56cb2, 98 },
    { 0x9b156cb3, 90 },
    { 0, 0 },
    { 0xe8d56cb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3056cb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32556cbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf56cc0, 33 },
    { 0xe6856cc1, 76 },
    { 0x2bd56cc1, 61 },
    { 0x64356cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14156cc7, 61 },
    { 0xae556cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd56cd2, 61 },
    { 0, 0 },
    { 0x2d456cd4, 55 },
    { 0, 0 },
    { 0x4656cd6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x95156cd9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea56cdf, 23 },
    { 0, 0 },
    { 0x7a756ce1, 66 },
    { 0, 0 },
    { 0x72c56ce3, 67 },
    { 0, 0 },
    { 0x47056ce5, 91 },
    { 0, 0 },
    { 0x50256ce7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b56ceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2856cef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41056cf2, 74 },
    { 0, 0 },
    { 0x67256cf4, 33 },
    { 0xefc56cf5, 33 },
    { 0x95356cf6, 91 },
    { 0x9fc56cf6, 98 },
    { 0x57d56cf7, 52 },
    { 0x16c56cf9, 90 },
    { 0xf9d56cf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c56d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f456d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c56d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0456d0d, 90 },
    { 0xa9356d0e, 87 },
    { 0xcbe56d0f, 93 },
    { 0x4ca56d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58256d1a, 95 },
    { 0x63756d1b, 61 },
    { 0xb1356d1c, 67 },
    { 0x3a856d1c, 90 },
    { 0x66f56d1b, 40 },
    { 0x63956d1d, 23 },
    { 0x8f056d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a56d28, 84 },
    { 0x7a956d28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956d2c, 95 },
    { 0x85e56d2d, 96 },
    { 0x8fe56d2d, 34 },
    { 0x3b056d2f, 66 },
    { 0, 0 },
    { 0xfe556d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d56d34, 98 },
    { 0x81856d35, 23 },
    { 0x4cf56d35, 33 },
    { 0xf1056d36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d56d4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44356d55, 23 },
    { 0x5fc56d56, 23 },
    { 0xee056d56, 33 },
    { 0x1c256d57, 61 },
    { 0, 0 },
    { 0xaf956d5a, 23 },
    { 0x4ee56d5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18756d60, 75 },
    { 0, 0 },
    { 0x55956d62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d856d66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a56d6a, 23 },
    { 0x4a956d6b, 23 },
    { 0x9b256d6c, 90 },
    { 0x80956d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa56d72, 33 },
    { 0, 0 },
    { 0xec056d74, 68 },
    { 0, 0 },
    { 0x68156d76, 84 },
    { 0x66e56d77, 95 },
    { 0x3b56d77, 61 },
    { 0x72456d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc56d7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7556d80, 95 },
    { 0x2e556d81, 98 },
    { 0, 0 },
    { 0xd8c56d83, 95 },
    { 0x81256d84, 33 },
    { 0x27e56d84, 95 },
    { 0x68156d85, 95 },
    { 0x67656d87, 81 },
    { 0x61856d86, 61 },
    { 0x62456d86, 85 },
    { 0x66f56d8a, 100 },
    { 0x3cf56d84, 33 },
    { 0x69c56d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5656d9a, 33 },
    { 0x8256d9b, 93 },
    { 0xae356d9c, 98 },
    { 0x15656d9d, 33 },
    { 0xdaa56d9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa56da2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2756da9, 84 },
    { 0xa7956daa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce56dad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27056db8, 90 },
    { 0x5ae56db9, 23 },
    { 0x33156dba, 33 },
    { 0xde956dbb, 98 },
    { 0x2be56dbc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0056dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa56dc5, 33 },
    { 0, 0 },
    { 0xc7156dc7, 91 },
    { 0, 0 },
    { 0x86656dc9, 23 },
    { 0xc5956dca, 76 },
    { 0xa8456dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36856dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d456dd6, 55 },
    { 0x17456dd6, 98 },
    { 0xe8556dd8, 76 },
    { 0x39956dd9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f56de2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da56de5, 64 },
    { 0x92956de6, 23 },
    { 0x27456de7, 76 },
    { 0xacb56de7, 33 },
    { 0x28356de9, 34 },
    { 0x42f56dea, 66 },
    { 0xd4d56dea, 98 },
    { 0xa2956dec, 91 },
    { 0x96256de7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1756df3, 98 },
    { 0, 0 },
    { 0xd2a56df5, 65 },
    { 0x32056df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29956df9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a56dfd, 33 },
    { 0x71656dfe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad256e03, 81 },
    { 0x21a56e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36056e10, 23 },
    { 0xf3656e11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56e14, 98 },
    { 0xeb756e15, 87 },
    { 0x35f56e16, 91 },
    { 0, 0 },
    { 0x5e756e18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a56e1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd656e20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac56e26, 33 },
    { 0x1e756e27, 61 },
    { 0, 0 },
    { 0x55056e29, 81 },
    { 0, 0 },
    { 0x43a56e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c56e31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb56e34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3456e37, 95 },
    { 0x13e56e38, 33 },
    { 0x9bd56e39, 90 },
    { 0x4c56e3a, 36 },
    { 0x70356e39, 90 },
    { 0x3dd56e3c, 66 },
    { 0, 0 },
    { 0x9f56e3e, 33 },
    { 0, 0 },
    { 0x4f056e40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead56e4a, 90 },
    { 0, 0 },
    { 0x13c56e4c, 43 },
    { 0x77356e4c, 90 },
    { 0x68256e4e, 93 },
    { 0xa1256e4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3156e55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24856e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce56e5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c56e67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad56e6b, 98 },
    { 0x8056e6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d56e6f, 43 },
    { 0x17d56e70, 61 },
    { 0, 0 },
    { 0x3c856e72, 98 },
    { 0x54956e73, 76 },
    { 0x83b56e72, 95 },
    { 0xc8356e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16056e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e56e7d, 77 },
    { 0, 0 },
    { 0x44556e7f, 43 },
    { 0xca856e80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2656e85, 90 },
    { 0x4f56e86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1856e8a, 65 },
    { 0x35356e8b, 34 },
    { 0x2756e8a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e56e94, 23 },
    { 0, 0 },
    { 0x7c456e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad56e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e56e9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6756ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56356ea7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d56eaa, 65 },
    { 0xe0d56eab, 76 },
    { 0xd7256eac, 33 },
    { 0x6f56eac, 90 },
    { 0x31756eae, 90 },
    { 0, 0 },
    { 0xa0a56eb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef56eb3, 23 },
    { 0x68356eb4, 61 },
    { 0xc2d56eb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca56ebb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd256ebe, 76 },
    { 0xe9e56ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e56ec2, 90 },
    { 0x25b56ec3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c356ec6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c256ec9, 98 },
    { 0x5656ec9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf956ecf, 90 },
    { 0, 0 },
    { 0xe8f56ed1, 46 },
    { 0x2b956ed1, 33 },
    { 0x60056ed3, 33 },
    { 0x51a56ed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5256eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd56edd, 99 },
    { 0, 0 },
    { 0xe7c56edf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf56ee5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f56ee9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b56eed, 85 },
    { 0x70456eee, 90 },
    { 0xb4a56eef, 99 },
    { 0, 0 },
    { 0x4c356ef1, 55 },
    { 0x43256ef2, 23 },
    { 0xc4856ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63756ef7, 33 },
    { 0x43656ef8, 90 },
    { 0x2ed56ef9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25256f02, 90 },
    { 0, 0 },
    { 0xf8556f04, 52 },
    { 0, 0 },
    { 0x2e456f06, 23 },
    { 0, 0 },
    { 0x67156f08, 55 },
    { 0xf3f56f09, 23 },
    { 0, 0 },
    { 0x79656f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b856f10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c256f13, 98 },
    { 0, 0 },
    { 0x72856f15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56f18, 95 },
    { 0, 0 },
    { 0xc0856f1a, 33 },
    { 0, 0 },
    { 0x90756f1c, 86 },
    { 0xf4556f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac56f21, 95 },
    { 0xc6456f22, 43 },
    { 0, 0 },
    { 0x95e56f24, 78 },
    { 0x84056f25, 90 },
    { 0, 0 },
    { 0xce056f27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd56f2d, 91 },
    { 0xf8356f2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d856f31, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f356f40, 61 },
    { 0, 0 },
    { 0x5756f42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27456f46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77256f4a, 90 },
    { 0x65856f4b, 38 },
    { 0x7e156f4c, 61 },
    { 0, 0 },
    { 0xcca56f4e, 61 },
    { 0x2d356f4f, 90 },
    { 0xdd356f4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d56f53, 34 },
    { 0xd4056f53, 46 },
    { 0xe9456f55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a56f66, 95 },
    { 0xef356f67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48856f6a, 76 },
    { 0xd7656f6b, 33 },
    { 0x7cb56f6c, 46 },
    { 0x64a56f6c, 33 },
    { 0xab956f6e, 23 },
    { 0xca456f6f, 65 },
    { 0x60356f70, 99 },
    { 0xace56f71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c56f7f, 94 },
    { 0xb8a56f7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b856f86, 95 },
    { 0x2ee56f87, 90 },
    { 0x2d456f87, 95 },
    { 0, 0 },
    { 0x68f56f8a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e156f90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1256f94, 95 },
    { 0xce956f94, 68 },
    { 0xa0556f96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f656fa3, 88 },
    { 0, 0 },
    { 0xe8f56fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a56fa8, 75 },
    { 0x1d756fa9, 61 },
    { 0x5ff56fa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6956fb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d56fb8, 90 },
    { 0xdc556fb8, 34 },
    { 0x13556fb8, 88 },
    { 0x7fd56fbb, 43 },
    { 0x21e56fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd756fc4, 33 },
    { 0x4f56fc4, 33 },
    { 0, 0 },
    { 0xdc156fc7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2356fcd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e56fd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93756fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13156fdc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4656fe2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e156fe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b56feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b556ff0, 98 },
    { 0xda256ff1, 55 },
    { 0, 0 },
    { 0x33c56ff3, 55 },
    { 0xcfd56ff4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64056ffa, 95 },
    { 0x556ffb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb756ffe, 23 },
    { 0x71c56ffe, 85 },
    { 0x45157000, 33 },
    { 0xdc957001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba57005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e457009, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a55700e, 55 },
    { 0xee95700e, 33 },
    { 0xc0557010, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b57014, 95 },
    { 0xd5157015, 97 },
    { 0xcda57015, 46 },
    { 0, 0 },
    { 0xb2d57018, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b5701d, 90 },
    { 0x66f5701d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13457023, 98 },
    { 0x54b57024, 55 },
    { 0x95157025, 52 },
    { 0xe3557024, 41 },
    { 0x6257025, 33 },
    { 0, 0 },
    { 0x42f57029, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b5702e, 23 },
    { 0x8e45702e, 91 },
    { 0xe975702e, 95 },
    { 0xbfa57030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91357036, 55 },
    { 0, 0 },
    { 0xc3257038, 55 },
    { 0xe0b57039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4757049, 23 },
    { 0x6fc57049, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5704f, 95 },
    { 0xad65704f, 52 },
    { 0x20757051, 90 },
    { 0x77a57052, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9357056, 77 },
    { 0, 0 },
    { 0x91c57058, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca5705c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4957061, 89 },
    { 0x4ca57062, 61 },
    { 0xf8f57061, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35706a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf5706d, 91 },
    { 0xc1a5706e, 52 },
    { 0, 0 },
    { 0x11d57070, 33 },
    { 0, 0 },
    { 0x76357072, 23 },
    { 0x16057073, 77 },
    { 0xf6b57073, 61 },
    { 0x4fa57074, 84 },
    { 0xf6d57076, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475707b, 90 },
    { 0, 0 },
    { 0xe895707d, 61 },
    { 0xe645707e, 33 },
    { 0xcd55707f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d57083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47357087, 90 },
    { 0xdeb57087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775708d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30857092, 84 },
    { 0x9eb57093, 55 },
    { 0x17e57093, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac457097, 90 },
    { 0, 0 },
    { 0xf6a57099, 55 },
    { 0, 0 },
    { 0x7b35709b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286570a5, 34 },
    { 0x8de570a6, 76 },
    { 0xb9c570a6, 23 },
    { 0x3df570a8, 90 },
    { 0x48e570a9, 91 },
    { 0x1f0570a8, 33 },
    { 0xf71570a6, 95 },
    { 0x906570ac, 33 },
    { 0x636570ad, 33 },
    { 0x32d570ad, 90 },
    { 0xbb2570ac, 90 },
    { 0xaf570ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf570b3, 78 },
    { 0, 0 },
    { 0x1e570b5, 98 },
    { 0xc41570b5, 95 },
    { 0xe99570b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4570bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97570c7, 33 },
    { 0xd91570c7, 28 },
    { 0x8fd570c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88570d2, 61 },
    { 0x960570d3, 93 },
    { 0x7fd570d2, 23 },
    { 0xc43570d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50570db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867570de, 98 },
    { 0xca1570de, 95 },
    { 0x44c570df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2570e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1570e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c570f2, 61 },
    { 0xa26570f2, 43 },
    { 0, 0 },
    { 0xe97570f5, 66 },
    { 0xb06570f5, 76 },
    { 0, 0 },
    { 0x125570f8, 95 },
    { 0x97e570f8, 23 },
    { 0, 0 },
    { 0x1f1570fb, 40 },
    { 0, 0 },
    { 0x781570fd, 92 },
    { 0, 0 },
    { 0x25d570ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb57102, 33 },
    { 0, 0 },
    { 0x80c57104, 23 },
    { 0x5ec57104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2445710d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44457112, 98 },
    { 0, 0 },
    { 0x2f957114, 23 },
    { 0, 0 },
    { 0xd9057116, 90 },
    { 0x97257117, 66 },
    { 0xd8d57116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6485711e, 34 },
    { 0, 0 },
    { 0xeb357120, 90 },
    { 0x81657120, 90 },
    { 0x70357122, 91 },
    { 0, 0 },
    { 0xea57124, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5712a, 93 },
    { 0xc05712b, 93 },
    { 0x7fe5712c, 75 },
    { 0, 0 },
    { 0xb495712e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f57133, 33 },
    { 0x51557133, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5713e, 90 },
    { 0, 0 },
    { 0x88457140, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf757143, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc757147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a5714b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5714e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd57158, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5715b, 90 },
    { 0xfe45715b, 81 },
    { 0x85a5715d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb57162, 98 },
    { 0xd1357162, 95 },
    { 0x2fe57164, 95 },
    { 0x95c57164, 33 },
    { 0, 0 },
    { 0xd1157167, 98 },
    { 0xd1657168, 34 },
    { 0x30357168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5935716f, 33 },
    { 0x7585716f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce357175, 77 },
    { 0, 0 },
    { 0x6f057177, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c5717b, 34 },
    { 0x8be5717b, 93 },
    { 0xe865717d, 91 },
    { 0x43a5717d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa57184, 66 },
    { 0x45b57185, 33 },
    { 0x7bd57186, 61 },
    { 0x15157186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1675718e, 23 },
    { 0, 0 },
    { 0xec57190, 100 },
    { 0xc9d57191, 66 },
    { 0, 0 },
    { 0x20157193, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e57199, 23 },
    { 0x48d5719a, 90 },
    { 0, 0 },
    { 0x7725719c, 96 },
    { 0, 0 },
    { 0x55f5719e, 42 },
    { 0x89a5719f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab571a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5571a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e571ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e571af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d571b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b571b8, 80 },
    { 0xd80571b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7571be, 48 },
    { 0xc81571bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x562571c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7571d3, 90 },
    { 0, 0 },
    { 0x9ef571d5, 95 },
    { 0xd1571d6, 65 },
    { 0xd55571d7, 68 },
    { 0, 0 },
    { 0xb99571d9, 75 },
    { 0, 0 },
    { 0xf06571db, 52 },
    { 0x51d571db, 98 },
    { 0x1af571dd, 76 },
    { 0xb34571dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc571e2, 34 },
    { 0xab1571e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c571e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd571eb, 84 },
    { 0x7f4571ec, 75 },
    { 0xf44571ed, 84 },
    { 0xf4e571ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0571f3, 90 },
    { 0xd5b571f3, 75 },
    { 0x20f571f5, 95 },
    { 0xf3c571f4, 98 },
    { 0x68f571f7, 61 },
    { 0x622571f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463571fe, 88 },
    { 0x404571ff, 69 },
    { 0xfdc57200, 23 },
    { 0x65057201, 95 },
    { 0x665571ff, 43 },
    { 0xac857203, 66 },
    { 0xa0557204, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2957207, 61 },
    { 0xf3857208, 43 },
    { 0, 0 },
    { 0xa4c5720a, 23 },
    { 0x56b5720b, 46 },
    { 0xb745720c, 23 },
    { 0xf595720d, 78 },
    { 0, 0 },
    { 0xb835720f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e57212, 93 },
    { 0, 0 },
    { 0xabf57214, 90 },
    { 0, 0 },
    { 0x31657216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f57220, 33 },
    { 0xd1c57221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa57227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5722b, 90 },
    { 0, 0 },
    { 0xeaa5722d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52857230, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a57235, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8857238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec5723c, 70 },
    { 0x605723d, 33 },
    { 0, 0 },
    { 0x4ac5723f, 90 },
    { 0x79057240, 98 },
    { 0xe8257241, 90 },
    { 0x51357242, 34 },
    { 0, 0 },
    { 0x78757244, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74657249, 33 },
    { 0, 0 },
    { 0xb2b5724b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5724e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2557251, 76 },
    { 0x42d57252, 66 },
    { 0xf1357253, 43 },
    { 0x7f457252, 95 },
    { 0xde657255, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e157258, 81 },
    { 0xd5357259, 23 },
    { 0x30b5725a, 90 },
    { 0xd35725b, 46 },
    { 0x19c57258, 38 },
    { 0, 0 },
    { 0x6c05725e, 84 },
    { 0xf435725e, 33 },
    { 0xb2457260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad57263, 95 },
    { 0x91d57264, 55 },
    { 0, 0 },
    { 0xeb457266, 98 },
    { 0, 0 },
    { 0x60157268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e5726b, 91 },
    { 0x9d25726c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9157274, 100 },
    { 0x1bf57275, 61 },
    { 0x6bb57276, 66 },
    { 0xb4357276, 51 },
    { 0x12a57276, 43 },
    { 0x3d957276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa135727f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c757288, 33 },
    { 0, 0 },
    { 0xa025728a, 98 },
    { 0x2775728b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c5728e, 61 },
    { 0x59a5728f, 99 },
    { 0xe725728e, 34 },
    { 0xde957291, 77 },
    { 0xa5557292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d57295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9057298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa035729e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9572ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572b4, 98 },
    { 0, 0 },
    { 0xb08572b6, 95 },
    { 0x25d572b7, 33 },
    { 0, 0 },
    { 0x9ee572b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd572bd, 98 },
    { 0xc5d572be, 95 },
    { 0x7dc572bf, 98 },
    { 0x9e7572be, 100 },
    { 0xfc8572c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7572c6, 93 },
    { 0x460572c7, 99 },
    { 0xceb572c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476572cc, 55 },
    { 0xee4572cc, 95 },
    { 0xc38572cd, 61 },
    { 0xcce572cf, 40 },
    { 0xded572d0, 76 },
    { 0, 0 },
    { 0x538572d2, 23 },
    { 0, 0 },
    { 0x461572d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3572da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac572e1, 76 },
    { 0xce7572e2, 100 },
    { 0xfca572e1, 33 },
    { 0xf35572e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483572e8, 33 },
    { 0x4ae572e8, 87 },
    { 0x7a0572e9, 34 },
    { 0x583572eb, 33 },
    { 0xf0b572e9, 36 },
    { 0xa93572ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x969572f0, 90 },
    { 0xf36572f1, 76 },
    { 0x269572f2, 75 },
    { 0, 0 },
    { 0x9a3572f4, 23 },
    { 0x401572f5, 91 },
    { 0x471572f5, 100 },
    { 0x913572f6, 90 },
    { 0xda0572f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b572fb, 75 },
    { 0x778572fc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257302, 83 },
    { 0, 0 },
    { 0xc0357304, 33 },
    { 0x2fa57305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955730c, 34 },
    { 0xda35730d, 92 },
    { 0x65a5730d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec57313, 98 },
    { 0xca357314, 61 },
    { 0x5fd57315, 43 },
    { 0xaaa57316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415731a, 43 },
    { 0x2855731b, 55 },
    { 0, 0 },
    { 0xf345731d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc857321, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1057324, 98 },
    { 0x2df57325, 53 },
    { 0xd257326, 55 },
    { 0x93257327, 76 },
    { 0x38157326, 17 },
    { 0xb6e57329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6057330, 90 },
    { 0x57657330, 87 },
    { 0xeec57332, 99 },
    { 0xf2057332, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f057337, 87 },
    { 0, 0 },
    { 0xdf457339, 76 },
    { 0x985733a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf35733f, 61 },
    { 0x29757340, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb57344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a957349, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d95734c, 17 },
    { 0, 0 },
    { 0x7f55734e, 33 },
    { 0, 0 },
    { 0x58657350, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96657355, 81 },
    { 0, 0 },
    { 0x53f57357, 98 },
    { 0x86e57358, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5735b, 43 },
    { 0, 0 },
    { 0x3e15735d, 34 },
    { 0x79e5735e, 76 },
    { 0xa5c5735e, 43 },
    { 0, 0 },
    { 0x76a57361, 61 },
    { 0x92a57361, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5736d, 33 },
    { 0x5955736e, 87 },
    { 0xf5a5736e, 43 },
    { 0, 0 },
    { 0xb4b57371, 98 },
    { 0, 0 },
    { 0x60c57373, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b457379, 90 },
    { 0x3a757379, 46 },
    { 0x60357379, 55 },
    { 0xb645737c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73557381, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e57385, 90 },
    { 0, 0 },
    { 0x45657387, 97 },
    { 0x58d57387, 76 },
    { 0x7b857387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5945738d, 61 },
    { 0xf885738e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7757396, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75739b, 34 },
    { 0, 0 },
    { 0xbf45739d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff573a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a573aa, 55 },
    { 0x58b573aa, 98 },
    { 0, 0 },
    { 0xd46573ad, 61 },
    { 0xd32573ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7573b1, 61 },
    { 0xd60573b2, 55 },
    { 0xd11573b3, 76 },
    { 0x597573b4, 23 },
    { 0x9de573b4, 43 },
    { 0x9d6573b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8573bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb573bf, 61 },
    { 0, 0 },
    { 0x201573c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7573c6, 61 },
    { 0x2e3573c7, 61 },
    { 0xc54573c7, 33 },
    { 0x1c573c9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574573ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be573d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b573db, 87 },
    { 0xa3c573db, 23 },
    { 0x730573dc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27573e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b573e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x199573e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9573e9, 33 },
    { 0x763573ea, 90 },
    { 0xe64573eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f573f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3573f8, 46 },
    { 0x9a9573f8, 23 },
    { 0, 0 },
    { 0xbd8573fb, 61 },
    { 0, 0 },
    { 0x1c1573fd, 76 },
    { 0xe67573fe, 46 },
    { 0x95e573fe, 98 },
    { 0, 0 },
    { 0x13457401, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3357408, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35740b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54457410, 34 },
    { 0x7b557411, 23 },
    { 0x68657412, 61 },
    { 0x1ab57413, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f557418, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35741e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8857422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d57425, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67757428, 90 },
    { 0, 0 },
    { 0x34d5742a, 81 },
    { 0, 0 },
    { 0x1b35742c, 66 },
    { 0, 0 },
    { 0xdb25742e, 56 },
    { 0xc115742e, 53 },
    { 0x5805742e, 66 },
    { 0xd0c5742e, 90 },
    { 0xa2657430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8957436, 76 },
    { 0x7ab57437, 98 },
    { 0x41157437, 66 },
    { 0xfb957437, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6385743d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf757440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4157448, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb895744c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede57453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35745a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5745d, 75 },
    { 0, 0 },
    { 0x6935745f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f5746f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b357472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa975747c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90657481, 98 },
    { 0x1f057482, 98 },
    { 0, 0 },
    { 0x19a57484, 33 },
    { 0x8a657485, 34 },
    { 0x78057486, 98 },
    { 0x2257487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc945748c, 34 },
    { 0xc1b5748c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22657490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5749c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe574a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4574a9, 61 },
    { 0x9574a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf574ad, 52 },
    { 0x7e1574ae, 61 },
    { 0xdc574af, 33 },
    { 0x551574b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6574b4, 98 },
    { 0, 0 },
    { 0xf25574b6, 33 },
    { 0xf6b574b6, 34 },
    { 0x6c574b7, 98 },
    { 0x558574b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375574c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781574c7, 90 },
    { 0x93f574c8, 33 },
    { 0x61f574c8, 60 },
    { 0x8cc574ca, 93 },
    { 0xa1a574cb, 87 },
    { 0xb46574cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2574d5, 90 },
    { 0, 0 },
    { 0x6fe574d7, 85 },
    { 0xeff574d8, 76 },
    { 0x18e574d9, 33 },
    { 0, 0 },
    { 0xda7574db, 55 },
    { 0x13d574dc, 33 },
    { 0xed3574dd, 98 },
    { 0x590574de, 96 },
    { 0xe7c574dc, 95 },
    { 0xe3f574e0, 33 },
    { 0x14b574df, 61 },
    { 0x3e4574e2, 67 },
    { 0x248574e3, 76 },
    { 0xffb574e3, 23 },
    { 0xfa2574e5, 52 },
    { 0xc94574e2, 95 },
    { 0x25c574e7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9574ea, 90 },
    { 0xc74574eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9574ee, 46 },
    { 0x5d1574ef, 88 },
    { 0x72b574ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a574f3, 77 },
    { 0, 0 },
    { 0xbf3574f5, 52 },
    { 0x99d574f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607574fa, 33 },
    { 0xe6c574fb, 84 },
    { 0xc4f574fa, 34 },
    { 0, 0 },
    { 0x369574fe, 65 },
    { 0xe8e574ff, 17 },
    { 0, 0 },
    { 0x3f557501, 93 },
    { 0xb2457502, 76 },
    { 0x13657503, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff57509, 84 },
    { 0x29e57509, 25 },
    { 0xb25750b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d57512, 76 },
    { 0xb9c57513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95751a, 91 },
    { 0, 0 },
    { 0xf905751c, 76 },
    { 0, 0 },
    { 0x3955751e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d57523, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085752b, 33 },
    { 0xc535752c, 55 },
    { 0x67b5752d, 90 },
    { 0, 0 },
    { 0xf605752f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93757533, 98 },
    { 0x14357534, 95 },
    { 0xa0157535, 61 },
    { 0x87257534, 42 },
    { 0, 0 },
    { 0x6b557538, 43 },
    { 0x21857539, 23 },
    { 0x1115753a, 90 },
    { 0xd005753b, 95 },
    { 0x11557538, 61 },
    { 0x7d85753c, 76 },
    { 0xc065753e, 98 },
    { 0xbe85753d, 61 },
    { 0x47357540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33357544, 61 },
    { 0xe9557545, 80 },
    { 0, 0 },
    { 0xa6957547, 93 },
    { 0, 0 },
    { 0x74657549, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5754c, 85 },
    { 0, 0 },
    { 0x7555754e, 23 },
    { 0, 0 },
    { 0x6cc57550, 98 },
    { 0x3e957551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf457556, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d35755d, 53 },
    { 0, 0 },
    { 0xf8c5755f, 33 },
    { 0, 0 },
    { 0xdb157561, 23 },
    { 0x65c57561, 90 },
    { 0x2a357563, 33 },
    { 0xe3257564, 90 },
    { 0xc457564, 43 },
    { 0x49157565, 33 },
    { 0xac757561, 46 },
    { 0xded57568, 90 },
    { 0, 0 },
    { 0xf935756a, 84 },
    { 0xb685756b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34657571, 61 },
    { 0xf9457572, 61 },
    { 0xecc57572, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865757a, 33 },
    { 0, 0 },
    { 0x7905757c, 98 },
    { 0x8885757d, 61 },
    { 0x3475757e, 52 },
    { 0, 0 },
    { 0x48857580, 33 },
    { 0x9a957580, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9457585, 53 },
    { 0x84357585, 52 },
    { 0x70d57585, 34 },
    { 0xddf57586, 95 },
    { 0xd357589, 95 },
    { 0x26d5758a, 23 },
    { 0, 0 },
    { 0xd525758c, 95 },
    { 0, 0 },
    { 0xbbc5758e, 98 },
    { 0x3305758f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0557593, 23 },
    { 0, 0 },
    { 0xf6257595, 88 },
    { 0x25d57595, 23 },
    { 0, 0 },
    { 0x36e57598, 33 },
    { 0, 0 },
    { 0x7355759a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f5759e, 43 },
    { 0x8835759f, 23 },
    { 0, 0 },
    { 0x8ab575a1, 61 },
    { 0, 0 },
    { 0x7ca575a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69575a6, 8 },
    { 0xbaf575a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3575ae, 43 },
    { 0xc65575af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124575b3, 76 },
    { 0, 0 },
    { 0x1c2575b5, 87 },
    { 0, 0 },
    { 0x1bd575b7, 65 },
    { 0x9a8575b8, 95 },
    { 0xbfd575b9, 99 },
    { 0x575575ba, 61 },
    { 0xa9a575bb, 61 },
    { 0x67e575bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff575c1, 90 },
    { 0x1e4575c2, 95 },
    { 0xac4575c1, 85 },
    { 0xf80575c2, 66 },
    { 0x5a4575c5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc575c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147575da, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d575e4, 23 },
    { 0xf86575e5, 75 },
    { 0xdab575e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78575e9, 87 },
    { 0xe23575ea, 66 },
    { 0x375575e9, 61 },
    { 0x5ef575ea, 90 },
    { 0x584575ed, 100 },
    { 0x2fe575e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c575f2, 66 },
    { 0, 0 },
    { 0x1be575f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x847575f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e575fc, 90 },
    { 0xf70575fd, 87 },
    { 0x3d7575fd, 95 },
    { 0xe4d575ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f757602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88057605, 61 },
    { 0x27157606, 98 },
    { 0x44d57606, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65760b, 95 },
    { 0x4bb5760c, 90 },
    { 0x8ec5760c, 88 },
    { 0, 0 },
    { 0x4585760f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357617, 95 },
    { 0, 0 },
    { 0x8b557619, 95 },
    { 0, 0 },
    { 0x9945761b, 48 },
    { 0xa475761b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4757622, 23 },
    { 0x40a57622, 76 },
    { 0xe9457624, 34 },
    { 0xd0c57624, 98 },
    { 0x50b57626, 23 },
    { 0, 0 },
    { 0x41f57628, 90 },
    { 0xcf657629, 23 },
    { 0x6275762a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5762f, 90 },
    { 0x7a857630, 75 },
    { 0, 0 },
    { 0xa3557632, 61 },
    { 0, 0 },
    { 0xd7a57634, 100 },
    { 0xc7157635, 95 },
    { 0x91957636, 90 },
    { 0x76257637, 66 },
    { 0xf8857634, 33 },
    { 0x94657635, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae5763c, 55 },
    { 0xe4a5763c, 61 },
    { 0x5e35763e, 91 },
    { 0x9a85763f, 75 },
    { 0x25b57640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f57643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d57647, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15764a, 87 },
    { 0, 0 },
    { 0x12a5764c, 90 },
    { 0xc8c5764d, 99 },
    { 0x815764e, 23 },
    { 0xddf5764d, 55 },
    { 0, 0 },
    { 0xef857651, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1557654, 46 },
    { 0xd4157654, 23 },
    { 0xf5357654, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9157659, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e5765e, 61 },
    { 0x2ab5765e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b57663, 98 },
    { 0xce657664, 33 },
    { 0x21757665, 67 },
    { 0x45257665, 33 },
    { 0x91857667, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d5766c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc57670, 23 },
    { 0, 0 },
    { 0x69957672, 93 },
    { 0, 0 },
    { 0xb3c57674, 17 },
    { 0x7de57675, 88 },
    { 0x63857675, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d65767b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05767f, 93 },
    { 0xc3b57680, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57557684, 23 },
    { 0xe7c57685, 33 },
    { 0, 0 },
    { 0x6aa57687, 78 },
    { 0, 0 },
    { 0xa7557689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37157690, 43 },
    { 0x67657691, 87 },
    { 0xd2357690, 85 },
    { 0x29457693, 33 },
    { 0x7ea57694, 33 },
    { 0x85557695, 33 },
    { 0xa6157691, 66 },
    { 0xefa57690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5769c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314576a1, 87 },
    { 0xf0c576a2, 33 },
    { 0x9ed576a3, 55 },
    { 0x594576a4, 90 },
    { 0x1576a1, 33 },
    { 0xe4c576a6, 91 },
    { 0x238576a7, 98 },
    { 0xe0f576a8, 36 },
    { 0x375576a8, 34 },
    { 0, 0 },
    { 0xb61576ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82576ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb576b8, 85 },
    { 0x441576b9, 23 },
    { 0x949576b9, 55 },
    { 0x80a576b8, 90 },
    { 0x359576ba, 61 },
    { 0x6c7576b8, 82 },
    { 0xb6d576be, 46 },
    { 0x844576bf, 55 },
    { 0x104576bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890576c4, 36 },
    { 0x61b576c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3576ca, 90 },
    { 0x605576cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f576ce, 81 },
    { 0xebf576cf, 99 },
    { 0x774576d0, 98 },
    { 0, 0 },
    { 0xf31576d2, 85 },
    { 0, 0 },
    { 0xd1576d4, 98 },
    { 0x562576d5, 69 },
    { 0x155576d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b576d9, 65 },
    { 0, 0 },
    { 0x5f7576db, 40 },
    { 0x2dd576dc, 33 },
    { 0xba4576dd, 33 },
    { 0x9eb576db, 69 },
    { 0x6f576dd, 90 },
    { 0xa9f576dd, 99 },
    { 0x495576e0, 90 },
    { 0x660576dd, 48 },
    { 0xb95576db, 76 },
    { 0, 0 },
    { 0xc9a576e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x487576e8, 52 },
    { 0xd5a576e9, 90 },
    { 0xaf6576e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc576ed, 99 },
    { 0xb50576ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730576f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63576f8, 90 },
    { 0x198576f9, 76 },
    { 0xc13576fa, 95 },
    { 0xf8576fb, 23 },
    { 0xa0f576f8, 98 },
    { 0x831576f8, 33 },
    { 0, 0 },
    { 0x42f576ff, 95 },
    { 0, 0 },
    { 0xcc857701, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93457705, 98 },
    { 0x7c557706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf545770d, 95 },
    { 0x3dd5770d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf657714, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e57717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8035771b, 59 },
    { 0x2fa5771c, 90 },
    { 0xabe5771d, 52 },
    { 0x8d85771b, 93 },
    { 0x94f5771b, 23 },
    { 0, 0 },
    { 0xdf657721, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45157725, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70957729, 90 },
    { 0, 0 },
    { 0x9345772b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be5772f, 95 },
    { 0xa835772f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e25773b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225773e, 33 },
    { 0x5165773f, 61 },
    { 0x6cf5773f, 61 },
    { 0x10a57740, 69 },
    { 0xb8957742, 95 },
    { 0xbba5773f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba357747, 98 },
    { 0, 0 },
    { 0x7f757749, 78 },
    { 0, 0 },
    { 0x3665774b, 76 },
    { 0x89b5774b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf035774f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94457752, 40 },
    { 0, 0 },
    { 0xe9e57754, 33 },
    { 0x83f57754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2557760, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5957764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d57769, 46 },
    { 0x8ac57769, 94 },
    { 0xe805776b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61457770, 33 },
    { 0, 0 },
    { 0x74357772, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f57775, 101 },
    { 0xe0d57776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66957779, 98 },
    { 0xca05777a, 33 },
    { 0, 0 },
    { 0xba05777c, 98 },
    { 0, 0 },
    { 0x315777e, 76 },
    { 0xbde5777f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d57785, 67 },
    { 0x79d57786, 100 },
    { 0xff857786, 76 },
    { 0, 0 },
    { 0xb3f57789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16057790, 95 },
    { 0x61357790, 66 },
    { 0, 0 },
    { 0xf2257793, 55 },
    { 0, 0 },
    { 0xe7c57795, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f5779b, 23 },
    { 0xb285779c, 55 },
    { 0x6e05779d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79577a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756577ab, 34 },
    { 0, 0 },
    { 0x9d2577ad, 90 },
    { 0x6f1577ae, 42 },
    { 0x22577ad, 35 },
    { 0xf1a577b0, 33 },
    { 0x360577b1, 33 },
    { 0x720577b1, 84 },
    { 0x1de577b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a577b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52577c1, 34 },
    { 0, 0 },
    { 0x65e577c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0577cc, 33 },
    { 0x9a7577cd, 91 },
    { 0x6c8577ce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c577d2, 98 },
    { 0xc8b577d3, 28 },
    { 0, 0 },
    { 0xfb6577d5, 90 },
    { 0xbb3577d6, 98 },
    { 0x345577d7, 61 },
    { 0x2f4577d8, 33 },
    { 0x837577d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e577dc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e577e0, 100 },
    { 0, 0 },
    { 0xd08577e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6577ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d577f0, 90 },
    { 0xbd5577f0, 95 },
    { 0x3a4577f2, 98 },
    { 0x200577f3, 95 },
    { 0x8de577f3, 98 },
    { 0, 0 },
    { 0xf9f577f6, 23 },
    { 0x4d6577f7, 55 },
    { 0xd21577f7, 23 },
    { 0xe00577f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac577fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e57805, 84 },
    { 0x2f757805, 70 },
    { 0xe457807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345780c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f857810, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e57819, 67 },
    { 0x2735781a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455781d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a657820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6257824, 88 },
    { 0xfcc57824, 90 },
    { 0xe2a57825, 66 },
    { 0x1ee57827, 23 },
    { 0x6b257828, 90 },
    { 0xde557824, 90 },
    { 0, 0 },
    { 0x7355782b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c57830, 87 },
    { 0, 0 },
    { 0x1057832, 91 },
    { 0, 0 },
    { 0x37457834, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d57837, 90 },
    { 0xa8a57838, 52 },
    { 0x3f257837, 43 },
    { 0x1bf5783a, 90 },
    { 0, 0 },
    { 0x8cf5783c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec57841, 98 },
    { 0x25c57842, 55 },
    { 0x8c557843, 85 },
    { 0x6ef57843, 99 },
    { 0x5557845, 77 },
    { 0, 0 },
    { 0xac757847, 61 },
    { 0x9ea57848, 40 },
    { 0xf8b57847, 55 },
    { 0xd7557848, 61 },
    { 0xad55784b, 90 },
    { 0x4df5784b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f57850, 33 },
    { 0x72757851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe645785f, 100 },
    { 0x7f957860, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd957863, 17 },
    { 0x93d57864, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a357867, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795786a, 68 },
    { 0, 0 },
    { 0x7835786c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa457872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c157878, 34 },
    { 0xd6c57878, 33 },
    { 0x6885787a, 95 },
    { 0xdd65787b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1357883, 61 },
    { 0xccb57884, 23 },
    { 0x53357885, 60 },
    { 0xee957885, 95 },
    { 0xce057884, 93 },
    { 0x4d557888, 17 },
    { 0x69a57888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d5788d, 90 },
    { 0x42d5788e, 52 },
    { 0, 0 },
    { 0xabf57890, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8357895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d5789c, 38 },
    { 0xdbd5789d, 88 },
    { 0xe495789d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb578a1, 23 },
    { 0x66578a2, 67 },
    { 0xa12578a1, 90 },
    { 0x211578a4, 66 },
    { 0x815578a4, 23 },
    { 0x9e7578a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc578a9, 100 },
    { 0, 0 },
    { 0xd67578ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91578af, 61 },
    { 0x8cf578af, 56 },
    { 0x41e578b1, 33 },
    { 0x762578b2, 85 },
    { 0x6af578b3, 33 },
    { 0xe93578b4, 68 },
    { 0x513578b0, 33 },
    { 0, 0 },
    { 0x4d6578b7, 95 },
    { 0, 0 },
    { 0xf9f578b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b578c3, 90 },
    { 0x403578c4, 95 },
    { 0x965578c5, 33 },
    { 0x63578c6, 95 },
    { 0, 0 },
    { 0x3f5578c8, 95 },
    { 0xbcc578c9, 33 },
    { 0xd02578ca, 34 },
    { 0x6c3578cb, 90 },
    { 0x211578cc, 98 },
    { 0xcc1578cc, 93 },
    { 0x486578c8, 78 },
    { 0xb21578ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29578d2, 23 },
    { 0, 0 },
    { 0x687578d4, 33 },
    { 0x881578d5, 90 },
    { 0, 0 },
    { 0x587578d7, 91 },
    { 0x121578d7, 87 },
    { 0, 0 },
    { 0x8a578da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56578df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc578e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7578f1, 90 },
    { 0x1fd578f2, 23 },
    { 0x3bd578f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28578fc, 91 },
    { 0xfba578fd, 33 },
    { 0xae2578fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9357906, 84 },
    { 0, 0 },
    { 0x7f857908, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b85790e, 66 },
    { 0x6465790f, 61 },
    { 0xe285790e, 90 },
    { 0x4fe57910, 55 },
    { 0x34d5790e, 61 },
    { 0xff257913, 33 },
    { 0x2b057913, 43 },
    { 0xcf757915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8457919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd57920, 97 },
    { 0xbef57921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79057928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc5792c, 84 },
    { 0, 0 },
    { 0x2795792e, 96 },
    { 0, 0 },
    { 0x36857930, 53 },
    { 0x1ad57931, 55 },
    { 0x1d857930, 99 },
    { 0x88957932, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed57936, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d57939, 61 },
    { 0x4db5793a, 87 },
    { 0x2d95793a, 81 },
    { 0x69f5793c, 90 },
    { 0x65c5793d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f57941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2657944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe135794b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd657951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57957, 61 },
    { 0, 0 },
    { 0x6bc57959, 46 },
    { 0xcf5795a, 77 },
    { 0x80b5795b, 100 },
    { 0xcea57959, 94 },
    { 0, 0 },
    { 0xc525795e, 85 },
    { 0, 0 },
    { 0xd7b57960, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b57964, 95 },
    { 0, 0 },
    { 0x43657966, 90 },
    { 0x65857966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd655796d, 61 },
    { 0x2dd5796d, 78 },
    { 0, 0 },
    { 0x8b857970, 95 },
    { 0, 0 },
    { 0xa8757972, 90 },
    { 0, 0 },
    { 0x47b57974, 76 },
    { 0x58b57975, 23 },
    { 0xf3657976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0657979, 92 },
    { 0xe165797a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5797d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c557982, 33 },
    { 0x99257982, 90 },
    { 0, 0 },
    { 0xe1257985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b57989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5798c, 67 },
    { 0, 0 },
    { 0xc8a5798e, 66 },
    { 0, 0 },
    { 0x97f57990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e857993, 84 },
    { 0x85557993, 33 },
    { 0x8cc57995, 61 },
    { 0x6e157996, 61 },
    { 0x2b357997, 98 },
    { 0xa0157994, 61 },
    { 0x13057993, 95 },
    { 0x41a57997, 33 },
    { 0xdef5799b, 33 },
    { 0, 0 },
    { 0x5515799d, 23 },
    { 0xa655799e, 53 },
    { 0xefc5799e, 93 },
    { 0x562579a0, 68 },
    { 0xc815799f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda579a6, 46 },
    { 0x32f579a7, 43 },
    { 0x46a579a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20579ae, 43 },
    { 0xfdd579af, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd579b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e579bc, 61 },
    { 0, 0 },
    { 0xf56579be, 91 },
    { 0x518579bf, 85 },
    { 0, 0 },
    { 0x876579c1, 90 },
    { 0, 0 },
    { 0x572579c3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4579c6, 90 },
    { 0, 0 },
    { 0x86c579c8, 99 },
    { 0xcf9579c8, 43 },
    { 0, 0 },
    { 0xb2a579cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c579ce, 91 },
    { 0xd36579cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659579da, 85 },
    { 0xecd579db, 87 },
    { 0x87f579db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c579df, 61 },
    { 0x285579e0, 90 },
    { 0, 0 },
    { 0x8d579e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509579e6, 98 },
    { 0, 0 },
    { 0x2a8579e8, 81 },
    { 0x982579e9, 93 },
    { 0x91579e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc579ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66579f3, 33 },
    { 0x81f579f4, 33 },
    { 0xfa8579f4, 43 },
    { 0, 0 },
    { 0xe72579f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b579fa, 61 },
    { 0, 0 },
    { 0x2aa579fc, 77 },
    { 0xf20579fd, 55 },
    { 0x36579fe, 90 },
    { 0, 0 },
    { 0x4db57a00, 78 },
    { 0, 0 },
    { 0x24257a02, 91 },
    { 0xa5357a03, 23 },
    { 0x56e57a04, 4 },
    { 0, 0 },
    { 0xdf257a06, 23 },
    { 0xff157a07, 90 },
    { 0xfac57a08, 33 },
    { 0x2ab57a09, 95 },
    { 0x10b57a09, 100 },
    { 0x6257a08, 23 },
    { 0x20c57a0c, 99 },
    { 0xf157a0c, 98 },
    { 0x11b57a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22257a1c, 33 },
    { 0x14757a1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d757a21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a57a24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa57a2d, 90 },
    { 0x9e957a2e, 34 },
    { 0x5f857a2e, 43 },
    { 0xf2557a30, 56 },
    { 0xfbf57a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d57a34, 33 },
    { 0, 0 },
    { 0x7b557a36, 95 },
    { 0x36d57a36, 100 },
    { 0xca57a38, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b57a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6557a45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c057a49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f57a51, 46 },
    { 0xc7857a52, 91 },
    { 0, 0 },
    { 0x4c057a54, 61 },
    { 0xae57a55, 76 },
    { 0xe5857a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3357a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a57a60, 33 },
    { 0, 0 },
    { 0xa8a57a62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68657a6b, 61 },
    { 0x18b57a6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1157a70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f57a7b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x94957a7e, 95 },
    { 0, 0 },
    { 0x56057a80, 95 },
    { 0xf8357a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62257a87, 100 },
    { 0, 0 },
    { 0x76e57a89, 23 },
    { 0xce857a8a, 90 },
    { 0, 0 },
    { 0xd7857a8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f57a90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92957a93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d57a98, 34 },
    { 0xf2257a99, 43 },
    { 0xf0257a98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d757a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93057aa4, 66 },
    { 0x31757aa5, 91 },
    { 0, 0 },
    { 0x6cb57aa7, 98 },
    { 0xb8757aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc57aad, 61 },
    { 0x71757aae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a57ab1, 95 },
    { 0, 0 },
    { 0x68957ab3, 90 },
    { 0, 0 },
    { 0xb9557ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6857abc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca57abf, 67 },
    { 0x1e57ac0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0057ac3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9157ac8, 43 },
    { 0xf5757ac9, 98 },
    { 0xca857ac9, 75 },
    { 0xe457acb, 61 },
    { 0x47e57acb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d57ad0, 93 },
    { 0x34057ad0, 61 },
    { 0xf0d57ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e57ad5, 90 },
    { 0, 0 },
    { 0xffc57ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed357ae3, 98 },
    { 0x19e57ae4, 93 },
    { 0x2f157ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3157aea, 97 },
    { 0xcfa57aeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37757aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9357af3, 61 },
    { 0x28057af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9157afb, 46 },
    { 0, 0 },
    { 0x5f757afd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe257b01, 77 },
    { 0xb4e57b02, 46 },
    { 0x32a57b03, 55 },
    { 0, 0 },
    { 0xe4257b05, 34 },
    { 0x35e57b05, 98 },
    { 0x82d57b07, 61 },
    { 0, 0 },
    { 0xb3157b09, 95 },
    { 0x93157b09, 90 },
    { 0x4057b09, 61 },
    { 0x9657b0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea57b11, 98 },
    { 0x6e857b11, 90 },
    { 0x3f357b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d57b17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7957b1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98757b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c57b2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84957b2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c57b34, 90 },
    { 0x86857b34, 33 },
    { 0x82257b34, 33 },
    { 0x77f57b37, 99 },
    { 0, 0 },
    { 0x80457b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2157b3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac57b3f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73557b43, 61 },
    { 0, 0 },
    { 0x1c57b45, 19 },
    { 0x10e57b45, 100 },
    { 0x52557b47, 38 },
    { 0, 0 },
    { 0x7ec57b49, 80 },
    { 0x4a057b49, 40 },
    { 0xf0857b49, 33 },
    { 0xa9857b49, 69 },
    { 0x7ba57b4b, 95 },
    { 0xac757b4e, 95 },
    { 0x82557b4e, 33 },
    { 0xfa157b50, 61 },
    { 0xfee57b50, 87 },
    { 0x20557b4c, 61 },
    { 0xc2857b53, 95 },
    { 0, 0 },
    { 0xc9157b55, 61 },
    { 0x62757b56, 61 },
    { 0xf3657b56, 90 },
    { 0x82457b56, 95 },
    { 0x20457b59, 93 },
    { 0x23e57b57, 76 },
    { 0xede57b5b, 76 },
    { 0x5fd57b5b, 43 },
    { 0x19357b5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56257b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c57b64, 95 },
    { 0xe4f57b65, 76 },
    { 0xaf857b64, 52 },
    { 0, 0 },
    { 0xa6a57b68, 98 },
    { 0xacd57b69, 23 },
    { 0x98d57b6a, 66 },
    { 0x5ac57b6a, 91 },
    { 0xf0557b6b, 98 },
    { 0xcc557b6d, 46 },
    { 0x9b557b69, 61 },
    { 0xe2c57b69, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3057b72, 61 },
    { 0x9db57b73, 98 },
    { 0x94657b74, 52 },
    { 0x66257b72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4257b78, 78 },
    { 0, 0 },
    { 0x69157b7a, 90 },
    { 0, 0 },
    { 0x80b57b7c, 61 },
    { 0, 0 },
    { 0x3c257b7e, 34 },
    { 0xe6e57b7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24357b84, 95 },
    { 0xf0a57b84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf57b89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8657b8c, 34 },
    { 0, 0 },
    { 0xc3757b8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb357b98, 61 },
    { 0, 0 },
    { 0x25757b9a, 95 },
    { 0, 0 },
    { 0x29357b9c, 90 },
    { 0xf8f57b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e857ba2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc57ba6, 98 },
    { 0xebb57ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5557bab, 55 },
    { 0, 0 },
    { 0x3fa57bad, 51 },
    { 0x66457bae, 95 },
    { 0x3757baf, 90 },
    { 0x52957baf, 90 },
    { 0xdd657baf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c657bb5, 90 },
    { 0, 0 },
    { 0x96357bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad57bba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c57bbf, 90 },
    { 0, 0 },
    { 0xba757bc1, 23 },
    { 0x38357bc1, 95 },
    { 0x25057bc1, 84 },
    { 0xa2257bc2, 34 },
    { 0, 0 },
    { 0x9d157bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d57bc9, 33 },
    { 0x17a57bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac57bd0, 90 },
    { 0xd8757bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe57bd4, 61 },
    { 0x98c57bd5, 61 },
    { 0x44157bd5, 43 },
    { 0x1b557bd4, 23 },
    { 0x28157bd8, 33 },
    { 0xa8757bd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56257be3, 36 },
    { 0x8bf57be4, 83 },
    { 0x19657be3, 23 },
    { 0xc2c57be6, 84 },
    { 0, 0 },
    { 0x57757be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf57beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a57bf1, 55 },
    { 0x4f357bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc757c01, 33 },
    { 0x46557c02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f57c06, 98 },
    { 0xd9957c07, 84 },
    { 0, 0 },
    { 0x5e57c09, 81 },
    { 0x78657c0a, 59 },
    { 0, 0 },
    { 0xc9057c0c, 93 },
    { 0, 0 },
    { 0x93257c0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb257c15, 90 },
    { 0x21e57c16, 90 },
    { 0x70057c17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac57c1a, 33 },
    { 0, 0 },
    { 0xec857c1c, 52 },
    { 0xfe357c1d, 68 },
    { 0, 0 },
    { 0xac457c1f, 95 },
    { 0, 0 },
    { 0x3c357c21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91357c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4457c31, 66 },
    { 0x3c657c32, 33 },
    { 0x99d57c33, 98 },
    { 0xd8757c34, 90 },
    { 0xf9257c34, 80 },
    { 0xb2857c34, 90 },
    { 0xf0357c37, 40 },
    { 0x13657c37, 66 },
    { 0x41657c37, 23 },
    { 0, 0 },
    { 0xa57c3b, 91 },
    { 0xa9657c3b, 40 },
    { 0x1b057c3d, 52 },
    { 0xd357c3d, 76 },
    { 0x17857c3d, 23 },
    { 0x93e57c40, 66 },
    { 0xba257c40, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae57c44, 81 },
    { 0, 0 },
    { 0xf5f57c46, 33 },
    { 0, 0 },
    { 0xf257c48, 87 },
    { 0x88057c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc57c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e57c53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54257c57, 61 },
    { 0xf5b57c57, 56 },
    { 0x33757c59, 90 },
    { 0, 0 },
    { 0xe8157c5b, 75 },
    { 0xaa157c5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0157c5f, 99 },
    { 0xc5b57c60, 61 },
    { 0x10557c60, 33 },
    { 0x4ed57c60, 95 },
    { 0x14357c63, 78 },
    { 0xb6357c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50557c69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd757c70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b57c73, 90 },
    { 0, 0 },
    { 0x8f857c75, 93 },
    { 0xc5357c76, 90 },
    { 0, 0 },
    { 0xe0257c78, 61 },
    { 0xd0f57c79, 98 },
    { 0x52457c79, 23 },
    { 0x8cd57c78, 85 },
    { 0x77c57c7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xea957c7f, 99 },
    { 0x6b157c80, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0157c83, 61 },
    { 0x67f57c84, 90 },
    { 0xdac57c83, 55 },
    { 0xdde57c86, 95 },
    { 0xed457c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf457c8c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31957c92, 52 },
    { 0xfd157c93, 97 },
    { 0, 0 },
    { 0xbbd57c95, 33 },
    { 0xa7657c96, 93 },
    { 0x25c57c97, 23 },
    { 0x1e457c98, 61 },
    { 0xa6b57c99, 90 },
    { 0xc7657c98, 76 },
    { 0x60057c97, 61 },
    { 0, 0 },
    { 0x9cc57c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13557ca2, 43 },
    { 0xce657ca3, 99 },
    { 0x64357ca4, 95 },
    { 0xd9357ca5, 90 },
    { 0x90c57ca6, 52 },
    { 0x7cd57ca6, 84 },
    { 0, 0 },
    { 0x1d557ca9, 33 },
    { 0x85e57ca9, 98 },
    { 0x89e57cab, 33 },
    { 0x31c57cac, 88 },
    { 0x5ed57cad, 91 },
    { 0xe4257cab, 95 },
    { 0x32e57caf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd057cb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e57cb8, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55457cd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48257cd6, 90 },
    { 0, 0 },
    { 0x7bd57cd8, 52 },
    { 0xdee57cd9, 61 },
    { 0xc8e57cd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb057cde, 90 },
    { 0x3c757cde, 95 },
    { 0, 0 },
    { 0xe2d57ce1, 23 },
    { 0, 0 },
    { 0x53857ce3, 61 },
    { 0x1557ce3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9057ce8, 90 },
    { 0x48657ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60457cec, 61 },
    { 0x7857ced, 100 },
    { 0x89457cee, 90 },
    { 0x5d357cef, 61 },
    { 0xdf457cf0, 23 },
    { 0, 0 },
    { 0x6e957cf2, 90 },
    { 0xe557cf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657cf6, 84 },
    { 0, 0 },
    { 0x38b57cf8, 90 },
    { 0, 0 },
    { 0x17a57cfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b57cff, 41 },
    { 0, 0 },
    { 0xfa757d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99257d05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1457d13, 46 },
    { 0x81157d14, 3 },
    { 0x8e57d15, 52 },
    { 0, 0 },
    { 0x45f57d17, 98 },
    { 0xfa757d18, 87 },
    { 0xbed57d18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad57d1e, 98 },
    { 0x28057d1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x93157d22, 87 },
    { 0, 0 },
    { 0x5e157d24, 76 },
    { 0x4e157d25, 43 },
    { 0, 0 },
    { 0x12157d27, 34 },
    { 0, 0 },
    { 0xc6f57d29, 43 },
    { 0, 0 },
    { 0xd3d57d2b, 76 },
    { 0x8b257d2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b57d31, 66 },
    { 0, 0 },
    { 0x65757d33, 34 },
    { 0, 0 },
    { 0x19f57d35, 65 },
    { 0xbeb57d36, 100 },
    { 0xfd857d37, 85 },
    { 0xad857d38, 23 },
    { 0xe5657d35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32357d43, 87 },
    { 0x40557d44, 98 },
    { 0x14557d45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb57d48, 19 },
    { 0x40e57d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa857d4c, 61 },
    { 0, 0 },
    { 0x13c57d4e, 76 },
    { 0x8f57d4f, 34 },
    { 0x42557d50, 98 },
    { 0, 0 },
    { 0x6c857d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c57d56, 92 },
    { 0x54e57d56, 43 },
    { 0xbef57d58, 33 },
    { 0xb4957d59, 78 },
    { 0x88057d57, 43 },
    { 0x7057d5a, 61 },
    { 0xd6c57d59, 93 },
    { 0x11157d59, 33 },
    { 0x4ef57d5e, 90 },
    { 0xcd357d5c, 33 },
    { 0xd6357d60, 68 },
    { 0x9b057d61, 67 },
    { 0x8c357d61, 33 },
    { 0xb8b57d61, 95 },
    { 0xb4157d61, 34 },
    { 0x8ed57d65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57357d70, 61 },
    { 0xb2f57d71, 61 },
    { 0xc857d71, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39557d75, 61 },
    { 0xda357d76, 33 },
    { 0xfd757d77, 95 },
    { 0xa9457d77, 99 },
    { 0x3af57d79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9557d7c, 90 },
    { 0xba857d7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e57d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c57d88, 55 },
    { 0, 0 },
    { 0xdfa57d8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c857d8d, 95 },
    { 0x3257d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16657d91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d257d96, 84 },
    { 0xb8d57d97, 34 },
    { 0x9dc57d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b57d9b, 98 },
    { 0, 0 },
    { 0x22757d9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a257da3, 33 },
    { 0xe9057da3, 90 },
    { 0x43857da3, 99 },
    { 0xe6b57da5, 33 },
    { 0xd257da3, 90 },
    { 0xaa157da8, 98 },
    { 0xaeb57da9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae57dac, 32 },
    { 0x3cb57dad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a57db0, 61 },
    { 0x7a857db1, 78 },
    { 0x10e57db2, 98 },
    { 0x3ec57db3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49557dba, 90 },
    { 0x40457dbb, 43 },
    { 0xb6957dbb, 84 },
    { 0x61457dbc, 98 },
    { 0x5a657dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17157dc2, 76 },
    { 0xb2d57dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b57dce, 33 },
    { 0, 0 },
    { 0x96b57dd0, 98 },
    { 0x49d57dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49057dd4, 90 },
    { 0x55457dd4, 90 },
    { 0x4b357dd5, 76 },
    { 0xb9a57dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1757ddb, 66 },
    { 0x89357ddc, 88 },
    { 0x38c57ddc, 61 },
    { 0xbcf57dde, 43 },
    { 0x4c657ddf, 95 },
    { 0xe7f57ddf, 95 },
    { 0, 0 },
    { 0x32357de2, 90 },
    { 0, 0 },
    { 0xde957de4, 61 },
    { 0x35c57de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f57dea, 23 },
    { 0x49c57dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057df0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d57df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98157df9, 90 },
    { 0xb5157df9, 100 },
    { 0, 0 },
    { 0xd4557dfc, 91 },
    { 0x98e57dfd, 33 },
    { 0x7bc57dfe, 76 },
    { 0x80a57dfe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95557e02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd157e05, 56 },
    { 0, 0 },
    { 0x4ed57e07, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78357e0a, 34 },
    { 0xebe57e0b, 34 },
    { 0, 0 },
    { 0x5d757e0d, 61 },
    { 0, 0 },
    { 0x35057e0f, 90 },
    { 0xd1557e10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9157e13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42457e1c, 23 },
    { 0x5aa57e1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b957e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a057e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8057e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e57e31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68757e34, 66 },
    { 0xe7657e35, 93 },
    { 0x5ab57e36, 84 },
    { 0xf8857e36, 33 },
    { 0x86457e35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f57e3d, 33 },
    { 0x1d357e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5757e41, 78 },
    { 0xbef57e41, 52 },
    { 0x19a57e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d457e49, 84 },
    { 0xf7457e49, 91 },
    { 0xa9157e4b, 90 },
    { 0, 0 },
    { 0x84e57e4d, 90 },
    { 0x25c57e4e, 23 },
    { 0x21457e4f, 93 },
    { 0xf3f57e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85957e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94557e57, 68 },
    { 0, 0 },
    { 0xed357e59, 98 },
    { 0xa7f57e59, 34 },
    { 0, 0 },
    { 0x1657e5c, 78 },
    { 0x7cd57e5d, 90 },
    { 0x77657e5d, 34 },
    { 0xb4757e5f, 76 },
    { 0, 0 },
    { 0x64c57e61, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9057e69, 81 },
    { 0, 0 },
    { 0xecb57e6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71657e6e, 95 },
    { 0x79557e6f, 76 },
    { 0x5fc57e6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac57e73, 43 },
    { 0x6ad57e74, 55 },
    { 0x3d957e74, 23 },
    { 0xe1057e73, 33 },
    { 0x36757e77, 76 },
    { 0x8a357e78, 90 },
    { 0, 0 },
    { 0xc0e57e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6957e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5557e83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46457e87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18157e8a, 61 },
    { 0, 0 },
    { 0x6a357e8c, 91 },
    { 0x2b657e8c, 98 },
    { 0xdd657e8d, 61 },
    { 0x8e57e8f, 87 },
    { 0, 0 },
    { 0x77857e91, 61 },
    { 0x2e757e92, 52 },
    { 0xec957e92, 23 },
    { 0xd9757e92, 33 },
    { 0x3c257e91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5157e9c, 23 },
    { 0xcfa57e9d, 23 },
    { 0, 0 },
    { 0xef257e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa57ea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b257ea7, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54957eae, 90 },
    { 0x6f857eaf, 61 },
    { 0x78057eaf, 46 },
    { 0x8b257eb0, 33 },
    { 0x21457eb2, 40 },
    { 0xc4257eb0, 90 },
    { 0x7ef57eb0, 43 },
    { 0x5c057eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e57ec4, 52 },
    { 0x40e57ec5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8557ec8, 43 },
    { 0xf057ec8, 99 },
    { 0x83b57eca, 90 },
    { 0x85657ecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19757ed2, 95 },
    { 0, 0 },
    { 0x8f357ed4, 23 },
    { 0xf8c57ed4, 33 },
    { 0xf6b57ed4, 100 },
    { 0x1fa57ed7, 69 },
    { 0xd3e57ed8, 95 },
    { 0, 0 },
    { 0x5b057eda, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa157ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd457ee4, 91 },
    { 0x68e57ee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657eea, 52 },
    { 0xf0257eeb, 100 },
    { 0, 0 },
    { 0xb857eed, 33 },
    { 0x58557eee, 55 },
    { 0, 0 },
    { 0xf1657ef0, 95 },
    { 0x23357ef1, 93 },
    { 0x1b157ef2, 61 },
    { 0, 0 },
    { 0xade57ef4, 95 },
    { 0x96d57ef5, 100 },
    { 0x81157ef6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28657ef9, 61 },
    { 0, 0 },
    { 0x46357efb, 23 },
    { 0, 0 },
    { 0xb6b57efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba57f0d, 43 },
    { 0x16957f0e, 44 },
    { 0, 0 },
    { 0x6b157f10, 88 },
    { 0xd3e57f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca57f14, 91 },
    { 0x71857f14, 61 },
    { 0xda657f14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c57f1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c457f1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd857f25, 23 },
    { 0, 0 },
    { 0x7257f27, 76 },
    { 0, 0 },
    { 0x94157f29, 17 },
    { 0x24a57f2a, 33 },
    { 0x30257f2b, 23 },
    { 0x59257f2b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd257f31, 43 },
    { 0x80c57f32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c57f35, 63 },
    { 0x2ac57f36, 33 },
    { 0x46d57f37, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27157f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb957f3f, 43 },
    { 0x9b157f40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8157f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2757f46, 84 },
    { 0xd1c57f46, 90 },
    { 0xa7a57f48, 93 },
    { 0xff857f48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce57f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a57f55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e57f5a, 61 },
    { 0x28257f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea57f60, 55 },
    { 0x8e357f61, 64 },
    { 0x53a57f62, 95 },
    { 0x98257f63, 36 },
    { 0xff057f64, 61 },
    { 0xd9e57f63, 33 },
    { 0, 0 },
    { 0x72a57f67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66257f6d, 42 },
    { 0, 0 },
    { 0xd7357f6f, 23 },
    { 0, 0 },
    { 0x6757f71, 23 },
    { 0, 0 },
    { 0xcc057f73, 33 },
    { 0, 0 },
    { 0x94357f75, 94 },
    { 0, 0 },
    { 0x85b57f77, 95 },
    { 0, 0 },
    { 0xe3157f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda057f7c, 99 },
    { 0xfc357f7c, 23 },
    { 0, 0 },
    { 0x80b57f7f, 23 },
    { 0, 0 },
    { 0xc5157f81, 23 },
    { 0x4f57f82, 95 },
    { 0xce057f82, 23 },
    { 0xd6157f84, 50 },
    { 0x87f57f84, 93 },
    { 0x4b257f86, 100 },
    { 0x7257f87, 98 },
    { 0xc7957f85, 55 },
    { 0x59957f89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c657f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2757f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa57f98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba57f9b, 23 },
    { 0xbd257f9c, 61 },
    { 0x51757f9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4057fa3, 90 },
    { 0x1a157fa4, 61 },
    { 0x7f657fa5, 95 },
    { 0, 0 },
    { 0xcdd57fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9457faa, 95 },
    { 0x3e057fab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7857fb2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8657fb5, 55 },
    { 0, 0 },
    { 0x9f657fb7, 90 },
    { 0x2e057fb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c57fbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af57fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb657fc1, 75 },
    { 0, 0 },
    { 0x6c457fc3, 61 },
    { 0, 0 },
    { 0xbd57fc5, 33 },
    { 0x97a57fc6, 23 },
    { 0, 0 },
    { 0x53457fc8, 95 },
    { 0, 0 },
    { 0xf6457fca, 95 },
    { 0x7257fca, 55 },
    { 0xb0f57fcc, 61 },
    { 0xffa57fcb, 95 },
    { 0, 0 },
    { 0x13057fcf, 64 },
    { 0x8f657fd0, 23 },
    { 0xa9c57fd0, 43 },
    { 0, 0 },
    { 0x43557fd3, 61 },
    { 0xd5457fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5157fd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b57fdc, 80 },
    { 0, 0 },
    { 0xbf657fde, 90 },
    { 0, 0 },
    { 0xabd57fe0, 95 },
    { 0x74857fe0, 43 },
    { 0xf6657fe2, 61 },
    { 0x17557fe2, 61 },
    { 0xff557fe3, 52 },
    { 0, 0 },
    { 0x29257fe6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9257fea, 90 },
    { 0xb8257fea, 43 },
    { 0xb1057fea, 68 },
    { 0xaa757fea, 33 },
    { 0x5657fee, 95 },
    { 0, 0 },
    { 0x75c57ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757ff4, 76 },
    { 0, 0 },
    { 0xb3657ff6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b57ffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe457ffe, 34 },
    { 0, 0 },
    { 0x76058000, 46 },
    { 0xcc158000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858004, 81 },
    { 0x99458004, 90 },
    { 0x3a858006, 23 },
    { 0, 0 },
    { 0x99558008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8855800d, 95 },
    { 0x3d95800d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3758011, 61 },
    { 0x12b58012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa025801a, 46 },
    { 0x72d5801b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a5801e, 42 },
    { 0, 0 },
    { 0x84a58020, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e58024, 46 },
    { 0x87558025, 46 },
    { 0x49b58025, 32 },
    { 0x39458026, 87 },
    { 0x4ba58027, 95 },
    { 0xc3258026, 55 },
    { 0x1305802a, 23 },
    { 0xe7b5802b, 81 },
    { 0x89b58025, 33 },
    { 0, 0 },
    { 0xab55802e, 23 },
    { 0, 0 },
    { 0xaf58030, 98 },
    { 0x35058031, 66 },
    { 0, 0 },
    { 0x8458033, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a58039, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25803e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea58043, 93 },
    { 0x24458044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97158047, 95 },
    { 0, 0 },
    { 0xd5658049, 85 },
    { 0x4465804a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd5804e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a058053, 23 },
    { 0x10058054, 98 },
    { 0x41558054, 61 },
    { 0x44c58054, 90 },
    { 0x1d658054, 76 },
    { 0x27e58057, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded58061, 23 },
    { 0x16258062, 43 },
    { 0xf2f58061, 90 },
    { 0xb3358064, 78 },
    { 0, 0 },
    { 0xd3158066, 76 },
    { 0xc8958067, 67 },
    { 0x5b558068, 98 },
    { 0xba258068, 51 },
    { 0xef05806a, 93 },
    { 0x2bc5806b, 61 },
    { 0x9495806c, 76 },
    { 0x4035806b, 55 },
    { 0x59d58068, 95 },
    { 0xd855806f, 100 },
    { 0, 0 },
    { 0x98258071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f58078, 90 },
    { 0x19a58079, 51 },
    { 0x7bc5807a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065807f, 33 },
    { 0, 0 },
    { 0x1a458081, 23 },
    { 0xdd058081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d58085, 90 },
    { 0x10458086, 76 },
    { 0x92658087, 91 },
    { 0x7fa58086, 95 },
    { 0x18e58089, 66 },
    { 0, 0 },
    { 0x9705808b, 95 },
    { 0xfca5808c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb5808f, 90 },
    { 0x3055808f, 33 },
    { 0xb115808f, 76 },
    { 0, 0 },
    { 0xe2b58093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0358096, 46 },
    { 0xd2658096, 33 },
    { 0xa6c58097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925809d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5580a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7580aa, 55 },
    { 0, 0 },
    { 0x357580ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881580b0, 61 },
    { 0xbcd580b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x364580b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3580b8, 76 },
    { 0xf8580b9, 23 },
    { 0, 0 },
    { 0x4c2580bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d580be, 61 },
    { 0x5b0580bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf580c5, 76 },
    { 0xab2580c6, 92 },
    { 0xcb4580c6, 34 },
    { 0, 0 },
    { 0xbc9580c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3580ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441580d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de580d5, 76 },
    { 0, 0 },
    { 0x272580d7, 98 },
    { 0x7d1580d8, 55 },
    { 0x9fb580d9, 76 },
    { 0, 0 },
    { 0x8fd580db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f580e6, 100 },
    { 0xf16580e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717580ec, 84 },
    { 0x5d4580ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53580f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28580f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df580f9, 99 },
    { 0xc7f580fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b58104, 98 },
    { 0x54758105, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa58109, 68 },
    { 0xe5658109, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5035810f, 95 },
    { 0x1fa58110, 23 },
    { 0x2d75810f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e58115, 4 },
    { 0x1d858115, 33 },
    { 0, 0 },
    { 0xc2c58118, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb5811f, 95 },
    { 0, 0 },
    { 0x7a958121, 33 },
    { 0xc2458122, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1775812b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a5812e, 98 },
    { 0, 0 },
    { 0x87b58130, 87 },
    { 0, 0 },
    { 0xd158132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f58135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe385813b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc315813e, 99 },
    { 0x71f5813f, 87 },
    { 0x2a65813e, 99 },
    { 0, 0 },
    { 0x33e58142, 33 },
    { 0, 0 },
    { 0xbf958144, 66 },
    { 0x42e58145, 93 },
    { 0xd5e58146, 81 },
    { 0x50258147, 93 },
    { 0, 0 },
    { 0xd7258149, 76 },
    { 0, 0 },
    { 0xd55814b, 99 },
    { 0, 0 },
    { 0x7205814d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d58152, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef558156, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea58159, 52 },
    { 0, 0 },
    { 0xad65815b, 23 },
    { 0, 0 },
    { 0x44f5815d, 90 },
    { 0xe585815e, 59 },
    { 0x7dc5815d, 43 },
    { 0xf4558160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de58163, 95 },
    { 0x58258164, 90 },
    { 0, 0 },
    { 0x4d258166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5816b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a958174, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a58177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a5817a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67158182, 23 },
    { 0xfaa58183, 95 },
    { 0x4ee58184, 23 },
    { 0x71c58184, 55 },
    { 0x56658184, 90 },
    { 0, 0 },
    { 0x96358188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55818b, 98 },
    { 0, 0 },
    { 0x6765818d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2958191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37958194, 33 },
    { 0x96158194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb158198, 95 },
    { 0x61658199, 23 },
    { 0x9765819a, 90 },
    { 0, 0 },
    { 0x3365819c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771581a3, 61 },
    { 0x3b5581a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228581aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1581b1, 58 },
    { 0x1ba581b2, 95 },
    { 0xc8b581b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a581b6, 90 },
    { 0x4b0581b7, 98 },
    { 0, 0 },
    { 0x59a581b9, 46 },
    { 0xb41581b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1581c5, 46 },
    { 0, 0 },
    { 0x486581c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71581ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a581cd, 90 },
    { 0, 0 },
    { 0x80f581cf, 68 },
    { 0, 0 },
    { 0x45a581d1, 61 },
    { 0, 0 },
    { 0x3c8581d3, 90 },
    { 0xa6a581d4, 23 },
    { 0xfa581d4, 43 },
    { 0xab581d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4581db, 61 },
    { 0, 0 },
    { 0x7cb581dd, 33 },
    { 0, 0 },
    { 0x652581df, 90 },
    { 0xfe581df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b581e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162581ed, 76 },
    { 0x4fd581ed, 61 },
    { 0xb93581ee, 77 },
    { 0xb0581ed, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9581f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120581f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7581fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e58201, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e758209, 88 },
    { 0x2fa58209, 40 },
    { 0x6cd5820a, 33 },
    { 0x5705820c, 61 },
    { 0xcfa5820d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015821a, 46 },
    { 0x4455821a, 90 },
    { 0xac85821c, 92 },
    { 0x6305821b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96158224, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd58229, 90 },
    { 0xbb65822a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc445822e, 87 },
    { 0x9c65822e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f58232, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e658239, 91 },
    { 0, 0 },
    { 0x3e35823b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6558247, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5265824b, 23 },
    { 0, 0 },
    { 0x44f5824d, 95 },
    { 0, 0 },
    { 0xf745824f, 23 },
    { 0, 0 },
    { 0x72658251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3558255, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2585825e, 43 },
    { 0xf415825f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b58263, 95 },
    { 0x97258263, 93 },
    { 0, 0 },
    { 0xd158266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c558269, 23 },
    { 0x1ce5826a, 90 },
    { 0, 0 },
    { 0x9545826c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c458270, 33 },
    { 0, 0 },
    { 0x59858272, 69 },
    { 0xc7058272, 100 },
    { 0, 0 },
    { 0xc7458275, 23 },
    { 0, 0 },
    { 0x72858277, 95 },
    { 0, 0 },
    { 0x57358279, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b5827e, 90 },
    { 0, 0 },
    { 0x58058280, 81 },
    { 0xc0658281, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3258289, 90 },
    { 0, 0 },
    { 0x2405828b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d85828f, 76 },
    { 0, 0 },
    { 0x5a258291, 90 },
    { 0, 0 },
    { 0xc5458293, 61 },
    { 0, 0 },
    { 0xf6a58295, 43 },
    { 0xa4058295, 91 },
    { 0xa5058296, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5582a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277582a5, 95 },
    { 0, 0 },
    { 0x80e582a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d582aa, 76 },
    { 0, 0 },
    { 0x1f3582ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528582b2, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370582ba, 34 },
    { 0x24b582bb, 34 },
    { 0, 0 },
    { 0x8e7582bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e582c3, 46 },
    { 0, 0 },
    { 0x5f2582c5, 40 },
    { 0xee8582c6, 23 },
    { 0xd16582c7, 93 },
    { 0xbca582c8, 95 },
    { 0x40b582c9, 90 },
    { 0x94e582c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0582ce, 87 },
    { 0x173582cf, 91 },
    { 0, 0 },
    { 0xdc7582d1, 78 },
    { 0x4e0582d1, 33 },
    { 0, 0 },
    { 0x92a582d4, 33 },
    { 0, 0 },
    { 0x37e582d6, 91 },
    { 0, 0 },
    { 0x7f4582d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x701582db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411582e0, 100 },
    { 0xb56582e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x627582e4, 33 },
    { 0xc6e582e4, 90 },
    { 0x6c8582e6, 61 },
    { 0, 0 },
    { 0x6d0582e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31582ec, 55 },
    { 0xa68582ed, 52 },
    { 0x39c582ed, 23 },
    { 0, 0 },
    { 0x37c582f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c582f6, 83 },
    { 0x4f582f6, 95 },
    { 0, 0 },
    { 0x648582f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a582fd, 23 },
    { 0xff4582fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4758301, 23 },
    { 0, 0 },
    { 0x60158303, 61 },
    { 0x4bb58304, 84 },
    { 0, 0 },
    { 0x8ef58306, 76 },
    { 0x65658307, 55 },
    { 0x95d58308, 90 },
    { 0xe8858309, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c5830c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x505830f, 23 },
    { 0x9d358310, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a258314, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17058317, 88 },
    { 0x30558317, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85831e, 95 },
    { 0x86d5831f, 23 },
    { 0x8045831f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc58323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c758327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade5832c, 95 },
    { 0x5a15832d, 61 },
    { 0x9fc5832c, 34 },
    { 0, 0 },
    { 0x86058330, 66 },
    { 0x19858331, 95 },
    { 0, 0 },
    { 0x70458333, 34 },
    { 0xae658334, 23 },
    { 0xc758335, 61 },
    { 0xdfe58336, 61 },
    { 0x70158334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55833c, 33 },
    { 0, 0 },
    { 0x60b5833e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e58341, 90 },
    { 0, 0 },
    { 0x3258343, 61 },
    { 0, 0 },
    { 0xad958345, 33 },
    { 0x81f58346, 55 },
    { 0x16658347, 23 },
    { 0x10b58347, 52 },
    { 0xbb58345, 99 },
    { 0, 0 },
    { 0xe735834b, 65 },
    { 0xd515834c, 76 },
    { 0, 0 },
    { 0xbfc5834e, 61 },
    { 0xcec5834f, 90 },
    { 0xdcb5834e, 95 },
    { 0, 0 },
    { 0xbd158352, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d58359, 34 },
    { 0xcf5835a, 55 },
    { 0, 0 },
    { 0xbac5835c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4345835f, 81 },
    { 0xe5c58360, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e5836f, 90 },
    { 0xd3158370, 98 },
    { 0x35b5836f, 66 },
    { 0x8de58372, 84 },
    { 0x14e58373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af58377, 76 },
    { 0, 0 },
    { 0x63f58379, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c5837c, 66 },
    { 0xb145837d, 90 },
    { 0x7995837c, 55 },
    { 0x85c5837c, 34 },
    { 0x42e58380, 98 },
    { 0x8af58380, 38 },
    { 0, 0 },
    { 0x69f58383, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005838a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad58391, 90 },
    { 0, 0 },
    { 0x89058393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50583a1, 100 },
    { 0, 0 },
    { 0xf95583a3, 85 },
    { 0x14a583a4, 95 },
    { 0x4e2583a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f583a8, 23 },
    { 0x602583a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7583ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x457583af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c583b2, 61 },
    { 0, 0 },
    { 0xc55583b4, 23 },
    { 0, 0 },
    { 0xf9c583b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4583ba, 80 },
    { 0, 0 },
    { 0xf65583bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e583bf, 75 },
    { 0x82e583bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72583c5, 23 },
    { 0, 0 },
    { 0x9e8583c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137583cc, 98 },
    { 0xa91583cc, 90 },
    { 0x5fa583ce, 76 },
    { 0xb73583cf, 61 },
    { 0xb9a583d0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x236583d3, 23 },
    { 0x57e583d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x499583d7, 35 },
    { 0, 0 },
    { 0x7c3583d9, 76 },
    { 0, 0 },
    { 0x468583db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d583e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7583e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6583ec, 69 },
    { 0xd3e583ec, 23 },
    { 0x358583ee, 68 },
    { 0x196583ee, 61 },
    { 0x5ec583ef, 93 },
    { 0x49b583f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422583f6, 95 },
    { 0, 0 },
    { 0xcb7583f8, 66 },
    { 0x435583f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1583ff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd58404, 53 },
    { 0xd8b58405, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1165840a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb458412, 34 },
    { 0xa858412, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79058416, 90 },
    { 0, 0 },
    { 0xddb58418, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125841b, 33 },
    { 0, 0 },
    { 0xc015841d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a558420, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6758426, 93 },
    { 0x32d58427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d58431, 76 },
    { 0xc4758431, 90 },
    { 0x94058433, 85 },
    { 0x22058433, 75 },
    { 0, 0 },
    { 0x9f158436, 90 },
    { 0x27958437, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15843a, 61 },
    { 0x3035843b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5843e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf58445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c05844a, 46 },
    { 0xb8a5844b, 61 },
    { 0xc075844a, 76 },
    { 0x77b5844d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16458451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb58454, 33 },
    { 0, 0 },
    { 0x7f858456, 52 },
    { 0xe9458456, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3215845a, 83 },
    { 0xb585845b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3058461, 3 },
    { 0, 0 },
    { 0x65f58463, 46 },
    { 0xfcd58464, 46 },
    { 0xbde58465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e358468, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5325846d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c358471, 33 },
    { 0xb5b58472, 95 },
    { 0, 0 },
    { 0x75a58474, 98 },
    { 0x7b858475, 33 },
    { 0x9f658476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd25847e, 23 },
    { 0, 0 },
    { 0x2d158480, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f458488, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795848f, 23 },
    { 0, 0 },
    { 0x86b58491, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b858498, 61 },
    { 0, 0 },
    { 0x47c5849a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481584a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x712584a3, 61 },
    { 0xc13584a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f584af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc584b2, 43 },
    { 0xf32584b3, 33 },
    { 0x4de584b4, 87 },
    { 0, 0 },
    { 0x861584b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9584b9, 95 },
    { 0x376584b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c584be, 23 },
    { 0x976584bf, 98 },
    { 0x33b584bf, 33 },
    { 0, 0 },
    { 0xdb2584c2, 98 },
    { 0x5b9584c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274584c8, 91 },
    { 0, 0 },
    { 0x2d0584ca, 61 },
    { 0x1d0584ca, 95 },
    { 0xf4a584ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x921584cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07584d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a584d5, 43 },
    { 0x77e584d6, 61 },
    { 0, 0 },
    { 0xd4e584d8, 61 },
    { 0x32c584d9, 66 },
    { 0xd35584d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4584e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5584eb, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f584ee, 95 },
    { 0, 0 },
    { 0xec5584f0, 95 },
    { 0x3c3584f1, 34 },
    { 0xe01584f2, 90 },
    { 0, 0 },
    { 0x93584f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd584f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4584ff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d158509, 98 },
    { 0x3875850a, 87 },
    { 0, 0 },
    { 0xa875850c, 90 },
    { 0, 0 },
    { 0x4e85850e, 84 },
    { 0x6185850f, 61 },
    { 0x62658510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85458518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7155851f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62458522, 40 },
    { 0x1e058523, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df58528, 90 },
    { 0x7d558529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5852d, 99 },
    { 0x8405852e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97358534, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f158538, 90 },
    { 0, 0 },
    { 0xede5853a, 93 },
    { 0x3b35853b, 55 },
    { 0, 0 },
    { 0x97a5853d, 95 },
    { 0xe575853d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac58541, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf58544, 90 },
    { 0x9a058545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd95854c, 84 },
    { 0x4005854d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab058554, 61 },
    { 0x4c358555, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2258559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa5855c, 43 },
    { 0, 0 },
    { 0xa5c5855e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16858565, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71358568, 84 },
    { 0xc9958569, 46 },
    { 0xaa858568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd855856f, 98 },
    { 0x6ca5856f, 66 },
    { 0, 0 },
    { 0x3e358572, 90 },
    { 0, 0 },
    { 0xaa58574, 90 },
    { 0x22558575, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d58578, 61 },
    { 0x71458578, 75 },
    { 0, 0 },
    { 0xd115857b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8665857e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a858583, 61 },
    { 0x61e58584, 98 },
    { 0, 0 },
    { 0x3f758586, 33 },
    { 0, 0 },
    { 0x4d058588, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35158590, 52 },
    { 0xcaa58590, 95 },
    { 0, 0 },
    { 0x24058593, 100 },
    { 0x9ff58594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44058597, 90 },
    { 0x6cc58598, 61 },
    { 0x47c58599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c15859c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971585a0, 98 },
    { 0x455585a0, 23 },
    { 0xbeb585a2, 33 },
    { 0xaa585a3, 61 },
    { 0x67a585a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c585a7, 66 },
    { 0x9d8585a7, 23 },
    { 0x672585a8, 61 },
    { 0x7df585aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b585b3, 33 },
    { 0x5ff585b3, 84 },
    { 0xeaf585b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da585b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16585bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x359585bf, 84 },
    { 0xcb3585c0, 33 },
    { 0x951585c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9585c4, 61 },
    { 0, 0 },
    { 0x96c585c6, 85 },
    { 0x630585c6, 67 },
    { 0x54585c7, 69 },
    { 0xaa1585c7, 66 },
    { 0xbad585ca, 90 },
    { 0, 0 },
    { 0xfe9585cc, 76 },
    { 0x6b0585cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25585d1, 23 },
    { 0xebc585d2, 33 },
    { 0, 0 },
    { 0xbe585d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x771585d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61585dc, 43 },
    { 0x93585dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7585e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655585e6, 23 },
    { 0xec9585e7, 61 },
    { 0x867585e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8585ee, 36 },
    { 0, 0 },
    { 0xc16585f0, 88 },
    { 0xc22585f1, 90 },
    { 0x982585f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7585f8, 46 },
    { 0xdd5585f9, 33 },
    { 0x8d585fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b585fd, 90 },
    { 0x2be585fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58258602, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf58609, 55 },
    { 0x7695860a, 55 },
    { 0x5945860b, 66 },
    { 0x53158609, 23 },
    { 0x1495860d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b358610, 33 },
    { 0x8e58610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1565861d, 33 },
    { 0x64d5861e, 34 },
    { 0x76e5861d, 76 },
    { 0, 0 },
    { 0x29958621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab58629, 23 },
    { 0xacc5862a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15862e, 76 },
    { 0x57c5862f, 85 },
    { 0x66958630, 95 },
    { 0x75c58631, 43 },
    { 0xab958632, 33 },
    { 0xae658633, 88 },
    { 0xfb45862f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77358639, 95 },
    { 0x3b55863a, 52 },
    { 0x5db5863b, 98 },
    { 0, 0 },
    { 0x2345863d, 52 },
    { 0x8885863e, 90 },
    { 0x2d5863d, 90 },
    { 0x775863d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8358644, 88 },
    { 0, 0 },
    { 0x63d58646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8335864b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585864f, 52 },
    { 0x44e58650, 66 },
    { 0xd7858651, 76 },
    { 0, 0 },
    { 0x41258653, 61 },
    { 0xc6558654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f58659, 98 },
    { 0x1b858659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2835865f, 66 },
    { 0, 0 },
    { 0x6fb58661, 61 },
    { 0, 0 },
    { 0xcc358663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef58666, 93 },
    { 0x4ad58666, 33 },
    { 0x19458668, 95 },
    { 0, 0 },
    { 0xc505866a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e85866f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5758675, 22 },
    { 0xce258675, 55 },
    { 0xc6458676, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a25867a, 55 },
    { 0x4ba5867b, 33 },
    { 0x975867b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6295867f, 98 },
    { 0x47558680, 90 },
    { 0, 0 },
    { 0xc6e58682, 99 },
    { 0x62158683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8958687, 76 },
    { 0x56358688, 90 },
    { 0xd9b58688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e5868c, 33 },
    { 0xf775868d, 98 },
    { 0x3985868e, 95 },
    { 0x7af5868f, 98 },
    { 0x40058690, 33 },
    { 0x11658691, 43 },
    { 0, 0 },
    { 0x3d958693, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf765869c, 90 },
    { 0x8475869d, 33 },
    { 0, 0 },
    { 0x2f85869f, 52 },
    { 0x195586a0, 52 },
    { 0xf15586a0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829586a5, 95 },
    { 0xe27586a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00586a9, 85 },
    { 0xf32586aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae586af, 90 },
    { 0, 0 },
    { 0xd0f586b1, 61 },
    { 0xace586b1, 66 },
    { 0x7e4586b1, 87 },
    { 0x557586b2, 98 },
    { 0xffc586b2, 52 },
    { 0x232586b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1586c6, 87 },
    { 0xde2586c6, 76 },
    { 0x2f1586c8, 98 },
    { 0, 0 },
    { 0x6bc586ca, 88 },
    { 0, 0 },
    { 0x5e1586cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96586d4, 33 },
    { 0xff9586d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c586d9, 77 },
    { 0xa8a586da, 34 },
    { 0xb93586da, 23 },
    { 0x38f586dc, 23 },
    { 0x385586dd, 61 },
    { 0x991586da, 93 },
    { 0xe04586dd, 100 },
    { 0xef1586df, 100 },
    { 0x254586e1, 90 },
    { 0, 0 },
    { 0xf17586e3, 33 },
    { 0x9e0586e4, 61 },
    { 0x353586e5, 90 },
    { 0xff4586e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7586e9, 90 },
    { 0, 0 },
    { 0xdc4586eb, 52 },
    { 0xd4f586eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb586f0, 43 },
    { 0x89e586f1, 95 },
    { 0x97f586f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7586f6, 76 },
    { 0, 0 },
    { 0xedd586f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25586fc, 76 },
    { 0xa8d586fc, 95 },
    { 0xa73586fe, 95 },
    { 0xbb9586ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86558702, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab58707, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc58710, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e58717, 95 },
    { 0, 0 },
    { 0xd9e58719, 61 },
    { 0xff65871a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e5871e, 46 },
    { 0x29d5871e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf158723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1658727, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f65872a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd75872e, 95 },
    { 0xc915872e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24158733, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1558736, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4205873b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3858741, 23 },
    { 0xbdc58741, 95 },
    { 0xde358743, 33 },
    { 0x60758744, 76 },
    { 0x9cc58745, 23 },
    { 0x2d658742, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d5874c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1565874f, 68 },
    { 0, 0 },
    { 0xb9f58751, 38 },
    { 0x4c358752, 33 },
    { 0x95958753, 95 },
    { 0, 0 },
    { 0xf2558755, 23 },
    { 0x29058756, 95 },
    { 0, 0 },
    { 0xc3558758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e58761, 90 },
    { 0, 0 },
    { 0x82e58763, 67 },
    { 0xda358764, 67 },
    { 0x46458763, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f858769, 46 },
    { 0xb5f58769, 98 },
    { 0xc7a5876b, 23 },
    { 0x4645876c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca58770, 61 },
    { 0x50258771, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f58777, 61 },
    { 0xd9c58778, 90 },
    { 0xbd958778, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a15877c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515877f, 55 },
    { 0xb9158780, 90 },
    { 0xa5958781, 98 },
    { 0x74958782, 95 },
    { 0xeda58780, 95 },
    { 0xc9c58784, 43 },
    { 0xdf058785, 76 },
    { 0x65758785, 90 },
    { 0xcc258787, 101 },
    { 0x7bb58788, 90 },
    { 0x55758789, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd885878e, 95 },
    { 0, 0 },
    { 0x82f58790, 83 },
    { 0xec958791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d58795, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9258799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a5879c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc855879f, 61 },
    { 0x508587a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd587a9, 67 },
    { 0xf36587aa, 93 },
    { 0, 0 },
    { 0xc50587ac, 38 },
    { 0, 0 },
    { 0xe14587ae, 61 },
    { 0x767587af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce587b6, 95 },
    { 0xee7587b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e587bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4587be, 90 },
    { 0x41b587bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b587c3, 61 },
    { 0xf8b587c4, 33 },
    { 0x252587c5, 90 },
    { 0x6bc587c6, 100 },
    { 0x293587c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8587cd, 91 },
    { 0x24a587cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e587d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f587d7, 76 },
    { 0xef0587d8, 76 },
    { 0x3d9587d9, 23 },
    { 0x6be587d9, 87 },
    { 0, 0 },
    { 0x7c1587dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841587e2, 90 },
    { 0, 0 },
    { 0xd68587e4, 43 },
    { 0xf35587e5, 88 },
    { 0x488587e6, 36 },
    { 0x318587e7, 88 },
    { 0xa7c587e5, 43 },
    { 0, 0 },
    { 0x6fc587ea, 99 },
    { 0xa0a587eb, 61 },
    { 0x1a6587ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61587f1, 33 },
    { 0x24c587f2, 90 },
    { 0x362587f2, 34 },
    { 0x26e587f4, 95 },
    { 0x1c2587f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886587fb, 55 },
    { 0x80d587fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27587ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e58805, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec958809, 61 },
    { 0x41b58809, 61 },
    { 0x2595880b, 90 },
    { 0x3dc5880a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d458811, 3 },
    { 0x37458812, 90 },
    { 0xa6b58813, 33 },
    { 0xf7a58814, 87 },
    { 0xaee58812, 76 },
    { 0xaed58816, 43 },
    { 0, 0 },
    { 0x27c58818, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70958828, 34 },
    { 0x66d58829, 53 },
    { 0x2d85882a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365882d, 95 },
    { 0x37f5882e, 76 },
    { 0x8755882f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf558836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e58841, 33 },
    { 0, 0 },
    { 0x88d58843, 61 },
    { 0, 0 },
    { 0xf1b58845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5884b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6558851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b358856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2675885a, 91 },
    { 0x6ca5885b, 96 },
    { 0, 0 },
    { 0xc865885d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f258864, 95 },
    { 0x7e58865, 76 },
    { 0xa0558866, 40 },
    { 0xe558866, 40 },
    { 0x38e58868, 34 },
    { 0, 0 },
    { 0x8ae5886a, 33 },
    { 0x98a5886a, 40 },
    { 0, 0 },
    { 0xe435886d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c58871, 66 },
    { 0, 0 },
    { 0x43b58873, 55 },
    { 0, 0 },
    { 0xced58875, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93558879, 61 },
    { 0, 0 },
    { 0x9e65887b, 61 },
    { 0, 0 },
    { 0xabf5887d, 90 },
    { 0xd265887e, 68 },
    { 0, 0 },
    { 0xd5158880, 80 },
    { 0xa1158880, 66 },
    { 0x4ed58881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68458888, 61 },
    { 0xcf858889, 98 },
    { 0x155888a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db5888e, 52 },
    { 0, 0 },
    { 0x9d358890, 76 },
    { 0, 0 },
    { 0xe4d58892, 53 },
    { 0, 0 },
    { 0x15f58894, 52 },
    { 0, 0 },
    { 0xcf858896, 34 },
    { 0, 0 },
    { 0xf8458898, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5889d, 95 },
    { 0xb485889e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2588a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc588a4, 97 },
    { 0x4fb588a4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d588a9, 90 },
    { 0, 0 },
    { 0x7cd588ab, 98 },
    { 0x25b588ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb588af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b588b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc588b5, 84 },
    { 0x6d8588b5, 84 },
    { 0x908588b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8588ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50588bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea588c0, 65 },
    { 0x9c2588c1, 90 },
    { 0xbf2588c0, 23 },
    { 0x19588c2, 99 },
    { 0xbfe588c4, 94 },
    { 0x947588c5, 90 },
    { 0xf2c588c6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6588c9, 43 },
    { 0x3d8588c9, 83 },
    { 0, 0 },
    { 0xb40588cc, 43 },
    { 0xf42588cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25588d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588d6, 93 },
    { 0x9b1588d6, 90 },
    { 0x150588d8, 34 },
    { 0xcd9588d8, 95 },
    { 0xa4f588da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0588dd, 61 },
    { 0xf9c588de, 90 },
    { 0x901588dd, 100 },
    { 0x3c588de, 98 },
    { 0xaa588dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00588e6, 84 },
    { 0, 0 },
    { 0x31a588e8, 100 },
    { 0xb13588e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae588ec, 23 },
    { 0x920588ed, 67 },
    { 0, 0 },
    { 0xb8b588ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe588f2, 61 },
    { 0x72b588f2, 43 },
    { 0xa27588f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37588f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29588fc, 90 },
    { 0x4d0588fd, 76 },
    { 0x729588fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14958901, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8075890a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9325890f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71458913, 46 },
    { 0, 0 },
    { 0x9af58915, 84 },
    { 0, 0 },
    { 0xff58917, 91 },
    { 0x17658918, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625891b, 90 },
    { 0, 0 },
    { 0x9a25891d, 67 },
    { 0, 0 },
    { 0x9cc5891f, 39 },
    { 0xffb58920, 23 },
    { 0xdc65891f, 55 },
    { 0x77858920, 61 },
    { 0x10e58923, 61 },
    { 0x60e58924, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b58931, 42 },
    { 0xb7d58931, 55 },
    { 0x8e458933, 81 },
    { 0xff258934, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69158941, 99 },
    { 0x7c58942, 98 },
    { 0x50358942, 76 },
    { 0xa8b58944, 76 },
    { 0, 0 },
    { 0x35758946, 34 },
    { 0x9c858947, 99 },
    { 0, 0 },
    { 0xfb758949, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15958951, 23 },
    { 0, 0 },
    { 0x21958953, 61 },
    { 0xded58954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1458957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6435895a, 92 },
    { 0, 0 },
    { 0xbf05895c, 23 },
    { 0, 0 },
    { 0x5995895e, 84 },
    { 0, 0 },
    { 0xe1e58960, 98 },
    { 0x99358961, 33 },
    { 0x17a58962, 91 },
    { 0x31858963, 88 },
    { 0x258964, 34 },
    { 0, 0 },
    { 0xfbd58966, 55 },
    { 0x90a58967, 90 },
    { 0x75d58968, 98 },
    { 0, 0 },
    { 0x4cd5896a, 98 },
    { 0xa7b5896b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63458973, 95 },
    { 0xa6758973, 90 },
    { 0xb3e58973, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2458978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf805897d, 98 },
    { 0x94e5897e, 76 },
    { 0x70f5897f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62558984, 23 },
    { 0, 0 },
    { 0x94158986, 61 },
    { 0x5bc58987, 33 },
    { 0x7dc58986, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45898b, 23 },
    { 0x8515898c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce58991, 90 },
    { 0xeed58992, 90 },
    { 0x53858992, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d5899e, 17 },
    { 0xd705899e, 43 },
    { 0x401589a0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf589a5, 52 },
    { 0x558589a5, 43 },
    { 0x285589a7, 99 },
    { 0, 0 },
    { 0x6d8589a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544589ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a589b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2589b8, 100 },
    { 0x7d9589b9, 43 },
    { 0, 0 },
    { 0xe0a589bb, 33 },
    { 0x970589bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3589c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca589d0, 90 },
    { 0, 0 },
    { 0x311589d2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13589d5, 100 },
    { 0x48589d6, 61 },
    { 0xd7b589d7, 33 },
    { 0, 0 },
    { 0xa0e589d9, 66 },
    { 0, 0 },
    { 0xa589db, 61 },
    { 0x877589dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f589e6, 76 },
    { 0x7f0589e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c589ee, 33 },
    { 0x64a589ef, 95 },
    { 0x938589f0, 56 },
    { 0x2ec589ef, 85 },
    { 0xba0589f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae589f6, 76 },
    { 0x40c589f7, 33 },
    { 0x28589f7, 33 },
    { 0xddb589f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba589fd, 67 },
    { 0xb96589fd, 61 },
    { 0x46f589fe, 17 },
    { 0, 0 },
    { 0x28158a01, 33 },
    { 0x12b58a02, 55 },
    { 0x69c58a02, 98 },
    { 0x69858a04, 98 },
    { 0xf5f58a05, 5 },
    { 0x93958a06, 98 },
    { 0xdb758a07, 90 },
    { 0, 0 },
    { 0x9f758a09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca558a0c, 98 },
    { 0x89c58a0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d558a11, 34 },
    { 0xfda58a12, 91 },
    { 0, 0 },
    { 0x2c758a14, 61 },
    { 0xbec58a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a858a19, 33 },
    { 0, 0 },
    { 0xdc658a1b, 90 },
    { 0x97e58a1c, 43 },
    { 0xaa358a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb558a21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1558a28, 90 },
    { 0x80f58a28, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd58a2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be58a30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e658a33, 90 },
    { 0x9f858a33, 61 },
    { 0xb7658a33, 90 },
    { 0x58158a36, 76 },
    { 0x47d58a37, 33 },
    { 0xf9658a38, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4058a3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0358a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24558a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79758a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca58a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb258a4f, 52 },
    { 0, 0 },
    { 0x18a58a51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c58a54, 33 },
    { 0x34e58a55, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d58a58, 90 },
    { 0xb9258a59, 33 },
    { 0x15758a5a, 95 },
    { 0x41558a5b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5558a60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c58a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2558a67, 67 },
    { 0xd2858a68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8058a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58a70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28958a74, 68 },
    { 0x74e58a75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7658a78, 91 },
    { 0x58458a79, 55 },
    { 0x1d758a7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c58a82, 26 },
    { 0xd7058a82, 61 },
    { 0xd2d58a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f958a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b658a99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc58a9c, 55 },
    { 0x81158a9d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82358aa3, 98 },
    { 0xfa458aa4, 55 },
    { 0, 0 },
    { 0xbb858aa6, 84 },
    { 0x63f58aa7, 68 },
    { 0x78358aa7, 52 },
    { 0x94a58aa9, 33 },
    { 0xa4f58aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58aae, 78 },
    { 0xaf158aaf, 34 },
    { 0x51558aaf, 90 },
    { 0, 0 },
    { 0x96a58ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b958ab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c58aba, 23 },
    { 0x62b58aba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec858abe, 23 },
    { 0x29e58abf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79258ac3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e58ac8, 95 },
    { 0xbcb58ac9, 95 },
    { 0, 0 },
    { 0xdc958acb, 88 },
    { 0xd7758acc, 33 },
    { 0x7058acd, 95 },
    { 0, 0 },
    { 0x1c958acf, 61 },
    { 0xd5558ad0, 95 },
    { 0x30c58ad0, 99 },
    { 0, 0 },
    { 0x79058ad3, 84 },
    { 0xf7258ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a58adb, 90 },
    { 0, 0 },
    { 0xb1a58add, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d058ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17658ae3, 98 },
    { 0xbb058ae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd58ae7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a158aea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af58aee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc58af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd58afb, 90 },
    { 0x5a158afc, 43 },
    { 0x1b358afd, 90 },
    { 0xbe158afe, 95 },
    { 0x47458aff, 76 },
    { 0, 0 },
    { 0x42b58b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d658b04, 46 },
    { 0xf558b04, 93 },
    { 0x1c058b05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd58b0c, 76 },
    { 0x80f58b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13258b10, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca58b13, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4458b18, 95 },
    { 0x50c58b19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f158b1c, 90 },
    { 0x5d758b1c, 98 },
    { 0, 0 },
    { 0x83e58b1f, 23 },
    { 0xf4a58b20, 95 },
    { 0xc5d58b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce358b28, 98 },
    { 0xff858b29, 69 },
    { 0, 0 },
    { 0x6ec58b2b, 17 },
    { 0x72d58b2c, 78 },
    { 0xe2158b2d, 51 },
    { 0x4358b2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e458b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66858b3d, 76 },
    { 0x98858b3e, 61 },
    { 0x15958b3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41058b47, 33 },
    { 0x4f158b48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4158b4c, 23 },
    { 0xfec58b4c, 33 },
    { 0xbff58b4d, 61 },
    { 0xb9158b4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b758b61, 61 },
    { 0, 0 },
    { 0x9ef58b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4658b66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb558b6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da58b72, 38 },
    { 0xd0c58b73, 46 },
    { 0xf4a58b73, 98 },
    { 0x36c58b74, 61 },
    { 0x75a58b76, 95 },
    { 0xf7a58b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b58b7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e358b7f, 55 },
    { 0xb9b58b7f, 98 },
    { 0x9db58b81, 61 },
    { 0x4c058b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c858b88, 61 },
    { 0xe1d58b89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84158b8d, 95 },
    { 0x2e558b8e, 98 },
    { 0x61358b8f, 91 },
    { 0, 0 },
    { 0xaea58b91, 68 },
    { 0x2a858b91, 33 },
    { 0x65958b93, 52 },
    { 0, 0 },
    { 0x20258b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9358b9e, 68 },
    { 0xc3758b9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6958ba4, 56 },
    { 0, 0 },
    { 0x8558ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b458ba9, 76 },
    { 0xb358ba9, 98 },
    { 0x56058baa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30758bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17858bb5, 46 },
    { 0x3e858bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36258bb9, 23 },
    { 0xe2858bba, 98 },
    { 0, 0 },
    { 0x32858bbc, 95 },
    { 0, 0 },
    { 0x5eb58bbe, 46 },
    { 0, 0 },
    { 0x9d558bc0, 61 },
    { 0, 0 },
    { 0xfd658bc2, 23 },
    { 0x3a158bc2, 91 },
    { 0x9e658bc2, 98 },
    { 0, 0 },
    { 0x3af58bc6, 98 },
    { 0xdf958bc6, 34 },
    { 0x78658bc8, 55 },
    { 0xc8558bc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34058bcd, 34 },
    { 0x5d58bce, 80 },
    { 0xfa158bcf, 34 },
    { 0x9ac58bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47558bd4, 90 },
    { 0x8fa58bd5, 61 },
    { 0xb0358bd4, 45 },
    { 0x6ad58bd6, 33 },
    { 0, 0 },
    { 0x9a58bd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30858bdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d58be0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd858be4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e958be8, 34 },
    { 0xc5258be9, 90 },
    { 0xce358bea, 87 },
    { 0xeb058beb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca358bee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d58bf1, 98 },
    { 0, 0 },
    { 0xf6c58bf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e58bf8, 53 },
    { 0, 0 },
    { 0xfe358bfa, 90 },
    { 0xe5058bfb, 90 },
    { 0x8ee58bfc, 93 },
    { 0x9e558bfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb58c02, 90 },
    { 0x7ca58c03, 98 },
    { 0, 0 },
    { 0x2b658c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b58c0b, 95 },
    { 0, 0 },
    { 0x9e458c0d, 33 },
    { 0, 0 },
    { 0x5a58c0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59758c13, 55 },
    { 0xdd358c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad058c17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee58c1b, 66 },
    { 0x2c958c1c, 23 },
    { 0, 0 },
    { 0x7c658c1e, 77 },
    { 0xc5258c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e58c26, 83 },
    { 0, 0 },
    { 0x5fe58c28, 84 },
    { 0x2a958c28, 46 },
    { 0, 0 },
    { 0x65058c2b, 91 },
    { 0, 0 },
    { 0x558c2d, 76 },
    { 0x20158c2e, 98 },
    { 0x71958c2f, 92 },
    { 0xb4158c30, 77 },
    { 0xa9058c30, 85 },
    { 0xe0158c30, 23 },
    { 0xa7858c33, 55 },
    { 0x91e58c34, 61 },
    { 0x41e58c34, 81 },
    { 0xc0658c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be58c39, 90 },
    { 0x46058c3a, 100 },
    { 0x77858c39, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f58c41, 90 },
    { 0xd6458c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28558c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f58c57, 78 },
    { 0, 0 },
    { 0x2fd58c59, 98 },
    { 0x18158c5a, 33 },
    { 0x63658c59, 52 },
    { 0xdd158c5c, 40 },
    { 0xa2f58c5b, 23 },
    { 0x72b58c5a, 61 },
    { 0xea58c5e, 55 },
    { 0x96658c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c458c63, 23 },
    { 0xb6058c64, 93 },
    { 0xb8058c65, 101 },
    { 0xe5758c66, 61 },
    { 0, 0 },
    { 0x55158c68, 98 },
    { 0xed658c68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68458c6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a58c70, 43 },
    { 0x66e58c71, 33 },
    { 0xf6258c71, 90 },
    { 0x6b758c71, 33 },
    { 0, 0 },
    { 0x5e458c75, 90 },
    { 0xbee58c75, 55 },
    { 0xe8e58c77, 93 },
    { 0x89e58c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d158c7c, 46 },
    { 0x29e58c7d, 33 },
    { 0x2fc58c7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec558c8d, 90 },
    { 0xad358c8e, 23 },
    { 0, 0 },
    { 0x51a58c90, 34 },
    { 0x86a58c91, 95 },
    { 0, 0 },
    { 0x23758c93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9058c96, 43 },
    { 0x8758c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe58c9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7658ca3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd358ca7, 66 },
    { 0, 0 },
    { 0x71358ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa58cac, 23 },
    { 0x8c358cad, 52 },
    { 0x61858cae, 95 },
    { 0x16658caf, 43 },
    { 0x41a58caf, 43 },
    { 0x94c58cb1, 97 },
    { 0xb9c58cb1, 76 },
    { 0, 0 },
    { 0x63f58cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a58cbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e58cbf, 95 },
    { 0x2a958cc0, 33 },
    { 0x4d158cc1, 72 },
    { 0xf4158cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44058cc6, 90 },
    { 0, 0 },
    { 0x59158cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a58cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3558cda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3358cdd, 91 },
    { 0xd1458cde, 23 },
    { 0x75058cdd, 61 },
    { 0xbb658ce0, 61 },
    { 0, 0 },
    { 0x2c858ce2, 98 },
    { 0, 0 },
    { 0x6e158ce4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86458ceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b58cf6, 90 },
    { 0xc5258cf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40158cfb, 88 },
    { 0xcbb58cfb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf258cff, 76 },
    { 0x3cb58d00, 66 },
    { 0xc858d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf558d06, 98 },
    { 0x35f58d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e58d10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde358d15, 76 },
    { 0, 0 },
    { 0x66458d17, 43 },
    { 0, 0 },
    { 0xf1f58d19, 33 },
    { 0xd1d58d19, 85 },
    { 0xdea58d1b, 76 },
    { 0x78e58d19, 43 },
    { 0x54358d1d, 95 },
    { 0x23b58d1e, 33 },
    { 0xf4c58d1e, 90 },
    { 0xf6a58d1d, 33 },
    { 0xff458d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce58d2e, 95 },
    { 0xfc858d2f, 33 },
    { 0, 0 },
    { 0x3fd58d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22158d34, 90 },
    { 0, 0 },
    { 0x6d958d36, 33 },
    { 0x7b658d37, 64 },
    { 0xe7958d36, 70 },
    { 0xbab58d38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad858d3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48158d43, 85 },
    { 0xab758d44, 90 },
    { 0x46758d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d858d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d358d4b, 55 },
    { 0xd7c58d4c, 56 },
    { 0x7a158d4d, 33 },
    { 0x2b858d4b, 23 },
    { 0, 0 },
    { 0xa6d58d50, 46 },
    { 0, 0 },
    { 0x6b258d52, 90 },
    { 0, 0 },
    { 0x90358d54, 23 },
    { 0, 0 },
    { 0xfe058d56, 23 },
    { 0, 0 },
    { 0xc5858d58, 77 },
    { 0, 0 },
    { 0xc7358d5a, 90 },
    { 0xe5658d5b, 95 },
    { 0, 0 },
    { 0x74158d5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a258d62, 93 },
    { 0, 0 },
    { 0x10258d64, 52 },
    { 0xbe658d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58158d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb558d6d, 23 },
    { 0, 0 },
    { 0x25958d6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57258d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde358d77, 23 },
    { 0x25958d78, 84 },
    { 0x58d58d78, 88 },
    { 0x758d78, 87 },
    { 0x27258d7a, 98 },
    { 0x9ce58d7a, 23 },
    { 0x1158d7d, 40 },
    { 0x9b58d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41658d85, 34 },
    { 0, 0 },
    { 0xe7a58d87, 93 },
    { 0x84f58d88, 96 },
    { 0xfd258d89, 2 },
    { 0x45758d87, 75 },
    { 0x7e158d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e958d90, 23 },
    { 0x97a58d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11958d97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d658d9b, 90 },
    { 0, 0 },
    { 0xf2758d9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a58da7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a58dab, 90 },
    { 0xe8258dac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba58db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb58db5, 33 },
    { 0, 0 },
    { 0x22558db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83358dbc, 100 },
    { 0xd1958dbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61458dc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb158dc7, 61 },
    { 0x1f658dc8, 46 },
    { 0, 0 },
    { 0x29658dca, 90 },
    { 0x92e58dca, 93 },
    { 0x8e458dcb, 84 },
    { 0x6cf58dcd, 17 },
    { 0xfa758dce, 98 },
    { 0x9c258dcd, 34 },
    { 0x57e58dcf, 43 },
    { 0xe0058dd1, 95 },
    { 0x6d358dd2, 33 },
    { 0x43c58dd3, 55 },
    { 0x50e58dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d758dda, 76 },
    { 0xd8158ddb, 90 },
    { 0xe0658dda, 11 },
    { 0x7e658ddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5458de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc558de8, 23 },
    { 0x67958de9, 61 },
    { 0xc6258de9, 34 },
    { 0x4a558dea, 36 },
    { 0x7fe58de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e758def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea58df2, 98 },
    { 0x96258df3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a858dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa158dff, 91 },
    { 0x67758dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e58e05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7758e0c, 23 },
    { 0x5df58e0d, 95 },
    { 0x1bc58e0d, 23 },
    { 0x7e58e0f, 76 },
    { 0x53658e0f, 100 },
    { 0x1b158e0d, 23 },
    { 0, 0 },
    { 0xa5558e13, 33 },
    { 0x70e58e14, 76 },
    { 0x9fd58e13, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f058e1c, 61 },
    { 0, 0 },
    { 0xd9e58e1e, 52 },
    { 0, 0 },
    { 0x3f558e20, 80 },
    { 0xc9d58e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0958e26, 23 },
    { 0x12258e27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f958e2a, 55 },
    { 0x47158e2b, 95 },
    { 0x5ac58e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b058e30, 61 },
    { 0, 0 },
    { 0xd5058e32, 90 },
    { 0x31758e32, 76 },
    { 0xb4e58e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e158e38, 55 },
    { 0xce558e38, 98 },
    { 0xee358e39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2758e3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41358e40, 61 },
    { 0x65358e41, 46 },
    { 0x1cf58e41, 33 },
    { 0xd3058e43, 87 },
    { 0x29058e43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc58e47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26658e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab58e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2358e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b458e5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e58e5e, 95 },
    { 0x1ee58e5f, 55 },
    { 0x3af58e60, 90 },
    { 0x1e358e61, 66 },
    { 0x5e258e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc58e65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca58e6a, 90 },
    { 0xfef58e6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b758e6f, 33 },
    { 0x53358e70, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e758e73, 98 },
    { 0x24558e74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee058e77, 99 },
    { 0x6d558e77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc58e7b, 95 },
    { 0xcb358e7c, 98 },
    { 0x1a158e7d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b458e80, 61 },
    { 0, 0 },
    { 0xc558e82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce58e85, 85 },
    { 0, 0 },
    { 0xdd58e87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97458e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a558e94, 46 },
    { 0xe6358e95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a58e98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d58e9e, 36 },
    { 0xa7a58e9f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62058ea7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c58eaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb258eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41958eb6, 43 },
    { 0x75358eb6, 95 },
    { 0x70358eb7, 43 },
    { 0, 0 },
    { 0x32858eba, 33 },
    { 0x31958eba, 33 },
    { 0x47658ebc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81458ebf, 90 },
    { 0x2f558ebf, 93 },
    { 0x83458ec1, 42 },
    { 0x16958ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42758ec7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25758ecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45158ed8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48958edd, 68 },
    { 0, 0 },
    { 0xc6e58edf, 61 },
    { 0, 0 },
    { 0x9ba58ee1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2758ee5, 98 },
    { 0x11658ee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf58eea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93358eee, 23 },
    { 0, 0 },
    { 0xc2a58ef0, 98 },
    { 0xb8158ef1, 55 },
    { 0xe6d58ef1, 84 },
    { 0, 0 },
    { 0x23a58ef4, 55 },
    { 0x9a958ef5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d258ef9, 78 },
    { 0x6c858efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37758efe, 55 },
    { 0x62458eff, 98 },
    { 0xa4058eff, 23 },
    { 0xc0d58eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c258f04, 81 },
    { 0x3d058f04, 34 },
    { 0xdc558f06, 93 },
    { 0x5ec58f07, 76 },
    { 0x6ac58f06, 61 },
    { 0x17f58f06, 61 },
    { 0xfd658f05, 48 },
    { 0xac058f07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10158f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b258f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e58f1c, 99 },
    { 0, 0 },
    { 0x40b58f1e, 76 },
    { 0x57358f1e, 91 },
    { 0, 0 },
    { 0xc9e58f21, 93 },
    { 0xedb58f22, 91 },
    { 0x8fb58f23, 90 },
    { 0x33358f24, 95 },
    { 0x6eb58f25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28358f29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4058f2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3258f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63858f3e, 93 },
    { 0, 0 },
    { 0x9f558f40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e58f45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88258f49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b58f4d, 43 },
    { 0x26858f4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b58f54, 61 },
    { 0x43b58f55, 68 },
    { 0, 0 },
    { 0xd4f58f57, 61 },
    { 0x95b58f58, 23 },
    { 0x9fe58f59, 78 },
    { 0x3d158f5a, 68 },
    { 0, 0 },
    { 0x8c458f5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a258f61, 33 },
    { 0, 0 },
    { 0x79e58f63, 76 },
    { 0x45e58f63, 91 },
    { 0, 0 },
    { 0xd8258f66, 61 },
    { 0x78058f66, 85 },
    { 0x37858f66, 67 },
    { 0, 0 },
    { 0xe5958f6a, 76 },
    { 0xdff58f6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e58f72, 33 },
    { 0xa4458f73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8458f78, 90 },
    { 0xa8d58f79, 33 },
    { 0x24c58f7a, 68 },
    { 0x11c58f7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9658f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1458f88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa158f8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0058f8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e58f94, 61 },
    { 0x14158f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f58f98, 98 },
    { 0xc8f58f99, 34 },
    { 0, 0 },
    { 0x4da58f9b, 95 },
    { 0xadf58f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22558f9f, 95 },
    { 0, 0 },
    { 0x52558fa1, 93 },
    { 0, 0 },
    { 0x1ae58fa3, 90 },
    { 0, 0 },
    { 0x21258fa5, 23 },
    { 0, 0 },
    { 0x72958fa7, 23 },
    { 0xdaa58fa7, 33 },
    { 0x32758fa9, 90 },
    { 0x1e258faa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5958fae, 33 },
    { 0, 0 },
    { 0x75358fb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82158fba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7158fbe, 23 },
    { 0, 0 },
    { 0xc9e58fc0, 93 },
    { 0x9d58fc0, 91 },
    { 0x76958fc1, 33 },
    { 0xf3958fc3, 95 },
    { 0xdce58fc4, 52 },
    { 0x2cd58fc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d58fce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d58fd1, 33 },
    { 0x38b58fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a158fd5, 33 },
    { 0, 0 },
    { 0x5e58fd7, 33 },
    { 0x9458fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77258fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd58fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb558fe6, 61 },
    { 0x72358fe7, 98 },
    { 0xae258fe8, 38 },
    { 0xcf358fe9, 98 },
    { 0x13358fea, 36 },
    { 0x2fe58feb, 33 },
    { 0xffb58feb, 61 },
    { 0, 0 },
    { 0xe8158fee, 61 },
    { 0x52e58fee, 78 },
    { 0xc4158ff0, 55 },
    { 0x73c58fef, 46 },
    { 0xde658ff2, 95 },
    { 0xbda58ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9558ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c58ffd, 90 },
    { 0x24058ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f59006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60359009, 61 },
    { 0xf945900a, 95 },
    { 0xa575900b, 23 },
    { 0, 0 },
    { 0xec35900d, 100 },
    { 0x4bf5900d, 78 },
    { 0x8c25900f, 98 },
    { 0xaf759010, 100 },
    { 0x8d559011, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a59017, 76 },
    { 0xc2e59018, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65901b, 95 },
    { 0x8cc5901c, 61 },
    { 0x6825901d, 43 },
    { 0, 0 },
    { 0x22b5901f, 52 },
    { 0, 0 },
    { 0x6d59021, 76 },
    { 0xc3559021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6559026, 33 },
    { 0xa7b59027, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a5902a, 98 },
    { 0x8365902b, 23 },
    { 0x4255902b, 80 },
    { 0x6d05902a, 23 },
    { 0x4365902e, 43 },
    { 0x3dc5902f, 34 },
    { 0, 0 },
    { 0x3259031, 76 },
    { 0, 0 },
    { 0xf0f59033, 61 },
    { 0x3d159034, 33 },
    { 0x89659035, 55 },
    { 0x51559036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565903c, 38 },
    { 0, 0 },
    { 0x9db5903e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b59045, 88 },
    { 0, 0 },
    { 0x36b59047, 91 },
    { 0, 0 },
    { 0xbc059049, 76 },
    { 0, 0 },
    { 0x1cb5904b, 46 },
    { 0x5655904c, 61 },
    { 0xe4a5904d, 76 },
    { 0x3705904e, 76 },
    { 0x3b55904f, 95 },
    { 0x1845904c, 61 },
    { 0xe5d5904d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d59056, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a95905a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca59060, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9159064, 77 },
    { 0xc59064, 76 },
    { 0x32e59064, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a35906c, 23 },
    { 0x7a25906d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f59070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7659073, 55 },
    { 0xe7859073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad15907e, 23 },
    { 0, 0 },
    { 0x31759080, 95 },
    { 0x22559080, 95 },
    { 0xe5359080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa775908d, 85 },
    { 0, 0 },
    { 0xd905908f, 61 },
    { 0x4d59090, 55 },
    { 0, 0 },
    { 0xe2659092, 76 },
    { 0x16b59093, 43 },
    { 0x44b59094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d590a8, 43 },
    { 0, 0 },
    { 0xc73590aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b590ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1590b1, 52 },
    { 0x6a2590b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x859590b5, 98 },
    { 0xdb1590b6, 23 },
    { 0, 0 },
    { 0xbbc590b8, 23 },
    { 0x3fc590b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0590bf, 68 },
    { 0xca7590c0, 90 },
    { 0xc7e590bf, 33 },
    { 0x583590c2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7590c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x170590c8, 33 },
    { 0, 0 },
    { 0x7c8590ca, 33 },
    { 0x563590ca, 38 },
    { 0, 0 },
    { 0x4f590cd, 34 },
    { 0, 0 },
    { 0xd87590cf, 77 },
    { 0x953590cf, 76 },
    { 0xb4a590d0, 90 },
    { 0xd25590d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592590d7, 99 },
    { 0xc49590d8, 61 },
    { 0x252590d9, 85 },
    { 0x476590d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59590dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743590e2, 90 },
    { 0x417590e3, 87 },
    { 0x4fe590e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d590ea, 33 },
    { 0x263590ea, 61 },
    { 0xb13590ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea590ef, 33 },
    { 0x4590f0, 61 },
    { 0x858590f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee590f4, 95 },
    { 0xfe3590f4, 55 },
    { 0, 0 },
    { 0x549590f7, 95 },
    { 0xdb6590f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e359100, 46 },
    { 0x559101, 95 },
    { 0, 0 },
    { 0xb9859103, 61 },
    { 0x31259104, 91 },
    { 0x4f459105, 43 },
    { 0x89859104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf95910d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb59110, 83 },
    { 0xdc159111, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5911a, 78 },
    { 0, 0 },
    { 0x3d55911c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4759120, 84 },
    { 0xec159121, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b59124, 98 },
    { 0xdd959124, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f59130, 76 },
    { 0xcac59131, 95 },
    { 0xc5c59132, 46 },
    { 0x74759130, 23 },
    { 0x58d59134, 91 },
    { 0x6fb59135, 33 },
    { 0x48d59134, 95 },
    { 0x58959137, 55 },
    { 0, 0 },
    { 0x2ed59139, 93 },
    { 0, 0 },
    { 0xde15913b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89359143, 91 },
    { 0x14e59144, 95 },
    { 0, 0 },
    { 0xe8359146, 76 },
    { 0, 0 },
    { 0x82659148, 76 },
    { 0x88059149, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465914d, 98 },
    { 0, 0 },
    { 0xbd95914f, 34 },
    { 0x91e59150, 90 },
    { 0xfbc59151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7959159, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5915c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed25916d, 90 },
    { 0x18f5916e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c159171, 33 },
    { 0x7c159172, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f59175, 55 },
    { 0, 0 },
    { 0xb3659177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b59188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee5918d, 67 },
    { 0xe955918e, 23 },
    { 0xfd5918f, 90 },
    { 0x1825918f, 48 },
    { 0x56459191, 55 },
    { 0x3b159191, 55 },
    { 0, 0 },
    { 0x16a59194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76759197, 61 },
    { 0, 0 },
    { 0x83e59199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195919c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244591a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3591a4, 91 },
    { 0x53c591a5, 90 },
    { 0, 0 },
    { 0x6b4591a7, 91 },
    { 0, 0 },
    { 0x29e591a9, 96 },
    { 0, 0 },
    { 0xeb5591ab, 34 },
    { 0x103591ac, 93 },
    { 0xec9591ac, 90 },
    { 0x510591ab, 61 },
    { 0xf78591af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86591b4, 23 },
    { 0, 0 },
    { 0xe9b591b6, 40 },
    { 0x885591b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee591bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7591c4, 55 },
    { 0, 0 },
    { 0xca9591c6, 99 },
    { 0xeae591c7, 33 },
    { 0x383591c7, 98 },
    { 0, 0 },
    { 0x807591ca, 55 },
    { 0x148591ca, 85 },
    { 0xf8e591cc, 90 },
    { 0x547591cd, 23 },
    { 0xfc591cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3591d1, 90 },
    { 0xcae591d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839591d9, 61 },
    { 0xc25591da, 33 },
    { 0, 0 },
    { 0xf68591dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x146591df, 75 },
    { 0x671591e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8591ec, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b591f1, 33 },
    { 0xc40591f2, 77 },
    { 0xf1b591f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e591f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d591ff, 90 },
    { 0, 0 },
    { 0xe3c59201, 72 },
    { 0, 0 },
    { 0x98759203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c45920b, 61 },
    { 0x8fb5920b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e159211, 33 },
    { 0, 0 },
    { 0x64359213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce59217, 33 },
    { 0, 0 },
    { 0x1f959219, 90 },
    { 0x3915921a, 52 },
    { 0x20359219, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5921f, 76 },
    { 0, 0 },
    { 0xac559221, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf659226, 90 },
    { 0x7e59227, 76 },
    { 0x9a159227, 90 },
    { 0x52259229, 95 },
    { 0x84a59227, 69 },
    { 0xba55922b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d5922e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16759233, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e659237, 23 },
    { 0x1f459238, 61 },
    { 0xdf159239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8945923d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b5924d, 93 },
    { 0x68e5924d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3659252, 33 },
    { 0x18659253, 46 },
    { 0xcd559254, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d759258, 61 },
    { 0x85a59259, 33 },
    { 0xfbe5925a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f5925e, 61 },
    { 0x3a25925f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d559263, 23 },
    { 0xf6759264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae5926b, 33 },
    { 0x9a95926b, 95 },
    { 0xaed5926d, 36 },
    { 0xdf95926e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77259271, 90 },
    { 0x1ba59272, 90 },
    { 0x35c59271, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30359278, 43 },
    { 0x8f459279, 95 },
    { 0xe1e59278, 23 },
    { 0x9355927b, 90 },
    { 0x755927c, 75 },
    { 0xbfb5927d, 98 },
    { 0xd645927d, 90 },
    { 0x1a55927d, 52 },
    { 0, 0 },
    { 0x81d59281, 61 },
    { 0xf2559282, 52 },
    { 0, 0 },
    { 0x88859284, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3459287, 55 },
    { 0, 0 },
    { 0xc9a59289, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32859291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a459296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205929c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b592a0, 95 },
    { 0x194592a1, 98 },
    { 0xbd9592a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d592a6, 34 },
    { 0, 0 },
    { 0x6e4592a8, 61 },
    { 0xe0f592a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f592ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d592b7, 81 },
    { 0x80592b8, 91 },
    { 0, 0 },
    { 0xe98592ba, 33 },
    { 0xf15592bb, 90 },
    { 0, 0 },
    { 0xc29592bd, 61 },
    { 0x37a592be, 93 },
    { 0xa64592be, 61 },
    { 0xce0592c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66592c6, 98 },
    { 0x19b592c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f592cd, 23 },
    { 0x69a592ce, 43 },
    { 0xb71592cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf592d2, 98 },
    { 0x972592d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0592d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0592e0, 98 },
    { 0xe75592e0, 76 },
    { 0xc04592e0, 55 },
    { 0x17c592e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c592e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x338592e9, 98 },
    { 0x998592ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8592ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e592f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338592ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da59303, 93 },
    { 0x39759304, 100 },
    { 0xeb59305, 77 },
    { 0xc3759306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15930e, 90 },
    { 0x2f15930f, 84 },
    { 0x44959310, 61 },
    { 0, 0 },
    { 0x63d59312, 91 },
    { 0xb4b59313, 66 },
    { 0xa2259314, 46 },
    { 0, 0 },
    { 0x4bc59316, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9855931c, 95 },
    { 0x7fa5931d, 95 },
    { 0xb525931e, 77 },
    { 0x5335931f, 88 },
    { 0x70e59320, 98 },
    { 0xd5c5931f, 52 },
    { 0xafb59321, 55 },
    { 0x38559320, 95 },
    { 0, 0 },
    { 0x40059325, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec059329, 46 },
    { 0x71759329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed59335, 91 },
    { 0xd4459335, 34 },
    { 0, 0 },
    { 0xfc859338, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa325933d, 55 },
    { 0x7d55933e, 98 },
    { 0xe8e5933f, 61 },
    { 0xb9059340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e55934c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb555934f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d59356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0659359, 90 },
    { 0xde85935a, 33 },
    { 0x8605935b, 43 },
    { 0x24359359, 91 },
    { 0, 0 },
    { 0x8975935e, 90 },
    { 0, 0 },
    { 0xc2559360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2059368, 33 },
    { 0xffc59368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a5936d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e59370, 33 },
    { 0xf8e59371, 61 },
    { 0, 0 },
    { 0xb0c59373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d59376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95359379, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b5937c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a5937f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1259384, 98 },
    { 0xde559385, 33 },
    { 0x59559386, 95 },
    { 0, 0 },
    { 0xfe959388, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb125938b, 61 },
    { 0x5b45938c, 76 },
    { 0xcb45938d, 43 },
    { 0xeff5938d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee759393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b59396, 65 },
    { 0x1e359397, 84 },
    { 0x48359396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5939b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5939f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x339593a2, 95 },
    { 0x552593a3, 95 },
    { 0, 0 },
    { 0x821593a5, 61 },
    { 0x3bc593a6, 23 },
    { 0, 0 },
    { 0x23d593a8, 87 },
    { 0x6bf593a9, 33 },
    { 0xfc2593a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a593af, 98 },
    { 0xa2d593b0, 46 },
    { 0xaf0593af, 95 },
    { 0x38c593b2, 90 },
    { 0xf5b593b0, 46 },
    { 0x9e4593b4, 56 },
    { 0, 0 },
    { 0x4e2593b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d593b9, 91 },
    { 0, 0 },
    { 0x8c4593bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e593c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7593c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b593ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4593d1, 61 },
    { 0xdbf593d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e593dd, 78 },
    { 0, 0 },
    { 0x90593df, 98 },
    { 0x3ed593e0, 87 },
    { 0xe6593e0, 90 },
    { 0, 0 },
    { 0x8593e3, 40 },
    { 0, 0 },
    { 0x15d593e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22593e9, 34 },
    { 0, 0 },
    { 0x5c7593eb, 95 },
    { 0xdcc593eb, 85 },
    { 0, 0 },
    { 0xd6d593ee, 23 },
    { 0, 0 },
    { 0xb06593f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff593f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9593fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a59400, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde15940b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a59411, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22459416, 23 },
    { 0, 0 },
    { 0xb9459418, 91 },
    { 0x33e59419, 100 },
    { 0xb775941a, 76 },
    { 0xa85941a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47559422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53359427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f59432, 84 },
    { 0x1ce59432, 98 },
    { 0x4d859432, 33 },
    { 0xeaa59433, 98 },
    { 0x12959436, 68 },
    { 0x56559436, 76 },
    { 0x11f59437, 33 },
    { 0xfd759439, 33 },
    { 0x5a959438, 67 },
    { 0x21e5943b, 84 },
    { 0x42e5943c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c5944a, 43 },
    { 0x605944b, 52 },
    { 0x1725944b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3965945f, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe359465, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af5946a, 98 },
    { 0x9915946b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40959471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b59474, 61 },
    { 0x24659475, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc59480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c59483, 33 },
    { 0, 0 },
    { 0xe8759485, 23 },
    { 0x45d59486, 53 },
    { 0xb159486, 76 },
    { 0xd7359488, 66 },
    { 0xc2d59489, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc5948c, 43 },
    { 0x3315948c, 23 },
    { 0xb65948c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xda159491, 95 },
    { 0, 0 },
    { 0x42759493, 83 },
    { 0xe3559494, 98 },
    { 0x33759495, 69 },
    { 0x6d759496, 61 },
    { 0, 0 },
    { 0x9a659498, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a55949b, 34 },
    { 0xb565949c, 17 },
    { 0xc9e5949d, 95 },
    { 0x5955949e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80594a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f594a8, 93 },
    { 0x765594a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420594ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13594b0, 61 },
    { 0, 0 },
    { 0xf9a594b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5594b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f594b9, 91 },
    { 0, 0 },
    { 0x4e3594bb, 85 },
    { 0x53b594bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f594c9, 78 },
    { 0, 0 },
    { 0x735594cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d594ce, 48 },
    { 0xe39594cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701594d7, 90 },
    { 0xbef594d8, 17 },
    { 0x31c594d8, 61 },
    { 0, 0 },
    { 0xe23594db, 91 },
    { 0x2b0594dc, 46 },
    { 0xc7b594db, 23 },
    { 0, 0 },
    { 0xc18594df, 43 },
    { 0xc00594e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x229594e3, 33 },
    { 0x292594e3, 61 },
    { 0xb1a594e4, 60 },
    { 0x299594e6, 33 },
    { 0x291594e7, 90 },
    { 0x9d4594e4, 61 },
    { 0x2e594e8, 75 },
    { 0xd50594ea, 77 },
    { 0x4f4594eb, 90 },
    { 0x946594ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389594f1, 23 },
    { 0x9e6594f1, 100 },
    { 0, 0 },
    { 0x492594f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7594ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba59502, 90 },
    { 0, 0 },
    { 0x2b559504, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d5950b, 77 },
    { 0x4865950c, 55 },
    { 0xc4d5950d, 76 },
    { 0x57e5950e, 100 },
    { 0xef75950f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5759512, 66 },
    { 0x49e59513, 55 },
    { 0x59959514, 33 },
    { 0xa9d59515, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b5951d, 61 },
    { 0xd615951e, 33 },
    { 0xafd5951d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e459522, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a59529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc59530, 93 },
    { 0x97f59531, 93 },
    { 0xe6e59532, 55 },
    { 0, 0 },
    { 0x22859534, 95 },
    { 0xd759535, 33 },
    { 0xcb59536, 33 },
    { 0xfe359537, 38 },
    { 0xe8159538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a5953c, 93 },
    { 0, 0 },
    { 0x5015953e, 88 },
    { 0x1365953f, 81 },
    { 0xefb59540, 90 },
    { 0x59559541, 93 },
    { 0xd9a59540, 95 },
    { 0x6455953e, 76 },
    { 0xb9b59544, 23 },
    { 0x5db59545, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5954a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925954d, 61 },
    { 0, 0 },
    { 0xdc45954f, 33 },
    { 0xcd959550, 87 },
    { 0xe8459550, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9659554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5955a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd95955e, 90 },
    { 0x4795955e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa59562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27159565, 34 },
    { 0, 0 },
    { 0x88859567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5956e, 95 },
    { 0xacb5956f, 61 },
    { 0x1fa59570, 68 },
    { 0xc0d5956e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60159575, 85 },
    { 0xa1759576, 76 },
    { 0x55b59577, 23 },
    { 0, 0 },
    { 0x5fa59579, 100 },
    { 0x6605957a, 61 },
    { 0xd235957b, 61 },
    { 0x77a5957a, 90 },
    { 0, 0 },
    { 0xbf75957e, 40 },
    { 0, 0 },
    { 0xad759580, 90 },
    { 0xef859581, 98 },
    { 0xa7f59582, 40 },
    { 0, 0 },
    { 0x1c459584, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8459590, 76 },
    { 0x88459591, 68 },
    { 0x40759590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f59598, 61 },
    { 0, 0 },
    { 0x7135959a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c595a1, 95 },
    { 0xb0e595a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e595a6, 99 },
    { 0xd7c595a6, 81 },
    { 0x753595a8, 77 },
    { 0xb80595a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86595ac, 76 },
    { 0xdde595ac, 98 },
    { 0x944595ae, 23 },
    { 0, 0 },
    { 0x113595b0, 33 },
    { 0xce8595b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df595b5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be595b8, 98 },
    { 0, 0 },
    { 0xf7b595ba, 75 },
    { 0, 0 },
    { 0x87595bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0595c3, 23 },
    { 0x58a595c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3595c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0595cf, 98 },
    { 0xfdb595cf, 33 },
    { 0xd17595d1, 23 },
    { 0x516595d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67595d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3595df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c595ea, 33 },
    { 0, 0 },
    { 0x27d595ec, 33 },
    { 0x3d8595ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1595f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453595f4, 23 },
    { 0xf23595f4, 61 },
    { 0x25f595f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1595ff, 33 },
    { 0, 0 },
    { 0x65059601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f59605, 33 },
    { 0, 0 },
    { 0x7bf59607, 68 },
    { 0x8e259608, 46 },
    { 0x6ea59609, 90 },
    { 0x1145960a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65960d, 23 },
    { 0, 0 },
    { 0x85960f, 66 },
    { 0xfb959610, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e59617, 23 },
    { 0, 0 },
    { 0x50f59619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f65961c, 99 },
    { 0x9145961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c59623, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5962d, 52 },
    { 0, 0 },
    { 0x1355962f, 98 },
    { 0x65d59630, 52 },
    { 0xf6f59631, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc59634, 90 },
    { 0x36059634, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8495963f, 90 },
    { 0x4ef5963f, 90 },
    { 0x1865963f, 90 },
    { 0x3b5963f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e59645, 33 },
    { 0x20159645, 85 },
    { 0, 0 },
    { 0x67559648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa835964c, 33 },
    { 0, 0 },
    { 0x3995964e, 23 },
    { 0x7b15964e, 33 },
    { 0x2b75964e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e59654, 61 },
    { 0x2ac59655, 43 },
    { 0, 0 },
    { 0xce59657, 90 },
    { 0, 0 },
    { 0xef959659, 95 },
    { 0x30e59659, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4005965e, 33 },
    { 0xa065965e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec59669, 98 },
    { 0x6955966a, 95 },
    { 0x3d05966b, 61 },
    { 0, 0 },
    { 0x21e5966d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4959679, 95 },
    { 0xfb65967a, 46 },
    { 0xde25967b, 99 },
    { 0x9225967c, 87 },
    { 0x4dd5967b, 61 },
    { 0x5585967e, 90 },
    { 0x8565967f, 64 },
    { 0xfa45967f, 23 },
    { 0x3b159681, 80 },
    { 0x70259681, 34 },
    { 0xf9e59683, 52 },
    { 0xccd59683, 61 },
    { 0xb8b59684, 33 },
    { 0, 0 },
    { 0xd2459687, 68 },
    { 0xf0959687, 61 },
    { 0x87259689, 98 },
    { 0xae5968a, 43 },
    { 0x9d15968b, 66 },
    { 0xf6a5968c, 95 },
    { 0xbc05968c, 93 },
    { 0x14e5968c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a59698, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4596a3, 61 },
    { 0x48a596a3, 23 },
    { 0x4e6596a5, 88 },
    { 0xd12596a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x491596a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa596ac, 90 },
    { 0, 0 },
    { 0x249596ae, 98 },
    { 0, 0 },
    { 0xeb9596b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931596b4, 78 },
    { 0, 0 },
    { 0x489596b6, 43 },
    { 0xaeb596b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110596c0, 34 },
    { 0x2a6596c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35596c4, 33 },
    { 0x789596c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389596c9, 85 },
    { 0, 0 },
    { 0x622596cb, 88 },
    { 0x8b8596cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221596d4, 93 },
    { 0x624596d5, 101 },
    { 0, 0 },
    { 0xa7596d7, 98 },
    { 0xdc8596d8, 87 },
    { 0, 0 },
    { 0xaf0596da, 76 },
    { 0, 0 },
    { 0xa35596dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30596df, 91 },
    { 0xbe3596e0, 98 },
    { 0x6f7596e1, 90 },
    { 0x94a596e1, 55 },
    { 0x792596df, 55 },
    { 0, 0 },
    { 0x88d596e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d596e8, 98 },
    { 0xf3596e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60596ef, 90 },
    { 0, 0 },
    { 0x4d6596f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb596f4, 87 },
    { 0, 0 },
    { 0xa4c596f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f596f9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef596fc, 76 },
    { 0x483596fd, 95 },
    { 0, 0 },
    { 0x1d8596ff, 98 },
    { 0xb6059700, 78 },
    { 0xe5959701, 90 },
    { 0x97759700, 67 },
    { 0x70659703, 33 },
    { 0x7da59704, 55 },
    { 0x71f59704, 90 },
    { 0x34b59706, 98 },
    { 0, 0 },
    { 0x6d059708, 33 },
    { 0, 0 },
    { 0x3135970a, 78 },
    { 0, 0 },
    { 0x2845970c, 76 },
    { 0x9385970d, 98 },
    { 0xc835970d, 43 },
    { 0xaef5970f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c859714, 34 },
    { 0x99859715, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d59719, 95 },
    { 0x7a45971a, 95 },
    { 0x8c45971a, 90 },
    { 0x1695971a, 100 },
    { 0, 0 },
    { 0x97e5971e, 98 },
    { 0x3915971f, 61 },
    { 0, 0 },
    { 0xbe059721, 52 },
    { 0x66b59722, 61 },
    { 0, 0 },
    { 0x76e59724, 33 },
    { 0xb4659725, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2365972a, 85 },
    { 0x7cf5972b, 85 },
    { 0xfb55972c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7255972f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25973b, 95 },
    { 0, 0 },
    { 0x135973d, 90 },
    { 0x4985973e, 77 },
    { 0x1395973d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f059748, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6005974b, 90 },
    { 0x3085974b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4245974f, 22 },
    { 0, 0 },
    { 0x43259751, 46 },
    { 0xd2659752, 98 },
    { 0, 0 },
    { 0x41359754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8245975c, 83 },
    { 0x66d5975c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0859761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab759765, 76 },
    { 0, 0 },
    { 0xb4459767, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf5976c, 17 },
    { 0xda5976d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d59770, 98 },
    { 0x44459771, 87 },
    { 0x6659772, 95 },
    { 0x56559770, 88 },
    { 0, 0 },
    { 0x21359775, 90 },
    { 0xa6c59776, 31 },
    { 0x3a359777, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5977b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995977f, 78 },
    { 0xe7859780, 77 },
    { 0xd2e59781, 95 },
    { 0xc7e5977f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9059785, 23 },
    { 0x9fa59786, 90 },
    { 0x4e59787, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25978d, 52 },
    { 0x5535978e, 77 },
    { 0x8285978f, 60 },
    { 0x78759790, 99 },
    { 0xbac59791, 23 },
    { 0x28959791, 61 },
    { 0x8da59792, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c5979a, 90 },
    { 0x6215979b, 34 },
    { 0x5695979c, 33 },
    { 0x6285979b, 33 },
    { 0xa3d5979e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53597a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4597a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6597ab, 90 },
    { 0xa52597ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1597b1, 33 },
    { 0, 0 },
    { 0x450597b3, 33 },
    { 0xd0e597b4, 95 },
    { 0x308597b5, 95 },
    { 0xa24597b5, 46 },
    { 0xf51597b7, 56 },
    { 0x727597b8, 90 },
    { 0xe43597b9, 33 },
    { 0xefa597ba, 99 },
    { 0x31a597b5, 85 },
    { 0, 0 },
    { 0xa2c597bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94597c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10597c4, 99 },
    { 0xb6f597c5, 90 },
    { 0x309597c4, 61 },
    { 0x500597c7, 95 },
    { 0, 0 },
    { 0x42a597c9, 98 },
    { 0, 0 },
    { 0x23b597cb, 65 },
    { 0x90597cb, 34 },
    { 0xb1d597cd, 23 },
    { 0x900597ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21597d2, 33 },
    { 0xdd5597d3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a597d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225597db, 60 },
    { 0, 0 },
    { 0xf7597dd, 85 },
    { 0, 0 },
    { 0x38b597df, 55 },
    { 0x781597df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f597e9, 23 },
    { 0xa43597ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x446597ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7597f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86597f8, 52 },
    { 0x4b3597f8, 61 },
    { 0xa78597f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab59801, 33 },
    { 0x3b59801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d59806, 90 },
    { 0, 0 },
    { 0x21359808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe645980e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d59812, 98 },
    { 0xe0059813, 61 },
    { 0x9da59814, 90 },
    { 0x91759815, 35 },
    { 0x6c259814, 33 },
    { 0x75c59815, 76 },
    { 0x13459817, 95 },
    { 0xb259819, 95 },
    { 0xc1e5981a, 90 },
    { 0x1e659815, 61 },
    { 0, 0 },
    { 0x735981d, 90 },
    { 0, 0 },
    { 0xf995981f, 61 },
    { 0x6b759820, 33 },
    { 0, 0 },
    { 0x7a459822, 61 },
    { 0xdda59822, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16959827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c5982b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42459830, 90 },
    { 0x4d359831, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a15983c, 33 },
    { 0, 0 },
    { 0xbe45983e, 91 },
    { 0, 0 },
    { 0x6d459840, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e59844, 99 },
    { 0, 0 },
    { 0x2b259846, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5705984c, 76 },
    { 0x4b55984c, 90 },
    { 0x8545984c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64359851, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x72959854, 91 },
    { 0x51159855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f559858, 55 },
    { 0xc2959859, 98 },
    { 0, 0 },
    { 0xacf5985b, 90 },
    { 0xb0a5985b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6259861, 61 },
    { 0x43859861, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad059867, 23 },
    { 0x33c59868, 85 },
    { 0x57e59869, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3485986f, 76 },
    { 0x31659870, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf959874, 90 },
    { 0xe2859875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc955987a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee59880, 33 },
    { 0xbe459880, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2159884, 23 },
    { 0xe9b59884, 55 },
    { 0x53059885, 75 },
    { 0xf1259884, 78 },
    { 0x30559887, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d15988c, 76 },
    { 0xd8f5988d, 98 },
    { 0xe785988e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa359891, 33 },
    { 0, 0 },
    { 0x6d459893, 76 },
    { 0xbd359894, 84 },
    { 0xf4459895, 95 },
    { 0xafd59893, 95 },
    { 0x1a59897, 95 },
    { 0x7d259898, 90 },
    { 0xa6d59898, 95 },
    { 0x1a459895, 43 },
    { 0x47159897, 98 },
    { 0xa759898, 78 },
    { 0xcd75989c, 90 },
    { 0xa225989d, 95 },
    { 0x71a5989a, 95 },
    { 0x9355989b, 93 },
    { 0, 0 },
    { 0x585598a2, 98 },
    { 0xd55598a3, 33 },
    { 0x17f598a4, 87 },
    { 0x357598a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0598ad, 99 },
    { 0, 0 },
    { 0xef5598af, 61 },
    { 0x60c598b0, 76 },
    { 0x660598b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67598b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db598be, 98 },
    { 0, 0 },
    { 0x93598c0, 51 },
    { 0, 0 },
    { 0xfd2598c2, 93 },
    { 0xd18598c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x303598c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75598ca, 33 },
    { 0, 0 },
    { 0xce1598cc, 91 },
    { 0x3ef598cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56598d0, 98 },
    { 0, 0 },
    { 0x24c598d2, 23 },
    { 0x99d598d2, 88 },
    { 0xb29598d4, 61 },
    { 0xfab598d4, 84 },
    { 0x628598d6, 93 },
    { 0, 0 },
    { 0x3b2598d8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93598dc, 90 },
    { 0x389598dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc598e2, 95 },
    { 0, 0 },
    { 0x4d598e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x875598e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12598eb, 99 },
    { 0, 0 },
    { 0x799598ed, 98 },
    { 0xd1c598ee, 55 },
    { 0, 0 },
    { 0x41b598f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b598f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d598f9, 43 },
    { 0xe9c598fa, 90 },
    { 0xeb3598fb, 33 },
    { 0x79a598fc, 61 },
    { 0, 0 },
    { 0x88e598fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d659905, 90 },
    { 0, 0 },
    { 0x37e59907, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55990b, 95 },
    { 0x50a5990c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb59914, 98 },
    { 0x1c559915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53159918, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d75991e, 95 },
    { 0x67d5991f, 61 },
    { 0xec159920, 23 },
    { 0x8f359921, 93 },
    { 0x42859920, 95 },
    { 0, 0 },
    { 0x84459924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55259927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265992a, 78 },
    { 0x8465992a, 34 },
    { 0x3525992c, 33 },
    { 0, 0 },
    { 0xc9f5992e, 96 },
    { 0x4335992e, 95 },
    { 0, 0 },
    { 0xa4a59931, 95 },
    { 0x59759932, 34 },
    { 0, 0 },
    { 0xe0b59934, 85 },
    { 0, 0 },
    { 0x3bc59936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef5993a, 33 },
    { 0xee95993b, 76 },
    { 0, 0 },
    { 0xa0e5993d, 61 },
    { 0x4445993d, 69 },
    { 0x6065993e, 33 },
    { 0xede59940, 34 },
    { 0x1b259940, 95 },
    { 0x68a59940, 55 },
    { 0x7dd59943, 98 },
    { 0x18059944, 76 },
    { 0x8e759943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96359949, 55 },
    { 0x7485994a, 84 },
    { 0x8fe5994a, 33 },
    { 0xea95994a, 55 },
    { 0xbad5994d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f59952, 87 },
    { 0x24159952, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65259958, 78 },
    { 0x43059958, 61 },
    { 0xd3a5995a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b5995f, 91 },
    { 0xcb359960, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb459968, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d5996c, 95 },
    { 0, 0 },
    { 0x8e65996e, 61 },
    { 0x95f5996e, 33 },
    { 0xe015996f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22059973, 78 },
    { 0xe2859973, 23 },
    { 0, 0 },
    { 0x42659976, 76 },
    { 0, 0 },
    { 0x90d59978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad05997c, 17 },
    { 0xb915997d, 52 },
    { 0x4585997d, 95 },
    { 0xaa05997d, 95 },
    { 0, 0 },
    { 0x56759981, 98 },
    { 0xda559982, 81 },
    { 0, 0 },
    { 0x82859984, 78 },
    { 0x5d859985, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2805998c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5459991, 85 },
    { 0xb3a59992, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64759999, 68 },
    { 0, 0 },
    { 0xdc65999b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245999e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c599a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a599a6, 98 },
    { 0x9f7599a7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f599b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75599b6, 99 },
    { 0x819599b7, 98 },
    { 0xc0b599b7, 95 },
    { 0x927599b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123599bf, 34 },
    { 0x8a1599bf, 67 },
    { 0x3e9599c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e599c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d599c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1599cd, 85 },
    { 0x436599cd, 95 },
    { 0xd0d599cd, 33 },
    { 0, 0 },
    { 0x189599d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984599d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd599da, 87 },
    { 0xfd8599db, 33 },
    { 0, 0 },
    { 0x4f3599dd, 91 },
    { 0x777599dd, 55 },
    { 0xe1d599de, 98 },
    { 0, 0 },
    { 0x6b599e1, 95 },
    { 0x366599e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35599e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b599e9, 33 },
    { 0x1ef599ea, 61 },
    { 0xabf599eb, 68 },
    { 0x24c599ec, 84 },
    { 0x696599ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x952599f0, 90 },
    { 0x4e1599f0, 33 },
    { 0, 0 },
    { 0x740599f3, 90 },
    { 0xb78599f4, 61 },
    { 0x8c0599f5, 98 },
    { 0x684599f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42599fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1599fe, 33 },
    { 0, 0 },
    { 0x99259a00, 61 },
    { 0x62459a00, 61 },
    { 0x60859a02, 98 },
    { 0xca759a03, 100 },
    { 0xc2b59a03, 90 },
    { 0x4d659a03, 65 },
    { 0, 0 },
    { 0x26459a07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1259a0b, 33 },
    { 0x49459a0c, 23 },
    { 0x34e59a0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa459a10, 95 },
    { 0, 0 },
    { 0xd5b59a12, 33 },
    { 0, 0 },
    { 0x29d59a14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc659a1b, 98 },
    { 0x9ea59a1c, 33 },
    { 0xaf759a1c, 85 },
    { 0x6be59a1b, 80 },
    { 0, 0 },
    { 0x70b59a20, 90 },
    { 0xf3c59a20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc59a26, 87 },
    { 0, 0 },
    { 0xfec59a28, 61 },
    { 0x4fa59a29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3959a2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad59a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b59a37, 24 },
    { 0x56259a38, 52 },
    { 0xc8d59a39, 23 },
    { 0xd8a59a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23959a3d, 38 },
    { 0x6d059a3d, 23 },
    { 0x9c759a3f, 43 },
    { 0x27859a3f, 98 },
    { 0x71959a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96659a44, 85 },
    { 0, 0 },
    { 0xd8959a46, 98 },
    { 0xe7859a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44859a4a, 76 },
    { 0, 0 },
    { 0xa4a59a4c, 90 },
    { 0x42d59a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0359a50, 90 },
    { 0xa6d59a51, 75 },
    { 0xf2459a52, 90 },
    { 0, 0 },
    { 0x86859a54, 90 },
    { 0, 0 },
    { 0xd7859a56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a59a5a, 64 },
    { 0, 0 },
    { 0xf6359a5c, 43 },
    { 0x13959a5c, 46 },
    { 0, 0 },
    { 0xf0759a5f, 76 },
    { 0xd8759a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7959a68, 61 },
    { 0xbc259a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d459a6f, 90 },
    { 0xd1059a70, 33 },
    { 0x21e59a70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99959a75, 46 },
    { 0x62e59a76, 33 },
    { 0xfcd59a77, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3259a7d, 84 },
    { 0xbf459a7d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9559a83, 43 },
    { 0x6b759a84, 90 },
    { 0x36259a85, 56 },
    { 0xeec59a86, 23 },
    { 0xc59a83, 23 },
    { 0x80c59a88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c659a8d, 43 },
    { 0xec59a8d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba959a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f759a96, 43 },
    { 0xd9d59a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0259a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c259a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5759aa0, 77 },
    { 0xe2259aa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59aa4, 61 },
    { 0x80359aa5, 90 },
    { 0x4d659aa4, 90 },
    { 0, 0 },
    { 0xf9b59aa8, 90 },
    { 0x6ce59aa8, 33 },
    { 0x55859aaa, 90 },
    { 0x4cd59aab, 17 },
    { 0, 0 },
    { 0x3a259aad, 61 },
    { 0xa8359aae, 88 },
    { 0x95e59aae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d859ab3, 88 },
    { 0xa0759ab4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90559ab9, 98 },
    { 0, 0 },
    { 0xffe59abb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6559abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8959ac1, 90 },
    { 0xbb459ac1, 93 },
    { 0, 0 },
    { 0x9be59ac4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda659ac7, 98 },
    { 0x17959ac8, 90 },
    { 0xf6d59ac9, 33 },
    { 0xe2b59ac9, 77 },
    { 0, 0 },
    { 0x57c59acc, 33 },
    { 0x5b359acc, 76 },
    { 0x33459acd, 76 },
    { 0, 0 },
    { 0xbe59ad0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x7959ad3, 78 },
    { 0xef059ad3, 23 },
    { 0xcba59ad4, 90 },
    { 0, 0 },
    { 0x6759ad7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96759adc, 66 },
    { 0x1d859add, 76 },
    { 0x8e959ade, 55 },
    { 0xa559adc, 23 },
    { 0xad959ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf59ae6, 23 },
    { 0, 0 },
    { 0xf5e59ae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f359aeb, 91 },
    { 0xd0159aec, 91 },
    { 0, 0 },
    { 0x45859aee, 90 },
    { 0x7ee59aef, 55 },
    { 0xf1459af0, 84 },
    { 0, 0 },
    { 0x42459af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c59af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f59af8, 95 },
    { 0x58859af8, 61 },
    { 0x96659afa, 33 },
    { 0, 0 },
    { 0xc6159afc, 33 },
    { 0xad759afc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e659b01, 98 },
    { 0, 0 },
    { 0x3ad59b03, 34 },
    { 0x2a259b04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b959b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f59b0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39959b0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b759b18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5959b1d, 88 },
    { 0x5cf59b1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e359b22, 61 },
    { 0x4aa59b23, 46 },
    { 0xca159b24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d59b2a, 96 },
    { 0xacf59b2b, 98 },
    { 0, 0 },
    { 0x92e59b2d, 33 },
    { 0, 0 },
    { 0x85059b2f, 98 },
    { 0, 0 },
    { 0xda359b31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b59b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb859b3a, 90 },
    { 0xeb259b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81759b3e, 34 },
    { 0xfbd59b3f, 46 },
    { 0x7c259b3e, 98 },
    { 0xb7159b41, 61 },
    { 0, 0 },
    { 0xb3e59b43, 90 },
    { 0, 0 },
    { 0x4ff59b45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94559b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e59b4b, 75 },
    { 0xb7c59b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2359b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e59b56, 23 },
    { 0, 0 },
    { 0xf3c59b58, 91 },
    { 0, 0 },
    { 0xb4859b5a, 93 },
    { 0xfb259b5b, 75 },
    { 0xea659b5c, 95 },
    { 0x5a059b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52959b62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b859b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c459b73, 93 },
    { 0, 0 },
    { 0x1359b75, 34 },
    { 0xbe359b76, 55 },
    { 0x62e59b76, 90 },
    { 0x6b559b78, 95 },
    { 0x2fb59b79, 84 },
    { 0x62a59b79, 60 },
    { 0x26559b7b, 17 },
    { 0, 0 },
    { 0x69c59b7d, 98 },
    { 0, 0 },
    { 0xa5759b7f, 23 },
    { 0x32559b80, 98 },
    { 0x5b759b81, 90 },
    { 0xe6359b82, 83 },
    { 0x75959b7f, 90 },
    { 0, 0 },
    { 0xd7e59b85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34559b8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf59b93, 53 },
    { 0, 0 },
    { 0xe2259b95, 33 },
    { 0, 0 },
    { 0x1f059b97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce59b9a, 69 },
    { 0x6c459b9b, 93 },
    { 0xad059b9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f59ba1, 95 },
    { 0x42059ba1, 98 },
    { 0xec859ba1, 95 },
    { 0x24059ba4, 98 },
    { 0, 0 },
    { 0x94b59ba6, 66 },
    { 0x1b659ba6, 43 },
    { 0, 0 },
    { 0x6ab59ba9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e259bb1, 93 },
    { 0, 0 },
    { 0xcb859bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43559bb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x17759bb9, 91 },
    { 0, 0 },
    { 0xccd59bbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b759bc0, 93 },
    { 0x15059bc1, 23 },
    { 0, 0 },
    { 0xd9e59bc3, 33 },
    { 0x73f59bc4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc59bca, 33 },
    { 0, 0 },
    { 0x4b259bcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa859bd1, 90 },
    { 0x69559bd2, 61 },
    { 0x54c59bd2, 33 },
    { 0xecb59bd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50959bd8, 48 },
    { 0x6dd59bd9, 97 },
    { 0x9f159bd8, 23 },
    { 0xd7359bdb, 99 },
    { 0xd4e59bda, 55 },
    { 0xbcc59bdb, 23 },
    { 0x5c659bde, 23 },
    { 0x39759bdf, 95 },
    { 0x86859bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a459be8, 87 },
    { 0xa0d59be8, 33 },
    { 0, 0 },
    { 0xb59beb, 46 },
    { 0xfc159bec, 52 },
    { 0xb3e59bed, 98 },
    { 0x16359bec, 36 },
    { 0xdc059bed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7059bf7, 66 },
    { 0x65959bf8, 33 },
    { 0xe3159bf9, 76 },
    { 0x45659bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e59c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6459c05, 12 },
    { 0, 0 },
    { 0xd4c59c07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea59c15, 55 },
    { 0xfe459c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a59c19, 33 },
    { 0x88459c1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58759c1e, 66 },
    { 0xa0e59c1e, 52 },
    { 0x159c1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf659c25, 33 },
    { 0x19059c26, 46 },
    { 0, 0 },
    { 0xd6059c28, 90 },
    { 0xb1859c29, 40 },
    { 0x65d59c2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63059c2f, 93 },
    { 0xc8659c30, 98 },
    { 0xf1e59c31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3259c38, 88 },
    { 0, 0 },
    { 0x1b359c3a, 97 },
    { 0xc1b59c3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38859c43, 23 },
    { 0x5f759c43, 98 },
    { 0x2d059c44, 55 },
    { 0, 0 },
    { 0x4f759c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df59c4e, 34 },
    { 0xd1459c4f, 33 },
    { 0x96a59c4e, 100 },
    { 0x22659c50, 78 },
    { 0xcf259c52, 61 },
    { 0xf5759c53, 99 },
    { 0, 0 },
    { 0x89959c55, 33 },
    { 0xf8c59c56, 98 },
    { 0, 0 },
    { 0x11259c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da59c5b, 90 },
    { 0, 0 },
    { 0xe1859c5d, 66 },
    { 0x9a059c5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50359c64, 90 },
    { 0x27259c65, 55 },
    { 0xa6659c66, 95 },
    { 0x29159c67, 52 },
    { 0x26d59c68, 52 },
    { 0xc3159c69, 90 },
    { 0, 0 },
    { 0xc059c6b, 95 },
    { 0x7fc59c6b, 95 },
    { 0xb0b59c6d, 90 },
    { 0, 0 },
    { 0x16059c6f, 36 },
    { 0x77d59c70, 75 },
    { 0xa2b59c6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab59c75, 23 },
    { 0, 0 },
    { 0xfe059c77, 43 },
    { 0, 0 },
    { 0x14359c79, 17 },
    { 0x4e859c7a, 33 },
    { 0x3d59c7b, 76 },
    { 0, 0 },
    { 0x71a59c7d, 33 },
    { 0x9159c7e, 90 },
    { 0xc0e59c7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad759c8d, 90 },
    { 0xb7059c8e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1259c91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89659c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc259c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef259ca4, 76 },
    { 0xbe159ca5, 91 },
    { 0xd8259ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e59cac, 90 },
    { 0xb4659cad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53859cb0, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f59cb3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b59cb6, 34 },
    { 0, 0 },
    { 0xa9659cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45959cbc, 46 },
    { 0x86b59cbc, 55 },
    { 0xa859cbe, 61 },
    { 0xfe959cbf, 76 },
    { 0xfcd59cc0, 23 },
    { 0xc4759cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a59cc5, 68 },
    { 0x2cd59cc5, 55 },
    { 0xca959cc6, 36 },
    { 0xd7859cc5, 65 },
    { 0x49059cc9, 66 },
    { 0xd3959cca, 55 },
    { 0xb9f59cc7, 91 },
    { 0x54a59cc7, 55 },
    { 0, 0 },
    { 0xd3259cce, 98 },
    { 0x75a59cce, 33 },
    { 0xed959cd0, 76 },
    { 0xc6b59cd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7659cd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8459cdc, 43 },
    { 0x1df59cdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74859ce6, 90 },
    { 0xad559ce6, 61 },
    { 0x4a559ce8, 34 },
    { 0xac759ce9, 91 },
    { 0xba359cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb59cf0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a59cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa59cf8, 43 },
    { 0xcd159cf8, 67 },
    { 0, 0 },
    { 0x5a859cfb, 33 },
    { 0xafb59cfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79559d00, 17 },
    { 0x6b559d00, 33 },
    { 0xe3b59d02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d59d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf59d0d, 38 },
    { 0xa6459d0e, 95 },
    { 0xed459d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a59d13, 33 },
    { 0, 0 },
    { 0x20559d15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd59d18, 90 },
    { 0xd759d19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a59d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f59d24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59d28, 23 },
    { 0x4dc59d29, 77 },
    { 0, 0 },
    { 0xb0f59d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19259d2f, 33 },
    { 0x88859d30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a59d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f59d39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea559d3d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6559d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f59d4b, 100 },
    { 0x66159d4c, 98 },
    { 0x2c59d4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7959d51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8759d56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6859d5a, 55 },
    { 0, 0 },
    { 0xc4359d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f59d5f, 61 },
    { 0x6959d60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db59d69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d859d6c, 33 },
    { 0x73e59d6d, 98 },
    { 0xcbf59d6e, 33 },
    { 0xa9a59d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a59d73, 76 },
    { 0, 0 },
    { 0xaed59d75, 83 },
    { 0x79b59d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14359d79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1359d7c, 95 },
    { 0x41259d7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c59d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d59d89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99259d8c, 90 },
    { 0x17459d8c, 34 },
    { 0, 0 },
    { 0x5b059d8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059d92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86859d9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a59d9f, 78 },
    { 0x2a359d9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c159da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f259daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10459dad, 90 },
    { 0x9ee59dad, 34 },
    { 0x68e59daf, 90 },
    { 0xf4e59daf, 43 },
    { 0xdd559db1, 55 },
    { 0xcc459db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c59db6, 23 },
    { 0xaa959db7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea59dba, 76 },
    { 0xf5359dbb, 90 },
    { 0x25759dbc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d59dc0, 91 },
    { 0xd559dc1, 98 },
    { 0, 0 },
    { 0x40559dc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x90659dc6, 61 },
    { 0x89159dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc59dd1, 61 },
    { 0x14759dd1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac59dd9, 33 },
    { 0, 0 },
    { 0xe459ddb, 85 },
    { 0x33859ddb, 43 },
    { 0, 0 },
    { 0xa0159dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98259de3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b59de9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd59dec, 90 },
    { 0x8c659dec, 90 },
    { 0xa5d59dec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf759df1, 43 },
    { 0, 0 },
    { 0xa759df3, 61 },
    { 0, 0 },
    { 0xe7d59df5, 65 },
    { 0x1fd59df5, 33 },
    { 0x71359df6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73259dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81659e00, 93 },
    { 0xdb459e00, 34 },
    { 0x5a959e02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28859e06, 90 },
    { 0, 0 },
    { 0xb3b59e08, 61 },
    { 0x55859e09, 33 },
    { 0xd4459e0a, 90 },
    { 0xbeb59e0a, 23 },
    { 0xd7659e09, 69 },
    { 0x6b759e0d, 67 },
    { 0, 0 },
    { 0x8e659e0f, 33 },
    { 0x57559e10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59e13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad059e1a, 33 },
    { 0, 0 },
    { 0xc8c59e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef959e1f, 55 },
    { 0x1ef59e20, 81 },
    { 0x9ee59e20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d59e24, 61 },
    { 0x7d359e24, 90 },
    { 0x1c159e26, 23 },
    { 0, 0 },
    { 0x55e59e28, 90 },
    { 0x37c59e29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99959e31, 98 },
    { 0, 0 },
    { 0x4c559e33, 87 },
    { 0xc9959e34, 68 },
    { 0xe8c59e33, 100 },
    { 0x7fd59e35, 61 },
    { 0x9ea59e37, 90 },
    { 0x81059e33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66859e3d, 93 },
    { 0x69659e3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b459e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c359e49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78059e4d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab59e64, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4259e68, 58 },
    { 0xc2659e69, 100 },
    { 0, 0 },
    { 0x37f59e6b, 33 },
    { 0xf6c59e6b, 76 },
    { 0x4b759e6b, 52 },
    { 0xe8459e6e, 85 },
    { 0x19259e6f, 65 },
    { 0x8eb59e6e, 23 },
    { 0x54f59e6d, 23 },
    { 0x82759e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36459e75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8559e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0959e7f, 33 },
    { 0x61659e80, 23 },
    { 0x90e59e7f, 33 },
    { 0xc7159e82, 81 },
    { 0, 0 },
    { 0x42459e84, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75359e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e59e8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82159e92, 23 },
    { 0x9f59e93, 61 },
    { 0xb7159e93, 98 },
    { 0x59959e95, 17 },
    { 0xef959e94, 84 },
    { 0x31959e97, 90 },
    { 0x34f59e96, 23 },
    { 0, 0 },
    { 0xe3659e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62559ea0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee759ea6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3359ea9, 46 },
    { 0, 0 },
    { 0x96559eab, 95 },
    { 0xc5f59eac, 76 },
    { 0x4b859ead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda59eb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84259ec0, 33 },
    { 0x90c59ec1, 76 },
    { 0xea259ec2, 10 },
    { 0, 0 },
    { 0xd2159ec4, 87 },
    { 0, 0 },
    { 0x3fd59ec6, 23 },
    { 0x85459ec7, 93 },
    { 0, 0 },
    { 0x64b59ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e59ece, 61 },
    { 0, 0 },
    { 0x2ed59ed0, 98 },
    { 0xe0859ed1, 23 },
    { 0xc6659ed1, 61 },
    { 0xbcd59ed1, 23 },
    { 0x30459ed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc859ed7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d59eda, 34 },
    { 0x9fb59edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c359edf, 90 },
    { 0xb3e59ee0, 90 },
    { 0x8d259ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8859ee9, 33 },
    { 0, 0 },
    { 0xc4c59eeb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b59ef2, 100 },
    { 0x8b759ef2, 93 },
    { 0xf3d59ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a159ef8, 33 },
    { 0x4cb59ef9, 90 },
    { 0x3f959efa, 99 },
    { 0xa9f59efb, 52 },
    { 0x14c59efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11159f01, 90 },
    { 0, 0 },
    { 0x7cf59f03, 46 },
    { 0x18b59f04, 34 },
    { 0xc9759f05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c59f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e159f0e, 76 },
    { 0, 0 },
    { 0x21459f10, 33 },
    { 0x38759f10, 76 },
    { 0, 0 },
    { 0x60559f13, 33 },
    { 0xc2b59f13, 61 },
    { 0, 0 },
    { 0x27f59f16, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73559f1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96859f21, 95 },
    { 0x41d59f22, 90 },
    { 0x18259f23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf59f26, 40 },
    { 0x42959f26, 76 },
    { 0x31559f27, 90 },
    { 0x53959f28, 33 },
    { 0x31459f2a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c59f32, 93 },
    { 0, 0 },
    { 0x7f859f34, 61 },
    { 0, 0 },
    { 0x9da59f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89459f39, 23 },
    { 0, 0 },
    { 0xb7159f3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14959f3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af59f42, 67 },
    { 0xa2259f43, 35 },
    { 0xd5a59f42, 90 },
    { 0xa9a59f45, 90 },
    { 0x60859f42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e59f49, 23 },
    { 0x29159f49, 76 },
    { 0xd3859f4b, 76 },
    { 0xe8859f49, 61 },
    { 0x5cf59f4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc59f5d, 43 },
    { 0xdef59f5e, 23 },
    { 0xeaf59f5f, 75 },
    { 0xdfa59f5e, 66 },
    { 0, 0 },
    { 0x99d59f62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd59f68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0859f6b, 44 },
    { 0, 0 },
    { 0xb8c59f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f59f72, 56 },
    { 0xdf859f73, 78 },
    { 0x38259f74, 17 },
    { 0x7ee59f74, 66 },
    { 0x57e59f73, 61 },
    { 0x7f459f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1359f7d, 55 },
    { 0xe9359f7d, 90 },
    { 0x25559f7e, 36 },
    { 0x6d159f80, 66 },
    { 0x52159f7f, 43 },
    { 0, 0 },
    { 0xbea59f83, 85 },
    { 0xd4159f83, 46 },
    { 0, 0 },
    { 0x6b059f86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62359f89, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x49459f8c, 43 },
    { 0xdef59f8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30659f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a59f95, 90 },
    { 0xdc659f96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd59f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78359f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76959f9f, 43 },
    { 0, 0 },
    { 0xc7a59fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d59fa8, 23 },
    { 0, 0 },
    { 0x6b159faa, 90 },
    { 0, 0 },
    { 0xeb159fac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x51859faf, 43 },
    { 0, 0 },
    { 0xeb959fb1, 76 },
    { 0, 0 },
    { 0xa5d59fb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b59fb7, 76 },
    { 0x9a459fb7, 90 },
    { 0, 0 },
    { 0x59259fba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c659fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84459fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff59fd8, 90 },
    { 0x2e59fd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe059fdd, 90 },
    { 0, 0 },
    { 0x32d59fdf, 76 },
    { 0, 0 },
    { 0x54059fe1, 95 },
    { 0x57159fe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86359fe5, 61 },
    { 0x79459fe5, 34 },
    { 0x12359fe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4559fec, 52 },
    { 0x2c59fed, 76 },
    { 0x8c159fed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba59ff4, 33 },
    { 0x68a59ff4, 90 },
    { 0x14159ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59ffb, 90 },
    { 0x78959ffc, 94 },
    { 0xb3959ffd, 40 },
    { 0xe9359ffd, 61 },
    { 0xb7c59fff, 93 },
    { 0x2d59ffc, 93 },
    { 0x2aa5a001, 67 },
    { 0xfa559ffd, 61 },
    { 0x91b5a003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115a006, 46 },
    { 0x5cf5a007, 95 },
    { 0x96f5a008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc5a00c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c5a00f, 33 },
    { 0x56c5a010, 90 },
    { 0x3b5a00f, 90 },
    { 0xb1f5a012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc615a018, 61 },
    { 0x68a5a019, 90 },
    { 0x8e45a01a, 76 },
    { 0xc445a01a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf625a022, 66 },
    { 0x7b75a023, 95 },
    { 0xad25a022, 90 },
    { 0, 0 },
    { 0x6745a026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba5a029, 76 },
    { 0, 0 },
    { 0xc115a02b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b5a032, 23 },
    { 0, 0 },
    { 0x3325a034, 76 },
    { 0xbfe5a034, 95 },
    { 0, 0 },
    { 0xb505a037, 93 },
    { 0x9085a038, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6725a03c, 33 },
    { 0x8d35a03d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a5a041, 23 },
    { 0, 0 },
    { 0x64c5a043, 84 },
    { 0xf3e5a044, 93 },
    { 0xb75a044, 98 },
    { 0, 0 },
    { 0x3905a047, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a04a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e35a04d, 98 },
    { 0x95a5a04e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4375a051, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d5a05f, 33 },
    { 0, 0 },
    { 0xd935a061, 90 },
    { 0, 0 },
    { 0x7b45a063, 23 },
    { 0x7095a063, 34 },
    { 0xa455a063, 69 },
    { 0x7855a065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c5a070, 95 },
    { 0xbeb5a071, 34 },
    { 0x1b95a072, 33 },
    { 0xebe5a072, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5a076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f5a080, 98 },
    { 0x1335a080, 98 },
    { 0xbea5a082, 88 },
    { 0x4a95a082, 90 },
    { 0xa195a084, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a089, 98 },
    { 0xa935a08a, 61 },
    { 0x2415a08b, 61 },
    { 0xa945a08c, 66 },
    { 0, 0 },
    { 0x25a5a08e, 98 },
    { 0, 0 },
    { 0xb275a090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa065a095, 61 },
    { 0x31a5a096, 95 },
    { 0x2235a096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7095a09a, 33 },
    { 0, 0 },
    { 0xa415a09c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb815a0a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf85a0a7, 95 },
    { 0xfa55a0a8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75a0ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda15a0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf975a0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd485a0b8, 61 },
    { 0, 0 },
    { 0xbeb5a0ba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d5a0bd, 33 },
    { 0x5825a0be, 23 },
    { 0x81e5a0bd, 61 },
    { 0xb585a0c0, 98 },
    { 0x1b55a0bf, 34 },
    { 0xb675a0c2, 76 },
    { 0xe0b5a0bf, 95 },
    { 0, 0 },
    { 0x44b5a0c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a5a0c8, 95 },
    { 0xde25a0c9, 34 },
    { 0xb335a0c9, 90 },
    { 0xe945a0cb, 98 },
    { 0xf055a0cc, 88 },
    { 0x61e5a0ca, 91 },
    { 0x3565a0cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd5a0dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df5a0e2, 100 },
    { 0xaa75a0e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef5a0e6, 90 },
    { 0, 0 },
    { 0x64d5a0e8, 33 },
    { 0xc525a0e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295a0ed, 94 },
    { 0xf285a0ee, 90 },
    { 0xe105a0ed, 46 },
    { 0x7c95a0f0, 33 },
    { 0x90c5a0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b55a0f4, 98 },
    { 0xd215a0f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255a0f8, 91 },
    { 0x8785a0f9, 76 },
    { 0xdb75a0fa, 61 },
    { 0xac45a0fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5a102, 31 },
    { 0, 0 },
    { 0xd255a104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce5a107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d5a10b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5a10e, 90 },
    { 0xb1a5a10f, 87 },
    { 0x78d5a110, 98 },
    { 0xea95a10e, 76 },
    { 0x10e5a10f, 23 },
    { 0x6815a10f, 77 },
    { 0xe1a5a114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7045a117, 52 },
    { 0xa215a118, 61 },
    { 0xcff5a119, 95 },
    { 0x7955a11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c5a11d, 76 },
    { 0xecb5a11e, 98 },
    { 0x7b85a11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75a125, 33 },
    { 0x9885a125, 34 },
    { 0x1925a127, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a35a12d, 88 },
    { 0x8925a12e, 98 },
    { 0xa7f5a12f, 95 },
    { 0x4ce5a12e, 61 },
    { 0x5985a131, 87 },
    { 0xcf45a131, 55 },
    { 0, 0 },
    { 0x1805a134, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635a142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb15a145, 46 },
    { 0x9475a146, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815a14a, 90 },
    { 0x5415a14b, 33 },
    { 0xe085a14c, 90 },
    { 0, 0 },
    { 0x5505a14e, 23 },
    { 0xd1f5a14f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d5a155, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875a15b, 52 },
    { 0, 0 },
    { 0xe735a15d, 76 },
    { 0xbc15a15e, 90 },
    { 0x3dd5a15f, 33 },
    { 0x9305a160, 34 },
    { 0, 0 },
    { 0xdd05a162, 70 },
    { 0, 0 },
    { 0x1325a164, 81 },
    { 0xf975a164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d5a169, 87 },
    { 0x7955a16a, 55 },
    { 0, 0 },
    { 0x12c5a16c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905a16f, 90 },
    { 0x9645a170, 23 },
    { 0xe9a5a171, 34 },
    { 0xe75a172, 77 },
    { 0x8165a16f, 61 },
    { 0xfc75a174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645a17e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b95a181, 23 },
    { 0, 0 },
    { 0xcce5a183, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef05a187, 90 },
    { 0, 0 },
    { 0x8695a189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8725a18c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25a18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a194, 90 },
    { 0x53a5a195, 90 },
    { 0xf455a194, 85 },
    { 0x7a75a197, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a85a19a, 33 },
    { 0, 0 },
    { 0x4595a19c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35a1a4, 61 },
    { 0x87d5a1a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf015a1a9, 83 },
    { 0x51b5a1aa, 23 },
    { 0x11f5a1ab, 61 },
    { 0, 0 },
    { 0xb995a1ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715a1b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5a1b4, 61 },
    { 0xf115a1b5, 23 },
    { 0x5395a1b6, 33 },
    { 0x8065a1b7, 65 },
    { 0xed15a1b6, 23 },
    { 0, 0 },
    { 0xe9c5a1ba, 60 },
    { 0x3e25a1bb, 87 },
    { 0xbbc5a1bc, 61 },
    { 0, 0 },
    { 0xa785a1be, 85 },
    { 0, 0 },
    { 0x26a5a1c0, 55 },
    { 0xdbe5a1c1, 95 },
    { 0xeea5a1c2, 67 },
    { 0xe0e5a1c3, 55 },
    { 0x7625a1c4, 90 },
    { 0, 0 },
    { 0xa105a1c6, 43 },
    { 0, 0 },
    { 0x21b5a1c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5a1cb, 67 },
    { 0x3ba5a1cc, 76 },
    { 0x1d05a1cc, 90 },
    { 0xeff5a1cc, 84 },
    { 0x9525a1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b5a1d3, 91 },
    { 0x13a5a1d3, 88 },
    { 0x38b5a1d5, 90 },
    { 0xc545a1d5, 95 },
    { 0x2585a1d7, 33 },
    { 0, 0 },
    { 0xcb95a1d9, 17 },
    { 0, 0 },
    { 0x6655a1db, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5a1e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a15a1e5, 33 },
    { 0, 0 },
    { 0x8de5a1e7, 34 },
    { 0, 0 },
    { 0xa225a1e9, 61 },
    { 0, 0 },
    { 0xb505a1eb, 90 },
    { 0xa095a1eb, 33 },
    { 0xd4e5a1ed, 52 },
    { 0x8e95a1ed, 90 },
    { 0xec15a1ef, 81 },
    { 0xe4c5a1f0, 61 },
    { 0x78d5a1ef, 52 },
    { 0, 0 },
    { 0xe7d5a1f3, 93 },
    { 0x1195a1f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d5a1f7, 93 },
    { 0x75e5a1f8, 67 },
    { 0x8cd5a1f7, 33 },
    { 0x8b35a1fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5a1fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e5a202, 61 },
    { 0x9fd5a203, 93 },
    { 0x3b15a203, 98 },
    { 0x9a05a204, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5a208, 95 },
    { 0, 0 },
    { 0xa4c5a20a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475a20e, 55 },
    { 0x6295a20f, 17 },
    { 0xf265a210, 23 },
    { 0x5825a211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a5a21a, 90 },
    { 0x5c55a21b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85a21e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c5a222, 76 },
    { 0, 0 },
    { 0x1695a224, 90 },
    { 0xb705a224, 90 },
    { 0xb705a225, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd545a229, 78 },
    { 0xe575a22a, 98 },
    { 0x6865a22b, 55 },
    { 0x41f5a229, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25a231, 95 },
    { 0x41a5a232, 90 },
    { 0x1955a233, 77 },
    { 0xcc85a231, 17 },
    { 0, 0 },
    { 0x7d85a236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025a239, 61 },
    { 0xce55a239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9165a23e, 98 },
    { 0xf075a23f, 76 },
    { 0x3e45a23e, 33 },
    { 0xc605a23f, 98 },
    { 0x66f5a242, 85 },
    { 0x2465a243, 76 },
    { 0x4d05a244, 87 },
    { 0xf315a240, 100 },
    { 0x2935a240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d5a249, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5a24c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5a24f, 98 },
    { 0x2ee5a24f, 19 },
    { 0x985a251, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4475a254, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb465a257, 33 },
    { 0, 0 },
    { 0x2e95a259, 78 },
    { 0, 0 },
    { 0x3f85a25b, 33 },
    { 0xecf5a25c, 23 },
    { 0x84c5a25d, 33 },
    { 0, 0 },
    { 0xa85a25f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d5a269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5a270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f5a273, 88 },
    { 0, 0 },
    { 0xb025a275, 36 },
    { 0xbb75a275, 94 },
    { 0xa65a277, 95 },
    { 0, 0 },
    { 0xaea5a279, 33 },
    { 0, 0 },
    { 0xd635a27b, 91 },
    { 0x8185a27c, 78 },
    { 0xd1c5a27d, 95 },
    { 0x2f5a27e, 34 },
    { 0x59d5a27f, 90 },
    { 0x2175a27c, 98 },
    { 0xa565a281, 52 },
    { 0, 0 },
    { 0x8425a283, 46 },
    { 0x675a284, 81 },
    { 0x1d05a285, 61 },
    { 0x625a286, 61 },
    { 0xb005a283, 76 },
    { 0, 0 },
    { 0x7165a289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555a291, 37 },
    { 0x4c75a291, 90 },
    { 0x55a293, 95 },
    { 0x275a293, 23 },
    { 0x8825a293, 33 },
    { 0xa65a296, 76 },
    { 0x6dc5a296, 33 },
    { 0xd9a5a296, 23 },
    { 0xe7a5a299, 90 },
    { 0x955a29a, 90 },
    { 0xf65a293, 23 },
    { 0x3685a294, 90 },
    { 0xe75a295, 23 },
    { 0, 0 },
    { 0x9885a29f, 95 },
    { 0x6ce5a29f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd5a2a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455a2a9, 61 },
    { 0, 0 },
    { 0xd365a2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1295a2b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd675a2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5a2b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a5a2bb, 95 },
    { 0xe385a2bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa15a2bf, 98 },
    { 0xc305a2c0, 23 },
    { 0xa5a2c1, 33 },
    { 0xe6f5a2c0, 95 },
    { 0, 0 },
    { 0x335a2c4, 61 },
    { 0, 0 },
    { 0xfc65a2c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb5a2cc, 23 },
    { 0x1585a2cd, 91 },
    { 0xaea5a2ce, 33 },
    { 0xdac5a2cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a2d3, 61 },
    { 0x3a15a2d3, 61 },
    { 0, 0 },
    { 0xfde5a2d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365a2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95a2dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc5a2e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5a2e9, 33 },
    { 0, 0 },
    { 0x825a2eb, 66 },
    { 0x6ca5a2eb, 40 },
    { 0, 0 },
    { 0x8995a2ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d5a2f2, 33 },
    { 0x5d05a2f3, 61 },
    { 0xa255a2f4, 33 },
    { 0xd975a2f4, 34 },
    { 0, 0 },
    { 0x2295a2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d15a305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995a308, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5a30b, 34 },
    { 0x6555a30b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb725a313, 98 },
    { 0x8e35a313, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa855a31a, 97 },
    { 0x1c55a31b, 95 },
    { 0xb205a31b, 33 },
    { 0x57e5a31c, 98 },
    { 0xf045a31e, 95 },
    { 0, 0 },
    { 0xdf5a320, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675a330, 90 },
    { 0x38c5a331, 59 },
    { 0x38d5a331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa5a335, 85 },
    { 0x36e5a336, 55 },
    { 0, 0 },
    { 0xd8a5a338, 95 },
    { 0x6205a339, 33 },
    { 0xaee5a33a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b5a346, 17 },
    { 0x7625a347, 33 },
    { 0x79c5a348, 85 },
    { 0x34b5a349, 66 },
    { 0xc1f5a347, 33 },
    { 0xe735a34b, 90 },
    { 0xcad5a34b, 91 },
    { 0x445a34b, 98 },
    { 0xd735a34d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc725a353, 34 },
    { 0xa745a354, 90 },
    { 0x59f5a355, 98 },
    { 0x70f5a356, 65 },
    { 0xf815a357, 90 },
    { 0x6a85a358, 100 },
    { 0x8745a355, 90 },
    { 0xca15a35a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd725a360, 95 },
    { 0x54c5a361, 100 },
    { 0x3325a362, 100 },
    { 0x79c5a363, 33 },
    { 0xe525a362, 94 },
    { 0x9495a365, 95 },
    { 0x6d45a365, 95 },
    { 0x10e5a367, 76 },
    { 0xbe15a368, 98 },
    { 0, 0 },
    { 0xe6a5a36a, 46 },
    { 0, 0 },
    { 0xf2f5a36c, 33 },
    { 0xbee5a36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d5a371, 75 },
    { 0, 0 },
    { 0xdaf5a373, 98 },
    { 0x1065a374, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5385a37c, 76 },
    { 0x8fd5a37d, 33 },
    { 0x5055a37e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf725a382, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c15a385, 76 },
    { 0xafc5a386, 90 },
    { 0xa885a385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b5a38a, 100 },
    { 0, 0 },
    { 0x1b35a38c, 81 },
    { 0x3da5a38d, 98 },
    { 0x785a38d, 100 },
    { 0xa825a38f, 91 },
    { 0xeef5a390, 61 },
    { 0xcf85a390, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac5a39b, 98 },
    { 0xdda5a39c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe615a39f, 33 },
    { 0x9cc5a3a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55a3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd285a3ab, 52 },
    { 0, 0 },
    { 0x8875a3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405a3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f5a3b6, 23 },
    { 0x1485a3b6, 61 },
    { 0x6bd5a3b8, 46 },
    { 0, 0 },
    { 0xbc05a3ba, 75 },
    { 0, 0 },
    { 0xdfc5a3bc, 33 },
    { 0xb365a3bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2605a3c1, 68 },
    { 0x10f5a3c2, 95 },
    { 0, 0 },
    { 0x1c95a3c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f5a3c7, 85 },
    { 0x8d95a3c8, 93 },
    { 0, 0 },
    { 0xf0c5a3ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975a3cd, 33 },
    { 0x1f75a3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f5a3d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab25a3d4, 23 },
    { 0x1ed5a3d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb015a3da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5a3e0, 95 },
    { 0, 0 },
    { 0xaa65a3e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e5a3eb, 66 },
    { 0xb5d5a3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5a3f0, 76 },
    { 0xe3c5a3f1, 90 },
    { 0x3895a3f0, 69 },
    { 0xeca5a3f3, 33 },
    { 0xcfb5a3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105a3f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4395a3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c5a3fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305a404, 33 },
    { 0, 0 },
    { 0x6b75a406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1795a40b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f15a412, 23 },
    { 0, 0 },
    { 0x975a414, 93 },
    { 0, 0 },
    { 0xce55a416, 95 },
    { 0x72f5a417, 90 },
    { 0x6f65a418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf335a41d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5a420, 36 },
    { 0xe7e5a421, 55 },
    { 0xa125a422, 75 },
    { 0xd3b5a423, 90 },
    { 0xdb95a423, 65 },
    { 0, 0 },
    { 0xbee5a426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5a429, 66 },
    { 0xb445a42a, 95 },
    { 0xa295a42a, 95 },
    { 0, 0 },
    { 0x93c5a42d, 98 },
    { 0xa5d5a42e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1545a437, 23 },
    { 0x15d5a438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55a43c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d35a441, 93 },
    { 0, 0 },
    { 0x4aa5a443, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c75a446, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5665a44b, 61 },
    { 0x8765a44c, 23 },
    { 0x1325a44d, 90 },
    { 0, 0 },
    { 0xfca5a44f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c45a454, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1015a457, 61 },
    { 0xcc95a457, 23 },
    { 0, 0 },
    { 0x23a5a45a, 98 },
    { 0xf795a45b, 98 },
    { 0xa2e5a45a, 98 },
    { 0x6b35a45b, 90 },
    { 0x5ff5a45e, 55 },
    { 0x3905a45f, 61 },
    { 0, 0 },
    { 0x2b45a461, 90 },
    { 0x7275a462, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf5a465, 98 },
    { 0x78e5a466, 43 },
    { 0x69f5a467, 76 },
    { 0, 0 },
    { 0x9635a469, 90 },
    { 0x5fe5a469, 40 },
    { 0x5e5a46b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8005a473, 46 },
    { 0, 0 },
    { 0x6ca5a475, 91 },
    { 0xe215a475, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be5a47a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf885a47e, 98 },
    { 0x2315a47f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa775a486, 61 },
    { 0xb675a487, 61 },
    { 0x5475a488, 93 },
    { 0xc095a488, 90 },
    { 0x9235a48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf595a48f, 100 },
    { 0x4005a490, 61 },
    { 0xe3a5a490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5a494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd5a497, 90 },
    { 0, 0 },
    { 0xf965a499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5a4a4, 98 },
    { 0xb0e5a4a5, 98 },
    { 0xb65a4a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad5a4a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905a4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1745a4b0, 34 },
    { 0xec65a4b1, 95 },
    { 0xbc85a4b0, 95 },
    { 0, 0 },
    { 0xb705a4b4, 61 },
    { 0, 0 },
    { 0xd15a4b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d5a4b9, 33 },
    { 0xde65a4b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85a4bf, 76 },
    { 0x7785a4c0, 52 },
    { 0, 0 },
    { 0x5b75a4c2, 61 },
    { 0xe0c5a4c3, 43 },
    { 0x36d5a4c4, 61 },
    { 0x3b05a4c5, 77 },
    { 0xd2e5a4c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2195a4c9, 90 },
    { 0x3f15a4ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25a4cd, 93 },
    { 0xeaf5a4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded5a4d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a05a4db, 61 },
    { 0, 0 },
    { 0x25a5a4dd, 95 },
    { 0x7095a4de, 55 },
    { 0xa095a4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65a4e3, 95 },
    { 0, 0 },
    { 0xd5f5a4e5, 98 },
    { 0x50e5a4e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9465a4eb, 98 },
    { 0x83a5a4ec, 98 },
    { 0x9995a4ed, 76 },
    { 0x8855a4ec, 95 },
    { 0x1b05a4ef, 61 },
    { 0x2835a4ee, 43 },
    { 0, 0 },
    { 0x4d95a4f2, 69 },
    { 0, 0 },
    { 0x5605a4f4, 95 },
    { 0xffc5a4f5, 95 },
    { 0xbec5a4f4, 55 },
    { 0, 0 },
    { 0xf0d5a4f8, 84 },
    { 0, 0 },
    { 0x94c5a4fa, 33 },
    { 0x89d5a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d5a4ff, 88 },
    { 0x8ae5a500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c5a503, 90 },
    { 0x55f5a503, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc5a507, 61 },
    { 0, 0 },
    { 0xe5f5a509, 99 },
    { 0x8375a50a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435a50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85a512, 76 },
    { 0xe855a512, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5a516, 55 },
    { 0x1b75a516, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf955a51b, 55 },
    { 0, 0 },
    { 0x9705a51d, 33 },
    { 0xfda5a51e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515a524, 98 },
    { 0xb425a525, 77 },
    { 0, 0 },
    { 0x9625a527, 56 },
    { 0x3765a527, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55a52d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8905a530, 90 },
    { 0xc625a531, 36 },
    { 0x8255a530, 93 },
    { 0, 0 },
    { 0x395a534, 90 },
    { 0, 0 },
    { 0x8605a536, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a53b, 91 },
    { 0x30d5a53c, 94 },
    { 0xb485a53d, 90 },
    { 0x805a53e, 61 },
    { 0x1ff5a53f, 34 },
    { 0x3405a53e, 93 },
    { 0x9535a53e, 52 },
    { 0x7715a542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8145a545, 100 },
    { 0xdbf5a545, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f5a54f, 52 },
    { 0x4995a54f, 90 },
    { 0, 0 },
    { 0x1e75a552, 52 },
    { 0x1375a552, 90 },
    { 0xfcc5a553, 93 },
    { 0x1075a554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a55a, 91 },
    { 0x5355a55b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f5a55e, 33 },
    { 0x39d5a55e, 23 },
    { 0x8845a55e, 23 },
    { 0, 0 },
    { 0xa55a562, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c65a567, 95 },
    { 0xfff5a568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185a56b, 52 },
    { 0x6b45a56b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff5a571, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215a574, 65 },
    { 0, 0 },
    { 0x9f15a576, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec55a57f, 61 },
    { 0x2215a580, 98 },
    { 0x4485a581, 34 },
    { 0xb6e5a582, 66 },
    { 0x4595a583, 33 },
    { 0, 0 },
    { 0xe45a585, 90 },
    { 0x8e55a585, 55 },
    { 0xb645a587, 55 },
    { 0x99c5a587, 78 },
    { 0, 0 },
    { 0xb215a58a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d75a594, 23 },
    { 0x6825a595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd5a59a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5a59d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba5a5aa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5a5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575a5b2, 98 },
    { 0x7565a5b3, 76 },
    { 0x4e85a5b3, 23 },
    { 0xa915a5b3, 99 },
    { 0, 0 },
    { 0x9a05a5b7, 33 },
    { 0, 0 },
    { 0x3ec5a5b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a5a5bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad75a5c2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5a5c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d5a5d1, 61 },
    { 0x8555a5d2, 93 },
    { 0xed05a5d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a5a5d8, 87 },
    { 0x8f75a5d8, 23 },
    { 0x7125a5da, 33 },
    { 0, 0 },
    { 0x8635a5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf05a5e1, 55 },
    { 0xd485a5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa545a5e5, 76 },
    { 0x8c45a5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa065a5ea, 90 },
    { 0x8055a5ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3935a5ee, 95 },
    { 0xc815a5ef, 33 },
    { 0x19e5a5ef, 90 },
    { 0x2315a5f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65a5f6, 95 },
    { 0x5b85a5f7, 46 },
    { 0, 0 },
    { 0xd3e5a5f9, 53 },
    { 0xe35a5fa, 75 },
    { 0x9795a5f9, 23 },
    { 0, 0 },
    { 0x5b5a5fd, 61 },
    { 0xa365a5fd, 43 },
    { 0x17f5a5ff, 90 },
    { 0x4015a5fe, 75 },
    { 0, 0 },
    { 0x98d5a602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9235a60a, 55 },
    { 0xdcd5a60b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385a60f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b5a614, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d5a617, 98 },
    { 0x9d25a618, 61 },
    { 0, 0 },
    { 0xb115a61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd05a61d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a622, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c75a625, 33 },
    { 0x57c5a626, 85 },
    { 0x10d5a627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da5a62b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65a62f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15a637, 23 },
    { 0x7d65a638, 98 },
    { 0xfa05a638, 55 },
    { 0, 0 },
    { 0xc7c5a63b, 83 },
    { 0x6ab5a63b, 55 },
    { 0, 0 },
    { 0xaff5a63e, 95 },
    { 0, 0 },
    { 0x7c95a640, 61 },
    { 0, 0 },
    { 0x89e5a642, 51 },
    { 0xddd5a643, 43 },
    { 0x8875a642, 68 },
    { 0x69b5a645, 85 },
    { 0xc165a642, 61 },
    { 0, 0 },
    { 0x7335a648, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205a64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645a64f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c5a653, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15a659, 99 },
    { 0x84d5a65a, 95 },
    { 0x4105a65b, 95 },
    { 0xb595a65c, 23 },
    { 0x26d5a65b, 32 },
    { 0x7f65a65e, 88 },
    { 0x60a5a65e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15a665, 43 },
    { 0, 0 },
    { 0x2c45a667, 61 },
    { 0x6455a668, 67 },
    { 0x5e55a669, 43 },
    { 0, 0 },
    { 0xfc05a66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e5a674, 87 },
    { 0x8535a675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee5a678, 98 },
    { 0, 0 },
    { 0xd515a67a, 84 },
    { 0xb4b5a67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8915a682, 95 },
    { 0x8b95a683, 95 },
    { 0xea95a684, 61 },
    { 0x74b5a685, 61 },
    { 0x56d5a686, 98 },
    { 0x5475a682, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db5a68c, 75 },
    { 0, 0 },
    { 0x99f5a68e, 23 },
    { 0, 0 },
    { 0x1825a690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9965a699, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f85a69c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1965a69f, 55 },
    { 0x5ab5a69f, 61 },
    { 0x7cf5a6a1, 52 },
    { 0xe905a6a0, 34 },
    { 0xe295a6a1, 23 },
    { 0xfa25a6a4, 61 },
    { 0x7715a6a4, 98 },
    { 0, 0 },
    { 0xe8a5a6a7, 61 },
    { 0, 0 },
    { 0x9095a6a9, 61 },
    { 0, 0 },
    { 0xcfe5a6ab, 95 },
    { 0x865a6ac, 78 },
    { 0x7ac5a6ad, 55 },
    { 0xfe35a6ae, 66 },
    { 0xad75a6ac, 33 },
    { 0x77b5a6ac, 87 },
    { 0xc085a6ad, 90 },
    { 0, 0 },
    { 0xd325a6b3, 33 },
    { 0xd855a6b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa35a6b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb35a6bf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5a6c7, 86 },
    { 0xd585a6c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5a6cd, 90 },
    { 0xd565a6ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8425a6d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535a6d7, 90 },
    { 0, 0 },
    { 0x79d5a6d9, 90 },
    { 0xe405a6d9, 93 },
    { 0x58a5a6d9, 88 },
    { 0, 0 },
    { 0xc425a6dd, 95 },
    { 0xa845a6de, 90 },
    { 0x43b5a6de, 68 },
    { 0x29d5a6de, 90 },
    { 0xd8e5a6df, 88 },
    { 0x6b75a6e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9995a6e8, 44 },
    { 0xd4c5a6e9, 53 },
    { 0x7e15a6ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb45a6f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15a6f3, 95 },
    { 0x5d05a6f4, 90 },
    { 0, 0 },
    { 0x7075a6f6, 98 },
    { 0, 0 },
    { 0xb645a6f8, 61 },
    { 0xf625a6f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc05a6fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf445a700, 67 },
    { 0x1155a701, 95 },
    { 0x6e65a702, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a35a707, 52 },
    { 0, 0 },
    { 0x2095a709, 23 },
    { 0, 0 },
    { 0x6315a70b, 23 },
    { 0, 0 },
    { 0xba35a70d, 90 },
    { 0x8ab5a70e, 98 },
    { 0, 0 },
    { 0xe575a710, 34 },
    { 0, 0 },
    { 0x8195a712, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc425a716, 34 },
    { 0x3275a717, 61 },
    { 0xf905a718, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a5a71e, 90 },
    { 0x4985a71e, 84 },
    { 0x1185a720, 76 },
    { 0xd035a71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7195a724, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305a727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa415a72f, 96 },
    { 0x7705a72f, 55 },
    { 0, 0 },
    { 0x9265a732, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415a735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e5a73c, 34 },
    { 0, 0 },
    { 0x9e65a73e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55a741, 61 },
    { 0, 0 },
    { 0xccc5a743, 59 },
    { 0, 0 },
    { 0xc635a745, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925a748, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5a74e, 23 },
    { 0xd395a74f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6935a753, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95a756, 95 },
    { 0xbea5a757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5a75a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5a75f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5a763, 99 },
    { 0, 0 },
    { 0x1845a765, 99 },
    { 0xa935a765, 23 },
    { 0x9095a765, 97 },
    { 0, 0 },
    { 0x60d5a769, 95 },
    { 0xd145a769, 90 },
    { 0, 0 },
    { 0x7105a76c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d5a770, 23 },
    { 0xbcf5a771, 95 },
    { 0x59f5a770, 23 },
    { 0xb7c5a770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3115a777, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f85a77d, 34 },
    { 0x9195a77e, 55 },
    { 0, 0 },
    { 0x8135a780, 98 },
    { 0x73c5a781, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5a788, 76 },
    { 0x7605a788, 81 },
    { 0xfa75a78a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a65a791, 90 },
    { 0, 0 },
    { 0xc305a793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae5a798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee25a79d, 33 },
    { 0xa9a5a79d, 47 },
    { 0x17a5a79f, 36 },
    { 0x5a95a79e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775a7a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f75a7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba5a7af, 95 },
    { 0xdb35a7b0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd615a7b7, 61 },
    { 0x1fa5a7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5a7bf, 66 },
    { 0x39e5a7bf, 34 },
    { 0xd935a7c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415a7c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb75a7c9, 88 },
    { 0, 0 },
    { 0xa3f5a7cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085a7d0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a7d3, 76 },
    { 0, 0 },
    { 0x3aa5a7d5, 23 },
    { 0xeb35a7d6, 37 },
    { 0x5fa5a7d6, 33 },
    { 0xc285a7d5, 55 },
    { 0xe5c5a7d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5a7df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb5a7e2, 61 },
    { 0, 0 },
    { 0xb4c5a7e4, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85a7e9, 81 },
    { 0x4265a7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5a7ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f25a7f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf095a7f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b5a7fc, 90 },
    { 0xf425a7fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1685a806, 90 },
    { 0, 0 },
    { 0x5ab5a808, 34 },
    { 0xbc15a808, 85 },
    { 0xc895a809, 95 },
    { 0xc705a80b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a75a816, 61 },
    { 0xeb15a816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb5a81a, 85 },
    { 0x1e05a81b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5a81e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65a824, 76 },
    { 0x1835a824, 95 },
    { 0x5b75a825, 33 },
    { 0x1bc5a827, 80 },
    { 0, 0 },
    { 0x9045a829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da5a830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975a834, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f5a83a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb685a83d, 95 },
    { 0x6de5a83e, 93 },
    { 0x69e5a83f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca5a842, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665a847, 90 },
    { 0, 0 },
    { 0x6795a849, 98 },
    { 0xe655a84a, 23 },
    { 0x6985a84b, 93 },
    { 0xf695a849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5a850, 23 },
    { 0xc1f5a851, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4325a854, 34 },
    { 0, 0 },
    { 0x6445a856, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85a85c, 43 },
    { 0, 0 },
    { 0xc8c5a85e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5a861, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7205a867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45a86c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb715a871, 18 },
    { 0x7225a872, 95 },
    { 0x48e5a872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415a876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a5a87c, 61 },
    { 0, 0 },
    { 0x87d5a87e, 90 },
    { 0x6025a87f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b5a884, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55a889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5a88c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5715a88f, 34 },
    { 0xf0a5a890, 84 },
    { 0, 0 },
    { 0xfc75a892, 90 },
    { 0xc375a893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e5a897, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa5a89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c5a8a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5a8a3, 99 },
    { 0xfb15a8a4, 43 },
    { 0x4be5a8a5, 67 },
    { 0x6d75a8a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f5a8b3, 100 },
    { 0xa995a8b4, 61 },
    { 0xc915a8b3, 24 },
    { 0x90e5a8b6, 61 },
    { 0x1315a8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e75a8bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6375a8bf, 98 },
    { 0xc205a8bf, 93 },
    { 0x2a15a8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365a8c4, 98 },
    { 0x3125a8c4, 69 },
    { 0x2cb5a8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5a8ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d95a8d1, 33 },
    { 0, 0 },
    { 0x5555a8d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a8d9, 95 },
    { 0, 0 },
    { 0x13e5a8db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c5a8de, 52 },
    { 0x2f85a8df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5a8e2, 55 },
    { 0x2725a8e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e5a8e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe805a8ec, 95 },
    { 0, 0 },
    { 0xe655a8ee, 66 },
    { 0x65a8ef, 87 },
    { 0xa025a8ef, 90 },
    { 0, 0 },
    { 0x3075a8f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a5a8f5, 61 },
    { 0x8a45a8f6, 33 },
    { 0, 0 },
    { 0xce5a8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2695a8fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2485a901, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa355a90a, 61 },
    { 0, 0 },
    { 0xa165a90c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5a912, 23 },
    { 0, 0 },
    { 0x8595a914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25a919, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe265a91d, 95 },
    { 0xfa25a91d, 36 },
    { 0, 0 },
    { 0xef55a920, 93 },
    { 0xe445a921, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf95a925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315a928, 61 },
    { 0x64a5a929, 98 },
    { 0xa665a92a, 91 },
    { 0x6e05a92b, 81 },
    { 0x285a92c, 95 },
    { 0x5af5a92d, 98 },
    { 0, 0 },
    { 0x3575a92f, 61 },
    { 0xf385a930, 93 },
    { 0x5905a931, 98 },
    { 0x1bf5a92f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a5a938, 23 },
    { 0xfd15a938, 61 },
    { 0xb665a938, 69 },
    { 0xb3c5a938, 61 },
    { 0xd055a93b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d65a941, 33 },
    { 0x67d5a942, 98 },
    { 0, 0 },
    { 0x43d5a944, 33 },
    { 0x59d5a945, 100 },
    { 0xbef5a945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5a949, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755a94c, 66 },
    { 0, 0 },
    { 0x86f5a94e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb5a951, 33 },
    { 0x8ef5a952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a95f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c45a965, 23 },
    { 0x54d5a966, 33 },
    { 0x38a5a966, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1835a96b, 61 },
    { 0, 0 },
    { 0x2635a96d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de5a970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4505a973, 34 },
    { 0xc7c5a974, 55 },
    { 0x7195a975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc125a97c, 100 },
    { 0, 0 },
    { 0x8635a97e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a5a986, 76 },
    { 0x2935a987, 52 },
    { 0x6475a987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5a98b, 78 },
    { 0xeb75a98c, 61 },
    { 0xa655a98d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5a996, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c25a99d, 88 },
    { 0xcde5a99e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d5a9a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf415a9a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a55a9b1, 90 },
    { 0xde75a9b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835a9b5, 98 },
    { 0, 0 },
    { 0x8b65a9b7, 48 },
    { 0x375a9b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975a9bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15a9c0, 76 },
    { 0, 0 },
    { 0xe2c5a9c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc5a9c7, 61 },
    { 0, 0 },
    { 0x9955a9c9, 61 },
    { 0x7765a9c9, 90 },
    { 0xeaf5a9cb, 61 },
    { 0x8ab5a9cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835a9cf, 33 },
    { 0xc865a9cf, 98 },
    { 0x4e15a9d1, 46 },
    { 0x32b5a9cf, 33 },
    { 0x1595a9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105a9d6, 45 },
    { 0x9285a9d7, 90 },
    { 0x44f5a9d8, 90 },
    { 0x89b5a9d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215a9dc, 100 },
    { 0x8c25a9dd, 90 },
    { 0, 0 },
    { 0x8ef5a9df, 34 },
    { 0xc5a5a9e0, 97 },
    { 0x5f25a9e0, 90 },
    { 0xc395a9e0, 17 },
    { 0, 0 },
    { 0x9a05a9e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a5a9e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc5a9f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6595a9fa, 75 },
    { 0xb1b5a9fb, 61 },
    { 0x8475a9fc, 93 },
    { 0x7eb5a9fd, 55 },
    { 0xa315a9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65aa01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435aa04, 61 },
    { 0xdd85aa05, 76 },
    { 0, 0 },
    { 0xec5aa07, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f5aa0a, 84 },
    { 0x47b5aa0b, 99 },
    { 0x3425aa0c, 33 },
    { 0x9f05aa0c, 76 },
    { 0x6d95aa0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5aa14, 95 },
    { 0xc345aa15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c5aa1a, 90 },
    { 0xe225aa1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f5aa20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075aa25, 84 },
    { 0xeaf5aa25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865aa29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7145aa2f, 95 },
    { 0x51b5aa2f, 61 },
    { 0x3015aa31, 43 },
    { 0x3485aa32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d55aa37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca25aa3b, 33 },
    { 0xb165aa3c, 52 },
    { 0xed65aa3c, 33 },
    { 0x6a5aa3c, 55 },
    { 0x7685aa3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1435aa44, 61 },
    { 0x9125aa45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f35aa48, 55 },
    { 0x6765aa48, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa5aa50, 90 },
    { 0xc425aa51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc5aa55, 100 },
    { 0x39c5aa55, 34 },
    { 0, 0 },
    { 0x9325aa58, 34 },
    { 0xe785aa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb5aa5c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715aa62, 95 },
    { 0, 0 },
    { 0x91c5aa64, 17 },
    { 0xdf65aa65, 52 },
    { 0x5225aa66, 33 },
    { 0x4355aa67, 43 },
    { 0xdda5aa67, 65 },
    { 0x1205aa69, 61 },
    { 0, 0 },
    { 0x1315aa6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1505aa71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065aa74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba55aa78, 61 },
    { 0xf575aa79, 53 },
    { 0xf5c5aa78, 91 },
    { 0x87a5aa7b, 46 },
    { 0xa685aa7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5aa85, 34 },
    { 0x7285aa85, 90 },
    { 0xd2e5aa87, 98 },
    { 0x9b45aa88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb95aa8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35aa97, 43 },
    { 0xd595aa97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2205aa9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45aa9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c95aaa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7685aaa8, 90 },
    { 0x5e45aaa9, 78 },
    { 0xd35aaa9, 87 },
    { 0x7b75aaab, 52 },
    { 0, 0 },
    { 0xac65aaad, 93 },
    { 0x5145aaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695aabc, 76 },
    { 0, 0 },
    { 0x2535aabe, 77 },
    { 0x5a55aabe, 23 },
    { 0xf4b5aac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b55aac4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5aac8, 55 },
    { 0, 0 },
    { 0xf925aaca, 55 },
    { 0x385aaca, 94 },
    { 0x47d5aacc, 90 },
    { 0xf115aacd, 43 },
    { 0, 0 },
    { 0x7c45aacf, 23 },
    { 0, 0 },
    { 0xf205aad1, 90 },
    { 0, 0 },
    { 0xbf5aad3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd185aad6, 90 },
    { 0, 0 },
    { 0x6c15aad8, 48 },
    { 0xf175aad9, 17 },
    { 0x5075aada, 98 },
    { 0xee05aadb, 66 },
    { 0x9de5aadc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95aadf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac5aae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf635aae8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd5aaeb, 98 },
    { 0xb185aaec, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a05aaef, 90 },
    { 0x3805aaef, 67 },
    { 0, 0 },
    { 0xbf95aaf2, 98 },
    { 0xdf5aaf3, 17 },
    { 0xe065aaf4, 90 },
    { 0, 0 },
    { 0xb595aaf6, 91 },
    { 0, 0 },
    { 0xee15aaf8, 95 },
    { 0, 0 },
    { 0x8815aafa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55aafd, 90 },
    { 0xc905aafe, 33 },
    { 0, 0 },
    { 0x5b65ab00, 98 },
    { 0xb085ab01, 90 },
    { 0x5965ab02, 78 },
    { 0xc485ab02, 93 },
    { 0x1255ab03, 43 },
    { 0xe9c5ab05, 66 },
    { 0xd4a5ab06, 93 },
    { 0x2bb5ab06, 33 },
    { 0xd275ab08, 61 },
    { 0x2905ab03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8795ab0c, 95 },
    { 0xbb15ab0d, 91 },
    { 0xf6f5ab0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b75ab11, 99 },
    { 0x5e75ab12, 23 },
    { 0xed25ab13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65ab1b, 55 },
    { 0x1495ab1b, 55 },
    { 0xb025ab1d, 36 },
    { 0, 0 },
    { 0x5bf5ab1f, 91 },
    { 0, 0 },
    { 0xc1c5ab21, 34 },
    { 0x1255ab21, 93 },
    { 0x5495ab23, 95 },
    { 0x4d05ab23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa385ab27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca35ab2b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x485ab2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe895ab33, 24 },
    { 0x8755ab34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb115ab38, 95 },
    { 0x7775ab38, 98 },
    { 0xc325ab38, 55 },
    { 0xef85ab3a, 23 },
    { 0x4be5ab39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e5ab46, 90 },
    { 0x3115ab46, 93 },
    { 0xfb95ab47, 98 },
    { 0xb635ab47, 99 },
    { 0xdb75ab49, 61 },
    { 0, 0 },
    { 0xc575ab4c, 90 },
    { 0x7365ab4d, 33 },
    { 0x2935ab4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5ab54, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f5ab57, 87 },
    { 0x6465ab58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed5ab5c, 98 },
    { 0, 0 },
    { 0x5d5ab5e, 33 },
    { 0xc4c5ab5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5ab6a, 100 },
    { 0xa735ab6b, 34 },
    { 0xccb5ab6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb25ab71, 43 },
    { 0, 0 },
    { 0xa0f5ab73, 98 },
    { 0, 0 },
    { 0x57a5ab75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775ab7a, 55 },
    { 0, 0 },
    { 0x83e5ab7c, 34 },
    { 0x4235ab7d, 90 },
    { 0xd0d5ab7d, 85 },
    { 0x64c5ab7e, 43 },
    { 0xa45ab7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb225ab84, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6945ab87, 76 },
    { 0x1725ab88, 43 },
    { 0xc675ab87, 87 },
    { 0x7d85ab8a, 76 },
    { 0x3735ab8a, 61 },
    { 0xbd45ab8c, 33 },
    { 0x30c5ab8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5ab93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ab97, 90 },
    { 0x7a65ab98, 23 },
    { 0, 0 },
    { 0x9655ab9a, 33 },
    { 0x18f5ab9a, 75 },
    { 0x55d5ab9a, 98 },
    { 0x1445ab9d, 33 },
    { 0x6415ab9a, 33 },
    { 0xd245ab9f, 61 },
    { 0x4605ab9a, 90 },
    { 0xb175aba1, 52 },
    { 0, 0 },
    { 0x5475aba3, 100 },
    { 0x5545aba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5aba8, 78 },
    { 0xf225aba8, 61 },
    { 0x5e85abaa, 23 },
    { 0x2ca5abab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7095abaf, 84 },
    { 0, 0 },
    { 0xfc15abb1, 90 },
    { 0x43c5abb2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d15abb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1305abbb, 95 },
    { 0x91e5abbc, 90 },
    { 0x7995abbd, 33 },
    { 0x62a5abbc, 82 },
    { 0, 0 },
    { 0x6225abc0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15abc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05abce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb925abd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a75abd6, 23 },
    { 0xd5b5abd7, 46 },
    { 0x8325abd8, 95 },
    { 0x42e5abd7, 55 },
    { 0xd205abda, 90 },
    { 0x9345abdb, 46 },
    { 0x7645abdb, 43 },
    { 0x8bb5abda, 33 },
    { 0xa2b5abda, 67 },
    { 0x1815abdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715abe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5abf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf5abf8, 81 },
    { 0xa7e5abf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b5ac00, 88 },
    { 0xdba5ac01, 61 },
    { 0, 0 },
    { 0x3e05ac03, 87 },
    { 0, 0 },
    { 0xac25ac05, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45ac0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf805ac11, 23 },
    { 0xdb5ac12, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ac15, 61 },
    { 0x575ac16, 90 },
    { 0xb235ac17, 33 },
    { 0x9345ac16, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe595ac20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff5ac25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea5ac28, 100 },
    { 0, 0 },
    { 0xf275ac2a, 61 },
    { 0, 0 },
    { 0xd9e5ac2c, 33 },
    { 0xb215ac2d, 43 },
    { 0xa1f5ac2e, 33 },
    { 0xbd05ac2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415ac38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a5ac3c, 98 },
    { 0x38e5ac3d, 93 },
    { 0x8785ac3d, 95 },
    { 0x2835ac3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5ac42, 84 },
    { 0, 0 },
    { 0x64a5ac44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7055ac47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff25ac4b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c5ac50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc85ac53, 95 },
    { 0xa0d5ac54, 64 },
    { 0x4c85ac55, 61 },
    { 0x7845ac54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215ac59, 33 },
    { 0xccf5ac5a, 66 },
    { 0x2c05ac59, 95 },
    { 0, 0 },
    { 0xbcc5ac5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265ac61, 61 },
    { 0x5995ac62, 98 },
    { 0x6965ac61, 98 },
    { 0x6695ac63, 84 },
    { 0xf605ac65, 90 },
    { 0xcbb5ac66, 95 },
    { 0xe655ac63, 90 },
    { 0xd1e5ac65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d5ac6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5ac6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5ac72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3375ac75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef15ac79, 95 },
    { 0x8655ac79, 95 },
    { 0x2f35ac7a, 100 },
    { 0, 0 },
    { 0xbc45ac7d, 98 },
    { 0xd845ac7e, 61 },
    { 0xd105ac7f, 100 },
    { 0xaf45ac7e, 33 },
    { 0xa9a5ac7e, 93 },
    { 0x71f5ac82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf5ac91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf75ac96, 90 },
    { 0x3535ac97, 55 },
    { 0, 0 },
    { 0xc265ac99, 90 },
    { 0x6b95ac9a, 91 },
    { 0x535ac99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5ac9f, 95 },
    { 0x6635aca0, 67 },
    { 0x70f5aca1, 60 },
    { 0xd4c5aca2, 33 },
    { 0xb115aca2, 100 },
    { 0xc815aca4, 90 },
    { 0x3e05aca5, 95 },
    { 0, 0 },
    { 0x4f15aca7, 90 },
    { 0x5af5aca8, 90 },
    { 0, 0 },
    { 0x7415acaa, 23 },
    { 0x48c5acab, 99 },
    { 0x1a95acac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd515acb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf225acb6, 61 },
    { 0xb8b5acb7, 55 },
    { 0x78e5acb6, 76 },
    { 0x1a15acb8, 61 },
    { 0x89f5acba, 33 },
    { 0xc2a5acba, 46 },
    { 0x2695acb7, 36 },
    { 0, 0 },
    { 0xfe85acbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe5acc1, 81 },
    { 0xb75acc2, 55 },
    { 0x7555acc3, 78 },
    { 0, 0 },
    { 0xfde5acc5, 33 },
    { 0, 0 },
    { 0x2e45acc7, 34 },
    { 0x19c5acc8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd5accb, 23 },
    { 0xe3d5accc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615accf, 55 },
    { 0, 0 },
    { 0x7e5acd1, 95 },
    { 0x795acd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75acd5, 76 },
    { 0xf855acd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5045acdc, 98 },
    { 0x16b5acdd, 87 },
    { 0x74f5acde, 23 },
    { 0xa75acdf, 46 },
    { 0x5365ace0, 90 },
    { 0x7d65acdd, 23 },
    { 0x9535acde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea5ace5, 88 },
    { 0xa0e5ace6, 78 },
    { 0x9865ace6, 93 },
    { 0x8275ace8, 17 },
    { 0x48a5ace9, 34 },
    { 0xef65ace5, 95 },
    { 0xcc25ace8, 61 },
    { 0xf415aceb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1405acef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a5acf4, 98 },
    { 0x6535acf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5acf9, 95 },
    { 0xdc15acfa, 90 },
    { 0x61c5acfa, 95 },
    { 0x4785acfc, 55 },
    { 0x85f5acfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce35ad00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f35ad05, 61 },
    { 0x2355ad05, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e5ad0a, 33 },
    { 0xab15ad0a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb855ad0e, 61 },
    { 0xf545ad0f, 90 },
    { 0x9805ad10, 91 },
    { 0x49d5ad0f, 90 },
    { 0x8a45ad12, 91 },
    { 0x55a5ad12, 99 },
    { 0, 0 },
    { 0x3d55ad15, 33 },
    { 0x5a55ad15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515ad1a, 55 },
    { 0x7aa5ad1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc515ad1f, 55 },
    { 0x22b5ad20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf815ad23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4575ad29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c5ad2e, 93 },
    { 0xd5e5ad2f, 37 },
    { 0, 0 },
    { 0x9d65ad31, 61 },
    { 0x8125ad32, 99 },
    { 0xca05ad31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295ad37, 95 },
    { 0, 0 },
    { 0x1eb5ad39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25ad3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a45ad43, 33 },
    { 0, 0 },
    { 0xaee5ad45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda15ad4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8125ad4e, 99 },
    { 0, 0 },
    { 0x69d5ad50, 68 },
    { 0xe515ad50, 61 },
    { 0xaf95ad50, 52 },
    { 0xd15ad53, 33 },
    { 0x1365ad54, 95 },
    { 0xbe65ad54, 33 },
    { 0x6dd5ad56, 75 },
    { 0x5105ad57, 23 },
    { 0x7225ad56, 23 },
    { 0xdc5ad59, 95 },
    { 0x11b5ad59, 23 },
    { 0xc9a5ad5a, 93 },
    { 0, 0 },
    { 0x3835ad5d, 90 },
    { 0xcae5ad5e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145ad63, 95 },
    { 0xa015ad64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8005ad67, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285ad6d, 65 },
    { 0xad85ad6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e95ad71, 84 },
    { 0x5d15ad71, 95 },
    { 0, 0 },
    { 0x2bf5ad74, 23 },
    { 0x2565ad75, 43 },
    { 0, 0 },
    { 0xacd5ad77, 81 },
    { 0, 0 },
    { 0x12a5ad79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7495ad7d, 95 },
    { 0xd325ad7e, 23 },
    { 0x97a5ad7f, 55 },
    { 0x67d5ad7e, 76 },
    { 0x1dc5ad7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ad8c, 90 },
    { 0x45e5ad8d, 90 },
    { 0x76e5ad8d, 33 },
    { 0, 0 },
    { 0xb315ad90, 34 },
    { 0xa5e5ad91, 66 },
    { 0x4335ad92, 76 },
    { 0, 0 },
    { 0x7255ad94, 84 },
    { 0xce05ad95, 69 },
    { 0x21b5ad96, 33 },
    { 0x4645ad95, 90 },
    { 0xa035ad97, 33 },
    { 0x6ad5ad99, 98 },
    { 0xdff5ad96, 61 },
    { 0x88e5ad9b, 23 },
    { 0x48b5ad95, 95 },
    { 0, 0 },
    { 0xf45ad9e, 23 },
    { 0x3ec5ad9f, 95 },
    { 0, 0 },
    { 0xe5e5ada1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5ada4, 93 },
    { 0xc915ada4, 46 },
    { 0x8a85ada6, 98 },
    { 0x8315ada7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc505adab, 23 },
    { 0, 0 },
    { 0xf255adad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5adb0, 90 },
    { 0x6a65adb0, 91 },
    { 0, 0 },
    { 0xe4b5adb3, 95 },
    { 0x40e5adb4, 98 },
    { 0x6125adb5, 61 },
    { 0xa595adb6, 33 },
    { 0x7f95adb7, 61 },
    { 0x6125adb8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5adc2, 90 },
    { 0x6d35adc3, 66 },
    { 0x2795adc4, 90 },
    { 0x3ff5adc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb995adc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2615adcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2495add7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa835adde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5405ade1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865ade5, 93 },
    { 0xc9f5ade6, 90 },
    { 0xe315ade7, 65 },
    { 0x9f65ade8, 34 },
    { 0x3885ade9, 90 },
    { 0x8145adea, 33 },
    { 0xb705adeb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15adef, 98 },
    { 0x8bd5adf0, 99 },
    { 0xbb25adf1, 67 },
    { 0x8cf5adf2, 65 },
    { 0xd3a5adf1, 98 },
    { 0x6ec5adf4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e5adf7, 23 },
    { 0, 0 },
    { 0xd105adf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905adfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415ae08, 90 },
    { 0xa235ae09, 33 },
    { 0x1ec5ae0a, 99 },
    { 0xc3e5ae0a, 99 },
    { 0xcc15ae0a, 98 },
    { 0x9b55ae0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb5ae13, 95 },
    { 0xae75ae13, 87 },
    { 0xa175ae13, 33 },
    { 0xee35ae13, 93 },
    { 0x4e05ae14, 58 },
    { 0x4925ae18, 51 },
    { 0x765ae18, 98 },
    { 0x12b5ae19, 90 },
    { 0, 0 },
    { 0x2ed5ae1c, 76 },
    { 0xaea5ae1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45ae20, 33 },
    { 0xe625ae20, 98 },
    { 0xfe15ae22, 90 },
    { 0, 0 },
    { 0x9c65ae24, 23 },
    { 0xaf45ae25, 61 },
    { 0xff85ae26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e5ae35, 33 },
    { 0x1685ae36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed5ae3b, 93 },
    { 0x85ae3c, 75 },
    { 0x3875ae3c, 33 },
    { 0x6e85ae3d, 85 },
    { 0xce15ae3b, 100 },
    { 0xd565ae40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a5ae43, 55 },
    { 0, 0 },
    { 0x44c5ae45, 90 },
    { 0, 0 },
    { 0x1565ae47, 55 },
    { 0x8e35ae47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8935ae4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7035ae51, 77 },
    { 0, 0 },
    { 0xa0e5ae53, 34 },
    { 0xca15ae54, 90 },
    { 0, 0 },
    { 0x7195ae56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa475ae59, 61 },
    { 0x9c85ae5a, 33 },
    { 0, 0 },
    { 0x67e5ae5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f5ae5f, 90 },
    { 0x2205ae5f, 90 },
    { 0x92e5ae61, 93 },
    { 0x27c5ae5f, 23 },
    { 0x5785ae62, 33 },
    { 0x2915ae62, 98 },
    { 0x31e5ae65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a5ae69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d5ae6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b5ae72, 99 },
    { 0, 0 },
    { 0x2845ae74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f5ae77, 87 },
    { 0x7fa5ae77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5165ae7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5ae81, 77 },
    { 0x2975ae81, 33 },
    { 0, 0 },
    { 0xb875ae84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235ae88, 93 },
    { 0, 0 },
    { 0xd955ae8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635ae90, 90 },
    { 0x855ae91, 95 },
    { 0xc4d5ae90, 88 },
    { 0, 0 },
    { 0x5305ae94, 40 },
    { 0xe8e5ae95, 23 },
    { 0x3c95ae96, 90 },
    { 0xb945ae97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605ae9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865aea2, 90 },
    { 0, 0 },
    { 0xf6a5aea4, 34 },
    { 0xb875aea5, 33 },
    { 0x89e5aea4, 81 },
    { 0xc9f5aea6, 46 },
    { 0x3015aea4, 76 },
    { 0x4de5aea9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4255aeb4, 23 },
    { 0xf165aeb5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb5aeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3655aebb, 95 },
    { 0xb795aebc, 100 },
    { 0xbb25aebd, 90 },
    { 0xb725aebd, 85 },
    { 0x1945aebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd315aec3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d15aec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c15aece, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5aed2, 61 },
    { 0, 0 },
    { 0x33b5aed4, 43 },
    { 0, 0 },
    { 0x3255aed6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195aedb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25aede, 90 },
    { 0x21e5aede, 61 },
    { 0xc8f5aedf, 90 },
    { 0xa135aee1, 90 },
    { 0xb705aee2, 23 },
    { 0x7655aee3, 90 },
    { 0xaf35aee4, 99 },
    { 0, 0 },
    { 0x2255aee6, 90 },
    { 0x9965aee7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c35aeea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795aef1, 95 },
    { 0x6795aef2, 17 },
    { 0x2e65aef3, 95 },
    { 0x7f65aef4, 17 },
    { 0, 0 },
    { 0x7bb5aef6, 95 },
    { 0, 0 },
    { 0x765aef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb5af00, 67 },
    { 0xff65af01, 33 },
    { 0, 0 },
    { 0xc215af03, 61 },
    { 0xb835af04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925af07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe65af0c, 55 },
    { 0xa145af0d, 61 },
    { 0xae25af0e, 95 },
    { 0xe135af0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5af12, 66 },
    { 0x42d5af13, 61 },
    { 0, 0 },
    { 0xc0e5af15, 95 },
    { 0x3355af16, 55 },
    { 0xb4c5af16, 88 },
    { 0x815af18, 98 },
    { 0x1365af18, 34 },
    { 0, 0 },
    { 0xe905af1b, 90 },
    { 0x63f5af1c, 90 },
    { 0x5bc5af1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4635af20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715af25, 93 },
    { 0, 0 },
    { 0xd445af27, 55 },
    { 0xdf35af27, 93 },
    { 0, 0 },
    { 0x3f95af2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c5af2d, 100 },
    { 0x39d5af2e, 81 },
    { 0xf245af2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995af33, 17 },
    { 0xf355af34, 93 },
    { 0x3ff5af35, 61 },
    { 0x6375af36, 90 },
    { 0x8ec5af35, 93 },
    { 0x9bf5af38, 95 },
    { 0x3e25af34, 95 },
    { 0xc2a5af34, 90 },
    { 0x3355af37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f35af3e, 55 },
    { 0, 0 },
    { 0xd325af40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5af43, 67 },
    { 0x7115af44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db5af52, 33 },
    { 0x6ac5af52, 76 },
    { 0, 0 },
    { 0x4785af55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2385af5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5af5e, 43 },
    { 0x3cc5af5f, 98 },
    { 0xa1c5af60, 90 },
    { 0, 0 },
    { 0x5ea5af62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac5af6c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa895af6f, 43 },
    { 0x5115af70, 61 },
    { 0, 0 },
    { 0x9cb5af72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe055af76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae15af7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c5af7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5af85, 98 },
    { 0, 0 },
    { 0xe25af87, 88 },
    { 0x2fd5af87, 61 },
    { 0x9ca5af87, 33 },
    { 0x2dd5af8a, 61 },
    { 0xe0a5af8b, 88 },
    { 0xee45af8c, 85 },
    { 0x1115af8b, 66 },
    { 0x1c75af87, 90 },
    { 0x7eb5af8f, 68 },
    { 0x9ab5af8e, 98 },
    { 0x1c35af91, 33 },
    { 0xd975af89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315af95, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xba75af98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65af9e, 95 },
    { 0, 0 },
    { 0x8b15afa0, 84 },
    { 0xd9e5afa1, 61 },
    { 0xda95afa2, 98 },
    { 0xe1f5afa3, 23 },
    { 0xdbe5afa3, 36 },
    { 0, 0 },
    { 0x875afa6, 81 },
    { 0xb165afa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5afab, 90 },
    { 0x20a5afac, 33 },
    { 0xd3f5afac, 55 },
    { 0x2695afae, 46 },
    { 0xbd65afaf, 3 },
    { 0xb485afb0, 90 },
    { 0, 0 },
    { 0x43f5afb2, 95 },
    { 0, 0 },
    { 0x8e25afb4, 29 },
    { 0, 0 },
    { 0xc985afb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee75afb9, 98 },
    { 0xe0a5afba, 33 },
    { 0, 0 },
    { 0x3975afbc, 76 },
    { 0x125afbc, 55 },
    { 0x4c55afbe, 95 },
    { 0x94c5afbf, 95 },
    { 0, 0 },
    { 0x61a5afc1, 23 },
    { 0x7325afc2, 23 },
    { 0xdb55afc2, 90 },
    { 0xe9a5afc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c95afca, 95 },
    { 0x35a5afcb, 78 },
    { 0xde85afcb, 76 },
    { 0x1245afcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7285afd6, 33 },
    { 0x64a5afd7, 81 },
    { 0, 0 },
    { 0x2885afd9, 33 },
    { 0, 0 },
    { 0x2f5afdb, 33 },
    { 0, 0 },
    { 0xf895afdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945afe1, 24 },
    { 0, 0 },
    { 0xe295afe3, 98 },
    { 0x2d55afe4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd105afe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5585afee, 34 },
    { 0, 0 },
    { 0xecd5aff0, 95 },
    { 0xd285aff1, 33 },
    { 0, 0 },
    { 0x825aff3, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005aff6, 78 },
    { 0xed15aff7, 23 },
    { 0x82e5aff7, 100 },
    { 0x27d5aff7, 33 },
    { 0x9bb5affa, 94 },
    { 0x4d5affa, 46 },
    { 0xdfc5affb, 33 },
    { 0xad35affb, 90 },
    { 0x4315affe, 61 },
    { 0x8225afff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca5b006, 85 },
    { 0x52c5b007, 98 },
    { 0x2525b007, 43 },
    { 0xd985b009, 55 },
    { 0x8b85b006, 87 },
    { 0x6595b00b, 90 },
    { 0xef95b00c, 77 },
    { 0x3395b00d, 55 },
    { 0xe845b00d, 90 },
    { 0x3005b00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5b015, 33 },
    { 0x8e5b015, 95 },
    { 0xc415b017, 23 },
    { 0x25d5b018, 95 },
    { 0x2cc5b019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c35b01c, 50 },
    { 0x2825b01d, 55 },
    { 0x5e05b01e, 33 },
    { 0xbd15b01f, 34 },
    { 0, 0 },
    { 0x995b021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b024, 93 },
    { 0xaac5b024, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945b028, 90 },
    { 0, 0 },
    { 0x28f5b02a, 23 },
    { 0, 0 },
    { 0x5335b02c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e5b033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5b038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed65b03b, 98 },
    { 0xc9d5b03c, 20 },
    { 0x5925b03d, 95 },
    { 0, 0 },
    { 0x6825b03f, 61 },
    { 0x80e5b040, 76 },
    { 0xa375b041, 45 },
    { 0, 0 },
    { 0x75b5b043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d25b047, 91 },
    { 0, 0 },
    { 0xea75b049, 17 },
    { 0, 0 },
    { 0xc935b04b, 87 },
    { 0, 0 },
    { 0x7795b04d, 61 },
    { 0x9555b04e, 61 },
    { 0x1f85b04f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f55b055, 81 },
    { 0x8345b056, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75b05b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff5b05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9095b062, 46 },
    { 0, 0 },
    { 0xbfe5b064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf265b067, 91 },
    { 0x1c55b067, 95 },
    { 0xa975b068, 85 },
    { 0xf3f5b06a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd255b06d, 78 },
    { 0, 0 },
    { 0x9df5b06f, 66 },
    { 0xdcc5b06f, 55 },
    { 0x7195b070, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x585b074, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5b07a, 95 },
    { 0, 0 },
    { 0xa655b07c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5b07f, 66 },
    { 0xd7b5b080, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a5b087, 85 },
    { 0xcd45b088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305b08f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf015b092, 90 },
    { 0x90c5b093, 95 },
    { 0x7e85b094, 33 },
    { 0x9815b095, 95 },
    { 0x9ea5b094, 98 },
    { 0x635b095, 61 },
    { 0x8565b092, 61 },
    { 0xb3b5b099, 90 },
    { 0x3e05b099, 90 },
    { 0, 0 },
    { 0xeb05b09c, 61 },
    { 0xd115b09c, 23 },
    { 0xb2a5b09e, 85 },
    { 0x325b09f, 90 },
    { 0, 0 },
    { 0xab55b0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3215b0a5, 46 },
    { 0x1da5b0a5, 34 },
    { 0xbd25b0a5, 99 },
    { 0x8975b0a8, 93 },
    { 0x67e5b0a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f65b0ac, 23 },
    { 0x1e25b0ad, 90 },
    { 0xf7b5b0ad, 88 },
    { 0x13c5b0ac, 23 },
    { 0, 0 },
    { 0x6a55b0b1, 34 },
    { 0xc085b0b1, 84 },
    { 0x705b0b3, 85 },
    { 0x355b0b3, 90 },
    { 0x4665b0b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed5b0bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa345b0c0, 85 },
    { 0x2d45b0c1, 87 },
    { 0, 0 },
    { 0x4135b0c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265b0c6, 80 },
    { 0x3a45b0c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b5b0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa855b0d2, 98 },
    { 0x3f35b0d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335b0d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c5b0da, 95 },
    { 0x60a5b0db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c5b0de, 98 },
    { 0, 0 },
    { 0x9ec5b0e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd5b0ea, 85 },
    { 0xf45b0ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2645b0ee, 93 },
    { 0x36d5b0ee, 34 },
    { 0x12d5b0ee, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85b0f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5b0fb, 90 },
    { 0, 0 },
    { 0xda45b0fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b104, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc65b10d, 81 },
    { 0x1655b10e, 23 },
    { 0xc285b10e, 90 },
    { 0, 0 },
    { 0xd35b111, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb5b116, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4275b119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4615b11c, 46 },
    { 0xfb35b11d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5b120, 61 },
    { 0x3bc5b121, 98 },
    { 0xf6b5b122, 95 },
    { 0, 0 },
    { 0xf0d5b124, 90 },
    { 0xea05b125, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5b12c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c85b132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5b13b, 61 },
    { 0, 0 },
    { 0xc975b13d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad5b143, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5b148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd415b14b, 98 },
    { 0, 0 },
    { 0xdaa5b14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5b152, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c5b15a, 66 },
    { 0x6a55b15a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985b166, 33 },
    { 0x97f5b166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc235b16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4335b171, 65 },
    { 0xd1b5b171, 99 },
    { 0x6b55b173, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad5b178, 76 },
    { 0x43f5b179, 90 },
    { 0x9f5b179, 23 },
    { 0x52d5b17b, 99 },
    { 0xd045b17c, 90 },
    { 0xb0d5b178, 75 },
    { 0x835b179, 48 },
    { 0, 0 },
    { 0x7685b180, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9525b187, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05b18d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c5b193, 43 },
    { 0xd315b194, 52 },
    { 0, 0 },
    { 0xbc25b196, 77 },
    { 0xc4c5b196, 33 },
    { 0x5ff5b198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7915b19d, 90 },
    { 0, 0 },
    { 0x35f5b19f, 67 },
    { 0xf2c5b1a0, 88 },
    { 0x4265b19f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5b1a4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c5b1a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd5b1ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e55b1b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5b1ba, 95 },
    { 0, 0 },
    { 0x4af5b1bc, 95 },
    { 0x4665b1bd, 17 },
    { 0x8a05b1bc, 33 },
    { 0x5ce5b1bf, 95 },
    { 0xeb35b1c0, 95 },
    { 0xf965b1c1, 76 },
    { 0x8a25b1c1, 98 },
    { 0, 0 },
    { 0xd715b1c4, 55 },
    { 0, 0 },
    { 0x9915b1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb905b1cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2815b1d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5b1d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b1db, 34 },
    { 0x57a5b1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5b1e1, 87 },
    { 0xbdd5b1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8425b1eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7485b1f0, 33 },
    { 0xbaa5b1f1, 93 },
    { 0xe2e5b1f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd545b1f5, 61 },
    { 0, 0 },
    { 0x3805b1f7, 33 },
    { 0xcc45b1f8, 87 },
    { 0xbb75b1f7, 84 },
    { 0, 0 },
    { 0x1555b1fb, 100 },
    { 0, 0 },
    { 0x2195b1fd, 23 },
    { 0xd735b1fe, 61 },
    { 0, 0 },
    { 0x7fd5b200, 90 },
    { 0xf4d5b201, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a5b205, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575b20e, 66 },
    { 0x6735b20f, 61 },
    { 0xa035b210, 34 },
    { 0x2c25b211, 95 },
    { 0, 0 },
    { 0x83e5b213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05b219, 33 },
    { 0xe4a5b21a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625b224, 55 },
    { 0xee25b225, 61 },
    { 0x6b95b226, 87 },
    { 0xd015b226, 61 },
    { 0x8475b226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa95b22b, 56 },
    { 0x10e5b22b, 46 },
    { 0xe9a5b22d, 77 },
    { 0xe485b22b, 17 },
    { 0xcd25b22d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c35b232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd5b239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635b23c, 52 },
    { 0xc3d5b23d, 87 },
    { 0x9c75b23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c65b245, 95 },
    { 0, 0 },
    { 0x99e5b247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b24f, 33 },
    { 0x7285b24f, 90 },
    { 0xa495b250, 55 },
    { 0xb365b252, 55 },
    { 0xb715b250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba5b258, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685b25b, 99 },
    { 0xfad5b25c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd55b266, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e5b275, 90 },
    { 0, 0 },
    { 0xeba5b277, 76 },
    { 0xaea5b278, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe5b27e, 90 },
    { 0x94b5b27e, 34 },
    { 0xead5b27f, 23 },
    { 0x6cb5b281, 61 },
    { 0x2185b282, 52 },
    { 0x9405b283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb5b289, 23 },
    { 0x5925b289, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6985b28d, 90 },
    { 0x6d5b28e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7965b294, 88 },
    { 0x3f25b294, 95 },
    { 0xc595b296, 84 },
    { 0x7b55b295, 61 },
    { 0xf6b5b298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35b29d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5b2a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455b2a5, 23 },
    { 0, 0 },
    { 0x1565b2a7, 98 },
    { 0, 0 },
    { 0xad05b2a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295b2ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e5b2b1, 90 },
    { 0x7ab5b2b2, 61 },
    { 0, 0 },
    { 0xa005b2b4, 61 },
    { 0, 0 },
    { 0xf465b2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975b2ba, 94 },
    { 0xfb85b2bb, 85 },
    { 0xe915b2bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5b2c2, 98 },
    { 0xe575b2c3, 33 },
    { 0x8635b2c4, 100 },
    { 0x7b45b2c2, 95 },
    { 0xeec5b2c2, 61 },
    { 0xb4e5b2c6, 76 },
    { 0x9e35b2c7, 76 },
    { 0xe8f5b2c9, 34 },
    { 0x4245b2c9, 69 },
    { 0x6695b2c9, 90 },
    { 0x4e5b2cc, 100 },
    { 0xdec5b2c5, 90 },
    { 0xe135b2ce, 95 },
    { 0xc65b2cc, 33 },
    { 0x7c65b2d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5b2d5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5b2da, 95 },
    { 0x7d85b2db, 33 },
    { 0, 0 },
    { 0xc365b2dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2705b2e1, 33 },
    { 0x7945b2e1, 23 },
    { 0xb2f5b2e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a5b2e6, 91 },
    { 0xa295b2e6, 23 },
    { 0x4f5b2e6, 100 },
    { 0, 0 },
    { 0xcab5b2ea, 61 },
    { 0x3f45b2eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945b2f0, 46 },
    { 0x5f25b2f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa795b2f4, 68 },
    { 0xa835b2f5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a5b2f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c5b301, 99 },
    { 0xf75b302, 33 },
    { 0, 0 },
    { 0xdf35b304, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa715b307, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b30d, 55 },
    { 0x8c35b30e, 33 },
    { 0x8ff5b30e, 76 },
    { 0x4e15b30e, 61 },
    { 0x6af5b310, 33 },
    { 0xef5b30f, 90 },
    { 0x3725b311, 95 },
    { 0x9ae5b314, 76 },
    { 0x2465b315, 61 },
    { 0x625b316, 80 },
    { 0xaf05b310, 76 },
    { 0x2b25b318, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a5b31b, 46 },
    { 0x7275b31b, 89 },
    { 0, 0 },
    { 0x44a5b31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815b322, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045b325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a55b329, 90 },
    { 0x8cd5b329, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc5b32e, 55 },
    { 0x9f95b32f, 43 },
    { 0xecd5b330, 23 },
    { 0, 0 },
    { 0x9f5b332, 90 },
    { 0xa025b332, 61 },
    { 0x7db5b334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5b338, 33 },
    { 0x9c05b339, 91 },
    { 0xa905b33a, 84 },
    { 0x3e55b33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a5b341, 61 },
    { 0x5b05b342, 90 },
    { 0xbfe5b342, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165b346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d5b34a, 68 },
    { 0xea5b34b, 100 },
    { 0x9385b34a, 98 },
    { 0x4645b34b, 34 },
    { 0xdd35b34e, 95 },
    { 0xc705b34a, 33 },
    { 0x50e5b350, 90 },
    { 0x43a5b351, 34 },
    { 0x20a5b351, 23 },
    { 0xb545b353, 61 },
    { 0x22b5b354, 66 },
    { 0x885b352, 66 },
    { 0x3c85b350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235b35a, 61 },
    { 0xa9c5b35b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65b361, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb615b364, 33 },
    { 0x7035b365, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5995b371, 90 },
    { 0x365b372, 33 },
    { 0, 0 },
    { 0xe3a5b374, 76 },
    { 0xb585b375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5b37b, 23 },
    { 0x4175b37b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed5b380, 95 },
    { 0, 0 },
    { 0x7375b382, 76 },
    { 0, 0 },
    { 0x3e15b384, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b388, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb85b394, 34 },
    { 0x8125b395, 66 },
    { 0, 0 },
    { 0x765b397, 78 },
    { 0xdb85b398, 95 },
    { 0x9fa5b398, 33 },
    { 0xad15b39a, 33 },
    { 0x2df5b398, 76 },
    { 0x5385b398, 90 },
    { 0xccd5b39d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x375b3a0, 90 },
    { 0, 0 },
    { 0x62a5b3a2, 33 },
    { 0xeae5b3a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5b3a8, 98 },
    { 0x5d05b3a9, 23 },
    { 0xdd15b3aa, 93 },
    { 0x9915b3aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d5b3ae, 81 },
    { 0xad55b3af, 43 },
    { 0xa125b3af, 55 },
    { 0x7ef5b3af, 90 },
    { 0xbb35b3b2, 95 },
    { 0xa8e5b3b3, 81 },
    { 0x7ad5b3b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d5b3b8, 90 },
    { 0, 0 },
    { 0x7005b3ba, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5b3bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba5b3c0, 97 },
    { 0x62e5b3c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5b3c4, 23 },
    { 0x7595b3c5, 100 },
    { 0, 0 },
    { 0xb765b3c7, 78 },
    { 0, 0 },
    { 0xd635b3c9, 61 },
    { 0, 0 },
    { 0xd535b3cb, 89 },
    { 0x2835b3cc, 23 },
    { 0xb3e5b3cd, 98 },
    { 0x6ae5b3ce, 34 },
    { 0xbb15b3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a5b3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6125b3de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925b3e1, 17 },
    { 0x81d5b3e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5b3eb, 68 },
    { 0, 0 },
    { 0x1ec5b3ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5b3f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85b3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f85b3fa, 85 },
    { 0x7d25b3fa, 43 },
    { 0x1eb5b3fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc135b400, 33 },
    { 0xa025b401, 61 },
    { 0x84c5b402, 46 },
    { 0, 0 },
    { 0x9bb5b404, 95 },
    { 0xfe15b405, 98 },
    { 0x5a05b406, 78 },
    { 0x1895b405, 52 },
    { 0x7d35b408, 90 },
    { 0xba35b408, 78 },
    { 0x9d25b408, 21 },
    { 0x66f5b405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3825b414, 74 },
    { 0x7d75b415, 55 },
    { 0x905b414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5b419, 85 },
    { 0x94f5b41a, 36 },
    { 0, 0 },
    { 0x93b5b41c, 66 },
    { 0x6125b41c, 90 },
    { 0x6565b41e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b422, 90 },
    { 0x6755b423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae55b426, 78 },
    { 0x7bc5b426, 43 },
    { 0x1bb5b428, 55 },
    { 0x2c95b429, 17 },
    { 0x505b428, 98 },
    { 0, 0 },
    { 0x5335b42c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275b42f, 90 },
    { 0x9535b42f, 33 },
    { 0x47b5b431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba15b435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e25b439, 33 },
    { 0x1465b43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce5b440, 76 },
    { 0x2285b440, 23 },
    { 0xdf55b442, 70 },
    { 0x8e95b443, 43 },
    { 0xe265b444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435b449, 90 },
    { 0xf535b449, 23 },
    { 0x8c05b44b, 68 },
    { 0xc9e5b44b, 67 },
    { 0, 0 },
    { 0xf4c5b44e, 93 },
    { 0, 0 },
    { 0xab95b450, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf565b453, 91 },
    { 0xf445b454, 90 },
    { 0x5125b455, 90 },
    { 0x5055b453, 90 },
    { 0xb665b454, 87 },
    { 0xb405b457, 90 },
    { 0xd635b459, 90 },
    { 0x9345b45a, 23 },
    { 0x83b5b45b, 77 },
    { 0xb155b456, 95 },
    { 0, 0 },
    { 0xe195b45e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb85b463, 61 },
    { 0x3ef5b463, 84 },
    { 0x5e65b465, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b55b46a, 76 },
    { 0, 0 },
    { 0xf865b46c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95b475, 88 },
    { 0x2b05b476, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f5b47b, 61 },
    { 0x9bd5b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b35b47f, 61 },
    { 0x6d35b480, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff35b483, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b5b487, 88 },
    { 0, 0 },
    { 0xbc75b489, 98 },
    { 0x3045b48a, 43 },
    { 0xc4b5b48a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa425b490, 98 },
    { 0x5c95b491, 78 },
    { 0x3745b491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575b496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb955b49c, 43 },
    { 0x19b5b49d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe505b4a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd5b4a8, 98 },
    { 0xbd05b4a9, 55 },
    { 0, 0 },
    { 0x9075b4ab, 67 },
    { 0, 0 },
    { 0xe65b4ad, 48 },
    { 0x87b5b4ae, 33 },
    { 0x88c5b4af, 98 },
    { 0, 0 },
    { 0x2c25b4b1, 87 },
    { 0x7555b4b2, 34 },
    { 0, 0 },
    { 0x93c5b4b4, 43 },
    { 0, 0 },
    { 0x17f5b4b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4095b4b9, 90 },
    { 0x6be5b4b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab5b4be, 90 },
    { 0, 0 },
    { 0x7385b4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3175b4c9, 67 },
    { 0, 0 },
    { 0x7535b4cb, 23 },
    { 0xaa55b4cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc5b4cf, 43 },
    { 0xbab5b4d0, 33 },
    { 0xa525b4d1, 95 },
    { 0xeb15b4d2, 66 },
    { 0xc075b4d3, 90 },
    { 0xe9e5b4d4, 65 },
    { 0xc975b4d2, 84 },
    { 0x70e5b4d6, 33 },
    { 0xee15b4d7, 87 },
    { 0x3ec5b4d7, 34 },
    { 0, 0 },
    { 0xd845b4da, 61 },
    { 0x7555b4db, 34 },
    { 0x7fd5b4dc, 41 },
    { 0x5355b4db, 90 },
    { 0x4455b4dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5955b4e1, 91 },
    { 0x5a45b4e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85b4e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b4eb, 23 },
    { 0, 0 },
    { 0x3415b4ed, 95 },
    { 0x1585b4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f5b4f1, 95 },
    { 0x9b35b4f2, 23 },
    { 0x58b5b4f3, 90 },
    { 0xe5e5b4f3, 76 },
    { 0, 0 },
    { 0x5bb5b4f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f5b4fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca85b505, 33 },
    { 0, 0 },
    { 0x8585b507, 99 },
    { 0, 0 },
    { 0x7195b509, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25b50e, 33 },
    { 0x6905b50f, 84 },
    { 0xde45b50f, 93 },
    { 0x7465b50e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625b51a, 33 },
    { 0, 0 },
    { 0xc8d5b51c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5b52a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665b532, 33 },
    { 0xb95b533, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef55b536, 95 },
    { 0x3035b536, 91 },
    { 0xc65b538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb525b53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e5b546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275b54b, 61 },
    { 0x19f5b54b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5b552, 61 },
    { 0, 0 },
    { 0x4b25b554, 98 },
    { 0x5ef5b555, 93 },
    { 0xd035b555, 61 },
    { 0xb235b557, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a5b55c, 23 },
    { 0x7d05b55d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5b561, 95 },
    { 0, 0 },
    { 0xd3a5b563, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85b56c, 76 },
    { 0x56b5b56d, 90 },
    { 0x49d5b56e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc675b574, 67 },
    { 0, 0 },
    { 0xe175b576, 95 },
    { 0x3335b576, 99 },
    { 0x1c55b578, 98 },
    { 0x6bd5b579, 98 },
    { 0, 0 },
    { 0x64c5b57b, 52 },
    { 0xc395b57b, 61 },
    { 0x12f5b57d, 90 },
    { 0x8675b57e, 95 },
    { 0x92e5b57f, 17 },
    { 0x93d5b580, 90 },
    { 0xf655b57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9775b588, 33 },
    { 0, 0 },
    { 0xc065b58a, 78 },
    { 0, 0 },
    { 0xab15b58c, 72 },
    { 0x9475b58d, 90 },
    { 0, 0 },
    { 0x8b55b58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb105b592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b598, 91 },
    { 0, 0 },
    { 0x8025b59a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc605b59d, 90 },
    { 0x5dc5b59e, 98 },
    { 0x30a5b59e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e5b5a2, 87 },
    { 0, 0 },
    { 0xf495b5a4, 95 },
    { 0xbfd5b5a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc5b5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1015b5ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd95b5b6, 61 },
    { 0x425b5b7, 90 },
    { 0x35b5b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d5b5bc, 23 },
    { 0xb475b5bd, 90 },
    { 0, 0 },
    { 0xbb05b5bf, 98 },
    { 0x4c25b5bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa635b5c6, 46 },
    { 0x5355b5c7, 90 },
    { 0xad85b5c7, 90 },
    { 0xd705b5c9, 93 },
    { 0x93f5b5ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f5b5ce, 52 },
    { 0xbb25b5cf, 98 },
    { 0x1485b5cf, 76 },
    { 0, 0 },
    { 0x9ff5b5d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f5b5d6, 23 },
    { 0xdf15b5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8805b5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a05b5e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5b5e4, 55 },
    { 0x3eb5b5e4, 93 },
    { 0, 0 },
    { 0xbb05b5e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf075b5ec, 33 },
    { 0, 0 },
    { 0x32c5b5ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5b5f4, 66 },
    { 0, 0 },
    { 0xd0d5b5f6, 81 },
    { 0x4c45b5f6, 95 },
    { 0x71a5b5f8, 93 },
    { 0x6d45b5f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45b606, 90 },
    { 0x26c5b607, 23 },
    { 0xf6d5b608, 95 },
    { 0x61d5b606, 33 },
    { 0x5405b607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5b60d, 69 },
    { 0x4c5b60d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f75b612, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa185b617, 90 },
    { 0x8945b617, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa5b61f, 33 },
    { 0, 0 },
    { 0xab5b621, 88 },
    { 0, 0 },
    { 0x2665b623, 51 },
    { 0x4815b624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f05b62b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3535b633, 46 },
    { 0, 0 },
    { 0xadc5b635, 23 },
    { 0x39c5b636, 91 },
    { 0xfcd5b637, 48 },
    { 0, 0 },
    { 0x9025b639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5b643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5b646, 33 },
    { 0, 0 },
    { 0xce55b648, 61 },
    { 0x4735b649, 42 },
    { 0x14e5b64a, 90 },
    { 0x3545b649, 80 },
    { 0x825b64c, 76 },
    { 0x9055b64c, 90 },
    { 0, 0 },
    { 0x49e5b64f, 33 },
    { 0, 0 },
    { 0x10c5b651, 98 },
    { 0xea55b652, 55 },
    { 0xfa85b653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf535b656, 75 },
    { 0x1e55b657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1525b65a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e95b65f, 61 },
    { 0, 0 },
    { 0x5fb5b661, 55 },
    { 0, 0 },
    { 0x67f5b663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85b666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525b66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d5b66e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5b671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b677, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5b67b, 17 },
    { 0x2a95b67b, 61 },
    { 0xde95b67d, 36 },
    { 0, 0 },
    { 0x30c5b67f, 98 },
    { 0x4945b680, 98 },
    { 0xae05b67f, 90 },
    { 0x1715b682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4775b68e, 95 },
    { 0, 0 },
    { 0x2165b690, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545b693, 36 },
    { 0xd9c5b694, 95 },
    { 0x99f5b695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb5b69a, 34 },
    { 0xe7b5b69b, 84 },
    { 0xaeb5b69b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745b6a1, 61 },
    { 0x9d35b6a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315b6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5b6b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305b6b4, 46 },
    { 0, 0 },
    { 0x7da5b6b6, 67 },
    { 0, 0 },
    { 0xb6e5b6b8, 80 },
    { 0xfbe5b6b9, 43 },
    { 0x4365b6b9, 95 },
    { 0x78d5b6bb, 33 },
    { 0xcc95b6b9, 33 },
    { 0, 0 },
    { 0xb815b6be, 90 },
    { 0xc915b6bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d45b6c4, 95 },
    { 0, 0 },
    { 0xae5b6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f5b6cb, 99 },
    { 0x3b65b6cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb525b6d1, 55 },
    { 0x2075b6d2, 61 },
    { 0xbd15b6d3, 33 },
    { 0xc415b6d4, 93 },
    { 0x8585b6d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e25b6dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7905b6e1, 91 },
    { 0xbbe5b6e1, 76 },
    { 0, 0 },
    { 0xbde5b6e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e75b6e8, 33 },
    { 0, 0 },
    { 0x9dc5b6ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b6f1, 95 },
    { 0x8a65b6f2, 33 },
    { 0, 0 },
    { 0x2d75b6f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5b6f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b5b6fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5115b703, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa5b706, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b70b, 66 },
    { 0x7525b70b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x925b70f, 76 },
    { 0, 0 },
    { 0x4495b711, 34 },
    { 0, 0 },
    { 0x8cd5b713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a5b718, 95 },
    { 0xbcd5b719, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde75b71d, 90 },
    { 0x1055b71d, 33 },
    { 0x55b5b71f, 98 },
    { 0xf3f5b71f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd005b724, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1395b728, 90 },
    { 0, 0 },
    { 0xc45b72a, 90 },
    { 0, 0 },
    { 0x72a5b72c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe5b72f, 77 },
    { 0x9ff5b730, 88 },
    { 0x6bf5b730, 90 },
    { 0x6e55b72f, 61 },
    { 0x3385b732, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf95b739, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c05b743, 43 },
    { 0, 0 },
    { 0x28c5b745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25b749, 90 },
    { 0x8e85b74a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5b74d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c5b752, 90 },
    { 0x3ed5b752, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8255b757, 33 },
    { 0, 0 },
    { 0xf905b759, 76 },
    { 0x72c5b75a, 33 },
    { 0x72d5b75b, 33 },
    { 0xe6f5b759, 34 },
    { 0, 0 },
    { 0x57e5b75e, 98 },
    { 0, 0 },
    { 0xc725b760, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3025b764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf825b767, 95 },
    { 0x155b768, 33 },
    { 0xa385b768, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd235b76d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd5b773, 98 },
    { 0x4965b773, 95 },
    { 0, 0 },
    { 0x4175b776, 66 },
    { 0x245b776, 90 },
    { 0x72f5b778, 61 },
    { 0x7585b779, 52 },
    { 0x7895b779, 23 },
    { 0x6cb5b777, 86 },
    { 0x9415b77c, 85 },
    { 0x4935b77d, 98 },
    { 0, 0 },
    { 0xca85b77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925b784, 23 },
    { 0x8675b785, 98 },
    { 0xf365b785, 93 },
    { 0xf975b784, 23 },
    { 0, 0 },
    { 0xe25b789, 67 },
    { 0xe665b78a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e5b78e, 95 },
    { 0, 0 },
    { 0xe005b790, 34 },
    { 0, 0 },
    { 0xf605b792, 90 },
    { 0x7ba5b793, 75 },
    { 0x44f5b794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3045b79b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb95b7a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5b7a7, 46 },
    { 0, 0 },
    { 0x5f95b7a9, 95 },
    { 0x9695b7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd55b7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5b7b1, 90 },
    { 0xde75b7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875b7b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95b7b9, 33 },
    { 0xe485b7ba, 90 },
    { 0, 0 },
    { 0xcc45b7bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc95b7bf, 75 },
    { 0, 0 },
    { 0xcac5b7c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x685b7c4, 100 },
    { 0xe355b7c5, 76 },
    { 0, 0 },
    { 0x2855b7c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc975b7ce, 34 },
    { 0xb8c5b7ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb5b7d3, 78 },
    { 0xf95b7d4, 55 },
    { 0xf7f5b7d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a5b7db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75b7df, 61 },
    { 0xaab5b7e0, 95 },
    { 0xad45b7e1, 78 },
    { 0x3305b7e0, 33 },
    { 0xe2d5b7e3, 98 },
    { 0xddc5b7e0, 95 },
    { 0x2835b7e3, 98 },
    { 0xc6f5b7e0, 90 },
    { 0xb695b7e1, 67 },
    { 0xacd5b7e4, 38 },
    { 0x70f5b7e4, 90 },
    { 0x62e5b7ea, 90 },
    { 0xe3a5b7eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75b7f0, 67 },
    { 0x4a55b7f0, 76 },
    { 0xf725b7f2, 46 },
    { 0x37c5b7f2, 55 },
    { 0xeaa5b7f2, 76 },
    { 0x2925b7f5, 61 },
    { 0xfbe5b7f5, 34 },
    { 0xf655b7f0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a5b7fa, 52 },
    { 0xae15b7fa, 60 },
    { 0, 0 },
    { 0x26b5b7fd, 99 },
    { 0, 0 },
    { 0x3315b7ff, 87 },
    { 0xda15b800, 90 },
    { 0x7d75b801, 100 },
    { 0x5875b800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5b805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5b808, 33 },
    { 0, 0 },
    { 0x2575b80a, 100 },
    { 0x2545b80a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b5b80e, 87 },
    { 0xf335b80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6345b816, 91 },
    { 0, 0 },
    { 0x2d65b818, 88 },
    { 0, 0 },
    { 0xa125b81a, 17 },
    { 0x915b81a, 38 },
    { 0, 0 },
    { 0x3855b81d, 65 },
    { 0, 0 },
    { 0x2ca5b81f, 95 },
    { 0x6de5b820, 23 },
    { 0x635b821, 51 },
    { 0xa995b822, 95 },
    { 0xacb5b822, 90 },
    { 0x7e45b824, 61 },
    { 0xd195b825, 43 },
    { 0x8045b820, 52 },
    { 0xc085b827, 97 },
    { 0, 0 },
    { 0x6855b829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5b82d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f15b831, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955b834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb245b838, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f5b83b, 90 },
    { 0xfd15b83b, 98 },
    { 0x7b65b83d, 61 },
    { 0xec85b83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45b845, 100 },
    { 0x3e45b846, 40 },
    { 0xf275b847, 90 },
    { 0xe935b847, 95 },
    { 0xaf15b849, 55 },
    { 0x1d5b847, 90 },
    { 0, 0 },
    { 0x4ae5b84c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf5b855, 76 },
    { 0x8315b856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85b85b, 93 },
    { 0x5995b85c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a5b85f, 38 },
    { 0xa405b85f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe845b866, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55b869, 55 },
    { 0xdc55b86a, 90 },
    { 0x2935b86a, 98 },
    { 0x9f05b86c, 98 },
    { 0x21e5b86c, 76 },
    { 0xfb65b86c, 84 },
    { 0x7a15b86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe5b874, 78 },
    { 0x3b25b874, 99 },
    { 0, 0 },
    { 0x6dd5b877, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555b87b, 95 },
    { 0x9695b87c, 87 },
    { 0x775b87c, 98 },
    { 0, 0 },
    { 0x6495b87f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda45b883, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a5b887, 76 },
    { 0, 0 },
    { 0x5085b889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5b88c, 43 },
    { 0x4ee5b88d, 76 },
    { 0xfb35b88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4715b891, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905b895, 46 },
    { 0, 0 },
    { 0xa8e5b897, 8 },
    { 0, 0 },
    { 0xc565b899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a5b89d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa85b8a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7205b8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b8af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e15b8b3, 77 },
    { 0xf815b8b3, 76 },
    { 0xef65b8b5, 87 },
    { 0x42e5b8b5, 38 },
    { 0x38a5b8b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba5b8ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de5b8c4, 33 },
    { 0x5be5b8c5, 98 },
    { 0xd455b8c5, 98 },
    { 0, 0 },
    { 0x5925b8c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc5b8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695b8ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe955b8d1, 23 },
    { 0xde85b8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f5b8e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa95b8ed, 61 },
    { 0x6bf5b8ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf795b8fd, 90 },
    { 0xaea5b8fe, 33 },
    { 0x8765b8fe, 96 },
    { 0xd465b8ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde35b905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5b908, 36 },
    { 0, 0 },
    { 0x5455b90a, 90 },
    { 0x8355b90b, 95 },
    { 0xd735b90a, 33 },
    { 0xe0d5b90d, 98 },
    { 0xe675b90e, 64 },
    { 0xcd55b90d, 95 },
    { 0x31f5b910, 43 },
    { 0, 0 },
    { 0xe935b912, 90 },
    { 0, 0 },
    { 0xc6f5b914, 46 },
    { 0xdfd5b915, 33 },
    { 0xcc15b916, 99 },
    { 0, 0 },
    { 0x9b25b918, 33 },
    { 0x6d5b919, 61 },
    { 0x4285b91a, 98 },
    { 0xc85b91a, 95 },
    { 0x4a5b91c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955b91f, 93 },
    { 0xd375b920, 90 },
    { 0x8cd5b921, 95 },
    { 0xbe55b922, 93 },
    { 0xd6c5b920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc615b926, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5b92f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95b932, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf5b935, 61 },
    { 0x3845b936, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc395b939, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf495b93f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d5b947, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25b94a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d5b94d, 88 },
    { 0x4b05b94e, 90 },
    { 0x9f85b94f, 98 },
    { 0x16a5b950, 90 },
    { 0xab15b951, 85 },
    { 0, 0 },
    { 0x2f05b953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8195b956, 90 },
    { 0xdd25b957, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605b95d, 90 },
    { 0, 0 },
    { 0x5fd5b95f, 60 },
    { 0xd105b95f, 76 },
    { 0x8f65b95f, 91 },
    { 0x3d75b961, 90 },
    { 0, 0 },
    { 0x65f5b964, 33 },
    { 0x265b964, 90 },
    { 0x3d95b966, 95 },
    { 0x2ae5b966, 33 },
    { 0x905b968, 33 },
    { 0xf625b964, 76 },
    { 0xdeb5b96a, 61 },
    { 0xa3c5b96b, 100 },
    { 0x7f15b96b, 99 },
    { 0x6d55b969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be5b972, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7675b975, 100 },
    { 0xc315b976, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2135b984, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d5b989, 85 },
    { 0xd2b5b98a, 33 },
    { 0, 0 },
    { 0x5e75b98c, 34 },
    { 0, 0 },
    { 0xa15b98e, 23 },
    { 0, 0 },
    { 0x1e5b990, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb605b998, 23 },
    { 0, 0 },
    { 0x66d5b99a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d5b9a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e35b9a9, 55 },
    { 0, 0 },
    { 0x2925b9ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff5b9b1, 34 },
    { 0, 0 },
    { 0xb2a5b9b3, 66 },
    { 0x31b5b9b4, 23 },
    { 0xc525b9b3, 46 },
    { 0xeda5b9b5, 77 },
    { 0, 0 },
    { 0x33d5b9b8, 33 },
    { 0xe005b9b9, 78 },
    { 0x4f05b9b9, 77 },
    { 0xc105b9b8, 66 },
    { 0, 0 },
    { 0xc685b9bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75b9c2, 34 },
    { 0x3fd5b9c3, 42 },
    { 0xe695b9c3, 63 },
    { 0, 0 },
    { 0xc115b9c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef5b9c9, 61 },
    { 0x22d5b9ca, 98 },
    { 0, 0 },
    { 0x84c5b9cc, 90 },
    { 0xc5b9cd, 52 },
    { 0xe65b9cd, 69 },
    { 0x1f75b9cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45b9d2, 90 },
    { 0, 0 },
    { 0xc235b9d4, 80 },
    { 0x99b5b9d4, 90 },
    { 0, 0 },
    { 0x46d5b9d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb15b9dd, 98 },
    { 0x8255b9dd, 43 },
    { 0x8bf5b9df, 90 },
    { 0x97c5b9e0, 98 },
    { 0, 0 },
    { 0x3145b9e2, 90 },
    { 0, 0 },
    { 0xa525b9e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9865b9e7, 61 },
    { 0xb375b9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b9eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a5b9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7875b9fd, 67 },
    { 0xb8d5b9fe, 23 },
    { 0x6655b9ff, 99 },
    { 0xd095ba00, 61 },
    { 0, 0 },
    { 0x2ac5ba02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d75ba05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf5ba0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf885ba11, 90 },
    { 0, 0 },
    { 0xc145ba13, 65 },
    { 0xada5ba13, 90 },
    { 0x4195ba15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5ba1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2035ba23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec15ba28, 98 },
    { 0, 0 },
    { 0x80b5ba2a, 85 },
    { 0xdc85ba2a, 46 },
    { 0x2505ba2c, 70 },
    { 0, 0 },
    { 0xd1b5ba2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385ba35, 66 },
    { 0, 0 },
    { 0x5675ba37, 98 },
    { 0x8f55ba38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f65ba3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1655ba3f, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a5ba42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865ba45, 98 },
    { 0, 0 },
    { 0xeba5ba47, 84 },
    { 0x5605ba48, 90 },
    { 0xc8e5ba47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e65ba4c, 33 },
    { 0x36c5ba4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015ba50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa945ba59, 67 },
    { 0, 0 },
    { 0xd2a5ba5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5ba63, 90 },
    { 0xd455ba64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a5ba6e, 34 },
    { 0, 0 },
    { 0x2785ba70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6365ba77, 33 },
    { 0xdfd5ba78, 95 },
    { 0xdd25ba79, 61 },
    { 0xabd5ba7a, 90 },
    { 0, 0 },
    { 0x3545ba7c, 93 },
    { 0xb745ba7c, 76 },
    { 0x2d75ba7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5935ba87, 34 },
    { 0xe225ba88, 99 },
    { 0xa895ba87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe105ba8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a55ba98, 58 },
    { 0, 0 },
    { 0x6a5ba9a, 33 },
    { 0x9a05ba9b, 76 },
    { 0, 0 },
    { 0x9935ba9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f5baa0, 52 },
    { 0xf825baa1, 33 },
    { 0x4125baa0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b35baa5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75baab, 34 },
    { 0xadd5baac, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15bab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45bab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905bab8, 61 },
    { 0, 0 },
    { 0x1045baba, 95 },
    { 0x4cc5babb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715bac1, 43 },
    { 0x7dd5bac2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc5bac5, 61 },
    { 0, 0 },
    { 0x2665bac7, 61 },
    { 0xf7b5bac8, 33 },
    { 0, 0 },
    { 0x26c5baca, 33 },
    { 0xb445baca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da5bacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae5bad3, 76 },
    { 0x7f75bad3, 55 },
    { 0xb535bad3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e5bada, 76 },
    { 0x5885badb, 23 },
    { 0, 0 },
    { 0x9f05badd, 43 },
    { 0x2535bade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65bae2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5bae6, 95 },
    { 0x8bc5bae7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35baec, 66 },
    { 0xe65baed, 65 },
    { 0, 0 },
    { 0xec95baef, 78 },
    { 0xb7c5baf0, 90 },
    { 0xb45baf1, 23 },
    { 0x52a5baf2, 34 },
    { 0xcfa5baf3, 64 },
    { 0x3795baf2, 17 },
    { 0xfd65baf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc35baf8, 98 },
    { 0, 0 },
    { 0xa8c5bafa, 33 },
    { 0xfe05bafb, 23 },
    { 0, 0 },
    { 0x1c15bafd, 90 },
    { 0, 0 },
    { 0xcbb5baff, 61 },
    { 0, 0 },
    { 0x3295bb01, 90 },
    { 0x3f35bb02, 76 },
    { 0x8fc5bb03, 78 },
    { 0x7cd5bb04, 23 },
    { 0xfdc5bb03, 87 },
    { 0x1c45bb04, 33 },
    { 0xc475bb07, 76 },
    { 0x4a75bb07, 100 },
    { 0x205bb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65bb0e, 98 },
    { 0xaf75bb0f, 55 },
    { 0, 0 },
    { 0xb25bb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad5bb17, 91 },
    { 0x8685bb18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985bb1d, 91 },
    { 0xfb65bb1e, 33 },
    { 0, 0 },
    { 0x45b5bb20, 17 },
    { 0x2a95bb21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7315bb25, 91 },
    { 0, 0 },
    { 0x2655bb27, 80 },
    { 0xf035bb27, 90 },
    { 0xee05bb28, 98 },
    { 0, 0 },
    { 0xdd55bb2b, 84 },
    { 0x5945bb2c, 76 },
    { 0x6875bb2d, 61 },
    { 0, 0 },
    { 0x6525bb2f, 90 },
    { 0xa955bb2f, 34 },
    { 0xf235bb31, 23 },
    { 0, 0 },
    { 0x29d5bb33, 53 },
    { 0xf2b5bb33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f65bb39, 61 },
    { 0x5805bb3a, 95 },
    { 0, 0 },
    { 0x93b5bb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d5bb41, 43 },
    { 0xf545bb42, 23 },
    { 0xb825bb43, 95 },
    { 0xd15bb42, 90 },
    { 0x9cb5bb45, 90 },
    { 0x5065bb46, 53 },
    { 0x3d45bb46, 87 },
    { 0, 0 },
    { 0x26f5bb49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5bb4d, 90 },
    { 0xc8c5bb4e, 33 },
    { 0xe895bb4f, 90 },
    { 0, 0 },
    { 0xdf75bb51, 99 },
    { 0xffb5bb51, 98 },
    { 0, 0 },
    { 0x1825bb54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c5bb59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae45bb5c, 98 },
    { 0x3185bb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc55bb65, 98 },
    { 0xd335bb66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5bb69, 95 },
    { 0, 0 },
    { 0xf7c5bb6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca5bb70, 67 },
    { 0xb2f5bb70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf5bb76, 34 },
    { 0, 0 },
    { 0x805bb78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645bb7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad5bb81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc95bb85, 65 },
    { 0, 0 },
    { 0x3185bb87, 76 },
    { 0x85d5bb88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85bb8c, 33 },
    { 0x6795bb8d, 67 },
    { 0xb375bb8c, 95 },
    { 0x6605bb8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305bb98, 84 },
    { 0x7d5bb99, 65 },
    { 0x7a95bb9a, 91 },
    { 0, 0 },
    { 0x9055bb9c, 95 },
    { 0x7625bb9d, 95 },
    { 0xafa5bb9e, 23 },
    { 0x7855bb9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265bba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2045bbac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85bbaf, 98 },
    { 0, 0 },
    { 0x1ca5bbb1, 90 },
    { 0x7885bbb1, 90 },
    { 0x58f5bbb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce5bbb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9955bbb9, 81 },
    { 0xdf45bbb9, 88 },
    { 0xa985bbb9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa75bbc0, 76 },
    { 0, 0 },
    { 0x4d95bbc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5bbc5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b5bbcb, 55 },
    { 0x795bbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2345bbcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865bbd4, 23 },
    { 0, 0 },
    { 0xf9f5bbd6, 76 },
    { 0x82d5bbd6, 77 },
    { 0x9db5bbd6, 95 },
    { 0x3655bbd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c5bbdc, 98 },
    { 0x9335bbdc, 99 },
    { 0xcaf5bbdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b5bbe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7845bbee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5bbf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275bbf9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d35bbfe, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255bc0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c05bc17, 33 },
    { 0xfd45bc18, 66 },
    { 0xc55bc17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135bc1e, 61 },
    { 0xab65bc1f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc5bc22, 43 },
    { 0, 0 },
    { 0x135bc24, 33 },
    { 0xcc95bc25, 76 },
    { 0xc865bc26, 98 },
    { 0x6aa5bc25, 87 },
    { 0xe125bc28, 98 },
    { 0xbb55bc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565bc2e, 55 },
    { 0xeef5bc2f, 99 },
    { 0x3715bc30, 90 },
    { 0xa195bc30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5bc34, 99 },
    { 0, 0 },
    { 0x83a5bc36, 33 },
    { 0x7f25bc37, 90 },
    { 0x12d5bc37, 55 },
    { 0xadf5bc36, 55 },
    { 0x2655bc3a, 34 },
    { 0x5bf5bc38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545bc40, 95 },
    { 0x59a5bc41, 90 },
    { 0x6ba5bc42, 98 },
    { 0xed05bc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5725bc4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d5bc4e, 46 },
    { 0, 0 },
    { 0x18a5bc50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf5bc55, 23 },
    { 0x43d5bc55, 95 },
    { 0, 0 },
    { 0xf9a5bc58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5bc5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295bc5e, 33 },
    { 0xb665bc5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc335bc64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55bc6e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea5bc71, 6 },
    { 0, 0 },
    { 0xb215bc73, 34 },
    { 0x9595bc73, 83 },
    { 0x9165bc75, 87 },
    { 0x9ea5bc76, 90 },
    { 0x4c35bc77, 61 },
    { 0xc945bc77, 84 },
    { 0x6b45bc79, 95 },
    { 0xdee5bc79, 33 },
    { 0xd0b5bc7b, 33 },
    { 0xc495bc7c, 33 },
    { 0x2f05bc79, 90 },
    { 0xcb35bc7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc5bc83, 84 },
    { 0x50b5bc84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bc89, 90 },
    { 0xe185bc8a, 87 },
    { 0x2345bc8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa05bc92, 68 },
    { 0x3c75bc93, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b5bc96, 87 },
    { 0xfba5bc97, 67 },
    { 0x2815bc98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105bca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f5bca3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8565bca9, 91 },
    { 0, 0 },
    { 0xe465bcab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a15bcb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305bcbb, 98 },
    { 0x6bd5bcbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa565bcbf, 33 },
    { 0xf065bcc0, 66 },
    { 0x2ca5bcc0, 78 },
    { 0xdfd5bcc1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c35bcc7, 95 },
    { 0xdda5bcc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5bccf, 33 },
    { 0x2285bcd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb275bcd4, 76 },
    { 0x3455bcd5, 100 },
    { 0x6625bcd6, 98 },
    { 0xdc45bcd5, 33 },
    { 0xc4d5bcd5, 23 },
    { 0xdb85bcd7, 90 },
    { 0, 0 },
    { 0x70f5bcdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5bcdf, 69 },
    { 0, 0 },
    { 0x8815bce1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe75bce5, 90 },
    { 0xba05bce5, 76 },
    { 0xe045bce7, 87 },
    { 0xd1a5bce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5bceb, 61 },
    { 0xbda5bcec, 84 },
    { 0x9f05bced, 90 },
    { 0x9285bcee, 85 },
    { 0x64e5bcef, 61 },
    { 0x2165bcf0, 95 },
    { 0x4465bcf1, 98 },
    { 0x7905bcf1, 23 },
    { 0x1d15bcf3, 43 },
    { 0x6f85bcf4, 66 },
    { 0xcf15bcf4, 33 },
    { 0x3c25bcf4, 90 },
    { 0x1685bcf7, 42 },
    { 0x34f5bcf5, 90 },
    { 0, 0 },
    { 0x5d85bcfa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe5bcfd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6205bd04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785bd07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5bd0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e15bd11, 99 },
    { 0xa55bd11, 67 },
    { 0xcfc5bd13, 90 },
    { 0x2f25bd14, 34 },
    { 0x9415bd15, 88 },
    { 0xfc5bd16, 24 },
    { 0xc1a5bd15, 77 },
    { 0xe375bd14, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2785bd1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4525bd24, 33 },
    { 0, 0 },
    { 0x9c45bd26, 97 },
    { 0xfcf5bd27, 95 },
    { 0, 0 },
    { 0x16a5bd29, 52 },
    { 0xd315bd29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e5bd35, 46 },
    { 0xf885bd35, 99 },
    { 0, 0 },
    { 0x5245bd38, 76 },
    { 0x6f65bd38, 46 },
    { 0x5ff5bd3a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe585bd3d, 61 },
    { 0, 0 },
    { 0x94d5bd3f, 95 },
    { 0x7b95bd40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb85bd46, 23 },
    { 0, 0 },
    { 0xc615bd48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb415bd4b, 55 },
    { 0, 0 },
    { 0x29e5bd4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd25bd52, 68 },
    { 0xdab5bd53, 33 },
    { 0x3475bd54, 23 },
    { 0x2395bd55, 90 },
    { 0x3815bd56, 98 },
    { 0xcbc5bd57, 65 },
    { 0xa005bd58, 90 },
    { 0x78f5bd59, 95 },
    { 0, 0 },
    { 0x9df5bd5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455bd60, 75 },
    { 0, 0 },
    { 0xa7e5bd62, 78 },
    { 0, 0 },
    { 0x7105bd64, 98 },
    { 0, 0 },
    { 0x785bd66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485bd6b, 90 },
    { 0xc965bd6c, 90 },
    { 0x8025bd6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855bd72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5bd76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee75bd7c, 76 },
    { 0x5075bd7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5bd81, 34 },
    { 0, 0 },
    { 0xffe5bd83, 90 },
    { 0x7155bd84, 52 },
    { 0xa2c5bd85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15bd8f, 55 },
    { 0xf465bd90, 33 },
    { 0xcf25bd91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45bd97, 61 },
    { 0x4215bd97, 33 },
    { 0, 0 },
    { 0xe3c5bd9a, 93 },
    { 0, 0 },
    { 0xde35bd9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c5bd9f, 33 },
    { 0x5955bd9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1665bda7, 23 },
    { 0, 0 },
    { 0x6a55bda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa915bdb1, 34 },
    { 0, 0 },
    { 0xb9a5bdb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe55bdbe, 76 },
    { 0x4f85bdbe, 90 },
    { 0x1eb5bdc0, 43 },
    { 0x50d5bdc1, 77 },
    { 0, 0 },
    { 0xbb45bdc3, 95 },
    { 0x9975bdc4, 55 },
    { 0xadd5bdc5, 52 },
    { 0x8325bdc5, 66 },
    { 0xc355bdc7, 99 },
    { 0xe5d5bdc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf185bdcb, 90 },
    { 0x47a5bdcb, 98 },
    { 0xbca5bdcd, 55 },
    { 0x7de5bdcb, 33 },
    { 0x4f45bdce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515bdd4, 55 },
    { 0x2055bdd5, 61 },
    { 0, 0 },
    { 0xae25bdd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee95bddb, 77 },
    { 0x38e5bddc, 33 },
    { 0x1f65bddb, 34 },
    { 0, 0 },
    { 0xae25bddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab5bde2, 33 },
    { 0, 0 },
    { 0xac75bde4, 65 },
    { 0x8505bde5, 33 },
    { 0xe295bde5, 75 },
    { 0xb8c5bde5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe175bdeb, 61 },
    { 0, 0 },
    { 0x7b45bded, 77 },
    { 0x4015bded, 46 },
    { 0, 0 },
    { 0x6905bdf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145bdf4, 84 },
    { 0, 0 },
    { 0xcc15bdf6, 33 },
    { 0x53b5bdf7, 76 },
    { 0x745bdf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c5bdfb, 33 },
    { 0x5335bdfc, 91 },
    { 0, 0 },
    { 0xa685bdfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb445be02, 77 },
    { 0xb4c5be02, 33 },
    { 0xfcc5be04, 91 },
    { 0x4fb5be03, 66 },
    { 0xd3d5be04, 95 },
    { 0x73c5be07, 98 },
    { 0x6115be05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45be0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315be10, 33 },
    { 0, 0 },
    { 0xe055be12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc25be15, 61 },
    { 0x6985be15, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa5be1e, 33 },
    { 0, 0 },
    { 0xb205be20, 95 },
    { 0, 0 },
    { 0xbe85be22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5be25, 33 },
    { 0xce35be25, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5be2a, 52 },
    { 0xe8b5be2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7165be2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05be38, 33 },
    { 0xda85be39, 95 },
    { 0xaa35be3a, 61 },
    { 0x63c5be3a, 84 },
    { 0x1725be38, 33 },
    { 0, 0 },
    { 0xf1f5be3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885be41, 98 },
    { 0, 0 },
    { 0x8055be43, 87 },
    { 0, 0 },
    { 0xa9d5be45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55be48, 46 },
    { 0, 0 },
    { 0xd025be4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5be51, 61 },
    { 0x50f5be52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5be57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb5be5a, 46 },
    { 0xd8a5be5a, 100 },
    { 0xccf5be5c, 95 },
    { 0x75be5d, 34 },
    { 0x4bf5be5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce85be65, 76 },
    { 0xd1f5be65, 43 },
    { 0, 0 },
    { 0xd5c5be68, 34 },
    { 0, 0 },
    { 0x5cd5be6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7955be6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5be71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae5be79, 23 },
    { 0x9a75be79, 43 },
    { 0x7b65be7b, 23 },
    { 0xa225be7c, 95 },
    { 0xf005be7d, 95 },
    { 0xb475be7e, 23 },
    { 0x9795be7f, 55 },
    { 0xbe45be7d, 23 },
    { 0xd2e5be79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac85be84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed5be8a, 23 },
    { 0x7de5be8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd55be90, 93 },
    { 0x4255be91, 43 },
    { 0xfc05be92, 93 },
    { 0x7ee5be90, 100 },
    { 0x365be94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765be9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3035bea1, 46 },
    { 0x7c25bea2, 100 },
    { 0x2795bea2, 90 },
    { 0xeff5bea4, 100 },
    { 0x9c05bea5, 98 },
    { 0, 0 },
    { 0xadb5bea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb45bead, 66 },
    { 0x38f5bead, 87 },
    { 0x2c45beaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca5beb3, 98 },
    { 0, 0 },
    { 0x3f25beb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5bec2, 81 },
    { 0, 0 },
    { 0x9ef5bec4, 33 },
    { 0x1195bec5, 97 },
    { 0x2125bec5, 98 },
    { 0, 0 },
    { 0xb245bec8, 69 },
    { 0, 0 },
    { 0x7735beca, 61 },
    { 0x6905beca, 90 },
    { 0, 0 },
    { 0xaad5becd, 91 },
    { 0x6e15becd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb645bed3, 23 },
    { 0, 0 },
    { 0xd345bed5, 23 },
    { 0, 0 },
    { 0xbcb5bed7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc5bede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a5bee1, 100 },
    { 0xb895bee2, 84 },
    { 0x2515bee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4275bee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3985beec, 88 },
    { 0xaad5beec, 90 },
    { 0x9995beee, 99 },
    { 0, 0 },
    { 0x26a5bef0, 66 },
    { 0xecd5bef0, 90 },
    { 0xfe5bef2, 23 },
    { 0, 0 },
    { 0x9b45bef4, 90 },
    { 0x2d75bef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc5bef9, 33 },
    { 0x2c85befa, 80 },
    { 0, 0 },
    { 0xdaf5befc, 93 },
    { 0x9bc5befd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c5bf02, 34 },
    { 0x1ff5bf03, 98 },
    { 0, 0 },
    { 0xd515bf05, 33 },
    { 0, 0 },
    { 0xc485bf07, 49 },
    { 0x3ce5bf08, 43 },
    { 0x9f5bf07, 58 },
    { 0, 0 },
    { 0x2d35bf0b, 33 },
    { 0, 0 },
    { 0xd695bf0d, 90 },
    { 0xf035bf0d, 98 },
    { 0, 0 },
    { 0x5d85bf10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf5bf13, 85 },
    { 0x1d45bf14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bf1a, 34 },
    { 0x84d5bf1b, 33 },
    { 0x2595bf1c, 61 },
    { 0x5765bf1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e5bf24, 84 },
    { 0, 0 },
    { 0x23c5bf26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd5bf29, 23 },
    { 0xe525bf2a, 98 },
    { 0x9495bf2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e55bf2f, 84 },
    { 0, 0 },
    { 0xcd65bf31, 23 },
    { 0, 0 },
    { 0x1955bf33, 66 },
    { 0x1b55bf34, 98 },
    { 0x9805bf35, 33 },
    { 0x3505bf36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa5bf39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5bf3e, 93 },
    { 0x8e95bf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd5bf42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9425bf46, 78 },
    { 0xcbb5bf46, 61 },
    { 0, 0 },
    { 0xe4d5bf49, 95 },
    { 0, 0 },
    { 0x1ad5bf4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f5bf4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3635bf53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6385bf56, 90 },
    { 0x4fd5bf57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815bf5a, 23 },
    { 0x5bf5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6735bf5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745bf64, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bf69, 33 },
    { 0xb795bf6a, 55 },
    { 0x8aa5bf6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a55bf6e, 95 },
    { 0, 0 },
    { 0xd335bf70, 90 },
    { 0xf3c5bf71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4555bf74, 99 },
    { 0xcd5bf75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895bf79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5bf80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8435bf87, 91 },
    { 0x6b65bf87, 55 },
    { 0, 0 },
    { 0x3da5bf8a, 33 },
    { 0, 0 },
    { 0x2855bf8c, 76 },
    { 0xde85bf8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9085bf94, 23 },
    { 0x7b5bf94, 55 },
    { 0x2965bf95, 98 },
    { 0xc6b5bf97, 61 },
    { 0x2cb5bf98, 76 },
    { 0x2145bf97, 95 },
    { 0x1c75bf9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095bf9d, 95 },
    { 0, 0 },
    { 0xa3e5bf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb315bfa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5bfa7, 43 },
    { 0, 0 },
    { 0xffb5bfa9, 76 },
    { 0, 0 },
    { 0x6fd5bfab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275bfb0, 23 },
    { 0x8985bfb0, 75 },
    { 0, 0 },
    { 0xd945bfb3, 61 },
    { 0xd715bfb3, 23 },
    { 0x4415bfb5, 91 },
    { 0, 0 },
    { 0x4bc5bfb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1875bfba, 55 },
    { 0xef65bfbb, 23 },
    { 0x8905bfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb5bfc1, 98 },
    { 0xea45bfc2, 93 },
    { 0x70d5bfc2, 23 },
    { 0, 0 },
    { 0x2445bfc5, 98 },
    { 0xa0e5bfc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5bfc9, 95 },
    { 0x7485bfc9, 90 },
    { 0xa545bfcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e5bfce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5bfd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805bfd7, 89 },
    { 0x79c5bfd7, 90 },
    { 0, 0 },
    { 0xccb5bfda, 90 },
    { 0, 0 },
    { 0xc8c5bfdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5bfe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda5bfe3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd705bfea, 23 },
    { 0x8c75bfea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3445bfee, 95 },
    { 0x5305bfee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b5bff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bff9, 61 },
    { 0xbaa5bffa, 90 },
    { 0, 0 },
    { 0x21e5bffc, 76 },
    { 0, 0 },
    { 0x1645bffe, 93 },
    { 0, 0 },
    { 0xc385c000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305c004, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4925c010, 95 },
    { 0, 0 },
    { 0x6805c012, 98 },
    { 0, 0 },
    { 0x8bf5c014, 61 },
    { 0x9025c015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb975c01e, 96 },
    { 0, 0 },
    { 0xd795c020, 23 },
    { 0xa275c021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25c024, 33 },
    { 0, 0 },
    { 0x2425c026, 99 },
    { 0xd965c027, 34 },
    { 0, 0 },
    { 0x4955c029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5c02c, 55 },
    { 0x93c5c02d, 90 },
    { 0x67e5c02e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x905c031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd5c03a, 83 },
    { 0x94d5c03a, 66 },
    { 0, 0 },
    { 0xb975c03d, 33 },
    { 0xc245c03d, 95 },
    { 0, 0 },
    { 0x41f5c040, 93 },
    { 0x6d15c041, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d45c046, 23 },
    { 0, 0 },
    { 0x47c5c048, 91 },
    { 0x76d5c049, 93 },
    { 0, 0 },
    { 0x1d55c04b, 95 },
    { 0xa2a5c04b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f5c051, 95 },
    { 0x8875c051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb35c05a, 98 },
    { 0, 0 },
    { 0x6895c05c, 33 },
    { 0x6d05c05c, 75 },
    { 0x8955c05c, 90 },
    { 0xf785c05d, 91 },
    { 0xe8a5c060, 66 },
    { 0x5f85c060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55c064, 23 },
    { 0x3d55c064, 98 },
    { 0xbba5c065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2235c06c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e5c06f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe085c072, 76 },
    { 0, 0 },
    { 0x38d5c074, 90 },
    { 0x5ea5c075, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc865c078, 76 },
    { 0xc3b5c079, 46 },
    { 0xe8c5c07a, 34 },
    { 0xa8d5c07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4025c07e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055c083, 85 },
    { 0xec25c084, 33 },
    { 0x47c5c085, 95 },
    { 0xc3f5c086, 95 },
    { 0x3f55c083, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf55c08c, 23 },
    { 0xfb55c08d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d5c094, 34 },
    { 0x60b5c094, 67 },
    { 0xbe35c096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc55c099, 23 },
    { 0x72c5c09a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec85c0a0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1165c0a4, 23 },
    { 0x2275c0a4, 23 },
    { 0x8665c0a4, 76 },
    { 0, 0 },
    { 0x6a25c0a8, 52 },
    { 0, 0 },
    { 0x3c25c0aa, 85 },
    { 0x9805c0ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e5c0af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a5c0b2, 76 },
    { 0x7d5c0b3, 43 },
    { 0, 0 },
    { 0xc075c0b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b5c0b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c95c0bd, 85 },
    { 0x78b5c0be, 98 },
    { 0xe425c0bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755c0c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55c0d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645c0d4, 61 },
    { 0, 0 },
    { 0xd4d5c0d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e55c0d9, 46 },
    { 0, 0 },
    { 0xa205c0db, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d5c0df, 90 },
    { 0, 0 },
    { 0xb175c0e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795c0e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95c0ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c85c0ef, 90 },
    { 0xae15c0f0, 34 },
    { 0x8b5c0f0, 33 },
    { 0, 0 },
    { 0x7be5c0f3, 98 },
    { 0xcc95c0f4, 93 },
    { 0x4485c0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c5c0f8, 90 },
    { 0x9c25c0f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc885c0ff, 90 },
    { 0xdb95c100, 100 },
    { 0, 0 },
    { 0x36a5c102, 55 },
    { 0x36d5c102, 95 },
    { 0xb715c103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf5c107, 61 },
    { 0xdf45c108, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c5c111, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7415c116, 46 },
    { 0xac5c117, 93 },
    { 0, 0 },
    { 0xcdd5c119, 61 },
    { 0x6575c119, 23 },
    { 0, 0 },
    { 0x3435c11c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae5c11f, 95 },
    { 0xbac5c11f, 46 },
    { 0x4f25c11f, 95 },
    { 0xdc95c122, 33 },
    { 0x3e45c123, 23 },
    { 0, 0 },
    { 0x3455c125, 87 },
    { 0xf095c125, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d35c12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe655c12f, 100 },
    { 0xec55c130, 98 },
    { 0x54a5c130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd245c134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d5c137, 85 },
    { 0x9755c138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x365c13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5c13e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75c142, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f5c14d, 66 },
    { 0x30d5c14e, 76 },
    { 0, 0 },
    { 0x5b65c150, 34 },
    { 0xcd85c150, 76 },
    { 0, 0 },
    { 0x6da5c153, 95 },
    { 0xd765c154, 52 },
    { 0x8715c154, 34 },
    { 0xfa25c153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd5c15b, 90 },
    { 0xd225c15c, 90 },
    { 0xcda5c15d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a5c160, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535c165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8795c169, 95 },
    { 0xc035c169, 17 },
    { 0x81f5c16b, 78 },
    { 0x7615c16b, 46 },
    { 0x84e5c16d, 69 },
    { 0, 0 },
    { 0xf125c16f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795c172, 90 },
    { 0x39f5c173, 90 },
    { 0, 0 },
    { 0x5875c175, 98 },
    { 0xc7f5c176, 52 },
    { 0xf0d5c177, 33 },
    { 0x8ca5c178, 90 },
    { 0x5e05c178, 98 },
    { 0x5f5c17a, 40 },
    { 0x1ac5c17b, 78 },
    { 0xe4d5c176, 33 },
    { 0x2305c17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415c180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115c183, 93 },
    { 0xb135c184, 33 },
    { 0xf4e5c185, 55 },
    { 0xe325c185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f5c189, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d5c18d, 90 },
    { 0, 0 },
    { 0x8375c18f, 95 },
    { 0xa015c190, 77 },
    { 0, 0 },
    { 0x2a95c192, 77 },
    { 0xb3c5c193, 25 },
    { 0x395c194, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215c197, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5495c19b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d5c19f, 90 },
    { 0, 0 },
    { 0x91d5c1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb95c1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d05c1a8, 76 },
    { 0x1195c1a9, 95 },
    { 0, 0 },
    { 0xa535c1ab, 76 },
    { 0, 0 },
    { 0x10a5c1ad, 45 },
    { 0xc65c1ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3055c1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5c1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc555c1b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d85c1bf, 61 },
    { 0x6cb5c1bf, 61 },
    { 0x64d5c1c1, 98 },
    { 0x73b5c1c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2795c1c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e75c1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f5c1d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad05c1da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc345c1df, 46 },
    { 0x48e5c1df, 66 },
    { 0x2775c1e0, 96 },
    { 0x61c5c1e2, 84 },
    { 0x4975c1e3, 90 },
    { 0, 0 },
    { 0x9c45c1e5, 23 },
    { 0x1a55c1e6, 98 },
    { 0xaa05c1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e05c1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb5c1ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1f2, 34 },
    { 0, 0 },
    { 0xcf45c1f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d45c1f7, 43 },
    { 0, 0 },
    { 0xfa55c1f9, 90 },
    { 0xdd5c1fa, 90 },
    { 0x2f15c1fb, 90 },
    { 0x91a5c1fc, 76 },
    { 0x97d5c1fd, 98 },
    { 0x1af5c1fc, 61 },
    { 0, 0 },
    { 0x3f55c200, 34 },
    { 0xda5c201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff05c206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9635c20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5385c20e, 98 },
    { 0, 0 },
    { 0x41c5c210, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5c216, 84 },
    { 0, 0 },
    { 0x44b5c218, 100 },
    { 0, 0 },
    { 0xf215c21a, 98 },
    { 0x9615c21b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb5c220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e5c225, 95 },
    { 0, 0 },
    { 0xbc95c227, 61 },
    { 0x6bd5c228, 55 },
    { 0x3d45c227, 98 },
    { 0x6e75c22a, 61 },
    { 0x80a5c229, 55 },
    { 0x1ee5c22c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a5c230, 33 },
    { 0x29a5c231, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe395c235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe215c239, 95 },
    { 0xa185c239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a5c23d, 87 },
    { 0, 0 },
    { 0xd15c23f, 61 },
    { 0x4945c240, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f5c243, 91 },
    { 0, 0 },
    { 0x1935c245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8275c249, 33 },
    { 0, 0 },
    { 0x3da5c24b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c5c24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad95c251, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85c258, 53 },
    { 0x7235c259, 90 },
    { 0, 0 },
    { 0xf135c25b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8555c25f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95c265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755c273, 98 },
    { 0, 0 },
    { 0x66e5c275, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5c27c, 78 },
    { 0x2a95c27d, 90 },
    { 0xfd65c27d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd15c286, 99 },
    { 0x2475c286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5c28a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c28f, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae5c295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e5c29b, 66 },
    { 0, 0 },
    { 0x2945c29d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265c2a0, 100 },
    { 0, 0 },
    { 0x8095c2a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc85c2a7, 90 },
    { 0x8b05c2a8, 34 },
    { 0x6ea5c2a8, 100 },
    { 0x21d5c2aa, 61 },
    { 0x4585c2ab, 52 },
    { 0xd6a5c2aa, 61 },
    { 0x53d5c2a7, 67 },
    { 0x8935c2a8, 76 },
    { 0xc505c2af, 98 },
    { 0xe025c2b0, 93 },
    { 0xe7a5c2b0, 61 },
    { 0x76d5c2ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65c2b5, 39 },
    { 0xd1b5c2b6, 23 },
    { 0x4b85c2b7, 61 },
    { 0x31e5c2b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4355c2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9185c2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf5c2c4, 76 },
    { 0x305c2c4, 84 },
    { 0, 0 },
    { 0x565c2c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5c2ca, 95 },
    { 0x785c2ca, 23 },
    { 0x3fe5c2cb, 91 },
    { 0xaf35c2cd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f5c2d0, 98 },
    { 0x3d65c2d1, 46 },
    { 0xf255c2d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25c2d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5c2dc, 40 },
    { 0, 0 },
    { 0x2f05c2de, 87 },
    { 0x4635c2df, 61 },
    { 0, 0 },
    { 0x51d5c2e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1995c2e4, 100 },
    { 0x34c5c2e5, 61 },
    { 0, 0 },
    { 0x8145c2e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9755c2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15c2f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995c2f5, 95 },
    { 0xfa5c2f6, 98 },
    { 0, 0 },
    { 0x1345c2f8, 87 },
    { 0, 0 },
    { 0x92a5c2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5c2fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa715c301, 34 },
    { 0, 0 },
    { 0x6625c303, 76 },
    { 0xe5d5c304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5c307, 61 },
    { 0xafd5c308, 23 },
    { 0x1015c307, 98 },
    { 0x6475c30a, 33 },
    { 0xc8c5c30a, 33 },
    { 0x3fa5c309, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2675c312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c85c317, 90 },
    { 0xd05c317, 95 },
    { 0x37f5c319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c31e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85c325, 76 },
    { 0xc955c326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65c32a, 61 },
    { 0x30f5c32a, 61 },
    { 0, 0 },
    { 0x8325c32d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f5c330, 95 },
    { 0x8315c331, 52 },
    { 0xdf55c332, 69 },
    { 0xb175c333, 98 },
    { 0xb875c334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4895c338, 55 },
    { 0xfd45c338, 74 },
    { 0xe3c5c339, 61 },
    { 0, 0 },
    { 0x6165c33c, 90 },
    { 0x16a5c33d, 33 },
    { 0xc385c33e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5c343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c348, 33 },
    { 0xa7f5c349, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e5c34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1105c351, 33 },
    { 0xeef5c351, 51 },
    { 0x3185c353, 98 },
    { 0, 0 },
    { 0xf475c355, 100 },
    { 0x5595c356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb985c35a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc25c35e, 46 },
    { 0, 0 },
    { 0x80b5c360, 61 },
    { 0, 0 },
    { 0x955c362, 98 },
    { 0x5495c363, 33 },
    { 0, 0 },
    { 0x7b55c365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435c368, 100 },
    { 0x4e45c369, 33 },
    { 0x8905c36a, 66 },
    { 0xdcb5c369, 85 },
    { 0x5865c36c, 98 },
    { 0, 0 },
    { 0xd805c36e, 61 },
    { 0xa9e5c36e, 90 },
    { 0, 0 },
    { 0xa985c371, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f5c374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d25c378, 98 },
    { 0, 0 },
    { 0x2145c37a, 40 },
    { 0x84b5c37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1635c37e, 95 },
    { 0x5215c37e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b25c384, 23 },
    { 0xb575c385, 52 },
    { 0x41c5c386, 88 },
    { 0x9655c387, 33 },
    { 0xefa5c388, 34 },
    { 0x93e5c388, 90 },
    { 0xd3e5c38a, 61 },
    { 0x3ae5c38b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a5c38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f65c391, 33 },
    { 0x2e95c391, 23 },
    { 0xc885c393, 40 },
    { 0xbd25c393, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5c397, 61 },
    { 0x8ae5c397, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9865c3a0, 33 },
    { 0x3ee5c3a1, 76 },
    { 0x8ab5c3a0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a5c3a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8445c3a8, 58 },
    { 0xd7e5c3a9, 61 },
    { 0x5765c3aa, 61 },
    { 0x4795c3aa, 23 },
    { 0x8925c3ac, 65 },
    { 0x2615c3ad, 96 },
    { 0xc425c3ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7555c3b2, 98 },
    { 0xa825c3b3, 95 },
    { 0x81e5c3b4, 87 },
    { 0xb065c3b5, 76 },
    { 0x6e35c3b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd785c3bc, 55 },
    { 0, 0 },
    { 0x52e5c3be, 95 },
    { 0, 0 },
    { 0xbf5c3c0, 80 },
    { 0xcfa5c3c1, 95 },
    { 0, 0 },
    { 0xd555c3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225c3c6, 33 },
    { 0xf565c3c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5c3d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d35c3d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635c3dc, 90 },
    { 0, 0 },
    { 0xbb05c3de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5c3e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec15c3e7, 90 },
    { 0xec5c3e7, 95 },
    { 0, 0 },
    { 0x475c3ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc445c3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635c3f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb065c3f6, 23 },
    { 0xc565c3f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525c3fb, 61 },
    { 0, 0 },
    { 0x32b5c3fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5035c404, 66 },
    { 0, 0 },
    { 0xe8f5c406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5c40c, 23 },
    { 0x2705c40d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5c413, 33 },
    { 0, 0 },
    { 0x245c415, 52 },
    { 0xe765c416, 101 },
    { 0, 0 },
    { 0xa5c418, 100 },
    { 0xb415c418, 43 },
    { 0x3965c418, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6665c422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5755c425, 66 },
    { 0x9905c426, 95 },
    { 0x8865c427, 33 },
    { 0xa405c427, 46 },
    { 0, 0 },
    { 0xe775c42a, 85 },
    { 0x745c42a, 64 },
    { 0xbab5c42c, 90 },
    { 0xc555c42d, 90 },
    { 0, 0 },
    { 0xc435c42f, 100 },
    { 0x7145c430, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd435c433, 61 },
    { 0, 0 },
    { 0x8615c435, 90 },
    { 0, 0 },
    { 0x5385c437, 90 },
    { 0, 0 },
    { 0x8ce5c439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335c444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a5c44a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5c44f, 55 },
    { 0x73c5c450, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625c454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b5c45a, 76 },
    { 0, 0 },
    { 0xed85c45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c460, 76 },
    { 0x3775c461, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9595c467, 91 },
    { 0, 0 },
    { 0x5be5c469, 23 },
    { 0xc3e5c46a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f95c473, 34 },
    { 0, 0 },
    { 0xdd95c475, 90 },
    { 0, 0 },
    { 0x7bd5c477, 23 },
    { 0, 0 },
    { 0x3125c479, 46 },
    { 0, 0 },
    { 0x2465c47b, 33 },
    { 0xe1c5c47c, 33 },
    { 0x3f05c47d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5c480, 76 },
    { 0x1f45c480, 90 },
    { 0, 0 },
    { 0xd5a5c483, 85 },
    { 0xc55c483, 76 },
    { 0x8125c483, 46 },
    { 0x7ed5c484, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb25c48c, 70 },
    { 0xfe25c48d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b15c495, 33 },
    { 0, 0 },
    { 0xc265c497, 67 },
    { 0x8125c498, 45 },
    { 0xab35c499, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1935c49f, 76 },
    { 0x6075c4a0, 90 },
    { 0x4085c4a1, 33 },
    { 0x75c5c4a2, 46 },
    { 0, 0 },
    { 0xdd55c4a4, 33 },
    { 0xf2d5c4a5, 75 },
    { 0, 0 },
    { 0xc95c4a7, 33 },
    { 0x695c4a8, 23 },
    { 0, 0 },
    { 0xb775c4aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb5c4ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa445c4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525c4b4, 93 },
    { 0xe0c5c4b5, 95 },
    { 0x6fa5c4b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9245c4b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515c4c3, 98 },
    { 0x2875c4c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa255c4c8, 33 },
    { 0, 0 },
    { 0xafc5c4ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f5c4cf, 36 },
    { 0, 0 },
    { 0xfbb5c4d1, 43 },
    { 0x9d95c4d1, 82 },
    { 0xa165c4d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4945c4d6, 95 },
    { 0x9735c4d6, 76 },
    { 0xd835c4d8, 56 },
    { 0, 0 },
    { 0x6895c4da, 46 },
    { 0, 0 },
    { 0xd605c4dc, 33 },
    { 0x3825c4dd, 90 },
    { 0x2e5c4de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d5c4e2, 23 },
    { 0x4b85c4e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7375c4ea, 90 },
    { 0x4e05c4eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df5c4ee, 95 },
    { 0xe015c4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c5c4f2, 52 },
    { 0x2d25c4f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e85c4f6, 33 },
    { 0xda45c4f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd5c4ff, 87 },
    { 0, 0 },
    { 0xfa05c501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d5c504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85c509, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35c50c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8285c514, 84 },
    { 0x2745c514, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda5c51a, 46 },
    { 0x4085c51b, 33 },
    { 0xbf35c51c, 76 },
    { 0x41b5c51a, 95 },
    { 0x46f5c51e, 33 },
    { 0x4745c51f, 90 },
    { 0x70e5c520, 90 },
    { 0x93a5c521, 80 },
    { 0, 0 },
    { 0x9b5c523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd865c526, 66 },
    { 0xba75c527, 88 },
    { 0x1745c527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1445c52e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c5c531, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca5c534, 23 },
    { 0xf195c535, 95 },
    { 0x96a5c536, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce5c53d, 93 },
    { 0x6245c53e, 95 },
    { 0xa915c53e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d5c543, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5c548, 97 },
    { 0xf05c549, 90 },
    { 0x2ea5c54a, 33 },
    { 0x695c549, 95 },
    { 0, 0 },
    { 0x5a15c54d, 33 },
    { 0xd9e5c54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a5c551, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3295c557, 33 },
    { 0xd8b5c557, 95 },
    { 0x22d5c558, 95 },
    { 0, 0 },
    { 0x665c55b, 90 },
    { 0xadc5c55c, 43 },
    { 0xff65c55c, 33 },
    { 0x4525c55c, 59 },
    { 0x9135c55c, 95 },
    { 0xba5c560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c5c568, 90 },
    { 0, 0 },
    { 0xc585c56a, 23 },
    { 0xe0e5c56b, 98 },
    { 0, 0 },
    { 0xca95c56d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea95c577, 33 },
    { 0x305c578, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f45c585, 55 },
    { 0x9225c586, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c5c58a, 98 },
    { 0xa2e5c58a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5c591, 55 },
    { 0x8e35c592, 90 },
    { 0, 0 },
    { 0x415c594, 90 },
    { 0, 0 },
    { 0x6a45c596, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8635c599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf5c59d, 87 },
    { 0x4d15c59e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5c5a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605c5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6825c5b2, 93 },
    { 0x8215c5b3, 90 },
    { 0xe9c5c5b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef5c5b7, 33 },
    { 0xcd65c5b7, 23 },
    { 0x4905c5b9, 66 },
    { 0x9115c5b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f05c5bd, 85 },
    { 0x90d5c5be, 43 },
    { 0x6085c5bf, 95 },
    { 0xcca5c5c0, 100 },
    { 0x69b5c5c1, 98 },
    { 0xb7c5c5c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5c5c5, 76 },
    { 0xc2f5c5c6, 98 },
    { 0, 0 },
    { 0xe9e5c5c8, 84 },
    { 0, 0 },
    { 0xe915c5ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea15c5d3, 100 },
    { 0x5a65c5d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd55c5d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b5c5dd, 85 },
    { 0x1aa5c5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5c5e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575c5e7, 90 },
    { 0x9115c5e7, 36 },
    { 0, 0 },
    { 0x2e75c5ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4805c5ed, 84 },
    { 0x7ed5c5ed, 34 },
    { 0, 0 },
    { 0x84f5c5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5c5f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c5fb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xca55c5fe, 90 },
    { 0x15e5c5ff, 55 },
    { 0x8575c5fe, 34 },
    { 0x27b5c601, 61 },
    { 0xfb15c602, 23 },
    { 0xf7e5c602, 66 },
    { 0xf975c604, 33 },
    { 0, 0 },
    { 0x8625c606, 66 },
    { 0x7345c607, 61 },
    { 0xcbb5c607, 98 },
    { 0x3965c609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5c60e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa955c611, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd935c614, 61 },
    { 0x8f55c615, 61 },
    { 0x31e5c615, 90 },
    { 0, 0 },
    { 0x97e5c618, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435c61b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe165c61e, 61 },
    { 0x44b5c61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf085c622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df5c625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635c629, 33 },
    { 0, 0 },
    { 0x14a5c62b, 61 },
    { 0xe8e5c62c, 61 },
    { 0, 0 },
    { 0x2315c62e, 85 },
    { 0, 0 },
    { 0xa985c630, 55 },
    { 0, 0 },
    { 0x15c632, 81 },
    { 0, 0 },
    { 0xf735c634, 23 },
    { 0xd3e5c635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945c638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5c63b, 22 },
    { 0xdbd5c63c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9255c63f, 90 },
    { 0, 0 },
    { 0xf925c641, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd65c644, 90 },
    { 0x8b05c645, 23 },
    { 0xc915c646, 90 },
    { 0, 0 },
    { 0x1f05c648, 87 },
    { 0xdbf5c649, 95 },
    { 0xcd45c648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f95c64d, 90 },
    { 0xf4e5c64e, 68 },
    { 0x8ff5c64d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6795c652, 61 },
    { 0x1e55c652, 78 },
    { 0x5dd5c654, 98 },
    { 0, 0 },
    { 0x39e5c656, 23 },
    { 0, 0 },
    { 0x5cc5c658, 23 },
    { 0, 0 },
    { 0xf475c65a, 52 },
    { 0x645c65b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb245c660, 23 },
    { 0xb255c661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125c664, 61 },
    { 0, 0 },
    { 0x5dc5c666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f05c669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f35c66d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a05c672, 87 },
    { 0x7395c672, 98 },
    { 0x2de5c673, 23 },
    { 0, 0 },
    { 0x3a5c676, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365c67d, 17 },
    { 0, 0 },
    { 0x56f5c67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da5c683, 40 },
    { 0xf075c684, 90 },
    { 0xd25c684, 33 },
    { 0, 0 },
    { 0x68e5c687, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd695c68b, 49 },
    { 0x3425c68c, 87 },
    { 0x8e75c68c, 98 },
    { 0x6565c68d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e5c696, 68 },
    { 0x4a75c696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3995c69a, 90 },
    { 0x8a35c69b, 61 },
    { 0x2be5c69a, 33 },
    { 0x5b95c69d, 33 },
    { 0x1395c69d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8445c6a2, 87 },
    { 0x5795c6a3, 67 },
    { 0xc035c6a4, 52 },
    { 0xe9a5c6a5, 66 },
    { 0, 0 },
    { 0xd0b5c6a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8375c6aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5895c6ad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15c6b0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c6b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c6b7, 23 },
    { 0x1c35c6b8, 33 },
    { 0, 0 },
    { 0xcd15c6ba, 61 },
    { 0xc715c6bb, 65 },
    { 0, 0 },
    { 0xd735c6bd, 100 },
    { 0x5495c6be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff5c6c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d5c6cf, 9 },
    { 0xcf85c6cf, 98 },
    { 0xf025c6d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb5c6d6, 99 },
    { 0x5775c6d7, 95 },
    { 0x61b5c6d7, 85 },
    { 0x9595c6d9, 88 },
    { 0x5a25c6d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf645c6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7045c6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5315c6e9, 61 },
    { 0x6d85c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7785c6ed, 23 },
    { 0, 0 },
    { 0xf735c6ef, 33 },
    { 0, 0 },
    { 0xb6f5c6f1, 98 },
    { 0x93e5c6f2, 93 },
    { 0xddd5c6f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b35c6f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e5c6fa, 78 },
    { 0, 0 },
    { 0x9405c6fc, 33 },
    { 0x5475c6fd, 98 },
    { 0xe305c6fe, 98 },
    { 0xae35c6ff, 46 },
    { 0x8375c6fe, 55 },
    { 0x7555c701, 98 },
    { 0, 0 },
    { 0xce15c703, 91 },
    { 0xb3b5c704, 90 },
    { 0, 0 },
    { 0xe6a5c706, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545c709, 76 },
    { 0xe125c70a, 98 },
    { 0x5e05c70b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5c716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865c71f, 61 },
    { 0x2f05c720, 76 },
    { 0x1d65c721, 90 },
    { 0x6665c722, 93 },
    { 0x1b75c723, 23 },
    { 0xaf5c724, 98 },
    { 0x3bb5c721, 90 },
    { 0x97d5c726, 33 },
    { 0xec25c726, 94 },
    { 0xc405c722, 76 },
    { 0xd8c5c729, 76 },
    { 0xa825c72a, 95 },
    { 0x9e65c72b, 90 },
    { 0xad25c72c, 55 },
    { 0xf2a5c722, 43 },
    { 0x6035c72e, 33 },
    { 0x3965c72f, 33 },
    { 0x1035c72e, 33 },
    { 0x9eb5c731, 100 },
    { 0xd5f5c732, 90 },
    { 0x9f75c728, 33 },
    { 0xecc5c734, 76 },
    { 0xa225c735, 99 },
    { 0x8715c721, 46 },
    { 0x8d5c730, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed35c73d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa465c747, 52 },
    { 0xba65c748, 33 },
    { 0, 0 },
    { 0x7045c74a, 67 },
    { 0x2a5c74a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2865c74e, 95 },
    { 0, 0 },
    { 0x4ba5c750, 52 },
    { 0xe835c751, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb5c757, 64 },
    { 0x7925c758, 34 },
    { 0x18f5c759, 90 },
    { 0x3615c759, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65c75e, 99 },
    { 0x3405c75f, 90 },
    { 0x4fb5c75e, 55 },
    { 0xb865c760, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8985c766, 33 },
    { 0xf9b5c767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba5c76a, 95 },
    { 0x66e5c76a, 33 },
    { 0x26a5c76c, 52 },
    { 0xa495c76c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff75c773, 78 },
    { 0, 0 },
    { 0x9205c775, 17 },
    { 0, 0 },
    { 0xf645c777, 77 },
    { 0, 0 },
    { 0xb565c779, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2125c77e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb405c784, 33 },
    { 0xf235c785, 81 },
    { 0x5575c784, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975c78e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25c791, 99 },
    { 0xa8b5c791, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c795, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f5c798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45c79c, 95 },
    { 0, 0 },
    { 0x13c5c79e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c5c7a1, 61 },
    { 0x2425c7a2, 55 },
    { 0xd185c7a1, 75 },
    { 0xe595c7a1, 90 },
    { 0xfba5c7a5, 40 },
    { 0xc85c7a5, 91 },
    { 0xfbf5c7a3, 55 },
    { 0, 0 },
    { 0x7625c7a9, 43 },
    { 0x95c5c7aa, 90 },
    { 0, 0 },
    { 0xb8f5c7ac, 95 },
    { 0xac65c7ac, 34 },
    { 0x3a25c7ac, 90 },
    { 0xa865c7ac, 90 },
    { 0xadf5c7b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3165c7b9, 95 },
    { 0xfc85c7ba, 33 },
    { 0x1425c7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45c7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc815c7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba5c7cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xac35c7cf, 68 },
    { 0, 0 },
    { 0xf2d5c7d1, 33 },
    { 0xc795c7d2, 90 },
    { 0x5c65c7d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d95c7da, 61 },
    { 0, 0 },
    { 0xe905c7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f5c7df, 61 },
    { 0xbf65c7e0, 95 },
    { 0x55e5c7df, 67 },
    { 0, 0 },
    { 0x105c7e3, 90 },
    { 0x6575c7e4, 67 },
    { 0x18e5c7e5, 93 },
    { 0x8f55c7e3, 78 },
    { 0x3ff5c7e6, 75 },
    { 0x77c5c7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5c7ec, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4625c7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c5c7fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa75c7fe, 78 },
    { 0x62e5c7fe, 40 },
    { 0x6395c7ff, 34 },
    { 0xbc15c801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe5c804, 90 },
    { 0x9be5c805, 38 },
    { 0, 0 },
    { 0x3bb5c807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695c80e, 78 },
    { 0, 0 },
    { 0x6495c810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205c817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d15c81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2695c81d, 17 },
    { 0x8b45c81e, 77 },
    { 0xffc5c81f, 90 },
    { 0x3d55c81d, 91 },
    { 0x5d75c821, 33 },
    { 0x80b5c822, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3565c825, 90 },
    { 0xadd5c826, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b5c82c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c5c831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa275c837, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xde55c83a, 23 },
    { 0, 0 },
    { 0xa275c83c, 85 },
    { 0x1cf5c83c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b85c841, 33 },
    { 0x9a15c842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe325c84a, 90 },
    { 0x3c95c84a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca5c850, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545c854, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b5c859, 95 },
    { 0x14c5c85a, 23 },
    { 0x5605c85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4575c85f, 98 },
    { 0x6615c85f, 55 },
    { 0, 0 },
    { 0xd565c862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d5c866, 23 },
    { 0x8535c867, 55 },
    { 0, 0 },
    { 0x7d15c869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea5c86e, 55 },
    { 0, 0 },
    { 0xc865c870, 55 },
    { 0xd595c871, 23 },
    { 0x3ab5c872, 95 },
    { 0xd425c872, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6205c87a, 43 },
    { 0x9075c87a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf15c881, 88 },
    { 0x6025c882, 91 },
    { 0xfd5c882, 67 },
    { 0x7475c884, 61 },
    { 0x1615c885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa405c88e, 78 },
    { 0x1a55c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef5c892, 52 },
    { 0xffc5c893, 78 },
    { 0x2095c893, 100 },
    { 0x3615c895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c89c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa605c89f, 90 },
    { 0xba75c8a0, 90 },
    { 0x3e25c89f, 90 },
    { 0x3c15c8a2, 23 },
    { 0, 0 },
    { 0x2035c8a4, 33 },
    { 0, 0 },
    { 0x7c85c8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255c8ac, 85 },
    { 0, 0 },
    { 0x6355c8ae, 40 },
    { 0, 0 },
    { 0xcb5c8b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf145c8b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c8ba, 61 },
    { 0x7935c8bb, 43 },
    { 0x62f5c8bc, 61 },
    { 0x635c8bd, 84 },
    { 0xce75c8bd, 34 },
    { 0, 0 },
    { 0x53b5c8c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e5c8c4, 98 },
    { 0, 0 },
    { 0x6445c8c6, 100 },
    { 0x33a5c8c6, 68 },
    { 0x9e15c8c6, 34 },
    { 0, 0 },
    { 0x2815c8ca, 33 },
    { 0x6195c8cb, 90 },
    { 0x23e5c8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f5c8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee65c8d5, 23 },
    { 0x4185c8d5, 95 },
    { 0xae65c8d7, 68 },
    { 0, 0 },
    { 0x82b5c8d9, 95 },
    { 0x6dd5c8d9, 52 },
    { 0xf955c8d9, 95 },
    { 0x4c75c8d9, 98 },
    { 0x2705c8dd, 43 },
    { 0xa955c8de, 33 },
    { 0x2145c8df, 88 },
    { 0, 0 },
    { 0xdfb5c8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5c8e4, 32 },
    { 0x29a5c8e5, 23 },
    { 0x80d5c8e6, 23 },
    { 0x7165c8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f5c8ec, 23 },
    { 0x2095c8ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755c900, 33 },
    { 0x3195c900, 76 },
    { 0, 0 },
    { 0xbe75c903, 90 },
    { 0x14a5c904, 33 },
    { 0x8c5c905, 90 },
    { 0x77e5c906, 46 },
    { 0xd45c906, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5c90b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed35c90f, 46 },
    { 0xa835c90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd175c915, 76 },
    { 0x4975c915, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf45c91d, 85 },
    { 0xa3b5c91d, 43 },
    { 0xba75c91f, 98 },
    { 0xd205c920, 93 },
    { 0, 0 },
    { 0x8255c922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8455c925, 85 },
    { 0x8445c926, 99 },
    { 0, 0 },
    { 0xc225c928, 90 },
    { 0, 0 },
    { 0x1b05c92a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b35c92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6785c939, 76 },
    { 0x8cb5c93a, 100 },
    { 0xdc05c93b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc5c93e, 23 },
    { 0xa015c93f, 23 },
    { 0xbd55c940, 46 },
    { 0, 0 },
    { 0x6d35c942, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a5c947, 55 },
    { 0, 0 },
    { 0x8cc5c949, 78 },
    { 0x5985c94a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd925c953, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf255c956, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8045c95b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe365c95f, 98 },
    { 0, 0 },
    { 0x4a35c961, 90 },
    { 0x5015c961, 61 },
    { 0xbcd5c963, 95 },
    { 0xd2e5c964, 55 },
    { 0x4715c965, 98 },
    { 0x9825c966, 84 },
    { 0x45c5c967, 43 },
    { 0xbe05c966, 61 },
    { 0x17e5c966, 33 },
    { 0x93d5c964, 95 },
    { 0x4c35c968, 98 },
    { 0, 0 },
    { 0x11d5c96d, 100 },
    { 0, 0 },
    { 0x48d5c96f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd5c972, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75c976, 78 },
    { 0xc635c977, 66 },
    { 0x9a5c978, 33 },
    { 0x5775c977, 95 },
    { 0x86e5c97a, 61 },
    { 0xd2b5c976, 33 },
    { 0xeb95c97a, 100 },
    { 0xd815c97c, 91 },
    { 0, 0 },
    { 0x1fb5c97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b5c982, 90 },
    { 0x8ae5c982, 64 },
    { 0xc415c982, 33 },
    { 0xfde5c984, 85 },
    { 0, 0 },
    { 0x185c987, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd835c98a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c55c98f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5c994, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb5c99a, 33 },
    { 0, 0 },
    { 0x4d65c99c, 98 },
    { 0x1375c99c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995c9a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f5c9a7, 23 },
    { 0x86e5c9a7, 33 },
    { 0x5445c9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165c9b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c9b3, 90 },
    { 0x8495c9b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb35c9be, 68 },
    { 0xb605c9bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa375c9c5, 90 },
    { 0xb1d5c9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d5c9c9, 90 },
    { 0, 0 },
    { 0x60e5c9cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d5c9ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f5c9d7, 55 },
    { 0, 0 },
    { 0x3dc5c9d9, 33 },
    { 0xeab5c9d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd275c9de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec5c9e1, 76 },
    { 0xf665c9e2, 98 },
    { 0xd865c9e3, 33 },
    { 0xfe55c9e4, 78 },
    { 0x5a15c9e5, 75 },
    { 0x5e65c9e6, 55 },
    { 0x2c35c9e7, 61 },
    { 0xbf55c9e7, 22 },
    { 0xf695c9e3, 34 },
    { 0x18e5c9ea, 99 },
    { 0x2f15c9e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5c9ee, 34 },
    { 0xbc85c9ef, 42 },
    { 0, 0 },
    { 0x295c9f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45c9f4, 23 },
    { 0, 0 },
    { 0xecf5c9f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc5c9fb, 55 },
    { 0x9295c9fb, 90 },
    { 0xdb15c9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75ca01, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775ca07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5ca0b, 23 },
    { 0, 0 },
    { 0x4ab5ca0d, 34 },
    { 0, 0 },
    { 0xdf05ca0f, 90 },
    { 0, 0 },
    { 0x27e5ca11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7645ca14, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c5ca17, 98 },
    { 0, 0 },
    { 0x6625ca19, 33 },
    { 0, 0 },
    { 0xce05ca1b, 87 },
    { 0x8a35ca1c, 65 },
    { 0, 0 },
    { 0xbd35ca1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f5ca24, 23 },
    { 0xdf35ca24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5ca29, 87 },
    { 0, 0 },
    { 0x8bc5ca2b, 90 },
    { 0xaa75ca2c, 77 },
    { 0x63a5ca2c, 95 },
    { 0x2065ca2e, 93 },
    { 0, 0 },
    { 0x6a55ca30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d25ca34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5ca38, 61 },
    { 0, 0 },
    { 0x9e85ca3a, 33 },
    { 0x6435ca3b, 91 },
    { 0xa185ca3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2455ca3f, 55 },
    { 0, 0 },
    { 0xa65ca41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ca44, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b95ca4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd5ca4f, 38 },
    { 0x1c05ca4f, 76 },
    { 0xfb55ca4f, 55 },
    { 0xc025ca52, 90 },
    { 0x76b5ca52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e75ca57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa05ca5c, 85 },
    { 0x6295ca5d, 34 },
    { 0xe825ca5d, 61 },
    { 0x765ca5f, 23 },
    { 0, 0 },
    { 0xe435ca61, 61 },
    { 0x20d5ca62, 34 },
    { 0x66d5ca62, 55 },
    { 0xcc65ca64, 98 },
    { 0xde95ca62, 90 },
    { 0x31b5ca66, 33 },
    { 0, 0 },
    { 0xca85ca68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b65ca6b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada5ca72, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635ca78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685ca7c, 61 },
    { 0xf0c5ca7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b5ca86, 98 },
    { 0x23e5ca87, 66 },
    { 0x7005ca88, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd425ca8f, 95 },
    { 0, 0 },
    { 0x1975ca91, 90 },
    { 0xe765ca91, 55 },
    { 0x3305ca92, 90 },
    { 0x8cb5ca91, 77 },
    { 0x77f5ca92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b5ca99, 23 },
    { 0, 0 },
    { 0xcd75ca9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae65ca9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c5caa5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5745caa9, 61 },
    { 0x8865caaa, 78 },
    { 0x65e5caab, 84 },
    { 0xd8c5caac, 49 },
    { 0, 0 },
    { 0x7f65caae, 64 },
    { 0, 0 },
    { 0x32e5cab0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e5cab3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f45cab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c5caba, 90 },
    { 0, 0 },
    { 0xadc5cabc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c85cac5, 23 },
    { 0xca05cac5, 67 },
    { 0, 0 },
    { 0xc305cac8, 55 },
    { 0x9a5cac9, 78 },
    { 0x5295cac9, 33 },
    { 0x2ea5cacb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b5cad3, 43 },
    { 0x92e5cad3, 98 },
    { 0, 0 },
    { 0x99f5cad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb15cae2, 23 },
    { 0x5575cae3, 90 },
    { 0x195cae3, 90 },
    { 0xeaf5cae5, 97 },
    { 0x7a95cae3, 93 },
    { 0x5015cae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195caea, 90 },
    { 0x19a5caea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc835caef, 61 },
    { 0x5115caef, 34 },
    { 0x6615caf0, 81 },
    { 0x2775caf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a5cafa, 50 },
    { 0x6175cafa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb185cb00, 61 },
    { 0, 0 },
    { 0x8155cb02, 61 },
    { 0xe7e5cb03, 33 },
    { 0x9cb5cb04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625cb08, 95 },
    { 0, 0 },
    { 0x6795cb0a, 66 },
    { 0x93d5cb0b, 98 },
    { 0, 0 },
    { 0xe05cb0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325cb14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de5cb19, 55 },
    { 0, 0 },
    { 0x7995cb1b, 23 },
    { 0xaeb5cb1b, 95 },
    { 0xa375cb1c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9135cb21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65cb24, 98 },
    { 0xc865cb25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa5cb2c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda5cb2f, 61 },
    { 0, 0 },
    { 0xe235cb31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a75cb3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa955cb3d, 52 },
    { 0x9d35cb3e, 77 },
    { 0x6cf5cb3f, 98 },
    { 0x6815cb3d, 99 },
    { 0x58b5cb41, 83 },
    { 0xf675cb3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb735cb4b, 33 },
    { 0x7675cb4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd745cb51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa5cb54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a5cb5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff5cb5e, 33 },
    { 0, 0 },
    { 0xce5cb60, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf5cb63, 90 },
    { 0xa5f5cb64, 98 },
    { 0xf455cb65, 23 },
    { 0x3a5cb66, 56 },
    { 0x7165cb67, 23 },
    { 0xf8b5cb67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515cb6d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25cb73, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b85cb76, 87 },
    { 0x2985cb77, 98 },
    { 0x9ae5cb76, 98 },
    { 0, 0 },
    { 0x4825cb7a, 33 },
    { 0x33b5cb7b, 78 },
    { 0x2a75cb7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e5cb7f, 94 },
    { 0xf05cb80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c5cb85, 76 },
    { 0, 0 },
    { 0x4935cb87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d5cb90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5cb94, 95 },
    { 0xd235cb94, 43 },
    { 0x9305cb94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1845cb9d, 33 },
    { 0x9965cb9e, 43 },
    { 0xad35cb9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565cba3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c35cba6, 100 },
    { 0xb685cba7, 90 },
    { 0, 0 },
    { 0x235cba9, 90 },
    { 0x8295cbaa, 61 },
    { 0, 0 },
    { 0x5925cbac, 33 },
    { 0x34f5cbad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb45cbb0, 90 },
    { 0, 0 },
    { 0xe045cbb2, 45 },
    { 0x52e5cbb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5cbb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d5cbbe, 53 },
    { 0x5f5cbbf, 33 },
    { 0xa6d5cbbf, 33 },
    { 0x3e35cbbe, 84 },
    { 0xa985cbc2, 33 },
    { 0x6735cbc3, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5cbc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c5cbcc, 23 },
    { 0x8c25cbcc, 61 },
    { 0, 0 },
    { 0xb0d5cbcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085cbd7, 98 },
    { 0xcc45cbd8, 17 },
    { 0, 0 },
    { 0xdf65cbda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65cbdd, 98 },
    { 0, 0 },
    { 0xce65cbdf, 43 },
    { 0xebc5cbe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x225cbe3, 90 },
    { 0x8cd5cbe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad25cbe7, 55 },
    { 0x5715cbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb65cbec, 78 },
    { 0xc765cbed, 66 },
    { 0, 0 },
    { 0x6125cbef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba5cbf2, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c5cbf8, 78 },
    { 0xfff5cbf8, 55 },
    { 0x27b5cbfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d5cbfd, 90 },
    { 0, 0 },
    { 0x23d5cbff, 78 },
    { 0, 0 },
    { 0x4305cc01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf5cc0a, 33 },
    { 0x8325cc0b, 95 },
    { 0xbb65cc0c, 55 },
    { 0, 0 },
    { 0x86a5cc0e, 61 },
    { 0, 0 },
    { 0x4455cc10, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625cc17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05cc1a, 61 },
    { 0x94d5cc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda55cc1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c45cc24, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b65cc28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec5cc2b, 76 },
    { 0xe9e5cc2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5cc32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255cc35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce05cc38, 55 },
    { 0, 0 },
    { 0x45f5cc3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc95cc3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c95cc42, 23 },
    { 0xcc5cc43, 61 },
    { 0x1335cc43, 33 },
    { 0x36c5cc44, 90 },
    { 0, 0 },
    { 0xe5cc47, 23 },
    { 0, 0 },
    { 0x4585cc49, 61 },
    { 0xa2b5cc4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c65cc56, 23 },
    { 0x4425cc57, 100 },
    { 0, 0 },
    { 0xfbc5cc59, 85 },
    { 0x8cc5cc5a, 61 },
    { 0, 0 },
    { 0x7b45cc5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75cc67, 34 },
    { 0x9065cc68, 34 },
    { 0, 0 },
    { 0x69d5cc6a, 98 },
    { 0xd8a5cc6b, 80 },
    { 0, 0 },
    { 0x7c25cc6d, 90 },
    { 0xd985cc6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2475cc72, 34 },
    { 0xb525cc73, 23 },
    { 0, 0 },
    { 0x3a05cc75, 88 },
    { 0, 0 },
    { 0x4ad5cc77, 95 },
    { 0xfde5cc77, 95 },
    { 0, 0 },
    { 0x3e5cc7a, 61 },
    { 0xc8d5cc7b, 34 },
    { 0x1385cc7a, 95 },
    { 0x1c05cc7c, 99 },
    { 0xe6b5cc7e, 98 },
    { 0xd695cc7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75cc83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5cc86, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35cc89, 23 },
    { 0, 0 },
    { 0x3705cc8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15cc8e, 33 },
    { 0xd0a5cc8e, 23 },
    { 0, 0 },
    { 0x64d5cc91, 55 },
    { 0, 0 },
    { 0xa725cc93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac5cc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb485cc9d, 93 },
    { 0, 0 },
    { 0xed35cc9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b5cca2, 33 },
    { 0xd9a5cca3, 95 },
    { 0xcdc5cca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85cca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5ccad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d65ccb3, 33 },
    { 0, 0 },
    { 0x2b35ccb5, 80 },
    { 0, 0 },
    { 0x9165ccb7, 100 },
    { 0x7575ccb8, 53 },
    { 0x5925ccb8, 34 },
    { 0x3f25ccba, 90 },
    { 0x96a5ccbb, 96 },
    { 0x2235ccbc, 66 },
    { 0xf475ccbd, 43 },
    { 0x6b15ccbe, 66 },
    { 0x3695ccbf, 23 },
    { 0x4675ccbd, 33 },
    { 0x67b5ccb8, 34 },
    { 0x2885ccbe, 55 },
    { 0, 0 },
    { 0x13c5ccc4, 76 },
    { 0xdd55ccc4, 95 },
    { 0x3315ccc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55ccca, 95 },
    { 0x8f45cccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d5ccce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5ccd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad5ccd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35ccdb, 33 },
    { 0x22f5ccdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda45cce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735cce4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b5cce9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc985ccef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d75ccf4, 34 },
    { 0xcb95ccf4, 67 },
    { 0xee55ccf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565ccf9, 61 },
    { 0, 0 },
    { 0x12f5ccfb, 91 },
    { 0x3a45ccfc, 78 },
    { 0x3af5ccfd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c5cd00, 61 },
    { 0x4225cd00, 67 },
    { 0x69c5cd01, 61 },
    { 0xa695cd03, 98 },
    { 0xc7e5cd03, 34 },
    { 0x9f85cd04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3055cd0e, 43 },
    { 0xd2b5cd0f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725cd16, 61 },
    { 0x5d45cd17, 90 },
    { 0x9eb5cd16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f45cd20, 90 },
    { 0x3015cd21, 23 },
    { 0, 0 },
    { 0xa035cd23, 76 },
    { 0x4745cd23, 90 },
    { 0, 0 },
    { 0xc165cd26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905cd29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d5cd30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8675cd35, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8325cd3b, 95 },
    { 0x1105cd3b, 93 },
    { 0xdf05cd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece5cd40, 95 },
    { 0, 0 },
    { 0x5415cd42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2275cd4d, 98 },
    { 0x5cb5cd4d, 23 },
    { 0x96d5cd4e, 95 },
    { 0xaf5cd50, 76 },
    { 0xf035cd50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9975cd5b, 90 },
    { 0xbff5cd5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55cd60, 23 },
    { 0xe3f5cd61, 90 },
    { 0x18e5cd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05cd65, 76 },
    { 0, 0 },
    { 0xb575cd67, 33 },
    { 0, 0 },
    { 0x6965cd69, 33 },
    { 0x9e35cd6a, 75 },
    { 0, 0 },
    { 0x1e85cd6c, 61 },
    { 0x33b5cd6d, 100 },
    { 0x56f5cd6c, 23 },
    { 0x5b75cd6f, 33 },
    { 0x31b5cd70, 90 },
    { 0x3545cd70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b5cd74, 33 },
    { 0x1be5cd74, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5cd7c, 90 },
    { 0xdbb5cd7d, 87 },
    { 0x7aa5cd7e, 100 },
    { 0x4d5cd7f, 67 },
    { 0x36d5cd7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b5cd89, 95 },
    { 0, 0 },
    { 0x65e5cd8b, 98 },
    { 0, 0 },
    { 0x68a5cd8d, 90 },
    { 0xb865cd8e, 66 },
    { 0xb2c5cd8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a15cd92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c5cd95, 77 },
    { 0xd365cd96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c5cd9c, 33 },
    { 0, 0 },
    { 0x7065cd9e, 76 },
    { 0x5a85cd9e, 23 },
    { 0x2205cd9f, 33 },
    { 0, 0 },
    { 0xffa5cda2, 23 },
    { 0xcd05cda2, 34 },
    { 0x3985cda4, 95 },
    { 0x7765cda5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a5cdaa, 76 },
    { 0x3405cdaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d5cdb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b5cdb4, 81 },
    { 0x74b5cdb4, 90 },
    { 0x86c5cdb6, 43 },
    { 0, 0 },
    { 0x7345cdb8, 98 },
    { 0x8725cdb8, 76 },
    { 0, 0 },
    { 0x8fe5cdbb, 90 },
    { 0x2685cdbb, 101 },
    { 0x465cdbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5cdc3, 69 },
    { 0, 0 },
    { 0xa275cdc5, 61 },
    { 0x1ed5cdc6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b5cdcd, 61 },
    { 0x2805cdce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55cdd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5cdd9, 33 },
    { 0, 0 },
    { 0x4885cddb, 98 },
    { 0x2885cddb, 66 },
    { 0x1d55cddc, 43 },
    { 0x5125cdde, 95 },
    { 0xdfc5cdde, 84 },
    { 0xbf45cddf, 46 },
    { 0xbab5cde1, 33 },
    { 0xae35cde2, 34 },
    { 0x7f25cde3, 38 },
    { 0x5135cde3, 90 },
    { 0xe605cddc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a5cde9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f5cdec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5cdef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2905ce03, 23 },
    { 0xebf5ce04, 52 },
    { 0, 0 },
    { 0xddf5ce06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b5ce0e, 23 },
    { 0x26b5ce0e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c5ce12, 90 },
    { 0, 0 },
    { 0xdaa5ce14, 34 },
    { 0xada5ce15, 55 },
    { 0, 0 },
    { 0x3045ce17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385ce1c, 23 },
    { 0x1635ce1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f65ce22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ce28, 98 },
    { 0x67d5ce28, 34 },
    { 0, 0 },
    { 0x9c55ce2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb85ce31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf75ce34, 95 },
    { 0x33f5ce35, 86 },
    { 0x7a35ce36, 95 },
    { 0x6b15ce37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f5ce3a, 33 },
    { 0x4d15ce3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc825ce42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe5ce4e, 33 },
    { 0xc295ce4f, 93 },
    { 0x3dd5ce50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f15ce53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf5ce59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5ce5f, 61 },
    { 0xb1e5ce60, 93 },
    { 0xcf75ce61, 88 },
    { 0xacd5ce62, 90 },
    { 0x3a95ce62, 95 },
    { 0, 0 },
    { 0x8c95ce65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5ce71, 33 },
    { 0, 0 },
    { 0xd065ce73, 52 },
    { 0xadd5ce74, 95 },
    { 0x74f5ce75, 33 },
    { 0x75d5ce76, 55 },
    { 0, 0 },
    { 0x1145ce78, 46 },
    { 0xb175ce78, 91 },
    { 0x1fe5ce78, 61 },
    { 0x7025ce7b, 61 },
    { 0x8525ce7c, 97 },
    { 0xc7f5ce7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e85ce82, 33 },
    { 0, 0 },
    { 0x6cd5ce84, 95 },
    { 0, 0 },
    { 0x6e25ce86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc85ce8b, 85 },
    { 0x8c45ce8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a5ce8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105ce92, 68 },
    { 0x82f5ce92, 55 },
    { 0, 0 },
    { 0x1ce5ce95, 23 },
    { 0x2ff5ce95, 17 },
    { 0x4cf5ce97, 61 },
    { 0xb55ce98, 67 },
    { 0x9fe5ce96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9065ce9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65cea1, 61 },
    { 0, 0 },
    { 0xca35cea3, 34 },
    { 0, 0 },
    { 0xc6f5cea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285ceac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5ceb1, 98 },
    { 0x3925ceb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4265ceb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace5cebb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1595cebf, 61 },
    { 0, 0 },
    { 0x455cec1, 90 },
    { 0, 0 },
    { 0xde05cec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1585cec6, 36 },
    { 0x9c05cec7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd5cecb, 94 },
    { 0xec95cecc, 33 },
    { 0xcb35cecb, 90 },
    { 0, 0 },
    { 0x8765cecf, 98 },
    { 0x64a5ced0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5ced8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85cedb, 43 },
    { 0x99d5cedc, 23 },
    { 0xaa75cedb, 90 },
    { 0, 0 },
    { 0xd695cedf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95cee3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae5ceeb, 90 },
    { 0xf535ceec, 33 },
    { 0, 0 },
    { 0x2e75ceee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7075cef2, 75 },
    { 0, 0 },
    { 0x4795cef4, 94 },
    { 0, 0 },
    { 0x415cef6, 33 },
    { 0xa5a5cef7, 90 },
    { 0xf0c5cef8, 43 },
    { 0xd8f5cef9, 76 },
    { 0x85d5cefa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f75cefe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8525cf03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7665cf07, 55 },
    { 0x5f45cf08, 66 },
    { 0xb335cf09, 84 },
    { 0x70e5cf0a, 90 },
    { 0x7cb5cf08, 98 },
    { 0xdb95cf08, 61 },
    { 0x5005cf09, 87 },
    { 0x86b5cf0e, 87 },
    { 0xf425cf0f, 95 },
    { 0xf05cf10, 91 },
    { 0xac05cf0f, 61 },
    { 0x7205cf10, 95 },
    { 0, 0 },
    { 0xc3a5cf14, 98 },
    { 0xd145cf15, 90 },
    { 0xaa05cf14, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5cf1f, 95 },
    { 0x185cf20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe875cf25, 23 },
    { 0x1ed5cf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635cf2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc5cf31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5cf36, 90 },
    { 0xbd15cf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5cf3b, 55 },
    { 0xa2c5cf3b, 23 },
    { 0xb1c5cf3d, 55 },
    { 0x64a5cf3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f15cf4e, 98 },
    { 0xa255cf4f, 66 },
    { 0x2345cf50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5cf58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15cf5b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9985cf62, 90 },
    { 0x5845cf63, 33 },
    { 0, 0 },
    { 0x3035cf65, 23 },
    { 0x35a5cf66, 66 },
    { 0xd975cf67, 90 },
    { 0xc615cf67, 95 },
    { 0x6bd5cf65, 98 },
    { 0xe615cf68, 33 },
    { 0x52c5cf68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4385cf6e, 78 },
    { 0xba85cf6e, 34 },
    { 0, 0 },
    { 0x4a85cf71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cf78, 46 },
    { 0x9d95cf79, 64 },
    { 0, 0 },
    { 0x93d5cf7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5cf7e, 98 },
    { 0, 0 },
    { 0xa9e5cf80, 90 },
    { 0xfe55cf81, 95 },
    { 0xad55cf80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe235cf87, 33 },
    { 0x3045cf88, 36 },
    { 0, 0 },
    { 0xe4b5cf8a, 95 },
    { 0xf525cf8a, 81 },
    { 0x6fe5cf8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e95cf98, 61 },
    { 0x6c35cf99, 96 },
    { 0x4c45cf9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5cf9e, 33 },
    { 0xf3a5cf9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5125cfa2, 84 },
    { 0xb3d5cfa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665cfa6, 94 },
    { 0x5795cfa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2395cfae, 90 },
    { 0, 0 },
    { 0xa6b5cfb0, 99 },
    { 0x6fc5cfb1, 68 },
    { 0xf255cfb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5cfb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe5cfba, 96 },
    { 0xd405cfbb, 93 },
    { 0x8815cfba, 90 },
    { 0, 0 },
    { 0xda85cfbe, 23 },
    { 0x4cb5cfbf, 90 },
    { 0x1bf5cfbf, 67 },
    { 0xd585cfc1, 90 },
    { 0x4845cfc2, 99 },
    { 0x7c85cfc2, 76 },
    { 0x5d05cfc3, 98 },
    { 0xf425cfc5, 87 },
    { 0x2265cfc2, 61 },
    { 0xfd75cfc7, 69 },
    { 0x84d5cfbf, 33 },
    { 0xb665cfc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075cfcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x435cfcf, 87 },
    { 0x6965cfcf, 69 },
    { 0x9145cfcf, 90 },
    { 0x6515cfd1, 33 },
    { 0, 0 },
    { 0x13b5cfd4, 84 },
    { 0, 0 },
    { 0xc9e5cfd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85cfda, 23 },
    { 0xd575cfda, 90 },
    { 0x9795cfdc, 93 },
    { 0x2b05cfdc, 43 },
    { 0xf2e5cfdd, 33 },
    { 0xb285cfdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315cfe6, 46 },
    { 0xfa55cfe7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6245cfed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb15cff0, 90 },
    { 0xd465cff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d5cff8, 76 },
    { 0, 0 },
    { 0xd325cffa, 33 },
    { 0, 0 },
    { 0x5d35cffc, 85 },
    { 0, 0 },
    { 0x1995cffe, 33 },
    { 0xe3a5cfff, 61 },
    { 0, 0 },
    { 0xc805d001, 90 },
    { 0x39c5d002, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a85d00a, 98 },
    { 0, 0 },
    { 0x1355d00c, 23 },
    { 0x8e35d00d, 88 },
    { 0x73d5d00e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515d012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435d019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c5d01f, 95 },
    { 0, 0 },
    { 0x9e25d021, 55 },
    { 0xdf45d021, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d5d026, 52 },
    { 0xa345d026, 55 },
    { 0x8e75d028, 98 },
    { 0x2ba5d029, 90 },
    { 0x5295d029, 66 },
    { 0, 0 },
    { 0x1ec5d02c, 55 },
    { 0, 0 },
    { 0x7805d02e, 98 },
    { 0x2205d02f, 98 },
    { 0xc315d030, 43 },
    { 0x9ac5d031, 90 },
    { 0xcc45d032, 34 },
    { 0x4d55d033, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a25d03b, 67 },
    { 0x6c25d03b, 69 },
    { 0, 0 },
    { 0x8415d03e, 90 },
    { 0x9a35d03f, 95 },
    { 0x9bc5d03e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935d048, 61 },
    { 0x1eb5d048, 43 },
    { 0, 0 },
    { 0xf065d04b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba55d04e, 52 },
    { 0x4b25d04f, 61 },
    { 0xccd5d050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605d053, 55 },
    { 0x5045d054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5d063, 66 },
    { 0xe1d5d063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb5d06e, 33 },
    { 0xddc5d06e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755d073, 34 },
    { 0, 0 },
    { 0x6e35d075, 34 },
    { 0x5b15d075, 76 },
    { 0xf395d077, 33 },
    { 0, 0 },
    { 0xc505d079, 55 },
    { 0x915d079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e65d07e, 33 },
    { 0x2b55d07f, 90 },
    { 0, 0 },
    { 0x8be5d081, 93 },
    { 0, 0 },
    { 0x3a35d083, 23 },
    { 0xb505d084, 90 },
    { 0xe6d5d083, 85 },
    { 0x9195d085, 90 },
    { 0xec55d086, 55 },
    { 0x9685d088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215d08b, 93 },
    { 0, 0 },
    { 0x7185d08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d5d098, 61 },
    { 0x8375d099, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d09c, 87 },
    { 0x32a5d09d, 90 },
    { 0x8035d09c, 87 },
    { 0x35e5d09f, 55 },
    { 0x9085d0a0, 61 },
    { 0x95f5d09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa985d0aa, 78 },
    { 0x6345d0aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc5d0ae, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65d0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35d0b6, 61 },
    { 0xdcf5d0b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e5d0bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c75d0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b85d0c6, 33 },
    { 0xb3a5d0c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5d0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e5d0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1405d0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d5d0d9, 100 },
    { 0xa015d0da, 61 },
    { 0x2d75d0da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a05d0de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa255d0e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5325d0e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d5d0ea, 40 },
    { 0xd935d0eb, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xec25d0ee, 90 },
    { 0x675d0ee, 23 },
    { 0xe475d0ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765d0f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f65d0fc, 78 },
    { 0, 0 },
    { 0xb95d0fe, 61 },
    { 0x6a65d0ff, 90 },
    { 0x5925d0ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4325d106, 61 },
    { 0, 0 },
    { 0xac95d108, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e5d10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5d115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5d11a, 46 },
    { 0x3fd5d11b, 76 },
    { 0, 0 },
    { 0x3a05d11d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe525d121, 85 },
    { 0xc6e5d121, 61 },
    { 0, 0 },
    { 0xbb45d124, 100 },
    { 0x4f45d125, 66 },
    { 0x74d5d126, 46 },
    { 0x8945d126, 34 },
    { 0x3c25d128, 23 },
    { 0x7795d129, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c5d12c, 33 },
    { 0xc7b5d12d, 33 },
    { 0xc3a5d12c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d135, 23 },
    { 0x1fb5d135, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2255d13a, 61 },
    { 0, 0 },
    { 0xe05d13c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c5d140, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1515d143, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f55d148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695d14d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa515d153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b75d15c, 93 },
    { 0xfa95d15c, 33 },
    { 0x1c15d15e, 66 },
    { 0xb265d15c, 76 },
    { 0, 0 },
    { 0x1185d161, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb395d16b, 55 },
    { 0, 0 },
    { 0xc625d16d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5d173, 55 },
    { 0x155d173, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa235d179, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc55d17c, 68 },
    { 0x5005d17c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb5d182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5d188, 34 },
    { 0x1235d188, 33 },
    { 0x9075d18a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1855d18d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5d195, 33 },
    { 0, 0 },
    { 0x9635d197, 95 },
    { 0x3c05d198, 33 },
    { 0xffc5d199, 33 },
    { 0x1635d197, 78 },
    { 0x2a35d19a, 98 },
    { 0xeb25d199, 90 },
    { 0, 0 },
    { 0x26c5d19e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7315d1a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d1a7, 33 },
    { 0x84a5d1a7, 96 },
    { 0x4515d1a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5d1af, 90 },
    { 0, 0 },
    { 0x7a5d1b1, 91 },
    { 0xbba5d1b1, 75 },
    { 0xc045d1b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c5d1b6, 78 },
    { 0x4135d1b7, 55 },
    { 0x1d45d1b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa545d1bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5d1c1, 91 },
    { 0, 0 },
    { 0x2235d1c3, 43 },
    { 0, 0 },
    { 0x7f35d1c5, 98 },
    { 0xb4a5d1c6, 95 },
    { 0xac45d1c7, 95 },
    { 0x8175d1c7, 98 },
    { 0x1895d1c8, 90 },
    { 0x3f35d1c5, 34 },
    { 0x1525d1cb, 91 },
    { 0, 0 },
    { 0xea25d1cd, 95 },
    { 0xa65d1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6355d1da, 66 },
    { 0x55d5d1db, 95 },
    { 0x8565d1dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7805d1e0, 36 },
    { 0x5f75d1e0, 98 },
    { 0x14f5d1e0, 81 },
    { 0x2a55d1e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795d1e7, 23 },
    { 0, 0 },
    { 0xfec5d1e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5d1ec, 61 },
    { 0x31c5d1ed, 91 },
    { 0, 0 },
    { 0xf5c5d1ef, 95 },
    { 0xc465d1f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e5d1f3, 59 },
    { 0xafb5d1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc5d1f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb445d1fc, 66 },
    { 0xae55d1fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf75d200, 43 },
    { 0xc405d200, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75d204, 23 },
    { 0x7de5d204, 61 },
    { 0xc7c5d204, 93 },
    { 0, 0 },
    { 0xa775d208, 61 },
    { 0xa25d209, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295d212, 66 },
    { 0x62e5d212, 80 },
    { 0x64e5d214, 61 },
    { 0xab55d214, 33 },
    { 0xd655d214, 95 },
    { 0xb2f5d215, 87 },
    { 0, 0 },
    { 0x8b85d219, 33 },
    { 0, 0 },
    { 0x7275d21b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d5d223, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xac85d226, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5285d229, 23 },
    { 0x3be5d22a, 52 },
    { 0x4605d22b, 61 },
    { 0xdd5d22c, 98 },
    { 0xbd95d22c, 47 },
    { 0, 0 },
    { 0x8235d22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5d233, 61 },
    { 0x5d234, 90 },
    { 0xdaf5d235, 61 },
    { 0x7a35d233, 43 },
    { 0, 0 },
    { 0x475d238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd895d23b, 83 },
    { 0xfa65d23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd335d241, 90 },
    { 0, 0 },
    { 0xe2d5d243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef85d253, 61 },
    { 0xf3d5d254, 61 },
    { 0, 0 },
    { 0x31e5d256, 95 },
    { 0x5e15d257, 87 },
    { 0, 0 },
    { 0x6f5d259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e5d25d, 85 },
    { 0x6a55d25e, 93 },
    { 0, 0 },
    { 0xdfa5d260, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb15d269, 95 },
    { 0x6c45d26a, 51 },
    { 0, 0 },
    { 0xc645d26c, 33 },
    { 0xc515d26d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095d271, 61 },
    { 0xa485d272, 33 },
    { 0, 0 },
    { 0x1d65d274, 91 },
    { 0x5c75d275, 33 },
    { 0, 0 },
    { 0xa465d277, 90 },
    { 0xd4a5d277, 90 },
    { 0, 0 },
    { 0x63a5d27a, 43 },
    { 0, 0 },
    { 0x75b5d27c, 90 },
    { 0x4e65d27c, 84 },
    { 0x84a5d27c, 99 },
    { 0x58a5d27c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc15d282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5d28b, 76 },
    { 0xef75d28b, 76 },
    { 0, 0 },
    { 0x9165d28e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf945d291, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb5d299, 43 },
    { 0xc375d29a, 90 },
    { 0xdcb5d29b, 95 },
    { 0x86b5d29c, 61 },
    { 0x1425d29d, 90 },
    { 0xb2e5d29e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a05d2a3, 61 },
    { 0, 0 },
    { 0x9325d2a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd45d2a8, 43 },
    { 0, 0 },
    { 0x7f05d2aa, 34 },
    { 0x6e75d2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6945d2b0, 61 },
    { 0xa765d2b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c15d2b5, 95 },
    { 0xdf75d2b6, 90 },
    { 0, 0 },
    { 0xd75d2b8, 76 },
    { 0xeaa5d2b9, 68 },
    { 0xcd35d2b8, 53 },
    { 0x58b5d2bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f85d2c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8315d2c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca45d2ca, 97 },
    { 0x4155d2cb, 61 },
    { 0, 0 },
    { 0x755d2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4495d2d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd35d2d8, 85 },
    { 0x61c5d2d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d5d2dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3765d2e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5d2e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635d2e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75d2ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325d2f0, 33 },
    { 0, 0 },
    { 0xaaf5d2f2, 95 },
    { 0x38f5d2f3, 90 },
    { 0x8de5d2f4, 33 },
    { 0xa835d2f3, 67 },
    { 0x65b5d2f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e5d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105d2fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb5d302, 85 },
    { 0x6f25d303, 23 },
    { 0x1005d302, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5d308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25d30c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef5d30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc185d314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9705d317, 66 },
    { 0x2bb5d318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225d320, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f5d323, 68 },
    { 0, 0 },
    { 0x7125d325, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7615d328, 91 },
    { 0, 0 },
    { 0x5595d32a, 84 },
    { 0xf475d32a, 33 },
    { 0, 0 },
    { 0x1ab5d32d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665d332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245d335, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe185d341, 98 },
    { 0x7645d342, 90 },
    { 0x3555d343, 33 },
    { 0xdb75d344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7405d349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455d351, 90 },
    { 0xd555d352, 61 },
    { 0xef65d353, 98 },
    { 0x92c5d354, 85 },
    { 0x1d55d353, 64 },
    { 0x255d356, 90 },
    { 0x72a5d356, 33 },
    { 0x1365d358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d35c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d363, 90 },
    { 0xbf85d363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5d36b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035d371, 90 },
    { 0, 0 },
    { 0xbd05d373, 55 },
    { 0x3d35d374, 56 },
    { 0x265d375, 33 },
    { 0x8925d376, 90 },
    { 0xde05d377, 76 },
    { 0x3a95d378, 23 },
    { 0x4e05d373, 90 },
    { 0xa985d379, 55 },
    { 0x7e95d374, 61 },
    { 0, 0 },
    { 0xc535d37d, 52 },
    { 0, 0 },
    { 0xd505d37f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905d383, 34 },
    { 0xce95d383, 61 },
    { 0x10e5d385, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd5d389, 33 },
    { 0, 0 },
    { 0x9f75d38b, 34 },
    { 0x205d38c, 91 },
    { 0, 0 },
    { 0x8655d38e, 40 },
    { 0x42b5d38f, 98 },
    { 0xeab5d390, 55 },
    { 0x3f45d391, 98 },
    { 0x1445d392, 55 },
    { 0x4575d38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5d398, 100 },
    { 0x54b5d399, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5d3a0, 46 },
    { 0x2bd5d3a0, 90 },
    { 0xb165d3a2, 33 },
    { 0xdde5d3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b5d3aa, 61 },
    { 0x4b25d3ab, 95 },
    { 0xeb45d3ac, 93 },
    { 0xd5f5d3ad, 95 },
    { 0x655d3ac, 90 },
    { 0x29f5d3ad, 33 },
    { 0x57a5d3b0, 33 },
    { 0xfe95d3b1, 93 },
    { 0xa2d5d3b2, 90 },
    { 0x7775d3b3, 61 },
    { 0x7665d3b2, 23 },
    { 0xe795d3b4, 43 },
    { 0x825d3b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d3bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7605d3c0, 84 },
    { 0x2945d3c1, 93 },
    { 0xb635d3c2, 23 },
    { 0x18e5d3c3, 95 },
    { 0x6a75d3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1175d3c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5d3cd, 61 },
    { 0, 0 },
    { 0x4a15d3cf, 76 },
    { 0x5f85d3d0, 87 },
    { 0xcaa5d3cf, 90 },
    { 0, 0 },
    { 0xadd5d3d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e5d3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce45d3da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d5d3de, 90 },
    { 0xbaa5d3df, 55 },
    { 0x48c5d3e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5545d3e4, 34 },
    { 0x7705d3e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95d3e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf35d3eb, 75 },
    { 0x5f85d3ec, 90 },
    { 0xd1f5d3ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda75d3f0, 33 },
    { 0, 0 },
    { 0x8b45d3f2, 33 },
    { 0x125d3f3, 90 },
    { 0xdf45d3f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d65d3f8, 23 },
    { 0x1675d3f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e5d3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215d400, 33 },
    { 0x6a45d401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5d40a, 55 },
    { 0, 0 },
    { 0x5255d40c, 95 },
    { 0, 0 },
    { 0x9b15d40e, 98 },
    { 0xe55d40e, 78 },
    { 0xd205d410, 65 },
    { 0x6ee5d411, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5d417, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc5d420, 33 },
    { 0x4d15d421, 98 },
    { 0x11f5d421, 75 },
    { 0xb1c5d423, 23 },
    { 0x70c5d424, 85 },
    { 0, 0 },
    { 0x4b35d426, 85 },
    { 0x5da5d427, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265d42a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd35d42e, 17 },
    { 0xb6a5d42f, 43 },
    { 0x8ff5d430, 93 },
    { 0x1935d431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5d43e, 55 },
    { 0x9d95d43f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde35d444, 34 },
    { 0x27f5d445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b85d44c, 90 },
    { 0, 0 },
    { 0x2a25d44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5415d451, 23 },
    { 0x3d5d452, 66 },
    { 0x57c5d453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3545d459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1825d461, 90 },
    { 0x6a35d461, 43 },
    { 0x3385d463, 93 },
    { 0, 0 },
    { 0x1a65d465, 95 },
    { 0x29d5d465, 46 },
    { 0x58d5d466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe825d46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625d477, 95 },
    { 0xc1e5d478, 67 },
    { 0x9d15d478, 93 },
    { 0, 0 },
    { 0x1685d47b, 33 },
    { 0x7145d47c, 23 },
    { 0x6e5d47c, 90 },
    { 0x8955d47d, 95 },
    { 0, 0 },
    { 0xfe75d480, 90 },
    { 0xe835d481, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd235d489, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065d48e, 55 },
    { 0x64f5d48f, 43 },
    { 0x16d5d490, 33 },
    { 0, 0 },
    { 0xce85d492, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a5d49b, 95 },
    { 0x915d49c, 92 },
    { 0x4745d49d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b95d4a1, 76 },
    { 0xc365d4a2, 33 },
    { 0x14d5d4a2, 30 },
    { 0xcdb5d4a4, 33 },
    { 0x1495d4a4, 33 },
    { 0x1b85d4a4, 34 },
    { 0x9715d4a7, 90 },
    { 0x4085d4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5d4ab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c5d4b2, 34 },
    { 0x6d95d4b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3585d4b6, 100 },
    { 0xe8c5d4b6, 55 },
    { 0xb4d5d4b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925d4bb, 61 },
    { 0x35a5d4bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e5d4c2, 67 },
    { 0x76d5d4c3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5625d4cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7285d4cf, 55 },
    { 0x3985d4d0, 78 },
    { 0x3555d4d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265d4d6, 77 },
    { 0x7fb5d4d7, 78 },
    { 0x76a5d4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb05d4dc, 76 },
    { 0x6e35d4dd, 23 },
    { 0x5495d4de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb735d4e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e5d4e6, 76 },
    { 0x125d4e7, 90 },
    { 0xac75d4e8, 88 },
    { 0xc275d4e7, 90 },
    { 0xa3c5d4ea, 46 },
    { 0, 0 },
    { 0xf395d4ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e5d4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf825d4f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5d4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc25d4ff, 77 },
    { 0, 0 },
    { 0x76a5d501, 95 },
    { 0x74e5d502, 99 },
    { 0xe635d503, 98 },
    { 0xe825d502, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25d507, 76 },
    { 0x4f75d507, 91 },
    { 0x9935d507, 61 },
    { 0x5c75d50a, 33 },
    { 0xe645d50b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855d512, 90 },
    { 0x1285d513, 34 },
    { 0x24b5d514, 69 },
    { 0, 0 },
    { 0x4605d516, 76 },
    { 0xf45d517, 33 },
    { 0, 0 },
    { 0x55d5d519, 55 },
    { 0, 0 },
    { 0x6ab5d51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665d51e, 93 },
    { 0, 0 },
    { 0x5795d520, 84 },
    { 0x60e5d521, 61 },
    { 0x6575d522, 43 },
    { 0x27c5d521, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f05d527, 52 },
    { 0x725d528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad5d52b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7465d52e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6285d531, 91 },
    { 0x6195d531, 55 },
    { 0x4f35d533, 34 },
    { 0, 0 },
    { 0xac85d535, 90 },
    { 0x6795d535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735d53b, 76 },
    { 0, 0 },
    { 0xe2b5d53d, 61 },
    { 0xde75d53e, 81 },
    { 0x69c5d53e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55d548, 51 },
    { 0xf4e5d549, 43 },
    { 0xbd75d548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15d551, 23 },
    { 0x92e5d552, 23 },
    { 0, 0 },
    { 0xfcb5d554, 95 },
    { 0x30b5d555, 91 },
    { 0, 0 },
    { 0xc0f5d557, 66 },
    { 0x69c5d557, 33 },
    { 0, 0 },
    { 0x4eb5d55a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b85d55d, 23 },
    { 0, 0 },
    { 0xbad5d55f, 78 },
    { 0, 0 },
    { 0x3145d561, 66 },
    { 0xc145d561, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd5d565, 52 },
    { 0xf845d566, 91 },
    { 0x2225d566, 83 },
    { 0x9295d567, 90 },
    { 0x1ce5d569, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5d56f, 91 },
    { 0xea55d570, 78 },
    { 0xb1f5d56f, 55 },
    { 0x5a05d572, 91 },
    { 0x1995d571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d5d576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2205d57a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9415d57e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5d584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc465d588, 43 },
    { 0xfae5d588, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa65d58c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f95d591, 40 },
    { 0, 0 },
    { 0x325d593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35d59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3175d59e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45d5a3, 95 },
    { 0, 0 },
    { 0xae15d5a5, 23 },
    { 0xda5d5a5, 76 },
    { 0x25a5d5a7, 65 },
    { 0xec55d5a7, 84 },
    { 0xc4c5d5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca5d5af, 90 },
    { 0xb905d5b0, 93 },
    { 0x26e5d5b0, 90 },
    { 0x78c5d5b1, 33 },
    { 0x2d55d5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a5d5b6, 23 },
    { 0x2335d5b7, 55 },
    { 0x805d5b8, 43 },
    { 0x6db5d5b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a15d5c4, 52 },
    { 0xa85d5c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3015d5cb, 87 },
    { 0x37d5d5cc, 77 },
    { 0x4b55d5cc, 33 },
    { 0x20b5d5cd, 34 },
    { 0, 0 },
    { 0xe985d5d0, 33 },
    { 0x3165d5d1, 23 },
    { 0x8155d5d0, 81 },
    { 0x625d5d3, 46 },
    { 0x4285d5d1, 81 },
    { 0, 0 },
    { 0x5e85d5d6, 78 },
    { 0x2d5d5d7, 23 },
    { 0x18e5d5d8, 91 },
    { 0x1955d5d9, 34 },
    { 0x6795d5da, 69 },
    { 0x7cb5d5db, 66 },
    { 0x4b55d5da, 33 },
    { 0x52c5d5db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a35d5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945d5ec, 99 },
    { 0xbff5d5ed, 46 },
    { 0x9935d5ee, 88 },
    { 0x2485d5ec, 34 },
    { 0, 0 },
    { 0xa0e5d5f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a5d5f8, 55 },
    { 0, 0 },
    { 0x9bb5d5fa, 90 },
    { 0x8d5d5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa135d600, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85d60c, 61 },
    { 0xf585d60d, 98 },
    { 0x82d5d60e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2535d612, 81 },
    { 0xd275d613, 87 },
    { 0x77a5d614, 23 },
    { 0xabc5d615, 55 },
    { 0x98b5d615, 98 },
    { 0x4f45d613, 95 },
    { 0x2845d618, 33 },
    { 0x6b15d614, 84 },
    { 0x5cf5d613, 84 },
    { 0x43a5d61b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc795d620, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5d628, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca05d62d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1375d630, 95 },
    { 0x8095d631, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5d634, 85 },
    { 0x35d635, 90 },
    { 0xca05d636, 61 },
    { 0x1505d634, 95 },
    { 0xa705d636, 61 },
    { 0x6a55d639, 90 },
    { 0xa665d639, 90 },
    { 0, 0 },
    { 0x7765d63c, 90 },
    { 0, 0 },
    { 0x9de5d63e, 33 },
    { 0, 0 },
    { 0x3075d640, 33 },
    { 0, 0 },
    { 0x78b5d642, 87 },
    { 0xcf55d643, 66 },
    { 0x8a75d644, 61 },
    { 0x1a95d645, 33 },
    { 0x9965d646, 95 },
    { 0xda55d647, 61 },
    { 0, 0 },
    { 0x9a55d649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc5d654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5d657, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c45d65b, 67 },
    { 0, 0 },
    { 0x97f5d65d, 76 },
    { 0, 0 },
    { 0x6bc5d65f, 76 },
    { 0xaa25d660, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d5d669, 33 },
    { 0xef45d66a, 33 },
    { 0, 0 },
    { 0xd565d66c, 87 },
    { 0x6d05d66d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025d670, 95 },
    { 0x7ee5d671, 88 },
    { 0x6e45d670, 90 },
    { 0x2ed5d673, 88 },
    { 0, 0 },
    { 0x7a55d675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae5d678, 90 },
    { 0x7615d679, 23 },
    { 0x10c5d679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5335d67e, 98 },
    { 0x4e05d67f, 55 },
    { 0x665d680, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba45d68e, 99 },
    { 0x5585d68e, 33 },
    { 0x7385d690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7605d694, 95 },
    { 0xe175d695, 38 },
    { 0x17d5d695, 78 },
    { 0x4035d694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c5d6a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b25d6a7, 33 },
    { 0xf1d5d6a7, 77 },
    { 0xc1a5d6a7, 98 },
    { 0x78a5d6a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae5d6ad, 23 },
    { 0xac85d6ae, 23 },
    { 0x7d05d6ad, 46 },
    { 0, 0 },
    { 0x14c5d6b1, 52 },
    { 0x94a5d6b1, 52 },
    { 0xb2d5d6b3, 98 },
    { 0xc935d6b3, 33 },
    { 0x3115d6b5, 80 },
    { 0, 0 },
    { 0x2305d6b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2535d6ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a5d6c0, 23 },
    { 0x5885d6c1, 34 },
    { 0, 0 },
    { 0x1845d6c3, 61 },
    { 0, 0 },
    { 0xb915d6c5, 90 },
    { 0x49b5d6c6, 46 },
    { 0xef5d6c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a75d6cb, 90 },
    { 0xac45d6cc, 75 },
    { 0, 0 },
    { 0x40a5d6ce, 33 },
    { 0x9d95d6cf, 46 },
    { 0x4a55d6d0, 55 },
    { 0x7a95d6cf, 33 },
    { 0x3375d6d0, 90 },
    { 0xebc5d6d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165d6d6, 61 },
    { 0x2935d6d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5d6e0, 33 },
    { 0x1f45d6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf875d6e6, 23 },
    { 0, 0 },
    { 0x8c55d6e8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5d6ee, 43 },
    { 0x7005d6ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a95d6f2, 55 },
    { 0, 0 },
    { 0x2285d6f4, 23 },
    { 0x3115d6f4, 76 },
    { 0x8825d6f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325d6f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5d6fe, 33 },
    { 0, 0 },
    { 0x1275d700, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1905d705, 90 },
    { 0, 0 },
    { 0x7095d707, 90 },
    { 0, 0 },
    { 0x5995d709, 88 },
    { 0x3765d70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6105d714, 23 },
    { 0, 0 },
    { 0x9f25d716, 90 },
    { 0xff05d716, 98 },
    { 0xc4e5d718, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675d71e, 46 },
    { 0xa925d71e, 43 },
    { 0xdb5d720, 98 },
    { 0xeca5d720, 46 },
    { 0, 0 },
    { 0xb185d723, 23 },
    { 0, 0 },
    { 0xf3e5d725, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a45d728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b5d72b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5d72f, 90 },
    { 0, 0 },
    { 0xfd45d731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5d734, 90 },
    { 0, 0 },
    { 0xc675d736, 90 },
    { 0x88c5d737, 55 },
    { 0xb955d738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d73c, 33 },
    { 0xfcb5d73d, 76 },
    { 0, 0 },
    { 0xf2e5d73f, 95 },
    { 0x53a5d73f, 67 },
    { 0, 0 },
    { 0xcad5d742, 90 },
    { 0x3ac5d742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca35d746, 95 },
    { 0x6d65d747, 90 },
    { 0x1335d747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7865d74d, 33 },
    { 0x8e45d74d, 66 },
    { 0, 0 },
    { 0x9555d750, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa5d753, 95 },
    { 0xddb5d753, 98 },
    { 0, 0 },
    { 0x27a5d756, 95 },
    { 0x9d35d756, 40 },
    { 0, 0 },
    { 0x5365d759, 77 },
    { 0x6be5d75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f5d75e, 61 },
    { 0x17d5d75e, 95 },
    { 0x33a5d75f, 61 },
    { 0x8815d75e, 90 },
    { 0, 0 },
    { 0x9245d763, 90 },
    { 0xf985d764, 98 },
    { 0, 0 },
    { 0xce25d766, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca5d76b, 95 },
    { 0xf575d76c, 33 },
    { 0x92c5d76d, 55 },
    { 0, 0 },
    { 0x2065d76f, 61 },
    { 0x2a85d770, 90 },
    { 0xbfa5d771, 40 },
    { 0, 0 },
    { 0x3695d773, 40 },
    { 0xaa75d774, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5d780, 99 },
    { 0, 0 },
    { 0x3455d782, 91 },
    { 0x4805d783, 84 },
    { 0, 0 },
    { 0xf9c5d785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85d78c, 66 },
    { 0x9015d78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d5d791, 99 },
    { 0, 0 },
    { 0x5c55d793, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3405d799, 65 },
    { 0x85f5d79a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65d79e, 33 },
    { 0x1a45d79f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7015d7a5, 55 },
    { 0x45d7a6, 43 },
    { 0x8b05d7a5, 98 },
    { 0xb325d7a8, 78 },
    { 0xe985d7a9, 58 },
    { 0x36e5d7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35d7b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a5d7b4, 33 },
    { 0xccd5d7b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb315d7bc, 100 },
    { 0xbfe5d7bd, 55 },
    { 0x9445d7bc, 55 },
    { 0x6525d7bf, 93 },
    { 0x7675d7be, 75 },
    { 0x455d7c1, 76 },
    { 0xfc25d7c2, 98 },
    { 0xc005d7c1, 23 },
    { 0x3975d7c3, 76 },
    { 0x3135d7c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a35d7c8, 98 },
    { 0x2185d7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5735d7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f05d7d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685d7d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b5d7d7, 33 },
    { 0, 0 },
    { 0xc1c5d7d9, 90 },
    { 0x8df5d7da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd335d7e0, 43 },
    { 0x6175d7e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5535d7ec, 95 },
    { 0x68e5d7ed, 95 },
    { 0xf315d7ed, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd765d7f1, 23 },
    { 0x6395d7f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5d7fa, 33 },
    { 0x8065d7fb, 23 },
    { 0xf645d7fb, 76 },
    { 0x16b5d7fc, 33 },
    { 0x7f45d7fa, 100 },
    { 0xf415d7ff, 98 },
    { 0xb35d800, 95 },
    { 0x5155d800, 91 },
    { 0xb2e5d802, 55 },
    { 0xf4a5d802, 61 },
    { 0xb6d5d804, 61 },
    { 0xaf75d805, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5d808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d80b, 95 },
    { 0x26a5d80c, 33 },
    { 0, 0 },
    { 0x22b5d80e, 61 },
    { 0xe225d80f, 95 },
    { 0, 0 },
    { 0xae45d811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe5d81b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d5d81e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d821, 90 },
    { 0, 0 },
    { 0x6e35d823, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1455d830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be5d83b, 23 },
    { 0, 0 },
    { 0x7c15d83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6715d846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5d849, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5d84e, 78 },
    { 0xbdf5d84e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec5d852, 76 },
    { 0, 0 },
    { 0x1a75d854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5d858, 33 },
    { 0x9295d859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f5d85c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5d861, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095d864, 61 },
    { 0x5f55d864, 76 },
    { 0, 0 },
    { 0xc535d867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4845d86c, 84 },
    { 0x9495d86c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3495d871, 90 },
    { 0x95b5d871, 55 },
    { 0xda65d871, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5d878, 61 },
    { 0x3b25d879, 95 },
    { 0, 0 },
    { 0x4a05d87b, 64 },
    { 0x7495d87b, 76 },
    { 0xa8f5d87d, 90 },
    { 0x7665d87e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef5d881, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9255d885, 61 },
    { 0x11d5d886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a95d88b, 33 },
    { 0xa035d88c, 61 },
    { 0, 0 },
    { 0xa805d88e, 84 },
    { 0x8225d88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9805d892, 33 },
    { 0xa4c5d893, 61 },
    { 0x6775d894, 93 },
    { 0x6615d895, 88 },
    { 0, 0 },
    { 0xcc65d897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5d89b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc5d8a6, 76 },
    { 0x8f85d8a7, 90 },
    { 0x3b35d8a6, 76 },
    { 0x1c75d8a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265d8b4, 46 },
    { 0x79b5d8b5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5d8bb, 98 },
    { 0x7f75d8bc, 55 },
    { 0x5975d8bd, 91 },
    { 0x7e5d8be, 61 },
    { 0xb395d8be, 46 },
    { 0, 0 },
    { 0x6105d8c1, 95 },
    { 0, 0 },
    { 0x1775d8c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad5d8c6, 17 },
    { 0xbac5d8c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a5d8cb, 52 },
    { 0x7bf5d8cb, 90 },
    { 0, 0 },
    { 0x6e05d8ce, 33 },
    { 0x2665d8cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535d8d3, 23 },
    { 0x1fe5d8d4, 91 },
    { 0xa455d8d5, 4 },
    { 0x8aa5d8d5, 87 },
    { 0, 0 },
    { 0x3835d8d8, 84 },
    { 0xd945d8d9, 95 },
    { 0x9d05d8da, 24 },
    { 0x3a75d8d9, 91 },
    { 0xf1e5d8d8, 55 },
    { 0x9c75d8dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef85d8e6, 66 },
    { 0x3245d8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d25d8ed, 33 },
    { 0, 0 },
    { 0xfe45d8ef, 95 },
    { 0x4c55d8f0, 95 },
    { 0x45e5d8f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd655d8fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5d8ff, 33 },
    { 0x1725d900, 66 },
    { 0, 0 },
    { 0xde75d902, 87 },
    { 0xb0d5d903, 61 },
    { 0x69c5d904, 68 },
    { 0x3e75d905, 61 },
    { 0x5f45d904, 87 },
    { 0x9ff5d907, 90 },
    { 0xfa45d904, 61 },
    { 0xb2a5d905, 98 },
    { 0x3125d90a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e5d90d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf5d911, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645d918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe5d91b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195d91e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045d926, 55 },
    { 0xcc45d927, 61 },
    { 0xcee5d928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f5d92e, 46 },
    { 0x4b25d92e, 90 },
    { 0, 0 },
    { 0x9885d931, 40 },
    { 0, 0 },
    { 0xb985d933, 77 },
    { 0x5d15d933, 91 },
    { 0x2fd5d933, 98 },
    { 0xee65d936, 55 },
    { 0xd035d937, 33 },
    { 0, 0 },
    { 0x195d939, 90 },
    { 0, 0 },
    { 0x5035d93b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf5d941, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825d948, 91 },
    { 0, 0 },
    { 0xadd5d94a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a05d950, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615d953, 98 },
    { 0xeef5d954, 78 },
    { 0xe8b5d955, 43 },
    { 0xd1a5d956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe215d95a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa555d95d, 33 },
    { 0x4ef5d95e, 84 },
    { 0x2435d95d, 23 },
    { 0xe785d960, 98 },
    { 0, 0 },
    { 0x1e5d962, 64 },
    { 0, 0 },
    { 0x66a5d964, 50 },
    { 0x1a95d965, 34 },
    { 0xdde5d966, 95 },
    { 0xa295d967, 90 },
    { 0, 0 },
    { 0x38f5d969, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd5d972, 95 },
    { 0x1645d973, 61 },
    { 0x66f5d974, 87 },
    { 0xf915d975, 90 },
    { 0, 0 },
    { 0x2d05d977, 38 },
    { 0x64d5d978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815d987, 34 },
    { 0x2b55d987, 90 },
    { 0, 0 },
    { 0x24d5d98a, 98 },
    { 0xae55d98a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f15d990, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55d995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5d999, 23 },
    { 0x6395d999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a5d9a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d5d9a5, 98 },
    { 0x4645d9a6, 88 },
    { 0, 0 },
    { 0x71e5d9a8, 95 },
    { 0x365d9a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b55d9af, 43 },
    { 0, 0 },
    { 0x7b95d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a65d9b4, 91 },
    { 0xbda5d9b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5d9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65d9bf, 33 },
    { 0, 0 },
    { 0xafc5d9c1, 95 },
    { 0x3325d9c2, 87 },
    { 0x9ac5d9c3, 33 },
    { 0x4335d9c1, 55 },
    { 0, 0 },
    { 0xd8e5d9c6, 23 },
    { 0x4645d9c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5d9cb, 66 },
    { 0xa465d9cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac5d9d0, 23 },
    { 0x82a5d9d0, 61 },
    { 0xf025d9d2, 33 },
    { 0x2a15d9d3, 91 },
    { 0x2e35d9d3, 23 },
    { 0, 0 },
    { 0x8be5d9d6, 34 },
    { 0xd5f5d9d7, 98 },
    { 0x6465d9d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee55d9df, 85 },
    { 0xb815d9df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9445d9e5, 23 },
    { 0xb145d9e5, 95 },
    { 0x90d5d9e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5005d9ef, 33 },
    { 0, 0 },
    { 0x8f85d9f1, 95 },
    { 0x2b45d9f2, 98 },
    { 0x11f5d9f2, 66 },
    { 0x5995d9f4, 90 },
    { 0x5925d9f5, 33 },
    { 0, 0 },
    { 0x9855d9f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a5d9fa, 93 },
    { 0x34e5d9fb, 33 },
    { 0, 0 },
    { 0xb3d5d9fd, 78 },
    { 0xdd45d9fe, 55 },
    { 0, 0 },
    { 0xe325da00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df5da07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa5da0a, 90 },
    { 0x8e05da0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5da0f, 95 },
    { 0, 0 },
    { 0xc2f5da11, 61 },
    { 0x5db5da11, 98 },
    { 0, 0 },
    { 0xdd55da14, 95 },
    { 0x38f5da15, 76 },
    { 0xe125da15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4925da19, 75 },
    { 0x21a5da1a, 91 },
    { 0xc875da1b, 66 },
    { 0xcae5da1b, 98 },
    { 0, 0 },
    { 0xfaf5da1e, 90 },
    { 0, 0 },
    { 0x61f5da20, 34 },
    { 0, 0 },
    { 0x1465da22, 95 },
    { 0x4725da22, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065da28, 65 },
    { 0x5eb5da29, 55 },
    { 0x98b5da2a, 23 },
    { 0xccc5da2b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf75da2e, 53 },
    { 0, 0 },
    { 0xc405da30, 84 },
    { 0xe6e5da31, 55 },
    { 0, 0 },
    { 0xd675da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675da37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de5da3b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff5da3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1705da48, 46 },
    { 0x8cd5da48, 61 },
    { 0x89e5da4a, 33 },
    { 0xb855da4b, 34 },
    { 0xfb55da4b, 33 },
    { 0xd675da4d, 33 },
    { 0xd875da4c, 90 },
    { 0xaaa5da4f, 52 },
    { 0xe555da4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa05da59, 93 },
    { 0xb995da5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c5da5e, 33 },
    { 0xf735da5f, 23 },
    { 0x765da60, 90 },
    { 0x66a5da60, 34 },
    { 0, 0 },
    { 0xc8d5da63, 77 },
    { 0x9a5da63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7235da68, 33 },
    { 0xa15da69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7595da71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5da74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a35da7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b5da7f, 55 },
    { 0xa3b5da80, 23 },
    { 0xc5c5da80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe95da85, 99 },
    { 0x74a5da85, 33 },
    { 0, 0 },
    { 0xef95da88, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c5da8e, 93 },
    { 0xc6c5da8f, 46 },
    { 0xa275da90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25da93, 55 },
    { 0x6bc5da94, 55 },
    { 0xf505da94, 95 },
    { 0x6e55da96, 61 },
    { 0xc455da93, 90 },
    { 0xfa75da97, 90 },
    { 0xa805da99, 61 },
    { 0xf125da9a, 88 },
    { 0xad95da9a, 43 },
    { 0x9015da9a, 90 },
    { 0x81c5da9d, 91 },
    { 0x6105da95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc5daa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5daa8, 80 },
    { 0x2655daa8, 100 },
    { 0, 0 },
    { 0x4785daab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1645daae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d5dab4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5dac1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5dac4, 100 },
    { 0, 0 },
    { 0xfa35dac6, 98 },
    { 0xaaa5dac7, 90 },
    { 0x4015dac8, 95 },
    { 0xfca5dac8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215dacd, 98 },
    { 0x6ac5dace, 33 },
    { 0x9dc5dacd, 24 },
    { 0xe915dad0, 90 },
    { 0x8015dad1, 33 },
    { 0xae45dad1, 17 },
    { 0xf475dad3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875dad6, 98 },
    { 0, 0 },
    { 0xe365dad8, 66 },
    { 0x9a25dad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5dade, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2895dae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1185daed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d85daf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425dafc, 61 },
    { 0xf25dafd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf735db02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a45db05, 46 },
    { 0, 0 },
    { 0x9565db07, 90 },
    { 0x6155db08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115db15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8535db18, 46 },
    { 0x9235db19, 95 },
    { 0x4505db18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f55db1f, 49 },
    { 0, 0 },
    { 0xea35db21, 43 },
    { 0, 0 },
    { 0x1065db23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995db28, 76 },
    { 0x3225db29, 76 },
    { 0, 0 },
    { 0x48f5db2b, 61 },
    { 0, 0 },
    { 0xed15db2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e5db32, 55 },
    { 0x2a55db33, 87 },
    { 0, 0 },
    { 0x9995db35, 34 },
    { 0, 0 },
    { 0x8d5db37, 90 },
    { 0x9335db38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f5db3b, 61 },
    { 0, 0 },
    { 0x36b5db3d, 55 },
    { 0xf965db3d, 76 },
    { 0x7d5db3d, 87 },
    { 0xa6a5db40, 76 },
    { 0xa25db3f, 43 },
    { 0x6fb5db3f, 98 },
    { 0x155db43, 40 },
    { 0x5eb5db40, 67 },
    { 0x45d5db45, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3915db48, 33 },
    { 0x2d85db49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf25db4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b5db52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6095db5b, 90 },
    { 0, 0 },
    { 0x4615db5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245db61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e25db64, 84 },
    { 0, 0 },
    { 0x4a15db66, 78 },
    { 0x4b85db67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5db6f, 76 },
    { 0xc1b5db70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f5db73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc635db79, 61 },
    { 0, 0 },
    { 0xce15db7b, 95 },
    { 0x3ad5db7c, 23 },
    { 0x6a35db7b, 90 },
    { 0x44a5db7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5db81, 69 },
    { 0x6fb5db81, 93 },
    { 0x4105db81, 100 },
    { 0x8e45db82, 61 },
    { 0, 0 },
    { 0x8be5db86, 76 },
    { 0x4c35db86, 33 },
    { 0x82b5db86, 34 },
    { 0x3b65db87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda85db8d, 61 },
    { 0xa355db8e, 33 },
    { 0xf935db8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5db92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1225db97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e5db9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5db9d, 93 },
    { 0x4c15db9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5dba6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a5dba9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa5dbae, 34 },
    { 0x1df5dbae, 98 },
    { 0, 0 },
    { 0x2bd5dbb1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875dbb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c35dbb9, 99 },
    { 0x26d5dbba, 90 },
    { 0x3a65dbba, 23 },
    { 0x7c55dbbc, 52 },
    { 0xfc55dbba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f5dbc1, 90 },
    { 0, 0 },
    { 0x35e5dbc3, 91 },
    { 0xb005dbc3, 77 },
    { 0xd295dbc5, 90 },
    { 0xccd5dbc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb475dbd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b75dbd4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda85dbd9, 61 },
    { 0xa9a5dbda, 61 },
    { 0xe9e5dbda, 33 },
    { 0x5a45dbdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755dbdf, 95 },
    { 0x20f5dbdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f5dbe4, 98 },
    { 0, 0 },
    { 0x25d5dbe6, 23 },
    { 0x8e5dbe7, 76 },
    { 0x8575dbe8, 68 },
    { 0x9245dbe7, 76 },
    { 0x9995dbea, 76 },
    { 0x4d5dbea, 33 },
    { 0x2f65dbe6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c5dbf0, 95 },
    { 0x3915dbf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5dbf5, 99 },
    { 0xaca5dbf6, 33 },
    { 0, 0 },
    { 0xc5e5dbf8, 55 },
    { 0xfdd5dbf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5dbfd, 78 },
    { 0xdbb5dbfe, 68 },
    { 0x89c5dbff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085dc06, 55 },
    { 0, 0 },
    { 0x77b5dc08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5dc0d, 43 },
    { 0, 0 },
    { 0xf4d5dc0f, 34 },
    { 0xd75dc0f, 98 },
    { 0x3525dc11, 76 },
    { 0xd7f5dc12, 69 },
    { 0, 0 },
    { 0x5505dc14, 95 },
    { 0x6615dc15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b5dc1d, 66 },
    { 0x6075dc1d, 98 },
    { 0, 0 },
    { 0x1815dc20, 100 },
    { 0xc305dc21, 88 },
    { 0x51b5dc20, 90 },
    { 0, 0 },
    { 0x1e25dc24, 90 },
    { 0x6365dc24, 90 },
    { 0x4615dc25, 33 },
    { 0, 0 },
    { 0xcfa5dc28, 93 },
    { 0xf025dc28, 61 },
    { 0xac25dc2a, 84 },
    { 0, 0 },
    { 0xc2f5dc2c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5dc2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5dc37, 85 },
    { 0x3245dc37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee5dc3b, 33 },
    { 0xf75dc3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4555dc41, 98 },
    { 0x5d45dc42, 45 },
    { 0xff45dc43, 75 },
    { 0x7c35dc42, 23 },
    { 0xf735dc45, 90 },
    { 0xdc65dc46, 65 },
    { 0x9f05dc47, 90 },
    { 0xda05dc47, 77 },
    { 0x68f5dc49, 33 },
    { 0x7a35dc44, 93 },
    { 0xfe95dc44, 46 },
    { 0x4f35dc4c, 33 },
    { 0xb115dc4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235dc51, 95 },
    { 0x5ab5dc52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335dc55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5dc58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115dc5c, 23 },
    { 0, 0 },
    { 0x80c5dc5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa225dc64, 55 },
    { 0x305dc65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5dc6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965dc70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d5dc74, 95 },
    { 0, 0 },
    { 0x6f45dc76, 61 },
    { 0, 0 },
    { 0x25f5dc78, 90 },
    { 0, 0 },
    { 0xfff5dc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7565dc7d, 90 },
    { 0x35d5dc7e, 46 },
    { 0x1d85dc7e, 55 },
    { 0x8ca5dc80, 53 },
    { 0, 0 },
    { 0x3315dc82, 61 },
    { 0xc035dc82, 61 },
    { 0x6445dc84, 34 },
    { 0x8275dc84, 93 },
    { 0xb4c5dc86, 90 },
    { 0x7935dc85, 76 },
    { 0x87a5dc88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5dc8d, 99 },
    { 0, 0 },
    { 0x8a95dc8f, 90 },
    { 0x9435dc90, 91 },
    { 0x5e45dc90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec95dc95, 51 },
    { 0x86f5dc96, 93 },
    { 0xffb5dc97, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc5dc9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5dc9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5dca5, 91 },
    { 0x5295dca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5dca9, 95 },
    { 0xa035dca9, 90 },
    { 0xed5dcab, 33 },
    { 0x7985dcab, 90 },
    { 0xe705dcaa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b5dcb4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15dcb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd15dcbc, 43 },
    { 0, 0 },
    { 0x6c5dcbe, 61 },
    { 0, 0 },
    { 0x41b5dcc0, 93 },
    { 0xe675dcc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb425dcc4, 90 },
    { 0xdcd5dcc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05dccb, 76 },
    { 0, 0 },
    { 0x1995dccd, 90 },
    { 0xf4c5dccd, 101 },
    { 0x3d15dccf, 53 },
    { 0xd185dccf, 77 },
    { 0, 0 },
    { 0x6095dcd2, 66 },
    { 0x9165dcd2, 67 },
    { 0, 0 },
    { 0xe885dcd5, 33 },
    { 0x83b5dcd6, 33 },
    { 0, 0 },
    { 0xda15dcd8, 61 },
    { 0xf8e5dcd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a45dcdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf865dce2, 55 },
    { 0x4d5dce3, 34 },
    { 0x24e5dce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905dcea, 88 },
    { 0x8c75dceb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5dcee, 95 },
    { 0, 0 },
    { 0x4d65dcf0, 76 },
    { 0x6ad5dcf1, 34 },
    { 0xf865dcf2, 68 },
    { 0, 0 },
    { 0xbf85dcf4, 23 },
    { 0, 0 },
    { 0xd755dcf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e5dcf9, 17 },
    { 0x825dcfa, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c65dd03, 88 },
    { 0, 0 },
    { 0xe145dd05, 67 },
    { 0xdcb5dd06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc85dd0a, 46 },
    { 0x3545dd0a, 90 },
    { 0xbf65dd0c, 87 },
    { 0xe9b5dd0d, 95 },
    { 0x7e05dd0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5dd11, 61 },
    { 0x4415dd11, 95 },
    { 0x8e15dd13, 78 },
    { 0, 0 },
    { 0xa1f5dd15, 91 },
    { 0x3755dd16, 100 },
    { 0, 0 },
    { 0x2c25dd18, 99 },
    { 0, 0 },
    { 0x1e05dd1a, 33 },
    { 0xdfe5dd1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8255dd21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35dd27, 33 },
    { 0, 0 },
    { 0xe615dd29, 93 },
    { 0x2925dd2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7465dd2d, 61 },
    { 0, 0 },
    { 0xb455dd2f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a65dd32, 89 },
    { 0x49e5dd32, 90 },
    { 0x1f25dd34, 76 },
    { 0xbd05dd35, 83 },
    { 0x63e5dd35, 61 },
    { 0xd7f5dd32, 90 },
    { 0x74f5dd32, 98 },
    { 0x45e5dd32, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe85dd3e, 99 },
    { 0x2625dd3e, 81 },
    { 0x4b85dd40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c5dd4a, 61 },
    { 0, 0 },
    { 0x8985dd4c, 40 },
    { 0, 0 },
    { 0xd9b5dd4e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5dd59, 90 },
    { 0xae65dd59, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc5dd61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd45dd66, 98 },
    { 0x80d5dd67, 33 },
    { 0x9e85dd68, 88 },
    { 0xef5dd69, 53 },
    { 0xb4c5dd6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755dd6e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5995dd73, 84 },
    { 0xda65dd74, 61 },
    { 0x8345dd75, 33 },
    { 0x98f5dd76, 98 },
    { 0xc955dd77, 77 },
    { 0xbdf5dd75, 98 },
    { 0, 0 },
    { 0x19e5dd7a, 95 },
    { 0x30d5dd7b, 33 },
    { 0xa8f5dd7b, 66 },
    { 0x6425dd7d, 95 },
    { 0x28f5dd7c, 33 },
    { 0xa175dd7c, 17 },
    { 0x9475dd7d, 34 },
    { 0xff75dd81, 55 },
    { 0x38a5dd82, 95 },
    { 0x8dd5dd81, 90 },
    { 0x6fb5dd84, 95 },
    { 0xa85dd85, 88 },
    { 0xfb85dd85, 98 },
    { 0x7b75dd85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f65dd91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa965dd96, 34 },
    { 0xc8b5dd97, 33 },
    { 0, 0 },
    { 0x8b95dd99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5dd9d, 66 },
    { 0, 0 },
    { 0x1215dd9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba5dda4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b5ddaa, 52 },
    { 0x1f95ddab, 33 },
    { 0x1e15ddab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695ddb1, 23 },
    { 0x8485ddb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf415ddb6, 76 },
    { 0, 0 },
    { 0xbf35ddb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b5ddbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5ddbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da5ddc2, 90 },
    { 0x7875ddc3, 61 },
    { 0x2e15ddc4, 55 },
    { 0, 0 },
    { 0x80e5ddc6, 85 },
    { 0x325ddc7, 68 },
    { 0xbac5ddc6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5ddcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9125ddcf, 23 },
    { 0x32f5ddd0, 85 },
    { 0x5b05ddd1, 61 },
    { 0x6d15ddd2, 43 },
    { 0x9335ddd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735ddd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9445dde1, 23 },
    { 0x5825dde2, 76 },
    { 0xf225dde3, 23 },
    { 0, 0 },
    { 0x1b65dde5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305ddea, 76 },
    { 0, 0 },
    { 0x3b35ddec, 45 },
    { 0xe355dded, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c5ddf3, 90 },
    { 0x9d45ddf4, 95 },
    { 0xaad5ddf4, 96 },
    { 0x5645ddf5, 34 },
    { 0x9b25ddf7, 53 },
    { 0x7455ddf7, 98 },
    { 0x5195ddf4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f5ddfc, 23 },
    { 0xba25ddfd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935de02, 98 },
    { 0xe3a5de03, 23 },
    { 0xde65de04, 76 },
    { 0xcb65de05, 75 },
    { 0xe845de04, 90 },
    { 0xb45de04, 55 },
    { 0x1b15de05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75de0b, 65 },
    { 0xab35de0b, 43 },
    { 0x9265de0c, 99 },
    { 0xe0c5de0e, 61 },
    { 0x7a5de0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa95de15, 33 },
    { 0xa935de16, 67 },
    { 0xe8b5de17, 23 },
    { 0x9795de17, 55 },
    { 0x6c45de19, 98 },
    { 0x8ce5de19, 95 },
    { 0x9905de15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405de25, 90 },
    { 0x4fc5de26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5de29, 23 },
    { 0, 0 },
    { 0x31b5de2b, 43 },
    { 0xb6f5de2c, 23 },
    { 0xf745de2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1155de34, 34 },
    { 0, 0 },
    { 0xaec5de36, 64 },
    { 0, 0 },
    { 0xa975de38, 75 },
    { 0xd1b5de38, 90 },
    { 0, 0 },
    { 0x9a55de3b, 52 },
    { 0xc2d5de3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975de3f, 100 },
    { 0x5635de40, 90 },
    { 0, 0 },
    { 0x9565de42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685de46, 78 },
    { 0, 0 },
    { 0x28f5de48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e5de50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215de53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb575de5c, 84 },
    { 0x1255de5d, 90 },
    { 0, 0 },
    { 0xabf5de5f, 61 },
    { 0, 0 },
    { 0x6a25de61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785de64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e85de6d, 90 },
    { 0xfb55de6e, 93 },
    { 0x6c75de6d, 75 },
    { 0x1dc5de70, 90 },
    { 0, 0 },
    { 0x7c35de72, 46 },
    { 0, 0 },
    { 0xddd5de74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d95de77, 23 },
    { 0xc4d5de78, 61 },
    { 0xdea5de79, 95 },
    { 0x6885de7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x245de7d, 65 },
    { 0xb255de7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95de82, 90 },
    { 0xd4b5de82, 95 },
    { 0, 0 },
    { 0xa875de85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7615de91, 95 },
    { 0, 0 },
    { 0x1135de93, 88 },
    { 0xb205de94, 23 },
    { 0xee55de95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65de9b, 98 },
    { 0x7e45de9c, 61 },
    { 0x7c05de9d, 40 },
    { 0xdb25de9e, 85 },
    { 0, 0 },
    { 0x1c45dea0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925dea3, 95 },
    { 0x6905dea4, 91 },
    { 0x1c65dea5, 90 },
    { 0, 0 },
    { 0x9705dea7, 100 },
    { 0, 0 },
    { 0xee75dea9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35deac, 77 },
    { 0x8a5dead, 98 },
    { 0, 0 },
    { 0x7ab5deaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c5deb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35deb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75deb9, 98 },
    { 0x56e5deb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745dec0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e5dec5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7555dec8, 66 },
    { 0, 0 },
    { 0xbc85deca, 76 },
    { 0x2a75decb, 88 },
    { 0x5a5decc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c5ded0, 66 },
    { 0x3725ded0, 43 },
    { 0xdc25ded2, 90 },
    { 0, 0 },
    { 0x2f55ded4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d5ded8, 53 },
    { 0xeee5ded9, 33 },
    { 0x6255deda, 61 },
    { 0xa9f5ded9, 93 },
    { 0xa505ded9, 34 },
    { 0x96e5dedb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b5dee0, 99 },
    { 0xcd45dee1, 90 },
    { 0xea35dee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5dee6, 33 },
    { 0, 0 },
    { 0xb75dee8, 67 },
    { 0xc445dee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5deef, 61 },
    { 0x9355deef, 46 },
    { 0xf255def1, 33 },
    { 0x34d5def2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd5def5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35def8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465df00, 61 },
    { 0, 0 },
    { 0xf105df02, 43 },
    { 0, 0 },
    { 0xdda5df04, 61 },
    { 0, 0 },
    { 0x6235df06, 81 },
    { 0, 0 },
    { 0xafb5df08, 90 },
    { 0xdf25df08, 61 },
    { 0x9135df0a, 90 },
    { 0, 0 },
    { 0xe3a5df0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415df18, 95 },
    { 0x8255df19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15df1e, 95 },
    { 0xb6e5df1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb5df2a, 61 },
    { 0xaf75df2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b5df2f, 90 },
    { 0, 0 },
    { 0x5275df31, 76 },
    { 0x8f35df32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345df37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e5df3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c5df41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2295df44, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35df4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab5df51, 84 },
    { 0x3655df51, 33 },
    { 0xe265df52, 84 },
    { 0x7fa5df54, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145df58, 90 },
    { 0, 0 },
    { 0xb515df5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd065df5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc5df61, 98 },
    { 0xc155df62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8365df65, 55 },
    { 0x8db5df66, 55 },
    { 0x99a5df66, 100 },
    { 0xa6d5df67, 23 },
    { 0x9c15df66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75df71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95df7c, 95 },
    { 0x2645df7c, 23 },
    { 0xfc5df7e, 76 },
    { 0x8465df7f, 23 },
    { 0x4525df7c, 61 },
    { 0x4375df81, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5df8b, 91 },
    { 0x8045df8c, 43 },
    { 0x1c25df8d, 95 },
    { 0x1635df8c, 93 },
    { 0, 0 },
    { 0xc485df90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8785df97, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315df9a, 68 },
    { 0xf7b5df9b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695df9f, 34 },
    { 0x15f5df9f, 99 },
    { 0x4445dfa1, 66 },
    { 0xbc65dfa2, 93 },
    { 0xb9d5dfa3, 61 },
    { 0xd8f5df9f, 95 },
    { 0xf575dfa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15dfa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6005dfab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25dfb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7065dfb6, 90 },
    { 0xdbc5dfb7, 33 },
    { 0xd8f5dfb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c5dfc0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2525dfc6, 95 },
    { 0, 0 },
    { 0x7b25dfc8, 76 },
    { 0x3915dfc9, 46 },
    { 0xcd95dfca, 99 },
    { 0x4685dfcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095dfcf, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c5dfd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5dfd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d5dfe0, 33 },
    { 0, 0 },
    { 0xff85dfe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa365dfe7, 95 },
    { 0xa795dfe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb765dff5, 91 },
    { 0x9595dff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5dff9, 93 },
    { 0, 0 },
    { 0xda65dffb, 90 },
    { 0xce85dffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba5e000, 98 },
    { 0xa0f5e001, 55 },
    { 0xa2e5e002, 66 },
    { 0xc925e000, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4055e007, 61 },
    { 0x4935e008, 61 },
    { 0xa3c5e007, 90 },
    { 0x11c5e00a, 46 },
    { 0xce65e00a, 61 },
    { 0, 0 },
    { 0x1225e00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9925e015, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e75e01a, 33 },
    { 0x6775e01b, 68 },
    { 0x1575e01a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b5e01f, 55 },
    { 0xe345e01f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e25e029, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe295e034, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb355e03b, 88 },
    { 0x29c5e03b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225e040, 98 },
    { 0, 0 },
    { 0x6d75e042, 100 },
    { 0x5ac5e043, 43 },
    { 0x6955e042, 61 },
    { 0x2c15e045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9775e04b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5e050, 33 },
    { 0, 0 },
    { 0x7985e052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b5e055, 56 },
    { 0x3695e056, 94 },
    { 0, 0 },
    { 0xd3d5e058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd785e05b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd5e05e, 87 },
    { 0x30a5e05e, 61 },
    { 0x4f15e05e, 90 },
    { 0xda55e05e, 93 },
    { 0xd045e061, 84 },
    { 0xb7d5e063, 81 },
    { 0, 0 },
    { 0xc3c5e065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195e06b, 95 },
    { 0x16a5e06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5e06f, 23 },
    { 0xb505e070, 95 },
    { 0xf345e071, 100 },
    { 0x6f25e072, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e5e07f, 90 },
    { 0xfb75e080, 76 },
    { 0x87d5e081, 23 },
    { 0xa4c5e082, 95 },
    { 0, 0 },
    { 0xc565e084, 90 },
    { 0xd975e085, 78 },
    { 0xe635e086, 43 },
    { 0x3095e087, 33 },
    { 0, 0 },
    { 0xf8f5e089, 98 },
    { 0xa965e08a, 80 },
    { 0x8465e08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5e08f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5e093, 61 },
    { 0, 0 },
    { 0x8bb5e095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675e098, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2625e09e, 61 },
    { 0xde95e09f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c85e0a2, 61 },
    { 0x16b5e0a3, 33 },
    { 0xbcb5e0a2, 33 },
    { 0, 0 },
    { 0x92b5e0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd145e0ab, 98 },
    { 0, 0 },
    { 0x83c5e0ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d5e0b4, 55 },
    { 0x85a5e0b4, 95 },
    { 0xaf45e0b6, 33 },
    { 0xcc55e0b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7835e0ba, 61 },
    { 0, 0 },
    { 0x6095e0bc, 34 },
    { 0xfc95e0bd, 95 },
    { 0xc245e0be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee5e0c3, 95 },
    { 0xf305e0c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b85e0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f5e0ca, 34 },
    { 0x8af5e0cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5e0d2, 61 },
    { 0, 0 },
    { 0x8635e0d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1075e0d8, 23 },
    { 0, 0 },
    { 0xdde5e0da, 36 },
    { 0x4c35e0da, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc5e0df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b5e0e5, 98 },
    { 0xcc45e0e6, 99 },
    { 0xfdc5e0e7, 76 },
    { 0xb895e0e8, 23 },
    { 0xcdc5e0e9, 55 },
    { 0x6445e0e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5e0f1, 17 },
    { 0xe895e0f2, 95 },
    { 0x4815e0f3, 98 },
    { 0xc345e0f3, 66 },
    { 0xb385e0f4, 99 },
    { 0, 0 },
    { 0x5e25e0f7, 98 },
    { 0x1125e0f8, 33 },
    { 0xa6f5e0f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145e0fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb615e0ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5e104, 61 },
    { 0, 0 },
    { 0x11f5e106, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5e10b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa845e10e, 91 },
    { 0xeb65e10e, 100 },
    { 0x56c5e110, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5e116, 84 },
    { 0x8675e116, 33 },
    { 0x9225e118, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e11b, 61 },
    { 0xad45e11c, 33 },
    { 0, 0 },
    { 0xe655e11e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee5e123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af5e12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f75e135, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25e13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f85e13f, 90 },
    { 0x8bc5e13f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a5e146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15e152, 33 },
    { 0, 0 },
    { 0x1595e154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e5e15a, 55 },
    { 0xad95e15b, 33 },
    { 0, 0 },
    { 0xf0f5e15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a75e163, 99 },
    { 0x8685e163, 88 },
    { 0x1ef5e164, 69 },
    { 0x7b75e165, 98 },
    { 0xbc35e167, 87 },
    { 0xe235e164, 61 },
    { 0xaf05e166, 90 },
    { 0xaac5e16a, 55 },
    { 0xf0c5e16b, 76 },
    { 0x7ad5e16a, 93 },
    { 0x4745e166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b95e172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e75e175, 33 },
    { 0, 0 },
    { 0x8835e177, 33 },
    { 0xad5e178, 98 },
    { 0x7105e179, 61 },
    { 0xbca5e179, 17 },
    { 0x97a5e17b, 33 },
    { 0x7745e17b, 55 },
    { 0x3195e17a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5e186, 61 },
    { 0x7195e186, 33 },
    { 0x7d85e188, 84 },
    { 0xf25e189, 33 },
    { 0xa4a5e188, 90 },
    { 0, 0 },
    { 0xd6a5e18c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a95e190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345e194, 61 },
    { 0x94d5e195, 98 },
    { 0x20b5e196, 61 },
    { 0xf735e194, 95 },
    { 0xb875e198, 90 },
    { 0x6795e199, 52 },
    { 0xc1a5e19a, 61 },
    { 0x1515e19b, 23 },
    { 0, 0 },
    { 0xb0f5e19d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd155e1a1, 98 },
    { 0xafc5e1a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7995e1a7, 95 },
    { 0xed65e1a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5e1ab, 95 },
    { 0x47e5e1ac, 85 },
    { 0xc655e1ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825e1b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe5e1ba, 95 },
    { 0, 0 },
    { 0x2ea5e1bc, 33 },
    { 0, 0 },
    { 0x4725e1be, 55 },
    { 0x9af5e1bf, 33 },
    { 0x5cb5e1c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5e1c4, 88 },
    { 0x6405e1c5, 85 },
    { 0x8645e1c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5e1cb, 23 },
    { 0x34b5e1cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e5e1d4, 78 },
    { 0x9035e1d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b5e1de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475e1e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2125e1e6, 23 },
    { 0x64a5e1e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e1ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7695e1f2, 61 },
    { 0xe9b5e1f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5e1f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285e1fa, 46 },
    { 0xaf5e1fb, 55 },
    { 0x1935e1fc, 61 },
    { 0, 0 },
    { 0xb945e1fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5e209, 23 },
    { 0x6de5e20a, 95 },
    { 0xda95e20a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d5e210, 84 },
    { 0x91c5e211, 100 },
    { 0xce85e212, 23 },
    { 0x92e5e210, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e35e218, 23 },
    { 0xa6c5e218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa305e21c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc175e21f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff45e223, 95 },
    { 0, 0 },
    { 0xb6c5e225, 98 },
    { 0xead5e225, 66 },
    { 0x78b5e225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55e22a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35e237, 43 },
    { 0, 0 },
    { 0x68f5e239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5e240, 52 },
    { 0, 0 },
    { 0xf435e242, 61 },
    { 0x8885e242, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c95e246, 33 },
    { 0x4bb5e247, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95e24d, 33 },
    { 0, 0 },
    { 0xead5e24f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe515e252, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe95e25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3095e25f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5e265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a85e26c, 99 },
    { 0x1f75e26c, 33 },
    { 0, 0 },
    { 0x7e15e26f, 78 },
    { 0x7545e26f, 76 },
    { 0x5a5e271, 98 },
    { 0xb245e272, 55 },
    { 0, 0 },
    { 0xa3e5e274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7705e277, 23 },
    { 0xa425e278, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa85e27c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc965e27f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805e284, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075e28a, 90 },
    { 0xc655e28b, 95 },
    { 0, 0 },
    { 0x5e65e28d, 98 },
    { 0x8f65e28d, 46 },
    { 0x5325e28f, 98 },
    { 0x7585e290, 33 },
    { 0, 0 },
    { 0x1165e292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5065e299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb225e29c, 43 },
    { 0xd945e29d, 100 },
    { 0xccb5e29e, 91 },
    { 0xfe25e29f, 90 },
    { 0x1805e29f, 95 },
    { 0x62b5e2a1, 67 },
    { 0x8be5e2a2, 23 },
    { 0, 0 },
    { 0x435e2a4, 95 },
    { 0x8065e2a5, 98 },
    { 0, 0 },
    { 0xb4a5e2a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a5e2ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5e2b0, 43 },
    { 0xe895e2b1, 55 },
    { 0x3db5e2b2, 76 },
    { 0xc025e2b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005e2b6, 34 },
    { 0, 0 },
    { 0x1935e2b8, 34 },
    { 0x35b5e2b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2165e2c9, 90 },
    { 0, 0 },
    { 0x18d5e2cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f5e2cf, 46 },
    { 0, 0 },
    { 0xd1c5e2d1, 88 },
    { 0x8fe5e2d2, 88 },
    { 0xc115e2d2, 95 },
    { 0xfb85e2d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7795e2d7, 90 },
    { 0x7f15e2d7, 61 },
    { 0xa5e5e2d9, 90 },
    { 0, 0 },
    { 0x4d15e2db, 52 },
    { 0, 0 },
    { 0x6045e2dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5755e2e1, 33 },
    { 0, 0 },
    { 0x8685e2e3, 33 },
    { 0, 0 },
    { 0x37b5e2e5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe05e2e9, 23 },
    { 0, 0 },
    { 0xc6c5e2eb, 98 },
    { 0x4675e2ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205e2ef, 24 },
    { 0xaea5e2f0, 78 },
    { 0x7755e2f1, 34 },
    { 0x9e45e2f1, 98 },
    { 0, 0 },
    { 0x3725e2f4, 98 },
    { 0xc0f5e2f4, 95 },
    { 0xac05e2f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe865e2fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe25e2fd, 23 },
    { 0xc75e2fe, 33 },
    { 0x4b75e2fd, 100 },
    { 0xee75e300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb495e309, 77 },
    { 0x90f5e309, 23 },
    { 0, 0 },
    { 0xbc65e30c, 95 },
    { 0x5325e30d, 61 },
    { 0x1bd5e30c, 90 },
    { 0, 0 },
    { 0xe765e310, 61 },
    { 0x6f5e311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb5e315, 52 },
    { 0, 0 },
    { 0xfb15e317, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e31a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da5e31e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a5e322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e35e32b, 98 },
    { 0x1d05e32c, 33 },
    { 0x15a5e32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f5e330, 76 },
    { 0x9dd5e331, 77 },
    { 0xfde5e331, 23 },
    { 0xabe5e333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf35e337, 55 },
    { 0, 0 },
    { 0x1075e339, 98 },
    { 0xdd55e33a, 23 },
    { 0, 0 },
    { 0x5195e33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625e341, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f5e345, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4625e349, 61 },
    { 0, 0 },
    { 0x18e5e34b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d75e34f, 87 },
    { 0xd775e34f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a45e353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a45e35b, 84 },
    { 0xe735e35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc275e362, 23 },
    { 0, 0 },
    { 0x5e85e364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5e367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e36b, 76 },
    { 0xfd75e36b, 61 },
    { 0, 0 },
    { 0xda05e36e, 95 },
    { 0, 0 },
    { 0x2815e370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada5e379, 38 },
    { 0xbe65e37a, 66 },
    { 0x7925e37b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355e381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f5e386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6395e389, 98 },
    { 0x4195e38a, 95 },
    { 0, 0 },
    { 0x1b5e38c, 77 },
    { 0, 0 },
    { 0xce75e38e, 34 },
    { 0, 0 },
    { 0x2745e390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b5e393, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa5e396, 76 },
    { 0, 0 },
    { 0x3675e398, 88 },
    { 0x93c5e399, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c5e39c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa975e39f, 77 },
    { 0xa705e39f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6025e3a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd045e3a6, 90 },
    { 0xe225e3a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95e3aa, 61 },
    { 0xb5d5e3ab, 33 },
    { 0xb395e3ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf045e3b0, 43 },
    { 0, 0 },
    { 0x5a75e3b2, 61 },
    { 0x73e5e3b3, 23 },
    { 0x8e25e3b4, 34 },
    { 0, 0 },
    { 0xd295e3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765e3bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e5e3c0, 68 },
    { 0x4845e3c1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85e3c8, 95 },
    { 0x67f5e3c9, 51 },
    { 0, 0 },
    { 0xa25e3cb, 95 },
    { 0x75f5e3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9385e3cf, 33 },
    { 0, 0 },
    { 0xa545e3d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e5e3d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec5e3d9, 61 },
    { 0x2255e3d9, 90 },
    { 0x8b65e3db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e55e3e5, 84 },
    { 0xb595e3e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e3e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715e3f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d85e3f3, 33 },
    { 0, 0 },
    { 0x6875e3f5, 55 },
    { 0x6685e3f6, 84 },
    { 0x61b5e3f6, 36 },
    { 0x4335e3f6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e5e3fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005e400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf45e404, 95 },
    { 0xfd45e405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe5e408, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8525e40d, 78 },
    { 0x1895e40e, 87 },
    { 0xaa85e40f, 61 },
    { 0xf185e410, 76 },
    { 0x8bc5e40d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b5e417, 76 },
    { 0x5e05e417, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd5e41e, 100 },
    { 0xdbe5e41e, 68 },
    { 0x8505e41e, 34 },
    { 0x5585e421, 61 },
    { 0, 0 },
    { 0xb315e423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde55e429, 61 },
    { 0, 0 },
    { 0x9525e42b, 95 },
    { 0x9845e42b, 93 },
    { 0, 0 },
    { 0x3fc5e42e, 95 },
    { 0x82a5e42f, 85 },
    { 0x55e42f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd765e440, 99 },
    { 0xe935e440, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985e446, 95 },
    { 0x7545e446, 33 },
    { 0x17e5e448, 90 },
    { 0xd645e446, 90 },
    { 0xc5b5e446, 23 },
    { 0x95e5e44b, 61 },
    { 0xe085e44c, 76 },
    { 0x31e5e44d, 76 },
    { 0x6ca5e44d, 23 },
    { 0x8f25e44e, 93 },
    { 0x7b45e450, 81 },
    { 0x9c75e450, 95 },
    { 0x8585e451, 91 },
    { 0, 0 },
    { 0x7da5e454, 66 },
    { 0, 0 },
    { 0xb675e456, 23 },
    { 0xd6c5e457, 66 },
    { 0, 0 },
    { 0xca75e459, 93 },
    { 0x6835e45a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865e45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4185e464, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c95e469, 90 },
    { 0, 0 },
    { 0x2ac5e46b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815e46f, 90 },
    { 0x1b5e46f, 83 },
    { 0, 0 },
    { 0xe7c5e472, 98 },
    { 0xbb25e473, 88 },
    { 0, 0 },
    { 0xadf5e475, 90 },
    { 0, 0 },
    { 0xc3a5e477, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835e481, 98 },
    { 0xe445e481, 33 },
    { 0x2cd5e483, 95 },
    { 0xa9a5e481, 90 },
    { 0, 0 },
    { 0x7885e486, 55 },
    { 0x8665e487, 90 },
    { 0, 0 },
    { 0x7d45e489, 91 },
    { 0, 0 },
    { 0x4d75e48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d15e491, 91 },
    { 0, 0 },
    { 0xcec5e493, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6965e496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525e49c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8335e4a1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525e4a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e5e4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a85e4ae, 33 },
    { 0x8595e4ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5e4b2, 61 },
    { 0xa0f5e4b2, 61 },
    { 0xd055e4b2, 76 },
    { 0, 0 },
    { 0xf245e4b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c5e4b9, 61 },
    { 0x6bd5e4ba, 43 },
    { 0, 0 },
    { 0x72a5e4bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef35e4c0, 76 },
    { 0xf915e4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e4c4, 17 },
    { 0x8395e4c5, 33 },
    { 0, 0 },
    { 0x6d75e4c7, 90 },
    { 0x3035e4c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885e4cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5e4ce, 33 },
    { 0x79f5e4cf, 90 },
    { 0xa795e4d0, 95 },
    { 0x7e35e4ce, 33 },
    { 0xaa65e4ce, 93 },
    { 0xd2c5e4d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205e4d6, 23 },
    { 0x9ec5e4d7, 61 },
    { 0x25e4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8695e4db, 61 },
    { 0x3b5e4dc, 98 },
    { 0x8195e4dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d5e4e0, 23 },
    { 0xd5a5e4e1, 98 },
    { 0x5215e4e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e4e8, 46 },
    { 0x5b55e4e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5e4f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5e4fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2395e4fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45e502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe85e50a, 33 },
    { 0x7eb5e50a, 84 },
    { 0x6d55e50c, 68 },
    { 0x1c95e50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5e513, 90 },
    { 0xa7c5e513, 98 },
    { 0, 0 },
    { 0xb575e516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f5e519, 90 },
    { 0, 0 },
    { 0x6455e51b, 33 },
    { 0x2355e51c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf5e51f, 55 },
    { 0, 0 },
    { 0x9355e521, 90 },
    { 0x7175e522, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d5e525, 90 },
    { 0x9b35e525, 55 },
    { 0xb095e526, 43 },
    { 0x65f5e525, 84 },
    { 0xa875e529, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e52f, 95 },
    { 0, 0 },
    { 0xbd95e531, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd215e535, 33 },
    { 0x3165e536, 76 },
    { 0x2565e535, 90 },
    { 0xc855e538, 61 },
    { 0x4e55e539, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5e53e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5e542, 55 },
    { 0x7ea5e543, 23 },
    { 0x4b25e544, 66 },
    { 0xa695e543, 100 },
    { 0xbc45e542, 61 },
    { 0xaee5e547, 33 },
    { 0xcb15e546, 46 },
    { 0xc435e543, 23 },
    { 0x2c75e54a, 98 },
    { 0xb675e54a, 96 },
    { 0xc9d5e54b, 43 },
    { 0x2a5e54d, 23 },
    { 0x655e54e, 76 },
    { 0x15e549, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb65e552, 98 },
    { 0, 0 },
    { 0xf2c5e554, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d5e557, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac5e55a, 52 },
    { 0xc4e5e55b, 100 },
    { 0x7ad5e55b, 55 },
    { 0xc295e55d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe065e561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f45e56a, 76 },
    { 0x2e75e56b, 33 },
    { 0xc0a5e56b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd05e572, 88 },
    { 0, 0 },
    { 0x14f5e574, 33 },
    { 0, 0 },
    { 0x3895e576, 46 },
    { 0x3815e576, 33 },
    { 0x47c5e576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e57c, 33 },
    { 0xd215e57d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc215e581, 95 },
    { 0, 0 },
    { 0xc0a5e583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5e58f, 55 },
    { 0x6ae5e590, 93 },
    { 0x7195e590, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a5e596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a5e599, 52 },
    { 0x31f5e59a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c5e59e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd445e5a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25e5a7, 33 },
    { 0x6885e5a8, 61 },
    { 0x3c05e5a8, 33 },
    { 0x2af5e5aa, 90 },
    { 0xbf95e5ab, 90 },
    { 0x8c65e5aa, 95 },
    { 0x3785e5ad, 87 },
    { 0x3065e5aa, 23 },
    { 0x1115e5af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865e5b2, 34 },
    { 0x94c5e5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e5e5bb, 55 },
    { 0, 0 },
    { 0xf925e5bd, 43 },
    { 0xe5e5e5be, 93 },
    { 0xc2c5e5bf, 90 },
    { 0x6a35e5bf, 99 },
    { 0, 0 },
    { 0x2195e5c2, 46 },
    { 0xe035e5c2, 61 },
    { 0xa2d5e5c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd5e5c9, 90 },
    { 0, 0 },
    { 0x32d5e5cb, 61 },
    { 0, 0 },
    { 0xd275e5cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa505e5d4, 95 },
    { 0xc955e5d5, 90 },
    { 0xb965e5d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f5e5dd, 33 },
    { 0x50d5e5de, 23 },
    { 0x1535e5de, 83 },
    { 0, 0 },
    { 0x8ee5e5e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd775e5e5, 55 },
    { 0, 0 },
    { 0xa645e5e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5585e5f1, 23 },
    { 0x645e5f2, 33 },
    { 0x1e65e5f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35e5f9, 61 },
    { 0xcfb5e5f9, 95 },
    { 0x9505e5f9, 61 },
    { 0x7d15e5fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a5e604, 93 },
    { 0, 0 },
    { 0x9b15e606, 43 },
    { 0x8375e607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5e60e, 91 },
    { 0x9585e60f, 52 },
    { 0xa875e60e, 61 },
    { 0x3b65e60f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5e614, 90 },
    { 0, 0 },
    { 0x7c15e616, 34 },
    { 0, 0 },
    { 0x8da5e618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc45e623, 92 },
    { 0x79b5e624, 43 },
    { 0x8ee5e625, 81 },
    { 0xe0d5e626, 33 },
    { 0x9cf5e625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5e62b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865e62e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5e632, 23 },
    { 0xfa95e633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf85e636, 100 },
    { 0x425e636, 95 },
    { 0, 0 },
    { 0x9345e639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6275e642, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c15e645, 34 },
    { 0, 0 },
    { 0xbdc5e647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a5e64a, 23 },
    { 0xac05e64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde05e64e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b5e651, 93 },
    { 0x6045e652, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6735e659, 98 },
    { 0x3855e65a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b5e65d, 98 },
    { 0xcb75e65e, 90 },
    { 0x7515e65e, 23 },
    { 0xf095e660, 15 },
    { 0xc1d5e661, 55 },
    { 0x14a5e662, 100 },
    { 0x4865e663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1365e666, 90 },
    { 0, 0 },
    { 0x67a5e668, 90 },
    { 0xe85e669, 43 },
    { 0xb4b5e66a, 55 },
    { 0xb6c5e66b, 90 },
    { 0xec95e66c, 98 },
    { 0x72b5e66b, 33 },
    { 0xbab5e66e, 98 },
    { 0, 0 },
    { 0xba55e670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf595e673, 90 },
    { 0xb965e673, 90 },
    { 0xa125e674, 31 },
    { 0x5635e676, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035e679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155e67d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5e681, 80 },
    { 0, 0 },
    { 0x3ea5e683, 55 },
    { 0x6ad5e684, 23 },
    { 0x70e5e684, 19 },
    { 0x4f65e686, 43 },
    { 0xf8c5e685, 98 },
    { 0xc005e687, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345e693, 67 },
    { 0x3cb5e694, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205e697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e5e69c, 66 },
    { 0x9a95e69d, 101 },
    { 0x3555e69c, 43 },
    { 0x4825e69f, 33 },
    { 0x7b75e6a0, 52 },
    { 0x47f5e69f, 33 },
    { 0, 0 },
    { 0x4e65e6a3, 76 },
    { 0x5e5e6a4, 98 },
    { 0xd285e6a4, 43 },
    { 0xccb5e6a5, 90 },
    { 0x2a75e6a7, 78 },
    { 0x32e5e6a7, 84 },
    { 0xb9f5e6a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c5e6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5e6b6, 90 },
    { 0xb265e6b7, 95 },
    { 0, 0 },
    { 0x5615e6b9, 33 },
    { 0xece5e6b9, 91 },
    { 0xafe5e6bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e6c0, 34 },
    { 0xf665e6c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5e6c6, 55 },
    { 0, 0 },
    { 0x1c55e6c8, 23 },
    { 0xb915e6c9, 17 },
    { 0xed65e6c9, 76 },
    { 0x2c75e6cb, 61 },
    { 0xd9b5e6cc, 98 },
    { 0x7ac5e6cd, 95 },
    { 0, 0 },
    { 0x4c25e6cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6195e6d2, 84 },
    { 0x125e6d3, 90 },
    { 0xb005e6d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3845e6e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5e6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db5e6e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a5e6ed, 33 },
    { 0x1355e6ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e5e6f2, 98 },
    { 0x5b55e6f2, 90 },
    { 0x79c5e6f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9535e706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b75e70a, 76 },
    { 0, 0 },
    { 0xe3c5e70c, 23 },
    { 0x1955e70d, 61 },
    { 0x4555e70d, 91 },
    { 0x14b5e70c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b5e718, 33 },
    { 0, 0 },
    { 0x43f5e71a, 95 },
    { 0xa305e71a, 52 },
    { 0x5e5e71a, 99 },
    { 0xf185e71d, 67 },
    { 0x9d55e71e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05e722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc625e725, 85 },
    { 0x2565e726, 46 },
    { 0x4e35e725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615e72b, 95 },
    { 0x6a25e72c, 85 },
    { 0x87b5e72d, 98 },
    { 0xf495e72e, 42 },
    { 0, 0 },
    { 0x1905e730, 98 },
    { 0x2aa5e731, 52 },
    { 0xd715e730, 33 },
    { 0xefe5e732, 95 },
    { 0x3205e734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a85e738, 61 },
    { 0x68c5e739, 67 },
    { 0x9955e739, 88 },
    { 0xf635e73b, 61 },
    { 0x2d65e73b, 76 },
    { 0xdac5e73b, 34 },
    { 0x5b35e73e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5e741, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5e746, 90 },
    { 0x5e15e747, 33 },
    { 0x8005e748, 95 },
    { 0xfe15e749, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5e750, 90 },
    { 0x7ae5e751, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf55e754, 91 },
    { 0xcf45e755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15e758, 2 },
    { 0xaad5e759, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325e75d, 85 },
    { 0, 0 },
    { 0x1625e75f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5e764, 23 },
    { 0xa5e5e765, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a5e76f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8675e773, 95 },
    { 0x9eb5e774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b05e779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e77e, 61 },
    { 0x7825e77f, 88 },
    { 0, 0 },
    { 0x3a35e781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7545e785, 55 },
    { 0, 0 },
    { 0x3655e787, 66 },
    { 0, 0 },
    { 0x155e789, 23 },
    { 0xa5f5e789, 46 },
    { 0xcbb5e78a, 46 },
    { 0x6b85e78c, 98 },
    { 0x6275e78d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35e791, 23 },
    { 0xa905e792, 93 },
    { 0x2105e792, 55 },
    { 0x3c25e794, 43 },
    { 0xc815e793, 85 },
    { 0x6be5e796, 61 },
    { 0x8305e791, 95 },
    { 0x2e85e798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5e79c, 43 },
    { 0xb435e79d, 85 },
    { 0x8675e79c, 90 },
    { 0, 0 },
    { 0xdaf5e7a0, 33 },
    { 0x2625e7a1, 78 },
    { 0x3eb5e7a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c5e7a8, 23 },
    { 0x4c05e7a9, 95 },
    { 0xb3c5e7a9, 23 },
    { 0xdc95e7ab, 95 },
    { 0x76d5e7a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5e7c0, 76 },
    { 0, 0 },
    { 0x7605e7c2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05e7c8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d85e7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9955e7d7, 23 },
    { 0x91e5e7d7, 91 },
    { 0x385e7d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5e7df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6985e7e3, 84 },
    { 0x93d5e7e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925e7f8, 52 },
    { 0x5b85e7f8, 34 },
    { 0x9d05e7fa, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d85e7fd, 90 },
    { 0, 0 },
    { 0x28d5e7ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5e807, 90 },
    { 0x97b5e808, 93 },
    { 0xd6d5e808, 93 },
    { 0xb3f5e80a, 61 },
    { 0x3015e80a, 46 },
    { 0x1bc5e80c, 23 },
    { 0x5aa5e80d, 99 },
    { 0x56a5e807, 34 },
    { 0x4115e80f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c5e812, 34 },
    { 0x8ee5e812, 98 },
    { 0xf0b5e814, 81 },
    { 0xb7d5e812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1815e819, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f5e820, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185e823, 55 },
    { 0x7cd5e824, 33 },
    { 0x8865e825, 95 },
    { 0xb705e825, 85 },
    { 0x5d55e824, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f5e82d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb25e834, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a65e837, 90 },
    { 0x2c5e837, 90 },
    { 0xfd15e839, 98 },
    { 0x4b25e838, 69 },
    { 0x845e83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb5e83e, 68 },
    { 0x4d85e83f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af5e842, 61 },
    { 0xfac5e843, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5e84c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1475e853, 33 },
    { 0x1285e854, 75 },
    { 0x8bf5e855, 33 },
    { 0, 0 },
    { 0x61a5e857, 90 },
    { 0x9425e857, 33 },
    { 0x3d85e859, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb175e85c, 90 },
    { 0xf785e85c, 61 },
    { 0x4845e85d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5e866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b5e869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885e86d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd115e873, 99 },
    { 0x6e15e874, 61 },
    { 0x7675e875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5e879, 46 },
    { 0x6bc5e879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c55e87e, 93 },
    { 0xaf25e87f, 43 },
    { 0x5005e880, 23 },
    { 0, 0 },
    { 0xc475e882, 91 },
    { 0xb695e883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5e88a, 46 },
    { 0x3c85e88a, 34 },
    { 0xba95e88a, 98 },
    { 0, 0 },
    { 0x5ff5e88e, 33 },
    { 0x3045e88f, 98 },
    { 0, 0 },
    { 0x2e75e891, 48 },
    { 0xbf55e892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745e899, 85 },
    { 0xefa5e89a, 98 },
    { 0x2215e89b, 23 },
    { 0, 0 },
    { 0x445e89d, 61 },
    { 0x7eb5e89e, 100 },
    { 0xe1f5e89f, 68 },
    { 0x5435e8a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3935e8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5e8b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5e8b7, 33 },
    { 0x62c5e8b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905e8be, 98 },
    { 0x3d45e8bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635e8c2, 68 },
    { 0xe3d5e8c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675e8c8, 100 },
    { 0x1ec5e8c9, 61 },
    { 0x54f5e8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695e8ce, 92 },
    { 0xa715e8ce, 98 },
    { 0x1f05e8d0, 33 },
    { 0, 0 },
    { 0x4575e8d2, 61 },
    { 0x9b5e8d2, 98 },
    { 0x1d75e8d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff75e8d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e15e8dc, 55 },
    { 0, 0 },
    { 0x9dd5e8de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc15e8e1, 33 },
    { 0x8405e8e2, 61 },
    { 0x8c05e8e3, 61 },
    { 0x8db5e8e2, 98 },
    { 0, 0 },
    { 0xa305e8e6, 55 },
    { 0, 0 },
    { 0xafb5e8e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a5e8ed, 91 },
    { 0x65b5e8ee, 93 },
    { 0, 0 },
    { 0xd3d5e8f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5e8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d5e8f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e8fc, 61 },
    { 0, 0 },
    { 0xc025e8fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae05e904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x235e907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a5e90e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4465e916, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5e923, 64 },
    { 0x3565e924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935e927, 81 },
    { 0x81a5e927, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55e92d, 34 },
    { 0, 0 },
    { 0x2f85e92f, 47 },
    { 0, 0 },
    { 0x9075e931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95e936, 51 },
    { 0, 0 },
    { 0x7565e938, 83 },
    { 0, 0 },
    { 0xa975e93a, 93 },
    { 0, 0 },
    { 0x23d5e93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195e940, 52 },
    { 0xae65e941, 76 },
    { 0xc4f5e940, 33 },
    { 0x9255e943, 52 },
    { 0, 0 },
    { 0x5975e945, 33 },
    { 0, 0 },
    { 0x6965e947, 96 },
    { 0x31c5e948, 90 },
    { 0xe065e949, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825e94c, 69 },
    { 0x8e5e94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635e951, 100 },
    { 0xe035e952, 93 },
    { 0x51d5e952, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d5e956, 33 },
    { 0, 0 },
    { 0x29e5e958, 61 },
    { 0xcd55e958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215e95f, 98 },
    { 0xe825e960, 61 },
    { 0, 0 },
    { 0xff95e962, 61 },
    { 0xfac5e963, 52 },
    { 0x22a5e964, 52 },
    { 0xf9b5e963, 61 },
    { 0x7475e966, 23 },
    { 0, 0 },
    { 0xbff5e968, 55 },
    { 0, 0 },
    { 0x12f5e96a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe135e96d, 61 },
    { 0x6d05e96e, 61 },
    { 0x9df5e96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd445e972, 38 },
    { 0x6605e973, 75 },
    { 0x1c75e974, 90 },
    { 0xfa35e974, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a5e978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525e981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55e984, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5e987, 46 },
    { 0, 0 },
    { 0xb955e989, 77 },
    { 0, 0 },
    { 0x8f45e98b, 98 },
    { 0x3475e98c, 95 },
    { 0x8225e98d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa5e990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf895e994, 95 },
    { 0x3585e995, 90 },
    { 0xd755e994, 98 },
    { 0, 0 },
    { 0xd875e998, 52 },
    { 0x4255e999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4295e9a0, 90 },
    { 0x40e5e9a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5e9a4, 77 },
    { 0x745e9a5, 99 },
    { 0x5835e9a6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505e9ab, 87 },
    { 0xe4e5e9ac, 23 },
    { 0x2355e9ac, 91 },
    { 0, 0 },
    { 0xbd45e9af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5155e9b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5e9bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd605e9c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a5e9c6, 55 },
    { 0x6cf5e9c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d55e9cd, 98 },
    { 0, 0 },
    { 0x3c05e9cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc65e9db, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e9e1, 43 },
    { 0, 0 },
    { 0x9485e9e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5e9ea, 76 },
    { 0, 0 },
    { 0x33e5e9ec, 61 },
    { 0x4e65e9ed, 44 },
    { 0, 0 },
    { 0xa615e9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5e9fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825e9fe, 33 },
    { 0xaa75e9ff, 93 },
    { 0, 0 },
    { 0xa65ea01, 55 },
    { 0x7fd5ea01, 95 },
    { 0x9375ea03, 34 },
    { 0x7715ea04, 46 },
    { 0xfb95ea05, 33 },
    { 0x4ed5ea06, 95 },
    { 0xfaf5ea01, 95 },
    { 0xfd35ea08, 88 },
    { 0x3885ea09, 33 },
    { 0, 0 },
    { 0x7985ea0b, 98 },
    { 0x84f5ea0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce5ea0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc115ea12, 43 },
    { 0, 0 },
    { 0xbc5ea14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5ea19, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd955ea1e, 52 },
    { 0xc175ea1e, 99 },
    { 0, 0 },
    { 0xa1b5ea21, 90 },
    { 0x5155ea22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1155ea29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf05ea2c, 90 },
    { 0x9325ea2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ea36, 55 },
    { 0xaf5ea37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6175ea3b, 84 },
    { 0xf475ea3b, 34 },
    { 0x8b5ea3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7525ea46, 61 },
    { 0x86d5ea47, 91 },
    { 0x2a15ea48, 98 },
    { 0, 0 },
    { 0x8035ea4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab5ea4f, 78 },
    { 0xb4f5ea4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f5ea54, 78 },
    { 0xf455ea55, 55 },
    { 0x985ea54, 85 },
    { 0xa695ea54, 77 },
    { 0x1145ea58, 98 },
    { 0x9e45ea56, 83 },
    { 0xeb15ea5a, 23 },
    { 0xa55ea5a, 55 },
    { 0xa525ea5a, 55 },
    { 0xb685ea5d, 99 },
    { 0xe5f5ea5d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c75ea63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4045ea66, 24 },
    { 0xda5ea67, 61 },
    { 0x6385ea67, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5ea6b, 84 },
    { 0x4865ea6b, 23 },
    { 0, 0 },
    { 0xa285ea6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e5ea71, 84 },
    { 0xffc5ea71, 77 },
    { 0xb045ea73, 34 },
    { 0x91b5ea74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35ea77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd15ea7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555ea7e, 90 },
    { 0x18d5ea7f, 81 },
    { 0x6615ea7f, 33 },
    { 0xc155ea81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55ea84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485ea8b, 61 },
    { 0xa505ea8c, 33 },
    { 0x5cc5ea8d, 61 },
    { 0x4635ea8d, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5ea91, 55 },
    { 0xc315ea92, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d95ea9b, 78 },
    { 0xf8f5ea9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa5ea9f, 95 },
    { 0x8335eaa0, 90 },
    { 0xb385eaa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b55eaa8, 90 },
    { 0x8d65eaa8, 33 },
    { 0, 0 },
    { 0x5015eaab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf105eaae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd415eab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55eabc, 80 },
    { 0x76a5eabc, 90 },
    { 0xf4f5eabe, 95 },
    { 0xf615eabf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4945eac2, 43 },
    { 0x5855eac3, 43 },
    { 0x8d05eac2, 95 },
    { 0xa325eac3, 98 },
    { 0x8d5eac3, 55 },
    { 0, 0 },
    { 0xe595eac8, 23 },
    { 0x2d25eac9, 91 },
    { 0xeff5eac9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2035ead9, 33 },
    { 0x4015eada, 100 },
    { 0x7165eadb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb395eae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75eae4, 96 },
    { 0, 0 },
    { 0x1545eae6, 84 },
    { 0x3905eae7, 100 },
    { 0xd575eae6, 61 },
    { 0x6d5eae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a5eaed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6485eaf0, 91 },
    { 0x62b5eaf1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5eaf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545eb02, 34 },
    { 0x7ac5eb02, 34 },
    { 0, 0 },
    { 0x33e5eb05, 76 },
    { 0x575eb05, 23 },
    { 0x5d5eb07, 76 },
    { 0x20f5eb08, 98 },
    { 0, 0 },
    { 0xda65eb0a, 46 },
    { 0x1ac5eb0b, 95 },
    { 0xe555eb0b, 17 },
    { 0xcda5eb0d, 75 },
    { 0xfc85eb0a, 98 },
    { 0x2275eb0f, 91 },
    { 0x5b45eb0b, 43 },
    { 0xa835eb11, 33 },
    { 0x3de5eb12, 46 },
    { 0x6695eb13, 95 },
    { 0xb8b5eb0f, 87 },
    { 0, 0 },
    { 0xb935eb16, 33 },
    { 0x75e5eb17, 33 },
    { 0xbc95eb17, 100 },
    { 0x48b5eb17, 61 },
    { 0, 0 },
    { 0x1365eb1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf925eb21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a15eb27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435eb2b, 33 },
    { 0, 0 },
    { 0x87f5eb2d, 43 },
    { 0x1235eb2d, 95 },
    { 0xf1b5eb2f, 34 },
    { 0x7ac5eb30, 90 },
    { 0x4745eb31, 23 },
    { 0x1ce5eb2f, 75 },
    { 0xf785eb31, 34 },
    { 0x2795eb34, 90 },
    { 0xca05eb34, 34 },
    { 0xeed5eb36, 33 },
    { 0xcf75eb2e, 93 },
    { 0, 0 },
    { 0x45e5eb39, 91 },
    { 0x9965eb39, 75 },
    { 0xe515eb3b, 55 },
    { 0, 0 },
    { 0xe195eb3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe265eb40, 61 },
    { 0x7685eb41, 55 },
    { 0xbee5eb42, 90 },
    { 0xb4f5eb43, 90 },
    { 0x1365eb44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5eb47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a5eb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5eb51, 61 },
    { 0x4715eb52, 87 },
    { 0x485eb53, 90 },
    { 0, 0 },
    { 0xfa35eb55, 98 },
    { 0x19a5eb56, 33 },
    { 0x9445eb56, 98 },
    { 0xa135eb57, 98 },
    { 0xe625eb59, 74 },
    { 0x1005eb5a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c35eb60, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5eb63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d95eb67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5eb6c, 52 },
    { 0xd8c5eb6d, 34 },
    { 0x8bd5eb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5eb72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5eb75, 61 },
    { 0, 0 },
    { 0xf605eb77, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd025eb7a, 61 },
    { 0x75eb7b, 75 },
    { 0x74c5eb7a, 33 },
    { 0xe705eb7b, 34 },
    { 0x3275eb7e, 87 },
    { 0x8915eb7b, 95 },
    { 0x69c5eb7c, 33 },
    { 0xf365eb81, 43 },
    { 0xef45eb82, 66 },
    { 0x3215eb83, 91 },
    { 0x89a5eb83, 55 },
    { 0, 0 },
    { 0x3c35eb86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb55eb89, 76 },
    { 0xe435eb8a, 23 },
    { 0xfd5eb8a, 76 },
    { 0x3505eb8a, 84 },
    { 0xb545eb8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e45eb90, 99 },
    { 0xf0e5eb91, 77 },
    { 0x20e5eb92, 98 },
    { 0x5365eb93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b5eb97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15eb9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7425eba2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f05eba8, 91 },
    { 0xf125eba8, 40 },
    { 0xf15eba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6815ebae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865ebb4, 61 },
    { 0, 0 },
    { 0x3a05ebb6, 61 },
    { 0xfa15ebb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145ebbc, 85 },
    { 0xb965ebbd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975ebc2, 90 },
    { 0x7d45ebc3, 90 },
    { 0, 0 },
    { 0x6d05ebc5, 93 },
    { 0, 0 },
    { 0x2c5ebc7, 98 },
    { 0x71c5ebc8, 33 },
    { 0x7405ebc8, 61 },
    { 0, 0 },
    { 0xaeb5ebcb, 95 },
    { 0x3fa5ebcb, 67 },
    { 0x8205ebcc, 61 },
    { 0, 0 },
    { 0xbe05ebcf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa195ebd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2525ebdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d5ebe1, 84 },
    { 0, 0 },
    { 0x8e95ebe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305ebe6, 78 },
    { 0x9db5ebe6, 95 },
    { 0xf765ebe8, 61 },
    { 0, 0 },
    { 0xa875ebea, 61 },
    { 0xec35ebea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c15ebee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec55ebf3, 90 },
    { 0x7485ebf4, 55 },
    { 0x9665ebf4, 66 },
    { 0x31c5ebf4, 61 },
    { 0xe915ebf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9595ebfa, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x5395ebfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5ec02, 90 },
    { 0xa735ec02, 33 },
    { 0, 0 },
    { 0xbb85ec05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275ec09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1885ec0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa585ec10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5ec14, 95 },
    { 0, 0 },
    { 0x155ec16, 95 },
    { 0x28d5ec16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4665ec1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb5ec27, 78 },
    { 0, 0 },
    { 0x2e85ec29, 100 },
    { 0x51b5ec2a, 75 },
    { 0, 0 },
    { 0xade5ec2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c35ec31, 98 },
    { 0x74b5ec31, 90 },
    { 0xe615ec33, 23 },
    { 0x8885ec34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba5ec3c, 76 },
    { 0x3c25ec3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4815ec40, 46 },
    { 0xb655ec41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465ec4a, 98 },
    { 0xc745ec4b, 95 },
    { 0, 0 },
    { 0x9b45ec4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95ec52, 76 },
    { 0, 0 },
    { 0xc485ec54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c5ec58, 33 },
    { 0x3b85ec58, 90 },
    { 0xdf95ec59, 84 },
    { 0xf2e5ec5b, 33 },
    { 0, 0 },
    { 0x3d15ec5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c05ec62, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc835ec67, 61 },
    { 0, 0 },
    { 0x7515ec69, 33 },
    { 0xea65ec69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc135ec6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05ec70, 61 },
    { 0x3595ec70, 90 },
    { 0x8585ec72, 90 },
    { 0x20a5ec73, 61 },
    { 0, 0 },
    { 0x3e85ec75, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc225ec78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ec7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5ec80, 23 },
    { 0xdcb5ec81, 61 },
    { 0xc4e5ec82, 61 },
    { 0xf0e5ec83, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d5ec87, 67 },
    { 0xf765ec87, 34 },
    { 0xbe05ec88, 76 },
    { 0, 0 },
    { 0x58b5ec8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1105ec93, 90 },
    { 0x3185ec94, 99 },
    { 0x21b5ec95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb145ec98, 46 },
    { 0x3b85ec99, 34 },
    { 0x3865ec99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2105eca0, 76 },
    { 0x7705eca0, 33 },
    { 0x6325eca2, 98 },
    { 0, 0 },
    { 0x4825eca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e05ecaa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e15ecae, 98 },
    { 0x88a5ecae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945ecb4, 90 },
    { 0xa095ecb4, 100 },
    { 0xe7b5ecb6, 33 },
    { 0x2315ecb4, 95 },
    { 0, 0 },
    { 0xd115ecb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf445ecbe, 23 },
    { 0, 0 },
    { 0xd895ecc0, 93 },
    { 0x6fc5ecc1, 61 },
    { 0, 0 },
    { 0x1275ecc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f95ecc7, 43 },
    { 0xc8d5ecc8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1805eccb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305ecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f5ecd1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d5ecd7, 55 },
    { 0xfe05ecd7, 61 },
    { 0, 0 },
    { 0xbe85ecda, 23 },
    { 0, 0 },
    { 0xae25ecdc, 37 },
    { 0x4405ecdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6535ece1, 90 },
    { 0x9bb5ece2, 98 },
    { 0x92e5ece2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b5ece7, 43 },
    { 0xd655ece7, 93 },
    { 0xc9e5ece9, 55 },
    { 0, 0 },
    { 0x8dd5eceb, 85 },
    { 0x9ce5ecec, 33 },
    { 0x5ab5eced, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc025ecf6, 34 },
    { 0xed45ecf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de5ecfa, 98 },
    { 0x90c5ecfb, 23 },
    { 0x62d5ecfc, 43 },
    { 0xb7c5ecfc, 95 },
    { 0x1ca5ecfa, 90 },
    { 0x5385ecfe, 23 },
    { 0x65a5ed00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2465ed05, 98 },
    { 0x8715ed06, 46 },
    { 0xaf95ed07, 61 },
    { 0x5685ed07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1365ed0b, 90 },
    { 0x37f5ed0c, 98 },
    { 0, 0 },
    { 0xe715ed0e, 90 },
    { 0xe3d5ed0f, 98 },
    { 0xf255ed10, 90 },
    { 0, 0 },
    { 0xda85ed12, 85 },
    { 0, 0 },
    { 0xa685ed14, 61 },
    { 0xfb75ed15, 33 },
    { 0, 0 },
    { 0xd2d5ed17, 77 },
    { 0, 0 },
    { 0x7005ed19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c5ed1e, 61 },
    { 0, 0 },
    { 0x9ba5ed20, 33 },
    { 0, 0 },
    { 0x7d55ed22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c5ed29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc5ed2c, 75 },
    { 0x1695ed2c, 33 },
    { 0x88c5ed2d, 61 },
    { 0x8555ed2f, 61 },
    { 0, 0 },
    { 0x1ec5ed31, 98 },
    { 0x77c5ed31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8965ed35, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5ed3a, 23 },
    { 0, 0 },
    { 0x26d5ed3c, 34 },
    { 0, 0 },
    { 0xde75ed3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4645ed43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba95ed4b, 98 },
    { 0, 0 },
    { 0x95f5ed4d, 34 },
    { 0x565ed4e, 85 },
    { 0x2005ed4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e5ed53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1385ed57, 33 },
    { 0, 0 },
    { 0x715ed59, 22 },
    { 0x7cb5ed5a, 76 },
    { 0x6185ed5a, 85 },
    { 0x5365ed5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb335ed60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5ed64, 100 },
    { 0x93f5ed64, 65 },
    { 0x9f5ed66, 23 },
    { 0, 0 },
    { 0x4925ed68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5ed6b, 95 },
    { 0, 0 },
    { 0xc85ed6d, 77 },
    { 0, 0 },
    { 0x39b5ed6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5ed72, 84 },
    { 0xd6e5ed72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7545ed7c, 83 },
    { 0, 0 },
    { 0x2215ed7e, 34 },
    { 0x8705ed7f, 61 },
    { 0xe355ed80, 98 },
    { 0x2dc5ed81, 34 },
    { 0x3645ed81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf205ed85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd645ed89, 61 },
    { 0x66f5ed8a, 61 },
    { 0x6c85ed8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85ed91, 90 },
    { 0x3635ed92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd055ed95, 55 },
    { 0xcf45ed96, 90 },
    { 0x5ed97, 98 },
    { 0x48d5ed98, 55 },
    { 0x4c75ed99, 87 },
    { 0, 0 },
    { 0xa755ed9b, 23 },
    { 0x98f5ed9c, 55 },
    { 0x69b5ed9b, 33 },
    { 0xc765ed9b, 83 },
    { 0x1255ed9f, 95 },
    { 0, 0 },
    { 0x9155eda1, 87 },
    { 0, 0 },
    { 0x4f5eda3, 53 },
    { 0xd145eda4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5475edaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7915edad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3855edb3, 91 },
    { 0xc7a5edb4, 90 },
    { 0xad75edb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e5edb9, 90 },
    { 0x1725edba, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb5edbe, 77 },
    { 0xe915edbe, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7755edc5, 76 },
    { 0x8ae5edc5, 84 },
    { 0xb8a5edc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d45edca, 94 },
    { 0, 0 },
    { 0x5165edcc, 34 },
    { 0x88f5edcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4045edd0, 90 },
    { 0xf305edd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c55edd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e5edd9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d15eddc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35ede0, 33 },
    { 0x585ede1, 90 },
    { 0xd845ede2, 23 },
    { 0xd315ede2, 98 },
    { 0x93e5ede4, 88 },
    { 0xf6f5ede0, 61 },
    { 0, 0 },
    { 0x7c65ede7, 33 },
    { 0xa8d5ede7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6245edeb, 87 },
    { 0, 0 },
    { 0xbf65eded, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15edf3, 75 },
    { 0, 0 },
    { 0xab25edf5, 85 },
    { 0x4385edf6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9765edfb, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3875ee00, 46 },
    { 0xbec5ee00, 61 },
    { 0x7195ee02, 33 },
    { 0x1855ee03, 90 },
    { 0xf615ee04, 76 },
    { 0, 0 },
    { 0x1cb5ee06, 34 },
    { 0, 0 },
    { 0x1a35ee08, 33 },
    { 0x1475ee09, 99 },
    { 0xa1c5ee08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475ee10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e45ee13, 34 },
    { 0xc995ee14, 34 },
    { 0x1a95ee15, 34 },
    { 0x9125ee15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f85ee19, 76 },
    { 0x7a95ee1a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15ee1e, 23 },
    { 0x3375ee1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a5ee23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4105ee26, 61 },
    { 0xf265ee27, 23 },
    { 0x5415ee27, 99 },
    { 0, 0 },
    { 0x2005ee2a, 95 },
    { 0x4a35ee2b, 23 },
    { 0x1675ee2b, 99 },
    { 0x7a75ee2b, 90 },
    { 0x1c45ee2e, 90 },
    { 0x59c5ee2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc185ee33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac05ee36, 68 },
    { 0x2715ee37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d5ee3c, 43 },
    { 0x5035ee3c, 90 },
    { 0xaf55ee3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a5ee41, 90 },
    { 0xadd5ee42, 95 },
    { 0x6b95ee43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e85ee47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace5ee5a, 90 },
    { 0xdaa5ee5b, 64 },
    { 0x9365ee5b, 95 },
    { 0x7a35ee5b, 95 },
    { 0xfc65ee5e, 33 },
    { 0, 0 },
    { 0xf875ee60, 98 },
    { 0x9cb5ee60, 55 },
    { 0, 0 },
    { 0x3835ee63, 77 },
    { 0, 0 },
    { 0x8ca5ee65, 68 },
    { 0x4485ee66, 34 },
    { 0x61d5ee67, 84 },
    { 0xfbe5ee68, 34 },
    { 0xa55ee69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa795ee6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e5ee71, 46 },
    { 0xdba5ee71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc25ee76, 98 },
    { 0x1265ee77, 61 },
    { 0, 0 },
    { 0x9385ee79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5ee7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc155ee7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x395ee82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb605ee88, 23 },
    { 0, 0 },
    { 0xc475ee8a, 91 },
    { 0x1425ee8b, 95 },
    { 0, 0 },
    { 0x8465ee8d, 77 },
    { 0x3b65ee8e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15ee97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d85ee9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4305ee9f, 23 },
    { 0, 0 },
    { 0xa275eea1, 90 },
    { 0x3945eea2, 98 },
    { 0x6885eea1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a5eea6, 98 },
    { 0xa175eea7, 55 },
    { 0x33c5eea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615eeab, 53 },
    { 0x45b5eeab, 98 },
    { 0x4fb5eeac, 95 },
    { 0, 0 },
    { 0x6425eeaf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e05eeb3, 95 },
    { 0, 0 },
    { 0x81a5eeb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c5eeba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf895eebd, 90 },
    { 0, 0 },
    { 0x7db5eebf, 33 },
    { 0, 0 },
    { 0x48f5eec1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4175eec4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65eec8, 87 },
    { 0x3f35eec9, 61 },
    { 0xc195eeca, 84 },
    { 0xa185eec9, 61 },
    { 0, 0 },
    { 0x6f25eecd, 33 },
    { 0, 0 },
    { 0xa9b5eecf, 69 },
    { 0x34f5eecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e15eed3, 61 },
    { 0x9e35eed4, 23 },
    { 0xc3a5eed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635eee4, 90 },
    { 0x1185eee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe585eeeb, 43 },
    { 0xf445eeec, 76 },
    { 0x1b05eeeb, 40 },
    { 0, 0 },
    { 0x29d5eeef, 17 },
    { 0x5695eef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785eef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e5eef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2105eef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda95eefd, 90 },
    { 0x8065eefe, 90 },
    { 0xcad5eeff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55ef06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9875ef0d, 91 },
    { 0x29f5ef0e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875ef19, 76 },
    { 0xac05ef1a, 33 },
    { 0xd695ef1b, 23 },
    { 0, 0 },
    { 0xe095ef1d, 33 },
    { 0, 0 },
    { 0x5705ef1f, 90 },
    { 0x7e05ef20, 23 },
    { 0xfc35ef1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3325ef25, 76 },
    { 0, 0 },
    { 0x4355ef27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b5ef2b, 66 },
    { 0, 0 },
    { 0xa2c5ef2d, 90 },
    { 0x685ef2e, 33 },
    { 0x3d5ef2d, 55 },
    { 0xbec5ef30, 33 },
    { 0x9105ef2d, 95 },
    { 0, 0 },
    { 0x85b5ef33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ef37, 55 },
    { 0xd625ef38, 78 },
    { 0xe2a5ef39, 76 },
    { 0xe405ef39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d5ef3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f5ef42, 93 },
    { 0xc1c5ef42, 90 },
    { 0x90c5ef43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a35ef48, 78 },
    { 0x265ef49, 55 },
    { 0x45d5ef4a, 99 },
    { 0xe445ef4b, 23 },
    { 0xa625ef4b, 95 },
    { 0x3fa5ef4d, 33 },
    { 0x3ef5ef4d, 17 },
    { 0xde65ef4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd695ef52, 76 },
    { 0, 0 },
    { 0x7925ef54, 46 },
    { 0x1925ef55, 52 },
    { 0xbd45ef56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc865ef60, 61 },
    { 0x9d85ef61, 17 },
    { 0x3945ef62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3815ef65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f55ef6a, 95 },
    { 0x185ef6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b75ef6f, 33 },
    { 0, 0 },
    { 0x6b35ef71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ef76, 90 },
    { 0, 0 },
    { 0xd025ef78, 61 },
    { 0xde35ef79, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb755ef7e, 33 },
    { 0x9b45ef7e, 95 },
    { 0x82a5ef7f, 61 },
    { 0, 0 },
    { 0x7fc5ef82, 23 },
    { 0x4235ef83, 23 },
    { 0x615ef84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe665ef8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf295ef90, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5ef9e, 61 },
    { 0xdf85ef9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5efa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65efac, 98 },
    { 0xd3d5efad, 55 },
    { 0xd9b5efae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa085efb2, 61 },
    { 0, 0 },
    { 0xd235efb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e5efbc, 84 },
    { 0, 0 },
    { 0x2e45efbe, 23 },
    { 0xcf85efbf, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815efc6, 87 },
    { 0, 0 },
    { 0xabf5efc8, 34 },
    { 0x9655efc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85efd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b55efd4, 56 },
    { 0xff75efd4, 46 },
    { 0xdbd5efd6, 95 },
    { 0, 0 },
    { 0xd9a5efd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc5efdf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55efe3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f15efe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885efea, 33 },
    { 0x6bb5efeb, 33 },
    { 0x4055efeb, 95 },
    { 0, 0 },
    { 0x44c5efee, 23 },
    { 0xe5e5efef, 95 },
    { 0x11e5efef, 95 },
    { 0x4835efee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf335eff6, 95 },
    { 0x9455eff6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc475effb, 98 },
    { 0, 0 },
    { 0x6045effd, 90 },
    { 0x73e5effe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f5f00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b5f017, 90 },
    { 0x8285f017, 64 },
    { 0, 0 },
    { 0xcd75f01a, 90 },
    { 0x7ba5f01b, 98 },
    { 0x9325f01c, 61 },
    { 0xca15f01d, 84 },
    { 0x5405f01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f022, 98 },
    { 0x7065f023, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485f028, 95 },
    { 0x8d45f029, 76 },
    { 0xea95f028, 99 },
    { 0x6c75f02b, 33 },
    { 0x7d65f02c, 90 },
    { 0x7285f02d, 76 },
    { 0x6765f02c, 93 },
    { 0x7be5f02e, 66 },
    { 0x1d55f030, 66 },
    { 0xdad5f031, 76 },
    { 0x46f5f032, 90 },
    { 0xad55f033, 66 },
    { 0, 0 },
    { 0xb445f035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d5f039, 93 },
    { 0x4595f03a, 33 },
    { 0, 0 },
    { 0x99f5f03c, 95 },
    { 0xbe65f03d, 55 },
    { 0xf9a5f03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe445f041, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c5f045, 96 },
    { 0xc75f046, 61 },
    { 0x1c15f046, 76 },
    { 0x1615f048, 85 },
    { 0xceb5f049, 33 },
    { 0, 0 },
    { 0xad35f04b, 91 },
    { 0, 0 },
    { 0x1465f04d, 61 },
    { 0xcfe5f04e, 95 },
    { 0x44d5f04e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5f052, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3505f056, 61 },
    { 0xf595f057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6975f05d, 61 },
    { 0, 0 },
    { 0x6ea5f05f, 39 },
    { 0, 0 },
    { 0x405f061, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b5f065, 23 },
    { 0x7085f065, 36 },
    { 0, 0 },
    { 0x6f45f068, 34 },
    { 0, 0 },
    { 0x665f06a, 91 },
    { 0x785f06a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b5f06e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825f072, 93 },
    { 0x43c5f072, 78 },
    { 0, 0 },
    { 0x7825f075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5f078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc075f07d, 33 },
    { 0, 0 },
    { 0xe9f5f07f, 97 },
    { 0x7d15f080, 93 },
    { 0x9dd5f081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1795f08a, 90 },
    { 0xcc45f08a, 98 },
    { 0x805f08c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c5f08f, 52 },
    { 0, 0 },
    { 0x6f45f091, 90 },
    { 0x9ba5f091, 43 },
    { 0x3665f093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3985f096, 90 },
    { 0x6fc5f097, 61 },
    { 0x365f096, 80 },
    { 0x1ea5f099, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b25f09c, 90 },
    { 0, 0 },
    { 0x1d35f09e, 99 },
    { 0, 0 },
    { 0xba65f0a0, 69 },
    { 0x2235f0a1, 33 },
    { 0, 0 },
    { 0x7215f0a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2815f0a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3325f0aa, 90 },
    { 0x2f5f0ab, 81 },
    { 0x82f5f0aa, 76 },
    { 0, 0 },
    { 0x215f0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275f0b2, 81 },
    { 0, 0 },
    { 0x88e5f0b4, 23 },
    { 0, 0 },
    { 0x97d5f0b6, 93 },
    { 0xb905f0b7, 100 },
    { 0xebc5f0b7, 95 },
    { 0, 0 },
    { 0x3cd5f0ba, 33 },
    { 0, 0 },
    { 0x2535f0bc, 38 },
    { 0x2985f0bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e15f0c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a5f0c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d5f0cc, 90 },
    { 0, 0 },
    { 0xd415f0ce, 90 },
    { 0x4e85f0cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5f0d4, 98 },
    { 0x1df5f0d5, 46 },
    { 0, 0 },
    { 0xc765f0d7, 61 },
    { 0, 0 },
    { 0xb3c5f0d9, 78 },
    { 0x6975f0da, 40 },
    { 0xfd75f0da, 95 },
    { 0xf365f0da, 23 },
    { 0xcc5f0dd, 91 },
    { 0xfe55f0dd, 65 },
    { 0x5d5f0df, 34 },
    { 0x2785f0dd, 44 },
    { 0x7a5f0e1, 33 },
    { 0x9445f0e2, 33 },
    { 0x8425f0e2, 77 },
    { 0xd125f0e4, 78 },
    { 0, 0 },
    { 0xa8f5f0e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65f0e9, 59 },
    { 0xe3c5f0ea, 23 },
    { 0x2e55f0e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4505f0f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9395f0f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8765f100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045f103, 76 },
    { 0x6825f103, 61 },
    { 0, 0 },
    { 0x4f65f106, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa5f109, 33 },
    { 0xe085f109, 55 },
    { 0x8f5f109, 95 },
    { 0x8155f10c, 98 },
    { 0, 0 },
    { 0x93f5f10e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe925f111, 43 },
    { 0xe965f112, 61 },
    { 0x3a15f113, 33 },
    { 0x1d55f113, 46 },
    { 0xaea5f113, 96 },
    { 0, 0 },
    { 0xd4f5f117, 46 },
    { 0, 0 },
    { 0xd3a5f119, 67 },
    { 0, 0 },
    { 0x6055f11b, 90 },
    { 0, 0 },
    { 0x81f5f11d, 56 },
    { 0, 0 },
    { 0x97f5f11f, 87 },
    { 0, 0 },
    { 0xa395f121, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f124, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf5f127, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85f12b, 66 },
    { 0xe925f12b, 76 },
    { 0, 0 },
    { 0x6b45f12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b5f133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3505f136, 90 },
    { 0x7415f137, 34 },
    { 0, 0 },
    { 0x8fd5f139, 75 },
    { 0x1d55f13a, 90 },
    { 0, 0 },
    { 0x4e15f13c, 90 },
    { 0x9ba5f13c, 61 },
    { 0x9895f13d, 98 },
    { 0xdc85f13f, 43 },
    { 0xafb5f140, 76 },
    { 0x7cb5f141, 33 },
    { 0, 0 },
    { 0x5f35f143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545f146, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5f14b, 90 },
    { 0xb835f14c, 83 },
    { 0xac15f14d, 98 },
    { 0x6845f14b, 23 },
    { 0x9cf5f14d, 90 },
    { 0, 0 },
    { 0x715f151, 33 },
    { 0x2a25f151, 76 },
    { 0, 0 },
    { 0xce85f154, 43 },
    { 0xe835f154, 90 },
    { 0x93d5f156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5f15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75f166, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5f169, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f5f16e, 95 },
    { 0xbf95f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e5f175, 76 },
    { 0x8175f176, 33 },
    { 0, 0 },
    { 0xc9b5f178, 23 },
    { 0x1be5f179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5f17c, 34 },
    { 0x5475f17c, 61 },
    { 0xfe75f17c, 33 },
    { 0xf6f5f17f, 67 },
    { 0x4935f17f, 84 },
    { 0xb055f181, 17 },
    { 0xf735f182, 76 },
    { 0xef5f183, 42 },
    { 0, 0 },
    { 0xcfa5f185, 84 },
    { 0, 0 },
    { 0x2535f187, 33 },
    { 0x30d5f188, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a5f18b, 95 },
    { 0, 0 },
    { 0xb865f18d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c65f196, 33 },
    { 0x9275f197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5f19c, 23 },
    { 0, 0 },
    { 0xc4e5f19e, 90 },
    { 0xab35f19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035f1a2, 33 },
    { 0x8435f1a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee5f1a7, 17 },
    { 0, 0 },
    { 0x5265f1a9, 95 },
    { 0xfcf5f1a9, 61 },
    { 0x1165f1ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475f1ae, 99 },
    { 0xd105f1af, 33 },
    { 0x2b95f1b0, 43 },
    { 0x60b5f1b1, 99 },
    { 0x4c15f1b1, 90 },
    { 0x76f5f1af, 87 },
    { 0x2ad5f1b4, 61 },
    { 0x6fc5f1b4, 95 },
    { 0x2bb5f1b5, 95 },
    { 0, 0 },
    { 0xabc5f1b8, 99 },
    { 0, 0 },
    { 0xd445f1ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd55f1c3, 55 },
    { 0, 0 },
    { 0xcf75f1c5, 33 },
    { 0, 0 },
    { 0x1905f1c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35f1ce, 78 },
    { 0xc595f1cf, 76 },
    { 0xa445f1d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095f1d5, 55 },
    { 0, 0 },
    { 0x9575f1d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a75f1da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7565f1de, 55 },
    { 0, 0 },
    { 0x1125f1e0, 95 },
    { 0x6255f1e1, 33 },
    { 0x1505f1e0, 33 },
    { 0, 0 },
    { 0x7405f1e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf935f1e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3255f1eb, 61 },
    { 0x6a5f1ec, 55 },
    { 0xf155f1ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f1f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205f1f8, 61 },
    { 0x24e5f1f8, 99 },
    { 0x80a5f1fa, 55 },
    { 0xdba5f1fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1915f1fe, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55f203, 61 },
    { 0x36b5f204, 98 },
    { 0xd615f204, 90 },
    { 0xe855f206, 98 },
    { 0xa15f203, 95 },
    { 0x14f5f204, 61 },
    { 0x5ac5f203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025f20d, 61 },
    { 0x6f65f20e, 90 },
    { 0x9665f20e, 90 },
    { 0x24c5f20e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb055f217, 34 },
    { 0x5905f218, 96 },
    { 0, 0 },
    { 0x8375f21a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad5f21e, 23 },
    { 0x8935f21f, 77 },
    { 0xc5d5f220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c85f226, 34 },
    { 0x5ac5f227, 75 },
    { 0xf9d5f227, 78 },
    { 0x89f5f228, 76 },
    { 0, 0 },
    { 0xf325f22b, 98 },
    { 0, 0 },
    { 0xd715f22d, 33 },
    { 0xf675f22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe165f232, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f236, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5f23b, 33 },
    { 0xb9c5f23b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435f240, 69 },
    { 0x5435f240, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25f245, 84 },
    { 0x8bf5f246, 23 },
    { 0xaf05f246, 23 },
    { 0xb765f248, 33 },
    { 0x9a5f249, 90 },
    { 0xf25f245, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5f24d, 33 },
    { 0xd535f24d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5f251, 77 },
    { 0xe5c5f251, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9015f255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5f259, 23 },
    { 0, 0 },
    { 0xb025f25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6235f25e, 78 },
    { 0x3ad5f25e, 61 },
    { 0, 0 },
    { 0x7525f261, 61 },
    { 0x5b15f262, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b95f265, 78 },
    { 0xb615f266, 95 },
    { 0xd585f267, 55 },
    { 0x765f265, 93 },
    { 0x62e5f265, 93 },
    { 0, 0 },
    { 0x7b35f26b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c5f26e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f272, 76 },
    { 0x68d5f272, 93 },
    { 0xee15f274, 87 },
    { 0x6ab5f275, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe435f283, 78 },
    { 0x7c95f283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585f28b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385f29b, 61 },
    { 0x51d5f29b, 61 },
    { 0x3d75f29b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da5f2a4, 68 },
    { 0x7015f2a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255f2a8, 66 },
    { 0x8d55f2a8, 31 },
    { 0, 0 },
    { 0x5a5f2ab, 46 },
    { 0x3845f2ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975f2b1, 46 },
    { 0, 0 },
    { 0xa0a5f2b3, 90 },
    { 0xd345f2b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1135f2c7, 23 },
    { 0, 0 },
    { 0xd675f2c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015f2ce, 61 },
    { 0xa3a5f2cf, 23 },
    { 0xcde5f2cf, 98 },
    { 0x2345f2d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b55f2d8, 98 },
    { 0x9b55f2d9, 90 },
    { 0xfd25f2da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe445f2e5, 95 },
    { 0, 0 },
    { 0xcbe5f2e7, 96 },
    { 0, 0 },
    { 0xfd85f2e9, 90 },
    { 0, 0 },
    { 0xe1d5f2eb, 61 },
    { 0x8495f2ec, 90 },
    { 0xb1b5f2ed, 76 },
    { 0xa305f2ed, 98 },
    { 0x3fb5f2ee, 33 },
    { 0xf1a5f2f0, 95 },
    { 0xdaa5f2f0, 55 },
    { 0x5e15f2ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65f2f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5f301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca5f305, 19 },
    { 0xe765f306, 55 },
    { 0, 0 },
    { 0xb505f308, 66 },
    { 0xab45f309, 76 },
    { 0x1a65f308, 76 },
    { 0x2485f308, 33 },
    { 0, 0 },
    { 0xa845f30d, 100 },
    { 0x715f30e, 61 },
    { 0xd1b5f30d, 98 },
    { 0xe705f30f, 76 },
    { 0x3905f311, 23 },
    { 0x75f5f311, 76 },
    { 0x56f5f311, 61 },
    { 0x8425f314, 93 },
    { 0x6e85f315, 90 },
    { 0x29d5f314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f5f32b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a55f331, 61 },
    { 0x9495f331, 91 },
    { 0x3525f333, 76 },
    { 0xd375f334, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf515f344, 29 },
    { 0xabe5f345, 93 },
    { 0x92a5f346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a5f34b, 40 },
    { 0x7925f34c, 46 },
    { 0xc015f34b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb5f352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba15f358, 61 },
    { 0xfa85f358, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8515f35e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a5f361, 43 },
    { 0xe95f362, 61 },
    { 0, 0 },
    { 0x72d5f364, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5f367, 55 },
    { 0, 0 },
    { 0xcaa5f369, 46 },
    { 0x3095f369, 93 },
    { 0x5715f36b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e55f36e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f5f372, 23 },
    { 0x58f5f372, 98 },
    { 0, 0 },
    { 0xccf5f375, 61 },
    { 0x48d5f375, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675f37b, 78 },
    { 0x67e5f37b, 33 },
    { 0xe5f37c, 80 },
    { 0xfc05f37e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e5f382, 43 },
    { 0xb825f382, 90 },
    { 0, 0 },
    { 0xe665f385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8355f38e, 98 },
    { 0, 0 },
    { 0xda05f390, 43 },
    { 0xd275f391, 23 },
    { 0, 0 },
    { 0x2f15f393, 93 },
    { 0x2d65f394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9935f39b, 84 },
    { 0x79f5f39c, 91 },
    { 0xe415f39c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35f3a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5f3a4, 33 },
    { 0, 0 },
    { 0x2bc5f3a6, 90 },
    { 0x61e5f3a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25f3ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95f3b5, 23 },
    { 0xa865f3b6, 90 },
    { 0x6a35f3b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce35f3bf, 90 },
    { 0, 0 },
    { 0x8925f3c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca5f3c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745f3ce, 95 },
    { 0x5965f3cf, 61 },
    { 0x6005f3d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f55f3d5, 55 },
    { 0xbca5f3d6, 46 },
    { 0x6805f3d6, 55 },
    { 0x2b5f3d6, 90 },
    { 0, 0 },
    { 0xbb95f3da, 90 },
    { 0x8f5f3da, 87 },
    { 0x89b5f3da, 95 },
    { 0xf125f3dd, 91 },
    { 0x9b55f3de, 98 },
    { 0x7b75f3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c75f3e9, 90 },
    { 0x3265f3e9, 75 },
    { 0x52d5f3e9, 95 },
    { 0xf465f3ec, 76 },
    { 0x6c45f3ea, 95 },
    { 0xff85f3e9, 98 },
    { 0xa455f3ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4105f3f2, 95 },
    { 0, 0 },
    { 0x5b5f3f4, 69 },
    { 0x9d95f3f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd135f3fb, 84 },
    { 0xecd5f3fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb405f402, 66 },
    { 0xff5f403, 76 },
    { 0x94a5f402, 55 },
    { 0, 0 },
    { 0x4c65f406, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef5f409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd5f40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9025f412, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9785f416, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab35f41a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe195f420, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2115f425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf5f429, 17 },
    { 0x7b95f429, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee5f430, 90 },
    { 0x72f5f431, 90 },
    { 0x4295f431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae85f439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad5f43c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed5f442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc5f445, 90 },
    { 0xb6b5f446, 23 },
    { 0, 0 },
    { 0x76b5f448, 61 },
    { 0x7695f449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c5f44f, 90 },
    { 0, 0 },
    { 0xc615f451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a65f455, 33 },
    { 0xe9d5f456, 77 },
    { 0x7f35f456, 33 },
    { 0x87a5f455, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9175f45b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x8205f45e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc205f461, 33 },
    { 0x6be5f462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435f467, 98 },
    { 0xd0e5f468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf195f46b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a45f47c, 90 },
    { 0x88f5f47c, 65 },
    { 0x8c55f47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f483, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b5f488, 2 },
    { 0xb7c5f489, 61 },
    { 0x9565f48a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f5f48f, 61 },
    { 0x4ce5f490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb35f498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35f49f, 98 },
    { 0xb775f4a0, 33 },
    { 0xf955f49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e5f4a7, 76 },
    { 0xec35f4a8, 33 },
    { 0xa2c5f4a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x395f4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745f4b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f25f4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb85f4be, 91 },
    { 0, 0 },
    { 0xe205f4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c5f4c4, 87 },
    { 0xd085f4c4, 95 },
    { 0xa0e5f4c6, 33 },
    { 0xf9f5f4c6, 98 },
    { 0xdba5f4c7, 33 },
    { 0x5e05f4c9, 61 },
    { 0x7255f4ca, 90 },
    { 0, 0 },
    { 0x6305f4cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5f4d0, 61 },
    { 0xb895f4d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e5f4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce45f4d8, 40 },
    { 0x445f4d9, 98 },
    { 0xbab5f4da, 78 },
    { 0x1b65f4db, 61 },
    { 0x7c05f4dc, 33 },
    { 0x81f5f4db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6155f4e2, 65 },
    { 0, 0 },
    { 0x255f4e4, 76 },
    { 0x9d05f4e5, 78 },
    { 0, 0 },
    { 0x8bb5f4e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5f4ec, 93 },
    { 0xb505f4ec, 61 },
    { 0, 0 },
    { 0xa615f4ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad5f4f2, 76 },
    { 0x7b15f4f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835f4f9, 33 },
    { 0, 0 },
    { 0xf795f4fb, 61 },
    { 0xd275f4fb, 85 },
    { 0x55c5f4fc, 33 },
    { 0, 0 },
    { 0x96a5f4ff, 90 },
    { 0, 0 },
    { 0xf8f5f501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa585f50c, 76 },
    { 0xa325f50d, 23 },
    { 0x9355f50e, 90 },
    { 0, 0 },
    { 0x2305f510, 33 },
    { 0xd4a5f511, 33 },
    { 0xda15f512, 95 },
    { 0xce5f510, 61 },
    { 0, 0 },
    { 0xbf15f515, 32 },
    { 0x22b5f516, 90 },
    { 0, 0 },
    { 0xd2c5f518, 93 },
    { 0x7ae5f519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b5f51e, 33 },
    { 0, 0 },
    { 0x7e25f520, 88 },
    { 0xeeb5f521, 55 },
    { 0x7ee5f520, 33 },
    { 0, 0 },
    { 0x7185f524, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d5f528, 76 },
    { 0, 0 },
    { 0x46d5f52a, 77 },
    { 0x4405f52b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15f52e, 61 },
    { 0, 0 },
    { 0xd105f530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af5f533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4285f536, 55 },
    { 0xc2d5f536, 91 },
    { 0x6c5f536, 75 },
    { 0, 0 },
    { 0x14a5f53a, 33 },
    { 0x29e5f53a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d5f540, 81 },
    { 0xc6f5f541, 61 },
    { 0, 0 },
    { 0x595f543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d5f546, 61 },
    { 0x4bc5f546, 23 },
    { 0x5145f548, 77 },
    { 0x6225f549, 34 },
    { 0x9725f549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9375f54d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695f550, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5f555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f5f55a, 33 },
    { 0, 0 },
    { 0x3b45f55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc5f562, 77 },
    { 0x2695f563, 100 },
    { 0, 0 },
    { 0xb365f565, 99 },
    { 0, 0 },
    { 0x2ef5f567, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9975f56b, 81 },
    { 0x6d15f56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745f56f, 33 },
    { 0x9ca5f56f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15f574, 67 },
    { 0xfcb5f574, 33 },
    { 0xa985f574, 23 },
    { 0, 0 },
    { 0x9c85f578, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a35f57c, 78 },
    { 0x2b15f57c, 23 },
    { 0, 0 },
    { 0xf5a5f57f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d65f585, 66 },
    { 0x5e95f585, 33 },
    { 0x1b15f585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7015f58a, 85 },
    { 0xbb95f58a, 46 },
    { 0x935f58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa145f58f, 55 },
    { 0, 0 },
    { 0xecb5f591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2285f598, 90 },
    { 0xb5f5f599, 53 },
    { 0xf4e5f598, 43 },
    { 0xda75f59b, 61 },
    { 0xb8c5f59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b5f59f, 33 },
    { 0, 0 },
    { 0x70a5f5a1, 90 },
    { 0x8ea5f5a2, 61 },
    { 0x3a75f5a1, 23 },
    { 0, 0 },
    { 0x5fe5f5a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc5f5a9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5f5b0, 84 },
    { 0x8d45f5b1, 76 },
    { 0x3a85f5b1, 90 },
    { 0, 0 },
    { 0x6ac5f5b4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff5f5bb, 61 },
    { 0x5475f5bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a5f5bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc955f5c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795f5ca, 97 },
    { 0xde75f5cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3755f5ce, 90 },
    { 0x9ed5f5cf, 55 },
    { 0x2b85f5d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675f5d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25f5d9, 61 },
    { 0xd2a5f5da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e5f5df, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b5f5e2, 83 },
    { 0x8625f5e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85f5e9, 95 },
    { 0, 0 },
    { 0xb915f5eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125f5f1, 75 },
    { 0, 0 },
    { 0x42b5f5f3, 76 },
    { 0xfab5f5f3, 69 },
    { 0xf8c5f5f3, 90 },
    { 0, 0 },
    { 0xc7c5f5f7, 90 },
    { 0x87a5f5f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda5f5fe, 33 },
    { 0, 0 },
    { 0xf2a5f600, 55 },
    { 0, 0 },
    { 0x3b5f602, 34 },
    { 0xf445f602, 91 },
    { 0xfe65f604, 17 },
    { 0, 0 },
    { 0xec5f606, 23 },
    { 0, 0 },
    { 0xa205f608, 90 },
    { 0x8875f609, 98 },
    { 0x85e5f608, 98 },
    { 0, 0 },
    { 0xb45f60c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc025f610, 98 },
    { 0x1a75f611, 43 },
    { 0x285f611, 95 },
    { 0x4ec5f613, 33 },
    { 0x6915f613, 23 },
    { 0xb4d5f615, 76 },
    { 0, 0 },
    { 0x5765f617, 100 },
    { 0x81d5f617, 90 },
    { 0x2f75f617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3165f61d, 98 },
    { 0, 0 },
    { 0x1f85f61f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5f626, 78 },
    { 0xa725f627, 99 },
    { 0xd0c5f628, 88 },
    { 0x2195f628, 98 },
    { 0xa195f626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d75f630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5f633, 61 },
    { 0xbcc5f634, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925f638, 46 },
    { 0xea75f638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf285f644, 84 },
    { 0xca85f645, 23 },
    { 0, 0 },
    { 0x12a5f647, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35f64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f5f650, 89 },
    { 0xa345f651, 33 },
    { 0x6fd5f651, 90 },
    { 0, 0 },
    { 0x6bd5f654, 55 },
    { 0xd2b5f655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6895f658, 23 },
    { 0, 0 },
    { 0xa525f65a, 55 },
    { 0x95d5f65b, 23 },
    { 0, 0 },
    { 0xa7b5f65d, 98 },
    { 0xeb75f65d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f5f664, 33 },
    { 0x8345f664, 98 },
    { 0x5395f666, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05f669, 33 },
    { 0x5a5f669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9905f66d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a5f670, 61 },
    { 0xa3e5f671, 66 },
    { 0xf735f672, 95 },
    { 0x23b5f673, 34 },
    { 0x3e05f673, 61 },
    { 0x63c5f675, 61 },
    { 0x7745f673, 90 },
    { 0xc405f677, 98 },
    { 0xe3c5f678, 95 },
    { 0x785f675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd5f67c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935f680, 52 },
    { 0x5375f680, 23 },
    { 0x2b25f682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd5f687, 99 },
    { 0xeb25f688, 90 },
    { 0x30f5f689, 100 },
    { 0xa045f68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925f68e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde45f691, 43 },
    { 0xde95f692, 87 },
    { 0x2335f693, 98 },
    { 0x3a85f693, 75 },
    { 0x785f693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6045f69a, 76 },
    { 0xce85f69b, 33 },
    { 0x2cc5f69c, 100 },
    { 0, 0 },
    { 0xbf15f69e, 93 },
    { 0x7fa5f69f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c5f6a2, 90 },
    { 0, 0 },
    { 0xef5f6a4, 61 },
    { 0xeee5f6a4, 55 },
    { 0, 0 },
    { 0xadb5f6a7, 33 },
    { 0x455f6a8, 33 },
    { 0x2385f6a9, 95 },
    { 0xed75f6aa, 90 },
    { 0x6e15f6aa, 100 },
    { 0, 0 },
    { 0xe8f5f6ad, 61 },
    { 0, 0 },
    { 0xaa95f6af, 90 },
    { 0x6c65f6b0, 23 },
    { 0x2cc5f6b1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5f6b4, 34 },
    { 0, 0 },
    { 0x13c5f6b6, 90 },
    { 0, 0 },
    { 0xdbf5f6b8, 76 },
    { 0, 0 },
    { 0x2425f6ba, 98 },
    { 0x7295f6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa5f6be, 33 },
    { 0x1485f6be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb5f6c4, 90 },
    { 0x6dd5f6c5, 90 },
    { 0, 0 },
    { 0x9465f6c7, 39 },
    { 0, 0 },
    { 0x9b5f6c9, 98 },
    { 0xedb5f6c9, 55 },
    { 0, 0 },
    { 0x2135f6cc, 90 },
    { 0x84c5f6cd, 90 },
    { 0x9085f6ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d05f6d3, 23 },
    { 0, 0 },
    { 0xae75f6d5, 33 },
    { 0x2e55f6d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7055f6e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab85f6e7, 33 },
    { 0, 0 },
    { 0xb775f6e9, 95 },
    { 0x1265f6ea, 61 },
    { 0x2b05f6eb, 80 },
    { 0x9905f6eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95f6ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5f6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd795f6f9, 98 },
    { 0, 0 },
    { 0xf295f6fb, 95 },
    { 0x19e5f6fc, 88 },
    { 0, 0 },
    { 0xcb35f6fe, 93 },
    { 0x9615f6ff, 43 },
    { 0x7ec5f6ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d45f705, 23 },
    { 0x85f706, 88 },
    { 0xa7e5f706, 33 },
    { 0x3825f707, 4 },
    { 0x2595f709, 80 },
    { 0, 0 },
    { 0xea5f70b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25f712, 78 },
    { 0, 0 },
    { 0xd9f5f714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb555f71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c5f71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb5f720, 52 },
    { 0x16b5f721, 23 },
    { 0x26b5f722, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f727, 61 },
    { 0, 0 },
    { 0xfd35f729, 91 },
    { 0x49c5f72a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd5f72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b5f735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7175f738, 90 },
    { 0x1215f739, 23 },
    { 0x2b5f73a, 95 },
    { 0x7b5f73b, 90 },
    { 0x1795f73c, 61 },
    { 0x7be5f739, 55 },
    { 0, 0 },
    { 0x2db5f73f, 76 },
    { 0x1f5f740, 66 },
    { 0, 0 },
    { 0x6ce5f742, 55 },
    { 0, 0 },
    { 0xa0c5f744, 90 },
    { 0x5735f744, 95 },
    { 0, 0 },
    { 0xb9e5f747, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575f74b, 61 },
    { 0x1cc5f74b, 33 },
    { 0xd765f74d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a35f750, 55 },
    { 0x2a95f751, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xea65f754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8415f759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf5f75c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a5f760, 76 },
    { 0x9f75f760, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605f768, 33 },
    { 0x55e5f769, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e45f76f, 68 },
    { 0x82e5f770, 98 },
    { 0x3bc5f76f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605f774, 98 },
    { 0x1605f774, 90 },
    { 0x9bf5f775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf15f77e, 75 },
    { 0xe955f77e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605f782, 67 },
    { 0x5e5f782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5f786, 64 },
    { 0x2805f786, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4695f78a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef75f791, 33 },
    { 0, 0 },
    { 0xec05f793, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa705f798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585f7a0, 33 },
    { 0, 0 },
    { 0xbbf5f7a2, 33 },
    { 0xe315f7a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5f7a6, 34 },
    { 0, 0 },
    { 0x6595f7a8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745f7ac, 90 },
    { 0, 0 },
    { 0xc3a5f7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35f7b9, 82 },
    { 0xfc75f7ba, 61 },
    { 0x6305f7ba, 53 },
    { 0x1b5f7bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd375f7c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4485f7c6, 76 },
    { 0xdec5f7c7, 98 },
    { 0x4e55f7c8, 84 },
    { 0x7c95f7c8, 76 },
    { 0xc3b5f7c8, 33 },
    { 0x3b25f7cb, 43 },
    { 0x7b35f7cc, 90 },
    { 0xfd15f7c7, 90 },
    { 0x9465f7ce, 33 },
    { 0xacb5f7cf, 67 },
    { 0xe5c5f7cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c5f7d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe325f7db, 100 },
    { 0x4055f7dc, 58 },
    { 0x2cf5f7dc, 38 },
    { 0xf0c5f7de, 90 },
    { 0x1165f7df, 76 },
    { 0xc6e5f7df, 96 },
    { 0xe1c5f7e1, 98 },
    { 0, 0 },
    { 0x8225f7e3, 100 },
    { 0xc875f7e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a85f7e7, 99 },
    { 0x8615f7e8, 23 },
    { 0xcdd5f7e9, 90 },
    { 0xc475f7ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c25f7f1, 95 },
    { 0x7d85f7f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9675f7f6, 52 },
    { 0x4dc5f7f7, 78 },
    { 0x375f7f8, 84 },
    { 0x5b05f7f8, 99 },
    { 0x9b45f7fa, 78 },
    { 0x1985f7f7, 96 },
    { 0x7425f7fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff5f800, 99 },
    { 0xc1a5f801, 100 },
    { 0xbdb5f802, 78 },
    { 0x5725f801, 81 },
    { 0x4b45f803, 85 },
    { 0xb055f802, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa175f808, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775f80b, 23 },
    { 0xd195f80c, 33 },
    { 0, 0 },
    { 0x48e5f80e, 34 },
    { 0x5a45f80e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb5f813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc65f816, 87 },
    { 0x10b5f817, 98 },
    { 0xa3b5f817, 98 },
    { 0x7085f819, 33 },
    { 0x6f35f81a, 91 },
    { 0, 0 },
    { 0xa1f5f81c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd165f820, 61 },
    { 0, 0 },
    { 0x7b35f822, 91 },
    { 0, 0 },
    { 0xbbb5f824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5f831, 98 },
    { 0x8f75f832, 34 },
    { 0, 0 },
    { 0xd515f834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf5f83e, 33 },
    { 0x4415f83f, 95 },
    { 0x505f840, 52 },
    { 0xa7c5f83f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3145f847, 98 },
    { 0x6955f847, 66 },
    { 0x2e85f849, 61 },
    { 0x2765f849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3435f84d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5f854, 33 },
    { 0xea55f855, 61 },
    { 0xe45f854, 76 },
    { 0, 0 },
    { 0xae65f858, 66 },
    { 0x3f65f859, 99 },
    { 0xafe5f858, 90 },
    { 0x44e5f85b, 90 },
    { 0xf125f859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab5f860, 95 },
    { 0xe595f861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355f867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5f86a, 90 },
    { 0x40b5f86a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9185f870, 98 },
    { 0, 0 },
    { 0xf8d5f872, 90 },
    { 0xbf15f872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b5f878, 33 },
    { 0x1795f879, 90 },
    { 0x4605f87a, 87 },
    { 0x3a15f879, 33 },
    { 0x54a5f87c, 26 },
    { 0xfd55f87d, 90 },
    { 0, 0 },
    { 0x4865f87f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1445f884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a5f887, 55 },
    { 0xb155f888, 93 },
    { 0x4715f889, 33 },
    { 0xe755f889, 95 },
    { 0x7715f889, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d5f896, 69 },
    { 0x63c5f897, 34 },
    { 0xe995f898, 46 },
    { 0x445f899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a65f89d, 33 },
    { 0, 0 },
    { 0x76c5f89f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85f8a4, 98 },
    { 0xd6f5f8a5, 77 },
    { 0x4b45f8a6, 23 },
    { 0x4c55f8a4, 61 },
    { 0x9df5f8a8, 55 },
    { 0x9ac5f8a9, 98 },
    { 0x2b75f8a9, 95 },
    { 0, 0 },
    { 0x8895f8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535f8af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d5f8b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35f8b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab5f8bd, 34 },
    { 0x1075f8be, 33 },
    { 0xae05f8bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7845f8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2855f8c7, 61 },
    { 0x9f5f8c7, 23 },
    { 0, 0 },
    { 0xa1c5f8ca, 34 },
    { 0xa065f8cb, 91 },
    { 0, 0 },
    { 0x5b85f8cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9035f8d0, 87 },
    { 0xea05f8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d5f8d4, 95 },
    { 0x5735f8d5, 95 },
    { 0x2ff5f8d6, 36 },
    { 0x5c55f8d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b5f8dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5f8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd5f8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225f8e7, 43 },
    { 0x9e25f8e7, 90 },
    { 0x8175f8e9, 43 },
    { 0x4155f8ea, 34 },
    { 0xdfb5f8e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685f8f1, 61 },
    { 0xb0b5f8f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f5f8f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac35f8f8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb5f8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa5f8ff, 61 },
    { 0, 0 },
    { 0x9135f901, 61 },
    { 0x3f25f902, 76 },
    { 0, 0 },
    { 0x3b45f904, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2245f90a, 77 },
    { 0xd5b5f90b, 77 },
    { 0x3265f90c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e75f90f, 95 },
    { 0xb655f90f, 23 },
    { 0x28d5f90f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x755f914, 34 },
    { 0x3035f915, 85 },
    { 0x8a65f916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2065f929, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc785f92d, 33 },
    { 0, 0 },
    { 0x13f5f92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835f935, 33 },
    { 0x97d5f936, 33 },
    { 0, 0 },
    { 0x9d05f938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5f940, 34 },
    { 0x3715f940, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb5f944, 76 },
    { 0x6e85f945, 52 },
    { 0x2795f945, 61 },
    { 0x6b95f947, 95 },
    { 0xfb5f948, 33 },
    { 0x1295f949, 76 },
    { 0x9515f94a, 61 },
    { 0x6805f949, 75 },
    { 0x6ed5f94a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5f953, 43 },
    { 0xf385f954, 76 },
    { 0, 0 },
    { 0x2505f956, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475f95b, 61 },
    { 0x5b75f95c, 91 },
    { 0x3535f95c, 43 },
    { 0xb465f95e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5f963, 52 },
    { 0, 0 },
    { 0x8545f965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a5f968, 100 },
    { 0, 0 },
    { 0x9fa5f96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d65f96e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff5f975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa185f979, 33 },
    { 0, 0 },
    { 0xf55f97b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f5f982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed75f985, 84 },
    { 0xb2f5f986, 43 },
    { 0, 0 },
    { 0x10a5f988, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2155f990, 55 },
    { 0xa895f991, 90 },
    { 0x36a5f991, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05f99d, 95 },
    { 0x38e5f99e, 95 },
    { 0x9cc5f99f, 88 },
    { 0xd555f99f, 75 },
    { 0x3675f9a1, 95 },
    { 0x8645f9a0, 90 },
    { 0x84d5f99e, 93 },
    { 0x76b5f9a4, 85 },
    { 0x4ad5f9a2, 47 },
    { 0xb535f9a6, 33 },
    { 0xc25f99e, 95 },
    { 0x7ce5f99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6135f9b1, 34 },
    { 0x8c55f9b2, 23 },
    { 0x1335f9b2, 67 },
    { 0, 0 },
    { 0x24f5f9b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5105f9b9, 34 },
    { 0, 0 },
    { 0xec75f9bb, 95 },
    { 0xc715f9bc, 68 },
    { 0x3985f9bb, 61 },
    { 0xbfb5f9bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115f9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe285f9c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df5f9cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8275f9d0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c5f9d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a5f9da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c5f9dd, 98 },
    { 0xf8b5f9de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc165f9e1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5f9e7, 98 },
    { 0x7375f9e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb5f9ef, 61 },
    { 0x4ff5f9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5175f9f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525fa13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9235fa1a, 76 },
    { 0xfa55fa1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd15fa28, 90 },
    { 0x92e5fa29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d5fa2e, 87 },
    { 0x4c25fa2f, 78 },
    { 0x1485fa2f, 33 },
    { 0x1575fa2f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775fa34, 52 },
    { 0x4a55fa35, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b5fa38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3975fa3b, 33 },
    { 0xeff5fa3c, 55 },
    { 0x3785fa3d, 98 },
    { 0xea15fa3e, 61 },
    { 0xb535fa3f, 23 },
    { 0, 0 },
    { 0xafe5fa41, 33 },
    { 0, 0 },
    { 0x7ed5fa43, 76 },
    { 0x3005fa44, 95 },
    { 0x2ae5fa45, 90 },
    { 0x69b5fa43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5fa53, 46 },
    { 0xb515fa54, 76 },
    { 0x1275fa54, 76 },
    { 0, 0 },
    { 0x2c65fa57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8615fa5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb5fa5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1335fa61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5855fa64, 87 },
    { 0x5ec5fa65, 91 },
    { 0x51d5fa66, 98 },
    { 0xd0b5fa67, 95 },
    { 0, 0 },
    { 0x89f5fa69, 34 },
    { 0xb055fa6a, 61 },
    { 0xde05fa69, 98 },
    { 0xb9e5fa6c, 98 },
    { 0, 0 },
    { 0x35c5fa6e, 93 },
    { 0x6a75fa6e, 98 },
    { 0x8a55fa6e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd5fa74, 13 },
    { 0, 0 },
    { 0xaaf5fa76, 76 },
    { 0, 0 },
    { 0x17c5fa78, 55 },
    { 0, 0 },
    { 0xe125fa7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2965fa7d, 95 },
    { 0, 0 },
    { 0x1135fa7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5245fa83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8175fa8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd85fa93, 34 },
    { 0x7cb5fa94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5fa9b, 33 },
    { 0, 0 },
    { 0x1a75fa9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc285faa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c85faa7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515faab, 20 },
    { 0x8cb5faac, 98 },
    { 0xd145faad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad55fab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc665fabd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa205fac2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595fac9, 34 },
    { 0xec65faca, 76 },
    { 0xa625facb, 33 },
    { 0x1a5faca, 76 },
    { 0x1695facd, 84 },
    { 0x6b15facd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe535fad6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f5fada, 90 },
    { 0, 0 },
    { 0xe1e5fadc, 61 },
    { 0, 0 },
    { 0x3d75fade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865fae3, 33 },
    { 0, 0 },
    { 0x3695fae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8915fae9, 95 },
    { 0xe635faea, 23 },
    { 0x8565faeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95faee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e85faf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5305fafe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9905fb02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a55fb0a, 36 },
    { 0x41d5fb0b, 52 },
    { 0x6465fb0b, 90 },
    { 0x2175fb0b, 46 },
    { 0x3a45fb0c, 93 },
    { 0x9f65fb0d, 91 },
    { 0xd3c5fb10, 84 },
    { 0x7bb5fb11, 46 },
    { 0x6d5fb11, 23 },
    { 0x9825fb13, 98 },
    { 0x3b75fb14, 33 },
    { 0xf0b5fb14, 33 },
    { 0x5185fb16, 66 },
    { 0xbac5fb15, 95 },
    { 0x47b5fb18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85fb1b, 46 },
    { 0x7d65fb1c, 95 },
    { 0x7a75fb1c, 98 },
    { 0xfd25fb1e, 76 },
    { 0x8045fb1f, 98 },
    { 0xf8f5fb1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d5fb23, 84 },
    { 0x865fb24, 95 },
    { 0, 0 },
    { 0x55a5fb26, 55 },
    { 0x6065fb27, 85 },
    { 0, 0 },
    { 0xa875fb29, 76 },
    { 0xb0d5fb2a, 23 },
    { 0x2f85fb2b, 88 },
    { 0xa6d5fb29, 55 },
    { 0x1ca5fb2d, 61 },
    { 0xd055fb2e, 46 },
    { 0xb5b5fb2f, 88 },
    { 0x75e5fb30, 52 },
    { 0x9e5fb31, 90 },
    { 0xf8e5fb32, 66 },
    { 0x83b5fb2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3445fb39, 91 },
    { 0x50c5fb3a, 61 },
    { 0x5805fb3b, 61 },
    { 0x2bd5fb3b, 95 },
    { 0, 0 },
    { 0xd295fb3e, 84 },
    { 0, 0 },
    { 0x4c75fb40, 90 },
    { 0x765fb41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d35fb44, 42 },
    { 0x29c5fb44, 98 },
    { 0x8585fb46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275fb4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5fb58, 33 },
    { 0, 0 },
    { 0x4635fb5a, 95 },
    { 0xbad5fb5b, 90 },
    { 0x3965fb5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d5fb63, 95 },
    { 0x8fa5fb64, 68 },
    { 0xcf5fb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff5fb68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5fb6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1905fb70, 98 },
    { 0x2bc5fb70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f95fb76, 33 },
    { 0xf295fb76, 98 },
    { 0, 0 },
    { 0x5235fb79, 91 },
    { 0x47c5fb7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5fb7d, 68 },
    { 0x9785fb7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5fb83, 98 },
    { 0x12a5fb84, 95 },
    { 0, 0 },
    { 0xa55fb86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5fb8b, 95 },
    { 0, 0 },
    { 0xed5fb8d, 90 },
    { 0, 0 },
    { 0xa2d5fb8f, 90 },
    { 0, 0 },
    { 0x8305fb91, 87 },
    { 0, 0 },
    { 0x20d5fb93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf855fb96, 95 },
    { 0x7e25fb96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf795fb9b, 78 },
    { 0x2725fb9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec5fb9f, 99 },
    { 0, 0 },
    { 0xde35fba1, 85 },
    { 0x1c25fba2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd725fba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075fbae, 23 },
    { 0x22b5fbae, 33 },
    { 0, 0 },
    { 0x5b95fbb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5fbb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5fbbc, 98 },
    { 0xde05fbbc, 23 },
    { 0xb965fbbe, 88 },
    { 0xb15fbbf, 43 },
    { 0x74b5fbbe, 76 },
    { 0x1115fbc1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea45fbc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee65fbcc, 85 },
    { 0xd55fbcd, 33 },
    { 0x5ab5fbcd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb15fbd2, 95 },
    { 0, 0 },
    { 0x5435fbd4, 33 },
    { 0, 0 },
    { 0x3b35fbd6, 90 },
    { 0, 0 },
    { 0x2f05fbd8, 61 },
    { 0, 0 },
    { 0xcd45fbda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b5fbdd, 86 },
    { 0xc5d5fbde, 76 },
    { 0xf815fbdf, 34 },
    { 0x7e65fbde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9995fbe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af5fbe8, 98 },
    { 0xa225fbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a65fbed, 59 },
    { 0, 0 },
    { 0xfc05fbef, 61 },
    { 0xfb35fbf0, 88 },
    { 0xa0d5fbf1, 55 },
    { 0xc175fbf0, 61 },
    { 0x9db5fbf0, 23 },
    { 0x33f5fbf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c5fbfc, 97 },
    { 0xc55fbfd, 85 },
    { 0xaee5fbfc, 88 },
    { 0xc225fbff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955fc02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf85fc05, 61 },
    { 0xf55fc06, 67 },
    { 0, 0 },
    { 0x7185fc08, 52 },
    { 0x2d65fc09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85fc11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c25fc16, 61 },
    { 0x1145fc16, 33 },
    { 0, 0 },
    { 0x3805fc19, 23 },
    { 0, 0 },
    { 0x5225fc1b, 95 },
    { 0xa5f5fc1c, 100 },
    { 0xca65fc1c, 76 },
    { 0xfbf5fc1c, 52 },
    { 0, 0 },
    { 0xc525fc20, 90 },
    { 0xf715fc21, 61 },
    { 0xcea5fc22, 23 },
    { 0xe805fc23, 90 },
    { 0x4135fc23, 46 },
    { 0x75e5fc25, 55 },
    { 0, 0 },
    { 0xbdb5fc27, 83 },
    { 0xeda5fc28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5fc2b, 90 },
    { 0x5bc5fc2c, 90 },
    { 0x6395fc2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd5fc30, 98 },
    { 0, 0 },
    { 0xd2a5fc32, 23 },
    { 0xefb5fc33, 23 },
    { 0xd4d5fc34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605fc3a, 66 },
    { 0, 0 },
    { 0xcc95fc3c, 90 },
    { 0, 0 },
    { 0x5eb5fc3e, 87 },
    { 0, 0 },
    { 0x8cf5fc40, 95 },
    { 0xf545fc41, 76 },
    { 0, 0 },
    { 0x1985fc43, 95 },
    { 0, 0 },
    { 0x3a55fc45, 77 },
    { 0, 0 },
    { 0x9ed5fc47, 64 },
    { 0xb945fc48, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565fc4f, 33 },
    { 0, 0 },
    { 0x8ec5fc51, 100 },
    { 0xe3b5fc51, 90 },
    { 0x1bf5fc53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e5fc57, 43 },
    { 0xbef5fc58, 90 },
    { 0xb3d5fc57, 98 },
    { 0x3eb5fc57, 61 },
    { 0xf635fc5b, 78 },
    { 0x3865fc59, 90 },
    { 0x4ca5fc59, 98 },
    { 0xc435fc5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475fc61, 90 },
    { 0x60f5fc61, 61 },
    { 0x2975fc62, 90 },
    { 0x8375fc64, 67 },
    { 0xe655fc65, 55 },
    { 0x49b5fc66, 61 },
    { 0x8245fc67, 76 },
    { 0, 0 },
    { 0xaf55fc69, 98 },
    { 0, 0 },
    { 0xdbd5fc6b, 77 },
    { 0, 0 },
    { 0xc9a5fc6d, 61 },
    { 0xefb5fc6e, 84 },
    { 0x82c5fc6f, 61 },
    { 0xefb5fc70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa495fc75, 98 },
    { 0, 0 },
    { 0x7655fc77, 91 },
    { 0x7a55fc77, 55 },
    { 0, 0 },
    { 0xc515fc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd465fc7e, 56 },
    { 0x74e5fc7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5fc85, 33 },
    { 0x78c5fc85, 61 },
    { 0xd735fc87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec5fc90, 55 },
    { 0, 0 },
    { 0x8395fc92, 98 },
    { 0, 0 },
    { 0x5ef5fc94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb5fc97, 52 },
    { 0, 0 },
    { 0xeec5fc99, 75 },
    { 0x5c55fc9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd5fca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e35fca6, 52 },
    { 0xf645fca6, 95 },
    { 0xa725fca8, 61 },
    { 0xc175fca7, 46 },
    { 0x10b5fcaa, 81 },
    { 0xbd75fca8, 33 },
    { 0x2775fcac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5fcaf, 98 },
    { 0, 0 },
    { 0xde45fcb1, 33 },
    { 0x7c85fcb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85fcb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d85fcbe, 46 },
    { 0, 0 },
    { 0x1965fcc0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b45fcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3475fcc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b35fccc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055fccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6915fcd6, 23 },
    { 0x7c45fcd6, 52 },
    { 0, 0 },
    { 0x1f35fcd9, 33 },
    { 0xfa85fcda, 90 },
    { 0xe75fcdb, 90 },
    { 0x3ff5fcdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a55fcdf, 61 },
    { 0x6cf5fce0, 23 },
    { 0x8095fce1, 55 },
    { 0x3155fce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee5fce6, 65 },
    { 0, 0 },
    { 0x7345fce8, 46 },
    { 0, 0 },
    { 0x3465fcea, 23 },
    { 0x9685fcea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5185fcee, 34 },
    { 0x1ac5fcee, 98 },
    { 0, 0 },
    { 0xa455fcf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55fcf6, 98 },
    { 0xa3b5fcf6, 95 },
    { 0, 0 },
    { 0x515fcf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc45fd01, 85 },
    { 0x86c5fd02, 87 },
    { 0, 0 },
    { 0xbb25fd04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b5fd08, 85 },
    { 0xeb25fd09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55fd0e, 90 },
    { 0x5955fd0e, 26 },
    { 0xc1e5fd10, 43 },
    { 0xa9d5fd0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b15fd15, 67 },
    { 0xa855fd16, 95 },
    { 0, 0 },
    { 0xbfb5fd18, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9495fd1b, 76 },
    { 0x26e5fd1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a95fd23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d25fd2a, 33 },
    { 0, 0 },
    { 0x13c5fd2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff35fd30, 85 },
    { 0xf7a5fd31, 95 },
    { 0x8885fd32, 33 },
    { 0x9f05fd32, 61 },
    { 0xdb85fd34, 23 },
    { 0x1ee5fd30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605fd38, 23 },
    { 0xb035fd38, 90 },
    { 0, 0 },
    { 0x6f25fd3b, 90 },
    { 0, 0 },
    { 0xe935fd3d, 84 },
    { 0xaa35fd3d, 23 },
    { 0xeac5fd3d, 67 },
    { 0xed25fd40, 23 },
    { 0, 0 },
    { 0x8ee5fd42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785fd45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275fd4d, 46 },
    { 0xa445fd4e, 17 },
    { 0x8b45fd4e, 93 },
    { 0, 0 },
    { 0x54f5fd51, 40 },
    { 0x5b15fd52, 95 },
    { 0xb275fd53, 81 },
    { 0x3fe5fd53, 98 },
    { 0x4565fd55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435fd5f, 98 },
    { 0x6d35fd60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe365fd72, 97 },
    { 0, 0 },
    { 0xb535fd74, 33 },
    { 0xb25fd75, 90 },
    { 0x18d5fd76, 76 },
    { 0x8ee5fd76, 33 },
    { 0, 0 },
    { 0xbea5fd79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa535fd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c95fd83, 76 },
    { 0x3045fd83, 95 },
    { 0, 0 },
    { 0x83c5fd86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b85fd8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff25fd92, 90 },
    { 0xa35fd92, 17 },
    { 0, 0 },
    { 0x49d5fd95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5fd9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5fda1, 52 },
    { 0x3f35fda2, 23 },
    { 0x5df5fda3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d35fdaa, 95 },
    { 0xdb85fdaa, 90 },
    { 0, 0 },
    { 0x4c05fdad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45fdb1, 76 },
    { 0x1cb5fdb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8365fdb6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5805fdbc, 42 },
    { 0, 0 },
    { 0x58c5fdbe, 19 },
    { 0x5795fdbf, 90 },
    { 0xd885fdc0, 33 },
    { 0x5305fdc1, 90 },
    { 0x6f75fdc2, 84 },
    { 0xbdf5fdc3, 23 },
    { 0x9095fdc2, 33 },
    { 0x3545fdc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc435fdc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf35fdd3, 33 },
    { 0, 0 },
    { 0xa345fdd5, 61 },
    { 0x67c5fdd6, 100 },
    { 0xd9c5fdd6, 23 },
    { 0x31a5fdd6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65fddd, 33 },
    { 0x9945fdde, 69 },
    { 0xc225fddf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd5fde2, 55 },
    { 0, 0 },
    { 0x9dc5fde4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5885fdea, 34 },
    { 0x48f5fdeb, 90 },
    { 0, 0 },
    { 0xcd35fded, 61 },
    { 0xd765fdee, 84 },
    { 0, 0 },
    { 0xa405fdf0, 76 },
    { 0x8635fdf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae5fdf6, 95 },
    { 0xfc75fdf6, 95 },
    { 0, 0 },
    { 0xc8b5fdf9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae25fdfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5fe01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac75fe04, 78 },
    { 0xd5fe04, 67 },
    { 0x1415fe06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5335fe09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905fe0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1755fe12, 23 },
    { 0x1505fe13, 100 },
    { 0x34f5fe13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5fe18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4965fe21, 75 },
    { 0x3e15fe22, 61 },
    { 0, 0 },
    { 0xd545fe24, 90 },
    { 0x1725fe25, 99 },
    { 0x9f65fe26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c5fe2a, 80 },
    { 0x3595fe2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b85fe2e, 55 },
    { 0, 0 },
    { 0x1df5fe30, 98 },
    { 0xeb25fe31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a35fe36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e5fe3a, 81 },
    { 0xcdf5fe3b, 95 },
    { 0, 0 },
    { 0x8295fe3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975fe43, 65 },
    { 0x3105fe44, 33 },
    { 0, 0 },
    { 0x6565fe46, 90 },
    { 0xfa15fe47, 98 },
    { 0, 0 },
    { 0x2f75fe49, 61 },
    { 0xcea5fe49, 55 },
    { 0xa0a5fe4b, 96 },
    { 0x6505fe4c, 77 },
    { 0x8c95fe4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba35fe50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235fe53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2875fe58, 33 },
    { 0x8035fe59, 90 },
    { 0xc4d5fe58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245fe5f, 76 },
    { 0x6b05fe60, 95 },
    { 0xdd15fe60, 76 },
    { 0x275fe62, 87 },
    { 0x9ee5fe61, 34 },
    { 0x7355fe60, 34 },
    { 0, 0 },
    { 0xca55fe66, 33 },
    { 0x60d5fe67, 55 },
    { 0xfd25fe68, 23 },
    { 0xd815fe69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a35fe6e, 95 },
    { 0xe135fe6f, 55 },
    { 0x1645fe70, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5fe75, 98 },
    { 0x5e95fe76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba5fe7e, 34 },
    { 0x80e5fe7e, 69 },
    { 0xac25fe7f, 99 },
    { 0, 0 },
    { 0x4965fe82, 61 },
    { 0x4805fe83, 91 },
    { 0x8b15fe82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5fe92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5fe95, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f5fe9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3965fe9e, 33 },
    { 0x7765fe9e, 61 },
    { 0x47f5fea0, 93 },
    { 0xe125fea1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da5fea4, 67 },
    { 0x4d35fea5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca95fea9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075feb0, 76 },
    { 0, 0 },
    { 0xdfa5feb2, 17 },
    { 0x2fa5feb2, 88 },
    { 0x9525feb4, 61 },
    { 0x5d95feb4, 98 },
    { 0xf555feb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05febb, 98 },
    { 0, 0 },
    { 0x3405febd, 90 },
    { 0xc2d5febd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa145fec5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df5fece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5fed3, 40 },
    { 0xbce5fed3, 56 },
    { 0x8375fed5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215fed9, 91 },
    { 0, 0 },
    { 0xae35fedb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5fede, 64 },
    { 0x6215fedf, 90 },
    { 0x47a5fee0, 81 },
    { 0x4275fee1, 46 },
    { 0x6475fee2, 100 },
    { 0xa9a5fee3, 67 },
    { 0x45e5fee3, 61 },
    { 0, 0 },
    { 0xbd65fee6, 90 },
    { 0xce85fee7, 90 },
    { 0, 0 },
    { 0x6d85fee9, 46 },
    { 0x35fee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6195fef0, 33 },
    { 0x4da5fef1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5205fefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a5ff03, 61 },
    { 0x92a5ff03, 95 },
    { 0xa7b5ff05, 98 },
    { 0xa25ff06, 43 },
    { 0xa6f5ff07, 33 },
    { 0x6125ff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5ff0b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6035ff10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe525ff13, 61 },
    { 0xccd5ff14, 55 },
    { 0xb8e5ff14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185ff18, 88 },
    { 0x90c5ff19, 56 },
    { 0, 0 },
    { 0x61a5ff1b, 95 },
    { 0xaa25ff1c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e95ff20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85ff28, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955ff2b, 43 },
    { 0x3f15ff2c, 99 },
    { 0x1755ff2d, 78 },
    { 0xbd45ff2e, 90 },
    { 0x2a35ff2e, 23 },
    { 0xea05ff2f, 95 },
    { 0x2335ff2c, 95 },
    { 0xad45ff31, 90 },
    { 0x9ea5ff2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45ff36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1475ff3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7395ff53, 90 },
    { 0, 0 },
    { 0x4aa5ff55, 98 },
    { 0x5be5ff56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc065ff62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca85ff67, 23 },
    { 0, 0 },
    { 0x5cc5ff69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5ff6d, 95 },
    { 0x145ff6e, 95 },
    { 0x43c5ff6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25ff77, 76 },
    { 0x5f85ff78, 95 },
    { 0xb775ff79, 33 },
    { 0xef65ff7a, 99 },
    { 0xd6c5ff7b, 61 },
    { 0x18d5ff7a, 99 },
    { 0x7e15ff7d, 98 },
    { 0x9355ff7e, 95 },
    { 0xd1a5ff7f, 93 },
    { 0xcc5ff7f, 87 },
    { 0x6b5ff81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b5ff84, 43 },
    { 0x8495ff85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb895ff89, 98 },
    { 0x5105ff8a, 43 },
    { 0xd4b5ff8b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1345ff90, 23 },
    { 0x1355ff90, 76 },
    { 0x7f85ff92, 88 },
    { 0x9a95ff93, 100 },
    { 0xac5ff93, 77 },
    { 0x4a5ff95, 98 },
    { 0xec5ff96, 100 },
    { 0x9735ff97, 98 },
    { 0xa0f5ff98, 90 },
    { 0x4305ff95, 76 },
    { 0x3355ff9a, 90 },
    { 0x1b15ff9b, 66 },
    { 0xb825ff9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5395ff9f, 98 },
    { 0xb2a5ff9f, 90 },
    { 0, 0 },
    { 0x7e45ffa2, 90 },
    { 0xf065ffa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615ffa6, 90 },
    { 0, 0 },
    { 0x325ffa8, 82 },
    { 0x4395ffa8, 23 },
    { 0x41f5ffaa, 17 },
    { 0x8805ffab, 98 },
    { 0x47f5ffab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25ffb0, 23 },
    { 0x7345ffb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d55ffb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c5ffbb, 98 },
    { 0x4f25ffbc, 23 },
    { 0xaec5ffbb, 61 },
    { 0xd725ffbe, 90 },
    { 0x2845ffbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f5ffc2, 43 },
    { 0xeb55ffc3, 26 },
    { 0xe295ffc2, 90 },
    { 0x5cf5ffc5, 98 },
    { 0x48c5ffc5, 87 },
    { 0x8d5ffc5, 33 },
    { 0x1f85ffc2, 95 },
    { 0x9785ffc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa875ffd3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5ffd6, 90 },
    { 0xc865ffd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5ffdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb385ffdf, 61 },
    { 0x49d5ffdf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5ffe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb5ffe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5fff3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1255fff6, 61 },
    { 0x4995fff7, 98 },
    { 0, 0 },
    { 0x5d75fff9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35fffd, 88 },
    { 0x11e5fffe, 66 },
    { 0x1335ffff, 43 },
    { 0xbd35fffe, 34 },
    { 0xe875ffff, 99 },
    { 0x8d160002, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4460007, 55 },
    { 0x87960008, 61 },
    { 0xaf360008, 91 },
    { 0xdcb6000a, 46 },
    { 0x7316000a, 90 },
    { 0x8086000c, 61 },
    { 0, 0 },
    { 0xdd06000e, 61 },
    { 0, 0 },
    { 0x2a760010, 99 },
    { 0, 0 },
    { 0x27660012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda660015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e56001b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc326001f, 90 },
    { 0, 0 },
    { 0xb8760021, 23 },
    { 0x3f360022, 55 },
    { 0x62760023, 53 },
    { 0xf4060023, 93 },
    { 0x2f760024, 76 },
    { 0, 0 },
    { 0x92260027, 61 },
    { 0x69460027, 34 },
    { 0x5b660029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b6002f, 46 },
    { 0x8c360030, 77 },
    { 0x4f060031, 61 },
    { 0, 0 },
    { 0xaf360033, 85 },
    { 0, 0 },
    { 0xb9e60035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96360038, 100 },
    { 0, 0 },
    { 0x5186003a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836003d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde60041, 52 },
    { 0, 0 },
    { 0x82b60043, 33 },
    { 0, 0 },
    { 0x6a660045, 76 },
    { 0x4fa60046, 99 },
    { 0xad460046, 33 },
    { 0x9d160048, 33 },
    { 0x36d60047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab6004c, 100 },
    { 0x19c6004d, 64 },
    { 0x8fc6004d, 95 },
    { 0, 0 },
    { 0x5db60050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b960053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a60056, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c96005a, 46 },
    { 0, 0 },
    { 0xf686005c, 61 },
    { 0x5b06005d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a60064, 46 },
    { 0x82360065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4596006b, 95 },
    { 0x22d6006b, 61 },
    { 0xeeb6006d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab360072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e60075, 33 },
    { 0x9d960075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e60080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23160085, 23 },
    { 0, 0 },
    { 0x26760087, 55 },
    { 0, 0 },
    { 0xb1960089, 90 },
    { 0xda460089, 66 },
    { 0xa9b6008b, 88 },
    { 0x1d360089, 77 },
    { 0, 0 },
    { 0x6b86008e, 96 },
    { 0xdcf6008f, 33 },
    { 0x20260090, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c60096, 99 },
    { 0, 0 },
    { 0x5f360098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d6009d, 95 },
    { 0x9c6009e, 81 },
    { 0xfef6009f, 77 },
    { 0, 0 },
    { 0x248600a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc600a7, 95 },
    { 0, 0 },
    { 0xe39600a9, 98 },
    { 0xf5d600aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x610600ad, 33 },
    { 0x369600ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f600b5, 98 },
    { 0x57b600b5, 77 },
    { 0xd5e600b5, 100 },
    { 0x268600b6, 98 },
    { 0x8ea600b9, 33 },
    { 0x421600b5, 43 },
    { 0x260600b7, 61 },
    { 0x76a600bc, 87 },
    { 0x74d600bc, 76 },
    { 0x57b600bb, 52 },
    { 0xc5f600bf, 17 },
    { 0xd34600c0, 90 },
    { 0xd54600bf, 33 },
    { 0x274600c0, 98 },
    { 0xd52600c3, 87 },
    { 0x9bf600c4, 95 },
    { 0x285600c3, 98 },
    { 0x30a600b5, 33 },
    { 0xf67600c7, 61 },
    { 0x3f7600c8, 34 },
    { 0xba1600c9, 33 },
    { 0x32b600ca, 33 },
    { 0x898600cb, 61 },
    { 0x1e5600cc, 33 },
    { 0x2d0600ca, 61 },
    { 0x123600c5, 100 },
    { 0x95e600cf, 34 },
    { 0x2d600d0, 81 },
    { 0xf16600d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582600db, 90 },
    { 0x625600dc, 33 },
    { 0xdf5600dd, 87 },
    { 0x7fc600dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17600e4, 34 },
    { 0, 0 },
    { 0x77b600e6, 33 },
    { 0, 0 },
    { 0xe28600e8, 93 },
    { 0x153600e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3600ec, 91 },
    { 0, 0 },
    { 0x134600ee, 90 },
    { 0x224600ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3600f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c600f6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x550600f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f60100, 61 },
    { 0x56060100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0260108, 70 },
    { 0, 0 },
    { 0x77f6010a, 99 },
    { 0x51a6010b, 90 },
    { 0, 0 },
    { 0xc386010d, 61 },
    { 0, 0 },
    { 0x1a86010f, 34 },
    { 0x6ad6010f, 95 },
    { 0xd2360110, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99560116, 91 },
    { 0x41e60117, 90 },
    { 0xd5960116, 33 },
    { 0xd8560118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef760121, 70 },
    { 0x37360122, 95 },
    { 0xbf260123, 90 },
    { 0x4a960124, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f60127, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b6012a, 98 },
    { 0, 0 },
    { 0x1b26012c, 98 },
    { 0x6716012d, 34 },
    { 0xcd86012c, 90 },
    { 0, 0 },
    { 0xa8360130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa796013b, 100 },
    { 0x7be6013c, 55 },
    { 0x366013d, 91 },
    { 0xf86013d, 61 },
    { 0x7ef6013f, 11 },
    { 0xf1a60140, 90 },
    { 0xd5b60141, 33 },
    { 0x3d560141, 61 },
    { 0, 0 },
    { 0x48160144, 23 },
    { 0x36960145, 91 },
    { 0, 0 },
    { 0xd8c60147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a6014c, 77 },
    { 0xdf56014d, 61 },
    { 0, 0 },
    { 0x2f56014f, 90 },
    { 0x2cc6014f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6560153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb676015b, 67 },
    { 0x4766015c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46015f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26560163, 76 },
    { 0x48260164, 33 },
    { 0xf6260165, 52 },
    { 0x6160164, 43 },
    { 0, 0 },
    { 0x88460168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf660176, 93 },
    { 0x46060177, 84 },
    { 0xf8260177, 76 },
    { 0x7860176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3716017c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce460181, 99 },
    { 0xcbd60182, 61 },
    { 0, 0 },
    { 0x27460184, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d6018a, 46 },
    { 0x29f6018a, 75 },
    { 0x8996018c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156018f, 90 },
    { 0xc1d60190, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7360194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6460197, 33 },
    { 0x97560198, 98 },
    { 0x1ee60198, 95 },
    { 0, 0 },
    { 0x2d86019b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362601a3, 90 },
    { 0, 0 },
    { 0x508601a5, 78 },
    { 0x81e601a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30601a9, 34 },
    { 0, 0 },
    { 0x432601ab, 33 },
    { 0x7f7601ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0601b0, 90 },
    { 0x571601b1, 87 },
    { 0xa3601b2, 43 },
    { 0xd15601b3, 23 },
    { 0x691601b4, 91 },
    { 0xff9601b0, 78 },
    { 0x34c601b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22601ba, 46 },
    { 0x42f601bb, 46 },
    { 0xd61601ba, 98 },
    { 0, 0 },
    { 0xb18601be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5601c4, 95 },
    { 0x163601c5, 90 },
    { 0, 0 },
    { 0x861601c7, 43 },
    { 0, 0 },
    { 0x40601c9, 55 },
    { 0x658601ca, 99 },
    { 0xf2601c9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e601ce, 100 },
    { 0xde6601cf, 88 },
    { 0x270601d0, 55 },
    { 0, 0 },
    { 0x678601d2, 87 },
    { 0x2c2601d3, 33 },
    { 0xca5601d2, 100 },
    { 0xeb0601d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee601da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57601e4, 93 },
    { 0x2bd601e5, 76 },
    { 0xf54601e5, 95 },
    { 0xe42601e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x227601ea, 34 },
    { 0, 0 },
    { 0x73f601ec, 96 },
    { 0x2c1601ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d601f3, 23 },
    { 0xebb601f4, 76 },
    { 0xe35601f5, 76 },
    { 0, 0 },
    { 0x3ed601f7, 52 },
    { 0x28a601f7, 95 },
    { 0x70d601f9, 90 },
    { 0xfa2601fa, 99 },
    { 0, 0 },
    { 0x310601fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc601ff, 61 },
    { 0xc8260200, 23 },
    { 0, 0 },
    { 0x39d60202, 75 },
    { 0xe5860203, 55 },
    { 0x4f560203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9960208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f66020c, 90 },
    { 0, 0 },
    { 0x6a36020e, 99 },
    { 0x3686020f, 76 },
    { 0x956020e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b960215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d56021a, 78 },
    { 0x1326021b, 98 },
    { 0, 0 },
    { 0x9146021d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5660220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7160224, 33 },
    { 0, 0 },
    { 0x4fa60226, 33 },
    { 0xf5e60227, 23 },
    { 0x6bb60227, 33 },
    { 0x85e60229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8236022d, 91 },
    { 0xccc6022e, 61 },
    { 0x7fd6022e, 23 },
    { 0x7e60230, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee60238, 34 },
    { 0, 0 },
    { 0x4d06023a, 90 },
    { 0, 0 },
    { 0xa286023c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39260240, 100 },
    { 0, 0 },
    { 0x74c60242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7760246, 98 },
    { 0, 0 },
    { 0x61a60248, 34 },
    { 0, 0 },
    { 0xd536024a, 98 },
    { 0, 0 },
    { 0xabf6024c, 61 },
    { 0, 0 },
    { 0x59d6024e, 95 },
    { 0x64a6024f, 55 },
    { 0, 0 },
    { 0x5e60251, 98 },
    { 0, 0 },
    { 0xde060253, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b260257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546025b, 33 },
    { 0xe296025c, 95 },
    { 0x5c6025d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e560264, 99 },
    { 0x80c60264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d160268, 33 },
    { 0x54360269, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a6026f, 61 },
    { 0x34e60270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb6027a, 95 },
    { 0x1d16027b, 93 },
    { 0x886027c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad60287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6028a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a60294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f60298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce16029b, 95 },
    { 0, 0 },
    { 0x6916029d, 61 },
    { 0x6a6029d, 98 },
    { 0xc566029d, 84 },
    { 0xddf6029e, 90 },
    { 0xa82602a1, 33 },
    { 0x7886029f, 90 },
    { 0, 0 },
    { 0x534602a4, 67 },
    { 0xf5e602a4, 90 },
    { 0xf17602a6, 74 },
    { 0x663602a7, 95 },
    { 0x7f7602a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275602b3, 90 },
    { 0x938602b3, 99 },
    { 0x6f0602b5, 61 },
    { 0x5b3602b4, 78 },
    { 0x20b602b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x290602ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a602bd, 91 },
    { 0x547602bd, 76 },
    { 0xf6b602bf, 56 },
    { 0, 0 },
    { 0x384602c1, 98 },
    { 0, 0 },
    { 0x84b602c3, 90 },
    { 0xaaa602c4, 95 },
    { 0xd43602c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e602cb, 61 },
    { 0, 0 },
    { 0xed1602cd, 61 },
    { 0, 0 },
    { 0x82b602cf, 23 },
    { 0xf3c602cf, 77 },
    { 0x671602cf, 90 },
    { 0x312602d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682602d6, 33 },
    { 0x429602d7, 91 },
    { 0, 0 },
    { 0xe8e602d9, 67 },
    { 0x293602d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f602dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x629602e0, 67 },
    { 0x531602e1, 40 },
    { 0x6a4602e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e602e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a602ea, 33 },
    { 0x41e602eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd602f2, 76 },
    { 0x188602f2, 90 },
    { 0, 0 },
    { 0x90a602f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2602f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf602fe, 33 },
    { 0x268602ff, 61 },
    { 0xd1660300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1060303, 99 },
    { 0x62f60304, 90 },
    { 0xf1e60304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa866030a, 100 },
    { 0x5f66030b, 61 },
    { 0x3806030a, 95 },
    { 0xf276030b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce60310, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d60313, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4960319, 78 },
    { 0xa726031a, 90 },
    { 0xc996031b, 84 },
    { 0x9c60319, 33 },
    { 0x9376031d, 81 },
    { 0x63a60319, 61 },
    { 0x8606031f, 98 },
    { 0x82d6031b, 55 },
    { 0, 0 },
    { 0xa060322, 90 },
    { 0xa4a60323, 68 },
    { 0xc6860322, 61 },
    { 0xebc60325, 34 },
    { 0, 0 },
    { 0x5eb60327, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc086032b, 66 },
    { 0xaee6032c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e60335, 33 },
    { 0x1960336, 33 },
    { 0x1f760336, 43 },
    { 0x84860336, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316033c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f260340, 68 },
    { 0, 0 },
    { 0x54760342, 90 },
    { 0, 0 },
    { 0x8d160344, 76 },
    { 0x75560345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14960348, 33 },
    { 0, 0 },
    { 0xa106034a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6034d, 46 },
    { 0x816034d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb060351, 33 },
    { 0x2c60352, 40 },
    { 0x4f760352, 95 },
    { 0xa5860354, 23 },
    { 0xa9560352, 80 },
    { 0x4e060356, 34 },
    { 0x5cc60357, 46 },
    { 0x12060358, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c96035d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67860361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a560365, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8460369, 90 },
    { 0x19e6036a, 85 },
    { 0xa66036b, 95 },
    { 0xa7f60369, 17 },
    { 0, 0 },
    { 0xb8c6036e, 23 },
    { 0x7076036e, 34 },
    { 0xea86036f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49960373, 84 },
    { 0xd2760374, 33 },
    { 0xe6e60373, 95 },
    { 0xf8260376, 98 },
    { 0, 0 },
    { 0x44860378, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6037c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41960380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b60383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f360386, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe560389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276038d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7860391, 98 },
    { 0x1660391, 90 },
    { 0xb2f60393, 95 },
    { 0xba960394, 61 },
    { 0x4d860393, 23 },
    { 0xec960396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad860399, 34 },
    { 0x1e66039a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc266039e, 90 },
    { 0, 0 },
    { 0xc50603a0, 95 },
    { 0xc33603a0, 76 },
    { 0x69f603a2, 98 },
    { 0x373603a1, 38 },
    { 0xd63603a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d603a7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4603aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb603b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c603b4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93603b8, 68 },
    { 0x8e9603b8, 90 },
    { 0x1f7603b8, 23 },
    { 0xdfc603b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0603c0, 93 },
    { 0, 0 },
    { 0xaf5603c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e603c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e603ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce603cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c603d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958603d5, 95 },
    { 0, 0 },
    { 0x5ea603d7, 90 },
    { 0, 0 },
    { 0x3cc603d9, 46 },
    { 0, 0 },
    { 0x203603db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329603e5, 80 },
    { 0xb50603e6, 61 },
    { 0x655603e7, 93 },
    { 0xc41603e7, 52 },
    { 0x7cf603e9, 91 },
    { 0xdff603e7, 34 },
    { 0x234603e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae603ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763603f2, 99 },
    { 0xa8a603f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75603f7, 33 },
    { 0x7603f8, 33 },
    { 0x9b603f9, 91 },
    { 0x471603fa, 43 },
    { 0x31d603f7, 23 },
    { 0xfa4603f7, 61 },
    { 0xbbc603fd, 61 },
    { 0x51e603fe, 23 },
    { 0, 0 },
    { 0x6cb60400, 76 },
    { 0x1b60400, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21360404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76460408, 65 },
    { 0x2ea60409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f96040c, 93 },
    { 0x43b6040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25560411, 95 },
    { 0x3f60412, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72560417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6041a, 61 },
    { 0xdab6041b, 17 },
    { 0, 0 },
    { 0x76e6041d, 75 },
    { 0x3326041d, 34 },
    { 0xd6041f, 90 },
    { 0x7da60420, 93 },
    { 0xde260420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52760424, 66 },
    { 0x57f60425, 17 },
    { 0x2c460425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b76042a, 90 },
    { 0, 0 },
    { 0x6b06042c, 95 },
    { 0x12c6042d, 90 },
    { 0x3a46042d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7660431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c960434, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d60437, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a6043b, 78 },
    { 0x2576043c, 37 },
    { 0, 0 },
    { 0xeac6043e, 76 },
    { 0, 0 },
    { 0x80760440, 91 },
    { 0x64d60441, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3260444, 84 },
    { 0x49b60445, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2376044d, 99 },
    { 0x4f86044e, 66 },
    { 0x2d76044f, 46 },
    { 0x33d60450, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x35660453, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97060457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36045b, 90 },
    { 0, 0 },
    { 0x1d36045d, 61 },
    { 0x33f6045e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb960462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436046c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d60473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf60476, 77 },
    { 0, 0 },
    { 0x9f960478, 93 },
    { 0x12c60479, 61 },
    { 0x33060478, 34 },
    { 0x5996047b, 98 },
    { 0x51b6047c, 68 },
    { 0x3036047d, 95 },
    { 0x51b6047e, 61 },
    { 0x20e6047e, 90 },
    { 0x1cb60480, 75 },
    { 0x9f660478, 95 },
    { 0x7bd60482, 81 },
    { 0xf2660483, 23 },
    { 0x19960484, 46 },
    { 0x9b260484, 95 },
    { 0x9c260486, 64 },
    { 0x26660485, 75 },
    { 0x57660488, 23 },
    { 0xac860489, 90 },
    { 0x3db6048a, 61 },
    { 0xc9a6048a, 43 },
    { 0x5f060482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d760490, 90 },
    { 0xcdd60491, 33 },
    { 0x1c360491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c60499, 23 },
    { 0, 0 },
    { 0x4586049b, 61 },
    { 0x6726049c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5604a0, 98 },
    { 0xf79604a1, 61 },
    { 0x70f604a2, 76 },
    { 0xbcf604a3, 98 },
    { 0x6d4604a3, 90 },
    { 0xbd4604a5, 98 },
    { 0x10b604a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49604aa, 55 },
    { 0, 0 },
    { 0xa23604ac, 23 },
    { 0xf2b604ac, 52 },
    { 0x2be604ae, 75 },
    { 0xa49604ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8604b6, 95 },
    { 0, 0 },
    { 0xc7604b8, 43 },
    { 0xb32604b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6604bd, 36 },
    { 0xf65604bd, 76 },
    { 0, 0 },
    { 0xaaa604c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6604c6, 33 },
    { 0x28e604c7, 90 },
    { 0x1ee604c8, 23 },
    { 0x7ef604c8, 95 },
    { 0x124604c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b604cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x210604d2, 61 },
    { 0, 0 },
    { 0x26b604d4, 90 },
    { 0x74b604d4, 90 },
    { 0x6de604d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22604d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4604dc, 95 },
    { 0xfad604dd, 98 },
    { 0, 0 },
    { 0xbca604df, 52 },
    { 0x8de604e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638604e7, 33 },
    { 0x2a7604e8, 61 },
    { 0xa2604e9, 23 },
    { 0x1e604ea, 33 },
    { 0, 0 },
    { 0x259604ec, 52 },
    { 0xb93604ed, 90 },
    { 0x1bc604ee, 81 },
    { 0x55d604ef, 90 },
    { 0, 0 },
    { 0x26604f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd604f8, 90 },
    { 0x56604f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e604fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c604ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd60506, 95 },
    { 0x56b60507, 98 },
    { 0x4db60508, 75 },
    { 0xae60509, 95 },
    { 0xb9c60509, 98 },
    { 0x5a26050b, 95 },
    { 0x6876050c, 23 },
    { 0xe1a6050c, 92 },
    { 0x6ed60508, 61 },
    { 0xa4f6050f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd760516, 33 },
    { 0x14360517, 98 },
    { 0x1560518, 55 },
    { 0xdfb60519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9066051e, 61 },
    { 0, 0 },
    { 0x7ea60520, 95 },
    { 0, 0 },
    { 0x96b60522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc60528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9186052c, 46 },
    { 0x70d6052c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa960531, 33 },
    { 0, 0 },
    { 0x8d560533, 95 },
    { 0x37e60534, 81 },
    { 0, 0 },
    { 0xa260536, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a260539, 33 },
    { 0x7026053a, 84 },
    { 0x3ad60539, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6053f, 91 },
    { 0x79e60540, 33 },
    { 0x79a60540, 43 },
    { 0x15460541, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead6054b, 90 },
    { 0x71a6054b, 95 },
    { 0x8406054b, 43 },
    { 0xbea6054e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b60553, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8146055b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94260564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6056a, 61 },
    { 0x5d96056a, 46 },
    { 0, 0 },
    { 0x6b66056d, 91 },
    { 0xbf16056e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d60572, 87 },
    { 0xb3f60573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76057d, 93 },
    { 0x4ea6057d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a260586, 80 },
    { 0xc2560586, 23 },
    { 0, 0 },
    { 0xfce60589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a6058c, 100 },
    { 0xb376058d, 90 },
    { 0xf226058d, 61 },
    { 0xc076058e, 23 },
    { 0, 0 },
    { 0xee460591, 77 },
    { 0x61d60591, 34 },
    { 0, 0 },
    { 0x95160594, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57660597, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b76059b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601605a4, 46 },
    { 0x2e6605a5, 33 },
    { 0xbc8605a5, 80 },
    { 0xbb4605a7, 23 },
    { 0x4db605a7, 33 },
    { 0x365605a9, 61 },
    { 0xf69605aa, 37 },
    { 0x9af605aa, 33 },
    { 0, 0 },
    { 0xc82605ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4605b0, 34 },
    { 0x19e605b1, 90 },
    { 0, 0 },
    { 0xff0605b3, 95 },
    { 0x1f5605b3, 33 },
    { 0xd50605b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4605b8, 78 },
    { 0, 0 },
    { 0xb83605ba, 91 },
    { 0x195605bb, 55 },
    { 0xad1605bb, 33 },
    { 0xe85605ba, 33 },
    { 0, 0 },
    { 0xefc605bf, 55 },
    { 0, 0 },
    { 0xb87605c1, 99 },
    { 0x9a6605c2, 77 },
    { 0, 0 },
    { 0x512605c4, 55 },
    { 0, 0 },
    { 0xfad605c6, 33 },
    { 0xb43605c6, 23 },
    { 0x555605c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3605cb, 23 },
    { 0, 0 },
    { 0x6a5605cd, 97 },
    { 0x73c605cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716605db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93605e5, 93 },
    { 0, 0 },
    { 0x772605e7, 77 },
    { 0, 0 },
    { 0x95f605e9, 91 },
    { 0, 0 },
    { 0x4d9605eb, 81 },
    { 0xe64605ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb605ef, 90 },
    { 0, 0 },
    { 0xd28605f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a605f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387605ff, 37 },
    { 0x12360600, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e60604, 90 },
    { 0, 0 },
    { 0x78c60606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726060c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d6060f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7460614, 90 },
    { 0x25d60615, 76 },
    { 0x39e60614, 61 },
    { 0xc7260617, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826061e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c60621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e60626, 78 },
    { 0x36c60627, 90 },
    { 0, 0 },
    { 0x10b60629, 43 },
    { 0, 0 },
    { 0xa26062b, 76 },
    { 0x5976062b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10860631, 77 },
    { 0x79060632, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2560639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c26063d, 85 },
    { 0xa166063e, 100 },
    { 0x8396063f, 90 },
    { 0x31c6063f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e60644, 61 },
    { 0xc4d60645, 46 },
    { 0xf6260645, 90 },
    { 0xf8260647, 34 },
    { 0, 0 },
    { 0x3ef60649, 65 },
    { 0x80360649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496064f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb860655, 90 },
    { 0xf3560656, 33 },
    { 0x7260657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b6065b, 90 },
    { 0x8646065b, 46 },
    { 0x3d76065d, 33 },
    { 0x5366065d, 76 },
    { 0x56b6065c, 76 },
    { 0x71060660, 55 },
    { 0x22e6065f, 90 },
    { 0, 0 },
    { 0x61760663, 95 },
    { 0x8a60664, 93 },
    { 0x4ba60665, 23 },
    { 0xaf260666, 52 },
    { 0x23060663, 91 },
    { 0xdc560666, 34 },
    { 0x84f60667, 17 },
    { 0xbf260665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7976066f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10560673, 33 },
    { 0xb0860673, 34 },
    { 0x5b60675, 98 },
    { 0x4e060675, 23 },
    { 0xf2060675, 75 },
    { 0, 0 },
    { 0xae060679, 84 },
    { 0, 0 },
    { 0x46f6067b, 23 },
    { 0, 0 },
    { 0x1e46067d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb60680, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c60683, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa460686, 76 },
    { 0x9aa60687, 90 },
    { 0xe8060688, 46 },
    { 0x5fd60689, 98 },
    { 0xf6b6068a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c660692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f60695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd706069b, 93 },
    { 0xe936069b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52606a1, 33 },
    { 0xb5606a1, 23 },
    { 0x2ad606a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a606a8, 76 },
    { 0xb83606a9, 90 },
    { 0, 0 },
    { 0x497606ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e606b0, 66 },
    { 0x4ce606b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e606b4, 53 },
    { 0x698606b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41606b9, 91 },
    { 0xa40606b9, 52 },
    { 0, 0 },
    { 0xe87606bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366606c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d606c3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb606cf, 88 },
    { 0, 0 },
    { 0x1c7606d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c606d8, 23 },
    { 0x52b606d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c606df, 90 },
    { 0x1ae606e0, 90 },
    { 0x828606df, 61 },
    { 0xaea606e0, 81 },
    { 0x8ee606df, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8606e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db606ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23606ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc606f5, 55 },
    { 0x3ec606f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a606fb, 52 },
    { 0x4a7606fc, 55 },
    { 0x26d606fd, 90 },
    { 0xbca606fb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3860701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30160705, 95 },
    { 0xf6360706, 91 },
    { 0x1c860707, 98 },
    { 0xc1d60708, 68 },
    { 0xd1b60708, 33 },
    { 0, 0 },
    { 0xdea6070b, 46 },
    { 0xab76070c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18260712, 33 },
    { 0, 0 },
    { 0xb4960714, 76 },
    { 0x48d60714, 55 },
    { 0xdc660716, 34 },
    { 0x88b60716, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a6071c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e60720, 55 },
    { 0, 0 },
    { 0x5ed60722, 66 },
    { 0, 0 },
    { 0x32960724, 33 },
    { 0xa8f60725, 66 },
    { 0xf760726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6072a, 76 },
    { 0xe306072b, 90 },
    { 0, 0 },
    { 0x6686072d, 76 },
    { 0x8f06072d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f60735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec60739, 95 },
    { 0x1c16073a, 99 },
    { 0, 0 },
    { 0x2706073c, 87 },
    { 0, 0 },
    { 0xebe6073e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf696074b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f260757, 90 },
    { 0, 0 },
    { 0x2dc60759, 23 },
    { 0xb956075a, 93 },
    { 0x5146075a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b60761, 90 },
    { 0, 0 },
    { 0x12b60763, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68460767, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36076f, 95 },
    { 0, 0 },
    { 0xb5f60771, 61 },
    { 0, 0 },
    { 0x47e60773, 55 },
    { 0xeab60774, 95 },
    { 0x8c460773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3960779, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6786077d, 84 },
    { 0x6846077d, 95 },
    { 0x3706077f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c60782, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb560785, 91 },
    { 0xa1860786, 98 },
    { 0xf5e60787, 23 },
    { 0x13160787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4360795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa46079b, 61 },
    { 0x33f6079b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4646079f, 90 },
    { 0x95c607a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428607a6, 95 },
    { 0, 0 },
    { 0xe8d607a8, 17 },
    { 0x93f607a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a607ad, 83 },
    { 0x32607ad, 84 },
    { 0x4a7607af, 90 },
    { 0, 0 },
    { 0x8d607b1, 23 },
    { 0x28c607b2, 90 },
    { 0, 0 },
    { 0xbd7607b4, 81 },
    { 0x5f0607b4, 33 },
    { 0xe75607b6, 87 },
    { 0, 0 },
    { 0xda2607b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8607bb, 33 },
    { 0xf85607bb, 98 },
    { 0x681607bd, 52 },
    { 0xbd607bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09607c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14607c4, 90 },
    { 0, 0 },
    { 0x845607c6, 98 },
    { 0x738607c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473607cb, 55 },
    { 0x73e607cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d607d0, 95 },
    { 0xf42607d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b607d4, 90 },
    { 0x809607d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a607d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61607dd, 35 },
    { 0, 0 },
    { 0x9cc607df, 33 },
    { 0xe6f607e0, 76 },
    { 0x866607e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6607e4, 77 },
    { 0x8af607e4, 68 },
    { 0x6ca607e4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b607eb, 98 },
    { 0xe2b607ec, 90 },
    { 0xeee607ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8607f4, 34 },
    { 0x360607f5, 90 },
    { 0x8af607f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4607f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x856607fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b60803, 81 },
    { 0xfdd60804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71660807, 34 },
    { 0, 0 },
    { 0x96d60809, 85 },
    { 0xaa260809, 33 },
    { 0xd36080b, 23 },
    { 0x1666080c, 46 },
    { 0x5126080d, 90 },
    { 0x5cd6080d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d460812, 76 },
    { 0x9ae60813, 78 },
    { 0xf7460813, 90 },
    { 0xa7660815, 56 },
    { 0xe0860815, 99 },
    { 0x92760817, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436081b, 100 },
    { 0xfe06081c, 61 },
    { 0xf8a6081d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96260821, 93 },
    { 0, 0 },
    { 0xcd560823, 46 },
    { 0x3b360824, 100 },
    { 0x4dd60825, 68 },
    { 0x64860826, 61 },
    { 0, 0 },
    { 0xc0860828, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa496082b, 78 },
    { 0xae96082b, 40 },
    { 0x6d6082d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe660830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b60836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a760841, 90 },
    { 0xbc260842, 90 },
    { 0xdb060843, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae60846, 98 },
    { 0x41960847, 55 },
    { 0, 0 },
    { 0x35660849, 61 },
    { 0x7456084a, 23 },
    { 0xbb56084a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d60851, 43 },
    { 0xa5f60852, 68 },
    { 0xccf60853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66085d, 46 },
    { 0x6be6085d, 99 },
    { 0x9976085e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87360863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c60869, 87 },
    { 0xe6a60869, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f6086e, 100 },
    { 0, 0 },
    { 0xf2660870, 23 },
    { 0x54160871, 98 },
    { 0x5d660872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab60875, 23 },
    { 0x18d60875, 33 },
    { 0, 0 },
    { 0xeab60878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6087b, 98 },
    { 0xd0e6087b, 33 },
    { 0xd266087b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60880, 33 },
    { 0x53660881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3860885, 98 },
    { 0x92060885, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6660890, 23 },
    { 0, 0 },
    { 0x90260892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6089b, 65 },
    { 0, 0 },
    { 0xd306089d, 76 },
    { 0x44d6089e, 90 },
    { 0xf766089f, 75 },
    { 0x6dc608a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0608a4, 95 },
    { 0, 0 },
    { 0x6cf608a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f608ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3608b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5608b6, 93 },
    { 0x98c608b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de608ba, 61 },
    { 0x11608bb, 90 },
    { 0x69b608bc, 65 },
    { 0, 0 },
    { 0x4a608be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e608c3, 98 },
    { 0x5d9608c4, 99 },
    { 0xa02608c4, 23 },
    { 0x198608c3, 33 },
    { 0, 0 },
    { 0x1dc608c8, 24 },
    { 0, 0 },
    { 0xc17608ca, 76 },
    { 0x2db608ca, 76 },
    { 0, 0 },
    { 0x3e1608cd, 90 },
    { 0xfd3608ce, 100 },
    { 0, 0 },
    { 0x61d608d0, 55 },
    { 0x175608d1, 55 },
    { 0x29608d2, 90 },
    { 0, 0 },
    { 0xcab608d4, 99 },
    { 0xa18608d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66608e1, 17 },
    { 0, 0 },
    { 0xdc4608e3, 90 },
    { 0x1a6608e4, 55 },
    { 0xb6b608e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b608ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e608f3, 84 },
    { 0, 0 },
    { 0x741608f5, 23 },
    { 0, 0 },
    { 0x7d4608f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f608fa, 98 },
    { 0xa4b608fa, 61 },
    { 0, 0 },
    { 0x953608fd, 95 },
    { 0xd97608fe, 80 },
    { 0xb6608ff, 91 },
    { 0xedd60900, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19360904, 91 },
    { 0x26260904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc436090a, 90 },
    { 0x7956090b, 95 },
    { 0xc366090c, 99 },
    { 0, 0 },
    { 0xb466090e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38060911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e60919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb86091d, 61 },
    { 0x1d26091e, 61 },
    { 0xb9f6091f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91660924, 90 },
    { 0, 0 },
    { 0x27160926, 23 },
    { 0x91e60927, 23 },
    { 0x4cf60927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf516092b, 90 },
    { 0, 0 },
    { 0x3486092d, 55 },
    { 0, 0 },
    { 0x7f66092f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c60933, 43 },
    { 0x40c60933, 34 },
    { 0x3a60935, 78 },
    { 0x63460933, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89260940, 76 },
    { 0x74360940, 90 },
    { 0x52060941, 55 },
    { 0xdef60940, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0260947, 62 },
    { 0x59260947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f60951, 67 },
    { 0xc8d60952, 39 },
    { 0x98b60951, 33 },
    { 0x28160953, 85 },
    { 0xf4c60955, 33 },
    { 0x76360956, 90 },
    { 0x5cf60952, 98 },
    { 0xe2460958, 56 },
    { 0x10060952, 87 },
    { 0x4ed60958, 76 },
    { 0x30260951, 43 },
    { 0xee6095c, 90 },
    { 0xcd56095c, 33 },
    { 0x49e60951, 98 },
    { 0xa226095f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1560962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60966, 46 },
    { 0xbd860966, 84 },
    { 0x33f60968, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab86096d, 23 },
    { 0xcbf6096e, 76 },
    { 0, 0 },
    { 0xfe360970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61460976, 98 },
    { 0x7d060977, 64 },
    { 0x20360977, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d6097c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76097f, 90 },
    { 0x7d960980, 84 },
    { 0x97260981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60985, 90 },
    { 0x7660985, 33 },
    { 0x7a760985, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c06098c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18160993, 33 },
    { 0, 0 },
    { 0x5e860995, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d60999, 98 },
    { 0, 0 },
    { 0xe676099b, 58 },
    { 0, 0 },
    { 0xf516099d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4609a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54609a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18609aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x162609ad, 23 },
    { 0x646609ad, 84 },
    { 0x872609af, 53 },
    { 0, 0 },
    { 0xb81609b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0609b5, 33 },
    { 0x7d8609b5, 93 },
    { 0x963609b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x870609ba, 33 },
    { 0, 0 },
    { 0x609bc, 61 },
    { 0x2f3609bd, 90 },
    { 0x4d1609bd, 100 },
    { 0xf4f609be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff609c4, 17 },
    { 0x3e0609c5, 33 },
    { 0xc13609c4, 90 },
    { 0xe95609c5, 84 },
    { 0xe84609c4, 43 },
    { 0x6ad609c9, 23 },
    { 0, 0 },
    { 0xb26609cb, 34 },
    { 0, 0 },
    { 0x33c609cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e609d0, 33 },
    { 0x4c6609d1, 61 },
    { 0x75d609d1, 43 },
    { 0xa39609d3, 55 },
    { 0, 0 },
    { 0xe9c609d5, 90 },
    { 0x36f609d6, 23 },
    { 0x10c609d7, 61 },
    { 0xf8f609d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8609db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296609e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4609ec, 23 },
    { 0x28e609ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597609f6, 33 },
    { 0, 0 },
    { 0x36f609f8, 90 },
    { 0x654609f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c609fd, 81 },
    { 0x837609fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f60a11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5460a14, 43 },
    { 0, 0 },
    { 0x1f160a16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31260a19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c60a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e60a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5360a26, 66 },
    { 0x26560a27, 76 },
    { 0xd3660a26, 90 },
    { 0x89560a29, 34 },
    { 0xde660a2a, 90 },
    { 0, 0 },
    { 0x30c60a2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7160a30, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf460a38, 81 },
    { 0, 0 },
    { 0xcce60a3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72160a41, 43 },
    { 0x6560a42, 93 },
    { 0xb8d60a42, 90 },
    { 0x84760a44, 76 },
    { 0, 0 },
    { 0x85d60a46, 77 },
    { 0x3ac60a47, 95 },
    { 0, 0 },
    { 0xbbf60a49, 36 },
    { 0x9bd60a4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81460a4f, 70 },
    { 0xd0760a50, 99 },
    { 0x97f60a51, 61 },
    { 0x80e60a4f, 61 },
    { 0x66560a53, 46 },
    { 0xb5a60a54, 90 },
    { 0x4060a54, 90 },
    { 0, 0 },
    { 0xdec60a57, 95 },
    { 0x84260a57, 98 },
    { 0xa9660a59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a60a62, 100 },
    { 0x95e60a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b60a66, 36 },
    { 0, 0 },
    { 0x82660a68, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49360a6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4160a72, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x20860a75, 66 },
    { 0, 0 },
    { 0x2a660a77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b60a7a, 90 },
    { 0x67260a7b, 76 },
    { 0xe1d60a7b, 61 },
    { 0x10c60a7d, 98 },
    { 0, 0 },
    { 0xceb60a7f, 61 },
    { 0, 0 },
    { 0xf8d60a81, 84 },
    { 0xd0660a81, 34 },
    { 0, 0 },
    { 0xfae60a84, 76 },
    { 0x59b60a85, 80 },
    { 0x10a60a86, 91 },
    { 0x38760a87, 38 },
    { 0x17160a88, 61 },
    { 0x42360a89, 36 },
    { 0x7a460a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7060a8d, 98 },
    { 0x5cf60a8d, 98 },
    { 0xe3f60a8f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d60a93, 90 },
    { 0x3ce60a93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda460a9c, 33 },
    { 0x7ad60a9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3860aa0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1860aa5, 23 },
    { 0x9360aa6, 32 },
    { 0x22d60aa7, 61 },
    { 0x8660aa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16260ab1, 66 },
    { 0x1cc60ab1, 33 },
    { 0x9b660ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f760abb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20060abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62060ac4, 40 },
    { 0xd7a60ac4, 76 },
    { 0xb7560ac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8660aca, 98 },
    { 0, 0 },
    { 0x2e060acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda60ad3, 34 },
    { 0xc2b60ad4, 75 },
    { 0xe7460ad3, 43 },
    { 0x72860ad4, 90 },
    { 0xede60ad7, 23 },
    { 0xa7660ad8, 75 },
    { 0x87260ad6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc360ade, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e560ae1, 61 },
    { 0, 0 },
    { 0xaa560ae3, 34 },
    { 0x91260ae4, 67 },
    { 0x83f60ae5, 91 },
    { 0xa0360ae6, 23 },
    { 0x80660ae5, 33 },
    { 0x46460ae8, 33 },
    { 0xba060ae6, 61 },
    { 0xd3f60aea, 90 },
    { 0xd3e60aea, 55 },
    { 0xed960ae5, 90 },
    { 0xa6560ae7, 43 },
    { 0, 0 },
    { 0x45060aef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff160af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55960af8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10760afc, 23 },
    { 0x3ea60afd, 91 },
    { 0x67160afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c760b03, 61 },
    { 0x42b60b04, 34 },
    { 0, 0 },
    { 0x1fc60b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a60b0c, 76 },
    { 0x8760b0d, 23 },
    { 0x89860b0e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5860b13, 33 },
    { 0xd0160b14, 68 },
    { 0x93c60b14, 61 },
    { 0xacc60b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9060b1a, 90 },
    { 0, 0 },
    { 0xc5560b1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6960b20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a060b25, 88 },
    { 0x2f960b26, 85 },
    { 0x3e160b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb60b30, 43 },
    { 0x61260b31, 23 },
    { 0x7d60b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76260b35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8960b3d, 95 },
    { 0x29560b3e, 84 },
    { 0xc5e60b3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30060b46, 93 },
    { 0, 0 },
    { 0x22860b48, 100 },
    { 0, 0 },
    { 0xfc160b4a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34360b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec60b55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a560b58, 34 },
    { 0, 0 },
    { 0xa5060b5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee60b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf60b63, 66 },
    { 0x97c60b63, 90 },
    { 0x23860b65, 100 },
    { 0, 0 },
    { 0x95660b67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xed860b6a, 78 },
    { 0x47260b6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e060b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e760b73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9160b7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc960b7f, 23 },
    { 0x42e60b80, 91 },
    { 0x4b560b81, 34 },
    { 0xd0a60b80, 61 },
    { 0, 0 },
    { 0x2b060b84, 93 },
    { 0, 0 },
    { 0xfc560b86, 52 },
    { 0x22560b87, 76 },
    { 0xa8a60b86, 95 },
    { 0x3ae60b87, 95 },
    { 0xed760b8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c060b91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca60b94, 33 },
    { 0, 0 },
    { 0x6b960b96, 90 },
    { 0x3b460b97, 90 },
    { 0, 0 },
    { 0x7b260b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7760b9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c760ba0, 33 },
    { 0x79760ba1, 61 },
    { 0x7c260ba2, 43 },
    { 0x89560ba0, 98 },
    { 0xef060ba4, 90 },
    { 0xe6c60ba5, 90 },
    { 0xca60ba2, 33 },
    { 0x67e60ba4, 61 },
    { 0x79e60ba8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d60bab, 90 },
    { 0, 0 },
    { 0x17460bad, 85 },
    { 0xa3e60bae, 98 },
    { 0x3ab60bae, 61 },
    { 0x69c60bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c60bb4, 61 },
    { 0xe3c60bb4, 81 },
    { 0, 0 },
    { 0x6db60bb7, 34 },
    { 0x36e60bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a060bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d460bc6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c360bc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63960bd1, 78 },
    { 0, 0 },
    { 0x72560bd3, 90 },
    { 0, 0 },
    { 0x7e460bd5, 61 },
    { 0x4dd60bd6, 95 },
    { 0x53060bd5, 61 },
    { 0x16760bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3760be0, 23 },
    { 0xf4260be0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63760be9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0660bee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a60bf7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc360bfe, 90 },
    { 0x24760bff, 98 },
    { 0xa0860c00, 33 },
    { 0x4bf60bff, 90 },
    { 0x6de60c01, 91 },
    { 0xc0360c03, 91 },
    { 0x5a660c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf60c09, 36 },
    { 0xb0e60c0a, 65 },
    { 0x61260c0b, 61 },
    { 0x77360c0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2760c0f, 98 },
    { 0x2fb60c10, 33 },
    { 0xdb460c11, 94 },
    { 0, 0 },
    { 0x6b260c13, 93 },
    { 0, 0 },
    { 0x10c60c15, 52 },
    { 0, 0 },
    { 0x8da60c17, 46 },
    { 0, 0 },
    { 0xd660c19, 33 },
    { 0x85660c1a, 45 },
    { 0x97960c1a, 61 },
    { 0x1a260c1b, 69 },
    { 0x4b360c1d, 87 },
    { 0xa7460c1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77760c21, 43 },
    { 0x60a60c22, 90 },
    { 0x99360c23, 90 },
    { 0x73b60c23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41360c2b, 17 },
    { 0x59b60c2c, 65 },
    { 0x16c60c2c, 76 },
    { 0xdf360c2e, 52 },
    { 0xa1360c2b, 98 },
    { 0, 0 },
    { 0x1960c31, 33 },
    { 0, 0 },
    { 0x90160c33, 33 },
    { 0xd8d60c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3260c37, 90 },
    { 0xe1d60c37, 22 },
    { 0xb1660c39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c60c3c, 43 },
    { 0, 0 },
    { 0x6cf60c3e, 76 },
    { 0, 0 },
    { 0x18360c40, 43 },
    { 0x45f60c40, 61 },
    { 0x90960c42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e60c46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e560c4b, 90 },
    { 0xc8b60c4b, 33 },
    { 0xcda60c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50160c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a60c54, 93 },
    { 0x7160c54, 33 },
    { 0x5b160c56, 33 },
    { 0x9e60c56, 61 },
    { 0x61c60c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91160c5b, 76 },
    { 0xd0260c5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc60c5f, 61 },
    { 0, 0 },
    { 0x2fc60c61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae60c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70060c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91960c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec60c71, 95 },
    { 0xc8a60c72, 66 },
    { 0, 0 },
    { 0xe8a60c74, 34 },
    { 0xf2160c75, 34 },
    { 0x54b60c76, 23 },
    { 0, 0 },
    { 0xc3160c78, 95 },
    { 0, 0 },
    { 0xe560c7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e460c7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2960c80, 46 },
    { 0x2e560c80, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb60c84, 23 },
    { 0x30e60c85, 46 },
    { 0x76f60c86, 99 },
    { 0x61360c87, 84 },
    { 0xc5960c88, 55 },
    { 0x7ec60c87, 99 },
    { 0, 0 },
    { 0x30860c8b, 98 },
    { 0xb3f60c8b, 23 },
    { 0x96760c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45060c93, 76 },
    { 0xf9660c94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5960c97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52460c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5060ca0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe260ca5, 90 },
    { 0xa6c60ca6, 61 },
    { 0x3d760ca7, 61 },
    { 0x85160ca5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57460cac, 52 },
    { 0x87d60cad, 90 },
    { 0, 0 },
    { 0x4bd60caf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff60cb2, 33 },
    { 0x8e460cb3, 69 },
    { 0xeb060cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960cbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2060cc0, 95 },
    { 0, 0 },
    { 0xe260cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38460cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4060ccb, 23 },
    { 0x41a60ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a60cd4, 56 },
    { 0, 0 },
    { 0xf1560cd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6360ce7, 97 },
    { 0x22360ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf60ceb, 34 },
    { 0, 0 },
    { 0x2a460ced, 95 },
    { 0x79d60cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c560cf2, 77 },
    { 0x44060cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9760cf8, 90 },
    { 0xde760cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0160cfd, 23 },
    { 0, 0 },
    { 0x74a60cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa960d06, 36 },
    { 0xe3e60d07, 61 },
    { 0xb8260d08, 33 },
    { 0xcc060d09, 23 },
    { 0x5a60d0a, 66 },
    { 0x38260d0b, 90 },
    { 0xb5960d0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c60d11, 98 },
    { 0, 0 },
    { 0x4360d13, 61 },
    { 0xa0360d14, 98 },
    { 0x9a060d13, 90 },
    { 0x38e60d13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99560d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51160d1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c60d23, 98 },
    { 0, 0 },
    { 0x20960d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe960d29, 84 },
    { 0, 0 },
    { 0x35760d2b, 98 },
    { 0, 0 },
    { 0x47360d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5960d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47860d38, 90 },
    { 0xb5360d39, 55 },
    { 0, 0 },
    { 0x12c60d3b, 34 },
    { 0x62560d3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32260d3f, 33 },
    { 0x2dc60d3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86660d45, 66 },
    { 0xa7660d45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20360d49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8660d50, 55 },
    { 0x69860d50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45460d57, 95 },
    { 0xd6160d57, 99 },
    { 0x10960d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e260d5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c160d62, 76 },
    { 0, 0 },
    { 0xa2f60d64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6860d67, 76 },
    { 0xae860d67, 98 },
    { 0, 0 },
    { 0xfc60d6a, 76 },
    { 0x77460d6b, 33 },
    { 0, 0 },
    { 0x9c460d6d, 66 },
    { 0, 0 },
    { 0xc6d60d6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c60d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc60d75, 61 },
    { 0, 0 },
    { 0xa5f60d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e60d7b, 91 },
    { 0x43360d7b, 61 },
    { 0xd5e60d7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea60d80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60060d84, 90 },
    { 0x11460d85, 90 },
    { 0, 0 },
    { 0xeb060d87, 61 },
    { 0x2ee60d88, 85 },
    { 0xef460d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44160d8d, 84 },
    { 0xc1760d8d, 33 },
    { 0xc1e60d8f, 55 },
    { 0x9dd60d90, 33 },
    { 0x53260d91, 33 },
    { 0x50c60d8d, 33 },
    { 0x4cb60d93, 90 },
    { 0x14760d91, 43 },
    { 0x77d60d8d, 76 },
    { 0xb0760d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e760d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74460da3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c60da8, 23 },
    { 0, 0 },
    { 0x39b60daa, 23 },
    { 0, 0 },
    { 0xfff60dac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea60daf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d560db2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4760db5, 98 },
    { 0x2fe60db6, 78 },
    { 0xaed60db6, 90 },
    { 0xcd560db8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17960dbf, 78 },
    { 0, 0 },
    { 0x3c160dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60dc4, 34 },
    { 0x91360dc5, 55 },
    { 0xd8860dc4, 33 },
    { 0x8d860dc7, 84 },
    { 0x3ea60dc4, 99 },
    { 0x6c060dc9, 46 },
    { 0x8de60dc4, 91 },
    { 0xea460dcb, 95 },
    { 0x27760dcb, 61 },
    { 0x9b560dcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb60dd4, 81 },
    { 0x3a60dd4, 61 },
    { 0x3c60dd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd60dda, 23 },
    { 0xc6360ddb, 76 },
    { 0, 0 },
    { 0x65b60ddd, 78 },
    { 0xbfd60dde, 90 },
    { 0, 0 },
    { 0x9ba60de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a260dea, 46 },
    { 0, 0 },
    { 0xf1160dec, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3760df1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef060df6, 33 },
    { 0, 0 },
    { 0x3ed60df8, 75 },
    { 0x47360df9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c60dfd, 90 },
    { 0, 0 },
    { 0xb9560dff, 55 },
    { 0, 0 },
    { 0x84260e01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60e07, 33 },
    { 0x7c960e08, 76 },
    { 0x89760e09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20560e0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d060e11, 95 },
    { 0xed60e11, 76 },
    { 0x94060e13, 61 },
    { 0x84260e14, 95 },
    { 0, 0 },
    { 0x47560e16, 33 },
    { 0, 0 },
    { 0xf3260e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53760e1e, 23 },
    { 0, 0 },
    { 0x55660e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2160e24, 33 },
    { 0xd5660e25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c60e29, 76 },
    { 0, 0 },
    { 0x25060e2b, 33 },
    { 0x4760e2c, 90 },
    { 0, 0 },
    { 0x87860e2e, 66 },
    { 0xb2c60e2f, 23 },
    { 0xa1d60e2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d60e34, 23 },
    { 0, 0 },
    { 0xf9360e36, 38 },
    { 0x4b60e37, 55 },
    { 0xbeb60e38, 53 },
    { 0x1eb60e38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x81160e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e660e42, 84 },
    { 0xe0060e42, 33 },
    { 0x83760e43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a060e47, 56 },
    { 0xaa460e48, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e60e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e60e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd460e57, 23 },
    { 0x6f660e58, 98 },
    { 0, 0 },
    { 0xc0060e5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c60e5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd760e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec760e69, 61 },
    { 0x4c760e6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b60e6e, 98 },
    { 0x7c960e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f860e76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d60e7b, 76 },
    { 0xdf60e7c, 23 },
    { 0xe4660e7c, 33 },
    { 0, 0 },
    { 0x18060e7f, 46 },
    { 0, 0 },
    { 0x8fd60e81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e560e84, 76 },
    { 0, 0 },
    { 0x40d60e86, 99 },
    { 0, 0 },
    { 0x5aa60e88, 85 },
    { 0x48a60e89, 93 },
    { 0, 0 },
    { 0xfc260e8b, 52 },
    { 0x5fd60e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac060e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4560e94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da60e98, 69 },
    { 0x86c60e98, 17 },
    { 0x4b460e9a, 98 },
    { 0x9d660e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa60e9f, 61 },
    { 0xc5360ea0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e60ea3, 23 },
    { 0x58460ea4, 69 },
    { 0x3bf60ea5, 23 },
    { 0x50160ea6, 98 },
    { 0, 0 },
    { 0x58160ea8, 61 },
    { 0x12c60ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf360eb0, 46 },
    { 0x80a60eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf60eb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3560eb8, 85 },
    { 0x7a060eb9, 81 },
    { 0x74f60eb9, 90 },
    { 0xae760ebb, 91 },
    { 0, 0 },
    { 0x9f60ebd, 98 },
    { 0x6ea60ebe, 90 },
    { 0xab860ebf, 97 },
    { 0x33260ec0, 52 },
    { 0x21b60ebd, 100 },
    { 0x76060ec2, 61 },
    { 0x76460ec3, 33 },
    { 0x75e60ec4, 77 },
    { 0x97d60ec1, 76 },
    { 0x5e960ec6, 88 },
    { 0xb7760ebf, 61 },
    { 0x9cf60ec4, 61 },
    { 0, 0 },
    { 0x91f60eca, 23 },
    { 0x75360eca, 61 },
    { 0xadf60ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f60ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a60ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d60edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c60edf, 43 },
    { 0xb8160edf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf60ee3, 95 },
    { 0x22a60ee3, 90 },
    { 0x75560ee5, 61 },
    { 0x1cc60ee3, 67 },
    { 0x97f60ee3, 33 },
    { 0x6cc60ee8, 98 },
    { 0xb2a60ee9, 90 },
    { 0x1e260eea, 91 },
    { 0xd5b60eeb, 23 },
    { 0x51d60eec, 76 },
    { 0xb8f60ee9, 23 },
    { 0, 0 },
    { 0xa60eef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29960ef3, 61 },
    { 0xdd460ef3, 95 },
    { 0, 0 },
    { 0x24060ef6, 46 },
    { 0xbfb60ef7, 91 },
    { 0x5b960ef6, 98 },
    { 0x31c60ef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b60efc, 46 },
    { 0x6eb60efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df60f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff160f0a, 95 },
    { 0xe60f0b, 98 },
    { 0, 0 },
    { 0x57360f0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd60f12, 85 },
    { 0xa1a60f13, 43 },
    { 0x69460f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e60f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc160f20, 23 },
    { 0x9c960f20, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b60f28, 90 },
    { 0x53660f29, 87 },
    { 0xe0860f29, 93 },
    { 0xf2560f2a, 98 },
    { 0, 0 },
    { 0x4e460f2d, 90 },
    { 0, 0 },
    { 0x5bd60f2f, 95 },
    { 0x45360f30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e60f38, 90 },
    { 0x98460f39, 61 },
    { 0x38960f3a, 87 },
    { 0x70560f3b, 98 },
    { 0x4360f3c, 65 },
    { 0, 0 },
    { 0x26360f3e, 84 },
    { 0x50a60f3f, 98 },
    { 0x5c060f40, 33 },
    { 0x5af60f41, 76 },
    { 0x5fe60f42, 46 },
    { 0x36960f43, 76 },
    { 0x3ff60f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb360f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87560f4c, 90 },
    { 0, 0 },
    { 0x31460f4e, 90 },
    { 0, 0 },
    { 0xe2e60f50, 55 },
    { 0xdb260f51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e60f59, 33 },
    { 0x3bc60f59, 33 },
    { 0x31860f5a, 53 },
    { 0x7d360f5c, 61 },
    { 0x21a60f5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab60f62, 95 },
    { 0xae560f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35760f68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c60f6c, 90 },
    { 0xf8c60f6d, 43 },
    { 0x9af60f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c60f71, 61 },
    { 0x7b360f72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6360f77, 23 },
    { 0xb3560f77, 95 },
    { 0x51060f79, 90 },
    { 0, 0 },
    { 0x54360f7b, 99 },
    { 0xe6d60f7c, 93 },
    { 0x72c60f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd860f80, 66 },
    { 0, 0 },
    { 0xbb760f82, 99 },
    { 0xc0c60f83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81660f89, 81 },
    { 0, 0 },
    { 0xc7760f8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc060f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89960f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b060fa2, 98 },
    { 0x25760fa2, 98 },
    { 0xb6d60fa4, 34 },
    { 0x82c60fa5, 76 },
    { 0, 0 },
    { 0xb5f60fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e060fab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84460faf, 100 },
    { 0x20b60fb0, 95 },
    { 0x8660fb0, 98 },
    { 0xc9660fb2, 95 },
    { 0x3e760faf, 61 },
    { 0x5dc60fb4, 90 },
    { 0x2fd60fb3, 34 },
    { 0xe2360fb5, 61 },
    { 0xb260fb2, 100 },
    { 0xcb360fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f60fbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb560fc1, 78 },
    { 0x5c060fc2, 98 },
    { 0x8d560fc1, 48 },
    { 0x2cb60fc4, 61 },
    { 0, 0 },
    { 0x23e60fc6, 87 },
    { 0xa6d60fc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd060fce, 68 },
    { 0, 0 },
    { 0x43a60fd0, 33 },
    { 0x29f60fd1, 61 },
    { 0xc8760fd2, 46 },
    { 0, 0 },
    { 0xac260fd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a60fdc, 85 },
    { 0x3760fdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a60fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc560fed, 33 },
    { 0x75960fed, 78 },
    { 0x39e60fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c60ff9, 55 },
    { 0, 0 },
    { 0x7a660ffb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf60fff, 90 },
    { 0, 0 },
    { 0xf3a61001, 78 },
    { 0x63f61001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a61005, 99 },
    { 0, 0 },
    { 0x2c761007, 91 },
    { 0xb5361008, 33 },
    { 0xdab61009, 90 },
    { 0x3926100a, 23 },
    { 0x6c26100b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226100e, 55 },
    { 0x1ed6100f, 33 },
    { 0, 0 },
    { 0x5bb61011, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a261017, 61 },
    { 0x9461018, 33 },
    { 0x13061019, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef56101d, 23 },
    { 0, 0 },
    { 0x25f6101f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda561022, 33 },
    { 0x39b61023, 43 },
    { 0xf9061024, 23 },
    { 0xb5b61025, 34 },
    { 0x29e61026, 75 },
    { 0x28661027, 88 },
    { 0x9ac61027, 98 },
    { 0x6bb61029, 66 },
    { 0x2e361022, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e56102f, 61 },
    { 0x4a361030, 88 },
    { 0, 0 },
    { 0x17c61032, 34 },
    { 0, 0 },
    { 0x94c61034, 98 },
    { 0x4c961035, 95 },
    { 0xbd361034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776103f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad61043, 90 },
    { 0xcca61044, 66 },
    { 0, 0 },
    { 0x55c61046, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76104e, 95 },
    { 0x3e16104f, 77 },
    { 0x7c76104e, 48 },
    { 0xecb61051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38661054, 61 },
    { 0x1eb61054, 23 },
    { 0, 0 },
    { 0x30061057, 34 },
    { 0x71a61058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e6105b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56105e, 91 },
    { 0xd526105f, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff61062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b61066, 76 },
    { 0xf6761067, 99 },
    { 0x35b61068, 34 },
    { 0, 0 },
    { 0x3a26106a, 67 },
    { 0x74b6106b, 93 },
    { 0x29a6106c, 99 },
    { 0x4d76106c, 46 },
    { 0, 0 },
    { 0x746106f, 23 },
    { 0, 0 },
    { 0x4c161071, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f61075, 95 },
    { 0x68561076, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96107c, 52 },
    { 0x3fb6107d, 95 },
    { 0xc206107e, 23 },
    { 0x7676107f, 90 },
    { 0x4b26107e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da61084, 90 },
    { 0xec861085, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c761088, 90 },
    { 0x21361088, 33 },
    { 0x71d6108a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6108e, 55 },
    { 0, 0 },
    { 0x39761090, 34 },
    { 0x3ff61091, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4561095, 76 },
    { 0x4bb61096, 95 },
    { 0xe9861096, 43 },
    { 0, 0 },
    { 0xdd361099, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa756109c, 23 },
    { 0xb7e6109d, 55 },
    { 0xf756109e, 67 },
    { 0x6686109e, 90 },
    { 0, 0 },
    { 0xecd610a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f610a5, 84 },
    { 0x6b7610a5, 34 },
    { 0x917610a7, 92 },
    { 0, 0 },
    { 0x63610a9, 61 },
    { 0x56a610aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb610b1, 100 },
    { 0, 0 },
    { 0xf6d610b3, 67 },
    { 0x9a0610b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76610b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d610ba, 93 },
    { 0xe2610bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75610be, 76 },
    { 0x49f610bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41610c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e610ca, 78 },
    { 0xc48610cb, 93 },
    { 0x7e7610cc, 33 },
    { 0x28e610cb, 33 },
    { 0, 0 },
    { 0xc0c610cf, 46 },
    { 0x804610d0, 95 },
    { 0x8e3610d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc610de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1610e1, 85 },
    { 0xa84610e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05610e7, 90 },
    { 0xf43610e8, 95 },
    { 0, 0 },
    { 0x3b6610ea, 55 },
    { 0, 0 },
    { 0x50d610ec, 52 },
    { 0xa8c610ed, 66 },
    { 0xa6b610ec, 98 },
    { 0, 0 },
    { 0xc83610f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff610f3, 90 },
    { 0xd4a610f3, 99 },
    { 0x514610f5, 90 },
    { 0, 0 },
    { 0xc610f7, 98 },
    { 0x1e610f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7610fe, 90 },
    { 0, 0 },
    { 0x56861100, 90 },
    { 0x55161101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8561106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de61109, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1756110c, 90 },
    { 0x51c6110d, 33 },
    { 0x7b76110d, 34 },
    { 0xac46110d, 55 },
    { 0x49b61110, 100 },
    { 0xda861110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f561117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed56111b, 61 },
    { 0, 0 },
    { 0x7176111d, 23 },
    { 0x3b06111e, 95 },
    { 0x4876111f, 55 },
    { 0x4dd61120, 34 },
    { 0xec361121, 69 },
    { 0x9cc61122, 61 },
    { 0xb56111e, 8 },
    { 0x84861124, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06112b, 46 },
    { 0, 0 },
    { 0x4056112d, 90 },
    { 0, 0 },
    { 0x5666112f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0861133, 93 },
    { 0, 0 },
    { 0xbe061135, 90 },
    { 0, 0 },
    { 0x47d61137, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e6113a, 61 },
    { 0x9a76113a, 33 },
    { 0xfe6113b, 98 },
    { 0x316113b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3661140, 33 },
    { 0x45761141, 55 },
    { 0xc5e61141, 33 },
    { 0, 0 },
    { 0x22061144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d461148, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec26114f, 61 },
    { 0x62d6114f, 77 },
    { 0x7d46114f, 90 },
    { 0xdb761152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d61156, 40 },
    { 0x73161156, 23 },
    { 0xdf761158, 78 },
    { 0, 0 },
    { 0x5656115a, 98 },
    { 0, 0 },
    { 0x9716115c, 61 },
    { 0x9e16115d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e61164, 90 },
    { 0xfec61165, 23 },
    { 0x19661164, 99 },
    { 0xfdc61167, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf36116a, 90 },
    { 0, 0 },
    { 0x4f96116c, 61 },
    { 0x98d6116d, 23 },
    { 0xbdb6116d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a361172, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb61175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad261179, 23 },
    { 0, 0 },
    { 0x30b6117b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd61183, 85 },
    { 0xa5861184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b61189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d6118e, 83 },
    { 0xca36118e, 95 },
    { 0, 0 },
    { 0x8e561191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20561194, 55 },
    { 0xa0261194, 33 },
    { 0x5ac61196, 98 },
    { 0x81861197, 61 },
    { 0xfdc61198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6119d, 43 },
    { 0, 0 },
    { 0xedc6119f, 99 },
    { 0xc8d611a0, 33 },
    { 0x142611a1, 85 },
    { 0, 0 },
    { 0xba2611a3, 76 },
    { 0xf6d611a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64611ac, 34 },
    { 0x837611ac, 76 },
    { 0x24d611ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x435611b1, 90 },
    { 0xfb3611b2, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c611b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a611bb, 90 },
    { 0x635611bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8611c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8611cb, 87 },
    { 0, 0 },
    { 0x8ce611cd, 33 },
    { 0xcab611ce, 95 },
    { 0, 0 },
    { 0x77e611d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b611d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6611dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09611ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1611f8, 23 },
    { 0x737611f8, 66 },
    { 0x833611fa, 90 },
    { 0x863611fb, 91 },
    { 0x3ba611fc, 90 },
    { 0x4b611fd, 55 },
    { 0, 0 },
    { 0xe7c611ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f61203, 40 },
    { 0, 0 },
    { 0xa2e61205, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab61208, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd36120d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x80061210, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61214, 23 },
    { 0, 0 },
    { 0x7e61216, 90 },
    { 0xe7c61217, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176121a, 43 },
    { 0xe786121b, 78 },
    { 0x9fe6121b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91461222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6961225, 34 },
    { 0x71f61226, 90 },
    { 0x3f161225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b86122b, 88 },
    { 0xa196122b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a61230, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a561233, 95 },
    { 0x29c61233, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116123a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26123e, 98 },
    { 0xc146123e, 54 },
    { 0xb2e61240, 46 },
    { 0x8d261241, 38 },
    { 0x7796123e, 95 },
    { 0x1c461243, 90 },
    { 0, 0 },
    { 0xf0961245, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad86124a, 23 },
    { 0xf156124a, 98 },
    { 0x5456124c, 33 },
    { 0x6d36124d, 61 },
    { 0xb96124e, 23 },
    { 0xb576124c, 33 },
    { 0xa0061250, 90 },
    { 0x1bb61250, 76 },
    { 0x39461251, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b61255, 17 },
    { 0, 0 },
    { 0x2b761257, 33 },
    { 0xc3061258, 85 },
    { 0x4a61258, 33 },
    { 0x83161258, 46 },
    { 0x3c06125a, 46 },
    { 0x1ff6125b, 76 },
    { 0xdf66125a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29561260, 76 },
    { 0x52a61261, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e361266, 66 },
    { 0xf7461266, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb586126a, 85 },
    { 0x7916126a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36126f, 61 },
    { 0x8fb6126f, 90 },
    { 0, 0 },
    { 0xcef61272, 84 },
    { 0xbd61272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e261277, 91 },
    { 0xffa61278, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646127b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb261280, 23 },
    { 0xb0961281, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e61285, 88 },
    { 0x22961285, 87 },
    { 0x6161287, 90 },
    { 0x9dc61288, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4346128b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba161290, 34 },
    { 0xe4961290, 76 },
    { 0, 0 },
    { 0xe0f61293, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d61297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa526129c, 90 },
    { 0x4bf6129c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c612a4, 88 },
    { 0xd5b612a4, 67 },
    { 0xd16612a6, 91 },
    { 0, 0 },
    { 0xdb612a8, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf612ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc612b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb612b7, 34 },
    { 0x9ed612b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251612bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18612c2, 90 },
    { 0x1b7612c3, 90 },
    { 0xdfd612c4, 95 },
    { 0, 0 },
    { 0xa94612c6, 98 },
    { 0x7c612c7, 76 },
    { 0x98c612c7, 55 },
    { 0xca3612c9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d612cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee612d2, 55 },
    { 0x364612d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db612d6, 46 },
    { 0xb74612d7, 99 },
    { 0x1b8612d8, 82 },
    { 0xefd612d6, 99 },
    { 0, 0 },
    { 0xe07612db, 95 },
    { 0x6f2612dc, 61 },
    { 0, 0 },
    { 0xaba612de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d612e8, 98 },
    { 0xddd612e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12612ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83612f4, 23 },
    { 0x4bf612f5, 98 },
    { 0xf2c612f6, 23 },
    { 0, 0 },
    { 0x88f612f8, 90 },
    { 0x9612f9, 94 },
    { 0xd80612f8, 48 },
    { 0, 0 },
    { 0x531612fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3612ff, 55 },
    { 0x82861300, 93 },
    { 0x5a4612ff, 95 },
    { 0, 0 },
    { 0xdbc61303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e61307, 81 },
    { 0x44161308, 61 },
    { 0xffa61309, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276130c, 61 },
    { 0x73a6130d, 61 },
    { 0x36d6130c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19561311, 33 },
    { 0x1dd61312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc61316, 67 },
    { 0, 0 },
    { 0xc2c61318, 100 },
    { 0xdfa61319, 76 },
    { 0, 0 },
    { 0x61a6131b, 76 },
    { 0xacb6131b, 61 },
    { 0, 0 },
    { 0x3ae6131e, 77 },
    { 0xf6f6131f, 17 },
    { 0x4f61320, 99 },
    { 0x9196131f, 95 },
    { 0x4456131f, 33 },
    { 0x3ed61321, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e61326, 61 },
    { 0x1f461327, 17 },
    { 0x9b861327, 43 },
    { 0x7e861328, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e06132f, 91 },
    { 0, 0 },
    { 0xd6761331, 98 },
    { 0xced61332, 77 },
    { 0x4b961333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d6133f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c961342, 75 },
    { 0x8f861342, 90 },
    { 0xe5761343, 60 },
    { 0xbeb61345, 33 },
    { 0, 0 },
    { 0xb9661347, 52 },
    { 0x1e361348, 90 },
    { 0xbd561349, 33 },
    { 0x9f66134a, 46 },
    { 0xf9c6134b, 36 },
    { 0x5e36134c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86134f, 23 },
    { 0xf6a61350, 90 },
    { 0, 0 },
    { 0x69c61352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd261357, 90 },
    { 0xb2661358, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466135d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b61361, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b261369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb946136c, 23 },
    { 0x39d6136d, 90 },
    { 0, 0 },
    { 0xb9b6136f, 55 },
    { 0x53461370, 52 },
    { 0x2456136f, 33 },
    { 0xcbe6136f, 40 },
    { 0x2f761372, 33 },
    { 0x7e161372, 76 },
    { 0x25061374, 33 },
    { 0, 0 },
    { 0x9c761377, 40 },
    { 0x19a61378, 87 },
    { 0xc1b61378, 33 },
    { 0x13f61379, 46 },
    { 0x3d6137b, 95 },
    { 0x4ab6137b, 61 },
    { 0xd376137d, 97 },
    { 0, 0 },
    { 0x7cc6137f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c61383, 90 },
    { 0x41861384, 55 },
    { 0x2b661384, 95 },
    { 0xc7961386, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36138b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b76138e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d561391, 80 },
    { 0xb4561392, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa61395, 33 },
    { 0xeed61396, 55 },
    { 0x11961395, 88 },
    { 0x64b61398, 33 },
    { 0x8df61398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6139e, 93 },
    { 0xeef6139f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83613a2, 61 },
    { 0x186613a2, 67 },
    { 0x14c613a4, 90 },
    { 0x458613a5, 98 },
    { 0x2e7613a5, 90 },
    { 0, 0 },
    { 0x6b1613a8, 76 },
    { 0x9f613a9, 90 },
    { 0x80e613aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e613ad, 23 },
    { 0xd4a613ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21613b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c613bb, 76 },
    { 0, 0 },
    { 0xeff613bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935613c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a613cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb613d0, 55 },
    { 0xaa0613d1, 40 },
    { 0, 0 },
    { 0xd70613d3, 81 },
    { 0x250613d4, 41 },
    { 0x27d613d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f613da, 93 },
    { 0x293613db, 23 },
    { 0x3ae613db, 98 },
    { 0x90b613db, 33 },
    { 0x174613de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87613e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2613e7, 90 },
    { 0x1f3613e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94613eb, 78 },
    { 0x4d3613ec, 66 },
    { 0x5e5613ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874613f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd613f7, 69 },
    { 0xad7613f8, 90 },
    { 0, 0 },
    { 0xc52613fa, 98 },
    { 0xabe613fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e613fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d61407, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd606140a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a61410, 61 },
    { 0xafb61411, 90 },
    { 0x2a261412, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0861419, 61 },
    { 0xcfb6141a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc6141e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19961421, 33 },
    { 0, 0 },
    { 0x9da61423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58461429, 95 },
    { 0x7e66142a, 23 },
    { 0x5bd6142a, 34 },
    { 0x3496142b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a61435, 90 },
    { 0x6c261435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc6143a, 55 },
    { 0, 0 },
    { 0x7636143c, 90 },
    { 0, 0 },
    { 0x3126143e, 85 },
    { 0xa6e6143f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18261448, 81 },
    { 0x5e061449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8086144d, 61 },
    { 0x5066144e, 55 },
    { 0x60f6144e, 82 },
    { 0x15561450, 61 },
    { 0, 0 },
    { 0xba061452, 90 },
    { 0, 0 },
    { 0x5a861454, 78 },
    { 0x5ef61455, 61 },
    { 0, 0 },
    { 0x52f61457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d6145b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5661460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d61467, 61 },
    { 0x5f561468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb361472, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac61477, 85 },
    { 0xae961478, 61 },
    { 0x9c161479, 33 },
    { 0xe9d61477, 93 },
    { 0x6a6147b, 33 },
    { 0x70b61477, 23 },
    { 0, 0 },
    { 0x7b66147e, 90 },
    { 0x7b16147f, 95 },
    { 0x5126147f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65261484, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb761488, 91 },
    { 0xb4b61489, 75 },
    { 0x2a76148a, 95 },
    { 0xbd461489, 40 },
    { 0x9e161489, 23 },
    { 0xe096148d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb61490, 76 },
    { 0xa8e61491, 33 },
    { 0x85861492, 61 },
    { 0, 0 },
    { 0xfa61494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b61498, 87 },
    { 0, 0 },
    { 0x8bb6149a, 98 },
    { 0, 0 },
    { 0x6bf6149c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36149f, 17 },
    { 0x288614a0, 65 },
    { 0x59b6149f, 85 },
    { 0x6a1614a2, 23 },
    { 0x2eb614a2, 95 },
    { 0x4d0614a4, 90 },
    { 0x372614a4, 98 },
    { 0xe6b614a2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe614aa, 91 },
    { 0, 0 },
    { 0xafd614ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61614af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x271614b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5614b5, 85 },
    { 0, 0 },
    { 0x219614b7, 90 },
    { 0xef3614b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x349614bb, 95 },
    { 0x11e614bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f614bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6614c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b614cb, 90 },
    { 0, 0 },
    { 0x4c6614cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3614d3, 43 },
    { 0x8f3614d4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99614e0, 33 },
    { 0x41614e0, 61 },
    { 0x813614e0, 53 },
    { 0xf5d614e3, 87 },
    { 0xc20614e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb614e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad614ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb614ef, 23 },
    { 0xb06614ef, 98 },
    { 0x629614f1, 61 },
    { 0x2b7614f2, 84 },
    { 0x1b5614f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1614f9, 76 },
    { 0, 0 },
    { 0xc3a614fb, 98 },
    { 0xc0e614fc, 55 },
    { 0x7f4614fd, 98 },
    { 0, 0 },
    { 0x8cc614ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d6150b, 84 },
    { 0xeaa6150c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa461510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90661518, 64 },
    { 0x61961519, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5686151d, 42 },
    { 0xd456151e, 23 },
    { 0x70f6151f, 76 },
    { 0xfce61520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b61527, 33 },
    { 0x50061528, 33 },
    { 0x51061529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336152c, 61 },
    { 0x31c6152c, 74 },
    { 0x5f96152c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0261531, 90 },
    { 0, 0 },
    { 0xa6561533, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a61536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc61539, 78 },
    { 0xadf6153a, 85 },
    { 0x44e6153b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11961540, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x24661543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7856154f, 73 },
    { 0xc4b61550, 43 },
    { 0x78c61550, 43 },
    { 0x14461551, 84 },
    { 0xe7261551, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5461566, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc56156f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde61575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a6157a, 23 },
    { 0, 0 },
    { 0xff66157c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a61585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c61588, 56 },
    { 0x4ea61589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46158c, 12 },
    { 0x1fb6158d, 23 },
    { 0x2d06158e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae61593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af61596, 76 },
    { 0xb0f61597, 95 },
    { 0x87b61597, 55 },
    { 0x5e961597, 90 },
    { 0x86c6159a, 61 },
    { 0x2bc6159b, 64 },
    { 0x70161599, 61 },
    { 0xd1f6159d, 87 },
    { 0xa9b6159e, 55 },
    { 0xabc6159f, 98 },
    { 0x135615a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba615a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881615ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411615b2, 33 },
    { 0, 0 },
    { 0xf7d615b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9615b7, 77 },
    { 0x36d615b8, 33 },
    { 0x8de615b9, 61 },
    { 0x825615ba, 95 },
    { 0x648615bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1615be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833615c6, 23 },
    { 0xa30615c6, 24 },
    { 0x827615c8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267615cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0615d0, 23 },
    { 0, 0 },
    { 0x718615d2, 33 },
    { 0xfcf615d2, 33 },
    { 0xf3e615d3, 85 },
    { 0x806615d3, 70 },
    { 0x2c9615d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c615d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54615e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5615eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13615ef, 33 },
    { 0xabc615ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x555615f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93615f6, 90 },
    { 0, 0 },
    { 0xae4615f8, 95 },
    { 0x51615f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3615fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1461600, 95 },
    { 0x4be61600, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a61605, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86161608, 95 },
    { 0, 0 },
    { 0xf576160a, 92 },
    { 0xd9b6160b, 81 },
    { 0, 0 },
    { 0x6386160d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c61610, 93 },
    { 0xeb561610, 43 },
    { 0xe2861612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed861618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346161c, 90 },
    { 0x1b6161d, 23 },
    { 0x4c26161d, 90 },
    { 0xaba6161c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58461624, 61 },
    { 0x85161624, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6162a, 85 },
    { 0, 0 },
    { 0xacb6162c, 33 },
    { 0, 0 },
    { 0x5fd6162e, 76 },
    { 0x2fb6162e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9061634, 36 },
    { 0xe3861635, 99 },
    { 0, 0 },
    { 0x79b61637, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c6163e, 95 },
    { 0x1326163f, 23 },
    { 0x92c6163f, 98 },
    { 0x6661641, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f661645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f061648, 84 },
    { 0, 0 },
    { 0xa996164a, 53 },
    { 0, 0 },
    { 0xf846164c, 17 },
    { 0x3f66164d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba61650, 92 },
    { 0, 0 },
    { 0x1bc61652, 95 },
    { 0x2f461652, 33 },
    { 0x90a61654, 55 },
    { 0x66061655, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba261659, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846165f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac61666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca6166a, 23 },
    { 0x5a6166b, 61 },
    { 0, 0 },
    { 0x6aa6166d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f61673, 95 },
    { 0x39c61674, 90 },
    { 0xe8c61673, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d61678, 90 },
    { 0xc0a61678, 87 },
    { 0x44261679, 55 },
    { 0xd366167a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd061681, 98 },
    { 0x99461682, 61 },
    { 0x6bf61683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90861689, 100 },
    { 0xf0a61689, 90 },
    { 0, 0 },
    { 0x38c6168c, 33 },
    { 0x9076168c, 62 },
    { 0xa226168e, 46 },
    { 0xc866168f, 99 },
    { 0x6bd61690, 33 },
    { 0xc1461690, 40 },
    { 0x2f661691, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d61695, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e61699, 34 },
    { 0xe8b61699, 98 },
    { 0x8ed6169b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab616a2, 61 },
    { 0x3aa616a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x143616a6, 67 },
    { 0x9ce616a7, 33 },
    { 0xfd9616a6, 43 },
    { 0, 0 },
    { 0xb34616aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7616af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33616b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be616bb, 81 },
    { 0x623616bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x828616bf, 46 },
    { 0xa0c616c0, 93 },
    { 0x6a5616c1, 95 },
    { 0x298616c2, 61 },
    { 0x3ec616bf, 76 },
    { 0, 0 },
    { 0xc0c616c5, 61 },
    { 0, 0 },
    { 0xe01616c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776616ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6616ce, 98 },
    { 0xbaf616ce, 95 },
    { 0x63d616ce, 61 },
    { 0, 0 },
    { 0xcf616d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e616d6, 93 },
    { 0x906616d7, 23 },
    { 0xba2616d8, 98 },
    { 0x82c616d9, 87 },
    { 0xd8c616d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25616de, 76 },
    { 0x564616de, 61 },
    { 0xcc6616de, 87 },
    { 0x121616e1, 81 },
    { 0, 0 },
    { 0xbbc616e3, 61 },
    { 0x87616e4, 23 },
    { 0x1c3616e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661616e9, 23 },
    { 0x38b616ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07616f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7616f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b616f8, 33 },
    { 0x97b616f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354616fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb661706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53361709, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d6170f, 84 },
    { 0xe8c6170f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61713, 61 },
    { 0xcac61713, 98 },
    { 0, 0 },
    { 0x9d761716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06171d, 90 },
    { 0, 0 },
    { 0xaa76171f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24461722, 95 },
    { 0, 0 },
    { 0x8b761724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36172a, 98 },
    { 0, 0 },
    { 0xea56172c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5036172f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61733, 24 },
    { 0, 0 },
    { 0x8361735, 90 },
    { 0, 0 },
    { 0x27e61737, 75 },
    { 0, 0 },
    { 0xc6b61739, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a61740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366174a, 46 },
    { 0, 0 },
    { 0x90e6174c, 33 },
    { 0, 0 },
    { 0xe606174e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f361751, 47 },
    { 0x661752, 95 },
    { 0x32961752, 23 },
    { 0x7b161754, 78 },
    { 0, 0 },
    { 0x8a861756, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2636175f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e761766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516176b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6176176f, 95 },
    { 0, 0 },
    { 0xabc61771, 23 },
    { 0xfb161772, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xee961775, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146177d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa61781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9161786, 76 },
    { 0xd7261786, 61 },
    { 0x7ef61788, 81 },
    { 0x76161786, 61 },
    { 0xfd261786, 93 },
    { 0x1b26178b, 33 },
    { 0xd816178c, 91 },
    { 0xdbe6178c, 55 },
    { 0x6e66178c, 34 },
    { 0xe286178d, 61 },
    { 0x8d261790, 56 },
    { 0xae66178e, 89 },
    { 0, 0 },
    { 0x84c61793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49561797, 33 },
    { 0, 0 },
    { 0xb5061799, 98 },
    { 0xca61799, 100 },
    { 0x3b36179b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107617a3, 69 },
    { 0, 0 },
    { 0x337617a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88617aa, 98 },
    { 0x47f617ab, 90 },
    { 0x5ad617ac, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a617bc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d617bf, 61 },
    { 0, 0 },
    { 0xab7617c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884617c5, 46 },
    { 0x7fb617c5, 95 },
    { 0xc65617c6, 33 },
    { 0, 0 },
    { 0x7c617c9, 34 },
    { 0, 0 },
    { 0x826617cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120617d1, 91 },
    { 0xf97617d2, 43 },
    { 0, 0 },
    { 0x321617d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x904617d7, 61 },
    { 0x96b617d8, 76 },
    { 0xe43617d9, 93 },
    { 0xbc9617d9, 61 },
    { 0, 0 },
    { 0xca8617dc, 99 },
    { 0, 0 },
    { 0xfb2617de, 78 },
    { 0x66a617df, 81 },
    { 0, 0 },
    { 0xd67617e1, 95 },
    { 0, 0 },
    { 0xbc0617e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293617e7, 95 },
    { 0xa9617e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad617ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69617f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869617f4, 33 },
    { 0x629617f5, 52 },
    { 0xa24617f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9617f9, 95 },
    { 0x14617fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a617fd, 61 },
    { 0, 0 },
    { 0xdc8617ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed61804, 33 },
    { 0, 0 },
    { 0xed361806, 95 },
    { 0, 0 },
    { 0x83661808, 23 },
    { 0xf8461809, 88 },
    { 0xfbb61809, 33 },
    { 0xfc361808, 84 },
    { 0xa266180a, 55 },
    { 0, 0 },
    { 0xa556180e, 65 },
    { 0xda56180f, 98 },
    { 0, 0 },
    { 0x3bd61811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b6181d, 95 },
    { 0xe236181e, 98 },
    { 0x39b6181f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd61822, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4161829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb6182c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e961834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586183a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad16183e, 76 },
    { 0x4dc6183f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d61842, 65 },
    { 0, 0 },
    { 0x19161844, 78 },
    { 0xbc561845, 33 },
    { 0x97061846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2606184a, 99 },
    { 0, 0 },
    { 0x8e96184c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6184f, 90 },
    { 0, 0 },
    { 0x52761851, 98 },
    { 0xb0e61852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3761856, 100 },
    { 0x36361856, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6185d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d661863, 61 },
    { 0, 0 },
    { 0x84c61865, 23 },
    { 0x4b361866, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b361869, 33 },
    { 0xed861869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d61870, 100 },
    { 0xd1861871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d61875, 23 },
    { 0x6ce61876, 33 },
    { 0x8ff61877, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6916187c, 23 },
    { 0xa6f6187d, 33 },
    { 0xde76187d, 87 },
    { 0x576187d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b761884, 95 },
    { 0, 0 },
    { 0xdd161886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb6188a, 90 },
    { 0x2a66188a, 95 },
    { 0xccf6188c, 46 },
    { 0xe2c6188c, 98 },
    { 0x7136188b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d861891, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf61894, 78 },
    { 0xaa261895, 81 },
    { 0xd7d61895, 84 },
    { 0xe2861896, 87 },
    { 0xb9761898, 95 },
    { 0x70b61899, 93 },
    { 0xa96189a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c66189d, 76 },
    { 0x1796189d, 67 },
    { 0x6236189f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe618a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2618a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946618ab, 78 },
    { 0x2dd618ac, 90 },
    { 0xd53618ac, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a618b1, 61 },
    { 0x98c618b2, 86 },
    { 0x207618b3, 31 },
    { 0xf63618b4, 33 },
    { 0x499618b1, 33 },
    { 0x465618b6, 33 },
    { 0x41e618b4, 90 },
    { 0x39d618b4, 43 },
    { 0xb67618b9, 23 },
    { 0x8b8618ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e618be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5618cb, 23 },
    { 0x832618cc, 34 },
    { 0x131618cd, 88 },
    { 0x472618ce, 98 },
    { 0x29f618cd, 61 },
    { 0, 0 },
    { 0x5b0618d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b618d4, 95 },
    { 0x3b2618d4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7618da, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa618e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x427618e4, 66 },
    { 0x36c618e4, 46 },
    { 0x577618e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6618e9, 98 },
    { 0, 0 },
    { 0x883618eb, 55 },
    { 0x8f5618ec, 57 },
    { 0xbe0618ed, 46 },
    { 0x971618ed, 66 },
    { 0x2ee618ef, 52 },
    { 0x366618ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1618fa, 98 },
    { 0, 0 },
    { 0x6f6618fc, 76 },
    { 0x3c9618fd, 90 },
    { 0xeb6618fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd61902, 76 },
    { 0x50061903, 24 },
    { 0xd2261904, 34 },
    { 0x3d261904, 99 },
    { 0x11e61906, 90 },
    { 0x17c61907, 76 },
    { 0x52461908, 90 },
    { 0xcf061907, 76 },
    { 0x6d16190a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b61911, 23 },
    { 0, 0 },
    { 0x4e961913, 43 },
    { 0, 0 },
    { 0xeb061915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c61919, 52 },
    { 0, 0 },
    { 0x90c6191b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d561921, 55 },
    { 0x69761921, 87 },
    { 0x4ad61922, 33 },
    { 0xefc61924, 98 },
    { 0xa2261925, 61 },
    { 0x4bf61926, 76 },
    { 0xf6661926, 23 },
    { 0, 0 },
    { 0x2ba61929, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f6192d, 46 },
    { 0x3946192e, 95 },
    { 0x23c6192d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e461932, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57261936, 65 },
    { 0xdd861937, 55 },
    { 0xd7161938, 90 },
    { 0xa5861938, 61 },
    { 0, 0 },
    { 0xcc66193b, 91 },
    { 0, 0 },
    { 0xa56193d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc661940, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e61945, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff6194d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e61956, 61 },
    { 0x35f61957, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016195b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda46195e, 43 },
    { 0x2466195f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561964, 43 },
    { 0x93b61965, 76 },
    { 0x4b161965, 87 },
    { 0, 0 },
    { 0xbfc61968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2336196e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6261971, 66 },
    { 0xc4761972, 40 },
    { 0x3dd61973, 66 },
    { 0x4ea61973, 33 },
    { 0xf7261973, 98 },
    { 0x65461976, 68 },
    { 0xd661977, 66 },
    { 0xab161973, 98 },
    { 0x1a861976, 23 },
    { 0, 0 },
    { 0x6076197b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c461981, 23 },
    { 0x29a61982, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66198a, 84 },
    { 0xd8b6198b, 66 },
    { 0x3fb6198a, 90 },
    { 0, 0 },
    { 0x6db6198e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d61994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78461997, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb66199e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a619a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e619aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93619ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d619b1, 55 },
    { 0xeeb619b2, 90 },
    { 0x62619b3, 66 },
    { 0xae4619b4, 61 },
    { 0x524619b3, 23 },
    { 0, 0 },
    { 0xb3c619b7, 87 },
    { 0, 0 },
    { 0x6af619b9, 76 },
    { 0xa69619ba, 38 },
    { 0x1b1619ba, 52 },
    { 0, 0 },
    { 0x753619bd, 76 },
    { 0x3a5619be, 61 },
    { 0xf4c619bf, 95 },
    { 0xa3b619bd, 100 },
    { 0, 0 },
    { 0x7bd619c2, 95 },
    { 0, 0 },
    { 0xcc2619c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602619c8, 61 },
    { 0, 0 },
    { 0x8cc619ca, 87 },
    { 0x6a9619ca, 9 },
    { 0xceb619cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f619d0, 65 },
    { 0xef8619d1, 43 },
    { 0x8d3619d0, 61 },
    { 0x930619d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e619d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348619db, 100 },
    { 0, 0 },
    { 0xc88619dd, 90 },
    { 0x9f7619de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x730619e1, 92 },
    { 0, 0 },
    { 0x9bc619e3, 33 },
    { 0, 0 },
    { 0xd97619e5, 90 },
    { 0, 0 },
    { 0xe34619e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5619ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15619ed, 98 },
    { 0x1d3619ed, 90 },
    { 0, 0 },
    { 0x376619f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35619f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e619fa, 91 },
    { 0, 0 },
    { 0xd8e619fc, 52 },
    { 0x114619fc, 98 },
    { 0x366619fc, 43 },
    { 0x2b619fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa61a03, 23 },
    { 0x33561a03, 91 },
    { 0xbd261a05, 23 },
    { 0xd7461a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a261a0a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd561a12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9061a1d, 61 },
    { 0xbf061a1d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61a25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0861a2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe61a30, 90 },
    { 0xa9961a30, 61 },
    { 0xf0061a30, 95 },
    { 0, 0 },
    { 0xdbe61a34, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb761a38, 61 },
    { 0xc061a39, 34 },
    { 0x86e61a3a, 33 },
    { 0x6f561a3b, 33 },
    { 0x3c361a39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd261a3f, 47 },
    { 0, 0 },
    { 0x14c61a41, 33 },
    { 0xe7261a42, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x99961a45, 101 },
    { 0, 0 },
    { 0xa9f61a47, 90 },
    { 0, 0 },
    { 0x24461a49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90761a53, 98 },
    { 0x22661a53, 85 },
    { 0x5c61a54, 98 },
    { 0x39261a56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22161a59, 43 },
    { 0, 0 },
    { 0x41a61a5b, 98 },
    { 0, 0 },
    { 0x3b161a5d, 46 },
    { 0, 0 },
    { 0xa7461a5f, 43 },
    { 0x75161a60, 23 },
    { 0xb3261a61, 55 },
    { 0, 0 },
    { 0xe2f61a63, 52 },
    { 0x6d661a63, 93 },
    { 0x3d161a65, 98 },
    { 0x34c61a63, 55 },
    { 0xa561a67, 23 },
    { 0, 0 },
    { 0xce61a69, 61 },
    { 0xe1f61a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6961a71, 81 },
    { 0, 0 },
    { 0xaa561a73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb61a77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d61a7d, 90 },
    { 0x74e61a7e, 76 },
    { 0x80c61a7f, 98 },
    { 0x47c61a7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f61a83, 43 },
    { 0xc4361a84, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0061a8c, 90 },
    { 0xe2961a8d, 55 },
    { 0xee261a8e, 43 },
    { 0xe961a8f, 98 },
    { 0x32061a8c, 84 },
    { 0xdd261a8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d61a97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761a9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061aa4, 76 },
    { 0xa0461aa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b61aaa, 90 },
    { 0, 0 },
    { 0xeca61aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5161ab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e61ab9, 85 },
    { 0x4561ab9, 34 },
    { 0, 0 },
    { 0xb8d61abc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb61abf, 33 },
    { 0, 0 },
    { 0xcd361ac1, 23 },
    { 0, 0 },
    { 0x4861ac3, 23 },
    { 0x2e861ac4, 93 },
    { 0x89061ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561ace, 33 },
    { 0x62e61ace, 99 },
    { 0x6cc61ace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5561ad3, 61 },
    { 0, 0 },
    { 0x10f61ad5, 23 },
    { 0xfd61ad6, 90 },
    { 0x28b61ad6, 98 },
    { 0xa6a61ad6, 95 },
    { 0x8b61ad7, 98 },
    { 0, 0 },
    { 0x8f561adb, 90 },
    { 0x50161adb, 52 },
    { 0xcee61adb, 53 },
    { 0x7f261ade, 99 },
    { 0x86e61adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c561ae4, 23 },
    { 0, 0 },
    { 0x38661ae6, 99 },
    { 0x24e61ae6, 66 },
    { 0x39461ae6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a61aef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63761af5, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98861af9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d61afd, 65 },
    { 0, 0 },
    { 0x66a61aff, 46 },
    { 0x8f561aff, 55 },
    { 0x73661b01, 61 },
    { 0x58361b01, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d661b06, 33 },
    { 0xf5c61b07, 23 },
    { 0, 0 },
    { 0x99561b09, 23 },
    { 0xdae61b0a, 76 },
    { 0, 0 },
    { 0xec461b0c, 90 },
    { 0x1161b0d, 33 },
    { 0, 0 },
    { 0x4cb61b0f, 91 },
    { 0x5a61b10, 95 },
    { 0, 0 },
    { 0x5de61b12, 34 },
    { 0x79761b13, 34 },
    { 0, 0 },
    { 0xf0e61b15, 91 },
    { 0x4eb61b16, 98 },
    { 0xa3161b15, 33 },
    { 0x8f061b18, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab61b1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab061b23, 95 },
    { 0xe9261b24, 34 },
    { 0x5bd61b23, 90 },
    { 0, 0 },
    { 0xca961b27, 90 },
    { 0x37c61b28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d61b2c, 55 },
    { 0x73661b2d, 95 },
    { 0xd4a61b2e, 99 },
    { 0xf0e61b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e61b32, 87 },
    { 0x73561b33, 98 },
    { 0, 0 },
    { 0xc7961b35, 98 },
    { 0x8d861b36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10961b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6261b3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b61b42, 91 },
    { 0, 0 },
    { 0x80761b44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83161b49, 33 },
    { 0x21d61b4a, 33 },
    { 0xac61b4b, 55 },
    { 0, 0 },
    { 0xa3e61b4d, 23 },
    { 0x59161b4e, 91 },
    { 0, 0 },
    { 0xd4361b50, 78 },
    { 0xdea61b51, 33 },
    { 0xc1261b52, 87 },
    { 0x5b861b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf61b57, 98 },
    { 0x34b61b57, 61 },
    { 0, 0 },
    { 0x66961b5a, 76 },
    { 0, 0 },
    { 0x77861b5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd61b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee61b63, 61 },
    { 0, 0 },
    { 0xbeb61b65, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e61b6b, 17 },
    { 0xc4d61b6c, 98 },
    { 0, 0 },
    { 0xea161b6e, 98 },
    { 0, 0 },
    { 0x13b61b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a261b75, 90 },
    { 0x57f61b75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd61b7c, 100 },
    { 0xd9261b7c, 55 },
    { 0xb1561b7e, 33 },
    { 0x54761b7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9461b83, 55 },
    { 0xc4a61b83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e61b89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9461b90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f61b97, 95 },
    { 0xaa061b97, 98 },
    { 0x5b961b97, 66 },
    { 0, 0 },
    { 0xdda61b9b, 99 },
    { 0xc6961b9c, 55 },
    { 0x30a61b9d, 98 },
    { 0xd4761b9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e61ba5, 76 },
    { 0x7b461ba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f61ba9, 95 },
    { 0xd0461baa, 90 },
    { 0xa6661bab, 34 },
    { 0, 0 },
    { 0xf3861bad, 34 },
    { 0x5c361bae, 61 },
    { 0x62661baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e561bb4, 75 },
    { 0, 0 },
    { 0x42d61bb6, 34 },
    { 0x8fa61bb7, 33 },
    { 0, 0 },
    { 0xbdb61bb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2761bc0, 33 },
    { 0, 0 },
    { 0x28f61bc2, 71 },
    { 0xfdc61bc3, 99 },
    { 0xf5861bc4, 98 },
    { 0xa2761bc3, 98 },
    { 0, 0 },
    { 0x59b61bc7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6061bca, 78 },
    { 0x2b561bcb, 68 },
    { 0xe8661bcc, 80 },
    { 0xe8b61bcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf161bd4, 52 },
    { 0xa261bd5, 76 },
    { 0x8cf61bd5, 23 },
    { 0x4e561bd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82261bda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc61bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c61be1, 90 },
    { 0x10261be2, 69 },
    { 0xbf461be3, 33 },
    { 0x7261be4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df61bee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c61bf1, 91 },
    { 0x7bb61bf2, 76 },
    { 0x83261bf3, 98 },
    { 0, 0 },
    { 0x17161bf5, 55 },
    { 0x5eb61bf5, 87 },
    { 0xb6861bf7, 36 },
    { 0xbd261bf5, 34 },
    { 0xd8261bf9, 98 },
    { 0x87461bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec61bfd, 90 },
    { 0xf8561bfe, 90 },
    { 0x82061bfd, 99 },
    { 0xfaf61c00, 17 },
    { 0xc4e61c01, 46 },
    { 0x8b661c01, 23 },
    { 0x21161c03, 33 },
    { 0xc2161c03, 90 },
    { 0, 0 },
    { 0xc9f61c06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xab761c09, 90 },
    { 0, 0 },
    { 0x12361c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81761c12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc561c15, 55 },
    { 0x89d61c16, 90 },
    { 0xe2c61c16, 90 },
    { 0xe0661c18, 43 },
    { 0x49b61c18, 61 },
    { 0x3061c1a, 49 },
    { 0x36b61c1b, 43 },
    { 0x8e461c1b, 23 },
    { 0xa3e61c1b, 34 },
    { 0xbc661c1e, 77 },
    { 0x55d61c1a, 99 },
    { 0x8cf61c20, 95 },
    { 0x86861c19, 90 },
    { 0x69061c22, 82 },
    { 0x21b61c22, 84 },
    { 0xe8761c1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c61c2c, 33 },
    { 0xbee61c2d, 55 },
    { 0xe561c2d, 76 },
    { 0, 0 },
    { 0xb61c30, 55 },
    { 0xd2761c31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32861c35, 43 },
    { 0x6c761c36, 91 },
    { 0xd6f61c37, 43 },
    { 0xb3761c38, 98 },
    { 0xd4161c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50261c45, 98 },
    { 0x5bf61c46, 33 },
    { 0x8c061c47, 43 },
    { 0, 0 },
    { 0x20161c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3961c4e, 43 },
    { 0xdd861c4f, 61 },
    { 0, 0 },
    { 0xf9361c51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67661c54, 90 },
    { 0x2c261c55, 76 },
    { 0x4d061c55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40061c5a, 90 },
    { 0xcbd61c5b, 93 },
    { 0xd5d61c5c, 46 },
    { 0x3c361c5c, 75 },
    { 0, 0 },
    { 0x5b61c5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63561c64, 33 },
    { 0xf1161c65, 61 },
    { 0x9361c65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea361c6d, 66 },
    { 0x33961c6d, 33 },
    { 0x84f61c6f, 76 },
    { 0x35561c70, 99 },
    { 0xa3e61c71, 55 },
    { 0xc3561c72, 33 },
    { 0x9d061c6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d161c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21261c7a, 61 },
    { 0x6961c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6861c7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34061c82, 65 },
    { 0x18861c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe61c86, 61 },
    { 0x61b61c87, 95 },
    { 0, 0 },
    { 0xc3561c89, 90 },
    { 0x48961c89, 90 },
    { 0, 0 },
    { 0xc5c61c8c, 91 },
    { 0x8561c8d, 33 },
    { 0, 0 },
    { 0x6a161c8f, 33 },
    { 0xf0861c90, 90 },
    { 0xa2a61c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de61c99, 90 },
    { 0, 0 },
    { 0x7a661c9b, 61 },
    { 0xea161c9b, 43 },
    { 0xf7461c9d, 76 },
    { 0x4ca61c9e, 90 },
    { 0xd261c9d, 95 },
    { 0xa8161c9d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d161ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d61ca8, 34 },
    { 0xd4e61ca9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81461cbb, 91 },
    { 0x1f161cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36861cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd61cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b61ccb, 55 },
    { 0x83e61ccb, 98 },
    { 0, 0 },
    { 0x9b861cce, 78 },
    { 0, 0 },
    { 0x89661cd0, 61 },
    { 0x12761cd0, 95 },
    { 0xa5d61cd0, 52 },
    { 0x99061cd1, 95 },
    { 0x14061cd4, 76 },
    { 0x8a61cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e161cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b061ce1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e61ceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9461cf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61cf4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed161cfc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee61d00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b61d05, 98 },
    { 0xda061d06, 55 },
    { 0x58661d07, 61 },
    { 0x67161d08, 66 },
    { 0x91161d08, 98 },
    { 0x53d61d0a, 77 },
    { 0xdb861d0b, 87 },
    { 0x6f661d0c, 33 },
    { 0x70e61d08, 99 },
    { 0xf2061d0e, 33 },
    { 0xd4b61d0f, 65 },
    { 0x21161d0e, 23 },
    { 0x66061d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e61d1c, 90 },
    { 0, 0 },
    { 0x7f561d1e, 90 },
    { 0xae061d1f, 23 },
    { 0, 0 },
    { 0xcb861d21, 69 },
    { 0x3961d21, 65 },
    { 0xac161d23, 43 },
    { 0, 0 },
    { 0xe4d61d25, 91 },
    { 0x48861d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb61d29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60261d32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf61d35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a61d3b, 75 },
    { 0xd3d61d3c, 61 },
    { 0x9a661d3c, 93 },
    { 0xcb861d3d, 90 },
    { 0x1a861d3e, 65 },
    { 0, 0 },
    { 0x3f161d41, 90 },
    { 0xc7661d41, 61 },
    { 0xfba61d43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee761d46, 52 },
    { 0x35661d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c61d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b61d4e, 51 },
    { 0, 0 },
    { 0x57661d50, 98 },
    { 0xb9e61d50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe061d58, 61 },
    { 0, 0 },
    { 0xe6a61d5a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b61d5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec661d61, 52 },
    { 0, 0 },
    { 0x1561d63, 87 },
    { 0xe6e61d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12961d67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93761d6c, 33 },
    { 0x33161d6d, 76 },
    { 0, 0 },
    { 0x78f61d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba61d74, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761d77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70261d7e, 90 },
    { 0, 0 },
    { 0xbe761d80, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61d84, 67 },
    { 0, 0 },
    { 0xd8961d86, 95 },
    { 0x73161d87, 33 },
    { 0xcab61d88, 95 },
    { 0xd4661d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f61d8e, 34 },
    { 0xa9761d8f, 78 },
    { 0x82d61d90, 34 },
    { 0x61f61d91, 52 },
    { 0x3a961d92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d61d95, 90 },
    { 0xcbd61d95, 87 },
    { 0x76e61d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92861d9b, 61 },
    { 0, 0 },
    { 0x64c61d9d, 98 },
    { 0xbe461d9e, 76 },
    { 0x2bc61d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef361da3, 33 },
    { 0xae861da4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4861dab, 52 },
    { 0, 0 },
    { 0x52d61dad, 87 },
    { 0, 0 },
    { 0x11a61daf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b61db2, 91 },
    { 0, 0 },
    { 0x8d261db4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x90761db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9361dbb, 87 },
    { 0x2361dbc, 33 },
    { 0xc6561dbd, 99 },
    { 0xc8061dbe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd61dc1, 88 },
    { 0x9a261dc2, 48 },
    { 0, 0 },
    { 0xc7e61dc4, 99 },
    { 0x31561dc5, 90 },
    { 0x4f561dc6, 46 },
    { 0xefe61dc7, 85 },
    { 0x5bc61dc6, 34 },
    { 0xee161dc8, 99 },
    { 0x60561dca, 33 },
    { 0xbf661dca, 90 },
    { 0, 0 },
    { 0xaa361dcd, 46 },
    { 0x3c561dcd, 65 },
    { 0xbb861dce, 61 },
    { 0x74961dd0, 33 },
    { 0x92f61dd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f61dd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32861dd7, 33 },
    { 0xf0161dd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89161ddc, 99 },
    { 0, 0 },
    { 0x5fb61dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc61de1, 100 },
    { 0xb7261de1, 33 },
    { 0xa4a61de3, 33 },
    { 0x28561de4, 90 },
    { 0xabc61de5, 33 },
    { 0x76c61de6, 67 },
    { 0x70d61de6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca061dea, 75 },
    { 0x4e761deb, 33 },
    { 0x7ab61dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a761def, 34 },
    { 0xbf361df0, 52 },
    { 0, 0 },
    { 0xb9a61df2, 100 },
    { 0x21761df3, 99 },
    { 0, 0 },
    { 0xa6d61df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c61dfb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d661e00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24661e03, 93 },
    { 0, 0 },
    { 0xe0a61e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb561e0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c61e11, 23 },
    { 0, 0 },
    { 0x4a361e13, 90 },
    { 0xfe61e14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60661e17, 61 },
    { 0, 0 },
    { 0xb4a61e19, 23 },
    { 0x62f61e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc61e1d, 76 },
    { 0xf0861e1e, 95 },
    { 0xe6b61e1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2461e25, 23 },
    { 0xcfa61e25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff661e2a, 75 },
    { 0, 0 },
    { 0xb9761e2c, 34 },
    { 0xeae61e2d, 33 },
    { 0xa3361e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79161e37, 52 },
    { 0x6bb61e37, 33 },
    { 0x15b61e39, 65 },
    { 0, 0 },
    { 0x5ce61e3b, 61 },
    { 0x48161e3c, 33 },
    { 0x82561e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f261e43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb261e47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd61e4a, 49 },
    { 0x88a61e4b, 43 },
    { 0x12261e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4961e50, 90 },
    { 0xcd861e51, 100 },
    { 0xb7e61e52, 95 },
    { 0xb7361e53, 33 },
    { 0xab761e54, 93 },
    { 0x38d61e55, 46 },
    { 0x87261e56, 95 },
    { 0xb8e61e57, 98 },
    { 0xc2561e57, 66 },
    { 0, 0 },
    { 0x5a61e5a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x25061e5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e561e64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d61e67, 76 },
    { 0x3a961e68, 33 },
    { 0xddb61e69, 33 },
    { 0, 0 },
    { 0x65661e6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d61e6e, 93 },
    { 0x2be61e6e, 56 },
    { 0x14861e6e, 90 },
    { 0x58361e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b61e7a, 76 },
    { 0x32661e7b, 99 },
    { 0xaa761e7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc361e81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d161e86, 33 },
    { 0x1bd61e87, 100 },
    { 0x77a61e87, 93 },
    { 0xa4361e89, 43 },
    { 0x19d61e8a, 33 },
    { 0x21d61e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b61e8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d61e92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce961e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a861e9f, 33 },
    { 0x46561ea0, 46 },
    { 0, 0 },
    { 0x30d61ea2, 95 },
    { 0xed961ea3, 78 },
    { 0xa4261ea3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d261eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b161eae, 34 },
    { 0, 0 },
    { 0x5d861eb0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76861eb3, 68 },
    { 0, 0 },
    { 0xde661eb5, 90 },
    { 0x2db61eb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f61ebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161ec0, 65 },
    { 0, 0 },
    { 0x7c961ec2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51661ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b61ece, 95 },
    { 0xc2661ecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe61ed2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x44061ed5, 66 },
    { 0, 0 },
    { 0x79961ed7, 23 },
    { 0, 0 },
    { 0xb7561ed9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe61ee1, 67 },
    { 0x35361ee1, 87 },
    { 0, 0 },
    { 0x78f61ee4, 98 },
    { 0x1ff61ee4, 84 },
    { 0x61661ee6, 61 },
    { 0, 0 },
    { 0x4de61ee8, 33 },
    { 0x63e61ee8, 68 },
    { 0, 0 },
    { 0xd7861eeb, 95 },
    { 0x7b761eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c561ef0, 87 },
    { 0x51361ef1, 91 },
    { 0, 0 },
    { 0xbf461ef3, 77 },
    { 0, 0 },
    { 0xbc961ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a61efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f461efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8261f02, 85 },
    { 0x28461f03, 98 },
    { 0x3e661f04, 90 },
    { 0x6d261f04, 66 },
    { 0xc5861f06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b61f09, 98 },
    { 0xbfb61f0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f461f0e, 33 },
    { 0x6e161f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d61f17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2661f1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c61f1e, 46 },
    { 0xfab61f1f, 95 },
    { 0x89f61f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9461f24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6061f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82361f2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf861f31, 90 },
    { 0x9de61f31, 33 },
    { 0xfce61f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a261f38, 33 },
    { 0x47e61f39, 96 },
    { 0x57f61f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e61f3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a61f40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0661f47, 55 },
    { 0, 0 },
    { 0x71d61f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e61f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33061f52, 87 },
    { 0xedf61f53, 84 },
    { 0, 0 },
    { 0x8e461f55, 90 },
    { 0x2bd61f56, 95 },
    { 0, 0 },
    { 0x59c61f58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1861f5d, 44 },
    { 0x45761f5e, 77 },
    { 0xb2161f5f, 100 },
    { 0x961f60, 23 },
    { 0x81561f61, 95 },
    { 0, 0 },
    { 0x71461f63, 88 },
    { 0xaa661f64, 91 },
    { 0xb1261f65, 55 },
    { 0x53261f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a61f6a, 95 },
    { 0x2e061f6b, 95 },
    { 0x37861f6a, 90 },
    { 0x9fc61f6a, 33 },
    { 0x26d61f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561f73, 88 },
    { 0x39b61f74, 100 },
    { 0x8be61f74, 69 },
    { 0x6fa61f73, 33 },
    { 0, 0 },
    { 0x4fb61f78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83361f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50261f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f261f89, 90 },
    { 0xdf661f89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf061f90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56461f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9261f97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9961f9b, 78 },
    { 0x7f361f9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad061fa0, 98 },
    { 0x2a361fa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8861fa7, 85 },
    { 0x94f61fa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc961fac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e461faf, 91 },
    { 0x3e661fb0, 43 },
    { 0xa7b61fb1, 61 },
    { 0, 0 },
    { 0x90d61fb3, 66 },
    { 0xcdd61fb3, 95 },
    { 0x19661fb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86261fb9, 91 },
    { 0x4af61fb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f361fbd, 33 },
    { 0x6761fbe, 76 },
    { 0, 0 },
    { 0xf4461fc0, 34 },
    { 0xb0d61fc1, 33 },
    { 0, 0 },
    { 0x44e61fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe961fcb, 98 },
    { 0, 0 },
    { 0x41b61fcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e61fd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad61fd5, 55 },
    { 0xbc061fd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e861fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3361fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a61fe1, 91 },
    { 0x62161fe2, 33 },
    { 0xb3161fe3, 85 },
    { 0x2fb61fe1, 40 },
    { 0xa0261fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f661fec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd661fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b761ff4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d62004, 61 },
    { 0x12d62004, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9062009, 61 },
    { 0x3bd62009, 33 },
    { 0xdc6200b, 77 },
    { 0xfd56200c, 90 },
    { 0xacf6200a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0862014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3386201a, 23 },
    { 0, 0 },
    { 0x2d36201c, 75 },
    { 0xb76201c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c62025, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f162029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce6202c, 75 },
    { 0, 0 },
    { 0x2e26202e, 88 },
    { 0x6a66202f, 61 },
    { 0x5ae62030, 90 },
    { 0xcda62030, 66 },
    { 0x1e162031, 33 },
    { 0xe4762033, 61 },
    { 0x4bd6202e, 61 },
    { 0xd9062031, 77 },
    { 0xc1b62036, 23 },
    { 0, 0 },
    { 0xb1762038, 34 },
    { 0x3f862039, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db6203e, 90 },
    { 0x5326203f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262043, 61 },
    { 0xb3662044, 99 },
    { 0, 0 },
    { 0x88062046, 69 },
    { 0, 0 },
    { 0xa4662048, 78 },
    { 0xa1462049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f6204c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb56204f, 23 },
    { 0x59062050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1962055, 90 },
    { 0x31c62055, 95 },
    { 0x57d62057, 23 },
    { 0xd0562058, 76 },
    { 0, 0 },
    { 0xda96205a, 100 },
    { 0, 0 },
    { 0x3da6205c, 46 },
    { 0, 0 },
    { 0x6006205e, 84 },
    { 0xa606205f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1662062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b62066, 34 },
    { 0xe1a62067, 94 },
    { 0x1db62068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6206b, 98 },
    { 0x59d6206c, 32 },
    { 0, 0 },
    { 0x8b06206e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10662071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49362074, 33 },
    { 0x5a962075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1862078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7706207b, 23 },
    { 0x92a6207b, 93 },
    { 0x1336207d, 77 },
    { 0x1096207c, 34 },
    { 0, 0 },
    { 0x8c362080, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f62083, 46 },
    { 0xa2162083, 100 },
    { 0, 0 },
    { 0xb2a62086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4416208c, 64 },
    { 0x4c86208d, 55 },
    { 0x486208e, 95 },
    { 0x1d66208e, 24 },
    { 0, 0 },
    { 0xd8c62091, 99 },
    { 0, 0 },
    { 0x1f662093, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b62099, 92 },
    { 0, 0 },
    { 0x8cc6209b, 91 },
    { 0, 0 },
    { 0x9d66209d, 78 },
    { 0x9966209d, 52 },
    { 0xd16209f, 98 },
    { 0xd356209f, 46 },
    { 0, 0 },
    { 0xfd2620a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94620a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7620ad, 91 },
    { 0, 0 },
    { 0x482620af, 95 },
    { 0x695620b0, 98 },
    { 0x1eb620af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f620b5, 85 },
    { 0x68f620b6, 59 },
    { 0, 0 },
    { 0xada620b8, 95 },
    { 0, 0 },
    { 0xab2620ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x828620bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902620c9, 99 },
    { 0xbfc620ca, 33 },
    { 0xb86620cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e620cf, 98 },
    { 0xa1e620cf, 23 },
    { 0x34e620d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96620d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21620da, 66 },
    { 0x363620db, 98 },
    { 0x44e620dc, 93 },
    { 0x721620da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477620e7, 75 },
    { 0x69d620e7, 46 },
    { 0x2cd620e7, 66 },
    { 0xf9e620ea, 66 },
    { 0xedf620ea, 61 },
    { 0xbf3620e7, 81 },
    { 0x7bc620ed, 61 },
    { 0, 0 },
    { 0xedd620ef, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73620f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d620fb, 33 },
    { 0, 0 },
    { 0xa8d620fd, 24 },
    { 0x36a620fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb62105, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b562108, 33 },
    { 0xd8a62109, 78 },
    { 0, 0 },
    { 0x69e6210b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d26210e, 33 },
    { 0xf646210f, 98 },
    { 0xb946210e, 33 },
    { 0, 0 },
    { 0xa9262112, 33 },
    { 0, 0 },
    { 0x20d62114, 90 },
    { 0xfa262115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa276211d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5162123, 90 },
    { 0x6db62124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5462127, 78 },
    { 0, 0 },
    { 0xe7362129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74662135, 68 },
    { 0xaf762136, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf656213b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a562141, 90 },
    { 0x6a362142, 95 },
    { 0xdc62142, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9662148, 23 },
    { 0, 0 },
    { 0x7886214a, 85 },
    { 0x1716214b, 90 },
    { 0xd4f6214c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53362150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2862153, 99 },
    { 0x76b62153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4062157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62163, 95 },
    { 0x94f62164, 98 },
    { 0x66162164, 80 },
    { 0x8fc62166, 61 },
    { 0xfde62165, 93 },
    { 0xd6162163, 95 },
    { 0xeb962169, 90 },
    { 0x91762169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58062173, 95 },
    { 0, 0 },
    { 0xeb262175, 98 },
    { 0xfd162176, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d06217c, 34 },
    { 0x86b6217d, 43 },
    { 0x6556217c, 23 },
    { 0xfe66217f, 90 },
    { 0x92f6217c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1562187, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe326218f, 78 },
    { 0xc9762190, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x64262193, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x44662196, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f56219e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x450621a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc621b0, 43 },
    { 0, 0 },
    { 0x75d621b2, 76 },
    { 0, 0 },
    { 0xc7e621b4, 90 },
    { 0xb61621b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5621b9, 33 },
    { 0, 0 },
    { 0x7f6621bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d621bf, 100 },
    { 0x6fb621c0, 84 },
    { 0, 0 },
    { 0x992621c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0621c7, 23 },
    { 0, 0 },
    { 0xe55621c9, 90 },
    { 0xe95621ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d621ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x736621d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493621d5, 55 },
    { 0x11621d5, 76 },
    { 0xf55621d7, 74 },
    { 0x5a3621d8, 95 },
    { 0x27621d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x737621dc, 34 },
    { 0x81621dd, 95 },
    { 0xf14621dd, 45 },
    { 0x373621df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd621e6, 90 },
    { 0, 0 },
    { 0x9a5621e8, 91 },
    { 0, 0 },
    { 0x8a6621ea, 77 },
    { 0, 0 },
    { 0x116621ec, 61 },
    { 0x20f621ed, 43 },
    { 0x61f621ee, 46 },
    { 0x45b621ef, 61 },
    { 0x23621ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b621fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e6220a, 33 },
    { 0x8436220b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc6220e, 33 },
    { 0x5646220f, 93 },
    { 0, 0 },
    { 0x34e62211, 77 },
    { 0, 0 },
    { 0x9de62213, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d62217, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1366221a, 95 },
    { 0x826221a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56362220, 33 },
    { 0x61862220, 90 },
    { 0xefc62222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed62228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646222d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a62232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12062237, 90 },
    { 0xa4f62237, 55 },
    { 0x3ae62238, 61 },
    { 0, 0 },
    { 0x5c56223b, 100 },
    { 0, 0 },
    { 0xed76223d, 93 },
    { 0, 0 },
    { 0x82c6223f, 61 },
    { 0x7d562240, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b62247, 90 },
    { 0x31862248, 98 },
    { 0x85d62247, 33 },
    { 0x8616224a, 99 },
    { 0xda862249, 77 },
    { 0x67f6224c, 88 },
    { 0x1736224c, 95 },
    { 0xeb96224c, 95 },
    { 0xec26224f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c62252, 61 },
    { 0x33c62252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d6225b, 77 },
    { 0x2356225c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc6225f, 100 },
    { 0x7a262260, 91 },
    { 0x9a462261, 46 },
    { 0x43762260, 95 },
    { 0, 0 },
    { 0xaa662264, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c66226b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a62275, 34 },
    { 0xd2d62275, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6227a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6227d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce62280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a362287, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56228d, 95 },
    { 0xc386228e, 33 },
    { 0x2c06228e, 55 },
    { 0x6b662290, 61 },
    { 0, 0 },
    { 0x20c62292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef62295, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d62298, 90 },
    { 0x5ce62298, 90 },
    { 0xb36229a, 33 },
    { 0x9066229b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8622a2, 61 },
    { 0xd76622a2, 90 },
    { 0xe49622a2, 98 },
    { 0, 0 },
    { 0x142622a6, 77 },
    { 0x9b2622a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63622aa, 61 },
    { 0xa7d622aa, 23 },
    { 0x8f1622ac, 71 },
    { 0x5e4622ac, 90 },
    { 0xa52622ae, 76 },
    { 0xa0f622af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6622b4, 33 },
    { 0x771622b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491622bf, 23 },
    { 0xd9c622c0, 61 },
    { 0x79622c0, 76 },
    { 0x759622c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42622c6, 90 },
    { 0x8e8622c7, 61 },
    { 0xcd5622c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97622cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b622d0, 93 },
    { 0x379622d1, 93 },
    { 0, 0 },
    { 0x440622d3, 61 },
    { 0x96d622d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d622dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1622e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52622e8, 46 },
    { 0, 0 },
    { 0x34622ea, 55 },
    { 0x405622eb, 90 },
    { 0xad622eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c622ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622f2, 34 },
    { 0x3b0622f2, 23 },
    { 0x406622f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d622f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97362300, 46 },
    { 0x8af62300, 95 },
    { 0xcf162301, 95 },
    { 0, 0 },
    { 0x60462304, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc6230b, 61 },
    { 0, 0 },
    { 0x4926230d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23662311, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f62319, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8656231c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62320, 90 },
    { 0x18b62321, 61 },
    { 0x79962322, 99 },
    { 0xa2262322, 90 },
    { 0x8b62323, 40 },
    { 0x63762325, 55 },
    { 0xd7962322, 101 },
    { 0xc4062327, 91 },
    { 0x1162328, 33 },
    { 0xf262322, 96 },
    { 0xfb762327, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e62330, 95 },
    { 0x68562330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f62337, 34 },
    { 0, 0 },
    { 0x13062339, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6233e, 98 },
    { 0x3496233e, 33 },
    { 0xa8f6233f, 33 },
    { 0, 0 },
    { 0xa3062342, 33 },
    { 0x3c962342, 98 },
    { 0, 0 },
    { 0xacb62345, 46 },
    { 0x7b562346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf856234a, 77 },
    { 0, 0 },
    { 0xba56234c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc6234f, 46 },
    { 0x1646234f, 68 },
    { 0x14062350, 78 },
    { 0x83e62350, 55 },
    { 0x8fd6234f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d62359, 87 },
    { 0, 0 },
    { 0xa946235b, 99 },
    { 0, 0 },
    { 0xc306235d, 95 },
    { 0, 0 },
    { 0x4a16235f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c262363, 90 },
    { 0xde162364, 90 },
    { 0x99162364, 46 },
    { 0xd8d62363, 95 },
    { 0, 0 },
    { 0x41d62368, 67 },
    { 0xdd562368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee66236e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b62373, 33 },
    { 0xdc962374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b6237b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e16237e, 95 },
    { 0x79a6237e, 95 },
    { 0x4546237e, 61 },
    { 0xf6062381, 93 },
    { 0, 0 },
    { 0x42362383, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6238b, 95 },
    { 0xdcf6238c, 52 },
    { 0x706238b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21562394, 90 },
    { 0x72a62395, 33 },
    { 0x3f362396, 61 },
    { 0x26362397, 61 },
    { 0x36c62398, 90 },
    { 0, 0 },
    { 0x9206239a, 91 },
    { 0xf66239b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e623a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2623a3, 90 },
    { 0x25a623a4, 33 },
    { 0xb6e623a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba623a8, 87 },
    { 0x789623a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37623ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x856623af, 43 },
    { 0x1a1623b0, 43 },
    { 0x6cb623b0, 23 },
    { 0, 0 },
    { 0x4a9623b3, 76 },
    { 0, 0 },
    { 0x6f4623b5, 90 },
    { 0xac0623b6, 90 },
    { 0xc71623b7, 90 },
    { 0, 0 },
    { 0xa2d623b9, 33 },
    { 0x3e1623ba, 90 },
    { 0x761623bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367623bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7623c2, 55 },
    { 0x1d6623c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e623c8, 66 },
    { 0x8db623c9, 33 },
    { 0x3dc623ca, 23 },
    { 0, 0 },
    { 0x5cf623cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7623cf, 93 },
    { 0x531623cf, 34 },
    { 0x4f7623d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6623d4, 90 },
    { 0x441623d5, 90 },
    { 0x711623d6, 33 },
    { 0, 0 },
    { 0x7ef623d8, 90 },
    { 0, 0 },
    { 0xe9a623da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f623dd, 55 },
    { 0x548623de, 61 },
    { 0x988623df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e623e3, 61 },
    { 0xddf623e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7623e7, 33 },
    { 0x5a2623e7, 95 },
    { 0xef2623e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e623ed, 98 },
    { 0, 0 },
    { 0x245623ef, 33 },
    { 0xce1623ef, 75 },
    { 0x246623f1, 90 },
    { 0x4be623f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6623f7, 97 },
    { 0x2ea623f8, 33 },
    { 0xfd5623f7, 90 },
    { 0, 0 },
    { 0xa3623fb, 33 },
    { 0xc13623fc, 61 },
    { 0, 0 },
    { 0x15b623fe, 33 },
    { 0x2fa623fe, 43 },
    { 0xf2662400, 77 },
    { 0, 0 },
    { 0x98562402, 33 },
    { 0xf4b62402, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2362406, 95 },
    { 0, 0 },
    { 0xa1d62408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca6240c, 66 },
    { 0x4906240d, 98 },
    { 0xd636240e, 34 },
    { 0x7af6240e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xee762412, 61 },
    { 0, 0 },
    { 0xa6762414, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a62417, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16662425, 90 },
    { 0xfb962426, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d62429, 95 },
    { 0xed762429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c6242d, 61 },
    { 0x5096242e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2862431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b62437, 52 },
    { 0x7af62437, 33 },
    { 0xdaf62439, 90 },
    { 0x8d862437, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be6243f, 96 },
    { 0x4c462440, 23 },
    { 0xd5662441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e62444, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd6244f, 88 },
    { 0, 0 },
    { 0x11c62451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f62455, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf762459, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56262462, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e62465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16246a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf462476, 36 },
    { 0x32c62476, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed6247b, 90 },
    { 0x84f6247b, 23 },
    { 0x6316247c, 98 },
    { 0, 0 },
    { 0x9976247f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62482, 76 },
    { 0xcc062483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac62486, 23 },
    { 0xf1c62487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f6248b, 76 },
    { 0xb096248c, 99 },
    { 0x7526248d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87562491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66962494, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e62497, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6249b, 87 },
    { 0, 0 },
    { 0x7776249d, 84 },
    { 0, 0 },
    { 0xfff6249f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce624a2, 78 },
    { 0, 0 },
    { 0x779624a4, 95 },
    { 0, 0 },
    { 0xdaf624a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d624a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46624ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c624af, 90 },
    { 0xe68624af, 98 },
    { 0, 0 },
    { 0x2cd624b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc624b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5624bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181624c4, 53 },
    { 0x896624c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d624c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0624cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4624d2, 23 },
    { 0, 0 },
    { 0xaaf624d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92624d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0624df, 90 },
    { 0x3c3624df, 90 },
    { 0xbb624e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a624e5, 55 },
    { 0x12d624e6, 40 },
    { 0, 0 },
    { 0x41b624e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a624ec, 33 },
    { 0xe05624ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a624f0, 90 },
    { 0xb38624f1, 61 },
    { 0xacf624f2, 95 },
    { 0xf2f624f3, 61 },
    { 0, 0 },
    { 0x801624f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6624f8, 88 },
    { 0xdad624f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb624fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe162506, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6250a, 61 },
    { 0x5f86250b, 23 },
    { 0, 0 },
    { 0x11d6250d, 99 },
    { 0, 0 },
    { 0xe7c6250f, 66 },
    { 0x2a962510, 90 },
    { 0xac62511, 93 },
    { 0x73c62510, 61 },
    { 0x50162511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa62516, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9962519, 99 },
    { 0x95662519, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac6251e, 98 },
    { 0, 0 },
    { 0xef362520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3862523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78762528, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb062532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6253a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92062540, 90 },
    { 0xa1862541, 95 },
    { 0xff962542, 43 },
    { 0xf9862540, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966254b, 90 },
    { 0, 0 },
    { 0xaac6254d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14762552, 55 },
    { 0, 0 },
    { 0xa2b62554, 84 },
    { 0x46962555, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f962558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac06255f, 55 },
    { 0, 0 },
    { 0x44062561, 17 },
    { 0x40a62562, 65 },
    { 0, 0 },
    { 0x5b162564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42962568, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c76256d, 78 },
    { 0x4456256d, 93 },
    { 0, 0 },
    { 0x32f62570, 76 },
    { 0x4ac62571, 38 },
    { 0x6a962572, 23 },
    { 0x59762572, 23 },
    { 0xa5c62573, 66 },
    { 0x8b462575, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x51862578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606257e, 90 },
    { 0x4f66257f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31962585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe636258a, 61 },
    { 0xef46258b, 90 },
    { 0x90d6258c, 23 },
    { 0x1736258d, 90 },
    { 0xf126258e, 95 },
    { 0x9246258f, 98 },
    { 0xb3d62590, 34 },
    { 0, 0 },
    { 0x7aa62592, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87562596, 95 },
    { 0xc6262596, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836259a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826259d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e625a0, 93 },
    { 0x50a625a1, 99 },
    { 0x40a625a2, 34 },
    { 0x38f625a3, 61 },
    { 0xc86625a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3625af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc625b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8625bc, 55 },
    { 0xc76625bd, 85 },
    { 0x205625be, 95 },
    { 0x869625bf, 69 },
    { 0xb4a625bc, 61 },
    { 0, 0 },
    { 0x59c625c2, 61 },
    { 0, 0 },
    { 0x897625c4, 99 },
    { 0xa0b625c5, 98 },
    { 0x8cf625c6, 90 },
    { 0xfb6625c5, 90 },
    { 0x8c5625c8, 98 },
    { 0x670625c9, 33 },
    { 0x1ed625c9, 95 },
    { 0x750625c8, 98 },
    { 0xb39625cc, 76 },
    { 0, 0 },
    { 0x5f1625ce, 33 },
    { 0x21625cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e625d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244625d8, 61 },
    { 0xdbd625d9, 95 },
    { 0xc7a625d9, 61 },
    { 0, 0 },
    { 0x6be625dc, 33 },
    { 0xcd2625dd, 33 },
    { 0xcbe625dd, 52 },
    { 0, 0 },
    { 0x90f625e0, 65 },
    { 0x3b0625e1, 82 },
    { 0x560625e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29625e5, 81 },
    { 0xa7d625e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442625ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x453625ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843625f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad625f7, 43 },
    { 0xcab625f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b625fc, 55 },
    { 0x83a625fc, 80 },
    { 0xf68625fe, 33 },
    { 0xed0625fe, 46 },
    { 0xf6e625fc, 33 },
    { 0x3fc62601, 61 },
    { 0xd2d62601, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x63962605, 81 },
    { 0x20062605, 90 },
    { 0xa4f62607, 61 },
    { 0xa9962608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe762612, 23 },
    { 0, 0 },
    { 0x3a062614, 98 },
    { 0x6562615, 88 },
    { 0xe4962615, 90 },
    { 0x29562617, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d6261a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24262625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5866262a, 66 },
    { 0x66b6262b, 68 },
    { 0xb796262b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9862630, 23 },
    { 0xdec62630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb762634, 61 },
    { 0x13e62635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e6263a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df62646, 76 },
    { 0xf6f62647, 23 },
    { 0, 0 },
    { 0x3d762649, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf576264c, 55 },
    { 0xcf06264c, 34 },
    { 0x6586264d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1962654, 98 },
    { 0, 0 },
    { 0x7b662656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36265b, 90 },
    { 0, 0 },
    { 0x7b46265d, 33 },
    { 0xbe96265e, 91 },
    { 0, 0 },
    { 0x13462660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61962665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46266b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6266e, 61 },
    { 0xde56266f, 52 },
    { 0x43f62670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4362674, 93 },
    { 0xf3562675, 61 },
    { 0x74062676, 90 },
    { 0, 0 },
    { 0xa1262678, 33 },
    { 0xc4b62679, 23 },
    { 0, 0 },
    { 0xcd76267b, 20 },
    { 0x7146267c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56268b, 33 },
    { 0, 0 },
    { 0xe696268d, 34 },
    { 0x8576268d, 90 },
    { 0x9c96268e, 78 },
    { 0xcc56268d, 66 },
    { 0x9936268d, 61 },
    { 0x60262692, 88 },
    { 0x77262693, 98 },
    { 0, 0 },
    { 0x2f762695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad362699, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1476269f, 88 },
    { 0, 0 },
    { 0xcf3626a1, 23 },
    { 0xd9b626a1, 23 },
    { 0x3aa626a1, 40 },
    { 0, 0 },
    { 0x799626a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66626aa, 95 },
    { 0, 0 },
    { 0x5f5626ac, 59 },
    { 0, 0 },
    { 0x77f626ae, 90 },
    { 0xe56626ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f626b2, 61 },
    { 0, 0 },
    { 0x51e626b4, 61 },
    { 0x8d1626b5, 76 },
    { 0xc88626b4, 98 },
    { 0, 0 },
    { 0xd0e626b8, 33 },
    { 0, 0 },
    { 0xd3c626ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7626be, 93 },
    { 0x8fe626be, 90 },
    { 0x7a7626be, 52 },
    { 0, 0 },
    { 0x10f626c2, 90 },
    { 0xe2a626c2, 98 },
    { 0x77f626c4, 90 },
    { 0xf93626c5, 90 },
    { 0xfa9626c4, 90 },
    { 0xc13626c4, 90 },
    { 0xc52626c6, 37 },
    { 0x59c626c3, 33 },
    { 0x45b626ca, 85 },
    { 0x27c626ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2626d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x738626d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6626d7, 98 },
    { 0xe7e626d8, 98 },
    { 0, 0 },
    { 0x912626da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1626df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab626ec, 96 },
    { 0, 0 },
    { 0xe89626ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4626f3, 68 },
    { 0, 0 },
    { 0x83a626f5, 76 },
    { 0, 0 },
    { 0xafc626f7, 85 },
    { 0xe27626f8, 93 },
    { 0x101626f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d626ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x61362702, 78 },
    { 0xd3162702, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e6270a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf986270f, 97 },
    { 0xcaf62710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62715, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12362718, 52 },
    { 0x5e262719, 91 },
    { 0xc3a6271a, 67 },
    { 0x40f62719, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf6271e, 85 },
    { 0xcb66271e, 98 },
    { 0, 0 },
    { 0xd4062721, 84 },
    { 0xf0f62722, 95 },
    { 0, 0 },
    { 0x7462724, 95 },
    { 0, 0 },
    { 0x54c62726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b62729, 33 },
    { 0x276272a, 95 },
    { 0, 0 },
    { 0x1826272c, 33 },
    { 0x13b6272d, 34 },
    { 0, 0 },
    { 0x4066272f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95962739, 17 },
    { 0x5162739, 34 },
    { 0x5666273b, 84 },
    { 0xb236273b, 98 },
    { 0xe686273d, 90 },
    { 0x87a6273e, 61 },
    { 0xe196273f, 98 },
    { 0x62462740, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b62743, 98 },
    { 0xa3f62744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc286274b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf6274f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0462754, 78 },
    { 0x1cf62754, 33 },
    { 0xa0062754, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb162759, 33 },
    { 0xcee6275a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc046275f, 90 },
    { 0, 0 },
    { 0x2fe62761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd766276b, 95 },
    { 0x56e6276c, 83 },
    { 0, 0 },
    { 0x9086276e, 98 },
    { 0, 0 },
    { 0x73962770, 95 },
    { 0xe1f62771, 91 },
    { 0x77662772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a62775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c62778, 99 },
    { 0xe1b62778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7966277c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d62781, 61 },
    { 0xdc262782, 58 },
    { 0xdee62782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62788, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e62790, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a62795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b62798, 88 },
    { 0xca662798, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be6279c, 33 },
    { 0, 0 },
    { 0x2fe6279e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa627a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df627a9, 90 },
    { 0, 0 },
    { 0x798627ab, 80 },
    { 0x1a5627ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8627b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f627bd, 98 },
    { 0xf81627bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67627c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a627c8, 99 },
    { 0xb9627c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x772627cc, 90 },
    { 0x77627cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d627d0, 47 },
    { 0x1d3627d0, 98 },
    { 0x6f2627d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x898627d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699627de, 78 },
    { 0x1fe627de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x695627e2, 85 },
    { 0x3bc627e2, 90 },
    { 0x87a627e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703627e8, 90 },
    { 0xa37627e9, 33 },
    { 0x510627e8, 99 },
    { 0, 0 },
    { 0xbc0627ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0627f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x249627f7, 90 },
    { 0xe6a627f8, 33 },
    { 0x799627f7, 55 },
    { 0, 0 },
    { 0x9b7627fb, 88 },
    { 0, 0 },
    { 0x825627fd, 33 },
    { 0xfc4627fd, 33 },
    { 0x40c627fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd262802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4462807, 52 },
    { 0x9ae62808, 33 },
    { 0xe7962809, 48 },
    { 0x72c6280a, 68 },
    { 0x4d062809, 23 },
    { 0x73c62808, 23 },
    { 0x5836280d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda162817, 95 },
    { 0xed262818, 46 },
    { 0x4cd62818, 46 },
    { 0, 0 },
    { 0x72c6281b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc006281f, 61 },
    { 0x7ac6281f, 33 },
    { 0x6856281f, 95 },
    { 0xc2a62822, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a362827, 90 },
    { 0x8a662828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba6282d, 48 },
    { 0xed36282d, 33 },
    { 0xf16282d, 75 },
    { 0xf4762830, 2 },
    { 0x83f62831, 34 },
    { 0x96162831, 61 },
    { 0xe96282d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd62836, 23 },
    { 0xdbc62837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046283e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad962841, 61 },
    { 0, 0 },
    { 0xc4162843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed662847, 23 },
    { 0, 0 },
    { 0x41162849, 64 },
    { 0x1ba6284a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc6284e, 34 },
    { 0xd1f6284f, 84 },
    { 0xc826284e, 90 },
    { 0x5536284f, 100 },
    { 0xdd06284f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82162858, 55 },
    { 0, 0 },
    { 0x8386285a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e62862, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff6286b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f62871, 23 },
    { 0x54462872, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd62876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4506287d, 61 },
    { 0x9656287e, 68 },
    { 0xc4d6287e, 90 },
    { 0x3d962880, 90 },
    { 0x37062881, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb62884, 23 },
    { 0x23a62885, 61 },
    { 0xd5f62886, 33 },
    { 0xb3b62887, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae6288d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67262891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81962895, 100 },
    { 0, 0 },
    { 0xa2862897, 78 },
    { 0x23362898, 90 },
    { 0x66f62897, 76 },
    { 0x35a6289a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b86289f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d628a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c628aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x931628ad, 61 },
    { 0, 0 },
    { 0x699628af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6628b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a628bf, 61 },
    { 0xf15628c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6628c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb628c7, 93 },
    { 0x148628c8, 33 },
    { 0, 0 },
    { 0x8e1628ca, 53 },
    { 0, 0 },
    { 0x76a628cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8628d3, 95 },
    { 0x239628d4, 34 },
    { 0xf1f628d5, 55 },
    { 0x727628d5, 33 },
    { 0x171628d7, 33 },
    { 0, 0 },
    { 0x98b628d9, 76 },
    { 0x8a5628d9, 61 },
    { 0, 0 },
    { 0xabb628dc, 98 },
    { 0x255628dd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88628e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x884628e6, 88 },
    { 0, 0 },
    { 0x1e8628e8, 64 },
    { 0x217628e9, 23 },
    { 0x5f9628ea, 61 },
    { 0xde6628eb, 95 },
    { 0x67c628ec, 75 },
    { 0x230628ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5628f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c628f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6628ff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xab862902, 67 },
    { 0xce062903, 67 },
    { 0xe3a62902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e62907, 91 },
    { 0, 0 },
    { 0xfbe62909, 23 },
    { 0, 0 },
    { 0x72b6290b, 77 },
    { 0, 0 },
    { 0x3596290d, 93 },
    { 0, 0 },
    { 0x49a6290f, 61 },
    { 0x6aa62910, 76 },
    { 0x5f86290f, 59 },
    { 0x3d462912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f06291a, 23 },
    { 0x1356291a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6291f, 98 },
    { 0xe826291f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc62923, 61 },
    { 0, 0 },
    { 0xac362925, 17 },
    { 0x5e262926, 61 },
    { 0x33062927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa056292c, 33 },
    { 0x6292d, 33 },
    { 0xc1b6292c, 90 },
    { 0xe576292f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68162932, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db62935, 90 },
    { 0, 0 },
    { 0xe4e62937, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9862942, 23 },
    { 0xbe562943, 76 },
    { 0x90662944, 100 },
    { 0xd0462943, 90 },
    { 0xf662946, 84 },
    { 0, 0 },
    { 0x5c462948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71762951, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54562957, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86295b, 46 },
    { 0xfbf6295c, 98 },
    { 0xbc86295c, 66 },
    { 0x46f6295d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa62964, 91 },
    { 0x61b62965, 33 },
    { 0, 0 },
    { 0xf2662967, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16296a, 91 },
    { 0x9896296b, 23 },
    { 0x7556296c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa262973, 55 },
    { 0x74e62974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42862979, 88 },
    { 0, 0 },
    { 0xc3c6297b, 61 },
    { 0x84c6297c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb62980, 67 },
    { 0x9b962981, 96 },
    { 0x64962981, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a62985, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c6298d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f062991, 34 },
    { 0x39462992, 90 },
    { 0x5e162991, 33 },
    { 0x4662994, 46 },
    { 0xc8362991, 95 },
    { 0x9da62996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3626299a, 33 },
    { 0xc2f6299a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a629a1, 90 },
    { 0xf25629a2, 90 },
    { 0, 0 },
    { 0x893629a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2629aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc629b2, 93 },
    { 0, 0 },
    { 0x11b629b4, 84 },
    { 0x238629b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x347629b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598629be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405629c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d629ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a629d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35629d7, 61 },
    { 0x98c629d8, 75 },
    { 0x4fd629d8, 77 },
    { 0, 0 },
    { 0xb09629db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5629de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4629e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397629f0, 90 },
    { 0x177629f0, 90 },
    { 0x99b629f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8629f5, 90 },
    { 0x4d1629f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b629f9, 95 },
    { 0xb5f629f9, 17 },
    { 0, 0 },
    { 0xeae629fc, 33 },
    { 0, 0 },
    { 0x147629fe, 61 },
    { 0, 0 },
    { 0x3a762a00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b62a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99562a07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c62a0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a662a12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17562a15, 33 },
    { 0x3e162a16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f62a1d, 90 },
    { 0, 0 },
    { 0x38d62a1f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df62a25, 66 },
    { 0x9ea62a25, 55 },
    { 0xd8562a26, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c62a2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8062a32, 91 },
    { 0, 0 },
    { 0x69a62a34, 92 },
    { 0, 0 },
    { 0xe5862a36, 61 },
    { 0x72562a37, 46 },
    { 0, 0 },
    { 0x10562a39, 101 },
    { 0, 0 },
    { 0xd8362a3b, 23 },
    { 0x7db62a3c, 76 },
    { 0x8be62a3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e262a41, 61 },
    { 0x42e62a41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70162a47, 90 },
    { 0, 0 },
    { 0xd5d62a49, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef62a4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea62a59, 61 },
    { 0xc4e62a5a, 61 },
    { 0xe2662a5b, 61 },
    { 0x27b62a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc62a5f, 90 },
    { 0x7262a60, 90 },
    { 0xb1862a61, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2762a66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75662a6c, 55 },
    { 0, 0 },
    { 0xedc62a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0262a71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b262a75, 67 },
    { 0x662a75, 99 },
    { 0x62462a77, 90 },
    { 0, 0 },
    { 0xf3c62a79, 76 },
    { 0, 0 },
    { 0xe4762a7b, 90 },
    { 0x44262a7b, 61 },
    { 0x58262a7d, 33 },
    { 0x96662a7e, 33 },
    { 0x85262a7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f62a82, 33 },
    { 0x4f562a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd62a86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d62a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc62a8d, 66 },
    { 0, 0 },
    { 0x7c762a8f, 52 },
    { 0x70162a8f, 95 },
    { 0x24f62a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f62a94, 98 },
    { 0, 0 },
    { 0x9fd62a96, 55 },
    { 0, 0 },
    { 0x71c62a98, 66 },
    { 0x35d62a99, 41 },
    { 0xe3c62a9a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54262aab, 78 },
    { 0x31962aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d62ab4, 61 },
    { 0, 0 },
    { 0x45662ab6, 90 },
    { 0x3ef62ab6, 23 },
    { 0x96d62ab8, 95 },
    { 0x26b62ab6, 55 },
    { 0, 0 },
    { 0x1f362abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362ac1, 61 },
    { 0xf8d62ac2, 95 },
    { 0, 0 },
    { 0x56b62ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83062ace, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90662ad3, 34 },
    { 0, 0 },
    { 0xe6262ad5, 81 },
    { 0xe5062ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d762ad9, 77 },
    { 0x50f62ad9, 40 },
    { 0xc0962ada, 23 },
    { 0x35562adc, 23 },
    { 0xea862ada, 23 },
    { 0x7762ade, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf62ae5, 95 },
    { 0, 0 },
    { 0xa5662ae7, 66 },
    { 0x5162ae7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a562af0, 55 },
    { 0, 0 },
    { 0x9f862af2, 95 },
    { 0, 0 },
    { 0x7dc62af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d762af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262afd, 90 },
    { 0x64462afe, 91 },
    { 0xbf962aff, 46 },
    { 0, 0 },
    { 0x8a562b01, 84 },
    { 0xf3d62b02, 46 },
    { 0xa2662b02, 90 },
    { 0, 0 },
    { 0xc9462b05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e262b09, 96 },
    { 0xe2962b0a, 61 },
    { 0, 0 },
    { 0x9362b0c, 95 },
    { 0, 0 },
    { 0x80f62b0e, 98 },
    { 0x52462b0f, 76 },
    { 0x6862b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5862b14, 43 },
    { 0, 0 },
    { 0xe0362b16, 23 },
    { 0xb7362b17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49662b1b, 61 },
    { 0xd0e62b1c, 90 },
    { 0xdf162b1d, 17 },
    { 0x19962b1e, 33 },
    { 0x5e862b1f, 76 },
    { 0x54b62b20, 90 },
    { 0xec162b1d, 90 },
    { 0x3dc62b22, 66 },
    { 0x41d62b22, 34 },
    { 0x94062b24, 61 },
    { 0x92862b25, 55 },
    { 0x53762b26, 90 },
    { 0x7962b27, 81 },
    { 0x53662b1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf62b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e62b35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac62b39, 76 },
    { 0xfff62b3a, 46 },
    { 0x82e62b3b, 95 },
    { 0xd162b3c, 99 },
    { 0xc3462b3d, 46 },
    { 0x25862b3d, 33 },
    { 0x8da62b3f, 83 },
    { 0x24f62b40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38162b44, 84 },
    { 0, 0 },
    { 0x30362b46, 23 },
    { 0xa1d62b47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d62b4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa662b51, 90 },
    { 0x8bb62b52, 90 },
    { 0x35f62b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd62b57, 90 },
    { 0, 0 },
    { 0xade62b59, 38 },
    { 0xdd662b5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d62b5d, 85 },
    { 0x20662b5d, 91 },
    { 0, 0 },
    { 0x58662b60, 90 },
    { 0xc9e62b61, 33 },
    { 0x46362b60, 23 },
    { 0x68b62b63, 76 },
    { 0, 0 },
    { 0x8e962b65, 81 },
    { 0x13962b66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62b6f, 17 },
    { 0x90962b6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e62b73, 91 },
    { 0xa2962b74, 33 },
    { 0x47062b75, 55 },
    { 0xd1b62b76, 33 },
    { 0x93262b75, 33 },
    { 0xa0c62b76, 33 },
    { 0x4f62b73, 61 },
    { 0x3cd62b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a62b7e, 46 },
    { 0x8de62b7f, 84 },
    { 0xc2562b80, 95 },
    { 0x8e962b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe62b8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5962b92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda762b9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4062ba0, 76 },
    { 0xad662ba1, 43 },
    { 0xa8362ba2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a62baa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb62bad, 87 },
    { 0x52e62bad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97862bb6, 91 },
    { 0x5a962bb7, 67 },
    { 0x8cb62bb8, 33 },
    { 0, 0 },
    { 0x8af62bba, 78 },
    { 0x9ce62bba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68062bbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d62bc4, 61 },
    { 0x6ce62bc5, 33 },
    { 0x25b62bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1062bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e362bcd, 33 },
    { 0xa4f62bce, 23 },
    { 0, 0 },
    { 0xd1b62bd0, 43 },
    { 0xbc862bd0, 23 },
    { 0x72762bd2, 77 },
    { 0x48662bd3, 61 },
    { 0xb2962bd2, 66 },
    { 0x8be62bd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53562be8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a62bf1, 90 },
    { 0xea262bf1, 38 },
    { 0x83062bf1, 33 },
    { 0xad162bf2, 90 },
    { 0xc9262bf5, 78 },
    { 0xbd462bf6, 98 },
    { 0x8a62bf3, 99 },
    { 0xc2862bf8, 90 },
    { 0x64662bf9, 23 },
    { 0x20662bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a62bff, 33 },
    { 0, 0 },
    { 0x53862c01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15562c04, 85 },
    { 0x93362c04, 99 },
    { 0, 0 },
    { 0x13762c07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82962c0c, 90 },
    { 0, 0 },
    { 0xb7262c0e, 43 },
    { 0, 0 },
    { 0x9f762c10, 55 },
    { 0x59662c11, 78 },
    { 0, 0 },
    { 0x72062c13, 78 },
    { 0x28262c14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45362c19, 61 },
    { 0x34a62c1a, 33 },
    { 0x5b362c1b, 98 },
    { 0xf7d62c1c, 98 },
    { 0xcb62c1b, 66 },
    { 0xe0362c1e, 98 },
    { 0x4ba62c1f, 90 },
    { 0xe7b62c20, 90 },
    { 0xeff62c21, 93 },
    { 0xdeb62c22, 76 },
    { 0x1862c23, 33 },
    { 0xd6b62c24, 98 },
    { 0, 0 },
    { 0x49762c26, 23 },
    { 0x96062c27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f62c30, 93 },
    { 0, 0 },
    { 0xbf062c32, 61 },
    { 0x7f62c33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13462c36, 66 },
    { 0, 0 },
    { 0x1f462c38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98962c43, 66 },
    { 0xe2162c44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb962c4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b62c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d862c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb462c53, 98 },
    { 0xb8262c54, 84 },
    { 0x2f462c55, 95 },
    { 0x89b62c55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a62c65, 33 },
    { 0x85c62c66, 67 },
    { 0xbb762c67, 87 },
    { 0x23062c68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a62c76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea62c7b, 90 },
    { 0, 0 },
    { 0x7bf62c7d, 98 },
    { 0xe162c7e, 66 },
    { 0, 0 },
    { 0x19462c80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15762c85, 76 },
    { 0, 0 },
    { 0x33562c87, 95 },
    { 0, 0 },
    { 0xecd62c89, 93 },
    { 0x3f462c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b62c90, 85 },
    { 0xdf062c91, 41 },
    { 0xf7a62c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e862c97, 23 },
    { 0, 0 },
    { 0xbe162c99, 98 },
    { 0xb962c99, 55 },
    { 0x9a562c9b, 90 },
    { 0x6dc62c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83462ca4, 17 },
    { 0xf2762ca5, 46 },
    { 0, 0 },
    { 0x9ea62ca7, 61 },
    { 0xa1e62ca8, 24 },
    { 0, 0 },
    { 0x4a262caa, 52 },
    { 0xd0962cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56062cb3, 23 },
    { 0x71362cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df62cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb62cbe, 90 },
    { 0x35e62cbf, 76 },
    { 0, 0 },
    { 0xb5d62cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d62cc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d62ccd, 78 },
    { 0xb6262ccd, 95 },
    { 0xa6e62ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe62cd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1962cd6, 88 },
    { 0xaaa62cd6, 76 },
    { 0x5ff62cd8, 76 },
    { 0x51762cd7, 78 },
    { 0x98c62cda, 23 },
    { 0x8dc62cdb, 98 },
    { 0, 0 },
    { 0x2c362cdd, 52 },
    { 0x95862cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a62ce2, 61 },
    { 0x8c662ce2, 55 },
    { 0, 0 },
    { 0x6c962ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a62ced, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13762cf1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a62cf5, 95 },
    { 0x31c62cf6, 61 },
    { 0x5f162cf7, 76 },
    { 0x58462cf8, 76 },
    { 0x3dd62cf9, 61 },
    { 0xba862cf9, 90 },
    { 0xb6e62cf8, 33 },
    { 0xb4962cf9, 90 },
    { 0x18f62cfc, 98 },
    { 0x6a462cfc, 85 },
    { 0x66662cff, 81 },
    { 0x86062d00, 90 },
    { 0xd8962cff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6162d08, 98 },
    { 0xdbe62d09, 98 },
    { 0, 0 },
    { 0x8da62d0b, 84 },
    { 0x67e62d0c, 90 },
    { 0xfb262d0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb62d10, 33 },
    { 0xf6162d11, 95 },
    { 0xbf562d11, 98 },
    { 0x24f62d11, 33 },
    { 0xeac62d10, 43 },
    { 0, 0 },
    { 0x96b62d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5162d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3662d20, 99 },
    { 0, 0 },
    { 0x98362d22, 98 },
    { 0x5ce62d23, 98 },
    { 0xd862d22, 99 },
    { 0, 0 },
    { 0x82c62d26, 23 },
    { 0x6bf62d27, 33 },
    { 0x81162d27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3562d2b, 61 },
    { 0x1d962d2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a462d31, 90 },
    { 0, 0 },
    { 0xd1462d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae862d36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48162d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71062d45, 78 },
    { 0x6b462d46, 55 },
    { 0xd9d62d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d62d54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe62d57, 33 },
    { 0x18962d57, 95 },
    { 0x96f62d58, 52 },
    { 0xf4162d59, 23 },
    { 0x88c62d57, 93 },
    { 0, 0 },
    { 0x80462d5d, 46 },
    { 0x3e262d5e, 98 },
    { 0xe6162d5f, 43 },
    { 0, 0 },
    { 0x4d262d61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae62d66, 13 },
    { 0, 0 },
    { 0x8d62d68, 33 },
    { 0xa8862d69, 95 },
    { 0x5d562d6a, 98 },
    { 0x62962d6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d62d70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a62d74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58562d7a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a562d7e, 95 },
    { 0x1ae62d7f, 61 },
    { 0x98162d7e, 23 },
    { 0, 0 },
    { 0xace62d82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa62d86, 67 },
    { 0x7bb62d86, 86 },
    { 0xce462d86, 76 },
    { 0, 0 },
    { 0x9ff62d8a, 67 },
    { 0x8dd62d8b, 61 },
    { 0, 0 },
    { 0x12f62d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51062d91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4962d97, 85 },
    { 0xb9b62d97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac62d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e262d9f, 88 },
    { 0, 0 },
    { 0xb1462da1, 33 },
    { 0x59d62da1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb762da6, 55 },
    { 0x9dc62da7, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c62daa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81262dad, 81 },
    { 0x82762dad, 77 },
    { 0, 0 },
    { 0xc5962db0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39462db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2262db8, 90 },
    { 0x47062db9, 98 },
    { 0x9ca62dba, 90 },
    { 0x27162db9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e62dc4, 95 },
    { 0, 0 },
    { 0x6062dc6, 90 },
    { 0x40f62dc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12562dcb, 95 },
    { 0x13862dcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8762dd1, 90 },
    { 0, 0 },
    { 0xfb062dd3, 90 },
    { 0, 0 },
    { 0xa2362dd5, 40 },
    { 0x7d062dd5, 23 },
    { 0x88b62dd7, 98 },
    { 0x13562dd8, 85 },
    { 0, 0 },
    { 0x2ef62dda, 85 },
    { 0x1f562ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f62ddf, 93 },
    { 0x762ddf, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8462de3, 78 },
    { 0x2ef62de4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda62de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13362dfe, 66 },
    { 0xa1e62dff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23262e04, 34 },
    { 0xaee62e05, 85 },
    { 0xed862e06, 23 },
    { 0x6f562e07, 84 },
    { 0xbc62e06, 34 },
    { 0x93c62e06, 23 },
    { 0xadc62e0a, 55 },
    { 0x46d62e0b, 98 },
    { 0x74962e0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67162e10, 61 },
    { 0x62062e10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef62e15, 98 },
    { 0x87c62e15, 75 },
    { 0x2962e17, 23 },
    { 0, 0 },
    { 0x4c162e19, 90 },
    { 0x3d362e19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6462e1f, 98 },
    { 0x3a862e1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97562e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce62e2c, 98 },
    { 0x68662e2d, 61 },
    { 0xcf762e2d, 34 },
    { 0, 0 },
    { 0xa3462e30, 55 },
    { 0xdd962e30, 34 },
    { 0xdd362e31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9662e36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0862e39, 61 },
    { 0xf4862e3a, 33 },
    { 0, 0 },
    { 0x58362e3c, 93 },
    { 0x21962e3c, 61 },
    { 0xca662e3e, 61 },
    { 0xc1a62e3f, 90 },
    { 0x55462e3e, 33 },
    { 0xba162e40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb862e47, 95 },
    { 0, 0 },
    { 0xafe62e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd62e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76262e50, 55 },
    { 0xf0362e51, 33 },
    { 0xf8562e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c62e55, 76 },
    { 0, 0 },
    { 0x69f62e57, 100 },
    { 0xbd462e57, 95 },
    { 0xe0362e57, 51 },
    { 0x54062e5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c262e5e, 33 },
    { 0x67962e5f, 33 },
    { 0xd7d62e5f, 91 },
    { 0xa7362e5f, 95 },
    { 0x76162e62, 61 },
    { 0xfad62e63, 90 },
    { 0x36b62e61, 90 },
    { 0x5d462e65, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25962e6b, 23 },
    { 0x3ea62e6c, 99 },
    { 0, 0 },
    { 0xcf562e6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed62e74, 55 },
    { 0x46862e75, 33 },
    { 0x24b62e76, 23 },
    { 0x4362e76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65462e7e, 61 },
    { 0, 0 },
    { 0x3c262e80, 61 },
    { 0xebb62e81, 68 },
    { 0, 0 },
    { 0xd5b62e83, 94 },
    { 0x92362e83, 61 },
    { 0xd7162e84, 100 },
    { 0x65062e84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e962e89, 98 },
    { 0x45e62e89, 61 },
    { 0, 0 },
    { 0x2f762e8c, 55 },
    { 0x23462e8c, 98 },
    { 0, 0 },
    { 0x40162e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59662e99, 90 },
    { 0, 0 },
    { 0xfcb62e9b, 95 },
    { 0, 0 },
    { 0xe2462e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0362ea0, 90 },
    { 0xacd62ea1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d062ea8, 87 },
    { 0xc4f62ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b62eac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64562eb0, 85 },
    { 0, 0 },
    { 0x7c362eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36862eb6, 80 },
    { 0, 0 },
    { 0xf4b62eb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3062ebe, 61 },
    { 0x7ec62ebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f262ec2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x61262ec5, 95 },
    { 0xb5d62ec6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a62eca, 23 },
    { 0, 0 },
    { 0xb4c62ecc, 93 },
    { 0x80e62ecd, 66 },
    { 0x1e062ece, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94362ed4, 76 },
    { 0x7fa62ed5, 61 },
    { 0, 0 },
    { 0xf9762ed7, 33 },
    { 0, 0 },
    { 0xfb862ed9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962eeb, 95 },
    { 0x37562eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b062eef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e62ef3, 61 },
    { 0x46d62ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde62ef7, 95 },
    { 0, 0 },
    { 0x3f062ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d62f00, 53 },
    { 0xcd162f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd762f07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72462f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b862f16, 55 },
    { 0x9f962f17, 34 },
    { 0xf9662f17, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7662f1e, 55 },
    { 0, 0 },
    { 0xacf62f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a62f24, 95 },
    { 0, 0 },
    { 0x4bf62f26, 23 },
    { 0xc2062f27, 98 },
    { 0x9d862f28, 77 },
    { 0xb362f29, 76 },
    { 0, 0 },
    { 0x10d62f2b, 90 },
    { 0x7d362f2b, 99 },
    { 0x10462f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8662f32, 23 },
    { 0x9f862f33, 34 },
    { 0xa5262f34, 33 },
    { 0, 0 },
    { 0x36b62f36, 98 },
    { 0x6e962f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e662f41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1362f44, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab62f47, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd062f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62f52, 65 },
    { 0xaaf62f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f362f5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c662f5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63762f62, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c62f65, 23 },
    { 0xf6662f66, 34 },
    { 0x4a662f66, 17 },
    { 0x29f62f68, 23 },
    { 0x60062f69, 87 },
    { 0x59b62f6a, 61 },
    { 0, 0 },
    { 0x15a62f6c, 90 },
    { 0, 0 },
    { 0xe9d62f6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a62f71, 33 },
    { 0, 0 },
    { 0x35662f73, 99 },
    { 0, 0 },
    { 0xa6162f75, 98 },
    { 0x55262f75, 67 },
    { 0x99a62f77, 61 },
    { 0xe2d62f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0462f7b, 90 },
    { 0xf6062f7b, 95 },
    { 0x3ed62f7d, 33 },
    { 0x77962f7e, 81 },
    { 0x3fd62f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78862f85, 98 },
    { 0x94862f86, 99 },
    { 0x9c162f87, 33 },
    { 0xb1562f87, 36 },
    { 0xdf862f89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b662f8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6762f8f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc62f92, 34 },
    { 0x79e62f92, 61 },
    { 0x25162f94, 76 },
    { 0xb1262f92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0762f98, 76 },
    { 0xf6062f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59062f9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a862fa3, 90 },
    { 0x21762fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c062faa, 33 },
    { 0xbd962faa, 95 },
    { 0x5362fac, 33 },
    { 0xfb562fad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b62fb3, 23 },
    { 0x18062fb4, 90 },
    { 0x34062fb5, 55 },
    { 0xcf62fb6, 95 },
    { 0xf1962fb7, 77 },
    { 0x32b62fb8, 100 },
    { 0x68262fb9, 33 },
    { 0x29462fb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2462fbe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b662fc4, 98 },
    { 0, 0 },
    { 0x71862fc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c62fcb, 93 },
    { 0, 0 },
    { 0xc6262fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c62fd3, 90 },
    { 0xb3862fd4, 78 },
    { 0xd5c62fd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a762fde, 100 },
    { 0xfe862fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c62fe3, 61 },
    { 0, 0 },
    { 0x51162fe5, 91 },
    { 0, 0 },
    { 0xd7662fe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2762fea, 33 },
    { 0xccc62fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc162fee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc62ff2, 23 },
    { 0x12562ff3, 90 },
    { 0, 0 },
    { 0x54e62ff5, 99 },
    { 0, 0 },
    { 0x88762ff7, 95 },
    { 0, 0 },
    { 0xe5762ff9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5962ffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58763004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e6300f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e163015, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5056301a, 33 },
    { 0x5986301b, 91 },
    { 0, 0 },
    { 0x89a6301d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35963023, 55 },
    { 0x4263024, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18563028, 61 },
    { 0x3ed63029, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726302f, 40 },
    { 0x7df63030, 33 },
    { 0xcf263031, 76 },
    { 0x34a63031, 77 },
    { 0xd6963031, 55 },
    { 0xda763034, 95 },
    { 0x89c63032, 90 },
    { 0xfae63036, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1816303e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc63044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e6304b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3263053, 23 },
    { 0x2a063054, 99 },
    { 0xf9763055, 55 },
    { 0x38363056, 32 },
    { 0x5a363057, 77 },
    { 0x89963058, 91 },
    { 0x14d63058, 98 },
    { 0xfa363059, 43 },
    { 0xce06305b, 96 },
    { 0x426305c, 67 },
    { 0xf136305d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8163065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a263068, 34 },
    { 0xf2c63069, 67 },
    { 0, 0 },
    { 0xbc96306b, 99 },
    { 0x1716306b, 61 },
    { 0x1a76306c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c763072, 90 },
    { 0x87a63073, 23 },
    { 0, 0 },
    { 0x95a63075, 33 },
    { 0, 0 },
    { 0x8ab63077, 23 },
    { 0x6bf63078, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6426307b, 55 },
    { 0x6216307c, 98 },
    { 0xe5e6307c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf63084, 98 },
    { 0, 0 },
    { 0xbe963086, 86 },
    { 0x26363087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e96308b, 76 },
    { 0, 0 },
    { 0x9336308d, 61 },
    { 0, 0 },
    { 0xf966308f, 40 },
    { 0x55063090, 23 },
    { 0, 0 },
    { 0x663092, 77 },
    { 0, 0 },
    { 0x2b763094, 90 },
    { 0x78363095, 61 },
    { 0x6a863094, 76 },
    { 0, 0 },
    { 0x69c63098, 34 },
    { 0xe3a63099, 11 },
    { 0x8156309a, 33 },
    { 0xaf26309b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b630a0, 76 },
    { 0xee2630a1, 95 },
    { 0, 0 },
    { 0xf04630a3, 98 },
    { 0x41e630a4, 90 },
    { 0x1d7630a4, 46 },
    { 0xd6f630a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x278630a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7630ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64630b1, 90 },
    { 0, 0 },
    { 0x7ea630b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844630c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b630c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38630c6, 95 },
    { 0, 0 },
    { 0x94f630c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4630cc, 93 },
    { 0x8c0630cc, 43 },
    { 0x33d630ce, 34 },
    { 0xe78630cf, 95 },
    { 0xd2e630d0, 43 },
    { 0xade630ce, 33 },
    { 0x874630d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3630d8, 95 },
    { 0x36b630d8, 98 },
    { 0xa98630da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b630de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475630e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6630e6, 33 },
    { 0x97d630e7, 33 },
    { 0x71b630e7, 55 },
    { 0x2fb630e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d630ee, 77 },
    { 0x2a3630ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783630f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718630fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f263100, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c63103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca63106, 95 },
    { 0, 0 },
    { 0x57763108, 46 },
    { 0, 0 },
    { 0x3596310a, 98 },
    { 0, 0 },
    { 0x21f6310c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e63112, 95 },
    { 0x4c163113, 78 },
    { 0x9ac63114, 76 },
    { 0, 0 },
    { 0xa5f63116, 33 },
    { 0, 0 },
    { 0xc9963118, 23 },
    { 0xf8063119, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7926311f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c563123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a63127, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2606312d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63131, 76 },
    { 0x6ce63131, 91 },
    { 0xd6e63133, 99 },
    { 0x43463132, 84 },
    { 0x74963135, 56 },
    { 0x8e263136, 33 },
    { 0x4f763136, 47 },
    { 0x9ee63133, 33 },
    { 0, 0 },
    { 0x3ff6313a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc006313d, 98 },
    { 0x90a6313e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e263141, 33 },
    { 0, 0 },
    { 0x9c663143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62363147, 91 },
    { 0xe8663148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4406314c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a163151, 95 },
    { 0x8fd63151, 40 },
    { 0x28463153, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x90963156, 41 },
    { 0x66963156, 61 },
    { 0x1063158, 76 },
    { 0x96d63159, 76 },
    { 0xe5a63159, 91 },
    { 0, 0 },
    { 0xfab6315c, 88 },
    { 0, 0 },
    { 0xd9a6315e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a63163, 95 },
    { 0x1ca63164, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd63167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f76316b, 55 },
    { 0xaec6316c, 90 },
    { 0x166316d, 67 },
    { 0x93b6316e, 78 },
    { 0x68a6316f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0463172, 66 },
    { 0x4c763172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb6317a, 33 },
    { 0x4c96317b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066317f, 98 },
    { 0xfae63180, 90 },
    { 0xd2763181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f63188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74863193, 76 },
    { 0, 0 },
    { 0xb1663195, 90 },
    { 0xa3963196, 74 },
    { 0xe1963197, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe76319e, 90 },
    { 0x2b26319f, 34 },
    { 0xf10631a0, 33 },
    { 0x4c3631a1, 100 },
    { 0x749631a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b631a5, 90 },
    { 0x196631a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5631ac, 66 },
    { 0x6d631ac, 55 },
    { 0xd2d631ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f631b1, 38 },
    { 0x77c631b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc631bd, 33 },
    { 0xb4d631bd, 90 },
    { 0x13c631bf, 61 },
    { 0x237631bf, 23 },
    { 0, 0 },
    { 0x90631c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8631c9, 33 },
    { 0x4e8631c9, 53 },
    { 0, 0 },
    { 0x5d6631cc, 90 },
    { 0xd31631cd, 85 },
    { 0x12f631ce, 67 },
    { 0, 0 },
    { 0xcd9631d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba631d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf631da, 61 },
    { 0xbc6631db, 61 },
    { 0xdc6631dc, 33 },
    { 0, 0 },
    { 0xd76631de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c631e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x415631e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4631eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55631ee, 91 },
    { 0xa4c631ee, 33 },
    { 0x754631ef, 98 },
    { 0, 0 },
    { 0x989631f2, 90 },
    { 0x121631f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c631f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a631fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f631ff, 55 },
    { 0x4d263200, 61 },
    { 0xb5e631ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34063204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf63208, 61 },
    { 0x7df63208, 61 },
    { 0x66e63209, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf616320d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d63212, 42 },
    { 0, 0 },
    { 0x5263214, 55 },
    { 0x8963215, 98 },
    { 0x70563214, 33 },
    { 0, 0 },
    { 0xf4363218, 33 },
    { 0xbae63219, 90 },
    { 0, 0 },
    { 0xbf46321b, 76 },
    { 0, 0 },
    { 0x1f06321d, 33 },
    { 0xf896321d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38063222, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a63226, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c63232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f63236, 90 },
    { 0, 0 },
    { 0x4dd63238, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f6323b, 33 },
    { 0xb946323b, 100 },
    { 0x6346323d, 33 },
    { 0, 0 },
    { 0x6f46323f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13163243, 55 },
    { 0xcb963244, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c63247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd116324b, 95 },
    { 0x5426324c, 43 },
    { 0xb2c6324d, 17 },
    { 0xa416324e, 90 },
    { 0x8c86324e, 43 },
    { 0xcbb63250, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a63253, 46 },
    { 0xf7963254, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6325f, 78 },
    { 0xe9c63260, 43 },
    { 0x65063261, 81 },
    { 0x6d663262, 90 },
    { 0xe5563263, 33 },
    { 0x9ca63262, 98 },
    { 0, 0 },
    { 0xa4f63266, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6326d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xab263270, 61 },
    { 0x31863271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa63276, 46 },
    { 0, 0 },
    { 0xd8e63278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056327e, 85 },
    { 0, 0 },
    { 0xba63280, 33 },
    { 0x5c163281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda363285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c56328a, 23 },
    { 0xc2d6328b, 87 },
    { 0x42e6328a, 61 },
    { 0x40d6328d, 98 },
    { 0, 0 },
    { 0xfba6328f, 59 },
    { 0x58663290, 78 },
    { 0x98063291, 23 },
    { 0, 0 },
    { 0x57263293, 85 },
    { 0, 0 },
    { 0x6263295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec6329b, 55 },
    { 0xf936329c, 90 },
    { 0x8186329c, 80 },
    { 0, 0 },
    { 0x8496329f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c632a5, 77 },
    { 0x650632a6, 90 },
    { 0x137632a7, 77 },
    { 0xe50632a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758632ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a632b0, 95 },
    { 0, 0 },
    { 0x606632b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94632ba, 76 },
    { 0xcca632bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67632c1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910632c6, 90 },
    { 0x1e8632c6, 66 },
    { 0xffa632c8, 98 },
    { 0xb57632c6, 33 },
    { 0x4f9632ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11632cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6632d0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1632d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5632db, 100 },
    { 0xc58632dc, 95 },
    { 0xece632dc, 23 },
    { 0xedd632de, 100 },
    { 0x428632de, 33 },
    { 0x273632e0, 76 },
    { 0x9b9632e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x924632e4, 90 },
    { 0x64a632e4, 23 },
    { 0x619632e5, 87 },
    { 0xae632e5, 81 },
    { 0, 0 },
    { 0x9cc632e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29632ec, 23 },
    { 0x548632ec, 90 },
    { 0xe04632ee, 23 },
    { 0, 0 },
    { 0xbfd632f0, 99 },
    { 0xcfb632f1, 61 },
    { 0xcef632f2, 33 },
    { 0xeb632f3, 95 },
    { 0xef632f4, 23 },
    { 0x25632f5, 90 },
    { 0xf05632f5, 33 },
    { 0xe71632f7, 98 },
    { 0xada632f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a632fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be63301, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x37163304, 61 },
    { 0, 0 },
    { 0x76963306, 95 },
    { 0x4c963307, 55 },
    { 0xdfb63307, 90 },
    { 0xc5663309, 61 },
    { 0x2b363309, 33 },
    { 0, 0 },
    { 0x336330c, 61 },
    { 0x6496330c, 34 },
    { 0x9f46330e, 88 },
    { 0, 0 },
    { 0x4db63310, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b63313, 85 },
    { 0xd5063314, 33 },
    { 0x47b63315, 77 },
    { 0xe3e63316, 76 },
    { 0xa1a63317, 61 },
    { 0xf1263318, 91 },
    { 0x7e963319, 90 },
    { 0x5863314, 76 },
    { 0xe6463313, 77 },
    { 0xdd16331c, 34 },
    { 0, 0 },
    { 0xc386331e, 76 },
    { 0, 0 },
    { 0x17963320, 43 },
    { 0xbe063320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2563329, 99 },
    { 0x8c36332a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7386332f, 33 },
    { 0x9406332f, 95 },
    { 0x3b563331, 52 },
    { 0x47663331, 61 },
    { 0xe2163333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a963336, 61 },
    { 0xdf763337, 98 },
    { 0, 0 },
    { 0x68263339, 91 },
    { 0xaed63339, 66 },
    { 0x5eb6333b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e86333e, 61 },
    { 0x96333f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c63342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d63346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e16334b, 90 },
    { 0xa666334c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4636334f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263352, 81 },
    { 0, 0 },
    { 0xb4663354, 52 },
    { 0, 0 },
    { 0xcde63356, 100 },
    { 0, 0 },
    { 0x2b463358, 61 },
    { 0xd1763359, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a6335e, 65 },
    { 0, 0 },
    { 0x9cb63360, 23 },
    { 0, 0 },
    { 0x7a363362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396336a, 93 },
    { 0x14a6336b, 76 },
    { 0x2bd6336c, 23 },
    { 0x2de6336d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63375, 91 },
    { 0, 0 },
    { 0x70a63377, 33 },
    { 0x1963377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x256337b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586337f, 66 },
    { 0, 0 },
    { 0xe5163381, 91 },
    { 0x9c963382, 90 },
    { 0x60263381, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3563387, 61 },
    { 0x3f463387, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6338b, 93 },
    { 0, 0 },
    { 0x5856338d, 46 },
    { 0xfb56338d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d63395, 55 },
    { 0x67863396, 61 },
    { 0x2bf63395, 66 },
    { 0xb6963398, 95 },
    { 0, 0 },
    { 0x2556339a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646339e, 85 },
    { 0x3d46339f, 61 },
    { 0, 0 },
    { 0xc71633a1, 90 },
    { 0x328633a2, 76 },
    { 0xdba633a3, 33 },
    { 0x97f633a4, 76 },
    { 0, 0 },
    { 0xf10633a6, 55 },
    { 0xce5633a6, 95 },
    { 0xab3633a6, 66 },
    { 0x413633a9, 23 },
    { 0x12d633a9, 80 },
    { 0xc33633a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616633af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0633b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9633b5, 76 },
    { 0xacc633b5, 23 },
    { 0x280633b7, 23 },
    { 0x48a633b8, 90 },
    { 0x10e633b9, 61 },
    { 0xab633b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa633bd, 95 },
    { 0x227633be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409633c8, 98 },
    { 0, 0 },
    { 0xb68633ca, 90 },
    { 0x34e633cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce633d1, 90 },
    { 0, 0 },
    { 0xe00633d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7633d6, 67 },
    { 0, 0 },
    { 0x91a633d8, 76 },
    { 0, 0 },
    { 0xcdd633da, 61 },
    { 0x7ea633db, 76 },
    { 0xb66633dc, 90 },
    { 0xf7633db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c633e0, 90 },
    { 0, 0 },
    { 0x118633e2, 76 },
    { 0, 0 },
    { 0x3d9633e4, 61 },
    { 0xb54633e5, 47 },
    { 0xd40633e6, 17 },
    { 0x827633e7, 61 },
    { 0x177633e8, 95 },
    { 0xbcb633e6, 90 },
    { 0xee7633ea, 68 },
    { 0x3a633e6, 95 },
    { 0, 0 },
    { 0x23b633ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5633f2, 34 },
    { 0xce7633f2, 95 },
    { 0x222633f2, 52 },
    { 0xbb7633f4, 61 },
    { 0xa5b633f6, 23 },
    { 0x81633f7, 99 },
    { 0x6a633f7, 33 },
    { 0xf32633f9, 100 },
    { 0xa14633f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0633fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91163408, 76 },
    { 0, 0 },
    { 0x3956340a, 90 },
    { 0, 0 },
    { 0xdb6340c, 33 },
    { 0x82e6340d, 91 },
    { 0x5c96340e, 33 },
    { 0xcff6340d, 23 },
    { 0xc526340e, 46 },
    { 0, 0 },
    { 0x96563412, 90 },
    { 0x15363413, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec63416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59763420, 55 },
    { 0xe1863420, 90 },
    { 0x47363422, 98 },
    { 0xca663422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e63426, 55 },
    { 0xd4d63427, 95 },
    { 0, 0 },
    { 0xaf663429, 98 },
    { 0x5aa63429, 23 },
    { 0, 0 },
    { 0x5166342c, 77 },
    { 0xac6342c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e63430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49463437, 95 },
    { 0x9aa63438, 23 },
    { 0x4ae63439, 76 },
    { 0xee46343a, 40 },
    { 0x30863439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4263442, 90 },
    { 0x3f363442, 95 },
    { 0x9863444, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb216344a, 85 },
    { 0, 0 },
    { 0x4c76344c, 66 },
    { 0xc7e6344d, 98 },
    { 0, 0 },
    { 0xb9e6344f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a63453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca63456, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa296345a, 53 },
    { 0, 0 },
    { 0x8d56345c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b63460, 91 },
    { 0xae263461, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8063465, 98 },
    { 0x38163466, 95 },
    { 0x42763465, 90 },
    { 0xdee63467, 90 },
    { 0xa3363469, 23 },
    { 0xe976346a, 61 },
    { 0x8c663467, 99 },
    { 0, 0 },
    { 0xea06346d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f63470, 33 },
    { 0xb7963471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63476, 25 },
    { 0xc1e63477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e6347a, 33 },
    { 0, 0 },
    { 0xad66347c, 23 },
    { 0x8c6347c, 90 },
    { 0x6d76347e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84763483, 33 },
    { 0x5e763484, 98 },
    { 0, 0 },
    { 0x50963486, 33 },
    { 0x7ac63486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2656348a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6348d, 76 },
    { 0x7766348d, 43 },
    { 0x8246348f, 55 },
    { 0x61a63490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96163496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6349c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de6349f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860634a8, 34 },
    { 0xe0c634a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536634ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4634b0, 65 },
    { 0x5ae634b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df634b7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317634bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f634c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d634c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c634c7, 66 },
    { 0, 0 },
    { 0x210634c9, 33 },
    { 0x6da634ca, 76 },
    { 0xd35634ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3634ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f634d2, 95 },
    { 0xb75634d3, 90 },
    { 0x1c634d2, 61 },
    { 0x356634d3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe634d8, 91 },
    { 0x226634d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x582634dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9634e1, 98 },
    { 0, 0 },
    { 0x76c634e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9634e8, 90 },
    { 0xc93634e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c634ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85634f2, 95 },
    { 0x877634f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3634f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0763501, 61 },
    { 0xa3763502, 55 },
    { 0x5f863502, 55 },
    { 0x763502, 43 },
    { 0x26163505, 43 },
    { 0x20b63506, 76 },
    { 0xdd963506, 40 },
    { 0x16863508, 93 },
    { 0x7d963508, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6350e, 55 },
    { 0, 0 },
    { 0x52b63510, 85 },
    { 0, 0 },
    { 0x55863512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53663515, 90 },
    { 0xc7263515, 98 },
    { 0x36163515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd236351f, 90 },
    { 0, 0 },
    { 0x74663521, 34 },
    { 0, 0 },
    { 0x2c163523, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91263527, 34 },
    { 0xc9563528, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b6352b, 93 },
    { 0x1a96352b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc63531, 90 },
    { 0x3ea63531, 33 },
    { 0x8b163533, 53 },
    { 0xbe763533, 61 },
    { 0xeb63531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91663538, 33 },
    { 0x80363539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a06353f, 90 },
    { 0x6a26353f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28563543, 34 },
    { 0, 0 },
    { 0xb6d63545, 46 },
    { 0xbd263546, 43 },
    { 0, 0 },
    { 0x38c63548, 34 },
    { 0xa3163549, 61 },
    { 0x4e26354a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6354d, 98 },
    { 0xfda6354e, 33 },
    { 0xbaf6354e, 76 },
    { 0, 0 },
    { 0xdfe63551, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca6355a, 33 },
    { 0xd6355b, 90 },
    { 0x8266355c, 87 },
    { 0xab26355a, 87 },
    { 0x3906355c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced63565, 76 },
    { 0, 0 },
    { 0xb8d63567, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3296356b, 23 },
    { 0x946356c, 23 },
    { 0xcd76356b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad363570, 55 },
    { 0xa2663571, 91 },
    { 0xcc163570, 98 },
    { 0x1363573, 43 },
    { 0, 0 },
    { 0x4c763575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea46357b, 33 },
    { 0xf0b6357c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28363580, 84 },
    { 0x63663580, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf63589, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46358c, 52 },
    { 0xbf16358c, 100 },
    { 0, 0 },
    { 0xb066358f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf63592, 90 },
    { 0, 0 },
    { 0xeba63594, 90 },
    { 0xd763595, 22 },
    { 0, 0 },
    { 0x23363597, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed635a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03635a4, 98 },
    { 0xfcc635a4, 55 },
    { 0x1f6635a6, 90 },
    { 0, 0 },
    { 0x945635a8, 90 },
    { 0, 0 },
    { 0x193635aa, 78 },
    { 0, 0 },
    { 0x1d5635ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81635af, 91 },
    { 0xb01635af, 95 },
    { 0xef0635b0, 90 },
    { 0x636635b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6635b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43635b9, 95 },
    { 0, 0 },
    { 0x939635bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388635bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee635c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x552635c6, 66 },
    { 0x219635c7, 65 },
    { 0xfd6635c8, 61 },
    { 0x323635c9, 76 },
    { 0, 0 },
    { 0x726635cb, 34 },
    { 0x284635cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae635d1, 33 },
    { 0xdf3635d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a635d5, 34 },
    { 0, 0 },
    { 0x18c635d7, 90 },
    { 0xf1635d8, 77 },
    { 0xab635d8, 93 },
    { 0x4f7635d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7635de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858635e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92635e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02635ec, 76 },
    { 0, 0 },
    { 0xe8a635ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x718635f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5635fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90363600, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e63605, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d6360b, 90 },
    { 0x4ab6360c, 33 },
    { 0x4296360b, 90 },
    { 0xbd86360e, 61 },
    { 0x18d6360b, 99 },
    { 0, 0 },
    { 0xbe563611, 56 },
    { 0xbf963612, 65 },
    { 0x68163611, 46 },
    { 0xb2163614, 100 },
    { 0x32963613, 43 },
    { 0, 0 },
    { 0xf1b63617, 33 },
    { 0x2f163617, 90 },
    { 0x18163619, 95 },
    { 0xab6361a, 34 },
    { 0x1786361a, 90 },
    { 0x90f63619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76361f, 46 },
    { 0, 0 },
    { 0x40363621, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64263625, 43 },
    { 0, 0 },
    { 0x66463627, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456362e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20663631, 33 },
    { 0, 0 },
    { 0xaf563633, 90 },
    { 0, 0 },
    { 0xc3363635, 33 },
    { 0, 0 },
    { 0xe3a63637, 90 },
    { 0x34763637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd676363c, 66 },
    { 0, 0 },
    { 0xf866363e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e463643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6364d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c63652, 98 },
    { 0x2f463652, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d663656, 17 },
    { 0x2963657, 90 },
    { 0, 0 },
    { 0x24d63659, 55 },
    { 0xf006365a, 98 },
    { 0x26363659, 34 },
    { 0x3086365c, 90 },
    { 0xaed6365a, 93 },
    { 0x2206365e, 93 },
    { 0, 0 },
    { 0xb5463660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f763664, 66 },
    { 0x63163665, 33 },
    { 0, 0 },
    { 0xe7963667, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d6366d, 76 },
    { 0x27f6366d, 95 },
    { 0, 0 },
    { 0x63d63670, 95 },
    { 0, 0 },
    { 0x3a263672, 99 },
    { 0, 0 },
    { 0x88f63674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1926367b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33063680, 98 },
    { 0xe1363680, 43 },
    { 0, 0 },
    { 0x8a863683, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd706368e, 23 },
    { 0x8c16368f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6763694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14663698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb06369c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6346369f, 86 },
    { 0, 0 },
    { 0xb00636a1, 34 },
    { 0x5fa636a2, 90 },
    { 0x69b636a3, 95 },
    { 0x526636a4, 76 },
    { 0xc56636a4, 34 },
    { 0x3b0636a5, 75 },
    { 0, 0 },
    { 0x7fb636a8, 68 },
    { 0, 0 },
    { 0x1af636aa, 76 },
    { 0xf1c636ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1636b3, 90 },
    { 0x874636b3, 87 },
    { 0xa5e636b4, 90 },
    { 0xf09636b6, 90 },
    { 0x8dc636b4, 90 },
    { 0xe5d636b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2636be, 90 },
    { 0x9d5636bf, 55 },
    { 0xb00636c0, 90 },
    { 0x887636c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d636d2, 46 },
    { 0, 0 },
    { 0x8e1636d4, 61 },
    { 0xa85636d5, 95 },
    { 0xfb7636d6, 98 },
    { 0xdb9636d7, 34 },
    { 0, 0 },
    { 0xcf4636d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731636de, 33 },
    { 0xf94636df, 55 },
    { 0x338636df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f636e4, 23 },
    { 0x90f636e4, 61 },
    { 0x3d4636e6, 93 },
    { 0x7d7636e6, 99 },
    { 0xabe636e7, 98 },
    { 0x7fa636e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f636ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c636f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07636f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3636fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3636fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b63706, 46 },
    { 0, 0 },
    { 0xb7063708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd6370d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c63714, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e6371b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26371f, 33 },
    { 0, 0 },
    { 0x4f363721, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd63726, 33 },
    { 0, 0 },
    { 0x8ff63728, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3616372c, 95 },
    { 0xe166372c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7363730, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa63734, 46 },
    { 0x45263735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf336373a, 85 },
    { 0, 0 },
    { 0x9d26373c, 23 },
    { 0x7ff6373d, 61 },
    { 0x8af6373c, 33 },
    { 0x7d16373f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17163743, 98 },
    { 0, 0 },
    { 0xe7263745, 33 },
    { 0, 0 },
    { 0x6cb63747, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876374a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb6374f, 78 },
    { 0, 0 },
    { 0x1ed63751, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f063754, 78 },
    { 0x5b163755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226375e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23963761, 90 },
    { 0xd6b63762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b63766, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226376b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b063770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b563773, 91 },
    { 0, 0 },
    { 0x32c63775, 23 },
    { 0, 0 },
    { 0x55563777, 61 },
    { 0x4d663778, 33 },
    { 0, 0 },
    { 0x8996377a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x576377d, 93 },
    { 0x6d76377e, 34 },
    { 0x2f66377e, 76 },
    { 0, 0 },
    { 0x8f763781, 91 },
    { 0x2de63781, 55 },
    { 0x46163781, 99 },
    { 0xd7d63784, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82863787, 68 },
    { 0x4af63788, 99 },
    { 0, 0 },
    { 0xa7e6378a, 90 },
    { 0x9bb6378b, 33 },
    { 0x6f66378a, 93 },
    { 0x3996378b, 61 },
    { 0xbf56378e, 90 },
    { 0xc0a6378b, 95 },
    { 0x53b6378a, 90 },
    { 0xabc63791, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d63796, 84 },
    { 0xfee63796, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46379d, 90 },
    { 0, 0 },
    { 0xa386379f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689637a7, 23 },
    { 0, 0 },
    { 0xc1637a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b637b0, 90 },
    { 0x432637b1, 94 },
    { 0xc0d637b0, 61 },
    { 0x524637b3, 95 },
    { 0x155637b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5637b9, 61 },
    { 0xc19637ba, 98 },
    { 0, 0 },
    { 0x9ac637bc, 75 },
    { 0x4dd637bd, 23 },
    { 0x27a637bd, 81 },
    { 0x41c637bf, 98 },
    { 0, 0 },
    { 0x598637c1, 40 },
    { 0xc14637c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62637c5, 17 },
    { 0x273637c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04637ca, 90 },
    { 0xc9637cb, 23 },
    { 0x1dd637cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1637d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x107637d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51637dc, 61 },
    { 0, 0 },
    { 0x837637de, 83 },
    { 0xe6c637de, 98 },
    { 0xcfa637df, 88 },
    { 0x924637e1, 90 },
    { 0x90e637e0, 98 },
    { 0xe74637e3, 34 },
    { 0x709637e4, 46 },
    { 0x4d5637e4, 90 },
    { 0x19f637e6, 33 },
    { 0x57f637e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x722637ea, 61 },
    { 0, 0 },
    { 0xc04637ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x418637ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc637f6, 77 },
    { 0x3d4637f6, 33 },
    { 0, 0 },
    { 0x562637f9, 33 },
    { 0, 0 },
    { 0x788637fb, 91 },
    { 0xb6b637fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b063804, 55 },
    { 0x88463805, 98 },
    { 0x56c63806, 69 },
    { 0x7c263805, 95 },
    { 0x83563807, 90 },
    { 0, 0 },
    { 0x8796380a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82563812, 85 },
    { 0x9cd63813, 61 },
    { 0xfcc63812, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13363819, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b6381e, 61 },
    { 0x7346381f, 84 },
    { 0, 0 },
    { 0xf6a63821, 23 },
    { 0xee663822, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86382b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d6383c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb263841, 95 },
    { 0, 0 },
    { 0x56563843, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d663849, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad63850, 100 },
    { 0xf6b63851, 66 },
    { 0x76463851, 90 },
    { 0x53e63852, 33 },
    { 0, 0 },
    { 0x26063855, 33 },
    { 0x22a63856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93863859, 68 },
    { 0xa056385a, 61 },
    { 0, 0 },
    { 0x8476385c, 33 },
    { 0, 0 },
    { 0x7396385e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed63861, 43 },
    { 0, 0 },
    { 0x56163863, 100 },
    { 0xd3163864, 33 },
    { 0x88763865, 80 },
    { 0x64763866, 95 },
    { 0xb5963864, 43 },
    { 0xd6263868, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996386d, 66 },
    { 0xe226386d, 98 },
    { 0x2f86386f, 55 },
    { 0xd326386f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49863875, 61 },
    { 0xb8063875, 90 },
    { 0x83963876, 23 },
    { 0x23f63878, 90 },
    { 0x8f63879, 93 },
    { 0xb5b63878, 23 },
    { 0, 0 },
    { 0xaaa6387c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead63880, 90 },
    { 0, 0 },
    { 0x9ec63882, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b63885, 61 },
    { 0, 0 },
    { 0x31d63887, 93 },
    { 0xe163887, 67 },
    { 0xa0963887, 87 },
    { 0x32d6388a, 90 },
    { 0x3d963889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be6388e, 69 },
    { 0xc9a6388e, 90 },
    { 0x6fc63890, 85 },
    { 0xa4a63890, 90 },
    { 0x8d063892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d563895, 95 },
    { 0xe2863896, 93 },
    { 0x3ab63897, 95 },
    { 0xd0b63898, 98 },
    { 0, 0 },
    { 0xd266389a, 100 },
    { 0x8406389b, 81 },
    { 0x5c56389a, 95 },
    { 0x7c6389c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6638a1, 98 },
    { 0, 0 },
    { 0x4fe638a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x672638a6, 34 },
    { 0x232638a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5638ab, 76 },
    { 0xcc8638ab, 33 },
    { 0x7aa638ad, 61 },
    { 0x4be638ae, 90 },
    { 0x7cd638af, 98 },
    { 0, 0 },
    { 0x4da638b1, 53 },
    { 0, 0 },
    { 0x4b1638b3, 78 },
    { 0, 0 },
    { 0xa93638b5, 77 },
    { 0x882638b5, 98 },
    { 0x184638b7, 76 },
    { 0x45b638b6, 85 },
    { 0x6b8638b9, 95 },
    { 0x4e4638ba, 38 },
    { 0, 0 },
    { 0x627638bc, 43 },
    { 0xbb1638bd, 95 },
    { 0, 0 },
    { 0xed1638bf, 95 },
    { 0, 0 },
    { 0x785638c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3638c8, 33 },
    { 0x33f638c9, 91 },
    { 0, 0 },
    { 0x151638cb, 23 },
    { 0x7f4638cb, 95 },
    { 0x81a638cb, 95 },
    { 0, 0 },
    { 0x8b638cf, 43 },
    { 0x572638cf, 61 },
    { 0x8ad638d0, 43 },
    { 0, 0 },
    { 0x179638d3, 93 },
    { 0x2b2638d3, 34 },
    { 0x86f638d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54638db, 34 },
    { 0xec9638dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4638df, 53 },
    { 0xf4c638df, 56 },
    { 0x2cd638e0, 76 },
    { 0, 0 },
    { 0xaeb638e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0638ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x815638ef, 43 },
    { 0x693638f0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11638f3, 99 },
    { 0xd2d638f4, 95 },
    { 0xc9b638f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x405638f8, 46 },
    { 0xe1c638f9, 95 },
    { 0x96b638fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7638fd, 43 },
    { 0x944638fe, 43 },
    { 0x5b1638fe, 90 },
    { 0x20963900, 98 },
    { 0, 0 },
    { 0x95963902, 61 },
    { 0x3563903, 23 },
    { 0x25063903, 33 },
    { 0x28b63905, 33 },
    { 0x82b63906, 95 },
    { 0xc3463906, 23 },
    { 0xfac63906, 33 },
    { 0, 0 },
    { 0x5df6390a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9663913, 61 },
    { 0xdbf63913, 90 },
    { 0xc4d63913, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4686391a, 98 },
    { 0xf376391b, 52 },
    { 0x75d6391b, 98 },
    { 0x4b26391d, 99 },
    { 0, 0 },
    { 0x206391f, 43 },
    { 0xe7e63920, 33 },
    { 0, 0 },
    { 0xa6763922, 34 },
    { 0x25c63923, 61 },
    { 0xa8363923, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c63927, 76 },
    { 0, 0 },
    { 0xceb63929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b6392e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46263933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf63939, 98 },
    { 0x85d6393a, 90 },
    { 0xaf76393a, 98 },
    { 0xad36393b, 77 },
    { 0, 0 },
    { 0x29a6393e, 95 },
    { 0, 0 },
    { 0xa1c63940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e63943, 76 },
    { 0xf7763944, 90 },
    { 0x2ff63945, 93 },
    { 0xbd563946, 64 },
    { 0x9e963943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e46394c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6394f, 95 },
    { 0xfc76394f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc063956, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3163959, 33 },
    { 0, 0 },
    { 0xe656395b, 67 },
    { 0xadb6395c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7163960, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e463964, 61 },
    { 0x33d63965, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f63970, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90463974, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b63979, 98 },
    { 0, 0 },
    { 0xd746397b, 90 },
    { 0x1486397c, 91 },
    { 0, 0 },
    { 0xc3e6397e, 23 },
    { 0x1f16397f, 98 },
    { 0x5b663980, 98 },
    { 0x57063981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd763991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e763995, 91 },
    { 0xfc63996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75363999, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c6399e, 90 },
    { 0x9946399f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19639a4, 59 },
    { 0, 0 },
    { 0x9c0639a6, 23 },
    { 0x83d639a6, 90 },
    { 0x46639a8, 23 },
    { 0xf50639a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8639b1, 94 },
    { 0, 0 },
    { 0xb20639b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9639b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20639bd, 34 },
    { 0xb7a639bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452639c2, 100 },
    { 0, 0 },
    { 0xca2639c4, 95 },
    { 0, 0 },
    { 0xac0639c6, 85 },
    { 0x33639c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02639cf, 55 },
    { 0x616639cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252639d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32639d8, 33 },
    { 0, 0 },
    { 0xd4d639da, 66 },
    { 0x55b639da, 98 },
    { 0x668639dc, 33 },
    { 0xf99639dc, 66 },
    { 0x46b639de, 98 },
    { 0x2e639dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8639e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0639e7, 90 },
    { 0x14b639e8, 55 },
    { 0, 0 },
    { 0x94c639ea, 46 },
    { 0x694639eb, 75 },
    { 0x1be639eb, 61 },
    { 0x349639ec, 95 },
    { 0xfbb639ed, 64 },
    { 0, 0 },
    { 0x1a4639f0, 90 },
    { 0x4c5639f1, 98 },
    { 0, 0 },
    { 0xfe8639f3, 43 },
    { 0x467639f4, 90 },
    { 0x7d4639f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb639fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f63a02, 95 },
    { 0xc7263a03, 23 },
    { 0x2d163a04, 61 },
    { 0x99863a05, 76 },
    { 0x94863a04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb63a09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd63a0c, 23 },
    { 0, 0 },
    { 0x16163a0e, 2 },
    { 0xef063a0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5263a15, 93 },
    { 0xdd263a16, 99 },
    { 0x72363a17, 90 },
    { 0, 0 },
    { 0x26863a19, 95 },
    { 0xe1763a1a, 95 },
    { 0xb6863a1a, 23 },
    { 0, 0 },
    { 0x53963a1d, 68 },
    { 0x69363a1d, 88 },
    { 0xb6e63a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d63a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd963a28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95963a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac63a38, 80 },
    { 0, 0 },
    { 0x27463a3a, 34 },
    { 0, 0 },
    { 0xb6d63a3c, 83 },
    { 0x29363a3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x59363a40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15163a43, 91 },
    { 0, 0 },
    { 0xa4463a45, 50 },
    { 0x3d863a46, 93 },
    { 0xa1663a46, 24 },
    { 0x5e363a46, 33 },
    { 0x16d63a46, 53 },
    { 0xc1263a4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38063a4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89363a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d63a56, 52 },
    { 0, 0 },
    { 0x79f63a58, 23 },
    { 0, 0 },
    { 0x31763a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb63a5f, 93 },
    { 0, 0 },
    { 0xf0963a61, 34 },
    { 0xc8663a62, 65 },
    { 0x51863a62, 33 },
    { 0x91663a64, 78 },
    { 0x14b63a62, 95 },
    { 0xff563a66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc63a6a, 95 },
    { 0, 0 },
    { 0x70463a6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b63a6f, 95 },
    { 0xce063a6f, 87 },
    { 0x4c63a70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b63a74, 90 },
    { 0x38163a74, 47 },
    { 0x1a763a76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30163a79, 84 },
    { 0, 0 },
    { 0x45563a7b, 83 },
    { 0xc5c63a7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3163a7f, 95 },
    { 0xfd763a80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d63a83, 34 },
    { 0, 0 },
    { 0x81b63a85, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c963a88, 55 },
    { 0x99d63a89, 95 },
    { 0x3bc63a8a, 95 },
    { 0x32463a8b, 87 },
    { 0x6b663a8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34263a8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe363a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7963a9a, 95 },
    { 0xfc563a9b, 61 },
    { 0, 0 },
    { 0xa3e63a9d, 23 },
    { 0x14163a9d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x29963aa1, 68 },
    { 0x19e63aa2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f63ac1, 76 },
    { 0xd0e63ac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73363ac5, 66 },
    { 0, 0 },
    { 0x2ad63ac7, 33 },
    { 0, 0 },
    { 0xcdc63ac9, 90 },
    { 0x2d663aca, 61 },
    { 0x2f163aca, 90 },
    { 0x6a063ac9, 90 },
    { 0xc6963acd, 43 },
    { 0, 0 },
    { 0x3f163acf, 90 },
    { 0xea863ad0, 98 },
    { 0xc1563ad1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f363ad4, 34 },
    { 0x99963ad4, 65 },
    { 0xf3363ad6, 93 },
    { 0x74c63ad7, 55 },
    { 0, 0 },
    { 0x1e363ad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d563ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86063ae9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56763aec, 55 },
    { 0x7e363aed, 61 },
    { 0, 0 },
    { 0xac763aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f963af8, 90 },
    { 0x77363af9, 61 },
    { 0x3e063afa, 95 },
    { 0, 0 },
    { 0x7da63afc, 93 },
    { 0x59663afc, 98 },
    { 0x9b363afe, 90 },
    { 0x8d63afe, 61 },
    { 0, 0 },
    { 0xa4963b01, 67 },
    { 0x29163b01, 85 },
    { 0x81863b03, 68 },
    { 0x34963b03, 61 },
    { 0xa4b63b05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe263b09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f563b0d, 99 },
    { 0, 0 },
    { 0x5a763b0f, 81 },
    { 0x29763b10, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c63b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48663b1c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc63b23, 76 },
    { 0, 0 },
    { 0xe4563b25, 76 },
    { 0x90363b26, 43 },
    { 0x72963b26, 33 },
    { 0xf0f63b25, 100 },
    { 0, 0 },
    { 0x91963b2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1463b31, 66 },
    { 0x2ef63b32, 46 },
    { 0x3863b33, 43 },
    { 0x5cd63b34, 61 },
    { 0xf3e63b31, 90 },
    { 0, 0 },
    { 0x8cf63b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf963b40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab63b43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9063b4b, 70 },
    { 0x6ce63b4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40163b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64063b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d63b5c, 90 },
    { 0x65063b5c, 87 },
    { 0, 0 },
    { 0xda363b5f, 92 },
    { 0, 0 },
    { 0x26563b61, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd63b69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d63b6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd63b73, 61 },
    { 0x1df63b74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c63b7b, 98 },
    { 0xac63b7c, 61 },
    { 0x3d563b7d, 99 },
    { 0x7f863b7e, 76 },
    { 0xa0663b7f, 61 },
    { 0, 0 },
    { 0xb863b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a963b84, 90 },
    { 0xd3563b84, 61 },
    { 0xc6b63b86, 76 },
    { 0xac863b87, 33 },
    { 0x56763b87, 90 },
    { 0x36f63b89, 78 },
    { 0xfcf63b8a, 33 },
    { 0x43f63b88, 46 },
    { 0xa0c63b8c, 90 },
    { 0x71c63b8c, 78 },
    { 0xa0f63b88, 90 },
    { 0x16663b88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21063b94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87263b99, 61 },
    { 0x1dd63b9a, 61 },
    { 0xb563b9b, 65 },
    { 0xf8a63b9a, 17 },
    { 0xcc063b9d, 84 },
    { 0xa7263b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e663ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c863baa, 95 },
    { 0xacd63bab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf63bae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f63bb2, 33 },
    { 0x22263bb2, 34 },
    { 0x14463bb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c63bb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e063bbb, 33 },
    { 0, 0 },
    { 0x78963bbd, 61 },
    { 0x6763bbe, 43 },
    { 0x24563bbf, 46 },
    { 0xf4c63bbe, 67 },
    { 0xd8a63bc1, 61 },
    { 0x77463bc1, 76 },
    { 0x18163bc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a63bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd663bdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64663bdf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa163be2, 34 },
    { 0x15763be3, 93 },
    { 0, 0 },
    { 0x82e63be5, 66 },
    { 0x8ce63be6, 78 },
    { 0x93063be5, 65 },
    { 0xa3b63be8, 67 },
    { 0x4ef63be9, 76 },
    { 0xfc863be9, 23 },
    { 0, 0 },
    { 0xd8e63bec, 91 },
    { 0x17363bed, 76 },
    { 0xb9d63bed, 90 },
    { 0, 0 },
    { 0x97963bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66763bf3, 33 },
    { 0, 0 },
    { 0x74863bf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29763bf9, 61 },
    { 0xb9063bfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a63c04, 90 },
    { 0, 0 },
    { 0x6c463c06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20463c0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f63c12, 91 },
    { 0x30563c13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47163c16, 77 },
    { 0xf8a63c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26763c20, 23 },
    { 0, 0 },
    { 0x23263c22, 84 },
    { 0x9d263c22, 95 },
    { 0x4dc63c24, 33 },
    { 0xb1263c24, 33 },
    { 0xa8263c26, 36 },
    { 0xdcf63c24, 95 },
    { 0xe0363c28, 95 },
    { 0x6c163c29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e63c2c, 33 },
    { 0xf0963c2d, 90 },
    { 0x40e63c2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e663c36, 90 },
    { 0x83363c37, 46 },
    { 0xadd63c38, 55 },
    { 0x56663c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39763c3d, 98 },
    { 0xe1463c3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa63c42, 55 },
    { 0x2ee63c43, 33 },
    { 0, 0 },
    { 0x83963c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d63c4b, 98 },
    { 0x10c63c4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36663c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4063c56, 78 },
    { 0x65d63c57, 43 },
    { 0xffb63c58, 76 },
    { 0x1f663c59, 40 },
    { 0xea963c5a, 95 },
    { 0x4c663c5b, 61 },
    { 0xe3f63c5b, 93 },
    { 0xece63c5d, 23 },
    { 0, 0 },
    { 0x90a63c5f, 33 },
    { 0, 0 },
    { 0xc5c63c61, 90 },
    { 0, 0 },
    { 0x55463c63, 34 },
    { 0x8df63c64, 52 },
    { 0x8663c65, 81 },
    { 0x8fb63c66, 61 },
    { 0x54963c67, 17 },
    { 0x22d63c67, 95 },
    { 0x33f63c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b463c6c, 77 },
    { 0xe8263c6d, 76 },
    { 0xe3363c6e, 90 },
    { 0xfec63c6e, 46 },
    { 0x16863c70, 33 },
    { 0xc7363c71, 24 },
    { 0xb9b63c72, 61 },
    { 0xaae63c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee63c77, 91 },
    { 0xb5d63c77, 93 },
    { 0x77c63c79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a763c7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c63c85, 98 },
    { 0, 0 },
    { 0x85763c87, 95 },
    { 0, 0 },
    { 0x95d63c89, 61 },
    { 0, 0 },
    { 0xe3a63c8b, 52 },
    { 0x763c8b, 98 },
    { 0x61463c8d, 61 },
    { 0x36463c8e, 34 },
    { 0x90963c8c, 92 },
    { 0xd4863c8d, 75 },
    { 0x41f63c91, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9563c94, 85 },
    { 0, 0 },
    { 0x2ad63c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85063c9a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7663c9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a163ca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3863ca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2163caa, 52 },
    { 0, 0 },
    { 0x87663cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b63cb5, 80 },
    { 0, 0 },
    { 0x21063cb7, 99 },
    { 0xc6963cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc863cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4163cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b263cc2, 61 },
    { 0x6c763cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d63cc6, 87 },
    { 0xe7663cc7, 34 },
    { 0, 0 },
    { 0x9cc63cc9, 61 },
    { 0xd3663cc9, 76 },
    { 0x62763cc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e63cd2, 92 },
    { 0xf0e63cd3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7963cd6, 67 },
    { 0xd1263cd7, 43 },
    { 0, 0 },
    { 0xd4563cd9, 53 },
    { 0x38f63cda, 61 },
    { 0xef563cdb, 33 },
    { 0, 0 },
    { 0xe8c63cdd, 77 },
    { 0, 0 },
    { 0x2463cdf, 34 },
    { 0x99563cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa63ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74463ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d63ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0763cf0, 91 },
    { 0x50e63cf1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16363cf6, 67 },
    { 0, 0 },
    { 0xfcc63cf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163cfb, 61 },
    { 0xa8163cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7263d02, 78 },
    { 0xc8563d02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90763d07, 87 },
    { 0x91463d07, 23 },
    { 0, 0 },
    { 0x89d63d0a, 66 },
    { 0, 0 },
    { 0x1af63d0c, 34 },
    { 0xc8163d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c963d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd63d16, 56 },
    { 0x36463d16, 67 },
    { 0, 0 },
    { 0xdee63d19, 95 },
    { 0x35963d1a, 88 },
    { 0x20963d19, 64 },
    { 0x10c63d1c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44963d22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b63d25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0163d30, 61 },
    { 0x98063d31, 76 },
    { 0x963d32, 88 },
    { 0xcf363d33, 81 },
    { 0x1d763d33, 90 },
    { 0x8fd63d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d63d38, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c063d3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c63d4a, 23 },
    { 0x55d63d4b, 61 },
    { 0x8ed63d4a, 68 },
    { 0x68363d4a, 33 },
    { 0xadf63d4e, 61 },
    { 0, 0 },
    { 0xda563d50, 88 },
    { 0, 0 },
    { 0x1f563d52, 100 },
    { 0x9c463d53, 95 },
    { 0xd5c63d53, 95 },
    { 0, 0 },
    { 0x5b063d56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9763d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5263d5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7063d65, 55 },
    { 0x8e863d66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5763d6a, 76 },
    { 0xa8963d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7663d6e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8763d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x80963d74, 78 },
    { 0xe9163d75, 33 },
    { 0xbba63d76, 23 },
    { 0xb6063d76, 90 },
    { 0x79963d77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a63d7f, 90 },
    { 0, 0 },
    { 0x39763d81, 61 },
    { 0x2eb63d81, 98 },
    { 0xed763d82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad63d8b, 55 },
    { 0x28f63d8b, 23 },
    { 0, 0 },
    { 0x8e63d8e, 88 },
    { 0x38063d8f, 37 },
    { 0x7d563d90, 95 },
    { 0x7b663d90, 23 },
    { 0xf1263d90, 95 },
    { 0x78f63d93, 33 },
    { 0xa7d63d92, 95 },
    { 0xc7f63d8e, 90 },
    { 0xb9b63d96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf63d9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad463d9e, 78 },
    { 0, 0 },
    { 0x1ca63da0, 65 },
    { 0xe3163da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e63dae, 61 },
    { 0xca463daf, 98 },
    { 0, 0 },
    { 0x5bc63db1, 33 },
    { 0x58663db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23363db5, 87 },
    { 0xbbf63db6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead63dbb, 46 },
    { 0x15c63dbc, 34 },
    { 0xf263dbd, 76 },
    { 0x72663dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29863dc1, 95 },
    { 0x17763dc2, 76 },
    { 0x16c63dc3, 95 },
    { 0, 0 },
    { 0xb3763dc5, 95 },
    { 0x8a263dc5, 61 },
    { 0x26b63dc5, 85 },
    { 0, 0 },
    { 0x29663dc9, 98 },
    { 0x10763dca, 34 },
    { 0, 0 },
    { 0x67163dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf63dcf, 34 },
    { 0, 0 },
    { 0x6db63dd1, 33 },
    { 0x80f63dd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b63dd5, 46 },
    { 0xf0f63dd6, 88 },
    { 0x6a063dd6, 95 },
    { 0x5b063dd7, 82 },
    { 0x16663dd9, 61 },
    { 0xbaa63dd9, 93 },
    { 0xe0463ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2663de2, 90 },
    { 0x28e63de2, 77 },
    { 0xaf563de4, 81 },
    { 0xc4263de5, 33 },
    { 0, 0 },
    { 0x15a63de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa763ded, 61 },
    { 0x55563dee, 76 },
    { 0x1f663def, 77 },
    { 0xa4063def, 95 },
    { 0x58763df1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a63df5, 93 },
    { 0x8463df6, 33 },
    { 0x40b63df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a63dfc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d63e01, 34 },
    { 0x2f763e02, 23 },
    { 0x8c663e02, 85 },
    { 0x20563e02, 90 },
    { 0xc3d63e05, 98 },
    { 0x32163e06, 82 },
    { 0xde263e07, 33 },
    { 0xd7963e07, 87 },
    { 0xf8063e08, 93 },
    { 0x64e63e0a, 61 },
    { 0, 0 },
    { 0x81363e0c, 61 },
    { 0x72663e0d, 88 },
    { 0xb0963e0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab063e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a263e18, 95 },
    { 0x89a63e18, 90 },
    { 0x72363e1a, 98 },
    { 0x4fc63e1b, 83 },
    { 0x27e63e1b, 43 },
    { 0x3bb63e1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c63e23, 100 },
    { 0x6dc63e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba63e28, 68 },
    { 0x9b363e29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36263e2f, 47 },
    { 0, 0 },
    { 0x42263e31, 61 },
    { 0, 0 },
    { 0x863e33, 24 },
    { 0, 0 },
    { 0xb7563e35, 46 },
    { 0x74a63e35, 87 },
    { 0, 0 },
    { 0xcd863e38, 66 },
    { 0xbbd63e38, 46 },
    { 0x67e63e3a, 98 },
    { 0x2763e3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x93263e3e, 84 },
    { 0, 0 },
    { 0x4a563e40, 61 },
    { 0x58563e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b63e47, 68 },
    { 0x9b163e48, 61 },
    { 0x9a663e49, 90 },
    { 0xea763e48, 90 },
    { 0xabc63e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef263e4f, 61 },
    { 0xa3d63e50, 69 },
    { 0x75563e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa563e54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d63e57, 91 },
    { 0xd9a63e58, 33 },
    { 0x23863e58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97963e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe163e6a, 46 },
    { 0xac63e6a, 98 },
    { 0x1d763e6c, 33 },
    { 0x86563e6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42563e75, 90 },
    { 0x43663e76, 91 },
    { 0, 0 },
    { 0xf7663e78, 61 },
    { 0, 0 },
    { 0x57663e7a, 76 },
    { 0x6c063e7b, 90 },
    { 0, 0 },
    { 0x50663e7d, 61 },
    { 0x7f663e7e, 33 },
    { 0x2f963e7d, 46 },
    { 0x6f863e7d, 76 },
    { 0x81e63e81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85063e85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf63e8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f63e96, 95 },
    { 0x3b463e97, 90 },
    { 0, 0 },
    { 0x5b163e99, 91 },
    { 0x68663e9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b63ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24363ea3, 34 },
    { 0x28b63ea4, 90 },
    { 0, 0 },
    { 0x24b63ea6, 61 },
    { 0, 0 },
    { 0xbfe63ea8, 61 },
    { 0, 0 },
    { 0xf5f63eaa, 38 },
    { 0xdeb63eab, 23 },
    { 0x4063eac, 23 },
    { 0x18263eac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17163eb4, 77 },
    { 0xdec63eb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61163eb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6263ec0, 90 },
    { 0x31c63ec0, 61 },
    { 0x30563ec1, 43 },
    { 0x70863ec2, 93 },
    { 0xc4d63ec1, 64 },
    { 0x54463ec5, 85 },
    { 0x5b063ec6, 93 },
    { 0xc0b63ec6, 98 },
    { 0xd0963ec3, 90 },
    { 0xe5063ec3, 80 },
    { 0x1e563eca, 55 },
    { 0x6af63ecb, 94 },
    { 0x99863eca, 95 },
    { 0x55563ecd, 88 },
    { 0xa6063eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4663ed1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95763ed4, 52 },
    { 0xda463ed4, 90 },
    { 0x35f63ed6, 83 },
    { 0x2763ed4, 76 },
    { 0, 0 },
    { 0x73d63ed9, 55 },
    { 0x5b763eda, 90 },
    { 0xad763edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1363ee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a463ee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22163eeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e63eee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e63ef6, 34 },
    { 0xf4863ef7, 19 },
    { 0xfcf63ef6, 61 },
    { 0, 0 },
    { 0xe9d63efa, 90 },
    { 0x61b63efb, 33 },
    { 0xf663efa, 90 },
    { 0, 0 },
    { 0xb7d63efe, 68 },
    { 0x5863eff, 66 },
    { 0x53363eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33163f04, 96 },
    { 0xd6f63f04, 93 },
    { 0x15a63f05, 76 },
    { 0, 0 },
    { 0xe2a63f08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x65363f0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c63f0e, 76 },
    { 0xb9b63f0e, 93 },
    { 0x9a563f10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x94763f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba763f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db63f1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c63f1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92063f23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d63f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29863f2d, 98 },
    { 0, 0 },
    { 0xcc963f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9963f36, 33 },
    { 0, 0 },
    { 0xe9163f38, 90 },
    { 0xd8b63f38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7863f3f, 55 },
    { 0x20663f3f, 76 },
    { 0xda763f40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee063f44, 90 },
    { 0x56063f45, 67 },
    { 0x1c663f45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d063f4c, 43 },
    { 0x6a463f4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd863f62, 98 },
    { 0xe3e63f62, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92663f68, 76 },
    { 0xa0463f68, 93 },
    { 0x81663f6a, 80 },
    { 0, 0 },
    { 0x10a63f6c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef63f81, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f963f85, 98 },
    { 0xb4163f86, 93 },
    { 0x46063f86, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70563f8b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8463f95, 91 },
    { 0, 0 },
    { 0x99d63f97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf463f9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc163f9f, 90 },
    { 0x22b63fa0, 90 },
    { 0x29163fa0, 100 },
    { 0, 0 },
    { 0x48c63fa3, 55 },
    { 0, 0 },
    { 0xfb463fa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17663fad, 33 },
    { 0x35663fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63fbb, 64 },
    { 0, 0 },
    { 0x4ab63fbd, 95 },
    { 0x4763fbd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0563fc2, 98 },
    { 0x71163fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3563fc7, 90 },
    { 0x2ea63fc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca963fcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc063fd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa763fe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4263fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb063fec, 95 },
    { 0, 0 },
    { 0xa8363fee, 90 },
    { 0, 0 },
    { 0x64563ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf063ff5, 52 },
    { 0x3ce63ff6, 61 },
    { 0x6dd63ff7, 34 },
    { 0xb6363ff8, 90 },
    { 0x69863ff7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663ffd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e64002, 90 },
    { 0, 0 },
    { 0x9df64004, 43 },
    { 0x48964005, 33 },
    { 0, 0 },
    { 0x8ae64007, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd6400a, 98 },
    { 0xceb6400a, 46 },
    { 0, 0 },
    { 0x36c6400d, 99 },
    { 0xf5c6400e, 95 },
    { 0x7aa6400f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xba64012, 90 },
    { 0, 0 },
    { 0xa4364014, 90 },
    { 0x4dd64015, 23 },
    { 0x9564016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d46401d, 69 },
    { 0xe416401d, 91 },
    { 0xa426401f, 88 },
    { 0xe106401f, 95 },
    { 0xd5e6401d, 76 },
    { 0x58d6401f, 86 },
    { 0xafb64023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df64028, 95 },
    { 0x1464029, 95 },
    { 0x4326402a, 91 },
    { 0xbdd6402a, 43 },
    { 0x5b26402b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9964030, 98 },
    { 0xd9c64031, 61 },
    { 0x9cc64032, 33 },
    { 0, 0 },
    { 0x92a64034, 46 },
    { 0x1af64034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b64038, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66064041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2164044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b64047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a6404a, 61 },
    { 0, 0 },
    { 0x6f46404c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686404f, 38 },
    { 0xef264050, 55 },
    { 0x55e64050, 23 },
    { 0, 0 },
    { 0x16364053, 55 },
    { 0x3be64054, 84 },
    { 0x62664054, 95 },
    { 0, 0 },
    { 0xf9164057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6405a, 17 },
    { 0x29a6405a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe6405e, 33 },
    { 0x12b6405f, 77 },
    { 0xbe364060, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91164066, 61 },
    { 0xbf964067, 33 },
    { 0x70e64067, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db6406e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46864071, 33 },
    { 0x7c764071, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d64075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046408a, 33 },
    { 0x5b66408b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa596408e, 43 },
    { 0xfd46408e, 90 },
    { 0xaea64090, 76 },
    { 0xeaf64091, 95 },
    { 0x7464092, 91 },
    { 0xf2f64093, 90 },
    { 0, 0 },
    { 0x5f064095, 61 },
    { 0x58c64096, 55 },
    { 0xae064096, 90 },
    { 0, 0 },
    { 0xc0864099, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5816409d, 52 },
    { 0, 0 },
    { 0x3b26409f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576640a7, 61 },
    { 0x5b0640a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18640ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8640b2, 78 },
    { 0x12b640b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64640b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c640b9, 33 },
    { 0x57b640b9, 98 },
    { 0, 0 },
    { 0x120640bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5640c1, 52 },
    { 0x4ae640c1, 90 },
    { 0xebc640c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d640c7, 24 },
    { 0x954640c8, 85 },
    { 0x39d640c7, 23 },
    { 0, 0 },
    { 0xc44640cb, 52 },
    { 0xc7b640cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x651640cf, 98 },
    { 0xdbe640cf, 17 },
    { 0xc96640cf, 43 },
    { 0, 0 },
    { 0xef5640d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838640d8, 85 },
    { 0x496640d9, 90 },
    { 0x975640da, 47 },
    { 0xdf0640db, 90 },
    { 0x2d8640dc, 52 },
    { 0x747640db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf640e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6640ea, 33 },
    { 0x865640ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d640f8, 55 },
    { 0x4d4640f8, 95 },
    { 0xf3a640f9, 33 },
    { 0xbc2640fb, 61 },
    { 0x52c640f9, 61 },
    { 0x6a8640fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b64102, 46 },
    { 0xaed64102, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c64107, 76 },
    { 0x12064108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb626410c, 61 },
    { 0xbcb6410c, 33 },
    { 0x3876410d, 90 },
    { 0xb646410f, 95 },
    { 0xe0c64110, 85 },
    { 0xf6164111, 33 },
    { 0x6ab64112, 33 },
    { 0x99064113, 90 },
    { 0x1fb64113, 66 },
    { 0x51464110, 76 },
    { 0, 0 },
    { 0xf3864117, 100 },
    { 0x4b864118, 38 },
    { 0, 0 },
    { 0xf0c6411a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18664123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9596412f, 61 },
    { 0, 0 },
    { 0xa1664131, 61 },
    { 0x7a864132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe716413a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af6413f, 90 },
    { 0x2cc64140, 55 },
    { 0, 0 },
    { 0xd5864142, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c164145, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc6414b, 68 },
    { 0, 0 },
    { 0xe686414d, 90 },
    { 0xefc6414e, 66 },
    { 0x8106414f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81564155, 85 },
    { 0xbb964155, 77 },
    { 0xb5e64157, 88 },
    { 0x1fb64155, 61 },
    { 0xc4a64159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6486415d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be64160, 95 },
    { 0x89464161, 65 },
    { 0x42a64162, 67 },
    { 0xbf464163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60464166, 61 },
    { 0x66f64167, 64 },
    { 0, 0 },
    { 0x66464169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16416c, 76 },
    { 0xff06416c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f64173, 80 },
    { 0, 0 },
    { 0x57464175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb16417b, 76 },
    { 0x8cb6417b, 99 },
    { 0xec76417b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4764180, 23 },
    { 0xf0464181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb964184, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6418a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4896418d, 77 },
    { 0, 0 },
    { 0x4f76418f, 95 },
    { 0x2fc6418f, 82 },
    { 0xc5364191, 90 },
    { 0xc1564190, 61 },
    { 0, 0 },
    { 0xf2964194, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d464198, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e6419b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b6419f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49641a4, 90 },
    { 0, 0 },
    { 0xb641a6, 90 },
    { 0, 0 },
    { 0x185641a8, 61 },
    { 0x19641a8, 88 },
    { 0x33f641a8, 67 },
    { 0, 0 },
    { 0xbad641ac, 76 },
    { 0xf07641ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50641b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8641ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x843641bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3641c3, 8 },
    { 0, 0 },
    { 0xa4e641c5, 90 },
    { 0xf11641c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5641c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6641cc, 23 },
    { 0xfba641cd, 89 },
    { 0x212641ce, 34 },
    { 0, 0 },
    { 0xf3e641d0, 69 },
    { 0xdc3641d0, 98 },
    { 0x180641d2, 67 },
    { 0xa4641d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7641d7, 90 },
    { 0x217641d8, 99 },
    { 0, 0 },
    { 0x752641da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb641e0, 100 },
    { 0x9e0641e1, 34 },
    { 0xf67641e2, 95 },
    { 0x434641e0, 23 },
    { 0xc6c641e2, 69 },
    { 0xc10641e2, 43 },
    { 0xeed641e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68641e9, 100 },
    { 0x7ff641ea, 85 },
    { 0x98a641ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e641f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41641f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9641fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb764200, 23 },
    { 0, 0 },
    { 0xdeb64202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd64205, 55 },
    { 0, 0 },
    { 0x52064207, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6420b, 33 },
    { 0, 0 },
    { 0xec86420d, 76 },
    { 0x6d06420d, 59 },
    { 0, 0 },
    { 0x65464210, 90 },
    { 0x31364211, 78 },
    { 0xcd64211, 90 },
    { 0x69964213, 93 },
    { 0x3a964214, 88 },
    { 0x60764215, 85 },
    { 0xf7864213, 23 },
    { 0x9e164217, 90 },
    { 0x79864215, 38 },
    { 0x2f964213, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464220, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a364225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7036422a, 98 },
    { 0x5c66422b, 99 },
    { 0x3fc6422c, 68 },
    { 0x4886422d, 76 },
    { 0x2636422c, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75664234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876423b, 33 },
    { 0x4cd6423c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc164243, 23 },
    { 0xb0a64244, 33 },
    { 0, 0 },
    { 0xf6564246, 66 },
    { 0x11564247, 90 },
    { 0xab464248, 98 },
    { 0xf2c64249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046424c, 87 },
    { 0xaaf6424d, 90 },
    { 0x61d6424e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf564253, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73464256, 98 },
    { 0x8c264256, 84 },
    { 0x2c164256, 87 },
    { 0xd0e64256, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8856425c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85664260, 33 },
    { 0x69f64261, 23 },
    { 0xb8e64262, 78 },
    { 0x80464263, 90 },
    { 0xee64264, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e6426a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c66426d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f664270, 55 },
    { 0x1e64270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6156427a, 55 },
    { 0x9706427a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc64282, 65 },
    { 0x8c764283, 23 },
    { 0x44864284, 76 },
    { 0, 0 },
    { 0x56e64286, 81 },
    { 0x16f64286, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6428a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c26428d, 34 },
    { 0, 0 },
    { 0x5a86428f, 76 },
    { 0x5dc6428f, 98 },
    { 0, 0 },
    { 0xd2664292, 46 },
    { 0, 0 },
    { 0xf1464294, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0964297, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b06429e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a642a6, 98 },
    { 0, 0 },
    { 0xa0a642a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3642af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9642b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5642b6, 90 },
    { 0x72c642b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873642bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc642be, 95 },
    { 0x143642bf, 65 },
    { 0xa04642c0, 43 },
    { 0x801642c1, 61 },
    { 0x373642bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8642c7, 90 },
    { 0x3d7642c7, 91 },
    { 0x528642c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e642cd, 33 },
    { 0x3a7642cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x284642d1, 93 },
    { 0xf23642d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88642d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c642d8, 87 },
    { 0xfe4642d9, 87 },
    { 0x6f2642da, 90 },
    { 0xc90642d8, 85 },
    { 0x647642da, 33 },
    { 0xc3b642db, 55 },
    { 0x29a642dd, 61 },
    { 0x5e6642dc, 90 },
    { 0xf13642db, 66 },
    { 0x179642da, 17 },
    { 0, 0 },
    { 0x5cc642e3, 99 },
    { 0, 0 },
    { 0x1c4642e5, 90 },
    { 0x620642e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9642eb, 66 },
    { 0x4dd642ec, 98 },
    { 0xa41642ec, 95 },
    { 0xf0f642ee, 43 },
    { 0x3db642eb, 33 },
    { 0xba6642eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x648642f3, 95 },
    { 0x395642f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e642f9, 90 },
    { 0xa5642fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36642ff, 55 },
    { 0xdf264300, 23 },
    { 0xba564301, 33 },
    { 0xad0642ff, 76 },
    { 0, 0 },
    { 0x3f064304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed66430a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7876430f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab64314, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd64318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee26431b, 33 },
    { 0, 0 },
    { 0x25d6431d, 90 },
    { 0xf176431e, 34 },
    { 0x4fc6431f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56864323, 43 },
    { 0x4e164323, 61 },
    { 0, 0 },
    { 0xffb64326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd6432b, 95 },
    { 0, 0 },
    { 0x7d6432d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3764332, 34 },
    { 0xc6164333, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6433b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86433e, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70264343, 33 },
    { 0, 0 },
    { 0xa1e64345, 95 },
    { 0x78064346, 87 },
    { 0, 0 },
    { 0x7cd64348, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a86434c, 76 },
    { 0xe6e6434d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27664351, 76 },
    { 0xdee64352, 42 },
    { 0x4264351, 61 },
    { 0, 0 },
    { 0x4064355, 33 },
    { 0, 0 },
    { 0xc2e64357, 95 },
    { 0xbdd64357, 90 },
    { 0x6fb64358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b264362, 91 },
    { 0xb5264362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e664368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d6436f, 90 },
    { 0, 0 },
    { 0xaaa64371, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40264374, 95 },
    { 0x7e164375, 90 },
    { 0, 0 },
    { 0xaea64377, 64 },
    { 0x1c864378, 84 },
    { 0x7a264378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f64382, 90 },
    { 0x9da64383, 98 },
    { 0x27a64384, 98 },
    { 0x95264382, 61 },
    { 0x91a64386, 90 },
    { 0x44464387, 91 },
    { 0x1a064387, 95 },
    { 0xd2964388, 37 },
    { 0x8ba64386, 76 },
    { 0x4e6438b, 83 },
    { 0x1dc6438c, 23 },
    { 0xd626438c, 23 },
    { 0, 0 },
    { 0x5006438f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf164393, 77 },
    { 0xa8664394, 98 },
    { 0x4e364393, 90 },
    { 0x8ca64395, 61 },
    { 0x69864394, 61 },
    { 0x40f64398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2643a0, 98 },
    { 0x205643a1, 100 },
    { 0, 0 },
    { 0x8c8643a3, 61 },
    { 0x9643a4, 33 },
    { 0xe21643a4, 95 },
    { 0xecb643a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79643ad, 23 },
    { 0xb82643ae, 23 },
    { 0xcbe643ae, 34 },
    { 0xea7643b0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36643b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1643b8, 77 },
    { 0x795643b8, 23 },
    { 0, 0 },
    { 0x3b1643bb, 43 },
    { 0xc38643bb, 61 },
    { 0, 0 },
    { 0x667643be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1643c5, 95 },
    { 0x63643c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f643cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9643d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d643d6, 90 },
    { 0x1e8643d7, 43 },
    { 0xbac643d8, 23 },
    { 0xb45643d9, 76 },
    { 0xbf643da, 84 },
    { 0x256643d9, 55 },
    { 0xc5b643db, 90 },
    { 0x4e5643dd, 90 },
    { 0x6ea643de, 90 },
    { 0xbe7643df, 33 },
    { 0xfa2643df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793643e9, 34 },
    { 0, 0 },
    { 0x4d5643eb, 65 },
    { 0xed3643ec, 90 },
    { 0, 0 },
    { 0x6a5643ee, 91 },
    { 0x951643ef, 33 },
    { 0, 0 },
    { 0x2bb643f1, 33 },
    { 0, 0 },
    { 0xe31643f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a643f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28643ff, 90 },
    { 0, 0 },
    { 0x3e464401, 23 },
    { 0x80c64402, 33 },
    { 0, 0 },
    { 0x32764404, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea64407, 98 },
    { 0x19464408, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe64411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91064414, 17 },
    { 0xc5b64415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9964418, 66 },
    { 0x4d564419, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e6441d, 87 },
    { 0, 0 },
    { 0x8e56441f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764422, 33 },
    { 0xbbe64422, 61 },
    { 0xbe864424, 98 },
    { 0x19164425, 61 },
    { 0xb0c64425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc64430, 87 },
    { 0x41764431, 90 },
    { 0, 0 },
    { 0xafd64433, 55 },
    { 0xf4a64434, 23 },
    { 0, 0 },
    { 0xdeb64436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71764439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f6443f, 55 },
    { 0x39264440, 76 },
    { 0x42164441, 55 },
    { 0xe2864441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc64445, 34 },
    { 0xfeb64445, 61 },
    { 0, 0 },
    { 0xde064448, 95 },
    { 0x53f64449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9764451, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf964457, 61 },
    { 0x49764457, 90 },
    { 0x50364459, 61 },
    { 0, 0 },
    { 0x5c66445b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c64460, 17 },
    { 0, 0 },
    { 0x3b564462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b364465, 93 },
    { 0x85264466, 84 },
    { 0x83a64467, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c6446d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c64470, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x42064473, 61 },
    { 0, 0 },
    { 0x8fb64475, 33 },
    { 0, 0 },
    { 0x70464477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f36447a, 34 },
    { 0, 0 },
    { 0xc46447c, 75 },
    { 0, 0 },
    { 0x6906447e, 61 },
    { 0xe326447f, 66 },
    { 0x24f64480, 23 },
    { 0xa3e6447f, 55 },
    { 0, 0 },
    { 0x90f64483, 55 },
    { 0xc6c64484, 90 },
    { 0x62b64484, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd226448a, 53 },
    { 0, 0 },
    { 0xa836448c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c64490, 34 },
    { 0x87264491, 95 },
    { 0x8b864492, 75 },
    { 0x44a64492, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c64498, 76 },
    { 0x52a64499, 98 },
    { 0x6666449a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb66449f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f644a2, 34 },
    { 0xb47644a3, 61 },
    { 0x91a644a2, 90 },
    { 0x593644a5, 61 },
    { 0, 0 },
    { 0xada644a7, 76 },
    { 0, 0 },
    { 0x4df644a9, 90 },
    { 0, 0 },
    { 0x851644ab, 90 },
    { 0, 0 },
    { 0x121644ad, 98 },
    { 0x370644ae, 51 },
    { 0x836644af, 76 },
    { 0, 0 },
    { 0x982644b1, 76 },
    { 0xfd0644b1, 61 },
    { 0x582644b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a644b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90644c1, 61 },
    { 0x3dc644c2, 95 },
    { 0xf40644c2, 90 },
    { 0xc1d644c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30644c7, 61 },
    { 0, 0 },
    { 0x120644c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412644ce, 95 },
    { 0, 0 },
    { 0xe2644d0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc644d5, 84 },
    { 0xaa7644d5, 76 },
    { 0x65d644d5, 34 },
    { 0xaaf644d8, 90 },
    { 0x35d644d7, 75 },
    { 0xce1644da, 90 },
    { 0, 0 },
    { 0x489644dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa644e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b644ea, 80 },
    { 0xfd5644eb, 76 },
    { 0xec9644ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23644f1, 23 },
    { 0, 0 },
    { 0x414644f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f644f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946644fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e64500, 98 },
    { 0xfb564501, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95964506, 34 },
    { 0, 0 },
    { 0xae264508, 17 },
    { 0x83764508, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9026450d, 55 },
    { 0xf5a6450e, 66 },
    { 0xfa26450e, 33 },
    { 0xca164510, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a64515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7564518, 76 },
    { 0, 0 },
    { 0xb2b6451a, 99 },
    { 0xa416451a, 98 },
    { 0x6f46451b, 99 },
    { 0, 0 },
    { 0xede6451e, 84 },
    { 0, 0 },
    { 0x49164520, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb464525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6452b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6766452f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa64533, 33 },
    { 0x60c64534, 61 },
    { 0x10664535, 90 },
    { 0x7f064535, 98 },
    { 0, 0 },
    { 0xa2064538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef6453d, 43 },
    { 0, 0 },
    { 0xb866453f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xac464542, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6454a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3596454d, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xab564550, 76 },
    { 0x1b064551, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d64554, 23 },
    { 0xc3764555, 17 },
    { 0x7ee64554, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc6455b, 66 },
    { 0xffd6455c, 95 },
    { 0x586455d, 90 },
    { 0x5136455b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564565, 52 },
    { 0, 0 },
    { 0xd1464567, 90 },
    { 0xba264568, 66 },
    { 0x3e364569, 85 },
    { 0x91d6456a, 33 },
    { 0x7ef64567, 67 },
    { 0xb7c6456c, 85 },
    { 0x8606456c, 34 },
    { 0x6366456e, 33 },
    { 0, 0 },
    { 0xa0364570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b264574, 95 },
    { 0xa7f64575, 61 },
    { 0x8eb64574, 75 },
    { 0xd6064577, 66 },
    { 0, 0 },
    { 0x91e64579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6457c, 85 },
    { 0xea86457c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58464580, 100 },
    { 0x49d64581, 33 },
    { 0x5dd64582, 88 },
    { 0x23c64583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564588, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf64591, 33 },
    { 0xfec64592, 33 },
    { 0x1e464593, 90 },
    { 0x66a64593, 61 },
    { 0, 0 },
    { 0x3d964596, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c264599, 66 },
    { 0, 0 },
    { 0x4fb6459b, 55 },
    { 0xc926459b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb766459f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3645a8, 61 },
    { 0, 0 },
    { 0x3e5645aa, 81 },
    { 0, 0 },
    { 0xbe3645ac, 23 },
    { 0, 0 },
    { 0x2b3645ae, 76 },
    { 0xe63645af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d645b2, 55 },
    { 0, 0 },
    { 0xbf8645b4, 36 },
    { 0x8f7645b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc645b9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4645bc, 55 },
    { 0xf52645bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b645c0, 33 },
    { 0xfd3645c1, 90 },
    { 0x37f645c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73645cb, 80 },
    { 0, 0 },
    { 0x851645cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2645d8, 61 },
    { 0x4a0645d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2645dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a645e0, 23 },
    { 0x509645e1, 61 },
    { 0x1e4645e2, 80 },
    { 0xcfd645e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea645ec, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4645ef, 87 },
    { 0x894645f0, 34 },
    { 0x1dc645f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144645f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d645fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d64600, 33 },
    { 0xd5a64601, 46 },
    { 0x30364601, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6764607, 43 },
    { 0, 0 },
    { 0x73f64609, 95 },
    { 0, 0 },
    { 0x83a6460b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f64610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64064613, 78 },
    { 0, 0 },
    { 0x30a64615, 23 },
    { 0, 0 },
    { 0xd7e64617, 34 },
    { 0x84064618, 34 },
    { 0xdab64619, 98 },
    { 0xae664619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b26461d, 95 },
    { 0xe2b6461d, 61 },
    { 0x5756461f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac36462f, 33 },
    { 0, 0 },
    { 0x92764631, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x78164634, 33 },
    { 0x67364635, 80 },
    { 0x64664636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a64639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd976463e, 90 },
    { 0x1976463f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a64643, 90 },
    { 0x68864643, 23 },
    { 0, 0 },
    { 0x5fd64646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93864649, 98 },
    { 0x3b46464a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276464e, 90 },
    { 0, 0 },
    { 0x72864650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6164653, 90 },
    { 0xb9064654, 46 },
    { 0x37264655, 91 },
    { 0xfdb64656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a64659, 90 },
    { 0x8666465a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e6465d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b64660, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b64667, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9536466d, 100 },
    { 0x8206466e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc064674, 33 },
    { 0, 0 },
    { 0xf9964676, 23 },
    { 0x5664677, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb6467a, 88 },
    { 0x7f46467a, 98 },
    { 0x4fe6467b, 33 },
    { 0x7bd6467a, 67 },
    { 0x68b6467b, 34 },
    { 0x4d46467f, 81 },
    { 0xa5f64680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e64683, 61 },
    { 0x11464684, 66 },
    { 0x78364685, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c6468b, 100 },
    { 0xb836468c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f56468f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f64693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3396469d, 61 },
    { 0, 0 },
    { 0x46d6469f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8646a5, 55 },
    { 0xb9a646a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0646af, 33 },
    { 0x142646af, 95 },
    { 0xd03646b1, 43 },
    { 0xaff646b2, 87 },
    { 0x842646b3, 95 },
    { 0x10b646b0, 83 },
    { 0, 0 },
    { 0x531646b6, 32 },
    { 0x5ad646b6, 68 },
    { 0xf9a646b8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12646be, 66 },
    { 0, 0 },
    { 0x2fc646c0, 88 },
    { 0x35d646c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a646c8, 78 },
    { 0xaac646c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f646cc, 55 },
    { 0x628646cd, 46 },
    { 0xaf1646cd, 67 },
    { 0xcd3646cf, 66 },
    { 0x5f9646cf, 76 },
    { 0xadd646ce, 90 },
    { 0, 0 },
    { 0x887646d3, 61 },
    { 0xeae646d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53646dd, 76 },
    { 0xb01646de, 90 },
    { 0x977646df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x315646e2, 98 },
    { 0xa16646e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01646ed, 76 },
    { 0x13646ed, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df646f3, 52 },
    { 0x114646f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31646f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161646fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f964701, 23 },
    { 0xdcc64701, 23 },
    { 0x4bf64703, 33 },
    { 0x7ad64704, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db64708, 23 },
    { 0xba764708, 78 },
    { 0xe3d64708, 33 },
    { 0x28e6470b, 23 },
    { 0x35864709, 83 },
    { 0, 0 },
    { 0xc886470e, 97 },
    { 0x7076470e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a364717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4576471b, 61 },
    { 0x12e6471c, 99 },
    { 0xeb06471c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd64724, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f264728, 87 },
    { 0x70064729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c6472e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98664732, 93 },
    { 0x19e64733, 98 },
    { 0, 0 },
    { 0x35664735, 95 },
    { 0x2eb64736, 88 },
    { 0xfce64736, 69 },
    { 0x21864737, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916473d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72064740, 76 },
    { 0x6bf64741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed64746, 99 },
    { 0x98964746, 99 },
    { 0x6c064748, 34 },
    { 0xfea64748, 8 },
    { 0x8f26474a, 46 },
    { 0x2166474a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab64750, 61 },
    { 0, 0 },
    { 0xcbf64752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6475e, 43 },
    { 0xe896475e, 90 },
    { 0x9376475e, 90 },
    { 0, 0 },
    { 0xa0464762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29864765, 61 },
    { 0, 0 },
    { 0x45064767, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf326476a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6476d, 76 },
    { 0x64e6476e, 95 },
    { 0, 0 },
    { 0x54564770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46764773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88064779, 46 },
    { 0x7b86477a, 80 },
    { 0xe8a6477b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b64783, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d64787, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b6478b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c06478e, 61 },
    { 0xa356478f, 100 },
    { 0xff64790, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164795, 61 },
    { 0, 0 },
    { 0x4ac64797, 76 },
    { 0, 0 },
    { 0xdea64799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2647a1, 90 },
    { 0x9ff647a2, 34 },
    { 0x19d647a3, 23 },
    { 0x87647a4, 61 },
    { 0x66647a3, 61 },
    { 0xc2a647a6, 61 },
    { 0x26a647a7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12647aa, 52 },
    { 0, 0 },
    { 0xb0a647ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6647b0, 61 },
    { 0x940647b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e647be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66647c3, 52 },
    { 0xc29647c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0647ca, 95 },
    { 0x4bc647cb, 77 },
    { 0x427647cc, 82 },
    { 0xd0d647cd, 87 },
    { 0x615647cb, 23 },
    { 0, 0 },
    { 0x9df647d0, 34 },
    { 0x59b647d0, 81 },
    { 0xce2647d1, 33 },
    { 0xb17647d3, 61 },
    { 0xd59647d3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6647de, 61 },
    { 0x374647de, 99 },
    { 0, 0 },
    { 0x1f8647e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db647e7, 95 },
    { 0x61e647e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe647eb, 23 },
    { 0xec9647eb, 33 },
    { 0x259647eb, 95 },
    { 0xcaf647ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16647f6, 23 },
    { 0x94b647f7, 85 },
    { 0xe01647f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x287647fb, 99 },
    { 0xc7b647fc, 55 },
    { 0x275647fd, 43 },
    { 0x7a5647fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48664804, 98 },
    { 0, 0 },
    { 0x43464806, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d64809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6480c, 98 },
    { 0x91f6480d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b64810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea964813, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464819, 87 },
    { 0x5ea6481a, 42 },
    { 0x7c66481a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6481e, 23 },
    { 0xc5d6481e, 90 },
    { 0x43d6481e, 99 },
    { 0, 0 },
    { 0xd4a64822, 61 },
    { 0xd5a64823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2564826, 95 },
    { 0x49d64826, 55 },
    { 0, 0 },
    { 0x64164829, 69 },
    { 0x6df6482a, 61 },
    { 0, 0 },
    { 0xe796482c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14964831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76483a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e6483f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d764842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a364847, 88 },
    { 0x6ec64847, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3236484c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13164850, 90 },
    { 0xd5f64851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d164856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f964861, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a64866, 55 },
    { 0x82264866, 34 },
    { 0x71164868, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x596486b, 93 },
    { 0xe56486b, 76 },
    { 0, 0 },
    { 0xbb6486e, 98 },
    { 0xf936486f, 2 },
    { 0, 0 },
    { 0x1e264871, 23 },
    { 0, 0 },
    { 0xecf64873, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f64879, 90 },
    { 0, 0 },
    { 0xb9a6487b, 43 },
    { 0xe2c6487c, 33 },
    { 0, 0 },
    { 0x5606487e, 100 },
    { 0, 0 },
    { 0x9fb64880, 65 },
    { 0xb8e64881, 40 },
    { 0, 0 },
    { 0x26b64883, 61 },
    { 0xe64884, 61 },
    { 0x67c64885, 90 },
    { 0x47564886, 36 },
    { 0x6d64886, 43 },
    { 0xc4464884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b64893, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55464898, 95 },
    { 0, 0 },
    { 0x7576489a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87648a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20648a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa648a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73648ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf648b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x954648b5, 64 },
    { 0xa02648b6, 98 },
    { 0x51a648b7, 99 },
    { 0x17e648b8, 61 },
    { 0xd7b648b9, 96 },
    { 0x135648ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44648c2, 33 },
    { 0x7f0648c3, 44 },
    { 0x208648c2, 95 },
    { 0x8d7648c4, 33 },
    { 0xfe0648c2, 46 },
    { 0x67c648c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209648cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e648d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27648d6, 87 },
    { 0x843648d7, 95 },
    { 0x401648d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d648de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339648e2, 23 },
    { 0x18648e3, 61 },
    { 0, 0 },
    { 0xc99648e5, 46 },
    { 0xcdc648e6, 81 },
    { 0x99e648e6, 34 },
    { 0, 0 },
    { 0x65a648e9, 90 },
    { 0x6cb648ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e648f2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0648fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22648fd, 98 },
    { 0x320648fd, 34 },
    { 0x882648ff, 61 },
    { 0x100648ff, 90 },
    { 0xf3e64901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c364910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1464915, 95 },
    { 0x49564916, 99 },
    { 0, 0 },
    { 0xdb964918, 90 },
    { 0, 0 },
    { 0xe366491a, 52 },
    { 0xdf66491a, 23 },
    { 0xb846491c, 90 },
    { 0xbb36491d, 33 },
    { 0, 0 },
    { 0x9dd6491f, 55 },
    { 0x28664920, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e264924, 97 },
    { 0xe4c64925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166492a, 93 },
    { 0xdb66492b, 33 },
    { 0x1ec6492a, 43 },
    { 0xdb06492b, 95 },
    { 0x5e56492e, 33 },
    { 0x15a6492f, 43 },
    { 0xb186492f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c864934, 61 },
    { 0x7b264935, 99 },
    { 0x8dd64936, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e6493b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6493e, 90 },
    { 0, 0 },
    { 0xc8364940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd64944, 33 },
    { 0, 0 },
    { 0xf0f64946, 61 },
    { 0x2664946, 33 },
    { 0x3cb64948, 23 },
    { 0x33e64949, 61 },
    { 0xad964948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe064950, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964955, 95 },
    { 0x55c64956, 9 },
    { 0, 0 },
    { 0x46764958, 24 },
    { 0x22f64959, 33 },
    { 0x9326495a, 61 },
    { 0, 0 },
    { 0x8276495c, 87 },
    { 0x3be6495d, 98 },
    { 0, 0 },
    { 0x5106495f, 61 },
    { 0x8326495f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6496d, 98 },
    { 0x31c6496e, 33 },
    { 0x46d6496f, 33 },
    { 0x3e464970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a064973, 23 },
    { 0x12264973, 90 },
    { 0xc1664973, 23 },
    { 0x3a764976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf46497b, 61 },
    { 0xef6497c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d66497f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a64985, 95 },
    { 0xa4b64985, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56498c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f64993, 55 },
    { 0xea164994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6964997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6499c, 33 },
    { 0x3d56499c, 23 },
    { 0x5226499c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7649a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85649ac, 85 },
    { 0, 0 },
    { 0xe83649ae, 88 },
    { 0xff9649ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x612649b2, 55 },
    { 0x726649b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381649bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569649c0, 65 },
    { 0, 0 },
    { 0xb8b649c2, 61 },
    { 0, 0 },
    { 0x5a8649c4, 76 },
    { 0x8cd649c4, 17 },
    { 0x39c649c6, 98 },
    { 0, 0 },
    { 0x222649c8, 93 },
    { 0xdb2649c8, 55 },
    { 0x29649ca, 33 },
    { 0x755649cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae649d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6649d4, 66 },
    { 0x78649d5, 84 },
    { 0x816649d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e649dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada649e1, 46 },
    { 0, 0 },
    { 0xde0649e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d649e8, 75 },
    { 0, 0 },
    { 0x238649ea, 75 },
    { 0xb0a649eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739649f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e649f5, 66 },
    { 0x96a649f6, 90 },
    { 0xc4a649f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196649fc, 65 },
    { 0x728649fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d64a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a464a05, 66 },
    { 0xdf64a06, 61 },
    { 0xb2c64a06, 91 },
    { 0xdc464a06, 46 },
    { 0xd8364a09, 34 },
    { 0xa4964a0a, 61 },
    { 0xce364a0a, 23 },
    { 0x37d64a0c, 66 },
    { 0x5d264a0c, 90 },
    { 0, 0 },
    { 0xe8064a0f, 83 },
    { 0x80064a0f, 43 },
    { 0x32364a11, 98 },
    { 0x4fd64a11, 90 },
    { 0xca264a11, 61 },
    { 0x78b64a14, 90 },
    { 0xc5764a13, 76 },
    { 0x69364a16, 85 },
    { 0xc8264a17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c664a1b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34764a20, 34 },
    { 0xfb464a21, 95 },
    { 0, 0 },
    { 0x6d064a23, 46 },
    { 0, 0 },
    { 0xba64a25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0664a29, 43 },
    { 0x764a29, 67 },
    { 0xc1a64a2b, 99 },
    { 0, 0 },
    { 0x8c364a2d, 90 },
    { 0, 0 },
    { 0x14964a2f, 59 },
    { 0x3a64a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d964a34, 93 },
    { 0x4b164a35, 61 },
    { 0x1e364a36, 61 },
    { 0x24864a36, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a964a3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81064a41, 98 },
    { 0x74864a41, 95 },
    { 0xe5f64a43, 66 },
    { 0x31564a44, 23 },
    { 0xde364a43, 43 },
    { 0x86664a43, 84 },
    { 0x9a164a43, 59 },
    { 0xe164a48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1064a4d, 99 },
    { 0x69564a4e, 90 },
    { 0xaf464a4f, 99 },
    { 0xa5064a50, 95 },
    { 0x12e64a51, 88 },
    { 0x33a64a4f, 33 },
    { 0x1064a53, 93 },
    { 0x7ce64a50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e364a5e, 90 },
    { 0x7ea64a5f, 98 },
    { 0, 0 },
    { 0x9e64a61, 65 },
    { 0xd0c64a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23264a72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x45064a75, 98 },
    { 0x50464a75, 61 },
    { 0, 0 },
    { 0x4ec64a78, 90 },
    { 0xd7864a78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a64a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d64a82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a64a85, 23 },
    { 0xdf064a86, 98 },
    { 0xbaa64a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6064a90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb964a94, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0364a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42364a9f, 33 },
    { 0, 0 },
    { 0xd5264aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e64aa5, 33 },
    { 0, 0 },
    { 0x7f664aa7, 61 },
    { 0, 0 },
    { 0xa8f64aa9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2664aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8464ab3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6364ab6, 81 },
    { 0xd6a64ab7, 77 },
    { 0xe3164ab8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d64abb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59264ac4, 90 },
    { 0x2eb64ac5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78564ac9, 70 },
    { 0, 0 },
    { 0x64acb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db64ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d064ad4, 23 },
    { 0xb1a64ad5, 90 },
    { 0x2bd64ad4, 91 },
    { 0x96764ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264ada, 61 },
    { 0x57964ada, 98 },
    { 0x68664adc, 33 },
    { 0x51464add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e64ae0, 98 },
    { 0x8b864ae1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d64ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1764aed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7864af0, 90 },
    { 0x8a664af1, 90 },
    { 0xabd64af2, 67 },
    { 0, 0 },
    { 0x7564af4, 34 },
    { 0xb2d64af5, 43 },
    { 0x68164af6, 90 },
    { 0x44164af6, 78 },
    { 0x5a164af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c64afb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62464afe, 100 },
    { 0xc8c64aff, 43 },
    { 0x79864b00, 33 },
    { 0, 0 },
    { 0xd6e64b02, 80 },
    { 0, 0 },
    { 0xdc64b04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d64b08, 90 },
    { 0xcb064b08, 91 },
    { 0xdfd64b08, 85 },
    { 0x5b564b0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7964b0f, 76 },
    { 0xea664b10, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47964b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e64b19, 34 },
    { 0xc7264b1a, 98 },
    { 0x9d64b1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7564b20, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd64b25, 61 },
    { 0, 0 },
    { 0x40864b27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b64b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35764b2e, 98 },
    { 0xf1764b2f, 61 },
    { 0x1de64b2f, 33 },
    { 0x5a064b31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16564b35, 98 },
    { 0xa2364b35, 34 },
    { 0, 0 },
    { 0x91664b38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5564b3e, 46 },
    { 0x3d764b3f, 33 },
    { 0xd564b40, 55 },
    { 0xe3a64b3f, 98 },
    { 0xc2064b42, 76 },
    { 0xde264b3e, 69 },
    { 0, 0 },
    { 0x6d364b45, 43 },
    { 0, 0 },
    { 0x4ca64b47, 95 },
    { 0, 0 },
    { 0x14064b49, 76 },
    { 0x94864b49, 61 },
    { 0xb8364b4b, 98 },
    { 0xaad64b4a, 33 },
    { 0, 0 },
    { 0xbf464b4e, 52 },
    { 0x42764b4e, 80 },
    { 0, 0 },
    { 0xf7064b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5064b57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6064b5a, 65 },
    { 0x16864b5b, 100 },
    { 0xaff64b5a, 61 },
    { 0x6c264b5c, 23 },
    { 0x71664b5e, 43 },
    { 0x1c764b5f, 33 },
    { 0, 0 },
    { 0xf2c64b61, 95 },
    { 0xfc864b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93764b67, 36 },
    { 0, 0 },
    { 0xe7764b69, 85 },
    { 0, 0 },
    { 0x32664b6b, 90 },
    { 0, 0 },
    { 0xf6364b6d, 92 },
    { 0x85c64b6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc964b79, 52 },
    { 0x86c64b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064b7f, 33 },
    { 0x80a64b7f, 95 },
    { 0xb1b64b81, 99 },
    { 0x43f64b82, 61 },
    { 0x5a364b83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f64b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad964b8d, 90 },
    { 0xd1164b8e, 76 },
    { 0xd8b64b8e, 81 },
    { 0x95864b90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3464b94, 90 },
    { 0x38964b95, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60264b9a, 98 },
    { 0x36b64b9b, 61 },
    { 0x43964b9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba864b9f, 33 },
    { 0x4d964ba0, 23 },
    { 0xe0e64ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70464ba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef564baa, 33 },
    { 0xc8264baa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc64bae, 81 },
    { 0x2c064baf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2064bb4, 90 },
    { 0x37f64bb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e364bb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35864bbd, 61 },
    { 0x3f664bbe, 33 },
    { 0x3ae64bbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d64bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8264bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19964bce, 43 },
    { 0xc9364bce, 33 },
    { 0xda564bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd64bd5, 90 },
    { 0x86f64bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38364bdb, 36 },
    { 0, 0 },
    { 0x11264bdd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4264bed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c64bf4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e364bfb, 76 },
    { 0, 0 },
    { 0x39964bfd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf464c00, 95 },
    { 0, 0 },
    { 0xd9664c02, 95 },
    { 0x4bc64c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd664c06, 95 },
    { 0xefd64c07, 67 },
    { 0xf0a64c08, 93 },
    { 0xed864c09, 43 },
    { 0x87e64c0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e64c14, 90 },
    { 0, 0 },
    { 0x17964c16, 33 },
    { 0xce064c17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c64c1a, 61 },
    { 0, 0 },
    { 0x5a264c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b64c20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b64c25, 98 },
    { 0xbb664c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86164c29, 33 },
    { 0, 0 },
    { 0x7764c2b, 61 },
    { 0x2c664c2c, 85 },
    { 0x28d64c2c, 98 },
    { 0, 0 },
    { 0xc9564c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe64c34, 33 },
    { 0xa2964c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1364c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61564c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e64c44, 52 },
    { 0x56064c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c064c4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22864c4e, 23 },
    { 0, 0 },
    { 0x4cb64c50, 33 },
    { 0xa9964c50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e364c55, 95 },
    { 0xdaf64c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9764c59, 93 },
    { 0x1b264c59, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e764c5e, 33 },
    { 0xa8464c5e, 98 },
    { 0, 0 },
    { 0xae664c61, 66 },
    { 0x57764c62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63564c68, 61 },
    { 0x99b64c69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a64c70, 92 },
    { 0xb6b64c70, 34 },
    { 0x2fd64c72, 33 },
    { 0xae964c71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd64c7a, 23 },
    { 0x88c64c7a, 98 },
    { 0, 0 },
    { 0xeed64c7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8564c81, 65 },
    { 0, 0 },
    { 0x8a364c83, 87 },
    { 0, 0 },
    { 0x4ca64c85, 33 },
    { 0x5a864c86, 66 },
    { 0x32e64c87, 90 },
    { 0, 0 },
    { 0x9ab64c89, 55 },
    { 0x32364c89, 90 },
    { 0x87d64c8b, 90 },
    { 0x16e64c8c, 61 },
    { 0xee864c8d, 33 },
    { 0xeca64c8e, 93 },
    { 0xd1664c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76064c92, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda64c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62664ca0, 95 },
    { 0x3e464ca1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6164ca4, 66 },
    { 0xa4764ca5, 33 },
    { 0xec764ca5, 23 },
    { 0xc5164ca6, 61 },
    { 0x73664ca4, 90 },
    { 0xbb964ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20664cb0, 95 },
    { 0x1764cb1, 98 },
    { 0x73c64cb2, 95 },
    { 0x2b964cb1, 34 },
    { 0x96e64cb0, 61 },
    { 0xb4564cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67464cb9, 87 },
    { 0, 0 },
    { 0x97264cbb, 61 },
    { 0x77f64cbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb364cbf, 46 },
    { 0xf864cc0, 66 },
    { 0xe0e64cc1, 40 },
    { 0xab664cc2, 90 },
    { 0x32764cc3, 91 },
    { 0x7ba64cc4, 78 },
    { 0, 0 },
    { 0xa2f64cc6, 43 },
    { 0, 0 },
    { 0xb9464cc8, 33 },
    { 0xbaf64cc9, 33 },
    { 0xbfd64cca, 90 },
    { 0xd5f64ccb, 33 },
    { 0x75664ccc, 58 },
    { 0x1c764ccd, 90 },
    { 0, 0 },
    { 0x61964ccf, 61 },
    { 0x99e64cd0, 90 },
    { 0x54264cd1, 80 },
    { 0x10364cd2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28064cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc64cdf, 43 },
    { 0, 0 },
    { 0x1d64ce1, 61 },
    { 0x24064ce1, 93 },
    { 0x98c64ce1, 55 },
    { 0x72b64ce4, 76 },
    { 0, 0 },
    { 0x1f764ce6, 46 },
    { 0, 0 },
    { 0x43b64ce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d664ceb, 95 },
    { 0x8f264ceb, 93 },
    { 0xc2464ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d64cf1, 23 },
    { 0x61d64cf2, 43 },
    { 0x74c64cf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb64cf7, 95 },
    { 0xe0f64cf8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x76164cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4264cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8864d02, 52 },
    { 0xe164d03, 98 },
    { 0, 0 },
    { 0xeae64d05, 52 },
    { 0xb0864d05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8764d0b, 23 },
    { 0xa1464d0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda964d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d764d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f64d18, 97 },
    { 0, 0 },
    { 0x53664d1a, 43 },
    { 0, 0 },
    { 0x4d464d1c, 33 },
    { 0xa9a64d1d, 61 },
    { 0x7cb64d1e, 33 },
    { 0, 0 },
    { 0x42564d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8764d24, 33 },
    { 0, 0 },
    { 0xf3764d26, 99 },
    { 0, 0 },
    { 0x6ed64d28, 61 },
    { 0xcd164d29, 61 },
    { 0x4ac64d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62964d32, 67 },
    { 0x6fa64d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da64d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9864d3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664d40, 98 },
    { 0x50664d40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de64d44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef64d47, 90 },
    { 0, 0 },
    { 0x3c064d49, 87 },
    { 0, 0 },
    { 0x6bc64d4b, 76 },
    { 0xc0464d4c, 40 },
    { 0x75064d4d, 23 },
    { 0x59e64d4e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db64d51, 78 },
    { 0x8b164d52, 61 },
    { 0xfca64d53, 95 },
    { 0xeb764d54, 56 },
    { 0x5b064d54, 98 },
    { 0x62364d53, 42 },
    { 0xba364d51, 33 },
    { 0x23164d57, 23 },
    { 0x36b64d58, 76 },
    { 0xa264d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd64d63, 77 },
    { 0xe7264d63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7564d67, 61 },
    { 0x12064d68, 100 },
    { 0x4e064d69, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xae564d6c, 23 },
    { 0xd5964d6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb64d71, 90 },
    { 0, 0 },
    { 0x50d64d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7064d7b, 90 },
    { 0, 0 },
    { 0xe7f64d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32364d84, 84 },
    { 0, 0 },
    { 0xb9b64d86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d64d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9664d95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7464d9b, 23 },
    { 0x1a964d9b, 85 },
    { 0x9dd64d9d, 88 },
    { 0, 0 },
    { 0x37364d9f, 90 },
    { 0xd8a64da0, 23 },
    { 0x4b964d9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70664da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38464dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5264db3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be64dbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a64dc4, 61 },
    { 0x54664dc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68664dce, 98 },
    { 0x99064dcf, 61 },
    { 0, 0 },
    { 0xf0f64dd1, 91 },
    { 0xc1364dd1, 97 },
    { 0x25b64dd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32964dd6, 43 },
    { 0x75264dd7, 23 },
    { 0x88664dd7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a64de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc64de5, 78 },
    { 0x1a864de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27864de9, 33 },
    { 0, 0 },
    { 0x8c364deb, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xda864dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5064df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4464dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7664dfd, 48 },
    { 0x51b64dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa64e03, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8564e08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f64e0d, 81 },
    { 0xa7d64e0d, 23 },
    { 0x87064e0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c64e12, 33 },
    { 0xcd964e13, 46 },
    { 0x69f64e14, 68 },
    { 0xb9864e14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d564e18, 55 },
    { 0x54a64e19, 23 },
    { 0xd3f64e18, 61 },
    { 0xfec64e18, 61 },
    { 0x77d64e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29964e21, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf64e28, 81 },
    { 0xde564e29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59664e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d364e37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a464e3a, 61 },
    { 0xa5464e3b, 76 },
    { 0xffc64e3c, 85 },
    { 0xd5d64e3d, 93 },
    { 0x4e164e3c, 90 },
    { 0, 0 },
    { 0x7da64e40, 61 },
    { 0xb5564e41, 43 },
    { 0x3cb64e41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f764e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b64e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb64e59, 33 },
    { 0, 0 },
    { 0xaef64e5b, 65 },
    { 0, 0 },
    { 0xb3d64e5d, 23 },
    { 0x21d64e5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77164e61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65464e65, 43 },
    { 0, 0 },
    { 0x65064e67, 40 },
    { 0, 0 },
    { 0x13464e69, 90 },
    { 0xb5c64e69, 55 },
    { 0xbbb64e6b, 76 },
    { 0, 0 },
    { 0xe2a64e6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c164e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50064e76, 75 },
    { 0xa6e64e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9064e7a, 33 },
    { 0xda764e7b, 95 },
    { 0xae864e7b, 90 },
    { 0x8a064e7d, 98 },
    { 0x6ec64e7e, 81 },
    { 0x76e64e7f, 76 },
    { 0x6ee64e7f, 66 },
    { 0x2c364e7e, 61 },
    { 0x17264e82, 55 },
    { 0x4bb64e83, 93 },
    { 0xf8364e82, 98 },
    { 0, 0 },
    { 0xaba64e86, 66 },
    { 0, 0 },
    { 0x7a564e88, 98 },
    { 0, 0 },
    { 0xa1f64e8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc64e97, 61 },
    { 0, 0 },
    { 0x73264e99, 90 },
    { 0x15164e9a, 61 },
    { 0x1864e9a, 33 },
    { 0xf5464e99, 61 },
    { 0x5bb64e9a, 90 },
    { 0xe9964e9e, 95 },
    { 0xc0664e9b, 95 },
    { 0x1d64ea0, 43 },
    { 0, 0 },
    { 0x42c64ea2, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66864eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23864ead, 85 },
    { 0xc3d64eae, 61 },
    { 0xc0364eaf, 95 },
    { 0, 0 },
    { 0xc0764eb1, 95 },
    { 0x51f64eb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d64eb7, 99 },
    { 0x40164eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b964ebb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28564ebf, 90 },
    { 0xa3c64ec0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f64ec5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86564ec8, 43 },
    { 0, 0 },
    { 0xac564eca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c64ecd, 90 },
    { 0xa1164ecd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8364ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac64ed6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1464edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d64edf, 61 },
    { 0x8af64ee0, 91 },
    { 0x39d64ee1, 76 },
    { 0x10864ee2, 33 },
    { 0x46064ee3, 33 },
    { 0xae664ee3, 23 },
    { 0xb9764ee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc364ee8, 67 },
    { 0, 0 },
    { 0x6964eea, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a64eee, 61 },
    { 0, 0 },
    { 0xca564ef0, 85 },
    { 0x59264ef1, 55 },
    { 0xb8464ef2, 90 },
    { 0x6b564ef2, 33 },
    { 0x17164ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77164ef7, 59 },
    { 0x1b64ef7, 90 },
    { 0x59364ef9, 78 },
    { 0x7ad64efa, 23 },
    { 0xd0364efb, 55 },
    { 0xa564efc, 23 },
    { 0x48664efd, 77 },
    { 0x33164efc, 98 },
    { 0x25264efc, 99 },
    { 0x48664efc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde864f04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65664f0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e464f14, 98 },
    { 0, 0 },
    { 0xb5f64f16, 78 },
    { 0x99264f17, 91 },
    { 0x71c64f18, 90 },
    { 0, 0 },
    { 0x66564f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464f1f, 76 },
    { 0, 0 },
    { 0x99d64f21, 81 },
    { 0xf6664f21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8664f25, 76 },
    { 0, 0 },
    { 0xf5d64f27, 55 },
    { 0x1d464f28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f964f2b, 66 },
    { 0xb0f64f2c, 78 },
    { 0, 0 },
    { 0x26a64f2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2764f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c64f38, 98 },
    { 0x8964f38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7164f3c, 46 },
    { 0x75b64f3c, 33 },
    { 0, 0 },
    { 0xe4464f3f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d564f4c, 95 },
    { 0xac164f4c, 76 },
    { 0x53464f4d, 90 },
    { 0x30d64f4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43964f53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9164f57, 87 },
    { 0xeb664f57, 61 },
    { 0x39164f59, 33 },
    { 0x13e64f5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b64f60, 95 },
    { 0x55764f61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40964f64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48864f6b, 90 },
    { 0, 0 },
    { 0x24e64f6d, 43 },
    { 0xdb64f6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56564f78, 67 },
    { 0xcd064f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c064f7c, 61 },
    { 0xffc64f7d, 87 },
    { 0, 0 },
    { 0xb9064f7f, 84 },
    { 0xa1564f80, 99 },
    { 0xd6e64f81, 98 },
    { 0, 0 },
    { 0xd7464f83, 84 },
    { 0xa9364f84, 69 },
    { 0xe1964f85, 98 },
    { 0x1e164f86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e664f8a, 76 },
    { 0xec964f8b, 87 },
    { 0, 0 },
    { 0xf8664f8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2464f95, 95 },
    { 0xd3364f95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f364f99, 34 },
    { 0x18064f99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73764f9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5164fb8, 69 },
    { 0xc5764fb9, 78 },
    { 0x5164fb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70164fc0, 23 },
    { 0x33c64fc1, 76 },
    { 0, 0 },
    { 0x7a964fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a664fc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb64fcc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb464fcf, 36 },
    { 0x95164fd0, 95 },
    { 0x5d564fd1, 91 },
    { 0x34064fd2, 34 },
    { 0x1ba64fd3, 61 },
    { 0x2f64fd4, 98 },
    { 0xe1e64fd5, 52 },
    { 0xc664fd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf64fd9, 88 },
    { 0x69964fda, 55 },
    { 0x5d164fdb, 93 },
    { 0x25c64fdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee764fea, 90 },
    { 0, 0 },
    { 0xd1764fec, 61 },
    { 0xbd864fed, 43 },
    { 0x19764fee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a64ff4, 77 },
    { 0x7cc64ff5, 90 },
    { 0, 0 },
    { 0x70264ff7, 33 },
    { 0x1bc64ff8, 95 },
    { 0x3c964ff9, 46 },
    { 0xe9364ff7, 99 },
    { 0, 0 },
    { 0xaf164ffc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6565001, 23 },
    { 0, 0 },
    { 0xde965003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa65006, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9165009, 55 },
    { 0, 0 },
    { 0x4bf6500b, 93 },
    { 0, 0 },
    { 0x3696500d, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc265014, 98 },
    { 0xf3465015, 23 },
    { 0, 0 },
    { 0x5e265017, 23 },
    { 0, 0 },
    { 0x1e265019, 43 },
    { 0, 0 },
    { 0xbaa6501b, 87 },
    { 0x3406501c, 90 },
    { 0xef66501d, 78 },
    { 0xcd36501d, 33 },
    { 0x1d46501b, 33 },
    { 0x9b465020, 93 },
    { 0x7e165021, 23 },
    { 0x8c66501b, 33 },
    { 0, 0 },
    { 0x4f165024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4065028, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe86502b, 75 },
    { 0xfe76502c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd556502f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a65033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79765039, 34 },
    { 0xe0065039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23665040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe65043, 98 },
    { 0, 0 },
    { 0x2ff65045, 43 },
    { 0x10865046, 90 },
    { 0, 0 },
    { 0x54265048, 43 },
    { 0xc765049, 98 },
    { 0, 0 },
    { 0xd126504b, 61 },
    { 0xb486504c, 33 },
    { 0x5616504b, 33 },
    { 0xf806504e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc65051, 76 },
    { 0x52365051, 61 },
    { 0, 0 },
    { 0x3c265054, 33 },
    { 0, 0 },
    { 0xf9865056, 90 },
    { 0x5d965057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6505b, 61 },
    { 0, 0 },
    { 0x7cc6505d, 43 },
    { 0, 0 },
    { 0x8af6505f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f165063, 33 },
    { 0x64965064, 77 },
    { 0x7c65063, 95 },
    { 0x69565066, 43 },
    { 0x2ea65065, 76 },
    { 0, 0 },
    { 0x36c65069, 98 },
    { 0x89065069, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff46506f, 100 },
    { 0x8cc65070, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28765076, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee6507b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2346507e, 34 },
    { 0xbb26507f, 61 },
    { 0x6a665080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17165083, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75565088, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef46508c, 55 },
    { 0, 0 },
    { 0x8556508e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa465098, 98 },
    { 0xaae65099, 33 },
    { 0, 0 },
    { 0x2106509b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956509e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e650a3, 34 },
    { 0xfbc650a4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e650a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae650ab, 95 },
    { 0xa09650ac, 43 },
    { 0x793650ac, 23 },
    { 0x1cc650ac, 23 },
    { 0, 0 },
    { 0xe11650b0, 23 },
    { 0xb7b650b1, 98 },
    { 0x476650b2, 90 },
    { 0x159650b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0650b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39650bb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb650be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf650c9, 23 },
    { 0x5a6650ca, 81 },
    { 0x877650ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9650cf, 98 },
    { 0xf29650d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d650d4, 98 },
    { 0x19e650d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8650da, 33 },
    { 0, 0 },
    { 0x921650dc, 90 },
    { 0x28f650dd, 46 },
    { 0, 0 },
    { 0x97b650df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f650e4, 61 },
    { 0, 0 },
    { 0x414650e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96650ec, 33 },
    { 0x27f650ed, 90 },
    { 0x459650ee, 34 },
    { 0xccd650ed, 23 },
    { 0xbba650ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe650f5, 90 },
    { 0x26b650f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x834650f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b650fd, 76 },
    { 0xb4e650fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a465105, 33 },
    { 0, 0 },
    { 0xa5f65107, 61 },
    { 0x1ec65108, 33 },
    { 0x7d165109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6510c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d6510f, 95 },
    { 0xcc16510f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db6511c, 98 },
    { 0x1ee6511d, 55 },
    { 0, 0 },
    { 0x5516511f, 76 },
    { 0x1ab65120, 23 },
    { 0xf5865121, 24 },
    { 0, 0 },
    { 0x7f165123, 98 },
    { 0xc6365124, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc65129, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9086512c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x46512f, 68 },
    { 0x5c06512f, 90 },
    { 0, 0 },
    { 0xbd665132, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69765136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076513e, 55 },
    { 0x9016513f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c65144, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2465148, 61 },
    { 0x5d865149, 90 },
    { 0xe56514a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6514f, 46 },
    { 0, 0 },
    { 0x20665151, 98 },
    { 0xa9865152, 98 },
    { 0x65965152, 33 },
    { 0x3e965154, 55 },
    { 0xbed65155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946515a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa36515d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a65162, 53 },
    { 0xfae65163, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a65169, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6516e, 76 },
    { 0xa106516f, 66 },
    { 0xe1765170, 55 },
    { 0x88665171, 23 },
    { 0x2065171, 34 },
    { 0x7e765172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad065177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056517a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9696517e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f65181, 95 },
    { 0xdd265182, 33 },
    { 0x13165181, 93 },
    { 0, 0 },
    { 0xa0165185, 61 },
    { 0, 0 },
    { 0xfc065187, 98 },
    { 0, 0 },
    { 0x56165189, 89 },
    { 0x8ec65189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde06518f, 90 },
    { 0, 0 },
    { 0xf1f65191, 66 },
    { 0xa2265192, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74065195, 52 },
    { 0x8165195, 61 },
    { 0x86c65197, 23 },
    { 0xc5e65196, 77 },
    { 0, 0 },
    { 0xdc76519a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5266519d, 23 },
    { 0xedb6519d, 93 },
    { 0, 0 },
    { 0xe43651a0, 98 },
    { 0x147651a1, 95 },
    { 0x50a651a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6651aa, 91 },
    { 0xf05651ab, 78 },
    { 0x39f651ab, 95 },
    { 0xcad651ab, 85 },
    { 0x391651ae, 61 },
    { 0xa9651af, 22 },
    { 0xdf1651ae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d651b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462651bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f651c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61651c5, 90 },
    { 0xd88651c5, 78 },
    { 0xf81651c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd651cd, 95 },
    { 0, 0 },
    { 0x57651cf, 66 },
    { 0xb6f651d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a651d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7651d9, 99 },
    { 0xc0b651da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69651e6, 55 },
    { 0x779651e7, 85 },
    { 0x31d651e8, 90 },
    { 0x475651e9, 95 },
    { 0x313651ea, 39 },
    { 0xdf6651eb, 69 },
    { 0x66a651ec, 90 },
    { 0x902651e9, 91 },
    { 0xe3b651e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5651f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95651f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd651fe, 90 },
    { 0x48c651fe, 23 },
    { 0x516651ff, 99 },
    { 0, 0 },
    { 0xb8865202, 76 },
    { 0x10665203, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce65206, 98 },
    { 0xeaa65207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece6520a, 90 },
    { 0xb706520b, 95 },
    { 0x4bb6520c, 55 },
    { 0xcdb6520d, 91 },
    { 0x4a76520a, 23 },
    { 0x1b36520e, 90 },
    { 0x1796520b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165214, 100 },
    { 0x92065215, 95 },
    { 0x53565214, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965222, 98 },
    { 0x32065223, 77 },
    { 0xa2f65224, 95 },
    { 0, 0 },
    { 0x45b65226, 90 },
    { 0xcea65226, 90 },
    { 0xbf465228, 90 },
    { 0x49265229, 95 },
    { 0, 0 },
    { 0xf1a6522b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80865230, 67 },
    { 0x9bc65231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de6523b, 55 },
    { 0x6d56523c, 90 },
    { 0xc726523b, 98 },
    { 0x7896523e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8565242, 33 },
    { 0xe4265242, 97 },
    { 0x48965242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a265257, 95 },
    { 0xfa765258, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5146525b, 66 },
    { 0, 0 },
    { 0xd596525d, 33 },
    { 0, 0 },
    { 0xf346525f, 33 },
    { 0xa1365260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20865265, 43 },
    { 0, 0 },
    { 0xf2565267, 40 },
    { 0xd0665267, 33 },
    { 0x46365269, 61 },
    { 0, 0 },
    { 0xee16526b, 90 },
    { 0, 0 },
    { 0x2796526d, 91 },
    { 0xfd16526e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae65275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2016527e, 95 },
    { 0x5f16527f, 34 },
    { 0x7cf65280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a65285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88665289, 61 },
    { 0, 0 },
    { 0xbe26528b, 98 },
    { 0, 0 },
    { 0xf86528d, 52 },
    { 0x9cb6528e, 90 },
    { 0xf276528e, 66 },
    { 0, 0 },
    { 0x45e65291, 34 },
    { 0, 0 },
    { 0xe2c65293, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc565296, 23 },
    { 0x9ee65297, 98 },
    { 0x16e65297, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf6529c, 91 },
    { 0, 0 },
    { 0xf886529e, 88 },
    { 0x9116529f, 52 },
    { 0xe68652a0, 95 },
    { 0, 0 },
    { 0xd56652a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3652a7, 34 },
    { 0x9d1652a8, 34 },
    { 0xa39652a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38652ad, 87 },
    { 0x103652ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2652b1, 55 },
    { 0xfa1652b2, 65 },
    { 0, 0 },
    { 0xa50652b4, 43 },
    { 0, 0 },
    { 0xca2652b6, 95 },
    { 0x792652b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b652bc, 33 },
    { 0x83c652bd, 61 },
    { 0x4bc652be, 61 },
    { 0x5db652bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1652c8, 33 },
    { 0, 0 },
    { 0xcc4652ca, 95 },
    { 0x7c652ca, 55 },
    { 0x853652ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b652d6, 75 },
    { 0x5a9652d7, 90 },
    { 0, 0 },
    { 0xcd3652d9, 34 },
    { 0x2c3652d9, 33 },
    { 0x2e652da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519652e4, 98 },
    { 0, 0 },
    { 0xd7c652e6, 95 },
    { 0xde9652e6, 74 },
    { 0x2d9652e7, 99 },
    { 0x275652e6, 46 },
    { 0, 0 },
    { 0xcd4652eb, 46 },
    { 0, 0 },
    { 0x802652ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89652f0, 90 },
    { 0x408652f1, 43 },
    { 0x22652f2, 76 },
    { 0x775652f3, 33 },
    { 0x3c652f4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d652f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8652fb, 90 },
    { 0x982652fc, 23 },
    { 0x126652fd, 61 },
    { 0xaf7652fd, 61 },
    { 0, 0 },
    { 0x46b65300, 24 },
    { 0xbfd65301, 90 },
    { 0xfb565302, 98 },
    { 0x1b665300, 33 },
    { 0, 0 },
    { 0x7665305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3286530a, 78 },
    { 0xc6f6530a, 64 },
    { 0x5906530c, 61 },
    { 0, 0 },
    { 0x2ea6530e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f65311, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce65314, 81 },
    { 0x6be65314, 33 },
    { 0x65f65316, 23 },
    { 0xbe65317, 56 },
    { 0x17665314, 95 },
    { 0x49165318, 99 },
    { 0xe186531a, 61 },
    { 0, 0 },
    { 0x57a6531c, 90 },
    { 0x1e76531d, 23 },
    { 0x6496531e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe065322, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a65326, 90 },
    { 0x53f65326, 33 },
    { 0xfd465328, 34 },
    { 0x30565329, 61 },
    { 0x9be65328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1246532d, 95 },
    { 0xfaf6532e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab65332, 36 },
    { 0x7ec65333, 98 },
    { 0, 0 },
    { 0x6e665335, 61 },
    { 0, 0 },
    { 0xa2065337, 43 },
    { 0, 0 },
    { 0x3ea65339, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b16533e, 75 },
    { 0x2f66533f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78465344, 43 },
    { 0xe6b65344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e665349, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626534e, 61 },
    { 0x41a6534e, 55 },
    { 0, 0 },
    { 0x14965351, 93 },
    { 0xac565351, 90 },
    { 0, 0 },
    { 0x1d965354, 93 },
    { 0xb9565355, 23 },
    { 0xbd565356, 46 },
    { 0x4e965356, 33 },
    { 0x86165356, 90 },
    { 0xd465357, 99 },
    { 0xaa6535a, 23 },
    { 0xa4e65356, 75 },
    { 0, 0 },
    { 0xdd16535d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d965361, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75465368, 93 },
    { 0, 0 },
    { 0x3186536a, 23 },
    { 0, 0 },
    { 0x4e86536c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c765372, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e65375, 78 },
    { 0, 0 },
    { 0x54c65377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae6537b, 98 },
    { 0xaff6537b, 98 },
    { 0x8eb6537d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2365386, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd376538f, 93 },
    { 0xb0e65390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a065397, 97 },
    { 0x4ca65398, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d6539d, 95 },
    { 0x6636539e, 33 },
    { 0, 0 },
    { 0xc55653a0, 100 },
    { 0x1653a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x484653a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b653a8, 75 },
    { 0xf4b653a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b653af, 33 },
    { 0x5ff653b0, 33 },
    { 0x122653af, 98 },
    { 0x142653af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef653c1, 55 },
    { 0x936653c1, 61 },
    { 0, 0 },
    { 0x56a653c4, 66 },
    { 0x72653c5, 99 },
    { 0x184653c6, 55 },
    { 0, 0 },
    { 0x1ce653c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171653cd, 33 },
    { 0x507653ce, 61 },
    { 0x86b653cf, 90 },
    { 0xf2e653d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08653d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501653d8, 55 },
    { 0xde4653d8, 33 },
    { 0, 0 },
    { 0x45d653db, 34 },
    { 0x8bd653dc, 75 },
    { 0xa7d653db, 95 },
    { 0x16d653de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d653e1, 61 },
    { 0x92c653e1, 43 },
    { 0xcc0653e3, 55 },
    { 0x588653e3, 100 },
    { 0x6a9653e4, 65 },
    { 0x954653e6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba653ef, 20 },
    { 0x88d653f0, 93 },
    { 0x396653f1, 88 },
    { 0, 0 },
    { 0x4b8653f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce653f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb653fb, 23 },
    { 0x5c3653fc, 90 },
    { 0, 0 },
    { 0x1a3653fe, 55 },
    { 0xa73653ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d65405, 46 },
    { 0x74b65406, 61 },
    { 0xa8165407, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e6540b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d65411, 95 },
    { 0x19065411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c65416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68165419, 65 },
    { 0x7026541a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86541d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42665423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b16542a, 53 },
    { 0x9386542b, 90 },
    { 0x9c26542a, 43 },
    { 0, 0 },
    { 0x14f6542e, 61 },
    { 0, 0 },
    { 0x4ad65430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1465438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c6543c, 95 },
    { 0x4646543c, 91 },
    { 0xb116543e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6544a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886544d, 33 },
    { 0x30c6544d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b665451, 33 },
    { 0, 0 },
    { 0x50a65453, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f65458, 91 },
    { 0x64665459, 66 },
    { 0x4e65459, 90 },
    { 0x5cd65458, 43 },
    { 0xc5a65459, 33 },
    { 0, 0 },
    { 0xf876545e, 76 },
    { 0x2b16545f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd65466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc765469, 88 },
    { 0x22b65469, 55 },
    { 0xa5b6546b, 40 },
    { 0, 0 },
    { 0x2616546d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b565471, 33 },
    { 0x30265472, 99 },
    { 0x4fa65473, 55 },
    { 0x5c365474, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac65478, 95 },
    { 0, 0 },
    { 0xd376547a, 98 },
    { 0xf466547b, 98 },
    { 0xf4f6547c, 33 },
    { 0x61f6547b, 23 },
    { 0x4ca6547d, 98 },
    { 0x5d56547f, 40 },
    { 0x4ca6547f, 61 },
    { 0xe7265480, 93 },
    { 0x2d965482, 88 },
    { 0x22b65483, 52 },
    { 0x45065484, 90 },
    { 0x3cf65485, 61 },
    { 0x2dd6547d, 33 },
    { 0x27665487, 76 },
    { 0x77f6547c, 75 },
    { 0x8aa65485, 90 },
    { 0xf466548a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd476548d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e765497, 98 },
    { 0x7b665498, 95 },
    { 0, 0 },
    { 0x4696549a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86549f, 95 },
    { 0x2ea654a0, 99 },
    { 0, 0 },
    { 0xcf2654a2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b654ad, 90 },
    { 0xe9d654ae, 66 },
    { 0x6c3654ae, 99 },
    { 0, 0 },
    { 0x77654b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7654b4, 61 },
    { 0x96654b4, 90 },
    { 0, 0 },
    { 0x572654b7, 67 },
    { 0xc35654b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb654bd, 80 },
    { 0, 0 },
    { 0xe80654bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e654c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677654ca, 61 },
    { 0, 0 },
    { 0x671654cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c654d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db654d8, 23 },
    { 0x83f654d8, 61 },
    { 0xb8b654da, 33 },
    { 0, 0 },
    { 0x9fe654dc, 98 },
    { 0x4a5654dc, 76 },
    { 0x9df654de, 85 },
    { 0, 0 },
    { 0x60d654e0, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882654ec, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd654f2, 91 },
    { 0x237654f3, 90 },
    { 0, 0 },
    { 0x8e0654f5, 96 },
    { 0xc47654f5, 95 },
    { 0x9aa654f5, 76 },
    { 0xd87654f8, 23 },
    { 0x41654f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f654fe, 90 },
    { 0, 0 },
    { 0x7f465500, 91 },
    { 0x28865500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8665504, 99 },
    { 0xcce65505, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x51065508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106550d, 95 },
    { 0xe416550e, 23 },
    { 0xe3f6550d, 33 },
    { 0x3456550f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65665513, 33 },
    { 0, 0 },
    { 0x11365515, 93 },
    { 0, 0 },
    { 0x8565517, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c6551a, 23 },
    { 0xcea6551b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8706551f, 99 },
    { 0x15965520, 82 },
    { 0xe1c65521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f865525, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76552b, 83 },
    { 0x67c6552b, 68 },
    { 0xc2c6552b, 33 },
    { 0x3a16552b, 33 },
    { 0x9026552f, 55 },
    { 0x9a165530, 33 },
    { 0xab765531, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87065534, 34 },
    { 0x55765535, 90 },
    { 0, 0 },
    { 0xb5265537, 61 },
    { 0xe0465538, 90 },
    { 0x7fd65537, 98 },
    { 0, 0 },
    { 0x5746553b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c46553f, 61 },
    { 0x8bb65540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde65548, 90 },
    { 0x59265549, 27 },
    { 0, 0 },
    { 0xf106554b, 33 },
    { 0x15a6554b, 74 },
    { 0x8c96554d, 33 },
    { 0, 0 },
    { 0x90f6554f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35765554, 91 },
    { 0x65865555, 77 },
    { 0x40d65555, 78 },
    { 0, 0 },
    { 0x4665558, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b6555e, 98 },
    { 0, 0 },
    { 0xd0a65560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf6556b, 90 },
    { 0xdcd6556c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc65571, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b65574, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b65577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec76557d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265581, 90 },
    { 0xab465582, 52 },
    { 0x52665583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad65586, 76 },
    { 0x9f165587, 76 },
    { 0x75b65588, 95 },
    { 0x7e965588, 61 },
    { 0xb6d6558a, 36 },
    { 0xe326558b, 98 },
    { 0x43365589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90065590, 98 },
    { 0, 0 },
    { 0x2165592, 17 },
    { 0x73c65593, 61 },
    { 0xd8365593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356559f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0655a4, 66 },
    { 0x3cf655a5, 33 },
    { 0x6de655a6, 81 },
    { 0, 0 },
    { 0xe23655a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c655ad, 46 },
    { 0, 0 },
    { 0x319655af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a655bc, 77 },
    { 0x91c655bd, 55 },
    { 0x178655be, 95 },
    { 0x7ce655be, 70 },
    { 0xf15655bf, 90 },
    { 0xe52655c1, 23 },
    { 0, 0 },
    { 0x6d4655c3, 98 },
    { 0, 0 },
    { 0x44c655c5, 96 },
    { 0xeeb655c5, 81 },
    { 0, 0 },
    { 0xb7f655c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e655cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93655ce, 95 },
    { 0x6c1655cf, 87 },
    { 0, 0 },
    { 0xb33655d1, 55 },
    { 0x290655d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34655d8, 23 },
    { 0xe71655d9, 90 },
    { 0x655da, 61 },
    { 0x6e5655db, 66 },
    { 0, 0 },
    { 0xd2e655dd, 33 },
    { 0x8ce655de, 78 },
    { 0x665655de, 98 },
    { 0x17a655df, 98 },
    { 0, 0 },
    { 0x4f4655e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e655e6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362655ea, 23 },
    { 0x132655eb, 65 },
    { 0, 0 },
    { 0x741655ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb655f1, 61 },
    { 0xb2e655f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f655f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac655fc, 90 },
    { 0xf0a655fd, 95 },
    { 0x4e8655fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd565601, 76 },
    { 0x32d65602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5256560b, 90 },
    { 0xd876560c, 66 },
    { 0xbfb6560d, 67 },
    { 0xdbc6560e, 98 },
    { 0, 0 },
    { 0x59965610, 33 },
    { 0xf7965611, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d65614, 76 },
    { 0xc7765615, 23 },
    { 0, 0 },
    { 0x2ce65617, 90 },
    { 0xe5e65617, 17 },
    { 0x49c65618, 96 },
    { 0xfe06561a, 68 },
    { 0x67465617, 76 },
    { 0x4a665618, 95 },
    { 0x5346561d, 91 },
    { 0xeec6561d, 90 },
    { 0x3106561f, 95 },
    { 0xc0d6561b, 78 },
    { 0xf3e65621, 46 },
    { 0x21e65621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65629, 99 },
    { 0, 0 },
    { 0xe316562b, 44 },
    { 0x36c6562b, 61 },
    { 0, 0 },
    { 0xffd6562e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf65635, 98 },
    { 0x4b865636, 90 },
    { 0, 0 },
    { 0xa7365638, 90 },
    { 0, 0 },
    { 0x7516563a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c6563d, 81 },
    { 0, 0 },
    { 0x4ba6563f, 33 },
    { 0, 0 },
    { 0x7a265641, 34 },
    { 0, 0 },
    { 0x88665643, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96564c, 88 },
    { 0, 0 },
    { 0x7fd6564e, 90 },
    { 0x7466564f, 78 },
    { 0x5706564e, 95 },
    { 0, 0 },
    { 0x30265652, 66 },
    { 0x7365653, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c6565d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x49665660, 46 },
    { 0xbe965661, 90 },
    { 0x47c65660, 43 },
    { 0xb4b65662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b765666, 34 },
    { 0x8a965666, 81 },
    { 0xafa65668, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd046566e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b65674, 98 },
    { 0, 0 },
    { 0x62865676, 95 },
    { 0, 0 },
    { 0x34265678, 33 },
    { 0, 0 },
    { 0xb6567a, 61 },
    { 0xc556567b, 93 },
    { 0x7956567c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b96567f, 61 },
    { 0x3f86567f, 46 },
    { 0x35665680, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd326568a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2086568d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2965691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e265695, 61 },
    { 0, 0 },
    { 0xbcd65697, 76 },
    { 0x73665697, 95 },
    { 0x4c065698, 33 },
    { 0x6dc65699, 55 },
    { 0x6ee65698, 33 },
    { 0, 0 },
    { 0x9886569d, 33 },
    { 0x3076569e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401656a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x78656a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94656ac, 87 },
    { 0x8dc656ad, 34 },
    { 0xcae656ad, 61 },
    { 0xa47656ad, 34 },
    { 0, 0 },
    { 0x110656b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68656b6, 90 },
    { 0, 0 },
    { 0xf1a656b8, 66 },
    { 0xb59656b8, 67 },
    { 0, 0 },
    { 0x7e1656bb, 33 },
    { 0x96e656bc, 33 },
    { 0, 0 },
    { 0x422656be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a656c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b656cd, 84 },
    { 0x500656cd, 23 },
    { 0x3f8656cf, 34 },
    { 0xd87656cf, 95 },
    { 0, 0 },
    { 0x4bf656d2, 87 },
    { 0x373656d3, 93 },
    { 0, 0 },
    { 0xe5656d5, 90 },
    { 0x9d0656d6, 33 },
    { 0, 0 },
    { 0xa3f656d8, 12 },
    { 0, 0 },
    { 0x641656da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb656dd, 34 },
    { 0, 0 },
    { 0x1c656df, 67 },
    { 0x4ff656e0, 65 },
    { 0x150656e0, 90 },
    { 0x27e656e0, 34 },
    { 0xf24656e2, 90 },
    { 0xd21656e4, 88 },
    { 0x288656e3, 76 },
    { 0x682656e6, 61 },
    { 0x598656e5, 23 },
    { 0x321656e8, 77 },
    { 0, 0 },
    { 0x3c656ea, 61 },
    { 0x409656eb, 76 },
    { 0x516656eb, 78 },
    { 0x993656ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28656f0, 88 },
    { 0, 0 },
    { 0x726656f2, 85 },
    { 0x737656f2, 80 },
    { 0x862656f4, 85 },
    { 0xad6656f5, 34 },
    { 0x799656f4, 98 },
    { 0xb3e656f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57656fd, 67 },
    { 0xf70656fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed65705, 23 },
    { 0x84065706, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265709, 46 },
    { 0xc8e6570a, 33 },
    { 0xfe76570b, 95 },
    { 0xe926570b, 26 },
    { 0x81d6570d, 95 },
    { 0x4146570e, 23 },
    { 0xa8765709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18765714, 61 },
    { 0x6ce65715, 78 },
    { 0, 0 },
    { 0x9fc65717, 95 },
    { 0xb3965718, 33 },
    { 0, 0 },
    { 0x4b16571a, 90 },
    { 0xb8e6571a, 85 },
    { 0x49c6571a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce65720, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9065724, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1765729, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6572f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c765734, 90 },
    { 0xf7e65734, 99 },
    { 0xd3465735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99765739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab65742, 17 },
    { 0, 0 },
    { 0xbb265744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51065748, 99 },
    { 0x66c65749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6574c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65750, 90 },
    { 0, 0 },
    { 0x3e165752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54565755, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd526575c, 33 },
    { 0, 0 },
    { 0x97e6575e, 68 },
    { 0xe696575e, 98 },
    { 0, 0 },
    { 0xf3465761, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4065765, 33 },
    { 0x4af65766, 76 },
    { 0x7165765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36576b, 98 },
    { 0, 0 },
    { 0xa1b6576d, 90 },
    { 0x8f26576e, 90 },
    { 0x7416576f, 95 },
    { 0xddd65770, 87 },
    { 0x1516576f, 95 },
    { 0xf2f65772, 61 },
    { 0x1b86576d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b65779, 33 },
    { 0x1486577a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6577d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55165780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2365784, 43 },
    { 0x10d65785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6578d, 91 },
    { 0, 0 },
    { 0xb436578f, 95 },
    { 0x1d65790, 98 },
    { 0, 0 },
    { 0x34a65792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d65797, 78 },
    { 0xa9a65798, 33 },
    { 0x95265798, 56 },
    { 0x20d65799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e6579e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc657a3, 90 },
    { 0, 0 },
    { 0xeae657a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e657af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa657b2, 95 },
    { 0x9e0657b3, 95 },
    { 0x930657b3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186657b8, 95 },
    { 0x897657b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf657c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30657c8, 87 },
    { 0, 0 },
    { 0x7657ca, 95 },
    { 0, 0 },
    { 0x7bf657cc, 43 },
    { 0, 0 },
    { 0x8d7657ce, 98 },
    { 0, 0 },
    { 0xaa0657d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x731657d3, 52 },
    { 0x4d0657d3, 43 },
    { 0x8e8657d5, 61 },
    { 0xdea657d4, 85 },
    { 0, 0 },
    { 0xd26657d8, 33 },
    { 0x31f657d8, 43 },
    { 0xd3d657d9, 100 },
    { 0xf65657d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3657e0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2657e6, 99 },
    { 0x1c3657e6, 34 },
    { 0x48c657e8, 33 },
    { 0xc4f657e9, 80 },
    { 0x5a8657e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34657f0, 33 },
    { 0, 0 },
    { 0xf0b657f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc657f5, 87 },
    { 0xf03657f6, 90 },
    { 0xfd657f7, 100 },
    { 0, 0 },
    { 0xf60657f9, 55 },
    { 0xf1e657f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71765804, 76 },
    { 0xb9065804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1865809, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8416580d, 61 },
    { 0x6c76580e, 61 },
    { 0x22a6580f, 61 },
    { 0xf3a65810, 91 },
    { 0xbdf6580e, 61 },
    { 0, 0 },
    { 0xa5065813, 61 },
    { 0xce265814, 76 },
    { 0x4c565815, 98 },
    { 0xd1365815, 23 },
    { 0xa7a65817, 23 },
    { 0, 0 },
    { 0xde565819, 93 },
    { 0, 0 },
    { 0x4916581b, 93 },
    { 0x4b66581c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156581f, 46 },
    { 0x52065820, 33 },
    { 0xf0465820, 95 },
    { 0xa336581f, 95 },
    { 0xf9c65823, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f565826, 88 },
    { 0x61765827, 87 },
    { 0x18e65828, 67 },
    { 0, 0 },
    { 0xb816582a, 85 },
    { 0x8406582a, 66 },
    { 0xb556582c, 90 },
    { 0xdfe6582c, 81 },
    { 0xda96582c, 90 },
    { 0xa166582f, 75 },
    { 0x4486582b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65836, 34 },
    { 0x76565836, 95 },
    { 0xbb865836, 23 },
    { 0xddc65836, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6583f, 33 },
    { 0xa965840, 33 },
    { 0xa9765841, 61 },
    { 0xb565842, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c65845, 76 },
    { 0x7d65846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66584d, 61 },
    { 0x7336584e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d65851, 23 },
    { 0xad65852, 95 },
    { 0x4c165853, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f26585a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb66585f, 76 },
    { 0x98d65860, 88 },
    { 0xc1965860, 93 },
    { 0x75465862, 98 },
    { 0, 0 },
    { 0xc0465864, 93 },
    { 0, 0 },
    { 0xd7e65866, 76 },
    { 0x47865866, 90 },
    { 0, 0 },
    { 0x9ac65869, 90 },
    { 0x88b6586a, 23 },
    { 0, 0 },
    { 0xad66586c, 85 },
    { 0xadc6586d, 90 },
    { 0xb3f6586d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93265871, 98 },
    { 0x4c565872, 84 },
    { 0xae865873, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f6587a, 84 },
    { 0, 0 },
    { 0xc466587c, 98 },
    { 0x9f26587d, 76 },
    { 0x7b76587e, 46 },
    { 0xcf76587c, 52 },
    { 0xfd265880, 34 },
    { 0x70165880, 33 },
    { 0x15b65882, 84 },
    { 0x30265882, 95 },
    { 0x5f065883, 34 },
    { 0xe3665885, 90 },
    { 0x8a665882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c65889, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336588f, 90 },
    { 0x54c65890, 88 },
    { 0xea565891, 33 },
    { 0x3af65892, 52 },
    { 0x1b165893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3165897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666589e, 40 },
    { 0xfa06589e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39658a6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a658ae, 98 },
    { 0xb6e658ae, 43 },
    { 0x717658b0, 61 },
    { 0x3dd658b0, 23 },
    { 0xf4d658b2, 84 },
    { 0xb45658b3, 34 },
    { 0xd7e658b4, 23 },
    { 0xc57658b5, 12 },
    { 0x2a7658b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7658b9, 98 },
    { 0x28f658ba, 64 },
    { 0x479658bb, 33 },
    { 0x60b658ba, 61 },
    { 0xcbf658bd, 55 },
    { 0, 0 },
    { 0x6ed658bf, 90 },
    { 0, 0 },
    { 0xbc1658c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9658c5, 95 },
    { 0, 0 },
    { 0x84d658c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5658ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc658d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58658d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4658d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8658dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13658e0, 90 },
    { 0, 0 },
    { 0x8f9658e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787658e9, 90 },
    { 0x113658e9, 67 },
    { 0x8cf658e9, 95 },
    { 0xdc0658ec, 34 },
    { 0xf1b658eb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08658f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f658f9, 67 },
    { 0x327658fa, 90 },
    { 0x66d658fb, 90 },
    { 0x918658fb, 61 },
    { 0x4c6658fd, 90 },
    { 0xaf658fe, 33 },
    { 0x3f1658ff, 84 },
    { 0x83965900, 98 },
    { 0xc82658fe, 90 },
    { 0x7a8658ff, 33 },
    { 0xdc65903, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe65906, 84 },
    { 0x3ba65907, 17 },
    { 0xc7065908, 61 },
    { 0x4ba65908, 61 },
    { 0, 0 },
    { 0x4a16590b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7365911, 95 },
    { 0, 0 },
    { 0x8b165913, 46 },
    { 0x5c665914, 95 },
    { 0x66765914, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c65920, 55 },
    { 0xdc865921, 95 },
    { 0x67765921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21865927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4436592d, 90 },
    { 0, 0 },
    { 0xea86592f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a65933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f65937, 76 },
    { 0, 0 },
    { 0xe0865939, 69 },
    { 0, 0 },
    { 0x8a06593b, 90 },
    { 0xc8e6593b, 85 },
    { 0x57a6593b, 34 },
    { 0x72a6593e, 34 },
    { 0xa596593f, 67 },
    { 0x94e65940, 33 },
    { 0, 0 },
    { 0xbc665942, 55 },
    { 0x85265943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a65947, 23 },
    { 0, 0 },
    { 0x91265949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd6594e, 78 },
    { 0xcb96594e, 66 },
    { 0, 0 },
    { 0x37665951, 90 },
    { 0x19565952, 61 },
    { 0xfe165953, 76 },
    { 0xf8965953, 90 },
    { 0xd6e65954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde865959, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb466595d, 90 },
    { 0, 0 },
    { 0xf596595f, 90 },
    { 0x3e76595f, 80 },
    { 0xf9865960, 46 },
    { 0x6d265962, 43 },
    { 0x32e65963, 99 },
    { 0xfe565964, 90 },
    { 0xcc86595f, 33 },
    { 0x19465966, 55 },
    { 0x24e65967, 33 },
    { 0x52665968, 94 },
    { 0, 0 },
    { 0xb7b6596a, 33 },
    { 0x72d6596b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa665970, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c6597f, 93 },
    { 0xe9165980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d565984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d565989, 36 },
    { 0xe4265989, 34 },
    { 0xcfc6598b, 81 },
    { 0, 0 },
    { 0x6a26598d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f65991, 98 },
    { 0xf2465992, 76 },
    { 0x23765992, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2065996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe266599a, 55 },
    { 0, 0 },
    { 0xed96599c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6599f, 91 },
    { 0xb31659a0, 61 },
    { 0xa8659a1, 67 },
    { 0x16c659a2, 69 },
    { 0x366659a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb659a9, 34 },
    { 0, 0 },
    { 0x7b659ab, 33 },
    { 0xe82659ab, 52 },
    { 0xc1f659ac, 100 },
    { 0xaf2659ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d659b3, 95 },
    { 0xd41659b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73659b9, 93 },
    { 0x323659ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf659be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf659c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7659d0, 87 },
    { 0xe82659d1, 43 },
    { 0xfee659d2, 98 },
    { 0x8ea659d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5659dc, 33 },
    { 0x99d659dc, 42 },
    { 0xfd3659dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8659e2, 90 },
    { 0xaa6659e2, 61 },
    { 0, 0 },
    { 0xacd659e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf659eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46659ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7659f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9659f8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6659fc, 61 },
    { 0xb41659fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad65a06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a09, 98 },
    { 0x36765a09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea165a0f, 90 },
    { 0x57165a10, 88 },
    { 0, 0 },
    { 0x4f565a12, 34 },
    { 0x64765a12, 61 },
    { 0xee865a14, 98 },
    { 0x6df65a15, 90 },
    { 0x16765a15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c65a1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x82665a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25665a26, 34 },
    { 0xeca65a26, 55 },
    { 0, 0 },
    { 0x3b665a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c65a2c, 33 },
    { 0xeb765a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed765a31, 43 },
    { 0x59c65a32, 77 },
    { 0xda165a31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd65a39, 98 },
    { 0x98865a3a, 90 },
    { 0, 0 },
    { 0x64d65a3c, 61 },
    { 0x97d65a3d, 90 },
    { 0xbdb65a3e, 68 },
    { 0x4fd65a3f, 90 },
    { 0xf6865a40, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d65a44, 90 },
    { 0xa5765a45, 33 },
    { 0, 0 },
    { 0xeb865a47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e465a4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64965a53, 33 },
    { 0xb3365a53, 90 },
    { 0xbea65a55, 76 },
    { 0xa8c65a56, 33 },
    { 0, 0 },
    { 0x4d065a58, 61 },
    { 0x7ca65a59, 93 },
    { 0xe3465a59, 77 },
    { 0, 0 },
    { 0x93f65a5c, 76 },
    { 0, 0 },
    { 0x14565a5e, 88 },
    { 0, 0 },
    { 0x59565a60, 55 },
    { 0x16a65a61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b65a65, 98 },
    { 0xf2865a65, 95 },
    { 0x28d65a67, 90 },
    { 0x86765a68, 33 },
    { 0x29d65a66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e565a71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b65a74, 99 },
    { 0, 0 },
    { 0xfa265a76, 90 },
    { 0, 0 },
    { 0xf8265a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae865a7d, 69 },
    { 0x77565a7e, 98 },
    { 0xa6065a7e, 95 },
    { 0x69265a80, 23 },
    { 0x21d65a7e, 98 },
    { 0xf265a7e, 95 },
    { 0x38765a83, 75 },
    { 0x66b65a84, 90 },
    { 0x86965a84, 91 },
    { 0xa065a86, 34 },
    { 0x1e665a87, 23 },
    { 0x6e265a85, 61 },
    { 0xfa765a80, 100 },
    { 0, 0 },
    { 0x65265a8b, 46 },
    { 0x6bc65a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a65a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db65a93, 33 },
    { 0, 0 },
    { 0x2ee65a95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a265a98, 55 },
    { 0x34165a99, 95 },
    { 0xd6e65a9a, 76 },
    { 0, 0 },
    { 0x3d265a9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94465aa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f665aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d865aae, 90 },
    { 0x9ec65aae, 90 },
    { 0x20a65ab0, 46 },
    { 0, 0 },
    { 0xae65ab2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac65abd, 55 },
    { 0x61f65abe, 90 },
    { 0x3bf65abf, 93 },
    { 0x84565ac0, 61 },
    { 0x32765ac0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86265ac6, 100 },
    { 0, 0 },
    { 0x4a165ac8, 55 },
    { 0x21165ac8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25465ace, 95 },
    { 0xc9665acf, 66 },
    { 0xfe65ace, 52 },
    { 0xed265ad1, 66 },
    { 0x34865ad1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2665ad5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf65adc, 43 },
    { 0x2ab65add, 93 },
    { 0x7ea65ade, 77 },
    { 0xba65adc, 34 },
    { 0xc6a65ae0, 67 },
    { 0, 0 },
    { 0x1b865ae2, 90 },
    { 0x9765ae3, 61 },
    { 0xebf65ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f65aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9865af6, 36 },
    { 0, 0 },
    { 0x3c365af8, 33 },
    { 0, 0 },
    { 0x1665afa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1065afd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83965b01, 33 },
    { 0x6c565b02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b565b05, 23 },
    { 0xd4c65b06, 90 },
    { 0, 0 },
    { 0x3f065b08, 61 },
    { 0x3b865b09, 76 },
    { 0, 0 },
    { 0x10265b0b, 92 },
    { 0x89665b0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba65b11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3465b14, 34 },
    { 0x70765b15, 33 },
    { 0, 0 },
    { 0x63f65b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb665b1c, 49 },
    { 0xebe65b1c, 61 },
    { 0xbd65b1e, 35 },
    { 0, 0 },
    { 0x2c465b20, 90 },
    { 0xb4d65b20, 24 },
    { 0, 0 },
    { 0x37165b23, 34 },
    { 0x9d665b24, 46 },
    { 0x8fc65b24, 23 },
    { 0xe6665b26, 61 },
    { 0xd2f65b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2365b2a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5665b2d, 93 },
    { 0xd1665b2d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87965b32, 65 },
    { 0x89c65b33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea365b37, 78 },
    { 0x84165b38, 55 },
    { 0xccb65b39, 43 },
    { 0x7b465b3a, 90 },
    { 0x83565b3b, 69 },
    { 0xe8a65b39, 99 },
    { 0, 0 },
    { 0xa6d65b3e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64765b42, 76 },
    { 0, 0 },
    { 0x17565b44, 23 },
    { 0x7c865b44, 61 },
    { 0x49365b46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d465b4f, 34 },
    { 0xa5965b50, 55 },
    { 0, 0 },
    { 0x80f65b52, 90 },
    { 0x46565b53, 68 },
    { 0x33c65b54, 34 },
    { 0, 0 },
    { 0x95965b56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c65b5a, 90 },
    { 0, 0 },
    { 0x7565b5c, 96 },
    { 0x6bb65b5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde65b69, 33 },
    { 0, 0 },
    { 0xbc965b6b, 78 },
    { 0, 0 },
    { 0x54a65b6d, 76 },
    { 0, 0 },
    { 0xe4f65b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74265b76, 61 },
    { 0, 0 },
    { 0x80965b78, 34 },
    { 0x29b65b79, 87 },
    { 0, 0 },
    { 0xbc765b7b, 76 },
    { 0x27d65b7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd65b7f, 61 },
    { 0, 0 },
    { 0x14765b81, 94 },
    { 0xa6565b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c65b89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca65b8d, 98 },
    { 0xf2465b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc265b95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6765b98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90865b9c, 76 },
    { 0x8bd65b9c, 40 },
    { 0x7e965b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2165ba3, 33 },
    { 0, 0 },
    { 0x98c65ba5, 33 },
    { 0xd0c65ba6, 90 },
    { 0x76565ba6, 90 },
    { 0, 0 },
    { 0xf3d65ba9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a065bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3265bb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95665bb8, 33 },
    { 0, 0 },
    { 0xc9b65bba, 61 },
    { 0x39265bba, 90 },
    { 0, 0 },
    { 0x79765bbd, 34 },
    { 0, 0 },
    { 0xc9265bbf, 33 },
    { 0x33e65bc0, 99 },
    { 0xad365bc1, 55 },
    { 0x6cc65bc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf865bc7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a865bce, 90 },
    { 0, 0 },
    { 0x1e165bd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a465bda, 80 },
    { 0x20565bdb, 33 },
    { 0x28665bdc, 95 },
    { 0xe1965bdd, 23 },
    { 0x8e365bde, 95 },
    { 0x5d665bde, 98 },
    { 0xc8365be0, 90 },
    { 0x43165bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d65be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x89565be9, 95 },
    { 0x7165bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b465bed, 90 },
    { 0, 0 },
    { 0x15865bef, 93 },
    { 0, 0 },
    { 0x8ff65bf1, 33 },
    { 0x2f465bf2, 95 },
    { 0, 0 },
    { 0xd8b65bf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d65bf8, 95 },
    { 0xaaa65bf8, 90 },
    { 0x87165bfa, 45 },
    { 0xfa265bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e65c00, 61 },
    { 0, 0 },
    { 0x91565c02, 33 },
    { 0x45b65c03, 98 },
    { 0x15a65c02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4865c07, 61 },
    { 0xe6765c08, 33 },
    { 0xa0d65c09, 95 },
    { 0xf4c65c0a, 95 },
    { 0xfe665c07, 61 },
    { 0x17665c07, 90 },
    { 0xc8665c0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed65c10, 98 },
    { 0, 0 },
    { 0x34165c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba65c21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de65c24, 48 },
    { 0x55065c24, 61 },
    { 0xb9f65c24, 23 },
    { 0x72d65c27, 25 },
    { 0x69465c28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b65c2b, 69 },
    { 0x8cb65c2c, 90 },
    { 0, 0 },
    { 0x52e65c2e, 23 },
    { 0x10065c2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41065c34, 90 },
    { 0x85965c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b965c38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b65c3b, 66 },
    { 0x5b65c3b, 85 },
    { 0x62365c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d865c41, 93 },
    { 0, 0 },
    { 0x82965c43, 33 },
    { 0xc0065c44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f065c47, 77 },
    { 0xfce65c48, 33 },
    { 0x7ef65c49, 51 },
    { 0xf465c4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0465c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a65c54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85165c58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58765c5b, 61 },
    { 0x60e65c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4265c5f, 93 },
    { 0, 0 },
    { 0x54865c61, 46 },
    { 0x9a065c62, 90 },
    { 0x9ba65c63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61965c68, 55 },
    { 0x8b65c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e865c6d, 75 },
    { 0xacb65c6e, 90 },
    { 0x66e65c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd165c72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8365c79, 23 },
    { 0x3f365c7a, 95 },
    { 0x28565c7b, 95 },
    { 0x89265c7c, 43 },
    { 0x9cf65c7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a965c80, 100 },
    { 0, 0 },
    { 0x1ed65c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db65c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc65c8b, 33 },
    { 0x22765c8c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5965c8f, 42 },
    { 0x27865c90, 98 },
    { 0xda165c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79665c95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e465c9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e765ca3, 33 },
    { 0xd7165ca3, 69 },
    { 0, 0 },
    { 0xca865ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90065ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65cac, 95 },
    { 0x2d465cad, 90 },
    { 0x3d665cae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e165cb1, 66 },
    { 0x99a65cb2, 33 },
    { 0x61f65cb2, 53 },
    { 0x70165cb4, 33 },
    { 0x21a65cb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda365cba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665cbd, 98 },
    { 0x56965cbd, 95 },
    { 0x13665cbf, 90 },
    { 0xd3f65cbe, 43 },
    { 0, 0 },
    { 0xdbc65cc2, 61 },
    { 0xbf565cc3, 76 },
    { 0xa6c65cc3, 80 },
    { 0x17d65cc5, 61 },
    { 0xe9965cc3, 90 },
    { 0xa8c65cc7, 84 },
    { 0x35a65cc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf65ccb, 99 },
    { 0x7d465ccc, 85 },
    { 0xf6465ccc, 98 },
    { 0, 0 },
    { 0x7f065ccf, 100 },
    { 0xf2765cd0, 80 },
    { 0xda865cd1, 90 },
    { 0, 0 },
    { 0xd9365cd3, 52 },
    { 0, 0 },
    { 0xc1a65cd5, 61 },
    { 0xd5765cd6, 81 },
    { 0xafd65cd5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a365ce0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f65ce5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7665cea, 100 },
    { 0x73165ceb, 43 },
    { 0, 0 },
    { 0x5ad65ced, 43 },
    { 0, 0 },
    { 0x66165cef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49865cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2265cfa, 76 },
    { 0xca265cfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c65cfe, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4965d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf765d04, 95 },
    { 0x27265d05, 90 },
    { 0x5ae65d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1965d10, 98 },
    { 0x49665d10, 33 },
    { 0xa7365d12, 90 },
    { 0, 0 },
    { 0x44465d14, 33 },
    { 0x92665d15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f65d1a, 95 },
    { 0, 0 },
    { 0xcbf65d1c, 85 },
    { 0x37e65d1d, 33 },
    { 0x25765d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35565d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b65d26, 100 },
    { 0x68365d26, 43 },
    { 0x67065d28, 77 },
    { 0xb2265d29, 98 },
    { 0xc665d2a, 26 },
    { 0x44165d2a, 95 },
    { 0x22365d26, 95 },
    { 0, 0 },
    { 0x26d65d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee65d36, 61 },
    { 0x5b65d36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d365d3a, 41 },
    { 0xe565d3b, 98 },
    { 0x1365d3c, 61 },
    { 0, 0 },
    { 0xe5865d3e, 90 },
    { 0xd0265d3f, 61 },
    { 0, 0 },
    { 0x20f65d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6865d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e65d4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e365d54, 33 },
    { 0xdc465d54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9565d58, 78 },
    { 0x31865d59, 93 },
    { 0xe1765d59, 88 },
    { 0xffa65d5a, 98 },
    { 0xa5d65d59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x67165d5f, 23 },
    { 0xfb465d5f, 90 },
    { 0x4a465d60, 43 },
    { 0x92a65d61, 76 },
    { 0xc8865d63, 34 },
    { 0xa9065d63, 90 },
    { 0x6a865d65, 78 },
    { 0, 0 },
    { 0x62465d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37365d6b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d65d73, 75 },
    { 0x62365d74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81865d79, 90 },
    { 0, 0 },
    { 0xbe865d7b, 33 },
    { 0, 0 },
    { 0xaf065d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90865d86, 98 },
    { 0, 0 },
    { 0x43965d88, 61 },
    { 0xd7365d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97965d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc65d9b, 87 },
    { 0, 0 },
    { 0x79a65d9d, 23 },
    { 0xa5765d9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c65da2, 55 },
    { 0x2d665da3, 96 },
    { 0xe5065da4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44265da8, 61 },
    { 0xab65da9, 34 },
    { 0, 0 },
    { 0x26f65dab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b365dae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7765db2, 69 },
    { 0xacc65db3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a265db6, 99 },
    { 0, 0 },
    { 0xb0065db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c65dbd, 80 },
    { 0, 0 },
    { 0xe9265dbf, 23 },
    { 0, 0 },
    { 0xa1e65dc1, 33 },
    { 0x4c665dc2, 78 },
    { 0xff265dc1, 88 },
    { 0x67665dc4, 33 },
    { 0x6f965dc5, 95 },
    { 0x61065dc4, 96 },
    { 0x8d665dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b865dca, 90 },
    { 0xf8965dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa165dd2, 90 },
    { 0, 0 },
    { 0xa7565dd4, 90 },
    { 0x4e565dd4, 90 },
    { 0x39065dd6, 40 },
    { 0x94365dd5, 40 },
    { 0xbd65dd8, 33 },
    { 0x3865dd9, 90 },
    { 0x9d665dd8, 23 },
    { 0x2c065ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6065ddf, 90 },
    { 0x4e865de0, 61 },
    { 0, 0 },
    { 0xd9a65de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c65de5, 91 },
    { 0xf8565de5, 61 },
    { 0xc5765de7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac865df1, 43 },
    { 0x29e65df1, 61 },
    { 0, 0 },
    { 0x99665df4, 33 },
    { 0, 0 },
    { 0x1b065df6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d165df9, 84 },
    { 0xa5465df9, 61 },
    { 0, 0 },
    { 0xda165dfc, 98 },
    { 0x35865dfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78865e01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f165e04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37065e08, 33 },
    { 0x84d65e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9265e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d65e14, 95 },
    { 0xa4065e15, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d65e18, 46 },
    { 0x72565e19, 78 },
    { 0, 0 },
    { 0x39465e1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13665e1e, 24 },
    { 0xf3f65e1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d365e22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a965e27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a65e2c, 90 },
    { 0x6f765e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9965e32, 34 },
    { 0xdcd65e32, 80 },
    { 0x25e65e34, 90 },
    { 0xd0c65e34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f65e3a, 34 },
    { 0x42165e3a, 98 },
    { 0, 0 },
    { 0x43465e3d, 46 },
    { 0x6c165e3e, 90 },
    { 0x63065e3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65e4b, 23 },
    { 0xc9f65e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4065e51, 100 },
    { 0, 0 },
    { 0xdb365e53, 90 },
    { 0xb3c65e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6265e57, 91 },
    { 0xb4f65e58, 33 },
    { 0xc1565e57, 90 },
    { 0, 0 },
    { 0x53665e5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e965e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda65e65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16365e68, 93 },
    { 0xbc65e69, 98 },
    { 0xa2b65e6a, 95 },
    { 0x9a165e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad65e6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89365e74, 43 },
    { 0x17965e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a65e78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b65e7c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda765e7f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865e83, 90 },
    { 0x3df65e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e65e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b265e90, 36 },
    { 0, 0 },
    { 0x75265e92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad65e96, 76 },
    { 0x50d65e96, 67 },
    { 0xefe65e98, 98 },
    { 0, 0 },
    { 0x50265e9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f865ea6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc65ea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf365eac, 98 },
    { 0x48965ead, 33 },
    { 0xfc765ead, 34 },
    { 0xe5b65eaf, 98 },
    { 0x70365eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d65eb3, 90 },
    { 0xc7b65eb4, 90 },
    { 0xfa465eb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee65eb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b165ebb, 61 },
    { 0x34365ebc, 87 },
    { 0, 0 },
    { 0x46965ebe, 55 },
    { 0xc665ebf, 61 },
    { 0, 0 },
    { 0x9a565ec1, 67 },
    { 0x3ae65ec2, 98 },
    { 0xbc665ec3, 98 },
    { 0x83165ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f65ec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64065ed3, 33 },
    { 0x18365ed4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d65ed7, 93 },
    { 0, 0 },
    { 0x2765ed9, 34 },
    { 0x99a65ed9, 53 },
    { 0x71f65edb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c965ee1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6165ee5, 76 },
    { 0x1e265ee5, 83 },
    { 0xa7265ee7, 61 },
    { 0, 0 },
    { 0xbe565ee9, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd465eed, 66 },
    { 0x18465eee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4165efb, 81 },
    { 0, 0 },
    { 0xe1765efd, 90 },
    { 0x54965efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40665f05, 86 },
    { 0x7a65f06, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3765f0a, 98 },
    { 0, 0 },
    { 0x9cf65f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82765f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2965f21, 76 },
    { 0xf1e65f22, 76 },
    { 0, 0 },
    { 0xc2465f24, 61 },
    { 0xc5665f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d65f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb65f30, 61 },
    { 0xd9d65f31, 61 },
    { 0x58b65f32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6065f39, 90 },
    { 0x2f265f39, 61 },
    { 0x31f65f3b, 33 },
    { 0x82665f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b65f42, 46 },
    { 0x5bb65f43, 90 },
    { 0xfd765f44, 95 },
    { 0x75165f45, 33 },
    { 0x76a65f44, 88 },
    { 0x9b65f43, 90 },
    { 0xb7665f44, 55 },
    { 0x6d465f46, 43 },
    { 0, 0 },
    { 0xbc465f4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c65f50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c165f55, 52 },
    { 0x1c65f55, 46 },
    { 0x9a865f57, 64 },
    { 0x8a965f55, 23 },
    { 0x24865f57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27065f5c, 61 },
    { 0x59965f5d, 90 },
    { 0x45e65f5c, 90 },
    { 0x20f65f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb65f6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d65f6e, 95 },
    { 0, 0 },
    { 0x31c65f70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f65f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7565f78, 90 },
    { 0x45e65f78, 99 },
    { 0x78065f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42765f7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a65f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76965f88, 90 },
    { 0xbbf65f88, 90 },
    { 0, 0 },
    { 0xc5665f8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9565f92, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d065f95, 23 },
    { 0xb8965f96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b65f9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5065fa1, 33 },
    { 0x8c865fa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11565fa7, 43 },
    { 0x6bb65fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a65fab, 61 },
    { 0x1b165fac, 90 },
    { 0x9db65fac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c565fb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4665fb5, 38 },
    { 0x1cb65fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c765fbb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae65fc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7565fc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d865fca, 95 },
    { 0x4bc65fcb, 61 },
    { 0x36d65fcb, 98 },
    { 0xc8765fcb, 33 },
    { 0x6bd65fce, 76 },
    { 0xf9665fce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7565fd7, 95 },
    { 0x59065fd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6665fdd, 33 },
    { 0x73c65fde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc165fe2, 90 },
    { 0, 0 },
    { 0xbaf65fe4, 76 },
    { 0, 0 },
    { 0xfba65fe6, 90 },
    { 0, 0 },
    { 0x60665fe8, 33 },
    { 0xbcf65fe9, 76 },
    { 0xa8f65fe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1865fed, 98 },
    { 0xa8265fee, 46 },
    { 0, 0 },
    { 0x67a65ff0, 52 },
    { 0xe1465ff0, 17 },
    { 0x38465ff2, 61 },
    { 0x97e65ff3, 96 },
    { 0, 0 },
    { 0xdca65ff5, 67 },
    { 0xb4565ff6, 95 },
    { 0x26665ff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df66002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91266005, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4366008, 95 },
    { 0xa3966009, 97 },
    { 0x17d66009, 61 },
    { 0x1236600b, 98 },
    { 0xb766009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f666012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c6601b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e66021, 90 },
    { 0xea866022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e6602d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c66031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a66035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46766039, 33 },
    { 0x7816603a, 61 },
    { 0x2ff6603b, 33 },
    { 0xfe66603c, 100 },
    { 0xc526603d, 90 },
    { 0x6786603d, 90 },
    { 0xd976603d, 98 },
    { 0x61866040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1366046, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066049, 34 },
    { 0, 0 },
    { 0xaae6604b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d56604e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20266055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee66059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7736605f, 55 },
    { 0x3ba66060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26566064, 23 },
    { 0, 0 },
    { 0xaf366066, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a866069, 90 },
    { 0x43366069, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf646606d, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb66072, 87 },
    { 0, 0 },
    { 0x4c66074, 33 },
    { 0, 0 },
    { 0xffa66076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c66079, 95 },
    { 0xb046607a, 90 },
    { 0x3166607b, 86 },
    { 0, 0 },
    { 0xb426607d, 23 },
    { 0, 0 },
    { 0x6a6607f, 61 },
    { 0, 0 },
    { 0xf6a66081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56566087, 33 },
    { 0xdf166087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76608b, 23 },
    { 0, 0 },
    { 0x5b26608d, 97 },
    { 0, 0 },
    { 0x3cc6608f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a66092, 90 },
    { 0xacc66093, 61 },
    { 0, 0 },
    { 0xea766095, 65 },
    { 0xb5866096, 33 },
    { 0xe3a66097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6609d, 77 },
    { 0, 0 },
    { 0x116609f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed660a9, 95 },
    { 0xd73660a9, 90 },
    { 0x90e660aa, 52 },
    { 0x4fc660ac, 46 },
    { 0x645660a9, 87 },
    { 0x788660a9, 84 },
    { 0xabc660af, 98 },
    { 0x58d660b0, 90 },
    { 0xb42660af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73660b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x625660bc, 90 },
    { 0xfa3660bd, 95 },
    { 0x157660be, 33 },
    { 0x8f660bc, 90 },
    { 0xd2d660bd, 76 },
    { 0x562660c1, 76 },
    { 0xccb660c2, 61 },
    { 0x5ca660c3, 53 },
    { 0x376660c4, 84 },
    { 0x568660c5, 23 },
    { 0x947660c5, 88 },
    { 0, 0 },
    { 0xdb9660c8, 90 },
    { 0xa9a660c9, 81 },
    { 0x99f660ca, 100 },
    { 0, 0 },
    { 0xb34660cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c660cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x350660d2, 74 },
    { 0xa8d660d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5660da, 55 },
    { 0x765660db, 33 },
    { 0x3be660db, 46 },
    { 0xb0d660dd, 52 },
    { 0xdd0660de, 93 },
    { 0x70e660df, 98 },
    { 0, 0 },
    { 0xfe660e1, 95 },
    { 0, 0 },
    { 0xcab660e3, 55 },
    { 0x5d4660e4, 78 },
    { 0x204660e4, 98 },
    { 0x30e660e6, 95 },
    { 0xdaf660e6, 43 },
    { 0x943660e8, 23 },
    { 0xba7660e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86660ef, 23 },
    { 0, 0 },
    { 0x91e660f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701660f7, 90 },
    { 0, 0 },
    { 0x1d660f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a660fd, 99 },
    { 0x5d5660fe, 95 },
    { 0xf2e660ff, 51 },
    { 0, 0 },
    { 0x30266101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6610c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386610f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca566112, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc966116, 61 },
    { 0xee566117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb6611a, 95 },
    { 0xf246611b, 90 },
    { 0, 0 },
    { 0x3e36611d, 23 },
    { 0x1946611e, 95 },
    { 0x6996611d, 95 },
    { 0x2da66120, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a66123, 33 },
    { 0, 0 },
    { 0x88a66125, 66 },
    { 0x8b166126, 33 },
    { 0xcba66127, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6612b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8766131, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf166137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106613b, 23 },
    { 0x1f76613c, 23 },
    { 0, 0 },
    { 0x3496613e, 101 },
    { 0x93e6613f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb666149, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6614c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb66150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49066153, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59666159, 52 },
    { 0xc266159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4666164, 76 },
    { 0x2f366165, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe866168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f6616c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26966175, 87 },
    { 0xcd066176, 43 },
    { 0xad466175, 95 },
    { 0, 0 },
    { 0xcbe66179, 90 },
    { 0xdd166179, 76 },
    { 0xe286617a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f66182, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6618a, 61 },
    { 0xbc86618b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5606618e, 66 },
    { 0x34a6618e, 76 },
    { 0xb9866190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe86619c, 90 },
    { 0xc0b6619c, 34 },
    { 0xa006619e, 81 },
    { 0x1a96619d, 46 },
    { 0, 0 },
    { 0x9a3661a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4661a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f661ac, 90 },
    { 0x72c661ad, 34 },
    { 0xf98661ae, 61 },
    { 0x596661af, 33 },
    { 0x4a661b0, 76 },
    { 0x506661af, 23 },
    { 0, 0 },
    { 0x9b661b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc661b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720661ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef661bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81661c3, 88 },
    { 0xf9b661c3, 84 },
    { 0, 0 },
    { 0xb7c661c6, 33 },
    { 0x81f661c7, 85 },
    { 0, 0 },
    { 0xe6e661c9, 46 },
    { 0x4a9661ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe661cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a661d1, 76 },
    { 0xf7a661d2, 80 },
    { 0xe0a661d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7661d7, 61 },
    { 0xab0661d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x677661db, 61 },
    { 0xe98661dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4661df, 52 },
    { 0, 0 },
    { 0x4a8661e1, 90 },
    { 0x50f661e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6661e9, 93 },
    { 0, 0 },
    { 0xff4661eb, 99 },
    { 0, 0 },
    { 0x9f3661ed, 52 },
    { 0, 0 },
    { 0x94b661ef, 76 },
    { 0x7cc661f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4661f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f661f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7661fb, 98 },
    { 0, 0 },
    { 0xfec661fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1966204, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6620b, 98 },
    { 0xb956620c, 23 },
    { 0, 0 },
    { 0x7396620e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x82266211, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8866218, 33 },
    { 0x3a466219, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f96621e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0366229, 75 },
    { 0xf346622a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf6622d, 81 },
    { 0xf076622e, 95 },
    { 0, 0 },
    { 0xe7066230, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8066235, 76 },
    { 0x7bd66236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce46623a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e6623f, 33 },
    { 0x26066240, 76 },
    { 0xcdd6623f, 34 },
    { 0xb8866241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f66247, 61 },
    { 0xc8a66248, 95 },
    { 0, 0 },
    { 0xad86624a, 33 },
    { 0x9be6624b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69666252, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda266255, 90 },
    { 0, 0 },
    { 0xf3b66257, 98 },
    { 0x7e166258, 61 },
    { 0x90766259, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a6625e, 43 },
    { 0x8db6625f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5266263, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4166267, 52 },
    { 0xc666268, 90 },
    { 0xe9366269, 61 },
    { 0xeae66267, 96 },
    { 0xc3666268, 87 },
    { 0, 0 },
    { 0x8226626d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x19866270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb866273, 66 },
    { 0xbe966273, 77 },
    { 0x9a666275, 34 },
    { 0xb2a66275, 61 },
    { 0xd566276, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916627c, 33 },
    { 0, 0 },
    { 0xe856627e, 23 },
    { 0x3806627f, 91 },
    { 0, 0 },
    { 0xb5a66281, 95 },
    { 0xb6c66281, 88 },
    { 0x10866283, 87 },
    { 0, 0 },
    { 0xd2d66285, 61 },
    { 0x3ae66286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566628c, 67 },
    { 0, 0 },
    { 0x66f6628e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c66295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91966298, 66 },
    { 0xe6266299, 23 },
    { 0x1946629a, 61 },
    { 0xde96629b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6629e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9662a2, 43 },
    { 0x2f3662a3, 66 },
    { 0x6ae662a4, 23 },
    { 0x2c9662a5, 100 },
    { 0, 0 },
    { 0x614662a7, 90 },
    { 0x178662a7, 34 },
    { 0x69f662a8, 68 },
    { 0x9f3662aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4662b0, 95 },
    { 0xd29662b1, 61 },
    { 0xce0662b2, 95 },
    { 0xc1662b3, 76 },
    { 0x81662b4, 23 },
    { 0xbfc662b5, 23 },
    { 0x7f1662b3, 23 },
    { 0, 0 },
    { 0xd87662b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac662bf, 46 },
    { 0x17a662c0, 34 },
    { 0x256662c1, 43 },
    { 0xcb5662c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc662c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4662cc, 33 },
    { 0xe40662cd, 77 },
    { 0, 0 },
    { 0xdef662cf, 55 },
    { 0xd45662d0, 98 },
    { 0x7ba662cf, 95 },
    { 0x1f4662d2, 76 },
    { 0xc79662d2, 61 },
    { 0x457662d1, 43 },
    { 0xe72662d5, 90 },
    { 0x85c662d6, 100 },
    { 0xdff662d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84662db, 98 },
    { 0x52b662dc, 33 },
    { 0, 0 },
    { 0x746662de, 95 },
    { 0x64d662df, 23 },
    { 0, 0 },
    { 0x713662e1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19662e7, 40 },
    { 0, 0 },
    { 0xd6a662e9, 95 },
    { 0, 0 },
    { 0x22e662eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc662f9, 93 },
    { 0, 0 },
    { 0x3b8662fb, 95 },
    { 0x30b662fc, 76 },
    { 0x869662fb, 96 },
    { 0, 0 },
    { 0xe6f662ff, 61 },
    { 0x99e66300, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71066305, 90 },
    { 0x78266306, 90 },
    { 0x9be66307, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6976630f, 96 },
    { 0x14166310, 98 },
    { 0, 0 },
    { 0x70766312, 33 },
    { 0x8f566312, 61 },
    { 0xb9866314, 90 },
    { 0x22c66315, 34 },
    { 0x95c66312, 95 },
    { 0xe4b66317, 95 },
    { 0, 0 },
    { 0xae966319, 33 },
    { 0x83e6631a, 95 },
    { 0x3ca6631a, 90 },
    { 0x4326631a, 77 },
    { 0x19566319, 98 },
    { 0x6ed6631e, 88 },
    { 0, 0 },
    { 0xeb666320, 98 },
    { 0, 0 },
    { 0x4fc66322, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66326, 23 },
    { 0, 0 },
    { 0x96f66328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a6632b, 55 },
    { 0xf966632b, 33 },
    { 0x5516632c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46266336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e6633a, 36 },
    { 0, 0 },
    { 0x2e26633c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c166342, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d66348, 90 },
    { 0x19166349, 98 },
    { 0x3b46634a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de66351, 61 },
    { 0xb4466352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf166356, 61 },
    { 0x50366357, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1746635a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d26635d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f66361, 23 },
    { 0, 0 },
    { 0x64566363, 91 },
    { 0x30d66364, 98 },
    { 0x51f66364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5596636a, 91 },
    { 0x3ad6636b, 90 },
    { 0x2cc6636b, 67 },
    { 0x9ff6636c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e66373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d166379, 98 },
    { 0xcaf66379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d6637f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89566385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf566392, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf386639b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd666639e, 65 },
    { 0, 0 },
    { 0x7d9663a0, 78 },
    { 0, 0 },
    { 0x84a663a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b663a7, 90 },
    { 0xfc1663a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb663ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43663b0, 23 },
    { 0x5b1663b1, 34 },
    { 0x369663b2, 100 },
    { 0xfa9663b0, 17 },
    { 0x766663b4, 98 },
    { 0xea1663b0, 61 },
    { 0x59e663b6, 98 },
    { 0x569663b7, 95 },
    { 0xd4b663b7, 95 },
    { 0, 0 },
    { 0xcca663ba, 8 },
    { 0x159663bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5663c3, 95 },
    { 0x80c663c3, 36 },
    { 0, 0 },
    { 0x30d663c6, 98 },
    { 0, 0 },
    { 0x2fd663c8, 76 },
    { 0x710663c9, 46 },
    { 0, 0 },
    { 0x7f2663cb, 46 },
    { 0x16a663cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438663d3, 76 },
    { 0, 0 },
    { 0x54a663d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6663e5, 65 },
    { 0xe9a663e6, 90 },
    { 0xe70663e7, 98 },
    { 0, 0 },
    { 0x4aa663e9, 95 },
    { 0xfe3663e9, 43 },
    { 0xf92663eb, 78 },
    { 0x8f3663eb, 76 },
    { 0xf28663ec, 61 },
    { 0x4b5663ee, 90 },
    { 0xd36663ef, 98 },
    { 0xbb7663ef, 34 },
    { 0x9d663ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4663f4, 95 },
    { 0, 0 },
    { 0xf09663f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0663fa, 90 },
    { 0x3c6663fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76866403, 98 },
    { 0, 0 },
    { 0x2166405, 43 },
    { 0x30466406, 90 },
    { 0x13366407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b6640e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x81766411, 34 },
    { 0xf9e66412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff66415, 33 },
    { 0, 0 },
    { 0x25b66417, 76 },
    { 0x16766417, 99 },
    { 0, 0 },
    { 0xf676641a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49266421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76366424, 95 },
    { 0x45866424, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b6642c, 95 },
    { 0x4556642c, 43 },
    { 0x5026642e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b566431, 90 },
    { 0, 0 },
    { 0x26a66433, 23 },
    { 0x2be66434, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e766441, 98 },
    { 0xdbd66442, 98 },
    { 0xfb666443, 90 },
    { 0xc9366444, 90 },
    { 0x86466445, 99 },
    { 0, 0 },
    { 0x51666447, 61 },
    { 0xfbf66448, 68 },
    { 0x78d66447, 34 },
    { 0xbdf6644a, 98 },
    { 0x5a466448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e66454, 95 },
    { 0xb4266454, 61 },
    { 0, 0 },
    { 0xf1366457, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda96645c, 17 },
    { 0x5476645d, 34 },
    { 0xd96645d, 38 },
    { 0, 0 },
    { 0xed266460, 75 },
    { 0, 0 },
    { 0xe9166462, 67 },
    { 0xf6066463, 23 },
    { 0, 0 },
    { 0xdfc66465, 100 },
    { 0x8ad66465, 90 },
    { 0, 0 },
    { 0x9a066468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846646e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d66471, 95 },
    { 0x89366472, 33 },
    { 0x12666472, 33 },
    { 0x71566474, 99 },
    { 0, 0 },
    { 0xb8a66476, 95 },
    { 0x96466477, 87 },
    { 0xa8266478, 84 },
    { 0xede66476, 100 },
    { 0x8f866477, 76 },
    { 0x3ae6647b, 38 },
    { 0x3ea66477, 95 },
    { 0, 0 },
    { 0xbcf6647e, 88 },
    { 0, 0 },
    { 0xd3e66480, 76 },
    { 0xb3f66481, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b366484, 95 },
    { 0, 0 },
    { 0x1a66486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de6648b, 96 },
    { 0x4926648c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x756648f, 78 },
    { 0x1a266490, 99 },
    { 0x64666490, 76 },
    { 0x39c66490, 98 },
    { 0, 0 },
    { 0x74566494, 98 },
    { 0xa7166494, 90 },
    { 0xd7566496, 23 },
    { 0x15d66494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe606649d, 61 },
    { 0, 0 },
    { 0x8236649f, 95 },
    { 0xaab6649f, 90 },
    { 0xd29664a0, 43 },
    { 0, 0 },
    { 0x4c664a3, 77 },
    { 0xf9d664a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5664a8, 95 },
    { 0xb0c664a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b664ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed664b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a664b5, 61 },
    { 0x308664b5, 90 },
    { 0, 0 },
    { 0x974664b8, 98 },
    { 0x6e8664b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3664bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1664bf, 61 },
    { 0xd47664c0, 33 },
    { 0xca8664c1, 76 },
    { 0xee9664c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x557664c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab664cc, 96 },
    { 0, 0 },
    { 0x986664ce, 23 },
    { 0xd664cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43664d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91664d9, 76 },
    { 0x438664da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a664de, 98 },
    { 0x8ee664de, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0664e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430664e6, 61 },
    { 0xa61664e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd664ea, 82 },
    { 0xfd8664ea, 34 },
    { 0xa84664ec, 61 },
    { 0x77c664ed, 8 },
    { 0xff2664ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29664f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e664f4, 95 },
    { 0, 0 },
    { 0x60b664f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54664f9, 55 },
    { 0x659664fa, 33 },
    { 0x7b5664fb, 65 },
    { 0x422664fc, 55 },
    { 0x86b664fb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b6650a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99766515, 37 },
    { 0x18f66516, 91 },
    { 0, 0 },
    { 0xc8266518, 23 },
    { 0x31e66519, 61 },
    { 0xf876651a, 90 },
    { 0, 0 },
    { 0x1f46651c, 52 },
    { 0x3306651c, 23 },
    { 0xaf16651d, 43 },
    { 0xb5b6651f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef66526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83866529, 85 },
    { 0x2196652a, 87 },
    { 0x10966529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8466533, 98 },
    { 0x25b66533, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5776653d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17266540, 80 },
    { 0x81466541, 77 },
    { 0, 0 },
    { 0xf9866543, 33 },
    { 0x8cd66544, 34 },
    { 0x4b466544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc36654a, 98 },
    { 0, 0 },
    { 0x476654c, 33 },
    { 0x5666654d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a866550, 98 },
    { 0, 0 },
    { 0xfe466552, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46366555, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae66560, 61 },
    { 0x70d66561, 23 },
    { 0, 0 },
    { 0x66e66563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e66566, 81 },
    { 0xd1b66566, 43 },
    { 0xcf66568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe66570, 34 },
    { 0, 0 },
    { 0x4e66572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18366575, 66 },
    { 0xa8066576, 90 },
    { 0xdf766576, 33 },
    { 0, 0 },
    { 0x2366579, 46 },
    { 0x8f36657a, 98 },
    { 0x46266579, 90 },
    { 0xa2b6657c, 33 },
    { 0xd086657b, 95 },
    { 0xd8866579, 55 },
    { 0x3876657f, 98 },
    { 0xf2d6657f, 100 },
    { 0x3c56657c, 43 },
    { 0x1de66582, 61 },
    { 0x29c66582, 82 },
    { 0x4b266584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca166587, 93 },
    { 0xac466588, 75 },
    { 0xb8666589, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5346658f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a66594, 46 },
    { 0xf9666595, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25665a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914665a8, 98 },
    { 0xb3665a9, 55 },
    { 0xe70665aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b665b0, 76 },
    { 0x1b4665b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0665b5, 23 },
    { 0xd59665b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0665b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2665bc, 61 },
    { 0x5ca665bd, 91 },
    { 0x846665be, 61 },
    { 0, 0 },
    { 0x8bd665c0, 87 },
    { 0, 0 },
    { 0xa14665c2, 100 },
    { 0x3fd665c3, 95 },
    { 0x25a665c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47665c7, 90 },
    { 0x465665c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46665cb, 100 },
    { 0x279665cc, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd665d0, 66 },
    { 0x47d665d1, 90 },
    { 0xf55665d0, 33 },
    { 0x31a665d3, 55 },
    { 0, 0 },
    { 0x8c5665d5, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886665da, 100 },
    { 0, 0 },
    { 0x5ce665dc, 61 },
    { 0xf0e665dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300665e1, 98 },
    { 0x63e665e2, 33 },
    { 0x194665e2, 77 },
    { 0x1e1665e4, 87 },
    { 0x98e665e2, 23 },
    { 0xcc4665e6, 33 },
    { 0xa83665e7, 90 },
    { 0x2f6665e2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b665ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a665f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640665fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2665ff, 76 },
    { 0xc5166600, 52 },
    { 0x53d665ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc566604, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb636660b, 33 },
    { 0xae46660b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0966612, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a66615, 66 },
    { 0, 0 },
    { 0x2366617, 87 },
    { 0, 0 },
    { 0x70f66619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd636661c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73666623, 98 },
    { 0x6f666624, 76 },
    { 0x99e66624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b666628, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16662b, 80 },
    { 0xc2d6662c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e66632, 46 },
    { 0x34d66633, 12 },
    { 0, 0 },
    { 0x9bf66635, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18666639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f96663d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d66640, 60 },
    { 0x2c166641, 88 },
    { 0xd7366641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec66645, 100 },
    { 0, 0 },
    { 0xf4b66647, 23 },
    { 0, 0 },
    { 0x93d66649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb396664c, 90 },
    { 0, 0 },
    { 0xcc86664e, 55 },
    { 0x1686664f, 55 },
    { 0xf216664f, 90 },
    { 0x57366651, 100 },
    { 0, 0 },
    { 0x18966653, 95 },
    { 0xde766653, 95 },
    { 0, 0 },
    { 0xf1b66656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd6665e, 33 },
    { 0xbe06665f, 55 },
    { 0, 0 },
    { 0x91266661, 91 },
    { 0, 0 },
    { 0xfd566663, 61 },
    { 0x81766663, 93 },
    { 0xfc066665, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746666d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8166678, 66 },
    { 0xc1166679, 34 },
    { 0x1336667a, 90 },
    { 0x20666678, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf66667f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b66682, 98 },
    { 0, 0 },
    { 0x28c66684, 55 },
    { 0x4e066684, 90 },
    { 0xe3e66686, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8346668c, 33 },
    { 0x3256668d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd66690, 80 },
    { 0, 0 },
    { 0x1d466692, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b6669a, 65 },
    { 0, 0 },
    { 0xba96669c, 90 },
    { 0, 0 },
    { 0x4036669e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6666a1, 33 },
    { 0, 0 },
    { 0x75c666a3, 100 },
    { 0, 0 },
    { 0x606666a5, 90 },
    { 0xb2f666a6, 33 },
    { 0xf71666a7, 23 },
    { 0x10a666a7, 55 },
    { 0xf77666a7, 65 },
    { 0x25d666aa, 87 },
    { 0xc38666a9, 90 },
    { 0, 0 },
    { 0xa54666ad, 23 },
    { 0x49f666ad, 90 },
    { 0, 0 },
    { 0x9d6666b0, 83 },
    { 0x8b0666b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x419666b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc666ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27666be, 76 },
    { 0xf51666bf, 52 },
    { 0, 0 },
    { 0xc74666c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3666c6, 88 },
    { 0xf5f666c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc666ca, 67 },
    { 0x7aa666ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2666ce, 90 },
    { 0, 0 },
    { 0xf30666d0, 55 },
    { 0, 0 },
    { 0x608666d2, 61 },
    { 0, 0 },
    { 0x6da666d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe666da, 46 },
    { 0xe2b666db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b666e1, 88 },
    { 0x7eb666e2, 61 },
    { 0xe0e666e3, 76 },
    { 0x879666e2, 81 },
    { 0xf48666e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568666ea, 90 },
    { 0xf59666eb, 83 },
    { 0xd3c666ea, 61 },
    { 0, 0 },
    { 0x922666ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61666f6, 43 },
    { 0, 0 },
    { 0xbab666f8, 52 },
    { 0, 0 },
    { 0xf32666fa, 93 },
    { 0x4ed666fa, 99 },
    { 0x161666fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef66702, 33 },
    { 0xbed66703, 66 },
    { 0xc4066702, 98 },
    { 0xc0a66703, 99 },
    { 0xd6866706, 98 },
    { 0x8666705, 91 },
    { 0x94866704, 98 },
    { 0, 0 },
    { 0x31d6670a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2976670e, 90 },
    { 0x4746670f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a66712, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86671e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c866723, 85 },
    { 0x55466724, 34 },
    { 0x5ae66725, 61 },
    { 0xd6766726, 98 },
    { 0xd6666727, 61 },
    { 0x5e466728, 90 },
    { 0x45b66728, 90 },
    { 0x7f566725, 17 },
    { 0x9d56672b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6672e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a266731, 23 },
    { 0x86c66731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57166735, 52 },
    { 0, 0 },
    { 0x45d66737, 33 },
    { 0, 0 },
    { 0x29866739, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696673d, 61 },
    { 0x4206673e, 66 },
    { 0x4eb6673f, 99 },
    { 0xb3b66740, 95 },
    { 0xf7a66741, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e366745, 61 },
    { 0x5d166745, 34 },
    { 0x4ac66746, 23 },
    { 0, 0 },
    { 0xf8c66749, 43 },
    { 0x1d166749, 84 },
    { 0xe6d66749, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76674e, 46 },
    { 0, 0 },
    { 0xc9266750, 17 },
    { 0x26d66751, 67 },
    { 0x1c166751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad66755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe566758, 46 },
    { 0x5d666758, 90 },
    { 0xe406675a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d6675d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb566761, 90 },
    { 0xfe166761, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa466765, 93 },
    { 0, 0 },
    { 0x43266767, 61 },
    { 0xa1666768, 43 },
    { 0x91066768, 98 },
    { 0, 0 },
    { 0x65f6676b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66676e, 99 },
    { 0x4b16676f, 55 },
    { 0xc2e6676e, 90 },
    { 0x7a366771, 85 },
    { 0x70866771, 76 },
    { 0x42e66773, 61 },
    { 0xc9f66774, 98 },
    { 0x3c666775, 46 },
    { 0x63066775, 61 },
    { 0xde766776, 76 },
    { 0x95966775, 61 },
    { 0x1b466779, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e6677e, 61 },
    { 0x9a56677f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e66782, 43 },
    { 0x6e466782, 94 },
    { 0x20466784, 98 },
    { 0x35366784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe366788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986678c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac66791, 77 },
    { 0xfc666791, 77 },
    { 0x5f466793, 66 },
    { 0x35c66793, 23 },
    { 0x73266795, 61 },
    { 0xd3666791, 98 },
    { 0x38166797, 81 },
    { 0x42366798, 23 },
    { 0x12266799, 52 },
    { 0x3a86679a, 98 },
    { 0x8836679b, 98 },
    { 0x7e26679a, 61 },
    { 0xdb566793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30667a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7667aa, 66 },
    { 0xbd7667aa, 98 },
    { 0xe1c667ac, 100 },
    { 0, 0 },
    { 0xdcf667ae, 91 },
    { 0xfed667af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736667b3, 55 },
    { 0xeb3667b4, 52 },
    { 0xed7667b3, 55 },
    { 0x56e667b6, 34 },
    { 0xe6e667b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff667bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd667c3, 84 },
    { 0xf6f667c3, 23 },
    { 0xcfc667c5, 33 },
    { 0x95a667c6, 90 },
    { 0xd33667c6, 33 },
    { 0x38c667c8, 95 },
    { 0, 0 },
    { 0x7f4667ca, 82 },
    { 0x79e667cb, 33 },
    { 0xd5a667cb, 23 },
    { 0x856667cc, 61 },
    { 0xa90667cb, 64 },
    { 0, 0 },
    { 0x3e5667d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e667d3, 93 },
    { 0x1e4667d4, 46 },
    { 0xf74667d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5667d8, 33 },
    { 0xafe667d8, 80 },
    { 0, 0 },
    { 0x838667db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6667de, 90 },
    { 0x70e667df, 98 },
    { 0, 0 },
    { 0x6fd667e1, 91 },
    { 0xa37667e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64667e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff667ec, 98 },
    { 0x197667ec, 84 },
    { 0x9e1667ee, 53 },
    { 0x2a3667ef, 55 },
    { 0xf7c667f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b667f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2667f9, 91 },
    { 0x761667fa, 100 },
    { 0xd68667fb, 98 },
    { 0x692667fa, 98 },
    { 0xb89667fb, 61 },
    { 0xeb4667fb, 67 },
    { 0, 0 },
    { 0xea066800, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca66803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a66808, 93 },
    { 0xe8266809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b26680d, 23 },
    { 0x3ed6680d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed66812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc66816, 23 },
    { 0x9b766816, 90 },
    { 0x30f66817, 87 },
    { 0xbe66817, 61 },
    { 0, 0 },
    { 0x8d86681b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb6681f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda166824, 23 },
    { 0x69866825, 85 },
    { 0x47e66824, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xef866829, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4936682d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce66833, 100 },
    { 0x8d566833, 90 },
    { 0x89266835, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51266844, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e66848, 51 },
    { 0, 0 },
    { 0xdf76684a, 81 },
    { 0xc8f6684a, 61 },
    { 0x1f96684c, 61 },
    { 0xadf6684d, 90 },
    { 0x8f36684a, 33 },
    { 0xc476684f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c766852, 23 },
    { 0xbde66853, 23 },
    { 0, 0 },
    { 0x48c66855, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a166860, 43 },
    { 0, 0 },
    { 0x33166862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49166866, 91 },
    { 0, 0 },
    { 0x9e366868, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e6686e, 61 },
    { 0xc876686f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e66875, 91 },
    { 0x66c66876, 90 },
    { 0, 0 },
    { 0xffa66878, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386687c, 36 },
    { 0, 0 },
    { 0xd906687e, 61 },
    { 0xdfc6687e, 61 },
    { 0x46666880, 76 },
    { 0xae866880, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f6688f, 66 },
    { 0, 0 },
    { 0x56f66891, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b866896, 61 },
    { 0x4f866897, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb346689a, 98 },
    { 0xa3d6689a, 46 },
    { 0, 0 },
    { 0x72e6689d, 66 },
    { 0xbed6689e, 90 },
    { 0x18f6689f, 93 },
    { 0xf5a668a0, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec668af, 90 },
    { 0xbc2668af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5668b3, 33 },
    { 0, 0 },
    { 0xc76668b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375668bd, 98 },
    { 0x72d668be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1668c3, 76 },
    { 0x533668c3, 76 },
    { 0xa8c668c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9668c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e668d1, 90 },
    { 0x174668d2, 33 },
    { 0x5d2668d2, 69 },
    { 0x1a668d4, 95 },
    { 0x472668d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80668e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f668e4, 65 },
    { 0, 0 },
    { 0x94668e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912668ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd668ee, 55 },
    { 0xac3668ef, 55 },
    { 0, 0 },
    { 0x913668f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4668fc, 23 },
    { 0, 0 },
    { 0xbd9668fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f466902, 95 },
    { 0, 0 },
    { 0x3e166904, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead6690c, 90 },
    { 0x7566690d, 43 },
    { 0, 0 },
    { 0x90d6690f, 95 },
    { 0, 0 },
    { 0x5ee66911, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72766915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be66919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d6691c, 85 },
    { 0xc6a6691c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f66926, 87 },
    { 0, 0 },
    { 0xe4f66928, 55 },
    { 0xe1266928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e6692c, 98 },
    { 0x91c6692c, 78 },
    { 0x9e56692d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8366931, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45866935, 85 },
    { 0, 0 },
    { 0x7c066937, 90 },
    { 0xf2466937, 98 },
    { 0xe6b66939, 34 },
    { 0, 0 },
    { 0x9706693b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f66940, 95 },
    { 0x39066941, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5266946, 33 },
    { 0, 0 },
    { 0x4ff66948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6695b, 88 },
    { 0, 0 },
    { 0x2286695d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x91666960, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a466964, 65 },
    { 0x51166965, 55 },
    { 0xe6d66966, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c366973, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df66976, 95 },
    { 0, 0 },
    { 0xd2f66978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7296697f, 93 },
    { 0x446697f, 56 },
    { 0xe0c66980, 90 },
    { 0x7b566980, 34 },
    { 0x8e266983, 65 },
    { 0x26f66984, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c66989, 93 },
    { 0xf6766989, 80 },
    { 0xc746698b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba866990, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81366995, 90 },
    { 0x94e66996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84466999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a76699f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653669a5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47669a8, 68 },
    { 0x657669a9, 88 },
    { 0x9d7669aa, 55 },
    { 0x430669ab, 61 },
    { 0xb95669ac, 75 },
    { 0x8c8669ad, 33 },
    { 0xe88669ac, 33 },
    { 0x98669af, 90 },
    { 0x9f8669af, 33 },
    { 0, 0 },
    { 0x361669b2, 98 },
    { 0xfd3669b2, 43 },
    { 0, 0 },
    { 0xd32669b5, 24 },
    { 0xa1c669b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x229669b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3669bf, 99 },
    { 0x5c669bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc669c4, 43 },
    { 0xd3b669c4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d669cc, 98 },
    { 0, 0 },
    { 0xaa9669ce, 88 },
    { 0x45c669cf, 95 },
    { 0, 0 },
    { 0xb52669d1, 23 },
    { 0x265669d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1669d5, 96 },
    { 0x73c669d5, 76 },
    { 0x91c669d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987669db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f669df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409669e7, 52 },
    { 0x31b669e8, 76 },
    { 0x446669e9, 52 },
    { 0x364669e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6669ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee669f3, 61 },
    { 0, 0 },
    { 0x64e669f5, 78 },
    { 0x2e7669f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f669f9, 33 },
    { 0, 0 },
    { 0xe50669fb, 23 },
    { 0xb5b669fc, 95 },
    { 0xfc5669fd, 23 },
    { 0, 0 },
    { 0x830669ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb66a09, 61 },
    { 0, 0 },
    { 0x3c66a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c66a0e, 98 },
    { 0x47a66a0f, 76 },
    { 0xca866a10, 98 },
    { 0x6fd66a0f, 90 },
    { 0x53366a0e, 46 },
    { 0xd7366a13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35466a18, 76 },
    { 0x86466a19, 72 },
    { 0, 0 },
    { 0xcdf66a1b, 61 },
    { 0, 0 },
    { 0xb6766a1d, 90 },
    { 0x6e366a1d, 43 },
    { 0x15e66a1f, 98 },
    { 0x31666a1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f66a23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70666a26, 76 },
    { 0xecf66a27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5766a2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc966a35, 33 },
    { 0xe066a35, 95 },
    { 0, 0 },
    { 0x50966a38, 98 },
    { 0xba366a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9066a40, 61 },
    { 0x80e66a40, 55 },
    { 0, 0 },
    { 0xb2366a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b166a46, 33 },
    { 0xf1e66a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b66a4a, 55 },
    { 0x52166a4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55466a4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d66a54, 91 },
    { 0, 0 },
    { 0x72466a56, 78 },
    { 0xbaf66a57, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de66a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c66a5e, 84 },
    { 0x69366a5f, 76 },
    { 0xadd66a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d466a63, 43 },
    { 0x1aa66a64, 76 },
    { 0xffe66a64, 90 },
    { 0x35f66a66, 66 },
    { 0xad366a67, 83 },
    { 0, 0 },
    { 0x9e766a69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94266a6f, 95 },
    { 0, 0 },
    { 0x84866a71, 33 },
    { 0x99966a72, 91 },
    { 0xc3266a73, 90 },
    { 0xdf166a74, 66 },
    { 0x64166a71, 91 },
    { 0, 0 },
    { 0xbdc66a77, 90 },
    { 0x93966a77, 95 },
    { 0x1dd66a79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a66a7c, 91 },
    { 0x31666a7c, 90 },
    { 0x6866a7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8166a81, 95 },
    { 0xec566a81, 34 },
    { 0x93b66a83, 90 },
    { 0xed966a82, 90 },
    { 0, 0 },
    { 0x17666a86, 98 },
    { 0, 0 },
    { 0x64d66a88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9666a8d, 95 },
    { 0x45566a8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43766a99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59866aa5, 76 },
    { 0, 0 },
    { 0xfbd66aa7, 95 },
    { 0xf4f66aa8, 95 },
    { 0x89f66aa9, 95 },
    { 0x93666aaa, 34 },
    { 0xeb966aab, 87 },
    { 0x49066aaa, 33 },
    { 0x1d966aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41866ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7766ab7, 33 },
    { 0xb1166ab8, 100 },
    { 0xc7266ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5666abf, 81 },
    { 0, 0 },
    { 0x11f66ac1, 90 },
    { 0x10f66ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87566ac9, 43 },
    { 0x9db66aca, 61 },
    { 0, 0 },
    { 0x42e66acc, 36 },
    { 0x59e66acd, 43 },
    { 0x1a266ace, 55 },
    { 0x63b66acf, 23 },
    { 0x2666acf, 46 },
    { 0xffb66ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1366ad4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa066ad8, 61 },
    { 0x7e666ad9, 98 },
    { 0, 0 },
    { 0xc0866adb, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x72966ade, 23 },
    { 0, 0 },
    { 0x58c66ae0, 46 },
    { 0, 0 },
    { 0x5ab66ae2, 98 },
    { 0x75766ae2, 55 },
    { 0x2ff66ae3, 55 },
    { 0xe8266ae4, 98 },
    { 0xf6d66ae6, 33 },
    { 0xc2766ae2, 99 },
    { 0x31066ae3, 90 },
    { 0x6a366ae9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50666aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74766af7, 8 },
    { 0, 0 },
    { 0x2d366af9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc66aff, 87 },
    { 0x99f66aff, 87 },
    { 0, 0 },
    { 0x96a66b02, 61 },
    { 0xbe266b03, 93 },
    { 0, 0 },
    { 0x43266b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a66b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83566b0b, 34 },
    { 0x4e466b0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b366b18, 23 },
    { 0xd0b66b18, 23 },
    { 0, 0 },
    { 0x9fc66b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf566b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81466b25, 66 },
    { 0xf5f66b26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62466b2b, 92 },
    { 0xb8666b2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d66b31, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b66b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b66b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b66b3f, 96 },
    { 0x72e66b40, 33 },
    { 0, 0 },
    { 0x58c66b42, 33 },
    { 0x23566b43, 87 },
    { 0x2b466b43, 75 },
    { 0x6e866b45, 76 },
    { 0xac666b46, 90 },
    { 0xece66b45, 90 },
    { 0x86d66b48, 33 },
    { 0xf6666b46, 78 },
    { 0x74366b4a, 23 },
    { 0x50a66b48, 98 },
    { 0x16d66b4c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73966b52, 23 },
    { 0xa7466b52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf66b5b, 99 },
    { 0x70966b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b64, 90 },
    { 0, 0 },
    { 0xfde66b66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9366b69, 98 },
    { 0x3e166b69, 66 },
    { 0xca466b6b, 93 },
    { 0xe366b6c, 90 },
    { 0x66b66b6b, 90 },
    { 0xeb766b6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a66b72, 96 },
    { 0xf466b72, 95 },
    { 0x10266b74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea66b78, 24 },
    { 0, 0 },
    { 0xc5466b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba466b7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66b82, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b566b89, 84 },
    { 0x48b66b89, 95 },
    { 0, 0 },
    { 0x79466b8c, 61 },
    { 0x6cf66b8c, 94 },
    { 0, 0 },
    { 0x21066b8f, 74 },
    { 0x7e266b90, 61 },
    { 0xbf66b91, 66 },
    { 0, 0 },
    { 0x4ec66b93, 77 },
    { 0xe2066b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff66b9a, 66 },
    { 0x77466b9b, 93 },
    { 0, 0 },
    { 0x67166b9d, 97 },
    { 0x7bc66b9e, 55 },
    { 0, 0 },
    { 0xbf566ba0, 95 },
    { 0x65c66ba1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8166ba4, 76 },
    { 0x8e066ba5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b466baa, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa766baf, 95 },
    { 0, 0 },
    { 0x4ae66bb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83366bb8, 78 },
    { 0x9cc66bb8, 23 },
    { 0x52f66bb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a066bbe, 93 },
    { 0xd6866bbe, 84 },
    { 0xc9d66bc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81466bc6, 91 },
    { 0x43566bc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f66bcb, 33 },
    { 0x84e66bcc, 90 },
    { 0x12566bcc, 98 },
    { 0xfc066bce, 33 },
    { 0x90c66bcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21966bd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a366bdb, 23 },
    { 0x8a166bdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47866bf0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca966bf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b866bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c366c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab66c08, 23 },
    { 0x26766c08, 61 },
    { 0x91a66c09, 67 },
    { 0x53766c0b, 98 },
    { 0x33466c0c, 23 },
    { 0, 0 },
    { 0x1a866c0e, 98 },
    { 0, 0 },
    { 0xf4166c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62666c13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb266c17, 76 },
    { 0x31866c18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d66c1b, 76 },
    { 0xa0166c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8366c22, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee66c26, 90 },
    { 0x5266c27, 90 },
    { 0x6c466c28, 76 },
    { 0, 0 },
    { 0xc9f66c2a, 84 },
    { 0xa866c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf066c2e, 61 },
    { 0xfd766c2f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60966c34, 33 },
    { 0x75766c35, 90 },
    { 0, 0 },
    { 0x25066c37, 98 },
    { 0x91166c37, 68 },
    { 0xa9a66c39, 95 },
    { 0xf4b66c39, 19 },
    { 0x36366c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5666c3e, 17 },
    { 0x18966c3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58366c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3966c4e, 68 },
    { 0x75666c4e, 61 },
    { 0, 0 },
    { 0x7c966c51, 93 },
    { 0xde866c52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b66c55, 67 },
    { 0xbc566c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a66c59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd466c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb966c65, 78 },
    { 0, 0 },
    { 0xa1766c67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc366c72, 33 },
    { 0xf8566c73, 76 },
    { 0x5c266c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28466c7a, 78 },
    { 0x57966c7b, 76 },
    { 0x84b66c7c, 23 },
    { 0x95366c7b, 23 },
    { 0x3a866c7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81866c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b566c87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd66c8b, 100 },
    { 0xef266c8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b66c94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xec366c97, 55 },
    { 0x4b566c97, 61 },
    { 0x54166c99, 23 },
    { 0, 0 },
    { 0xec566c9b, 101 },
    { 0xdc966c9b, 55 },
    { 0x9e166c9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c66ca1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa366ca8, 95 },
    { 0x4ff66ca9, 91 },
    { 0x30f66ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1466cad, 78 },
    { 0, 0 },
    { 0x8c666caf, 34 },
    { 0x4fb66caf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92666cb4, 78 },
    { 0x3fa66cb5, 99 },
    { 0, 0 },
    { 0xe7366cb7, 33 },
    { 0x1cd66cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b66cbb, 87 },
    { 0x79366cbc, 64 },
    { 0x87866cbd, 34 },
    { 0, 0 },
    { 0x5a466cbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e66cc2, 17 },
    { 0, 0 },
    { 0xbbb66cc4, 95 },
    { 0xcfe66cc5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd766cc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65566ccd, 88 },
    { 0xf1266ccd, 40 },
    { 0xf0866cce, 95 },
    { 0, 0 },
    { 0x28c66cd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c66cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2066ce1, 33 },
    { 0, 0 },
    { 0x84066ce3, 33 },
    { 0x98266ce4, 52 },
    { 0x35266ce5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff766ce9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a566cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb766cf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e66cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d66cfe, 43 },
    { 0x18c66cff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9266d04, 23 },
    { 0x83d66d05, 98 },
    { 0x2b766d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b866d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f66d0f, 100 },
    { 0xd3766d10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1666d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca66d17, 90 },
    { 0x26566d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8166d1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67866d23, 77 },
    { 0x2c966d23, 91 },
    { 0xaae66d25, 46 },
    { 0x3a066d23, 100 },
    { 0x61766d24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55566d2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac466d3f, 90 },
    { 0xc8466d40, 100 },
    { 0x50666d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b766d45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f366d4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a66d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff66d59, 93 },
    { 0x1b166d5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58166d5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d866d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb566d65, 43 },
    { 0, 0 },
    { 0x3f566d67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9966d6a, 61 },
    { 0x52a66d6b, 76 },
    { 0, 0 },
    { 0xf6466d6d, 76 },
    { 0x46466d6d, 98 },
    { 0, 0 },
    { 0x4c666d70, 33 },
    { 0xbfc66d70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2666d77, 96 },
    { 0x41f66d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15466d81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37666d88, 61 },
    { 0x2a766d88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b66d91, 95 },
    { 0x56a66d92, 90 },
    { 0x40066d92, 33 },
    { 0, 0 },
    { 0xb1e66d95, 33 },
    { 0, 0 },
    { 0xb5066d97, 84 },
    { 0xcbf66d97, 43 },
    { 0x19d66d97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31466d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d66da3, 46 },
    { 0, 0 },
    { 0xb1366da5, 90 },
    { 0xf5d66da6, 90 },
    { 0x84e66da5, 12 },
    { 0, 0 },
    { 0xa7f66da9, 93 },
    { 0x45a66da9, 96 },
    { 0, 0 },
    { 0x9f066dac, 33 },
    { 0, 0 },
    { 0x30666dae, 33 },
    { 0xb8e66dae, 95 },
    { 0x38066db0, 96 },
    { 0, 0 },
    { 0x38d66db2, 98 },
    { 0xcbc66db2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90566dbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b166dbe, 61 },
    { 0, 0 },
    { 0xa7666dc0, 61 },
    { 0xb6566dc0, 23 },
    { 0, 0 },
    { 0x9b166dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f66dc9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x90666dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b66dcf, 34 },
    { 0x98966dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f66dd3, 61 },
    { 0xf4f66dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e66de1, 93 },
    { 0x6cb66de2, 95 },
    { 0x1d166de3, 90 },
    { 0x89c66de3, 34 },
    { 0xf0766de2, 61 },
    { 0x8a366de6, 23 },
    { 0, 0 },
    { 0x6c666de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb66deb, 76 },
    { 0x9e566deb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa466def, 98 },
    { 0, 0 },
    { 0x65466df1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c266df6, 84 },
    { 0x6d66df7, 78 },
    { 0x52e66df8, 43 },
    { 0x8b866df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f066dff, 40 },
    { 0xb3e66e00, 95 },
    { 0x1fc66e01, 90 },
    { 0x6de66e00, 76 },
    { 0x27b66e01, 98 },
    { 0, 0 },
    { 0x2e066e05, 98 },
    { 0x7bf66e06, 66 },
    { 0x2ed66e06, 17 },
    { 0, 0 },
    { 0x59966e09, 98 },
    { 0xc6566e0a, 60 },
    { 0xaf866e0b, 23 },
    { 0, 0 },
    { 0x9c366e0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa66e12, 83 },
    { 0x31d66e13, 90 },
    { 0xe1166e14, 90 },
    { 0xaf66e13, 61 },
    { 0x95d66e14, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95166e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61066e27, 84 },
    { 0xe0e66e27, 23 },
    { 0x50f66e27, 90 },
    { 0xde766e27, 80 },
    { 0xd3466e2b, 34 },
    { 0xa4f66e2b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d66e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46566e33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96066e36, 95 },
    { 0, 0 },
    { 0xfb166e38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf066e3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3466e3e, 40 },
    { 0x46766e3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9766e42, 96 },
    { 0, 0 },
    { 0x1166e44, 76 },
    { 0x24366e45, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a66e4b, 61 },
    { 0x2b66e4b, 46 },
    { 0, 0 },
    { 0x5e766e4e, 55 },
    { 0x76f66e4f, 23 },
    { 0x69166e4f, 33 },
    { 0xa6766e51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a66e54, 33 },
    { 0, 0 },
    { 0xffd66e56, 66 },
    { 0, 0 },
    { 0x2a866e58, 98 },
    { 0x4c766e59, 91 },
    { 0xdff66e5a, 80 },
    { 0x55a66e59, 81 },
    { 0, 0 },
    { 0x9ab66e5d, 99 },
    { 0x94b66e5d, 95 },
    { 0, 0 },
    { 0x65266e60, 77 },
    { 0x8ba66e60, 91 },
    { 0x3ec66e62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a66e65, 55 },
    { 0xbcd66e65, 90 },
    { 0x36d66e67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a66e6a, 90 },
    { 0, 0 },
    { 0xe4c66e6c, 93 },
    { 0x20966e6c, 75 },
    { 0x62666e6e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b766e71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc66e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5566e79, 80 },
    { 0x8b066e7a, 17 },
    { 0xe2366e7b, 90 },
    { 0xfe666e7c, 100 },
    { 0, 0 },
    { 0xf8366e7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4366e82, 78 },
    { 0, 0 },
    { 0x72866e84, 33 },
    { 0x70166e84, 88 },
    { 0x56e66e86, 33 },
    { 0x5ea66e87, 90 },
    { 0x68566e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb866e8e, 90 },
    { 0x8f966e8e, 90 },
    { 0, 0 },
    { 0x44c66e91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31266e94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1166e98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36866e9e, 95 },
    { 0xedc66e9f, 61 },
    { 0x2fc66e9e, 33 },
    { 0x15566ea1, 90 },
    { 0, 0 },
    { 0x3e166ea3, 61 },
    { 0x76466ea4, 55 },
    { 0, 0 },
    { 0x55866ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d66ea9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f66eac, 95 },
    { 0x40466eac, 85 },
    { 0x1c066eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9966eb4, 98 },
    { 0xa6266eb5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5066ebd, 54 },
    { 0, 0 },
    { 0x9a866ebf, 46 },
    { 0x11c66ec0, 33 },
    { 0, 0 },
    { 0x8f966ec2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac66ec5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0066eca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e66ecd, 76 },
    { 0, 0 },
    { 0xd3266ecf, 90 },
    { 0x48166ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23066ed4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d66ed8, 34 },
    { 0, 0 },
    { 0x77766eda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34466ee5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f66eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc66eed, 90 },
    { 0x67c66eee, 61 },
    { 0x83866eee, 46 },
    { 0xc3266eef, 61 },
    { 0xe4566eed, 95 },
    { 0x4c466ef2, 94 },
    { 0x55366ef3, 33 },
    { 0xa9c66ef2, 93 },
    { 0, 0 },
    { 0xb1766ef6, 23 },
    { 0x9b266ef7, 33 },
    { 0x38d66ef7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf566efe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f66f01, 61 },
    { 0, 0 },
    { 0x1e066f03, 33 },
    { 0xc2566f04, 98 },
    { 0, 0 },
    { 0x80e66f06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5966f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c66f16, 23 },
    { 0x18966f16, 100 },
    { 0x91e66f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d366f1c, 61 },
    { 0xdd566f1d, 77 },
    { 0x1cc66f1e, 55 },
    { 0x7fe66f1f, 95 },
    { 0xc1d66f20, 61 },
    { 0, 0 },
    { 0xf1466f22, 90 },
    { 0x97366f22, 95 },
    { 0xd3466f22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa566f2c, 33 },
    { 0xa366f2c, 33 },
    { 0, 0 },
    { 0x6f566f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f66f39, 66 },
    { 0, 0 },
    { 0xd5c66f3b, 33 },
    { 0x8166f3c, 90 },
    { 0xed566f3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f43, 90 },
    { 0x49566f43, 34 },
    { 0x63166f45, 98 },
    { 0x2266f46, 43 },
    { 0x3f266f46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d66f55, 61 },
    { 0x20a66f56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57166f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32466f5c, 90 },
    { 0xf2b66f5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef166f60, 95 },
    { 0xc3866f61, 98 },
    { 0, 0 },
    { 0x4e066f63, 33 },
    { 0x4e366f63, 90 },
    { 0x20966f65, 42 },
    { 0x67666f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2666f6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46066f70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc266f74, 33 },
    { 0, 0 },
    { 0xbc366f76, 43 },
    { 0xf1b66f77, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65766f7e, 90 },
    { 0, 0 },
    { 0x3866f80, 55 },
    { 0xc3166f81, 100 },
    { 0xda066f81, 23 },
    { 0x90e66f83, 33 },
    { 0x50066f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff66f89, 43 },
    { 0, 0 },
    { 0xe1566f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39266f95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d66f98, 90 },
    { 0x68e66f99, 61 },
    { 0x97f66f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5666fa1, 76 },
    { 0x49966fa1, 83 },
    { 0xb3566fa3, 95 },
    { 0xd166fa4, 98 },
    { 0x1ee66fa5, 93 },
    { 0xf0a66fa4, 100 },
    { 0x34a66fa7, 46 },
    { 0, 0 },
    { 0x59666fa9, 98 },
    { 0x27866faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85466fb2, 85 },
    { 0x40966fb2, 55 },
    { 0, 0 },
    { 0x22566fb5, 61 },
    { 0x39466fb5, 33 },
    { 0x15066fb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6466fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd66fbe, 23 },
    { 0x11666fbe, 43 },
    { 0x8c666fbe, 61 },
    { 0xcb666fc1, 61 },
    { 0, 0 },
    { 0x2b366fc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf66fc6, 76 },
    { 0, 0 },
    { 0x6ac66fc8, 33 },
    { 0xd0b66fc9, 90 },
    { 0x9e366fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4566fcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a66fd5, 95 },
    { 0x6b266fd6, 46 },
    { 0x78666fd7, 33 },
    { 0x4c266fd8, 98 },
    { 0xd0b66fd6, 33 },
    { 0x1d066fd5, 55 },
    { 0x1466fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a66fdf, 93 },
    { 0xa1266fe0, 87 },
    { 0, 0 },
    { 0x78966fe2, 85 },
    { 0x65666fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c66fe6, 43 },
    { 0xca466fe6, 80 },
    { 0xf8166fe6, 76 },
    { 0, 0 },
    { 0x3eb66fea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1866fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66ff6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b366ffc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8466fff, 55 },
    { 0, 0 },
    { 0xfaf67001, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x17867004, 76 },
    { 0, 0 },
    { 0x1c667006, 78 },
    { 0xf5767006, 76 },
    { 0xca267008, 55 },
    { 0x6467008, 23 },
    { 0, 0 },
    { 0xddc6700b, 55 },
    { 0, 0 },
    { 0xeb66700d, 80 },
    { 0x1df6700e, 23 },
    { 0x3206700d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f767012, 76 },
    { 0x57d67012, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06701a, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c36701e, 33 },
    { 0x5076701e, 33 },
    { 0, 0 },
    { 0x60667021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9267029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a6702c, 98 },
    { 0x7266702d, 23 },
    { 0x2b46702d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b67031, 93 },
    { 0x8cc67032, 43 },
    { 0, 0 },
    { 0xfba67034, 48 },
    { 0x9dd67034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026703b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e67049, 95 },
    { 0, 0 },
    { 0x3996704b, 93 },
    { 0x80f6704c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde467057, 46 },
    { 0xeca67058, 99 },
    { 0x97a67059, 61 },
    { 0, 0 },
    { 0x6346705b, 76 },
    { 0x91d6705c, 93 },
    { 0x5686705d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a67060, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18667067, 76 },
    { 0x2467068, 53 },
    { 0x25667068, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec6706c, 36 },
    { 0xb456706d, 55 },
    { 0x5426706c, 65 },
    { 0, 0 },
    { 0x11367070, 23 },
    { 0xc0367070, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e367077, 52 },
    { 0x93667077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87067080, 99 },
    { 0, 0 },
    { 0xc9867082, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e67086, 23 },
    { 0xb9167087, 64 },
    { 0x7df67086, 34 },
    { 0xd867089, 65 },
    { 0x55167086, 52 },
    { 0x4926708b, 23 },
    { 0x99a6708b, 61 },
    { 0xd576708c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xae667090, 23 },
    { 0xa0667090, 23 },
    { 0xa1067092, 76 },
    { 0xd3d67090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4106709b, 33 },
    { 0x7f6709c, 98 },
    { 0x4c76709d, 90 },
    { 0, 0 },
    { 0x9486709f, 95 },
    { 0x97a670a0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63670a4, 90 },
    { 0, 0 },
    { 0x61f670a6, 93 },
    { 0, 0 },
    { 0x44b670a8, 61 },
    { 0x2e8670a8, 98 },
    { 0xac1670aa, 98 },
    { 0x219670a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f670af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701670b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x614670b8, 61 },
    { 0, 0 },
    { 0xce8670ba, 63 },
    { 0x5ce670ba, 33 },
    { 0xe1d670bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x337670bf, 90 },
    { 0x995670bf, 98 },
    { 0x145670c1, 68 },
    { 0x61f670c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617670c6, 93 },
    { 0xec7670c6, 61 },
    { 0x2c4670c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500670d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba670d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2670d9, 61 },
    { 0, 0 },
    { 0xef3670db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264670e2, 90 },
    { 0xeb0670e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807670e7, 43 },
    { 0x8ee670e8, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941670ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165670f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e670fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4767100, 98 },
    { 0xdfe67100, 65 },
    { 0xe2367102, 56 },
    { 0xd3467102, 98 },
    { 0, 0 },
    { 0x43e67105, 95 },
    { 0x8ef67106, 43 },
    { 0x73667106, 61 },
    { 0xfd167108, 91 },
    { 0, 0 },
    { 0xab6710a, 99 },
    { 0x8296710b, 46 },
    { 0xcae6710b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9967116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d6711e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4267121, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376712d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90967135, 90 },
    { 0, 0 },
    { 0x3a767137, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa6713a, 95 },
    { 0xa856713b, 94 },
    { 0x9046713c, 61 },
    { 0x64e6713b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a67142, 95 },
    { 0x5de67143, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae67148, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96714c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23467150, 61 },
    { 0, 0 },
    { 0xa2967152, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf66715c, 43 },
    { 0xbe16715c, 55 },
    { 0xf66715c, 34 },
    { 0x80f6715f, 98 },
    { 0x4cf67160, 46 },
    { 0xdac67160, 98 },
    { 0xa876715f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80767166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c6716a, 90 },
    { 0, 0 },
    { 0x5c36716c, 62 },
    { 0xf1d6716c, 33 },
    { 0xc206716d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32967172, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f267176, 95 },
    { 0, 0 },
    { 0x1c67178, 61 },
    { 0x25567178, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b66717c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6717f, 66 },
    { 0, 0 },
    { 0x64867181, 61 },
    { 0x18c67182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e167187, 101 },
    { 0xd6367188, 90 },
    { 0, 0 },
    { 0x8c46718a, 77 },
    { 0xfa86718b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c867191, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac567194, 33 },
    { 0x7b767194, 90 },
    { 0x93a67196, 46 },
    { 0x15367196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da6719a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5806719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5671a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x186671a9, 52 },
    { 0xcd1671aa, 76 },
    { 0x4bd671a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c671b7, 9 },
    { 0, 0 },
    { 0x170671b9, 91 },
    { 0xc7b671ba, 33 },
    { 0x760671bb, 34 },
    { 0xfec671bc, 95 },
    { 0xf60671bd, 46 },
    { 0, 0 },
    { 0x3cf671bf, 98 },
    { 0x88c671bf, 61 },
    { 0x7d1671bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a671c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d671c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147671cc, 61 },
    { 0, 0 },
    { 0x662671ce, 90 },
    { 0x74c671ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f671d4, 23 },
    { 0, 0 },
    { 0x4f1671d6, 61 },
    { 0xb49671d6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302671dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764671e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x804671e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191671e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8671eb, 61 },
    { 0x76b671ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04671f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d671f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6671fa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc67203, 34 },
    { 0, 0 },
    { 0x26767205, 65 },
    { 0, 0 },
    { 0x5dc67207, 93 },
    { 0xcf267208, 23 },
    { 0xe4067209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e96720d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7267210, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24267219, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176721c, 33 },
    { 0, 0 },
    { 0x5d36721e, 87 },
    { 0, 0 },
    { 0x86467220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed67223, 33 },
    { 0, 0 },
    { 0x89a67225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6256722c, 91 },
    { 0xe4d6722d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7467230, 98 },
    { 0xc0967231, 34 },
    { 0, 0 },
    { 0x81267233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6723b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf86723e, 91 },
    { 0x5636723f, 90 },
    { 0x4406723f, 98 },
    { 0, 0 },
    { 0x22267242, 90 },
    { 0x7f267243, 33 },
    { 0x92667243, 43 },
    { 0, 0 },
    { 0xb9667246, 55 },
    { 0x2f67247, 90 },
    { 0xb5a67248, 76 },
    { 0x5a267249, 90 },
    { 0xa367248, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e6724f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23267256, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c067259, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30367268, 98 },
    { 0xd4267268, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6726c, 76 },
    { 0xcf66726d, 33 },
    { 0, 0 },
    { 0x8ff6726f, 98 },
    { 0, 0 },
    { 0xc2367271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6067274, 76 },
    { 0x9a367275, 81 },
    { 0, 0 },
    { 0x8ab67277, 33 },
    { 0, 0 },
    { 0xfe167279, 93 },
    { 0xd196727a, 98 },
    { 0, 0 },
    { 0x3ba6727c, 61 },
    { 0xb1e6727d, 93 },
    { 0, 0 },
    { 0x8dd6727f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc867282, 76 },
    { 0xf6267283, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566728b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1256728e, 46 },
    { 0xceb6728f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb367294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e66729f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576672a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c672ab, 98 },
    { 0x787672ab, 33 },
    { 0x27c672ad, 90 },
    { 0xc4a672ae, 61 },
    { 0x10c672af, 23 },
    { 0, 0 },
    { 0x645672b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8672b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2672bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x950672c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399672c4, 33 },
    { 0x2e4672c5, 23 },
    { 0, 0 },
    { 0x8e1672c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b672ca, 33 },
    { 0x7da672cb, 90 },
    { 0, 0 },
    { 0xded672cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59672d6, 43 },
    { 0, 0 },
    { 0x6cf672d8, 33 },
    { 0, 0 },
    { 0xcde672da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe672df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708672e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03672e9, 90 },
    { 0x51f672ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de672ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac672f4, 90 },
    { 0xd45672f5, 84 },
    { 0, 0 },
    { 0xf6672f7, 52 },
    { 0xe85672f7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb672fc, 95 },
    { 0x38e672fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2367300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38967305, 90 },
    { 0xc5d67305, 55 },
    { 0x8a367307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6730a, 66 },
    { 0x256730a, 80 },
    { 0x696730c, 66 },
    { 0xa76730d, 23 },
    { 0x2706730d, 76 },
    { 0, 0 },
    { 0x4267310, 96 },
    { 0x49e67311, 100 },
    { 0xe2967311, 46 },
    { 0xf3267313, 67 },
    { 0xe3e67311, 78 },
    { 0xe5067313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc6731a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81567320, 94 },
    { 0, 0 },
    { 0x77667322, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb86732d, 90 },
    { 0xd726732e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x98067331, 23 },
    { 0, 0 },
    { 0xe6867333, 69 },
    { 0, 0 },
    { 0x45f67335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7376733b, 61 },
    { 0, 0 },
    { 0x9ee6733d, 90 },
    { 0xf466733e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef067341, 90 },
    { 0xda67342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d67345, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be67349, 24 },
    { 0x5516734a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa56734d, 95 },
    { 0x1df6734e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b367352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81567359, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21867360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78267364, 43 },
    { 0x8a167365, 66 },
    { 0xf467366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe767369, 90 },
    { 0xf5e6736a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2836736d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b67375, 61 },
    { 0, 0 },
    { 0x7267377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e6737a, 23 },
    { 0xb536737a, 33 },
    { 0xcd16737c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56737f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90667384, 98 },
    { 0, 0 },
    { 0x27567386, 33 },
    { 0xeb167387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816738b, 93 },
    { 0, 0 },
    { 0xc666738d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7867390, 23 },
    { 0x77567391, 33 },
    { 0x94467392, 90 },
    { 0, 0 },
    { 0x8bb67394, 61 },
    { 0x48c67395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9536739a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436739d, 23 },
    { 0xa606739d, 98 },
    { 0xadb6739f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x609673a2, 90 },
    { 0xd53673a3, 55 },
    { 0x99d673a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45673a8, 95 },
    { 0, 0 },
    { 0xb8d673aa, 67 },
    { 0x739673ab, 68 },
    { 0x950673ab, 34 },
    { 0x624673aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d673b2, 99 },
    { 0xbf0673b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2673b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47673be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723673c2, 17 },
    { 0x8b9673c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9673cc, 66 },
    { 0x7b7673cc, 95 },
    { 0x623673cd, 98 },
    { 0xe39673cf, 100 },
    { 0, 0 },
    { 0x4ac673d1, 34 },
    { 0x9b7673d2, 93 },
    { 0, 0 },
    { 0xdea673d4, 76 },
    { 0, 0 },
    { 0xf9673d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5673db, 34 },
    { 0, 0 },
    { 0x719673dd, 87 },
    { 0x745673de, 23 },
    { 0x51673df, 90 },
    { 0x99673dd, 43 },
    { 0xc36673df, 46 },
    { 0, 0 },
    { 0x9a673e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75673e8, 90 },
    { 0xcb8673e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80673ec, 77 },
    { 0, 0 },
    { 0x949673ee, 43 },
    { 0x3de673ef, 95 },
    { 0, 0 },
    { 0x96673f1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726673f9, 61 },
    { 0x742673f9, 23 },
    { 0x828673fb, 95 },
    { 0, 0 },
    { 0x2e3673fd, 98 },
    { 0xce1673fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x84167401, 77 },
    { 0xbc67401, 86 },
    { 0, 0 },
    { 0xedd67404, 87 },
    { 0x69a67404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f67409, 100 },
    { 0x92a6740a, 95 },
    { 0x69667409, 55 },
    { 0x9396740c, 43 },
    { 0, 0 },
    { 0x2346740e, 78 },
    { 0x8e46740f, 95 },
    { 0x81b6740f, 66 },
    { 0x69967411, 90 },
    { 0xc7767412, 78 },
    { 0xebe67412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb67418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef26741b, 76 },
    { 0xbec6741c, 81 },
    { 0x4516741c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f567420, 61 },
    { 0x19367421, 55 },
    { 0xb7967421, 98 },
    { 0x25667421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab67426, 61 },
    { 0, 0 },
    { 0xabb67428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d76742c, 23 },
    { 0xc576742c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11667431, 33 },
    { 0, 0 },
    { 0xe9367433, 23 },
    { 0x44967434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73267439, 55 },
    { 0xe5f67439, 66 },
    { 0x87a6743b, 78 },
    { 0x4846743c, 87 },
    { 0xe246743d, 23 },
    { 0x3356743d, 33 },
    { 0xc2b6743c, 85 },
    { 0, 0 },
    { 0x36c67441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63767444, 61 },
    { 0xa7567445, 99 },
    { 0xb3067446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e56744c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90767452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16967459, 99 },
    { 0xecd6745a, 23 },
    { 0, 0 },
    { 0x1ab6745c, 91 },
    { 0x9956745d, 76 },
    { 0xff56745c, 61 },
    { 0, 0 },
    { 0x9c667460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba67467, 90 },
    { 0xf5467468, 98 },
    { 0, 0 },
    { 0x6916746a, 55 },
    { 0x5566746b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6746f, 90 },
    { 0, 0 },
    { 0x51a67471, 97 },
    { 0, 0 },
    { 0xfaa67473, 76 },
    { 0xd4967473, 95 },
    { 0, 0 },
    { 0x46067476, 81 },
    { 0xd1d67476, 99 },
    { 0, 0 },
    { 0xe0e67479, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c06747c, 78 },
    { 0x7056747d, 88 },
    { 0, 0 },
    { 0x246747f, 61 },
    { 0xb9867480, 61 },
    { 0, 0 },
    { 0xd7e67482, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73467486, 98 },
    { 0xa3067486, 90 },
    { 0xdad67488, 90 },
    { 0x267488, 43 },
    { 0xb9667487, 33 },
    { 0xe7e6748b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9566748f, 90 },
    { 0x13c6748f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c67496, 99 },
    { 0x53a67497, 34 },
    { 0xa267498, 46 },
    { 0x9967499, 67 },
    { 0x1d467499, 43 },
    { 0xfab6749b, 76 },
    { 0x2516749b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b674a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d674a4, 61 },
    { 0x19e674a4, 100 },
    { 0x931674a6, 37 },
    { 0xd6a674a6, 34 },
    { 0xcf8674a7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43674ac, 52 },
    { 0, 0 },
    { 0x9fe674ae, 61 },
    { 0xd84674ae, 23 },
    { 0xf5b674b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25674b3, 67 },
    { 0x34b674b4, 55 },
    { 0x211674b5, 17 },
    { 0xd83674b5, 90 },
    { 0, 0 },
    { 0xbb7674b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6674bb, 90 },
    { 0x6d1674bc, 88 },
    { 0xb6f674bd, 90 },
    { 0x3c1674be, 53 },
    { 0x57e674bd, 55 },
    { 0xa8f674c0, 33 },
    { 0x533674c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68674c6, 87 },
    { 0, 0 },
    { 0x743674c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b674cf, 33 },
    { 0x3ab674d0, 34 },
    { 0x985674d1, 90 },
    { 0xdf5674d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5674d5, 23 },
    { 0x384674d6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1674da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78674dd, 95 },
    { 0, 0 },
    { 0xd08674df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42674e4, 78 },
    { 0x36f674e5, 55 },
    { 0xce3674e6, 100 },
    { 0x445674e7, 95 },
    { 0xca674e8, 61 },
    { 0x798674e4, 33 },
    { 0x48b674e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2674f1, 46 },
    { 0xa2674f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46674f7, 85 },
    { 0, 0 },
    { 0x8d6674f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526674ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5467502, 52 },
    { 0x63e67503, 67 },
    { 0, 0 },
    { 0x9f067505, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd886750a, 77 },
    { 0, 0 },
    { 0x1c36750c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb36750f, 90 },
    { 0xef867510, 76 },
    { 0xb6467511, 52 },
    { 0x71767512, 84 },
    { 0xea367513, 85 },
    { 0xd7767513, 55 },
    { 0xbc167515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16667519, 99 },
    { 0x6f66751a, 77 },
    { 0xb5d6751a, 75 },
    { 0x70c6751b, 61 },
    { 0x3486751d, 68 },
    { 0x8ce6751d, 98 },
    { 0x6c56751e, 98 },
    { 0x66a67520, 61 },
    { 0, 0 },
    { 0x71b67522, 100 },
    { 0x12067522, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27467526, 75 },
    { 0x72267527, 33 },
    { 0x74b67527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d96752e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b67534, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb67537, 33 },
    { 0x43f67538, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346753c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e167541, 76 },
    { 0, 0 },
    { 0xc1167543, 90 },
    { 0x77267544, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd67554, 33 },
    { 0xd6e67555, 23 },
    { 0x51367556, 23 },
    { 0xe5e67557, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1836755f, 35 },
    { 0x15367560, 90 },
    { 0, 0 },
    { 0xca967562, 90 },
    { 0xc1c67562, 90 },
    { 0x48167564, 61 },
    { 0x52067565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f6756b, 100 },
    { 0, 0 },
    { 0x7ab6756d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0467572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c67576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386757a, 87 },
    { 0, 0 },
    { 0x8d36757c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab767585, 77 },
    { 0, 0 },
    { 0x98767587, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf6758a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2967592, 33 },
    { 0x34667593, 90 },
    { 0x41f67594, 43 },
    { 0xec67595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83867598, 55 },
    { 0x83667599, 90 },
    { 0xf896759a, 34 },
    { 0xbb56759a, 90 },
    { 0, 0 },
    { 0x1bb6759d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a675a4, 95 },
    { 0x1eb675a5, 90 },
    { 0xc7e675a6, 99 },
    { 0x131675a7, 46 },
    { 0xbf1675a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d675ac, 93 },
    { 0xf84675ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69675b3, 88 },
    { 0, 0 },
    { 0x505675b5, 99 },
    { 0xb5675b6, 90 },
    { 0, 0 },
    { 0xd83675b8, 23 },
    { 0xe1b675b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306675bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85675c0, 90 },
    { 0x5d9675c1, 75 },
    { 0x2f8675c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1675ca, 75 },
    { 0x497675ca, 75 },
    { 0xf9d675cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743675d1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0675d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0675de, 43 },
    { 0, 0 },
    { 0xff1675e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c675e9, 76 },
    { 0x141675ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x817675ed, 76 },
    { 0, 0 },
    { 0x6dc675ef, 95 },
    { 0x92f675f0, 94 },
    { 0, 0 },
    { 0xc0e675f2, 43 },
    { 0, 0 },
    { 0x3c9675f4, 84 },
    { 0, 0 },
    { 0xa28675f6, 61 },
    { 0x740675f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410675fe, 33 },
    { 0, 0 },
    { 0xb0167600, 19 },
    { 0, 0 },
    { 0xe2867602, 55 },
    { 0xfbd67603, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c67608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd6760b, 87 },
    { 0xd5e6760c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b67610, 61 },
    { 0x18f67611, 66 },
    { 0x8fc67610, 23 },
    { 0x10767612, 90 },
    { 0x99667614, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47067621, 98 },
    { 0x85967622, 95 },
    { 0x2967623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267629, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d16762d, 33 },
    { 0, 0 },
    { 0x5066762f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e467634, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4716763b, 34 },
    { 0x5d96763c, 87 },
    { 0x6ff6763c, 33 },
    { 0x56a6763b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96567641, 34 },
    { 0xf1167642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f67647, 53 },
    { 0xc4567648, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd556764b, 33 },
    { 0xc0a6764b, 8 },
    { 0xdb86764d, 23 },
    { 0x176764b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a67652, 90 },
    { 0, 0 },
    { 0x7e567654, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee67658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f36765b, 61 },
    { 0xe006765b, 23 },
    { 0x5266765d, 93 },
    { 0xd2a6765c, 76 },
    { 0x5d16765f, 78 },
    { 0xa486765f, 61 },
    { 0x2636765c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced6766a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6766e, 90 },
    { 0x1eb6766e, 98 },
    { 0xc7f6766f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4267675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5336767a, 92 },
    { 0xc776767b, 87 },
    { 0x4066767c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe906767f, 33 },
    { 0, 0 },
    { 0xd4667681, 33 },
    { 0x4d767681, 52 },
    { 0x89667681, 33 },
    { 0, 0 },
    { 0x56167685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82867689, 90 },
    { 0x96a67689, 33 },
    { 0x66768a, 33 },
    { 0, 0 },
    { 0xd1b6768d, 95 },
    { 0x1226768e, 81 },
    { 0, 0 },
    { 0xb9367690, 17 },
    { 0x36f67691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd667698, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995676a2, 46 },
    { 0x496676a3, 95 },
    { 0xd9d676a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730676a8, 55 },
    { 0x414676a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522676ad, 61 },
    { 0, 0 },
    { 0x39f676af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a676b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554676b8, 78 },
    { 0x264676b9, 65 },
    { 0x6b3676b9, 17 },
    { 0, 0 },
    { 0xf90676bc, 90 },
    { 0xf54676bd, 84 },
    { 0xc75676be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2676c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x833676c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x957676c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f676cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54676d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc676d4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a676e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353676e7, 76 },
    { 0, 0 },
    { 0xf85676e9, 38 },
    { 0x7dc676ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693676f0, 90 },
    { 0xb29676f0, 100 },
    { 0, 0 },
    { 0x659676f3, 52 },
    { 0xaf4676f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x830676f7, 61 },
    { 0xd7b676f7, 43 },
    { 0, 0 },
    { 0xb74676fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x347676fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44267703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92867707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c66770b, 43 },
    { 0x5fd6770c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb67712, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf367715, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5626771b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23167725, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26772c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f567731, 95 },
    { 0, 0 },
    { 0x25867733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b767737, 90 },
    { 0x9df67738, 76 },
    { 0x98b67739, 61 },
    { 0x7346773a, 34 },
    { 0xf826773a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6773e, 98 },
    { 0x9a26773e, 100 },
    { 0xa0b6773e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14867744, 23 },
    { 0xed267745, 53 },
    { 0xdda67746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f67749, 61 },
    { 0x3f267749, 76 },
    { 0xefb67749, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536774e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5267752, 67 },
    { 0, 0 },
    { 0x9867754, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d67758, 98 },
    { 0, 0 },
    { 0x4056775a, 98 },
    { 0x1fc6775b, 95 },
    { 0x2736775c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93467762, 61 },
    { 0, 0 },
    { 0xe7067764, 90 },
    { 0, 0 },
    { 0x7f767766, 43 },
    { 0, 0 },
    { 0xc5267768, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e46776b, 33 },
    { 0xcb6776c, 61 },
    { 0xc526776b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b67771, 98 },
    { 0x83c67771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6067775, 33 },
    { 0x8ca67776, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf767779, 99 },
    { 0xe6a6777a, 90 },
    { 0xa226777a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff67782, 65 },
    { 0, 0 },
    { 0xd4667784, 95 },
    { 0x60c67785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c67789, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266778f, 98 },
    { 0xb6b6778f, 93 },
    { 0x63267791, 61 },
    { 0xfb967790, 93 },
    { 0, 0 },
    { 0x37167794, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18767799, 76 },
    { 0x5e267799, 61 },
    { 0xaf567799, 91 },
    { 0x28d6779a, 87 },
    { 0x3f56779d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7677a0, 95 },
    { 0, 0 },
    { 0x127677a2, 61 },
    { 0xb06677a2, 90 },
    { 0x74c677a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b677a7, 90 },
    { 0x26c677a7, 93 },
    { 0, 0 },
    { 0x5cf677aa, 95 },
    { 0x712677aa, 61 },
    { 0, 0 },
    { 0x181677ad, 78 },
    { 0x5cf677ae, 95 },
    { 0x829677af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd677b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b677b5, 99 },
    { 0xed1677b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00677b9, 84 },
    { 0xec4677ba, 90 },
    { 0, 0 },
    { 0xedc677bc, 98 },
    { 0, 0 },
    { 0x82a677be, 99 },
    { 0, 0 },
    { 0x99e677c0, 98 },
    { 0xc9b677c1, 23 },
    { 0, 0 },
    { 0xc92677c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77677cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b677d3, 61 },
    { 0xcc2677d4, 98 },
    { 0x3a6677d5, 55 },
    { 0x5ba677d6, 95 },
    { 0x73d677d6, 43 },
    { 0x6bd677d8, 95 },
    { 0x996677d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8677e4, 93 },
    { 0xc31677e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3677e8, 90 },
    { 0xbe2677e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3677ec, 77 },
    { 0xb86677ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8677f1, 76 },
    { 0x82a677f2, 95 },
    { 0, 0 },
    { 0x698677f4, 90 },
    { 0x517677f5, 84 },
    { 0xb4b677f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85677fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72677fd, 53 },
    { 0xe5e677fe, 23 },
    { 0xdf5677fe, 76 },
    { 0, 0 },
    { 0xd0f67801, 98 },
    { 0x25a67802, 23 },
    { 0, 0 },
    { 0xb8867804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7086780e, 95 },
    { 0xf516780f, 98 },
    { 0x33667810, 34 },
    { 0x98a6780e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd67815, 43 },
    { 0, 0 },
    { 0xe5467817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae6781d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b67821, 90 },
    { 0x5c267822, 34 },
    { 0x48e67821, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb67827, 23 },
    { 0, 0 },
    { 0x85167829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2306782c, 81 },
    { 0xbcf6782c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a67830, 61 },
    { 0, 0 },
    { 0x2ab67832, 90 },
    { 0xb4067833, 23 },
    { 0, 0 },
    { 0x2c467835, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4476783f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f67846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9246784b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6784f, 76 },
    { 0, 0 },
    { 0x87767851, 48 },
    { 0x13067852, 87 },
    { 0xd0567852, 40 },
    { 0x2e367854, 75 },
    { 0xf1367855, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a867858, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c6785b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec167861, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81967864, 84 },
    { 0x65f67864, 77 },
    { 0, 0 },
    { 0x31067867, 84 },
    { 0, 0 },
    { 0x9ab67869, 93 },
    { 0, 0 },
    { 0x4e16786b, 55 },
    { 0x1c86786c, 33 },
    { 0xb866786d, 33 },
    { 0x5d26786d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b67880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15867883, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf416788b, 91 },
    { 0x776788b, 98 },
    { 0x9ca6788b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8767890, 77 },
    { 0x9b267890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d6789c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777678a2, 52 },
    { 0, 0 },
    { 0xf53678a4, 55 },
    { 0, 0 },
    { 0x91e678a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x713678a9, 43 },
    { 0xd87678aa, 17 },
    { 0x15e678ab, 77 },
    { 0xde0678ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f678b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb678b3, 61 },
    { 0xb1a678b3, 15 },
    { 0x7b678b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a678b8, 61 },
    { 0, 0 },
    { 0x4a8678ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d678bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5678c2, 61 },
    { 0, 0 },
    { 0xdcf678c4, 4 },
    { 0x71f678c5, 77 },
    { 0x65a678c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5678ca, 67 },
    { 0, 0 },
    { 0x4ca678cc, 61 },
    { 0xaab678cd, 55 },
    { 0x41d678ce, 100 },
    { 0xea678cf, 55 },
    { 0, 0 },
    { 0xa88678d1, 61 },
    { 0xed7678d2, 98 },
    { 0, 0 },
    { 0xd07678d4, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x674678d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14678da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23678e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a678e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09678e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12678e9, 34 },
    { 0x418678e9, 85 },
    { 0x4ac678e9, 88 },
    { 0xb3f678ec, 87 },
    { 0x685678ec, 76 },
    { 0xa05678ee, 66 },
    { 0xa71678ec, 78 },
    { 0x84f678f0, 90 },
    { 0x67d678ef, 90 },
    { 0, 0 },
    { 0x9df678f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2678f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce67901, 44 },
    { 0, 0 },
    { 0x7c167903, 95 },
    { 0x38867904, 76 },
    { 0, 0 },
    { 0x62267906, 23 },
    { 0x70867907, 61 },
    { 0x2f67908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6790b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86790f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14267926, 61 },
    { 0, 0 },
    { 0x4e867928, 93 },
    { 0x79567928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d86792c, 98 },
    { 0x7d96792d, 23 },
    { 0xefc6792d, 52 },
    { 0x1096792f, 95 },
    { 0, 0 },
    { 0x78d67931, 23 },
    { 0xaa067931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a67937, 76 },
    { 0x33867938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f6793c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a06793f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc67942, 93 },
    { 0x83867943, 33 },
    { 0x70e67944, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c67948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1726794b, 43 },
    { 0, 0 },
    { 0xc46794d, 33 },
    { 0, 0 },
    { 0xcb26794f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c67959, 33 },
    { 0xc056795a, 91 },
    { 0xf896795a, 96 },
    { 0xde96795c, 66 },
    { 0x3456795a, 33 },
    { 0xa586795e, 85 },
    { 0x8e96795f, 76 },
    { 0, 0 },
    { 0x20c67961, 76 },
    { 0, 0 },
    { 0x1fb67963, 90 },
    { 0x68567964, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a67967, 90 },
    { 0x30767968, 98 },
    { 0x58e67969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1867971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26797a, 23 },
    { 0xa5a6797a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60967988, 19 },
    { 0x79667988, 34 },
    { 0xaec6798a, 33 },
    { 0, 0 },
    { 0x2356798c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf567995, 61 },
    { 0xd6d67996, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2066799a, 98 },
    { 0x3c66799a, 90 },
    { 0, 0 },
    { 0xb406799d, 23 },
    { 0, 0 },
    { 0xd4e6799f, 33 },
    { 0x4d5679a0, 91 },
    { 0xd22679a1, 83 },
    { 0, 0 },
    { 0x875679a3, 90 },
    { 0x26679a3, 95 },
    { 0x6f679a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd679af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa679b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f679ba, 98 },
    { 0xee9679bb, 76 },
    { 0x89d679bb, 69 },
    { 0xdda679bb, 52 },
    { 0x2eb679be, 43 },
    { 0x91f679bb, 95 },
    { 0xeb4679bb, 100 },
    { 0, 0 },
    { 0xc40679c2, 76 },
    { 0xef679c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7679c6, 33 },
    { 0xbb7679c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a679cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5679d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34679d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a679d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed679e3, 46 },
    { 0x462679e3, 67 },
    { 0x9ac679e5, 90 },
    { 0x3e8679e3, 33 },
    { 0xe8d679e5, 98 },
    { 0x495679e8, 90 },
    { 0xde6679e8, 33 },
    { 0x9d6679ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8679ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06679f0, 55 },
    { 0, 0 },
    { 0xd27679f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8679f9, 76 },
    { 0xde8679fa, 95 },
    { 0x8b3679f9, 95 },
    { 0xa1c679f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c467a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4867a06, 61 },
    { 0x6b267a07, 61 },
    { 0xaac67a08, 90 },
    { 0xbc667a09, 76 },
    { 0xd9167a0a, 23 },
    { 0x10e67a0b, 88 },
    { 0xf7567a06, 76 },
    { 0xa4867a0d, 98 },
    { 0x46267a0d, 43 },
    { 0xfe467a0d, 85 },
    { 0xa2267a0c, 84 },
    { 0x35a67a11, 23 },
    { 0, 0 },
    { 0x25067a13, 46 },
    { 0x63d67a14, 90 },
    { 0xcc667a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97067a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9467a1f, 76 },
    { 0xe0c67a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea967a23, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8167a28, 52 },
    { 0xc8f67a29, 33 },
    { 0xbc467a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b67a31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77867a38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a67a43, 33 },
    { 0x4f267a44, 90 },
    { 0xb0167a45, 78 },
    { 0x9db67a45, 95 },
    { 0x85667a45, 98 },
    { 0x23567a44, 55 },
    { 0xf5567a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91267a50, 23 },
    { 0x90067a51, 33 },
    { 0xc0967a52, 33 },
    { 0x2367a53, 46 },
    { 0xe3d67a50, 83 },
    { 0xe8f67a51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc67a60, 95 },
    { 0xf6867a61, 90 },
    { 0, 0 },
    { 0x2e867a63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed067a6a, 90 },
    { 0x9b267a6a, 33 },
    { 0xca767a6b, 55 },
    { 0x56067a6a, 23 },
    { 0xf0567a6d, 61 },
    { 0xd6067a6f, 61 },
    { 0xb7e67a70, 98 },
    { 0xb067a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e167a74, 100 },
    { 0, 0 },
    { 0x54367a76, 33 },
    { 0, 0 },
    { 0x20d67a78, 76 },
    { 0x99967a78, 90 },
    { 0x1bd67a7a, 61 },
    { 0x48967a7b, 98 },
    { 0x3dd67a7c, 91 },
    { 0, 0 },
    { 0xfd167a7e, 95 },
    { 0, 0 },
    { 0x34c67a80, 76 },
    { 0, 0 },
    { 0xe5767a82, 76 },
    { 0x82a67a83, 95 },
    { 0x3db67a82, 17 },
    { 0x9bd67a85, 61 },
    { 0xa7e67a85, 55 },
    { 0x55367a87, 98 },
    { 0x30567a88, 23 },
    { 0xcf567a89, 81 },
    { 0, 0 },
    { 0x10f67a8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0767a92, 76 },
    { 0, 0 },
    { 0xf6167a94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da67a97, 52 },
    { 0x51067a97, 90 },
    { 0, 0 },
    { 0x71f67a9a, 90 },
    { 0, 0 },
    { 0xe6667a9c, 95 },
    { 0xcc167a9c, 82 },
    { 0x3f367a9c, 95 },
    { 0x8bd67a9f, 90 },
    { 0x58a67a9f, 33 },
    { 0x54067aa1, 93 },
    { 0x70b67aa2, 52 },
    { 0xd9667aa2, 66 },
    { 0x10a67aa3, 95 },
    { 0xce767aa2, 23 },
    { 0xa7b67aa6, 40 },
    { 0x29a67aa7, 43 },
    { 0xbca67aa8, 33 },
    { 0xb3767aa9, 98 },
    { 0x6767aa9, 99 },
    { 0xd1067aa9, 85 },
    { 0x47767aac, 23 },
    { 0x50b67aac, 34 },
    { 0x35f67aa9, 76 },
    { 0, 0 },
    { 0x46467ab0, 34 },
    { 0x72767ab1, 34 },
    { 0xee867ab2, 95 },
    { 0xf5967ab3, 61 },
    { 0x87467ab1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67aba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7367ac0, 95 },
    { 0, 0 },
    { 0x7de67ac2, 34 },
    { 0x62867ac2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c67ac6, 81 },
    { 0x48a67ac6, 85 },
    { 0x9e067ac8, 38 },
    { 0x4bb67ac9, 95 },
    { 0x9c867ac7, 98 },
    { 0x31b67aca, 67 },
    { 0x1c667acc, 90 },
    { 0, 0 },
    { 0x1af67ace, 65 },
    { 0, 0 },
    { 0x8f467ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a767ada, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce167ae4, 90 },
    { 0, 0 },
    { 0xc3867ae6, 90 },
    { 0x75b67ae7, 53 },
    { 0, 0 },
    { 0x1e667ae9, 66 },
    { 0x4b367ae9, 99 },
    { 0, 0 },
    { 0x2c767aec, 98 },
    { 0, 0 },
    { 0x2ea67aee, 48 },
    { 0, 0 },
    { 0xd4a67af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae267af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69867af9, 61 },
    { 0, 0 },
    { 0xbe467afb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b67b01, 34 },
    { 0x5f767b01, 98 },
    { 0x49267b03, 46 },
    { 0xdf767b02, 100 },
    { 0, 0 },
    { 0xb1867b06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de67b0a, 61 },
    { 0, 0 },
    { 0x3b167b0c, 17 },
    { 0x3f467b0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb67b10, 61 },
    { 0xe5e67b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb67b16, 66 },
    { 0x19c67b17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95467b1a, 61 },
    { 0, 0 },
    { 0x3c167b1c, 98 },
    { 0xb3867b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e67b23, 78 },
    { 0x85867b24, 51 },
    { 0, 0 },
    { 0xec267b26, 90 },
    { 0x12867b27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea367b2a, 93 },
    { 0, 0 },
    { 0x30367b2c, 52 },
    { 0x5a67b2c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e67b34, 99 },
    { 0xee267b35, 23 },
    { 0x5b767b36, 33 },
    { 0x7da67b36, 95 },
    { 0, 0 },
    { 0x29c67b39, 61 },
    { 0xa6667b39, 53 },
    { 0x5df67b39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e867b3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b67b42, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6367b46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x38267b49, 76 },
    { 0x15767b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf67b4f, 33 },
    { 0x38a67b50, 95 },
    { 0, 0 },
    { 0x4a767b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78967b59, 66 },
    { 0xdad67b5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd67b5d, 23 },
    { 0, 0 },
    { 0x77e67b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe967b63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42967b69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38167b6d, 93 },
    { 0x69167b6d, 23 },
    { 0, 0 },
    { 0xcac67b70, 33 },
    { 0x86867b71, 95 },
    { 0xb6767b71, 33 },
    { 0x53167b73, 95 },
    { 0, 0 },
    { 0xdfd67b75, 76 },
    { 0x18767b76, 61 },
    { 0xa7867b77, 23 },
    { 0xdaf67b78, 52 },
    { 0x27d67b78, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2167b7e, 95 },
    { 0xc3667b7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc67b84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d567b89, 33 },
    { 0x76767b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1867b8d, 46 },
    { 0xce467b8d, 33 },
    { 0x86767b8f, 23 },
    { 0xf6b67b90, 81 },
    { 0xd5a67b90, 23 },
    { 0, 0 },
    { 0x8e367b93, 77 },
    { 0x5e967b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5567b9c, 90 },
    { 0x39167b9d, 90 },
    { 0, 0 },
    { 0x58767b9f, 76 },
    { 0, 0 },
    { 0xe3b67ba1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567ba7, 93 },
    { 0, 0 },
    { 0x7c467ba9, 88 },
    { 0, 0 },
    { 0xd6867bab, 61 },
    { 0x28f67bac, 66 },
    { 0x66d67bac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d67bb5, 100 },
    { 0, 0 },
    { 0x9ab67bb7, 43 },
    { 0x6a367bb7, 33 },
    { 0x9267bb9, 77 },
    { 0x11c67bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68067bbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d467bc7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95267bca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56067bcf, 61 },
    { 0xae667bd0, 61 },
    { 0xa4767bd0, 88 },
    { 0x50567bd0, 95 },
    { 0x43067bd2, 69 },
    { 0, 0 },
    { 0xf6e67bd5, 100 },
    { 0, 0 },
    { 0x90d67bd7, 34 },
    { 0xbde67bd7, 100 },
    { 0x7b967bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6767bde, 84 },
    { 0x2367bdf, 98 },
    { 0x6f967bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11367be3, 33 },
    { 0xc1367be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21967be9, 93 },
    { 0xd8367bea, 88 },
    { 0xda967bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31767bf7, 90 },
    { 0, 0 },
    { 0xdd767bf9, 95 },
    { 0xe2d67bf9, 65 },
    { 0x8b767bfa, 84 },
    { 0x63a67bfc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a67c02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9567c05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc167c0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60267c0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf767c12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc067c16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae467c1a, 90 },
    { 0, 0 },
    { 0x5b67c1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48367c20, 61 },
    { 0x32d67c21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b67c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90967c2c, 33 },
    { 0xc8567c2d, 90 },
    { 0xa6a67c2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e67c31, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc67c3c, 95 },
    { 0x44f67c3d, 40 },
    { 0xe3967c3e, 34 },
    { 0x9c267c3f, 43 },
    { 0x8b867c40, 55 },
    { 0xc9c67c3f, 23 },
    { 0, 0 },
    { 0x42c67c43, 87 },
    { 0xf7967c43, 61 },
    { 0, 0 },
    { 0xe7167c46, 68 },
    { 0, 0 },
    { 0xc1667c48, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67c4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b067c58, 77 },
    { 0x4a267c59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46967c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78467c61, 100 },
    { 0xb4667c61, 91 },
    { 0x7ac67c63, 99 },
    { 0x28867c63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b67c6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20867c75, 93 },
    { 0x40c67c76, 23 },
    { 0x2dd67c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b67c84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2267c8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59767c90, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb67c94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a367c97, 96 },
    { 0, 0 },
    { 0xfdc67c99, 36 },
    { 0xa5c67c99, 23 },
    { 0, 0 },
    { 0xfbf67c9c, 90 },
    { 0x7c067c9d, 80 },
    { 0x53067c9e, 76 },
    { 0xa8b67c9f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d67ca2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d67ca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1467cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92767cb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae67cb4, 34 },
    { 0x7b067cb5, 61 },
    { 0xa2f67cb5, 90 },
    { 0xfc067cb5, 23 },
    { 0x87667cb6, 93 },
    { 0x38667cb8, 34 },
    { 0x1e367cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe067cbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66967cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd67cc5, 68 },
    { 0x4ee67cc6, 98 },
    { 0x19c67cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14767cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6467cd9, 23 },
    { 0x5bd67cda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5767ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67ce5, 55 },
    { 0xcf367ce5, 43 },
    { 0x48967ce7, 95 },
    { 0x7e167ce7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c67ced, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3767cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767d02, 78 },
    { 0x8c167d02, 52 },
    { 0xac467d04, 95 },
    { 0x15667d05, 90 },
    { 0, 0 },
    { 0xc7067d07, 33 },
    { 0xdb867d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe67d10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15967d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8167d1c, 36 },
    { 0x43e67d1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b67d23, 90 },
    { 0xb2967d24, 84 },
    { 0x16567d25, 23 },
    { 0x65967d26, 33 },
    { 0xad67d27, 55 },
    { 0x83167d28, 51 },
    { 0x93167d29, 95 },
    { 0, 0 },
    { 0xa9b67d2b, 55 },
    { 0x42067d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9367d2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a467d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c67d3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe367d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f67d49, 33 },
    { 0x7bd67d4a, 17 },
    { 0, 0 },
    { 0xc3c67d4c, 98 },
    { 0xb3567d4c, 33 },
    { 0, 0 },
    { 0x49367d4f, 98 },
    { 0x1a067d50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93267d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd67d5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23067d60, 88 },
    { 0x76167d61, 90 },
    { 0xbad67d60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f667d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab867d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8667d72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93967d75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf67d7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e67d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef67d89, 43 },
    { 0x5a467d89, 76 },
    { 0, 0 },
    { 0xcab67d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15167d92, 95 },
    { 0x9c67d93, 33 },
    { 0xd2f67d94, 91 },
    { 0x6f67d95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd967d9a, 61 },
    { 0xa6667d9b, 90 },
    { 0x99367d9c, 91 },
    { 0xbd967d9d, 90 },
    { 0xab467d9d, 98 },
    { 0x4bd67d9c, 100 },
    { 0x44c67d9b, 17 },
    { 0x27767d9e, 100 },
    { 0x91767da2, 33 },
    { 0x58d67d9a, 84 },
    { 0x63767da4, 96 },
    { 0xed867da5, 46 },
    { 0x96667da6, 34 },
    { 0x1567da6, 100 },
    { 0x11867da8, 75 },
    { 0x9e067d9b, 95 },
    { 0xf4867da1, 90 },
    { 0xd967dab, 33 },
    { 0x3a667dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c67db1, 97 },
    { 0x89367db2, 90 },
    { 0x9a67db3, 33 },
    { 0x61367db3, 23 },
    { 0xced67db5, 90 },
    { 0x7fe67db6, 43 },
    { 0, 0 },
    { 0x50c67db8, 90 },
    { 0, 0 },
    { 0x1f67dba, 73 },
    { 0, 0 },
    { 0xc4067dbc, 34 },
    { 0, 0 },
    { 0xa1467dbe, 95 },
    { 0xf4267dbf, 23 },
    { 0xb9667dc0, 46 },
    { 0xb9067dc1, 68 },
    { 0xdb767dc2, 93 },
    { 0x17667dc2, 61 },
    { 0, 0 },
    { 0x93967dc5, 66 },
    { 0x62b67dc6, 90 },
    { 0x8ec67dc6, 33 },
    { 0, 0 },
    { 0x2667dc9, 61 },
    { 0x17567dca, 85 },
    { 0x61c67dca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xac567dce, 33 },
    { 0x9e367dcf, 23 },
    { 0x3e867dcf, 46 },
    { 0xe2567dce, 90 },
    { 0x9267dce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e67dd9, 66 },
    { 0, 0 },
    { 0xbca67ddb, 61 },
    { 0x66367ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d367de0, 61 },
    { 0x30967de1, 61 },
    { 0, 0 },
    { 0xd3667de3, 17 },
    { 0xe5c67de3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d67de7, 90 },
    { 0x2c167de8, 80 },
    { 0xced67de9, 52 },
    { 0, 0 },
    { 0x7ae67deb, 61 },
    { 0x1f967dec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1267df2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0467df6, 84 },
    { 0, 0 },
    { 0xe0e67df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3567dfb, 61 },
    { 0xa3867dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a67dff, 90 },
    { 0, 0 },
    { 0x44367e01, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63467e06, 61 },
    { 0x61b67e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5367e0f, 33 },
    { 0xe3167e10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c67e13, 87 },
    { 0xfe567e14, 90 },
    { 0, 0 },
    { 0xf5d67e16, 55 },
    { 0x4867e16, 95 },
    { 0xc6467e16, 75 },
    { 0, 0 },
    { 0x66267e1a, 81 },
    { 0xda467e1b, 77 },
    { 0xfa267e1c, 23 },
    { 0x5ae67e1d, 84 },
    { 0, 0 },
    { 0x4f567e1f, 84 },
    { 0xd9b67e1f, 61 },
    { 0x30267e21, 85 },
    { 0x8e767e21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c67e2c, 34 },
    { 0x5cc67e2d, 33 },
    { 0xd5b67e2e, 23 },
    { 0xca867e2f, 85 },
    { 0, 0 },
    { 0x8c867e31, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d67e39, 90 },
    { 0, 0 },
    { 0x27967e3b, 61 },
    { 0, 0 },
    { 0xe0067e3d, 23 },
    { 0x6aa67e3d, 61 },
    { 0xe9a67e3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51367e42, 99 },
    { 0xefc67e43, 65 },
    { 0xab467e44, 34 },
    { 0x16867e45, 43 },
    { 0xbe867e46, 84 },
    { 0xca367e47, 90 },
    { 0xb8967e48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff67e4b, 61 },
    { 0, 0 },
    { 0x64f67e4d, 61 },
    { 0xc7567e4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48667e52, 33 },
    { 0x92867e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec67e59, 87 },
    { 0, 0 },
    { 0x1c967e5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd67e5f, 33 },
    { 0xe9267e60, 24 },
    { 0, 0 },
    { 0xd2367e62, 43 },
    { 0x6a367e63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x60767e66, 66 },
    { 0xfa267e67, 78 },
    { 0xa3a67e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46367e71, 23 },
    { 0xf1c67e71, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d67e78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18867e7b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b667e7f, 23 },
    { 0xb7067e7f, 90 },
    { 0xea967e7f, 17 },
    { 0x99267e80, 23 },
    { 0xbea67e81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c767e86, 40 },
    { 0, 0 },
    { 0xe9567e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3667e8b, 78 },
    { 0x6d467e8b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0467e98, 90 },
    { 0, 0 },
    { 0xd7967e9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e467ea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad67eab, 61 },
    { 0x4e067eab, 84 },
    { 0x2eb67ead, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a67eb0, 81 },
    { 0, 0 },
    { 0x22667eb2, 98 },
    { 0xe9067eb2, 17 },
    { 0, 0 },
    { 0xac67eb5, 61 },
    { 0, 0 },
    { 0xc2567eb7, 23 },
    { 0xcf767eb8, 88 },
    { 0xf8067eb8, 95 },
    { 0x26667eb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce567ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd67ec6, 28 },
    { 0x36167ec7, 78 },
    { 0, 0 },
    { 0x10267ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4067ecf, 55 },
    { 0, 0 },
    { 0xa0867ed1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55767ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1867ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4267ee3, 61 },
    { 0x42667ee3, 23 },
    { 0, 0 },
    { 0x9bb67ee6, 33 },
    { 0, 0 },
    { 0x67d67ee8, 34 },
    { 0x2cb67ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a67eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2667ef0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b767ef4, 95 },
    { 0, 0 },
    { 0x9c767ef6, 77 },
    { 0x9c67ef7, 52 },
    { 0x6e567ef8, 90 },
    { 0x33467ef9, 61 },
    { 0x41467efa, 61 },
    { 0x32267ef8, 33 },
    { 0x2f067efa, 76 },
    { 0xec567efb, 95 },
    { 0x67967efe, 17 },
    { 0x36967eff, 97 },
    { 0x4dd67eff, 53 },
    { 0xff667f01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f467f09, 69 },
    { 0x59167f0a, 98 },
    { 0xe0867f0b, 46 },
    { 0xb1367f0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff967f10, 61 },
    { 0, 0 },
    { 0x2e167f12, 69 },
    { 0x9c067f13, 95 },
    { 0x5ab67f13, 93 },
    { 0, 0 },
    { 0x9b467f16, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x11267f19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca67f20, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e67f26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f67f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a67f2e, 88 },
    { 0x2b967f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab367f35, 75 },
    { 0, 0 },
    { 0xa8d67f37, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb67f3d, 56 },
    { 0, 0 },
    { 0x40867f3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa967f43, 23 },
    { 0x75167f43, 23 },
    { 0xbce67f44, 76 },
    { 0x2a367f46, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc767f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff367f4c, 33 },
    { 0x33167f4d, 61 },
    { 0x82067f4e, 90 },
    { 0, 0 },
    { 0x47e67f50, 76 },
    { 0x9a267f51, 87 },
    { 0x17a67f51, 90 },
    { 0x81267f51, 76 },
    { 0x5dc67f53, 33 },
    { 0xe3167f50, 33 },
    { 0xd8567f56, 34 },
    { 0x5c767f54, 93 },
    { 0xcc867f58, 61 },
    { 0x3ab67f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa67f5c, 90 },
    { 0xb7567f5d, 67 },
    { 0x9d267f5d, 61 },
    { 0, 0 },
    { 0xcd967f60, 81 },
    { 0x6aa67f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b67f65, 98 },
    { 0xb0e67f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79667f6d, 98 },
    { 0xb2a67f6d, 33 },
    { 0x91967f6f, 33 },
    { 0x8c667f6f, 90 },
    { 0, 0 },
    { 0xac667f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc67f75, 84 },
    { 0xe767f75, 98 },
    { 0x44c67f76, 95 },
    { 0, 0 },
    { 0x3f067f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2767f81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b467f88, 95 },
    { 0x8867f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4367f8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d467f90, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb67f96, 23 },
    { 0xdd67f96, 23 },
    { 0x55267f96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90167f9b, 90 },
    { 0, 0 },
    { 0x667f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb167fa3, 17 },
    { 0x4ed67fa4, 98 },
    { 0, 0 },
    { 0x93e67fa6, 90 },
    { 0, 0 },
    { 0xb2a67fa8, 76 },
    { 0xcb567fa8, 99 },
    { 0xc6267faa, 77 },
    { 0xc2c67fab, 87 },
    { 0xffb67fac, 23 },
    { 0x3fd67fab, 98 },
    { 0x6ff67fae, 68 },
    { 0xdd367fa9, 23 },
    { 0, 0 },
    { 0x8f767fb1, 91 },
    { 0, 0 },
    { 0x5cd67fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c167fb6, 23 },
    { 0xf767fb6, 75 },
    { 0, 0 },
    { 0x4b867fb9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64967fbd, 98 },
    { 0, 0 },
    { 0xf2567fbf, 87 },
    { 0xeac67fc0, 46 },
    { 0xc8067fbf, 90 },
    { 0x42d67fc2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0067fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a67fcb, 98 },
    { 0, 0 },
    { 0x8a767fcd, 23 },
    { 0xbcf67fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba867fd1, 95 },
    { 0xd1a67fd2, 95 },
    { 0, 0 },
    { 0x90967fd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a67fd8, 95 },
    { 0, 0 },
    { 0x7e567fda, 23 },
    { 0x66967fdb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f667fdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b67fe3, 55 },
    { 0, 0 },
    { 0x48867fe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf67fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab67fef, 33 },
    { 0xc067ff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d67ff6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad167ff9, 45 },
    { 0xb8d67ff9, 33 },
    { 0x4e167ff9, 84 },
    { 0xb3c67ffc, 61 },
    { 0xe0267ffd, 23 },
    { 0xda767ffd, 23 },
    { 0x59e67ffc, 61 },
    { 0, 0 },
    { 0xeed68001, 81 },
    { 0xcfa68001, 77 },
    { 0xf2468001, 93 },
    { 0x94a68004, 61 },
    { 0x51868005, 95 },
    { 0x52868006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2168009, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e568014, 90 },
    { 0xb4e68015, 61 },
    { 0xb7068016, 90 },
    { 0, 0 },
    { 0x73f68018, 91 },
    { 0x74f68019, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a568021, 99 },
    { 0x8c68022, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43068029, 93 },
    { 0x4c06802a, 91 },
    { 0x8236802a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6768030, 85 },
    { 0x9a468031, 95 },
    { 0x9df68030, 95 },
    { 0, 0 },
    { 0xb6768034, 90 },
    { 0x25068035, 77 },
    { 0xf5d68035, 61 },
    { 0xbf568037, 99 },
    { 0xdfc68035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a68040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2268049, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546804d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4768051, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e968055, 33 },
    { 0xfdf68056, 88 },
    { 0x60c68057, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a36805a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b68066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6806e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b68071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa68075, 61 },
    { 0, 0 },
    { 0x1e68077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a868082, 33 },
    { 0x9a368082, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb68087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6808a, 33 },
    { 0x8a26808a, 33 },
    { 0x69a6808c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec6808f, 68 },
    { 0xe296808f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae68096, 95 },
    { 0x2168097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596809d, 61 },
    { 0x6fa6809e, 77 },
    { 0x7d6809d, 100 },
    { 0, 0 },
    { 0x222680a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6680a4, 43 },
    { 0xf64680a4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90680a9, 34 },
    { 0x439680aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71680ad, 95 },
    { 0x271680ae, 95 },
    { 0x5fe680af, 74 },
    { 0xbea680b0, 78 },
    { 0x199680b1, 91 },
    { 0, 0 },
    { 0x5f6680b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319680b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b680ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119680bf, 88 },
    { 0, 0 },
    { 0x2c7680c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff680c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d680ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89680d0, 97 },
    { 0, 0 },
    { 0xdb2680d2, 93 },
    { 0x6d0680d2, 76 },
    { 0, 0 },
    { 0x526680d5, 46 },
    { 0xcf5680d5, 95 },
    { 0x738680d7, 90 },
    { 0x8a680d6, 61 },
    { 0xa83680d9, 23 },
    { 0x757680d9, 60 },
    { 0x289680db, 61 },
    { 0x7c2680dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89680e1, 23 },
    { 0xbe1680e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3680e8, 23 },
    { 0xb6a680e9, 33 },
    { 0, 0 },
    { 0xf4f680eb, 90 },
    { 0xe0f680eb, 33 },
    { 0, 0 },
    { 0x134680ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac680f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb680f8, 52 },
    { 0x32e680f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x249680fc, 98 },
    { 0x377680fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1868105, 34 },
    { 0x4768105, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x77968109, 68 },
    { 0xb9f68109, 90 },
    { 0, 0 },
    { 0xcd56810c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a68115, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36811d, 90 },
    { 0x68a6811e, 33 },
    { 0, 0 },
    { 0xc8a68120, 90 },
    { 0x4b168120, 98 },
    { 0xe4068122, 23 },
    { 0, 0 },
    { 0x60b68124, 55 },
    { 0xacd68125, 76 },
    { 0xa8668126, 96 },
    { 0xec768127, 33 },
    { 0x6b468127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286812c, 33 },
    { 0x73b6812d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f168133, 90 },
    { 0, 0 },
    { 0xe1368135, 61 },
    { 0xf3b68136, 95 },
    { 0x9a268135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa936813a, 23 },
    { 0, 0 },
    { 0x4206813c, 95 },
    { 0, 0 },
    { 0xa6b6813e, 61 },
    { 0xb766813f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6068144, 23 },
    { 0x3ce68145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d568148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af6814d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb268150, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b68154, 33 },
    { 0, 0 },
    { 0x24968156, 33 },
    { 0xdf868157, 53 },
    { 0, 0 },
    { 0x1a268159, 76 },
    { 0, 0 },
    { 0xfa26815b, 34 },
    { 0x1a6815c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc68163, 95 },
    { 0x82668164, 46 },
    { 0xb3668165, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24668168, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e6816b, 23 },
    { 0x3566816c, 90 },
    { 0xa136816c, 33 },
    { 0xed26816c, 98 },
    { 0xac46816d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e68175, 76 },
    { 0xe3868175, 90 },
    { 0, 0 },
    { 0xd5968178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa06817b, 97 },
    { 0, 0 },
    { 0x186817d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c68181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b368184, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a68187, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d46818a, 90 },
    { 0xbce6818b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6818e, 61 },
    { 0xd056818f, 33 },
    { 0x1a068190, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d68197, 46 },
    { 0, 0 },
    { 0xa5568199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe6819c, 98 },
    { 0x54e6819d, 99 },
    { 0xe596819d, 95 },
    { 0xe2e6819f, 61 },
    { 0xbd76819f, 100 },
    { 0x9ef6819e, 90 },
    { 0xb21681a2, 90 },
    { 0x2f7681a3, 43 },
    { 0xd3681a0, 93 },
    { 0x563681a0, 95 },
    { 0x7e1681a6, 95 },
    { 0xdf681a2, 93 },
    { 0xa076819e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a681ad, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4681b3, 65 },
    { 0x58a681b4, 33 },
    { 0xa45681b4, 76 },
    { 0x367681b6, 33 },
    { 0x1a8681b6, 34 },
    { 0x2aa681b8, 43 },
    { 0x52c681b9, 52 },
    { 0xbf0681b6, 90 },
    { 0, 0 },
    { 0xc7681bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x781681bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82681c2, 90 },
    { 0xe1f681c3, 49 },
    { 0xad6681c4, 46 },
    { 0x885681c5, 98 },
    { 0x2e5681c5, 43 },
    { 0xf51681c7, 61 },
    { 0xddc681c8, 61 },
    { 0x1da681c5, 91 },
    { 0x2d7681ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915681cf, 23 },
    { 0, 0 },
    { 0x58a681d1, 46 },
    { 0, 0 },
    { 0xdf0681d3, 34 },
    { 0xeaf681d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef681da, 34 },
    { 0xc2681db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8681df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f681e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9681e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08681ec, 33 },
    { 0, 0 },
    { 0x6bf681ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c681f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc681f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec681fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c68201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa768205, 95 },
    { 0, 0 },
    { 0xadd68207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86820c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6820f, 43 },
    { 0xdb368210, 66 },
    { 0x7516820f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1768219, 43 },
    { 0x95e6821a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56821e, 85 },
    { 0xc136821f, 61 },
    { 0x9e068220, 95 },
    { 0x81068221, 90 },
    { 0x3f868222, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc668225, 76 },
    { 0x6c68226, 99 },
    { 0xf1168226, 81 },
    { 0xeec68226, 33 },
    { 0x80468226, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf6822c, 33 },
    { 0x5936822d, 76 },
    { 0x6586822d, 90 },
    { 0x5d66822f, 95 },
    { 0, 0 },
    { 0x37268231, 90 },
    { 0x6a968232, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc368235, 90 },
    { 0, 0 },
    { 0x4d568237, 33 },
    { 0, 0 },
    { 0x77968239, 55 },
    { 0x59c68239, 90 },
    { 0x9996823a, 75 },
    { 0xc276823c, 99 },
    { 0x5c56823d, 91 },
    { 0x1046823d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9168241, 61 },
    { 0x2ed68241, 61 },
    { 0x49868241, 55 },
    { 0x42e68244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b768247, 95 },
    { 0x3668247, 23 },
    { 0x23168247, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd726824e, 55 },
    { 0x2836824f, 90 },
    { 0x7336824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4468253, 55 },
    { 0x1ee68253, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2768257, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc76825a, 61 },
    { 0x5c86825b, 98 },
    { 0, 0 },
    { 0xc2b6825d, 23 },
    { 0x8bc6825d, 34 },
    { 0x8ec6825f, 98 },
    { 0xc7568260, 91 },
    { 0xec46825e, 90 },
    { 0x29868262, 33 },
    { 0, 0 },
    { 0xe3868264, 85 },
    { 0x89e68264, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19668268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf56826b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16668271, 55 },
    { 0x55868272, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a68278, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6586827e, 84 },
    { 0xad86827e, 66 },
    { 0xfa468280, 100 },
    { 0xc4c6827e, 95 },
    { 0, 0 },
    { 0x2c68283, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8168288, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f568290, 76 },
    { 0x64b68291, 61 },
    { 0, 0 },
    { 0xd2068293, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x60268296, 40 },
    { 0x84d68297, 34 },
    { 0, 0 },
    { 0xdfe68299, 90 },
    { 0x176829a, 90 },
    { 0, 0 },
    { 0x28b6829c, 87 },
    { 0x6ed6829d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae682a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e682a5, 84 },
    { 0xcbf682a5, 52 },
    { 0x14a682a7, 95 },
    { 0x1fc682a8, 67 },
    { 0x898682a9, 90 },
    { 0xd0f682aa, 90 },
    { 0x326682a9, 52 },
    { 0x8e9682a6, 61 },
    { 0x947682ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c682b2, 33 },
    { 0xb9b682b2, 23 },
    { 0xb65682b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04682b7, 93 },
    { 0, 0 },
    { 0x10d682b9, 78 },
    { 0xb6b682ba, 78 },
    { 0x206682b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a682c1, 95 },
    { 0xa1d682c2, 38 },
    { 0x12f682c1, 34 },
    { 0x14e682c4, 93 },
    { 0xdd1682c2, 33 },
    { 0x11682c6, 43 },
    { 0xded682c6, 33 },
    { 0x2b9682c5, 61 },
    { 0x7a4682c9, 88 },
    { 0x465682ca, 61 },
    { 0x86e682c9, 33 },
    { 0x9ac682cc, 23 },
    { 0xb3a682cd, 95 },
    { 0xa43682cd, 75 },
    { 0x94d682cf, 17 },
    { 0x701682ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38682d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36682d7, 52 },
    { 0x185682d7, 33 },
    { 0x7ab682d9, 61 },
    { 0, 0 },
    { 0x221682db, 33 },
    { 0xac9682dc, 23 },
    { 0, 0 },
    { 0x24e682de, 61 },
    { 0, 0 },
    { 0xec1682e0, 69 },
    { 0, 0 },
    { 0xd3c682e2, 95 },
    { 0x2a1682e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea682e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe682ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f682f0, 90 },
    { 0xc10682f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358682f7, 33 },
    { 0x9f5682f7, 90 },
    { 0, 0 },
    { 0xc01682fa, 61 },
    { 0xdd9682fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed682fe, 78 },
    { 0xa50682ff, 4 },
    { 0xade682ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae368305, 84 },
    { 0xa1f68306, 61 },
    { 0xcd468307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4166830a, 43 },
    { 0x6dc6830b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5596830f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a468317, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1168321, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd68324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf468327, 84 },
    { 0x9d868327, 23 },
    { 0x50468329, 90 },
    { 0, 0 },
    { 0x3da6832b, 33 },
    { 0x6cc6832c, 55 },
    { 0x7a66832d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee68330, 61 },
    { 0, 0 },
    { 0x81868332, 23 },
    { 0x36568333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29568336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4826833c, 95 },
    { 0x55d6833d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906834a, 76 },
    { 0, 0 },
    { 0xb796834c, 90 },
    { 0xf156834d, 33 },
    { 0x3056834d, 76 },
    { 0, 0 },
    { 0x6c068350, 61 },
    { 0x67a68350, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5768355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6835e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a68369, 55 },
    { 0x8ec6836a, 46 },
    { 0xbd46836a, 100 },
    { 0xb6b6836a, 46 },
    { 0x5fe6836d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62868373, 91 },
    { 0, 0 },
    { 0xf8468375, 90 },
    { 0xae768376, 55 },
    { 0, 0 },
    { 0x59168378, 75 },
    { 0, 0 },
    { 0x9fa6837a, 52 },
    { 0x9716837a, 23 },
    { 0, 0 },
    { 0x58b6837d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e968382, 56 },
    { 0x6c668382, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a68387, 90 },
    { 0x8ab68387, 100 },
    { 0x81768387, 95 },
    { 0x5668389, 43 },
    { 0x3066838b, 76 },
    { 0xb416838a, 95 },
    { 0x666838d, 95 },
    { 0xea96838d, 90 },
    { 0x2ba6838f, 84 },
    { 0x65b68390, 61 },
    { 0x77668391, 76 },
    { 0xe9c68390, 95 },
    { 0, 0 },
    { 0x44368394, 23 },
    { 0x1aa68394, 76 },
    { 0x99068394, 55 },
    { 0x6e068394, 77 },
    { 0x54668398, 98 },
    { 0x73b68398, 76 },
    { 0x2a66839a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd6839e, 90 },
    { 0x82b6839e, 95 },
    { 0xa0c6839f, 67 },
    { 0, 0 },
    { 0x24d683a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930683a9, 99 },
    { 0xf92683a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e683ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74683b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1683b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58683b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0683ba, 64 },
    { 0x837683ba, 76 },
    { 0, 0 },
    { 0x7f5683bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e683c4, 33 },
    { 0x52e683c4, 46 },
    { 0x3a1683c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a683cd, 95 },
    { 0xf42683ce, 55 },
    { 0x78683cf, 95 },
    { 0xc05683cf, 95 },
    { 0x44f683d1, 33 },
    { 0, 0 },
    { 0x695683d3, 23 },
    { 0, 0 },
    { 0xd683d5, 33 },
    { 0x1e6683d6, 76 },
    { 0xa19683d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145683df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155683e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4683ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab683ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab683f0, 61 },
    { 0, 0 },
    { 0x8a3683f2, 43 },
    { 0x8bd683f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510683fa, 90 },
    { 0x613683fa, 52 },
    { 0, 0 },
    { 0x8b8683fd, 61 },
    { 0x707683fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3868403, 65 },
    { 0, 0 },
    { 0xa2268405, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56840c, 61 },
    { 0x85a6840c, 34 },
    { 0x7ba6840d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c68412, 68 },
    { 0x3f168412, 33 },
    { 0, 0 },
    { 0x4c868415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8246841a, 61 },
    { 0xf6b6841b, 90 },
    { 0x10e6841c, 85 },
    { 0, 0 },
    { 0xb896841e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb68422, 33 },
    { 0x20e68422, 46 },
    { 0x88568424, 61 },
    { 0x90468422, 61 },
    { 0xc6768426, 34 },
    { 0xe2d68426, 76 },
    { 0, 0 },
    { 0x4eb68429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c68431, 78 },
    { 0x9b068432, 23 },
    { 0x2a068433, 35 },
    { 0, 0 },
    { 0x6cc68435, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24268439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9868440, 23 },
    { 0xa2b68441, 100 },
    { 0xcec68441, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b68445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd746844b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e68450, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6868454, 43 },
    { 0, 0 },
    { 0x68068456, 33 },
    { 0x23468457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd976845b, 34 },
    { 0x6fd6845c, 23 },
    { 0xcdc6845c, 98 },
    { 0xfa76845d, 95 },
    { 0x2ed6845e, 95 },
    { 0x65568460, 77 },
    { 0xf1168461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11768467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc396846a, 90 },
    { 0, 0 },
    { 0xeeb6846c, 61 },
    { 0x61f6846d, 90 },
    { 0xe416846e, 90 },
    { 0xe8d6846f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf768473, 95 },
    { 0xf2768474, 98 },
    { 0x62968475, 61 },
    { 0x66a68474, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x60968479, 98 },
    { 0x95c6847a, 23 },
    { 0, 0 },
    { 0xd0e6847c, 34 },
    { 0x8fe6847d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40868481, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd768485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16868489, 90 },
    { 0x136848a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14468491, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb868495, 95 },
    { 0x3a268496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db6849a, 23 },
    { 0x8646849b, 90 },
    { 0, 0 },
    { 0xa556849d, 33 },
    { 0xcd16849e, 23 },
    { 0xe156849e, 66 },
    { 0x950684a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x484684a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x604684a6, 61 },
    { 0x5b4684a7, 90 },
    { 0, 0 },
    { 0x726684a9, 23 },
    { 0x765684a9, 61 },
    { 0x789684aa, 61 },
    { 0xd24684a9, 95 },
    { 0x973684ad, 67 },
    { 0xf7684ae, 55 },
    { 0x352684af, 69 },
    { 0xa22684ad, 61 },
    { 0x7cc684b1, 76 },
    { 0xda5684b0, 76 },
    { 0x7c9684b3, 99 },
    { 0x7f2684b4, 95 },
    { 0xe0f684aa, 68 },
    { 0x5ba684b6, 68 },
    { 0, 0 },
    { 0xc5c684b8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7684c8, 90 },
    { 0x260684c8, 52 },
    { 0, 0 },
    { 0xd5684cb, 97 },
    { 0, 0 },
    { 0xca3684cd, 76 },
    { 0x80f684cd, 95 },
    { 0x205684cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f684d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2684db, 43 },
    { 0xff1684db, 76 },
    { 0x968684dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4684e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf684e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c684e8, 78 },
    { 0x6b1684e9, 88 },
    { 0xc13684e9, 90 },
    { 0x8d2684e9, 76 },
    { 0x29e684eb, 75 },
    { 0, 0 },
    { 0xbcf684ee, 90 },
    { 0x541684ee, 91 },
    { 0x5a6684f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d684f3, 23 },
    { 0x3e6684f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88684f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x655684fb, 36 },
    { 0xaa0684fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790684ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c68502, 90 },
    { 0x5968502, 23 },
    { 0x8a68504, 43 },
    { 0x61a68505, 76 },
    { 0xf0968506, 90 },
    { 0xd6468504, 93 },
    { 0x6c468507, 90 },
    { 0xb9168507, 95 },
    { 0x61c68504, 93 },
    { 0x5eb6850b, 37 },
    { 0x8526850c, 55 },
    { 0x5e468509, 96 },
    { 0x5646850c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e668512, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b36851b, 23 },
    { 0, 0 },
    { 0x4276851d, 93 },
    { 0x59e6851d, 33 },
    { 0x5ae6851f, 68 },
    { 0xd3e68520, 87 },
    { 0, 0 },
    { 0xad168522, 33 },
    { 0, 0 },
    { 0xf3e68524, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4868528, 98 },
    { 0, 0 },
    { 0x2886852a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36852f, 76 },
    { 0, 0 },
    { 0xda268531, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5936853a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336853e, 100 },
    { 0x8d76853f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b68542, 92 },
    { 0, 0 },
    { 0x69f68544, 64 },
    { 0x58368544, 61 },
    { 0xd9568546, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6854a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6854e, 23 },
    { 0x2a46854f, 61 },
    { 0, 0 },
    { 0x25568551, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3068555, 43 },
    { 0xcc068556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2686855d, 67 },
    { 0x83d6855e, 95 },
    { 0x2b76855f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f68562, 98 },
    { 0x8e668563, 90 },
    { 0x1ce68564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6068567, 46 },
    { 0x56b68568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd16856b, 33 },
    { 0x8586856c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15568571, 23 },
    { 0x6c768571, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b68577, 76 },
    { 0xbdb68578, 23 },
    { 0, 0 },
    { 0x9956857a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1976857d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6168587, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac6858c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6858f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b68592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6859b, 55 },
    { 0xb1a6859b, 90 },
    { 0x24f6859c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5685a2, 98 },
    { 0, 0 },
    { 0x74e685a4, 46 },
    { 0xbcd685a4, 76 },
    { 0, 0 },
    { 0xb82685a7, 93 },
    { 0x54e685a7, 76 },
    { 0x11f685a9, 90 },
    { 0, 0 },
    { 0xd9f685ab, 90 },
    { 0xc1a685ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966685b1, 90 },
    { 0x6c685b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db685bb, 23 },
    { 0xca8685bb, 99 },
    { 0x835685bb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64685c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6685c6, 87 },
    { 0x557685c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c685cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5685d2, 90 },
    { 0x854685d3, 33 },
    { 0, 0 },
    { 0x4f9685d5, 98 },
    { 0x431685d5, 56 },
    { 0x753685d7, 95 },
    { 0, 0 },
    { 0x2fb685d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a685de, 100 },
    { 0x79b685de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9685e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af685e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b685ef, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0685f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e685f5, 33 },
    { 0x477685f6, 98 },
    { 0, 0 },
    { 0x6c5685f8, 98 },
    { 0x35d685f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d685fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca685ff, 90 },
    { 0x18368600, 33 },
    { 0x27568601, 61 },
    { 0x9bb68601, 58 },
    { 0, 0 },
    { 0x68168604, 90 },
    { 0x9fe68605, 34 },
    { 0xe7668606, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56860b, 55 },
    { 0x5bb6860c, 95 },
    { 0xbb96860d, 90 },
    { 0x7886860d, 43 },
    { 0xed46860f, 43 },
    { 0xd136860d, 98 },
    { 0xba26860f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7968617, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66861b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe506861e, 23 },
    { 0, 0 },
    { 0xf768620, 33 },
    { 0x99868621, 33 },
    { 0x3ef68622, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f6862a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416862d, 95 },
    { 0, 0 },
    { 0x1e76862f, 43 },
    { 0x60468630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a968633, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e468636, 98 },
    { 0x1f468637, 84 },
    { 0, 0 },
    { 0xdb668639, 90 },
    { 0x46f6863a, 17 },
    { 0, 0 },
    { 0x3086863c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b76863f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x31068642, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78368645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e068649, 98 },
    { 0xd1c6864a, 61 },
    { 0xdcb6864b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d26864f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a68657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4676865c, 43 },
    { 0xc8c6865d, 76 },
    { 0xc846865e, 61 },
    { 0, 0 },
    { 0xee668660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d368668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd368670, 95 },
    { 0x47868671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86868674, 61 },
    { 0, 0 },
    { 0xf6a68676, 19 },
    { 0x1da68676, 99 },
    { 0x50a68676, 67 },
    { 0x44068678, 98 },
    { 0x9d96867a, 76 },
    { 0xba66867a, 93 },
    { 0xe026867c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d6867f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df68682, 33 },
    { 0xe568683, 38 },
    { 0xf0168683, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c068688, 95 },
    { 0x52b68689, 76 },
    { 0x8a56868a, 68 },
    { 0xdf36868b, 52 },
    { 0x6cd6868b, 31 },
    { 0, 0 },
    { 0xa996868e, 67 },
    { 0, 0 },
    { 0x91f68690, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a468693, 61 },
    { 0x98a68694, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd06869c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f686a2, 9 },
    { 0, 0 },
    { 0xd62686a4, 90 },
    { 0, 0 },
    { 0xe37686a6, 95 },
    { 0, 0 },
    { 0xc33686a8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f686ac, 88 },
    { 0x703686ad, 61 },
    { 0x122686ad, 33 },
    { 0x981686af, 61 },
    { 0xcd8686ae, 46 },
    { 0x22686b1, 61 },
    { 0x3be686b2, 40 },
    { 0x584686b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc686b6, 81 },
    { 0x887686b7, 43 },
    { 0, 0 },
    { 0xf5b686b9, 23 },
    { 0, 0 },
    { 0x559686bb, 98 },
    { 0xf5d686bb, 33 },
    { 0, 0 },
    { 0x40e686be, 38 },
    { 0, 0 },
    { 0xc4c686c0, 76 },
    { 0, 0 },
    { 0x4e9686c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1686c5, 65 },
    { 0xd68686c5, 90 },
    { 0xda2686c7, 33 },
    { 0xd9d686c8, 68 },
    { 0x4ec686c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725686cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba686d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278686d7, 33 },
    { 0x994686d8, 61 },
    { 0xe32686d9, 95 },
    { 0, 0 },
    { 0x529686db, 98 },
    { 0x3c7686dc, 90 },
    { 0xc4a686dd, 23 },
    { 0, 0 },
    { 0x76b686df, 93 },
    { 0x6f8686e0, 33 },
    { 0xa33686e0, 61 },
    { 0xca8686e2, 43 },
    { 0x338686e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d686e9, 33 },
    { 0, 0 },
    { 0xb7f686eb, 17 },
    { 0x2e3686eb, 95 },
    { 0x9bb686ed, 78 },
    { 0, 0 },
    { 0x20c686ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3686f5, 84 },
    { 0, 0 },
    { 0x790686f7, 22 },
    { 0, 0 },
    { 0x96a686f9, 23 },
    { 0xff6686fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f68701, 99 },
    { 0xe0468702, 55 },
    { 0xfbe68703, 90 },
    { 0x1af68702, 90 },
    { 0x81a68704, 100 },
    { 0xab968706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566870d, 99 },
    { 0xce06870e, 78 },
    { 0x8236870d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83068715, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6871b, 33 },
    { 0x8ce6871b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89568720, 61 },
    { 0xc3468721, 55 },
    { 0, 0 },
    { 0xbb868723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a68726, 61 },
    { 0, 0 },
    { 0xccd68728, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79968733, 17 },
    { 0xa4f68733, 90 },
    { 0xe4068735, 23 },
    { 0x97268736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556873b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c06873e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4568741, 55 },
    { 0x1e168741, 76 },
    { 0, 0 },
    { 0xff068744, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa906874d, 61 },
    { 0x5d96874d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e168752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa836875a, 33 },
    { 0xa636875b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f68763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f468766, 23 },
    { 0x28168767, 95 },
    { 0x1cd68767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb856876c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a668770, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b268773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26468776, 99 },
    { 0x16e68777, 76 },
    { 0xd6168777, 90 },
    { 0xa9568776, 43 },
    { 0, 0 },
    { 0xa416877b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b68780, 23 },
    { 0xaaa68780, 61 },
    { 0, 0 },
    { 0xb9068783, 33 },
    { 0xb2b68783, 23 },
    { 0xb5f68784, 33 },
    { 0xc1e68786, 43 },
    { 0x82f68786, 33 },
    { 0xcbe68788, 65 },
    { 0xff268788, 55 },
    { 0, 0 },
    { 0x47a6878b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf066878f, 33 },
    { 0xc0868790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62668796, 81 },
    { 0xfd568796, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec16879b, 33 },
    { 0x3fd6879c, 87 },
    { 0x6776879d, 90 },
    { 0x1036879e, 33 },
    { 0, 0 },
    { 0x76c687a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779687a4, 33 },
    { 0xf30687a5, 85 },
    { 0x762687a6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e687a9, 76 },
    { 0x65b687aa, 61 },
    { 0x673687ab, 33 },
    { 0x9ac687ac, 33 },
    { 0xfd6687ac, 55 },
    { 0x732687ae, 61 },
    { 0xa23687af, 95 },
    { 0xb96687b0, 67 },
    { 0xa50687b1, 95 },
    { 0x77a687ad, 43 },
    { 0x1b687b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19687b7, 100 },
    { 0, 0 },
    { 0xb60687b9, 90 },
    { 0x8ff687ba, 76 },
    { 0x312687bb, 95 },
    { 0, 0 },
    { 0xf51687bd, 90 },
    { 0xc61687bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04687c6, 76 },
    { 0, 0 },
    { 0x22e687c8, 61 },
    { 0x6ff687c9, 55 },
    { 0xcae687c9, 84 },
    { 0x7ba687c8, 95 },
    { 0xd5d687cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985687cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765687d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2687dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82687e1, 23 },
    { 0, 0 },
    { 0x391687e3, 78 },
    { 0xbd3687e4, 33 },
    { 0, 0 },
    { 0xa4a687e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64687ed, 78 },
    { 0, 0 },
    { 0xe6a687ef, 90 },
    { 0x25687f0, 90 },
    { 0xe9687ef, 61 },
    { 0, 0 },
    { 0xa25687f3, 46 },
    { 0xe56687f4, 61 },
    { 0, 0 },
    { 0x1b1687f6, 90 },
    { 0x5aa687f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0468802, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1168805, 34 },
    { 0x27e68806, 98 },
    { 0x7da68807, 90 },
    { 0x64168808, 90 },
    { 0x13168809, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86880e, 61 },
    { 0, 0 },
    { 0x3aa68810, 90 },
    { 0x48868811, 93 },
    { 0xb1a68810, 61 },
    { 0x1fa68813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42668816, 93 },
    { 0, 0 },
    { 0x38f68818, 95 },
    { 0x50668819, 23 },
    { 0x10468819, 38 },
    { 0, 0 },
    { 0xa1f6881c, 55 },
    { 0x4f6881c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9468821, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c68826, 55 },
    { 0, 0 },
    { 0xe6768828, 80 },
    { 0x99b68829, 95 },
    { 0, 0 },
    { 0x4a6882b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df68832, 33 },
    { 0, 0 },
    { 0xc4168834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc568838, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b68840, 33 },
    { 0, 0 },
    { 0x9d968842, 76 },
    { 0xc6168843, 99 },
    { 0, 0 },
    { 0xf7c68845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f68849, 59 },
    { 0x3bc6884a, 23 },
    { 0, 0 },
    { 0x2076884c, 90 },
    { 0xfee6884d, 76 },
    { 0x2376884d, 100 },
    { 0, 0 },
    { 0x2a068850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b68853, 81 },
    { 0, 0 },
    { 0xfcf68855, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe68859, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b6885c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476885f, 33 },
    { 0x7636885f, 99 },
    { 0x63868860, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6468865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde86886a, 95 },
    { 0x36886b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d368870, 23 },
    { 0xba268871, 33 },
    { 0, 0 },
    { 0xe2c68873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1268876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d68879, 61 },
    { 0x34b6887a, 61 },
    { 0x19a6887b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8068880, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b68883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27168887, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc576888a, 93 },
    { 0x9a76888a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13668894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d26889b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2688a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5688aa, 37 },
    { 0, 0 },
    { 0x278688ac, 23 },
    { 0, 0 },
    { 0x7ef688ae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833688b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde688b7, 90 },
    { 0xe90688b7, 55 },
    { 0, 0 },
    { 0xf54688ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc688bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291688c2, 80 },
    { 0xb41688c3, 76 },
    { 0, 0 },
    { 0x36b688c5, 67 },
    { 0x772688c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265688d0, 90 },
    { 0, 0 },
    { 0xb0f688d2, 98 },
    { 0xada688d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731688d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab688e2, 90 },
    { 0x1b2688e3, 34 },
    { 0, 0 },
    { 0x1d1688e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec688e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d688eb, 90 },
    { 0x388688eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0688f9, 98 },
    { 0, 0 },
    { 0x941688fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f68906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6136890a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a6890e, 33 },
    { 0x63d6890f, 33 },
    { 0x96b68910, 90 },
    { 0xa8b68910, 67 },
    { 0xf9568910, 46 },
    { 0, 0 },
    { 0xb8168914, 76 },
    { 0x50168915, 67 },
    { 0, 0 },
    { 0x98068917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd426891b, 34 },
    { 0, 0 },
    { 0x79d6891d, 81 },
    { 0x7cd6891e, 100 },
    { 0, 0 },
    { 0x8e968920, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd768923, 72 },
    { 0x4ca68923, 34 },
    { 0xd3468925, 76 },
    { 0, 0 },
    { 0x4e568927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e36892a, 67 },
    { 0x51c6892b, 33 },
    { 0x31c6892c, 90 },
    { 0, 0 },
    { 0xcc06892e, 95 },
    { 0x9e76892f, 87 },
    { 0xe936892f, 60 },
    { 0x16968931, 91 },
    { 0xafc68931, 61 },
    { 0x14468933, 100 },
    { 0x7ca6892f, 95 },
    { 0, 0 },
    { 0x14368936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25068939, 91 },
    { 0x3ec68939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6893d, 34 },
    { 0, 0 },
    { 0xef46893f, 90 },
    { 0x31168940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368944, 93 },
    { 0xc9068945, 61 },
    { 0x77b68946, 98 },
    { 0, 0 },
    { 0xa0d68948, 90 },
    { 0xbab68948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c46894e, 95 },
    { 0x9696894e, 98 },
    { 0xd4d6894f, 34 },
    { 0xaad6894f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67168955, 76 },
    { 0x70d68956, 55 },
    { 0, 0 },
    { 0x8ce68958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066895c, 55 },
    { 0x556895c, 34 },
    { 0, 0 },
    { 0x4a36895f, 33 },
    { 0xca068960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5068967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9736896a, 68 },
    { 0xb796896b, 33 },
    { 0x1816896b, 55 },
    { 0, 0 },
    { 0xd2c6896e, 90 },
    { 0, 0 },
    { 0x55068970, 66 },
    { 0, 0 },
    { 0x6ef68972, 23 },
    { 0, 0 },
    { 0xa5568974, 78 },
    { 0x6a368975, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7568979, 95 },
    { 0, 0 },
    { 0x9376897b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db68981, 95 },
    { 0xeca68982, 69 },
    { 0x64068983, 99 },
    { 0xb5a68983, 90 },
    { 0x21c68985, 38 },
    { 0, 0 },
    { 0x24768987, 67 },
    { 0xcf68987, 69 },
    { 0x21f68989, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7566898c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e66899a, 33 },
    { 0x5546899a, 34 },
    { 0x9bb6899a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147689a0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3689a3, 93 },
    { 0, 0 },
    { 0x763689a5, 25 },
    { 0x9b3689a6, 78 },
    { 0x1d3689a7, 76 },
    { 0, 0 },
    { 0x78a689a9, 98 },
    { 0x2c7689aa, 23 },
    { 0x35d689ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8689af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d689b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8689b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735689be, 23 },
    { 0x9a1689bf, 61 },
    { 0xb2b689c0, 80 },
    { 0x718689c1, 76 },
    { 0x633689c2, 67 },
    { 0x79d689c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f689c6, 98 },
    { 0x901689c7, 55 },
    { 0xa46689c6, 33 },
    { 0x2bc689c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e689cc, 76 },
    { 0xc27689cd, 90 },
    { 0x255689cc, 65 },
    { 0, 0 },
    { 0x3e5689d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2689d7, 47 },
    { 0xf21689d8, 95 },
    { 0xfb9689d9, 90 },
    { 0x359689da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa689de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5689e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0689e6, 90 },
    { 0x646689e7, 81 },
    { 0xf40689e8, 95 },
    { 0, 0 },
    { 0x15f689ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4689f1, 98 },
    { 0x6f3689f1, 90 },
    { 0x83d689f1, 33 },
    { 0x140689f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90689f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3689ff, 33 },
    { 0, 0 },
    { 0x55768a01, 93 },
    { 0x4f368a02, 95 },
    { 0x23b68a01, 34 },
    { 0x61d68a04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50568a07, 23 },
    { 0, 0 },
    { 0x90068a09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e68a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4268a11, 91 },
    { 0, 0 },
    { 0x5aa68a13, 34 },
    { 0, 0 },
    { 0xaa968a15, 90 },
    { 0x44a68a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb68a19, 90 },
    { 0x33b68a1a, 23 },
    { 0xe7968a1b, 90 },
    { 0, 0 },
    { 0xbec68a1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22968a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b68a24, 98 },
    { 0xd1168a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9968a2b, 23 },
    { 0xa6768a2b, 34 },
    { 0xc2268a2c, 90 },
    { 0x27968a2e, 23 },
    { 0xb4c68a2f, 90 },
    { 0x9b168a30, 33 },
    { 0x5bc68a2d, 95 },
    { 0x29668a32, 98 },
    { 0xc2e68a2c, 43 },
    { 0x97f68a34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5668a37, 56 },
    { 0x57e68a38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b68a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea68a3e, 55 },
    { 0xc9b68a3e, 98 },
    { 0xf0d68a40, 78 },
    { 0xa5a68a41, 34 },
    { 0x3fa68a3e, 61 },
    { 0xc5168a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68168a4b, 98 },
    { 0xd2b68a4c, 99 },
    { 0x69c68a4d, 91 },
    { 0x3fe68a4d, 40 },
    { 0xda268a4f, 33 },
    { 0x93768a4f, 85 },
    { 0xa4468a51, 33 },
    { 0xbe368a52, 90 },
    { 0xca068a51, 23 },
    { 0xd3468a4c, 66 },
    { 0x18668a55, 90 },
    { 0x3d868a55, 52 },
    { 0x30168a53, 33 },
    { 0xc9268a58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c368a5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd568a61, 58 },
    { 0xcf968a61, 84 },
    { 0, 0 },
    { 0x6fd68a64, 55 },
    { 0, 0 },
    { 0x5a68a66, 33 },
    { 0xc5a68a67, 90 },
    { 0x6d068a68, 23 },
    { 0x13468a69, 95 },
    { 0x71568a68, 95 },
    { 0x75e68a6b, 33 },
    { 0xe6668a68, 43 },
    { 0x76868a68, 78 },
    { 0x90468a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a668a71, 23 },
    { 0x2fc68a72, 33 },
    { 0x12868a72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e268a77, 53 },
    { 0x3168a78, 33 },
    { 0x93f68a78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c68a7d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7968a81, 76 },
    { 0xdc368a82, 46 },
    { 0xf3668a83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e68a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a568a9d, 66 },
    { 0, 0 },
    { 0xc7268a9f, 33 },
    { 0xdc768aa0, 46 },
    { 0, 0 },
    { 0xff568aa2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a68aa8, 98 },
    { 0, 0 },
    { 0xeb68aaa, 98 },
    { 0x68f68aaa, 68 },
    { 0, 0 },
    { 0x86068aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c668ab1, 95 },
    { 0xbcd68ab2, 98 },
    { 0xc5168ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb68ab8, 98 },
    { 0xe3468ab8, 78 },
    { 0x4d868ab9, 84 },
    { 0, 0 },
    { 0x80568abc, 66 },
    { 0x24168abd, 23 },
    { 0xf2968abe, 90 },
    { 0x9a468abd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17468ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9768ace, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d68ad3, 34 },
    { 0x6dc68ad4, 61 },
    { 0x78568ad5, 33 },
    { 0, 0 },
    { 0x8368ad7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4368ada, 76 },
    { 0x88e68adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63268adf, 55 },
    { 0xd6f68ae0, 61 },
    { 0x18d68ae1, 76 },
    { 0xa3668ae0, 83 },
    { 0x58568ae1, 100 },
    { 0x95368ae4, 68 },
    { 0x13768ae0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42868ae9, 90 },
    { 0x6ef68aea, 90 },
    { 0xfaa68aea, 64 },
    { 0x92d68aec, 61 },
    { 0xc5968aeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xad968af0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0068af5, 67 },
    { 0x16568af6, 90 },
    { 0x24168af7, 33 },
    { 0xf2868af7, 90 },
    { 0x91168af7, 91 },
    { 0x9af68af7, 46 },
    { 0x94e68afb, 95 },
    { 0xda468af8, 23 },
    { 0, 0 },
    { 0x6a368afe, 33 },
    { 0, 0 },
    { 0x66d68b00, 55 },
    { 0x3d68b00, 33 },
    { 0, 0 },
    { 0x7e368b03, 66 },
    { 0, 0 },
    { 0x4e468b05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c68b0b, 61 },
    { 0x27d68b0c, 95 },
    { 0x62368b0d, 33 },
    { 0x91f68b0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7068b11, 23 },
    { 0x6d168b12, 55 },
    { 0x2af68b11, 100 },
    { 0x99368b14, 90 },
    { 0x73468b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab268b19, 98 },
    { 0xcdc68b19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e68b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda68b21, 33 },
    { 0xde868b22, 98 },
    { 0, 0 },
    { 0xa1c68b24, 95 },
    { 0x6a568b25, 52 },
    { 0x5b368b26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a68b2d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60468b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9468b35, 76 },
    { 0xa7568b35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1168b43, 90 },
    { 0, 0 },
    { 0x50868b45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb68b48, 61 },
    { 0, 0 },
    { 0x51068b4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d68b50, 61 },
    { 0xf0268b51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c68b54, 61 },
    { 0x1df68b54, 33 },
    { 0xd6968b56, 56 },
    { 0xed968b57, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e68b5e, 90 },
    { 0xff168b5f, 98 },
    { 0x2e668b60, 91 },
    { 0xa2468b61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b68b64, 23 },
    { 0, 0 },
    { 0xa8568b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf668b6b, 76 },
    { 0, 0 },
    { 0x6e568b6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd668b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a468b86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e68b89, 85 },
    { 0x36b68b8a, 85 },
    { 0, 0 },
    { 0x6bc68b8c, 34 },
    { 0x4b768b8c, 61 },
    { 0xb5168b8c, 98 },
    { 0, 0 },
    { 0x12168b90, 23 },
    { 0xd5768b91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f168b95, 88 },
    { 0xeef68b95, 88 },
    { 0x98668b96, 23 },
    { 0x48668b98, 90 },
    { 0x2b568b99, 61 },
    { 0xd7468b9a, 95 },
    { 0xfe868b97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12868b9e, 61 },
    { 0x7ec68b9f, 91 },
    { 0xde668b9e, 33 },
    { 0x70768ba0, 61 },
    { 0x48668ba2, 34 },
    { 0xc2d68ba0, 33 },
    { 0x2e768b9e, 33 },
    { 0x51f68ba5, 33 },
    { 0x53d68ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62968bab, 95 },
    { 0x16768bab, 98 },
    { 0, 0 },
    { 0x2bc68bae, 23 },
    { 0xc7068baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59968bb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb668bb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb068bba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4568bbf, 34 },
    { 0, 0 },
    { 0x69a68bc1, 90 },
    { 0, 0 },
    { 0x57e68bc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e68bcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a68bd3, 81 },
    { 0, 0 },
    { 0x6868bd5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb368bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e68bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e68be6, 98 },
    { 0x1db68be7, 95 },
    { 0, 0 },
    { 0x27468be9, 23 },
    { 0, 0 },
    { 0x96f68beb, 33 },
    { 0x14568bec, 33 },
    { 0x71e68beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e68bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e68bf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b68c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c68c05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d68c08, 65 },
    { 0x58768c09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b968c15, 95 },
    { 0x88868c16, 61 },
    { 0xcc068c17, 61 },
    { 0x36168c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd568c1b, 55 },
    { 0x7a68c1c, 87 },
    { 0, 0 },
    { 0x33668c1e, 61 },
    { 0, 0 },
    { 0x10668c20, 66 },
    { 0xe9d68c20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0768c2a, 95 },
    { 0x35968c2b, 52 },
    { 0xab68c2b, 23 },
    { 0, 0 },
    { 0x5d668c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2668c31, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab68c36, 46 },
    { 0xc7168c37, 55 },
    { 0xd0568c36, 90 },
    { 0, 0 },
    { 0x25a68c3a, 98 },
    { 0x57e68c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d168c3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25368c44, 88 },
    { 0, 0 },
    { 0xc8b68c46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc668c49, 77 },
    { 0, 0 },
    { 0xfff68c4b, 81 },
    { 0, 0 },
    { 0xe8a68c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6668c51, 33 },
    { 0x64d68c52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e268c58, 61 },
    { 0, 0 },
    { 0x78968c5a, 33 },
    { 0xca168c5a, 23 },
    { 0xd2c68c5c, 33 },
    { 0x76868c5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf668c60, 61 },
    { 0x2ac68c61, 61 },
    { 0x7268c60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e68c67, 23 },
    { 0xc468c68, 61 },
    { 0xea68c68, 55 },
    { 0x1f168c68, 95 },
    { 0xfee68c6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8168c6e, 33 },
    { 0x36868c6f, 98 },
    { 0x2e68c70, 77 },
    { 0, 0 },
    { 0xafe68c72, 23 },
    { 0x1de68c73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd968c76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0468c79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c768c7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c68c82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3468c85, 76 },
    { 0x14468c85, 33 },
    { 0x28268c87, 75 },
    { 0x8cc68c88, 95 },
    { 0xa7068c89, 84 },
    { 0x42e68c8a, 75 },
    { 0x57868c8b, 23 },
    { 0x94a68c8c, 100 },
    { 0, 0 },
    { 0x5c968c8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e568c92, 55 },
    { 0xc8968c93, 61 },
    { 0x31368c92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68068c97, 98 },
    { 0, 0 },
    { 0x76268c99, 55 },
    { 0xe8d68c9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96068c9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f68ca4, 100 },
    { 0xc8868ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39568ca8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f68cad, 98 },
    { 0xe6468cad, 61 },
    { 0x60568caf, 84 },
    { 0x6a468cb0, 61 },
    { 0x62e68cb1, 65 },
    { 0xa5368cb2, 55 },
    { 0x9bf68cb3, 78 },
    { 0xa6868cb4, 98 },
    { 0, 0 },
    { 0x4aa68cb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa68cb9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc868cbd, 76 },
    { 0xd2b68cbd, 90 },
    { 0xf1668cbd, 61 },
    { 0, 0 },
    { 0x47068cc1, 93 },
    { 0, 0 },
    { 0xc5368cc3, 55 },
    { 0, 0 },
    { 0x22b68cc5, 100 },
    { 0, 0 },
    { 0x20c68cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27968ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da68cd1, 95 },
    { 0, 0 },
    { 0x31668cd3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16268cd9, 90 },
    { 0x53768cda, 90 },
    { 0, 0 },
    { 0x96f68cdc, 98 },
    { 0x22b68cdd, 90 },
    { 0, 0 },
    { 0x94a68cdf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f68ce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88468ce5, 48 },
    { 0, 0 },
    { 0x62668ce7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b368ced, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf68cf1, 93 },
    { 0xfd668cf2, 93 },
    { 0x1d968cf3, 23 },
    { 0x96968cf4, 66 },
    { 0x90a68cf5, 100 },
    { 0x19968cf6, 33 },
    { 0, 0 },
    { 0x46868cf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c668d00, 23 },
    { 0x3fe68d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6668d09, 46 },
    { 0x9868d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b68d11, 90 },
    { 0, 0 },
    { 0x5a068d13, 61 },
    { 0xfaf68d14, 36 },
    { 0, 0 },
    { 0xe6068d16, 61 },
    { 0xd4268d17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b68d1d, 23 },
    { 0x10868d1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4068d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f168d29, 23 },
    { 0, 0 },
    { 0xae868d2b, 90 },
    { 0x72768d2c, 90 },
    { 0xca168d2c, 55 },
    { 0x4a868d2e, 69 },
    { 0, 0 },
    { 0xff968d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc168d37, 34 },
    { 0, 0 },
    { 0x44568d39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f68d3d, 76 },
    { 0x4a968d3e, 100 },
    { 0xf3668d3f, 95 },
    { 0x8668d3f, 98 },
    { 0x24768d3e, 33 },
    { 0x5d668d3d, 87 },
    { 0x92568d43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d468d48, 95 },
    { 0xf8668d48, 90 },
    { 0x8ea68d4a, 95 },
    { 0xedb68d4a, 95 },
    { 0xbde68d4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d68d4f, 64 },
    { 0, 0 },
    { 0xf6668d51, 61 },
    { 0x2cd68d52, 33 },
    { 0x6a068d53, 36 },
    { 0x1a468d54, 40 },
    { 0xca68d52, 88 },
    { 0xe9368d56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce68d5d, 43 },
    { 0x72768d5e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6468d63, 61 },
    { 0x8068d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc68d67, 61 },
    { 0xf4b68d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268d6b, 23 },
    { 0x48768d6c, 66 },
    { 0xfae68d6c, 17 },
    { 0xbe668d6e, 44 },
    { 0x93568d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c68d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc68d83, 99 },
    { 0xaa568d84, 61 },
    { 0x79968d84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56968d96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91268d9d, 23 },
    { 0xe4868d9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde168da3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f68da7, 61 },
    { 0xd1d68da7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4968dab, 23 },
    { 0x9e668dac, 46 },
    { 0xa9468dad, 34 },
    { 0, 0 },
    { 0x42368daf, 90 },
    { 0x94668db0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d468db3, 76 },
    { 0xf3668db4, 55 },
    { 0xb1568db5, 90 },
    { 0xb7a68db3, 95 },
    { 0xfa768db6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc668dbe, 95 },
    { 0x74968dbf, 99 },
    { 0x5968dc0, 90 },
    { 0xf5768dbe, 76 },
    { 0, 0 },
    { 0x59368dc3, 98 },
    { 0, 0 },
    { 0x9e068dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b68dcb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8868dd2, 100 },
    { 0xe2b68dd3, 88 },
    { 0x3db68dd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b68ddb, 61 },
    { 0x41668ddc, 53 },
    { 0, 0 },
    { 0x4ec68dde, 76 },
    { 0xed468dde, 90 },
    { 0, 0 },
    { 0x13768de1, 76 },
    { 0x9c768de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d68de5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d68de8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x31768deb, 95 },
    { 0x77f68deb, 84 },
    { 0x7ea68ded, 94 },
    { 0x75c68ded, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c68df4, 78 },
    { 0xe4368df4, 76 },
    { 0xe0f68df5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc868df9, 43 },
    { 0, 0 },
    { 0x6ea68dfb, 43 },
    { 0x4fd68dfb, 61 },
    { 0, 0 },
    { 0xb0168dfe, 78 },
    { 0x2d768dff, 95 },
    { 0xf1e68e00, 76 },
    { 0x69268e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0268e09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6868e0c, 93 },
    { 0, 0 },
    { 0x8a968e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef568e1c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab868e22, 36 },
    { 0x26668e23, 90 },
    { 0x51c68e24, 93 },
    { 0x1a468e25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4068e28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54068e2c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e368e2f, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8068e32, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46168e3a, 61 },
    { 0x5d968e3b, 95 },
    { 0x4d068e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c468e4d, 93 },
    { 0, 0 },
    { 0xab268e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e968e54, 98 },
    { 0xbc068e55, 23 },
    { 0x9ad68e56, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6968e5a, 66 },
    { 0x4f768e5a, 90 },
    { 0xc6d68e5c, 76 },
    { 0xc5f68e5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19268e61, 68 },
    { 0x4468e62, 94 },
    { 0, 0 },
    { 0xa5a68e64, 33 },
    { 0, 0 },
    { 0x85368e66, 23 },
    { 0x27b68e67, 81 },
    { 0, 0 },
    { 0x65f68e69, 95 },
    { 0xa1c68e6a, 84 },
    { 0x62068e6a, 99 },
    { 0xa3068e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a968e75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a68e7a, 84 },
    { 0x2f668e7a, 88 },
    { 0xd0f68e7a, 95 },
    { 0xe7668e7c, 98 },
    { 0xab68e7a, 34 },
    { 0x13368e7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83868e86, 87 },
    { 0xc5768e87, 76 },
    { 0xa1768e88, 87 },
    { 0x60468e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a68e8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f68e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f968e9c, 77 },
    { 0x17368e9c, 78 },
    { 0x9fd68e9e, 23 },
    { 0xb5d68e9f, 24 },
    { 0xfd768ea0, 33 },
    { 0x58968ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2368ea5, 23 },
    { 0x4cb68ea6, 90 },
    { 0, 0 },
    { 0x8f268ea8, 98 },
    { 0x4e068ea9, 91 },
    { 0x66b68eaa, 61 },
    { 0x31668eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2568eb0, 90 },
    { 0x2f868eb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e68eb8, 77 },
    { 0x4ac68eb8, 61 },
    { 0x6c768eb8, 61 },
    { 0, 0 },
    { 0x61568ebc, 34 },
    { 0xc7f68ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7068ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70168ec6, 46 },
    { 0, 0 },
    { 0xde268ec8, 61 },
    { 0x71168ec8, 95 },
    { 0x41468eca, 52 },
    { 0, 0 },
    { 0xf9368ecc, 66 },
    { 0, 0 },
    { 0xe6068ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a68ed6, 34 },
    { 0xc8868ed7, 33 },
    { 0x1a668ed7, 33 },
    { 0x32668ed8, 98 },
    { 0xc8468eda, 40 },
    { 0x4e168ed8, 17 },
    { 0x7fd68edc, 33 },
    { 0x70468edd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53768ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b68ef2, 93 },
    { 0x3b868ef2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb68ef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b68f03, 85 },
    { 0xa5668f03, 29 },
    { 0xc5668f05, 87 },
    { 0x95168f05, 100 },
    { 0x87868f07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61268f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8568f17, 90 },
    { 0x1b68f17, 80 },
    { 0x56468f19, 66 },
    { 0xa1268f1a, 76 },
    { 0x5e568f1b, 33 },
    { 0xa9a68f1c, 34 },
    { 0x22b68f1d, 98 },
    { 0xe0b68f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7468f22, 77 },
    { 0x56768f22, 61 },
    { 0x19f68f24, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c68f27, 90 },
    { 0, 0 },
    { 0xc3d68f29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93668f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c68f31, 43 },
    { 0, 0 },
    { 0xee068f33, 100 },
    { 0xd068f33, 33 },
    { 0xa3b68f34, 34 },
    { 0, 0 },
    { 0x90e68f37, 95 },
    { 0x6e568f38, 33 },
    { 0x49968f38, 34 },
    { 0x8f968f37, 40 },
    { 0, 0 },
    { 0xa8468f3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f768f42, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed568f48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5768f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c68f53, 98 },
    { 0x67a68f54, 23 },
    { 0, 0 },
    { 0x6dd68f56, 91 },
    { 0, 0 },
    { 0xe3668f58, 55 },
    { 0x7c368f59, 99 },
    { 0xc2e68f5a, 61 },
    { 0xcf468f59, 61 },
    { 0x8db68f5c, 98 },
    { 0x2a968f5d, 66 },
    { 0xf4068f5c, 61 },
    { 0x9c868f5f, 98 },
    { 0xda368f60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39368f64, 66 },
    { 0xb9468f64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f68f68, 100 },
    { 0xb0668f68, 34 },
    { 0x63b68f68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa268f70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e068f7a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d68f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d68f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec68f86, 98 },
    { 0, 0 },
    { 0x19268f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71668f8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a568f90, 90 },
    { 0xd4068f91, 90 },
    { 0, 0 },
    { 0xb768f93, 93 },
    { 0, 0 },
    { 0xc7568f95, 23 },
    { 0xb2f68f95, 95 },
    { 0x53968f96, 66 },
    { 0, 0 },
    { 0x4db68f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b68f9c, 65 },
    { 0x87568f9d, 87 },
    { 0x39f68f9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d68fa3, 33 },
    { 0xbdd68fa4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0168fa7, 61 },
    { 0, 0 },
    { 0x3e368fa9, 90 },
    { 0x94b68faa, 67 },
    { 0x89e68fa9, 61 },
    { 0xada68fac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf68faf, 81 },
    { 0xceb68faf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3968fbc, 23 },
    { 0x62568fbc, 17 },
    { 0, 0 },
    { 0xc0768fbf, 61 },
    { 0xf068fbf, 46 },
    { 0x2cd68fc1, 76 },
    { 0x2b68fc1, 34 },
    { 0xb1f68fc1, 33 },
    { 0x86168fc4, 99 },
    { 0x9ce68fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed68fc8, 34 },
    { 0xd3468fc9, 90 },
    { 0, 0 },
    { 0xfc68fcb, 89 },
    { 0x33068fcc, 65 },
    { 0x26c68fcd, 33 },
    { 0, 0 },
    { 0x95568fcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b68fd5, 43 },
    { 0, 0 },
    { 0xc6d68fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0868fe3, 23 },
    { 0x7c268fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c468fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc68fea, 61 },
    { 0, 0 },
    { 0x2e568fec, 84 },
    { 0x53e68fed, 81 },
    { 0xb368fee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81168ff3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf368ff7, 98 },
    { 0xeeb68ff8, 76 },
    { 0, 0 },
    { 0x94a68ffa, 23 },
    { 0x79668ffb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c68fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4869003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b69007, 85 },
    { 0, 0 },
    { 0x3f469009, 46 },
    { 0xb666900a, 90 },
    { 0x20f6900a, 76 },
    { 0x32f6900c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed269010, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd269014, 95 },
    { 0, 0 },
    { 0x3969016, 98 },
    { 0, 0 },
    { 0xdc869018, 34 },
    { 0x2c469019, 95 },
    { 0x3a96901a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d36901f, 41 },
    { 0xde269020, 93 },
    { 0x9036901f, 66 },
    { 0x99369020, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc69026, 77 },
    { 0x40b69027, 80 },
    { 0, 0 },
    { 0xef769029, 23 },
    { 0, 0 },
    { 0xf5a6902b, 85 },
    { 0, 0 },
    { 0x29f6902d, 23 },
    { 0x5d66902e, 61 },
    { 0, 0 },
    { 0x1b869030, 95 },
    { 0x2d469031, 76 },
    { 0x14b69030, 33 },
    { 0x52069033, 61 },
    { 0xc1669031, 33 },
    { 0xb0d69030, 61 },
    { 0x6c169036, 100 },
    { 0x2ec69035, 61 },
    { 0x93c69038, 43 },
    { 0xd7369039, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f69041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b469044, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa669048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85769050, 23 },
    { 0, 0 },
    { 0xa5269052, 75 },
    { 0x69b69052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8669056, 90 },
    { 0x44369057, 23 },
    { 0x15269056, 34 },
    { 0xdf169056, 90 },
    { 0xe2969059, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4769061, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44669068, 87 },
    { 0xb5069069, 93 },
    { 0xe286906a, 76 },
    { 0xfb56906b, 76 },
    { 0x49069069, 95 },
    { 0x7c969068, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca69073, 90 },
    { 0x51269074, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b69078, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48469083, 90 },
    { 0xf7969083, 90 },
    { 0x36a69085, 65 },
    { 0, 0 },
    { 0xf2c69087, 78 },
    { 0x30269088, 61 },
    { 0xf469089, 23 },
    { 0x11e6908a, 91 },
    { 0x9316908b, 101 },
    { 0xc536908c, 90 },
    { 0xa9c6908d, 76 },
    { 0xa46908a, 90 },
    { 0xd0c69088, 98 },
    { 0x1f669090, 98 },
    { 0x81169091, 95 },
    { 0x59769089, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d369097, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266909a, 58 },
    { 0, 0 },
    { 0x1b66909c, 93 },
    { 0, 0 },
    { 0xc516909e, 98 },
    { 0x3f96909e, 90 },
    { 0x87e6909f, 55 },
    { 0x339690a1, 90 },
    { 0x2b1690a2, 23 },
    { 0xc47690a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0690a7, 34 },
    { 0xd39690a8, 61 },
    { 0xeb5690a9, 33 },
    { 0xab6690aa, 90 },
    { 0x579690ab, 34 },
    { 0x3fa690ac, 93 },
    { 0x414690a8, 40 },
    { 0xeca690ae, 65 },
    { 0x97a690ae, 68 },
    { 0x6a9690ae, 34 },
    { 0x3ac690b1, 43 },
    { 0, 0 },
    { 0x926690b3, 76 },
    { 0x423690b3, 93 },
    { 0xd9b690b4, 40 },
    { 0x192690b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x265690b9, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0690bf, 83 },
    { 0, 0 },
    { 0xd3b690c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1690c6, 61 },
    { 0, 0 },
    { 0x6c6690c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3690cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e690d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a690d9, 43 },
    { 0xec0690d9, 90 },
    { 0x566690da, 87 },
    { 0, 0 },
    { 0xb7c690dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0690e0, 90 },
    { 0x465690e1, 76 },
    { 0x737690e2, 33 },
    { 0xfd0690e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a690e8, 90 },
    { 0xd31690e9, 55 },
    { 0xf54690e9, 98 },
    { 0x755690eb, 98 },
    { 0x82c690ec, 33 },
    { 0x782690ea, 19 },
    { 0, 0 },
    { 0x21e690ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76690f5, 90 },
    { 0, 0 },
    { 0x10b690f7, 81 },
    { 0x448690f7, 46 },
    { 0x8c0690f7, 98 },
    { 0xc3b690fa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce690ff, 95 },
    { 0xfab690ff, 98 },
    { 0, 0 },
    { 0xff969102, 95 },
    { 0x4af69102, 24 },
    { 0, 0 },
    { 0xc6969105, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1369108, 95 },
    { 0, 0 },
    { 0xb056910a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f169110, 76 },
    { 0, 0 },
    { 0xfd669112, 90 },
    { 0, 0 },
    { 0xe1069114, 61 },
    { 0, 0 },
    { 0x62269116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce69119, 34 },
    { 0x5786911a, 88 },
    { 0, 0 },
    { 0x8cf6911c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd036911f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2369126, 55 },
    { 0, 0 },
    { 0xa7169128, 95 },
    { 0xa169129, 99 },
    { 0x6bc6912a, 90 },
    { 0, 0 },
    { 0xa786912c, 90 },
    { 0x4ac6912d, 93 },
    { 0x3b56912e, 55 },
    { 0x4856912f, 33 },
    { 0x5a69130, 55 },
    { 0x2d469130, 95 },
    { 0xfd16912d, 85 },
    { 0x2169131, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94269137, 33 },
    { 0, 0 },
    { 0x78d69139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd869142, 61 },
    { 0x4a69143, 43 },
    { 0x92469144, 55 },
    { 0x70e69142, 33 },
    { 0x85269146, 90 },
    { 0xd0469146, 55 },
    { 0, 0 },
    { 0x3ba69149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb16914d, 46 },
    { 0, 0 },
    { 0x1776914f, 23 },
    { 0xc369150, 90 },
    { 0, 0 },
    { 0xc3f69152, 66 },
    { 0x58369153, 23 },
    { 0, 0 },
    { 0x2f869155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b69159, 84 },
    { 0x77a6915a, 55 },
    { 0, 0 },
    { 0x99a6915c, 23 },
    { 0x6d46915c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64469160, 52 },
    { 0, 0 },
    { 0x70369162, 76 },
    { 0x39d69163, 76 },
    { 0x22869164, 91 },
    { 0xd1a69164, 66 },
    { 0x9e869165, 23 },
    { 0x8e069165, 34 },
    { 0x43969168, 43 },
    { 0x5f769169, 61 },
    { 0xc0669169, 98 },
    { 0, 0 },
    { 0x92b6916c, 33 },
    { 0xff6916c, 100 },
    { 0x19a6916e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1569179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6917c, 33 },
    { 0, 0 },
    { 0x1a96917e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc769185, 43 },
    { 0x5d869185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7176918f, 98 },
    { 0x8656918f, 81 },
    { 0, 0 },
    { 0xa2169192, 23 },
    { 0x73c69192, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d69197, 95 },
    { 0xfa069198, 61 },
    { 0xd7a69198, 23 },
    { 0, 0 },
    { 0x1b06919b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a691a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62691a7, 95 },
    { 0x886691a7, 95 },
    { 0x3f4691a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2691ac, 81 },
    { 0, 0 },
    { 0xf3f691ae, 33 },
    { 0, 0 },
    { 0x5e6691b0, 61 },
    { 0x909691b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x413691b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab691bd, 90 },
    { 0xcc4691be, 65 },
    { 0xf5b691bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44691c4, 65 },
    { 0, 0 },
    { 0x7a8691c6, 23 },
    { 0xc46691c6, 46 },
    { 0, 0 },
    { 0x6b691c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3691cc, 61 },
    { 0x8fe691cc, 61 },
    { 0x4f1691ce, 33 },
    { 0xad8691ce, 61 },
    { 0xe28691d0, 52 },
    { 0x32e691cf, 34 },
    { 0xd1691cd, 95 },
    { 0x60c691d3, 55 },
    { 0x221691d3, 84 },
    { 0x8fb691d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3691d8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x507691db, 33 },
    { 0x303691dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa691e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c691eb, 34 },
    { 0x62b691eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397691f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e691fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375691ff, 91 },
    { 0x27c69200, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d669206, 67 },
    { 0, 0 },
    { 0x1c869208, 91 },
    { 0xbeb69209, 76 },
    { 0x84a6920a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba66920d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9169210, 33 },
    { 0, 0 },
    { 0x7c169212, 97 },
    { 0x63e69212, 77 },
    { 0x67b69213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34969218, 96 },
    { 0xc8969219, 99 },
    { 0x7e66921a, 55 },
    { 0x6bb6921b, 33 },
    { 0, 0 },
    { 0x9c36921d, 61 },
    { 0x40f6921e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc69224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c269228, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2569231, 34 },
    { 0xf1169232, 98 },
    { 0x47369232, 61 },
    { 0, 0 },
    { 0x87569235, 76 },
    { 0x3a769235, 33 },
    { 0, 0 },
    { 0x3c669238, 90 },
    { 0, 0 },
    { 0xe6923a, 33 },
    { 0x3a66923a, 84 },
    { 0xb2c6923c, 84 },
    { 0, 0 },
    { 0x8306923e, 90 },
    { 0x1396923f, 69 },
    { 0x3756923f, 93 },
    { 0x12b6923f, 90 },
    { 0xb1269242, 34 },
    { 0x9a69242, 33 },
    { 0, 0 },
    { 0xcaa69245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78469251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e069257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a36925b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976925f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8769262, 33 },
    { 0, 0 },
    { 0x2569264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569268, 91 },
    { 0x65769269, 36 },
    { 0xe9769269, 95 },
    { 0x8486926b, 65 },
    { 0xf426926c, 55 },
    { 0x29b6926d, 95 },
    { 0x5fb6926e, 33 },
    { 0x10c6926b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede69278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa6927e, 61 },
    { 0, 0 },
    { 0x10c69280, 99 },
    { 0, 0 },
    { 0x23e69282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede69286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef86928e, 32 },
    { 0xfe16928e, 33 },
    { 0x5d06928f, 95 },
    { 0xfe969291, 90 },
    { 0xbde69292, 67 },
    { 0, 0 },
    { 0xc9e69294, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe169297, 61 },
    { 0x6169298, 90 },
    { 0, 0 },
    { 0xa266929a, 90 },
    { 0, 0 },
    { 0xa9e6929c, 17 },
    { 0xd706929d, 23 },
    { 0, 0 },
    { 0x3cb6929f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951692a6, 51 },
    { 0, 0 },
    { 0x5f3692a8, 90 },
    { 0x5d0692a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480692ae, 34 },
    { 0xf0f692af, 34 },
    { 0xd1692af, 90 },
    { 0x718692ae, 46 },
    { 0, 0 },
    { 0x9c2692b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xede692b6, 61 },
    { 0xff2692b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71692bd, 95 },
    { 0xe7b692be, 61 },
    { 0xe58692bf, 23 },
    { 0x523692c0, 40 },
    { 0xd4c692c0, 33 },
    { 0x9a9692c2, 56 },
    { 0, 0 },
    { 0xb6c692c4, 61 },
    { 0, 0 },
    { 0xc6692c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776692ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae692cf, 52 },
    { 0x7c692d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f692d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd692d9, 90 },
    { 0x915692da, 66 },
    { 0xfc1692db, 23 },
    { 0x83d692da, 34 },
    { 0, 0 },
    { 0xfda692de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34692e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d692e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x587692ec, 23 },
    { 0x44c692ed, 90 },
    { 0x3fe692ee, 55 },
    { 0x7bf692ee, 90 },
    { 0xd7c692ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18692f4, 77 },
    { 0x4c6692f4, 76 },
    { 0x961692f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03692f9, 61 },
    { 0xc8692f9, 61 },
    { 0, 0 },
    { 0xe2f692fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a69301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69169304, 91 },
    { 0xeec69305, 61 },
    { 0x6ae69304, 80 },
    { 0xab169307, 100 },
    { 0x19869308, 43 },
    { 0xbde69307, 84 },
    { 0xd3569309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8626930e, 22 },
    { 0x6ba6930f, 23 },
    { 0, 0 },
    { 0x96969311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93269315, 61 },
    { 0x4eb69316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a6931d, 55 },
    { 0x2326931e, 90 },
    { 0x8756931f, 96 },
    { 0x77a69320, 34 },
    { 0xf946931f, 43 },
    { 0xed569320, 23 },
    { 0, 0 },
    { 0x71669324, 76 },
    { 0x65969324, 90 },
    { 0xee869326, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf926932a, 61 },
    { 0x4326932b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9966932f, 95 },
    { 0x7c469330, 94 },
    { 0, 0 },
    { 0xdea69332, 76 },
    { 0, 0 },
    { 0x8a69334, 98 },
    { 0x46869335, 65 },
    { 0xf2469336, 46 },
    { 0x12669337, 90 },
    { 0xbba69338, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e16933b, 90 },
    { 0xe66933c, 90 },
    { 0x2eb6933c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e369340, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88969348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe416934e, 65 },
    { 0xd356934f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb69354, 43 },
    { 0x36c69355, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26935b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f6935e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd569363, 23 },
    { 0x3869364, 93 },
    { 0, 0 },
    { 0x7e069366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6936b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6869372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1169375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33969378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a6937c, 23 },
    { 0xde46937d, 76 },
    { 0x8266937c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75669387, 23 },
    { 0xbc469387, 23 },
    { 0x94b69389, 53 },
    { 0xdb56938a, 33 },
    { 0x66069389, 99 },
    { 0, 0 },
    { 0xf126938d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd069393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b69397, 23 },
    { 0x16369398, 61 },
    { 0xca269399, 95 },
    { 0xde669399, 23 },
    { 0xa6169399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f6939e, 55 },
    { 0, 0 },
    { 0x932693a0, 43 },
    { 0, 0 },
    { 0x822693a2, 76 },
    { 0x978693a2, 46 },
    { 0x20d693a4, 52 },
    { 0x3e9693a2, 98 },
    { 0x84d693a6, 85 },
    { 0x682693a6, 33 },
    { 0, 0 },
    { 0xd9f693a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b693ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c693b5, 61 },
    { 0xe47693b5, 98 },
    { 0x7b7693b6, 46 },
    { 0xe03693b7, 87 },
    { 0x2e8693b7, 61 },
    { 0x289693ba, 61 },
    { 0xb7e693bb, 84 },
    { 0xcf6693bc, 87 },
    { 0x2a4693bd, 90 },
    { 0x6d8693bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf693c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570693c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c693cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x147693cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1693d3, 33 },
    { 0, 0 },
    { 0x232693d5, 87 },
    { 0xded693d6, 33 },
    { 0x79c693d7, 33 },
    { 0xc49693d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983693df, 90 },
    { 0x998693e0, 33 },
    { 0xb32693e1, 23 },
    { 0x803693e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6693e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb693ee, 90 },
    { 0xae8693ee, 93 },
    { 0, 0 },
    { 0xcd9693f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129693f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41693fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af693fd, 33 },
    { 0x9d0693fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc69402, 61 },
    { 0x89269403, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1169408, 70 },
    { 0, 0 },
    { 0xd9a6940a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4016940d, 34 },
    { 0x69f6940e, 87 },
    { 0x48a6940f, 90 },
    { 0xafc69410, 95 },
    { 0, 0 },
    { 0x37969412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d69418, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9106941d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75769421, 78 },
    { 0x65269421, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f69425, 90 },
    { 0xc0669426, 88 },
    { 0x2dc69425, 66 },
    { 0xf1169425, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f36942f, 23 },
    { 0xba669430, 68 },
    { 0x94f69431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c69435, 93 },
    { 0x4169435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6943d, 23 },
    { 0x73d6943e, 95 },
    { 0xa556943e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8169448, 46 },
    { 0, 0 },
    { 0xe746944a, 61 },
    { 0x38d6944a, 46 },
    { 0, 0 },
    { 0x8d86944d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81069450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac869457, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb69462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80969465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34569469, 95 },
    { 0, 0 },
    { 0x3d46946b, 95 },
    { 0x9956946c, 78 },
    { 0x96b6946c, 95 },
    { 0x3756946c, 90 },
    { 0x9256946f, 55 },
    { 0xd306946d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816947d, 33 },
    { 0x2ad6947e, 64 },
    { 0xc4d6947f, 90 },
    { 0x1af69480, 33 },
    { 0, 0 },
    { 0x8af69482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda169488, 100 },
    { 0xe2469488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf206948c, 98 },
    { 0, 0 },
    { 0xd676948e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13669491, 76 },
    { 0xff569492, 61 },
    { 0xba169492, 61 },
    { 0x59269494, 95 },
    { 0xc6e69495, 98 },
    { 0x7ac69496, 98 },
    { 0x8f769496, 23 },
    { 0, 0 },
    { 0xa6a69499, 100 },
    { 0, 0 },
    { 0xd466949b, 90 },
    { 0, 0 },
    { 0x8386949d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a694a0, 77 },
    { 0, 0 },
    { 0xf1f694a2, 61 },
    { 0x50a694a3, 33 },
    { 0xb47694a3, 76 },
    { 0x109694a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06694a8, 98 },
    { 0x1e2694a9, 98 },
    { 0x4f2694a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0694af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f694b3, 91 },
    { 0x435694b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5694bb, 95 },
    { 0xac1694bc, 95 },
    { 0, 0 },
    { 0x12f694be, 93 },
    { 0xb9d694bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a694ca, 33 },
    { 0xecc694cb, 97 },
    { 0xd84694ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8694d0, 55 },
    { 0, 0 },
    { 0xe73694d2, 95 },
    { 0x239694d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9694d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111694dd, 84 },
    { 0x5ab694de, 33 },
    { 0xebc694de, 33 },
    { 0xfab694df, 25 },
    { 0x188694de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab694e5, 98 },
    { 0xd7694e5, 33 },
    { 0x91f694e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x738694ea, 75 },
    { 0x1de694eb, 46 },
    { 0x127694eb, 36 },
    { 0x126694ea, 23 },
    { 0x683694ee, 76 },
    { 0x4eb694ef, 91 },
    { 0, 0 },
    { 0xf68694f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde694f5, 95 },
    { 0, 0 },
    { 0xd0f694f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880694fc, 78 },
    { 0, 0 },
    { 0x949694fe, 23 },
    { 0, 0 },
    { 0x99a69500, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4669504, 61 },
    { 0xe3b69505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40669509, 98 },
    { 0xdcd6950a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52869513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a69517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0369520, 23 },
    { 0x5b769521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84469524, 43 },
    { 0xaac69525, 61 },
    { 0xaf769526, 34 },
    { 0, 0 },
    { 0x15c69528, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c6952c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f469535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd69538, 33 },
    { 0x5e969539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266953d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6869545, 65 },
    { 0, 0 },
    { 0x11e69547, 33 },
    { 0x61b69547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3156954d, 68 },
    { 0x4a46954e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f69551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b69556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16869559, 61 },
    { 0, 0 },
    { 0xd286955b, 78 },
    { 0, 0 },
    { 0xa8d6955d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d269560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569567, 90 },
    { 0x55169568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3236956b, 90 },
    { 0x866956c, 34 },
    { 0xb16956c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5469571, 51 },
    { 0x85269571, 61 },
    { 0xcb669573, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e69578, 46 },
    { 0, 0 },
    { 0xe756957a, 93 },
    { 0, 0 },
    { 0x8206957c, 95 },
    { 0x7b96957c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c69580, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f469587, 100 },
    { 0x4869588, 33 },
    { 0x4f369589, 87 },
    { 0xc096958a, 90 },
    { 0x3f169589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56958f, 61 },
    { 0x81869590, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4969593, 98 },
    { 0x99169593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f69599, 52 },
    { 0xfdc6959a, 52 },
    { 0xcce69599, 97 },
    { 0x2386959c, 95 },
    { 0xd886959d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871695a3, 34 },
    { 0xd16695a4, 61 },
    { 0, 0 },
    { 0x55d695a6, 34 },
    { 0x5aa695a7, 23 },
    { 0x46d695a7, 90 },
    { 0, 0 },
    { 0xecf695aa, 85 },
    { 0x80d695ab, 66 },
    { 0, 0 },
    { 0xfd3695ad, 33 },
    { 0, 0 },
    { 0xbcd695af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd695b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2695bb, 93 },
    { 0, 0 },
    { 0x2af695bd, 61 },
    { 0x16d695be, 76 },
    { 0xd35695bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03695c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1695c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb695cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8695cf, 46 },
    { 0, 0 },
    { 0xbd1695d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4695d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2695d9, 74 },
    { 0xc64695da, 52 },
    { 0xe88695db, 77 },
    { 0xce5695dc, 95 },
    { 0xe98695dd, 93 },
    { 0xae1695de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf695e1, 91 },
    { 0xc2695e2, 93 },
    { 0x80f695e1, 23 },
    { 0, 0 },
    { 0x72695e5, 33 },
    { 0, 0 },
    { 0x7f3695e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108695eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f695ef, 61 },
    { 0x806695f0, 91 },
    { 0, 0 },
    { 0x457695f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c695f7, 91 },
    { 0, 0 },
    { 0xb8d695f9, 67 },
    { 0xa81695f9, 33 },
    { 0x776695fb, 61 },
    { 0x305695fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4769600, 95 },
    { 0x1f369600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74869608, 34 },
    { 0x3ff69609, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f569610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce169613, 98 },
    { 0xb2569614, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52969618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a6961d, 95 },
    { 0x1306961e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b69621, 90 },
    { 0x10a69622, 95 },
    { 0x6ce69623, 33 },
    { 0x8be69624, 80 },
    { 0xa5169624, 98 },
    { 0x77e69626, 95 },
    { 0x95369627, 90 },
    { 0x8c769623, 46 },
    { 0xad069628, 65 },
    { 0xaf969629, 90 },
    { 0x6ca6962b, 34 },
    { 0xe036962c, 90 },
    { 0xfc86962d, 77 },
    { 0x5ec6962e, 100 },
    { 0xa446962e, 77 },
    { 0x20169628, 93 },
    { 0x20969627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50169635, 90 },
    { 0xc6a69636, 90 },
    { 0, 0 },
    { 0x3e669638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f069643, 43 },
    { 0xf4b69643, 55 },
    { 0x30969645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce669648, 52 },
    { 0x49869649, 76 },
    { 0x4f06964a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f69651, 95 },
    { 0, 0 },
    { 0x3169653, 91 },
    { 0, 0 },
    { 0x70f69655, 17 },
    { 0, 0 },
    { 0xd6369657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f46965e, 33 },
    { 0x43d6965f, 33 },
    { 0xa2e69660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2069665, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e46966d, 99 },
    { 0, 0 },
    { 0xa536966f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c69672, 85 },
    { 0x22869673, 95 },
    { 0x53369673, 76 },
    { 0x40c69675, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a86967c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c69680, 98 },
    { 0, 0 },
    { 0xbfd69682, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b69685, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b69688, 95 },
    { 0, 0 },
    { 0x3b76968a, 34 },
    { 0x54b6968b, 93 },
    { 0x6b6968c, 52 },
    { 0, 0 },
    { 0x2276968e, 33 },
    { 0xf9a6968f, 43 },
    { 0, 0 },
    { 0x90069691, 46 },
    { 0, 0 },
    { 0xd5369693, 23 },
    { 0xd4569694, 17 },
    { 0xcde69695, 95 },
    { 0x5969695, 33 },
    { 0x26269697, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076969b, 90 },
    { 0x9686969c, 100 },
    { 0xc8a6969d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40696a1, 12 },
    { 0x399696a2, 76 },
    { 0xa0a696a3, 95 },
    { 0x94f696a4, 33 },
    { 0, 0 },
    { 0x15b696a6, 55 },
    { 0xa76696a7, 61 },
    { 0x436696a7, 33 },
    { 0x371696a9, 33 },
    { 0x2fe696aa, 34 },
    { 0x5d2696ab, 61 },
    { 0xd1a696ab, 81 },
    { 0x9a1696ab, 88 },
    { 0, 0 },
    { 0xa37696af, 46 },
    { 0x8d0696af, 90 },
    { 0x1fe696b1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332696b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a696b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66696bc, 76 },
    { 0x6f7696bd, 53 },
    { 0, 0 },
    { 0xa6a696bf, 81 },
    { 0, 0 },
    { 0x9a3696c1, 34 },
    { 0x508696c1, 95 },
    { 0, 0 },
    { 0x56d696c4, 69 },
    { 0x58a696c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134696cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7696ce, 99 },
    { 0xf63696ce, 76 },
    { 0x946696d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e696d7, 90 },
    { 0x212696d8, 98 },
    { 0x751696d9, 33 },
    { 0x1dc696d9, 33 },
    { 0xec696db, 68 },
    { 0x8c8696d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30696e1, 33 },
    { 0xd29696e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1696e5, 43 },
    { 0x5e6696e5, 93 },
    { 0xc85696e6, 61 },
    { 0x370696e8, 61 },
    { 0x90e696e9, 81 },
    { 0xd7f696e8, 61 },
    { 0xdd696eb, 90 },
    { 0xb74696ec, 23 },
    { 0x1bd696eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54696f0, 61 },
    { 0, 0 },
    { 0xdbc696f2, 95 },
    { 0x6ba696f2, 100 },
    { 0xdf0696f4, 95 },
    { 0x162696f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9696fd, 33 },
    { 0x6a9696fd, 61 },
    { 0xae2696ff, 33 },
    { 0x821696ff, 95 },
    { 0xe1069701, 93 },
    { 0x25669702, 90 },
    { 0x9e069703, 95 },
    { 0xdfa69700, 93 },
    { 0x23f69705, 88 },
    { 0x57e69705, 95 },
    { 0xc9569707, 90 },
    { 0xc7f69708, 46 },
    { 0x7b669707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6970d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd69711, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29569718, 93 },
    { 0x55e69719, 46 },
    { 0xe1e69719, 95 },
    { 0xd3069719, 23 },
    { 0x3f06971c, 43 },
    { 0xd156971c, 90 },
    { 0, 0 },
    { 0xcc26971f, 23 },
    { 0xc426971f, 90 },
    { 0x3e969721, 34 },
    { 0x2f569722, 33 },
    { 0x74e69723, 52 },
    { 0xc0d69723, 33 },
    { 0xb8269724, 98 },
    { 0x20169726, 61 },
    { 0, 0 },
    { 0x1d869728, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b69731, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56069734, 90 },
    { 0x6ce69735, 56 },
    { 0x1a969736, 43 },
    { 0x24869735, 78 },
    { 0x4a69735, 90 },
    { 0xed669739, 95 },
    { 0, 0 },
    { 0x87f6973b, 90 },
    { 0x2c56973b, 90 },
    { 0x2de6973d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37769742, 98 },
    { 0x5b69743, 46 },
    { 0xf8c69742, 92 },
    { 0x94569742, 90 },
    { 0x54369746, 33 },
    { 0xd7969743, 90 },
    { 0x2f069748, 33 },
    { 0, 0 },
    { 0x4bf6974a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb436974e, 77 },
    { 0xae86974f, 52 },
    { 0x14669750, 90 },
    { 0x2c86974f, 76 },
    { 0x6869751, 55 },
    { 0x6f569753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81069756, 23 },
    { 0xb0069756, 61 },
    { 0xd6269758, 90 },
    { 0x8c169756, 99 },
    { 0xba86975a, 98 },
    { 0, 0 },
    { 0x92d6975c, 34 },
    { 0x8286975d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec869763, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f469767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456976c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12369770, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f69779, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe366977c, 36 },
    { 0x35c6977d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf69782, 87 },
    { 0, 0 },
    { 0xae169784, 33 },
    { 0x95069785, 88 },
    { 0x10c69786, 78 },
    { 0, 0 },
    { 0x69e69788, 55 },
    { 0, 0 },
    { 0x4ea6978a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6896978e, 90 },
    { 0xa7c6978f, 34 },
    { 0x6636978f, 95 },
    { 0xa8469791, 98 },
    { 0x81069792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee69799, 33 },
    { 0, 0 },
    { 0xb7e6979b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d697a2, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x650697a5, 33 },
    { 0, 0 },
    { 0xcd4697a7, 98 },
    { 0x537697a8, 23 },
    { 0x4a8697a9, 60 },
    { 0xad0697a9, 100 },
    { 0, 0 },
    { 0x9dd697ac, 90 },
    { 0xbf697ad, 61 },
    { 0xd03697ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e697b2, 46 },
    { 0xa65697b3, 43 },
    { 0x9cb697b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe697b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443697c0, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c697c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e697cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621697d3, 55 },
    { 0x2d7697d3, 23 },
    { 0xaf2697d5, 61 },
    { 0x731697d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7697db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9697de, 90 },
    { 0xb5c697de, 95 },
    { 0, 0 },
    { 0xbeb697e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a697e6, 98 },
    { 0x2cc697e7, 90 },
    { 0, 0 },
    { 0xf21697e9, 95 },
    { 0xf33697ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a697ef, 55 },
    { 0, 0 },
    { 0xd8d697f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4697f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0697f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5697fc, 55 },
    { 0, 0 },
    { 0x90697fe, 61 },
    { 0xbfc697ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069807, 23 },
    { 0xf1e69808, 85 },
    { 0x3c469809, 90 },
    { 0x9ed6980a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06980d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa69812, 23 },
    { 0xd2d69813, 95 },
    { 0x3bb69814, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586981b, 90 },
    { 0, 0 },
    { 0xe5f6981d, 90 },
    { 0xdfb6981e, 33 },
    { 0x12f6981f, 91 },
    { 0x7d96981e, 75 },
    { 0x25f6981e, 23 },
    { 0x31a69820, 95 },
    { 0x6ac69823, 43 },
    { 0x17f69823, 33 },
    { 0x3769823, 33 },
    { 0xf9969822, 61 },
    { 0xa7c69827, 23 },
    { 0x82b69827, 76 },
    { 0x78a69827, 66 },
    { 0, 0 },
    { 0xffc6982b, 95 },
    { 0x1a6982c, 33 },
    { 0x7e46982d, 98 },
    { 0, 0 },
    { 0xa996982f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38369839, 99 },
    { 0x3756983a, 76 },
    { 0x7ae6983b, 23 },
    { 0x3126983c, 87 },
    { 0x60b6983b, 23 },
    { 0xd786983a, 93 },
    { 0, 0 },
    { 0x6fd69840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e69845, 95 },
    { 0, 0 },
    { 0x72669847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6984b, 61 },
    { 0xd926984b, 23 },
    { 0xd4f6984b, 76 },
    { 0x40c6984e, 78 },
    { 0xad86984d, 34 },
    { 0xb626984c, 33 },
    { 0x43369851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b69854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4469858, 23 },
    { 0x39b69859, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee66985d, 95 },
    { 0x8f76985e, 78 },
    { 0, 0 },
    { 0x6e969860, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46986a, 90 },
    { 0x496986b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d06986e, 61 },
    { 0x80d6986f, 61 },
    { 0x8c96986e, 98 },
    { 0x6a36986e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa769874, 61 },
    { 0, 0 },
    { 0xe7769876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b869879, 55 },
    { 0xd4069879, 95 },
    { 0x5dc6987b, 76 },
    { 0, 0 },
    { 0xb966987d, 23 },
    { 0x5596987e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10669888, 76 },
    { 0x8f669889, 34 },
    { 0xd7b6988a, 61 },
    { 0x756988b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d069891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc369894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d369897, 34 },
    { 0x8c369898, 23 },
    { 0, 0 },
    { 0xd126989a, 33 },
    { 0xc426989a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc186989e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6698a6, 33 },
    { 0, 0 },
    { 0x434698a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b698b0, 90 },
    { 0xc6698b1, 33 },
    { 0, 0 },
    { 0x1fa698b3, 23 },
    { 0x143698b4, 61 },
    { 0x404698b5, 76 },
    { 0x86698b6, 90 },
    { 0xe74698b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd698bc, 33 },
    { 0x685698bc, 94 },
    { 0x9d0698be, 33 },
    { 0, 0 },
    { 0x380698c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x278698c3, 95 },
    { 0xa11698c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da698c7, 23 },
    { 0xbe7698c8, 77 },
    { 0xb16698c8, 33 },
    { 0x576698c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b698d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54698db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac698de, 95 },
    { 0xcd5698de, 23 },
    { 0x474698e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e698e5, 93 },
    { 0, 0 },
    { 0xb97698e7, 52 },
    { 0x97d698e8, 55 },
    { 0x50698e9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83698ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc698f1, 88 },
    { 0, 0 },
    { 0x87b698f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70698f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb698ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e869902, 23 },
    { 0x35b69903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b69907, 23 },
    { 0, 0 },
    { 0xa6d69909, 98 },
    { 0xccb69909, 95 },
    { 0, 0 },
    { 0x2ad6990c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd69913, 65 },
    { 0xd6c69913, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c69919, 36 },
    { 0xb416991a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666991e, 46 },
    { 0x8716991f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8469923, 95 },
    { 0x8ab69924, 95 },
    { 0x15469925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb69928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc26992b, 95 },
    { 0, 0 },
    { 0x71d6992d, 55 },
    { 0x8076992e, 23 },
    { 0x49e6992e, 33 },
    { 0, 0 },
    { 0xd6e69931, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87769939, 78 },
    { 0x22f69939, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca069940, 90 },
    { 0, 0 },
    { 0x63f69942, 23 },
    { 0, 0 },
    { 0xda569944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa756994a, 55 },
    { 0x1bb6994b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1796994f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8469954, 61 },
    { 0, 0 },
    { 0x2e869956, 61 },
    { 0, 0 },
    { 0x87269958, 33 },
    { 0x7c569958, 33 },
    { 0x4a969959, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a6995e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd69962, 53 },
    { 0xb6f69962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41869966, 61 },
    { 0x57169966, 76 },
    { 0, 0 },
    { 0x5da69969, 90 },
    { 0xcc969969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b6996f, 23 },
    { 0xa606996f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35369974, 90 },
    { 0x80669975, 77 },
    { 0, 0 },
    { 0x7da69977, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe426997b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926997f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d69982, 98 },
    { 0xa4b69982, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26469988, 98 },
    { 0xe8769989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d6998d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e69994, 23 },
    { 0x90769994, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636999b, 8 },
    { 0x4476999c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab6999f, 98 },
    { 0x56d699a0, 61 },
    { 0x748699a1, 33 },
    { 0x5aa699a1, 33 },
    { 0x56f699a3, 33 },
    { 0x7d0699a4, 33 },
    { 0xd2b699a4, 34 },
    { 0xdc3699a6, 93 },
    { 0x13699a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x840699aa, 23 },
    { 0x403699aa, 91 },
    { 0x630699ac, 17 },
    { 0x71699ac, 23 },
    { 0x11d699ae, 87 },
    { 0x794699aa, 95 },
    { 0xefa699b0, 65 },
    { 0xe8e699b1, 23 },
    { 0x735699ad, 61 },
    { 0x2c1699b0, 55 },
    { 0, 0 },
    { 0xcf5699b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89699b8, 93 },
    { 0x8f0699b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5699bc, 61 },
    { 0xefb699bc, 34 },
    { 0x83e699bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28699c5, 87 },
    { 0, 0 },
    { 0x994699c7, 61 },
    { 0, 0 },
    { 0xb70699c9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce699ce, 90 },
    { 0x63f699cf, 52 },
    { 0xb6699d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20699d4, 90 },
    { 0xec0699d5, 90 },
    { 0xc14699d6, 66 },
    { 0x1f1699d7, 98 },
    { 0x821699d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28699db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5699de, 61 },
    { 0, 0 },
    { 0xba0699e0, 61 },
    { 0xd13699e1, 76 },
    { 0, 0 },
    { 0xf96699e3, 98 },
    { 0, 0 },
    { 0x4f9699e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe699e9, 33 },
    { 0x69a699e9, 52 },
    { 0x434699eb, 90 },
    { 0xdd5699ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x604699ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2699f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965699f7, 83 },
    { 0xa60699f8, 98 },
    { 0x398699f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c699fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1269a03, 81 },
    { 0, 0 },
    { 0x83369a05, 43 },
    { 0xd2969a06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f069a14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75969a18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57369a1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55069a24, 97 },
    { 0xef769a25, 61 },
    { 0x22a69a26, 100 },
    { 0, 0 },
    { 0xc6e69a28, 66 },
    { 0xacf69a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd69a2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c569a33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62569a36, 52 },
    { 0x99a69a37, 90 },
    { 0, 0 },
    { 0x1d969a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a69a3c, 75 },
    { 0x51469a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65669a40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee69a44, 90 },
    { 0xb9969a44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42169a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7269a56, 94 },
    { 0xb2069a57, 61 },
    { 0xf4269a57, 76 },
    { 0, 0 },
    { 0x52169a5a, 33 },
    { 0x66569a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c969a5e, 61 },
    { 0, 0 },
    { 0x4da69a60, 98 },
    { 0, 0 },
    { 0x60b69a62, 55 },
    { 0, 0 },
    { 0xf5069a64, 33 },
    { 0xe3669a65, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be69a6b, 90 },
    { 0x9b569a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e569a71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e69a74, 76 },
    { 0, 0 },
    { 0x9ed69a76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80269a79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf669a7f, 98 },
    { 0x90869a80, 78 },
    { 0xda969a81, 91 },
    { 0xa0469a82, 61 },
    { 0, 0 },
    { 0x5cb69a84, 65 },
    { 0xc1069a85, 61 },
    { 0x9cc69a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11869a89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97869a99, 66 },
    { 0x10869a99, 99 },
    { 0x61c69a9b, 34 },
    { 0xe1a69a9c, 91 },
    { 0xdfa69a9b, 90 },
    { 0xfcb69a9e, 33 },
    { 0, 0 },
    { 0x41169aa0, 68 },
    { 0xb4969aa0, 61 },
    { 0xa5e69aa2, 23 },
    { 0x2eb69aa1, 98 },
    { 0x7dc69aa3, 76 },
    { 0x87969aa0, 99 },
    { 0x38869aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4769aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e69ab2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2469abb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa69ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d69ac9, 68 },
    { 0x24169aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269adb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57969adf, 55 },
    { 0, 0 },
    { 0xefd69ae1, 33 },
    { 0x3ea69ae2, 55 },
    { 0x50c69ae2, 61 },
    { 0x6b869ae2, 55 },
    { 0x19769ae5, 33 },
    { 0xfb069ae6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf69aef, 33 },
    { 0xfa469aef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469af3, 36 },
    { 0xef269af4, 90 },
    { 0xcb269af5, 52 },
    { 0x3e69af5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21069af9, 61 },
    { 0xed869afa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd969afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0769b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c269b07, 61 },
    { 0x8a469b08, 65 },
    { 0xd2769b08, 96 },
    { 0xb1669b09, 76 },
    { 0x46769b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa69b0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2569b12, 85 },
    { 0, 0 },
    { 0x89369b14, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f69b17, 76 },
    { 0x88669b18, 23 },
    { 0, 0 },
    { 0x11f69b1a, 76 },
    { 0, 0 },
    { 0x8fa69b1c, 84 },
    { 0x5869b1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0269b21, 34 },
    { 0x8d169b22, 43 },
    { 0x53569b22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f69b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f69b2c, 17 },
    { 0xb9e69b2c, 100 },
    { 0, 0 },
    { 0xd5569b2f, 76 },
    { 0x43a69b2f, 91 },
    { 0, 0 },
    { 0x81369b32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac869b35, 61 },
    { 0x5269b36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c969b3a, 87 },
    { 0xbb69b3b, 62 },
    { 0xe8869b3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e069b3f, 61 },
    { 0x57a69b40, 65 },
    { 0x95169b41, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x97369b44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87169b49, 76 },
    { 0x9e569b49, 33 },
    { 0, 0 },
    { 0x99a69b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff969b4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff069b52, 66 },
    { 0xa1869b53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55969b61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f669b64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d69b6a, 55 },
    { 0x24a69b6b, 61 },
    { 0x84569b6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d769b70, 100 },
    { 0xa4a69b70, 23 },
    { 0x71a69b72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43869b77, 17 },
    { 0, 0 },
    { 0xf0d69b79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c69b80, 90 },
    { 0xa3e69b81, 52 },
    { 0x9a069b82, 68 },
    { 0x3fc69b82, 98 },
    { 0xceb69b82, 43 },
    { 0xce469b85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec769b88, 90 },
    { 0, 0 },
    { 0xa8269b8a, 61 },
    { 0, 0 },
    { 0xcdb69b8c, 33 },
    { 0x95169b8c, 33 },
    { 0xd5569b8d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669b91, 90 },
    { 0xcf369b91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e569b96, 33 },
    { 0, 0 },
    { 0xdb369b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b69b9b, 90 },
    { 0x4c269b9b, 66 },
    { 0xd4d69b9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56169ba1, 61 },
    { 0x6a669ba2, 46 },
    { 0x66f69ba3, 95 },
    { 0xdee69ba2, 78 },
    { 0x15969ba2, 23 },
    { 0x1e069ba6, 23 },
    { 0xdfd69ba6, 95 },
    { 0x4bf69ba8, 67 },
    { 0, 0 },
    { 0xff269baa, 99 },
    { 0x84669bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d069baf, 23 },
    { 0, 0 },
    { 0x4a869bb1, 95 },
    { 0xa9a69bb2, 66 },
    { 0, 0 },
    { 0x55969bb4, 95 },
    { 0xdf869bb5, 61 },
    { 0x1f269bb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d969bba, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca69bc3, 65 },
    { 0, 0 },
    { 0x2f769bc5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13769bc9, 61 },
    { 0x9c069bc9, 100 },
    { 0xc1d69bc9, 43 },
    { 0, 0 },
    { 0x7f469bcd, 95 },
    { 0, 0 },
    { 0xeee69bcf, 95 },
    { 0xcb269bd0, 17 },
    { 0x66469bd1, 98 },
    { 0, 0 },
    { 0xad469bd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca169bd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169bdc, 66 },
    { 0, 0 },
    { 0x29d69bde, 98 },
    { 0xf9969bdf, 93 },
    { 0, 0 },
    { 0x77c69be1, 61 },
    { 0xbe169be2, 46 },
    { 0xd8969be3, 61 },
    { 0, 0 },
    { 0x58169be5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67369be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e69bed, 23 },
    { 0xb0a69bed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b69bf4, 75 },
    { 0, 0 },
    { 0xbb169bf6, 97 },
    { 0xb9869bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a69bfc, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f69c08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3669c0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20769c11, 98 },
    { 0, 0 },
    { 0x31169c13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c69c18, 17 },
    { 0, 0 },
    { 0x9169c1a, 46 },
    { 0, 0 },
    { 0xffe69c1c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a69c20, 29 },
    { 0, 0 },
    { 0x82a69c22, 95 },
    { 0xbaf69c23, 85 },
    { 0x6f69c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3369c27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39769c2b, 95 },
    { 0xef069c2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c869c30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b69c35, 90 },
    { 0xd0b69c35, 55 },
    { 0, 0 },
    { 0xf2769c38, 46 },
    { 0x8fe69c39, 61 },
    { 0, 0 },
    { 0xb6369c3b, 95 },
    { 0xb3b69c3c, 90 },
    { 0xd7569c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47369c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db69c4b, 85 },
    { 0, 0 },
    { 0x41a69c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b69c51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f69c57, 98 },
    { 0xec369c58, 93 },
    { 0xd8b69c59, 55 },
    { 0xf9369c5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb569c60, 90 },
    { 0x8ac69c60, 33 },
    { 0x3d169c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd69c66, 95 },
    { 0x4c669c67, 34 },
    { 0, 0 },
    { 0xb0a69c69, 76 },
    { 0x14f69c6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e69c6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31669c77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e769c7f, 90 },
    { 0xc4c69c80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4269c83, 66 },
    { 0x2c869c84, 88 },
    { 0x6bb69c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a69c8a, 33 },
    { 0x63969c8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb69c8e, 55 },
    { 0x43069c8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda369c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c9d, 97 },
    { 0, 0 },
    { 0xe1569c9f, 81 },
    { 0, 0 },
    { 0xade69ca1, 90 },
    { 0, 0 },
    { 0xd8669ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97769caa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef69cae, 61 },
    { 0xea769cae, 55 },
    { 0x81c69cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40869cb6, 23 },
    { 0x84169cb7, 95 },
    { 0x25469cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d669cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe269cc2, 23 },
    { 0x58269cc3, 98 },
    { 0x8e569cc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc369cc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d69cca, 93 },
    { 0, 0 },
    { 0xafa69ccc, 98 },
    { 0xc2e69ccd, 98 },
    { 0x83969ccc, 85 },
    { 0x92969ccf, 33 },
    { 0xb3569cd0, 98 },
    { 0xe4f69cd1, 33 },
    { 0x4369ccd, 61 },
    { 0x6ba69cd3, 90 },
    { 0xfb769cd4, 46 },
    { 0x25e69cd5, 98 },
    { 0x85969cd6, 61 },
    { 0xcbd69cd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65469cdf, 90 },
    { 0, 0 },
    { 0x16f69ce1, 95 },
    { 0x83469ce2, 90 },
    { 0, 0 },
    { 0xb0a69ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49069ce9, 95 },
    { 0xe1a69ce9, 61 },
    { 0, 0 },
    { 0x9ee69cec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b269cf0, 78 },
    { 0xd5269cf1, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d69cf4, 23 },
    { 0xd4069cf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af69cf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c69d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86069d11, 52 },
    { 0xe8d69d11, 61 },
    { 0xf8569d11, 90 },
    { 0xcd669d14, 92 },
    { 0xb1069d14, 90 },
    { 0, 0 },
    { 0x93169d17, 98 },
    { 0x70969d18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb669d1b, 55 },
    { 0x1b669d1b, 100 },
    { 0x83069d1d, 78 },
    { 0xda569d1c, 34 },
    { 0x98d69d1d, 91 },
    { 0x14969d20, 47 },
    { 0, 0 },
    { 0x47669d22, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f69d26, 74 },
    { 0x11e69d27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f69d2a, 98 },
    { 0xb2b69d2b, 46 },
    { 0, 0 },
    { 0xdf969d2d, 61 },
    { 0xa9769d2e, 77 },
    { 0x27369d2f, 23 },
    { 0x3c669d2f, 98 },
    { 0xa6269d31, 43 },
    { 0x39b69d2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e69d3b, 66 },
    { 0xced69d3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0969d40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a69d43, 85 },
    { 0xd1f69d43, 90 },
    { 0, 0 },
    { 0xf69d46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b69d49, 36 },
    { 0x9169d4a, 17 },
    { 0, 0 },
    { 0x8d69d4c, 33 },
    { 0x9cd69d4d, 76 },
    { 0xc3669d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d969d52, 95 },
    { 0x13969d52, 91 },
    { 0x80569d53, 91 },
    { 0x5bf69d54, 90 },
    { 0xb2d69d56, 66 },
    { 0x90569d56, 69 },
    { 0x55169d54, 33 },
    { 0x66b69d52, 95 },
    { 0, 0 },
    { 0x169d5b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b69d5f, 62 },
    { 0xdb969d60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf69d67, 43 },
    { 0xf2069d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c69d6d, 76 },
    { 0, 0 },
    { 0x5a069d6f, 61 },
    { 0xe2f69d6f, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe669d73, 99 },
    { 0x74f69d74, 33 },
    { 0, 0 },
    { 0x1fc69d76, 90 },
    { 0x8569d77, 33 },
    { 0x95a69d78, 36 },
    { 0x7d869d79, 95 },
    { 0, 0 },
    { 0xa2d69d7b, 34 },
    { 0xd6d69d7c, 43 },
    { 0xc4469d7b, 84 },
    { 0x6169d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c69d85, 93 },
    { 0, 0 },
    { 0xf2969d87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b69d8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f69d8f, 19 },
    { 0x9de69d8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f369d96, 61 },
    { 0xf1669d97, 95 },
    { 0xb8269d96, 23 },
    { 0, 0 },
    { 0xaaf69d9a, 34 },
    { 0, 0 },
    { 0xe769d9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b69d9f, 61 },
    { 0, 0 },
    { 0x54a69da1, 23 },
    { 0xef869da2, 55 },
    { 0x9dc69da1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10469daa, 95 },
    { 0x7f169dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca769db0, 33 },
    { 0, 0 },
    { 0x9d269db2, 33 },
    { 0x1aa69db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce69db8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f69dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c69dc3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec69dc6, 98 },
    { 0xdae69dc7, 67 },
    { 0xb1569dc8, 46 },
    { 0, 0 },
    { 0xb569dca, 52 },
    { 0x31969dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c69dd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f869dd7, 78 },
    { 0x1ae69dd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f69ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb969de3, 77 },
    { 0x70569de4, 99 },
    { 0, 0 },
    { 0x6fb69de6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c69de9, 77 },
    { 0x4c869de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52869def, 55 },
    { 0x9d269df0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14569df3, 100 },
    { 0xec169df4, 37 },
    { 0, 0 },
    { 0x4a269df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be69e04, 85 },
    { 0, 0 },
    { 0x9e269e06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75969e0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e69e0d, 61 },
    { 0x37f69e0e, 33 },
    { 0xf5469e0e, 97 },
    { 0x98469e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15369e14, 33 },
    { 0, 0 },
    { 0x1469e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2269e19, 99 },
    { 0, 0 },
    { 0x53569e1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da69e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61469e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f69e3b, 17 },
    { 0x21e69e3b, 90 },
    { 0, 0 },
    { 0x32969e3e, 34 },
    { 0x26a69e3e, 78 },
    { 0xe7d69e3e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d869e4a, 90 },
    { 0x99769e4b, 34 },
    { 0xa2669e4c, 23 },
    { 0x2d69e4d, 61 },
    { 0, 0 },
    { 0x2ba69e4f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b169e52, 34 },
    { 0, 0 },
    { 0xc3669e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad69e57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a269e5b, 61 },
    { 0, 0 },
    { 0x43a69e5d, 66 },
    { 0, 0 },
    { 0x96569e5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea69e62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf569e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2669e6b, 91 },
    { 0x2c969e6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a69e6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde369e75, 61 },
    { 0xd6869e76, 43 },
    { 0x2c069e77, 23 },
    { 0xfb869e78, 66 },
    { 0x32e69e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3369e7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80669e83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba69e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b69e8e, 65 },
    { 0xf7869e8e, 33 },
    { 0xc3569e90, 23 },
    { 0, 0 },
    { 0x7c69e92, 90 },
    { 0, 0 },
    { 0xcf369e94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f069e98, 85 },
    { 0xf8c69e99, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6869e9c, 33 },
    { 0x10c69e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f69ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98369ea3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a269ea6, 95 },
    { 0xcfc69ea7, 95 },
    { 0x4ed69ea7, 95 },
    { 0xc2469ea7, 33 },
    { 0x4b69eaa, 33 },
    { 0, 0 },
    { 0x55869eac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9269eaf, 90 },
    { 0, 0 },
    { 0xc4a69eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0669ebb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb69ec2, 95 },
    { 0, 0 },
    { 0xb7369ec4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82669ecd, 34 },
    { 0x4d469ece, 66 },
    { 0x2d169ecf, 33 },
    { 0xdf69ed0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5669ed4, 61 },
    { 0x21e69ed5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f69ed8, 95 },
    { 0x8569ed8, 61 },
    { 0x76269ed8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be69edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8369ee3, 61 },
    { 0x28969ee3, 61 },
    { 0x1a969ee5, 23 },
    { 0x90c69ee4, 23 },
    { 0xb0769ee7, 98 },
    { 0x3dd69ee7, 78 },
    { 0x8f69ee4, 87 },
    { 0xade69ee6, 76 },
    { 0x34569ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f69ef3, 46 },
    { 0xd8d69ef3, 17 },
    { 0, 0 },
    { 0xbb469ef6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb369efa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f869f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d69f04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e069f0b, 95 },
    { 0x5b569f0b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1069f10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa769f13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9269f1a, 23 },
    { 0x4a869f1b, 33 },
    { 0, 0 },
    { 0x5c369f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b69f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc769f23, 55 },
    { 0, 0 },
    { 0xc6369f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed69f31, 75 },
    { 0x64069f32, 55 },
    { 0x23569f33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7069f36, 84 },
    { 0, 0 },
    { 0x9db69f38, 78 },
    { 0, 0 },
    { 0x62669f3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f69f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a69f49, 23 },
    { 0x5669f4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1969f52, 77 },
    { 0xbfe69f53, 33 },
    { 0x7ea69f54, 23 },
    { 0, 0 },
    { 0x1d169f56, 95 },
    { 0, 0 },
    { 0x87369f58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d69f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6669f61, 98 },
    { 0, 0 },
    { 0x68169f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d569f66, 68 },
    { 0, 0 },
    { 0x5f869f68, 76 },
    { 0, 0 },
    { 0xe8969f6a, 93 },
    { 0xa3669f6b, 78 },
    { 0x92469f6b, 95 },
    { 0, 0 },
    { 0x22c69f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22169f71, 90 },
    { 0xb0d69f72, 95 },
    { 0x55f69f73, 76 },
    { 0x3f969f74, 33 },
    { 0x39d69f73, 46 },
    { 0x87969f76, 98 },
    { 0x2aa69f77, 98 },
    { 0xa5369f77, 43 },
    { 0x7ff69f74, 68 },
    { 0xac269f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b69f7e, 23 },
    { 0xd1b69f7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2769f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b69f86, 87 },
    { 0x98369f87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c69f8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da69f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57169f96, 90 },
    { 0x54d69f97, 33 },
    { 0x5d769f97, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf69f9b, 61 },
    { 0x6ff69f9b, 90 },
    { 0x65e69f9c, 91 },
    { 0, 0 },
    { 0x8b669f9f, 99 },
    { 0x37469f9f, 33 },
    { 0x5e769fa1, 90 },
    { 0xce369f9f, 61 },
    { 0, 0 },
    { 0x5f569fa4, 90 },
    { 0xbfa69fa4, 95 },
    { 0, 0 },
    { 0x5de69fa7, 55 },
    { 0x18269fa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25969fae, 94 },
    { 0x50d69faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee69fb3, 33 },
    { 0, 0 },
    { 0xa3969fb5, 90 },
    { 0x76869fb6, 33 },
    { 0xc6369fb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f69fba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d69fbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28969fcb, 56 },
    { 0x80169fcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf69fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f69fd4, 61 },
    { 0xb2b69fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c669fdb, 34 },
    { 0xb9d69fdc, 97 },
    { 0xb3169fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec569fe2, 90 },
    { 0xcca69fe3, 90 },
    { 0x12e69fe3, 33 },
    { 0x5df69fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba69fe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9769ff0, 33 },
    { 0, 0 },
    { 0xba869ff2, 98 },
    { 0x26e69ff3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8769ff9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d69ffd, 61 },
    { 0x3df69ffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b6a008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6a00f, 43 },
    { 0, 0 },
    { 0xe96a011, 23 },
    { 0, 0 },
    { 0xb496a013, 75 },
    { 0x3f16a013, 10 },
    { 0x9c06a015, 76 },
    { 0xb266a015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe036a01b, 91 },
    { 0x4356a01b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06a020, 55 },
    { 0xfc86a020, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36a025, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5216a02a, 34 },
    { 0x2da6a02a, 84 },
    { 0x2676a02c, 56 },
    { 0x8f16a02c, 68 },
    { 0x696a02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf6a033, 23 },
    { 0, 0 },
    { 0x1896a035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d6a03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b66a03f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa256a043, 90 },
    { 0x48a6a044, 23 },
    { 0, 0 },
    { 0xd5a6a046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce06a04b, 99 },
    { 0x53a6a04c, 90 },
    { 0x7266a04b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff6a052, 95 },
    { 0, 0 },
    { 0xf5e6a054, 90 },
    { 0, 0 },
    { 0x61b6a056, 85 },
    { 0xc156a057, 55 },
    { 0x9486a058, 55 },
    { 0, 0 },
    { 0xbc86a05a, 33 },
    { 0xe776a05a, 34 },
    { 0xc226a05c, 33 },
    { 0xa2b6a05d, 83 },
    { 0xe336a05c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a6a06a, 34 },
    { 0xb1a6a06b, 66 },
    { 0x8ca6a06c, 61 },
    { 0, 0 },
    { 0x446a06e, 90 },
    { 0, 0 },
    { 0xb726a070, 33 },
    { 0xab26a070, 90 },
    { 0, 0 },
    { 0x8506a073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86a079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966a07f, 98 },
    { 0x8286a080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646a086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d6a08a, 99 },
    { 0, 0 },
    { 0x17c6a08c, 33 },
    { 0, 0 },
    { 0x6a26a08e, 33 },
    { 0x68b6a08e, 80 },
    { 0xd466a090, 95 },
    { 0x3126a08f, 55 },
    { 0x4b86a08f, 90 },
    { 0xe366a093, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16a097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76a0a1, 98 },
    { 0x1316a0a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac6a0ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2226a0b4, 46 },
    { 0, 0 },
    { 0xad26a0b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe986a0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6a0bd, 33 },
    { 0, 0 },
    { 0x1356a0bf, 33 },
    { 0x3126a0bf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb336a0c6, 33 },
    { 0, 0 },
    { 0xb516a0c8, 43 },
    { 0x85d6a0c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7496a0cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd616a0d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566a0d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c6a0db, 76 },
    { 0xfd26a0dc, 93 },
    { 0xfba6a0dd, 90 },
    { 0xdc66a0de, 98 },
    { 0x87c6a0de, 43 },
    { 0, 0 },
    { 0x8dd6a0e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386a0e6, 33 },
    { 0, 0 },
    { 0xf086a0e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a6a0eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e6a0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3296a0f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac6a0ff, 90 },
    { 0x7e16a100, 93 },
    { 0xfb96a100, 33 },
    { 0xb736a102, 90 },
    { 0x30b6a103, 33 },
    { 0x5916a104, 76 },
    { 0x67f6a105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6a108, 91 },
    { 0xef36a109, 33 },
    { 0x3a66a10a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde96a10e, 90 },
    { 0x8fd6a10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906a11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd306a11f, 93 },
    { 0xbd96a120, 23 },
    { 0, 0 },
    { 0xf4b6a122, 95 },
    { 0x2f86a123, 61 },
    { 0x9ff6a124, 93 },
    { 0xc696a125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad6a128, 34 },
    { 0x6566a129, 66 },
    { 0x70b6a12a, 33 },
    { 0xed06a12b, 33 },
    { 0x3e76a12b, 84 },
    { 0x336a12c, 97 },
    { 0xe8c6a12d, 66 },
    { 0, 0 },
    { 0x80d6a130, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6a134, 61 },
    { 0x27a6a134, 98 },
    { 0x2196a136, 76 },
    { 0x6086a137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076a13a, 76 },
    { 0xed46a13b, 99 },
    { 0x6fd6a13c, 90 },
    { 0x85e6a13d, 87 },
    { 0xaff6a13b, 93 },
    { 0, 0 },
    { 0xa2e6a140, 76 },
    { 0x27a6a141, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86a145, 23 },
    { 0xc0b6a146, 55 },
    { 0, 0 },
    { 0x6a26a148, 99 },
    { 0x3726a149, 52 },
    { 0, 0 },
    { 0x67e6a14b, 76 },
    { 0x5106a14c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8076a155, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb46a158, 33 },
    { 0x6246a159, 52 },
    { 0xfd66a15a, 97 },
    { 0xc946a15b, 76 },
    { 0, 0 },
    { 0x216a15d, 34 },
    { 0, 0 },
    { 0xe1e6a15f, 23 },
    { 0x5706a15f, 83 },
    { 0xf166a15f, 69 },
    { 0, 0 },
    { 0x5a86a163, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6a166, 90 },
    { 0x5576a166, 43 },
    { 0, 0 },
    { 0x4686a169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4976a16d, 33 },
    { 0xe076a16e, 61 },
    { 0x7fd6a16f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d6a173, 91 },
    { 0x4a46a174, 84 },
    { 0xca86a175, 34 },
    { 0xd766a176, 91 },
    { 0xaf26a175, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd6a17f, 33 },
    { 0xda6a180, 33 },
    { 0x3256a181, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe786a18a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf6a190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf476a196, 95 },
    { 0x3036a196, 33 },
    { 0xd796a196, 33 },
    { 0x6336a198, 23 },
    { 0x42d6a197, 36 },
    { 0xcc86a19b, 90 },
    { 0, 0 },
    { 0xd516a19d, 46 },
    { 0x2456a19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e6a1a2, 90 },
    { 0xed76a1a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646a1a6, 97 },
    { 0xe416a1a6, 98 },
    { 0xc216a1a7, 90 },
    { 0, 0 },
    { 0x35c6a1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5276a1af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8956a1b4, 98 },
    { 0, 0 },
    { 0x43f6a1b6, 90 },
    { 0x3316a1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f96a1ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e86a1bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d76a1c3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb156a1c7, 75 },
    { 0x766a1c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6a1cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6a1d3, 61 },
    { 0x5336a1d3, 33 },
    { 0, 0 },
    { 0xc3b6a1d6, 61 },
    { 0, 0 },
    { 0xf936a1d8, 76 },
    { 0x3bd6a1d8, 33 },
    { 0, 0 },
    { 0xaf06a1db, 56 },
    { 0x10e6a1db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686a1e1, 55 },
    { 0x6886a1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf06a1e6, 33 },
    { 0x7b26a1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446a1f5, 78 },
    { 0, 0 },
    { 0x87d6a1f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26a1fa, 61 },
    { 0x8736a1fb, 90 },
    { 0x7226a1fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1566a201, 61 },
    { 0, 0 },
    { 0xa8e6a203, 43 },
    { 0xee56a204, 98 },
    { 0x4ea6a203, 36 },
    { 0, 0 },
    { 0x3366a207, 90 },
    { 0xa016a208, 61 },
    { 0x4e96a209, 98 },
    { 0xf6b6a209, 46 },
    { 0x6986a20b, 98 },
    { 0x80f6a20a, 55 },
    { 0x5246a20d, 98 },
    { 0, 0 },
    { 0xb396a20f, 55 },
    { 0x1486a210, 95 },
    { 0xc1a6a211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc6a214, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe106a21a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f6a21f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf066a223, 90 },
    { 0xa666a223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c6a234, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396a23d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6a241, 61 },
    { 0, 0 },
    { 0x4c86a243, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb896a24b, 98 },
    { 0x2926a24c, 23 },
    { 0, 0 },
    { 0x776a24e, 90 },
    { 0x2066a24f, 55 },
    { 0x5026a250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf6a255, 43 },
    { 0xa856a256, 98 },
    { 0x9976a256, 87 },
    { 0, 0 },
    { 0x2626a259, 90 },
    { 0, 0 },
    { 0xbad6a25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab6a25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd6a263, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe286a266, 90 },
    { 0x5e36a267, 23 },
    { 0xb566a268, 66 },
    { 0x6e26a269, 33 },
    { 0, 0 },
    { 0xf606a26b, 93 },
    { 0xe366a26b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176a26f, 90 },
    { 0, 0 },
    { 0xe736a271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb6a275, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb486a27a, 61 },
    { 0, 0 },
    { 0xa226a27c, 77 },
    { 0xfa56a27d, 33 },
    { 0xb076a27d, 96 },
    { 0xf76a27f, 99 },
    { 0x5896a280, 55 },
    { 0xe6d6a281, 95 },
    { 0xc826a282, 23 },
    { 0x93c6a27f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe6a288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c6a28e, 87 },
    { 0x8e6a28f, 55 },
    { 0xd836a290, 17 },
    { 0x8036a291, 61 },
    { 0xc736a292, 23 },
    { 0x2346a292, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326a29a, 55 },
    { 0xa436a29b, 75 },
    { 0x146a29c, 90 },
    { 0xf6e6a29d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046a2a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc6a2aa, 34 },
    { 0, 0 },
    { 0xf526a2ac, 61 },
    { 0x2036a2ad, 77 },
    { 0xa46a2ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146a2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6a2b5, 33 },
    { 0xacd6a2b6, 61 },
    { 0, 0 },
    { 0xe136a2b8, 33 },
    { 0, 0 },
    { 0xc096a2ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826a2bd, 61 },
    { 0x5956a2be, 75 },
    { 0, 0 },
    { 0x3fd6a2c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa616a2c8, 52 },
    { 0xc8f6a2c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c6a2cc, 61 },
    { 0, 0 },
    { 0xd246a2ce, 52 },
    { 0xa0c6a2cf, 100 },
    { 0xc496a2d0, 33 },
    { 0xeb56a2d1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336a2d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab6a2d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde6a2da, 93 },
    { 0x70e6a2db, 83 },
    { 0x9476a2db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf6a2e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7816a2e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826a2e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a86a2ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab6a2f2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646a2f5, 93 },
    { 0x12f6a2f6, 55 },
    { 0x6cd6a2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e6a2fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf676a301, 40 },
    { 0xfc56a302, 95 },
    { 0x5f06a302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c6a307, 90 },
    { 0, 0 },
    { 0x33a6a309, 46 },
    { 0, 0 },
    { 0x4836a30b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef76a30f, 95 },
    { 0xac26a310, 100 },
    { 0x9f46a310, 90 },
    { 0x7fd6a312, 33 },
    { 0x21b6a312, 23 },
    { 0xea16a314, 23 },
    { 0x8786a315, 34 },
    { 0, 0 },
    { 0xb3b6a317, 90 },
    { 0, 0 },
    { 0x71d6a319, 75 },
    { 0, 0 },
    { 0x8206a31b, 96 },
    { 0x2db6a31c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3406a31f, 33 },
    { 0xbc56a320, 95 },
    { 0xe426a321, 98 },
    { 0, 0 },
    { 0x4446a323, 33 },
    { 0x1dd6a324, 46 },
    { 0, 0 },
    { 0x1716a326, 68 },
    { 0, 0 },
    { 0x9d16a328, 90 },
    { 0, 0 },
    { 0x6c76a32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf56a32f, 34 },
    { 0x78d6a330, 23 },
    { 0x5656a331, 55 },
    { 0x1916a331, 40 },
    { 0xf4a6a331, 17 },
    { 0xd3c6a32f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a338, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb6a33b, 81 },
    { 0, 0 },
    { 0x9ac6a33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7096a343, 77 },
    { 0x6906a343, 90 },
    { 0, 0 },
    { 0x6836a346, 33 },
    { 0x8c6a347, 90 },
    { 0x9b96a346, 61 },
    { 0, 0 },
    { 0xd266a34a, 33 },
    { 0x4e26a34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a6a351, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e6a359, 23 },
    { 0xb566a359, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c76a35d, 90 },
    { 0, 0 },
    { 0x6666a35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c6a362, 84 },
    { 0x9cd6a363, 90 },
    { 0x5d96a364, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6a367, 33 },
    { 0x3796a368, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a66a36f, 95 },
    { 0, 0 },
    { 0x8476a371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c6a377, 90 },
    { 0x6226a377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c76a37d, 95 },
    { 0x54f6a37e, 76 },
    { 0x656a37e, 55 },
    { 0x93c6a380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3526a383, 101 },
    { 0x8676a383, 43 },
    { 0x3e36a383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8616a38d, 87 },
    { 0x6b36a38e, 33 },
    { 0xe9e6a38d, 61 },
    { 0x8956a38d, 90 },
    { 0x5276a390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6a395, 90 },
    { 0x4b36a396, 90 },
    { 0, 0 },
    { 0x6606a398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6a39c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6a39f, 75 },
    { 0xb716a39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a6a3a6, 95 },
    { 0x1f16a3a6, 65 },
    { 0xc486a3a8, 98 },
    { 0, 0 },
    { 0xa336a3aa, 90 },
    { 0, 0 },
    { 0x3546a3ac, 91 },
    { 0, 0 },
    { 0x7266a3ae, 81 },
    { 0x2416a3ae, 75 },
    { 0xe756a3b0, 95 },
    { 0, 0 },
    { 0x3c66a3b2, 88 },
    { 0, 0 },
    { 0xe886a3b4, 95 },
    { 0x7ab6a3b5, 38 },
    { 0x47f6a3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a6a3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a3c0, 2 },
    { 0, 0 },
    { 0x216a3c2, 75 },
    { 0, 0 },
    { 0xbf66a3c4, 95 },
    { 0x686a3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5606a3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e6a3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb6a3d7, 90 },
    { 0, 0 },
    { 0xc126a3d9, 53 },
    { 0xa526a3da, 90 },
    { 0xabc6a3db, 23 },
    { 0x8c6a3dc, 40 },
    { 0x3756a3dc, 100 },
    { 0, 0 },
    { 0x7876a3df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f6a3e9, 33 },
    { 0xa146a3ea, 55 },
    { 0x2696a3eb, 33 },
    { 0x85d6a3ea, 98 },
    { 0xb186a3ea, 23 },
    { 0, 0 },
    { 0xb906a3ef, 78 },
    { 0xf766a3ef, 33 },
    { 0x7176a3f0, 90 },
    { 0xd3c6a3f2, 77 },
    { 0x7016a3ef, 93 },
    { 0xf336a3f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc826a3f8, 91 },
    { 0, 0 },
    { 0xd1b6a3fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866a408, 61 },
    { 0, 0 },
    { 0x50b6a40a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1656a40e, 90 },
    { 0xeaa6a40e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d6a41d, 90 },
    { 0x7276a41e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6396a42a, 66 },
    { 0x3fc6a42a, 93 },
    { 0, 0 },
    { 0x4456a42d, 33 },
    { 0x9536a42e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c6a432, 33 },
    { 0, 0 },
    { 0x296a434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad6a437, 61 },
    { 0, 0 },
    { 0xa796a439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66a441, 90 },
    { 0x5126a442, 34 },
    { 0xac66a441, 34 },
    { 0, 0 },
    { 0x506a445, 55 },
    { 0x5d56a445, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e66a44a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a44d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d6a456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e26a45e, 61 },
    { 0x4286a45e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4486a462, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8796a468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c66a46b, 33 },
    { 0xc66a46c, 97 },
    { 0, 0 },
    { 0x1016a46e, 23 },
    { 0xc8f6a46e, 90 },
    { 0x4e36a470, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a6a474, 98 },
    { 0, 0 },
    { 0xacd6a476, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8946a479, 90 },
    { 0, 0 },
    { 0x66b6a47b, 34 },
    { 0, 0 },
    { 0x4746a47d, 98 },
    { 0x75d6a47e, 66 },
    { 0, 0 },
    { 0xd4f6a480, 90 },
    { 0x3c6a481, 76 },
    { 0x78a6a482, 23 },
    { 0x8a86a483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956a486, 33 },
    { 0x6596a486, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6a48a, 64 },
    { 0, 0 },
    { 0x2d16a48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e6a48f, 88 },
    { 0xf766a490, 55 },
    { 0x67c6a48f, 98 },
    { 0xb86a491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c6a495, 94 },
    { 0x8e66a496, 23 },
    { 0x3cc6a497, 77 },
    { 0x2a36a496, 33 },
    { 0x1196a499, 91 },
    { 0xd806a497, 98 },
    { 0xc976a498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac06a4a2, 87 },
    { 0, 0 },
    { 0xf516a4a4, 34 },
    { 0x92b6a4a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b6a4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d6a4af, 78 },
    { 0x19c6a4b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a36a4bc, 23 },
    { 0x7b76a4bd, 98 },
    { 0, 0 },
    { 0xbfa6a4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f6a4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d16a4c7, 67 },
    { 0, 0 },
    { 0xb6e6a4c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d6a4d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd36a4d9, 23 },
    { 0xf9d6a4da, 90 },
    { 0x2b86a4d9, 40 },
    { 0x17d6a4dc, 98 },
    { 0xbab6a4dd, 90 },
    { 0x8a6a4dc, 90 },
    { 0x1d76a4dc, 87 },
    { 0xe0b6a4da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f6a4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7136a4e6, 78 },
    { 0x3226a4e7, 43 },
    { 0x8346a4e8, 33 },
    { 0x5836a4e9, 95 },
    { 0x5b26a4ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a46a4ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b36a4f1, 90 },
    { 0xe256a4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6a4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d6a4fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c86a518, 23 },
    { 0, 0 },
    { 0x75d6a51a, 90 },
    { 0xd5a6a51a, 91 },
    { 0, 0 },
    { 0xfe06a51d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a6a522, 33 },
    { 0x4916a523, 93 },
    { 0x7c76a524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6a527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb6a52d, 90 },
    { 0xf616a52e, 96 },
    { 0x5a06a52f, 61 },
    { 0x71c6a530, 90 },
    { 0x39a6a52f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f66a537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab56a53d, 90 },
    { 0, 0 },
    { 0xa2b6a53f, 98 },
    { 0, 0 },
    { 0x28d6a541, 87 },
    { 0, 0 },
    { 0xa9b6a543, 33 },
    { 0xea66a544, 76 },
    { 0x37c6a545, 93 },
    { 0x7356a543, 90 },
    { 0x4ff6a544, 34 },
    { 0xf956a548, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f6a553, 76 },
    { 0xd4b6a554, 87 },
    { 0x386a554, 68 },
    { 0x1b66a556, 46 },
    { 0xbbd6a557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf6a55b, 98 },
    { 0x54b6a55c, 61 },
    { 0x1e36a55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696a562, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96a566, 33 },
    { 0xc256a566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266a56c, 33 },
    { 0x4246a56d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9106a570, 76 },
    { 0xac06a571, 93 },
    { 0xb796a570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6a575, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456a578, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed6a57e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406a582, 23 },
    { 0x2bb6a582, 100 },
    { 0xdcf6a584, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed6a592, 85 },
    { 0xd8a6a592, 90 },
    { 0x4b56a594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6a598, 100 },
    { 0x8846a598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b66a59c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d06a5a2, 33 },
    { 0x98f6a5a3, 87 },
    { 0, 0 },
    { 0x4dc6a5a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a86a5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6a5ac, 88 },
    { 0xaf76a5ad, 90 },
    { 0x2636a5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f6a5b9, 43 },
    { 0x4bc6a5ba, 61 },
    { 0x6416a5ba, 90 },
    { 0, 0 },
    { 0x4e16a5bd, 95 },
    { 0, 0 },
    { 0xa396a5bf, 90 },
    { 0x1276a5c0, 56 },
    { 0x8656a5c1, 98 },
    { 0x6326a5bf, 98 },
    { 0x4266a5c0, 76 },
    { 0x8c16a5bf, 61 },
    { 0x66a6a5c1, 33 },
    { 0, 0 },
    { 0x4436a5c7, 67 },
    { 0x2706a5c8, 43 },
    { 0x29e6a5c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f6a5cf, 33 },
    { 0x3da6a5d0, 23 },
    { 0x43f6a5d1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x296a5d4, 33 },
    { 0x1006a5d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056a5da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a6a5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf6a5e2, 23 },
    { 0x2686a5e2, 76 },
    { 0x7696a5e4, 23 },
    { 0xaf46a5e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a6a5e9, 88 },
    { 0x7296a5ea, 43 },
    { 0xfe86a5eb, 95 },
    { 0x3346a5ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1836a5ef, 33 },
    { 0, 0 },
    { 0x91e6a5f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a6a5f5, 33 },
    { 0, 0 },
    { 0x1cd6a5f7, 99 },
    { 0x39d6a5f8, 43 },
    { 0xefb6a5f7, 34 },
    { 0xbab6a5fa, 100 },
    { 0xfdb6a5f8, 55 },
    { 0, 0 },
    { 0x25f6a5fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd26a602, 95 },
    { 0xda16a603, 76 },
    { 0x3786a604, 33 },
    { 0, 0 },
    { 0xfe86a606, 95 },
    { 0x5896a606, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f6a60c, 76 },
    { 0xf06a60d, 23 },
    { 0x40c6a60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096a613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd36a617, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4516a61b, 98 },
    { 0, 0 },
    { 0xe436a61d, 61 },
    { 0, 0 },
    { 0x99d6a61f, 90 },
    { 0xd536a620, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa946a623, 98 },
    { 0x6846a624, 43 },
    { 0xbf06a624, 75 },
    { 0xc8f6a626, 52 },
    { 0x7366a624, 98 },
    { 0x4a46a628, 33 },
    { 0xfa26a624, 98 },
    { 0, 0 },
    { 0x6146a62b, 90 },
    { 0xa596a62c, 23 },
    { 0xe256a62d, 90 },
    { 0xa06a62c, 90 },
    { 0xb2c6a62f, 61 },
    { 0xbb06a62e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e6a634, 76 },
    { 0x5526a634, 95 },
    { 0x1b76a636, 91 },
    { 0xe8a6a637, 23 },
    { 0xfd46a638, 80 },
    { 0xbcc6a637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c6a63e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6a646, 61 },
    { 0xbe46a647, 87 },
    { 0x6776a648, 46 },
    { 0, 0 },
    { 0xc66a64a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076a64d, 76 },
    { 0, 0 },
    { 0x4a6a64f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd196a653, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a6a659, 98 },
    { 0x99d6a65a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe116a663, 76 },
    { 0, 0 },
    { 0x366a665, 33 },
    { 0x6486a665, 95 },
    { 0xda56a665, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6a66b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046a678, 90 },
    { 0x3466a679, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56a681, 95 },
    { 0xfac6a682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe66a689, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736a691, 90 },
    { 0x7816a692, 90 },
    { 0xd096a693, 98 },
    { 0, 0 },
    { 0x3096a695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6a699, 90 },
    { 0xced6a69a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec6a69d, 61 },
    { 0x5606a69e, 61 },
    { 0x26f6a69d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa6a6a7, 33 },
    { 0x6a6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a6ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae6a6ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f96a6b1, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc426a6b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1556a6ba, 61 },
    { 0x5bc6a6bb, 48 },
    { 0, 0 },
    { 0xef6a6bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e6a6c0, 33 },
    { 0x6a96a6c1, 84 },
    { 0xa336a6c2, 98 },
    { 0x2376a6c3, 95 },
    { 0, 0 },
    { 0x6f76a6c5, 76 },
    { 0, 0 },
    { 0x2246a6c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e6a6ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd76a6cd, 43 },
    { 0, 0 },
    { 0x1eb6a6cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e6a6d6, 90 },
    { 0x1c26a6d7, 66 },
    { 0x2bb6a6d8, 90 },
    { 0xdb06a6d9, 55 },
    { 0xe366a6d9, 95 },
    { 0xc166a6d7, 55 },
    { 0xb9b6a6db, 93 },
    { 0xf4b6a6db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc386a6e0, 91 },
    { 0x1826a6e1, 61 },
    { 0, 0 },
    { 0x1c86a6e3, 34 },
    { 0, 0 },
    { 0xd186a6e5, 90 },
    { 0x6016a6e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386a6ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a6f2, 90 },
    { 0, 0 },
    { 0x27c6a6f4, 87 },
    { 0, 0 },
    { 0x55f6a6f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad86a6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266a701, 33 },
    { 0x33b6a701, 23 },
    { 0xeca6a701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a6a706, 90 },
    { 0x5866a706, 76 },
    { 0x1b56a708, 91 },
    { 0x8b96a706, 99 },
    { 0x7ef6a70a, 43 },
    { 0x6cc6a708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a6a70e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee86a714, 90 },
    { 0xee76a715, 76 },
    { 0x2aa6a716, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd56a719, 40 },
    { 0x9246a71a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a71d, 87 },
    { 0xc386a71e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b6a728, 90 },
    { 0, 0 },
    { 0xeb86a72a, 75 },
    { 0xe5f6a72a, 33 },
    { 0, 0 },
    { 0xce66a72d, 95 },
    { 0xc116a72e, 34 },
    { 0x7416a72e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a6a739, 33 },
    { 0, 0 },
    { 0x8e26a73b, 90 },
    { 0, 0 },
    { 0x74a6a73d, 93 },
    { 0, 0 },
    { 0x61a6a73f, 61 },
    { 0xb556a740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946a746, 95 },
    { 0x9006a747, 91 },
    { 0x8296a748, 40 },
    { 0x24a6a747, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16a751, 17 },
    { 0x5f46a752, 93 },
    { 0x3cc6a753, 61 },
    { 0xdc6a753, 34 },
    { 0x5366a754, 90 },
    { 0, 0 },
    { 0xf946a757, 61 },
    { 0x31b6a758, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276a75d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a6a762, 90 },
    { 0, 0 },
    { 0x8f76a764, 91 },
    { 0xc506a764, 23 },
    { 0xb696a766, 83 },
    { 0, 0 },
    { 0x82e6a768, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6166a779, 33 },
    { 0x21d6a77a, 78 },
    { 0, 0 },
    { 0x7a76a77c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef96a77f, 33 },
    { 0x6eb6a780, 17 },
    { 0x1446a780, 81 },
    { 0x2ff6a780, 33 },
    { 0xe166a783, 98 },
    { 0xfed6a781, 76 },
    { 0xcaa6a785, 33 },
    { 0xafb6a786, 34 },
    { 0x6f16a787, 90 },
    { 0xd356a788, 95 },
    { 0xee6a788, 98 },
    { 0xf0b6a789, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36a78f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc216a794, 52 },
    { 0x6e36a795, 33 },
    { 0x5206a795, 98 },
    { 0x2386a797, 61 },
    { 0x2546a798, 52 },
    { 0xece6a797, 93 },
    { 0, 0 },
    { 0x6c26a79b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc706a7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4446a7a6, 23 },
    { 0xb406a7a7, 33 },
    { 0xc2e6a7a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe626a7ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb6a7b3, 93 },
    { 0x6636a7b3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6916a7b8, 93 },
    { 0x5076a7b9, 17 },
    { 0x51c6a7ba, 90 },
    { 0xa006a7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff6a7be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab16a7c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e06a7c5, 90 },
    { 0, 0 },
    { 0x34d6a7c7, 90 },
    { 0xcdc6a7c8, 23 },
    { 0, 0 },
    { 0xf6e6a7ca, 61 },
    { 0x6006a7ca, 76 },
    { 0x5346a7cb, 93 },
    { 0x13a6a7cc, 85 },
    { 0x75f6a7ce, 33 },
    { 0xf996a7cf, 90 },
    { 0x73d6a7d0, 23 },
    { 0x2b86a7d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386a7d6, 66 },
    { 0x30e6a7d7, 62 },
    { 0xf6a7d7, 90 },
    { 0x87c6a7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd66a7df, 93 },
    { 0x5006a7df, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366a7e7, 61 },
    { 0x6a6a7e7, 91 },
    { 0xe876a7e9, 9 },
    { 0xcc86a7e8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xf616a7ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976a7f1, 61 },
    { 0xab56a7f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6a806, 93 },
    { 0x54e6a806, 33 },
    { 0, 0 },
    { 0x766a809, 98 },
    { 0xa3a6a80a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe136a80d, 61 },
    { 0xc286a80e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd786a813, 33 },
    { 0xad26a814, 99 },
    { 0xf036a815, 61 },
    { 0x3db6a815, 33 },
    { 0x4826a814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc036a81d, 91 },
    { 0, 0 },
    { 0x3d46a81f, 90 },
    { 0x7186a81f, 90 },
    { 0, 0 },
    { 0x1856a822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096a826, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e76a82c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a46a839, 23 },
    { 0, 0 },
    { 0xe206a83b, 75 },
    { 0xceb6a83b, 61 },
    { 0xe346a83d, 90 },
    { 0xae46a83e, 33 },
    { 0xc586a83e, 23 },
    { 0xe586a83d, 23 },
    { 0x9d56a841, 33 },
    { 0x3c66a83d, 61 },
    { 0x5836a843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb6a846, 56 },
    { 0, 0 },
    { 0xfb96a848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b56a850, 88 },
    { 0, 0 },
    { 0xfb26a852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f6a855, 77 },
    { 0, 0 },
    { 0x76b6a857, 85 },
    { 0xe826a858, 88 },
    { 0x8336a859, 33 },
    { 0xf226a85a, 99 },
    { 0xb896a858, 98 },
    { 0xff56a857, 33 },
    { 0xa2f6a85c, 43 },
    { 0xa256a85e, 95 },
    { 0x28a6a85f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06a866, 34 },
    { 0, 0 },
    { 0x4b56a868, 23 },
    { 0, 0 },
    { 0xb8f6a86a, 61 },
    { 0xbd6a86b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd286a870, 43 },
    { 0xe396a871, 55 },
    { 0x87c6a872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a06a87d, 81 },
    { 0xf9a6a87e, 55 },
    { 0x4856a87f, 85 },
    { 0xd0a6a880, 34 },
    { 0xfd86a881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb346a884, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e6a88c, 76 },
    { 0x6d6a88c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c6a890, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e6a899, 38 },
    { 0, 0 },
    { 0xaf06a89b, 98 },
    { 0, 0 },
    { 0x32a6a89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa206a8a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa06a8ac, 52 },
    { 0, 0 },
    { 0x4446a8ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db6a8b1, 93 },
    { 0x79a6a8b1, 90 },
    { 0, 0 },
    { 0x6876a8b4, 80 },
    { 0x61c6a8b4, 55 },
    { 0, 0 },
    { 0xe726a8b7, 98 },
    { 0xed86a8b8, 55 },
    { 0x8b96a8b9, 90 },
    { 0xf056a8ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb126a8c0, 23 },
    { 0xb626a8c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8606a8c6, 33 },
    { 0xf9b6a8c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c6a8ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6a8cd, 90 },
    { 0x1cb6a8cd, 81 },
    { 0, 0 },
    { 0xd36a8d0, 43 },
    { 0x5176a8d1, 95 },
    { 0, 0 },
    { 0xdd36a8d3, 77 },
    { 0, 0 },
    { 0x6bd6a8d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa86a8d8, 67 },
    { 0x2376a8d9, 85 },
    { 0x7116a8d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f76a8dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6a8e4, 93 },
    { 0xce56a8e5, 90 },
    { 0x6246a8e6, 80 },
    { 0xd486a8e6, 55 },
    { 0xdf16a8e8, 84 },
    { 0x7516a8e9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb696a8ef, 61 },
    { 0xe386a8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd286a8f3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df6a8fa, 85 },
    { 0xf226a8fb, 15 },
    { 0x3cb6a8fc, 33 },
    { 0x256a8fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86a900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146a90f, 88 },
    { 0xf896a910, 98 },
    { 0x816a911, 95 },
    { 0xc4c6a912, 93 },
    { 0, 0 },
    { 0x8d26a914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906a91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3726a91f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb86a925, 84 },
    { 0x5826a926, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f6a929, 66 },
    { 0x5c96a929, 66 },
    { 0xd626a92b, 91 },
    { 0x4496a92c, 69 },
    { 0xa8b6a92d, 98 },
    { 0x4156a92e, 98 },
    { 0xcc86a92f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf846a932, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde6a939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316a93c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76a942, 43 },
    { 0, 0 },
    { 0xfc56a944, 23 },
    { 0x6386a944, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd66a948, 87 },
    { 0xfeb6a948, 33 },
    { 0, 0 },
    { 0x32e6a94b, 61 },
    { 0x3cc6a94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee6a950, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b6a953, 33 },
    { 0, 0 },
    { 0xb1b6a955, 33 },
    { 0x9836a956, 90 },
    { 0x49d6a955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed56a960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c6a963, 33 },
    { 0, 0 },
    { 0x1cd6a965, 33 },
    { 0x7076a965, 90 },
    { 0, 0 },
    { 0x1c76a968, 95 },
    { 0x8246a969, 99 },
    { 0xebb6a96a, 85 },
    { 0x1fc6a968, 90 },
    { 0x6076a96a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5006a972, 98 },
    { 0, 0 },
    { 0xa8b6a974, 23 },
    { 0xd276a975, 61 },
    { 0x9ab6a976, 95 },
    { 0, 0 },
    { 0xf0a6a978, 100 },
    { 0xbee6a979, 78 },
    { 0x1c6a979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7446a980, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6a983, 33 },
    { 0, 0 },
    { 0x38c6a985, 88 },
    { 0x3b76a986, 33 },
    { 0xe446a987, 33 },
    { 0xbee6a988, 55 },
    { 0xb7e6a989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a6a98e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb126a992, 43 },
    { 0, 0 },
    { 0x1616a994, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26a997, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb156a99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe086a9a1, 98 },
    { 0x7ca6a9a2, 98 },
    { 0x1516a9a3, 23 },
    { 0x1266a9a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a9a7, 55 },
    { 0xe306a9a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba6a9ac, 46 },
    { 0xc076a9ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec6a9b0, 49 },
    { 0, 0 },
    { 0x1c06a9b2, 33 },
    { 0xd7b6a9b3, 98 },
    { 0x4fd6a9b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c46a9ba, 43 },
    { 0, 0 },
    { 0xedf6a9bc, 98 },
    { 0x70e6a9bd, 90 },
    { 0x6bf6a9be, 33 },
    { 0x9956a9be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f6a9c4, 99 },
    { 0, 0 },
    { 0x1b6a9c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c6a9cd, 33 },
    { 0x6036a9cd, 46 },
    { 0xb786a9cf, 23 },
    { 0x6476a9cd, 91 },
    { 0, 0 },
    { 0x3b46a9d2, 61 },
    { 0x62e6a9d3, 93 },
    { 0x5206a9d3, 33 },
    { 0, 0 },
    { 0x18c6a9d6, 61 },
    { 0xd8a6a9d7, 34 },
    { 0xc916a9d8, 93 },
    { 0xf9e6a9d9, 48 },
    { 0xf5c6a9da, 78 },
    { 0x51e6a9d9, 90 },
    { 0x7126a9db, 95 },
    { 0, 0 },
    { 0x2156a9de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe106a9e2, 98 },
    { 0, 0 },
    { 0xda16a9e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a9e7, 34 },
    { 0, 0 },
    { 0x8bd6a9e9, 98 },
    { 0xbcd6a9ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416a9f0, 61 },
    { 0, 0 },
    { 0xe206a9f2, 55 },
    { 0x6106a9f3, 95 },
    { 0x4ba6a9f3, 98 },
    { 0x5bf6a9f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e6a9fa, 61 },
    { 0xc9e6a9fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xba16a9fe, 76 },
    { 0x826a9fe, 95 },
    { 0xffe6aa00, 53 },
    { 0x9a66aa00, 55 },
    { 0, 0 },
    { 0x3166aa03, 43 },
    { 0, 0 },
    { 0x5386aa05, 17 },
    { 0, 0 },
    { 0xafe6aa07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb36aa0a, 61 },
    { 0xdc36aa0b, 61 },
    { 0x9126aa0c, 33 },
    { 0, 0 },
    { 0xe3f6aa0e, 61 },
    { 0x2f56aa0f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3786aa13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb6aa16, 33 },
    { 0x6ad6aa17, 55 },
    { 0, 0 },
    { 0x3316aa19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6aa26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516aa2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2696aa30, 61 },
    { 0x4736aa31, 77 },
    { 0x286aa31, 66 },
    { 0x82a6aa33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee46aa38, 90 },
    { 0, 0 },
    { 0x78e6aa3a, 33 },
    { 0xbce6aa3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc6aa3f, 91 },
    { 0xd2c6aa3f, 95 },
    { 0xe4b6aa41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d6aa44, 33 },
    { 0, 0 },
    { 0xf016aa46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1206aa49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396aa4e, 91 },
    { 0x9a36aa4e, 34 },
    { 0x2c6aa4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2996aa54, 95 },
    { 0x67b6aa55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b36aa58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6aa5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486aa65, 99 },
    { 0xa8b6aa66, 25 },
    { 0xef16aa67, 71 },
    { 0x3e16aa68, 55 },
    { 0, 0 },
    { 0x826aa6a, 85 },
    { 0, 0 },
    { 0x9106aa6c, 61 },
    { 0x64f6aa6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e6aa74, 61 },
    { 0, 0 },
    { 0x7dc6aa76, 81 },
    { 0xac16aa77, 81 },
    { 0x1596aa76, 61 },
    { 0x6886aa79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe6aa7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa396aa82, 98 },
    { 0xa346aa83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326aa8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7276aa8d, 52 },
    { 0x70f6aa8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf416aa93, 90 },
    { 0x6326aa94, 38 },
    { 0x4966aa93, 55 },
    { 0x7786aa94, 95 },
    { 0xbc56aa95, 99 },
    { 0x5116aa98, 33 },
    { 0x966aa98, 23 },
    { 0x68a6aa9a, 99 },
    { 0x2146aa9b, 90 },
    { 0x1246aa96, 55 },
    { 0x1996aa9d, 23 },
    { 0x5476aa9e, 55 },
    { 0x2c96aa97, 98 },
    { 0x94a6aaa0, 52 },
    { 0xda6aaa0, 61 },
    { 0x7966aaa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a6aaa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf6aaae, 98 },
    { 0x9c86aaaf, 61 },
    { 0x5246aab0, 95 },
    { 0xedd6aab1, 94 },
    { 0xfa26aab2, 33 },
    { 0x8ac6aab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f96aab9, 34 },
    { 0xe876aab9, 93 },
    { 0xde76aabb, 93 },
    { 0, 0 },
    { 0x576aabd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6aac1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b6aac7, 61 },
    { 0x3db6aac8, 90 },
    { 0x3a96aac9, 100 },
    { 0, 0 },
    { 0x7a26aacb, 93 },
    { 0, 0 },
    { 0xf7a6aacd, 61 },
    { 0xa86aacd, 85 },
    { 0xa596aacf, 61 },
    { 0x12c6aad0, 90 },
    { 0x7516aacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf6aad4, 99 },
    { 0, 0 },
    { 0xffd6aad6, 98 },
    { 0x7c86aad7, 61 },
    { 0x7e66aad8, 95 },
    { 0xc516aad9, 81 },
    { 0x3ea6aada, 95 },
    { 0x6236aada, 84 },
    { 0x6366aad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e86aae0, 98 },
    { 0, 0 },
    { 0xf1f6aae2, 43 },
    { 0xb806aae3, 81 },
    { 0xc6c6aae3, 33 },
    { 0, 0 },
    { 0xe1b6aae6, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6aae9, 61 },
    { 0x90b6aae9, 87 },
    { 0x8b26aaeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d6aaee, 61 },
    { 0xf896aaef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3326aaf2, 78 },
    { 0x6c66aaf3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe966aaff, 98 },
    { 0xec26ab00, 95 },
    { 0xe446ab01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726ab07, 23 },
    { 0xa5c6ab07, 84 },
    { 0xdb06ab09, 98 },
    { 0xb5f6ab08, 93 },
    { 0xb296ab0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6ab0f, 81 },
    { 0x2676ab10, 33 },
    { 0, 0 },
    { 0xbc26ab12, 23 },
    { 0, 0 },
    { 0x5c56ab14, 90 },
    { 0xd896ab15, 52 },
    { 0x6ab16, 93 },
    { 0x6586ab16, 75 },
    { 0xcdd6ab18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae6ab1c, 95 },
    { 0x6716ab1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8396ab20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456ab24, 34 },
    { 0xdcc6ab24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056ab2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce66ab2d, 2 },
    { 0, 0 },
    { 0xf4f6ab2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e26ab32, 34 },
    { 0, 0 },
    { 0x3ce6ab34, 33 },
    { 0x4f6ab34, 33 },
    { 0x47f6ab34, 33 },
    { 0xd756ab37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f86ab3c, 76 },
    { 0x1d86ab3d, 61 },
    { 0x32c6ab3e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056ab41, 61 },
    { 0xdbd6ab42, 17 },
    { 0x6ba6ab43, 95 },
    { 0x53c6ab43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f66ab4c, 98 },
    { 0xe186ab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186ab57, 98 },
    { 0x6a86ab57, 98 },
    { 0x6f76ab57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6ab5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586ab66, 23 },
    { 0, 0 },
    { 0xe4c6ab68, 33 },
    { 0, 0 },
    { 0x5886ab6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276ab6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516ab72, 61 },
    { 0, 0 },
    { 0x7376ab74, 55 },
    { 0x2bb6ab75, 87 },
    { 0xe316ab76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706ab79, 92 },
    { 0, 0 },
    { 0xb996ab7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466ab80, 23 },
    { 0x9706ab80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc596ab88, 84 },
    { 0x1f26ab89, 33 },
    { 0, 0 },
    { 0x70d6ab8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd316ab8e, 53 },
    { 0xceb6ab8e, 61 },
    { 0x2556ab8f, 95 },
    { 0x6236ab91, 55 },
    { 0, 0 },
    { 0xf66ab93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1206ab97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2936ab9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5886aba2, 77 },
    { 0x76f6aba2, 23 },
    { 0x36f6aba2, 23 },
    { 0x136aba3, 23 },
    { 0xfd06aba6, 61 },
    { 0xc726aba7, 77 },
    { 0xb2b6aba7, 95 },
    { 0xddc6aba2, 95 },
    { 0xd406abaa, 95 },
    { 0x4146abab, 33 },
    { 0x5c76abab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce46abaf, 95 },
    { 0, 0 },
    { 0x4c36abb1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6abb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a6abc1, 76 },
    { 0xa2f6abc2, 100 },
    { 0xc8f6abc3, 90 },
    { 0x7ab6abc2, 83 },
    { 0x8cb6abc5, 76 },
    { 0xa206abc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff86abcc, 90 },
    { 0x4536abcd, 95 },
    { 0x7f66abce, 87 },
    { 0xde76abcf, 33 },
    { 0xa2b6abcc, 61 },
    { 0x97c6abcc, 99 },
    { 0x9146abd2, 95 },
    { 0x3916abd2, 68 },
    { 0xc5a6abd2, 98 },
    { 0xe356abce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6abd9, 46 },
    { 0x71f6abda, 66 },
    { 0x7d6abd9, 33 },
    { 0xf726abd9, 33 },
    { 0xfac6abd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6abe1, 33 },
    { 0, 0 },
    { 0x4106abe3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1016abe6, 61 },
    { 0x63f6abe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2296abec, 34 },
    { 0x16c6abed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6abf9, 55 },
    { 0, 0 },
    { 0xcc76abfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976ac00, 95 },
    { 0, 0 },
    { 0x43e6ac02, 98 },
    { 0x2ea6ac02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd6ac07, 34 },
    { 0, 0 },
    { 0x9716ac09, 78 },
    { 0xd886ac0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5656ac0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa376ac12, 76 },
    { 0x8b6ac12, 75 },
    { 0, 0 },
    { 0x1976ac15, 90 },
    { 0x3a46ac15, 23 },
    { 0x1006ac15, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ac1d, 90 },
    { 0xfeb6ac1e, 33 },
    { 0xfd56ac1e, 90 },
    { 0x97d6ac20, 75 },
    { 0xb056ac20, 98 },
    { 0x35c6ac21, 46 },
    { 0x696ac20, 85 },
    { 0xa8e6ac1f, 23 },
    { 0xf706ac20, 23 },
    { 0xc996ac1d, 33 },
    { 0x406ac20, 93 },
    { 0x9ca6ac28, 61 },
    { 0x1c66ac29, 90 },
    { 0x1316ac29, 93 },
    { 0x3c66ac2b, 88 },
    { 0x9846ac2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a6ac2f, 100 },
    { 0xb8a6ac30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6ac33, 34 },
    { 0xb266ac34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc736ac37, 98 },
    { 0x25b6ac38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2936ac43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed36ac47, 43 },
    { 0xa536ac47, 23 },
    { 0xd036ac49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb576ac51, 75 },
    { 0, 0 },
    { 0x5766ac53, 61 },
    { 0x9586ac53, 61 },
    { 0x88a6ac55, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6ac58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6ac5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726ac65, 98 },
    { 0x3be6ac66, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56ac6a, 76 },
    { 0x1056ac6a, 85 },
    { 0x7b66ac6c, 90 },
    { 0xfc96ac6c, 78 },
    { 0xedc6ac6e, 76 },
    { 0xe0b6ac6f, 98 },
    { 0x3f6ac70, 87 },
    { 0xadd6ac71, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ac74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf56ac7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7606ac7e, 33 },
    { 0x4b86ac7f, 90 },
    { 0xfca6ac7e, 95 },
    { 0, 0 },
    { 0xe8f6ac82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f26ac88, 76 },
    { 0, 0 },
    { 0xddd6ac8a, 90 },
    { 0, 0 },
    { 0xf396ac8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176ac90, 33 },
    { 0, 0 },
    { 0xd2c6ac92, 95 },
    { 0x34d6ac93, 90 },
    { 0x79d6ac94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6ac98, 43 },
    { 0, 0 },
    { 0x8016ac9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac6ac9f, 90 },
    { 0x5e6aca0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b66aca5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e36acac, 67 },
    { 0xa606acad, 34 },
    { 0xc736acad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c26acb2, 84 },
    { 0xd786acb2, 23 },
    { 0xed56acb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x916acb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746acbc, 61 },
    { 0, 0 },
    { 0x4d56acbe, 95 },
    { 0x3066acbf, 90 },
    { 0, 0 },
    { 0xf8b6acc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf6acc5, 85 },
    { 0xd176acc5, 33 },
    { 0xe186acc7, 61 },
    { 0xfef6acc7, 55 },
    { 0, 0 },
    { 0x556acca, 52 },
    { 0x7f96accb, 95 },
    { 0xb586accb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6acd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5376acd5, 90 },
    { 0, 0 },
    { 0xc226acd7, 64 },
    { 0x19d6acd8, 34 },
    { 0, 0 },
    { 0x2796acda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2206acde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a6ace2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7726ace5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726ace8, 98 },
    { 0x6a66ace8, 91 },
    { 0x1c56ace8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb986acef, 61 },
    { 0x3a06acef, 90 },
    { 0x3b6acf1, 23 },
    { 0x6cb6acf1, 95 },
    { 0x53e6acf1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8216acf7, 61 },
    { 0x1586acf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6acfc, 99 },
    { 0, 0 },
    { 0xc496acfe, 61 },
    { 0x3146acff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2756ad04, 67 },
    { 0x99c6ad04, 90 },
    { 0xa7f6ad06, 23 },
    { 0x5f26ad04, 95 },
    { 0xe3c6ad08, 23 },
    { 0x8f56ad08, 34 },
    { 0xb5f6ad0a, 64 },
    { 0x2006ad0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c6ad0f, 90 },
    { 0xd4a6ad0f, 23 },
    { 0xabb6ad11, 23 },
    { 0x3cf6ad11, 61 },
    { 0xd2f6ad13, 45 },
    { 0, 0 },
    { 0x44f6ad15, 91 },
    { 0xc2b6ad16, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae6ad19, 61 },
    { 0x4a36ad1a, 95 },
    { 0x62e6ad1b, 76 },
    { 0x2976ad1a, 61 },
    { 0x7c66ad1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5146ad20, 95 },
    { 0, 0 },
    { 0x8626ad22, 77 },
    { 0x2796ad23, 76 },
    { 0x2cb6ad23, 33 },
    { 0xe1e6ad23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6166ad28, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e6ad2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c6ad34, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc6ad38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7746ad3e, 23 },
    { 0, 0 },
    { 0xfad6ad40, 61 },
    { 0x9006ad40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496ad45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a6ad4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466ad4d, 67 },
    { 0xa746ad4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9426ad57, 85 },
    { 0, 0 },
    { 0x7ad6ad59, 95 },
    { 0x72c6ad5a, 55 },
    { 0x2cd6ad5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2556ad5e, 95 },
    { 0xcf36ad5e, 46 },
    { 0x2d26ad60, 99 },
    { 0x4276ad61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd426ad65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8886ad6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3986ad72, 98 },
    { 0xe3a6ad73, 61 },
    { 0x1146ad74, 95 },
    { 0x5036ad74, 90 },
    { 0x6946ad75, 92 },
    { 0x4ac6ad74, 99 },
    { 0xfa66ad77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26ad7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8306ad87, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116ad8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ad97, 95 },
    { 0xf126ad98, 95 },
    { 0x5116ad99, 23 },
    { 0x5ff6ad9a, 66 },
    { 0xb806ad9b, 33 },
    { 0x38d6ad9c, 33 },
    { 0xf3f6ad9d, 77 },
    { 0x9dc6ad9d, 33 },
    { 0, 0 },
    { 0xa9c6ada0, 61 },
    { 0x5d86ada1, 66 },
    { 0x3486ada2, 61 },
    { 0xe386ada3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6ada6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd186adab, 95 },
    { 0xff36adab, 61 },
    { 0x10c6adad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c16adb5, 46 },
    { 0xccd6adb5, 98 },
    { 0x7b46adb5, 93 },
    { 0xfec6adb8, 84 },
    { 0x9a46adb9, 68 },
    { 0x4d66adba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1116adc0, 93 },
    { 0x2886adc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c6adc8, 61 },
    { 0, 0 },
    { 0x4476adca, 24 },
    { 0xb886adcb, 61 },
    { 0x9016adcb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda6adcf, 23 },
    { 0x84b6adcf, 33 },
    { 0xfa26add1, 68 },
    { 0, 0 },
    { 0xf406add3, 90 },
    { 0, 0 },
    { 0x3ab6add5, 77 },
    { 0, 0 },
    { 0xb2c6add7, 66 },
    { 0, 0 },
    { 0x4b06add9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1146ade0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016ade3, 61 },
    { 0, 0 },
    { 0xcc96ade5, 61 },
    { 0x3e56ade5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046ade9, 90 },
    { 0xd9d6adea, 90 },
    { 0x3706ade9, 40 },
    { 0xc806ade9, 23 },
    { 0xd6c6aded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f6adf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3176adf8, 95 },
    { 0xdbd6adf9, 90 },
    { 0x7996adfa, 23 },
    { 0, 0 },
    { 0xbdb6adfc, 42 },
    { 0xd366adfc, 85 },
    { 0xa206adfd, 98 },
    { 0xfdb6adff, 90 },
    { 0x99d6ae00, 90 },
    { 0xcc26ae01, 90 },
    { 0x6286ae02, 96 },
    { 0x2f66ae02, 23 },
    { 0, 0 },
    { 0xb7d6ae05, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ae0a, 90 },
    { 0x2d06ae0b, 46 },
    { 0, 0 },
    { 0xe7f6ae0d, 90 },
    { 0xafe6ae0d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b46ae17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3936ae1a, 88 },
    { 0xaad6ae1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a6ae20, 81 },
    { 0x9e56ae20, 85 },
    { 0xe8a6ae22, 78 },
    { 0x7d66ae23, 91 },
    { 0xf216ae21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5676ae28, 76 },
    { 0x6c76ae28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6126ae2c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66ae30, 91 },
    { 0xd076ae31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7506ae35, 55 },
    { 0xeac6ae36, 90 },
    { 0x72a6ae37, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be6ae3c, 84 },
    { 0, 0 },
    { 0xdc66ae3e, 33 },
    { 0xb256ae3e, 33 },
    { 0x48c6ae40, 61 },
    { 0x42f6ae41, 68 },
    { 0x1bd6ae42, 53 },
    { 0xc4c6ae43, 85 },
    { 0x9c26ae44, 55 },
    { 0xdfa6ae45, 52 },
    { 0xce16ae46, 61 },
    { 0x4a46ae47, 95 },
    { 0x1c56ae45, 100 },
    { 0x8766ae49, 98 },
    { 0xa816ae4a, 77 },
    { 0xde06ae4b, 61 },
    { 0x9996ae4b, 33 },
    { 0x5526ae49, 61 },
    { 0xe396ae4e, 66 },
    { 0x9476ae46, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8106ae52, 90 },
    { 0, 0 },
    { 0xfc96ae54, 34 },
    { 0x6cd6ae54, 11 },
    { 0xe476ae54, 65 },
    { 0xd436ae55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af6ae5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226ae5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf6ae62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3526ae6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646ae6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc6ae71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e6ae79, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ae80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb766ae84, 98 },
    { 0x6406ae84, 85 },
    { 0x1886ae86, 76 },
    { 0x2d86ae87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb86ae8f, 99 },
    { 0x53c6ae90, 23 },
    { 0x7236ae91, 90 },
    { 0x87a6ae92, 96 },
    { 0, 0 },
    { 0xbcd6ae94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb226ae99, 43 },
    { 0xbfa6ae9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca6ae9f, 76 },
    { 0x7d36ae9f, 40 },
    { 0, 0 },
    { 0x6bd6aea2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426aeb2, 77 },
    { 0x3e26aeb3, 61 },
    { 0x6976aeb4, 61 },
    { 0, 0 },
    { 0x5f46aeb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e66aeba, 100 },
    { 0x4486aebb, 23 },
    { 0, 0 },
    { 0xfa06aebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c16aec0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6aec6, 93 },
    { 0x7d36aec6, 75 },
    { 0x4606aec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x716aecb, 52 },
    { 0x8e96aecc, 95 },
    { 0x1f6aecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa6aed1, 81 },
    { 0x6dc6aed2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9056aed5, 98 },
    { 0x9526aed6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c96aedb, 98 },
    { 0x8456aedb, 95 },
    { 0x2876aedd, 76 },
    { 0, 0 },
    { 0x116aedf, 12 },
    { 0, 0 },
    { 0x2e86aee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e6aee6, 98 },
    { 0xcf46aee6, 90 },
    { 0, 0 },
    { 0xd546aee9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3386aeef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d56aef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1146aef8, 85 },
    { 0, 0 },
    { 0xc9a6aefa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6aefd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9306af00, 96 },
    { 0x556af01, 95 },
    { 0x2ef6af02, 98 },
    { 0x4396af02, 23 },
    { 0x346af00, 95 },
    { 0, 0 },
    { 0xb7b6af06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46af0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf986af0e, 55 },
    { 0, 0 },
    { 0x78c6af10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d6af14, 90 },
    { 0x9836af14, 43 },
    { 0x8a6af16, 43 },
    { 0, 0 },
    { 0xe876af18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be6af21, 34 },
    { 0, 0 },
    { 0x95b6af23, 95 },
    { 0x2046af24, 90 },
    { 0, 0 },
    { 0x8906af26, 95 },
    { 0x1e6af27, 78 },
    { 0x4756af28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5836af2e, 56 },
    { 0x596af2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd6af34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836af3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4866af4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876af5e, 33 },
    { 0xd066af5f, 61 },
    { 0x906af5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa46af66, 90 },
    { 0xfcb6af66, 33 },
    { 0xd3a6af68, 66 },
    { 0xc4f6af69, 23 },
    { 0x5836af6a, 90 },
    { 0x6b26af67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd6af73, 98 },
    { 0x9f76af74, 40 },
    { 0x7bf6af74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336af7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc36af80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a36af83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6146af86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6af89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e6af8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc36af94, 61 },
    { 0x1a16af94, 99 },
    { 0x45d6af96, 61 },
    { 0xa596af96, 90 },
    { 0x3e96af97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066af9b, 53 },
    { 0x7026af9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6afa0, 97 },
    { 0xe1a6afa1, 61 },
    { 0x4446afa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556afa5, 34 },
    { 0x13d6afa6, 90 },
    { 0xcb16afa6, 52 },
    { 0x3476afa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36afad, 56 },
    { 0x3c06afad, 61 },
    { 0x1486afae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db6afb2, 43 },
    { 0x5d06afb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a6afb6, 23 },
    { 0xbad6afb7, 17 },
    { 0xd366afb8, 23 },
    { 0x2d46afb6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046afc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa696afd3, 33 },
    { 0x4ba6afd4, 66 },
    { 0, 0 },
    { 0x3156afd6, 93 },
    { 0x92c6afd7, 85 },
    { 0x4db6afd7, 90 },
    { 0x4cf6afd9, 17 },
    { 0, 0 },
    { 0x4746afdb, 90 },
    { 0xae06afdb, 98 },
    { 0x9df6afdb, 61 },
    { 0, 0 },
    { 0x7c56afdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b06afe3, 23 },
    { 0xbb26afe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b26afe9, 78 },
    { 0x336afea, 68 },
    { 0x5386afea, 95 },
    { 0x4a46afec, 17 },
    { 0xaa06afed, 66 },
    { 0xd226afec, 33 },
    { 0, 0 },
    { 0xcbc6aff0, 90 },
    { 0, 0 },
    { 0xe1a6aff2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56aff7, 61 },
    { 0, 0 },
    { 0x1576aff9, 23 },
    { 0, 0 },
    { 0x5926affb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6b003, 55 },
    { 0x4ef6b003, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4786b00a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d6b00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176b012, 95 },
    { 0xeee6b013, 52 },
    { 0x44a6b014, 52 },
    { 0xc6e6b015, 61 },
    { 0, 0 },
    { 0x2fc6b017, 23 },
    { 0xd6b018, 90 },
    { 0, 0 },
    { 0x21f6b01a, 90 },
    { 0, 0 },
    { 0xdf06b01c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf6b026, 95 },
    { 0x85b6b026, 87 },
    { 0x5256b027, 75 },
    { 0x5ed6b029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a6b02c, 33 },
    { 0x9fc6b02d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6b030, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb366b034, 91 },
    { 0x92a6b035, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566b039, 55 },
    { 0xdd6b03a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b6b03e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a66b044, 96 },
    { 0x55b6b045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96b049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156b04d, 91 },
    { 0x2716b04d, 33 },
    { 0, 0 },
    { 0x9ff6b050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c6b060, 90 },
    { 0, 0 },
    { 0x7b26b062, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6b065, 55 },
    { 0x5e86b066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456b06b, 33 },
    { 0xe0e6b06b, 90 },
    { 0, 0 },
    { 0x5d46b06e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff6b072, 85 },
    { 0x56b6b073, 23 },
    { 0xde16b072, 95 },
    { 0xd166b075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126b078, 33 },
    { 0x89e6b079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb096b07d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b081, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9436b08a, 23 },
    { 0, 0 },
    { 0x8816b08c, 90 },
    { 0xe436b08c, 77 },
    { 0xe276b08e, 93 },
    { 0, 0 },
    { 0xb816b090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4186b094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6b099, 17 },
    { 0xa526b09a, 17 },
    { 0x2826b09b, 67 },
    { 0x79b6b09c, 76 },
    { 0xd7c6b09d, 93 },
    { 0xef96b09e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc16b0a5, 76 },
    { 0x6986b0a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf386b0af, 90 },
    { 0xfc96b0b0, 98 },
    { 0x3726b0af, 75 },
    { 0xccb6b0b2, 33 },
    { 0xf576b0b3, 34 },
    { 0xa806b0b2, 46 },
    { 0, 0 },
    { 0x7b36b0b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe506b0bf, 100 },
    { 0xb9f6b0c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066b0c6, 95 },
    { 0xdbc6b0c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6b0ca, 84 },
    { 0x4106b0cb, 66 },
    { 0x7516b0cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff6b0d2, 42 },
    { 0xa2d6b0d2, 23 },
    { 0x1016b0d3, 93 },
    { 0, 0 },
    { 0x95c6b0d6, 33 },
    { 0x2056b0d7, 95 },
    { 0xada6b0d6, 84 },
    { 0xa9a6b0d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a6b0de, 33 },
    { 0x146b0df, 33 },
    { 0xcb16b0e0, 98 },
    { 0xf1b6b0e1, 43 },
    { 0x1ee6b0e1, 33 },
    { 0xdf46b0e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6b0ea, 90 },
    { 0xff6b0eb, 33 },
    { 0xd936b0ec, 90 },
    { 0x20c6b0ec, 91 },
    { 0xbfe6b0ea, 99 },
    { 0x67e6b0ee, 98 },
    { 0, 0 },
    { 0x8e56b0f1, 61 },
    { 0xef66b0f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566b0f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f8, 100 },
    { 0x43a6b0f9, 33 },
    { 0, 0 },
    { 0xc9d6b0fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356b0fe, 37 },
    { 0, 0 },
    { 0xb3e6b100, 81 },
    { 0xbdc6b100, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b6b104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8466b107, 52 },
    { 0x6c26b108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6b10e, 23 },
    { 0, 0 },
    { 0x3316b110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9966b115, 23 },
    { 0, 0 },
    { 0xdb96b117, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6b11d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396b120, 90 },
    { 0x2786b121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106b127, 33 },
    { 0x2c86b127, 91 },
    { 0, 0 },
    { 0x5ed6b12a, 33 },
    { 0x6a86b12b, 61 },
    { 0, 0 },
    { 0x9236b12d, 91 },
    { 0x80b6b12e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa6b131, 23 },
    { 0, 0 },
    { 0xdc96b133, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb286b136, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f6b13f, 41 },
    { 0x9746b140, 80 },
    { 0x7b96b141, 61 },
    { 0x2676b142, 65 },
    { 0x5656b140, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c6b149, 76 },
    { 0x93e6b14a, 91 },
    { 0x4686b14b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca6b14e, 93 },
    { 0x7916b14f, 90 },
    { 0xcd06b14f, 90 },
    { 0x2946b14f, 66 },
    { 0x5726b14f, 95 },
    { 0xbb46b153, 66 },
    { 0xa286b153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc6b159, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc376b15c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6b15f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3766b162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b16c, 99 },
    { 0x49a6b16d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd006b174, 95 },
    { 0x6e76b174, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5876b178, 76 },
    { 0x1486b179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5956b17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6b185, 33 },
    { 0xe106b186, 75 },
    { 0, 0 },
    { 0xd1a6b188, 76 },
    { 0, 0 },
    { 0x3156b18a, 33 },
    { 0x8256b18a, 23 },
    { 0x1ef6b18c, 91 },
    { 0, 0 },
    { 0x8166b18e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f66b192, 61 },
    { 0x1f76b193, 91 },
    { 0xf7b6b192, 46 },
    { 0x1106b195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd6b199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b19c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6b19f, 61 },
    { 0, 0 },
    { 0x7646b1a1, 61 },
    { 0x5086b1a2, 81 },
    { 0xe76b1a3, 33 },
    { 0xaeb6b1a4, 90 },
    { 0xfb26b1a5, 34 },
    { 0xbb36b1a6, 43 },
    { 0xe8f6b1a7, 95 },
    { 0, 0 },
    { 0xeee6b1a9, 93 },
    { 0x5366b1a9, 33 },
    { 0xdd26b1ab, 61 },
    { 0xd6b6b1ac, 98 },
    { 0x9466b1ab, 98 },
    { 0, 0 },
    { 0xb6c6b1af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6b1b5, 90 },
    { 0xb9d6b1b6, 76 },
    { 0xd836b1b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f46b1bc, 76 },
    { 0xf2b6b1bd, 98 },
    { 0, 0 },
    { 0x38b6b1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e36b1c5, 90 },
    { 0x8f66b1c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066b1c9, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26b1d2, 91 },
    { 0x4026b1d2, 59 },
    { 0, 0 },
    { 0x1d56b1d5, 90 },
    { 0, 0 },
    { 0xcf06b1d7, 95 },
    { 0xf46b1d7, 88 },
    { 0xef86b1d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c76b1dd, 93 },
    { 0x9e76b1de, 100 },
    { 0xaa86b1df, 93 },
    { 0x3806b1dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6b1e6, 55 },
    { 0x1ae6b1e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6b1f0, 85 },
    { 0x1c96b1f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586b1f4, 33 },
    { 0, 0 },
    { 0x87f6b1f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd6b1fa, 61 },
    { 0x76a6b1fa, 91 },
    { 0, 0 },
    { 0xf3b6b1fd, 77 },
    { 0x98d6b1fd, 61 },
    { 0xb6e6b1fd, 95 },
    { 0, 0 },
    { 0x7d96b201, 53 },
    { 0x8ef6b202, 90 },
    { 0x7856b203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196b206, 98 },
    { 0x5686b206, 78 },
    { 0xc776b208, 90 },
    { 0x4076b207, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26b20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6b20f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56b212, 61 },
    { 0, 0 },
    { 0x35a6b214, 86 },
    { 0x10c6b214, 90 },
    { 0xd096b216, 77 },
    { 0, 0 },
    { 0x41e6b218, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe926b21b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc156b221, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6b227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf6b22d, 65 },
    { 0x8d86b22e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b6b232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86b236, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf936b239, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf6b23d, 93 },
    { 0x3f86b23d, 90 },
    { 0, 0 },
    { 0xb46b240, 46 },
    { 0xb5d6b240, 43 },
    { 0x7286b242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e6b245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8636b24d, 98 },
    { 0, 0 },
    { 0xa8c6b24f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966b252, 23 },
    { 0, 0 },
    { 0x6a86b254, 87 },
    { 0x326b255, 90 },
    { 0, 0 },
    { 0x6d36b257, 90 },
    { 0, 0 },
    { 0xc056b259, 23 },
    { 0xe2e6b25a, 90 },
    { 0xc5d6b25b, 61 },
    { 0, 0 },
    { 0xfe96b25d, 52 },
    { 0xb526b25e, 76 },
    { 0x33c6b25e, 77 },
    { 0x2006b260, 43 },
    { 0xf576b25e, 90 },
    { 0xedb6b262, 33 },
    { 0, 0 },
    { 0x6e26b264, 43 },
    { 0x34a6b265, 78 },
    { 0, 0 },
    { 0xf256b267, 95 },
    { 0, 0 },
    { 0x5e56b269, 33 },
    { 0, 0 },
    { 0xd476b26b, 90 },
    { 0x9b56b26b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236b273, 52 },
    { 0x74d6b274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606b277, 23 },
    { 0, 0 },
    { 0x5d26b279, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b36b283, 90 },
    { 0, 0 },
    { 0x4eb6b285, 90 },
    { 0xbc96b286, 76 },
    { 0x486b285, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9286b28b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06b28e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956b292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a16b295, 95 },
    { 0, 0 },
    { 0x1736b297, 33 },
    { 0x37d6b298, 93 },
    { 0x3656b299, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986b29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a6b29f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b6b2a3, 78 },
    { 0x9dc6b2a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6b2a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456b2ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf156b2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646b2b3, 33 },
    { 0xfeb6b2b3, 90 },
    { 0, 0 },
    { 0x1886b2b6, 98 },
    { 0, 0 },
    { 0x786b2b8, 34 },
    { 0xde06b2b9, 33 },
    { 0xcd66b2ba, 90 },
    { 0x19d6b2ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4736b2bf, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c6b2c3, 61 },
    { 0, 0 },
    { 0x386b2c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae06b2ca, 81 },
    { 0x51a6b2cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f86b2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36b2d6, 23 },
    { 0xea36b2d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f46b2db, 99 },
    { 0x5e6b2dc, 90 },
    { 0, 0 },
    { 0x5596b2de, 61 },
    { 0x8826b2de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9186b2e3, 33 },
    { 0x7fd6b2e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106b2e7, 43 },
    { 0, 0 },
    { 0x9f6b2e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b6b2ec, 77 },
    { 0x9cd6b2ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb96b2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5436b2f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae6b2fa, 78 },
    { 0x8206b2fb, 38 },
    { 0xbc26b2fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886b2ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b304, 76 },
    { 0, 0 },
    { 0xbe6b306, 40 },
    { 0, 0 },
    { 0x7b66b308, 98 },
    { 0, 0 },
    { 0x1c76b30a, 78 },
    { 0, 0 },
    { 0x4856b30c, 43 },
    { 0xa416b30d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a76b310, 90 },
    { 0, 0 },
    { 0x7486b312, 23 },
    { 0, 0 },
    { 0xe676b314, 95 },
    { 0xeab6b314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd96b318, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356b31d, 90 },
    { 0x3b26b31e, 46 },
    { 0, 0 },
    { 0x3ee6b320, 33 },
    { 0, 0 },
    { 0x5676b322, 76 },
    { 0x2b06b322, 90 },
    { 0, 0 },
    { 0xc056b325, 33 },
    { 0x9e76b326, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf426b32c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5356b330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd6b334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb066b337, 33 },
    { 0, 0 },
    { 0x49e6b339, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56b33c, 95 },
    { 0, 0 },
    { 0x8bb6b33e, 93 },
    { 0, 0 },
    { 0x87e6b340, 90 },
    { 0x7026b341, 75 },
    { 0xbe86b342, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96b34d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2796b352, 66 },
    { 0, 0 },
    { 0x1fa6b354, 98 },
    { 0xe196b355, 23 },
    { 0, 0 },
    { 0x9416b357, 91 },
    { 0, 0 },
    { 0x78c6b359, 95 },
    { 0x5ef6b359, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6b35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46b363, 76 },
    { 0xcf56b364, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f06b369, 23 },
    { 0x57a6b36a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16b36d, 43 },
    { 0x4216b36e, 61 },
    { 0x4ad6b36f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca6b380, 46 },
    { 0x88d6b381, 55 },
    { 0x7286b381, 91 },
    { 0x8546b383, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686b387, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af6b38b, 95 },
    { 0xabb6b38c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326b39a, 78 },
    { 0x7566b39a, 33 },
    { 0, 0 },
    { 0x45a6b39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c86b3a7, 64 },
    { 0, 0 },
    { 0x5236b3a9, 33 },
    { 0x4906b3aa, 33 },
    { 0x9c96b3ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc686b3ae, 76 },
    { 0xb226b3ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986b3b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6b3b6, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56b3ba, 33 },
    { 0x8f76b3bb, 53 },
    { 0xbd66b3ba, 95 },
    { 0xf976b3bb, 87 },
    { 0x6616b3be, 34 },
    { 0x2636b3bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f6b3c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe376b3c5, 99 },
    { 0, 0 },
    { 0x7b06b3c7, 88 },
    { 0x1d96b3c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46b3cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f96b3d1, 34 },
    { 0xd3d6b3d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf596b3dd, 76 },
    { 0x506b3de, 23 },
    { 0x98e6b3de, 98 },
    { 0xc7b6b3e0, 46 },
    { 0x7da6b3e1, 33 },
    { 0x9ba6b3e0, 93 },
    { 0xaec6b3e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6b3ea, 91 },
    { 0, 0 },
    { 0x37c6b3ec, 61 },
    { 0xd6f6b3ec, 75 },
    { 0, 0 },
    { 0x7946b3ef, 68 },
    { 0x2266b3ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a6b3f6, 90 },
    { 0xe436b3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c86b3fa, 61 },
    { 0x5386b3fa, 96 },
    { 0x5cb6b3fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba6b402, 23 },
    { 0xef06b402, 98 },
    { 0xc946b403, 88 },
    { 0x8926b405, 55 },
    { 0x6cf6b406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6b40a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2306b40f, 23 },
    { 0xfde6b40f, 46 },
    { 0x4de6b411, 82 },
    { 0xd706b411, 66 },
    { 0, 0 },
    { 0x13a6b414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d6b417, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc496b41c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d96b41f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426b423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d6b42b, 76 },
    { 0x3766b42c, 67 },
    { 0, 0 },
    { 0xe0f6b42e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6b431, 43 },
    { 0xbdc6b432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6b43c, 33 },
    { 0xcf96b43c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96b448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c6b44b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb646b451, 23 },
    { 0x1ed6b451, 46 },
    { 0x91d6b453, 52 },
    { 0x1f16b453, 66 },
    { 0x1926b455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c6b458, 61 },
    { 0x9dd6b459, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216b45d, 33 },
    { 0x8036b45d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e6b461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86b464, 61 },
    { 0, 0 },
    { 0xdf46b466, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa116b46b, 61 },
    { 0xbfd6b46c, 55 },
    { 0, 0 },
    { 0x8a16b46e, 33 },
    { 0, 0 },
    { 0xa0d6b470, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd816b473, 55 },
    { 0x8046b474, 43 },
    { 0xb66b475, 98 },
    { 0x7016b474, 90 },
    { 0x41e6b477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d26b47b, 23 },
    { 0xe166b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd16b481, 101 },
    { 0x74f6b481, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b485, 95 },
    { 0xc686b486, 43 },
    { 0, 0 },
    { 0x8c86b488, 90 },
    { 0, 0 },
    { 0x54c6b48a, 53 },
    { 0x71d6b48a, 43 },
    { 0xb756b48c, 61 },
    { 0x8ea6b48c, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f36b492, 76 },
    { 0, 0 },
    { 0x5996b494, 23 },
    { 0xe906b495, 43 },
    { 0x84b6b496, 33 },
    { 0x3276b497, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906b49c, 33 },
    { 0x426b49d, 33 },
    { 0x6976b49c, 76 },
    { 0x5746b49d, 90 },
    { 0x8e86b49d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc566b4a3, 23 },
    { 0x3456b4a3, 61 },
    { 0xc466b4a5, 76 },
    { 0xe0c6b4a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6b4b3, 61 },
    { 0, 0 },
    { 0x7606b4b5, 61 },
    { 0, 0 },
    { 0x7776b4b7, 95 },
    { 0xe616b4b8, 61 },
    { 0, 0 },
    { 0x5c76b4ba, 66 },
    { 0, 0 },
    { 0x82c6b4bc, 61 },
    { 0x3f56b4bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe6b4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc626b4c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a6b4cb, 91 },
    { 0x7a26b4cc, 95 },
    { 0xbb86b4cd, 85 },
    { 0x3426b4ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e6b4d3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x996b4d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6b4da, 33 },
    { 0, 0 },
    { 0x86c6b4dc, 90 },
    { 0, 0 },
    { 0x2fd6b4de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06b4e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2296b4f1, 61 },
    { 0x6466b4f2, 23 },
    { 0x9b46b4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7386b4f7, 61 },
    { 0xd636b4f7, 34 },
    { 0xeb66b4f9, 95 },
    { 0xf3d6b4fa, 33 },
    { 0, 0 },
    { 0x16c6b4fc, 33 },
    { 0, 0 },
    { 0x93b6b4fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6b501, 90 },
    { 0, 0 },
    { 0xc836b503, 33 },
    { 0, 0 },
    { 0x736b505, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b36b508, 17 },
    { 0x60b6b509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b50d, 98 },
    { 0x2666b50e, 85 },
    { 0xad76b50e, 34 },
    { 0x60e6b50e, 33 },
    { 0x1956b50f, 96 },
    { 0x66c6b512, 84 },
    { 0xb566b513, 66 },
    { 0x7d6b513, 33 },
    { 0, 0 },
    { 0xd2d6b516, 95 },
    { 0, 0 },
    { 0x7276b518, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a16b51b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c6b51f, 77 },
    { 0x2a86b520, 76 },
    { 0x5c6b51f, 61 },
    { 0x70b6b522, 87 },
    { 0x46f6b522, 86 },
    { 0, 0 },
    { 0x9046b525, 87 },
    { 0x9666b526, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb6b52b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd526b53b, 98 },
    { 0x236b53c, 61 },
    { 0xac86b53d, 33 },
    { 0x5e66b53e, 61 },
    { 0xc966b53f, 61 },
    { 0, 0 },
    { 0x33a6b541, 33 },
    { 0x21c6b542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1446b545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef6b548, 90 },
    { 0x2636b549, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca6b550, 98 },
    { 0x8c56b550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1716b558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c6b55c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd806b560, 46 },
    { 0x3ad6b561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e56b565, 78 },
    { 0x5986b566, 87 },
    { 0x2036b566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6b56c, 34 },
    { 0x20f6b56d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5226b572, 95 },
    { 0, 0 },
    { 0x7aa6b574, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a16b578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146b57c, 23 },
    { 0, 0 },
    { 0x27b6b57e, 90 },
    { 0, 0 },
    { 0x7726b580, 96 },
    { 0x4436b580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec6b588, 61 },
    { 0x9c6b589, 36 },
    { 0xa0d6b589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef6b58d, 98 },
    { 0x7ad6b58e, 23 },
    { 0x67c6b58d, 90 },
    { 0xe416b58f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b16b597, 90 },
    { 0x5d76b598, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6b59c, 91 },
    { 0xbf6b59d, 93 },
    { 0x556b59c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa76b5a1, 99 },
    { 0xd096b5a2, 90 },
    { 0, 0 },
    { 0x19f6b5a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c86b5aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6b5ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a6b5b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d6b5b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b96b5be, 90 },
    { 0xecb6b5be, 55 },
    { 0x8456b5be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6b5c3, 55 },
    { 0xe516b5c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b5c7, 90 },
    { 0x2c56b5c8, 84 },
    { 0, 0 },
    { 0x7fb6b5ca, 55 },
    { 0, 0 },
    { 0x7056b5cc, 78 },
    { 0xa426b5cd, 55 },
    { 0x39c6b5ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb46b5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3606b5d9, 36 },
    { 0x1cb6b5da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8876b5dd, 45 },
    { 0xe816b5dd, 61 },
    { 0x9ab6b5df, 90 },
    { 0x9426b5e0, 23 },
    { 0xd26b5e1, 55 },
    { 0x9466b5e1, 23 },
    { 0x9836b5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26b5e9, 23 },
    { 0x8cc6b5e9, 61 },
    { 0, 0 },
    { 0xddd6b5ec, 61 },
    { 0, 0 },
    { 0xe806b5ee, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a6b5f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3626b5fa, 90 },
    { 0, 0 },
    { 0xb4a6b5fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6b5ff, 93 },
    { 0x42e6b600, 90 },
    { 0x7ef6b601, 33 },
    { 0x1a06b602, 90 },
    { 0xbd86b603, 90 },
    { 0xa3b6b603, 92 },
    { 0x5c46b602, 77 },
    { 0x5206b606, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4476b60b, 67 },
    { 0x1ec6b60c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2766b611, 87 },
    { 0, 0 },
    { 0x1f46b613, 90 },
    { 0x55d6b614, 23 },
    { 0xf9f6b615, 33 },
    { 0xa8e6b616, 90 },
    { 0, 0 },
    { 0xf686b618, 100 },
    { 0, 0 },
    { 0x5e36b61a, 61 },
    { 0x8ae6b61a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6b620, 23 },
    { 0xdf86b620, 81 },
    { 0x3c96b622, 61 },
    { 0x93f6b621, 91 },
    { 0xc826b620, 61 },
    { 0x2d76b625, 91 },
    { 0xc0d6b622, 90 },
    { 0x9256b627, 87 },
    { 0, 0 },
    { 0x1c76b629, 52 },
    { 0, 0 },
    { 0xd4a6b62b, 90 },
    { 0xffa6b62c, 95 },
    { 0xfef6b62c, 23 },
    { 0, 0 },
    { 0x3e66b62f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e6b637, 88 },
    { 0xd356b638, 66 },
    { 0xf746b637, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe606b63d, 69 },
    { 0x7186b63d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f46b647, 55 },
    { 0, 0 },
    { 0xb226b649, 81 },
    { 0, 0 },
    { 0x3cb6b64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb936b64e, 33 },
    { 0, 0 },
    { 0x45a6b650, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa16b653, 91 },
    { 0, 0 },
    { 0xa826b655, 17 },
    { 0xdba6b656, 76 },
    { 0x5316b657, 100 },
    { 0, 0 },
    { 0xc036b659, 61 },
    { 0x9556b659, 33 },
    { 0xb5d6b65a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6b663, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b6b666, 46 },
    { 0xe156b667, 61 },
    { 0x75f6b668, 52 },
    { 0xdbc6b668, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556b66c, 38 },
    { 0x8b66b66c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4306b673, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2316b678, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f6b680, 33 },
    { 0, 0 },
    { 0xacb6b682, 23 },
    { 0xd1a6b683, 90 },
    { 0x6756b684, 23 },
    { 0x42f6b685, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe376b68b, 75 },
    { 0x70e6b68b, 77 },
    { 0x4366b68c, 100 },
    { 0xe416b68b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496b698, 87 },
    { 0, 0 },
    { 0x92a6b69a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5166b69e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca6b6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b46b6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4866b6a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf06b6ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b6b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da6b6b3, 95 },
    { 0xc1b6b6b4, 90 },
    { 0x1ee6b6b4, 87 },
    { 0x2d16b6b6, 95 },
    { 0x6896b6b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b46b6bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a36b6c0, 23 },
    { 0x2dc6b6c1, 84 },
    { 0, 0 },
    { 0xed86b6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9806b6c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1396b6cb, 77 },
    { 0x9d6b6cc, 90 },
    { 0x3e46b6cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4876b6d1, 100 },
    { 0xea6b6d2, 75 },
    { 0x9a86b6d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6b6d8, 100 },
    { 0x5e66b6d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a16b6de, 46 },
    { 0xf346b6df, 90 },
    { 0x4f96b6e0, 55 },
    { 0x3a16b6e0, 93 },
    { 0x7c76b6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c26b6e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa6b6ec, 46 },
    { 0x1e6b6ec, 76 },
    { 0x78f6b6ee, 78 },
    { 0x43f6b6ee, 61 },
    { 0xcd26b6ec, 23 },
    { 0x2526b6ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec86b6f4, 61 },
    { 0x8116b6f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6b6f8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe766b702, 61 },
    { 0xee46b703, 57 },
    { 0x2636b703, 33 },
    { 0, 0 },
    { 0x1cd6b706, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb6b70c, 85 },
    { 0, 0 },
    { 0xd546b70e, 55 },
    { 0x7676b70f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb66b713, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f6b718, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556b71c, 98 },
    { 0xfea6b71d, 95 },
    { 0x8ed6b71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e6b723, 61 },
    { 0x96b6b723, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc6b736, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d46b73c, 90 },
    { 0x84a6b73c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc046b746, 33 },
    { 0xc266b747, 98 },
    { 0, 0 },
    { 0xe6a6b749, 33 },
    { 0xc916b74a, 23 },
    { 0xc186b74b, 46 },
    { 0xf506b74c, 80 },
    { 0xa3f6b74d, 90 },
    { 0, 0 },
    { 0x79d6b74f, 87 },
    { 0x9e6b750, 33 },
    { 0xb346b751, 90 },
    { 0x8a76b751, 43 },
    { 0x8c86b752, 33 },
    { 0xc846b754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836b75c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7216b761, 43 },
    { 0, 0 },
    { 0x8976b763, 33 },
    { 0x1546b763, 91 },
    { 0xae96b763, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb6b774, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7126b777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e6b77b, 33 },
    { 0x3b66b77b, 67 },
    { 0xc76b77d, 76 },
    { 0xa056b77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f6b786, 76 },
    { 0xaa6b787, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1096b78d, 68 },
    { 0xe906b78e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6b79b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b79f, 93 },
    { 0xcf56b7a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836b7a3, 23 },
    { 0x1276b7a4, 43 },
    { 0x21e6b7a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb6b7a9, 95 },
    { 0xb0d6b7a9, 84 },
    { 0x12c6b7ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c76b7ae, 76 },
    { 0, 0 },
    { 0x4b66b7b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c16b7b3, 52 },
    { 0x216b7b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4096b7c2, 95 },
    { 0x8466b7c3, 95 },
    { 0xf506b7c4, 90 },
    { 0, 0 },
    { 0xaf46b7c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe536b7ca, 23 },
    { 0x4b86b7cb, 55 },
    { 0x9986b7cb, 90 },
    { 0x55f6b7cd, 46 },
    { 0x21f6b7cd, 33 },
    { 0x8ce6b7cf, 78 },
    { 0xe6c6b7cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6b7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe656b7d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe46b7dd, 98 },
    { 0x3696b7dd, 85 },
    { 0x6cb6b7df, 61 },
    { 0x6816b7e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8476b7e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126b7ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc6b7f4, 23 },
    { 0, 0 },
    { 0xe196b7f6, 76 },
    { 0x6006b7f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8476b7fa, 67 },
    { 0xb46b7fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb6b801, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6b809, 87 },
    { 0x7ce6b809, 35 },
    { 0, 0 },
    { 0xf586b80c, 80 },
    { 0, 0 },
    { 0xa116b80e, 95 },
    { 0, 0 },
    { 0x7666b810, 100 },
    { 0xc316b811, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6b818, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b81b, 95 },
    { 0x8a06b81c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5696b81f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866b823, 33 },
    { 0x2726b824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7716b829, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa746b82c, 98 },
    { 0, 0 },
    { 0x9776b82e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a86b833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e6b843, 33 },
    { 0, 0 },
    { 0x2346b845, 98 },
    { 0x38f6b846, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4296b84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6b852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b26b856, 100 },
    { 0x3fa6b857, 58 },
    { 0, 0 },
    { 0x9cd6b859, 98 },
    { 0, 0 },
    { 0xc156b85b, 98 },
    { 0x4766b85c, 34 },
    { 0xd816b85b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa686b864, 46 },
    { 0xea56b865, 98 },
    { 0, 0 },
    { 0x7746b867, 33 },
    { 0, 0 },
    { 0x7c66b869, 34 },
    { 0, 0 },
    { 0xe06b86b, 98 },
    { 0x34b6b86b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a86b86f, 93 },
    { 0xab96b870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb296b877, 76 },
    { 0x9b6b877, 95 },
    { 0x1436b879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2596b87c, 61 },
    { 0x6f6b87d, 33 },
    { 0xcfa6b87e, 33 },
    { 0x4786b87f, 90 },
    { 0x9ca6b87c, 83 },
    { 0, 0 },
    { 0xe4c6b882, 23 },
    { 0xccb6b883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96b886, 76 },
    { 0x83d6b887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836b88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906b896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b6b89a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e6b89d, 33 },
    { 0, 0 },
    { 0x6286b89f, 90 },
    { 0xa8e6b8a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd6b8a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea6b8a8, 90 },
    { 0xa576b8a8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb146b8ac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6b8b1, 33 },
    { 0xfbd6b8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa936b8b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96b8b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666b8bd, 46 },
    { 0, 0 },
    { 0x4096b8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a76b8c2, 100 },
    { 0x43e6b8c3, 61 },
    { 0, 0 },
    { 0x91b6b8c5, 100 },
    { 0, 0 },
    { 0xd7b6b8c7, 61 },
    { 0, 0 },
    { 0xbc26b8c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36b8d5, 83 },
    { 0x306b8d5, 43 },
    { 0xeec6b8d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb26b8dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a6b8e1, 36 },
    { 0x4e86b8e2, 84 },
    { 0x2616b8e2, 95 },
    { 0xd3e6b8e4, 55 },
    { 0xa776b8e4, 98 },
    { 0, 0 },
    { 0x2fa6b8e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b46b8ea, 46 },
    { 0xab06b8eb, 33 },
    { 0xe076b8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6b8ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86b8f2, 34 },
    { 0x8d06b8f2, 23 },
    { 0xd926b8f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d6b8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1466b8fc, 98 },
    { 0xde26b8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6b901, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6b905, 93 },
    { 0x46d6b905, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b6b90d, 61 },
    { 0x36f6b90e, 46 },
    { 0xceb6b90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a26b913, 80 },
    { 0x67c6b913, 76 },
    { 0xe6d6b915, 78 },
    { 0, 0 },
    { 0xabc6b917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6b91b, 87 },
    { 0xd766b91c, 95 },
    { 0, 0 },
    { 0x20d6b91e, 95 },
    { 0xcea6b91e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf6b922, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e6b925, 33 },
    { 0xc9f6b925, 84 },
    { 0x6b46b927, 33 },
    { 0x5496b927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc096b92c, 52 },
    { 0xe296b92d, 23 },
    { 0x5126b92d, 33 },
    { 0xad06b92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2796b932, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56b935, 33 },
    { 0, 0 },
    { 0x58d6b937, 33 },
    { 0xe896b938, 90 },
    { 0, 0 },
    { 0x82b6b93a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9936b93d, 66 },
    { 0x88c6b93d, 61 },
    { 0x66c6b93f, 93 },
    { 0x8986b93e, 34 },
    { 0x1d26b93d, 61 },
    { 0, 0 },
    { 0xae96b943, 90 },
    { 0, 0 },
    { 0xac76b945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb266b948, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e56b94c, 76 },
    { 0, 0 },
    { 0x8766b94e, 90 },
    { 0xb9d6b94f, 98 },
    { 0x98d6b950, 53 },
    { 0, 0 },
    { 0x79d6b952, 33 },
    { 0, 0 },
    { 0x5326b954, 85 },
    { 0x7ea6b955, 61 },
    { 0, 0 },
    { 0x35d6b957, 95 },
    { 0x7bc6b958, 100 },
    { 0x8796b959, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e96b95e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3456b962, 61 },
    { 0x5116b963, 95 },
    { 0x72e6b963, 52 },
    { 0x8c56b964, 93 },
    { 0xadd6b966, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816b96b, 90 },
    { 0, 0 },
    { 0x5c06b96d, 90 },
    { 0x33c6b96e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316b973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406b976, 52 },
    { 0x68d6b977, 34 },
    { 0x33f6b977, 95 },
    { 0x5186b979, 33 },
    { 0x58f6b977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc156b97f, 33 },
    { 0xc696b980, 43 },
    { 0x1e26b981, 33 },
    { 0x8246b980, 90 },
    { 0, 0 },
    { 0x6de6b984, 100 },
    { 0xc956b985, 95 },
    { 0x8e6b984, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06b98d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e6b990, 61 },
    { 0xb156b991, 61 },
    { 0, 0 },
    { 0x8d96b993, 93 },
    { 0xc386b993, 17 },
    { 0, 0 },
    { 0x1fe6b996, 76 },
    { 0x9b96b997, 66 },
    { 0x2c76b998, 55 },
    { 0xa876b997, 90 },
    { 0x8076b998, 94 },
    { 0x8f76b99b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac6b9a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b9a7, 98 },
    { 0xabb6b9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b9ab, 33 },
    { 0, 0 },
    { 0xbb56b9ad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de6b9b2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26b9b7, 87 },
    { 0x8896b9b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef6b9bc, 43 },
    { 0xc096b9bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d6b9c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d56b9c8, 34 },
    { 0xe396b9c8, 91 },
    { 0x8bd6b9ca, 23 },
    { 0x45c6b9cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6b9ce, 23 },
    { 0, 0 },
    { 0xca46b9d0, 61 },
    { 0x5ba6b9d1, 33 },
    { 0xa386b9d2, 38 },
    { 0xad56b9d3, 84 },
    { 0xa4b6b9d4, 23 },
    { 0, 0 },
    { 0x31d6b9d6, 23 },
    { 0x6e16b9d6, 76 },
    { 0xa5c6b9d8, 23 },
    { 0, 0 },
    { 0xced6b9da, 90 },
    { 0x9916b9db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c6b9de, 33 },
    { 0xb5a6b9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab6b9e2, 100 },
    { 0xa716b9e2, 55 },
    { 0xa7f6b9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296b9ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe526b9ed, 75 },
    { 0xd566b9ed, 23 },
    { 0x9b86b9ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa16b9f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7576b9f6, 98 },
    { 0, 0 },
    { 0xc996b9f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb896b9fb, 34 },
    { 0, 0 },
    { 0xf266b9fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2686ba08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606ba0b, 90 },
    { 0, 0 },
    { 0x88e6ba0d, 61 },
    { 0x2fa6ba0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6376ba12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a96ba15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5746ba19, 90 },
    { 0x4cd6ba1a, 78 },
    { 0xc836ba19, 23 },
    { 0x6316ba1c, 33 },
    { 0x3666ba19, 46 },
    { 0x1566ba1e, 61 },
    { 0xc576ba1e, 23 },
    { 0x3386ba20, 33 },
    { 0xdce6ba21, 95 },
    { 0x9726ba21, 61 },
    { 0x7966ba20, 95 },
    { 0x2056ba23, 61 },
    { 0xf956ba22, 75 },
    { 0xd706ba26, 90 },
    { 0x636ba24, 33 },
    { 0x5826ba1e, 55 },
    { 0x6b06ba29, 66 },
    { 0x6606ba2a, 90 },
    { 0xfbb6ba28, 61 },
    { 0x4566ba2c, 33 },
    { 0x6886ba2d, 90 },
    { 0xb4b6ba2e, 99 },
    { 0xc3b6ba2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a6ba35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf26ba3a, 90 },
    { 0xe726ba3b, 93 },
    { 0, 0 },
    { 0x946ba3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c6ba40, 99 },
    { 0xfef6ba41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca6ba46, 87 },
    { 0, 0 },
    { 0xcb26ba48, 93 },
    { 0, 0 },
    { 0xd216ba4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a6ba4f, 97 },
    { 0, 0 },
    { 0xd4c6ba51, 90 },
    { 0xd036ba52, 95 },
    { 0xc626ba53, 100 },
    { 0xeb66ba54, 95 },
    { 0xe4a6ba55, 61 },
    { 0x9096ba56, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc6ba5b, 93 },
    { 0x8db6ba5c, 85 },
    { 0, 0 },
    { 0xb376ba5e, 68 },
    { 0x2936ba5e, 34 },
    { 0xedf6ba60, 91 },
    { 0, 0 },
    { 0x6bf6ba62, 61 },
    { 0xd976ba63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6ba6b, 95 },
    { 0xe936ba6b, 33 },
    { 0x1a56ba6c, 76 },
    { 0xf356ba6c, 61 },
    { 0x1736ba6c, 92 },
    { 0x34d6ba6c, 43 },
    { 0xb0d6ba71, 61 },
    { 0x2b6ba6c, 55 },
    { 0, 0 },
    { 0xc56ba74, 87 },
    { 0, 0 },
    { 0x1696ba76, 5 },
    { 0, 0 },
    { 0xa766ba78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1056ba7b, 61 },
    { 0, 0 },
    { 0x48f6ba7d, 33 },
    { 0xad26ba7e, 95 },
    { 0x63b6ba7d, 93 },
    { 0xc226ba7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf256ba85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd896ba8e, 55 },
    { 0xc156ba8f, 61 },
    { 0, 0 },
    { 0x3dc6ba91, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f6ba94, 77 },
    { 0x3466ba95, 90 },
    { 0x7746ba94, 65 },
    { 0xb8f6ba97, 76 },
    { 0x7e16ba98, 90 },
    { 0x4886ba97, 66 },
    { 0, 0 },
    { 0x5d36ba9b, 98 },
    { 0, 0 },
    { 0x2c26ba9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5506baa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c6baaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc6bab0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc6bab5, 33 },
    { 0xebe6bab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736bab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x546babc, 61 },
    { 0x4d6babc, 23 },
    { 0, 0 },
    { 0xf5b6babf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8416bac3, 90 },
    { 0x17d6bac4, 61 },
    { 0xc406bac3, 95 },
    { 0xe7d6bac6, 80 },
    { 0x886bac6, 76 },
    { 0x87c6bac8, 90 },
    { 0x4e06bac9, 61 },
    { 0x2876baca, 34 },
    { 0x8c36bacb, 90 },
    { 0x9626bacc, 90 },
    { 0x57a6baca, 55 },
    { 0, 0 },
    { 0x1406bacf, 90 },
    { 0, 0 },
    { 0x7666bad1, 43 },
    { 0xc966bad2, 61 },
    { 0, 0 },
    { 0x65d6bad4, 95 },
    { 0x7e46bad5, 67 },
    { 0xebc6bad6, 98 },
    { 0xc396bad6, 61 },
    { 0x6416bad7, 90 },
    { 0x1276bad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f86bae4, 33 },
    { 0xdf46bae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6bae9, 90 },
    { 0x6f06baea, 93 },
    { 0x9506baea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36baf2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d6baf5, 93 },
    { 0xf766baf5, 43 },
    { 0, 0 },
    { 0x3296baf8, 23 },
    { 0xafd6baf9, 33 },
    { 0x976bafa, 98 },
    { 0xce96bafb, 98 },
    { 0x31c6bafb, 52 },
    { 0, 0 },
    { 0x2186bafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2526bb01, 66 },
    { 0xb746bb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a36bb0a, 95 },
    { 0x25b6bb0b, 90 },
    { 0, 0 },
    { 0xf6b6bb0d, 48 },
    { 0xb266bb0d, 90 },
    { 0xcda6bb0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa686bb12, 55 },
    { 0xf196bb13, 78 },
    { 0x9ae6bb13, 98 },
    { 0, 0 },
    { 0x6b6bb16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb466bb19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a6bb1d, 90 },
    { 0x84e6bb1d, 99 },
    { 0xdaf6bb1f, 92 },
    { 0, 0 },
    { 0x2f76bb21, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5896bb26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a36bb2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566bb36, 91 },
    { 0, 0 },
    { 0xb9e6bb38, 46 },
    { 0xcba6bb38, 95 },
    { 0xcd26bb39, 23 },
    { 0xffe6bb38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436bb41, 93 },
    { 0xfff6bb42, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed16bb45, 97 },
    { 0, 0 },
    { 0x37e6bb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6bb4a, 33 },
    { 0x2876bb4a, 61 },
    { 0x2bf6bb4c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126bb50, 55 },
    { 0x1656bb50, 33 },
    { 0xb706bb52, 98 },
    { 0xa276bb53, 43 },
    { 0xb816bb52, 33 },
    { 0x5856bb55, 23 },
    { 0x9446bb55, 34 },
    { 0x4136bb57, 98 },
    { 0xaed6bb58, 90 },
    { 0, 0 },
    { 0xd896bb5a, 56 },
    { 0x6c6bb5b, 52 },
    { 0x24c6bb5a, 98 },
    { 0x8d36bb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe356bb62, 53 },
    { 0xdeb6bb62, 33 },
    { 0, 0 },
    { 0x7c6bb65, 89 },
    { 0, 0 },
    { 0xd056bb67, 90 },
    { 0, 0 },
    { 0x1a36bb69, 61 },
    { 0xd276bb6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6bb6e, 61 },
    { 0x6176bb6e, 90 },
    { 0x8556bb70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d6bb73, 61 },
    { 0xb376bb74, 90 },
    { 0, 0 },
    { 0x9e56bb76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b66bb7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6bb7e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226bb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176bb8a, 53 },
    { 0, 0 },
    { 0xe5e6bb8c, 95 },
    { 0x8f36bb8c, 55 },
    { 0xb906bb8c, 61 },
    { 0x11c6bb8f, 61 },
    { 0x16e6bb8c, 95 },
    { 0xea16bb91, 76 },
    { 0xb996bb92, 43 },
    { 0x6186bb93, 34 },
    { 0xac6bb94, 90 },
    { 0x366bb95, 48 },
    { 0xb7b6bb95, 33 },
    { 0x9a66bb96, 95 },
    { 0x5af6bb98, 95 },
    { 0, 0 },
    { 0xb6c6bb9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6bb9d, 23 },
    { 0, 0 },
    { 0x9ba6bb9f, 98 },
    { 0, 0 },
    { 0x4b6bba1, 90 },
    { 0x2526bba2, 55 },
    { 0xebd6bba3, 95 },
    { 0x27d6bba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad6bba9, 33 },
    { 0, 0 },
    { 0xa276bbab, 84 },
    { 0, 0 },
    { 0x6246bbad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de6bbb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6bbb5, 23 },
    { 0xd8d6bbb5, 95 },
    { 0xcfb6bbb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4616bbba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d6bbc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6bbc5, 40 },
    { 0x4946bbc6, 55 },
    { 0x7d66bbc7, 52 },
    { 0x6196bbc6, 43 },
    { 0xc66bbc6, 91 },
    { 0xffb6bbc6, 61 },
    { 0xf8e6bbcb, 61 },
    { 0, 0 },
    { 0x4d36bbcd, 100 },
    { 0xd0e6bbcd, 90 },
    { 0, 0 },
    { 0x5ef6bbd0, 61 },
    { 0x9e36bbd0, 36 },
    { 0, 0 },
    { 0xa786bbd3, 98 },
    { 0x9976bbd4, 98 },
    { 0x5146bbd5, 95 },
    { 0xf0e6bbd6, 33 },
    { 0xe726bbd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906bbdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6bbde, 34 },
    { 0xd836bbdf, 34 },
    { 0xe106bbe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846bbe3, 95 },
    { 0xa386bbe4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c56bbed, 23 },
    { 0x8876bbed, 88 },
    { 0, 0 },
    { 0x196bbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436bbf6, 61 },
    { 0xef26bbf7, 90 },
    { 0, 0 },
    { 0xb626bbf9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196bbff, 17 },
    { 0xa986bc00, 87 },
    { 0, 0 },
    { 0x8556bc02, 78 },
    { 0xa426bc02, 93 },
    { 0, 0 },
    { 0x466bc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b6bc0e, 9 },
    { 0x5836bc0e, 90 },
    { 0, 0 },
    { 0x3366bc11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b46bc14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6bc1b, 89 },
    { 0x8066bc1c, 38 },
    { 0, 0 },
    { 0xf0c6bc1e, 98 },
    { 0xd416bc1f, 23 },
    { 0xd636bc20, 65 },
    { 0x2df6bc21, 78 },
    { 0x94d6bc22, 46 },
    { 0x35d6bc22, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a96bc28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe886bc32, 77 },
    { 0x2916bc33, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf996bc38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf246bc3c, 23 },
    { 0x8586bc3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6bc43, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe76bc47, 76 },
    { 0x3a36bc48, 48 },
    { 0x2876bc48, 90 },
    { 0xaa36bc4a, 33 },
    { 0x916bc4b, 33 },
    { 0x876bc4c, 99 },
    { 0xbf16bc48, 91 },
    { 0xf576bc49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6bc57, 90 },
    { 0, 0 },
    { 0x36b6bc59, 33 },
    { 0x1aa6bc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06bc5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe706bc62, 66 },
    { 0x28f6bc62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f6bc66, 46 },
    { 0, 0 },
    { 0x88f6bc68, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d6bc6c, 43 },
    { 0, 0 },
    { 0xfd26bc6e, 23 },
    { 0x84f6bc6e, 95 },
    { 0x4646bc70, 95 },
    { 0xeea6bc6e, 91 },
    { 0xf3b6bc72, 98 },
    { 0xe196bc72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6bc7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816bc81, 88 },
    { 0x8026bc82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866bc88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d6bc8b, 90 },
    { 0xfd36bc8c, 98 },
    { 0xb2a6bc8d, 45 },
    { 0xf536bc8e, 52 },
    { 0xbb66bc8c, 90 },
    { 0xd3a6bc8c, 90 },
    { 0x4fe6bc8b, 76 },
    { 0, 0 },
    { 0x2b16bc93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2626bc9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba6bca3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc86bca6, 90 },
    { 0xc9c6bca7, 43 },
    { 0x85a6bca8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3056bcae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x956bcb1, 99 },
    { 0xe886bcb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c6bcba, 43 },
    { 0x6c86bcbb, 77 },
    { 0x17b6bcbc, 23 },
    { 0, 0 },
    { 0x6726bcbe, 33 },
    { 0, 0 },
    { 0xc4e6bcc0, 90 },
    { 0, 0 },
    { 0xb926bcc2, 55 },
    { 0x2616bcc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956bcc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b6bcc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c6bccd, 97 },
    { 0xc9b6bcce, 33 },
    { 0x3066bcce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176bcd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe066bcda, 65 },
    { 0xae06bcdb, 46 },
    { 0, 0 },
    { 0x1b36bcdd, 75 },
    { 0xc456bcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f6bce4, 87 },
    { 0, 0 },
    { 0x4776bce6, 79 },
    { 0x6ae6bce6, 100 },
    { 0x5ad6bce6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56bcf2, 80 },
    { 0xad36bcf3, 23 },
    { 0x3306bcf2, 95 },
    { 0xd686bcf5, 61 },
    { 0xca06bcf6, 80 },
    { 0xf946bcf6, 34 },
    { 0x916bcf8, 90 },
    { 0, 0 },
    { 0xbb46bcfa, 95 },
    { 0xe326bcfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee86bd06, 88 },
    { 0, 0 },
    { 0xe376bd08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7846bd0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde56bd0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb986bd16, 95 },
    { 0xcaa6bd17, 78 },
    { 0xb3f6bd17, 95 },
    { 0, 0 },
    { 0xc026bd1a, 23 },
    { 0x7186bd1a, 55 },
    { 0, 0 },
    { 0xd3e6bd1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1466bd20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a6bd24, 95 },
    { 0x3746bd24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe6bd2b, 90 },
    { 0, 0 },
    { 0xa9b6bd2d, 23 },
    { 0, 0 },
    { 0x3326bd2f, 61 },
    { 0xb2d6bd30, 56 },
    { 0x62a6bd31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956bd35, 61 },
    { 0, 0 },
    { 0x5fc6bd37, 90 },
    { 0, 0 },
    { 0x7426bd39, 91 },
    { 0x1f76bd3a, 98 },
    { 0x33f6bd3b, 42 },
    { 0x7f6bd3b, 61 },
    { 0x3166bd3a, 25 },
    { 0xd2e6bd3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886bd44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf36bd48, 33 },
    { 0x87c6bd49, 66 },
    { 0x97c6bd4a, 67 },
    { 0xc66bd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a66bd4f, 95 },
    { 0xe646bd4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e6bd58, 23 },
    { 0x67d6bd59, 85 },
    { 0x3bd6bd5a, 90 },
    { 0x796bd59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6bd62, 75 },
    { 0xd86bd62, 61 },
    { 0, 0 },
    { 0x16b6bd65, 98 },
    { 0xde96bd66, 98 },
    { 0xc46bd67, 17 },
    { 0x6986bd68, 90 },
    { 0xd386bd69, 95 },
    { 0xe586bd69, 55 },
    { 0, 0 },
    { 0x2266bd6c, 90 },
    { 0xee16bd6c, 69 },
    { 0xeab6bd6e, 78 },
    { 0x9316bd6c, 36 },
    { 0x2fa6bd70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566bd78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec6bd7f, 33 },
    { 0x7c76bd80, 77 },
    { 0, 0 },
    { 0x2b46bd82, 76 },
    { 0x4896bd83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c6bd86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1636bd89, 61 },
    { 0x936bd89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe156bd8f, 33 },
    { 0xe5c6bd90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6266bd97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a6bd9e, 55 },
    { 0x16e6bd9f, 66 },
    { 0x5406bda0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266bdac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5726bdb3, 33 },
    { 0x106bdb3, 61 },
    { 0x9a86bdb5, 83 },
    { 0xc616bdb6, 23 },
    { 0, 0 },
    { 0x4d16bdb8, 90 },
    { 0, 0 },
    { 0xb466bdba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16bdbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216bdc5, 90 },
    { 0x96d6bdc6, 61 },
    { 0x5b6bdc7, 34 },
    { 0xd536bdc8, 98 },
    { 0x7c26bdc9, 55 },
    { 0x9fb6bdca, 23 },
    { 0x2556bdcb, 100 },
    { 0x57e6bdc8, 90 },
    { 0x5a96bdcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8076bdd2, 95 },
    { 0xfd26bdd3, 88 },
    { 0xf916bdd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c6bdd7, 61 },
    { 0xd5c6bdd8, 90 },
    { 0x6fd6bdd7, 66 },
    { 0, 0 },
    { 0x3c56bddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed46bdde, 76 },
    { 0xcb56bddf, 66 },
    { 0, 0 },
    { 0x3626bde1, 90 },
    { 0x5ab6bde2, 90 },
    { 0, 0 },
    { 0xe486bde4, 90 },
    { 0xc9f6bde4, 55 },
    { 0x1336bde6, 46 },
    { 0x6086bde7, 81 },
    { 0, 0 },
    { 0x1b56bde9, 98 },
    { 0xe0e6bdea, 33 },
    { 0x3896bdeb, 33 },
    { 0xefc6bdeb, 90 },
    { 0x72b6bdea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6bdf5, 61 },
    { 0x1326bdf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4006bdf9, 34 },
    { 0xecf6bdfa, 88 },
    { 0xadc6bdfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd666bdfe, 98 },
    { 0x6c66bdfe, 69 },
    { 0x7466bdfe, 91 },
    { 0xdb26be01, 100 },
    { 0, 0 },
    { 0x25c6be03, 33 },
    { 0, 0 },
    { 0x8ea6be05, 61 },
    { 0x3ba6be06, 69 },
    { 0x3be6be07, 33 },
    { 0x3906be07, 67 },
    { 0xd116be09, 46 },
    { 0xc7c6be0a, 97 },
    { 0xc156be0b, 90 },
    { 0xc7e6be0b, 95 },
    { 0xec96be09, 46 },
    { 0x2c86be09, 90 },
    { 0x9ba6be0f, 33 },
    { 0x2d56be0d, 33 },
    { 0xaf26be09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec16be14, 43 },
    { 0x7116be14, 35 },
    { 0xce26be16, 93 },
    { 0xb436be16, 23 },
    { 0x6566be14, 95 },
    { 0xfbd6be19, 33 },
    { 0xec06be18, 83 },
    { 0x5136be1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4306be1e, 23 },
    { 0, 0 },
    { 0x5256be20, 76 },
    { 0xee46be21, 33 },
    { 0xf156be21, 23 },
    { 0x33a6be21, 34 },
    { 0xfac6be24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526be2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a06be31, 95 },
    { 0, 0 },
    { 0x4b36be33, 61 },
    { 0xcab6be34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6be3a, 99 },
    { 0x5096be3a, 84 },
    { 0x1ac6be3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7366be40, 46 },
    { 0x9e16be41, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd86be44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8906be4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566be54, 23 },
    { 0x4116be55, 33 },
    { 0, 0 },
    { 0xff46be57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff36be5d, 90 },
    { 0x7d66be5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf6be61, 55 },
    { 0xd166be62, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4286be68, 61 },
    { 0, 0 },
    { 0xefa6be6a, 95 },
    { 0xe6d6be6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7066be6e, 66 },
    { 0, 0 },
    { 0x2e76be70, 95 },
    { 0x3f06be71, 61 },
    { 0xbaa6be72, 61 },
    { 0xcd76be73, 95 },
    { 0x3576be74, 55 },
    { 0x6756be75, 33 },
    { 0x2e26be75, 98 },
    { 0xa9a6be71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9346be7b, 75 },
    { 0x49d6be7c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a76be80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb76be84, 95 },
    { 0x8026be85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c6be8c, 93 },
    { 0x1686be8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf346be90, 33 },
    { 0x7b76be90, 98 },
    { 0, 0 },
    { 0x5026be93, 95 },
    { 0, 0 },
    { 0x9b86be95, 90 },
    { 0x18b6be96, 95 },
    { 0x4fd6be97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6be9a, 36 },
    { 0x8f36be9b, 84 },
    { 0, 0 },
    { 0xe596be9d, 76 },
    { 0, 0 },
    { 0xf786be9f, 61 },
    { 0x7a76bea0, 100 },
    { 0x5e46bea0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c86bea7, 95 },
    { 0x9076bea8, 55 },
    { 0xbfd6bea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c6beac, 23 },
    { 0xf8c6bead, 93 },
    { 0x85d6bead, 52 },
    { 0x4356bead, 100 },
    { 0x44f6beb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf076beb5, 66 },
    { 0x6126beb5, 76 },
    { 0xd6b6beb5, 61 },
    { 0x3206beb8, 33 },
    { 0x7766beb8, 95 },
    { 0x9246beb7, 67 },
    { 0, 0 },
    { 0x48e6bebc, 61 },
    { 0x73a6bebd, 68 },
    { 0xb8d6bebe, 98 },
    { 0xe376bebf, 33 },
    { 0xa296bec0, 66 },
    { 0xeda6bebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6bec8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a6becd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa056bed1, 99 },
    { 0x8616bed2, 90 },
    { 0x1076bed3, 33 },
    { 0x6336bed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6bed8, 77 },
    { 0x8006bed8, 43 },
    { 0x10a6bed8, 46 },
    { 0xd3e6bedb, 33 },
    { 0x9996bedc, 95 },
    { 0x7f56bedd, 91 },
    { 0x3606bedd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f76bee9, 99 },
    { 0x9826beea, 46 },
    { 0, 0 },
    { 0x28d6beec, 95 },
    { 0x4416beed, 99 },
    { 0x45e6beed, 19 },
    { 0, 0 },
    { 0x1b46bef0, 90 },
    { 0xcdc6bef1, 98 },
    { 0xc706bef2, 95 },
    { 0x6e36bef3, 78 },
    { 0xeb16bef0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546befc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f6beff, 76 },
    { 0x4dd6bf00, 90 },
    { 0x15a6bf00, 90 },
    { 0x5ac6bf02, 78 },
    { 0x68e6bf02, 76 },
    { 0x75d6bf04, 95 },
    { 0x2b66bf05, 90 },
    { 0x2e86bf03, 80 },
    { 0xd476bf07, 93 },
    { 0x9506bf08, 95 },
    { 0, 0 },
    { 0x8d36bf0a, 64 },
    { 0x7e76bf0b, 90 },
    { 0x1f36bf0b, 98 },
    { 0, 0 },
    { 0xa756bf0e, 33 },
    { 0xdf06bf0f, 88 },
    { 0xa8a6bf10, 33 },
    { 0x4746bf11, 90 },
    { 0x2996bf10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f56bf15, 77 },
    { 0, 0 },
    { 0x5916bf17, 61 },
    { 0x6e46bf18, 90 },
    { 0xe246bf19, 90 },
    { 0x1eb6bf19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae6bf1e, 36 },
    { 0, 0 },
    { 0xe466bf20, 97 },
    { 0x44f6bf20, 100 },
    { 0xb696bf21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36bf28, 33 },
    { 0xfb36bf29, 90 },
    { 0x9fe6bf2a, 98 },
    { 0x8786bf2a, 43 },
    { 0, 0 },
    { 0x8a46bf2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe826bf32, 98 },
    { 0xbba6bf33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x976bf36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb6bf3f, 87 },
    { 0xe066bf40, 33 },
    { 0x26b6bf41, 95 },
    { 0xeb76bf40, 67 },
    { 0xb656bf43, 90 },
    { 0xf4a6bf40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6bf4b, 23 },
    { 0, 0 },
    { 0x9b66bf4d, 23 },
    { 0x19b6bf4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6bf56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86bf5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6bf5e, 55 },
    { 0, 0 },
    { 0x8006bf60, 99 },
    { 0, 0 },
    { 0x826bf62, 43 },
    { 0xecd6bf63, 34 },
    { 0, 0 },
    { 0x2796bf65, 76 },
    { 0xf736bf65, 95 },
    { 0x6416bf67, 23 },
    { 0x4396bf66, 33 },
    { 0xe9e6bf69, 96 },
    { 0, 0 },
    { 0x9076bf6b, 61 },
    { 0, 0 },
    { 0x6d96bf6d, 33 },
    { 0x8586bf6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286bf73, 90 },
    { 0, 0 },
    { 0xed56bf75, 67 },
    { 0xc276bf76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16bf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116bf7d, 95 },
    { 0, 0 },
    { 0xa246bf7f, 69 },
    { 0x24d6bf80, 34 },
    { 0x9e86bf81, 76 },
    { 0xece6bf81, 55 },
    { 0xbec6bf81, 98 },
    { 0x5716bf81, 33 },
    { 0xbe56bf81, 75 },
    { 0x1b26bf83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6616bf89, 65 },
    { 0x1d86bf8a, 100 },
    { 0x7c86bf8b, 67 },
    { 0xed76bf8a, 75 },
    { 0, 0 },
    { 0x14a6bf8e, 90 },
    { 0x57a6bf8f, 90 },
    { 0x28e6bf8e, 43 },
    { 0xb7d6bf91, 90 },
    { 0xa526bf90, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b56bf96, 55 },
    { 0x6656bf97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf726bf9b, 55 },
    { 0x8bd6bf9b, 87 },
    { 0, 0 },
    { 0x58b6bf9e, 61 },
    { 0xcf26bf9f, 34 },
    { 0x1f86bf9e, 81 },
    { 0x2546bf9f, 34 },
    { 0x4f06bf9e, 58 },
    { 0x6816bfa3, 43 },
    { 0xd306bfa3, 100 },
    { 0xa886bfa3, 90 },
    { 0xc876bfa6, 69 },
    { 0xfa36bfa7, 90 },
    { 0xdb6bfa8, 75 },
    { 0x2bc6bfa9, 60 },
    { 0x2a56bfaa, 98 },
    { 0x2396bfab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x796bfae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a16bfbc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc6bfbf, 83 },
    { 0x8e26bfbf, 90 },
    { 0, 0 },
    { 0x5b66bfc2, 33 },
    { 0, 0 },
    { 0x4a36bfc4, 93 },
    { 0x2ca6bfc5, 93 },
    { 0x5c56bfc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c6bfcf, 90 },
    { 0x17e6bfd0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab6bfd3, 55 },
    { 0, 0 },
    { 0xed66bfd5, 33 },
    { 0x5396bfd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6bfd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6bfdd, 55 },
    { 0xd726bfdd, 95 },
    { 0xd486bfdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236bfe4, 76 },
    { 0xadd6bfe5, 90 },
    { 0, 0 },
    { 0xe436bfe7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a6bfeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7516bff1, 61 },
    { 0x8976bff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd6bff7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda6bffc, 87 },
    { 0x6a6bffc, 55 },
    { 0, 0 },
    { 0xfe16bfff, 33 },
    { 0xf2e6bfff, 98 },
    { 0, 0 },
    { 0xb5d6c002, 23 },
    { 0xbec6c003, 34 },
    { 0x4256c003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc936c008, 19 },
    { 0, 0 },
    { 0x4406c00a, 95 },
    { 0xc1c6c00a, 99 },
    { 0x21e6c00c, 33 },
    { 0xd686c00c, 66 },
    { 0x74e6c00d, 55 },
    { 0x4766c00f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d6c016, 77 },
    { 0, 0 },
    { 0x39d6c018, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf406c020, 34 },
    { 0xb996c021, 101 },
    { 0x4236c022, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8486c026, 98 },
    { 0x1356c027, 43 },
    { 0xa2d6c027, 90 },
    { 0xbed6c029, 98 },
    { 0x7c6c02a, 61 },
    { 0x1566c02a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516c030, 87 },
    { 0x6586c030, 95 },
    { 0, 0 },
    { 0x4176c033, 90 },
    { 0x8a16c033, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x506c037, 23 },
    { 0, 0 },
    { 0x5206c039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3996c03c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6c044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb496c047, 67 },
    { 0x3136c048, 33 },
    { 0x6e56c047, 61 },
    { 0, 0 },
    { 0x84d6c04b, 55 },
    { 0xfa6c04c, 76 },
    { 0x4ae6c04b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d6c053, 61 },
    { 0x1a96c054, 95 },
    { 0xa236c055, 93 },
    { 0x8d96c056, 97 },
    { 0x4366c057, 101 },
    { 0, 0 },
    { 0x4fc6c059, 55 },
    { 0x5ff6c05a, 23 },
    { 0, 0 },
    { 0xa236c05c, 95 },
    { 0xd196c05d, 61 },
    { 0xce66c05d, 23 },
    { 0x8046c05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac6c062, 33 },
    { 0x56f6c063, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e86c06a, 61 },
    { 0, 0 },
    { 0x4346c06c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c16c071, 93 },
    { 0xeb36c072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56c077, 93 },
    { 0xc6d6c078, 95 },
    { 0, 0 },
    { 0x5ee6c07a, 90 },
    { 0x47c6c07a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266c07e, 61 },
    { 0x8ad6c07e, 93 },
    { 0x9266c080, 77 },
    { 0, 0 },
    { 0x4016c082, 95 },
    { 0xfb76c083, 46 },
    { 0xc386c083, 43 },
    { 0, 0 },
    { 0xef66c086, 98 },
    { 0, 0 },
    { 0xf356c088, 34 },
    { 0x276c089, 55 },
    { 0x8c16c08a, 68 },
    { 0xa066c08b, 61 },
    { 0xcf6c08c, 90 },
    { 0xc4e6c08c, 84 },
    { 0xbac6c08b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c6c091, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586c094, 46 },
    { 0, 0 },
    { 0xec66c096, 85 },
    { 0, 0 },
    { 0x5296c098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346c09b, 77 },
    { 0xcc6c09b, 90 },
    { 0, 0 },
    { 0x3a6c09e, 68 },
    { 0x6a26c09e, 34 },
    { 0xbfb6c0a0, 33 },
    { 0xf526c0a1, 90 },
    { 0x5876c09e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626c0a7, 33 },
    { 0, 0 },
    { 0x3106c0a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556c0af, 91 },
    { 0x1b06c0af, 95 },
    { 0, 0 },
    { 0xb0d6c0b2, 98 },
    { 0, 0 },
    { 0xbfc6c0b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6076c0bf, 61 },
    { 0xbe86c0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6c0c4, 74 },
    { 0, 0 },
    { 0x5b96c0c6, 46 },
    { 0x7886c0c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd966c0ce, 61 },
    { 0, 0 },
    { 0x256c0d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4686c0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6c0d9, 95 },
    { 0xf0a6c0da, 98 },
    { 0, 0 },
    { 0x9d06c0dc, 52 },
    { 0xfe66c0dd, 76 },
    { 0x19b6c0de, 36 },
    { 0x2486c0df, 88 },
    { 0x486c0dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8566c0e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6c0ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d56c0f1, 23 },
    { 0x2546c0f2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e6c0f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a6c0fb, 46 },
    { 0xb5b6c0fb, 76 },
    { 0xebb6c0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4776c102, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc736c105, 60 },
    { 0xadf6c106, 66 },
    { 0, 0 },
    { 0xf756c108, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee06c10b, 33 },
    { 0x76f6c10c, 61 },
    { 0xed66c10d, 34 },
    { 0xbe46c10e, 46 },
    { 0x9c06c10f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf6c112, 75 },
    { 0x8a16c113, 76 },
    { 0xe326c114, 90 },
    { 0xe4d6c115, 76 },
    { 0xc456c116, 34 },
    { 0x40f6c114, 23 },
    { 0x3396c118, 85 },
    { 0x78c6c119, 90 },
    { 0x5a6c11a, 55 },
    { 0xf086c11b, 85 },
    { 0x9976c113, 61 },
    { 0x4b6c11d, 34 },
    { 0x8296c11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a76c128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176c12c, 23 },
    { 0, 0 },
    { 0x9856c12e, 68 },
    { 0x60c6c12e, 78 },
    { 0x46e6c12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066c134, 33 },
    { 0xbcb6c135, 55 },
    { 0x3006c135, 33 },
    { 0x1ee6c134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa676c144, 95 },
    { 0, 0 },
    { 0xe816c146, 33 },
    { 0xaaf6c146, 95 },
    { 0, 0 },
    { 0x3ca6c149, 66 },
    { 0x1d06c14a, 87 },
    { 0x2236c149, 33 },
    { 0x6a46c14c, 67 },
    { 0x9da6c14d, 95 },
    { 0xe1c6c14b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc836c154, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c6c157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726c15a, 23 },
    { 0x1716c15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5336c15e, 85 },
    { 0x3486c15f, 61 },
    { 0xc6c6c160, 36 },
    { 0xa5d6c15f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956c167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a6c16c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b06c170, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6c174, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6c177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316c17a, 33 },
    { 0, 0 },
    { 0xb006c17c, 90 },
    { 0, 0 },
    { 0xe776c17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236c183, 36 },
    { 0xc3f6c184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x596c187, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec6c18b, 77 },
    { 0x1db6c18b, 90 },
    { 0x8de6c18d, 91 },
    { 0xeaf6c18b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af6c192, 97 },
    { 0x9b66c193, 80 },
    { 0x63a6c194, 78 },
    { 0x3946c195, 98 },
    { 0xf6c6c196, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b6c19d, 98 },
    { 0x26e6c19d, 91 },
    { 0x3f16c19f, 90 },
    { 0xac46c1a0, 78 },
    { 0xa026c19d, 90 },
    { 0x5ad6c1a2, 33 },
    { 0xb5c6c1a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56c1a9, 33 },
    { 0, 0 },
    { 0xd016c1ab, 67 },
    { 0xebb6c1ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c6c1b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd506c1b4, 23 },
    { 0, 0 },
    { 0xcca6c1b6, 61 },
    { 0x9426c1b6, 23 },
    { 0xe1d6c1b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb566c1bb, 77 },
    { 0xe086c1bc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb746c1c1, 78 },
    { 0x7066c1c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6c1c7, 95 },
    { 0x4e06c1c7, 61 },
    { 0, 0 },
    { 0x1e76c1ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f66c1cf, 46 },
    { 0x1e6c1cf, 76 },
    { 0, 0 },
    { 0x6266c1d2, 23 },
    { 0x8db6c1d3, 91 },
    { 0x8586c1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336c1d7, 99 },
    { 0x1926c1d8, 46 },
    { 0xcda6c1d8, 98 },
    { 0, 0 },
    { 0x9cb6c1db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056c1e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb76c1e7, 46 },
    { 0xaa36c1e8, 61 },
    { 0x9976c1e9, 90 },
    { 0x1936c1ea, 98 },
    { 0, 0 },
    { 0xdbd6c1ec, 36 },
    { 0x78a6c1ed, 33 },
    { 0x9386c1ee, 46 },
    { 0x7456c1ef, 98 },
    { 0xdd76c1ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd766c1f7, 55 },
    { 0, 0 },
    { 0x19c6c1f9, 61 },
    { 0x2636c1fa, 23 },
    { 0xdd66c1fb, 46 },
    { 0x82d6c1fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c96c200, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b86c206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16c20c, 34 },
    { 0, 0 },
    { 0xbb26c20e, 91 },
    { 0xa1c6c20f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46c21a, 84 },
    { 0, 0 },
    { 0xa376c21c, 23 },
    { 0xc7f6c21c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b6c220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c6c223, 43 },
    { 0x7556c224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb456c229, 23 },
    { 0x50e6c22a, 61 },
    { 0xc706c229, 88 },
    { 0xbe26c22c, 91 },
    { 0x9fa6c22b, 85 },
    { 0x3476c229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d6c231, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1846c234, 34 },
    { 0xa3f6c235, 61 },
    { 0xfb6c236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc176c239, 90 },
    { 0x6716c23a, 23 },
    { 0x4816c23a, 89 },
    { 0xefa6c23c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc876c242, 90 },
    { 0, 0 },
    { 0x2e86c244, 66 },
    { 0xb466c244, 33 },
    { 0xe916c246, 46 },
    { 0x8c6c246, 98 },
    { 0x58e6c248, 52 },
    { 0xb756c245, 95 },
    { 0xc326c249, 75 },
    { 0, 0 },
    { 0xc3a6c24c, 76 },
    { 0xd666c24c, 95 },
    { 0x1606c24e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7286c255, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c6c25c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e6c260, 81 },
    { 0xcda6c261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e6c264, 90 },
    { 0xe4b6c264, 34 },
    { 0x8676c266, 61 },
    { 0x6836c266, 61 },
    { 0x6656c264, 90 },
    { 0, 0 },
    { 0x8556c26a, 98 },
    { 0x7736c26b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac6c26f, 95 },
    { 0x6fa6c270, 99 },
    { 0x2a46c271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa786c275, 46 },
    { 0xdc36c276, 61 },
    { 0xd6d6c277, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196c27f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c56c285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4056c28b, 51 },
    { 0x5e16c28c, 68 },
    { 0xd6e6c28d, 90 },
    { 0, 0 },
    { 0x2e06c28f, 76 },
    { 0xed86c28f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6c293, 75 },
    { 0x4926c294, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xff86c297, 46 },
    { 0x53c6c298, 23 },
    { 0xe286c299, 90 },
    { 0, 0 },
    { 0x2d86c29b, 66 },
    { 0x6286c29c, 23 },
    { 0xecf6c29b, 43 },
    { 0xe276c29c, 23 },
    { 0, 0 },
    { 0x3a36c2a0, 61 },
    { 0x166c2a1, 55 },
    { 0, 0 },
    { 0x9a16c2a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6c2a8, 33 },
    { 0xeeb6c2a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a6c2af, 61 },
    { 0xb6f6c2b0, 76 },
    { 0x7f86c2b0, 53 },
    { 0x38a6c2b2, 33 },
    { 0x1ed6c2b0, 90 },
    { 0, 0 },
    { 0x526c2b5, 33 },
    { 0xd276c2b5, 43 },
    { 0, 0 },
    { 0x72c6c2b8, 46 },
    { 0xb0a6c2b9, 90 },
    { 0xab76c2b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef6c2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b46c2c3, 61 },
    { 0x9ee6c2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd6c2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9946c2cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f26c2cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab6c2d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd16c2d5, 55 },
    { 0xf376c2d5, 66 },
    { 0xd0c6c2d7, 23 },
    { 0, 0 },
    { 0x2746c2d9, 98 },
    { 0xa2d6c2da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd986c2df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8786c2ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596c2ef, 34 },
    { 0xd256c2f0, 95 },
    { 0x95e6c2f1, 78 },
    { 0x4b16c2f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b86c2f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a6c2fc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc76c300, 83 },
    { 0, 0 },
    { 0x7c46c302, 98 },
    { 0, 0 },
    { 0xdd86c304, 34 },
    { 0xfba6c304, 43 },
    { 0x9e46c306, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f6c309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056c30f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a6c313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496c318, 98 },
    { 0, 0 },
    { 0xff86c31a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6c31d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc656c322, 90 },
    { 0xb196c323, 78 },
    { 0x5796c324, 78 },
    { 0xc236c325, 43 },
    { 0x9b86c324, 33 },
    { 0x3a66c325, 61 },
    { 0, 0 },
    { 0xc9e6c329, 66 },
    { 0xb856c329, 61 },
    { 0, 0 },
    { 0xd606c32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d6c331, 98 },
    { 0xe936c331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6c33a, 98 },
    { 0x7876c33b, 78 },
    { 0x5236c33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7326c340, 69 },
    { 0xa9d6c341, 85 },
    { 0x2cd6c342, 90 },
    { 0, 0 },
    { 0x8b26c344, 61 },
    { 0x1e96c345, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86c34b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6c34e, 93 },
    { 0x73e6c34e, 23 },
    { 0, 0 },
    { 0x2b26c351, 98 },
    { 0xb566c351, 90 },
    { 0x4046c352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe366c35c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6526c362, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6c365, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a6c369, 95 },
    { 0x9cd6c36a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1336c36d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c6c370, 90 },
    { 0x7b66c371, 66 },
    { 0xa946c372, 34 },
    { 0xf706c373, 61 },
    { 0x7ec6c374, 23 },
    { 0, 0 },
    { 0x9306c376, 33 },
    { 0x4ed6c377, 76 },
    { 0x2b76c376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f6c37b, 98 },
    { 0x5976c37c, 65 },
    { 0x3ab6c37c, 61 },
    { 0, 0 },
    { 0xc9a6c37f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6c383, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316c388, 66 },
    { 0xd2b6c389, 100 },
    { 0, 0 },
    { 0xc996c38b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca6c38e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706c394, 90 },
    { 0, 0 },
    { 0xebb6c396, 81 },
    { 0x3606c397, 61 },
    { 0xf2e6c397, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f6c39c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06c3a0, 23 },
    { 0, 0 },
    { 0x6536c3a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe736c3a6, 34 },
    { 0xba56c3a7, 88 },
    { 0xd446c3a7, 53 },
    { 0, 0 },
    { 0xb76c3aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816c3ad, 87 },
    { 0x2f36c3ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d86c3b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b66c3b9, 34 },
    { 0x7c56c3ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6c3bf, 98 },
    { 0, 0 },
    { 0x6606c3c1, 23 },
    { 0, 0 },
    { 0xf436c3c3, 75 },
    { 0x1ba6c3c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6706c3cb, 99 },
    { 0xac6c3cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3256c3d0, 99 },
    { 0, 0 },
    { 0xde46c3d2, 33 },
    { 0x9186c3d3, 76 },
    { 0x7826c3d4, 90 },
    { 0xae96c3d3, 34 },
    { 0x3c46c3d6, 99 },
    { 0x7916c3d7, 85 },
    { 0x196c3d8, 85 },
    { 0x5a36c3d8, 23 },
    { 0xdec6c3d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd856c3dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6c3e0, 98 },
    { 0x18f6c3e1, 98 },
    { 0x26d6c3e2, 39 },
    { 0x8b56c3e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6c3ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd836c3ef, 61 },
    { 0x2f86c3ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906c3f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3256c3fc, 87 },
    { 0xd866c3fd, 84 },
    { 0, 0 },
    { 0xf116c3ff, 92 },
    { 0xd2a6c400, 33 },
    { 0x2566c400, 61 },
    { 0xcd76c3ff, 98 },
    { 0x7c36c403, 95 },
    { 0x556c402, 98 },
    { 0xe616c405, 77 },
    { 0xb3b6c405, 43 },
    { 0xb616c405, 33 },
    { 0xb4b6c408, 61 },
    { 0xa2b6c409, 98 },
    { 0xb0a6c40a, 33 },
    { 0xe7b6c40a, 23 },
    { 0x3266c40c, 95 },
    { 0x9946c40c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c26c410, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e36c416, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c41d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb6c420, 87 },
    { 0x5656c421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7806c427, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb216c42b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a6c431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f16c435, 23 },
    { 0x4286c436, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e96c43b, 43 },
    { 0x6036c43c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5516c448, 93 },
    { 0xd336c448, 23 },
    { 0xcec6c44a, 90 },
    { 0xe4b6c44b, 98 },
    { 0, 0 },
    { 0x4546c44d, 23 },
    { 0xbe46c44d, 91 },
    { 0x8136c44d, 75 },
    { 0x98c6c44f, 34 },
    { 0x2df6c44d, 95 },
    { 0x8f16c452, 90 },
    { 0xa196c453, 98 },
    { 0x2a76c454, 33 },
    { 0x7026c454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6656c45c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b6c461, 66 },
    { 0x516c461, 90 },
    { 0xa6e6c463, 87 },
    { 0x1d26c464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3276c46f, 23 },
    { 0, 0 },
    { 0x52d6c471, 98 },
    { 0x4b06c471, 23 },
    { 0x4dd6c473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6c477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a06c47f, 90 },
    { 0xba6c480, 85 },
    { 0xc5c6c481, 40 },
    { 0x1ed6c482, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9816c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c06c490, 77 },
    { 0, 0 },
    { 0x97b6c492, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb566c498, 23 },
    { 0, 0 },
    { 0xff36c49a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c46c4a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6c4a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3466c4a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e6c4ab, 84 },
    { 0, 0 },
    { 0xfe16c4ad, 76 },
    { 0xb566c4ae, 52 },
    { 0xf706c4ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe986c4b2, 99 },
    { 0x4d16c4b3, 33 },
    { 0, 0 },
    { 0x4966c4b5, 61 },
    { 0x8656c4b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce6c4c5, 34 },
    { 0x7936c4c6, 90 },
    { 0x4566c4c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f6c4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6c4d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456c4d4, 87 },
    { 0, 0 },
    { 0x7196c4d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e6c4dc, 33 },
    { 0, 0 },
    { 0x71d6c4de, 52 },
    { 0xa5d6c4df, 40 },
    { 0xc906c4df, 61 },
    { 0x3d76c4df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6c4e4, 33 },
    { 0x8a46c4e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46c4e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c6c4ec, 33 },
    { 0xdce6c4ed, 90 },
    { 0x9166c4ec, 46 },
    { 0, 0 },
    { 0x2f16c4f0, 23 },
    { 0x6676c4f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea26c4f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6c501, 53 },
    { 0x8446c502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06c508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496c511, 90 },
    { 0x2786c511, 33 },
    { 0x2836c512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76c516, 33 },
    { 0x22b6c516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d6c51c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5016c51f, 88 },
    { 0, 0 },
    { 0xde16c521, 66 },
    { 0, 0 },
    { 0x37c6c523, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf706c526, 66 },
    { 0, 0 },
    { 0xca26c528, 23 },
    { 0x3ca6c529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846c52c, 46 },
    { 0, 0 },
    { 0x1596c52e, 61 },
    { 0x1dd6c52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb16c538, 98 },
    { 0, 0 },
    { 0x54b6c53a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456c53d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd6c540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276c547, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e66c54a, 34 },
    { 0, 0 },
    { 0x9516c54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f6c550, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526c554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e6c559, 33 },
    { 0x3cf6c55a, 90 },
    { 0xb3d6c55a, 81 },
    { 0x206c55c, 66 },
    { 0xe406c55c, 23 },
    { 0x5536c55a, 95 },
    { 0xb956c55f, 90 },
    { 0, 0 },
    { 0xce46c561, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca6c564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe546c567, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6c56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f6c571, 76 },
    { 0x7376c572, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e46c57b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2356c57e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6c581, 33 },
    { 0, 0 },
    { 0xd2d6c583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26c587, 52 },
    { 0x906c588, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b6c591, 76 },
    { 0xd126c591, 76 },
    { 0x41f6c593, 33 },
    { 0xb646c594, 95 },
    { 0xfa66c595, 87 },
    { 0x4c36c596, 81 },
    { 0xf426c596, 55 },
    { 0x6616c598, 33 },
    { 0x6b26c596, 98 },
    { 0, 0 },
    { 0x80f6c59b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc916c5aa, 84 },
    { 0x3a76c5ab, 87 },
    { 0, 0 },
    { 0x61f6c5ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d86c5b1, 95 },
    { 0, 0 },
    { 0xae6c5b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed6c5b8, 78 },
    { 0xe346c5b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf6c5bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd6c5c5, 46 },
    { 0xd8d6c5c5, 23 },
    { 0x2836c5c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b6c5ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9576c5ce, 61 },
    { 0, 0 },
    { 0x1206c5d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd6c5d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa816c5d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe016c5de, 75 },
    { 0xa726c5df, 77 },
    { 0x7d56c5e0, 23 },
    { 0, 0 },
    { 0xdff6c5e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26c5e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06c5ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86c5f3, 98 },
    { 0xa666c5f4, 100 },
    { 0x5806c5f3, 55 },
    { 0x9526c5f4, 67 },
    { 0x84b6c5f7, 66 },
    { 0, 0 },
    { 0x7ab6c5f9, 98 },
    { 0xcab6c5f9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436c5fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb56c601, 90 },
    { 0x5796c601, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb806c605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e26c60a, 66 },
    { 0x3716c60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96c60f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9556c615, 17 },
    { 0, 0 },
    { 0x9b76c617, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9216c61d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f36c622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d6c627, 98 },
    { 0, 0 },
    { 0x2906c629, 90 },
    { 0xc5f6c62a, 96 },
    { 0x3c16c62b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa966c631, 23 },
    { 0x8ab6c632, 98 },
    { 0, 0 },
    { 0x3236c634, 90 },
    { 0, 0 },
    { 0xcbd6c636, 90 },
    { 0, 0 },
    { 0x64c6c638, 91 },
    { 0xf7f6c639, 43 },
    { 0xc5b6c63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe6c63e, 33 },
    { 0x3066c63f, 98 },
    { 0, 0 },
    { 0xd9f6c641, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f46c646, 98 },
    { 0xbc36c647, 98 },
    { 0x3d86c648, 84 },
    { 0xae46c649, 95 },
    { 0x6336c64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7586c64e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c6c652, 87 },
    { 0x6086c652, 61 },
    { 0x29a6c654, 65 },
    { 0x5836c652, 98 },
    { 0x6c46c656, 98 },
    { 0xdbd6c652, 98 },
    { 0x98e6c654, 90 },
    { 0xb5f6c654, 17 },
    { 0x60e6c65a, 61 },
    { 0x1206c65b, 61 },
    { 0x2d36c65c, 78 },
    { 0xe846c659, 61 },
    { 0x3a86c65e, 90 },
    { 0x3696c65f, 78 },
    { 0x26e6c660, 90 },
    { 0x1cf6c65f, 61 },
    { 0, 0 },
    { 0x4206c663, 95 },
    { 0x7ce6c664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6c668, 98 },
    { 0xfb66c669, 66 },
    { 0x67f6c66a, 25 },
    { 0xbc06c669, 65 },
    { 0x8956c66b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb6c670, 84 },
    { 0x6556c671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8936c674, 95 },
    { 0x7656c675, 33 },
    { 0x1286c676, 98 },
    { 0x40b6c677, 34 },
    { 0xa266c677, 84 },
    { 0, 0 },
    { 0x6cc6c67a, 23 },
    { 0, 0 },
    { 0xccd6c67c, 61 },
    { 0xe316c67d, 55 },
    { 0x67b6c67d, 34 },
    { 0x36b6c67c, 99 },
    { 0, 0 },
    { 0x2c26c681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8156c684, 39 },
    { 0, 0 },
    { 0xbb16c686, 23 },
    { 0, 0 },
    { 0xf9b6c688, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6c68d, 23 },
    { 0xa8f6c68e, 93 },
    { 0x1336c68f, 90 },
    { 0x456c68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556c693, 90 },
    { 0x5e36c694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336c698, 90 },
    { 0x9a6c699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b46c6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6526c6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb306c6a8, 77 },
    { 0x4926c6a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2116c6b1, 65 },
    { 0xe316c6b2, 46 },
    { 0xbac6c6b3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe636c6b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd186c6bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c46c6c3, 33 },
    { 0xbd56c6c4, 61 },
    { 0, 0 },
    { 0xe616c6c6, 34 },
    { 0x12a6c6c7, 98 },
    { 0, 0 },
    { 0x8476c6c9, 43 },
    { 0xe236c6ca, 76 },
    { 0x8c96c6cb, 33 },
    { 0x9b46c6ca, 76 },
    { 0x12e6c6cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df6c6d0, 98 },
    { 0, 0 },
    { 0xe236c6d2, 81 },
    { 0xf856c6d3, 46 },
    { 0x846c6d2, 100 },
    { 0x6fc6c6d5, 90 },
    { 0x6916c6d6, 30 },
    { 0x59a6c6d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f66c6da, 17 },
    { 0, 0 },
    { 0xc006c6dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046c6e4, 61 },
    { 0x3e16c6e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c6c6e8, 33 },
    { 0x24a6c6e8, 34 },
    { 0xec06c6ea, 98 },
    { 0x2df6c6eb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f56c6f3, 53 },
    { 0xc9d6c6f4, 33 },
    { 0xe626c6f5, 76 },
    { 0x8336c6f5, 23 },
    { 0, 0 },
    { 0x3bf6c6f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf56c6fb, 95 },
    { 0xaf16c6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16c702, 34 },
    { 0xa7a6c703, 61 },
    { 0x35c6c704, 84 },
    { 0, 0 },
    { 0xa416c706, 76 },
    { 0x6996c707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6c711, 23 },
    { 0x43b6c712, 90 },
    { 0x1da6c713, 33 },
    { 0, 0 },
    { 0x56e6c715, 93 },
    { 0, 0 },
    { 0x8336c717, 23 },
    { 0xb4e6c718, 55 },
    { 0xa2b6c719, 55 },
    { 0x1476c71a, 17 },
    { 0x8ff6c71a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb416c71e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8086c721, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726c725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea46c728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee6c72b, 95 },
    { 0xb8d6c72c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6c731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b6c736, 75 },
    { 0x6796c736, 95 },
    { 0xdc66c738, 23 },
    { 0, 0 },
    { 0xeb56c73a, 95 },
    { 0, 0 },
    { 0xe776c73c, 61 },
    { 0, 0 },
    { 0xf716c73e, 90 },
    { 0xeb56c73f, 98 },
    { 0, 0 },
    { 0xb7b6c741, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396c745, 23 },
    { 0, 0 },
    { 0x47d6c747, 98 },
    { 0, 0 },
    { 0x99d6c749, 23 },
    { 0, 0 },
    { 0x36b6c74b, 23 },
    { 0x5996c74c, 87 },
    { 0, 0 },
    { 0x8cf6c74e, 23 },
    { 0x57f6c74f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66c753, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566c762, 33 },
    { 0x8d66c763, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746c766, 61 },
    { 0xfc6c767, 52 },
    { 0, 0 },
    { 0x7186c769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6c770, 95 },
    { 0xe5d6c771, 78 },
    { 0x6486c772, 95 },
    { 0, 0 },
    { 0xe6c774, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a6c777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e6c77e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6c786, 76 },
    { 0x8dc6c787, 95 },
    { 0xb526c788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666c791, 23 },
    { 0, 0 },
    { 0xce06c793, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46c798, 100 },
    { 0, 0 },
    { 0x9546c79a, 23 },
    { 0x7be6c79b, 61 },
    { 0xa046c79b, 34 },
    { 0xdbf6c79c, 61 },
    { 0x4cd6c79e, 78 },
    { 0x69b6c79f, 76 },
    { 0xf8d6c79b, 90 },
    { 0, 0 },
    { 0xe906c7a2, 98 },
    { 0xd336c7a3, 33 },
    { 0x41b6c7a4, 55 },
    { 0x1f36c7a5, 99 },
    { 0x38f6c7a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf06c7a9, 98 },
    { 0xccc6c7a9, 95 },
    { 0xb556c7aa, 95 },
    { 0x3866c7ac, 75 },
    { 0x8926c7ad, 23 },
    { 0xa986c7ae, 95 },
    { 0x3a6c7af, 85 },
    { 0x2ac6c7b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9506c7b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa46c7b7, 23 },
    { 0x23d6c7b7, 33 },
    { 0, 0 },
    { 0x7526c7ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f6c7bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16c7c2, 61 },
    { 0xee86c7c2, 97 },
    { 0, 0 },
    { 0x6376c7c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1976c7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d36c7cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b66c7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf6c7d1, 43 },
    { 0xf9f6c7d1, 55 },
    { 0x9896c7d3, 90 },
    { 0x4976c7d4, 51 },
    { 0x3796c7d5, 95 },
    { 0x5c56c7d3, 38 },
    { 0, 0 },
    { 0xc9e6c7d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6c7df, 84 },
    { 0x1496c7e0, 77 },
    { 0x24c6c7e0, 34 },
    { 0x5826c7df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba06c7e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc076c7ed, 90 },
    { 0x2966c7ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6c7f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb716c7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976c802, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b16c806, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a6c809, 77 },
    { 0, 0 },
    { 0x3286c80b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56c810, 67 },
    { 0x1296c811, 33 },
    { 0, 0 },
    { 0xeed6c813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736c81b, 36 },
    { 0x6bb6c81c, 43 },
    { 0xe986c81d, 95 },
    { 0xa5a6c81e, 87 },
    { 0x5486c81d, 46 },
    { 0xd166c81d, 95 },
    { 0x89a6c821, 80 },
    { 0x92e6c821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3346c826, 61 },
    { 0x40b6c827, 90 },
    { 0xc426c828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6c82f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c6c833, 55 },
    { 0, 0 },
    { 0x6d36c835, 23 },
    { 0xe306c836, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6c83c, 43 },
    { 0x2266c83c, 33 },
    { 0xc516c83c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e6c846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6c849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c84e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1616c851, 90 },
    { 0x5736c852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6c857, 55 },
    { 0, 0 },
    { 0xdd66c859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9756c85e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646c862, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9216c86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c26c871, 67 },
    { 0x7896c871, 76 },
    { 0, 0 },
    { 0xd506c874, 95 },
    { 0, 0 },
    { 0xd96c876, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b6c879, 98 },
    { 0, 0 },
    { 0xaec6c87b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb56c886, 36 },
    { 0, 0 },
    { 0x5a66c888, 90 },
    { 0, 0 },
    { 0x316c88a, 66 },
    { 0, 0 },
    { 0x67d6c88c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7076c88f, 78 },
    { 0, 0 },
    { 0x2e56c891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3806c894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6c898, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde86c8a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb6c8ae, 52 },
    { 0xd676c8af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b6c8b2, 98 },
    { 0x3a06c8b3, 61 },
    { 0xbd56c8b4, 85 },
    { 0, 0 },
    { 0x6a96c8b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6c8ba, 33 },
    { 0x7566c8bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c6c8c0, 23 },
    { 0xc4b6c8c1, 60 },
    { 0x9146c8c2, 100 },
    { 0x2e36c8c0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776c8c6, 98 },
    { 0xf9c6c8c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6c8d0, 56 },
    { 0x9816c8d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3736c8d4, 98 },
    { 0, 0 },
    { 0x81c6c8d6, 33 },
    { 0xa76c8d7, 95 },
    { 0xca26c8d6, 67 },
    { 0, 0 },
    { 0xb106c8da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16c8dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6356c8e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a26c8ea, 81 },
    { 0, 0 },
    { 0x9326c8ec, 76 },
    { 0xcac6c8ed, 90 },
    { 0x9ad6c8ee, 90 },
    { 0x1d06c8ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9086c8f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226c8f7, 98 },
    { 0x926c8f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6c902, 61 },
    { 0, 0 },
    { 0x3d76c904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5736c908, 78 },
    { 0x64d6c909, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706c911, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a66c914, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe836c919, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56c921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc6c924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d6c927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e6c92c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6866c936, 33 },
    { 0x9526c937, 95 },
    { 0xb8b6c938, 40 },
    { 0xac86c937, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456c93d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1006c940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa396c943, 33 },
    { 0, 0 },
    { 0xf886c945, 43 },
    { 0x9a6c946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd116c94a, 33 },
    { 0xdbb6c94b, 67 },
    { 0x3af6c94b, 90 },
    { 0xbb76c94a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb846c958, 61 },
    { 0xaff6c959, 33 },
    { 0, 0 },
    { 0x7276c95b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c56c960, 38 },
    { 0, 0 },
    { 0x50f6c962, 28 },
    { 0x5e66c962, 76 },
    { 0x21b6c964, 90 },
    { 0xf0a6c963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be6c968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e6c96b, 33 },
    { 0xb066c96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6c96f, 76 },
    { 0, 0 },
    { 0x6ed6c971, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146c975, 76 },
    { 0xbee6c975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb26c979, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e6c97c, 91 },
    { 0x7c66c97d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad6c981, 95 },
    { 0, 0 },
    { 0x7726c983, 33 },
    { 0xc906c984, 81 },
    { 0x4d96c984, 52 },
    { 0x6216c986, 55 },
    { 0xe4e6c987, 33 },
    { 0xd646c987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc856c98c, 93 },
    { 0x6986c98d, 87 },
    { 0x7806c98e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976c994, 23 },
    { 0xf556c995, 41 },
    { 0x5bf6c994, 33 },
    { 0x9136c996, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c26c99f, 99 },
    { 0xa236c99f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6536c9ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c66c9b0, 9 },
    { 0, 0 },
    { 0xa356c9b2, 43 },
    { 0xeb86c9b2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f16c9b7, 90 },
    { 0x8dd6c9b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb76c9bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa806c9be, 90 },
    { 0xd576c9bf, 46 },
    { 0x4e36c9bf, 33 },
    { 0xa0f6c9bf, 75 },
    { 0xeb16c9bf, 99 },
    { 0xab26c9c3, 98 },
    { 0, 0 },
    { 0x8a86c9c5, 76 },
    { 0x8d46c9c6, 75 },
    { 0x6136c9c7, 78 },
    { 0x8e96c9c7, 90 },
    { 0x6366c9c9, 46 },
    { 0x4406c9c9, 81 },
    { 0xd206c9c9, 90 },
    { 0x1e46c9c8, 73 },
    { 0x86f6c9cb, 95 },
    { 0xcbd6c9ce, 34 },
    { 0x7bf6c9ce, 95 },
    { 0xa826c9cb, 84 },
    { 0x39e6c9d1, 23 },
    { 0x6316c9d2, 91 },
    { 0x8296c9d3, 61 },
    { 0xccb6c9d4, 61 },
    { 0x9456c9cc, 90 },
    { 0xe496c9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd466c9e0, 95 },
    { 0x91b6c9e1, 90 },
    { 0, 0 },
    { 0x6616c9e3, 61 },
    { 0x5de6c9e4, 90 },
    { 0x4f96c9e4, 68 },
    { 0x7636c9e6, 46 },
    { 0x68b6c9e6, 43 },
    { 0x6ab6c9e5, 61 },
    { 0xd2e6c9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee26c9ec, 33 },
    { 0x3506c9ed, 61 },
    { 0, 0 },
    { 0x8e36c9ef, 69 },
    { 0x866c9f0, 76 },
    { 0xb536c9f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa6c9f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326c9fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc146ca06, 93 },
    { 0, 0 },
    { 0xc86ca08, 76 },
    { 0, 0 },
    { 0xe5d6ca0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca6ca0d, 33 },
    { 0, 0 },
    { 0x7276ca0f, 90 },
    { 0xe9c6ca10, 84 },
    { 0, 0 },
    { 0x426ca12, 23 },
    { 0x6c6ca13, 33 },
    { 0, 0 },
    { 0xc956ca15, 85 },
    { 0x1d76ca16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4046ca21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6ca26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76ca2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ca33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06ca37, 23 },
    { 0, 0 },
    { 0xf6b6ca39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2616ca3c, 33 },
    { 0x4326ca3d, 66 },
    { 0x2606ca3c, 61 },
    { 0x2916ca3f, 55 },
    { 0xce6ca40, 90 },
    { 0xdc86ca41, 34 },
    { 0, 0 },
    { 0xddf6ca43, 90 },
    { 0x17e6ca44, 61 },
    { 0x8e96ca43, 23 },
    { 0, 0 },
    { 0x26e6ca47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26ca4b, 34 },
    { 0, 0 },
    { 0xb326ca4d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36ca54, 95 },
    { 0xe1d6ca55, 75 },
    { 0xd36ca55, 68 },
    { 0x2146ca57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6ca5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6ca5e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f16ca61, 77 },
    { 0xc856ca62, 93 },
    { 0, 0 },
    { 0x46a6ca64, 61 },
    { 0x8846ca64, 61 },
    { 0x15c6ca66, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6ca6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb306ca72, 46 },
    { 0x8016ca72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956ca78, 61 },
    { 0x77f6ca79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6366ca7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2836ca82, 61 },
    { 0x9de6ca82, 95 },
    { 0xafb6ca83, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6ca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6ca90, 23 },
    { 0xb496ca91, 33 },
    { 0x1156ca92, 95 },
    { 0x1036ca91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f6ca96, 90 },
    { 0x9656ca97, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6ca9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7116ca9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb636caa1, 46 },
    { 0x9ad6caa2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd086caa6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746caaa, 17 },
    { 0xa116caab, 55 },
    { 0xf5b6caac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6caaf, 23 },
    { 0x3b96cab0, 95 },
    { 0, 0 },
    { 0x9456cab2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f6cab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa876caba, 33 },
    { 0, 0 },
    { 0xcc6cabc, 90 },
    { 0xfe86cabc, 77 },
    { 0x80a6cabc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe676cac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6cac6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a76cacb, 77 },
    { 0x7496cacc, 93 },
    { 0, 0 },
    { 0x79f6cace, 34 },
    { 0x4276cacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe6cada, 23 },
    { 0x9f26cadb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a6cae3, 98 },
    { 0xc576cae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4586cae7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e6caea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86caee, 98 },
    { 0, 0 },
    { 0xbed6caf0, 100 },
    { 0, 0 },
    { 0x2876caf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2786cafa, 95 },
    { 0xfc66cafb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x846cafe, 77 },
    { 0, 0 },
    { 0x15c6cb00, 77 },
    { 0x49c6cb01, 55 },
    { 0x396cb01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de6cb06, 23 },
    { 0x7ce6cb07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4316cb0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86cb11, 87 },
    { 0x4396cb11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd626cb15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6cb18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b6cb1b, 34 },
    { 0, 0 },
    { 0xd896cb1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6716cb32, 33 },
    { 0x5396cb33, 33 },
    { 0x2236cb33, 23 },
    { 0x226cb35, 98 },
    { 0x4056cb36, 91 },
    { 0xab16cb37, 43 },
    { 0xd2b6cb38, 33 },
    { 0xe176cb39, 61 },
    { 0xccb6cb34, 67 },
    { 0xaee6cb38, 96 },
    { 0xa426cb33, 38 },
    { 0, 0 },
    { 0xf416cb3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1936cb41, 93 },
    { 0x6916cb42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe6cb46, 35 },
    { 0x4d66cb47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e6cb4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36cb4e, 33 },
    { 0xb546cb4f, 33 },
    { 0xa356cb4e, 95 },
    { 0x3276cb50, 95 },
    { 0xcb16cb52, 34 },
    { 0, 0 },
    { 0x5576cb54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf46cb57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e76cb5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e26cb5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa846cb64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b6cb67, 90 },
    { 0xb8e6cb67, 55 },
    { 0x6b66cb69, 75 },
    { 0x5146cb68, 23 },
    { 0x2426cb69, 98 },
    { 0x5066cb67, 76 },
    { 0x4716cb6d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade6cb72, 46 },
    { 0xdea6cb73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e6cb7a, 76 },
    { 0x116cb7a, 90 },
    { 0x10e6cb7c, 83 },
    { 0xcb76cb7d, 55 },
    { 0, 0 },
    { 0xbba6cb7f, 75 },
    { 0xf906cb80, 33 },
    { 0x1966cb81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8286cb84, 81 },
    { 0, 0 },
    { 0xbeb6cb86, 46 },
    { 0, 0 },
    { 0xca46cb88, 8 },
    { 0xe6d6cb89, 76 },
    { 0xc396cb8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d36cb8f, 61 },
    { 0xf1a6cb90, 23 },
    { 0, 0 },
    { 0xf5e6cb92, 33 },
    { 0, 0 },
    { 0xd256cb94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de6cb98, 99 },
    { 0x5e96cb99, 61 },
    { 0, 0 },
    { 0xc566cb9b, 95 },
    { 0, 0 },
    { 0x1f96cb9d, 23 },
    { 0xb4d6cb9d, 33 },
    { 0xe546cb9d, 34 },
    { 0x5db6cb9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c6cba7, 96 },
    { 0, 0 },
    { 0xd296cba9, 67 },
    { 0xd036cbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb706cbad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af6cbb1, 85 },
    { 0x9a56cbb2, 84 },
    { 0xb546cbb2, 77 },
    { 0xa6c6cbb1, 84 },
    { 0x2a06cbb5, 23 },
    { 0, 0 },
    { 0x8f66cbb7, 90 },
    { 0, 0 },
    { 0xee56cbb9, 76 },
    { 0x2bd6cbb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2036cbbe, 33 },
    { 0xf76cbbe, 87 },
    { 0xf216cbbf, 100 },
    { 0, 0 },
    { 0x7056cbc2, 34 },
    { 0xe046cbc3, 23 },
    { 0, 0 },
    { 0xc3a6cbc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab6cbc9, 55 },
    { 0x1966cbca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266cbd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316cbd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d6cbda, 55 },
    { 0xd256cbdb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f6cbdf, 52 },
    { 0x5ce6cbdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4026cbe8, 91 },
    { 0x8676cbe8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea6cbec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb796cbef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6cbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae6cbf5, 93 },
    { 0x9066cbf5, 99 },
    { 0xd6b6cbf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a66cbfb, 34 },
    { 0, 0 },
    { 0xbb46cbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2676cc00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e46cc04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566cc07, 98 },
    { 0, 0 },
    { 0x91b6cc09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa226cc0f, 78 },
    { 0xe8b6cc10, 61 },
    { 0x4d66cc11, 34 },
    { 0, 0 },
    { 0x52a6cc13, 87 },
    { 0, 0 },
    { 0x316cc15, 46 },
    { 0, 0 },
    { 0xc736cc17, 61 },
    { 0x4a46cc17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e46cc1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86cc28, 67 },
    { 0, 0 },
    { 0xf9a6cc2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f6cc2d, 23 },
    { 0, 0 },
    { 0x8fe6cc2f, 76 },
    { 0x90a6cc2f, 43 },
    { 0x13c6cc30, 68 },
    { 0xa936cc31, 98 },
    { 0x4976cc33, 95 },
    { 0, 0 },
    { 0x1196cc35, 55 },
    { 0x5a6cc36, 90 },
    { 0x2116cc37, 23 },
    { 0x43e6cc36, 33 },
    { 0x8616cc35, 61 },
    { 0x99c6cc37, 46 },
    { 0, 0 },
    { 0xdc6cc3c, 40 },
    { 0x6026cc3d, 98 },
    { 0x7856cc3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376cc43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c6cc46, 98 },
    { 0xa816cc47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776cc4a, 95 },
    { 0, 0 },
    { 0x1e66cc4c, 33 },
    { 0, 0 },
    { 0xa2f6cc4e, 33 },
    { 0x4536cc4f, 40 },
    { 0xbe26cc4f, 76 },
    { 0x4006cc51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6cc54, 43 },
    { 0x3a56cc54, 23 },
    { 0, 0 },
    { 0xe436cc57, 90 },
    { 0xb856cc57, 33 },
    { 0, 0 },
    { 0x9a36cc5a, 75 },
    { 0x8f46cc5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc386cc61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb6cc6d, 52 },
    { 0x7b76cc6e, 52 },
    { 0x91d6cc6f, 76 },
    { 0x2e56cc70, 91 },
    { 0x43e6cc6e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd96cc74, 77 },
    { 0x34b6cc74, 90 },
    { 0xe746cc76, 85 },
    { 0xeb46cc74, 84 },
    { 0, 0 },
    { 0x7cb6cc79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf246cc7c, 66 },
    { 0x58f6cc7d, 90 },
    { 0, 0 },
    { 0x6516cc7f, 55 },
    { 0x8e76cc80, 33 },
    { 0x83a6cc80, 61 },
    { 0x8286cc82, 98 },
    { 0, 0 },
    { 0x2596cc84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6cc89, 88 },
    { 0, 0 },
    { 0x5796cc8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16cc96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16cc99, 98 },
    { 0, 0 },
    { 0x5e56cc9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6cc9f, 76 },
    { 0, 0 },
    { 0xbdc6cca1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac16cca5, 55 },
    { 0xcaa6cca6, 90 },
    { 0xdaa6cca6, 98 },
    { 0x8326cca8, 90 },
    { 0xc716cca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946ccb1, 90 },
    { 0, 0 },
    { 0x8dc6ccb3, 95 },
    { 0, 0 },
    { 0xb706ccb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7776ccc9, 90 },
    { 0x67d6ccca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6cccd, 33 },
    { 0, 0 },
    { 0xfb86cccf, 99 },
    { 0x8766ccd0, 61 },
    { 0xea96ccd1, 98 },
    { 0xa126ccd0, 33 },
    { 0xe7f6ccd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6ccd8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8946ccdb, 33 },
    { 0, 0 },
    { 0x5cd6ccdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6cce0, 98 },
    { 0xc616cce0, 46 },
    { 0x33e6cce2, 80 },
    { 0, 0 },
    { 0x4956cce4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6cce7, 90 },
    { 0x58a6cce8, 23 },
    { 0x37b6cce9, 44 },
    { 0x74a6cce7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e6ccf3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6ccf6, 34 },
    { 0x1956ccf6, 90 },
    { 0x25d6ccf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae16ccfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6cd02, 95 },
    { 0, 0 },
    { 0x7206cd04, 61 },
    { 0x8486cd04, 90 },
    { 0xdf96cd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea6cd09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826cd0e, 95 },
    { 0, 0 },
    { 0x7ef6cd10, 95 },
    { 0x3b26cd11, 46 },
    { 0, 0 },
    { 0xa596cd13, 90 },
    { 0, 0 },
    { 0xb8a6cd15, 91 },
    { 0xc676cd15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cd1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86cd22, 55 },
    { 0xcb46cd22, 23 },
    { 0x4c76cd24, 90 },
    { 0x3236cd25, 34 },
    { 0, 0 },
    { 0x9626cd27, 77 },
    { 0x5e26cd28, 100 },
    { 0, 0 },
    { 0x9a66cd2a, 81 },
    { 0x2fb6cd2b, 46 },
    { 0, 0 },
    { 0x1d6cd2d, 67 },
    { 0x79f6cd2d, 33 },
    { 0, 0 },
    { 0x6366cd30, 75 },
    { 0xd196cd31, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956cd35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6cd3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686cd42, 19 },
    { 0, 0 },
    { 0xd8f6cd44, 33 },
    { 0xa2d6cd44, 37 },
    { 0xbf86cd46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d96cd4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7256cd53, 23 },
    { 0, 0 },
    { 0x7096cd55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf6cd59, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226cd66, 98 },
    { 0x9d66cd66, 23 },
    { 0x3846cd67, 95 },
    { 0xf486cd67, 61 },
    { 0x1f56cd6a, 78 },
    { 0x57d6cd6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a6cd72, 65 },
    { 0x7876cd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1186cd76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca6cd80, 76 },
    { 0xb026cd80, 81 },
    { 0x3086cd82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba6cd86, 78 },
    { 0x4076cd87, 23 },
    { 0x2cd6cd86, 95 },
    { 0x4a6cd89, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc16cd8c, 56 },
    { 0x7c56cd8c, 90 },
    { 0xa76cd8c, 99 },
    { 0x5916cd8e, 90 },
    { 0, 0 },
    { 0xdf76cd91, 23 },
    { 0x1ce6cd92, 38 },
    { 0, 0 },
    { 0x1c86cd94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a66cd9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f6cda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96cda4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956cda7, 99 },
    { 0x9db6cda8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e36cdab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d6cdaf, 61 },
    { 0x6d6cdb0, 23 },
    { 0, 0 },
    { 0x8e96cdb2, 33 },
    { 0, 0 },
    { 0x5606cdb4, 61 },
    { 0x99b6cdb5, 61 },
    { 0x9286cdb5, 33 },
    { 0, 0 },
    { 0x5246cdb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6cdbb, 23 },
    { 0x8376cdbb, 33 },
    { 0x29f6cdbd, 61 },
    { 0x7c86cdbe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d86cdc1, 33 },
    { 0x48e6cdc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956cdc6, 23 },
    { 0xe666cdc6, 61 },
    { 0x16a6cdc6, 23 },
    { 0x9a96cdc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef6cdcf, 90 },
    { 0xd2a6cdd0, 33 },
    { 0xa936cdcf, 101 },
    { 0x42b6cdd2, 66 },
    { 0x3266cdcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846cdd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56cdda, 95 },
    { 0x37c6cdda, 61 },
    { 0xda36cddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff6cde4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6cdeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e6cdf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026cdfb, 98 },
    { 0, 0 },
    { 0xf576cdfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4486ce01, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff6ce0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f6ce0d, 76 },
    { 0x1d56ce0e, 23 },
    { 0x2486ce0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe6ce13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226ce18, 98 },
    { 0xb4d6ce19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc6ce1d, 46 },
    { 0, 0 },
    { 0xc406ce1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d6ce22, 90 },
    { 0, 0 },
    { 0xf396ce24, 46 },
    { 0, 0 },
    { 0x2a16ce26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d06ce2d, 17 },
    { 0x45a6ce2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec6ce31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ce35, 93 },
    { 0, 0 },
    { 0x8cc6ce37, 43 },
    { 0, 0 },
    { 0x6056ce39, 55 },
    { 0, 0 },
    { 0x2c36ce3b, 81 },
    { 0x9ae6ce3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6ce42, 23 },
    { 0, 0 },
    { 0x88d6ce44, 90 },
    { 0, 0 },
    { 0x7176ce46, 43 },
    { 0xc826ce46, 90 },
    { 0x1ff6ce48, 90 },
    { 0x9c6ce49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e6ce4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7596ce50, 17 },
    { 0xa396ce51, 76 },
    { 0x5846ce51, 90 },
    { 0xcb6ce51, 98 },
    { 0, 0 },
    { 0xb576ce55, 90 },
    { 0xd016ce56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d6ce59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ce5c, 34 },
    { 0, 0 },
    { 0xae06ce5e, 94 },
    { 0x6756ce5f, 83 },
    { 0x1b56ce5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8236ce64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf776ce67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea6ce6a, 17 },
    { 0x4356ce6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6646ce6e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516ce72, 90 },
    { 0, 0 },
    { 0x25b6ce74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe766ce79, 98 },
    { 0xeac6ce7a, 85 },
    { 0x88a6ce79, 33 },
    { 0xcdb6ce7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ce84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f6ce87, 76 },
    { 0, 0 },
    { 0x5e46ce89, 61 },
    { 0xcfb6ce89, 93 },
    { 0xf766ce8a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56ce8e, 93 },
    { 0x8126ce8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b66ce92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc726ce95, 99 },
    { 0x5186ce96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d6ce9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56ce9d, 76 },
    { 0x736ce9e, 69 },
    { 0x946ce9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06cea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586cea5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d46cea8, 75 },
    { 0x63d6cea8, 46 },
    { 0x8bc6cea9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8916ceae, 66 },
    { 0x8b26ceaf, 17 },
    { 0, 0 },
    { 0x296ceb1, 33 },
    { 0x5626ceb1, 90 },
    { 0x8bf6ceb3, 46 },
    { 0x7136ceb4, 52 },
    { 0x5896ceb2, 33 },
    { 0x9eb6ceb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb136ceba, 23 },
    { 0x9e26cebb, 33 },
    { 0, 0 },
    { 0x87f6cebd, 66 },
    { 0x7996cebe, 33 },
    { 0xe406cebf, 67 },
    { 0x8316cebf, 61 },
    { 0xb206cec1, 90 },
    { 0xf4f6cec1, 33 },
    { 0x1856cec3, 95 },
    { 0x38c6cec3, 90 },
    { 0x7d6cec2, 33 },
    { 0xfd16cec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6cec9, 84 },
    { 0x79b6ceca, 33 },
    { 0x1946cecb, 61 },
    { 0xcec6cec9, 90 },
    { 0x10d6cecd, 67 },
    { 0, 0 },
    { 0xdf46cecf, 95 },
    { 0xe586ced0, 93 },
    { 0, 0 },
    { 0xba46ced2, 23 },
    { 0x5276ced3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d6cedc, 33 },
    { 0xd046cedd, 95 },
    { 0x54b6cede, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f6cee2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa156cee8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd6ceed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc966cef1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe936cef5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6cefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab6cf00, 98 },
    { 0x61b6cf01, 43 },
    { 0, 0 },
    { 0x1bc6cf03, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b6cf0c, 61 },
    { 0xe726cf0d, 84 },
    { 0x8c36cf0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c6cf11, 46 },
    { 0, 0 },
    { 0x4cb6cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8586cf18, 61 },
    { 0x5506cf19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076cf21, 55 },
    { 0x64c6cf21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6cf27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6116cf2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa246cf2f, 76 },
    { 0x8996cf30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc366cf33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf056cf37, 90 },
    { 0xb696cf38, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a96cf3c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b6cf41, 87 },
    { 0, 0 },
    { 0x88b6cf43, 76 },
    { 0x67b6cf43, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f66cf49, 98 },
    { 0xcc56cf4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326cf4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf376cf50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f6cf53, 78 },
    { 0x9e86cf54, 61 },
    { 0x6206cf53, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa166cf5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486cf61, 33 },
    { 0xecd6cf62, 23 },
    { 0x606cf63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb6cf66, 23 },
    { 0x4b06cf67, 75 },
    { 0xf6c6cf66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef6cf6d, 90 },
    { 0xac86cf6e, 76 },
    { 0x25e6cf6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c6cf72, 61 },
    { 0, 0 },
    { 0xdeb6cf74, 77 },
    { 0xb8c6cf75, 61 },
    { 0xe0e6cf74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7226cf7a, 33 },
    { 0x2056cf7b, 90 },
    { 0x8466cf7b, 98 },
    { 0xcec6cf7d, 23 },
    { 0xf276cf7e, 99 },
    { 0xc976cf7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7166cf83, 23 },
    { 0x1606cf83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe6cf88, 61 },
    { 0xeb56cf89, 90 },
    { 0xda76cf89, 90 },
    { 0x766cf89, 99 },
    { 0, 0 },
    { 0x52b6cf8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b6cf90, 90 },
    { 0x20a6cf91, 78 },
    { 0xb836cf90, 98 },
    { 0, 0 },
    { 0xd4d6cf94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6cf9a, 33 },
    { 0x31d6cf9a, 99 },
    { 0x8ad6cf9a, 23 },
    { 0x7926cf9d, 61 },
    { 0x1c6cf9e, 52 },
    { 0x8c16cf9d, 23 },
    { 0xd9f6cfa0, 95 },
    { 0xd8a6cf9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a6cfa9, 23 },
    { 0x1336cfaa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56cfb0, 34 },
    { 0x49d6cfb0, 77 },
    { 0x95d6cfb0, 93 },
    { 0xfe76cfb2, 98 },
    { 0x79e6cfb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6cfb8, 98 },
    { 0x9d56cfb9, 40 },
    { 0xb676cfb8, 3 },
    { 0xc336cfbb, 90 },
    { 0x7d16cfbc, 33 },
    { 0xc996cfbd, 94 },
    { 0xa2b6cfbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf86cfc1, 75 },
    { 0, 0 },
    { 0x59b6cfc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab26cfc6, 95 },
    { 0xdec6cfc7, 67 },
    { 0, 0 },
    { 0xe9e6cfc9, 36 },
    { 0x4de6cfc9, 61 },
    { 0x1516cfcb, 90 },
    { 0x8636cfcc, 67 },
    { 0xfd36cfcd, 33 },
    { 0xe9d6cfce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556cfd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1576cfd7, 52 },
    { 0xa706cfd8, 95 },
    { 0, 0 },
    { 0x7656cfda, 98 },
    { 0, 0 },
    { 0x2d96cfdc, 52 },
    { 0xf9f6cfdc, 95 },
    { 0, 0 },
    { 0xdd96cfdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd46cfe3, 93 },
    { 0xc836cfe4, 90 },
    { 0xd396cfe4, 95 },
    { 0xfee6cfe6, 23 },
    { 0x2076cfe6, 55 },
    { 0, 0 },
    { 0xc1a6cfe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd6cfed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4956cff0, 95 },
    { 0x3d76cff1, 90 },
    { 0x7666cff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6cff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3446d002, 55 },
    { 0xe406d003, 70 },
    { 0x9786d004, 55 },
    { 0xe4d6d005, 23 },
    { 0, 0 },
    { 0xc4e6d007, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca6d00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6d010, 98 },
    { 0, 0 },
    { 0xbdf6d012, 99 },
    { 0xe3d6d012, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a6d01b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4936d020, 23 },
    { 0xa346d021, 90 },
    { 0, 0 },
    { 0x5026d023, 90 },
    { 0, 0 },
    { 0x6146d025, 95 },
    { 0, 0 },
    { 0xf666d027, 66 },
    { 0xbb46d027, 100 },
    { 0xf3c6d029, 98 },
    { 0xebb6d02a, 98 },
    { 0x4626d029, 33 },
    { 0x8576d02c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426d030, 81 },
    { 0, 0 },
    { 0x8396d032, 55 },
    { 0xad36d033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966d03a, 61 },
    { 0, 0 },
    { 0x7706d03c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff46d03f, 76 },
    { 0x76b6d040, 93 },
    { 0xb796d041, 99 },
    { 0x1da6d042, 48 },
    { 0xeba6d043, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6d046, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6796d049, 84 },
    { 0, 0 },
    { 0xe046d04b, 98 },
    { 0x1ec6d04c, 91 },
    { 0xaf86d04d, 98 },
    { 0x8d36d04e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1096d055, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76d059, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16d05d, 76 },
    { 0x526d05d, 46 },
    { 0, 0 },
    { 0x35e6d060, 43 },
    { 0x9396d061, 98 },
    { 0xd336d062, 90 },
    { 0xbfe6d063, 56 },
    { 0x6746d064, 46 },
    { 0x2336d063, 76 },
    { 0xcbc6d066, 76 },
    { 0x4916d067, 43 },
    { 0x6cd6d067, 95 },
    { 0x7766d066, 99 },
    { 0x3086d060, 90 },
    { 0x47a6d067, 55 },
    { 0x1516d06c, 67 },
    { 0xeb96d062, 33 },
    { 0x8016d06e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6d072, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6d075, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7026d078, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5746d07f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9356d085, 90 },
    { 0, 0 },
    { 0x6896d087, 33 },
    { 0xc5f6d087, 43 },
    { 0x6636d088, 55 },
    { 0xd7f6d08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c6d093, 91 },
    { 0xd376d093, 98 },
    { 0x6946d095, 84 },
    { 0x52c6d093, 23 },
    { 0xbad6d095, 38 },
    { 0x676d097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf6d09e, 33 },
    { 0, 0 },
    { 0x9df6d0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7646d0a5, 33 },
    { 0x9a46d0a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d0ac, 91 },
    { 0, 0 },
    { 0xddf6d0ae, 95 },
    { 0xedd6d0ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9906d0b2, 95 },
    { 0x3696d0b3, 81 },
    { 0x4c76d0b4, 23 },
    { 0xc7a6d0b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff76d0bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca96d0c1, 81 },
    { 0x52e6d0c2, 61 },
    { 0x7ee6d0c2, 92 },
    { 0x7896d0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6d0c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d0ca, 55 },
    { 0xbae6d0ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc06d0ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd36d0d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa6d0d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d0d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6d0db, 93 },
    { 0x2336d0db, 85 },
    { 0x3f26d0dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be6d0e1, 90 },
    { 0, 0 },
    { 0x8926d0e3, 46 },
    { 0, 0 },
    { 0xf2e6d0e5, 67 },
    { 0xa956d0e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa6d0ed, 23 },
    { 0, 0 },
    { 0x63a6d0ef, 33 },
    { 0, 0 },
    { 0x3ef6d0f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646d0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6d0fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe756d0fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6746d101, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e6d106, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde66d10c, 66 },
    { 0, 0 },
    { 0x53b6d10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7606d111, 95 },
    { 0x7686d111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d6d11a, 98 },
    { 0x8e46d11b, 23 },
    { 0x1ec6d11c, 84 },
    { 0x1706d11d, 66 },
    { 0xdcc6d11d, 43 },
    { 0x9e86d11e, 34 },
    { 0x726d11f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f6d123, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3506d128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8606d12b, 61 },
    { 0xcb96d12c, 33 },
    { 0x60c6d12b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6d131, 98 },
    { 0x236d132, 99 },
    { 0, 0 },
    { 0x1d06d134, 99 },
    { 0, 0 },
    { 0xf8f6d136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc6d145, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf46d14a, 46 },
    { 0x7216d14b, 85 },
    { 0xe666d14c, 98 },
    { 0, 0 },
    { 0x206d14e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6d154, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846d15a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1656d165, 61 },
    { 0, 0 },
    { 0x90a6d167, 90 },
    { 0x63b6d168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc76d16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe6d16f, 33 },
    { 0, 0 },
    { 0x7556d171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56d17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d180, 100 },
    { 0xbb36d180, 98 },
    { 0xde06d180, 95 },
    { 0xf896d183, 34 },
    { 0xf2e6d184, 43 },
    { 0xda96d185, 90 },
    { 0x7fc6d186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c56d18b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186d18e, 55 },
    { 0, 0 },
    { 0xb746d190, 43 },
    { 0, 0 },
    { 0x3ec6d192, 61 },
    { 0x72e6d193, 87 },
    { 0x49f6d194, 95 },
    { 0xc166d195, 43 },
    { 0x586d196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b6d19b, 95 },
    { 0xd0a6d19c, 23 },
    { 0x61e6d19c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026d1a1, 95 },
    { 0x8ac6d1a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06d1a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf36d1b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd26d1bc, 98 },
    { 0x6de6d1bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c6d1c5, 33 },
    { 0, 0 },
    { 0x6416d1c7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316d1cc, 55 },
    { 0, 0 },
    { 0xd266d1ce, 90 },
    { 0, 0 },
    { 0xfcd6d1d0, 95 },
    { 0x27a6d1d1, 66 },
    { 0, 0 },
    { 0xd16d1d3, 61 },
    { 0x6ee6d1d3, 23 },
    { 0, 0 },
    { 0x5c66d1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616d1db, 98 },
    { 0, 0 },
    { 0x5b86d1dd, 34 },
    { 0, 0 },
    { 0x5a56d1df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5646d1e7, 43 },
    { 0xa76d1e8, 33 },
    { 0, 0 },
    { 0xf1f6d1ea, 99 },
    { 0xc796d1eb, 98 },
    { 0xe9c6d1eb, 78 },
    { 0xef26d1ea, 100 },
    { 0x6bc6d1ee, 55 },
    { 0, 0 },
    { 0xa5c6d1f0, 91 },
    { 0, 0 },
    { 0x93f6d1f2, 40 },
    { 0xbb86d1f3, 33 },
    { 0x53f6d1f4, 66 },
    { 0xbfa6d1f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336d1f8, 90 },
    { 0xcb76d1f8, 100 },
    { 0x7316d1fa, 91 },
    { 0x60d6d1fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46d200, 67 },
    { 0, 0 },
    { 0x1986d202, 33 },
    { 0xa6c6d203, 52 },
    { 0, 0 },
    { 0xfce6d205, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b36d208, 98 },
    { 0x98a6d209, 95 },
    { 0x85b6d20a, 76 },
    { 0x3fc6d20b, 90 },
    { 0x1c96d20c, 76 },
    { 0x3486d20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab76d21a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc6d21e, 95 },
    { 0, 0 },
    { 0xda76d220, 76 },
    { 0xb896d220, 33 },
    { 0xd46d220, 61 },
    { 0x8386d220, 100 },
    { 0, 0 },
    { 0x2d6d225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe86d22a, 93 },
    { 0x3346d22a, 95 },
    { 0x7b6d22c, 90 },
    { 0xe736d22a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc876d230, 33 },
    { 0x6fc6d230, 33 },
    { 0xfd6d230, 61 },
    { 0x3b6d231, 95 },
    { 0, 0 },
    { 0xe0c6d235, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06d23a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26d23e, 98 },
    { 0x2be6d23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c6d243, 55 },
    { 0, 0 },
    { 0xe426d245, 95 },
    { 0, 0 },
    { 0x1a86d247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad6d24b, 90 },
    { 0x2f96d24b, 23 },
    { 0xe356d24d, 90 },
    { 0xff86d24e, 40 },
    { 0x5216d24b, 34 },
    { 0, 0 },
    { 0x7f16d251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406d257, 95 },
    { 0x7a96d257, 33 },
    { 0x4296d258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e6d25c, 76 },
    { 0, 0 },
    { 0x266d25e, 69 },
    { 0, 0 },
    { 0xde36d260, 94 },
    { 0, 0 },
    { 0xc266d262, 99 },
    { 0, 0 },
    { 0x1e46d264, 34 },
    { 0, 0 },
    { 0x6816d266, 55 },
    { 0, 0 },
    { 0x6a36d268, 76 },
    { 0x4166d268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd596d26c, 100 },
    { 0x1b96d26c, 17 },
    { 0xe946d26e, 33 },
    { 0xc9d6d26e, 98 },
    { 0x3af6d270, 23 },
    { 0x5746d271, 77 },
    { 0x4806d26d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6d278, 61 },
    { 0, 0 },
    { 0x95e6d27a, 55 },
    { 0xfab6d27b, 99 },
    { 0x1006d27c, 33 },
    { 0, 0 },
    { 0xaf96d27e, 98 },
    { 0x8d16d27f, 61 },
    { 0, 0 },
    { 0x3ef6d281, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe976d286, 95 },
    { 0, 0 },
    { 0x5c66d288, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba86d291, 23 },
    { 0, 0 },
    { 0x1866d293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956d297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f6d29a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46d2a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e6d2a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b26d2b0, 93 },
    { 0x7816d2b1, 34 },
    { 0x9e96d2b2, 95 },
    { 0, 0 },
    { 0xbd66d2b4, 61 },
    { 0xc976d2b5, 95 },
    { 0x72f6d2b5, 61 },
    { 0x4346d2b7, 61 },
    { 0, 0 },
    { 0x6046d2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176d2bd, 55 },
    { 0, 0 },
    { 0xea66d2bf, 43 },
    { 0, 0 },
    { 0x94f6d2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d2c5, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5376d2cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646d2ce, 55 },
    { 0x4896d2cf, 76 },
    { 0x8ab6d2d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb926d2d3, 33 },
    { 0, 0 },
    { 0xd2f6d2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed76d2da, 75 },
    { 0x53c6d2da, 40 },
    { 0, 0 },
    { 0x26d2dd, 99 },
    { 0xd8d6d2de, 90 },
    { 0xee26d2df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d6d2e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b86d2e6, 98 },
    { 0, 0 },
    { 0x7336d2e8, 61 },
    { 0, 0 },
    { 0x85c6d2ea, 61 },
    { 0x81a6d2ea, 88 },
    { 0x60d6d2ec, 77 },
    { 0x7a46d2ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc96d2f0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x1486d2f3, 46 },
    { 0x4bf6d2f4, 33 },
    { 0, 0 },
    { 0xe7c6d2f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e76d2fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6d2fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c6d301, 66 },
    { 0x8506d302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb976d308, 43 },
    { 0xfbd6d309, 23 },
    { 0, 0 },
    { 0xff86d30b, 33 },
    { 0xd586d30b, 67 },
    { 0x9836d30c, 90 },
    { 0xfde6d30e, 90 },
    { 0xe2c6d30c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336d314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6d31d, 65 },
    { 0, 0 },
    { 0x4916d31f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26d324, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2236d32b, 90 },
    { 0xbc26d32c, 23 },
    { 0x2d96d32c, 90 },
    { 0x96d6d32d, 80 },
    { 0x10c6d32f, 100 },
    { 0x5976d330, 98 },
    { 0x8a06d331, 76 },
    { 0x756d32f, 98 },
    { 0x6156d333, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b16d33b, 76 },
    { 0xc06d33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f6d340, 91 },
    { 0xe056d341, 90 },
    { 0x7d36d341, 76 },
    { 0x6046d340, 75 },
    { 0x70b6d342, 67 },
    { 0, 0 },
    { 0xd176d346, 23 },
    { 0xfda6d347, 61 },
    { 0xda26d347, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d6d35d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6d360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd146d364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc6d369, 100 },
    { 0, 0 },
    { 0x72a6d36b, 98 },
    { 0xd476d36c, 76 },
    { 0, 0 },
    { 0x9686d36e, 100 },
    { 0x9a96d36f, 76 },
    { 0, 0 },
    { 0x77b6d371, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16d374, 93 },
    { 0xef86d374, 61 },
    { 0xd076d376, 97 },
    { 0x7816d374, 55 },
    { 0x3766d378, 98 },
    { 0xec6d379, 85 },
    { 0x1a86d376, 55 },
    { 0, 0 },
    { 0x7b06d37c, 90 },
    { 0xb066d37d, 95 },
    { 0, 0 },
    { 0x2bb6d37f, 90 },
    { 0x35b6d380, 52 },
    { 0xf016d380, 24 },
    { 0xcb76d380, 90 },
    { 0x41a6d380, 69 },
    { 0xb496d384, 90 },
    { 0, 0 },
    { 0x8576d386, 93 },
    { 0x2946d387, 90 },
    { 0x2356d388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6d38e, 33 },
    { 0xe746d38f, 61 },
    { 0x4706d38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186d396, 34 },
    { 0, 0 },
    { 0xc1d6d398, 61 },
    { 0x28e6d399, 61 },
    { 0x5596d399, 77 },
    { 0x27d6d399, 33 },
    { 0xa106d39c, 98 },
    { 0xe0f6d398, 33 },
    { 0x2d6d398, 64 },
    { 0xb5e6d39c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6676d3a2, 90 },
    { 0x70f6d3a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab6d3a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96d3ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6d3b1, 93 },
    { 0x15d6d3b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4246d3b9, 23 },
    { 0x6026d3ba, 90 },
    { 0x7056d3ba, 46 },
    { 0, 0 },
    { 0xb0b6d3bd, 94 },
    { 0, 0 },
    { 0x2596d3bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9326d3c2, 61 },
    { 0x8166d3c3, 46 },
    { 0xce06d3c3, 8 },
    { 0xea16d3c5, 94 },
    { 0, 0 },
    { 0xc746d3c7, 98 },
    { 0x5ae6d3c8, 55 },
    { 0x1bc6d3c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266d3cd, 61 },
    { 0xb9a6d3cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126d3d3, 83 },
    { 0xbc46d3d4, 95 },
    { 0, 0 },
    { 0xa0c6d3d6, 95 },
    { 0, 0 },
    { 0x5ab6d3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586d3e3, 100 },
    { 0, 0 },
    { 0xe056d3e5, 33 },
    { 0x5a46d3e6, 33 },
    { 0x5396d3e6, 33 },
    { 0x1d76d3e8, 98 },
    { 0, 0 },
    { 0x2626d3ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf946d3ed, 99 },
    { 0, 0 },
    { 0x5ca6d3ef, 87 },
    { 0x8276d3f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e6d3f6, 93 },
    { 0xffe6d3f7, 33 },
    { 0x5746d3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6266d3fd, 84 },
    { 0, 0 },
    { 0x29c6d3ff, 76 },
    { 0x1f16d400, 99 },
    { 0x476d401, 90 },
    { 0xebc6d401, 90 },
    { 0x9ad6d3ff, 33 },
    { 0xe476d400, 95 },
    { 0xc036d401, 67 },
    { 0xc506d406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce86d409, 76 },
    { 0xb656d40a, 66 },
    { 0x41a6d40b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb6d40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b66d412, 100 },
    { 0x1296d413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96d416, 33 },
    { 0xe7e6d417, 65 },
    { 0x4f86d417, 33 },
    { 0xa196d416, 76 },
    { 0x4f16d41a, 90 },
    { 0xf026d41b, 65 },
    { 0x84e6d41b, 23 },
    { 0, 0 },
    { 0xe316d41e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3306d424, 78 },
    { 0x1066d424, 23 },
    { 0, 0 },
    { 0x4556d427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b6d42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf16d42e, 38 },
    { 0xb796d42f, 90 },
    { 0xe3c6d430, 76 },
    { 0x1176d431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc86d435, 93 },
    { 0x56b6d435, 34 },
    { 0x1556d437, 68 },
    { 0x6f76d438, 90 },
    { 0x5c46d437, 61 },
    { 0x8ad6d437, 55 },
    { 0xcad6d43b, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd6d43f, 90 },
    { 0, 0 },
    { 0x4416d441, 51 },
    { 0, 0 },
    { 0xb316d443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb876d446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1256d44a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f6d44e, 93 },
    { 0x6546d44f, 23 },
    { 0xe006d450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016d453, 90 },
    { 0, 0 },
    { 0x4576d455, 80 },
    { 0x7d36d456, 33 },
    { 0, 0 },
    { 0xc966d458, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6476d45c, 46 },
    { 0x6c56d45d, 34 },
    { 0x9ad6d45e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06d461, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef66d469, 95 },
    { 0x8d56d46a, 98 },
    { 0x9406d46a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6d46e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad6d475, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf176d47b, 91 },
    { 0x8d96d47c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d482, 90 },
    { 0x6be6d483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d06d489, 34 },
    { 0x2c26d48a, 48 },
    { 0xcd56d48b, 76 },
    { 0x6736d489, 55 },
    { 0x9b16d48c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b6d499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3596d49c, 98 },
    { 0, 0 },
    { 0x3726d49e, 55 },
    { 0, 0 },
    { 0x9566d4a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956d4a4, 87 },
    { 0, 0 },
    { 0x31c6d4a6, 76 },
    { 0xddd6d4a7, 76 },
    { 0xbbc6d4a7, 85 },
    { 0, 0 },
    { 0xc2e6d4aa, 90 },
    { 0x16f6d4aa, 78 },
    { 0xba06d4ab, 34 },
    { 0x5e36d4ac, 23 },
    { 0x8ec6d4ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f6d4b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf6d4b6, 98 },
    { 0xaf6d4b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56d4ba, 98 },
    { 0, 0 },
    { 0x1436d4bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826d4c1, 90 },
    { 0x9fb6d4c2, 95 },
    { 0xc3e6d4c2, 90 },
    { 0xc926d4c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736d4c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d4cf, 76 },
    { 0xd866d4cf, 95 },
    { 0x48a6d4d1, 76 },
    { 0, 0 },
    { 0x66f6d4d3, 88 },
    { 0, 0 },
    { 0x2426d4d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e16d4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46d4e1, 90 },
    { 0x6ed6d4e1, 33 },
    { 0x5ff6d4e3, 98 },
    { 0x17a6d4e4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xda96d4e7, 46 },
    { 0xc3a6d4e8, 95 },
    { 0x4456d4e9, 34 },
    { 0x3006d4ea, 61 },
    { 0x7946d4e7, 55 },
    { 0x986d4ec, 90 },
    { 0xcae6d4ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696d4f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed16d4f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36d4f9, 90 },
    { 0x1e96d4f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f6d4fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c6d509, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196d50f, 93 },
    { 0, 0 },
    { 0x3616d511, 80 },
    { 0, 0 },
    { 0x9396d513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab6d517, 60 },
    { 0x416d518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566d51f, 95 },
    { 0x5a86d520, 98 },
    { 0xd686d520, 76 },
    { 0x7706d51f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d525, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c6d529, 90 },
    { 0x8086d529, 61 },
    { 0x5956d52b, 19 },
    { 0xb566d52c, 93 },
    { 0, 0 },
    { 0x1906d52e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6d539, 84 },
    { 0, 0 },
    { 0xf1a6d53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c76d53e, 33 },
    { 0xea06d53f, 76 },
    { 0x2e66d53f, 34 },
    { 0x14e6d541, 20 },
    { 0xc106d542, 33 },
    { 0xd86d543, 98 },
    { 0x2d6d540, 52 },
    { 0x6966d541, 98 },
    { 0x6b6d545, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd136d549, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c96d54c, 40 },
    { 0x9a66d54d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1736d550, 95 },
    { 0, 0 },
    { 0x2256d552, 90 },
    { 0xe3c6d553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d6d557, 95 },
    { 0xc376d558, 23 },
    { 0x1d36d559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8546d564, 47 },
    { 0xe2b6d565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc6d569, 95 },
    { 0x9806d56a, 90 },
    { 0, 0 },
    { 0x97a6d56c, 61 },
    { 0, 0 },
    { 0x3346d56e, 55 },
    { 0xd26d56f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4416d573, 78 },
    { 0xe116d573, 95 },
    { 0x5d66d574, 78 },
    { 0x31c6d576, 33 },
    { 0x91c6d576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2626d57b, 76 },
    { 0, 0 },
    { 0x61e6d57d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46d581, 98 },
    { 0xae6d582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe866d585, 91 },
    { 0, 0 },
    { 0x7a6d587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476d58a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1726d58d, 76 },
    { 0x1af6d58e, 33 },
    { 0xda46d58f, 98 },
    { 0x54f6d58e, 98 },
    { 0, 0 },
    { 0x4126d592, 90 },
    { 0x2cb6d592, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6d596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5706d59b, 33 },
    { 0x7326d59c, 55 },
    { 0, 0 },
    { 0x25d6d59e, 90 },
    { 0x416d59e, 33 },
    { 0, 0 },
    { 0x3a96d5a1, 33 },
    { 0, 0 },
    { 0x3756d5a3, 48 },
    { 0x4dc6d5a4, 77 },
    { 0xae16d5a3, 90 },
    { 0x4796d5a5, 99 },
    { 0x3676d5a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad26d5ab, 61 },
    { 0x5176d5ac, 90 },
    { 0x6126d5ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f6d5b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c6d5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f16d5bf, 67 },
    { 0x8556d5c0, 23 },
    { 0, 0 },
    { 0x6556d5c2, 61 },
    { 0x4d96d5c2, 33 },
    { 0x8836d5c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456d5cd, 66 },
    { 0xe4f6d5ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf6d5d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc886d5d5, 84 },
    { 0x39a6d5d6, 81 },
    { 0xdab6d5d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb776d5db, 58 },
    { 0xdfa6d5db, 76 },
    { 0xbdd6d5dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76d5e1, 33 },
    { 0x9506d5e2, 61 },
    { 0xd9d6d5e3, 33 },
    { 0, 0 },
    { 0xe26d5e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb056d5e8, 43 },
    { 0, 0 },
    { 0x37b6d5ea, 34 },
    { 0, 0 },
    { 0xf836d5ec, 88 },
    { 0, 0 },
    { 0xf876d5ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266d5f1, 23 },
    { 0xb806d5f2, 90 },
    { 0x4676d5f2, 52 },
    { 0x8656d5f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a6d5f7, 33 },
    { 0x6606d5f7, 90 },
    { 0xf3b6d5f8, 55 },
    { 0x29c6d5fa, 66 },
    { 0xf036d5fb, 95 },
    { 0x6c96d5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d602, 90 },
    { 0, 0 },
    { 0x1e76d604, 95 },
    { 0, 0 },
    { 0xda66d606, 33 },
    { 0, 0 },
    { 0x6c56d608, 93 },
    { 0xca66d609, 17 },
    { 0x666d60a, 93 },
    { 0x3896d60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb6d60e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f6d612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2716d619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6d61d, 61 },
    { 0xe126d61e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96d623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca76d628, 90 },
    { 0x3976d628, 95 },
    { 0x92c6d62a, 90 },
    { 0x476d62b, 33 },
    { 0xe156d62b, 90 },
    { 0xb2a6d62d, 90 },
    { 0xac16d62b, 98 },
    { 0xbb36d62c, 33 },
    { 0xdcf6d62d, 85 },
    { 0x7846d631, 90 },
    { 0, 0 },
    { 0x7f56d633, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56d636, 99 },
    { 0xef56d636, 84 },
    { 0, 0 },
    { 0x9886d639, 61 },
    { 0x5416d63a, 33 },
    { 0x2f76d63b, 33 },
    { 0x6396d63c, 90 },
    { 0xcb36d63a, 46 },
    { 0xb6d6d63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d641, 33 },
    { 0x3206d642, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9416d645, 33 },
    { 0, 0 },
    { 0x1fb6d647, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b6d64a, 98 },
    { 0, 0 },
    { 0xa76d64c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466d652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c6d655, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb676d65d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac6d661, 76 },
    { 0x2e56d662, 98 },
    { 0x5d26d661, 61 },
    { 0, 0 },
    { 0x45c6d665, 40 },
    { 0, 0 },
    { 0xe366d667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf176d66a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366d671, 61 },
    { 0xe76d672, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7866d67c, 90 },
    { 0x2286d67d, 98 },
    { 0xdd56d67c, 95 },
    { 0, 0 },
    { 0xc7b6d680, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56d688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf476d68b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f6d68f, 98 },
    { 0xf966d690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7216d695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6d698, 85 },
    { 0x8366d699, 90 },
    { 0xdbd6d698, 93 },
    { 0xafa6d69a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6d6a1, 97 },
    { 0, 0 },
    { 0x92c6d6a3, 95 },
    { 0x406d6a4, 52 },
    { 0xad06d6a5, 100 },
    { 0x30e6d6a6, 76 },
    { 0xdf36d6a7, 77 },
    { 0xad6d6a4, 61 },
    { 0xfde6d6a4, 76 },
    { 0x6de6d6aa, 90 },
    { 0x9d76d6aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a6d6af, 99 },
    { 0x5f26d6af, 61 },
    { 0x6546d6b1, 76 },
    { 0x2d26d6b2, 78 },
    { 0x8876d6b1, 55 },
    { 0, 0 },
    { 0x7316d6b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776d6b9, 98 },
    { 0, 0 },
    { 0xcb6d6bb, 33 },
    { 0x79c6d6bb, 55 },
    { 0xa286d6bd, 76 },
    { 0x65b6d6bd, 98 },
    { 0x9f6d6be, 98 },
    { 0xd166d6c0, 87 },
    { 0xc956d6c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad6d6c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4626d6ce, 23 },
    { 0x4126d6cf, 91 },
    { 0x16a6d6d0, 95 },
    { 0x82f6d6d1, 90 },
    { 0x3cd6d6d0, 33 },
    { 0x79b6d6d3, 67 },
    { 0xfe06d6d1, 33 },
    { 0, 0 },
    { 0x4ee6d6d6, 67 },
    { 0xc406d6d6, 95 },
    { 0x90e6d6d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f46d6dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b36d6e1, 38 },
    { 0, 0 },
    { 0xa6a6d6e3, 100 },
    { 0, 0 },
    { 0x9dc6d6e5, 85 },
    { 0xb626d6e5, 76 },
    { 0x7a56d6e7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb236d6ea, 75 },
    { 0x1df6d6eb, 95 },
    { 0xbf56d6ec, 33 },
    { 0xc576d6ed, 46 },
    { 0x5686d6ee, 90 },
    { 0xc5b6d6ef, 61 },
    { 0, 0 },
    { 0x2416d6f1, 23 },
    { 0x1cc6d6f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd86d6f6, 65 },
    { 0xced6d6f7, 90 },
    { 0xd3a6d6f8, 93 },
    { 0xc1e6d6f8, 33 },
    { 0xddb6d6f7, 33 },
    { 0x2d06d6f8, 98 },
    { 0x606d6fa, 87 },
    { 0x7fd6d6f7, 23 },
    { 0, 0 },
    { 0x29b6d6ff, 98 },
    { 0xbe06d700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd656d704, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c6d708, 46 },
    { 0x9ed6d709, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df6d70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586d710, 61 },
    { 0x9506d711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6d715, 17 },
    { 0x196d715, 84 },
    { 0, 0 },
    { 0xef46d718, 23 },
    { 0xcea6d718, 98 },
    { 0xb396d71a, 33 },
    { 0x3e46d71a, 33 },
    { 0x8f46d719, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa916d722, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386d727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba6d72a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b6d72d, 33 },
    { 0x99c6d72d, 60 },
    { 0x6e6d72f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e6d733, 90 },
    { 0, 0 },
    { 0xed96d735, 95 },
    { 0, 0 },
    { 0x45d6d737, 91 },
    { 0, 0 },
    { 0xf2e6d739, 77 },
    { 0x5786d739, 90 },
    { 0xe956d73b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f6d747, 78 },
    { 0xd726d748, 96 },
    { 0x9e16d749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196d74c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe386d755, 23 },
    { 0x1876d756, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286d75b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe046d75e, 55 },
    { 0x3d16d75f, 81 },
    { 0x63a6d760, 43 },
    { 0x39e6d761, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a6d765, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d769, 76 },
    { 0x85a6d76a, 23 },
    { 0xf556d76b, 61 },
    { 0xfd56d76c, 98 },
    { 0xac86d76d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9656d770, 52 },
    { 0, 0 },
    { 0x7866d772, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446d779, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86d77f, 87 },
    { 0xe106d780, 95 },
    { 0xa9c6d781, 90 },
    { 0x4596d781, 43 },
    { 0x3e06d783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc986d788, 91 },
    { 0xb396d789, 61 },
    { 0x6b6d78a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc216d78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6d793, 95 },
    { 0, 0 },
    { 0x9326d795, 52 },
    { 0xcd16d795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5036d79e, 33 },
    { 0, 0 },
    { 0x2366d7a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5016d7a3, 46 },
    { 0x4136d7a4, 55 },
    { 0x76f6d7a3, 23 },
    { 0x7d26d7a3, 34 },
    { 0x2ed6d7a6, 95 },
    { 0, 0 },
    { 0x5da6d7a9, 23 },
    { 0xd3f6d7aa, 65 },
    { 0x8f6d7ab, 23 },
    { 0, 0 },
    { 0xa2b6d7ad, 67 },
    { 0x1836d7ae, 38 },
    { 0xa666d7af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe066d7b2, 23 },
    { 0xe606d7b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2726d7b6, 61 },
    { 0, 0 },
    { 0x72e6d7b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06d7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e6d7c7, 90 },
    { 0x2886d7c8, 33 },
    { 0xf296d7c8, 43 },
    { 0xfbc6d7c7, 66 },
    { 0, 0 },
    { 0xc5c6d7cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e6d7d7, 23 },
    { 0x9546d7d8, 93 },
    { 0, 0 },
    { 0xd356d7da, 76 },
    { 0, 0 },
    { 0xe826d7dc, 46 },
    { 0, 0 },
    { 0x6466d7de, 95 },
    { 0x916d7df, 33 },
    { 0x30b6d7df, 90 },
    { 0xd206d7de, 51 },
    { 0, 0 },
    { 0x3a26d7e3, 43 },
    { 0, 0 },
    { 0x2716d7e5, 90 },
    { 0xfaa6d7e6, 61 },
    { 0xea56d7e7, 23 },
    { 0xc096d7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86d7ec, 61 },
    { 0x2266d7ed, 43 },
    { 0x7286d7ee, 87 },
    { 0xc1d6d7ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3576d7f6, 76 },
    { 0x9016d7f7, 95 },
    { 0xf116d7f8, 98 },
    { 0, 0 },
    { 0xd06d7fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66d7fd, 90 },
    { 0x3a66d7fe, 90 },
    { 0, 0 },
    { 0x7856d800, 34 },
    { 0, 0 },
    { 0xbbb6d802, 52 },
    { 0, 0 },
    { 0xc196d804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a6d807, 90 },
    { 0, 0 },
    { 0xd406d809, 90 },
    { 0, 0 },
    { 0x8106d80b, 23 },
    { 0x1a36d80c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126d80f, 64 },
    { 0xc326d810, 61 },
    { 0x20a6d80f, 52 },
    { 0xfc46d810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6d815, 78 },
    { 0, 0 },
    { 0xcba6d817, 33 },
    { 0xb1f6d818, 61 },
    { 0x2c66d817, 90 },
    { 0xd236d81a, 91 },
    { 0x1db6d81b, 98 },
    { 0xb1c6d81c, 98 },
    { 0x39b6d81b, 76 },
    { 0xe556d81c, 98 },
    { 0xfb6d81c, 83 },
    { 0x83c6d820, 90 },
    { 0xcc76d821, 98 },
    { 0xa3a6d821, 93 },
    { 0x1a36d81b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6d828, 100 },
    { 0, 0 },
    { 0xba36d82a, 33 },
    { 0x7706d82b, 46 },
    { 0x1476d82a, 34 },
    { 0xe2c6d82d, 33 },
    { 0x10a6d82c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e6d832, 34 },
    { 0xcd56d832, 87 },
    { 0, 0 },
    { 0x4bb6d835, 33 },
    { 0x21d6d836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb796d839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86d83e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b66d84a, 61 },
    { 0, 0 },
    { 0xa696d84c, 59 },
    { 0x3da6d84c, 23 },
    { 0x5226d84c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda6d858, 78 },
    { 0, 0 },
    { 0x5ce6d85a, 33 },
    { 0, 0 },
    { 0x1576d85c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4496d862, 52 },
    { 0xe4b6d863, 100 },
    { 0, 0 },
    { 0xf746d865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6d86c, 100 },
    { 0xdbd6d86d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6d876, 33 },
    { 0x1026d877, 90 },
    { 0x93c6d877, 52 },
    { 0xdd96d879, 33 },
    { 0xacd6d879, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb76d881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a6d885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d76d88c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06d88f, 81 },
    { 0x6786d890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce76d895, 84 },
    { 0, 0 },
    { 0xcd36d897, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6d89d, 96 },
    { 0xe946d89e, 66 },
    { 0x4706d89f, 76 },
    { 0x76c6d89f, 61 },
    { 0x58a6d8a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5676d8aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe126d8ae, 34 },
    { 0, 0 },
    { 0x2846d8b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd416d8b3, 23 },
    { 0x4986d8b4, 90 },
    { 0x50e6d8b5, 90 },
    { 0xeb16d8b6, 23 },
    { 0xb506d8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd236d8bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956d8c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc6d8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb276d8cf, 23 },
    { 0xb0e6d8d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f36d8d4, 100 },
    { 0x9e66d8d5, 81 },
    { 0xfcc6d8d4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf6d8d9, 95 },
    { 0x7166d8da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a6d8de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa516d8ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf36d8ed, 61 },
    { 0x8736d8ed, 76 },
    { 0, 0 },
    { 0x9c36d8f0, 76 },
    { 0x4f76d8f0, 98 },
    { 0, 0 },
    { 0xc1b6d8f3, 52 },
    { 0, 0 },
    { 0xcff6d8f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b16d8fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2256d900, 90 },
    { 0, 0 },
    { 0x35e6d902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7556d90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16d911, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a6d917, 40 },
    { 0, 0 },
    { 0x2a26d919, 95 },
    { 0x4346d919, 76 },
    { 0, 0 },
    { 0xeac6d91c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7716d922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3076d929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc316d92c, 77 },
    { 0x2406d92c, 55 },
    { 0x66d6d92e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8826d932, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8256d936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36d93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe436d941, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316d944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6d949, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6d950, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2976d953, 67 },
    { 0x6766d954, 90 },
    { 0, 0 },
    { 0xdea6d956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866d95f, 67 },
    { 0x4f86d960, 24 },
    { 0x786d960, 98 },
    { 0xc626d962, 43 },
    { 0x34a6d963, 76 },
    { 0xcd66d964, 68 },
    { 0x7bd6d95f, 90 },
    { 0x3bf6d95f, 38 },
    { 0, 0 },
    { 0x4e26d968, 61 },
    { 0x35b6d969, 23 },
    { 0x8696d96a, 34 },
    { 0xc736d96b, 93 },
    { 0x4356d968, 33 },
    { 0x7886d96d, 91 },
    { 0xc0f6d96e, 66 },
    { 0x9e26d96f, 90 },
    { 0x6546d970, 95 },
    { 0x7ca6d971, 46 },
    { 0xa476d972, 84 },
    { 0x1c26d96f, 90 },
    { 0, 0 },
    { 0x2316d975, 52 },
    { 0x6036d976, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8626d979, 23 },
    { 0x2c86d97a, 99 },
    { 0x5776d97a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536d97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6d983, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316d986, 90 },
    { 0xa26d987, 33 },
    { 0xfc6d986, 53 },
    { 0, 0 },
    { 0x7336d98a, 55 },
    { 0xf36d98b, 46 },
    { 0x7426d98c, 97 },
    { 0xa6d98b, 25 },
    { 0x1ec6d98e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226d991, 95 },
    { 0xef66d991, 83 },
    { 0, 0 },
    { 0x2a26d994, 90 },
    { 0xe6d995, 55 },
    { 0x2216d995, 34 },
    { 0x7786d997, 68 },
    { 0x9dc6d998, 33 },
    { 0xa06d998, 99 },
    { 0xbf16d99a, 46 },
    { 0xe216d99a, 78 },
    { 0xc036d99c, 90 },
    { 0x19d6d99d, 87 },
    { 0x4196d99e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf626d9a4, 61 },
    { 0, 0 },
    { 0x8db6d9a6, 61 },
    { 0x18d6d9a7, 75 },
    { 0x5816d9a7, 55 },
    { 0x4296d9a6, 87 },
    { 0, 0 },
    { 0xe006d9ab, 87 },
    { 0xbe26d9ac, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c6d9b0, 64 },
    { 0x29f6d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb6d9b4, 33 },
    { 0x6c6d9b4, 95 },
    { 0x7d6d9b6, 33 },
    { 0x45a6d9b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666d9bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236d9c4, 28 },
    { 0, 0 },
    { 0x7ee6d9c6, 95 },
    { 0x6ea6d9c7, 34 },
    { 0xd7e6d9c6, 33 },
    { 0xcc36d9c9, 90 },
    { 0, 0 },
    { 0xd076d9cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6d9cf, 99 },
    { 0, 0 },
    { 0xb296d9d1, 90 },
    { 0x1d56d9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66d9d5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b6d9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6d9db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe16d9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4666d9e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1346d9ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9126d9f4, 33 },
    { 0x64e6d9f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6d9f8, 66 },
    { 0xb286d9f8, 33 },
    { 0x6776d9f8, 87 },
    { 0xb976d9fb, 49 },
    { 0xd4f6d9fb, 76 },
    { 0x5186d9fd, 34 },
    { 0x76a6d9fd, 90 },
    { 0x57a6d9fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66da03, 55 },
    { 0, 0 },
    { 0xaaa6da05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6da0a, 90 },
    { 0, 0 },
    { 0x4686da0c, 23 },
    { 0xb426da0d, 90 },
    { 0x1936da0e, 55 },
    { 0xac86da0f, 76 },
    { 0x9856da0f, 91 },
    { 0x2df6da11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846da15, 99 },
    { 0xa676da16, 93 },
    { 0x7036da17, 90 },
    { 0, 0 },
    { 0x4b86da19, 76 },
    { 0, 0 },
    { 0xb806da1b, 88 },
    { 0x9a06da1b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba66da21, 85 },
    { 0, 0 },
    { 0x5216da23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316da2a, 90 },
    { 0x38b6da2a, 76 },
    { 0x5fd6da2c, 33 },
    { 0xd126da2c, 95 },
    { 0xd1e6da2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1506da38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b6da3b, 95 },
    { 0xe126da3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6da42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb76da46, 98 },
    { 0x20d6da47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5866da4f, 61 },
    { 0xac96da4f, 98 },
    { 0xedd6da4f, 81 },
    { 0x716da52, 40 },
    { 0x6286da52, 91 },
    { 0x76b6da52, 33 },
    { 0x2de6da52, 38 },
    { 0, 0 },
    { 0x8566da57, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26da5b, 46 },
    { 0x5a46da5c, 33 },
    { 0x7896da5d, 68 },
    { 0xd286da5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06da61, 61 },
    { 0, 0 },
    { 0xe486da63, 95 },
    { 0, 0 },
    { 0xf3c6da65, 33 },
    { 0xa446da65, 95 },
    { 0, 0 },
    { 0x2156da68, 95 },
    { 0x6f96da69, 52 },
    { 0, 0 },
    { 0xb7f6da6b, 43 },
    { 0xa356da6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc516da71, 90 },
    { 0x8306da72, 88 },
    { 0xc246da73, 33 },
    { 0xa696da73, 61 },
    { 0, 0 },
    { 0x7026da76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd6da7a, 90 },
    { 0x5946da7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa216da7f, 76 },
    { 0, 0 },
    { 0xc466da81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b6da84, 98 },
    { 0x7496da85, 98 },
    { 0, 0 },
    { 0x23a6da87, 98 },
    { 0, 0 },
    { 0x6ea6da89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e26da8e, 67 },
    { 0x5846da8f, 84 },
    { 0x1946da90, 61 },
    { 0xcf46da91, 91 },
    { 0x1386da91, 76 },
    { 0xb2f6da93, 66 },
    { 0x27b6da91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d6da97, 76 },
    { 0x1ab6da97, 33 },
    { 0xa16da98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b66da9c, 85 },
    { 0x62c6da9c, 98 },
    { 0xc3e6da9c, 95 },
    { 0, 0 },
    { 0xc06daa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56daa4, 66 },
    { 0, 0 },
    { 0x67a6daa6, 43 },
    { 0x8da6daa7, 68 },
    { 0xe4c6daa6, 66 },
    { 0x5f76daa9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e6daac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6306dab5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6796dab9, 67 },
    { 0x3826dab9, 95 },
    { 0x3156dabb, 93 },
    { 0xbde6dabb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6dac0, 33 },
    { 0x7706dac1, 43 },
    { 0xbef6dac2, 33 },
    { 0x1ac6dac1, 90 },
    { 0, 0 },
    { 0x8466dac5, 46 },
    { 0, 0 },
    { 0xb1b6dac7, 43 },
    { 0xe1f6dac8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a6dacb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f6dace, 37 },
    { 0xf0b6dacf, 33 },
    { 0x1b16dad0, 60 },
    { 0, 0 },
    { 0xdc26dad2, 33 },
    { 0x846dad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8216dade, 95 },
    { 0x18e6dade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d6dae2, 55 },
    { 0x8096dae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a26daed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f46daf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4086daf5, 84 },
    { 0xde26daf6, 61 },
    { 0x7266daf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d6dafa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06dafd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c6db04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea6db0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6946db11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6db16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6db1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7466db22, 95 },
    { 0x18e6db23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66db27, 34 },
    { 0xbf06db28, 17 },
    { 0x5f56db29, 23 },
    { 0x21e6db29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4186db3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1596db40, 78 },
    { 0xcb76db41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc296db45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad56db49, 88 },
    { 0xcbc6db4a, 61 },
    { 0x666db4a, 90 },
    { 0x27b6db4c, 81 },
    { 0xd7f6db4c, 43 },
    { 0x126db4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6db54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326db5a, 90 },
    { 0x80a6db5b, 85 },
    { 0, 0 },
    { 0x3706db5d, 78 },
    { 0xea86db5e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee6db61, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b6db64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc6db67, 33 },
    { 0x2416db68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b76db6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6db76, 77 },
    { 0xc4a6db77, 78 },
    { 0x9b96db78, 55 },
    { 0x8706db79, 93 },
    { 0x6396db7a, 90 },
    { 0x496db77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b6db7f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b96db82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce6db86, 61 },
    { 0x9466db86, 90 },
    { 0, 0 },
    { 0x9b06db89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c76db8d, 23 },
    { 0x7d06db8e, 43 },
    { 0, 0 },
    { 0xcf66db90, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56db96, 98 },
    { 0, 0 },
    { 0x5a06db98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6dbab, 90 },
    { 0xae56dbab, 87 },
    { 0xf0b6dbac, 34 },
    { 0, 0 },
    { 0xc546dbaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce86dbb2, 96 },
    { 0x26dbb3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb6dbb6, 55 },
    { 0, 0 },
    { 0xc336dbb8, 90 },
    { 0x3e46dbb8, 43 },
    { 0x1676dbb9, 90 },
    { 0, 0 },
    { 0x2426dbbc, 75 },
    { 0, 0 },
    { 0x2296dbbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56dbc1, 46 },
    { 0x6f86dbc2, 61 },
    { 0xb096dbc2, 100 },
    { 0xe046dbc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6836dbc7, 76 },
    { 0, 0 },
    { 0x7f76dbc9, 61 },
    { 0xd516dbca, 33 },
    { 0, 0 },
    { 0x7946dbcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6dbd1, 87 },
    { 0, 0 },
    { 0xb426dbd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf6dbd7, 34 },
    { 0xf626dbd8, 55 },
    { 0xb1b6dbd9, 85 },
    { 0, 0 },
    { 0x79c6dbdb, 66 },
    { 0x8c66dbdc, 55 },
    { 0xfe46dbdd, 95 },
    { 0x4296dbde, 34 },
    { 0xad86dbdf, 78 },
    { 0, 0 },
    { 0xfe96dbe1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b6dbe4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16dbeb, 85 },
    { 0x1ca6dbec, 90 },
    { 0x8b06dbed, 90 },
    { 0x3e86dbed, 61 },
    { 0xbb16dbef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa096dbf3, 33 },
    { 0x66d6dbf3, 23 },
    { 0xc7a6dbf5, 23 },
    { 0x1e6dbf6, 55 },
    { 0x3c76dbf7, 90 },
    { 0, 0 },
    { 0x476dbf9, 76 },
    { 0x2e06dbf9, 61 },
    { 0, 0 },
    { 0x8c16dbfc, 95 },
    { 0x44a6dbfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f66dc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536dc0a, 83 },
    { 0x1696dc0a, 98 },
    { 0x2026dc0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6dc0f, 23 },
    { 0x83f6dc0f, 68 },
    { 0x5dd6dc11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6dc14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd486dc1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826dc1e, 23 },
    { 0, 0 },
    { 0xfad6dc20, 61 },
    { 0x50a6dc21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1236dc28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef16dc2b, 66 },
    { 0, 0 },
    { 0xf2a6dc2d, 33 },
    { 0, 0 },
    { 0x3ab6dc2f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6dc34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2206dc37, 90 },
    { 0, 0 },
    { 0xf9e6dc39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6dc3c, 22 },
    { 0, 0 },
    { 0xc466dc3e, 95 },
    { 0x4c26dc3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf16dc43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf766dc47, 95 },
    { 0x3496dc48, 98 },
    { 0xf556dc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6dc52, 87 },
    { 0xf236dc53, 61 },
    { 0x5b06dc53, 23 },
    { 0, 0 },
    { 0x52b6dc56, 95 },
    { 0x43d6dc56, 34 },
    { 0x7086dc57, 61 },
    { 0xc76dc59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd66dc5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa076dc61, 23 },
    { 0, 0 },
    { 0xe806dc63, 48 },
    { 0, 0 },
    { 0x2b06dc65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6dc69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1266dc6c, 33 },
    { 0xcf06dc6c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956dc73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06dc76, 95 },
    { 0xf306dc76, 99 },
    { 0x4e36dc78, 61 },
    { 0x4d46dc79, 61 },
    { 0xd696dc79, 90 },
    { 0xb8c6dc7b, 93 },
    { 0xd036dc7b, 98 },
    { 0xe776dc7c, 90 },
    { 0x1566dc7e, 33 },
    { 0x4c96dc7b, 100 },
    { 0x78f6dc78, 98 },
    { 0xc206dc81, 61 },
    { 0xfee6dc77, 87 },
    { 0, 0 },
    { 0xc846dc84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3576dc88, 95 },
    { 0xa396dc89, 23 },
    { 0xedb6dc8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf436dc8f, 76 },
    { 0x1606dc90, 23 },
    { 0x526dc91, 33 },
    { 0xb2d6dc92, 76 },
    { 0xe216dc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa96dc98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776dc9c, 76 },
    { 0x4646dc9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf6dca2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc076dca9, 33 },
    { 0, 0 },
    { 0xa426dcab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2356dcae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad26dcb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06dcb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46dcbe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8336dcc3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4166dcc6, 43 },
    { 0xe0e6dcc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a6dcca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6dcd0, 34 },
    { 0x3c6dcd0, 95 },
    { 0xf616dcd0, 95 },
    { 0xa746dcd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b6dcd7, 98 },
    { 0, 0 },
    { 0xbb96dcd9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856dcdd, 43 },
    { 0, 0 },
    { 0x9d6dcdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8716dce2, 23 },
    { 0, 0 },
    { 0x98d6dce4, 61 },
    { 0xee36dce5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86dcec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276dcef, 87 },
    { 0, 0 },
    { 0xc3d6dcf1, 95 },
    { 0x1df6dcf2, 90 },
    { 0x66e6dcf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06dcf8, 67 },
    { 0x2c86dcf9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd966dcfd, 98 },
    { 0x1786dcfd, 90 },
    { 0, 0 },
    { 0x2036dd00, 95 },
    { 0x4766dd00, 33 },
    { 0x76dd01, 67 },
    { 0xe66dd03, 33 },
    { 0x7246dd03, 23 },
    { 0xb06dd05, 90 },
    { 0, 0 },
    { 0x1126dd07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586dd0d, 90 },
    { 0, 0 },
    { 0x8056dd0f, 67 },
    { 0x8516dd10, 65 },
    { 0, 0 },
    { 0x9c76dd12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106dd1c, 52 },
    { 0xa8a6dd1d, 90 },
    { 0xb606dd1e, 90 },
    { 0xd7d6dd1c, 95 },
    { 0x7316dd20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc6dd2b, 66 },
    { 0xdd06dd2c, 76 },
    { 0x44c6dd2d, 75 },
    { 0xe3d6dd2d, 76 },
    { 0xad36dd2f, 91 },
    { 0x46c6dd2f, 55 },
    { 0, 0 },
    { 0xd0d6dd32, 23 },
    { 0x7836dd32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc336dd38, 90 },
    { 0, 0 },
    { 0x3a56dd3a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x5576dd3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796dd40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16dd43, 23 },
    { 0x51b6dd43, 46 },
    { 0xfe46dd45, 84 },
    { 0xea56dd45, 55 },
    { 0xdc76dd47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe336dd4e, 76 },
    { 0xa16dd4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde6dd56, 61 },
    { 0, 0 },
    { 0x1226dd58, 33 },
    { 0x2666dd59, 95 },
    { 0xbf96dd58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7596dd62, 61 },
    { 0x2e66dd62, 46 },
    { 0, 0 },
    { 0x5d86dd65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f6dd68, 33 },
    { 0, 0 },
    { 0x5146dd6a, 61 },
    { 0x1746dd6a, 55 },
    { 0, 0 },
    { 0x1206dd6d, 23 },
    { 0xe2b6dd6d, 43 },
    { 0, 0 },
    { 0xdb6dd70, 33 },
    { 0xf906dd71, 17 },
    { 0x4796dd71, 52 },
    { 0xf486dd71, 76 },
    { 0x2346dd74, 33 },
    { 0x1026dd73, 84 },
    { 0xcca6dd76, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a6dd7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c06dd83, 90 },
    { 0x2236dd84, 43 },
    { 0x6296dd84, 76 },
    { 0x82b6dd86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1156dd8e, 61 },
    { 0xb1b6dd8f, 76 },
    { 0x99f6dd90, 91 },
    { 0, 0 },
    { 0x3036dd92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c6dd95, 61 },
    { 0, 0 },
    { 0xf0a6dd97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb576dd9a, 43 },
    { 0x3146dd9a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8116dda2, 87 },
    { 0x7286dda3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f6dda7, 76 },
    { 0x4376dda7, 98 },
    { 0, 0 },
    { 0xc7d6ddaa, 80 },
    { 0xd386ddaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2846ddae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9006ddb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b6ddb7, 81 },
    { 0xed86ddb8, 61 },
    { 0, 0 },
    { 0x71d6ddba, 85 },
    { 0x27f6ddba, 90 },
    { 0x4646ddbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6ddc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad16ddca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2486ddd1, 76 },
    { 0xc56ddd1, 99 },
    { 0, 0 },
    { 0xf366ddd4, 61 },
    { 0, 0 },
    { 0x4ac6ddd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec36ddda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb316dde1, 55 },
    { 0x8fe6dde2, 98 },
    { 0xd66dde2, 55 },
    { 0xda96dde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed76ddea, 90 },
    { 0x8426ddeb, 98 },
    { 0x89f6ddec, 81 },
    { 0xed56ddeb, 33 },
    { 0x6886ddeb, 90 },
    { 0x2716ddec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd6ddf2, 23 },
    { 0xdd96ddf3, 46 },
    { 0xf376ddf4, 39 },
    { 0, 0 },
    { 0xae66ddf6, 43 },
    { 0, 0 },
    { 0x266ddf8, 90 },
    { 0xb2e6ddf8, 55 },
    { 0xf8b6ddfa, 34 },
    { 0x23c6ddfa, 55 },
    { 0x90b6ddfc, 75 },
    { 0x1de6ddfd, 77 },
    { 0xdb76ddfd, 33 },
    { 0x3946ddfe, 90 },
    { 0xae86de00, 46 },
    { 0, 0 },
    { 0x2f56de02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb276de05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb66de08, 90 },
    { 0xfaa6de09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9406de10, 95 },
    { 0, 0 },
    { 0x94e6de12, 76 },
    { 0x1836de12, 46 },
    { 0, 0 },
    { 0xa4d6de15, 61 },
    { 0, 0 },
    { 0x5eb6de17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe526de1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a6de21, 95 },
    { 0x5586de22, 23 },
    { 0x8006de23, 23 },
    { 0xa066de24, 34 },
    { 0xc636de24, 33 },
    { 0x5066de25, 23 },
    { 0x55c6de27, 23 },
    { 0x7776de27, 61 },
    { 0, 0 },
    { 0x8ab6de2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886de2d, 33 },
    { 0, 0 },
    { 0xc836de2f, 101 },
    { 0xe6f6de30, 76 },
    { 0xd646de2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5876de34, 95 },
    { 0, 0 },
    { 0xf36de36, 40 },
    { 0x1dd6de36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6de3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb46de40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6de44, 61 },
    { 0, 0 },
    { 0x6e76de46, 61 },
    { 0, 0 },
    { 0xa9c6de48, 95 },
    { 0xb316de49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c6de4c, 95 },
    { 0xc6f6de4d, 84 },
    { 0xbe26de4d, 34 },
    { 0x7a16de4d, 61 },
    { 0, 0 },
    { 0x4806de51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f96de54, 23 },
    { 0x4296de55, 99 },
    { 0x96a6de56, 33 },
    { 0x3456de57, 34 },
    { 0x8f36de57, 90 },
    { 0x6396de58, 95 },
    { 0xcd56de56, 98 },
    { 0, 0 },
    { 0xf46de5c, 32 },
    { 0xf4c6de5c, 90 },
    { 0xad26de5d, 23 },
    { 0, 0 },
    { 0x4316de60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb706de63, 61 },
    { 0x13b6de64, 95 },
    { 0x9ec6de65, 90 },
    { 0xf536de66, 61 },
    { 0xbbf6de66, 98 },
    { 0x2756de64, 61 },
    { 0x6a46de69, 95 },
    { 0x5c26de6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9976de71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906de77, 77 },
    { 0, 0 },
    { 0x17d6de79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176de7d, 99 },
    { 0, 0 },
    { 0x2aa6de7f, 68 },
    { 0xcaf6de7f, 90 },
    { 0x4676de80, 90 },
    { 0xf7e6de82, 95 },
    { 0x9196de82, 95 },
    { 0x7f36de82, 95 },
    { 0xe956de85, 46 },
    { 0, 0 },
    { 0xebe6de87, 90 },
    { 0, 0 },
    { 0x1816de89, 98 },
    { 0x21b6de8a, 33 },
    { 0xaa26de8b, 85 },
    { 0, 0 },
    { 0xc086de8d, 40 },
    { 0x7196de8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56de97, 98 },
    { 0, 0 },
    { 0x5b36de99, 87 },
    { 0x3bc6de99, 98 },
    { 0x5ae6de9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236de9f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386deaa, 90 },
    { 0xeef6deab, 55 },
    { 0, 0 },
    { 0x4946dead, 95 },
    { 0xcd6dead, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab6deb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c96debb, 61 },
    { 0x6916debc, 61 },
    { 0xc956debb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956dec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6dec6, 67 },
    { 0xd616dec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516decd, 61 },
    { 0, 0 },
    { 0xfec6decf, 98 },
    { 0, 0 },
    { 0x3686ded1, 95 },
    { 0x6996ded2, 23 },
    { 0x2a66ded2, 87 },
    { 0, 0 },
    { 0x8f96ded5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6ded8, 49 },
    { 0x3c26ded8, 91 },
    { 0, 0 },
    { 0x8486dedb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf836dee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x916dee5, 23 },
    { 0x3416dee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f76deea, 90 },
    { 0, 0 },
    { 0xf7d6deec, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe406deef, 33 },
    { 0x9f16def0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe296def6, 61 },
    { 0xd026def7, 34 },
    { 0xaa6def8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d06defe, 61 },
    { 0x3576defe, 55 },
    { 0x3536df00, 52 },
    { 0xc496df01, 90 },
    { 0x8f06defe, 84 },
    { 0, 0 },
    { 0x59f6df04, 77 },
    { 0, 0 },
    { 0x12a6df06, 68 },
    { 0, 0 },
    { 0xc016df08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd6df0d, 23 },
    { 0, 0 },
    { 0xb226df0f, 98 },
    { 0x1bd6df10, 84 },
    { 0x7ae6df10, 81 },
    { 0xc56df12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2106df15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd6df19, 33 },
    { 0, 0 },
    { 0xbeb6df1b, 23 },
    { 0x6d66df1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6df22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5536df29, 88 },
    { 0xc126df2a, 87 },
    { 0, 0 },
    { 0xa166df2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd6df36, 61 },
    { 0, 0 },
    { 0x1bc6df38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1996df3d, 33 },
    { 0xc426df3e, 87 },
    { 0x9156df3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e46df42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab66df45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126df4a, 90 },
    { 0x2d96df4b, 66 },
    { 0xcb36df4c, 56 },
    { 0x1956df4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f86df53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6df5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e6df5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566df61, 23 },
    { 0, 0 },
    { 0x98b6df63, 69 },
    { 0xf616df64, 46 },
    { 0x60c6df65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb166df6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2346df74, 95 },
    { 0x7886df75, 23 },
    { 0xc3b6df76, 23 },
    { 0xeee6df77, 43 },
    { 0, 0 },
    { 0x3ea6df79, 23 },
    { 0xc8d6df7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad6df7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c6df89, 33 },
    { 0, 0 },
    { 0x9a66df8b, 81 },
    { 0x4ba6df8c, 76 },
    { 0x8896df8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f6df94, 90 },
    { 0, 0 },
    { 0x8156df96, 98 },
    { 0xcbf6df97, 17 },
    { 0xfe16df97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe896df9b, 23 },
    { 0, 0 },
    { 0x82f6df9d, 33 },
    { 0xfa46df9e, 91 },
    { 0x1426df9d, 100 },
    { 0x27f6dfa0, 95 },
    { 0xed16dfa1, 23 },
    { 0x4046dfa2, 68 },
    { 0x1d36dfa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c6dfa6, 76 },
    { 0x5cf6dfa7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc76dfad, 90 },
    { 0xa46dfae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa776dfb4, 91 },
    { 0x95c6dfb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a6dfb8, 90 },
    { 0, 0 },
    { 0xe796dfba, 95 },
    { 0, 0 },
    { 0x8cc6dfbc, 23 },
    { 0, 0 },
    { 0x95a6dfbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c56dfc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b36dfc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e56dfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe616dfd1, 91 },
    { 0xebb6dfd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f36dfd7, 64 },
    { 0x236dfd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc6dfdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa26dfde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b16dfe8, 95 },
    { 0xd246dfe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c6dfec, 76 },
    { 0xb0c6dfed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b16dff5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9786dff8, 46 },
    { 0xb6d6dff8, 33 },
    { 0, 0 },
    { 0x97a6dffb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966dffe, 33 },
    { 0xb5d6dfff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf6e007, 46 },
    { 0, 0 },
    { 0xc826e009, 91 },
    { 0x55b6e00a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe146e010, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b6e014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5716e017, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276e01f, 90 },
    { 0xeec6e020, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9546e02a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa86e02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8526e032, 95 },
    { 0xda66e033, 90 },
    { 0xb346e033, 33 },
    { 0xea96e035, 33 },
    { 0x46f6e036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa696e03c, 88 },
    { 0x5556e03c, 33 },
    { 0xfc46e03e, 23 },
    { 0xdf36e03d, 23 },
    { 0xb5b6e040, 34 },
    { 0, 0 },
    { 0xce56e042, 76 },
    { 0, 0 },
    { 0x1f46e044, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b6e047, 34 },
    { 0, 0 },
    { 0x44e6e049, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc76e04c, 43 },
    { 0, 0 },
    { 0x8fd6e04e, 33 },
    { 0x99e6e04f, 95 },
    { 0x3246e050, 33 },
    { 0x226e051, 95 },
    { 0x7ac6e04f, 34 },
    { 0xd896e053, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126e056, 23 },
    { 0x7376e057, 52 },
    { 0, 0 },
    { 0x97e6e059, 34 },
    { 0x3356e05a, 92 },
    { 0x4556e05b, 90 },
    { 0, 0 },
    { 0xf906e05d, 90 },
    { 0, 0 },
    { 0x30e6e05f, 61 },
    { 0, 0 },
    { 0x5196e061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e065, 52 },
    { 0, 0 },
    { 0xefc6e067, 61 },
    { 0x6136e068, 100 },
    { 0x956e069, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e06c, 90 },
    { 0x3e56e06c, 55 },
    { 0xaa46e06e, 23 },
    { 0xf6c6e06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56e079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7856e07c, 52 },
    { 0, 0 },
    { 0x9936e07e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd46e08a, 33 },
    { 0x9716e08a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a6e08e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106e098, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846e0a0, 34 },
    { 0x54d6e0a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af6e0a5, 90 },
    { 0, 0 },
    { 0x5e46e0a7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a6e0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f96e0b4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a6e0b9, 33 },
    { 0xeb6e0b9, 23 },
    { 0xfb36e0b9, 33 },
    { 0x9186e0bb, 90 },
    { 0, 0 },
    { 0x7db6e0be, 93 },
    { 0xaf36e0be, 90 },
    { 0xd56e0c0, 52 },
    { 0x46e6e0c1, 55 },
    { 0xa3f6e0c2, 90 },
    { 0xf206e0c1, 43 },
    { 0x3226e0c4, 46 },
    { 0x926e0c5, 84 },
    { 0x77e6e0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7926e0c9, 98 },
    { 0x51f6e0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c6e0ce, 61 },
    { 0x6386e0cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a56e0d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f6e0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b56e0dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e6e0e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56e0e4, 76 },
    { 0xf276e0e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6e0ee, 85 },
    { 0, 0 },
    { 0xe636e0f0, 88 },
    { 0xc476e0f1, 43 },
    { 0, 0 },
    { 0x7976e0f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb256e0fd, 43 },
    { 0, 0 },
    { 0xf126e0ff, 33 },
    { 0, 0 },
    { 0x1186e101, 61 },
    { 0xa386e102, 93 },
    { 0x5036e103, 95 },
    { 0, 0 },
    { 0x34a6e105, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6e10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be6e113, 52 },
    { 0, 0 },
    { 0xe7b6e115, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e56e119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b86e11e, 46 },
    { 0xbda6e11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9686e122, 98 },
    { 0x5f06e123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6e126, 67 },
    { 0x2016e127, 43 },
    { 0, 0 },
    { 0x706e129, 87 },
    { 0, 0 },
    { 0x6756e12b, 52 },
    { 0x7da6e12b, 98 },
    { 0, 0 },
    { 0x4fc6e12e, 68 },
    { 0xf716e12f, 90 },
    { 0xa8e6e130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae66e134, 76 },
    { 0, 0 },
    { 0xc3b6e136, 91 },
    { 0, 0 },
    { 0x44d6e138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd156e13b, 76 },
    { 0x90d6e13b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee6e13f, 95 },
    { 0x6c76e13f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9356e144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956e147, 90 },
    { 0x5206e148, 90 },
    { 0, 0 },
    { 0xc8d6e14a, 55 },
    { 0, 0 },
    { 0x9496e14c, 76 },
    { 0xcc86e14c, 23 },
    { 0, 0 },
    { 0x81d6e14f, 61 },
    { 0x4fe6e150, 61 },
    { 0x5d56e151, 23 },
    { 0xc2c6e14f, 78 },
    { 0x5ab6e153, 93 },
    { 0xb0d6e154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c6e158, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f6e15d, 95 },
    { 0, 0 },
    { 0xa306e15f, 93 },
    { 0x2c66e160, 34 },
    { 0x9c6e160, 68 },
    { 0x6726e160, 55 },
    { 0, 0 },
    { 0x18f6e164, 69 },
    { 0xfc66e164, 90 },
    { 0x6116e166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796e16a, 34 },
    { 0, 0 },
    { 0x9896e16c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2176e16f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd86e172, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d36e177, 90 },
    { 0x6bf6e177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6e17b, 61 },
    { 0x2856e17c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe826e180, 33 },
    { 0x5b56e181, 76 },
    { 0, 0 },
    { 0xbe36e183, 95 },
    { 0, 0 },
    { 0x3a86e185, 96 },
    { 0x9936e186, 95 },
    { 0x10a6e187, 33 },
    { 0xd056e188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe566e18d, 77 },
    { 0x8656e18e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8726e193, 98 },
    { 0xfde6e193, 61 },
    { 0, 0 },
    { 0x6866e196, 90 },
    { 0, 0 },
    { 0xd756e198, 98 },
    { 0x9696e199, 95 },
    { 0x5446e199, 23 },
    { 0xba6e19b, 78 },
    { 0x5826e19c, 90 },
    { 0xd876e19a, 99 },
    { 0x4006e19d, 90 },
    { 0x6426e19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1706e1a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d66e1a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc76e1a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6736e1ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7696e1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc436e1b7, 43 },
    { 0x5016e1b8, 95 },
    { 0xa4a6e1b9, 23 },
    { 0x21a6e1ba, 99 },
    { 0x6506e1ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8446e1c0, 90 },
    { 0, 0 },
    { 0x52d6e1c2, 34 },
    { 0, 0 },
    { 0xa556e1c4, 61 },
    { 0x6746e1c5, 61 },
    { 0, 0 },
    { 0x9cc6e1c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6e1ca, 91 },
    { 0xc376e1ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd516e1ce, 52 },
    { 0xb926e1cf, 33 },
    { 0x1586e1d0, 64 },
    { 0x4b96e1d0, 93 },
    { 0x3816e1d0, 95 },
    { 0x85b6e1d2, 90 },
    { 0xd6d6e1d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e06e1d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4826e1dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2556e1df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a86e1e2, 76 },
    { 0x7876e1e3, 67 },
    { 0xf6e6e1e2, 65 },
    { 0x6ed6e1e2, 90 },
    { 0x55a6e1e4, 98 },
    { 0xad86e1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76e1ee, 61 },
    { 0x1a6e1ef, 88 },
    { 0x79d6e1ee, 93 },
    { 0, 0 },
    { 0x8fe6e1f2, 61 },
    { 0xfbd6e1f2, 76 },
    { 0, 0 },
    { 0x22b6e1f5, 98 },
    { 0, 0 },
    { 0xe9b6e1f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5216e1fb, 90 },
    { 0, 0 },
    { 0xcd96e1fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b96e202, 56 },
    { 0, 0 },
    { 0xac6e204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc766e207, 90 },
    { 0xa226e208, 43 },
    { 0, 0 },
    { 0x7cb6e20a, 88 },
    { 0x8f96e20a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b06e212, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8726e217, 33 },
    { 0x4f16e217, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6e21c, 87 },
    { 0xd286e21d, 34 },
    { 0xd376e21e, 55 },
    { 0, 0 },
    { 0x8056e220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d6e22a, 55 },
    { 0x3e96e22b, 43 },
    { 0x4f76e22b, 90 },
    { 0x3dd6e22c, 55 },
    { 0x3526e22d, 90 },
    { 0x9da6e22d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496e235, 98 },
    { 0x2706e235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6e239, 23 },
    { 0x22e6e239, 88 },
    { 0x3c16e23b, 98 },
    { 0xd196e239, 78 },
    { 0x2d96e23d, 90 },
    { 0x15a6e23e, 33 },
    { 0x16d6e23f, 90 },
    { 0xb46e240, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5056e24b, 98 },
    { 0x3226e24c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad66e251, 67 },
    { 0xf856e251, 90 },
    { 0x2db6e253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496e256, 61 },
    { 0x1336e257, 87 },
    { 0x65e6e257, 34 },
    { 0xc166e259, 98 },
    { 0, 0 },
    { 0xf276e25b, 69 },
    { 0xdc36e25c, 36 },
    { 0x6356e25c, 98 },
    { 0xba86e25d, 91 },
    { 0, 0 },
    { 0x5666e260, 98 },
    { 0x5a36e260, 95 },
    { 0x6486e262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c46e266, 33 },
    { 0x7376e266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276e26f, 98 },
    { 0xe8d6e270, 99 },
    { 0x8326e271, 66 },
    { 0xd816e270, 81 },
    { 0x9276e270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf266e276, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6e27d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1996e282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc6e28a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf256e28e, 33 },
    { 0x98f6e28e, 93 },
    { 0xf796e28e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86e298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c66e29b, 98 },
    { 0x4216e29c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386e2a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf6e2a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a56e2ad, 95 },
    { 0, 0 },
    { 0xffd6e2af, 99 },
    { 0, 0 },
    { 0x52f6e2b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e2b4, 90 },
    { 0xa7e6e2b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc136e2b9, 61 },
    { 0, 0 },
    { 0xc056e2bb, 93 },
    { 0, 0 },
    { 0x9386e2bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e2cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d6e2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6486e2d2, 43 },
    { 0xc096e2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e2df, 67 },
    { 0xb536e2df, 90 },
    { 0x9876e2e0, 33 },
    { 0x3cd6e2e2, 43 },
    { 0x4156e2e1, 55 },
    { 0, 0 },
    { 0x1cb6e2e5, 76 },
    { 0x1bb6e2e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996e2e9, 33 },
    { 0xe986e2ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6e2f0, 68 },
    { 0, 0 },
    { 0xd006e2f2, 98 },
    { 0x1cc6e2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde26e2f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6e2fa, 48 },
    { 0xf4e6e2fa, 33 },
    { 0xdf46e2fc, 90 },
    { 0x4ec6e2fa, 33 },
    { 0, 0 },
    { 0x9ea6e2ff, 85 },
    { 0x8cb6e300, 98 },
    { 0, 0 },
    { 0x7746e302, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6e305, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f6e308, 52 },
    { 0x27e6e309, 61 },
    { 0, 0 },
    { 0x74b6e30b, 90 },
    { 0xc9e6e30c, 66 },
    { 0xbb66e30c, 33 },
    { 0, 0 },
    { 0xfba6e30f, 84 },
    { 0x7c06e310, 88 },
    { 0xdf86e30f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66e315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8996e31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e6e320, 61 },
    { 0x356e321, 61 },
    { 0x5356e321, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6e32b, 66 },
    { 0xf156e32c, 43 },
    { 0xf926e32b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4126e331, 55 },
    { 0xcbe6e331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe26e336, 90 },
    { 0x1c6e336, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126e33a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626e341, 90 },
    { 0x3a96e341, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb356e350, 55 },
    { 0xe496e350, 95 },
    { 0xbad6e351, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6e361, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a66e368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea66e36b, 100 },
    { 0x1ed6e36c, 55 },
    { 0x6186e36c, 61 },
    { 0x9c06e36d, 98 },
    { 0xcce6e36e, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456e379, 99 },
    { 0xcaf6e379, 43 },
    { 0x8776e37a, 84 },
    { 0x8526e37b, 84 },
    { 0x4076e37a, 98 },
    { 0x59c6e37e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb46e382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb56e38a, 95 },
    { 0, 0 },
    { 0x7d6e38c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236e38f, 98 },
    { 0, 0 },
    { 0xac06e391, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16e396, 83 },
    { 0, 0 },
    { 0xdbb6e398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e6e39c, 33 },
    { 0xb746e39c, 66 },
    { 0, 0 },
    { 0x4d56e39f, 98 },
    { 0xe06e3a0, 34 },
    { 0xdff6e3a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd6e3a4, 43 },
    { 0, 0 },
    { 0x12b6e3a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6e3ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066e3b0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b6e3b3, 87 },
    { 0, 0 },
    { 0x1636e3b5, 33 },
    { 0x5f96e3b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f6e3bb, 100 },
    { 0x1e86e3bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c6e3c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16e3c7, 90 },
    { 0x96f6e3c8, 88 },
    { 0x20e6e3c7, 99 },
    { 0x5946e3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d76e3d2, 46 },
    { 0x7fd6e3d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd46e3d7, 61 },
    { 0, 0 },
    { 0x8d26e3d9, 90 },
    { 0x7306e3da, 76 },
    { 0, 0 },
    { 0xc866e3dc, 96 },
    { 0x8ea6e3dc, 85 },
    { 0xcb96e3de, 87 },
    { 0x9f26e3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd6e3e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd6e3e9, 23 },
    { 0, 0 },
    { 0x7446e3eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e6e3fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836e3fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6e403, 55 },
    { 0, 0 },
    { 0x3fa6e405, 99 },
    { 0x766e405, 66 },
    { 0x7646e406, 75 },
    { 0xefe6e406, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6e40b, 33 },
    { 0x2886e40c, 46 },
    { 0x806e40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c6e410, 33 },
    { 0x82a6e411, 77 },
    { 0xe976e411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef66e415, 23 },
    { 0xae46e415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc576e422, 69 },
    { 0, 0 },
    { 0x9616e424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd226e42b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6e42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7306e436, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76e43d, 93 },
    { 0x9a36e43d, 100 },
    { 0x5ec6e43f, 95 },
    { 0x6fa6e440, 33 },
    { 0xe116e440, 98 },
    { 0x9f66e440, 34 },
    { 0xb9d6e443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6e44a, 43 },
    { 0x9be6e44a, 52 },
    { 0x5f66e44c, 95 },
    { 0xca06e44c, 90 },
    { 0xc436e44e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f6e453, 99 },
    { 0xc736e454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6e45f, 42 },
    { 0x57e6e460, 95 },
    { 0x81f6e45f, 99 },
    { 0, 0 },
    { 0xd826e463, 33 },
    { 0, 0 },
    { 0xde96e465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf636e469, 23 },
    { 0x5646e46a, 61 },
    { 0xac26e46b, 95 },
    { 0, 0 },
    { 0x1046e46d, 95 },
    { 0xb5e6e46e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e06e471, 66 },
    { 0, 0 },
    { 0x98e6e473, 46 },
    { 0x18d6e474, 87 },
    { 0xd076e474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6e478, 61 },
    { 0, 0 },
    { 0xa246e47a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e6e488, 95 },
    { 0xa9b6e489, 90 },
    { 0, 0 },
    { 0x4046e48b, 33 },
    { 0xda96e48c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86e48f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e492, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6e495, 91 },
    { 0, 0 },
    { 0xa106e497, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6e49a, 97 },
    { 0x6e56e49b, 61 },
    { 0xda56e49a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc326e4a0, 90 },
    { 0xa516e4a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a6e4a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e4aa, 95 },
    { 0x5196e4aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176e4b4, 34 },
    { 0, 0 },
    { 0x50d6e4b6, 34 },
    { 0x2576e4b7, 85 },
    { 0xf2f6e4b7, 61 },
    { 0x6456e4b8, 43 },
    { 0xac36e4ba, 98 },
    { 0x63b6e4bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b76e4c1, 66 },
    { 0x726e4c2, 98 },
    { 0x8bd6e4c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b6e4c7, 53 },
    { 0xc8c6e4c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6e4cb, 77 },
    { 0xfb56e4cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc6e4cf, 33 },
    { 0xde6e4d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e76e4d3, 55 },
    { 0x7886e4d4, 85 },
    { 0, 0 },
    { 0x65a6e4d6, 79 },
    { 0x1456e4d7, 90 },
    { 0xbc56e4d7, 61 },
    { 0x7486e4d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96e4df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6e4e6, 85 },
    { 0xa9f6e4e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f06e4ea, 76 },
    { 0x7106e4eb, 61 },
    { 0x12c6e4ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd76e4f7, 66 },
    { 0x3556e4f8, 23 },
    { 0, 0 },
    { 0x6a96e4fa, 61 },
    { 0xa4a6e4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac6e4fe, 84 },
    { 0, 0 },
    { 0xb86e500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586e506, 61 },
    { 0, 0 },
    { 0x7a06e508, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b96e50b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876e510, 91 },
    { 0xc966e510, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e514, 61 },
    { 0x6076e515, 90 },
    { 0xadf6e514, 90 },
    { 0xf196e517, 84 },
    { 0x8fd6e517, 33 },
    { 0x36e519, 88 },
    { 0, 0 },
    { 0xd696e51b, 76 },
    { 0, 0 },
    { 0x4426e51d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996e528, 90 },
    { 0x70d6e529, 76 },
    { 0, 0 },
    { 0xdc86e52b, 91 },
    { 0, 0 },
    { 0x5626e52d, 61 },
    { 0x3976e52e, 23 },
    { 0x2036e52f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac6e532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf576e537, 61 },
    { 0x5c6e537, 33 },
    { 0xcb16e538, 95 },
    { 0x40a6e53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3306e53e, 98 },
    { 0, 0 },
    { 0xa3a6e540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d6e543, 61 },
    { 0x49f6e544, 77 },
    { 0x9c06e544, 95 },
    { 0x4596e546, 55 },
    { 0x3b46e547, 67 },
    { 0x3a86e548, 61 },
    { 0x74e6e549, 34 },
    { 0, 0 },
    { 0x4546e54b, 34 },
    { 0x9586e54b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d6e551, 87 },
    { 0x316e552, 83 },
    { 0, 0 },
    { 0x3996e554, 87 },
    { 0x41c6e555, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5816e558, 94 },
    { 0x16a6e558, 53 },
    { 0x7cf6e55a, 61 },
    { 0x8936e55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f6e561, 61 },
    { 0xe346e562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4116e566, 55 },
    { 0x5c76e567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466e56b, 90 },
    { 0xebe6e56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa246e570, 98 },
    { 0x946e571, 83 },
    { 0xc4c6e571, 67 },
    { 0x3146e571, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3926e57b, 95 },
    { 0x7db6e57c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc6e581, 23 },
    { 0xaa16e581, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3706e589, 75 },
    { 0, 0 },
    { 0xf736e58b, 61 },
    { 0, 0 },
    { 0x43e6e58d, 95 },
    { 0, 0 },
    { 0x2776e58f, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a6e594, 33 },
    { 0x7cb6e595, 90 },
    { 0x6056e595, 90 },
    { 0x1966e594, 61 },
    { 0, 0 },
    { 0x2906e599, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1416e59c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa6e5a1, 61 },
    { 0x9d36e5a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6e5a7, 78 },
    { 0, 0 },
    { 0xca46e5a9, 90 },
    { 0x35b6e5aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e5b3, 61 },
    { 0, 0 },
    { 0x31a6e5b5, 33 },
    { 0x9816e5b5, 61 },
    { 0x56e5b7, 61 },
    { 0xe96e5b7, 56 },
    { 0x366e5b8, 98 },
    { 0x61f6e5ba, 88 },
    { 0xf126e5b7, 90 },
    { 0x5af6e5bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1766e5bf, 98 },
    { 0, 0 },
    { 0xaa66e5c1, 90 },
    { 0x6026e5c2, 34 },
    { 0, 0 },
    { 0x686e5c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6136e5d0, 43 },
    { 0x5976e5d1, 33 },
    { 0x5cf6e5d1, 38 },
    { 0, 0 },
    { 0xe006e5d4, 61 },
    { 0, 0 },
    { 0x3db6e5d6, 66 },
    { 0xb326e5d6, 90 },
    { 0x5046e5d7, 85 },
    { 0xf826e5d8, 100 },
    { 0, 0 },
    { 0x4706e5db, 98 },
    { 0x59b6e5db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a6e5df, 88 },
    { 0xba66e5e0, 96 },
    { 0, 0 },
    { 0x51b6e5e2, 90 },
    { 0xb426e5e3, 69 },
    { 0x5366e5e4, 100 },
    { 0xafa6e5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e6e5ee, 99 },
    { 0, 0 },
    { 0x8776e5f0, 33 },
    { 0xb066e5f1, 33 },
    { 0, 0 },
    { 0xa516e5f3, 23 },
    { 0, 0 },
    { 0xa206e5f5, 52 },
    { 0x6916e5f6, 61 },
    { 0, 0 },
    { 0xb96e5f8, 33 },
    { 0x2c36e5f9, 100 },
    { 0, 0 },
    { 0x1f56e5fb, 95 },
    { 0, 0 },
    { 0xd446e5fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a6e600, 61 },
    { 0, 0 },
    { 0x566e602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a6e60c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86e612, 98 },
    { 0x5646e613, 61 },
    { 0x2696e614, 90 },
    { 0x976e615, 61 },
    { 0xd26e613, 46 },
    { 0xcee6e617, 61 },
    { 0xfe86e617, 43 },
    { 0xd4d6e619, 43 },
    { 0x66d6e61a, 90 },
    { 0x5db6e61a, 55 },
    { 0xc896e61b, 95 },
    { 0xaa76e61d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x196e620, 90 },
    { 0x5526e621, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe46e62a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1366e62e, 55 },
    { 0x7446e62f, 23 },
    { 0x7eb6e630, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86e634, 86 },
    { 0xca06e635, 90 },
    { 0x4436e635, 23 },
    { 0xbf6e637, 43 },
    { 0x6536e636, 90 },
    { 0x2df6e639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5186e63c, 66 },
    { 0, 0 },
    { 0x38f6e63e, 68 },
    { 0, 0 },
    { 0x90c6e640, 96 },
    { 0xb856e640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26e644, 55 },
    { 0, 0 },
    { 0x3ee6e646, 95 },
    { 0, 0 },
    { 0x4446e648, 42 },
    { 0x6e86e648, 95 },
    { 0xaaf6e649, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626e64d, 87 },
    { 0x5106e64d, 23 },
    { 0, 0 },
    { 0xc746e650, 23 },
    { 0x6726e651, 98 },
    { 0x5006e651, 98 },
    { 0x4526e653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36e656, 34 },
    { 0, 0 },
    { 0x7a66e658, 95 },
    { 0xb3d6e659, 98 },
    { 0x3cb6e65a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff66e65e, 76 },
    { 0, 0 },
    { 0x1656e660, 99 },
    { 0, 0 },
    { 0x5506e662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436e665, 65 },
    { 0x2a46e665, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e6e66f, 76 },
    { 0xc746e670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6e674, 76 },
    { 0, 0 },
    { 0x456e676, 98 },
    { 0, 0 },
    { 0xc66e678, 95 },
    { 0x1dc6e678, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3856e681, 23 },
    { 0x1056e682, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816e68b, 61 },
    { 0x5136e68c, 98 },
    { 0, 0 },
    { 0x296e68e, 27 },
    { 0x1df6e68f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716e695, 55 },
    { 0, 0 },
    { 0xc386e697, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06e69a, 34 },
    { 0xa556e69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b96e69e, 61 },
    { 0xa896e69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa16e6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66e6b2, 90 },
    { 0x6236e6b3, 23 },
    { 0x5176e6b4, 43 },
    { 0xb296e6b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3286e6bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd786e6c1, 43 },
    { 0xd666e6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf6e6c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6e6cd, 85 },
    { 0x6d86e6cd, 93 },
    { 0x8eb6e6ce, 23 },
    { 0x86e6d0, 33 },
    { 0, 0 },
    { 0xa226e6d2, 61 },
    { 0, 0 },
    { 0x1d46e6d4, 34 },
    { 0, 0 },
    { 0x4196e6d6, 98 },
    { 0x4426e6d7, 23 },
    { 0xb836e6d7, 46 },
    { 0xe326e6d9, 23 },
    { 0xff26e6d9, 23 },
    { 0x3326e6d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e16e6e4, 61 },
    { 0, 0 },
    { 0x8fa6e6e6, 61 },
    { 0x5056e6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8426e6ea, 61 },
    { 0, 0 },
    { 0x9856e6ec, 90 },
    { 0x85f6e6ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d66e6f8, 98 },
    { 0x4826e6f9, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b6e6ff, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9756e705, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d6e709, 95 },
    { 0, 0 },
    { 0x3e06e70b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee6e70f, 17 },
    { 0x5206e70f, 23 },
    { 0x6556e710, 95 },
    { 0xbc86e710, 34 },
    { 0xb716e713, 95 },
    { 0xeb26e714, 90 },
    { 0xc746e715, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf86e718, 56 },
    { 0xd8c6e718, 95 },
    { 0xbe76e718, 98 },
    { 0xd496e719, 90 },
    { 0x39b6e71c, 75 },
    { 0xc8a6e71d, 46 },
    { 0x3a36e71d, 53 },
    { 0xcc06e71f, 91 },
    { 0xcf86e71f, 61 },
    { 0x52b6e71d, 56 },
    { 0, 0 },
    { 0xe5c6e723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d6e727, 96 },
    { 0x7016e728, 61 },
    { 0x8cf6e729, 33 },
    { 0x9526e72a, 33 },
    { 0xf966e72a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6e72e, 90 },
    { 0xda66e72e, 33 },
    { 0, 0 },
    { 0x64b6e731, 98 },
    { 0x9876e732, 61 },
    { 0x2d66e732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6466e736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c6e73b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa846e73e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c96e742, 90 },
    { 0, 0 },
    { 0xc5c6e744, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4656e747, 90 },
    { 0x8786e748, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c6e74e, 90 },
    { 0, 0 },
    { 0xb0e6e750, 52 },
    { 0, 0 },
    { 0x6aa6e752, 87 },
    { 0, 0 },
    { 0x2656e754, 95 },
    { 0x1306e755, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc476e75b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf156e760, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf6e766, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26e76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4516e76d, 76 },
    { 0xb456e76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86e771, 46 },
    { 0, 0 },
    { 0x3076e773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e96e77b, 84 },
    { 0xbd96e77c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6e780, 90 },
    { 0x5a86e781, 33 },
    { 0xb3b6e782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5626e788, 87 },
    { 0xd5a6e788, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d36e78c, 43 },
    { 0x84a6e78d, 87 },
    { 0x8486e78e, 46 },
    { 0, 0 },
    { 0x4a96e790, 34 },
    { 0x2516e790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6e797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276e79a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f6e79e, 61 },
    { 0x99d6e79f, 98 },
    { 0x93b6e79f, 98 },
    { 0x67e6e7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc546e7a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2776e7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886e7b1, 46 },
    { 0xafa6e7b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2376e7ba, 69 },
    { 0x6c56e7bb, 90 },
    { 0xc036e7bc, 61 },
    { 0x2e96e7bd, 61 },
    { 0x17a6e7bd, 33 },
    { 0x46d6e7bf, 55 },
    { 0xeaf6e7bf, 43 },
    { 0x8936e7bf, 55 },
    { 0xa96e7be, 55 },
    { 0, 0 },
    { 0xe5a6e7c4, 43 },
    { 0, 0 },
    { 0x56c6e7c6, 23 },
    { 0xe096e7c7, 33 },
    { 0xbb06e7c8, 87 },
    { 0x29b6e7c8, 61 },
    { 0xd1a6e7ca, 95 },
    { 0xfb36e7ca, 76 },
    { 0x2ac6e7ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a6e7cf, 61 },
    { 0x4e56e7cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6e7d3, 43 },
    { 0x63a6e7d4, 19 },
    { 0, 0 },
    { 0x5326e7d6, 61 },
    { 0x2e26e7d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4036e7de, 61 },
    { 0, 0 },
    { 0x6c16e7e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda06e7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa6e7e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd486e7ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6e7f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4906e7f4, 52 },
    { 0xf766e7f5, 90 },
    { 0x1d36e7f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e6e7fb, 93 },
    { 0x3fb6e7fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c36e7ff, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a6e802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d6e80d, 23 },
    { 0x8286e80d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa186e812, 33 },
    { 0xc586e813, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd596e818, 98 },
    { 0x8c26e819, 95 },
    { 0xd1c6e81a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86e81d, 98 },
    { 0, 0 },
    { 0x1556e81f, 95 },
    { 0x3466e820, 90 },
    { 0x1e26e821, 95 },
    { 0x3986e822, 87 },
    { 0xfb36e822, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc6e826, 99 },
    { 0xc346e827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd346e82a, 46 },
    { 0x61e6e82b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6076e834, 33 },
    { 0x3ba6e835, 76 },
    { 0, 0 },
    { 0xd076e837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3456e83b, 91 },
    { 0x41d6e83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e844, 95 },
    { 0x2566e845, 61 },
    { 0xadd6e846, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d16e849, 58 },
    { 0, 0 },
    { 0x1056e84b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6e852, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6e86b, 33 },
    { 0x8a16e86b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2846e86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c6e873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e877, 23 },
    { 0xf946e878, 98 },
    { 0, 0 },
    { 0xb896e87a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe6e87d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b6e888, 33 },
    { 0xdb86e888, 51 },
    { 0x83b6e88a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7006e88f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1526e895, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6e89c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4966e8a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab6e8ab, 35 },
    { 0x8e06e8ab, 90 },
    { 0x376e8ab, 33 },
    { 0x7cb6e8ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b6e8b6, 90 },
    { 0x1716e8b7, 52 },
    { 0, 0 },
    { 0xed46e8b9, 55 },
    { 0xfb26e8b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa106e8bd, 91 },
    { 0x1576e8be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f6e8c5, 99 },
    { 0, 0 },
    { 0xd936e8c7, 64 },
    { 0, 0 },
    { 0xa066e8c9, 55 },
    { 0, 0 },
    { 0xcf96e8cb, 78 },
    { 0x2bd6e8cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326e8d1, 43 },
    { 0xad36e8d2, 87 },
    { 0xeb76e8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66e8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4526e8d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36e8de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe6e8e1, 98 },
    { 0xaae6e8e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6e8e5, 87 },
    { 0xec26e8e6, 46 },
    { 0, 0 },
    { 0x2af6e8e8, 95 },
    { 0, 0 },
    { 0x6e6e8ea, 55 },
    { 0x5876e8eb, 90 },
    { 0xe3d6e8eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396e8f1, 85 },
    { 0xd856e8f1, 67 },
    { 0x87d6e8f1, 95 },
    { 0, 0 },
    { 0x336e8f5, 95 },
    { 0x5f46e8f6, 67 },
    { 0xfc06e8f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2476e8fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f6e900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6e904, 67 },
    { 0x53e6e905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c06e909, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6e90e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a6e915, 33 },
    { 0x2806e916, 76 },
    { 0xd6b6e917, 100 },
    { 0x55c6e917, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec6e923, 90 },
    { 0x46e924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba16e927, 76 },
    { 0xca6e928, 81 },
    { 0, 0 },
    { 0x9bc6e92a, 60 },
    { 0, 0 },
    { 0x2106e92c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb6e930, 85 },
    { 0x24f6e930, 77 },
    { 0x93d6e932, 95 },
    { 0, 0 },
    { 0x3b36e934, 43 },
    { 0x7446e935, 80 },
    { 0, 0 },
    { 0x3d26e937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f6e93b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966e940, 55 },
    { 0, 0 },
    { 0xa816e942, 90 },
    { 0x4a36e943, 98 },
    { 0x7d96e944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9206e94d, 61 },
    { 0xb936e94d, 33 },
    { 0x49d6e94e, 90 },
    { 0x2936e94f, 67 },
    { 0, 0 },
    { 0x3fa6e952, 90 },
    { 0xec86e953, 23 },
    { 0, 0 },
    { 0x74b6e955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c86e958, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6e95b, 98 },
    { 0xb206e95c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e6e960, 90 },
    { 0, 0 },
    { 0x9f6e962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06e965, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2126e96a, 34 },
    { 0, 0 },
    { 0xb5f6e96c, 95 },
    { 0, 0 },
    { 0xfff6e96e, 33 },
    { 0x5616e96e, 91 },
    { 0x10d6e970, 33 },
    { 0x7d36e970, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b6e974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc6e977, 43 },
    { 0xa606e978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7156e97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca6e983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd846e986, 98 },
    { 0xfa46e986, 91 },
    { 0, 0 },
    { 0x2c36e989, 98 },
    { 0x6e6e98a, 77 },
    { 0x3546e98a, 99 },
    { 0x5576e98a, 43 },
    { 0xc1d6e98d, 34 },
    { 0x1926e98a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd6e993, 61 },
    { 0x5706e994, 43 },
    { 0x8b36e995, 33 },
    { 0xd3c6e996, 17 },
    { 0x86f6e995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8356e9a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d6e9a4, 63 },
    { 0x1aa6e9a5, 95 },
    { 0x12f6e9a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd046e9a9, 80 },
    { 0x3486e9aa, 61 },
    { 0, 0 },
    { 0x8776e9ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2706e9b2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5956e9b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6e9bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8866e9c0, 43 },
    { 0x3e76e9c1, 90 },
    { 0xc136e9c1, 98 },
    { 0x3d76e9c3, 43 },
    { 0x6a76e9c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6e9cb, 61 },
    { 0x6e6e9cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc6e9d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f6e9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff6e9de, 78 },
    { 0xed56e9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d6e9e3, 76 },
    { 0xdc26e9e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296e9ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6986e9fa, 98 },
    { 0xe856e9fb, 61 },
    { 0xe806e9fb, 65 },
    { 0xfd46e9fc, 77 },
    { 0, 0 },
    { 0x6306e9ff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6396ea06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86ea0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6ea10, 52 },
    { 0x8686ea11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1376ea15, 93 },
    { 0, 0 },
    { 0x8506ea17, 33 },
    { 0x1fc6ea18, 90 },
    { 0, 0 },
    { 0x9bf6ea1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7426ea22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8466ea2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e16ea30, 66 },
    { 0xbc96ea31, 66 },
    { 0xd446ea32, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126ea36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c6ea3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6ea3d, 61 },
    { 0x17e6ea3e, 99 },
    { 0xa2a6ea3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6ea43, 90 },
    { 0, 0 },
    { 0x866ea45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6ea48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa306ea4b, 90 },
    { 0, 0 },
    { 0x9126ea4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc6ea51, 98 },
    { 0x4e86ea52, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6ea58, 98 },
    { 0x2b66ea58, 52 },
    { 0xf6ea5a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c96ea5f, 99 },
    { 0xfdd6ea60, 88 },
    { 0x2d46ea5f, 96 },
    { 0x9cc6ea61, 33 },
    { 0, 0 },
    { 0xe4f6ea64, 90 },
    { 0xbd96ea65, 61 },
    { 0xc846ea66, 100 },
    { 0x8196ea67, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596ea6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826ea6e, 98 },
    { 0, 0 },
    { 0x36d6ea70, 38 },
    { 0x7af6ea71, 23 },
    { 0x6786ea72, 61 },
    { 0xea96ea73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc616ea79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb976ea7f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5646ea84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f6ea8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7266ea92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8206ea96, 98 },
    { 0x456ea97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876ea9c, 90 },
    { 0, 0 },
    { 0x9116ea9e, 43 },
    { 0xcbc6ea9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56eaa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6eaa9, 61 },
    { 0xcaf6eaa9, 75 },
    { 0x2916eaab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec6eaae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9136eab3, 33 },
    { 0, 0 },
    { 0x5266eab5, 90 },
    { 0x8d96eab5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386eabb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9726eac4, 19 },
    { 0x5b86eac4, 68 },
    { 0x8ec6eac6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb6eac9, 61 },
    { 0xfb66eaca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626eacd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1946ead0, 61 },
    { 0xba86ead1, 90 },
    { 0x9276ead0, 52 },
    { 0x2d66ead3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e36ead6, 61 },
    { 0xfb86ead7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe036eada, 99 },
    { 0x5ae6eada, 87 },
    { 0x59d6eadb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1006eae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b6eae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb926eaed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216eaf0, 75 },
    { 0x9266eaf1, 61 },
    { 0xc346eaf2, 53 },
    { 0xc9d6eaf2, 91 },
    { 0x5756eaf3, 36 },
    { 0x14c6eaf5, 90 },
    { 0x11f6eaf1, 34 },
    { 0xdea6eaf5, 95 },
    { 0x31d6eaf6, 33 },
    { 0xc806eaf9, 93 },
    { 0x27a6eaf9, 33 },
    { 0xc826eaf4, 90 },
    { 0x7aa6eafc, 98 },
    { 0x2f06eafd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2996eb00, 90 },
    { 0x97f6eb00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde76eb05, 98 },
    { 0x1be6eb06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a6eb09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636eb0e, 65 },
    { 0x8426eb0f, 93 },
    { 0, 0 },
    { 0xd886eb11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc86eb16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6eb1d, 90 },
    { 0, 0 },
    { 0x8116eb1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56eb33, 61 },
    { 0xb6f6eb34, 61 },
    { 0xd7d6eb34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd6eb3a, 85 },
    { 0xd1b6eb3b, 33 },
    { 0x13a6eb3c, 76 },
    { 0x36a6eb3c, 18 },
    { 0x5b66eb3e, 76 },
    { 0x4e66eb3f, 85 },
    { 0x62a6eb3c, 91 },
    { 0xf926eb3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd6eb44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe96eb4d, 87 },
    { 0xd7d6eb4d, 33 },
    { 0x7db6eb4f, 95 },
    { 0, 0 },
    { 0x40f6eb51, 85 },
    { 0xdd06eb52, 61 },
    { 0xc6b6eb51, 55 },
    { 0x806eb54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6eb57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4786eb5b, 33 },
    { 0xe5e6eb5c, 33 },
    { 0xf2b6eb5c, 93 },
    { 0xd376eb5e, 95 },
    { 0, 0 },
    { 0x8896eb60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406eb68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa236eb6d, 95 },
    { 0x74a6eb6e, 99 },
    { 0xe856eb6f, 98 },
    { 0x6246eb70, 23 },
    { 0x1af6eb70, 40 },
    { 0x47e6eb6f, 90 },
    { 0xe246eb72, 23 },
    { 0, 0 },
    { 0x5916eb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb776eb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb6eb81, 95 },
    { 0xe6c6eb82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6eb89, 33 },
    { 0xa886eb8a, 66 },
    { 0x5d36eb8b, 80 },
    { 0x9086eb8a, 33 },
    { 0x5e16eb8d, 5 },
    { 0, 0 },
    { 0x8516eb8f, 91 },
    { 0xa946eb90, 36 },
    { 0x3096eb91, 90 },
    { 0x6826eb91, 61 },
    { 0x92f6eb91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe906eb96, 23 },
    { 0, 0 },
    { 0x2146eb98, 33 },
    { 0xc4d6eb99, 43 },
    { 0xfb76eb99, 90 },
    { 0x3716eb9b, 90 },
    { 0, 0 },
    { 0x1c46eb9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e46eba4, 33 },
    { 0x3a6eba5, 43 },
    { 0xf926eba5, 33 },
    { 0, 0 },
    { 0x8026eba8, 84 },
    { 0xd336eba9, 76 },
    { 0x3366ebaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96ebb1, 84 },
    { 0, 0 },
    { 0x69c6ebb3, 55 },
    { 0, 0 },
    { 0xba86ebb5, 23 },
    { 0x4436ebb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b46ebba, 90 },
    { 0x4586ebba, 23 },
    { 0x7ba6ebba, 95 },
    { 0x5e06ebbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d6ebc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6ebc5, 99 },
    { 0x6f76ebc6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d6ebcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6ebd2, 90 },
    { 0, 0 },
    { 0x36c6ebd4, 85 },
    { 0x1286ebd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2586ebd9, 55 },
    { 0xec6ebda, 93 },
    { 0x7a86ebdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b6ebe5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf26ebe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b6ebee, 90 },
    { 0, 0 },
    { 0xec46ebf0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b6ebf3, 98 },
    { 0xc086ebf3, 98 },
    { 0xd606ebf3, 34 },
    { 0x3816ebf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6ebfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8506ec02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6ec05, 90 },
    { 0x7766ec06, 90 },
    { 0, 0 },
    { 0x6d96ec08, 33 },
    { 0xa446ec09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9456ec0d, 85 },
    { 0x26b6ec0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46ec15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196ec18, 33 },
    { 0, 0 },
    { 0xb716ec1a, 33 },
    { 0, 0 },
    { 0x3376ec1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5246ec1f, 52 },
    { 0x5826ec20, 55 },
    { 0xb186ec21, 61 },
    { 0x3426ec22, 84 },
    { 0x6ec23, 90 },
    { 0xbe46ec22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b6ec2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b6ec30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6ec36, 43 },
    { 0, 0 },
    { 0xa6a6ec38, 90 },
    { 0xf766ec38, 95 },
    { 0x866ec3a, 43 },
    { 0xec6ec3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6ec40, 61 },
    { 0x7e86ec41, 33 },
    { 0, 0 },
    { 0xda16ec43, 81 },
    { 0, 0 },
    { 0xfbc6ec45, 81 },
    { 0, 0 },
    { 0x7e96ec47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f6ec4a, 55 },
    { 0xbc46ec4b, 59 },
    { 0x4bf6ec4b, 33 },
    { 0x4686ec4d, 90 },
    { 0x9f86ec4c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e76ec51, 43 },
    { 0, 0 },
    { 0xab26ec53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6ec58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f6ec5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686ec62, 61 },
    { 0xb696ec63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c66ec6d, 61 },
    { 0, 0 },
    { 0x7ae6ec6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c6ec72, 33 },
    { 0xd146ec73, 91 },
    { 0, 0 },
    { 0xd8e6ec75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c6ec7f, 23 },
    { 0, 0 },
    { 0xf556ec81, 85 },
    { 0x7ea6ec81, 61 },
    { 0, 0 },
    { 0x73b6ec84, 93 },
    { 0x8b96ec85, 90 },
    { 0xd186ec86, 98 },
    { 0x3716ec87, 64 },
    { 0xf956ec86, 95 },
    { 0xf7c6ec89, 98 },
    { 0xec96ec89, 98 },
    { 0, 0 },
    { 0xba26ec8c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce6ec8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d6ec92, 76 },
    { 0, 0 },
    { 0x166ec94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f6ec97, 91 },
    { 0xce6ec98, 98 },
    { 0xaf36ec97, 90 },
    { 0x8a6ec9a, 96 },
    { 0xd696ec9b, 55 },
    { 0, 0 },
    { 0x7ca6ec9d, 46 },
    { 0, 0 },
    { 0xd766ec9f, 95 },
    { 0, 0 },
    { 0xc106eca1, 87 },
    { 0xc7b6eca2, 76 },
    { 0xcf6eca3, 55 },
    { 0xf2f6eca3, 56 },
    { 0x9fb6eca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846ecab, 61 },
    { 0x2a26ecac, 95 },
    { 0, 0 },
    { 0x3526ecae, 98 },
    { 0x9776ecaf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456ecb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb6ecb8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f6ecbb, 23 },
    { 0x8606ecbc, 91 },
    { 0, 0 },
    { 0x67e6ecbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a56ecc2, 95 },
    { 0xaf76ecc3, 23 },
    { 0x8536ecc2, 33 },
    { 0x6c66ecc5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a6ecca, 56 },
    { 0x3ad6ecca, 52 },
    { 0x4836eccc, 90 },
    { 0xaa36eccd, 87 },
    { 0xee76ecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd6ecd1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4436ecd9, 61 },
    { 0xa2c6ecda, 95 },
    { 0x4216ecda, 84 },
    { 0x28d6ecda, 65 },
    { 0, 0 },
    { 0xa906ecde, 95 },
    { 0x5ef6ecdf, 99 },
    { 0x61a6ece0, 84 },
    { 0x70d6ece1, 77 },
    { 0x6026ece0, 23 },
    { 0xffa6ece3, 65 },
    { 0xb4f6ece4, 85 },
    { 0xb126ece0, 93 },
    { 0, 0 },
    { 0xf3f6ece7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8376ecea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1616ecf0, 67 },
    { 0xdd56ecf1, 34 },
    { 0x7706ecf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc016ecf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc676ecfe, 76 },
    { 0, 0 },
    { 0x1806ed00, 55 },
    { 0xee56ed01, 34 },
    { 0x44d6ed00, 98 },
    { 0xdf06ed03, 98 },
    { 0, 0 },
    { 0x8ee6ed05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb36ed08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a6ed0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b6ed0f, 46 },
    { 0x1b06ed10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf756ed1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f6ed20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd06ed23, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6496ed28, 55 },
    { 0, 0 },
    { 0xa1f6ed2a, 91 },
    { 0x58a6ed2b, 33 },
    { 0xb846ed2a, 43 },
    { 0x9866ed2d, 33 },
    { 0, 0 },
    { 0x83f6ed2f, 33 },
    { 0xb126ed30, 34 },
    { 0x4df6ed31, 98 },
    { 0xfad6ed32, 52 },
    { 0x47b6ed30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6ed38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b6ed3c, 61 },
    { 0, 0 },
    { 0xf916ed3e, 23 },
    { 0xacf6ed3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3346ed44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266ed47, 61 },
    { 0x7ca6ed48, 61 },
    { 0xb506ed49, 59 },
    { 0x9216ed48, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b6ed4d, 46 },
    { 0x2f26ed4e, 95 },
    { 0, 0 },
    { 0xa696ed50, 61 },
    { 0xfb56ed51, 46 },
    { 0x75b6ed52, 91 },
    { 0x6ba6ed51, 98 },
    { 0x976ed54, 33 },
    { 0x64b6ed55, 99 },
    { 0x30f6ed53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc26ed5c, 61 },
    { 0, 0 },
    { 0x86ed5e, 99 },
    { 0xdc16ed5f, 33 },
    { 0x81f6ed5f, 34 },
    { 0x1fb6ed61, 68 },
    { 0x3fa6ed5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8256ed65, 88 },
    { 0xf026ed65, 80 },
    { 0x1e16ed67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb36ed6a, 61 },
    { 0xaac6ed6b, 46 },
    { 0x1f76ed6c, 33 },
    { 0xc856ed6c, 95 },
    { 0xcde6ed6c, 33 },
    { 0x3616ed6f, 34 },
    { 0x7a66ed70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6ed77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d6ed7f, 98 },
    { 0x96d6ed80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66ed84, 98 },
    { 0, 0 },
    { 0xaf26ed86, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d6ed89, 98 },
    { 0x186ed89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476ed8e, 90 },
    { 0xbb66ed8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646ed93, 98 },
    { 0xde06ed94, 34 },
    { 0x7e16ed95, 24 },
    { 0, 0 },
    { 0x4876ed97, 90 },
    { 0, 0 },
    { 0x716ed99, 23 },
    { 0xe5e6ed9a, 87 },
    { 0, 0 },
    { 0x2ee6ed9c, 61 },
    { 0xd9c6ed9d, 90 },
    { 0xbce6ed9e, 33 },
    { 0x74b6ed9e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326eda3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3196eda8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6edab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe996edae, 88 },
    { 0x72f6edae, 46 },
    { 0xb486edb0, 23 },
    { 0, 0 },
    { 0x3536edb2, 76 },
    { 0x9106edb3, 46 },
    { 0, 0 },
    { 0x7fb6edb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6edbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6edbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5136edc1, 33 },
    { 0xce76edc2, 33 },
    { 0, 0 },
    { 0xcac6edc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa6edc9, 95 },
    { 0x8df6edca, 23 },
    { 0x636edcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8826edce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f6edd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c6edd5, 90 },
    { 0x6566edd6, 33 },
    { 0x7966edd7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e6edda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c6eddd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a46ede4, 99 },
    { 0x6226ede5, 53 },
    { 0xa356ede5, 85 },
    { 0x9f46ede7, 87 },
    { 0xd4f6ede8, 90 },
    { 0x5796ede4, 82 },
    { 0xd246edea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6edef, 77 },
    { 0xb996edf0, 23 },
    { 0, 0 },
    { 0x6b56edf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c6edf9, 64 },
    { 0xcf16edfa, 66 },
    { 0, 0 },
    { 0xdb06edfc, 61 },
    { 0x4326edfd, 99 },
    { 0x3ae6edfe, 95 },
    { 0xf266edfc, 76 },
    { 0, 0 },
    { 0x9a96ee01, 76 },
    { 0x8da6ee02, 68 },
    { 0x2146ee03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf146ee0a, 76 },
    { 0, 0 },
    { 0x4036ee0c, 90 },
    { 0, 0 },
    { 0x72a6ee0e, 43 },
    { 0, 0 },
    { 0xc326ee10, 95 },
    { 0xc0a6ee10, 23 },
    { 0xbc96ee12, 87 },
    { 0, 0 },
    { 0xba06ee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab56ee1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176ee24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8866ee2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c6ee32, 82 },
    { 0x6a06ee32, 76 },
    { 0xd336ee32, 34 },
    { 0x24a6ee34, 95 },
    { 0x5316ee36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd476ee3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106ee42, 38 },
    { 0x7a66ee42, 23 },
    { 0xd3c6ee42, 95 },
    { 0xdd96ee44, 97 },
    { 0, 0 },
    { 0x9226ee47, 55 },
    { 0, 0 },
    { 0x1486ee49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6ee51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6ee54, 83 },
    { 0x4466ee55, 84 },
    { 0x51e6ee56, 95 },
    { 0x6556ee57, 87 },
    { 0, 0 },
    { 0x3326ee59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386ee5d, 87 },
    { 0, 0 },
    { 0xb886ee5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa6ee62, 46 },
    { 0x3f36ee63, 95 },
    { 0xce96ee64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3506ee68, 95 },
    { 0, 0 },
    { 0x9236ee6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776ee77, 90 },
    { 0xda46ee77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xad46ee7b, 23 },
    { 0x28f6ee7c, 23 },
    { 0x4026ee7c, 90 },
    { 0x82d6ee7c, 33 },
    { 0xb396ee7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f76ee83, 76 },
    { 0xf396ee84, 61 },
    { 0xd336ee85, 33 },
    { 0xd346ee86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7196ee8b, 33 },
    { 0, 0 },
    { 0x3c66ee8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd276ee93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6ee97, 23 },
    { 0x636ee98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd6eea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a6eead, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe216eeb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546eebd, 90 },
    { 0x92e6eebd, 52 },
    { 0, 0 },
    { 0xb8b6eec0, 98 },
    { 0x3916eec1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6eec4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e16eec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb56eecd, 90 },
    { 0x3f06eece, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66eed1, 56 },
    { 0x4e66eed2, 33 },
    { 0, 0 },
    { 0x5676eed4, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa6eed7, 90 },
    { 0, 0 },
    { 0xfb66eed9, 33 },
    { 0, 0 },
    { 0xd856eedb, 90 },
    { 0xc0c6eedc, 98 },
    { 0, 0 },
    { 0x5966eede, 95 },
    { 0xc906eedf, 75 },
    { 0x90a6eedf, 46 },
    { 0xe016eee1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b56eeeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6eeee, 98 },
    { 0x7bc6eeef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6eef2, 33 },
    { 0xdcf6eef2, 90 },
    { 0x5ee6eef4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6596eef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d6eefb, 93 },
    { 0xf506eefc, 99 },
    { 0x3b66eefd, 61 },
    { 0xeb56eefe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe566ef01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3336ef14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1056ef17, 90 },
    { 0xf016ef18, 91 },
    { 0x1fb6ef18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516ef20, 55 },
    { 0xc0b6ef21, 23 },
    { 0x7a16ef22, 98 },
    { 0x76d6ef22, 65 },
    { 0xb5c6ef23, 90 },
    { 0x5de6ef25, 75 },
    { 0xfdc6ef26, 76 },
    { 0xe0e6ef27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16ef2e, 99 },
    { 0x2126ef2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa6ef3f, 61 },
    { 0, 0 },
    { 0x8216ef41, 33 },
    { 0, 0 },
    { 0x7756ef43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a96ef4c, 93 },
    { 0, 0 },
    { 0x9fe6ef4e, 87 },
    { 0xfad6ef4f, 23 },
    { 0x4f46ef50, 55 },
    { 0x6d16ef51, 95 },
    { 0xc2c6ef52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016ef55, 76 },
    { 0x8a66ef55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3886ef5a, 98 },
    { 0xf216ef5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b26ef5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5476ef64, 61 },
    { 0, 0 },
    { 0x8a86ef66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6586ef6b, 76 },
    { 0xed76ef6c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f6ef6f, 90 },
    { 0x4ff6ef6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a6ef73, 81 },
    { 0x4af6ef74, 99 },
    { 0x986ef75, 61 },
    { 0x31b6ef75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26ef7e, 95 },
    { 0x5ad6ef7f, 27 },
    { 0x6af6ef7e, 33 },
    { 0x73a6ef81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa016ef86, 98 },
    { 0x94b6ef87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f06ef8b, 61 },
    { 0x1cd6ef8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc256ef99, 61 },
    { 0x33d6ef9a, 95 },
    { 0xa206ef9a, 17 },
    { 0xe416ef9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436efa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6efa9, 52 },
    { 0x85b6efa9, 90 },
    { 0x33c6efaa, 46 },
    { 0x1b46efac, 95 },
    { 0, 0 },
    { 0x846efae, 78 },
    { 0xcff6efaf, 23 },
    { 0, 0 },
    { 0x6266efb1, 93 },
    { 0x4696efb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb96efc6, 95 },
    { 0x6856efc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6efcc, 46 },
    { 0x7e26efcd, 76 },
    { 0x60d6efce, 78 },
    { 0, 0 },
    { 0x24a6efd0, 61 },
    { 0xe1e6efd1, 98 },
    { 0x8cb6efd1, 93 },
    { 0x4216efd1, 95 },
    { 0xc316efd1, 95 },
    { 0x6f46efd1, 61 },
    { 0x22f6efd6, 76 },
    { 0, 0 },
    { 0xc096efd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6efdb, 84 },
    { 0x6cd6efdc, 61 },
    { 0xa026efdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a6efe5, 83 },
    { 0, 0 },
    { 0x87b6efe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac6efec, 66 },
    { 0x7386efed, 90 },
    { 0, 0 },
    { 0xef56efef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426eff2, 91 },
    { 0, 0 },
    { 0x9bf6eff4, 91 },
    { 0x4716eff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f36effb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86effe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046f001, 95 },
    { 0xaa26f001, 90 },
    { 0xff46f002, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d6f011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b6f014, 90 },
    { 0xa906f015, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2106f019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc266f01f, 55 },
    { 0, 0 },
    { 0x3746f021, 61 },
    { 0, 0 },
    { 0xc2c6f023, 33 },
    { 0, 0 },
    { 0x48d6f025, 34 },
    { 0x3a06f025, 90 },
    { 0x31b6f027, 98 },
    { 0x15a6f028, 66 },
    { 0xaf86f025, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6f030, 61 },
    { 0xff76f031, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16f034, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a36f037, 17 },
    { 0xe6a6f038, 61 },
    { 0x4ab6f039, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc586f03f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5476f046, 76 },
    { 0, 0 },
    { 0x2bc6f048, 98 },
    { 0, 0 },
    { 0xb2d6f04a, 78 },
    { 0, 0 },
    { 0x92a6f04c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f6f04f, 34 },
    { 0xcfd6f050, 98 },
    { 0x7ee6f051, 98 },
    { 0xb106f052, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa86f056, 34 },
    { 0xe4b6f057, 98 },
    { 0xce56f057, 93 },
    { 0xa1c6f057, 23 },
    { 0xcfb6f05a, 94 },
    { 0x1856f056, 36 },
    { 0xff76f05c, 90 },
    { 0xafe6f05d, 98 },
    { 0x9ea6f05e, 33 },
    { 0x8be6f05f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3806f064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396f068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9856f06b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176f073, 77 },
    { 0x59f6f074, 97 },
    { 0xe9c6f074, 90 },
    { 0x9b26f076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2066f079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a6f07c, 55 },
    { 0, 0 },
    { 0xb986f07e, 90 },
    { 0, 0 },
    { 0xdba6f080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b86f083, 61 },
    { 0xd756f084, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6f088, 33 },
    { 0x7a36f089, 100 },
    { 0xe4f6f08a, 66 },
    { 0x9426f08b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2806f08f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2736f096, 23 },
    { 0xe586f097, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96f09a, 33 },
    { 0, 0 },
    { 0xdb26f09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a6f0ad, 99 },
    { 0x5736f0ae, 90 },
    { 0x5916f0af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7536f0b6, 76 },
    { 0xf5e6f0b7, 90 },
    { 0, 0 },
    { 0x3916f0b9, 98 },
    { 0x9ad6f0ba, 33 },
    { 0x6da6f0bb, 33 },
    { 0x8096f0bc, 61 },
    { 0x7d96f0bd, 90 },
    { 0x6816f0be, 33 },
    { 0x7396f0bd, 76 },
    { 0xbf86f0bf, 12 },
    { 0, 0 },
    { 0x5f26f0c2, 88 },
    { 0, 0 },
    { 0xb7b6f0c4, 87 },
    { 0x476f0c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc46f0cb, 23 },
    { 0, 0 },
    { 0x9f36f0cd, 17 },
    { 0xb716f0ce, 91 },
    { 0xa186f0ce, 90 },
    { 0x3676f0ce, 95 },
    { 0xc016f0d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b6f0d4, 90 },
    { 0, 0 },
    { 0xacf6f0d6, 34 },
    { 0xeba6f0d7, 98 },
    { 0xc226f0d8, 80 },
    { 0x5506f0d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3756f0de, 84 },
    { 0, 0 },
    { 0xc166f0e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f6f0e6, 98 },
    { 0xef86f0e7, 95 },
    { 0x2b76f0e8, 90 },
    { 0x7ab6f0e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c6f0ee, 100 },
    { 0x8646f0ee, 61 },
    { 0x1206f0f0, 99 },
    { 0x5586f0f0, 61 },
    { 0, 0 },
    { 0xe66f0f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6f0f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6f0fb, 33 },
    { 0xf196f0fc, 78 },
    { 0xf626f0fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6616f106, 61 },
    { 0xbcc6f107, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436f10b, 99 },
    { 0xeff6f10c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86f114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb76f11d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a06f126, 55 },
    { 0, 0 },
    { 0xb1a6f128, 77 },
    { 0, 0 },
    { 0xd5b6f12a, 68 },
    { 0x4176f12b, 90 },
    { 0x5446f12b, 98 },
    { 0x58b6f12d, 51 },
    { 0x4906f12d, 98 },
    { 0xda26f12e, 64 },
    { 0x22b6f130, 99 },
    { 0x34d6f12f, 95 },
    { 0x25d6f12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f06f136, 90 },
    { 0x66b6f136, 93 },
    { 0, 0 },
    { 0x16f139, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46f13d, 95 },
    { 0, 0 },
    { 0xd526f13f, 100 },
    { 0xb646f13f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466f144, 90 },
    { 0x4e16f144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f14b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7526f14f, 95 },
    { 0, 0 },
    { 0xad16f151, 33 },
    { 0, 0 },
    { 0x8576f153, 75 },
    { 0xe466f154, 34 },
    { 0xa3a6f155, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6f160, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8146f168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6f170, 23 },
    { 0x3106f170, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6756f17c, 77 },
    { 0xccc6f17d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed6f182, 55 },
    { 0x8cb6f183, 33 },
    { 0x8066f184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7776f187, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa786f18a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06f190, 23 },
    { 0xe646f191, 64 },
    { 0x8016f192, 90 },
    { 0xd396f191, 90 },
    { 0xe2b6f194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f6f199, 36 },
    { 0xdd56f19a, 66 },
    { 0x7d06f199, 87 },
    { 0xf4d6f19c, 90 },
    { 0, 0 },
    { 0xf556f19e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c6f1a5, 61 },
    { 0xcd76f1a6, 34 },
    { 0x15b6f1a5, 34 },
    { 0xc676f1a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d6f1ab, 87 },
    { 0x95f6f1ac, 61 },
    { 0x1f96f1ad, 90 },
    { 0x90f6f1ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36f1b1, 55 },
    { 0xb8f6f1b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b26f1b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876f1be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7646f1c1, 91 },
    { 0x50a6f1c1, 90 },
    { 0x1386f1c3, 95 },
    { 0x7096f1c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa506f1c8, 99 },
    { 0xf4a6f1c9, 91 },
    { 0, 0 },
    { 0x8d16f1cb, 90 },
    { 0x1cc6f1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf716f1d3, 95 },
    { 0x4ab6f1d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb26f1db, 99 },
    { 0x6d6f1dc, 61 },
    { 0xb1c6f1db, 90 },
    { 0xb886f1de, 77 },
    { 0x9056f1df, 43 },
    { 0x5a86f1db, 98 },
    { 0xf686f1e0, 33 },
    { 0xc366f1e2, 98 },
    { 0x14f6f1e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6f1e6, 95 },
    { 0x2356f1e7, 55 },
    { 0xcf16f1e6, 94 },
    { 0x4f06f1e9, 23 },
    { 0xc186f1ea, 91 },
    { 0x65f6f1eb, 33 },
    { 0x8ac6f1ec, 52 },
    { 0xad36f1ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd506f1f1, 55 },
    { 0xb986f1f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446f1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456f1fd, 95 },
    { 0xaea6f1fd, 91 },
    { 0x7ec6f1ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6f202, 34 },
    { 0x4fc6f203, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d6f207, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96f20b, 85 },
    { 0xd126f20c, 33 },
    { 0x6156f20d, 33 },
    { 0x2d56f20e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2706f212, 61 },
    { 0x41a6f212, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966f219, 55 },
    { 0x1d46f21a, 66 },
    { 0x20e6f21a, 95 },
    { 0xd646f21a, 33 },
    { 0x4ee6f219, 33 },
    { 0xcfd6f21e, 55 },
    { 0x1fc6f21d, 33 },
    { 0x56f220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff76f225, 33 },
    { 0x7bf6f226, 55 },
    { 0x5ab6f227, 55 },
    { 0xe5b6f227, 90 },
    { 0x9bd6f229, 55 },
    { 0x3606f228, 95 },
    { 0x6016f22b, 17 },
    { 0, 0 },
    { 0x136f22d, 33 },
    { 0x4f36f22e, 98 },
    { 0, 0 },
    { 0xd246f230, 46 },
    { 0, 0 },
    { 0x45b6f232, 95 },
    { 0, 0 },
    { 0x5516f234, 90 },
    { 0x4446f235, 64 },
    { 0x5586f234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e6f23b, 43 },
    { 0x4d96f23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc336f242, 33 },
    { 0, 0 },
    { 0x3436f244, 61 },
    { 0x9a16f245, 90 },
    { 0x22c6f244, 95 },
    { 0x7f76f247, 90 },
    { 0x5276f247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8306f24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7396f252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a6f256, 90 },
    { 0, 0 },
    { 0x2ab6f258, 43 },
    { 0x69c6f259, 75 },
    { 0, 0 },
    { 0xbe16f25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e96f262, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea6f265, 55 },
    { 0xa8d6f265, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef76f26a, 90 },
    { 0xde16f26b, 75 },
    { 0xad6f26c, 90 },
    { 0, 0 },
    { 0xb666f26e, 34 },
    { 0xbf46f26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b86f277, 90 },
    { 0xdec6f277, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa6f27b, 78 },
    { 0x5af6f27c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd6f27f, 43 },
    { 0, 0 },
    { 0xecd6f281, 90 },
    { 0x3a96f282, 90 },
    { 0x65d6f281, 61 },
    { 0, 0 },
    { 0x4bf6f285, 34 },
    { 0xcc76f286, 55 },
    { 0, 0 },
    { 0x8cb6f288, 78 },
    { 0xa536f289, 46 },
    { 0x8c26f28a, 55 },
    { 0xd3b6f28b, 76 },
    { 0x7716f28b, 33 },
    { 0, 0 },
    { 0x22c6f28e, 95 },
    { 0xce96f28e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6f293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc346f29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d6f2a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d6f2a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a26f2a8, 90 },
    { 0, 0 },
    { 0x4626f2aa, 33 },
    { 0x76f2aa, 34 },
    { 0, 0 },
    { 0x7ef6f2ad, 90 },
    { 0, 0 },
    { 0x3326f2af, 95 },
    { 0xb6c6f2b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a6f2b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c6f2b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d6f2c6, 60 },
    { 0, 0 },
    { 0x1896f2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d6f2d0, 48 },
    { 0x23e6f2d1, 90 },
    { 0xa3b6f2d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f2d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d6f2da, 83 },
    { 0xdf76f2da, 23 },
    { 0x95c6f2dc, 52 },
    { 0x9dd6f2da, 34 },
    { 0xdcf6f2dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1956f2e1, 90 },
    { 0, 0 },
    { 0x8bd6f2e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e36f2e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9066f2ef, 95 },
    { 0xfc16f2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e6f2f3, 95 },
    { 0x6f96f2f4, 95 },
    { 0x5f36f2f5, 61 },
    { 0xa3d6f2f4, 95 },
    { 0, 0 },
    { 0xfb96f2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e6f2fc, 33 },
    { 0, 0 },
    { 0x8fb6f2fe, 76 },
    { 0, 0 },
    { 0x63c6f300, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x556f303, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56f306, 63 },
    { 0x5206f306, 23 },
    { 0x91c6f308, 46 },
    { 0x82a6f307, 33 },
    { 0x2166f30a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96f30d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7296f311, 91 },
    { 0xca46f311, 23 },
    { 0x7b56f313, 98 },
    { 0x3076f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516f31e, 77 },
    { 0x7e6f31f, 23 },
    { 0xe46f320, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee6f323, 33 },
    { 0x6d66f324, 17 },
    { 0xf316f324, 87 },
    { 0xf186f324, 33 },
    { 0xa906f327, 34 },
    { 0x4666f326, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd316f32d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886f330, 91 },
    { 0x4556f331, 76 },
    { 0xa466f332, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6f33f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea6f342, 61 },
    { 0, 0 },
    { 0x6f26f344, 61 },
    { 0xbe56f344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe136f348, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c6f34c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6f356, 36 },
    { 0x4de6f356, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f6f35a, 75 },
    { 0x3f46f35a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16f360, 76 },
    { 0, 0 },
    { 0xe186f362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4406f365, 23 },
    { 0xd106f366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a6f36a, 61 },
    { 0xb556f36b, 55 },
    { 0x1226f36b, 23 },
    { 0xc76f36c, 88 },
    { 0x18e6f36e, 33 },
    { 0x1476f36d, 33 },
    { 0, 0 },
    { 0x26f371, 46 },
    { 0x56f371, 98 },
    { 0xe366f371, 61 },
    { 0x3bf6f374, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566f37c, 34 },
    { 0x296f37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966f380, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b6f387, 76 },
    { 0, 0 },
    { 0x5f16f389, 43 },
    { 0, 0 },
    { 0xaa96f38b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae6f390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc6f398, 61 },
    { 0, 0 },
    { 0xe706f39a, 78 },
    { 0xc1d6f39b, 66 },
    { 0x26b6f39a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae46f3a1, 23 },
    { 0, 0 },
    { 0xd316f3a3, 99 },
    { 0, 0 },
    { 0x9a56f3a5, 61 },
    { 0xb296f3a6, 61 },
    { 0xe06f3a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e6f3b2, 90 },
    { 0x3e86f3b2, 43 },
    { 0x9896f3b4, 61 },
    { 0x86a6f3b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7566f3bf, 90 },
    { 0xd146f3c0, 61 },
    { 0x6946f3c1, 55 },
    { 0xc366f3c1, 76 },
    { 0, 0 },
    { 0xa046f3c4, 91 },
    { 0xe196f3c4, 36 },
    { 0, 0 },
    { 0x1d06f3c7, 99 },
    { 0x1136f3c8, 33 },
    { 0xa0b6f3c9, 61 },
    { 0x706f3ca, 95 },
    { 0, 0 },
    { 0x6596f3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436f3cf, 87 },
    { 0xf0e6f3d0, 33 },
    { 0x7ab6f3d1, 95 },
    { 0x9c36f3d1, 61 },
    { 0x7e86f3d2, 95 },
    { 0xb8b6f3d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4906f3d7, 99 },
    { 0x4316f3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a6f3de, 98 },
    { 0x5846f3df, 95 },
    { 0x4106f3de, 95 },
    { 0xe8b6f3e0, 33 },
    { 0x8e76f3e2, 33 },
    { 0xaea6f3de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc6f3e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d6f3e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d56f3ed, 33 },
    { 0, 0 },
    { 0xa486f3ef, 84 },
    { 0x3146f3ef, 55 },
    { 0, 0 },
    { 0x6ac6f3f2, 52 },
    { 0x8246f3f3, 61 },
    { 0xf026f3f4, 90 },
    { 0xc9f6f3f3, 98 },
    { 0xfec6f3f6, 66 },
    { 0xfba6f3f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586f3fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6926f400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966f406, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9706f40a, 61 },
    { 0x2f56f40a, 90 },
    { 0xc9a6f40c, 81 },
    { 0xe346f40c, 90 },
    { 0x5c66f40a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8396f413, 93 },
    { 0xcb96f414, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca56f417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa296f41e, 93 },
    { 0xc096f41e, 43 },
    { 0x2b26f41f, 63 },
    { 0x5db6f421, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe6f424, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd746f427, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc566f42d, 85 },
    { 0x5a36f42e, 33 },
    { 0, 0 },
    { 0x82e6f430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6f433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f6f43b, 90 },
    { 0, 0 },
    { 0x2676f43d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06f444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x666f447, 95 },
    { 0, 0 },
    { 0x5f76f449, 23 },
    { 0x41a6f44a, 75 },
    { 0, 0 },
    { 0x2d46f44c, 66 },
    { 0, 0 },
    { 0x4616f44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316f455, 61 },
    { 0, 0 },
    { 0x7476f457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7626f45a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb966f45f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6f465, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f468, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe896f46d, 100 },
    { 0, 0 },
    { 0x4e36f46f, 98 },
    { 0, 0 },
    { 0xc9f6f471, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f475, 95 },
    { 0xb506f475, 66 },
    { 0x7996f477, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7636f47a, 85 },
    { 0xf6b6f47a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc956f481, 76 },
    { 0xc476f482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6f485, 37 },
    { 0x3916f485, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66f48b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96f48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c6f492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2186f496, 61 },
    { 0x55e6f497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c6f49a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff6f49d, 55 },
    { 0x9906f49e, 65 },
    { 0xb3c6f49f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf426f4a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396f4a5, 76 },
    { 0xc26f4a5, 80 },
    { 0x2f26f4a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7796f4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b6f4af, 85 },
    { 0, 0 },
    { 0xffc6f4b1, 95 },
    { 0x4436f4b1, 91 },
    { 0x2616f4b2, 95 },
    { 0x2506f4b4, 33 },
    { 0x2626f4b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196f4c3, 76 },
    { 0x69f6f4c4, 46 },
    { 0xcb56f4c3, 85 },
    { 0, 0 },
    { 0xf346f4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b56f4cb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76f4d8, 80 },
    { 0x8106f4d8, 87 },
    { 0xf866f4da, 53 },
    { 0x5ea6f4db, 33 },
    { 0xd526f4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06f4e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad6f4e4, 90 },
    { 0xde56f4e5, 98 },
    { 0x5256f4e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06f4e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776f4f0, 87 },
    { 0x356f4f1, 23 },
    { 0x65d6f4f1, 33 },
    { 0, 0 },
    { 0x62f6f4f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a6f4f7, 98 },
    { 0xdd76f4f8, 43 },
    { 0x2536f4f8, 90 },
    { 0x6916f4fa, 33 },
    { 0x94d6f4fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d6f500, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9056f505, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6506f509, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56f512, 33 },
    { 0, 0 },
    { 0xb506f514, 34 },
    { 0xd1a6f514, 90 },
    { 0x3b06f514, 33 },
    { 0xa346f517, 95 },
    { 0x10f6f514, 98 },
    { 0, 0 },
    { 0x8806f51a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596f522, 91 },
    { 0xc916f522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f528, 33 },
    { 0x3fb6f529, 52 },
    { 0xa3a6f52a, 61 },
    { 0x8a56f529, 90 },
    { 0x81b6f52c, 61 },
    { 0x6036f52c, 90 },
    { 0xb506f52c, 94 },
    { 0x3e26f52f, 100 },
    { 0, 0 },
    { 0x2176f531, 90 },
    { 0, 0 },
    { 0x3306f533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6f53b, 33 },
    { 0x6456f53c, 75 },
    { 0x84c6f53d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b6f548, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6f54b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3446f54e, 33 },
    { 0xe316f54f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f6f553, 23 },
    { 0x8be6f554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496f55b, 93 },
    { 0xd036f55c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a6f560, 55 },
    { 0xb4c6f561, 61 },
    { 0x4b46f562, 90 },
    { 0x5cd6f562, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6f569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36f56c, 90 },
    { 0x3926f56d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726f570, 88 },
    { 0, 0 },
    { 0x3476f572, 85 },
    { 0x45d6f573, 90 },
    { 0x4f06f574, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6f577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796f57a, 46 },
    { 0xed66f57b, 88 },
    { 0x7466f57b, 46 },
    { 0x5fa6f57a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6f584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6f58b, 61 },
    { 0x1096f58b, 59 },
    { 0xa906f58d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8036f590, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216f597, 33 },
    { 0x1736f597, 55 },
    { 0x79f6f599, 98 },
    { 0, 0 },
    { 0x6336f59b, 33 },
    { 0xbe36f59b, 33 },
    { 0x1dd6f59d, 76 },
    { 0x1896f59c, 66 },
    { 0xa96f59f, 56 },
    { 0x5006f59f, 95 },
    { 0, 0 },
    { 0x84e6f5a2, 91 },
    { 0xfe16f5a2, 61 },
    { 0x5596f5a2, 60 },
    { 0xbaf6f5a5, 23 },
    { 0xeef6f5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696f5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff56f5ae, 90 },
    { 0xbf16f5af, 95 },
    { 0xac06f5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc26f5b6, 33 },
    { 0, 0 },
    { 0x3e6f5b8, 93 },
    { 0x5596f5b9, 90 },
    { 0x6f16f5b9, 55 },
    { 0, 0 },
    { 0xbcc6f5bc, 98 },
    { 0xfeb6f5bd, 93 },
    { 0x22b6f5be, 23 },
    { 0, 0 },
    { 0x8996f5c0, 34 },
    { 0x2f06f5c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf16f5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe116f5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6f5cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6196f5d1, 95 },
    { 0xd956f5d2, 33 },
    { 0x31c6f5d3, 77 },
    { 0x8146f5d3, 90 },
    { 0xb486f5d5, 95 },
    { 0xe756f5d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9696f5da, 33 },
    { 0, 0 },
    { 0x75f6f5dc, 98 },
    { 0x626f5dd, 61 },
    { 0x7856f5dd, 43 },
    { 0x72c6f5df, 61 },
    { 0x4d36f5df, 69 },
    { 0xf3f6f5e1, 99 },
    { 0xa116f5e2, 90 },
    { 0x8b66f5e3, 23 },
    { 0xf3d6f5e2, 55 },
    { 0x16e6f5e2, 93 },
    { 0x6be6f5e6, 61 },
    { 0x4be6f5e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d6f5ea, 98 },
    { 0x70d6f5ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7876f5fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b6f5ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4976f605, 52 },
    { 0xd06f606, 33 },
    { 0x5806f606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f60b, 91 },
    { 0x4cf6f60b, 98 },
    { 0x5486f60d, 40 },
    { 0xd3b6f60e, 98 },
    { 0x4906f60f, 95 },
    { 0xdbf6f610, 53 },
    { 0xaea6f611, 95 },
    { 0xe106f612, 35 },
    { 0x3de6f613, 90 },
    { 0xb7e6f614, 76 },
    { 0xb336f613, 93 },
    { 0xd716f616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6f619, 98 },
    { 0, 0 },
    { 0x33e6f61b, 65 },
    { 0xbd46f61c, 93 },
    { 0xf3d6f61d, 33 },
    { 0x58c6f61e, 95 },
    { 0, 0 },
    { 0x2086f620, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e66f629, 43 },
    { 0, 0 },
    { 0xa386f62b, 33 },
    { 0x4e16f62b, 90 },
    { 0x5466f62b, 98 },
    { 0x5be6f62e, 34 },
    { 0xa236f62b, 55 },
    { 0xbf96f630, 55 },
    { 0x24a6f62f, 76 },
    { 0x9f86f62d, 95 },
    { 0x7c6f62e, 90 },
    { 0xefc6f631, 46 },
    { 0x9936f635, 87 },
    { 0x8906f636, 100 },
    { 0x1786f62f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f16f63a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b86f63e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5826f641, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee6f648, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16f64b, 33 },
    { 0x4f76f64c, 84 },
    { 0, 0 },
    { 0xb486f64e, 34 },
    { 0xb636f64f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8016f653, 99 },
    { 0, 0 },
    { 0xe096f655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e6f65a, 84 },
    { 0x2aa6f65b, 46 },
    { 0, 0 },
    { 0xbfd6f65d, 66 },
    { 0x60c6f65d, 59 },
    { 0x8596f65e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116f662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e6f66a, 76 },
    { 0, 0 },
    { 0x8466f66c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e6f675, 33 },
    { 0x3e86f676, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6f67b, 55 },
    { 0x4cc6f67c, 95 },
    { 0xdd76f67d, 98 },
    { 0xa916f67e, 98 },
    { 0x8b46f67e, 95 },
    { 0, 0 },
    { 0x7b46f681, 90 },
    { 0, 0 },
    { 0x46e6f683, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6f688, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96f68b, 33 },
    { 0xb2b6f68c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1756f692, 90 },
    { 0xf256f693, 91 },
    { 0xc226f694, 76 },
    { 0x3406f695, 93 },
    { 0x6886f693, 87 },
    { 0xba86f697, 33 },
    { 0xe26f696, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366f69f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb6f6a5, 90 },
    { 0x25d6f6a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c6f6ab, 77 },
    { 0, 0 },
    { 0x1ff6f6ad, 43 },
    { 0xa206f6ae, 98 },
    { 0x65c6f6af, 34 },
    { 0x1246f6b0, 76 },
    { 0x9986f6b1, 34 },
    { 0xf276f6af, 84 },
    { 0, 0 },
    { 0xd866f6b4, 77 },
    { 0x6c96f6b5, 85 },
    { 0x816f6b5, 33 },
    { 0, 0 },
    { 0xe6b6f6b8, 33 },
    { 0xa5f6f6b9, 34 },
    { 0x59b6f6ba, 76 },
    { 0x59d6f6bb, 90 },
    { 0xd96f6ba, 43 },
    { 0xea56f6bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c6f6c6, 76 },
    { 0, 0 },
    { 0xb426f6c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b6f6cc, 61 },
    { 0xc806f6cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe446f6d2, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xd386f6d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f6d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db6f6dc, 98 },
    { 0x1316f6dc, 61 },
    { 0, 0 },
    { 0x8896f6df, 23 },
    { 0x646f6e0, 23 },
    { 0x7a66f6e1, 61 },
    { 0x8426f6e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe816f6e5, 40 },
    { 0xfa16f6e6, 33 },
    { 0x9136f6e7, 33 },
    { 0, 0 },
    { 0x6c6f6e9, 33 },
    { 0xf576f6ea, 81 },
    { 0xe3d6f6eb, 93 },
    { 0x3996f6ec, 55 },
    { 0x76c6f6ed, 76 },
    { 0x14c6f6ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f6f1, 95 },
    { 0x39c6f6f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16f6f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6f6fe, 95 },
    { 0x89a6f6ff, 43 },
    { 0x8c46f700, 33 },
    { 0xd256f701, 90 },
    { 0x2036f701, 93 },
    { 0xe686f703, 90 },
    { 0x9de6f704, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac6f709, 76 },
    { 0x5036f70a, 98 },
    { 0x5d26f70b, 90 },
    { 0, 0 },
    { 0x7bf6f70d, 61 },
    { 0, 0 },
    { 0x396f70f, 61 },
    { 0, 0 },
    { 0xce26f711, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16f720, 46 },
    { 0x2b6f721, 81 },
    { 0x3e06f721, 23 },
    { 0x1036f723, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab36f729, 34 },
    { 0x1506f72a, 99 },
    { 0, 0 },
    { 0x2836f72c, 33 },
    { 0x4526f72d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8716f731, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d6f73a, 95 },
    { 0x9b16f73b, 91 },
    { 0x18a6f73b, 90 },
    { 0xc816f73d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16f746, 61 },
    { 0x71a6f747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc936f74a, 95 },
    { 0x7866f74a, 23 },
    { 0x7e16f74c, 61 },
    { 0, 0 },
    { 0xc266f74e, 61 },
    { 0x8366f74f, 33 },
    { 0x9a66f74f, 55 },
    { 0x5516f751, 68 },
    { 0x3e6f752, 95 },
    { 0x54b6f753, 76 },
    { 0x8d6f752, 61 },
    { 0x7f6f754, 61 },
    { 0xc8f6f756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe66f75a, 98 },
    { 0, 0 },
    { 0x3096f75c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f86f75f, 95 },
    { 0, 0 },
    { 0x3456f761, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd6f768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6f76b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf6f76e, 93 },
    { 0xeaf6f76e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096f777, 34 },
    { 0x6de6f778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1196f77b, 61 },
    { 0x5be6f77b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f6f783, 23 },
    { 0x5e86f784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe076f788, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b6f78b, 95 },
    { 0x6f46f78c, 85 },
    { 0, 0 },
    { 0xb296f78e, 33 },
    { 0, 0 },
    { 0x6fc6f790, 33 },
    { 0x9896f791, 88 },
    { 0x5a6f792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a36f795, 76 },
    { 0x80f6f796, 44 },
    { 0, 0 },
    { 0x4466f798, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb756f79d, 43 },
    { 0, 0 },
    { 0x6626f79f, 55 },
    { 0x62d6f79f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646f7a6, 100 },
    { 0x6d16f7a7, 85 },
    { 0x4476f7a7, 81 },
    { 0xb2e6f7a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e96f7ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f7b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d96f7b4, 61 },
    { 0x3116f7b5, 61 },
    { 0x4576f7b6, 76 },
    { 0x9886f7b7, 100 },
    { 0x7726f7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2286f7bb, 95 },
    { 0xe3b6f7bb, 33 },
    { 0xa0a6f7bd, 83 },
    { 0xe66f7bb, 33 },
    { 0x89c6f7bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9196f7c2, 98 },
    { 0x3db6f7c2, 33 },
    { 0xc826f7c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf86f7c9, 98 },
    { 0xe566f7ca, 40 },
    { 0x9a96f7cb, 43 },
    { 0xf8a6f7cc, 88 },
    { 0xe56f7cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe516f7d2, 40 },
    { 0x9e16f7d3, 85 },
    { 0x1f66f7d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8916f7dd, 61 },
    { 0, 0 },
    { 0x6f7df, 98 },
    { 0, 0 },
    { 0x4cb6f7e1, 90 },
    { 0, 0 },
    { 0x89e6f7e3, 87 },
    { 0x1656f7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e6f7e7, 84 },
    { 0x396f7e7, 95 },
    { 0, 0 },
    { 0x88b6f7ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6f7ed, 43 },
    { 0xde06f7ee, 34 },
    { 0xb5b6f7ef, 78 },
    { 0xbfc6f7f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3586f7f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d96f7fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06f7ff, 33 },
    { 0x1de6f7ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef56f803, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f6f808, 98 },
    { 0x9976f808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5946f80d, 33 },
    { 0x5526f80e, 34 },
    { 0, 0 },
    { 0x70a6f810, 90 },
    { 0xd086f810, 33 },
    { 0xfc66f812, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc646f81d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436f821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06f829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2496f830, 90 },
    { 0, 0 },
    { 0x7376f832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb226f835, 61 },
    { 0xbfc6f835, 95 },
    { 0x35a6f837, 93 },
    { 0x2da6f838, 23 },
    { 0, 0 },
    { 0x9d46f83a, 90 },
    { 0, 0 },
    { 0xe856f83c, 90 },
    { 0, 0 },
    { 0x2886f83e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad6f845, 98 },
    { 0x5586f846, 23 },
    { 0xe8b6f845, 66 },
    { 0x93d6f848, 88 },
    { 0x3376f849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d96f851, 61 },
    { 0xda46f852, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf96f857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196f85a, 61 },
    { 0, 0 },
    { 0xe176f85c, 40 },
    { 0, 0 },
    { 0x1d16f85e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd6f863, 96 },
    { 0x4996f864, 99 },
    { 0xa286f865, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb56f869, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe756f86e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a6f871, 61 },
    { 0, 0 },
    { 0xe5b6f873, 96 },
    { 0xe566f874, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb996f879, 99 },
    { 0x39d6f87a, 95 },
    { 0x6606f879, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396f87e, 17 },
    { 0x3526f87f, 33 },
    { 0, 0 },
    { 0x2f46f881, 33 },
    { 0x6346f882, 34 },
    { 0, 0 },
    { 0xeb96f884, 78 },
    { 0xba6f885, 100 },
    { 0x6fe6f884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf806f88c, 81 },
    { 0, 0 },
    { 0x6b56f88e, 90 },
    { 0x59b6f88e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6f893, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f89a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06f8a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea76f8a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb6f8ad, 91 },
    { 0, 0 },
    { 0x8fc6f8af, 33 },
    { 0, 0 },
    { 0xafa6f8b1, 98 },
    { 0, 0 },
    { 0x4fe6f8b3, 61 },
    { 0, 0 },
    { 0xf156f8b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd66f8bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa996f8c3, 43 },
    { 0x7466f8c4, 55 },
    { 0, 0 },
    { 0x9896f8c6, 61 },
    { 0x5546f8c7, 90 },
    { 0x3b06f8c7, 41 },
    { 0x7ed6f8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8746f8cd, 33 },
    { 0x6836f8ce, 34 },
    { 0, 0 },
    { 0xacb6f8d0, 43 },
    { 0xe726f8d1, 23 },
    { 0xfa6f8d2, 33 },
    { 0x68f6f8d1, 43 },
    { 0xb386f8d2, 33 },
    { 0, 0 },
    { 0xfdc6f8d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6f8da, 97 },
    { 0, 0 },
    { 0x1b56f8dc, 61 },
    { 0, 0 },
    { 0xd0e6f8de, 90 },
    { 0x7dc6f8df, 85 },
    { 0x27a6f8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456f8e3, 97 },
    { 0x776f8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa666f8e8, 61 },
    { 0, 0 },
    { 0x21e6f8ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6f8f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f8f6, 77 },
    { 0x2a36f8f6, 90 },
    { 0xa3a6f8f8, 43 },
    { 0x56c6f8f7, 23 },
    { 0x3916f8fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d6f8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466f904, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f86f907, 33 },
    { 0x2d66f908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b6f90b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c06f90f, 33 },
    { 0x4e06f910, 98 },
    { 0, 0 },
    { 0xce56f912, 90 },
    { 0, 0 },
    { 0x4186f914, 98 },
    { 0x8346f914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2436f918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556f91b, 90 },
    { 0x3ae6f91b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b6f921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6f927, 98 },
    { 0x72e6f928, 49 },
    { 0x5096f929, 46 },
    { 0x9726f929, 78 },
    { 0x88f6f92b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b6f92e, 90 },
    { 0, 0 },
    { 0x4f16f930, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb6f93a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d6f93d, 33 },
    { 0x6b66f93e, 52 },
    { 0xd0d6f93d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf206f943, 98 },
    { 0xb0c6f943, 61 },
    { 0xbaf6f945, 75 },
    { 0xa496f946, 33 },
    { 0x3e26f944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd886f94a, 55 },
    { 0, 0 },
    { 0xa446f94c, 98 },
    { 0, 0 },
    { 0xaa06f94e, 23 },
    { 0x5166f94e, 81 },
    { 0xdbc6f950, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc6f954, 95 },
    { 0x22c6f955, 52 },
    { 0xe5a6f956, 61 },
    { 0x2776f954, 51 },
    { 0xec16f957, 35 },
    { 0xba6f957, 33 },
    { 0x9f66f95a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f6f95f, 33 },
    { 0xcee6f95f, 23 },
    { 0x99d6f961, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96f964, 78 },
    { 0x1396f965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e26f96c, 23 },
    { 0, 0 },
    { 0x4286f96e, 87 },
    { 0x1456f96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f6f972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e6f975, 38 },
    { 0xbc86f975, 61 },
    { 0x7466f977, 90 },
    { 0xa396f977, 61 },
    { 0x5de6f979, 95 },
    { 0x7176f977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f986, 88 },
    { 0x2096f987, 99 },
    { 0xad06f987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e6f990, 55 },
    { 0x4b26f991, 23 },
    { 0xe4e6f992, 23 },
    { 0x79f6f993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d6f999, 42 },
    { 0x2ac6f99a, 98 },
    { 0x9f56f999, 19 },
    { 0, 0 },
    { 0x54f6f99d, 61 },
    { 0xf266f99e, 41 },
    { 0x11d6f99f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c86f9a2, 98 },
    { 0, 0 },
    { 0x4dc6f9a4, 33 },
    { 0x3506f9a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda46f9a8, 33 },
    { 0xa7c6f9a9, 99 },
    { 0x8166f9aa, 98 },
    { 0, 0 },
    { 0x3356f9ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9256f9b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f26f9b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076f9be, 90 },
    { 0x8476f9bf, 34 },
    { 0x6ea6f9bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a6f9c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c56f9c7, 90 },
    { 0xb3c6f9c8, 87 },
    { 0x7d26f9c9, 99 },
    { 0x95a6f9ca, 90 },
    { 0x60e6f9ca, 66 },
    { 0x2e06f9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd606f9cf, 40 },
    { 0, 0 },
    { 0x49e6f9d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816f9d7, 76 },
    { 0x39f6f9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196f9ec, 67 },
    { 0x3af6f9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a6f9f0, 61 },
    { 0x5376f9f1, 61 },
    { 0, 0 },
    { 0x5bc6f9f3, 55 },
    { 0xb036f9f3, 26 },
    { 0x4f26f9f5, 90 },
    { 0xe9b6f9f6, 90 },
    { 0x2a66f9f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f6f9fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7176fa00, 56 },
    { 0x2836fa01, 95 },
    { 0x8cb6fa02, 33 },
    { 0, 0 },
    { 0xce46fa04, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf316fa08, 90 },
    { 0x52a6fa09, 49 },
    { 0, 0 },
    { 0x9ce6fa0b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686fa10, 88 },
    { 0x6a16fa10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc6fa14, 52 },
    { 0, 0 },
    { 0x6a66fa16, 90 },
    { 0x13a6fa16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7706fa1b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a6fa21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1216fa25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd6fa28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa6fa2f, 55 },
    { 0xdda6fa30, 61 },
    { 0xef56fa2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc86fa38, 95 },
    { 0x69d6fa39, 33 },
    { 0x2596fa3a, 98 },
    { 0x8856fa3b, 46 },
    { 0x7756fa3c, 33 },
    { 0, 0 },
    { 0x8f6fa3e, 95 },
    { 0xb5a6fa3f, 23 },
    { 0x23e6fa3f, 52 },
    { 0x8266fa3f, 87 },
    { 0x8216fa3e, 90 },
    { 0, 0 },
    { 0x7b66fa44, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846fa4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f06fa56, 91 },
    { 0x2696fa56, 68 },
    { 0x4186fa57, 90 },
    { 0x5046fa59, 90 },
    { 0xcd86fa57, 90 },
    { 0xc1b6fa57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6fa5f, 33 },
    { 0x87d6fa60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3246fa64, 84 },
    { 0, 0 },
    { 0xa0d6fa66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a16fa69, 34 },
    { 0, 0 },
    { 0xc136fa6b, 76 },
    { 0x2246fa6c, 93 },
    { 0, 0 },
    { 0xcda6fa6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b6fa79, 90 },
    { 0xbd6fa79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f56fa7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b6fa81, 95 },
    { 0xe516fa81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706fa88, 95 },
    { 0, 0 },
    { 0x9d06fa8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c6fa8d, 76 },
    { 0x1c76fa8e, 55 },
    { 0x74f6fa8f, 23 },
    { 0x65e6fa90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db6fa96, 90 },
    { 0xe036fa96, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b46fa9b, 53 },
    { 0x8bb6fa9b, 23 },
    { 0, 0 },
    { 0x7da6fa9e, 23 },
    { 0x1db6fa9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc066faa2, 77 },
    { 0xdba6faa2, 95 },
    { 0xeb66faa2, 98 },
    { 0x4726faa5, 33 },
    { 0, 0 },
    { 0x80a6faa7, 95 },
    { 0, 0 },
    { 0xc846faa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a6faac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6fab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06fab3, 100 },
    { 0xbef6fab3, 88 },
    { 0x4496fab5, 33 },
    { 0, 0 },
    { 0x4656fab7, 61 },
    { 0x9a96fab7, 95 },
    { 0xaf46fab9, 90 },
    { 0xef16faba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6fabe, 90 },
    { 0, 0 },
    { 0xf956fac0, 61 },
    { 0xea66fac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546fac5, 23 },
    { 0xf46fac6, 99 },
    { 0, 0 },
    { 0x4c06fac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e6facd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c46fad0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6fad4, 23 },
    { 0x5306fad4, 66 },
    { 0x5036fad6, 91 },
    { 0, 0 },
    { 0x43f6fad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b6fade, 98 },
    { 0, 0 },
    { 0x39f6fae0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d6fae5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266fae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56faef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2516faf2, 75 },
    { 0xf3c6faf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c6faf6, 55 },
    { 0x5396faf7, 23 },
    { 0xf8f6faf7, 61 },
    { 0x9ed6faf9, 43 },
    { 0xf6e6faf9, 23 },
    { 0xf5c6fafb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4006fafe, 23 },
    { 0, 0 },
    { 0x1066fb00, 23 },
    { 0x35a6fb01, 35 },
    { 0, 0 },
    { 0xbc86fb03, 71 },
    { 0xce56fb04, 98 },
    { 0xec06fb05, 34 },
    { 0, 0 },
    { 0x5166fb07, 33 },
    { 0x5786fb08, 95 },
    { 0, 0 },
    { 0x9776fb0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6fb0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6fb15, 61 },
    { 0x86fb16, 85 },
    { 0, 0 },
    { 0xd556fb18, 33 },
    { 0, 0 },
    { 0xc2b6fb1a, 90 },
    { 0x16b6fb1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e86fb1e, 33 },
    { 0x5ea6fb1f, 93 },
    { 0x7986fb1f, 98 },
    { 0x4586fb21, 98 },
    { 0xcc6fb20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e56fb26, 87 },
    { 0, 0 },
    { 0xf7a6fb28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6fb2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226fb2e, 67 },
    { 0, 0 },
    { 0xc2a6fb30, 52 },
    { 0xf666fb31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf6fb34, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd176fb39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606fb3d, 17 },
    { 0x366fb3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd6fb41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe256fb4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96fb4f, 23 },
    { 0xfec6fb4f, 55 },
    { 0x256fb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x466fb54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f66fb59, 93 },
    { 0x1b96fb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d6fb5e, 85 },
    { 0x7056fb5f, 61 },
    { 0x69b6fb5f, 98 },
    { 0xf1a6fb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6fb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06fb69, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5526fb6e, 87 },
    { 0xe0b6fb6f, 95 },
    { 0xcd26fb70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7166fb74, 78 },
    { 0x7916fb75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f86fb79, 76 },
    { 0xb736fb7a, 76 },
    { 0, 0 },
    { 0x1c36fb7c, 34 },
    { 0x4ff6fb7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa96fb81, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a86fb87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe516fb8a, 47 },
    { 0xcc16fb8b, 61 },
    { 0x7166fb8c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6246fb90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc06fb93, 66 },
    { 0, 0 },
    { 0xc3d6fb95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf446fb98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46fb9b, 84 },
    { 0x44f6fb9b, 85 },
    { 0, 0 },
    { 0x6516fb9e, 23 },
    { 0xed36fb9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da6fba4, 33 },
    { 0x6fba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426fba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6fbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706fbb1, 95 },
    { 0x7116fbb2, 95 },
    { 0xa826fbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6fbb9, 46 },
    { 0x2346fbba, 98 },
    { 0xf516fbba, 33 },
    { 0xd526fbbc, 95 },
    { 0x1db6fbbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc16fbc2, 76 },
    { 0x5026fbc2, 55 },
    { 0xec86fbc4, 90 },
    { 0xe856fbc4, 12 },
    { 0x706fbc6, 98 },
    { 0xb406fbc7, 90 },
    { 0xcaf6fbc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b6fbcb, 33 },
    { 0x3a26fbcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116fbd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe6fbd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d6fbd8, 91 },
    { 0x5cd6fbd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb046fbdc, 90 },
    { 0, 0 },
    { 0x11f6fbde, 61 },
    { 0xe2f6fbdf, 100 },
    { 0, 0 },
    { 0x4ed6fbe1, 61 },
    { 0xe3d6fbe2, 91 },
    { 0, 0 },
    { 0x4356fbe4, 23 },
    { 0x986fbe4, 76 },
    { 0x76fbe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f6fbe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe216fbed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3746fbf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26fbfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5636fc02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c6fc05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa546fc09, 88 },
    { 0xcbd6fc0a, 95 },
    { 0x1b56fc0b, 77 },
    { 0x9586fc0b, 98 },
    { 0xe686fc0d, 33 },
    { 0, 0 },
    { 0x63f6fc0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436fc15, 43 },
    { 0x8f06fc16, 90 },
    { 0x89b6fc15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e6fc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6fc27, 42 },
    { 0x80c6fc27, 95 },
    { 0xb1e6fc28, 99 },
    { 0x62e6fc2a, 85 },
    { 0x1226fc2b, 61 },
    { 0x1a56fc2c, 33 },
    { 0x61a6fc2d, 66 },
    { 0x6616fc2a, 68 },
    { 0x6a16fc2e, 34 },
    { 0x9326fc2b, 69 },
    { 0xc896fc2a, 33 },
    { 0xb6a6fc2b, 61 },
    { 0x9e66fc27, 90 },
    { 0xf516fc33, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6fc3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e6fc46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456fc4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4986fc4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a36fc55, 23 },
    { 0, 0 },
    { 0xc66fc57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6fc5a, 46 },
    { 0xe946fc5b, 90 },
    { 0x6a46fc5c, 90 },
    { 0x8606fc5c, 95 },
    { 0x7006fc5e, 98 },
    { 0x6e6fc5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6fc62, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf676fc68, 55 },
    { 0x4d66fc69, 95 },
    { 0, 0 },
    { 0xc066fc6b, 66 },
    { 0x4996fc6b, 95 },
    { 0x3ac6fc6d, 90 },
    { 0x8806fc6e, 61 },
    { 0xe696fc6e, 33 },
    { 0x7a46fc70, 45 },
    { 0xe1d6fc6d, 85 },
    { 0x1b86fc72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5536fc75, 99 },
    { 0x2ac6fc75, 61 },
    { 0xb4f6fc77, 33 },
    { 0xc076fc76, 95 },
    { 0, 0 },
    { 0x8b16fc7a, 63 },
    { 0, 0 },
    { 0x3656fc7c, 90 },
    { 0, 0 },
    { 0x2d36fc7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac6fc81, 93 },
    { 0xf216fc82, 100 },
    { 0x4ef6fc81, 75 },
    { 0, 0 },
    { 0x5eb6fc85, 23 },
    { 0xa536fc86, 61 },
    { 0, 0 },
    { 0xf3e6fc88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6fc8b, 98 },
    { 0x76c6fc8c, 52 },
    { 0x9f56fc8d, 23 },
    { 0, 0 },
    { 0xbaa6fc8f, 90 },
    { 0, 0 },
    { 0xc756fc91, 23 },
    { 0x5056fc91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a26fc9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc6fc9f, 34 },
    { 0xb796fca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca6fca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df6fcab, 33 },
    { 0xbdf6fcab, 90 },
    { 0, 0 },
    { 0x97f6fcae, 76 },
    { 0, 0 },
    { 0xc816fcb0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6fcb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316fcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586fcc9, 61 },
    { 0x7f46fcca, 76 },
    { 0xe4a6fccb, 51 },
    { 0xd086fccb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b96fccf, 66 },
    { 0xa3a6fcd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3656fcd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcd6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb6fcdc, 17 },
    { 0, 0 },
    { 0xa096fcde, 78 },
    { 0x23d6fcde, 84 },
    { 0xea26fcde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd326fce3, 33 },
    { 0xaa06fce4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa16fce7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a76fcea, 90 },
    { 0xdbc6fceb, 90 },
    { 0x61d6fcec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526fcef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd06fcf2, 81 },
    { 0x1f06fcf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6876fcf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f6fcfd, 81 },
    { 0, 0 },
    { 0xc376fcff, 100 },
    { 0, 0 },
    { 0x5576fd01, 17 },
    { 0xc476fd02, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa6fd08, 98 },
    { 0xd446fd09, 85 },
    { 0, 0 },
    { 0x3d16fd0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb6fd11, 66 },
    { 0x4006fd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb916fd15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686fd18, 90 },
    { 0x6dd6fd19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4566fd22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06fd25, 90 },
    { 0x4d06fd25, 91 },
    { 0x3b96fd27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e6fd2a, 33 },
    { 0x7b36fd2b, 61 },
    { 0xd736fd2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386fd30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e6fd34, 69 },
    { 0xfa16fd35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6fd3d, 76 },
    { 0xf566fd3d, 87 },
    { 0xf036fd3f, 90 },
    { 0x9ee6fd40, 96 },
    { 0xd206fd3d, 34 },
    { 0x31f6fd3f, 98 },
    { 0x9e26fd43, 93 },
    { 0x9166fd44, 76 },
    { 0xb4e6fd44, 61 },
    { 0x4a36fd45, 33 },
    { 0xdd86fd3d, 95 },
    { 0, 0 },
    { 0x5e86fd49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226fd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1666fd54, 88 },
    { 0x5e66fd55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a6fd5a, 56 },
    { 0x5966fd5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06fd5e, 68 },
    { 0x35f6fd5f, 90 },
    { 0xf0a6fd60, 84 },
    { 0x5816fd61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8456fd64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6fd6e, 61 },
    { 0x7456fd6f, 34 },
    { 0xcfa6fd70, 90 },
    { 0x22f6fd71, 78 },
    { 0, 0 },
    { 0xc096fd73, 98 },
    { 0x73d6fd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6756fd77, 76 },
    { 0xdbb6fd77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb6fd81, 69 },
    { 0, 0 },
    { 0x2626fd83, 66 },
    { 0xcd6fd84, 76 },
    { 0, 0 },
    { 0xb056fd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa76fd8b, 95 },
    { 0x9386fd8b, 96 },
    { 0, 0 },
    { 0x7bb6fd8e, 43 },
    { 0xb446fd8e, 33 },
    { 0, 0 },
    { 0xa2d6fd91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1886fd95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee6fd98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe806fda1, 22 },
    { 0xeba6fda2, 23 },
    { 0xa666fda3, 76 },
    { 0xc956fda2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc716fdaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a6fdba, 90 },
    { 0x4616fdbb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab76fdc2, 61 },
    { 0xe8f6fdc2, 64 },
    { 0x3576fdc4, 46 },
    { 0xb256fdc5, 52 },
    { 0, 0 },
    { 0x4676fdc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826fdca, 95 },
    { 0x62c6fdcb, 83 },
    { 0xe486fdcc, 98 },
    { 0xf916fdca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7666fdd1, 97 },
    { 0x76f6fdd2, 91 },
    { 0xf1a6fdd3, 40 },
    { 0xc6d6fdd1, 34 },
    { 0x38b6fdd5, 61 },
    { 0x4d66fdd5, 90 },
    { 0xd206fdd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2236fdda, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7736fddd, 23 },
    { 0x1da6fddd, 33 },
    { 0x7266fdde, 61 },
    { 0x9936fde0, 98 },
    { 0xd886fde1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326fde8, 95 },
    { 0xd016fde9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6fdec, 43 },
    { 0x2336fded, 23 },
    { 0x1cf6fded, 91 },
    { 0x4a06fdef, 90 },
    { 0, 0 },
    { 0x9f86fdf1, 23 },
    { 0xc136fdf2, 99 },
    { 0, 0 },
    { 0xe656fdf4, 90 },
    { 0, 0 },
    { 0x8b96fdf6, 95 },
    { 0x3946fdf7, 98 },
    { 0x42b6fdf7, 55 },
    { 0xdcd6fdf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496fdfc, 61 },
    { 0, 0 },
    { 0x6e26fdfe, 76 },
    { 0xd556fdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd136fe02, 90 },
    { 0, 0 },
    { 0x2226fe04, 77 },
    { 0x9eb6fe05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f6fe0b, 33 },
    { 0, 0 },
    { 0x13f6fe0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466fe16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6fe1f, 90 },
    { 0x766fe20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b96fe25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa046fe2a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4776fe2f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86fe33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6fe3a, 15 },
    { 0, 0 },
    { 0xb96fe3c, 85 },
    { 0x1216fe3d, 33 },
    { 0x4596fe3e, 98 },
    { 0x6136fe3e, 33 },
    { 0xe696fe40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fe43, 66 },
    { 0xd346fe43, 33 },
    { 0, 0 },
    { 0x70a6fe46, 80 },
    { 0, 0 },
    { 0x9df6fe48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9596fe4e, 33 },
    { 0, 0 },
    { 0x2766fe50, 84 },
    { 0, 0 },
    { 0x2986fe52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x856fe55, 99 },
    { 0x9306fe55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb836fe5b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d6fe62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce6fe67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da6fe6b, 95 },
    { 0, 0 },
    { 0x2a06fe6d, 100 },
    { 0xbd66fe6e, 38 },
    { 0, 0 },
    { 0xc526fe70, 95 },
    { 0xd196fe71, 99 },
    { 0, 0 },
    { 0x26b6fe73, 99 },
    { 0, 0 },
    { 0x16c6fe75, 66 },
    { 0x74f6fe76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826fe7d, 90 },
    { 0x2bb6fe7d, 38 },
    { 0xf696fe7f, 61 },
    { 0xbe26fe80, 66 },
    { 0x8476fe80, 24 },
    { 0x7826fe82, 90 },
    { 0, 0 },
    { 0x9716fe84, 33 },
    { 0x6de6fe85, 23 },
    { 0xea56fe84, 98 },
    { 0x5166fe87, 78 },
    { 0xc06fe88, 81 },
    { 0x2e26fe87, 34 },
    { 0xa236fe86, 87 },
    { 0, 0 },
    { 0x8bc6fe8c, 61 },
    { 0xc226fe8d, 98 },
    { 0x6aa6fe8e, 98 },
    { 0x1b66fe8e, 61 },
    { 0xf966fe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196fe93, 61 },
    { 0, 0 },
    { 0x4c16fe95, 34 },
    { 0xce26fe96, 85 },
    { 0x2a96fe96, 52 },
    { 0x6c06fe98, 90 },
    { 0xca16fe99, 76 },
    { 0xdc36fe99, 33 },
    { 0xa006fe96, 95 },
    { 0x2ee6fe96, 43 },
    { 0, 0 },
    { 0x5826fe9e, 95 },
    { 0, 0 },
    { 0xd1e6fea0, 90 },
    { 0x5c76fea0, 91 },
    { 0xb146fea2, 90 },
    { 0x6d76fea3, 23 },
    { 0xa5d6fea3, 43 },
    { 0x75d6fea5, 95 },
    { 0x3da6fea6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136feb5, 66 },
    { 0xf7e6feb6, 33 },
    { 0x4f66feb7, 43 },
    { 0x93a6feb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d16febf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9666fec2, 55 },
    { 0, 0 },
    { 0x7416fec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f6feca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb456fecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa346fed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa906fee4, 33 },
    { 0, 0 },
    { 0x4e16fee6, 75 },
    { 0, 0 },
    { 0xc3a6fee8, 55 },
    { 0, 0 },
    { 0xc696feea, 33 },
    { 0xd926feea, 95 },
    { 0, 0 },
    { 0xd536feed, 61 },
    { 0, 0 },
    { 0x3cb6feef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e6fef2, 23 },
    { 0xb9f6fef3, 87 },
    { 0x9d36fef4, 90 },
    { 0xdbc6fef5, 33 },
    { 0x69c6fef6, 40 },
    { 0x9376fef2, 98 },
    { 0xd9e6fef8, 43 },
    { 0x7446fef9, 95 },
    { 0x34b6fef9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fefd, 34 },
    { 0, 0 },
    { 0xbba6feff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a6ff02, 33 },
    { 0, 0 },
    { 0x4ad6ff04, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e6ff0b, 78 },
    { 0x25b6ff0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126ff10, 23 },
    { 0, 0 },
    { 0x2ea6ff12, 61 },
    { 0x9a76ff13, 61 },
    { 0xcd6ff14, 84 },
    { 0xdc86ff15, 90 },
    { 0, 0 },
    { 0x636ff17, 33 },
    { 0x23e6ff18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b6ff1d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6ff21, 90 },
    { 0, 0 },
    { 0x4e6ff23, 33 },
    { 0x7026ff23, 76 },
    { 0x2f56ff25, 98 },
    { 0x1e26ff24, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e6ff29, 34 },
    { 0x2e26ff2a, 91 },
    { 0, 0 },
    { 0x1156ff2c, 23 },
    { 0xf9c6ff2d, 52 },
    { 0x4916ff2d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea6ff33, 90 },
    { 0, 0 },
    { 0x36ff35, 91 },
    { 0, 0 },
    { 0x1386ff37, 61 },
    { 0, 0 },
    { 0x95a6ff39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a16ff41, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56ff44, 61 },
    { 0, 0 },
    { 0xe136ff46, 43 },
    { 0x4996ff46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b6ff4c, 90 },
    { 0x83f6ff4d, 46 },
    { 0xde56ff4e, 61 },
    { 0x9dd6ff4f, 98 },
    { 0x3376ff4d, 90 },
    { 0xcbe6ff50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe356ff55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9396ff59, 43 },
    { 0, 0 },
    { 0xf5f6ff5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c6ff63, 95 },
    { 0, 0 },
    { 0x6566ff65, 81 },
    { 0xa6ff65, 61 },
    { 0, 0 },
    { 0xf416ff68, 93 },
    { 0, 0 },
    { 0x71e6ff6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ff6e, 95 },
    { 0, 0 },
    { 0x76ff70, 23 },
    { 0, 0 },
    { 0x9366ff72, 23 },
    { 0x7066ff73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7376ff76, 61 },
    { 0x4ad6ff76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3176ff7b, 55 },
    { 0x70f6ff7b, 46 },
    { 0xf326ff7d, 78 },
    { 0x42b6ff7d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36ff81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956ff86, 98 },
    { 0xa9e6ff86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16ff8c, 46 },
    { 0x9aa6ff8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16ff92, 61 },
    { 0, 0 },
    { 0x506ff94, 90 },
    { 0x9116ff94, 46 },
    { 0x51b6ff96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b76ff99, 98 },
    { 0x55d6ff9a, 65 },
    { 0x5386ff9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6406ff9f, 76 },
    { 0xbba6ff9f, 90 },
    { 0, 0 },
    { 0x2fd6ffa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc6ffa7, 61 },
    { 0, 0 },
    { 0x8286ffa9, 53 },
    { 0x1b46ffaa, 76 },
    { 0x536ffa9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646ffaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b6ffb5, 90 },
    { 0x596ffb6, 90 },
    { 0x3336ffb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6ffbd, 17 },
    { 0, 0 },
    { 0x536ffbf, 23 },
    { 0xede6ffc0, 90 },
    { 0x1f36ffc1, 100 },
    { 0, 0 },
    { 0x5326ffc3, 95 },
    { 0x5176ffc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6626ffc7, 90 },
    { 0xc866ffc8, 99 },
    { 0, 0 },
    { 0x5be6ffca, 76 },
    { 0x80e6ffca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e76ffcf, 68 },
    { 0x19f6ffd0, 91 },
    { 0x6506ffd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa506ffd8, 17 },
    { 0, 0 },
    { 0xb786ffda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6ffe2, 95 },
    { 0, 0 },
    { 0xeda6ffe4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ffed, 19 },
    { 0x67b6ffee, 85 },
    { 0x9ca6ffee, 84 },
    { 0xd16fff0, 78 },
    { 0xe2d6fff1, 90 },
    { 0xf986fff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be6fff6, 46 },
    { 0x16d6fff7, 23 },
    { 0x5ec6fff6, 98 },
    { 0xed06fff7, 61 },
    { 0x7316fffa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516fffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68770000, 98 },
    { 0x7dc70001, 98 },
    { 0, 0 },
    { 0x51070003, 84 },
    { 0, 0 },
    { 0x77070005, 34 },
    { 0x9d670006, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb987000b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4170011, 98 },
    { 0, 0 },
    { 0x96770013, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c170016, 90 },
    { 0, 0 },
    { 0xc7970018, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157001b, 23 },
    { 0x8917001b, 64 },
    { 0, 0 },
    { 0x62f7001e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda870022, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3870028, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a7002e, 34 },
    { 0, 0 },
    { 0x3aa70030, 93 },
    { 0x2d370030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d70034, 88 },
    { 0x75370034, 46 },
    { 0xd2370036, 55 },
    { 0x43a70036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac7003a, 95 },
    { 0xe8a7003b, 77 },
    { 0x7537003c, 43 },
    { 0x4677003b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8470043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24370046, 45 },
    { 0x3dc70046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7004b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff7004f, 33 },
    { 0x4747004f, 83 },
    { 0x7fa70051, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac70058, 90 },
    { 0, 0 },
    { 0x28a7005a, 90 },
    { 0, 0 },
    { 0xa967005c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc70060, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd470063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc70067, 95 },
    { 0xf1170068, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a07006b, 81 },
    { 0x5d7006c, 77 },
    { 0xd5f7006d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99470071, 33 },
    { 0xbce70072, 76 },
    { 0, 0 },
    { 0xfa770074, 76 },
    { 0xb0e70075, 46 },
    { 0x47670076, 90 },
    { 0x72870077, 46 },
    { 0, 0 },
    { 0x76170079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7007d, 90 },
    { 0xafa7007e, 95 },
    { 0xd47007d, 55 },
    { 0x4bb70080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca170083, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f70086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c570089, 68 },
    { 0x57370089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7008d, 61 },
    { 0xf277008d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc70093, 34 },
    { 0x53870093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd117009a, 90 },
    { 0, 0 },
    { 0xad57009c, 66 },
    { 0x2bd7009d, 95 },
    { 0, 0 },
    { 0x937009f, 61 },
    { 0x798700a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710700aa, 66 },
    { 0, 0 },
    { 0xd0c700ac, 43 },
    { 0x1fb700ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0700b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42700b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d700b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664700bc, 95 },
    { 0, 0 },
    { 0x1d0700be, 97 },
    { 0xa9b700bf, 76 },
    { 0x575700c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57700c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f700c9, 87 },
    { 0x3a9700ca, 95 },
    { 0xd49700ca, 33 },
    { 0x39e700ca, 90 },
    { 0xbd3700cd, 87 },
    { 0x498700ce, 98 },
    { 0, 0 },
    { 0x59a700d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad700d5, 33 },
    { 0xb2e700d5, 55 },
    { 0x60b700d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382700db, 33 },
    { 0, 0 },
    { 0xced700dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1700e0, 33 },
    { 0, 0 },
    { 0xf6a700e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x74700e5, 33 },
    { 0x9bb700e5, 95 },
    { 0x59b700e6, 98 },
    { 0x17b700e8, 43 },
    { 0x4a1700e9, 95 },
    { 0x727700e9, 98 },
    { 0, 0 },
    { 0x83a700ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79700f0, 82 },
    { 0, 0 },
    { 0x8b7700f2, 23 },
    { 0, 0 },
    { 0xd68700f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933700fa, 76 },
    { 0x99b700fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe700fe, 46 },
    { 0xda7700ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58170103, 23 },
    { 0x94570103, 95 },
    { 0xfb470105, 23 },
    { 0x5e370106, 65 },
    { 0xd7e70107, 23 },
    { 0x21a70107, 33 },
    { 0x7470107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53670114, 100 },
    { 0x25770114, 33 },
    { 0x24f70116, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c70119, 61 },
    { 0, 0 },
    { 0x4ce7011b, 95 },
    { 0x6997011c, 98 },
    { 0xd2c7011b, 43 },
    { 0xef17011c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca070126, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6670129, 75 },
    { 0x7db70129, 100 },
    { 0x5f7012a, 19 },
    { 0x7012c, 98 },
    { 0x22c7012d, 98 },
    { 0x5487012d, 90 },
    { 0xbe37012c, 40 },
    { 0x8807012c, 84 },
    { 0x8577012d, 95 },
    { 0x8e170129, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22970138, 91 },
    { 0xef570139, 95 },
    { 0x99c70138, 90 },
    { 0xb5a7013b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37013e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d70148, 98 },
    { 0xd1e70148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf497014c, 97 },
    { 0x15d7014d, 76 },
    { 0x7437014d, 91 },
    { 0xfbe7014d, 95 },
    { 0, 0 },
    { 0xc670151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56270154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4370157, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc7015a, 90 },
    { 0, 0 },
    { 0xbe97015c, 87 },
    { 0xfef7015d, 17 },
    { 0x9a87015d, 34 },
    { 0x3ad7015f, 98 },
    { 0x9a67015d, 98 },
    { 0x4ef70161, 23 },
    { 0x8f670162, 33 },
    { 0x36f70162, 61 },
    { 0x27270163, 61 },
    { 0xaa77015f, 95 },
    { 0xf8770161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e70169, 61 },
    { 0x35f7016a, 23 },
    { 0x51570169, 75 },
    { 0x1c77016c, 33 },
    { 0, 0 },
    { 0xa1f7016e, 66 },
    { 0, 0 },
    { 0x16c70170, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6070173, 90 },
    { 0, 0 },
    { 0xea270175, 97 },
    { 0x59470176, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a7017a, 66 },
    { 0, 0 },
    { 0x167017c, 91 },
    { 0x6f67017c, 23 },
    { 0x10b7017e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e70182, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc70185, 18 },
    { 0xb7770186, 95 },
    { 0xa2f70187, 90 },
    { 0xbb270188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7007018b, 84 },
    { 0x1217018c, 65 },
    { 0, 0 },
    { 0xfb87018e, 66 },
    { 0xe2e7018f, 100 },
    { 0x8fd7018f, 33 },
    { 0x5c370191, 61 },
    { 0x4aa7018f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23170196, 33 },
    { 0, 0 },
    { 0x8c870198, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb7019c, 93 },
    { 0xc67019d, 17 },
    { 0x1447019e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a701a4, 87 },
    { 0xb28701a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a701a8, 90 },
    { 0x427701a9, 95 },
    { 0xe3f701aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73701af, 47 },
    { 0xb76701b0, 76 },
    { 0x520701b1, 90 },
    { 0xa10701b1, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da701b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6701bf, 90 },
    { 0x506701c0, 66 },
    { 0xa95701c0, 67 },
    { 0x50d701c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2701cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851701cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc701d3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07701d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866701dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02701df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb701e2, 95 },
    { 0, 0 },
    { 0xcb6701e4, 33 },
    { 0x32701e4, 55 },
    { 0xde3701e5, 66 },
    { 0xa81701e6, 76 },
    { 0x232701e8, 33 },
    { 0x2ec701e8, 98 },
    { 0xbff701ea, 34 },
    { 0xab3701ea, 34 },
    { 0, 0 },
    { 0x6f5701ed, 77 },
    { 0, 0 },
    { 0xb77701ef, 77 },
    { 0x355701f0, 90 },
    { 0x536701f0, 46 },
    { 0xce7701ef, 95 },
    { 0xea7701f3, 34 },
    { 0xc62701f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26701f7, 100 },
    { 0x6cb701f8, 36 },
    { 0x51b701f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d70200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e470203, 98 },
    { 0x71a70204, 76 },
    { 0xcfc70205, 95 },
    { 0xddc70205, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a7020e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f70212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46670215, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f70218, 43 },
    { 0, 0 },
    { 0x7ac7021a, 98 },
    { 0xb6d7021a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84970222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc70225, 23 },
    { 0x41370225, 98 },
    { 0x6ed70227, 14 },
    { 0x4bc70227, 66 },
    { 0x13570227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77022f, 76 },
    { 0x3a70230, 95 },
    { 0xa6670230, 93 },
    { 0, 0 },
    { 0x20f70233, 23 },
    { 0x86170234, 98 },
    { 0, 0 },
    { 0xb5270236, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70239, 23 },
    { 0x3f97023a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137023f, 47 },
    { 0xd057023f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c70243, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe70246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e17024e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf70252, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf870256, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e17025c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf70261, 33 },
    { 0xbc570262, 61 },
    { 0x2de70262, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3070268, 34 },
    { 0x4a870269, 90 },
    { 0, 0 },
    { 0x6187026b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d7026f, 99 },
    { 0, 0 },
    { 0x4c70271, 33 },
    { 0, 0 },
    { 0x98d70273, 95 },
    { 0xcbb70274, 64 },
    { 0, 0 },
    { 0xb4e70276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb17027e, 90 },
    { 0, 0 },
    { 0xad70280, 76 },
    { 0x6b270280, 98 },
    { 0x19b70282, 33 },
    { 0, 0 },
    { 0xada70284, 96 },
    { 0x5a770284, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8470288, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d7028d, 90 },
    { 0xddf7028e, 98 },
    { 0, 0 },
    { 0xf7670290, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b470297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607029a, 23 },
    { 0, 0 },
    { 0x3817029c, 23 },
    { 0x6657029d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x440702a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59702a3, 61 },
    { 0xff702a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9702aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87702b3, 61 },
    { 0xeff702b3, 61 },
    { 0x7ea702b5, 76 },
    { 0x69f702b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0702b9, 81 },
    { 0x540702b9, 87 },
    { 0x4fd702bb, 90 },
    { 0x7f4702bc, 61 },
    { 0xbfb702bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49702c1, 23 },
    { 0x6fa702c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5702c6, 87 },
    { 0, 0 },
    { 0x659702c8, 52 },
    { 0xde7702c9, 23 },
    { 0, 0 },
    { 0x95c702cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4702d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb702d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf702dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1702e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed702ec, 98 },
    { 0xb3702ed, 98 },
    { 0x3d702ee, 17 },
    { 0, 0 },
    { 0x5eb702f0, 33 },
    { 0xb09702f1, 99 },
    { 0, 0 },
    { 0x12b702f3, 85 },
    { 0, 0 },
    { 0xd70702f5, 33 },
    { 0xe24702f6, 76 },
    { 0xf59702f7, 67 },
    { 0x7a3702f6, 95 },
    { 0x539702f9, 52 },
    { 0, 0 },
    { 0x6a3702fb, 76 },
    { 0x41e702fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d270300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7170305, 69 },
    { 0, 0 },
    { 0xc0b70307, 76 },
    { 0x87d70307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a7030d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b70311, 33 },
    { 0x14570311, 90 },
    { 0x57670313, 93 },
    { 0xd4970314, 61 },
    { 0x7bf70315, 87 },
    { 0xb3370313, 98 },
    { 0xbd170311, 23 },
    { 0xc3870318, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa717031c, 55 },
    { 0xfec7031d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b70321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2570327, 98 },
    { 0xb1470328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7032c, 77 },
    { 0, 0 },
    { 0x7317032e, 61 },
    { 0, 0 },
    { 0xd1870330, 61 },
    { 0x5b670331, 33 },
    { 0, 0 },
    { 0xfa870333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a970337, 43 },
    { 0x9e270338, 33 },
    { 0xda70339, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d670345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4397034a, 23 },
    { 0, 0 },
    { 0x4467034c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f07034f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9970352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b070355, 90 },
    { 0x60c70356, 33 },
    { 0x61870357, 61 },
    { 0x79e70357, 90 },
    { 0xe8470355, 98 },
    { 0, 0 },
    { 0x9e57035b, 90 },
    { 0x9a77035c, 90 },
    { 0xd037035c, 90 },
    { 0, 0 },
    { 0xd367035f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd70364, 87 },
    { 0x76a70365, 53 },
    { 0x7dd70366, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7036a, 52 },
    { 0xd807036b, 77 },
    { 0xc877036c, 90 },
    { 0x4b7036d, 23 },
    { 0x4f87036d, 84 },
    { 0xb827036f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29570374, 87 },
    { 0xd4f70375, 68 },
    { 0xbd770376, 97 },
    { 0xc6070376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e7037c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20570384, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7038a, 34 },
    { 0xdd97038a, 33 },
    { 0x3f17038a, 88 },
    { 0x3b97038b, 66 },
    { 0x4c47038e, 43 },
    { 0x8937038a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c70392, 78 },
    { 0x7dd70393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32870396, 90 },
    { 0, 0 },
    { 0x22670398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497039c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c703a2, 95 },
    { 0, 0 },
    { 0x8f5703a4, 96 },
    { 0, 0 },
    { 0x7b7703a6, 33 },
    { 0x61a703a7, 61 },
    { 0x236703a8, 46 },
    { 0xf3703a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e703ac, 61 },
    { 0xa21703ac, 23 },
    { 0x144703ae, 91 },
    { 0x914703af, 61 },
    { 0x24f703b0, 61 },
    { 0xf83703b0, 32 },
    { 0x6f9703b2, 91 },
    { 0xf0b703b3, 93 },
    { 0x44f703b4, 34 },
    { 0xbfa703b3, 85 },
    { 0xf6a703b5, 33 },
    { 0xeaa703ac, 33 },
    { 0xef2703b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7703bf, 64 },
    { 0x16a703c0, 85 },
    { 0x9d9703c1, 90 },
    { 0, 0 },
    { 0xe4703c3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7703c6, 23 },
    { 0x39b703c7, 91 },
    { 0x530703c8, 98 },
    { 0x435703c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x215703cc, 33 },
    { 0, 0 },
    { 0xced703ce, 61 },
    { 0, 0 },
    { 0xacb703d0, 55 },
    { 0, 0 },
    { 0xd1a703d2, 97 },
    { 0, 0 },
    { 0x142703d4, 96 },
    { 0x8de703d5, 98 },
    { 0, 0 },
    { 0x4a5703d7, 81 },
    { 0xb83703d7, 46 },
    { 0x6e703d9, 46 },
    { 0xc43703d9, 76 },
    { 0, 0 },
    { 0xe2a703dc, 65 },
    { 0, 0 },
    { 0xddf703de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2703eb, 61 },
    { 0, 0 },
    { 0x230703ed, 46 },
    { 0x431703ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08703f1, 90 },
    { 0x4e5703f2, 33 },
    { 0, 0 },
    { 0xb92703f4, 85 },
    { 0xe52703f5, 46 },
    { 0, 0 },
    { 0xbfd703f7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33703fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a370400, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b70403, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b70406, 33 },
    { 0x40270407, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217040b, 66 },
    { 0xed07040c, 56 },
    { 0xf9e7040d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3970410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32070414, 61 },
    { 0x1e70415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c7041a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7041d, 33 },
    { 0, 0 },
    { 0x3fc7041f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d70425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64970429, 55 },
    { 0xdb570429, 33 },
    { 0x91c7042b, 46 },
    { 0xdf97042c, 33 },
    { 0x9827042b, 96 },
    { 0x5447042e, 33 },
    { 0xaa87042e, 61 },
    { 0xff37042b, 90 },
    { 0x7cf70430, 90 },
    { 0x4387042c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c070438, 81 },
    { 0xccc70438, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9637043c, 99 },
    { 0xd507043d, 96 },
    { 0xf757043e, 55 },
    { 0x1497043f, 61 },
    { 0xcc77043f, 55 },
    { 0xdb470441, 52 },
    { 0xde77043f, 92 },
    { 0x76470442, 98 },
    { 0xd2f70443, 33 },
    { 0xbeb70445, 61 },
    { 0x9dd70446, 91 },
    { 0xd8870447, 66 },
    { 0x35570447, 95 },
    { 0xa9970443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c77044c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d70450, 47 },
    { 0x8f870451, 66 },
    { 0, 0 },
    { 0x76f70453, 95 },
    { 0, 0 },
    { 0x69570455, 95 },
    { 0xfd070456, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa7045d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1870467, 90 },
    { 0x6e670468, 97 },
    { 0x78870469, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c7046f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270475, 76 },
    { 0x68b70475, 95 },
    { 0x4bc70475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67047b, 23 },
    { 0x1f27047b, 55 },
    { 0, 0 },
    { 0xeaf7047e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf570481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65570487, 95 },
    { 0x30f70487, 95 },
    { 0x74470488, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17048c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb570499, 33 },
    { 0x6a07049a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757049d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe704a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be704a8, 46 },
    { 0x906704a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x917704ac, 64 },
    { 0x24d704ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2704b1, 98 },
    { 0, 0 },
    { 0x585704b3, 34 },
    { 0x705704b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f704b8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd704bc, 55 },
    { 0xe94704bc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb704c1, 76 },
    { 0xab9704c1, 33 },
    { 0x7c9704c3, 33 },
    { 0xf84704c4, 43 },
    { 0x8e3704c4, 55 },
    { 0x4da704c6, 76 },
    { 0x34d704c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e704cc, 61 },
    { 0xd9d704cd, 61 },
    { 0, 0 },
    { 0xc66704cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7704d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86704d6, 90 },
    { 0, 0 },
    { 0xaa7704d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14704db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4704e0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x873704e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1704e7, 23 },
    { 0, 0 },
    { 0x26d704e9, 52 },
    { 0xd77704ea, 33 },
    { 0xd5d704eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4704f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402704f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f704fd, 95 },
    { 0, 0 },
    { 0xa92704ff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf70504, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7050b, 61 },
    { 0, 0 },
    { 0x6537050d, 93 },
    { 0x2e67050d, 85 },
    { 0x1517050d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a70514, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda470518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137051b, 34 },
    { 0xfbb7051b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a70520, 46 },
    { 0xa7370520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a970529, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4770530, 65 },
    { 0xd2470530, 90 },
    { 0x1b270530, 33 },
    { 0x41670533, 61 },
    { 0x21070534, 61 },
    { 0xa1670535, 90 },
    { 0xe9070536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe7053a, 95 },
    { 0x2337053b, 98 },
    { 0x13e7053c, 33 },
    { 0x42f7053a, 98 },
    { 0x9647053e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2170543, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab70546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b70549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7317054c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287054f, 43 },
    { 0, 0 },
    { 0x75670551, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f70557, 95 },
    { 0, 0 },
    { 0x13a70559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c17055d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58170565, 19 },
    { 0xa7170566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda370569, 23 },
    { 0xb8e70569, 33 },
    { 0, 0 },
    { 0x9e47056c, 67 },
    { 0, 0 },
    { 0xfc67056e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a70571, 33 },
    { 0x47b70572, 55 },
    { 0xd0c70572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a470577, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7057b, 90 },
    { 0xea07057c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff7057f, 55 },
    { 0, 0 },
    { 0x33570581, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa7058a, 93 },
    { 0, 0 },
    { 0x1b37058c, 33 },
    { 0, 0 },
    { 0x3e47058e, 95 },
    { 0xc4a7058f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89270592, 95 },
    { 0x55e70592, 33 },
    { 0x4f570594, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175705a5, 88 },
    { 0x2f1705a6, 68 },
    { 0xb02705a6, 66 },
    { 0xb91705a8, 66 },
    { 0xd50705a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27705ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6705b0, 90 },
    { 0x68f705b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52705be, 87 },
    { 0x11f705bf, 55 },
    { 0x9d8705c0, 61 },
    { 0xa81705be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe705ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0705d5, 33 },
    { 0x6a3705d5, 87 },
    { 0xb6705d7, 43 },
    { 0x827705d8, 78 },
    { 0x20a705d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858705df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7705f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d705f5, 23 },
    { 0x787705f5, 85 },
    { 0x5df705f7, 90 },
    { 0x9d8705f8, 34 },
    { 0x226705f5, 85 },
    { 0x4cb705f6, 61 },
    { 0xd3d705fb, 66 },
    { 0xdf3705fa, 77 },
    { 0xa56705fd, 98 },
    { 0x9e705f9, 95 },
    { 0x7b705fc, 95 },
    { 0, 0 },
    { 0x46470601, 46 },
    { 0, 0 },
    { 0x6ae70603, 33 },
    { 0x43b70604, 86 },
    { 0x5d370604, 33 },
    { 0xe0770606, 43 },
    { 0x42c70606, 33 },
    { 0x6b070608, 90 },
    { 0x5a770604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767060f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42570613, 23 },
    { 0x5ae70614, 61 },
    { 0x42270615, 81 },
    { 0x5b170613, 34 },
    { 0xbf170617, 26 },
    { 0xd3670618, 55 },
    { 0x8be70615, 61 },
    { 0xe9a7061a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81170620, 90 },
    { 0x2ae70621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe370626, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe857062c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5270630, 59 },
    { 0xbd70631, 52 },
    { 0x68870632, 90 },
    { 0x23270633, 23 },
    { 0, 0 },
    { 0x66670635, 61 },
    { 0, 0 },
    { 0xb5470637, 23 },
    { 0x64d70637, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a870644, 90 },
    { 0, 0 },
    { 0xa4b70646, 90 },
    { 0x6a770647, 87 },
    { 0, 0 },
    { 0xdaf70649, 61 },
    { 0xac47064a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e070651, 99 },
    { 0x96670651, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64470657, 33 },
    { 0xf6c70658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd977065b, 100 },
    { 0x5ba7065c, 23 },
    { 0xdd57065c, 76 },
    { 0, 0 },
    { 0xa777065f, 98 },
    { 0, 0 },
    { 0xf8370661, 61 },
    { 0xb2770661, 87 },
    { 0xbe270662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b70668, 34 },
    { 0x8d70669, 90 },
    { 0x7cd7066a, 23 },
    { 0x4f57066b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c7066e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9370673, 90 },
    { 0, 0 },
    { 0xc3170675, 84 },
    { 0xd0c70676, 61 },
    { 0xb0070677, 99 },
    { 0xbfc70678, 23 },
    { 0, 0 },
    { 0x1f37067a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817067f, 23 },
    { 0x9a370680, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba070686, 75 },
    { 0x1c270687, 90 },
    { 0xa3070686, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47470694, 100 },
    { 0, 0 },
    { 0x80470696, 95 },
    { 0xb8a70697, 90 },
    { 0xe0070696, 17 },
    { 0x37570699, 91 },
    { 0x1ab7069a, 90 },
    { 0x5af7069b, 33 },
    { 0x3667069b, 33 },
    { 0xcc07069d, 33 },
    { 0x4be70699, 23 },
    { 0xee77069f, 93 },
    { 0x1f17069a, 95 },
    { 0x6a0706a1, 61 },
    { 0, 0 },
    { 0x404706a3, 33 },
    { 0xcfb706a4, 99 },
    { 0, 0 },
    { 0x69a706a6, 87 },
    { 0, 0 },
    { 0xdf3706a8, 17 },
    { 0xdd8706a9, 61 },
    { 0xf19706a9, 55 },
    { 0xe46706a8, 33 },
    { 0xdb9706ac, 46 },
    { 0x51a706ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d706b1, 61 },
    { 0, 0 },
    { 0x3ca706b3, 100 },
    { 0, 0 },
    { 0xd69706b5, 61 },
    { 0xabe706b6, 76 },
    { 0x735706b7, 85 },
    { 0x8c4706b7, 76 },
    { 0xea6706b6, 61 },
    { 0x755706ba, 90 },
    { 0x9fe706bb, 23 },
    { 0xb49706b6, 95 },
    { 0xc6f706ba, 90 },
    { 0x77d706bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d706c1, 90 },
    { 0x623706c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6706c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23706ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e706d1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6706d4, 91 },
    { 0xe49706d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36706e0, 17 },
    { 0x8f0706e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7706e5, 88 },
    { 0xf83706e6, 61 },
    { 0xedb706e5, 90 },
    { 0, 0 },
    { 0x652706e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e706f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19706f3, 46 },
    { 0xc32706f4, 90 },
    { 0xeca706f4, 90 },
    { 0xb3d706f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377706fa, 33 },
    { 0x629706fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7706ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d70708, 98 },
    { 0x46470709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3567070f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f70717, 90 },
    { 0x3c070718, 33 },
    { 0, 0 },
    { 0xf837071a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a7071e, 46 },
    { 0xa967071f, 55 },
    { 0xee770720, 33 },
    { 0x7c570720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f270725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b770728, 61 },
    { 0x78170729, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a97072f, 23 },
    { 0, 0 },
    { 0x98f70731, 61 },
    { 0, 0 },
    { 0x8770733, 95 },
    { 0x9d970734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee970737, 81 },
    { 0x9e570738, 61 },
    { 0x8ea70739, 33 },
    { 0x37370738, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7073d, 67 },
    { 0xbd37073d, 33 },
    { 0x24d7073f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e70746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57074b, 23 },
    { 0x9c77074c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94070750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75170755, 34 },
    { 0x47270755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e70759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7077075c, 33 },
    { 0, 0 },
    { 0xea97075e, 77 },
    { 0xddd7075e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8570767, 99 },
    { 0, 0 },
    { 0x49f70769, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a970773, 87 },
    { 0xf6770773, 85 },
    { 0, 0 },
    { 0xa1e70776, 98 },
    { 0x63170776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e7077c, 66 },
    { 0xf607077c, 43 },
    { 0, 0 },
    { 0x82d7077f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57570782, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d170785, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17078a, 93 },
    { 0, 0 },
    { 0x2307078c, 95 },
    { 0x8f67078d, 98 },
    { 0xd8d7078c, 93 },
    { 0, 0 },
    { 0x23870790, 34 },
    { 0xac570791, 81 },
    { 0xc5f70791, 43 },
    { 0xaed70792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c70796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b7079e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x205707a1, 46 },
    { 0x29b707a2, 61 },
    { 0xed4707a3, 53 },
    { 0, 0 },
    { 0xa68707a5, 87 },
    { 0x412707a6, 67 },
    { 0x667707a7, 91 },
    { 0x2c2707a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a707ab, 52 },
    { 0x9a9707ab, 34 },
    { 0x39707ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1707b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b707b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706707bc, 90 },
    { 0x67d707bd, 98 },
    { 0x670707be, 33 },
    { 0xec8707be, 55 },
    { 0xa53707c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7707c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee707c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c707e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a707e7, 90 },
    { 0xeea707e7, 61 },
    { 0xc94707e9, 61 },
    { 0, 0 },
    { 0x7ce707eb, 90 },
    { 0, 0 },
    { 0x1ac707ed, 76 },
    { 0xc2c707ed, 33 },
    { 0, 0 },
    { 0xf34707f0, 23 },
    { 0x626707f1, 23 },
    { 0xa46707f2, 23 },
    { 0x4f4707f2, 99 },
    { 0x23e707f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b707f8, 34 },
    { 0x21f707f8, 90 },
    { 0x676707fa, 75 },
    { 0xb8707f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98707fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab70807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6807080a, 61 },
    { 0, 0 },
    { 0x2f87080c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c70810, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0070817, 82 },
    { 0x3870818, 33 },
    { 0xdd770817, 90 },
    { 0, 0 },
    { 0x18a7081b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3670828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657082b, 33 },
    { 0, 0 },
    { 0x4ad7082d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0870833, 55 },
    { 0xccc70834, 90 },
    { 0, 0 },
    { 0x58070836, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2670840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a70843, 34 },
    { 0, 0 },
    { 0xed870845, 93 },
    { 0, 0 },
    { 0x93170847, 93 },
    { 0, 0 },
    { 0x7c370849, 87 },
    { 0xc087084a, 61 },
    { 0, 0 },
    { 0x5717084c, 65 },
    { 0x5ee7084d, 98 },
    { 0x4387084d, 95 },
    { 0x2197084f, 98 },
    { 0x1a87084d, 98 },
    { 0xeac7084d, 96 },
    { 0xb7870852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b70856, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3770859, 91 },
    { 0xbf970859, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec70861, 33 },
    { 0xed770862, 91 },
    { 0xe8170862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa70866, 59 },
    { 0xb1270867, 61 },
    { 0, 0 },
    { 0x87a70869, 100 },
    { 0x76f7086a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e17086f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66170872, 61 },
    { 0x6c470872, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x28970876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca370879, 43 },
    { 0, 0 },
    { 0x1207087b, 76 },
    { 0xf787087c, 95 },
    { 0x1807087d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c70881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc70885, 80 },
    { 0x50070886, 93 },
    { 0xfe170886, 87 },
    { 0x8c070888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87088b, 75 },
    { 0x6e47088c, 61 },
    { 0, 0 },
    { 0xc737088e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3270896, 33 },
    { 0x28e70897, 95 },
    { 0x7aa70897, 46 },
    { 0x3ad70899, 91 },
    { 0x4267089a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97089d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1708aa, 85 },
    { 0x484708ab, 17 },
    { 0xf96708ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db708b0, 97 },
    { 0x787708b1, 100 },
    { 0x892708b0, 82 },
    { 0, 0 },
    { 0x64e708b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28708b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800708bd, 46 },
    { 0xc8f708be, 61 },
    { 0, 0 },
    { 0xd0f708c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30708c3, 23 },
    { 0x684708c4, 93 },
    { 0, 0 },
    { 0xcfb708c6, 76 },
    { 0xb82708c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0708ca, 55 },
    { 0x488708ca, 90 },
    { 0x4d708cc, 98 },
    { 0x77a708cc, 55 },
    { 0, 0 },
    { 0xf3b708cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d708d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28708d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47708db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f708e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16708e3, 76 },
    { 0, 0 },
    { 0xf89708e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66708ec, 55 },
    { 0xdc2708ec, 55 },
    { 0xb1d708ee, 90 },
    { 0xf14708ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530708f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f708f8, 77 },
    { 0, 0 },
    { 0xdc3708fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60708fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25270904, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe170908, 98 },
    { 0x1b370908, 55 },
    { 0xe7370908, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a97090d, 61 },
    { 0xb1e7090e, 23 },
    { 0, 0 },
    { 0xc9370910, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac70913, 23 },
    { 0, 0 },
    { 0x44170915, 99 },
    { 0, 0 },
    { 0x78470917, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7091b, 43 },
    { 0xaa57091c, 61 },
    { 0, 0 },
    { 0xe9c7091e, 75 },
    { 0, 0 },
    { 0x68b70920, 90 },
    { 0xf2d70921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14570925, 80 },
    { 0x29470926, 55 },
    { 0xebd70926, 90 },
    { 0xbb70928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7092e, 17 },
    { 0, 0 },
    { 0x21f70930, 52 },
    { 0xc6970930, 80 },
    { 0, 0 },
    { 0xefb70933, 23 },
    { 0x15470934, 99 },
    { 0xaf870934, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87093c, 95 },
    { 0xb17093d, 75 },
    { 0x4867093d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7670941, 23 },
    { 0x6c070942, 90 },
    { 0xaeb70943, 61 },
    { 0x63b70943, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e70948, 90 },
    { 0x93e70948, 90 },
    { 0xb077094a, 95 },
    { 0x41b7094b, 90 },
    { 0x46570949, 93 },
    { 0xe97094c, 61 },
    { 0x2e57094c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b370952, 33 },
    { 0x3a770953, 84 },
    { 0x34870954, 61 },
    { 0xc2070952, 61 },
    { 0x33770955, 76 },
    { 0x53070957, 23 },
    { 0x7c70958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf70962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad70965, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x16570968, 33 },
    { 0xae370968, 61 },
    { 0x53e7096a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9077096f, 98 },
    { 0x81c70970, 33 },
    { 0x6a27096f, 78 },
    { 0, 0 },
    { 0x45e70973, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d170979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd577097d, 90 },
    { 0, 0 },
    { 0x3f37097f, 66 },
    { 0x1407097f, 90 },
    { 0xd0670980, 55 },
    { 0x50f70982, 90 },
    { 0xebd70983, 55 },
    { 0, 0 },
    { 0x42170985, 75 },
    { 0xccc70986, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7098e, 76 },
    { 0xa137098e, 33 },
    { 0xf5070990, 59 },
    { 0, 0 },
    { 0x65f70992, 34 },
    { 0x54770993, 100 },
    { 0x8cd70993, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14370998, 90 },
    { 0x38670999, 34 },
    { 0, 0 },
    { 0x6e67099b, 61 },
    { 0x1d47099b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7099f, 33 },
    { 0, 0 },
    { 0xcaf709a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61709a7, 76 },
    { 0xbfc709a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16709ac, 23 },
    { 0, 0 },
    { 0xb17709ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4709b1, 33 },
    { 0x81b709b1, 95 },
    { 0x4b7709b3, 90 },
    { 0x9b9709b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f709b7, 55 },
    { 0xcc4709b8, 96 },
    { 0x693709b9, 85 },
    { 0x458709b9, 87 },
    { 0xf32709ba, 69 },
    { 0xa2d709bb, 33 },
    { 0x586709bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9709c8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3709cb, 43 },
    { 0, 0 },
    { 0x8c7709cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad709d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312709d5, 98 },
    { 0x984709d6, 98 },
    { 0x484709d7, 88 },
    { 0x269709d7, 98 },
    { 0xea8709d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445709e1, 33 },
    { 0, 0 },
    { 0x2ba709e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70709e7, 100 },
    { 0x846709e8, 90 },
    { 0, 0 },
    { 0xf8c709ea, 33 },
    { 0, 0 },
    { 0xae0709ec, 77 },
    { 0x4de709ed, 98 },
    { 0, 0 },
    { 0x15709ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc709f2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d709f5, 23 },
    { 0, 0 },
    { 0xef2709f7, 94 },
    { 0xbb6709f8, 90 },
    { 0xd40709f9, 67 },
    { 0x787709fa, 61 },
    { 0xc7d709f7, 55 },
    { 0xa0c709fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed70a02, 61 },
    { 0x51370a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db70a06, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d70a09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d70a10, 84 },
    { 0, 0 },
    { 0xe0a70a12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9370a15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b70a1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a70a1f, 95 },
    { 0x6ab70a1f, 91 },
    { 0, 0 },
    { 0x53770a22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf70a25, 90 },
    { 0xc9a70a26, 45 },
    { 0x9aa70a26, 55 },
    { 0x4df70a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b70a2d, 90 },
    { 0x66070a2e, 23 },
    { 0x9d070a2e, 52 },
    { 0x73b70a30, 81 },
    { 0xeb870a30, 61 },
    { 0x98c70a32, 46 },
    { 0, 0 },
    { 0x68670a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e70a38, 90 },
    { 0, 0 },
    { 0x3070a3a, 95 },
    { 0, 0 },
    { 0x9f670a3c, 61 },
    { 0xfee70a3d, 100 },
    { 0x3a170a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec370a49, 64 },
    { 0x4af70a49, 61 },
    { 0x6d470a4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24670a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc70a51, 6 },
    { 0, 0 },
    { 0x2de70a53, 23 },
    { 0xaf870a53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44370a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d70a5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a70a64, 55 },
    { 0, 0 },
    { 0xea070a66, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c70a69, 33 },
    { 0x4e870a6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e070a71, 52 },
    { 0x6d470a72, 33 },
    { 0, 0 },
    { 0x76270a74, 77 },
    { 0, 0 },
    { 0xbc970a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2870a7a, 17 },
    { 0x8dd70a7b, 90 },
    { 0xeab70a7c, 81 },
    { 0x1b170a7a, 80 },
    { 0xa9970a7c, 36 },
    { 0xcff70a7f, 90 },
    { 0x88c70a7a, 43 },
    { 0, 0 },
    { 0xa8d70a82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce170a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62570a88, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d370a8e, 95 },
    { 0, 0 },
    { 0x20270a90, 90 },
    { 0x65170a91, 61 },
    { 0xb4870a92, 76 },
    { 0x2fb70a92, 88 },
    { 0x67f70a92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85670a9c, 75 },
    { 0, 0 },
    { 0xf670a9e, 98 },
    { 0x76270a9f, 34 },
    { 0xd1970a9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55670aa5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df70aa9, 33 },
    { 0, 0 },
    { 0x6e270aab, 98 },
    { 0x7a570aac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf70ab0, 33 },
    { 0x4b670ab1, 93 },
    { 0x8bd70ab2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c70ab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e70abe, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b70ac3, 89 },
    { 0, 0 },
    { 0xa9070ac5, 90 },
    { 0x85a70ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d70ad3, 55 },
    { 0, 0 },
    { 0xfcb70ad5, 33 },
    { 0xba270ad6, 90 },
    { 0x52c70ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa070adc, 43 },
    { 0, 0 },
    { 0x6c370ade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69170ae4, 100 },
    { 0x53f70ae5, 90 },
    { 0x9a670ae6, 90 },
    { 0xbd270ae7, 87 },
    { 0x5ac70ae8, 33 },
    { 0x27d70ae9, 95 },
    { 0x90770ae6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c70aed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74670af1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d70af8, 76 },
    { 0xc2270af9, 98 },
    { 0, 0 },
    { 0x14670afb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7170afe, 61 },
    { 0x44d70aff, 76 },
    { 0, 0 },
    { 0xd570b01, 87 },
    { 0x15670b02, 90 },
    { 0xdd70b01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c70b06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe70b0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb470b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d570b15, 75 },
    { 0xf5470b16, 23 },
    { 0x3c270b15, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6570b1c, 46 },
    { 0x22b70b1d, 33 },
    { 0x24c70b1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b70b22, 52 },
    { 0, 0 },
    { 0x91170b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9170b28, 98 },
    { 0x35870b29, 84 },
    { 0xf2b70b2a, 52 },
    { 0, 0 },
    { 0x4bb70b2c, 75 },
    { 0x32270b2d, 68 },
    { 0x98170b2d, 33 },
    { 0x44470b2f, 90 },
    { 0x7e770b2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef70b38, 95 },
    { 0x3ac70b38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee70b3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf70b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1170b46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33370b4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4070b52, 34 },
    { 0xb470b53, 95 },
    { 0xad170b52, 34 },
    { 0, 0 },
    { 0x90370b56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70970b5c, 91 },
    { 0x56b70b5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d70b64, 43 },
    { 0x6a270b65, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91170b69, 93 },
    { 0xab670b6a, 98 },
    { 0, 0 },
    { 0x9870b6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970b70, 43 },
    { 0x17d70b71, 33 },
    { 0xcbb70b71, 77 },
    { 0x14d70b73, 52 },
    { 0x10470b71, 93 },
    { 0, 0 },
    { 0x78f70b76, 67 },
    { 0xea970b77, 95 },
    { 0x9e070b77, 61 },
    { 0x7170b79, 78 },
    { 0, 0 },
    { 0x28c70b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc70b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0470b83, 60 },
    { 0, 0 },
    { 0x99c70b85, 87 },
    { 0x8170b86, 91 },
    { 0x61370b85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40170b8e, 61 },
    { 0x80b70b8e, 90 },
    { 0x1e270b90, 17 },
    { 0x2b70b8e, 61 },
    { 0x56670b92, 99 },
    { 0, 0 },
    { 0xcfd70b94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73070b9b, 66 },
    { 0xee270b9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d70ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d70ba9, 93 },
    { 0, 0 },
    { 0xae670bab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f70bae, 33 },
    { 0xb2970bae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470bb2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7470bb6, 90 },
    { 0, 0 },
    { 0xa0070bb8, 61 },
    { 0x91770bb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71470bbf, 33 },
    { 0x38170bc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee70bc6, 61 },
    { 0x91c70bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2870bca, 84 },
    { 0, 0 },
    { 0xbdd70bcc, 61 },
    { 0x29c70bcd, 90 },
    { 0x6b570bce, 87 },
    { 0xbd570bcd, 87 },
    { 0x83770bd0, 90 },
    { 0xb6070bd1, 95 },
    { 0, 0 },
    { 0xfc70bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9370bd7, 90 },
    { 0, 0 },
    { 0x9d570bd9, 98 },
    { 0x7bb70bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15970be0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x27470be3, 76 },
    { 0, 0 },
    { 0x7ab70be5, 98 },
    { 0xc1670be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e70be9, 61 },
    { 0, 0 },
    { 0xf5970beb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe770bef, 55 },
    { 0x17070bf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c70bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0770bf6, 33 },
    { 0xcdd70bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f70bfd, 33 },
    { 0xab570bfe, 61 },
    { 0x5b570bff, 33 },
    { 0x5070c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2970c05, 23 },
    { 0x18470c06, 90 },
    { 0, 0 },
    { 0x3f170c08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75070c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e70c17, 33 },
    { 0xc3470c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30770c1d, 81 },
    { 0, 0 },
    { 0xe3070c1f, 95 },
    { 0x1d170c20, 43 },
    { 0, 0 },
    { 0x2b570c22, 52 },
    { 0x3a070c23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47370c2d, 76 },
    { 0xaa170c2e, 95 },
    { 0, 0 },
    { 0xee670c30, 88 },
    { 0xe5f70c31, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x52170c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63170c38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60870c47, 98 },
    { 0x7e570c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1970c4b, 90 },
    { 0x70c4c, 95 },
    { 0xd6070c4c, 76 },
    { 0x3e370c4c, 100 },
    { 0x41b70c4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28970c55, 93 },
    { 0xcba70c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45470c5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66370c60, 55 },
    { 0x7fa70c61, 23 },
    { 0xe3270c61, 90 },
    { 0x30870c63, 43 },
    { 0x50970c64, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5670c68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70c6b, 55 },
    { 0x7bc70c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c70c71, 95 },
    { 0x71070c72, 90 },
    { 0x8370c73, 23 },
    { 0xb570c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be70c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b670c7d, 95 },
    { 0x46470c7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e70c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed370c88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42870c8d, 99 },
    { 0x5e770c8e, 99 },
    { 0, 0 },
    { 0x65670c90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e770c94, 85 },
    { 0xa4670c95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5370c99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0070c9d, 33 },
    { 0, 0 },
    { 0xda870c9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f70ca2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b70ca5, 23 },
    { 0x93170ca6, 98 },
    { 0xb3170ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c70cb4, 66 },
    { 0x7f170cb4, 23 },
    { 0x3f970cb6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97070cba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e70cbd, 93 },
    { 0x23a70cbe, 17 },
    { 0, 0 },
    { 0xcd470cc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8770cc4, 95 },
    { 0x59870cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10570cc9, 67 },
    { 0x5ac70cc9, 33 },
    { 0x35a70cca, 98 },
    { 0, 0 },
    { 0x13c70ccd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14070cd0, 23 },
    { 0x4b970cd1, 90 },
    { 0x45c70cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e70cdb, 43 },
    { 0xd4470cdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a70ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf70cee, 78 },
    { 0, 0 },
    { 0x24570cf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a70cfa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f70d02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5770d06, 87 },
    { 0x21870d07, 100 },
    { 0x7a670d06, 31 },
    { 0, 0 },
    { 0x5ee70d0a, 46 },
    { 0x57770d0b, 46 },
    { 0x36270d0a, 69 },
    { 0x54770d0d, 61 },
    { 0xa6570d0e, 98 },
    { 0x8e970d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6970d15, 52 },
    { 0xb8670d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7470d19, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e70d24, 95 },
    { 0xc6d70d25, 33 },
    { 0xa0270d26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0370d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce70d2f, 95 },
    { 0x19370d30, 33 },
    { 0x39d70d30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17370d37, 98 },
    { 0x8fd70d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3570d3d, 46 },
    { 0xe870d3e, 55 },
    { 0x26b70d3f, 76 },
    { 0x3da70d40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e570d44, 36 },
    { 0x4d170d44, 33 },
    { 0x8d70d44, 23 },
    { 0xf3670d47, 52 },
    { 0, 0 },
    { 0x2bd70d49, 33 },
    { 0xfc870d4a, 33 },
    { 0xbea70d4b, 85 },
    { 0xc7170d49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88870d4f, 23 },
    { 0x89870d50, 76 },
    { 0x4f770d50, 23 },
    { 0, 0 },
    { 0x96a70d53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3570d58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37670d5b, 68 },
    { 0x11870d5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad570d63, 33 },
    { 0, 0 },
    { 0xace70d65, 34 },
    { 0xa8370d66, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9770d6c, 55 },
    { 0x4a870d6d, 95 },
    { 0xd4770d6d, 40 },
    { 0, 0 },
    { 0xfd570d70, 38 },
    { 0x76570d70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec70d77, 61 },
    { 0, 0 },
    { 0x23a70d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda70d7c, 61 },
    { 0, 0 },
    { 0x9b70d7e, 61 },
    { 0, 0 },
    { 0x5dc70d80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16470d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d70d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c70d8d, 90 },
    { 0, 0 },
    { 0xb5770d8f, 85 },
    { 0xf8570d90, 76 },
    { 0xfd170d90, 98 },
    { 0xd1470d92, 85 },
    { 0xcce70d93, 46 },
    { 0x83170d94, 95 },
    { 0, 0 },
    { 0xf4870d96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea70da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2070da5, 90 },
    { 0x71470da5, 93 },
    { 0, 0 },
    { 0xc5c70da8, 52 },
    { 0xc0570da8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76770db3, 98 },
    { 0x4be70db4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e970db8, 33 },
    { 0, 0 },
    { 0xcb770dba, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4370dbf, 98 },
    { 0, 0 },
    { 0x44370dc1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6370dc4, 90 },
    { 0x34e70dc5, 90 },
    { 0xa1770dc6, 95 },
    { 0xfce70dc7, 65 },
    { 0xaa170dc4, 93 },
    { 0x8d070dc9, 90 },
    { 0xc9370dca, 61 },
    { 0x3f270dca, 61 },
    { 0x21270dc9, 33 },
    { 0xf6f70dcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b670dd2, 23 },
    { 0x30670dd2, 76 },
    { 0xc8170dd3, 95 },
    { 0x5fd70dd5, 91 },
    { 0x19070dd4, 55 },
    { 0x89a70dd7, 84 },
    { 0xec670dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e70ddb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24370de5, 95 },
    { 0, 0 },
    { 0x12c70de7, 90 },
    { 0x45170de7, 34 },
    { 0x7fa70de9, 95 },
    { 0x3fb70de9, 33 },
    { 0, 0 },
    { 0xd1670dec, 68 },
    { 0x29870ded, 33 },
    { 0xf3270dec, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51170df5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f670df8, 83 },
    { 0x34270df9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1670dff, 33 },
    { 0x7a370dff, 55 },
    { 0, 0 },
    { 0x1c170e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f970e06, 17 },
    { 0x9670e06, 33 },
    { 0xb8770e08, 61 },
    { 0, 0 },
    { 0xbc870e0a, 90 },
    { 0, 0 },
    { 0xac070e0c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb70e13, 34 },
    { 0xeca70e14, 61 },
    { 0, 0 },
    { 0xbd570e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee470e19, 84 },
    { 0x36370e1a, 65 },
    { 0x83770e1a, 43 },
    { 0xd4a70e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f870e25, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3470e2a, 98 },
    { 0xf6770e2a, 95 },
    { 0x9ba70e2c, 90 },
    { 0xe4670e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9570e35, 61 },
    { 0x9c170e36, 90 },
    { 0, 0 },
    { 0xd0e70e38, 61 },
    { 0x8c270e39, 98 },
    { 0xac170e39, 95 },
    { 0x45f70e3b, 61 },
    { 0x26c70e3c, 23 },
    { 0x62370e3d, 90 },
    { 0x13970e3d, 43 },
    { 0x64170e3e, 33 },
    { 0x50970e40, 90 },
    { 0xfcf70e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52070e47, 24 },
    { 0xeb170e48, 95 },
    { 0, 0 },
    { 0x1d370e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c70e54, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x21970e57, 95 },
    { 0, 0 },
    { 0xa9670e59, 95 },
    { 0x5c470e59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2070e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf570e62, 61 },
    { 0x87070e63, 78 },
    { 0, 0 },
    { 0xb1b70e65, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a70e6a, 98 },
    { 0, 0 },
    { 0xeff70e6c, 99 },
    { 0xc8e70e6d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a770e76, 90 },
    { 0, 0 },
    { 0x8d370e78, 91 },
    { 0xb70e79, 23 },
    { 0xaf570e7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6970e7e, 23 },
    { 0xcfe70e7f, 23 },
    { 0, 0 },
    { 0x8ba70e81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11070e88, 87 },
    { 0, 0 },
    { 0x6d070e8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf370e95, 67 },
    { 0xdd970e96, 34 },
    { 0x8f770e96, 33 },
    { 0, 0 },
    { 0x2e470e99, 61 },
    { 0xf1570e9a, 98 },
    { 0, 0 },
    { 0x45070e9c, 46 },
    { 0x40a70e9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19070ea1, 61 },
    { 0xdd570ea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd670eaa, 90 },
    { 0x9ff70eab, 88 },
    { 0xf2d70eaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16670eb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48770ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c770ebe, 52 },
    { 0, 0 },
    { 0xe3e70ec0, 33 },
    { 0x53e70ec1, 17 },
    { 0x26f70ec2, 55 },
    { 0x7b170ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe170ec6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1670eca, 95 },
    { 0xe3e70ecb, 55 },
    { 0xaf70ecc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23270ed1, 90 },
    { 0x14970ed2, 88 },
    { 0xc3470ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf470ed6, 76 },
    { 0, 0 },
    { 0xd1e70ed8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd70edc, 98 },
    { 0, 0 },
    { 0x99e70ede, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c270ee3, 33 },
    { 0x47c70ee4, 33 },
    { 0xc7970ee5, 98 },
    { 0xa7a70ee6, 67 },
    { 0x63470ee3, 76 },
    { 0x1e970ee8, 34 },
    { 0xdd770ee6, 98 },
    { 0, 0 },
    { 0x6a370eeb, 80 },
    { 0, 0 },
    { 0x69d70eed, 87 },
    { 0x1a70eee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7670ef1, 46 },
    { 0xb2470ef2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c70ef6, 23 },
    { 0x75c70ef6, 78 },
    { 0x47070ef8, 76 },
    { 0, 0 },
    { 0xbb670efa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe70efd, 33 },
    { 0xa1e70efe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f70f05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7570f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1470f0f, 23 },
    { 0, 0 },
    { 0xaac70f11, 34 },
    { 0, 0 },
    { 0xdc870f13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f670f18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c470f1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1270f1f, 61 },
    { 0x95170f1f, 90 },
    { 0xa1270f21, 100 },
    { 0, 0 },
    { 0xd3770f23, 61 },
    { 0xb9770f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e70f2a, 34 },
    { 0xdf070f2b, 98 },
    { 0x41470f2c, 52 },
    { 0, 0 },
    { 0xa9e70f2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d870f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a70f35, 90 },
    { 0xb4370f36, 66 },
    { 0x14a70f37, 68 },
    { 0x99070f38, 61 },
    { 0, 0 },
    { 0xb570f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52570f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c70f47, 90 },
    { 0, 0 },
    { 0x12870f49, 98 },
    { 0, 0 },
    { 0xf4e70f4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf970f53, 61 },
    { 0xd8f70f54, 77 },
    { 0x96c70f55, 33 },
    { 0xc3770f54, 88 },
    { 0xe5270f53, 99 },
    { 0xbb670f58, 33 },
    { 0x98c70f58, 90 },
    { 0, 0 },
    { 0xea870f5b, 85 },
    { 0, 0 },
    { 0x4ca70f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15570f63, 98 },
    { 0xed170f64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0770f69, 47 },
    { 0x45c70f6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff70f70, 90 },
    { 0, 0 },
    { 0xa5870f72, 99 },
    { 0x9a470f72, 76 },
    { 0x47670f74, 67 },
    { 0x2cc70f75, 90 },
    { 0x47c70f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45370f7f, 66 },
    { 0xaa370f80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f070f86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6470f8a, 90 },
    { 0xb3570f8a, 55 },
    { 0xf0670f8c, 61 },
    { 0x9a470f8d, 34 },
    { 0x68270f8e, 55 },
    { 0xeb370f8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98770f92, 100 },
    { 0, 0 },
    { 0x72970f94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2170f97, 34 },
    { 0xd6070f97, 55 },
    { 0x3f670f99, 98 },
    { 0, 0 },
    { 0xeaa70f9b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x55070f9e, 43 },
    { 0x3cf70f9f, 61 },
    { 0xbba70fa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20270fa9, 85 },
    { 0x45870fa9, 90 },
    { 0, 0 },
    { 0xc2a70fac, 67 },
    { 0, 0 },
    { 0x34170fae, 61 },
    { 0x15770fae, 55 },
    { 0, 0 },
    { 0xdb70fb1, 17 },
    { 0x47570fb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd170fb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b70fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b70fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77870fc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf870fce, 90 },
    { 0xd8a70fce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0670fd2, 23 },
    { 0xd0370fd3, 68 },
    { 0x41a70fd4, 88 },
    { 0xc7a70fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4570fd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b70fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb870fe1, 88 },
    { 0, 0 },
    { 0x94b70fe3, 90 },
    { 0x50470fe4, 95 },
    { 0x84a70fe5, 87 },
    { 0, 0 },
    { 0x5a70fe7, 46 },
    { 0xd7370fe8, 85 },
    { 0xda270fe8, 84 },
    { 0xab170fe9, 33 },
    { 0x6e470fe9, 55 },
    { 0x1d770fec, 33 },
    { 0x85570fec, 33 },
    { 0, 0 },
    { 0x76070fef, 46 },
    { 0, 0 },
    { 0xda070ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f970ffc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a71000, 90 },
    { 0, 0 },
    { 0x74e71002, 66 },
    { 0, 0 },
    { 0x4ad71004, 43 },
    { 0xcdc71005, 95 },
    { 0x65e71006, 23 },
    { 0x16071007, 66 },
    { 0, 0 },
    { 0xa9171009, 90 },
    { 0x26d7100a, 33 },
    { 0x76371009, 40 },
    { 0, 0 },
    { 0xda47100d, 95 },
    { 0x17f7100e, 93 },
    { 0xc927100f, 93 },
    { 0x6577100e, 55 },
    { 0xd5271010, 33 },
    { 0xf3a71012, 66 },
    { 0xdf071013, 61 },
    { 0x84771014, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a871017, 76 },
    { 0x51371018, 52 },
    { 0xd0671018, 76 },
    { 0, 0 },
    { 0x3a07101b, 61 },
    { 0xd867101c, 55 },
    { 0, 0 },
    { 0x4097101e, 33 },
    { 0, 0 },
    { 0xc1771020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71023, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7102a, 93 },
    { 0xc8c7102a, 95 },
    { 0, 0 },
    { 0x2ae7102d, 78 },
    { 0xcc27102d, 67 },
    { 0x9a7102d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd071034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed071038, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a37103b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3417103e, 69 },
    { 0x857103f, 33 },
    { 0xd3e7103f, 100 },
    { 0, 0 },
    { 0x73471042, 95 },
    { 0xba71043, 23 },
    { 0x24771042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de71047, 23 },
    { 0xdcf71048, 90 },
    { 0x2c71048, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3471051, 61 },
    { 0, 0 },
    { 0x10f71053, 47 },
    { 0x61271054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37571058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7105c, 90 },
    { 0xa0a7105d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33671062, 33 },
    { 0xfa171062, 23 },
    { 0, 0 },
    { 0x9c771065, 90 },
    { 0x6a371065, 33 },
    { 0x5c571067, 55 },
    { 0x5b671066, 55 },
    { 0xc1271068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf7106d, 98 },
    { 0x69d7106e, 34 },
    { 0xdf17106e, 90 },
    { 0x3ba71070, 34 },
    { 0xe1271070, 23 },
    { 0, 0 },
    { 0xb5d71073, 23 },
    { 0xf9a71073, 87 },
    { 0, 0 },
    { 0x65171076, 100 },
    { 0x9f571076, 95 },
    { 0x94b71078, 18 },
    { 0x2c871076, 61 },
    { 0x6227107a, 67 },
    { 0x287107b, 100 },
    { 0xa757107b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8557107f, 17 },
    { 0x37a71080, 84 },
    { 0x4cf71081, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5671086, 87 },
    { 0xc4671086, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb857108b, 33 },
    { 0x2427108c, 93 },
    { 0, 0 },
    { 0x3a57108e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7671092, 68 },
    { 0x5c071092, 23 },
    { 0x83371094, 33 },
    { 0x56471095, 98 },
    { 0, 0 },
    { 0xab71097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf37109a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217109e, 43 },
    { 0x31a7109e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63710a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2710aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1710ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb710b2, 88 },
    { 0, 0 },
    { 0x9aa710b4, 90 },
    { 0x5fa710b5, 76 },
    { 0x97e710b5, 100 },
    { 0xf82710b5, 90 },
    { 0, 0 },
    { 0x2c1710b9, 61 },
    { 0x90e710ba, 23 },
    { 0xbf6710bb, 55 },
    { 0x3c3710bb, 66 },
    { 0xaff710ba, 100 },
    { 0x309710bc, 90 },
    { 0x78e710bf, 98 },
    { 0xade710bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673710c5, 55 },
    { 0x856710c6, 33 },
    { 0, 0 },
    { 0x101710c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a710cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98710d0, 76 },
    { 0, 0 },
    { 0xe09710d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2710d6, 23 },
    { 0, 0 },
    { 0x690710d8, 95 },
    { 0, 0 },
    { 0xa9710da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9710dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6710e4, 90 },
    { 0x37c710e5, 99 },
    { 0x725710e6, 33 },
    { 0xb10710e7, 36 },
    { 0x48c710e8, 34 },
    { 0x281710e9, 23 },
    { 0, 0 },
    { 0x2d4710eb, 91 },
    { 0xe1710ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f710f3, 90 },
    { 0x73e710f4, 99 },
    { 0x883710f4, 61 },
    { 0x2be710f6, 43 },
    { 0xf2b710f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970710fb, 33 },
    { 0, 0 },
    { 0xe9a710fd, 98 },
    { 0x8ba710fd, 33 },
    { 0x7ac710ff, 87 },
    { 0x7f671100, 85 },
    { 0x33071100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70371104, 33 },
    { 0, 0 },
    { 0x80d71106, 97 },
    { 0x43e71107, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff7110a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1171110, 23 },
    { 0, 0 },
    { 0xbd171112, 90 },
    { 0x40871112, 53 },
    { 0xc8b71114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed671117, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17111a, 61 },
    { 0, 0 },
    { 0xd427111c, 90 },
    { 0x3e67111d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a671120, 61 },
    { 0, 0 },
    { 0x8e871122, 90 },
    { 0x9e371122, 61 },
    { 0x75e71123, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc71127, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c7112b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba271133, 61 },
    { 0xfd371134, 98 },
    { 0xb8071135, 61 },
    { 0, 0 },
    { 0xb8371137, 52 },
    { 0, 0 },
    { 0x84571139, 88 },
    { 0x69871139, 61 },
    { 0xa777113a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf237113e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed171145, 90 },
    { 0x44571146, 43 },
    { 0x45b71147, 33 },
    { 0x31471148, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e67114c, 23 },
    { 0xde47114c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd971156, 95 },
    { 0xeeb71157, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07115a, 90 },
    { 0xf947115b, 33 },
    { 0xdc97115c, 33 },
    { 0, 0 },
    { 0xdcb7115e, 97 },
    { 0x55a7115f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f771167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc867116c, 33 },
    { 0x7bb7116d, 95 },
    { 0, 0 },
    { 0x4fe7116f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89571173, 95 },
    { 0x3b271174, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d71177, 61 },
    { 0x90071178, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b7117f, 46 },
    { 0, 0 },
    { 0xacf71181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971188, 98 },
    { 0xb9071189, 54 },
    { 0x9e87118a, 95 },
    { 0xbb77118b, 33 },
    { 0x75d71188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb071190, 17 },
    { 0xc4071191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7119e, 23 },
    { 0xa8f7119f, 95 },
    { 0, 0 },
    { 0x421711a1, 61 },
    { 0xef2711a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x131711a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799711ae, 90 },
    { 0x777711af, 77 },
    { 0x7b5711b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee711b3, 93 },
    { 0xfc0711b4, 81 },
    { 0, 0 },
    { 0xdba711b6, 90 },
    { 0xd6b711b7, 95 },
    { 0xb1a711b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b711bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1711c3, 76 },
    { 0x362711c4, 95 },
    { 0, 0 },
    { 0x4c9711c6, 99 },
    { 0, 0 },
    { 0xa94711c8, 33 },
    { 0x724711c9, 33 },
    { 0xec1711ca, 61 },
    { 0x1ad711cb, 33 },
    { 0, 0 },
    { 0xd3e711cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32711d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b711d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190711dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29711e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb711e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c711e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862711eb, 78 },
    { 0xc34711ec, 33 },
    { 0x3d7711ec, 43 },
    { 0x5d8711eb, 24 },
    { 0xeda711ee, 23 },
    { 0xf08711f0, 55 },
    { 0xace711f1, 93 },
    { 0xd26711f1, 90 },
    { 0xf12711f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1711f8, 61 },
    { 0xa7c711f9, 91 },
    { 0x85b711fa, 95 },
    { 0, 0 },
    { 0x63711fc, 98 },
    { 0x697711fd, 98 },
    { 0xfa1711fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38471203, 78 },
    { 0xc0f71204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b71207, 23 },
    { 0xf1571208, 46 },
    { 0xa1d71209, 53 },
    { 0x70471209, 55 },
    { 0x4157120a, 87 },
    { 0xce07120c, 33 },
    { 0xd9d7120d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771213, 33 },
    { 0xdd471213, 98 },
    { 0xf8671215, 23 },
    { 0x35971214, 23 },
    { 0xa7071217, 98 },
    { 0xa1571218, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77121b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9471221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7571224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82771227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3747122d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f7123a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587123d, 95 },
    { 0x7657123d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd371241, 61 },
    { 0, 0 },
    { 0x8b071243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6171248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987124c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a571250, 76 },
    { 0xe3d71251, 95 },
    { 0x26771252, 52 },
    { 0x63671253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd71257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc517125a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c7125d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b71267, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe727126c, 98 },
    { 0xfd17126c, 61 },
    { 0x807126d, 98 },
    { 0, 0 },
    { 0xb1d71270, 76 },
    { 0, 0 },
    { 0x20071272, 33 },
    { 0xecb71273, 98 },
    { 0xfeb71274, 76 },
    { 0xba071275, 93 },
    { 0xed471274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9371279, 81 },
    { 0xc5f7127a, 91 },
    { 0x9247127b, 23 },
    { 0xc7c7127c, 55 },
    { 0x75b7127a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c571281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52771287, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc637128b, 90 },
    { 0x74a7128b, 90 },
    { 0, 0 },
    { 0xc777128e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9371291, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f7129a, 90 },
    { 0x8c97129b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f712a1, 81 },
    { 0, 0 },
    { 0xad1712a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae712a7, 95 },
    { 0x412712a8, 99 },
    { 0x39b712a8, 23 },
    { 0xdeb712aa, 76 },
    { 0xeec712ab, 76 },
    { 0xd27712a7, 95 },
    { 0xe3712ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321712b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d712b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14712bb, 76 },
    { 0x194712bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97712c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6712cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde712d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91712d8, 77 },
    { 0x552712d8, 23 },
    { 0xfa7712da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234712de, 85 },
    { 0x21712df, 33 },
    { 0x89f712e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28712e3, 23 },
    { 0, 0 },
    { 0xfd5712e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee712eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751712f0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38712f3, 76 },
    { 0xa76712f4, 95 },
    { 0xcc2712f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2712f8, 33 },
    { 0x4f6712f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4712ff, 95 },
    { 0x82071300, 76 },
    { 0, 0 },
    { 0x7ee71302, 93 },
    { 0x85071303, 34 },
    { 0, 0 },
    { 0x3b371305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf71309, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c671310, 93 },
    { 0xc1e71311, 33 },
    { 0xb1671312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7871318, 95 },
    { 0xa6f71318, 33 },
    { 0, 0 },
    { 0x7337131b, 98 },
    { 0x6a77131c, 46 },
    { 0, 0 },
    { 0x1ff7131e, 61 },
    { 0x2597131f, 90 },
    { 0, 0 },
    { 0x1fc71321, 55 },
    { 0x72971321, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4871327, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c67132a, 100 },
    { 0xbf67132a, 100 },
    { 0xce57132c, 40 },
    { 0x3327132c, 43 },
    { 0xb87132d, 43 },
    { 0xc1a7132d, 76 },
    { 0xc067132d, 93 },
    { 0xf727132f, 76 },
    { 0x81071332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3617133a, 65 },
    { 0xca47133b, 95 },
    { 0x8c07133c, 67 },
    { 0x54f7133b, 34 },
    { 0x3267133e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc971347, 23 },
    { 0x19f71348, 33 },
    { 0, 0 },
    { 0x9a47134a, 65 },
    { 0x9687134a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8371350, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe71357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357135a, 77 },
    { 0x47c7135b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee71361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32071369, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7136e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d71375, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7457137c, 98 },
    { 0x7457137d, 61 },
    { 0x3947137e, 61 },
    { 0xcb77137f, 46 },
    { 0xeaf7137c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c971384, 64 },
    { 0x29e71385, 96 },
    { 0x11071386, 33 },
    { 0x7fc71386, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5627138c, 90 },
    { 0xdab7138c, 78 },
    { 0x6c7138e, 87 },
    { 0x5147138e, 52 },
    { 0, 0 },
    { 0x1b271391, 98 },
    { 0, 0 },
    { 0xf4271393, 33 },
    { 0xdb471394, 95 },
    { 0, 0 },
    { 0x29c71396, 25 },
    { 0, 0 },
    { 0x16871398, 61 },
    { 0x3d571398, 55 },
    { 0, 0 },
    { 0x83f7139b, 67 },
    { 0xc217139c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96713a2, 91 },
    { 0x344713a3, 96 },
    { 0, 0 },
    { 0xc49713a5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b713aa, 33 },
    { 0xca713ab, 93 },
    { 0x693713ac, 90 },
    { 0x873713aa, 23 },
    { 0x807713ae, 34 },
    { 0xad713af, 33 },
    { 0x82e713ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6713b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x537713ba, 43 },
    { 0xe4a713ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x181713be, 100 },
    { 0, 0 },
    { 0xc73713c0, 66 },
    { 0xfb0713c1, 90 },
    { 0, 0 },
    { 0x74c713c3, 61 },
    { 0x385713c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df713ca, 98 },
    { 0x7b7713cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793713cf, 23 },
    { 0, 0 },
    { 0x382713d1, 90 },
    { 0x432713d2, 61 },
    { 0xb95713d3, 76 },
    { 0xaf6713d3, 93 },
    { 0x487713d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3713d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b713db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a713e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b713e9, 33 },
    { 0xbd4713e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3713ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d713f1, 61 },
    { 0xcec713f1, 76 },
    { 0xf46713f3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4713f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc713fa, 46 },
    { 0, 0 },
    { 0xead713fc, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4713ff, 23 },
    { 0x2cf71400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46071403, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc71406, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6071409, 43 },
    { 0x7017140a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74371410, 43 },
    { 0x8d471411, 34 },
    { 0x83f71412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80871416, 33 },
    { 0, 0 },
    { 0x11971418, 23 },
    { 0xd2d71418, 52 },
    { 0, 0 },
    { 0xe647141b, 55 },
    { 0xb517141b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d7141f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe171424, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85871428, 66 },
    { 0xac771429, 78 },
    { 0x27d7142a, 33 },
    { 0x5437142b, 43 },
    { 0, 0 },
    { 0xce87142d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce71430, 33 },
    { 0x6af71431, 53 },
    { 0x4b971432, 34 },
    { 0, 0 },
    { 0xc8371434, 33 },
    { 0xb0d71435, 55 },
    { 0, 0 },
    { 0xbde71437, 33 },
    { 0, 0 },
    { 0xb4a71439, 66 },
    { 0x3d37143a, 17 },
    { 0x83a7143b, 55 },
    { 0xd947143c, 90 },
    { 0xbbe7143d, 98 },
    { 0xe2c7143e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1667144d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd71450, 80 },
    { 0x8d571451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb137145a, 61 },
    { 0xe137145a, 98 },
    { 0x10b7145c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c17145f, 33 },
    { 0x4a571460, 61 },
    { 0xbde7145f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x93271464, 98 },
    { 0x99771465, 21 },
    { 0x6371466, 84 },
    { 0xf0371467, 100 },
    { 0xbc771468, 76 },
    { 0xd3771468, 77 },
    { 0x1c71467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b371472, 90 },
    { 0xbb471472, 66 },
    { 0xebc71473, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c471477, 95 },
    { 0x84f71478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e07147b, 90 },
    { 0, 0 },
    { 0xd727147d, 33 },
    { 0x20a7147e, 90 },
    { 0xe7b7147e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18071484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92371489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1147148c, 34 },
    { 0x92c7148c, 95 },
    { 0xcc47148c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4571492, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc71496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7149b, 88 },
    { 0xd237149b, 23 },
    { 0xe747149d, 61 },
    { 0x73c7149e, 23 },
    { 0xd5e7149f, 90 },
    { 0, 0 },
    { 0xea3714a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f714a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a714ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1714b4, 76 },
    { 0xa64714b5, 76 },
    { 0x9d4714b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f714bd, 93 },
    { 0, 0 },
    { 0xbd3714bf, 95 },
    { 0, 0 },
    { 0x527714c1, 95 },
    { 0, 0 },
    { 0xcba714c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e714c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d714cb, 46 },
    { 0xbf2714cc, 23 },
    { 0x895714cd, 55 },
    { 0xa1a714ce, 99 },
    { 0xff1714cf, 98 },
    { 0, 0 },
    { 0xdbc714d1, 99 },
    { 0, 0 },
    { 0xe6b714d3, 97 },
    { 0x6c6714d4, 23 },
    { 0xb1c714d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986714d9, 52 },
    { 0, 0 },
    { 0xd19714db, 33 },
    { 0x38d714dc, 52 },
    { 0, 0 },
    { 0x124714de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155714e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854714e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b714ea, 17 },
    { 0x2ef714eb, 77 },
    { 0x96f714eb, 90 },
    { 0xc85714ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201714f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f714f4, 90 },
    { 0x832714f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a714f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa714fc, 23 },
    { 0xbb5714fd, 61 },
    { 0, 0 },
    { 0xcbe714ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b771506, 76 },
    { 0x2bb71507, 64 },
    { 0xaa371508, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5977150c, 90 },
    { 0xa787150d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xece71510, 61 },
    { 0x4f71511, 90 },
    { 0, 0 },
    { 0xf7a71513, 98 },
    { 0xdbb71513, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4777151d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a71520, 99 },
    { 0xdf671520, 90 },
    { 0x1c071520, 23 },
    { 0x57271521, 100 },
    { 0x81371520, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c671529, 23 },
    { 0x5d871529, 100 },
    { 0x28a7152b, 100 },
    { 0x7f471529, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a71530, 23 },
    { 0xa6871531, 76 },
    { 0x29e71532, 75 },
    { 0x85071532, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x8971536, 84 },
    { 0xc771537, 95 },
    { 0, 0 },
    { 0x40c71539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31071540, 98 },
    { 0xdb871540, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b571547, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e7154b, 90 },
    { 0x597154c, 33 },
    { 0x2da7154d, 55 },
    { 0, 0 },
    { 0xdbc7154f, 90 },
    { 0, 0 },
    { 0xd171551, 90 },
    { 0xf2d71551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b971556, 94 },
    { 0, 0 },
    { 0xf7a71558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7277155b, 59 },
    { 0x61c7155b, 55 },
    { 0, 0 },
    { 0x47f7155e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e171562, 95 },
    { 0xb2771562, 98 },
    { 0, 0 },
    { 0xad671565, 61 },
    { 0xd4771565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe907156a, 85 },
    { 0xe057156b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf97156f, 95 },
    { 0x4871570, 76 },
    { 0x23c7156f, 17 },
    { 0, 0 },
    { 0x49a71573, 68 },
    { 0x3b271574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a17157a, 90 },
    { 0x8d17157b, 43 },
    { 0x1c37157c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86371583, 33 },
    { 0x72171584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb37158b, 84 },
    { 0xa767158c, 43 },
    { 0xed7158c, 87 },
    { 0xd327158e, 61 },
    { 0x1bd7158e, 66 },
    { 0xa71590, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb771593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e71599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8077159c, 61 },
    { 0x7c07159d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f715a0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d715a3, 95 },
    { 0, 0 },
    { 0x46e715a5, 68 },
    { 0x3e0715a6, 23 },
    { 0x75f715a6, 55 },
    { 0x931715a8, 98 },
    { 0xb1e715a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7715af, 98 },
    { 0, 0 },
    { 0xf7d715b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342715b5, 33 },
    { 0x486715b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845715bc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x64715bf, 17 },
    { 0x8bd715c0, 85 },
    { 0, 0 },
    { 0x5da715c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35715c6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449715ca, 98 },
    { 0x9a6715cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x718715ce, 84 },
    { 0x4f4715cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb715d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3715d7, 90 },
    { 0, 0 },
    { 0x180715d9, 33 },
    { 0x15d715da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8715e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246715e6, 69 },
    { 0x73a715e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x714715ea, 90 },
    { 0x8ee715eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82715f1, 88 },
    { 0, 0 },
    { 0xf15715f3, 99 },
    { 0xc7715f4, 33 },
    { 0x5ba715f5, 78 },
    { 0x726715f6, 34 },
    { 0xb79715f4, 93 },
    { 0x2f715f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f715fb, 98 },
    { 0x291715fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad715ff, 98 },
    { 0x51e71600, 61 },
    { 0xef571601, 95 },
    { 0x18071602, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71605, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d571608, 90 },
    { 0, 0 },
    { 0x11b7160a, 99 },
    { 0x13d7160a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c7160e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c071611, 33 },
    { 0x17071612, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d271615, 90 },
    { 0xc6271615, 46 },
    { 0x11f71617, 33 },
    { 0xf7671617, 95 },
    { 0x74e71619, 34 },
    { 0, 0 },
    { 0xd1f7161b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f7161e, 90 },
    { 0x6307161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce71623, 23 },
    { 0x36271624, 66 },
    { 0xc3971624, 98 },
    { 0x93671626, 95 },
    { 0xf6f71626, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7162a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8647162d, 90 },
    { 0, 0 },
    { 0xc697162f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b471639, 55 },
    { 0xde77163a, 81 },
    { 0xa857163b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4597163f, 77 },
    { 0xeaa71640, 61 },
    { 0x72471641, 23 },
    { 0x87d71640, 83 },
    { 0, 0 },
    { 0x4b771644, 75 },
    { 0, 0 },
    { 0x1a71646, 61 },
    { 0, 0 },
    { 0x12971648, 81 },
    { 0x8e071648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d47164c, 80 },
    { 0x8f67164d, 91 },
    { 0x2037164d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e71653, 76 },
    { 0xbda71654, 61 },
    { 0, 0 },
    { 0x25e71656, 33 },
    { 0xf2271657, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a47165a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d671665, 33 },
    { 0, 0 },
    { 0xe0171667, 88 },
    { 0, 0 },
    { 0x5a371669, 95 },
    { 0x517166a, 33 },
    { 0x16c7166a, 55 },
    { 0x71c7166c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d7167b, 95 },
    { 0x4047167c, 33 },
    { 0, 0 },
    { 0xd7f7167e, 95 },
    { 0x12d7167f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb071685, 38 },
    { 0xf5771686, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647168a, 76 },
    { 0x64c7168b, 95 },
    { 0x38f7168c, 90 },
    { 0x18b7168d, 91 },
    { 0xe6e7168b, 98 },
    { 0xe167168f, 78 },
    { 0xf4171690, 90 },
    { 0xf227168a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1171694, 33 },
    { 0x5f171695, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57169d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584716a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a716a5, 98 },
    { 0x9bf716a6, 55 },
    { 0, 0 },
    { 0x5d3716a8, 23 },
    { 0xd77716a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x463716ac, 98 },
    { 0xa01716ad, 61 },
    { 0x774716ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda716b6, 87 },
    { 0xc3a716b7, 33 },
    { 0x736716b7, 95 },
    { 0xfa2716b7, 100 },
    { 0, 0 },
    { 0xa40716bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x791716be, 75 },
    { 0xa52716bf, 90 },
    { 0xd74716c0, 52 },
    { 0, 0 },
    { 0x444716c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da716c6, 90 },
    { 0x44b716c7, 46 },
    { 0, 0 },
    { 0x924716c9, 78 },
    { 0x583716ca, 72 },
    { 0, 0 },
    { 0xeba716cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82716d2, 33 },
    { 0xb2f716d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b716d7, 65 },
    { 0x65b716d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78716dd, 90 },
    { 0, 0 },
    { 0x74b716df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2716e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50716ee, 76 },
    { 0xe09716ee, 87 },
    { 0, 0 },
    { 0x72e716f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b716f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e716f8, 93 },
    { 0x125716f8, 95 },
    { 0, 0 },
    { 0xdf3716fb, 65 },
    { 0, 0 },
    { 0x795716fd, 61 },
    { 0, 0 },
    { 0x8d3716ff, 33 },
    { 0, 0 },
    { 0xb4a71701, 95 },
    { 0, 0 },
    { 0x7a171703, 92 },
    { 0xc7c71704, 91 },
    { 0x85171704, 34 },
    { 0xcbc71706, 61 },
    { 0x65771704, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e87170a, 33 },
    { 0xbaa7170b, 33 },
    { 0xcda7170c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a771711, 77 },
    { 0xa3b71711, 34 },
    { 0, 0 },
    { 0x8a971714, 75 },
    { 0, 0 },
    { 0x6c471716, 33 },
    { 0x9a371717, 78 },
    { 0x52571717, 95 },
    { 0x3aa71717, 87 },
    { 0xe957171a, 90 },
    { 0xad071719, 23 },
    { 0xfb97171c, 76 },
    { 0x6547171d, 76 },
    { 0, 0 },
    { 0xeea7171f, 61 },
    { 0xd071720, 88 },
    { 0xae471720, 33 },
    { 0x32271722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae371725, 90 },
    { 0x38471726, 90 },
    { 0xa1871726, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d7172d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13771730, 52 },
    { 0, 0 },
    { 0x44971732, 39 },
    { 0, 0 },
    { 0x32671734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e37173a, 90 },
    { 0x91c7173b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc867173e, 90 },
    { 0x5567173f, 23 },
    { 0, 0 },
    { 0xd0a71741, 33 },
    { 0xc4371741, 66 },
    { 0, 0 },
    { 0x4e071744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7174f, 83 },
    { 0, 0 },
    { 0xf9e71751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d71754, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc71759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7175c, 97 },
    { 0xc407175c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57371760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3771763, 90 },
    { 0x91071763, 33 },
    { 0xd5b71765, 55 },
    { 0x8771766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6271769, 61 },
    { 0xc4e71769, 33 },
    { 0xf5771769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98871771, 90 },
    { 0xa0271772, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e71775, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb7177f, 33 },
    { 0, 0 },
    { 0x9bc71781, 61 },
    { 0xce271782, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1271788, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb7178b, 76 },
    { 0, 0 },
    { 0xecc7178d, 61 },
    { 0xcec7178e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf971799, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17179d, 90 },
    { 0, 0 },
    { 0xd957179f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b717a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46717a7, 84 },
    { 0, 0 },
    { 0xe2b717a9, 34 },
    { 0xdf5717a9, 98 },
    { 0, 0 },
    { 0xb3c717ac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d717b3, 43 },
    { 0x4d9717b4, 61 },
    { 0xdb7717b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26717b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94717bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d717c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e717c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785717ce, 78 },
    { 0xd96717cf, 75 },
    { 0x54e717d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76717d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92717e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7717e3, 23 },
    { 0x64f717e3, 61 },
    { 0x50717e5, 33 },
    { 0xc58717e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc717ec, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960717f0, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b717f6, 55 },
    { 0, 0 },
    { 0x95f717f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48771800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3071804, 98 },
    { 0, 0 },
    { 0x3c371806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07180d, 61 },
    { 0xbd77180e, 98 },
    { 0x3cf7180f, 90 },
    { 0x2e97180e, 95 },
    { 0x94f7180f, 98 },
    { 0x86671812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b571819, 65 },
    { 0xd8b7181a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63371821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50671824, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0171829, 95 },
    { 0xa1c7182a, 82 },
    { 0xaef7182b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c7182e, 84 },
    { 0, 0 },
    { 0xb0871830, 99 },
    { 0, 0 },
    { 0xbe071832, 98 },
    { 0, 0 },
    { 0x68871834, 90 },
    { 0x43d71835, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f7183f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a71845, 55 },
    { 0x4b271846, 90 },
    { 0x5c871847, 33 },
    { 0x16a71847, 76 },
    { 0xf8471849, 91 },
    { 0x1947184a, 78 },
    { 0x2d57184b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7184e, 78 },
    { 0x1517184e, 61 },
    { 0, 0 },
    { 0xe3471851, 52 },
    { 0xcfa71851, 40 },
    { 0, 0 },
    { 0xd3771854, 33 },
    { 0xebd71855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb87185c, 85 },
    { 0xbc87185d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf571860, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63671866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce71869, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa7186c, 68 },
    { 0, 0 },
    { 0xdd37186e, 90 },
    { 0, 0 },
    { 0xdfa71870, 76 },
    { 0x1f971870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde71877, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff47187b, 61 },
    { 0x86e7187c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7187f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db71888, 66 },
    { 0xeed71889, 84 },
    { 0xec271889, 90 },
    { 0xf1271888, 95 },
    { 0xfe7188c, 80 },
    { 0x6497188d, 23 },
    { 0xaf97188e, 61 },
    { 0xddd7188e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86271893, 81 },
    { 0x3c971893, 98 },
    { 0x18471893, 76 },
    { 0xad771896, 98 },
    { 0x3f671895, 25 },
    { 0xdb171898, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe637189b, 67 },
    { 0x95c7189c, 90 },
    { 0x57d7189d, 98 },
    { 0xc867189e, 95 },
    { 0, 0 },
    { 0xce2718a0, 65 },
    { 0x858718a1, 46 },
    { 0x69718a0, 95 },
    { 0x8bb718a3, 91 },
    { 0x6f3718a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x162718a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8718ab, 67 },
    { 0x9a718ac, 90 },
    { 0x6a4718ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7718b0, 46 },
    { 0x862718b1, 34 },
    { 0, 0 },
    { 0x9fc718b3, 61 },
    { 0x275718b3, 100 },
    { 0x799718b3, 100 },
    { 0x2d0718b6, 33 },
    { 0x664718b5, 67 },
    { 0xa6e718b8, 77 },
    { 0x7da718b3, 76 },
    { 0, 0 },
    { 0xb1f718bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356718c5, 95 },
    { 0xe7a718c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76718ca, 34 },
    { 0x434718ca, 81 },
    { 0x68a718cc, 76 },
    { 0x1e8718cd, 43 },
    { 0x9f9718ce, 23 },
    { 0xded718cb, 33 },
    { 0x37d718d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec718d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a718da, 33 },
    { 0x4bf718db, 33 },
    { 0x287718da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6718e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92718ea, 66 },
    { 0x3b8718eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0718f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5718f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe718f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0718fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0718ff, 52 },
    { 0x5f571900, 33 },
    { 0, 0 },
    { 0x4d771902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a271908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb497191a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7191d, 34 },
    { 0, 0 },
    { 0x9ee7191f, 77 },
    { 0xb3d7191f, 81 },
    { 0, 0 },
    { 0x77b71922, 43 },
    { 0x82071923, 90 },
    { 0xa6d71924, 61 },
    { 0x4a371924, 90 },
    { 0xf9871926, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a971929, 61 },
    { 0xa2b7192a, 33 },
    { 0, 0 },
    { 0x7ed7192c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96971930, 61 },
    { 0x4d771931, 85 },
    { 0, 0 },
    { 0xa6a71933, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30571937, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb7193a, 77 },
    { 0x8467193b, 33 },
    { 0x3a27193c, 89 },
    { 0xd3f7193a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd571944, 33 },
    { 0xe9871945, 33 },
    { 0xc1271946, 33 },
    { 0xbcc71946, 98 },
    { 0x59071948, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7194b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d71957, 100 },
    { 0x1a371958, 99 },
    { 0x93371959, 61 },
    { 0x4617195a, 76 },
    { 0x7337195b, 61 },
    { 0x9227195a, 34 },
    { 0xb47195d, 33 },
    { 0xa797195e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79871961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62071964, 78 },
    { 0, 0 },
    { 0x49d71966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f71969, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a57196c, 90 },
    { 0xd417196d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d71971, 90 },
    { 0, 0 },
    { 0x7ee71973, 65 },
    { 0x87c71973, 23 },
    { 0, 0 },
    { 0x27971976, 90 },
    { 0xfa771976, 76 },
    { 0xe4071978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f7197c, 98 },
    { 0xf427197d, 95 },
    { 0, 0 },
    { 0x8fb7197f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1471984, 33 },
    { 0x7d971984, 61 },
    { 0, 0 },
    { 0xd571987, 46 },
    { 0, 0 },
    { 0x75071989, 99 },
    { 0xbf17198a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7198d, 98 },
    { 0xc847198e, 98 },
    { 0xd7d7198f, 98 },
    { 0xf4571990, 78 },
    { 0x89371991, 55 },
    { 0x47d71992, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c71996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c87199d, 33 },
    { 0x9de7199e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc719a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475719a7, 90 },
    { 0xe75719a7, 90 },
    { 0x564719a8, 33 },
    { 0, 0 },
    { 0xad719ab, 33 },
    { 0, 0 },
    { 0x5cb719ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x602719b0, 90 },
    { 0x97c719b1, 98 },
    { 0x3c5719b2, 23 },
    { 0x75d719b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577719b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6719bd, 23 },
    { 0x297719be, 93 },
    { 0xb1a719bf, 88 },
    { 0, 0 },
    { 0x9ec719c1, 61 },
    { 0x64f719c2, 78 },
    { 0xff719c3, 93 },
    { 0, 0 },
    { 0x189719c5, 55 },
    { 0, 0 },
    { 0x633719c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a719ca, 98 },
    { 0xe64719cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a719d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88719d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e719d9, 52 },
    { 0, 0 },
    { 0x4ed719db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875719e1, 61 },
    { 0, 0 },
    { 0x250719e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b719e7, 87 },
    { 0xd57719e8, 65 },
    { 0, 0 },
    { 0xd1d719ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877719ef, 100 },
    { 0x77e719f0, 23 },
    { 0, 0 },
    { 0xa0f719f2, 77 },
    { 0x912719f3, 98 },
    { 0xf9f719f2, 55 },
    { 0, 0 },
    { 0x9ab719f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae719fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a671a07, 98 },
    { 0x6f871a07, 23 },
    { 0x7f71a08, 61 },
    { 0, 0 },
    { 0x15d71a0b, 23 },
    { 0, 0 },
    { 0x3df71a0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3471a11, 76 },
    { 0xd6871a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7071a16, 95 },
    { 0, 0 },
    { 0x9d271a18, 84 },
    { 0, 0 },
    { 0xc0a71a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371a1d, 93 },
    { 0x18771a1e, 17 },
    { 0x33071a1d, 95 },
    { 0, 0 },
    { 0xa9b71a21, 98 },
    { 0xb2971a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50771a26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb71a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed71a2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe971a31, 98 },
    { 0xfaf71a32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d771a37, 33 },
    { 0, 0 },
    { 0x39a71a39, 90 },
    { 0, 0 },
    { 0x53971a3b, 67 },
    { 0x60471a3c, 55 },
    { 0xc1971a3d, 33 },
    { 0x4f471a3e, 23 },
    { 0x2fe71a3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a771a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71a48, 61 },
    { 0x2f571a49, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43071a50, 99 },
    { 0, 0 },
    { 0xc2471a52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x78371a55, 98 },
    { 0xd9071a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54371a59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b471a5c, 91 },
    { 0x20371a5d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b71a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3871a67, 80 },
    { 0x58471a68, 33 },
    { 0xad971a69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0071a6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc71a71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x44271a74, 90 },
    { 0x66771a75, 99 },
    { 0x1a971a75, 33 },
    { 0x17171a76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36871a7a, 76 },
    { 0x6af71a7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc871a7e, 98 },
    { 0, 0 },
    { 0xf2d71a80, 100 },
    { 0xb2e71a80, 84 },
    { 0xb6371a81, 76 },
    { 0x55871a83, 90 },
    { 0xa3a71a83, 66 },
    { 0x60471a85, 90 },
    { 0x62c71a86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48871a8a, 61 },
    { 0xcfe71a8a, 90 },
    { 0x21471a8c, 61 },
    { 0x66c71a8d, 66 },
    { 0xc9871a8d, 99 },
    { 0xc1d71a8e, 84 },
    { 0xbcf71a90, 23 },
    { 0x1db71a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71a97, 75 },
    { 0x48371a98, 43 },
    { 0, 0 },
    { 0x28371a9a, 95 },
    { 0x21d71a9b, 33 },
    { 0x3cc71a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd71a9f, 13 },
    { 0x9671aa0, 23 },
    { 0xb571a9f, 90 },
    { 0, 0 },
    { 0xc9071aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4571aa7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7771aac, 61 },
    { 0, 0 },
    { 0x20271aae, 98 },
    { 0xf3771aae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b471ab4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b871ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18571abd, 33 },
    { 0x9cf71abd, 98 },
    { 0, 0 },
    { 0x9c871ac0, 33 },
    { 0xf0771ac0, 90 },
    { 0x5c571ac2, 93 },
    { 0x5d571ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde771ac9, 98 },
    { 0x5eb71ac9, 55 },
    { 0xc0571aca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d71acf, 84 },
    { 0xe6971acf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f471ad6, 81 },
    { 0x9071ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a71ada, 95 },
    { 0x64771adb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7071ade, 68 },
    { 0x2ec71adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde371ae2, 61 },
    { 0x60271ae2, 98 },
    { 0xb5f71ae4, 55 },
    { 0xe7071ae4, 75 },
    { 0x99371ae5, 61 },
    { 0x8e71ae6, 85 },
    { 0xd971ae8, 90 },
    { 0x6fb71ae8, 33 },
    { 0x4de71ae6, 33 },
    { 0x65571aeb, 91 },
    { 0xa6771ae4, 91 },
    { 0xc9771ae8, 95 },
    { 0, 0 },
    { 0xd2671aef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea71af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c71afa, 95 },
    { 0xf7871afa, 99 },
    { 0, 0 },
    { 0xced71afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd71b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed671b05, 34 },
    { 0x3af71b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a371b09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671b0c, 98 },
    { 0xa4b71b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75771b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71b13, 78 },
    { 0, 0 },
    { 0x93c71b15, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f771b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c271b1e, 95 },
    { 0, 0 },
    { 0x3b71b20, 75 },
    { 0x79771b21, 98 },
    { 0x4a271b22, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c71b26, 95 },
    { 0x67771b27, 23 },
    { 0, 0 },
    { 0xe7d71b29, 66 },
    { 0xf2a71b29, 61 },
    { 0xa7b71b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda71b34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe671b38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea71b3b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec71b3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f71b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a71b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3271b4e, 33 },
    { 0, 0 },
    { 0x2ba71b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1671b55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d71b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x61671b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe971b63, 34 },
    { 0, 0 },
    { 0x7a471b65, 88 },
    { 0xbe371b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc571b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b071b72, 55 },
    { 0x9ea71b73, 23 },
    { 0x90571b73, 90 },
    { 0x1dc71b73, 23 },
    { 0x6cd71b75, 23 },
    { 0xfd971b77, 100 },
    { 0x8f371b78, 61 },
    { 0xa9971b76, 17 },
    { 0x7b371b7a, 66 },
    { 0xdca71b7b, 76 },
    { 0x26871b7c, 90 },
    { 0xec471b7d, 33 },
    { 0, 0 },
    { 0x4c271b7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f71b82, 33 },
    { 0, 0 },
    { 0x4c771b84, 43 },
    { 0, 0 },
    { 0xeb871b86, 95 },
    { 0xa2c71b87, 99 },
    { 0x52971b88, 61 },
    { 0xa6871b89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a71b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea971b93, 98 },
    { 0, 0 },
    { 0x21171b95, 78 },
    { 0xd2971b96, 90 },
    { 0, 0 },
    { 0xb7271b98, 93 },
    { 0xd2771b99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99471b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f771ba2, 95 },
    { 0, 0 },
    { 0x33471ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0471ba8, 61 },
    { 0xf8671ba9, 33 },
    { 0xf9271ba8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3771bae, 17 },
    { 0, 0 },
    { 0xfce71bb0, 23 },
    { 0x14471bb0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd71bb5, 98 },
    { 0, 0 },
    { 0x36971bb7, 90 },
    { 0x82c71bb8, 33 },
    { 0x74a71bb8, 33 },
    { 0xecc71bba, 90 },
    { 0xd3771bba, 98 },
    { 0xf2c71bbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b471bbf, 95 },
    { 0xa8a71bbf, 84 },
    { 0, 0 },
    { 0xfa971bc2, 23 },
    { 0xbb671bc3, 78 },
    { 0x28271bc2, 90 },
    { 0x70a71bc5, 84 },
    { 0xb9371bc4, 90 },
    { 0, 0 },
    { 0x61371bc8, 61 },
    { 0, 0 },
    { 0x17771bca, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf371bd0, 55 },
    { 0xa7c71bd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e71bd4, 55 },
    { 0x52771bd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3371bda, 95 },
    { 0xd5d71bdb, 61 },
    { 0x11a71bdc, 90 },
    { 0, 0 },
    { 0xf6b71bde, 46 },
    { 0x40271bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d571be3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3271be9, 33 },
    { 0x50871bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49571bf1, 33 },
    { 0xd4971bf2, 67 },
    { 0xddf71bf3, 98 },
    { 0, 0 },
    { 0x20d71bf5, 33 },
    { 0x3ba71bf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78671bf9, 76 },
    { 0x88571bfa, 23 },
    { 0x71871bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed371c06, 61 },
    { 0x6ae71c06, 95 },
    { 0, 0 },
    { 0xd5471c09, 55 },
    { 0xb8471c0a, 76 },
    { 0xd4b71c0b, 68 },
    { 0x1d771c0a, 52 },
    { 0x60a71c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c71c12, 77 },
    { 0xea371c13, 95 },
    { 0xea971c14, 33 },
    { 0xbed71c15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b871c1b, 95 },
    { 0xe9771c1c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x78271c1f, 90 },
    { 0, 0 },
    { 0x13871c21, 90 },
    { 0xb7b71c22, 68 },
    { 0x9ea71c23, 95 },
    { 0, 0 },
    { 0x58571c25, 55 },
    { 0x68871c26, 46 },
    { 0x25d71c26, 55 },
    { 0, 0 },
    { 0xab171c29, 76 },
    { 0x89871c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96571c2d, 61 },
    { 0x27971c2e, 61 },
    { 0x33771c2e, 77 },
    { 0x91071c2d, 33 },
    { 0xd5971c30, 33 },
    { 0x90d71c32, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37071c38, 52 },
    { 0x93671c39, 90 },
    { 0x7cc71c3a, 91 },
    { 0xb6b71c3b, 61 },
    { 0x45671c3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb471c48, 23 },
    { 0, 0 },
    { 0x8ba71c4a, 33 },
    { 0x54171c4b, 93 },
    { 0xaa471c4a, 61 },
    { 0, 0 },
    { 0x53971c4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e671c58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21271c5c, 33 },
    { 0x7c671c5d, 66 },
    { 0xf1d71c5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f871c62, 34 },
    { 0, 0 },
    { 0xb0871c64, 61 },
    { 0x18b71c65, 93 },
    { 0x43e71c65, 75 },
    { 0x13071c64, 66 },
    { 0, 0 },
    { 0x69c71c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e71c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f71c71, 33 },
    { 0xf8d71c72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8571c75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f171c7c, 90 },
    { 0x4671c7d, 23 },
    { 0xda071c7e, 88 },
    { 0xc4c71c7c, 88 },
    { 0x34471c7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c71c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89171c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf671c8d, 61 },
    { 0x1371c8e, 78 },
    { 0, 0 },
    { 0x22f71c90, 99 },
    { 0x1af71c90, 90 },
    { 0, 0 },
    { 0x6ed71c93, 23 },
    { 0x16571c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2671c97, 33 },
    { 0xca871c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89771ca0, 98 },
    { 0x72a71ca0, 68 },
    { 0x86d71ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47071ca8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87871cac, 23 },
    { 0x271cac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a571cb2, 98 },
    { 0xaf071cb2, 43 },
    { 0xd0971cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa71cb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce71cbd, 98 },
    { 0x94071cbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17671cc1, 61 },
    { 0xf1271cc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a371cc6, 99 },
    { 0x70471cc7, 34 },
    { 0x58071cc7, 67 },
    { 0xe1671cc8, 33 },
    { 0x77b71cca, 23 },
    { 0xf8371cc7, 33 },
    { 0x30d71cc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e71cd3, 65 },
    { 0x6e471cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34471cda, 78 },
    { 0xc7f71cda, 36 },
    { 0xd8071cdb, 23 },
    { 0xdb871cdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28371ce2, 90 },
    { 0xdf371ce2, 33 },
    { 0x88371ce4, 76 },
    { 0, 0 },
    { 0x59871ce6, 33 },
    { 0xc6871ce7, 66 },
    { 0x16671ce8, 61 },
    { 0xa4171ce6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca71ced, 95 },
    { 0x90e71ced, 23 },
    { 0, 0 },
    { 0xc8e71cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e371cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3471cf7, 46 },
    { 0xf8971cf8, 95 },
    { 0xff471cf7, 55 },
    { 0x12471cfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5271cff, 98 },
    { 0x67d71d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a71d06, 33 },
    { 0, 0 },
    { 0xb3571d08, 43 },
    { 0xa0571d09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0371d0f, 88 },
    { 0, 0 },
    { 0xe8271d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf571d16, 99 },
    { 0, 0 },
    { 0x56271d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa071d1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f271d24, 84 },
    { 0x9bc71d24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab071d2b, 66 },
    { 0xe9571d2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a71d2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d071d32, 90 },
    { 0xb71d33, 55 },
    { 0x4e771d33, 33 },
    { 0xbe471d35, 99 },
    { 0xad671d35, 76 },
    { 0x3cb71d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85671d3d, 61 },
    { 0xa471d3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17271d41, 94 },
    { 0, 0 },
    { 0xfb871d43, 33 },
    { 0xe1371d43, 33 },
    { 0x80071d45, 93 },
    { 0, 0 },
    { 0x3f571d47, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e71d50, 91 },
    { 0xfc871d50, 93 },
    { 0, 0 },
    { 0xa4771d53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0571d57, 33 },
    { 0x33e71d58, 33 },
    { 0xfd071d59, 98 },
    { 0, 0 },
    { 0xe0871d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd171d5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd371d62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19471d65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd71d6a, 90 },
    { 0x43171d6a, 65 },
    { 0x58571d6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f71d7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac271d7d, 88 },
    { 0x9b271d7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5271d81, 33 },
    { 0, 0 },
    { 0x14971d83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c871d87, 61 },
    { 0, 0 },
    { 0x49371d89, 98 },
    { 0xd4071d8a, 76 },
    { 0, 0 },
    { 0x89171d8c, 93 },
    { 0x95f71d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63771d90, 33 },
    { 0x4f071d91, 56 },
    { 0x7cd71d91, 37 },
    { 0x24a71d93, 87 },
    { 0xbc471d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9671d9e, 99 },
    { 0x13371d9f, 52 },
    { 0xe8f71d9e, 33 },
    { 0xe6371da0, 90 },
    { 0xd2671da2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea571da5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e571da9, 52 },
    { 0xd4371da9, 43 },
    { 0xea371daa, 33 },
    { 0x10371da9, 33 },
    { 0, 0 },
    { 0x46271dae, 78 },
    { 0, 0 },
    { 0xac371db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0571db3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a71db7, 61 },
    { 0, 0 },
    { 0x74d71db9, 61 },
    { 0x28771dba, 56 },
    { 0x81171dbb, 23 },
    { 0x81f71dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f71dc3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd71dc6, 61 },
    { 0xfda71dc7, 87 },
    { 0x971dc8, 23 },
    { 0xfc671dc9, 64 },
    { 0x3df71dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f671dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b71dd4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7771dde, 61 },
    { 0, 0 },
    { 0xce671de0, 61 },
    { 0x6f571de1, 55 },
    { 0, 0 },
    { 0x1da71de3, 61 },
    { 0, 0 },
    { 0x36871de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71deb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e71dee, 90 },
    { 0, 0 },
    { 0xf5771df0, 33 },
    { 0, 0 },
    { 0x9e071df2, 98 },
    { 0xc071df2, 46 },
    { 0x51471df4, 61 },
    { 0x88b71df5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b71dfa, 91 },
    { 0x86b71dfb, 87 },
    { 0x89571dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa71e00, 98 },
    { 0x28571e00, 61 },
    { 0xea171e00, 98 },
    { 0x75371e00, 91 },
    { 0x4c71e00, 90 },
    { 0xbf971e05, 87 },
    { 0x98b71e02, 33 },
    { 0xdc671e04, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x21871e0a, 43 },
    { 0, 0 },
    { 0x1f371e0c, 95 },
    { 0, 0 },
    { 0xee971e0e, 75 },
    { 0xe7d71e0e, 8 },
    { 0x74471e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba71e16, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9971e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd871e21, 90 },
    { 0x97871e22, 99 },
    { 0, 0 },
    { 0x2f471e24, 23 },
    { 0xb7f71e25, 100 },
    { 0x39571e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c71e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a71e2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e671e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67371e35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e271e3b, 90 },
    { 0xf5471e3c, 91 },
    { 0x5c971e3c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a71e40, 90 },
    { 0, 0 },
    { 0x4b371e42, 17 },
    { 0x49f71e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb71e48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63171e4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a71e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4471e5e, 93 },
    { 0x38f71e5f, 98 },
    { 0xc8e71e5e, 90 },
    { 0, 0 },
    { 0x24f71e62, 76 },
    { 0xe4e71e62, 91 },
    { 0xacc71e64, 34 },
    { 0xb1771e65, 33 },
    { 0xcfb71e63, 91 },
    { 0x2b171e67, 90 },
    { 0x14271e67, 33 },
    { 0x59d71e67, 34 },
    { 0, 0 },
    { 0x80671e6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d71e6e, 33 },
    { 0xc871e6f, 66 },
    { 0x87571e70, 90 },
    { 0x6b171e71, 33 },
    { 0x4c971e72, 61 },
    { 0xb9f71e73, 95 },
    { 0x91471e74, 34 },
    { 0, 0 },
    { 0xadb71e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58171e7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76571e81, 98 },
    { 0xe0071e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed471e85, 33 },
    { 0x86a71e86, 98 },
    { 0x8a971e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed71e8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a71e8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b71e9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba71e9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b71ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d071ea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b71ebf, 90 },
    { 0x3b871ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b71ec3, 23 },
    { 0xec771ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f71ec9, 33 },
    { 0xb2371eca, 55 },
    { 0xea471ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe971ece, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4171ed2, 90 },
    { 0x9f171ed2, 95 },
    { 0xb4671ed4, 46 },
    { 0, 0 },
    { 0x22271ed6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca171eda, 52 },
    { 0xad71edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c71ede, 61 },
    { 0x67771edf, 23 },
    { 0x7bd71edf, 95 },
    { 0xa6b71edf, 43 },
    { 0, 0 },
    { 0xe3771ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67271ee9, 33 },
    { 0xf1271ee9, 33 },
    { 0xe7d71eeb, 46 },
    { 0x21c71eeb, 61 },
    { 0xa7571eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f171ef0, 98 },
    { 0xd9571ef1, 23 },
    { 0xe1571ef0, 55 },
    { 0xe0271ef3, 98 },
    { 0xfc071ef4, 34 },
    { 0x9d271ef3, 95 },
    { 0x93f71ef6, 75 },
    { 0x2771ef6, 19 },
    { 0x8f971ef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f71efb, 44 },
    { 0xd0771efc, 91 },
    { 0x9f771efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef71f00, 93 },
    { 0x91271f00, 98 },
    { 0, 0 },
    { 0x53271f03, 61 },
    { 0xe1b71f03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36571f09, 93 },
    { 0, 0 },
    { 0xad671f0b, 33 },
    { 0x3d71f0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b471f0f, 87 },
    { 0x1d771f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d471f13, 23 },
    { 0, 0 },
    { 0x41571f15, 46 },
    { 0x6ed71f15, 90 },
    { 0, 0 },
    { 0x1dd71f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29971f20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x32871f23, 90 },
    { 0x19f71f24, 84 },
    { 0, 0 },
    { 0x7bd71f26, 78 },
    { 0, 0 },
    { 0xeab71f28, 90 },
    { 0xf9871f28, 23 },
    { 0xb9271f28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc71f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f71f36, 23 },
    { 0x15471f36, 34 },
    { 0, 0 },
    { 0x4bb71f39, 33 },
    { 0x44f71f3a, 88 },
    { 0xdeb71f3a, 88 },
    { 0, 0 },
    { 0x62c71f3d, 61 },
    { 0x4d771f3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53571f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6871f4f, 76 },
    { 0xdfe71f50, 61 },
    { 0, 0 },
    { 0xa8571f52, 90 },
    { 0, 0 },
    { 0xb7171f54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef471f59, 33 },
    { 0x36d71f5a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3971f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7771f66, 88 },
    { 0xa1371f66, 80 },
    { 0xac371f66, 87 },
    { 0, 0 },
    { 0x23a71f6a, 61 },
    { 0, 0 },
    { 0x33b71f6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9071f70, 90 },
    { 0, 0 },
    { 0x42a71f72, 61 },
    { 0x65d71f73, 98 },
    { 0x59c71f74, 95 },
    { 0x2e671f75, 33 },
    { 0x1bb71f75, 61 },
    { 0xf0671f77, 76 },
    { 0xee671f75, 67 },
    { 0, 0 },
    { 0x49071f7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b71f7e, 77 },
    { 0, 0 },
    { 0xd471f80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f871f83, 95 },
    { 0x7b871f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3871f8a, 33 },
    { 0x87e71f8b, 90 },
    { 0x4a571f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64071f92, 61 },
    { 0xe671f93, 84 },
    { 0, 0 },
    { 0xa6071f95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7671f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce71f9b, 43 },
    { 0, 0 },
    { 0xd6871f9d, 91 },
    { 0x83371f9e, 66 },
    { 0xd4071f9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e571fa8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee371fad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d171fb0, 55 },
    { 0x4cb71fb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7471fb8, 91 },
    { 0x28071fb8, 61 },
    { 0x6cf71fb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d271fbe, 98 },
    { 0xfda71fbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d271fc2, 90 },
    { 0xf1d71fc3, 95 },
    { 0x1c871fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac71fc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef071fcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c71fd3, 90 },
    { 0x4d271fd4, 90 },
    { 0xb1671fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3371fdb, 91 },
    { 0, 0 },
    { 0x7e571fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa971fe2, 90 },
    { 0xf7d71fe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5971fe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a471feb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79471ff5, 52 },
    { 0x37371ff6, 91 },
    { 0xe2271ff7, 43 },
    { 0x35071ff8, 27 },
    { 0x6d471ff6, 23 },
    { 0xfbc71ffa, 33 },
    { 0xfc571ffb, 55 },
    { 0x7c671ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa671fff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32772003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b72008, 90 },
    { 0xaa572009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67200c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a72015, 98 },
    { 0xdd772016, 23 },
    { 0xcf572017, 23 },
    { 0x80472016, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d72022, 33 },
    { 0xb7772022, 88 },
    { 0x3cc72022, 90 },
    { 0xb1f72025, 55 },
    { 0xaf772023, 33 },
    { 0xf5372023, 55 },
    { 0, 0 },
    { 0x79672029, 23 },
    { 0, 0 },
    { 0x6fb7202b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57072031, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed72035, 55 },
    { 0xd3972036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e72039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7203e, 80 },
    { 0, 0 },
    { 0x2aa72040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14272049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7204f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d37205b, 80 },
    { 0xa197205c, 95 },
    { 0x9017205d, 95 },
    { 0, 0 },
    { 0xd97205f, 98 },
    { 0x87d72060, 90 },
    { 0, 0 },
    { 0x75772062, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc172069, 99 },
    { 0xd5b72069, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047206e, 61 },
    { 0, 0 },
    { 0xa9f72070, 61 },
    { 0, 0 },
    { 0x9fd72072, 76 },
    { 0, 0 },
    { 0x4e072074, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc7207b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d17207e, 61 },
    { 0xc4e7207e, 93 },
    { 0, 0 },
    { 0x38e72081, 91 },
    { 0x3d272081, 95 },
    { 0xa1a72081, 33 },
    { 0x2a372084, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x972087, 90 },
    { 0x76572088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297208b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6372091, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9172096, 66 },
    { 0xd4972096, 90 },
    { 0x23072098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950720a4, 55 },
    { 0xa29720a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1720a8, 76 },
    { 0x73a720a9, 93 },
    { 0, 0 },
    { 0x9ab720ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4720b0, 77 },
    { 0x5a7720b1, 36 },
    { 0x7ff720b2, 61 },
    { 0x1ac720b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27720b8, 90 },
    { 0x88d720b9, 61 },
    { 0x236720ba, 46 },
    { 0x709720bb, 95 },
    { 0xe45720bc, 61 },
    { 0x1d7720b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c720c5, 33 },
    { 0x98a720c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x724720c9, 17 },
    { 0, 0 },
    { 0xdc4720cb, 90 },
    { 0xb8d720cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6720d9, 98 },
    { 0, 0 },
    { 0x70a720db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05720de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2720e1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810720e7, 42 },
    { 0, 0 },
    { 0x39b720e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4720ef, 23 },
    { 0, 0 },
    { 0x37b720f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8720f6, 33 },
    { 0x7ca720f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e720fd, 76 },
    { 0x57a720fd, 90 },
    { 0xc0d720fd, 95 },
    { 0xe7172100, 23 },
    { 0x8c720ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7972104, 78 },
    { 0x5da72105, 97 },
    { 0xd8872105, 95 },
    { 0x8b872107, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7210b, 100 },
    { 0, 0 },
    { 0x85e7210d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e572111, 91 },
    { 0, 0 },
    { 0x19e72113, 95 },
    { 0x21672114, 55 },
    { 0xc3972114, 66 },
    { 0x9d72115, 47 },
    { 0xa7e72117, 87 },
    { 0x87572117, 33 },
    { 0x57872115, 98 },
    { 0xe947211a, 95 },
    { 0, 0 },
    { 0x53e7211c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff72122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8872131, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a72137, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67213a, 61 },
    { 0xcbc7213b, 99 },
    { 0, 0 },
    { 0x3df7213d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0272140, 87 },
    { 0x1be72141, 95 },
    { 0xb1472142, 76 },
    { 0xa9172142, 33 },
    { 0x6b172144, 77 },
    { 0x17972141, 90 },
    { 0, 0 },
    { 0xf672147, 46 },
    { 0xe5272148, 23 },
    { 0, 0 },
    { 0x8eb7214a, 34 },
    { 0x6de7214b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea7214e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e72155, 95 },
    { 0xd7072155, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77215a, 43 },
    { 0, 0 },
    { 0x67d7215c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b72163, 98 },
    { 0xf2072164, 46 },
    { 0xe8772165, 77 },
    { 0xe4a72164, 90 },
    { 0x19c72167, 76 },
    { 0, 0 },
    { 0x84172169, 17 },
    { 0x1337216a, 91 },
    { 0x2af7216b, 34 },
    { 0x1ff7216b, 90 },
    { 0, 0 },
    { 0xbdf7216e, 90 },
    { 0x53e7216f, 78 },
    { 0xb8f7216f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0872173, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe7217a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a7217d, 90 },
    { 0x2fa7217e, 33 },
    { 0x7cf7217f, 98 },
    { 0, 0 },
    { 0x68072181, 76 },
    { 0, 0 },
    { 0xf5a72183, 33 },
    { 0x77672184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f72187, 81 },
    { 0x14872188, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b7218d, 98 },
    { 0x8537218d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x872191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc72194, 33 },
    { 0, 0 },
    { 0x70772196, 90 },
    { 0, 0 },
    { 0x35772198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc7219d, 33 },
    { 0xdb7219e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507721a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e721a8, 67 },
    { 0x861721a9, 81 },
    { 0x3e0721a9, 23 },
    { 0x574721ab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb721ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459721b3, 23 },
    { 0xe47721b3, 33 },
    { 0x386721b5, 65 },
    { 0x3ca721b5, 52 },
    { 0x944721b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe721bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867721c1, 100 },
    { 0x73f721c2, 95 },
    { 0xfb9721c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736721ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89721cd, 98 },
    { 0, 0 },
    { 0xbcd721cf, 55 },
    { 0xadf721cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x211721d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773721d8, 90 },
    { 0x6bb721d9, 66 },
    { 0, 0 },
    { 0xfee721db, 55 },
    { 0, 0 },
    { 0xc69721dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539721e3, 90 },
    { 0, 0 },
    { 0xf1f721e5, 34 },
    { 0, 0 },
    { 0x4fe721e7, 34 },
    { 0xfde721e8, 78 },
    { 0x86f721e9, 77 },
    { 0x47f721e9, 34 },
    { 0x711721e9, 99 },
    { 0xc52721ec, 61 },
    { 0xfbe721ed, 34 },
    { 0, 0 },
    { 0xddc721ef, 61 },
    { 0xfc721f0, 76 },
    { 0x712721f1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x533721f4, 61 },
    { 0xac8721f5, 95 },
    { 0x7e6721f6, 61 },
    { 0x878721f5, 34 },
    { 0, 0 },
    { 0xe6a721f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d721fc, 33 },
    { 0xcab721fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce72201, 95 },
    { 0, 0 },
    { 0x2bc72203, 23 },
    { 0x4e72204, 38 },
    { 0xc5e72205, 85 },
    { 0x11f72206, 90 },
    { 0x11b72205, 23 },
    { 0xe0a72204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa537220c, 55 },
    { 0x16c7220d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b72210, 23 },
    { 0xd6d72211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f72215, 23 },
    { 0, 0 },
    { 0x42b72217, 23 },
    { 0x73472218, 77 },
    { 0xce172218, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d77221c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad72220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa72225, 43 },
    { 0x14472226, 85 },
    { 0x25472225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f27222a, 66 },
    { 0, 0 },
    { 0x31b7222c, 55 },
    { 0, 0 },
    { 0xbf07222e, 55 },
    { 0xadd7222f, 98 },
    { 0, 0 },
    { 0x75372231, 75 },
    { 0x20372232, 61 },
    { 0x89372231, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61272239, 89 },
    { 0, 0 },
    { 0xd017223b, 93 },
    { 0xe6a7223c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f7223f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e72248, 95 },
    { 0x84172249, 95 },
    { 0xe572249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c7224f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd872252, 93 },
    { 0, 0 },
    { 0xc1572254, 91 },
    { 0x30e72255, 90 },
    { 0, 0 },
    { 0xac972257, 95 },
    { 0xf6e72257, 99 },
    { 0xe872259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27225e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e72262, 55 },
    { 0, 0 },
    { 0x10872264, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c7226d, 98 },
    { 0xdab7226d, 90 },
    { 0x9447226d, 61 },
    { 0x1c172270, 23 },
    { 0xb1872271, 90 },
    { 0x8be72272, 78 },
    { 0xea07226d, 67 },
    { 0xea372274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d77227a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc7227e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a72282, 49 },
    { 0x80372283, 33 },
    { 0x9d272284, 98 },
    { 0x26772285, 93 },
    { 0, 0 },
    { 0xf7572287, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9257228a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf947228f, 43 },
    { 0x2eb72290, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd72296, 95 },
    { 0, 0 },
    { 0x33272298, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757229b, 90 },
    { 0x5c97229c, 91 },
    { 0xecf7229c, 33 },
    { 0xf707229c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f722a6, 91 },
    { 0, 0 },
    { 0x4c7722a8, 98 },
    { 0xfcb722a8, 90 },
    { 0xe8a722aa, 84 },
    { 0xc46722ab, 61 },
    { 0x438722ac, 52 },
    { 0xf7c722ab, 95 },
    { 0x5a0722ad, 93 },
    { 0xcdb722af, 61 },
    { 0x687722b0, 77 },
    { 0, 0 },
    { 0x89722b2, 87 },
    { 0x659722b3, 33 },
    { 0x13b722b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7722b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65722bd, 90 },
    { 0, 0 },
    { 0x2c4722bf, 90 },
    { 0x2e5722c0, 95 },
    { 0, 0 },
    { 0xd30722c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c722cc, 98 },
    { 0xe65722cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c722d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2722d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376722da, 33 },
    { 0, 0 },
    { 0xacd722dc, 88 },
    { 0xbe6722dc, 34 },
    { 0xfbd722de, 98 },
    { 0x119722de, 23 },
    { 0, 0 },
    { 0x34a722e1, 87 },
    { 0x64f722e2, 52 },
    { 0x490722e2, 23 },
    { 0xc4b722e3, 61 },
    { 0xda722e5, 90 },
    { 0x371722e6, 33 },
    { 0xedc722e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x206722ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0722ed, 51 },
    { 0xe07722ee, 61 },
    { 0xc6722ee, 33 },
    { 0xcce722ee, 85 },
    { 0, 0 },
    { 0xb48722f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b722f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873722fe, 71 },
    { 0xafb722ff, 91 },
    { 0x2dc722ff, 33 },
    { 0xe5d722ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b72304, 23 },
    { 0x1bb72304, 33 },
    { 0xbc272306, 77 },
    { 0xb8472307, 95 },
    { 0xbda72308, 66 },
    { 0xd8072307, 90 },
    { 0xa2d7230a, 40 },
    { 0x5ea7230a, 98 },
    { 0, 0 },
    { 0x1037230d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa72311, 98 },
    { 0xb9572312, 85 },
    { 0, 0 },
    { 0x2072314, 90 },
    { 0x80f72314, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8672319, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f72324, 99 },
    { 0x46072325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7407232a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217232d, 98 },
    { 0, 0 },
    { 0xba87232f, 61 },
    { 0xe6f72330, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13572335, 33 },
    { 0x3d772336, 34 },
    { 0x3b72337, 95 },
    { 0xfde72335, 60 },
    { 0x5a572339, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527233c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb57233f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c72345, 91 },
    { 0, 0 },
    { 0xf7d72347, 33 },
    { 0, 0 },
    { 0xeb872349, 52 },
    { 0, 0 },
    { 0xa967234b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe172351, 90 },
    { 0xcc472351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73072356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd7235b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3447235f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c972367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7236c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c372370, 90 },
    { 0x10c72371, 98 },
    { 0, 0 },
    { 0xf8e72373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3027237a, 23 },
    { 0, 0 },
    { 0xc3c7237c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de7237f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac72383, 61 },
    { 0, 0 },
    { 0xe6f72385, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b572389, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f27238f, 33 },
    { 0, 0 },
    { 0x1072391, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb372395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59772399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797239d, 90 },
    { 0x1737239e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a723a1, 34 },
    { 0x54a723a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f723a5, 33 },
    { 0xc4e723a6, 90 },
    { 0x98b723a7, 98 },
    { 0, 0 },
    { 0x138723a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43723ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5723af, 98 },
    { 0x2b0723b0, 61 },
    { 0x381723b0, 61 },
    { 0, 0 },
    { 0xca6723b3, 33 },
    { 0, 0 },
    { 0x997723b5, 90 },
    { 0x4e9723b6, 90 },
    { 0xa06723b7, 46 },
    { 0x4b4723b8, 67 },
    { 0, 0 },
    { 0xff1723ba, 90 },
    { 0x9a723bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0723be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36723c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9723c5, 61 },
    { 0, 0 },
    { 0x1e3723c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37723ca, 61 },
    { 0x7c2723cb, 46 },
    { 0xaaa723cc, 93 },
    { 0xe33723ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce723d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3723d3, 34 },
    { 0xa9f723d4, 76 },
    { 0xa4f723d4, 98 },
    { 0x5bc723d5, 33 },
    { 0xdb1723d6, 76 },
    { 0x3e5723d8, 76 },
    { 0x934723d4, 55 },
    { 0xc21723da, 76 },
    { 0x8ee723db, 85 },
    { 0x799723dc, 98 },
    { 0xf8a723dd, 33 },
    { 0x67f723db, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474723e8, 38 },
    { 0, 0 },
    { 0x569723ea, 43 },
    { 0, 0 },
    { 0x40d723ec, 33 },
    { 0x6c0723ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73723f0, 95 },
    { 0, 0 },
    { 0xe5b723f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c723f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93723fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba72400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a72404, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x56472407, 98 },
    { 0x19572407, 55 },
    { 0x5e572408, 43 },
    { 0x1267240a, 91 },
    { 0x47f7240b, 98 },
    { 0, 0 },
    { 0x9997240d, 100 },
    { 0x4f47240e, 40 },
    { 0x2ca7240e, 66 },
    { 0x3a672410, 93 },
    { 0x5637240f, 61 },
    { 0x50472412, 76 },
    { 0x1c77240f, 33 },
    { 0xf1072414, 43 },
    { 0x3c72415, 78 },
    { 0x73372416, 23 },
    { 0, 0 },
    { 0x89872418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f07241c, 33 },
    { 0xaee7241c, 95 },
    { 0xcb97241e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8472421, 43 },
    { 0, 0 },
    { 0xc9172423, 96 },
    { 0xc4472424, 61 },
    { 0xd4272423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f7242b, 95 },
    { 0xbd57242c, 90 },
    { 0, 0 },
    { 0x4a77242e, 92 },
    { 0x8f97242e, 34 },
    { 0x84a72430, 93 },
    { 0xe5472431, 43 },
    { 0xc9c72432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2087243a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7243d, 68 },
    { 0, 0 },
    { 0x5c87243f, 34 },
    { 0x4a572440, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79772446, 23 },
    { 0x23272447, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7097244c, 61 },
    { 0x2767244d, 23 },
    { 0, 0 },
    { 0x5f07244f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6772453, 69 },
    { 0x22b72453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b672457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x737245a, 95 },
    { 0x2b37245b, 95 },
    { 0xe7a7245c, 61 },
    { 0x6727245d, 67 },
    { 0xdbf7245e, 70 },
    { 0xae7245e, 93 },
    { 0x9b672460, 46 },
    { 0xcd37245c, 33 },
    { 0xa8572461, 84 },
    { 0x1f72461, 81 },
    { 0, 0 },
    { 0xb4d72465, 90 },
    { 0xaf272466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56672469, 90 },
    { 0x9027246a, 43 },
    { 0x4d87246a, 91 },
    { 0x6757246c, 33 },
    { 0x10b7246c, 33 },
    { 0x8a87246b, 100 },
    { 0xcd37246f, 23 },
    { 0xeba7246b, 34 },
    { 0, 0 },
    { 0xe1d72472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b972476, 46 },
    { 0, 0 },
    { 0xc72478, 61 },
    { 0xa5f72479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6667247d, 99 },
    { 0x4767247e, 78 },
    { 0xcb47247d, 95 },
    { 0x69472480, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe72484, 43 },
    { 0, 0 },
    { 0xede72486, 23 },
    { 0xb6572487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3697248d, 43 },
    { 0x3ad7248e, 90 },
    { 0xb157248d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b72493, 61 },
    { 0, 0 },
    { 0x92272495, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a7249c, 98 },
    { 0xda77249c, 90 },
    { 0xa867249d, 34 },
    { 0xaad7249d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8724a3, 91 },
    { 0x8b8724a4, 98 },
    { 0, 0 },
    { 0x896724a6, 66 },
    { 0x5d0724a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9724af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47724b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70724b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae724bb, 61 },
    { 0, 0 },
    { 0x2d5724bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f724c0, 65 },
    { 0x28d724c1, 24 },
    { 0x957724c1, 33 },
    { 0xcdb724c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0724c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5724cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d724d6, 46 },
    { 0xc31724d7, 90 },
    { 0x4b7724d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4724dc, 90 },
    { 0xa36724dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a724e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34724ec, 90 },
    { 0x9d8724ed, 34 },
    { 0x940724ee, 23 },
    { 0, 0 },
    { 0x5a6724f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9724f9, 95 },
    { 0x6bd724fa, 75 },
    { 0xab0724fb, 46 },
    { 0x6c9724fb, 33 },
    { 0x218724fd, 88 },
    { 0xbac724fe, 91 },
    { 0xf86724ff, 43 },
    { 0x1cb724fa, 33 },
    { 0xeb872501, 81 },
    { 0xf5772502, 34 },
    { 0x4af72501, 95 },
    { 0x1ad72504, 75 },
    { 0xf8172505, 61 },
    { 0x7d272505, 61 },
    { 0x54c72507, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427250a, 100 },
    { 0x727250a, 67 },
    { 0, 0 },
    { 0xc617250d, 95 },
    { 0, 0 },
    { 0x8c97250f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8972512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a872516, 98 },
    { 0, 0 },
    { 0x1db72518, 90 },
    { 0x35772519, 76 },
    { 0xda7251a, 23 },
    { 0xbd972518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977251f, 46 },
    { 0x2772520, 23 },
    { 0x37372520, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef72524, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472527, 98 },
    { 0, 0 },
    { 0xb6d72529, 76 },
    { 0x7787252a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe047252d, 95 },
    { 0x9907252e, 76 },
    { 0x6767252f, 55 },
    { 0x8f47252e, 67 },
    { 0x7e372530, 23 },
    { 0xa172530, 33 },
    { 0xc7472532, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a7253a, 4 },
    { 0xd5e7253b, 100 },
    { 0x87d7253c, 55 },
    { 0x9ba7253b, 33 },
    { 0xe057253d, 61 },
    { 0x38c7253f, 77 },
    { 0x3187253b, 61 },
    { 0, 0 },
    { 0xf3e72542, 76 },
    { 0x15c72542, 95 },
    { 0x43372542, 33 },
    { 0xc5e72542, 93 },
    { 0x25e72546, 17 },
    { 0x6a072547, 99 },
    { 0xa4c72546, 61 },
    { 0x20872549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7254c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9972554, 55 },
    { 0, 0 },
    { 0x6f172556, 66 },
    { 0xbad72556, 90 },
    { 0x2b572558, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d772565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe372569, 76 },
    { 0, 0 },
    { 0x8937256b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c67256e, 23 },
    { 0, 0 },
    { 0x93272570, 95 },
    { 0x95c72570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb872575, 33 },
    { 0, 0 },
    { 0x85272577, 78 },
    { 0, 0 },
    { 0xb2b72579, 78 },
    { 0xf167257a, 90 },
    { 0x51872579, 61 },
    { 0, 0 },
    { 0xfb07257d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe72583, 90 },
    { 0x4fe72583, 23 },
    { 0x31772583, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b472588, 90 },
    { 0, 0 },
    { 0xddd7258a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87258e, 90 },
    { 0x6f87258f, 98 },
    { 0x9e7258f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e72597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107259b, 61 },
    { 0x7297259c, 90 },
    { 0x9fb7259d, 76 },
    { 0x6af7259c, 99 },
    { 0xe887259e, 66 },
    { 0x26e7259c, 90 },
    { 0xcaf7259e, 43 },
    { 0xc5e725a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933725a6, 75 },
    { 0xf02725a7, 78 },
    { 0xc53725a7, 95 },
    { 0x133725a9, 43 },
    { 0xba2725a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe725b5, 34 },
    { 0x4ac725b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f725bb, 98 },
    { 0, 0 },
    { 0x68a725bd, 96 },
    { 0xcef725be, 95 },
    { 0, 0 },
    { 0x44725c0, 90 },
    { 0, 0 },
    { 0x644725c2, 98 },
    { 0xf6d725c2, 90 },
    { 0, 0 },
    { 0x996725c5, 33 },
    { 0xd0d725c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69725cc, 76 },
    { 0xa74725cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x277725d0, 17 },
    { 0x6e7725d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b725d7, 77 },
    { 0x799725d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x269725db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2725e0, 100 },
    { 0x7fe725e1, 98 },
    { 0xc55725e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42725e5, 90 },
    { 0x1e2725e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd725eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270725ee, 87 },
    { 0x6a4725ee, 90 },
    { 0xcd2725f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9725f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d725f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8725fb, 95 },
    { 0x923725fc, 85 },
    { 0xfd4725fd, 67 },
    { 0xca1725fe, 98 },
    { 0x3a5725ff, 55 },
    { 0x38472600, 33 },
    { 0x4f372601, 76 },
    { 0x68672600, 33 },
    { 0x6b172603, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f72606, 33 },
    { 0, 0 },
    { 0x50a72608, 90 },
    { 0x5c472609, 61 },
    { 0x5b57260a, 83 },
    { 0x24072608, 33 },
    { 0xed87260c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7557260f, 52 },
    { 0x34e7260f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x24372613, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8172616, 76 },
    { 0xcc72617, 76 },
    { 0, 0 },
    { 0x87a72619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7637261c, 23 },
    { 0x7ed7261d, 100 },
    { 0x8407261d, 98 },
    { 0xcfb7261f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2472623, 90 },
    { 0x82572624, 91 },
    { 0x63472624, 98 },
    { 0xb6072623, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4872629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67262f, 52 },
    { 0xf2d7262f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2567263d, 33 },
    { 0x2387263d, 75 },
    { 0, 0 },
    { 0xedf72640, 85 },
    { 0xc8072641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c72647, 23 },
    { 0, 0 },
    { 0xd7d72649, 23 },
    { 0x1137264a, 99 },
    { 0xadc72649, 99 },
    { 0x22a7264c, 23 },
    { 0x4827264d, 77 },
    { 0x3a372649, 95 },
    { 0x53d7264d, 61 },
    { 0, 0 },
    { 0xff72651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84872659, 61 },
    { 0x3c172659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a72663, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b72668, 95 },
    { 0xa7772669, 95 },
    { 0xfb472668, 33 },
    { 0, 0 },
    { 0x4207266c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd017266f, 23 },
    { 0xb8572670, 90 },
    { 0x84f72671, 91 },
    { 0x2a72670, 43 },
    { 0xd1672670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e672676, 91 },
    { 0, 0 },
    { 0x4ce72678, 34 },
    { 0xa9372679, 55 },
    { 0, 0 },
    { 0x1d67267b, 23 },
    { 0xe177267c, 61 },
    { 0xc407267b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb72680, 64 },
    { 0x92072681, 95 },
    { 0, 0 },
    { 0xd7472683, 61 },
    { 0x87872684, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa7268c, 33 },
    { 0, 0 },
    { 0x9ed7268e, 100 },
    { 0xaf47268e, 69 },
    { 0x1ca7268e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7269a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257269f, 95 },
    { 0xd04726a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x355726a3, 23 },
    { 0xc726a4, 100 },
    { 0xaa9726a5, 95 },
    { 0xfc7726a6, 81 },
    { 0xd5f726a5, 61 },
    { 0x61f726a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed726ab, 33 },
    { 0xd11726ab, 76 },
    { 0x2e0726ab, 90 },
    { 0x8ba726ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20726b5, 23 },
    { 0, 0 },
    { 0xd2d726b7, 76 },
    { 0x554726b8, 61 },
    { 0xa2726b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688726c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d726c4, 98 },
    { 0x802726c5, 99 },
    { 0x29d726c5, 61 },
    { 0x5fb726c6, 33 },
    { 0xb9f726c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3726d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81726d7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x573726da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c726dd, 46 },
    { 0, 0 },
    { 0x194726df, 33 },
    { 0x5d2726df, 90 },
    { 0, 0 },
    { 0x197726e2, 23 },
    { 0xdb0726e2, 33 },
    { 0x4cf726e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01726f3, 75 },
    { 0xba3726f4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438726f8, 95 },
    { 0, 0 },
    { 0x4ee726fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590726ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26972703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8172708, 43 },
    { 0, 0 },
    { 0xd097270a, 33 },
    { 0xd507270a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d72711, 23 },
    { 0x66472712, 78 },
    { 0x7c672713, 90 },
    { 0x72f72712, 98 },
    { 0xb9772714, 85 },
    { 0x34872716, 23 },
    { 0x62072717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5247271e, 84 },
    { 0xc627271f, 23 },
    { 0xcdc72720, 95 },
    { 0x6ec72721, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd72726, 23 },
    { 0xa1d72727, 23 },
    { 0, 0 },
    { 0x9eb72729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf837272c, 33 },
    { 0x5837272d, 77 },
    { 0xf507272c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41972736, 23 },
    { 0, 0 },
    { 0x2dc72738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577273c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c72746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657274a, 90 },
    { 0x6f47274b, 33 },
    { 0x7ad7274c, 99 },
    { 0, 0 },
    { 0x8ad7274e, 55 },
    { 0x5597274f, 88 },
    { 0x7cc72750, 98 },
    { 0xf3772751, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a72755, 98 },
    { 0, 0 },
    { 0x58072757, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767275c, 90 },
    { 0xd277275c, 33 },
    { 0, 0 },
    { 0xfe67275f, 98 },
    { 0, 0 },
    { 0xa272761, 23 },
    { 0xe8b72761, 78 },
    { 0x30972763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6572766, 95 },
    { 0, 0 },
    { 0x22e72768, 91 },
    { 0, 0 },
    { 0x70b7276a, 98 },
    { 0x8a37276b, 33 },
    { 0x66b7276c, 33 },
    { 0x68a7276d, 90 },
    { 0x39d7276d, 98 },
    { 0x20e7276d, 95 },
    { 0x41372770, 43 },
    { 0x46a7276e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed872778, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de7277c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4777277f, 33 },
    { 0x5d572780, 49 },
    { 0, 0 },
    { 0x2a672782, 90 },
    { 0xf9b72783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27972786, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b72789, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc187278c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2772791, 46 },
    { 0x8b372792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7279a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47279f, 23 },
    { 0, 0 },
    { 0xff1727a1, 17 },
    { 0, 0 },
    { 0x93b727a3, 34 },
    { 0x2b1727a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d727a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619727ab, 33 },
    { 0, 0 },
    { 0x4a5727ad, 61 },
    { 0x680727ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e727b1, 77 },
    { 0xf1727b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d727b9, 77 },
    { 0, 0 },
    { 0x142727bb, 87 },
    { 0x87c727bc, 66 },
    { 0, 0 },
    { 0x9aa727be, 84 },
    { 0x7c1727bf, 19 },
    { 0x56727bf, 76 },
    { 0x297727c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7727c5, 33 },
    { 0xe50727c6, 98 },
    { 0xb44727c6, 90 },
    { 0xd2727c6, 61 },
    { 0xb3a727c9, 76 },
    { 0x6bf727c5, 34 },
    { 0, 0 },
    { 0xfd4727cc, 34 },
    { 0xa7f727cd, 33 },
    { 0, 0 },
    { 0x6fb727cf, 61 },
    { 0xf81727cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e727d3, 95 },
    { 0x330727d4, 90 },
    { 0, 0 },
    { 0xb7e727d6, 90 },
    { 0x7fc727d7, 90 },
    { 0x33f727d6, 61 },
    { 0xe26727d6, 23 },
    { 0, 0 },
    { 0x975727db, 93 },
    { 0x388727dc, 76 },
    { 0xf0c727db, 53 },
    { 0xfd7727de, 43 },
    { 0x797727df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a727e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e727e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750727ed, 33 },
    { 0xca9727ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0727fc, 23 },
    { 0xaa5727fd, 33 },
    { 0x2de727fc, 85 },
    { 0x266727ff, 83 },
    { 0x85e727ff, 100 },
    { 0x4a8727ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73872804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f72808, 95 },
    { 0, 0 },
    { 0xbd07280a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7472818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6547281e, 68 },
    { 0xb8a7281e, 95 },
    { 0x13b7281e, 61 },
    { 0x94472821, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1372824, 24 },
    { 0x76872825, 33 },
    { 0x60672825, 88 },
    { 0xaee72827, 23 },
    { 0x30a72828, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6427282d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72836, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a7283b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69572840, 81 },
    { 0, 0 },
    { 0x7ca72842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56972848, 93 },
    { 0x25a72849, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e72850, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec872855, 40 },
    { 0xd4372856, 23 },
    { 0xd1f72856, 93 },
    { 0xc2072856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd737285b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26272860, 95 },
    { 0x30c72860, 76 },
    { 0xc5872862, 55 },
    { 0x57d72862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de7286a, 90 },
    { 0, 0 },
    { 0xaee7286c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2272872, 76 },
    { 0, 0 },
    { 0xd7172874, 90 },
    { 0, 0 },
    { 0x30172876, 23 },
    { 0xac772877, 84 },
    { 0x86772876, 61 },
    { 0, 0 },
    { 0xcd77287a, 74 },
    { 0x9187287b, 61 },
    { 0, 0 },
    { 0x3057287d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19572885, 100 },
    { 0x15672886, 67 },
    { 0, 0 },
    { 0x4a272888, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7288b, 61 },
    { 0xbaa7288b, 34 },
    { 0xb07288b, 90 },
    { 0x1b07288c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66372893, 95 },
    { 0, 0 },
    { 0x98172895, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba72899, 98 },
    { 0xc567289a, 33 },
    { 0x1ab7289b, 90 },
    { 0x50e7289c, 95 },
    { 0xf1a7289b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc728a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3728a5, 66 },
    { 0x1fc728a5, 98 },
    { 0x6bb728a5, 90 },
    { 0x2c9728a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728ad, 55 },
    { 0xd728ad, 46 },
    { 0, 0 },
    { 0xae2728b0, 90 },
    { 0, 0 },
    { 0xcf8728b2, 80 },
    { 0, 0 },
    { 0x728728b4, 33 },
    { 0xa91728b4, 90 },
    { 0x46a728b5, 55 },
    { 0x36d728b7, 61 },
    { 0xe08728b6, 93 },
    { 0x1dc728b8, 84 },
    { 0xed1728ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce728bd, 90 },
    { 0x9e6728be, 95 },
    { 0x37f728bf, 61 },
    { 0x211728c0, 61 },
    { 0xb21728bf, 34 },
    { 0xe16728c2, 90 },
    { 0x371728c1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245728c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738728ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b728d4, 55 },
    { 0, 0 },
    { 0x4b4728d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb728db, 90 },
    { 0xc79728db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x94728df, 84 },
    { 0xcbc728df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a728e5, 46 },
    { 0, 0 },
    { 0x1bf728e7, 32 },
    { 0x645728e8, 34 },
    { 0x220728e8, 91 },
    { 0xf97728e8, 87 },
    { 0x59c728e8, 98 },
    { 0xeba728ec, 65 },
    { 0x666728ed, 98 },
    { 0x2ff728ee, 93 },
    { 0x651728ef, 85 },
    { 0x7aa728ed, 46 },
    { 0xfa728e7, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450728f5, 88 },
    { 0x860728f5, 98 },
    { 0xffa728f7, 61 },
    { 0x64a728f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26728fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82372900, 98 },
    { 0x67b72901, 76 },
    { 0x16072901, 43 },
    { 0, 0 },
    { 0xbaf72904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4447290a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da7290d, 43 },
    { 0xbf47290e, 33 },
    { 0, 0 },
    { 0xb2172910, 90 },
    { 0, 0 },
    { 0xf9772912, 90 },
    { 0xf4b72913, 33 },
    { 0xc9d72913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c372918, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f7291e, 61 },
    { 0, 0 },
    { 0xfde72920, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b72926, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c7292f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5572932, 98 },
    { 0x2aa72932, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927293a, 46 },
    { 0x37f7293b, 78 },
    { 0xa847293a, 95 },
    { 0xb37293d, 90 },
    { 0x63e7293b, 34 },
    { 0x8b67293f, 99 },
    { 0, 0 },
    { 0x90872941, 34 },
    { 0x30672942, 98 },
    { 0xd1a72941, 90 },
    { 0, 0 },
    { 0x1a472945, 61 },
    { 0xaad72946, 90 },
    { 0, 0 },
    { 0xe5372948, 46 },
    { 0xf9d72949, 55 },
    { 0x3827294a, 33 },
    { 0xc937294b, 90 },
    { 0x73c7294a, 87 },
    { 0x6497294d, 52 },
    { 0x15d7294d, 33 },
    { 0xc2972948, 95 },
    { 0x81272950, 95 },
    { 0x7957294b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa72956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9472959, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217295c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c72969, 76 },
    { 0xe4b7296a, 46 },
    { 0, 0 },
    { 0x1017296c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1772976, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2377297d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a72992, 23 },
    { 0, 0 },
    { 0x73972994, 76 },
    { 0x5a472995, 46 },
    { 0, 0 },
    { 0xb7b72997, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b7299d, 46 },
    { 0, 0 },
    { 0xce07299f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7729a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c729a9, 23 },
    { 0x9c9729aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9729b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff729b7, 93 },
    { 0, 0 },
    { 0x819729b9, 84 },
    { 0x431729ba, 33 },
    { 0x89c729bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328729c5, 99 },
    { 0x737729c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b729c9, 61 },
    { 0xcc2729ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5729d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc729da, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab729e2, 33 },
    { 0x6af729e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88729e8, 84 },
    { 0x152729e9, 33 },
    { 0xf57729e9, 61 },
    { 0, 0 },
    { 0x2b4729ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x674729ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a729f2, 95 },
    { 0, 0 },
    { 0xcba729f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93729f8, 95 },
    { 0x1d9729f9, 61 },
    { 0x68e729f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce729fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d72a00, 33 },
    { 0xd4572a00, 67 },
    { 0x44d72a02, 33 },
    { 0x3d072a03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c72a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91572a09, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e72a0c, 98 },
    { 0x77872a0d, 34 },
    { 0x32472a0e, 33 },
    { 0x24b72a0c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f72a16, 66 },
    { 0x44d72a17, 61 },
    { 0xd7e72a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef472a1b, 23 },
    { 0xce872a1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x72572a1f, 90 },
    { 0xd9372a1f, 55 },
    { 0xf5572a21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32272a24, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0172a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0872a2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c572a2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1472a32, 98 },
    { 0xba572a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d72a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48172a3a, 90 },
    { 0x7a72a3a, 98 },
    { 0xa7972a3a, 76 },
    { 0xf3a72a3d, 90 },
    { 0xf1f72a3e, 95 },
    { 0x41d72a3b, 93 },
    { 0xc0a72a3e, 55 },
    { 0, 0 },
    { 0x61572a42, 61 },
    { 0xdf572a42, 17 },
    { 0x81f72a44, 95 },
    { 0, 0 },
    { 0x38672a46, 69 },
    { 0, 0 },
    { 0xcb972a48, 36 },
    { 0, 0 },
    { 0xa1372a4a, 66 },
    { 0x17472a4a, 87 },
    { 0x6a972a4c, 46 },
    { 0x1cb72a4d, 52 },
    { 0, 0 },
    { 0x74972a4f, 12 },
    { 0xf9072a50, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e272a56, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x47172a59, 17 },
    { 0xe9372a5a, 90 },
    { 0x26672a59, 55 },
    { 0xc4f72a5b, 93 },
    { 0x3a272a5d, 55 },
    { 0xdeb72a5e, 95 },
    { 0x32372a5c, 98 },
    { 0x1c72a60, 61 },
    { 0, 0 },
    { 0x30e72a62, 95 },
    { 0xfc072a63, 76 },
    { 0xb6972a64, 33 },
    { 0x9a472a63, 43 },
    { 0, 0 },
    { 0x2972a67, 90 },
    { 0x5d372a68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d72a6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36872a6f, 91 },
    { 0xe5272a70, 90 },
    { 0, 0 },
    { 0xbbb72a72, 33 },
    { 0xb172a72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff572a76, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2172a7a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4072a81, 100 },
    { 0xeec72a82, 100 },
    { 0, 0 },
    { 0x64e72a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf372a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea72a91, 98 },
    { 0, 0 },
    { 0x7b072a93, 90 },
    { 0xd4f72a94, 78 },
    { 0xbb572a94, 90 },
    { 0xb6572a96, 66 },
    { 0x86272a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972a9a, 77 },
    { 0, 0 },
    { 0x70472a9c, 67 },
    { 0xb4772a9d, 91 },
    { 0xc5a72a9e, 61 },
    { 0x28172a9f, 98 },
    { 0x872a9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e72aa3, 99 },
    { 0x14472aa3, 90 },
    { 0x7f972aa5, 61 },
    { 0, 0 },
    { 0x7d672aa7, 52 },
    { 0x38972aa8, 40 },
    { 0x47972aa7, 75 },
    { 0xa8572aaa, 90 },
    { 0x28072aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1772ab8, 90 },
    { 0x3c272ab9, 33 },
    { 0xba372aba, 95 },
    { 0x99872aba, 95 },
    { 0x7ab72ab8, 95 },
    { 0x24f72aba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84272ac3, 23 },
    { 0xac972ac4, 87 },
    { 0xf4872ac4, 34 },
    { 0xef772ac5, 93 },
    { 0x45672ac7, 23 },
    { 0, 0 },
    { 0xe672ac9, 69 },
    { 0x16372aca, 98 },
    { 0x4c572acb, 78 },
    { 0x19072acc, 23 },
    { 0xb8172ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8872ad1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f72ad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c72adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d572adf, 23 },
    { 0, 0 },
    { 0xa8672ae1, 55 },
    { 0xb9072ae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda72ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1272aea, 43 },
    { 0x3d072aeb, 67 },
    { 0xf9f72aec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2472aef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83072af3, 91 },
    { 0x3d472af4, 90 },
    { 0xb2c72af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa72af9, 55 },
    { 0x89372afa, 100 },
    { 0x33172af9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91572aff, 94 },
    { 0, 0 },
    { 0x67f72b01, 23 },
    { 0x97d72b02, 61 },
    { 0, 0 },
    { 0x67172b04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc72b09, 61 },
    { 0xab972b0a, 66 },
    { 0xbe072b0b, 61 },
    { 0, 0 },
    { 0xb9372b0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772b11, 33 },
    { 0x3be72b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f72b15, 98 },
    { 0x39e72b16, 23 },
    { 0x6d672b17, 23 },
    { 0xf3b72b18, 98 },
    { 0xe2972b19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c72b1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92172b26, 33 },
    { 0x66b72b27, 68 },
    { 0xf8b72b28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5072b2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91872b30, 34 },
    { 0x74672b30, 90 },
    { 0x74772b32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc672b38, 23 },
    { 0x4aa72b39, 23 },
    { 0x94972b3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce072b3e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2772b41, 98 },
    { 0x84f72b42, 90 },
    { 0xc2a72b42, 34 },
    { 0x38472b44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d272b49, 95 },
    { 0x30472b4a, 23 },
    { 0, 0 },
    { 0x35c72b4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97072b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98372b57, 96 },
    { 0x62472b57, 43 },
    { 0x6c772b59, 76 },
    { 0xdde72b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b72b5d, 93 },
    { 0xf6472b5e, 56 },
    { 0, 0 },
    { 0x2f772b60, 33 },
    { 0x45272b61, 34 },
    { 0x2f272b62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae72b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e272b6a, 33 },
    { 0, 0 },
    { 0x1fd72b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e72b73, 23 },
    { 0x12c72b74, 23 },
    { 0xa272b75, 101 },
    { 0xa1072b76, 23 },
    { 0x57d72b74, 97 },
    { 0, 0 },
    { 0x76b72b79, 57 },
    { 0x1d272b79, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf372b81, 76 },
    { 0, 0 },
    { 0x9a772b83, 95 },
    { 0x3a572b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb72b87, 33 },
    { 0, 0 },
    { 0xb8a72b89, 34 },
    { 0, 0 },
    { 0x1c372b8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85572b8e, 61 },
    { 0x92a72b8f, 90 },
    { 0, 0 },
    { 0x40872b91, 90 },
    { 0x1f872b91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d72b95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x24172b98, 87 },
    { 0x9a172b99, 23 },
    { 0x4d72b9a, 78 },
    { 0x78a72b9b, 34 },
    { 0, 0 },
    { 0xda272b9d, 76 },
    { 0xd0c72b9d, 75 },
    { 0, 0 },
    { 0x99472ba0, 23 },
    { 0, 0 },
    { 0xfd272ba2, 93 },
    { 0x5cf72ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e72ba8, 61 },
    { 0x21e72ba9, 93 },
    { 0, 0 },
    { 0xaab72bab, 76 },
    { 0, 0 },
    { 0x50d72bad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc72bb4, 33 },
    { 0x21e72bb5, 90 },
    { 0x76c72bb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be72bbc, 93 },
    { 0xd0e72bbd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af72bc5, 61 },
    { 0x54172bc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e872bc9, 90 },
    { 0xfbd72bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b72bce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472bd2, 34 },
    { 0x9d372bd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52472bd7, 34 },
    { 0xb6d72bd8, 77 },
    { 0xc9f72bd9, 90 },
    { 0x7dc72bda, 23 },
    { 0xd2872bd9, 98 },
    { 0x1d872bdb, 61 },
    { 0, 0 },
    { 0x8c72bde, 23 },
    { 0xf5472bdf, 23 },
    { 0xd0e72be0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072be4, 17 },
    { 0, 0 },
    { 0xc7a72be6, 95 },
    { 0xd0872be7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a72bf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5972c04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a872c09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b72c0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67972c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc72c15, 76 },
    { 0x63172c16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa72c1e, 76 },
    { 0x8572c1f, 90 },
    { 0x2f272c1f, 95 },
    { 0x1c872c1e, 90 },
    { 0x87e72c21, 95 },
    { 0x29e72c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3772c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd72c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1272c3b, 100 },
    { 0xbde72c3c, 95 },
    { 0x1a772c3d, 66 },
    { 0x30472c3e, 52 },
    { 0x33472c3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c672c47, 95 },
    { 0, 0 },
    { 0xf9872c49, 61 },
    { 0x16372c4a, 75 },
    { 0xb4b72c49, 55 },
    { 0x29d72c4c, 95 },
    { 0xe4472c4d, 98 },
    { 0x91972c4d, 91 },
    { 0x4f872c4d, 33 },
    { 0xae072c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7472c5a, 98 },
    { 0xd7a72c5a, 87 },
    { 0x8672c5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad572c62, 95 },
    { 0x40472c62, 23 },
    { 0xa7d72c64, 78 },
    { 0x8e972c64, 75 },
    { 0x9b72c63, 90 },
    { 0x9be72c67, 55 },
    { 0xdd872c64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a072c6b, 90 },
    { 0xe8b72c6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c72c70, 98 },
    { 0x12a72c70, 91 },
    { 0x4f72c71, 76 },
    { 0xf3e72c73, 98 },
    { 0, 0 },
    { 0x28e72c75, 90 },
    { 0, 0 },
    { 0xf6572c77, 61 },
    { 0, 0 },
    { 0x72272c79, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f72c7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c72c87, 55 },
    { 0, 0 },
    { 0xdab72c89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c572c8f, 33 },
    { 0x872c8f, 76 },
    { 0xed772c90, 55 },
    { 0, 0 },
    { 0x84672c93, 23 },
    { 0, 0 },
    { 0x4d172c95, 76 },
    { 0, 0 },
    { 0xc5a72c97, 98 },
    { 0xfb572c97, 33 },
    { 0x4bf72c97, 90 },
    { 0x37672c9a, 98 },
    { 0x25472c9b, 61 },
    { 0xac972c9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62772ca0, 61 },
    { 0, 0 },
    { 0x36b72ca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d72ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f572cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96272caf, 90 },
    { 0xe8272cb0, 99 },
    { 0x16172cb0, 55 },
    { 0xb4572cb0, 23 },
    { 0, 0 },
    { 0x3bc72cb4, 61 },
    { 0, 0 },
    { 0x91972cb6, 34 },
    { 0xb2d72cb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8372cbf, 36 },
    { 0x10972cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab72cc3, 98 },
    { 0x5f872cc3, 78 },
    { 0, 0 },
    { 0xd0c72cc6, 90 },
    { 0x45b72cc6, 99 },
    { 0x13e72cc8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72ccc, 90 },
    { 0xcb272ccd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d472cd3, 33 },
    { 0x41d72cd4, 78 },
    { 0, 0 },
    { 0x20f72cd6, 87 },
    { 0, 0 },
    { 0xa172cd8, 98 },
    { 0xfd372cd9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8172cdc, 68 },
    { 0, 0 },
    { 0xc8972cde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b72ce1, 61 },
    { 0, 0 },
    { 0x39d72ce3, 46 },
    { 0xce072ce3, 67 },
    { 0x45f72ce3, 61 },
    { 0x10e72ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44972ce9, 77 },
    { 0x7e572ce9, 90 },
    { 0xafc72ce9, 61 },
    { 0x5572ce9, 75 },
    { 0x30572ced, 23 },
    { 0xce772cee, 90 },
    { 0x59a72cef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94072cfd, 87 },
    { 0xfaa72cfe, 61 },
    { 0x7a872cfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a72d03, 23 },
    { 0x9c772d03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea572d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c72d0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb72d12, 95 },
    { 0xe1672d12, 33 },
    { 0xfe272d14, 95 },
    { 0, 0 },
    { 0x1072d16, 90 },
    { 0, 0 },
    { 0x50272d18, 46 },
    { 0, 0 },
    { 0x4a472d1a, 65 },
    { 0, 0 },
    { 0xa4172d1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d72d1f, 61 },
    { 0, 0 },
    { 0xee872d21, 85 },
    { 0, 0 },
    { 0x23972d23, 90 },
    { 0, 0 },
    { 0xc0472d25, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a72d30, 95 },
    { 0xbea72d31, 95 },
    { 0, 0 },
    { 0x7ca72d33, 93 },
    { 0xf272d34, 90 },
    { 0x4aa72d33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92972d38, 76 },
    { 0x17c72d39, 95 },
    { 0x8fd72d39, 78 },
    { 0x78972d3b, 98 },
    { 0x1fa72d3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372d41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e972d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df72d4f, 90 },
    { 0x68f72d50, 55 },
    { 0xef972d51, 68 },
    { 0xb5d72d4f, 69 },
    { 0x7d72d53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4672d59, 98 },
    { 0, 0 },
    { 0x68272d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d972d5e, 85 },
    { 0, 0 },
    { 0x2e772d60, 55 },
    { 0x41272d61, 61 },
    { 0x3bd72d61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb72d66, 46 },
    { 0x3fa72d67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72d6c, 61 },
    { 0x81b72d6d, 23 },
    { 0x49372d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53772d72, 95 },
    { 0, 0 },
    { 0xfd772d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f72d77, 90 },
    { 0x90172d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca772d7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72672d87, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6372d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a72d8e, 25 },
    { 0xbb272d8e, 33 },
    { 0, 0 },
    { 0x5ed72d91, 61 },
    { 0x90a72d92, 66 },
    { 0x5b972d92, 23 },
    { 0xf3072d94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e972d98, 91 },
    { 0, 0 },
    { 0x6e672d9a, 61 },
    { 0x50972d9b, 91 },
    { 0x67372d9a, 98 },
    { 0, 0 },
    { 0x8ba72d9e, 61 },
    { 0xdaa72d9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15972da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61372da8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac72dac, 76 },
    { 0x7ab72dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f72db3, 75 },
    { 0x3b72db4, 43 },
    { 0xc1d72db5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea72dba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc72dc2, 95 },
    { 0xe0f72dc3, 55 },
    { 0x2fe72dc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60972dc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a72dd1, 34 },
    { 0x2e672dd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5172dd5, 33 },
    { 0, 0 },
    { 0x7ec72dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4472ddc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93772de2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad072de6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d72dea, 75 },
    { 0xd5d72deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1072dee, 98 },
    { 0xf2572dee, 84 },
    { 0x3af72def, 33 },
    { 0, 0 },
    { 0xee472df2, 80 },
    { 0xec772df2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b72df8, 87 },
    { 0xcb172df9, 33 },
    { 0x2f572dfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd72e05, 85 },
    { 0, 0 },
    { 0x63572e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f172e0a, 95 },
    { 0, 0 },
    { 0x3f472e0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc72e10, 52 },
    { 0xcf172e10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1372e15, 98 },
    { 0x2d672e15, 95 },
    { 0xef272e17, 61 },
    { 0x2bb72e17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0872e1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8372e26, 34 },
    { 0, 0 },
    { 0x6a472e28, 95 },
    { 0, 0 },
    { 0xb2772e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef72e30, 73 },
    { 0, 0 },
    { 0x3e372e32, 75 },
    { 0, 0 },
    { 0xe8672e34, 43 },
    { 0x71972e34, 91 },
    { 0, 0 },
    { 0x57472e37, 76 },
    { 0x88272e38, 77 },
    { 0xd8672e38, 99 },
    { 0x5d872e38, 87 },
    { 0, 0 },
    { 0x12272e3c, 85 },
    { 0, 0 },
    { 0xefd72e3e, 23 },
    { 0x6de72e3f, 61 },
    { 0, 0 },
    { 0xd2e72e41, 98 },
    { 0x7a672e42, 95 },
    { 0x8f772e42, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca72e47, 88 },
    { 0xf0a72e48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56872e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3672e55, 66 },
    { 0x7e72e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc172e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d872e66, 95 },
    { 0xbf172e67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44372e6b, 61 },
    { 0x58072e6c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca672e71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d72e76, 95 },
    { 0, 0 },
    { 0x1c272e78, 95 },
    { 0x1f372e78, 99 },
    { 0xa9672e78, 95 },
    { 0xff172e7b, 43 },
    { 0, 0 },
    { 0x65072e7d, 98 },
    { 0x50172e7d, 85 },
    { 0xe8772e7d, 99 },
    { 0x3d172e80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87372e84, 61 },
    { 0xcf972e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e972e88, 23 },
    { 0x54472e89, 78 },
    { 0xd1272e8a, 40 },
    { 0x3f372e8b, 33 },
    { 0xbb072e8a, 95 },
    { 0x5f972e8d, 95 },
    { 0xb3872e8e, 76 },
    { 0xf0072e8f, 33 },
    { 0xb8372e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec772e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48172e97, 61 },
    { 0xdb172e98, 90 },
    { 0xb4d72e99, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec72ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a672ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89472eaa, 98 },
    { 0x3f72eab, 55 },
    { 0, 0 },
    { 0xcdc72ead, 90 },
    { 0xd2072ead, 33 },
    { 0x67872eaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66072eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87572eb6, 23 },
    { 0xe6972eb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93072ebc, 33 },
    { 0xe6672ebd, 23 },
    { 0x9a672ebc, 33 },
    { 0xa3672ebf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a72ec3, 34 },
    { 0x6a672ec3, 33 },
    { 0, 0 },
    { 0xd2572ec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a72ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a72ecd, 52 },
    { 0xca972ece, 90 },
    { 0x5972ece, 33 },
    { 0x3772ed0, 77 },
    { 0x8872ed1, 98 },
    { 0x9072ecf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd672ed5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b572ee0, 76 },
    { 0x872ee0, 43 },
    { 0x34872ee2, 90 },
    { 0x3d872ee3, 34 },
    { 0x79772ee1, 93 },
    { 0x87472ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2572eea, 76 },
    { 0x8e72eea, 76 },
    { 0xc5172eea, 98 },
    { 0xf2372eed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c72ef1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472ef8, 90 },
    { 0x77172ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d572efc, 98 },
    { 0xa5072efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58972f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda772f0e, 95 },
    { 0x8272f0f, 38 },
    { 0x2f472f10, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced72f15, 98 },
    { 0x45572f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf172f1b, 34 },
    { 0x52f72f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51572f23, 12 },
    { 0x2e272f24, 33 },
    { 0, 0 },
    { 0x48872f26, 66 },
    { 0xf4b72f26, 33 },
    { 0xa2d72f28, 61 },
    { 0xc2272f28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6872f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a72f34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db72f42, 52 },
    { 0x5cf72f42, 34 },
    { 0, 0 },
    { 0xac672f45, 33 },
    { 0x6cf72f45, 33 },
    { 0xd8072f46, 61 },
    { 0x61372f48, 100 },
    { 0xf7272f49, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2872f51, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5772f55, 66 },
    { 0x2e072f56, 76 },
    { 0xd8572f57, 67 },
    { 0x10d72f55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10872f5b, 76 },
    { 0, 0 },
    { 0x74c72f5d, 23 },
    { 0, 0 },
    { 0xe2072f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c72f62, 43 },
    { 0x92372f63, 93 },
    { 0x4bf72f63, 95 },
    { 0x63072f63, 99 },
    { 0xa7872f66, 61 },
    { 0xc9772f66, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68872f6d, 23 },
    { 0x16d72f6e, 61 },
    { 0x41f72f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc72f72, 93 },
    { 0, 0 },
    { 0xdf872f74, 56 },
    { 0x48a72f75, 55 },
    { 0, 0 },
    { 0x44a72f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10472f7d, 61 },
    { 0, 0 },
    { 0xbad72f7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35372f83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e72f86, 43 },
    { 0xd3072f87, 67 },
    { 0xe9672f86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa72f8d, 93 },
    { 0xabd72f8e, 33 },
    { 0xc8b72f8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a072f94, 23 },
    { 0, 0 },
    { 0x59172f96, 85 },
    { 0, 0 },
    { 0x4b572f98, 95 },
    { 0xba372f99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80672f9e, 52 },
    { 0x15472f9f, 33 },
    { 0xef472fa0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1072fa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f172faa, 90 },
    { 0x94272faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34272fae, 34 },
    { 0x1d872faf, 90 },
    { 0, 0 },
    { 0xf6472fb1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x31572fb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d72fb8, 61 },
    { 0x18172fb9, 38 },
    { 0x7fb72fba, 98 },
    { 0x9772fbb, 46 },
    { 0x79d72fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f72fbf, 95 },
    { 0xe2e72fc0, 55 },
    { 0x2a672fc0, 93 },
    { 0, 0 },
    { 0x95472fc3, 98 },
    { 0, 0 },
    { 0xc7972fc5, 97 },
    { 0xb1772fc5, 46 },
    { 0x44d72fc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb72fca, 46 },
    { 0xa1372fcb, 33 },
    { 0, 0 },
    { 0x96e72fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c872fd7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e772fdd, 85 },
    { 0, 0 },
    { 0x34672fdf, 61 },
    { 0, 0 },
    { 0x60372fe1, 98 },
    { 0xe0e72fe2, 91 },
    { 0x16c72fe1, 61 },
    { 0x1b672fe4, 99 },
    { 0xfd872fe5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46972fe8, 61 },
    { 0, 0 },
    { 0x6f572fea, 68 },
    { 0, 0 },
    { 0xbb772fec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d772ff7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb972ffd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f73003, 66 },
    { 0x63573004, 55 },
    { 0x5c73005, 76 },
    { 0x38473005, 95 },
    { 0xf3473007, 98 },
    { 0, 0 },
    { 0x3b973009, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce173011, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2387301d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75673024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36173028, 90 },
    { 0, 0 },
    { 0x2e27302a, 43 },
    { 0, 0 },
    { 0x1f67302c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4273031, 23 },
    { 0x12573032, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83373036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7303c, 46 },
    { 0x91b7303d, 100 },
    { 0xe627303e, 76 },
    { 0, 0 },
    { 0x41473040, 98 },
    { 0xd0473041, 46 },
    { 0xa8f73040, 23 },
    { 0x80673041, 90 },
    { 0x5c573044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07304f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073052, 61 },
    { 0x30873053, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40473058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d7305f, 95 },
    { 0x4b673060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d673063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6367306f, 68 },
    { 0x4773070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c873074, 61 },
    { 0xa173075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f873078, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3247307c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7307f, 98 },
    { 0, 0 },
    { 0x7ba73081, 98 },
    { 0xbd673081, 38 },
    { 0xadd73081, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7308a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a7308d, 90 },
    { 0x5077308e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d73091, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae73094, 98 },
    { 0x33a73095, 95 },
    { 0xb0273095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7309b, 33 },
    { 0x6a47309c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf730a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0730a6, 81 },
    { 0, 0 },
    { 0x6c5730a8, 98 },
    { 0x9db730a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x843730ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6730b2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395730b6, 87 },
    { 0, 0 },
    { 0xeb1730b8, 76 },
    { 0x99730b9, 98 },
    { 0xe2b730b8, 65 },
    { 0xe13730bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f730be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b730c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d730c7, 90 },
    { 0xe5c730c7, 84 },
    { 0x6ec730c9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39730cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97730d1, 90 },
    { 0xec3730d1, 87 },
    { 0x58e730d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775730d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3730dc, 95 },
    { 0x7f3730dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65730e0, 95 },
    { 0, 0 },
    { 0xa42730e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364730e6, 85 },
    { 0, 0 },
    { 0x6c9730e8, 95 },
    { 0x933730e8, 61 },
    { 0x3db730e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e730ed, 96 },
    { 0xc98730ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5730f6, 95 },
    { 0, 0 },
    { 0xfe1730f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3730fb, 65 },
    { 0xdfd730fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f73102, 52 },
    { 0x9fc73102, 61 },
    { 0, 0 },
    { 0x1c573105, 23 },
    { 0x1a073106, 66 },
    { 0, 0 },
    { 0x2f673108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17310b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe47310e, 75 },
    { 0x5667310f, 90 },
    { 0, 0 },
    { 0x1b573111, 33 },
    { 0, 0 },
    { 0xfed73113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4273116, 76 },
    { 0x2a773117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e7311c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d37311f, 91 },
    { 0xb2873120, 90 },
    { 0x1d773120, 93 },
    { 0xe0773120, 34 },
    { 0x2d273123, 33 },
    { 0x3d573124, 90 },
    { 0, 0 },
    { 0xfbc73126, 43 },
    { 0xdb973127, 78 },
    { 0xe7e73128, 67 },
    { 0xfd373128, 76 },
    { 0x7167312a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7573130, 90 },
    { 0, 0 },
    { 0xcf73132, 43 },
    { 0xa4c73133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54873137, 38 },
    { 0x13d73137, 99 },
    { 0xa8f73139, 61 },
    { 0x7477313a, 17 },
    { 0xa3c73137, 43 },
    { 0x69773139, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3287313f, 78 },
    { 0x4e77313f, 76 },
    { 0x97773141, 33 },
    { 0xe5173142, 61 },
    { 0, 0 },
    { 0x78d73144, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e473148, 95 },
    { 0x31073149, 33 },
    { 0x4237314a, 61 },
    { 0x8ff7314a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x117314e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a73152, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c973158, 23 },
    { 0x53f73159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f7315d, 77 },
    { 0x2507315e, 90 },
    { 0xfe27315f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab473166, 95 },
    { 0x1b973167, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857316c, 36 },
    { 0, 0 },
    { 0x4517316e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda573174, 76 },
    { 0xbd473174, 87 },
    { 0x4cf73176, 90 },
    { 0, 0 },
    { 0x29d73178, 61 },
    { 0, 0 },
    { 0xb717317a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb57317d, 78 },
    { 0xd137317d, 98 },
    { 0x8b77317f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b73186, 90 },
    { 0x6e73187, 56 },
    { 0xbe273187, 55 },
    { 0xfab73189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c7318e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc873191, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce73196, 76 },
    { 0x6ab73197, 8 },
    { 0, 0 },
    { 0x7b173199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a7319c, 67 },
    { 0xec97319d, 61 },
    { 0, 0 },
    { 0x65b7319f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21731a2, 33 },
    { 0, 0 },
    { 0x442731a4, 93 },
    { 0x7fd731a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4731a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904731ad, 76 },
    { 0, 0 },
    { 0xef1731af, 84 },
    { 0, 0 },
    { 0x546731b1, 90 },
    { 0x92c731b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13731b9, 76 },
    { 0, 0 },
    { 0x90a731bb, 17 },
    { 0, 0 },
    { 0xadc731bd, 84 },
    { 0, 0 },
    { 0x9f2731bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8731c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff731d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2731d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x807731db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c731e3, 43 },
    { 0x20731e4, 43 },
    { 0xb21731e5, 55 },
    { 0xade731e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1731e9, 55 },
    { 0, 0 },
    { 0xfd0731eb, 90 },
    { 0, 0 },
    { 0xd73731ed, 52 },
    { 0xf94731ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea731f1, 95 },
    { 0, 0 },
    { 0x507731f3, 76 },
    { 0xafb731f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835731f8, 33 },
    { 0, 0 },
    { 0xd4d731fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23731fe, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d273205, 90 },
    { 0xe3b73206, 33 },
    { 0, 0 },
    { 0xa5f73208, 90 },
    { 0, 0 },
    { 0x9407320a, 61 },
    { 0xdfb7320b, 98 },
    { 0xbb7320c, 34 },
    { 0, 0 },
    { 0xb527320e, 58 },
    { 0x71d7320f, 93 },
    { 0x1e07320f, 76 },
    { 0xeed73211, 33 },
    { 0xd3173211, 33 },
    { 0x9c573212, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab573218, 61 },
    { 0xa3173219, 93 },
    { 0x45f73219, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17321f, 87 },
    { 0x3657321f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca73227, 100 },
    { 0, 0 },
    { 0x92573229, 55 },
    { 0x9b673229, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17322d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d73238, 61 },
    { 0x42173239, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327323d, 17 },
    { 0x11c7323d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53373241, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49373246, 61 },
    { 0x97373246, 33 },
    { 0xb3b73248, 98 },
    { 0x44973249, 84 },
    { 0, 0 },
    { 0x5c67324b, 46 },
    { 0x8887324c, 85 },
    { 0x5857324d, 88 },
    { 0x437324d, 93 },
    { 0xcf87324f, 100 },
    { 0, 0 },
    { 0x4e973251, 34 },
    { 0x24873252, 61 },
    { 0xe5c73253, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc73256, 98 },
    { 0x1f573257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe687325c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c73264, 40 },
    { 0, 0 },
    { 0x59c73266, 23 },
    { 0xe1973267, 95 },
    { 0, 0 },
    { 0xaca73269, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec7326f, 99 },
    { 0, 0 },
    { 0xdf873271, 93 },
    { 0x3bb73271, 64 },
    { 0, 0 },
    { 0xb1373274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2373277, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7327d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42073286, 33 },
    { 0, 0 },
    { 0x373288, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee7328c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b7328f, 55 },
    { 0x1fa73290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c73293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1373296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea67329c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15732a0, 61 },
    { 0, 0 },
    { 0x282732a2, 95 },
    { 0xb89732a2, 95 },
    { 0xf41732a4, 94 },
    { 0x768732a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0732a8, 33 },
    { 0, 0 },
    { 0xede732aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e732ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5732b5, 23 },
    { 0xd7e732b6, 34 },
    { 0x40d732b7, 95 },
    { 0, 0 },
    { 0x355732b9, 91 },
    { 0x7fc732ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8732bd, 95 },
    { 0x170732be, 61 },
    { 0xc81732bf, 61 },
    { 0, 0 },
    { 0xacd732c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633732c5, 33 },
    { 0x18a732c6, 23 },
    { 0x7fc732c5, 95 },
    { 0x431732c7, 33 },
    { 0xef9732c9, 95 },
    { 0, 0 },
    { 0xdfc732cb, 78 },
    { 0xbb4732cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb732cf, 95 },
    { 0xc5d732d0, 33 },
    { 0xcd6732d0, 33 },
    { 0x7bc732d1, 33 },
    { 0x679732d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781732d7, 95 },
    { 0xf08732d7, 23 },
    { 0xbef732d9, 61 },
    { 0, 0 },
    { 0x40c732db, 17 },
    { 0x5f7732dc, 95 },
    { 0x13c732dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9732e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3732e3, 43 },
    { 0, 0 },
    { 0x876732e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5732e8, 33 },
    { 0xe44732e9, 33 },
    { 0, 0 },
    { 0x707732eb, 98 },
    { 0, 0 },
    { 0x5f5732ed, 98 },
    { 0xcb1732ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d732f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd732fb, 100 },
    { 0x1e5732fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60732ff, 95 },
    { 0x4f0732ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe973305, 65 },
    { 0xf5573306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4847330a, 90 },
    { 0x98b7330b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a73312, 84 },
    { 0x78273313, 76 },
    { 0x4fa73312, 76 },
    { 0, 0 },
    { 0x31b73316, 33 },
    { 0x81a73317, 90 },
    { 0xf3073317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae7331c, 90 },
    { 0x5527331d, 34 },
    { 0x72a7331e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f773322, 93 },
    { 0x6c173322, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20873327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa7332a, 66 },
    { 0x59b7332b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7332e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1673333, 81 },
    { 0x79c73334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad77333a, 90 },
    { 0xb6a7333b, 76 },
    { 0x52a7333a, 33 },
    { 0x7a17333c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81073342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5773346, 90 },
    { 0xe2273347, 33 },
    { 0x41f73346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37334b, 61 },
    { 0xc697334c, 65 },
    { 0x6107334d, 61 },
    { 0, 0 },
    { 0x5107334f, 34 },
    { 0, 0 },
    { 0xdee73351, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e73355, 100 },
    { 0, 0 },
    { 0xf4c73357, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f7335b, 46 },
    { 0x8fd7335b, 87 },
    { 0, 0 },
    { 0x3587335e, 46 },
    { 0x1ac7335f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c73366, 78 },
    { 0xadf73366, 34 },
    { 0, 0 },
    { 0x30c73369, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87336c, 77 },
    { 0, 0 },
    { 0x3927336e, 52 },
    { 0, 0 },
    { 0x3ea73370, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba173374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d77337a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7337d, 98 },
    { 0, 0 },
    { 0xdc67337f, 55 },
    { 0, 0 },
    { 0xe8973381, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b77338d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da73393, 33 },
    { 0xd5d73393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a97339a, 83 },
    { 0, 0 },
    { 0xa717339c, 61 },
    { 0xfcc7339d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c733a0, 68 },
    { 0x2d3733a1, 76 },
    { 0xfe2733a2, 77 },
    { 0xea9733a2, 55 },
    { 0x9a0733a4, 52 },
    { 0x95733a2, 90 },
    { 0xb1b733a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e733ad, 75 },
    { 0xf05733ae, 46 },
    { 0xd0b733af, 75 },
    { 0xc2733b0, 95 },
    { 0x884733ae, 52 },
    { 0x754733b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c733b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b733bb, 43 },
    { 0x5a0733bc, 61 },
    { 0x2d7733bd, 98 },
    { 0xcb5733bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30733c1, 61 },
    { 0x366733c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1733c5, 33 },
    { 0xaed733c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b733ca, 43 },
    { 0, 0 },
    { 0xe02733cc, 95 },
    { 0x905733cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47733d5, 85 },
    { 0xdc8733d6, 85 },
    { 0xf57733d7, 33 },
    { 0x384733d8, 81 },
    { 0x745733d7, 76 },
    { 0x640733d5, 99 },
    { 0xdef733db, 33 },
    { 0xf5e733dc, 95 },
    { 0x24c733dc, 66 },
    { 0x7e5733de, 23 },
    { 0x10d733d5, 61 },
    { 0x6ad733e0, 61 },
    { 0xb97733d9, 61 },
    { 0x452733dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x338733e5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3733e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e733ed, 33 },
    { 0x320733ee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53733f3, 46 },
    { 0xad6733f3, 76 },
    { 0x698733f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686733fb, 90 },
    { 0x41f733fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad73408, 33 },
    { 0x3c373408, 76 },
    { 0x77573408, 85 },
    { 0xdfe7340a, 23 },
    { 0xbd573409, 61 },
    { 0x9d67340d, 33 },
    { 0x1457340c, 98 },
    { 0, 0 },
    { 0x1e873410, 95 },
    { 0x3f473411, 93 },
    { 0xf1e73410, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb673419, 90 },
    { 0x82b7341a, 90 },
    { 0xc237341b, 90 },
    { 0x6267341b, 99 },
    { 0x6137341d, 77 },
    { 0x52e7341c, 76 },
    { 0xe0f7341f, 91 },
    { 0x6a773420, 98 },
    { 0xf0f73421, 90 },
    { 0x5bb73422, 87 },
    { 0, 0 },
    { 0x49673424, 34 },
    { 0, 0 },
    { 0x50473426, 98 },
    { 0xf4a73427, 33 },
    { 0x1c673426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7342b, 33 },
    { 0x8a67342c, 65 },
    { 0x7517342d, 34 },
    { 0, 0 },
    { 0x5a07342f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45173433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56073437, 87 },
    { 0x7e673437, 95 },
    { 0xdc273437, 55 },
    { 0x16273438, 84 },
    { 0xed7343b, 76 },
    { 0x49073439, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa873440, 43 },
    { 0x1be73440, 23 },
    { 0, 0 },
    { 0xefa73443, 95 },
    { 0x7073444, 78 },
    { 0x13a73444, 46 },
    { 0xf5a73443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca7344c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e73450, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d773459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x67345c, 23 },
    { 0x4e47345c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe973461, 33 },
    { 0, 0 },
    { 0xc2c73463, 81 },
    { 0xfeb73463, 52 },
    { 0x68f73465, 87 },
    { 0x93b73465, 35 },
    { 0xdd573464, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f7346d, 87 },
    { 0, 0 },
    { 0xddf7346f, 19 },
    { 0x95a7346f, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b473476, 34 },
    { 0x39273476, 33 },
    { 0xe4a73478, 43 },
    { 0xff973479, 99 },
    { 0, 0 },
    { 0x6e87347b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c73485, 90 },
    { 0xb8a73486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f73489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da7348d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc273495, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7349b, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe7349f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e734a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed734ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705734b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c734b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355734bf, 23 },
    { 0xc69734c0, 33 },
    { 0x389734c1, 61 },
    { 0xb9c734c1, 90 },
    { 0, 0 },
    { 0x9fd734c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f734ca, 75 },
    { 0xe70734cb, 61 },
    { 0xb03734cb, 33 },
    { 0x489734cb, 93 },
    { 0, 0 },
    { 0x37a734cf, 33 },
    { 0, 0 },
    { 0xf71734d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df734d4, 55 },
    { 0, 0 },
    { 0x8ae734d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5734db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e734e6, 84 },
    { 0x21734e6, 61 },
    { 0x257734e7, 61 },
    { 0x6d9734e9, 98 },
    { 0xad9734ea, 88 },
    { 0x6cc734eb, 33 },
    { 0x139734e6, 75 },
    { 0, 0 },
    { 0xbfc734ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39734f4, 23 },
    { 0xa68734f5, 64 },
    { 0x804734f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1734ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a73502, 91 },
    { 0x32473502, 87 },
    { 0, 0 },
    { 0x61073505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21373509, 95 },
    { 0x6577350a, 76 },
    { 0xef77350b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11973511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc07351c, 33 },
    { 0xd727351d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f73523, 46 },
    { 0, 0 },
    { 0xe3773525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df73529, 76 },
    { 0x6077352a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7352e, 95 },
    { 0x82e7352f, 33 },
    { 0x137352e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc73533, 100 },
    { 0, 0 },
    { 0x2f373535, 33 },
    { 0x2aa73535, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde473539, 95 },
    { 0x62b7353a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7073544, 23 },
    { 0xf7273545, 100 },
    { 0, 0 },
    { 0x28873547, 68 },
    { 0x54173548, 33 },
    { 0x26673547, 33 },
    { 0xc9f73549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17354d, 90 },
    { 0, 0 },
    { 0x9837354f, 87 },
    { 0x3f873550, 81 },
    { 0xa017354f, 46 },
    { 0xf4c73550, 44 },
    { 0xecd73553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10073557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e7355a, 61 },
    { 0x767355a, 98 },
    { 0x7b67355c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a73560, 33 },
    { 0xea73561, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3173565, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc73569, 100 },
    { 0x73273569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff7356f, 67 },
    { 0x16c73570, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9173575, 77 },
    { 0x3f73576, 95 },
    { 0x72473577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817357a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33173580, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d773583, 65 },
    { 0, 0 },
    { 0xa2173585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10773588, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187358d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee473596, 90 },
    { 0, 0 },
    { 0xf5673598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6735a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6735a5, 67 },
    { 0, 0 },
    { 0xa9d735a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba735ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07735ae, 23 },
    { 0, 0 },
    { 0x232735b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664735b5, 34 },
    { 0x917735b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55735ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905735c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e735c6, 87 },
    { 0xfa9735c7, 78 },
    { 0xe3b735c8, 95 },
    { 0x5d9735c9, 98 },
    { 0x5b2735c8, 91 },
    { 0x62f735c6, 68 },
    { 0, 0 },
    { 0xd14735cd, 76 },
    { 0xef1735cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233735d3, 99 },
    { 0xb5b735d4, 61 },
    { 0x600735d5, 52 },
    { 0x7af735d5, 23 },
    { 0, 0 },
    { 0xcb9735d8, 95 },
    { 0x225735d8, 55 },
    { 0, 0 },
    { 0x1c7735db, 61 },
    { 0xa8d735dc, 23 },
    { 0x979735dd, 100 },
    { 0xfb8735de, 98 },
    { 0xcd2735df, 67 },
    { 0x1f7735de, 68 },
    { 0, 0 },
    { 0x473735e2, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d735e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80735ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7735ef, 47 },
    { 0xd7a735ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e735f6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191735fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d73601, 23 },
    { 0x5d473602, 23 },
    { 0, 0 },
    { 0x23973604, 55 },
    { 0, 0 },
    { 0x25c73606, 33 },
    { 0x9fb73607, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4817360a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17360d, 95 },
    { 0x9f07360e, 38 },
    { 0, 0 },
    { 0xae73610, 90 },
    { 0x69d73610, 33 },
    { 0, 0 },
    { 0x4ee73613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73618, 66 },
    { 0xe1273619, 34 },
    { 0x6ab73619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5173622, 23 },
    { 0, 0 },
    { 0x80073624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad473627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8087362c, 33 },
    { 0x3957362d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb73632, 64 },
    { 0xf4173633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94873636, 98 },
    { 0x30f73636, 23 },
    { 0x2a73638, 93 },
    { 0, 0 },
    { 0x3cd7363a, 61 },
    { 0xc9a7363b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2373640, 90 },
    { 0x3ab73640, 61 },
    { 0, 0 },
    { 0xfc773643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e47364a, 61 },
    { 0xe107364b, 90 },
    { 0x4377364c, 90 },
    { 0x46c7364b, 98 },
    { 0xe1b7364c, 69 },
    { 0x23f7364a, 55 },
    { 0xf1a7364d, 98 },
    { 0xa2173651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93173656, 55 },
    { 0xa7373657, 98 },
    { 0, 0 },
    { 0x41a73659, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb207365d, 95 },
    { 0, 0 },
    { 0xbab7365f, 55 },
    { 0x7767365f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787366a, 33 },
    { 0, 0 },
    { 0x4a37366c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f73671, 98 },
    { 0xb9573672, 93 },
    { 0xdf173672, 84 },
    { 0xb1d73674, 98 },
    { 0x40673675, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a27367d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c73683, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5873687, 33 },
    { 0x15373688, 76 },
    { 0, 0 },
    { 0x3397368a, 67 },
    { 0x2d97368b, 87 },
    { 0x7ae7368c, 83 },
    { 0x5b7368d, 98 },
    { 0xee27368e, 90 },
    { 0x26f7368e, 23 },
    { 0, 0 },
    { 0xdaa73691, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee73695, 33 },
    { 0, 0 },
    { 0x8773697, 98 },
    { 0x71073698, 33 },
    { 0, 0 },
    { 0x1977369a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17369f, 76 },
    { 0x403736a0, 76 },
    { 0xae27369f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2736a6, 76 },
    { 0xad2736a7, 33 },
    { 0, 0 },
    { 0xf4c736a9, 90 },
    { 0x577736aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a736b6, 78 },
    { 0x720736b6, 61 },
    { 0xa7b736b8, 34 },
    { 0x59c736b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825736be, 98 },
    { 0xc93736bf, 90 },
    { 0xffa736bf, 81 },
    { 0x93d736c1, 76 },
    { 0, 0 },
    { 0xaf736c3, 23 },
    { 0xa06736c3, 80 },
    { 0, 0 },
    { 0xf66736c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9736cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06736ce, 33 },
    { 0, 0 },
    { 0x944736d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3736d8, 76 },
    { 0, 0 },
    { 0x701736da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167736e3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6736ed, 90 },
    { 0, 0 },
    { 0xac4736ef, 33 },
    { 0x504736f0, 90 },
    { 0xef7736f1, 87 },
    { 0, 0 },
    { 0xfc5736f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf736f9, 61 },
    { 0x85a736f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529736fe, 66 },
    { 0x9f7736ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3173704, 95 },
    { 0xdec73705, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3373708, 90 },
    { 0x73d73708, 23 },
    { 0x667370a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf273712, 61 },
    { 0x89b73712, 33 },
    { 0, 0 },
    { 0x9ab73715, 98 },
    { 0x8b873716, 93 },
    { 0xfe473717, 95 },
    { 0x78073718, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47371c, 95 },
    { 0x1527371d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec373723, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b73728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e57372b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86073733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8873736, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c17373a, 94 },
    { 0, 0 },
    { 0xd77373c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e73743, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x45873746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c17374b, 17 },
    { 0x19d7374c, 76 },
    { 0xbc07374d, 34 },
    { 0x2777374b, 66 },
    { 0x9de7374f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf73754, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e73757, 76 },
    { 0x17d73757, 76 },
    { 0, 0 },
    { 0x19f7375a, 90 },
    { 0xc2d7375b, 81 },
    { 0xd577375b, 33 },
    { 0, 0 },
    { 0xe5d7375e, 23 },
    { 0xe257375f, 98 },
    { 0, 0 },
    { 0xa1673761, 61 },
    { 0xd8273761, 91 },
    { 0, 0 },
    { 0x6a473764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85873767, 65 },
    { 0, 0 },
    { 0xd4373769, 55 },
    { 0xb787376a, 23 },
    { 0, 0 },
    { 0x8b37376c, 34 },
    { 0, 0 },
    { 0xc767376e, 23 },
    { 0xb0d7376f, 66 },
    { 0x1f17376f, 67 },
    { 0x4ab7376f, 46 },
    { 0x4d173772, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74673776, 76 },
    { 0x72f73777, 87 },
    { 0x8b373776, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8777377c, 76 },
    { 0x9f27377c, 33 },
    { 0x2117377e, 90 },
    { 0, 0 },
    { 0x1f73780, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d373786, 52 },
    { 0x8f873787, 55 },
    { 0xfcd73788, 34 },
    { 0x3f073788, 75 },
    { 0, 0 },
    { 0xc517378b, 23 },
    { 0, 0 },
    { 0x7087378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc73795, 99 },
    { 0x14f73795, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a573799, 91 },
    { 0x7217379a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb737a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f737ad, 68 },
    { 0x816737ae, 85 },
    { 0x305737af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x354737b2, 66 },
    { 0, 0 },
    { 0xd95737b4, 33 },
    { 0, 0 },
    { 0xc78737b6, 61 },
    { 0xc92737b7, 97 },
    { 0x3d1737b7, 99 },
    { 0x447737b9, 95 },
    { 0xa4737b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2737c1, 98 },
    { 0xa34737c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8737c7, 23 },
    { 0, 0 },
    { 0x2ff737c9, 81 },
    { 0xebf737c9, 84 },
    { 0x874737c9, 98 },
    { 0, 0 },
    { 0xf51737cd, 95 },
    { 0x87c737ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7737d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x568737d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6737dc, 55 },
    { 0xa2737dc, 52 },
    { 0, 0 },
    { 0x1ed737df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a737e2, 62 },
    { 0, 0 },
    { 0x9f1737e4, 46 },
    { 0xd04737e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde737e8, 34 },
    { 0, 0 },
    { 0x3dd737ea, 33 },
    { 0xdd6737eb, 98 },
    { 0xd2737eb, 61 },
    { 0xcf4737ec, 90 },
    { 0xf3c737ed, 61 },
    { 0xe27737ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1737f4, 97 },
    { 0x759737f4, 90 },
    { 0, 0 },
    { 0x4a1737f7, 61 },
    { 0xb73737f8, 98 },
    { 0, 0 },
    { 0x4a3737fa, 33 },
    { 0x1aa737fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x550737fe, 64 },
    { 0, 0 },
    { 0xc6d73800, 66 },
    { 0, 0 },
    { 0xac573802, 90 },
    { 0xd6973803, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e73806, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6273809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c7380f, 91 },
    { 0x88373810, 34 },
    { 0xe6b73811, 95 },
    { 0x64c73812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf7381a, 52 },
    { 0, 0 },
    { 0xeb17381c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b7381f, 90 },
    { 0x81673820, 81 },
    { 0xd107381f, 52 },
    { 0xb8073822, 76 },
    { 0x31473820, 90 },
    { 0x5c973820, 95 },
    { 0x96373824, 90 },
    { 0x1d073820, 19 },
    { 0x17a73821, 55 },
    { 0x18873821, 23 },
    { 0, 0 },
    { 0x17d7382a, 76 },
    { 0x98c7382b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91573832, 95 },
    { 0x3c873833, 75 },
    { 0x67973834, 61 },
    { 0xc7573834, 33 },
    { 0xa0f73836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc73839, 95 },
    { 0, 0 },
    { 0x77c7383b, 91 },
    { 0xfe47383c, 33 },
    { 0, 0 },
    { 0x1647383e, 99 },
    { 0x8657383f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c373847, 33 },
    { 0xe5b73848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8873850, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f173853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41473858, 90 },
    { 0x6b773859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7385f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc73866, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7427386e, 75 },
    { 0, 0 },
    { 0xb2973870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b273873, 33 },
    { 0x47c73874, 34 },
    { 0xad273875, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c473878, 82 },
    { 0x3f573879, 98 },
    { 0, 0 },
    { 0x4ae7387b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c73881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773886, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597388a, 33 },
    { 0x4517388a, 55 },
    { 0x66b7388a, 93 },
    { 0x7b27388d, 95 },
    { 0xe627388d, 61 },
    { 0x94c7388e, 88 },
    { 0x7eb73890, 75 },
    { 0x9477388f, 95 },
    { 0xd6e73892, 23 },
    { 0xd437388b, 43 },
    { 0x6d873894, 34 },
    { 0x1a37388c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e7389f, 95 },
    { 0x8a738a0, 24 },
    { 0x5bd738a1, 34 },
    { 0xb0b738a2, 95 },
    { 0x765738a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b738a9, 33 },
    { 0, 0 },
    { 0x38f738ab, 98 },
    { 0x60e738ac, 34 },
    { 0xb44738ac, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7738b0, 90 },
    { 0, 0 },
    { 0xe3f738b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c738b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10738bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571738c2, 90 },
    { 0xd32738c3, 90 },
    { 0xd52738c3, 78 },
    { 0, 0 },
    { 0x1c6738c6, 98 },
    { 0, 0 },
    { 0xb8a738c8, 95 },
    { 0x25d738c8, 65 },
    { 0xb3d738c8, 61 },
    { 0x807738ca, 95 },
    { 0x204738c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c738d7, 88 },
    { 0, 0 },
    { 0x4da738d9, 95 },
    { 0xd8c738da, 34 },
    { 0xf76738da, 67 },
    { 0, 0 },
    { 0x2738dd, 34 },
    { 0xbf7738de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff738e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943738e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9738eb, 61 },
    { 0xe37738ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5738fb, 46 },
    { 0, 0 },
    { 0x269738fd, 93 },
    { 0x138738fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b973904, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb573908, 98 },
    { 0x8f73908, 61 },
    { 0xb3d7390a, 90 },
    { 0x7747390b, 81 },
    { 0x89d7390c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee97390f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac73914, 92 },
    { 0, 0 },
    { 0x25273916, 95 },
    { 0xc1a73917, 90 },
    { 0x3f773916, 87 },
    { 0x38a73919, 34 },
    { 0x4cb7391a, 98 },
    { 0xd2773918, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92773921, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe73925, 81 },
    { 0xbb873926, 46 },
    { 0, 0 },
    { 0x53273928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa967392c, 46 },
    { 0, 0 },
    { 0xedc7392e, 43 },
    { 0xb837392f, 67 },
    { 0x1537392e, 100 },
    { 0, 0 },
    { 0x3dd73932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a73937, 55 },
    { 0, 0 },
    { 0x45c73939, 23 },
    { 0x5657393a, 76 },
    { 0x5557393b, 61 },
    { 0, 0 },
    { 0x2287393d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5373945, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31073949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17394c, 91 },
    { 0xa267394d, 98 },
    { 0xde37394e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b73952, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d773956, 23 },
    { 0, 0 },
    { 0x33173958, 66 },
    { 0, 0 },
    { 0x6fa7395a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d7395f, 55 },
    { 0, 0 },
    { 0x83d73961, 76 },
    { 0xfcf73962, 93 },
    { 0, 0 },
    { 0x62073964, 66 },
    { 0x9d873964, 88 },
    { 0, 0 },
    { 0xc2273967, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07396b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x887396e, 90 },
    { 0xd3d7396e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f73972, 85 },
    { 0, 0 },
    { 0x18a73974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7397a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad673981, 52 },
    { 0xec173982, 98 },
    { 0x81273983, 17 },
    { 0, 0 },
    { 0x2e473985, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd7398e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a973991, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3373995, 23 },
    { 0x27073996, 33 },
    { 0x88d73995, 61 },
    { 0, 0 },
    { 0xf5073999, 93 },
    { 0x9ea7399a, 98 },
    { 0x77399b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7399f, 90 },
    { 0, 0 },
    { 0x435739a1, 93 },
    { 0x7b0739a2, 90 },
    { 0xd08739a3, 38 },
    { 0, 0 },
    { 0x937739a5, 23 },
    { 0x27739a5, 34 },
    { 0x46739a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d739aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x436739ad, 33 },
    { 0x6ca739ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536739b2, 98 },
    { 0, 0 },
    { 0x2d8739b4, 76 },
    { 0x7fd739b4, 33 },
    { 0xc5a739b6, 61 },
    { 0xa93739b6, 98 },
    { 0x11b739b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e739bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478739c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3739d2, 61 },
    { 0x58d739d2, 76 },
    { 0xdde739d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8739d7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555739df, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd739e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455739e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7739ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239739f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b739fb, 98 },
    { 0xda2739fb, 76 },
    { 0, 0 },
    { 0x843739fe, 98 },
    { 0xa5739fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70973a04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73173a08, 23 },
    { 0, 0 },
    { 0xab173a0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe173a0e, 23 },
    { 0xe1073a0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd173a12, 90 },
    { 0, 0 },
    { 0x14173a14, 61 },
    { 0, 0 },
    { 0xb5e73a16, 90 },
    { 0x2e873a16, 77 },
    { 0x1373a18, 34 },
    { 0x3b973a18, 61 },
    { 0, 0 },
    { 0x51973a1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e73a1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b73a23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff373a26, 23 },
    { 0x5ea73a27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc273a2c, 61 },
    { 0xff373a2d, 23 },
    { 0x7f973a2e, 84 },
    { 0x3b173a2f, 90 },
    { 0x1a273a30, 84 },
    { 0x14c73a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa273a34, 55 },
    { 0, 0 },
    { 0x63873a36, 33 },
    { 0, 0 },
    { 0xdb73a38, 81 },
    { 0, 0 },
    { 0xe0373a3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1373a3d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9873a41, 23 },
    { 0, 0 },
    { 0xb3873a43, 90 },
    { 0xcc573a44, 84 },
    { 0, 0 },
    { 0xea473a46, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f373a4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be73a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50173a52, 95 },
    { 0x9ce73a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e73a56, 23 },
    { 0, 0 },
    { 0xfd873a58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f73a5b, 23 },
    { 0, 0 },
    { 0x5eb73a5d, 90 },
    { 0x97073a5e, 33 },
    { 0, 0 },
    { 0xdc373a60, 59 },
    { 0, 0 },
    { 0xea873a62, 76 },
    { 0, 0 },
    { 0x3e973a64, 61 },
    { 0xa1973a65, 61 },
    { 0x73973a64, 98 },
    { 0x4e673a67, 90 },
    { 0xd9f73a66, 23 },
    { 0x19073a69, 42 },
    { 0x27073a6a, 98 },
    { 0xfbc73a6b, 95 },
    { 0xdfa73a6b, 23 },
    { 0, 0 },
    { 0x5e773a6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21273a75, 90 },
    { 0x58373a76, 55 },
    { 0x15173a77, 61 },
    { 0x12373a78, 61 },
    { 0, 0 },
    { 0xdd073a7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d573a7d, 95 },
    { 0x31073a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93973a82, 23 },
    { 0x21773a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73a88, 75 },
    { 0, 0 },
    { 0xe9f73a8a, 61 },
    { 0, 0 },
    { 0xc0773a8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52173a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf173a96, 61 },
    { 0x12673a97, 78 },
    { 0xa0773a96, 95 },
    { 0x93f73a97, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb373a9c, 34 },
    { 0xfec73a9d, 33 },
    { 0, 0 },
    { 0x82073a9f, 61 },
    { 0x6573aa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f73aa3, 46 },
    { 0x96273aa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90773aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6373aab, 46 },
    { 0x4973aac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7573aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d673ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51273aba, 99 },
    { 0xdb273abb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17373ac1, 24 },
    { 0x5373ac2, 95 },
    { 0x43773ac3, 33 },
    { 0xa5c73ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c73ace, 90 },
    { 0, 0 },
    { 0xa673ad0, 99 },
    { 0xbe073ad0, 61 },
    { 0x1e873ad1, 23 },
    { 0x47a73ad3, 33 },
    { 0x33e73ad1, 45 },
    { 0xc9573ad5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22473ad9, 90 },
    { 0xb8273ad9, 91 },
    { 0x5be73ad9, 90 },
    { 0x85b73ada, 95 },
    { 0xbe673add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c73ae2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e73ae6, 90 },
    { 0, 0 },
    { 0x3b973ae8, 99 },
    { 0x46773ae9, 78 },
    { 0, 0 },
    { 0x7e573aeb, 90 },
    { 0xdd873aeb, 98 },
    { 0xc1973aed, 43 },
    { 0xac173aee, 76 },
    { 0x1ad73aee, 34 },
    { 0, 0 },
    { 0xc6373af1, 81 },
    { 0x37573af2, 61 },
    { 0xaf273af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d73af8, 66 },
    { 0x11d73af9, 33 },
    { 0xca073afa, 77 },
    { 0x8e73af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a73afe, 33 },
    { 0xad273afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e73b05, 76 },
    { 0, 0 },
    { 0x4f773b07, 66 },
    { 0x1b273b07, 33 },
    { 0x8ca73b09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf973b0d, 99 },
    { 0x37873b0d, 90 },
    { 0xb6773b0e, 38 },
    { 0x68873b10, 46 },
    { 0x54a73b11, 95 },
    { 0x49b73b12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b673b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc573b18, 65 },
    { 0xaa273b19, 52 },
    { 0x52473b1a, 91 },
    { 0x63c73b19, 99 },
    { 0x42773b1c, 98 },
    { 0, 0 },
    { 0x2bd73b1e, 87 },
    { 0, 0 },
    { 0xf0d73b20, 95 },
    { 0, 0 },
    { 0x49373b22, 93 },
    { 0x34f73b22, 98 },
    { 0xc673b24, 67 },
    { 0, 0 },
    { 0xd573b26, 90 },
    { 0xd2d73b27, 62 },
    { 0x88a73b28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2373b2d, 95 },
    { 0x57473b2e, 99 },
    { 0, 0 },
    { 0xf1173b30, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64973b35, 98 },
    { 0, 0 },
    { 0x5e473b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae73b3b, 33 },
    { 0xcd73b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1173b42, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52473b4d, 90 },
    { 0x55f73b4e, 91 },
    { 0x14e73b4f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51373b54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0573b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a473b5d, 43 },
    { 0, 0 },
    { 0x77e73b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72573b63, 95 },
    { 0x97873b63, 87 },
    { 0xf6a73b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce73b71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c73b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5473b77, 98 },
    { 0x87b73b78, 61 },
    { 0xad073b79, 23 },
    { 0x3973b79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f973b81, 88 },
    { 0xcc773b82, 90 },
    { 0, 0 },
    { 0xb7c73b84, 78 },
    { 0xcc873b85, 95 },
    { 0x82e73b84, 97 },
    { 0x12c73b85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073b8b, 33 },
    { 0xe2373b8c, 43 },
    { 0, 0 },
    { 0xaef73b8e, 81 },
    { 0, 0 },
    { 0x43273b90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b73b94, 34 },
    { 0, 0 },
    { 0x57973b96, 75 },
    { 0x5c073b97, 76 },
    { 0, 0 },
    { 0xd0173b99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f73b9c, 33 },
    { 0xa4773b9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d273ba1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x79173ba4, 94 },
    { 0xacf73ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af73ba9, 23 },
    { 0x80773baa, 61 },
    { 0xf1a73bab, 61 },
    { 0xb2b73bac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16773bb2, 61 },
    { 0, 0 },
    { 0xb7373bb4, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e73bb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd73bbc, 34 },
    { 0, 0 },
    { 0x57f73bbe, 95 },
    { 0xd2073bbf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f73bc3, 95 },
    { 0, 0 },
    { 0x3773bc5, 78 },
    { 0xbcd73bc5, 46 },
    { 0, 0 },
    { 0x87b73bc8, 69 },
    { 0x15673bc9, 93 },
    { 0, 0 },
    { 0x1da73bcb, 75 },
    { 0x24673bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6173bd0, 61 },
    { 0x94173bd1, 23 },
    { 0x32373bd1, 91 },
    { 0xf6173bd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1273bd8, 90 },
    { 0x1a973bd9, 90 },
    { 0x15a73bda, 98 },
    { 0x94973bd9, 55 },
    { 0x37273bdb, 98 },
    { 0x8ef73bdd, 61 },
    { 0x82973bde, 43 },
    { 0x56973bdf, 91 },
    { 0x92773bdd, 95 },
    { 0xa9e73be1, 90 },
    { 0x8a173be2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373be8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff873beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb73bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f173bf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d73bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd673bfd, 98 },
    { 0xb6e73bfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71573c02, 98 },
    { 0x7773c02, 33 },
    { 0xcd473c04, 33 },
    { 0, 0 },
    { 0xb2e73c06, 76 },
    { 0, 0 },
    { 0x2d473c08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da73c0c, 98 },
    { 0, 0 },
    { 0x24073c0e, 93 },
    { 0, 0 },
    { 0x45c73c10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b673c13, 98 },
    { 0xe573c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f173c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5273c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf73c21, 98 },
    { 0, 0 },
    { 0xbb273c23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25473c26, 55 },
    { 0x70b73c27, 99 },
    { 0xd0973c27, 34 },
    { 0x18773c29, 85 },
    { 0x13c73c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573c2e, 90 },
    { 0xa1b73c2f, 61 },
    { 0xd2673c2f, 90 },
    { 0x2c573c31, 78 },
    { 0xd6173c31, 23 },
    { 0x31873c2e, 66 },
    { 0x84e73c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c73c3a, 33 },
    { 0x1ce73c3b, 23 },
    { 0x2d373c3c, 99 },
    { 0x3fb73c3a, 94 },
    { 0, 0 },
    { 0x7973c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff173c43, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c73c49, 98 },
    { 0xad073c4a, 55 },
    { 0xbde73c4b, 90 },
    { 0xa9573c4c, 33 },
    { 0x8b773c4d, 90 },
    { 0x95573c4e, 84 },
    { 0, 0 },
    { 0x89b73c50, 23 },
    { 0, 0 },
    { 0x6d573c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d573c55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43473c5b, 76 },
    { 0xbf173c5c, 90 },
    { 0, 0 },
    { 0xa3d73c5e, 61 },
    { 0x56b73c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a73c67, 90 },
    { 0x7e873c67, 23 },
    { 0, 0 },
    { 0x9dd73c6a, 55 },
    { 0x28773c6b, 40 },
    { 0xee973c6c, 68 },
    { 0x6c873c6c, 17 },
    { 0, 0 },
    { 0x2d573c6f, 95 },
    { 0x15a73c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48773c75, 90 },
    { 0xa6473c76, 78 },
    { 0, 0 },
    { 0xb4173c78, 23 },
    { 0x93b73c79, 33 },
    { 0xb673c7a, 33 },
    { 0x89573c7b, 52 },
    { 0xe2c73c7b, 61 },
    { 0x68c73c7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c973c80, 66 },
    { 0x7fc73c80, 52 },
    { 0x66b73c81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31673c85, 33 },
    { 0xdca73c86, 76 },
    { 0, 0 },
    { 0x36573c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64773c8d, 43 },
    { 0xa2373c8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51873c92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a73c96, 61 },
    { 0xc5a73c96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3273c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a73c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea73ca3, 67 },
    { 0x5cd73ca4, 90 },
    { 0x5b73ca5, 66 },
    { 0, 0 },
    { 0x7fc73ca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7173cab, 78 },
    { 0x32b73cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a73caf, 33 },
    { 0x26d73caf, 91 },
    { 0, 0 },
    { 0xca073cb2, 33 },
    { 0, 0 },
    { 0xb0d73cb4, 98 },
    { 0, 0 },
    { 0x96373cb6, 95 },
    { 0x6d673cb6, 90 },
    { 0xa7773cb8, 76 },
    { 0xfe473cb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b773cbd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc73cc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8973cc5, 90 },
    { 0, 0 },
    { 0x6e73cc7, 95 },
    { 0, 0 },
    { 0x1b873cc9, 85 },
    { 0, 0 },
    { 0x3373ccb, 76 },
    { 0, 0 },
    { 0xc3a73ccd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48473cd6, 61 },
    { 0x73a73cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73973cda, 91 },
    { 0xf873cda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb73ce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24973ce5, 95 },
    { 0, 0 },
    { 0xf673ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c73cea, 98 },
    { 0x86473cea, 90 },
    { 0x89b73ceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59973cf0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91573cf4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac73cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b573d04, 75 },
    { 0xb6c73d05, 68 },
    { 0x9673d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57873d09, 33 },
    { 0, 0 },
    { 0xe0673d0b, 43 },
    { 0xb6773d0c, 66 },
    { 0, 0 },
    { 0x5c773d0e, 99 },
    { 0, 0 },
    { 0xa2573d10, 61 },
    { 0, 0 },
    { 0xe0b73d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5473d17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a673d1b, 33 },
    { 0xca173d1c, 84 },
    { 0xba273d1d, 91 },
    { 0, 0 },
    { 0xdfc73d1f, 90 },
    { 0, 0 },
    { 0xcd373d21, 95 },
    { 0, 0 },
    { 0x6473d23, 61 },
    { 0xb2373d23, 87 },
    { 0x32373d25, 95 },
    { 0xbe673d25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d73d2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573d2f, 98 },
    { 0x44873d30, 33 },
    { 0x81b73d30, 34 },
    { 0x79373d30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed73d36, 76 },
    { 0x25f73d37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8973d3a, 46 },
    { 0x6d873d3b, 33 },
    { 0xe4273d3b, 83 },
    { 0xa2573d3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf73d44, 90 },
    { 0x3a673d45, 61 },
    { 0xb2373d45, 97 },
    { 0x14273d47, 66 },
    { 0x87873d45, 33 },
    { 0xc4073d46, 95 },
    { 0xeab73d4a, 90 },
    { 0x87973d4b, 88 },
    { 0x51473d4c, 95 },
    { 0x81973d4d, 43 },
    { 0x50273d4e, 55 },
    { 0x9cf73d4f, 78 },
    { 0x88873d50, 17 },
    { 0xb7b73d50, 66 },
    { 0, 0 },
    { 0x69b73d53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0073d56, 23 },
    { 0x7b173d57, 36 },
    { 0, 0 },
    { 0xdf273d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1873d5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93873d62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d73d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe73d70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae073d77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e73d7a, 76 },
    { 0x4cf73d7a, 85 },
    { 0x45f73d7c, 34 },
    { 0x14a73d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e73d88, 40 },
    { 0x18073d89, 93 },
    { 0xbb773d8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4773d8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda73d92, 90 },
    { 0x4f373d92, 99 },
    { 0x5cb73d94, 17 },
    { 0, 0 },
    { 0x17b73d96, 61 },
    { 0, 0 },
    { 0xcc373d98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d773d9b, 95 },
    { 0xe6d73d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb473da7, 33 },
    { 0, 0 },
    { 0x5dd73da9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a173dad, 91 },
    { 0, 0 },
    { 0xaba73daf, 98 },
    { 0x47f73db0, 90 },
    { 0xa9173db1, 90 },
    { 0, 0 },
    { 0xdeb73db3, 85 },
    { 0, 0 },
    { 0xcfb73db5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39473dba, 55 },
    { 0x63673dba, 61 },
    { 0, 0 },
    { 0xd3f73dbd, 61 },
    { 0xfcd73dbd, 98 },
    { 0x50d73dbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da73dc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb473dc6, 43 },
    { 0xbfd73dc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee73dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba73dcf, 95 },
    { 0x9ca73dd0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff73dd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c373dd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca073de2, 76 },
    { 0xc6173de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373de6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3673dea, 85 },
    { 0xe2773deb, 95 },
    { 0x4d273dec, 87 },
    { 0x85d73deb, 95 },
    { 0x27a73dee, 61 },
    { 0xb5173def, 55 },
    { 0x1ec73df0, 33 },
    { 0xb8f73df1, 55 },
    { 0, 0 },
    { 0xceb73df3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb473df6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa573e00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed473e03, 97 },
    { 0, 0 },
    { 0x16a73e05, 88 },
    { 0x1e073e05, 76 },
    { 0xdff73e05, 84 },
    { 0xa5f73e07, 78 },
    { 0xca373e05, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d73e0c, 98 },
    { 0xe1f73e0c, 98 },
    { 0x59b73e0e, 76 },
    { 0, 0 },
    { 0xbf273e10, 90 },
    { 0, 0 },
    { 0xce973e12, 61 },
    { 0x89e73e13, 90 },
    { 0x7dc73e14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf73e1a, 95 },
    { 0x73673e1b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573e23, 55 },
    { 0, 0 },
    { 0x6c873e25, 98 },
    { 0, 0 },
    { 0x90973e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a73e2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d73e2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5873e30, 90 },
    { 0xc9473e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a73e35, 34 },
    { 0, 0 },
    { 0x89073e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa73e3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92673e43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c673e46, 23 },
    { 0x4ef73e46, 84 },
    { 0, 0 },
    { 0x60f73e49, 90 },
    { 0, 0 },
    { 0x2cf73e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c773e50, 98 },
    { 0xfcf73e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d673e54, 23 },
    { 0xf9a73e55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1573e5b, 98 },
    { 0x99873e5c, 53 },
    { 0x8da73e5c, 93 },
    { 0x2e073e5e, 34 },
    { 0xef473e5e, 17 },
    { 0xec73e5f, 90 },
    { 0xcc973e5d, 99 },
    { 0x93e73e62, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc73e6c, 23 },
    { 0x12d73e6d, 85 },
    { 0x44373e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a373e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef73e78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773e7f, 95 },
    { 0, 0 },
    { 0x38273e81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13673e86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a273e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf73e95, 100 },
    { 0xe4973e96, 76 },
    { 0xda073e97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4273e9c, 52 },
    { 0xfa673e9c, 33 },
    { 0xcd273e9e, 43 },
    { 0x7bc73e9e, 90 },
    { 0x4e473e9e, 87 },
    { 0x8dd73ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69173eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70273eb3, 98 },
    { 0x3e973eb3, 95 },
    { 0xc8573eb5, 90 },
    { 0x47d73eb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f73ec0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x29173ec3, 76 },
    { 0x66573ec3, 95 },
    { 0x8b373ec5, 95 },
    { 0xfbc73ec5, 33 },
    { 0, 0 },
    { 0xe0973ec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb73ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b73ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5273ed6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88173ee0, 98 },
    { 0x46173ee1, 23 },
    { 0xd7773ee0, 90 },
    { 0x83c73ee3, 33 },
    { 0, 0 },
    { 0x48973ee5, 19 },
    { 0, 0 },
    { 0xedb73ee7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f73eed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84373ef1, 24 },
    { 0x4c73ef1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b73ef5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76273ef9, 95 },
    { 0, 0 },
    { 0xee973efb, 55 },
    { 0, 0 },
    { 0x49973efd, 46 },
    { 0xac173efd, 76 },
    { 0x68b73eff, 66 },
    { 0x15a73f00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46373f07, 55 },
    { 0x46773f08, 98 },
    { 0x66e73f07, 95 },
    { 0, 0 },
    { 0x2dd73f0b, 76 },
    { 0, 0 },
    { 0xd5d73f0d, 61 },
    { 0x3ef73f0e, 76 },
    { 0, 0 },
    { 0x8773f10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b73f14, 90 },
    { 0x60173f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24173f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc73f1d, 23 },
    { 0, 0 },
    { 0xf6373f1f, 23 },
    { 0x5ee73f1f, 90 },
    { 0, 0 },
    { 0x21e73f22, 23 },
    { 0x48f73f23, 75 },
    { 0x5d173f24, 99 },
    { 0xc4b73f25, 33 },
    { 0x5b273f24, 90 },
    { 0x78873f23, 46 },
    { 0, 0 },
    { 0x3873f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9373f2c, 55 },
    { 0xcf173f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b73f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1673f34, 61 },
    { 0x37773f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7573f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e573f3b, 95 },
    { 0x1f173f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd473f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a73f45, 90 },
    { 0x2c673f46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a573f49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2973f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8573f55, 61 },
    { 0x15473f56, 90 },
    { 0x2c973f56, 95 },
    { 0xc0773f58, 33 },
    { 0x22d73f58, 76 },
    { 0xc9073f59, 55 },
    { 0x83873f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e73f60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77773f63, 100 },
    { 0xa5573f63, 95 },
    { 0, 0 },
    { 0x9c973f66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6073f6b, 46 },
    { 0x5673f6c, 95 },
    { 0x3f573f6d, 61 },
    { 0x54273f6e, 95 },
    { 0x43b73f6f, 61 },
    { 0x2c73f6f, 90 },
    { 0x76873f70, 90 },
    { 0, 0 },
    { 0x6d873f73, 81 },
    { 0x77d73f74, 34 },
    { 0, 0 },
    { 0x94f73f76, 81 },
    { 0, 0 },
    { 0x6bb73f78, 98 },
    { 0x8b073f79, 61 },
    { 0xcfa73f7a, 95 },
    { 0x28f73f78, 33 },
    { 0x10273f7c, 98 },
    { 0x4173f7d, 76 },
    { 0xfa673f7d, 95 },
    { 0, 0 },
    { 0x23373f80, 95 },
    { 0x60b73f80, 98 },
    { 0, 0 },
    { 0x2b773f83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99373f8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6073f8e, 33 },
    { 0, 0 },
    { 0x52473f90, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5673f96, 87 },
    { 0x88073f96, 98 },
    { 0x80573f98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec73f9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60273fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8473fa7, 61 },
    { 0, 0 },
    { 0xb5673fa9, 98 },
    { 0x28173faa, 98 },
    { 0x24973fab, 95 },
    { 0, 0 },
    { 0xcce73fad, 64 },
    { 0, 0 },
    { 0x5aa73faf, 46 },
    { 0, 0 },
    { 0xe0c73fb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c73fb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86373fbc, 24 },
    { 0xf0f73fbd, 95 },
    { 0xf0e73fbc, 33 },
    { 0x90473fbf, 98 },
    { 0x68473fc0, 52 },
    { 0xd4373fc1, 61 },
    { 0x65e73fc1, 91 },
    { 0x4dd73fc2, 76 },
    { 0x3373fc4, 66 },
    { 0x36c73fc2, 95 },
    { 0x47973fc3, 46 },
    { 0x9ae73fc2, 95 },
    { 0xc7073fc8, 78 },
    { 0x5e973fc7, 33 },
    { 0xf6273fc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb73fcd, 95 },
    { 0xe7b73fce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85973fd4, 61 },
    { 0xf2473fd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c73fdd, 23 },
    { 0x64773fde, 34 },
    { 0xef073fdf, 23 },
    { 0xba173fdf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89273fe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b73fec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66173fef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34473ff5, 34 },
    { 0xab873ff5, 76 },
    { 0xe2873ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba73ffb, 33 },
    { 0, 0 },
    { 0xab973ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd74003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097400f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f074012, 91 },
    { 0, 0 },
    { 0xff374014, 86 },
    { 0x4fd74015, 33 },
    { 0x4a774014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057401b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797401e, 68 },
    { 0xdd7401f, 99 },
    { 0, 0 },
    { 0x39374021, 34 },
    { 0, 0 },
    { 0xaf374023, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b87402c, 46 },
    { 0xa527402d, 90 },
    { 0xe897402e, 95 },
    { 0, 0 },
    { 0xb4b74030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d574033, 33 },
    { 0xe0874034, 23 },
    { 0x75974034, 33 },
    { 0, 0 },
    { 0xe1074037, 17 },
    { 0x81174038, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf727403d, 61 },
    { 0x8657403d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d74043, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff74049, 90 },
    { 0x93c7404a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef74051, 61 },
    { 0xdad74052, 88 },
    { 0xd8274052, 98 },
    { 0x2f474052, 23 },
    { 0xe3874054, 46 },
    { 0x71974056, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b74059, 76 },
    { 0x4274059, 99 },
    { 0xc347405b, 93 },
    { 0x6ce7405c, 61 },
    { 0xd1e7405b, 85 },
    { 0x24e7405d, 90 },
    { 0xe97405e, 33 },
    { 0xad27405f, 34 },
    { 0x9127405e, 65 },
    { 0xf0374062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb874066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d774070, 43 },
    { 0, 0 },
    { 0x8c974072, 66 },
    { 0xb6274073, 95 },
    { 0xa0174073, 95 },
    { 0x83274075, 33 },
    { 0x81a74076, 61 },
    { 0x5cf74076, 33 },
    { 0xa5574078, 55 },
    { 0xba874079, 90 },
    { 0xc2f74077, 23 },
    { 0xef374075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac174082, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774085, 95 },
    { 0xafc74086, 98 },
    { 0xf6b74087, 95 },
    { 0x22274088, 33 },
    { 0x71474089, 52 },
    { 0xc774089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c7408d, 61 },
    { 0x9657408e, 40 },
    { 0xe6b7408f, 23 },
    { 0, 0 },
    { 0x2dc74091, 23 },
    { 0x42374092, 33 },
    { 0x7c474093, 98 },
    { 0x45a74093, 92 },
    { 0xc74095, 23 },
    { 0xde074094, 90 },
    { 0x2ce74094, 61 },
    { 0, 0 },
    { 0xffb74099, 76 },
    { 0, 0 },
    { 0x9377409b, 61 },
    { 0x98f7409c, 90 },
    { 0x5517409c, 95 },
    { 0x41e7409e, 61 },
    { 0xea17409d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4740a6, 69 },
    { 0x5fe740a7, 85 },
    { 0xacf740a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6740ab, 61 },
    { 0, 0 },
    { 0xb2e740ad, 61 },
    { 0x2b2740ae, 99 },
    { 0, 0 },
    { 0x4bb740b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x136740b3, 55 },
    { 0, 0 },
    { 0xa48740b5, 98 },
    { 0xc1e740b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe740ba, 33 },
    { 0xf4c740bb, 33 },
    { 0, 0 },
    { 0x389740bd, 77 },
    { 0, 0 },
    { 0x819740bf, 76 },
    { 0xd7a740c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc740c3, 98 },
    { 0x6f4740c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171740c7, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3740cd, 78 },
    { 0x52c740ce, 90 },
    { 0x731740cf, 90 },
    { 0x405740cf, 90 },
    { 0, 0 },
    { 0x226740d2, 66 },
    { 0xd68740d3, 85 },
    { 0x78a740d4, 93 },
    { 0xdb740d5, 33 },
    { 0, 0 },
    { 0x796740d7, 61 },
    { 0x852740d7, 93 },
    { 0xb97740d9, 61 },
    { 0x8c7740d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x441740dd, 99 },
    { 0x9fa740dd, 33 },
    { 0xa23740df, 90 },
    { 0x528740e0, 90 },
    { 0x972740e0, 90 },
    { 0, 0 },
    { 0xe59740e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3740e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08740ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164740f1, 77 },
    { 0, 0 },
    { 0xefd740f3, 91 },
    { 0, 0 },
    { 0x39d740f5, 23 },
    { 0x12e740f5, 61 },
    { 0xbae740f6, 90 },
    { 0xb56740f7, 90 },
    { 0x1dd740f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf74104, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa527410b, 90 },
    { 0xfe97410c, 76 },
    { 0x4967410d, 98 },
    { 0, 0 },
    { 0xf657410f, 98 },
    { 0x35474110, 95 },
    { 0x41974110, 33 },
    { 0, 0 },
    { 0x56674113, 23 },
    { 0xa8b74114, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9547411e, 61 },
    { 0, 0 },
    { 0x86974120, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c74123, 95 },
    { 0x30774124, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af74131, 78 },
    { 0xebc74132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb174135, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba7413a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc707413e, 46 },
    { 0xc257413f, 61 },
    { 0x87874140, 23 },
    { 0xb5474141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c74144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3474147, 78 },
    { 0, 0 },
    { 0x30974149, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57414d, 77 },
    { 0, 0 },
    { 0x48e7414f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8417415b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e74165, 91 },
    { 0xc2974165, 90 },
    { 0x49174167, 23 },
    { 0xf7874166, 33 },
    { 0xbcb74168, 55 },
    { 0, 0 },
    { 0xd587416b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd407416f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d74173, 93 },
    { 0xd1274174, 55 },
    { 0xc6a74174, 98 },
    { 0x23c74176, 66 },
    { 0x90474174, 98 },
    { 0x8af74178, 61 },
    { 0, 0 },
    { 0x5187417a, 98 },
    { 0, 0 },
    { 0x6857417c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77417f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d74185, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3574188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57418e, 43 },
    { 0x7a57418f, 61 },
    { 0x28c7418e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f674198, 61 },
    { 0xba274199, 65 },
    { 0xd3e7419a, 33 },
    { 0xfd67419b, 90 },
    { 0x3af7419c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366741a0, 33 },
    { 0x928741a1, 61 },
    { 0xfc0741a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e741a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625741aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd741ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa741b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x208741b3, 90 },
    { 0, 0 },
    { 0xaec741b5, 61 },
    { 0x2d3741b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae741b9, 95 },
    { 0, 0 },
    { 0x9d9741bb, 98 },
    { 0x86c741bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b741bf, 55 },
    { 0, 0 },
    { 0xbc0741c1, 85 },
    { 0xe56741c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7741c5, 95 },
    { 0, 0 },
    { 0xfa5741c7, 78 },
    { 0x59741c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d741cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7741cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c741d3, 34 },
    { 0xf2e741d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7741d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851741e0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb741e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282741ee, 46 },
    { 0x3df741ef, 76 },
    { 0x31741f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35741f3, 23 },
    { 0, 0 },
    { 0x777741f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488741f9, 85 },
    { 0x6cf741f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x686741fd, 95 },
    { 0x264741fe, 90 },
    { 0x1a3741ff, 55 },
    { 0, 0 },
    { 0x7a74201, 98 },
    { 0, 0 },
    { 0xae274203, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb74209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf107420e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d74214, 61 },
    { 0, 0 },
    { 0x46e74216, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae74219, 91 },
    { 0x3c74219, 55 },
    { 0xbc674219, 33 },
    { 0x91f7421c, 61 },
    { 0x4257421d, 64 },
    { 0x1937421a, 100 },
    { 0x2477421c, 61 },
    { 0, 0 },
    { 0xf4c74221, 90 },
    { 0x28174222, 61 },
    { 0, 0 },
    { 0x45b74224, 98 },
    { 0x5c74224, 98 },
    { 0x1bf74226, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f7422c, 90 },
    { 0xc807422d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba74231, 33 },
    { 0, 0 },
    { 0x84674233, 85 },
    { 0, 0 },
    { 0x81574235, 34 },
    { 0xf274235, 33 },
    { 0x95e74237, 90 },
    { 0x1ef74235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7423f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c274244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa307424d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a74258, 100 },
    { 0x8aa74258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf7425d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca74262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c74265, 46 },
    { 0x6e974266, 33 },
    { 0x9aa74267, 51 },
    { 0xedb74266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f7426b, 76 },
    { 0, 0 },
    { 0xccd7426d, 61 },
    { 0x8417426e, 23 },
    { 0x3cd7426f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d74274, 90 },
    { 0x4dd74274, 76 },
    { 0x1d174275, 90 },
    { 0xb6d74276, 46 },
    { 0xd374274, 76 },
    { 0, 0 },
    { 0xbae7427a, 23 },
    { 0, 0 },
    { 0xdf7427c, 55 },
    { 0xc427427c, 95 },
    { 0x2c87427e, 75 },
    { 0x6777427f, 95 },
    { 0xee274280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb674285, 60 },
    { 0xbba74286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae7428a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb74291, 61 },
    { 0x45a74292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf074295, 61 },
    { 0, 0 },
    { 0x6be74297, 33 },
    { 0x3b874298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f7429e, 33 },
    { 0xce37429f, 98 },
    { 0xa99742a0, 61 },
    { 0xed7742a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913742a7, 23 },
    { 0, 0 },
    { 0xa76742a9, 90 },
    { 0x69c742aa, 99 },
    { 0x2a0742ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6742b5, 90 },
    { 0x3ab742b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7742ba, 91 },
    { 0x705742bb, 33 },
    { 0, 0 },
    { 0xf73742bd, 34 },
    { 0, 0 },
    { 0xc8a742bf, 91 },
    { 0x397742c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba742cc, 78 },
    { 0x17f742cd, 23 },
    { 0, 0 },
    { 0x31d742cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9742d3, 68 },
    { 0x7f4742d3, 23 },
    { 0, 0 },
    { 0xf9a742d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8742d9, 17 },
    { 0x5d0742da, 99 },
    { 0, 0 },
    { 0x815742dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53742df, 98 },
    { 0x3f2742e0, 33 },
    { 0x58a742e1, 55 },
    { 0, 0 },
    { 0xfdb742e3, 65 },
    { 0, 0 },
    { 0x1ab742e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7742e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4742f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee742f8, 91 },
    { 0x6ce742f8, 87 },
    { 0xc59742f8, 91 },
    { 0x6ed742fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69742fe, 8 },
    { 0, 0 },
    { 0x2ab74300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a74303, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c74306, 77 },
    { 0, 0 },
    { 0x97474308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e374311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77431a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267431e, 33 },
    { 0xbe17431f, 23 },
    { 0x40d74320, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc74325, 55 },
    { 0, 0 },
    { 0x55074327, 76 },
    { 0x40974327, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd37432b, 61 },
    { 0, 0 },
    { 0x3b47432d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8174334, 61 },
    { 0, 0 },
    { 0xcfb74336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7433a, 90 },
    { 0x2b57433b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7407433f, 81 },
    { 0, 0 },
    { 0xf4c74341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74349, 61 },
    { 0, 0 },
    { 0x22b7434b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d374355, 90 },
    { 0xeae74356, 46 },
    { 0x44f74355, 98 },
    { 0x97074358, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7435b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83974365, 90 },
    { 0x34174366, 33 },
    { 0, 0 },
    { 0x33b74368, 90 },
    { 0xfd074369, 33 },
    { 0x13574368, 98 },
    { 0xc077436b, 84 },
    { 0x1ad7436c, 46 },
    { 0x2627436c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4274370, 98 },
    { 0xf5174370, 33 },
    { 0xe74371, 95 },
    { 0x21f74372, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b74376, 84 },
    { 0xd6274377, 61 },
    { 0xb7374377, 98 },
    { 0x49b74376, 98 },
    { 0xb407437a, 24 },
    { 0xeb27437a, 61 },
    { 0xaad7437c, 23 },
    { 0x8d07437d, 61 },
    { 0x7b27437a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af74387, 95 },
    { 0, 0 },
    { 0x25674389, 61 },
    { 0x71b74389, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f7438f, 90 },
    { 0x81b74390, 93 },
    { 0xf2374391, 90 },
    { 0, 0 },
    { 0xf4c74393, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b7439a, 76 },
    { 0, 0 },
    { 0x84b7439c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9743a2, 40 },
    { 0, 0 },
    { 0x70c743a4, 87 },
    { 0xb37743a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3743ab, 55 },
    { 0x76a743ac, 46 },
    { 0, 0 },
    { 0x81a743ae, 93 },
    { 0xc36743af, 34 },
    { 0, 0 },
    { 0x6b0743b1, 34 },
    { 0xa66743b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9743b9, 33 },
    { 0, 0 },
    { 0x40e743bb, 43 },
    { 0xd0f743bc, 33 },
    { 0xba1743bd, 33 },
    { 0xda743be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743743c3, 76 },
    { 0, 0 },
    { 0xf2d743c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984743cd, 90 },
    { 0, 0 },
    { 0xb00743cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23743d2, 76 },
    { 0, 0 },
    { 0x43f743d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4743d7, 95 },
    { 0xe8b743d8, 43 },
    { 0, 0 },
    { 0x420743da, 95 },
    { 0x7f6743db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf743e5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede743ec, 90 },
    { 0xa71743ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd743f7, 93 },
    { 0xc4c743f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f743fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f74400, 91 },
    { 0xbca74400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd774405, 61 },
    { 0, 0 },
    { 0x90674407, 61 },
    { 0x37f74408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457440d, 33 },
    { 0x5b87440e, 43 },
    { 0, 0 },
    { 0xdda74410, 33 },
    { 0xdf474410, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d74414, 75 },
    { 0, 0 },
    { 0x8df74416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31474419, 98 },
    { 0x9fc7441a, 77 },
    { 0x59f7441a, 76 },
    { 0x7d47441c, 46 },
    { 0xc437441a, 90 },
    { 0x4d7441e, 78 },
    { 0x7ee7441b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x77974422, 33 },
    { 0x40b74422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb74428, 47 },
    { 0xdd274428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb7442c, 23 },
    { 0x5e87442d, 61 },
    { 0xb977442c, 55 },
    { 0x1487442d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92574435, 66 },
    { 0, 0 },
    { 0x6c474437, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a7443a, 88 },
    { 0xb37443a, 34 },
    { 0x6c67443c, 34 },
    { 0, 0 },
    { 0x8697443e, 58 },
    { 0xf547443e, 33 },
    { 0x40774440, 23 },
    { 0x2f474440, 90 },
    { 0xd5474441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc974446, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa947444d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7445a, 77 },
    { 0x3177445a, 46 },
    { 0xd707445a, 98 },
    { 0, 0 },
    { 0x2567445e, 99 },
    { 0x7d27445f, 36 },
    { 0xe3d74460, 99 },
    { 0x34c74460, 23 },
    { 0xdc74462, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917446c, 95 },
    { 0x8227446d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91474471, 90 },
    { 0x9ec74472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a74478, 33 },
    { 0x70574479, 33 },
    { 0x1167447a, 33 },
    { 0x9477447b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c7447e, 98 },
    { 0x67b7447e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d474485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e7448b, 66 },
    { 0x4057448c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a974490, 95 },
    { 0, 0 },
    { 0x76774492, 52 },
    { 0xdbe74493, 77 },
    { 0x36074494, 95 },
    { 0xf7674495, 87 },
    { 0, 0 },
    { 0x1a174497, 23 },
    { 0xd8074498, 98 },
    { 0xf3d74498, 95 },
    { 0x7657449a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x168744a1, 33 },
    { 0xb72744a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e744ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6744b2, 34 },
    { 0, 0 },
    { 0xb2a744b4, 100 },
    { 0x356744b5, 61 },
    { 0x234744b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40744be, 33 },
    { 0xa84744be, 90 },
    { 0xd66744be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729744c9, 55 },
    { 0xba3744ca, 61 },
    { 0, 0 },
    { 0x8e6744cc, 76 },
    { 0, 0 },
    { 0x149744ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3744d2, 23 },
    { 0xdfa744d3, 61 },
    { 0x2f2744d2, 90 },
    { 0xc33744d4, 46 },
    { 0, 0 },
    { 0x406744d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54744e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f744e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x323744ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae744f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b744f6, 98 },
    { 0xb35744f7, 97 },
    { 0xe27744f7, 33 },
    { 0, 0 },
    { 0x916744fa, 91 },
    { 0xe08744fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x260744fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af74502, 23 },
    { 0x69974502, 98 },
    { 0xf9274504, 98 },
    { 0x2a474505, 90 },
    { 0x93474506, 81 },
    { 0x63c74507, 55 },
    { 0, 0 },
    { 0x10c74509, 88 },
    { 0xf0e7450a, 67 },
    { 0x45c7450b, 61 },
    { 0x3fc7450b, 33 },
    { 0xa557450a, 95 },
    { 0xfce7450b, 98 },
    { 0x1447450e, 85 },
    { 0xc1674509, 61 },
    { 0x42f7450d, 33 },
    { 0xa6574512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a74516, 98 },
    { 0xd4474517, 34 },
    { 0xc5e74518, 76 },
    { 0xe6c74519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4767451d, 46 },
    { 0x587451e, 98 },
    { 0x6a97451f, 17 },
    { 0, 0 },
    { 0x4f274521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1974526, 17 },
    { 0x3a474527, 98 },
    { 0, 0 },
    { 0xb3174529, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9367452e, 78 },
    { 0xe4c7452f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30474539, 55 },
    { 0x357453a, 46 },
    { 0xa127453b, 98 },
    { 0xbdb7453c, 100 },
    { 0x3527453a, 33 },
    { 0x3c57453a, 84 },
    { 0x9a67453f, 90 },
    { 0x50074540, 78 },
    { 0, 0 },
    { 0x87774542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc87454b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d74550, 76 },
    { 0, 0 },
    { 0x35074552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed74556, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e7455e, 23 },
    { 0, 0 },
    { 0xc4274560, 95 },
    { 0, 0 },
    { 0x7e274562, 88 },
    { 0x68e74563, 95 },
    { 0, 0 },
    { 0xc1b74565, 17 },
    { 0, 0 },
    { 0x69974567, 95 },
    { 0, 0 },
    { 0xc0f74569, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf417456c, 90 },
    { 0x95e7456d, 84 },
    { 0xd087456e, 98 },
    { 0x357456f, 95 },
    { 0, 0 },
    { 0x8f374571, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab74574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb74579, 23 },
    { 0xc2a7457a, 43 },
    { 0, 0 },
    { 0xc77457c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8097457f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627458b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd47458f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63374593, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d7459c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71745ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6745b3, 67 },
    { 0xc5a745b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4745ba, 55 },
    { 0x3ac745bb, 23 },
    { 0x6fb745bc, 55 },
    { 0x300745bd, 93 },
    { 0, 0 },
    { 0x138745bf, 93 },
    { 0xa29745c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659745c5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6745cb, 23 },
    { 0xd58745cc, 43 },
    { 0, 0 },
    { 0xbe3745ce, 61 },
    { 0x5be745ce, 93 },
    { 0xf62745d0, 95 },
    { 0xf745ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4745d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160745db, 76 },
    { 0xcf1745db, 95 },
    { 0xd4e745dc, 95 },
    { 0x21b745de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0745e4, 90 },
    { 0xef5745e4, 23 },
    { 0x5be745e6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4745f2, 98 },
    { 0x7c745f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442745fe, 93 },
    { 0xac0745ff, 98 },
    { 0x5a5745ff, 52 },
    { 0, 0 },
    { 0xcba74602, 93 },
    { 0xc4a74602, 95 },
    { 0x3c174604, 95 },
    { 0, 0 },
    { 0x9c474606, 90 },
    { 0, 0 },
    { 0x68974608, 95 },
    { 0, 0 },
    { 0x25a7460a, 91 },
    { 0xd967460b, 55 },
    { 0xecd7460a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d7460f, 78 },
    { 0xb7d74610, 98 },
    { 0x7b574611, 46 },
    { 0, 0 },
    { 0xe5474613, 91 },
    { 0xb2d74614, 46 },
    { 0x59674614, 84 },
    { 0, 0 },
    { 0x3e474617, 65 },
    { 0x5d074618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4797461b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf467461f, 98 },
    { 0, 0 },
    { 0x88474621, 77 },
    { 0x5e74622, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f74625, 69 },
    { 0, 0 },
    { 0xf9174627, 51 },
    { 0, 0 },
    { 0x19574629, 46 },
    { 0x9627462a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07462f, 52 },
    { 0x93f7462f, 61 },
    { 0x50874630, 98 },
    { 0x1f674632, 55 },
    { 0x9c774630, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb74637, 90 },
    { 0x13074638, 98 },
    { 0xade74639, 100 },
    { 0x8017463a, 93 },
    { 0x9d77463b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb87463e, 66 },
    { 0x5627463f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a74648, 55 },
    { 0x75574649, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d7464d, 55 },
    { 0x8ef7464e, 90 },
    { 0xee77464d, 55 },
    { 0xd167464d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b74658, 55 },
    { 0x9c574659, 33 },
    { 0xabb7465a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757465d, 99 },
    { 0x1177465d, 23 },
    { 0x3b47465f, 61 },
    { 0x6c77465f, 100 },
    { 0xd4674661, 90 },
    { 0xfb574662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa74669, 85 },
    { 0x4377466a, 95 },
    { 0xc2f7466a, 33 },
    { 0, 0 },
    { 0x61a7466d, 95 },
    { 0, 0 },
    { 0x1197466f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc974673, 33 },
    { 0x4f974673, 64 },
    { 0xd7d74673, 95 },
    { 0xccf74676, 55 },
    { 0x9ca74677, 33 },
    { 0x2d74678, 61 },
    { 0x19774679, 66 },
    { 0xd07467a, 90 },
    { 0x48c7467a, 76 },
    { 0xb877467c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11374685, 55 },
    { 0, 0 },
    { 0xc4874687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657468b, 88 },
    { 0xa4f7468c, 66 },
    { 0xc737468d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5974692, 90 },
    { 0x6a574692, 33 },
    { 0x7e374694, 8 },
    { 0x44074692, 95 },
    { 0x63974695, 88 },
    { 0x5cf74697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547469a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957469e, 99 },
    { 0x1a07469f, 76 },
    { 0x57a7469f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45746a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4746ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6746b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772746b8, 98 },
    { 0x4b7746b8, 90 },
    { 0x7746b9, 91 },
    { 0xbc0746ba, 33 },
    { 0xe57746b8, 33 },
    { 0x851746b8, 95 },
    { 0x850746b8, 46 },
    { 0x265746b9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0746c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146746ca, 95 },
    { 0x16a746cb, 68 },
    { 0xe1b746cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0746cf, 43 },
    { 0, 0 },
    { 0xc36746d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348746d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x896746d8, 90 },
    { 0, 0 },
    { 0x1c0746da, 90 },
    { 0, 0 },
    { 0x887746dc, 98 },
    { 0xdf9746dc, 100 },
    { 0, 0 },
    { 0xf8e746df, 23 },
    { 0, 0 },
    { 0x7c7746e1, 98 },
    { 0xe6a746e2, 23 },
    { 0x91a746e3, 88 },
    { 0x5746e2, 33 },
    { 0xd88746e4, 66 },
    { 0xbd4746e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28746ec, 61 },
    { 0, 0 },
    { 0x503746ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662746f4, 33 },
    { 0x9e746f5, 90 },
    { 0, 0 },
    { 0x9a6746f7, 98 },
    { 0, 0 },
    { 0xb00746f9, 66 },
    { 0, 0 },
    { 0x324746fb, 84 },
    { 0xc46746fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f746ff, 61 },
    { 0xb83746ff, 33 },
    { 0x97974701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b74705, 65 },
    { 0x10a74706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b7470a, 61 },
    { 0x8d7470b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a574714, 90 },
    { 0x8c974715, 95 },
    { 0xc874714, 98 },
    { 0xa4074714, 61 },
    { 0, 0 },
    { 0xdbe74719, 33 },
    { 0, 0 },
    { 0x2757471b, 77 },
    { 0x43b7471c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f874725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7472a, 91 },
    { 0x4267472b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37574730, 34 },
    { 0x3e674731, 95 },
    { 0x32f74731, 95 },
    { 0, 0 },
    { 0xbc74734, 23 },
    { 0xe5574734, 61 },
    { 0x5c074734, 55 },
    { 0, 0 },
    { 0x7f74738, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17474740, 98 },
    { 0x75b74741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d74746, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7474b, 33 },
    { 0xcf17474c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10574751, 52 },
    { 0x14774751, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9417476b, 90 },
    { 0x7727476c, 93 },
    { 0x3ae7476c, 33 },
    { 0xe0b7476c, 51 },
    { 0, 0 },
    { 0xda574770, 33 },
    { 0xfdc74771, 98 },
    { 0x1a674770, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe74775, 100 },
    { 0, 0 },
    { 0xcc674777, 76 },
    { 0x3d474778, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f7477d, 38 },
    { 0, 0 },
    { 0x3b37477f, 48 },
    { 0x2f874780, 95 },
    { 0, 0 },
    { 0x1a674782, 61 },
    { 0, 0 },
    { 0xcf374784, 77 },
    { 0xa8574785, 33 },
    { 0xd4b74786, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17478d, 95 },
    { 0xd7c7478e, 100 },
    { 0, 0 },
    { 0x25274790, 98 },
    { 0x2c174791, 81 },
    { 0x4d774791, 87 },
    { 0, 0 },
    { 0xcd074794, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4887479b, 43 },
    { 0x34c7479c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8747a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2747a7, 98 },
    { 0xd8b747a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x532747ab, 87 },
    { 0, 0 },
    { 0xed9747ad, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9747b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46747b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86747b9, 68 },
    { 0xe4a747ba, 61 },
    { 0x13e747b9, 43 },
    { 0x637747bc, 36 },
    { 0xdd747b9, 93 },
    { 0x705747b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e747cb, 17 },
    { 0x8cf747cb, 55 },
    { 0x26747cd, 34 },
    { 0x7f9747cd, 98 },
    { 0x410747cf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3747d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2747d6, 90 },
    { 0xd2b747d7, 88 },
    { 0x333747d8, 23 },
    { 0xc5747d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161747df, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed747e6, 93 },
    { 0x686747e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6747ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20747f0, 33 },
    { 0, 0 },
    { 0x458747f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f747f5, 61 },
    { 0xab3747f6, 67 },
    { 0, 0 },
    { 0xa00747f8, 90 },
    { 0x508747f9, 67 },
    { 0xbe2747fa, 75 },
    { 0x336747fb, 55 },
    { 0xfa6747fc, 66 },
    { 0x6bb747fb, 66 },
    { 0xd7e747fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa74801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b74804, 90 },
    { 0, 0 },
    { 0xc0274806, 90 },
    { 0x57774807, 98 },
    { 0, 0 },
    { 0x3e174809, 55 },
    { 0xcaf7480a, 95 },
    { 0, 0 },
    { 0x6f87480c, 61 },
    { 0xd0b7480d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074810, 81 },
    { 0x3674811, 84 },
    { 0xdb874812, 98 },
    { 0, 0 },
    { 0x95174814, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117481a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7481e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca574822, 91 },
    { 0x1e374822, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb357482a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf074830, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb874834, 93 },
    { 0, 0 },
    { 0xd0a74836, 95 },
    { 0x15174837, 55 },
    { 0xb8d74836, 33 },
    { 0xa7874839, 99 },
    { 0x86a7483a, 88 },
    { 0x2d74838, 61 },
    { 0, 0 },
    { 0xf077483d, 88 },
    { 0x3d7483d, 95 },
    { 0xae7483f, 100 },
    { 0xa3174840, 55 },
    { 0xedf74841, 33 },
    { 0, 0 },
    { 0xaa174843, 100 },
    { 0x7bd74843, 75 },
    { 0, 0 },
    { 0xda074846, 40 },
    { 0, 0 },
    { 0xcc774848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7484b, 95 },
    { 0x5c97484b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7484f, 33 },
    { 0xad974850, 93 },
    { 0, 0 },
    { 0x8c074852, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf74855, 61 },
    { 0x1ae74856, 61 },
    { 0xe7774857, 81 },
    { 0x38d74858, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4227485c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d974860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df74865, 78 },
    { 0x27774866, 90 },
    { 0xd9b74866, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7486c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07486f, 48 },
    { 0, 0 },
    { 0x50e74871, 55 },
    { 0, 0 },
    { 0x26f74873, 55 },
    { 0, 0 },
    { 0x97774875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9947487f, 33 },
    { 0x10b74880, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad874888, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7488c, 23 },
    { 0x7517488d, 43 },
    { 0x45c7488e, 46 },
    { 0x6fe7488f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7274892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c74895, 81 },
    { 0x39674896, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc77489b, 76 },
    { 0xb6c7489c, 93 },
    { 0x2567489d, 100 },
    { 0xb667489e, 67 },
    { 0, 0 },
    { 0xbd3748a0, 78 },
    { 0x3d6748a0, 43 },
    { 0xdd6748a2, 98 },
    { 0x588748a2, 98 },
    { 0x893748a2, 55 },
    { 0, 0 },
    { 0x326748a6, 23 },
    { 0, 0 },
    { 0xf17748a8, 98 },
    { 0, 0 },
    { 0xe87748aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4748ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc748b0, 91 },
    { 0xbf0748b0, 99 },
    { 0x7c5748b0, 98 },
    { 0x57748b2, 87 },
    { 0xe10748b1, 23 },
    { 0xbbe748b5, 77 },
    { 0xe21748b6, 90 },
    { 0, 0 },
    { 0x447748b8, 98 },
    { 0, 0 },
    { 0x40f748ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293748c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c748c4, 97 },
    { 0x982748c5, 75 },
    { 0x503748c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2748c9, 61 },
    { 0x3d3748c9, 87 },
    { 0xed4748cb, 66 },
    { 0xeb5748cb, 90 },
    { 0x7ba748c9, 33 },
    { 0x244748ce, 76 },
    { 0x806748cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d748d5, 66 },
    { 0xb71748d6, 100 },
    { 0, 0 },
    { 0x27c748d8, 23 },
    { 0x8dd748d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94748dd, 85 },
    { 0x4c9748dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4748e2, 98 },
    { 0x2d748e3, 66 },
    { 0xb6c748e4, 46 },
    { 0xee3748e3, 55 },
    { 0x1d1748e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc748ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f748f6, 52 },
    { 0, 0 },
    { 0x55d748f8, 33 },
    { 0x77d748f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7748fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1274900, 65 },
    { 0xcb874901, 78 },
    { 0x6f674901, 33 },
    { 0x88f74901, 90 },
    { 0x40a74904, 33 },
    { 0xebc74904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54574908, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67490c, 98 },
    { 0x7f87490d, 75 },
    { 0xb0c7490e, 52 },
    { 0xc9a7490f, 23 },
    { 0x79f74910, 95 },
    { 0x974910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb74918, 99 },
    { 0, 0 },
    { 0x5997491a, 50 },
    { 0x2487491b, 98 },
    { 0x4dd7491c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b274920, 33 },
    { 0x6f974920, 61 },
    { 0xc0074922, 81 },
    { 0x51a74923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63874929, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7007492c, 78 },
    { 0x92b7492d, 98 },
    { 0xa8a7492e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e474932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e74937, 67 },
    { 0x73e74938, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a7493b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b7493f, 100 },
    { 0xe0e7493f, 100 },
    { 0, 0 },
    { 0xd0c74942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e674945, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127494a, 61 },
    { 0x5ed7494b, 63 },
    { 0x45b7494c, 76 },
    { 0xe3f7494a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11274950, 33 },
    { 0x8fb74951, 100 },
    { 0x95274952, 33 },
    { 0xcf74951, 33 },
    { 0xade74954, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc874958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e37495b, 76 },
    { 0x5d47495c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7307495f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed74962, 34 },
    { 0xd2a74963, 40 },
    { 0xf2e74964, 90 },
    { 0x3c074965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8274968, 55 },
    { 0, 0 },
    { 0x7957496a, 61 },
    { 0x3207496a, 91 },
    { 0x6227496c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34374970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4674973, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb437497d, 52 },
    { 0xf397497e, 61 },
    { 0x55c7497f, 61 },
    { 0x43d7497d, 98 },
    { 0, 0 },
    { 0x6a874982, 100 },
    { 0x76f74982, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5674987, 84 },
    { 0, 0 },
    { 0xfa174989, 33 },
    { 0xb157498a, 34 },
    { 0x5827498a, 33 },
    { 0, 0 },
    { 0xf007498d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d874998, 34 },
    { 0, 0 },
    { 0xef27499a, 55 },
    { 0x37d7499b, 87 },
    { 0x6237499a, 90 },
    { 0, 0 },
    { 0x4327499e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad749a5, 80 },
    { 0x67d749a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0749a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c749ac, 61 },
    { 0x16b749ad, 55 },
    { 0, 0 },
    { 0x2bf749af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x378749b2, 23 },
    { 0xc4e749b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65749b7, 24 },
    { 0xf47749b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x606749bb, 81 },
    { 0x9c1749bc, 65 },
    { 0x901749bb, 46 },
    { 0x261749be, 43 },
    { 0x5f9749bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3749c2, 90 },
    { 0x152749c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93749c9, 95 },
    { 0x7cc749c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653749d1, 99 },
    { 0xe6749d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8749d5, 85 },
    { 0, 0 },
    { 0x6a1749d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388749de, 66 },
    { 0x83c749df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e749e4, 33 },
    { 0xc63749e5, 98 },
    { 0x89749e6, 61 },
    { 0x340749e4, 61 },
    { 0x46b749e5, 68 },
    { 0x181749e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b749ec, 43 },
    { 0, 0 },
    { 0xfc9749ee, 91 },
    { 0xc1749ef, 90 },
    { 0x1f2749f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1749f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3749fa, 61 },
    { 0x87749fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x261749fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b74a01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55774a05, 98 },
    { 0xe9b74a06, 61 },
    { 0, 0 },
    { 0x6c974a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf674a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10774a11, 65 },
    { 0, 0 },
    { 0xc9574a13, 46 },
    { 0xe9874a14, 33 },
    { 0xe8b74a15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd874a18, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57474a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19974a20, 33 },
    { 0, 0 },
    { 0x9c674a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20574a27, 46 },
    { 0, 0 },
    { 0xb3e74a29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90574a2f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77574a33, 98 },
    { 0x2b474a34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d674a37, 99 },
    { 0xd8274a37, 86 },
    { 0xc5974a39, 100 },
    { 0x86f74a3a, 93 },
    { 0xa2174a3b, 87 },
    { 0xdae74a39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f074a3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca974a42, 84 },
    { 0, 0 },
    { 0x93474a44, 90 },
    { 0xb8174a45, 52 },
    { 0xb2674a46, 90 },
    { 0, 0 },
    { 0x38d74a48, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2874a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e974a4e, 46 },
    { 0, 0 },
    { 0x62574a50, 81 },
    { 0xb7074a51, 45 },
    { 0xc8074a52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68774a55, 34 },
    { 0x46f74a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6274a5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce774a5d, 12 },
    { 0xa974a5e, 94 },
    { 0xbe574a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf874a68, 95 },
    { 0x2b374a69, 33 },
    { 0x78f74a6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27374a6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3574a70, 55 },
    { 0x72074a70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98074a74, 61 },
    { 0xda974a75, 95 },
    { 0xf4474a75, 84 },
    { 0x16174a76, 23 },
    { 0, 0 },
    { 0x8474a79, 90 },
    { 0xb5074a79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf74a80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c174a86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f74a8c, 61 },
    { 0xa5574a8d, 100 },
    { 0, 0 },
    { 0xa9774a8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e274a92, 95 },
    { 0xe1c74a93, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174a98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82174aa0, 91 },
    { 0xbdd74aa0, 98 },
    { 0x19b74aa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e074aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea74aab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2974aae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0474ab2, 34 },
    { 0, 0 },
    { 0x75674ab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a374aba, 33 },
    { 0x3c274abb, 33 },
    { 0x15774abc, 98 },
    { 0x4fc74abd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x8174ac0, 23 },
    { 0, 0 },
    { 0x1b374ac2, 55 },
    { 0x86d74ac3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c74aca, 43 },
    { 0x7b374aca, 98 },
    { 0x72774aca, 95 },
    { 0xac774acd, 93 },
    { 0xfe674ace, 61 },
    { 0x3ed74aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a74ad2, 76 },
    { 0xf5274ad2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b74ad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4574add, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f74ae0, 33 },
    { 0x53874ae1, 61 },
    { 0x77874ae1, 62 },
    { 0xdf074ae3, 81 },
    { 0x62574ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58574ae7, 76 },
    { 0xbf274ae7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43074aec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9074af1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6974afa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c74b06, 46 },
    { 0x78d74b07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c74b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74b0e, 66 },
    { 0, 0 },
    { 0x73b74b10, 66 },
    { 0xd2074b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d74b16, 95 },
    { 0x15f74b16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae74b1d, 76 },
    { 0xb7874b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec574b21, 55 },
    { 0xa7574b22, 33 },
    { 0xc1774b21, 98 },
    { 0xab674b24, 76 },
    { 0, 0 },
    { 0xa5974b26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f274b2d, 33 },
    { 0, 0 },
    { 0xaf374b2f, 61 },
    { 0xaad74b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2074b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a74b3b, 61 },
    { 0xf074b3c, 66 },
    { 0x8f74b3c, 85 },
    { 0, 0 },
    { 0xe8074b3f, 55 },
    { 0x8c074b3f, 90 },
    { 0xbb074b41, 95 },
    { 0x8b774b42, 97 },
    { 0xb2a74b42, 23 },
    { 0x24474b40, 33 },
    { 0x68a74b44, 98 },
    { 0, 0 },
    { 0x74674b47, 61 },
    { 0xfbc74b47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74b4b, 33 },
    { 0x98a74b4b, 97 },
    { 0, 0 },
    { 0x9bf74b4e, 90 },
    { 0xa6474b4f, 55 },
    { 0x4a74b4f, 66 },
    { 0xc0174b51, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44074b5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a974b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78674b60, 98 },
    { 0, 0 },
    { 0x29774b62, 43 },
    { 0x2f974b63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36474b68, 61 },
    { 0x87874b69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49374b71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f74b77, 90 },
    { 0, 0 },
    { 0x1774b79, 33 },
    { 0, 0 },
    { 0x42274b7b, 61 },
    { 0, 0 },
    { 0x47774b7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8674b83, 2 },
    { 0xe1a74b84, 98 },
    { 0xd7274b84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d74b8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f374b8f, 61 },
    { 0, 0 },
    { 0x1ad74b91, 66 },
    { 0xe7b74b92, 67 },
    { 0, 0 },
    { 0x6f574b94, 68 },
    { 0x12374b94, 98 },
    { 0x9b974b95, 33 },
    { 0x1ad74b97, 55 },
    { 0, 0 },
    { 0x13174b99, 99 },
    { 0, 0 },
    { 0xf9674b9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46974ba0, 98 },
    { 0xe0d74ba0, 33 },
    { 0xa7974ba2, 43 },
    { 0x16774ba3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c274ba7, 81 },
    { 0x7dd74ba8, 95 },
    { 0x75b74ba9, 55 },
    { 0xf3c74ba8, 4 },
    { 0x6ee74bab, 95 },
    { 0x2bf74ba9, 52 },
    { 0x92d74bad, 33 },
    { 0xea74bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f374bb5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d74bbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b74bc1, 66 },
    { 0xdd674bc1, 98 },
    { 0xa874bc1, 84 },
    { 0xd4074bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac674bc7, 33 },
    { 0, 0 },
    { 0x98674bc9, 99 },
    { 0x5ba74bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a474bcd, 23 },
    { 0x42f74bce, 52 },
    { 0xd7974bce, 74 },
    { 0x2d874bce, 99 },
    { 0xa9774bd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f74bd4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2774bda, 61 },
    { 0x2c174bdb, 85 },
    { 0, 0 },
    { 0x9d874bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b74be1, 55 },
    { 0x97074be2, 43 },
    { 0, 0 },
    { 0xad574be4, 55 },
    { 0xc3f74be4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea874be8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76574beb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a874bf7, 61 },
    { 0x41b74bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7874c01, 90 },
    { 0x58e74c01, 95 },
    { 0x7f274c03, 23 },
    { 0xd4a74c04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d174c0d, 98 },
    { 0, 0 },
    { 0x53f74c0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea74c12, 99 },
    { 0x23674c12, 90 },
    { 0x84b74c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b74c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef74c22, 75 },
    { 0, 0 },
    { 0xe8874c24, 93 },
    { 0x57174c25, 95 },
    { 0x7a774c26, 61 },
    { 0, 0 },
    { 0x35374c28, 61 },
    { 0xcf274c28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb74c32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71974c35, 97 },
    { 0x69a74c36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d74c3b, 90 },
    { 0x95874c3c, 77 },
    { 0, 0 },
    { 0x7cf74c3e, 98 },
    { 0, 0 },
    { 0x37274c40, 90 },
    { 0, 0 },
    { 0x27e74c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf74c48, 61 },
    { 0x50074c49, 90 },
    { 0xc974c49, 76 },
    { 0xe5474c49, 99 },
    { 0x1b974c4b, 66 },
    { 0x86b74c4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9374c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d674c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c174c58, 23 },
    { 0xf9774c58, 33 },
    { 0xd8c74c5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd74c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d174c61, 95 },
    { 0x62674c62, 61 },
    { 0x7ac74c63, 34 },
    { 0, 0 },
    { 0xb3174c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c74c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d74c6d, 52 },
    { 0, 0 },
    { 0x9f174c6f, 90 },
    { 0, 0 },
    { 0x8a574c71, 43 },
    { 0, 0 },
    { 0x9674c73, 23 },
    { 0x42174c73, 99 },
    { 0x81374c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22074c78, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7874c7d, 76 },
    { 0x17774c7d, 98 },
    { 0, 0 },
    { 0x9ef74c80, 68 },
    { 0x6bd74c81, 90 },
    { 0, 0 },
    { 0x2bb74c83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16474c87, 76 },
    { 0x70574c88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed974c8b, 33 },
    { 0x8bb74c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe274c93, 61 },
    { 0, 0 },
    { 0xf6774c95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b74c99, 95 },
    { 0x22b74c9a, 78 },
    { 0, 0 },
    { 0x20774c9c, 61 },
    { 0xedf74c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c874ca5, 33 },
    { 0x47a74ca5, 75 },
    { 0x29c74ca6, 55 },
    { 0x7ff74ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b74cab, 76 },
    { 0xd1e74cab, 23 },
    { 0x51b74cad, 61 },
    { 0xcd74cae, 100 },
    { 0x82374cad, 23 },
    { 0xd174cb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0774cb6, 95 },
    { 0, 0 },
    { 0xfea74cb8, 76 },
    { 0xce074cb9, 33 },
    { 0xb4a74cb8, 90 },
    { 0x5b274cb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5574cc0, 97 },
    { 0, 0 },
    { 0x22774cc2, 55 },
    { 0xa9974cc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e74cc7, 81 },
    { 0x2c174cc7, 43 },
    { 0xd5c74cc9, 90 },
    { 0, 0 },
    { 0x9cc74ccb, 66 },
    { 0, 0 },
    { 0xc7074ccd, 87 },
    { 0x9b74cce, 87 },
    { 0xcff74ccf, 75 },
    { 0, 0 },
    { 0x80374cd1, 52 },
    { 0xde674cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6974cd5, 98 },
    { 0x74174cd5, 61 },
    { 0, 0 },
    { 0x97d74cd8, 90 },
    { 0, 0 },
    { 0x1c074cda, 95 },
    { 0xbc174cdb, 23 },
    { 0xc5874cdc, 91 },
    { 0xf8674cdd, 33 },
    { 0x8b674cdb, 98 },
    { 0x17274cdd, 23 },
    { 0x5f874cdf, 90 },
    { 0, 0 },
    { 0x5b574ce2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a74ce6, 46 },
    { 0, 0 },
    { 0x3874ce8, 77 },
    { 0, 0 },
    { 0xf1974cea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8474cf0, 97 },
    { 0x75f74cf1, 33 },
    { 0xcb274cf0, 52 },
    { 0xdb074cf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed274cf7, 52 },
    { 0, 0 },
    { 0xccd74cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f74cfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7274d08, 61 },
    { 0, 0 },
    { 0xef674d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674d14, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a74d1b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x474d1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b374d22, 90 },
    { 0x6b674d22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a974d26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac174d2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8674d39, 33 },
    { 0x68e74d3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25074d3d, 46 },
    { 0x5e574d3d, 98 },
    { 0xb8f74d3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80574d43, 61 },
    { 0, 0 },
    { 0x61574d45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc74d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62774d4e, 61 },
    { 0, 0 },
    { 0x68974d50, 55 },
    { 0xd1574d50, 90 },
    { 0x43a74d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e74d58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63774d5e, 38 },
    { 0xb9974d5f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed74d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd74d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66674d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d574d6d, 96 },
    { 0x15874d6e, 95 },
    { 0xcff74d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d76, 23 },
    { 0x6b974d77, 23 },
    { 0xe8374d76, 95 },
    { 0, 0 },
    { 0xfde74d7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf74d7e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb274d83, 51 },
    { 0x20874d84, 98 },
    { 0x5ef74d84, 33 },
    { 0x1a774d84, 38 },
    { 0xb1674d86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4074d91, 23 },
    { 0, 0 },
    { 0xc374d93, 23 },
    { 0xf2f74d93, 43 },
    { 0x70b74d95, 23 },
    { 0xc774d96, 67 },
    { 0x78f74d97, 90 },
    { 0x7e374d98, 98 },
    { 0x28274d98, 90 },
    { 0, 0 },
    { 0x7874d9b, 90 },
    { 0xcbc74d9b, 90 },
    { 0x4a74d9d, 90 },
    { 0, 0 },
    { 0xe8d74d9f, 91 },
    { 0x2f874d9f, 23 },
    { 0x7674d9f, 81 },
    { 0x6974da2, 99 },
    { 0x9ba74da3, 90 },
    { 0x24174da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec074da8, 61 },
    { 0x58374da9, 93 },
    { 0x34874daa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a74db1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e774db5, 23 },
    { 0, 0 },
    { 0x56674db7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff274dc4, 98 },
    { 0x22374dc5, 46 },
    { 0, 0 },
    { 0x75274dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69474dcc, 43 },
    { 0x97f74dcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb74dd1, 95 },
    { 0xda974dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9874dd7, 98 },
    { 0xa2074dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77774ddd, 84 },
    { 0, 0 },
    { 0x18474ddf, 90 },
    { 0, 0 },
    { 0x88e74de1, 85 },
    { 0, 0 },
    { 0x70574de3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3774ded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57974df1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a574df6, 65 },
    { 0x77a74df7, 61 },
    { 0x5d374df6, 66 },
    { 0, 0 },
    { 0x89c74dfa, 61 },
    { 0xdd974dfb, 90 },
    { 0x6c874dfc, 61 },
    { 0x96274dfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5074e02, 81 },
    { 0xd6d74e03, 66 },
    { 0, 0 },
    { 0xf6e74e05, 90 },
    { 0, 0 },
    { 0x4f274e07, 34 },
    { 0, 0 },
    { 0x774e09, 61 },
    { 0, 0 },
    { 0xd7a74e0b, 23 },
    { 0xcb374e0c, 90 },
    { 0, 0 },
    { 0xbba74e0e, 85 },
    { 0xab974e0f, 34 },
    { 0x60b74e10, 76 },
    { 0, 0 },
    { 0xcd574e12, 68 },
    { 0x84e74e13, 68 },
    { 0, 0 },
    { 0x8e74e15, 33 },
    { 0xfed74e16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c74e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27574e21, 90 },
    { 0x67174e22, 33 },
    { 0x66f74e23, 76 },
    { 0x17274e24, 90 },
    { 0x74374e24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf574e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b674e2f, 33 },
    { 0x34174e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf74e33, 33 },
    { 0, 0 },
    { 0x5fb74e35, 90 },
    { 0x70e74e35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb74e39, 34 },
    { 0x51a74e39, 93 },
    { 0x48474e3b, 67 },
    { 0x38774e3b, 43 },
    { 0xa3674e3d, 90 },
    { 0, 0 },
    { 0x35c74e3f, 33 },
    { 0x1d674e40, 84 },
    { 0x1f574e41, 90 },
    { 0x2cf74e41, 95 },
    { 0xb8574e41, 46 },
    { 0x4c274e40, 43 },
    { 0, 0 },
    { 0xa1174e46, 76 },
    { 0x7c674e46, 34 },
    { 0x45374e48, 85 },
    { 0x1e274e48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b74e4c, 90 },
    { 0x6f174e4d, 90 },
    { 0x96774e4e, 61 },
    { 0xa9a74e4f, 23 },
    { 0x98c74e4d, 33 },
    { 0x65c74e4d, 90 },
    { 0x8be74e52, 90 },
    { 0x77574e53, 33 },
    { 0, 0 },
    { 0xae374e55, 34 },
    { 0x3f974e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d74e59, 33 },
    { 0xa4e74e5a, 23 },
    { 0, 0 },
    { 0x90974e5c, 93 },
    { 0xaaf74e5d, 99 },
    { 0x2ca74e5e, 33 },
    { 0xfed74e5f, 23 },
    { 0x5b674e5d, 98 },
    { 0x8d74e61, 78 },
    { 0xe2774e5e, 33 },
    { 0, 0 },
    { 0x5bc74e64, 101 },
    { 0x5af74e65, 55 },
    { 0, 0 },
    { 0xa6d74e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c174e6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8974e73, 90 },
    { 0xff174e73, 95 },
    { 0xa3974e73, 90 },
    { 0x22774e73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e74e7d, 87 },
    { 0, 0 },
    { 0xa3674e7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a74e83, 98 },
    { 0xdc074e84, 78 },
    { 0xd0674e85, 33 },
    { 0x30374e85, 23 },
    { 0x15a74e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68774e8b, 61 },
    { 0xe5974e8c, 95 },
    { 0, 0 },
    { 0xc0f74e8e, 98 },
    { 0x6d374e8f, 61 },
    { 0, 0 },
    { 0xcda74e91, 78 },
    { 0, 0 },
    { 0xf0c74e93, 83 },
    { 0, 0 },
    { 0x8e574e95, 96 },
    { 0, 0 },
    { 0x9a974e97, 33 },
    { 0, 0 },
    { 0x8a74e99, 90 },
    { 0xf1374e9a, 61 },
    { 0xd9274e9a, 95 },
    { 0x3be74e9b, 98 },
    { 0x30674e9d, 23 },
    { 0x83574e9e, 81 },
    { 0x23174e9f, 34 },
    { 0x8cb74e9e, 98 },
    { 0x35c74ea1, 23 },
    { 0x43974e9e, 75 },
    { 0xc5f74ea3, 88 },
    { 0xa6a74ea4, 85 },
    { 0x1f774ea5, 55 },
    { 0xc2f74ea6, 90 },
    { 0xc4b74ea7, 55 },
    { 0x3b374e9d, 17 },
    { 0x3d274ea6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474ead, 17 },
    { 0xb7274ead, 23 },
    { 0, 0 },
    { 0x68a74eb0, 90 },
    { 0x4cd74eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f574eb7, 34 },
    { 0xbdc74eb8, 98 },
    { 0x8b074eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71874ebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73074ec2, 36 },
    { 0, 0 },
    { 0x39c74ec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3574ec7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc074ecc, 98 },
    { 0x7bf74ecd, 98 },
    { 0xed974ecd, 40 },
    { 0x39274ecf, 90 },
    { 0x9b674ed0, 23 },
    { 0x9d174ed1, 90 },
    { 0x1c874ed2, 33 },
    { 0, 0 },
    { 0xd8574ed4, 95 },
    { 0x1bf74ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874ed8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64874ee2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a74ee6, 93 },
    { 0x90174ee7, 85 },
    { 0x17374ee8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc74eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e74eef, 76 },
    { 0xad174eef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b074ef3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66674efa, 98 },
    { 0xb1174efa, 98 },
    { 0x6db74efc, 93 },
    { 0x47774efa, 46 },
    { 0x99e74efe, 61 },
    { 0x23f74eff, 98 },
    { 0x7c774f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a74f08, 93 },
    { 0x48274f09, 90 },
    { 0x73874f0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd74f0e, 98 },
    { 0, 0 },
    { 0xdb074f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf74f14, 76 },
    { 0, 0 },
    { 0xc0c74f16, 90 },
    { 0xea274f16, 88 },
    { 0x99e74f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4474f1f, 33 },
    { 0xbe774f20, 43 },
    { 0x90c74f21, 23 },
    { 0x7974f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a74f25, 65 },
    { 0xb4974f26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd974f30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91574f33, 23 },
    { 0xc6474f34, 90 },
    { 0xa8874f34, 33 },
    { 0x62a74f34, 98 },
    { 0x82c74f37, 98 },
    { 0x274f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56674f3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4374f3e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e74f42, 66 },
    { 0, 0 },
    { 0x62b74f44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e574f47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d74f4a, 33 },
    { 0x7d774f4b, 95 },
    { 0, 0 },
    { 0x86374f4d, 61 },
    { 0xa9974f4e, 95 },
    { 0xb8574f4f, 93 },
    { 0xa3474f50, 52 },
    { 0, 0 },
    { 0xe1374f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b74f57, 33 },
    { 0xbac74f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1074f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c074f5e, 93 },
    { 0xdea74f5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4674f63, 23 },
    { 0xa5e74f63, 90 },
    { 0, 0 },
    { 0x76074f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf74f69, 61 },
    { 0xd7174f69, 61 },
    { 0x8a774f6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d74f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c074f78, 46 },
    { 0xdd174f79, 98 },
    { 0xc9f74f78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a574f80, 98 },
    { 0, 0 },
    { 0xdb674f82, 61 },
    { 0x61f74f83, 77 },
    { 0x97374f84, 61 },
    { 0xb6c74f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb74f89, 95 },
    { 0x69c74f89, 90 },
    { 0, 0 },
    { 0x48b74f8c, 76 },
    { 0, 0 },
    { 0xb0774f8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6774f95, 98 },
    { 0x81774f96, 95 },
    { 0xe1174f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a74f9f, 100 },
    { 0x13b74f9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99274fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8974fac, 61 },
    { 0xccd74fad, 66 },
    { 0x65a74fae, 98 },
    { 0x87374fae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d674fb2, 90 },
    { 0x42074fb3, 95 },
    { 0x6d74fb4, 33 },
    { 0x3e574fb5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c74fb8, 90 },
    { 0x9bc74fb8, 33 },
    { 0x50374fba, 98 },
    { 0x80774fb9, 52 },
    { 0, 0 },
    { 0xf6374fbd, 87 },
    { 0x94074fbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af74fc2, 34 },
    { 0x23874fc2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed174fc7, 85 },
    { 0x5bd74fc8, 76 },
    { 0x2d674fc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d74fcd, 93 },
    { 0x37574fcd, 46 },
    { 0, 0 },
    { 0xdaa74fd0, 33 },
    { 0, 0 },
    { 0x45e74fd2, 23 },
    { 0xbb074fd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e574fe1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa74fe5, 23 },
    { 0, 0 },
    { 0xf3a74fe7, 98 },
    { 0xa2474fe7, 90 },
    { 0x7e74fe8, 91 },
    { 0x8474fe7, 23 },
    { 0x4fa74fea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56774ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80274ff8, 55 },
    { 0, 0 },
    { 0x79074ffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63674ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c75002, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa075006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc275009, 61 },
    { 0x8dc7500a, 66 },
    { 0x2087500b, 78 },
    { 0xd3c7500a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc267500f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46275012, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb775015, 95 },
    { 0xeb775016, 85 },
    { 0, 0 },
    { 0x30175018, 33 },
    { 0xa9c75019, 33 },
    { 0xf4475019, 98 },
    { 0x8207501b, 23 },
    { 0, 0 },
    { 0x14c7501d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa75021, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f75024, 95 },
    { 0x19c75025, 90 },
    { 0x37475026, 33 },
    { 0xfcc75027, 33 },
    { 0x31375028, 98 },
    { 0x30e75029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd427502f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf175033, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x25175036, 95 },
    { 0x41075037, 91 },
    { 0xca575038, 55 },
    { 0x60475038, 33 },
    { 0x88875039, 55 },
    { 0x40c7503b, 85 },
    { 0x47c7503c, 33 },
    { 0x1d07503b, 90 },
    { 0xcf57503e, 33 },
    { 0xcd97503b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94175043, 95 },
    { 0x37075044, 55 },
    { 0, 0 },
    { 0xb9e75046, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4175049, 90 },
    { 0xa5d7504a, 33 },
    { 0xca37504b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d75055, 88 },
    { 0, 0 },
    { 0xf4e75057, 78 },
    { 0xd7f75058, 98 },
    { 0xa4975059, 100 },
    { 0, 0 },
    { 0x26d7505b, 33 },
    { 0xe2d7505c, 34 },
    { 0x1697505d, 9 },
    { 0x72d7505b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a75062, 55 },
    { 0, 0 },
    { 0xbcf75064, 90 },
    { 0, 0 },
    { 0xf475066, 46 },
    { 0xc3075067, 43 },
    { 0x8af75068, 55 },
    { 0xa0075066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a7506c, 76 },
    { 0x8c77506d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91875072, 34 },
    { 0, 0 },
    { 0xb75074, 55 },
    { 0, 0 },
    { 0xf8775076, 33 },
    { 0xc5d75077, 23 },
    { 0x15d75078, 91 },
    { 0x9ac75077, 90 },
    { 0x13f75077, 23 },
    { 0x5747507b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc937507e, 33 },
    { 0x1247507f, 90 },
    { 0x47475080, 33 },
    { 0x6e675080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64575084, 43 },
    { 0xd7b75085, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7508b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947508f, 23 },
    { 0x7c575090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc375094, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d75098, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37509b, 65 },
    { 0, 0 },
    { 0x16c7509d, 77 },
    { 0x94d7509e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4750a6, 100 },
    { 0xb4d750a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf750be, 35 },
    { 0xde5750bf, 33 },
    { 0xe89750c0, 98 },
    { 0, 0 },
    { 0x168750c2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08750c7, 87 },
    { 0xdd4750c8, 76 },
    { 0, 0 },
    { 0xff5750ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157750ce, 55 },
    { 0, 0 },
    { 0x291750d0, 33 },
    { 0xc4c750d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x491750d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a750d7, 98 },
    { 0xb29750d7, 66 },
    { 0x95f750d9, 34 },
    { 0, 0 },
    { 0x448750db, 78 },
    { 0xcfa750dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0750df, 61 },
    { 0x8c3750df, 94 },
    { 0x158750e1, 23 },
    { 0xd9750e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984750e8, 90 },
    { 0xdf750e9, 52 },
    { 0xc08750ea, 98 },
    { 0xfe8750ea, 90 },
    { 0xc33750e9, 46 },
    { 0x68c750ed, 33 },
    { 0x65c750ec, 61 },
    { 0x731750ef, 78 },
    { 0x32b750f0, 85 },
    { 0xb05750f1, 61 },
    { 0xb80750f1, 90 },
    { 0xb53750ef, 94 },
    { 0xabb750f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245750fc, 87 },
    { 0, 0 },
    { 0x5dc750fe, 69 },
    { 0xd80750fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d75102, 95 },
    { 0, 0 },
    { 0x63f75104, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a175108, 76 },
    { 0, 0 },
    { 0xc357510a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc07510d, 95 },
    { 0xba87510d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea975113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e75119, 90 },
    { 0, 0 },
    { 0x2f47511b, 90 },
    { 0xeeb7511c, 98 },
    { 0x2057511c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48275120, 61 },
    { 0x17375121, 90 },
    { 0x19475121, 95 },
    { 0x31d75120, 34 },
    { 0xea475122, 33 },
    { 0xd075124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf637512a, 90 },
    { 0x3f07512a, 78 },
    { 0, 0 },
    { 0x1b17512d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96175130, 98 },
    { 0x9ad75131, 34 },
    { 0, 0 },
    { 0x55e75133, 43 },
    { 0x4ef75134, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x34175137, 33 },
    { 0x84175138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1317513b, 90 },
    { 0, 0 },
    { 0x43b7513d, 61 },
    { 0, 0 },
    { 0x1a97513f, 98 },
    { 0xf1475140, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87575143, 23 },
    { 0xe4475144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df75149, 43 },
    { 0, 0 },
    { 0xb8b7514b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b7514e, 98 },
    { 0, 0 },
    { 0xe2575150, 98 },
    { 0, 0 },
    { 0x6a575152, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59375155, 93 },
    { 0, 0 },
    { 0xad575157, 33 },
    { 0xca675158, 90 },
    { 0x68375158, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5087515c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b75160, 98 },
    { 0, 0 },
    { 0xb675162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab75166, 77 },
    { 0, 0 },
    { 0xd1c75168, 99 },
    { 0xd0175169, 91 },
    { 0x907516a, 33 },
    { 0xb267516b, 61 },
    { 0x28775169, 33 },
    { 0xa227516d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60975170, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d775174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f475177, 61 },
    { 0x6cc75177, 33 },
    { 0x30d75179, 76 },
    { 0x9d7517a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f97517d, 90 },
    { 0xd97517e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e75184, 23 },
    { 0xed975185, 98 },
    { 0xba475185, 66 },
    { 0x91b75186, 81 },
    { 0xce175184, 76 },
    { 0xf1075187, 76 },
    { 0x2b47518a, 76 },
    { 0x7bb7518b, 76 },
    { 0, 0 },
    { 0x6117518d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c675194, 23 },
    { 0xe9175195, 43 },
    { 0xcb175196, 76 },
    { 0, 0 },
    { 0x7fd75198, 98 },
    { 0, 0 },
    { 0x5eb7519a, 61 },
    { 0, 0 },
    { 0xd757519c, 46 },
    { 0, 0 },
    { 0x5367519e, 23 },
    { 0, 0 },
    { 0x715751a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6751a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3751aa, 82 },
    { 0xa88751ab, 87 },
    { 0x7b9751ac, 61 },
    { 0xdfe751ad, 45 },
    { 0x25f751ae, 90 },
    { 0, 0 },
    { 0xcfa751b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8751b7, 88 },
    { 0xca4751b8, 55 },
    { 0xcd0751b7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e751c0, 95 },
    { 0x289751c1, 90 },
    { 0x924751c0, 78 },
    { 0x14b751c3, 23 },
    { 0, 0 },
    { 0x280751c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff751c9, 33 },
    { 0x69c751ca, 76 },
    { 0xbe7751cb, 33 },
    { 0, 0 },
    { 0xf5e751cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c751d3, 81 },
    { 0xbca751d4, 90 },
    { 0x74751d5, 23 },
    { 0x5bb751d6, 34 },
    { 0xe66751d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29751df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff751e4, 95 },
    { 0, 0 },
    { 0x248751e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea751ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5751ed, 52 },
    { 0xcd2751ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67751f2, 61 },
    { 0, 0 },
    { 0xb6c751f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77751f8, 87 },
    { 0x21d751f9, 78 },
    { 0xf69751fa, 33 },
    { 0xab5751fa, 53 },
    { 0x92a751f9, 33 },
    { 0, 0 },
    { 0xa8f751fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc75204, 66 },
    { 0, 0 },
    { 0xc7875206, 76 },
    { 0xf9675206, 98 },
    { 0, 0 },
    { 0xad275209, 66 },
    { 0xd547520a, 100 },
    { 0x82e75209, 88 },
    { 0x137520c, 40 },
    { 0, 0 },
    { 0xbb27520e, 90 },
    { 0xa297520f, 23 },
    { 0x35a75210, 33 },
    { 0, 0 },
    { 0xe5f75212, 90 },
    { 0x7f75212, 98 },
    { 0xa1c75214, 90 },
    { 0x12575212, 61 },
    { 0xe5775212, 90 },
    { 0x5d875217, 43 },
    { 0x9b675218, 81 },
    { 0x32a75217, 76 },
    { 0xb1475219, 100 },
    { 0x1cf7521b, 46 },
    { 0x8557521b, 95 },
    { 0xd407521b, 93 },
    { 0xee7521b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc75221, 76 },
    { 0xba575222, 98 },
    { 0xf3d75221, 90 },
    { 0x6ab75224, 67 },
    { 0xd5c75225, 95 },
    { 0, 0 },
    { 0x1675227, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a7522c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17522f, 46 },
    { 0x4f7522f, 78 },
    { 0, 0 },
    { 0x3de75232, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87775237, 93 },
    { 0, 0 },
    { 0xab575239, 93 },
    { 0x7107523a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee75241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8675245, 23 },
    { 0x79175246, 93 },
    { 0x57e75245, 23 },
    { 0xc0575246, 34 },
    { 0, 0 },
    { 0x4807524a, 84 },
    { 0, 0 },
    { 0x23c7524c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9197524f, 90 },
    { 0, 0 },
    { 0x5d175251, 98 },
    { 0xdc675252, 75 },
    { 0, 0 },
    { 0xb7d75254, 87 },
    { 0x6bc75254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a75259, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea175265, 23 },
    { 0x1e75266, 55 },
    { 0xc4975266, 55 },
    { 0x9ee75268, 77 },
    { 0xf5775269, 34 },
    { 0x5767526a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07526e, 52 },
    { 0, 0 },
    { 0x56c75270, 90 },
    { 0, 0 },
    { 0xfb375272, 90 },
    { 0x8f175272, 43 },
    { 0x3e75273, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf067527b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd75284, 85 },
    { 0x64975284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c75288, 67 },
    { 0xe4075289, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e27528c, 67 },
    { 0x7777528d, 90 },
    { 0x35a7528e, 61 },
    { 0x65f7528f, 33 },
    { 0xb477528f, 92 },
    { 0, 0 },
    { 0x9f675292, 90 },
    { 0x4f875293, 33 },
    { 0, 0 },
    { 0x4c875295, 98 },
    { 0xaa575295, 77 },
    { 0x57f75296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37529d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1752a2, 90 },
    { 0xe40752a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4752ad, 33 },
    { 0x55a752ae, 34 },
    { 0x2cd752af, 65 },
    { 0xa4752b0, 23 },
    { 0xa85752b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524752b4, 55 },
    { 0xfb752b5, 90 },
    { 0, 0 },
    { 0xd72752b7, 87 },
    { 0, 0 },
    { 0x280752b9, 55 },
    { 0x346752b9, 98 },
    { 0, 0 },
    { 0x6b9752bc, 77 },
    { 0xa15752bc, 83 },
    { 0, 0 },
    { 0xee5752bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f752c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x788752c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80752d1, 34 },
    { 0x26b752d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0752d8, 98 },
    { 0xe63752d9, 99 },
    { 0, 0 },
    { 0x251752db, 52 },
    { 0, 0 },
    { 0x973752dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b752e1, 61 },
    { 0, 0 },
    { 0x904752e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303752e7, 55 },
    { 0x1a5752e8, 34 },
    { 0xfc3752e8, 55 },
    { 0, 0 },
    { 0xfc6752eb, 43 },
    { 0x101752ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95752f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45752fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a752fd, 78 },
    { 0xf1f752fe, 34 },
    { 0x64e752fe, 33 },
    { 0, 0 },
    { 0xe8d75301, 95 },
    { 0x24b75302, 33 },
    { 0xa2a75302, 33 },
    { 0xba175304, 84 },
    { 0x5d475302, 95 },
    { 0x98875306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27530b, 23 },
    { 0, 0 },
    { 0xa97530d, 77 },
    { 0, 0 },
    { 0x53e7530f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d75312, 90 },
    { 0x3b675313, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a675317, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f7531c, 61 },
    { 0x2f7531d, 34 },
    { 0x9947531d, 90 },
    { 0xb027531c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2075323, 61 },
    { 0x68f75323, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c375329, 46 },
    { 0, 0 },
    { 0xaf57532b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e75337, 33 },
    { 0, 0 },
    { 0x96875339, 33 },
    { 0x9077533a, 23 },
    { 0xdc17533b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee75340, 23 },
    { 0x4de75341, 95 },
    { 0x95075342, 33 },
    { 0x52275342, 33 },
    { 0x40775344, 91 },
    { 0x7f275345, 23 },
    { 0x18575343, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87534a, 98 },
    { 0xff07534b, 93 },
    { 0x5c27534c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9975350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47535a, 90 },
    { 0x7e97535b, 84 },
    { 0x6f17535c, 23 },
    { 0, 0 },
    { 0x2147535e, 38 },
    { 0xd367535f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f275362, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68975367, 93 },
    { 0x8a475367, 23 },
    { 0xa3575369, 33 },
    { 0xcd475369, 98 },
    { 0x17b75369, 76 },
    { 0, 0 },
    { 0xc2d7536d, 52 },
    { 0x5387536d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3575371, 34 },
    { 0xa3875371, 61 },
    { 0x82b75373, 33 },
    { 0x7d375374, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37537a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c75380, 93 },
    { 0, 0 },
    { 0x9bd75382, 23 },
    { 0x55175383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca475387, 67 },
    { 0x3d775388, 68 },
    { 0x21275387, 33 },
    { 0x77b75387, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e7538d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48575399, 90 },
    { 0xa287539a, 98 },
    { 0x1a27539b, 33 },
    { 0x8767539c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5537539f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd753a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7753a8, 93 },
    { 0xe5d753a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x313753ac, 90 },
    { 0, 0 },
    { 0x68c753ae, 33 },
    { 0x883753af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f753b3, 88 },
    { 0x89d753b3, 61 },
    { 0, 0 },
    { 0x420753b6, 90 },
    { 0, 0 },
    { 0x5d753b8, 88 },
    { 0x126753b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83753bc, 88 },
    { 0, 0 },
    { 0x7f3753be, 34 },
    { 0, 0 },
    { 0xd3d753c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1753ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c753cd, 98 },
    { 0x907753ce, 23 },
    { 0xfa9753cd, 90 },
    { 0, 0 },
    { 0x201753d1, 32 },
    { 0, 0 },
    { 0x783753d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba753d6, 95 },
    { 0, 0 },
    { 0x2f3753d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668753df, 55 },
    { 0xcfd753df, 61 },
    { 0, 0 },
    { 0x582753e2, 33 },
    { 0x7e0753e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x47753e6, 61 },
    { 0x4ad753e6, 82 },
    { 0x8f3753e8, 95 },
    { 0x744753e8, 95 },
    { 0, 0 },
    { 0x31e753eb, 67 },
    { 0, 0 },
    { 0x2a5753ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16753f5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843753fa, 33 },
    { 0, 0 },
    { 0xd6753fc, 98 },
    { 0, 0 },
    { 0xbb7753fe, 100 },
    { 0xe2b753fe, 61 },
    { 0x137753ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f275403, 77 },
    { 0, 0 },
    { 0x62375405, 61 },
    { 0x1bb75405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f7540a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd247540f, 90 },
    { 0xac77540f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c675414, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77541c, 33 },
    { 0x4b97541d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4575422, 85 },
    { 0, 0 },
    { 0x7ea75424, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e375427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a7542c, 90 },
    { 0, 0 },
    { 0x4727542e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba75432, 61 },
    { 0x66975433, 90 },
    { 0xd6775434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8275437, 100 },
    { 0xb4d75437, 90 },
    { 0, 0 },
    { 0x997543a, 34 },
    { 0xeb87543b, 46 },
    { 0x6637543c, 66 },
    { 0xdc07543d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f75445, 43 },
    { 0x6c675446, 61 },
    { 0xd9f75446, 76 },
    { 0xbf375448, 76 },
    { 0xbef75447, 99 },
    { 0xb067544a, 67 },
    { 0x56a7544a, 33 },
    { 0x5797544c, 64 },
    { 0x20f75446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3775455, 34 },
    { 0xddf75456, 23 },
    { 0, 0 },
    { 0x32b75458, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147545c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a75460, 81 },
    { 0xba775460, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b475464, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e57546d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7547a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a7547d, 90 },
    { 0, 0 },
    { 0x1f57547f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9875483, 90 },
    { 0x4da75484, 52 },
    { 0x5e075484, 23 },
    { 0x36475484, 83 },
    { 0xffb75487, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff7548a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757548d, 88 },
    { 0x31e7548e, 90 },
    { 0xc27548d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2075496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b7549a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa7549d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5754a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a754a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6754ad, 87 },
    { 0x124754ae, 33 },
    { 0x765754ad, 61 },
    { 0xfd1754b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x358754b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55754b7, 23 },
    { 0xabd754b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df754c3, 61 },
    { 0xbcc754c4, 33 },
    { 0xa6b754c3, 53 },
    { 0xf3d754c6, 23 },
    { 0xcc8754c6, 33 },
    { 0x1ed754c6, 98 },
    { 0x8e4754c9, 33 },
    { 0xe92754ca, 90 },
    { 0x9dc754c9, 33 },
    { 0xc57754cc, 61 },
    { 0xbe3754c7, 93 },
    { 0x589754ca, 67 },
    { 0, 0 },
    { 0xdfd754d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917754d6, 61 },
    { 0xbcc754d6, 98 },
    { 0xf44754d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f754df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931754e3, 43 },
    { 0xa66754e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7754e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d754eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa754ee, 68 },
    { 0xff754ef, 61 },
    { 0x525754ef, 87 },
    { 0x475754f1, 61 },
    { 0xc44754f2, 98 },
    { 0xe07754ee, 33 },
    { 0x1f0754f4, 23 },
    { 0xf9754ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5754f8, 88 },
    { 0xc92754f9, 33 },
    { 0xe82754fa, 61 },
    { 0xb27754fb, 33 },
    { 0x9f754fa, 61 },
    { 0, 0 },
    { 0xb27754fe, 23 },
    { 0, 0 },
    { 0xc1375500, 34 },
    { 0x90675500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd75504, 61 },
    { 0xd2775504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58875508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58875510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26275513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b75516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc287551a, 90 },
    { 0x3307551a, 95 },
    { 0x2277551c, 98 },
    { 0, 0 },
    { 0x7aa7551e, 81 },
    { 0x8f27551f, 90 },
    { 0x497551e, 36 },
    { 0, 0 },
    { 0x78275522, 34 },
    { 0x10f75523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77552a, 95 },
    { 0xab17552a, 93 },
    { 0x3c67552c, 78 },
    { 0xffe7552c, 61 },
    { 0x8777552e, 33 },
    { 0x6da7552e, 33 },
    { 0x7517552f, 83 },
    { 0xb937552f, 65 },
    { 0xfd7552f, 76 },
    { 0x7517552a, 66 },
    { 0x7e875533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe87553a, 43 },
    { 0x9537553b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9375545, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a67554b, 61 },
    { 0x4d47554c, 34 },
    { 0, 0 },
    { 0xeee7554e, 98 },
    { 0x6c87554f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e75552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da75556, 90 },
    { 0xd8575557, 67 },
    { 0, 0 },
    { 0xc3975559, 61 },
    { 0x9bf7555a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7555e, 78 },
    { 0, 0 },
    { 0x39f75560, 61 },
    { 0x48175560, 91 },
    { 0x63b75561, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x54775565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d75568, 76 },
    { 0x44475569, 61 },
    { 0x3277556a, 93 },
    { 0x9cb75568, 46 },
    { 0x6d27556c, 66 },
    { 0x17e7556c, 98 },
    { 0x417556d, 55 },
    { 0, 0 },
    { 0x5ba75570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807557d, 100 },
    { 0xf4e7557e, 95 },
    { 0x3b67557f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a975582, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd7558f, 95 },
    { 0, 0 },
    { 0xb2675591, 90 },
    { 0x82675592, 76 },
    { 0, 0 },
    { 0x97175594, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a075598, 99 },
    { 0x73e75599, 66 },
    { 0, 0 },
    { 0xf0e7559b, 99 },
    { 0, 0 },
    { 0xd127559d, 61 },
    { 0, 0 },
    { 0xa827559f, 34 },
    { 0xb0e755a0, 66 },
    { 0x26f755a1, 87 },
    { 0x7f755a0, 93 },
    { 0x5cf755a1, 61 },
    { 0xe4d755a4, 33 },
    { 0xd14755a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70755a9, 78 },
    { 0xc35755a9, 19 },
    { 0, 0 },
    { 0xb9a755ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef755af, 23 },
    { 0, 0 },
    { 0xf6c755b1, 98 },
    { 0x466755b1, 23 },
    { 0x313755b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91755ba, 61 },
    { 0xf22755ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1755be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x457755c1, 34 },
    { 0, 0 },
    { 0xca7755c3, 88 },
    { 0xa8c755c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x637755c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268755cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1755d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c755d4, 33 },
    { 0x879755d5, 78 },
    { 0xcf3755d6, 23 },
    { 0xd4b755d7, 43 },
    { 0x13755d8, 33 },
    { 0x469755d8, 23 },
    { 0x1bb755d4, 33 },
    { 0x2ac755d9, 98 },
    { 0xa13755dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d755e0, 33 },
    { 0, 0 },
    { 0x30f755e2, 90 },
    { 0xf63755e3, 23 },
    { 0x866755e3, 95 },
    { 0xe24755e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9755ea, 23 },
    { 0xd83755ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c755f4, 55 },
    { 0, 0 },
    { 0x234755f6, 23 },
    { 0x682755f6, 33 },
    { 0xfbc755f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f755fb, 98 },
    { 0x96e755fc, 61 },
    { 0xa1f755fd, 97 },
    { 0x88b755fc, 76 },
    { 0, 0 },
    { 0x21775600, 90 },
    { 0, 0 },
    { 0x52c75602, 99 },
    { 0x12075603, 95 },
    { 0x28475604, 33 },
    { 0xef175605, 95 },
    { 0xca075605, 76 },
    { 0xf3e75605, 95 },
    { 0x8ce75608, 87 },
    { 0xd6d75609, 23 },
    { 0x7b57560a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be75610, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1075615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc75619, 90 },
    { 0, 0 },
    { 0x417561b, 46 },
    { 0xec27561b, 98 },
    { 0x9177561b, 76 },
    { 0, 0 },
    { 0x1487561f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b075627, 33 },
    { 0, 0 },
    { 0x85d75629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68375635, 87 },
    { 0, 0 },
    { 0x5d75637, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7563b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c75641, 90 },
    { 0, 0 },
    { 0x20175643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd75647, 38 },
    { 0x39475648, 90 },
    { 0x39975648, 61 },
    { 0xeb375649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75975650, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0275655, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f75659, 95 },
    { 0x9af7565a, 90 },
    { 0, 0 },
    { 0x887565c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b575660, 61 },
    { 0, 0 },
    { 0x38e75662, 98 },
    { 0x52875663, 43 },
    { 0x6775663, 46 },
    { 0, 0 },
    { 0x9a275666, 33 },
    { 0xd2075666, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627566a, 33 },
    { 0x53c7566b, 87 },
    { 0xba17566c, 34 },
    { 0, 0 },
    { 0xb777566e, 61 },
    { 0xd4f7566e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc275673, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c75678, 90 },
    { 0x1b975679, 98 },
    { 0x13e75678, 43 },
    { 0xf2e7567b, 95 },
    { 0xf5475679, 55 },
    { 0, 0 },
    { 0xa0e7567e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82375682, 90 },
    { 0xe5c75682, 61 },
    { 0xfe975683, 33 },
    { 0x84875685, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7997568f, 95 },
    { 0xe5275690, 98 },
    { 0xfdd75691, 85 },
    { 0x96c75692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44275697, 33 },
    { 0, 0 },
    { 0x4f175699, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246756a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663756a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3756ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f756b0, 84 },
    { 0, 0 },
    { 0xceb756b2, 98 },
    { 0x910756b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501756ba, 90 },
    { 0xcf8756bb, 55 },
    { 0x8cc756bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x135756bf, 55 },
    { 0xce0756bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x100756c3, 78 },
    { 0x43756c4, 33 },
    { 0xcee756c5, 85 },
    { 0xfc8756c6, 90 },
    { 0xfa5756c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f756cd, 76 },
    { 0xcf4756ce, 46 },
    { 0x4d756ce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e756d3, 23 },
    { 0xda8756d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6756d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x921756db, 68 },
    { 0, 0 },
    { 0xe54756dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c756e1, 91 },
    { 0xc9e756e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b756e9, 98 },
    { 0xd05756ea, 78 },
    { 0x4cc756eb, 61 },
    { 0x121756ea, 98 },
    { 0x8a2756eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d756f0, 34 },
    { 0x9756f1, 75 },
    { 0xf4d756f2, 43 },
    { 0x999756f3, 90 },
    { 0x796756f3, 98 },
    { 0x5e0756f4, 23 },
    { 0x79d756f3, 90 },
    { 0x833756f7, 93 },
    { 0xb0b756f6, 33 },
    { 0x9d6756f9, 95 },
    { 0xb45756fa, 46 },
    { 0, 0 },
    { 0xcde756fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e756ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78175704, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8775708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437570b, 95 },
    { 0, 0 },
    { 0x7aa7570d, 55 },
    { 0, 0 },
    { 0xaf37570f, 23 },
    { 0x9a175710, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9075718, 55 },
    { 0xa1075719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb7571e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe475724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d275727, 90 },
    { 0x55475728, 65 },
    { 0xfb075728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e7572f, 75 },
    { 0, 0 },
    { 0xc75731, 95 },
    { 0, 0 },
    { 0x40775733, 85 },
    { 0, 0 },
    { 0xcad75735, 95 },
    { 0xa4575736, 46 },
    { 0, 0 },
    { 0x6ce75738, 33 },
    { 0, 0 },
    { 0x1b87573a, 76 },
    { 0xc167573b, 55 },
    { 0x30b7573c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e75740, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce975744, 90 },
    { 0x4a775745, 98 },
    { 0xb5975746, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d75756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c7575d, 23 },
    { 0xf77575e, 90 },
    { 0x7427575f, 33 },
    { 0x1777575d, 55 },
    { 0x2c775761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27375764, 90 },
    { 0, 0 },
    { 0xa6875766, 34 },
    { 0x16775767, 76 },
    { 0x3c175768, 95 },
    { 0x6175768, 95 },
    { 0x55375767, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9875774, 76 },
    { 0, 0 },
    { 0xfc675776, 17 },
    { 0x12875777, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f7577b, 23 },
    { 0x4817577c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5837577f, 43 },
    { 0, 0 },
    { 0x41d75781, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e75784, 52 },
    { 0xd9975785, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11475788, 95 },
    { 0, 0 },
    { 0x5337578a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7578d, 78 },
    { 0xd7b7578d, 95 },
    { 0x2f37578f, 36 },
    { 0, 0 },
    { 0xe8575791, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26975794, 68 },
    { 0x17b75794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d975798, 43 },
    { 0xeab75799, 46 },
    { 0x3577579a, 85 },
    { 0, 0 },
    { 0x4c37579c, 33 },
    { 0x2747579d, 90 },
    { 0xae07579d, 95 },
    { 0, 0 },
    { 0x124757a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b757a6, 33 },
    { 0xf0f757a7, 91 },
    { 0, 0 },
    { 0x450757a9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191757af, 61 },
    { 0x2f7757b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf757bc, 55 },
    { 0x5f757bd, 88 },
    { 0xa94757be, 33 },
    { 0xe39757bd, 87 },
    { 0xc54757bf, 90 },
    { 0, 0 },
    { 0x381757c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43757c5, 88 },
    { 0xaa9757c5, 61 },
    { 0x599757c5, 95 },
    { 0, 0 },
    { 0xfc8757c9, 61 },
    { 0xa5d757ca, 95 },
    { 0xe3f757cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61757cf, 95 },
    { 0x2d4757d0, 61 },
    { 0x768757cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa757d5, 66 },
    { 0x7ed757d6, 61 },
    { 0xbc0757d7, 61 },
    { 0x942757d8, 61 },
    { 0x1b7757d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9757df, 90 },
    { 0xc8c757e0, 33 },
    { 0xdc9757e1, 23 },
    { 0xf10757e2, 23 },
    { 0x350757e3, 33 },
    { 0x915757e3, 90 },
    { 0xbd757e1, 90 },
    { 0xc25757e6, 33 },
    { 0xfb0757e6, 33 },
    { 0x45e757e2, 47 },
    { 0xd96757e9, 90 },
    { 0, 0 },
    { 0xd5d757eb, 61 },
    { 0, 0 },
    { 0x3ef757ed, 60 },
    { 0x362757ee, 52 },
    { 0, 0 },
    { 0x64c757f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f757f3, 90 },
    { 0x5ed757f3, 85 },
    { 0xfc757f4, 76 },
    { 0xc14757f6, 46 },
    { 0xd65757f7, 61 },
    { 0xe85757f8, 91 },
    { 0, 0 },
    { 0xdc6757fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d675801, 23 },
    { 0x6b975801, 33 },
    { 0x53475802, 93 },
    { 0x94b75802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a7580b, 23 },
    { 0, 0 },
    { 0xaa37580d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e875816, 61 },
    { 0xe1875817, 91 },
    { 0xee475818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7581b, 61 },
    { 0x8b27581c, 76 },
    { 0x7587581d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d75822, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b575828, 33 },
    { 0xd2975829, 34 },
    { 0, 0 },
    { 0x3667582b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e75830, 93 },
    { 0, 0 },
    { 0xbc975832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb175839, 99 },
    { 0, 0 },
    { 0x97c7583b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2775842, 76 },
    { 0x2f975843, 75 },
    { 0xa5e75842, 33 },
    { 0x6c175842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf47584a, 33 },
    { 0x26d7584a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5737584e, 90 },
    { 0x7317584f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c75858, 77 },
    { 0xf5475858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46875860, 67 },
    { 0xd2375861, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975865, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86575869, 90 },
    { 0x9c87586a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf75871, 90 },
    { 0xd1975872, 98 },
    { 0xfaa75873, 61 },
    { 0x50275874, 87 },
    { 0x47f75871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a575881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2275884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65975887, 95 },
    { 0x26b75888, 90 },
    { 0xcf875887, 90 },
    { 0x8b7588a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7588d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e575891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b75894, 90 },
    { 0x44075895, 33 },
    { 0xa8b75895, 17 },
    { 0, 0 },
    { 0x6d475898, 93 },
    { 0xb675899, 55 },
    { 0x1e97589a, 98 },
    { 0xdb07589a, 23 },
    { 0xa4c7589a, 34 },
    { 0, 0 },
    { 0xad77589e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f758a1, 23 },
    { 0x92c758a2, 46 },
    { 0x5ea758a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb758a6, 98 },
    { 0xd0a758a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e758aa, 67 },
    { 0xfcd758aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee758b0, 55 },
    { 0, 0 },
    { 0x6ef758b2, 76 },
    { 0, 0 },
    { 0xcc1758b4, 100 },
    { 0, 0 },
    { 0xb77758b6, 41 },
    { 0xa21758b7, 93 },
    { 0x150758b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36758bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c758bf, 61 },
    { 0, 0 },
    { 0xa9e758c1, 90 },
    { 0xb1758c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8758c6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734758cd, 55 },
    { 0x56d758cd, 76 },
    { 0x784758cd, 95 },
    { 0, 0 },
    { 0x5dc758d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a758d6, 93 },
    { 0x140758d7, 43 },
    { 0x98a758d6, 23 },
    { 0, 0 },
    { 0x57e758da, 93 },
    { 0x776758da, 90 },
    { 0x617758dc, 33 },
    { 0x491758db, 61 },
    { 0xc97758de, 90 },
    { 0xcd3758de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd758e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0758e9, 32 },
    { 0xbc5758ea, 88 },
    { 0xe6e758eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x781758ee, 66 },
    { 0x5ce758ef, 67 },
    { 0x93b758f0, 100 },
    { 0, 0 },
    { 0x233758f2, 90 },
    { 0, 0 },
    { 0xb8c758f4, 78 },
    { 0x695758f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77758fb, 90 },
    { 0x46f758fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c75902, 98 },
    { 0x8f075903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b75906, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b75909, 96 },
    { 0xd9275909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e175910, 35 },
    { 0x3a475910, 46 },
    { 0, 0 },
    { 0xd5775913, 66 },
    { 0x9e775914, 61 },
    { 0x4a775915, 33 },
    { 0x4ff75916, 98 },
    { 0xb3575917, 91 },
    { 0x85f75915, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92475920, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec75927, 23 },
    { 0xf8875927, 61 },
    { 0x81975929, 33 },
    { 0x89875929, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c75930, 33 },
    { 0, 0 },
    { 0x48a75932, 98 },
    { 0, 0 },
    { 0x13275934, 93 },
    { 0, 0 },
    { 0xc5f75936, 64 },
    { 0x28575937, 99 },
    { 0xcab75937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de7593b, 33 },
    { 0, 0 },
    { 0xbed7593d, 85 },
    { 0xc117593d, 98 },
    { 0xa037593f, 23 },
    { 0xc307593f, 33 },
    { 0xbb075940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45375946, 23 },
    { 0x52b75947, 97 },
    { 0, 0 },
    { 0xae775949, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e7594e, 33 },
    { 0xd1c7594e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f275955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6475958, 81 },
    { 0x32575958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df7595c, 99 },
    { 0, 0 },
    { 0x26c7595e, 90 },
    { 0x5d17595f, 55 },
    { 0, 0 },
    { 0xda075961, 76 },
    { 0, 0 },
    { 0xcc775963, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e75968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7596b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c7596f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36575975, 90 },
    { 0x7d975975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa267597a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe297597d, 33 },
    { 0x4f87597e, 87 },
    { 0xe6f7597e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43775988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2175991, 23 },
    { 0, 0 },
    { 0x13f75993, 23 },
    { 0, 0 },
    { 0x39575995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db75998, 87 },
    { 0xb9075999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf7599c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475759a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c759a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae759ae, 43 },
    { 0x371759af, 64 },
    { 0xebe759b0, 66 },
    { 0x8fb759b0, 34 },
    { 0x71d759b2, 95 },
    { 0x622759b3, 55 },
    { 0x108759b3, 95 },
    { 0, 0 },
    { 0x17759b6, 98 },
    { 0, 0 },
    { 0x869759b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd759bf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553759c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211759d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728759dc, 23 },
    { 0xcc4759dc, 83 },
    { 0xc13759de, 61 },
    { 0x62d759df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45759e4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913759e8, 76 },
    { 0, 0 },
    { 0xc65759ea, 43 },
    { 0x8d3759eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1759ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301759f2, 76 },
    { 0, 0 },
    { 0xeca759f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x550759f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5759fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7759fe, 33 },
    { 0x963759fe, 95 },
    { 0x76875a00, 46 },
    { 0x7b775a01, 95 },
    { 0x2a675a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6875a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa575a0d, 95 },
    { 0xedb75a0e, 87 },
    { 0x88975a0e, 98 },
    { 0xb6475a10, 98 },
    { 0xfdc75a11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de75a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc75a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6275a1c, 78 },
    { 0x52375a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62975a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8975a28, 98 },
    { 0xd375a28, 95 },
    { 0x7a175a2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73975a2f, 61 },
    { 0, 0 },
    { 0xf1f75a31, 98 },
    { 0x5f075a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74775a36, 90 },
    { 0xf4275a36, 34 },
    { 0x17075a36, 90 },
    { 0x9b575a37, 90 },
    { 0x43875a3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96375a3e, 90 },
    { 0x38575a3f, 34 },
    { 0x9b175a3e, 98 },
    { 0x98575a40, 33 },
    { 0x6bd75a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f75a45, 93 },
    { 0x36975a46, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4375a4b, 23 },
    { 0xb7575a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb75a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a75a56, 91 },
    { 0x21275a56, 98 },
    { 0xc7775a58, 23 },
    { 0xfad75a59, 98 },
    { 0, 0 },
    { 0x35775a5b, 66 },
    { 0x32675a5c, 67 },
    { 0xc3575a5d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d75a66, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c75a69, 91 },
    { 0xa5e75a6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc975a6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c75a77, 95 },
    { 0x88275a77, 55 },
    { 0, 0 },
    { 0x6c175a7a, 95 },
    { 0xb0875a7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78475a7f, 23 },
    { 0, 0 },
    { 0xdc175a81, 55 },
    { 0x5de75a81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a575a85, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f75a88, 61 },
    { 0, 0 },
    { 0xc8f75a8a, 34 },
    { 0, 0 },
    { 0xb8f75a8c, 95 },
    { 0xd7875a8d, 61 },
    { 0, 0 },
    { 0x6d575a8f, 67 },
    { 0x76375a8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc75a93, 33 },
    { 0x39075a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35275a9c, 66 },
    { 0xd8575a9d, 90 },
    { 0x5e175a9c, 98 },
    { 0, 0 },
    { 0x26675aa0, 55 },
    { 0x53b75aa0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f75aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96475aab, 95 },
    { 0x68f75aac, 34 },
    { 0x1b575aad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13675ab5, 78 },
    { 0, 0 },
    { 0x81c75ab7, 43 },
    { 0xe8475ab8, 75 },
    { 0x70175ab9, 91 },
    { 0x4e875aba, 61 },
    { 0xabe75abb, 95 },
    { 0x1675abc, 90 },
    { 0x8be75abd, 95 },
    { 0xdde75abe, 90 },
    { 0x8a575abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5675ac2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff75acf, 90 },
    { 0x1b175ad0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d75ad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13475ada, 90 },
    { 0x70675adb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b775ae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e075ae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2475ae9, 43 },
    { 0x18b75ae9, 61 },
    { 0xd8e75aeb, 33 },
    { 0x7b875aec, 67 },
    { 0xab75aec, 23 },
    { 0, 0 },
    { 0x5da75aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca075af2, 33 },
    { 0xb0875af3, 61 },
    { 0xb4775af4, 98 },
    { 0, 0 },
    { 0x59575af6, 84 },
    { 0xa2375af7, 23 },
    { 0, 0 },
    { 0x90175af9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x53375afc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf75b00, 85 },
    { 0, 0 },
    { 0xbca75b02, 33 },
    { 0, 0 },
    { 0x6b175b04, 61 },
    { 0x10775b05, 95 },
    { 0x78f75b06, 23 },
    { 0x54475b07, 17 },
    { 0x7e75b07, 76 },
    { 0x64875b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9375b0e, 100 },
    { 0, 0 },
    { 0x17575b10, 100 },
    { 0x2b175b11, 84 },
    { 0xbda75b10, 88 },
    { 0, 0 },
    { 0xee475b14, 76 },
    { 0xa2f75b15, 46 },
    { 0, 0 },
    { 0x5e075b17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea575b20, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b75b2e, 61 },
    { 0xa2175b2e, 61 },
    { 0xedd75b2e, 55 },
    { 0, 0 },
    { 0x7075b32, 90 },
    { 0x6a275b33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5575b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd75b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e75b3f, 55 },
    { 0x95d75b40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36675b44, 90 },
    { 0xb6375b45, 90 },
    { 0, 0 },
    { 0xe5075b47, 90 },
    { 0xc5975b48, 61 },
    { 0xd3375b49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475b4e, 61 },
    { 0xe675b4f, 33 },
    { 0x54a75b4e, 33 },
    { 0, 0 },
    { 0x4a375b52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf75b57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a75b5c, 95 },
    { 0, 0 },
    { 0xe0d75b5e, 98 },
    { 0xd3175b5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea75b64, 90 },
    { 0x11075b65, 88 },
    { 0, 0 },
    { 0x5d275b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b375b6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd75b6d, 98 },
    { 0, 0 },
    { 0xe6875b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f675b73, 23 },
    { 0, 0 },
    { 0xe4675b75, 90 },
    { 0x4d475b76, 66 },
    { 0x9f475b75, 33 },
    { 0x5b375b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f75b7c, 95 },
    { 0, 0 },
    { 0x15475b7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3875b81, 23 },
    { 0, 0 },
    { 0xa7a75b83, 23 },
    { 0, 0 },
    { 0xada75b85, 61 },
    { 0x61675b86, 66 },
    { 0xf9b75b86, 93 },
    { 0xb9d75b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e75b8b, 61 },
    { 0, 0 },
    { 0x92575b8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc75b90, 34 },
    { 0, 0 },
    { 0xe4b75b92, 33 },
    { 0xf775b93, 98 },
    { 0x86c75b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45775b97, 98 },
    { 0x55375b97, 93 },
    { 0x5f775b98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f75b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59175ba4, 83 },
    { 0, 0 },
    { 0x34e75ba6, 91 },
    { 0xc7c75ba7, 76 },
    { 0x6f275ba8, 34 },
    { 0x7ca75ba9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59675bae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e75bb3, 23 },
    { 0x4175bb4, 95 },
    { 0, 0 },
    { 0x59775bb6, 55 },
    { 0, 0 },
    { 0x78075bb8, 13 },
    { 0xdd275bb8, 34 },
    { 0xedb75bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd75bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6675bc1, 91 },
    { 0, 0 },
    { 0x61175bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b375bc7, 77 },
    { 0x58f75bc8, 61 },
    { 0x4ba75bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9175bd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd75bd4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b75bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db75bdb, 23 },
    { 0xfa375bdc, 34 },
    { 0x14f75bdb, 23 },
    { 0, 0 },
    { 0x19075bdf, 67 },
    { 0xe7775be0, 95 },
    { 0xbf875be1, 66 },
    { 0x5475be1, 40 },
    { 0, 0 },
    { 0xcdf75be4, 33 },
    { 0, 0 },
    { 0x4a175be6, 98 },
    { 0x94475be6, 46 },
    { 0xcf75be7, 17 },
    { 0, 0 },
    { 0x84c75bea, 90 },
    { 0x7175beb, 98 },
    { 0x17e75bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba175bf4, 55 },
    { 0x67375bf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5875bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10375bfc, 83 },
    { 0x73675bfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5675c02, 55 },
    { 0x21675c03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa375c07, 77 },
    { 0xbee75c08, 88 },
    { 0x3be75c07, 61 },
    { 0xa2875c09, 91 },
    { 0, 0 },
    { 0xac575c0c, 33 },
    { 0, 0 },
    { 0x1b575c0e, 43 },
    { 0x96775c0f, 67 },
    { 0x44775c0e, 95 },
    { 0xdc575c11, 93 },
    { 0xcda75c11, 98 },
    { 0x5ff75c0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c775c16, 33 },
    { 0xfdb75c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a775c1e, 46 },
    { 0xd9a75c1e, 99 },
    { 0xa4975c20, 65 },
    { 0x57975c1e, 95 },
    { 0xbab75c22, 46 },
    { 0xc9375c21, 55 },
    { 0x14675c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca75c29, 87 },
    { 0xf3675c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe75c2e, 61 },
    { 0, 0 },
    { 0xaf575c30, 98 },
    { 0x55b75c31, 98 },
    { 0xb0875c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a475c35, 90 },
    { 0, 0 },
    { 0xea675c37, 93 },
    { 0x34675c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63375c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea175c41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5575c45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf75c48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72275c50, 23 },
    { 0x2d875c51, 42 },
    { 0x8e975c51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1575c55, 95 },
    { 0xfae75c56, 90 },
    { 0xf9775c57, 75 },
    { 0x5bf75c57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8775c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e75c66, 23 },
    { 0x57675c67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c275c6d, 23 },
    { 0xd8e75c6e, 53 },
    { 0xd3075c6f, 23 },
    { 0x90575c70, 43 },
    { 0x6b575c70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f75c75, 95 },
    { 0x4d575c76, 76 },
    { 0xe7675c77, 66 },
    { 0xc6375c76, 61 },
    { 0x51275c79, 36 },
    { 0x41575c77, 66 },
    { 0xf5a75c7b, 99 },
    { 0xa7d75c7c, 88 },
    { 0x4ea75c7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5675c82, 33 },
    { 0xd7875c83, 33 },
    { 0x8bb75c84, 61 },
    { 0xabf75c82, 87 },
    { 0xd6675c82, 90 },
    { 0x4a975c87, 87 },
    { 0x7ee75c88, 90 },
    { 0x59975c88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77675c8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1975c8f, 61 },
    { 0x29875c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf575c93, 33 },
    { 0x2d375c94, 95 },
    { 0, 0 },
    { 0xf8775c96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67675c99, 61 },
    { 0x47875c99, 91 },
    { 0xda375c9b, 43 },
    { 0xbe975c9c, 33 },
    { 0, 0 },
    { 0x86275c9e, 93 },
    { 0, 0 },
    { 0x56575ca0, 90 },
    { 0x9e075ca1, 76 },
    { 0x41375ca2, 33 },
    { 0x69e75ca2, 33 },
    { 0x575ca4, 17 },
    { 0, 0 },
    { 0x55a75ca6, 76 },
    { 0x69e75ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2175caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e75cae, 61 },
    { 0x9475cae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55775cb7, 43 },
    { 0xcd275cb8, 98 },
    { 0xe5575cb8, 98 },
    { 0x71b75cb9, 34 },
    { 0x27775cbb, 61 },
    { 0xa5d75cbb, 66 },
    { 0x51675cb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43175cc6, 61 },
    { 0xac275cc7, 34 },
    { 0xd375cc8, 95 },
    { 0xa6075cc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd175ccd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9575cd2, 33 },
    { 0x9f275cd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe75cda, 76 },
    { 0xc7475cdb, 43 },
    { 0xb8c75cdc, 87 },
    { 0x1875cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e75ce2, 90 },
    { 0x84275ce3, 96 },
    { 0x55275ce4, 98 },
    { 0xb9a75ce2, 87 },
    { 0, 0 },
    { 0xd6f75ce7, 95 },
    { 0x7dd75ce8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce75ceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa75cee, 95 },
    { 0xb8175cef, 61 },
    { 0x67475cf0, 95 },
    { 0x5975cf1, 43 },
    { 0x48875cf2, 55 },
    { 0x60175cf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e75cf6, 78 },
    { 0, 0 },
    { 0x79675cf8, 76 },
    { 0, 0 },
    { 0x53b75cfa, 33 },
    { 0x62275cfb, 34 },
    { 0xb6a75cfc, 75 },
    { 0xc4d75cfd, 90 },
    { 0, 0 },
    { 0x8e775cff, 87 },
    { 0, 0 },
    { 0xe9875d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e75d07, 55 },
    { 0x4af75d08, 23 },
    { 0x73c75d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba875d0d, 95 },
    { 0x81b75d0d, 90 },
    { 0, 0 },
    { 0x79175d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa075d13, 43 },
    { 0x4d975d13, 46 },
    { 0x90b75d13, 23 },
    { 0xf7175d14, 99 },
    { 0, 0 },
    { 0xded75d18, 82 },
    { 0x1c75d19, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d75d1d, 90 },
    { 0x58175d1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76475d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3175d28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1875d2d, 23 },
    { 0x4d275d2e, 23 },
    { 0x4f975d2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075d33, 66 },
    { 0, 0 },
    { 0x29c75d35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace75d3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b75d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c275d4d, 77 },
    { 0x44475d4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc975d51, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c575d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc75d5a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c775d5d, 100 },
    { 0x24c75d5e, 46 },
    { 0xcd275d5e, 61 },
    { 0, 0 },
    { 0x9b675d61, 90 },
    { 0, 0 },
    { 0x2ba75d63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8675d67, 51 },
    { 0xc4375d68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb575d6d, 43 },
    { 0, 0 },
    { 0x94375d6f, 90 },
    { 0, 0 },
    { 0x45375d71, 78 },
    { 0, 0 },
    { 0xf7d75d73, 33 },
    { 0, 0 },
    { 0x65f75d75, 81 },
    { 0, 0 },
    { 0x79c75d77, 33 },
    { 0xa6175d78, 90 },
    { 0x3a475d78, 90 },
    { 0xa1e75d7a, 43 },
    { 0x80475d77, 55 },
    { 0xd6775d7c, 67 },
    { 0x63075d79, 66 },
    { 0xfc875d7e, 90 },
    { 0, 0 },
    { 0x7c675d80, 55 },
    { 0, 0 },
    { 0x8c375d82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e75d87, 77 },
    { 0, 0 },
    { 0x90375d89, 61 },
    { 0x6b75d89, 84 },
    { 0x7cc75d8b, 88 },
    { 0, 0 },
    { 0xdba75d8d, 23 },
    { 0, 0 },
    { 0xe2575d8f, 90 },
    { 0, 0 },
    { 0xd2375d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c75d95, 90 },
    { 0xf6775d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1875d99, 76 },
    { 0x14c75d9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd75d9f, 46 },
    { 0xbff75da0, 85 },
    { 0, 0 },
    { 0x87375da2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c375da7, 95 },
    { 0, 0 },
    { 0x28e75da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c75dac, 59 },
    { 0x9b675dac, 95 },
    { 0xaa475dae, 52 },
    { 0x12375dae, 55 },
    { 0x38c75db0, 76 },
    { 0x95c75db1, 52 },
    { 0x86875db2, 95 },
    { 0x53b75daf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f075db7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a75dbf, 55 },
    { 0xa375dc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69275dc4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a75dc8, 95 },
    { 0xa2875dc8, 95 },
    { 0, 0 },
    { 0x60075dcb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb775dce, 34 },
    { 0, 0 },
    { 0x28b75dd0, 93 },
    { 0xac975dd0, 33 },
    { 0xdfa75dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75dd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c75ddb, 76 },
    { 0x58375ddc, 81 },
    { 0x50d75ddd, 33 },
    { 0xce975dde, 90 },
    { 0, 0 },
    { 0x9e775de0, 98 },
    { 0x8de75de1, 76 },
    { 0x81675de0, 66 },
    { 0x79a75de3, 91 },
    { 0x92975de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b175ded, 33 },
    { 0x55975dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675df5, 52 },
    { 0x88175df6, 90 },
    { 0x675df7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8875dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe275e00, 84 },
    { 0, 0 },
    { 0x4ae75e02, 95 },
    { 0xc8375e03, 93 },
    { 0x87a75e04, 33 },
    { 0x83d75e03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5675e09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d375e11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae875e15, 55 },
    { 0x1a775e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f75e1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56975e1f, 84 },
    { 0x7e175e20, 33 },
    { 0x70575e21, 94 },
    { 0x3eb75e22, 61 },
    { 0xa4475e23, 67 },
    { 0, 0 },
    { 0x50775e25, 99 },
    { 0xd9375e26, 98 },
    { 0, 0 },
    { 0x12875e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df75e2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2375e41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc75e45, 98 },
    { 0x76e75e46, 83 },
    { 0, 0 },
    { 0x3f375e48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c875e4d, 68 },
    { 0, 0 },
    { 0xe3f75e4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae75e54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f75e57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0575e5c, 43 },
    { 0, 0 },
    { 0x2e175e5e, 23 },
    { 0x5ce75e5f, 61 },
    { 0, 0 },
    { 0x18475e61, 23 },
    { 0x9a975e61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e75e67, 66 },
    { 0, 0 },
    { 0xb3a75e69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d475e7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c775e7e, 93 },
    { 0xd7d75e7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f75e82, 95 },
    { 0xbb175e82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30475e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d75e8a, 84 },
    { 0xfda75e8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e75e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a75e95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c75eac, 43 },
    { 0x2f375eac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87075eb0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c75eb3, 100 },
    { 0, 0 },
    { 0xf1c75eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b75eba, 61 },
    { 0, 0 },
    { 0x4e775ebc, 85 },
    { 0xe8675ebc, 87 },
    { 0x53175ebc, 23 },
    { 0xdc775ebc, 95 },
    { 0x66d75ec0, 90 },
    { 0x57475ec1, 75 },
    { 0x95775ec2, 34 },
    { 0xeda75ec3, 52 },
    { 0xf1f75ebd, 90 },
    { 0x5f575ec5, 43 },
    { 0xdfe75ec2, 65 },
    { 0, 0 },
    { 0x79275ec8, 23 },
    { 0x65e75ec9, 90 },
    { 0xb4075ec8, 34 },
    { 0x78675ecb, 61 },
    { 0, 0 },
    { 0x98d75ecd, 90 },
    { 0, 0 },
    { 0xb3f75ecf, 95 },
    { 0x34e75ed0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a75ed3, 33 },
    { 0xe7775ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60675edb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a75ee0, 93 },
    { 0xf4575ee1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a75ee5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d175ee8, 33 },
    { 0, 0 },
    { 0xc5175eea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6075eed, 95 },
    { 0, 0 },
    { 0x4d975eef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d175ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a875efa, 95 },
    { 0x92675efb, 95 },
    { 0x3e375efc, 34 },
    { 0, 0 },
    { 0x5f075efe, 23 },
    { 0, 0 },
    { 0x62a75f00, 100 },
    { 0x32b75f01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3775f08, 34 },
    { 0x27375f08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb475f0c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975f0f, 98 },
    { 0x4b775f10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c75f14, 98 },
    { 0xa5275f15, 33 },
    { 0x58475f16, 96 },
    { 0x16c75f16, 85 },
    { 0, 0 },
    { 0xec775f19, 94 },
    { 0xb9f75f1a, 99 },
    { 0, 0 },
    { 0x2bf75f1c, 95 },
    { 0x86375f1d, 34 },
    { 0x38875f1e, 76 },
    { 0x71e75f1f, 90 },
    { 0xd0775f20, 55 },
    { 0x71075f20, 95 },
    { 0x24b75f22, 90 },
    { 0x6dd75f23, 98 },
    { 0xb475f22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a75f27, 23 },
    { 0x99775f27, 34 },
    { 0x2c575f27, 87 },
    { 0x62375f2a, 52 },
    { 0x50175f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db75f36, 23 },
    { 0x87e75f36, 86 },
    { 0xc7375f38, 95 },
    { 0, 0 },
    { 0x50b75f3a, 23 },
    { 0xbd975f3b, 76 },
    { 0x43075f3a, 95 },
    { 0x44e75f3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f75f43, 76 },
    { 0xdc375f43, 66 },
    { 0, 0 },
    { 0x69f75f46, 61 },
    { 0, 0 },
    { 0xe2e75f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49475f4b, 61 },
    { 0x88c75f4c, 33 },
    { 0xfd75f4d, 95 },
    { 0x37375f4e, 23 },
    { 0x6de75f4f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae75f55, 76 },
    { 0x29675f56, 93 },
    { 0xec175f56, 91 },
    { 0x3f675f56, 33 },
    { 0x1d575f56, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95275f5e, 90 },
    { 0xf4175f5e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c575f62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675f66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc775f6c, 90 },
    { 0x6a475f6d, 90 },
    { 0x55a75f6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb475f72, 61 },
    { 0xa6a75f73, 98 },
    { 0x60d75f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52775f7b, 43 },
    { 0, 0 },
    { 0x49675f7d, 61 },
    { 0x4ef75f7d, 90 },
    { 0, 0 },
    { 0xb5875f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66375f83, 76 },
    { 0x42e75f84, 91 },
    { 0x4f175f84, 68 },
    { 0, 0 },
    { 0x77d75f87, 76 },
    { 0, 0 },
    { 0x4ea75f89, 90 },
    { 0, 0 },
    { 0x94475f8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a75f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b75f93, 61 },
    { 0, 0 },
    { 0x3f975f95, 65 },
    { 0x4fe75f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a275f99, 94 },
    { 0xa0d75f99, 33 },
    { 0x29475f9b, 98 },
    { 0x49675f9c, 95 },
    { 0x16a75f9d, 61 },
    { 0xc3275f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca75fa7, 87 },
    { 0x30375fa7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d75fac, 33 },
    { 0x82f75fad, 33 },
    { 0x9e475fae, 33 },
    { 0x31b75fac, 80 },
    { 0x98a75fb0, 33 },
    { 0, 0 },
    { 0xace75fb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f75fb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5475fba, 33 },
    { 0xc2275fbb, 90 },
    { 0x97575fbc, 61 },
    { 0x49475fbb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f75fc3, 94 },
    { 0xe9875fc4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89475fc8, 95 },
    { 0xdec75fc9, 90 },
    { 0x90a75fc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7975fd1, 34 },
    { 0xb9275fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f75fd9, 34 },
    { 0, 0 },
    { 0x8ac75fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5075fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb975fe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6175fe7, 89 },
    { 0, 0 },
    { 0x6b575fe9, 84 },
    { 0xaea75fea, 34 },
    { 0x2e675feb, 46 },
    { 0, 0 },
    { 0xe5a75fed, 77 },
    { 0, 0 },
    { 0x96075fef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac175ffd, 98 },
    { 0xd8b75ffe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f76006, 33 },
    { 0, 0 },
    { 0xb3a76008, 95 },
    { 0, 0 },
    { 0xbeb7600a, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b7600e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59176014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7601a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc357601d, 93 },
    { 0, 0 },
    { 0x57601f, 33 },
    { 0x3ce76020, 34 },
    { 0, 0 },
    { 0x2f576022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf76025, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7602a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc7602e, 90 },
    { 0xf7e7602e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f76032, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf7603e, 66 },
    { 0, 0 },
    { 0xaaf76040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3876048, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b07604b, 46 },
    { 0x5037604c, 95 },
    { 0x1ce7604d, 67 },
    { 0xd7d7604d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c76051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0676054, 55 },
    { 0xb276055, 95 },
    { 0xc7676056, 96 },
    { 0, 0 },
    { 0x7d876058, 46 },
    { 0x4f076058, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c7605d, 76 },
    { 0xbb77605e, 55 },
    { 0x9167605f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6476063, 90 },
    { 0x20376064, 38 },
    { 0xc7476065, 78 },
    { 0x25e76064, 33 },
    { 0x1cc76067, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd167606d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a676072, 34 },
    { 0x6d176073, 43 },
    { 0, 0 },
    { 0x1e276075, 23 },
    { 0x38076075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf676088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7608b, 95 },
    { 0xd457608b, 61 },
    { 0x5ff7608c, 95 },
    { 0xdf37608e, 55 },
    { 0xc127608b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98976092, 77 },
    { 0xe9d76093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77976098, 33 },
    { 0x52f76099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8807609e, 61 },
    { 0x5057609e, 33 },
    { 0xee0760a0, 81 },
    { 0x25b7609e, 55 },
    { 0xe6a760a2, 88 },
    { 0, 0 },
    { 0x13d760a4, 46 },
    { 0, 0 },
    { 0x58760a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7760ab, 67 },
    { 0x247760ac, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793760b7, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6760bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a760c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59760c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4760d5, 87 },
    { 0xfdb760d6, 33 },
    { 0, 0 },
    { 0x26c760d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06760db, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3760de, 99 },
    { 0x7a6760df, 66 },
    { 0xe58760e0, 76 },
    { 0, 0 },
    { 0x1b0760e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9760e5, 55 },
    { 0x785760e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1760e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567760ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806760f1, 90 },
    { 0x519760f2, 46 },
    { 0xf13760f3, 88 },
    { 0xc7f760f4, 88 },
    { 0, 0 },
    { 0x1e9760f6, 61 },
    { 0xc6e760f6, 61 },
    { 0xba9760f8, 23 },
    { 0x78e760f8, 58 },
    { 0, 0 },
    { 0x50b760fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5476100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f76103, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c76106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae7610a, 33 },
    { 0x61e7610a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497610f, 91 },
    { 0, 0 },
    { 0x6af76111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc476117, 98 },
    { 0xe4876117, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47611b, 87 },
    { 0xb347611c, 90 },
    { 0, 0 },
    { 0x60f7611e, 95 },
    { 0, 0 },
    { 0xdb776120, 98 },
    { 0x1ed76120, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f176125, 65 },
    { 0x12a76126, 52 },
    { 0xfc076126, 40 },
    { 0xa7476128, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d87612c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57612f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e76133, 93 },
    { 0, 0 },
    { 0xf076135, 98 },
    { 0x16776136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67613b, 34 },
    { 0, 0 },
    { 0x9ea7613d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b76140, 66 },
    { 0, 0 },
    { 0xfbe76142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee76146, 39 },
    { 0xc1476146, 33 },
    { 0xc6176148, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb767614c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec476159, 99 },
    { 0x9f97615a, 90 },
    { 0xc367615a, 52 },
    { 0x4347615c, 90 },
    { 0x3c47615b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9176161, 33 },
    { 0, 0 },
    { 0xa0676163, 65 },
    { 0xd2d76163, 23 },
    { 0xde476165, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597616b, 66 },
    { 0xe407616c, 78 },
    { 0x9107616d, 17 },
    { 0xea57616d, 61 },
    { 0x1927616e, 61 },
    { 0x3137616b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde776174, 78 },
    { 0x2ec76175, 84 },
    { 0x8f776175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7617c, 84 },
    { 0, 0 },
    { 0x4217617e, 80 },
    { 0x9627617f, 90 },
    { 0x85176180, 98 },
    { 0x39576181, 98 },
    { 0x75e76180, 23 },
    { 0x6276181, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc7618a, 90 },
    { 0x68e7618a, 87 },
    { 0x8ee7618c, 90 },
    { 0, 0 },
    { 0x4ba7618e, 61 },
    { 0x6a7618f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58276192, 40 },
    { 0, 0 },
    { 0xefb76194, 90 },
    { 0x2bd76195, 55 },
    { 0xd2876196, 93 },
    { 0x88676197, 90 },
    { 0x95476198, 23 },
    { 0, 0 },
    { 0x3a37619a, 95 },
    { 0x7b7619a, 93 },
    { 0x2347619c, 90 },
    { 0, 0 },
    { 0x3de7619e, 23 },
    { 0x1fe7619e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c761a6, 67 },
    { 0x6fc761a7, 81 },
    { 0x464761a8, 34 },
    { 0, 0 },
    { 0x151761aa, 76 },
    { 0x1fb761ab, 23 },
    { 0x86e761ac, 23 },
    { 0, 0 },
    { 0xcb4761ae, 61 },
    { 0xd2e761ae, 33 },
    { 0x9de761af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6761b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f761b7, 95 },
    { 0x4cc761b7, 90 },
    { 0xd2d761b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0761bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9761c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43761c4, 46 },
    { 0xaee761c4, 55 },
    { 0xf20761c6, 90 },
    { 0x373761c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce761cb, 95 },
    { 0, 0 },
    { 0x7cd761cd, 34 },
    { 0xd6e761ce, 33 },
    { 0x766761cd, 91 },
    { 0, 0 },
    { 0x369761d1, 33 },
    { 0, 0 },
    { 0xc4e761d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231761d7, 95 },
    { 0, 0 },
    { 0x5c8761d9, 34 },
    { 0, 0 },
    { 0x2cd761db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47761e0, 88 },
    { 0, 0 },
    { 0xebd761e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad761e7, 81 },
    { 0xe24761e7, 52 },
    { 0x6e3761e9, 96 },
    { 0xca6761e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb761ed, 99 },
    { 0x5d6761ee, 98 },
    { 0xb71761ed, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af761f2, 95 },
    { 0xad761f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x588761f6, 61 },
    { 0xb0b761f7, 98 },
    { 0x424761f7, 61 },
    { 0x3ab761f8, 90 },
    { 0xb72761f9, 33 },
    { 0x58d761f7, 87 },
    { 0, 0 },
    { 0x2e3761fd, 55 },
    { 0x86c761fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf76205, 76 },
    { 0, 0 },
    { 0x59d76207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127620b, 98 },
    { 0x4797620b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1097620f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f76214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b76217, 95 },
    { 0, 0 },
    { 0x6776219, 99 },
    { 0x5337621a, 23 },
    { 0xacf7621b, 67 },
    { 0x1bc76219, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067621f, 34 },
    { 0x6d876220, 33 },
    { 0x90376220, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3476227, 66 },
    { 0xc4276227, 23 },
    { 0xbaf76229, 51 },
    { 0xde276228, 55 },
    { 0, 0 },
    { 0x99f7622c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac476230, 90 },
    { 0x27776230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda776234, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277623a, 75 },
    { 0xb667623b, 17 },
    { 0x53b7623c, 93 },
    { 0x7a57623c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36776240, 98 },
    { 0x81d76241, 67 },
    { 0, 0 },
    { 0x6a76243, 100 },
    { 0xf6d76244, 9 },
    { 0x7a176245, 52 },
    { 0xd7d76245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b7624b, 33 },
    { 0x5b7624c, 34 },
    { 0xcd17624d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93876258, 61 },
    { 0, 0 },
    { 0x3e17625a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1117625d, 66 },
    { 0x2f87625e, 46 },
    { 0x667625e, 36 },
    { 0x82476260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d76263, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7447626a, 81 },
    { 0xd5d7626a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4976270, 33 },
    { 0, 0 },
    { 0x25776272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a976278, 58 },
    { 0xd4a76279, 78 },
    { 0x9e576278, 61 },
    { 0x72f76279, 43 },
    { 0xda87627c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3776281, 34 },
    { 0x2a976281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc76286, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a87628f, 90 },
    { 0, 0 },
    { 0x2c176291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76276295, 33 },
    { 0xb1a76295, 23 },
    { 0x8e976297, 88 },
    { 0x8c276297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297629e, 66 },
    { 0x4147629e, 33 },
    { 0x607762a0, 87 },
    { 0x56f762a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7762ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c762b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8762b3, 87 },
    { 0x73762b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03762b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582762bc, 76 },
    { 0x15b762bd, 23 },
    { 0, 0 },
    { 0xb7e762bf, 90 },
    { 0xeb0762c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae762c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55762c7, 95 },
    { 0, 0 },
    { 0xac1762c9, 68 },
    { 0x517762ca, 33 },
    { 0x3bf762cb, 95 },
    { 0x975762cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da762cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf762d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91762d7, 87 },
    { 0x67c762d8, 93 },
    { 0xd6c762d9, 87 },
    { 0, 0 },
    { 0x88e762db, 61 },
    { 0xebd762dc, 55 },
    { 0xfdf762dd, 78 },
    { 0xd26762db, 95 },
    { 0, 0 },
    { 0xa7f762e0, 76 },
    { 0, 0 },
    { 0xfba762e2, 34 },
    { 0xd40762e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad762e7, 98 },
    { 0, 0 },
    { 0xcf3762e9, 33 },
    { 0, 0 },
    { 0xc8f762eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906762ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc762f2, 98 },
    { 0x296762f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1762f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d762fb, 33 },
    { 0, 0 },
    { 0x3f1762fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0276308, 95 },
    { 0x3ad76309, 80 },
    { 0x6317630a, 98 },
    { 0, 0 },
    { 0x70d7630c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb677630f, 56 },
    { 0, 0 },
    { 0x42f76311, 55 },
    { 0x79576312, 61 },
    { 0x9e476312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f7631f, 33 },
    { 0xba576320, 34 },
    { 0xd1976320, 55 },
    { 0x9dc76322, 66 },
    { 0xb0b76322, 33 },
    { 0, 0 },
    { 0x78176325, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b476328, 81 },
    { 0xd6d76329, 78 },
    { 0x3f97632a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7632e, 33 },
    { 0, 0 },
    { 0xb2176330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38076337, 88 },
    { 0, 0 },
    { 0xf4576339, 76 },
    { 0, 0 },
    { 0xd387633b, 76 },
    { 0, 0 },
    { 0x677633d, 81 },
    { 0, 0 },
    { 0x56c7633f, 93 },
    { 0x55a76340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d76344, 81 },
    { 0xec76344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26276348, 52 },
    { 0, 0 },
    { 0xf807634a, 84 },
    { 0, 0 },
    { 0xfc67634c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a76352, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f76356, 90 },
    { 0xd0576357, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c77635c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847635f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f76362, 84 },
    { 0, 0 },
    { 0x31a76364, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9276367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7636e, 95 },
    { 0, 0 },
    { 0x76876370, 17 },
    { 0xafc76371, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e76375, 67 },
    { 0xfd176375, 90 },
    { 0x19576375, 98 },
    { 0x75576378, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77637c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15076380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c976385, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87576388, 95 },
    { 0, 0 },
    { 0x9b7638a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a076390, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae76393, 87 },
    { 0xe876394, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a76397, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae7639b, 23 },
    { 0x3fe7639c, 98 },
    { 0x3f67639c, 23 },
    { 0x52c7639e, 23 },
    { 0x5687639d, 95 },
    { 0xebe763a0, 90 },
    { 0xdd9763a1, 95 },
    { 0xc36763a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc763a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb763a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6763ab, 95 },
    { 0xdcd763ac, 67 },
    { 0, 0 },
    { 0x884763ae, 85 },
    { 0, 0 },
    { 0x6c2763b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5763b3, 61 },
    { 0xbd763b4, 95 },
    { 0x815763b5, 23 },
    { 0xc83763b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6763b9, 95 },
    { 0xf7f763ba, 98 },
    { 0x833763bb, 85 },
    { 0xb06763bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd763c2, 77 },
    { 0, 0 },
    { 0x6a5763c4, 55 },
    { 0, 0 },
    { 0xf9e763c6, 90 },
    { 0xbd4763c7, 61 },
    { 0, 0 },
    { 0xe7f763c9, 17 },
    { 0xdfb763ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363763ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66763d2, 99 },
    { 0x25e763d3, 17 },
    { 0, 0 },
    { 0xba1763d5, 85 },
    { 0, 0 },
    { 0x1c0763d7, 40 },
    { 0, 0 },
    { 0xcb2763d9, 23 },
    { 0x8c3763d9, 76 },
    { 0xe18763db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe763de, 90 },
    { 0, 0 },
    { 0x720763e0, 33 },
    { 0x3d4763e1, 90 },
    { 0xf03763e1, 77 },
    { 0, 0 },
    { 0x31d763e4, 90 },
    { 0x39a763e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc763eb, 61 },
    { 0x76763ec, 95 },
    { 0x43e763eb, 82 },
    { 0xcf6763ee, 93 },
    { 0x11c763ef, 75 },
    { 0x200763eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238763f7, 34 },
    { 0, 0 },
    { 0x491763f9, 61 },
    { 0, 0 },
    { 0xa1a763fb, 33 },
    { 0x803763fc, 23 },
    { 0xb37763fd, 68 },
    { 0xd08763fd, 55 },
    { 0x60e763fb, 98 },
    { 0, 0 },
    { 0xb4c76401, 98 },
    { 0x2476402, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e776406, 52 },
    { 0, 0 },
    { 0xbec76408, 55 },
    { 0, 0 },
    { 0x6d97640a, 23 },
    { 0, 0 },
    { 0x16b7640c, 88 },
    { 0x3a97640d, 85 },
    { 0xc277640d, 96 },
    { 0x7307640e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6376412, 23 },
    { 0, 0 },
    { 0xd3476414, 23 },
    { 0xe1576414, 40 },
    { 0x37176416, 23 },
    { 0xda776417, 61 },
    { 0, 0 },
    { 0xc8c76419, 75 },
    { 0, 0 },
    { 0x6ee7641b, 33 },
    { 0x4477641c, 43 },
    { 0x1517641b, 95 },
    { 0xba17641e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89776424, 23 },
    { 0x80c76424, 61 },
    { 0x91a76424, 61 },
    { 0x67b76426, 61 },
    { 0x20476426, 66 },
    { 0xf4d76429, 84 },
    { 0, 0 },
    { 0x3c07642b, 61 },
    { 0xb107642c, 98 },
    { 0x2987642c, 90 },
    { 0xc1d7642e, 43 },
    { 0, 0 },
    { 0xd976430, 46 },
    { 0xc9476431, 23 },
    { 0x63976431, 76 },
    { 0xd1a76431, 33 },
    { 0x46876434, 87 },
    { 0xdf76435, 90 },
    { 0x32176430, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48076439, 85 },
    { 0xa097643a, 78 },
    { 0x3fe7643b, 38 },
    { 0x88c7643c, 91 },
    { 0x96c7643b, 55 },
    { 0x227643c, 17 },
    { 0xd937643a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14076442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76076446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07644b, 98 },
    { 0x6497644c, 90 },
    { 0x26f7644d, 95 },
    { 0, 0 },
    { 0x88f7644f, 90 },
    { 0, 0 },
    { 0x22376451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89476456, 77 },
    { 0x99176457, 87 },
    { 0xbe576456, 85 },
    { 0, 0 },
    { 0xc3e7645a, 98 },
    { 0, 0 },
    { 0x94e7645c, 95 },
    { 0, 0 },
    { 0x4467645e, 98 },
    { 0x8937645f, 98 },
    { 0, 0 },
    { 0xb6976461, 98 },
    { 0, 0 },
    { 0xa9676463, 52 },
    { 0, 0 },
    { 0x3f776465, 76 },
    { 0, 0 },
    { 0xf7976467, 99 },
    { 0x27c76468, 34 },
    { 0xf8776469, 33 },
    { 0x82276469, 98 },
    { 0x7b17646b, 76 },
    { 0x41276469, 33 },
    { 0x27c7646a, 43 },
    { 0x4187646b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f776471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b76474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7527647b, 61 },
    { 0x3c7647b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7647f, 61 },
    { 0x8f376480, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf76486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b876489, 99 },
    { 0x2bc7648a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d7648d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8376495, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead76499, 90 },
    { 0xa8d76499, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a7649e, 43 },
    { 0, 0 },
    { 0x1aa764a0, 38 },
    { 0xc97764a1, 98 },
    { 0x3ba764a2, 33 },
    { 0x54764a2, 66 },
    { 0, 0 },
    { 0xb23764a5, 70 },
    { 0x9ad764a6, 23 },
    { 0x6c6764a7, 95 },
    { 0x876764a5, 55 },
    { 0x4b1764a9, 90 },
    { 0, 0 },
    { 0x6f7764ab, 95 },
    { 0x98764ab, 95 },
    { 0x477764ad, 34 },
    { 0xc42764ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc764b8, 46 },
    { 0xe22764b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3764bc, 95 },
    { 0, 0 },
    { 0xe05764be, 9 },
    { 0xd65764be, 90 },
    { 0x132764c0, 55 },
    { 0xe68764c0, 76 },
    { 0x767764c2, 95 },
    { 0xf39764c3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173764cb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f764cf, 82 },
    { 0xb43764d0, 75 },
    { 0, 0 },
    { 0x188764d2, 98 },
    { 0x927764d3, 33 },
    { 0, 0 },
    { 0x5f6764d5, 61 },
    { 0x9e1764d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27764d9, 93 },
    { 0x771764da, 91 },
    { 0xe9b764da, 95 },
    { 0, 0 },
    { 0xb4b764dd, 23 },
    { 0xc91764de, 46 },
    { 0xb19764df, 43 },
    { 0x4da764df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10764f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40764f3, 98 },
    { 0xff8764f3, 33 },
    { 0x875764f5, 55 },
    { 0xad2764f6, 33 },
    { 0xe91764f6, 95 },
    { 0x523764f8, 33 },
    { 0xe39764f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba764fc, 33 },
    { 0, 0 },
    { 0x59d764fe, 33 },
    { 0xd0e764fe, 76 },
    { 0, 0 },
    { 0xea176501, 61 },
    { 0xfad76502, 33 },
    { 0x81876503, 95 },
    { 0x6cd76503, 33 },
    { 0, 0 },
    { 0xd1076506, 59 },
    { 0x8a376507, 43 },
    { 0, 0 },
    { 0xc4d76509, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad47650e, 23 },
    { 0, 0 },
    { 0xd8876510, 61 },
    { 0x57876511, 61 },
    { 0x47b76512, 93 },
    { 0x13676513, 91 },
    { 0x22476514, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e176517, 93 },
    { 0, 0 },
    { 0x4b576519, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97651f, 43 },
    { 0x30c76520, 85 },
    { 0xc2076521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9176529, 33 },
    { 0xd3776529, 55 },
    { 0x1167652a, 23 },
    { 0x5587652c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227652f, 55 },
    { 0, 0 },
    { 0x23f76531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d77653a, 23 },
    { 0, 0 },
    { 0x9fd7653c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76540, 77 },
    { 0, 0 },
    { 0xf9476542, 61 },
    { 0xdf876543, 93 },
    { 0x53476544, 55 },
    { 0x22776544, 75 },
    { 0x5cf76546, 17 },
    { 0x2e76546, 23 },
    { 0x1176545, 90 },
    { 0x53976549, 76 },
    { 0x9bb76549, 90 },
    { 0x80976549, 99 },
    { 0x64876544, 67 },
    { 0x5f476544, 95 },
    { 0x38f7654e, 24 },
    { 0xd2276548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35576552, 33 },
    { 0x5eb76553, 33 },
    { 0xee976552, 43 },
    { 0, 0 },
    { 0xdb576556, 55 },
    { 0, 0 },
    { 0xd1a76558, 66 },
    { 0xe0776558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe76562, 78 },
    { 0xa4b76562, 98 },
    { 0, 0 },
    { 0x35b76565, 33 },
    { 0x5b76565, 93 },
    { 0xe9876566, 87 },
    { 0x75576565, 23 },
    { 0xc8a76567, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db7656d, 68 },
    { 0x6a47656d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3176571, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19776576, 100 },
    { 0x4ee76577, 55 },
    { 0, 0 },
    { 0xf8776579, 87 },
    { 0xee97657a, 88 },
    { 0x8507657b, 95 },
    { 0x2157657c, 66 },
    { 0x3647657c, 90 },
    { 0x9267657e, 33 },
    { 0x1357657c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74376584, 43 },
    { 0, 0 },
    { 0x85f76586, 33 },
    { 0x72e76587, 98 },
    { 0, 0 },
    { 0x22676589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce7658f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3176593, 61 },
    { 0xdef76593, 55 },
    { 0x92a76595, 90 },
    { 0x35076595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3617659f, 61 },
    { 0x7367659f, 77 },
    { 0x4fe7659f, 93 },
    { 0, 0 },
    { 0xa93765a3, 23 },
    { 0, 0 },
    { 0x90765a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1765ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f765b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d765b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10765b7, 70 },
    { 0xdde765b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c765bb, 43 },
    { 0xc8f765bc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb765bf, 90 },
    { 0x972765c0, 76 },
    { 0xb26765c1, 78 },
    { 0x59765c0, 61 },
    { 0xeb9765c3, 55 },
    { 0, 0 },
    { 0x18e765c5, 55 },
    { 0, 0 },
    { 0x62765c7, 99 },
    { 0, 0 },
    { 0x9d5765c9, 23 },
    { 0x2e0765c9, 23 },
    { 0x990765cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c765ce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb765d1, 65 },
    { 0, 0 },
    { 0x8b765d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x376765d6, 33 },
    { 0, 0 },
    { 0x34a765d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef765dd, 76 },
    { 0xc6a765dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a765e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943765e6, 61 },
    { 0xa32765e7, 33 },
    { 0x2b765e7, 93 },
    { 0, 0 },
    { 0xf95765ea, 95 },
    { 0x1d1765eb, 46 },
    { 0, 0 },
    { 0x3dc765ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190765f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817765f6, 75 },
    { 0x5e1765f7, 55 },
    { 0x4f3765f7, 76 },
    { 0, 0 },
    { 0x4c2765fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca76602, 93 },
    { 0x8d976603, 100 },
    { 0xd0876603, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21176607, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287660b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e17660e, 55 },
    { 0x2a87660f, 67 },
    { 0x9f27660f, 34 },
    { 0x6e376611, 61 },
    { 0, 0 },
    { 0x71376613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7661d, 55 },
    { 0x19d7661e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f76621, 77 },
    { 0xf0576622, 77 },
    { 0, 0 },
    { 0xe1876624, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b076628, 90 },
    { 0, 0 },
    { 0x7ce7662a, 95 },
    { 0, 0 },
    { 0x5977662c, 81 },
    { 0x4767662d, 33 },
    { 0, 0 },
    { 0x5707662f, 61 },
    { 0, 0 },
    { 0x24276631, 95 },
    { 0x9ba76632, 61 },
    { 0x92b76633, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3476639, 33 },
    { 0, 0 },
    { 0x3b37663b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d76640, 87 },
    { 0xf476640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb76644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8976648, 46 },
    { 0x8f476649, 98 },
    { 0xd576649, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7664e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6776652, 61 },
    { 0x9f776653, 33 },
    { 0, 0 },
    { 0xfc576655, 95 },
    { 0, 0 },
    { 0xc2976657, 61 },
    { 0x9d676658, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4287665c, 61 },
    { 0x4b77665d, 33 },
    { 0xecb7665d, 52 },
    { 0xc507665e, 33 },
    { 0xaa776660, 91 },
    { 0x8d876661, 95 },
    { 0x3de76662, 46 },
    { 0xb1476663, 61 },
    { 0xa0b76664, 95 },
    { 0x26b76665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8576669, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7666e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62776671, 55 },
    { 0, 0 },
    { 0x2576673, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92476679, 98 },
    { 0x23f7667a, 98 },
    { 0x6947667b, 33 },
    { 0xca7667a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d676684, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x81276687, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db7668a, 93 },
    { 0, 0 },
    { 0x9ba7668c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b76694, 91 },
    { 0xeae76695, 90 },
    { 0xa7376696, 66 },
    { 0xe0476697, 75 },
    { 0x28a76698, 66 },
    { 0x76e76695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d7669d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b766a1, 90 },
    { 0x87a766a1, 52 },
    { 0, 0 },
    { 0xa82766a4, 33 },
    { 0x3b6766a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58766a8, 91 },
    { 0, 0 },
    { 0x3fd766aa, 33 },
    { 0xa30766aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb766b2, 23 },
    { 0xf34766b2, 90 },
    { 0x71766b4, 40 },
    { 0x9ff766b4, 98 },
    { 0x916766b3, 76 },
    { 0x8bb766b4, 38 },
    { 0x935766b8, 17 },
    { 0xcfa766b9, 23 },
    { 0x5d0766ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed766be, 90 },
    { 0, 0 },
    { 0xef3766c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b766ca, 33 },
    { 0xc1d766ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f766d0, 95 },
    { 0xadd766d1, 17 },
    { 0x381766d2, 90 },
    { 0x586766d2, 46 },
    { 0x670766d3, 55 },
    { 0xac5766d1, 55 },
    { 0xd26766d6, 66 },
    { 0x734766d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x732766da, 48 },
    { 0x735766db, 37 },
    { 0x568766dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3766e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa766e5, 93 },
    { 0xd2766e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a766ea, 80 },
    { 0x7af766ea, 100 },
    { 0, 0 },
    { 0xee5766ed, 61 },
    { 0, 0 },
    { 0x991766ef, 52 },
    { 0xd7d766f0, 90 },
    { 0x21f766f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e766f6, 87 },
    { 0x276766f6, 33 },
    { 0xa04766f6, 90 },
    { 0x4766f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a766fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d976701, 61 },
    { 0, 0 },
    { 0x9a076703, 90 },
    { 0x64c76704, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97670a, 90 },
    { 0x7747670b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3457670e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1376712, 46 },
    { 0xb5876713, 95 },
    { 0x61f76713, 65 },
    { 0, 0 },
    { 0x7d276716, 61 },
    { 0x2a276717, 43 },
    { 0xd3376718, 55 },
    { 0, 0 },
    { 0x8707671a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c7671e, 33 },
    { 0xb807671f, 23 },
    { 0x3cf76720, 87 },
    { 0x5a976720, 76 },
    { 0x2697671e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd76725, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38876733, 23 },
    { 0xbf76734, 23 },
    { 0x94f76734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf476739, 35 },
    { 0, 0 },
    { 0x2887673b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee07673f, 76 },
    { 0x4cb76740, 90 },
    { 0xaa676740, 90 },
    { 0xf9a7673f, 90 },
    { 0x3a876743, 90 },
    { 0xe7876741, 90 },
    { 0, 0 },
    { 0xb0c76746, 95 },
    { 0x81776746, 84 },
    { 0, 0 },
    { 0xcbd76749, 76 },
    { 0x8e47674a, 87 },
    { 0x3557674b, 46 },
    { 0, 0 },
    { 0x56f7674d, 67 },
    { 0xe3c7674d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14476752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b76755, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2997675d, 90 },
    { 0, 0 },
    { 0xe97675f, 98 },
    { 0x99176760, 61 },
    { 0x82376760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac76764, 90 },
    { 0xa3a76765, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a276769, 33 },
    { 0x8827676a, 51 },
    { 0xb607676b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b76775, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca76778, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8487677b, 84 },
    { 0x2c37677c, 100 },
    { 0xe7f7677d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x376780, 90 },
    { 0, 0 },
    { 0x58b76782, 81 },
    { 0x2c176782, 43 },
    { 0xc9b76784, 76 },
    { 0, 0 },
    { 0x3de76786, 90 },
    { 0xab676786, 33 },
    { 0x22676787, 84 },
    { 0x38876788, 81 },
    { 0x4a976789, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e67678e, 99 },
    { 0xcc47678e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e76795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d76799, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc7679e, 77 },
    { 0xf907679f, 55 },
    { 0, 0 },
    { 0x871767a1, 98 },
    { 0x4f9767a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7767a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3767b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544767b9, 95 },
    { 0x1cf767ba, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06767bd, 87 },
    { 0x76f767be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2767c2, 17 },
    { 0x893767c3, 100 },
    { 0, 0 },
    { 0x524767c5, 95 },
    { 0x405767c6, 90 },
    { 0, 0 },
    { 0x3de767c8, 23 },
    { 0x5c1767c9, 90 },
    { 0x885767c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32767cd, 33 },
    { 0x7be767ce, 43 },
    { 0, 0 },
    { 0xf9b767d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a767d3, 23 },
    { 0, 0 },
    { 0x859767d5, 96 },
    { 0, 0 },
    { 0xad2767d7, 90 },
    { 0x527767d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41767db, 61 },
    { 0x626767dc, 76 },
    { 0xa8e767dd, 61 },
    { 0xc76767de, 80 },
    { 0xb8f767df, 90 },
    { 0xb7e767df, 98 },
    { 0x7bb767e1, 90 },
    { 0, 0 },
    { 0x88e767e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4767e6, 33 },
    { 0x215767e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537767f1, 68 },
    { 0, 0 },
    { 0x49c767f3, 23 },
    { 0, 0 },
    { 0x48767f5, 90 },
    { 0x338767f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27767f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4767fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d376800, 95 },
    { 0xc4c76801, 77 },
    { 0x11076801, 66 },
    { 0xf9d76801, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xd576806, 61 },
    { 0x6b776807, 55 },
    { 0x70d76806, 26 },
    { 0, 0 },
    { 0xea87680a, 52 },
    { 0x7617680b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e76810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e76818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257681d, 76 },
    { 0x6c7681e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a976823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47976827, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3807682c, 33 },
    { 0xa77682d, 98 },
    { 0x5e27682e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90976834, 55 },
    { 0x8e976835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a76838, 33 },
    { 0x9b476839, 98 },
    { 0x1bb7683a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7683d, 61 },
    { 0x41c7683d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f976841, 23 },
    { 0xc3976842, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c276845, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c57684f, 76 },
    { 0, 0 },
    { 0x63976851, 52 },
    { 0x77276851, 100 },
    { 0x2a176853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a76858, 17 },
    { 0x4dd76858, 95 },
    { 0xb0376858, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7685e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e76862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29676866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f476869, 68 },
    { 0x2e87686a, 90 },
    { 0, 0 },
    { 0xc517686c, 88 },
    { 0xa0f7686d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85776873, 52 },
    { 0x6e876873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897687f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a76882, 84 },
    { 0xbd76883, 88 },
    { 0x90f76883, 33 },
    { 0x73576885, 33 },
    { 0xe5f76886, 85 },
    { 0x3c676887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7688f, 61 },
    { 0xee17688f, 99 },
    { 0x46f76891, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x60676894, 55 },
    { 0x15276894, 23 },
    { 0, 0 },
    { 0x75a76897, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567689b, 98 },
    { 0, 0 },
    { 0x3287689d, 100 },
    { 0xe1c7689e, 76 },
    { 0x1297689f, 90 },
    { 0xe107689f, 76 },
    { 0x8837689e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7768a7, 95 },
    { 0x2f0768a8, 84 },
    { 0xc44768a7, 95 },
    { 0, 0 },
    { 0xe1e768ab, 98 },
    { 0x933768ab, 93 },
    { 0x924768ad, 46 },
    { 0x822768ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650768b2, 61 },
    { 0xb1c768b2, 23 },
    { 0xb8768b4, 34 },
    { 0xd0f768b5, 43 },
    { 0, 0 },
    { 0x3f7768b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9768bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd768c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16768c4, 23 },
    { 0xdc9768c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb768cb, 84 },
    { 0, 0 },
    { 0x9a9768cd, 78 },
    { 0, 0 },
    { 0x4ca768cf, 34 },
    { 0xf60768d0, 98 },
    { 0, 0 },
    { 0xce2768d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a768d5, 43 },
    { 0, 0 },
    { 0x9bf768d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b768dc, 90 },
    { 0x824768dd, 93 },
    { 0xcc7768dd, 61 },
    { 0, 0 },
    { 0xb9e768e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x455768e3, 23 },
    { 0x21e768e4, 95 },
    { 0x1dc768e3, 46 },
    { 0x720768e6, 61 },
    { 0xa76768e5, 33 },
    { 0xa49768e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357768f1, 23 },
    { 0, 0 },
    { 0xec8768f3, 17 },
    { 0, 0 },
    { 0x23d768f5, 33 },
    { 0x8ff768f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a768f9, 46 },
    { 0xf84768fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8876905, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f07690b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f67690f, 90 },
    { 0x4fa7690f, 23 },
    { 0xaf676911, 34 },
    { 0xe7576911, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87691e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c876924, 100 },
    { 0x57276925, 66 },
    { 0, 0 },
    { 0x30e76927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1667692a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25576933, 98 },
    { 0x8e276934, 43 },
    { 0xf9876933, 38 },
    { 0, 0 },
    { 0x40376937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097693a, 33 },
    { 0, 0 },
    { 0x98b7693c, 43 },
    { 0x1b37693c, 33 },
    { 0xac27693e, 98 },
    { 0x6607693f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c76942, 66 },
    { 0xca076942, 43 },
    { 0x69176944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e76947, 55 },
    { 0xe6576948, 33 },
    { 0x35476948, 98 },
    { 0xa9c7694a, 85 },
    { 0x29176949, 34 },
    { 0xef07694b, 80 },
    { 0x8347694a, 96 },
    { 0, 0 },
    { 0x3a7694f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde276952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b76955, 23 },
    { 0xa1c76955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7695a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1327695f, 61 },
    { 0, 0 },
    { 0x55c76961, 91 },
    { 0x41e76962, 56 },
    { 0xbf876963, 34 },
    { 0xe276964, 98 },
    { 0x99f76965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f7696b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe67696f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2476977, 61 },
    { 0x93c76978, 55 },
    { 0xf176977, 90 },
    { 0, 0 },
    { 0x29b7697b, 61 },
    { 0xd87697c, 61 },
    { 0, 0 },
    { 0x5617697e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d876988, 23 },
    { 0x6c276988, 23 },
    { 0x8d276989, 34 },
    { 0xdaa7698b, 84 },
    { 0xf407698b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547698f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54176994, 95 },
    { 0xbf676995, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc76998, 61 },
    { 0, 0 },
    { 0xf37699a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d87699e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x160769a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe769a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x897769aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a769ae, 90 },
    { 0, 0 },
    { 0x8bd769b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d769b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62769b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2769c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4769c6, 95 },
    { 0, 0 },
    { 0x638769c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759769ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x560769d1, 99 },
    { 0x5fb769d2, 100 },
    { 0x82b769d3, 56 },
    { 0xef8769d3, 93 },
    { 0x12d769d2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae769d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b769dd, 76 },
    { 0xb85769de, 99 },
    { 0xb3769dd, 23 },
    { 0x96c769dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a769e3, 90 },
    { 0x8b7769e4, 46 },
    { 0x462769e4, 23 },
    { 0xf56769e4, 23 },
    { 0x20f769e4, 33 },
    { 0xcbd769e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4769ec, 85 },
    { 0x5a1769ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5769f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06769f7, 23 },
    { 0x570769f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b769fb, 90 },
    { 0, 0 },
    { 0xff9769fd, 52 },
    { 0x55e769fe, 33 },
    { 0xcc5769fd, 61 },
    { 0x50b76a00, 76 },
    { 0xaae76a01, 80 },
    { 0x8e976a00, 61 },
    { 0x9cd76a01, 76 },
    { 0, 0 },
    { 0x7d476a05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc76a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4576a14, 95 },
    { 0xbe776a15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf176a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e576a27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c76a2a, 90 },
    { 0x2af76a2a, 99 },
    { 0x1fe76a2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1876a2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae976a36, 23 },
    { 0, 0 },
    { 0xae676a38, 46 },
    { 0x53376a39, 52 },
    { 0x1e876a3a, 23 },
    { 0xce076a3b, 33 },
    { 0x53176a3c, 96 },
    { 0x42276a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e776a42, 53 },
    { 0xdbc76a43, 93 },
    { 0xf9d76a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22076a4b, 33 },
    { 0xbb376a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e76a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf176a59, 23 },
    { 0x9576a5a, 95 },
    { 0xd5876a5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xce576a5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8576a63, 61 },
    { 0, 0 },
    { 0x5f376a65, 23 },
    { 0x93476a65, 84 },
    { 0x8ce76a65, 76 },
    { 0xa6e76a68, 40 },
    { 0x25676a68, 95 },
    { 0x30376a69, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76376a6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf76a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea76a7a, 61 },
    { 0x80476a7b, 55 },
    { 0x6a276a7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39876a7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca76a86, 88 },
    { 0xe2a76a87, 90 },
    { 0x38076a88, 99 },
    { 0x1ad76a87, 95 },
    { 0x35b76a8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c76a8f, 33 },
    { 0xf4976a90, 90 },
    { 0x39376a90, 95 },
    { 0x9b376a90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0376a95, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e976a99, 76 },
    { 0x5f76a9a, 90 },
    { 0x7ff76a9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d376aa0, 17 },
    { 0x4d676aa1, 17 },
    { 0x32d76aa1, 43 },
    { 0xff676aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51776aa7, 78 },
    { 0xb9776aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c76aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d476ab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad376ab7, 90 },
    { 0x73776ab8, 90 },
    { 0x89e76ab9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e76abc, 59 },
    { 0x8b976abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b76ac0, 33 },
    { 0x3b876ac1, 90 },
    { 0x5fb76ac2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x36476ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf376ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb876ad0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca76ad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f176adb, 95 },
    { 0x6ac76adb, 84 },
    { 0, 0 },
    { 0x21f76ade, 90 },
    { 0x5db76adf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd76aea, 78 },
    { 0x30676aea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5076aef, 98 },
    { 0xd5976af0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76af8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176b02, 90 },
    { 0, 0 },
    { 0xf2676b04, 23 },
    { 0xc9076b05, 23 },
    { 0, 0 },
    { 0x74776b07, 23 },
    { 0, 0 },
    { 0xca976b09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e076b0e, 95 },
    { 0x41876b0e, 100 },
    { 0x10f76b0f, 66 },
    { 0, 0 },
    { 0xa0276b12, 72 },
    { 0x81976b12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab76b17, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x19376b1a, 77 },
    { 0, 0 },
    { 0x45876b1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa476b23, 33 },
    { 0x50476b23, 33 },
    { 0xdba76b25, 33 },
    { 0x1dd76b26, 67 },
    { 0xa2c76b27, 33 },
    { 0, 0 },
    { 0xea376b29, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f376b2c, 61 },
    { 0xab876b2d, 100 },
    { 0, 0 },
    { 0xa5a76b2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa976b32, 84 },
    { 0x3a76b33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb976b3a, 66 },
    { 0xd7a76b3b, 99 },
    { 0, 0 },
    { 0x6a976b3d, 43 },
    { 0xe7776b3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81376b41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d76b45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f76b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b76b53, 46 },
    { 0, 0 },
    { 0x15076b55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d476b5a, 87 },
    { 0x12176b5b, 98 },
    { 0x14776b5c, 98 },
    { 0xb7a76b5d, 23 },
    { 0xd8f76b5e, 76 },
    { 0x7576b5e, 98 },
    { 0xeb776b5d, 90 },
    { 0xc9376b61, 33 },
    { 0x2a176b5f, 23 },
    { 0x7f776b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f476b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68776b6d, 90 },
    { 0xc1676b6d, 81 },
    { 0, 0 },
    { 0x4d876b70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b76b75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e676b78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4676b7f, 78 },
    { 0x2f276b80, 90 },
    { 0xc6776b80, 61 },
    { 0x176b82, 24 },
    { 0, 0 },
    { 0xbcc76b84, 23 },
    { 0x57576b85, 76 },
    { 0xdd276b86, 23 },
    { 0, 0 },
    { 0x41176b88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a76b8e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83276b93, 23 },
    { 0x69c76b94, 55 },
    { 0, 0 },
    { 0xf3776b96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d76b9b, 61 },
    { 0, 0 },
    { 0x6b876b9d, 90 },
    { 0x41e76b9d, 43 },
    { 0, 0 },
    { 0x1a76ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc176ba3, 99 },
    { 0, 0 },
    { 0x77876ba5, 91 },
    { 0xb8076ba6, 76 },
    { 0x9c776ba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac276bba, 61 },
    { 0xa976bbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5376bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a076bc2, 100 },
    { 0, 0 },
    { 0x57e76bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6576bc8, 23 },
    { 0, 0 },
    { 0xd8776bca, 34 },
    { 0x72f76bca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8476bce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce76bd9, 52 },
    { 0x63576bd9, 87 },
    { 0x81676bda, 90 },
    { 0, 0 },
    { 0xf3276bdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8276be0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c076be3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e476be8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a76bec, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa76bef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8576bf2, 95 },
    { 0x81176bf3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e876bf9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc76bfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a576c06, 93 },
    { 0xa5476c07, 99 },
    { 0, 0 },
    { 0x3b376c09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4676c0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d76c10, 68 },
    { 0x9a576c10, 67 },
    { 0x69076c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f076c1f, 98 },
    { 0, 0 },
    { 0x5976c21, 66 },
    { 0x69576c22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b76c28, 91 },
    { 0x20276c29, 61 },
    { 0xfa476c2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f76c32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b676c35, 52 },
    { 0x60276c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92676c39, 33 },
    { 0x51476c3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6576c3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17576c41, 23 },
    { 0xc3e76c41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57676c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9876c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48676c54, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb76c58, 33 },
    { 0xd1776c58, 90 },
    { 0x54176c58, 95 },
    { 0, 0 },
    { 0x82f76c5c, 40 },
    { 0, 0 },
    { 0x11976c5e, 98 },
    { 0xa6f76c5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d76c62, 46 },
    { 0x40b76c62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac676c67, 87 },
    { 0, 0 },
    { 0xe8776c69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b76c6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0876c72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd76c75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48376c7a, 76 },
    { 0, 0 },
    { 0x9fa76c7c, 90 },
    { 0x11576c7d, 100 },
    { 0xfa876c7e, 73 },
    { 0x15176c7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5176c82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b76c85, 52 },
    { 0xc4676c86, 98 },
    { 0x9bc76c86, 34 },
    { 0xc5c76c88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b76c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a76c92, 47 },
    { 0x5ff76c93, 34 },
    { 0xe3d76c94, 34 },
    { 0, 0 },
    { 0x1df76c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b76c99, 91 },
    { 0xa9d76c9a, 43 },
    { 0x5da76c9b, 33 },
    { 0xfd176c9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea76ca1, 95 },
    { 0x98f76ca2, 90 },
    { 0x94e76ca1, 61 },
    { 0, 0 },
    { 0x9b476ca5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa76ca8, 91 },
    { 0x48776ca9, 23 },
    { 0x6a776caa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41776cb0, 66 },
    { 0xd0676cb1, 33 },
    { 0xb3b76cb1, 80 },
    { 0x7ab76cb3, 90 },
    { 0xdf676cb3, 76 },
    { 0xb9b76cb5, 91 },
    { 0, 0 },
    { 0xf276cb7, 98 },
    { 0x72d76cb7, 33 },
    { 0, 0 },
    { 0x69576cba, 65 },
    { 0xe0876cbb, 95 },
    { 0x9e476cbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad76cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d676cc8, 13 },
    { 0x43d76cc9, 33 },
    { 0x51876cc9, 91 },
    { 0x4e776cc8, 95 },
    { 0, 0 },
    { 0xafa76ccd, 61 },
    { 0xdfe76cce, 88 },
    { 0, 0 },
    { 0x85e76cd0, 90 },
    { 0xa4776cd1, 81 },
    { 0x9d576cd2, 95 },
    { 0xd4076cd0, 90 },
    { 0xf5f76cd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d776cd8, 33 },
    { 0xae576cd8, 33 },
    { 0x7f76cd9, 76 },
    { 0x78676cdb, 43 },
    { 0xde776cdc, 99 },
    { 0x64676cdc, 66 },
    { 0xbb876cde, 100 },
    { 0xff776cd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6376ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58476ce8, 85 },
    { 0xd5976ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c76cef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75676cf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd76cf8, 90 },
    { 0, 0 },
    { 0x27876cfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f76d00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0476d07, 95 },
    { 0, 0 },
    { 0xf5d76d09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda76d0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c76d1d, 43 },
    { 0x2df76d1e, 98 },
    { 0, 0 },
    { 0xec276d20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a76d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63576d27, 34 },
    { 0xaa976d28, 33 },
    { 0x66d76d29, 92 },
    { 0x5d776d27, 23 },
    { 0x16176d27, 85 },
    { 0, 0 },
    { 0xf0876d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16176d34, 80 },
    { 0xd8176d34, 33 },
    { 0x2ce76d36, 99 },
    { 0x42776d37, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29176d3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e76d3e, 23 },
    { 0x41876d3e, 95 },
    { 0xf6676d40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f776d43, 33 },
    { 0xeaf76d43, 43 },
    { 0x62976d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92576d49, 95 },
    { 0x29b76d4a, 23 },
    { 0xbe76d4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e876d4f, 90 },
    { 0, 0 },
    { 0xc1976d51, 88 },
    { 0x4e076d52, 23 },
    { 0x85176d53, 61 },
    { 0x26e76d52, 95 },
    { 0, 0 },
    { 0xe1d76d56, 38 },
    { 0x83976d57, 67 },
    { 0xf6d76d56, 34 },
    { 0x1f76d59, 90 },
    { 0x24476d59, 98 },
    { 0x82c76d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c076d5f, 46 },
    { 0, 0 },
    { 0x42e76d61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3476d64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a076d6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f876d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b76d71, 95 },
    { 0x65176d71, 95 },
    { 0x42c76d73, 76 },
    { 0xb6476d73, 77 },
    { 0x2a276d73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63476d7d, 23 },
    { 0xc3476d7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6776d82, 90 },
    { 0x46376d82, 52 },
    { 0, 0 },
    { 0x16876d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d76d8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda576d9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f676d9f, 96 },
    { 0x17676da0, 90 },
    { 0x22376da1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce976da6, 97 },
    { 0, 0 },
    { 0xa0776da8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa76dad, 87 },
    { 0x5a376dae, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24076db5, 46 },
    { 0, 0 },
    { 0xf7d76db7, 55 },
    { 0xe8d76db7, 55 },
    { 0xa976db7, 55 },
    { 0xdfb76dba, 100 },
    { 0x3f076db9, 93 },
    { 0xbda76dbc, 61 },
    { 0, 0 },
    { 0x2ee76dbe, 95 },
    { 0x48476dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a276dc2, 46 },
    { 0, 0 },
    { 0xee276dc4, 40 },
    { 0, 0 },
    { 0x6d076dc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x29476dc9, 95 },
    { 0, 0 },
    { 0x6eb76dcb, 76 },
    { 0x6d176dcb, 90 },
    { 0x39376dcd, 40 },
    { 0x1c376dce, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed76dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81876dd8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f76ddd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b76de2, 76 },
    { 0, 0 },
    { 0xe976de4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2576dea, 34 },
    { 0x41876dea, 93 },
    { 0, 0 },
    { 0x6ce76ded, 95 },
    { 0, 0 },
    { 0xbd676def, 24 },
    { 0xa7c76df0, 98 },
    { 0x3cb76def, 90 },
    { 0xdfc76df2, 87 },
    { 0x22276df3, 75 },
    { 0x4f076def, 83 },
    { 0x5ed76df0, 90 },
    { 0, 0 },
    { 0xe1276df7, 61 },
    { 0x69e76df8, 90 },
    { 0x34076df7, 81 },
    { 0x3c376dfa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee76e00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb76e04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65076e08, 76 },
    { 0x26e76e09, 98 },
    { 0x3fc76e09, 23 },
    { 0, 0 },
    { 0x24076e0c, 90 },
    { 0, 0 },
    { 0x96276e0e, 52 },
    { 0x49276e0f, 49 },
    { 0, 0 },
    { 0xca776e11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d676e15, 85 },
    { 0, 0 },
    { 0x38276e17, 98 },
    { 0, 0 },
    { 0x53176e19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92476e1d, 98 },
    { 0, 0 },
    { 0x56576e1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee076e2b, 98 },
    { 0x24276e2b, 98 },
    { 0xcbc76e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a76e33, 43 },
    { 0xab876e34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb76e38, 98 },
    { 0xc0976e39, 84 },
    { 0, 0 },
    { 0x4af76e3b, 33 },
    { 0x6a076e3c, 100 },
    { 0x47576e3d, 98 },
    { 0x32276e3e, 90 },
    { 0, 0 },
    { 0x27f76e40, 90 },
    { 0xb5176e41, 91 },
    { 0, 0 },
    { 0xae576e43, 61 },
    { 0x69576e44, 61 },
    { 0, 0 },
    { 0xd776e46, 52 },
    { 0, 0 },
    { 0x69f76e48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c76e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf276e53, 98 },
    { 0x55c76e53, 46 },
    { 0x9f576e55, 68 },
    { 0xc5276e56, 33 },
    { 0xe7976e57, 95 },
    { 0x79b76e58, 90 },
    { 0x9f176e58, 33 },
    { 0x7db76e56, 55 },
    { 0x12376e55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e776e67, 78 },
    { 0x6276e68, 68 },
    { 0, 0 },
    { 0x4676e6a, 55 },
    { 0xbe776e6b, 61 },
    { 0xff576e6b, 94 },
    { 0xf4a76e6d, 98 },
    { 0x8a876e6b, 52 },
    { 0x3e176e6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6676e72, 84 },
    { 0x4c276e73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10576e77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65876e7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82976e80, 90 },
    { 0x56376e81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a76e88, 75 },
    { 0, 0 },
    { 0xfff76e8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef76e8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9776e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e176e95, 99 },
    { 0xa4676e95, 43 },
    { 0x22776e95, 34 },
    { 0, 0 },
    { 0xa6476e99, 33 },
    { 0xc9376e9a, 90 },
    { 0, 0 },
    { 0xc9e76e9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82176ea0, 33 },
    { 0xee76ea0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f076ea8, 100 },
    { 0x3ca76ea9, 66 },
    { 0x75276eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a76eaf, 85 },
    { 0, 0 },
    { 0xa6476eb1, 95 },
    { 0, 0 },
    { 0x1ec76eb3, 95 },
    { 0, 0 },
    { 0xfb76eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4176eb9, 34 },
    { 0x17676eba, 76 },
    { 0x9b776eba, 95 },
    { 0x56c76eb9, 33 },
    { 0x90776ebc, 77 },
    { 0x90276ebe, 98 },
    { 0xa2476ebf, 33 },
    { 0, 0 },
    { 0x476ec1, 33 },
    { 0x84d76ec2, 55 },
    { 0, 0 },
    { 0x84576ec4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276ec9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88176ecc, 93 },
    { 0xeb676ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f76ed0, 90 },
    { 0x81976ed1, 87 },
    { 0, 0 },
    { 0xd9b76ed3, 61 },
    { 0xe5f76ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40976ed8, 43 },
    { 0, 0 },
    { 0x55176eda, 34 },
    { 0, 0 },
    { 0x8d476edc, 51 },
    { 0xa7276edc, 90 },
    { 0xb8976edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a076ee3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde976ee9, 66 },
    { 0, 0 },
    { 0x4b476eeb, 23 },
    { 0x53476eec, 99 },
    { 0x27376eec, 76 },
    { 0, 0 },
    { 0xa8276eef, 90 },
    { 0, 0 },
    { 0x17176ef1, 33 },
    { 0x6eb76ef2, 33 },
    { 0xdf876ef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41376ef6, 99 },
    { 0x29676ef7, 75 },
    { 0x61b76ef8, 23 },
    { 0, 0 },
    { 0x7376efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8676efe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24376f03, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97976f0a, 95 },
    { 0xc3676f0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b76f10, 33 },
    { 0xfe76f11, 23 },
    { 0x29576f12, 98 },
    { 0x5da76f13, 100 },
    { 0xc9b76f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9976f18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x16776f1b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa776f1f, 98 },
    { 0, 0 },
    { 0xe976f21, 80 },
    { 0xf5876f22, 55 },
    { 0x7d776f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62576f2a, 34 },
    { 0x67876f2a, 23 },
    { 0x76576f2c, 95 },
    { 0xbda76f2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54876f31, 61 },
    { 0xc6076f31, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f76f39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe76f3d, 61 },
    { 0, 0 },
    { 0x6c76f3f, 90 },
    { 0xfdf76f40, 95 },
    { 0x8d976f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53676f44, 95 },
    { 0xd2a76f44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42976f48, 90 },
    { 0xd9776f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd76f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34376f54, 67 },
    { 0xfcf76f54, 61 },
    { 0x17d76f55, 43 },
    { 0x89976f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd76f5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb76f5d, 77 },
    { 0, 0 },
    { 0x49876f5f, 23 },
    { 0x85376f60, 90 },
    { 0x50b76f60, 93 },
    { 0xbe776f61, 61 },
    { 0, 0 },
    { 0x24f76f64, 67 },
    { 0xe6276f65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc76f6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4276f71, 90 },
    { 0, 0 },
    { 0x7ca76f73, 90 },
    { 0, 0 },
    { 0x43976f75, 78 },
    { 0, 0 },
    { 0x41576f77, 43 },
    { 0x37e76f77, 33 },
    { 0x24476f79, 23 },
    { 0xaed76f7a, 52 },
    { 0xeb76f79, 61 },
    { 0xd4476f79, 33 },
    { 0xe3e76f7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46576f80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e176f83, 95 },
    { 0, 0 },
    { 0xc3376f85, 33 },
    { 0xaa876f86, 90 },
    { 0, 0 },
    { 0xbe076f88, 23 },
    { 0xd5e76f89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79476f8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e76f90, 87 },
    { 0x78676f90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b576f98, 93 },
    { 0, 0 },
    { 0x7d276f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f76f9d, 55 },
    { 0xa8276f9d, 33 },
    { 0, 0 },
    { 0x10d76fa0, 55 },
    { 0, 0 },
    { 0x84076fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d76fa6, 98 },
    { 0xb5076fa7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce776fac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf876faf, 67 },
    { 0xfd576fb0, 98 },
    { 0x62876fb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28076fb5, 90 },
    { 0xb0176fb6, 40 },
    { 0, 0 },
    { 0x46176fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b76fbb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb976fbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876fc5, 38 },
    { 0xc5676fc6, 98 },
    { 0, 0 },
    { 0xd5476fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47776fcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e876fd3, 98 },
    { 0, 0 },
    { 0x7f976fd5, 100 },
    { 0, 0 },
    { 0x8a476fd7, 90 },
    { 0x86e76fd8, 90 },
    { 0x5e576fd9, 76 },
    { 0xa9b76fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5076fe1, 66 },
    { 0, 0 },
    { 0x2b176fe3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb176fe7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f76fea, 91 },
    { 0x64f76feb, 90 },
    { 0xaf276fec, 91 },
    { 0, 0 },
    { 0x6c676fee, 95 },
    { 0, 0 },
    { 0x90276ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3476ff3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1676ff7, 33 },
    { 0x24676ff8, 55 },
    { 0x276ff9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x32776ffc, 85 },
    { 0xdf976ffc, 33 },
    { 0xf0e76ffe, 95 },
    { 0, 0 },
    { 0x13477000, 76 },
    { 0xaf977000, 23 },
    { 0, 0 },
    { 0xc2b77003, 93 },
    { 0, 0 },
    { 0xabf77005, 41 },
    { 0x63577006, 96 },
    { 0x50c77006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7700a, 61 },
    { 0x5d97700a, 34 },
    { 0, 0 },
    { 0x69c7700d, 33 },
    { 0x8537700d, 90 },
    { 0x7057700f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b77013, 34 },
    { 0x9f777014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca77019, 69 },
    { 0, 0 },
    { 0xb007701b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec077020, 85 },
    { 0x74e77021, 98 },
    { 0, 0 },
    { 0x8a377023, 93 },
    { 0, 0 },
    { 0xbf177025, 95 },
    { 0, 0 },
    { 0x61c77027, 23 },
    { 0x8c477028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb907702c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c877033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48277038, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6437703b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf77040, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21277044, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e077049, 33 },
    { 0, 0 },
    { 0x6f67704b, 66 },
    { 0, 0 },
    { 0x5e57704d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e77051, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xde277054, 44 },
    { 0x8d077055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467705b, 91 },
    { 0x3227705c, 81 },
    { 0x2237705d, 34 },
    { 0, 0 },
    { 0x8637705f, 84 },
    { 0, 0 },
    { 0xd5777061, 90 },
    { 0x20377062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91977067, 100 },
    { 0, 0 },
    { 0x13677069, 99 },
    { 0xdcd7706a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27477078, 23 },
    { 0xb4577079, 33 },
    { 0xecc7707a, 61 },
    { 0x9c677079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6677082, 61 },
    { 0xfc477082, 55 },
    { 0x8277082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d77089, 61 },
    { 0x4a977089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127708d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec77094, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417709a, 90 },
    { 0x5a57709a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7709f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c770a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4770aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde770b4, 66 },
    { 0x8cf770b5, 61 },
    { 0x948770b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61770b9, 23 },
    { 0x13d770ba, 61 },
    { 0x996770bb, 23 },
    { 0x134770bb, 61 },
    { 0x2c770bd, 100 },
    { 0x933770bb, 90 },
    { 0, 0 },
    { 0x435770c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d770c3, 23 },
    { 0, 0 },
    { 0x189770c5, 52 },
    { 0x575770c6, 43 },
    { 0xe29770c6, 98 },
    { 0xd30770c8, 90 },
    { 0xdb4770c9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c770d4, 77 },
    { 0xf14770d5, 27 },
    { 0x4e5770d6, 90 },
    { 0xf21770d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93770da, 85 },
    { 0xe87770db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994770e0, 33 },
    { 0x41a770e1, 90 },
    { 0x806770e2, 98 },
    { 0xc56770e0, 61 },
    { 0xc6c770e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c770f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f770f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145770fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e77100, 76 },
    { 0xc4477100, 93 },
    { 0, 0 },
    { 0xa9177103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4677106, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a977110, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff77115, 91 },
    { 0xd4177115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb7711b, 95 },
    { 0x6477711b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a77121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a7712a, 76 },
    { 0x9e77712a, 76 },
    { 0x7af7712a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd677130, 34 },
    { 0, 0 },
    { 0xa5577132, 100 },
    { 0, 0 },
    { 0x57c77134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d777137, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2897713e, 93 },
    { 0x1687713f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f177145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac977148, 46 },
    { 0x7eb77149, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e7714d, 95 },
    { 0, 0 },
    { 0xedc7714f, 23 },
    { 0x15477150, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f477154, 55 },
    { 0, 0 },
    { 0xd5677156, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db7715b, 78 },
    { 0x3fe7715c, 90 },
    { 0xe647715b, 90 },
    { 0xa2c7715e, 55 },
    { 0xfca7715f, 95 },
    { 0xb407715d, 90 },
    { 0xfb777161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de77164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23777168, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f7716d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x277170, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62777178, 61 },
    { 0x20e77179, 90 },
    { 0x40b77178, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c7717e, 66 },
    { 0xc057717f, 17 },
    { 0xa507717f, 43 },
    { 0xab977181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38577184, 76 },
    { 0x18577185, 90 },
    { 0xf0477185, 98 },
    { 0x8db77187, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d7718d, 90 },
    { 0xc8f7718e, 47 },
    { 0, 0 },
    { 0x69e77190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c177195, 95 },
    { 0x3ef77196, 17 },
    { 0xe2577197, 33 },
    { 0, 0 },
    { 0xa7477199, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1771a4, 90 },
    { 0xdb3771a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e771b0, 65 },
    { 0x26e771b0, 95 },
    { 0xb2e771b2, 80 },
    { 0, 0 },
    { 0xb7c771b4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e771b9, 56 },
    { 0xab6771ba, 95 },
    { 0xcf6771bb, 33 },
    { 0xda3771bc, 55 },
    { 0, 0 },
    { 0x3d9771be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753771c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4771c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f771c8, 61 },
    { 0xae3771c9, 52 },
    { 0xa79771c9, 95 },
    { 0x51b771cb, 33 },
    { 0x56771cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9771cf, 95 },
    { 0, 0 },
    { 0xdba771d1, 76 },
    { 0, 0 },
    { 0x9ef771d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3771d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb771dd, 61 },
    { 0xd85771de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1771e3, 99 },
    { 0xa53771e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f771e7, 91 },
    { 0x4c1771e8, 99 },
    { 0, 0 },
    { 0x5e7771ea, 33 },
    { 0, 0 },
    { 0x1cc771ec, 34 },
    { 0, 0 },
    { 0x82e771ee, 90 },
    { 0x159771ee, 90 },
    { 0xcdf771f0, 61 },
    { 0x16a771ee, 23 },
    { 0x1e3771f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943771f5, 33 },
    { 0x781771f5, 87 },
    { 0x6dd771f5, 95 },
    { 0x96b771f8, 92 },
    { 0x876771f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c771fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b77200, 88 },
    { 0x5eb77201, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77077209, 95 },
    { 0, 0 },
    { 0x4207720b, 52 },
    { 0xcff7720b, 98 },
    { 0x7507720d, 98 },
    { 0xf037720e, 33 },
    { 0xaeb7720e, 100 },
    { 0, 0 },
    { 0x27977211, 67 },
    { 0xc0577212, 95 },
    { 0, 0 },
    { 0xca977214, 33 },
    { 0xb1477215, 66 },
    { 0, 0 },
    { 0xd7677217, 77 },
    { 0x71277217, 87 },
    { 0xefc77219, 100 },
    { 0xab57721a, 23 },
    { 0xdcb7721b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7721f, 91 },
    { 0xd327721f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda077223, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e977227, 33 },
    { 0xe6b77228, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42477234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59877237, 97 },
    { 0xce977238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2967723b, 90 },
    { 0, 0 },
    { 0x91f7723d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58277240, 91 },
    { 0xafc77241, 84 },
    { 0x97577242, 43 },
    { 0xdb77240, 46 },
    { 0x48377241, 95 },
    { 0xe4277240, 33 },
    { 0, 0 },
    { 0x68377247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67724a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227724f, 95 },
    { 0, 0 },
    { 0x98d77251, 90 },
    { 0xef177251, 90 },
    { 0x3b377253, 52 },
    { 0x7e777254, 97 },
    { 0x71577255, 84 },
    { 0xa2f77253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77725c, 90 },
    { 0, 0 },
    { 0x1a47725e, 90 },
    { 0x5e27725f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b77266, 100 },
    { 0x5e277266, 90 },
    { 0x5dd77268, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6487726c, 33 },
    { 0, 0 },
    { 0xa3e7726e, 90 },
    { 0xdb07726f, 48 },
    { 0, 0 },
    { 0xc8777271, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7297727a, 98 },
    { 0xf547727b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26277281, 90 },
    { 0, 0 },
    { 0xecc77283, 98 },
    { 0, 0 },
    { 0x8c177285, 93 },
    { 0x63c77286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1547728c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c677299, 17 },
    { 0, 0 },
    { 0x4cf7729b, 23 },
    { 0, 0 },
    { 0x21f7729d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218772a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca772a9, 61 },
    { 0xa4b772a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9772b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98772b5, 90 },
    { 0x64a772b6, 68 },
    { 0xc03772b7, 93 },
    { 0xbd4772b7, 90 },
    { 0x73f772b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a772bd, 88 },
    { 0x40b772bd, 91 },
    { 0xdaa772bd, 78 },
    { 0x437772c0, 90 },
    { 0xcfa772c1, 90 },
    { 0xe9f772c1, 90 },
    { 0xd48772c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7772c7, 78 },
    { 0, 0 },
    { 0x911772c9, 90 },
    { 0, 0 },
    { 0x1bb772cb, 98 },
    { 0x36c772cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1772cf, 90 },
    { 0xe4a772cf, 84 },
    { 0x609772d1, 61 },
    { 0x8ac772d1, 23 },
    { 0x145772d3, 23 },
    { 0x87c772d1, 33 },
    { 0x39772d5, 65 },
    { 0x7ad772cf, 95 },
    { 0x450772d7, 66 },
    { 0xa2772d8, 33 },
    { 0x7bb772d7, 61 },
    { 0x5ca772da, 76 },
    { 0x5a772db, 98 },
    { 0x3a2772dc, 95 },
    { 0xefd772dc, 43 },
    { 0, 0 },
    { 0xb74772df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d772e2, 95 },
    { 0, 0 },
    { 0xbfa772e4, 55 },
    { 0x772e5, 52 },
    { 0x8be772e6, 55 },
    { 0x2fe772e7, 76 },
    { 0x63772e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849772ec, 98 },
    { 0, 0 },
    { 0xd93772ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b772f6, 23 },
    { 0x918772f7, 90 },
    { 0, 0 },
    { 0x137772f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f772fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc77305, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2797730a, 99 },
    { 0, 0 },
    { 0x6be7730c, 33 },
    { 0x5577730d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca77314, 100 },
    { 0xe9877315, 84 },
    { 0x94c77316, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a77731c, 98 },
    { 0, 0 },
    { 0x6827731e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377326, 90 },
    { 0xd8c77326, 95 },
    { 0, 0 },
    { 0x9a277329, 77 },
    { 0x5847732a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b7732d, 23 },
    { 0x2e27732e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72077333, 23 },
    { 0x90577334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8597733a, 98 },
    { 0, 0 },
    { 0x9a7733c, 52 },
    { 0xab17733d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1877340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17177343, 90 },
    { 0x2cf77344, 93 },
    { 0x7e577344, 91 },
    { 0xbbe77346, 61 },
    { 0xb4c77347, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a97734a, 93 },
    { 0xa6c7734b, 43 },
    { 0x1fc7734c, 46 },
    { 0, 0 },
    { 0x5687734e, 95 },
    { 0xb887734e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d377355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c7735a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc347735d, 98 },
    { 0x2607735e, 23 },
    { 0xded7735d, 90 },
    { 0xe6c77360, 95 },
    { 0, 0 },
    { 0xcd277362, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c77366, 98 },
    { 0x6c77366, 87 },
    { 0, 0 },
    { 0xc1d77369, 90 },
    { 0x3357736a, 95 },
    { 0x5267736b, 23 },
    { 0x2ff7736b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0077371, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a7737b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147737e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d077382, 61 },
    { 0x9b777382, 84 },
    { 0xa9e77384, 95 },
    { 0x69077384, 88 },
    { 0x3d777384, 33 },
    { 0xe7577385, 33 },
    { 0x29e77383, 43 },
    { 0xd1177389, 23 },
    { 0x9a377389, 61 },
    { 0, 0 },
    { 0xe757738c, 61 },
    { 0xe247738d, 34 },
    { 0x2787738d, 83 },
    { 0x2ed7738d, 61 },
    { 0x87a77390, 90 },
    { 0xf4277391, 34 },
    { 0xf5e7738c, 46 },
    { 0xd097738e, 23 },
    { 0x79177394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67739a, 95 },
    { 0x80c7739b, 66 },
    { 0xc147739c, 95 },
    { 0xdc7739d, 77 },
    { 0xcc17739e, 78 },
    { 0x82a7739f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1773a7, 76 },
    { 0, 0 },
    { 0xd24773a9, 23 },
    { 0, 0 },
    { 0xd64773ab, 93 },
    { 0xc25773ac, 98 },
    { 0, 0 },
    { 0x7cb773ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0773ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae773c4, 43 },
    { 0x43773c5, 85 },
    { 0xc5f773c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58773ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21773ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6773d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00773db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971773e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf773e3, 33 },
    { 0, 0 },
    { 0x8d3773e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d773e8, 76 },
    { 0xa42773e9, 23 },
    { 0, 0 },
    { 0x4c6773eb, 46 },
    { 0x201773ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea773f8, 90 },
    { 0xded773f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510773ff, 93 },
    { 0x4dc77400, 95 },
    { 0x42077401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca677408, 52 },
    { 0x93477409, 95 },
    { 0xe457740a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac77411, 90 },
    { 0xdba77411, 61 },
    { 0x2ca77411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb77419, 87 },
    { 0xb7a77419, 75 },
    { 0, 0 },
    { 0x33c7741c, 57 },
    { 0xeb97741c, 61 },
    { 0x1737741e, 33 },
    { 0, 0 },
    { 0x6a277420, 99 },
    { 0, 0 },
    { 0xd1977422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26377429, 95 },
    { 0x29e7742a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa77433, 99 },
    { 0x7b977433, 88 },
    { 0x71e77435, 23 },
    { 0x96e77435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5607743c, 55 },
    { 0x78b7743d, 46 },
    { 0x6b47743d, 96 },
    { 0xfc87743f, 33 },
    { 0, 0 },
    { 0xb2a77441, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x277444, 66 },
    { 0x6d877445, 84 },
    { 0xd4b77445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4977744d, 95 },
    { 0x46a7744e, 76 },
    { 0xbfb7744d, 76 },
    { 0x20977450, 23 },
    { 0x8c277451, 98 },
    { 0xe0f77451, 76 },
    { 0xa6777451, 78 },
    { 0x70d77454, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb97745e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44677464, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa77468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57746d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1777473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7747a, 43 },
    { 0, 0 },
    { 0x9b57747c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe607747f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf977483, 23 },
    { 0, 0 },
    { 0xb2c77485, 64 },
    { 0xcff77486, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f77495, 61 },
    { 0, 0 },
    { 0xa2277497, 81 },
    { 0xde277497, 91 },
    { 0x12a77497, 33 },
    { 0x3177749a, 90 },
    { 0x4797749b, 98 },
    { 0x8a07749c, 61 },
    { 0xdcc7749b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec774a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6774a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a774ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123774b4, 90 },
    { 0xdea774b5, 59 },
    { 0xb4c774b6, 23 },
    { 0x822774b7, 90 },
    { 0xd10774b7, 61 },
    { 0xabc774b9, 90 },
    { 0x6ba774b9, 33 },
    { 0x74a774b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916774c1, 90 },
    { 0x674774c2, 76 },
    { 0xbcf774c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5774cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c774d3, 95 },
    { 0xcb4774d4, 90 },
    { 0x68c774d4, 46 },
    { 0xca8774d6, 23 },
    { 0xabb774d7, 87 },
    { 0x4fd774d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef774db, 90 },
    { 0, 0 },
    { 0x395774dd, 46 },
    { 0xc8b774dd, 17 },
    { 0x5b1774df, 95 },
    { 0x333774e0, 95 },
    { 0x443774e0, 33 },
    { 0x6f5774e2, 33 },
    { 0x9fb774df, 90 },
    { 0x2cb774e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9774ec, 80 },
    { 0x2c1774ed, 55 },
    { 0xa8f774ee, 33 },
    { 0x38f774ef, 90 },
    { 0x2fc774f0, 95 },
    { 0xb39774ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1774f8, 46 },
    { 0x416774f9, 98 },
    { 0xcfa774fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2377505, 33 },
    { 0, 0 },
    { 0x8f877507, 99 },
    { 0x5f377508, 100 },
    { 0xcc777509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b87750d, 76 },
    { 0x9f77750e, 98 },
    { 0x2f07750f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e77518, 87 },
    { 0x3b377519, 76 },
    { 0xd4877519, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b97751d, 90 },
    { 0, 0 },
    { 0x1537751f, 46 },
    { 0x5b177520, 33 },
    { 0x27177521, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x17277524, 61 },
    { 0, 0 },
    { 0x8f177526, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3477529, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a7752e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8377533, 33 },
    { 0xbc177533, 95 },
    { 0x84177535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21277539, 43 },
    { 0, 0 },
    { 0x36e7753b, 61 },
    { 0xe857753c, 55 },
    { 0xa607753c, 34 },
    { 0xcce7753e, 97 },
    { 0xa947753f, 98 },
    { 0x9207753e, 95 },
    { 0xcb877541, 61 },
    { 0x1c877542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf277546, 98 },
    { 0, 0 },
    { 0xb4477548, 95 },
    { 0x9e177549, 95 },
    { 0x4687754a, 46 },
    { 0x8637754a, 90 },
    { 0x25a7754c, 90 },
    { 0x5ba7754d, 61 },
    { 0x5787754d, 33 },
    { 0x15b7754f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d577554, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f77557, 77 },
    { 0x7e077557, 23 },
    { 0, 0 },
    { 0x1957755a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc047755d, 33 },
    { 0, 0 },
    { 0xd337755f, 76 },
    { 0xf817755f, 34 },
    { 0x7f777561, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d77564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e77568, 95 },
    { 0xb3677569, 81 },
    { 0x7c777569, 90 },
    { 0, 0 },
    { 0x9a47756c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34877570, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46177574, 23 },
    { 0xda777574, 84 },
    { 0x86d77576, 23 },
    { 0x59777576, 17 },
    { 0x99977575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f57757c, 61 },
    { 0xdb77757d, 33 },
    { 0, 0 },
    { 0x4e37757f, 98 },
    { 0xe7e77580, 55 },
    { 0x8fb77580, 90 },
    { 0x42e77582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a677589, 75 },
    { 0, 0 },
    { 0xfec7758b, 90 },
    { 0x8667758c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2477590, 90 },
    { 0xfb677591, 93 },
    { 0, 0 },
    { 0x77477593, 90 },
    { 0x81677593, 59 },
    { 0x40377594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa677759b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87759f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f775a2, 90 },
    { 0, 0 },
    { 0x15f775a4, 65 },
    { 0, 0 },
    { 0x184775a6, 95 },
    { 0x39e775a7, 90 },
    { 0x1e9775a8, 33 },
    { 0x8bc775a9, 55 },
    { 0x1a0775a9, 90 },
    { 0x2f0775a8, 61 },
    { 0x49f775ac, 90 },
    { 0xf07775ad, 76 },
    { 0xcb2775ac, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8775b3, 98 },
    { 0x1a9775b3, 98 },
    { 0x3de775b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db775b9, 33 },
    { 0, 0 },
    { 0xa4b775bb, 61 },
    { 0, 0 },
    { 0xffb775bd, 23 },
    { 0x2c1775be, 90 },
    { 0xc96775be, 76 },
    { 0x95b775bf, 87 },
    { 0x776775c1, 95 },
    { 0x3b0775c1, 52 },
    { 0xbdd775bd, 76 },
    { 0x4b5775c4, 95 },
    { 0xa71775c4, 90 },
    { 0xc53775bd, 43 },
    { 0x37b775c7, 33 },
    { 0xc4d775c8, 17 },
    { 0xf73775c8, 90 },
    { 0xbdc775ca, 66 },
    { 0x2e4775c3, 23 },
    { 0x73775cc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a775cf, 90 },
    { 0x6e6775d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a775d4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x262775d7, 90 },
    { 0, 0 },
    { 0x405775d9, 23 },
    { 0x13775d9, 95 },
    { 0x7d7775db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x427775de, 61 },
    { 0x6d0775de, 43 },
    { 0xcc6775e0, 98 },
    { 0, 0 },
    { 0x587775e2, 43 },
    { 0xac0775e3, 84 },
    { 0, 0 },
    { 0x368775e5, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a775e9, 55 },
    { 0, 0 },
    { 0x11c775eb, 68 },
    { 0xe8775ec, 98 },
    { 0x7c6775ed, 55 },
    { 0x788775eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e775fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6775ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb77603, 33 },
    { 0xe0477603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c77608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d37760c, 17 },
    { 0x4667760d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x73877610, 55 },
    { 0, 0 },
    { 0xa4177612, 61 },
    { 0x7d977613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd7761e, 90 },
    { 0, 0 },
    { 0xe377620, 88 },
    { 0x58077621, 23 },
    { 0x7c577622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81277626, 98 },
    { 0x77077627, 99 },
    { 0x9d277628, 90 },
    { 0, 0 },
    { 0xea47762a, 61 },
    { 0xc477762b, 90 },
    { 0x23a7762c, 43 },
    { 0xf057762d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b77630, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e677633, 77 },
    { 0, 0 },
    { 0x64e77635, 84 },
    { 0x88677635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed67763b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07763e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d477644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75477648, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7764d, 61 },
    { 0xf7764e, 95 },
    { 0xdab7764e, 90 },
    { 0x2fa7764e, 61 },
    { 0, 0 },
    { 0x65077652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62877655, 43 },
    { 0, 0 },
    { 0x29477657, 33 },
    { 0xf8f77658, 34 },
    { 0x40177659, 77 },
    { 0xabc77659, 33 },
    { 0x7ca7765b, 90 },
    { 0, 0 },
    { 0x1cf7765d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae77662, 90 },
    { 0x32f77663, 95 },
    { 0xa377662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d77667, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87766a, 55 },
    { 0, 0 },
    { 0x4177766c, 95 },
    { 0x827766d, 43 },
    { 0, 0 },
    { 0xad37766f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e77675, 76 },
    { 0xbb277676, 84 },
    { 0xabb77677, 76 },
    { 0x7d077678, 76 },
    { 0x39677678, 64 },
    { 0x9cb77679, 33 },
    { 0xd7b7767a, 95 },
    { 0x96c77678, 23 },
    { 0x5877767a, 66 },
    { 0x36b7767e, 95 },
    { 0xa4077677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f97768a, 93 },
    { 0, 0 },
    { 0xdb97768c, 61 },
    { 0xd7768c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f077692, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11977696, 40 },
    { 0x27177696, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f7769b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3776a0, 55 },
    { 0x3b776a0, 76 },
    { 0x70a776a2, 61 },
    { 0, 0 },
    { 0xa88776a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405776ae, 75 },
    { 0xf3c776ae, 23 },
    { 0, 0 },
    { 0x7a9776b1, 46 },
    { 0x87f776b2, 90 },
    { 0, 0 },
    { 0xee7776b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d776ba, 55 },
    { 0, 0 },
    { 0xa18776bc, 92 },
    { 0xd6a776bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165776c2, 33 },
    { 0x239776c2, 46 },
    { 0, 0 },
    { 0x641776c5, 43 },
    { 0x6cf776c6, 90 },
    { 0x7f9776c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146776ce, 90 },
    { 0x9e8776cf, 34 },
    { 0, 0 },
    { 0x1d0776d1, 90 },
    { 0x745776d1, 98 },
    { 0x74f776d1, 43 },
    { 0, 0 },
    { 0xabb776d5, 90 },
    { 0xc49776d6, 90 },
    { 0xbd9776d6, 93 },
    { 0, 0 },
    { 0xad1776d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37776df, 91 },
    { 0xd47776e0, 33 },
    { 0x4fd776e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3776eb, 84 },
    { 0xc2a776eb, 77 },
    { 0x666776eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c776f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329776f9, 98 },
    { 0x6ad776fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75277700, 34 },
    { 0, 0 },
    { 0x68d77702, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd77708, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827770d, 91 },
    { 0x14d7770d, 76 },
    { 0x3ac7770e, 55 },
    { 0xbdd7770d, 95 },
    { 0x177711, 55 },
    { 0x99c77711, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8777715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c77718, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7771b, 33 },
    { 0xe2d7771c, 66 },
    { 0, 0 },
    { 0x7207771e, 34 },
    { 0xab67771e, 55 },
    { 0x54277720, 76 },
    { 0x96977721, 93 },
    { 0x1ab77720, 95 },
    { 0xfe977723, 33 },
    { 0xc8977722, 98 },
    { 0xdd77721, 23 },
    { 0xae677726, 55 },
    { 0, 0 },
    { 0x38b77728, 76 },
    { 0x6a577728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2507772e, 95 },
    { 0x4b57772f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0677739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097773c, 98 },
    { 0, 0 },
    { 0x9b27773e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de77742, 90 },
    { 0, 0 },
    { 0x4e577744, 33 },
    { 0xc7a77745, 99 },
    { 0xae177746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44077749, 95 },
    { 0x80e7774a, 99 },
    { 0xa5f77749, 23 },
    { 0x3cd77749, 100 },
    { 0, 0 },
    { 0x9507774e, 92 },
    { 0xef47774f, 34 },
    { 0xb3877750, 91 },
    { 0x9ba7774f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23277755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e7775a, 91 },
    { 0x2987775b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9417775e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f677761, 95 },
    { 0x2777762, 95 },
    { 0x5e77761, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e477768, 17 },
    { 0xc0277768, 38 },
    { 0, 0 },
    { 0xfd07776b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557776f, 90 },
    { 0xeda77770, 76 },
    { 0x94677771, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79077777, 33 },
    { 0xae977778, 23 },
    { 0xddf77779, 34 },
    { 0, 0 },
    { 0x9777777b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f77780, 61 },
    { 0xea777780, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75377786, 43 },
    { 0x27a77786, 43 },
    { 0, 0 },
    { 0xeee77789, 61 },
    { 0, 0 },
    { 0xa487778b, 68 },
    { 0x5cd7778c, 61 },
    { 0x51d7778d, 80 },
    { 0xb0b7778b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17377792, 98 },
    { 0, 0 },
    { 0xe5877794, 76 },
    { 0xd8c77794, 55 },
    { 0xc3777794, 61 },
    { 0xb1b77794, 36 },
    { 0, 0 },
    { 0x17a77799, 81 },
    { 0xf4d7779a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7779e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1777a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c777a7, 55 },
    { 0xca8777a7, 90 },
    { 0, 0 },
    { 0x6de777aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4777b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515777bc, 90 },
    { 0x81f777bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a777c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda777c4, 55 },
    { 0, 0 },
    { 0xceb777c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x408777c9, 98 },
    { 0x947777c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16777cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x934777d0, 98 },
    { 0xc2e777d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b777d4, 78 },
    { 0, 0 },
    { 0xe55777d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab777db, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4777e2, 40 },
    { 0x5df777e3, 23 },
    { 0xc31777e3, 55 },
    { 0xcf777e2, 95 },
    { 0, 0 },
    { 0xfaa777e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72777ee, 52 },
    { 0, 0 },
    { 0xf06777f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29777f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x198777f6, 84 },
    { 0x427777f7, 33 },
    { 0, 0 },
    { 0x3bf777f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80577801, 90 },
    { 0, 0 },
    { 0xe2c77803, 62 },
    { 0, 0 },
    { 0x21077805, 61 },
    { 0xba677805, 76 },
    { 0, 0 },
    { 0xcfd77808, 34 },
    { 0x6d77808, 95 },
    { 0xd927780a, 85 },
    { 0xa7677808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af7780f, 100 },
    { 0, 0 },
    { 0x80377811, 67 },
    { 0xac277812, 94 },
    { 0xc4177813, 98 },
    { 0x84677814, 23 },
    { 0xfb277815, 90 },
    { 0x84877814, 33 },
    { 0x44377817, 78 },
    { 0xdb177818, 90 },
    { 0x21777815, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1347781c, 85 },
    { 0x757781d, 85 },
    { 0, 0 },
    { 0x8477781f, 91 },
    { 0x69e7781f, 23 },
    { 0, 0 },
    { 0x25f77822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647782c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c77830, 93 },
    { 0, 0 },
    { 0x42c77832, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30377838, 90 },
    { 0xadf77839, 76 },
    { 0x5267783a, 33 },
    { 0xd1c77838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa7783f, 90 },
    { 0x8a477840, 86 },
    { 0x1c077841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c77847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7784c, 72 },
    { 0, 0 },
    { 0xf2f7784e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab277854, 23 },
    { 0x63877854, 33 },
    { 0xf3d77856, 33 },
    { 0x2f377856, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa487785a, 67 },
    { 0, 0 },
    { 0x6907785c, 55 },
    { 0xce07785c, 98 },
    { 0x95c7785e, 33 },
    { 0xd5e7785e, 93 },
    { 0xbc27785f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8377868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1157786b, 17 },
    { 0x74b7786c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977870, 76 },
    { 0xd3877871, 61 },
    { 0, 0 },
    { 0x90577873, 61 },
    { 0x43a77874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db77878, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657787b, 61 },
    { 0xe3a7787c, 43 },
    { 0xec97787d, 34 },
    { 0, 0 },
    { 0x8937787f, 98 },
    { 0xdbb7787f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e177883, 84 },
    { 0, 0 },
    { 0xf077885, 76 },
    { 0xf1777886, 23 },
    { 0, 0 },
    { 0x92977888, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547788c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b7788f, 61 },
    { 0x79577890, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c77896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497789b, 50 },
    { 0x4c67789c, 87 },
    { 0xba7789b, 98 },
    { 0, 0 },
    { 0x7517789f, 23 },
    { 0x227778a0, 40 },
    { 0x623778a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b778a4, 77 },
    { 0x742778a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478778ad, 61 },
    { 0x571778ad, 55 },
    { 0xd58778ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x686778b2, 23 },
    { 0xff1778b3, 69 },
    { 0x64b778b3, 95 },
    { 0x24c778b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c778b9, 40 },
    { 0x626778ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c778be, 43 },
    { 0x4a8778bf, 99 },
    { 0x4df778c0, 98 },
    { 0, 0 },
    { 0xb5c778c2, 61 },
    { 0xf27778c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab778c6, 23 },
    { 0x887778c7, 95 },
    { 0x7f2778c8, 55 },
    { 0xaf4778c9, 33 },
    { 0x4d4778ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc778d3, 33 },
    { 0x71778d4, 23 },
    { 0xf78778d5, 61 },
    { 0, 0 },
    { 0xb94778d7, 95 },
    { 0x57d778d8, 46 },
    { 0, 0 },
    { 0x54e778da, 61 },
    { 0, 0 },
    { 0x489778dc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6778df, 95 },
    { 0, 0 },
    { 0x960778e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174778eb, 95 },
    { 0xabd778ec, 98 },
    { 0, 0 },
    { 0xa8b778ee, 82 },
    { 0, 0 },
    { 0x55c778f0, 66 },
    { 0xcd778f0, 85 },
    { 0, 0 },
    { 0x8a6778f3, 95 },
    { 0x8da778f4, 56 },
    { 0, 0 },
    { 0xe7d778f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08778fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f77904, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f57790a, 100 },
    { 0, 0 },
    { 0x6d37790c, 90 },
    { 0x8997790d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377913, 33 },
    { 0, 0 },
    { 0xabf77915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7791d, 95 },
    { 0xdf37791e, 67 },
    { 0x1387791f, 76 },
    { 0, 0 },
    { 0x4d477921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d477927, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf47792c, 23 },
    { 0x8447792d, 33 },
    { 0x64c7792d, 76 },
    { 0x1717792f, 66 },
    { 0xd947792e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9877937, 43 },
    { 0, 0 },
    { 0xca477939, 76 },
    { 0, 0 },
    { 0xfd77793b, 93 },
    { 0x35b7793c, 90 },
    { 0xb707793d, 78 },
    { 0, 0 },
    { 0xd367793f, 33 },
    { 0, 0 },
    { 0xdcf77941, 98 },
    { 0x20277942, 88 },
    { 0x7e77943, 76 },
    { 0x74d77941, 93 },
    { 0x73877945, 90 },
    { 0xd8877945, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72777949, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a7794e, 100 },
    { 0xb487794f, 66 },
    { 0x7387794f, 55 },
    { 0xa2377951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5777958, 33 },
    { 0, 0 },
    { 0xfde7795a, 78 },
    { 0, 0 },
    { 0xa0d7795c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47795f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e777964, 95 },
    { 0, 0 },
    { 0x5f677966, 90 },
    { 0xbb77967, 61 },
    { 0x82177968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf417796b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377796e, 95 },
    { 0xaa77796f, 83 },
    { 0x1077970, 99 },
    { 0, 0 },
    { 0x2c777972, 61 },
    { 0x66277972, 66 },
    { 0xa1d77974, 93 },
    { 0, 0 },
    { 0xff77976, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74977979, 91 },
    { 0xa0c77979, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4127797d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5477980, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b77983, 66 },
    { 0xa1877984, 55 },
    { 0x47d77985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d77988, 33 },
    { 0, 0 },
    { 0x74e7798a, 90 },
    { 0xc6d7798b, 76 },
    { 0x24f7798c, 90 },
    { 0xe267798d, 61 },
    { 0x39c7798c, 85 },
    { 0x1b77798f, 90 },
    { 0xe9b77990, 90 },
    { 0x1f77798e, 98 },
    { 0xe7777990, 84 },
    { 0xba57798b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f37799c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d779a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49779a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709779a7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f779aa, 90 },
    { 0x1bc779aa, 33 },
    { 0xef5779ab, 23 },
    { 0x253779ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1779b1, 61 },
    { 0, 0 },
    { 0x19f779b3, 90 },
    { 0x557779b4, 33 },
    { 0x1de779b3, 100 },
    { 0x1c7779b3, 90 },
    { 0x6c0779b3, 90 },
    { 0, 0 },
    { 0x6b0779b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8779bd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e779c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d779c6, 84 },
    { 0x64e779c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd779cc, 34 },
    { 0xf0b779cc, 33 },
    { 0x9c0779ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x123779d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f779d7, 66 },
    { 0, 0 },
    { 0xa1d779d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x398779dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e779df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1779e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87779e7, 43 },
    { 0xc23779e8, 33 },
    { 0, 0 },
    { 0xfc7779ea, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1779ed, 66 },
    { 0x219779ed, 84 },
    { 0x7b0779ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b779f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc779f7, 88 },
    { 0xcf8779f7, 90 },
    { 0xe5e779f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d779fc, 90 },
    { 0x94779fd, 93 },
    { 0, 0 },
    { 0xfda779ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77a02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84177a08, 91 },
    { 0, 0 },
    { 0xaca77a0a, 33 },
    { 0x7db77a0a, 33 },
    { 0, 0 },
    { 0xb2777a0d, 98 },
    { 0xc8677a0e, 34 },
    { 0xad877a0f, 66 },
    { 0, 0 },
    { 0x82277a11, 90 },
    { 0x26a77a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e77a18, 66 },
    { 0xd6677a18, 76 },
    { 0x23677a19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f377a20, 90 },
    { 0xcef77a21, 55 },
    { 0, 0 },
    { 0x65677a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd977a2e, 23 },
    { 0x88977a2f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf177a32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82477a39, 23 },
    { 0, 0 },
    { 0x3d577a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73577a45, 90 },
    { 0xe5377a46, 33 },
    { 0x1f577a45, 61 },
    { 0xf9a77a48, 95 },
    { 0, 0 },
    { 0xcc177a4a, 52 },
    { 0x3f77a4b, 64 },
    { 0x56077a4c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d877a50, 90 },
    { 0, 0 },
    { 0xc5d77a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177a56, 95 },
    { 0, 0 },
    { 0x3a877a58, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5877a5c, 100 },
    { 0x5f977a5c, 90 },
    { 0, 0 },
    { 0x12a77a5f, 95 },
    { 0xbf77a5f, 91 },
    { 0x51d77a60, 43 },
    { 0x43d77a61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37477a69, 99 },
    { 0xdac77a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7877a6e, 33 },
    { 0xe2677a6e, 55 },
    { 0, 0 },
    { 0xd7077a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d777a74, 95 },
    { 0x71577a75, 99 },
    { 0, 0 },
    { 0xb9577a77, 23 },
    { 0x61377a78, 99 },
    { 0xe3e77a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f177a7e, 23 },
    { 0x61e77a7f, 87 },
    { 0x68e77a80, 66 },
    { 0xfe977a81, 61 },
    { 0xed477a80, 90 },
    { 0xf8877a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57077a87, 82 },
    { 0, 0 },
    { 0x21d77a89, 40 },
    { 0xbee77a8a, 28 },
    { 0, 0 },
    { 0xf077a8c, 61 },
    { 0x76077a8c, 75 },
    { 0x8fd77a8e, 33 },
    { 0xb2f77a8c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e77a92, 76 },
    { 0x5d977a92, 51 },
    { 0x45077a94, 33 },
    { 0, 0 },
    { 0x7877a96, 33 },
    { 0, 0 },
    { 0xaa577a98, 68 },
    { 0, 0 },
    { 0x6d277a9a, 85 },
    { 0, 0 },
    { 0x22877a9c, 34 },
    { 0, 0 },
    { 0x77177a9e, 90 },
    { 0x12a77a9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea77aa3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c77aa6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f77aae, 90 },
    { 0xf0d77aaf, 34 },
    { 0x75877aae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c377ab7, 55 },
    { 0x87877ab8, 52 },
    { 0, 0 },
    { 0x2d877aba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec77ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85177ac4, 34 },
    { 0x2d777ac4, 98 },
    { 0x69d77ac5, 91 },
    { 0xf2077ac5, 61 },
    { 0xc3c77ac8, 78 },
    { 0x7b877ac9, 90 },
    { 0x31777ac9, 66 },
    { 0xb1877ac8, 91 },
    { 0xf0177acc, 95 },
    { 0, 0 },
    { 0xf7e77ace, 33 },
    { 0xbae77ace, 90 },
    { 0x27e77ace, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f777ad3, 33 },
    { 0x65b77ad4, 84 },
    { 0, 0 },
    { 0x92977ad6, 68 },
    { 0, 0 },
    { 0xae977ad8, 65 },
    { 0x5cf77ad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d577adc, 61 },
    { 0x74d77add, 33 },
    { 0xf6d77ade, 17 },
    { 0xfa977adf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e77aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2677aed, 100 },
    { 0, 0 },
    { 0xa7277aef, 55 },
    { 0xadc77af0, 33 },
    { 0xb3577af0, 43 },
    { 0x3a077af1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92277af5, 98 },
    { 0xde477af5, 84 },
    { 0x17f77af6, 34 },
    { 0xefa77af7, 38 },
    { 0x24c77af9, 55 },
    { 0, 0 },
    { 0xd6c77afb, 98 },
    { 0xfa377afc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x21177aff, 88 },
    { 0x2dc77aff, 90 },
    { 0, 0 },
    { 0xb3c77b02, 91 },
    { 0xe7377b03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb777b06, 100 },
    { 0xff777b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4377b0a, 95 },
    { 0xb7577b0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c77b0e, 95 },
    { 0x9a977b0e, 43 },
    { 0x2a077b0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7877b1c, 55 },
    { 0xb4677b1d, 98 },
    { 0, 0 },
    { 0x76c77b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14777b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f77b2a, 90 },
    { 0x42a77b2a, 98 },
    { 0x2ed77b2b, 34 },
    { 0x5fc77b2d, 55 },
    { 0, 0 },
    { 0x10277b2f, 91 },
    { 0x4177b30, 85 },
    { 0x5977b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e77b36, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e77b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a77b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6577b40, 33 },
    { 0x18f77b41, 61 },
    { 0xc4f77b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30077b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb77b49, 61 },
    { 0x2fb77b49, 78 },
    { 0xb9777b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa177b4e, 33 },
    { 0x5d777b4f, 23 },
    { 0, 0 },
    { 0x87a77b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2077b54, 68 },
    { 0x6f77b55, 23 },
    { 0xb0377b56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e177b59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d977b5d, 95 },
    { 0, 0 },
    { 0x62c77b5f, 76 },
    { 0x33877b60, 76 },
    { 0xe0177b61, 38 },
    { 0x9bf77b62, 23 },
    { 0xaf077b62, 91 },
    { 0xa3577b64, 90 },
    { 0x8af77b65, 23 },
    { 0xc2b77b61, 85 },
    { 0xdf777b65, 33 },
    { 0x12177b63, 76 },
    { 0x1d477b61, 61 },
    { 0x44c77b6a, 43 },
    { 0x3c177b66, 61 },
    { 0x7b277b6c, 61 },
    { 0x5a077b6d, 90 },
    { 0x77b77b6d, 33 },
    { 0xf0977b6f, 98 },
    { 0xb0c77b6f, 97 },
    { 0xcdb77b70, 84 },
    { 0x8ce77b72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e77b79, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd77b7d, 98 },
    { 0x80a77b7e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a377b83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7077b86, 34 },
    { 0xed977b86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee177b8a, 84 },
    { 0x35777b8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb677b8f, 23 },
    { 0xd9277b8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7477b93, 23 },
    { 0x7fd77b94, 77 },
    { 0xa4b77b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc77b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70377b9d, 84 },
    { 0xb5e77b9e, 33 },
    { 0x66277b9d, 23 },
    { 0xfa577ba0, 98 },
    { 0, 0 },
    { 0xcce77ba2, 95 },
    { 0xbf377ba3, 33 },
    { 0, 0 },
    { 0x93a77ba5, 61 },
    { 0, 0 },
    { 0x53577ba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac477bad, 98 },
    { 0x52177bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377bb5, 23 },
    { 0x96777bb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd77bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80177bbf, 98 },
    { 0, 0 },
    { 0x17f77bc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0577bc5, 55 },
    { 0xfbe77bc6, 34 },
    { 0x9ad77bc6, 33 },
    { 0x64577bc6, 76 },
    { 0x51177bc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88977bcc, 98 },
    { 0, 0 },
    { 0xedf77bce, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb77bd4, 95 },
    { 0xf8b77bd5, 98 },
    { 0x1f577bd6, 95 },
    { 0xeeb77bd4, 93 },
    { 0xe9577bd7, 84 },
    { 0xf1c77bd9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37877be3, 33 },
    { 0x66777be4, 99 },
    { 0xf2b77be5, 95 },
    { 0x7eb77be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c077bf1, 95 },
    { 0x21a77bf1, 43 },
    { 0x3c377bf1, 33 },
    { 0xe9077bf1, 33 },
    { 0xece77bf5, 23 },
    { 0x53477bf6, 46 },
    { 0xa8677bf7, 46 },
    { 0x6fc77bf8, 55 },
    { 0x59a77bf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe77bfe, 61 },
    { 0x83877bff, 33 },
    { 0, 0 },
    { 0x5e077c01, 100 },
    { 0, 0 },
    { 0x7c077c03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e477c0a, 81 },
    { 0x9d177c0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b977c0f, 33 },
    { 0x8d177c0f, 98 },
    { 0x67977c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79777c14, 98 },
    { 0x80477c15, 33 },
    { 0x3f777c16, 34 },
    { 0xfca77c15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed277c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9177c20, 65 },
    { 0xc2a77c21, 66 },
    { 0x9a677c20, 23 },
    { 0x97677c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb277c26, 90 },
    { 0xc4877c27, 61 },
    { 0x71f77c28, 33 },
    { 0xc0777c28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b77c2c, 98 },
    { 0x33277c2c, 67 },
    { 0x86e77c2e, 77 },
    { 0x6ed77c2e, 95 },
    { 0xa8f77c2e, 87 },
    { 0xc977c30, 43 },
    { 0x6ab77c32, 46 },
    { 0xd3277c33, 23 },
    { 0xad277c33, 34 },
    { 0xb3477c34, 55 },
    { 0x63e77c2d, 34 },
    { 0, 0 },
    { 0x23c77c38, 76 },
    { 0, 0 },
    { 0xd9677c3a, 34 },
    { 0, 0 },
    { 0xcf77c3c, 95 },
    { 0x6cc77c3c, 36 },
    { 0xcc977c3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32777c43, 33 },
    { 0x81477c44, 85 },
    { 0x9df77c43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf477c48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f77c51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f77c54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x64977c57, 60 },
    { 0xaeb77c58, 61 },
    { 0xa1477c59, 100 },
    { 0x97777c59, 23 },
    { 0x26c77c5b, 23 },
    { 0xbf677c5c, 91 },
    { 0x28f77c5d, 98 },
    { 0, 0 },
    { 0x16877c5f, 33 },
    { 0xea977c60, 94 },
    { 0x54277c61, 90 },
    { 0x17a77c62, 98 },
    { 0x9b677c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb977c66, 90 },
    { 0xd1f77c67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14877c6f, 90 },
    { 0xad477c6f, 87 },
    { 0, 0 },
    { 0x93b77c72, 77 },
    { 0xbc277c72, 61 },
    { 0xc8177c74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f77c77, 98 },
    { 0xd3877c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d677c7b, 98 },
    { 0x87177c7b, 33 },
    { 0x1d777c7d, 90 },
    { 0xd1477c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c77c82, 34 },
    { 0x6d077c83, 61 },
    { 0, 0 },
    { 0x1fb77c85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8977c91, 91 },
    { 0x20577c92, 95 },
    { 0x4a077c92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e77c96, 66 },
    { 0xc3a77c97, 98 },
    { 0xf8f77c96, 87 },
    { 0x3b077c99, 90 },
    { 0xdde77c99, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc77c9d, 90 },
    { 0xf3377c9e, 53 },
    { 0xf5a77c9f, 44 },
    { 0, 0 },
    { 0x47877ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7477ca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x22077cab, 98 },
    { 0xab077cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed77cb4, 76 },
    { 0x9c77cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd77cba, 61 },
    { 0xa0a77cbb, 93 },
    { 0xed377cbc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea77cbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71177cc4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b277cc7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26377ccf, 90 },
    { 0x2f777cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d977cd5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c77cdb, 69 },
    { 0x97477cdb, 43 },
    { 0, 0 },
    { 0xc2b77cde, 98 },
    { 0, 0 },
    { 0x5c477ce0, 93 },
    { 0x4a77ce0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a77ce9, 40 },
    { 0x6f177cea, 84 },
    { 0, 0 },
    { 0x27677cec, 61 },
    { 0x34d77ced, 61 },
    { 0xac077cee, 54 },
    { 0xd1677cef, 78 },
    { 0xbb677cef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28977cf4, 33 },
    { 0x11b77cf5, 55 },
    { 0, 0 },
    { 0xe6577cf7, 52 },
    { 0xc9977cf8, 55 },
    { 0x40777cf9, 67 },
    { 0, 0 },
    { 0xa8477cfb, 98 },
    { 0x3fa77cfc, 61 },
    { 0x10b77cfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2977d01, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca77d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d77d0b, 95 },
    { 0, 0 },
    { 0xe8477d0d, 33 },
    { 0xef577d0e, 95 },
    { 0x33d77d0f, 46 },
    { 0xd2e77d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf477d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3777d19, 90 },
    { 0xa3177d1a, 66 },
    { 0x49e77d1a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc77d1e, 61 },
    { 0xf5477d1e, 61 },
    { 0x44e77d20, 95 },
    { 0xc5f77d1e, 23 },
    { 0xf9477d22, 36 },
    { 0, 0 },
    { 0xc4177d24, 55 },
    { 0xecd77d25, 76 },
    { 0x81477d25, 52 },
    { 0xafc77d27, 76 },
    { 0xbfb77d28, 93 },
    { 0xec77d28, 77 },
    { 0x13477d26, 98 },
    { 0x80b77d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e77d30, 52 },
    { 0x26177d31, 61 },
    { 0x33077d32, 90 },
    { 0, 0 },
    { 0xfc477d34, 61 },
    { 0x75777d35, 33 },
    { 0x1e577d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe677d3b, 61 },
    { 0, 0 },
    { 0xce477d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d77d42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d377d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81177d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3377d50, 65 },
    { 0xa3777d51, 98 },
    { 0, 0 },
    { 0xd7a77d53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f77d58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d77d5b, 61 },
    { 0x86877d5c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67877d62, 98 },
    { 0, 0 },
    { 0x88d77d64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee877d68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4977d73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85277d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c677d86, 91 },
    { 0xfff77d87, 99 },
    { 0, 0 },
    { 0x6e977d89, 96 },
    { 0, 0 },
    { 0x9b577d8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8477d8f, 91 },
    { 0x42c77d8f, 34 },
    { 0x5f977d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd77d95, 34 },
    { 0x4fc77d95, 95 },
    { 0x88777d95, 90 },
    { 0x72e77d98, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba077d9d, 90 },
    { 0xd7977d9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa77da2, 98 },
    { 0xc2477da2, 33 },
    { 0x53977da4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75277da8, 34 },
    { 0, 0 },
    { 0x3bc77daa, 33 },
    { 0xef77dab, 95 },
    { 0x9ab77dab, 90 },
    { 0xe7977dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a177db4, 76 },
    { 0, 0 },
    { 0xf0377db6, 33 },
    { 0x2a077db7, 23 },
    { 0x91977db8, 46 },
    { 0x4e277db9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f777dbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f77dc2, 76 },
    { 0, 0 },
    { 0x79c77dc4, 90 },
    { 0xed777dc5, 23 },
    { 0xfbf77dc6, 76 },
    { 0x69277dc5, 61 },
    { 0, 0 },
    { 0xa2077dc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e77dd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c277dd9, 33 },
    { 0x68277dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a77dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46177de2, 90 },
    { 0x3e177de3, 61 },
    { 0x42777de3, 76 },
    { 0x11177de2, 90 },
    { 0xe9a77de6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f77dec, 33 },
    { 0, 0 },
    { 0xd7577dee, 34 },
    { 0, 0 },
    { 0xc7477df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f77df3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7777df9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd77dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc977e02, 99 },
    { 0xe2777e02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d77e06, 34 },
    { 0, 0 },
    { 0x1d477e08, 98 },
    { 0x5a977e09, 33 },
    { 0x16477e08, 23 },
    { 0, 0 },
    { 0x2cb77e0c, 46 },
    { 0xdb477e0c, 67 },
    { 0x68d77e0e, 55 },
    { 0x75a77e0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f77e1a, 78 },
    { 0x4ab77e1a, 90 },
    { 0, 0 },
    { 0x33177e1d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c377e28, 23 },
    { 0x79b77e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf577e2f, 90 },
    { 0xf1e77e30, 76 },
    { 0x76277e31, 95 },
    { 0xd7b77e2f, 98 },
    { 0, 0 },
    { 0x41277e34, 76 },
    { 0xd3677e34, 33 },
    { 0x19277e36, 23 },
    { 0x20f77e37, 61 },
    { 0, 0 },
    { 0x33577e39, 95 },
    { 0x64b77e3a, 34 },
    { 0x7bb77e3b, 88 },
    { 0x6ad77e3b, 90 },
    { 0x5e177e39, 90 },
    { 0x4a777e3e, 78 },
    { 0x60277e3c, 43 },
    { 0x4277e3c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a477e46, 98 },
    { 0, 0 },
    { 0x43a77e48, 43 },
    { 0, 0 },
    { 0xaef77e4a, 17 },
    { 0xbff77e4a, 84 },
    { 0xdcb77e4a, 33 },
    { 0xc3f77e4d, 33 },
    { 0xabf77e4d, 33 },
    { 0x72277e4b, 58 },
    { 0, 0 },
    { 0x22a77e51, 95 },
    { 0x95877e52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf477e56, 43 },
    { 0x78477e57, 77 },
    { 0x74277e58, 55 },
    { 0x7f277e58, 90 },
    { 0xd9a77e58, 76 },
    { 0x93077e58, 95 },
    { 0x43277e59, 98 },
    { 0xb5077e5d, 23 },
    { 0x2c377e5e, 33 },
    { 0x3b377e5f, 90 },
    { 0xeb77e60, 78 },
    { 0xdec77e5d, 90 },
    { 0x95077e62, 95 },
    { 0x67d77e5a, 84 },
    { 0, 0 },
    { 0x9b177e65, 96 },
    { 0x25b77e66, 83 },
    { 0x4f577e67, 90 },
    { 0xc6c77e68, 95 },
    { 0x28b77e69, 61 },
    { 0xad977e69, 90 },
    { 0xf5b77e67, 98 },
    { 0x8b377e6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf677e72, 87 },
    { 0xa5e77e73, 98 },
    { 0x9177e73, 33 },
    { 0x28077e72, 46 },
    { 0, 0 },
    { 0x8b077e77, 75 },
    { 0x82077e78, 38 },
    { 0x45b77e79, 85 },
    { 0, 0 },
    { 0x9de77e7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38977e82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0977e85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4377e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc77e8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82577e90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3677e94, 23 },
    { 0xcc77e95, 90 },
    { 0xa2077e96, 61 },
    { 0, 0 },
    { 0x5c277e98, 76 },
    { 0, 0 },
    { 0x89a77e9a, 76 },
    { 0x27777e9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc77e9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78677ea2, 98 },
    { 0xddb77ea2, 51 },
    { 0xfeb77ea4, 33 },
    { 0x4c277ea5, 77 },
    { 0, 0 },
    { 0x94477ea7, 90 },
    { 0, 0 },
    { 0x25177ea9, 100 },
    { 0x2ef77ea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e77eae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f77eb5, 61 },
    { 0, 0 },
    { 0x9df77eb7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x95177eba, 61 },
    { 0x9ab77ebb, 95 },
    { 0xd1677ebc, 43 },
    { 0x6d477ebd, 76 },
    { 0x9bd77ebe, 90 },
    { 0x61077ebf, 98 },
    { 0xf1f77eba, 76 },
    { 0xd7877ec1, 61 },
    { 0x58a77ec2, 23 },
    { 0xa977ec2, 95 },
    { 0xd3c77ec0, 76 },
    { 0, 0 },
    { 0x4177ec6, 38 },
    { 0xad477ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38477ecc, 76 },
    { 0x77877ecd, 61 },
    { 0x58177ece, 36 },
    { 0x7b77ecf, 98 },
    { 0, 0 },
    { 0xc0577ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d77ed5, 84 },
    { 0, 0 },
    { 0x98777ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d77edc, 95 },
    { 0, 0 },
    { 0x1277ede, 100 },
    { 0xceb77edf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69677ee2, 43 },
    { 0xb6777ee3, 52 },
    { 0xedd77ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb77eec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d577eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df77ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda577ef9, 78 },
    { 0xfbf77efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b77eff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6277f04, 34 },
    { 0xf1277f05, 43 },
    { 0x3af77f06, 90 },
    { 0x8a777f04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59877f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c577f19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4877f1c, 93 },
    { 0xef677f1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d777f21, 43 },
    { 0, 0 },
    { 0x6c877f23, 61 },
    { 0xdc077f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5377f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a977f3f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x90277f42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc77f46, 77 },
    { 0xa6e77f46, 99 },
    { 0xd9a77f48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e077f4e, 95 },
    { 0xc5977f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb77f54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8477f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86477f64, 33 },
    { 0xd0377f64, 95 },
    { 0xaa077f64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0477f69, 65 },
    { 0x94877f6a, 34 },
    { 0xacb77f6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be77f74, 12 },
    { 0xc3d77f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1277f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd77f80, 61 },
    { 0xbee77f81, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5177f89, 33 },
    { 0xa7477f89, 99 },
    { 0, 0 },
    { 0x9ae77f8c, 76 },
    { 0xdfc77f8d, 90 },
    { 0x2df77f8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb77f92, 93 },
    { 0x9f277f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63277f96, 81 },
    { 0xfb77f96, 78 },
    { 0xb4277f96, 46 },
    { 0, 0 },
    { 0xbf677f9a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a77f9e, 85 },
    { 0, 0 },
    { 0x5f077fa0, 61 },
    { 0x58877fa1, 55 },
    { 0x74a77fa2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e77fb2, 33 },
    { 0, 0 },
    { 0x86a77fb4, 33 },
    { 0x25277fb5, 91 },
    { 0, 0 },
    { 0xbe677fb7, 33 },
    { 0, 0 },
    { 0x61677fb9, 33 },
    { 0x3ac77fba, 66 },
    { 0x82577fbb, 98 },
    { 0x99677fba, 23 },
    { 0xcc577fbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f877fc0, 61 },
    { 0x37277fc1, 46 },
    { 0, 0 },
    { 0x14c77fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba677fcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec77fd3, 99 },
    { 0xf5477fd4, 23 },
    { 0, 0 },
    { 0xf3177fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c777fd9, 90 },
    { 0, 0 },
    { 0x5f977fdb, 95 },
    { 0, 0 },
    { 0x3af77fdd, 55 },
    { 0x9ca77fde, 43 },
    { 0, 0 },
    { 0xa377fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf77fe4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f377fe8, 43 },
    { 0xcab77fe9, 95 },
    { 0x38577fe9, 67 },
    { 0xbf577fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d77fee, 98 },
    { 0, 0 },
    { 0x24b77ff0, 34 },
    { 0, 0 },
    { 0xae077ff2, 76 },
    { 0xd7077ff3, 95 },
    { 0, 0 },
    { 0x3e177ff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda677ffe, 56 },
    { 0, 0 },
    { 0x9af78000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec878007, 87 },
    { 0xd4c78008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6677800c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd78012, 46 },
    { 0x1c278012, 95 },
    { 0xc1a78014, 46 },
    { 0, 0 },
    { 0xcb878016, 76 },
    { 0xc5278017, 95 },
    { 0x92878018, 34 },
    { 0x8eb78019, 95 },
    { 0xc2d7801a, 46 },
    { 0xabb78016, 98 },
    { 0x6d778019, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f78020, 98 },
    { 0, 0 },
    { 0xc3278022, 99 },
    { 0x23a78023, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4297802e, 33 },
    { 0xa587802f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3578032, 98 },
    { 0xe0d78032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98478039, 33 },
    { 0xbed7803a, 33 },
    { 0xc507803b, 85 },
    { 0xdb87803b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567803f, 33 },
    { 0x87478040, 78 },
    { 0x2a178041, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12078049, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee78051, 65 },
    { 0xf0378051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e27805d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4378060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687806a, 75 },
    { 0x6917806b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7806f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a78072, 23 },
    { 0x8eb78072, 95 },
    { 0, 0 },
    { 0xddc78075, 93 },
    { 0x3b678076, 61 },
    { 0x76478077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417807b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c7807e, 98 },
    { 0x2067807e, 98 },
    { 0, 0 },
    { 0x70a78081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc78085, 93 },
    { 0, 0 },
    { 0xfa478087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f57808d, 55 },
    { 0x9f07808e, 33 },
    { 0x747808d, 90 },
    { 0x3b27808e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e78093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b78096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7697809a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a780a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60780b8, 93 },
    { 0xac5780b9, 85 },
    { 0xf1a780ba, 33 },
    { 0x5c0780bb, 23 },
    { 0, 0 },
    { 0xafa780bd, 33 },
    { 0, 0 },
    { 0xd11780bf, 84 },
    { 0x6b9780c0, 61 },
    { 0, 0 },
    { 0x28f780c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f780c6, 23 },
    { 0xd02780c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d780ca, 90 },
    { 0x5a5780cb, 33 },
    { 0x6a3780cc, 90 },
    { 0xa3b780cd, 33 },
    { 0, 0 },
    { 0x9c2780cf, 90 },
    { 0xdd3780d0, 46 },
    { 0x64780d0, 61 },
    { 0xada780cf, 95 },
    { 0xe50780cf, 43 },
    { 0x344780d4, 98 },
    { 0, 0 },
    { 0x9a780d6, 90 },
    { 0x2be780d7, 95 },
    { 0x232780d8, 46 },
    { 0, 0 },
    { 0x5c8780da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3780e1, 99 },
    { 0xec9780e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f780e6, 98 },
    { 0xb0780e7, 100 },
    { 0xee6780e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9780ec, 77 },
    { 0, 0 },
    { 0xf45780ee, 87 },
    { 0xa8b780ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c780f8, 33 },
    { 0xb74780f9, 93 },
    { 0x5c8780fa, 66 },
    { 0, 0 },
    { 0x808780fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75780ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf78105, 93 },
    { 0, 0 },
    { 0x51178107, 98 },
    { 0x59378108, 53 },
    { 0x17f78108, 95 },
    { 0x69b7810a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9047810d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd278110, 78 },
    { 0xfbc78111, 23 },
    { 0xa0678112, 95 },
    { 0xfe678110, 43 },
    { 0, 0 },
    { 0x70778115, 92 },
    { 0, 0 },
    { 0x83478117, 33 },
    { 0x73978117, 33 },
    { 0x97e78118, 95 },
    { 0xf47811a, 87 },
    { 0, 0 },
    { 0xc957811c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097811f, 61 },
    { 0xb3378120, 61 },
    { 0, 0 },
    { 0xead78122, 46 },
    { 0, 0 },
    { 0xaf278124, 33 },
    { 0x8c178125, 98 },
    { 0x90278126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a57812a, 46 },
    { 0xffd7812b, 46 },
    { 0xc4a7812c, 98 },
    { 0, 0 },
    { 0x3cd7812e, 23 },
    { 0xd817812e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82878138, 28 },
    { 0xf4778138, 33 },
    { 0x5e978138, 55 },
    { 0x1ee7813b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc167813e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53478143, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2678146, 100 },
    { 0, 0 },
    { 0x2e678148, 88 },
    { 0xe0578148, 33 },
    { 0, 0 },
    { 0x1ba7814b, 77 },
    { 0x6937814b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14278150, 61 },
    { 0x89878150, 46 },
    { 0x31778150, 33 },
    { 0xb8a78153, 61 },
    { 0x1c478154, 95 },
    { 0, 0 },
    { 0x3378156, 61 },
    { 0x6478157, 48 },
    { 0x9ba78157, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f97815e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e878163, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63778166, 34 },
    { 0, 0 },
    { 0x65a78168, 95 },
    { 0xcc078168, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617816c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a178172, 88 },
    { 0x4fa78172, 90 },
    { 0x4f478173, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a7817b, 90 },
    { 0x1667817b, 61 },
    { 0x167817d, 61 },
    { 0x1eb7817d, 88 },
    { 0xf027817d, 34 },
    { 0, 0 },
    { 0xb2e78181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd78186, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77818a, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d7818e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92678195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc78199, 98 },
    { 0xcb27819a, 90 },
    { 0x68f7819b, 95 },
    { 0xdf77819b, 33 },
    { 0xd567819d, 93 },
    { 0x13b78199, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38781a2, 80 },
    { 0, 0 },
    { 0x1fd781a4, 98 },
    { 0xd23781a4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa781a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8781ad, 91 },
    { 0x75e781ae, 85 },
    { 0x350781ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f781b3, 23 },
    { 0, 0 },
    { 0x86781b5, 90 },
    { 0, 0 },
    { 0x839781b7, 93 },
    { 0x5a6781b8, 100 },
    { 0x592781b8, 33 },
    { 0x289781ba, 23 },
    { 0x958781bb, 46 },
    { 0xd3b781ba, 55 },
    { 0, 0 },
    { 0x9781be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1781c3, 90 },
    { 0x8ec781c4, 61 },
    { 0, 0 },
    { 0xbdc781c6, 46 },
    { 0x8b4781c6, 99 },
    { 0, 0 },
    { 0xd40781c9, 33 },
    { 0x6d5781ca, 98 },
    { 0x16a781ca, 61 },
    { 0xa85781cb, 98 },
    { 0x6f0781cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff781d0, 80 },
    { 0, 0 },
    { 0x8b6781d2, 90 },
    { 0x9f781d2, 91 },
    { 0xf03781d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81781d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b781de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a781e3, 90 },
    { 0x421781e4, 90 },
    { 0, 0 },
    { 0x1f9781e6, 33 },
    { 0xa9c781e7, 76 },
    { 0xde9781e6, 61 },
    { 0x9be781e8, 90 },
    { 0xe87781ea, 88 },
    { 0xb3a781eb, 34 },
    { 0xb22781eb, 76 },
    { 0x212781ed, 33 },
    { 0x99c781ee, 61 },
    { 0x4cf781ef, 88 },
    { 0x771781ea, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602781f4, 46 },
    { 0, 0 },
    { 0x99a781f6, 55 },
    { 0, 0 },
    { 0xe05781f8, 98 },
    { 0xb94781f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42878201, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe177820e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7778212, 90 },
    { 0x66278212, 95 },
    { 0x3d678214, 33 },
    { 0, 0 },
    { 0x72878216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c7821b, 76 },
    { 0xc387821c, 100 },
    { 0xf707821c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b78225, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x42778228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417822e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e78235, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf7823b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba78240, 95 },
    { 0, 0 },
    { 0xe3f78242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1578246, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb67824b, 84 },
    { 0x5cd7824b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56078251, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86978254, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78259, 36 },
    { 0, 0 },
    { 0xe077825b, 87 },
    { 0x1937825c, 66 },
    { 0x5957825d, 43 },
    { 0, 0 },
    { 0xb607825f, 78 },
    { 0x8fb78260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f7826b, 98 },
    { 0, 0 },
    { 0xa137826d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f78277, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4127827c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb878283, 95 },
    { 0, 0 },
    { 0xbca78285, 43 },
    { 0xd5378285, 33 },
    { 0x91d78285, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa087828a, 90 },
    { 0xb897828b, 33 },
    { 0xa417828c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68078291, 90 },
    { 0x4c978291, 98 },
    { 0, 0 },
    { 0x77678294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78297, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc7829e, 23 },
    { 0x7567829f, 88 },
    { 0x5b97829f, 95 },
    { 0x5807829f, 61 },
    { 0x3b0782a2, 61 },
    { 0xc2b7829e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0782a7, 23 },
    { 0xf55782a7, 34 },
    { 0x1b0782a9, 61 },
    { 0x8bf782a9, 98 },
    { 0xd9b782a8, 76 },
    { 0, 0 },
    { 0x5fa782ad, 66 },
    { 0, 0 },
    { 0xc63782af, 90 },
    { 0, 0 },
    { 0x98c782b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac782b6, 77 },
    { 0x3fa782b7, 34 },
    { 0xc53782b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f782bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc782bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33782c2, 76 },
    { 0x798782c3, 90 },
    { 0x1f4782c4, 90 },
    { 0, 0 },
    { 0x216782c6, 76 },
    { 0xf14782c6, 95 },
    { 0, 0 },
    { 0x6fe782c9, 61 },
    { 0x251782c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2782d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce782d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13782d7, 87 },
    { 0x4ed782d8, 46 },
    { 0, 0 },
    { 0xf9a782da, 34 },
    { 0xbaa782da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd782de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2782e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24782e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x323782e8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172782ec, 85 },
    { 0xfd9782ed, 76 },
    { 0x7e9782ee, 61 },
    { 0, 0 },
    { 0x8de782f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56782f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x594782f7, 23 },
    { 0x4e5782f7, 23 },
    { 0, 0 },
    { 0x597782fa, 95 },
    { 0x8f7782fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e782fe, 99 },
    { 0xc51782ff, 33 },
    { 0xa2e782fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89878309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe257830d, 67 },
    { 0xc467830e, 95 },
    { 0xb987830e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a078313, 76 },
    { 0x21978314, 90 },
    { 0xf0278314, 61 },
    { 0x56578316, 98 },
    { 0, 0 },
    { 0x1578318, 95 },
    { 0, 0 },
    { 0x21f7831a, 98 },
    { 0x18c7831b, 99 },
    { 0xc087831a, 43 },
    { 0x9977831d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded78321, 61 },
    { 0, 0 },
    { 0x7ff78323, 90 },
    { 0xa9d78323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7832d, 90 },
    { 0, 0 },
    { 0xa0e7832f, 76 },
    { 0xa7578330, 87 },
    { 0xd5078330, 76 },
    { 0xdc078331, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f478335, 81 },
    { 0xd5978336, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3577833c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b78344, 90 },
    { 0, 0 },
    { 0x58878346, 87 },
    { 0xb4c78346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1277834a, 95 },
    { 0, 0 },
    { 0xa6c7834c, 95 },
    { 0xd597834d, 33 },
    { 0, 0 },
    { 0xa57834f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa78354, 33 },
    { 0x3c878355, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca78358, 98 },
    { 0xa4378359, 91 },
    { 0xa8d78359, 77 },
    { 0x68578359, 95 },
    { 0x5297835c, 90 },
    { 0x15078359, 100 },
    { 0x89778359, 17 },
    { 0, 0 },
    { 0x27d78360, 90 },
    { 0x5d978360, 61 },
    { 0x8f278362, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c078367, 61 },
    { 0xab878367, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227836d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96778370, 33 },
    { 0, 0 },
    { 0xb4c78372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35278376, 23 },
    { 0, 0 },
    { 0x7d778378, 76 },
    { 0x79278379, 61 },
    { 0xfda78379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff278384, 37 },
    { 0, 0 },
    { 0xf4f78386, 90 },
    { 0x5e578387, 95 },
    { 0x1c78386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d7838b, 61 },
    { 0, 0 },
    { 0xa337838d, 55 },
    { 0, 0 },
    { 0x4147838f, 61 },
    { 0xa6f7838f, 61 },
    { 0x3b978391, 61 },
    { 0x5678392, 23 },
    { 0x69178393, 61 },
    { 0xbc278394, 53 },
    { 0xa0b78392, 34 },
    { 0x33978396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10783a0, 34 },
    { 0, 0 },
    { 0x7b783a2, 61 },
    { 0x798783a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2783ab, 90 },
    { 0xb9a783ab, 43 },
    { 0xae783ad, 83 },
    { 0xe3a783ae, 90 },
    { 0xf4b783af, 98 },
    { 0xcdc783b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61783b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x228783b7, 95 },
    { 0, 0 },
    { 0xe3c783b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc783bc, 43 },
    { 0xf3f783bd, 61 },
    { 0, 0 },
    { 0x32f783bf, 61 },
    { 0, 0 },
    { 0xf81783c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x767783c4, 95 },
    { 0xd9f783c4, 80 },
    { 0x56a783c4, 77 },
    { 0, 0 },
    { 0x8a7783c8, 90 },
    { 0x7f783c9, 98 },
    { 0, 0 },
    { 0x476783cb, 61 },
    { 0x11b783cc, 75 },
    { 0x793783cd, 43 },
    { 0, 0 },
    { 0xdf0783cf, 65 },
    { 0xb7f783cf, 61 },
    { 0, 0 },
    { 0xb4d783d2, 33 },
    { 0, 0 },
    { 0x9af783d4, 98 },
    { 0x8be783d5, 23 },
    { 0x474783d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1783db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816783e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09783e5, 61 },
    { 0, 0 },
    { 0x27b783e7, 43 },
    { 0xcc1783e8, 78 },
    { 0x7d6783e9, 87 },
    { 0x78f783e9, 46 },
    { 0, 0 },
    { 0x806783ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb783f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f978406, 61 },
    { 0x26578406, 61 },
    { 0, 0 },
    { 0x69978409, 33 },
    { 0, 0 },
    { 0x25d7840b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a278413, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf878417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197841b, 90 },
    { 0xc2b7841b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3507841f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e378423, 76 },
    { 0x5d478424, 34 },
    { 0x83c78425, 78 },
    { 0xb4578424, 88 },
    { 0, 0 },
    { 0x35378428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7842b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac7842f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d78432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b78435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc78439, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7843c, 90 },
    { 0x4ed7843c, 95 },
    { 0x7e47843e, 95 },
    { 0xf47843f, 97 },
    { 0x5ff78440, 90 },
    { 0x62978440, 98 },
    { 0x89b7843c, 93 },
    { 0, 0 },
    { 0xe4378444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097844d, 34 },
    { 0x2f77844d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a78454, 61 },
    { 0xe6178455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46078458, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b7845e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc978467, 76 },
    { 0, 0 },
    { 0x27b78469, 46 },
    { 0x52178469, 93 },
    { 0x5777846a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca78470, 90 },
    { 0x77c78471, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b78476, 66 },
    { 0xe8078476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca57847d, 55 },
    { 0xb627847d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478481, 55 },
    { 0x46078481, 90 },
    { 0x5fb78483, 98 },
    { 0x71778484, 61 },
    { 0, 0 },
    { 0x30178486, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7378489, 98 },
    { 0x7077848a, 61 },
    { 0x8f17848b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7848f, 34 },
    { 0xb9078490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf878493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c37849d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3784ad, 90 },
    { 0xf70784ae, 33 },
    { 0xb28784af, 98 },
    { 0xf07784b0, 78 },
    { 0x64784b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d784b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be784bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00784be, 66 },
    { 0, 0 },
    { 0x365784c0, 95 },
    { 0, 0 },
    { 0x477784c2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94784cd, 46 },
    { 0, 0 },
    { 0xc02784cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3784d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9784d6, 87 },
    { 0x10e784d6, 98 },
    { 0x6d7784d8, 90 },
    { 0x9e3784d7, 33 },
    { 0x823784d6, 33 },
    { 0, 0 },
    { 0xe74784dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x786784df, 55 },
    { 0, 0 },
    { 0xe6b784e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d784e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x881784e9, 90 },
    { 0xf12784ea, 33 },
    { 0x9f9784eb, 90 },
    { 0x1e6784ea, 95 },
    { 0, 0 },
    { 0x26784ee, 90 },
    { 0x5ed784ef, 95 },
    { 0xab5784f0, 95 },
    { 0x55e784ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255784f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x753784f9, 90 },
    { 0xcf9784fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x608784fd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9978500, 69 },
    { 0, 0 },
    { 0x69c78502, 55 },
    { 0, 0 },
    { 0x8c178504, 95 },
    { 0xbf878504, 98 },
    { 0, 0 },
    { 0xd2178507, 100 },
    { 0x2c778508, 90 },
    { 0x2ce78507, 95 },
    { 0, 0 },
    { 0xc6b7850b, 33 },
    { 0x44a7850b, 90 },
    { 0, 0 },
    { 0xb97850e, 61 },
    { 0x9a07850f, 95 },
    { 0xd7f7850f, 61 },
    { 0xf527850e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd78516, 46 },
    { 0x74278516, 23 },
    { 0x24b78518, 98 },
    { 0x16278519, 91 },
    { 0, 0 },
    { 0xfb37851b, 23 },
    { 0xe57851c, 33 },
    { 0x4cb7851b, 61 },
    { 0x5c87851e, 77 },
    { 0x2f67851d, 61 },
    { 0, 0 },
    { 0x93f78521, 72 },
    { 0x5ad78521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48578528, 95 },
    { 0, 0 },
    { 0x5947852a, 61 },
    { 0x6477852b, 95 },
    { 0x47d7852b, 33 },
    { 0xe217852d, 92 },
    { 0xe0d7852e, 61 },
    { 0xb77852e, 98 },
    { 0xa8b78530, 61 },
    { 0x4be7852f, 61 },
    { 0xe4f7852b, 90 },
    { 0x69078531, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x62178536, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21078539, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c7853c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b578542, 76 },
    { 0xfd478543, 90 },
    { 0x30d78544, 56 },
    { 0x62478544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d7854b, 90 },
    { 0xd077854c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73778551, 33 },
    { 0x9fc78552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74978557, 66 },
    { 0x61178557, 61 },
    { 0x91e78559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d07855e, 33 },
    { 0, 0 },
    { 0xf6878560, 33 },
    { 0x1e378561, 98 },
    { 0xd7a78562, 98 },
    { 0x4eb78563, 90 },
    { 0, 0 },
    { 0xbf278565, 61 },
    { 0x69778565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7856b, 61 },
    { 0xcc27856c, 95 },
    { 0xfc27856c, 81 },
    { 0xaed7856c, 87 },
    { 0x18a7856f, 85 },
    { 0x3207856c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5878573, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4178576, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a78579, 69 },
    { 0x3547857a, 78 },
    { 0x1fb78579, 61 },
    { 0x1eb7857b, 34 },
    { 0x9037857b, 90 },
    { 0xb627857e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1078581, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1078587, 33 },
    { 0x38978587, 93 },
    { 0, 0 },
    { 0xdeb7858a, 78 },
    { 0xb987858b, 33 },
    { 0, 0 },
    { 0xa4e7858d, 76 },
    { 0xaf57858e, 23 },
    { 0xfec7858e, 33 },
    { 0, 0 },
    { 0x95f78591, 77 },
    { 0x32a78592, 90 },
    { 0xfe278592, 23 },
    { 0x4eb78591, 56 },
    { 0x3c278595, 100 },
    { 0xae178596, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc807859a, 87 },
    { 0xb1a7859a, 43 },
    { 0, 0 },
    { 0xa3f7859d, 23 },
    { 0x76f7859e, 99 },
    { 0xfc07859f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1785a6, 83 },
    { 0, 0 },
    { 0x5ba785a8, 84 },
    { 0, 0 },
    { 0xcfb785aa, 61 },
    { 0xb24785aa, 91 },
    { 0x979785ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d785af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192785b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f785be, 34 },
    { 0xd12785be, 23 },
    { 0x242785c0, 53 },
    { 0, 0 },
    { 0xc58785c2, 23 },
    { 0xca8785c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x593785c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1785c9, 65 },
    { 0x7e0785ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f785ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x922785d1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce785df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f785eb, 61 },
    { 0, 0 },
    { 0xd4d785ed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x862785f0, 78 },
    { 0, 0 },
    { 0x64e785f2, 90 },
    { 0x139785f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba785f6, 61 },
    { 0x8ce785f7, 55 },
    { 0x474785f7, 76 },
    { 0xa8f785f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3785fc, 68 },
    { 0x171785fc, 76 },
    { 0, 0 },
    { 0x45b785ff, 23 },
    { 0x78d785ff, 23 },
    { 0x4e78600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee278605, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7860c, 98 },
    { 0x9847860c, 65 },
    { 0xa907860d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2978611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3478615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45878619, 100 },
    { 0xdbe7861a, 98 },
    { 0, 0 },
    { 0xdb57861c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d178621, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b78627, 55 },
    { 0, 0 },
    { 0xc8f78629, 52 },
    { 0x3c678629, 23 },
    { 0, 0 },
    { 0x3ba7862c, 55 },
    { 0x1ef7862d, 80 },
    { 0x1b7862c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3678639, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa837863c, 88 },
    { 0x4c7863d, 69 },
    { 0xaf17863e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb7864a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1078651, 80 },
    { 0x6c678652, 90 },
    { 0x55a78653, 81 },
    { 0x92c78651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe967865a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0178664, 78 },
    { 0x3c278664, 52 },
    { 0x69f78666, 100 },
    { 0xeb978666, 67 },
    { 0x26b78668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1557866f, 88 },
    { 0, 0 },
    { 0xa8378671, 98 },
    { 0xc6f78671, 90 },
    { 0xeca78673, 90 },
    { 0x16178673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f78677, 33 },
    { 0x6a778678, 76 },
    { 0xd2178677, 33 },
    { 0xdd27867a, 99 },
    { 0x8797867b, 61 },
    { 0xedf7867b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a478683, 95 },
    { 0xe7778684, 90 },
    { 0, 0 },
    { 0x25c78686, 76 },
    { 0xc1378687, 90 },
    { 0x97c78686, 61 },
    { 0x9a278689, 36 },
    { 0x32478687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567868e, 33 },
    { 0, 0 },
    { 0x45978690, 78 },
    { 0, 0 },
    { 0xf8678692, 61 },
    { 0xff378693, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x37578696, 98 },
    { 0, 0 },
    { 0x1b278698, 23 },
    { 0x35878699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea786a6, 69 },
    { 0, 0 },
    { 0xa93786a8, 34 },
    { 0x294786a9, 95 },
    { 0xdc1786aa, 55 },
    { 0, 0 },
    { 0x192786ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff786bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6786c4, 95 },
    { 0x51f786c5, 23 },
    { 0xc51786c4, 93 },
    { 0x15d786c6, 61 },
    { 0, 0 },
    { 0x1f2786c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb786ce, 23 },
    { 0x569786cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0786d8, 96 },
    { 0xd78786d9, 33 },
    { 0xaa1786da, 98 },
    { 0x10786db, 33 },
    { 0x976786dc, 23 },
    { 0x689786dc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e786e1, 23 },
    { 0, 0 },
    { 0xe4a786e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37786e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x341786eb, 61 },
    { 0x14f786ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754786fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f278705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e78709, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7870d, 23 },
    { 0x6007870d, 90 },
    { 0, 0 },
    { 0x76f78710, 61 },
    { 0xcd978711, 98 },
    { 0x23678712, 98 },
    { 0, 0 },
    { 0x70978714, 23 },
    { 0x6ea78715, 81 },
    { 0xb5878714, 95 },
    { 0x1178716, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27871c, 91 },
    { 0x73e7871c, 61 },
    { 0, 0 },
    { 0xe2e7871f, 76 },
    { 0xc0f78720, 23 },
    { 0xa5d78721, 95 },
    { 0xbd578722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a78726, 33 },
    { 0xf1578726, 61 },
    { 0x73578728, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc97872e, 78 },
    { 0xd447872e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db78732, 93 },
    { 0xb0b78733, 33 },
    { 0xf9378734, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb78737, 91 },
    { 0x3b078738, 95 },
    { 0, 0 },
    { 0x8cc7873a, 95 },
    { 0, 0 },
    { 0xa827873c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78742, 95 },
    { 0xafb78743, 100 },
    { 0xe3778744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e78747, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977874a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd27874d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8678750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe137875e, 55 },
    { 0x6df7875e, 90 },
    { 0xd447875f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f78763, 95 },
    { 0xf0378764, 43 },
    { 0xcb678763, 82 },
    { 0x3b78766, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee478769, 95 },
    { 0, 0 },
    { 0x9157876b, 33 },
    { 0xf1e7876b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef578770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21478775, 52 },
    { 0xdf678775, 99 },
    { 0x76478776, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a7877f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a78783, 33 },
    { 0x3f878783, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xce978787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217878a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb87878f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83878794, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e578798, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d787a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29787a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0787b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87787b7, 55 },
    { 0x905787b8, 33 },
    { 0xf787b9, 33 },
    { 0, 0 },
    { 0x4e5787bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74787c1, 85 },
    { 0x927787c2, 36 },
    { 0x79e787c3, 52 },
    { 0xda1787c2, 99 },
    { 0x48d787c2, 61 },
    { 0, 0 },
    { 0x1ad787c7, 91 },
    { 0xd1e787c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92787cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63787d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2787d5, 33 },
    { 0, 0 },
    { 0xbdd787d7, 55 },
    { 0xf4787d8, 90 },
    { 0x556787d9, 98 },
    { 0xd17787da, 84 },
    { 0xf2d787db, 33 },
    { 0, 0 },
    { 0x2cf787dd, 33 },
    { 0x346787de, 33 },
    { 0x270787df, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c787e3, 23 },
    { 0, 0 },
    { 0x5bc787e5, 90 },
    { 0xef1787e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3787e9, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea787ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786787f2, 100 },
    { 0xa0787f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6787f6, 40 },
    { 0x39c787f7, 90 },
    { 0xefc787f7, 90 },
    { 0x521787f9, 90 },
    { 0xc76787fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0787ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5378803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa247880a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a97880d, 23 },
    { 0, 0 },
    { 0xb197880f, 61 },
    { 0, 0 },
    { 0x8dd78811, 78 },
    { 0, 0 },
    { 0x7378813, 66 },
    { 0x5e478813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b78817, 100 },
    { 0xc4478818, 98 },
    { 0x46f78819, 76 },
    { 0, 0 },
    { 0x9527881b, 46 },
    { 0x38c7881c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f178821, 34 },
    { 0x80278821, 66 },
    { 0x27f78823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24278828, 55 },
    { 0, 0 },
    { 0x3be7882a, 78 },
    { 0, 0 },
    { 0x88d7882c, 99 },
    { 0x1e37882d, 33 },
    { 0x1217882c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4678833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b378836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c77883a, 90 },
    { 0xbd87883b, 90 },
    { 0, 0 },
    { 0xf527883d, 98 },
    { 0x5687883e, 87 },
    { 0xb147883f, 23 },
    { 0, 0 },
    { 0x5c578841, 94 },
    { 0, 0 },
    { 0x9e378843, 90 },
    { 0x9a078844, 61 },
    { 0xef578845, 33 },
    { 0x93f78846, 76 },
    { 0xa9c78847, 91 },
    { 0, 0 },
    { 0xa7378849, 46 },
    { 0, 0 },
    { 0x6207884b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c78855, 33 },
    { 0xd0f78856, 61 },
    { 0xca878857, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8867885c, 23 },
    { 0, 0 },
    { 0xa727885e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77878863, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478867, 98 },
    { 0, 0 },
    { 0xa7478869, 65 },
    { 0x5217886a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c7886e, 33 },
    { 0xc937886f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478872, 98 },
    { 0, 0 },
    { 0xee378874, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3078877, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3267887d, 98 },
    { 0xac57887e, 85 },
    { 0xa8b7887e, 98 },
    { 0, 0 },
    { 0xfdf78881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b78885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e7888a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d7888f, 76 },
    { 0x3d378890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65278893, 95 },
    { 0, 0 },
    { 0x6278895, 98 },
    { 0, 0 },
    { 0x14d78897, 33 },
    { 0x49678897, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8627889c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b7889f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0788a5, 46 },
    { 0, 0 },
    { 0x7b8788a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3788aa, 34 },
    { 0x85f788ab, 28 },
    { 0x566788ac, 76 },
    { 0x666788ac, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210788b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31788b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117788b8, 99 },
    { 0xc7e788b9, 91 },
    { 0x99b788b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d788c1, 43 },
    { 0xb0c788c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a788c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd788cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf788cf, 90 },
    { 0xbb788d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8788d5, 90 },
    { 0, 0 },
    { 0x2fd788d7, 90 },
    { 0, 0 },
    { 0xabb788d9, 33 },
    { 0xc31788da, 97 },
    { 0x705788db, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2788df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee788e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x183788e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e788e8, 98 },
    { 0, 0 },
    { 0x8e5788ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee788f5, 33 },
    { 0x421788f5, 87 },
    { 0x13b788f6, 55 },
    { 0, 0 },
    { 0x6db788f9, 76 },
    { 0x79a788f9, 99 },
    { 0xd54788fb, 87 },
    { 0, 0 },
    { 0x98d788fd, 85 },
    { 0, 0 },
    { 0xbb788ff, 33 },
    { 0xd9b78900, 34 },
    { 0xb1678901, 33 },
    { 0x54378902, 98 },
    { 0x6c578903, 98 },
    { 0xf4678903, 90 },
    { 0x69d78905, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1817890a, 23 },
    { 0x5f7890b, 98 },
    { 0x3d97890a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf7890f, 33 },
    { 0, 0 },
    { 0xc5278911, 68 },
    { 0, 0 },
    { 0xe1f78913, 67 },
    { 0x86678913, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c7891a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82178924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68978927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc247892a, 90 },
    { 0xe327892b, 23 },
    { 0, 0 },
    { 0x78a7892d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf78934, 76 },
    { 0xadc78935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8367893c, 99 },
    { 0x2287893d, 91 },
    { 0xf137893d, 43 },
    { 0x6257893d, 33 },
    { 0x3097893d, 61 },
    { 0x85678941, 23 },
    { 0x5877893f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52878945, 67 },
    { 0x84578946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7894a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe687894d, 81 },
    { 0x6ac7894e, 90 },
    { 0x3a47894f, 33 },
    { 0x61378950, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88678958, 61 },
    { 0x48978959, 36 },
    { 0x4dc7895a, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd178960, 33 },
    { 0, 0 },
    { 0x51678962, 95 },
    { 0x2d878962, 98 },
    { 0, 0 },
    { 0x80678965, 23 },
    { 0x9cf78966, 61 },
    { 0x7878967, 90 },
    { 0xb1678968, 55 },
    { 0x14d78969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae87896e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63378971, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5378976, 34 },
    { 0xdbe78977, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547897c, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2378985, 90 },
    { 0, 0 },
    { 0x64b78987, 55 },
    { 0x6a778987, 46 },
    { 0x66178989, 33 },
    { 0x4f67898a, 76 },
    { 0x71d7898b, 61 },
    { 0xb777898a, 90 },
    { 0, 0 },
    { 0x8847898e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d878993, 99 },
    { 0, 0 },
    { 0x7f578995, 90 },
    { 0x1a778996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8578999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a7899c, 87 },
    { 0, 0 },
    { 0xbe47899e, 56 },
    { 0, 0 },
    { 0xa33789a0, 33 },
    { 0x99b789a0, 98 },
    { 0xd65789a2, 90 },
    { 0x9a0789a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b789a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46789ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b789af, 65 },
    { 0, 0 },
    { 0x419789b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f789b5, 98 },
    { 0x6a2789b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11789bc, 34 },
    { 0, 0 },
    { 0x2eb789be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d789c8, 95 },
    { 0, 0 },
    { 0x397789ca, 90 },
    { 0x890789cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c789d1, 95 },
    { 0x44b789d1, 90 },
    { 0, 0 },
    { 0xd1c789d4, 90 },
    { 0x3e789d5, 98 },
    { 0x7fb789d6, 90 },
    { 0x5d4789d6, 98 },
    { 0x7d5789d8, 66 },
    { 0, 0 },
    { 0xe77789da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x703789dd, 61 },
    { 0, 0 },
    { 0xa50789df, 61 },
    { 0, 0 },
    { 0x973789e1, 51 },
    { 0, 0 },
    { 0xe6a789e3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x195789e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f789ec, 100 },
    { 0xc9b789ed, 33 },
    { 0, 0 },
    { 0x6d4789ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476789f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472789f7, 98 },
    { 0x8cf789f7, 61 },
    { 0x57e789f8, 61 },
    { 0x829789fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f789fe, 77 },
    { 0xadd789ff, 87 },
    { 0x21b78a00, 65 },
    { 0xa4478a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3978a04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5678a0a, 66 },
    { 0xd0978a0b, 33 },
    { 0xc078a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e178a0f, 85 },
    { 0, 0 },
    { 0xbee78a11, 76 },
    { 0x35678a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf978a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69878a1a, 23 },
    { 0, 0 },
    { 0xd9778a1c, 90 },
    { 0x2eb78a1d, 34 },
    { 0xa4378a1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d78a23, 33 },
    { 0xeff78a24, 23 },
    { 0xb5e78a25, 84 },
    { 0xafb78a26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9978a2d, 61 },
    { 0, 0 },
    { 0x6e478a2f, 61 },
    { 0xc6b78a30, 33 },
    { 0, 0 },
    { 0xf8478a32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2778a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78a38, 90 },
    { 0xca978a38, 98 },
    { 0x9d378a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c578a3e, 61 },
    { 0x8f978a3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778a42, 33 },
    { 0x44178a42, 33 },
    { 0xd6d78a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb78a47, 97 },
    { 0x33f78a47, 99 },
    { 0x8c78a47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84678a4e, 98 },
    { 0, 0 },
    { 0xaca78a50, 61 },
    { 0x13678a50, 95 },
    { 0xac178a52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f878a57, 78 },
    { 0xa9878a57, 61 },
    { 0xef178a59, 34 },
    { 0x7d678a57, 95 },
    { 0xc6778a58, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66378a5f, 93 },
    { 0x6ea78a5f, 78 },
    { 0xa7a78a60, 23 },
    { 0xbc878a62, 95 },
    { 0, 0 },
    { 0xd9078a64, 98 },
    { 0xb4078a65, 76 },
    { 0x4978a66, 88 },
    { 0xe8b78a67, 90 },
    { 0x77a78a67, 90 },
    { 0x4e578a65, 95 },
    { 0xad378a6a, 90 },
    { 0x79f78a64, 75 },
    { 0xcc478a6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68478a70, 33 },
    { 0, 0 },
    { 0x5a378a72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df78a83, 61 },
    { 0xa7878a83, 98 },
    { 0x5bb78a85, 61 },
    { 0x1a978a84, 33 },
    { 0xc7278a87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb078a8f, 76 },
    { 0, 0 },
    { 0x34978a91, 55 },
    { 0, 0 },
    { 0xb1c78a93, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d78a96, 61 },
    { 0xe778a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a78a9a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c978a9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d78aa0, 23 },
    { 0x7978aa1, 77 },
    { 0xdbd78aa2, 33 },
    { 0xb5c78aa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43178aa8, 23 },
    { 0, 0 },
    { 0xc1578aaa, 43 },
    { 0, 0 },
    { 0x84f78aac, 55 },
    { 0xb1d78aad, 55 },
    { 0x96b78aad, 87 },
    { 0x8f678aac, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2478ab7, 90 },
    { 0xcda78ab7, 67 },
    { 0xa0078ab8, 90 },
    { 0x18c78aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92978abf, 69 },
    { 0, 0 },
    { 0xc7378ac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16478acb, 90 },
    { 0, 0 },
    { 0x2b178acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb78ad0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed78ada, 93 },
    { 0xb5b78adb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15678adf, 90 },
    { 0x68c78adf, 95 },
    { 0x45078ae0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48178ae9, 38 },
    { 0x56d78ae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8378aee, 23 },
    { 0, 0 },
    { 0x4f678af0, 33 },
    { 0, 0 },
    { 0x85e78af2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c78afc, 23 },
    { 0xa978afd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478b03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a378b06, 55 },
    { 0, 0 },
    { 0xef478b08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed78b0d, 34 },
    { 0, 0 },
    { 0x57a78b0f, 85 },
    { 0xa0778b0f, 61 },
    { 0x2ba78b0f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x11378b14, 33 },
    { 0, 0 },
    { 0xa9978b16, 34 },
    { 0, 0 },
    { 0xf7278b18, 99 },
    { 0, 0 },
    { 0x39578b1a, 95 },
    { 0, 0 },
    { 0x47c78b1c, 95 },
    { 0x25178b1c, 90 },
    { 0, 0 },
    { 0x8c078b1f, 84 },
    { 0xf4778b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e378b2a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e78b2e, 99 },
    { 0xbd178b2f, 90 },
    { 0x9b978b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f78b37, 76 },
    { 0xe6878b37, 94 },
    { 0, 0 },
    { 0xa9478b3a, 95 },
    { 0x8d078b3b, 76 },
    { 0x4bc78b3b, 34 },
    { 0, 0 },
    { 0x45978b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66778b43, 84 },
    { 0, 0 },
    { 0x50e78b45, 52 },
    { 0x56f78b46, 33 },
    { 0, 0 },
    { 0xb3a78b48, 66 },
    { 0xf2b78b49, 91 },
    { 0, 0 },
    { 0x47778b4b, 23 },
    { 0x3bc78b4c, 98 },
    { 0x73e78b4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a78b50, 33 },
    { 0x47478b50, 95 },
    { 0, 0 },
    { 0xa6c78b53, 23 },
    { 0x5f678b54, 90 },
    { 0x20e78b53, 90 },
    { 0, 0 },
    { 0xd2478b57, 23 },
    { 0xf1a78b58, 98 },
    { 0xe6378b57, 98 },
    { 0xb5578b5a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f78b5f, 23 },
    { 0x61078b60, 33 },
    { 0, 0 },
    { 0xa7378b62, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd78b65, 99 },
    { 0x2e178b66, 34 },
    { 0xa8978b67, 61 },
    { 0xda778b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c78b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce78b71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd78b75, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7078b78, 99 },
    { 0xc8578b79, 90 },
    { 0xd9e78b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c278b7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50378b80, 66 },
    { 0x6eb78b81, 95 },
    { 0, 0 },
    { 0x4c078b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33178b86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56478b8a, 95 },
    { 0x86a78b8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd778b8e, 61 },
    { 0, 0 },
    { 0xef478b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b578b94, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e78b98, 55 },
    { 0xd7a78b99, 76 },
    { 0x2c678b9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e78b9d, 66 },
    { 0x71d78b9d, 61 },
    { 0xcea78b9f, 99 },
    { 0xbdd78ba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2978ba5, 78 },
    { 0xc3278ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8078ba9, 76 },
    { 0xf2578baa, 61 },
    { 0x57778bab, 33 },
    { 0x2a78bac, 34 },
    { 0xd7a78bab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8778bb2, 78 },
    { 0x84d78bb2, 90 },
    { 0x3a978bb2, 55 },
    { 0, 0 },
    { 0x9eb78bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1078bb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2178bbd, 90 },
    { 0x89978bbe, 61 },
    { 0xd8678bbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66678bc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec78bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb78bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d778bcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc78bd0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad78bd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb78bdd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10878be2, 17 },
    { 0xbbb78be2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf78be6, 34 },
    { 0x11578be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc78bec, 98 },
    { 0, 0 },
    { 0x55578bee, 87 },
    { 0xd0c78bef, 90 },
    { 0x15b78bf0, 56 },
    { 0xcbf78bf0, 43 },
    { 0xe578bf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d878bf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb78bfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e178c09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78c0f, 100 },
    { 0x42a78c10, 23 },
    { 0x59978c11, 78 },
    { 0, 0 },
    { 0xa9c78c13, 95 },
    { 0, 0 },
    { 0x68578c15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x97878c18, 77 },
    { 0xe5b78c19, 90 },
    { 0, 0 },
    { 0xc5878c1b, 55 },
    { 0, 0 },
    { 0x84678c1d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa178c22, 78 },
    { 0x21978c22, 90 },
    { 0x73278c22, 91 },
    { 0x30c78c25, 90 },
    { 0x65278c25, 90 },
    { 0, 0 },
    { 0x2c478c28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3778c2b, 61 },
    { 0x48c78c2b, 98 },
    { 0x42a78c2b, 43 },
    { 0x74178c2e, 90 },
    { 0x53978c2f, 56 },
    { 0xf1d78c2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a78c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65778c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b78c40, 23 },
    { 0x2a178c41, 33 },
    { 0x48178c41, 95 },
    { 0x44678c43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e978c46, 90 },
    { 0, 0 },
    { 0x45a78c48, 41 },
    { 0x59a78c49, 23 },
    { 0x4eb78c49, 61 },
    { 0xaaa78c4a, 98 },
    { 0, 0 },
    { 0x4d278c4d, 52 },
    { 0, 0 },
    { 0x48d78c4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87978c52, 84 },
    { 0x9da78c52, 76 },
    { 0xbb378c54, 33 },
    { 0x55078c54, 98 },
    { 0x3ee78c56, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5078c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b78c5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5478c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0678c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67278c6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18778c75, 76 },
    { 0x87e78c76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57378c79, 90 },
    { 0x16178c7a, 23 },
    { 0xec778c7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17678c7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18178c88, 81 },
    { 0, 0 },
    { 0xef278c8a, 23 },
    { 0xe0878c8b, 90 },
    { 0, 0 },
    { 0x7e678c8d, 33 },
    { 0x5b378c8e, 90 },
    { 0x6378c8e, 52 },
    { 0xf8878c8d, 23 },
    { 0x8c178c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b78c95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97778c99, 91 },
    { 0, 0 },
    { 0x86378c9b, 23 },
    { 0x12778c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b278c9f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc78ca9, 97 },
    { 0, 0 },
    { 0x51378cab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e78cae, 33 },
    { 0, 0 },
    { 0x4e278cb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd78cb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed178cbc, 61 },
    { 0, 0 },
    { 0x76778cbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d78cc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f678cc4, 61 },
    { 0x8ed78cc5, 95 },
    { 0x1e378cc6, 90 },
    { 0, 0 },
    { 0x10f78cc8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2178cce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98878cd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2878cda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85578cde, 46 },
    { 0x7a578cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3778ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed778cef, 93 },
    { 0xee978cf0, 33 },
    { 0xe5878cf1, 76 },
    { 0x2b078cf2, 83 },
    { 0, 0 },
    { 0x9a278cf4, 97 },
    { 0x44d78cf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea78cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51978cfe, 93 },
    { 0, 0 },
    { 0x81b78d00, 97 },
    { 0xe1778d01, 90 },
    { 0xa4978d02, 95 },
    { 0xe3378d03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de78d07, 70 },
    { 0x28678d07, 95 },
    { 0xca078d08, 99 },
    { 0, 0 },
    { 0x84378d0b, 90 },
    { 0x5f378d0c, 52 },
    { 0x42c78d0d, 61 },
    { 0x478d0c, 91 },
    { 0x34a78d0c, 33 },
    { 0x35578d10, 33 },
    { 0xeed78d10, 33 },
    { 0x85978d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14978d15, 55 },
    { 0xd5478d15, 55 },
    { 0, 0 },
    { 0x22378d18, 43 },
    { 0xece78d19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f978d1c, 100 },
    { 0, 0 },
    { 0xfa878d1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd178d21, 35 },
    { 0, 0 },
    { 0xfa678d23, 100 },
    { 0xa0078d24, 68 },
    { 0x17478d25, 93 },
    { 0x41378d24, 61 },
    { 0x7e978d24, 90 },
    { 0xd2478d28, 85 },
    { 0xe8e78d28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d78d2d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba78d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31378d38, 33 },
    { 0, 0 },
    { 0x60578d3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6678d41, 91 },
    { 0x1ce78d42, 87 },
    { 0xaff78d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3178d48, 88 },
    { 0xb5e78d48, 55 },
    { 0xabf78d4a, 61 },
    { 0x8578d4a, 33 },
    { 0x7dd78d4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78d50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc78d61, 61 },
    { 0x4cd78d61, 76 },
    { 0x34678d63, 55 },
    { 0x7d178d64, 43 },
    { 0x9f978d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c78d69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf178d6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c78d6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64778d73, 34 },
    { 0xb1f78d74, 90 },
    { 0x3c678d73, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46178d7b, 61 },
    { 0, 0 },
    { 0xc8078d7d, 98 },
    { 0, 0 },
    { 0xcaa78d7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76778d83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a78d89, 98 },
    { 0x34a78d8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27478d8d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23478d91, 95 },
    { 0x4f578d92, 100 },
    { 0x4f078d91, 84 },
    { 0x9dc78d94, 99 },
    { 0x4cf78d92, 33 },
    { 0, 0 },
    { 0x81b78d97, 23 },
    { 0x66c78d98, 98 },
    { 0xbd478d99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c78d9e, 23 },
    { 0x46678d9f, 61 },
    { 0xd6e78da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b78da3, 61 },
    { 0, 0 },
    { 0x2df78da5, 88 },
    { 0, 0 },
    { 0x77b78da7, 61 },
    { 0xc7d78da8, 90 },
    { 0, 0 },
    { 0xc0c78daa, 90 },
    { 0x62b78dab, 98 },
    { 0x9c178dac, 46 },
    { 0, 0 },
    { 0x5cc78dae, 100 },
    { 0xa4578daf, 98 },
    { 0x11d78db0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62278db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc78dba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc78dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84978dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478dd7, 98 },
    { 0, 0 },
    { 0x56978dd9, 33 },
    { 0, 0 },
    { 0x6df78ddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1178ddf, 98 },
    { 0x54778ddf, 61 },
    { 0xbc678de1, 90 },
    { 0x2b978de2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a678de6, 98 },
    { 0x3fe78de7, 33 },
    { 0, 0 },
    { 0xba678de9, 23 },
    { 0x17878dea, 95 },
    { 0, 0 },
    { 0xf2a78dec, 76 },
    { 0, 0 },
    { 0x68f78dee, 95 },
    { 0xc3e78def, 55 },
    { 0, 0 },
    { 0xc1378df1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678df5, 93 },
    { 0x7ac78df6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b678dfa, 33 },
    { 0xa3478dfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3478e0a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd78e10, 55 },
    { 0x12978e11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb78e1a, 98 },
    { 0, 0 },
    { 0xcc78e1c, 90 },
    { 0, 0 },
    { 0x53a78e1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e78e26, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x65878e29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15478e2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b78e30, 33 },
    { 0x81b78e31, 68 },
    { 0, 0 },
    { 0x21a78e33, 61 },
    { 0x21078e34, 23 },
    { 0, 0 },
    { 0xc8078e36, 95 },
    { 0x1b978e37, 90 },
    { 0x7ea78e36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9078e3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78e3e, 33 },
    { 0xd3678e3f, 90 },
    { 0x65f78e3e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef78e47, 76 },
    { 0xbd178e48, 17 },
    { 0x6f78e47, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x878e4c, 95 },
    { 0x51078e4d, 80 },
    { 0x91078e4e, 96 },
    { 0, 0 },
    { 0x6df78e50, 95 },
    { 0, 0 },
    { 0x31c78e52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b078e58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a78e5e, 76 },
    { 0xf3d78e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2078e72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48978e77, 90 },
    { 0xbdf78e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e78e7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89378e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8578e87, 46 },
    { 0, 0 },
    { 0x7b978e89, 36 },
    { 0, 0 },
    { 0xec78e8b, 95 },
    { 0x8df78e8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c978e8f, 64 },
    { 0x7bf78e8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44378e94, 78 },
    { 0, 0 },
    { 0x64a78e96, 33 },
    { 0x9b78e97, 100 },
    { 0x1e978e96, 61 },
    { 0, 0 },
    { 0xd3978e9a, 23 },
    { 0, 0 },
    { 0x41778e9c, 88 },
    { 0, 0 },
    { 0xf5f78e9e, 90 },
    { 0xad678e9e, 61 },
    { 0xe8f78ea0, 61 },
    { 0x8b078ea1, 90 },
    { 0x5d578ea2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xff478ea5, 46 },
    { 0x6c978ea6, 90 },
    { 0x33978ea7, 33 },
    { 0xb3278ea5, 33 },
    { 0, 0 },
    { 0xa678eaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7578eb2, 26 },
    { 0x2778eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf078eb9, 23 },
    { 0x91378eba, 52 },
    { 0, 0 },
    { 0xf5f78ebc, 61 },
    { 0x5e178ebc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c278ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3878ec7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478ed0, 98 },
    { 0x87478ed0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78ed6, 61 },
    { 0x11178ed7, 90 },
    { 0xfa78ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4678ee1, 76 },
    { 0x3b678ee2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75778ee7, 84 },
    { 0x80c78ee7, 33 },
    { 0x15678ee9, 33 },
    { 0, 0 },
    { 0x7e578eeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a78ef1, 34 },
    { 0xb7778ef1, 23 },
    { 0xb5778ef3, 61 },
    { 0x36b78ef4, 77 },
    { 0xa6a78ef5, 90 },
    { 0x97278ef6, 77 },
    { 0xac378ef4, 93 },
    { 0x2b178ef6, 46 },
    { 0x34478ef4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9878f00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf78f03, 46 },
    { 0xa0e78f04, 88 },
    { 0, 0 },
    { 0x7cc78f06, 23 },
    { 0, 0 },
    { 0x4a578f08, 33 },
    { 0x3f978f09, 61 },
    { 0, 0 },
    { 0x66b78f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4778f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478f11, 99 },
    { 0xce878f12, 43 },
    { 0xe9478f11, 34 },
    { 0x41a78f11, 98 },
    { 0, 0 },
    { 0x72a78f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f78f19, 68 },
    { 0, 0 },
    { 0xa7478f1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b78f24, 61 },
    { 0x1fe78f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78f29, 80 },
    { 0, 0 },
    { 0x71978f2b, 33 },
    { 0x9a378f2c, 98 },
    { 0x36678f2d, 23 },
    { 0x8da78f2e, 66 },
    { 0xf6e78f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478f33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a778f38, 55 },
    { 0x84578f39, 81 },
    { 0x51a78f3a, 81 },
    { 0xaf678f39, 75 },
    { 0xea878f3c, 99 },
    { 0xeea78f39, 34 },
    { 0xa9478f3e, 23 },
    { 0x65478f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8678f42, 95 },
    { 0, 0 },
    { 0x2b978f44, 76 },
    { 0x85278f45, 23 },
    { 0x16b78f45, 67 },
    { 0xb2978f47, 61 },
    { 0x33e78f47, 91 },
    { 0x24578f44, 90 },
    { 0, 0 },
    { 0x68078f4b, 76 },
    { 0xfb178f4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b178f53, 23 },
    { 0x76078f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62478f58, 23 },
    { 0, 0 },
    { 0x19278f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37778f60, 61 },
    { 0xc7678f61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be78f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac278f69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc578f74, 55 },
    { 0, 0 },
    { 0x8b678f76, 55 },
    { 0, 0 },
    { 0x20978f78, 90 },
    { 0x93d78f78, 52 },
    { 0x91078f7a, 23 },
    { 0x5e978f7a, 90 },
    { 0x15e78f7c, 82 },
    { 0xecc78f79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb78f83, 33 },
    { 0xc8a78f83, 33 },
    { 0, 0 },
    { 0x50f78f86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c378f8a, 76 },
    { 0x3ec78f8b, 52 },
    { 0xb5c78f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c78f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7478f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba878f97, 55 },
    { 0xbc078f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b178f9c, 33 },
    { 0x46b78f9d, 77 },
    { 0x12c78f9e, 61 },
    { 0, 0 },
    { 0xeea78fa0, 93 },
    { 0x39778fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f78fa5, 23 },
    { 0x1cc78fa6, 34 },
    { 0, 0 },
    { 0xcc978fa8, 33 },
    { 0xae678fa8, 23 },
    { 0xe3378faa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc378fad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd78fb0, 61 },
    { 0, 0 },
    { 0xcb578fb2, 61 },
    { 0, 0 },
    { 0x8be78fb4, 61 },
    { 0, 0 },
    { 0x38478fb6, 76 },
    { 0xea678fb6, 100 },
    { 0xb3e78fb8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34678fbd, 55 },
    { 0x8f378fbe, 77 },
    { 0x3bf78fbe, 66 },
    { 0x9e078fbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd78fd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7578fd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed78fdb, 85 },
    { 0xe9278fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49678fe2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x90878fe5, 33 },
    { 0x9578fe5, 90 },
    { 0xcc378fe5, 95 },
    { 0, 0 },
    { 0x87478fe9, 98 },
    { 0xcf378fea, 61 },
    { 0x88278feb, 78 },
    { 0x3f478fe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5778fef, 98 },
    { 0xd6c78fef, 23 },
    { 0, 0 },
    { 0x23f78ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65679000, 83 },
    { 0xb6779001, 17 },
    { 0x6b679002, 33 },
    { 0x6e779000, 43 },
    { 0xa2c79002, 56 },
    { 0x28779001, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0679008, 75 },
    { 0x46e79008, 98 },
    { 0, 0 },
    { 0xf27900b, 2 },
    { 0xad17900c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83879011, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e579014, 23 },
    { 0x6f479014, 93 },
    { 0xc7479015, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4007901d, 93 },
    { 0xf617901e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0879021, 34 },
    { 0, 0 },
    { 0xfc179023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379026, 90 },
    { 0xfb779027, 76 },
    { 0x64b79028, 95 },
    { 0x77279029, 43 },
    { 0xaa87902a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc79033, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7903b, 23 },
    { 0xfae7903b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af79046, 17 },
    { 0, 0 },
    { 0x3a279048, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6667904d, 93 },
    { 0x50b7904e, 43 },
    { 0x5677904f, 55 },
    { 0, 0 },
    { 0x5ae79051, 95 },
    { 0x36879052, 23 },
    { 0x85279052, 87 },
    { 0xe7f79054, 61 },
    { 0x70c79052, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf7905b, 90 },
    { 0xb057905b, 77 },
    { 0x7dc7905d, 90 },
    { 0x8097905c, 33 },
    { 0x2007905d, 23 },
    { 0x62279060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a79069, 77 },
    { 0, 0 },
    { 0xa147906b, 90 },
    { 0, 0 },
    { 0x39f7906d, 99 },
    { 0x1ad7906d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73679073, 95 },
    { 0x10379074, 90 },
    { 0xaca79074, 87 },
    { 0x50379076, 85 },
    { 0xe2279076, 14 },
    { 0, 0 },
    { 0x66279079, 61 },
    { 0, 0 },
    { 0xf627907b, 23 },
    { 0, 0 },
    { 0xde17907d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad79081, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f779085, 76 },
    { 0x41979086, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7908a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b579091, 33 },
    { 0, 0 },
    { 0xbf779093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7879099, 76 },
    { 0, 0 },
    { 0xed37909b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830790a0, 90 },
    { 0x419790a0, 95 },
    { 0xa2b790a2, 87 },
    { 0x3da790a3, 52 },
    { 0x8af790a3, 23 },
    { 0x24790a5, 61 },
    { 0x29a790a5, 33 },
    { 0x9cb790a5, 55 },
    { 0x77d790a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588790ac, 90 },
    { 0x939790ad, 90 },
    { 0x73e790ae, 100 },
    { 0x891790af, 66 },
    { 0x547790ae, 52 },
    { 0, 0 },
    { 0x4b1790b2, 33 },
    { 0, 0 },
    { 0x560790b4, 90 },
    { 0, 0 },
    { 0x1d4790b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b790bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96790bf, 33 },
    { 0x124790c0, 90 },
    { 0, 0 },
    { 0x9a4790c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224790ca, 23 },
    { 0x8ed790cb, 98 },
    { 0x4f6790cc, 84 },
    { 0x34a790cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969790d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3790d7, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790da, 33 },
    { 0xc1e790db, 95 },
    { 0x5f790dc, 95 },
    { 0, 0 },
    { 0xa8a790de, 67 },
    { 0xffc790df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4790e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648790e8, 33 },
    { 0xc0e790e9, 33 },
    { 0x3a6790e9, 61 },
    { 0x61d790eb, 33 },
    { 0x52f790e9, 95 },
    { 0x857790ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc790f1, 66 },
    { 0x2dd790f2, 34 },
    { 0, 0 },
    { 0xdc3790f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae790f7, 23 },
    { 0, 0 },
    { 0xd53790f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5790fc, 76 },
    { 0x889790fd, 36 },
    { 0x86c790fe, 98 },
    { 0, 0 },
    { 0xf5879100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b79103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d79108, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a47910d, 61 },
    { 0x29e7910e, 68 },
    { 0, 0 },
    { 0x82f79110, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f79113, 91 },
    { 0xdb79113, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b79118, 98 },
    { 0, 0 },
    { 0xb577911a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c79126, 46 },
    { 0, 0 },
    { 0x33379128, 55 },
    { 0, 0 },
    { 0x14c7912a, 91 },
    { 0xaae7912a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9079130, 95 },
    { 0x30679131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc687913a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x547913d, 88 },
    { 0xf877913d, 80 },
    { 0xe7b7913f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c679142, 98 },
    { 0x72779143, 95 },
    { 0x7c79144, 43 },
    { 0x10d79145, 90 },
    { 0x1e379145, 61 },
    { 0x5f279144, 95 },
    { 0, 0 },
    { 0x3c779149, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e7914d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c779156, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7915a, 33 },
    { 0x14c7915b, 95 },
    { 0xb057915c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17479169, 34 },
    { 0, 0 },
    { 0x6cb7916b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857916f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2679173, 61 },
    { 0x39179173, 46 },
    { 0, 0 },
    { 0x8ab79176, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c79179, 46 },
    { 0xa0e7917a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23679182, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df79185, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20479189, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9579190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db79193, 67 },
    { 0xae079194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33791a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d791a8, 90 },
    { 0xce6791a9, 98 },
    { 0x50d791aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1791b7, 33 },
    { 0x33a791b8, 23 },
    { 0xb2e791b7, 91 },
    { 0x2de791ba, 23 },
    { 0xf6a791bb, 61 },
    { 0x81d791bb, 91 },
    { 0x8f6791b7, 87 },
    { 0x379791be, 95 },
    { 0x171791bc, 100 },
    { 0xdd791bd, 98 },
    { 0xdc8791b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c791c6, 36 },
    { 0xf3c791c7, 46 },
    { 0x74a791c6, 43 },
    { 0, 0 },
    { 0x33e791ca, 76 },
    { 0xa5e791cb, 90 },
    { 0xca9791cc, 90 },
    { 0x706791cd, 23 },
    { 0xdb2791cd, 66 },
    { 0x1f2791cd, 32 },
    { 0x69791d0, 88 },
    { 0, 0 },
    { 0xe48791d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c791dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474791e1, 23 },
    { 0x7ee791e1, 55 },
    { 0xbcf791e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5791e6, 76 },
    { 0xbbb791e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834791f0, 76 },
    { 0, 0 },
    { 0xf1f791f2, 90 },
    { 0x345791f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11791f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00791fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed79200, 40 },
    { 0xdc879200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e79205, 52 },
    { 0x45579206, 33 },
    { 0x73d79205, 64 },
    { 0xbac79205, 95 },
    { 0x9b379208, 61 },
    { 0, 0 },
    { 0x4c97920b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7920f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd79212, 33 },
    { 0xd9579213, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7307921a, 76 },
    { 0, 0 },
    { 0x96f7921c, 75 },
    { 0xb7e7921d, 90 },
    { 0, 0 },
    { 0x6057921f, 69 },
    { 0, 0 },
    { 0xcb979221, 91 },
    { 0x31b79222, 23 },
    { 0x7079222, 90 },
    { 0xb979224, 33 },
    { 0x9f879222, 76 },
    { 0xc7a79226, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c07922c, 64 },
    { 0x1dc7922d, 23 },
    { 0xe97922e, 33 },
    { 0x54a7922f, 43 },
    { 0xf1879230, 46 },
    { 0x9b07922e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88679234, 23 },
    { 0xdf979235, 90 },
    { 0x99979235, 23 },
    { 0, 0 },
    { 0x7b779238, 93 },
    { 0xc5e79238, 85 },
    { 0xe2279238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb79240, 81 },
    { 0x8c679241, 98 },
    { 0x4da79241, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x44479245, 95 },
    { 0x35a79246, 23 },
    { 0x26879246, 23 },
    { 0xd9879248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a7924b, 8 },
    { 0xd167924b, 23 },
    { 0x9377924d, 46 },
    { 0xd987924c, 61 },
    { 0, 0 },
    { 0x62f79250, 95 },
    { 0, 0 },
    { 0x4a279252, 90 },
    { 0, 0 },
    { 0xb6b79254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf579258, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c7925c, 95 },
    { 0xe5e7925d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a79265, 23 },
    { 0xf6279265, 84 },
    { 0, 0 },
    { 0xf4f79268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687926f, 90 },
    { 0x7e879270, 95 },
    { 0xbba79270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d79274, 90 },
    { 0x7dd79275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd7927a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c79283, 95 },
    { 0, 0 },
    { 0x22b79285, 90 },
    { 0x52179286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f679289, 33 },
    { 0, 0 },
    { 0xcbf7928b, 90 },
    { 0xeb7928b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6307928f, 34 },
    { 0x47928f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1947929a, 23 },
    { 0x44a7929a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd657929e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43792a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24792a9, 95 },
    { 0, 0 },
    { 0x9c9792ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0792ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1792b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5792bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc792be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5792c2, 90 },
    { 0, 0 },
    { 0x287792c4, 76 },
    { 0x244792c5, 40 },
    { 0xc8792c6, 76 },
    { 0x6ea792c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6792cc, 61 },
    { 0xb30792cd, 85 },
    { 0, 0 },
    { 0xe60792cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46792d2, 53 },
    { 0xbb9792d3, 61 },
    { 0x792792d2, 33 },
    { 0x97e792d5, 90 },
    { 0xa0c792d6, 101 },
    { 0xc2f792d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db792e3, 98 },
    { 0xfb3792e4, 61 },
    { 0, 0 },
    { 0xe2d792e6, 46 },
    { 0xf82792e6, 23 },
    { 0x4c4792e7, 95 },
    { 0xd7792e9, 33 },
    { 0x63e792ea, 23 },
    { 0x86a792eb, 33 },
    { 0xea8792ea, 76 },
    { 0x456792ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x652792f0, 43 },
    { 0x6ad792f1, 46 },
    { 0, 0 },
    { 0xb81792f3, 91 },
    { 0, 0 },
    { 0x152792f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8792fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334792ff, 91 },
    { 0, 0 },
    { 0x13979301, 68 },
    { 0xc8e79301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff79305, 46 },
    { 0x38679306, 23 },
    { 0x67e79307, 55 },
    { 0xf5e79306, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab79311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf379315, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7931c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9979320, 61 },
    { 0xc6779321, 23 },
    { 0xd5279321, 84 },
    { 0, 0 },
    { 0xbb179324, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1879327, 98 },
    { 0, 0 },
    { 0x40279329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7932d, 90 },
    { 0xb327932e, 43 },
    { 0xb87932d, 90 },
    { 0x71e7932d, 88 },
    { 0x5d17932e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b179334, 43 },
    { 0x52079335, 76 },
    { 0, 0 },
    { 0x83979337, 61 },
    { 0, 0 },
    { 0xd6079339, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff779343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3679346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6977934b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737934e, 33 },
    { 0xee07934f, 61 },
    { 0, 0 },
    { 0x30779351, 61 },
    { 0x89a79351, 33 },
    { 0xbcc79353, 84 },
    { 0xbe379353, 46 },
    { 0, 0 },
    { 0x1ac79356, 46 },
    { 0x11a79356, 61 },
    { 0x8f979358, 66 },
    { 0xd5f79359, 98 },
    { 0, 0 },
    { 0xef17935b, 43 },
    { 0x5d07935c, 23 },
    { 0x7077935d, 94 },
    { 0xcb77935e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55779362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf147936a, 43 },
    { 0, 0 },
    { 0x7397936c, 84 },
    { 0, 0 },
    { 0x3007936e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa079376, 95 },
    { 0xaf379376, 76 },
    { 0, 0 },
    { 0xb6e79379, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb7937c, 93 },
    { 0xf257937d, 33 },
    { 0xd5d7937e, 34 },
    { 0, 0 },
    { 0x8f279380, 61 },
    { 0x62a79381, 61 },
    { 0, 0 },
    { 0xd2479383, 55 },
    { 0, 0 },
    { 0x32f79385, 81 },
    { 0xd6e79386, 78 },
    { 0xd9879385, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877938b, 33 },
    { 0xce47938c, 34 },
    { 0, 0 },
    { 0xbcc7938e, 36 },
    { 0x2507938f, 55 },
    { 0xfaa79390, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8679397, 90 },
    { 0, 0 },
    { 0x2e679399, 68 },
    { 0x7a27939a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147939e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383793a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f793a6, 96 },
    { 0x2a2793a7, 53 },
    { 0xd63793a8, 23 },
    { 0x274793a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b793ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb793b4, 93 },
    { 0xbc4793b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x884793b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819793c4, 95 },
    { 0x988793c5, 33 },
    { 0x88f793c5, 33 },
    { 0x2cf793c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55793ca, 67 },
    { 0, 0 },
    { 0x21e793cc, 61 },
    { 0, 0 },
    { 0x59e793ce, 90 },
    { 0xdc8793cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e793d2, 61 },
    { 0, 0 },
    { 0x309793d4, 33 },
    { 0xfa7793d5, 65 },
    { 0x79a793d6, 43 },
    { 0, 0 },
    { 0xd0793d8, 52 },
    { 0x9eb793d9, 95 },
    { 0xe1e793da, 90 },
    { 0x708793d9, 95 },
    { 0xc0b793dc, 90 },
    { 0, 0 },
    { 0x256793de, 23 },
    { 0x258793df, 90 },
    { 0x901793e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2793e4, 61 },
    { 0xc62793e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e793ec, 98 },
    { 0x57d793ed, 98 },
    { 0x946793ee, 55 },
    { 0xb04793ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d793f2, 90 },
    { 0xa7a793f3, 90 },
    { 0x89a793f2, 33 },
    { 0x7a5793f5, 90 },
    { 0, 0 },
    { 0xbc0793f7, 90 },
    { 0x79a793f8, 95 },
    { 0x213793f9, 100 },
    { 0xb1b793f7, 52 },
    { 0xf1f793fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1793fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf679401, 90 },
    { 0, 0 },
    { 0xe5679403, 23 },
    { 0x4779404, 98 },
    { 0, 0 },
    { 0x67b79406, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f17940d, 68 },
    { 0, 0 },
    { 0x9a77940f, 90 },
    { 0x88e79410, 90 },
    { 0x23d79411, 99 },
    { 0, 0 },
    { 0xe0a79413, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79418, 93 },
    { 0x76079418, 55 },
    { 0, 0 },
    { 0x1b17941b, 90 },
    { 0x9f77941c, 61 },
    { 0x9ce7941c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd679420, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce279423, 92 },
    { 0, 0 },
    { 0x47279425, 52 },
    { 0x8c79426, 61 },
    { 0x4d979426, 61 },
    { 0xea479426, 68 },
    { 0x2479428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc727942c, 95 },
    { 0, 0 },
    { 0xf0b7942e, 90 },
    { 0xd147942f, 98 },
    { 0xc3879430, 68 },
    { 0x33279430, 85 },
    { 0xad879432, 46 },
    { 0x53e79432, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d379437, 78 },
    { 0xe0779438, 23 },
    { 0xeee79437, 87 },
    { 0, 0 },
    { 0xf2b7943b, 38 },
    { 0x9617943c, 33 },
    { 0xb87943c, 46 },
    { 0x7437943d, 76 },
    { 0xcbc7943b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f79442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30279446, 90 },
    { 0x47979446, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917944e, 90 },
    { 0x1db7944f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c979454, 43 },
    { 0, 0 },
    { 0xd0579456, 17 },
    { 0xc0f79457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf7945b, 55 },
    { 0x2ce7945c, 69 },
    { 0x28c7945d, 90 },
    { 0xf47945d, 85 },
    { 0xf4c7945f, 23 },
    { 0xceb7945e, 78 },
    { 0x34d79461, 95 },
    { 0x3347945d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4379465, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e879468, 90 },
    { 0, 0 },
    { 0xad27946a, 61 },
    { 0xfaa7946a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b7946f, 33 },
    { 0xe2379470, 34 },
    { 0x8779471, 95 },
    { 0xc7179472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce579475, 61 },
    { 0, 0 },
    { 0x23c79477, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb857947a, 34 },
    { 0x5bd7947b, 93 },
    { 0, 0 },
    { 0x5447947d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed79484, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597948b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff79491, 69 },
    { 0, 0 },
    { 0xab179493, 47 },
    { 0xe9679494, 52 },
    { 0x57b79494, 90 },
    { 0xefb79496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7949a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b794a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904794a6, 76 },
    { 0, 0 },
    { 0x4a5794a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674794ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640794b7, 33 },
    { 0x842794b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4794bb, 23 },
    { 0x74f794bc, 75 },
    { 0x278794bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712794c7, 76 },
    { 0x1d7794c7, 98 },
    { 0, 0 },
    { 0xf72794ca, 90 },
    { 0xe21794ca, 90 },
    { 0x9bd794cc, 33 },
    { 0, 0 },
    { 0x879794ce, 66 },
    { 0xbac794ce, 33 },
    { 0, 0 },
    { 0xa44794d1, 78 },
    { 0xff9794d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2794d7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650794db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db794e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733794e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2794e6, 95 },
    { 0x62d794e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab794ee, 46 },
    { 0x89e794ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4794f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec794fc, 98 },
    { 0, 0 },
    { 0xc36794fe, 90 },
    { 0, 0 },
    { 0xe6e79500, 95 },
    { 0xa2e79501, 33 },
    { 0, 0 },
    { 0xb2279503, 91 },
    { 0x8dd79503, 90 },
    { 0x96279503, 67 },
    { 0x2ac79506, 87 },
    { 0x47579507, 90 },
    { 0xae179507, 58 },
    { 0x8af79509, 90 },
    { 0, 0 },
    { 0x9a27950b, 33 },
    { 0, 0 },
    { 0xadf7950d, 93 },
    { 0, 0 },
    { 0xf87950f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2879519, 61 },
    { 0x88979519, 98 },
    { 0x6517951a, 78 },
    { 0x3157951c, 23 },
    { 0, 0 },
    { 0x5567951e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4479523, 76 },
    { 0, 0 },
    { 0x86279525, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597952b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c579531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae79534, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b479537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87953a, 33 },
    { 0x40d7953b, 23 },
    { 0, 0 },
    { 0xcfe7953d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab79541, 33 },
    { 0xe9d79542, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d679547, 61 },
    { 0xb4079548, 23 },
    { 0x28479548, 76 },
    { 0x62d7954a, 98 },
    { 0, 0 },
    { 0x6137954c, 68 },
    { 0x9f87954c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef79552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c579556, 83 },
    { 0x5d979557, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e7955a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df7955d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb079561, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a79566, 53 },
    { 0x6f379566, 43 },
    { 0x88f79567, 90 },
    { 0x83e79569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3879573, 76 },
    { 0x5b779573, 65 },
    { 0x5e879574, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e379578, 96 },
    { 0, 0 },
    { 0xb527957a, 43 },
    { 0xe5c7957b, 68 },
    { 0x16a7957c, 90 },
    { 0x207957c, 43 },
    { 0, 0 },
    { 0xe4f7957f, 95 },
    { 0xd5279580, 23 },
    { 0x52779581, 61 },
    { 0, 0 },
    { 0xc6a79583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63879588, 23 },
    { 0, 0 },
    { 0xad97958a, 61 },
    { 0, 0 },
    { 0x1717958c, 43 },
    { 0, 0 },
    { 0x5d57958e, 36 },
    { 0x3db7958f, 34 },
    { 0, 0 },
    { 0x6d779591, 59 },
    { 0, 0 },
    { 0x9f779593, 90 },
    { 0, 0 },
    { 0xa7779595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9287959d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4795a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158795a5, 34 },
    { 0x4a0795a6, 93 },
    { 0x23f795a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58795ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba795b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd795b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e795bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322795c7, 61 },
    { 0x59f795c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73795cd, 46 },
    { 0, 0 },
    { 0x60d795cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415795d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf795d7, 45 },
    { 0xbe6795d8, 33 },
    { 0, 0 },
    { 0x4ee795da, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab795e2, 55 },
    { 0, 0 },
    { 0xfa4795e4, 55 },
    { 0, 0 },
    { 0xbcc795e6, 95 },
    { 0x5dd795e7, 23 },
    { 0x10b795e8, 90 },
    { 0xc51795e9, 76 },
    { 0x454795e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x927795ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd795f0, 46 },
    { 0x708795f1, 61 },
    { 0x973795f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7795f8, 56 },
    { 0, 0 },
    { 0xdd7795fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136795fe, 78 },
    { 0, 0 },
    { 0x26379600, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f479603, 77 },
    { 0xe2879603, 23 },
    { 0x1f079605, 17 },
    { 0x5bb79606, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd7960c, 47 },
    { 0x1507960d, 23 },
    { 0, 0 },
    { 0x4f57960f, 61 },
    { 0x6ee79610, 98 },
    { 0x89279611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf79614, 33 },
    { 0xf1379614, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d79618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb077961b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99079621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be79628, 99 },
    { 0x84779628, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5379632, 46 },
    { 0, 0 },
    { 0x20879634, 97 },
    { 0x51479635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc7963c, 76 },
    { 0x6937963d, 76 },
    { 0x99d7963c, 61 },
    { 0xdfa7963f, 85 },
    { 0x5d7963f, 90 },
    { 0x59379640, 55 },
    { 0x58a79642, 67 },
    { 0, 0 },
    { 0xc7a79644, 61 },
    { 0x2b279645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b7964a, 91 },
    { 0, 0 },
    { 0x3007964c, 76 },
    { 0x4297964d, 93 },
    { 0xfc07964e, 76 },
    { 0xeb07964f, 91 },
    { 0x1eb7964e, 33 },
    { 0, 0 },
    { 0x24c79652, 61 },
    { 0, 0 },
    { 0x3a779654, 33 },
    { 0x53579655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47965f, 61 },
    { 0, 0 },
    { 0xe079661, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x44279664, 33 },
    { 0x6879665, 43 },
    { 0x5e479666, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49279669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c27966e, 90 },
    { 0, 0 },
    { 0xd7f79670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5479673, 98 },
    { 0, 0 },
    { 0x5cb79675, 34 },
    { 0x2079676, 38 },
    { 0xe0579675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7967d, 95 },
    { 0x2c47967e, 61 },
    { 0x7947967d, 23 },
    { 0xd3679680, 34 },
    { 0x77079680, 61 },
    { 0xac179682, 67 },
    { 0xfbb79683, 90 },
    { 0x79479684, 23 },
    { 0x1ce79684, 61 },
    { 0x96679684, 84 },
    { 0x11879687, 23 },
    { 0x52679682, 95 },
    { 0x13d79689, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c79691, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef79696, 87 },
    { 0x96d79696, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734796a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f796a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae796ae, 33 },
    { 0xc9e796af, 33 },
    { 0x1e2796b0, 90 },
    { 0xcc2796b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20796b6, 95 },
    { 0xa796b7, 61 },
    { 0xc76796b8, 61 },
    { 0x903796b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c796c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778796c7, 77 },
    { 0xe12796c8, 61 },
    { 0xa3d796c9, 90 },
    { 0xd96796c8, 93 },
    { 0x54d796cb, 40 },
    { 0x1f0796cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x904796cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f796d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82796d6, 17 },
    { 0xd0f796d7, 33 },
    { 0, 0 },
    { 0x1dd796d9, 33 },
    { 0xc18796da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90796e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277796e5, 77 },
    { 0x85d796e6, 90 },
    { 0xbc6796e7, 17 },
    { 0x9a0796e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b796ec, 67 },
    { 0, 0 },
    { 0x51d796ee, 95 },
    { 0, 0 },
    { 0x410796f0, 33 },
    { 0x510796f1, 23 },
    { 0, 0 },
    { 0x9e0796f3, 100 },
    { 0, 0 },
    { 0xb22796f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22796f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb796ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d79705, 61 },
    { 0, 0 },
    { 0xf7179707, 52 },
    { 0x64179708, 98 },
    { 0, 0 },
    { 0xb737970a, 67 },
    { 0x3397970a, 93 },
    { 0xb357970b, 90 },
    { 0xcb97970d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e879713, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56379718, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x917971b, 55 },
    { 0xbac7971c, 61 },
    { 0, 0 },
    { 0xdc27971e, 93 },
    { 0xb7f7971e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6479723, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4879726, 61 },
    { 0, 0 },
    { 0x74d79728, 61 },
    { 0, 0 },
    { 0xd4c7972a, 33 },
    { 0xc1c7972b, 33 },
    { 0x46a7972c, 76 },
    { 0, 0 },
    { 0xe17972e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d79732, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd79735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd79739, 95 },
    { 0, 0 },
    { 0x6d37973b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd987973e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c79743, 90 },
    { 0, 0 },
    { 0x3479745, 98 },
    { 0x5ed79746, 93 },
    { 0, 0 },
    { 0x38979748, 46 },
    { 0x2b379748, 90 },
    { 0x71079748, 95 },
    { 0, 0 },
    { 0x8d47974c, 61 },
    { 0x4e67974c, 52 },
    { 0xcaf7974e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b79751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24779754, 23 },
    { 0xec379754, 95 },
    { 0xccb79755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd7975b, 98 },
    { 0, 0 },
    { 0x1357975d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47779761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f79765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a279768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb7976b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e79778, 68 },
    { 0xe3579778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6357977f, 90 },
    { 0xcab79780, 52 },
    { 0, 0 },
    { 0xfdb79782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8779785, 90 },
    { 0x80679786, 95 },
    { 0x12a79786, 90 },
    { 0x2aa79788, 33 },
    { 0xc0e79789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x47978c, 91 },
    { 0xa7978d, 98 },
    { 0x5767978e, 17 },
    { 0xa1b7978f, 61 },
    { 0x67d7978c, 33 },
    { 0x4be7978d, 90 },
    { 0xdd17978f, 90 },
    { 0x97f79793, 95 },
    { 0xa0079794, 23 },
    { 0xbda79795, 85 },
    { 0x8c679795, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f979799, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56797a0, 93 },
    { 0xd9d797a0, 81 },
    { 0x6db797a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7797aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1797ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435797b7, 43 },
    { 0x549797b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8797c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2797cc, 95 },
    { 0xa92797cd, 77 },
    { 0x388797ce, 90 },
    { 0x8cd797cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x797797d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e797d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf797e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace797ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac797ef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e797f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee797fb, 84 },
    { 0xc42797fc, 33 },
    { 0x8c2797fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79279802, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c079807, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f7980c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d979814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf179818, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae7981b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8379823, 17 },
    { 0xd6c79823, 33 },
    { 0x5f079824, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7982c, 90 },
    { 0x11e7982c, 46 },
    { 0, 0 },
    { 0x17e7982f, 90 },
    { 0xb8979830, 68 },
    { 0x2e679830, 96 },
    { 0x9b79832, 77 },
    { 0xeeb79832, 95 },
    { 0x6e579834, 34 },
    { 0, 0 },
    { 0x6179836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5307983b, 55 },
    { 0xc767983c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e379841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22879847, 49 },
    { 0x10779848, 98 },
    { 0xbfc79849, 76 },
    { 0x95c7984a, 95 },
    { 0xb307984a, 43 },
    { 0x5357984b, 46 },
    { 0x5c97984a, 61 },
    { 0x2487984e, 33 },
    { 0, 0 },
    { 0x12c79850, 95 },
    { 0xe3b79851, 61 },
    { 0xc3579852, 95 },
    { 0, 0 },
    { 0x7ee79854, 90 },
    { 0xd3679855, 33 },
    { 0xda179856, 61 },
    { 0, 0 },
    { 0x55979858, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5627985d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd679866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97179872, 61 },
    { 0, 0 },
    { 0x9d179874, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a79878, 98 },
    { 0x27479879, 61 },
    { 0x5417987a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8079884, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa79887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf857988a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa417988f, 90 },
    { 0, 0 },
    { 0x41f79891, 98 },
    { 0x7b879891, 98 },
    { 0x68379891, 55 },
    { 0xacb79894, 23 },
    { 0xf6d79891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487989b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a7989f, 23 },
    { 0, 0 },
    { 0xd0798a1, 98 },
    { 0, 0 },
    { 0x809798a3, 33 },
    { 0xde798a4, 52 },
    { 0x7d798a5, 23 },
    { 0x97798a4, 33 },
    { 0xb25798a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7798aa, 95 },
    { 0x746798ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b798ae, 90 },
    { 0xc59798ae, 52 },
    { 0xb64798b0, 33 },
    { 0xd91798b1, 90 },
    { 0xe28798b0, 23 },
    { 0x211798b3, 61 },
    { 0x9a2798b4, 81 },
    { 0x9a5798b5, 43 },
    { 0xbe7798b4, 33 },
    { 0x941798b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500798bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19798c0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a798c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x277798c9, 95 },
    { 0, 0 },
    { 0x793798cb, 33 },
    { 0, 0 },
    { 0x18a798cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x778798d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x375798d3, 90 },
    { 0x715798d4, 46 },
    { 0xc02798d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee798da, 61 },
    { 0xbba798db, 23 },
    { 0xa5b798da, 66 },
    { 0, 0 },
    { 0x296798de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c798e1, 33 },
    { 0x705798e1, 88 },
    { 0, 0 },
    { 0x8e0798e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c798e9, 89 },
    { 0x2f6798ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b798ee, 69 },
    { 0x6a2798ef, 93 },
    { 0x842798ee, 61 },
    { 0, 0 },
    { 0x35d798f2, 76 },
    { 0xfb1798f2, 98 },
    { 0x1cc798f4, 93 },
    { 0, 0 },
    { 0x8de798f6, 90 },
    { 0, 0 },
    { 0x70a798f8, 17 },
    { 0xeb7798f9, 66 },
    { 0xf95798fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49798ff, 98 },
    { 0x68579900, 98 },
    { 0xc2c79901, 90 },
    { 0xd8979902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b179906, 49 },
    { 0xa579907, 77 },
    { 0x92579907, 90 },
    { 0, 0 },
    { 0xfa7990a, 95 },
    { 0xc8d7990b, 55 },
    { 0x62d7990c, 85 },
    { 0, 0 },
    { 0x9ec7990e, 33 },
    { 0xc607990e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a079913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1379919, 67 },
    { 0, 0 },
    { 0xbd07991b, 87 },
    { 0xa687991c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7991f, 69 },
    { 0x3af79920, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4079923, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db79926, 23 },
    { 0xd1379926, 61 },
    { 0x55a79928, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f7992b, 33 },
    { 0x5937992b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f79930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3379939, 55 },
    { 0x1387993a, 98 },
    { 0, 0 },
    { 0x85c7993c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe379948, 23 },
    { 0xb2579948, 61 },
    { 0, 0 },
    { 0x9f27994b, 33 },
    { 0, 0 },
    { 0xcbc7994d, 23 },
    { 0, 0 },
    { 0x26f7994f, 33 },
    { 0, 0 },
    { 0x1fb79951, 76 },
    { 0xb4979952, 23 },
    { 0, 0 },
    { 0xa2679954, 91 },
    { 0xe5579955, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc79959, 33 },
    { 0x91a7995a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227995d, 23 },
    { 0, 0 },
    { 0xe347995f, 90 },
    { 0xba57995f, 46 },
    { 0xec979961, 33 },
    { 0x43779962, 55 },
    { 0, 0 },
    { 0x3279964, 91 },
    { 0x23979965, 66 },
    { 0, 0 },
    { 0xf8179967, 85 },
    { 0x5db79968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3347996e, 90 },
    { 0, 0 },
    { 0x6e079970, 95 },
    { 0xb3979970, 23 },
    { 0x58c79972, 93 },
    { 0x7d579972, 33 },
    { 0x57b79974, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e79978, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7997d, 61 },
    { 0x1d17997d, 90 },
    { 0xb837997f, 90 },
    { 0x36479980, 55 },
    { 0xb0579980, 23 },
    { 0, 0 },
    { 0x4a679983, 90 },
    { 0xc9279984, 23 },
    { 0xd679984, 67 },
    { 0xcc079986, 23 },
    { 0x6fb79984, 46 },
    { 0x43679985, 98 },
    { 0x8ef79984, 33 },
    { 0x81579989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397998e, 33 },
    { 0x217998f, 46 },
    { 0x3e879990, 91 },
    { 0, 0 },
    { 0x9ac79992, 100 },
    { 0, 0 },
    { 0xaac79994, 90 },
    { 0x43c79995, 34 },
    { 0, 0 },
    { 0x9d579997, 90 },
    { 0x6d79997, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff27999b, 61 },
    { 0, 0 },
    { 0xc7d7999d, 52 },
    { 0x3ef7999e, 98 },
    { 0x1067999e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae799a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0799a8, 99 },
    { 0, 0 },
    { 0xb82799aa, 90 },
    { 0, 0 },
    { 0x7ab799ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c799af, 93 },
    { 0, 0 },
    { 0xe1f799b1, 61 },
    { 0x8c4799b2, 55 },
    { 0x88a799b3, 90 },
    { 0x91c799b1, 23 },
    { 0xe31799b4, 33 },
    { 0xaff799b6, 23 },
    { 0x51b799b7, 55 },
    { 0x4f6799b6, 61 },
    { 0xf99799b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd799c0, 34 },
    { 0xd74799c1, 61 },
    { 0x7a7799c1, 33 },
    { 0xb48799c3, 98 },
    { 0x280799c2, 76 },
    { 0x33c799c5, 98 },
    { 0x5eb799c6, 90 },
    { 0x52799c7, 78 },
    { 0x876799c8, 95 },
    { 0x50d799c9, 40 },
    { 0x659799c0, 97 },
    { 0x15a799cb, 99 },
    { 0x3fe799cc, 52 },
    { 0x8b6799cd, 95 },
    { 0x285799ce, 28 },
    { 0x843799cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3799d3, 33 },
    { 0x6fb799d4, 23 },
    { 0x814799d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15799da, 76 },
    { 0, 0 },
    { 0x3f1799dc, 61 },
    { 0x45e799dc, 77 },
    { 0xcba799dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e799e6, 55 },
    { 0x819799e7, 67 },
    { 0x1cd799e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a799ed, 33 },
    { 0, 0 },
    { 0xd16799ef, 90 },
    { 0x99799f0, 33 },
    { 0x1f5799f1, 98 },
    { 0, 0 },
    { 0x2f0799f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e799f6, 33 },
    { 0, 0 },
    { 0xb7e799f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42799fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a79a02, 33 },
    { 0xb3c79a03, 23 },
    { 0x4eb79a04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53279a0c, 33 },
    { 0, 0 },
    { 0x79179a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc479a11, 98 },
    { 0x63d79a12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79a15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b679a1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9579a1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6079a21, 61 },
    { 0, 0 },
    { 0xb1d79a23, 90 },
    { 0xdf179a23, 61 },
    { 0, 0 },
    { 0xe3c79a26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c579a2f, 85 },
    { 0x78779a30, 76 },
    { 0x40979a31, 90 },
    { 0x46479a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46679a35, 17 },
    { 0, 0 },
    { 0x6b179a37, 90 },
    { 0, 0 },
    { 0x53d79a39, 78 },
    { 0, 0 },
    { 0xf1479a3b, 93 },
    { 0, 0 },
    { 0x8b479a3d, 52 },
    { 0x1fb79a3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x89479a41, 55 },
    { 0xe2379a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51179a46, 55 },
    { 0, 0 },
    { 0xb5d79a48, 61 },
    { 0x3dc79a48, 65 },
    { 0, 0 },
    { 0xa1379a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61779a51, 75 },
    { 0xadb79a52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a79a55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b579a5b, 34 },
    { 0x56079a5c, 55 },
    { 0, 0 },
    { 0x2fe79a5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e979a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b79a69, 90 },
    { 0x65579a6a, 78 },
    { 0x81079a69, 84 },
    { 0x2d579a6b, 95 },
    { 0xfea79a6d, 93 },
    { 0x86a79a6d, 33 },
    { 0x16679a6d, 98 },
    { 0x92179a6d, 69 },
    { 0xdb379a6d, 95 },
    { 0xb1879a72, 98 },
    { 0, 0 },
    { 0x93079a74, 85 },
    { 0xe1b79a74, 61 },
    { 0, 0 },
    { 0x14879a77, 23 },
    { 0xae279a78, 23 },
    { 0xe2279a79, 98 },
    { 0x79579a7a, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e79a7e, 43 },
    { 0, 0 },
    { 0xc1179a80, 23 },
    { 0x41979a81, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac79a86, 52 },
    { 0x99a79a87, 23 },
    { 0xfb279a88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d79a8c, 34 },
    { 0x94079a8c, 33 },
    { 0x1b879a8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93479a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6479a98, 55 },
    { 0x3e579a99, 61 },
    { 0x4ea79a99, 76 },
    { 0xe8079a9b, 43 },
    { 0xcc979a9b, 23 },
    { 0xfeb79a9c, 91 },
    { 0x14479a9c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d79aa6, 76 },
    { 0xd4679aa6, 33 },
    { 0xea379aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae179aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6579aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79ab2, 61 },
    { 0x60679ab3, 88 },
    { 0xe8a79ab4, 61 },
    { 0x14079ab3, 61 },
    { 0, 0 },
    { 0xa7579ab7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc479aba, 55 },
    { 0x30f79abb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4779abf, 91 },
    { 0, 0 },
    { 0x78c79ac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6679ac4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f79ac8, 90 },
    { 0x88679ac9, 90 },
    { 0xb5179aca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6379acd, 23 },
    { 0xfde79acd, 90 },
    { 0x17f79acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd479ad2, 43 },
    { 0x94179ad3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d679ad9, 55 },
    { 0xe4079ad9, 77 },
    { 0, 0 },
    { 0xd3479adc, 92 },
    { 0x56079add, 23 },
    { 0xd3d79ade, 95 },
    { 0, 0 },
    { 0x14479ae0, 33 },
    { 0x6d579ae0, 95 },
    { 0x26879ae2, 95 },
    { 0xc8079ae3, 65 },
    { 0xfe879ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a79aeb, 55 },
    { 0x94979aec, 33 },
    { 0x1a179aed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71679af8, 95 },
    { 0x13d79af9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af79afc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc79aff, 88 },
    { 0x4de79aff, 90 },
    { 0, 0 },
    { 0x2d079b02, 61 },
    { 0, 0 },
    { 0x3ba79b04, 76 },
    { 0x5d879b04, 65 },
    { 0, 0 },
    { 0xd1179b07, 85 },
    { 0xc2a79b07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa879b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79379b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c79b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f79b1e, 91 },
    { 0x6ac79b1f, 55 },
    { 0xc7279b1e, 33 },
    { 0x3c179b20, 55 },
    { 0xdd779b20, 100 },
    { 0x82779b23, 34 },
    { 0, 0 },
    { 0x7cc79b25, 95 },
    { 0x4e679b26, 36 },
    { 0, 0 },
    { 0xe5079b28, 85 },
    { 0, 0 },
    { 0x93e79b2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a979b2d, 61 },
    { 0x57279b2d, 61 },
    { 0x29979b2f, 61 },
    { 0x56e79b30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3179b33, 95 },
    { 0xc6979b34, 98 },
    { 0x9d279b35, 53 },
    { 0, 0 },
    { 0x27479b37, 98 },
    { 0, 0 },
    { 0xc8d79b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2879b3c, 90 },
    { 0x82379b3d, 61 },
    { 0x6479b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11779b41, 47 },
    { 0xa1579b42, 90 },
    { 0x59979b43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8079b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e679b4a, 61 },
    { 0, 0 },
    { 0x12379b4c, 40 },
    { 0x65279b4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e79b53, 23 },
    { 0, 0 },
    { 0xacc79b55, 61 },
    { 0x6f879b56, 90 },
    { 0x8df79b56, 85 },
    { 0xd6b79b58, 90 },
    { 0x99279b59, 61 },
    { 0x57f79b57, 67 },
    { 0x6279b5b, 23 },
    { 0, 0 },
    { 0x9b879b5d, 69 },
    { 0, 0 },
    { 0x76879b5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d079b65, 87 },
    { 0x5c979b66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79b69, 17 },
    { 0x14579b6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58779b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1879b7b, 95 },
    { 0, 0 },
    { 0xd0679b7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da79b80, 66 },
    { 0xb1779b81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91579b85, 68 },
    { 0xa9c79b86, 61 },
    { 0xa0979b87, 95 },
    { 0x6c179b88, 33 },
    { 0x39f79b88, 95 },
    { 0x64e79b88, 55 },
    { 0, 0 },
    { 0x1779b8c, 76 },
    { 0x18679b8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6779b91, 90 },
    { 0xf2a79b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c79b97, 23 },
    { 0x3c479b98, 95 },
    { 0x71b79b99, 81 },
    { 0x26679b99, 90 },
    { 0x2b579b9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1679b9e, 85 },
    { 0x91179b9f, 84 },
    { 0x8679ba0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c179ba5, 75 },
    { 0x65479ba5, 90 },
    { 0x8a079ba7, 81 },
    { 0x5b879ba8, 46 },
    { 0x7379ba6, 67 },
    { 0x57c79ba6, 23 },
    { 0, 0 },
    { 0x3ff79bac, 34 },
    { 0xeb979bac, 80 },
    { 0x6a579bae, 90 },
    { 0xdc79baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb79bb2, 61 },
    { 0, 0 },
    { 0xb9c79bb4, 61 },
    { 0xd4079bb5, 61 },
    { 0x5a479bb6, 33 },
    { 0x3b379bb7, 91 },
    { 0x1b279bb4, 90 },
    { 0x83a79bb9, 93 },
    { 0xbd979bba, 78 },
    { 0xd8379bbb, 98 },
    { 0xa5979bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c79bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f79bc3, 86 },
    { 0x61879bc4, 46 },
    { 0x1b979bc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e79bcc, 55 },
    { 0x53079bcd, 78 },
    { 0x64379bcd, 52 },
    { 0xc2279bcd, 98 },
    { 0x7e679bd0, 95 },
    { 0, 0 },
    { 0xd9579bd2, 90 },
    { 0, 0 },
    { 0x97b79bd4, 95 },
    { 0, 0 },
    { 0x3479bd6, 61 },
    { 0x9d979bd6, 90 },
    { 0, 0 },
    { 0x6dd79bd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b79bdc, 99 },
    { 0x97379bdd, 91 },
    { 0xab79bdd, 46 },
    { 0x34079bde, 48 },
    { 0, 0 },
    { 0xcf879be1, 55 },
    { 0x54879be2, 33 },
    { 0xdea79be1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79079be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db79bea, 68 },
    { 0x9ca79beb, 95 },
    { 0x8d79bea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf079bf0, 69 },
    { 0x94579bf0, 90 },
    { 0, 0 },
    { 0xc79bf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39679bf7, 93 },
    { 0x71979bf8, 95 },
    { 0x32579bf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9579bfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84279c00, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4279c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d79c07, 43 },
    { 0x7c879c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c979c10, 23 },
    { 0x97679c11, 33 },
    { 0x95479c12, 52 },
    { 0x179c10, 34 },
    { 0xf2279c14, 66 },
    { 0x4b379c15, 55 },
    { 0x56279c10, 95 },
    { 0xde279c17, 33 },
    { 0xaff79c14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57079c1d, 80 },
    { 0x7c379c1e, 90 },
    { 0xf9a79c1f, 91 },
    { 0x6b079c20, 93 },
    { 0x92879c21, 33 },
    { 0xb6779c1d, 34 },
    { 0xc8f79c21, 36 },
    { 0xf9f79c22, 61 },
    { 0, 0 },
    { 0x5f279c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b79c2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb79c30, 93 },
    { 0xcfd79c31, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd79c36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c79c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e579c45, 61 },
    { 0xaaf79c46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc079c4b, 98 },
    { 0xc5679c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab79c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a79c5b, 90 },
    { 0xc179c5c, 33 },
    { 0x12979c5d, 76 },
    { 0, 0 },
    { 0x74b79c5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39479c63, 61 },
    { 0xb6679c64, 98 },
    { 0x40379c63, 98 },
    { 0x89579c65, 81 },
    { 0, 0 },
    { 0x69c79c68, 93 },
    { 0xb8279c69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74779c6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41579c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a079c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c079c7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7079c7d, 55 },
    { 0x1c279c7e, 90 },
    { 0x84079c7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b779c82, 61 },
    { 0xfe879c83, 33 },
    { 0xa3079c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90679c8f, 95 },
    { 0x93079c90, 46 },
    { 0xe8579c90, 98 },
    { 0xe0479c92, 33 },
    { 0x30079c93, 43 },
    { 0x60679c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca979c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba79c9e, 90 },
    { 0xc4a79c9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec79ca8, 36 },
    { 0, 0 },
    { 0x59c79caa, 66 },
    { 0xaee79cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e479caf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb79cb2, 76 },
    { 0xa8b79cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51079cba, 90 },
    { 0, 0 },
    { 0x3d079cbc, 23 },
    { 0x32879cbd, 85 },
    { 0xc8979cbe, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab79cc8, 61 },
    { 0, 0 },
    { 0x5d79cca, 33 },
    { 0x7b979ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d79cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69079cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e79cd6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19679cdb, 34 },
    { 0, 0 },
    { 0xdf979cdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d79ce5, 95 },
    { 0xb3979ce5, 98 },
    { 0x73579ce7, 33 },
    { 0x5f579ce8, 98 },
    { 0xeea79ce9, 61 },
    { 0xaec79cea, 78 },
    { 0, 0 },
    { 0xd2779cec, 93 },
    { 0, 0 },
    { 0x20479cee, 90 },
    { 0x9c979cef, 33 },
    { 0xa0b79cf0, 98 },
    { 0x9a779cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb179cf6, 81 },
    { 0x67f79cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca79cfa, 90 },
    { 0xd2579cfb, 61 },
    { 0, 0 },
    { 0x5fc79cfd, 23 },
    { 0, 0 },
    { 0x83379cff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95279d03, 78 },
    { 0x67d79d04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3479d0b, 98 },
    { 0, 0 },
    { 0x48879d0d, 36 },
    { 0xaeb79d0d, 36 },
    { 0x1e479d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf79d12, 64 },
    { 0x14379d12, 46 },
    { 0x26479d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38179d18, 23 },
    { 0xe4079d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad79d1d, 95 },
    { 0, 0 },
    { 0x2fd79d1f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26079d24, 93 },
    { 0xf3279d25, 61 },
    { 0x60679d24, 98 },
    { 0xf9579d25, 93 },
    { 0x4e979d26, 80 },
    { 0, 0 },
    { 0x48979d2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df79d2d, 95 },
    { 0xf6f79d2e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a79d3c, 98 },
    { 0xdd879d3d, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14279d41, 90 },
    { 0, 0 },
    { 0x37279d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76079d47, 55 },
    { 0x89279d48, 90 },
    { 0x3cb79d49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a679d4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57579d4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd379d53, 84 },
    { 0xed279d54, 17 },
    { 0xfd79d55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b79d58, 23 },
    { 0x8c479d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46579d60, 95 },
    { 0xf6a79d60, 90 },
    { 0, 0 },
    { 0x1379d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1279d69, 43 },
    { 0xce579d6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8679d6d, 98 },
    { 0, 0 },
    { 0xc2279d6f, 98 },
    { 0x50f79d70, 33 },
    { 0x8a579d70, 100 },
    { 0x30879d70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e79d77, 61 },
    { 0xa1979d78, 33 },
    { 0, 0 },
    { 0xc5779d7a, 33 },
    { 0xe3779d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da79d8c, 61 },
    { 0xd4c79d8d, 23 },
    { 0xfa79d8c, 52 },
    { 0, 0 },
    { 0x99879d90, 88 },
    { 0, 0 },
    { 0xff379d92, 61 },
    { 0x42f79d93, 77 },
    { 0x29779d93, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979d97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb79d9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80679da1, 100 },
    { 0, 0 },
    { 0xefb79da3, 33 },
    { 0, 0 },
    { 0xa6579da5, 61 },
    { 0xfd79da5, 84 },
    { 0x21e79da7, 93 },
    { 0, 0 },
    { 0x90079da9, 95 },
    { 0x20179daa, 34 },
    { 0x1eb79da9, 61 },
    { 0, 0 },
    { 0x54f79dad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26579db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33179db3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb79dbe, 99 },
    { 0, 0 },
    { 0x1d779dc0, 55 },
    { 0, 0 },
    { 0x7a779dc2, 85 },
    { 0, 0 },
    { 0xa1479dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f79dc7, 55 },
    { 0x56779dc8, 95 },
    { 0x98579dc9, 77 },
    { 0x5aa79dc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x35379dcd, 43 },
    { 0x59079dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b579dd2, 33 },
    { 0xe5679dd3, 34 },
    { 0, 0 },
    { 0xe2c79dd5, 91 },
    { 0x23879dd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e79dd9, 46 },
    { 0x12179dda, 23 },
    { 0x5d679ddb, 61 },
    { 0x8a279ddb, 90 },
    { 0, 0 },
    { 0x29679dde, 23 },
    { 0xb6779dde, 61 },
    { 0x5b779de0, 34 },
    { 0xd9179de0, 23 },
    { 0x8fb79dde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c79de5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee79dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f79def, 93 },
    { 0x3f179df0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2079df7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada79dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd79e00, 87 },
    { 0, 0 },
    { 0xc5479e02, 91 },
    { 0, 0 },
    { 0x77379e04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4579e07, 87 },
    { 0, 0 },
    { 0xd1a79e09, 93 },
    { 0, 0 },
    { 0xe879e0b, 76 },
    { 0x28e79e0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4779e0f, 17 },
    { 0x75e79e0f, 100 },
    { 0, 0 },
    { 0x91779e12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f79e16, 33 },
    { 0, 0 },
    { 0x21979e18, 33 },
    { 0x85579e19, 81 },
    { 0xdcd79e1a, 84 },
    { 0xc0879e1b, 87 },
    { 0xdeb79e1c, 95 },
    { 0x54279e1b, 91 },
    { 0x379e1e, 81 },
    { 0x4de79e1c, 43 },
    { 0xbe879e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a179e23, 23 },
    { 0, 0 },
    { 0x44579e25, 95 },
    { 0xc3379e26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37279e32, 77 },
    { 0xcba79e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9779e37, 95 },
    { 0x3fd79e38, 68 },
    { 0x4a779e39, 34 },
    { 0xfa779e3a, 98 },
    { 0xc7079e38, 55 },
    { 0, 0 },
    { 0x41479e3d, 33 },
    { 0xe0a79e3e, 55 },
    { 0xa4c79e3f, 93 },
    { 0x179e3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d79e49, 33 },
    { 0x94379e4a, 61 },
    { 0xed279e4b, 23 },
    { 0xfc279e49, 55 },
    { 0x37e79e49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa279e51, 23 },
    { 0xd1679e52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4879e55, 33 },
    { 0x51179e56, 23 },
    { 0, 0 },
    { 0x6179e58, 90 },
    { 0x13879e58, 88 },
    { 0x5e79e5a, 75 },
    { 0x88379e5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c79e5e, 52 },
    { 0, 0 },
    { 0xd8579e60, 91 },
    { 0, 0 },
    { 0x1d379e62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4079e68, 67 },
    { 0, 0 },
    { 0xc3579e6a, 23 },
    { 0xa1a79e6b, 55 },
    { 0x5ec79e6c, 95 },
    { 0xed79e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c79e70, 99 },
    { 0xd4f79e71, 69 },
    { 0xddd79e72, 23 },
    { 0x62179e70, 95 },
    { 0x2b679e70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea79e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4179e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed79e81, 61 },
    { 0, 0 },
    { 0xcef79e83, 98 },
    { 0, 0 },
    { 0x91679e85, 68 },
    { 0x6eb79e86, 61 },
    { 0x2bc79e86, 23 },
    { 0x82d79e86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc79e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71279e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda879e96, 33 },
    { 0, 0 },
    { 0x8d079e98, 23 },
    { 0xfeb79e99, 61 },
    { 0xc8279e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41479ea0, 34 },
    { 0x5c479ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0879ea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed79eab, 61 },
    { 0xcec79eac, 66 },
    { 0, 0 },
    { 0x24579eae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa979eb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc79eb6, 90 },
    { 0, 0 },
    { 0x64479eb8, 52 },
    { 0x99979eb8, 100 },
    { 0, 0 },
    { 0x30179ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1379ebf, 92 },
    { 0x13979ebf, 78 },
    { 0xa1c79ec1, 90 },
    { 0x42479ec2, 65 },
    { 0x5fb79ec2, 61 },
    { 0xd5279ec1, 33 },
    { 0xc6d79ec5, 55 },
    { 0x3279ec5, 78 },
    { 0x99b79ec4, 93 },
    { 0, 0 },
    { 0xd7379ec9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e79ece, 46 },
    { 0xfef79ece, 55 },
    { 0xe1679ecf, 61 },
    { 0x51079ed0, 23 },
    { 0x41f79ed2, 90 },
    { 0xda279ed2, 33 },
    { 0, 0 },
    { 0xd0779ed5, 78 },
    { 0xbfd79ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa79ed9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82179edd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3979ee2, 27 },
    { 0, 0 },
    { 0x97e79ee4, 98 },
    { 0x82b79ee5, 65 },
    { 0x37e79ee5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76579ee9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec79eee, 61 },
    { 0x13679eee, 90 },
    { 0x4cc79eef, 23 },
    { 0, 0 },
    { 0xd7679ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb279ef5, 81 },
    { 0xe6979ef5, 55 },
    { 0, 0 },
    { 0x48679ef8, 33 },
    { 0x78879ef9, 91 },
    { 0x5479efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6479efe, 91 },
    { 0, 0 },
    { 0x95979f00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c79f03, 76 },
    { 0, 0 },
    { 0xaab79f05, 23 },
    { 0, 0 },
    { 0x3e579f07, 90 },
    { 0x60b79f07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9279f0b, 33 },
    { 0x2de79f0b, 36 },
    { 0xdfe79f0d, 28 },
    { 0xb2479f0d, 76 },
    { 0, 0 },
    { 0xef379f10, 90 },
    { 0xc4979f11, 66 },
    { 0x88d79f10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21679f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b79f1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8779f1d, 80 },
    { 0x6d579f1e, 53 },
    { 0, 0 },
    { 0x41a79f20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c79f25, 88 },
    { 0xe4b79f25, 90 },
    { 0xf4179f27, 66 },
    { 0xaaa79f27, 67 },
    { 0xe0679f29, 65 },
    { 0x4179f29, 87 },
    { 0x77879f2b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b79f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed79f33, 99 },
    { 0xf2c79f34, 61 },
    { 0x60379f34, 95 },
    { 0x8f579f36, 98 },
    { 0x4f279f37, 90 },
    { 0, 0 },
    { 0xc4979f39, 55 },
    { 0x5f979f39, 61 },
    { 0x49179f39, 75 },
    { 0xf979f3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e579f3f, 90 },
    { 0, 0 },
    { 0x58e79f41, 76 },
    { 0, 0 },
    { 0xa2779f43, 90 },
    { 0xf4179f44, 76 },
    { 0xbc679f45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd379f4b, 78 },
    { 0, 0 },
    { 0x2ab79f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c79f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0179f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97679f59, 43 },
    { 0xb7579f5a, 68 },
    { 0xb3879f5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1279f5f, 76 },
    { 0xdba79f5f, 33 },
    { 0x27279f60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e79f67, 46 },
    { 0, 0 },
    { 0x89379f69, 46 },
    { 0x13079f6a, 99 },
    { 0xd0079f6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7079f6f, 23 },
    { 0xdc279f70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c779f74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde79f78, 33 },
    { 0x7ba79f79, 90 },
    { 0x4ed79f7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a79f80, 93 },
    { 0, 0 },
    { 0xe5d79f82, 46 },
    { 0, 0 },
    { 0xbfc79f84, 34 },
    { 0xaa979f85, 33 },
    { 0x63479f86, 76 },
    { 0xd9179f85, 76 },
    { 0x85979f88, 91 },
    { 0, 0 },
    { 0x75f79f8a, 33 },
    { 0xfaa79f8a, 76 },
    { 0x33679f8a, 95 },
    { 0x2e979f8d, 99 },
    { 0x8df79f8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73579f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb479f9a, 78 },
    { 0x16779f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8479f9e, 23 },
    { 0x37279f9f, 61 },
    { 0, 0 },
    { 0xc6e79fa1, 17 },
    { 0, 0 },
    { 0x4f179fa3, 78 },
    { 0, 0 },
    { 0x7c079fa5, 88 },
    { 0, 0 },
    { 0x80c79fa7, 95 },
    { 0x97d79fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91279fac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba79faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18679fb8, 90 },
    { 0x83f79fb9, 61 },
    { 0x8ac79fba, 93 },
    { 0xb2879fbb, 95 },
    { 0, 0 },
    { 0x5f579fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c279fc5, 61 },
    { 0x71a79fc6, 61 },
    { 0x12d79fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63179fcb, 87 },
    { 0, 0 },
    { 0xe0d79fcd, 66 },
    { 0xf5f79fce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc79fe1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e79fe8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2379feb, 78 },
    { 0x4e079feb, 33 },
    { 0x2d179fed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63579ff3, 61 },
    { 0xa2179ff4, 43 },
    { 0, 0 },
    { 0x92679ff6, 95 },
    { 0x11879ff7, 61 },
    { 0xe4579ff8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf579ffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4217a000, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe117a006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697a009, 69 },
    { 0x9477a00a, 76 },
    { 0, 0 },
    { 0x8be7a00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707a013, 98 },
    { 0xc1d7a014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e7a019, 81 },
    { 0x7957a019, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297a020, 68 },
    { 0x8d07a021, 90 },
    { 0, 0 },
    { 0xff37a023, 43 },
    { 0xfd57a023, 95 },
    { 0, 0 },
    { 0x807a026, 90 },
    { 0xc607a027, 95 },
    { 0, 0 },
    { 0x5817a029, 43 },
    { 0, 0 },
    { 0xb647a02b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c57a02e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7a031, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7a034, 34 },
    { 0x2f07a035, 23 },
    { 0x557a036, 61 },
    { 0x91d7a037, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1737a03b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a042, 34 },
    { 0, 0 },
    { 0x5f37a044, 87 },
    { 0xa4a7a044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae27a04c, 77 },
    { 0x13e7a04c, 98 },
    { 0xa707a04e, 52 },
    { 0xb277a04f, 51 },
    { 0xd647a050, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e7a057, 95 },
    { 0, 0 },
    { 0x7ba7a059, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787a05c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b47a05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f17a065, 61 },
    { 0xfe37a066, 61 },
    { 0x5447a067, 93 },
    { 0xdb57a067, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57a06b, 99 },
    { 0xe747a06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d7a076, 95 },
    { 0x1e77a077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7147a07a, 64 },
    { 0, 0 },
    { 0x5487a07c, 76 },
    { 0, 0 },
    { 0x9787a07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b77a081, 87 },
    { 0xe7a082, 67 },
    { 0x2ee7a083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187a086, 90 },
    { 0x42d7a087, 43 },
    { 0xc27a086, 23 },
    { 0, 0 },
    { 0x9f97a08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147a08f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f7a092, 33 },
    { 0x93f7a093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a7a098, 23 },
    { 0xaea7a098, 83 },
    { 0, 0 },
    { 0xafb7a09b, 61 },
    { 0xb047a09c, 93 },
    { 0x5467a09c, 55 },
    { 0x5477a09e, 23 },
    { 0xbff7a09d, 23 },
    { 0xaf17a0a0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7a0a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17a0b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa7a0b4, 55 },
    { 0x6707a0b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a0b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1437a0bb, 55 },
    { 0, 0 },
    { 0xdfa7a0bd, 33 },
    { 0, 0 },
    { 0xebc7a0bf, 33 },
    { 0x3607a0c0, 66 },
    { 0xdfb7a0c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f27a0c7, 95 },
    { 0xe357a0c8, 61 },
    { 0x27c7a0c9, 33 },
    { 0x1557a0c7, 98 },
    { 0, 0 },
    { 0xbf07a0cc, 33 },
    { 0xbf77a0cc, 84 },
    { 0, 0 },
    { 0x9ec7a0cf, 61 },
    { 0x50e7a0d0, 61 },
    { 0, 0 },
    { 0xa0b7a0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5177a0df, 80 },
    { 0, 0 },
    { 0x84b7a0e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787a0e5, 23 },
    { 0xda07a0e6, 99 },
    { 0xd377a0e6, 76 },
    { 0xafb7a0e5, 33 },
    { 0, 0 },
    { 0x1be7a0ea, 33 },
    { 0xad37a0eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a7a0ee, 33 },
    { 0x5567a0ef, 75 },
    { 0, 0 },
    { 0x5bb7a0f1, 34 },
    { 0xf497a0f2, 87 },
    { 0, 0 },
    { 0xa3b7a0f4, 85 },
    { 0, 0 },
    { 0xd877a0f6, 43 },
    { 0x1837a0f7, 99 },
    { 0, 0 },
    { 0xdde7a0f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7a0fc, 36 },
    { 0x5097a0fc, 61 },
    { 0x76e7a0fc, 90 },
    { 0xe427a0fc, 22 },
    { 0x5c7a0fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47a103, 55 },
    { 0x4db7a104, 76 },
    { 0xeb57a105, 100 },
    { 0x4bb7a105, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xda17a109, 33 },
    { 0, 0 },
    { 0x6567a10b, 61 },
    { 0x4d7a10c, 87 },
    { 0x7fa7a10c, 90 },
    { 0x22e7a10d, 90 },
    { 0xc4f7a10f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde27a115, 88 },
    { 0x5a57a115, 34 },
    { 0xa97a117, 38 },
    { 0xc857a115, 84 },
    { 0x1ec7a117, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7747a11d, 95 },
    { 0x4467a11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9367a128, 46 },
    { 0, 0 },
    { 0x6c57a12a, 65 },
    { 0x2cd7a12b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a12f, 76 },
    { 0x9057a130, 33 },
    { 0x6a47a131, 77 },
    { 0xfb7a132, 33 },
    { 0x5457a132, 90 },
    { 0xae07a130, 85 },
    { 0xe217a133, 61 },
    { 0x2867a130, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c97a139, 76 },
    { 0, 0 },
    { 0x8fd7a13b, 98 },
    { 0, 0 },
    { 0x8ee7a13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7a142, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d7a145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e7a14c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd247a152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd527a158, 88 },
    { 0x16b7a159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7a15c, 95 },
    { 0xb6a7a15d, 33 },
    { 0, 0 },
    { 0x9eb7a15f, 61 },
    { 0x9227a15f, 98 },
    { 0x2f17a161, 66 },
    { 0, 0 },
    { 0x1b37a163, 85 },
    { 0xd1a7a164, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e7a171, 90 },
    { 0x6087a172, 76 },
    { 0xd67a173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea97a178, 33 },
    { 0x7067a179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b87a17c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d7a181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa57a184, 61 },
    { 0xb7e7a185, 23 },
    { 0, 0 },
    { 0xeb77a187, 61 },
    { 0x5757a188, 33 },
    { 0xcce7a189, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f67a18d, 34 },
    { 0, 0 },
    { 0xe127a18f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7a197, 61 },
    { 0x1de7a198, 81 },
    { 0x9607a199, 90 },
    { 0x117a19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa857a19d, 94 },
    { 0x44a7a19e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d77a1ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd7a1b0, 90 },
    { 0, 0 },
    { 0xc067a1b2, 67 },
    { 0xc2b7a1b3, 95 },
    { 0xb267a1b4, 66 },
    { 0x3dd7a1b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437a1b8, 61 },
    { 0, 0 },
    { 0xfdb7a1ba, 90 },
    { 0, 0 },
    { 0x2057a1bc, 75 },
    { 0x60c7a1bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07a1c0, 36 },
    { 0x6a87a1c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3007a1c4, 61 },
    { 0xcd07a1c4, 90 },
    { 0xf947a1c5, 91 },
    { 0x2f07a1c7, 61 },
    { 0xa567a1c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177a1cd, 55 },
    { 0, 0 },
    { 0xed47a1cf, 23 },
    { 0, 0 },
    { 0xe117a1d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d7a1d8, 84 },
    { 0, 0 },
    { 0xe997a1da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa67a1dd, 98 },
    { 0, 0 },
    { 0xf1f7a1df, 78 },
    { 0, 0 },
    { 0xea7a1e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a1e7, 33 },
    { 0, 0 },
    { 0xeaf7a1e9, 61 },
    { 0xd3a7a1ea, 78 },
    { 0xfcc7a1e9, 52 },
    { 0, 0 },
    { 0x7ef7a1ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba7a1f0, 66 },
    { 0x28a7a1f0, 98 },
    { 0, 0 },
    { 0xe707a1f3, 33 },
    { 0xd677a1f4, 98 },
    { 0xb187a1f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf497a1f9, 98 },
    { 0x2707a1fa, 95 },
    { 0xdfd7a1f9, 80 },
    { 0, 0 },
    { 0x5b47a1fd, 90 },
    { 0x8eb7a1fd, 100 },
    { 0xc2e7a1ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267a202, 55 },
    { 0, 0 },
    { 0xaa77a204, 66 },
    { 0xbf7a204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea7a208, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557a20b, 61 },
    { 0x9c87a20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4697a20f, 75 },
    { 0, 0 },
    { 0x4037a211, 90 },
    { 0xa697a211, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b37a219, 61 },
    { 0xf207a21a, 90 },
    { 0x9be7a21b, 95 },
    { 0xb5b7a21c, 90 },
    { 0x807a21d, 55 },
    { 0xcda7a219, 55 },
    { 0, 0 },
    { 0x4a17a220, 99 },
    { 0, 0 },
    { 0x9367a222, 90 },
    { 0x40a7a223, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a7a226, 17 },
    { 0xf067a226, 55 },
    { 0xe967a226, 90 },
    { 0x6ee7a229, 33 },
    { 0xf1d7a22a, 68 },
    { 0xb1b7a22b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff7a22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d7a235, 33 },
    { 0x3437a235, 98 },
    { 0x9f87a237, 78 },
    { 0xf457a237, 84 },
    { 0xcd77a237, 90 },
    { 0x4807a23a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7a23d, 95 },
    { 0, 0 },
    { 0xae67a23f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f07a242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff7a246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d7a249, 90 },
    { 0, 0 },
    { 0x84e7a24b, 77 },
    { 0xd987a24b, 55 },
    { 0x9257a24c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277a252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617a25f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd017a263, 33 },
    { 0, 0 },
    { 0x6a7a265, 76 },
    { 0x9bc7a266, 95 },
    { 0xa577a267, 90 },
    { 0, 0 },
    { 0xb137a269, 87 },
    { 0xd597a26a, 61 },
    { 0x5087a26b, 100 },
    { 0x1dd7a26a, 55 },
    { 0, 0 },
    { 0x6b7a26e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197a272, 28 },
    { 0xa3b7a272, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2127a278, 8 },
    { 0, 0 },
    { 0x78e7a27a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7887a27d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d7a282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe307a28b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f37a292, 96 },
    { 0x47b7a293, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2837a297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d7a29b, 33 },
    { 0xd267a29b, 78 },
    { 0, 0 },
    { 0x7ff7a29e, 78 },
    { 0, 0 },
    { 0x9667a2a0, 90 },
    { 0, 0 },
    { 0x8fd7a2a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577a2a5, 90 },
    { 0xfea7a2a6, 90 },
    { 0x3a17a2a7, 90 },
    { 0xf0f7a2a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad7a2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4837a2b4, 33 },
    { 0x9327a2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7a2be, 98 },
    { 0xdce7a2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac7a2c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4057a2c7, 61 },
    { 0x5167a2c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87a2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc57a2d9, 61 },
    { 0x4fe7a2da, 98 },
    { 0, 0 },
    { 0x1d17a2dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd087a2df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7a2e2, 33 },
    { 0x9ad7a2e3, 98 },
    { 0x427a2e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707a2e8, 33 },
    { 0xc537a2e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7a2ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237a2f4, 23 },
    { 0x75c7a2f4, 61 },
    { 0, 0 },
    { 0x2a77a2f7, 70 },
    { 0x3eb7a2f8, 23 },
    { 0xb7a7a2f9, 85 },
    { 0, 0 },
    { 0x9487a2fb, 99 },
    { 0x6d47a2fc, 33 },
    { 0xe447a2fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7a304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a309, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c97a314, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea7a318, 23 },
    { 0x4f7a319, 46 },
    { 0, 0 },
    { 0x52e7a31b, 61 },
    { 0x4837a31c, 65 },
    { 0x3667a31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e7a322, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f7a325, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e7a328, 56 },
    { 0, 0 },
    { 0x6fa7a32a, 100 },
    { 0x5af7a32a, 33 },
    { 0x8697a32c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b7a331, 95 },
    { 0x3427a331, 91 },
    { 0xd4c7a333, 88 },
    { 0xf1b7a334, 95 },
    { 0x7fe7a335, 90 },
    { 0xc37a336, 95 },
    { 0xb487a335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb877a33a, 33 },
    { 0xc5e7a33b, 23 },
    { 0, 0 },
    { 0x7bf7a33d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda7a341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e47a347, 55 },
    { 0x5e47a348, 34 },
    { 0xd737a349, 61 },
    { 0x5317a348, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2337a34f, 90 },
    { 0xf2a7a34f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f7a358, 23 },
    { 0, 0 },
    { 0xad77a35a, 33 },
    { 0, 0 },
    { 0x3ec7a35c, 78 },
    { 0xad47a35c, 95 },
    { 0x6e37a35e, 98 },
    { 0xe677a35f, 78 },
    { 0x6707a35e, 66 },
    { 0x4037a361, 87 },
    { 0, 0 },
    { 0x78c7a363, 66 },
    { 0x20b7a364, 23 },
    { 0xda87a365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe937a368, 33 },
    { 0, 0 },
    { 0xbb57a36a, 23 },
    { 0x1237a36b, 90 },
    { 0x9f27a36c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3067a370, 76 },
    { 0xf2b7a371, 55 },
    { 0, 0 },
    { 0x6467a373, 90 },
    { 0xe847a374, 91 },
    { 0xeb27a374, 33 },
    { 0x7417a376, 81 },
    { 0x2ef7a376, 95 },
    { 0x1e27a378, 90 },
    { 0x3717a378, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf7a37e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7a381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd37a384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe37a387, 78 },
    { 0xaba7a388, 76 },
    { 0x3997a389, 98 },
    { 0x5a07a389, 55 },
    { 0x5187a38b, 85 },
    { 0x7e97a387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc57a38f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c37a395, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37a398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5267a39b, 33 },
    { 0, 0 },
    { 0x86f7a39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7a3a0, 90 },
    { 0, 0 },
    { 0x4bb7a3a2, 93 },
    { 0x87d7a3a3, 23 },
    { 0xd1e7a3a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6407a3aa, 98 },
    { 0, 0 },
    { 0xd457a3ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8187a3b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a97a3b6, 93 },
    { 0, 0 },
    { 0x7b97a3b8, 43 },
    { 0x4257a3b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c7a3bd, 61 },
    { 0, 0 },
    { 0x5207a3bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea7a3c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e7a3cf, 33 },
    { 0xeeb7a3d0, 87 },
    { 0, 0 },
    { 0x2e67a3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce7a3d5, 61 },
    { 0x9aa7a3d6, 66 },
    { 0xd307a3d6, 98 },
    { 0xb677a3d8, 67 },
    { 0, 0 },
    { 0x9bd7a3da, 90 },
    { 0, 0 },
    { 0xbca7a3dc, 55 },
    { 0x5cf7a3dd, 95 },
    { 0, 0 },
    { 0xbd17a3df, 34 },
    { 0x95e7a3e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb7a3e7, 98 },
    { 0xa187a3e8, 76 },
    { 0, 0 },
    { 0x7377a3ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa7a3f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c7a3f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb307a3fe, 93 },
    { 0x1087a3ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907a403, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a17a406, 55 },
    { 0x5de7a407, 48 },
    { 0x6867a408, 33 },
    { 0xeb27a408, 12 },
    { 0x1d27a40a, 90 },
    { 0xc4b7a40b, 43 },
    { 0, 0 },
    { 0xb677a40d, 68 },
    { 0xc577a40e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd807a412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e7a417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb77a41a, 90 },
    { 0x2d17a41b, 61 },
    { 0x5fb7a41a, 38 },
    { 0x5bd7a41c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7a420, 90 },
    { 0, 0 },
    { 0x3fa7a422, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8777a426, 34 },
    { 0xf107a427, 98 },
    { 0, 0 },
    { 0x2007a429, 46 },
    { 0x7a97a429, 33 },
    { 0x3b67a42a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3777a42e, 66 },
    { 0x86e7a42f, 52 },
    { 0, 0 },
    { 0x8b37a431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb227a435, 90 },
    { 0x26a7a435, 90 },
    { 0, 0 },
    { 0x4e37a438, 33 },
    { 0xc757a439, 33 },
    { 0xd027a43a, 33 },
    { 0x4bc7a43a, 61 },
    { 0, 0 },
    { 0x1227a43d, 55 },
    { 0xdfb7a43e, 90 },
    { 0xb867a43e, 23 },
    { 0xc097a440, 61 },
    { 0, 0 },
    { 0xb217a442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97a44a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6207a44e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2577a451, 43 },
    { 0x9c77a452, 95 },
    { 0xbb67a452, 43 },
    { 0, 0 },
    { 0x3bd7a455, 23 },
    { 0x9287a456, 85 },
    { 0, 0 },
    { 0x1fd7a458, 61 },
    { 0x1497a459, 95 },
    { 0x5887a458, 95 },
    { 0, 0 },
    { 0x3537a45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a07a462, 83 },
    { 0x72a7a463, 43 },
    { 0xc287a464, 80 },
    { 0xa937a465, 76 },
    { 0, 0 },
    { 0x7427a467, 90 },
    { 0xdc67a467, 90 },
    { 0xda57a469, 52 },
    { 0x2307a469, 43 },
    { 0x5477a46b, 55 },
    { 0xab37a468, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047a473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557a477, 61 },
    { 0xbed7a478, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b7a47b, 90 },
    { 0x5c17a47c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37a484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057a487, 68 },
    { 0xccc7a487, 95 },
    { 0xf67a489, 50 },
    { 0, 0 },
    { 0xd037a48b, 52 },
    { 0x2f7a48c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a37a48f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157a493, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7a497, 55 },
    { 0, 0 },
    { 0x7057a499, 33 },
    { 0x267a49a, 40 },
    { 0xe257a499, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967a4a0, 90 },
    { 0x9757a4a1, 90 },
    { 0x2bd7a4a2, 55 },
    { 0x16b7a4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7a4a7, 43 },
    { 0xaab7a4a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407a4ac, 66 },
    { 0, 0 },
    { 0xb787a4ae, 68 },
    { 0x7ad7a4af, 66 },
    { 0xd77a4ae, 95 },
    { 0x46e7a4ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7547a4b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb967a4ba, 56 },
    { 0, 0 },
    { 0x7317a4bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc7a4c2, 90 },
    { 0, 0 },
    { 0x94f7a4c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb117a4c7, 46 },
    { 0, 0 },
    { 0xdbb7a4c9, 98 },
    { 0, 0 },
    { 0x1507a4cb, 91 },
    { 0x837a4cb, 61 },
    { 0xc7a7a4cd, 46 },
    { 0, 0 },
    { 0x67e7a4cf, 80 },
    { 0, 0 },
    { 0x83d7a4d1, 43 },
    { 0x79f7a4d2, 55 },
    { 0x6cb7a4d2, 46 },
    { 0, 0 },
    { 0x23a7a4d5, 95 },
    { 0x68a7a4d5, 23 },
    { 0x40b7a4d7, 33 },
    { 0xfc17a4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1007a4db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a7a4de, 77 },
    { 0, 0 },
    { 0xba37a4e0, 33 },
    { 0x17a4e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb7a4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f87a4f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf077a4f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x717a4fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab7a4fe, 90 },
    { 0x6a87a4fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe287a504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8177a509, 81 },
    { 0, 0 },
    { 0x7e67a50b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda7a510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287a519, 33 },
    { 0xf5e7a519, 33 },
    { 0x1e7a51b, 98 },
    { 0x987a51b, 100 },
    { 0xc9c7a51c, 61 },
    { 0x7a51e, 33 },
    { 0xb2b7a51a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8807a522, 85 },
    { 0x1b97a522, 77 },
    { 0x5ee7a523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4247a529, 91 },
    { 0, 0 },
    { 0x147a52b, 67 },
    { 0, 0 },
    { 0x1de7a52d, 90 },
    { 0x3e7a52d, 55 },
    { 0xe547a52f, 98 },
    { 0xea17a530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7a535, 53 },
    { 0x8217a535, 43 },
    { 0, 0 },
    { 0x1517a538, 76 },
    { 0xd3d7a539, 44 },
    { 0x45b7a539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db7a53d, 43 },
    { 0, 0 },
    { 0xa607a53f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7a549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547a553, 61 },
    { 0xffc7a554, 98 },
    { 0x29e7a555, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3167a559, 61 },
    { 0xdcb7a55a, 93 },
    { 0x7397a55b, 84 },
    { 0x8297a55a, 90 },
    { 0, 0 },
    { 0x6167a55e, 46 },
    { 0, 0 },
    { 0x8ef7a560, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee7a563, 90 },
    { 0, 0 },
    { 0x7cd7a565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b7a56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9327a56e, 85 },
    { 0x6a77a56f, 76 },
    { 0xe87a570, 61 },
    { 0xc8d7a570, 61 },
    { 0xf4e7a56f, 61 },
    { 0x2a07a571, 34 },
    { 0x6987a56e, 98 },
    { 0x2377a575, 76 },
    { 0xd097a575, 55 },
    { 0xd57a577, 33 },
    { 0x3847a575, 61 },
    { 0xaa77a578, 26 },
    { 0x97d7a57a, 75 },
    { 0x2bc7a57b, 98 },
    { 0xc437a571, 43 },
    { 0, 0 },
    { 0x1fc7a57e, 67 },
    { 0, 0 },
    { 0xc497a580, 55 },
    { 0xadf7a580, 33 },
    { 0xd6a7a582, 95 },
    { 0xdbc7a583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f7a586, 43 },
    { 0, 0 },
    { 0xf7b7a588, 33 },
    { 0, 0 },
    { 0x4ed7a58a, 66 },
    { 0x8eb7a58b, 98 },
    { 0, 0 },
    { 0xc737a58d, 95 },
    { 0, 0 },
    { 0x6877a58f, 85 },
    { 0, 0 },
    { 0xe147a591, 52 },
    { 0x1eb7a592, 76 },
    { 0xf727a591, 95 },
    { 0x6757a594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c87a598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e27a59b, 90 },
    { 0, 0 },
    { 0x9437a59d, 61 },
    { 0x3707a59e, 90 },
    { 0xe17a59f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f07a5a3, 88 },
    { 0x357a5a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2667a5aa, 95 },
    { 0, 0 },
    { 0x9bc7a5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7a5b2, 61 },
    { 0, 0 },
    { 0xe887a5b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a5b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5007a5ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8337a5be, 55 },
    { 0, 0 },
    { 0xa2a7a5c0, 90 },
    { 0xbe47a5c1, 90 },
    { 0x41b7a5c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5837a5c6, 85 },
    { 0x28b7a5c6, 95 },
    { 0xdf87a5c8, 68 },
    { 0xfa87a5c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237a5cc, 8 },
    { 0x6557a5cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd47a5d0, 33 },
    { 0, 0 },
    { 0xddf7a5d2, 90 },
    { 0x7f07a5d3, 78 },
    { 0x3c87a5d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b37a5da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac07a5e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca37a5eb, 98 },
    { 0x9647a5ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa227a5f1, 52 },
    { 0xb8b7a5f2, 95 },
    { 0x73d7a5f3, 95 },
    { 0x8587a5f4, 90 },
    { 0x2897a5f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb7a5fe, 23 },
    { 0xb587a5ff, 69 },
    { 0, 0 },
    { 0x6d57a601, 90 },
    { 0x33f7a601, 100 },
    { 0x3e57a601, 80 },
    { 0, 0 },
    { 0x8917a605, 61 },
    { 0xc17a605, 95 },
    { 0xaa77a607, 90 },
    { 0x42e7a607, 52 },
    { 0x51f7a609, 95 },
    { 0x94c7a609, 61 },
    { 0x8527a609, 23 },
    { 0x1a7a60b, 34 },
    { 0xf4a7a605, 100 },
    { 0xce87a606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7a611, 43 },
    { 0xe4b7a612, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e27a616, 95 },
    { 0x57f7a616, 85 },
    { 0xb877a618, 100 },
    { 0xb6b7a619, 34 },
    { 0x6f17a618, 55 },
    { 0x53e7a61b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a7a61f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf7a629, 33 },
    { 0xf97a629, 61 },
    { 0xb7f7a62b, 90 },
    { 0xf117a62a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87a634, 76 },
    { 0, 0 },
    { 0x6517a636, 33 },
    { 0, 0 },
    { 0x9fc7a638, 43 },
    { 0, 0 },
    { 0x7007a63a, 90 },
    { 0, 0 },
    { 0xcf07a63c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee7a63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1687a646, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c37a64e, 98 },
    { 0x8e87a64e, 95 },
    { 0x2c17a650, 90 },
    { 0x1547a651, 61 },
    { 0, 0 },
    { 0x5b7a653, 84 },
    { 0xc097a654, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe447a658, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee7a65b, 46 },
    { 0xc8b7a65b, 23 },
    { 0x6567a65d, 90 },
    { 0x7597a65d, 98 },
    { 0xc5e7a65d, 55 },
    { 0xb827a660, 33 },
    { 0, 0 },
    { 0x4a87a662, 33 },
    { 0x5cd7a663, 61 },
    { 0x50e7a664, 80 },
    { 0x68e7a662, 33 },
    { 0, 0 },
    { 0xe817a667, 98 },
    { 0x1607a667, 95 },
    { 0, 0 },
    { 0xb4d7a66a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b7a66d, 98 },
    { 0x3377a66d, 59 },
    { 0xd647a66f, 33 },
    { 0x9c37a66f, 81 },
    { 0xf567a671, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907a674, 61 },
    { 0xcdb7a675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc027a679, 87 },
    { 0x9007a67a, 43 },
    { 0xfe87a679, 100 },
    { 0x2b97a679, 33 },
    { 0x1627a67d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a7a681, 75 },
    { 0xcaf7a682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2037a686, 90 },
    { 0xe497a686, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda27a68b, 78 },
    { 0, 0 },
    { 0xa157a68d, 90 },
    { 0, 0 },
    { 0x5807a68f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7a698, 33 },
    { 0x95f7a699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee7a69d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2697a6a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17a6a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd7a6a7, 33 },
    { 0xf497a6a8, 76 },
    { 0x8e67a6a8, 48 },
    { 0, 0 },
    { 0x19f7a6ab, 33 },
    { 0xdcf7a6ab, 46 },
    { 0x5357a6ad, 68 },
    { 0x47b7a6ad, 96 },
    { 0xdce7a6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a6b7, 90 },
    { 0, 0 },
    { 0x4e67a6b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77a6c7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b7a6cb, 33 },
    { 0x8c17a6cc, 91 },
    { 0x8fe7a6cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a6d4, 33 },
    { 0, 0 },
    { 0x6e77a6d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57a6d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5997a6e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657a6e5, 76 },
    { 0xabb7a6e6, 33 },
    { 0x1137a6e6, 95 },
    { 0x5fd7a6e8, 33 },
    { 0xd757a6e9, 90 },
    { 0xe827a6e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9607a6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d7a6fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc367a700, 95 },
    { 0x5c77a701, 90 },
    { 0xe1c7a702, 90 },
    { 0xa7f7a703, 43 },
    { 0xe617a704, 99 },
    { 0xe437a705, 95 },
    { 0xb917a706, 77 },
    { 0xab7a705, 55 },
    { 0x6637a706, 95 },
    { 0x3137a707, 99 },
    { 0x4cf7a70a, 95 },
    { 0x9e27a70a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557a713, 33 },
    { 0, 0 },
    { 0x22b7a715, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a7a722, 76 },
    { 0x8f17a723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7907a726, 98 },
    { 0, 0 },
    { 0x7f27a728, 90 },
    { 0x67b7a729, 90 },
    { 0x7207a729, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc477a72d, 91 },
    { 0xf3a7a72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657a737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a7a73a, 33 },
    { 0, 0 },
    { 0x1c57a73c, 92 },
    { 0xf7f7a73d, 90 },
    { 0x957a73e, 33 },
    { 0, 0 },
    { 0x8647a740, 55 },
    { 0x7e07a741, 78 },
    { 0, 0 },
    { 0x7177a743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e7a74a, 90 },
    { 0x1dd7a74b, 76 },
    { 0x927a74a, 67 },
    { 0x4e27a74d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb337a751, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5277a757, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047a75d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c7a762, 91 },
    { 0x9707a763, 76 },
    { 0x79b7a764, 43 },
    { 0x4ed7a762, 61 },
    { 0x91a7a766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307a76a, 34 },
    { 0xbf97a76b, 67 },
    { 0xaa37a76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57a76f, 90 },
    { 0x5c67a770, 81 },
    { 0xd707a771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7907a774, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf17a777, 100 },
    { 0xf497a778, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567a77d, 95 },
    { 0x8527a77e, 90 },
    { 0x2c67a77d, 43 },
    { 0x1457a780, 76 },
    { 0x12d7a781, 61 },
    { 0, 0 },
    { 0x36d7a783, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1747a786, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a7a78b, 34 },
    { 0x2f77a78c, 61 },
    { 0xd9c7a78c, 23 },
    { 0xd197a78e, 49 },
    { 0x47f7a78d, 98 },
    { 0x8a77a790, 38 },
    { 0, 0 },
    { 0x2eb7a792, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597a795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee7a798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4977a79c, 61 },
    { 0xaa87a79d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b47a7a0, 84 },
    { 0xd2b7a7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057a7a5, 67 },
    { 0, 0 },
    { 0x8177a7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f7a7aa, 23 },
    { 0xbb27a7ab, 61 },
    { 0x687a7ac, 76 },
    { 0xab37a7aa, 95 },
    { 0xce97a7aa, 61 },
    { 0x56b7a7ab, 46 },
    { 0x89a7a7b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa7a7bc, 90 },
    { 0xc067a7bd, 81 },
    { 0x1b97a7bc, 61 },
    { 0xf067a7be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf697a7c3, 90 },
    { 0xf967a7c4, 78 },
    { 0, 0 },
    { 0xa5c7a7c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27a7c9, 66 },
    { 0x5a27a7c9, 98 },
    { 0x46c7a7ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf167a7cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67a7d8, 84 },
    { 0, 0 },
    { 0xdca7a7da, 95 },
    { 0x6327a7da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c7a7e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7a7ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f67a7ee, 90 },
    { 0, 0 },
    { 0xb647a7f0, 85 },
    { 0, 0 },
    { 0x29b7a7f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa7a7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d87a805, 23 },
    { 0x7d37a806, 80 },
    { 0x9677a807, 87 },
    { 0x47d7a808, 95 },
    { 0x4f17a808, 91 },
    { 0x987a80a, 90 },
    { 0xdc7a806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7a811, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307a816, 93 },
    { 0x24e7a817, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd87a81e, 34 },
    { 0x6107a81e, 34 },
    { 0xfe57a81e, 53 },
    { 0x4047a821, 98 },
    { 0x5c77a822, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7a826, 95 },
    { 0xc9e7a826, 52 },
    { 0, 0 },
    { 0xeac7a829, 90 },
    { 0, 0 },
    { 0xef47a82b, 100 },
    { 0xbb47a82c, 90 },
    { 0, 0 },
    { 0x5e97a82e, 98 },
    { 0x9ef7a82f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707a832, 95 },
    { 0xcca7a833, 52 },
    { 0xd4f7a834, 91 },
    { 0x9f67a835, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367a839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b7a83c, 61 },
    { 0, 0 },
    { 0x2027a83e, 23 },
    { 0xc407a83e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee7a842, 67 },
    { 0xf837a842, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017a846, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87a84a, 61 },
    { 0x1427a84a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2507a84f, 23 },
    { 0, 0 },
    { 0x8d37a851, 76 },
    { 0xf2a7a852, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7a858, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc417a85d, 52 },
    { 0, 0 },
    { 0xe4a7a85f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a17a862, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8387a868, 61 },
    { 0xf587a869, 40 },
    { 0xca7a868, 36 },
    { 0, 0 },
    { 0xa1b7a86c, 67 },
    { 0xa57a86d, 100 },
    { 0xa37a86c, 96 },
    { 0x3fa7a86f, 17 },
    { 0x237a870, 23 },
    { 0x95a7a86d, 43 },
    { 0xc857a86e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07a876, 41 },
    { 0x7617a877, 61 },
    { 0, 0 },
    { 0x7187a879, 23 },
    { 0x2057a879, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e7a886, 40 },
    { 0x9317a887, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd7a88a, 95 },
    { 0x7ec7a88b, 86 },
    { 0xc0f7a88c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007a88f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf07a894, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf637a898, 95 },
    { 0, 0 },
    { 0x5e17a89a, 91 },
    { 0x4b87a89b, 61 },
    { 0x1267a89b, 88 },
    { 0, 0 },
    { 0x6a97a89e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e7a8a4, 66 },
    { 0, 0 },
    { 0x8bc7a8a6, 98 },
    { 0xf417a8a6, 61 },
    { 0x3977a8a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017a8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7a8ae, 61 },
    { 0xedc7a8af, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7a8b3, 46 },
    { 0x7477a8b4, 55 },
    { 0x5f27a8b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb7a8b9, 23 },
    { 0xd417a8ba, 46 },
    { 0x6a37a8bb, 90 },
    { 0xec67a8bb, 67 },
    { 0x2a57a8bc, 90 },
    { 0x2e97a8bb, 95 },
    { 0xa477a8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c7a8c3, 76 },
    { 0x2b87a8c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a8c7, 90 },
    { 0xd447a8c8, 95 },
    { 0x3bc7a8c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d57a8ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf167a8d2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a8d8, 90 },
    { 0x9007a8d9, 17 },
    { 0, 0 },
    { 0x5437a8db, 67 },
    { 0x6817a8dc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4447a8df, 33 },
    { 0x38a7a8e0, 95 },
    { 0x55b7a8e1, 88 },
    { 0, 0 },
    { 0xc517a8e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7a8e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e27a8ed, 67 },
    { 0x8917a8ee, 95 },
    { 0xcc7a8ef, 90 },
    { 0x8af7a8ef, 46 },
    { 0, 0 },
    { 0x7cc7a8f2, 24 },
    { 0x8417a8f2, 33 },
    { 0, 0 },
    { 0x6d7a8f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc507a8f8, 34 },
    { 0, 0 },
    { 0x7f77a8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a8fd, 85 },
    { 0x2fb7a8fe, 55 },
    { 0x1bd7a8ff, 33 },
    { 0x747a8fd, 33 },
    { 0x88e7a8fe, 98 },
    { 0x4047a901, 95 },
    { 0, 0 },
    { 0x9377a904, 78 },
    { 0xefa7a905, 61 },
    { 0, 0 },
    { 0x4137a907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d7a90f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627a913, 67 },
    { 0x8287a913, 68 },
    { 0x1fd7a915, 99 },
    { 0, 0 },
    { 0xff97a917, 90 },
    { 0xdc57a917, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7a91e, 90 },
    { 0, 0 },
    { 0xcc77a920, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207a925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb7a92d, 52 },
    { 0x5c27a92e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17a933, 90 },
    { 0xcf67a933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8227a937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc7a93a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc7a93d, 33 },
    { 0x1a87a93e, 77 },
    { 0x3f07a93f, 95 },
    { 0x4847a940, 90 },
    { 0x8d7a93e, 52 },
    { 0x3e17a942, 61 },
    { 0x9227a943, 90 },
    { 0x6ff7a940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3477a948, 34 },
    { 0, 0 },
    { 0xd347a94a, 33 },
    { 0x5d7a94a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2017a950, 95 },
    { 0x6e47a951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7a954, 94 },
    { 0x6717a954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d7a95a, 88 },
    { 0, 0 },
    { 0x4167a95c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca7a960, 98 },
    { 0, 0 },
    { 0xd717a962, 46 },
    { 0, 0 },
    { 0x2fa7a964, 95 },
    { 0, 0 },
    { 0xb677a966, 43 },
    { 0, 0 },
    { 0x3dc7a968, 23 },
    { 0, 0 },
    { 0xd7c7a96a, 61 },
    { 0, 0 },
    { 0xa447a96c, 33 },
    { 0, 0 },
    { 0x8ca7a96e, 99 },
    { 0xed67a96f, 61 },
    { 0x2c97a970, 93 },
    { 0x6e57a96e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47a975, 36 },
    { 0xdf17a976, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9527a97b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7a97e, 33 },
    { 0xe327a97f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd857a984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7177a988, 55 },
    { 0, 0 },
    { 0xfde7a98a, 96 },
    { 0, 0 },
    { 0xa777a98c, 95 },
    { 0, 0 },
    { 0x9a27a98e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187a996, 98 },
    { 0x9c47a996, 95 },
    { 0xf5e7a998, 33 },
    { 0xc1a7a999, 33 },
    { 0x2597a99a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c7a99d, 93 },
    { 0x4f47a99e, 61 },
    { 0x29e7a99f, 88 },
    { 0, 0 },
    { 0x4897a9a1, 76 },
    { 0x4ca7a9a2, 98 },
    { 0x3957a9a3, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xec67a9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577a9a9, 90 },
    { 0xb1a7a9aa, 33 },
    { 0x7ca7a9ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897a9ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b7a9b7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d7a9bd, 61 },
    { 0x8177a9be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e37a9c1, 76 },
    { 0x7607a9c2, 98 },
    { 0x5a47a9c3, 43 },
    { 0xb5d7a9c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b7a9c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f7a9cd, 55 },
    { 0xa1f7a9cd, 98 },
    { 0, 0 },
    { 0x6457a9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587a9d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba7a9d8, 33 },
    { 0x347a9d9, 17 },
    { 0xb417a9da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f7a9de, 90 },
    { 0xc637a9df, 95 },
    { 0xd1a7a9de, 23 },
    { 0x2517a9e1, 95 },
    { 0x69b7a9df, 33 },
    { 0xb027a9df, 66 },
    { 0, 0 },
    { 0x5fd7a9e5, 33 },
    { 0, 0 },
    { 0xe377a9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1867a9eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec07a9f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557a9fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c7aa00, 69 },
    { 0xc8a7aa01, 61 },
    { 0x45c7aa02, 93 },
    { 0, 0 },
    { 0x8367aa04, 95 },
    { 0xab17aa05, 23 },
    { 0x437aa04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7aa09, 33 },
    { 0x1a77aa09, 93 },
    { 0x4447aa0b, 23 },
    { 0xcfb7aa0c, 33 },
    { 0, 0 },
    { 0x31c7aa0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7947aa14, 95 },
    { 0xdc57aa15, 98 },
    { 0x1ad7aa14, 99 },
    { 0x3527aa16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af7aa20, 90 },
    { 0x1b7aa21, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f7aa24, 67 },
    { 0x1857aa25, 84 },
    { 0, 0 },
    { 0xca87aa27, 69 },
    { 0xb017aa28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa7aa2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147aa2e, 23 },
    { 0, 0 },
    { 0x32b7aa30, 66 },
    { 0xa207aa31, 95 },
    { 0xcf67aa32, 33 },
    { 0xd537aa33, 61 },
    { 0x4077aa34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe607aa38, 95 },
    { 0xc3b7aa39, 90 },
    { 0, 0 },
    { 0x3b7aa3b, 76 },
    { 0xd2a7aa3b, 95 },
    { 0x9197aa3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe767aa42, 88 },
    { 0, 0 },
    { 0xec07aa44, 61 },
    { 0xca47aa44, 87 },
    { 0x2277aa46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1617aa4b, 87 },
    { 0xd3a7aa4b, 55 },
    { 0x4497aa4c, 90 },
    { 0x6da7aa4c, 90 },
    { 0x7417aa4d, 43 },
    { 0x3f97aa4c, 61 },
    { 0, 0 },
    { 0xf507aa52, 43 },
    { 0x767aa53, 90 },
    { 0x5e87aa52, 78 },
    { 0, 0 },
    { 0xa557aa56, 61 },
    { 0, 0 },
    { 0xd7b7aa58, 55 },
    { 0xad57aa59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047aa5c, 90 },
    { 0xfe37aa5c, 23 },
    { 0, 0 },
    { 0x6e87aa5f, 43 },
    { 0x3a17aa60, 98 },
    { 0x5537aa60, 95 },
    { 0xe8d7aa62, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc237aa65, 90 },
    { 0xd887aa66, 55 },
    { 0x5777aa67, 61 },
    { 0x3477aa66, 23 },
    { 0x3a47aa69, 98 },
    { 0xb847aa6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7aa6d, 99 },
    { 0, 0 },
    { 0x6777aa6f, 36 },
    { 0, 0 },
    { 0xc607aa71, 23 },
    { 0x1a27aa72, 33 },
    { 0x6a77aa73, 33 },
    { 0x5d67aa72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6497aa80, 98 },
    { 0, 0 },
    { 0xdc77aa82, 46 },
    { 0x7747aa82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007aa87, 91 },
    { 0, 0 },
    { 0x4617aa89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577aa8e, 23 },
    { 0xde17aa8e, 23 },
    { 0xab37aa90, 90 },
    { 0x287aa91, 84 },
    { 0x817aa91, 61 },
    { 0x1bd7aa92, 61 },
    { 0xb697aa90, 93 },
    { 0xf957aa95, 95 },
    { 0, 0 },
    { 0x4077aa97, 33 },
    { 0, 0 },
    { 0x6107aa99, 23 },
    { 0x59b7aa9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb227aa9d, 98 },
    { 0, 0 },
    { 0xd167aa9f, 76 },
    { 0, 0 },
    { 0xcba7aaa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf07aaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd7aaab, 90 },
    { 0x1167aaac, 17 },
    { 0x5637aaad, 34 },
    { 0x35e7aaab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a7aab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d7aab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c7aab9, 65 },
    { 0x6717aab9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c7aac4, 33 },
    { 0, 0 },
    { 0x7de7aac6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e7aac9, 100 },
    { 0x6db7aaca, 43 },
    { 0, 0 },
    { 0xb4c7aacc, 99 },
    { 0xa337aacd, 68 },
    { 0xe8a7aace, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657aada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a7aadd, 91 },
    { 0xef7aade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67aae1, 38 },
    { 0x3e67aae2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b7aae5, 61 },
    { 0x1287aae6, 95 },
    { 0x2a37aae5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2517aaeb, 33 },
    { 0x1f97aaec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77aaf3, 84 },
    { 0xa857aaf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf7aaf8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17aafb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f97aafe, 85 },
    { 0, 0 },
    { 0xb997ab00, 55 },
    { 0xe627ab01, 90 },
    { 0x3b7ab02, 95 },
    { 0x5b17ab02, 61 },
    { 0xb17ab04, 33 },
    { 0x31c7ab04, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6937ab0a, 100 },
    { 0, 0 },
    { 0x2f7ab0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9177ab0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547ab14, 91 },
    { 0x2a47ab14, 99 },
    { 0x8897ab16, 77 },
    { 0, 0 },
    { 0xbe67ab18, 23 },
    { 0xd0f7ab18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ab1d, 23 },
    { 0xcf47ab1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc907ab21, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d7ab28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297ab2d, 90 },
    { 0xe057ab2d, 90 },
    { 0, 0 },
    { 0x2b47ab30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967ab39, 23 },
    { 0, 0 },
    { 0x4c17ab3b, 23 },
    { 0x4807ab3c, 53 },
    { 0xa337ab3c, 98 },
    { 0x227ab3e, 23 },
    { 0x3c47ab3f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67ab46, 76 },
    { 0xc6e7ab46, 33 },
    { 0x4c97ab47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a7ab51, 98 },
    { 0, 0 },
    { 0xd1d7ab53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b7ab57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f27ab5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb7ab62, 91 },
    { 0x8527ab63, 34 },
    { 0x22f7ab62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd637ab68, 17 },
    { 0x3d47ab69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe597ab6d, 55 },
    { 0x3337ab6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9267ab73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2067ab78, 61 },
    { 0xa197ab79, 90 },
    { 0xc637ab79, 23 },
    { 0xd147ab7b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827ab80, 52 },
    { 0, 0 },
    { 0x617ab82, 61 },
    { 0x51d7ab83, 68 },
    { 0x4aa7ab84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7ab8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407ab98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7ab9c, 46 },
    { 0x3107ab9c, 90 },
    { 0, 0 },
    { 0xdb37ab9f, 88 },
    { 0x8917aba0, 61 },
    { 0xae97aba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb7aba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8897abab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd067abae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7347abb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f17abba, 90 },
    { 0x4507abbb, 61 },
    { 0x31c7abbb, 77 },
    { 0x9d37abbb, 98 },
    { 0x787abbe, 76 },
    { 0x1167abbd, 90 },
    { 0, 0 },
    { 0xc917abc1, 61 },
    { 0x5b67abc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df7abc7, 90 },
    { 0xead7abc7, 98 },
    { 0x3b67abc7, 52 },
    { 0, 0 },
    { 0x8e67abcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b37abd1, 33 },
    { 0x5407abd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f67abd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b7abda, 23 },
    { 0x33d7abdb, 23 },
    { 0x8ad7abda, 98 },
    { 0x7f47abdd, 88 },
    { 0, 0 },
    { 0xd517abdf, 33 },
    { 0xeeb7abe0, 98 },
    { 0xbe67abe0, 23 },
    { 0x1a77abe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa847abe7, 33 },
    { 0x9e77abe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd927abfe, 33 },
    { 0xf417abff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677ac06, 61 },
    { 0x2bf7ac07, 98 },
    { 0x17ac08, 76 },
    { 0, 0 },
    { 0x7657ac0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27ac13, 90 },
    { 0x71b7ac14, 98 },
    { 0x6777ac15, 50 },
    { 0x7227ac16, 90 },
    { 0x7207ac17, 61 },
    { 0xbb97ac17, 55 },
    { 0xafa7ac19, 90 },
    { 0x5f37ac1a, 55 },
    { 0x6097ac1a, 68 },
    { 0xed27ac1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a77ac21, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7ac28, 46 },
    { 0x317ac29, 98 },
    { 0x8d7ac2a, 95 },
    { 0xd397ac2a, 75 },
    { 0xfae7ac2a, 95 },
    { 0, 0 },
    { 0x8e27ac2e, 91 },
    { 0x14f7ac2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6317ac32, 46 },
    { 0, 0 },
    { 0x8137ac34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b7ac39, 61 },
    { 0, 0 },
    { 0xfe27ac3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5417ac40, 90 },
    { 0, 0 },
    { 0xc87ac42, 100 },
    { 0x3a77ac43, 33 },
    { 0xc537ac43, 46 },
    { 0, 0 },
    { 0x9667ac46, 55 },
    { 0xc9b7ac47, 52 },
    { 0xeeb7ac46, 90 },
    { 0xae7ac49, 75 },
    { 0xa507ac49, 90 },
    { 0x9227ac4b, 23 },
    { 0x1377ac4c, 33 },
    { 0x96d7ac4d, 23 },
    { 0, 0 },
    { 0xe1a7ac4f, 68 },
    { 0x85c7ac50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917ac59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc547ac5f, 61 },
    { 0, 0 },
    { 0xd087ac61, 59 },
    { 0, 0 },
    { 0x4b07ac63, 90 },
    { 0, 0 },
    { 0xe107ac65, 85 },
    { 0xcd17ac66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d7ac6c, 90 },
    { 0, 0 },
    { 0x8207ac6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7ac72, 95 },
    { 0, 0 },
    { 0x3aa7ac74, 90 },
    { 0, 0 },
    { 0x6107ac76, 61 },
    { 0x4337ac77, 33 },
    { 0xf607ac78, 81 },
    { 0x19a7ac79, 76 },
    { 0x5e77ac7a, 95 },
    { 0, 0 },
    { 0xef17ac7c, 67 },
    { 0, 0 },
    { 0xe027ac7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467ac82, 33 },
    { 0x2727ac82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7ac8a, 95 },
    { 0x6c17ac8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb247ac8e, 17 },
    { 0x7897ac8f, 90 },
    { 0x7ba7ac90, 33 },
    { 0, 0 },
    { 0x1f87ac92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8057ac97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d77ac9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f87ac9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7567aca9, 55 },
    { 0, 0 },
    { 0xf9e7acab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7acaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe727acb3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x897acb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b7acb9, 43 },
    { 0x56b7acba, 61 },
    { 0x4ec7acba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c7acbf, 98 },
    { 0x8f97acbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b7acc4, 33 },
    { 0x46f7acc5, 55 },
    { 0x10c7acc6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7087acce, 76 },
    { 0xeb57accf, 78 },
    { 0xf0b7accf, 90 },
    { 0x3887acd1, 33 },
    { 0x7d47acd1, 23 },
    { 0xaba7acd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf47acd6, 90 },
    { 0, 0 },
    { 0x6787acd8, 23 },
    { 0xb57acd9, 95 },
    { 0xd437acda, 34 },
    { 0x1aa7acd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ace2, 43 },
    { 0, 0 },
    { 0xdfb7ace4, 76 },
    { 0, 0 },
    { 0x1a87ace6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4987ace9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377acf4, 76 },
    { 0x2307acf5, 78 },
    { 0x5687acf6, 33 },
    { 0x3a97acf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf737ad00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd7ad08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47ad0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7ad0f, 76 },
    { 0x9067ad0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec7ad17, 61 },
    { 0xf017ad18, 69 },
    { 0x1637ad18, 33 },
    { 0xd237ad1a, 46 },
    { 0, 0 },
    { 0xeeb7ad1c, 91 },
    { 0, 0 },
    { 0x5e7ad1e, 98 },
    { 0xa967ad1f, 34 },
    { 0x2267ad20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f7ad23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e7ad2d, 90 },
    { 0x3af7ad2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb007ad39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa07ad43, 66 },
    { 0, 0 },
    { 0xab07ad45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d57ad48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa7ad51, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77ad5d, 98 },
    { 0x4b97ad5e, 34 },
    { 0x6947ad5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c17ad63, 90 },
    { 0, 0 },
    { 0x3ee7ad65, 87 },
    { 0xb337ad65, 61 },
    { 0, 0 },
    { 0x86e7ad68, 67 },
    { 0x8c37ad69, 84 },
    { 0xd767ad68, 93 },
    { 0x84d7ad6b, 90 },
    { 0x9cd7ad6b, 33 },
    { 0xda37ad6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197ad7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb7ad7e, 33 },
    { 0x33a7ad7e, 55 },
    { 0, 0 },
    { 0x1d37ad81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27ad84, 33 },
    { 0x6837ad85, 33 },
    { 0xe7d7ad85, 23 },
    { 0x77a7ad85, 23 },
    { 0x6f07ad88, 43 },
    { 0, 0 },
    { 0xf2a7ad8a, 88 },
    { 0xf697ad8a, 33 },
    { 0xd347ad8c, 23 },
    { 0, 0 },
    { 0xa637ad8e, 55 },
    { 0x4807ad8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47ad96, 83 },
    { 0x777ad96, 95 },
    { 0, 0 },
    { 0x3447ad99, 34 },
    { 0x7837ad99, 87 },
    { 0x84e7ad9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b87ad9e, 84 },
    { 0x8957ad9f, 90 },
    { 0, 0 },
    { 0xe057ada1, 55 },
    { 0x13d7ada2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee17ada7, 98 },
    { 0, 0 },
    { 0xf487ada9, 98 },
    { 0, 0 },
    { 0x9e27adab, 85 },
    { 0xc897adac, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d47adaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa7adbc, 76 },
    { 0, 0 },
    { 0x9b87adbe, 98 },
    { 0x37c7adbf, 66 },
    { 0x2937adc0, 34 },
    { 0xa8c7adbe, 33 },
    { 0x2d17adbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2617adc5, 98 },
    { 0x5bb7adc6, 52 },
    { 0x93c7adc6, 95 },
    { 0x7917adc8, 95 },
    { 0xb217adc5, 55 },
    { 0xe2a7adc5, 33 },
    { 0xbf47adca, 43 },
    { 0x3f87adcc, 87 },
    { 0, 0 },
    { 0xe587adce, 34 },
    { 0x5e97adcf, 95 },
    { 0, 0 },
    { 0x1ad7add1, 90 },
    { 0xab27add2, 23 },
    { 0xe247add2, 61 },
    { 0xfbc7add3, 76 },
    { 0, 0 },
    { 0x1c17add6, 98 },
    { 0x1d87add7, 23 },
    { 0, 0 },
    { 0x9857add9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5297addc, 52 },
    { 0x23c7addd, 55 },
    { 0xed57adde, 34 },
    { 0x30a7addf, 90 },
    { 0x8a77ade0, 76 },
    { 0, 0 },
    { 0xe1f7ade2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb37ade5, 76 },
    { 0x2287ade6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8227ade9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e47adec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77adef, 33 },
    { 0, 0 },
    { 0x57b7adf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe637adf4, 95 },
    { 0xeb7adf4, 90 },
    { 0x1697adf6, 23 },
    { 0xd6c7adf6, 90 },
    { 0x1117adf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17adfb, 98 },
    { 0x367adfc, 61 },
    { 0x88f7adfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee7ae05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee27ae0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2947ae0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f47ae13, 43 },
    { 0x18a7ae13, 85 },
    { 0x83f7ae15, 53 },
    { 0x6b67ae16, 72 },
    { 0xfc07ae16, 90 },
    { 0x4d27ae16, 100 },
    { 0xe317ae16, 67 },
    { 0x6d07ae1a, 46 },
    { 0xccd7ae1a, 61 },
    { 0, 0 },
    { 0x38d7ae1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e97ae20, 23 },
    { 0x5387ae20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587ae2a, 61 },
    { 0xb6b7ae2b, 90 },
    { 0xaeb7ae2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf507ae30, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577ae33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd7ae36, 46 },
    { 0, 0 },
    { 0x98b7ae38, 87 },
    { 0xec57ae38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567ae40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf707ae44, 76 },
    { 0xb697ae45, 33 },
    { 0x38b7ae44, 61 },
    { 0, 0 },
    { 0xc657ae48, 66 },
    { 0, 0 },
    { 0x507ae4a, 78 },
    { 0, 0 },
    { 0x427ae4c, 84 },
    { 0x4f57ae4d, 90 },
    { 0, 0 },
    { 0x9317ae4f, 55 },
    { 0x1677ae4f, 90 },
    { 0x3787ae50, 34 },
    { 0xa667ae52, 75 },
    { 0x627ae50, 34 },
    { 0x5177ae54, 65 },
    { 0, 0 },
    { 0xba17ae56, 55 },
    { 0, 0 },
    { 0xbe37ae58, 90 },
    { 0x8cb7ae59, 90 },
    { 0x97ae5a, 95 },
    { 0, 0 },
    { 0xf377ae5c, 52 },
    { 0xf647ae5c, 95 },
    { 0xcd7ae5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37ae64, 76 },
    { 0x5267ae65, 23 },
    { 0xf187ae65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67ae6b, 77 },
    { 0x917ae6b, 33 },
    { 0x7577ae6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b97ae70, 98 },
    { 0xf037ae71, 98 },
    { 0x2cd7ae71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc7ae76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf897ae7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97ae80, 98 },
    { 0, 0 },
    { 0x5397ae82, 49 },
    { 0xf797ae82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27ae8d, 55 },
    { 0, 0 },
    { 0xdab7ae8f, 55 },
    { 0xccd7ae8f, 98 },
    { 0, 0 },
    { 0xe837ae92, 23 },
    { 0x6e77ae93, 61 },
    { 0x5487ae94, 61 },
    { 0x1157ae93, 99 },
    { 0x45e7ae96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf897ae9c, 3 },
    { 0, 0 },
    { 0x6e7ae9e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa957aea7, 99 },
    { 0, 0 },
    { 0x8587aea9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207aeb0, 33 },
    { 0x40d7aeb1, 23 },
    { 0, 0 },
    { 0x8707aeb3, 90 },
    { 0x22d7aeb4, 23 },
    { 0xbd87aeb5, 90 },
    { 0x6277aeb4, 88 },
    { 0, 0 },
    { 0x4e07aeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc107aebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917aebf, 87 },
    { 0x1f17aebf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c7aec4, 87 },
    { 0, 0 },
    { 0x9f7aec6, 33 },
    { 0x2217aec6, 98 },
    { 0, 0 },
    { 0xcb27aec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a87aecc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067aed3, 65 },
    { 0xdc17aed3, 55 },
    { 0xd3b7aed5, 61 },
    { 0, 0 },
    { 0xe7e7aed7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7aeda, 98 },
    { 0xb257aedb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7057aede, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf077aee2, 33 },
    { 0, 0 },
    { 0xc387aee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07aeea, 33 },
    { 0x5757aeeb, 76 },
    { 0xc427aeec, 90 },
    { 0x1eb7aeed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c7aef2, 61 },
    { 0x4c57aef2, 98 },
    { 0xfeb7aef4, 90 },
    { 0xcdb7aef4, 85 },
    { 0xe07aef6, 100 },
    { 0x4e87aef7, 33 },
    { 0xc577aef8, 61 },
    { 0x76e7aef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c7aefc, 55 },
    { 0x6a57aefd, 90 },
    { 0x4ae7aefe, 43 },
    { 0x8007aeff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd617af0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c7af13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd37af17, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7af1b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb417af21, 90 },
    { 0x50d7af22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f27af28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x307af2b, 33 },
    { 0xfe27af2c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f7af30, 90 },
    { 0xdc27af31, 33 },
    { 0x877af32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147af43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a57af48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xced7af4b, 93 },
    { 0x21c7af4c, 90 },
    { 0xdd87af4d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957af50, 33 },
    { 0x697af51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7af54, 68 },
    { 0, 0 },
    { 0xb3b7af56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc827af5c, 90 },
    { 0, 0 },
    { 0xa4e7af5e, 76 },
    { 0, 0 },
    { 0x77d7af60, 67 },
    { 0x5757af61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f07af69, 61 },
    { 0x7c57af6a, 55 },
    { 0x61d7af6a, 61 },
    { 0x627af6b, 76 },
    { 0xff97af6c, 75 },
    { 0x6527af6d, 33 },
    { 0, 0 },
    { 0x3817af70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a57af73, 23 },
    { 0x3b97af74, 55 },
    { 0x50e7af73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7af80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e77af86, 84 },
    { 0xe7e7af86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf807af8a, 23 },
    { 0xfe27af8b, 78 },
    { 0x6747af8c, 33 },
    { 0, 0 },
    { 0x2697af8e, 34 },
    { 0xdcf7af8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe427af97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37afa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe7afa3, 61 },
    { 0x6187afa4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef7afa9, 81 },
    { 0x3f47afa9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67afaf, 33 },
    { 0, 0 },
    { 0x3c47afb1, 23 },
    { 0xc4b7afb2, 90 },
    { 0x5c87afb2, 23 },
    { 0xfb37afb3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77afbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7afc4, 90 },
    { 0x5f7afc5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d57afc8, 93 },
    { 0x6637afc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa847afd4, 98 },
    { 0, 0 },
    { 0xee67afd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537afd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa317afdd, 33 },
    { 0x85a7afdd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817afe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd7afea, 46 },
    { 0, 0 },
    { 0xfca7afec, 73 },
    { 0x8d07afec, 95 },
    { 0xb1f7afee, 66 },
    { 0x7547afef, 33 },
    { 0x2a47aff0, 23 },
    { 0x3fd7aff0, 95 },
    { 0x51e7afec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aff5, 66 },
    { 0xb2f7aff6, 76 },
    { 0x6dd7aff7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa7affa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9687affd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e47b009, 101 },
    { 0xd5f7b00a, 61 },
    { 0xeea7b00b, 95 },
    { 0xb407b009, 61 },
    { 0xa117b00d, 33 },
    { 0x3f17b00e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7b016, 87 },
    { 0x1f97b017, 95 },
    { 0, 0 },
    { 0x9a7b019, 98 },
    { 0, 0 },
    { 0x5af7b01b, 90 },
    { 0x3177b01c, 95 },
    { 0, 0 },
    { 0x4b37b01e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7b021, 99 },
    { 0, 0 },
    { 0x4f17b023, 78 },
    { 0x9c07b024, 95 },
    { 0x6b37b025, 33 },
    { 0x4db7b026, 46 },
    { 0x7157b024, 61 },
    { 0x6b17b028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7b02e, 33 },
    { 0, 0 },
    { 0x6657b030, 90 },
    { 0xdd87b031, 55 },
    { 0x7b032, 100 },
    { 0x2a37b033, 61 },
    { 0x4fb7b034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d7b037, 93 },
    { 0, 0 },
    { 0x35e7b039, 95 },
    { 0xca57b03a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907b041, 34 },
    { 0xeaa7b042, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf7b049, 33 },
    { 0x7237b04a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe747b04f, 38 },
    { 0, 0 },
    { 0xfcf7b051, 95 },
    { 0xc457b052, 95 },
    { 0x66f7b051, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd7b056, 66 },
    { 0, 0 },
    { 0xbcc7b058, 65 },
    { 0x6817b058, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c7b05e, 95 },
    { 0x9407b05e, 98 },
    { 0xe587b060, 95 },
    { 0, 0 },
    { 0xc587b062, 76 },
    { 0x4b7b063, 75 },
    { 0xa617b063, 68 },
    { 0xb57b063, 61 },
    { 0, 0 },
    { 0x4397b067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877b06a, 98 },
    { 0xf277b06b, 61 },
    { 0, 0 },
    { 0x9187b06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7b071, 9 },
    { 0xe117b071, 90 },
    { 0xcb47b073, 76 },
    { 0x9a77b071, 43 },
    { 0xc797b071, 90 },
    { 0, 0 },
    { 0x8087b077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957b07b, 91 },
    { 0x9d27b07c, 23 },
    { 0, 0 },
    { 0x92d7b07e, 69 },
    { 0x1fd7b07f, 77 },
    { 0x7cf7b080, 55 },
    { 0xbb57b080, 61 },
    { 0xe17b082, 81 },
    { 0, 0 },
    { 0xe447b084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767b087, 61 },
    { 0x7717b088, 90 },
    { 0xc617b088, 95 },
    { 0xe6a7b088, 84 },
    { 0xdfa7b089, 95 },
    { 0, 0 },
    { 0x4f67b08d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa197b094, 85 },
    { 0, 0 },
    { 0x1ac7b096, 68 },
    { 0x7d37b096, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f57b09c, 66 },
    { 0, 0 },
    { 0x5307b09e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xed57b0a1, 23 },
    { 0xf5f7b0a2, 43 },
    { 0xaa97b0a3, 81 },
    { 0x4f77b0a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa977b0a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe237b0ac, 95 },
    { 0x8f87b0ad, 34 },
    { 0, 0 },
    { 0xb697b0af, 75 },
    { 0, 0 },
    { 0x4247b0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127b0b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77b0bd, 61 },
    { 0x3ec7b0bd, 33 },
    { 0xf867b0bd, 90 },
    { 0x3da7b0c0, 77 },
    { 0x53a7b0be, 42 },
    { 0x7307b0c2, 76 },
    { 0xe1a7b0c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b7b0c6, 99 },
    { 0x57f7b0c7, 90 },
    { 0, 0 },
    { 0x11e7b0c9, 78 },
    { 0x9d57b0ca, 98 },
    { 0, 0 },
    { 0xe4a7b0cc, 55 },
    { 0, 0 },
    { 0x2af7b0ce, 39 },
    { 0, 0 },
    { 0x93e7b0d0, 100 },
    { 0, 0 },
    { 0x2237b0d2, 46 },
    { 0x94a7b0d3, 93 },
    { 0xef97b0d2, 85 },
    { 0xf9d7b0d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1897b0d8, 95 },
    { 0x6297b0d9, 69 },
    { 0x4d97b0da, 95 },
    { 0x57e7b0d8, 91 },
    { 0xc317b0da, 55 },
    { 0x1087b0dc, 48 },
    { 0xd537b0de, 46 },
    { 0x5a97b0d8, 96 },
    { 0xe727b0e0, 78 },
    { 0, 0 },
    { 0x4aa7b0e2, 90 },
    { 0x4297b0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4987b0e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e97b0e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe7b0ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7b0f1, 85 },
    { 0x4277b0f2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e7b0f5, 98 },
    { 0xda87b0f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77b0fb, 96 },
    { 0, 0 },
    { 0x3307b0fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1917b102, 23 },
    { 0x2f37b102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c7b106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3207b10c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa227b10f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df7b115, 95 },
    { 0x8927b116, 77 },
    { 0x5b67b117, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c77b11a, 33 },
    { 0x8b67b11a, 33 },
    { 0xa057b11a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb7b11f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787b125, 34 },
    { 0, 0 },
    { 0x8277b127, 33 },
    { 0x1b97b128, 23 },
    { 0x6f47b129, 93 },
    { 0xfc77b12a, 81 },
    { 0x53b7b12b, 34 },
    { 0xb587b12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877b12f, 84 },
    { 0x5377b130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c87b137, 61 },
    { 0x937b138, 33 },
    { 0x5f17b139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b141, 34 },
    { 0, 0 },
    { 0xa9f7b143, 97 },
    { 0, 0 },
    { 0x3e47b145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc67b149, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd357b14f, 33 },
    { 0x1c17b150, 93 },
    { 0x60b7b151, 61 },
    { 0, 0 },
    { 0xcba7b153, 76 },
    { 0x2087b154, 23 },
    { 0x69e7b153, 91 },
    { 0x7e07b156, 98 },
    { 0x877b155, 17 },
    { 0xbf7b154, 76 },
    { 0xe477b153, 76 },
    { 0, 0 },
    { 0xc107b15b, 87 },
    { 0xc587b15c, 90 },
    { 0x33b7b15b, 77 },
    { 0x6cc7b15e, 52 },
    { 0xfb37b15b, 23 },
    { 0x9587b15b, 61 },
    { 0xce37b15e, 95 },
    { 0xa0a7b162, 85 },
    { 0xb717b161, 33 },
    { 0x45b7b164, 61 },
    { 0x1907b15d, 95 },
    { 0xcaa7b166, 98 },
    { 0x1dc7b167, 55 },
    { 0x7477b164, 34 },
    { 0, 0 },
    { 0x6d97b16a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227b170, 98 },
    { 0, 0 },
    { 0x1c67b172, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237b178, 33 },
    { 0, 0 },
    { 0x41b7b17a, 61 },
    { 0, 0 },
    { 0xd267b17c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5447b181, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1377b187, 46 },
    { 0xd7f7b188, 98 },
    { 0x5ef7b189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6497b18c, 98 },
    { 0, 0 },
    { 0x27d7b18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8177b192, 80 },
    { 0x63f7b193, 95 },
    { 0xc5a7b193, 91 },
    { 0x7e17b195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf27b198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7b19d, 90 },
    { 0x9d77b19e, 33 },
    { 0x4ba7b19f, 98 },
    { 0x88d7b1a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e7b1a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d47b1a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437b1af, 33 },
    { 0, 0 },
    { 0xc217b1b1, 72 },
    { 0xb627b1b2, 48 },
    { 0, 0 },
    { 0x147b1b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d7b1b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187b1bd, 98 },
    { 0xbd67b1be, 93 },
    { 0x7177b1bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7b1c7, 93 },
    { 0xabe7b1c8, 56 },
    { 0x1917b1c9, 40 },
    { 0xe1a7b1ca, 23 },
    { 0, 0 },
    { 0x6b07b1cc, 38 },
    { 0x89f7b1cd, 61 },
    { 0, 0 },
    { 0x22e7b1cf, 93 },
    { 0x2607b1cf, 61 },
    { 0, 0 },
    { 0x32e7b1d2, 99 },
    { 0x7687b1d3, 87 },
    { 0x1c7b1d4, 87 },
    { 0, 0 },
    { 0x2b57b1d6, 75 },
    { 0x2767b1d7, 78 },
    { 0, 0 },
    { 0x8ab7b1d9, 95 },
    { 0x6257b1d9, 23 },
    { 0x79e7b1db, 23 },
    { 0x7557b1dc, 69 },
    { 0, 0 },
    { 0x7757b1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe7b1e4, 88 },
    { 0, 0 },
    { 0x80f7b1e6, 90 },
    { 0x3f97b1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17b1ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b7b1f2, 23 },
    { 0x2ba7b1f3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e7b1f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7b1fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8347b200, 87 },
    { 0xf6c7b201, 95 },
    { 0x47c7b201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37b20c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27b20f, 75 },
    { 0, 0 },
    { 0xc597b211, 38 },
    { 0xafb7b212, 23 },
    { 0xbd27b213, 95 },
    { 0xf477b211, 23 },
    { 0x3347b212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b21a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f7b21d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7b220, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8077b225, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6577b229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7b22e, 93 },
    { 0, 0 },
    { 0xf6b7b230, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb337b235, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec97b23a, 98 },
    { 0, 0 },
    { 0xfa77b23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7b23f, 33 },
    { 0x5d7b240, 66 },
    { 0xd577b240, 93 },
    { 0x3707b240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6697b246, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8247b24c, 98 },
    { 0, 0 },
    { 0xac37b24e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7b254, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4047b258, 8 },
    { 0x17b258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf47b25e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc7b264, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa87b267, 95 },
    { 0, 0 },
    { 0x2ad7b269, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27b26e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d7b275, 55 },
    { 0x7097b275, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577b27f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf787b282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9157b286, 91 },
    { 0x49e7b287, 46 },
    { 0x8047b287, 92 },
    { 0xc97b289, 98 },
    { 0, 0 },
    { 0xefb7b28b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8617b28f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c7b293, 61 },
    { 0xf9f7b294, 23 },
    { 0x8997b295, 33 },
    { 0xd657b295, 55 },
    { 0x3597b294, 34 },
    { 0, 0 },
    { 0xc597b299, 66 },
    { 0xb137b29a, 66 },
    { 0x14f7b299, 90 },
    { 0x8bc7b29a, 33 },
    { 0, 0 },
    { 0xaf77b29e, 95 },
    { 0xe247b29f, 33 },
    { 0xfba7b29f, 98 },
    { 0x2097b2a1, 90 },
    { 0x81a7b2a2, 46 },
    { 0xa597b2a2, 76 },
    { 0, 0 },
    { 0xe447b2a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd657b2a9, 76 },
    { 0, 0 },
    { 0x8847b2ab, 68 },
    { 0xc497b2ac, 34 },
    { 0xb3e7b2ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be7b2b8, 95 },
    { 0, 0 },
    { 0xb757b2ba, 98 },
    { 0x1877b2bb, 84 },
    { 0x5da7b2bc, 100 },
    { 0x3547b2bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb37b2c0, 91 },
    { 0x10d7b2c1, 61 },
    { 0x29b7b2c1, 52 },
    { 0, 0 },
    { 0xda27b2c4, 90 },
    { 0xd387b2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe87b2c8, 55 },
    { 0, 0 },
    { 0xb357b2ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887b2d5, 100 },
    { 0x87f7b2d5, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7617b2da, 61 },
    { 0xebe7b2db, 23 },
    { 0xa297b2dc, 66 },
    { 0xadd7b2da, 43 },
    { 0xaf97b2de, 38 },
    { 0, 0 },
    { 0xdc87b2e0, 55 },
    { 0, 0 },
    { 0x6ed7b2e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa477b2e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f7b2f1, 84 },
    { 0xbe67b2f2, 61 },
    { 0x7947b2f2, 33 },
    { 0, 0 },
    { 0x99c7b2f5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7b2ff, 94 },
    { 0x4f47b2ff, 34 },
    { 0, 0 },
    { 0x367b302, 23 },
    { 0xcca7b303, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07b306, 43 },
    { 0, 0 },
    { 0x23e7b308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77b30c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7b310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427b323, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7b326, 81 },
    { 0x4ac7b327, 67 },
    { 0xa77b327, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f7b331, 9 },
    { 0xe3c7b332, 85 },
    { 0x6787b333, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7b336, 40 },
    { 0xb247b336, 93 },
    { 0, 0 },
    { 0xfb47b339, 90 },
    { 0x8317b33a, 98 },
    { 0, 0 },
    { 0xefd7b33c, 76 },
    { 0x1857b33d, 98 },
    { 0x45d7b33e, 95 },
    { 0xfba7b33f, 23 },
    { 0x99a7b33e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8607b344, 95 },
    { 0x5177b345, 98 },
    { 0xb7e7b346, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7917b34b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97b34e, 33 },
    { 0, 0 },
    { 0xae77b350, 90 },
    { 0x47b351, 91 },
    { 0xd2a7b351, 33 },
    { 0, 0 },
    { 0x76c7b354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9977b358, 61 },
    { 0x8507b358, 23 },
    { 0x86c7b35a, 88 },
    { 0, 0 },
    { 0xdb67b35c, 90 },
    { 0x6d77b35d, 61 },
    { 0x1c37b35e, 95 },
    { 0xa77b35e, 68 },
    { 0x9a87b35e, 17 },
    { 0, 0 },
    { 0xeb17b362, 90 },
    { 0, 0 },
    { 0x1a47b364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe057b368, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb7b36c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6067b371, 95 },
    { 0x8257b372, 33 },
    { 0x5517b373, 90 },
    { 0x9fd7b373, 98 },
    { 0, 0 },
    { 0x38c7b376, 23 },
    { 0, 0 },
    { 0x5d97b378, 99 },
    { 0x5827b378, 90 },
    { 0, 0 },
    { 0x1e37b37b, 84 },
    { 0x4cd7b37c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf7b382, 98 },
    { 0, 0 },
    { 0x4407b384, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9107b387, 34 },
    { 0, 0 },
    { 0x8c47b389, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137b38d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e7b395, 83 },
    { 0, 0 },
    { 0xf7f7b397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7b39d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea7b3a3, 34 },
    { 0x8387b3a4, 98 },
    { 0xcb17b3a5, 95 },
    { 0x9027b3a6, 61 },
    { 0, 0 },
    { 0x6b47b3a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77b3ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9027b3b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e7b3b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7b3b8, 17 },
    { 0x5d57b3b8, 33 },
    { 0xfe07b3ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf7b3bd, 55 },
    { 0x9737b3bd, 95 },
    { 0x98b7b3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff07b3c3, 90 },
    { 0x7767b3c4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e7b3ca, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x967b3cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07b3d1, 66 },
    { 0xa3c7b3d1, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7b3d7, 43 },
    { 0xb577b3d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7b3db, 23 },
    { 0, 0 },
    { 0xd037b3dd, 61 },
    { 0, 0 },
    { 0xec97b3df, 33 },
    { 0, 0 },
    { 0xb027b3e1, 66 },
    { 0, 0 },
    { 0x2367b3e3, 33 },
    { 0x6337b3e3, 95 },
    { 0x6b77b3e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde7b3ef, 93 },
    { 0xeca7b3f0, 46 },
    { 0, 0 },
    { 0x23e7b3f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d7b3f8, 53 },
    { 0x9057b3f9, 67 },
    { 0x88f7b3f8, 99 },
    { 0x77b7b3f8, 61 },
    { 0, 0 },
    { 0x4037b3fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b7b401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f17b405, 95 },
    { 0xd717b406, 52 },
    { 0x2587b407, 100 },
    { 0xdef7b408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea37b411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7387b414, 17 },
    { 0xf1f7b415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127b41d, 78 },
    { 0x4917b41e, 95 },
    { 0, 0 },
    { 0x537b420, 23 },
    { 0x8af7b420, 95 },
    { 0xa727b422, 17 },
    { 0, 0 },
    { 0x34d7b424, 85 },
    { 0xc907b424, 34 },
    { 0, 0 },
    { 0x4057b427, 43 },
    { 0xf2a7b428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227b42e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e17b431, 52 },
    { 0x1dd7b431, 52 },
    { 0xf977b433, 76 },
    { 0xdb07b433, 76 },
    { 0xcc97b434, 93 },
    { 0, 0 },
    { 0xb727b437, 75 },
    { 0xb8a7b438, 46 },
    { 0, 0 },
    { 0x4a97b43a, 76 },
    { 0x1c27b43b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc497b43e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b442, 55 },
    { 0x8237b442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf77b448, 33 },
    { 0xbaf7b448, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4157b44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd417b453, 90 },
    { 0x12f7b454, 87 },
    { 0, 0 },
    { 0x5157b456, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc927b45b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3487b460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87b463, 34 },
    { 0x5a07b463, 66 },
    { 0, 0 },
    { 0xbc47b466, 100 },
    { 0xfa17b466, 93 },
    { 0x7897b468, 88 },
    { 0x9677b469, 55 },
    { 0x74e7b46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57b46d, 76 },
    { 0x1d37b46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc7b473, 98 },
    { 0x8817b474, 33 },
    { 0x60e7b475, 87 },
    { 0xc577b473, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d7b47b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7b47e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa77b481, 52 },
    { 0xf1b7b482, 55 },
    { 0x3aa7b483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d7b489, 95 },
    { 0, 0 },
    { 0x24a7b48b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af7b48f, 61 },
    { 0x4847b48f, 95 },
    { 0x12b7b491, 76 },
    { 0x2ec7b491, 93 },
    { 0x9547b490, 88 },
    { 0xeeb7b48f, 90 },
    { 0x3bb7b48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7b49a, 61 },
    { 0, 0 },
    { 0xe1b7b49c, 33 },
    { 0x41a7b49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9687b4a2, 93 },
    { 0x9f57b4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff17b4a9, 43 },
    { 0, 0 },
    { 0x1bd7b4ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f97b4af, 80 },
    { 0, 0 },
    { 0xc8f7b4b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7b4b6, 99 },
    { 0x9507b4b6, 52 },
    { 0xd2d7b4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337b4bf, 69 },
    { 0xda47b4c0, 75 },
    { 0, 0 },
    { 0xa3c7b4c2, 66 },
    { 0x2907b4c2, 61 },
    { 0x9197b4c4, 88 },
    { 0xdba7b4c5, 52 },
    { 0x1817b4c3, 23 },
    { 0xe947b4c7, 98 },
    { 0, 0 },
    { 0x31a7b4c9, 98 },
    { 0x30c7b4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed7b4cd, 43 },
    { 0x2ac7b4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067b4d7, 34 },
    { 0xf237b4d7, 95 },
    { 0x78d7b4d9, 33 },
    { 0x49f7b4da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab7b4de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa7b4e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887b4e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd477b4ee, 90 },
    { 0, 0 },
    { 0xcd77b4f0, 87 },
    { 0xc257b4f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a37b4fa, 61 },
    { 0x7127b4fb, 23 },
    { 0xc307b4fc, 73 },
    { 0xaf57b4fd, 23 },
    { 0xfc17b4fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab67b502, 90 },
    { 0xea27b503, 95 },
    { 0xf8c7b504, 95 },
    { 0x2367b502, 33 },
    { 0, 0 },
    { 0xf8a7b507, 98 },
    { 0xaeb7b507, 25 },
    { 0x55a7b509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8317b50c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7b510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e7b514, 33 },
    { 0xb307b515, 55 },
    { 0xfa97b515, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7b51b, 43 },
    { 0, 0 },
    { 0xa5d7b51d, 95 },
    { 0x48e7b51e, 100 },
    { 0x2cf7b51f, 66 },
    { 0xabb7b520, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4927b523, 100 },
    { 0x49c7b524, 76 },
    { 0x3407b525, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed7b530, 53 },
    { 0x2307b531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d7b535, 33 },
    { 0xcf57b535, 91 },
    { 0x32b7b537, 34 },
    { 0x9997b535, 61 },
    { 0x1037b539, 36 },
    { 0x4ae7b539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa517b540, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae7b543, 34 },
    { 0, 0 },
    { 0x1ff7b545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4867b548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c27b550, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6837b553, 61 },
    { 0, 0 },
    { 0xf397b555, 76 },
    { 0x1dc7b556, 61 },
    { 0x9ed7b557, 66 },
    { 0x2997b557, 98 },
    { 0x767b557, 65 },
    { 0x3637b55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc897b562, 23 },
    { 0x3127b562, 61 },
    { 0xdfb7b564, 61 },
    { 0x6ab7b564, 23 },
    { 0xfce7b564, 55 },
    { 0x8ea7b567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec7b56a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8517b56e, 87 },
    { 0xc557b56f, 98 },
    { 0xa587b56e, 84 },
    { 0, 0 },
    { 0xd07b572, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5727b576, 52 },
    { 0xa557b577, 92 },
    { 0x8637b578, 33 },
    { 0x9677b577, 76 },
    { 0x22e7b57a, 98 },
    { 0x2507b57a, 98 },
    { 0x3a77b57a, 99 },
    { 0x257b57a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2657b580, 61 },
    { 0x7617b580, 55 },
    { 0xd187b582, 34 },
    { 0x7857b583, 33 },
    { 0xf787b584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917b58c, 84 },
    { 0xbf7b58d, 84 },
    { 0xa977b58e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b87b591, 34 },
    { 0xa047b592, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547b59f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc77b5a4, 77 },
    { 0xdb7b5a5, 33 },
    { 0x7ec7b5a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c7b5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c7b5ac, 61 },
    { 0x1fa7b5ad, 95 },
    { 0x9fd7b5ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97b5b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee7b5ba, 23 },
    { 0, 0 },
    { 0x4ad7b5bc, 95 },
    { 0, 0 },
    { 0x2027b5be, 90 },
    { 0, 0 },
    { 0x5f87b5c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e7b5c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce7b5cf, 90 },
    { 0xa827b5d0, 55 },
    { 0, 0 },
    { 0x50a7b5d2, 52 },
    { 0xdb17b5d2, 52 },
    { 0, 0 },
    { 0xec67b5d5, 90 },
    { 0xc927b5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5517b5db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f27b5e0, 78 },
    { 0x1237b5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d07b5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe387b5ea, 23 },
    { 0x1987b5eb, 85 },
    { 0x53a7b5ea, 33 },
    { 0x64e7b5ea, 90 },
    { 0x3537b5ee, 77 },
    { 0xf237b5ee, 88 },
    { 0x7a7b5ec, 23 },
    { 0x8a37b5f1, 61 },
    { 0x1817b5f1, 34 },
    { 0xb467b5ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d7b5f7, 90 },
    { 0x7e87b5f7, 23 },
    { 0x1757b5f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec7b5fc, 23 },
    { 0x95d7b5fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8487b602, 17 },
    { 0xc237b603, 91 },
    { 0xa9e7b603, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc7b608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d7b60b, 87 },
    { 0, 0 },
    { 0xc4d7b60d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd347b611, 95 },
    { 0, 0 },
    { 0x6e97b613, 46 },
    { 0xb407b613, 99 },
    { 0xcf57b613, 90 },
    { 0xf797b616, 90 },
    { 0x5e97b617, 95 },
    { 0xcf17b614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c57b61e, 98 },
    { 0xecb7b61f, 33 },
    { 0, 0 },
    { 0x387b621, 98 },
    { 0x5af7b622, 98 },
    { 0, 0 },
    { 0x6787b624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7727b627, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd97b62c, 78 },
    { 0xc827b62c, 34 },
    { 0x3937b62d, 34 },
    { 0, 0 },
    { 0xb37b630, 90 },
    { 0xdb47b631, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab7b634, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e7b638, 90 },
    { 0, 0 },
    { 0x5df7b63a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef97b63d, 99 },
    { 0x1997b63e, 99 },
    { 0xbf97b63f, 61 },
    { 0x927b63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb407b643, 61 },
    { 0xf467b643, 94 },
    { 0xe47b645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b97b648, 23 },
    { 0x32c7b648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7b64d, 91 },
    { 0xb567b64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad17b652, 43 },
    { 0x88b7b652, 95 },
    { 0xb8c7b654, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb7b658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787b65e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787b666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d7b669, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e7b66c, 66 },
    { 0x4057b66c, 91 },
    { 0xf597b66e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f7b679, 33 },
    { 0xa957b679, 95 },
    { 0xa5a7b67b, 92 },
    { 0xec7b67c, 61 },
    { 0x3257b67d, 95 },
    { 0x17e7b67c, 23 },
    { 0x3487b67f, 55 },
    { 0, 0 },
    { 0xb7c7b681, 45 },
    { 0xe147b681, 46 },
    { 0x3927b683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd7b689, 34 },
    { 0x22f7b689, 34 },
    { 0xd257b68b, 95 },
    { 0xf567b68c, 23 },
    { 0xbae7b68b, 66 },
    { 0x1be7b68a, 90 },
    { 0xa1f7b68f, 98 },
    { 0x3f87b68f, 61 },
    { 0x53e7b691, 98 },
    { 0xdce7b692, 61 },
    { 0xffe7b693, 88 },
    { 0, 0 },
    { 0xd5c7b695, 46 },
    { 0x4cf7b696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf7b69a, 33 },
    { 0xcae7b69a, 90 },
    { 0x7467b69a, 76 },
    { 0x35d7b69b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3037b6a2, 34 },
    { 0, 0 },
    { 0x5e7b6a4, 33 },
    { 0, 0 },
    { 0x2527b6a6, 99 },
    { 0, 0 },
    { 0x2e87b6a8, 98 },
    { 0xc0b7b6a8, 75 },
    { 0x5ab7b6aa, 46 },
    { 0x1557b6ab, 33 },
    { 0xa427b6a9, 52 },
    { 0x1f77b6ad, 55 },
    { 0xd7f7b6ac, 33 },
    { 0xf277b6ad, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427b6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e7b6b9, 87 },
    { 0xd4b7b6ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b27b6bf, 90 },
    { 0, 0 },
    { 0x7737b6c1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd7b6c4, 98 },
    { 0x7ef7b6c4, 52 },
    { 0, 0 },
    { 0x9507b6c7, 76 },
    { 0x64f7b6c8, 61 },
    { 0xa567b6c9, 90 },
    { 0x4a57b6c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d17b6cd, 33 },
    { 0x6dd7b6cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb7b6d2, 34 },
    { 0x18f7b6d3, 95 },
    { 0, 0 },
    { 0x3fc7b6d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557b6da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e7b6e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d87b6e5, 37 },
    { 0xf5e7b6e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c47b6eb, 23 },
    { 0xab97b6ec, 33 },
    { 0x6027b6ec, 99 },
    { 0x97e7b6eb, 33 },
    { 0xde17b6eb, 55 },
    { 0x4427b6f0, 90 },
    { 0xb067b6f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b27b6f4, 76 },
    { 0x4347b6f5, 90 },
    { 0x6a67b6f6, 43 },
    { 0xe357b6f7, 33 },
    { 0, 0 },
    { 0x5f47b6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8697b6fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c07b703, 99 },
    { 0x2537b704, 90 },
    { 0xb9d7b705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57b708, 90 },
    { 0, 0 },
    { 0x5c77b70a, 90 },
    { 0, 0 },
    { 0x9257b70c, 23 },
    { 0x6d7b70d, 61 },
    { 0x7ae7b70d, 80 },
    { 0x63a7b70d, 85 },
    { 0xf277b710, 99 },
    { 0x47b70f, 46 },
    { 0, 0 },
    { 0xe027b713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d97b716, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a07b719, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7b71e, 61 },
    { 0, 0 },
    { 0x7ec7b720, 46 },
    { 0x85d7b720, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf17b724, 33 },
    { 0xd0f7b725, 66 },
    { 0x1d27b724, 88 },
    { 0x39e7b727, 95 },
    { 0x9a07b727, 46 },
    { 0xf667b727, 93 },
    { 0x9d37b72a, 100 },
    { 0x51f7b72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa547b732, 61 },
    { 0, 0 },
    { 0x8ef7b734, 34 },
    { 0x1e57b734, 100 },
    { 0x5d57b735, 23 },
    { 0x6937b734, 76 },
    { 0x1f67b736, 22 },
    { 0, 0 },
    { 0x4b47b73a, 95 },
    { 0, 0 },
    { 0x10f7b73c, 90 },
    { 0, 0 },
    { 0xdfb7b73e, 98 },
    { 0x31d7b73f, 61 },
    { 0xe87b740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc147b747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107b74e, 23 },
    { 0x1127b74f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf057b753, 95 },
    { 0x8e57b754, 76 },
    { 0, 0 },
    { 0xc667b756, 33 },
    { 0, 0 },
    { 0x7d17b758, 90 },
    { 0x3677b759, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b7b75d, 95 },
    { 0x4c57b75d, 90 },
    { 0, 0 },
    { 0x3d57b760, 33 },
    { 0x6b17b761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f67b766, 93 },
    { 0xfcd7b766, 76 },
    { 0x5e7b768, 93 },
    { 0xcd27b769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c7b76c, 87 },
    { 0xdbe7b76d, 98 },
    { 0, 0 },
    { 0x7177b76f, 23 },
    { 0x8b17b770, 98 },
    { 0x5ee7b770, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7b775, 84 },
    { 0x1137b775, 33 },
    { 0, 0 },
    { 0xd5e7b778, 98 },
    { 0x2c07b779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17b780, 49 },
    { 0x4dd7b780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf947b785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9287b789, 34 },
    { 0x2667b789, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427b78f, 23 },
    { 0x1e27b78f, 46 },
    { 0xf6a7b791, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2277b794, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d7b798, 61 },
    { 0, 0 },
    { 0x4bf7b79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67b79f, 46 },
    { 0, 0 },
    { 0x7cc7b7a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827b7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b77b7a7, 66 },
    { 0xd877b7a8, 61 },
    { 0x99e7b7a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e7b7b0, 95 },
    { 0x8bb7b7b1, 93 },
    { 0x407b7b2, 93 },
    { 0xbec7b7b3, 76 },
    { 0x1a87b7b3, 96 },
    { 0x52d7b7b4, 76 },
    { 0xad57b7b2, 53 },
    { 0xf527b7b2, 55 },
    { 0xde87b7b8, 87 },
    { 0x9ee7b7b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a77b7bc, 34 },
    { 0, 0 },
    { 0xbc27b7be, 98 },
    { 0, 0 },
    { 0x98f7b7c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a7b7c7, 61 },
    { 0xb4e7b7c7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2717b7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb77b7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3557b7d7, 61 },
    { 0xcb67b7d7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d67b7de, 23 },
    { 0, 0 },
    { 0x9ea7b7e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7b7e3, 76 },
    { 0xacb7b7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e7b7e8, 85 },
    { 0xa1e7b7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b7ec, 98 },
    { 0x2187b7ec, 23 },
    { 0xe207b7ee, 87 },
    { 0x9e07b7ed, 55 },
    { 0x1a57b7ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb27b7f4, 33 },
    { 0x4eb7b7f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b7b7f8, 95 },
    { 0x2ea7b7f9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607b7fc, 76 },
    { 0, 0 },
    { 0x3727b7fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2717b804, 61 },
    { 0, 0 },
    { 0x4127b806, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547b809, 87 },
    { 0x45c7b809, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c7b80d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb17b815, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e7b81b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7b81f, 59 },
    { 0, 0 },
    { 0x5c27b821, 90 },
    { 0x2f97b822, 23 },
    { 0x977b823, 24 },
    { 0x77d7b824, 100 },
    { 0x20f7b825, 95 },
    { 0xed27b826, 52 },
    { 0x9b57b827, 90 },
    { 0x5987b828, 95 },
    { 0, 0 },
    { 0xd1f7b82a, 52 },
    { 0xce7b82a, 69 },
    { 0x7217b82c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647b830, 98 },
    { 0xf417b831, 23 },
    { 0xcb57b830, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87b835, 76 },
    { 0x4df7b835, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c7b839, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67b83c, 90 },
    { 0x7ed7b83d, 99 },
    { 0x9267b83c, 90 },
    { 0x47f7b83f, 55 },
    { 0xca77b840, 61 },
    { 0x2f47b83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d57b844, 55 },
    { 0, 0 },
    { 0x5837b846, 77 },
    { 0x1fe7b846, 33 },
    { 0xb637b847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b7b84d, 95 },
    { 0x4a77b84d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7b85a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567b861, 95 },
    { 0x5cc7b862, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff57b86a, 61 },
    { 0xafa7b86b, 90 },
    { 0x6177b86b, 75 },
    { 0x70a7b86c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367b870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597b873, 33 },
    { 0xb497b874, 23 },
    { 0, 0 },
    { 0xfad7b876, 61 },
    { 0x2f67b877, 76 },
    { 0x6dd7b878, 95 },
    { 0xce17b879, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c7b87c, 52 },
    { 0xf777b87d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7b880, 101 },
    { 0xc537b881, 100 },
    { 0xb5e7b881, 33 },
    { 0xb1c7b883, 43 },
    { 0, 0 },
    { 0xebf7b885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887b888, 98 },
    { 0, 0 },
    { 0xa4a7b88a, 95 },
    { 0xc8e7b88a, 99 },
    { 0xcdf7b88c, 90 },
    { 0, 0 },
    { 0x25a7b88e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27b891, 61 },
    { 0xa5a7b891, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba17b895, 61 },
    { 0x7417b896, 90 },
    { 0x3787b897, 76 },
    { 0x677b898, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6157b89f, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc477b8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c7b8a8, 61 },
    { 0, 0 },
    { 0xc427b8aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3737b8ae, 66 },
    { 0xcd97b8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b8b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167b8b6, 33 },
    { 0xc9f7b8b7, 98 },
    { 0, 0 },
    { 0x6da7b8b9, 43 },
    { 0xf437b8ba, 93 },
    { 0x61e7b8ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd57b8c6, 85 },
    { 0xab7b8c7, 61 },
    { 0xeb47b8c7, 33 },
    { 0xcad7b8c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b8cc, 33 },
    { 0xeb97b8cd, 55 },
    { 0x7c47b8ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7b8d3, 93 },
    { 0x86f7b8d4, 90 },
    { 0, 0 },
    { 0x86d7b8d6, 33 },
    { 0xd017b8d7, 61 },
    { 0x68d7b8d6, 34 },
    { 0xaa67b8d9, 34 },
    { 0x6e7b8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577b8df, 95 },
    { 0xe117b8df, 98 },
    { 0x9f87b8e1, 98 },
    { 0x9f77b8e2, 85 },
    { 0xd8f7b8e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617b8e6, 98 },
    { 0, 0 },
    { 0x6997b8e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127b8ed, 78 },
    { 0, 0 },
    { 0xaaa7b8ef, 98 },
    { 0, 0 },
    { 0xc8d7b8f1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8377b8f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b8fd, 23 },
    { 0xbd77b8fe, 33 },
    { 0xde17b8ff, 34 },
    { 0x84a7b8fd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87b903, 33 },
    { 0, 0 },
    { 0x2787b905, 90 },
    { 0, 0 },
    { 0x5487b907, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c77b90b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc017b90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc7b914, 33 },
    { 0x6eb7b915, 90 },
    { 0xf3a7b915, 77 },
    { 0x6217b917, 55 },
    { 0x6987b918, 55 },
    { 0, 0 },
    { 0x7547b91a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717b91e, 33 },
    { 0x4aa7b91e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec77b923, 87 },
    { 0xbc57b923, 34 },
    { 0x38e7b925, 52 },
    { 0xc677b923, 33 },
    { 0xb7b924, 55 },
    { 0xf17b925, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb7b92b, 91 },
    { 0x5d37b92b, 99 },
    { 0x6bc7b92c, 40 },
    { 0xfc77b92e, 90 },
    { 0xfa37b92c, 85 },
    { 0x9f67b930, 23 },
    { 0x6677b931, 55 },
    { 0x11b7b932, 33 },
    { 0xa657b92e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb837b937, 61 },
    { 0x1347b938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7787b93c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce97b944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417b947, 46 },
    { 0xe947b948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1747b952, 66 },
    { 0, 0 },
    { 0xc6c7b954, 33 },
    { 0x76d7b954, 61 },
    { 0xef7b956, 66 },
    { 0x5d77b957, 43 },
    { 0, 0 },
    { 0x8827b959, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d7b95c, 100 },
    { 0x7bb7b95d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf337b964, 98 },
    { 0, 0 },
    { 0x2467b966, 98 },
    { 0xf37b967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b57b96b, 95 },
    { 0xcf07b96b, 93 },
    { 0x7067b96d, 77 },
    { 0xe977b96d, 90 },
    { 0x3337b96d, 34 },
    { 0x90f7b96f, 55 },
    { 0x12f7b971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6157b977, 90 },
    { 0xfcb7b978, 99 },
    { 0xad77b978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x157b97c, 98 },
    { 0x6747b97c, 90 },
    { 0xabc7b97c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e67b985, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a97b98b, 91 },
    { 0xc037b98b, 90 },
    { 0x9517b98d, 61 },
    { 0xfef7b98e, 100 },
    { 0xf707b98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf447b993, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c7b996, 95 },
    { 0, 0 },
    { 0xf567b998, 61 },
    { 0x91b7b998, 76 },
    { 0x92e7b999, 52 },
    { 0x9b77b999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a7b9a3, 55 },
    { 0x2e27b9a4, 34 },
    { 0x2907b9a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367b9a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf217b9ae, 61 },
    { 0, 0 },
    { 0xb0c7b9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7b9b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7367b9b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7b9ba, 33 },
    { 0x15d7b9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e17b9bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7b9c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c27b9c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d27b9ca, 90 },
    { 0x1c87b9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa097b9d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc67b9df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa657b9e8, 95 },
    { 0, 0 },
    { 0x5d57b9ea, 95 },
    { 0x5d27b9eb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb7b9ef, 75 },
    { 0, 0 },
    { 0xf977b9f1, 91 },
    { 0x82a7b9f1, 76 },
    { 0x8607b9f1, 23 },
    { 0, 0 },
    { 0xd497b9f5, 34 },
    { 0xc277b9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf247b9f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1127b9fd, 90 },
    { 0x6487b9fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9317ba03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407ba07, 55 },
    { 0x5d27ba08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x967ba0b, 98 },
    { 0xdb67ba0c, 61 },
    { 0, 0 },
    { 0x6da7ba0e, 85 },
    { 0xbb37ba0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37ba14, 43 },
    { 0x1067ba15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f97ba1b, 90 },
    { 0xd6e7ba1c, 23 },
    { 0, 0 },
    { 0x9b87ba1e, 34 },
    { 0x5fd7ba1f, 66 },
    { 0xa5f7ba20, 33 },
    { 0xc337ba21, 95 },
    { 0x54e7ba22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae7ba28, 85 },
    { 0xd5e7ba28, 99 },
    { 0x1dc7ba2a, 98 },
    { 0, 0 },
    { 0xff07ba2c, 78 },
    { 0x1d67ba2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957ba31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7ba36, 61 },
    { 0, 0 },
    { 0x18a7ba38, 90 },
    { 0x7e47ba39, 59 },
    { 0x8a47ba3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ba40, 61 },
    { 0x7347ba41, 90 },
    { 0x8557ba41, 90 },
    { 0x5fe7ba43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba7ba46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e87ba50, 98 },
    { 0, 0 },
    { 0x6a17ba52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac47ba55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77ba58, 87 },
    { 0x4bf7ba59, 85 },
    { 0x4837ba5a, 65 },
    { 0, 0 },
    { 0x16d7ba5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3547ba5f, 97 },
    { 0, 0 },
    { 0x4787ba61, 49 },
    { 0xf457ba62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557ba65, 33 },
    { 0x6c27ba66, 61 },
    { 0xe337ba67, 34 },
    { 0x277ba66, 98 },
    { 0, 0 },
    { 0x8c77ba6a, 95 },
    { 0, 0 },
    { 0x1aa7ba6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c7ba76, 90 },
    { 0x89e7ba77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb7ba7b, 61 },
    { 0xc707ba7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb87ba7f, 61 },
    { 0x6f7ba80, 34 },
    { 0, 0 },
    { 0xabb7ba82, 46 },
    { 0xd297ba83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d7ba8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b7ba97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b7ba9a, 98 },
    { 0x6e27ba9b, 61 },
    { 0x82f7ba9a, 61 },
    { 0xece7ba9d, 61 },
    { 0x4797ba9e, 33 },
    { 0, 0 },
    { 0xc0a7baa0, 90 },
    { 0, 0 },
    { 0xf377baa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc137baa5, 33 },
    { 0xac77baa5, 90 },
    { 0, 0 },
    { 0xbd67baa8, 76 },
    { 0x8ce7baa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d7baaf, 55 },
    { 0x6707bab0, 98 },
    { 0x1ad7bab1, 61 },
    { 0xb6a7baaf, 33 },
    { 0xe927bab3, 43 },
    { 0xe227bab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7bab8, 61 },
    { 0, 0 },
    { 0x3f77baba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8897babe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47bac9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc7bacc, 42 },
    { 0x2807bacd, 95 },
    { 0x867bacd, 23 },
    { 0, 0 },
    { 0xba77bad0, 67 },
    { 0, 0 },
    { 0xa9f7bad2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7bad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4387badb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c7bae3, 98 },
    { 0x4c77bae3, 67 },
    { 0, 0 },
    { 0x49f7bae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3687bae9, 12 },
    { 0xdfe7baea, 85 },
    { 0x6a57baeb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb7baee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67baf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e7bafa, 33 },
    { 0xa617bafb, 64 },
    { 0xd557bafc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7bb02, 90 },
    { 0x9327bb03, 33 },
    { 0x8bd7bb02, 93 },
    { 0x94d7bb02, 90 },
    { 0x28f7bb06, 90 },
    { 0, 0 },
    { 0x13f7bb08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb117bb0b, 95 },
    { 0, 0 },
    { 0x3297bb0d, 91 },
    { 0xcce7bb0e, 43 },
    { 0x61c7bb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77bb13, 33 },
    { 0x6187bb14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab7bb17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda87bb1c, 90 },
    { 0, 0 },
    { 0x9347bb1e, 43 },
    { 0xa387bb1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c7bb25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8367bb2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9677bb30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7bb33, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x3007bb36, 33 },
    { 0x3927bb36, 85 },
    { 0x4da7bb37, 55 },
    { 0xfb7bb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf647bb48, 99 },
    { 0xcf27bb49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7bb4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8647bb50, 98 },
    { 0x2f57bb51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7bb57, 33 },
    { 0xd2e7bb58, 23 },
    { 0x7967bb59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7117bb60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc7bb64, 55 },
    { 0xebc7bb65, 75 },
    { 0x4597bb66, 90 },
    { 0xfce7bb67, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297bb71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce07bb79, 33 },
    { 0, 0 },
    { 0x64b7bb7b, 87 },
    { 0, 0 },
    { 0x7cf7bb7d, 90 },
    { 0xa707bb7e, 61 },
    { 0x8bf7bb7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547bb89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b7bb8e, 76 },
    { 0x7587bb8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607bb92, 34 },
    { 0x6c37bb93, 81 },
    { 0, 0 },
    { 0xe517bb95, 78 },
    { 0x2e27bb96, 77 },
    { 0, 0 },
    { 0x92e7bb98, 23 },
    { 0x2457bb98, 23 },
    { 0, 0 },
    { 0x2607bb9b, 24 },
    { 0x557bb9c, 61 },
    { 0xb2d7bb9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7bba6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c7bbac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9987bbb3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f7bbb7, 52 },
    { 0x417bbb8, 23 },
    { 0, 0 },
    { 0x61f7bbba, 85 },
    { 0x2bf7bbbb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4537bbc2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7bbcb, 77 },
    { 0xa2f7bbcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a7bbd2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b7bbd6, 55 },
    { 0x72a7bbd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5667bbda, 90 },
    { 0xda17bbda, 99 },
    { 0x5c57bbdc, 34 },
    { 0x54d7bbdd, 67 },
    { 0xcf87bbda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27bbe2, 90 },
    { 0, 0 },
    { 0xe9b7bbe4, 90 },
    { 0x72d7bbe5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7bbea, 60 },
    { 0x7227bbeb, 90 },
    { 0, 0 },
    { 0xafd7bbed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a7bbf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7bbf6, 96 },
    { 0x1f37bbf7, 98 },
    { 0xcef7bbf8, 98 },
    { 0x9c67bbf6, 83 },
    { 0x9d57bbf7, 69 },
    { 0x44f7bbfb, 66 },
    { 0, 0 },
    { 0x39c7bbfd, 46 },
    { 0, 0 },
    { 0x3f87bbff, 98 },
    { 0, 0 },
    { 0xc317bc01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa97bc05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a57bc09, 46 },
    { 0, 0 },
    { 0xde57bc0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc337bc0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f7bc12, 23 },
    { 0x8887bc13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457bc18, 87 },
    { 0, 0 },
    { 0x45e7bc1a, 98 },
    { 0, 0 },
    { 0xdbd7bc1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe847bc22, 34 },
    { 0x6b77bc23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d7bc27, 77 },
    { 0x7417bc28, 33 },
    { 0xccc7bc27, 76 },
    { 0x25f7bc2a, 85 },
    { 0xa987bc28, 95 },
    { 0x9597bc2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc037bc34, 17 },
    { 0, 0 },
    { 0xe157bc36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd027bc39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b7bc42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f7bc46, 98 },
    { 0x8797bc46, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7bc4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7bc53, 100 },
    { 0, 0 },
    { 0x5b17bc55, 55 },
    { 0, 0 },
    { 0x7757bc57, 90 },
    { 0x2f97bc57, 66 },
    { 0x8327bc57, 90 },
    { 0x197bc5a, 33 },
    { 0x5cb7bc5a, 76 },
    { 0x17bc5c, 90 },
    { 0, 0 },
    { 0x39e7bc5e, 63 },
    { 0x1967bc5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb7bc62, 95 },
    { 0x14c7bc63, 23 },
    { 0xfde7bc64, 55 },
    { 0, 0 },
    { 0x93f7bc66, 95 },
    { 0, 0 },
    { 0x5a7bc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007bc6c, 65 },
    { 0x1597bc6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa397bc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7bc74, 95 },
    { 0x427bc75, 33 },
    { 0x6397bc76, 90 },
    { 0xdcb7bc77, 33 },
    { 0, 0 },
    { 0xbed7bc79, 61 },
    { 0x9a07bc79, 33 },
    { 0x1fd7bc79, 90 },
    { 0xbbf7bc79, 76 },
    { 0, 0 },
    { 0x6267bc7e, 76 },
    { 0x3f27bc7f, 76 },
    { 0x6007bc80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef7bc83, 61 },
    { 0x9577bc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d7bc8d, 90 },
    { 0, 0 },
    { 0x4937bc8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c07bc95, 46 },
    { 0xf577bc95, 93 },
    { 0x4797bc97, 87 },
    { 0x8ae7bc96, 93 },
    { 0, 0 },
    { 0xfb57bc9a, 90 },
    { 0, 0 },
    { 0x90a7bc9c, 61 },
    { 0, 0 },
    { 0x8477bc9e, 34 },
    { 0x4897bc9f, 100 },
    { 0x7f97bca0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77bca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747bca7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787bcab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957bcae, 88 },
    { 0x9ca7bcaf, 99 },
    { 0, 0 },
    { 0x7d07bcb1, 77 },
    { 0xcec7bcb2, 66 },
    { 0xc9e7bcb3, 34 },
    { 0xd2b7bcb1, 33 },
    { 0, 0 },
    { 0x8d17bcb6, 95 },
    { 0x12b7bcb7, 90 },
    { 0x65c7bcb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d47bcbe, 90 },
    { 0, 0 },
    { 0xfb57bcc0, 98 },
    { 0xbda7bcc1, 40 },
    { 0xd97bcc2, 61 },
    { 0x9a27bcc1, 61 },
    { 0, 0 },
    { 0xb307bcc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb737bcc9, 13 },
    { 0x6187bcca, 46 },
    { 0x7ce7bccb, 95 },
    { 0, 0 },
    { 0xe4c7bccd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7bcd0, 61 },
    { 0xbb7bcd1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd17bcd5, 55 },
    { 0, 0 },
    { 0x2927bcd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67bcda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8527bcde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4927bce2, 17 },
    { 0xca7bce3, 43 },
    { 0, 0 },
    { 0x8167bce5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe037bce9, 90 },
    { 0xbb37bcea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07bced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c57bcf1, 98 },
    { 0xf757bcf2, 90 },
    { 0, 0 },
    { 0xb947bcf4, 61 },
    { 0, 0 },
    { 0x5097bcf6, 67 },
    { 0, 0 },
    { 0xe867bcf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8547bcfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc7bd00, 96 },
    { 0, 0 },
    { 0x5a37bd02, 61 },
    { 0xa5e7bd03, 95 },
    { 0x85c7bd04, 95 },
    { 0x9f67bd03, 55 },
    { 0x1757bd03, 76 },
    { 0x60d7bd04, 80 },
    { 0xa177bd08, 84 },
    { 0x3427bd09, 76 },
    { 0x35d7bd07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5877bd0d, 76 },
    { 0x7cc7bd0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77bd12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257bd18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb767bd21, 90 },
    { 0xde97bd22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3347bd25, 55 },
    { 0xc707bd25, 75 },
    { 0xb907bd27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377bd2c, 61 },
    { 0x17f7bd2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97bd31, 33 },
    { 0xc947bd32, 77 },
    { 0x3f97bd31, 61 },
    { 0x19a7bd31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a17bd37, 95 },
    { 0, 0 },
    { 0x7597bd39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2367bd3d, 98 },
    { 0, 0 },
    { 0x6b07bd3f, 95 },
    { 0x6ac7bd40, 8 },
    { 0x5397bd3f, 33 },
    { 0, 0 },
    { 0xf657bd43, 67 },
    { 0xec87bd44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c27bd4b, 55 },
    { 0xb557bd4b, 93 },
    { 0xda27bd4d, 98 },
    { 0xdd17bd4d, 98 },
    { 0x79f7bd4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1627bd54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17bd57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c7bd5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7bd5d, 43 },
    { 0, 0 },
    { 0x3337bd5f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7bd65, 98 },
    { 0, 0 },
    { 0x8d97bd67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f7bd6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7bd74, 52 },
    { 0x6097bd75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc977bd78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9067bd7d, 34 },
    { 0, 0 },
    { 0xc717bd7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9587bd84, 95 },
    { 0x6077bd85, 85 },
    { 0xc77bd84, 33 },
    { 0x6867bd85, 90 },
    { 0xcc67bd88, 23 },
    { 0, 0 },
    { 0x9427bd8a, 78 },
    { 0, 0 },
    { 0x5557bd8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2537bd91, 88 },
    { 0x7537bd91, 90 },
    { 0, 0 },
    { 0x5d17bd94, 34 },
    { 0x5ae7bd94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2227bd99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817bd9e, 90 },
    { 0, 0 },
    { 0x15e7bda0, 98 },
    { 0, 0 },
    { 0xfa27bda2, 65 },
    { 0x1317bda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917bda6, 90 },
    { 0x5fe7bda7, 90 },
    { 0xad7bda8, 90 },
    { 0, 0 },
    { 0x8987bdaa, 93 },
    { 0, 0 },
    { 0x87b7bdac, 90 },
    { 0x9447bdad, 95 },
    { 0xfd97bdae, 23 },
    { 0x9c37bdac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7097bdb6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d7bdb9, 33 },
    { 0x4037bdba, 33 },
    { 0xf2d7bdba, 61 },
    { 0x7b27bdbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b7bdc6, 23 },
    { 0x407bdc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7bdcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5517bdd2, 61 },
    { 0, 0 },
    { 0x5167bdd4, 85 },
    { 0, 0 },
    { 0x1a77bdd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7bddd, 84 },
    { 0x5207bdde, 90 },
    { 0xab97bddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d97bde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147bdee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb887bdf1, 43 },
    { 0xaa17bdf1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7bdf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7bdf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7bdfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3907be02, 76 },
    { 0, 0 },
    { 0x7727be04, 55 },
    { 0, 0 },
    { 0x657be06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e7be09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf217be0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5617be15, 54 },
    { 0x1f77be16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd117be1a, 80 },
    { 0x8947be1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd7be23, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17be29, 33 },
    { 0, 0 },
    { 0x8577be2b, 46 },
    { 0x9177be2c, 61 },
    { 0xf477be2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe7be34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c7be39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8847be3c, 61 },
    { 0x9077be3d, 90 },
    { 0xd877be3e, 55 },
    { 0x2c27be3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce97be42, 99 },
    { 0xbd87be43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067be48, 61 },
    { 0xa6f7be49, 69 },
    { 0, 0 },
    { 0x2517be4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7be4f, 55 },
    { 0xca67be50, 98 },
    { 0x2537be51, 67 },
    { 0x83d7be4f, 95 },
    { 0xd2a7be53, 23 },
    { 0x57c7be54, 34 },
    { 0x3ee7be54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc37be5b, 95 },
    { 0xdd07be5b, 76 },
    { 0xd617be5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a57be64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597be68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6457be6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f7be77, 61 },
    { 0, 0 },
    { 0x9517be79, 93 },
    { 0, 0 },
    { 0x7d77be7b, 76 },
    { 0xdb47be7c, 95 },
    { 0xbee7be7d, 95 },
    { 0x6727be7e, 90 },
    { 0x2877be7d, 23 },
    { 0xb187be7f, 90 },
    { 0xba7be7d, 61 },
    { 0x8f27be80, 46 },
    { 0x9de7be7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7be89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e7be90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f87be98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27be9c, 90 },
    { 0x1567be9c, 87 },
    { 0x81a7be9c, 76 },
    { 0x3cf7be9f, 90 },
    { 0x3377bea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e7bea5, 98 },
    { 0x2707bea6, 23 },
    { 0xe5a7bea7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527beab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7beae, 90 },
    { 0xb7c7beaf, 66 },
    { 0, 0 },
    { 0xff87beb1, 95 },
    { 0, 0 },
    { 0x1d77beb3, 2 },
    { 0, 0 },
    { 0x9907beb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b7bebc, 99 },
    { 0xb427bebc, 76 },
    { 0x8b57bebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507bec1, 33 },
    { 0, 0 },
    { 0x197bec3, 61 },
    { 0, 0 },
    { 0xb6f7bec5, 76 },
    { 0x5797bec6, 33 },
    { 0xa4b7bec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac7beca, 99 },
    { 0, 0 },
    { 0x2b17becc, 46 },
    { 0, 0 },
    { 0xfb87bece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1607bed1, 75 },
    { 0xe257bed2, 90 },
    { 0xacf7bed1, 23 },
    { 0, 0 },
    { 0x5b37bed5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x947bed8, 55 },
    { 0x6d27bed9, 23 },
    { 0x5767bed8, 23 },
    { 0xa297bedb, 33 },
    { 0xcff7bed8, 91 },
    { 0, 0 },
    { 0x3ad7bede, 33 },
    { 0, 0 },
    { 0xf6e7bee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f87bee6, 87 },
    { 0x55d7bee7, 33 },
    { 0x58d7bee8, 33 },
    { 0xf27bee7, 100 },
    { 0xeb7bee8, 90 },
    { 0xf77bee8, 61 },
    { 0x2aa7beec, 83 },
    { 0xd137beec, 23 },
    { 0x4ed7bee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc07bef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf7bef5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77bef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd7bf00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd857bf08, 44 },
    { 0xf3e7bf09, 95 },
    { 0xb357bf08, 90 },
    { 0x30f7bf09, 20 },
    { 0xe7e7bf0c, 93 },
    { 0xf4a7bf0c, 55 },
    { 0x6807bf0e, 91 },
    { 0, 0 },
    { 0x2397bf10, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67bf13, 33 },
    { 0x3057bf14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7bf1b, 23 },
    { 0, 0 },
    { 0x22e7bf1d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a07bf20, 43 },
    { 0x1db7bf20, 90 },
    { 0x2f7bf22, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de7bf2b, 59 },
    { 0x2c7bf2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6147bf31, 100 },
    { 0, 0 },
    { 0x7c57bf33, 76 },
    { 0, 0 },
    { 0x23f7bf35, 95 },
    { 0xe267bf36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4667bf3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1477bf3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617bf49, 46 },
    { 0x9fe7bf49, 90 },
    { 0xaba7bf4b, 76 },
    { 0x2fd7bf49, 100 },
    { 0, 0 },
    { 0x8007bf4e, 61 },
    { 0, 0 },
    { 0xcd67bf50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d7bf53, 95 },
    { 0x36f7bf53, 55 },
    { 0, 0 },
    { 0xbb97bf56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb57bf5a, 61 },
    { 0x3877bf5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057bf5f, 23 },
    { 0xb7bf60, 90 },
    { 0x5037bf61, 61 },
    { 0x8f17bf61, 90 },
    { 0xcd97bf63, 65 },
    { 0x9027bf63, 88 },
    { 0x27e7bf65, 78 },
    { 0xb187bf66, 61 },
    { 0xb2b7bf64, 61 },
    { 0x8937bf67, 61 },
    { 0xf017bf67, 43 },
    { 0xdda7bf6a, 93 },
    { 0x5d7bf64, 93 },
    { 0x5767bf6c, 32 },
    { 0xcbc7bf6c, 43 },
    { 0x807bf6e, 33 },
    { 0xcb17bf64, 90 },
    { 0x1a17bf70, 98 },
    { 0xf807bf60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577bf75, 23 },
    { 0x5737bf76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf79, 33 },
    { 0x9997bf79, 33 },
    { 0, 0 },
    { 0xfa57bf7c, 33 },
    { 0x26e7bf7d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d7bf81, 33 },
    { 0, 0 },
    { 0x3127bf83, 33 },
    { 0xd527bf83, 66 },
    { 0, 0 },
    { 0xe507bf86, 98 },
    { 0xf6b7bf87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2367bf8a, 43 },
    { 0, 0 },
    { 0x9d7bf8c, 95 },
    { 0, 0 },
    { 0x57a7bf8e, 34 },
    { 0, 0 },
    { 0xcee7bf90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb7bf95, 46 },
    { 0, 0 },
    { 0x37b7bf97, 95 },
    { 0x1d17bf98, 90 },
    { 0x57bf99, 23 },
    { 0xf957bf9a, 61 },
    { 0xe427bf99, 33 },
    { 0x3407bf97, 87 },
    { 0x99a7bf9d, 65 },
    { 0x9f27bf9a, 55 },
    { 0x91f7bf9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb7bfa3, 95 },
    { 0xf117bfa3, 90 },
    { 0x6c7bfa4, 67 },
    { 0x3427bfa6, 34 },
    { 0, 0 },
    { 0xe117bfa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7bfb0, 98 },
    { 0, 0 },
    { 0xc667bfb2, 85 },
    { 0x7cd7bfb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa717bfb8, 55 },
    { 0x5a97bfb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e7bfbd, 46 },
    { 0x9cc7bfbe, 33 },
    { 0, 0 },
    { 0x4317bfc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a7bfc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9477bfc6, 33 },
    { 0xecb7bfc6, 84 },
    { 0, 0 },
    { 0x58e7bfc9, 43 },
    { 0xd027bfca, 90 },
    { 0xc1c7bfcb, 55 },
    { 0x4f67bfcc, 61 },
    { 0xbff7bfcb, 61 },
    { 0xd8a7bfce, 98 },
    { 0x10f7bfcf, 88 },
    { 0x3fb7bfcf, 33 },
    { 0x1267bfd1, 90 },
    { 0x4be7bfd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4717bfd5, 43 },
    { 0xf837bfd6, 100 },
    { 0x77f7bfd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7bfda, 98 },
    { 0x4457bfda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc257bfde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87bfe2, 52 },
    { 0xc9e7bfe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f77bfe6, 23 },
    { 0, 0 },
    { 0xd67bfe8, 94 },
    { 0, 0 },
    { 0x9117bfea, 33 },
    { 0xa7bfeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797bff4, 23 },
    { 0, 0 },
    { 0x6357bff6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047bffb, 33 },
    { 0, 0 },
    { 0x3a07bffd, 81 },
    { 0, 0 },
    { 0xee87bfff, 67 },
    { 0xa1c7bfff, 76 },
    { 0x3087bfff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4617c005, 76 },
    { 0x4d57c006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3337c00b, 33 },
    { 0x3ca7c00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357c012, 93 },
    { 0x7597c013, 17 },
    { 0xbf87c013, 55 },
    { 0x2c57c015, 93 },
    { 0x52c7c012, 43 },
    { 0xb457c017, 61 },
    { 0xee67c016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba57c01c, 87 },
    { 0, 0 },
    { 0xecf7c01e, 61 },
    { 0xdee7c01e, 90 },
    { 0x9447c020, 53 },
    { 0xed7c020, 34 },
    { 0x5917c020, 61 },
    { 0xd6b7c023, 61 },
    { 0, 0 },
    { 0x12a7c025, 46 },
    { 0, 0 },
    { 0x20a7c027, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa387c02e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7137c032, 33 },
    { 0x4447c033, 90 },
    { 0xb9b7c033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c7c038, 98 },
    { 0x26d7c038, 23 },
    { 0xb547c038, 61 },
    { 0xa287c03b, 98 },
    { 0x8f97c03b, 55 },
    { 0x3917c03c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a7c041, 96 },
    { 0, 0 },
    { 0x1c17c043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87c048, 78 },
    { 0x32c7c049, 99 },
    { 0x6917c049, 52 },
    { 0xbd67c049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7c050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c054, 33 },
    { 0xb777c055, 93 },
    { 0x4667c056, 66 },
    { 0, 0 },
    { 0x1e7c058, 98 },
    { 0xb107c059, 52 },
    { 0x6367c05a, 98 },
    { 0xbb7c05b, 90 },
    { 0xd587c05b, 98 },
    { 0x2177c05d, 93 },
    { 0xd5e7c058, 90 },
    { 0x2737c05c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b7c066, 91 },
    { 0xef97c066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb257c06d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c071, 67 },
    { 0x57f7c072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa877c076, 90 },
    { 0xab47c077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7c07e, 33 },
    { 0, 0 },
    { 0xa2e7c080, 76 },
    { 0, 0 },
    { 0x4197c082, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a7c08b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b7c090, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c094, 90 },
    { 0, 0 },
    { 0x52a7c096, 33 },
    { 0, 0 },
    { 0xf3d7c098, 90 },
    { 0xa987c099, 93 },
    { 0x73f7c098, 23 },
    { 0xbc47c098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787c0a4, 55 },
    { 0x95b7c0a5, 55 },
    { 0x4577c0a6, 90 },
    { 0, 0 },
    { 0xbf67c0a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377c0ac, 93 },
    { 0x1d7c0ad, 95 },
    { 0x96b7c0ae, 94 },
    { 0x8fe7c0af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac57c0b3, 43 },
    { 0x1017c0b4, 76 },
    { 0xcc17c0b4, 95 },
    { 0xd757c0b6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f67c0b9, 33 },
    { 0xae27c0ba, 33 },
    { 0, 0 },
    { 0x2a07c0bc, 88 },
    { 0, 0 },
    { 0x4a57c0be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3997c0c1, 95 },
    { 0x6807c0c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67c0c7, 95 },
    { 0xafe7c0c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1997c0cd, 76 },
    { 0, 0 },
    { 0x42d7c0cf, 55 },
    { 0x9e87c0d0, 90 },
    { 0x5297c0d0, 46 },
    { 0x7167c0d2, 61 },
    { 0x9cb7c0cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe07c0e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7637c0ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6707c0fb, 90 },
    { 0xd8b7c0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017c102, 99 },
    { 0, 0 },
    { 0xb417c104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9187c107, 52 },
    { 0xf2c7c108, 55 },
    { 0xd027c109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527c10d, 46 },
    { 0x3a77c10e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe707c111, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f7c11f, 46 },
    { 0xe777c120, 93 },
    { 0x89d7c11f, 33 },
    { 0, 0 },
    { 0x7b87c123, 90 },
    { 0x5f67c124, 61 },
    { 0x89a7c125, 23 },
    { 0x7e27c126, 46 },
    { 0x15e7c127, 46 },
    { 0x9367c126, 33 },
    { 0x39e7c129, 76 },
    { 0x16a7c12a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367c12d, 90 },
    { 0xee7c12e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807c135, 23 },
    { 0xce17c136, 34 },
    { 0xefe7c137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137c13a, 23 },
    { 0xe817c13b, 98 },
    { 0x3d47c13a, 95 },
    { 0, 0 },
    { 0x9627c13e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f7c145, 23 },
    { 0x96d7c146, 81 },
    { 0x8e27c147, 23 },
    { 0xdee7c148, 91 },
    { 0x1377c149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7c14c, 95 },
    { 0, 0 },
    { 0x12c7c14e, 52 },
    { 0x64c7c14f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c153, 87 },
    { 0x60e7c153, 98 },
    { 0x7f37c155, 98 },
    { 0x2647c156, 90 },
    { 0xab47c155, 38 },
    { 0x4207c156, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57c161, 84 },
    { 0xb947c162, 52 },
    { 0, 0 },
    { 0x5417c164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa017c168, 98 },
    { 0x8797c168, 90 },
    { 0, 0 },
    { 0x7f77c16b, 90 },
    { 0x2e77c16b, 90 },
    { 0xeb57c16b, 23 },
    { 0x9237c16d, 84 },
    { 0xb17c16d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047c174, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe7c17b, 61 },
    { 0, 0 },
    { 0xc9e7c17d, 98 },
    { 0x9247c17e, 99 },
    { 0xe6f7c17e, 23 },
    { 0x8717c17d, 55 },
    { 0, 0 },
    { 0x6037c182, 84 },
    { 0xca77c183, 99 },
    { 0x4827c184, 66 },
    { 0xa3e7c183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f7c188, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1357c18e, 95 },
    { 0xab47c18f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f7c193, 100 },
    { 0x73f7c193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf507c197, 98 },
    { 0, 0 },
    { 0x1887c199, 61 },
    { 0x26e7c19a, 78 },
    { 0xe4b7c19a, 78 },
    { 0xb2a7c19c, 23 },
    { 0xa7b7c19b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd357c1a3, 77 },
    { 0, 0 },
    { 0xf5a7c1a5, 85 },
    { 0x7c07c1a6, 61 },
    { 0x3227c1a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb27c1ad, 55 },
    { 0x6077c1ae, 90 },
    { 0, 0 },
    { 0x7997c1b0, 61 },
    { 0xf7c7c1b1, 43 },
    { 0, 0 },
    { 0xf0b7c1b3, 88 },
    { 0x8cf7c1b3, 90 },
    { 0x6a37c1b4, 88 },
    { 0, 0 },
    { 0xf0c7c1b7, 90 },
    { 0x9517c1b7, 61 },
    { 0x3ff7c1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa127c1c1, 55 },
    { 0, 0 },
    { 0xae07c1c3, 90 },
    { 0x6597c1c4, 91 },
    { 0xa827c1c3, 34 },
    { 0, 0 },
    { 0xbb37c1c7, 23 },
    { 0x4257c1c7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad47c1cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3367c1dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8527c1e1, 91 },
    { 0x6517c1e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67c1e5, 99 },
    { 0x5847c1e6, 61 },
    { 0x50e7c1e7, 77 },
    { 0, 0 },
    { 0xd477c1e9, 95 },
    { 0x42e7c1ea, 66 },
    { 0x31a7c1ea, 40 },
    { 0x9457c1ea, 76 },
    { 0, 0 },
    { 0xf347c1ee, 98 },
    { 0, 0 },
    { 0xff77c1f0, 98 },
    { 0, 0 },
    { 0x2327c1f2, 34 },
    { 0, 0 },
    { 0x9277c1f4, 33 },
    { 0xdc47c1f4, 81 },
    { 0x2197c1f6, 90 },
    { 0x4a87c1f7, 76 },
    { 0, 0 },
    { 0xc5a7c1f9, 46 },
    { 0, 0 },
    { 0x7d17c1fb, 33 },
    { 0, 0 },
    { 0x17c7c1fd, 61 },
    { 0, 0 },
    { 0xef37c1ff, 43 },
    { 0x7107c200, 84 },
    { 0x8987c201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7c205, 76 },
    { 0x4a7c206, 98 },
    { 0x6ff7c206, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf907c210, 33 },
    { 0x8907c211, 23 },
    { 0xcca7c212, 87 },
    { 0x68e7c210, 90 },
    { 0xc477c214, 61 },
    { 0x46f7c215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc7c21e, 61 },
    { 0, 0 },
    { 0x177c220, 93 },
    { 0x9dc7c220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9007c225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e7c229, 68 },
    { 0x3e87c229, 76 },
    { 0xc0c7c22b, 90 },
    { 0x2307c22b, 90 },
    { 0x3a57c22b, 83 },
    { 0x83b7c22d, 90 },
    { 0x66a7c22f, 90 },
    { 0xead7c22e, 90 },
    { 0xe7b7c231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4917c235, 61 },
    { 0x8c37c236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2537c23a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7c23d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e37c245, 23 },
    { 0x2b37c245, 68 },
    { 0xb247c247, 90 },
    { 0x8857c248, 90 },
    { 0x30b7c247, 55 },
    { 0xc727c24a, 81 },
    { 0xd777c246, 93 },
    { 0x1587c24c, 23 },
    { 0x9817c24c, 76 },
    { 0x9527c24e, 93 },
    { 0x6657c248, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa7c256, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e7c25b, 23 },
    { 0x6837c25c, 93 },
    { 0x2f7c25d, 98 },
    { 0xa407c25b, 55 },
    { 0x56c7c25d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe57c264, 97 },
    { 0xd107c265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a7c27a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5797c27e, 95 },
    { 0, 0 },
    { 0xc57c280, 61 },
    { 0x4d7c280, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c284, 95 },
    { 0xd797c285, 93 },
    { 0x8bc7c286, 90 },
    { 0, 0 },
    { 0xbdb7c288, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b87c28c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe97c290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf27c294, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9387c297, 55 },
    { 0x1557c298, 90 },
    { 0x97c298, 90 },
    { 0x4767c29a, 98 },
    { 0x57c7c29b, 55 },
    { 0x18b7c29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd957c2a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x137c2a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe987c2b1, 52 },
    { 0x4e17c2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca77c2b8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7c2bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7987c2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f7c2c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a7c2c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b77c2ca, 61 },
    { 0x8e47c2cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4877c2d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b7c2d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc797c2d8, 43 },
    { 0xda17c2d9, 52 },
    { 0x3687c2da, 65 },
    { 0x6d7c2db, 46 },
    { 0x3e37c2db, 90 },
    { 0, 0 },
    { 0x2f7c2de, 99 },
    { 0, 0 },
    { 0x1fe7c2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b27c2e3, 76 },
    { 0xad7c2e3, 25 },
    { 0x3167c2e5, 90 },
    { 0xe287c2e6, 77 },
    { 0, 0 },
    { 0x5107c2e8, 61 },
    { 0x4be7c2e8, 95 },
    { 0x2b47c2ea, 46 },
    { 0x6947c2eb, 98 },
    { 0x8e57c2ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff17c2ef, 84 },
    { 0, 0 },
    { 0x35d7c2f1, 23 },
    { 0x6987c2f2, 90 },
    { 0xb527c2f3, 98 },
    { 0xda47c2f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe7c2f8, 84 },
    { 0x47e7c2f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf77c2fc, 66 },
    { 0x5097c2fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a7c302, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7c305, 46 },
    { 0xd547c306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b97c30a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab7c30d, 40 },
    { 0x9d07c30e, 55 },
    { 0x6ce7c30f, 93 },
    { 0xd1b7c30f, 61 },
    { 0x5477c30f, 90 },
    { 0, 0 },
    { 0xcd17c313, 90 },
    { 0xed97c314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f97c31a, 90 },
    { 0xbed7c31a, 76 },
    { 0, 0 },
    { 0x15c7c31d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7c320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c325, 90 },
    { 0x2107c326, 98 },
    { 0, 0 },
    { 0xc347c328, 98 },
    { 0x3b87c329, 88 },
    { 0xc537c329, 90 },
    { 0x5767c32b, 76 },
    { 0x4a17c32c, 61 },
    { 0x5c57c329, 95 },
    { 0x29b7c32e, 33 },
    { 0, 0 },
    { 0xe787c330, 90 },
    { 0x4427c330, 95 },
    { 0xbd87c330, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047c336, 33 },
    { 0, 0 },
    { 0xd1f7c338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe977c341, 90 },
    { 0xd777c342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847c345, 46 },
    { 0, 0 },
    { 0xbbd7c347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf557c350, 93 },
    { 0x1157c351, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd887c356, 88 },
    { 0x5617c357, 61 },
    { 0, 0 },
    { 0xa987c359, 55 },
    { 0x6a67c359, 91 },
    { 0, 0 },
    { 0x25e7c35c, 61 },
    { 0xe5b7c35d, 68 },
    { 0, 0 },
    { 0xed77c35f, 95 },
    { 0x3b7c360, 90 },
    { 0x60d7c360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7c365, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917c36b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd7c36e, 85 },
    { 0, 0 },
    { 0x1e27c370, 90 },
    { 0xc9f7c371, 90 },
    { 0x1627c372, 76 },
    { 0x9187c373, 34 },
    { 0x267c370, 33 },
    { 0xaa37c375, 100 },
    { 0x7ae7c370, 98 },
    { 0, 0 },
    { 0xc6e7c378, 23 },
    { 0x1097c379, 65 },
    { 0x3447c37a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd7c37d, 68 },
    { 0xff77c37d, 76 },
    { 0x4f27c37d, 23 },
    { 0x2507c37d, 55 },
    { 0, 0 },
    { 0x8f57c382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c388, 66 },
    { 0, 0 },
    { 0xbd57c38a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9637c390, 61 },
    { 0x8c07c391, 33 },
    { 0, 0 },
    { 0xb017c393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c398, 33 },
    { 0xfb97c399, 23 },
    { 0x3567c39a, 61 },
    { 0, 0 },
    { 0x19a7c39c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f7c39f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c3a2, 23 },
    { 0xe677c3a3, 61 },
    { 0xde27c3a4, 98 },
    { 0x9e77c3a3, 33 },
    { 0xceb7c3a4, 90 },
    { 0xedb7c3a7, 55 },
    { 0xa9b7c3a7, 76 },
    { 0x5727c3a7, 33 },
    { 0, 0 },
    { 0xb0a7c3ab, 43 },
    { 0xb017c3ac, 55 },
    { 0x8177c3ac, 76 },
    { 0, 0 },
    { 0xed97c3af, 23 },
    { 0x2317c3b0, 55 },
    { 0xe697c3b1, 33 },
    { 0x5ba7c3b2, 90 },
    { 0xc1c7c3b3, 23 },
    { 0x9b37c3b4, 68 },
    { 0x2497c3b2, 76 },
    { 0x4607c3b3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e7c3bc, 43 },
    { 0, 0 },
    { 0x4eb7c3be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237c3c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c3c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7c3cf, 90 },
    { 0, 0 },
    { 0x7187c3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a27c3d5, 61 },
    { 0xd0c7c3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa837c3da, 93 },
    { 0x3657c3db, 67 },
    { 0x1bf7c3dc, 23 },
    { 0, 0 },
    { 0x8527c3de, 33 },
    { 0x5657c3df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3407c3e5, 95 },
    { 0, 0 },
    { 0x40f7c3e7, 95 },
    { 0xc387c3e7, 98 },
    { 0, 0 },
    { 0xa007c3ea, 34 },
    { 0, 0 },
    { 0x4b17c3ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f7c3ef, 61 },
    { 0xb6b7c3f0, 90 },
    { 0xe3f7c3f1, 33 },
    { 0x7a37c3f2, 33 },
    { 0x4397c3f3, 98 },
    { 0x15f7c3f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77c3fc, 90 },
    { 0, 0 },
    { 0xcf37c3fe, 95 },
    { 0x8c47c3fe, 76 },
    { 0x52b7c400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887c40c, 33 },
    { 0x29f7c40d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e7c412, 61 },
    { 0xb4d7c413, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd167c416, 76 },
    { 0x16a7c417, 77 },
    { 0xacc7c416, 55 },
    { 0x5497c419, 90 },
    { 0, 0 },
    { 0xc237c41b, 66 },
    { 0, 0 },
    { 0xfa57c41d, 84 },
    { 0x817c41e, 76 },
    { 0x98f7c41d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47c422, 43 },
    { 0xf587c423, 88 },
    { 0x5517c424, 95 },
    { 0x437c425, 43 },
    { 0xec77c423, 90 },
    { 0x717c422, 23 },
    { 0xc887c424, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4107c42b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde47c42f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067c432, 33 },
    { 0x8847c433, 61 },
    { 0, 0 },
    { 0xf597c435, 100 },
    { 0x1bc7c436, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127c43b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b57c43e, 90 },
    { 0, 0 },
    { 0x717c440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197c443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2167c44c, 76 },
    { 0x2a47c44d, 90 },
    { 0, 0 },
    { 0x8a37c44f, 34 },
    { 0x7e27c450, 23 },
    { 0x80d7c450, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8387c454, 91 },
    { 0xc057c455, 52 },
    { 0x9c57c455, 34 },
    { 0x7c97c457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda17c45b, 98 },
    { 0xdd67c45c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7c460, 91 },
    { 0x68e7c461, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7c469, 65 },
    { 0x1577c469, 34 },
    { 0x8867c46a, 23 },
    { 0xbdb7c46b, 64 },
    { 0x6a07c46a, 78 },
    { 0xfbb7c46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd87c471, 61 },
    { 0, 0 },
    { 0xe1d7c473, 66 },
    { 0x3097c473, 98 },
    { 0, 0 },
    { 0x2887c476, 34 },
    { 0xeae7c477, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057c47a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c47f, 46 },
    { 0xec7c480, 83 },
    { 0xc77c481, 33 },
    { 0xa2e7c480, 33 },
    { 0xda37c481, 61 },
    { 0x7be7c484, 55 },
    { 0, 0 },
    { 0xa4e7c486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde7c48d, 23 },
    { 0xac37c48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed97c492, 61 },
    { 0xac17c492, 90 },
    { 0x9847c493, 76 },
    { 0x17c7c492, 90 },
    { 0, 0 },
    { 0xa17c497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07c49c, 98 },
    { 0, 0 },
    { 0xf857c49e, 76 },
    { 0x7167c49f, 43 },
    { 0, 0 },
    { 0x9ba7c4a1, 78 },
    { 0x9b47c4a2, 52 },
    { 0x6cf7c4a2, 99 },
    { 0, 0 },
    { 0x4e57c4a5, 98 },
    { 0, 0 },
    { 0xe367c4a7, 98 },
    { 0x4957c4a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577c4ab, 61 },
    { 0x4537c4ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2777c4b0, 95 },
    { 0x6167c4b1, 34 },
    { 0x8847c4b2, 34 },
    { 0x6097c4b2, 33 },
    { 0x8487c4b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d7c4bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07c4bf, 43 },
    { 0xc447c4bf, 91 },
    { 0x37a7c4c1, 98 },
    { 0xaa7c4c2, 76 },
    { 0xbae7c4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa657c4cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b27c4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5507c4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f7c4dd, 91 },
    { 0, 0 },
    { 0x3357c4df, 66 },
    { 0x2d97c4df, 34 },
    { 0x2e77c4e0, 23 },
    { 0xc167c4e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7c4e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a7c4e9, 61 },
    { 0x247c4ea, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d7c4ed, 90 },
    { 0x1c7c4ee, 90 },
    { 0x99b7c4ee, 98 },
    { 0xb2d7c4f0, 76 },
    { 0x81f7c4ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x937c4f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377c4fc, 61 },
    { 0, 0 },
    { 0x8977c4fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437c502, 33 },
    { 0x9027c502, 61 },
    { 0x7e17c504, 90 },
    { 0x9a37c502, 61 },
    { 0x7947c505, 84 },
    { 0x7cd7c507, 78 },
    { 0xce57c504, 95 },
    { 0x2f97c506, 95 },
    { 0x95d7c506, 98 },
    { 0xc7c7c502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b7c510, 99 },
    { 0, 0 },
    { 0x3217c512, 66 },
    { 0x6357c512, 90 },
    { 0xc4c7c514, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa887c518, 33 },
    { 0xbfe7c519, 78 },
    { 0x85e7c51a, 93 },
    { 0xbfb7c51a, 95 },
    { 0x30a7c51b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847c51f, 90 },
    { 0x1c17c520, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27c526, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7c52c, 95 },
    { 0xb6c7c52d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a7c530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457c535, 33 },
    { 0x6867c536, 33 },
    { 0x8ec7c536, 91 },
    { 0x8cc7c538, 76 },
    { 0x1b07c539, 33 },
    { 0x767c53a, 23 },
    { 0x7ae7c538, 98 },
    { 0xe9b7c53c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa157c542, 98 },
    { 0, 0 },
    { 0x1417c544, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7867c54a, 66 },
    { 0x9877c54b, 35 },
    { 0xeb67c54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d7c54f, 34 },
    { 0x3867c54f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2057c554, 61 },
    { 0x11d7c554, 93 },
    { 0x8ba7c554, 33 },
    { 0x93d7c554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec47c55a, 90 },
    { 0xebc7c55b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e97c563, 17 },
    { 0x4287c564, 17 },
    { 0xcad7c565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717c569, 95 },
    { 0x6857c569, 88 },
    { 0x5f27c56a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1977c572, 77 },
    { 0, 0 },
    { 0xcf67c574, 55 },
    { 0x5e7c575, 90 },
    { 0x6957c575, 98 },
    { 0xbd37c577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07c57b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6397c57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3847c582, 98 },
    { 0xc307c583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7c589, 66 },
    { 0, 0 },
    { 0xfdb7c58b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9117c58f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c594, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6087c59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627c59f, 55 },
    { 0xd997c5a0, 95 },
    { 0x4487c5a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1797c5a7, 55 },
    { 0x7c77c5a8, 84 },
    { 0xe6b7c5a9, 95 },
    { 0xd687c5a8, 61 },
    { 0xdf7c5a9, 23 },
    { 0, 0 },
    { 0x12f7c5ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd7c5b4, 55 },
    { 0xb9c7c5b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e7c5c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc587c5c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3127c5cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7c5cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc7c5d3, 65 },
    { 0, 0 },
    { 0xdd87c5d5, 76 },
    { 0, 0 },
    { 0xc3d7c5d7, 68 },
    { 0x4d07c5d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf707c5db, 90 },
    { 0, 0 },
    { 0xbb57c5dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa017c5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57c5e6, 90 },
    { 0, 0 },
    { 0xf097c5e8, 95 },
    { 0xd2a7c5e9, 90 },
    { 0, 0 },
    { 0xffa7c5eb, 90 },
    { 0, 0 },
    { 0x9437c5ed, 95 },
    { 0, 0 },
    { 0x2df7c5ef, 33 },
    { 0, 0 },
    { 0xe567c5f1, 90 },
    { 0x79f7c5f2, 91 },
    { 0xf167c5f2, 95 },
    { 0x6fe7c5f2, 23 },
    { 0x7037c5f2, 76 },
    { 0x21a7c5f6, 98 },
    { 0x8f87c5f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7c5fe, 95 },
    { 0x7a17c5ff, 43 },
    { 0x687c5fe, 55 },
    { 0xb397c600, 55 },
    { 0x38e7c602, 99 },
    { 0x717c603, 43 },
    { 0xa4c7c602, 95 },
    { 0xe437c605, 61 },
    { 0xabb7c605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1457c609, 95 },
    { 0, 0 },
    { 0xf127c60b, 46 },
    { 0x6a97c60c, 23 },
    { 0, 0 },
    { 0x9817c60e, 42 },
    { 0x32b7c60f, 46 },
    { 0xf337c60f, 61 },
    { 0x8537c611, 23 },
    { 0xb097c611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c7c616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf87c61d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e7c624, 61 },
    { 0xd6b7c624, 90 },
    { 0, 0 },
    { 0x2507c627, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87c62a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa937c62f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77c632, 61 },
    { 0xac77c632, 67 },
    { 0x1dc7c633, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x8707c637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007c63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e67c63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd337c640, 33 },
    { 0xb4e7c641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd537c644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5637c64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147c650, 88 },
    { 0xfac7c651, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa467c655, 34 },
    { 0xf887c656, 96 },
    { 0x8867c657, 91 },
    { 0xc557c658, 46 },
    { 0, 0 },
    { 0x4fd7c65a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b7c660, 34 },
    { 0xbae7c661, 90 },
    { 0xf6b7c662, 81 },
    { 0xfe87c662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7c668, 91 },
    { 0xeda7c668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87c66d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca07c670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4907c673, 61 },
    { 0xbe67c673, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb37c67a, 43 },
    { 0xe207c67a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa287c67e, 90 },
    { 0, 0 },
    { 0x6767c680, 76 },
    { 0xf7b7c681, 61 },
    { 0, 0 },
    { 0xa087c683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab47c686, 23 },
    { 0xa767c687, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e97c68a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a7c68d, 96 },
    { 0xb8c7c68e, 61 },
    { 0, 0 },
    { 0x5257c690, 66 },
    { 0, 0 },
    { 0x5d17c692, 76 },
    { 0, 0 },
    { 0x85c7c694, 98 },
    { 0, 0 },
    { 0x39f7c696, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7c699, 98 },
    { 0, 0 },
    { 0x9a17c69b, 17 },
    { 0x68f7c69c, 61 },
    { 0xc877c69d, 88 },
    { 0x27a7c69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b7c6a4, 61 },
    { 0xffd7c6a5, 23 },
    { 0x70b7c6a5, 90 },
    { 0x81f7c6a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7c6b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517c6b3, 33 },
    { 0, 0 },
    { 0x6567c6b5, 76 },
    { 0x3c47c6b5, 61 },
    { 0x2c97c6b5, 23 },
    { 0, 0 },
    { 0x40a7c6b9, 98 },
    { 0xa337c6ba, 88 },
    { 0, 0 },
    { 0x6dc7c6bc, 90 },
    { 0, 0 },
    { 0x7757c6be, 61 },
    { 0xa817c6bf, 90 },
    { 0x9577c6c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc877c6c5, 67 },
    { 0x6877c6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4337c6c9, 52 },
    { 0x3b07c6ca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf7c6d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4667c6d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd897c6d8, 84 },
    { 0xce57c6d9, 17 },
    { 0, 0 },
    { 0x6937c6db, 55 },
    { 0x57b7c6db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7c6e0, 75 },
    { 0x8367c6e1, 75 },
    { 0xf5a7c6e2, 85 },
    { 0x5c57c6e2, 99 },
    { 0xe687c6e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67c6ea, 85 },
    { 0, 0 },
    { 0x54c7c6ec, 100 },
    { 0x2d97c6ec, 90 },
    { 0x96f7c6ee, 46 },
    { 0, 0 },
    { 0xbc07c6f0, 69 },
    { 0xd0f7c6f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b7c6f5, 33 },
    { 0xdc37c6f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8727c6fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217c704, 84 },
    { 0xec67c704, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd7c708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7747c70f, 23 },
    { 0, 0 },
    { 0x10b7c711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab47c714, 76 },
    { 0, 0 },
    { 0xe487c716, 99 },
    { 0x30f7c716, 61 },
    { 0, 0 },
    { 0xfce7c719, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e7c71e, 23 },
    { 0, 0 },
    { 0x8067c720, 91 },
    { 0, 0 },
    { 0x32c7c722, 100 },
    { 0xb527c722, 33 },
    { 0, 0 },
    { 0xb607c725, 95 },
    { 0x70d7c726, 91 },
    { 0xe0b7c725, 61 },
    { 0xe927c728, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c17c72c, 61 },
    { 0x68f7c72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707c734, 76 },
    { 0x9867c735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d7c738, 88 },
    { 0x6f37c738, 55 },
    { 0, 0 },
    { 0x747c73b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab7c741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a7c746, 90 },
    { 0x327c747, 95 },
    { 0xfd37c748, 23 },
    { 0xec97c747, 84 },
    { 0x22e7c746, 43 },
    { 0x3037c74b, 66 },
    { 0, 0 },
    { 0x4be7c74d, 98 },
    { 0, 0 },
    { 0x16c7c74f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2667c755, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557c759, 55 },
    { 0x7197c75a, 23 },
    { 0xce67c75b, 61 },
    { 0xea77c75c, 52 },
    { 0x2b77c759, 78 },
    { 0, 0 },
    { 0xea97c75f, 52 },
    { 0x1157c760, 95 },
    { 0x21b7c761, 76 },
    { 0, 0 },
    { 0xc8a7c763, 95 },
    { 0xf687c763, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2897c768, 34 },
    { 0x15f7c769, 33 },
    { 0x1027c76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8477c76e, 98 },
    { 0, 0 },
    { 0xe377c770, 33 },
    { 0, 0 },
    { 0xdba7c772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b67c77b, 52 },
    { 0xf957c77b, 34 },
    { 0, 0 },
    { 0x6f87c77e, 76 },
    { 0xb27c77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb077c787, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af7c793, 90 },
    { 0x9957c793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f7c798, 95 },
    { 0x2147c799, 98 },
    { 0x9607c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827c7a1, 61 },
    { 0xac77c7a2, 90 },
    { 0, 0 },
    { 0xf427c7a4, 90 },
    { 0, 0 },
    { 0x2767c7a6, 98 },
    { 0x83d7c7a7, 33 },
    { 0x4317c7a8, 91 },
    { 0, 0 },
    { 0x3017c7aa, 33 },
    { 0x9b77c7aa, 23 },
    { 0x1667c7ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca07c7af, 61 },
    { 0, 0 },
    { 0x90c7c7b1, 66 },
    { 0xe347c7b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7617c7b7, 76 },
    { 0, 0 },
    { 0xcf47c7b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f67c7be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d67c7c3, 34 },
    { 0, 0 },
    { 0x1967c7c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7037c7cc, 90 },
    { 0x1767c7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe307c7d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7c7d5, 34 },
    { 0x2337c7d6, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d17c7db, 95 },
    { 0xfd7c7dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7c7e2, 87 },
    { 0xe707c7e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e7c7e6, 91 },
    { 0x6167c7e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f97c7ea, 76 },
    { 0xd567c7ea, 93 },
    { 0, 0 },
    { 0x3d67c7ed, 90 },
    { 0xa287c7ed, 95 },
    { 0xeaa7c7ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be7c7f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e37c7ff, 78 },
    { 0x9177c7ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb57c807, 95 },
    { 0, 0 },
    { 0x19f7c809, 98 },
    { 0, 0 },
    { 0xa977c80b, 66 },
    { 0xf777c80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427c811, 61 },
    { 0x21e7c812, 61 },
    { 0x4e87c813, 23 },
    { 0xd077c814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c7c817, 94 },
    { 0x50e7c817, 95 },
    { 0x9a57c819, 67 },
    { 0xf5e7c81a, 87 },
    { 0, 0 },
    { 0x1707c81c, 95 },
    { 0, 0 },
    { 0x9007c81e, 61 },
    { 0, 0 },
    { 0x2f37c820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7c823, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47c827, 61 },
    { 0x2d7c827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1307c831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea27c835, 93 },
    { 0x49e7c836, 61 },
    { 0, 0 },
    { 0xa647c838, 61 },
    { 0x8ba7c839, 98 },
    { 0x4de7c83a, 66 },
    { 0xc437c83b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77c83f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d7c843, 84 },
    { 0, 0 },
    { 0xab67c845, 90 },
    { 0, 0 },
    { 0xcc27c847, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf887c84e, 55 },
    { 0, 0 },
    { 0xaed7c850, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb57c854, 33 },
    { 0, 0 },
    { 0xb1b7c856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe7c859, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd587c85c, 77 },
    { 0x10d7c85c, 61 },
    { 0x1637c85c, 67 },
    { 0xc5a7c85d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e37c864, 84 },
    { 0xaf67c865, 34 },
    { 0x47b7c866, 95 },
    { 0x8c07c867, 98 },
    { 0xfa77c868, 90 },
    { 0, 0 },
    { 0x3837c86a, 33 },
    { 0xf407c86b, 61 },
    { 0x1da7c86c, 76 },
    { 0x6647c86c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea7c873, 33 },
    { 0x5087c873, 34 },
    { 0, 0 },
    { 0x6387c876, 66 },
    { 0x517c877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7c87a, 90 },
    { 0x38e7c87b, 90 },
    { 0x3807c87c, 90 },
    { 0x53f7c87d, 17 },
    { 0xfaa7c87d, 95 },
    { 0x2287c87f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f77c884, 33 },
    { 0, 0 },
    { 0x1677c886, 22 },
    { 0, 0 },
    { 0xcd47c888, 61 },
    { 0x4aa7c888, 80 },
    { 0x7577c88a, 90 },
    { 0xac17c88b, 100 },
    { 0, 0 },
    { 0x4e47c88d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5697c892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7c895, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e37c8a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c8a5, 90 },
    { 0xf707c8a5, 76 },
    { 0x2307c8a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed7c8ad, 81 },
    { 0x6dd7c8ae, 34 },
    { 0xb5c7c8af, 90 },
    { 0x1397c8b0, 87 },
    { 0xf7c7c8b0, 99 },
    { 0x9d37c8ae, 33 },
    { 0xad97c8ae, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b7c8b8, 85 },
    { 0x53d7c8b9, 77 },
    { 0x3677c8ba, 95 },
    { 0x7757c8ba, 90 },
    { 0, 0 },
    { 0xbda7c8bd, 61 },
    { 0, 0 },
    { 0x5c27c8bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7c8c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7c8c8, 93 },
    { 0x8467c8c9, 76 },
    { 0x5f37c8c8, 77 },
    { 0x1287c8cb, 55 },
    { 0x1dd7c8cc, 65 },
    { 0, 0 },
    { 0x45e7c8ce, 90 },
    { 0x85e7c8cf, 98 },
    { 0x9707c8d0, 52 },
    { 0x2df7c8d0, 95 },
    { 0, 0 },
    { 0x5ef7c8d3, 34 },
    { 0x9c77c8d4, 90 },
    { 0, 0 },
    { 0x7b17c8d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3237c8d9, 55 },
    { 0xd2b7c8d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7c8e5, 98 },
    { 0, 0 },
    { 0x8e87c8e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9607c8ea, 33 },
    { 0xc747c8eb, 43 },
    { 0xe077c8eb, 95 },
    { 0xb5e7c8ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547c8f1, 91 },
    { 0x9b17c8f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee7c8f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57c8fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67c901, 98 },
    { 0x6607c901, 98 },
    { 0xcff7c901, 34 },
    { 0xf967c902, 60 },
    { 0x7d47c905, 81 },
    { 0x18c7c906, 33 },
    { 0x9157c907, 61 },
    { 0xa607c907, 52 },
    { 0, 0 },
    { 0xde87c90a, 69 },
    { 0, 0 },
    { 0x6597c90c, 84 },
    { 0x2777c90d, 34 },
    { 0xa07c90d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07c911, 23 },
    { 0xe3a7c912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007c916, 90 },
    { 0x7697c917, 98 },
    { 0x25b7c918, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd847c91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c7c921, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa077c927, 90 },
    { 0xe907c928, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e87c92b, 61 },
    { 0x8e47c92b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4367c935, 95 },
    { 0x6607c936, 98 },
    { 0x4a97c936, 95 },
    { 0x5137c936, 36 },
    { 0x7f7c938, 85 },
    { 0, 0 },
    { 0x92d7c93b, 23 },
    { 0, 0 },
    { 0x5db7c93d, 98 },
    { 0, 0 },
    { 0xde37c93f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7c947, 61 },
    { 0xb5b7c947, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c37c94c, 43 },
    { 0x9c37c94d, 61 },
    { 0x7157c94e, 23 },
    { 0xaa47c94f, 43 },
    { 0, 0 },
    { 0x1767c951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b7c954, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc87c958, 95 },
    { 0x5337c959, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7c95f, 95 },
    { 0xb097c960, 31 },
    { 0, 0 },
    { 0x8777c962, 90 },
    { 0x9c7c963, 33 },
    { 0xa6b7c964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb7c968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47c96b, 61 },
    { 0x8d87c96b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7c96f, 88 },
    { 0x5c07c970, 33 },
    { 0xc317c970, 2 },
    { 0x6b17c972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5127c975, 34 },
    { 0x3157c976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe837c983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb7c986, 95 },
    { 0, 0 },
    { 0xa867c988, 75 },
    { 0xc867c989, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b7c990, 17 },
    { 0x5a17c991, 93 },
    { 0x8f7c992, 55 },
    { 0x17b7c993, 34 },
    { 0x2c7c994, 90 },
    { 0xf797c993, 93 },
    { 0xcf17c990, 90 },
    { 0, 0 },
    { 0x5327c998, 33 },
    { 0, 0 },
    { 0x7817c99a, 95 },
    { 0, 0 },
    { 0x9157c99c, 90 },
    { 0xd2c7c99c, 23 },
    { 0x6957c99d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd7c9a1, 68 },
    { 0x6097c9a1, 19 },
    { 0, 0 },
    { 0x7fb7c9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f37c9a8, 90 },
    { 0, 0 },
    { 0xdcd7c9aa, 90 },
    { 0, 0 },
    { 0xa217c9ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad97c9b3, 90 },
    { 0, 0 },
    { 0xde87c9b5, 33 },
    { 0x1cf7c9b6, 85 },
    { 0x8397c9b6, 23 },
    { 0, 0 },
    { 0xd3d7c9b9, 98 },
    { 0, 0 },
    { 0x8b87c9bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2867c9be, 34 },
    { 0, 0 },
    { 0x85b7c9c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe757c9c3, 52 },
    { 0x4b27c9c3, 95 },
    { 0xe207c9c5, 23 },
    { 0xf807c9c3, 90 },
    { 0, 0 },
    { 0x1e17c9c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c9ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47c9d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b7c9d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27c9dc, 98 },
    { 0, 0 },
    { 0xe7f7c9de, 93 },
    { 0xe737c9df, 61 },
    { 0x6377c9e0, 61 },
    { 0x9847c9df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a7c9e9, 23 },
    { 0x9707c9ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa047c9f5, 93 },
    { 0x217c9f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7c9fa, 93 },
    { 0, 0 },
    { 0x2967c9fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f17ca01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea7ca05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a7ca08, 55 },
    { 0xd4e7ca09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237ca0d, 23 },
    { 0x4ce7ca0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc7ca11, 46 },
    { 0x5167ca12, 90 },
    { 0xda57ca13, 33 },
    { 0xffd7ca14, 55 },
    { 0x7f17ca15, 98 },
    { 0x9767ca16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd337ca19, 95 },
    { 0, 0 },
    { 0x4437ca1b, 90 },
    { 0, 0 },
    { 0x89a7ca1d, 98 },
    { 0x43f7ca1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ca22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d7ca26, 80 },
    { 0x6577ca27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe7ca31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb567ca39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187ca3c, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f7ca40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf7ca43, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6577ca46, 78 },
    { 0x5897ca47, 55 },
    { 0xc9f7ca48, 98 },
    { 0x9997ca49, 52 },
    { 0xcff7ca48, 33 },
    { 0xa677ca48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f7ca51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7ca5c, 93 },
    { 0x9b07ca5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7ca62, 98 },
    { 0, 0 },
    { 0x3817ca64, 55 },
    { 0x8c87ca65, 98 },
    { 0x9917ca66, 75 },
    { 0x4137ca66, 78 },
    { 0xa707ca66, 93 },
    { 0, 0 },
    { 0x2027ca6a, 91 },
    { 0x81b7ca6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb587ca6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7ca77, 46 },
    { 0, 0 },
    { 0x81f7ca79, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3387ca7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea7ca81, 76 },
    { 0x26e7ca81, 96 },
    { 0x27b7ca83, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9187ca88, 23 },
    { 0xa9c7ca88, 61 },
    { 0x9b37ca88, 84 },
    { 0x617ca8b, 90 },
    { 0x547ca8b, 90 },
    { 0x5057ca8c, 76 },
    { 0x3107ca8e, 75 },
    { 0x1ba7ca8f, 33 },
    { 0xb0d7ca8f, 33 },
    { 0, 0 },
    { 0xa8a7ca92, 61 },
    { 0, 0 },
    { 0xb527ca94, 61 },
    { 0, 0 },
    { 0x9457ca96, 61 },
    { 0xe667ca97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe337ca9d, 80 },
    { 0, 0 },
    { 0xecd7ca9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787caa3, 97 },
    { 0x49e7caa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6037caa8, 43 },
    { 0xc6c7caa9, 38 },
    { 0x2377caaa, 55 },
    { 0x8227caa9, 91 },
    { 0, 0 },
    { 0x9d77caad, 37 },
    { 0, 0 },
    { 0x8767caaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7cabc, 75 },
    { 0x3037cabc, 95 },
    { 0x1db7cabd, 90 },
    { 0x4b77cabd, 60 },
    { 0xbea7cac0, 67 },
    { 0x8bd7cabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9507cacd, 66 },
    { 0, 0 },
    { 0xfc7cacf, 61 },
    { 0x4db7cad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b7cad3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda07cada, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe557cae1, 95 },
    { 0xe937cae2, 98 },
    { 0x33e7cae3, 34 },
    { 0xd7a7cae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b47cae8, 34 },
    { 0, 0 },
    { 0x2757caea, 33 },
    { 0, 0 },
    { 0xfa17caec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2637caf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7cafc, 34 },
    { 0x96c7cafc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c7cb03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67cb08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17cb0f, 77 },
    { 0x1c37cb0f, 85 },
    { 0xdfe7cb10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd547cb16, 33 },
    { 0x6af7cb16, 23 },
    { 0xaea7cb18, 55 },
    { 0x45d7cb16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb47cb1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c27cb20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9957cb25, 55 },
    { 0x9847cb26, 98 },
    { 0x9d37cb27, 36 },
    { 0x25e7cb28, 85 },
    { 0x5f7cb28, 90 },
    { 0x4e67cb28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377cb2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf607cb30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e67cb34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7cb38, 90 },
    { 0x5527cb38, 61 },
    { 0, 0 },
    { 0x1fe7cb3b, 53 },
    { 0, 0 },
    { 0x98f7cb3d, 61 },
    { 0x2d77cb3d, 90 },
    { 0x8a97cb3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657cb49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7cb54, 93 },
    { 0x26c7cb55, 99 },
    { 0x7307cb55, 33 },
    { 0x4487cb55, 55 },
    { 0xd457cb56, 34 },
    { 0x3ca7cb59, 33 },
    { 0, 0 },
    { 0x287cb5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d7cb65, 95 },
    { 0, 0 },
    { 0xcfc7cb67, 76 },
    { 0x90c7cb68, 90 },
    { 0x7fb7cb67, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57cb6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec7cb76, 95 },
    { 0, 0 },
    { 0x2ba7cb78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7cb7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3337cb7e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7cb82, 78 },
    { 0xd657cb82, 76 },
    { 0xc757cb83, 90 },
    { 0xddc7cb83, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7cb8a, 23 },
    { 0, 0 },
    { 0x63c7cb8c, 46 },
    { 0x17cb8d, 95 },
    { 0x3d07cb8d, 98 },
    { 0xeb87cb8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1087cb95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd07cb9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7cba0, 34 },
    { 0x2267cba1, 55 },
    { 0x9b47cba2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907cba5, 55 },
    { 0x50f7cba5, 61 },
    { 0x63d7cba7, 23 },
    { 0x44b7cba7, 90 },
    { 0xe067cba6, 90 },
    { 0xc0e7cbaa, 90 },
    { 0x4b17cbab, 76 },
    { 0x7937cbaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc7cbb7, 90 },
    { 0x69d7cbb7, 55 },
    { 0x62d7cbb9, 90 },
    { 0x62f7cbb8, 76 },
    { 0x36e7cbbb, 76 },
    { 0xa107cbbb, 84 },
    { 0xe997cbbc, 87 },
    { 0x43d7cbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1127cbc1, 78 },
    { 0x1a27cbc2, 23 },
    { 0x2397cbc3, 100 },
    { 0xc447cbc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7cbc9, 67 },
    { 0, 0 },
    { 0xbdd7cbcb, 23 },
    { 0x70e7cbcb, 61 },
    { 0x46e7cbcd, 91 },
    { 0x7607cbce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957cbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4047cbd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf857cbda, 98 },
    { 0, 0 },
    { 0xf017cbdc, 61 },
    { 0xd957cbdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4277cbe0, 98 },
    { 0xb147cbe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf7cbe5, 34 },
    { 0x2187cbe6, 95 },
    { 0x8077cbe5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2247cbf1, 78 },
    { 0x6387cbf1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e77cbf5, 46 },
    { 0x6387cbf6, 95 },
    { 0x6b7cbf7, 33 },
    { 0x1437cbf7, 76 },
    { 0x2667cbf9, 90 },
    { 0xbdb7cbfa, 98 },
    { 0xc1f7cbfb, 95 },
    { 0xa977cbfb, 100 },
    { 0x5687cbfd, 81 },
    { 0xc2c7cbfe, 90 },
    { 0x6c67cbff, 93 },
    { 0x4c87cc00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7cc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717cc0a, 23 },
    { 0, 0 },
    { 0x7aa7cc0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e7cc10, 23 },
    { 0x1f7cc11, 55 },
    { 0x71f7cc11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e57cc1a, 55 },
    { 0x8677cc1b, 75 },
    { 0, 0 },
    { 0x9f97cc1d, 17 },
    { 0x9e7cc1e, 55 },
    { 0x62f7cc1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7cc23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b87cc2d, 61 },
    { 0, 0 },
    { 0xe877cc2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3917cc33, 90 },
    { 0x5577cc33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f47cc37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7cc3a, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xd097cc3d, 68 },
    { 0xc7cc3e, 55 },
    { 0xa297cc3f, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417cc43, 53 },
    { 0, 0 },
    { 0x9c17cc45, 33 },
    { 0x6c07cc46, 52 },
    { 0, 0 },
    { 0x14e7cc48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe247cc4e, 95 },
    { 0x8947cc4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17cc52, 95 },
    { 0x8e67cc53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957cc56, 91 },
    { 0, 0 },
    { 0xc1f7cc58, 61 },
    { 0x4297cc59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17cc60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147cc64, 75 },
    { 0xf917cc65, 52 },
    { 0x3f27cc66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707cc69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab7cc6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b7cc74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd267cc77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c87cc7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa037cc83, 34 },
    { 0x9ed7cc84, 90 },
    { 0x9a07cc85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7cc8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c17cc91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a7cc99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb27cc9e, 95 },
    { 0x4eb7cc9e, 43 },
    { 0xe447cca0, 90 },
    { 0xf557cca0, 90 },
    { 0x2577cca1, 90 },
    { 0x3cb7cca0, 95 },
    { 0xee97cca3, 93 },
    { 0xcd97cca1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a97cca8, 93 },
    { 0x7607cca9, 90 },
    { 0x45b7ccaa, 61 },
    { 0x5937ccab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6227ccb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7ccb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b7ccbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x827ccbf, 90 },
    { 0x1da7ccc0, 75 },
    { 0xb337ccc1, 76 },
    { 0x38e7ccc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7ccc5, 61 },
    { 0xe877ccc6, 46 },
    { 0x3157ccc7, 61 },
    { 0x2d17ccc8, 23 },
    { 0xede7ccc6, 52 },
    { 0x8d47ccca, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xde87cccd, 43 },
    { 0xb317ccce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e7ccd5, 33 },
    { 0x6c07ccd5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf557ccdb, 34 },
    { 0xbe87ccdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7ccdf, 43 },
    { 0xf757cce0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e57cce8, 23 },
    { 0xb377cce9, 33 },
    { 0x907ccea, 95 },
    { 0, 0 },
    { 0x48b7ccec, 33 },
    { 0, 0 },
    { 0x3b67ccee, 61 },
    { 0x42f7ccef, 34 },
    { 0, 0 },
    { 0x48a7ccf1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f7ccf4, 33 },
    { 0x4ed7ccf5, 33 },
    { 0x5367ccf6, 95 },
    { 0x6e97ccf7, 99 },
    { 0x6a7ccf6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe387ccfe, 99 },
    { 0x18a7ccfe, 32 },
    { 0xb637ccfe, 90 },
    { 0x5ee7cd01, 90 },
    { 0xdeb7cd02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed97cd08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6917cd0c, 67 },
    { 0x3a67cd0d, 33 },
    { 0x7d47cd0d, 90 },
    { 0xa197cd0d, 83 },
    { 0x7be7cd0c, 84 },
    { 0xa967cd0c, 55 },
    { 0, 0 },
    { 0xcd07cd13, 55 },
    { 0x79a7cd14, 46 },
    { 0, 0 },
    { 0x1dc7cd16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3517cd1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea7cd20, 61 },
    { 0, 0 },
    { 0x8f77cd22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e7cd25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b7cd2a, 85 },
    { 0, 0 },
    { 0x8347cd2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b07cd2f, 95 },
    { 0x5b47cd30, 90 },
    { 0x3087cd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9757cd34, 52 },
    { 0x62d7cd34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb57cd3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907cd40, 98 },
    { 0, 0 },
    { 0x9837cd42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae7cd47, 33 },
    { 0, 0 },
    { 0xda77cd49, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8607cd4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77cd53, 99 },
    { 0xf3c7cd54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3987cd57, 61 },
    { 0xce87cd58, 61 },
    { 0x837cd59, 61 },
    { 0x9fc7cd59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7cd5f, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7cd64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f7cd6c, 34 },
    { 0, 0 },
    { 0x44f7cd6e, 95 },
    { 0, 0 },
    { 0xc957cd70, 23 },
    { 0x3767cd70, 84 },
    { 0x53e7cd70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd697cd77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a47cd7d, 34 },
    { 0, 0 },
    { 0x2877cd7f, 95 },
    { 0x87e7cd7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x607cd83, 95 },
    { 0, 0 },
    { 0x807cd85, 33 },
    { 0, 0 },
    { 0xb817cd87, 23 },
    { 0xc587cd87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a7cd8b, 90 },
    { 0xd147cd8c, 55 },
    { 0x77e7cd8c, 23 },
    { 0xd577cd8e, 34 },
    { 0x3307cd8f, 61 },
    { 0x277cd90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7cd94, 95 },
    { 0xa257cd95, 90 },
    { 0xda7cd96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd27cd9e, 87 },
    { 0, 0 },
    { 0x2a67cda0, 43 },
    { 0x93f7cda0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7cda7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa7cdaf, 36 },
    { 0xe037cdb0, 55 },
    { 0, 0 },
    { 0xbcc7cdb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc7cdb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3437cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdc4, 61 },
    { 0x9a47cdc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177cdcb, 67 },
    { 0xe377cdcc, 34 },
    { 0x8f87cdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87cdd2, 99 },
    { 0x6f67cdd3, 34 },
    { 0, 0 },
    { 0xcbc7cdd5, 76 },
    { 0, 0 },
    { 0x8f37cdd7, 33 },
    { 0x7797cdd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1807cddb, 90 },
    { 0xf667cddc, 33 },
    { 0xdf07cddc, 96 },
    { 0xb127cddd, 95 },
    { 0xeb47cddf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf157cde4, 95 },
    { 0, 0 },
    { 0x5947cde6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67cde9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb687cdec, 34 },
    { 0, 0 },
    { 0xe97cdee, 82 },
    { 0, 0 },
    { 0x47b7cdf0, 33 },
    { 0x1d37cdf1, 90 },
    { 0x7cd7cdf2, 95 },
    { 0xcfa7cdf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7cdf9, 98 },
    { 0x17b7cdfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb417cdfd, 93 },
    { 0, 0 },
    { 0x3747cdff, 33 },
    { 0xbf97ce00, 90 },
    { 0xc937ce00, 61 },
    { 0xfc97ce02, 33 },
    { 0xd227cdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57ce07, 90 },
    { 0x31e7ce07, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b7ce0b, 78 },
    { 0, 0 },
    { 0x647ce0d, 33 },
    { 0, 0 },
    { 0x8777ce0f, 87 },
    { 0x7f57ce10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6097ce19, 43 },
    { 0x58e7ce19, 98 },
    { 0x41d7ce1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ce1f, 34 },
    { 0xe427ce20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147ce23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237ce26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5167ce30, 33 },
    { 0xe937ce31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff7ce35, 98 },
    { 0xc7e7ce35, 23 },
    { 0xc5d7ce37, 90 },
    { 0x7897ce36, 90 },
    { 0xa507ce37, 98 },
    { 0x67f7ce3a, 95 },
    { 0xa947ce3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b7ce40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7ce44, 95 },
    { 0x7017ce44, 90 },
    { 0xf9f7ce45, 33 },
    { 0x2f47ce47, 90 },
    { 0xf607ce45, 24 },
    { 0x8487ce49, 76 },
    { 0, 0 },
    { 0xf07ce4b, 76 },
    { 0, 0 },
    { 0x64a7ce4d, 23 },
    { 0x8067ce4e, 46 },
    { 0xd3b7ce4f, 95 },
    { 0x22e7ce4f, 98 },
    { 0, 0 },
    { 0xb9b7ce52, 87 },
    { 0x2ce7ce52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb87ce5a, 85 },
    { 0, 0 },
    { 0x1267ce5c, 98 },
    { 0xf9c7ce5c, 90 },
    { 0x2257ce5d, 93 },
    { 0xbf07ce5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3647ce63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc537ce67, 88 },
    { 0, 0 },
    { 0x44a7ce69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7ce6e, 46 },
    { 0x5ec7ce6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7ce72, 46 },
    { 0x6e67ce72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7ce7c, 43 },
    { 0x3af7ce7c, 95 },
    { 0x5f17ce7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8207ce82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb297ce87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc67ce8c, 87 },
    { 0, 0 },
    { 0xb4c7ce8e, 66 },
    { 0, 0 },
    { 0x6867ce90, 90 },
    { 0x4ae7ce90, 61 },
    { 0xe8c7ce90, 95 },
    { 0xeb37ce91, 61 },
    { 0x1527ce90, 46 },
    { 0xbd67ce95, 55 },
    { 0x6f87ce93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc817ce9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d7ceae, 67 },
    { 0x6cc7ceaf, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6127ceb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad87cebb, 84 },
    { 0, 0 },
    { 0xb9e7cebd, 34 },
    { 0x6e87cebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47cec1, 33 },
    { 0, 0 },
    { 0x7cd7cec3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b7cec6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397cec9, 90 },
    { 0x1a97ceca, 90 },
    { 0x1507cecb, 77 },
    { 0, 0 },
    { 0x8e27cecd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97ced0, 23 },
    { 0x5c67ced0, 99 },
    { 0x307ced2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d7ced9, 34 },
    { 0xaf87ced9, 34 },
    { 0x7f27cedb, 98 },
    { 0xfc97cedb, 98 },
    { 0x4ad7cedd, 90 },
    { 0x4557cedd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257cee1, 98 },
    { 0xc3c7cee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57cee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e7ceea, 85 },
    { 0x5bd7ceea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7ceee, 34 },
    { 0, 0 },
    { 0x6b67cef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57cef3, 90 },
    { 0x3cd7cef4, 76 },
    { 0x7b17cef5, 77 },
    { 0x7b67cef5, 93 },
    { 0x6237cef7, 49 },
    { 0x4d97cef7, 61 },
    { 0x2e47cef5, 90 },
    { 0xce27cef3, 55 },
    { 0x7657cefb, 95 },
    { 0, 0 },
    { 0x36f7cefd, 93 },
    { 0x9f97cefe, 76 },
    { 0xd2c7ceff, 98 },
    { 0xba7ceff, 90 },
    { 0x3837cf00, 61 },
    { 0x5b27cefe, 93 },
    { 0x7527cf03, 33 },
    { 0x4277cf04, 90 },
    { 0, 0 },
    { 0x7ba7cf06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e7cf09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287cf0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6137cf10, 61 },
    { 0x44d7cf11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7cf15, 56 },
    { 0xed77cf15, 95 },
    { 0xea07cf17, 33 },
    { 0x7c07cf16, 91 },
    { 0x9d27cf19, 40 },
    { 0x5a07cf19, 69 },
    { 0x34f7cf19, 33 },
    { 0xcac7cf1a, 95 },
    { 0xe797cf1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xac97cf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5207cf26, 95 },
    { 0x3827cf27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e57cf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb7cf4c, 93 },
    { 0x8b67cf4d, 33 },
    { 0x2c37cf4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7cf54, 98 },
    { 0x7c7cf55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d7cf58, 100 },
    { 0x1007cf58, 33 },
    { 0, 0 },
    { 0x2607cf5b, 90 },
    { 0xb567cf5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd987cf61, 43 },
    { 0, 0 },
    { 0xde7cf63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b67cf69, 98 },
    { 0x8a67cf69, 43 },
    { 0x3d67cf6b, 23 },
    { 0x1a97cf6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf47cf6f, 90 },
    { 0, 0 },
    { 0x7f67cf71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a7cf76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c7cf7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7cf7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7627cf81, 76 },
    { 0xfbf7cf81, 34 },
    { 0xba47cf83, 61 },
    { 0xc2c7cf84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427cf8e, 34 },
    { 0x5ac7cf8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf197cf92, 34 },
    { 0, 0 },
    { 0xbb37cf94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57cf97, 90 },
    { 0, 0 },
    { 0x38d7cf99, 90 },
    { 0xe337cf9a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7cf9e, 98 },
    { 0, 0 },
    { 0x4907cfa0, 55 },
    { 0xadf7cfa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f7cfa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847cfa8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d7cfab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de7cfae, 98 },
    { 0x56b7cfaf, 76 },
    { 0, 0 },
    { 0xc227cfb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e17cfb4, 34 },
    { 0, 0 },
    { 0xc7c7cfb6, 91 },
    { 0x7177cfb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3037cfba, 93 },
    { 0, 0 },
    { 0xaac7cfbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7cfbf, 61 },
    { 0x6b67cfc0, 84 },
    { 0x3797cfc1, 46 },
    { 0x2287cfc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277cfc7, 55 },
    { 0x6cc7cfc7, 95 },
    { 0xa797cfc9, 76 },
    { 0x7027cfc7, 98 },
    { 0x3ce7cfcb, 90 },
    { 0xd0f7cfcc, 46 },
    { 0, 0 },
    { 0x31b7cfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17cfd3, 33 },
    { 0x32e7cfd3, 91 },
    { 0xb187cfd5, 23 },
    { 0xe3b7cfd6, 98 },
    { 0xcc17cfd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd7cfda, 98 },
    { 0, 0 },
    { 0x8397cfdc, 90 },
    { 0x2d07cfdd, 99 },
    { 0, 0 },
    { 0x73e7cfdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cfe3, 61 },
    { 0xc077cfe4, 76 },
    { 0x5c97cfe5, 90 },
    { 0x6d77cfe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827cfe9, 46 },
    { 0, 0 },
    { 0xf207cfeb, 33 },
    { 0x597cfec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d37cff0, 66 },
    { 0x5587cff0, 69 },
    { 0xe167cff2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507cffa, 91 },
    { 0x987cffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d7cfff, 64 },
    { 0xfbd7cfff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27d009, 67 },
    { 0xa117d00a, 75 },
    { 0x3de7d00a, 76 },
    { 0x5037d009, 95 },
    { 0xcda7d00d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d77d017, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6427d01b, 85 },
    { 0x3f97d01c, 33 },
    { 0, 0 },
    { 0x1227d01e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d023, 95 },
    { 0, 0 },
    { 0xcfe7d025, 52 },
    { 0xf247d025, 61 },
    { 0xd197d027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f57d02b, 95 },
    { 0xbfc7d02c, 98 },
    { 0xba27d02d, 61 },
    { 0x4c07d02b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee7d031, 69 },
    { 0x2987d032, 98 },
    { 0x6337d032, 77 },
    { 0x67a7d033, 55 },
    { 0xea47d033, 61 },
    { 0x8777d036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb207d039, 33 },
    { 0x6997d03a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7d03f, 95 },
    { 0, 0 },
    { 0x29d7d041, 34 },
    { 0xaac7d042, 93 },
    { 0, 0 },
    { 0xf947d044, 81 },
    { 0x9987d045, 98 },
    { 0x6817d046, 33 },
    { 0x1337d046, 34 },
    { 0x46c7d048, 23 },
    { 0x53a7d048, 95 },
    { 0x39b7d044, 85 },
    { 0x4577d046, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb67d04e, 88 },
    { 0x61d7d04f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7d052, 98 },
    { 0x8047d053, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb817d057, 46 },
    { 0xe1f7d058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747d05c, 76 },
    { 0xa067d05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a7d064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9957d069, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607d06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2377d071, 34 },
    { 0xd9d7d071, 95 },
    { 0x5c47d073, 98 },
    { 0xfc97d072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f7d077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c7d07d, 77 },
    { 0xf27d07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017d081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8787d085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c7d090, 98 },
    { 0x3317d091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b7d099, 61 },
    { 0x1047d09a, 65 },
    { 0x61a7d09a, 95 },
    { 0, 0 },
    { 0x39c7d09d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7d0a2, 61 },
    { 0x3107d0a3, 98 },
    { 0x5d37d0a4, 87 },
    { 0x3427d0a2, 99 },
    { 0x2027d0a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1957d0b0, 52 },
    { 0, 0 },
    { 0x3167d0b2, 90 },
    { 0xecc7d0b3, 98 },
    { 0, 0 },
    { 0x6bd7d0b5, 93 },
    { 0, 0 },
    { 0x1537d0b7, 64 },
    { 0, 0 },
    { 0x4ba7d0b9, 61 },
    { 0x1087d0ba, 76 },
    { 0xb4d7d0bb, 95 },
    { 0, 0 },
    { 0x76f7d0bd, 90 },
    { 0x4be7d0bd, 76 },
    { 0xda27d0bd, 90 },
    { 0x4d67d0c0, 23 },
    { 0x8007d0c1, 90 },
    { 0x87c7d0bf, 95 },
    { 0, 0 },
    { 0x94a7d0c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f7d0c7, 34 },
    { 0x9fd7d0c7, 33 },
    { 0x7e57d0c9, 17 },
    { 0x8357d0ca, 65 },
    { 0, 0 },
    { 0xaf17d0cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd7d0d4, 82 },
    { 0x1057d0d5, 95 },
    { 0x21a7d0d5, 55 },
    { 0, 0 },
    { 0xa777d0d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a7d0db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4557d0de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b7d0e6, 90 },
    { 0xbaf7d0e7, 61 },
    { 0x1837d0e8, 90 },
    { 0x97c7d0e9, 23 },
    { 0, 0 },
    { 0x47e7d0eb, 33 },
    { 0xac27d0eb, 34 },
    { 0x6d27d0ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17d0f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb357d0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e7d0fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407d0ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc677d102, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b7d107, 91 },
    { 0x6307d107, 55 },
    { 0, 0 },
    { 0xcba7d10a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d10d, 33 },
    { 0xc397d10d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047d112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe647d116, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d7d11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4737d121, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c67d125, 43 },
    { 0x6cb7d126, 95 },
    { 0, 0 },
    { 0xe307d128, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647d137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de7d13c, 95 },
    { 0x5a87d13c, 61 },
    { 0xc457d13e, 93 },
    { 0, 0 },
    { 0xe537d140, 55 },
    { 0xb037d140, 81 },
    { 0x61e7d142, 52 },
    { 0x3d17d143, 98 },
    { 0xda07d140, 76 },
    { 0xfdf7d145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e7d148, 90 },
    { 0, 0 },
    { 0xb247d14a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7d14f, 33 },
    { 0xe007d150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497d153, 98 },
    { 0x58f7d154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b87d157, 23 },
    { 0, 0 },
    { 0xd667d159, 90 },
    { 0xc727d15a, 81 },
    { 0xcb27d159, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4967d160, 23 },
    { 0x12c7d161, 88 },
    { 0x8807d162, 52 },
    { 0, 0 },
    { 0xa177d164, 34 },
    { 0x1727d165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7d16b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9697d16f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe937d174, 90 },
    { 0xe507d174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d17d, 98 },
    { 0, 0 },
    { 0xc427d17f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb7d184, 23 },
    { 0x18e7d184, 61 },
    { 0x2147d185, 66 },
    { 0x4887d186, 55 },
    { 0, 0 },
    { 0xd0d7d189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad97d194, 34 },
    { 0x54e7d194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8247d19a, 23 },
    { 0, 0 },
    { 0xa647d19c, 67 },
    { 0x9d17d19d, 85 },
    { 0, 0 },
    { 0x77e7d19f, 61 },
    { 0x4f17d1a0, 99 },
    { 0xdb47d1a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb037d1a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b07d1a8, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xac87d1ab, 84 },
    { 0x6b97d1ab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae57d1b0, 55 },
    { 0, 0 },
    { 0x6e47d1b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b7d1b5, 90 },
    { 0xd437d1b6, 95 },
    { 0xd877d1b7, 33 },
    { 0x8587d1b6, 61 },
    { 0x6627d1b9, 46 },
    { 0, 0 },
    { 0x92a7d1bb, 77 },
    { 0x3f97d1bc, 61 },
    { 0, 0 },
    { 0x2bc7d1be, 55 },
    { 0x8fe7d1be, 34 },
    { 0x2fc7d1c0, 76 },
    { 0x5337d1c1, 95 },
    { 0, 0 },
    { 0x5be7d1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d17d1c8, 90 },
    { 0x89a7d1c9, 95 },
    { 0x2617d1c8, 76 },
    { 0x1717d1c8, 88 },
    { 0x9027d1cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d1d0, 90 },
    { 0x2da7d1d0, 98 },
    { 0x54b7d1d2, 55 },
    { 0xfd87d1d1, 90 },
    { 0, 0 },
    { 0xb347d1d5, 23 },
    { 0xa2a7d1d6, 98 },
    { 0x6bb7d1d7, 90 },
    { 0x4c57d1d5, 34 },
    { 0x4a17d1d9, 33 },
    { 0xe4f7d1d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f7d1de, 46 },
    { 0, 0 },
    { 0xbac7d1e0, 42 },
    { 0x2be7d1e0, 33 },
    { 0xd377d1e2, 98 },
    { 0, 0 },
    { 0x4ee7d1e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4457d1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa677d1ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc97d1fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac7d1fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d07d201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d57d204, 99 },
    { 0x3b47d205, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7d208, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff37d20c, 55 },
    { 0x67d20d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917d210, 33 },
    { 0x7b47d211, 43 },
    { 0, 0 },
    { 0xf437d213, 95 },
    { 0x78d7d214, 98 },
    { 0, 0 },
    { 0x7547d216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887d21f, 90 },
    { 0xc8f7d220, 100 },
    { 0xe397d220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1677d228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a57d231, 46 },
    { 0xf6f7d231, 23 },
    { 0x88a7d232, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7d238, 61 },
    { 0x8987d239, 98 },
    { 0x54e7d23a, 23 },
    { 0xa0e7d23b, 78 },
    { 0x1257d238, 81 },
    { 0x2237d23d, 43 },
    { 0xad67d23b, 95 },
    { 0x6fe7d23f, 61 },
    { 0xa107d240, 23 },
    { 0x7c57d241, 85 },
    { 0x83f7d241, 55 },
    { 0x7447d243, 33 },
    { 0, 0 },
    { 0x7207d245, 23 },
    { 0xdb57d246, 34 },
    { 0x7357d247, 66 },
    { 0xf057d245, 33 },
    { 0x7837d249, 99 },
    { 0xdea7d249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd7d24f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b7d252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a7d255, 98 },
    { 0x1da7d255, 90 },
    { 0x8a27d256, 98 },
    { 0xe837d257, 95 },
    { 0x9ae7d259, 43 },
    { 0x9f67d259, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c57d25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27d263, 100 },
    { 0x307d264, 23 },
    { 0x8b17d265, 23 },
    { 0xac97d266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7d26e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527d271, 93 },
    { 0x667d272, 90 },
    { 0xb137d272, 100 },
    { 0x5f7d274, 95 },
    { 0, 0 },
    { 0x19d7d276, 95 },
    { 0xcf47d276, 46 },
    { 0x16f7d276, 95 },
    { 0x4237d279, 33 },
    { 0x9017d279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a7d27d, 99 },
    { 0xe97d27e, 90 },
    { 0, 0 },
    { 0xb477d280, 55 },
    { 0xe607d281, 96 },
    { 0xb4e7d282, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827d285, 55 },
    { 0x8db7d286, 76 },
    { 0, 0 },
    { 0x4a07d288, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c47d292, 96 },
    { 0x9197d292, 85 },
    { 0xc0a7d294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377d297, 74 },
    { 0, 0 },
    { 0x5067d299, 33 },
    { 0, 0 },
    { 0x2e67d29b, 66 },
    { 0x4bc7d29b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577d29f, 77 },
    { 0, 0 },
    { 0x9647d2a1, 43 },
    { 0, 0 },
    { 0xbaf7d2a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d2aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc7d2ae, 55 },
    { 0x85f7d2af, 46 },
    { 0xaa97d2b0, 90 },
    { 0xb807d2b1, 61 },
    { 0x6c97d2b0, 98 },
    { 0xac87d2ae, 34 },
    { 0, 0 },
    { 0xfff7d2b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe207d2ba, 66 },
    { 0, 0 },
    { 0x1de7d2bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe197d2c0, 91 },
    { 0x1897d2c1, 61 },
    { 0x9e17d2c2, 23 },
    { 0, 0 },
    { 0xde87d2c4, 98 },
    { 0x9ab7d2c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817d2cb, 33 },
    { 0xa4c7d2cb, 91 },
    { 0x2a87d2cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf7d2d4, 90 },
    { 0x1c97d2d5, 93 },
    { 0xbe97d2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7d2d9, 65 },
    { 0x3e67d2d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1767d2df, 61 },
    { 0, 0 },
    { 0x2cd7d2e1, 90 },
    { 0, 0 },
    { 0x8587d2e3, 76 },
    { 0, 0 },
    { 0xaba7d2e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa437d2eb, 64 },
    { 0x23d7d2ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f47d2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe187d2f4, 78 },
    { 0x5967d2f5, 91 },
    { 0xd087d2f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7d2f9, 33 },
    { 0x6ac7d2fa, 33 },
    { 0, 0 },
    { 0x65a7d2fc, 46 },
    { 0x5157d2fd, 53 },
    { 0xbca7d2fe, 98 },
    { 0x3f57d2ff, 90 },
    { 0x2c37d2fe, 95 },
    { 0, 0 },
    { 0x54a7d302, 33 },
    { 0x6ac7d303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd7d30a, 23 },
    { 0, 0 },
    { 0x4fd7d30c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a7d311, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad27d314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db7d318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17d31c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d7d320, 98 },
    { 0x3237d320, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d7d32b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7d335, 60 },
    { 0x5777d336, 95 },
    { 0xb157d337, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57d33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a87d343, 33 },
    { 0xbc67d344, 98 },
    { 0, 0 },
    { 0xd27d346, 90 },
    { 0, 0 },
    { 0x6f77d348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907d34d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6397d352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097d355, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe137d359, 90 },
    { 0xdb47d35a, 95 },
    { 0x2527d35b, 95 },
    { 0x1b07d35c, 93 },
    { 0xdc27d359, 95 },
    { 0x6dc7d35d, 33 },
    { 0, 0 },
    { 0xe7a7d360, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a7d366, 76 },
    { 0, 0 },
    { 0x4177d368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7357d36b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657d371, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7157d375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5937d378, 67 },
    { 0x377d379, 33 },
    { 0xa877d378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f07d37f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe167d386, 90 },
    { 0x1867d387, 33 },
    { 0x5d37d387, 52 },
    { 0x1497d387, 76 },
    { 0x2257d38a, 90 },
    { 0xf917d386, 93 },
    { 0xc157d387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5587d391, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa817d39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7d39d, 91 },
    { 0x4277d39e, 23 },
    { 0xe347d39d, 76 },
    { 0x4e97d39e, 90 },
    { 0, 0 },
    { 0x60c7d3a2, 90 },
    { 0x3b97d3a3, 81 },
    { 0, 0 },
    { 0x8397d3a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b7d3a9, 93 },
    { 0, 0 },
    { 0x23d7d3ab, 23 },
    { 0x9d37d3ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4287d3b2, 34 },
    { 0x8f87d3b3, 34 },
    { 0x5a97d3b3, 67 },
    { 0, 0 },
    { 0x18b7d3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b7d3ba, 64 },
    { 0xb77d3bb, 34 },
    { 0x76e7d3bb, 95 },
    { 0, 0 },
    { 0x76e7d3be, 33 },
    { 0, 0 },
    { 0x92f7d3c0, 95 },
    { 0xeab7d3c0, 87 },
    { 0x6457d3c2, 88 },
    { 0x7a07d3c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f7d3c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba67d3ce, 95 },
    { 0, 0 },
    { 0x4a67d3d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c7d3d3, 61 },
    { 0x5c67d3d4, 46 },
    { 0x2597d3d5, 33 },
    { 0x15c7d3d6, 90 },
    { 0xb1b7d3d6, 23 },
    { 0x67a7d3d4, 61 },
    { 0xd197d3d8, 90 },
    { 0xdc57d3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7d3e4, 55 },
    { 0, 0 },
    { 0x3c87d3e6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b47d3ed, 93 },
    { 0, 0 },
    { 0x9137d3ef, 33 },
    { 0x59f7d3f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697d3f8, 33 },
    { 0x2ce7d3f9, 61 },
    { 0xb6c7d3fa, 90 },
    { 0x27a7d3fb, 33 },
    { 0x697d3f8, 23 },
    { 0x9977d3fd, 55 },
    { 0x2f37d3fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf527d402, 23 },
    { 0x8d27d403, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2007d406, 91 },
    { 0x3897d407, 90 },
    { 0xfed7d408, 90 },
    { 0, 0 },
    { 0xd087d40a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7d411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe707d417, 95 },
    { 0, 0 },
    { 0x19e7d419, 84 },
    { 0x1c37d41a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187d41d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d420, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2997d425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf207d428, 99 },
    { 0xb77d429, 33 },
    { 0, 0 },
    { 0xec17d42b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c07d42f, 46 },
    { 0xd537d430, 98 },
    { 0x67a7d42f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897d436, 95 },
    { 0xe727d437, 55 },
    { 0xff27d436, 23 },
    { 0x4b67d438, 81 },
    { 0x9e17d43a, 33 },
    { 0x2f87d43b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6277d43e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc327d444, 23 },
    { 0, 0 },
    { 0x70d7d446, 33 },
    { 0xd3f7d446, 33 },
    { 0, 0 },
    { 0x7d17d449, 95 },
    { 0x2447d44a, 69 },
    { 0x5997d44b, 9 },
    { 0, 0 },
    { 0xd857d44d, 46 },
    { 0x2607d44e, 98 },
    { 0x46d7d44f, 66 },
    { 0x7c47d450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f77d458, 61 },
    { 0x3787d459, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7d45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f67d461, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7d465, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe347d46d, 23 },
    { 0x77e7d46e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f77d471, 61 },
    { 0xae7d472, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b17d476, 33 },
    { 0xbc37d477, 33 },
    { 0x8e87d478, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b7d47c, 55 },
    { 0, 0 },
    { 0xe807d47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d7d487, 90 },
    { 0, 0 },
    { 0x8c57d489, 90 },
    { 0, 0 },
    { 0xc777d48b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4687d48f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7d492, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa387d497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1247d49a, 84 },
    { 0x3e27d49b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7357d4a6, 66 },
    { 0xc5f7d4a7, 61 },
    { 0x52f7d4a7, 90 },
    { 0x5dd7d4a8, 33 },
    { 0x8eb7d4a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17d4ad, 33 },
    { 0xd897d4ae, 93 },
    { 0x5357d4ae, 61 },
    { 0x7f97d4b0, 33 },
    { 0xd07d4b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb437d4b5, 91 },
    { 0xc277d4b6, 90 },
    { 0xcd67d4b7, 76 },
    { 0x9167d4b5, 98 },
    { 0xf187d4b6, 98 },
    { 0x1057d4ba, 77 },
    { 0x9967d4ba, 84 },
    { 0xeb77d4bc, 61 },
    { 0x5c37d4bd, 100 },
    { 0xa667d4be, 95 },
    { 0x8fa7d4bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7d4c2, 23 },
    { 0x1157d4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd127d4cd, 95 },
    { 0x5f37d4ce, 61 },
    { 0xad67d4cf, 43 },
    { 0xec47d4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987d4d3, 23 },
    { 0xdc67d4d4, 23 },
    { 0, 0 },
    { 0xa4c7d4d6, 98 },
    { 0x14f7d4d6, 99 },
    { 0x4377d4d8, 33 },
    { 0x46c7d4d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787d4dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627d4e3, 55 },
    { 0, 0 },
    { 0x7547d4e5, 34 },
    { 0xde47d4e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1597d4ec, 77 },
    { 0x4a37d4ed, 91 },
    { 0x5fe7d4ee, 95 },
    { 0, 0 },
    { 0x9697d4f0, 76 },
    { 0xaf07d4f1, 23 },
    { 0xea37d4f0, 95 },
    { 0x7987d4f3, 76 },
    { 0x35d7d4f4, 90 },
    { 0x81e7d4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a07d4fa, 98 },
    { 0, 0 },
    { 0xc2d7d4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb7d500, 77 },
    { 0xd447d500, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9527d505, 23 },
    { 0, 0 },
    { 0x53f7d507, 78 },
    { 0x2c67d507, 90 },
    { 0x67e7d509, 34 },
    { 0x9aa7d509, 90 },
    { 0xf037d509, 93 },
    { 0x3fa7d507, 91 },
    { 0xd587d50d, 55 },
    { 0xab97d50d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7d514, 76 },
    { 0x25f7d515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f7d51a, 55 },
    { 0, 0 },
    { 0xed67d51c, 78 },
    { 0xe6e7d51d, 54 },
    { 0, 0 },
    { 0x557d51f, 23 },
    { 0xa1f7d520, 95 },
    { 0xa8d7d521, 98 },
    { 0xd627d521, 95 },
    { 0x3c97d522, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c67d531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc097d534, 40 },
    { 0xa4d7d534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac7d538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67d53b, 33 },
    { 0x6ff7d53b, 100 },
    { 0xac57d53d, 43 },
    { 0x22b7d53e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd967d547, 99 },
    { 0x6687d548, 95 },
    { 0, 0 },
    { 0xa0a7d54a, 33 },
    { 0x9ef7d54a, 93 },
    { 0xe317d54c, 85 },
    { 0xfa37d54c, 60 },
    { 0x6167d54e, 55 },
    { 0x3947d54d, 98 },
    { 0x4717d550, 90 },
    { 0xa8a7d54d, 98 },
    { 0xb457d552, 61 },
    { 0x9c7d54e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2477d557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee7d55c, 95 },
    { 0, 0 },
    { 0xa057d55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a67d561, 93 },
    { 0, 0 },
    { 0x2557d563, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d67d567, 90 },
    { 0, 0 },
    { 0x5e17d569, 46 },
    { 0x75f7d56a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87d56f, 33 },
    { 0, 0 },
    { 0x2b77d571, 100 },
    { 0x6f27d572, 43 },
    { 0x7ed7d573, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d576, 93 },
    { 0x7787d577, 78 },
    { 0xec17d576, 52 },
    { 0x4ad7d579, 90 },
    { 0, 0 },
    { 0xd027d57b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c7d57e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf537d582, 33 },
    { 0, 0 },
    { 0xdf37d584, 43 },
    { 0, 0 },
    { 0xbdb7d586, 61 },
    { 0, 0 },
    { 0x3677d588, 85 },
    { 0, 0 },
    { 0x97f7d58a, 99 },
    { 0x4e17d58b, 61 },
    { 0x8457d58c, 34 },
    { 0xaa57d58b, 33 },
    { 0xbbd7d58e, 33 },
    { 0x3c47d58f, 55 },
    { 0, 0 },
    { 0x9357d591, 46 },
    { 0x5c47d591, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257d598, 95 },
    { 0, 0 },
    { 0x2ba7d59a, 55 },
    { 0, 0 },
    { 0xace7d59c, 55 },
    { 0x7e7d59d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a07d5a1, 90 },
    { 0x4067d5a1, 61 },
    { 0, 0 },
    { 0x5ea7d5a4, 85 },
    { 0x1ae7d5a5, 90 },
    { 0x917d5a6, 33 },
    { 0x6697d5a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7897d5ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e7d5b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7d5bc, 98 },
    { 0x2a47d5bc, 33 },
    { 0xb1a7d5bc, 87 },
    { 0x84d7d5bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc7d5c4, 98 },
    { 0xa107d5c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17d5cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7d5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1387d5d8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7d5db, 77 },
    { 0xdd77d5dc, 66 },
    { 0, 0 },
    { 0xe717d5de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7d5e1, 93 },
    { 0x1237d5e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf7d5ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae7d5ed, 23 },
    { 0xaf87d5ed, 66 },
    { 0xa57d5ed, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7d5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b07d5fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827d603, 55 },
    { 0, 0 },
    { 0xfce7d605, 34 },
    { 0xacc7d606, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc707d60f, 61 },
    { 0, 0 },
    { 0x6927d611, 17 },
    { 0x7757d612, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d7d615, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827d61c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b7d61f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd617d622, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xded7d625, 93 },
    { 0x2037d626, 85 },
    { 0, 0 },
    { 0x3cf7d628, 98 },
    { 0, 0 },
    { 0x4697d62a, 23 },
    { 0x11a7d62b, 90 },
    { 0, 0 },
    { 0x2177d62d, 23 },
    { 0xc127d62d, 95 },
    { 0x8907d62f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a7d635, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f7d63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957d63d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b7d640, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c87d649, 45 },
    { 0xb7e7d649, 90 },
    { 0x4de7d649, 84 },
    { 0xdb97d64c, 23 },
    { 0x5387d64d, 33 },
    { 0xece7d64e, 99 },
    { 0x71d7d64d, 23 },
    { 0x8447d64b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7d658, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4117d660, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4157d664, 34 },
    { 0xfa7d665, 53 },
    { 0xd467d666, 23 },
    { 0xbce7d667, 75 },
    { 0x2bc7d666, 76 },
    { 0x3cd7d668, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb457d66d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5707d674, 33 },
    { 0, 0 },
    { 0xb887d676, 43 },
    { 0, 0 },
    { 0x5c77d678, 43 },
    { 0x7197d678, 55 },
    { 0, 0 },
    { 0x46e7d67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5637d684, 23 },
    { 0xe2a7d684, 95 },
    { 0xb27d685, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed7d68a, 90 },
    { 0, 0 },
    { 0xedc7d68c, 55 },
    { 0xc4a7d68d, 56 },
    { 0x7037d68e, 87 },
    { 0x22a7d68f, 61 },
    { 0xb927d690, 95 },
    { 0xa157d691, 90 },
    { 0, 0 },
    { 0x6d7d693, 90 },
    { 0xae37d694, 90 },
    { 0xf957d694, 23 },
    { 0x8ae7d695, 98 },
    { 0xab17d697, 55 },
    { 0, 0 },
    { 0xb707d699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf7d69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7d6a0, 76 },
    { 0, 0 },
    { 0x7647d6a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17d6a7, 96 },
    { 0x4dd7d6a8, 34 },
    { 0xc4e7d6a9, 17 },
    { 0x157d6a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2757d6af, 23 },
    { 0x70e7d6af, 95 },
    { 0x8cc7d6b1, 33 },
    { 0x46e7d6b2, 46 },
    { 0x1b77d6b0, 23 },
    { 0xa5c7d6b4, 98 },
    { 0xa337d6b2, 40 },
    { 0, 0 },
    { 0xd2e7d6b7, 85 },
    { 0, 0 },
    { 0x82a7d6b9, 91 },
    { 0xc7d6ba, 84 },
    { 0x6d7d6bb, 34 },
    { 0x7657d6bc, 23 },
    { 0x6807d6bc, 55 },
    { 0, 0 },
    { 0xd317d6bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf787d6c4, 72 },
    { 0, 0 },
    { 0x24a7d6c6, 98 },
    { 0, 0 },
    { 0x4a27d6c8, 38 },
    { 0xc97d6c9, 98 },
    { 0, 0 },
    { 0x9617d6cb, 66 },
    { 0x12b7d6cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77d6d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe047d6d5, 99 },
    { 0xe847d6d6, 61 },
    { 0xb707d6d7, 33 },
    { 0, 0 },
    { 0xf917d6d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847d6dc, 33 },
    { 0x8397d6dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b7d6e6, 23 },
    { 0, 0 },
    { 0x2f07d6e8, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617d6ec, 90 },
    { 0, 0 },
    { 0xe947d6ee, 36 },
    { 0, 0 },
    { 0x6b47d6f0, 93 },
    { 0xa837d6f1, 55 },
    { 0xe7d7d6f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde7d6f6, 90 },
    { 0, 0 },
    { 0x6627d6f8, 83 },
    { 0x33f7d6f8, 34 },
    { 0, 0 },
    { 0x7d6fb, 61 },
    { 0x63b7d6fc, 34 },
    { 0x11a7d6fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7d704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b7d707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7d70b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7667d713, 34 },
    { 0, 0 },
    { 0xe407d715, 68 },
    { 0x44d7d715, 90 },
    { 0x2507d717, 43 },
    { 0xcd17d718, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db7d720, 98 },
    { 0x17b7d721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9257d724, 23 },
    { 0xd627d725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e7d72c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e7d732, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe537d735, 17 },
    { 0x6c37d735, 38 },
    { 0, 0 },
    { 0xc887d738, 33 },
    { 0, 0 },
    { 0xfb27d73a, 76 },
    { 0, 0 },
    { 0xb97d73c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c7d744, 23 },
    { 0x58c7d745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc257d74d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa027d750, 77 },
    { 0, 0 },
    { 0x6ab7d752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb737d75c, 19 },
    { 0, 0 },
    { 0xd6a7d75e, 91 },
    { 0xba47d75f, 23 },
    { 0xf9f7d760, 85 },
    { 0x8eb7d761, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc7d764, 61 },
    { 0, 0 },
    { 0x8e7d766, 98 },
    { 0, 0 },
    { 0x1bf7d768, 34 },
    { 0xfdc7d768, 91 },
    { 0, 0 },
    { 0x4f97d76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3377d76f, 23 },
    { 0x3ce7d770, 90 },
    { 0xc47d770, 77 },
    { 0x7dd7d770, 76 },
    { 0xaa17d773, 33 },
    { 0xdb07d774, 55 },
    { 0x10a7d775, 85 },
    { 0xa2c7d771, 33 },
    { 0xbd67d777, 76 },
    { 0x1087d774, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9217d780, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc07d783, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827d787, 90 },
    { 0xecf7d787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9557d78b, 67 },
    { 0x11a7d78b, 55 },
    { 0, 0 },
    { 0xd9b7d78e, 33 },
    { 0x5007d78f, 33 },
    { 0x47c7d78e, 96 },
    { 0x6f7d791, 76 },
    { 0x2ed7d791, 23 },
    { 0x1647d792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd947d797, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6707d79c, 99 },
    { 0, 0 },
    { 0x8117d79e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637d7a5, 77 },
    { 0x26e7d7a6, 23 },
    { 0x1447d7a7, 33 },
    { 0xc5a7d7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1317d7ad, 43 },
    { 0xe057d7ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f77d7b6, 43 },
    { 0xc6e7d7b7, 78 },
    { 0x5597d7b6, 43 },
    { 0xdc37d7b7, 40 },
    { 0x82f7d7ba, 88 },
    { 0x7087d7ba, 46 },
    { 0x96f7d7bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7d7c1, 98 },
    { 0xb5c7d7c1, 61 },
    { 0x94e7d7c3, 34 },
    { 0x76b7d7c3, 98 },
    { 0, 0 },
    { 0x96f7d7c6, 90 },
    { 0, 0 },
    { 0xa027d7c8, 98 },
    { 0xa4e7d7c8, 67 },
    { 0xbac7d7c9, 61 },
    { 0x76c7d7cb, 61 },
    { 0xf9e7d7c8, 33 },
    { 0x40c7d7cc, 78 },
    { 0, 0 },
    { 0x6707d7cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7d7d2, 43 },
    { 0, 0 },
    { 0xb3a7d7d4, 34 },
    { 0, 0 },
    { 0x1d87d7d6, 23 },
    { 0, 0 },
    { 0xc777d7d8, 33 },
    { 0x2407d7d8, 98 },
    { 0x2af7d7d9, 98 },
    { 0, 0 },
    { 0xbed7d7dc, 33 },
    { 0xde97d7dd, 61 },
    { 0, 0 },
    { 0x8ab7d7df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c47d7e4, 34 },
    { 0x527d7e5, 55 },
    { 0xe6b7d7e6, 52 },
    { 0xb5c7d7e7, 88 },
    { 0xca7d7e7, 43 },
    { 0x4b07d7e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6247d7ef, 95 },
    { 0x2397d7ef, 95 },
    { 0xb317d7f1, 67 },
    { 0x4cf7d7f2, 61 },
    { 0xf2a7d7f3, 85 },
    { 0, 0 },
    { 0xfe67d7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2737d7fd, 98 },
    { 0, 0 },
    { 0xaec7d7ff, 23 },
    { 0xcc97d7ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1537d80b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff7d811, 90 },
    { 0x407d812, 61 },
    { 0xbf37d813, 98 },
    { 0, 0 },
    { 0xc317d815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b17d81c, 55 },
    { 0x4fb7d81d, 61 },
    { 0xe9b7d81d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d822, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa7d829, 55 },
    { 0xff87d829, 90 },
    { 0xf217d829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b7d831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3127d836, 61 },
    { 0, 0 },
    { 0x7b87d838, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba7d83b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a7d83f, 95 },
    { 0xae07d840, 55 },
    { 0x7fe7d83f, 33 },
    { 0, 0 },
    { 0x8d57d843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57d847, 98 },
    { 0x6c17d847, 61 },
    { 0x6637d847, 95 },
    { 0x1997d847, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287d851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7d857, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1447d85a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c87d85d, 90 },
    { 0x8817d85d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7d862, 90 },
    { 0x5b77d862, 91 },
    { 0x1077d863, 55 },
    { 0xae67d863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5527d86b, 98 },
    { 0xde97d86c, 90 },
    { 0, 0 },
    { 0x5057d86e, 77 },
    { 0xbaa7d86f, 61 },
    { 0xd27d870, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xce17d873, 100 },
    { 0xf797d873, 61 },
    { 0xd777d875, 85 },
    { 0xbc87d873, 55 },
    { 0xd4a7d877, 55 },
    { 0xaa97d873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57d87e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7d885, 68 },
    { 0, 0 },
    { 0x5da7d887, 65 },
    { 0x47d888, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857d88b, 84 },
    { 0xfa97d88c, 76 },
    { 0xce77d88d, 55 },
    { 0xe447d88e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7d891, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c7d896, 90 },
    { 0xd717d897, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7d89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367d89f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda07d8a2, 85 },
    { 0, 0 },
    { 0xdd77d8a4, 34 },
    { 0x5b97d8a5, 61 },
    { 0x20f7d8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4417d8a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e7d8ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657d8af, 88 },
    { 0xf947d8af, 33 },
    { 0xc577d8b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067d8b5, 33 },
    { 0, 0 },
    { 0xd1d7d8b7, 90 },
    { 0x3797d8b8, 95 },
    { 0, 0 },
    { 0xcbc7d8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a7d8c4, 43 },
    { 0, 0 },
    { 0xd1f7d8c6, 101 },
    { 0xdcf7d8c7, 55 },
    { 0x5137d8c8, 90 },
    { 0x9bb7d8c8, 78 },
    { 0x9207d8ca, 90 },
    { 0xd3e7d8ca, 90 },
    { 0x8a27d8c9, 90 },
    { 0x8e97d8cd, 34 },
    { 0x5f47d8ce, 90 },
    { 0xd0c7d8cf, 91 },
    { 0, 0 },
    { 0xdc57d8d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4217d8d5, 77 },
    { 0x237d8d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a7d8d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f7d8dc, 66 },
    { 0x7137d8dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b7d8e1, 75 },
    { 0x7c67d8e2, 46 },
    { 0xbf07d8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7d8e9, 80 },
    { 0x5447d8e9, 75 },
    { 0x4617d8ea, 100 },
    { 0xd77d8ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c7d8ef, 33 },
    { 0x71d7d8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07d8f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b67d8f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd727d8fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557d8fe, 76 },
    { 0xdbb7d8ff, 55 },
    { 0xdbe7d900, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c7d909, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3737d90e, 90 },
    { 0xa557d90f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a67d914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa707d919, 93 },
    { 0xd4a7d919, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a7d91f, 99 },
    { 0, 0 },
    { 0xc077d921, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c87d929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba7d92d, 34 },
    { 0x2337d92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77d931, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7d93b, 23 },
    { 0xca17d93b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577d93f, 23 },
    { 0xefd7d93f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3887d947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87d94e, 84 },
    { 0x2007d94f, 76 },
    { 0xc2d7d950, 55 },
    { 0, 0 },
    { 0xe317d952, 55 },
    { 0x4d37d953, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f77d958, 55 },
    { 0xe027d959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc7d95e, 85 },
    { 0xf757d95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f7d969, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf67d96c, 98 },
    { 0x207d96d, 90 },
    { 0x5127d96e, 55 },
    { 0x7297d96c, 43 },
    { 0xd477d970, 68 },
    { 0xee47d970, 95 },
    { 0, 0 },
    { 0xcd87d973, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77d978, 93 },
    { 0x1c67d978, 66 },
    { 0, 0 },
    { 0xf8f7d97b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a7d97e, 66 },
    { 0x92c7d97e, 90 },
    { 0x9517d980, 95 },
    { 0x2417d981, 23 },
    { 0x1dc7d982, 75 },
    { 0x76c7d981, 87 },
    { 0x1497d984, 23 },
    { 0x8097d984, 33 },
    { 0, 0 },
    { 0x7f87d987, 90 },
    { 0x95b7d987, 98 },
    { 0x67d7d988, 61 },
    { 0xe287d989, 43 },
    { 0xe437d98b, 91 },
    { 0xf27d98c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4407d991, 88 },
    { 0xd837d991, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac17d999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e7d99c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc47d9a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb77d9a8, 61 },
    { 0x4057d9a8, 23 },
    { 0x3a77d9a9, 95 },
    { 0, 0 },
    { 0x7667d9ac, 95 },
    { 0xe567d9ac, 23 },
    { 0x1667d9ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd7d9b2, 19 },
    { 0, 0 },
    { 0x5197d9b4, 61 },
    { 0xeac7d9b4, 95 },
    { 0x2077d9b6, 99 },
    { 0, 0 },
    { 0x3db7d9b8, 34 },
    { 0, 0 },
    { 0xd6c7d9ba, 23 },
    { 0, 0 },
    { 0xb837d9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47d9c0, 95 },
    { 0x7697d9c1, 93 },
    { 0, 0 },
    { 0xffe7d9c3, 33 },
    { 0xe047d9c3, 98 },
    { 0x6b67d9c5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7d9c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77d9d0, 98 },
    { 0xa957d9d0, 33 },
    { 0x5f27d9d1, 46 },
    { 0x73c7d9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb7d9d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397d9da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d87d9dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae7d9e2, 93 },
    { 0x45a7d9e3, 23 },
    { 0x88f7d9e2, 98 },
    { 0xba27d9e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b7d9e8, 50 },
    { 0x2247d9e8, 61 },
    { 0x6f57d9ea, 90 },
    { 0xd2d7d9e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e7d9ee, 95 },
    { 0xc9b7d9ef, 43 },
    { 0x8257d9f0, 84 },
    { 0, 0 },
    { 0x69d7d9f2, 98 },
    { 0, 0 },
    { 0xc697d9f4, 90 },
    { 0xc517d9f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c7d9fd, 95 },
    { 0, 0 },
    { 0x41f7d9ff, 34 },
    { 0, 0 },
    { 0x93d7da01, 76 },
    { 0xadd7da02, 61 },
    { 0x8b87da03, 61 },
    { 0x37da02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597da08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7da10, 95 },
    { 0, 0 },
    { 0x77da12, 33 },
    { 0x6fe7da13, 99 },
    { 0xffd7da14, 61 },
    { 0x2897da14, 33 },
    { 0x9677da16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77da1d, 90 },
    { 0x55d7da1e, 61 },
    { 0x2c37da1f, 90 },
    { 0xa387da1f, 99 },
    { 0xb5c7da21, 68 },
    { 0xa417da21, 76 },
    { 0xabc7da22, 52 },
    { 0xcbe7da21, 40 },
    { 0x1f77da25, 60 },
    { 0xe037da26, 90 },
    { 0x3ce7da27, 88 },
    { 0x6507da26, 33 },
    { 0xfd97da26, 90 },
    { 0xfda7da2a, 55 },
    { 0xcaa7da24, 33 },
    { 0x1df7da2c, 52 },
    { 0xf0a7da2d, 43 },
    { 0xaf37da1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa7da34, 43 },
    { 0x3387da35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6437da39, 76 },
    { 0x3bd7da39, 90 },
    { 0x1b87da3b, 23 },
    { 0x1c67da3a, 98 },
    { 0x49d7da3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657da40, 91 },
    { 0, 0 },
    { 0x5167da42, 23 },
    { 0xff37da42, 23 },
    { 0x17da42, 98 },
    { 0, 0 },
    { 0xa657da46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d57da49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567da4f, 91 },
    { 0x8477da4f, 55 },
    { 0x8f67da51, 91 },
    { 0x567da51, 95 },
    { 0xf87da53, 90 },
    { 0xd167da54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3647da58, 33 },
    { 0xdf67da59, 61 },
    { 0x7ae7da5a, 73 },
    { 0x3c07da59, 61 },
    { 0x6907da5c, 96 },
    { 0, 0 },
    { 0x7337da5e, 46 },
    { 0, 0 },
    { 0x8c77da60, 98 },
    { 0xc97da60, 93 },
    { 0x9667da62, 76 },
    { 0xaa27da63, 61 },
    { 0x3787da64, 55 },
    { 0x8e7da64, 98 },
    { 0, 0 },
    { 0xb147da67, 99 },
    { 0, 0 },
    { 0xa157da69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7da6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f7da72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2447da7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a47da85, 34 },
    { 0, 0 },
    { 0x5f7da87, 97 },
    { 0x43a7da87, 61 },
    { 0x48e7da89, 66 },
    { 0x897da89, 81 },
    { 0x2c77da8b, 95 },
    { 0x9b87da8a, 19 },
    { 0x9b77da8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4737da90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba7da94, 23 },
    { 0, 0 },
    { 0xda77da96, 55 },
    { 0xfed7da97, 61 },
    { 0, 0 },
    { 0x61d7da99, 55 },
    { 0x16c7da9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7da9d, 78 },
    { 0xfe47da9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137daa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127daa7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2087daad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa7dab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7227dab6, 66 },
    { 0, 0 },
    { 0xc327dab8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237dabc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb997dabf, 90 },
    { 0, 0 },
    { 0xf507dac1, 55 },
    { 0xab47dac2, 90 },
    { 0, 0 },
    { 0xddd7dac4, 23 },
    { 0xccd7dac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277dad0, 46 },
    { 0x4647dad1, 99 },
    { 0x8a47dad2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d7dad7, 76 },
    { 0, 0 },
    { 0xddc7dad9, 98 },
    { 0xa8e7dada, 46 },
    { 0xcbe7dad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297dade, 61 },
    { 0xa297dadf, 98 },
    { 0, 0 },
    { 0x8707dae1, 76 },
    { 0, 0 },
    { 0xc0d7dae3, 61 },
    { 0xf257dae4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb97dae8, 76 },
    { 0x1ca7dae9, 61 },
    { 0xd3c7daea, 43 },
    { 0, 0 },
    { 0x30a7daec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47daef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6147daf4, 52 },
    { 0xf87daf4, 23 },
    { 0x42f7daf6, 55 },
    { 0, 0 },
    { 0xc337daf8, 33 },
    { 0x9ef7daf8, 95 },
    { 0x46e7dafa, 66 },
    { 0xe207daf8, 33 },
    { 0xe177daf8, 90 },
    { 0x4dd7dafd, 77 },
    { 0xbd57daf9, 23 },
    { 0xc487daf9, 90 },
    { 0xe157daff, 98 },
    { 0x777daf9, 38 },
    { 0xfb27db02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47db05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c87db0e, 17 },
    { 0x8ac7db0f, 98 },
    { 0xaa27db10, 33 },
    { 0x6167db11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e07db1a, 46 },
    { 0, 0 },
    { 0xad77db1c, 81 },
    { 0x5af7db1d, 34 },
    { 0x10b7db1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c7db21, 33 },
    { 0xf047db22, 77 },
    { 0, 0 },
    { 0x3107db24, 61 },
    { 0xa757db24, 78 },
    { 0x39e7db24, 46 },
    { 0, 0 },
    { 0x1b37db28, 99 },
    { 0, 0 },
    { 0x9c17db2a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8517db2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd837db39, 23 },
    { 0xcac7db3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7db41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee7db47, 43 },
    { 0xaa67db47, 40 },
    { 0, 0 },
    { 0x9b37db4a, 98 },
    { 0x8a27db4b, 95 },
    { 0x55f7db4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507db53, 90 },
    { 0xf4c7db53, 61 },
    { 0xb007db55, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187db5b, 95 },
    { 0x8fb7db5c, 78 },
    { 0, 0 },
    { 0xa9e7db5e, 96 },
    { 0x1467db5f, 95 },
    { 0, 0 },
    { 0x367db61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a47db65, 61 },
    { 0x1577db66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd747db69, 90 },
    { 0x7947db6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627db6d, 95 },
    { 0, 0 },
    { 0x5397db6f, 66 },
    { 0xd5b7db70, 44 },
    { 0, 0 },
    { 0xf227db72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7db77, 43 },
    { 0xa1e7db78, 90 },
    { 0, 0 },
    { 0x1567db7a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9157db89, 81 },
    { 0x3397db89, 38 },
    { 0x4eb7db8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7887db94, 90 },
    { 0x5237db94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7db9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597dba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x707dba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a37dbaf, 95 },
    { 0, 0 },
    { 0x92d7dbb1, 98 },
    { 0x5e57dbb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7dbb5, 95 },
    { 0x2cb7dbb6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4797dbb9, 61 },
    { 0, 0 },
    { 0xa777dbbb, 34 },
    { 0, 0 },
    { 0x6f57dbbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67dbc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf87dbc8, 95 },
    { 0, 0 },
    { 0x737dbca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d77dbd0, 61 },
    { 0x4b37dbd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d7dbd5, 75 },
    { 0x49b7dbd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c7dbe3, 66 },
    { 0x61d7dbe4, 98 },
    { 0xa957dbe5, 23 },
    { 0xfb67dbe5, 81 },
    { 0x8537dbe6, 52 },
    { 0x307dbe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d7dbec, 43 },
    { 0x9497dbed, 77 },
    { 0x8b27dbec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a7dbf1, 76 },
    { 0x2767dbf2, 98 },
    { 0, 0 },
    { 0xfe7dbf4, 90 },
    { 0x28c7dbf5, 95 },
    { 0x1ec7dbf6, 69 },
    { 0x7017dbf6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967dbfc, 55 },
    { 0x9ae7dbfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7dc03, 43 },
    { 0, 0 },
    { 0x2aa7dc05, 93 },
    { 0, 0 },
    { 0x53d7dc07, 77 },
    { 0x637dc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6867dc0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597dc10, 34 },
    { 0xdc87dc11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca7dc17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a7dc1c, 61 },
    { 0x27f7dc1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7dc23, 90 },
    { 0xa247dc23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde37dc29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf547dc2d, 43 },
    { 0xf427dc2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b37dc36, 23 },
    { 0, 0 },
    { 0xaae7dc38, 33 },
    { 0xfe17dc39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707dc3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127dc40, 90 },
    { 0, 0 },
    { 0x4e87dc42, 61 },
    { 0x4a07dc42, 76 },
    { 0x9be7dc44, 33 },
    { 0, 0 },
    { 0x48d7dc46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd37dc4a, 55 },
    { 0x1a07dc4b, 69 },
    { 0xbcc7dc4c, 61 },
    { 0x7c57dc4d, 95 },
    { 0xa077dc4d, 23 },
    { 0x8547dc4f, 88 },
    { 0xa6f7dc4c, 17 },
    { 0x91d7dc51, 98 },
    { 0x2017dc4d, 33 },
    { 0x7c7dc53, 88 },
    { 0xeb07dc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87dc57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd27dc5f, 33 },
    { 0, 0 },
    { 0xc687dc61, 23 },
    { 0, 0 },
    { 0x5ea7dc63, 66 },
    { 0x1357dc64, 33 },
    { 0xe077dc65, 98 },
    { 0, 0 },
    { 0x1637dc67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167dc6f, 98 },
    { 0xb9c7dc6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf7dc73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x667dc76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d7dc7a, 98 },
    { 0xadd7dc7a, 76 },
    { 0x3397dc7c, 91 },
    { 0x1b7dc7c, 61 },
    { 0x6c87dc7d, 61 },
    { 0x1f47dc7f, 61 },
    { 0x69b7dc7a, 96 },
    { 0x82f7dc7c, 100 },
    { 0x9367dc82, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a7dc87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a7dc8a, 95 },
    { 0x7277dc8b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b7dc91, 61 },
    { 0x1be7dc91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe847dc96, 90 },
    { 0, 0 },
    { 0x6b87dc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc397dc9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597dc9e, 43 },
    { 0, 0 },
    { 0xea67dca0, 23 },
    { 0xe327dca1, 90 },
    { 0xe6f7dca2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7427dca5, 78 },
    { 0x4a77dca6, 33 },
    { 0xb7c7dca5, 90 },
    { 0, 0 },
    { 0x1ff7dca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa247dcac, 85 },
    { 0x42b7dcad, 77 },
    { 0, 0 },
    { 0x3597dcaf, 98 },
    { 0x26e7dcb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be7dcb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d77dcb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f47dcbc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa87dcc1, 99 },
    { 0xb397dcc2, 34 },
    { 0xbac7dcc1, 84 },
    { 0x3427dcc4, 95 },
    { 0x9967dcc5, 98 },
    { 0xe937dcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9837dcca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7dcd0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67dcd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437dcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de7dce1, 23 },
    { 0xf2c7dce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2227dce8, 98 },
    { 0x3f37dce9, 93 },
    { 0, 0 },
    { 0xda77dceb, 84 },
    { 0x7397dcec, 23 },
    { 0x5c07dced, 61 },
    { 0xacf7dcec, 34 },
    { 0, 0 },
    { 0xd087dcf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b7dcf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617dcf8, 90 },
    { 0xd597dcf8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c7dcfd, 43 },
    { 0xa5f7dcfe, 17 },
    { 0xd257dcff, 23 },
    { 0x88b7dd00, 76 },
    { 0xddb7dcfe, 33 },
    { 0x9cf7dcfe, 61 },
    { 0xc927dd03, 43 },
    { 0xf387dcff, 98 },
    { 0x5d7dcfd, 100 },
    { 0x5ec7dcff, 90 },
    { 0, 0 },
    { 0xddf7dd08, 83 },
    { 0, 0 },
    { 0x9087dd0a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb7dd0f, 36 },
    { 0, 0 },
    { 0x3747dd11, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3507dd14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb7dd19, 33 },
    { 0xb87dd19, 43 },
    { 0xd3c7dd1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3897dd1f, 98 },
    { 0xa127dd20, 84 },
    { 0x25e7dd21, 100 },
    { 0xede7dd22, 38 },
    { 0, 0 },
    { 0x1587dd24, 23 },
    { 0x3147dd24, 61 },
    { 0, 0 },
    { 0x72e7dd27, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3627dd2a, 87 },
    { 0x1e57dd2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157dd2e, 87 },
    { 0x7ff7dd2e, 33 },
    { 0xf087dd2f, 61 },
    { 0xa7d7dd2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e77dd34, 95 },
    { 0x6747dd34, 33 },
    { 0x3b7dd36, 17 },
    { 0x9527dd36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567dd3a, 52 },
    { 0x6fe7dd3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c17dd3e, 55 },
    { 0x2d57dd3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257dd42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b7dd47, 76 },
    { 0xea27dd48, 33 },
    { 0xc6d7dd48, 88 },
    { 0x42e7dd49, 99 },
    { 0x85c7dd4a, 95 },
    { 0x1567dd4c, 34 },
    { 0xc9c7dd4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7dd50, 61 },
    { 0, 0 },
    { 0x357dd52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227dd5a, 90 },
    { 0x6e7dd5b, 66 },
    { 0, 0 },
    { 0x11c7dd5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f7dd64, 46 },
    { 0xabc7dd65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557dd6b, 67 },
    { 0, 0 },
    { 0x7d87dd6d, 99 },
    { 0, 0 },
    { 0x2917dd6f, 33 },
    { 0xb017dd70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7dd73, 55 },
    { 0x4e67dd74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d67dd78, 77 },
    { 0xb6a7dd79, 75 },
    { 0x8307dd7a, 61 },
    { 0x25c7dd7b, 34 },
    { 0x8a97dd79, 84 },
    { 0, 0 },
    { 0x3fa7dd7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577dd8a, 61 },
    { 0x8f67dd8b, 98 },
    { 0x6557dd8a, 33 },
    { 0xd327dd8d, 69 },
    { 0, 0 },
    { 0x4c17dd8f, 77 },
    { 0, 0 },
    { 0x7487dd91, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c27dd98, 17 },
    { 0xec7dd98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6587dda1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8967dda6, 61 },
    { 0x21b7dda6, 76 },
    { 0, 0 },
    { 0xa687dda9, 84 },
    { 0, 0 },
    { 0xc0e7ddab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf387ddb0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe087ddb5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27ddb9, 90 },
    { 0x5da7ddba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde17ddbf, 95 },
    { 0, 0 },
    { 0x137ddc1, 90 },
    { 0x1f47ddc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee7ddc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287ddcd, 84 },
    { 0xb147ddce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17ddd1, 95 },
    { 0, 0 },
    { 0xd47ddd3, 61 },
    { 0x35d7ddd3, 66 },
    { 0x4c07ddd5, 77 },
    { 0, 0 },
    { 0xf457ddd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb947ddda, 34 },
    { 0, 0 },
    { 0x7c67dddc, 78 },
    { 0xba37dddc, 76 },
    { 0, 0 },
    { 0xd167dddf, 93 },
    { 0x6287dde0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67dde6, 87 },
    { 0x5ea7dde7, 61 },
    { 0x6b17dde7, 23 },
    { 0, 0 },
    { 0x6cc7ddea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e37ddef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf67ddf5, 33 },
    { 0xd157ddf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797ddfa, 55 },
    { 0xc467ddfa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797de02, 95 },
    { 0x51d7de03, 78 },
    { 0x14d7de04, 33 },
    { 0x2927de05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde7de0c, 76 },
    { 0xe587de0d, 90 },
    { 0x2ce7de0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc597de12, 67 },
    { 0xb5f7de12, 4 },
    { 0xb5a7de14, 90 },
    { 0xfff7de15, 90 },
    { 0xc237de15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c7de1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17de1e, 23 },
    { 0x4cb7de1f, 23 },
    { 0x43f7de20, 98 },
    { 0x86c7de21, 65 },
    { 0xbca7de21, 90 },
    { 0xfb7de23, 95 },
    { 0xd567de23, 33 },
    { 0, 0 },
    { 0xad47de26, 33 },
    { 0, 0 },
    { 0x8007de28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2397de2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa77de2e, 98 },
    { 0x9e67de2f, 90 },
    { 0, 0 },
    { 0x96f7de31, 90 },
    { 0x9c77de31, 66 },
    { 0x7827de31, 98 },
    { 0xba87de32, 98 },
    { 0x5b07de33, 55 },
    { 0x6647de34, 96 },
    { 0xf567de37, 80 },
    { 0x9f07de37, 91 },
    { 0xfb77de37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5297de3f, 23 },
    { 0, 0 },
    { 0xc6c7de41, 61 },
    { 0, 0 },
    { 0xa337de43, 61 },
    { 0, 0 },
    { 0x177de45, 23 },
    { 0xfcb7de46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c7de4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b7de51, 93 },
    { 0x72e7de51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5727de57, 90 },
    { 0xfdc7de57, 76 },
    { 0xdba7de57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7de5c, 33 },
    { 0x9547de5c, 76 },
    { 0, 0 },
    { 0x3347de5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf437de64, 77 },
    { 0x8c7de65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de7de68, 90 },
    { 0x3147de69, 61 },
    { 0x5877de6a, 61 },
    { 0xf3f7de69, 98 },
    { 0xce47de6b, 95 },
    { 0, 0 },
    { 0xe3d7de6e, 76 },
    { 0, 0 },
    { 0xaf97de70, 85 },
    { 0x8a97de71, 34 },
    { 0x6757de70, 66 },
    { 0x6ed7de72, 66 },
    { 0x4027de74, 43 },
    { 0, 0 },
    { 0xadb7de76, 33 },
    { 0x5507de76, 91 },
    { 0x3cd7de77, 67 },
    { 0x9897de79, 17 },
    { 0, 0 },
    { 0xfe97de7b, 69 },
    { 0xd327de7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9647de7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a7de82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4557de87, 46 },
    { 0, 0 },
    { 0xe6b7de89, 33 },
    { 0xb6f7de8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f7de8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b37de91, 61 },
    { 0, 0 },
    { 0x3537de93, 90 },
    { 0x83d7de94, 93 },
    { 0x6227de95, 99 },
    { 0x9097de94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7de9c, 85 },
    { 0xbc57de9d, 46 },
    { 0xe517de9e, 76 },
    { 0, 0 },
    { 0x8c57dea0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c67dea5, 84 },
    { 0x33c7dea6, 80 },
    { 0x2c27dea6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f7deaa, 92 },
    { 0xe597deab, 23 },
    { 0x4c97deab, 78 },
    { 0, 0 },
    { 0x837deae, 84 },
    { 0xa947deaf, 55 },
    { 0x12f7deaf, 33 },
    { 0, 0 },
    { 0x3a37deb2, 93 },
    { 0x2557deb2, 76 },
    { 0x1497deb4, 55 },
    { 0, 0 },
    { 0x3737deb6, 93 },
    { 0xafb7deb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec97debc, 90 },
    { 0, 0 },
    { 0x9bf7debe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8677dec2, 95 },
    { 0xc867dec2, 90 },
    { 0, 0 },
    { 0x1e27dec5, 90 },
    { 0xe3e7dec6, 100 },
    { 0x2637dec7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b7decb, 95 },
    { 0, 0 },
    { 0x9127decd, 65 },
    { 0x2727dece, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2107ded4, 61 },
    { 0x2d07ded5, 23 },
    { 0, 0 },
    { 0xdda7ded7, 46 },
    { 0, 0 },
    { 0xe387ded9, 88 },
    { 0x8877deda, 32 },
    { 0x4d77ded9, 34 },
    { 0xbc47dedb, 78 },
    { 0, 0 },
    { 0x187dede, 50 },
    { 0x7257dedf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7dee5, 33 },
    { 0, 0 },
    { 0x6c97dee7, 89 },
    { 0xb77dee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc7deee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677def2, 46 },
    { 0x1837def3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7defa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67df00, 33 },
    { 0xaaf7df01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe087df04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147df12, 34 },
    { 0xea07df12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a87df17, 76 },
    { 0, 0 },
    { 0x99d7df19, 23 },
    { 0x6947df1a, 76 },
    { 0, 0 },
    { 0xa637df1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6657df1f, 81 },
    { 0x7337df20, 33 },
    { 0x54e7df21, 90 },
    { 0x88b7df20, 76 },
    { 0, 0 },
    { 0xd07df24, 53 },
    { 0, 0 },
    { 0xea47df26, 90 },
    { 0x26b7df27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547df2e, 23 },
    { 0, 0 },
    { 0x5657df30, 61 },
    { 0x7927df31, 61 },
    { 0, 0 },
    { 0x4797df33, 93 },
    { 0x6f47df34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc317df37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27df3c, 34 },
    { 0, 0 },
    { 0xfb37df3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57df41, 93 },
    { 0x5ab7df42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf7df45, 68 },
    { 0xfde7df46, 88 },
    { 0xe897df46, 48 },
    { 0xc207df48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb947df4c, 90 },
    { 0xba17df4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5327df50, 52 },
    { 0xdd97df50, 98 },
    { 0xded7df52, 99 },
    { 0xc207df53, 33 },
    { 0, 0 },
    { 0x5fc7df55, 83 },
    { 0, 0 },
    { 0x1ed7df57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3027df5a, 64 },
    { 0, 0 },
    { 0x7757df5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f7df5f, 90 },
    { 0x8037df60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d7df64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a7df6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7df6e, 43 },
    { 0x3497df6f, 90 },
    { 0x1cb7df70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xab57df73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9827df76, 93 },
    { 0x5f07df77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda57df7c, 95 },
    { 0x6a97df7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787df80, 87 },
    { 0xeef7df81, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba7df86, 78 },
    { 0xb917df86, 90 },
    { 0, 0 },
    { 0xc2b7df89, 76 },
    { 0x1347df8a, 99 },
    { 0x9337df8b, 98 },
    { 0xd4f7df8b, 23 },
    { 0, 0 },
    { 0xef7df8e, 55 },
    { 0x8de7df8e, 95 },
    { 0x54e7df90, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd767df94, 98 },
    { 0xc3e7df95, 43 },
    { 0x28c7df96, 99 },
    { 0x6497df97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77df9b, 90 },
    { 0x94a7df9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf607df9f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737dfa5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d7dfa9, 34 },
    { 0, 0 },
    { 0xe1c7dfab, 95 },
    { 0x5b37dfab, 76 },
    { 0x7007dfac, 94 },
    { 0, 0 },
    { 0xd187dfaf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1897dfb3, 81 },
    { 0x7a17dfb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7dfb7, 17 },
    { 0, 0 },
    { 0x55e7dfb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3987dfbe, 93 },
    { 0xc227dfbf, 33 },
    { 0, 0 },
    { 0x2267dfc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437dfc7, 98 },
    { 0x3f47dfc8, 61 },
    { 0, 0 },
    { 0xcfa7dfca, 55 },
    { 0xeaa7dfcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b7dfd1, 95 },
    { 0xae77dfd2, 61 },
    { 0xb067dfd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947dfd8, 95 },
    { 0, 0 },
    { 0xf3c7dfda, 23 },
    { 0x4107dfdb, 46 },
    { 0x6007dfdc, 65 },
    { 0x9977dfdc, 98 },
    { 0xeee7dfde, 83 },
    { 0x3fd7dfdc, 61 },
    { 0, 0 },
    { 0xd747dfe1, 91 },
    { 0xbf37dfe2, 33 },
    { 0x8ce7dfe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07dfe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197dfea, 33 },
    { 0xa3c7dfeb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc7dfef, 33 },
    { 0x9947dff0, 95 },
    { 0xee77dfef, 23 },
    { 0, 0 },
    { 0x3447dff3, 90 },
    { 0x74f7dff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c7dff7, 23 },
    { 0x63b7dff8, 33 },
    { 0x42c7dff9, 17 },
    { 0x8d27dff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfff, 90 },
    { 0x77c7dfff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c27e003, 23 },
    { 0x1847e004, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa457e007, 40 },
    { 0x7247e008, 95 },
    { 0, 0 },
    { 0xa817e00a, 33 },
    { 0, 0 },
    { 0x8837e00c, 55 },
    { 0xdbf7e00d, 55 },
    { 0xc9e7e00d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e7e014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7667e018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e021, 51 },
    { 0x1367e022, 98 },
    { 0xa0f7e023, 98 },
    { 0, 0 },
    { 0x5cd7e025, 61 },
    { 0, 0 },
    { 0x717e027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7e02d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2197e035, 48 },
    { 0x9df7e036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef7e03c, 90 },
    { 0, 0 },
    { 0x9ae7e03e, 88 },
    { 0x9137e03f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1357e044, 95 },
    { 0xf147e045, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1507e04d, 55 },
    { 0xb337e04e, 55 },
    { 0, 0 },
    { 0x2fc7e050, 90 },
    { 0, 0 },
    { 0x72a7e052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7077e056, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e7e059, 88 },
    { 0x4c67e05a, 61 },
    { 0x1787e059, 93 },
    { 0xa2e7e05c, 55 },
    { 0x6a87e05c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4997e061, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d7e065, 55 },
    { 0xbea7e066, 46 },
    { 0xb3e7e067, 90 },
    { 0, 0 },
    { 0x7717e069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057e072, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e7e076, 58 },
    { 0xcf87e077, 69 },
    { 0x5487e078, 98 },
    { 0x20e7e079, 90 },
    { 0xbba7e07a, 33 },
    { 0x1487e079, 76 },
    { 0x43e7e078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67e07f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2457e085, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9017e08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c7e08e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda7e095, 33 },
    { 0, 0 },
    { 0xe757e097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7e09c, 61 },
    { 0xe027e09d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced7e0ab, 33 },
    { 0x3f47e0ab, 90 },
    { 0xcde7e0ad, 76 },
    { 0xf027e0ad, 90 },
    { 0x3537e0ae, 46 },
    { 0x61d7e0ad, 33 },
    { 0x7067e0b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe537e0b6, 61 },
    { 0, 0 },
    { 0x6157e0b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe827e0c3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7e0c8, 23 },
    { 0xd267e0c9, 63 },
    { 0x717e0ca, 88 },
    { 0, 0 },
    { 0x2387e0cc, 98 },
    { 0, 0 },
    { 0xb147e0ce, 43 },
    { 0, 0 },
    { 0x8e67e0d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d7e0d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b7e0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf57e0dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c57e0e2, 33 },
    { 0xd357e0e2, 36 },
    { 0x93d7e0e4, 95 },
    { 0x2107e0e3, 98 },
    { 0x9507e0e6, 98 },
    { 0x2a17e0e5, 76 },
    { 0xc867e0e8, 87 },
    { 0xf3f7e0e9, 85 },
    { 0x8bc7e0e8, 55 },
    { 0xe727e0eb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a7e0ee, 95 },
    { 0xe267e0ee, 61 },
    { 0x6467e0f0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf57e0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c67e0f9, 85 },
    { 0, 0 },
    { 0x6707e0fb, 88 },
    { 0, 0 },
    { 0x4e47e0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7e103, 55 },
    { 0x7ad7e104, 53 },
    { 0xcd37e105, 93 },
    { 0x7e106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a7e10b, 61 },
    { 0xa6f7e10b, 61 },
    { 0xed57e10c, 98 },
    { 0xa2a7e10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c7e111, 61 },
    { 0xc9f7e112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1617e115, 33 },
    { 0, 0 },
    { 0x63d7e117, 76 },
    { 0x6ad7e118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b27e11b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227e11f, 98 },
    { 0xd5f7e120, 65 },
    { 0x98c7e120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197e125, 90 },
    { 0x15c7e126, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde77e12b, 33 },
    { 0xb317e12b, 99 },
    { 0xde67e12b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb37e131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb017e134, 76 },
    { 0xb497e135, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c7e138, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7e13c, 23 },
    { 0, 0 },
    { 0x3437e13e, 99 },
    { 0x4977e13e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7e144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe207e14c, 33 },
    { 0xc577e14c, 46 },
    { 0x4097e14c, 23 },
    { 0x4167e14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7e154, 61 },
    { 0xb617e154, 23 },
    { 0xea47e154, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917e161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e7e165, 81 },
    { 0xb57e165, 22 },
    { 0xaa7e166, 90 },
    { 0x5737e168, 98 },
    { 0, 0 },
    { 0xacd7e16a, 76 },
    { 0x12f7e16b, 55 },
    { 0x63d7e16a, 43 },
    { 0, 0 },
    { 0x2a07e16e, 33 },
    { 0xcf77e16f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57e172, 61 },
    { 0x4217e173, 46 },
    { 0x40c7e174, 90 },
    { 0x3ff7e175, 90 },
    { 0x2407e176, 88 },
    { 0x9cb7e176, 76 },
    { 0x637e178, 98 },
    { 0, 0 },
    { 0xa877e17a, 61 },
    { 0xb087e17a, 33 },
    { 0x3977e17c, 90 },
    { 0x4dc7e17d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7e181, 46 },
    { 0, 0 },
    { 0x9527e183, 60 },
    { 0x45f7e183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787e188, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5827e18c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc627e192, 61 },
    { 0x29e7e193, 80 },
    { 0x46d7e193, 95 },
    { 0x1747e193, 61 },
    { 0, 0 },
    { 0x8647e197, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8477e19a, 100 },
    { 0xef27e19a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517e19f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5677e1a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc107e1a9, 98 },
    { 0xc227e1aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7e1ad, 48 },
    { 0, 0 },
    { 0xb147e1af, 90 },
    { 0xc6e7e1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767e1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217e1ba, 33 },
    { 0xc97e1ba, 66 },
    { 0xbc77e1bc, 98 },
    { 0x1367e1bd, 23 },
    { 0, 0 },
    { 0x44f7e1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557e1c3, 90 },
    { 0, 0 },
    { 0x3767e1c5, 61 },
    { 0xdd7e1c6, 95 },
    { 0xf27e1c6, 76 },
    { 0xac17e1c8, 84 },
    { 0x4e57e1c9, 98 },
    { 0, 0 },
    { 0xa7e7e1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7e1d2, 46 },
    { 0x187e1d3, 46 },
    { 0x22a7e1d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f7e1da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b7e1de, 23 },
    { 0, 0 },
    { 0xa1a7e1e0, 79 },
    { 0, 0 },
    { 0x8ea7e1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4077e1ec, 91 },
    { 0, 0 },
    { 0x92d7e1ee, 98 },
    { 0x8837e1ef, 61 },
    { 0, 0 },
    { 0x29e7e1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47e1f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb7e1f8, 88 },
    { 0xe517e1f9, 33 },
    { 0xd937e1fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17e1fd, 33 },
    { 0xa117e1fe, 95 },
    { 0xfb17e1fd, 95 },
    { 0x7137e1ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7e205, 76 },
    { 0x5bc7e206, 23 },
    { 0xc4a7e205, 93 },
    { 0xe917e208, 33 },
    { 0, 0 },
    { 0x95e7e20a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867e20d, 80 },
    { 0xe7a7e20e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7e212, 55 },
    { 0x3457e213, 90 },
    { 0xf6f7e214, 36 },
    { 0xe197e214, 23 },
    { 0xfc07e215, 33 },
    { 0x8be7e214, 90 },
    { 0, 0 },
    { 0xcac7e219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27e21f, 95 },
    { 0xc037e21f, 99 },
    { 0xdf47e221, 61 },
    { 0xddf7e222, 55 },
    { 0x4337e223, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b7e229, 84 },
    { 0x8957e22a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3897e22e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a7e232, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9067e236, 34 },
    { 0x62f7e237, 61 },
    { 0xb117e238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7e23b, 78 },
    { 0x2a67e23c, 90 },
    { 0, 0 },
    { 0x4ac7e23e, 87 },
    { 0x9157e23f, 95 },
    { 0x6847e240, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee7e245, 98 },
    { 0x5f87e246, 33 },
    { 0x1597e245, 98 },
    { 0xf9c7e248, 34 },
    { 0x8aa7e248, 90 },
    { 0xa287e24a, 90 },
    { 0x6ea7e24b, 33 },
    { 0xe887e249, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7817e24f, 95 },
    { 0xc3f7e250, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87e253, 84 },
    { 0xe967e253, 95 },
    { 0xc5a7e254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d7e25d, 90 },
    { 0xf07e25d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6927e263, 84 },
    { 0x5227e264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c7e267, 99 },
    { 0, 0 },
    { 0xb407e269, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497e26c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff77e273, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb37e27b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb7e27f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7e282, 90 },
    { 0xd597e283, 91 },
    { 0x9777e282, 33 },
    { 0x81e7e285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b77e28d, 61 },
    { 0x6267e28d, 75 },
    { 0xa277e28f, 34 },
    { 0xfcf7e28d, 23 },
    { 0x4a77e28f, 33 },
    { 0xa317e28d, 33 },
    { 0x447e293, 61 },
    { 0x50c7e293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7e297, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e7e29a, 61 },
    { 0x9c7e29b, 90 },
    { 0x2a57e29a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747e29f, 61 },
    { 0x6277e2a0, 90 },
    { 0xc047e29f, 87 },
    { 0xddc7e2a2, 78 },
    { 0x7987e2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9817e2a7, 88 },
    { 0x30c7e2a8, 95 },
    { 0, 0 },
    { 0xe9a7e2aa, 93 },
    { 0x3bd7e2ab, 88 },
    { 0xf1f7e2ac, 33 },
    { 0x3027e2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6267e2b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f7e2b3, 90 },
    { 0x3c47e2b3, 58 },
    { 0, 0 },
    { 0x8177e2b6, 67 },
    { 0x3d07e2b7, 23 },
    { 0xe7e7e2b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd917e2bb, 100 },
    { 0x4337e2bc, 98 },
    { 0xb147e2bd, 59 },
    { 0, 0 },
    { 0x9dd7e2bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c17e2c4, 95 },
    { 0x207e2c4, 61 },
    { 0xf7a7e2c5, 95 },
    { 0x86b7e2c5, 91 },
    { 0x5537e2c8, 76 },
    { 0xe677e2c9, 77 },
    { 0, 0 },
    { 0x1577e2cb, 88 },
    { 0xfa57e2cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee7e2d1, 52 },
    { 0x13f7e2d1, 23 },
    { 0xe1a7e2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127e2d7, 66 },
    { 0x3c07e2d8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7e2de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c7e2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77e2e8, 90 },
    { 0xbdc7e2e8, 95 },
    { 0x74e7e2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f7e2ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a7e2f3, 61 },
    { 0x3537e2f4, 90 },
    { 0x3b37e2f5, 98 },
    { 0xe7a7e2f6, 95 },
    { 0x9607e2f4, 34 },
    { 0x4e97e2f4, 34 },
    { 0xb417e2f9, 98 },
    { 0xb397e2f9, 23 },
    { 0, 0 },
    { 0x2827e2fc, 90 },
    { 0x49d7e2fd, 95 },
    { 0x29c7e2fe, 61 },
    { 0xfaa7e2ff, 55 },
    { 0xd17e2fe, 43 },
    { 0x29e7e2fe, 33 },
    { 0, 0 },
    { 0xbfd7e303, 90 },
    { 0, 0 },
    { 0x5737e305, 75 },
    { 0xedc7e306, 33 },
    { 0xa467e306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27e30a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e30f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7e313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7e318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7e324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec27e327, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b47e32f, 33 },
    { 0x7927e330, 61 },
    { 0xeed7e331, 90 },
    { 0xfcc7e330, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f7e336, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca7e33f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de7e342, 81 },
    { 0x3cd7e343, 46 },
    { 0xcad7e344, 43 },
    { 0, 0 },
    { 0x50f7e346, 43 },
    { 0xafc7e347, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7e359, 76 },
    { 0, 0 },
    { 0x1227e35b, 87 },
    { 0x8367e35c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4437e361, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe597e364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57e367, 23 },
    { 0, 0 },
    { 0x18a7e369, 90 },
    { 0xdbe7e36a, 95 },
    { 0, 0 },
    { 0xd017e36c, 33 },
    { 0x10c7e36c, 61 },
    { 0x61b7e36e, 68 },
    { 0, 0 },
    { 0x2da7e370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227e374, 76 },
    { 0x3a27e375, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7e378, 38 },
    { 0, 0 },
    { 0xb2e7e37a, 61 },
    { 0, 0 },
    { 0xefe7e37c, 95 },
    { 0xe727e37d, 17 },
    { 0xf827e37e, 90 },
    { 0xb897e37e, 33 },
    { 0xd237e37e, 61 },
    { 0, 0 },
    { 0x79a7e382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb617e385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557e388, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5427e38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7e395, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe767e39b, 61 },
    { 0xd617e39c, 43 },
    { 0xc1b7e39d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb807e3a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e47e3a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c7e3ad, 90 },
    { 0x1c67e3ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4317e3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147e3b5, 76 },
    { 0, 0 },
    { 0x8ff7e3b7, 61 },
    { 0x36d7e3b8, 98 },
    { 0, 0 },
    { 0x4fb7e3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc7e3bd, 61 },
    { 0xaff7e3be, 91 },
    { 0x1f67e3bf, 90 },
    { 0xc647e3be, 99 },
    { 0xa217e3be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f7e3c4, 55 },
    { 0, 0 },
    { 0xe097e3c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd67e3c9, 90 },
    { 0x81c7e3c9, 23 },
    { 0xe627e3cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8597e3d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057e3db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9217e3df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f07e3e3, 52 },
    { 0xea97e3e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd257e3ea, 33 },
    { 0, 0 },
    { 0x7ff7e3ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5407e3ef, 33 },
    { 0, 0 },
    { 0xa467e3f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7e3f6, 88 },
    { 0xa937e3f7, 77 },
    { 0x4417e3f8, 98 },
    { 0x7277e3f7, 23 },
    { 0, 0 },
    { 0x827e3fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf537e3ff, 33 },
    { 0xe4f7e400, 33 },
    { 0, 0 },
    { 0x7337e402, 68 },
    { 0, 0 },
    { 0x4057e404, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567e409, 90 },
    { 0x9b67e40a, 95 },
    { 0, 0 },
    { 0x2e17e40c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e7e40f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57e412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27e415, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b7e420, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d7e423, 69 },
    { 0x86c7e423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7e428, 33 },
    { 0x3b37e428, 91 },
    { 0, 0 },
    { 0x9817e42b, 66 },
    { 0x5ba7e42b, 93 },
    { 0x5467e42d, 61 },
    { 0xee47e42d, 95 },
    { 0xb9a7e42f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27e435, 95 },
    { 0x34e7e436, 95 },
    { 0x8c7e437, 100 },
    { 0xb127e438, 55 },
    { 0xbfd7e436, 46 },
    { 0x3447e43a, 23 },
    { 0, 0 },
    { 0xd237e43c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e43f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd507e443, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e7e448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4697e450, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27e453, 98 },
    { 0, 0 },
    { 0xaf77e455, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7e464, 90 },
    { 0xb3e7e464, 43 },
    { 0xa667e465, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c97e472, 33 },
    { 0, 0 },
    { 0x85d7e474, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5847e477, 95 },
    { 0, 0 },
    { 0x8187e479, 88 },
    { 0x567e479, 95 },
    { 0, 0 },
    { 0x29f7e47c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc827e47f, 43 },
    { 0, 0 },
    { 0x68b7e481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7027e484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47e487, 23 },
    { 0x9587e487, 33 },
    { 0, 0 },
    { 0x4947e48a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987e496, 98 },
    { 0xa957e496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab7e49e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e4ae, 98 },
    { 0, 0 },
    { 0xd817e4b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe837e4b4, 46 },
    { 0x6be7e4b5, 33 },
    { 0xaa87e4b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4467e4b9, 90 },
    { 0x75a7e4b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c7e4be, 90 },
    { 0, 0 },
    { 0x7087e4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7e4c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3177e4c7, 99 },
    { 0x2367e4c8, 61 },
    { 0xac47e4c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b7e4cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea7e4d0, 64 },
    { 0, 0 },
    { 0x2027e4d2, 90 },
    { 0xd837e4d2, 23 },
    { 0xfae7e4d4, 34 },
    { 0x3947e4d5, 56 },
    { 0x7fe7e4d6, 76 },
    { 0x837e4d5, 90 },
    { 0, 0 },
    { 0x9207e4d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa927e4df, 67 },
    { 0, 0 },
    { 0xd5a7e4e1, 61 },
    { 0, 0 },
    { 0x5617e4e3, 90 },
    { 0, 0 },
    { 0xc4c7e4e5, 33 },
    { 0x9ab7e4e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7e4e9, 61 },
    { 0, 0 },
    { 0x44f7e4eb, 66 },
    { 0xb1f7e4ec, 61 },
    { 0, 0 },
    { 0x24b7e4ee, 46 },
    { 0x5187e4ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7e4f3, 61 },
    { 0x2f07e4f3, 95 },
    { 0x8c07e4f3, 77 },
    { 0x6647e4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77e4fa, 96 },
    { 0x7a37e4fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc977e502, 33 },
    { 0x3b67e503, 91 },
    { 0, 0 },
    { 0x2bf7e505, 43 },
    { 0xb327e506, 61 },
    { 0x9d57e507, 98 },
    { 0xdec7e508, 98 },
    { 0xd8f7e507, 76 },
    { 0xf907e50a, 51 },
    { 0x2127e50b, 66 },
    { 0xc387e508, 98 },
    { 0x9a57e50d, 66 },
    { 0x9407e50c, 61 },
    { 0, 0 },
    { 0x6937e510, 33 },
    { 0x8847e511, 95 },
    { 0xe957e512, 95 },
    { 0, 0 },
    { 0xaf7e514, 61 },
    { 0x59d7e515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497e518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3137e51b, 95 },
    { 0xee07e51b, 34 },
    { 0, 0 },
    { 0x4e7e51e, 23 },
    { 0x6717e51e, 90 },
    { 0x6727e51e, 91 },
    { 0xee57e51f, 55 },
    { 0xb207e522, 94 },
    { 0xa2b7e521, 34 },
    { 0x8467e51f, 55 },
    { 0xc857e525, 90 },
    { 0, 0 },
    { 0xda77e527, 61 },
    { 0x5e97e527, 38 },
    { 0x15c7e528, 75 },
    { 0x83b7e52a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe97e531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce17e536, 61 },
    { 0xcbd7e537, 99 },
    { 0, 0 },
    { 0x9387e539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d7e53f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e7e542, 33 },
    { 0x8207e543, 90 },
    { 0xedb7e544, 61 },
    { 0x79e7e544, 90 },
    { 0xab97e546, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7e54c, 43 },
    { 0, 0 },
    { 0x96b7e54e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e07e556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8667e559, 87 },
    { 0, 0 },
    { 0x86c7e55b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d7e561, 33 },
    { 0xc4c7e562, 33 },
    { 0, 0 },
    { 0x6127e564, 61 },
    { 0x8267e565, 43 },
    { 0xf3d7e566, 90 },
    { 0xeca7e566, 76 },
    { 0x70d7e566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b57e56b, 90 },
    { 0x4ce7e56c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce7e570, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3997e573, 90 },
    { 0xb2a7e574, 61 },
    { 0xc9f7e574, 23 },
    { 0x82b7e576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f7e57f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8617e586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd57e589, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e58e, 77 },
    { 0xf507e58f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd107e594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b87e598, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77e59d, 84 },
    { 0xa4d7e59e, 68 },
    { 0xfcd7e59f, 55 },
    { 0, 0 },
    { 0x5537e5a1, 90 },
    { 0x8fb7e5a2, 95 },
    { 0xef17e5a3, 99 },
    { 0, 0 },
    { 0x927e5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc47e5a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe867e5b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e7e5b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba7e5bd, 66 },
    { 0, 0 },
    { 0x5747e5bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9277e5c5, 37 },
    { 0, 0 },
    { 0x9f87e5c7, 57 },
    { 0xced7e5c7, 23 },
    { 0xa2f7e5c7, 98 },
    { 0xf927e5ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c97e5d0, 88 },
    { 0xebf7e5d1, 95 },
    { 0, 0 },
    { 0x8be7e5d3, 95 },
    { 0xfa17e5d4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487e5d7, 90 },
    { 0x1007e5d8, 84 },
    { 0xbbd7e5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e5dc, 33 },
    { 0, 0 },
    { 0x917e5de, 61 },
    { 0xf877e5de, 84 },
    { 0x417e5e0, 77 },
    { 0, 0 },
    { 0xfcc7e5e2, 99 },
    { 0x6f67e5e3, 90 },
    { 0, 0 },
    { 0x36c7e5e5, 33 },
    { 0x2597e5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8707e5ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7e5ed, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc767e5f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8577e5f5, 90 },
    { 0x8847e5f6, 61 },
    { 0x9197e5f5, 55 },
    { 0xfce7e5f8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc7e600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e7e604, 61 },
    { 0xd197e605, 87 },
    { 0x5897e606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf447e60b, 23 },
    { 0x8b37e60c, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7e613, 34 },
    { 0x9ac7e614, 61 },
    { 0x8b17e615, 90 },
    { 0xdb77e614, 98 },
    { 0, 0 },
    { 0xc657e618, 90 },
    { 0, 0 },
    { 0xb777e61a, 23 },
    { 0x6a07e61b, 96 },
    { 0, 0 },
    { 0xa4b7e61d, 85 },
    { 0xce57e61e, 61 },
    { 0x97e61f, 33 },
    { 0x15a7e61d, 66 },
    { 0, 0 },
    { 0x75c7e622, 90 },
    { 0x997e623, 33 },
    { 0x1da7e624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647e629, 90 },
    { 0xf827e62a, 95 },
    { 0xb0b7e62b, 43 },
    { 0x5cc7e629, 87 },
    { 0x95b7e62a, 23 },
    { 0x6597e62e, 90 },
    { 0x1a37e62f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707e633, 75 },
    { 0xc827e634, 87 },
    { 0xbb37e635, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7e639, 61 },
    { 0xfe27e63a, 52 },
    { 0, 0 },
    { 0x10d7e63c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7e642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07e646, 84 },
    { 0, 0 },
    { 0x8c47e648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87e64b, 98 },
    { 0xbd87e64c, 66 },
    { 0xa897e64d, 95 },
    { 0x5cf7e64e, 61 },
    { 0x71c7e64d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a7e652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1487e656, 23 },
    { 0, 0 },
    { 0xa027e658, 55 },
    { 0x6f87e659, 88 },
    { 0x1e57e65a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef7e65d, 23 },
    { 0x6bc7e65d, 61 },
    { 0x1da7e65f, 90 },
    { 0, 0 },
    { 0xd9e7e661, 66 },
    { 0, 0 },
    { 0xb1c7e663, 34 },
    { 0x7127e663, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757e667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e47e66d, 90 },
    { 0, 0 },
    { 0xe4a7e66f, 78 },
    { 0xd47e66f, 90 },
    { 0xec77e670, 95 },
    { 0x9947e66f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7e679, 90 },
    { 0xa417e679, 101 },
    { 0xa8d7e67b, 61 },
    { 0x5b27e679, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd17e67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e682, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4847e685, 95 },
    { 0x3a97e686, 93 },
    { 0, 0 },
    { 0xe837e688, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad7e68b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f57e68f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b27e693, 61 },
    { 0x4417e694, 67 },
    { 0x3477e695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f7e699, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1757e69c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d7e69f, 33 },
    { 0, 0 },
    { 0xd37e6a1, 23 },
    { 0xc657e6a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc7e6a5, 61 },
    { 0, 0 },
    { 0x5697e6a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e6b4, 66 },
    { 0, 0 },
    { 0xe837e6b6, 90 },
    { 0x8777e6b7, 55 },
    { 0x3447e6b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae67e6bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7e6c1, 95 },
    { 0x1ae7e6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8107e6c5, 67 },
    { 0xf857e6c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca7e6ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e17e6d2, 33 },
    { 0x727e6d3, 90 },
    { 0xb3b7e6d2, 33 },
    { 0xbf67e6d4, 90 },
    { 0x4167e6d2, 43 },
    { 0x3f07e6d6, 61 },
    { 0xb47e6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa07e6dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa027e6e2, 97 },
    { 0xcdf7e6e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb7e6ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7e6f1, 61 },
    { 0, 0 },
    { 0x3d97e6f3, 100 },
    { 0xc847e6f4, 90 },
    { 0x6ab7e6f5, 76 },
    { 0x5b07e6f6, 46 },
    { 0, 0 },
    { 0x2937e6f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa7e6fb, 61 },
    { 0, 0 },
    { 0x4477e6fd, 98 },
    { 0x1e7e6fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7e70b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b57e712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f7e718, 88 },
    { 0x33f7e719, 90 },
    { 0xb127e71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf77e722, 33 },
    { 0x1297e722, 33 },
    { 0xa587e722, 93 },
    { 0xfe67e725, 75 },
    { 0, 0 },
    { 0xf9d7e727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b7e72b, 55 },
    { 0, 0 },
    { 0xeb97e72d, 95 },
    { 0x89e7e72e, 90 },
    { 0xae47e72f, 61 },
    { 0, 0 },
    { 0x4be7e731, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47e737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb7e73a, 95 },
    { 0xcc87e73b, 46 },
    { 0, 0 },
    { 0xd577e73d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e741, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567e74d, 69 },
    { 0xa927e74e, 85 },
    { 0x6627e74d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e77e754, 85 },
    { 0xcda7e755, 76 },
    { 0x95a7e756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7e75d, 61 },
    { 0x7c57e75e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5537e762, 76 },
    { 0x5037e763, 91 },
    { 0, 0 },
    { 0xcb77e765, 75 },
    { 0xa5d7e766, 98 },
    { 0, 0 },
    { 0xc4c7e768, 61 },
    { 0xc4f7e768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d87e770, 46 },
    { 0, 0 },
    { 0x3d67e772, 90 },
    { 0xc377e773, 90 },
    { 0x72c7e774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a7e77b, 43 },
    { 0, 0 },
    { 0x1427e77d, 56 },
    { 0, 0 },
    { 0x68e7e77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1937e782, 23 },
    { 0x4077e782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7697e78b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb277e790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa97e79f, 61 },
    { 0, 0 },
    { 0x17a7e7a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7197e7a7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe587e7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3567e7af, 23 },
    { 0, 0 },
    { 0x8147e7b1, 90 },
    { 0, 0 },
    { 0x13b7e7b3, 83 },
    { 0x21e7e7b4, 43 },
    { 0xcdc7e7b5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a7e7bd, 33 },
    { 0xa9c7e7be, 78 },
    { 0x8ba7e7be, 98 },
    { 0x68b7e7c0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c7e7c4, 88 },
    { 0x7297e7c5, 92 },
    { 0, 0 },
    { 0x8f87e7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f47e7cd, 100 },
    { 0, 0 },
    { 0x47b7e7cf, 46 },
    { 0, 0 },
    { 0x1767e7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e77e7da, 90 },
    { 0xa7f7e7da, 61 },
    { 0x5b07e7dc, 85 },
    { 0xbb37e7dc, 95 },
    { 0x5df7e7de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba7e7e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c7e7e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6367e7ec, 90 },
    { 0x8fe7e7ec, 55 },
    { 0x2d27e7ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a77e7f1, 90 },
    { 0, 0 },
    { 0x7647e7f3, 52 },
    { 0, 0 },
    { 0x2f37e7f5, 52 },
    { 0x62d7e7f5, 75 },
    { 0xc717e7f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb027e7fa, 66 },
    { 0x78f7e7fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff7e7ff, 78 },
    { 0, 0 },
    { 0x6557e801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b7e806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27e80e, 69 },
    { 0x8fd7e80f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab7e813, 33 },
    { 0, 0 },
    { 0x2e97e815, 52 },
    { 0xa4b7e816, 85 },
    { 0, 0 },
    { 0xa227e818, 95 },
    { 0xc1d7e819, 85 },
    { 0x6467e81a, 90 },
    { 0x8897e81b, 95 },
    { 0x75a7e81a, 67 },
    { 0x667e81d, 66 },
    { 0xd127e81d, 33 },
    { 0xb467e81f, 85 },
    { 0xb77e81f, 55 },
    { 0xc477e81f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c7e828, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c7e82c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e7e82f, 38 },
    { 0xf407e82f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7e835, 61 },
    { 0, 0 },
    { 0x1067e837, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e83c, 87 },
    { 0xe757e83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee07e840, 67 },
    { 0, 0 },
    { 0xe07e842, 34 },
    { 0, 0 },
    { 0xd467e844, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c7e847, 34 },
    { 0x23e7e848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b17e84e, 80 },
    { 0, 0 },
    { 0x63e7e850, 77 },
    { 0x37e7e851, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6517e854, 61 },
    { 0xd427e855, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da7e85e, 84 },
    { 0x93b7e85f, 84 },
    { 0x7137e85f, 85 },
    { 0, 0 },
    { 0x7fe7e862, 33 },
    { 0xac27e863, 69 },
    { 0, 0 },
    { 0x8027e865, 83 },
    { 0xe0b7e865, 33 },
    { 0x8647e867, 61 },
    { 0x2aa7e868, 34 },
    { 0xaa07e869, 98 },
    { 0xbaa7e866, 67 },
    { 0, 0 },
    { 0xe6e7e86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7e872, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e875, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad7e878, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a87e87b, 90 },
    { 0x9987e87c, 76 },
    { 0x8de7e87b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd867e880, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167e887, 33 },
    { 0x8f77e888, 80 },
    { 0, 0 },
    { 0xd5a7e88a, 33 },
    { 0, 0 },
    { 0x8b7e88c, 90 },
    { 0x48b7e88d, 90 },
    { 0x8e97e88e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747e899, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df7e89e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f57e8a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157e8af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa887e8b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e57e8b7, 66 },
    { 0x7ea7e8b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef7e8c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5247e8c7, 90 },
    { 0, 0 },
    { 0xc557e8c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa207e8cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d67e8d0, 90 },
    { 0x1d7e8d1, 87 },
    { 0xea77e8d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17e8da, 76 },
    { 0xff77e8da, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc377e8de, 46 },
    { 0xf677e8df, 61 },
    { 0x1a07e8e0, 33 },
    { 0xc977e8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57e8e8, 55 },
    { 0x5897e8e9, 78 },
    { 0, 0 },
    { 0xf3e7e8eb, 33 },
    { 0, 0 },
    { 0x60f7e8ed, 93 },
    { 0x9777e8ee, 76 },
    { 0, 0 },
    { 0x44a7e8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb257e8fb, 23 },
    { 0, 0 },
    { 0x69d7e8fd, 84 },
    { 0x3dc7e8fd, 90 },
    { 0, 0 },
    { 0xe427e900, 23 },
    { 0xc207e901, 90 },
    { 0xfda7e902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef7e906, 98 },
    { 0x2367e906, 99 },
    { 0x7b87e907, 33 },
    { 0xf8e7e909, 93 },
    { 0xa677e909, 46 },
    { 0xe197e907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597e90e, 67 },
    { 0x21f7e90f, 93 },
    { 0x1307e90e, 65 },
    { 0, 0 },
    { 0xf187e912, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b7e917, 93 },
    { 0xaee7e918, 91 },
    { 0x59f7e919, 61 },
    { 0xf4c7e91a, 55 },
    { 0xafe7e91a, 24 },
    { 0x5237e917, 33 },
    { 0x38f7e91d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a57e925, 98 },
    { 0, 0 },
    { 0xd587e927, 75 },
    { 0, 0 },
    { 0x98b7e929, 93 },
    { 0x5647e92a, 34 },
    { 0, 0 },
    { 0xd797e92c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a7e930, 90 },
    { 0x2ac7e930, 61 },
    { 0, 0 },
    { 0x8e87e933, 46 },
    { 0x5887e934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17e939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7e93d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b47e946, 84 },
    { 0, 0 },
    { 0xd6e7e948, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b37e94b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8507e94e, 88 },
    { 0xb047e94f, 95 },
    { 0xd487e950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c57e953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a7e957, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc57e960, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e963, 77 },
    { 0, 0 },
    { 0xff97e965, 90 },
    { 0, 0 },
    { 0x7dc7e967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87e971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed7e976, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e87e97a, 34 },
    { 0, 0 },
    { 0x8a97e97c, 95 },
    { 0x4737e97d, 34 },
    { 0x6537e97e, 91 },
    { 0x9877e97f, 90 },
    { 0x6987e97c, 33 },
    { 0x6147e981, 99 },
    { 0x14e7e982, 76 },
    { 0, 0 },
    { 0xae97e984, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17e989, 69 },
    { 0, 0 },
    { 0x9907e98b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187e990, 90 },
    { 0x1fb7e991, 23 },
    { 0xcf27e990, 78 },
    { 0x3197e992, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7e99f, 46 },
    { 0, 0 },
    { 0x2617e9a1, 91 },
    { 0, 0 },
    { 0x2367e9a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557e9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007e9ad, 2 },
    { 0x5287e9ae, 61 },
    { 0, 0 },
    { 0xd9d7e9b0, 52 },
    { 0xf5a7e9b1, 90 },
    { 0xbea7e9b2, 61 },
    { 0x177e9b3, 98 },
    { 0x70d7e9b4, 76 },
    { 0x42b7e9b0, 61 },
    { 0x2f97e9b6, 17 },
    { 0xee17e9b7, 78 },
    { 0x3bc7e9b7, 90 },
    { 0, 0 },
    { 0x1097e9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e9be, 87 },
    { 0x53c7e9be, 43 },
    { 0x6ad7e9be, 61 },
    { 0x4a97e9c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb387e9c5, 33 },
    { 0x6927e9c6, 90 },
    { 0xd1d7e9c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387e9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8827e9d2, 61 },
    { 0x11f7e9d3, 90 },
    { 0, 0 },
    { 0x8c7e9d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7e9da, 43 },
    { 0xe267e9db, 67 },
    { 0x5877e9db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797e9ea, 90 },
    { 0x7437e9ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a7e9f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7947e9fc, 33 },
    { 0, 0 },
    { 0x8a77e9fe, 90 },
    { 0x1f77e9fe, 98 },
    { 0x85d7ea00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa727ea03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d7ea06, 78 },
    { 0xf937ea07, 78 },
    { 0x6d7ea08, 91 },
    { 0x94d7ea08, 76 },
    { 0xaf27ea0a, 55 },
    { 0x9927ea09, 34 },
    { 0xac47ea0c, 98 },
    { 0xfd57ea0b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c7ea12, 61 },
    { 0xef37ea13, 23 },
    { 0x5387ea13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc7ea17, 90 },
    { 0x8187ea18, 76 },
    { 0x6357ea17, 76 },
    { 0x9f27ea1a, 95 },
    { 0x52f7ea1a, 66 },
    { 0x4587ea1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7ea20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a97ea23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3577ea27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4367ea31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287ea35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187ea39, 90 },
    { 0x1d77ea39, 33 },
    { 0, 0 },
    { 0x5867ea3c, 33 },
    { 0x34a7ea3d, 33 },
    { 0x9f7ea3e, 43 },
    { 0x8b27ea3f, 33 },
    { 0xcff7ea3f, 90 },
    { 0x3d67ea41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b7ea4a, 61 },
    { 0, 0 },
    { 0x8d97ea4c, 23 },
    { 0xa967ea4c, 95 },
    { 0xd677ea4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a7ea56, 61 },
    { 0, 0 },
    { 0xe027ea58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f7ea5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb7ea61, 46 },
    { 0x75d7ea61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc147ea69, 43 },
    { 0, 0 },
    { 0x2e07ea6b, 95 },
    { 0, 0 },
    { 0x7867ea6d, 65 },
    { 0, 0 },
    { 0xa057ea6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37ea75, 55 },
    { 0, 0 },
    { 0x5b37ea77, 90 },
    { 0x3bf7ea78, 80 },
    { 0x4377ea78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d97ea7c, 33 },
    { 0xe897ea7d, 90 },
    { 0xd1b7ea7e, 76 },
    { 0x41f7ea7e, 90 },
    { 0x7577ea7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8497ea89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b7ea8c, 76 },
    { 0xf0f7ea8d, 68 },
    { 0x4bd7ea8d, 61 },
    { 0x3117ea8e, 33 },
    { 0xb327ea90, 23 },
    { 0x3907ea91, 81 },
    { 0x5c97ea92, 90 },
    { 0x65b7ea91, 90 },
    { 0x2637ea94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf847ea9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe757ea9f, 64 },
    { 0, 0 },
    { 0xe0e7eaa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa27eaa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf147eaa8, 43 },
    { 0, 0 },
    { 0x29d7eaaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9917eab1, 95 },
    { 0, 0 },
    { 0xbb47eab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb27eab7, 93 },
    { 0x3127eab7, 95 },
    { 0, 0 },
    { 0x19d7eaba, 95 },
    { 0x3f87eabb, 99 },
    { 0x92b7eabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27eabf, 90 },
    { 0x5927eac0, 34 },
    { 0xd407eac1, 33 },
    { 0xb3d7eac1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e7eac5, 33 },
    { 0, 0 },
    { 0x3337eac7, 76 },
    { 0xb5f7eac8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac7eacc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647ead1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7ead4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327ead9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e7eadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f7eae2, 90 },
    { 0, 0 },
    { 0xbd47eae4, 61 },
    { 0x5197eae5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b87eaea, 33 },
    { 0x5937eaeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b7eaf3, 76 },
    { 0xbd37eaf3, 76 },
    { 0, 0 },
    { 0x9357eaf6, 55 },
    { 0, 0 },
    { 0x4757eaf8, 84 },
    { 0x3f67eaf9, 55 },
    { 0, 0 },
    { 0xad87eafb, 68 },
    { 0x1097eafc, 61 },
    { 0xa3a7eafb, 93 },
    { 0xd027eafc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f27eb02, 23 },
    { 0, 0 },
    { 0xc867eb04, 52 },
    { 0xd257eb04, 95 },
    { 0x7c57eb06, 43 },
    { 0, 0 },
    { 0x61b7eb08, 51 },
    { 0xe7d7eb08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7eb0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9177eb10, 33 },
    { 0, 0 },
    { 0x1407eb12, 43 },
    { 0xf517eb13, 61 },
    { 0x68d7eb14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab17eb18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b87eb1f, 90 },
    { 0xaa7eb20, 98 },
    { 0xbf67eb21, 100 },
    { 0, 0 },
    { 0x757eb23, 76 },
    { 0x8cc7eb24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4497eb28, 98 },
    { 0, 0 },
    { 0x1ce7eb2a, 34 },
    { 0xeb17eb2b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a47eb2f, 85 },
    { 0x5bc7eb30, 90 },
    { 0, 0 },
    { 0xf9c7eb32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747eb35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7eb3b, 61 },
    { 0xe217eb3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947eb40, 23 },
    { 0x7e67eb41, 52 },
    { 0xd687eb40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa77eb46, 43 },
    { 0, 0 },
    { 0x98c7eb48, 51 },
    { 0x7b77eb48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b67eb53, 33 },
    { 0xf347eb54, 90 },
    { 0, 0 },
    { 0xac27eb56, 17 },
    { 0xcaa7eb56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7eb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7eb62, 85 },
    { 0, 0 },
    { 0x7c57eb64, 55 },
    { 0x9367eb65, 33 },
    { 0x3257eb64, 80 },
    { 0xdca7eb67, 34 },
    { 0x1827eb67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b07eb6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187eb6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657eb76, 90 },
    { 0x79c7eb77, 91 },
    { 0xc7eb77, 98 },
    { 0x9e77eb79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b7eb80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c07eb84, 99 },
    { 0x3807eb84, 98 },
    { 0, 0 },
    { 0xd37eb87, 61 },
    { 0, 0 },
    { 0x4ac7eb89, 61 },
    { 0x8217eb89, 98 },
    { 0xdd47eb89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4027eb90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed7eb96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc677eb9a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177eba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607eba5, 33 },
    { 0x4de7eba5, 33 },
    { 0xf707eba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3767ebab, 66 },
    { 0xb627ebac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ebb2, 90 },
    { 0x2bb7ebb2, 95 },
    { 0xf9f7ebb4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff7ebbb, 98 },
    { 0xbcb7ebbc, 85 },
    { 0x8887ebbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde97ebc1, 87 },
    { 0, 0 },
    { 0xebe7ebc3, 95 },
    { 0x9017ebc3, 95 },
    { 0xec47ebc5, 95 },
    { 0xb977ebc5, 93 },
    { 0xfb37ebc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27ebcb, 88 },
    { 0xfc47ebcc, 23 },
    { 0, 0 },
    { 0xb827ebce, 34 },
    { 0x34a7ebcf, 61 },
    { 0x8227ebcf, 23 },
    { 0, 0 },
    { 0x657ebd2, 61 },
    { 0x1f67ebd3, 95 },
    { 0xeef7ebd4, 61 },
    { 0xc517ebd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ebd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7ebdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e67ebdf, 76 },
    { 0xfb47ebe0, 23 },
    { 0x3c7ebe0, 90 },
    { 0x5ff7ebe0, 55 },
    { 0xefb7ebe3, 33 },
    { 0x5ea7ebe3, 100 },
    { 0xb987ebe5, 66 },
    { 0x8817ebe3, 61 },
    { 0xca47ebe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c7ebea, 55 },
    { 0, 0 },
    { 0xd867ebec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd7ebf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627ebf4, 77 },
    { 0xd467ebf5, 98 },
    { 0x3747ebf6, 88 },
    { 0x3a97ebf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc7ebfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d7ebfd, 78 },
    { 0x5077ebfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae7ec09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa097ec0c, 85 },
    { 0xcac7ec0c, 34 },
    { 0x3577ec0e, 90 },
    { 0x4127ec0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7ec14, 95 },
    { 0x5277ec14, 95 },
    { 0x9897ec16, 90 },
    { 0, 0 },
    { 0x4b57ec18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe617ec1d, 43 },
    { 0xe207ec1e, 61 },
    { 0xc337ec1f, 33 },
    { 0x9c87ec20, 90 },
    { 0, 0 },
    { 0x9507ec22, 2 },
    { 0x65a7ec23, 61 },
    { 0, 0 },
    { 0xdc37ec25, 61 },
    { 0, 0 },
    { 0x3967ec27, 34 },
    { 0, 0 },
    { 0x7387ec29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7ec2e, 87 },
    { 0xf6e7ec2e, 43 },
    { 0x1027ec30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc47ec33, 66 },
    { 0, 0 },
    { 0x3b07ec35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7ec38, 93 },
    { 0x9d07ec38, 34 },
    { 0x8b27ec3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207ec40, 90 },
    { 0xb07ec41, 55 },
    { 0xe4f7ec42, 90 },
    { 0xe587ec43, 61 },
    { 0, 0 },
    { 0x5b67ec45, 23 },
    { 0x14b7ec46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227ec49, 93 },
    { 0x76f7ec4a, 90 },
    { 0xa207ec4a, 43 },
    { 0x4197ec4a, 90 },
    { 0x7f07ec4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee87ec53, 61 },
    { 0x9b7ec53, 88 },
    { 0xdee7ec55, 93 },
    { 0, 0 },
    { 0x7d27ec57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37ec5a, 98 },
    { 0x3a77ec5b, 61 },
    { 0x8707ec5b, 33 },
    { 0x12e7ec5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc297ec62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257ec68, 33 },
    { 0x587ec68, 46 },
    { 0x4f37ec6a, 90 },
    { 0, 0 },
    { 0xad17ec6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d7ec6f, 52 },
    { 0xc127ec6f, 43 },
    { 0x7a37ec71, 90 },
    { 0, 0 },
    { 0xa977ec73, 55 },
    { 0xb8c7ec74, 78 },
    { 0xe357ec75, 55 },
    { 0x2437ec76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197ec7a, 40 },
    { 0x1427ec7a, 34 },
    { 0x5737ec7c, 33 },
    { 0, 0 },
    { 0x7d07ec7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d7ec83, 25 },
    { 0, 0 },
    { 0xa897ec85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b7ec8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d7ec8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c57ec94, 98 },
    { 0, 0 },
    { 0xb717ec96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647ec9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb7eca0, 76 },
    { 0, 0 },
    { 0xcbd7eca2, 94 },
    { 0x5127eca2, 98 },
    { 0xf597eca4, 78 },
    { 0xcc17eca5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597ecb2, 34 },
    { 0xbd67ecb3, 46 },
    { 0, 0 },
    { 0xac07ecb5, 48 },
    { 0x4477ecb5, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7037ecbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd97ecbf, 55 },
    { 0xc9c7ecc0, 93 },
    { 0, 0 },
    { 0xc377ecc2, 33 },
    { 0xe797ecc2, 34 },
    { 0x6b7ecc4, 52 },
    { 0xbc97ecc4, 90 },
    { 0, 0 },
    { 0xa4c7ecc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c97ecca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf7eccd, 40 },
    { 0xdb17ecce, 76 },
    { 0x9bf7eccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e7ecd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7ecdd, 61 },
    { 0xaec7ecde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767ece6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7eceb, 90 },
    { 0, 0 },
    { 0xd107eced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7ecf2, 46 },
    { 0xeb87ecf2, 46 },
    { 0xea77ecf4, 84 },
    { 0x8c67ecf5, 88 },
    { 0x3017ecf5, 84 },
    { 0x2fa7ecf7, 90 },
    { 0xd7f7ecf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7ecfc, 23 },
    { 0, 0 },
    { 0xc797ecfe, 90 },
    { 0, 0 },
    { 0x4a57ed00, 95 },
    { 0, 0 },
    { 0x3667ed02, 33 },
    { 0x97d7ed02, 84 },
    { 0x8d77ed04, 93 },
    { 0x6517ed04, 55 },
    { 0x8297ed05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537ed0a, 90 },
    { 0xd17ed0a, 98 },
    { 0, 0 },
    { 0xd227ed0d, 33 },
    { 0, 0 },
    { 0xbda7ed0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7ed14, 88 },
    { 0, 0 },
    { 0x29e7ed16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf567ed1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d7ed23, 61 },
    { 0x3d37ed24, 95 },
    { 0x77a7ed23, 98 },
    { 0xf547ed26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2167ed2d, 64 },
    { 0xb4c7ed2e, 46 },
    { 0x4987ed2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8927ed34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427ed3a, 61 },
    { 0xfd27ed3a, 90 },
    { 0xead7ed3a, 23 },
    { 0x3537ed3d, 85 },
    { 0xfde7ed3c, 90 },
    { 0xede7ed3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d7ed4a, 90 },
    { 0x5bd7ed4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7ed51, 90 },
    { 0xee47ed51, 55 },
    { 0xbef7ed53, 67 },
    { 0xac57ed54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc527ed5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27ed60, 61 },
    { 0, 0 },
    { 0x9b37ed62, 95 },
    { 0xaf07ed63, 95 },
    { 0x30e7ed64, 23 },
    { 0x8757ed64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b7ed6d, 99 },
    { 0, 0 },
    { 0x2e87ed6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47ed74, 90 },
    { 0, 0 },
    { 0x7077ed76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b77ed80, 76 },
    { 0x3b7ed80, 84 },
    { 0xe457ed80, 99 },
    { 0, 0 },
    { 0x9607ed84, 23 },
    { 0xfff7ed84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b7ed8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff37ed8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7ed96, 17 },
    { 0x5897ed97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf17ed9d, 23 },
    { 0, 0 },
    { 0x3b17ed9f, 61 },
    { 0xcc67eda0, 90 },
    { 0x3ee7eda0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea07eda7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae7edae, 76 },
    { 0xb8a7edaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827edb5, 90 },
    { 0, 0 },
    { 0x4b97edb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a7edc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc7edc9, 33 },
    { 0xa887edca, 43 },
    { 0x5897edcb, 95 },
    { 0x5307edca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c7edd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc327edd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007eddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7eddf, 52 },
    { 0x87c7ede0, 90 },
    { 0xb8e7ede1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327ede6, 93 },
    { 0x357ede6, 46 },
    { 0x6397ede8, 33 },
    { 0x2807ede6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597edec, 98 },
    { 0, 0 },
    { 0x4817edee, 99 },
    { 0xe617edef, 61 },
    { 0xbaf7edf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67edf3, 33 },
    { 0xcf87edf4, 33 },
    { 0, 0 },
    { 0x5d27edf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a7edf9, 84 },
    { 0x7ea7edfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b7edfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a7ee02, 33 },
    { 0, 0 },
    { 0xc807ee04, 52 },
    { 0x55d7ee05, 64 },
    { 0xc917ee06, 76 },
    { 0, 0 },
    { 0xb07ee08, 90 },
    { 0x89d7ee09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f77ee0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557ee0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c7ee14, 95 },
    { 0xee57ee14, 61 },
    { 0xc7d7ee14, 90 },
    { 0x1087ee15, 61 },
    { 0x71b7ee16, 90 },
    { 0xed97ee19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7ee1c, 61 },
    { 0xeea7ee1d, 100 },
    { 0x6c07ee1d, 100 },
    { 0x5407ee1d, 34 },
    { 0x4577ee20, 95 },
    { 0, 0 },
    { 0xf027ee22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e27ee27, 10 },
    { 0x6a07ee27, 100 },
    { 0, 0 },
    { 0xc7f7ee2a, 78 },
    { 0, 0 },
    { 0x577ee2c, 34 },
    { 0x1b77ee2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6757ee36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b27ee3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7ee45, 66 },
    { 0x64b7ee45, 94 },
    { 0, 0 },
    { 0x8177ee48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe017ee4c, 66 },
    { 0x4ff7ee4d, 61 },
    { 0x1eb7ee4d, 78 },
    { 0x73e7ee4e, 93 },
    { 0x3ca7ee4e, 33 },
    { 0x5ee7ee4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e7ee57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37ee5c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde07ee61, 68 },
    { 0xc7c7ee62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e57ee66, 93 },
    { 0x46f7ee67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe27ee6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc7ee6d, 33 },
    { 0xae87ee6e, 33 },
    { 0x67d7ee6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc737ee72, 34 },
    { 0xdbc7ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc7ee76, 88 },
    { 0xa327ee76, 90 },
    { 0, 0 },
    { 0x1f17ee79, 99 },
    { 0, 0 },
    { 0x43c7ee7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d7ee80, 53 },
    { 0x9067ee80, 17 },
    { 0, 0 },
    { 0x5897ee83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb157ee87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a17ee8b, 68 },
    { 0, 0 },
    { 0x8a77ee8d, 61 },
    { 0xa2c7ee8e, 90 },
    { 0x8037ee8f, 98 },
    { 0x9317ee8f, 43 },
    { 0, 0 },
    { 0x9ad7ee92, 98 },
    { 0xbec7ee93, 95 },
    { 0x5307ee94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7ee97, 48 },
    { 0x5867ee97, 43 },
    { 0, 0 },
    { 0x337ee9a, 90 },
    { 0, 0 },
    { 0x7347ee9c, 93 },
    { 0xf447ee9c, 98 },
    { 0x3b37ee9e, 98 },
    { 0x4167ee9f, 43 },
    { 0, 0 },
    { 0xdc17eea1, 61 },
    { 0xa507eea1, 77 },
    { 0x3767eea2, 95 },
    { 0x4357eea4, 23 },
    { 0xc467eea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f7eea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7eeb0, 95 },
    { 0x86f7eeb1, 33 },
    { 0xce67eeb2, 66 },
    { 0xbf67eeb3, 98 },
    { 0x6a67eeb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7287eeb7, 56 },
    { 0xa1b7eeb8, 78 },
    { 0, 0 },
    { 0x9c27eeba, 77 },
    { 0x8087eebb, 61 },
    { 0, 0 },
    { 0x2c67eebd, 98 },
    { 0x32f7eebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe57eec3, 46 },
    { 0x8fa7eec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc27eec7, 23 },
    { 0x49c7eec8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3837eecf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e67eed6, 34 },
    { 0x8687eed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4087eede, 90 },
    { 0, 0 },
    { 0x4647eee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc307eee8, 61 },
    { 0x4367eee8, 23 },
    { 0x12f7eeea, 4 },
    { 0xe097eeeb, 23 },
    { 0xa687eee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887eef0, 61 },
    { 0, 0 },
    { 0xe1d7eef2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee7eef5, 98 },
    { 0x63a7eef6, 52 },
    { 0xb697eef7, 90 },
    { 0x6bb7eef8, 90 },
    { 0x5c27eef9, 95 },
    { 0x7137eef6, 61 },
    { 0, 0 },
    { 0xa267eefc, 95 },
    { 0, 0 },
    { 0x9787eefe, 46 },
    { 0, 0 },
    { 0xfcc7ef00, 93 },
    { 0, 0 },
    { 0x8ac7ef02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd07ef06, 95 },
    { 0xd3f7ef06, 61 },
    { 0xd9c7ef08, 90 },
    { 0x9b87ef07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217ef0c, 33 },
    { 0, 0 },
    { 0xba67ef0e, 78 },
    { 0, 0 },
    { 0x8127ef10, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc7ef21, 34 },
    { 0xc87ef22, 61 },
    { 0x4f27ef23, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f7ef27, 36 },
    { 0xe787ef28, 90 },
    { 0xf827ef29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67ef39, 34 },
    { 0x59c7ef3a, 100 },
    { 0xc207ef39, 61 },
    { 0, 0 },
    { 0xb2f7ef3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de7ef41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae57ef44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7ef49, 34 },
    { 0, 0 },
    { 0x1687ef4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87ef50, 99 },
    { 0, 0 },
    { 0x5d37ef52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7ef56, 36 },
    { 0xaa27ef57, 98 },
    { 0x1f07ef58, 41 },
    { 0x8067ef59, 61 },
    { 0x5bc7ef57, 90 },
    { 0x2337ef5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa7ef60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817ef65, 98 },
    { 0x13d7ef66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547ef6a, 90 },
    { 0xb3d7ef6a, 90 },
    { 0, 0 },
    { 0x7b77ef6d, 46 },
    { 0x2c07ef6d, 23 },
    { 0, 0 },
    { 0x8b07ef70, 61 },
    { 0, 0 },
    { 0xe657ef72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127ef76, 34 },
    { 0x3a67ef77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc87ef7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6447ef82, 61 },
    { 0x5c37ef82, 98 },
    { 0, 0 },
    { 0x4e37ef85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407ef89, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf797ef8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c7ef90, 98 },
    { 0, 0 },
    { 0xd637ef92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7ef99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf197efab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe987efaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4397efb5, 33 },
    { 0xad67efb6, 78 },
    { 0xce67efb6, 43 },
    { 0xd4f7efb6, 76 },
    { 0xe097efb7, 90 },
    { 0x7cc7efb6, 55 },
    { 0xf357efb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb727efbe, 76 },
    { 0xe667efbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b7efc4, 61 },
    { 0xd287efc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7efc9, 84 },
    { 0x1dd7efca, 80 },
    { 0xbc17efcb, 90 },
    { 0xeda7efcb, 65 },
    { 0x2977efcd, 52 },
    { 0x4327efce, 95 },
    { 0xd837efcf, 69 },
    { 0x96f7efce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a67efd6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057efdb, 61 },
    { 0, 0 },
    { 0xa247efdd, 98 },
    { 0x99c7efde, 90 },
    { 0xb327efdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa287efe3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3767efe7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d7efed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a7eff0, 95 },
    { 0, 0 },
    { 0xc847eff2, 77 },
    { 0, 0 },
    { 0x3007eff4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7eff9, 90 },
    { 0x1467eff9, 33 },
    { 0x2a87effb, 52 },
    { 0, 0 },
    { 0x5d87effd, 91 },
    { 0x2e77effe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f7f003, 95 },
    { 0xe5f7f004, 95 },
    { 0x9f87f005, 84 },
    { 0x5ee7f004, 99 },
    { 0x16f7f005, 98 },
    { 0xf17f008, 17 },
    { 0x69a7f009, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07f010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c97f015, 78 },
    { 0xde77f015, 76 },
    { 0xff7f015, 98 },
    { 0, 0 },
    { 0x15e7f019, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c7f01c, 33 },
    { 0, 0 },
    { 0x65f7f01e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77f027, 23 },
    { 0xfdd7f028, 52 },
    { 0xa2a7f029, 68 },
    { 0xfdb7f028, 38 },
    { 0x6a67f02b, 98 },
    { 0xc8f7f028, 33 },
    { 0xd127f02d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a7f032, 61 },
    { 0, 0 },
    { 0x1687f034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8317f037, 98 },
    { 0xd97f038, 90 },
    { 0xf77f039, 64 },
    { 0x8d17f03a, 90 },
    { 0, 0 },
    { 0xd047f03c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147f03f, 90 },
    { 0xad77f03f, 34 },
    { 0x46f7f03f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427f046, 66 },
    { 0xca17f046, 23 },
    { 0xfd97f047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d7f04b, 93 },
    { 0x5cd7f04c, 33 },
    { 0, 0 },
    { 0x5957f04e, 90 },
    { 0x3617f04e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747f053, 84 },
    { 0x4947f054, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897f061, 33 },
    { 0xde77f062, 61 },
    { 0x24c7f063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07f067, 61 },
    { 0x53f7f068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb7f06e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a47f071, 61 },
    { 0, 0 },
    { 0xd2e7f073, 34 },
    { 0, 0 },
    { 0x4b7f075, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307f079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e7f07f, 98 },
    { 0, 0 },
    { 0x8eb7f081, 36 },
    { 0, 0 },
    { 0xa097f083, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd757f086, 23 },
    { 0x2fd7f086, 90 },
    { 0xa977f087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc37f094, 33 },
    { 0x6747f095, 91 },
    { 0, 0 },
    { 0x3e87f097, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487f09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017f09f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf7f0a6, 53 },
    { 0, 0 },
    { 0x4d87f0a8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b37f0ae, 3 },
    { 0x9c97f0af, 61 },
    { 0x11c7f0b0, 88 },
    { 0x13e7f0b1, 23 },
    { 0xdd87f0af, 67 },
    { 0x3717f0b1, 33 },
    { 0x7d57f0b0, 65 },
    { 0xe7f0b5, 90 },
    { 0x4467f0b4, 91 },
    { 0xede7f0b7, 30 },
    { 0, 0 },
    { 0xe8f7f0b9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd697f0bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907f0c3, 90 },
    { 0xabc7f0c4, 100 },
    { 0xcd57f0c3, 98 },
    { 0, 0 },
    { 0x5487f0c7, 99 },
    { 0x8907f0c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2767f0cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7387f0cf, 55 },
    { 0x5a17f0cf, 81 },
    { 0xf7e7f0d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7757f0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7f0dd, 61 },
    { 0x7857f0de, 88 },
    { 0xc937f0de, 38 },
    { 0xc3a7f0dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b7f0e3, 90 },
    { 0x9bd7f0e4, 61 },
    { 0xb997f0e5, 61 },
    { 0x8ba7f0e6, 87 },
    { 0xa8b7f0e7, 76 },
    { 0xeec7f0e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77f0eb, 95 },
    { 0x4fd7f0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d7f0f0, 23 },
    { 0x71a7f0f1, 33 },
    { 0x4957f0f1, 85 },
    { 0, 0 },
    { 0xf0d7f0f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847f0f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c7f0fb, 55 },
    { 0x8f7f0fc, 17 },
    { 0, 0 },
    { 0x987f0fe, 46 },
    { 0, 0 },
    { 0xa347f100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b7f103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b7f10b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417f110, 95 },
    { 0, 0 },
    { 0xe0c7f112, 78 },
    { 0xc767f113, 23 },
    { 0, 0 },
    { 0xb747f115, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf327f118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c67f11c, 100 },
    { 0x3b77f11c, 67 },
    { 0x87b7f11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797f121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7f125, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f12a, 98 },
    { 0x62c7f12b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4507f12e, 68 },
    { 0x1467f12f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7f134, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f57f13b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1277f140, 61 },
    { 0, 0 },
    { 0xbb57f142, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37f148, 61 },
    { 0x83a7f149, 95 },
    { 0xd3f7f149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557f152, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb7f157, 61 },
    { 0x8fa7f158, 17 },
    { 0xb947f159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8987f15c, 23 },
    { 0x7f87f15d, 61 },
    { 0xea57f15e, 93 },
    { 0x3137f15d, 61 },
    { 0x56a7f15c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87f165, 76 },
    { 0xef7f165, 88 },
    { 0, 0 },
    { 0x7207f168, 93 },
    { 0x7077f169, 61 },
    { 0xbeb7f16a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f16d, 23 },
    { 0, 0 },
    { 0x1e67f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47f173, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767f178, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d77f17b, 99 },
    { 0x77b7f17b, 23 },
    { 0xd117f17c, 78 },
    { 0xdaf7f17e, 93 },
    { 0xc5e7f17e, 33 },
    { 0xddc7f180, 95 },
    { 0, 0 },
    { 0xcbf7f182, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3177f185, 90 },
    { 0x2fe7f186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb127f18d, 98 },
    { 0x2277f18e, 90 },
    { 0x2487f18f, 34 },
    { 0xbd97f190, 67 },
    { 0xacc7f18d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb87f19a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa497f19d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097f1a8, 61 },
    { 0x1a27f1a9, 67 },
    { 0x5a27f1a9, 55 },
    { 0x9eb7f1aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07f1af, 78 },
    { 0x8747f1b0, 90 },
    { 0, 0 },
    { 0x1367f1b2, 33 },
    { 0xb227f1b3, 33 },
    { 0, 0 },
    { 0x9257f1b5, 90 },
    { 0x8707f1b5, 98 },
    { 0xcd47f1b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6967f1c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227f1c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f7f1d1, 76 },
    { 0x7667f1d2, 85 },
    { 0x46d7f1d2, 33 },
    { 0x8767f1d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf767f1d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3667f1da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e7f1dd, 61 },
    { 0, 0 },
    { 0xaf17f1df, 58 },
    { 0xe8a7f1e0, 48 },
    { 0x7547f1e1, 5 },
    { 0xed77f1e1, 90 },
    { 0x8b07f1e3, 90 },
    { 0x7307f1e4, 90 },
    { 0x24d7f1e2, 23 },
    { 0xec57f1e6, 87 },
    { 0x4df7f1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27f1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f1ef, 96 },
    { 0xfca7f1f0, 98 },
    { 0, 0 },
    { 0x5f67f1f2, 66 },
    { 0xad57f1f3, 95 },
    { 0, 0 },
    { 0x4ed7f1f5, 90 },
    { 0x1de7f1f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377f1fb, 95 },
    { 0, 0 },
    { 0x2297f1fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb27f200, 81 },
    { 0x6ed7f200, 34 },
    { 0xa357f202, 23 },
    { 0xc337f203, 90 },
    { 0x7d27f202, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f7f207, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a7f20a, 33 },
    { 0x40b7f20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817f20f, 23 },
    { 0, 0 },
    { 0x3c17f211, 75 },
    { 0x3b07f211, 93 },
    { 0x5067f213, 33 },
    { 0, 0 },
    { 0xf87f215, 23 },
    { 0xaf7f216, 66 },
    { 0xa6a7f216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb7f21c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a7f21f, 65 },
    { 0x1d07f220, 76 },
    { 0x1a87f220, 52 },
    { 0, 0 },
    { 0x78f7f223, 55 },
    { 0xe867f224, 90 },
    { 0x7987f225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c7f22a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb287f22f, 75 },
    { 0xe847f230, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b7f233, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff97f237, 33 },
    { 0, 0 },
    { 0xe667f239, 33 },
    { 0, 0 },
    { 0x1697f23b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca7f241, 98 },
    { 0, 0 },
    { 0xfb37f243, 23 },
    { 0, 0 },
    { 0x7237f245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9227f248, 85 },
    { 0x2f57f249, 75 },
    { 0, 0 },
    { 0xf447f24b, 23 },
    { 0x3827f24c, 93 },
    { 0x6267f24c, 76 },
    { 0x7a17f24e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a97f251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237f25e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f17f265, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a17f269, 55 },
    { 0xb287f26a, 75 },
    { 0xa247f26b, 78 },
    { 0x77b7f26c, 95 },
    { 0xb267f26d, 91 },
    { 0, 0 },
    { 0xe2f7f26f, 55 },
    { 0x5ba7f26f, 66 },
    { 0x7cc7f271, 90 },
    { 0x9f7f26f, 61 },
    { 0x86d7f270, 90 },
    { 0x2a07f273, 98 },
    { 0, 0 },
    { 0x14f7f276, 23 },
    { 0xfc67f276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd7f27b, 98 },
    { 0xb867f27c, 55 },
    { 0x6f77f27b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a7f282, 33 },
    { 0x1b67f283, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f28a, 61 },
    { 0x2d27f28b, 67 },
    { 0, 0 },
    { 0x8917f28d, 33 },
    { 0x27f7f28e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3417f293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3877f296, 76 },
    { 0xf597f297, 90 },
    { 0, 0 },
    { 0xcf27f299, 76 },
    { 0, 0 },
    { 0x4297f29b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa67f2a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed7f2a6, 43 },
    { 0x1e7f2a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f57f2ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc747f2b2, 83 },
    { 0, 0 },
    { 0xdc07f2b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e7f2ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d7f2bf, 90 },
    { 0, 0 },
    { 0x36a7f2c1, 87 },
    { 0xcf7f2c1, 10 },
    { 0xca47f2c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07f2c6, 55 },
    { 0x75c7f2c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca37f2cd, 93 },
    { 0x3d7f2ce, 61 },
    { 0, 0 },
    { 0x3647f2d0, 76 },
    { 0x25e7f2d1, 61 },
    { 0x5db7f2d2, 95 },
    { 0x5387f2d3, 66 },
    { 0xc9e7f2d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7f2db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b7f2df, 90 },
    { 0x8417f2df, 61 },
    { 0x517f2e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917f2e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d97f2e8, 95 },
    { 0x16d7f2e9, 61 },
    { 0x3c77f2ea, 90 },
    { 0, 0 },
    { 0xcfd7f2ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc807f2f0, 93 },
    { 0, 0 },
    { 0x5d77f2f2, 90 },
    { 0xaa37f2f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c7f2f6, 88 },
    { 0xab7f2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc27f2fb, 90 },
    { 0xe467f2fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d17f2ff, 23 },
    { 0xc977f2ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f308, 65 },
    { 0xb407f308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7f30e, 90 },
    { 0x8467f30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877f312, 90 },
    { 0x1317f313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9657f318, 46 },
    { 0xfba7f319, 90 },
    { 0xf047f319, 90 },
    { 0, 0 },
    { 0x7557f31c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7f322, 23 },
    { 0xd717f322, 96 },
    { 0x6877f324, 84 },
    { 0, 0 },
    { 0x2a07f326, 95 },
    { 0xa4d7f326, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797f330, 34 },
    { 0xc837f330, 72 },
    { 0x397f330, 67 },
    { 0, 0 },
    { 0x19a7f334, 33 },
    { 0x8e77f335, 33 },
    { 0xf5d7f334, 55 },
    { 0xa487f337, 33 },
    { 0x8867f338, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2957f33b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad7f33e, 33 },
    { 0x4207f33e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1067f343, 66 },
    { 0x4c57f344, 90 },
    { 0x8d07f344, 61 },
    { 0x3af7f346, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5887f34a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf87f34f, 95 },
    { 0x4fd7f350, 99 },
    { 0x9817f350, 33 },
    { 0xf597f352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7f358, 43 },
    { 0x58f7f358, 34 },
    { 0x4737f35a, 33 },
    { 0xdd57f358, 90 },
    { 0, 0 },
    { 0x23c7f35d, 99 },
    { 0xc477f35d, 93 },
    { 0xfdf7f35f, 33 },
    { 0x6e37f35f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2647f368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8497f36b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b7f36f, 91 },
    { 0, 0 },
    { 0x9327f371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8047f379, 55 },
    { 0, 0 },
    { 0x2d37f37b, 98 },
    { 0x88a7f37c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7f37f, 98 },
    { 0x35f7f380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7f38d, 46 },
    { 0xb57f38e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e7f393, 100 },
    { 0x77c7f393, 98 },
    { 0xabc7f394, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77f39a, 34 },
    { 0xf37f39b, 55 },
    { 0x45a7f39a, 38 },
    { 0x35f7f39d, 87 },
    { 0x9487f39e, 23 },
    { 0xd6b7f39e, 90 },
    { 0xb5e7f39d, 55 },
    { 0x527f39b, 98 },
    { 0xa327f39c, 35 },
    { 0, 0 },
    { 0xded7f3a4, 90 },
    { 0x6cf7f3a5, 98 },
    { 0xd5f7f3a5, 95 },
    { 0x18e7f3a7, 43 },
    { 0, 0 },
    { 0x4f17f3a9, 33 },
    { 0, 0 },
    { 0xd507f3ab, 34 },
    { 0xd9f7f3ac, 78 },
    { 0xc277f3ad, 76 },
    { 0xe7d7f3ac, 98 },
    { 0x6177f3af, 95 },
    { 0x6457f3b0, 93 },
    { 0x8ef7f3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17f3b4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e7f3b9, 42 },
    { 0, 0 },
    { 0x8257f3bb, 67 },
    { 0x60a7f3bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e7f3c0, 88 },
    { 0, 0 },
    { 0x40f7f3c2, 98 },
    { 0xc937f3c2, 99 },
    { 0x6f7f3c4, 78 },
    { 0x5547f3c4, 52 },
    { 0xe7f7f3c6, 34 },
    { 0x8157f3c7, 90 },
    { 0xf767f3c8, 46 },
    { 0x6757f3c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd157f3cc, 88 },
    { 0xbbd7f3cc, 76 },
    { 0xcb07f3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae37f3d1, 52 },
    { 0xd3e7f3d1, 95 },
    { 0, 0 },
    { 0x9c67f3d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca7f3d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb137f3dd, 95 },
    { 0, 0 },
    { 0xe6c7f3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67f3e5, 33 },
    { 0xf727f3e5, 95 },
    { 0, 0 },
    { 0x4a7f3e8, 55 },
    { 0x8b7f3e8, 100 },
    { 0x11e7f3ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc497f3ed, 90 },
    { 0xbe97f3ee, 95 },
    { 0xa6b7f3ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd417f3f6, 90 },
    { 0, 0 },
    { 0x5c77f3f8, 76 },
    { 0, 0 },
    { 0xe197f3fa, 95 },
    { 0x4e47f3fb, 66 },
    { 0x667f3fc, 75 },
    { 0x267f3fa, 43 },
    { 0x33e7f3fb, 76 },
    { 0, 0 },
    { 0xe247f400, 90 },
    { 0, 0 },
    { 0x9757f402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b7f405, 76 },
    { 0, 0 },
    { 0x5ca7f407, 95 },
    { 0x5107f408, 61 },
    { 0, 0 },
    { 0xf047f40a, 84 },
    { 0, 0 },
    { 0xc7a7f40c, 33 },
    { 0, 0 },
    { 0x93e7f40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927f411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd7f415, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea7f41b, 90 },
    { 0, 0 },
    { 0x3db7f41d, 90 },
    { 0xef67f41e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7f422, 98 },
    { 0xdac7f423, 87 },
    { 0x39d7f424, 33 },
    { 0xd3f7f425, 33 },
    { 0xea37f423, 61 },
    { 0x6877f427, 43 },
    { 0xc617f426, 98 },
    { 0xdb87f427, 87 },
    { 0xe7b7f42a, 90 },
    { 0x3347f42a, 52 },
    { 0x7847f426, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57f430, 23 },
    { 0xc267f431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe617f435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc577f43c, 55 },
    { 0, 0 },
    { 0xadd7f43e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8207f445, 33 },
    { 0xa367f445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1737f449, 46 },
    { 0, 0 },
    { 0xb17f44b, 61 },
    { 0, 0 },
    { 0x85f7f44d, 23 },
    { 0x2df7f44e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67f457, 66 },
    { 0x9957f457, 95 },
    { 0x2b27f459, 61 },
    { 0x2a67f459, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda7f460, 87 },
    { 0xd857f461, 99 },
    { 0x1157f461, 99 },
    { 0, 0 },
    { 0xd607f464, 81 },
    { 0x52c7f465, 90 },
    { 0xbde7f465, 94 },
    { 0, 0 },
    { 0xb197f468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7f46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f7f475, 33 },
    { 0x9307f476, 34 },
    { 0x7067f477, 95 },
    { 0xd847f477, 66 },
    { 0x8aa7f479, 34 },
    { 0xf417f47a, 95 },
    { 0x1c7f478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e67f47e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd397f482, 73 },
    { 0, 0 },
    { 0x51a7f484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5367f48c, 33 },
    { 0x2217f48d, 95 },
    { 0x13b7f48e, 23 },
    { 0, 0 },
    { 0x2677f490, 23 },
    { 0x3407f491, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5167f495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237f4a1, 95 },
    { 0x29f7f4a2, 90 },
    { 0x7867f4a1, 98 },
    { 0x6b7f4a3, 23 },
    { 0x2447f4a5, 90 },
    { 0xc127f4a6, 33 },
    { 0x8df7f4a2, 76 },
    { 0x6597f4a5, 61 },
    { 0xf7d7f4a7, 93 },
    { 0, 0 },
    { 0x2237f4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7f4af, 95 },
    { 0x3ec7f4af, 61 },
    { 0x2c97f4b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e7f4b8, 76 },
    { 0, 0 },
    { 0x6287f4ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7f4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407f4c0, 76 },
    { 0, 0 },
    { 0xce97f4c2, 90 },
    { 0xf67f4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7f4c9, 78 },
    { 0xe767f4c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e7f4d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647f4d7, 78 },
    { 0, 0 },
    { 0x21a7f4d9, 43 },
    { 0xb7f7f4d9, 80 },
    { 0x8417f4db, 76 },
    { 0xe117f4dc, 23 },
    { 0x39a7f4dc, 61 },
    { 0x9eb7f4db, 90 },
    { 0x9767f4df, 23 },
    { 0x2727f4de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa547f4e6, 33 },
    { 0x9cf7f4e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce7f4f3, 46 },
    { 0, 0 },
    { 0xfc57f4f5, 90 },
    { 0x4f57f4f6, 55 },
    { 0xa0d7f4f7, 23 },
    { 0xbc97f4f7, 95 },
    { 0xec77f4f5, 23 },
    { 0xdc87f4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f7f4fd, 46 },
    { 0x2597f4fe, 81 },
    { 0, 0 },
    { 0x3cf7f500, 33 },
    { 0x7277f501, 90 },
    { 0x8017f500, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe77f506, 76 },
    { 0x4e97f507, 61 },
    { 0x5ff7f508, 90 },
    { 0xf087f509, 55 },
    { 0xbc17f50a, 66 },
    { 0xe8a7f50b, 95 },
    { 0x6767f508, 55 },
    { 0, 0 },
    { 0x49e7f50e, 76 },
    { 0x13c7f50f, 65 },
    { 0xb6b7f50e, 90 },
    { 0xf297f50e, 23 },
    { 0, 0 },
    { 0x3607f513, 95 },
    { 0x90c7f514, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507f51b, 33 },
    { 0x1437f51c, 33 },
    { 0x5357f51d, 98 },
    { 0, 0 },
    { 0xf687f51f, 43 },
    { 0, 0 },
    { 0xdb07f521, 55 },
    { 0xd637f521, 76 },
    { 0x60e7f521, 84 },
    { 0xb117f524, 76 },
    { 0x23a7f523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567f529, 33 },
    { 0, 0 },
    { 0x4367f52b, 34 },
    { 0x4a47f52c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9397f530, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177f533, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d7f536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef77f53c, 33 },
    { 0, 0 },
    { 0x8f87f53e, 23 },
    { 0x9347f53e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc537f546, 55 },
    { 0, 0 },
    { 0xa357f548, 76 },
    { 0xb907f548, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97f54e, 55 },
    { 0x3737f54f, 84 },
    { 0xe787f54f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb7f555, 92 },
    { 0x8c97f556, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647f55f, 34 },
    { 0xdeb7f560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e27f563, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b7f568, 55 },
    { 0, 0 },
    { 0xcea7f56a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5107f56d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7f570, 90 },
    { 0xe627f571, 66 },
    { 0xac7f572, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc547f576, 33 },
    { 0, 0 },
    { 0xc647f578, 91 },
    { 0, 0 },
    { 0xe487f57a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87f586, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa7f589, 98 },
    { 0xa147f58a, 33 },
    { 0x6dc7f58b, 95 },
    { 0xb9f7f58c, 90 },
    { 0x70b7f58d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87f591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c47f595, 55 },
    { 0xb9f7f596, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea7f5a2, 76 },
    { 0x8337f5a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f5a7, 92 },
    { 0x51b7f5a8, 46 },
    { 0xf667f5a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf57f5ae, 91 },
    { 0xca67f5ae, 33 },
    { 0x3ba7f5b0, 90 },
    { 0x3a47f5b1, 23 },
    { 0x9697f5b2, 55 },
    { 0, 0 },
    { 0xf077f5b4, 67 },
    { 0xf977f5b5, 98 },
    { 0x9c57f5b6, 90 },
    { 0x9ca7f5b6, 85 },
    { 0x3357f5b7, 90 },
    { 0xddc7f5b9, 33 },
    { 0x28e7f5ba, 34 },
    { 0x73e7f5b5, 90 },
    { 0xcc67f5b7, 90 },
    { 0x8e87f5bc, 33 },
    { 0, 0 },
    { 0x2b07f5bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557f5c4, 90 },
    { 0xae57f5c5, 55 },
    { 0x4e97f5c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437f5d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c87f5d3, 66 },
    { 0xb127f5d4, 23 },
    { 0xf647f5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f5e0, 23 },
    { 0xd477f5e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa057f5e4, 61 },
    { 0, 0 },
    { 0x3a07f5e6, 23 },
    { 0x5967f5e6, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf7f5eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d7f5f7, 61 },
    { 0xb4c7f5f7, 46 },
    { 0x9717f5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc7f5ff, 90 },
    { 0xb5f7f600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d47f603, 66 },
    { 0x5807f604, 61 },
    { 0x59e7f605, 43 },
    { 0x5b7f606, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7f60e, 46 },
    { 0xe4e7f60f, 23 },
    { 0x3737f60e, 55 },
    { 0x5bb7f611, 98 },
    { 0x21d7f60e, 96 },
    { 0xeea7f613, 34 },
    { 0x7c97f60e, 80 },
    { 0xf1e7f615, 90 },
    { 0x80c7f615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e67f61a, 70 },
    { 0x3377f61a, 23 },
    { 0x86e7f61c, 78 },
    { 0x647f61d, 98 },
    { 0x7e07f61c, 23 },
    { 0xb3b7f61e, 52 },
    { 0x1237f620, 90 },
    { 0x2457f621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd7f625, 55 },
    { 0, 0 },
    { 0x7e97f627, 93 },
    { 0x22d7f628, 98 },
    { 0xc787f629, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7f62e, 98 },
    { 0x5ff7f62e, 100 },
    { 0x50e7f630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f47f633, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f17f637, 61 },
    { 0x54c7f638, 90 },
    { 0x6087f637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47f63c, 43 },
    { 0xb387f63c, 90 },
    { 0x1b67f63d, 75 },
    { 0xa687f63f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc757f642, 77 },
    { 0x6a07f642, 78 },
    { 0x8f37f644, 33 },
    { 0x3167f645, 61 },
    { 0x2037f644, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce7f64a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77f64f, 61 },
    { 0x1847f650, 23 },
    { 0x1e17f64f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb97f655, 61 },
    { 0x4bd7f656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3187f659, 81 },
    { 0, 0 },
    { 0x8d67f65b, 90 },
    { 0x10a7f65c, 90 },
    { 0x78e7f65d, 46 },
    { 0x66c7f65e, 98 },
    { 0x2ab7f65d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f97f664, 61 },
    { 0, 0 },
    { 0x56a7f666, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae7f669, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc397f66f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf7f673, 61 },
    { 0, 0 },
    { 0x6d97f675, 61 },
    { 0x987f676, 93 },
    { 0xd3f7f677, 33 },
    { 0x8f87f677, 98 },
    { 0x727f679, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe87f67f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a7f68e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7f692, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87f695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b7f69f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77f6a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1517f6a9, 95 },
    { 0x6b77f6aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7f6ad, 75 },
    { 0, 0 },
    { 0xeb37f6af, 91 },
    { 0x7207f6af, 61 },
    { 0x1a7f6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7f6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea7f6bc, 23 },
    { 0x8b87f6bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f7f6c0, 87 },
    { 0x9f7f6c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b7f6c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf847f6c7, 90 },
    { 0x4ca7f6c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27f6cb, 90 },
    { 0xbc37f6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc847f6d5, 33 },
    { 0x9637f6d6, 52 },
    { 0x4617f6d6, 95 },
    { 0x5e47f6d7, 76 },
    { 0x72b7f6d8, 23 },
    { 0x2ea7f6da, 93 },
    { 0, 0 },
    { 0xd877f6dc, 95 },
    { 0x7657f6dd, 46 },
    { 0xb867f6de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d7f6e1, 76 },
    { 0x9427f6e1, 61 },
    { 0, 0 },
    { 0x4e37f6e4, 76 },
    { 0x95a7f6e5, 95 },
    { 0x3b87f6e5, 33 },
    { 0xd9a7f6e5, 55 },
    { 0, 0 },
    { 0x4f57f6e9, 100 },
    { 0xeb07f6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7f6ed, 90 },
    { 0x44c7f6ee, 55 },
    { 0xc2d7f6ef, 55 },
    { 0xc997f6f0, 33 },
    { 0xc0d7f6f0, 33 },
    { 0x6a57f6f2, 95 },
    { 0, 0 },
    { 0x6067f6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c7f6ff, 61 },
    { 0xeda7f6ff, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f7f704, 23 },
    { 0, 0 },
    { 0xeb67f706, 33 },
    { 0x37a7f707, 61 },
    { 0, 0 },
    { 0x8a37f709, 100 },
    { 0, 0 },
    { 0x547f70b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa687f712, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337f715, 61 },
    { 0, 0 },
    { 0x5f57f717, 61 },
    { 0x5e07f718, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb197f71d, 61 },
    { 0, 0 },
    { 0x2797f71f, 95 },
    { 0, 0 },
    { 0xed67f721, 33 },
    { 0x3437f722, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9927f725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e7f728, 66 },
    { 0xd777f728, 68 },
    { 0, 0 },
    { 0x31c7f72b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d7f733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227f73a, 100 },
    { 0xf957f73a, 99 },
    { 0, 0 },
    { 0xd417f73d, 90 },
    { 0xff87f73e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f7f741, 75 },
    { 0, 0 },
    { 0x95c7f743, 87 },
    { 0xd8c7f743, 76 },
    { 0x34d7f745, 95 },
    { 0, 0 },
    { 0x2ae7f747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf337f74c, 95 },
    { 0, 0 },
    { 0xdaa7f74e, 98 },
    { 0, 0 },
    { 0x4237f750, 90 },
    { 0xd0d7f750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257f758, 76 },
    { 0x4bd7f758, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb67f762, 55 },
    { 0xb4e7f762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb7f76a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab7f76e, 85 },
    { 0, 0 },
    { 0xa4a7f770, 98 },
    { 0, 0 },
    { 0x99c7f772, 65 },
    { 0x53f7f773, 90 },
    { 0x5d37f774, 87 },
    { 0, 0 },
    { 0x1bd7f776, 61 },
    { 0x15e7f777, 94 },
    { 0x1b27f778, 76 },
    { 0x16f7f779, 76 },
    { 0x6f17f779, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e17f784, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe997f787, 33 },
    { 0xe997f788, 91 },
    { 0, 0 },
    { 0xda47f78a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9817f78f, 91 },
    { 0x7c77f790, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5077f794, 90 },
    { 0x7f17f794, 97 },
    { 0x1787f794, 98 },
    { 0xc697f797, 61 },
    { 0x5207f798, 95 },
    { 0x88d7f797, 33 },
    { 0xe4f7f799, 66 },
    { 0xb707f79b, 100 },
    { 0x73a7f799, 98 },
    { 0, 0 },
    { 0x3697f79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc117f7a1, 33 },
    { 0x40e7f7a2, 23 },
    { 0xc377f7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67f7a6, 33 },
    { 0xde07f7a7, 93 },
    { 0x6527f7a8, 77 },
    { 0x24b7f7a9, 99 },
    { 0xe5d7f7a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d87f7ad, 33 },
    { 0x6a47f7ae, 23 },
    { 0xda37f7af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457f7b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe7f7b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd97f7b9, 81 },
    { 0, 0 },
    { 0xa7c7f7bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc947f7c1, 33 },
    { 0x86d7f7c1, 98 },
    { 0x2557f7c2, 61 },
    { 0xa477f7c4, 90 },
    { 0xddb7f7c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7f7c8, 61 },
    { 0xb287f7c9, 40 },
    { 0, 0 },
    { 0xdcd7f7cb, 66 },
    { 0, 0 },
    { 0xe447f7cd, 88 },
    { 0x1547f7cd, 23 },
    { 0x55a7f7cf, 67 },
    { 0xc2b7f7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297f7d3, 23 },
    { 0x2c7f7d3, 61 },
    { 0xe917f7d3, 33 },
    { 0, 0 },
    { 0x7de7f7d7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77f7da, 55 },
    { 0xf817f7db, 34 },
    { 0x6517f7dc, 98 },
    { 0x7ad7f7dc, 76 },
    { 0x5e87f7da, 88 },
    { 0x1f97f7df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef7f7e2, 90 },
    { 0x5fa7f7e3, 46 },
    { 0x5fb7f7e4, 43 },
    { 0xce7f7e4, 33 },
    { 0x45e7f7e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d67f7e9, 67 },
    { 0xcef7f7e9, 61 },
    { 0, 0 },
    { 0xe87f7ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e7f7ef, 43 },
    { 0, 0 },
    { 0x9b07f7f1, 34 },
    { 0, 0 },
    { 0xba37f7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2417f7f8, 61 },
    { 0x3df7f7f9, 90 },
    { 0x9f27f7fa, 23 },
    { 0x7327f7fa, 95 },
    { 0x7b7f7f8, 43 },
    { 0x817f7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb467f800, 55 },
    { 0xfef7f801, 23 },
    { 0x23c7f802, 95 },
    { 0x4e57f803, 95 },
    { 0, 0 },
    { 0xbd47f805, 98 },
    { 0x3727f805, 55 },
    { 0xe0a7f807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7f80a, 33 },
    { 0, 0 },
    { 0x8ca7f80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3707f811, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557f817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f7f81b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef7f81f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1687f826, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc97f82b, 77 },
    { 0xb097f82c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc7f832, 61 },
    { 0xfdb7f832, 90 },
    { 0x3517f833, 33 },
    { 0x4487f834, 98 },
    { 0, 0 },
    { 0x1e87f837, 48 },
    { 0, 0 },
    { 0x85d7f839, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef7f840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd07f847, 95 },
    { 0, 0 },
    { 0xcec7f849, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427f84c, 43 },
    { 0, 0 },
    { 0xab27f84e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1857f855, 61 },
    { 0, 0 },
    { 0x6487f857, 33 },
    { 0xd977f858, 52 },
    { 0, 0 },
    { 0x39a7f85a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7f85e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf297f861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c07f869, 75 },
    { 0xb587f869, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917f870, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc367f874, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe037f87b, 98 },
    { 0xb887f87c, 87 },
    { 0xa7f7f87b, 23 },
    { 0xf027f87e, 33 },
    { 0x2047f87d, 99 },
    { 0x4f7f87c, 76 },
    { 0x5de7f87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f885, 90 },
    { 0xeb37f885, 84 },
    { 0x39b7f887, 33 },
    { 0, 0 },
    { 0xc507f889, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7347f88e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd37f891, 95 },
    { 0, 0 },
    { 0xcc77f893, 90 },
    { 0x987f894, 95 },
    { 0, 0 },
    { 0xd9b7f896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6897f89b, 76 },
    { 0, 0 },
    { 0xf257f89d, 84 },
    { 0xf7a7f89e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7f8a6, 91 },
    { 0x8bb7f8a6, 98 },
    { 0x3c87f8a7, 98 },
    { 0, 0 },
    { 0x1657f8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8027f8ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d7f8b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7f8b6, 33 },
    { 0x73e7f8b7, 90 },
    { 0x5437f8b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e7f8bc, 61 },
    { 0xc9b7f8bd, 61 },
    { 0x1737f8be, 93 },
    { 0x2487f8bf, 99 },
    { 0x8a67f8c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f7f8c5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb7f8d0, 68 },
    { 0xc7f7f8d1, 67 },
    { 0xa327f8d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147f8d9, 67 },
    { 0xac77f8da, 95 },
    { 0, 0 },
    { 0xb587f8dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f7f8e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e97f8e8, 33 },
    { 0x4857f8e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9787f8ed, 98 },
    { 0, 0 },
    { 0x7c37f8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7f8f3, 46 },
    { 0x6727f8f3, 33 },
    { 0x2c27f8f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a97f8f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7397f8fb, 43 },
    { 0x4a17f8fc, 23 },
    { 0x1be7f8fd, 95 },
    { 0xaf87f8fb, 33 },
    { 0x4517f8ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617f911, 33 },
    { 0xc4d7f912, 66 },
    { 0x4717f911, 85 },
    { 0x4aa7f912, 17 },
    { 0xecd7f912, 23 },
    { 0xa527f913, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7f919, 33 },
    { 0x92c7f91a, 33 },
    { 0x5157f91a, 66 },
    { 0, 0 },
    { 0x7b77f91d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf7f922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337f925, 55 },
    { 0xfd37f926, 33 },
    { 0xe157f927, 90 },
    { 0xd487f926, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7f92c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7f930, 95 },
    { 0x907f931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7f934, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf57f93a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd087f940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca7f944, 99 },
    { 0, 0 },
    { 0x5407f946, 34 },
    { 0xc767f946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7f94a, 33 },
    { 0x7db7f94b, 68 },
    { 0, 0 },
    { 0x1e17f94d, 87 },
    { 0x9167f94e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e7f953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d7f95c, 98 },
    { 0, 0 },
    { 0xaa37f95e, 95 },
    { 0, 0 },
    { 0xe317f960, 17 },
    { 0xa117f961, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc7f965, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d7f96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7f970, 66 },
    { 0x7c07f971, 33 },
    { 0x1bf7f972, 34 },
    { 0x5387f972, 98 },
    { 0x3767f970, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff7f97b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7f97f, 98 },
    { 0, 0 },
    { 0xe007f981, 55 },
    { 0x9057f982, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787f989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8037f98f, 61 },
    { 0x5d57f990, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae37f996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3377f999, 76 },
    { 0x3337f99a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7f99f, 33 },
    { 0xfb57f9a0, 55 },
    { 0xb6d7f99f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa7f9a6, 95 },
    { 0x92b7f9a7, 90 },
    { 0x8d57f9a7, 81 },
    { 0x2c97f9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde77f9ac, 95 },
    { 0x9e57f9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7f9b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607f9b7, 65 },
    { 0xf117f9b8, 55 },
    { 0x1a17f9b7, 100 },
    { 0xaba7f9ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4587f9bd, 78 },
    { 0, 0 },
    { 0x2ee7f9bf, 95 },
    { 0xfdc7f9bf, 61 },
    { 0x3bd7f9c1, 76 },
    { 0x7667f9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f9ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e7f9d0, 67 },
    { 0, 0 },
    { 0x6f17f9d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4817f9d5, 61 },
    { 0xaba7f9d6, 95 },
    { 0x8127f9d7, 90 },
    { 0xb947f9d5, 77 },
    { 0xcfb7f9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf7f9dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe177f9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff87f9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7f9e9, 98 },
    { 0, 0 },
    { 0xcf27f9eb, 90 },
    { 0xcff7f9ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x707f9ef, 23 },
    { 0, 0 },
    { 0xa897f9f1, 61 },
    { 0, 0 },
    { 0xc807f9f3, 76 },
    { 0, 0 },
    { 0xb497f9f5, 34 },
    { 0x7327f9f5, 55 },
    { 0x1f97f9f7, 95 },
    { 0, 0 },
    { 0xf247f9f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2947f9fd, 76 },
    { 0, 0 },
    { 0x1767f9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157fa03, 98 },
    { 0xb767fa04, 38 },
    { 0x9d67fa04, 33 },
    { 0, 0 },
    { 0x1307fa07, 23 },
    { 0xc2e7fa07, 95 },
    { 0, 0 },
    { 0xd07fa0a, 61 },
    { 0x3d57fa0a, 33 },
    { 0xa5f7fa0a, 67 },
    { 0xa227fa0d, 34 },
    { 0x2b7fa0e, 23 },
    { 0xc297fa0d, 90 },
    { 0, 0 },
    { 0x1867fa11, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb7fa18, 33 },
    { 0, 0 },
    { 0x857fa1a, 61 },
    { 0, 0 },
    { 0x8787fa1c, 66 },
    { 0x2f37fa1c, 55 },
    { 0x9947fa1e, 33 },
    { 0x8477fa1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa7fa23, 95 },
    { 0, 0 },
    { 0xa757fa25, 23 },
    { 0, 0 },
    { 0x9b97fa27, 98 },
    { 0x9467fa28, 33 },
    { 0x9397fa28, 90 },
    { 0, 0 },
    { 0xd777fa2b, 78 },
    { 0xe687fa2c, 56 },
    { 0xcd97fa2b, 81 },
    { 0xc097fa2e, 91 },
    { 0xbf77fa2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde87fa34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd527fa38, 76 },
    { 0, 0 },
    { 0xea07fa3a, 95 },
    { 0, 0 },
    { 0x6dc7fa3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7fa3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757fa42, 98 },
    { 0x28b7fa43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a37fa48, 55 },
    { 0xafd7fa48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197fa50, 95 },
    { 0x107fa51, 98 },
    { 0x83d7fa52, 33 },
    { 0, 0 },
    { 0xc5f7fa54, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7fa58, 98 },
    { 0, 0 },
    { 0x2517fa5a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c37fa5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27fa62, 33 },
    { 0xeea7fa63, 84 },
    { 0xd9c7fa64, 43 },
    { 0xd2e7fa65, 61 },
    { 0xa7d7fa66, 55 },
    { 0x637fa67, 55 },
    { 0x4687fa68, 95 },
    { 0x3537fa69, 99 },
    { 0xfe07fa69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b7fa6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9557fa74, 84 },
    { 0, 0 },
    { 0xedc7fa76, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab7fa7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f7fa80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf77fa84, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5047fa87, 76 },
    { 0x3aa7fa88, 46 },
    { 0x9147fa89, 76 },
    { 0, 0 },
    { 0x9cc7fa8b, 90 },
    { 0, 0 },
    { 0xfb67fa8d, 93 },
    { 0x7067fa8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77fa92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847fa98, 61 },
    { 0xe817fa99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7fa9d, 90 },
    { 0, 0 },
    { 0x4ab7fa9f, 78 },
    { 0, 0 },
    { 0x3d27faa1, 100 },
    { 0, 0 },
    { 0x14e7faa3, 98 },
    { 0x4397faa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7faa9, 34 },
    { 0, 0 },
    { 0x3217faab, 61 },
    { 0xef97faac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1677fab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc897fabc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de7fac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7fac4, 84 },
    { 0, 0 },
    { 0x4037fac6, 61 },
    { 0, 0 },
    { 0xcd07fac8, 33 },
    { 0, 0 },
    { 0x6f67faca, 33 },
    { 0x9a17facb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb7fad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5827fad3, 98 },
    { 0x1e57fad4, 88 },
    { 0x3b07fad5, 99 },
    { 0xe6a7fad6, 53 },
    { 0x57a7fad7, 34 },
    { 0xa0d7fad8, 88 },
    { 0xd57fad8, 75 },
    { 0x4137fada, 90 },
    { 0xc8e7fad9, 33 },
    { 0xd417fad5, 94 },
    { 0, 0 },
    { 0xfbc7fade, 76 },
    { 0xe577fadf, 77 },
    { 0x6997fae0, 78 },
    { 0x9f07fae1, 65 },
    { 0, 0 },
    { 0x54f7fae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa707faea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d17faee, 61 },
    { 0xd647faef, 46 },
    { 0x1f17faef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1837faf6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9507faf9, 93 },
    { 0xb807faf9, 33 },
    { 0x3cf7fafb, 52 },
    { 0x8247fafb, 76 },
    { 0x7147fafd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7fb00, 76 },
    { 0, 0 },
    { 0xe9c7fb02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f7fb07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de7fb0d, 95 },
    { 0x64c7fb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b7fb15, 23 },
    { 0xe667fb16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda47fb1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e37fb1d, 46 },
    { 0, 0 },
    { 0x71b7fb1f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d97fb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf967fb2b, 90 },
    { 0xae57fb2c, 95 },
    { 0x38f7fb2b, 24 },
    { 0xaa07fb2e, 33 },
    { 0, 0 },
    { 0x1e67fb30, 68 },
    { 0xd837fb31, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6457fb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f67fb3a, 55 },
    { 0x37c7fb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7fb3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa517fb4b, 98 },
    { 0, 0 },
    { 0x18d7fb4d, 46 },
    { 0, 0 },
    { 0x71f7fb4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf177fb55, 95 },
    { 0, 0 },
    { 0x6aa7fb57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27fb5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a97fb5f, 88 },
    { 0x8447fb5f, 91 },
    { 0x3c97fb61, 95 },
    { 0x3397fb62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2437fb69, 43 },
    { 0x5197fb69, 69 },
    { 0, 0 },
    { 0x1567fb6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3707fb6f, 61 },
    { 0xa5d7fb6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7fb75, 33 },
    { 0x1b37fb75, 34 },
    { 0x1e67fb77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f7fb7a, 55 },
    { 0xd5f7fb7b, 98 },
    { 0, 0 },
    { 0x80e7fb7d, 61 },
    { 0x9a27fb7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9457fb84, 33 },
    { 0xcca7fb84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7fb8a, 55 },
    { 0x53e7fb8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd457fb8f, 77 },
    { 0x7e57fb90, 90 },
    { 0xdec7fb8f, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x9407fb94, 90 },
    { 0, 0 },
    { 0x3f87fb96, 23 },
    { 0x9bb7fb97, 76 },
    { 0x4f07fb96, 43 },
    { 0xd3d7fb99, 68 },
    { 0x92c7fb9a, 96 },
    { 0xb7f7fb98, 90 },
    { 0x3907fb9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c97fba2, 66 },
    { 0x4e37fba2, 43 },
    { 0x18f7fba2, 99 },
    { 0xe4f7fba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f07fbab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27fbae, 66 },
    { 0, 0 },
    { 0x5ba7fbb0, 68 },
    { 0x9837fbb1, 33 },
    { 0xfeb7fbb2, 90 },
    { 0x3357fbb3, 61 },
    { 0x25c7fbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a67fbb7, 61 },
    { 0, 0 },
    { 0x5847fbb9, 95 },
    { 0x7847fbba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17fbbd, 33 },
    { 0xb347fbbe, 17 },
    { 0, 0 },
    { 0x7ac7fbc0, 87 },
    { 0x6c17fbc1, 76 },
    { 0, 0 },
    { 0xa867fbc3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd7fbc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef7fbc9, 90 },
    { 0xf6d7fbc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc047fbcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4317fbd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f7fbd7, 61 },
    { 0x6897fbd7, 84 },
    { 0x9f7fbd9, 98 },
    { 0x127fbda, 99 },
    { 0x9117fbd8, 55 },
    { 0x6b57fbda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707fbe2, 98 },
    { 0, 0 },
    { 0x7497fbe4, 99 },
    { 0, 0 },
    { 0xf447fbe6, 33 },
    { 0x37d7fbe6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7fbeb, 90 },
    { 0, 0 },
    { 0x68b7fbed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d7fbf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf997fbf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817fbf8, 98 },
    { 0, 0 },
    { 0x9e37fbfa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67fbff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc7fc05, 90 },
    { 0, 0 },
    { 0xb1e7fc07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c57fc0a, 90 },
    { 0xd4e7fc0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa197fc0e, 34 },
    { 0x6487fc0f, 33 },
    { 0x4e7fc10, 33 },
    { 0x8977fc11, 52 },
    { 0x7087fc12, 100 },
    { 0x9477fc13, 90 },
    { 0, 0 },
    { 0x4527fc15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b7fc1b, 33 },
    { 0xccc7fc1b, 33 },
    { 0, 0 },
    { 0x1ec7fc1e, 96 },
    { 0x22b7fc1f, 100 },
    { 0xdd77fc20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe07fc29, 90 },
    { 0, 0 },
    { 0xc8e7fc2b, 61 },
    { 0xd687fc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8657fc2f, 98 },
    { 0, 0 },
    { 0x19e7fc31, 61 },
    { 0xfed7fc32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4497fc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b97fc3a, 95 },
    { 0xdc47fc3b, 76 },
    { 0x1b87fc3a, 90 },
    { 0, 0 },
    { 0xed7fc3e, 80 },
    { 0x95c7fc3e, 55 },
    { 0x87c7fc3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d7fc46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657fc4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa647fc53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c7fc57, 76 },
    { 0x48d7fc58, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7fc5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d7fc61, 90 },
    { 0x8927fc61, 99 },
    { 0xd157fc63, 23 },
    { 0, 0 },
    { 0xc237fc65, 33 },
    { 0x5347fc65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc717fc6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67fc71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2957fc75, 55 },
    { 0xe057fc76, 76 },
    { 0x6707fc77, 95 },
    { 0xdc17fc78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca7fc80, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7fc87, 23 },
    { 0, 0 },
    { 0x2237fc89, 23 },
    { 0x3637fc8a, 90 },
    { 0x3347fc8b, 23 },
    { 0xeb37fc8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff7fc90, 80 },
    { 0xc887fc91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f7fc94, 55 },
    { 0, 0 },
    { 0xcf07fc96, 100 },
    { 0x13b7fc97, 43 },
    { 0x93a7fc98, 91 },
    { 0x4ab7fc98, 78 },
    { 0, 0 },
    { 0x5dd7fc9b, 33 },
    { 0, 0 },
    { 0x9ac7fc9d, 76 },
    { 0x5337fc9e, 91 },
    { 0, 0 },
    { 0x62f7fca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa67fca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf67fcab, 61 },
    { 0xdff7fcab, 17 },
    { 0, 0 },
    { 0x3c67fcae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607fcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f7fcb8, 61 },
    { 0x16a7fcb9, 66 },
    { 0xc7c7fcba, 61 },
    { 0, 0 },
    { 0xd27fcbc, 23 },
    { 0, 0 },
    { 0xb537fcbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1717fcc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb17fcca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb997fccd, 95 },
    { 0, 0 },
    { 0x77a7fccf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c7fcd3, 53 },
    { 0x8947fcd4, 34 },
    { 0x7f67fcd4, 23 },
    { 0x2297fcd3, 34 },
    { 0xe517fcd4, 66 },
    { 0x63f7fcd8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d7fcdb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff7fce0, 55 },
    { 0x7ef7fce0, 66 },
    { 0x29d7fce2, 61 },
    { 0xa147fce3, 90 },
    { 0x91d7fce4, 55 },
    { 0x48c7fce5, 76 },
    { 0xc637fce5, 90 },
    { 0x7957fce5, 61 },
    { 0x30e7fce2, 76 },
    { 0x1cb7fce9, 85 },
    { 0x8377fce9, 52 },
    { 0, 0 },
    { 0xd887fcec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7fcef, 75 },
    { 0xda77fcf0, 46 },
    { 0x5947fcf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e27fcf4, 99 },
    { 0x5667fcf5, 93 },
    { 0xae97fcf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e07fcf9, 90 },
    { 0x2967fcfa, 61 },
    { 0x3467fcfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f7fcfe, 90 },
    { 0, 0 },
    { 0xc3a7fd00, 88 },
    { 0x45d7fd00, 43 },
    { 0, 0 },
    { 0x9117fd03, 43 },
    { 0, 0 },
    { 0x5447fd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c77fd09, 42 },
    { 0x8357fd0a, 61 },
    { 0x6a87fd0a, 95 },
    { 0xb977fd0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe877fd10, 76 },
    { 0x16c7fd11, 23 },
    { 0xc1e7fd11, 95 },
    { 0x6487fd11, 43 },
    { 0, 0 },
    { 0xa6c7fd15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d7fd1a, 95 },
    { 0xf5a7fd1b, 43 },
    { 0, 0 },
    { 0xbcd7fd1d, 61 },
    { 0xcbb7fd1e, 76 },
    { 0x46e7fd1f, 90 },
    { 0xefc7fd1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d7fd2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7fd2e, 87 },
    { 0xb8d7fd2f, 61 },
    { 0, 0 },
    { 0xd4c7fd31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b57fd34, 99 },
    { 0x6877fd35, 95 },
    { 0x3757fd36, 56 },
    { 0x6fa7fd36, 95 },
    { 0x8d87fd38, 98 },
    { 0x1ca7fd39, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b47fd3c, 98 },
    { 0x6257fd3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7fd46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf547fd49, 33 },
    { 0, 0 },
    { 0xc0c7fd4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf7fd55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea7fd59, 88 },
    { 0x69d7fd5a, 90 },
    { 0x4177fd5b, 55 },
    { 0x2b17fd5c, 33 },
    { 0x8dd7fd5b, 74 },
    { 0x9147fd5e, 95 },
    { 0x48e7fd5f, 61 },
    { 0x7b97fd60, 17 },
    { 0x43b7fd5f, 95 },
    { 0xdb87fd60, 38 },
    { 0xad27fd5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2767fd69, 23 },
    { 0xa3e7fd6a, 23 },
    { 0x6a57fd6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b7fd6f, 76 },
    { 0, 0 },
    { 0x78d7fd71, 76 },
    { 0xd587fd72, 90 },
    { 0xf1c7fd73, 33 },
    { 0x1d37fd74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7fd78, 46 },
    { 0xe4d7fd79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a7fd7e, 55 },
    { 0, 0 },
    { 0x1c47fd80, 61 },
    { 0xd427fd81, 55 },
    { 0, 0 },
    { 0xcf07fd83, 23 },
    { 0x9c17fd84, 63 },
    { 0x8aa7fd85, 65 },
    { 0xa6e7fd86, 90 },
    { 0x1497fd86, 91 },
    { 0x6f27fd86, 99 },
    { 0, 0 },
    { 0x68b7fd8a, 65 },
    { 0x77a7fd8b, 95 },
    { 0, 0 },
    { 0x49f7fd8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a7fd90, 93 },
    { 0, 0 },
    { 0x8bd7fd92, 90 },
    { 0, 0 },
    { 0x9b67fd94, 76 },
    { 0x37a7fd94, 33 },
    { 0, 0 },
    { 0xb557fd97, 61 },
    { 0, 0 },
    { 0x5ac7fd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5447fda5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be7fda9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747fdad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77fdb3, 23 },
    { 0x9947fdb4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf87fdb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce7fdbd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c87fdc0, 55 },
    { 0xb617fdc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97fdc4, 40 },
    { 0x1727fdc5, 90 },
    { 0x2287fdc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47fdcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a77fdcf, 90 },
    { 0, 0 },
    { 0x45d7fdd1, 61 },
    { 0x7717fdd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7fdda, 33 },
    { 0x1467fddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047fde3, 90 },
    { 0, 0 },
    { 0xa097fde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677fdef, 46 },
    { 0, 0 },
    { 0x27fdf1, 90 },
    { 0x76d7fdf2, 90 },
    { 0x7127fdf2, 90 },
    { 0xad07fdf4, 89 },
    { 0x157fdf5, 90 },
    { 0x16f7fdf6, 33 },
    { 0x1b87fdf6, 43 },
    { 0x6107fdf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5607fdfc, 55 },
    { 0xf137fdfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77fe06, 98 },
    { 0xde27fe07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d7fe0e, 98 },
    { 0, 0 },
    { 0xac47fe10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67fe15, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe18, 100 },
    { 0x1c7fe19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587fe21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5277fe25, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7fe28, 84 },
    { 0xc2d7fe28, 93 },
    { 0x65b7fe2a, 95 },
    { 0x3597fe2a, 43 },
    { 0x69a7fe28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b7fe2f, 98 },
    { 0x5807fe30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7fe38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa667fe3d, 55 },
    { 0, 0 },
    { 0x2607fe3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e7fe42, 77 },
    { 0x89d7fe42, 33 },
    { 0x3167fe42, 43 },
    { 0x3f37fe42, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e27fe4b, 66 },
    { 0x70a7fe4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa7fe50, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9897fe54, 61 },
    { 0x89a7fe54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x657fe58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f77fe5b, 61 },
    { 0x1f87fe5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7fe60, 55 },
    { 0x4ba7fe60, 75 },
    { 0x25b7fe61, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d77fe66, 23 },
    { 0xbc27fe67, 93 },
    { 0x89d7fe66, 34 },
    { 0x5e77fe69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527fe72, 90 },
    { 0x90a7fe72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8877fe76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d7fe7d, 34 },
    { 0x957fe7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7fe81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fe85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c7fe8f, 76 },
    { 0xa887fe90, 90 },
    { 0, 0 },
    { 0x91d7fe92, 33 },
    { 0x3577fe93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77fe9a, 76 },
    { 0xd397fe9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827fe9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd97fea7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b17feac, 17 },
    { 0, 0 },
    { 0xa0d7feae, 78 },
    { 0x84d7feaf, 88 },
    { 0x4e47feb0, 98 },
    { 0x1b07feb1, 90 },
    { 0x3907feae, 33 },
    { 0, 0 },
    { 0x24f7feb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc727febe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7fec2, 90 },
    { 0xffc7fec3, 33 },
    { 0, 0 },
    { 0x517fec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a7fec8, 61 },
    { 0xe2f7fec9, 96 },
    { 0x8097feca, 23 },
    { 0, 0 },
    { 0xd6f7fecc, 76 },
    { 0xd4a7fecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c7fed0, 43 },
    { 0, 0 },
    { 0xe617fed2, 61 },
    { 0, 0 },
    { 0xfc57fed4, 76 },
    { 0x1457fed5, 77 },
    { 0x3307fed6, 33 },
    { 0x2e37fed6, 61 },
    { 0xa4b7fed8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437fedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7fee3, 55 },
    { 0x6267fee4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd777fee7, 95 },
    { 0, 0 },
    { 0xafa7fee9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7fef0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77fef4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a7fef9, 75 },
    { 0xe3f7fefa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c17ff00, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c7ff03, 100 },
    { 0, 0 },
    { 0x8ca7ff05, 98 },
    { 0xb447ff06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d07ff0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797ff0e, 23 },
    { 0, 0 },
    { 0xf207ff10, 34 },
    { 0x6267ff11, 69 },
    { 0x23d7ff10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5997ff1f, 23 },
    { 0x9897ff1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7ff23, 98 },
    { 0x7957ff24, 81 },
    { 0xbe27ff25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa7ff28, 76 },
    { 0x4747ff29, 88 },
    { 0xa5b7ff29, 61 },
    { 0xa667ff2b, 91 },
    { 0x39e7ff2c, 95 },
    { 0, 0 },
    { 0xf5b7ff2e, 76 },
    { 0, 0 },
    { 0x5e7ff30, 90 },
    { 0, 0 },
    { 0x91a7ff32, 90 },
    { 0x157ff33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77ff37, 95 },
    { 0x7b97ff37, 93 },
    { 0, 0 },
    { 0x6a47ff3a, 61 },
    { 0x8f67ff3b, 43 },
    { 0xf0c7ff3a, 61 },
    { 0xae57ff3d, 23 },
    { 0x31f7ff3d, 76 },
    { 0x18f7ff3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7ff47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37ff4b, 43 },
    { 0x72c7ff4c, 98 },
    { 0, 0 },
    { 0xf827ff4e, 98 },
    { 0xf797ff4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27ff53, 43 },
    { 0xb207ff54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277ff58, 99 },
    { 0xb97ff59, 34 },
    { 0, 0 },
    { 0xe257ff5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3317ff60, 43 },
    { 0x91e7ff61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7ff65, 23 },
    { 0, 0 },
    { 0x14b7ff67, 43 },
    { 0xa037ff68, 76 },
    { 0, 0 },
    { 0x93c7ff6a, 33 },
    { 0xe347ff6b, 33 },
    { 0x5587ff6c, 33 },
    { 0, 0 },
    { 0x2957ff6e, 55 },
    { 0xd307ff6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c7ff73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c7ff76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa217ff7f, 33 },
    { 0xf127ff7f, 95 },
    { 0, 0 },
    { 0x1b7ff82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea27ff87, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787ff90, 76 },
    { 0x47ff91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d47ff98, 34 },
    { 0, 0 },
    { 0xdfc7ff9a, 100 },
    { 0xec77ff9b, 40 },
    { 0, 0 },
    { 0x59e7ff9d, 33 },
    { 0x1187ff9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ffa1, 55 },
    { 0xef37ffa2, 84 },
    { 0, 0 },
    { 0x1a07ffa4, 80 },
    { 0x4487ffa5, 76 },
    { 0xebc7ffa6, 61 },
    { 0xd8c7ffa6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ffaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f7ffad, 17 },
    { 0x6377ffae, 33 },
    { 0xa987ffaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8297ffb4, 43 },
    { 0x8157ffb4, 98 },
    { 0, 0 },
    { 0xcf67ffb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9577ffbb, 76 },
    { 0xa797ffbc, 95 },
    { 0xb097ffbb, 61 },
    { 0x27e7ffbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047ffc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47ffc6, 23 },
    { 0, 0 },
    { 0xeab7ffc8, 95 },
    { 0x1417ffc9, 33 },
    { 0xeae7ffc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c7ffd2, 66 },
    { 0, 0 },
    { 0x7447ffd4, 46 },
    { 0x3587ffd5, 98 },
    { 0x91e7ffd4, 88 },
    { 0xb937ffd7, 76 },
    { 0xb707ffd5, 85 },
    { 0xb367ffd8, 23 },
    { 0xdcf7ffda, 34 },
    { 0xda47ffdb, 43 },
    { 0x7e87ffdc, 90 },
    { 0xb977ffdd, 76 },
    { 0x9107ffdb, 33 },
    { 0x6fa7ffdb, 90 },
    { 0x5387ffe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7ffe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7ffe8, 90 },
    { 0, 0 },
    { 0xdaf7ffea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d97ffef, 81 },
    { 0, 0 },
    { 0xd1e7fff1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f7fffd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e80000, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d80008, 61 },
    { 0x1a680008, 98 },
    { 0xd8b80009, 76 },
    { 0, 0 },
    { 0x7378000c, 90 },
    { 0, 0 },
    { 0xbdf8000e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b80012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb480015, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd8001a, 85 },
    { 0x9788001b, 90 },
    { 0, 0 },
    { 0xa8e8001d, 76 },
    { 0x3d28001e, 33 },
    { 0, 0 },
    { 0xae680020, 55 },
    { 0x82980021, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d80024, 98 },
    { 0x13980025, 61 },
    { 0x61980025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb580029, 100 },
    { 0xfb8002a, 23 },
    { 0xa168002b, 95 },
    { 0x2fc8002a, 46 },
    { 0, 0 },
    { 0x4b98002e, 61 },
    { 0x468002f, 43 },
    { 0, 0 },
    { 0xbf680031, 36 },
    { 0, 0 },
    { 0x68880033, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea80037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab8003b, 23 },
    { 0x6068003c, 76 },
    { 0xfb08003d, 61 },
    { 0x1cb8003d, 90 },
    { 0, 0 },
    { 0xf0880040, 23 },
    { 0xb4b80041, 33 },
    { 0xe680042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30280047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888004e, 61 },
    { 0, 0 },
    { 0xf4080050, 46 },
    { 0xee980051, 69 },
    { 0, 0 },
    { 0xc2480053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede80058, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888005f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2580064, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3780067, 34 },
    { 0x90380068, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee8006b, 99 },
    { 0, 0 },
    { 0xb298006d, 75 },
    { 0, 0 },
    { 0xbf8006f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda80072, 30 },
    { 0x69f80073, 90 },
    { 0x3c080073, 52 },
    { 0x9c880074, 90 },
    { 0xc1d80076, 55 },
    { 0x8da80077, 76 },
    { 0xcaa80078, 55 },
    { 0x7ad80076, 23 },
    { 0xbf98007a, 61 },
    { 0x91f8007b, 17 },
    { 0, 0 },
    { 0x7c58007d, 77 },
    { 0xd378007d, 61 },
    { 0xea8007f, 91 },
    { 0x97980080, 76 },
    { 0x9de80081, 55 },
    { 0x46480081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b80088, 23 },
    { 0xc880089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e8008e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b680092, 95 },
    { 0, 0 },
    { 0xe9680094, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45800a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1800a4, 85 },
    { 0xc3800a4, 95 },
    { 0xec8800a4, 100 },
    { 0x113800a7, 90 },
    { 0xbaa800a7, 66 },
    { 0x673800a8, 23 },
    { 0xd0800a9, 78 },
    { 0xad3800a7, 95 },
    { 0x597800ac, 33 },
    { 0x243800a7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x934800b0, 67 },
    { 0, 0 },
    { 0xf7b800b2, 33 },
    { 0, 0 },
    { 0x911800b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a800ba, 99 },
    { 0xb94800bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f800bf, 98 },
    { 0x41a800c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd800c6, 53 },
    { 0xc9d800c7, 34 },
    { 0xcd4800c7, 93 },
    { 0x647800c9, 91 },
    { 0xdc1800ca, 88 },
    { 0x937800cb, 90 },
    { 0, 0 },
    { 0xadb800cd, 90 },
    { 0x282800ce, 95 },
    { 0x216800cd, 76 },
    { 0x38800d0, 90 },
    { 0, 0 },
    { 0x5b9800d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1800d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522800dc, 95 },
    { 0x261800dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b800e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb800e8, 33 },
    { 0x1d9800e9, 90 },
    { 0, 0 },
    { 0xe50800eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2800ef, 90 },
    { 0x8bc800f0, 33 },
    { 0x1a4800f1, 61 },
    { 0x36d800f1, 66 },
    { 0x49a800f1, 61 },
    { 0x2c6800f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11800fc, 77 },
    { 0x785800fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b180102, 61 },
    { 0xe1d80103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc280106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868010c, 95 },
    { 0, 0 },
    { 0xda18010e, 61 },
    { 0xc9d8010f, 78 },
    { 0, 0 },
    { 0xd0f80111, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94980118, 95 },
    { 0x18a80119, 91 },
    { 0, 0 },
    { 0xad58011b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e8011e, 93 },
    { 0x9338011e, 76 },
    { 0x4a780120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c80123, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f80129, 33 },
    { 0x6948012a, 78 },
    { 0x6fd8012a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf380130, 33 },
    { 0x85580131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c980135, 33 },
    { 0xa0980135, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b80140, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e80143, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe180146, 61 },
    { 0, 0 },
    { 0x71c80148, 33 },
    { 0x32180149, 67 },
    { 0x87680148, 61 },
    { 0x5580149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe208014e, 66 },
    { 0x1528014e, 90 },
    { 0x1fa8014e, 33 },
    { 0xfab80151, 88 },
    { 0x9ca80152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728015b, 43 },
    { 0x88b8015c, 33 },
    { 0, 0 },
    { 0xbb18015e, 76 },
    { 0x9a88015f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f80168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf588016b, 33 },
    { 0x2ac8016c, 61 },
    { 0, 0 },
    { 0x9e18016e, 65 },
    { 0xfe18016e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a80174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc208017d, 76 },
    { 0x7a98017e, 98 },
    { 0x5b18017e, 34 },
    { 0, 0 },
    { 0xa9f80181, 88 },
    { 0, 0 },
    { 0x43180183, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61180186, 33 },
    { 0, 0 },
    { 0xed880188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d28018b, 90 },
    { 0x8ea8018c, 95 },
    { 0, 0 },
    { 0x8258018e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87580191, 98 },
    { 0x6ba80192, 61 },
    { 0xbf180192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15801a1, 68 },
    { 0x1c4801a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0801a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461801ae, 55 },
    { 0, 0 },
    { 0x157801b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29801b3, 76 },
    { 0x5a7801b3, 23 },
    { 0xd05801b4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f801b9, 95 },
    { 0x926801b9, 95 },
    { 0, 0 },
    { 0x175801bc, 87 },
    { 0xb32801bd, 90 },
    { 0, 0 },
    { 0xceb801bf, 23 },
    { 0, 0 },
    { 0x12a801c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5801c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e801ce, 95 },
    { 0xce7801ce, 33 },
    { 0, 0 },
    { 0xb2f801d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc801d4, 66 },
    { 0, 0 },
    { 0x665801d6, 55 },
    { 0, 0 },
    { 0x87f801d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x385801db, 90 },
    { 0, 0 },
    { 0xf32801dd, 95 },
    { 0xf88801dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef801e5, 90 },
    { 0xab0801e6, 98 },
    { 0x8bc801e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711801f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d801f6, 36 },
    { 0xcf6801f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2801fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde180200, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7980206, 98 },
    { 0xb3480207, 52 },
    { 0, 0 },
    { 0xf2f80209, 61 },
    { 0x11c80209, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8020d, 23 },
    { 0x5458020e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d580212, 61 },
    { 0xef280212, 100 },
    { 0xe2d80214, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa880217, 68 },
    { 0x16280218, 91 },
    { 0x58580217, 55 },
    { 0xf528021a, 98 },
    { 0xa778021a, 93 },
    { 0, 0 },
    { 0x5078021d, 76 },
    { 0, 0 },
    { 0x8e18021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76580226, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a8022b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76780235, 34 },
    { 0x28280236, 90 },
    { 0x69080237, 95 },
    { 0x6d080238, 34 },
    { 0, 0 },
    { 0xf938023a, 76 },
    { 0x3bb8023b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af8023f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27080242, 99 },
    { 0x68880243, 95 },
    { 0xd880244, 33 },
    { 0, 0 },
    { 0xd0080246, 96 },
    { 0x4ab80246, 95 },
    { 0, 0 },
    { 0x52280249, 90 },
    { 0, 0 },
    { 0x8738024b, 98 },
    { 0, 0 },
    { 0xf1e8024d, 90 },
    { 0xb458024e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa580252, 98 },
    { 0, 0 },
    { 0xde080254, 32 },
    { 0xe7480255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8025b, 23 },
    { 0xfaa8025c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb580267, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8026f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8080272, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b38027d, 76 },
    { 0x138027d, 34 },
    { 0x5b68027d, 76 },
    { 0xefa8027f, 84 },
    { 0, 0 },
    { 0x67c80282, 88 },
    { 0x42480282, 90 },
    { 0x19a80282, 43 },
    { 0x54980285, 75 },
    { 0xa5880283, 23 },
    { 0xaee80287, 78 },
    { 0x67680286, 33 },
    { 0xf8880287, 90 },
    { 0x20e8028a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc78028e, 81 },
    { 0x9798028f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e80294, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5428029a, 87 },
    { 0xd8029b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e78029e, 91 },
    { 0x56f8029e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815802a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60802a6, 96 },
    { 0xe31802a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b802ae, 90 },
    { 0, 0 },
    { 0x71d802b0, 90 },
    { 0x50d802b0, 33 },
    { 0x9e9802b2, 61 },
    { 0x69802b3, 75 },
    { 0xbec802b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459802b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31802bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159802bf, 90 },
    { 0x40c802bf, 95 },
    { 0xe43802c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105802c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf802d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a802db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed802df, 90 },
    { 0xf4d802e0, 95 },
    { 0, 0 },
    { 0x9e1802e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154802eb, 90 },
    { 0x38a802ec, 81 },
    { 0x472802eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2802f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba802f4, 61 },
    { 0xf64802f5, 90 },
    { 0, 0 },
    { 0x23a802f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a802fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec80301, 52 },
    { 0, 0 },
    { 0xf9e80303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b58030b, 91 },
    { 0x2238030c, 95 },
    { 0xa6d8030b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb80313, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc80316, 76 },
    { 0xdfb80316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b8031a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c8031e, 95 },
    { 0x8d98031f, 33 },
    { 0, 0 },
    { 0xd3380321, 91 },
    { 0xab380322, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d580334, 98 },
    { 0x91880335, 61 },
    { 0x10c80336, 95 },
    { 0x28580337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf628033d, 33 },
    { 0, 0 },
    { 0xc0d8033f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc680345, 61 },
    { 0xf0280345, 90 },
    { 0x9ae80346, 33 },
    { 0x93780348, 61 },
    { 0x56e80349, 66 },
    { 0x5a28034a, 98 },
    { 0x8ee8034a, 55 },
    { 0x1758034c, 76 },
    { 0x2b38034d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece80352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8035c, 40 },
    { 0x9208035d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e280360, 93 },
    { 0xdfe80360, 34 },
    { 0x70480362, 76 },
    { 0x4dc80363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31980366, 84 },
    { 0, 0 },
    { 0xfd680368, 87 },
    { 0, 0 },
    { 0x3288036a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38036e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33880379, 43 },
    { 0, 0 },
    { 0x7ca8037b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb28037e, 55 },
    { 0xb28037f, 93 },
    { 0, 0 },
    { 0x3e480381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a80388, 33 },
    { 0, 0 },
    { 0x998038a, 33 },
    { 0, 0 },
    { 0xeea8038c, 84 },
    { 0xc8a8038d, 33 },
    { 0xae68038d, 55 },
    { 0, 0 },
    { 0x7f80390, 61 },
    { 0x9a780391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d80394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d80399, 72 },
    { 0xd8a8039a, 95 },
    { 0, 0 },
    { 0x9478039c, 33 },
    { 0xb278039d, 87 },
    { 0, 0 },
    { 0x1a28039f, 33 },
    { 0x493803a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a803a3, 46 },
    { 0, 0 },
    { 0x39c803a5, 80 },
    { 0xb01803a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a803ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66803af, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca803b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4803b9, 40 },
    { 0x5bb803ba, 44 },
    { 0xf9e803bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec803bf, 85 },
    { 0xb5c803bf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520803c4, 86 },
    { 0x227803c5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1803cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93803d0, 46 },
    { 0x101803d1, 67 },
    { 0, 0 },
    { 0x106803d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248803df, 90 },
    { 0xf6803e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9803e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7803e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7803f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c803f3, 95 },
    { 0x53f803f4, 98 },
    { 0xb90803f3, 23 },
    { 0x546803f6, 66 },
    { 0x70e803f7, 87 },
    { 0x88c803f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d803fd, 84 },
    { 0, 0 },
    { 0xd17803ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d580404, 52 },
    { 0x59c80405, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38040c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e80412, 95 },
    { 0xf8180413, 23 },
    { 0xba880414, 33 },
    { 0x7580415, 33 },
    { 0xbbc80416, 61 },
    { 0, 0 },
    { 0x4dc80418, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b8041c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de80420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a80423, 61 },
    { 0x6a280424, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f80427, 93 },
    { 0x6d280427, 38 },
    { 0, 0 },
    { 0x2008042a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb68042d, 55 },
    { 0, 0 },
    { 0xc168042f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f680432, 91 },
    { 0x36a80433, 55 },
    { 0x10b80434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a80437, 90 },
    { 0, 0 },
    { 0xd4d80439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08043d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab880443, 76 },
    { 0xf3e80444, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e280447, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08044b, 90 },
    { 0x8f78044c, 98 },
    { 0xfdb8044c, 43 },
    { 0xae48044e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41380451, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10480457, 33 },
    { 0x89e80458, 34 },
    { 0x1e280459, 10 },
    { 0x5aa80459, 61 },
    { 0x4398045b, 33 },
    { 0x2b98045c, 98 },
    { 0, 0 },
    { 0xeda8045e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b80463, 93 },
    { 0xe5180464, 99 },
    { 0xe5080465, 84 },
    { 0x71380466, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5280469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9958046d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f980473, 90 },
    { 0x1f880474, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c80478, 17 },
    { 0xd3980478, 33 },
    { 0xa9c80479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688047f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3880485, 98 },
    { 0xfe680485, 69 },
    { 0, 0 },
    { 0x7a580488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b80490, 76 },
    { 0xd6080490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85080494, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb8049b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d08049e, 90 },
    { 0x3208049f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9804a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03804aa, 95 },
    { 0x4ef804aa, 61 },
    { 0x972804ab, 61 },
    { 0x315804ad, 76 },
    { 0x458804ad, 61 },
    { 0x310804af, 90 },
    { 0x36e804b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16804b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73804b7, 23 },
    { 0x49a804b7, 34 },
    { 0xb35804b9, 23 },
    { 0xa34804b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e804bf, 23 },
    { 0x4cb804c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a804c5, 84 },
    { 0xb5a804c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e804ce, 76 },
    { 0x72e804cf, 43 },
    { 0xe61804cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89804d3, 33 },
    { 0x7a2804d4, 34 },
    { 0xcc7804d5, 65 },
    { 0, 0 },
    { 0x60c804d7, 83 },
    { 0x4a3804d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c804db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e804e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9804e9, 95 },
    { 0x8f8804e9, 55 },
    { 0x9c804eb, 23 },
    { 0, 0 },
    { 0x137804ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20804f2, 100 },
    { 0, 0 },
    { 0x3e7804f4, 43 },
    { 0xbc7804f4, 96 },
    { 0, 0 },
    { 0xee9804f7, 43 },
    { 0, 0 },
    { 0xfe9804f9, 33 },
    { 0, 0 },
    { 0x8ff804fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x809804fe, 68 },
    { 0, 0 },
    { 0x5f380500, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6980507, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2928050a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb418050d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x81480510, 78 },
    { 0x45280511, 95 },
    { 0x18b80512, 66 },
    { 0x42a80513, 46 },
    { 0x6f280512, 23 },
    { 0x92280512, 46 },
    { 0x4f80516, 61 },
    { 0xc6780516, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d8051e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8280521, 33 },
    { 0xe8e80522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1180529, 95 },
    { 0x1928052a, 33 },
    { 0, 0 },
    { 0xb368052c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd280534, 90 },
    { 0x20f80535, 98 },
    { 0xd6180536, 50 },
    { 0x4e480536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c8053b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc778053e, 61 },
    { 0xfe38053e, 38 },
    { 0xe9a80540, 98 },
    { 0xc880541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd80549, 34 },
    { 0xd118054a, 87 },
    { 0x3e48054b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8054e, 66 },
    { 0xeea8054f, 33 },
    { 0x388054f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef80553, 90 },
    { 0x9f180554, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1188055b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7980566, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b80569, 90 },
    { 0x5298056a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828056e, 23 },
    { 0xc598056f, 76 },
    { 0x8e080570, 33 },
    { 0x8f580570, 95 },
    { 0x4d280572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d280579, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a8057e, 93 },
    { 0, 0 },
    { 0x9b380580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad80583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f68058a, 90 },
    { 0xe488058a, 33 },
    { 0, 0 },
    { 0x50c8058d, 95 },
    { 0x92c8058e, 34 },
    { 0, 0 },
    { 0xbc180590, 95 },
    { 0, 0 },
    { 0xda080592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a80597, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f8059c, 98 },
    { 0xbde8059d, 95 },
    { 0x2708059d, 61 },
    { 0, 0 },
    { 0xff5805a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f805a9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25805ac, 90 },
    { 0x996805ad, 76 },
    { 0x6dc805ae, 98 },
    { 0x1c4805ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6805b3, 87 },
    { 0, 0 },
    { 0xa5e805b5, 90 },
    { 0xb84805b6, 90 },
    { 0, 0 },
    { 0xe4f805b8, 61 },
    { 0xac5805b9, 46 },
    { 0, 0 },
    { 0x2af805bb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0805c3, 77 },
    { 0, 0 },
    { 0xf91805c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198805ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703805ce, 33 },
    { 0, 0 },
    { 0x32a805d0, 23 },
    { 0xcad805d0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3805d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372805db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75805de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e805e5, 76 },
    { 0xf1c805e6, 99 },
    { 0, 0 },
    { 0x42c805e8, 34 },
    { 0xba805e9, 90 },
    { 0, 0 },
    { 0x910805eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a805f0, 77 },
    { 0xa00805f0, 23 },
    { 0x6ad805f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5805f9, 95 },
    { 0xf70805f9, 43 },
    { 0, 0 },
    { 0x610805fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78805ff, 88 },
    { 0xfc680600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab80603, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79580609, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c8060f, 98 },
    { 0x3468060f, 99 },
    { 0, 0 },
    { 0x43b80612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5380615, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d80619, 87 },
    { 0x5be8061a, 81 },
    { 0x68061b, 76 },
    { 0, 0 },
    { 0xbce8061d, 43 },
    { 0, 0 },
    { 0xc38061f, 55 },
    { 0, 0 },
    { 0x8580621, 95 },
    { 0xa3680621, 76 },
    { 0x46080622, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128062a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1238062e, 17 },
    { 0x62f8062f, 33 },
    { 0x1a280630, 33 },
    { 0xce380631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71080635, 23 },
    { 0x8c580636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa080639, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3748063c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af80640, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf080643, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8180646, 94 },
    { 0x6d380646, 33 },
    { 0x3bc80648, 90 },
    { 0xbcf80646, 90 },
    { 0, 0 },
    { 0x8658064b, 95 },
    { 0x6c48064c, 23 },
    { 0, 0 },
    { 0xb4a8064e, 83 },
    { 0, 0 },
    { 0x84f80650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5480657, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9058065a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8065e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80662, 33 },
    { 0, 0 },
    { 0xd0280664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8280669, 91 },
    { 0, 0 },
    { 0xc288066b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38066f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9180673, 33 },
    { 0x6fb80674, 33 },
    { 0x34380675, 65 },
    { 0, 0 },
    { 0x81380677, 99 },
    { 0xc6b80677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb8067c, 96 },
    { 0x1f08067d, 100 },
    { 0, 0 },
    { 0x2158067f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a80688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a8068c, 34 },
    { 0x8538068d, 61 },
    { 0x918068d, 90 },
    { 0xb538068f, 98 },
    { 0xab38068f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2580695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d80699, 90 },
    { 0x5cb8069a, 66 },
    { 0xa5c80699, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb58069e, 76 },
    { 0xb6a8069f, 99 },
    { 0x51806a0, 93 },
    { 0, 0 },
    { 0x5e1806a2, 88 },
    { 0xe76806a2, 40 },
    { 0xac8806a4, 33 },
    { 0x916806a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90806a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f806ab, 98 },
    { 0, 0 },
    { 0x8c806ad, 34 },
    { 0xe79806ae, 61 },
    { 0x35c806af, 23 },
    { 0, 0 },
    { 0x687806b1, 94 },
    { 0, 0 },
    { 0xda806b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0806bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b806c1, 95 },
    { 0xb37806c2, 90 },
    { 0x7db806c2, 93 },
    { 0, 0 },
    { 0xddb806c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x833806c8, 87 },
    { 0xcf9806c9, 33 },
    { 0xa5a806c9, 98 },
    { 0xf00806c9, 55 },
    { 0, 0 },
    { 0xbd6806cd, 99 },
    { 0xadf806ce, 98 },
    { 0x92806ce, 88 },
    { 0xeff806ce, 61 },
    { 0xd4806d1, 24 },
    { 0x597806d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507806d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x766806da, 98 },
    { 0, 0 },
    { 0x413806dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24806e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x51806e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda806ea, 76 },
    { 0x7f6806eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd806f0, 23 },
    { 0x471806f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1806f4, 67 },
    { 0, 0 },
    { 0xa78806f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655806fe, 85 },
    { 0x95806ff, 95 },
    { 0, 0 },
    { 0xd2880701, 23 },
    { 0xde580702, 61 },
    { 0, 0 },
    { 0xde980704, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x26080707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e80713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb980716, 91 },
    { 0xd5580717, 90 },
    { 0x96a80716, 17 },
    { 0xc7b80718, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd68071d, 52 },
    { 0x3268071e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51680721, 43 },
    { 0x67080722, 85 },
    { 0xe8080723, 99 },
    { 0xe3b80724, 43 },
    { 0, 0 },
    { 0xb6580726, 34 },
    { 0xc0b80727, 23 },
    { 0xee980727, 98 },
    { 0x4dd80726, 33 },
    { 0, 0 },
    { 0x98072b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97580730, 23 },
    { 0, 0 },
    { 0x93080732, 99 },
    { 0, 0 },
    { 0x72280734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2680739, 95 },
    { 0x13780739, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xed28073d, 36 },
    { 0x7688073e, 98 },
    { 0, 0 },
    { 0x78180740, 55 },
    { 0, 0 },
    { 0xd3880742, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7780746, 90 },
    { 0xf7180746, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08074c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8074f, 75 },
    { 0x4c80750, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1380757, 77 },
    { 0x5e780757, 90 },
    { 0x8d880757, 34 },
    { 0xd398075a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca80766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e80769, 87 },
    { 0, 0 },
    { 0x91a8076b, 98 },
    { 0x60d8076c, 80 },
    { 0x4858076b, 61 },
    { 0x5688076e, 77 },
    { 0x5b48076d, 95 },
    { 0x8da80770, 33 },
    { 0x68076f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b80776, 61 },
    { 0xd9680777, 95 },
    { 0x88280778, 33 },
    { 0, 0 },
    { 0x6cf8077a, 61 },
    { 0x3c68077b, 95 },
    { 0xeff8077c, 23 },
    { 0x3ff8077a, 84 },
    { 0, 0 },
    { 0x9ee8077f, 55 },
    { 0xad380780, 98 },
    { 0, 0 },
    { 0x1c380782, 33 },
    { 0x77d80782, 23 },
    { 0x16980784, 34 },
    { 0x8dc80785, 93 },
    { 0x1fd80786, 43 },
    { 0x57080784, 98 },
    { 0xfda80784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28078d, 90 },
    { 0xd108078e, 76 },
    { 0xa68078d, 33 },
    { 0x3b480790, 23 },
    { 0x94080791, 34 },
    { 0xddd80791, 95 },
    { 0xb3880791, 95 },
    { 0xf5c80790, 90 },
    { 0xb7e80791, 33 },
    { 0xb4880796, 80 },
    { 0xe8880797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be8079a, 46 },
    { 0, 0 },
    { 0x9508079c, 81 },
    { 0, 0 },
    { 0x2458079e, 95 },
    { 0xa8b8079f, 81 },
    { 0x3ee807a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7807a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595807aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3807b1, 99 },
    { 0xc08807b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936807b7, 99 },
    { 0xf6807b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77807be, 43 },
    { 0xad807bf, 67 },
    { 0xae1807c0, 88 },
    { 0xc68807c1, 87 },
    { 0, 0 },
    { 0x60c807c3, 61 },
    { 0x6e1807c4, 61 },
    { 0x291807c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be807c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9807d0, 90 },
    { 0x13f807d1, 95 },
    { 0, 0 },
    { 0x79c807d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97807d6, 90 },
    { 0x8b807d6, 90 },
    { 0x210807d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b807de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f807e1, 67 },
    { 0x361807e1, 98 },
    { 0x68a807e2, 93 },
    { 0x558807e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf807e8, 33 },
    { 0x65a807e8, 34 },
    { 0x9b807ea, 46 },
    { 0xdb5807ea, 34 },
    { 0x1c2807eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36807f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c807fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae807ff, 81 },
    { 0x79f807ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c80803, 33 },
    { 0xde080804, 55 },
    { 0x26780805, 98 },
    { 0x8c580803, 61 },
    { 0x44180804, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b68080d, 91 },
    { 0x2d98080d, 98 },
    { 0xe0c8080f, 98 },
    { 0x46e8080e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2880813, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53380818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18081f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c80822, 61 },
    { 0xd4380823, 33 },
    { 0x18e80824, 90 },
    { 0xb2d80825, 61 },
    { 0x5d680823, 33 },
    { 0xdd380827, 95 },
    { 0x89f80828, 95 },
    { 0x99380829, 55 },
    { 0, 0 },
    { 0xf4f8082b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x428082e, 61 },
    { 0x18e8082f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a80832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32980835, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478083b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080842, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce080845, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15280848, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d8084d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0780850, 61 },
    { 0, 0 },
    { 0xd5b80852, 33 },
    { 0x7b680853, 90 },
    { 0x2b580854, 95 },
    { 0x60d80852, 56 },
    { 0x8ef80856, 90 },
    { 0xabe80856, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa538085a, 34 },
    { 0x9238085b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7680861, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e80865, 64 },
    { 0x6a480866, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d8086a, 98 },
    { 0xfef8086b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c480874, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63580878, 55 },
    { 0, 0 },
    { 0x58a8087a, 33 },
    { 0xc128087b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2748087f, 66 },
    { 0xa1980880, 46 },
    { 0xaff8087f, 61 },
    { 0x5980881, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b80885, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc80888, 98 },
    { 0, 0 },
    { 0x7ff8088a, 33 },
    { 0x3ce8088b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7278088f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45980894, 85 },
    { 0x70680894, 61 },
    { 0, 0 },
    { 0xad780897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7468089d, 76 },
    { 0, 0 },
    { 0x78a8089f, 61 },
    { 0, 0 },
    { 0x2d6808a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0808a8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9808ac, 87 },
    { 0, 0 },
    { 0xfa5808ae, 76 },
    { 0xf43808af, 60 },
    { 0x1d9808ae, 34 },
    { 0x99c808b1, 66 },
    { 0, 0 },
    { 0x7b5808b3, 33 },
    { 0xd67808b4, 43 },
    { 0, 0 },
    { 0x146808b6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7808b9, 90 },
    { 0x6c4808ba, 98 },
    { 0, 0 },
    { 0x14808bc, 61 },
    { 0x6e808bc, 61 },
    { 0, 0 },
    { 0x7ea808bf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae808c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b808cb, 61 },
    { 0xeda808cb, 33 },
    { 0xf92808cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62808d2, 76 },
    { 0x2f7808d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e808d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d808e2, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37808eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a808f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0808f7, 76 },
    { 0xadf808f8, 61 },
    { 0x28c808f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53808fe, 98 },
    { 0x4bd808fe, 99 },
    { 0x744808fe, 61 },
    { 0xa8f80901, 65 },
    { 0x39480902, 76 },
    { 0x9480903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee680906, 52 },
    { 0, 0 },
    { 0x49980908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd198090c, 78 },
    { 0, 0 },
    { 0xbb88090e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3080912, 88 },
    { 0, 0 },
    { 0xce80914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58980917, 61 },
    { 0, 0 },
    { 0x91d80919, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e8091e, 95 },
    { 0x8b78091f, 87 },
    { 0, 0 },
    { 0x10880921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62980926, 33 },
    { 0xe5280927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf58092d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a80932, 33 },
    { 0xa480932, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1080937, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8093a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b38093f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd480943, 90 },
    { 0x21180944, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a180947, 55 },
    { 0xd4080947, 33 },
    { 0x74380949, 26 },
    { 0xd898094a, 76 },
    { 0x5bc8094b, 91 },
    { 0xeaa8094a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80956, 46 },
    { 0x7b580957, 98 },
    { 0x64080957, 23 },
    { 0xa9480958, 33 },
    { 0xecc80958, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8095f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6780962, 98 },
    { 0xe7e80962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea78096a, 76 },
    { 0x7a18096a, 78 },
    { 0xd618096c, 98 },
    { 0xb568096c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e980973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d80982, 23 },
    { 0, 0 },
    { 0x1ab80984, 43 },
    { 0x5e180985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88380989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a280991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b980994, 33 },
    { 0xbc180995, 61 },
    { 0x8c080995, 84 },
    { 0x34480995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4358099d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a809a3, 90 },
    { 0x184809a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda809a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f809ab, 66 },
    { 0x146809ac, 84 },
    { 0x7f5809ad, 69 },
    { 0xf5d809ae, 91 },
    { 0xab1809af, 23 },
    { 0xdbd809b0, 69 },
    { 0, 0 },
    { 0xfda809b2, 77 },
    { 0xb14809b3, 55 },
    { 0xade809b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0809ba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48809be, 33 },
    { 0x1d7809bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x794809c2, 78 },
    { 0, 0 },
    { 0xf94809c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75809ca, 93 },
    { 0x8a1809cb, 33 },
    { 0xbcc809ca, 40 },
    { 0x9a5809cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414809d9, 34 },
    { 0xea6809da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508809e5, 48 },
    { 0x2bc809e6, 98 },
    { 0, 0 },
    { 0x89e809e8, 52 },
    { 0xc52809e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5809ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c809f1, 38 },
    { 0x6d2809f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960809f8, 61 },
    { 0x128809f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1809fd, 33 },
    { 0x16e809fe, 61 },
    { 0x49e809ff, 55 },
    { 0xa33809ff, 93 },
    { 0xe2880a01, 55 },
    { 0xe8280a02, 33 },
    { 0x55980a01, 81 },
    { 0xae680a01, 100 },
    { 0xeb780a01, 61 },
    { 0xe5c80a01, 80 },
    { 0x3e580a07, 98 },
    { 0x74080a08, 76 },
    { 0xb6e80a09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e80a0c, 33 },
    { 0xcad80a0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf580a13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b980a18, 55 },
    { 0, 0 },
    { 0xfc780a1a, 95 },
    { 0xc2880a1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5680a1f, 33 },
    { 0x5b180a20, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x62480a23, 23 },
    { 0xca080a23, 98 },
    { 0x95180a23, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a80a34, 90 },
    { 0xe8f80a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee80a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe980a3d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b80a41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac80a4d, 33 },
    { 0xa2680a4e, 98 },
    { 0xffa80a4d, 34 },
    { 0x62f80a50, 48 },
    { 0x12e80a51, 98 },
    { 0x6ff80a52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3780a57, 61 },
    { 0x4cd80a57, 33 },
    { 0, 0 },
    { 0x35a80a5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee80a5d, 95 },
    { 0xc5c80a5e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4080a61, 33 },
    { 0x6a580a62, 77 },
    { 0x2b80a63, 66 },
    { 0, 0 },
    { 0x2dd80a65, 61 },
    { 0x8be80a65, 61 },
    { 0xc6980a67, 52 },
    { 0x6d380a65, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3580a6c, 33 },
    { 0x16780a6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19980a77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c80a7b, 90 },
    { 0xf5480a7b, 33 },
    { 0xd280a7b, 33 },
    { 0xd1c80a7e, 40 },
    { 0x62180a7f, 95 },
    { 0xbee80a7f, 66 },
    { 0xd5580a80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f80a85, 98 },
    { 0, 0 },
    { 0xe1280a87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d80a8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7480a91, 33 },
    { 0, 0 },
    { 0xbe880a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac80a97, 90 },
    { 0x21580a98, 34 },
    { 0, 0 },
    { 0xb2080a9a, 66 },
    { 0, 0 },
    { 0x9c780a9c, 90 },
    { 0x96780a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd80aa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b380aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af80aae, 90 },
    { 0, 0 },
    { 0x580ab0, 99 },
    { 0xce980ab1, 90 },
    { 0x67f80ab2, 61 },
    { 0xa4a80ab3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e80ab6, 61 },
    { 0xd480ab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c80abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed780ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15780ace, 61 },
    { 0x63380acf, 98 },
    { 0xbb980ad0, 90 },
    { 0x24980ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad280ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c80ada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5180adf, 23 },
    { 0, 0 },
    { 0x7e780ae1, 33 },
    { 0x2b680ae1, 33 },
    { 0, 0 },
    { 0x4480ae4, 61 },
    { 0x82680ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3380aea, 95 },
    { 0x5c080aea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1580aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f80af2, 61 },
    { 0xad80af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25180af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c80aff, 46 },
    { 0, 0 },
    { 0xf0d80b01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a80b0e, 33 },
    { 0x1e380b0f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x31980b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32480b16, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x35480b19, 55 },
    { 0, 0 },
    { 0xc9480b1b, 67 },
    { 0, 0 },
    { 0xf0580b1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7180b20, 33 },
    { 0, 0 },
    { 0x5c780b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f080b26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0680b2d, 61 },
    { 0x13e80b2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9780b31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2080b3a, 77 },
    { 0x9ca80b3b, 101 },
    { 0xbef80b3c, 91 },
    { 0xba280b3b, 48 },
    { 0x24b80b3b, 61 },
    { 0, 0 },
    { 0x2ed80b40, 23 },
    { 0x85880b41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde80b45, 95 },
    { 0x97180b46, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf80b49, 93 },
    { 0x34080b4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b680b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7480b52, 90 },
    { 0xde380b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33480b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71380b5c, 90 },
    { 0xd4880b5c, 61 },
    { 0, 0 },
    { 0xd4f80b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d80b64, 66 },
    { 0x7b580b65, 33 },
    { 0x11380b66, 100 },
    { 0x45580b67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83880b6f, 95 },
    { 0xda480b70, 43 },
    { 0x84680b70, 100 },
    { 0x80080b72, 95 },
    { 0x7b880b72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42980b77, 95 },
    { 0x30480b78, 76 },
    { 0xec580b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc480b80, 94 },
    { 0xf0e80b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb880b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d680b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0480b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae580b9b, 61 },
    { 0xc4c80b9b, 33 },
    { 0, 0 },
    { 0x78c80b9e, 55 },
    { 0x43b80b9f, 33 },
    { 0x87680b9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e80ba3, 94 },
    { 0x4ce80ba4, 80 },
    { 0xf4880ba5, 23 },
    { 0xe2a80ba5, 90 },
    { 0x99b80ba4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef80baf, 33 },
    { 0x6c580bb0, 90 },
    { 0, 0 },
    { 0xeb180bb2, 61 },
    { 0xc7a80bb3, 43 },
    { 0x89080bb4, 87 },
    { 0xd2180bb5, 100 },
    { 0xa2780bb3, 90 },
    { 0x66180bb7, 95 },
    { 0xa4680bb5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd680bbd, 23 },
    { 0xaa180bbe, 23 },
    { 0, 0 },
    { 0x28b80bc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5480bc4, 68 },
    { 0, 0 },
    { 0x3a480bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86180bcd, 23 },
    { 0, 0 },
    { 0x65080bcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18880bd5, 17 },
    { 0x3d980bd5, 66 },
    { 0x18180bd7, 95 },
    { 0x60d80bd6, 90 },
    { 0x9f880bd7, 100 },
    { 0x83880bda, 37 },
    { 0xdd780bda, 46 },
    { 0xb3780bdc, 81 },
    { 0xd3a80bdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f80be0, 67 },
    { 0, 0 },
    { 0xf9080be2, 76 },
    { 0x6de80be3, 90 },
    { 0, 0 },
    { 0xfab80be5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d80be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90280bed, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb280bf3, 95 },
    { 0xfe180bf3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e80bf9, 76 },
    { 0x81f80bfa, 76 },
    { 0x44c80bfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba80bfe, 61 },
    { 0xc280bff, 90 },
    { 0, 0 },
    { 0xe7e80c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb80c04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e80c07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380c0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd080c0e, 52 },
    { 0, 0 },
    { 0x6780c10, 95 },
    { 0, 0 },
    { 0x89b80c12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15280c1a, 38 },
    { 0x25380c1b, 90 },
    { 0x45780c1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf80c20, 23 },
    { 0xd8380c21, 61 },
    { 0x57680c20, 33 },
    { 0x87d80c23, 43 },
    { 0xcc380c23, 55 },
    { 0xa8980c25, 90 },
    { 0xb7e80c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4880c29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f80c2c, 61 },
    { 0, 0 },
    { 0x25c80c2e, 66 },
    { 0x42580c2e, 98 },
    { 0xa3780c30, 87 },
    { 0, 0 },
    { 0x79f80c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e80c37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6780c3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0380c3f, 23 },
    { 0, 0 },
    { 0xd6880c41, 76 },
    { 0xd780c41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac80c45, 34 },
    { 0x5db80c46, 90 },
    { 0x2d680c47, 33 },
    { 0x53680c45, 34 },
    { 0x7ed80c48, 76 },
    { 0xe3480c47, 33 },
    { 0, 0 },
    { 0xe3f80c4c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f80c53, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9180c56, 23 },
    { 0xaee80c57, 90 },
    { 0, 0 },
    { 0xcea80c59, 90 },
    { 0, 0 },
    { 0x6a880c5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f80c64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7780c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4580c6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56280c6e, 85 },
    { 0xd0980c6e, 90 },
    { 0, 0 },
    { 0x5fb80c71, 90 },
    { 0, 0 },
    { 0x1c980c73, 43 },
    { 0x80b80c74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0080c78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc80c7b, 95 },
    { 0, 0 },
    { 0xd1e80c7d, 95 },
    { 0x49d80c7e, 68 },
    { 0xaeb80c7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5180c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e880c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1580c8b, 76 },
    { 0, 0 },
    { 0x25780c8d, 43 },
    { 0, 0 },
    { 0xb2980c8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xada80c92, 82 },
    { 0x5b580c92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a80c98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf80ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb80ca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9380ca6, 23 },
    { 0xd1d80ca6, 61 },
    { 0xcf480ca7, 98 },
    { 0x86280ca9, 76 },
    { 0x58980caa, 76 },
    { 0xda280caa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4380cb0, 34 },
    { 0x36380cb1, 95 },
    { 0x76b80cb1, 97 },
    { 0xeab80cb3, 90 },
    { 0x84d80cb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e80cb8, 46 },
    { 0, 0 },
    { 0xfb280cba, 90 },
    { 0xc1180cbb, 68 },
    { 0xd6180cbc, 76 },
    { 0x2aa80cbc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13380cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6080cc5, 95 },
    { 0x9ca80cc6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d880ccd, 33 },
    { 0x12a80cce, 23 },
    { 0x62180ccf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3080cd2, 93 },
    { 0, 0 },
    { 0xd480cd4, 97 },
    { 0, 0 },
    { 0x2080cd6, 67 },
    { 0x5a680cd7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x57980cda, 61 },
    { 0xb7080cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d780ce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6680ce5, 77 },
    { 0x8bd80ce6, 98 },
    { 0xf8c80ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29280ced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7080cf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9780cf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d80cfa, 33 },
    { 0x94e80cfb, 33 },
    { 0xffe80cfb, 98 },
    { 0x28d80cfd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5580d02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b80d05, 99 },
    { 0, 0 },
    { 0x67f80d07, 65 },
    { 0x69780d07, 90 },
    { 0, 0 },
    { 0xb9580d0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41880d0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec80d12, 8 },
    { 0, 0 },
    { 0x30180d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db80d17, 95 },
    { 0xa780d18, 61 },
    { 0x69680d19, 33 },
    { 0xf1a80d1a, 67 },
    { 0xd6880d1b, 87 },
    { 0, 0 },
    { 0x5d80d1d, 66 },
    { 0x4a680d1d, 61 },
    { 0x6f380d1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe380d22, 95 },
    { 0x5b280d23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd980d26, 43 },
    { 0x91b80d26, 90 },
    { 0x1ae80d28, 98 },
    { 0x37b80d27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e580d2e, 76 },
    { 0x51080d2f, 34 },
    { 0, 0 },
    { 0x66880d31, 61 },
    { 0x1b980d31, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80d3a, 90 },
    { 0x68880d3b, 68 },
    { 0, 0 },
    { 0xee280d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e80d43, 33 },
    { 0xc7b80d44, 43 },
    { 0xac080d45, 52 },
    { 0, 0 },
    { 0x68e80d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49880d4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b80d53, 76 },
    { 0, 0 },
    { 0x33a80d55, 23 },
    { 0xa2180d55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9180d63, 90 },
    { 0, 0 },
    { 0xd1b80d65, 33 },
    { 0xd5a80d66, 90 },
    { 0xe9a80d67, 75 },
    { 0x5480d66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80d6b, 55 },
    { 0x13680d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68580d71, 76 },
    { 0, 0 },
    { 0xe0c80d73, 33 },
    { 0x81e80d73, 43 },
    { 0xa4b80d73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d780d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25480d7f, 43 },
    { 0x22c80d7f, 46 },
    { 0, 0 },
    { 0x6080d82, 94 },
    { 0xb2f80d83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9880d87, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60980d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3480d91, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6880d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34880d9c, 76 },
    { 0xbbd80d9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9780da0, 84 },
    { 0xe0f80da1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda380da5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83480da8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c980dab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380dae, 90 },
    { 0, 0 },
    { 0x82580db0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f80db3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc80db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21580dbb, 23 },
    { 0x4780dbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99780dc2, 56 },
    { 0, 0 },
    { 0x3c280dc4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9680dca, 76 },
    { 0x2d980dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff080dce, 34 },
    { 0xfec80dce, 100 },
    { 0xba480dd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2080dd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c080dd9, 61 },
    { 0, 0 },
    { 0x15c80ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39580ddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8780de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a80de7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2280dea, 91 },
    { 0x2e380deb, 23 },
    { 0x86980deb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f80df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f80df3, 77 },
    { 0x36080df4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c80dff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c80e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6780e07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b80e0b, 23 },
    { 0xb2e80e0c, 78 },
    { 0x3c680e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30280e11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b980e15, 98 },
    { 0, 0 },
    { 0x6db80e17, 46 },
    { 0x7cf80e17, 98 },
    { 0x5c080e17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4080e22, 98 },
    { 0xd4780e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc80e26, 55 },
    { 0, 0 },
    { 0xc2680e28, 99 },
    { 0, 0 },
    { 0x7cd80e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b880e2d, 98 },
    { 0xb4f80e2e, 22 },
    { 0xb8f80e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac80e32, 34 },
    { 0x5ca80e32, 95 },
    { 0x76680e32, 75 },
    { 0x44180e35, 61 },
    { 0x7fd80e36, 33 },
    { 0x89480e37, 34 },
    { 0x88780e38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec80e3b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e980e3e, 87 },
    { 0xe0a80e3f, 95 },
    { 0, 0 },
    { 0x5480e41, 23 },
    { 0x68d80e42, 95 },
    { 0xe9680e41, 23 },
    { 0x96480e42, 95 },
    { 0x91580e43, 87 },
    { 0, 0 },
    { 0xb1580e47, 98 },
    { 0x31e80e47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1480e4c, 87 },
    { 0x5e180e4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d80e55, 91 },
    { 0xaa980e56, 40 },
    { 0x7ab80e56, 33 },
    { 0, 0 },
    { 0x52c80e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d80e5c, 91 },
    { 0xc1b80e5d, 98 },
    { 0x42e80e5c, 33 },
    { 0, 0 },
    { 0x69680e60, 78 },
    { 0, 0 },
    { 0xfd580e62, 100 },
    { 0x65280e62, 61 },
    { 0x67e80e64, 93 },
    { 0x49b80e63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0280e6b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57580e6f, 76 },
    { 0, 0 },
    { 0x84880e71, 98 },
    { 0x63980e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e480e78, 91 },
    { 0, 0 },
    { 0x94c80e7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73580e80, 77 },
    { 0x14280e81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c580e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e80e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d580ea0, 90 },
    { 0x96280ea0, 78 },
    { 0x6e980ea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83180ea5, 91 },
    { 0x43580ea6, 76 },
    { 0xe7780ea5, 34 },
    { 0x6b980ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb80eae, 52 },
    { 0, 0 },
    { 0x86480eb0, 95 },
    { 0xfa880eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71280eb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c080eb7, 100 },
    { 0xa8980eb7, 77 },
    { 0xa2180eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52280ebd, 46 },
    { 0, 0 },
    { 0x10080ebf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db80ec5, 90 },
    { 0, 0 },
    { 0xee80ec7, 98 },
    { 0, 0 },
    { 0x6de80ec9, 52 },
    { 0xedf80eca, 33 },
    { 0xb8880eca, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef780ed2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4480ed5, 46 },
    { 0, 0 },
    { 0x25e80ed7, 99 },
    { 0x19880ed8, 33 },
    { 0xed680ed9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d80ede, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe380ee1, 66 },
    { 0xe6980ee2, 55 },
    { 0, 0 },
    { 0x47080ee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e780ee7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680eea, 98 },
    { 0, 0 },
    { 0x51280eec, 55 },
    { 0, 0 },
    { 0x98a80eee, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd80ef1, 33 },
    { 0x6db80ef2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81580ef9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51980f01, 98 },
    { 0x7c80f02, 95 },
    { 0xbca80f02, 87 },
    { 0, 0 },
    { 0xd9f80f05, 40 },
    { 0x4d180f06, 61 },
    { 0xcdd80f06, 90 },
    { 0, 0 },
    { 0xd6580f09, 17 },
    { 0x2480f0a, 90 },
    { 0xeeb80f0b, 100 },
    { 0x59680f0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab80f0f, 90 },
    { 0x6e980f0f, 36 },
    { 0x96880f11, 93 },
    { 0x96280f12, 46 },
    { 0, 0 },
    { 0xd1780f14, 61 },
    { 0x68080f15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a80f19, 23 },
    { 0xb7980f1a, 85 },
    { 0, 0 },
    { 0xbc880f1c, 33 },
    { 0x28080f1c, 23 },
    { 0, 0 },
    { 0xea580f1f, 76 },
    { 0x85f80f20, 23 },
    { 0x65f80f21, 43 },
    { 0, 0 },
    { 0x15b80f23, 76 },
    { 0xc7080f23, 33 },
    { 0x61580f25, 98 },
    { 0x81280f26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a280f29, 93 },
    { 0x3e180f2a, 91 },
    { 0x17b80f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c80f2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e80f33, 34 },
    { 0, 0 },
    { 0x1a580f35, 78 },
    { 0xa7880f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25880f3b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d80f41, 33 },
    { 0x97980f41, 90 },
    { 0xef080f43, 65 },
    { 0x72580f42, 76 },
    { 0xb4680f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c80f4c, 33 },
    { 0x34a80f4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c80f54, 90 },
    { 0xfbf80f54, 55 },
    { 0x91a80f54, 34 },
    { 0x21f80f57, 95 },
    { 0xb2180f58, 34 },
    { 0x69680f59, 95 },
    { 0x91080f58, 61 },
    { 0xc6c80f54, 95 },
    { 0xa8880f5c, 98 },
    { 0, 0 },
    { 0x5e280f5e, 33 },
    { 0xef180f5f, 61 },
    { 0, 0 },
    { 0x6f380f61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e80f64, 77 },
    { 0x2c480f65, 90 },
    { 0, 0 },
    { 0xf0480f67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c80f6c, 98 },
    { 0xc4980f6d, 98 },
    { 0x3b280f6e, 100 },
    { 0x25d80f6f, 33 },
    { 0xdce80f6f, 85 },
    { 0xcf880f71, 95 },
    { 0x7d980f71, 95 },
    { 0x4d280f70, 90 },
    { 0x7ff80f6f, 98 },
    { 0x80e80f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80f79, 52 },
    { 0, 0 },
    { 0xec480f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f180f81, 64 },
    { 0, 0 },
    { 0xbd480f83, 46 },
    { 0x78380f84, 66 },
    { 0, 0 },
    { 0xfcf80f86, 55 },
    { 0, 0 },
    { 0xfac80f88, 52 },
    { 0, 0 },
    { 0x41880f8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c80f8d, 68 },
    { 0x24380f8d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d80f92, 61 },
    { 0x12380f92, 90 },
    { 0x67b80f92, 33 },
    { 0x45280f95, 98 },
    { 0x3380f95, 91 },
    { 0xd3d80f95, 95 },
    { 0xaf780f98, 53 },
    { 0x5b180f98, 84 },
    { 0xa8a80f9a, 34 },
    { 0x85c80f96, 23 },
    { 0x8c780f98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a480fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0180fa6, 23 },
    { 0xd5480fa7, 9 },
    { 0xf380fa8, 99 },
    { 0x2d580fa8, 90 },
    { 0, 0 },
    { 0xe4880fab, 49 },
    { 0xbfb80fac, 14 },
    { 0xb7b80fad, 95 },
    { 0xac080fae, 46 },
    { 0x46480fae, 77 },
    { 0x6ad80fb0, 61 },
    { 0x77680fae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b80fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5880fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14980fbc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c80fc2, 55 },
    { 0x7ef80fc3, 95 },
    { 0x19480fc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f80fc7, 90 },
    { 0, 0 },
    { 0x3a480fc9, 88 },
    { 0xec080fc9, 34 },
    { 0x22c80fc9, 65 },
    { 0x85580fcc, 90 },
    { 0xb3780fcd, 90 },
    { 0x8c780fcd, 98 },
    { 0xa7380fcf, 61 },
    { 0x64380fcf, 95 },
    { 0xfd80fcf, 98 },
    { 0x2b80fd2, 23 },
    { 0x47e80fd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9980fd7, 98 },
    { 0, 0 },
    { 0xda980fd9, 43 },
    { 0xa2a80fda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9280fe2, 33 },
    { 0xdc780fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b980fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed80fea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec680fed, 61 },
    { 0, 0 },
    { 0x6d980fef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7080ff2, 98 },
    { 0xdd980ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb80ff8, 84 },
    { 0x8480ff8, 43 },
    { 0x2ad80ffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae81002, 66 },
    { 0, 0 },
    { 0xf7a81004, 33 },
    { 0x62d81005, 90 },
    { 0x1f381004, 33 },
    { 0, 0 },
    { 0x67781008, 33 },
    { 0, 0 },
    { 0xf158100a, 80 },
    { 0x7d78100b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb8100e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b81012, 90 },
    { 0, 0 },
    { 0x72c81014, 33 },
    { 0xe0081015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8448101a, 95 },
    { 0x8338101b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d8101f, 98 },
    { 0x7a281020, 95 },
    { 0, 0 },
    { 0xf1c81022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde081025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45681028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e8102b, 95 },
    { 0x4a8102b, 95 },
    { 0x7458102d, 91 },
    { 0, 0 },
    { 0xaaf8102f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef681035, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf181038, 95 },
    { 0x92b81039, 43 },
    { 0, 0 },
    { 0x9b98103b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8103e, 90 },
    { 0, 0 },
    { 0x66d81040, 40 },
    { 0x24b81041, 33 },
    { 0xe0781040, 33 },
    { 0xaa381043, 91 },
    { 0, 0 },
    { 0x8381045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8104a, 66 },
    { 0xacc8104a, 75 },
    { 0, 0 },
    { 0xdb08104d, 61 },
    { 0x9698104e, 66 },
    { 0x39e8104e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d281053, 98 },
    { 0, 0 },
    { 0x8b381055, 76 },
    { 0, 0 },
    { 0x8fc81057, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118105e, 40 },
    { 0, 0 },
    { 0x76681060, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f981064, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f81069, 43 },
    { 0x2d98106a, 90 },
    { 0x1658106a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b81070, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a481073, 90 },
    { 0, 0 },
    { 0xeab81075, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84381078, 61 },
    { 0xbd281078, 33 },
    { 0xdf08107a, 98 },
    { 0x95e8107a, 90 },
    { 0xe398107c, 23 },
    { 0xd518107d, 23 },
    { 0xee28107e, 61 },
    { 0, 0 },
    { 0xeab81080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef681084, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba81088, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac481095, 76 },
    { 0xc5281096, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8848109a, 52 },
    { 0x1508109b, 33 },
    { 0x8ec8109c, 34 },
    { 0x5d38109d, 95 },
    { 0x68e8109e, 33 },
    { 0x7be8109e, 33 },
    { 0x1bc8109e, 61 },
    { 0x6fe810a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d810a4, 90 },
    { 0x618810a5, 76 },
    { 0, 0 },
    { 0x486810a7, 23 },
    { 0xe14810a8, 76 },
    { 0xda810a9, 87 },
    { 0xd5e810a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895810b0, 61 },
    { 0xeb1810b1, 85 },
    { 0xd78810b0, 61 },
    { 0, 0 },
    { 0x12810b4, 55 },
    { 0, 0 },
    { 0xd89810b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985810bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280810c0, 100 },
    { 0, 0 },
    { 0x4c3810c2, 93 },
    { 0xb73810c3, 61 },
    { 0, 0 },
    { 0xd59810c5, 98 },
    { 0, 0 },
    { 0x105810c7, 34 },
    { 0, 0 },
    { 0x6b1810c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d810cc, 66 },
    { 0xe11810cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a810d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52810d6, 95 },
    { 0x53c810d7, 55 },
    { 0x189810d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63810dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199810e8, 87 },
    { 0xe1a810e9, 55 },
    { 0xddc810ea, 64 },
    { 0xfac810eb, 98 },
    { 0, 0 },
    { 0xe4a810ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x296810f0, 33 },
    { 0xc36810f0, 90 },
    { 0, 0 },
    { 0xce0810f3, 23 },
    { 0x1f2810f4, 76 },
    { 0x71e810f4, 78 },
    { 0, 0 },
    { 0xfe4810f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d810ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82881102, 81 },
    { 0x7c681103, 91 },
    { 0xae481102, 40 },
    { 0x74b81103, 90 },
    { 0xc4081102, 68 },
    { 0x8b581107, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a18110e, 61 },
    { 0xcea8110f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6781112, 90 },
    { 0x7c081113, 90 },
    { 0x42481114, 93 },
    { 0x44e81115, 95 },
    { 0x14781115, 90 },
    { 0x6f381112, 84 },
    { 0, 0 },
    { 0xdbd81119, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9181120, 77 },
    { 0x57b81121, 23 },
    { 0xd2581122, 55 },
    { 0xea581121, 87 },
    { 0, 0 },
    { 0x84b81125, 90 },
    { 0x8a81125, 2 },
    { 0, 0 },
    { 0x40a81128, 27 },
    { 0, 0 },
    { 0x55d8112a, 33 },
    { 0x7c88112b, 76 },
    { 0x8768112b, 95 },
    { 0x9e18112a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d981131, 33 },
    { 0x99081132, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x92281135, 95 },
    { 0x281136, 61 },
    { 0xedb81137, 69 },
    { 0xea181136, 23 },
    { 0x4381139, 43 },
    { 0xf928113a, 17 },
    { 0, 0 },
    { 0x3498113c, 55 },
    { 0xb498113d, 77 },
    { 0x4b98113e, 20 },
    { 0x7f48113d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca81142, 91 },
    { 0x3b81142, 95 },
    { 0, 0 },
    { 0xf7f81145, 99 },
    { 0xa2781146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8114c, 99 },
    { 0x3d48114d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e981152, 90 },
    { 0, 0 },
    { 0xaa581154, 91 },
    { 0, 0 },
    { 0x58c81156, 62 },
    { 0xf3981157, 99 },
    { 0, 0 },
    { 0x59f81159, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc278115f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13881163, 23 },
    { 0x9a181163, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54681168, 46 },
    { 0x58681169, 90 },
    { 0xc268116a, 95 },
    { 0xc6881168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c81170, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f81173, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb81177, 78 },
    { 0, 0 },
    { 0x7a781179, 90 },
    { 0, 0 },
    { 0x6938117b, 76 },
    { 0, 0 },
    { 0x6208117d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f81180, 76 },
    { 0x68081180, 98 },
    { 0xb6b81181, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3181187, 61 },
    { 0, 0 },
    { 0xaf181189, 95 },
    { 0x16e8118a, 81 },
    { 0xd188118b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac8118e, 90 },
    { 0x99c8118f, 90 },
    { 0x3381190, 33 },
    { 0x87d81191, 55 },
    { 0x2c981191, 98 },
    { 0, 0 },
    { 0x9c481194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa138119d, 81 },
    { 0, 0 },
    { 0x8a18119f, 23 },
    { 0x6e38119f, 95 },
    { 0, 0 },
    { 0xe64811a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x694811a5, 78 },
    { 0xfcb811a5, 98 },
    { 0, 0 },
    { 0xd6811a8, 95 },
    { 0x830811a9, 33 },
    { 0x966811a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d811af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c811b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf811b6, 17 },
    { 0xdc4811b7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7811bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b811be, 61 },
    { 0xe71811be, 61 },
    { 0xa86811be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b811c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9811cd, 84 },
    { 0x3a3811ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9811d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3811d4, 98 },
    { 0xcbb811d5, 87 },
    { 0, 0 },
    { 0xea5811d7, 98 },
    { 0x2ef811d8, 98 },
    { 0xc89811d7, 100 },
    { 0xfb7811d9, 98 },
    { 0x3a7811db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17811df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16811e2, 98 },
    { 0x320811e3, 90 },
    { 0, 0 },
    { 0xdca811e5, 90 },
    { 0x6c811e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e811e9, 33 },
    { 0x585811ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52811ef, 98 },
    { 0xb06811f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4811f3, 90 },
    { 0, 0 },
    { 0x737811f5, 95 },
    { 0x95811f6, 95 },
    { 0xa3a811f5, 76 },
    { 0xe7b811f8, 76 },
    { 0, 0 },
    { 0x194811fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573811fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c81202, 98 },
    { 0x60981202, 95 },
    { 0, 0 },
    { 0x23481205, 76 },
    { 0xd681206, 78 },
    { 0x44d81205, 76 },
    { 0, 0 },
    { 0x59081209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c8120c, 85 },
    { 0xe3b8120c, 95 },
    { 0x6268120e, 43 },
    { 0x8f18120f, 90 },
    { 0x2908120f, 76 },
    { 0xbbf8120f, 98 },
    { 0xb898120d, 46 },
    { 0x3f88120e, 33 },
    { 0xb588120e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc208121a, 85 },
    { 0x6348121b, 76 },
    { 0x7008121c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f581220, 23 },
    { 0x4bd81220, 84 },
    { 0x3b781222, 55 },
    { 0xaa81221, 23 },
    { 0x9a881224, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60981227, 90 },
    { 0x71981228, 55 },
    { 0, 0 },
    { 0x9c58122a, 34 },
    { 0, 0 },
    { 0x21b8122c, 33 },
    { 0xe7f8122c, 61 },
    { 0xf698122d, 85 },
    { 0x21f8122c, 90 },
    { 0, 0 },
    { 0x7e81231, 78 },
    { 0x6b281232, 55 },
    { 0xf5d81232, 90 },
    { 0, 0 },
    { 0x38f81235, 76 },
    { 0, 0 },
    { 0x7ed81237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a8123b, 91 },
    { 0x52b8123c, 90 },
    { 0xecd8123b, 36 },
    { 0, 0 },
    { 0x7ba8123f, 90 },
    { 0x78581240, 69 },
    { 0, 0 },
    { 0x1be81242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94781247, 23 },
    { 0x6f981247, 52 },
    { 0x19481249, 91 },
    { 0xa838124a, 94 },
    { 0, 0 },
    { 0x4968124c, 95 },
    { 0xbb8124d, 61 },
    { 0xf908124d, 33 },
    { 0x6d8124f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15881252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40581255, 98 },
    { 0, 0 },
    { 0x68f81257, 90 },
    { 0x2f381257, 90 },
    { 0xc8181258, 61 },
    { 0, 0 },
    { 0x23a8125b, 90 },
    { 0, 0 },
    { 0x7758125d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2281260, 40 },
    { 0x12981260, 95 },
    { 0x12c81262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad81267, 98 },
    { 0xa7e81267, 90 },
    { 0x52181269, 23 },
    { 0x2f98126a, 84 },
    { 0, 0 },
    { 0x3898126c, 75 },
    { 0xed58126d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e81270, 95 },
    { 0, 0 },
    { 0xf8d81272, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x48681275, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878127c, 33 },
    { 0, 0 },
    { 0xb08127e, 61 },
    { 0xb8b8127f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8181287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2768128b, 90 },
    { 0, 0 },
    { 0xd4b8128d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f81292, 98 },
    { 0x60c81293, 23 },
    { 0x8d781294, 90 },
    { 0x9f081295, 85 },
    { 0x89881293, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38129f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x955812a2, 65 },
    { 0x219812a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c812a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb812b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5812b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89812bb, 95 },
    { 0x87f812bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c812c2, 33 },
    { 0x967812c3, 33 },
    { 0, 0 },
    { 0x177812c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0812c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7812d4, 78 },
    { 0x6da812d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x152812d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da812db, 88 },
    { 0xc3f812dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c812df, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265812ea, 61 },
    { 0xcc3812eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9812ee, 43 },
    { 0x808812ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c812f5, 98 },
    { 0x4c0812f5, 55 },
    { 0x10812f7, 90 },
    { 0x899812f8, 66 },
    { 0x34c812f7, 33 },
    { 0x97f812f8, 91 },
    { 0xe1d812fa, 95 },
    { 0xfe812fc, 23 },
    { 0x5e9812fc, 90 },
    { 0xf92812f6, 87 },
    { 0x509812ff, 61 },
    { 0x37f81300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4881303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df81307, 98 },
    { 0xe3481307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0281312, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681319, 90 },
    { 0, 0 },
    { 0xfc68131b, 43 },
    { 0, 0 },
    { 0x5968131d, 34 },
    { 0, 0 },
    { 0xac8131f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e81322, 76 },
    { 0x73e81323, 90 },
    { 0x47e81322, 98 },
    { 0x87d81323, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb918132b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff981330, 90 },
    { 0xfca81331, 90 },
    { 0, 0 },
    { 0xb881333, 52 },
    { 0x41281334, 90 },
    { 0, 0 },
    { 0x77081336, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8133c, 46 },
    { 0x9ca8133d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e81345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb81348, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858134c, 17 },
    { 0x78134d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b81350, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc81357, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9198135a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f381361, 98 },
    { 0x7c681362, 55 },
    { 0x30881363, 55 },
    { 0x92b81362, 33 },
    { 0xb1a81365, 33 },
    { 0, 0 },
    { 0x71581367, 90 },
    { 0, 0 },
    { 0x43a81369, 90 },
    { 0, 0 },
    { 0x2f78136b, 67 },
    { 0x47c8136c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b81372, 61 },
    { 0x5281372, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83581379, 98 },
    { 0x95c81379, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed8137d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7881381, 98 },
    { 0x7fd81381, 61 },
    { 0xb7581382, 98 },
    { 0, 0 },
    { 0x28a81385, 95 },
    { 0x2a681386, 61 },
    { 0x65e81387, 67 },
    { 0xd2d81387, 98 },
    { 0x89281388, 34 },
    { 0xfe081387, 76 },
    { 0xb3e81385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a781393, 23 },
    { 0x66a81393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7728139a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e813a6, 61 },
    { 0, 0 },
    { 0x86c813a8, 61 },
    { 0x1bc813a9, 95 },
    { 0x7c7813aa, 34 },
    { 0x571813ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x424813ae, 61 },
    { 0x542813af, 55 },
    { 0xdc813af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e813b5, 75 },
    { 0, 0 },
    { 0x98f813b7, 98 },
    { 0x8d6813b8, 98 },
    { 0, 0 },
    { 0x21b813ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83813be, 42 },
    { 0xe35813be, 84 },
    { 0xb3b813be, 43 },
    { 0x4df813c1, 56 },
    { 0x185813c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3813cb, 78 },
    { 0x423813cc, 69 },
    { 0x79a813cc, 75 },
    { 0x87c813cb, 98 },
    { 0xed3813cf, 85 },
    { 0, 0 },
    { 0x7c8813d1, 90 },
    { 0x6c6813d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee813d5, 43 },
    { 0, 0 },
    { 0x697813d7, 84 },
    { 0, 0 },
    { 0x3d8813d9, 61 },
    { 0x9cd813da, 76 },
    { 0x653813da, 96 },
    { 0x292813dc, 95 },
    { 0, 0 },
    { 0xb58813de, 87 },
    { 0x820813df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f813e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b813ea, 52 },
    { 0x2c9813ea, 33 },
    { 0x328813ec, 23 },
    { 0x4c2813ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07813f2, 90 },
    { 0xe7a813f3, 68 },
    { 0x3d813f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52813f9, 33 },
    { 0xb61813f9, 33 },
    { 0x78f813fa, 40 },
    { 0xccf813fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6381404, 55 },
    { 0xe7c81405, 93 },
    { 0xb7081406, 94 },
    { 0x2a481406, 76 },
    { 0x37881405, 87 },
    { 0xe1c81409, 23 },
    { 0x1ba81405, 93 },
    { 0, 0 },
    { 0xd308140c, 37 },
    { 0x5148140d, 61 },
    { 0x4908140e, 52 },
    { 0x29c8140e, 76 },
    { 0xe9b8140e, 95 },
    { 0xc4a8140e, 23 },
    { 0, 0 },
    { 0x71281413, 33 },
    { 0, 0 },
    { 0xd8381415, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a881418, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b8141e, 76 },
    { 0x6348141f, 33 },
    { 0x2738141f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f81427, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788142c, 98 },
    { 0xe978142d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce81431, 95 },
    { 0x92481432, 95 },
    { 0xbd381433, 80 },
    { 0, 0 },
    { 0xe0881435, 23 },
    { 0x13581435, 23 },
    { 0x1fe81437, 46 },
    { 0xa7a81438, 46 },
    { 0, 0 },
    { 0x4f58143a, 55 },
    { 0x7648143b, 33 },
    { 0, 0 },
    { 0x41f8143d, 56 },
    { 0xdf48143e, 65 },
    { 0x98e8143f, 93 },
    { 0, 0 },
    { 0xe5381441, 61 },
    { 0x3a581441, 99 },
    { 0xb2981443, 33 },
    { 0, 0 },
    { 0xc2081445, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b8144a, 97 },
    { 0xd0f8144a, 84 },
    { 0, 0 },
    { 0x9ec8144d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e681451, 95 },
    { 0x17281452, 90 },
    { 0xfa181452, 61 },
    { 0x77a81454, 33 },
    { 0, 0 },
    { 0x3c381456, 34 },
    { 0x9ce81457, 33 },
    { 0, 0 },
    { 0x8e381459, 90 },
    { 0x4428145a, 76 },
    { 0xfb68145b, 66 },
    { 0x5028145c, 90 },
    { 0xeb68145d, 76 },
    { 0xb28145d, 95 },
    { 0xd188145e, 95 },
    { 0xab681460, 47 },
    { 0x2881461, 43 },
    { 0, 0 },
    { 0x2a381463, 87 },
    { 0x54e81464, 23 },
    { 0, 0 },
    { 0x8c781466, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd18146b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9518146f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69281473, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be81476, 17 },
    { 0, 0 },
    { 0xcdc81478, 97 },
    { 0xfcb81479, 90 },
    { 0x9bf8147a, 77 },
    { 0xa5a8147b, 75 },
    { 0, 0 },
    { 0xbb98147d, 99 },
    { 0x8cc8147e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd81484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8148b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a81490, 61 },
    { 0x5a681491, 33 },
    { 0x4681491, 33 },
    { 0x92781493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7481496, 76 },
    { 0xd5e81497, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5218149a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1814a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f814a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31814ad, 66 },
    { 0, 0 },
    { 0x2cc814af, 55 },
    { 0, 0 },
    { 0xa8f814b1, 56 },
    { 0, 0 },
    { 0x236814b3, 33 },
    { 0xb8c814b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392814b8, 76 },
    { 0x9e3814b9, 78 },
    { 0x814814b9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc814bf, 93 },
    { 0, 0 },
    { 0xcb7814c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df814c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88814ca, 61 },
    { 0, 0 },
    { 0xbf8814cc, 87 },
    { 0x2f3814cd, 85 },
    { 0xdd7814cc, 33 },
    { 0xec2814cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91814d3, 17 },
    { 0, 0 },
    { 0x23e814d5, 80 },
    { 0, 0 },
    { 0xa7e814d7, 66 },
    { 0xd99814d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b814df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01814e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16814ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0814ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8814f2, 72 },
    { 0x55a814f3, 34 },
    { 0x7c814f3, 77 },
    { 0x24d814f5, 34 },
    { 0, 0 },
    { 0x710814f7, 34 },
    { 0xc3814f8, 61 },
    { 0x1af814f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8281502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35681507, 23 },
    { 0, 0 },
    { 0x29681509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4081510, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec81519, 33 },
    { 0x17681519, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4081522, 85 },
    { 0x5c281523, 66 },
    { 0x4b781523, 61 },
    { 0xce281525, 73 },
    { 0, 0 },
    { 0x91f81527, 46 },
    { 0x75d81528, 61 },
    { 0x9581528, 91 },
    { 0xd5e8152a, 67 },
    { 0xe138152b, 46 },
    { 0, 0 },
    { 0xb798152d, 90 },
    { 0x2928152e, 78 },
    { 0xff68152d, 61 },
    { 0, 0 },
    { 0x1ec81531, 61 },
    { 0x8c181531, 95 },
    { 0xd5a81533, 87 },
    { 0, 0 },
    { 0x59181535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e181538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a8153f, 95 },
    { 0x9bf8153f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5281543, 61 },
    { 0xf3581544, 33 },
    { 0x56781545, 90 },
    { 0xc0f81544, 93 },
    { 0x12281544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff68154f, 98 },
    { 0x7381550, 76 },
    { 0, 0 },
    { 0xffb81552, 33 },
    { 0x31281553, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8155b, 90 },
    { 0x318155c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87381561, 66 },
    { 0x50281562, 52 },
    { 0, 0 },
    { 0x39481564, 33 },
    { 0, 0 },
    { 0x4a981566, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xded81569, 55 },
    { 0, 0 },
    { 0x33a8156b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f81571, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10381575, 23 },
    { 0x43281575, 87 },
    { 0x11281576, 77 },
    { 0xc481575, 95 },
    { 0x53b81579, 34 },
    { 0x34981579, 61 },
    { 0xcef81578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2448157e, 33 },
    { 0x9d8157e, 95 },
    { 0, 0 },
    { 0x92081581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a81585, 98 },
    { 0x84881586, 55 },
    { 0xb0c81585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d98158b, 34 },
    { 0xe558158b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c81595, 98 },
    { 0, 0 },
    { 0x75781597, 46 },
    { 0x50581597, 56 },
    { 0xd4d81599, 93 },
    { 0, 0 },
    { 0xeba8159b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x518159e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f815a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab815aa, 90 },
    { 0, 0 },
    { 0xd90815ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3815b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c815b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6815b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47815bd, 95 },
    { 0xf40815be, 23 },
    { 0, 0 },
    { 0x17b815c0, 23 },
    { 0xd34815c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168815c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500815ca, 75 },
    { 0, 0 },
    { 0x646815cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e815cf, 66 },
    { 0x5b3815cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302815da, 78 },
    { 0, 0 },
    { 0x685815dc, 90 },
    { 0xd16815dd, 23 },
    { 0x6c0815de, 75 },
    { 0x52a815dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58815e2, 90 },
    { 0, 0 },
    { 0x610815e4, 90 },
    { 0, 0 },
    { 0x14a815e6, 23 },
    { 0, 0 },
    { 0xf76815e8, 80 },
    { 0x9dc815e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad815ee, 95 },
    { 0, 0 },
    { 0xe10815f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71815f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2815f6, 33 },
    { 0x5ca815f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea815fa, 65 },
    { 0xbbd815fa, 61 },
    { 0, 0 },
    { 0xc34815fd, 94 },
    { 0, 0 },
    { 0x9fb815ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64981603, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1081607, 90 },
    { 0x8ab81608, 52 },
    { 0, 0 },
    { 0xce28160a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23681612, 81 },
    { 0x49181613, 98 },
    { 0, 0 },
    { 0x5d081615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb481619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd8161c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8381622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd181627, 76 },
    { 0x42681628, 33 },
    { 0x75e81629, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca88162d, 76 },
    { 0, 0 },
    { 0x4f38162f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e481632, 65 },
    { 0xb5981633, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x54881636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af8163e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb81643, 90 },
    { 0, 0 },
    { 0x59581645, 46 },
    { 0xeaf81646, 84 },
    { 0, 0 },
    { 0x53981648, 93 },
    { 0xc7981648, 98 },
    { 0, 0 },
    { 0x4098164b, 78 },
    { 0, 0 },
    { 0x9168164d, 98 },
    { 0x9d28164e, 61 },
    { 0x9018164d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11881655, 95 },
    { 0xb9981656, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab8165f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc781664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78166a, 96 },
    { 0, 0 },
    { 0xfba8166c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608166f, 98 },
    { 0, 0 },
    { 0xb3c81671, 33 },
    { 0x36481672, 99 },
    { 0, 0 },
    { 0x72081674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d81678, 46 },
    { 0, 0 },
    { 0x87a8167a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8167e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc181682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781687, 98 },
    { 0, 0 },
    { 0xb9581689, 23 },
    { 0x7ee8168a, 67 },
    { 0x2bb81689, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28881691, 55 },
    { 0x4281692, 76 },
    { 0, 0 },
    { 0x7ec81694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd881698, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f78169b, 98 },
    { 0xb338169c, 95 },
    { 0, 0 },
    { 0x3228169e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e816a2, 90 },
    { 0x98e816a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8816a7, 61 },
    { 0x4b3816a8, 90 },
    { 0x85c816a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01816ac, 61 },
    { 0x2a3816ad, 23 },
    { 0x109816ae, 23 },
    { 0xbc4816af, 55 },
    { 0x217816b0, 90 },
    { 0x5e6816b1, 43 },
    { 0xf4d816ac, 34 },
    { 0x718816ae, 75 },
    { 0xae1816ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e816ba, 55 },
    { 0x94e816bb, 90 },
    { 0x89d816ba, 34 },
    { 0x90816bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df816c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23816c7, 69 },
    { 0xe1a816c7, 90 },
    { 0, 0 },
    { 0x55d816ca, 23 },
    { 0, 0 },
    { 0xaf9816cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92816d0, 90 },
    { 0x8816d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x311816d4, 76 },
    { 0xb46816d5, 78 },
    { 0xc7d816d5, 43 },
    { 0, 0 },
    { 0x99d816d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09816e2, 76 },
    { 0x28d816e3, 91 },
    { 0, 0 },
    { 0x15a816e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191816f1, 61 },
    { 0x3fc816f2, 17 },
    { 0x919816f2, 33 },
    { 0, 0 },
    { 0x5bf816f5, 96 },
    { 0x665816f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c816ff, 61 },
    { 0, 0 },
    { 0x36c81701, 33 },
    { 0x17681702, 93 },
    { 0, 0 },
    { 0xfb981704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f81707, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa988170b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c81713, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe18171c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218171f, 61 },
    { 0, 0 },
    { 0x8b081721, 55 },
    { 0, 0 },
    { 0x881723, 90 },
    { 0x2a681723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72281727, 84 },
    { 0xe7681727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1488172b, 95 },
    { 0x10a8172c, 99 },
    { 0x9b28172b, 98 },
    { 0, 0 },
    { 0x9d38172f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3881732, 98 },
    { 0x93581732, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c581736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e681739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d8173c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b81747, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2288174d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81759, 65 },
    { 0xb6181759, 98 },
    { 0xc888175b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74681764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b68176c, 76 },
    { 0, 0 },
    { 0xfbe8176e, 52 },
    { 0x18c8176f, 61 },
    { 0xe148176e, 61 },
    { 0x6348176e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa8177b, 98 },
    { 0, 0 },
    { 0x6fc8177d, 49 },
    { 0xac8177e, 97 },
    { 0x4ce8177d, 90 },
    { 0x3e581780, 78 },
    { 0x6c881781, 95 },
    { 0x30781782, 55 },
    { 0x9aa81782, 98 },
    { 0, 0 },
    { 0x6bb81785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728178c, 95 },
    { 0, 0 },
    { 0x1fd8178e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa8179a, 95 },
    { 0xf188179a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235817a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728817aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x847817ad, 98 },
    { 0, 0 },
    { 0xccf817af, 93 },
    { 0, 0 },
    { 0xcc817b1, 23 },
    { 0xd1817b2, 23 },
    { 0x70817b3, 61 },
    { 0x9e4817b4, 98 },
    { 0x456817b5, 23 },
    { 0, 0 },
    { 0x3c0817b7, 76 },
    { 0x80a817b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a817bb, 95 },
    { 0, 0 },
    { 0x39817bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878817c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9817c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e817cc, 98 },
    { 0xbb9817cd, 98 },
    { 0xf3e817ce, 90 },
    { 0x562817cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a817d2, 61 },
    { 0x84e817d3, 99 },
    { 0x80c817d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb817da, 88 },
    { 0xf35817da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751817e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da817e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c817eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4817ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb81801, 85 },
    { 0xf3281802, 99 },
    { 0x84a81803, 100 },
    { 0xecb81804, 23 },
    { 0xf4881803, 90 },
    { 0xf0e81804, 90 },
    { 0xc681804, 33 },
    { 0xe7c81808, 34 },
    { 0xaf281809, 95 },
    { 0xc0781808, 23 },
    { 0x8e38180b, 78 },
    { 0xecb8180b, 67 },
    { 0x8518180c, 93 },
    { 0xd608180e, 40 },
    { 0x49d8180a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a81817, 76 },
    { 0xdbf81817, 90 },
    { 0x26181817, 90 },
    { 0xfed81817, 61 },
    { 0x3eb8181b, 90 },
    { 0x9d581818, 43 },
    { 0x3848181d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d281827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad98182a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb881834, 95 },
    { 0xcff81835, 78 },
    { 0x58381836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8258183a, 61 },
    { 0xd178183b, 61 },
    { 0xcb98183b, 52 },
    { 0xa08183a, 81 },
    { 0, 0 },
    { 0x4d38183f, 55 },
    { 0xbf681840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9781843, 93 },
    { 0, 0 },
    { 0x2f581845, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be8184c, 90 },
    { 0, 0 },
    { 0xa588184e, 67 },
    { 0x44c8184f, 99 },
    { 0x44c8184e, 84 },
    { 0x1a381851, 95 },
    { 0x6a881851, 23 },
    { 0x2381853, 33 },
    { 0, 0 },
    { 0x14d81855, 67 },
    { 0xd1d81856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb08185a, 91 },
    { 0, 0 },
    { 0xca58185c, 23 },
    { 0, 0 },
    { 0x5458185e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b981862, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a81865, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e81869, 23 },
    { 0xcc081869, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d08186d, 33 },
    { 0x3618186d, 65 },
    { 0, 0 },
    { 0x38781870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11581876, 98 },
    { 0xdad81877, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a8187a, 67 },
    { 0x48f8187b, 87 },
    { 0xb048187c, 23 },
    { 0, 0 },
    { 0x8f78187e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f81881, 34 },
    { 0, 0 },
    { 0x2b681883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55881886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb981889, 23 },
    { 0, 0 },
    { 0x9a68188b, 65 },
    { 0xedb8188c, 84 },
    { 0x35c8188d, 52 },
    { 0x6918188e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4681892, 61 },
    { 0x6d681892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5281896, 46 },
    { 0x37c81896, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a78189b, 95 },
    { 0x4aa8189c, 98 },
    { 0xbc38189d, 95 },
    { 0x6d18189e, 83 },
    { 0x5488189f, 95 },
    { 0x81b8189e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b818ab, 90 },
    { 0, 0 },
    { 0x355818ad, 33 },
    { 0xba5818ae, 98 },
    { 0x79d818ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e818b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8818bc, 33 },
    { 0x240818bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a818c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f818cb, 34 },
    { 0, 0 },
    { 0xf0f818cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df818d0, 93 },
    { 0x9b7818d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c818d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264818d8, 61 },
    { 0x5b9818d9, 93 },
    { 0xe0f818da, 33 },
    { 0x73f818db, 36 },
    { 0xb1f818dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c818e0, 33 },
    { 0, 0 },
    { 0xebb818e2, 23 },
    { 0x463818e3, 91 },
    { 0xced818e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646818e9, 66 },
    { 0xdd9818ea, 61 },
    { 0x7cd818ea, 77 },
    { 0x584818ec, 78 },
    { 0x681818ed, 17 },
    { 0xe2f818ec, 66 },
    { 0xeee818ef, 33 },
    { 0x6f8818f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3818f7, 23 },
    { 0, 0 },
    { 0xbe3818f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc281900, 90 },
    { 0, 0 },
    { 0x3bc81902, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13381905, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a81908, 99 },
    { 0x34181908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c18190c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb81911, 33 },
    { 0x5ec81912, 33 },
    { 0, 0 },
    { 0x34e81914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd81917, 91 },
    { 0, 0 },
    { 0x19181919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9508191d, 93 },
    { 0, 0 },
    { 0xf278191f, 95 },
    { 0, 0 },
    { 0xd7e81921, 67 },
    { 0x91481921, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90981926, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f81929, 95 },
    { 0, 0 },
    { 0xf398192b, 55 },
    { 0, 0 },
    { 0x9fc8192d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4481930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81381933, 46 },
    { 0x17c81934, 46 },
    { 0x6ea81933, 61 },
    { 0x13481936, 76 },
    { 0x92f81937, 77 },
    { 0x39181937, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c8193c, 61 },
    { 0xc338193d, 78 },
    { 0x3fc8193d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b781945, 33 },
    { 0x5bf81945, 23 },
    { 0xdbc81946, 98 },
    { 0, 0 },
    { 0xffd81949, 90 },
    { 0xcd88194a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded8194f, 76 },
    { 0xd381950, 61 },
    { 0, 0 },
    { 0xa6e81952, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1681957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8195a, 87 },
    { 0, 0 },
    { 0x66d8195c, 68 },
    { 0, 0 },
    { 0x698195e, 61 },
    { 0, 0 },
    { 0x24281960, 90 },
    { 0x1db81961, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb81964, 66 },
    { 0x66f81964, 61 },
    { 0x1c081965, 85 },
    { 0x98e81967, 99 },
    { 0x86181967, 95 },
    { 0x8d281969, 93 },
    { 0xe3181968, 33 },
    { 0x50b81966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc948196e, 34 },
    { 0xf118196f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36381974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a81978, 67 },
    { 0xdeb81979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e81982, 55 },
    { 0x47281983, 95 },
    { 0xb0881984, 55 },
    { 0xef881984, 34 },
    { 0xc1a81985, 67 },
    { 0, 0 },
    { 0xb0681988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e8198b, 99 },
    { 0xf748198c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0281997, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8199c, 77 },
    { 0x4438199d, 69 },
    { 0, 0 },
    { 0x2cd8199f, 78 },
    { 0x228819a0, 90 },
    { 0x168819a0, 84 },
    { 0x156819a2, 46 },
    { 0, 0 },
    { 0x296819a4, 33 },
    { 0xee9819a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b819ac, 61 },
    { 0xdf1819ad, 61 },
    { 0x108819ad, 88 },
    { 0x81b819af, 23 },
    { 0x6a0819b0, 84 },
    { 0x35f819af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e819b6, 93 },
    { 0xb55819b7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e819bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x878819bf, 23 },
    { 0xae8819c0, 47 },
    { 0xd0a819c1, 61 },
    { 0, 0 },
    { 0x61b819c3, 88 },
    { 0, 0 },
    { 0x2c7819c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d819c9, 87 },
    { 0x4fb819c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd819d2, 69 },
    { 0, 0 },
    { 0x998819d4, 75 },
    { 0, 0 },
    { 0x57f819d6, 71 },
    { 0xdaf819d6, 61 },
    { 0xd7b819d7, 33 },
    { 0xd9e819d8, 33 },
    { 0x398819da, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74819de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f819e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56819e5, 93 },
    { 0xf61819e5, 90 },
    { 0xd15819e7, 90 },
    { 0xf5c819e8, 23 },
    { 0x329819e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe819ec, 23 },
    { 0, 0 },
    { 0xb0f819ee, 76 },
    { 0, 0 },
    { 0xe78819f0, 95 },
    { 0xc22819f1, 34 },
    { 0, 0 },
    { 0x34a819f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d819f9, 98 },
    { 0x13e819fa, 68 },
    { 0xba3819fa, 95 },
    { 0x6c5819fb, 95 },
    { 0xece819fd, 90 },
    { 0xf74819fe, 23 },
    { 0xed7819ff, 33 },
    { 0xdd2819fc, 90 },
    { 0x6ad819fc, 100 },
    { 0x86b81a02, 34 },
    { 0x799819fe, 61 },
    { 0x13d819fb, 61 },
    { 0xdf581a05, 33 },
    { 0xdf881a05, 93 },
    { 0, 0 },
    { 0x7dd81a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3781a12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3681a15, 61 },
    { 0, 0 },
    { 0xa8981a17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc081a1f, 90 },
    { 0x53b81a20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd481a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481a29, 99 },
    { 0xd5381a2a, 55 },
    { 0x99481a2b, 90 },
    { 0, 0 },
    { 0x8b281a2d, 76 },
    { 0x65f81a2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47681a36, 78 },
    { 0x11881a36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7581a3c, 84 },
    { 0xe7881a3d, 61 },
    { 0x28781a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca81a44, 23 },
    { 0, 0 },
    { 0x15281a46, 95 },
    { 0xe4681a47, 90 },
    { 0, 0 },
    { 0x95281a49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd381a4c, 61 },
    { 0xab81a4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e81a53, 100 },
    { 0xd8081a54, 95 },
    { 0xa6081a55, 90 },
    { 0xf7381a54, 33 },
    { 0x55581a57, 33 },
    { 0xc3581a58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8881a5b, 98 },
    { 0, 0 },
    { 0x8cb81a5d, 68 },
    { 0x7cf81a5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb81a61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd81a64, 76 },
    { 0x45781a65, 95 },
    { 0xe1d81a66, 76 },
    { 0xaf81a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43081a6b, 90 },
    { 0, 0 },
    { 0x1cb81a6d, 34 },
    { 0x5681a6e, 33 },
    { 0x3d281a6e, 95 },
    { 0, 0 },
    { 0x9bc81a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7381a78, 96 },
    { 0xbc881a78, 76 },
    { 0xff481a78, 98 },
    { 0, 0 },
    { 0x2d881a7c, 33 },
    { 0xd2681a7d, 98 },
    { 0xf3f81a7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd481a81, 33 },
    { 0xc8581a82, 61 },
    { 0x4181a83, 87 },
    { 0x2e181a84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64781a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c81a94, 76 },
    { 0xbc081a95, 61 },
    { 0, 0 },
    { 0x54e81a97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181a9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda281aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4781aa3, 77 },
    { 0x37281aa4, 95 },
    { 0x23481aa3, 55 },
    { 0, 0 },
    { 0x6af81aa7, 33 },
    { 0xe8381aa7, 23 },
    { 0x3fe81aa9, 23 },
    { 0, 0 },
    { 0x52f81aab, 33 },
    { 0, 0 },
    { 0xfb781aad, 23 },
    { 0, 0 },
    { 0x1881aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86981ab3, 84 },
    { 0x2df81ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee81ab9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f781abc, 100 },
    { 0xc9281abd, 33 },
    { 0x1f281abe, 66 },
    { 0xdbe81abe, 90 },
    { 0x14f81abf, 91 },
    { 0x15081ac1, 98 },
    { 0x8e581abc, 90 },
    { 0x7de81ac3, 85 },
    { 0x70a81ac4, 61 },
    { 0xe4b81ac4, 95 },
    { 0, 0 },
    { 0x86c81ac7, 34 },
    { 0x34681ac8, 88 },
    { 0x9c481ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32881acc, 98 },
    { 0x5a081acd, 23 },
    { 0xf6881ace, 52 },
    { 0x16a81acf, 95 },
    { 0, 0 },
    { 0x37981ad1, 34 },
    { 0, 0 },
    { 0xdb081ad3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f81ada, 78 },
    { 0, 0 },
    { 0xb0981adc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8281adf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a081ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a81ae7, 61 },
    { 0, 0 },
    { 0x17481ae9, 90 },
    { 0x4fb81aea, 46 },
    { 0xb3a81aeb, 43 },
    { 0xef881aec, 82 },
    { 0x3d81aed, 55 },
    { 0x4ad81aed, 90 },
    { 0x3d881aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb81af3, 33 },
    { 0x3ac81af3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e681af7, 38 },
    { 0x7f581af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c81afc, 78 },
    { 0xd7881afd, 61 },
    { 0x77f81afe, 49 },
    { 0x81f81afe, 95 },
    { 0x9ce81b00, 88 },
    { 0xf1381afd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac081b06, 90 },
    { 0x7f81b07, 33 },
    { 0x5d681b08, 98 },
    { 0x7a881b08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be81b0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2881b11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87981b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65481b19, 95 },
    { 0x14f81b1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c81b26, 4 },
    { 0, 0 },
    { 0xaf681b28, 66 },
    { 0x65e81b28, 58 },
    { 0x58281b2a, 66 },
    { 0x75381b2b, 33 },
    { 0, 0 },
    { 0x3dc81b2d, 90 },
    { 0x65081b2d, 33 },
    { 0xbe481b2f, 93 },
    { 0, 0 },
    { 0x77a81b31, 33 },
    { 0x5db81b31, 76 },
    { 0x70a81b32, 43 },
    { 0, 0 },
    { 0x8c781b35, 34 },
    { 0, 0 },
    { 0xb5481b37, 61 },
    { 0xe9c81b38, 23 },
    { 0x9181b39, 51 },
    { 0xca681b39, 61 },
    { 0xa3381b39, 93 },
    { 0xc3281b3c, 90 },
    { 0xf8881b3d, 90 },
    { 0x7d581b39, 90 },
    { 0, 0 },
    { 0x54f81b40, 97 },
    { 0, 0 },
    { 0xa0a81b42, 77 },
    { 0x7bc81b43, 33 },
    { 0x2c981b43, 90 },
    { 0x1ab81b45, 66 },
    { 0, 0 },
    { 0x54e81b47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d81b4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b81b4f, 45 },
    { 0x36c81b50, 33 },
    { 0x17681b50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91681b56, 90 },
    { 0, 0 },
    { 0x60181b58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a81b5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e81b62, 33 },
    { 0, 0 },
    { 0x65d81b64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x96081b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8581b6a, 80 },
    { 0, 0 },
    { 0xce981b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c81b73, 92 },
    { 0x95181b74, 90 },
    { 0xd9c81b75, 95 },
    { 0xf3781b73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86381b79, 33 },
    { 0xdfa81b7a, 100 },
    { 0xd5e81b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53781b7e, 90 },
    { 0x97181b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2581b83, 33 },
    { 0xe6481b84, 90 },
    { 0x43b81b84, 61 },
    { 0xdb581b86, 93 },
    { 0xf8c81b85, 43 },
    { 0x49f81b85, 90 },
    { 0xa2081b85, 34 },
    { 0xe5e81b8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c81b8d, 78 },
    { 0, 0 },
    { 0x2b981b8f, 99 },
    { 0xf2781b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee81b93, 61 },
    { 0x6ee81b94, 95 },
    { 0xfe281b95, 76 },
    { 0x5a781b96, 98 },
    { 0x43c81b97, 98 },
    { 0xf8481b97, 55 },
    { 0, 0 },
    { 0x74f81b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf81b9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf81ba0, 75 },
    { 0x1b781ba0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb81baa, 61 },
    { 0, 0 },
    { 0x8f681bac, 76 },
    { 0, 0 },
    { 0x49e81bae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c81bb1, 34 },
    { 0x2a681bb2, 90 },
    { 0xf7281bb2, 99 },
    { 0, 0 },
    { 0x16b81bb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb81bb8, 90 },
    { 0, 0 },
    { 0x1d381bba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7981bbf, 33 },
    { 0x1f081bc0, 90 },
    { 0x51c81bbf, 95 },
    { 0, 0 },
    { 0x55281bc3, 76 },
    { 0x6ac81bc3, 88 },
    { 0x25381bc4, 61 },
    { 0x19b81bc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c81bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c81bcc, 23 },
    { 0, 0 },
    { 0xb4a81bce, 33 },
    { 0x7a281bcf, 93 },
    { 0, 0 },
    { 0x69281bd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e81bd4, 55 },
    { 0x44081bd4, 90 },
    { 0x60781bd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c81bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4081bde, 33 },
    { 0, 0 },
    { 0x7de81be0, 88 },
    { 0x3e281be0, 87 },
    { 0x35581be2, 55 },
    { 0x1c81be3, 67 },
    { 0, 0 },
    { 0x85881be5, 84 },
    { 0xcc481be6, 93 },
    { 0x1b181be6, 78 },
    { 0xcb081be6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb81beb, 78 },
    { 0xc5081bec, 98 },
    { 0xdd481bed, 61 },
    { 0x92781bee, 99 },
    { 0x9f81bef, 23 },
    { 0x44881bf0, 76 },
    { 0xc2881bef, 95 },
    { 0, 0 },
    { 0xf7581bf3, 90 },
    { 0xe2481bf3, 61 },
    { 0x79481bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc81bf8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92781c00, 33 },
    { 0x42681c01, 43 },
    { 0, 0 },
    { 0xa5081c03, 99 },
    { 0x99381c04, 33 },
    { 0x7d981c03, 23 },
    { 0, 0 },
    { 0x15881c07, 71 },
    { 0xb3681c08, 23 },
    { 0x84481c09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3381c0c, 46 },
    { 0, 0 },
    { 0x46c81c0e, 78 },
    { 0x4d81c0e, 84 },
    { 0x98081c0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c81c15, 61 },
    { 0, 0 },
    { 0xe0981c17, 84 },
    { 0, 0 },
    { 0x5ff81c19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4381c21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4381c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12781c28, 76 },
    { 0, 0 },
    { 0x3d881c2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8181c30, 95 },
    { 0, 0 },
    { 0x45781c32, 95 },
    { 0x9cd81c33, 90 },
    { 0xa2381c34, 34 },
    { 0x31981c35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63181c3a, 90 },
    { 0, 0 },
    { 0x60681c3c, 90 },
    { 0x81f81c3c, 40 },
    { 0, 0 },
    { 0x87781c3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e81c45, 90 },
    { 0, 0 },
    { 0x84481c47, 61 },
    { 0, 0 },
    { 0x2fe81c49, 33 },
    { 0, 0 },
    { 0xb2281c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0681c56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a81c5a, 90 },
    { 0xa4781c5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c481c61, 95 },
    { 0xbfb81c62, 85 },
    { 0x55c81c61, 55 },
    { 0x55e81c63, 33 },
    { 0xcdb81c65, 61 },
    { 0xd6181c64, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581c6c, 98 },
    { 0xc1481c6d, 95 },
    { 0xdb281c6e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac81c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74581c75, 61 },
    { 0xc0781c76, 98 },
    { 0xac581c77, 33 },
    { 0x7b681c78, 98 },
    { 0x5ba81c76, 95 },
    { 0xe1381c7a, 95 },
    { 0x9b681c7b, 61 },
    { 0, 0 },
    { 0x2ab81c7d, 90 },
    { 0, 0 },
    { 0x51181c7f, 90 },
    { 0x50e81c7f, 90 },
    { 0x7ae81c7f, 34 },
    { 0xe6481c80, 34 },
    { 0x83e81c80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc81c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f781c8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9881c8d, 33 },
    { 0, 0 },
    { 0xd3181c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb681c92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a81c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23281ca3, 95 },
    { 0x4f681ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a81cab, 33 },
    { 0x19481cac, 99 },
    { 0x52281cab, 33 },
    { 0xc1c81cad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a81cb2, 98 },
    { 0, 0 },
    { 0x97981cb4, 88 },
    { 0x1d281cb5, 76 },
    { 0, 0 },
    { 0xfb981cb7, 23 },
    { 0x47c81cb7, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde81cbd, 61 },
    { 0x74f81cbe, 76 },
    { 0x4eb81cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a81cc6, 90 },
    { 0xdc981cc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef81cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef381cd3, 61 },
    { 0xe6381cd3, 33 },
    { 0xf8f81cd4, 61 },
    { 0, 0 },
    { 0xe1a81cd7, 33 },
    { 0, 0 },
    { 0x46281cd9, 92 },
    { 0xb9181cd9, 23 },
    { 0, 0 },
    { 0xb4281cdc, 76 },
    { 0, 0 },
    { 0x3e081cde, 23 },
    { 0, 0 },
    { 0x90981ce0, 86 },
    { 0, 0 },
    { 0x7b81ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6781ce8, 80 },
    { 0x31d81ce9, 23 },
    { 0, 0 },
    { 0x3bd81ceb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64481cee, 33 },
    { 0, 0 },
    { 0x32a81cf0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481cf5, 55 },
    { 0, 0 },
    { 0xfb981cf7, 61 },
    { 0x32281cf7, 55 },
    { 0x9d81cf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb081cfc, 33 },
    { 0xd5c81cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b481d03, 43 },
    { 0, 0 },
    { 0xeb481d05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a81d08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb81d12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa181d17, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3081d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59381d23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca81d27, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36381d2e, 33 },
    { 0x74381d2f, 90 },
    { 0x1c181d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2581d39, 76 },
    { 0xad481d3a, 99 },
    { 0, 0 },
    { 0xacd81d3c, 33 },
    { 0, 0 },
    { 0x8b381d3e, 76 },
    { 0x42e81d3f, 76 },
    { 0x2d181d3f, 94 },
    { 0xbf581d3f, 84 },
    { 0x90981d42, 59 },
    { 0x92b81d43, 66 },
    { 0x56081d43, 90 },
    { 0, 0 },
    { 0x7b981d46, 88 },
    { 0xb4a81d46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a781d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83881d52, 76 },
    { 0xce181d53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c81d5b, 76 },
    { 0, 0 },
    { 0x77881d5d, 87 },
    { 0x71d81d5e, 43 },
    { 0x26981d5e, 90 },
    { 0, 0 },
    { 0x4b981d61, 95 },
    { 0x7d981d62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6781d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781d68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd681d6c, 98 },
    { 0, 0 },
    { 0xc5f81d6e, 98 },
    { 0xe5d81d6f, 43 },
    { 0xe0e81d6f, 34 },
    { 0xf1881d70, 55 },
    { 0x85d81d72, 64 },
    { 0x6d081d72, 52 },
    { 0x85c81d74, 91 },
    { 0, 0 },
    { 0x12881d76, 85 },
    { 0x4d681d77, 78 },
    { 0xee81d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb981d7b, 33 },
    { 0, 0 },
    { 0xd6e81d7d, 78 },
    { 0x7e181d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb381d81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed081d89, 95 },
    { 0x60d81d8a, 95 },
    { 0x29e81d8a, 34 },
    { 0x7d181d8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45581d8f, 90 },
    { 0xc2981d90, 76 },
    { 0x24c81d91, 90 },
    { 0x19081d8f, 23 },
    { 0x2d681d93, 95 },
    { 0x8c681d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f81d9c, 98 },
    { 0x99381d9d, 93 },
    { 0xc1c81d9e, 90 },
    { 0x62481d9f, 61 },
    { 0xb2c81d9f, 64 },
    { 0xd8f81da1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3181da6, 23 },
    { 0, 0 },
    { 0x2f481da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d81dab, 90 },
    { 0x7e181dac, 34 },
    { 0x87481dad, 83 },
    { 0x6c081dad, 88 },
    { 0, 0 },
    { 0xdd181db0, 23 },
    { 0, 0 },
    { 0xf181db2, 90 },
    { 0x31c81db2, 76 },
    { 0xbcf81db3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81db8, 98 },
    { 0, 0 },
    { 0x64a81dba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7381dbd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68281dc0, 61 },
    { 0x52781dc0, 53 },
    { 0x51881dc2, 61 },
    { 0xc1981dc3, 78 },
    { 0x22e81dc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef281dcc, 61 },
    { 0x26881dcd, 65 },
    { 0xe9081dcc, 90 },
    { 0x5a981dce, 61 },
    { 0, 0 },
    { 0xfca81dd1, 34 },
    { 0x16a81dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e81dd5, 33 },
    { 0x68081dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b181ddd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e81de0, 90 },
    { 0x91b81de1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90181de6, 23 },
    { 0, 0 },
    { 0x7d681de8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90481ded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d81df5, 23 },
    { 0x96381df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3881dfb, 100 },
    { 0x84681dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb81e02, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52681e09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9681e0e, 43 },
    { 0, 0 },
    { 0x80481e10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4781e16, 61 },
    { 0, 0 },
    { 0x3a381e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7581e1c, 52 },
    { 0x15f81e1c, 98 },
    { 0, 0 },
    { 0x3be81e1f, 91 },
    { 0x98781e20, 61 },
    { 0x9bd81e21, 33 },
    { 0x3e081e22, 61 },
    { 0, 0 },
    { 0x5e181e24, 76 },
    { 0, 0 },
    { 0xb4381e26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca81e29, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f81e2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89381e31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c081e3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4181e3e, 55 },
    { 0xdcf81e3f, 76 },
    { 0x39581e3f, 61 },
    { 0, 0 },
    { 0x29c81e42, 33 },
    { 0x29d81e43, 100 },
    { 0x5bd81e43, 23 },
    { 0x86d81e43, 66 },
    { 0x47581e46, 77 },
    { 0, 0 },
    { 0xa1881e48, 17 },
    { 0xb5081e48, 61 },
    { 0xc3881e48, 61 },
    { 0x27e81e4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e81e4e, 33 },
    { 0x23c81e4e, 33 },
    { 0, 0 },
    { 0xe781e51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x11881e54, 95 },
    { 0, 0 },
    { 0xc8981e56, 76 },
    { 0x8d681e56, 33 },
    { 0x33881e56, 84 },
    { 0xb2d81e56, 90 },
    { 0xae581e5a, 23 },
    { 0x6c881e59, 61 },
    { 0x6a381e5a, 34 },
    { 0x53781e5d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681e63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54981e6c, 61 },
    { 0xd3f81e6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45081e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db81e78, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3281e7c, 61 },
    { 0xca181e7d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d81e81, 46 },
    { 0, 0 },
    { 0x46581e83, 96 },
    { 0xada81e84, 98 },
    { 0x6bf81e85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7681e8c, 100 },
    { 0xc5a81e8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c81e90, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d981e94, 61 },
    { 0x69481e95, 90 },
    { 0, 0 },
    { 0xd7e81e97, 58 },
    { 0x8b081e98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb81e9c, 23 },
    { 0x1e981e9d, 90 },
    { 0xf2881e9e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c581ea2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be81ea6, 66 },
    { 0, 0 },
    { 0x8a381ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af81eac, 95 },
    { 0, 0 },
    { 0xd3781eae, 71 },
    { 0x1181eaf, 52 },
    { 0x87d81eaf, 93 },
    { 0x2db81eb0, 95 },
    { 0xd9681eb1, 95 },
    { 0xaaa81eb1, 66 },
    { 0x23381eb4, 90 },
    { 0xed81eb0, 33 },
    { 0x9ae81eb6, 90 },
    { 0x81881eb6, 91 },
    { 0x76e81eb8, 100 },
    { 0, 0 },
    { 0xb1a81eba, 90 },
    { 0x27c81eba, 55 },
    { 0x33c81eba, 43 },
    { 0, 0 },
    { 0x43d81ebe, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f081ec1, 19 },
    { 0, 0 },
    { 0xcc381ec3, 34 },
    { 0x12681ec3, 33 },
    { 0x9db81ec4, 66 },
    { 0xf2c81ec5, 90 },
    { 0x71981ec7, 90 },
    { 0x6d481ec8, 61 },
    { 0xe4981ec5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf981ecc, 61 },
    { 0, 0 },
    { 0x32c81ece, 55 },
    { 0xb2681ecf, 76 },
    { 0xf6881ed0, 77 },
    { 0, 0 },
    { 0x1d781ed2, 98 },
    { 0, 0 },
    { 0x2a081ed4, 48 },
    { 0x43981ed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381edb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a81ee5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d81ee9, 33 },
    { 0xd3481eea, 100 },
    { 0, 0 },
    { 0x80981eec, 90 },
    { 0, 0 },
    { 0x82e81eee, 95 },
    { 0, 0 },
    { 0x74681ef0, 100 },
    { 0xcac81ef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0081ef6, 46 },
    { 0x1a181ef7, 95 },
    { 0x15c81ef6, 95 },
    { 0x7f481ef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f81efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a81f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac81f06, 99 },
    { 0xf9b81f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c81f0b, 84 },
    { 0x3ba81f0c, 95 },
    { 0, 0 },
    { 0x55e81f0e, 98 },
    { 0, 0 },
    { 0xaa581f10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a81f14, 33 },
    { 0, 0 },
    { 0xbeb81f16, 76 },
    { 0x17981f16, 90 },
    { 0, 0 },
    { 0x32c81f19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e81f1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1181f23, 91 },
    { 0x37381f24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45381f29, 90 },
    { 0x20a81f2a, 61 },
    { 0x6c381f29, 34 },
    { 0x5ba81f2c, 95 },
    { 0, 0 },
    { 0xa3781f2e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc181f33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14481f37, 33 },
    { 0x88781f38, 100 },
    { 0xe9481f39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb081f3e, 99 },
    { 0x1be81f3f, 23 },
    { 0x6d281f40, 85 },
    { 0xf9c81f40, 67 },
    { 0x13d81f40, 33 },
    { 0x6fd81f41, 98 },
    { 0xca981f44, 90 },
    { 0, 0 },
    { 0xc4281f46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa281f49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f781f4c, 90 },
    { 0xe1f81f4c, 33 },
    { 0, 0 },
    { 0x58481f4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74681f53, 98 },
    { 0x89f81f54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66681f59, 55 },
    { 0x1ec81f59, 65 },
    { 0x3b081f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae981f5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc981f61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c81f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93581f6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54581f71, 87 },
    { 0xd781f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96781f7c, 67 },
    { 0, 0 },
    { 0x74081f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d81f87, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d81f8c, 55 },
    { 0, 0 },
    { 0x55481f8e, 81 },
    { 0, 0 },
    { 0xa6981f90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82881f94, 66 },
    { 0xe5181f95, 100 },
    { 0x1ed81f96, 33 },
    { 0xeda81f96, 76 },
    { 0x87f81f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a81f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f281fa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f081fa6, 95 },
    { 0x7ac81fa6, 34 },
    { 0x7981fa8, 77 },
    { 0x1bf81fa7, 98 },
    { 0xd1581fa9, 61 },
    { 0xac81fab, 33 },
    { 0x57b81fac, 43 },
    { 0xa7281fa7, 43 },
    { 0xb8b81fae, 98 },
    { 0, 0 },
    { 0xa0981fb0, 98 },
    { 0x86381fb0, 88 },
    { 0, 0 },
    { 0x7a581fb3, 78 },
    { 0xc7981fb3, 66 },
    { 0xfc581fb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89081fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde981fbd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87081fc2, 55 },
    { 0x8c481fc3, 90 },
    { 0x39481fc4, 23 },
    { 0xf6a81fc5, 65 },
    { 0x14981fc5, 33 },
    { 0x4b681fc7, 23 },
    { 0x71d81fc6, 33 },
    { 0xa0181fc9, 39 },
    { 0, 0 },
    { 0xad581fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35881fd7, 77 },
    { 0x38681fd8, 90 },
    { 0x78a81fd9, 85 },
    { 0x4e781fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be81fdf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac81fe2, 55 },
    { 0x45381fe3, 65 },
    { 0xde081fe4, 46 },
    { 0x88381fe5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28181fef, 33 },
    { 0, 0 },
    { 0xaf481ff1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f81ff5, 55 },
    { 0x3b281ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa081ffb, 76 },
    { 0x89681ffc, 46 },
    { 0x4fd81ffc, 75 },
    { 0x3b681ffd, 95 },
    { 0x90581fff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f82004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b8200a, 61 },
    { 0, 0 },
    { 0xdb28200c, 90 },
    { 0x318200d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac82011, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78482015, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x69082018, 95 },
    { 0x75382019, 61 },
    { 0x4ea82018, 95 },
    { 0xd38201a, 98 },
    { 0x7818201b, 95 },
    { 0, 0 },
    { 0x1c18201e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c82026, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60482029, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f82032, 23 },
    { 0x26b82032, 33 },
    { 0x61382034, 61 },
    { 0x5dd82032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed38203a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668203f, 23 },
    { 0x2ac82040, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf182044, 61 },
    { 0x24a82045, 85 },
    { 0x1ad82046, 55 },
    { 0x5ed82045, 33 },
    { 0x91482048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d18204b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c28205a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c8205d, 88 },
    { 0x7c08205d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61582062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c82066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe782069, 61 },
    { 0xf4e82069, 66 },
    { 0xceb82069, 61 },
    { 0x82b8206c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868206f, 90 },
    { 0x35f82070, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xde482073, 33 },
    { 0x4be82074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808207c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0682083, 46 },
    { 0x91b82084, 23 },
    { 0x38682083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac68208a, 93 },
    { 0x42b8208b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff82092, 55 },
    { 0x1a482093, 90 },
    { 0x6f782094, 100 },
    { 0x9a882095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec8209a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b8209d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d820a0, 93 },
    { 0xc31820a1, 33 },
    { 0x254820a2, 61 },
    { 0, 0 },
    { 0xc25820a4, 90 },
    { 0, 0 },
    { 0x848820a6, 68 },
    { 0, 0 },
    { 0x594820a8, 78 },
    { 0xe3820a9, 95 },
    { 0x2d5820a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e820ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6820b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45820b5, 38 },
    { 0x3ee820b6, 23 },
    { 0xab5820b5, 90 },
    { 0x579820b8, 98 },
    { 0, 0 },
    { 0x9d0820ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d820bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda820c9, 43 },
    { 0xeeb820ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x481820cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968820d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b820d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1820db, 26 },
    { 0x767820db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e820e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13820e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1820ed, 95 },
    { 0, 0 },
    { 0x6bc820ef, 98 },
    { 0x546820f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849820f4, 85 },
    { 0x579820f4, 34 },
    { 0x2c5820f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5820fa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d820fe, 93 },
    { 0x8c1820ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f082102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a82105, 95 },
    { 0, 0 },
    { 0x82582107, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48210c, 90 },
    { 0xb728210d, 33 },
    { 0, 0 },
    { 0x9728210f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80382113, 98 },
    { 0, 0 },
    { 0x4f382115, 33 },
    { 0, 0 },
    { 0xdb282117, 46 },
    { 0, 0 },
    { 0x17a82119, 97 },
    { 0x52882119, 66 },
    { 0, 0 },
    { 0x9ab8211c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d982122, 52 },
    { 0x9d282123, 61 },
    { 0x8ac82123, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f82129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d8212c, 90 },
    { 0xe0d8212c, 78 },
    { 0xe388212e, 98 },
    { 0, 0 },
    { 0x9a882130, 33 },
    { 0x82b82131, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b82134, 46 },
    { 0x63182134, 44 },
    { 0, 0 },
    { 0xd4982137, 61 },
    { 0x1fd82138, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a18213c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87682140, 99 },
    { 0xa0582140, 81 },
    { 0, 0 },
    { 0x2982143, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd82146, 95 },
    { 0, 0 },
    { 0x35382148, 88 },
    { 0, 0 },
    { 0x6228214a, 52 },
    { 0, 0 },
    { 0x6738214c, 23 },
    { 0x8b98214d, 90 },
    { 0xba08214e, 90 },
    { 0xf938214f, 23 },
    { 0xf468214d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb382153, 95 },
    { 0xc5a82154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f682157, 23 },
    { 0x54182158, 95 },
    { 0x93c82157, 33 },
    { 0x11382158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f68215d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a82163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8982169, 33 },
    { 0, 0 },
    { 0x3718216b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9768216e, 23 },
    { 0xe3b8216e, 93 },
    { 0x9b382170, 90 },
    { 0x50982171, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8482178, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d8217c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92782188, 61 },
    { 0x80e82189, 95 },
    { 0x4d08218a, 90 },
    { 0xada82188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe038218e, 34 },
    { 0x69a8218f, 51 },
    { 0, 0 },
    { 0x2fe82191, 61 },
    { 0x81582192, 87 },
    { 0xbfd82193, 55 },
    { 0x98f82193, 95 },
    { 0x5fe82195, 88 },
    { 0xe7d82191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7882199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408219d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41821a0, 55 },
    { 0x54e821a1, 78 },
    { 0x504821a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183821a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1821ab, 90 },
    { 0xd0f821ac, 61 },
    { 0, 0 },
    { 0x4af821ae, 59 },
    { 0xe4f821ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6821b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a821b7, 83 },
    { 0, 0 },
    { 0x427821b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40821bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465821c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0821c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd821d0, 61 },
    { 0xd93821d0, 87 },
    { 0, 0 },
    { 0x2aa821d3, 100 },
    { 0x27b821d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32821d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x981821da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3821de, 93 },
    { 0x95821df, 23 },
    { 0, 0 },
    { 0x42a821e1, 84 },
    { 0xdb1821e1, 81 },
    { 0xb19821e2, 90 },
    { 0x54d821e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8821e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7821ea, 100 },
    { 0x3d7821eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae821f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f821f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e82203, 90 },
    { 0x8a082204, 83 },
    { 0x35682203, 90 },
    { 0x5f182206, 23 },
    { 0x60982206, 93 },
    { 0xe8b82208, 93 },
    { 0x10c82209, 95 },
    { 0x35982203, 33 },
    { 0xb3582206, 37 },
    { 0x72c8220c, 33 },
    { 0, 0 },
    { 0x8718220e, 61 },
    { 0x3a8220f, 40 },
    { 0, 0 },
    { 0xa2882211, 100 },
    { 0, 0 },
    { 0xcec82213, 95 },
    { 0xf7a82214, 76 },
    { 0x44182215, 61 },
    { 0x85b82216, 23 },
    { 0xd2782216, 90 },
    { 0x85682214, 93 },
    { 0, 0 },
    { 0x6728221a, 68 },
    { 0x2678221a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c88221e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba82228, 61 },
    { 0, 0 },
    { 0x15a8222a, 61 },
    { 0x61b8222b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7768223e, 90 },
    { 0x70d8223e, 43 },
    { 0, 0 },
    { 0xb6a82241, 90 },
    { 0x17582242, 33 },
    { 0x52b82242, 75 },
    { 0xa3782241, 43 },
    { 0x49c82244, 98 },
    { 0, 0 },
    { 0x48182247, 87 },
    { 0, 0 },
    { 0xfff82249, 99 },
    { 0, 0 },
    { 0xd3a8224b, 98 },
    { 0xa668224c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7098224f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd82254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2482258, 76 },
    { 0x1582259, 98 },
    { 0, 0 },
    { 0x9b08225b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe8225e, 44 },
    { 0x9608225f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac82262, 90 },
    { 0x9ff82262, 83 },
    { 0x99382263, 34 },
    { 0, 0 },
    { 0x63e82266, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a82269, 99 },
    { 0x53b82269, 33 },
    { 0x7188226b, 34 },
    { 0x6088226c, 46 },
    { 0xaf48226b, 90 },
    { 0x3d78226c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69482276, 98 },
    { 0xceb82277, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608227c, 81 },
    { 0, 0 },
    { 0xc2f8227e, 90 },
    { 0xd0e8227e, 100 },
    { 0, 0 },
    { 0x7c382281, 98 },
    { 0, 0 },
    { 0xd4c82283, 23 },
    { 0xaf282283, 93 },
    { 0x39282285, 90 },
    { 0x20582286, 81 },
    { 0x19682286, 34 },
    { 0x16082288, 76 },
    { 0, 0 },
    { 0xa188228a, 43 },
    { 0x8188228b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee8228f, 55 },
    { 0xd758228f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b28229a, 76 },
    { 0, 0 },
    { 0xaa88229c, 37 },
    { 0xf368229d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c822a2, 23 },
    { 0, 0 },
    { 0x90e822a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd822a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658822ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72822b4, 90 },
    { 0xd86822b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9822b8, 95 },
    { 0x1e9822b9, 23 },
    { 0xa6822ba, 88 },
    { 0xe42822b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f822be, 90 },
    { 0xd6d822bf, 90 },
    { 0x153822c0, 33 },
    { 0, 0 },
    { 0x46a822c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de822c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801822ce, 99 },
    { 0, 0 },
    { 0xed9822d0, 100 },
    { 0xa0822d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3822d6, 43 },
    { 0x22d822d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635822db, 98 },
    { 0x689822db, 98 },
    { 0x466822dd, 90 },
    { 0, 0 },
    { 0x6e8822df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07822e5, 36 },
    { 0xed4822e6, 88 },
    { 0x89f822e7, 95 },
    { 0x719822e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c822ec, 87 },
    { 0xf21822ed, 90 },
    { 0xf98822ed, 23 },
    { 0xdf8822ef, 91 },
    { 0x14b822ef, 61 },
    { 0x386822ef, 61 },
    { 0x591822f2, 33 },
    { 0xd88822f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a822f9, 76 },
    { 0xa0822fa, 95 },
    { 0x846822fb, 99 },
    { 0xa0b822fc, 95 },
    { 0xb8f822fa, 75 },
    { 0, 0 },
    { 0x40822ff, 90 },
    { 0x5e282300, 78 },
    { 0x68382301, 75 },
    { 0x1f882302, 34 },
    { 0x62182300, 78 },
    { 0xc7182304, 33 },
    { 0xdce82304, 93 },
    { 0x4ce82301, 91 },
    { 0x25d82307, 88 },
    { 0x5de82307, 66 },
    { 0x5c382308, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af8230e, 33 },
    { 0xdb08230e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe82312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9382315, 95 },
    { 0xbae82316, 66 },
    { 0xfe282316, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818231d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15482320, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982326, 90 },
    { 0, 0 },
    { 0x33682328, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa708232d, 76 },
    { 0, 0 },
    { 0x8b38232f, 100 },
    { 0, 0 },
    { 0x46b82331, 95 },
    { 0x3dc82332, 98 },
    { 0, 0 },
    { 0xc7d82334, 33 },
    { 0, 0 },
    { 0x67382336, 76 },
    { 0, 0 },
    { 0x79f82338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438233e, 23 },
    { 0x46e8233e, 91 },
    { 0x35f8233f, 76 },
    { 0x8db8233e, 61 },
    { 0x8c082342, 33 },
    { 0xad82343, 55 },
    { 0x49782344, 95 },
    { 0xfc782345, 61 },
    { 0x5b382345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78234c, 23 },
    { 0x9a08234d, 34 },
    { 0, 0 },
    { 0x7bd8234f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66882353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8235b, 92 },
    { 0x6f68235b, 43 },
    { 0x6f88235d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe082360, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c82364, 43 },
    { 0x42282365, 90 },
    { 0x77882366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a8236c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8236f, 61 },
    { 0x1d282370, 95 },
    { 0x67f82371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66482375, 98 },
    { 0, 0 },
    { 0x40282377, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48237c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5482384, 33 },
    { 0x9de82384, 43 },
    { 0xe1e82386, 91 },
    { 0x5682387, 75 },
    { 0, 0 },
    { 0x87282389, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda8238f, 93 },
    { 0, 0 },
    { 0x90d82391, 55 },
    { 0x37882392, 90 },
    { 0x13c82392, 95 },
    { 0x94e82392, 95 },
    { 0x53782395, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c182399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf38239e, 66 },
    { 0, 0 },
    { 0xc8a823a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3823a8, 55 },
    { 0x25b823a8, 98 },
    { 0xe0d823a9, 78 },
    { 0, 0 },
    { 0x5eb823ac, 80 },
    { 0x9e6823ad, 84 },
    { 0x982823ae, 93 },
    { 0x50c823af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56823b7, 23 },
    { 0x8be823b8, 33 },
    { 0, 0 },
    { 0xd4b823ba, 61 },
    { 0xda5823bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746823bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b823c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1823c8, 33 },
    { 0, 0 },
    { 0x2ee823ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba823cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x333823d0, 61 },
    { 0x4a2823d1, 17 },
    { 0, 0 },
    { 0xd52823d3, 61 },
    { 0xa63823d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35823d9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4823df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168823e6, 90 },
    { 0xff2823e7, 98 },
    { 0x454823e8, 67 },
    { 0x9ac823e9, 77 },
    { 0x7b2823e8, 33 },
    { 0x857823ea, 76 },
    { 0, 0 },
    { 0xb28823ed, 100 },
    { 0, 0 },
    { 0xc49823ef, 84 },
    { 0, 0 },
    { 0x852823f1, 90 },
    { 0, 0 },
    { 0x140823f3, 23 },
    { 0, 0 },
    { 0x438823f5, 90 },
    { 0, 0 },
    { 0x8e8823f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b823fa, 90 },
    { 0, 0 },
    { 0x2c4823fc, 55 },
    { 0x475823fc, 98 },
    { 0xbd1823fe, 43 },
    { 0xfe5823ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0682403, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d8240a, 33 },
    { 0xa368240b, 52 },
    { 0x8e28240b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc68240f, 33 },
    { 0xcc582410, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638241a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588241d, 99 },
    { 0x24d8241e, 87 },
    { 0x7bc8241f, 66 },
    { 0x7d28241d, 33 },
    { 0x8ba82420, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x67382424, 99 },
    { 0x58382424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa82428, 46 },
    { 0x9cc82429, 90 },
    { 0xe0082429, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b8242f, 93 },
    { 0x13f82430, 33 },
    { 0x64082431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b82438, 90 },
    { 0x9ab82438, 55 },
    { 0xdbd82439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8198243e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b82442, 90 },
    { 0, 0 },
    { 0x18982444, 23 },
    { 0x2882444, 61 },
    { 0, 0 },
    { 0xad682447, 46 },
    { 0x99e82448, 76 },
    { 0, 0 },
    { 0x6018244a, 93 },
    { 0xc7d8244b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e582450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe382454, 23 },
    { 0, 0 },
    { 0x75482456, 61 },
    { 0, 0 },
    { 0x9ce82458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918245d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f82462, 90 },
    { 0, 0 },
    { 0x9182464, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b282467, 81 },
    { 0x8e282468, 43 },
    { 0, 0 },
    { 0xf328246a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8246d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6882470, 33 },
    { 0, 0 },
    { 0xeac82472, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4582479, 98 },
    { 0x38c8247a, 99 },
    { 0xd298247b, 98 },
    { 0, 0 },
    { 0x4d98247d, 95 },
    { 0x3018247e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c82485, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede8248c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee82497, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118249a, 72 },
    { 0x39c8249a, 33 },
    { 0x7a28249c, 53 },
    { 0x77b8249c, 90 },
    { 0xfa18249d, 33 },
    { 0xee78249e, 46 },
    { 0x6458249f, 38 },
    { 0, 0 },
    { 0x876824a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c824a6, 33 },
    { 0xb75824a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f824aa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2824ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b824b2, 76 },
    { 0xcbe824b2, 95 },
    { 0xf78824b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40824b9, 61 },
    { 0x54a824ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1824be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577824c3, 98 },
    { 0x1c0824c4, 76 },
    { 0x8bb824c5, 34 },
    { 0x44d824c6, 23 },
    { 0x6ed824c5, 90 },
    { 0, 0 },
    { 0x391824c9, 33 },
    { 0x2b6824c9, 55 },
    { 0xd1b824ca, 85 },
    { 0x796824ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x991824cf, 90 },
    { 0, 0 },
    { 0xf25824d1, 90 },
    { 0x1b3824d1, 23 },
    { 0, 0 },
    { 0x4e4824d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac824de, 61 },
    { 0xb1a824de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x912824e2, 61 },
    { 0, 0 },
    { 0x687824e4, 66 },
    { 0x14824e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd824e8, 76 },
    { 0x11e824e9, 94 },
    { 0xa96824ea, 90 },
    { 0x218824ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f824f1, 90 },
    { 0xd9b824f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5824f9, 55 },
    { 0xd83824fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x771824fd, 33 },
    { 0xc1a824fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a482502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e8250b, 95 },
    { 0x318250b, 90 },
    { 0x5e48250d, 76 },
    { 0xcf28250b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc82512, 61 },
    { 0xafc82512, 34 },
    { 0, 0 },
    { 0x12582515, 55 },
    { 0, 0 },
    { 0x7982517, 52 },
    { 0x9ab82518, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd328251b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe8251e, 33 },
    { 0, 0 },
    { 0x48b82520, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf682523, 61 },
    { 0x95582524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84882529, 88 },
    { 0x96982529, 90 },
    { 0x6658252b, 95 },
    { 0xe758252c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f82530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d82533, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c08253c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c82545, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8254b, 95 },
    { 0x2488254c, 100 },
    { 0, 0 },
    { 0xc338254e, 95 },
    { 0xfe38254f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c582552, 33 },
    { 0, 0 },
    { 0x21682554, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682557, 77 },
    { 0x3f582557, 33 },
    { 0, 0 },
    { 0x75c8255a, 76 },
    { 0x4ea8255b, 100 },
    { 0x1ed8255c, 95 },
    { 0xe238255d, 61 },
    { 0xdf98255c, 33 },
    { 0, 0 },
    { 0xa6a82560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d482565, 76 },
    { 0xe4482565, 98 },
    { 0x81982567, 33 },
    { 0x6e582567, 98 },
    { 0xb8d82569, 85 },
    { 0x1908256a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a38256d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2082570, 91 },
    { 0xe9e82570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40382576, 33 },
    { 0x3ba82576, 98 },
    { 0x68082578, 76 },
    { 0x6cc82579, 38 },
    { 0x8f48257a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e8257f, 34 },
    { 0xc1b82580, 52 },
    { 0xb3282581, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8258c, 33 },
    { 0xb9e8258c, 86 },
    { 0xacf8258e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4382593, 78 },
    { 0, 0 },
    { 0x62a82595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b482599, 33 },
    { 0xb0182599, 23 },
    { 0, 0 },
    { 0x1a98259c, 23 },
    { 0, 0 },
    { 0xfe68259e, 33 },
    { 0x1558259f, 93 },
    { 0x5628259e, 43 },
    { 0, 0 },
    { 0x4b6825a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02825aa, 75 },
    { 0x5aa825ab, 98 },
    { 0xe3f825ac, 61 },
    { 0x77d825ad, 77 },
    { 0, 0 },
    { 0x25825af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe825b2, 23 },
    { 0x1e6825b3, 43 },
    { 0x95c825b4, 90 },
    { 0x2e825b5, 95 },
    { 0xffe825b6, 61 },
    { 0xd0d825b4, 55 },
    { 0, 0 },
    { 0x805825b9, 76 },
    { 0, 0 },
    { 0x144825bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88825bf, 91 },
    { 0x8fd825bf, 91 },
    { 0x4f825c1, 81 },
    { 0, 0 },
    { 0xce5825c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698825c8, 23 },
    { 0, 0 },
    { 0xe43825ca, 55 },
    { 0x722825cb, 17 },
    { 0x885825cc, 98 },
    { 0, 0 },
    { 0x8c0825ce, 90 },
    { 0x603825ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4825d2, 61 },
    { 0, 0 },
    { 0x2ef825d4, 95 },
    { 0x5f7825d4, 33 },
    { 0xbd9825d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557825da, 98 },
    { 0x19d825da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x978825de, 76 },
    { 0x5fa825df, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd825e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d825ea, 38 },
    { 0xa38825eb, 84 },
    { 0x10e825ec, 98 },
    { 0x122825ed, 36 },
    { 0x204825ee, 90 },
    { 0xe17825ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd825f3, 64 },
    { 0x230825f4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4825f7, 33 },
    { 0, 0 },
    { 0x4ce825f9, 90 },
    { 0, 0 },
    { 0x56c825fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3825fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc82601, 90 },
    { 0, 0 },
    { 0xba282603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d82606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd82609, 68 },
    { 0x23682609, 90 },
    { 0xd348260b, 61 },
    { 0xd1d8260a, 95 },
    { 0x9e88260d, 95 },
    { 0xd378260e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88382611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47782614, 61 },
    { 0x4b482614, 34 },
    { 0xc8982616, 23 },
    { 0x5f982617, 76 },
    { 0x46182617, 48 },
    { 0xa8682619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48261d, 34 },
    { 0x8f38261e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8182621, 90 },
    { 0, 0 },
    { 0x4ec82623, 56 },
    { 0x9be82624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37682627, 98 },
    { 0xb1a82628, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a8262c, 90 },
    { 0, 0 },
    { 0xff48262e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27182632, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd82637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8263b, 80 },
    { 0x6b88263b, 23 },
    { 0, 0 },
    { 0x2b88263e, 91 },
    { 0, 0 },
    { 0x94182640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99782648, 95 },
    { 0xa3282649, 34 },
    { 0xccc8264a, 85 },
    { 0, 0 },
    { 0xb028264c, 33 },
    { 0, 0 },
    { 0xfca8264e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15182651, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d282658, 23 },
    { 0x96682659, 24 },
    { 0x1558265a, 90 },
    { 0x5a68265b, 100 },
    { 0x3c68265b, 90 },
    { 0xc168265d, 23 },
    { 0xb328265d, 11 },
    { 0x3aa8265f, 95 },
    { 0xbbf8265b, 66 },
    { 0xeb682661, 76 },
    { 0x7a082662, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b82670, 61 },
    { 0xe2e82671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d82678, 99 },
    { 0xccf82679, 61 },
    { 0xd8c8267a, 98 },
    { 0x2fd8267b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3282682, 76 },
    { 0xc0982683, 55 },
    { 0, 0 },
    { 0x34f82685, 84 },
    { 0x1bf82686, 100 },
    { 0x8d082687, 61 },
    { 0x51e82686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e982695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d82699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c8269c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818269f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a826a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1826a6, 84 },
    { 0x5b4826a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f826b3, 55 },
    { 0xb76826b4, 33 },
    { 0, 0 },
    { 0xa5d826b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1826ba, 52 },
    { 0xb8a826ba, 33 },
    { 0, 0 },
    { 0x19826bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b826c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729826c9, 23 },
    { 0x2cb826c9, 85 },
    { 0x3a3826cb, 61 },
    { 0x953826cc, 55 },
    { 0x513826cc, 90 },
    { 0xcd7826ce, 90 },
    { 0xa60826cf, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e826d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e826d5, 43 },
    { 0x16d826d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d826db, 66 },
    { 0xd5c826dc, 84 },
    { 0x452826dd, 33 },
    { 0x90d826dc, 61 },
    { 0, 0 },
    { 0xf32826e0, 55 },
    { 0, 0 },
    { 0xe64826e2, 46 },
    { 0xbe826e3, 40 },
    { 0x9f7826e3, 76 },
    { 0x88b826e5, 76 },
    { 0x269826e4, 76 },
    { 0, 0 },
    { 0x2bc826e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72826ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d826f4, 90 },
    { 0x3d6826f5, 90 },
    { 0x766826f6, 55 },
    { 0xeee826f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a826fb, 76 },
    { 0xc50826fb, 90 },
    { 0x452826fd, 33 },
    { 0x1ff826fe, 98 },
    { 0x17a826ff, 23 },
    { 0xf46826fc, 66 },
    { 0xc5982700, 34 },
    { 0, 0 },
    { 0x7fe82703, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e82707, 98 },
    { 0x7e682708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78270d, 61 },
    { 0x9c8270e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf282711, 33 },
    { 0xd5182711, 17 },
    { 0, 0 },
    { 0xe0882714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a82718, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38271c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd882721, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc82729, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8272e, 91 },
    { 0xe678272f, 52 },
    { 0xbb18272f, 88 },
    { 0xddd8272e, 95 },
    { 0xa1f82732, 66 },
    { 0x7b282733, 90 },
    { 0x4ba82734, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908273e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb82749, 98 },
    { 0, 0 },
    { 0x74c8274b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86582750, 93 },
    { 0xaf482751, 33 },
    { 0, 0 },
    { 0x97882753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d82757, 84 },
    { 0x7c882758, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8275e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b82766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d8276a, 65 },
    { 0, 0 },
    { 0xbbe8276c, 98 },
    { 0x9748276d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61482771, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b82778, 95 },
    { 0, 0 },
    { 0x28d8277a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858277d, 61 },
    { 0, 0 },
    { 0xbdf8277f, 61 },
    { 0xd0282780, 98 },
    { 0xa4f82781, 33 },
    { 0, 0 },
    { 0x3cd82783, 98 },
    { 0xaa482784, 33 },
    { 0xdad82785, 93 },
    { 0x1ec82783, 65 },
    { 0x5ad82787, 33 },
    { 0xe982786, 95 },
    { 0x33182787, 84 },
    { 0, 0 },
    { 0x6088278b, 98 },
    { 0x3e38278c, 90 },
    { 0xc508278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6882795, 68 },
    { 0x1bf82796, 61 },
    { 0x4af82796, 90 },
    { 0xfef82798, 76 },
    { 0, 0 },
    { 0x26e8279a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe248279e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563827a2, 95 },
    { 0x24827a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9827a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d827ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3827b0, 43 },
    { 0, 0 },
    { 0xace827b2, 48 },
    { 0x585827b3, 91 },
    { 0, 0 },
    { 0xc99827b5, 46 },
    { 0, 0 },
    { 0xe3e827b7, 55 },
    { 0xbe8827b7, 95 },
    { 0x921827b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807827c5, 58 },
    { 0xef8827c6, 89 },
    { 0x9a1827c7, 47 },
    { 0xe73827c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9827d1, 66 },
    { 0, 0 },
    { 0x85b827d3, 90 },
    { 0x165827d4, 33 },
    { 0x428827d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb827d8, 33 },
    { 0, 0 },
    { 0xf3b827da, 66 },
    { 0, 0 },
    { 0x141827dc, 98 },
    { 0, 0 },
    { 0x5d827de, 76 },
    { 0x22d827df, 84 },
    { 0, 0 },
    { 0x8fc827e1, 100 },
    { 0x451827e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3827e5, 90 },
    { 0xe65827e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e827e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x132827ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5827f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec827f6, 90 },
    { 0x850827f7, 33 },
    { 0xd96827f6, 90 },
    { 0x235827f9, 55 },
    { 0xc0a827f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb827ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e82807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3408280a, 55 },
    { 0, 0 },
    { 0x3ef8280c, 33 },
    { 0xe2c8280d, 87 },
    { 0, 0 },
    { 0x1fa8280f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df82812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2682816, 91 },
    { 0xd7e82816, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d78281a, 61 },
    { 0, 0 },
    { 0x16f8281c, 77 },
    { 0xdc78281d, 98 },
    { 0x4968281c, 33 },
    { 0, 0 },
    { 0x7a882820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b82826, 87 },
    { 0x45782826, 31 },
    { 0, 0 },
    { 0xe9d82829, 67 },
    { 0xe608282a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe078282f, 83 },
    { 0x5fc82830, 46 },
    { 0xa5982830, 98 },
    { 0x9298282f, 90 },
    { 0xa2d82831, 76 },
    { 0x9b982834, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8283a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58283e, 100 },
    { 0xeaf8283f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382843, 33 },
    { 0x62582844, 95 },
    { 0xebd82843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75182848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898284b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118284f, 98 },
    { 0, 0 },
    { 0xf1b82851, 33 },
    { 0, 0 },
    { 0x4fa82853, 34 },
    { 0x81282854, 34 },
    { 0x1a982855, 90 },
    { 0, 0 },
    { 0xc4382857, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8285c, 33 },
    { 0, 0 },
    { 0x5bc8285e, 61 },
    { 0x9628285f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3182863, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd682866, 93 },
    { 0xacf82867, 33 },
    { 0xe8a82868, 43 },
    { 0, 0 },
    { 0x55b8286a, 61 },
    { 0xb638286b, 23 },
    { 0x3a68286a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc82873, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2908287c, 98 },
    { 0, 0 },
    { 0xeb88287e, 77 },
    { 0, 0 },
    { 0x8d382880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe182886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe482889, 52 },
    { 0xe7682889, 90 },
    { 0x42a8288b, 98 },
    { 0x9028288c, 88 },
    { 0x7578288d, 95 },
    { 0xe9d82889, 98 },
    { 0x56982889, 95 },
    { 0x1f82890, 68 },
    { 0x2e082890, 100 },
    { 0x6598288e, 43 },
    { 0xe8a82893, 33 },
    { 0xb0d82894, 52 },
    { 0xd4282893, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb082898, 23 },
    { 0, 0 },
    { 0xeb08289a, 25 },
    { 0x5c18289b, 23 },
    { 0xd478289c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e8289f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1828a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba828a5, 98 },
    { 0, 0 },
    { 0xed2828a7, 61 },
    { 0x91c828a8, 90 },
    { 0, 0 },
    { 0xd55828aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb828ad, 95 },
    { 0xd57828ae, 98 },
    { 0x2aa828ad, 85 },
    { 0x9f6828b0, 33 },
    { 0x51f828ae, 80 },
    { 0x7ba828b2, 95 },
    { 0, 0 },
    { 0x8d7828b4, 75 },
    { 0xf1b828b5, 87 },
    { 0xc02828b6, 95 },
    { 0, 0 },
    { 0xd2d828b8, 95 },
    { 0x3e1828b9, 46 },
    { 0x138828b9, 84 },
    { 0x352828bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff828be, 84 },
    { 0x634828be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c828c9, 90 },
    { 0x4cb828ca, 61 },
    { 0x97f828ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d828d1, 55 },
    { 0x41e828d1, 76 },
    { 0xa90828d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0828dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09828e9, 76 },
    { 0xf84828ea, 98 },
    { 0, 0 },
    { 0x530828ec, 78 },
    { 0, 0 },
    { 0xc6a828ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b828f2, 64 },
    { 0, 0 },
    { 0xb6828f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3828f8, 77 },
    { 0, 0 },
    { 0xb62828fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a82901, 90 },
    { 0xfca82902, 66 },
    { 0x52082903, 95 },
    { 0x96282901, 87 },
    { 0x58682905, 77 },
    { 0x17882902, 43 },
    { 0, 0 },
    { 0x24d82908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588290d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b882910, 61 },
    { 0xadd82910, 23 },
    { 0, 0 },
    { 0x96482913, 93 },
    { 0, 0 },
    { 0xd0782915, 61 },
    { 0, 0 },
    { 0xe1182917, 33 },
    { 0, 0 },
    { 0x7c482919, 85 },
    { 0x7158291a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5582928, 90 },
    { 0x6b482929, 78 },
    { 0x1098292a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc8292f, 61 },
    { 0, 0 },
    { 0xcf782931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f82934, 67 },
    { 0x20582935, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a82939, 33 },
    { 0xfc58293a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf48293d, 98 },
    { 0x98c8293d, 61 },
    { 0xd038293e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d82942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f98294b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f8294e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81182952, 76 },
    { 0x37582953, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26782959, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8295e, 23 },
    { 0, 0 },
    { 0x72b82960, 90 },
    { 0, 0 },
    { 0xe8282962, 61 },
    { 0, 0 },
    { 0x9bb82964, 84 },
    { 0xc4c82965, 80 },
    { 0x2d782965, 33 },
    { 0x6cc82967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf358296c, 98 },
    { 0xbd28296d, 34 },
    { 0x5898296e, 53 },
    { 0xe568296d, 90 },
    { 0x1d082970, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f482974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b82977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9788297b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2958297f, 98 },
    { 0x9ef8297f, 76 },
    { 0, 0 },
    { 0xec82982, 19 },
    { 0xab682983, 43 },
    { 0x7d482984, 95 },
    { 0x19a82985, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8298a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b48298d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f982990, 34 },
    { 0, 0 },
    { 0x95982992, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa182997, 98 },
    { 0xdca82997, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0829a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d829a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42829ab, 99 },
    { 0x38e829ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3829b2, 33 },
    { 0xd8f829b3, 55 },
    { 0, 0 },
    { 0x27d829b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e829bd, 17 },
    { 0, 0 },
    { 0x43d829bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c829c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8829ce, 67 },
    { 0xa95829ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737829d3, 69 },
    { 0x46d829d4, 61 },
    { 0x6d5829d3, 85 },
    { 0x15b829d5, 95 },
    { 0x10b829d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6829df, 33 },
    { 0, 0 },
    { 0xd38829e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d829e4, 23 },
    { 0x1cf829e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e829e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc829ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9829f0, 65 },
    { 0xd90829f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd482a01, 90 },
    { 0x48e82a02, 76 },
    { 0x1a382a03, 33 },
    { 0xe3582a01, 95 },
    { 0x21082a03, 98 },
    { 0, 0 },
    { 0x14982a07, 90 },
    { 0xb8782a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e82a0c, 33 },
    { 0xb7682a0d, 76 },
    { 0xf5082a0c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb282a1c, 43 },
    { 0x49782a1d, 61 },
    { 0xd8982a1e, 90 },
    { 0x74582a1f, 90 },
    { 0xf0982a20, 78 },
    { 0xbea82a21, 90 },
    { 0xeee82a22, 33 },
    { 0, 0 },
    { 0x18682a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b282a28, 77 },
    { 0, 0 },
    { 0x65b82a2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97682a33, 61 },
    { 0x6fc82a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef82a39, 34 },
    { 0xdc582a3a, 95 },
    { 0x59582a3b, 33 },
    { 0x61582a3c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2582a42, 61 },
    { 0xa7b82a43, 61 },
    { 0xd0182a43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2282a49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d82a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8082a51, 61 },
    { 0x83782a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34182a5b, 67 },
    { 0, 0 },
    { 0x15782a5d, 98 },
    { 0, 0 },
    { 0xbf182a5f, 95 },
    { 0xaec82a60, 61 },
    { 0xfe282a5f, 90 },
    { 0xc2c82a62, 46 },
    { 0x3af82a63, 90 },
    { 0xe2b82a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c82a68, 46 },
    { 0x74c82a69, 75 },
    { 0x4c382a6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb282a6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e82a70, 84 },
    { 0, 0 },
    { 0x86a82a72, 95 },
    { 0x17082a73, 99 },
    { 0x56582a72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e82a78, 55 },
    { 0x45182a78, 95 },
    { 0x25282a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b82a7d, 23 },
    { 0xe1482a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17582a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61382a8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a82a8e, 87 },
    { 0x30382a8f, 78 },
    { 0x97d82a90, 33 },
    { 0x1f82a90, 23 },
    { 0x12282a92, 43 },
    { 0xbf482a91, 76 },
    { 0xdbc82a93, 76 },
    { 0x25782a95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a682a98, 98 },
    { 0xb2782a99, 76 },
    { 0x58282a9a, 90 },
    { 0xeac82a9b, 77 },
    { 0xafb82a9c, 33 },
    { 0x8fa82a9d, 55 },
    { 0x4d182a9d, 78 },
    { 0xc0182a9d, 95 },
    { 0, 0 },
    { 0x76f82aa1, 87 },
    { 0xc5882aa2, 34 },
    { 0x6f882aa1, 19 },
    { 0, 0 },
    { 0x2c882aa5, 85 },
    { 0xa7b82aa6, 90 },
    { 0xca282aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c82aac, 23 },
    { 0xbf882aad, 33 },
    { 0xc1482aac, 33 },
    { 0xffa82aac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c082ab2, 90 },
    { 0x36d82ab3, 46 },
    { 0, 0 },
    { 0xa3982ab5, 93 },
    { 0x1f182ab6, 95 },
    { 0x6bd82ab7, 33 },
    { 0xd0182ab8, 95 },
    { 0, 0 },
    { 0xe4982aba, 90 },
    { 0, 0 },
    { 0xf2182abc, 61 },
    { 0, 0 },
    { 0x37082abe, 77 },
    { 0xe3782abf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b82ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd82acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40382ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c82ad2, 34 },
    { 0xd0d82ad3, 61 },
    { 0x76d82ad3, 61 },
    { 0, 0 },
    { 0x45c82ad6, 43 },
    { 0xa6982ad6, 61 },
    { 0x1e382ad8, 76 },
    { 0x35082ad9, 98 },
    { 0xb8282ad8, 76 },
    { 0xe9882adb, 23 },
    { 0x9d382ad6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4182ae6, 90 },
    { 0x8882ae7, 43 },
    { 0xf4a82ae8, 17 },
    { 0xa0f82ae8, 95 },
    { 0xe0882ae6, 75 },
    { 0x7ad82aeb, 93 },
    { 0, 0 },
    { 0xc2582aed, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda82af2, 34 },
    { 0x3cd82af3, 87 },
    { 0, 0 },
    { 0xc1f82af5, 76 },
    { 0, 0 },
    { 0xa2a82af7, 33 },
    { 0xb7f82af8, 95 },
    { 0x75882af9, 99 },
    { 0, 0 },
    { 0x4182afb, 33 },
    { 0, 0 },
    { 0x6a882afd, 23 },
    { 0x56d82afe, 93 },
    { 0, 0 },
    { 0x51d82b00, 23 },
    { 0x79082b00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda282b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa82b07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c582b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41382b13, 91 },
    { 0x9182b14, 17 },
    { 0xe1982b15, 55 },
    { 0, 0 },
    { 0x20182b17, 93 },
    { 0x27082b17, 95 },
    { 0x5f382b18, 95 },
    { 0, 0 },
    { 0x57982b1b, 95 },
    { 0x87182b1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3082b22, 43 },
    { 0xe9b82b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f182b2e, 46 },
    { 0, 0 },
    { 0x34b82b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96482b41, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15682b46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7482b49, 23 },
    { 0x11182b49, 33 },
    { 0, 0 },
    { 0x9b882b4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xff682b4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d82b56, 46 },
    { 0x5c482b57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e82b5a, 95 },
    { 0xf1182b5b, 90 },
    { 0xaef82b5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c782b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f82b65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8082b6d, 28 },
    { 0xdf682b6d, 98 },
    { 0xc6382b6d, 34 },
    { 0x77a82b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a82b74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1682b7d, 55 },
    { 0x96882b7e, 43 },
    { 0x93c82b7d, 33 },
    { 0, 0 },
    { 0x7c782b81, 95 },
    { 0, 0 },
    { 0x83382b83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39582b86, 43 },
    { 0x16882b87, 91 },
    { 0xd2582b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8382b8c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b782b91, 78 },
    { 0x68a82b92, 91 },
    { 0x5ac82b93, 52 },
    { 0xd1682b94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d82b99, 87 },
    { 0, 0 },
    { 0xd7282b9b, 95 },
    { 0, 0 },
    { 0xa4182b9d, 77 },
    { 0xa2282b9e, 46 },
    { 0xeef82b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f82ba2, 78 },
    { 0, 0 },
    { 0x85282ba4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd82ba7, 76 },
    { 0x7eb82ba8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0582bac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c82bb3, 43 },
    { 0, 0 },
    { 0x64182bb5, 33 },
    { 0x12282bb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f82bba, 33 },
    { 0xdaf82bbb, 83 },
    { 0xdf082bbc, 76 },
    { 0xd2a82bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad82bc1, 43 },
    { 0, 0 },
    { 0xfd182bc3, 67 },
    { 0xd6582bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61782bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d82bcc, 34 },
    { 0xbc982bcd, 87 },
    { 0x8dd82bce, 46 },
    { 0x98a82bcd, 33 },
    { 0, 0 },
    { 0x31082bd1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce82bd6, 43 },
    { 0, 0 },
    { 0x95c82bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef82bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe082bdf, 33 },
    { 0x77082be0, 33 },
    { 0x69582be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f82be5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb82beb, 38 },
    { 0xa5c82bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd82bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c82bf3, 33 },
    { 0xa6682bf4, 90 },
    { 0xad082bf5, 77 },
    { 0xa382bf4, 90 },
    { 0x83e82bf7, 46 },
    { 0xfdb82bf7, 61 },
    { 0x75682bf3, 55 },
    { 0x12982bfa, 93 },
    { 0x3de82bfb, 90 },
    { 0, 0 },
    { 0x64382bfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc82c07, 78 },
    { 0x27482c08, 23 },
    { 0, 0 },
    { 0x54882c0a, 61 },
    { 0xd5382c0a, 23 },
    { 0x1e182c0c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d682c12, 23 },
    { 0x9f82c13, 33 },
    { 0x2d182c14, 33 },
    { 0, 0 },
    { 0xc182c16, 95 },
    { 0x84f82c17, 34 },
    { 0, 0 },
    { 0x8f82c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96582c20, 27 },
    { 0, 0 },
    { 0x41582c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e82c25, 33 },
    { 0xf1682c26, 78 },
    { 0xec882c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83082c2d, 98 },
    { 0xbfd82c2e, 90 },
    { 0x1c182c2f, 61 },
    { 0, 0 },
    { 0x4db82c31, 33 },
    { 0, 0 },
    { 0xd6482c33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd82c3a, 95 },
    { 0x6c582c3b, 23 },
    { 0, 0 },
    { 0x12082c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1782c4b, 33 },
    { 0x5e182c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f82c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982c52, 76 },
    { 0x57d82c53, 34 },
    { 0x6cb82c54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32482c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff82c6a, 99 },
    { 0x1c82c6b, 87 },
    { 0x21f82c6c, 76 },
    { 0xb6582c6d, 61 },
    { 0x86082c6c, 23 },
    { 0x64482c6c, 43 },
    { 0x38f82c70, 93 },
    { 0x48782c6c, 66 },
    { 0x12282c72, 90 },
    { 0x8fd82c6d, 67 },
    { 0x19582c74, 85 },
    { 0x20f82c6e, 36 },
    { 0x83182c74, 33 },
    { 0, 0 },
    { 0xa1e82c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb82c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e82c80, 81 },
    { 0, 0 },
    { 0x6982c82, 61 },
    { 0x6182c83, 85 },
    { 0x37a82c84, 66 },
    { 0, 0 },
    { 0x26b82c86, 90 },
    { 0x15c82c86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb382c8b, 55 },
    { 0xa2e82c8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa382c91, 61 },
    { 0xae682c92, 84 },
    { 0xc9b82c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c982c9b, 100 },
    { 0x26082c9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e82ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d82ca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a82cac, 23 },
    { 0xc9182cad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea82cb2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc982cb6, 34 },
    { 0, 0 },
    { 0xe0d82cb8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x69382cbb, 23 },
    { 0x63582cbc, 87 },
    { 0xb9a82cbd, 84 },
    { 0x97882cbe, 61 },
    { 0, 0 },
    { 0x43b82cc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2682cc3, 34 },
    { 0xa3982cc3, 98 },
    { 0x16d82cc5, 67 },
    { 0, 0 },
    { 0x35e82cc7, 33 },
    { 0x3f82cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41882cd1, 61 },
    { 0xcf682cd2, 61 },
    { 0, 0 },
    { 0x9e782cd4, 95 },
    { 0x66b82cd5, 9 },
    { 0x69882cd4, 97 },
    { 0x70582cd6, 33 },
    { 0, 0 },
    { 0x90082cd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4182cdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b382ce2, 90 },
    { 0x1c282ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b82ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d082cf1, 90 },
    { 0x79a82cf1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf82cfb, 33 },
    { 0x9982cfc, 90 },
    { 0x1ee82cfd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42182d04, 61 },
    { 0, 0 },
    { 0xa3282d06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea82d09, 61 },
    { 0xc7c82d0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86882d0f, 77 },
    { 0x94882d10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a82d14, 95 },
    { 0x89382d15, 90 },
    { 0x18782d16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba82d1e, 95 },
    { 0x19e82d1e, 61 },
    { 0, 0 },
    { 0x4d582d21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x91482d24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa182d27, 33 },
    { 0x90682d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e782d2c, 61 },
    { 0x69282d2d, 95 },
    { 0x87a82d2e, 95 },
    { 0x35882d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c282d36, 23 },
    { 0x73b82d37, 88 },
    { 0x84382d38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed682d42, 61 },
    { 0xcb182d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83782d4b, 61 },
    { 0x64882d4b, 91 },
    { 0xef382d4b, 43 },
    { 0x35e82d4d, 61 },
    { 0xb5e82d4b, 23 },
    { 0x31882d4b, 33 },
    { 0x43282d4f, 76 },
    { 0x97482d52, 95 },
    { 0, 0 },
    { 0xb1582d54, 81 },
    { 0, 0 },
    { 0x19482d56, 90 },
    { 0x76a82d57, 95 },
    { 0x56482d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc182d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8282d5f, 77 },
    { 0, 0 },
    { 0x22782d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d182d65, 23 },
    { 0, 0 },
    { 0x4b682d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18982d6a, 66 },
    { 0x44882d6b, 90 },
    { 0x78e82d6c, 33 },
    { 0, 0 },
    { 0x98c82d6e, 46 },
    { 0x7d882d6e, 90 },
    { 0, 0 },
    { 0xd3782d71, 76 },
    { 0x1fb82d72, 34 },
    { 0, 0 },
    { 0x9a782d74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef82d7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75982d84, 46 },
    { 0, 0 },
    { 0x82b82d86, 95 },
    { 0xd2382d87, 98 },
    { 0x2d182d88, 70 },
    { 0xde82d89, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d82d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23182d96, 23 },
    { 0x45582d97, 76 },
    { 0x55782d97, 91 },
    { 0x14b82d97, 98 },
    { 0x30882d98, 81 },
    { 0xdc082d9b, 23 },
    { 0x36182d9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c882d9f, 55 },
    { 0x5c282d9f, 90 },
    { 0, 0 },
    { 0xc8782da2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f82da6, 100 },
    { 0x1d982da6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd82dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9682db8, 65 },
    { 0, 0 },
    { 0x82f82dba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d982dbd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62882dc2, 23 },
    { 0x2e582dc3, 33 },
    { 0, 0 },
    { 0x78c82dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58382dcb, 100 },
    { 0x59882dcc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x60682dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5782dd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b82dd6, 76 },
    { 0, 0 },
    { 0x5e282dd8, 5 },
    { 0xe4882dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37182ddd, 34 },
    { 0xf2182dde, 33 },
    { 0x94c82dde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde182de2, 98 },
    { 0x1ad82de3, 90 },
    { 0xacb82de3, 95 },
    { 0xf6e82de5, 34 },
    { 0x49082de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e82dec, 61 },
    { 0x8ae82ded, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd82dfd, 100 },
    { 0, 0 },
    { 0x82a82dff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24882e07, 61 },
    { 0xa6382e08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b82e0b, 100 },
    { 0x30082e0c, 23 },
    { 0xb7c82e0c, 76 },
    { 0xab482e0d, 76 },
    { 0xca682e0e, 76 },
    { 0xe2482e0b, 34 },
    { 0, 0 },
    { 0xdb382e12, 61 },
    { 0x79d82e12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca82e1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58582e1f, 98 },
    { 0, 0 },
    { 0xa6c82e21, 61 },
    { 0xdfa82e21, 93 },
    { 0xfa982e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8482e27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1782e2b, 93 },
    { 0x8a782e2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c682e31, 61 },
    { 0x87d82e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b82e3e, 46 },
    { 0xf6382e3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c882e43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d82e47, 100 },
    { 0x43582e48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34882e4c, 90 },
    { 0, 0 },
    { 0x96282e4e, 78 },
    { 0, 0 },
    { 0x6f82e50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1782e57, 77 },
    { 0x1fe82e57, 61 },
    { 0x24682e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd082e5f, 33 },
    { 0xeeb82e60, 61 },
    { 0x52a82e61, 33 },
    { 0xe9b82e62, 61 },
    { 0x96182e63, 98 },
    { 0xac082e64, 87 },
    { 0x2382e65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x99882e68, 33 },
    { 0, 0 },
    { 0xd6382e6a, 87 },
    { 0x4a182e6a, 66 },
    { 0x78182e6a, 83 },
    { 0x65782e6c, 98 },
    { 0x93d82e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb82e72, 33 },
    { 0x7ef82e72, 61 },
    { 0x60b82e72, 98 },
    { 0xe9282e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5982e78, 33 },
    { 0, 0 },
    { 0x2ff82e7a, 66 },
    { 0, 0 },
    { 0xca882e7c, 90 },
    { 0x19f82e7d, 95 },
    { 0x54682e7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e982e82, 90 },
    { 0x96c82e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de82e86, 56 },
    { 0x8f582e87, 95 },
    { 0xed482e87, 76 },
    { 0xbd882e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e382e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd182e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88582e95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e582e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce082e9f, 23 },
    { 0, 0 },
    { 0xed582ea1, 55 },
    { 0xd4d82ea1, 61 },
    { 0, 0 },
    { 0x24682ea4, 46 },
    { 0x9b482ea4, 23 },
    { 0xd2282ea6, 46 },
    { 0xca582ea7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8182eaa, 23 },
    { 0x6c082eab, 98 },
    { 0x92c82eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f682eb0, 52 },
    { 0x76782eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa182eb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90182eb7, 66 },
    { 0, 0 },
    { 0x1bb82eb9, 98 },
    { 0x17582eb9, 93 },
    { 0x68782ebb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df82ebe, 23 },
    { 0x73782ebf, 61 },
    { 0xb0782ebe, 55 },
    { 0, 0 },
    { 0x5bb82ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d982ec5, 52 },
    { 0x48d82ec5, 95 },
    { 0xf3982ec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9482ed0, 61 },
    { 0x36782ed1, 90 },
    { 0x48082ed2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782edb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xde482ede, 76 },
    { 0x62482edf, 34 },
    { 0x8e682ede, 23 },
    { 0x46582ee1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade82ee5, 33 },
    { 0xbc282ee6, 90 },
    { 0x69c82ee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5082eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7082eee, 100 },
    { 0x83082eef, 98 },
    { 0x54482ef0, 52 },
    { 0x8f782eef, 43 },
    { 0xabf82ef2, 23 },
    { 0xfe282ef0, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35782ef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf82efb, 61 },
    { 0x15482efb, 98 },
    { 0xcdd82efd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb482f03, 78 },
    { 0, 0 },
    { 0x6082f05, 61 },
    { 0x53082f05, 33 },
    { 0x71e82f07, 66 },
    { 0xec82f08, 90 },
    { 0, 0 },
    { 0xd0282f0a, 90 },
    { 0x37c82f0b, 53 },
    { 0x1fa82f0c, 97 },
    { 0, 0 },
    { 0xacc82f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae082f12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e082f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782f19, 87 },
    { 0, 0 },
    { 0x6ce82f1b, 98 },
    { 0xbee82f1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8782f1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d82f25, 61 },
    { 0xc0d82f26, 33 },
    { 0, 0 },
    { 0x49f82f28, 90 },
    { 0x59982f29, 23 },
    { 0, 0 },
    { 0xa0782f2b, 98 },
    { 0x12582f2c, 33 },
    { 0x12882f2c, 61 },
    { 0xcca82f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b82f32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87682f36, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b82f3e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b82f47, 46 },
    { 0, 0 },
    { 0x11f82f49, 68 },
    { 0x17f82f49, 98 },
    { 0x76282f4b, 95 },
    { 0x2b182f4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37682f4f, 88 },
    { 0, 0 },
    { 0xef282f51, 81 },
    { 0, 0 },
    { 0x1cc82f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a282f57, 90 },
    { 0x6f282f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a82f5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69282f5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1582f64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91182f6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9082f6f, 95 },
    { 0x43f82f6f, 55 },
    { 0x54b82f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4182f78, 98 },
    { 0xa5282f78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80882f84, 95 },
    { 0, 0 },
    { 0xd7c82f86, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b82f89, 33 },
    { 0xa4382f89, 43 },
    { 0x3de82f8b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc82f8e, 33 },
    { 0x83e82f8f, 65 },
    { 0x8b482f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58182f95, 95 },
    { 0x84782f96, 33 },
    { 0x8dc82f97, 91 },
    { 0x6c682f98, 33 },
    { 0xbf582f99, 23 },
    { 0x7a882f99, 100 },
    { 0xabb82f9b, 55 },
    { 0x73382f9b, 90 },
    { 0x36f82f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27782fa9, 76 },
    { 0x23282fa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e82fad, 61 },
    { 0, 0 },
    { 0x99182faf, 90 },
    { 0xa382fb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96782fb6, 68 },
    { 0, 0 },
    { 0x99682fb8, 98 },
    { 0x2a582fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c82fbf, 98 },
    { 0x3e582fc0, 98 },
    { 0x37782fc1, 78 },
    { 0x4eb82fc0, 34 },
    { 0x7ab82fc3, 66 },
    { 0xbe82fc4, 90 },
    { 0x11182fc4, 98 },
    { 0x4d882fc2, 33 },
    { 0, 0 },
    { 0x71882fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba82fcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66182fd1, 23 },
    { 0x5e782fd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c82fd6, 46 },
    { 0x45c82fd7, 96 },
    { 0xa4982fd8, 76 },
    { 0x20882fd7, 93 },
    { 0xfd982fda, 34 },
    { 0x6fd82fda, 66 },
    { 0xb7182fdb, 40 },
    { 0xef982fdd, 95 },
    { 0, 0 },
    { 0x85c82fdf, 87 },
    { 0xa8d82fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb282fe7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c282fef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60982ff3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e482ff9, 23 },
    { 0x44b82ffa, 78 },
    { 0x35e82ffb, 90 },
    { 0x9b82ffc, 95 },
    { 0, 0 },
    { 0x36182ffe, 98 },
    { 0xa2182fff, 98 },
    { 0xe5682fff, 46 },
    { 0x5683001, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c183004, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5358300b, 33 },
    { 0xde38300c, 77 },
    { 0xd0b8300d, 90 },
    { 0x2918300e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df83012, 61 },
    { 0, 0 },
    { 0x13283014, 23 },
    { 0xa2a83014, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd883018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1728301c, 61 },
    { 0xc528301d, 55 },
    { 0, 0 },
    { 0x6cd8301f, 23 },
    { 0xdb783020, 55 },
    { 0x8bc83020, 33 },
    { 0x4d183022, 87 },
    { 0, 0 },
    { 0x96b83024, 52 },
    { 0x6dc83024, 95 },
    { 0, 0 },
    { 0x9f583027, 23 },
    { 0xda083028, 43 },
    { 0x5083029, 76 },
    { 0, 0 },
    { 0x3888302b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8302e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66583033, 19 },
    { 0x8cf83034, 23 },
    { 0, 0 },
    { 0x17e83036, 61 },
    { 0x5eb83036, 66 },
    { 0xd2a83038, 23 },
    { 0, 0 },
    { 0x84a8303a, 61 },
    { 0x4d28303b, 51 },
    { 0x90f8303a, 90 },
    { 0x4328303b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x30483040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2883043, 33 },
    { 0xb7883043, 33 },
    { 0, 0 },
    { 0x1a83046, 23 },
    { 0x94983046, 95 },
    { 0x4a283048, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc018304b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b83050, 90 },
    { 0x1d883051, 93 },
    { 0x46f83052, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca83055, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a83058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e8305f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c483063, 33 },
    { 0, 0 },
    { 0xc1f83065, 98 },
    { 0xcc283066, 78 },
    { 0xa1783067, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8306c, 55 },
    { 0x5988306d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33983077, 95 },
    { 0, 0 },
    { 0x66f83079, 61 },
    { 0x5648307a, 98 },
    { 0xd938307b, 95 },
    { 0, 0 },
    { 0x9ab8307d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb83083, 23 },
    { 0xbf883084, 90 },
    { 0xd3d83084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d83088, 80 },
    { 0x89983089, 33 },
    { 0, 0 },
    { 0xec98308b, 88 },
    { 0xc8b8308c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c8308f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d83092, 90 },
    { 0, 0 },
    { 0x3b983094, 68 },
    { 0x34983094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24783099, 76 },
    { 0x7fd8309a, 23 },
    { 0xa398309b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3830a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c830a7, 98 },
    { 0xe04830a7, 83 },
    { 0x129830a9, 90 },
    { 0x3cd830a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0830af, 90 },
    { 0x9a6830b0, 87 },
    { 0, 0 },
    { 0xa58830b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03830b5, 17 },
    { 0, 0 },
    { 0x507830b7, 98 },
    { 0x9bf830b8, 90 },
    { 0xe80830b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398830bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf830c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6830c5, 81 },
    { 0xec6830c5, 61 },
    { 0xef3830c5, 61 },
    { 0x511830c8, 23 },
    { 0, 0 },
    { 0x1cd830ca, 52 },
    { 0x886830ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b830ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a830d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d830da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a830e4, 90 },
    { 0x72a830e5, 100 },
    { 0x1ac830e5, 61 },
    { 0, 0 },
    { 0x689830e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e830ee, 33 },
    { 0xab830ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd830f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a830f7, 26 },
    { 0x64e830f8, 33 },
    { 0xd34830f8, 61 },
    { 0, 0 },
    { 0x615830fb, 90 },
    { 0xd2b830fb, 95 },
    { 0, 0 },
    { 0x3c0830fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30983103, 77 },
    { 0x4683103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3983113, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea83118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18311d, 33 },
    { 0xdac8311e, 98 },
    { 0, 0 },
    { 0xb7e83120, 85 },
    { 0x7a183120, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f183124, 33 },
    { 0x1b983125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738312a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35783130, 76 },
    { 0, 0 },
    { 0x53183132, 34 },
    { 0x5b283133, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f683139, 55 },
    { 0, 0 },
    { 0xadd8313b, 23 },
    { 0, 0 },
    { 0x3928313d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92783142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f83146, 91 },
    { 0x6bb83147, 61 },
    { 0xbfd83148, 23 },
    { 0, 0 },
    { 0xa588314a, 67 },
    { 0x3ca8314b, 66 },
    { 0, 0 },
    { 0x3ee8314d, 78 },
    { 0x3ec8314e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a983154, 95 },
    { 0x4f883155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee983158, 34 },
    { 0, 0 },
    { 0x6548315a, 85 },
    { 0xdeb8315a, 33 },
    { 0x6d28315b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69483162, 91 },
    { 0x42783163, 75 },
    { 0, 0 },
    { 0x6e583165, 76 },
    { 0xa5e83165, 93 },
    { 0x3e183166, 52 },
    { 0xe2d83165, 23 },
    { 0xc1883167, 34 },
    { 0x96183169, 46 },
    { 0x68c8316b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c8316f, 78 },
    { 0x4bc83170, 23 },
    { 0, 0 },
    { 0xe9c83172, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b383177, 91 },
    { 0x24a83177, 52 },
    { 0xa5183179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7518317d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43583180, 33 },
    { 0xffc83181, 90 },
    { 0, 0 },
    { 0x59c83183, 90 },
    { 0xa3383184, 98 },
    { 0x5b083185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8318c, 90 },
    { 0x13b8318d, 90 },
    { 0x9838318e, 90 },
    { 0, 0 },
    { 0xb583190, 78 },
    { 0, 0 },
    { 0x52783192, 34 },
    { 0x68483193, 100 },
    { 0xfbc83194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48183197, 68 },
    { 0xe383197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a88319d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4831a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2831a7, 99 },
    { 0xd1831a8, 23 },
    { 0x510831a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c831ac, 88 },
    { 0xba5831ac, 85 },
    { 0xa45831ae, 98 },
    { 0, 0 },
    { 0xef8831b0, 50 },
    { 0, 0 },
    { 0xdb831b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632831b9, 34 },
    { 0x2f831ba, 76 },
    { 0xd5b831ba, 33 },
    { 0, 0 },
    { 0xc03831bd, 100 },
    { 0xc15831bd, 76 },
    { 0x1f5831be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8831c2, 90 },
    { 0, 0 },
    { 0x601831c4, 61 },
    { 0xe38831c5, 95 },
    { 0x7c6831c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248831cc, 52 },
    { 0x587831cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5831d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e831da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x598831dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884831e3, 98 },
    { 0x99d831e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9831e8, 33 },
    { 0x272831e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1831ed, 61 },
    { 0, 0 },
    { 0xf52831ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4831f3, 55 },
    { 0, 0 },
    { 0x399831f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f831fa, 75 },
    { 0x617831fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac831fe, 33 },
    { 0x858831ff, 61 },
    { 0x1c383200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78683209, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf483210, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1483216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c483219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538321c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48321f, 61 },
    { 0xd9483220, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19183225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c983229, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f8322c, 98 },
    { 0x1268322d, 81 },
    { 0xf938322e, 95 },
    { 0x2dc8322c, 52 },
    { 0, 0 },
    { 0x1d483231, 93 },
    { 0x39483231, 61 },
    { 0, 0 },
    { 0x65283234, 61 },
    { 0, 0 },
    { 0x17983236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728323a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab08323d, 23 },
    { 0, 0 },
    { 0xda8323f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e183246, 46 },
    { 0x48683247, 34 },
    { 0x2a783248, 43 },
    { 0xd4c83249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9583254, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8648325b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78325e, 67 },
    { 0x4508325e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79083263, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66483267, 98 },
    { 0, 0 },
    { 0x1bb83269, 43 },
    { 0, 0 },
    { 0xbd48326b, 43 },
    { 0x4488326b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed38326f, 98 },
    { 0x5318326f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff83273, 96 },
    { 0, 0 },
    { 0xfa783275, 90 },
    { 0, 0 },
    { 0x77383277, 43 },
    { 0x7a283277, 33 },
    { 0, 0 },
    { 0xdc68327a, 76 },
    { 0, 0 },
    { 0xce68327c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc8327f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea83285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57983288, 61 },
    { 0xf8483288, 90 },
    { 0xba283288, 98 },
    { 0x4d683288, 98 },
    { 0x85d8328c, 46 },
    { 0x40983288, 33 },
    { 0, 0 },
    { 0x4628328f, 69 },
    { 0xc383290, 98 },
    { 0x5c983290, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc83294, 55 },
    { 0, 0 },
    { 0x42983296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908329b, 78 },
    { 0, 0 },
    { 0xead8329d, 98 },
    { 0xa468329e, 91 },
    { 0, 0 },
    { 0x2f832a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1832a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520832ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x301832ae, 61 },
    { 0xb61832af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07832b2, 67 },
    { 0, 0 },
    { 0xbf0832b4, 91 },
    { 0xdc4832b4, 34 },
    { 0x81832b6, 90 },
    { 0, 0 },
    { 0xea2832b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15832bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400832bf, 90 },
    { 0, 0 },
    { 0xde7832c1, 34 },
    { 0x910832c2, 33 },
    { 0, 0 },
    { 0x2f6832c4, 98 },
    { 0x62e832c4, 33 },
    { 0x741832c6, 36 },
    { 0xae9832c7, 87 },
    { 0x71d832c4, 55 },
    { 0x143832c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94832ce, 95 },
    { 0, 0 },
    { 0x4ef832d0, 33 },
    { 0x43a832d0, 69 },
    { 0x5a6832d2, 78 },
    { 0x703832d1, 98 },
    { 0x465832d1, 95 },
    { 0, 0 },
    { 0x938832d6, 43 },
    { 0x7c9832d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f832da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f832de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2832e2, 75 },
    { 0x5d3832e3, 76 },
    { 0x6ce832e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b832e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8832f0, 100 },
    { 0xc1d832f1, 70 },
    { 0, 0 },
    { 0xf52832f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e832f6, 99 },
    { 0, 0 },
    { 0x1cc832f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8832fd, 66 },
    { 0x29832fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac883301, 81 },
    { 0x6fb83301, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83306, 98 },
    { 0xc1983306, 59 },
    { 0x28f83308, 55 },
    { 0x51f83309, 61 },
    { 0x8a38330a, 34 },
    { 0, 0 },
    { 0x7a28330c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708330f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96183313, 62 },
    { 0x37a83313, 23 },
    { 0xd8683314, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead8331d, 93 },
    { 0x3068331e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a83326, 95 },
    { 0, 0 },
    { 0x23983328, 34 },
    { 0xc4183329, 55 },
    { 0x38d8332a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c08332e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa83332, 46 },
    { 0, 0 },
    { 0x7883334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf83339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb8333c, 76 },
    { 0xb868333d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83344, 33 },
    { 0, 0 },
    { 0xa6e83346, 67 },
    { 0xfdf83347, 95 },
    { 0x5d783348, 23 },
    { 0x1c183347, 100 },
    { 0, 0 },
    { 0xb048334b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f8334e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d83352, 55 },
    { 0x8fa83353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90283357, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce18335a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e83366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f8336c, 99 },
    { 0x4188336d, 101 },
    { 0xca38336c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f383374, 33 },
    { 0x40c83374, 90 },
    { 0xb583376, 90 },
    { 0x14a83377, 55 },
    { 0x9ed83377, 46 },
    { 0xc0d83376, 93 },
    { 0x15183378, 34 },
    { 0xe288337b, 85 },
    { 0x93783376, 95 },
    { 0x29c83378, 90 },
    { 0x7408337e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3483381, 85 },
    { 0, 0 },
    { 0xb5383383, 46 },
    { 0x57983384, 76 },
    { 0xae183384, 33 },
    { 0x86883386, 66 },
    { 0xe1383387, 52 },
    { 0x2cc83385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c8338e, 89 },
    { 0x6838338e, 100 },
    { 0, 0 },
    { 0x56583391, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa838339e, 46 },
    { 0xafd8339f, 34 },
    { 0x615833a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x347833a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f833a7, 33 },
    { 0xf7b833a7, 33 },
    { 0, 0 },
    { 0x90833aa, 98 },
    { 0, 0 },
    { 0x7c833ac, 78 },
    { 0, 0 },
    { 0xcd1833ae, 52 },
    { 0x9f4833ae, 99 },
    { 0, 0 },
    { 0xd3833b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace833b4, 52 },
    { 0xb84833b4, 95 },
    { 0xa7e833b5, 23 },
    { 0x24e833b5, 77 },
    { 0, 0 },
    { 0xa29833b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e833bf, 56 },
    { 0, 0 },
    { 0xa15833c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a833c4, 61 },
    { 0xd77833c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7833ce, 23 },
    { 0x224833cf, 23 },
    { 0, 0 },
    { 0xfda833d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16833d6, 43 },
    { 0x2de833d7, 93 },
    { 0x7bb833d7, 98 },
    { 0x839833d9, 87 },
    { 0x2d7833da, 93 },
    { 0x5f2833db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b833ec, 33 },
    { 0, 0 },
    { 0x8f1833ee, 33 },
    { 0, 0 },
    { 0x570833f0, 61 },
    { 0x942833f1, 99 },
    { 0, 0 },
    { 0x493833f3, 66 },
    { 0xe5833f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac833f8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196833fc, 95 },
    { 0, 0 },
    { 0x99f833fe, 48 },
    { 0xe0b833fe, 100 },
    { 0xd5f83400, 55 },
    { 0x54483401, 61 },
    { 0, 0 },
    { 0xd5f83403, 17 },
    { 0x2e683404, 98 },
    { 0, 0 },
    { 0xca283406, 33 },
    { 0xf1283407, 90 },
    { 0xaf483408, 61 },
    { 0x88e83408, 23 },
    { 0x6b78340a, 52 },
    { 0xc083408, 23 },
    { 0xd7d8340a, 98 },
    { 0x55c8340c, 33 },
    { 0x4cd8340e, 98 },
    { 0xbdd8340c, 87 },
    { 0xc168340e, 90 },
    { 0x42983406, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c83415, 23 },
    { 0x52d83415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64083426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed88342a, 61 },
    { 0xb308342b, 98 },
    { 0xffa8342a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef8342f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8983435, 90 },
    { 0, 0 },
    { 0x3fb83437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d68343b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc748343f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b83443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4183447, 95 },
    { 0xb5983447, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8344d, 90 },
    { 0xb298344d, 98 },
    { 0x1c48344f, 61 },
    { 0x82a8344f, 77 },
    { 0x8278344f, 95 },
    { 0x94b83452, 87 },
    { 0xdec83452, 23 },
    { 0, 0 },
    { 0x2aa83455, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17483459, 84 },
    { 0, 0 },
    { 0x8ab8345b, 43 },
    { 0x1918345c, 90 },
    { 0, 0 },
    { 0x5168345e, 98 },
    { 0xd208345f, 55 },
    { 0x9a383460, 90 },
    { 0xd7d83461, 61 },
    { 0x2e48345e, 23 },
    { 0, 0 },
    { 0xfc283464, 23 },
    { 0x1db83465, 23 },
    { 0x82383465, 90 },
    { 0x46283465, 34 },
    { 0, 0 },
    { 0x30583469, 33 },
    { 0xabd83469, 33 },
    { 0, 0 },
    { 0x1ec8346c, 76 },
    { 0x4998346d, 77 },
    { 0x5698346e, 95 },
    { 0, 0 },
    { 0x77683470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8083473, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d183476, 68 },
    { 0xf3e83477, 98 },
    { 0x40e83478, 90 },
    { 0xa9983479, 88 },
    { 0x31483479, 81 },
    { 0, 0 },
    { 0x5708347c, 100 },
    { 0xb298347d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc83480, 99 },
    { 0xedc83481, 52 },
    { 0xb0e83482, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e783485, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4718348a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528348f, 64 },
    { 0xd4b83490, 23 },
    { 0xc7583490, 23 },
    { 0x52d83490, 83 },
    { 0, 0 },
    { 0xd3c83494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae83497, 33 },
    { 0xf6883498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1818349e, 64 },
    { 0x9f28349f, 46 },
    { 0x20a8349e, 33 },
    { 0xb048349e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c834a8, 46 },
    { 0, 0 },
    { 0x7bc834aa, 95 },
    { 0x4b9834ab, 90 },
    { 0xe95834ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e834af, 87 },
    { 0x855834b0, 90 },
    { 0xc5a834b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255834b5, 46 },
    { 0x47e834b6, 34 },
    { 0xff9834b6, 23 },
    { 0xab2834b8, 40 },
    { 0x360834b9, 43 },
    { 0xc32834b7, 99 },
    { 0, 0 },
    { 0x41b834bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95834c6, 33 },
    { 0xc62834c6, 55 },
    { 0x624834c8, 93 },
    { 0, 0 },
    { 0x68834ca, 98 },
    { 0x2ed834cb, 88 },
    { 0xef0834cc, 58 },
    { 0x40f834cc, 61 },
    { 0xc93834cb, 33 },
    { 0xa8c834cd, 61 },
    { 0x687834cd, 33 },
    { 0xbee834ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c834db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0834df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef834e4, 99 },
    { 0, 0 },
    { 0x42834e6, 90 },
    { 0x60a834e7, 90 },
    { 0x5b4834e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa834eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd834f0, 99 },
    { 0x337834f1, 33 },
    { 0xa08834f2, 90 },
    { 0x504834f2, 95 },
    { 0xb21834f4, 37 },
    { 0xcc0834f5, 55 },
    { 0xe58834f1, 33 },
    { 0, 0 },
    { 0x4dc834f8, 23 },
    { 0x8e6834f8, 55 },
    { 0xffd834f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c834fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd83501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83509, 52 },
    { 0xb5d8350a, 55 },
    { 0x3928350b, 90 },
    { 0x3e28350c, 88 },
    { 0xb688350c, 23 },
    { 0xe638350d, 23 },
    { 0x8ab8350c, 98 },
    { 0xa688350f, 61 },
    { 0x44183509, 34 },
    { 0x4483511, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c183515, 43 },
    { 0, 0 },
    { 0x7da83517, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb8351b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888351e, 33 },
    { 0xfb28351f, 99 },
    { 0, 0 },
    { 0x41f83521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb83524, 33 },
    { 0xbce83524, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f8352a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef8352d, 33 },
    { 0x6fb8352e, 50 },
    { 0x8888352f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58983533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xda483536, 98 },
    { 0x69d83537, 23 },
    { 0xeda83538, 51 },
    { 0, 0 },
    { 0x78353a, 90 },
    { 0xde08353a, 42 },
    { 0, 0 },
    { 0x8ae8353d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3283545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc8354a, 95 },
    { 0xbc68354b, 23 },
    { 0x6288354c, 85 },
    { 0x2f38354d, 76 },
    { 0, 0 },
    { 0x86c8354f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27183554, 90 },
    { 0, 0 },
    { 0xc7583556, 76 },
    { 0x52383557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e8355e, 99 },
    { 0x8428355e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa83563, 82 },
    { 0x33483564, 87 },
    { 0x88a83563, 90 },
    { 0x67283566, 90 },
    { 0x37f83566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc8356a, 33 },
    { 0, 0 },
    { 0xb058356c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x168356f, 52 },
    { 0x22f8356f, 76 },
    { 0, 0 },
    { 0x28383572, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e83577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab78357b, 66 },
    { 0x5898357c, 33 },
    { 0x49b8357d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0083580, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b83585, 100 },
    { 0, 0 },
    { 0x98083587, 77 },
    { 0x90e83587, 100 },
    { 0xc1c83589, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8508358c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e583592, 33 },
    { 0, 0 },
    { 0x7b83594, 95 },
    { 0, 0 },
    { 0xf5083596, 85 },
    { 0xcde83596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede8359e, 23 },
    { 0x2b28359e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05835a3, 95 },
    { 0xb3c835a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1835ae, 43 },
    { 0x4f1835af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71835b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345835b8, 76 },
    { 0, 0 },
    { 0x14e835ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c835c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21835c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533835cd, 95 },
    { 0xd64835ce, 61 },
    { 0xbf1835cd, 34 },
    { 0x280835d0, 98 },
    { 0, 0 },
    { 0x4f9835d2, 23 },
    { 0xeec835d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5835e0, 90 },
    { 0xed2835e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48835e7, 64 },
    { 0xe7d835e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8835ef, 33 },
    { 0x777835ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x209835f3, 33 },
    { 0xb7c835f4, 76 },
    { 0xa84835f4, 91 },
    { 0xd6f835f4, 91 },
    { 0xf08835f3, 33 },
    { 0xdd8835f8, 33 },
    { 0, 0 },
    { 0xb8835fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c835fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd83602, 34 },
    { 0, 0 },
    { 0x2fd83604, 76 },
    { 0x7ec83604, 61 },
    { 0x683604, 33 },
    { 0xa2d83607, 80 },
    { 0, 0 },
    { 0x3a283609, 61 },
    { 0xb9e8360a, 90 },
    { 0x6e8360b, 93 },
    { 0x1038360b, 33 },
    { 0x4e18360d, 98 },
    { 0, 0 },
    { 0x3318360f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a183612, 90 },
    { 0xf6e83612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1783616, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e8361a, 95 },
    { 0x9ff8361b, 34 },
    { 0xe3f8361c, 23 },
    { 0x59b8361d, 43 },
    { 0x71d8361e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6883621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92083627, 100 },
    { 0, 0 },
    { 0x48683629, 67 },
    { 0x4f28362a, 46 },
    { 0x7ee83629, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad8362f, 75 },
    { 0x3d083630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d83637, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8628363c, 74 },
    { 0x2a48363d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab83643, 43 },
    { 0xd8e83644, 34 },
    { 0xd1a83644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638364c, 98 },
    { 0xb438364d, 95 },
    { 0xe5a8364d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1383653, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f83659, 76 },
    { 0x208365a, 99 },
    { 0, 0 },
    { 0x9008365c, 77 },
    { 0xaff8365c, 52 },
    { 0x6fd8365e, 46 },
    { 0x79f8365d, 90 },
    { 0x5248365d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5383664, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283668, 98 },
    { 0, 0 },
    { 0x9378366a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d83672, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583679, 23 },
    { 0xb1183679, 81 },
    { 0xa018367a, 33 },
    { 0, 0 },
    { 0x5848367d, 95 },
    { 0xe858367d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde283683, 33 },
    { 0x5083684, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d8368f, 98 },
    { 0x9d183690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9383694, 61 },
    { 0xaaa83695, 33 },
    { 0x78183694, 95 },
    { 0xc8183697, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d8369f, 76 },
    { 0, 0 },
    { 0xee2836a1, 23 },
    { 0xb7e836a2, 91 },
    { 0x264836a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c836a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38836ac, 99 },
    { 0x783836ad, 27 },
    { 0x312836ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2836b5, 95 },
    { 0x8c3836b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x918836b9, 61 },
    { 0, 0 },
    { 0xb61836bb, 19 },
    { 0x3b5836bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62836c0, 61 },
    { 0x6d0836c0, 85 },
    { 0xe6e836c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426836c6, 43 },
    { 0, 0 },
    { 0x3c8836c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9836cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea836d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8836d3, 61 },
    { 0xf99836d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf836d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87836da, 61 },
    { 0, 0 },
    { 0x992836dc, 90 },
    { 0x4dd836dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4836e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894836e5, 33 },
    { 0, 0 },
    { 0xa5b836e7, 90 },
    { 0xf39836e7, 95 },
    { 0xaf6836e7, 100 },
    { 0x156836ea, 98 },
    { 0xd55836ea, 76 },
    { 0xc9d836e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd836f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7836f7, 61 },
    { 0, 0 },
    { 0xc39836f9, 90 },
    { 0x293836fa, 23 },
    { 0, 0 },
    { 0xa70836fc, 33 },
    { 0x810836fd, 61 },
    { 0xce5836fc, 38 },
    { 0x193836fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87483703, 52 },
    { 0x94c83704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8370b, 33 },
    { 0, 0 },
    { 0x7c48370d, 76 },
    { 0, 0 },
    { 0xf868370f, 34 },
    { 0x23c8370f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa983715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd8371e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36183721, 76 },
    { 0x4e183721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e483731, 92 },
    { 0, 0 },
    { 0xd9c83733, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1978373a, 95 },
    { 0x32a8373b, 34 },
    { 0x2c68373c, 33 },
    { 0xdcc8373c, 85 },
    { 0x4248373e, 33 },
    { 0x2408373f, 99 },
    { 0, 0 },
    { 0x6fe83741, 90 },
    { 0, 0 },
    { 0xab83743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c583747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78374a, 53 },
    { 0x7b58374b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb883750, 78 },
    { 0xc3f83750, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d083756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef48375b, 90 },
    { 0x85a8375b, 90 },
    { 0xc258375b, 95 },
    { 0x76e8375e, 52 },
    { 0xc418375f, 81 },
    { 0x5df83760, 2 },
    { 0xcf883760, 46 },
    { 0xc0c83760, 93 },
    { 0xb3a8375c, 76 },
    { 0x19383764, 77 },
    { 0xc6883765, 90 },
    { 0x97283764, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478376b, 98 },
    { 0xe448376c, 61 },
    { 0xb658376d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f383771, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4483776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f8377a, 95 },
    { 0x4a18377b, 61 },
    { 0x9908377c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c83783, 90 },
    { 0xf3983784, 90 },
    { 0xd5783785, 33 },
    { 0xed083784, 66 },
    { 0xc1f83784, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc48378a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38378d, 90 },
    { 0x82d8378e, 98 },
    { 0, 0 },
    { 0xef483790, 23 },
    { 0x9c83791, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b83798, 90 },
    { 0, 0 },
    { 0x8e38379a, 55 },
    { 0, 0 },
    { 0x4b08379c, 33 },
    { 0x66a8379d, 90 },
    { 0x8be8379d, 23 },
    { 0x55b8379d, 43 },
    { 0xe21837a0, 75 },
    { 0x2ab837a1, 23 },
    { 0x689837a2, 75 },
    { 0, 0 },
    { 0xb71837a4, 75 },
    { 0xde1837a5, 33 },
    { 0xf59837a5, 100 },
    { 0x1a1837a4, 85 },
    { 0x357837a8, 68 },
    { 0xa3a837a7, 33 },
    { 0x581837aa, 95 },
    { 0x97837aa, 61 },
    { 0x8e837ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0837af, 33 },
    { 0x574837b0, 61 },
    { 0x745837b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f837b5, 33 },
    { 0x4dd837b5, 95 },
    { 0x936837b7, 68 },
    { 0x649837b8, 23 },
    { 0x1ce837b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1837c1, 17 },
    { 0x36b837c2, 66 },
    { 0, 0 },
    { 0x820837c4, 99 },
    { 0, 0 },
    { 0xef4837c6, 84 },
    { 0x9e9837c7, 61 },
    { 0x434837c8, 65 },
    { 0x8fb837c8, 43 },
    { 0, 0 },
    { 0xd6c837cb, 33 },
    { 0xd1c837cc, 74 },
    { 0xde6837cd, 85 },
    { 0x1b8837ce, 61 },
    { 0x7a6837cf, 101 },
    { 0xdd8837ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e837d3, 61 },
    { 0xd2f837d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4837da, 76 },
    { 0x3e3837db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf837e5, 61 },
    { 0xab1837e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c837ea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941837f0, 33 },
    { 0x616837f0, 84 },
    { 0xa7f837f1, 80 },
    { 0xd6e837f3, 76 },
    { 0xb63837f4, 55 },
    { 0xd86837f3, 23 },
    { 0x1f8837f4, 95 },
    { 0xe5d837f7, 90 },
    { 0x2a8837f8, 23 },
    { 0xf10837f7, 95 },
    { 0x9f1837f9, 23 },
    { 0xf26837f8, 55 },
    { 0x178837f3, 99 },
    { 0xc9c837fc, 84 },
    { 0, 0 },
    { 0x3f6837ff, 85 },
    { 0, 0 },
    { 0xcc683801, 61 },
    { 0, 0 },
    { 0x1a683803, 91 },
    { 0, 0 },
    { 0x2a883805, 98 },
    { 0, 0 },
    { 0x2e883807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da83810, 76 },
    { 0xf3483811, 96 },
    { 0xcd383812, 33 },
    { 0x7fb83810, 99 },
    { 0xab83814, 33 },
    { 0x8fd83815, 23 },
    { 0xddb83815, 34 },
    { 0, 0 },
    { 0x61883818, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77083822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef383827, 98 },
    { 0, 0 },
    { 0xe4f83829, 65 },
    { 0, 0 },
    { 0x3ea8382b, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9883832, 93 },
    { 0x41d83833, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b83837, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8383c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa608383f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20983845, 90 },
    { 0x73b83846, 91 },
    { 0x63183847, 90 },
    { 0x9ef83847, 90 },
    { 0x6e483847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8384d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3783852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a583855, 33 },
    { 0x62c83856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb383859, 90 },
    { 0, 0 },
    { 0x9818385b, 98 },
    { 0xf488385c, 46 },
    { 0x9bf8385b, 23 },
    { 0x33c8385e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e83861, 85 },
    { 0x7a183861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad83867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d8386d, 61 },
    { 0xc4a8386e, 33 },
    { 0x9f48386d, 34 },
    { 0x62e83870, 88 },
    { 0x5028386d, 76 },
    { 0xff18386f, 30 },
    { 0x14083873, 43 },
    { 0, 0 },
    { 0xcdc83875, 33 },
    { 0xc6d83876, 78 },
    { 0x55683877, 98 },
    { 0x31d83875, 90 },
    { 0xa8c83876, 90 },
    { 0x683879, 46 },
    { 0, 0 },
    { 0xc4f8387c, 90 },
    { 0, 0 },
    { 0x1198387e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56783889, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483895, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b83898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5838a0, 33 },
    { 0, 0 },
    { 0xb94838a2, 98 },
    { 0x657838a2, 61 },
    { 0, 0 },
    { 0x5e4838a5, 98 },
    { 0xc21838a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1838af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x884838b2, 55 },
    { 0, 0 },
    { 0xe4d838b4, 59 },
    { 0, 0 },
    { 0x1f3838b6, 17 },
    { 0x59a838b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3838bd, 95 },
    { 0xf8a838bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e838c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b838d4, 33 },
    { 0xf63838d5, 61 },
    { 0x3a4838d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32838de, 46 },
    { 0, 0 },
    { 0x445838e0, 61 },
    { 0xcc5838e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82838e5, 85 },
    { 0, 0 },
    { 0x5bd838e7, 61 },
    { 0xa8b838e8, 66 },
    { 0x6fa838e8, 90 },
    { 0, 0 },
    { 0x3d5838eb, 33 },
    { 0, 0 },
    { 0x42f838ed, 76 },
    { 0x31d838ee, 55 },
    { 0xf28838ef, 76 },
    { 0x52c838ef, 23 },
    { 0, 0 },
    { 0xf01838f2, 93 },
    { 0x1a6838f3, 61 },
    { 0, 0 },
    { 0xbb0838f5, 93 },
    { 0x19f838f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f838fc, 95 },
    { 0xf6f838fc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec583902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b583906, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8390c, 95 },
    { 0, 0 },
    { 0xcf08390e, 33 },
    { 0xc4b8390f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc83912, 95 },
    { 0, 0 },
    { 0xa9583914, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8983917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8228391b, 88 },
    { 0x4578391b, 90 },
    { 0, 0 },
    { 0xc768391e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b83922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d8392a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba18392f, 76 },
    { 0, 0 },
    { 0x4f583931, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a83935, 98 },
    { 0xabc83935, 83 },
    { 0, 0 },
    { 0x5a883938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18393b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e8393f, 52 },
    { 0, 0 },
    { 0x57f83941, 98 },
    { 0xf5a83942, 76 },
    { 0xc2283943, 61 },
    { 0xca783943, 76 },
    { 0xc4183944, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c8394d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c883950, 90 },
    { 0x23683950, 37 },
    { 0, 0 },
    { 0x2ff83953, 55 },
    { 0xd2a83954, 38 },
    { 0x67883954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4883959, 100 },
    { 0x9258395a, 61 },
    { 0, 0 },
    { 0xb488395c, 95 },
    { 0xafb8395d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad83964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3983967, 61 },
    { 0, 0 },
    { 0xdbb83969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb828396c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe788396f, 51 },
    { 0, 0 },
    { 0x83683971, 61 },
    { 0, 0 },
    { 0xe0583973, 66 },
    { 0, 0 },
    { 0xff383975, 46 },
    { 0xe8a83975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808397a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab68397e, 98 },
    { 0x5808397e, 90 },
    { 0xfa48397e, 33 },
    { 0, 0 },
    { 0xfd683982, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa83985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0783995, 61 },
    { 0, 0 },
    { 0xf0e83997, 23 },
    { 0x44b83998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1378399d, 93 },
    { 0xded8399d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a839a1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43839a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2839ac, 95 },
    { 0x84c839ad, 76 },
    { 0xc2a839ae, 33 },
    { 0x87f839af, 52 },
    { 0x9e3839b0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0839bc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6839c7, 61 },
    { 0, 0 },
    { 0xcc0839c9, 34 },
    { 0x131839ca, 95 },
    { 0xa75839c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5839ce, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea839d3, 33 },
    { 0xd0a839d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb839d7, 76 },
    { 0x72d839d8, 95 },
    { 0xe8b839d7, 61 },
    { 0, 0 },
    { 0x714839db, 55 },
    { 0, 0 },
    { 0xcf839dd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0839e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53839ee, 81 },
    { 0, 0 },
    { 0x33839f0, 90 },
    { 0x14839f1, 76 },
    { 0, 0 },
    { 0xd01839f3, 43 },
    { 0, 0 },
    { 0x79839f5, 95 },
    { 0xf81839f5, 61 },
    { 0xc1b839f6, 91 },
    { 0, 0 },
    { 0xc73839f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe983a00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5683a09, 87 },
    { 0x11a83a0a, 23 },
    { 0x26e83a0b, 76 },
    { 0x40d83a0b, 43 },
    { 0xa9d83a0d, 99 },
    { 0x31f83a0e, 33 },
    { 0x13c83a0f, 87 },
    { 0xc6383a10, 23 },
    { 0x7a383a11, 43 },
    { 0x4483a10, 61 },
    { 0, 0 },
    { 0x95a83a14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f83a18, 91 },
    { 0, 0 },
    { 0xecd83a1a, 85 },
    { 0x10483a1a, 90 },
    { 0xe6683a1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc083a26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8883a29, 33 },
    { 0x55883a2a, 87 },
    { 0x72b83a2b, 23 },
    { 0, 0 },
    { 0x48283a2d, 76 },
    { 0, 0 },
    { 0x3d783a2f, 85 },
    { 0x85a83a2f, 46 },
    { 0, 0 },
    { 0x9a983a32, 33 },
    { 0xbf283a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53183a38, 34 },
    { 0, 0 },
    { 0x9fc83a3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe883a3d, 90 },
    { 0, 0 },
    { 0xa4083a3f, 23 },
    { 0xacf83a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15283a44, 90 },
    { 0, 0 },
    { 0x32083a46, 55 },
    { 0x1ca83a47, 23 },
    { 0x25f83a47, 33 },
    { 0x97083a49, 23 },
    { 0x50d83a46, 90 },
    { 0x6d783a48, 95 },
    { 0, 0 },
    { 0x7fe83a4d, 76 },
    { 0xfc183a4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0283a51, 98 },
    { 0x63d83a51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa83a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec83a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13483a60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d83a65, 23 },
    { 0x33583a66, 78 },
    { 0x34283a66, 43 },
    { 0x8583a67, 95 },
    { 0xc2a83a65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27783a6f, 76 },
    { 0x74183a70, 33 },
    { 0xbde83a70, 69 },
    { 0xcc283a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2983a77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b83a7e, 43 },
    { 0x13283a7f, 55 },
    { 0, 0 },
    { 0xad783a81, 94 },
    { 0, 0 },
    { 0xfa583a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f983a87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3283a8a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b83a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d383a91, 23 },
    { 0, 0 },
    { 0xf7283a93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac83a98, 33 },
    { 0x31183a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe283aa2, 88 },
    { 0x29383aa3, 55 },
    { 0, 0 },
    { 0xe9c83aa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c83aa9, 56 },
    { 0xf983aaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b283aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff783ab0, 76 },
    { 0xff483ab1, 61 },
    { 0x78783ab1, 23 },
    { 0, 0 },
    { 0xfe283ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0383abe, 17 },
    { 0xd0783abf, 23 },
    { 0x63b83abf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b83ac9, 95 },
    { 0x7d483aca, 81 },
    { 0x8bb83acb, 61 },
    { 0xd6f83aca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae83acf, 17 },
    { 0xd2a83ad0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c483ad4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97983adc, 90 },
    { 0x38a83adc, 55 },
    { 0x84e83ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab83ae2, 76 },
    { 0xa8983ae3, 61 },
    { 0x3cc83ae3, 33 },
    { 0x95983ae4, 34 },
    { 0, 0 },
    { 0xdba83ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70583aea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc483aef, 23 },
    { 0x1d783aef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe83af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af83af7, 90 },
    { 0x19d83af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc83afc, 61 },
    { 0xa2d83afd, 61 },
    { 0, 0 },
    { 0xcce83aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d83b02, 95 },
    { 0x1183b03, 87 },
    { 0x4fa83b04, 33 },
    { 0x6fa83b03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3883b09, 95 },
    { 0xee783b0a, 88 },
    { 0xc1983b0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3283b0e, 52 },
    { 0x96183b0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc83b12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff83b15, 66 },
    { 0x4a083b16, 17 },
    { 0xc9a83b17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e83b20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d83b25, 98 },
    { 0xe0983b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e383b2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b83b30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d983b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab83b3d, 23 },
    { 0x1a783b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde683b43, 95 },
    { 0, 0 },
    { 0xad383b45, 92 },
    { 0, 0 },
    { 0xad983b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6483b4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11183b50, 87 },
    { 0, 0 },
    { 0x7f983b52, 55 },
    { 0x57683b53, 66 },
    { 0x7d83b54, 85 },
    { 0x90483b54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52083b58, 98 },
    { 0x45983b59, 36 },
    { 0x4d83b5a, 67 },
    { 0, 0 },
    { 0x2b183b5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c83b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af83b65, 93 },
    { 0xa6783b66, 61 },
    { 0xc5683b65, 43 },
    { 0x1fb83b65, 90 },
    { 0xa6583b69, 61 },
    { 0xc6983b6a, 34 },
    { 0xe6e83b6b, 98 },
    { 0x3ca83b6c, 88 },
    { 0x8da83b6b, 90 },
    { 0x64d83b6e, 76 },
    { 0xd5883b6f, 46 },
    { 0, 0 },
    { 0x62283b71, 61 },
    { 0, 0 },
    { 0xf9b83b73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59383b78, 33 },
    { 0x97e83b78, 61 },
    { 0xc1883b7a, 33 },
    { 0x1f83b7a, 95 },
    { 0, 0 },
    { 0x71e83b7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34583b81, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc83b88, 95 },
    { 0xe4983b89, 23 },
    { 0, 0 },
    { 0xdff83b8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c83b91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37283b94, 95 },
    { 0, 0 },
    { 0x33683b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc83ba0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92383ba4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a83ba7, 23 },
    { 0xbc283ba8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0883bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a583bb2, 76 },
    { 0x71983bb3, 98 },
    { 0x8db83bb4, 84 },
    { 0x8c083bb3, 55 },
    { 0xbfb83bb4, 76 },
    { 0x5a883bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e383bba, 61 },
    { 0x82d83bbb, 53 },
    { 0x24683bbb, 76 },
    { 0xcf883bbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57783bc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d583bc6, 61 },
    { 0xf8383bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec083bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27983bd8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb483bdd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d83be1, 65 },
    { 0x8fa83be1, 90 },
    { 0x38083be3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b83be8, 46 },
    { 0x5da83be9, 34 },
    { 0x95d83bea, 90 },
    { 0xaa883beb, 43 },
    { 0xc0b83bec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x57883bef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x93583bf2, 91 },
    { 0x1ce83bf2, 61 },
    { 0x9fb83bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff83bfd, 17 },
    { 0, 0 },
    { 0xac783bff, 98 },
    { 0x13683c00, 67 },
    { 0x28d83c01, 34 },
    { 0, 0 },
    { 0xe2483c03, 40 },
    { 0xa5983c04, 33 },
    { 0xf2783c05, 76 },
    { 0, 0 },
    { 0x24483c07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5183c13, 98 },
    { 0xa3883c14, 46 },
    { 0x93c83c13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a783c18, 61 },
    { 0xae483c19, 98 },
    { 0, 0 },
    { 0xcf883c1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81783c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c83c2e, 81 },
    { 0x4e383c2f, 34 },
    { 0xab583c2f, 43 },
    { 0xe9183c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c83c34, 90 },
    { 0, 0 },
    { 0xe4683c36, 88 },
    { 0x9e583c37, 52 },
    { 0x75083c38, 34 },
    { 0xa3483c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff983c40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x94083c43, 23 },
    { 0xa0e83c43, 61 },
    { 0, 0 },
    { 0x92e83c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b83c4a, 55 },
    { 0, 0 },
    { 0xd1083c4c, 95 },
    { 0xaf683c4d, 90 },
    { 0x56983c4e, 61 },
    { 0xbee83c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64883c57, 95 },
    { 0xaed83c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34883c5b, 98 },
    { 0x39483c5c, 33 },
    { 0x8dd83c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd83c60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c83c6a, 33 },
    { 0xa4383c6b, 98 },
    { 0xbc583c6c, 90 },
    { 0xd5e83c6a, 90 },
    { 0, 0 },
    { 0x4c383c6f, 34 },
    { 0xcac83c70, 76 },
    { 0xb2283c71, 76 },
    { 0x24183c70, 69 },
    { 0x1a883c6f, 34 },
    { 0xdb283c72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e83c7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e83c82, 33 },
    { 0x96383c83, 95 },
    { 0x50e83c84, 98 },
    { 0x8583c85, 100 },
    { 0, 0 },
    { 0xb5283c87, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60983c8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad83c8f, 100 },
    { 0, 0 },
    { 0x9f383c91, 99 },
    { 0x55b83c92, 61 },
    { 0x81d83c91, 98 },
    { 0, 0 },
    { 0x2e883c95, 55 },
    { 0x51883c96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6483c9b, 90 },
    { 0, 0 },
    { 0x7cc83c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81083ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee883ca6, 100 },
    { 0x96083ca7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a83cae, 88 },
    { 0xa83cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb83cb5, 95 },
    { 0xeb583cb6, 55 },
    { 0x4d883cb5, 98 },
    { 0xeb883cb5, 33 },
    { 0, 0 },
    { 0xd6883cba, 33 },
    { 0, 0 },
    { 0xc5383cbc, 76 },
    { 0x3af83cbd, 93 },
    { 0xb7c83cbd, 98 },
    { 0x97783cbf, 33 },
    { 0x5bf83cbe, 55 },
    { 0x583cbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe183cc4, 33 },
    { 0x46283cc5, 90 },
    { 0x2a783cc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d783cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57483cd6, 78 },
    { 0x81883cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b83cda, 43 },
    { 0xe2483cdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae183cde, 34 },
    { 0xe4b83cdf, 91 },
    { 0, 0 },
    { 0xe3583ce1, 33 },
    { 0xab883ce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0383ce7, 95 },
    { 0x9d583ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83ceb, 33 },
    { 0xe7283cec, 98 },
    { 0x13d83cec, 23 },
    { 0x77e83cec, 76 },
    { 0x42083cef, 78 },
    { 0x9e83cef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da83cf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f683cf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4683cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b83cff, 46 },
    { 0x7bf83cff, 46 },
    { 0, 0 },
    { 0xe8d83d02, 100 },
    { 0x27983d03, 98 },
    { 0xffa83d02, 87 },
    { 0x7e283d03, 33 },
    { 0xbd283d02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc83d09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96783d0c, 95 },
    { 0, 0 },
    { 0x13783d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30283d14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xad183d17, 23 },
    { 0xf2a83d18, 23 },
    { 0x20d83d19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x78783d1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11183d1f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6483d22, 95 },
    { 0x85d83d23, 75 },
    { 0x70b83d23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d83d2c, 61 },
    { 0x25583d2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b883d30, 90 },
    { 0, 0 },
    { 0x39f83d32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x883d35, 52 },
    { 0, 0 },
    { 0x40483d37, 98 },
    { 0xd0183d37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f83d46, 91 },
    { 0xed683d47, 99 },
    { 0x21383d48, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1483d4e, 61 },
    { 0xe4e83d4f, 80 },
    { 0, 0 },
    { 0x68583d51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e683d55, 33 },
    { 0xb1f83d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2483d5a, 98 },
    { 0x89c83d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff83d5e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad83d61, 93 },
    { 0, 0 },
    { 0x76283d63, 99 },
    { 0xa4583d63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d83d67, 34 },
    { 0xd7483d68, 61 },
    { 0, 0 },
    { 0xbce83d6a, 68 },
    { 0xc5583d6b, 99 },
    { 0, 0 },
    { 0x7e183d6d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9783d70, 66 },
    { 0xb4583d71, 100 },
    { 0xd8083d72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec83d76, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d7a, 23 },
    { 0xf8083d7a, 77 },
    { 0xc8f83d7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c83d7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41583d83, 61 },
    { 0x26a83d84, 93 },
    { 0, 0 },
    { 0xafc83d86, 55 },
    { 0x8ba83d87, 80 },
    { 0x92383d88, 84 },
    { 0xdb583d89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf83d8d, 43 },
    { 0xee83d8e, 55 },
    { 0x29f83d8d, 98 },
    { 0xc0f83d8d, 61 },
    { 0, 0 },
    { 0x6783d92, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x21183d95, 40 },
    { 0, 0 },
    { 0x51483d97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15083d9c, 23 },
    { 0x25e83d9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65383da0, 61 },
    { 0xb883da1, 100 },
    { 0xf0583da0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d83da5, 33 },
    { 0x80e83da6, 91 },
    { 0xeb983da7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e83daf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75383db9, 61 },
    { 0xff083dba, 23 },
    { 0x4c983dbb, 95 },
    { 0xba983dba, 90 },
    { 0x93c83dbd, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0783dc0, 95 },
    { 0x1b483dc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83483dc5, 23 },
    { 0x7f783dc6, 52 },
    { 0x483dc7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e83dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c83dd1, 90 },
    { 0xe4783dd1, 90 },
    { 0xa883dd3, 66 },
    { 0xd2983dd4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f083dd7, 52 },
    { 0xada83dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c83ddd, 90 },
    { 0xece83dde, 33 },
    { 0, 0 },
    { 0x5a583de0, 33 },
    { 0xe9383de1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e83de7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb783dec, 84 },
    { 0x92483dec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b083df4, 91 },
    { 0xd5583df4, 69 },
    { 0xff283df4, 99 },
    { 0xddd83df7, 76 },
    { 0xbd283df7, 61 },
    { 0xa2d83df8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9383dff, 76 },
    { 0xa7f83e00, 90 },
    { 0x21183dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a83e04, 76 },
    { 0x77f83e05, 46 },
    { 0xba383e06, 81 },
    { 0x70183e05, 66 },
    { 0, 0 },
    { 0xf0b83e09, 95 },
    { 0xf6983e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1383e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc383e11, 61 },
    { 0x18483e12, 90 },
    { 0xd2883e13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a83e16, 33 },
    { 0, 0 },
    { 0x83b83e18, 87 },
    { 0, 0 },
    { 0x9f83e1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83e1d, 33 },
    { 0, 0 },
    { 0xfa083e1f, 93 },
    { 0, 0 },
    { 0x56b83e21, 23 },
    { 0x2d083e21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1183e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83e28, 98 },
    { 0x85383e29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f983e2d, 61 },
    { 0xf8683e2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d83e31, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e83e34, 90 },
    { 0xb8983e35, 97 },
    { 0xf0a83e35, 55 },
    { 0xbb183e36, 23 },
    { 0, 0 },
    { 0xb7083e39, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283e42, 98 },
    { 0x53883e42, 55 },
    { 0x18083e44, 46 },
    { 0x34f83e45, 93 },
    { 0x66083e44, 61 },
    { 0x41d83e47, 40 },
    { 0x83283e48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c83e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13883e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83e58, 33 },
    { 0x9183e59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d83e5e, 66 },
    { 0x21483e5f, 33 },
    { 0x26b83e5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a83e64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba783e68, 66 },
    { 0x3ff83e69, 33 },
    { 0x46a83e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7083e6e, 61 },
    { 0, 0 },
    { 0xdb883e70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b483e77, 87 },
    { 0, 0 },
    { 0x23183e79, 84 },
    { 0, 0 },
    { 0x983e7b, 23 },
    { 0x41c83e7c, 98 },
    { 0xcd083e7c, 55 },
    { 0, 0 },
    { 0x6e083e7f, 65 },
    { 0, 0 },
    { 0x64083e81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb83e86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb583e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0883e95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b783e98, 81 },
    { 0xaa383e99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22083e9f, 23 },
    { 0x5fb83e9f, 33 },
    { 0xd4d83ea0, 55 },
    { 0x38783ea1, 33 },
    { 0xf5583ea1, 95 },
    { 0, 0 },
    { 0xc3083ea5, 33 },
    { 0, 0 },
    { 0x70583ea7, 98 },
    { 0x1ce83ea7, 61 },
    { 0xdb183ea9, 95 },
    { 0xf3583eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f83eaf, 38 },
    { 0, 0 },
    { 0x4ff83eb1, 78 },
    { 0, 0 },
    { 0x1a983eb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e83eb6, 60 },
    { 0xac383eb7, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2383ec1, 98 },
    { 0, 0 },
    { 0xf1883ec3, 61 },
    { 0x13d83ec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c083ecb, 90 },
    { 0xd1583ecc, 93 },
    { 0, 0 },
    { 0x3fb83ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e83ed1, 19 },
    { 0x7c83ed2, 34 },
    { 0x10e83ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483ed8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb783ee2, 100 },
    { 0xc9b83ee3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d83eea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb83eed, 95 },
    { 0xee883eee, 61 },
    { 0xc3483eed, 34 },
    { 0xebf83ef0, 93 },
    { 0xd1d83eee, 96 },
    { 0, 0 },
    { 0x8c483ef3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9583ef8, 84 },
    { 0x65783ef9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83f00, 87 },
    { 0x51283f00, 23 },
    { 0xfa983f02, 40 },
    { 0, 0 },
    { 0x51383f04, 33 },
    { 0x66883f04, 46 },
    { 0xd7a83f05, 65 },
    { 0x5ea83f04, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d83f0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb83f17, 95 },
    { 0, 0 },
    { 0x19583f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d83f1c, 40 },
    { 0x94383f1d, 33 },
    { 0x9ae83f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83f22, 33 },
    { 0x95f83f23, 61 },
    { 0, 0 },
    { 0x2f983f25, 90 },
    { 0x46983f25, 98 },
    { 0x3fc83f27, 17 },
    { 0x16f83f27, 98 },
    { 0x52783f28, 95 },
    { 0x58c83f2a, 43 },
    { 0, 0 },
    { 0x87483f2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883f31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72883f35, 75 },
    { 0x5583f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26483f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae83f3e, 94 },
    { 0, 0 },
    { 0x91083f40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d383f44, 90 },
    { 0, 0 },
    { 0x54383f46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x35983f49, 34 },
    { 0x63083f4a, 33 },
    { 0x3f583f49, 95 },
    { 0, 0 },
    { 0xfe83f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d83f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56483f53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc83f5a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d283f5d, 90 },
    { 0xab383f5d, 90 },
    { 0, 0 },
    { 0x10983f60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3583f64, 17 },
    { 0xdd683f64, 90 },
    { 0x3fa83f66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44383f69, 97 },
    { 0, 0 },
    { 0x13283f6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73183f71, 61 },
    { 0x1dd83f71, 17 },
    { 0x1f283f73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa183f76, 76 },
    { 0x3db83f77, 23 },
    { 0xb3883f77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3983f7c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82883f80, 90 },
    { 0, 0 },
    { 0x4c283f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d83f86, 55 },
    { 0xb3383f87, 84 },
    { 0x77583f86, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a83f8b, 90 },
    { 0, 0 },
    { 0xe4883f8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe83f90, 90 },
    { 0, 0 },
    { 0xd7883f92, 43 },
    { 0xd9e83f93, 52 },
    { 0x24b83f94, 93 },
    { 0xf5983f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34383f9c, 61 },
    { 0, 0 },
    { 0xd3883f9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68283fa6, 88 },
    { 0, 0 },
    { 0x59083fa8, 90 },
    { 0, 0 },
    { 0x88283faa, 66 },
    { 0x6e283faa, 38 },
    { 0xc7d83fac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5483fb0, 33 },
    { 0, 0 },
    { 0x45783fb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c83fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf983fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e83fc0, 64 },
    { 0xe1583fc1, 90 },
    { 0x8a083fc1, 67 },
    { 0, 0 },
    { 0xd283fc4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef83fca, 33 },
    { 0, 0 },
    { 0x81483fcc, 61 },
    { 0x73d83fcd, 33 },
    { 0x85f83fce, 23 },
    { 0x36a83fcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c83fd7, 85 },
    { 0, 0 },
    { 0x21383fd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96083fe0, 43 },
    { 0x1d83fe1, 8 },
    { 0x9e983fe2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80183fe5, 33 },
    { 0x53783fe6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b583fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3183fed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70083ff1, 40 },
    { 0, 0 },
    { 0x84183ff3, 90 },
    { 0, 0 },
    { 0x13e83ff5, 61 },
    { 0x89583ff6, 17 },
    { 0, 0 },
    { 0x52783ff8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba84002, 43 },
    { 0xe4a84003, 33 },
    { 0, 0 },
    { 0x9e784005, 78 },
    { 0x78184005, 23 },
    { 0x7c84007, 52 },
    { 0x81b84008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe58400b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de84011, 95 },
    { 0xee784011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec84015, 99 },
    { 0, 0 },
    { 0x36784017, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a8401e, 33 },
    { 0, 0 },
    { 0x8f984020, 83 },
    { 0, 0 },
    { 0x74d84022, 23 },
    { 0, 0 },
    { 0x52c84024, 61 },
    { 0, 0 },
    { 0xbd84026, 99 },
    { 0x31f84027, 61 },
    { 0, 0 },
    { 0x84029, 90 },
    { 0x17d8402a, 98 },
    { 0x1d68402b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8084031, 98 },
    { 0x70b84031, 98 },
    { 0, 0 },
    { 0xab84034, 76 },
    { 0xdbd84035, 99 },
    { 0x8ff84035, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb318403f, 98 },
    { 0x40a84040, 100 },
    { 0x6de84041, 91 },
    { 0xbc984041, 91 },
    { 0xc2084043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f84047, 95 },
    { 0, 0 },
    { 0xdbe84049, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f88404c, 47 },
    { 0x98b8404d, 90 },
    { 0x7878404e, 99 },
    { 0xc368404d, 61 },
    { 0, 0 },
    { 0x53884051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0084054, 33 },
    { 0x28e84055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c78405b, 100 },
    { 0x2358405c, 66 },
    { 0xacb8405d, 98 },
    { 0, 0 },
    { 0x14b8405f, 55 },
    { 0, 0 },
    { 0x50184061, 90 },
    { 0xc7884062, 95 },
    { 0, 0 },
    { 0x2a784064, 90 },
    { 0, 0 },
    { 0x68a84066, 43 },
    { 0xdf584067, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d8406a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d8406d, 98 },
    { 0xeb38406d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b88407c, 90 },
    { 0x6fc8407d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54984084, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448408a, 61 },
    { 0x7508408b, 34 },
    { 0x6fb8408b, 65 },
    { 0x46d8408d, 87 },
    { 0, 0 },
    { 0xd798408f, 90 },
    { 0x7778408f, 84 },
    { 0x4e284091, 98 },
    { 0, 0 },
    { 0x1284093, 90 },
    { 0x4ee84094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d84098, 23 },
    { 0, 0 },
    { 0x2f38409a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8409d, 55 },
    { 0x74d8409d, 90 },
    { 0x57e8409e, 68 },
    { 0x7b1840a0, 99 },
    { 0x225840a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f840a4, 100 },
    { 0x912840a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b840a8, 100 },
    { 0xa7840a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac840ad, 95 },
    { 0xa20840ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de840b1, 33 },
    { 0xc98840b1, 87 },
    { 0xee1840b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9840bb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9840bf, 55 },
    { 0x6b7840c0, 90 },
    { 0x4ea840c1, 91 },
    { 0x1e8840c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807840c6, 43 },
    { 0x1ad840c7, 91 },
    { 0x73b840c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3840cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d840d5, 69 },
    { 0xe46840d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4840db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e840de, 84 },
    { 0, 0 },
    { 0xe12840e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b840e7, 93 },
    { 0x76d840e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6840ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1840f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x459840f7, 76 },
    { 0x307840f8, 46 },
    { 0, 0 },
    { 0x40b840fa, 33 },
    { 0, 0 },
    { 0xe61840fc, 90 },
    { 0x663840fd, 67 },
    { 0x8c7840fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1484103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2884106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a8410a, 95 },
    { 0x7648410b, 43 },
    { 0x8f18410c, 80 },
    { 0xe6e8410b, 34 },
    { 0, 0 },
    { 0x73c8410f, 90 },
    { 0, 0 },
    { 0x5984111, 98 },
    { 0x88484112, 36 },
    { 0x8d684113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb84116, 98 },
    { 0xc1b84117, 61 },
    { 0xd6584116, 95 },
    { 0x44d84119, 61 },
    { 0xcbb8411a, 90 },
    { 0xffe8411b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288411e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd84123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe284126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2958412c, 23 },
    { 0x2038412d, 90 },
    { 0xb788412c, 95 },
    { 0, 0 },
    { 0x56b84130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41284134, 23 },
    { 0xad784135, 95 },
    { 0, 0 },
    { 0x16184137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe758413a, 90 },
    { 0xf898413b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d84143, 64 },
    { 0x56e84144, 86 },
    { 0x5df84145, 43 },
    { 0xd9484145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b8414b, 59 },
    { 0x3968414c, 95 },
    { 0xf178414d, 93 },
    { 0, 0 },
    { 0xc7b8414f, 23 },
    { 0xaeb84150, 90 },
    { 0, 0 },
    { 0xaa484152, 90 },
    { 0, 0 },
    { 0x1da84154, 23 },
    { 0x6e84154, 55 },
    { 0, 0 },
    { 0xb8c84157, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd848415a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b8415e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd84161, 66 },
    { 0, 0 },
    { 0x55a84163, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25384166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2084173, 93 },
    { 0, 0 },
    { 0x81384175, 34 },
    { 0, 0 },
    { 0xfe184177, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a8417d, 98 },
    { 0xe6d8417e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f84181, 90 },
    { 0x1284181, 95 },
    { 0x25a84183, 46 },
    { 0xaca84184, 61 },
    { 0x83284185, 90 },
    { 0x5cf84184, 76 },
    { 0xa8e84187, 43 },
    { 0x22a84188, 85 },
    { 0x5be84187, 98 },
    { 0xb308418a, 43 },
    { 0x2b584186, 91 },
    { 0x10184182, 95 },
    { 0, 0 },
    { 0x5a88418e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0184192, 90 },
    { 0xa5684192, 77 },
    { 0x46d84194, 43 },
    { 0x88784195, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4584199, 41 },
    { 0, 0 },
    { 0x4e18419b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15841a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9841a5, 95 },
    { 0x542841a6, 78 },
    { 0xa66841a5, 23 },
    { 0, 0 },
    { 0x5ea841a9, 61 },
    { 0x98c841aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03841ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5841ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4841bd, 100 },
    { 0x333841be, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef841c1, 34 },
    { 0x300841c2, 91 },
    { 0, 0 },
    { 0x6eb841c4, 98 },
    { 0, 0 },
    { 0x5ca841c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f841c9, 52 },
    { 0, 0 },
    { 0x9df841cb, 23 },
    { 0x288841cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a841d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811841d5, 66 },
    { 0x824841d5, 34 },
    { 0, 0 },
    { 0x5da841d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b841db, 23 },
    { 0xf92841db, 55 },
    { 0xddd841dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5841e3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5841ea, 33 },
    { 0, 0 },
    { 0x6ef841ec, 96 },
    { 0x7fe841ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb841f0, 33 },
    { 0x571841f1, 98 },
    { 0x322841f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7841f5, 95 },
    { 0, 0 },
    { 0x604841f7, 85 },
    { 0xe6e841f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f841fb, 61 },
    { 0x6e6841fc, 95 },
    { 0, 0 },
    { 0xf95841fe, 90 },
    { 0, 0 },
    { 0x37c84200, 98 },
    { 0, 0 },
    { 0xeef84202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c84205, 91 },
    { 0x63c84205, 76 },
    { 0, 0 },
    { 0x73084208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b8420b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d08420e, 23 },
    { 0x5648420f, 61 },
    { 0, 0 },
    { 0x1ab84211, 33 },
    { 0x6c84212, 95 },
    { 0xf6f84213, 66 },
    { 0x12984211, 85 },
    { 0, 0 },
    { 0x5f784216, 33 },
    { 0xa5184217, 61 },
    { 0x7a84218, 90 },
    { 0xc9c84217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8421d, 91 },
    { 0, 0 },
    { 0x5778421f, 33 },
    { 0x2a584220, 43 },
    { 0, 0 },
    { 0xe4284222, 40 },
    { 0x4d584223, 61 },
    { 0x6e784224, 61 },
    { 0xc184223, 38 },
    { 0xc8c84226, 91 },
    { 0xbd684222, 95 },
    { 0, 0 },
    { 0xa6184229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78422f, 55 },
    { 0x2de8422f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c84240, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a784243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d8424a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf8424f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6384256, 23 },
    { 0xdd384256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f8425c, 34 },
    { 0xf4c8425d, 98 },
    { 0x21e8425e, 99 },
    { 0xa358425d, 90 },
    { 0x10b84260, 90 },
    { 0, 0 },
    { 0x22484262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a84267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98426a, 23 },
    { 0, 0 },
    { 0xfeb8426c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7384273, 95 },
    { 0x7b84274, 85 },
    { 0x1fd84275, 33 },
    { 0, 0 },
    { 0x7984277, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe88427a, 95 },
    { 0xf038427b, 61 },
    { 0, 0 },
    { 0xd8f8427d, 77 },
    { 0x17e8427e, 52 },
    { 0, 0 },
    { 0x4b084280, 43 },
    { 0, 0 },
    { 0xf4a84282, 65 },
    { 0xc984282, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e284287, 78 },
    { 0xb4084288, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb678428e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab84293, 61 },
    { 0xddf84294, 55 },
    { 0xdf184295, 93 },
    { 0xdb384295, 90 },
    { 0xb0284297, 76 },
    { 0x3b984298, 98 },
    { 0x81384299, 67 },
    { 0x8db84295, 61 },
    { 0x1988429b, 90 },
    { 0x6448429b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be8429f, 95 },
    { 0x339842a0, 55 },
    { 0x368842a1, 78 },
    { 0xd4f842a1, 67 },
    { 0x692842a1, 46 },
    { 0x6958429f, 36 },
    { 0, 0 },
    { 0xd5b842a6, 23 },
    { 0, 0 },
    { 0x50842a8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371842ac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42842b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba842b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c842bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c842be, 43 },
    { 0xd79842bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5842c2, 34 },
    { 0, 0 },
    { 0xa44842c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8842c9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e842cd, 90 },
    { 0xc11842cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd842d1, 76 },
    { 0x7bc842d1, 90 },
    { 0x803842d1, 43 },
    { 0x9df842d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda842d8, 98 },
    { 0xe23842d8, 90 },
    { 0xf87842da, 98 },
    { 0x646842db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18842de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6842e1, 61 },
    { 0, 0 },
    { 0x8a1842e3, 78 },
    { 0x29f842e4, 61 },
    { 0x3c2842e4, 33 },
    { 0, 0 },
    { 0x820842e7, 66 },
    { 0xf1d842e7, 81 },
    { 0x9e4842e9, 66 },
    { 0x82e842e7, 23 },
    { 0x2b3842eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb842f2, 64 },
    { 0x981842f2, 78 },
    { 0x28e842f2, 90 },
    { 0x931842f2, 33 },
    { 0xee3842f6, 95 },
    { 0, 0 },
    { 0x32d842f8, 88 },
    { 0xe83842f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363842fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0984300, 43 },
    { 0xc9984301, 90 },
    { 0x35484301, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe8430a, 76 },
    { 0x3438430b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33584312, 61 },
    { 0x17084313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b84318, 98 },
    { 0, 0 },
    { 0xfc58431a, 61 },
    { 0xf998431b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2138431e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b084322, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f84326, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae98432a, 33 },
    { 0x9848432b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa84335, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f38433c, 90 },
    { 0x96b8433c, 80 },
    { 0x8a48433e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91384342, 33 },
    { 0xada84343, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b84346, 95 },
    { 0x9f384347, 33 },
    { 0x85484346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8434c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a984350, 90 },
    { 0, 0 },
    { 0x17d84352, 91 },
    { 0x98784352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec78435a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85084362, 61 },
    { 0xe5384362, 90 },
    { 0, 0 },
    { 0xf2284365, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3278436b, 33 },
    { 0x3978436b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd884372, 39 },
    { 0x3e984373, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb884378, 78 },
    { 0, 0 },
    { 0xf1b8437a, 98 },
    { 0, 0 },
    { 0x5948437c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c684385, 98 },
    { 0xb1184386, 76 },
    { 0x6d784386, 43 },
    { 0x30284385, 90 },
    { 0x7284386, 33 },
    { 0x31d84385, 84 },
    { 0, 0 },
    { 0x64a8438c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0184394, 98 },
    { 0x34f84395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce84398, 33 },
    { 0, 0 },
    { 0x1e8439a, 76 },
    { 0x5e28439a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28439e, 67 },
    { 0, 0 },
    { 0xd8843a0, 33 },
    { 0x697843a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa843a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9843aa, 33 },
    { 0x7d9843ab, 95 },
    { 0xf55843ac, 90 },
    { 0x156843ad, 33 },
    { 0x95d843aa, 23 },
    { 0x52e843af, 90 },
    { 0xb8e843b0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6843b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81843b8, 90 },
    { 0, 0 },
    { 0x461843ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77843bf, 33 },
    { 0x4a7843c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160843c5, 33 },
    { 0, 0 },
    { 0xaed843c7, 61 },
    { 0, 0 },
    { 0x1fc843c9, 67 },
    { 0xe05843c9, 76 },
    { 0xb6a843cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a843cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc843d3, 95 },
    { 0, 0 },
    { 0x43a843d5, 34 },
    { 0x457843d6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9843db, 76 },
    { 0x91b843db, 90 },
    { 0xf3d843dd, 23 },
    { 0x361843db, 55 },
    { 0x505843dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a843e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686843ea, 66 },
    { 0, 0 },
    { 0x589843ec, 43 },
    { 0, 0 },
    { 0xe76843ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa843f2, 98 },
    { 0, 0 },
    { 0xeee843f4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf843f7, 66 },
    { 0, 0 },
    { 0x975843f9, 23 },
    { 0, 0 },
    { 0x6b3843fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc84403, 95 },
    { 0x1b784404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78440b, 95 },
    { 0x87a8440c, 88 },
    { 0xf7d8440d, 87 },
    { 0xc38440e, 95 },
    { 0x4db8440d, 23 },
    { 0x57e8440c, 55 },
    { 0xcae84411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc08441a, 99 },
    { 0, 0 },
    { 0xa2e8441c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c84420, 55 },
    { 0, 0 },
    { 0x38b84422, 76 },
    { 0, 0 },
    { 0x34884424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1284427, 69 },
    { 0xaa784428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6268442d, 33 },
    { 0x118442d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b84433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d84436, 43 },
    { 0xaf584437, 23 },
    { 0xc8084438, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3458443d, 40 },
    { 0x7a18443e, 76 },
    { 0x3b18443e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c84442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17084446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88444a, 40 },
    { 0x2868444b, 99 },
    { 0xad88444c, 52 },
    { 0x8ac8444b, 98 },
    { 0xc108444a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d384451, 17 },
    { 0x17384452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35184455, 91 },
    { 0x7f484456, 78 },
    { 0xc9584455, 95 },
    { 0xd0b84455, 85 },
    { 0x32e84459, 76 },
    { 0xacb84459, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7698445f, 61 },
    { 0x4d984460, 90 },
    { 0x5e084461, 95 },
    { 0x12e8445f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d84465, 84 },
    { 0xc2e84466, 17 },
    { 0, 0 },
    { 0xb3184468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98446d, 99 },
    { 0x56f8446d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c84471, 33 },
    { 0x8bc84472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d084477, 98 },
    { 0xb5f84478, 91 },
    { 0, 0 },
    { 0xd5a8447a, 34 },
    { 0, 0 },
    { 0xf878447c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9988447f, 37 },
    { 0xd3384480, 90 },
    { 0x13284481, 95 },
    { 0x13584480, 90 },
    { 0x59184483, 90 },
    { 0x59684484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b84487, 90 },
    { 0xadd84488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf818448b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a08448f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90184493, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f84497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f98449d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12844a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc844a3, 33 },
    { 0, 0 },
    { 0xc02844a5, 40 },
    { 0x8cc844a6, 61 },
    { 0x4a8844a7, 100 },
    { 0xa76844a8, 40 },
    { 0, 0 },
    { 0xe3c844aa, 99 },
    { 0xc6844ab, 90 },
    { 0, 0 },
    { 0x708844ad, 23 },
    { 0, 0 },
    { 0x792844af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167844b4, 95 },
    { 0, 0 },
    { 0xc5d844b6, 33 },
    { 0x6b9844b7, 84 },
    { 0x5de844b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50844bb, 61 },
    { 0x843844bc, 95 },
    { 0x880844bd, 99 },
    { 0x4bf844bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce844c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x473844c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed844c7, 34 },
    { 0xd5a844c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf844ce, 33 },
    { 0, 0 },
    { 0x586844d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e844d5, 90 },
    { 0x499844d5, 98 },
    { 0x927844d7, 33 },
    { 0x966844d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54844e0, 76 },
    { 0x657844e0, 90 },
    { 0x755844e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d844e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8844e8, 98 },
    { 0xd16844e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b844ec, 90 },
    { 0, 0 },
    { 0x864844ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc844f1, 95 },
    { 0xa83844f2, 33 },
    { 0x487844f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38844f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf844fe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb84508, 52 },
    { 0x89b84508, 84 },
    { 0xca38450a, 23 },
    { 0, 0 },
    { 0x3a38450c, 52 },
    { 0xb08450c, 98 },
    { 0x59c8450c, 61 },
    { 0xa218450f, 90 },
    { 0x1698450e, 33 },
    { 0x8384511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8584516, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1918451c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db84524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd84527, 33 },
    { 0, 0 },
    { 0xdf684529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf8452d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958453b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf8453e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db84545, 85 },
    { 0, 0 },
    { 0xb6184547, 23 },
    { 0xd2184548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71184551, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a84554, 23 },
    { 0x91a84554, 90 },
    { 0x92d84556, 52 },
    { 0x58b84556, 23 },
    { 0x7b184557, 61 },
    { 0xfb084558, 34 },
    { 0x65a84557, 65 },
    { 0xe418455b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb84560, 61 },
    { 0x5e784561, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b584564, 61 },
    { 0xf2c84564, 67 },
    { 0, 0 },
    { 0x2e384567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e48456b, 61 },
    { 0x5c78456c, 90 },
    { 0x8898456b, 98 },
    { 0x5218456e, 90 },
    { 0x83f8456b, 90 },
    { 0x72f84570, 33 },
    { 0x41f8456b, 84 },
    { 0, 0 },
    { 0xcd184573, 55 },
    { 0xec584574, 46 },
    { 0x94b84575, 61 },
    { 0xaad84575, 23 },
    { 0xf4584577, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e28457b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e184582, 78 },
    { 0xb1884583, 95 },
    { 0x6b84582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33584589, 91 },
    { 0x63d84589, 61 },
    { 0xe738458b, 95 },
    { 0xd168458c, 61 },
    { 0xf108458d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c84590, 33 },
    { 0xb3b84591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a84595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47084598, 90 },
    { 0xe9b84599, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d845a1, 98 },
    { 0x877845a2, 93 },
    { 0x561845a3, 93 },
    { 0, 0 },
    { 0x462845a5, 61 },
    { 0x149845a6, 76 },
    { 0, 0 },
    { 0xd80845a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8845ac, 46 },
    { 0xa6c845ad, 90 },
    { 0, 0 },
    { 0x6bc845af, 61 },
    { 0, 0 },
    { 0x13b845b1, 90 },
    { 0x725845b2, 33 },
    { 0x279845b3, 90 },
    { 0, 0 },
    { 0xa5a845b5, 75 },
    { 0x652845b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9845ba, 34 },
    { 0x394845bb, 83 },
    { 0xa8b845bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd845c1, 61 },
    { 0xda5845c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734845c7, 23 },
    { 0x417845c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db845cc, 74 },
    { 0xaca845cc, 95 },
    { 0, 0 },
    { 0xc7b845cf, 23 },
    { 0xb11845cf, 100 },
    { 0xf7c845d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515845d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf845dc, 34 },
    { 0x8ce845dd, 90 },
    { 0xb97845de, 98 },
    { 0x810845df, 98 },
    { 0x5fe845e0, 98 },
    { 0x2cc845e0, 99 },
    { 0xa2d845dd, 85 },
    { 0x89845e0, 34 },
    { 0x891845e4, 33 },
    { 0x92b845e0, 23 },
    { 0x6a8845dd, 33 },
    { 0x3eb845e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16845ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8845f2, 33 },
    { 0x3ef845f3, 43 },
    { 0xe88845f3, 43 },
    { 0, 0 },
    { 0xdad845f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff845f9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x641845fc, 61 },
    { 0, 0 },
    { 0x51e845fe, 46 },
    { 0, 0 },
    { 0x11f84600, 66 },
    { 0x5d684601, 61 },
    { 0x15984602, 43 },
    { 0xb7784601, 55 },
    { 0x88884601, 34 },
    { 0x4eb84605, 33 },
    { 0x11c84606, 90 },
    { 0xe984607, 33 },
    { 0x88a84608, 90 },
    { 0xde784608, 46 },
    { 0x76e8460a, 23 },
    { 0x46a8460a, 90 },
    { 0xa5984607, 90 },
    { 0xa8784609, 99 },
    { 0, 0 },
    { 0x638460f, 82 },
    { 0x5b84610, 95 },
    { 0xcf884611, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0784619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2684620, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb84626, 88 },
    { 0x4b84626, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58462c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d884635, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db84643, 52 },
    { 0, 0 },
    { 0xc7784645, 34 },
    { 0xb8184646, 52 },
    { 0xaf84645, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8464b, 84 },
    { 0, 0 },
    { 0x9e38464d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67184653, 55 },
    { 0, 0 },
    { 0x88284655, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf684659, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a84661, 98 },
    { 0xd1c84662, 85 },
    { 0, 0 },
    { 0xd484664, 84 },
    { 0xfe684665, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc8466f, 77 },
    { 0x8da8466f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2884677, 99 },
    { 0x73484678, 99 },
    { 0x6a384677, 87 },
    { 0xa1584679, 99 },
    { 0, 0 },
    { 0x5db8467c, 90 },
    { 0, 0 },
    { 0x1688467e, 83 },
    { 0xd628467f, 61 },
    { 0, 0 },
    { 0xd4f84681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c884686, 33 },
    { 0x6cc84687, 66 },
    { 0, 0 },
    { 0x5084689, 33 },
    { 0x50184689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8468d, 61 },
    { 0x49d8468d, 52 },
    { 0, 0 },
    { 0x18284690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e84697, 83 },
    { 0xb284698, 55 },
    { 0x79784699, 95 },
    { 0x95d84697, 98 },
    { 0x99584698, 98 },
    { 0x7ad84698, 90 },
    { 0, 0 },
    { 0xe708469e, 90 },
    { 0, 0 },
    { 0x12a846a0, 90 },
    { 0x94e846a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d846a5, 90 },
    { 0xc68846a6, 49 },
    { 0x29f846a5, 90 },
    { 0x4df846a8, 98 },
    { 0x760846a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6846ac, 91 },
    { 0x4f5846ad, 90 },
    { 0x6f846ae, 90 },
    { 0, 0 },
    { 0xc94846b0, 81 },
    { 0, 0 },
    { 0x54e846b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb846b7, 93 },
    { 0x1eb846b8, 95 },
    { 0xca9846b8, 17 },
    { 0, 0 },
    { 0x8b846bb, 66 },
    { 0xfdf846bc, 33 },
    { 0x673846bd, 90 },
    { 0, 0 },
    { 0x9ca846bf, 46 },
    { 0xf7b846c0, 33 },
    { 0, 0 },
    { 0x607846c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f846cc, 78 },
    { 0, 0 },
    { 0x6e4846ce, 67 },
    { 0x523846ce, 61 },
    { 0xeec846d0, 67 },
    { 0, 0 },
    { 0xd45846d2, 61 },
    { 0x982846d3, 43 },
    { 0, 0 },
    { 0x94b846d5, 23 },
    { 0xe17846d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844846db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147846df, 36 },
    { 0x5dd846e0, 52 },
    { 0xe9a846df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27846e4, 98 },
    { 0xf7c846e5, 93 },
    { 0x12a846e4, 46 },
    { 0x3f5846e7, 61 },
    { 0x434846e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737846ec, 33 },
    { 0x5ae846ed, 90 },
    { 0xf81846ec, 76 },
    { 0, 0 },
    { 0x73846f0, 90 },
    { 0, 0 },
    { 0x981846f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4846f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09846fd, 15 },
    { 0, 0 },
    { 0x7ef846ff, 43 },
    { 0x53b84700, 83 },
    { 0xa2584700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468470c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce84715, 33 },
    { 0, 0 },
    { 0x39684717, 91 },
    { 0x1d284718, 34 },
    { 0x7b84717, 70 },
    { 0x4a78471a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af84721, 66 },
    { 0x5ac84721, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b684726, 90 },
    { 0, 0 },
    { 0x96084728, 61 },
    { 0x9f884729, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e38472c, 17 },
    { 0, 0 },
    { 0xddd8472e, 90 },
    { 0xb178472f, 56 },
    { 0, 0 },
    { 0x42284731, 23 },
    { 0, 0 },
    { 0x48d84733, 95 },
    { 0xf7b84734, 98 },
    { 0, 0 },
    { 0xe8e84736, 34 },
    { 0, 0 },
    { 0x18184738, 33 },
    { 0x76684739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158473c, 23 },
    { 0x6108473d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4884740, 33 },
    { 0, 0 },
    { 0x6e884742, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c84747, 95 },
    { 0xaad84748, 98 },
    { 0x67784748, 95 },
    { 0x4838474a, 90 },
    { 0, 0 },
    { 0x4448474c, 61 },
    { 0x63f8474d, 78 },
    { 0, 0 },
    { 0x6458474f, 95 },
    { 0x53c84750, 90 },
    { 0xe718474f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea84754, 90 },
    { 0xff84755, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478475a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb628475d, 52 },
    { 0, 0 },
    { 0x6c38475f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b884764, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd8476c, 55 },
    { 0xa48476d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b84771, 90 },
    { 0xa4284772, 33 },
    { 0x4aa84773, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc84779, 34 },
    { 0xdc88477a, 23 },
    { 0x6718477a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x848477e, 33 },
    { 0x7668477f, 95 },
    { 0, 0 },
    { 0x34984781, 55 },
    { 0x81d84782, 33 },
    { 0xb5084783, 90 },
    { 0xa2084784, 87 },
    { 0x76184784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f38478d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b8479b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8479e, 90 },
    { 0, 0 },
    { 0x3e1847a0, 69 },
    { 0xe80847a1, 33 },
    { 0x73847a0, 95 },
    { 0x78f847a1, 85 },
    { 0x3f6847a4, 91 },
    { 0, 0 },
    { 0xe9d847a6, 90 },
    { 0, 0 },
    { 0x984847a8, 33 },
    { 0x1a3847a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc847ae, 90 },
    { 0x260847af, 90 },
    { 0, 0 },
    { 0xe63847b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede847b8, 55 },
    { 0, 0 },
    { 0x7db847ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a847be, 90 },
    { 0x45b847be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3847c4, 46 },
    { 0, 0 },
    { 0xb50847c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1847cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9847d1, 23 },
    { 0, 0 },
    { 0xa65847d3, 88 },
    { 0x522847d3, 23 },
    { 0x243847d5, 98 },
    { 0, 0 },
    { 0x419847d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60847dc, 90 },
    { 0xdfe847dd, 55 },
    { 0x202847de, 95 },
    { 0x41b847df, 76 },
    { 0x3cd847dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x578847e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af847e8, 46 },
    { 0, 0 },
    { 0x28d847ea, 31 },
    { 0xeab847eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d847ef, 33 },
    { 0xd92847f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf847f4, 90 },
    { 0xa0847f5, 46 },
    { 0, 0 },
    { 0xd56847f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b847fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d84800, 33 },
    { 0, 0 },
    { 0x17d84802, 91 },
    { 0, 0 },
    { 0x4b684804, 90 },
    { 0xeca84805, 33 },
    { 0x2ed84806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118480a, 93 },
    { 0x9708480b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e584810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f884818, 75 },
    { 0, 0 },
    { 0x9af8481a, 90 },
    { 0, 0 },
    { 0x5278481c, 95 },
    { 0x8e68481c, 23 },
    { 0x168481e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54984821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884824, 98 },
    { 0, 0 },
    { 0x9cb84826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa84829, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd18482e, 61 },
    { 0, 0 },
    { 0xb0c84830, 99 },
    { 0x25a84831, 34 },
    { 0xe7584830, 84 },
    { 0xf3584830, 33 },
    { 0x19e84834, 78 },
    { 0xff984834, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc778483a, 91 },
    { 0xeac8483b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4108483e, 33 },
    { 0xb9b8483e, 90 },
    { 0x2f384840, 98 },
    { 0x4e8483e, 90 },
    { 0, 0 },
    { 0xe0584843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b184848, 33 },
    { 0, 0 },
    { 0xf598484a, 76 },
    { 0xf3a8484b, 34 },
    { 0x6b08484c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8484f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef84856, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08485c, 95 },
    { 0, 0 },
    { 0x96e8485e, 90 },
    { 0xb6c8485f, 40 },
    { 0x38d8485f, 23 },
    { 0x83d84861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x35984864, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6884867, 55 },
    { 0, 0 },
    { 0xd5484869, 61 },
    { 0x2688486a, 61 },
    { 0x7738486b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63284870, 68 },
    { 0x4db84871, 33 },
    { 0, 0 },
    { 0xd2a84873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76784878, 46 },
    { 0xeb984879, 91 },
    { 0xd208487a, 77 },
    { 0, 0 },
    { 0x5268487c, 52 },
    { 0x11d8487d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad784880, 85 },
    { 0x78184881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61884884, 100 },
    { 0, 0 },
    { 0x87184886, 61 },
    { 0x4b384886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3618488c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8984890, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38984895, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b84898, 93 },
    { 0, 0 },
    { 0xd5c8489a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9118489e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128848a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8848af, 90 },
    { 0xc63848b0, 67 },
    { 0xcb3848b1, 78 },
    { 0xb41848b1, 90 },
    { 0x1cb848b0, 95 },
    { 0xd28848b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da848b7, 61 },
    { 0x7d5848b7, 46 },
    { 0, 0 },
    { 0x923848ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691848be, 34 },
    { 0xb24848bf, 84 },
    { 0xe58848c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6848c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c848ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3848d3, 98 },
    { 0x20b848d4, 23 },
    { 0x75c848d5, 67 },
    { 0xb59848d5, 33 },
    { 0x7f1848d3, 90 },
    { 0xb0a848d8, 90 },
    { 0x680848d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2848dd, 34 },
    { 0xe47848dd, 43 },
    { 0x1b5848dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116848e3, 61 },
    { 0x353848e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7848e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648848ee, 99 },
    { 0x77e848ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7848f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x438848f7, 55 },
    { 0x540848f7, 34 },
    { 0x652848f9, 52 },
    { 0x6b4848f9, 98 },
    { 0, 0 },
    { 0xbfd848fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4584900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c384903, 33 },
    { 0xaa184903, 23 },
    { 0xf5984903, 55 },
    { 0x8e284906, 91 },
    { 0xbf684907, 90 },
    { 0x42884908, 61 },
    { 0, 0 },
    { 0x80c8490a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b8490f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f584913, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc518491a, 90 },
    { 0x758491a, 78 },
    { 0xa128491b, 67 },
    { 0x588491d, 46 },
    { 0x168491c, 95 },
    { 0xe98491c, 95 },
    { 0x82e8491e, 65 },
    { 0x3b184921, 87 },
    { 0x40384922, 77 },
    { 0, 0 },
    { 0x8b684924, 95 },
    { 0xd6284925, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a684928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf084931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec84936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb08493a, 90 },
    { 0, 0 },
    { 0xf7a8493c, 98 },
    { 0, 0 },
    { 0xb5a8493e, 64 },
    { 0x8b88493f, 55 },
    { 0x81184940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d84944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab84949, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28494d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa484950, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d84954, 23 },
    { 0x6e784955, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa8495a, 55 },
    { 0, 0 },
    { 0xc738495c, 46 },
    { 0xd418495d, 95 },
    { 0x38f8495e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb84962, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1084965, 33 },
    { 0, 0 },
    { 0x40884967, 90 },
    { 0x8ba84968, 91 },
    { 0xb584969, 90 },
    { 0, 0 },
    { 0x7a8496b, 98 },
    { 0x9368496b, 90 },
    { 0x5a48496d, 61 },
    { 0x1a48496d, 77 },
    { 0x58b8496f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c684972, 98 },
    { 0xc3684973, 98 },
    { 0x54184972, 43 },
    { 0, 0 },
    { 0xb9f84976, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf78497c, 23 },
    { 0, 0 },
    { 0xae78497e, 76 },
    { 0, 0 },
    { 0x52384980, 77 },
    { 0x27084981, 36 },
    { 0x23184981, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b84987, 77 },
    { 0xd2884987, 66 },
    { 0x88b84988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf168498c, 52 },
    { 0, 0 },
    { 0x598498e, 95 },
    { 0x8938498e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7284992, 98 },
    { 0, 0 },
    { 0xa4c84994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38384998, 82 },
    { 0xe4684999, 43 },
    { 0, 0 },
    { 0xff98499b, 76 },
    { 0, 0 },
    { 0x7078499d, 23 },
    { 0x9318499e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6849a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d849a8, 33 },
    { 0x34a849a9, 43 },
    { 0x530849aa, 55 },
    { 0xf6d849a9, 96 },
    { 0, 0 },
    { 0xe51849ad, 90 },
    { 0x367849ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865849b3, 55 },
    { 0xc95849b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e849b9, 76 },
    { 0, 0 },
    { 0x250849bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7849c1, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x302849c4, 61 },
    { 0x792849c4, 95 },
    { 0, 0 },
    { 0x549849c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181849cb, 95 },
    { 0x4f1849cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b849dd, 23 },
    { 0x7b2849de, 100 },
    { 0xa15849de, 76 },
    { 0x82d849e0, 98 },
    { 0, 0 },
    { 0x6dc849e2, 61 },
    { 0xcdf849e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20849e6, 76 },
    { 0x35c849e7, 33 },
    { 0xd72849e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41849ee, 55 },
    { 0, 0 },
    { 0x50f849f0, 23 },
    { 0x3f849f0, 65 },
    { 0, 0 },
    { 0x77e849f3, 23 },
    { 0x4b5849f4, 55 },
    { 0, 0 },
    { 0xa8b849f6, 90 },
    { 0x33f849f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28849fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5849fd, 55 },
    { 0xaff849fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2284a08, 77 },
    { 0, 0 },
    { 0xad584a0a, 90 },
    { 0xa1584a0a, 84 },
    { 0xf1884a0c, 93 },
    { 0, 0 },
    { 0x7b84a0e, 23 },
    { 0xf9884a0f, 90 },
    { 0xb5884a10, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63484a14, 66 },
    { 0x92684a15, 33 },
    { 0x4f984a15, 66 },
    { 0xd2584a17, 90 },
    { 0xcdf84a15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a84a1d, 98 },
    { 0, 0 },
    { 0x12284a1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55984a22, 95 },
    { 0xf0e84a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67384a29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74484a2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c84a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b684a37, 34 },
    { 0, 0 },
    { 0xfb184a39, 55 },
    { 0x1eb84a3a, 76 },
    { 0x10e84a3a, 76 },
    { 0, 0 },
    { 0xc7984a3d, 92 },
    { 0x2e084a3d, 76 },
    { 0xd284a3e, 55 },
    { 0xf2684a3f, 23 },
    { 0x77b84a41, 36 },
    { 0x6d284a40, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e84a45, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f84a49, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab84a4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e84a51, 55 },
    { 0x9ba84a52, 93 },
    { 0x32b84a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4684a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4184a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46084a63, 76 },
    { 0xa5d84a64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9784a68, 93 },
    { 0, 0 },
    { 0x5e884a6a, 90 },
    { 0x21d84a6a, 85 },
    { 0, 0 },
    { 0x99c84a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6684a74, 61 },
    { 0xb1384a74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9484a7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f784a82, 17 },
    { 0x2e084a82, 33 },
    { 0x2b684a83, 40 },
    { 0xd9e84a83, 101 },
    { 0x63b84a86, 33 },
    { 0, 0 },
    { 0x6884a88, 77 },
    { 0xc9b84a88, 23 },
    { 0, 0 },
    { 0x5784a8b, 55 },
    { 0x2d684a8b, 52 },
    { 0xafd84a8d, 23 },
    { 0xa7984a8b, 95 },
    { 0x3e584a8e, 98 },
    { 0xc9184a90, 61 },
    { 0x46684a8d, 23 },
    { 0x16084a8b, 90 },
    { 0x78784a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a384a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0684a9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31384aa3, 55 },
    { 0, 0 },
    { 0x4584aa5, 98 },
    { 0, 0 },
    { 0xdc684aa7, 93 },
    { 0xfab84aa8, 46 },
    { 0x5fc84aa9, 98 },
    { 0x3b284aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c84ab6, 99 },
    { 0, 0 },
    { 0x3ba84ab8, 98 },
    { 0xf9084ab9, 17 },
    { 0xfc584aba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a484abe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac84ac1, 93 },
    { 0x1ee84ac2, 43 },
    { 0x19984ac3, 55 },
    { 0xddd84ac2, 98 },
    { 0x25784ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb084aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87484acd, 93 },
    { 0, 0 },
    { 0x6b384acf, 61 },
    { 0xca284ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac84ad3, 90 },
    { 0xa1684ad3, 98 },
    { 0xe8284ad5, 52 },
    { 0xdac84ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c984ad9, 17 },
    { 0xcb284ad9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49784add, 61 },
    { 0x12984ade, 44 },
    { 0x26684adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3084ae4, 93 },
    { 0x3b384ae5, 90 },
    { 0x1e884ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45884aed, 90 },
    { 0xa1184aee, 34 },
    { 0, 0 },
    { 0xc4a84af0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43584afb, 34 },
    { 0xe5a84afc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac84aff, 55 },
    { 0xe3c84b00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab84b04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93884b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1584b0c, 61 },
    { 0xe7784b0d, 76 },
    { 0x78584b0e, 98 },
    { 0x56f84b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c84b12, 90 },
    { 0xd2184b13, 23 },
    { 0xce84b14, 55 },
    { 0, 0 },
    { 0x36584b16, 90 },
    { 0xcc884b17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a84b1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b84b1f, 43 },
    { 0x6a584b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4884b25, 90 },
    { 0x16f84b26, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf584b2b, 77 },
    { 0, 0 },
    { 0x97f84b2d, 61 },
    { 0xff484b2e, 33 },
    { 0xa3d84b2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75684b36, 78 },
    { 0, 0 },
    { 0x52384b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f984b3b, 55 },
    { 0xc2e84b3c, 96 },
    { 0xbf84b3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75084b43, 96 },
    { 0xedd84b44, 90 },
    { 0x70484b45, 33 },
    { 0x98c84b46, 66 },
    { 0x95884b44, 23 },
    { 0, 0 },
    { 0xf2f84b49, 46 },
    { 0xa6784b4a, 23 },
    { 0x5084b4a, 78 },
    { 0, 0 },
    { 0x8f384b4d, 95 },
    { 0x2e084b4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb384b53, 90 },
    { 0, 0 },
    { 0xec584b55, 46 },
    { 0x2184b55, 55 },
    { 0xcc084b57, 55 },
    { 0, 0 },
    { 0x13b84b59, 43 },
    { 0x79e84b5a, 52 },
    { 0xb8384b59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62184b5e, 99 },
    { 0xea84b5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a84b66, 90 },
    { 0x61b84b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee84b6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed584b70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f784b76, 52 },
    { 0, 0 },
    { 0x7ae84b78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f84b7b, 34 },
    { 0x4ea84b7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b484b80, 90 },
    { 0x6b684b81, 66 },
    { 0x9cf84b81, 23 },
    { 0x43d84b80, 33 },
    { 0x3b984b82, 33 },
    { 0x6c284b82, 66 },
    { 0x82584b86, 55 },
    { 0x27984b87, 95 },
    { 0xa3484b86, 90 },
    { 0, 0 },
    { 0x93784b8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb84b90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9484b9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb484b9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85984ba1, 43 },
    { 0xcea84ba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e84ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d84bac, 34 },
    { 0, 0 },
    { 0x74684bae, 90 },
    { 0xbe184bae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae84bb3, 33 },
    { 0, 0 },
    { 0x19f84bb5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22484bbb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae184bc2, 17 },
    { 0, 0 },
    { 0x8c584bc4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d84bc8, 23 },
    { 0x56884bc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e584bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4984bdd, 91 },
    { 0, 0 },
    { 0x2c984bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9084be3, 98 },
    { 0x7f884be4, 46 },
    { 0x40484be5, 100 },
    { 0xf9184be6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d384bf2, 90 },
    { 0x3a984bf3, 95 },
    { 0x184bf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0284bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d84bfe, 40 },
    { 0x9a884bff, 33 },
    { 0xf8d84c00, 66 },
    { 0xd1984bfe, 95 },
    { 0x89584bff, 23 },
    { 0x62e84c01, 33 },
    { 0x2e084c04, 95 },
    { 0, 0 },
    { 0x52784c06, 55 },
    { 0x1ea84c06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be84c0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f84c10, 76 },
    { 0xeca84c11, 23 },
    { 0x37184c11, 23 },
    { 0xf9684c13, 33 },
    { 0x36a84c14, 33 },
    { 0x3a384c11, 71 },
    { 0x1f284c16, 34 },
    { 0, 0 },
    { 0x11b84c18, 95 },
    { 0xb8384c18, 90 },
    { 0, 0 },
    { 0x9f484c1b, 95 },
    { 0x45384c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6084c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb884c2c, 66 },
    { 0xb3e84c2d, 61 },
    { 0x3d684c2d, 61 },
    { 0xba684c2f, 66 },
    { 0xf6b84c2e, 34 },
    { 0xa0f84c31, 95 },
    { 0, 0 },
    { 0xb4b84c33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb84c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc84c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc784c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9284c43, 61 },
    { 0, 0 },
    { 0x88f84c45, 61 },
    { 0xa6d84c45, 95 },
    { 0xc4084c47, 23 },
    { 0xb4484c48, 98 },
    { 0xb7c84c47, 95 },
    { 0xc1884c49, 98 },
    { 0, 0 },
    { 0x1f484c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f84c50, 55 },
    { 0x50d84c50, 99 },
    { 0, 0 },
    { 0xe5984c53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac84c57, 55 },
    { 0x94984c58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce84c5f, 90 },
    { 0, 0 },
    { 0x3c084c61, 98 },
    { 0x7bb84c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce084c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d884c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb584c72, 61 },
    { 0xc3684c72, 99 },
    { 0xca684c73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26684c79, 76 },
    { 0xaee84c7a, 90 },
    { 0, 0 },
    { 0x73184c7c, 95 },
    { 0xa2e84c7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda484c80, 99 },
    { 0, 0 },
    { 0xfa584c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a84c85, 23 },
    { 0, 0 },
    { 0xb6a84c87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6284c8a, 90 },
    { 0, 0 },
    { 0x4584c8c, 67 },
    { 0x8f984c8d, 93 },
    { 0x41884c8e, 90 },
    { 0x7ac84c8e, 55 },
    { 0, 0 },
    { 0x9a184c91, 98 },
    { 0x4b684c91, 61 },
    { 0xd8e84c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb84c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5784c9c, 23 },
    { 0x8bb84c9c, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d084ca3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df84ca7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8584cac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf984caf, 33 },
    { 0x68484cb0, 23 },
    { 0, 0 },
    { 0x5f84cb2, 61 },
    { 0x94584cb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed84cb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a84cc1, 43 },
    { 0xa0c84cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12284cc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7084ccd, 78 },
    { 0x9e884cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95984cd4, 40 },
    { 0x1ba84cd4, 55 },
    { 0x8c584cd6, 61 },
    { 0x12684cd7, 90 },
    { 0xce284cd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce84cdd, 91 },
    { 0xa5f84cde, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb84ce1, 33 },
    { 0, 0 },
    { 0x3d484ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1284ce8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2484cef, 100 },
    { 0xe3a84cf0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e84cf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9184cfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86084d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7284d04, 66 },
    { 0, 0 },
    { 0xef184d06, 81 },
    { 0x8f684d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9584d0b, 95 },
    { 0x8c584d0c, 99 },
    { 0xec184d0d, 97 },
    { 0xca584d0b, 55 },
    { 0, 0 },
    { 0x85184d10, 40 },
    { 0, 0 },
    { 0xbc184d12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e84d19, 98 },
    { 0, 0 },
    { 0xcb184d1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b284d1e, 95 },
    { 0xa6f84d1f, 90 },
    { 0x31184d20, 76 },
    { 0xb8f84d1e, 27 },
    { 0x13784d22, 33 },
    { 0xdee84d23, 95 },
    { 0x5a484d24, 61 },
    { 0x13e84d25, 76 },
    { 0xeb284d25, 43 },
    { 0xc8e84d20, 44 },
    { 0x15684d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e784d2e, 23 },
    { 0x57c84d2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9284d34, 95 },
    { 0x79784d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6384d38, 76 },
    { 0, 0 },
    { 0x13084d3a, 67 },
    { 0xc9484d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30184d40, 90 },
    { 0x9d084d40, 84 },
    { 0, 0 },
    { 0x22384d43, 61 },
    { 0x9dd84d44, 100 },
    { 0, 0 },
    { 0x6e084d46, 55 },
    { 0x58284d46, 98 },
    { 0xb9684d46, 33 },
    { 0x52e84d49, 34 },
    { 0xd5784d4a, 61 },
    { 0xa7c84d47, 76 },
    { 0xf2984d4c, 34 },
    { 0xe5c84d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0484d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7384d56, 78 },
    { 0xc2f84d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7484d5c, 61 },
    { 0xb84d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c284d62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a84d65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b84d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4484d70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec84d76, 61 },
    { 0xd5584d77, 23 },
    { 0, 0 },
    { 0x34184d79, 52 },
    { 0x8784d7a, 91 },
    { 0x88184d7b, 61 },
    { 0x2e984d7b, 46 },
    { 0x88784d79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b84d82, 90 },
    { 0, 0 },
    { 0x6be84d84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22484d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a84d8b, 52 },
    { 0x6484d8c, 90 },
    { 0, 0 },
    { 0x6aa84d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc984d92, 76 },
    { 0, 0 },
    { 0xe9884d94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e84d9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6684d9f, 95 },
    { 0xbad84da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e84da7, 66 },
    { 0x9e584da8, 88 },
    { 0, 0 },
    { 0xaf784daa, 61 },
    { 0x70484dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea84dae, 67 },
    { 0xa1b84daf, 55 },
    { 0, 0 },
    { 0x47b84db1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x97384db4, 33 },
    { 0x4fc84db5, 55 },
    { 0xfb884db6, 77 },
    { 0xcac84db4, 23 },
    { 0xbe084db8, 52 },
    { 0x42384db9, 76 },
    { 0, 0 },
    { 0x85d84dbb, 23 },
    { 0x29f84dbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44784dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f84dc5, 46 },
    { 0, 0 },
    { 0xfbc84dc7, 61 },
    { 0xd784dc8, 90 },
    { 0x96784dc8, 90 },
    { 0, 0 },
    { 0xb7384dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97484dcf, 55 },
    { 0x60c84dcf, 33 },
    { 0xa1b84dd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf84dd4, 77 },
    { 0, 0 },
    { 0x35084dd6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be84dda, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64384dde, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b84de1, 43 },
    { 0xcca84de1, 90 },
    { 0x27584de3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b84de8, 55 },
    { 0, 0 },
    { 0x40984dea, 90 },
    { 0x7c984deb, 42 },
    { 0xf0884deb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab84df1, 61 },
    { 0, 0 },
    { 0xa2184df3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24484df6, 90 },
    { 0xc2084df7, 76 },
    { 0x12084df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b484dfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e84e00, 98 },
    { 0x53e84e01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1284e06, 78 },
    { 0x58384e07, 90 },
    { 0x88c84e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c684e0c, 33 },
    { 0, 0 },
    { 0xdca84e0e, 46 },
    { 0x53784e0f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384e12, 64 },
    { 0, 0 },
    { 0xaba84e14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6884e18, 67 },
    { 0xc3384e19, 95 },
    { 0x3f684e1a, 23 },
    { 0x42984e1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e20, 52 },
    { 0x25d84e21, 23 },
    { 0xa8284e20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea984e28, 93 },
    { 0, 0 },
    { 0xc4a84e2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b84e2d, 78 },
    { 0, 0 },
    { 0x8be84e2f, 78 },
    { 0x6984e30, 33 },
    { 0x11f84e31, 85 },
    { 0x30f84e32, 61 },
    { 0xfda84e32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd984e3a, 76 },
    { 0xdf584e3b, 95 },
    { 0, 0 },
    { 0xbac84e3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54784e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c84e45, 61 },
    { 0xe1084e46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d84e49, 61 },
    { 0, 0 },
    { 0x9f984e4b, 89 },
    { 0, 0 },
    { 0xbc84e4d, 65 },
    { 0x88084e4e, 23 },
    { 0, 0 },
    { 0x34f84e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5684e57, 81 },
    { 0xf4384e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83684e5d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15584e62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e84e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x93984e68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7184e70, 70 },
    { 0x36684e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc84e75, 98 },
    { 0xa1b84e75, 61 },
    { 0, 0 },
    { 0x2d884e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1784e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25684e7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c084e81, 33 },
    { 0xba84e82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b84e85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c84e88, 97 },
    { 0xc3984e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13584e8d, 90 },
    { 0xf5484e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a84e91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde384e97, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e84e9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c284e9e, 61 },
    { 0x2584e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e84ea7, 33 },
    { 0x69284ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a84eab, 90 },
    { 0, 0 },
    { 0xf7184ead, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a84eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc84eb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e84ebe, 93 },
    { 0, 0 },
    { 0xe2684ec0, 84 },
    { 0, 0 },
    { 0xd9584ec2, 76 },
    { 0x4ba84ec2, 90 },
    { 0x28884ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc484eca, 33 },
    { 0x12584ecb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69484ed1, 90 },
    { 0, 0 },
    { 0x40f84ed3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x54584ed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe84edb, 85 },
    { 0xa8184edb, 33 },
    { 0x85a84edd, 85 },
    { 0x89584ede, 90 },
    { 0, 0 },
    { 0xe4684ee0, 33 },
    { 0x3a684ee0, 95 },
    { 0, 0 },
    { 0x81d84ee3, 55 },
    { 0, 0 },
    { 0x48f84ee5, 98 },
    { 0xaa884ee6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb784eec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a84ef5, 83 },
    { 0xdb284ef6, 66 },
    { 0, 0 },
    { 0x4f84ef8, 90 },
    { 0x4eb84ef8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31584efd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x56684f00, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e84f03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5884f06, 61 },
    { 0xd3684f06, 61 },
    { 0x4af84f08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f84f11, 43 },
    { 0x5c084f12, 68 },
    { 0xe884f13, 61 },
    { 0x7784f13, 61 },
    { 0x4cf84f12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f84f18, 95 },
    { 0x44584f19, 98 },
    { 0, 0 },
    { 0xc5784f1b, 90 },
    { 0x56784f1b, 23 },
    { 0xb3c84f1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f84f20, 93 },
    { 0, 0 },
    { 0x75f84f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b84f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b84f2a, 34 },
    { 0xebb84f2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90984f30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc784f33, 66 },
    { 0x8a384f34, 90 },
    { 0xcd284f35, 65 },
    { 0x25884f36, 87 },
    { 0x48384f35, 90 },
    { 0xd9584f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c84f3b, 100 },
    { 0, 0 },
    { 0x92584f3d, 91 },
    { 0x76384f3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70484f41, 34 },
    { 0xdbe84f42, 34 },
    { 0x11484f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48984f46, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c384f49, 61 },
    { 0x7c784f4a, 90 },
    { 0x7cd84f49, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1484f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e884f52, 33 },
    { 0, 0 },
    { 0xa6984f54, 23 },
    { 0x4bb84f55, 33 },
    { 0, 0 },
    { 0x5a684f57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b84f5b, 65 },
    { 0xe2d84f5c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e84f61, 84 },
    { 0xbb484f62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6084f67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e584f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd84f71, 61 },
    { 0xb2984f72, 61 },
    { 0x12084f73, 43 },
    { 0x98284f71, 82 },
    { 0x46f84f72, 61 },
    { 0xa5784f74, 52 },
    { 0, 0 },
    { 0x2ad84f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc84f81, 34 },
    { 0xe1e84f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d84f87, 95 },
    { 0x4a184f87, 90 },
    { 0xd6e84f88, 33 },
    { 0x1e784f8a, 90 },
    { 0, 0 },
    { 0x45a84f8c, 46 },
    { 0x7ae84f8c, 93 },
    { 0, 0 },
    { 0x24984f8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34384f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a884f9a, 66 },
    { 0xee484f9b, 93 },
    { 0, 0 },
    { 0xd7084f9d, 77 },
    { 0xc6d84f9e, 64 },
    { 0xeb684f9f, 61 },
    { 0x90b84fa0, 55 },
    { 0x5d884f9f, 55 },
    { 0x9e984fa2, 84 },
    { 0x49c84fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f84fa6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16084fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19584fac, 76 },
    { 0x36a84fad, 98 },
    { 0x47684fac, 61 },
    { 0x67684faf, 61 },
    { 0xc3884fb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f84fb3, 95 },
    { 0x16284fb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x62984fb7, 90 },
    { 0x9a984fb8, 61 },
    { 0x39584fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9684fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b84fc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e84fc9, 52 },
    { 0x10b84fc9, 61 },
    { 0xd7d84fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7584fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d584fd3, 95 },
    { 0x27484fd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b84fdc, 97 },
    { 0x43a84fdc, 33 },
    { 0xd8e84fdd, 61 },
    { 0, 0 },
    { 0xce884fe0, 33 },
    { 0, 0 },
    { 0x51784fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c84fe6, 23 },
    { 0, 0 },
    { 0x51684fe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b84feb, 23 },
    { 0, 0 },
    { 0xbe084fed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6584ff3, 76 },
    { 0x7c84ff4, 67 },
    { 0x67484ff5, 61 },
    { 0, 0 },
    { 0x89d84ff7, 48 },
    { 0x7fc84ff7, 46 },
    { 0x22384ff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10284ffc, 23 },
    { 0x5c884ffd, 91 },
    { 0x9f84ffe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34385005, 98 },
    { 0x61285006, 61 },
    { 0x91c85007, 95 },
    { 0, 0 },
    { 0x66b85009, 61 },
    { 0x6bb85009, 33 },
    { 0xfc48500b, 83 },
    { 0x1518500c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f85014, 98 },
    { 0x9c385015, 76 },
    { 0x83585016, 43 },
    { 0, 0 },
    { 0xfdd85018, 55 },
    { 0x7b885019, 95 },
    { 0, 0 },
    { 0x298501b, 76 },
    { 0xb328501c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa085020, 46 },
    { 0x46685021, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf185024, 33 },
    { 0xc2a85025, 96 },
    { 0xeb585026, 90 },
    { 0x62f85027, 55 },
    { 0, 0 },
    { 0x3f685029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c38502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca085031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e85034, 90 },
    { 0x2f985035, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6918503b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d88503f, 95 },
    { 0x5c85040, 99 },
    { 0x2238503f, 33 },
    { 0x33d85042, 90 },
    { 0x9e285040, 76 },
    { 0, 0 },
    { 0xaa085045, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a85049, 95 },
    { 0xdec85049, 61 },
    { 0, 0 },
    { 0xe738504c, 76 },
    { 0x4008504d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa685056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64885059, 95 },
    { 0xf6285059, 46 },
    { 0xdcc8505a, 52 },
    { 0, 0 },
    { 0xa528505d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae85060, 23 },
    { 0, 0 },
    { 0xb0b85062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f85068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8506b, 23 },
    { 0x1fd8506c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae85072, 61 },
    { 0x83c85073, 76 },
    { 0x8d185072, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0085077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd658507f, 84 },
    { 0xe685080, 23 },
    { 0, 0 },
    { 0xb2485082, 43 },
    { 0x94185083, 23 },
    { 0xb9685084, 90 },
    { 0x10485083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8508b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a385095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f85099, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf8509c, 23 },
    { 0xe1e8509d, 98 },
    { 0xc1c8509e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263850a2, 33 },
    { 0xfce850a2, 33 },
    { 0xf1b850a2, 99 },
    { 0x323850a4, 90 },
    { 0, 0 },
    { 0x226850a7, 55 },
    { 0x743850a8, 90 },
    { 0x994850a9, 98 },
    { 0, 0 },
    { 0x716850ab, 78 },
    { 0, 0 },
    { 0x5ac850ad, 33 },
    { 0xba8850ae, 34 },
    { 0xa3e850af, 90 },
    { 0x472850ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x958850b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55850b8, 98 },
    { 0xc9a850b8, 55 },
    { 0x600850ba, 93 },
    { 0xeac850bb, 95 },
    { 0xaaf850ba, 99 },
    { 0, 0 },
    { 0x3db850be, 23 },
    { 0xeb6850be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353850c6, 84 },
    { 0xbb850c7, 66 },
    { 0x532850c7, 93 },
    { 0x449850c9, 95 },
    { 0, 0 },
    { 0xdf3850cb, 33 },
    { 0x691850cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86850d0, 23 },
    { 0x3ea850d0, 33 },
    { 0x903850d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39850d6, 61 },
    { 0xeca850d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb850dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5850e0, 61 },
    { 0x600850e0, 23 },
    { 0xb24850e2, 43 },
    { 0x738850e3, 95 },
    { 0x5e0850e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29850eb, 78 },
    { 0xcc4850ec, 34 },
    { 0xf56850ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6850f2, 34 },
    { 0, 0 },
    { 0xaa1850f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d850f7, 90 },
    { 0x4b850f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e685101, 85 },
    { 0, 0 },
    { 0xa3785103, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99285107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38510a, 87 },
    { 0, 0 },
    { 0xd6f8510c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5028510f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8511a, 78 },
    { 0x2868511a, 98 },
    { 0x9578511a, 99 },
    { 0x8bb8511d, 33 },
    { 0x2d58511e, 67 },
    { 0x2f38511a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60485122, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f985126, 52 },
    { 0x89d85127, 98 },
    { 0xdc385127, 90 },
    { 0xf6885129, 98 },
    { 0xa8f85129, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe85130, 33 },
    { 0x4e285131, 61 },
    { 0xaa385130, 68 },
    { 0x5e985133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38513d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37785140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98385146, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44585155, 46 },
    { 0xf7b85155, 84 },
    { 0xa5085157, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b8515c, 98 },
    { 0x5288515d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7685163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3385168, 33 },
    { 0, 0 },
    { 0x4b98516a, 23 },
    { 0x40c8516b, 95 },
    { 0xe818516a, 43 },
    { 0x3f68516d, 33 },
    { 0x47e8516a, 99 },
    { 0xb7a8516f, 90 },
    { 0xa6885170, 23 },
    { 0xddf8516f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a85179, 98 },
    { 0x4da85179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78517e, 98 },
    { 0x2538517f, 78 },
    { 0x578517e, 84 },
    { 0xd3e85181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43685184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe758518c, 90 },
    { 0, 0 },
    { 0x4bd8518e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9785191, 34 },
    { 0x25085191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f85195, 84 },
    { 0xc6985196, 90 },
    { 0, 0 },
    { 0xff285198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe8519b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8519e, 76 },
    { 0x9548519f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec851a2, 67 },
    { 0, 0 },
    { 0x55b851a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473851a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63851ad, 46 },
    { 0xf8b851ad, 87 },
    { 0xf97851af, 83 },
    { 0x82851b0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d851b3, 65 },
    { 0x9a1851b4, 91 },
    { 0x51f851b5, 78 },
    { 0xc60851b5, 61 },
    { 0x814851b6, 23 },
    { 0xbb4851b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61851c1, 90 },
    { 0x58f851c2, 34 },
    { 0x7b1851c3, 34 },
    { 0x92d851c1, 93 },
    { 0x75c851c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4851cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a851d0, 33 },
    { 0x797851d1, 84 },
    { 0xbd0851d2, 61 },
    { 0xbfa851d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x676851d6, 23 },
    { 0xc6851d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a851da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x279851dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40851e0, 100 },
    { 0x510851e1, 90 },
    { 0x254851e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52851ea, 61 },
    { 0, 0 },
    { 0xc88851ec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163851f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6851f3, 33 },
    { 0x80851f4, 90 },
    { 0x29e851f5, 95 },
    { 0x8dd851f6, 100 },
    { 0xf3c851f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a851fc, 90 },
    { 0x36b851fd, 91 },
    { 0x4ff851fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d185204, 34 },
    { 0x36485205, 34 },
    { 0, 0 },
    { 0xf1985207, 59 },
    { 0x81c85208, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685211, 87 },
    { 0xe2985212, 34 },
    { 0, 0 },
    { 0xed185214, 61 },
    { 0xaad85214, 91 },
    { 0, 0 },
    { 0x4df85217, 76 },
    { 0x47385218, 100 },
    { 0x38b85219, 90 },
    { 0x4618521a, 90 },
    { 0x9838521b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b85221, 98 },
    { 0, 0 },
    { 0xb1d85223, 90 },
    { 0x4c085224, 61 },
    { 0x2eb85225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7808522c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7685235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8523b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb85240, 90 },
    { 0, 0 },
    { 0x56f85242, 95 },
    { 0, 0 },
    { 0x83f85244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7885249, 90 },
    { 0xc618524a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8524e, 46 },
    { 0xcf98524f, 33 },
    { 0, 0 },
    { 0x9dd85251, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9585254, 90 },
    { 0xcfd85255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca18525a, 36 },
    { 0xfa58525b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28525e, 93 },
    { 0x50d8525f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5085262, 95 },
    { 0x61985263, 34 },
    { 0, 0 },
    { 0xa1085265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38526b, 40 },
    { 0x1ed8526c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71785271, 36 },
    { 0, 0 },
    { 0x4dd85273, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf85278, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e08527c, 99 },
    { 0x6ae8527d, 40 },
    { 0, 0 },
    { 0x8458527f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60785285, 61 },
    { 0xb7c85285, 90 },
    { 0x52385285, 43 },
    { 0xf7285288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e8528b, 76 },
    { 0xed48528b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e8528f, 90 },
    { 0xc158528f, 90 },
    { 0xc38528f, 61 },
    { 0x3eb85292, 77 },
    { 0, 0 },
    { 0x78b85294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77185299, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf068529e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34852aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a852ad, 55 },
    { 0, 0 },
    { 0x498852af, 76 },
    { 0x558852af, 98 },
    { 0x28852b1, 75 },
    { 0x19a852b2, 95 },
    { 0xefa852af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c852b6, 90 },
    { 0xce6852b7, 78 },
    { 0xe61852b8, 23 },
    { 0, 0 },
    { 0xd4e852ba, 33 },
    { 0x4e8852ba, 91 },
    { 0xd57852bc, 90 },
    { 0xbae852bd, 23 },
    { 0, 0 },
    { 0x350852bf, 88 },
    { 0x593852bf, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef852c5, 55 },
    { 0xb95852c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e852c9, 90 },
    { 0, 0 },
    { 0x780852cb, 87 },
    { 0x9f2852cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a852d0, 33 },
    { 0x22e852d1, 96 },
    { 0xe4852d2, 45 },
    { 0x85e852d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91852d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x799852dc, 98 },
    { 0x895852dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2852e2, 43 },
    { 0, 0 },
    { 0x651852e4, 99 },
    { 0xfa0852e5, 98 },
    { 0xf7a852e6, 77 },
    { 0xefa852e6, 99 },
    { 0, 0 },
    { 0x84852e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29852ee, 76 },
    { 0xfc7852ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2852f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be852fc, 90 },
    { 0x8dc852fd, 23 },
    { 0x61f852fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c85305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4658530b, 61 },
    { 0xee8530b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c85310, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96985314, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97385318, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668531e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c85323, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b85326, 69 },
    { 0x6eb85327, 59 },
    { 0xfed85326, 34 },
    { 0xcbf85329, 23 },
    { 0xf4585327, 61 },
    { 0, 0 },
    { 0xdea8532c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f685330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12285338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb8533b, 81 },
    { 0x8ab8533b, 33 },
    { 0x98533d, 76 },
    { 0x7528533d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78485341, 91 },
    { 0x55e85342, 61 },
    { 0xde085341, 76 },
    { 0xb8185341, 33 },
    { 0xd3285344, 33 },
    { 0, 0 },
    { 0x34d85347, 81 },
    { 0xc7c85348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5468534c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208534f, 85 },
    { 0x32a85350, 23 },
    { 0xf2885351, 23 },
    { 0, 0 },
    { 0x15685353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b8535b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a85361, 24 },
    { 0, 0 },
    { 0x72d85363, 99 },
    { 0, 0 },
    { 0x7ab85365, 66 },
    { 0, 0 },
    { 0x57c85367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38536a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98536d, 67 },
    { 0, 0 },
    { 0x1a78536f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc85372, 87 },
    { 0x37f85373, 78 },
    { 0x20085372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d85378, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd8537b, 90 },
    { 0x2858537c, 59 },
    { 0x988537d, 95 },
    { 0xf2c8537d, 33 },
    { 0x1178537d, 55 },
    { 0, 0 },
    { 0xcfb85381, 85 },
    { 0x9a985382, 61 },
    { 0, 0 },
    { 0xc7485384, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed085388, 99 },
    { 0x55185388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8538e, 95 },
    { 0xd3d8538f, 55 },
    { 0xcee85390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18585393, 37 },
    { 0x63985394, 98 },
    { 0x25785395, 23 },
    { 0xb6485394, 95 },
    { 0, 0 },
    { 0xc5585398, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928539e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16853a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462853a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f853a8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d853ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4853ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83853b6, 34 },
    { 0, 0 },
    { 0x6d6853b8, 61 },
    { 0x95e853b9, 46 },
    { 0x6cd853ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec853be, 65 },
    { 0x93b853bf, 34 },
    { 0xf4c853bf, 95 },
    { 0, 0 },
    { 0xea7853c2, 90 },
    { 0xc76853c3, 76 },
    { 0x89e853c4, 46 },
    { 0, 0 },
    { 0xfbd853c6, 33 },
    { 0xf7d853c6, 76 },
    { 0xa1d853c8, 23 },
    { 0x438853c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4853cf, 98 },
    { 0x4d1853cf, 61 },
    { 0x900853d1, 98 },
    { 0x422853d2, 61 },
    { 0x3d7853d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe853d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c853de, 91 },
    { 0xcb6853df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x523853e2, 88 },
    { 0x3eb853e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4853e8, 90 },
    { 0xd98853e9, 88 },
    { 0x5ef853e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b853ed, 93 },
    { 0, 0 },
    { 0x44d853ef, 34 },
    { 0xd23853f0, 93 },
    { 0, 0 },
    { 0xa7d853f2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973853f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a853fb, 98 },
    { 0x74853fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33685403, 66 },
    { 0x90285404, 83 },
    { 0, 0 },
    { 0x35d85406, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8540a, 93 },
    { 0x2758540b, 81 },
    { 0x1878540b, 90 },
    { 0x8638540d, 76 },
    { 0x4468540d, 61 },
    { 0x32d8540f, 34 },
    { 0xbae85410, 68 },
    { 0, 0 },
    { 0x84e85412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a685419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d8541e, 98 },
    { 0xef18541f, 98 },
    { 0x1f685420, 34 },
    { 0, 0 },
    { 0x38f85422, 61 },
    { 0, 0 },
    { 0x9f785424, 65 },
    { 0x8d785425, 87 },
    { 0xb7185424, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc8542f, 61 },
    { 0, 0 },
    { 0xbee85431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3585434, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2385437, 81 },
    { 0xade85438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80985442, 98 },
    { 0xf9185443, 98 },
    { 0xf2085443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8544b, 23 },
    { 0xcf28544c, 78 },
    { 0xb608544b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf085450, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde685454, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8545d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89385460, 23 },
    { 0xe2085461, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b485465, 34 },
    { 0x78085465, 76 },
    { 0x5585467, 90 },
    { 0x9fd85467, 91 },
    { 0xa7285466, 66 },
    { 0xa268546a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe778546e, 95 },
    { 0, 0 },
    { 0xe5185470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a85474, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64785478, 23 },
    { 0, 0 },
    { 0x25f8547a, 95 },
    { 0x31e8547b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8547e, 33 },
    { 0x3f18547f, 52 },
    { 0x2a48547e, 23 },
    { 0xcc48547e, 55 },
    { 0xff88547f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c85488, 90 },
    { 0xd3f85489, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb8548c, 76 },
    { 0, 0 },
    { 0x2ed8548e, 34 },
    { 0x67c8548f, 45 },
    { 0xa848548e, 66 },
    { 0xe7685491, 61 },
    { 0, 0 },
    { 0xbf985493, 76 },
    { 0x985494, 98 },
    { 0x4ed85494, 55 },
    { 0x4d085494, 61 },
    { 0x63885497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be8549d, 90 },
    { 0x7ff8549d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f854a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b854a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1854ad, 88 },
    { 0x521854ae, 23 },
    { 0x635854ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86854b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a854ba, 61 },
    { 0, 0 },
    { 0xe0b854bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2854c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3854cb, 90 },
    { 0, 0 },
    { 0x444854cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86854d1, 33 },
    { 0xcfc854d1, 61 },
    { 0x5af854d1, 76 },
    { 0, 0 },
    { 0xbfb854d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f854df, 33 },
    { 0x6e4854df, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874854e4, 33 },
    { 0, 0 },
    { 0x63f854e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1854ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f854f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1854f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d854fd, 78 },
    { 0x53854fe, 90 },
    { 0x49a854ff, 90 },
    { 0, 0 },
    { 0x24385501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde85504, 98 },
    { 0x9ef85505, 93 },
    { 0, 0 },
    { 0xe7085507, 78 },
    { 0, 0 },
    { 0x87085509, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8550e, 84 },
    { 0, 0 },
    { 0xb7485510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be85514, 61 },
    { 0x6a485515, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c8551b, 33 },
    { 0xd168551c, 23 },
    { 0xcba8551d, 92 },
    { 0x2748551e, 43 },
    { 0, 0 },
    { 0x57085520, 66 },
    { 0xaff85521, 90 },
    { 0xc6785522, 76 },
    { 0x3af85521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08552f, 43 },
    { 0, 0 },
    { 0xe9385531, 33 },
    { 0, 0 },
    { 0x4c685533, 90 },
    { 0, 0 },
    { 0x6eb85535, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f85539, 100 },
    { 0xd1285539, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62285540, 52 },
    { 0xbc485541, 61 },
    { 0x9e685540, 34 },
    { 0x55685542, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4185546, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86685549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078554e, 23 },
    { 0x4c98554f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685552, 98 },
    { 0x2da85553, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb585556, 33 },
    { 0x1d385556, 66 },
    { 0x81785557, 61 },
    { 0x81585558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9938555c, 98 },
    { 0, 0 },
    { 0x4f68555e, 90 },
    { 0x7988555f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84285564, 46 },
    { 0xbe785565, 55 },
    { 0x3ce85566, 46 },
    { 0xba785567, 46 },
    { 0x70385565, 77 },
    { 0, 0 },
    { 0x4728556a, 53 },
    { 0x2468556b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158556f, 75 },
    { 0xac58556f, 76 },
    { 0x5c85570, 23 },
    { 0xbcf85572, 43 },
    { 0x33185572, 33 },
    { 0xd7d85574, 90 },
    { 0xdfa85572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa8557f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd185583, 98 },
    { 0xbff85583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34685588, 95 },
    { 0xb6e85589, 98 },
    { 0xcb28558a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c8558e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d85592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb85596, 61 },
    { 0x9c485597, 93 },
    { 0x8e485598, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f8559f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832855a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2855a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481855ab, 76 },
    { 0, 0 },
    { 0x190855ad, 61 },
    { 0x682855ae, 93 },
    { 0xe7e855af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd855b2, 100 },
    { 0, 0 },
    { 0xe8b855b4, 55 },
    { 0, 0 },
    { 0x3c4855b6, 23 },
    { 0, 0 },
    { 0x1e9855b8, 33 },
    { 0xf70855b9, 76 },
    { 0x993855b8, 77 },
    { 0x48a855bb, 76 },
    { 0x87855bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c855c0, 23 },
    { 0x76e855c1, 34 },
    { 0x275855c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2855ca, 43 },
    { 0xb60855ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3855d0, 90 },
    { 0x45d855d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7855d8, 98 },
    { 0, 0 },
    { 0x5fc855da, 99 },
    { 0xc7d855db, 23 },
    { 0x999855dc, 33 },
    { 0x771855dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x571855e0, 61 },
    { 0xc49855e1, 33 },
    { 0x367855e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af855e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a855eb, 95 },
    { 0, 0 },
    { 0x7d0855ed, 99 },
    { 0x726855ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe855f4, 90 },
    { 0x58855f5, 76 },
    { 0, 0 },
    { 0x6e5855f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81855fd, 55 },
    { 0x6c6855fe, 80 },
    { 0x2d5855fe, 76 },
    { 0x2f985600, 33 },
    { 0xe8785601, 23 },
    { 0xf6a85602, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a8560e, 19 },
    { 0xf6e8560f, 90 },
    { 0xb8e85610, 34 },
    { 0x98f85610, 23 },
    { 0x1b785610, 98 },
    { 0x3cd85612, 90 },
    { 0xa6985611, 86 },
    { 0x50a85612, 75 },
    { 0x1ba85612, 55 },
    { 0x3be85617, 68 },
    { 0x59285618, 43 },
    { 0x78585619, 23 },
    { 0x22d85613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98285622, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3285625, 61 },
    { 0, 0 },
    { 0x8a485627, 43 },
    { 0x9ff85628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb428562c, 90 },
    { 0xe6e8562d, 33 },
    { 0xa3f8562e, 46 },
    { 0xe5c8562c, 33 },
    { 0xc3785630, 33 },
    { 0x18185631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed385635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd85639, 46 },
    { 0x5538563a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1848563d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31485648, 23 },
    { 0, 0 },
    { 0x30b8564a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68564f, 90 },
    { 0xe2a85650, 23 },
    { 0, 0 },
    { 0x53d85652, 55 },
    { 0x85e85653, 23 },
    { 0xde185653, 61 },
    { 0x71e85652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d8565b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e8565f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e85662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc85668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf368566b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db8566e, 64 },
    { 0xeb08566e, 91 },
    { 0xbe685670, 63 },
    { 0, 0 },
    { 0x85b85672, 90 },
    { 0xc6285673, 34 },
    { 0, 0 },
    { 0x50a85675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad85679, 90 },
    { 0, 0 },
    { 0x7d8567b, 52 },
    { 0, 0 },
    { 0xdae8567d, 46 },
    { 0x46a8567e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7685686, 43 },
    { 0x26e85687, 81 },
    { 0x82085687, 33 },
    { 0x4db85689, 43 },
    { 0x60c8568a, 76 },
    { 0x49f8568b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee8568e, 61 },
    { 0x4f28568f, 46 },
    { 0x1b885690, 61 },
    { 0x86f85690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec85694, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8385698, 68 },
    { 0xfe985698, 90 },
    { 0xc9485698, 33 },
    { 0xafe8569a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4918569e, 100 },
    { 0xc358569f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3856a3, 43 },
    { 0xbaf856a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd856a8, 95 },
    { 0x19a856a9, 90 },
    { 0x3c4856aa, 90 },
    { 0x35c856aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79856b0, 88 },
    { 0x610856b1, 98 },
    { 0, 0 },
    { 0x26f856b3, 88 },
    { 0xd0e856b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823856ba, 90 },
    { 0, 0 },
    { 0x108856bc, 95 },
    { 0x75c856bd, 61 },
    { 0, 0 },
    { 0xdfd856bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325856c3, 76 },
    { 0, 0 },
    { 0x383856c5, 90 },
    { 0, 0 },
    { 0xd67856c7, 95 },
    { 0, 0 },
    { 0x484856c9, 55 },
    { 0x4ff856c9, 33 },
    { 0xb08856ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2856ce, 95 },
    { 0x983856cf, 43 },
    { 0xf76856cf, 98 },
    { 0x9a6856d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c856e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c856e6, 90 },
    { 0, 0 },
    { 0x4db856e8, 91 },
    { 0xee4856e8, 76 },
    { 0, 0 },
    { 0x88856eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736856ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0856f2, 46 },
    { 0xdb5856f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8856f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed856fc, 76 },
    { 0x247856fc, 61 },
    { 0x507856fc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef85707, 52 },
    { 0xac185708, 33 },
    { 0x40485707, 33 },
    { 0x21c85709, 90 },
    { 0xa0d85707, 61 },
    { 0, 0 },
    { 0x1848570d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d85710, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b85716, 33 },
    { 0xb9485717, 61 },
    { 0xae385716, 98 },
    { 0x3a385719, 61 },
    { 0, 0 },
    { 0xccd8571b, 23 },
    { 0xf368571b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8571f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e85726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda985729, 61 },
    { 0x11c8572a, 90 },
    { 0xa3685729, 23 },
    { 0, 0 },
    { 0xbde8572d, 78 },
    { 0x2838572e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2785732, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b185737, 33 },
    { 0x56785738, 93 },
    { 0x72785738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e885745, 61 },
    { 0x72f85746, 76 },
    { 0xc1e85747, 91 },
    { 0x8c985746, 83 },
    { 0xf6d85747, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa85750, 46 },
    { 0x9a285751, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b85759, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e98575c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44685761, 76 },
    { 0xac785762, 80 },
    { 0xc9185763, 55 },
    { 0x9a785762, 33 },
    { 0xd7d85765, 72 },
    { 0x5bd85765, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98576a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c8577b, 61 },
    { 0x9438577b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d85786, 90 },
    { 0xbce85787, 34 },
    { 0, 0 },
    { 0xbb685789, 80 },
    { 0x6688578a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9068578d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc85791, 75 },
    { 0x7185792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3538579d, 90 },
    { 0x3008579e, 76 },
    { 0xb608579f, 43 },
    { 0, 0 },
    { 0xc04857a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed857a6, 55 },
    { 0, 0 },
    { 0xf48857a8, 98 },
    { 0xd93857a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba857ac, 55 },
    { 0xf8d857ad, 90 },
    { 0xb53857ae, 76 },
    { 0x5a5857ae, 23 },
    { 0, 0 },
    { 0x2db857b1, 46 },
    { 0x54b857b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697857b6, 90 },
    { 0x164857b7, 80 },
    { 0xe41857b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4857bc, 95 },
    { 0, 0 },
    { 0xc41857be, 98 },
    { 0x6fd857bf, 61 },
    { 0, 0 },
    { 0x5d9857c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50857ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce857cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54857d0, 61 },
    { 0xa0c857d1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f857d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x320857d7, 90 },
    { 0x3e9857d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x423857db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4857de, 76 },
    { 0, 0 },
    { 0x23e857e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6857e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1857e7, 65 },
    { 0xaf857e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93857eb, 60 },
    { 0, 0 },
    { 0x433857ed, 95 },
    { 0xf70857ee, 95 },
    { 0x144857ed, 61 },
    { 0, 0 },
    { 0xed8857f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd857f4, 98 },
    { 0xe97857f5, 34 },
    { 0, 0 },
    { 0xde6857f7, 23 },
    { 0x79d857f8, 90 },
    { 0x21a857f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x384857fc, 81 },
    { 0, 0 },
    { 0x202857fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff785801, 75 },
    { 0x3f485802, 77 },
    { 0x7f685802, 23 },
    { 0x34285804, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18580d, 98 },
    { 0xeb68580e, 33 },
    { 0x9268580d, 89 },
    { 0, 0 },
    { 0x70085811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc585814, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f85818, 23 },
    { 0, 0 },
    { 0x9458581a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc588581f, 61 },
    { 0, 0 },
    { 0xf9685821, 61 },
    { 0, 0 },
    { 0xcf885823, 85 },
    { 0x55d85824, 55 },
    { 0xb5985825, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8582d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa685830, 61 },
    { 0xc6e85831, 98 },
    { 0x98885832, 33 },
    { 0xd3485832, 83 },
    { 0x71385833, 76 },
    { 0xe7a85833, 99 },
    { 0xa5985834, 61 },
    { 0xba085837, 61 },
    { 0xfb285837, 42 },
    { 0x2be85839, 95 },
    { 0x69c85839, 95 },
    { 0x7848583b, 91 },
    { 0x3978583c, 95 },
    { 0x6e38583c, 93 },
    { 0x54785837, 95 },
    { 0x6d68583f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d885842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a85845, 55 },
    { 0x14e85846, 61 },
    { 0x58e85847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c8584d, 46 },
    { 0, 0 },
    { 0xe028584f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85852, 93 },
    { 0, 0 },
    { 0xcbf85854, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30185858, 34 },
    { 0, 0 },
    { 0xea58585a, 84 },
    { 0xb068585b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac85860, 90 },
    { 0x2c985861, 93 },
    { 0xa0085861, 33 },
    { 0xe5485861, 75 },
    { 0, 0 },
    { 0xebf85865, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78586c, 90 },
    { 0, 0 },
    { 0x2b48586e, 100 },
    { 0xa998586f, 44 },
    { 0xbc18586e, 23 },
    { 0x99f85870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80885877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba8587a, 98 },
    { 0xf2d8587a, 98 },
    { 0, 0 },
    { 0x3bd8587d, 67 },
    { 0, 0 },
    { 0x3d18587f, 93 },
    { 0x15785880, 17 },
    { 0, 0 },
    { 0x32d85882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785885, 98 },
    { 0x37385886, 85 },
    { 0x74585887, 61 },
    { 0xad585886, 61 },
    { 0xcd985888, 91 },
    { 0x95e8588a, 98 },
    { 0xe3885886, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb428588e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73585891, 90 },
    { 0xf8485891, 95 },
    { 0x83e85892, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc85896, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c85899, 91 },
    { 0xf5f8589a, 61 },
    { 0x3128589b, 83 },
    { 0x6c08589c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471858a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0858a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5858aa, 46 },
    { 0xd858ab, 77 },
    { 0x989858ab, 95 },
    { 0x6c2858ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86858b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8858bb, 33 },
    { 0x95b858bb, 100 },
    { 0x6c1858bb, 93 },
    { 0xc8b858be, 9 },
    { 0xd39858bb, 17 },
    { 0x82d858bb, 95 },
    { 0x5eb858be, 46 },
    { 0, 0 },
    { 0xa2858c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3858c6, 23 },
    { 0, 0 },
    { 0xf53858c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54858cc, 68 },
    { 0, 0 },
    { 0x75f858ce, 95 },
    { 0x55d858ce, 34 },
    { 0x149858cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4858d7, 90 },
    { 0, 0 },
    { 0x759858d9, 90 },
    { 0, 0 },
    { 0x885858db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb858e1, 43 },
    { 0xd39858e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1858e6, 61 },
    { 0x91c858e7, 93 },
    { 0xbeb858e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe858f9, 95 },
    { 0, 0 },
    { 0x71d858fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a858fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe85901, 80 },
    { 0xc6385902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36585909, 91 },
    { 0, 0 },
    { 0x3888590b, 43 },
    { 0x49c8590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67085910, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f85914, 91 },
    { 0x8a785915, 98 },
    { 0, 0 },
    { 0xdae85917, 34 },
    { 0x67485918, 23 },
    { 0x94785917, 69 },
    { 0, 0 },
    { 0x8da8591b, 46 },
    { 0, 0 },
    { 0xc68591d, 98 },
    { 0, 0 },
    { 0x6b58591f, 100 },
    { 0, 0 },
    { 0xe5a85921, 33 },
    { 0x37785921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1185925, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb585928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7298592d, 76 },
    { 0x9bf8592e, 98 },
    { 0x1028592d, 95 },
    { 0x5bd85930, 61 },
    { 0x6fa85931, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418593a, 22 },
    { 0, 0 },
    { 0x2a98593c, 98 },
    { 0xa28593d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55085940, 88 },
    { 0xc5185941, 90 },
    { 0x85985940, 65 },
    { 0, 0 },
    { 0x87785944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb278594a, 23 },
    { 0x9288594b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48594f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24185955, 22 },
    { 0x5a385956, 98 },
    { 0xa0985956, 95 },
    { 0xc3885958, 95 },
    { 0, 0 },
    { 0xfa78595a, 61 },
    { 0xc578595b, 66 },
    { 0, 0 },
    { 0xd9c8595d, 55 },
    { 0xc8595e, 33 },
    { 0xf0c8595f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba85962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f85967, 33 },
    { 0x82c85967, 91 },
    { 0x7b685969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8588596c, 33 },
    { 0x8b78596c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68885978, 99 },
    { 0x80085979, 23 },
    { 0xb4b8597a, 87 },
    { 0x91f8597b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a85981, 66 },
    { 0x88f85981, 15 },
    { 0, 0 },
    { 0x56485984, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e785989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe85991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef885998, 43 },
    { 0x32285998, 99 },
    { 0xc4d8599a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2859a1, 76 },
    { 0xaaf859a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f859a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e859a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546859b1, 93 },
    { 0xe83859b2, 23 },
    { 0x662859b3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713859bd, 76 },
    { 0x94f859be, 43 },
    { 0x8f859be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x711859c2, 61 },
    { 0, 0 },
    { 0x251859c4, 61 },
    { 0x340859c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x384859c8, 91 },
    { 0xb4a859c9, 46 },
    { 0xd61859c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x272859cd, 61 },
    { 0, 0 },
    { 0x435859cf, 33 },
    { 0x480859cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639859d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81859d7, 95 },
    { 0x169859d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f859de, 92 },
    { 0xcf0859df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0859e4, 91 },
    { 0x257859e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f859ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b859f0, 66 },
    { 0, 0 },
    { 0xb5e859f2, 55 },
    { 0x527859f3, 46 },
    { 0xbc8859f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3859fa, 33 },
    { 0xa6b859fb, 43 },
    { 0, 0 },
    { 0xa1859fd, 94 },
    { 0xef7859fe, 78 },
    { 0, 0 },
    { 0xf1a85a00, 34 },
    { 0xc2b85a00, 87 },
    { 0xfbf85a00, 85 },
    { 0x2d85a03, 95 },
    { 0x3df85a04, 95 },
    { 0xa2885a04, 43 },
    { 0xc6085a04, 40 },
    { 0, 0 },
    { 0x24885a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76285a0e, 52 },
    { 0, 0 },
    { 0x86385a10, 76 },
    { 0, 0 },
    { 0xd0385a12, 98 },
    { 0xfea85a12, 33 },
    { 0xab685a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf85a19, 84 },
    { 0, 0 },
    { 0x17985a1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38785a1e, 66 },
    { 0, 0 },
    { 0x13585a20, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc585a24, 76 },
    { 0, 0 },
    { 0xe0085a26, 33 },
    { 0, 0 },
    { 0x19585a28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c885a2b, 76 },
    { 0x5485a2c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b885a31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff885a38, 66 },
    { 0x71d85a39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a085a41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f385a48, 98 },
    { 0, 0 },
    { 0x46385a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe85a53, 55 },
    { 0x64885a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34685a58, 95 },
    { 0xd4985a59, 38 },
    { 0, 0 },
    { 0x67985a5b, 61 },
    { 0, 0 },
    { 0x38285a5d, 68 },
    { 0xc4185a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e85a66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7985a6a, 90 },
    { 0xbad85a6b, 55 },
    { 0x7ee85a6b, 98 },
    { 0x9c285a6b, 23 },
    { 0xb0385a6b, 90 },
    { 0xefa85a6f, 33 },
    { 0x4a285a6d, 90 },
    { 0, 0 },
    { 0xa9785a72, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd85a75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b85a79, 33 },
    { 0xcd85a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f885a7d, 34 },
    { 0x4f685a7e, 93 },
    { 0x1c885a7f, 98 },
    { 0xae185a80, 90 },
    { 0x75f85a7f, 66 },
    { 0x3aa85a82, 90 },
    { 0x95e85a83, 56 },
    { 0xafc85a82, 69 },
    { 0, 0 },
    { 0xd0b85a86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e885a8e, 90 },
    { 0xa0b85a8f, 90 },
    { 0x3c485a90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee085a98, 37 },
    { 0xe885a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64085a9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc85aa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe85aa4, 34 },
    { 0x15b85aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4085aa9, 34 },
    { 0xc1385aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785ab4, 95 },
    { 0x32585ab4, 23 },
    { 0xda285ab6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39085abe, 24 },
    { 0x5f985abe, 43 },
    { 0, 0 },
    { 0x53c85ac1, 48 },
    { 0x2ac85ac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd285ac7, 95 },
    { 0xc7d85ac8, 99 },
    { 0x77f85ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c85acf, 61 },
    { 0x48285ad0, 95 },
    { 0xae285ad1, 23 },
    { 0x5e085ad2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7185ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e85ada, 90 },
    { 0x1a185adb, 90 },
    { 0xb7a85adc, 88 },
    { 0xff585adb, 76 },
    { 0xb0f85add, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c85ae7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f685aea, 95 },
    { 0x7e385aeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c85af0, 33 },
    { 0x62685af1, 93 },
    { 0x1ae85af2, 23 },
    { 0xdc385af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad85af6, 33 },
    { 0x64285af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2885afa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8285b01, 61 },
    { 0, 0 },
    { 0xbfd85b03, 33 },
    { 0x1d85b03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed85b0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61785b15, 97 },
    { 0xfcb85b16, 99 },
    { 0, 0 },
    { 0xddd85b18, 33 },
    { 0x3da85b19, 75 },
    { 0xc2685b1a, 98 },
    { 0xddf85b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7885b1e, 98 },
    { 0, 0 },
    { 0x79e85b20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb85b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e285b28, 33 },
    { 0x8aa85b29, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d85b2c, 34 },
    { 0x21285b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda885b31, 46 },
    { 0, 0 },
    { 0x82e85b33, 77 },
    { 0xe0d85b34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd85b38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd85b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0785b48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e385b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42385b50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f85b54, 90 },
    { 0, 0 },
    { 0x9af85b56, 90 },
    { 0xdda85b56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33885b5a, 33 },
    { 0, 0 },
    { 0x7b485b5c, 34 },
    { 0x5de85b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80785b63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f85b66, 99 },
    { 0, 0 },
    { 0x94385b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f585b6b, 98 },
    { 0, 0 },
    { 0x42f85b6d, 93 },
    { 0, 0 },
    { 0x1ca85b6f, 90 },
    { 0xef885b70, 38 },
    { 0x91785b70, 90 },
    { 0x3c185b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8785b77, 43 },
    { 0xa1285b78, 33 },
    { 0xfd185b79, 98 },
    { 0x37a85b78, 95 },
    { 0, 0 },
    { 0xf3185b7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f85b7f, 55 },
    { 0xf8085b80, 93 },
    { 0x5aa85b81, 43 },
    { 0x87685b80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3485b85, 23 },
    { 0xaad85b86, 61 },
    { 0xfe285b86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e985b8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd85b8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b85b92, 99 },
    { 0x7ed85b93, 80 },
    { 0xc3a85b93, 92 },
    { 0xe7b85b95, 76 },
    { 0x13a85b95, 61 },
    { 0xbc385b92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9185ba0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f285ba4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19685ba8, 55 },
    { 0, 0 },
    { 0x60585baa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8885bb0, 61 },
    { 0xa6885bb0, 33 },
    { 0, 0 },
    { 0xcbc85bb3, 66 },
    { 0x56685bb4, 87 },
    { 0xd0e85bb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e285bbe, 98 },
    { 0x50185bbf, 90 },
    { 0xb1185bc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70385bc4, 98 },
    { 0, 0 },
    { 0x6a885bc6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e85bc9, 98 },
    { 0x12b85bca, 49 },
    { 0xa6585bc9, 95 },
    { 0xd1685bc9, 76 },
    { 0xa1585bcc, 33 },
    { 0xcb485bce, 99 },
    { 0, 0 },
    { 0xd6c85bd0, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa85bd4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96685bd8, 90 },
    { 0x79d85bd9, 33 },
    { 0xf3085bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde85be1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf985be4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0485be7, 52 },
    { 0x4f085be8, 61 },
    { 0x51b85be9, 75 },
    { 0x3d485be9, 100 },
    { 0x90c85beb, 61 },
    { 0x68a85bec, 78 },
    { 0x21e85bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5585bf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2585bf4, 76 },
    { 0xf7b85bf5, 61 },
    { 0xdba85bf5, 95 },
    { 0x1ac85bf7, 98 },
    { 0xaf785bf8, 95 },
    { 0xa0d85bf4, 90 },
    { 0xa3585bfa, 93 },
    { 0x96585bfb, 53 },
    { 0x5fc85bfc, 98 },
    { 0x1f285bfb, 61 },
    { 0xc5385bfc, 90 },
    { 0xe2185bff, 46 },
    { 0xb0185c00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7885c08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48885c0c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf785c12, 97 },
    { 0, 0 },
    { 0xc1f85c14, 95 },
    { 0x45185c15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59985c19, 52 },
    { 0xd7085c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10485c1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d85c27, 85 },
    { 0xe7585c28, 90 },
    { 0x5bb85c27, 84 },
    { 0xc9985c29, 61 },
    { 0, 0 },
    { 0x50485c2c, 52 },
    { 0x78385c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e85c31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b85c36, 23 },
    { 0xcdc85c37, 93 },
    { 0xf8985c36, 100 },
    { 0x37685c39, 43 },
    { 0x68185c3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e685c3f, 90 },
    { 0xb5d85c40, 61 },
    { 0x37885c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b885c45, 78 },
    { 0xa285c45, 95 },
    { 0xda485c47, 23 },
    { 0x10385c47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085c4d, 61 },
    { 0, 0 },
    { 0x4ef85c4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8685c52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad85c59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22085c5c, 51 },
    { 0, 0 },
    { 0xad685c5e, 34 },
    { 0x21f85c5e, 33 },
    { 0x4b685c5f, 36 },
    { 0, 0 },
    { 0xa3485c62, 23 },
    { 0xef85c63, 17 },
    { 0x25a85c64, 55 },
    { 0xcb685c65, 90 },
    { 0xb8b85c65, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca85c6f, 43 },
    { 0xd7785c6f, 61 },
    { 0, 0 },
    { 0x62b85c72, 52 },
    { 0x6b685c73, 90 },
    { 0x34985c74, 76 },
    { 0xf3785c73, 98 },
    { 0xe1c85c76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe85c7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c85c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0885c8a, 33 },
    { 0xc1985c8a, 93 },
    { 0, 0 },
    { 0x98685c8d, 77 },
    { 0x4e485c8e, 95 },
    { 0x2b585c8d, 90 },
    { 0x15b85c90, 33 },
    { 0, 0 },
    { 0xf8c85c92, 43 },
    { 0xcb085c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a85c96, 95 },
    { 0x16085c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685c9a, 98 },
    { 0xbf085c9a, 76 },
    { 0x9c985c9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a85ca7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41385cad, 95 },
    { 0x74f85cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec285cb1, 39 },
    { 0x61b85cb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb785cb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b85cb9, 33 },
    { 0xeb685cba, 46 },
    { 0xda185cb9, 99 },
    { 0x91085cbc, 99 },
    { 0x9d385cbc, 23 },
    { 0, 0 },
    { 0x89d85cbf, 23 },
    { 0, 0 },
    { 0xb785cc1, 95 },
    { 0xd4e85cc2, 23 },
    { 0x36985cc2, 55 },
    { 0, 0 },
    { 0x5db85cc5, 90 },
    { 0x5c585cc6, 76 },
    { 0xc5b85cc5, 34 },
    { 0x11b85cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e385ccb, 93 },
    { 0x90685ccc, 55 },
    { 0x1bb85ccd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7785cd3, 76 },
    { 0x27f85cd4, 90 },
    { 0xc6185cd3, 55 },
    { 0xdb185cd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b85cda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac85cdf, 98 },
    { 0x7f85ce0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa385ce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c785ce7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1985cee, 61 },
    { 0, 0 },
    { 0xb4885cf0, 81 },
    { 0xbfb85cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b85cf9, 23 },
    { 0x50a85cfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84985cff, 61 },
    { 0, 0 },
    { 0x21085d01, 46 },
    { 0, 0 },
    { 0x14c85d03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c85d06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74085d09, 55 },
    { 0x1fe85d0a, 84 },
    { 0xee585d0b, 52 },
    { 0x41385d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c85d12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac85d16, 34 },
    { 0, 0 },
    { 0x26b85d18, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf85d22, 93 },
    { 0, 0 },
    { 0xcd785d24, 98 },
    { 0xdc085d25, 23 },
    { 0xda585d26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a85d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a585d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f85d41, 95 },
    { 0, 0 },
    { 0x1285d43, 61 },
    { 0, 0 },
    { 0xd1185d45, 43 },
    { 0, 0 },
    { 0xf5985d47, 61 },
    { 0xfe585d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16385d4c, 61 },
    { 0x18085d4d, 83 },
    { 0x6c285d4e, 33 },
    { 0x8685d4f, 34 },
    { 0xac185d50, 23 },
    { 0x3dd85d51, 66 },
    { 0xa5885d51, 61 },
    { 0x7d185d53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4985d5e, 43 },
    { 0, 0 },
    { 0xf4385d60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b585d6c, 95 },
    { 0x5cf85d6d, 90 },
    { 0x73b85d6d, 23 },
    { 0x64c85d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0685d75, 98 },
    { 0xbf085d76, 55 },
    { 0xefc85d77, 66 },
    { 0x8de85d76, 46 },
    { 0xd7785d78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6385d7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73785d83, 33 },
    { 0, 0 },
    { 0x6685d85, 90 },
    { 0xf8b85d86, 84 },
    { 0x4bc85d87, 33 },
    { 0x11785d88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff85d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8285d8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe85d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb85d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd385d9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb85d9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c85da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49985db3, 68 },
    { 0, 0 },
    { 0x19c85db5, 90 },
    { 0, 0 },
    { 0x68c85db7, 67 },
    { 0, 0 },
    { 0x19f85db9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1085dbd, 85 },
    { 0xa1b85dbd, 61 },
    { 0x2d785dbf, 46 },
    { 0x8285dbf, 98 },
    { 0x11185dbe, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a85dc8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x64485dcb, 61 },
    { 0xafe85dcc, 93 },
    { 0x75785dcc, 61 },
    { 0xb3585dce, 90 },
    { 0x9485dce, 90 },
    { 0x10285dcb, 98 },
    { 0xce85dce, 99 },
    { 0, 0 },
    { 0xdd185dd3, 23 },
    { 0, 0 },
    { 0x78a85dd5, 76 },
    { 0x3f585dd5, 95 },
    { 0x96185dd7, 33 },
    { 0x66b85dd6, 98 },
    { 0x14285dd9, 34 },
    { 0xa0f85dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd685de2, 55 },
    { 0xea385de2, 61 },
    { 0xc5685de3, 95 },
    { 0xb3185de4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x28585de8, 23 },
    { 0, 0 },
    { 0xf0c85dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51985ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51985df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13585df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d785dfc, 93 },
    { 0x55485dfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45285e01, 98 },
    { 0x20c85e01, 95 },
    { 0xfc085e03, 93 },
    { 0xf9985e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b85e07, 85 },
    { 0, 0 },
    { 0x7ce85e09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd85e0f, 90 },
    { 0xef685e0f, 90 },
    { 0x1c985e11, 33 },
    { 0, 0 },
    { 0x4c685e13, 90 },
    { 0xc9285e14, 61 },
    { 0x94485e14, 91 },
    { 0x80a85e16, 98 },
    { 0x38a85e14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5785e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7785e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9785e23, 46 },
    { 0x41b85e24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f85e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785e2f, 95 },
    { 0xed385e30, 90 },
    { 0xd0685e31, 61 },
    { 0xf5285e32, 90 },
    { 0xd5b85e32, 77 },
    { 0, 0 },
    { 0xe8a85e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86885e38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2785e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a85e42, 68 },
    { 0xe0f85e43, 33 },
    { 0x6fc85e42, 55 },
    { 0x29485e42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f485e4c, 77 },
    { 0, 0 },
    { 0xd9485e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85885e51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce85e55, 33 },
    { 0x72185e55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82785e5d, 98 },
    { 0xdb785e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac85e66, 88 },
    { 0x7aa85e67, 98 },
    { 0, 0 },
    { 0x43c85e69, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c585e6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea085e6f, 33 },
    { 0, 0 },
    { 0x86285e71, 33 },
    { 0x9685e72, 90 },
    { 0xf7f85e72, 34 },
    { 0xa585e74, 98 },
    { 0x31b85e75, 34 },
    { 0x5685e74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa85e7a, 61 },
    { 0xc6085e7b, 67 },
    { 0x9a185e7c, 23 },
    { 0x6ff85e7b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b985e80, 95 },
    { 0, 0 },
    { 0x5c485e82, 90 },
    { 0, 0 },
    { 0x1ef85e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67085e92, 33 },
    { 0, 0 },
    { 0x55a85e94, 34 },
    { 0xcdb85e94, 95 },
    { 0xded85e95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf85e9a, 87 },
    { 0xfdd85e9b, 33 },
    { 0x30d85e9a, 33 },
    { 0xbaa85e9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a85ea4, 61 },
    { 0x96085ea5, 100 },
    { 0x60585ea6, 78 },
    { 0xab485ea5, 46 },
    { 0x14785ea6, 55 },
    { 0x2bd85ea6, 33 },
    { 0, 0 },
    { 0x7e585eab, 95 },
    { 0x9da85eac, 65 },
    { 0x24885eab, 52 },
    { 0xc5485eac, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b85eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b85eb6, 83 },
    { 0xb3085eb6, 33 },
    { 0, 0 },
    { 0x84a85eb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6285ebc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1685ebf, 23 },
    { 0, 0 },
    { 0x68185ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a085ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b385eca, 85 },
    { 0xd2b85eca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99585ece, 52 },
    { 0, 0 },
    { 0x6c685ed0, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33385ed6, 95 },
    { 0xd9885ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96085eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd85ede, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xae385ee1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f85ee5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a85ee9, 33 },
    { 0x87485eea, 23 },
    { 0x99685eeb, 100 },
    { 0x7d985eec, 90 },
    { 0x76d85eec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa85efa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80085f03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x13585f06, 87 },
    { 0xdac85f07, 34 },
    { 0, 0 },
    { 0xaa585f09, 93 },
    { 0x88485f0a, 33 },
    { 0, 0 },
    { 0xef585f0c, 33 },
    { 0, 0 },
    { 0x5ae85f0e, 23 },
    { 0, 0 },
    { 0xfb85f10, 43 },
    { 0xa5985f11, 23 },
    { 0x7d585f12, 100 },
    { 0x20485f12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0185f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6985f19, 61 },
    { 0x26785f1a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb485f23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5885f2a, 55 },
    { 0, 0 },
    { 0xc9585f2c, 55 },
    { 0x87d85f2c, 85 },
    { 0xfd085f2c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65485f33, 80 },
    { 0x53985f34, 46 },
    { 0xc2c85f34, 83 },
    { 0xd5185f36, 43 },
    { 0x18d85f37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c85f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b085f3d, 47 },
    { 0, 0 },
    { 0xece85f3f, 100 },
    { 0x4d185f40, 90 },
    { 0, 0 },
    { 0x87685f42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d585f45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f85f48, 77 },
    { 0xc5385f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded85f4d, 98 },
    { 0x2d85f4e, 78 },
    { 0x59e85f4d, 17 },
    { 0x59985f4f, 75 },
    { 0x23a85f51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d785f59, 90 },
    { 0, 0 },
    { 0x8de85f5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9685f5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44485f62, 78 },
    { 0, 0 },
    { 0x84885f64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785f68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18385f6b, 95 },
    { 0x1f885f6b, 76 },
    { 0xc9e85f6d, 88 },
    { 0xc7585f6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f85f71, 77 },
    { 0x55e85f71, 66 },
    { 0, 0 },
    { 0x53785f74, 98 },
    { 0, 0 },
    { 0xebe85f76, 93 },
    { 0, 0 },
    { 0xc1285f78, 77 },
    { 0x56985f78, 76 },
    { 0x51b85f7a, 55 },
    { 0x70a85f7b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8785f7e, 61 },
    { 0xd2385f7e, 52 },
    { 0, 0 },
    { 0x5aa85f81, 61 },
    { 0x4bd85f82, 46 },
    { 0, 0 },
    { 0x93585f84, 46 },
    { 0, 0 },
    { 0xa9f85f86, 40 },
    { 0x97c85f87, 23 },
    { 0, 0 },
    { 0x2285f89, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a185f8e, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d085f91, 46 },
    { 0x3a85f92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de85f96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82085f99, 90 },
    { 0x30785f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13785f9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c085fa5, 33 },
    { 0, 0 },
    { 0x9685fa7, 46 },
    { 0x4c785fa8, 33 },
    { 0x22e85fa7, 33 },
    { 0x41885fa7, 56 },
    { 0x97b85fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf85fae, 95 },
    { 0x64d85faf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82685fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d185fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f85fc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c85fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71785fd3, 95 },
    { 0, 0 },
    { 0xe0785fd5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb85fd8, 43 },
    { 0x36085fd9, 65 },
    { 0x5a385fda, 33 },
    { 0x2cd85fda, 55 },
    { 0x82485fdc, 23 },
    { 0x1d485fda, 90 },
    { 0xd1385fde, 33 },
    { 0x38c85fde, 90 },
    { 0xecf85fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b985fe3, 90 },
    { 0, 0 },
    { 0xcf085fe5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d85feb, 80 },
    { 0, 0 },
    { 0x57785fed, 87 },
    { 0x13185fed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e85ff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a85ff9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x76585ffc, 91 },
    { 0, 0 },
    { 0x29485ffe, 93 },
    { 0xe3085fff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a86009, 99 },
    { 0xe6e86009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c8600e, 23 },
    { 0xc798600f, 34 },
    { 0, 0 },
    { 0x22186011, 98 },
    { 0, 0 },
    { 0xb3486013, 33 },
    { 0, 0 },
    { 0xa7f86015, 19 },
    { 0xfd986016, 90 },
    { 0xc1886017, 95 },
    { 0x40f86018, 98 },
    { 0x9ad86018, 61 },
    { 0x7fe8601a, 90 },
    { 0x7f686018, 77 },
    { 0x1a986019, 46 },
    { 0xe5c8601d, 43 },
    { 0x9b18601a, 95 },
    { 0x64e8601f, 55 },
    { 0x9658601f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c86023, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f86026, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9358602a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe86030, 46 },
    { 0x9a786030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9986034, 90 },
    { 0x25886035, 61 },
    { 0xbf186036, 78 },
    { 0xf3686035, 83 },
    { 0, 0 },
    { 0xba286039, 90 },
    { 0x95086039, 95 },
    { 0, 0 },
    { 0x76e8603c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb086047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf8604c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45186050, 90 },
    { 0, 0 },
    { 0x91286052, 66 },
    { 0, 0 },
    { 0xe2486054, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf586057, 76 },
    { 0xecd86057, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8605f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84286064, 55 },
    { 0x5cb86065, 33 },
    { 0xfdc86066, 23 },
    { 0x8cf86067, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a28606b, 43 },
    { 0xafe8606c, 33 },
    { 0x7a88606c, 61 },
    { 0xce08606e, 66 },
    { 0x40e8606d, 66 },
    { 0x7f186070, 55 },
    { 0x1d886071, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f86079, 100 },
    { 0xbe88607a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9786081, 52 },
    { 0xd1c86082, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52986086, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xad886089, 76 },
    { 0x8258608a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac8608e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c86091, 75 },
    { 0x29986091, 23 },
    { 0, 0 },
    { 0x97b86094, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b98609a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868609d, 95 },
    { 0x4158609d, 61 },
    { 0xc748609f, 98 },
    { 0xb518609d, 90 },
    { 0x83860a1, 81 },
    { 0x9a860a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a860a5, 52 },
    { 0xbc0860a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x378860a9, 84 },
    { 0x7a860aa, 36 },
    { 0xb0c860ab, 81 },
    { 0x49f860ab, 95 },
    { 0x9c3860aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7860b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3860b7, 55 },
    { 0x192860b8, 68 },
    { 0, 0 },
    { 0x5ac860ba, 82 },
    { 0xdbd860bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91860c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f860c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc860ca, 100 },
    { 0xfb4860cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3860cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4860d4, 76 },
    { 0x379860d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e860d9, 67 },
    { 0xf67860da, 76 },
    { 0x2c4860db, 97 },
    { 0xfb5860dc, 34 },
    { 0, 0 },
    { 0x47a860de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3860e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62860e7, 93 },
    { 0x669860e8, 100 },
    { 0, 0 },
    { 0x6ec860ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df860ed, 55 },
    { 0x891860ee, 90 },
    { 0x152860ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00860f7, 67 },
    { 0, 0 },
    { 0x346860f9, 43 },
    { 0, 0 },
    { 0x58860fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28610c, 99 },
    { 0, 0 },
    { 0x9228610e, 99 },
    { 0, 0 },
    { 0x75f86110, 34 },
    { 0xa9d86111, 95 },
    { 0xa7286112, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff86118, 77 },
    { 0, 0 },
    { 0xdbe8611a, 65 },
    { 0, 0 },
    { 0xc7e8611c, 95 },
    { 0, 0 },
    { 0x2798611e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb486125, 90 },
    { 0x9d386126, 90 },
    { 0, 0 },
    { 0x9a586128, 23 },
    { 0x67786129, 85 },
    { 0, 0 },
    { 0xbb98612b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d886134, 95 },
    { 0, 0 },
    { 0x53186136, 61 },
    { 0x88386136, 93 },
    { 0x8fc86138, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb486140, 23 },
    { 0xbb586141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a86144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb086147, 90 },
    { 0, 0 },
    { 0xe0086149, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9718614d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4286159, 66 },
    { 0xb3686159, 34 },
    { 0x7fa8615b, 43 },
    { 0xa68615a, 23 },
    { 0x4b28615c, 95 },
    { 0x3698615e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5886167, 81 },
    { 0xe6d86168, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2408616d, 55 },
    { 0x7698616e, 61 },
    { 0, 0 },
    { 0x2986170, 95 },
    { 0, 0 },
    { 0x1b186172, 90 },
    { 0x29486172, 94 },
    { 0x7c786174, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26386179, 90 },
    { 0x23686179, 33 },
    { 0xc248617b, 33 },
    { 0, 0 },
    { 0x5a28617d, 91 },
    { 0xba8617e, 33 },
    { 0x62f8617e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f86183, 98 },
    { 0x46886183, 46 },
    { 0x2a986184, 61 },
    { 0x87f86186, 76 },
    { 0, 0 },
    { 0x70086188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de8618b, 33 },
    { 0, 0 },
    { 0xaa18618d, 90 },
    { 0xef98618e, 84 },
    { 0x3348618e, 76 },
    { 0xab086190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0086198, 33 },
    { 0x51586199, 23 },
    { 0xb7086199, 66 },
    { 0x4218619b, 46 },
    { 0xf818619a, 55 },
    { 0x3d78619a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1861a1, 88 },
    { 0xf56861a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63861a7, 98 },
    { 0x7dc861a8, 61 },
    { 0x1d4861a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da861ad, 95 },
    { 0x34c861ad, 33 },
    { 0x15861af, 90 },
    { 0x872861b0, 61 },
    { 0, 0 },
    { 0xcfc861b2, 95 },
    { 0, 0 },
    { 0x13c861b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331861b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512861be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8861c2, 93 },
    { 0, 0 },
    { 0x779861c4, 23 },
    { 0x76861c4, 98 },
    { 0, 0 },
    { 0xd4a861c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7861ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4861cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb861d3, 61 },
    { 0, 0 },
    { 0x4fc861d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x306861d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a861dc, 100 },
    { 0, 0 },
    { 0xc3b861de, 61 },
    { 0, 0 },
    { 0xfe1861e0, 61 },
    { 0xf8b861e1, 61 },
    { 0, 0 },
    { 0x8b6861e3, 61 },
    { 0xe7c861e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0861e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75861ea, 95 },
    { 0, 0 },
    { 0x16b861ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a861f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267861f4, 76 },
    { 0, 0 },
    { 0x789861f6, 93 },
    { 0, 0 },
    { 0x505861f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48861fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27586200, 23 },
    { 0, 0 },
    { 0x6e86202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a86205, 36 },
    { 0x62186206, 61 },
    { 0, 0 },
    { 0x85386208, 23 },
    { 0x68286209, 98 },
    { 0xff58620a, 55 },
    { 0, 0 },
    { 0xfb18620c, 61 },
    { 0x858620d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f286211, 98 },
    { 0x4c386211, 90 },
    { 0x1686213, 95 },
    { 0xdf186213, 61 },
    { 0xce886215, 93 },
    { 0xa9786216, 33 },
    { 0x51a86216, 61 },
    { 0x35d86211, 95 },
    { 0xa486219, 33 },
    { 0xf9e8621a, 52 },
    { 0x20686219, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de86220, 33 },
    { 0xcc086221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda186224, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9886228, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1748622c, 33 },
    { 0x8c48622d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c86230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16386233, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386237, 33 },
    { 0x13e86238, 66 },
    { 0, 0 },
    { 0x61d8623a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018623e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65886241, 90 },
    { 0xcb686242, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8624a, 23 },
    { 0x5fb8624b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88624e, 66 },
    { 0, 0 },
    { 0xfd886250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb286254, 90 },
    { 0, 0 },
    { 0x93b86256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98625a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398625d, 77 },
    { 0x6078625e, 87 },
    { 0x36b8625d, 90 },
    { 0xfdd86260, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c86263, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee786267, 95 },
    { 0, 0 },
    { 0x41586269, 34 },
    { 0x7a786269, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c8626d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf786277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8627b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb886280, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4486285, 61 },
    { 0, 0 },
    { 0x49d86287, 81 },
    { 0xe2786288, 90 },
    { 0xe2686289, 90 },
    { 0xdd686288, 61 },
    { 0x5a86289, 34 },
    { 0xaf38628c, 61 },
    { 0x17c86289, 23 },
    { 0x4eb8628e, 99 },
    { 0x75a8628e, 33 },
    { 0x5486290, 98 },
    { 0x74786288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5886296, 61 },
    { 0xf1f86297, 84 },
    { 0xfdc86298, 33 },
    { 0x99e86299, 76 },
    { 0xe1f86299, 55 },
    { 0x4e98629b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934862a1, 42 },
    { 0x371862a2, 34 },
    { 0, 0 },
    { 0x5b4862a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c862a7, 61 },
    { 0x371862a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8862ac, 90 },
    { 0, 0 },
    { 0x6b7862ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0862b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f862b4, 34 },
    { 0x9fd862b5, 61 },
    { 0, 0 },
    { 0xc70862b7, 61 },
    { 0, 0 },
    { 0xbae862b9, 88 },
    { 0, 0 },
    { 0x44862bb, 61 },
    { 0xab6862bb, 76 },
    { 0xe862bd, 87 },
    { 0, 0 },
    { 0x149862bf, 77 },
    { 0x198862bf, 61 },
    { 0x83d862c1, 33 },
    { 0, 0 },
    { 0xe7862c3, 95 },
    { 0x13c862c3, 84 },
    { 0xab9862c5, 61 },
    { 0xf03862c6, 90 },
    { 0, 0 },
    { 0x2af862c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c862ce, 55 },
    { 0xfbf862cf, 61 },
    { 0x3f0862cf, 91 },
    { 0x793862d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9862d5, 46 },
    { 0xfc7862d5, 23 },
    { 0xb05862d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x927862da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13862e0, 34 },
    { 0, 0 },
    { 0x9e9862e2, 48 },
    { 0x39c862e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785862ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66862f0, 61 },
    { 0x89862f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f862f4, 90 },
    { 0, 0 },
    { 0xe41862f6, 33 },
    { 0xa83862f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x712862fa, 52 },
    { 0, 0 },
    { 0x393862fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46486300, 95 },
    { 0x44f86300, 23 },
    { 0x5df86301, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa086306, 61 },
    { 0, 0 },
    { 0xc1886308, 87 },
    { 0xa7d86308, 95 },
    { 0xd2f8630a, 61 },
    { 0x4e386308, 93 },
    { 0xe7e8630a, 98 },
    { 0, 0 },
    { 0x2308630e, 43 },
    { 0, 0 },
    { 0x19f86310, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5586313, 33 },
    { 0, 0 },
    { 0xf5086315, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd186318, 77 },
    { 0xecf86319, 95 },
    { 0x3d38631a, 33 },
    { 0xbe48631b, 81 },
    { 0, 0 },
    { 0xb648631d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8586320, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1086323, 100 },
    { 0, 0 },
    { 0xc8286325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f86328, 90 },
    { 0x27586329, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4118632e, 60 },
    { 0xef08632f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63586338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08633c, 46 },
    { 0x48e8633d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf986341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56386344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb86347, 87 },
    { 0, 0 },
    { 0xa7f86349, 23 },
    { 0xc0186349, 52 },
    { 0xf0186349, 95 },
    { 0x20f8634c, 100 },
    { 0, 0 },
    { 0xd978634e, 74 },
    { 0xfa98634f, 93 },
    { 0x9b486350, 23 },
    { 0xb7886351, 90 },
    { 0xe7f86351, 95 },
    { 0x43d86352, 34 },
    { 0, 0 },
    { 0x4f986355, 55 },
    { 0x53d86356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79186359, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548635e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b86362, 90 },
    { 0xf3a86363, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd86367, 33 },
    { 0, 0 },
    { 0x59886369, 17 },
    { 0xfab8636a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1986370, 33 },
    { 0x19686371, 33 },
    { 0xb6686372, 95 },
    { 0, 0 },
    { 0xfe886374, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9068637a, 95 },
    { 0x8d68637b, 95 },
    { 0x2768637b, 76 },
    { 0, 0 },
    { 0x6ec8637e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x79186381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4286385, 52 },
    { 0x92486385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10486389, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf168638c, 23 },
    { 0, 0 },
    { 0xa298638e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34586392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae86395, 34 },
    { 0, 0 },
    { 0xf5586397, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d8639a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e88639f, 23 },
    { 0xc21863a0, 85 },
    { 0x6f7863a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2863a4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8863aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73863ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f863b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488863b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309863be, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09863c8, 98 },
    { 0x4db863c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0863cc, 61 },
    { 0, 0 },
    { 0x2cc863ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1863d8, 91 },
    { 0x304863d8, 91 },
    { 0xd34863da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08863e2, 95 },
    { 0xe6a863e3, 46 },
    { 0xdfe863e4, 61 },
    { 0xbfc863e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d863e8, 33 },
    { 0xb1f863e9, 81 },
    { 0x57e863e9, 33 },
    { 0, 0 },
    { 0x4d863ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60863ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1863f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6863fd, 43 },
    { 0, 0 },
    { 0xce6863ff, 61 },
    { 0xe3286400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16086405, 90 },
    { 0, 0 },
    { 0xd8786407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d86416, 90 },
    { 0xeb286417, 90 },
    { 0x16a86417, 76 },
    { 0xb0b86419, 98 },
    { 0x20c86417, 61 },
    { 0, 0 },
    { 0x8918641c, 95 },
    { 0x178641c, 98 },
    { 0xdd48641d, 76 },
    { 0x67d8641c, 95 },
    { 0x86f8641c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3078642e, 61 },
    { 0, 0 },
    { 0x37486430, 55 },
    { 0x95f86431, 40 },
    { 0xdb586432, 95 },
    { 0x1db86433, 33 },
    { 0, 0 },
    { 0xfbd86435, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b786439, 23 },
    { 0x1b88643a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c8643d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1086447, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd938644d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f586453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb786456, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f8645b, 55 },
    { 0x28645c, 61 },
    { 0, 0 },
    { 0x8d28645e, 98 },
    { 0x1a88645f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35486462, 90 },
    { 0, 0 },
    { 0x30586464, 33 },
    { 0, 0 },
    { 0x59a86466, 95 },
    { 0xf8b86467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf8646a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e8646d, 23 },
    { 0, 0 },
    { 0xe7c8646f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93086483, 33 },
    { 0x5ba86484, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11486487, 61 },
    { 0x8886487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b8648c, 50 },
    { 0, 0 },
    { 0xc188648e, 88 },
    { 0x958648f, 33 },
    { 0xfb086490, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9486495, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa328649b, 84 },
    { 0xfef8649b, 61 },
    { 0, 0 },
    { 0x6fb8649e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485864a9, 96 },
    { 0xe1864aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf864ad, 99 },
    { 0x2e6864ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b864b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9864bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d864c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef864c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a864d0, 61 },
    { 0x98a864d0, 90 },
    { 0, 0 },
    { 0x397864d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846864da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0864de, 88 },
    { 0, 0 },
    { 0xc27864e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699864e4, 34 },
    { 0x5d864e5, 91 },
    { 0x578864e5, 95 },
    { 0x72f864e7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c864ec, 46 },
    { 0x104864ed, 46 },
    { 0x62c864ee, 33 },
    { 0x840864ed, 38 },
    { 0x8d7864f0, 23 },
    { 0x1a3864f0, 61 },
    { 0xda9864f1, 33 },
    { 0x7c0864f2, 90 },
    { 0xb2b864f2, 33 },
    { 0xdb8864f5, 98 },
    { 0xe15864f5, 23 },
    { 0, 0 },
    { 0x5a5864f8, 84 },
    { 0x7e7864f8, 33 },
    { 0xfc4864f9, 75 },
    { 0xe99864fb, 33 },
    { 0x2c8864fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede86500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59886506, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa308650a, 87 },
    { 0x9508650b, 90 },
    { 0xd818650c, 61 },
    { 0x2c8650b, 65 },
    { 0xe4f8650e, 33 },
    { 0xd0d8650f, 34 },
    { 0xd3386510, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f8651a, 23 },
    { 0xeb08651b, 95 },
    { 0x7008651b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8651f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42586523, 98 },
    { 0xc6b86523, 90 },
    { 0x97886525, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d86529, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb86530, 87 },
    { 0xbdf86531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e86536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8653f, 33 },
    { 0xb8f8653f, 46 },
    { 0x5ac86541, 33 },
    { 0x6d686542, 43 },
    { 0xfe686542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c86547, 66 },
    { 0, 0 },
    { 0xbc586549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8654d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab286554, 99 },
    { 0xdd486555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c38655b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8655e, 90 },
    { 0xb808655f, 76 },
    { 0xe4986560, 23 },
    { 0, 0 },
    { 0x2c286562, 59 },
    { 0x9dd86562, 76 },
    { 0xe3286564, 99 },
    { 0x20986565, 33 },
    { 0, 0 },
    { 0x96b86567, 95 },
    { 0, 0 },
    { 0xf9586569, 90 },
    { 0xd18656a, 88 },
    { 0x3908656a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed586570, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd486573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a686579, 33 },
    { 0xe508657a, 67 },
    { 0, 0 },
    { 0x5d78657c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d8657f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad886583, 33 },
    { 0xb6386583, 87 },
    { 0, 0 },
    { 0x18986586, 66 },
    { 0x2f786587, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e58658b, 95 },
    { 0, 0 },
    { 0xb98658d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f486592, 98 },
    { 0x7f586593, 38 },
    { 0, 0 },
    { 0x52486595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c86598, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f8659b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d865a0, 98 },
    { 0, 0 },
    { 0xebe865a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442865a7, 90 },
    { 0xbfe865a7, 99 },
    { 0x109865a9, 95 },
    { 0, 0 },
    { 0xd17865ab, 87 },
    { 0x9af865ac, 90 },
    { 0, 0 },
    { 0xc38865ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50865b2, 90 },
    { 0xd2e865b3, 90 },
    { 0x7865b4, 61 },
    { 0x516865b3, 95 },
    { 0x9ec865b3, 95 },
    { 0x89b865b7, 43 },
    { 0x98f865b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1865bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x576865c0, 61 },
    { 0x21a865c0, 67 },
    { 0, 0 },
    { 0x71e865c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905865c8, 55 },
    { 0, 0 },
    { 0x78c865ca, 90 },
    { 0x285865cb, 95 },
    { 0xe19865cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71865d0, 52 },
    { 0x92d865d0, 23 },
    { 0, 0 },
    { 0x685865d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e865d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53865d9, 85 },
    { 0x317865da, 33 },
    { 0x44a865db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71865de, 90 },
    { 0x17d865df, 61 },
    { 0, 0 },
    { 0xd7d865e1, 77 },
    { 0x886865e1, 95 },
    { 0, 0 },
    { 0x273865e4, 100 },
    { 0, 0 },
    { 0xbc7865e6, 68 },
    { 0xc05865e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8865ed, 90 },
    { 0x8e9865ee, 23 },
    { 0, 0 },
    { 0xbb4865f0, 90 },
    { 0xb9865f1, 61 },
    { 0x8e865f2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb865f5, 33 },
    { 0x11c865f6, 61 },
    { 0, 0 },
    { 0x7f6865f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x101865fb, 33 },
    { 0, 0 },
    { 0x2d5865fd, 61 },
    { 0x63f865fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x41186601, 90 },
    { 0x7e986602, 33 },
    { 0x51786603, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x47786606, 90 },
    { 0xdd486607, 90 },
    { 0x96d86608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e38660b, 87 },
    { 0x47a8660c, 84 },
    { 0x18e8660d, 43 },
    { 0xbee8660c, 12 },
    { 0xa078660f, 85 },
    { 0, 0 },
    { 0xd7986611, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d86614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2948661c, 61 },
    { 0x60f8661c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf086622, 95 },
    { 0x48f86623, 55 },
    { 0x88286623, 37 },
    { 0xcc686624, 34 },
    { 0x65d86626, 33 },
    { 0xe6d86627, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38662b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88662e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc786633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba38663c, 34 },
    { 0x408663d, 61 },
    { 0x2318663e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47586644, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24086647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18664c, 76 },
    { 0x7318664c, 76 },
    { 0, 0 },
    { 0x8598664f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5886656, 61 },
    { 0, 0 },
    { 0x60686658, 33 },
    { 0, 0 },
    { 0xf188665a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8665e, 83 },
    { 0xef48665f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd886663, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f86667, 100 },
    { 0, 0 },
    { 0x7cf86669, 33 },
    { 0xc7086669, 33 },
    { 0x8208666b, 66 },
    { 0x17f8666b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e8666f, 90 },
    { 0xa3386670, 88 },
    { 0xc3f86670, 61 },
    { 0xdc686672, 33 },
    { 0xdf886670, 61 },
    { 0, 0 },
    { 0x4ca86675, 61 },
    { 0, 0 },
    { 0xa1586677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18667c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a86688, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xf548668b, 65 },
    { 0x5bf8668c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78668f, 78 },
    { 0x73286690, 78 },
    { 0, 0 },
    { 0xd286692, 93 },
    { 0x1e86693, 23 },
    { 0x35186694, 98 },
    { 0xc186695, 33 },
    { 0xbe986693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb8669a, 65 },
    { 0x8bd8669b, 88 },
    { 0xe4c8669c, 95 },
    { 0xaa88669a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd866a0, 95 },
    { 0xde8866a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14866a8, 95 },
    { 0xf84866a8, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8866ac, 91 },
    { 0, 0 },
    { 0xc1866ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68866b2, 90 },
    { 0, 0 },
    { 0x6b5866b4, 33 },
    { 0x519866b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8866b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x281866bb, 55 },
    { 0x3c0866bc, 68 },
    { 0x1ca866bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e866c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf866c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91866ca, 98 },
    { 0, 0 },
    { 0xdca866cc, 95 },
    { 0xe94866cc, 90 },
    { 0xc23866ce, 52 },
    { 0xa32866ce, 84 },
    { 0xaa2866d0, 61 },
    { 0xb26866cf, 93 },
    { 0x721866cf, 95 },
    { 0x3ce866d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa866d7, 33 },
    { 0x1ba866d8, 95 },
    { 0xa36866d8, 46 },
    { 0, 0 },
    { 0x421866db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d866df, 34 },
    { 0x59a866e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f866e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1866e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c866eb, 100 },
    { 0, 0 },
    { 0x8aa866ed, 94 },
    { 0, 0 },
    { 0xf5866ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f866f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e866f7, 90 },
    { 0x348866f7, 43 },
    { 0x380866f9, 77 },
    { 0x67c866fa, 23 },
    { 0xee8866fa, 33 },
    { 0x2ae866f9, 67 },
    { 0, 0 },
    { 0xe84866fe, 23 },
    { 0xf1d866fe, 95 },
    { 0, 0 },
    { 0xe4086701, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc486705, 95 },
    { 0x91386706, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac86709, 23 },
    { 0xc578670a, 90 },
    { 0x1b68670b, 90 },
    { 0, 0 },
    { 0x5d18670d, 67 },
    { 0x1828670e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x99286711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98286715, 46 },
    { 0, 0 },
    { 0x21c86717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d786722, 23 },
    { 0x38286722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31386727, 33 },
    { 0xe8f86728, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b78672c, 93 },
    { 0x8d48672c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe886735, 17 },
    { 0xa4786736, 76 },
    { 0x7c686737, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158673e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b786742, 76 },
    { 0, 0 },
    { 0x48b86744, 90 },
    { 0x97086745, 87 },
    { 0x43386744, 47 },
    { 0x1e386746, 53 },
    { 0xce886748, 55 },
    { 0xa3586749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a8674c, 46 },
    { 0x44b8674c, 90 },
    { 0xa118674c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d386751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b986754, 90 },
    { 0, 0 },
    { 0xf9c86756, 75 },
    { 0x15886757, 90 },
    { 0xd886758, 56 },
    { 0x9e186759, 23 },
    { 0xdb18675a, 98 },
    { 0x5886757, 77 },
    { 0, 0 },
    { 0xabb8675d, 98 },
    { 0x2a8675d, 46 },
    { 0, 0 },
    { 0x4af86760, 84 },
    { 0xfa586760, 95 },
    { 0x72086762, 90 },
    { 0x80086761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7086768, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd8676b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce86771, 85 },
    { 0xad086772, 66 },
    { 0xcd286772, 90 },
    { 0xb8086773, 90 },
    { 0, 0 },
    { 0xf3386776, 76 },
    { 0, 0 },
    { 0x6cb86778, 76 },
    { 0xa8c86778, 81 },
    { 0xa7c86778, 33 },
    { 0x71d8677b, 99 },
    { 0, 0 },
    { 0x17d8677d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac86780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd686787, 66 },
    { 0xf1986788, 91 },
    { 0xbea86787, 100 },
    { 0xa048678a, 90 },
    { 0x98c8678b, 33 },
    { 0, 0 },
    { 0x7d18678d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10486790, 90 },
    { 0xb9186791, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d86798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8398679b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d867a1, 34 },
    { 0, 0 },
    { 0x7d867a3, 76 },
    { 0x9a4867a4, 95 },
    { 0xbf8867a5, 33 },
    { 0x179867a6, 91 },
    { 0x24f867a6, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e867b1, 95 },
    { 0xa6a867b2, 76 },
    { 0xcdc867b1, 61 },
    { 0x706867b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809867b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c867be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b867c4, 66 },
    { 0x8ad867c5, 28 },
    { 0x3e3867c5, 61 },
    { 0x4b9867c7, 95 },
    { 0x3ba867c4, 48 },
    { 0, 0 },
    { 0x412867ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1867d6, 52 },
    { 0xf40867d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db867da, 43 },
    { 0x30b867da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c867de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a867eb, 98 },
    { 0, 0 },
    { 0x735867ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3867fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c86801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1286804, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c086808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48680f, 95 },
    { 0x23f8680f, 55 },
    { 0xc2786811, 23 },
    { 0xa5886812, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa786816, 87 },
    { 0xb6f86817, 78 },
    { 0, 0 },
    { 0xc086819, 61 },
    { 0, 0 },
    { 0x47b8681b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7268681e, 34 },
    { 0xc698681e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79886822, 33 },
    { 0, 0 },
    { 0xe2886824, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59786828, 61 },
    { 0, 0 },
    { 0x5e18682a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c686831, 61 },
    { 0xe286831, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20786838, 23 },
    { 0, 0 },
    { 0xf2b8683a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5648683d, 90 },
    { 0xd378683d, 23 },
    { 0x4e58683f, 61 },
    { 0xf2f86840, 98 },
    { 0x8b986841, 77 },
    { 0x12086842, 76 },
    { 0xb586842, 100 },
    { 0x50586840, 96 },
    { 0, 0 },
    { 0xd5f86846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c86849, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46586850, 98 },
    { 0x19d86851, 90 },
    { 0xc8a86852, 61 },
    { 0xc5486852, 33 },
    { 0x65786851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42386858, 80 },
    { 0xd4c86859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5038685c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9986864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e8686c, 67 },
    { 0, 0 },
    { 0x6ad8686e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5086872, 46 },
    { 0x6f286872, 46 },
    { 0xdc686874, 78 },
    { 0x10786874, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d86878, 95 },
    { 0x46f86879, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8687d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d86881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe786886, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f86889, 93 },
    { 0x5798688a, 98 },
    { 0x6a08688b, 61 },
    { 0xbaf8688a, 61 },
    { 0x81b86889, 90 },
    { 0, 0 },
    { 0x938688f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa486892, 75 },
    { 0x81986893, 33 },
    { 0x75886894, 61 },
    { 0x15986894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae86899, 53 },
    { 0, 0 },
    { 0x2b28689b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329868a1, 33 },
    { 0, 0 },
    { 0x437868a3, 68 },
    { 0x30b868a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc868a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87868ad, 98 },
    { 0xbd4868ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a868b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab868b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44868bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c868c4, 98 },
    { 0xe77868c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b868d1, 33 },
    { 0x63c868d2, 95 },
    { 0xfd6868d1, 46 },
    { 0, 0 },
    { 0x438868d5, 90 },
    { 0xc30868d5, 33 },
    { 0, 0 },
    { 0xbb9868d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc868db, 95 },
    { 0x522868db, 23 },
    { 0x663868dd, 78 },
    { 0xefe868dd, 90 },
    { 0xb868dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x372868e2, 96 },
    { 0xfbb868e3, 33 },
    { 0, 0 },
    { 0xca4868e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b868e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04868ec, 90 },
    { 0x9de868ec, 61 },
    { 0x45b868ee, 61 },
    { 0, 0 },
    { 0xbff868f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3868f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e868f6, 76 },
    { 0x167868f7, 36 },
    { 0x24c868f8, 100 },
    { 0x523868f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x871868fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51686900, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab986905, 76 },
    { 0xac486905, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc086909, 95 },
    { 0x1dd8690a, 17 },
    { 0xd348690a, 90 },
    { 0xd5f86909, 78 },
    { 0x5e8690d, 90 },
    { 0xd288690e, 95 },
    { 0x56e8690e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed086913, 88 },
    { 0, 0 },
    { 0xa4286915, 23 },
    { 0xa3b86916, 90 },
    { 0xca786917, 87 },
    { 0x45d86916, 77 },
    { 0x39086919, 61 },
    { 0x64286916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x948691d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf386922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec86927, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c86931, 95 },
    { 0xb4986931, 76 },
    { 0xf5b86933, 36 },
    { 0, 0 },
    { 0x94686935, 33 },
    { 0x10e86936, 55 },
    { 0x25986935, 55 },
    { 0x74a86937, 93 },
    { 0x44686935, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068693c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85786942, 99 },
    { 0, 0 },
    { 0x7d586944, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8286947, 33 },
    { 0xcef86947, 43 },
    { 0xb6c86948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8468694c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2686951, 98 },
    { 0x55486952, 84 },
    { 0xc9086951, 95 },
    { 0, 0 },
    { 0x70f86955, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c486959, 55 },
    { 0xef98695a, 84 },
    { 0xef08695b, 67 },
    { 0, 0 },
    { 0xbc08695d, 33 },
    { 0, 0 },
    { 0xc038695f, 52 },
    { 0xa908695f, 17 },
    { 0, 0 },
    { 0x2d486962, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52086969, 34 },
    { 0, 0 },
    { 0x73c8696b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf548696e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e286976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61086979, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e8697c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8786980, 93 },
    { 0x70286981, 34 },
    { 0x99d86981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c86988, 38 },
    { 0xa3686989, 68 },
    { 0x8478698a, 36 },
    { 0x2a28698b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa886996, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4486999, 90 },
    { 0x70486999, 65 },
    { 0x6c78699b, 88 },
    { 0x1dd8699c, 23 },
    { 0x40c8699a, 33 },
    { 0x4478699e, 82 },
    { 0xb9d8699c, 33 },
    { 0x2fa869a0, 34 },
    { 0xada869a0, 61 },
    { 0x6bc869a2, 98 },
    { 0x7e1869a0, 61 },
    { 0x5ce869a4, 67 },
    { 0xec8869a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55869a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5869b3, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2869bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c869c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348869c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459869c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854869d3, 84 },
    { 0xc9a869d4, 61 },
    { 0x464869d5, 90 },
    { 0x7fa869d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6869dd, 90 },
    { 0xf1e869de, 46 },
    { 0, 0 },
    { 0xb4f869e0, 67 },
    { 0, 0 },
    { 0x7b4869e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977869e6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920869eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7869f2, 61 },
    { 0xe0f869f3, 68 },
    { 0x8f0869f4, 95 },
    { 0xab1869f5, 52 },
    { 0xdef869f4, 61 },
    { 0xa55869f5, 90 },
    { 0x748869f8, 95 },
    { 0, 0 },
    { 0xbbf869fa, 43 },
    { 0xaa9869fb, 23 },
    { 0x882869fc, 34 },
    { 0x2ba869fb, 23 },
    { 0xccf869fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b86a04, 90 },
    { 0xbba86a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b86a09, 91 },
    { 0x30586a0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b86a0e, 91 },
    { 0x7b986a0f, 23 },
    { 0, 0 },
    { 0xec886a11, 76 },
    { 0xf1386a11, 43 },
    { 0xac386a13, 61 },
    { 0x4f586a14, 67 },
    { 0x7f986a14, 87 },
    { 0, 0 },
    { 0x64f86a17, 33 },
    { 0x3fb86a18, 74 },
    { 0, 0 },
    { 0x8fa86a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a486a1f, 23 },
    { 0x92286a20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b886a27, 66 },
    { 0, 0 },
    { 0xcc986a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66686a2c, 68 },
    { 0x7d86a2d, 61 },
    { 0xba586a2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24586a31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda86a36, 90 },
    { 0xf1c86a37, 98 },
    { 0x47086a37, 34 },
    { 0x5f586a38, 95 },
    { 0x3fd86a37, 98 },
    { 0x35386a3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43286a40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b86a43, 34 },
    { 0xb86a44, 98 },
    { 0, 0 },
    { 0xcd286a46, 23 },
    { 0, 0 },
    { 0x43086a48, 19 },
    { 0, 0 },
    { 0xb0086a4a, 68 },
    { 0x33286a4a, 77 },
    { 0x8ac86a4a, 76 },
    { 0x93d86a4c, 46 },
    { 0, 0 },
    { 0x2ef86a4f, 78 },
    { 0xb9886a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec86a55, 87 },
    { 0x54586a56, 95 },
    { 0xa0186a55, 90 },
    { 0x65986a58, 23 },
    { 0x13086a59, 33 },
    { 0xa8d86a57, 66 },
    { 0x2b186a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab86a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f86a65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93886a69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f86a6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96886a73, 90 },
    { 0xf0786a73, 55 },
    { 0, 0 },
    { 0x95f86a76, 95 },
    { 0, 0 },
    { 0x6686a78, 90 },
    { 0xc6486a79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca086a80, 34 },
    { 0xa4286a81, 61 },
    { 0x5a286a82, 61 },
    { 0x7b786a80, 95 },
    { 0x3bb86a82, 61 },
    { 0xafa86a85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d586a8f, 55 },
    { 0xc0286a90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62286a93, 23 },
    { 0xd6186a93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68686a9a, 85 },
    { 0, 0 },
    { 0x44786a9c, 61 },
    { 0xfca86a9d, 76 },
    { 0x83d86a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c986aa2, 90 },
    { 0, 0 },
    { 0x2dd86aa4, 90 },
    { 0, 0 },
    { 0x8d386aa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22986ab0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37786ab3, 43 },
    { 0, 0 },
    { 0x74b86ab5, 88 },
    { 0xb2086ab6, 33 },
    { 0, 0 },
    { 0xc5586ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4986abd, 33 },
    { 0xd5686abe, 43 },
    { 0x61186abf, 76 },
    { 0x5ed86abe, 33 },
    { 0xf2c86ac1, 84 },
    { 0xe5986ac1, 81 },
    { 0xa2286ac3, 23 },
    { 0xe3986ac4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6286ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3586ad0, 91 },
    { 0, 0 },
    { 0x5c786ad2, 23 },
    { 0, 0 },
    { 0xbd886ad4, 85 },
    { 0xc7286ad5, 61 },
    { 0, 0 },
    { 0x28586ad7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a86adb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f86ae0, 93 },
    { 0xfd86ae0, 98 },
    { 0xb7886ae2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd86ae6, 55 },
    { 0, 0 },
    { 0x9c586ae8, 46 },
    { 0x53c86ae9, 78 },
    { 0x29086aea, 34 },
    { 0x57886aeb, 61 },
    { 0xb0086aec, 61 },
    { 0x46f86aea, 99 },
    { 0x7c386aea, 61 },
    { 0x4d886aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a86af2, 66 },
    { 0, 0 },
    { 0xa1286af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd086af7, 90 },
    { 0x7a186af8, 98 },
    { 0x8d986af9, 66 },
    { 0xc1e86afa, 98 },
    { 0x64886af9, 34 },
    { 0x9c886afa, 74 },
    { 0xa4686afd, 55 },
    { 0x63a86afd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4886b06, 90 },
    { 0xa9c86b06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b86b0f, 23 },
    { 0x74386b0f, 98 },
    { 0x45286b11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2986b14, 81 },
    { 0x49986b15, 95 },
    { 0xb6386b14, 33 },
    { 0xadc86b16, 34 },
    { 0x9bc86b18, 76 },
    { 0x2ca86b16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56786b1f, 61 },
    { 0, 0 },
    { 0x4786b21, 76 },
    { 0x9b886b22, 100 },
    { 0x55b86b23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b86b29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e786b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4386b33, 34 },
    { 0, 0 },
    { 0xb6786b35, 76 },
    { 0x96e86b36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe286b3a, 61 },
    { 0x86986b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f786b43, 77 },
    { 0xdad86b44, 98 },
    { 0, 0 },
    { 0xd0586b46, 23 },
    { 0x1b586b47, 98 },
    { 0x90a86b48, 90 },
    { 0x49486b49, 76 },
    { 0xcd186b4a, 90 },
    { 0xcf386b4b, 33 },
    { 0x14886b4c, 98 },
    { 0xf8286b4d, 76 },
    { 0x7c686b4a, 76 },
    { 0x87486b4f, 90 },
    { 0, 0 },
    { 0x7aa86b51, 33 },
    { 0x56a86b52, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42186b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a86b5b, 98 },
    { 0xd5186b5c, 61 },
    { 0xf3786b5d, 90 },
    { 0x3b886b5b, 23 },
    { 0x35186b5e, 67 },
    { 0xa1186b60, 23 },
    { 0xbb786b60, 61 },
    { 0, 0 },
    { 0x9d886b63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b486b67, 40 },
    { 0xb8686b68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15986b6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38586b6f, 43 },
    { 0xa2686b6f, 61 },
    { 0x17386b71, 95 },
    { 0xd086b6f, 46 },
    { 0xbb386b70, 23 },
    { 0x35d86b74, 78 },
    { 0xceb86b75, 98 },
    { 0x44986b76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e586b7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4986b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74386b87, 61 },
    { 0x77386b88, 52 },
    { 0, 0 },
    { 0xbf286b8a, 90 },
    { 0, 0 },
    { 0x6e286b8c, 88 },
    { 0x11386b8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f86b97, 90 },
    { 0, 0 },
    { 0xf9e86b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f86b9f, 93 },
    { 0x23b86b9f, 61 },
    { 0x6ab86ba0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66086ba4, 23 },
    { 0xb8486ba5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84186bad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75586bb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1386bbd, 90 },
    { 0x17a86bbe, 91 },
    { 0, 0 },
    { 0x3e286bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41086bc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd86bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d886bcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a286bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b886bd2, 69 },
    { 0xdd586bd3, 98 },
    { 0xe8d86bd3, 23 },
    { 0xb5a86bd3, 65 },
    { 0x5286bd3, 90 },
    { 0x6286bd6, 61 },
    { 0xe9f86bd8, 61 },
    { 0x2ed86bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42886be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80886bea, 67 },
    { 0x1b686beb, 67 },
    { 0xa5486bec, 76 },
    { 0x8286bed, 43 },
    { 0, 0 },
    { 0x54d86bef, 90 },
    { 0x75086bf0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8786bf4, 61 },
    { 0xf4286bf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e186bf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72186bfd, 91 },
    { 0, 0 },
    { 0x4d286bff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf86c07, 85 },
    { 0, 0 },
    { 0x9b486c09, 81 },
    { 0xe3186c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a86c12, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d86c17, 76 },
    { 0, 0 },
    { 0xfaa86c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa86c20, 46 },
    { 0x7e886c21, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f86c28, 90 },
    { 0, 0 },
    { 0xbbd86c2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd786c31, 95 },
    { 0, 0 },
    { 0x5b786c33, 99 },
    { 0xacc86c34, 52 },
    { 0x50486c35, 23 },
    { 0x38e86c36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed86c39, 76 },
    { 0x1c786c39, 34 },
    { 0x19d86c3b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41986c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30986c44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae886c4c, 98 },
    { 0x6f086c4d, 98 },
    { 0, 0 },
    { 0x3e886c4f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f86c54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79386c58, 52 },
    { 0xc6c86c59, 76 },
    { 0xbf686c59, 96 },
    { 0xd0d86c5b, 98 },
    { 0, 0 },
    { 0x65186c5d, 90 },
    { 0, 0 },
    { 0xfa486c5f, 77 },
    { 0x3ff86c60, 76 },
    { 0, 0 },
    { 0x77f86c62, 98 },
    { 0x78586c62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd86c66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e86c6a, 90 },
    { 0x64186c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d86c6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35586c78, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a286c7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f86c81, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a86c88, 23 },
    { 0xbba86c89, 95 },
    { 0, 0 },
    { 0xa5a86c8b, 77 },
    { 0x2e086c8c, 95 },
    { 0xfe886c8c, 61 },
    { 0xc6986c8d, 23 },
    { 0xb5686c8f, 33 },
    { 0, 0 },
    { 0xd2286c91, 90 },
    { 0xe1f86c92, 33 },
    { 0x41b86c93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23986c96, 84 },
    { 0, 0 },
    { 0xafc86c98, 85 },
    { 0xcca86c99, 40 },
    { 0, 0 },
    { 0x3d786c9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57686c9f, 61 },
    { 0xb1686c9f, 61 },
    { 0x2b686c9f, 84 },
    { 0xc2086ca2, 33 },
    { 0, 0 },
    { 0x71686ca4, 98 },
    { 0x9f86ca4, 95 },
    { 0x40986ca6, 61 },
    { 0xb1e86ca7, 33 },
    { 0x4e686ca5, 46 },
    { 0, 0 },
    { 0x91b86caa, 23 },
    { 0, 0 },
    { 0xb0786cac, 90 },
    { 0x81c86cad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f86cb0, 98 },
    { 0, 0 },
    { 0x4986cb2, 91 },
    { 0xfd686cb3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0986cb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5086cba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19386cbd, 61 },
    { 0xabb86cbe, 90 },
    { 0xbd86cbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f86cc3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b086cc7, 61 },
    { 0xa0f86cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42186ccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f86ccf, 90 },
    { 0x64286cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62486cd3, 99 },
    { 0xfd986cd4, 90 },
    { 0, 0 },
    { 0xc4886cd6, 61 },
    { 0x6af86cd7, 46 },
    { 0xf386cd7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b86cdc, 90 },
    { 0, 0 },
    { 0xe8f86cde, 34 },
    { 0x58d86cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0186ce4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c86cea, 90 },
    { 0xe8786ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42586cef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c86cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a86cf6, 90 },
    { 0xe9086cf7, 43 },
    { 0x1f686cf7, 98 },
    { 0xbed86cf9, 33 },
    { 0x6bd86cfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba86cfd, 68 },
    { 0xc2886cfd, 95 },
    { 0xc4086cfe, 85 },
    { 0x28886cfe, 98 },
    { 0xfee86cfd, 34 },
    { 0xc3986d02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f86d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d86d0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef86d0f, 90 },
    { 0x92086d10, 98 },
    { 0xbb086d0f, 46 },
    { 0x3be86d11, 91 },
    { 0, 0 },
    { 0x30b86d14, 34 },
    { 0x81186d15, 61 },
    { 0xb6086d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a686d19, 95 },
    { 0, 0 },
    { 0x37486d1b, 95 },
    { 0x3f086d1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b86d1f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a386d23, 85 },
    { 0xf2586d24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40386d2b, 23 },
    { 0, 0 },
    { 0x5e586d2d, 98 },
    { 0xc6286d2e, 100 },
    { 0x3ec86d2f, 23 },
    { 0xb5d86d2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5386d36, 76 },
    { 0xddf86d37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95086d3b, 101 },
    { 0xfe086d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73086d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67686d43, 98 },
    { 0x41986d43, 67 },
    { 0xb4f86d44, 23 },
    { 0x28f86d44, 63 },
    { 0xa8086d47, 90 },
    { 0xc6e86d45, 33 },
    { 0x7a386d49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc586d4d, 92 },
    { 0x71886d4e, 68 },
    { 0x29886d4e, 98 },
    { 0, 0 },
    { 0x77086d51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20186d55, 69 },
    { 0, 0 },
    { 0xfbd86d57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f386d5b, 33 },
    { 0xca86d5b, 90 },
    { 0, 0 },
    { 0xa2f86d5e, 95 },
    { 0, 0 },
    { 0x55586d60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49686d64, 61 },
    { 0x21586d65, 78 },
    { 0xe1986d66, 95 },
    { 0x1a586d67, 87 },
    { 0, 0 },
    { 0x86d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30386d6d, 90 },
    { 0xdf086d6e, 23 },
    { 0, 0 },
    { 0x67386d70, 95 },
    { 0xf3586d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1486d74, 68 },
    { 0, 0 },
    { 0x3c486d76, 43 },
    { 0, 0 },
    { 0xcef86d78, 33 },
    { 0, 0 },
    { 0xd8e86d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a786d82, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e86d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60786d8a, 78 },
    { 0x34e86d8b, 95 },
    { 0x19f86d8c, 33 },
    { 0x6fb86d8d, 61 },
    { 0x32286d8e, 68 },
    { 0x68186d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b86d93, 61 },
    { 0x52d86d93, 61 },
    { 0x83e86d95, 98 },
    { 0xe8d86d95, 90 },
    { 0xaa186d97, 67 },
    { 0xa0186d98, 33 },
    { 0x68886d95, 55 },
    { 0x8c86d9a, 90 },
    { 0, 0 },
    { 0x31a86d9c, 33 },
    { 0, 0 },
    { 0xa2f86d9e, 23 },
    { 0x15f86d9f, 66 },
    { 0x52286d9f, 98 },
    { 0x87486da1, 66 },
    { 0xdb186d9e, 91 },
    { 0, 0 },
    { 0x17b86da4, 34 },
    { 0xff386da5, 34 },
    { 0, 0 },
    { 0xe4f86da7, 76 },
    { 0x28f86da8, 95 },
    { 0, 0 },
    { 0x2c486daa, 52 },
    { 0x5c386dab, 55 },
    { 0, 0 },
    { 0x83a86dad, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd086db0, 43 },
    { 0x1986db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50786db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4186dbb, 95 },
    { 0x50b86dbc, 101 },
    { 0, 0 },
    { 0xc9686dbe, 66 },
    { 0xc4a86dbf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75186dc4, 61 },
    { 0, 0 },
    { 0xc7786dc6, 90 },
    { 0x62486dc7, 90 },
    { 0xd9a86dc8, 90 },
    { 0xa9c86dc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb86dcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86dd3, 98 },
    { 0, 0 },
    { 0x9ad86dd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd86dd8, 95 },
    { 0x8f786dd9, 76 },
    { 0x11b86dda, 85 },
    { 0xc586ddb, 75 },
    { 0x74e86ddb, 90 },
    { 0xdb886ddd, 82 },
    { 0x9a786dda, 87 },
    { 0, 0 },
    { 0x9fd86de0, 88 },
    { 0x3cf86de1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20286de7, 98 },
    { 0x47e86de8, 95 },
    { 0x24d86de8, 98 },
    { 0xe1686dea, 90 },
    { 0, 0 },
    { 0x91086dec, 61 },
    { 0, 0 },
    { 0xb5586dee, 90 },
    { 0x6ce86dee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7486df2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e86df7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d686dfa, 33 },
    { 0xea486dfa, 95 },
    { 0x56d86dfa, 61 },
    { 0xb1986dfd, 33 },
    { 0xaf286dfe, 90 },
    { 0x64786dff, 90 },
    { 0xf8286dfd, 43 },
    { 0xcda86e01, 90 },
    { 0x6aa86e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe786e05, 23 },
    { 0, 0 },
    { 0xf3186e07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d86e0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0886e11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb786e18, 66 },
    { 0, 0 },
    { 0x5a286e1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8386e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df86e2b, 98 },
    { 0, 0 },
    { 0x1ca86e2d, 90 },
    { 0xb1786e2e, 98 },
    { 0xd3986e2f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19686e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a386e37, 33 },
    { 0x20886e37, 61 },
    { 0x486e39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48186e40, 90 },
    { 0x94686e41, 61 },
    { 0, 0 },
    { 0x6c486e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50886e4c, 34 },
    { 0, 0 },
    { 0x3c86e4e, 98 },
    { 0xd7886e4e, 46 },
    { 0x2e586e4f, 90 },
    { 0xdd786e4f, 51 },
    { 0xe6686e52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e86e58, 95 },
    { 0x80e86e58, 61 },
    { 0x65386e5a, 78 },
    { 0xd386e5b, 99 },
    { 0xcb86e5c, 91 },
    { 0x22886e5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc86e60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc86e65, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57686e6a, 61 },
    { 0xcf86e6b, 23 },
    { 0x21d86e6c, 90 },
    { 0, 0 },
    { 0xa5c86e6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55486e71, 88 },
    { 0, 0 },
    { 0xfa086e73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79086e7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98586e81, 55 },
    { 0, 0 },
    { 0xd0e86e83, 90 },
    { 0, 0 },
    { 0x4b186e85, 87 },
    { 0xfcd86e86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0586e8b, 33 },
    { 0xff86e8b, 90 },
    { 0x51786e8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db86e90, 46 },
    { 0xf5e86e90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe586e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a86e9a, 78 },
    { 0x63d86e9b, 23 },
    { 0x74e86e9c, 55 },
    { 0x96f86e9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94986eb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b86eb7, 33 },
    { 0xe5886eb8, 97 },
    { 0x36686eb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53786ebd, 95 },
    { 0, 0 },
    { 0x77d86ebf, 55 },
    { 0x98186ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a86ec5, 100 },
    { 0xa6b86ec5, 98 },
    { 0xaf86ec5, 66 },
    { 0x8fb86ec8, 72 },
    { 0x38b86ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e86ecd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11486ed2, 90 },
    { 0x98e86ed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2586ed8, 43 },
    { 0x14886ed9, 61 },
    { 0xcf386eda, 81 },
    { 0x67386edb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d86ee1, 61 },
    { 0xaf586ee2, 33 },
    { 0x8d786ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb86ee6, 61 },
    { 0x1e086ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86eec, 90 },
    { 0xf1786eed, 93 },
    { 0, 0 },
    { 0xdfd86eef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf86ef2, 100 },
    { 0, 0 },
    { 0x10586ef4, 99 },
    { 0xc4986ef5, 98 },
    { 0x1dc86ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36286efb, 76 },
    { 0xb286efc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f86eff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d586f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f86f06, 77 },
    { 0x9ca86f07, 46 },
    { 0xfc386f06, 70 },
    { 0x3af86f09, 41 },
    { 0x26386f0a, 77 },
    { 0x56286f0a, 85 },
    { 0x6e86f0c, 81 },
    { 0x63f86f0c, 84 },
    { 0x2e886f0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff86f11, 76 },
    { 0, 0 },
    { 0xfd786f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf586f16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f86f1c, 61 },
    { 0xd5b86f1d, 90 },
    { 0, 0 },
    { 0xfbd86f1f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96186f22, 99 },
    { 0x38386f23, 66 },
    { 0x99a86f22, 84 },
    { 0x3bf86f23, 4 },
    { 0xf3c86f26, 98 },
    { 0x8ca86f27, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d086f32, 90 },
    { 0, 0 },
    { 0x75786f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d686f37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b286f45, 61 },
    { 0, 0 },
    { 0x46086f47, 98 },
    { 0xa2586f48, 61 },
    { 0xd5386f49, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d86f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d786f54, 76 },
    { 0, 0 },
    { 0xae286f56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xff286f59, 61 },
    { 0, 0 },
    { 0x15786f5b, 98 },
    { 0x3d786f5c, 99 },
    { 0x5cc86f5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9286f61, 91 },
    { 0, 0 },
    { 0xb9186f63, 95 },
    { 0, 0 },
    { 0x90186f65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6486f6b, 33 },
    { 0x65086f6c, 34 },
    { 0xae586f6d, 61 },
    { 0, 0 },
    { 0x27086f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6186f72, 90 },
    { 0x3ee86f73, 66 },
    { 0, 0 },
    { 0x7be86f75, 87 },
    { 0x7a986f75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9386f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31886f7d, 61 },
    { 0x99186f7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c86f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10386f8e, 94 },
    { 0, 0 },
    { 0x84686f90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1186f96, 95 },
    { 0xa2686f97, 95 },
    { 0x8286f96, 36 },
    { 0x83486f99, 61 },
    { 0x1bb86f97, 90 },
    { 0x9f786f9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a86f9e, 65 },
    { 0x5a286f9f, 90 },
    { 0xfc486f9f, 43 },
    { 0xffd86f9f, 38 },
    { 0x6b986fa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8186fa7, 95 },
    { 0x45986fa7, 95 },
    { 0xe2f86fa8, 46 },
    { 0, 0 },
    { 0xa686fab, 88 },
    { 0, 0 },
    { 0x3bd86fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b486fb2, 19 },
    { 0xf286fb3, 67 },
    { 0, 0 },
    { 0x96486fb5, 84 },
    { 0xb5c86fb6, 50 },
    { 0xa9986fb7, 95 },
    { 0, 0 },
    { 0x68e86fb9, 43 },
    { 0, 0 },
    { 0x22286fbb, 55 },
    { 0xe0486fbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10986fbf, 46 },
    { 0x8ec86fc0, 46 },
    { 0xcd286fbf, 98 },
    { 0x42e86fc2, 91 },
    { 0, 0 },
    { 0x9e786fc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x19186fc7, 66 },
    { 0x14186fc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5086fcc, 91 },
    { 0x60486fcd, 33 },
    { 0x9e486fcd, 93 },
    { 0x22886fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d86fd3, 33 },
    { 0, 0 },
    { 0xc5d86fd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc86fd9, 99 },
    { 0x49d86fda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2786fe2, 90 },
    { 0, 0 },
    { 0xc3486fe4, 17 },
    { 0xd4f86fe4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4186ff5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a86ff8, 23 },
    { 0xd3b86ff9, 61 },
    { 0x29b86ffa, 61 },
    { 0x4a186ffb, 61 },
    { 0x7a786ffb, 100 },
    { 0x22986ffc, 90 },
    { 0xaba86ffc, 34 },
    { 0x8ba86fff, 98 },
    { 0x9ed86fff, 90 },
    { 0x41f87000, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c987005, 34 },
    { 0xef687006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e87009, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f8700c, 55 },
    { 0x2d48700d, 88 },
    { 0, 0 },
    { 0x4db8700f, 98 },
    { 0, 0 },
    { 0x68b87011, 95 },
    { 0xf8e87012, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x75287015, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f87018, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8701c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc888701f, 90 },
    { 0x6f88701f, 23 },
    { 0x3787020, 61 },
    { 0xf2187022, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c87028, 93 },
    { 0x4d887028, 95 },
    { 0x6218702a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38702d, 63 },
    { 0x6c68702e, 93 },
    { 0xc428702f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83387036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3178703b, 33 },
    { 0x1408703c, 61 },
    { 0x4e68703c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b87040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd87045, 46 },
    { 0x30687045, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d687049, 61 },
    { 0, 0 },
    { 0xd7a8704b, 98 },
    { 0xa6d8704c, 93 },
    { 0x74e8704b, 76 },
    { 0, 0 },
    { 0x5868704f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec87056, 98 },
    { 0, 0 },
    { 0x33587058, 92 },
    { 0, 0 },
    { 0x81e8705a, 67 },
    { 0, 0 },
    { 0x10a8705c, 95 },
    { 0x66b8705d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61887060, 33 },
    { 0xbe287060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a387066, 34 },
    { 0x8e987067, 33 },
    { 0x71a87068, 98 },
    { 0xf5387069, 52 },
    { 0x8df87069, 90 },
    { 0, 0 },
    { 0xce98706c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487070, 33 },
    { 0xac687071, 90 },
    { 0xe5f87072, 23 },
    { 0, 0 },
    { 0x7787074, 82 },
    { 0, 0 },
    { 0xa5887076, 98 },
    { 0, 0 },
    { 0xcbb87078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8707b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987080, 23 },
    { 0, 0 },
    { 0x57887082, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a187086, 52 },
    { 0x8b987087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe8708a, 90 },
    { 0x6a28708b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c8708e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa87092, 55 },
    { 0, 0 },
    { 0xa3c87094, 84 },
    { 0x8bf87095, 33 },
    { 0x6c087095, 93 },
    { 0x29587097, 76 },
    { 0x55487096, 81 },
    { 0xf0987099, 99 },
    { 0, 0 },
    { 0xf1d8709b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa870a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb870a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15870ad, 34 },
    { 0x1d1870ae, 95 },
    { 0xc49870ae, 76 },
    { 0, 0 },
    { 0x4c4870b1, 33 },
    { 0xb9870b2, 98 },
    { 0x284870b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743870b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa870be, 90 },
    { 0xd28870be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b870ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42870cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x890870d2, 85 },
    { 0x2c4870d3, 83 },
    { 0x708870d3, 100 },
    { 0x2c5870d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f870dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556870e6, 95 },
    { 0xa56870e7, 90 },
    { 0xbc5870e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6870eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0870f3, 36 },
    { 0, 0 },
    { 0xbc3870f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e870fd, 91 },
    { 0x985870fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48487101, 34 },
    { 0, 0 },
    { 0x14c87103, 23 },
    { 0x68487104, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a8710e, 77 },
    { 0, 0 },
    { 0x63387110, 66 },
    { 0xad587111, 78 },
    { 0x4f087111, 33 },
    { 0xf7a87113, 23 },
    { 0x92687111, 99 },
    { 0x64787114, 31 },
    { 0, 0 },
    { 0x68087117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc08711d, 100 },
    { 0xd2e8711d, 95 },
    { 0xec38711f, 61 },
    { 0xf58711d, 43 },
    { 0, 0 },
    { 0x8bb87122, 76 },
    { 0xdb687123, 33 },
    { 0x8a087124, 61 },
    { 0, 0 },
    { 0x3f87126, 33 },
    { 0, 0 },
    { 0xf2a87128, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b18712f, 17 },
    { 0x40f8712f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8713c, 98 },
    { 0xcd88713d, 101 },
    { 0xf308713e, 34 },
    { 0x3a28713f, 52 },
    { 0x9438713f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc87143, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x91787146, 91 },
    { 0x26d87147, 90 },
    { 0x15f87146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd8714c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db87154, 96 },
    { 0x60387154, 90 },
    { 0xcc087154, 40 },
    { 0x69487154, 98 },
    { 0x33987158, 61 },
    { 0x37387159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c38715e, 98 },
    { 0x6f78715e, 77 },
    { 0xdfd87160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54587163, 93 },
    { 0x4f287164, 76 },
    { 0, 0 },
    { 0xbd487166, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a8716f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54387173, 78 },
    { 0xb2c87173, 95 },
    { 0xdbd87175, 90 },
    { 0, 0 },
    { 0x8187177, 55 },
    { 0, 0 },
    { 0x43c87179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc098717d, 23 },
    { 0x2c38717e, 78 },
    { 0x8c8717f, 66 },
    { 0xc778717f, 33 },
    { 0xf778717e, 48 },
    { 0xf0487180, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe687187, 90 },
    { 0x9b187188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8718f, 55 },
    { 0, 0 },
    { 0xd7187191, 76 },
    { 0xe687192, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x48587195, 44 },
    { 0x7eb87196, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec871a2, 90 },
    { 0, 0 },
    { 0x2d871a4, 90 },
    { 0, 0 },
    { 0x732871a6, 55 },
    { 0, 0 },
    { 0x85f871a8, 87 },
    { 0, 0 },
    { 0xcfc871aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102871ae, 52 },
    { 0xfe3871af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378871b3, 61 },
    { 0x193871b3, 95 },
    { 0x459871b5, 99 },
    { 0x5cd871b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de871b9, 93 },
    { 0, 0 },
    { 0x4d3871bb, 43 },
    { 0xb9d871bc, 90 },
    { 0, 0 },
    { 0x70e871be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98871c3, 90 },
    { 0, 0 },
    { 0x2ef871c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd871c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29871ce, 61 },
    { 0, 0 },
    { 0x876871d0, 95 },
    { 0, 0 },
    { 0x912871d2, 76 },
    { 0x67a871d2, 98 },
    { 0xd4f871d2, 90 },
    { 0x375871d3, 23 },
    { 0, 0 },
    { 0x7a4871d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b871db, 61 },
    { 0x996871db, 90 },
    { 0, 0 },
    { 0x84f871de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14871e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85871e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a871eb, 66 },
    { 0x7f8871eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9871f5, 98 },
    { 0xbd871f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b871fa, 95 },
    { 0, 0 },
    { 0x1f3871fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce87200, 76 },
    { 0x1a787201, 90 },
    { 0, 0 },
    { 0x6b687203, 55 },
    { 0x76a87204, 22 },
    { 0xbf487203, 98 },
    { 0x4e987205, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa87209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9138720d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b87211, 78 },
    { 0x4187211, 90 },
    { 0xe3287211, 43 },
    { 0x53487214, 91 },
    { 0, 0 },
    { 0xbc387216, 43 },
    { 0x2fc87216, 55 },
    { 0xa687216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f28721c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef28721f, 93 },
    { 0x42c87220, 61 },
    { 0x61e87220, 90 },
    { 0x8b387220, 93 },
    { 0x66787223, 90 },
    { 0xab687222, 33 },
    { 0x22e87225, 95 },
    { 0x80b87225, 90 },
    { 0, 0 },
    { 0xa0f87228, 95 },
    { 0xa6487229, 61 },
    { 0x17887228, 90 },
    { 0, 0 },
    { 0x6ef8722c, 90 },
    { 0xb38722d, 33 },
    { 0x7d58722e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c87236, 61 },
    { 0, 0 },
    { 0x87587238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd168723b, 95 },
    { 0x7d68723c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8723f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19387242, 55 },
    { 0, 0 },
    { 0xe4d87244, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d87247, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe418724c, 17 },
    { 0xe508724c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb987257, 61 },
    { 0, 0 },
    { 0x73c87259, 90 },
    { 0xa1987259, 95 },
    { 0x10587259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8725e, 33 },
    { 0x7378725e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82287264, 90 },
    { 0xab987265, 33 },
    { 0xf1b87265, 55 },
    { 0xd8787264, 61 },
    { 0x93a87268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb8726b, 87 },
    { 0x4818726c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce8726f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25087272, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b87276, 23 },
    { 0x91487276, 61 },
    { 0, 0 },
    { 0x4aa87279, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2358727c, 90 },
    { 0xd008727d, 33 },
    { 0xd28727d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb587281, 75 },
    { 0, 0 },
    { 0x47e87283, 43 },
    { 0xf1687284, 61 },
    { 0xd1e87284, 90 },
    { 0x6f487283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a18728f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f687292, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b87298, 61 },
    { 0, 0 },
    { 0x3528729a, 95 },
    { 0, 0 },
    { 0x7b08729c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd872a2, 32 },
    { 0, 0 },
    { 0x32e872a4, 68 },
    { 0x803872a5, 61 },
    { 0x95a872a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x952872a9, 61 },
    { 0x91c872a9, 99 },
    { 0x67f872a9, 55 },
    { 0x2ff872ab, 61 },
    { 0xcf6872ab, 33 },
    { 0, 0 },
    { 0x144872af, 81 },
    { 0, 0 },
    { 0x293872b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f872b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d872b7, 84 },
    { 0xe8a872b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee872bc, 95 },
    { 0, 0 },
    { 0xfd2872be, 78 },
    { 0xeb9872be, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f872c3, 90 },
    { 0, 0 },
    { 0xf91872c5, 93 },
    { 0, 0 },
    { 0x17872c7, 34 },
    { 0, 0 },
    { 0xe8c872c9, 90 },
    { 0, 0 },
    { 0xb9e872cb, 34 },
    { 0xd29872cb, 61 },
    { 0, 0 },
    { 0xd99872ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc872d3, 61 },
    { 0x940872d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6872d9, 43 },
    { 0x84a872d9, 66 },
    { 0x4bb872db, 98 },
    { 0x915872db, 33 },
    { 0, 0 },
    { 0x174872de, 66 },
    { 0x2da872df, 33 },
    { 0, 0 },
    { 0xaf2872e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59872e8, 43 },
    { 0x119872e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9872ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c872f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9872fa, 90 },
    { 0xe5d872fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812872ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d87306, 93 },
    { 0xfcb87306, 78 },
    { 0x12687307, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278730c, 99 },
    { 0xa668730d, 33 },
    { 0xd9d8730d, 61 },
    { 0x5398730f, 91 },
    { 0xa6387310, 90 },
    { 0x21c8730f, 23 },
    { 0xfa8730d, 90 },
    { 0x65d8730d, 33 },
    { 0xe8a87314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb87317, 99 },
    { 0, 0 },
    { 0x13387319, 23 },
    { 0xc998731a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a087320, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x10887323, 33 },
    { 0, 0 },
    { 0xf6487325, 55 },
    { 0, 0 },
    { 0xf5a87327, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f8732c, 61 },
    { 0, 0 },
    { 0xee98732e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f87333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce87337, 61 },
    { 0, 0 },
    { 0x5b87339, 76 },
    { 0, 0 },
    { 0x9b48733b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898733e, 55 },
    { 0xda88733f, 33 },
    { 0x4ce87340, 75 },
    { 0x61587341, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xff887344, 76 },
    { 0xbe987345, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8087349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x688734c, 91 },
    { 0, 0 },
    { 0xa718734e, 98 },
    { 0xa098734f, 66 },
    { 0xf5587350, 81 },
    { 0x99e8734e, 33 },
    { 0x27387352, 90 },
    { 0xcf587352, 32 },
    { 0xa7987352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53187359, 90 },
    { 0x9a98735a, 61 },
    { 0, 0 },
    { 0xfd8735c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96487362, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b87365, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86687368, 33 },
    { 0, 0 },
    { 0x3be8736a, 90 },
    { 0, 0 },
    { 0x75c8736c, 61 },
    { 0, 0 },
    { 0xc668736e, 34 },
    { 0, 0 },
    { 0x6d87370, 96 },
    { 0xac187371, 77 },
    { 0x9bb87372, 76 },
    { 0x33987371, 52 },
    { 0x43887372, 95 },
    { 0xd3787374, 43 },
    { 0x65987374, 93 },
    { 0x87287377, 61 },
    { 0xfa187373, 95 },
    { 0x28487379, 90 },
    { 0x287378, 61 },
    { 0x6478737b, 95 },
    { 0x4e087373, 53 },
    { 0xd418737d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38738b, 93 },
    { 0x5f68738b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf278738f, 99 },
    { 0xcdb87390, 23 },
    { 0x8b087390, 64 },
    { 0x92787391, 95 },
    { 0xc368738f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c8739d, 61 },
    { 0xc4f8739e, 90 },
    { 0xaf68739f, 33 },
    { 0xe2a8739f, 33 },
    { 0x248739d, 85 },
    { 0xc5873a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f873a6, 76 },
    { 0x12c873a6, 34 },
    { 0, 0 },
    { 0xe76873a9, 93 },
    { 0, 0 },
    { 0x4c2873ab, 52 },
    { 0, 0 },
    { 0x587873ad, 95 },
    { 0x293873ae, 90 },
    { 0x880873af, 66 },
    { 0, 0 },
    { 0x1d0873b1, 100 },
    { 0xd0e873b2, 23 },
    { 0x777873b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599873b9, 33 },
    { 0xc56873b9, 55 },
    { 0xf60873bb, 67 },
    { 0xced873bb, 90 },
    { 0x7a6873bd, 95 },
    { 0xefb873bd, 75 },
    { 0x66c873bf, 83 },
    { 0x108873ba, 95 },
    { 0xda5873c1, 95 },
    { 0x23c873ba, 55 },
    { 0xc5b873c2, 65 },
    { 0xc50873c4, 96 },
    { 0x192873c4, 76 },
    { 0xf7873c6, 52 },
    { 0x694873c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3873cb, 33 },
    { 0, 0 },
    { 0x49b873cd, 33 },
    { 0, 0 },
    { 0x263873cf, 17 },
    { 0, 0 },
    { 0x98f873d1, 88 },
    { 0x95c873d1, 61 },
    { 0xaf6873d1, 98 },
    { 0xea9873d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7873d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec873de, 23 },
    { 0, 0 },
    { 0xadd873e0, 43 },
    { 0x229873e0, 77 },
    { 0x524873e2, 61 },
    { 0x4a7873e0, 64 },
    { 0xd99873e4, 91 },
    { 0, 0 },
    { 0xe9c873e6, 90 },
    { 0xc8e873e7, 98 },
    { 0x2ac873e6, 99 },
    { 0xb17873e8, 55 },
    { 0xc04873ea, 97 },
    { 0x1e1873eb, 61 },
    { 0x7e7873ec, 43 },
    { 0xb49873ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173873f1, 33 },
    { 0xb99873f2, 43 },
    { 0x25a873f2, 76 },
    { 0x852873f2, 76 },
    { 0, 0 },
    { 0x143873f6, 90 },
    { 0xeae873f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72873fb, 93 },
    { 0x19b873fb, 33 },
    { 0x225873fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0087407, 61 },
    { 0xa9b87408, 33 },
    { 0x50c87409, 43 },
    { 0x8988740a, 46 },
    { 0x8e87407, 98 },
    { 0xdba8740b, 95 },
    { 0, 0 },
    { 0x9418740e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8587413, 17 },
    { 0x7587414, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1187417, 98 },
    { 0xdbc87417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c8741f, 61 },
    { 0x17587420, 99 },
    { 0x5e087421, 78 },
    { 0x55987422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3887425, 90 },
    { 0x19b87426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc08742e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787432, 33 },
    { 0x69387432, 85 },
    { 0x10387434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec87438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78743c, 95 },
    { 0xd1a8743c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee587440, 95 },
    { 0x587441, 61 },
    { 0xa087442, 61 },
    { 0x1b787442, 33 },
    { 0x50387442, 23 },
    { 0x8c987445, 8 },
    { 0, 0 },
    { 0xa387447, 46 },
    { 0x8bf87448, 90 },
    { 0xf4687448, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf68744e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d87454, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa87458, 78 },
    { 0x85387458, 98 },
    { 0x73287459, 38 },
    { 0x2dc8745b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8745e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76787463, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8746a, 98 },
    { 0x8fb8746a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c387470, 33 },
    { 0, 0 },
    { 0xf4a87472, 61 },
    { 0x6ec87472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d87477, 55 },
    { 0x7a87478, 61 },
    { 0x31187479, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98747c, 61 },
    { 0xfa38747d, 55 },
    { 0x268747e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d687481, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a887488, 34 },
    { 0x6ad87489, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8748c, 90 },
    { 0xe788748c, 99 },
    { 0xb178748d, 33 },
    { 0xd4e8748f, 34 },
    { 0x7f787490, 76 },
    { 0x9458748f, 77 },
    { 0x60887492, 68 },
    { 0x93d87491, 90 },
    { 0x30d87494, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f287497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68749d, 61 },
    { 0xa368749e, 33 },
    { 0x7fe8749f, 98 },
    { 0xd29874a0, 95 },
    { 0x337874a1, 98 },
    { 0x1c88749d, 23 },
    { 0xf958749d, 90 },
    { 0x6fa874a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399874a8, 90 },
    { 0, 0 },
    { 0x367874aa, 78 },
    { 0, 0 },
    { 0xf01874ac, 85 },
    { 0x728874ac, 90 },
    { 0, 0 },
    { 0x6e3874af, 52 },
    { 0x2e5874b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7874b5, 34 },
    { 0x7a874b6, 87 },
    { 0, 0 },
    { 0xf52874b8, 67 },
    { 0xfb1874b9, 97 },
    { 0x674874b8, 84 },
    { 0xe38874bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8874c4, 76 },
    { 0x950874c5, 52 },
    { 0xb6c874c4, 33 },
    { 0x246874c6, 42 },
    { 0x6d4874c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d874cc, 61 },
    { 0xc98874cd, 90 },
    { 0xb85874cd, 90 },
    { 0x837874cf, 98 },
    { 0x81874d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa874d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec874d6, 98 },
    { 0xd0874d7, 23 },
    { 0xf31874d8, 61 },
    { 0x3b1874d7, 43 },
    { 0, 0 },
    { 0x7d5874db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50874de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3874e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37874e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa874ea, 90 },
    { 0xd29874ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d874f4, 46 },
    { 0, 0 },
    { 0xb24874f6, 76 },
    { 0xf77874f7, 90 },
    { 0xbf2874f6, 98 },
    { 0xd87874f9, 55 },
    { 0xe48874fa, 61 },
    { 0x3f3874fa, 66 },
    { 0xe44874fa, 33 },
    { 0, 0 },
    { 0xc88874fe, 33 },
    { 0xf84874ff, 33 },
    { 0, 0 },
    { 0xacc87501, 90 },
    { 0, 0 },
    { 0x12587503, 90 },
    { 0, 0 },
    { 0xafc87505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158750c, 61 },
    { 0xd198750d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a287513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e287516, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58751a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78751f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e87523, 87 },
    { 0, 0 },
    { 0x1b987525, 98 },
    { 0x2a087526, 23 },
    { 0x3dc87526, 61 },
    { 0xfcc87528, 33 },
    { 0xe9587527, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8752c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a87531, 98 },
    { 0xaa87532, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x15687535, 81 },
    { 0x19d87536, 76 },
    { 0x86b87537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8118753c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb387541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7998754a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7068754d, 95 },
    { 0x40b8754e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b587551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e987555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d87558, 33 },
    { 0x89087559, 93 },
    { 0xf198755a, 46 },
    { 0x1788755a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d8755f, 43 },
    { 0x8e58755f, 33 },
    { 0xc1f87561, 33 },
    { 0x68f87562, 34 },
    { 0x4a487562, 33 },
    { 0x46a87562, 61 },
    { 0xd3e87564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a187568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2508756e, 99 },
    { 0x3298756e, 98 },
    { 0xdd87570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d987573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f87579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a8757c, 23 },
    { 0xeb28757c, 78 },
    { 0x8e28757e, 95 },
    { 0xaf88757d, 87 },
    { 0, 0 },
    { 0x1987581, 90 },
    { 0x47787581, 61 },
    { 0x3ee87583, 78 },
    { 0xac987583, 100 },
    { 0x25987584, 93 },
    { 0, 0 },
    { 0xa1887587, 61 },
    { 0, 0 },
    { 0xce187589, 95 },
    { 0x72b8758a, 90 },
    { 0x4d98758b, 23 },
    { 0x66d8758c, 90 },
    { 0, 0 },
    { 0xa908758e, 55 },
    { 0xaaf8758e, 85 },
    { 0, 0 },
    { 0x69387591, 33 },
    { 0, 0 },
    { 0xafd87593, 90 },
    { 0xc5187594, 95 },
    { 0x4fc87594, 95 },
    { 0x7dd87596, 61 },
    { 0x6ff87597, 61 },
    { 0x5a387596, 85 },
    { 0x76c87599, 46 },
    { 0, 0 },
    { 0xd578759b, 91 },
    { 0, 0 },
    { 0xe678759d, 88 },
    { 0xbfb8759e, 23 },
    { 0xef18759f, 17 },
    { 0xf85875a0, 90 },
    { 0xf3c875a1, 98 },
    { 0x38c875a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5875a7, 66 },
    { 0, 0 },
    { 0xd6e875a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82875ad, 34 },
    { 0xeea875ae, 46 },
    { 0, 0 },
    { 0x2cb875b0, 98 },
    { 0x3b6875b1, 85 },
    { 0xc74875b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47875b8, 99 },
    { 0xf7d875b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75875bf, 91 },
    { 0, 0 },
    { 0x905875c1, 84 },
    { 0, 0 },
    { 0xaeb875c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51875c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5875cc, 98 },
    { 0x42b875cd, 98 },
    { 0x16e875cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36875d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x602875d9, 90 },
    { 0, 0 },
    { 0x4dc875db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d875df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef875e2, 90 },
    { 0x921875e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93875e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289875ea, 66 },
    { 0xff4875eb, 83 },
    { 0xce5875ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab875f0, 95 },
    { 0, 0 },
    { 0xfbd875f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5875f5, 33 },
    { 0x65c875f6, 61 },
    { 0x5d5875f7, 23 },
    { 0x78b875f8, 61 },
    { 0x2c3875f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911875ff, 85 },
    { 0xe5887600, 61 },
    { 0xfff87601, 66 },
    { 0, 0 },
    { 0xef187603, 61 },
    { 0, 0 },
    { 0x97a87605, 66 },
    { 0, 0 },
    { 0xdc887607, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec38760b, 90 },
    { 0xd1c8760c, 61 },
    { 0x3258760d, 61 },
    { 0, 0 },
    { 0xbe38760f, 87 },
    { 0xf2c87610, 93 },
    { 0x4b98760f, 33 },
    { 0x19687611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b87615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298761b, 90 },
    { 0, 0 },
    { 0x4878761d, 80 },
    { 0x76a8761d, 36 },
    { 0xb318761f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8762a, 61 },
    { 0, 0 },
    { 0x9808762c, 99 },
    { 0x9df8762d, 100 },
    { 0x1328762e, 99 },
    { 0x97f8762f, 95 },
    { 0x2418762c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f687633, 46 },
    { 0xf2687634, 99 },
    { 0x89487633, 98 },
    { 0x12387634, 33 },
    { 0x91487635, 61 },
    { 0x287638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff48763e, 90 },
    { 0xaae8763f, 61 },
    { 0xb5d8763f, 99 },
    { 0x76687641, 61 },
    { 0x5cb87642, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b87645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9588764a, 23 },
    { 0, 0 },
    { 0x1978764c, 61 },
    { 0x6c8764d, 33 },
    { 0, 0 },
    { 0xbfd8764f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f687654, 95 },
    { 0, 0 },
    { 0x2aa87656, 23 },
    { 0x38787657, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb928765a, 43 },
    { 0xb958765b, 61 },
    { 0x2428765b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f887661, 85 },
    { 0, 0 },
    { 0x6b287663, 66 },
    { 0x2c187664, 61 },
    { 0xe6d87665, 43 },
    { 0x70087665, 80 },
    { 0x84887665, 99 },
    { 0, 0 },
    { 0x1087669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3487675, 17 },
    { 0, 0 },
    { 0xda87677, 99 },
    { 0, 0 },
    { 0xca587679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x368767c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2987682, 91 },
    { 0x89e87683, 43 },
    { 0x49387684, 23 },
    { 0x89987683, 33 },
    { 0xaa887684, 33 },
    { 0x19c87684, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf818768f, 34 },
    { 0, 0 },
    { 0xeb287691, 90 },
    { 0x75d87692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a87695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98769a, 98 },
    { 0, 0 },
    { 0x6c8769c, 23 },
    { 0, 0 },
    { 0xa4c8769e, 42 },
    { 0, 0 },
    { 0x82876a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f876a3, 91 },
    { 0x2e2876a3, 90 },
    { 0x65d876a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687876ac, 43 },
    { 0x66d876ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35876b4, 33 },
    { 0x26876b5, 77 },
    { 0xd05876b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x767876b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a876bc, 95 },
    { 0x4bd876bd, 76 },
    { 0x34876be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a876c2, 46 },
    { 0x78a876c3, 55 },
    { 0x826876c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85876ca, 33 },
    { 0x1c5876ca, 91 },
    { 0, 0 },
    { 0xc72876cd, 23 },
    { 0, 0 },
    { 0xc1b876cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58876d2, 61 },
    { 0, 0 },
    { 0xab876d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85876da, 61 },
    { 0x5ad876db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db876df, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563876e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539876ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x920876f0, 98 },
    { 0xe9b876f1, 33 },
    { 0x27c876f2, 23 },
    { 0xcf6876f3, 33 },
    { 0, 0 },
    { 0x1a2876f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845876f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d876fc, 93 },
    { 0x93876fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5487703, 78 },
    { 0, 0 },
    { 0x59387705, 78 },
    { 0, 0 },
    { 0x63c87707, 23 },
    { 0xd4c87708, 76 },
    { 0, 0 },
    { 0x568770a, 78 },
    { 0, 0 },
    { 0x1948770c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8770f, 61 },
    { 0xb3e8770f, 61 },
    { 0x32587711, 98 },
    { 0xc1f8770f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e87717, 91 },
    { 0x5a087717, 67 },
    { 0, 0 },
    { 0x9a28771a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8771d, 46 },
    { 0xeb48771e, 23 },
    { 0x4f8771f, 90 },
    { 0xcce87720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe887725, 66 },
    { 0x4c087726, 46 },
    { 0, 0 },
    { 0xd1a87728, 90 },
    { 0x6b87729, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838772c, 55 },
    { 0x84e8772d, 98 },
    { 0, 0 },
    { 0xafb8772f, 100 },
    { 0xc038772f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187736, 96 },
    { 0x68487737, 96 },
    { 0x99687738, 95 },
    { 0x94087736, 87 },
    { 0x4cf87737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1288773f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b87743, 23 },
    { 0x9a887744, 8 },
    { 0x75187745, 5 },
    { 0x28f87746, 61 },
    { 0xab887747, 43 },
    { 0x77e87747, 61 },
    { 0xd1d87747, 43 },
    { 0xe838774a, 98 },
    { 0x63887746, 23 },
    { 0xdd38774c, 46 },
    { 0xada87743, 34 },
    { 0, 0 },
    { 0x7df8774f, 23 },
    { 0xf3087750, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe87753, 55 },
    { 0, 0 },
    { 0x64787755, 90 },
    { 0x58b87756, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3088775d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b8776d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f87770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58887778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f8777d, 34 },
    { 0x7f18777d, 87 },
    { 0, 0 },
    { 0xfc787780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a287784, 78 },
    { 0x9087785, 90 },
    { 0xceb87785, 33 },
    { 0x12d87787, 93 },
    { 0xb6f87787, 23 },
    { 0xc1d87789, 98 },
    { 0x95487789, 33 },
    { 0x7fc8778b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5118778e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd87791, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88887799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b8779d, 90 },
    { 0xf9e8779e, 33 },
    { 0xe3c8779f, 90 },
    { 0x998779f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e877a5, 75 },
    { 0xdea877a6, 98 },
    { 0x238877a7, 70 },
    { 0xd56877a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1877ac, 85 },
    { 0xadb877ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f877b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c877b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc877c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270877c9, 98 },
    { 0, 0 },
    { 0x242877cb, 95 },
    { 0x981877cc, 61 },
    { 0x4d6877cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b877d3, 90 },
    { 0xc58877d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71877da, 98 },
    { 0x13f877db, 98 },
    { 0x73b877dc, 80 },
    { 0xc80877db, 90 },
    { 0x87d877de, 81 },
    { 0x267877de, 90 },
    { 0xc1b877e0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c877e3, 43 },
    { 0xbc7877e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf877e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0877ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e877f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606877f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee387801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d87804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1487807, 80 },
    { 0, 0 },
    { 0x50987809, 33 },
    { 0x3e28780a, 98 },
    { 0xfbb87809, 90 },
    { 0xe538780c, 98 },
    { 0x22187809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33787810, 43 },
    { 0xbb87811, 95 },
    { 0x99687812, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3287816, 80 },
    { 0, 0 },
    { 0xe1187818, 98 },
    { 0x2dc87819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8781c, 98 },
    { 0xa1b8781d, 88 },
    { 0x6d78781d, 98 },
    { 0xaa28781d, 23 },
    { 0x5c987820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6787827, 34 },
    { 0, 0 },
    { 0x81f87829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db87834, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28783b, 33 },
    { 0x6038783b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e8783f, 23 },
    { 0, 0 },
    { 0x4d987841, 33 },
    { 0, 0 },
    { 0x3dd87843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c87847, 33 },
    { 0x10c87848, 76 },
    { 0, 0 },
    { 0x4078784a, 40 },
    { 0, 0 },
    { 0x8508784c, 81 },
    { 0, 0 },
    { 0xb5c8784e, 76 },
    { 0xb2a8784f, 48 },
    { 0xe5d87850, 93 },
    { 0x13d87851, 76 },
    { 0xa387851, 69 },
    { 0x90b87851, 76 },
    { 0x24a87854, 61 },
    { 0x1db87852, 95 },
    { 0x43587856, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf668785f, 95 },
    { 0, 0 },
    { 0xb8087861, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9587864, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de87867, 23 },
    { 0xaf887868, 61 },
    { 0, 0 },
    { 0x5548786a, 23 },
    { 0xfe18786b, 83 },
    { 0xc578786c, 95 },
    { 0xfbc8786d, 76 },
    { 0xa78786e, 43 },
    { 0xb6e8786b, 33 },
    { 0, 0 },
    { 0x4f87871, 100 },
    { 0, 0 },
    { 0xfa487873, 98 },
    { 0, 0 },
    { 0xe6d87875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8787d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3887886, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958788a, 91 },
    { 0, 0 },
    { 0x6ad8788c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda887890, 68 },
    { 0x47287891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a87894, 91 },
    { 0, 0 },
    { 0x37687896, 33 },
    { 0xb6287896, 34 },
    { 0xee187898, 23 },
    { 0x6cc87898, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8789d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3878a0, 90 },
    { 0x309878a1, 55 },
    { 0xcb0878a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5878ae, 61 },
    { 0x2ab878ae, 95 },
    { 0x120878af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x391878b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22878b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb878ba, 96 },
    { 0, 0 },
    { 0xb6f878bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb878c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344878c9, 61 },
    { 0, 0 },
    { 0x889878cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x187878ce, 61 },
    { 0xfce878cf, 90 },
    { 0x2c2878d0, 95 },
    { 0, 0 },
    { 0x748878d2, 23 },
    { 0x289878d3, 33 },
    { 0, 0 },
    { 0xed2878d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f878d8, 46 },
    { 0x2fd878d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e878e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a878e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3878e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59878eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x476878ee, 61 },
    { 0, 0 },
    { 0xd1f878f0, 17 },
    { 0x9c6878f0, 90 },
    { 0x248878f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a878f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee878f9, 55 },
    { 0x19a878f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19287904, 76 },
    { 0x99887905, 76 },
    { 0xf7187906, 33 },
    { 0xb4f87907, 52 },
    { 0xf3c87904, 33 },
    { 0, 0 },
    { 0x5d38790a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268790f, 77 },
    { 0xc87910, 23 },
    { 0, 0 },
    { 0x5287912, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587919, 55 },
    { 0x36d8791a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e8791f, 61 },
    { 0x48787920, 61 },
    { 0x16c8791f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18792e, 84 },
    { 0, 0 },
    { 0xea887930, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f87933, 90 },
    { 0, 0 },
    { 0xd2487935, 61 },
    { 0x50787936, 55 },
    { 0, 0 },
    { 0x26687938, 33 },
    { 0, 0 },
    { 0xcd8793a, 98 },
    { 0x8ab8793b, 93 },
    { 0, 0 },
    { 0x3388793d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51787942, 46 },
    { 0xfee87943, 84 },
    { 0x6a87944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8794c, 34 },
    { 0xc88794d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25787955, 81 },
    { 0, 0 },
    { 0x32a87957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58795a, 83 },
    { 0x2808795b, 61 },
    { 0x378795c, 92 },
    { 0xb298795c, 33 },
    { 0xa658795c, 61 },
    { 0x26c8795f, 98 },
    { 0x3198795f, 23 },
    { 0, 0 },
    { 0x46587962, 78 },
    { 0x1c587962, 52 },
    { 0x97987964, 46 },
    { 0x2f387965, 95 },
    { 0x1387966, 95 },
    { 0xe4187967, 87 },
    { 0xc4887966, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x8648796b, 90 },
    { 0x3bd8796c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b87972, 76 },
    { 0xa5087973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c87978, 61 },
    { 0x3eb87979, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98797c, 99 },
    { 0xc438797d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a87980, 76 },
    { 0x91487981, 61 },
    { 0xdb787982, 93 },
    { 0xbc187983, 98 },
    { 0, 0 },
    { 0x57e87985, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc68798b, 98 },
    { 0x87f8798c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52887990, 76 },
    { 0, 0 },
    { 0x61e87992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97887995, 95 },
    { 0x92e87995, 61 },
    { 0xa1887997, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad8799a, 90 },
    { 0x35d8799b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409879a1, 64 },
    { 0xeab879a2, 90 },
    { 0x492879a3, 98 },
    { 0x907879a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b879a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198879b4, 76 },
    { 0xad1879b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x333879b8, 98 },
    { 0xc61879b9, 33 },
    { 0, 0 },
    { 0x2bf879bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e879be, 90 },
    { 0x939879bf, 84 },
    { 0x31f879bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b879c4, 76 },
    { 0x863879c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3879ca, 95 },
    { 0x681879ca, 98 },
    { 0xf23879cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x912879cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f879d3, 76 },
    { 0x4ef879d3, 83 },
    { 0, 0 },
    { 0x129879d6, 17 },
    { 0, 0 },
    { 0xab8879d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1879db, 95 },
    { 0xeb879dc, 61 },
    { 0xeda879dd, 33 },
    { 0, 0 },
    { 0xaf4879df, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55879e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d879f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e879f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d879f7, 23 },
    { 0, 0 },
    { 0xecc879f9, 98 },
    { 0x81d879fa, 23 },
    { 0x269879fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f87a04, 85 },
    { 0xa4987a05, 84 },
    { 0, 0 },
    { 0xeed87a07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a287a0a, 91 },
    { 0x3b887a0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c287a10, 95 },
    { 0, 0 },
    { 0xff587a12, 23 },
    { 0xb4387a12, 99 },
    { 0xdde87a13, 23 },
    { 0x36887a15, 34 },
    { 0xd6987a13, 90 },
    { 0xe6e87a17, 90 },
    { 0x8d387a18, 52 },
    { 0xcbf87a18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c87a1d, 19 },
    { 0, 0 },
    { 0xd9b87a1f, 46 },
    { 0x72887a20, 68 },
    { 0x25c87a21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51987a27, 61 },
    { 0x59487a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf887a2b, 90 },
    { 0, 0 },
    { 0xda187a2d, 88 },
    { 0xcda87a2e, 61 },
    { 0x97b87a2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e387a32, 23 },
    { 0xcfd87a32, 99 },
    { 0xc6187a34, 43 },
    { 0x3fc87a34, 33 },
    { 0x21d87a36, 23 },
    { 0x76787a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc87a3a, 61 },
    { 0x4b787a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b87a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2687a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61787a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a687a4a, 90 },
    { 0x72187a4a, 90 },
    { 0xe5c87a4b, 61 },
    { 0, 0 },
    { 0x4be87a4e, 33 },
    { 0x94d87a4f, 99 },
    { 0x9e587a4e, 35 },
    { 0, 0 },
    { 0x2a987a52, 52 },
    { 0, 0 },
    { 0xd7f87a54, 23 },
    { 0x69587a55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0387a58, 95 },
    { 0, 0 },
    { 0xe3987a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c287a5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6387a67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d887a6a, 95 },
    { 0x2f387a6a, 33 },
    { 0, 0 },
    { 0x80387a6d, 61 },
    { 0xb9d87a6e, 33 },
    { 0, 0 },
    { 0xc87a70, 33 },
    { 0xdcd87a70, 90 },
    { 0xced87a72, 87 },
    { 0x45387a72, 33 },
    { 0xa8887a74, 33 },
    { 0x9787a75, 90 },
    { 0x5d587a74, 91 },
    { 0x44787a76, 59 },
    { 0x7f787a78, 43 },
    { 0x16a87a79, 98 },
    { 0x11687a7a, 90 },
    { 0, 0 },
    { 0xac87a7c, 33 },
    { 0x3c987a7c, 90 },
    { 0x63e87a7e, 67 },
    { 0xc9687a7f, 23 },
    { 0x3e87a7e, 90 },
    { 0xca287a81, 78 },
    { 0x8e487a7e, 53 },
    { 0xcd287a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf87a87, 98 },
    { 0, 0 },
    { 0x78287a89, 55 },
    { 0x6e887a8a, 90 },
    { 0x19e87a8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef87a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2087a99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c87aa3, 23 },
    { 0, 0 },
    { 0x20987aa5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff87aa8, 81 },
    { 0xa4987aa9, 98 },
    { 0x1fd87aaa, 90 },
    { 0xd2387aab, 33 },
    { 0x33787aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd87abe, 33 },
    { 0, 0 },
    { 0xac087ac0, 67 },
    { 0, 0 },
    { 0xa4b87ac2, 90 },
    { 0xa0687ac2, 34 },
    { 0, 0 },
    { 0xfdd87ac5, 98 },
    { 0x20387ac6, 34 },
    { 0x2a87ac5, 52 },
    { 0, 0 },
    { 0xb9c87ac9, 78 },
    { 0, 0 },
    { 0x39887acb, 61 },
    { 0x6d787acc, 17 },
    { 0x2e487acd, 90 },
    { 0x26a87acd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf87ad4, 85 },
    { 0, 0 },
    { 0xe8987ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77287ada, 76 },
    { 0, 0 },
    { 0x53d87adc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e87ae3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a87ae6, 98 },
    { 0, 0 },
    { 0xb3087ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa187aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c787af2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c87aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde787b03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a287b0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f87b0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b87b12, 77 },
    { 0, 0 },
    { 0x6da87b14, 76 },
    { 0xb5b87b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0287b1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb87b1f, 77 },
    { 0, 0 },
    { 0x65387b21, 43 },
    { 0xe2b87b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38987b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb787b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25387b33, 93 },
    { 0xdad87b34, 46 },
    { 0, 0 },
    { 0xff787b36, 95 },
    { 0, 0 },
    { 0x5fc87b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d87b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d87b47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f87b4b, 23 },
    { 0, 0 },
    { 0x38187b4d, 33 },
    { 0, 0 },
    { 0x5ff87b4f, 93 },
    { 0, 0 },
    { 0xade87b51, 61 },
    { 0x27f87b52, 98 },
    { 0x26a87b52, 88 },
    { 0xd3987b54, 46 },
    { 0x37487b52, 93 },
    { 0x90c87b56, 61 },
    { 0x56187b57, 90 },
    { 0xd7087b53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4887b5c, 33 },
    { 0xf787b5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987b63, 64 },
    { 0, 0 },
    { 0xc9d87b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea87b6b, 33 },
    { 0, 0 },
    { 0x40287b6d, 33 },
    { 0x76e87b6e, 90 },
    { 0x61487b6f, 37 },
    { 0x45e87b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb687b74, 55 },
    { 0, 0 },
    { 0xbd287b76, 98 },
    { 0xa2687b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c87b7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25587b83, 61 },
    { 0xed187b84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91087b89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1487b8d, 98 },
    { 0xdf487b8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a187b91, 43 },
    { 0x31287b92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1087b95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55187b98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c87b9d, 33 },
    { 0xfa087b9e, 65 },
    { 0, 0 },
    { 0xd1d87ba0, 85 },
    { 0xf7d87ba1, 87 },
    { 0x6a087ba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e87ba8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d87bab, 87 },
    { 0xa6f87bac, 98 },
    { 0xd6b87bac, 88 },
    { 0xdea87bae, 67 },
    { 0, 0 },
    { 0xd3887bb0, 33 },
    { 0xe7687bb1, 98 },
    { 0, 0 },
    { 0xca587bb3, 36 },
    { 0x7d387bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc87bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b87bbb, 43 },
    { 0xefd87bbb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19587bbf, 23 },
    { 0x7787bbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b687bc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c87bcc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d287bcf, 88 },
    { 0x63f87bcf, 87 },
    { 0x12887bd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec087bd8, 48 },
    { 0x59987bd8, 76 },
    { 0x99787bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56787bdd, 52 },
    { 0x6b087bde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be87be5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c87be9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2987bec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x28887bef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8987bf2, 46 },
    { 0, 0 },
    { 0x81987bf4, 81 },
    { 0x5ac87bf4, 95 },
    { 0, 0 },
    { 0xc2d87bf7, 72 },
    { 0xa2e87bf8, 23 },
    { 0, 0 },
    { 0x1ba87bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b87c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58787c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc87c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0187c0b, 90 },
    { 0, 0 },
    { 0xcd987c0d, 76 },
    { 0xa9987c0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x85787c11, 98 },
    { 0xf4c87c12, 61 },
    { 0xdee87c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d287c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e87c1c, 95 },
    { 0x1e287c1d, 23 },
    { 0x7f587c1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d87c22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f87c2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1987c31, 88 },
    { 0x1de87c32, 43 },
    { 0x90d87c33, 40 },
    { 0xe7b87c32, 23 },
    { 0x1b887c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce87c39, 23 },
    { 0xc1787c3a, 55 },
    { 0x52b87c3b, 61 },
    { 0x7f87c3c, 23 },
    { 0, 0 },
    { 0x2b187c3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73387c41, 46 },
    { 0xc2887c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f87c4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0987c4e, 95 },
    { 0x84087c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf87c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce887c55, 77 },
    { 0x3bf87c56, 98 },
    { 0xc4c87c57, 90 },
    { 0x9fb87c58, 23 },
    { 0xb1c87c59, 34 },
    { 0x8eb87c5a, 84 },
    { 0x6db87c5b, 90 },
    { 0x16587c5b, 95 },
    { 0xa7e87c5d, 87 },
    { 0x9d587c5c, 52 },
    { 0xa0887c5b, 23 },
    { 0x38887c60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c87c64, 90 },
    { 0x3e487c65, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x74587c68, 55 },
    { 0xff987c69, 90 },
    { 0xc6387c6a, 85 },
    { 0, 0 },
    { 0x22287c6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec87c72, 90 },
    { 0x35187c73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc87c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35487c7d, 65 },
    { 0xa1b87c7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b587c83, 33 },
    { 0, 0 },
    { 0x9fa87c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x38987c88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea87c90, 65 },
    { 0, 0 },
    { 0x34f87c92, 33 },
    { 0, 0 },
    { 0x3c087c94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99687c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad787c9e, 96 },
    { 0x29187c9f, 93 },
    { 0x9087ca0, 76 },
    { 0x49887ca1, 61 },
    { 0xeb487ca2, 90 },
    { 0x15587c9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da87ca9, 23 },
    { 0xdc287caa, 55 },
    { 0xb2d87caa, 33 },
    { 0x75387cac, 95 },
    { 0, 0 },
    { 0x3f787cae, 23 },
    { 0xb8387caf, 61 },
    { 0x63d87cb0, 66 },
    { 0xe6087cb0, 77 },
    { 0x93987cb2, 33 },
    { 0x74587cb3, 76 },
    { 0xae687cb4, 40 },
    { 0x98787cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e587cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87cbc, 99 },
    { 0, 0 },
    { 0x5a187cbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e87cc7, 33 },
    { 0xff87cc8, 67 },
    { 0x5f487cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c87ccc, 78 },
    { 0x52987ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27287cd0, 74 },
    { 0xa6e87cd1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x10487cd4, 88 },
    { 0x18d87cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f287cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf587cde, 47 },
    { 0, 0 },
    { 0x56187ce0, 90 },
    { 0, 0 },
    { 0xafc87ce2, 90 },
    { 0xbe587ce3, 48 },
    { 0xab487ce4, 33 },
    { 0xe1787ce5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a87cea, 85 },
    { 0x36c87cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4987cee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6587cf3, 76 },
    { 0, 0 },
    { 0xd9987cf5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187cf9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8787cfc, 33 },
    { 0xe6a87cfd, 84 },
    { 0xd0e87cfc, 33 },
    { 0x98787cfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61187d02, 46 },
    { 0, 0 },
    { 0x4b187d04, 23 },
    { 0x5d287d05, 76 },
    { 0x12787d06, 81 },
    { 0x61687d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad87d12, 76 },
    { 0x8b87d13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd87d21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af87d25, 52 },
    { 0, 0 },
    { 0x5be87d27, 100 },
    { 0, 0 },
    { 0x1e587d29, 46 },
    { 0, 0 },
    { 0x21a87d2b, 66 },
    { 0x52f87d2c, 23 },
    { 0x4f387d2b, 95 },
    { 0x3a587d2e, 93 },
    { 0x6a887d2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51187d32, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7187d36, 90 },
    { 0, 0 },
    { 0x9a587d38, 90 },
    { 0x78f87d39, 95 },
    { 0, 0 },
    { 0x2af87d3b, 67 },
    { 0, 0 },
    { 0xa0287d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb787d42, 23 },
    { 0xd8c87d43, 87 },
    { 0, 0 },
    { 0xcc87d45, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a87d4e, 46 },
    { 0, 0 },
    { 0xe5387d50, 52 },
    { 0x73687d50, 67 },
    { 0xdf687d52, 76 },
    { 0x5c787d50, 33 },
    { 0xdb487d54, 84 },
    { 0x7e387d54, 90 },
    { 0x55087d56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71987d5d, 91 },
    { 0x8b087d5d, 33 },
    { 0x52287d5f, 34 },
    { 0xefb87d5f, 23 },
    { 0x89287d61, 95 },
    { 0xc8e87d60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d687d68, 98 },
    { 0xc3a87d69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c887d6d, 43 },
    { 0, 0 },
    { 0x4b987d6f, 90 },
    { 0x80987d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76987d78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe787d84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e87d87, 98 },
    { 0, 0 },
    { 0xb1f87d89, 33 },
    { 0x27087d8a, 77 },
    { 0, 0 },
    { 0x8fe87d8c, 96 },
    { 0x4e387d8d, 61 },
    { 0x73c87d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f87d93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90287d98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed487d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36887da0, 90 },
    { 0xdc687da1, 67 },
    { 0, 0 },
    { 0x52f87da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb887da7, 90 },
    { 0x6287da8, 23 },
    { 0x5a087da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb687dae, 34 },
    { 0x60087daf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea87db4, 69 },
    { 0, 0 },
    { 0x82287db6, 95 },
    { 0xbc687db7, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x46787dba, 95 },
    { 0xc3887dbb, 90 },
    { 0, 0 },
    { 0xd5a87dbd, 61 },
    { 0x5087dbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb187dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90787dc5, 83 },
    { 0xb6087dc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39187dc9, 33 },
    { 0xf4087dca, 53 },
    { 0x76287dcb, 93 },
    { 0xb2087dcb, 98 },
    { 0x19b87dc9, 95 },
    { 0xcc087dc9, 90 },
    { 0xe3187dc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b987dd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b87dd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b87dd8, 33 },
    { 0, 0 },
    { 0xf4c87dda, 61 },
    { 0xb887dda, 100 },
    { 0x2bc87dda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc687ddf, 67 },
    { 0xb287ddf, 76 },
    { 0xcbc87de1, 34 },
    { 0x73587de2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d87de7, 98 },
    { 0xb5a87de8, 98 },
    { 0x9d487de9, 38 },
    { 0xb6187de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f87ded, 34 },
    { 0x9b687dee, 33 },
    { 0xfe87def, 61 },
    { 0x8d387def, 34 },
    { 0x8b487dee, 34 },
    { 0x37787ded, 93 },
    { 0x62887df2, 90 },
    { 0x45b87dee, 93 },
    { 0x9e187df4, 65 },
    { 0xbe587df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a87e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7887e0b, 61 },
    { 0x49c87e0c, 61 },
    { 0xf4887e0d, 61 },
    { 0xb7187e0b, 55 },
    { 0x1be87e0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69887e12, 34 },
    { 0x7d87e13, 55 },
    { 0x4ad87e13, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb287e17, 86 },
    { 0x1e987e18, 46 },
    { 0xc7787e18, 87 },
    { 0x3dd87e1a, 23 },
    { 0x8fb87e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22587e21, 34 },
    { 0x74d87e21, 61 },
    { 0, 0 },
    { 0xf8d87e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4287e29, 23 },
    { 0x1a287e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84187e30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb987e36, 34 },
    { 0x81787e37, 98 },
    { 0x61b87e38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f87e3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d487e43, 23 },
    { 0x95187e44, 78 },
    { 0, 0 },
    { 0xe7887e46, 34 },
    { 0, 0 },
    { 0x5ca87e48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3587e4f, 34 },
    { 0xab87e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb587e54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc987e5c, 55 },
    { 0, 0 },
    { 0x89187e5e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c887e61, 98 },
    { 0x83687e62, 61 },
    { 0x86c87e63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc487e66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c87e6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c87e6e, 76 },
    { 0x9da87e6f, 33 },
    { 0x75487e70, 33 },
    { 0xc1287e6e, 90 },
    { 0xfd887e72, 23 },
    { 0xd4b87e73, 33 },
    { 0, 0 },
    { 0x79987e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a987e78, 90 },
    { 0x12b87e79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c87e7e, 46 },
    { 0x91f87e7f, 90 },
    { 0x52a87e7f, 43 },
    { 0x58d87e80, 55 },
    { 0, 0 },
    { 0xd1387e83, 99 },
    { 0x6087e84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6987e87, 75 },
    { 0xea987e88, 80 },
    { 0xd4787e89, 33 },
    { 0x80387e87, 46 },
    { 0x51a87e8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df87e92, 93 },
    { 0x88a87e93, 95 },
    { 0x29987e94, 55 },
    { 0x90d87e95, 89 },
    { 0x4ce87e95, 90 },
    { 0x38f87e97, 95 },
    { 0, 0 },
    { 0x85e87e99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f87ea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a87ea3, 34 },
    { 0xbee87ea4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f387ea7, 33 },
    { 0xf8687ea8, 61 },
    { 0x4b687ea9, 23 },
    { 0xcc887eaa, 90 },
    { 0x88987eaa, 61 },
    { 0xf887eac, 23 },
    { 0xf9887ead, 90 },
    { 0x47a87ead, 33 },
    { 0x86287eaa, 33 },
    { 0x3f387eb0, 76 },
    { 0xa1a87eb0, 76 },
    { 0xf4887eb1, 77 },
    { 0x46287eb3, 52 },
    { 0xd1787eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a87eb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d587ebd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a87ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2287ec5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7087ecb, 46 },
    { 0x79787ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2987ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c87ed5, 61 },
    { 0xbdd87ed5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d87edb, 66 },
    { 0xdda87edc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b87edf, 66 },
    { 0x1e487ee0, 98 },
    { 0xcf087ee1, 43 },
    { 0x7d587ee1, 23 },
    { 0x39687ee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba87ee6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c87ee9, 23 },
    { 0xc5987ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b87eed, 61 },
    { 0, 0 },
    { 0x61f87eef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5387ef2, 93 },
    { 0, 0 },
    { 0x91c87ef4, 90 },
    { 0xef087ef4, 98 },
    { 0xf8c87ef4, 33 },
    { 0xd4187ef5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c987efb, 33 },
    { 0, 0 },
    { 0x4e187efd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1487f00, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6587f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587f08, 76 },
    { 0xb3787f09, 61 },
    { 0x14887f08, 95 },
    { 0x31087f09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3687f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb087f12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78187f1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e887f23, 100 },
    { 0, 0 },
    { 0xe5687f25, 46 },
    { 0x27687f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2087f2d, 98 },
    { 0x1c187f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9587f32, 90 },
    { 0xda887f33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37887f3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7387f3f, 33 },
    { 0x20687f40, 90 },
    { 0, 0 },
    { 0x5f787f42, 61 },
    { 0xc8a87f43, 61 },
    { 0x59287f44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2887f49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78087f4d, 34 },
    { 0, 0 },
    { 0x77487f4f, 46 },
    { 0x5c787f4f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97387f59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb787f5d, 95 },
    { 0, 0 },
    { 0x21787f5f, 43 },
    { 0xb5487f60, 23 },
    { 0x2e387f61, 34 },
    { 0, 0 },
    { 0x33587f63, 34 },
    { 0xdd787f63, 55 },
    { 0xe2787f65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9787f6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60987f71, 23 },
    { 0xccc87f71, 90 },
    { 0x47487f73, 76 },
    { 0, 0 },
    { 0x4ca87f75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f87f7a, 90 },
    { 0x94e87f7a, 95 },
    { 0xfdf87f7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc287f81, 61 },
    { 0xcbf87f81, 34 },
    { 0x43987f83, 66 },
    { 0x3a987f82, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98687f8a, 90 },
    { 0x38587f8b, 99 },
    { 0x82f87f8c, 61 },
    { 0xf987f8c, 23 },
    { 0x84987f8e, 95 },
    { 0x587f8f, 61 },
    { 0xd7b87f8f, 46 },
    { 0xbc087f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0787f94, 90 },
    { 0xd3f87f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59987f99, 93 },
    { 0xbf087f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b87f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d87fa1, 95 },
    { 0x57587fa1, 95 },
    { 0x47387fa3, 33 },
    { 0xa2987fa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd087fa8, 43 },
    { 0x33b87fa9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38887fad, 90 },
    { 0xdfe87fad, 87 },
    { 0xecf87faf, 95 },
    { 0x93687fb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a87fb3, 66 },
    { 0, 0 },
    { 0xfce87fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf87fb8, 33 },
    { 0, 0 },
    { 0x5ac87fba, 90 },
    { 0x2b887fbb, 90 },
    { 0x8dd87fbc, 81 },
    { 0xc9987fba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df87fc1, 23 },
    { 0xcfc87fc1, 52 },
    { 0xb1d87fc3, 98 },
    { 0, 0 },
    { 0x1e987fc5, 34 },
    { 0, 0 },
    { 0x48087fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f87fcb, 98 },
    { 0, 0 },
    { 0x3c887fcd, 98 },
    { 0xfa687fce, 87 },
    { 0xd8087fce, 90 },
    { 0x65887fd0, 46 },
    { 0, 0 },
    { 0x86f87fd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41487fd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb87fd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a587fdc, 33 },
    { 0xf3b87fdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc87fe1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4187fea, 43 },
    { 0xd7287fea, 90 },
    { 0x76787fec, 23 },
    { 0x74187fed, 61 },
    { 0, 0 },
    { 0x72b87fef, 90 },
    { 0xf5787fef, 33 },
    { 0x49587ff1, 98 },
    { 0x1de87ff0, 85 },
    { 0x79d87ff3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86787ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3287ff9, 93 },
    { 0xa3f87ffa, 52 },
    { 0x29e87ffa, 91 },
    { 0x22887ffb, 88 },
    { 0x1de87ffb, 98 },
    { 0, 0 },
    { 0x2bb87fff, 76 },
    { 0xf5e87fff, 34 },
    { 0x8687fff, 100 },
    { 0, 0 },
    { 0xe7288003, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a088006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91988009, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108800e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be88011, 61 },
    { 0xf4e88012, 77 },
    { 0xd4788012, 61 },
    { 0x4c388013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13388017, 55 },
    { 0x54e88018, 85 },
    { 0xeac88018, 90 },
    { 0x15a88018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2598801d, 67 },
    { 0x9b78801e, 33 },
    { 0, 0 },
    { 0x64088020, 33 },
    { 0, 0 },
    { 0xb0a88022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b988027, 58 },
    { 0x5c388028, 43 },
    { 0xb5188028, 90 },
    { 0x14088029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc88033, 33 },
    { 0xab88034, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a88042, 87 },
    { 0xced88043, 66 },
    { 0, 0 },
    { 0xa4188045, 33 },
    { 0x34988045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b68804b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f58804e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26788055, 98 },
    { 0x23088056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1888059, 66 },
    { 0xf5a88059, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a58805e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3988061, 61 },
    { 0xc3788062, 90 },
    { 0xcff88063, 43 },
    { 0xa888062, 99 },
    { 0x3f88065, 85 },
    { 0xb0988066, 33 },
    { 0x11a88067, 100 },
    { 0, 0 },
    { 0xd1488069, 67 },
    { 0, 0 },
    { 0x3508806b, 76 },
    { 0, 0 },
    { 0x9f18806d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f88072, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28807a, 23 },
    { 0x4e58807b, 95 },
    { 0xd6e8807c, 55 },
    { 0xa1e8807c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f88081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f88085, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8808d, 76 },
    { 0, 0 },
    { 0x50e8808f, 65 },
    { 0, 0 },
    { 0x31c88091, 90 },
    { 0x9b788092, 98 },
    { 0, 0 },
    { 0x19588094, 68 },
    { 0, 0 },
    { 0x1188096, 34 },
    { 0, 0 },
    { 0xc4288098, 90 },
    { 0xa4488099, 91 },
    { 0xcaa88098, 95 },
    { 0, 0 },
    { 0xaad8809c, 98 },
    { 0x2798809d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d880a1, 98 },
    { 0x978880a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8880a5, 76 },
    { 0xed7880a6, 98 },
    { 0, 0 },
    { 0xb59880a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84880ad, 95 },
    { 0x1c1880ae, 61 },
    { 0xe97880ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5880b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x850880b8, 78 },
    { 0x1fb880b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23880bc, 82 },
    { 0x55a880bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c880c1, 33 },
    { 0x699880c2, 55 },
    { 0xd9880c3, 100 },
    { 0, 0 },
    { 0xd41880c5, 33 },
    { 0x226880c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71880c9, 90 },
    { 0xd02880ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129880cf, 98 },
    { 0x1bc880cf, 33 },
    { 0, 0 },
    { 0xce6880d2, 23 },
    { 0, 0 },
    { 0x325880d4, 43 },
    { 0xdc8880d5, 55 },
    { 0xe63880d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc880d9, 90 },
    { 0, 0 },
    { 0x60c880db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1880e4, 90 },
    { 0x6f880e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c880eb, 43 },
    { 0x48d880ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe880f0, 91 },
    { 0x226880f1, 95 },
    { 0xcaa880f2, 93 },
    { 0xe09880f1, 67 },
    { 0, 0 },
    { 0x238880f5, 61 },
    { 0xd83880f5, 33 },
    { 0x2f4880f5, 87 },
    { 0xb85880f5, 95 },
    { 0, 0 },
    { 0x8df880fa, 85 },
    { 0x688880fa, 84 },
    { 0x5dd880fc, 34 },
    { 0xff6880fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb88100, 68 },
    { 0x7888100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78810b, 95 },
    { 0x2a08810c, 61 },
    { 0, 0 },
    { 0x8f28810e, 95 },
    { 0, 0 },
    { 0x79a88110, 99 },
    { 0x2f588110, 90 },
    { 0x57c88112, 52 },
    { 0xf2988113, 98 },
    { 0xe188112, 33 },
    { 0x2b88111, 33 },
    { 0x1e988112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b88126, 97 },
    { 0x2688127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd968812a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65088132, 61 },
    { 0xecb88132, 100 },
    { 0x60788133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa198813a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4998813f, 33 },
    { 0xcb788140, 90 },
    { 0xd7588141, 33 },
    { 0, 0 },
    { 0xb6d88143, 95 },
    { 0, 0 },
    { 0x31888145, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08814b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe148814e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1188151, 55 },
    { 0x60a88151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f88155, 65 },
    { 0x1e388156, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c8815c, 87 },
    { 0, 0 },
    { 0xade8815e, 99 },
    { 0, 0 },
    { 0x5d188160, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61788166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e28816a, 55 },
    { 0xe378816a, 34 },
    { 0xd1b8816c, 90 },
    { 0x6938816d, 98 },
    { 0xb548816e, 61 },
    { 0, 0 },
    { 0xc1b88170, 93 },
    { 0x43e88171, 95 },
    { 0x6f388172, 69 },
    { 0x1d088173, 33 },
    { 0xc0388171, 61 },
    { 0x9bf88175, 33 },
    { 0x5e588176, 66 },
    { 0xdd188175, 34 },
    { 0xc488178, 23 },
    { 0x87a88175, 90 },
    { 0x51f88179, 98 },
    { 0xdb788172, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de8817f, 47 },
    { 0x53188180, 55 },
    { 0xd9c88181, 87 },
    { 0x5a88181, 33 },
    { 0xd6a88182, 76 },
    { 0x62a88181, 33 },
    { 0x65b88185, 99 },
    { 0x72988185, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33188193, 90 },
    { 0, 0 },
    { 0xb8188195, 76 },
    { 0x32c88196, 55 },
    { 0, 0 },
    { 0xca88198, 90 },
    { 0x5b788198, 98 },
    { 0xaa88819a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f881a1, 74 },
    { 0x1f3881a2, 98 },
    { 0x80e881a3, 33 },
    { 0x1a881a3, 90 },
    { 0x6a5881a4, 34 },
    { 0x8b6881a6, 98 },
    { 0x71e881a7, 90 },
    { 0xf41881a7, 78 },
    { 0x2cb881a9, 34 },
    { 0x858881a6, 46 },
    { 0x834881a7, 90 },
    { 0x24c881aa, 23 },
    { 0x4fa881ad, 45 },
    { 0xea6881ad, 77 },
    { 0x25d881ad, 23 },
    { 0x5dc881ae, 90 },
    { 0x7c1881aa, 95 },
    { 0x4e7881b0, 76 },
    { 0, 0 },
    { 0xf79881b4, 98 },
    { 0, 0 },
    { 0x863881b6, 98 },
    { 0xd71881b7, 91 },
    { 0xf49881b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759881c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884881c6, 33 },
    { 0xbee881c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c881ca, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b881d0, 95 },
    { 0xa56881d1, 98 },
    { 0x31c881d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a881d6, 87 },
    { 0x2fc881d7, 33 },
    { 0xe6b881d8, 99 },
    { 0x950881d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9881dd, 76 },
    { 0, 0 },
    { 0xd76881df, 61 },
    { 0xdc6881e0, 34 },
    { 0xaec881e1, 33 },
    { 0x45881e1, 61 },
    { 0x892881e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x75881e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e881e9, 93 },
    { 0xa89881e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0881ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef881f1, 33 },
    { 0xbfe881f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954881f9, 68 },
    { 0xe17881f9, 33 },
    { 0xc15881f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x642881fe, 43 },
    { 0, 0 },
    { 0x9e788200, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84188204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14388208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3488213, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde988218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe488220, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48388226, 98 },
    { 0xb6388227, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98822d, 95 },
    { 0x1b38822e, 90 },
    { 0x60c8822f, 91 },
    { 0x2bc8822d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a88235, 23 },
    { 0, 0 },
    { 0xf9e88237, 90 },
    { 0xa0088237, 76 },
    { 0xfe488239, 98 },
    { 0, 0 },
    { 0x42f8823b, 34 },
    { 0, 0 },
    { 0xce68823d, 95 },
    { 0x7f18823e, 61 },
    { 0xb818823d, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4188242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d88246, 33 },
    { 0, 0 },
    { 0xbdc88248, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e58824b, 90 },
    { 0x6178824b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d88252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa88256, 95 },
    { 0xb7788256, 48 },
    { 0xdf688258, 77 },
    { 0x5f388257, 38 },
    { 0xf838825a, 91 },
    { 0, 0 },
    { 0x9608825c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a8825f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55388262, 66 },
    { 0, 0 },
    { 0x45588264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f18826d, 55 },
    { 0x2628826e, 33 },
    { 0x5518826e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f88277, 61 },
    { 0, 0 },
    { 0xedb88279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3358827d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88282, 52 },
    { 0, 0 },
    { 0x35988284, 91 },
    { 0x51788285, 64 },
    { 0x13288284, 23 },
    { 0x67b88284, 98 },
    { 0x7a788288, 23 },
    { 0x1b688288, 59 },
    { 0x3c88289, 23 },
    { 0xe2d88286, 90 },
    { 0x19988284, 96 },
    { 0xabc88289, 99 },
    { 0xf8b88287, 46 },
    { 0xebc8828f, 36 },
    { 0xc7c88290, 91 },
    { 0x4118828f, 99 },
    { 0x95a8828d, 76 },
    { 0x76f8828c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d288299, 61 },
    { 0x8d08829a, 98 },
    { 0x7528829b, 43 },
    { 0x74f8829b, 23 },
    { 0, 0 },
    { 0xb0d8829e, 75 },
    { 0x5428829f, 34 },
    { 0x4ab8829e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2882a5, 61 },
    { 0xfbe882a5, 98 },
    { 0x168882a7, 100 },
    { 0x11a882a7, 90 },
    { 0x6b0882a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3882ac, 97 },
    { 0x568882ac, 95 },
    { 0x335882ae, 95 },
    { 0x5c2882af, 65 },
    { 0x628882ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x531882b3, 90 },
    { 0x438882b3, 91 },
    { 0xfeb882b3, 90 },
    { 0xae1882b4, 82 },
    { 0x475882b4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc882bb, 93 },
    { 0x7f7882bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539882c2, 90 },
    { 0, 0 },
    { 0x85882c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac882c7, 43 },
    { 0x61d882c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2882cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55882d3, 93 },
    { 0xdd882d4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230882da, 90 },
    { 0xffc882db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a882e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x338882e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685882e8, 75 },
    { 0x81c882e9, 33 },
    { 0xb54882e9, 23 },
    { 0x2d4882eb, 33 },
    { 0x7fb882eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420882f1, 17 },
    { 0xbcf882f1, 61 },
    { 0xbcc882f1, 90 },
    { 0x3da882f4, 55 },
    { 0xe1f882f5, 76 },
    { 0xa7b882f1, 33 },
    { 0xb99882f3, 34 },
    { 0xaf3882f8, 33 },
    { 0, 0 },
    { 0xcae882fa, 56 },
    { 0, 0 },
    { 0xdce882fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b688300, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72988309, 76 },
    { 0x47988309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a8830e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80888311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08831b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23188323, 23 },
    { 0, 0 },
    { 0xc3188325, 76 },
    { 0, 0 },
    { 0xb2588327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab8832b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f8832e, 84 },
    { 0xea48832f, 95 },
    { 0xa058832e, 76 },
    { 0xbb688330, 95 },
    { 0xb08832e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e388337, 95 },
    { 0x88488338, 96 },
    { 0x39488339, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea8833e, 76 },
    { 0x66f8833f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45988343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53388347, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa38834b, 52 },
    { 0, 0 },
    { 0x2e8834d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca88350, 95 },
    { 0, 0 },
    { 0x43288352, 90 },
    { 0x24888353, 88 },
    { 0x24388352, 34 },
    { 0x94888355, 61 },
    { 0x90688356, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a78835e, 95 },
    { 0xa508835e, 61 },
    { 0x6e588360, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5488363, 61 },
    { 0x38f88363, 76 },
    { 0x45988365, 68 },
    { 0xac488366, 99 },
    { 0xab188366, 34 },
    { 0x8b288367, 81 },
    { 0, 0 },
    { 0xfa38836a, 67 },
    { 0x6c58836b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a88836e, 34 },
    { 0x4628836f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97788376, 43 },
    { 0x26688377, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1298837a, 64 },
    { 0x88c8837b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1388381, 76 },
    { 0x62088382, 34 },
    { 0x85e88383, 95 },
    { 0x98788382, 61 },
    { 0, 0 },
    { 0xb0088386, 95 },
    { 0x25988387, 52 },
    { 0, 0 },
    { 0x6d988389, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb88394, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f288399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf8839f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0883a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70883a6, 43 },
    { 0, 0 },
    { 0x8f9883a8, 23 },
    { 0, 0 },
    { 0x4ac883aa, 95 },
    { 0, 0 },
    { 0x599883ac, 43 },
    { 0x702883ad, 76 },
    { 0x8b6883ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468883b5, 78 },
    { 0xf1a883b5, 23 },
    { 0xbdf883b7, 46 },
    { 0xc51883b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887883bd, 43 },
    { 0xe3a883bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796883c3, 23 },
    { 0, 0 },
    { 0xe9f883c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30883cc, 77 },
    { 0x315883cd, 65 },
    { 0xc7d883cd, 43 },
    { 0xc98883cd, 43 },
    { 0x8b3883cd, 76 },
    { 0x163883d1, 68 },
    { 0xbfb883d1, 87 },
    { 0x6d6883d2, 61 },
    { 0x8da883ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9883da, 33 },
    { 0, 0 },
    { 0xa97883dc, 61 },
    { 0, 0 },
    { 0x469883de, 91 },
    { 0x3d3883df, 98 },
    { 0x98a883e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x976883e3, 61 },
    { 0x5c9883e4, 66 },
    { 0xb07883e5, 67 },
    { 0xcf0883e6, 23 },
    { 0x15c883e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6883ed, 23 },
    { 0xfa4883ee, 77 },
    { 0x290883ef, 91 },
    { 0x4b883ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3883f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x745883fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11088405, 93 },
    { 0x9b88405, 33 },
    { 0, 0 },
    { 0x5d088408, 23 },
    { 0, 0 },
    { 0x8ab8840a, 95 },
    { 0, 0 },
    { 0x6f08840c, 23 },
    { 0x70f8840c, 100 },
    { 0x66b8840e, 52 },
    { 0x5368840f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c088414, 90 },
    { 0x52288415, 61 },
    { 0x20288416, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b88419, 69 },
    { 0xd178841a, 99 },
    { 0xc788419, 43 },
    { 0x7938841c, 95 },
    { 0x2288841d, 93 },
    { 0, 0 },
    { 0xd1a8841f, 98 },
    { 0xe4a8841f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb948842b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28842e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb988432, 23 },
    { 0x47188432, 61 },
    { 0x13188433, 61 },
    { 0xf1a88432, 46 },
    { 0, 0 },
    { 0x47e88437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c38843c, 78 },
    { 0, 0 },
    { 0x328843e, 33 },
    { 0x8278843f, 43 },
    { 0, 0 },
    { 0x59488441, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa988446, 23 },
    { 0xe988447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb8844b, 23 },
    { 0x6078844b, 61 },
    { 0x7788844d, 17 },
    { 0x5878844d, 93 },
    { 0x9be8844f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588456, 90 },
    { 0, 0 },
    { 0xdbd88458, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd58845c, 40 },
    { 0xdaa8845d, 34 },
    { 0xa288845e, 43 },
    { 0x3158845f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e88467, 67 },
    { 0xfca88467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d8846b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18846e, 61 },
    { 0, 0 },
    { 0x4f988470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd088473, 61 },
    { 0x84c88474, 90 },
    { 0x57888474, 98 },
    { 0xe0388474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb8847d, 43 },
    { 0, 0 },
    { 0xd538847f, 93 },
    { 0xda88847f, 98 },
    { 0x23f88481, 61 },
    { 0, 0 },
    { 0x42e88483, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2188486, 98 },
    { 0xf5188487, 90 },
    { 0x9c088486, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f8848d, 61 },
    { 0, 0 },
    { 0x61b8848f, 23 },
    { 0, 0 },
    { 0xe5288491, 95 },
    { 0, 0 },
    { 0xfc488493, 98 },
    { 0x5f288493, 87 },
    { 0x80a88495, 33 },
    { 0x8de88493, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8849b, 91 },
    { 0xbca8849c, 91 },
    { 0xc508849c, 90 },
    { 0xc438849e, 59 },
    { 0x1df8849f, 78 },
    { 0x1ea8849d, 66 },
    { 0xb218849d, 33 },
    { 0, 0 },
    { 0xfea884a3, 61 },
    { 0x81e884a4, 23 },
    { 0xb25884a5, 55 },
    { 0xd4d884a5, 90 },
    { 0xb2a884a7, 45 },
    { 0x96d884a6, 90 },
    { 0, 0 },
    { 0x8f7884aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98884ad, 98 },
    { 0xa05884ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a884b5, 33 },
    { 0x831884b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8884ba, 76 },
    { 0xe11884ba, 34 },
    { 0xff5884bc, 76 },
    { 0x8f3884bd, 68 },
    { 0xe00884bb, 93 },
    { 0x2fb884bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8884c3, 23 },
    { 0x80e884c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf884cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8884d1, 76 },
    { 0xe02884d2, 55 },
    { 0xb80884d3, 17 },
    { 0x2b9884d2, 46 },
    { 0x2cc884d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81884d9, 44 },
    { 0, 0 },
    { 0x5ea884db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2884de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x744884e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a884ec, 95 },
    { 0x78f884ec, 17 },
    { 0xb2a884ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d884f3, 95 },
    { 0, 0 },
    { 0xeb9884f5, 23 },
    { 0xb4e884f6, 98 },
    { 0x8b1884f7, 95 },
    { 0x5dd884f6, 55 },
    { 0x334884f9, 95 },
    { 0xaed884f9, 17 },
    { 0x529884fb, 23 },
    { 0xc7a884f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d88500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc88509, 33 },
    { 0xc918850a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3988850e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5488515, 43 },
    { 0x93588515, 33 },
    { 0xefe88517, 23 },
    { 0x95d88518, 43 },
    { 0, 0 },
    { 0x29d8851a, 78 },
    { 0xd1f8851a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b688521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad388527, 90 },
    { 0x90488528, 61 },
    { 0x18e88529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb238852f, 84 },
    { 0xa4588530, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14988533, 88 },
    { 0x8e88534, 84 },
    { 0, 0 },
    { 0x7a588536, 66 },
    { 0xe0988536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb088853b, 95 },
    { 0, 0 },
    { 0x4c8853d, 34 },
    { 0x8a78853e, 90 },
    { 0x1e8853f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b88543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf988547, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8854a, 76 },
    { 0, 0 },
    { 0xe308854c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb88552, 33 },
    { 0x99088553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1988556, 23 },
    { 0x19d88557, 46 },
    { 0xef888558, 33 },
    { 0x40188559, 90 },
    { 0, 0 },
    { 0xfd08855b, 61 },
    { 0x9dd8855c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1588560, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd88564, 55 },
    { 0x4488565, 46 },
    { 0, 0 },
    { 0xb4c88567, 33 },
    { 0x5fd88568, 43 },
    { 0xe088569, 61 },
    { 0xf1e88568, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b8856d, 33 },
    { 0xa798856e, 95 },
    { 0x10a8856e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca288573, 90 },
    { 0x1c488574, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84188578, 90 },
    { 0, 0 },
    { 0x48d8857a, 33 },
    { 0, 0 },
    { 0xb658857c, 85 },
    { 0, 0 },
    { 0x388857e, 98 },
    { 0, 0 },
    { 0x4da88580, 90 },
    { 0x4e388581, 100 },
    { 0x99788582, 33 },
    { 0x14d88580, 33 },
    { 0, 0 },
    { 0x40088585, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x54088588, 78 },
    { 0xf1488588, 23 },
    { 0x5f488589, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78858d, 33 },
    { 0xfa38858d, 23 },
    { 0x2648858d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff588592, 95 },
    { 0, 0 },
    { 0x80d88594, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a88598, 97 },
    { 0, 0 },
    { 0xb4b8859a, 55 },
    { 0xfea8859b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31885a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37885a8, 77 },
    { 0x95d885a9, 94 },
    { 0xb8c885aa, 34 },
    { 0xf4a885a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa885ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8885b2, 98 },
    { 0xd90885b2, 61 },
    { 0x6ce885b4, 85 },
    { 0, 0 },
    { 0x1d2885b6, 90 },
    { 0xfdc885b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0885bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a885be, 50 },
    { 0x820885be, 88 },
    { 0xee8885bf, 76 },
    { 0xc5b885be, 61 },
    { 0x775885c2, 76 },
    { 0x8ea885c3, 56 },
    { 0, 0 },
    { 0xf73885c5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f885c8, 82 },
    { 0, 0 },
    { 0x2ef885ca, 80 },
    { 0, 0 },
    { 0x8fd885cc, 76 },
    { 0, 0 },
    { 0xd19885ce, 93 },
    { 0xd40885cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a885d2, 33 },
    { 0, 0 },
    { 0x76d885d4, 98 },
    { 0x984885d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38885d8, 90 },
    { 0xcb4885d8, 76 },
    { 0xe99885d9, 33 },
    { 0x2a0885db, 98 },
    { 0x1bd885dc, 98 },
    { 0xc4c885d8, 90 },
    { 0x738885dd, 66 },
    { 0x3b8885de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264885e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece885ec, 33 },
    { 0, 0 },
    { 0x61f885ee, 90 },
    { 0x176885ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b885f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef885f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a885fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce88601, 76 },
    { 0xaaf88602, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4688605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f88608, 90 },
    { 0x73c88609, 61 },
    { 0x99e8860a, 95 },
    { 0x6dd8860a, 61 },
    { 0x7b58860b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96588611, 40 },
    { 0x3ff88612, 46 },
    { 0xfa788613, 90 },
    { 0x95d88612, 23 },
    { 0x71f88612, 91 },
    { 0x3ba88613, 46 },
    { 0x76f88617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2408861d, 90 },
    { 0x5458861d, 61 },
    { 0x8d68861f, 84 },
    { 0, 0 },
    { 0x21188621, 93 },
    { 0xbb388622, 40 },
    { 0x80d88623, 33 },
    { 0x83f88624, 38 },
    { 0, 0 },
    { 0xb4488626, 66 },
    { 0xfa188627, 33 },
    { 0xc5088628, 95 },
    { 0xb9888629, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e8862c, 80 },
    { 0x398862c, 95 },
    { 0, 0 },
    { 0x5a78862f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d88632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d88637, 23 },
    { 0x4388638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f088644, 23 },
    { 0, 0 },
    { 0x1af88646, 33 },
    { 0x2a288647, 75 },
    { 0x6a88648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f8864d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b8865a, 33 },
    { 0xaac8865a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d88662, 33 },
    { 0x8f788663, 93 },
    { 0xe6388664, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf788667, 33 },
    { 0, 0 },
    { 0x9bf88669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d8866f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd88679, 91 },
    { 0, 0 },
    { 0xf148867b, 99 },
    { 0, 0 },
    { 0xb468867d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3188682, 76 },
    { 0xd0e88683, 33 },
    { 0xf3488684, 23 },
    { 0xc8588685, 90 },
    { 0, 0 },
    { 0x49488687, 76 },
    { 0x93388687, 55 },
    { 0xc88689, 98 },
    { 0x79e88687, 99 },
    { 0, 0 },
    { 0xac78868c, 77 },
    { 0xf7f8868c, 90 },
    { 0, 0 },
    { 0x73a8868f, 88 },
    { 0, 0 },
    { 0x9388691, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f88694, 23 },
    { 0x7bf88695, 93 },
    { 0x81188694, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2398869b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54886a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47886a5, 33 },
    { 0x996886a6, 98 },
    { 0x6af886a6, 90 },
    { 0x214886a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc886ad, 61 },
    { 0xa08886ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2886b3, 76 },
    { 0xe31886b3, 33 },
    { 0x9d0886b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b886ba, 61 },
    { 0, 0 },
    { 0x452886bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c886cb, 34 },
    { 0x225886cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101886d2, 93 },
    { 0x662886d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10886db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd886e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860886e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf886ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a886f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc886f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3886f8, 90 },
    { 0x929886f8, 87 },
    { 0x22e886fa, 91 },
    { 0x81d886fb, 61 },
    { 0, 0 },
    { 0x3b6886fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13088700, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c088704, 34 },
    { 0xb4288705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65488708, 17 },
    { 0x7c188709, 33 },
    { 0x71a88708, 95 },
    { 0x6168870b, 77 },
    { 0x53d8870b, 23 },
    { 0x4e48870d, 90 },
    { 0x51b8870b, 98 },
    { 0, 0 },
    { 0x43688710, 34 },
    { 0x1dc88710, 23 },
    { 0x84a88712, 33 },
    { 0x6a988713, 23 },
    { 0xbd788712, 69 },
    { 0x73f88712, 33 },
    { 0xe288712, 76 },
    { 0x2ed88713, 90 },
    { 0xaf488718, 46 },
    { 0xac988713, 33 },
    { 0xa408871a, 61 },
    { 0xd2c8871b, 85 },
    { 0xe78871c, 95 },
    { 0xf568871a, 98 },
    { 0x96588714, 66 },
    { 0x50d88717, 91 },
    { 0xee588720, 67 },
    { 0xc088720, 34 },
    { 0x33f88722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d8872a, 90 },
    { 0x8bb8872a, 33 },
    { 0, 0 },
    { 0x7298872d, 88 },
    { 0xb348872e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b88732, 33 },
    { 0xa1d88733, 67 },
    { 0x5a188734, 23 },
    { 0, 0 },
    { 0xc4088736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac08873a, 90 },
    { 0x1928873b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc88748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf48874f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea588757, 33 },
    { 0x84588758, 61 },
    { 0x6f588759, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78875d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69788763, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7818876c, 61 },
    { 0x4518876c, 95 },
    { 0x7be8876e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82988778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a8877b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8877f, 34 },
    { 0x1b988780, 90 },
    { 0xb2f88781, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b88789, 90 },
    { 0xc2d88789, 95 },
    { 0xfb48878b, 23 },
    { 0x8eb8878c, 67 },
    { 0x40f8878d, 58 },
    { 0x87b8878e, 56 },
    { 0, 0 },
    { 0x97c88790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f88794, 61 },
    { 0x6d588795, 34 },
    { 0x5ee88796, 100 },
    { 0, 0 },
    { 0x5df88798, 90 },
    { 0, 0 },
    { 0x7f48879a, 65 },
    { 0xa5d8879b, 66 },
    { 0xfb38879b, 65 },
    { 0x2808879b, 91 },
    { 0xe888879b, 95 },
    { 0x59d8879c, 52 },
    { 0xb3b8879a, 55 },
    { 0, 0 },
    { 0x733887a2, 98 },
    { 0x501887a3, 33 },
    { 0x866887a2, 100 },
    { 0, 0 },
    { 0x41e887a6, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30887a9, 92 },
    { 0x729887a9, 66 },
    { 0xc7c887ab, 46 },
    { 0x271887ac, 84 },
    { 0xf0d887ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171887b0, 98 },
    { 0x4a6887b1, 33 },
    { 0, 0 },
    { 0x973887b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9887b6, 90 },
    { 0xc73887b7, 90 },
    { 0xf7e887b8, 98 },
    { 0x46c887b9, 98 },
    { 0, 0 },
    { 0x3b2887bb, 76 },
    { 0x471887bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93887bf, 87 },
    { 0, 0 },
    { 0x9aa887c1, 33 },
    { 0xf60887c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d887c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da887ca, 34 },
    { 0x5d3887cb, 43 },
    { 0xb92887cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106887d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d887da, 95 },
    { 0xe3f887db, 61 },
    { 0x2cd887dc, 98 },
    { 0xd6a887dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d887e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11887e4, 61 },
    { 0, 0 },
    { 0x39c887e6, 61 },
    { 0xb6e887e7, 100 },
    { 0xace887e8, 90 },
    { 0x45d887e9, 23 },
    { 0xcd3887ea, 33 },
    { 0, 0 },
    { 0x7b3887ec, 95 },
    { 0x1a0887ed, 90 },
    { 0, 0 },
    { 0x92887ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca887f9, 87 },
    { 0, 0 },
    { 0x624887fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd88804, 84 },
    { 0, 0 },
    { 0x67688806, 84 },
    { 0x33e88807, 61 },
    { 0x2f488807, 91 },
    { 0xc7188806, 37 },
    { 0x8918880a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8880d, 33 },
    { 0xb2c8880d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b88811, 65 },
    { 0x37988812, 76 },
    { 0xf9e88813, 23 },
    { 0x58488814, 90 },
    { 0xffd88812, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d288818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca98881f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e788822, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79288828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888832, 98 },
    { 0xd2988833, 33 },
    { 0xaf088834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8883f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd8884c, 77 },
    { 0xfa28884d, 43 },
    { 0, 0 },
    { 0xf898884f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e88855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc788858, 33 },
    { 0x60b88859, 33 },
    { 0xdbf8885a, 76 },
    { 0, 0 },
    { 0xa4a8885c, 98 },
    { 0x5778885d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef88860, 91 },
    { 0, 0 },
    { 0x40288862, 64 },
    { 0xfa288863, 33 },
    { 0xbbc88863, 61 },
    { 0xf4788865, 34 },
    { 0xd1388866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458886b, 93 },
    { 0x6548886c, 77 },
    { 0xbce8886d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54488871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef88875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff88878, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6388881, 95 },
    { 0, 0 },
    { 0xb5b88883, 99 },
    { 0x4c588884, 33 },
    { 0, 0 },
    { 0x19c88886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb8888c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b88891, 95 },
    { 0x2ea88892, 95 },
    { 0xc9588893, 95 },
    { 0, 0 },
    { 0x37788895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff88898, 34 },
    { 0xe5188898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58889d, 90 },
    { 0x8cd8889d, 55 },
    { 0x6b98889f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588888a4, 23 },
    { 0xff888a4, 95 },
    { 0x5c7888a4, 61 },
    { 0x63d888a6, 90 },
    { 0xf65888a4, 90 },
    { 0x6c888a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba888b3, 76 },
    { 0, 0 },
    { 0x5fd888b5, 90 },
    { 0x6e9888b5, 95 },
    { 0x4a3888b7, 95 },
    { 0xe14888b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd888bc, 90 },
    { 0x503888bc, 67 },
    { 0x83e888be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06888c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12888c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967888cb, 91 },
    { 0xb83888cc, 99 },
    { 0, 0 },
    { 0xca1888ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e888d4, 51 },
    { 0x30b888d5, 33 },
    { 0xa888d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f888da, 43 },
    { 0xdf1888db, 33 },
    { 0x120888dc, 88 },
    { 0x674888dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696888e3, 68 },
    { 0x885888e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54888ea, 89 },
    { 0, 0 },
    { 0xf78888ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b888f4, 61 },
    { 0, 0 },
    { 0x879888f6, 91 },
    { 0, 0 },
    { 0x920888f8, 76 },
    { 0, 0 },
    { 0xce7888fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f88903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1288906, 33 },
    { 0, 0 },
    { 0x8c588908, 90 },
    { 0, 0 },
    { 0x39e8890a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5228890d, 38 },
    { 0x70f8890d, 67 },
    { 0, 0 },
    { 0x2ce88910, 95 },
    { 0, 0 },
    { 0x18f88912, 85 },
    { 0xe5588912, 33 },
    { 0x4088912, 96 },
    { 0x98588915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43588922, 33 },
    { 0, 0 },
    { 0x43a88924, 98 },
    { 0x17888925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88929, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9018892c, 93 },
    { 0, 0 },
    { 0xbab8892e, 55 },
    { 0, 0 },
    { 0x7f88930, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc88936, 90 },
    { 0, 0 },
    { 0x70e88938, 90 },
    { 0xd5b88939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf568893d, 84 },
    { 0, 0 },
    { 0x9448893f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e88942, 91 },
    { 0xe7a88943, 23 },
    { 0xb7388944, 34 },
    { 0, 0 },
    { 0xee688946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f88949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058894f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d388953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51088959, 95 },
    { 0xf198895a, 87 },
    { 0xea788959, 61 },
    { 0, 0 },
    { 0xb5e8895d, 61 },
    { 0x90b8895e, 43 },
    { 0, 0 },
    { 0x5d088960, 90 },
    { 0x7cb88961, 93 },
    { 0xad988962, 95 },
    { 0x60a88960, 95 },
    { 0xa5688964, 98 },
    { 0xb4f88965, 87 },
    { 0x99388966, 93 },
    { 0x28188967, 34 },
    { 0xded88962, 36 },
    { 0xb8a88969, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b788976, 46 },
    { 0, 0 },
    { 0xc2988978, 91 },
    { 0x69788979, 90 },
    { 0x79888979, 34 },
    { 0x4618897b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f8897f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce88983, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9488988, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148898c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd488991, 61 },
    { 0x40688992, 78 },
    { 0x96188993, 66 },
    { 0x86588994, 23 },
    { 0x52888995, 17 },
    { 0x14088996, 90 },
    { 0x87c88997, 99 },
    { 0x9bd88998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28899b, 94 },
    { 0xe948899b, 61 },
    { 0, 0 },
    { 0xfec8899e, 85 },
    { 0, 0 },
    { 0xd53889a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96889b9, 61 },
    { 0xf24889b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99889bd, 61 },
    { 0xb37889be, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565889c4, 34 },
    { 0x5b8889c5, 75 },
    { 0, 0 },
    { 0x162889c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8889cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c889d7, 95 },
    { 0xd7a889d8, 77 },
    { 0x145889d7, 56 },
    { 0, 0 },
    { 0xaea889db, 95 },
    { 0, 0 },
    { 0x130889dd, 34 },
    { 0xe19889de, 98 },
    { 0, 0 },
    { 0xeca889e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88889e4, 83 },
    { 0, 0 },
    { 0xf31889e6, 55 },
    { 0x87b889e6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d889ed, 66 },
    { 0xc89889ee, 33 },
    { 0xeb3889ee, 34 },
    { 0, 0 },
    { 0xe28889f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501889f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29889fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x154889fd, 98 },
    { 0, 0 },
    { 0x48889ff, 23 },
    { 0x5cd88a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d88a03, 90 },
    { 0x58388a04, 81 },
    { 0xbef88a04, 61 },
    { 0xe9b88a06, 99 },
    { 0xf2588a05, 91 },
    { 0x6ed88a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf88a0b, 55 },
    { 0x16688a0c, 61 },
    { 0x63f88a0d, 23 },
    { 0x34f88a0e, 91 },
    { 0x64a88a0f, 8 },
    { 0x1088a10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b988a15, 55 },
    { 0x9de88a15, 93 },
    { 0, 0 },
    { 0x5e688a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73088a1d, 88 },
    { 0x51088a1e, 33 },
    { 0xf8e88a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e88a26, 52 },
    { 0x16988a26, 34 },
    { 0, 0 },
    { 0x70888a29, 95 },
    { 0xb8a88a29, 90 },
    { 0, 0 },
    { 0xc2988a2c, 61 },
    { 0x7b88a2c, 98 },
    { 0xb1288a2e, 76 },
    { 0x61d88a2e, 61 },
    { 0x9a688a30, 100 },
    { 0x86588a31, 69 },
    { 0, 0 },
    { 0xc2c88a33, 90 },
    { 0x7de88a34, 33 },
    { 0x7a988a35, 61 },
    { 0x5a188a36, 95 },
    { 0, 0 },
    { 0xb2c88a38, 95 },
    { 0, 0 },
    { 0xe7688a3a, 91 },
    { 0x63988a3a, 23 },
    { 0x45388a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d88a41, 90 },
    { 0x44688a41, 46 },
    { 0x17e88a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a988a47, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b88a4b, 95 },
    { 0, 0 },
    { 0x82588a4d, 23 },
    { 0, 0 },
    { 0xae088a4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e88a52, 87 },
    { 0x5e588a52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3688a59, 33 },
    { 0, 0 },
    { 0xfa988a5b, 90 },
    { 0, 0 },
    { 0x56e88a5d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f88a64, 33 },
    { 0xb7288a65, 90 },
    { 0, 0 },
    { 0x55288a67, 61 },
    { 0x67988a67, 23 },
    { 0x49a88a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d788a6d, 55 },
    { 0, 0 },
    { 0x9288a6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25688a72, 83 },
    { 0, 0 },
    { 0x71288a74, 91 },
    { 0x7ee88a74, 52 },
    { 0xc6d88a75, 52 },
    { 0, 0 },
    { 0xa8288a78, 100 },
    { 0x32888a79, 76 },
    { 0x5e188a78, 76 },
    { 0x25788a79, 61 },
    { 0x2c488a7c, 55 },
    { 0x72288a7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3088a82, 43 },
    { 0, 0 },
    { 0xe0588a84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c88a89, 66 },
    { 0x90f88a8a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e88a8d, 61 },
    { 0x89c88a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af88a96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a088a9b, 98 },
    { 0, 0 },
    { 0x59d88a9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c988aa2, 90 },
    { 0x3c088aa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49288aa8, 97 },
    { 0, 0 },
    { 0xb7288aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3088aaf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f88ab2, 23 },
    { 0x2b488ab3, 61 },
    { 0x10388ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e88ab9, 95 },
    { 0x39988ab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b88abd, 23 },
    { 0xe8888abe, 33 },
    { 0x5dc88abd, 61 },
    { 0xaae88ac0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b788ac4, 93 },
    { 0x93788ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71888aca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df88acd, 92 },
    { 0x26f88acd, 23 },
    { 0x41588acf, 98 },
    { 0xe3988ad0, 55 },
    { 0xfaa88ad1, 66 },
    { 0xd5a88ad2, 87 },
    { 0x36388ad2, 40 },
    { 0x3be88ad4, 66 },
    { 0x87288acd, 23 },
    { 0, 0 },
    { 0xb7e88ad7, 33 },
    { 0xa6288ad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88adb, 90 },
    { 0, 0 },
    { 0x17b88add, 55 },
    { 0x41488ade, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2788ae3, 34 },
    { 0x66488ae4, 33 },
    { 0xe3488ae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14288aec, 87 },
    { 0x53388aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688af0, 33 },
    { 0xc5b88af1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x53888af4, 23 },
    { 0x14588af5, 61 },
    { 0x82e88af6, 84 },
    { 0xe4888af4, 90 },
    { 0xf188af5, 95 },
    { 0x6e388af4, 90 },
    { 0, 0 },
    { 0x56b88afb, 61 },
    { 0, 0 },
    { 0x31b88afd, 92 },
    { 0x22588afe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52188b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b88b04, 76 },
    { 0x15e88b05, 93 },
    { 0x62888b05, 61 },
    { 0x8fd88b04, 90 },
    { 0x16788b08, 95 },
    { 0xfab88b07, 75 },
    { 0xb5888b09, 33 },
    { 0x82188b0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa188b0e, 61 },
    { 0x69688b0f, 99 },
    { 0, 0 },
    { 0x56b88b11, 33 },
    { 0x59788b11, 90 },
    { 0xd5b88b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa588b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70888b1f, 90 },
    { 0xb3d88b1f, 46 },
    { 0x89d88b21, 91 },
    { 0x82388b21, 90 },
    { 0xdf188b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd88b29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x50988b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd588b33, 61 },
    { 0x15688b34, 90 },
    { 0xcc988b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52488b3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc088b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c588b43, 95 },
    { 0xa6b88b44, 95 },
    { 0, 0 },
    { 0xfae88b46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12588b4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda488b4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81188b51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5888b57, 90 },
    { 0x43d88b57, 99 },
    { 0x6b088b59, 95 },
    { 0xffd88b58, 90 },
    { 0x5bd88b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1888b5e, 61 },
    { 0xd9288b5f, 95 },
    { 0x9b788b60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd88b63, 99 },
    { 0, 0 },
    { 0xf388b65, 76 },
    { 0, 0 },
    { 0xa5588b67, 34 },
    { 0x7f88b67, 98 },
    { 0x18188b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee088b6d, 76 },
    { 0xf8e88b6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff288b72, 90 },
    { 0x8e988b72, 61 },
    { 0xbcc88b74, 90 },
    { 0x2a988b75, 95 },
    { 0x96188b76, 78 },
    { 0xe0b88b77, 78 },
    { 0x67a88b74, 34 },
    { 0xc4188b79, 23 },
    { 0x45988b79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe388b7d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c88b84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee88b8a, 61 },
    { 0, 0 },
    { 0x5288b8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8788b8f, 90 },
    { 0x93788b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f688b95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25988b98, 61 },
    { 0x13388b99, 90 },
    { 0, 0 },
    { 0x4688b9b, 23 },
    { 0x9e388b9c, 33 },
    { 0xbba88b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b88ba0, 33 },
    { 0x65e88ba0, 90 },
    { 0x10e88ba2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf588ba5, 98 },
    { 0, 0 },
    { 0x18688ba7, 33 },
    { 0x8ca88ba7, 23 },
    { 0xb4688ba9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c288bac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4688bb4, 23 },
    { 0, 0 },
    { 0x15788bb6, 90 },
    { 0x4c988bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f88bba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f88bbe, 95 },
    { 0, 0 },
    { 0xf0388bc0, 91 },
    { 0x35a88bc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc88bc4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf88bc7, 34 },
    { 0x91888bc7, 33 },
    { 0xc2988bc8, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a988bcc, 90 },
    { 0x93988bcc, 33 },
    { 0xf7f88bce, 90 },
    { 0, 0 },
    { 0x2688bd0, 38 },
    { 0xe9488bd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a88bd4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b88bd7, 99 },
    { 0xe7b88bd8, 33 },
    { 0x6ff88bd9, 34 },
    { 0, 0 },
    { 0x1d388bdb, 77 },
    { 0, 0 },
    { 0x37988bdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a688be0, 52 },
    { 0xf4388be0, 22 },
    { 0x60288be2, 33 },
    { 0xc3688be3, 81 },
    { 0x6a588be3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb88be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4288bec, 23 },
    { 0x4e988bed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4388bf8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba388bfc, 95 },
    { 0xe3d88bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97788c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86788c03, 43 },
    { 0xfdd88c03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a88c07, 95 },
    { 0, 0 },
    { 0x43388c09, 61 },
    { 0x62e88c0a, 51 },
    { 0x3a988c0b, 99 },
    { 0x29d88c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de88c0f, 55 },
    { 0x99e88c0f, 90 },
    { 0x64988c11, 98 },
    { 0x9a288c12, 93 },
    { 0x78688c10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88c16, 98 },
    { 0x2fe88c16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d88c1b, 95 },
    { 0xc3e88c1c, 68 },
    { 0, 0 },
    { 0xca488c1e, 90 },
    { 0xd9988c1e, 33 },
    { 0x9588c1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51388c26, 93 },
    { 0x57688c26, 33 },
    { 0, 0 },
    { 0x52e88c29, 17 },
    { 0xccb88c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1888c33, 34 },
    { 0x9df88c34, 55 },
    { 0x54e88c35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e988c39, 76 },
    { 0x3f088c3a, 91 },
    { 0x87688c3b, 98 },
    { 0xbb788c3b, 95 },
    { 0xd9e88c3d, 98 },
    { 0x2f888c3d, 98 },
    { 0x3d888c3d, 84 },
    { 0xdf288c40, 55 },
    { 0xbca88c41, 33 },
    { 0, 0 },
    { 0xae488c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94088c46, 23 },
    { 0x8d988c46, 34 },
    { 0xf0588c48, 23 },
    { 0x58788c48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c88c4c, 100 },
    { 0xc4888c4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d88c50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee88c54, 76 },
    { 0xaa688c55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7488c59, 95 },
    { 0xd1e88c59, 90 },
    { 0x87188c5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5188c5f, 52 },
    { 0x71788c60, 33 },
    { 0, 0 },
    { 0xb0888c62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56488c65, 95 },
    { 0x52c88c66, 98 },
    { 0x55d88c66, 90 },
    { 0x94a88c68, 90 },
    { 0, 0 },
    { 0x6d588c6a, 87 },
    { 0xd3588c6a, 99 },
    { 0xe9f88c6a, 98 },
    { 0x6d588c6c, 55 },
    { 0x40688c6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a88c72, 98 },
    { 0x3b88c73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac88c78, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2488c7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1988c7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60288c82, 90 },
    { 0x1a188c83, 90 },
    { 0xcfe88c83, 55 },
    { 0xe9f88c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46988c8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87288c92, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5288c96, 52 },
    { 0xd4088c97, 61 },
    { 0xb0788c96, 61 },
    { 0x16188c98, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5188c9c, 33 },
    { 0x91e88c9d, 61 },
    { 0x32088c9d, 98 },
    { 0x79488c9e, 70 },
    { 0x49588ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b88caa, 95 },
    { 0, 0 },
    { 0x15c88cac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a88caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c388cb6, 46 },
    { 0x6a488cb6, 52 },
    { 0x90088cb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14488cc1, 64 },
    { 0, 0 },
    { 0xeef88cc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65288cc6, 90 },
    { 0x67188cc7, 85 },
    { 0x29b88cc7, 98 },
    { 0x57d88cc7, 33 },
    { 0xdb288cc8, 95 },
    { 0x61088ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba888cd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea88cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6388cdd, 98 },
    { 0xc3688cde, 23 },
    { 0x21988cdf, 76 },
    { 0x82d88ce0, 93 },
    { 0x20e88cdd, 61 },
    { 0x2f988ce0, 90 },
    { 0xe0888ce3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75088cf1, 69 },
    { 0, 0 },
    { 0xace88cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88cf6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44688cfb, 33 },
    { 0x7c188cfc, 23 },
    { 0x59c88cfd, 95 },
    { 0x95c88cfe, 90 },
    { 0, 0 },
    { 0x59588d00, 81 },
    { 0x17488d01, 23 },
    { 0xd4e88d01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb488d06, 61 },
    { 0x80f88d06, 61 },
    { 0xe2988d07, 68 },
    { 0x7af88d06, 34 },
    { 0xd0688d0a, 90 },
    { 0x98688d0a, 61 },
    { 0x7dd88d0c, 99 },
    { 0x2d88d0d, 76 },
    { 0x21088d0d, 22 },
    { 0x11588d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c788d12, 33 },
    { 0, 0 },
    { 0xee688d14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f388d1a, 33 },
    { 0x3cc88d1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf88d1f, 98 },
    { 0, 0 },
    { 0x9de88d21, 33 },
    { 0xd4388d22, 90 },
    { 0x7c788d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83188d27, 100 },
    { 0x2cd88d27, 33 },
    { 0x2a088d28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b88d32, 43 },
    { 0, 0 },
    { 0x5788d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29288d3a, 61 },
    { 0x26a88d3b, 33 },
    { 0xb0c88d3c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae88d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b88d42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb688d4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d88d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89288d57, 91 },
    { 0x38688d58, 61 },
    { 0x7dc88d58, 61 },
    { 0x6f788d59, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50788d62, 38 },
    { 0x28588d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc588d66, 88 },
    { 0x81088d67, 90 },
    { 0, 0 },
    { 0xc9888d69, 61 },
    { 0, 0 },
    { 0xaf888d6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21288d71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11188d76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x90688d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc88d7c, 90 },
    { 0x9da88d7c, 78 },
    { 0xa0988d7d, 23 },
    { 0, 0 },
    { 0x89488d80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x86588d83, 46 },
    { 0xfed88d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab88d88, 90 },
    { 0xcf488d89, 90 },
    { 0, 0 },
    { 0x1a88d8b, 85 },
    { 0xde788d8c, 99 },
    { 0x43888d8c, 55 },
    { 0x6d988d8d, 90 },
    { 0, 0 },
    { 0x8da88d90, 33 },
    { 0x77d88d90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f88d97, 92 },
    { 0, 0 },
    { 0xc188d99, 91 },
    { 0xf3188d99, 4 },
    { 0xdd388d99, 99 },
    { 0, 0 },
    { 0x83888d9d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe688da0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49388da7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4388dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee88dae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92288db2, 85 },
    { 0, 0 },
    { 0x54b88db4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7788db8, 66 },
    { 0xbf188db9, 77 },
    { 0x1e188dba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x64888dbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80788dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed88dc8, 90 },
    { 0xb3288dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17288dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74688dd7, 33 },
    { 0x3f288dd7, 33 },
    { 0xcd488dd9, 90 },
    { 0x32f88dd9, 95 },
    { 0, 0 },
    { 0x5088ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e88ddf, 33 },
    { 0, 0 },
    { 0x84188de1, 55 },
    { 0xba788de2, 90 },
    { 0x57f88de3, 55 },
    { 0xd4b88de4, 90 },
    { 0x97e88de5, 33 },
    { 0, 0 },
    { 0x54888de7, 52 },
    { 0, 0 },
    { 0x5ce88de9, 90 },
    { 0x46288de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b88ded, 95 },
    { 0xa7a88dee, 46 },
    { 0xb0988dee, 99 },
    { 0, 0 },
    { 0xfc588df1, 52 },
    { 0, 0 },
    { 0xbb888df3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81288df8, 98 },
    { 0xc9488df8, 90 },
    { 0x60388dfa, 99 },
    { 0xa7b88dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a88dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d288e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f88e07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12988e0a, 90 },
    { 0x6c988e0a, 75 },
    { 0, 0 },
    { 0x68388e0d, 33 },
    { 0x49188e0e, 90 },
    { 0xb6b88e0e, 66 },
    { 0xab788e10, 61 },
    { 0x54688e11, 90 },
    { 0x12388e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e788e17, 33 },
    { 0x40f88e18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b88e1f, 26 },
    { 0, 0 },
    { 0x68288e21, 43 },
    { 0x72a88e22, 34 },
    { 0x44388e23, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50388e26, 33 },
    { 0x8ee88e27, 24 },
    { 0, 0 },
    { 0xf4688e29, 34 },
    { 0, 0 },
    { 0xa2488e2b, 90 },
    { 0, 0 },
    { 0xd6a88e2d, 66 },
    { 0x46a88e2d, 76 },
    { 0xe7088e2e, 46 },
    { 0x95d88e30, 43 },
    { 0, 0 },
    { 0x16088e32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e88e35, 75 },
    { 0xcb188e36, 91 },
    { 0, 0 },
    { 0x85f88e38, 95 },
    { 0x5f388e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38388e3d, 78 },
    { 0x67288e3e, 33 },
    { 0x9dc88e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a88e45, 76 },
    { 0x68288e46, 43 },
    { 0xac88e47, 33 },
    { 0xb9d88e48, 84 },
    { 0, 0 },
    { 0x74288e4a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce688e4f, 100 },
    { 0x24d88e50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0388e54, 95 },
    { 0, 0 },
    { 0x54788e56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88e5a, 76 },
    { 0, 0 },
    { 0xe2288e5c, 98 },
    { 0x4c488e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec88e60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc88e64, 33 },
    { 0, 0 },
    { 0x20688e66, 61 },
    { 0, 0 },
    { 0x32388e68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x35588e6b, 33 },
    { 0, 0 },
    { 0xde688e6d, 33 },
    { 0x5a988e6e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb388e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91988e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1588e7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a188e87, 95 },
    { 0xb0e88e88, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97988e90, 100 },
    { 0x30388e91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c088e97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf88e9a, 77 },
    { 0x48688e9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44288ea1, 78 },
    { 0xc9f88ea2, 93 },
    { 0, 0 },
    { 0x45188ea4, 90 },
    { 0xdf688ea5, 17 },
    { 0, 0 },
    { 0xb1788ea7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x55088eaa, 85 },
    { 0, 0 },
    { 0x46088eac, 23 },
    { 0xe688ead, 44 },
    { 0, 0 },
    { 0xd0288eaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d88eb5, 90 },
    { 0x32f88eb6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a888ebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc188ec0, 90 },
    { 0, 0 },
    { 0xf0688ec2, 55 },
    { 0x4988ec2, 95 },
    { 0x59688ec2, 53 },
    { 0xac388ec5, 55 },
    { 0x46c88ec2, 67 },
    { 0x7c888ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16988eca, 55 },
    { 0xcd88ecb, 52 },
    { 0x9ad88ecc, 61 },
    { 0, 0 },
    { 0xa8b88ece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e88ed1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33388ee1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4788ee8, 61 },
    { 0x28e88ee9, 90 },
    { 0x21288ee9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88eed, 55 },
    { 0xd1788eee, 98 },
    { 0x6c388eef, 66 },
    { 0x3f288ef0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3488ef7, 90 },
    { 0x86388ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31788f00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c88f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb88f08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0788f0c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99388f10, 40 },
    { 0, 0 },
    { 0x5188f12, 91 },
    { 0, 0 },
    { 0xcb488f14, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e188f18, 90 },
    { 0, 0 },
    { 0x13788f1a, 95 },
    { 0, 0 },
    { 0xbe088f1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f088f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d88f24, 87 },
    { 0x33388f25, 98 },
    { 0x9d688f26, 23 },
    { 0x28e88f27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c588f2b, 90 },
    { 0xbdc88f2c, 61 },
    { 0x9d988f2c, 90 },
    { 0x51d88f2d, 98 },
    { 0x4a88f2f, 46 },
    { 0xe2b88f30, 91 },
    { 0xc7c88f30, 43 },
    { 0x65f88f32, 70 },
    { 0x48a88f30, 61 },
    { 0, 0 },
    { 0xf588f35, 23 },
    { 0xd2688f36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9488f3a, 43 },
    { 0x14288f3b, 33 },
    { 0, 0 },
    { 0x9cf88f3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40988f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b88f44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae188f49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88f51, 95 },
    { 0x89788f52, 55 },
    { 0, 0 },
    { 0x12388f54, 81 },
    { 0x94088f55, 76 },
    { 0x76f88f56, 55 },
    { 0x64b88f57, 33 },
    { 0, 0 },
    { 0x1aa88f59, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83488f5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad88f6a, 61 },
    { 0, 0 },
    { 0x9cd88f6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5788f72, 61 },
    { 0xcf688f72, 90 },
    { 0xb4388f72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb188f7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c788f83, 43 },
    { 0xb1088f83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1388f87, 98 },
    { 0x76a88f88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91688f8c, 33 },
    { 0x7e588f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84688f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a88f97, 61 },
    { 0x4e288f98, 55 },
    { 0, 0 },
    { 0x1ef88f9a, 43 },
    { 0xf788f9a, 95 },
    { 0x46488f9c, 76 },
    { 0, 0 },
    { 0xdeb88f9e, 84 },
    { 0xf8888f9e, 60 },
    { 0x46d88fa0, 90 },
    { 0, 0 },
    { 0x20688fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16988fb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e88fb4, 98 },
    { 0x43988fb5, 17 },
    { 0xdc588fb6, 40 },
    { 0x73a88fb7, 90 },
    { 0xa5988fb7, 61 },
    { 0x29a88fb9, 66 },
    { 0x1e688fba, 43 },
    { 0x18e88fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a88fc0, 88 },
    { 0, 0 },
    { 0x47188fc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5888fcc, 90 },
    { 0xbd288fcc, 33 },
    { 0x3c088fcc, 23 },
    { 0xab488fcc, 90 },
    { 0x9a588fce, 76 },
    { 0xd8f88fd1, 52 },
    { 0x34988fd2, 93 },
    { 0xa4888fce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d788fd6, 23 },
    { 0xbf288fd7, 55 },
    { 0x3da88fd6, 34 },
    { 0x81588fd9, 98 },
    { 0x23688fd9, 80 },
    { 0xa4f88fdb, 23 },
    { 0xa6b88fdc, 23 },
    { 0xce888fdd, 23 },
    { 0x4cd88fde, 55 },
    { 0x38988fdd, 91 },
    { 0xe1d88fd6, 33 },
    { 0xb0b88fd6, 91 },
    { 0, 0 },
    { 0x3d688fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d488fe7, 34 },
    { 0x83a88fe8, 23 },
    { 0xc0888fe9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae88fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39288ff0, 98 },
    { 0x56088ff1, 33 },
    { 0x6588ff2, 46 },
    { 0xfff88ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d88ff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42488ffa, 100 },
    { 0x55488ffb, 61 },
    { 0, 0 },
    { 0xaf388ffd, 61 },
    { 0x91388ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7389002, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f89007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b8900a, 52 },
    { 0, 0 },
    { 0xd78900c, 78 },
    { 0, 0 },
    { 0xe2e8900e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b89018, 46 },
    { 0xabc89019, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8901d, 61 },
    { 0, 0 },
    { 0x9e28901f, 69 },
    { 0xe208901f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f89026, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff98902b, 98 },
    { 0, 0 },
    { 0xf0c8902d, 43 },
    { 0x7af8902d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb789038, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8598903b, 61 },
    { 0x5528903c, 88 },
    { 0xb5f8903d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf489040, 90 },
    { 0xe3389041, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96189049, 33 },
    { 0, 0 },
    { 0x32a8904b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe248904f, 87 },
    { 0x48b8904f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd89054, 90 },
    { 0xc1a89055, 23 },
    { 0xfb589055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a89059, 33 },
    { 0x2508905a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1128905f, 61 },
    { 0, 0 },
    { 0x6a289061, 90 },
    { 0x8ea89061, 64 },
    { 0x61089063, 98 },
    { 0x9a89064, 99 },
    { 0xdc189065, 23 },
    { 0xd3189063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd89073, 88 },
    { 0x58789074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf448907a, 60 },
    { 0x7f98907b, 43 },
    { 0, 0 },
    { 0xa2a8907d, 61 },
    { 0xb268907d, 65 },
    { 0x2a18907e, 17 },
    { 0, 0 },
    { 0xfab89081, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58889084, 90 },
    { 0xff089084, 85 },
    { 0x73c89086, 69 },
    { 0x5d189084, 98 },
    { 0, 0 },
    { 0x5c289089, 98 },
    { 0x5d489089, 24 },
    { 0, 0 },
    { 0xc248908c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c989095, 100 },
    { 0x92a89096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e38909b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f8909e, 78 },
    { 0xe3b8909f, 95 },
    { 0xfc68909e, 91 },
    { 0x803890a1, 68 },
    { 0x4218909e, 83 },
    { 0x2b890a0, 33 },
    { 0x899890a4, 61 },
    { 0x9b6890a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658890aa, 99 },
    { 0x3a7890ab, 84 },
    { 0xc21890ab, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c890b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72890bd, 66 },
    { 0xff2890be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569890c3, 75 },
    { 0x78890c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2890cc, 52 },
    { 0, 0 },
    { 0x6c9890ce, 95 },
    { 0x3d9890cf, 87 },
    { 0x72f890ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e890d3, 78 },
    { 0, 0 },
    { 0xf5d890d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f890d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b890dc, 75 },
    { 0x74890dd, 98 },
    { 0x1a8890de, 61 },
    { 0x158890de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d890e5, 43 },
    { 0xbce890e5, 33 },
    { 0x58b890e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87890ed, 85 },
    { 0xcd5890ee, 88 },
    { 0xf83890ee, 90 },
    { 0xfff890f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41890f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920890fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15089100, 90 },
    { 0x6489100, 95 },
    { 0, 0 },
    { 0x4f589103, 84 },
    { 0xb2c89103, 88 },
    { 0xd6189104, 95 },
    { 0xc489106, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc078910c, 55 },
    { 0xdb98910d, 33 },
    { 0xba8910e, 30 },
    { 0x65e8910e, 95 },
    { 0, 0 },
    { 0x58089111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93889115, 55 },
    { 0xac889116, 95 },
    { 0, 0 },
    { 0xaad89118, 95 },
    { 0, 0 },
    { 0xde8911a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe889122, 66 },
    { 0xa0389122, 33 },
    { 0x29e89124, 85 },
    { 0, 0 },
    { 0xdd589126, 90 },
    { 0xf589127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28912c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf08912f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b89133, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc89136, 52 },
    { 0xbbe89137, 98 },
    { 0x24889138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c8913c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e89140, 61 },
    { 0, 0 },
    { 0x9ff89142, 90 },
    { 0x48c89143, 34 },
    { 0xf1e89144, 33 },
    { 0xc789145, 23 },
    { 0xac689146, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb789149, 34 },
    { 0x4958914a, 98 },
    { 0x9689149, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3938914f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb789152, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8189156, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b8915a, 95 },
    { 0x48915b, 23 },
    { 0xb8a8915c, 52 },
    { 0, 0 },
    { 0x92d8915e, 78 },
    { 0xbf58915f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c89163, 95 },
    { 0x20f89163, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac8916a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8916d, 33 },
    { 0x498916e, 98 },
    { 0xbb58916f, 90 },
    { 0x61589170, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d89173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a89178, 43 },
    { 0xfcb89178, 98 },
    { 0xfd38917a, 61 },
    { 0xbff89178, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa8917e, 42 },
    { 0x2c98917f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a89182, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41489186, 61 },
    { 0, 0 },
    { 0x15a89188, 76 },
    { 0xdab89189, 90 },
    { 0, 0 },
    { 0xf1c8918b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db8918f, 61 },
    { 0x9bf8918f, 100 },
    { 0, 0 },
    { 0x91489192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb89196, 77 },
    { 0, 0 },
    { 0x1f289198, 81 },
    { 0x10989199, 98 },
    { 0, 0 },
    { 0x1218919b, 98 },
    { 0xc368919c, 78 },
    { 0x28919d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26891a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62891af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11891b2, 81 },
    { 0, 0 },
    { 0x63a891b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f891bd, 78 },
    { 0xb42891bd, 98 },
    { 0x426891bf, 23 },
    { 0x5cd891c0, 87 },
    { 0xf43891c0, 33 },
    { 0x52e891c0, 95 },
    { 0, 0 },
    { 0xb2e891c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779891ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8891d3, 95 },
    { 0x92891d3, 33 },
    { 0xe2b891d4, 95 },
    { 0x245891d6, 93 },
    { 0x239891d7, 99 },
    { 0xc1b891d8, 52 },
    { 0xa39891d8, 98 },
    { 0xa9e891d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5891df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c891e5, 46 },
    { 0xef3891e6, 33 },
    { 0x56891e6, 95 },
    { 0xe32891e8, 40 },
    { 0xd0c891e8, 34 },
    { 0x9e5891e9, 43 },
    { 0x58a891eb, 55 },
    { 0xb9f891ec, 55 },
    { 0xcd5891ed, 67 },
    { 0x432891e6, 23 },
    { 0xc91891ef, 89 },
    { 0, 0 },
    { 0x82b891f1, 90 },
    { 0x45f891f1, 95 },
    { 0, 0 },
    { 0x893891f4, 55 },
    { 0x46c891f4, 23 },
    { 0x3ba891f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e891f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d89200, 33 },
    { 0xb7389200, 43 },
    { 0x25689202, 95 },
    { 0x78689203, 98 },
    { 0x55c89201, 86 },
    { 0x8af89205, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce58920b, 78 },
    { 0, 0 },
    { 0xe958920d, 55 },
    { 0, 0 },
    { 0xf5b8920f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8289212, 95 },
    { 0xb1d89213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89217, 61 },
    { 0x90c89218, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a8921b, 17 },
    { 0x3f48921c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1878921f, 88 },
    { 0, 0 },
    { 0x64c89221, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59689226, 76 },
    { 0x83889227, 90 },
    { 0, 0 },
    { 0x18289229, 46 },
    { 0x6b58922a, 52 },
    { 0, 0 },
    { 0x7a48922c, 91 },
    { 0x70d8922d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a89236, 43 },
    { 0xdd989236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3848923b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6689240, 33 },
    { 0, 0 },
    { 0xf8789242, 23 },
    { 0x84089243, 33 },
    { 0xbcb89244, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16389252, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48589259, 95 },
    { 0x82a8925a, 95 },
    { 0xf028925b, 61 },
    { 0x6b98925c, 91 },
    { 0x7a18925b, 61 },
    { 0xcdd8925e, 34 },
    { 0x3e58925c, 95 },
    { 0, 0 },
    { 0x5a789261, 99 },
    { 0xcce89262, 87 },
    { 0x6a089262, 23 },
    { 0xc3f89264, 76 },
    { 0xf0389265, 100 },
    { 0xb4489264, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d88926a, 34 },
    { 0, 0 },
    { 0x798926c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa89273, 61 },
    { 0xfec89273, 33 },
    { 0xf1a89273, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f89279, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d78927c, 34 },
    { 0x8bb8927d, 84 },
    { 0, 0 },
    { 0xe448927f, 95 },
    { 0x18489280, 76 },
    { 0x4af89280, 98 },
    { 0xed489280, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b389290, 76 },
    { 0x32289291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff289295, 55 },
    { 0xc9a89296, 33 },
    { 0, 0 },
    { 0xc4d89298, 95 },
    { 0x1c089299, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4892a1, 33 },
    { 0xb32892a2, 33 },
    { 0x5f2892a3, 90 },
    { 0xb00892a3, 34 },
    { 0, 0 },
    { 0xb42892a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed892a9, 76 },
    { 0, 0 },
    { 0xac8892ab, 40 },
    { 0x2ba892ab, 67 },
    { 0, 0 },
    { 0xfde892ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc892b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e892b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0892bb, 43 },
    { 0xe78892bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b892bf, 38 },
    { 0xfaa892c0, 33 },
    { 0, 0 },
    { 0xd06892c2, 56 },
    { 0x56c892c2, 33 },
    { 0xe6d892c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c892ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67892d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a892d5, 43 },
    { 0xf85892d6, 23 },
    { 0x492892d5, 84 },
    { 0xe9a892d6, 95 },
    { 0xace892d9, 76 },
    { 0x9dd892da, 75 },
    { 0xbde892db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37892de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1892e4, 98 },
    { 0x590892e5, 93 },
    { 0xbb1892e6, 61 },
    { 0xa98892e5, 98 },
    { 0x547892e8, 67 },
    { 0, 0 },
    { 0x85b892ea, 90 },
    { 0, 0 },
    { 0x4bc892ec, 55 },
    { 0x205892ed, 75 },
    { 0x510892ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213892f2, 23 },
    { 0x435892f2, 76 },
    { 0x882892f4, 98 },
    { 0, 0 },
    { 0x500892f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f892fb, 61 },
    { 0x1fe892fb, 43 },
    { 0x213892fd, 95 },
    { 0x870892fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f089301, 34 },
    { 0, 0 },
    { 0x4d389303, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26489306, 33 },
    { 0, 0 },
    { 0x37389308, 64 },
    { 0x4ab89309, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88930c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68930f, 90 },
    { 0x31789310, 77 },
    { 0x4e08930f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6289314, 95 },
    { 0xe6389314, 91 },
    { 0x6b489316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e689319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa678931d, 93 },
    { 0xb948931e, 61 },
    { 0xbf18931f, 33 },
    { 0xf98931f, 78 },
    { 0xb8389320, 99 },
    { 0xb3e8931d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39989326, 93 },
    { 0, 0 },
    { 0x3c689328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d89330, 23 },
    { 0, 0 },
    { 0xf1589332, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f89335, 93 },
    { 0x1589335, 34 },
    { 0x30489337, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d8933a, 90 },
    { 0x8898933b, 23 },
    { 0xf8933a, 33 },
    { 0x6728933d, 95 },
    { 0xbab8933c, 33 },
    { 0x5308933f, 53 },
    { 0x32889340, 90 },
    { 0xf9289341, 23 },
    { 0x82e8933e, 99 },
    { 0, 0 },
    { 0xa1c89344, 90 },
    { 0xc2289344, 61 },
    { 0, 0 },
    { 0x3b389347, 67 },
    { 0x71189348, 33 },
    { 0, 0 },
    { 0x6648934a, 95 },
    { 0x44f8934b, 68 },
    { 0xdb58934b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d89350, 95 },
    { 0xf2389351, 88 },
    { 0xfda89351, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1289355, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e89358, 88 },
    { 0xa1589358, 95 },
    { 0xce18935a, 43 },
    { 0, 0 },
    { 0x11e8935c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98689362, 33 },
    { 0, 0 },
    { 0x5f389364, 33 },
    { 0x4c289364, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf28936b, 34 },
    { 0x2ac8936c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b89373, 61 },
    { 0x1d989373, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7889379, 55 },
    { 0xc489379, 61 },
    { 0xfd28937b, 61 },
    { 0x67b8937c, 61 },
    { 0x3ec8937c, 76 },
    { 0xe48937a, 91 },
    { 0xf48937f, 95 },
    { 0, 0 },
    { 0x8a389381, 61 },
    { 0x1f389382, 91 },
    { 0x2e089382, 33 },
    { 0x7c789382, 43 },
    { 0, 0 },
    { 0x1e789386, 84 },
    { 0, 0 },
    { 0x28689388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408939d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0893a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x322893a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e893a8, 98 },
    { 0, 0 },
    { 0x99893aa, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7893ad, 34 },
    { 0xa5d893ae, 76 },
    { 0x382893ad, 91 },
    { 0, 0 },
    { 0x5cc893b1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x179893b4, 90 },
    { 0, 0 },
    { 0x9e9893b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b893b9, 98 },
    { 0x839893b9, 55 },
    { 0, 0 },
    { 0x1de893bc, 88 },
    { 0xefa893bd, 46 },
    { 0xcf1893be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777893c4, 87 },
    { 0, 0 },
    { 0xa47893c6, 100 },
    { 0x164893c7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad893cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0893ce, 55 },
    { 0xe05893ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d893d5, 33 },
    { 0xbef893d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0893dc, 90 },
    { 0xe37893dd, 85 },
    { 0xae8893dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x377893e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858893e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2893ea, 34 },
    { 0xfb5893ea, 95 },
    { 0, 0 },
    { 0xe0893ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba893f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b893f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb893fb, 33 },
    { 0x596893fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac893ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab989405, 98 },
    { 0, 0 },
    { 0x3bc89407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b8940f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d89413, 23 },
    { 0x29589413, 90 },
    { 0x4c189415, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x63589418, 34 },
    { 0, 0 },
    { 0xdd98941a, 33 },
    { 0x9308941b, 66 },
    { 0x1498941c, 33 },
    { 0x9988941c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d389420, 76 },
    { 0, 0 },
    { 0x42789422, 55 },
    { 0, 0 },
    { 0x52089424, 33 },
    { 0x43489425, 76 },
    { 0x64889426, 93 },
    { 0xba689427, 23 },
    { 0xce489425, 90 },
    { 0, 0 },
    { 0x4668942a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a8942d, 87 },
    { 0, 0 },
    { 0x4758942f, 76 },
    { 0, 0 },
    { 0x41a89431, 33 },
    { 0x51389432, 23 },
    { 0x40589433, 90 },
    { 0x3b489434, 23 },
    { 0x36189431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4189439, 33 },
    { 0xf4489439, 95 },
    { 0, 0 },
    { 0x7828943c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e89442, 33 },
    { 0xf889443, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd89446, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488944a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa488944d, 61 },
    { 0xb1d8944e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25989451, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae89456, 33 },
    { 0x90c89457, 43 },
    { 0x44c89457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc098945b, 96 },
    { 0x6998945b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3968945f, 90 },
    { 0, 0 },
    { 0xc7789461, 33 },
    { 0x2b389462, 90 },
    { 0, 0 },
    { 0x99289464, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1589468, 5 },
    { 0xb0489469, 98 },
    { 0xf8e8946a, 93 },
    { 0xe328946b, 23 },
    { 0xd108946a, 98 },
    { 0xc088946d, 76 },
    { 0x22b8946e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b89475, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789479, 95 },
    { 0x9798947a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b289480, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc189487, 36 },
    { 0x5c789487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd618948b, 100 },
    { 0, 0 },
    { 0xb948948d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de89493, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf008949a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e8949f, 23 },
    { 0xb18949f, 90 },
    { 0x96d894a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e894ae, 61 },
    { 0x20894af, 33 },
    { 0xbdf894b0, 53 },
    { 0, 0 },
    { 0x70e894b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343894b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e894bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa894bf, 23 },
    { 0x1e3894c0, 90 },
    { 0x9d894bf, 95 },
    { 0xe26894c2, 84 },
    { 0x22f894c2, 100 },
    { 0x6de894c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f894c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c894cd, 34 },
    { 0x678894ce, 46 },
    { 0, 0 },
    { 0xb1e894d0, 88 },
    { 0xea7894d0, 95 },
    { 0x40c894d0, 90 },
    { 0, 0 },
    { 0x633894d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154894d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5894e3, 33 },
    { 0x15c894e3, 90 },
    { 0xb13894e5, 95 },
    { 0xfad894e6, 98 },
    { 0, 0 },
    { 0x5894e8, 90 },
    { 0xadb894e9, 33 },
    { 0xd74894ea, 98 },
    { 0xf84894eb, 85 },
    { 0x842894ec, 98 },
    { 0xe3a894ed, 78 },
    { 0x41e894ed, 80 },
    { 0xdee894ee, 91 },
    { 0x6b3894f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0894f3, 98 },
    { 0x6de894f4, 61 },
    { 0, 0 },
    { 0x558894f6, 55 },
    { 0, 0 },
    { 0xb7e894f8, 90 },
    { 0xcdc894f9, 100 },
    { 0xd19894fa, 90 },
    { 0xa2b894fb, 33 },
    { 0x468894f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a089501, 61 },
    { 0, 0 },
    { 0x18889503, 99 },
    { 0x40489504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6089507, 95 },
    { 0x6ac89508, 76 },
    { 0, 0 },
    { 0x7dc8950a, 86 },
    { 0xb0c8950a, 33 },
    { 0xe098950c, 55 },
    { 0, 0 },
    { 0x6f88950e, 77 },
    { 0, 0 },
    { 0xf8289510, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189515, 90 },
    { 0xffe89515, 61 },
    { 0xd2a89516, 33 },
    { 0xaa189518, 23 },
    { 0xfc489518, 90 },
    { 0xb108951a, 95 },
    { 0xde08951b, 76 },
    { 0xc338951b, 23 },
    { 0x3f28951c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a89521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a89524, 95 },
    { 0, 0 },
    { 0x55989526, 77 },
    { 0xff389527, 33 },
    { 0, 0 },
    { 0x74c89529, 34 },
    { 0, 0 },
    { 0xf528952b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e89531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e189536, 78 },
    { 0, 0 },
    { 0x54489538, 98 },
    { 0, 0 },
    { 0xbd8953a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82989544, 93 },
    { 0xbb289545, 99 },
    { 0, 0 },
    { 0x31889547, 84 },
    { 0, 0 },
    { 0x35089549, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8954d, 76 },
    { 0, 0 },
    { 0xfa18954f, 23 },
    { 0x10a8954f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64389555, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3018955b, 33 },
    { 0x36a8955c, 88 },
    { 0x7398955c, 23 },
    { 0x5c8955e, 33 },
    { 0, 0 },
    { 0x9dc89560, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b89563, 84 },
    { 0, 0 },
    { 0x81f89565, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc89570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac389578, 34 },
    { 0x6cb89578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d8957d, 93 },
    { 0x5278957e, 95 },
    { 0, 0 },
    { 0xd1789580, 76 },
    { 0x99889581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec8958a, 46 },
    { 0, 0 },
    { 0x68c8958c, 52 },
    { 0xdf88958c, 34 },
    { 0xac8958e, 90 },
    { 0xda48958e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a89594, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa398959a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268959e, 23 },
    { 0xb3a8959f, 76 },
    { 0x6028959f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3895a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f895a6, 33 },
    { 0x4f2895a7, 85 },
    { 0xf3e895a8, 33 },
    { 0, 0 },
    { 0xe8c895aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb895ad, 61 },
    { 0, 0 },
    { 0xd14895af, 78 },
    { 0x28c895af, 23 },
    { 0xf46895b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b895b4, 90 },
    { 0xc48895b5, 98 },
    { 0x927895b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b895bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3895c5, 59 },
    { 0, 0 },
    { 0x332895c7, 33 },
    { 0x891895c7, 33 },
    { 0x979895c7, 83 },
    { 0, 0 },
    { 0xd33895cb, 43 },
    { 0x5f6895cc, 33 },
    { 0xf82895cc, 55 },
    { 0xa74895ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76895d2, 34 },
    { 0, 0 },
    { 0x675895d4, 61 },
    { 0xc2f895d4, 77 },
    { 0, 0 },
    { 0x380895d7, 88 },
    { 0xc3c895d8, 95 },
    { 0x5cd895d7, 100 },
    { 0, 0 },
    { 0x81e895db, 90 },
    { 0x53f895dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x488895df, 76 },
    { 0x58c895df, 34 },
    { 0x4cd895e0, 61 },
    { 0x328895e2, 90 },
    { 0x541895e3, 46 },
    { 0x670895e3, 33 },
    { 0xa4c895e3, 33 },
    { 0, 0 },
    { 0x74895e7, 90 },
    { 0xb7b895e8, 99 },
    { 0x34c895e7, 100 },
    { 0, 0 },
    { 0xefc895eb, 56 },
    { 0x500895ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813895f6, 34 },
    { 0x515895f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x459895fa, 52 },
    { 0x4a6895fb, 23 },
    { 0x82c895fc, 61 },
    { 0x6c1895fb, 55 },
    { 0xda9895fb, 98 },
    { 0x49f895ff, 98 },
    { 0x749895ff, 33 },
    { 0x88a895ff, 95 },
    { 0xc5b895fc, 33 },
    { 0xa15895fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83389606, 98 },
    { 0x50589607, 90 },
    { 0x1b189608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8960c, 61 },
    { 0, 0 },
    { 0xdb28960e, 90 },
    { 0x4088960f, 95 },
    { 0x8718960f, 98 },
    { 0xd2789611, 33 },
    { 0x6db89611, 100 },
    { 0xa0e89613, 85 },
    { 0xaee89614, 23 },
    { 0x8ed89615, 76 },
    { 0x35a8960f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff8961d, 90 },
    { 0x34c8961e, 34 },
    { 0xb238961e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a389625, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea89628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7628962e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97489631, 43 },
    { 0x39a89632, 100 },
    { 0xfb989632, 99 },
    { 0x2689633, 61 },
    { 0, 0 },
    { 0xbdb89636, 85 },
    { 0xfda89637, 90 },
    { 0, 0 },
    { 0xf5d89639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e18963c, 43 },
    { 0, 0 },
    { 0xf488963e, 46 },
    { 0x37a8963f, 33 },
    { 0xb698963f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37989646, 61 },
    { 0xd5e89647, 55 },
    { 0x9b789646, 55 },
    { 0x4ae89649, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368964c, 76 },
    { 0, 0 },
    { 0xf9b8964e, 46 },
    { 0, 0 },
    { 0x52c89650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d8965a, 98 },
    { 0x52a8965b, 46 },
    { 0xae48965c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a68965f, 90 },
    { 0xbe589660, 55 },
    { 0, 0 },
    { 0x64089662, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36589667, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6348966d, 90 },
    { 0xf558966e, 99 },
    { 0xb8a8966f, 55 },
    { 0xf2e89670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b8967b, 80 },
    { 0, 0 },
    { 0x3b88967d, 61 },
    { 0x9288967d, 61 },
    { 0x6f18967f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f389682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868968a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b78968d, 33 },
    { 0x1f78968e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f89692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e89697, 90 },
    { 0x32289698, 98 },
    { 0x94c89699, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3896a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755896b1, 98 },
    { 0, 0 },
    { 0x210896b3, 66 },
    { 0xd75896b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5896b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e896ba, 76 },
    { 0x3fc896bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78896be, 68 },
    { 0x6a896be, 33 },
    { 0, 0 },
    { 0x973896c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8896c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7896c9, 87 },
    { 0, 0 },
    { 0x863896cb, 33 },
    { 0xdd1896cc, 61 },
    { 0, 0 },
    { 0xdd5896ce, 81 },
    { 0xa3e896cf, 46 },
    { 0x9e7896ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3896d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70896dd, 23 },
    { 0, 0 },
    { 0x5c4896df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981896e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3896e6, 61 },
    { 0x3f1896e6, 90 },
    { 0x2c1896e7, 66 },
    { 0, 0 },
    { 0xbe7896ea, 55 },
    { 0x892896eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25896f1, 93 },
    { 0, 0 },
    { 0x14a896f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4896f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda589704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81089708, 33 },
    { 0xba989709, 95 },
    { 0x6748970a, 23 },
    { 0x9558970b, 55 },
    { 0xa9c8970b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51189710, 61 },
    { 0x70689710, 67 },
    { 0, 0 },
    { 0x75b89713, 33 },
    { 0x42089714, 52 },
    { 0xcb889714, 23 },
    { 0, 0 },
    { 0xc5a89717, 77 },
    { 0, 0 },
    { 0x71c89719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd18971e, 43 },
    { 0x1b18971f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f89724, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f089729, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf788972c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9589730, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a89735, 52 },
    { 0, 0 },
    { 0xb8e89737, 55 },
    { 0x5fb89738, 90 },
    { 0x3d589737, 76 },
    { 0, 0 },
    { 0xf358973b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef89740, 52 },
    { 0x90189741, 98 },
    { 0x42489742, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3589749, 61 },
    { 0x948974a, 61 },
    { 0x9178974b, 98 },
    { 0x96d8974a, 23 },
    { 0xcea8974d, 61 },
    { 0x2b78974e, 61 },
    { 0x8e8974b, 43 },
    { 0xeee89750, 85 },
    { 0xa189751, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x90489754, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f89758, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe598975c, 75 },
    { 0x5ef8975c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b89760, 23 },
    { 0, 0 },
    { 0x62489762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec48976b, 66 },
    { 0x1b38976c, 23 },
    { 0x27d8976c, 90 },
    { 0x8b28976e, 87 },
    { 0x8f28976f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d89773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d89777, 46 },
    { 0xfb489777, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1278977c, 78 },
    { 0, 0 },
    { 0xcb68977e, 98 },
    { 0x24e8977f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c89783, 61 },
    { 0, 0 },
    { 0xa2789785, 64 },
    { 0, 0 },
    { 0x42389787, 95 },
    { 0xf1289788, 95 },
    { 0, 0 },
    { 0x5e08978a, 34 },
    { 0, 0 },
    { 0x7228978c, 33 },
    { 0x7b28978d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64189793, 95 },
    { 0, 0 },
    { 0x2ed89795, 66 },
    { 0xc0789796, 33 },
    { 0x8f189797, 86 },
    { 0xdd189798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7788979c, 33 },
    { 0, 0 },
    { 0x828979e, 23 },
    { 0, 0 },
    { 0x78f897a0, 77 },
    { 0x24c897a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61897a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76897ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd897b0, 40 },
    { 0x3f2897b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf897b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2897b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b897bd, 90 },
    { 0x92b897bd, 76 },
    { 0x9f7897bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93897c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72897c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce897d0, 65 },
    { 0xf1a897d1, 33 },
    { 0xcf9897d2, 23 },
    { 0x818897d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0897d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29897e0, 75 },
    { 0x8f2897e1, 23 },
    { 0x12e897e1, 91 },
    { 0x4a1897e3, 34 },
    { 0x6ca897e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792897e9, 85 },
    { 0, 0 },
    { 0x615897eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0897f3, 43 },
    { 0x4c0897f4, 95 },
    { 0x999897f5, 88 },
    { 0xcb2897f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec897fe, 95 },
    { 0x6b9897ff, 55 },
    { 0, 0 },
    { 0xfff89801, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf489806, 84 },
    { 0, 0 },
    { 0xccb89808, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8980b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba89811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b889818, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c8981b, 34 },
    { 0x29a8981c, 99 },
    { 0xaf28981d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d89821, 95 },
    { 0, 0 },
    { 0x92789823, 90 },
    { 0x4c589823, 95 },
    { 0xa0d89825, 33 },
    { 0x2289824, 90 },
    { 0xd3789827, 23 },
    { 0xbe089828, 95 },
    { 0xe7889828, 95 },
    { 0, 0 },
    { 0x8a38982b, 84 },
    { 0x6b08982b, 33 },
    { 0x6b88982b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad89837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9118983b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a88983f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4589843, 61 },
    { 0x38389844, 43 },
    { 0x65889845, 78 },
    { 0xac289845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9789849, 66 },
    { 0xba68984a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448984f, 46 },
    { 0xa2189850, 65 },
    { 0xb5689851, 78 },
    { 0x63a8984f, 90 },
    { 0x83689853, 78 },
    { 0x63889853, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6689859, 98 },
    { 0, 0 },
    { 0xbfe8985b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0889862, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c89865, 90 },
    { 0x3a089866, 98 },
    { 0xf7789867, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb528986b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8986f, 68 },
    { 0xc7789870, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c88987b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d389880, 98 },
    { 0x5989881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3668988d, 43 },
    { 0xcb58988d, 34 },
    { 0, 0 },
    { 0x46e89890, 85 },
    { 0xc3689891, 33 },
    { 0xf8489891, 56 },
    { 0, 0 },
    { 0xd2089894, 90 },
    { 0xe4889894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488989b, 34 },
    { 0xb8c8989c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d898a0, 100 },
    { 0, 0 },
    { 0x322898a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710898a7, 91 },
    { 0x40f898a8, 78 },
    { 0xe77898a9, 95 },
    { 0x5b8898aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c898ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae898b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5898b9, 43 },
    { 0x4c9898ba, 98 },
    { 0x7b7898ba, 23 },
    { 0, 0 },
    { 0xe2d898bd, 98 },
    { 0xdf4898be, 98 },
    { 0x7ed898be, 88 },
    { 0x5f6898c0, 85 },
    { 0x53898c1, 61 },
    { 0x48b898be, 76 },
    { 0x98b898c3, 90 },
    { 0x5cd898c4, 55 },
    { 0x81e898c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977898ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab898d3, 98 },
    { 0x67e898d4, 95 },
    { 0x7ac898d5, 91 },
    { 0xb0c898d6, 34 },
    { 0, 0 },
    { 0xf6898d8, 93 },
    { 0x90a898d9, 95 },
    { 0, 0 },
    { 0x87f898db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b898e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c898e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8898ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996898ee, 66 },
    { 0x2d898ef, 95 },
    { 0x718898f0, 93 },
    { 0, 0 },
    { 0x70a898f2, 46 },
    { 0x141898f3, 65 },
    { 0xd81898f4, 96 },
    { 0x2e3898f4, 93 },
    { 0x23898f2, 24 },
    { 0xef6898f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c898fb, 90 },
    { 0x23c898fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8089904, 80 },
    { 0, 0 },
    { 0x2f589906, 90 },
    { 0, 0 },
    { 0x80a89908, 90 },
    { 0xaa389908, 98 },
    { 0, 0 },
    { 0xc998990b, 23 },
    { 0xe508990c, 23 },
    { 0, 0 },
    { 0x3958990e, 61 },
    { 0xaf88990e, 33 },
    { 0xea8990f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd89915, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1389919, 23 },
    { 0xca48991a, 99 },
    { 0x7e58991b, 17 },
    { 0x5cb8991c, 95 },
    { 0x6118991d, 98 },
    { 0x3cd8991d, 61 },
    { 0x5468991a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a89922, 78 },
    { 0xff989922, 23 },
    { 0x97c89924, 56 },
    { 0xd5389925, 43 },
    { 0x6a389926, 95 },
    { 0x57589926, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8992a, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48992e, 98 },
    { 0xd798992f, 55 },
    { 0x6c989930, 61 },
    { 0x13f8992f, 100 },
    { 0, 0 },
    { 0xfd89933, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7189936, 23 },
    { 0xd6789937, 33 },
    { 0xc3b89938, 33 },
    { 0, 0 },
    { 0xd6c8993a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8993d, 61 },
    { 0xad98993d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938994b, 66 },
    { 0xa788994c, 68 },
    { 0xdac8994d, 90 },
    { 0, 0 },
    { 0x5418994f, 36 },
    { 0x3d58994f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6689955, 23 },
    { 0x1a289955, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd8995a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46689964, 98 },
    { 0x33a89964, 61 },
    { 0, 0 },
    { 0x32f89967, 92 },
    { 0, 0 },
    { 0xa0089969, 40 },
    { 0, 0 },
    { 0x67e8996b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f489971, 23 },
    { 0xb9889972, 47 },
    { 0x93989972, 23 },
    { 0x7df89974, 90 },
    { 0x3e989975, 23 },
    { 0xb4b89975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4868997d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d89981, 38 },
    { 0xb2d89982, 93 },
    { 0xa0289982, 23 },
    { 0xf4689981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d089988, 90 },
    { 0x94789988, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf888998c, 99 },
    { 0x6af8998d, 98 },
    { 0x4ee8998e, 55 },
    { 0x8358998e, 34 },
    { 0, 0 },
    { 0x9b989991, 33 },
    { 0x27f89991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a89996, 93 },
    { 0x29d89997, 98 },
    { 0x7b689998, 61 },
    { 0x9e489999, 61 },
    { 0xb1589997, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f899a1, 33 },
    { 0, 0 },
    { 0x94c899a3, 78 },
    { 0x3ef899a4, 23 },
    { 0xa5c899a3, 34 },
    { 0xe28899a6, 78 },
    { 0xd92899a7, 78 },
    { 0xc6a899a8, 61 },
    { 0x150899a9, 90 },
    { 0x237899aa, 92 },
    { 0x75b899ab, 55 },
    { 0x246899a9, 33 },
    { 0x9ec899a5, 65 },
    { 0xe7f899aa, 95 },
    { 0, 0 },
    { 0xd14899b0, 99 },
    { 0x6dc899b1, 98 },
    { 0x1e5899b2, 93 },
    { 0x307899b3, 58 },
    { 0x8c4899b3, 85 },
    { 0, 0 },
    { 0x4b1899b6, 23 },
    { 0x5f4899b7, 76 },
    { 0x14b899b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f899bc, 98 },
    { 0x3ab899bd, 66 },
    { 0, 0 },
    { 0xc6c899bf, 23 },
    { 0x7c0899c0, 78 },
    { 0xa0e899c1, 33 },
    { 0x36c899c0, 69 },
    { 0x43e899c3, 23 },
    { 0, 0 },
    { 0x150899c5, 95 },
    { 0, 0 },
    { 0x7de899c7, 23 },
    { 0x298899c7, 33 },
    { 0x7b0899c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d899cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc899d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47899dc, 46 },
    { 0x1b6899dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d899e0, 66 },
    { 0x873899e0, 95 },
    { 0x704899e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9899e8, 33 },
    { 0x6b5899e9, 33 },
    { 0xee6899ea, 33 },
    { 0, 0 },
    { 0x96899ec, 95 },
    { 0xdcf899ed, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1899f0, 95 },
    { 0x606899f1, 61 },
    { 0xa28899f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f899f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf899fa, 55 },
    { 0x55e899fb, 61 },
    { 0xa05899fc, 76 },
    { 0xaea899fd, 55 },
    { 0x702899fe, 91 },
    { 0xf65899ff, 46 },
    { 0x77a89a00, 90 },
    { 0x18899fd, 75 },
    { 0xfb5899fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27389a06, 90 },
    { 0x16489a06, 49 },
    { 0xc2789a08, 76 },
    { 0x6ca89a08, 38 },
    { 0x67189a0a, 53 },
    { 0xbcd89a0a, 90 },
    { 0xe4389a0c, 61 },
    { 0, 0 },
    { 0x39f89a0e, 61 },
    { 0x6f389a0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c989a12, 23 },
    { 0x21f89a13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b89a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb89a1d, 46 },
    { 0, 0 },
    { 0xef689a1f, 84 },
    { 0x76389a20, 95 },
    { 0, 0 },
    { 0xc9589a22, 61 },
    { 0, 0 },
    { 0x1c489a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a89a2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4889a32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17789a35, 78 },
    { 0x49089a35, 90 },
    { 0xba089a37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d489a3b, 33 },
    { 0x29989a3c, 35 },
    { 0xd3389a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d389a40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3089a43, 51 },
    { 0x69c89a43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e89a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a389a4e, 52 },
    { 0x91e89a4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c289a53, 23 },
    { 0x6fa89a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff489a59, 95 },
    { 0xd2c89a59, 90 },
    { 0x40789a5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc89a60, 95 },
    { 0xec689a61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e89a66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f89a6c, 97 },
    { 0, 0 },
    { 0x61d89a6e, 78 },
    { 0xfa989a6e, 78 },
    { 0, 0 },
    { 0xc189a71, 23 },
    { 0x6c489a72, 91 },
    { 0x91f89a72, 61 },
    { 0x8e89a74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32289a7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a89a7e, 91 },
    { 0x15d89a7e, 95 },
    { 0x3d489a7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64789a87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d89a8a, 90 },
    { 0, 0 },
    { 0x83489a8c, 33 },
    { 0xeaf89a8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d389a90, 93 },
    { 0x9ee89a91, 93 },
    { 0, 0 },
    { 0x74d89a93, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x40089a96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33489a9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d89a9f, 98 },
    { 0xbd889aa0, 95 },
    { 0x8ef89aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52089aa6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d89aae, 92 },
    { 0xc7d89aaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd89ab2, 61 },
    { 0x30389ab2, 90 },
    { 0x55e89ab2, 23 },
    { 0, 0 },
    { 0x36189ab6, 46 },
    { 0, 0 },
    { 0xc8b89ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b089ac3, 33 },
    { 0xb6c89ac4, 90 },
    { 0, 0 },
    { 0x43489ac6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c89aca, 23 },
    { 0, 0 },
    { 0xa3b89acc, 90 },
    { 0x91889acd, 95 },
    { 0, 0 },
    { 0xbfb89acf, 33 },
    { 0x76489ad0, 33 },
    { 0x91089ad0, 90 },
    { 0x46e89ad2, 85 },
    { 0xfea89ad0, 33 },
    { 0x99289ad3, 90 },
    { 0x47789ad1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce689ad9, 84 },
    { 0, 0 },
    { 0x43e89adb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e289ade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67189ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e489ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc89aed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec689af4, 91 },
    { 0x70889af5, 100 },
    { 0xef289af5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2289afa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a89afe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d589b04, 95 },
    { 0, 0 },
    { 0xe6089b06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef289b09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7589b0d, 23 },
    { 0x28089b0d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6289b13, 46 },
    { 0xab489b14, 90 },
    { 0, 0 },
    { 0x5189b16, 90 },
    { 0x2d689b17, 88 },
    { 0x8c589b16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57389b1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d889b25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae89b29, 98 },
    { 0x9a689b2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e589b2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81489b32, 61 },
    { 0xd2c89b32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b89b36, 98 },
    { 0xa9889b36, 87 },
    { 0xe5189b37, 99 },
    { 0, 0 },
    { 0xbe089b3a, 33 },
    { 0x13089b3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8589b4d, 98 },
    { 0x10c89b4d, 43 },
    { 0xd6a89b4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f89b54, 95 },
    { 0xff789b55, 98 },
    { 0x9489b56, 33 },
    { 0xf0689b56, 90 },
    { 0xf2b89b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd89b5d, 17 },
    { 0x66789b5e, 61 },
    { 0x2e489b5e, 90 },
    { 0xfbc89b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce89b66, 75 },
    { 0x26089b67, 69 },
    { 0xd8a89b68, 61 },
    { 0xc8189b69, 43 },
    { 0, 0 },
    { 0x18089b6b, 90 },
    { 0, 0 },
    { 0x3c289b6d, 34 },
    { 0xb4789b6e, 90 },
    { 0x2ea89b6e, 90 },
    { 0xc7789b70, 61 },
    { 0x57e89b71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6689b75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a689b81, 66 },
    { 0x9c289b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc89b8c, 61 },
    { 0x3bd89b8d, 95 },
    { 0, 0 },
    { 0x78689b8f, 55 },
    { 0, 0 },
    { 0x2b889b91, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889b99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c89b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96389ba2, 34 },
    { 0xfe789ba2, 33 },
    { 0xe2389ba4, 93 },
    { 0xb2989ba5, 95 },
    { 0x25989ba6, 33 },
    { 0x5eb89ba4, 34 },
    { 0x62589ba7, 61 },
    { 0x6ac89ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf89baf, 97 },
    { 0xbb589bb0, 85 },
    { 0x33089bb0, 90 },
    { 0xd5289baf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d89bb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e89bb9, 55 },
    { 0x38e89bb9, 90 },
    { 0, 0 },
    { 0x36b89bbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b89bbf, 93 },
    { 0, 0 },
    { 0x56189bc1, 76 },
    { 0xb2989bc2, 98 },
    { 0, 0 },
    { 0x9f489bc4, 68 },
    { 0xb3e89bc4, 23 },
    { 0, 0 },
    { 0xbf689bc7, 76 },
    { 0, 0 },
    { 0x5f289bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51989bcc, 91 },
    { 0x82b89bcc, 84 },
    { 0x26589bcc, 87 },
    { 0x7b89bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa89bd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f189bd7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e89bdd, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea89be2, 33 },
    { 0xeca89be3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6889bec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7089bf3, 23 },
    { 0, 0 },
    { 0x18189bf5, 23 },
    { 0x5df89bf6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d89bfa, 98 },
    { 0, 0 },
    { 0x96589bfc, 33 },
    { 0xbb389bfd, 23 },
    { 0x4a789bfc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7089c03, 23 },
    { 0xcbe89c04, 98 },
    { 0x54f89c03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a89c09, 23 },
    { 0x53789c0a, 23 },
    { 0x39089c0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c89c0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89789c11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4989c16, 99 },
    { 0, 0 },
    { 0xfcb89c18, 90 },
    { 0, 0 },
    { 0x1c89c1a, 17 },
    { 0, 0 },
    { 0xe0c89c1c, 34 },
    { 0x9b389c1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a889c20, 87 },
    { 0x2b589c21, 85 },
    { 0xc9a89c22, 95 },
    { 0, 0 },
    { 0x31b89c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a789c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f89c33, 33 },
    { 0xb2889c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb89c3b, 90 },
    { 0xdf889c3c, 98 },
    { 0x14089c3d, 23 },
    { 0x26c89c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a989c41, 23 },
    { 0xc9389c42, 95 },
    { 0xb0189c42, 95 },
    { 0, 0 },
    { 0x5d689c45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e89c51, 90 },
    { 0xcce89c52, 91 },
    { 0xee289c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb189c57, 61 },
    { 0, 0 },
    { 0x1bd89c59, 95 },
    { 0x8fa89c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab89c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b789c6d, 48 },
    { 0x74189c6d, 55 },
    { 0, 0 },
    { 0x82189c70, 90 },
    { 0xe8689c70, 99 },
    { 0, 0 },
    { 0x8489c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23989c76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb689c7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed89c7f, 98 },
    { 0xe1189c80, 65 },
    { 0xad489c81, 38 },
    { 0, 0 },
    { 0xc7489c83, 66 },
    { 0, 0 },
    { 0x82e89c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6189c8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b89c94, 78 },
    { 0x96489c95, 80 },
    { 0x33189c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3189c9b, 90 },
    { 0, 0 },
    { 0x2c989c9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25689ca0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48789ca5, 91 },
    { 0x9689ca6, 61 },
    { 0x79a89ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57889cab, 77 },
    { 0, 0 },
    { 0xbe289cad, 90 },
    { 0xd9989cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf89cb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae489cb5, 76 },
    { 0x1f989cb6, 34 },
    { 0xd089cb6, 23 },
    { 0, 0 },
    { 0x7689cb9, 61 },
    { 0x82389cba, 72 },
    { 0, 0 },
    { 0x76989cbc, 80 },
    { 0xd3a89cbd, 95 },
    { 0x42d89cbe, 43 },
    { 0xeb589cbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d89cc3, 78 },
    { 0, 0 },
    { 0x35789cc5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989cc8, 90 },
    { 0, 0 },
    { 0x15f89cca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9089cd9, 23 },
    { 0x1889cd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a389ce0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3789ceb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d89cef, 74 },
    { 0xe5c89cef, 90 },
    { 0x8e989cef, 55 },
    { 0x30689cf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba89cf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9489d03, 95 },
    { 0x3e689d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc489d11, 85 },
    { 0, 0 },
    { 0x8f489d13, 33 },
    { 0x38289d14, 33 },
    { 0x6c689d15, 61 },
    { 0x51189d16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x84589d19, 33 },
    { 0x5c89d1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73589d1e, 61 },
    { 0x86d89d1f, 78 },
    { 0xc6589d20, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0589d23, 40 },
    { 0xe9c89d24, 80 },
    { 0x70f89d23, 46 },
    { 0x48589d24, 33 },
    { 0x58889d23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda89d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c489d34, 75 },
    { 0, 0 },
    { 0xf1089d36, 76 },
    { 0, 0 },
    { 0x47889d38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e89d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc389d48, 90 },
    { 0xda089d49, 99 },
    { 0, 0 },
    { 0x4ce89d4b, 61 },
    { 0, 0 },
    { 0xdff89d4d, 33 },
    { 0xd6e89d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e289d52, 69 },
    { 0x7e389d53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34689d57, 33 },
    { 0, 0 },
    { 0x16c89d59, 77 },
    { 0x4ae89d5a, 61 },
    { 0, 0 },
    { 0x18289d5c, 95 },
    { 0x15189d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9989d64, 23 },
    { 0xd5589d65, 55 },
    { 0xe3089d66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc89d6b, 76 },
    { 0x50489d6b, 95 },
    { 0x22d89d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea89d70, 95 },
    { 0, 0 },
    { 0x46b89d72, 33 },
    { 0x2c689d73, 33 },
    { 0x3689d73, 95 },
    { 0x30189d75, 61 },
    { 0x71e89d76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1189d7a, 23 },
    { 0, 0 },
    { 0x25a89d7c, 46 },
    { 0xb2c89d7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46489d89, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b89d8d, 23 },
    { 0x21d89d8e, 76 },
    { 0, 0 },
    { 0x55b89d90, 33 },
    { 0, 0 },
    { 0x89489d92, 34 },
    { 0x5c189d93, 76 },
    { 0xf2189d92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3389d9b, 46 },
    { 0, 0 },
    { 0xff589d9d, 90 },
    { 0xa0589d9e, 43 },
    { 0, 0 },
    { 0xcd089da0, 90 },
    { 0x16e89da0, 23 },
    { 0x68a89da2, 33 },
    { 0x1ab89da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88789da6, 76 },
    { 0x82889da6, 67 },
    { 0xdc889da8, 90 },
    { 0, 0 },
    { 0xa6d89daa, 23 },
    { 0, 0 },
    { 0xfa989dac, 90 },
    { 0xd9689dad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88089db4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de89db8, 100 },
    { 0xe0189db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd89dbd, 52 },
    { 0, 0 },
    { 0x8aa89dbf, 66 },
    { 0xa3089dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d589dc9, 33 },
    { 0, 0 },
    { 0x36689dcb, 34 },
    { 0, 0 },
    { 0xefa89dcd, 90 },
    { 0x1a089dcd, 95 },
    { 0x7d689dcf, 99 },
    { 0xe2789dcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc489dd5, 33 },
    { 0x19189dd5, 46 },
    { 0, 0 },
    { 0x68489dd8, 33 },
    { 0x70889dd8, 43 },
    { 0x5689dda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9289ddd, 33 },
    { 0x9c089dde, 33 },
    { 0x64f89ddf, 34 },
    { 0x9ce89de0, 34 },
    { 0xc7589de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f89de6, 93 },
    { 0x59489de6, 61 },
    { 0x69289de7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f89dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8889df2, 34 },
    { 0xba789df3, 91 },
    { 0xb1589df3, 23 },
    { 0x9289df5, 41 },
    { 0xf8e89df6, 90 },
    { 0x9c489df7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15389dfb, 40 },
    { 0xdf189dfc, 76 },
    { 0xb6b89dfd, 76 },
    { 0, 0 },
    { 0x6f589dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59689e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84389e07, 34 },
    { 0x49f89e08, 43 },
    { 0xd6589e09, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87889e0c, 96 },
    { 0x689e0d, 55 },
    { 0, 0 },
    { 0xcda89e0f, 93 },
    { 0x68989e10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f89e16, 93 },
    { 0x31489e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10489e1d, 33 },
    { 0, 0 },
    { 0x86089e1f, 76 },
    { 0xaf789e1f, 61 },
    { 0x69989e21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb89e24, 66 },
    { 0, 0 },
    { 0x90689e26, 76 },
    { 0x82d89e27, 33 },
    { 0x4b289e28, 78 },
    { 0x1d689e28, 84 },
    { 0x3e689e2a, 100 },
    { 0x8e689e29, 86 },
    { 0xd0c89e29, 61 },
    { 0x8ed89e2d, 61 },
    { 0x34689e2d, 78 },
    { 0xd8a89e2f, 90 },
    { 0xd9c89e2c, 33 },
    { 0x9fe89e31, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7389e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3489e3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b89e45, 55 },
    { 0, 0 },
    { 0xf9289e47, 93 },
    { 0, 0 },
    { 0x15d89e49, 84 },
    { 0xd689e49, 95 },
    { 0xbc489e4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e89e4e, 17 },
    { 0, 0 },
    { 0x79889e50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d889e55, 55 },
    { 0xc2589e56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb89e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c89e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789e61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e89e64, 33 },
    { 0, 0 },
    { 0xba589e66, 76 },
    { 0xb5289e67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc889e6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c889e70, 90 },
    { 0, 0 },
    { 0x35c89e72, 90 },
    { 0x46389e72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2189e76, 80 },
    { 0x6dc89e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22789e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91789e80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a89e85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68089e88, 34 },
    { 0, 0 },
    { 0xeb89e8a, 93 },
    { 0x4ed89e8b, 33 },
    { 0, 0 },
    { 0xc3289e8d, 99 },
    { 0xb6389e8e, 33 },
    { 0, 0 },
    { 0x7a89e90, 33 },
    { 0x67989e90, 91 },
    { 0, 0 },
    { 0x3ab89e93, 78 },
    { 0xaa89e94, 99 },
    { 0xaa489e95, 38 },
    { 0x1bd89e96, 93 },
    { 0xd5589e97, 83 },
    { 0x7ee89e94, 67 },
    { 0x49189e99, 61 },
    { 0x1dd89e99, 78 },
    { 0xdc689e9a, 34 },
    { 0, 0 },
    { 0x81f89e9d, 55 },
    { 0, 0 },
    { 0x78989e9f, 61 },
    { 0xf7b89e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92189ea3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1989ea6, 90 },
    { 0x33389ea7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a89eac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb89eb4, 98 },
    { 0, 0 },
    { 0xbe89eb6, 46 },
    { 0, 0 },
    { 0x3bb89eb8, 81 },
    { 0xb9989eb9, 33 },
    { 0xe689eb8, 95 },
    { 0xa3689eb8, 90 },
    { 0xcb389ebc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb89ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca689ec3, 95 },
    { 0x9cd89ec4, 43 },
    { 0x1c89ec4, 33 },
    { 0x7e989ec6, 99 },
    { 0, 0 },
    { 0x52889ec8, 98 },
    { 0x23189ec9, 55 },
    { 0xb0689eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99089ecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c189ed2, 90 },
    { 0, 0 },
    { 0xf0989ed4, 100 },
    { 0xe5c89ed5, 95 },
    { 0, 0 },
    { 0x4689ed7, 46 },
    { 0x76889ed7, 95 },
    { 0xb9489ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34889edc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac889ee1, 93 },
    { 0x3f889ee1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34789ee8, 93 },
    { 0xe1689ee9, 23 },
    { 0x31989eea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57389eef, 87 },
    { 0x87089ef0, 76 },
    { 0, 0 },
    { 0x65589ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f89ef5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea89efb, 33 },
    { 0x63789efb, 68 },
    { 0x19789efd, 61 },
    { 0xbd089efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc689f02, 33 },
    { 0x7cc89f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f89f0a, 33 },
    { 0xff989f0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74289f0f, 34 },
    { 0x48689f10, 55 },
    { 0xb5d89f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78889f16, 61 },
    { 0x16289f16, 33 },
    { 0x7c89f16, 98 },
    { 0x8ce89f19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1189f1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e89f26, 76 },
    { 0xb7c89f27, 33 },
    { 0xa5089f26, 67 },
    { 0x3ff89f26, 90 },
    { 0x1ea89f2a, 78 },
    { 0xba489f26, 34 },
    { 0xf7689f2c, 61 },
    { 0x79f89f28, 61 },
    { 0x47e89f2e, 90 },
    { 0x90589f2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e89f33, 61 },
    { 0x3b89f34, 38 },
    { 0xfa089f35, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe89f3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99789f47, 76 },
    { 0, 0 },
    { 0xbcd89f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d789f4e, 76 },
    { 0x64789f4f, 34 },
    { 0x8e689f50, 90 },
    { 0, 0 },
    { 0xece89f52, 91 },
    { 0xd2889f53, 98 },
    { 0x35989f54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa089f59, 43 },
    { 0x1a89f5a, 43 },
    { 0x76089f5b, 23 },
    { 0xcca89f59, 95 },
    { 0, 0 },
    { 0xe4089f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9489f63, 80 },
    { 0x33889f64, 76 },
    { 0, 0 },
    { 0x73a89f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb489f6b, 91 },
    { 0xbee89f6b, 33 },
    { 0xce489f6c, 98 },
    { 0xfb389f6d, 33 },
    { 0, 0 },
    { 0x86989f70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d89f78, 85 },
    { 0x60589f79, 90 },
    { 0x82e89f78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d89f80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3789f87, 90 },
    { 0, 0 },
    { 0x46a89f89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a889f8f, 61 },
    { 0x18f89f90, 85 },
    { 0xf889f91, 95 },
    { 0xa5789f91, 66 },
    { 0xb0489f93, 46 },
    { 0xb0c89f93, 99 },
    { 0xbf889f8f, 34 },
    { 0x889f96, 76 },
    { 0xd0989f97, 76 },
    { 0xf0289f97, 34 },
    { 0x1e489f8f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e89f9e, 91 },
    { 0x7ca89f9f, 98 },
    { 0x16e89fa0, 61 },
    { 0xd6889fa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a89fac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87589fb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b89fb4, 61 },
    { 0x55e89fb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f89fb8, 33 },
    { 0x3ba89fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6889fbc, 23 },
    { 0x33b89fbd, 91 },
    { 0xee889fbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac489fc1, 90 },
    { 0x1ae89fc1, 96 },
    { 0, 0 },
    { 0xa2389fc4, 84 },
    { 0x3a89fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea89fcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9489fd0, 95 },
    { 0x72b89fd0, 33 },
    { 0x56789fd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xea589fd5, 61 },
    { 0xfb089fd6, 90 },
    { 0, 0 },
    { 0x6e889fd8, 38 },
    { 0xffb89fd9, 90 },
    { 0xdc789fda, 43 },
    { 0xab989fdb, 33 },
    { 0x82c89fdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac389fdf, 98 },
    { 0, 0 },
    { 0x3f89fe1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26889fe8, 23 },
    { 0xd1889fe9, 81 },
    { 0x8e889fea, 52 },
    { 0xaa889fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c289fef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3989ff2, 90 },
    { 0x6f89ff2, 61 },
    { 0xf5b89ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c389ff8, 93 },
    { 0x19489ff8, 99 },
    { 0x9de89ffa, 98 },
    { 0x38889ffb, 90 },
    { 0x93e89ffc, 61 },
    { 0x24189ffb, 33 },
    { 0x5ee89ffd, 23 },
    { 0x1b489fff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a003, 90 },
    { 0xed98a003, 33 },
    { 0xf7d8a003, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348a00b, 90 },
    { 0x1548a00c, 33 },
    { 0x3678a00b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca8a010, 76 },
    { 0x198a010, 43 },
    { 0xa618a012, 94 },
    { 0x7248a012, 33 },
    { 0, 0 },
    { 0x3a48a015, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf438a020, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf398a027, 95 },
    { 0xd2e8a028, 38 },
    { 0x94a8a027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc748a02e, 55 },
    { 0, 0 },
    { 0x9b98a030, 98 },
    { 0x3938a031, 34 },
    { 0xd8a032, 95 },
    { 0x7e08a033, 90 },
    { 0x1788a032, 33 },
    { 0, 0 },
    { 0xf088a036, 90 },
    { 0x5bc8a037, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc018a03a, 55 },
    { 0, 0 },
    { 0x4c48a03c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98a040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9618a043, 55 },
    { 0, 0 },
    { 0xe9b8a045, 100 },
    { 0, 0 },
    { 0x218a047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1488a04d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb8a051, 99 },
    { 0x7e98a051, 43 },
    { 0, 0 },
    { 0x8de8a054, 95 },
    { 0, 0 },
    { 0xeb78a056, 46 },
    { 0xd518a056, 78 },
    { 0xb8e8a058, 88 },
    { 0xb298a059, 61 },
    { 0x7c08a058, 90 },
    { 0xfdf8a056, 90 },
    { 0x2c58a05c, 43 },
    { 0x97f8a05d, 98 },
    { 0, 0 },
    { 0x3418a05f, 34 },
    { 0, 0 },
    { 0x6e88a061, 98 },
    { 0xca8a062, 33 },
    { 0x4428a063, 61 },
    { 0, 0 },
    { 0x1f18a065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648a06b, 98 },
    { 0xdde8a06c, 90 },
    { 0, 0 },
    { 0xd9a8a06e, 46 },
    { 0xefe8a06f, 99 },
    { 0xdf78a06f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868a074, 98 },
    { 0x7538a075, 23 },
    { 0xd098a075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da8a07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f8a07d, 61 },
    { 0xbb08a07e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb698a089, 68 },
    { 0x948a089, 61 },
    { 0x4608a08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2918a08e, 90 },
    { 0, 0 },
    { 0xd538a090, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d8a097, 61 },
    { 0, 0 },
    { 0x4328a099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a8a0a1, 93 },
    { 0, 0 },
    { 0x8268a0a3, 55 },
    { 0x9b8a0a3, 67 },
    { 0, 0 },
    { 0xdbf8a0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08a0aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c18a0ae, 34 },
    { 0xe2d8a0af, 87 },
    { 0xf4e8a0b0, 33 },
    { 0, 0 },
    { 0x56a8a0b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9218a0b5, 98 },
    { 0x2c88a0b6, 90 },
    { 0, 0 },
    { 0x5178a0b8, 33 },
    { 0x4d08a0b9, 23 },
    { 0x5118a0b9, 46 },
    { 0x2228a0bb, 76 },
    { 0xcc38a0bc, 33 },
    { 0xeb98a0b8, 34 },
    { 0x9ed8a0bc, 90 },
    { 0, 0 },
    { 0x8868a0c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d8a0c6, 43 },
    { 0, 0 },
    { 0xc4b8a0c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338a0cc, 80 },
    { 0xeb68a0cd, 46 },
    { 0xfc8a0ce, 76 },
    { 0x78b8a0cc, 76 },
    { 0xc478a0cd, 90 },
    { 0, 0 },
    { 0x4098a0d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a58a0d5, 33 },
    { 0xc788a0d6, 52 },
    { 0, 0 },
    { 0xb5d8a0d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8a0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4248a0e9, 95 },
    { 0, 0 },
    { 0xf058a0eb, 98 },
    { 0x2988a0eb, 81 },
    { 0x40f8a0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d28a0f0, 76 },
    { 0x5a18a0f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8a0f4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2338a100, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e8a103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48a106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d8a10b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8a10f, 98 },
    { 0, 0 },
    { 0xb0a8a111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f8a115, 55 },
    { 0x5898a116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638a120, 34 },
    { 0x3518a121, 90 },
    { 0xcd08a122, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf198a129, 46 },
    { 0, 0 },
    { 0x4b88a12b, 93 },
    { 0, 0 },
    { 0xaa48a12d, 23 },
    { 0xa888a12e, 91 },
    { 0x4908a12d, 33 },
    { 0xddb8a130, 55 },
    { 0x7fd8a131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa298a134, 81 },
    { 0x6338a135, 61 },
    { 0x348a136, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e98a13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe668a13d, 34 },
    { 0, 0 },
    { 0x3e78a13f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748a144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4448a148, 95 },
    { 0x5938a149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa038a14e, 23 },
    { 0x5708a14f, 65 },
    { 0x5f08a14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8a154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b08a15c, 91 },
    { 0x6298a15d, 90 },
    { 0, 0 },
    { 0xf178a15f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e48a165, 87 },
    { 0xb398a166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8a16c, 61 },
    { 0, 0 },
    { 0xd3e8a16e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928a178, 68 },
    { 0xde18a178, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc528a17d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c8a181, 76 },
    { 0xfa58a182, 91 },
    { 0xcb28a183, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428a186, 61 },
    { 0, 0 },
    { 0xc838a188, 88 },
    { 0xd308a189, 33 },
    { 0xeb8a18a, 61 },
    { 0x34f8a18b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc8a18f, 33 },
    { 0xdeb8a190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f8a19c, 90 },
    { 0x1008a19c, 76 },
    { 0x4868a19c, 76 },
    { 0x4658a19f, 66 },
    { 0x7d08a1a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c98a1a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a8a1a7, 90 },
    { 0xc0e8a1a8, 46 },
    { 0x3408a1a8, 83 },
    { 0, 0 },
    { 0x46b8a1ab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe588a1af, 46 },
    { 0, 0 },
    { 0x7798a1b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c8a1ba, 90 },
    { 0x1138a1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1998a1be, 90 },
    { 0xd18a1bf, 43 },
    { 0x2138a1c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7118a1c5, 87 },
    { 0xb0b8a1c6, 43 },
    { 0x6548a1c7, 23 },
    { 0xa768a1c5, 84 },
    { 0x1568a1c9, 40 },
    { 0x9278a1ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b8a1d0, 42 },
    { 0x2cf8a1d0, 61 },
    { 0xe298a1d2, 55 },
    { 0, 0 },
    { 0xfda8a1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b8a1d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8a1da, 33 },
    { 0x9978a1db, 87 },
    { 0x6218a1db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a08a1df, 84 },
    { 0x8b8a1e0, 66 },
    { 0, 0 },
    { 0xfbc8a1e2, 87 },
    { 0x6a8a1e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc98a1e6, 34 },
    { 0x8778a1e7, 76 },
    { 0x1df8a1e8, 98 },
    { 0xe428a1e8, 98 },
    { 0, 0 },
    { 0xb928a1eb, 40 },
    { 0x978a1eb, 23 },
    { 0xbd38a1eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9658a1f0, 17 },
    { 0, 0 },
    { 0x95f8a1f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e8a1f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc668a1fe, 95 },
    { 0, 0 },
    { 0x4188a200, 33 },
    { 0, 0 },
    { 0xc6d8a202, 61 },
    { 0xf8e8a203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba38a207, 34 },
    { 0x8dd8a208, 76 },
    { 0xb5d8a209, 33 },
    { 0xdbf8a208, 84 },
    { 0, 0 },
    { 0xec68a20c, 90 },
    { 0x5758a20d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6898a210, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee18a219, 33 },
    { 0xdf8a21a, 46 },
    { 0xc718a21a, 65 },
    { 0x6508a21c, 66 },
    { 0x92c8a21b, 66 },
    { 0x458a21e, 76 },
    { 0x79f8a21d, 61 },
    { 0xd518a21c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc638a223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5278a227, 98 },
    { 0, 0 },
    { 0xbb38a229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb438a22f, 93 },
    { 0x67b8a230, 61 },
    { 0x7c48a22f, 43 },
    { 0x3628a230, 40 },
    { 0x5b98a232, 33 },
    { 0, 0 },
    { 0x5848a235, 43 },
    { 0x85e8a236, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48a239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a8a23d, 76 },
    { 0x3418a23d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18a246, 98 },
    { 0xb458a246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4678a24b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8a24e, 95 },
    { 0xa5c8a24e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38a252, 83 },
    { 0x2088a253, 93 },
    { 0x7f18a254, 69 },
    { 0x5098a255, 87 },
    { 0x66b8a256, 68 },
    { 0x5928a257, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c58a25c, 46 },
    { 0x3c18a25d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18a261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb8a264, 90 },
    { 0xc8c8a265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398a26e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8a271, 68 },
    { 0xb778a271, 90 },
    { 0x5528a273, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8a27b, 23 },
    { 0xc648a27b, 85 },
    { 0xd268a27d, 85 },
    { 0xa708a27d, 46 },
    { 0xf188a27f, 95 },
    { 0x57d8a27f, 90 },
    { 0x2f08a281, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4608a287, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38a28a, 90 },
    { 0x1168a28b, 23 },
    { 0x8a18a28a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018a294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa58a29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd8a2a2, 91 },
    { 0, 0 },
    { 0x5458a2a4, 95 },
    { 0xe788a2a5, 66 },
    { 0x7d08a2a6, 87 },
    { 0, 0 },
    { 0x9018a2a8, 76 },
    { 0xd5a8a2a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a8a2ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d8a2b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a8a2b5, 61 },
    { 0x5488a2b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8a2bb, 33 },
    { 0xea98a2bb, 23 },
    { 0xfa98a2bd, 33 },
    { 0xa2c8a2be, 23 },
    { 0x2238a2bf, 95 },
    { 0, 0 },
    { 0x38f8a2c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4088a2c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d8a2c7, 23 },
    { 0, 0 },
    { 0x99d8a2c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf228a2cf, 91 },
    { 0x21b8a2d0, 65 },
    { 0x2868a2d0, 95 },
    { 0xa48a2d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda78a2d8, 99 },
    { 0xbb08a2d9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5628a2e0, 87 },
    { 0x5568a2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea68a2e7, 61 },
    { 0x18e8a2e8, 61 },
    { 0x878a2e7, 23 },
    { 0, 0 },
    { 0x5ce8a2eb, 90 },
    { 0xeb18a2ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d48a2f2, 95 },
    { 0x8ef8a2f3, 95 },
    { 0xee88a2f3, 76 },
    { 0x77f8a2f5, 33 },
    { 0xaa78a2f4, 55 },
    { 0, 0 },
    { 0xf588a2f8, 99 },
    { 0xd048a2f8, 95 },
    { 0x5228a2f9, 33 },
    { 0x90f8a2fb, 33 },
    { 0x3c08a2fa, 98 },
    { 0x79f8a2f9, 100 },
    { 0x3228a2fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed8a305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2868a309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa988a30f, 95 },
    { 0x7e28a310, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a88a314, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8128a31a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8a321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e8a32a, 34 },
    { 0x7f08a32a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38a32e, 95 },
    { 0x2b38a32f, 76 },
    { 0x71f8a32f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf188a334, 61 },
    { 0xecf8a334, 83 },
    { 0x4f98a336, 33 },
    { 0xee88a335, 43 },
    { 0xcdb8a338, 93 },
    { 0x2f08a339, 23 },
    { 0x6538a334, 90 },
    { 0xade8a33b, 33 },
    { 0xa848a33c, 98 },
    { 0x45a8a338, 23 },
    { 0x95e8a339, 67 },
    { 0x1818a33f, 66 },
    { 0x2508a340, 52 },
    { 0x9ff8a341, 38 },
    { 0x2b58a341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b8a348, 83 },
    { 0, 0 },
    { 0x1e58a34a, 93 },
    { 0x4b98a34a, 25 },
    { 0xadc8a34c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c88a355, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8a360, 33 },
    { 0x6f68a361, 98 },
    { 0x5638a362, 61 },
    { 0x65e8a363, 33 },
    { 0xd848a363, 90 },
    { 0, 0 },
    { 0x3b48a366, 33 },
    { 0, 0 },
    { 0x2768a368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8a36d, 98 },
    { 0xc318a36e, 91 },
    { 0, 0 },
    { 0x64f8a370, 98 },
    { 0, 0 },
    { 0xe968a372, 90 },
    { 0xb0b8a372, 98 },
    { 0x81f8a372, 87 },
    { 0, 0 },
    { 0x8408a376, 61 },
    { 0x7868a376, 90 },
    { 0, 0 },
    { 0x4608a379, 90 },
    { 0, 0 },
    { 0xd6a8a37b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198a37f, 85 },
    { 0, 0 },
    { 0xc5f8a381, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f8a386, 34 },
    { 0x8198a387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6788a38d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6038a390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528a393, 90 },
    { 0xa908a393, 90 },
    { 0, 0 },
    { 0x1d68a396, 99 },
    { 0, 0 },
    { 0x7168a398, 76 },
    { 0, 0 },
    { 0x4a18a39a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2098a39e, 95 },
    { 0, 0 },
    { 0xadb8a3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a8a3a7, 33 },
    { 0x3a28a3a7, 43 },
    { 0x8808a3a8, 65 },
    { 0x9f68a3aa, 99 },
    { 0xd668a3aa, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd38a3af, 23 },
    { 0, 0 },
    { 0x19b8a3b1, 99 },
    { 0x2538a3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc338a3b6, 67 },
    { 0x5cd8a3b6, 95 },
    { 0xc878a3b6, 97 },
    { 0x6e98a3b7, 95 },
    { 0x3f18a3ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808a3c4, 90 },
    { 0, 0 },
    { 0xcb78a3c6, 95 },
    { 0x7c98a3c7, 23 },
    { 0, 0 },
    { 0x88d8a3c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc8a3cf, 17 },
    { 0x2858a3d0, 33 },
    { 0xd9a8a3d0, 46 },
    { 0, 0 },
    { 0xacf8a3d3, 66 },
    { 0, 0 },
    { 0x6d18a3d5, 77 },
    { 0, 0 },
    { 0x2378a3d7, 36 },
    { 0x5de8a3d8, 78 },
    { 0x1128a3d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a8a3dc, 66 },
    { 0xdc88a3dd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b8a3e2, 90 },
    { 0x51a8a3e2, 34 },
    { 0xaf88a3e4, 96 },
    { 0x3698a3e3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe8a3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c28a3ef, 90 },
    { 0x2e88a3ef, 90 },
    { 0xc0f8a3f1, 61 },
    { 0, 0 },
    { 0xb638a3f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba8a3fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c8a3fe, 34 },
    { 0x5e68a3ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878a403, 61 },
    { 0, 0 },
    { 0xc388a405, 23 },
    { 0, 0 },
    { 0x6b58a407, 67 },
    { 0x56d8a408, 61 },
    { 0xf0a8a409, 61 },
    { 0, 0 },
    { 0x3638a40b, 33 },
    { 0xd708a40c, 33 },
    { 0, 0 },
    { 0x56f8a40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8a411, 23 },
    { 0x958a412, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e98a415, 90 },
    { 0, 0 },
    { 0xee58a417, 61 },
    { 0x7d68a418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8a41b, 61 },
    { 0x6848a41c, 78 },
    { 0, 0 },
    { 0xf4d8a41e, 98 },
    { 0x1128a41f, 17 },
    { 0, 0 },
    { 0x708a421, 69 },
    { 0, 0 },
    { 0x4d98a423, 55 },
    { 0, 0 },
    { 0xdd58a425, 23 },
    { 0, 0 },
    { 0x49c8a427, 84 },
    { 0x5b78a427, 90 },
    { 0x4fa8a429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b8a42c, 33 },
    { 0x1dd8a42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee8a432, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78a436, 98 },
    { 0xb088a436, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb38a43c, 34 },
    { 0, 0 },
    { 0xd028a43e, 23 },
    { 0xa8f8a43f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd948a442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e38a446, 84 },
    { 0, 0 },
    { 0x83b8a448, 93 },
    { 0x29c8a449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8308a44d, 66 },
    { 0x69a8a44e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368a453, 95 },
    { 0x8f38a453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8a45a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6068a45e, 34 },
    { 0x9c18a45f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf58a463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078a46b, 95 },
    { 0, 0 },
    { 0x7858a46d, 78 },
    { 0, 0 },
    { 0xef88a46f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798a473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c8a47c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a8a47f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5798a489, 33 },
    { 0x8c38a48a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908a48f, 90 },
    { 0x6de8a48f, 90 },
    { 0x4538a491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368a494, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478a497, 76 },
    { 0xf948a498, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8a49c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468a4a1, 23 },
    { 0x3b78a4a2, 55 },
    { 0xe68a4a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8a4a7, 95 },
    { 0x2618a4a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5868a4b3, 66 },
    { 0x9a88a4b4, 75 },
    { 0, 0 },
    { 0xff18a4b6, 87 },
    { 0x1ba8a4b7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38a4ba, 61 },
    { 0x3fb8a4ba, 33 },
    { 0, 0 },
    { 0x8d28a4bd, 61 },
    { 0xc758a4be, 23 },
    { 0x6ae8a4bf, 95 },
    { 0xe7c8a4c0, 66 },
    { 0x5e8a4bd, 95 },
    { 0x3d8a4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8a4c5, 23 },
    { 0x7958a4c6, 23 },
    { 0x3818a4c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88a4ca, 33 },
    { 0x43c8a4cb, 93 },
    { 0x7508a4cc, 93 },
    { 0x6978a4ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded8a4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8608a4d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8a4db, 33 },
    { 0x6ee8a4dc, 66 },
    { 0x4308a4dd, 90 },
    { 0xd8b8a4dd, 50 },
    { 0x30f8a4df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd8a4e2, 98 },
    { 0x95f8a4e3, 33 },
    { 0xb248a4e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8a4ed, 85 },
    { 0xe188a4ee, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8a4f5, 33 },
    { 0x6328a4f6, 100 },
    { 0xf738a4f7, 33 },
    { 0xa368a4f8, 98 },
    { 0xf9c8a4f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8a504, 90 },
    { 0, 0 },
    { 0x1888a506, 95 },
    { 0x7ef8a507, 95 },
    { 0x6ab8a508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28a50b, 34 },
    { 0, 0 },
    { 0x3578a50d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc698a511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d8a515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b8a518, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948a521, 33 },
    { 0xd398a521, 51 },
    { 0xe2b8a521, 61 },
    { 0xcae8a524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4638a527, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f8a530, 95 },
    { 0x9798a530, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b58a538, 90 },
    { 0, 0 },
    { 0x60c8a53a, 78 },
    { 0x7258a53b, 2 },
    { 0xa588a53c, 41 },
    { 0, 0 },
    { 0x23f8a53e, 34 },
    { 0, 0 },
    { 0x76c8a540, 76 },
    { 0xdca8a540, 55 },
    { 0x98a8a542, 95 },
    { 0xb568a543, 90 },
    { 0, 0 },
    { 0x1128a545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f8a54a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8a550, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018a556, 17 },
    { 0x3be8a556, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88a55f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8a564, 90 },
    { 0, 0 },
    { 0x8058a566, 61 },
    { 0x8bf8a567, 68 },
    { 0xe368a567, 90 },
    { 0, 0 },
    { 0x6c38a56a, 87 },
    { 0xa248a56b, 75 },
    { 0x5638a56a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8a572, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8a575, 81 },
    { 0x1af8a576, 90 },
    { 0x5448a576, 90 },
    { 0x5ad8a576, 33 },
    { 0, 0 },
    { 0x2268a57a, 98 },
    { 0xa9c8a57b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf38a57f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8098a583, 55 },
    { 0, 0 },
    { 0xddb8a585, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2668a58b, 55 },
    { 0, 0 },
    { 0xbd98a58d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8a590, 96 },
    { 0x1f28a591, 91 },
    { 0x9438a592, 33 },
    { 0x8148a593, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8a596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb8a599, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388a59e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9378a5a6, 75 },
    { 0xd238a5a7, 90 },
    { 0, 0 },
    { 0x5c48a5a9, 78 },
    { 0xe668a5a9, 61 },
    { 0x32e8a5a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1288a5af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5428a5b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba68a5b8, 78 },
    { 0xf748a5b9, 98 },
    { 0x5808a5b8, 93 },
    { 0x7628a5b8, 98 },
    { 0x5f78a5bc, 34 },
    { 0xb7c8a5bc, 55 },
    { 0, 0 },
    { 0x828a5bf, 90 },
    { 0x9448a5c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc8a5c3, 99 },
    { 0x4308a5c3, 65 },
    { 0, 0 },
    { 0x54d8a5c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd308a5c9, 33 },
    { 0x1d58a5ca, 100 },
    { 0, 0 },
    { 0xf748a5cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7488a5d0, 33 },
    { 0x4138a5d1, 33 },
    { 0xb188a5d2, 77 },
    { 0xe298a5d3, 61 },
    { 0x678a5d2, 76 },
    { 0x4e68a5d5, 84 },
    { 0x4268a5d4, 23 },
    { 0xe558a5d7, 33 },
    { 0, 0 },
    { 0x7078a5d9, 23 },
    { 0, 0 },
    { 0xa628a5db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4208a5de, 49 },
    { 0, 0 },
    { 0xfc8a5e0, 98 },
    { 0x8278a5e1, 76 },
    { 0x1c98a5e0, 34 },
    { 0xd8a8a5e3, 55 },
    { 0, 0 },
    { 0xcc8a5e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe798a5ed, 46 },
    { 0x5a08a5ed, 78 },
    { 0x46e8a5ef, 34 },
    { 0xfd08a5f0, 90 },
    { 0x41b8a5f0, 96 },
    { 0x7ee8a5ed, 98 },
    { 0, 0 },
    { 0x3b58a5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8a5f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf8a5fb, 99 },
    { 0x8528a5fc, 95 },
    { 0x1c28a5fc, 98 },
    { 0x6938a5fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6118a601, 99 },
    { 0x7138a602, 61 },
    { 0x8d58a603, 52 },
    { 0xad08a603, 61 },
    { 0xb428a605, 23 },
    { 0x5cb8a606, 33 },
    { 0xe5e8a604, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58a60a, 98 },
    { 0, 0 },
    { 0xb0a8a60c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4988a60f, 46 },
    { 0xf1d8a60f, 23 },
    { 0xa68a60f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f48a615, 95 },
    { 0x7838a616, 55 },
    { 0, 0 },
    { 0x4678a618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef18a61f, 17 },
    { 0x9e38a61f, 61 },
    { 0, 0 },
    { 0xd588a622, 33 },
    { 0, 0 },
    { 0x4af8a624, 61 },
    { 0, 0 },
    { 0xc458a626, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1138a629, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce68a62e, 46 },
    { 0, 0 },
    { 0x2b18a630, 98 },
    { 0, 0 },
    { 0x61d8a632, 23 },
    { 0x8658a632, 33 },
    { 0xd5e8a634, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b78a63b, 33 },
    { 0x3818a63c, 61 },
    { 0x5ea8a63d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e48a640, 23 },
    { 0x4d8a641, 99 },
    { 0xe878a642, 91 },
    { 0x3988a643, 55 },
    { 0xcbd8a644, 63 },
    { 0, 0 },
    { 0xa938a646, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd8a64c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a657, 87 },
    { 0, 0 },
    { 0x90d8a659, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd8a65c, 78 },
    { 0, 0 },
    { 0x6958a65e, 46 },
    { 0, 0 },
    { 0x618a660, 76 },
    { 0x688a661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e8a665, 46 },
    { 0x1a48a666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec8a669, 55 },
    { 0x3088a66a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8a672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7228a677, 76 },
    { 0xe618a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd578a67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8a67e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f8a681, 93 },
    { 0, 0 },
    { 0x4ea8a683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d58a687, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd78a68f, 61 },
    { 0x6cf8a690, 61 },
    { 0x5088a691, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb8a694, 34 },
    { 0xa288a695, 98 },
    { 0x888a696, 45 },
    { 0x5ba8a697, 46 },
    { 0, 0 },
    { 0x1a48a699, 87 },
    { 0, 0 },
    { 0x9db8a69b, 90 },
    { 0xdb48a69b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8a6a1, 90 },
    { 0x8a28a6a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8a6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4248a6af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c68a6b3, 33 },
    { 0x6748a6b3, 23 },
    { 0xf518a6b5, 66 },
    { 0x2ff8a6b3, 55 },
    { 0x7f8a6b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb038a6ba, 94 },
    { 0x6a48a6bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3988a6be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e8a6c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08a6c7, 100 },
    { 0x4678a6c8, 23 },
    { 0x4f18a6c8, 46 },
    { 0, 0 },
    { 0x408a6cb, 34 },
    { 0x71a8a6cc, 98 },
    { 0x9d8a6cd, 90 },
    { 0x4998a6ce, 66 },
    { 0x8bd8a6cf, 66 },
    { 0x628a6cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5158a6db, 76 },
    { 0, 0 },
    { 0xf978a6dd, 90 },
    { 0x3da8a6dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8a6e5, 61 },
    { 0x5f58a6e6, 23 },
    { 0x8f58a6e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668a6f0, 90 },
    { 0x4708a6f1, 80 },
    { 0xff68a6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb48a6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88a6fa, 90 },
    { 0xe348a6fb, 55 },
    { 0, 0 },
    { 0x81d8a6fd, 61 },
    { 0x3d78a6fd, 98 },
    { 0xf088a6fd, 23 },
    { 0x2e48a6ff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8a703, 77 },
    { 0, 0 },
    { 0xc308a705, 46 },
    { 0x898a705, 100 },
    { 0, 0 },
    { 0x93e8a708, 90 },
    { 0x50f8a709, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e58a70c, 46 },
    { 0, 0 },
    { 0xdaa8a70e, 61 },
    { 0, 0 },
    { 0xb848a710, 90 },
    { 0, 0 },
    { 0xddc8a712, 88 },
    { 0xc308a713, 17 },
    { 0xcf88a714, 90 },
    { 0x6bd8a715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f08a718, 90 },
    { 0x5848a719, 78 },
    { 0xae88a718, 90 },
    { 0xf2b8a71b, 33 },
    { 0, 0 },
    { 0xc018a71d, 93 },
    { 0xc068a71e, 66 },
    { 0xb928a71f, 98 },
    { 0x7cf8a720, 34 },
    { 0x3fd8a71f, 33 },
    { 0xf328a722, 78 },
    { 0xba88a722, 91 },
    { 0xd118a71e, 98 },
    { 0x5658a71e, 90 },
    { 0xa348a71e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb8a729, 66 },
    { 0, 0 },
    { 0x1008a72b, 93 },
    { 0, 0 },
    { 0x25f8a72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648a731, 61 },
    { 0xa488a732, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d8a735, 90 },
    { 0x48f8a736, 52 },
    { 0x6008a737, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8a73c, 88 },
    { 0, 0 },
    { 0x2548a73e, 95 },
    { 0, 0 },
    { 0xca58a740, 33 },
    { 0, 0 },
    { 0x7a58a742, 52 },
    { 0, 0 },
    { 0x7ed8a744, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd108a74c, 55 },
    { 0xd908a74d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538a751, 33 },
    { 0, 0 },
    { 0x37d8a753, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9238a758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e88a75c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c8a764, 95 },
    { 0x2f88a765, 78 },
    { 0x9f48a764, 61 },
    { 0xaf58a766, 95 },
    { 0xd608a764, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb48a76e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898a776, 61 },
    { 0xefd8a776, 33 },
    { 0x21d8a776, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a782, 98 },
    { 0, 0 },
    { 0xacb8a784, 51 },
    { 0, 0 },
    { 0xf178a786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5908a78d, 61 },
    { 0, 0 },
    { 0xff78a78f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a8a794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618a797, 90 },
    { 0xa908a797, 85 },
    { 0, 0 },
    { 0xd7a8a79a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa228a7a2, 90 },
    { 0x2ae8a7a3, 58 },
    { 0x3e18a7a4, 33 },
    { 0, 0 },
    { 0x7e98a7a6, 23 },
    { 0x36a8a7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc388a7aa, 52 },
    { 0, 0 },
    { 0xebc8a7ac, 61 },
    { 0xef28a7ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8a7b8, 84 },
    { 0x69e8a7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378a7be, 98 },
    { 0x218a7bf, 55 },
    { 0xf638a7bf, 61 },
    { 0x7eb8a7bf, 98 },
    { 0xc488a7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78a7c8, 76 },
    { 0x798a7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7238a7cd, 34 },
    { 0, 0 },
    { 0xc8e8a7cf, 95 },
    { 0x5e78a7d0, 93 },
    { 0x128a7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a7d6, 91 },
    { 0x6bc8a7d7, 68 },
    { 0x9c98a7d8, 98 },
    { 0x21a8a7d9, 55 },
    { 0, 0 },
    { 0x8e98a7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8a7e2, 66 },
    { 0xaa98a7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee68a7e7, 90 },
    { 0x748a7e7, 90 },
    { 0xef38a7e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8a7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98a7f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478a7f9, 61 },
    { 0xe688a7fa, 90 },
    { 0xd838a7fa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a8a7fe, 46 },
    { 0xd378a7fe, 80 },
    { 0xb298a7ff, 76 },
    { 0xee58a801, 98 },
    { 0xfbe8a801, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8a807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e08a80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd98a817, 52 },
    { 0x3418a817, 95 },
    { 0xe1f8a819, 66 },
    { 0, 0 },
    { 0xf5a8a81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5498a81e, 61 },
    { 0, 0 },
    { 0x1298a820, 64 },
    { 0, 0 },
    { 0xccb8a822, 55 },
    { 0x8d38a823, 66 },
    { 0x8e68a824, 90 },
    { 0x58a823, 23 },
    { 0x23b8a826, 56 },
    { 0x4f8a827, 61 },
    { 0xe898a828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f8a82b, 90 },
    { 0x3a48a82b, 83 },
    { 0x9d38a82d, 66 },
    { 0x1098a82e, 90 },
    { 0, 0 },
    { 0x1818a830, 33 },
    { 0, 0 },
    { 0x4bd8a832, 77 },
    { 0x5b48a832, 98 },
    { 0xe5c8a834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2258a83b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2048a83e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7008a843, 44 },
    { 0, 0 },
    { 0x1858a845, 93 },
    { 0xafa8a846, 90 },
    { 0x1bd8a847, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608a84a, 23 },
    { 0x6d08a84b, 98 },
    { 0, 0 },
    { 0x6cf8a84d, 76 },
    { 0xbe8a84d, 95 },
    { 0x6388a84f, 33 },
    { 0, 0 },
    { 0x6e28a851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a8a859, 46 },
    { 0x68e8a859, 85 },
    { 0x76a8a859, 23 },
    { 0xc0f8a85b, 90 },
    { 0xcf58a85a, 43 },
    { 0xe2b8a85e, 55 },
    { 0xda8a85a, 98 },
    { 0x3a48a859, 76 },
    { 0xa658a85c, 90 },
    { 0, 0 },
    { 0x1a98a863, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e58a866, 55 },
    { 0, 0 },
    { 0x3558a868, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188a86b, 61 },
    { 0xdc88a86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5188a86f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238a873, 23 },
    { 0xc168a874, 85 },
    { 0x6188a875, 33 },
    { 0x5d58a876, 34 },
    { 0x73d8a876, 93 },
    { 0xc198a878, 61 },
    { 0x44b8a879, 91 },
    { 0xc6b8a874, 90 },
    { 0, 0 },
    { 0x3c68a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78a883, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398a886, 67 },
    { 0, 0 },
    { 0x3d18a888, 64 },
    { 0xb288a889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc068a88e, 23 },
    { 0xba08a88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc8a898, 90 },
    { 0xe048a899, 98 },
    { 0x2c78a89a, 61 },
    { 0x18c8a89b, 90 },
    { 0x8368a89c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8688a8a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8a8a4, 95 },
    { 0, 0 },
    { 0x2d28a8a6, 76 },
    { 0xc7e8a8a7, 98 },
    { 0x5018a8a8, 34 },
    { 0x53f8a8a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7018a8b1, 23 },
    { 0x49f8a8b1, 98 },
    { 0x2cf8a8b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4288a8bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc068a8c1, 98 },
    { 0x9b68a8c1, 46 },
    { 0x9678a8c3, 43 },
    { 0xbb58a8c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8a8d1, 68 },
    { 0x3a68a8d1, 55 },
    { 0x4ee8a8d2, 33 },
    { 0xbf68a8d4, 90 },
    { 0x44e8a8d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a48a8d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa008a8dc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6968a8e3, 44 },
    { 0, 0 },
    { 0x2be8a8e5, 95 },
    { 0x7198a8e6, 23 },
    { 0x5358a8e5, 85 },
    { 0xf9e8a8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c58a8eb, 34 },
    { 0xee58a8ec, 90 },
    { 0xe918a8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a58a8f1, 76 },
    { 0x3468a8f2, 84 },
    { 0x2b78a8f3, 34 },
    { 0, 0 },
    { 0xaca8a8f5, 66 },
    { 0x47b8a8f5, 33 },
    { 0, 0 },
    { 0xc848a8f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1668a8fb, 23 },
    { 0, 0 },
    { 0xf228a8fd, 90 },
    { 0x6d8a8fe, 90 },
    { 0x75f8a8ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e8a905, 90 },
    { 0, 0 },
    { 0xc2b8a907, 91 },
    { 0xb698a907, 43 },
    { 0xc3a8a909, 91 },
    { 0x7938a90a, 76 },
    { 0x4fa8a90b, 40 },
    { 0xf6a8a90c, 33 },
    { 0xec88a90d, 78 },
    { 0x1b08a90c, 98 },
    { 0x64d8a90c, 95 },
    { 0xd978a90d, 98 },
    { 0xab8a911, 61 },
    { 0xb048a90a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b8a917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba8a91e, 52 },
    { 0xafd8a91e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898a924, 23 },
    { 0x74b8a925, 23 },
    { 0x77e8a926, 43 },
    { 0x7ae8a927, 99 },
    { 0x9768a927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc508a92b, 95 },
    { 0x21b8a92c, 23 },
    { 0, 0 },
    { 0xefe8a92e, 55 },
    { 0, 0 },
    { 0x48f8a930, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe858a934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f8a937, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb8a93a, 61 },
    { 0xcba8a93b, 82 },
    { 0xb068a93b, 100 },
    { 0xad18a93d, 23 },
    { 0, 0 },
    { 0x48a93f, 98 },
    { 0x3a48a940, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a94a, 93 },
    { 0xa718a94b, 99 },
    { 0xcb78a94b, 61 },
    { 0, 0 },
    { 0xa0b8a94e, 55 },
    { 0xf18a94f, 90 },
    { 0, 0 },
    { 0x2388a951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8a954, 46 },
    { 0xd838a955, 78 },
    { 0x98a955, 98 },
    { 0, 0 },
    { 0x4078a958, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb8a95c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a8a964, 76 },
    { 0xf818a965, 63 },
    { 0, 0 },
    { 0xca88a967, 87 },
    { 0x67c8a968, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58a96d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b8a971, 43 },
    { 0xab48a972, 23 },
    { 0x6118a973, 90 },
    { 0x57b8a974, 61 },
    { 0x72d8a975, 76 },
    { 0xf3e8a976, 36 },
    { 0x75b8a976, 83 },
    { 0xf508a976, 100 },
    { 0xf378a979, 87 },
    { 0xd458a97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe098a97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38a987, 90 },
    { 0x39f8a988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8a98b, 90 },
    { 0xb2b8a98c, 87 },
    { 0x3d68a98d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8a991, 75 },
    { 0x77b8a992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa08a995, 33 },
    { 0x9a28a996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb258a99f, 61 },
    { 0xe7c8a99f, 95 },
    { 0x9768a9a1, 95 },
    { 0x2d38a99f, 23 },
    { 0, 0 },
    { 0x4628a9a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb378a9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf428a9ab, 78 },
    { 0xa468a9ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c48a9b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08a9b7, 95 },
    { 0xe958a9b8, 83 },
    { 0xe3a8a9b8, 77 },
    { 0x25c8a9b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78a9c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9778a9c5, 61 },
    { 0x4638a9c5, 23 },
    { 0, 0 },
    { 0xf468a9c8, 91 },
    { 0x9ce8a9c9, 33 },
    { 0x95d8a9c9, 85 },
    { 0xb6a8a9cb, 66 },
    { 0xc8a9ca, 61 },
    { 0, 0 },
    { 0x8868a9ce, 23 },
    { 0, 0 },
    { 0xfb18a9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff8a9d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b48a9d8, 33 },
    { 0x31b8a9d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858a9dc, 81 },
    { 0, 0 },
    { 0x2678a9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad88a9e6, 95 },
    { 0x8668a9e7, 61 },
    { 0x14c8a9e8, 23 },
    { 0x3878a9e9, 85 },
    { 0xad58a9ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8a9fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8918aa01, 33 },
    { 0x5898aa01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8aa08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f8aa13, 43 },
    { 0x7a68aa14, 33 },
    { 0, 0 },
    { 0x37c8aa16, 98 },
    { 0, 0 },
    { 0x5348aa18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8aa1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c8aa1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8aa21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa018aa25, 61 },
    { 0xc7e8aa26, 90 },
    { 0, 0 },
    { 0x8c88aa28, 23 },
    { 0x9e58aa28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf8aa31, 65 },
    { 0x9fc8aa31, 34 },
    { 0, 0 },
    { 0xa908aa34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1518aa38, 33 },
    { 0, 0 },
    { 0x41f8aa3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e8aa3d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xde98aa40, 84 },
    { 0xc688aa41, 64 },
    { 0, 0 },
    { 0x5e18aa43, 66 },
    { 0x15c8aa44, 24 },
    { 0, 0 },
    { 0x2698aa46, 23 },
    { 0, 0 },
    { 0xc5a8aa48, 23 },
    { 0, 0 },
    { 0xdd68aa4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8aa4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8668aa51, 99 },
    { 0x2278aa51, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2028aa56, 88 },
    { 0x6c18aa57, 81 },
    { 0xfb58aa58, 90 },
    { 0x8328aa57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58aa5c, 46 },
    { 0xffc8aa5d, 33 },
    { 0xfcd8aa5e, 76 },
    { 0x5fb8aa5f, 90 },
    { 0xf288aa5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc8aa63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec78aa67, 98 },
    { 0xc778aa67, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d98aa6b, 34 },
    { 0, 0 },
    { 0x2a88aa6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde58aa70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6858aa73, 90 },
    { 0xa5d8aa74, 90 },
    { 0x1f98aa74, 56 },
    { 0x7158aa76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe28aa7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8878aa7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e98aa80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce68aa83, 34 },
    { 0xc678aa84, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd138aa89, 98 },
    { 0, 0 },
    { 0x5fd8aa8b, 95 },
    { 0, 0 },
    { 0xcdf8aa8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7718aa92, 90 },
    { 0x9f98aa92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8aa97, 34 },
    { 0xb8f8aa97, 76 },
    { 0, 0 },
    { 0xd3f8aa9a, 61 },
    { 0, 0 },
    { 0x4648aa9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28aaa5, 100 },
    { 0x818aaa6, 91 },
    { 0x22f8aaa5, 95 },
    { 0xda8aaa8, 67 },
    { 0x9958aaa9, 100 },
    { 0x6868aaa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2188aaad, 66 },
    { 0xe7c8aaad, 61 },
    { 0x11b8aaad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b98aab4, 34 },
    { 0xd228aab5, 90 },
    { 0x3828aab6, 38 },
    { 0x2bc8aab7, 90 },
    { 0x9458aab5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88aabb, 33 },
    { 0x65e8aabc, 76 },
    { 0, 0 },
    { 0x96a8aabe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7798aac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca78aad4, 90 },
    { 0xaca8aad5, 43 },
    { 0x5a38aad6, 17 },
    { 0xfa38aad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a8aadd, 55 },
    { 0, 0 },
    { 0xd508aadf, 93 },
    { 0xe558aae0, 85 },
    { 0xc0a8aae1, 90 },
    { 0xa48aae2, 61 },
    { 0xb3d8aae3, 76 },
    { 0x8398aae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8aaf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf228aafa, 80 },
    { 0xe028aafb, 38 },
    { 0xa3c8aafc, 49 },
    { 0, 0 },
    { 0x7c58aafe, 23 },
    { 0xb528aaff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9468ab04, 95 },
    { 0x79d8ab05, 75 },
    { 0xcd98ab06, 76 },
    { 0x8878ab07, 67 },
    { 0xa3f8ab08, 98 },
    { 0xb508ab07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158ab0d, 46 },
    { 0, 0 },
    { 0x8878ab0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8ab12, 90 },
    { 0x1358ab12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b8ab16, 90 },
    { 0, 0 },
    { 0x5668ab18, 66 },
    { 0xdd58ab18, 85 },
    { 0x3508ab1a, 87 },
    { 0xd408ab1b, 61 },
    { 0x8d68ab1c, 33 },
    { 0, 0 },
    { 0xdad8ab1e, 43 },
    { 0, 0 },
    { 0x8f38ab20, 93 },
    { 0xc398ab21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8ab25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98ab2b, 55 },
    { 0xb448ab2c, 100 },
    { 0, 0 },
    { 0xe2d8ab2e, 90 },
    { 0, 0 },
    { 0x9588ab30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438ab34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8678ab3b, 33 },
    { 0, 0 },
    { 0xb358ab3d, 93 },
    { 0xc9a8ab3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848ab42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc658ab48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7508ab4f, 98 },
    { 0x32a8ab50, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58ab53, 33 },
    { 0, 0 },
    { 0xc8ab55, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4268ab59, 33 },
    { 0x5e58ab59, 61 },
    { 0x9828ab5b, 61 },
    { 0x2c58ab5c, 61 },
    { 0x6a38ab5d, 43 },
    { 0x9f88ab5e, 90 },
    { 0x4128ab5e, 76 },
    { 0x6a28ab60, 100 },
    { 0x4438ab5c, 43 },
    { 0xa448ab62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e8ab65, 90 },
    { 0x8af8ab66, 90 },
    { 0xd3b8ab66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a8ab6a, 67 },
    { 0, 0 },
    { 0x5eb8ab6c, 98 },
    { 0xda78ab6d, 95 },
    { 0x808ab6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a88ab74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe88ab78, 67 },
    { 0xcf08ab78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8ab7d, 33 },
    { 0, 0 },
    { 0x5b48ab7f, 33 },
    { 0x4048ab7f, 34 },
    { 0x4508ab80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc048ab87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd458ab8a, 90 },
    { 0, 0 },
    { 0x5558ab8c, 90 },
    { 0, 0 },
    { 0x5848ab8e, 61 },
    { 0xd58ab8f, 53 },
    { 0xbea8ab90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef8ab95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8ab98, 33 },
    { 0, 0 },
    { 0x56a8ab9a, 61 },
    { 0x7ea8ab9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8aba3, 93 },
    { 0x8618aba4, 61 },
    { 0x8798aba5, 98 },
    { 0xa598aba6, 96 },
    { 0, 0 },
    { 0x4c78aba8, 98 },
    { 0x9228aba8, 84 },
    { 0xa908abaa, 23 },
    { 0, 0 },
    { 0x2ff8abac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5738abb0, 93 },
    { 0, 0 },
    { 0x988abb2, 87 },
    { 0, 0 },
    { 0xea8abb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9158abb9, 33 },
    { 0x4378abba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f28abbf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a88abc3, 98 },
    { 0xd718abc3, 95 },
    { 0x8048abc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c8abc9, 61 },
    { 0xaed8abca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x868abcd, 76 },
    { 0x1108abcd, 33 },
    { 0x1718abcf, 76 },
    { 0x9b38abd0, 33 },
    { 0xc4b8abd0, 61 },
    { 0xbc8abd2, 69 },
    { 0x5048abd2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c8abd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x378abda, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8abde, 52 },
    { 0x9308abde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8abe4, 61 },
    { 0x24b8abe5, 61 },
    { 0x5b88abe6, 23 },
    { 0xd98abe5, 61 },
    { 0x3718abe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea8abeb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998abee, 61 },
    { 0, 0 },
    { 0xf788abf0, 61 },
    { 0x5018abf1, 61 },
    { 0x2318abf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2828abf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28abf8, 61 },
    { 0x5ca8abf8, 78 },
    { 0x6068abfa, 33 },
    { 0xefd8abfa, 90 },
    { 0xe208abfb, 95 },
    { 0x14e8abfd, 90 },
    { 0x4558abfe, 43 },
    { 0xf488abff, 33 },
    { 0, 0 },
    { 0x4ff8ac01, 23 },
    { 0, 0 },
    { 0x1a48ac03, 95 },
    { 0, 0 },
    { 0xa78ac05, 34 },
    { 0, 0 },
    { 0xd288ac07, 98 },
    { 0x5b68ac08, 90 },
    { 0xdeb8ac09, 23 },
    { 0xc338ac0a, 67 },
    { 0x6e48ac0a, 52 },
    { 0x9ef8ac0c, 81 },
    { 0x6038ac0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7268ac14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac19, 81 },
    { 0x9488ac19, 98 },
    { 0x42b8ac1a, 33 },
    { 0x71a8ac1c, 66 },
    { 0xc818ac1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b8ac23, 68 },
    { 0x5a18ac23, 33 },
    { 0xb658ac24, 76 },
    { 0xcef8ac26, 34 },
    { 0, 0 },
    { 0x30c8ac28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e8ac2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc498ac30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce48ac33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8ac38, 61 },
    { 0xb4b8ac38, 75 },
    { 0xf8a8ac3a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef98ac3d, 36 },
    { 0xa68ac3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf388ac41, 95 },
    { 0, 0 },
    { 0x97e8ac43, 98 },
    { 0x2148ac44, 33 },
    { 0xd8a8ac45, 88 },
    { 0, 0 },
    { 0xec78ac47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde98ac4f, 66 },
    { 0x2e18ac50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d08ac53, 90 },
    { 0, 0 },
    { 0xad08ac55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd58ac60, 46 },
    { 0xa0d8ac61, 98 },
    { 0x7708ac62, 76 },
    { 0, 0 },
    { 0x3248ac64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a8ac67, 95 },
    { 0x6368ac67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88ac6c, 93 },
    { 0x32d8ac6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1148ac71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe678ac75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f58ac78, 91 },
    { 0x92c8ac79, 90 },
    { 0, 0 },
    { 0xeaf8ac7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a68ac7e, 90 },
    { 0xa58ac7f, 91 },
    { 0x45f8ac7e, 84 },
    { 0xf218ac81, 55 },
    { 0, 0 },
    { 0x41c8ac83, 33 },
    { 0x74f8ac83, 94 },
    { 0, 0 },
    { 0x6648ac86, 46 },
    { 0x5bc8ac86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5018ac92, 90 },
    { 0, 0 },
    { 0x3418ac94, 78 },
    { 0, 0 },
    { 0xe118ac96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538ac9a, 46 },
    { 0, 0 },
    { 0x128ac9c, 95 },
    { 0x2358ac9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d8aca0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8aca3, 23 },
    { 0x5df8aca3, 95 },
    { 0, 0 },
    { 0x9718aca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38acac, 90 },
    { 0xa068acad, 90 },
    { 0, 0 },
    { 0xac08acaf, 90 },
    { 0, 0 },
    { 0xfa08acb1, 23 },
    { 0xe5f8acb1, 87 },
    { 0x9578acb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58acbe, 23 },
    { 0x2ef8acbf, 23 },
    { 0x9f98acc0, 55 },
    { 0xa328acc1, 61 },
    { 0x4088acc1, 76 },
    { 0xa308acbf, 95 },
    { 0x8c18acc4, 99 },
    { 0x1808acc5, 66 },
    { 0x1ec8acbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8accb, 90 },
    { 0xca28accb, 46 },
    { 0xffe8accd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718acd2, 98 },
    { 0x82b8acd3, 33 },
    { 0x41b8acd4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d88ace1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5468ace5, 52 },
    { 0x6f78ace5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb8aceb, 23 },
    { 0x9c48aceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898acef, 76 },
    { 0, 0 },
    { 0xac98acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f68acf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee28acfd, 90 },
    { 0x3558acfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d88ad02, 85 },
    { 0xdd38ad02, 66 },
    { 0xaeb8ad04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb138ad09, 95 },
    { 0, 0 },
    { 0x1c98ad0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ad15, 43 },
    { 0, 0 },
    { 0xea98ad17, 80 },
    { 0x1048ad18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7908ad1f, 76 },
    { 0x92b8ad20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908ad23, 61 },
    { 0x2388ad24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08ad2d, 95 },
    { 0xc2a8ad2e, 90 },
    { 0xb748ad2d, 98 },
    { 0xbe48ad30, 23 },
    { 0x1128ad30, 93 },
    { 0xe0e8ad32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb128ad36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd818ad39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd198ad3e, 100 },
    { 0x558ad3f, 90 },
    { 0x20d8ad3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398ad43, 61 },
    { 0, 0 },
    { 0xcbf8ad45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb8ad48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x448ad4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa298ad4f, 46 },
    { 0xce8ad50, 33 },
    { 0xefd8ad51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa378ad54, 99 },
    { 0x10f8ad55, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e8ad58, 100 },
    { 0, 0 },
    { 0xae98ad5a, 55 },
    { 0xe3b8ad5a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118ad63, 55 },
    { 0x6728ad64, 90 },
    { 0xfe78ad65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c8ad6a, 33 },
    { 0x6a18ad6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8898ad6e, 95 },
    { 0, 0 },
    { 0x29c8ad70, 76 },
    { 0xd68ad71, 88 },
    { 0xd648ad70, 64 },
    { 0x8c78ad73, 52 },
    { 0xa6f8ad73, 61 },
    { 0x6fa8ad75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf8ad78, 88 },
    { 0xd038ad78, 95 },
    { 0x6368ad7a, 43 },
    { 0x8d48ad7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8ad7f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8ad84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8ad88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3528ad8e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xa828ad91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e8ad9c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8ad9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a8ada5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9648ada8, 61 },
    { 0x7008ada9, 23 },
    { 0xa538ada9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278adaf, 55 },
    { 0, 0 },
    { 0x47d8adb1, 99 },
    { 0x4928adb2, 87 },
    { 0, 0 },
    { 0xf9c8adb4, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1218adb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88adc1, 55 },
    { 0, 0 },
    { 0x4488adc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8add6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd418adde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ade6, 85 },
    { 0xf358ade7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe38aded, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968adf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98adf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88adff, 46 },
    { 0xc3f8ae00, 23 },
    { 0xb568ae01, 76 },
    { 0x9988ae01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa8ae06, 55 },
    { 0, 0 },
    { 0x8eb8ae08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688ae0b, 23 },
    { 0xee18ae0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8ae0f, 90 },
    { 0x9448ae10, 23 },
    { 0, 0 },
    { 0x3618ae12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f8ae15, 33 },
    { 0xafb8ae16, 98 },
    { 0x5788ae17, 61 },
    { 0x9128ae18, 98 },
    { 0x94f8ae19, 90 },
    { 0xd6d8ae18, 66 },
    { 0xad78ae1a, 36 },
    { 0x92e8ae1c, 23 },
    { 0xdbb8ae1c, 93 },
    { 0x7f8ae18, 95 },
    { 0xcac8ae16, 95 },
    { 0xd808ae1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928ae23, 99 },
    { 0xdcd8ae24, 83 },
    { 0x4588ae24, 98 },
    { 0xe768ae24, 34 },
    { 0xe48ae27, 46 },
    { 0xd358ae28, 66 },
    { 0xb328ae28, 61 },
    { 0x4e08ae29, 23 },
    { 0xa6e8ae2a, 53 },
    { 0, 0 },
    { 0x4688ae2d, 99 },
    { 0, 0 },
    { 0x3438ae2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1038ae33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe818ae3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2868ae40, 90 },
    { 0x7068ae41, 66 },
    { 0xa9a8ae41, 23 },
    { 0x13a8ae42, 33 },
    { 0x308ae40, 67 },
    { 0xa488ae41, 76 },
    { 0xf8d8ae46, 84 },
    { 0x5a18ae46, 33 },
    { 0x6c98ae48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa768ae4c, 36 },
    { 0xc6f8ae4d, 90 },
    { 0x3d08ae4d, 61 },
    { 0xcf98ae4f, 61 },
    { 0x4ea8ae4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38ae5c, 98 },
    { 0x8058ae5d, 39 },
    { 0, 0 },
    { 0x1388ae5f, 61 },
    { 0x4538ae60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9428ae65, 98 },
    { 0xdd88ae65, 98 },
    { 0xccb8ae65, 98 },
    { 0x7958ae68, 61 },
    { 0x9cd8ae69, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ae6c, 75 },
    { 0xe3c8ae6d, 43 },
    { 0x5e48ae6e, 95 },
    { 0x50e8ae6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce8ae75, 55 },
    { 0, 0 },
    { 0xd838ae77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd8ae7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc8ae81, 84 },
    { 0, 0 },
    { 0xa3f8ae83, 43 },
    { 0, 0 },
    { 0x8a28ae85, 17 },
    { 0x18a8ae86, 90 },
    { 0xc008ae85, 61 },
    { 0, 0 },
    { 0xd3e8ae89, 33 },
    { 0x7078ae8a, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f58ae8d, 75 },
    { 0x2a18ae8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e8ae91, 55 },
    { 0xc9a8ae92, 33 },
    { 0xcae8ae91, 90 },
    { 0, 0 },
    { 0x7268ae95, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x4308ae98, 24 },
    { 0, 0 },
    { 0xb528ae9a, 95 },
    { 0x3c98ae9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa718ae9f, 90 },
    { 0xe528aea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd358aead, 65 },
    { 0, 0 },
    { 0x608aeaf, 85 },
    { 0xbc28aeaf, 33 },
    { 0x1208aeb1, 33 },
    { 0, 0 },
    { 0x65d8aeb3, 33 },
    { 0xb128aeb4, 85 },
    { 0x4078aeb4, 52 },
    { 0x48b8aeb6, 91 },
    { 0xdaa8aeb5, 81 },
    { 0x70a8aeb3, 33 },
    { 0x8598aeb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a78aebd, 46 },
    { 0x4598aebe, 33 },
    { 0x2de8aebf, 23 },
    { 0x53e8aebf, 90 },
    { 0x42c8aebe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8aec4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb578aec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d8aecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438aed1, 61 },
    { 0, 0 },
    { 0x7d58aed3, 33 },
    { 0xd948aed4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2648aed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1108aedb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d38aee3, 90 },
    { 0, 0 },
    { 0x8bf8aee5, 43 },
    { 0x1008aee6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a88aeea, 65 },
    { 0x42f8aeeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a08aeef, 23 },
    { 0xf28aef0, 23 },
    { 0x4198aeef, 17 },
    { 0, 0 },
    { 0x7068aef3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a8aefc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e8af01, 98 },
    { 0, 0 },
    { 0xfe8af03, 90 },
    { 0, 0 },
    { 0x1638af05, 98 },
    { 0x1f98af06, 33 },
    { 0xd578af06, 23 },
    { 0x1278af07, 93 },
    { 0, 0 },
    { 0xba8af0a, 33 },
    { 0, 0 },
    { 0xb0d8af0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd168af0f, 76 },
    { 0, 0 },
    { 0xf9c8af11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78af14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c8af17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d8af1a, 23 },
    { 0, 0 },
    { 0xc3e8af1c, 33 },
    { 0x93c8af1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08af20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28af27, 34 },
    { 0x438af28, 61 },
    { 0xbf48af27, 91 },
    { 0x6ea8af28, 17 },
    { 0x7a38af2b, 84 },
    { 0, 0 },
    { 0x1f88af2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc78af31, 93 },
    { 0, 0 },
    { 0xe598af33, 61 },
    { 0, 0 },
    { 0xb5b8af35, 23 },
    { 0x9738af35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2558af3a, 34 },
    { 0xd38af3b, 96 },
    { 0x2078af3c, 100 },
    { 0xe6c8af3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7838af41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8af45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f28af48, 33 },
    { 0, 0 },
    { 0x7448af4a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd278af55, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8af5a, 91 },
    { 0x2a48af5b, 87 },
    { 0, 0 },
    { 0x4b68af5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c8af67, 55 },
    { 0, 0 },
    { 0xe2f8af69, 55 },
    { 0, 0 },
    { 0xd038af6b, 95 },
    { 0x7118af6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8af6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa8af74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8af77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9668af7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf808af81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c8af84, 88 },
    { 0, 0 },
    { 0x3208af86, 77 },
    { 0x648af86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748af95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc78af99, 55 },
    { 0, 0 },
    { 0x4898af9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a8af9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a68afab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98afb3, 61 },
    { 0x7a78afb4, 23 },
    { 0, 0 },
    { 0x258afb6, 78 },
    { 0, 0 },
    { 0x30d8afb8, 52 },
    { 0xc7b8afb9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f8afbe, 61 },
    { 0, 0 },
    { 0xd068afc0, 61 },
    { 0xa358afc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8afc8, 90 },
    { 0x8408afc9, 78 },
    { 0xd928afc9, 100 },
    { 0, 0 },
    { 0x6d38afcc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8afcf, 95 },
    { 0xe398afcf, 90 },
    { 0x1b28afd0, 23 },
    { 0xe748afd1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc398afd6, 76 },
    { 0xb7f8afd6, 34 },
    { 0xb7e8afd8, 61 },
    { 0x5d78afd9, 36 },
    { 0xe9e8afd7, 76 },
    { 0x6d28afdb, 33 },
    { 0xfb68afdb, 69 },
    { 0xcfb8afdb, 95 },
    { 0x6258afde, 95 },
    { 0xf598afd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408afe7, 92 },
    { 0xe1c8afe7, 23 },
    { 0xa7a8afe8, 77 },
    { 0x75b8afea, 78 },
    { 0x75f8afea, 90 },
    { 0x9538afe7, 34 },
    { 0xf948afed, 78 },
    { 0xdf28afe9, 98 },
    { 0xd2c8afea, 61 },
    { 0x2b8afea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e8aff6, 76 },
    { 0, 0 },
    { 0xada8aff8, 34 },
    { 0x5fb8aff9, 84 },
    { 0x83c8affa, 43 },
    { 0x6a78affa, 40 },
    { 0x39d8affa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe448b000, 90 },
    { 0xded8b001, 85 },
    { 0xaca8b001, 95 },
    { 0, 0 },
    { 0x2148b004, 90 },
    { 0x2948b004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208b008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f8b00b, 61 },
    { 0x2568b00b, 56 },
    { 0xec28b00c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418b011, 68 },
    { 0x86f8b012, 33 },
    { 0, 0 },
    { 0xc9d8b014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5278b01a, 33 },
    { 0, 0 },
    { 0xe2c8b01c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa318b021, 86 },
    { 0, 0 },
    { 0x2fc8b023, 55 },
    { 0xdc48b024, 100 },
    { 0x88f8b023, 90 },
    { 0x4a38b026, 78 },
    { 0x7ba8b026, 93 },
    { 0x9018b028, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b02f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d8b032, 95 },
    { 0, 0 },
    { 0x21d8b034, 81 },
    { 0xae88b034, 23 },
    { 0x50c8b036, 90 },
    { 0x7f18b037, 83 },
    { 0, 0 },
    { 0x85e8b039, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6138b03d, 23 },
    { 0, 0 },
    { 0x7308b03f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838b042, 61 },
    { 0x8b043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268b046, 33 },
    { 0x7248b047, 66 },
    { 0, 0 },
    { 0xf9c8b049, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a8b04f, 95 },
    { 0x4128b050, 90 },
    { 0x4728b050, 33 },
    { 0, 0 },
    { 0x7b78b053, 90 },
    { 0, 0 },
    { 0x8aa8b055, 95 },
    { 0, 0 },
    { 0x4c8b057, 61 },
    { 0xe5e8b058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8b05d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a8b060, 90 },
    { 0, 0 },
    { 0x38b062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c8b065, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d8b072, 95 },
    { 0xc818b073, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8b077, 87 },
    { 0x8df8b077, 95 },
    { 0x968b079, 78 },
    { 0x52b8b07a, 33 },
    { 0xc198b079, 95 },
    { 0x3578b07a, 46 },
    { 0xd028b07d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8b082, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x848b085, 61 },
    { 0xc888b086, 76 },
    { 0xee58b087, 98 },
    { 0x41b8b088, 43 },
    { 0, 0 },
    { 0x1ac8b08a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c8b08e, 61 },
    { 0, 0 },
    { 0xe658b090, 40 },
    { 0x8688b090, 61 },
    { 0x7f88b092, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8b095, 61 },
    { 0xfaa8b096, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678b099, 43 },
    { 0, 0 },
    { 0xa248b09b, 81 },
    { 0, 0 },
    { 0xb468b09d, 98 },
    { 0xf658b09e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea8b0a3, 43 },
    { 0xebd8b0a4, 98 },
    { 0x5158b0a5, 90 },
    { 0xb008b0a3, 33 },
    { 0x2e8b0a7, 43 },
    { 0xb038b0a3, 63 },
    { 0xee08b0a4, 33 },
    { 0x9d68b0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8b0b1, 83 },
    { 0x24c8b0b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e8b0be, 33 },
    { 0x1778b0bf, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8b0c5, 66 },
    { 0x45b8b0c6, 33 },
    { 0x13b8b0c7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b0d1, 88 },
    { 0xde58b0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4908b0d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8b0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8b0e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd118b0e6, 95 },
    { 0xc048b0e7, 55 },
    { 0xc5a8b0e8, 43 },
    { 0x18b8b0e9, 91 },
    { 0x6c98b0e6, 33 },
    { 0, 0 },
    { 0xeb78b0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868b0f2, 76 },
    { 0xc4d8b0f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2908b0fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1298b101, 95 },
    { 0x7208b101, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88b105, 61 },
    { 0, 0 },
    { 0xc3d8b107, 98 },
    { 0x1c98b108, 33 },
    { 0x99f8b107, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c8b10e, 23 },
    { 0x1238b10e, 23 },
    { 0x1688b110, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8b115, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8b11c, 95 },
    { 0x6098b11d, 93 },
    { 0, 0 },
    { 0x8c8b11f, 90 },
    { 0x2e78b120, 88 },
    { 0x4e88b121, 55 },
    { 0xdf48b11f, 23 },
    { 0x9168b123, 98 },
    { 0x2918b124, 61 },
    { 0x2af8b11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5518b128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e58b12d, 34 },
    { 0xec08b12e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf8b132, 33 },
    { 0, 0 },
    { 0xf008b134, 50 },
    { 0x48b135, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba8b13a, 81 },
    { 0x7e58b13a, 68 },
    { 0x7f8b13a, 34 },
    { 0x78e8b13d, 61 },
    { 0xb6f8b13e, 98 },
    { 0xf0f8b13f, 78 },
    { 0xc878b13b, 100 },
    { 0x4408b13f, 98 },
    { 0x72a8b13e, 93 },
    { 0, 0 },
    { 0xa1e8b144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8b148, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1408b14e, 84 },
    { 0xcc98b14f, 98 },
    { 0, 0 },
    { 0xd748b151, 33 },
    { 0x8718b152, 23 },
    { 0xa948b153, 61 },
    { 0xc2f8b154, 61 },
    { 0x3878b155, 52 },
    { 0x1a88b153, 76 },
    { 0, 0 },
    { 0xf758b158, 55 },
    { 0x58d8b159, 61 },
    { 0x9b48b15a, 85 },
    { 0xf1d8b15a, 55 },
    { 0xeea8b159, 33 },
    { 0x16a8b15c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d8b162, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2898b165, 78 },
    { 0, 0 },
    { 0x72e8b167, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408b16c, 98 },
    { 0x99c8b16c, 90 },
    { 0xed58b16e, 17 },
    { 0, 0 },
    { 0x53c8b170, 61 },
    { 0x2528b170, 23 },
    { 0x61e8b172, 43 },
    { 0x9268b171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288b176, 55 },
    { 0x33d8b176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba8b183, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a08b189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a8b18d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b8b194, 33 },
    { 0x2598b195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208b198, 96 },
    { 0xb888b199, 68 },
    { 0xfac8b199, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808b19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8b1a2, 34 },
    { 0, 0 },
    { 0xfc78b1a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c88b1aa, 33 },
    { 0x5848b1ab, 33 },
    { 0x6eb8b1ac, 34 },
    { 0x5018b1aa, 98 },
    { 0, 0 },
    { 0xf018b1af, 75 },
    { 0xc878b1af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d8b1b3, 91 },
    { 0x47e8b1b3, 92 },
    { 0, 0 },
    { 0x98c8b1b6, 76 },
    { 0x62f8b1b7, 95 },
    { 0, 0 },
    { 0x78a8b1b9, 90 },
    { 0x9c68b1ba, 78 },
    { 0, 0 },
    { 0x3578b1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868b1c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc8b1c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f8b1d0, 84 },
    { 0xe888b1d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a78b1d6, 23 },
    { 0, 0 },
    { 0xf478b1d8, 97 },
    { 0, 0 },
    { 0x20f8b1da, 55 },
    { 0, 0 },
    { 0xc498b1dc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588b1e1, 78 },
    { 0xfec8b1e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x998b1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d98b1ee, 33 },
    { 0x4888b1ef, 23 },
    { 0x5688b1f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd838b1f7, 90 },
    { 0xbd48b1f8, 33 },
    { 0, 0 },
    { 0x7958b1fa, 88 },
    { 0x6c68b1fb, 99 },
    { 0xca38b1fa, 99 },
    { 0x65a8b1fd, 33 },
    { 0x3a98b1fd, 33 },
    { 0x5028b1fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc518b202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd8b208, 33 },
    { 0x7b58b209, 98 },
    { 0x77b8b20a, 84 },
    { 0xda18b20a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b212, 90 },
    { 0xee18b213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9438b216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c68b21c, 23 },
    { 0x3c98b21c, 40 },
    { 0x3e68b21d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8b227, 76 },
    { 0xc988b227, 33 },
    { 0, 0 },
    { 0xe7f8b22a, 55 },
    { 0, 0 },
    { 0xf68b22c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118b22f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a8b234, 90 },
    { 0, 0 },
    { 0x20b8b236, 87 },
    { 0x3ce8b237, 80 },
    { 0xf688b238, 77 },
    { 0x8a98b238, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d8b23c, 81 },
    { 0, 0 },
    { 0xc98b23e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e8b241, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd88b245, 98 },
    { 0x1af8b246, 34 },
    { 0xf318b247, 43 },
    { 0xebb8b248, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d88b24c, 23 },
    { 0x7ac8b24d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5958b251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8b254, 43 },
    { 0x9388b255, 33 },
    { 0x4238b254, 90 },
    { 0x5ad8b256, 100 },
    { 0x8198b254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe978b25c, 90 },
    { 0, 0 },
    { 0x1b58b25e, 61 },
    { 0xcc78b25f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec8b268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8b274, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac8b279, 61 },
    { 0xc8b8b279, 46 },
    { 0, 0 },
    { 0x7958b27c, 90 },
    { 0x6db8b27d, 46 },
    { 0x9398b27e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98b281, 61 },
    { 0x9208b281, 75 },
    { 0xa168b281, 23 },
    { 0xb168b281, 93 },
    { 0x7268b285, 56 },
    { 0, 0 },
    { 0xafd8b287, 23 },
    { 0x93e8b288, 33 },
    { 0xe748b289, 61 },
    { 0x7548b288, 61 },
    { 0xd728b287, 93 },
    { 0xd9f8b28c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8448b290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d8b294, 33 },
    { 0, 0 },
    { 0xb668b296, 98 },
    { 0x26d8b297, 90 },
    { 0x70c8b296, 95 },
    { 0, 0 },
    { 0xc238b29a, 33 },
    { 0x7368b29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe668b29f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe328b2a3, 76 },
    { 0x6628b2a3, 99 },
    { 0xd38b2a5, 97 },
    { 0, 0 },
    { 0xe638b2a7, 23 },
    { 0, 0 },
    { 0x19c8b2a9, 52 },
    { 0x1ea8b2a9, 34 },
    { 0xaf38b2ab, 87 },
    { 0xd38b2ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38b2af, 33 },
    { 0xff18b2b0, 76 },
    { 0x4838b2b1, 90 },
    { 0x19e8b2b1, 98 },
    { 0x4c98b2b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18b2c0, 98 },
    { 0, 0 },
    { 0x7328b2c2, 90 },
    { 0, 0 },
    { 0xdcb8b2c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac8b2c7, 90 },
    { 0x8278b2c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b2cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f8b2d0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb328b2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988b2d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc708b2dc, 90 },
    { 0, 0 },
    { 0x6448b2de, 69 },
    { 0, 0 },
    { 0xab8b2e0, 65 },
    { 0xa838b2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8b2e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8b2e9, 55 },
    { 0x5d8b2ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef8b2ed, 76 },
    { 0, 0 },
    { 0xb1f8b2ef, 90 },
    { 0, 0 },
    { 0x5db8b2f1, 95 },
    { 0x8d48b2f1, 78 },
    { 0x8a08b2f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec8b2f6, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88b2fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a8b2ff, 34 },
    { 0xe108b300, 90 },
    { 0x9508b300, 34 },
    { 0xe358b300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe178b307, 52 },
    { 0xa98b308, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7148b30d, 33 },
    { 0xa3c8b30d, 95 },
    { 0x8c58b30d, 90 },
    { 0, 0 },
    { 0x5858b311, 90 },
    { 0xe4d8b312, 61 },
    { 0, 0 },
    { 0x44e8b314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e08b317, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf648b31a, 46 },
    { 0x4468b31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3128b31e, 52 },
    { 0x8e58b31f, 87 },
    { 0x248b31f, 88 },
    { 0x4988b320, 90 },
    { 0x58b8b322, 52 },
    { 0xe2b8b31e, 55 },
    { 0, 0 },
    { 0xe838b325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9978b32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee8b32d, 90 },
    { 0, 0 },
    { 0x68a8b32f, 17 },
    { 0, 0 },
    { 0xb0a8b331, 90 },
    { 0xb608b331, 95 },
    { 0, 0 },
    { 0xe348b334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9398b33b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b33e, 90 },
    { 0x188b33f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f8b348, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8838b34d, 33 },
    { 0xa8b8b34e, 52 },
    { 0xb3f8b34d, 100 },
    { 0xe008b350, 90 },
    { 0x78b8b351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c8b357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8828b35c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8b361, 95 },
    { 0xf678b362, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28b36c, 90 },
    { 0xe168b36d, 95 },
    { 0, 0 },
    { 0x5a38b36f, 34 },
    { 0xae98b370, 55 },
    { 0x778b370, 34 },
    { 0xe0d8b372, 95 },
    { 0, 0 },
    { 0x83c8b374, 98 },
    { 0x2fe8b375, 77 },
    { 0xbc48b375, 33 },
    { 0, 0 },
    { 0xac18b378, 98 },
    { 0xcd78b379, 43 },
    { 0x2d08b378, 61 },
    { 0, 0 },
    { 0x478b37c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508b37f, 33 },
    { 0xa038b380, 90 },
    { 0x6d38b380, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4088b386, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de8b38a, 55 },
    { 0x44d8b38a, 33 },
    { 0x52a8b38c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5058b391, 95 },
    { 0x91d8b391, 23 },
    { 0xd778b393, 61 },
    { 0, 0 },
    { 0x178b395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d8b398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c8b39b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8b39f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x498b3a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf948b3a9, 98 },
    { 0x4e88b3a9, 95 },
    { 0xe48b3ab, 95 },
    { 0, 0 },
    { 0x1f88b3ad, 61 },
    { 0x9958b3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3628b3b1, 23 },
    { 0x7d78b3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba8b3b6, 90 },
    { 0x2da8b3b7, 33 },
    { 0x6198b3b7, 23 },
    { 0, 0 },
    { 0x4878b3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa608b3be, 93 },
    { 0x9e08b3bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e8b3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc608b3c5, 85 },
    { 0, 0 },
    { 0xdd88b3c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3608b3cc, 61 },
    { 0xf828b3cd, 95 },
    { 0, 0 },
    { 0xd8e8b3cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf688b3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb398b3d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8b3da, 34 },
    { 0xff18b3db, 52 },
    { 0xd5c8b3dc, 90 },
    { 0xde08b3dc, 98 },
    { 0x64f8b3de, 76 },
    { 0xb658b3dc, 61 },
    { 0x95f8b3e0, 98 },
    { 0, 0 },
    { 0x74d8b3e2, 90 },
    { 0xc358b3e3, 34 },
    { 0xdf28b3e4, 90 },
    { 0x8ee8b3e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058b3e8, 76 },
    { 0x6be8b3e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8b3f3, 99 },
    { 0xf0a8b3f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38b3f8, 87 },
    { 0xfb98b3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d8b405, 43 },
    { 0, 0 },
    { 0xa898b407, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f68b40f, 98 },
    { 0xa598b40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7818b413, 81 },
    { 0x4508b414, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf398b41d, 33 },
    { 0, 0 },
    { 0xb298b41f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8b422, 84 },
    { 0x7a68b422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac8b428, 78 },
    { 0, 0 },
    { 0xcf98b42a, 52 },
    { 0xb7b8b42b, 77 },
    { 0, 0 },
    { 0x3d38b42d, 95 },
    { 0x8f38b42e, 33 },
    { 0x9918b42f, 61 },
    { 0, 0 },
    { 0xf9c8b431, 98 },
    { 0, 0 },
    { 0x2318b433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b8b439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c78b43f, 87 },
    { 0x5888b440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6528b443, 34 },
    { 0xe1a8b444, 90 },
    { 0xdc28b443, 90 },
    { 0xd618b444, 90 },
    { 0x1c28b447, 85 },
    { 0x5bc8b443, 61 },
    { 0xede8b449, 90 },
    { 0x3478b44a, 65 },
    { 0x2e68b449, 55 },
    { 0xd2d8b44c, 40 },
    { 0, 0 },
    { 0x95b8b44e, 33 },
    { 0, 0 },
    { 0x4378b450, 90 },
    { 0x2c18b451, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a68b454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad8b45a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918b45f, 95 },
    { 0x9c18b45f, 90 },
    { 0x2c08b461, 55 },
    { 0xca38b45f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe478b465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b46a, 43 },
    { 0, 0 },
    { 0xc2d8b46c, 61 },
    { 0xd578b46d, 61 },
    { 0x32d8b46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7548b477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d8b47a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb8b480, 61 },
    { 0x7568b481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8b489, 52 },
    { 0, 0 },
    { 0x5fe8b48b, 95 },
    { 0xc3c8b48c, 75 },
    { 0, 0 },
    { 0x4e58b48e, 23 },
    { 0, 0 },
    { 0x9d8b490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18b495, 46 },
    { 0, 0 },
    { 0xc068b497, 77 },
    { 0x8db8b498, 95 },
    { 0x86f8b499, 33 },
    { 0x8008b49a, 98 },
    { 0x2e38b49b, 90 },
    { 0x6898b49a, 23 },
    { 0x8408b49d, 76 },
    { 0xf5e8b49d, 90 },
    { 0x8b78b49f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88b4a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca68b4a8, 84 },
    { 0x9788b4a9, 52 },
    { 0xf8c8b4aa, 48 },
    { 0, 0 },
    { 0x7358b4ac, 34 },
    { 0xca68b4ac, 23 },
    { 0, 0 },
    { 0x8038b4af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8b4b4, 55 },
    { 0x24e8b4b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818b4bb, 43 },
    { 0x7568b4bc, 67 },
    { 0, 0 },
    { 0x7648b4be, 61 },
    { 0x3078b4be, 75 },
    { 0x718b4bf, 36 },
    { 0x45c8b4bf, 33 },
    { 0, 0 },
    { 0x37a8b4c3, 61 },
    { 0x9b48b4c4, 89 },
    { 0x86a8b4c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee18b4c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf8b4ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948b4d1, 77 },
    { 0xd358b4d2, 93 },
    { 0x6318b4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2798b4d7, 95 },
    { 0x6ca8b4d8, 76 },
    { 0x9d88b4d9, 93 },
    { 0x27a8b4da, 43 },
    { 0, 0 },
    { 0x8d58b4dc, 84 },
    { 0x7188b4dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c8b4e0, 61 },
    { 0x84e8b4e1, 87 },
    { 0xf458b4e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c8b4e7, 100 },
    { 0, 0 },
    { 0x20c8b4e9, 91 },
    { 0, 0 },
    { 0xbaf8b4eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b4ee, 66 },
    { 0x7d78b4ee, 61 },
    { 0x7f8b4f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe8b4f3, 34 },
    { 0, 0 },
    { 0x6518b4f5, 76 },
    { 0x3358b4f6, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728b4f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e8b4fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d8b506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4298b50c, 87 },
    { 0x8138b50c, 61 },
    { 0, 0 },
    { 0x33d8b50f, 78 },
    { 0, 0 },
    { 0x4d68b511, 33 },
    { 0x4a68b512, 34 },
    { 0xb648b513, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8b51a, 81 },
    { 0, 0 },
    { 0x26a8b51c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478b523, 43 },
    { 0xe528b524, 90 },
    { 0x4fc8b524, 76 },
    { 0x1d48b523, 33 },
    { 0x1ea8b524, 80 },
    { 0, 0 },
    { 0xcd08b529, 23 },
    { 0xa698b529, 33 },
    { 0xdf28b52b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab28b52f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3028b535, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8b53a, 23 },
    { 0x8cc8b53b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8b53e, 90 },
    { 0x28c8b53f, 34 },
    { 0xd718b540, 23 },
    { 0xbf38b540, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8b546, 84 },
    { 0, 0 },
    { 0xc778b548, 95 },
    { 0xab48b549, 33 },
    { 0x3b58b54a, 90 },
    { 0x83d8b549, 84 },
    { 0xaab8b54c, 61 },
    { 0xae38b54d, 61 },
    { 0x598b54b, 34 },
    { 0x6c28b54f, 65 },
    { 0xb508b54e, 68 },
    { 0x5798b551, 61 },
    { 0x42e8b54d, 99 },
    { 0xdb58b54f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad28b559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a8b560, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d8b564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b98b568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348b56e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528b571, 33 },
    { 0xcf58b571, 34 },
    { 0x1aa8b573, 90 },
    { 0, 0 },
    { 0xdcc8b575, 55 },
    { 0xa318b576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc8b57e, 90 },
    { 0, 0 },
    { 0xbfd8b580, 61 },
    { 0, 0 },
    { 0xc298b582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a98b586, 78 },
    { 0xe648b587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1338b58a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2818b599, 76 },
    { 0, 0 },
    { 0xf08b59b, 61 },
    { 0xbcc8b59b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e8b5a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e8b5aa, 77 },
    { 0, 0 },
    { 0x90b8b5ac, 99 },
    { 0xfbd8b5ad, 98 },
    { 0x11d8b5ac, 99 },
    { 0, 0 },
    { 0xfa08b5b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf28b5b4, 52 },
    { 0x5008b5b4, 90 },
    { 0xf818b5b6, 67 },
    { 0xef78b5b7, 61 },
    { 0xf6a8b5b8, 66 },
    { 0x73d8b5b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af8b5be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd68b5c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df8b5c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f98b5cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7738b5d0, 90 },
    { 0x4e88b5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e48b5d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd8b5d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408b5dc, 99 },
    { 0x5f28b5dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b5e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc538b5e7, 90 },
    { 0, 0 },
    { 0x7018b5e9, 98 },
    { 0, 0 },
    { 0x5648b5eb, 98 },
    { 0xa978b5ec, 95 },
    { 0xfa78b5ed, 23 },
    { 0xdb58b5ee, 90 },
    { 0, 0 },
    { 0x9bf8b5f0, 23 },
    { 0xdba8b5f0, 90 },
    { 0xe038b5f0, 91 },
    { 0, 0 },
    { 0xe388b5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5598b5f7, 95 },
    { 0xac08b5f8, 23 },
    { 0, 0 },
    { 0x1418b5fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58b5fe, 23 },
    { 0x798b5ff, 90 },
    { 0xa898b600, 61 },
    { 0xedf8b600, 93 },
    { 0xbcc8b602, 46 },
    { 0x5748b602, 98 },
    { 0xc2d8b604, 90 },
    { 0x9818b604, 93 },
    { 0x6fb8b605, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8b609, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5768b60d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38b611, 90 },
    { 0xb898b612, 90 },
    { 0xf78b613, 33 },
    { 0, 0 },
    { 0xc1f8b615, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8b621, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8b624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8b62c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7498b630, 76 },
    { 0x1e88b631, 43 },
    { 0xfdf8b632, 46 },
    { 0xeda8b630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad78b637, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d28b644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5298b64b, 65 },
    { 0x7d18b64c, 90 },
    { 0, 0 },
    { 0x5058b64e, 23 },
    { 0, 0 },
    { 0x4a98b650, 34 },
    { 0, 0 },
    { 0x2c58b652, 55 },
    { 0, 0 },
    { 0x67c8b654, 98 },
    { 0xe908b654, 23 },
    { 0, 0 },
    { 0x4488b657, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878b65c, 33 },
    { 0x2058b65d, 78 },
    { 0, 0 },
    { 0xca68b65f, 33 },
    { 0x87f8b660, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a8b664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8248b668, 93 },
    { 0x6ac8b668, 36 },
    { 0x5718b66a, 34 },
    { 0x3958b66b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68b670, 55 },
    { 0x518b671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a58b676, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568b67a, 33 },
    { 0x118b67b, 95 },
    { 0xa1a8b67c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c78b680, 90 },
    { 0xf038b680, 33 },
    { 0xd588b681, 95 },
    { 0x81f8b683, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4308b68a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3928b694, 99 },
    { 0, 0 },
    { 0xc118b696, 76 },
    { 0, 0 },
    { 0x47b8b698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x158b69b, 98 },
    { 0x6948b69c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe8b6a0, 90 },
    { 0x1958b6a1, 61 },
    { 0x6408b6a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe208b6a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe8b6ae, 61 },
    { 0x9398b6af, 44 },
    { 0x20c8b6b0, 34 },
    { 0x2898b6ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec8b6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd598b6ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a8b6bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b68b6c2, 90 },
    { 0, 0 },
    { 0xff58b6c4, 95 },
    { 0, 0 },
    { 0xc3c8b6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e8b6ca, 98 },
    { 0x15a8b6ca, 33 },
    { 0xad38b6cc, 35 },
    { 0xc2b8b6cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6098b6d2, 61 },
    { 0xbc58b6d3, 43 },
    { 0x7e48b6d4, 33 },
    { 0, 0 },
    { 0x5188b6d6, 66 },
    { 0xfbc8b6d6, 98 },
    { 0x7be8b6d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478b6de, 85 },
    { 0xb028b6df, 33 },
    { 0x3408b6e0, 76 },
    { 0xe008b6df, 33 },
    { 0x2e58b6e2, 33 },
    { 0x93e8b6e3, 40 },
    { 0x13e8b6e4, 55 },
    { 0xb5c8b6e0, 55 },
    { 0x8dc8b6e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8b6ea, 93 },
    { 0x32c8b6ea, 95 },
    { 0x8d98b6ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a8b6ef, 100 },
    { 0, 0 },
    { 0x2a8b6f1, 43 },
    { 0x6c38b6f2, 98 },
    { 0, 0 },
    { 0xbdd8b6f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c8b6fd, 33 },
    { 0, 0 },
    { 0x50c8b6ff, 96 },
    { 0xaa88b700, 90 },
    { 0x3278b701, 99 },
    { 0xa858b700, 43 },
    { 0x31f8b703, 38 },
    { 0, 0 },
    { 0xb098b705, 90 },
    { 0xd028b705, 75 },
    { 0x8968b707, 98 },
    { 0xf768b707, 43 },
    { 0xb188b707, 100 },
    { 0x5178b707, 95 },
    { 0x7278b70b, 98 },
    { 0xccf8b70c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb798b716, 43 },
    { 0, 0 },
    { 0xa5c8b718, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b8b71b, 100 },
    { 0xcdb8b71c, 76 },
    { 0x9568b71d, 98 },
    { 0x2078b71e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d78b723, 98 },
    { 0xdf8b723, 55 },
    { 0, 0 },
    { 0x17c8b726, 98 },
    { 0, 0 },
    { 0xd3a8b728, 61 },
    { 0x1048b729, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8b72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8b737, 84 },
    { 0x6248b737, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d88b73e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8b741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8b744, 61 },
    { 0, 0 },
    { 0x5678b746, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5978b74b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8b750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb088b753, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8b75c, 61 },
    { 0x4378b75d, 23 },
    { 0xf458b75e, 98 },
    { 0xbef8b75f, 61 },
    { 0, 0 },
    { 0x3b8b761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58b768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8338b76d, 90 },
    { 0x8f28b76d, 75 },
    { 0xe2d8b76e, 85 },
    { 0x2eb8b770, 34 },
    { 0, 0 },
    { 0x39c8b772, 33 },
    { 0xba68b773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea8b776, 95 },
    { 0xbe98b777, 66 },
    { 0, 0 },
    { 0xd068b779, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7188b77f, 76 },
    { 0, 0 },
    { 0xbaf8b781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d8b785, 52 },
    { 0x1718b786, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8b789, 67 },
    { 0, 0 },
    { 0xece8b78b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb8b791, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8b795, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8b798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b8b79d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d8b7a5, 61 },
    { 0xbfa8b7a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea18b7a9, 61 },
    { 0x4cb8b7aa, 84 },
    { 0x44f8b7a9, 95 },
    { 0x4408b7ac, 23 },
    { 0, 0 },
    { 0x5f18b7ae, 95 },
    { 0x27b8b7af, 55 },
    { 0x8508b7b0, 99 },
    { 0x5138b7af, 85 },
    { 0x9808b7b2, 23 },
    { 0x6ab8b7b2, 23 },
    { 0xf068b7b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08b7b7, 52 },
    { 0xb9f8b7b7, 23 },
    { 0, 0 },
    { 0xb4f8b7ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd8b7be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8b7c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb738b7c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba88b7cd, 55 },
    { 0x4cc8b7ce, 61 },
    { 0x3608b7cf, 66 },
    { 0xe918b7d0, 90 },
    { 0, 0 },
    { 0x22b8b7d2, 69 },
    { 0x2e98b7d3, 41 },
    { 0x7e28b7d4, 95 },
    { 0xe388b7d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b7da, 61 },
    { 0, 0 },
    { 0x418b7dc, 90 },
    { 0xe978b7dc, 99 },
    { 0x2658b7de, 98 },
    { 0xfb18b7de, 55 },
    { 0xc1b8b7de, 90 },
    { 0, 0 },
    { 0x7598b7e2, 34 },
    { 0, 0 },
    { 0xacd8b7e4, 55 },
    { 0xaa28b7e5, 95 },
    { 0x4158b7e5, 46 },
    { 0x4218b7e7, 33 },
    { 0x8848b7e8, 61 },
    { 0x86a8b7e9, 95 },
    { 0x7708b7ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8b7ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c8b7f5, 90 },
    { 0x5e48b7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec8b7fa, 55 },
    { 0x3118b7fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb48b802, 23 },
    { 0xc978b803, 95 },
    { 0x91c8b804, 99 },
    { 0, 0 },
    { 0x3278b806, 35 },
    { 0x5058b807, 33 },
    { 0x8908b808, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a8b80d, 93 },
    { 0x6928b80d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c08b814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b819, 61 },
    { 0x23e8b819, 34 },
    { 0, 0 },
    { 0x4b08b81c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8b81f, 90 },
    { 0, 0 },
    { 0x68b821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e38b827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1218b82c, 98 },
    { 0x96b8b82d, 33 },
    { 0x9558b82d, 88 },
    { 0x5b98b82f, 90 },
    { 0xfb38b82e, 23 },
    { 0xdae8b82c, 91 },
    { 0, 0 },
    { 0x2a98b833, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb8b836, 17 },
    { 0x7f68b836, 55 },
    { 0x3068b837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6638b842, 91 },
    { 0xe2e8b843, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398b84b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8b850, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18b854, 67 },
    { 0x9738b855, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88b859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3578b85d, 48 },
    { 0, 0 },
    { 0xf768b85f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a8b862, 61 },
    { 0xdd28b863, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd558b866, 90 },
    { 0x78c8b867, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c8b86a, 33 },
    { 0xd568b86a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8b86f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8b872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c8b875, 90 },
    { 0x42d8b876, 95 },
    { 0xbde8b875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d38b87d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8b882, 91 },
    { 0, 0 },
    { 0x8678b884, 98 },
    { 0, 0 },
    { 0xead8b886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9518b88b, 95 },
    { 0, 0 },
    { 0xe608b88d, 68 },
    { 0xf4c8b88e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab8b893, 76 },
    { 0x3ea8b894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de8b897, 90 },
    { 0xdb38b898, 23 },
    { 0x4788b898, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f8b89f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8b8a2, 23 },
    { 0xdfe8b8a3, 32 },
    { 0xc948b8a3, 23 },
    { 0x9c8b8a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb338b8a8, 90 },
    { 0xba38b8a9, 98 },
    { 0x2e38b8a8, 76 },
    { 0xdfb8b8ab, 100 },
    { 0xc78b8ac, 34 },
    { 0, 0 },
    { 0x2c98b8ae, 98 },
    { 0, 0 },
    { 0x3fe8b8b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8b8b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5788b8c0, 49 },
    { 0, 0 },
    { 0x8668b8c2, 81 },
    { 0x9b28b8c3, 33 },
    { 0x3d78b8c4, 90 },
    { 0xa988b8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538b8cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc8b8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b8d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f8b8db, 23 },
    { 0, 0 },
    { 0xda28b8dd, 23 },
    { 0x89b8b8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808b8e8, 88 },
    { 0x1168b8e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28b8ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18b8f1, 65 },
    { 0xca88b8f2, 66 },
    { 0xdfa8b8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e8b8f6, 98 },
    { 0xfb68b8f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a8b8fa, 61 },
    { 0xb078b8fb, 90 },
    { 0xdc98b8fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa8b900, 76 },
    { 0xc648b901, 34 },
    { 0x5128b901, 46 },
    { 0xcfc8b903, 93 },
    { 0xbfb8b900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe688b907, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2498b90d, 61 },
    { 0xa5f8b90d, 77 },
    { 0x8d8b90d, 23 },
    { 0x9158b90e, 61 },
    { 0xb7a8b911, 33 },
    { 0, 0 },
    { 0x3588b913, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538b918, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8938b91b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc8b91f, 98 },
    { 0, 0 },
    { 0xea98b921, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf8b925, 78 },
    { 0, 0 },
    { 0xe068b927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec8b939, 90 },
    { 0, 0 },
    { 0xa858b93b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec08b943, 34 },
    { 0xc118b943, 99 },
    { 0xc2a8b945, 23 },
    { 0x4e18b945, 23 },
    { 0, 0 },
    { 0xf028b948, 61 },
    { 0, 0 },
    { 0x5cd8b94a, 61 },
    { 0xc038b94a, 90 },
    { 0x19a8b94c, 90 },
    { 0x3688b94c, 90 },
    { 0x23f8b94c, 98 },
    { 0, 0 },
    { 0x33e8b950, 23 },
    { 0, 0 },
    { 0x5248b952, 55 },
    { 0xa978b952, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c8b958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2488b95f, 93 },
    { 0, 0 },
    { 0x7228b961, 68 },
    { 0xc778b961, 23 },
    { 0, 0 },
    { 0x43c8b964, 77 },
    { 0, 0 },
    { 0xb5b8b966, 23 },
    { 0x84f8b966, 88 },
    { 0xca78b968, 78 },
    { 0x57b8b966, 90 },
    { 0xd1f8b967, 91 },
    { 0xe8b96b, 76 },
    { 0x258b96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8b975, 95 },
    { 0x49b8b975, 99 },
    { 0x9778b977, 61 },
    { 0, 0 },
    { 0xc768b979, 61 },
    { 0, 0 },
    { 0xb468b97b, 78 },
    { 0, 0 },
    { 0xb2b8b97d, 68 },
    { 0, 0 },
    { 0x878b97f, 61 },
    { 0x1808b980, 61 },
    { 0x9f18b981, 61 },
    { 0xebe8b981, 76 },
    { 0xc5d8b981, 78 },
    { 0xa98b983, 43 },
    { 0x318b981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a8b98a, 46 },
    { 0x6118b98b, 98 },
    { 0x74c8b98c, 61 },
    { 0xef88b98c, 95 },
    { 0xcb48b98c, 33 },
    { 0xeab8b98f, 23 },
    { 0xb5f8b98f, 40 },
    { 0xffd8b991, 33 },
    { 0xd9e8b98d, 34 },
    { 0, 0 },
    { 0x1038b994, 95 },
    { 0xf7f8b994, 98 },
    { 0, 0 },
    { 0x91f8b997, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd08b99f, 98 },
    { 0, 0 },
    { 0x69f8b9a1, 89 },
    { 0xe288b9a2, 33 },
    { 0xa628b9a3, 68 },
    { 0x7ca8b9a3, 33 },
    { 0x2798b9a5, 76 },
    { 0x1c88b9a6, 90 },
    { 0x6b78b9a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e8b9ab, 33 },
    { 0, 0 },
    { 0x8778b9ad, 99 },
    { 0xe808b9ae, 33 },
    { 0x51c8b9af, 34 },
    { 0x8ea8b9b0, 61 },
    { 0x4a48b9b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8b9b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc8b9bd, 90 },
    { 0x7e8b9be, 78 },
    { 0x3908b9be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b9c4, 33 },
    { 0xcf88b9c5, 90 },
    { 0xcd8b9c6, 33 },
    { 0xacd8b9c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b8b9cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608b9cf, 66 },
    { 0, 0 },
    { 0xc438b9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238b9d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8b9db, 46 },
    { 0x7f28b9db, 95 },
    { 0xd2f8b9dd, 88 },
    { 0x4048b9de, 43 },
    { 0, 0 },
    { 0x6bc8b9e0, 33 },
    { 0x9ca8b9e0, 61 },
    { 0x6908b9e1, 98 },
    { 0x2d98b9e3, 90 },
    { 0x6f48b9e0, 55 },
    { 0x2048b9e5, 43 },
    { 0xfd28b9e6, 84 },
    { 0, 0 },
    { 0x50e8b9e8, 90 },
    { 0, 0 },
    { 0x13e8b9ea, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf8b9ee, 23 },
    { 0x2588b9ef, 23 },
    { 0xbf78b9ee, 75 },
    { 0x6b88b9f1, 100 },
    { 0xdff8b9ef, 61 },
    { 0x7308b9f3, 95 },
    { 0xb878b9f0, 90 },
    { 0, 0 },
    { 0x26c8b9f6, 55 },
    { 0, 0 },
    { 0xd7e8b9f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6588b9fc, 91 },
    { 0x8c68b9fc, 23 },
    { 0xac18b9fe, 90 },
    { 0xa138b9fe, 98 },
    { 0xeb78b9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf058ba08, 90 },
    { 0xc2d8ba08, 98 },
    { 0, 0 },
    { 0x8db8ba0b, 76 },
    { 0xd108ba0b, 90 },
    { 0x448ba0b, 23 },
    { 0xa178ba0b, 55 },
    { 0x24b8ba0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7678ba12, 33 },
    { 0x8c78ba12, 98 },
    { 0xb0e8ba14, 66 },
    { 0, 0 },
    { 0x4588ba16, 91 },
    { 0x1148ba17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f8ba1b, 61 },
    { 0, 0 },
    { 0x96d8ba1d, 90 },
    { 0, 0 },
    { 0x6498ba1f, 90 },
    { 0x3638ba20, 33 },
    { 0, 0 },
    { 0x8e8ba22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b8ba29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3778ba2c, 90 },
    { 0x1938ba2c, 34 },
    { 0, 0 },
    { 0xde08ba2f, 78 },
    { 0, 0 },
    { 0x3a58ba31, 84 },
    { 0, 0 },
    { 0x2d8ba33, 93 },
    { 0xad68ba34, 61 },
    { 0xfda8ba35, 98 },
    { 0xa048ba36, 90 },
    { 0xac78ba37, 34 },
    { 0x8768ba38, 65 },
    { 0, 0 },
    { 0x4958ba3a, 81 },
    { 0x4028ba3a, 33 },
    { 0x67d8ba3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e8ba40, 88 },
    { 0, 0 },
    { 0x9898ba42, 43 },
    { 0, 0 },
    { 0x5f8ba44, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08ba48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8ba4c, 91 },
    { 0x8b8ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8ba50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d8ba53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e28ba59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58ba5d, 59 },
    { 0x398ba5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e8ba64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e8ba67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf58ba6f, 76 },
    { 0x7b98ba6f, 95 },
    { 0x5188ba70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8ba78, 90 },
    { 0x1b38ba79, 99 },
    { 0x4f68ba7a, 33 },
    { 0x5018ba7b, 95 },
    { 0x8838ba7c, 43 },
    { 0x8468ba7a, 100 },
    { 0, 0 },
    { 0xffa8ba7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738ba82, 95 },
    { 0xd48ba82, 95 },
    { 0x21a8ba84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b8ba88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558ba8c, 41 },
    { 0xdda8ba8c, 98 },
    { 0x8698ba8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6418ba91, 87 },
    { 0x8a78ba91, 90 },
    { 0x1048ba93, 98 },
    { 0, 0 },
    { 0xeec8ba95, 43 },
    { 0x33e8ba96, 61 },
    { 0, 0 },
    { 0x13f8ba98, 93 },
    { 0, 0 },
    { 0xd588ba9a, 61 },
    { 0, 0 },
    { 0xd168ba9c, 33 },
    { 0x54e8ba9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288baa0, 78 },
    { 0xe458baa1, 33 },
    { 0xecf8baa1, 61 },
    { 0x9bd8baa3, 43 },
    { 0x8888baa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8baa7, 85 },
    { 0x77d8baa8, 55 },
    { 0, 0 },
    { 0x2f58baaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf958baad, 33 },
    { 0xbf8baae, 17 },
    { 0, 0 },
    { 0x1ee8bab0, 33 },
    { 0x3238bab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5448bab7, 46 },
    { 0xc3c8bab8, 76 },
    { 0xea98bab7, 90 },
    { 0x838bab7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b8babe, 23 },
    { 0xe528babe, 58 },
    { 0, 0 },
    { 0xe248bac1, 90 },
    { 0, 0 },
    { 0x17e8bac3, 81 },
    { 0xb878bac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe378bac9, 23 },
    { 0, 0 },
    { 0x83b8bacb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c8bacf, 99 },
    { 0x2948bad0, 34 },
    { 0x34a8bad0, 33 },
    { 0x4ea8bad2, 75 },
    { 0xdfb8bad3, 33 },
    { 0, 0 },
    { 0x5188bad5, 90 },
    { 0xab58bad5, 90 },
    { 0, 0 },
    { 0xfbb8bad8, 99 },
    { 0xb978bad9, 43 },
    { 0, 0 },
    { 0x6498badb, 90 },
    { 0x2ef8badc, 83 },
    { 0x448badc, 31 },
    { 0x3fb8badd, 76 },
    { 0x67d8bade, 23 },
    { 0xa6b8bae0, 43 },
    { 0xed68bae0, 95 },
    { 0xc828bae2, 33 },
    { 0x8388bae3, 90 },
    { 0xd968bae2, 17 },
    { 0x6f88badc, 90 },
    { 0x4b8bae1, 61 },
    { 0x2538bae7, 76 },
    { 0x3798bae8, 93 },
    { 0xd3b8bae9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef88baee, 85 },
    { 0x7798baef, 88 },
    { 0x7598baef, 90 },
    { 0, 0 },
    { 0x1b58baf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8baf5, 55 },
    { 0x2dd8baf6, 33 },
    { 0x16f8baf7, 97 },
    { 0xab98baf7, 43 },
    { 0xd968baf9, 90 },
    { 0xe618baf9, 33 },
    { 0xcc18baf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe8bb01, 23 },
    { 0, 0 },
    { 0x5668bb03, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb38bb0d, 55 },
    { 0xf818bb0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398bb1b, 95 },
    { 0x3cf8bb1c, 33 },
    { 0xc488bb1d, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7448bb22, 46 },
    { 0xfb68bb23, 99 },
    { 0x84f8bb24, 99 },
    { 0, 0 },
    { 0x5308bb26, 33 },
    { 0, 0 },
    { 0x8188bb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf738bb2d, 33 },
    { 0xd248bb2e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d58bb31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd428bb35, 53 },
    { 0, 0 },
    { 0xef78bb37, 89 },
    { 0, 0 },
    { 0xfd88bb39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd8bb3d, 98 },
    { 0x8d98bb3d, 33 },
    { 0, 0 },
    { 0xeb48bb40, 23 },
    { 0x6968bb41, 43 },
    { 0xfd8bb42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298bb45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad08bb55, 95 },
    { 0, 0 },
    { 0xc218bb57, 98 },
    { 0xd1e8bb58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa598bb5b, 40 },
    { 0x9808bb5b, 23 },
    { 0xfbf8bb5d, 33 },
    { 0xa2c8bb5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4338bb65, 23 },
    { 0, 0 },
    { 0x778bb67, 23 },
    { 0x34b8bb67, 55 },
    { 0xd5e8bb69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8bb6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3768bb70, 95 },
    { 0xe6e8bb71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c88bb83, 67 },
    { 0, 0 },
    { 0xbbc8bb85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd698bb88, 61 },
    { 0x988bb88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468bb8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a8bb93, 81 },
    { 0xdf18bb94, 23 },
    { 0xdb68bb94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8bb99, 53 },
    { 0, 0 },
    { 0x15e8bb9b, 33 },
    { 0, 0 },
    { 0x2dc8bb9d, 33 },
    { 0xace8bb9e, 98 },
    { 0x3ff8bb9e, 34 },
    { 0, 0 },
    { 0x4af8bba1, 61 },
    { 0x6748bba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8358bba8, 33 },
    { 0, 0 },
    { 0xf0b8bbaa, 100 },
    { 0xb928bbab, 95 },
    { 0xdf78bbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8bbaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c98bbb3, 33 },
    { 0x6908bbb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e8bbb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8bbbf, 66 },
    { 0, 0 },
    { 0x7688bbc1, 66 },
    { 0xdfe8bbc2, 99 },
    { 0, 0 },
    { 0x3af8bbc4, 61 },
    { 0, 0 },
    { 0xa548bbc6, 95 },
    { 0xbd8bbc6, 33 },
    { 0xc638bbc8, 24 },
    { 0, 0 },
    { 0x5b98bbca, 33 },
    { 0xc828bbcb, 98 },
    { 0xdce8bbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a8bbd0, 33 },
    { 0x1268bbd0, 40 },
    { 0, 0 },
    { 0x638bbd3, 98 },
    { 0, 0 },
    { 0xcbf8bbd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb458bbd9, 95 },
    { 0xff38bbd9, 61 },
    { 0, 0 },
    { 0x6f18bbdc, 93 },
    { 0x4d08bbdc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628bbe0, 93 },
    { 0x62a8bbe1, 90 },
    { 0, 0 },
    { 0x8998bbe3, 77 },
    { 0x85d8bbe3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838bbe7, 94 },
    { 0xe0d8bbe7, 61 },
    { 0xc368bbe9, 56 },
    { 0xe278bbea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7518bbf1, 76 },
    { 0, 0 },
    { 0xd208bbf3, 93 },
    { 0x3738bbf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e8bbf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8bbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3588bc03, 95 },
    { 0xd518bc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2788bc0b, 77 },
    { 0x5728bc0b, 90 },
    { 0, 0 },
    { 0xba18bc0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef68bc18, 61 },
    { 0x1b58bc18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c28bc1e, 91 },
    { 0, 0 },
    { 0xaf8bc20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf8bc23, 66 },
    { 0x1378bc24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78bc27, 98 },
    { 0, 0 },
    { 0x3038bc29, 90 },
    { 0, 0 },
    { 0x6878bc2b, 33 },
    { 0x4de8bc2c, 76 },
    { 0, 0 },
    { 0x5ce8bc2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb498bc31, 93 },
    { 0, 0 },
    { 0xdf28bc33, 76 },
    { 0x7cf8bc34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c8bc39, 67 },
    { 0x7ff8bc3a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8bc3d, 33 },
    { 0, 0 },
    { 0xaa28bc3f, 61 },
    { 0, 0 },
    { 0xd438bc41, 78 },
    { 0xf328bc41, 69 },
    { 0x5318bc43, 34 },
    { 0x75f8bc43, 95 },
    { 0xedf8bc45, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e58bc49, 43 },
    { 0xca88bc4a, 90 },
    { 0xcba8bc4b, 90 },
    { 0, 0 },
    { 0x9fa8bc4d, 98 },
    { 0xa628bc4d, 95 },
    { 0x4938bc4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5858bc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e98bc55, 23 },
    { 0x1d18bc56, 61 },
    { 0xf3f8bc57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8bc5a, 33 },
    { 0xb8b8bc5a, 46 },
    { 0, 0 },
    { 0x3ec8bc5d, 85 },
    { 0, 0 },
    { 0x9608bc5f, 85 },
    { 0x8248bc60, 34 },
    { 0xf408bc61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08bc64, 61 },
    { 0, 0 },
    { 0x44d8bc66, 90 },
    { 0x54c8bc67, 56 },
    { 0x668bc68, 23 },
    { 0x268bc69, 43 },
    { 0, 0 },
    { 0x23f8bc6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e8bc6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe8bc72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9348bc76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f48bc79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c8bc7f, 95 },
    { 0x6688bc80, 34 },
    { 0, 0 },
    { 0xb408bc82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8388bc85, 66 },
    { 0, 0 },
    { 0x6fa8bc87, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc8bc8c, 66 },
    { 0xf538bc8c, 43 },
    { 0, 0 },
    { 0x4938bc8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd78bc92, 95 },
    { 0x8228bc93, 46 },
    { 0, 0 },
    { 0x4458bc95, 98 },
    { 0x17d8bc95, 52 },
    { 0x4738bc97, 99 },
    { 0x16d8bc95, 61 },
    { 0xe5a8bc99, 52 },
    { 0x9e38bc97, 33 },
    { 0xcb58bc98, 90 },
    { 0xabd8bc9c, 95 },
    { 0x8d18bc9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418bca4, 90 },
    { 0x2738bca4, 53 },
    { 0, 0 },
    { 0xda48bca7, 90 },
    { 0x24f8bca7, 77 },
    { 0x42d8bca8, 33 },
    { 0x6248bca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98bcad, 90 },
    { 0xdd98bcae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b58bcb2, 90 },
    { 0x17b8bcb3, 55 },
    { 0, 0 },
    { 0x1278bcb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68bcba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8bcbd, 93 },
    { 0x8038bcbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a98bcc5, 77 },
    { 0x1fb8bcc6, 68 },
    { 0x35a8bcc7, 23 },
    { 0x5eb8bcc7, 81 },
    { 0x2248bcc9, 93 },
    { 0x4668bcca, 55 },
    { 0xb048bcca, 23 },
    { 0xf728bccc, 90 },
    { 0x43b8bccd, 76 },
    { 0, 0 },
    { 0x6b8bccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d38bcd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8bcd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28bcdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d98bce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438bce8, 78 },
    { 0x78f8bce8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248bcef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d8bcf3, 55 },
    { 0x33f8bcf4, 90 },
    { 0xc178bcf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8bcfa, 52 },
    { 0, 0 },
    { 0xd668bcfc, 33 },
    { 0, 0 },
    { 0x9ae8bcfe, 61 },
    { 0xd5e8bcfe, 91 },
    { 0, 0 },
    { 0x578bd01, 90 },
    { 0xe408bd01, 33 },
    { 0x3638bd03, 98 },
    { 0x28a8bd04, 98 },
    { 0x90a8bd05, 78 },
    { 0x6598bd03, 90 },
    { 0xf1e8bd01, 34 },
    { 0, 0 },
    { 0x5f68bd09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188bd10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8bd17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8bd21, 65 },
    { 0, 0 },
    { 0x9058bd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3428bd26, 76 },
    { 0x4d68bd26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8bd2e, 55 },
    { 0xe6d8bd2f, 61 },
    { 0x1608bd30, 95 },
    { 0, 0 },
    { 0x2b48bd32, 76 },
    { 0x62b8bd32, 66 },
    { 0, 0 },
    { 0xd688bd35, 76 },
    { 0, 0 },
    { 0xfa18bd37, 99 },
    { 0xda18bd38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa458bd3b, 98 },
    { 0x60b8bd3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f38bd3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78bd42, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1498bd49, 77 },
    { 0, 0 },
    { 0x8648bd4b, 98 },
    { 0x43f8bd4c, 23 },
    { 0x6168bd4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad8bd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8bd55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c48bd59, 77 },
    { 0x5188bd5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8bd61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08bd66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d58bd69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8bd6d, 61 },
    { 0xba78bd6e, 91 },
    { 0x2188bd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee08bd74, 43 },
    { 0x6a88bd74, 61 },
    { 0, 0 },
    { 0x3978bd77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4978bd7a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d8bd81, 95 },
    { 0x6de8bd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4138bd85, 98 },
    { 0x4578bd85, 23 },
    { 0x2668bd85, 100 },
    { 0x4d68bd88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148bd8e, 99 },
    { 0x5728bd8f, 43 },
    { 0xe78bd90, 61 },
    { 0x54b8bd8e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c8bd98, 61 },
    { 0xed98bd98, 61 },
    { 0x39b8bd99, 55 },
    { 0xbf88bd99, 33 },
    { 0, 0 },
    { 0x82f8bd9d, 33 },
    { 0xb718bd9d, 98 },
    { 0, 0 },
    { 0xda98bda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8bda8, 23 },
    { 0, 0 },
    { 0x4838bdaa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8bdae, 90 },
    { 0xb288bdae, 41 },
    { 0, 0 },
    { 0x758bdb1, 85 },
    { 0x9a38bdb1, 34 },
    { 0, 0 },
    { 0x74c8bdb4, 90 },
    { 0xdbd8bdb5, 99 },
    { 0xff98bdb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2458bdb9, 90 },
    { 0x1058bdba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc8bdbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2348bdc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848bdc5, 90 },
    { 0xe188bdc5, 85 },
    { 0, 0 },
    { 0xee88bdc8, 91 },
    { 0xc3b8bdc8, 87 },
    { 0, 0 },
    { 0xd9d8bdcb, 77 },
    { 0x5318bdcc, 90 },
    { 0x4f78bdcb, 17 },
    { 0x8088bdcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8bdd7, 99 },
    { 0x5ab8bdd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98bddf, 23 },
    { 0xd6a8bddf, 61 },
    { 0, 0 },
    { 0x2c58bde2, 98 },
    { 0x9a68bde2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e8bdf2, 33 },
    { 0x1e78bdf3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8bdf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9498bdfc, 52 },
    { 0x7b28bdfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3628be04, 91 },
    { 0x9c88be05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba28be12, 85 },
    { 0xeba8be13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d8be19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8be1c, 66 },
    { 0, 0 },
    { 0x1678be1e, 90 },
    { 0x3478be1f, 90 },
    { 0, 0 },
    { 0x3768be21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3258be26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968be2c, 33 },
    { 0, 0 },
    { 0xd398be2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8be31, 33 },
    { 0x7948be32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78be35, 99 },
    { 0xafd8be36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc68be3f, 100 },
    { 0x8ac8be40, 61 },
    { 0x58a8be41, 61 },
    { 0xdf08be40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef8be46, 90 },
    { 0x3348be47, 96 },
    { 0, 0 },
    { 0x9d38be49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3428be50, 90 },
    { 0xf888be50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8be55, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408be59, 23 },
    { 0xcc18be59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c8be5d, 61 },
    { 0xd7c8be5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1018be63, 46 },
    { 0, 0 },
    { 0x5128be65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd8be6b, 51 },
    { 0xc48be6c, 64 },
    { 0x46c8be6d, 95 },
    { 0x49e8be6d, 84 },
    { 0x5c8be6b, 61 },
    { 0xd528be6f, 76 },
    { 0x3848be6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a8be75, 66 },
    { 0x25a8be76, 55 },
    { 0x2788be75, 33 },
    { 0xa4d8be78, 90 },
    { 0x28f8be79, 87 },
    { 0x9448be7a, 23 },
    { 0xdf38be79, 52 },
    { 0, 0 },
    { 0x52c8be7d, 75 },
    { 0x6178be7e, 100 },
    { 0xdab8be7f, 33 },
    { 0x7f88be80, 23 },
    { 0x4e8be80, 90 },
    { 0x1d98be81, 43 },
    { 0x54b8be83, 90 },
    { 0x9d98be84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd8be89, 61 },
    { 0, 0 },
    { 0x7a58be8b, 65 },
    { 0xbf38be8c, 76 },
    { 0xadb8be8d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f8be94, 93 },
    { 0x408be95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a98be9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8be9d, 78 },
    { 0x3848be9d, 43 },
    { 0xde88be9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478bea4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8bead, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb8beb0, 90 },
    { 0x19f8beb1, 76 },
    { 0xf038beb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8beba, 38 },
    { 0x41d8bebb, 91 },
    { 0xd118bebc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3248bec1, 98 },
    { 0xfac8bec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608becd, 23 },
    { 0x9df8bece, 52 },
    { 0xff48becd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778bed4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1448bedb, 91 },
    { 0x9d98bedc, 81 },
    { 0xd758bedd, 98 },
    { 0xe508bedc, 34 },
    { 0x9d68bedb, 43 },
    { 0, 0 },
    { 0x5708bee1, 78 },
    { 0x76a8bee2, 98 },
    { 0x78bee3, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9278bee7, 43 },
    { 0, 0 },
    { 0x2b98bee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb28beed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478bef3, 55 },
    { 0, 0 },
    { 0x1748bef5, 88 },
    { 0x7d28bef6, 55 },
    { 0x1f28bef7, 61 },
    { 0xc3b8bef8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7728befc, 77 },
    { 0x16d8befd, 34 },
    { 0, 0 },
    { 0x6088beff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f8bf04, 95 },
    { 0, 0 },
    { 0x24d8bf06, 23 },
    { 0xc268bf07, 95 },
    { 0xc308bf08, 98 },
    { 0xc508bf09, 66 },
    { 0xfe88bf06, 99 },
    { 0xa308bf06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae08bf11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac8bf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed18bf17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08bf1b, 87 },
    { 0, 0 },
    { 0x2478bf1d, 65 },
    { 0x1bd8bf1e, 90 },
    { 0, 0 },
    { 0x3838bf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e8bf23, 66 },
    { 0x55b8bf24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x618bf27, 33 },
    { 0x5f68bf28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28bf2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8bf32, 78 },
    { 0xace8bf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748bf3e, 23 },
    { 0, 0 },
    { 0xfc08bf40, 76 },
    { 0x3ab8bf41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e8bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a8bf47, 33 },
    { 0xda08bf48, 23 },
    { 0x6348bf48, 90 },
    { 0x9078bf49, 46 },
    { 0, 0 },
    { 0xfb08bf4c, 91 },
    { 0xed88bf4d, 33 },
    { 0x4b88bf4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d8bf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb598bf5c, 61 },
    { 0, 0 },
    { 0xa408bf5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368bf61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e08bf65, 23 },
    { 0xe438bf66, 93 },
    { 0x10d8bf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e8bf6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908bf6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898bf73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9648bf76, 88 },
    { 0, 0 },
    { 0xbf48bf78, 33 },
    { 0xf38bf79, 95 },
    { 0xca68bf7a, 90 },
    { 0x4428bf79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8bf80, 23 },
    { 0x1078bf81, 61 },
    { 0x9588bf82, 76 },
    { 0xde28bf81, 76 },
    { 0, 0 },
    { 0x9f68bf85, 55 },
    { 0, 0 },
    { 0x30d8bf87, 90 },
    { 0xb328bf88, 23 },
    { 0, 0 },
    { 0x7858bf8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b8bf90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18bf95, 67 },
    { 0x3038bf96, 85 },
    { 0xdd88bf95, 23 },
    { 0x51d8bf96, 23 },
    { 0x50a8bf99, 98 },
    { 0x6338bf98, 43 },
    { 0x7098bf9a, 95 },
    { 0xb998bf9c, 42 },
    { 0x22a8bf9c, 90 },
    { 0xf68bf9e, 33 },
    { 0xb158bf9f, 95 },
    { 0x34b8bf9d, 76 },
    { 0xd328bfa0, 80 },
    { 0x398bfa2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b48bfa6, 99 },
    { 0x75e8bfa7, 85 },
    { 0, 0 },
    { 0x348bfa9, 91 },
    { 0x2618bfaa, 61 },
    { 0x7e98bfaa, 17 },
    { 0x3fc8bfac, 61 },
    { 0x1828bfa9, 33 },
    { 0xc288bfa9, 94 },
    { 0x6188bfaf, 90 },
    { 0xe708bfaf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8bfb3, 75 },
    { 0xbb68bfb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa8bfb8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa38bfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed8bfbf, 67 },
    { 0x1b78bfc0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3198bfc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c8bfca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d88bfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28bfd0, 69 },
    { 0, 0 },
    { 0xd8f8bfd2, 61 },
    { 0, 0 },
    { 0x768bfd4, 99 },
    { 0, 0 },
    { 0x1c08bfd6, 34 },
    { 0, 0 },
    { 0xb368bfd8, 68 },
    { 0xc1d8bfd9, 61 },
    { 0xc2e8bfd9, 98 },
    { 0xbca8bfd8, 81 },
    { 0xde98bfdc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8578bfe2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2138bfe6, 99 },
    { 0, 0 },
    { 0x1388bfe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d8bfef, 23 },
    { 0xa6c8bfef, 95 },
    { 0xc5f8bff1, 46 },
    { 0xa3c8bff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc128bff5, 61 },
    { 0xe8c8bff6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8bff9, 95 },
    { 0x2d28bffa, 48 },
    { 0, 0 },
    { 0x5698bffc, 90 },
    { 0x81b8bffc, 67 },
    { 0, 0 },
    { 0xc9a8bfff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b28c004, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea8c007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68c00b, 90 },
    { 0xe88c00c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x728c00f, 95 },
    { 0x62d8c00f, 36 },
    { 0, 0 },
    { 0xa228c012, 46 },
    { 0x3d78c013, 90 },
    { 0xa4d8c014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18c01b, 83 },
    { 0x3108c01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a8c029, 87 },
    { 0, 0 },
    { 0xfa8c02b, 99 },
    { 0xb5c8c02c, 98 },
    { 0xbde8c02c, 78 },
    { 0xc2d8c02c, 33 },
    { 0xb318c02f, 95 },
    { 0xd508c030, 93 },
    { 0x9958c031, 98 },
    { 0x1d18c032, 98 },
    { 0, 0 },
    { 0x35a8c034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938c037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28c03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa498c040, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc318c044, 55 },
    { 0x3f48c045, 99 },
    { 0, 0 },
    { 0x9c68c047, 93 },
    { 0x85f8c047, 33 },
    { 0, 0 },
    { 0x3758c04a, 90 },
    { 0x1698c04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7088c051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3658c057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec98c05a, 66 },
    { 0xfdc8c05a, 43 },
    { 0xed08c05c, 91 },
    { 0xdb08c05d, 61 },
    { 0, 0 },
    { 0x16c8c05f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b8c069, 61 },
    { 0xf188c06a, 90 },
    { 0x258c06b, 61 },
    { 0x9308c06c, 23 },
    { 0x4fa8c06d, 34 },
    { 0xbf08c06d, 33 },
    { 0xf0c8c06e, 55 },
    { 0x7c58c070, 33 },
    { 0, 0 },
    { 0xbbc8c072, 98 },
    { 0x9028c072, 98 },
    { 0x60b8c074, 90 },
    { 0xb258c074, 83 },
    { 0x9798c072, 76 },
    { 0x70d8c077, 33 },
    { 0xae8c076, 61 },
    { 0, 0 },
    { 0x72c8c07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc38c07e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf278c084, 77 },
    { 0, 0 },
    { 0xb628c086, 33 },
    { 0x2028c087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c8c08a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8168c08e, 67 },
    { 0x4bf8c08e, 46 },
    { 0xfe98c08f, 95 },
    { 0x3d88c091, 52 },
    { 0x438c08e, 91 },
    { 0x26e8c090, 33 },
    { 0x95a8c094, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb008c099, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5098c0a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc958c0a8, 61 },
    { 0x8498c0a8, 83 },
    { 0x9ae8c0a8, 33 },
    { 0xf0f8c0ab, 84 },
    { 0xc918c0ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8c0af, 23 },
    { 0x12d8c0b0, 33 },
    { 0, 0 },
    { 0x4bb8c0b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798c0b5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8c0ba, 23 },
    { 0xb3a8c0ba, 99 },
    { 0x1c98c0ba, 90 },
    { 0xa728c0ba, 61 },
    { 0x6078c0bc, 95 },
    { 0xfd8c0bf, 61 },
    { 0xb2a8c0bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28c0c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c8c0ce, 46 },
    { 0x2808c0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c78c0d3, 23 },
    { 0xfad8c0d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa598c0da, 33 },
    { 0x3d08c0db, 97 },
    { 0, 0 },
    { 0x3168c0dd, 98 },
    { 0x6148c0de, 91 },
    { 0xdd38c0dd, 33 },
    { 0x638c0e0, 90 },
    { 0x7e08c0e1, 84 },
    { 0xfd58c0e2, 98 },
    { 0x23e8c0e1, 95 },
    { 0xabf8c0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5938c0e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8c0ef, 76 },
    { 0xd298c0f0, 90 },
    { 0, 0 },
    { 0x8f68c0f2, 93 },
    { 0xeaa8c0f3, 90 },
    { 0xbb58c0f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd8c0f9, 80 },
    { 0, 0 },
    { 0x5d98c0fb, 61 },
    { 0x5a38c0fc, 99 },
    { 0x4068c0fb, 90 },
    { 0, 0 },
    { 0x7968c0ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98c103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368c109, 91 },
    { 0x7228c10a, 90 },
    { 0x3418c10b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138c10f, 84 },
    { 0x1b58c110, 84 },
    { 0x64e8c111, 61 },
    { 0xa768c110, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f38c116, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc8c11a, 94 },
    { 0, 0 },
    { 0x3678c11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c122, 90 },
    { 0x9688c123, 23 },
    { 0xfb88c123, 95 },
    { 0x7808c125, 46 },
    { 0xd018c125, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8c129, 52 },
    { 0, 0 },
    { 0x6718c12b, 95 },
    { 0x2bc8c12b, 98 },
    { 0, 0 },
    { 0x9678c12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6778c138, 33 },
    { 0, 0 },
    { 0x76a8c13a, 76 },
    { 0x2468c13b, 95 },
    { 0x3538c13c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638c13f, 43 },
    { 0, 0 },
    { 0x398c141, 60 },
    { 0, 0 },
    { 0x9f08c143, 90 },
    { 0xff58c144, 76 },
    { 0, 0 },
    { 0x7f18c146, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d48c149, 88 },
    { 0x8f18c149, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b8c153, 33 },
    { 0xbb58c154, 69 },
    { 0x5208c154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68c164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b8c168, 95 },
    { 0, 0 },
    { 0xb228c16a, 88 },
    { 0, 0 },
    { 0xaa18c16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8c171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6018c174, 95 },
    { 0x5ca8c174, 90 },
    { 0xf6d8c176, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8c17a, 56 },
    { 0x70f8c17b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff8c17e, 61 },
    { 0, 0 },
    { 0xdd08c180, 23 },
    { 0xb98c181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd368c185, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa8c188, 66 },
    { 0xb898c189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78c18c, 90 },
    { 0, 0 },
    { 0x9598c18e, 52 },
    { 0x7598c18e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad98c198, 98 },
    { 0xea28c198, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee8c19c, 90 },
    { 0x17c8c19c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8c1a1, 46 },
    { 0x3458c1a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88c1a6, 95 },
    { 0x5288c1a7, 93 },
    { 0x6318c1a7, 88 },
    { 0x7cd8c1a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2818c1b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa078c1bb, 95 },
    { 0xbb08c1bc, 90 },
    { 0x908c1bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9388c1c4, 95 },
    { 0x7a38c1c5, 90 },
    { 0x32f8c1c5, 19 },
    { 0xd6f8c1c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f8c1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc18c1d0, 33 },
    { 0x42b8c1d1, 95 },
    { 0x7258c1d1, 85 },
    { 0xe9a8c1d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9568c1d8, 66 },
    { 0x4bf8c1d8, 33 },
    { 0x97d8c1da, 23 },
    { 0xcdf8c1db, 61 },
    { 0x3e08c1dc, 17 },
    { 0x8058c1dc, 95 },
    { 0xa9a8c1dc, 90 },
    { 0x8318c1dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb618c1e2, 61 },
    { 0x3d68c1e3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe378c1e9, 65 },
    { 0x2878c1ea, 61 },
    { 0xd278c1ea, 61 },
    { 0, 0 },
    { 0x3b18c1ed, 34 },
    { 0x6558c1ee, 43 },
    { 0x3de8c1ef, 85 },
    { 0xdcb8c1ef, 76 },
    { 0, 0 },
    { 0xdde8c1f2, 95 },
    { 0x1368c1f3, 76 },
    { 0xa6c8c1f4, 95 },
    { 0, 0 },
    { 0xd148c1f6, 90 },
    { 0, 0 },
    { 0x74e8c1f8, 23 },
    { 0xdab8c1f9, 90 },
    { 0x7148c1f8, 90 },
    { 0, 0 },
    { 0x33d8c1fc, 61 },
    { 0x32a8c1fd, 55 },
    { 0, 0 },
    { 0xb8e8c1ff, 23 },
    { 0, 0 },
    { 0xcbe8c201, 17 },
    { 0x1a38c202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac8c205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5338c20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc8c211, 98 },
    { 0x8b38c212, 76 },
    { 0x1b08c212, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c58c217, 15 },
    { 0x1378c218, 98 },
    { 0x10b8c218, 95 },
    { 0, 0 },
    { 0xf3a8c21b, 76 },
    { 0, 0 },
    { 0x368c21d, 95 },
    { 0x128c21d, 76 },
    { 0, 0 },
    { 0x248c220, 98 },
    { 0xd4d8c221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f8c224, 100 },
    { 0x9d48c225, 100 },
    { 0x4e38c226, 2 },
    { 0x5438c227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed8c22a, 95 },
    { 0x8a8c22a, 95 },
    { 0x7c08c22a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878c230, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9528c234, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d8c239, 33 },
    { 0x2488c23a, 43 },
    { 0x1a28c23a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f8c241, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd58c247, 91 },
    { 0xaaf8c248, 95 },
    { 0, 0 },
    { 0x498c24a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9658c254, 61 },
    { 0xd998c254, 90 },
    { 0x5408c255, 99 },
    { 0xed38c257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb538c25c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7878c25f, 78 },
    { 0x9c58c260, 17 },
    { 0x9448c261, 100 },
    { 0x8158c262, 93 },
    { 0x1788c260, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1258c266, 90 },
    { 0xda38c267, 34 },
    { 0, 0 },
    { 0x1f08c269, 61 },
    { 0x5288c269, 90 },
    { 0x3ca8c26b, 34 },
    { 0x6178c26c, 95 },
    { 0x1618c26d, 67 },
    { 0x9948c26e, 90 },
    { 0x5758c26f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d48c275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438c27a, 23 },
    { 0, 0 },
    { 0xaf88c27c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd088c282, 38 },
    { 0x2a48c283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc78c286, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8948c28a, 90 },
    { 0x6b28c28b, 61 },
    { 0, 0 },
    { 0xfe38c28d, 100 },
    { 0, 0 },
    { 0x278c28f, 33 },
    { 0x82b8c28f, 95 },
    { 0, 0 },
    { 0x9298c292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8c29e, 66 },
    { 0, 0 },
    { 0x87c8c2a0, 66 },
    { 0x53a8c2a0, 98 },
    { 0xf488c2a2, 76 },
    { 0, 0 },
    { 0xef38c2a4, 99 },
    { 0xfbc8c2a5, 65 },
    { 0x418c2a5, 87 },
    { 0, 0 },
    { 0xdc28c2a8, 40 },
    { 0, 0 },
    { 0xfa38c2aa, 98 },
    { 0x1568c2aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2548c2ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718c2b6, 90 },
    { 0xdd88c2b7, 76 },
    { 0xa148c2b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f48c2bd, 23 },
    { 0xc8c8c2bd, 99 },
    { 0, 0 },
    { 0xb428c2c0, 55 },
    { 0, 0 },
    { 0x6e88c2c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd788c2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878c2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28c2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed78c2d0, 93 },
    { 0, 0 },
    { 0xe5e8c2d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8c2d6, 77 },
    { 0xadc8c2d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d28c2db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738c2de, 61 },
    { 0, 0 },
    { 0xdde8c2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4718c2e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958c2ec, 76 },
    { 0xfe28c2ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a98c2f1, 52 },
    { 0xb868c2f2, 95 },
    { 0x3b58c2f2, 90 },
    { 0x1d78c2f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c2f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8c300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f8c305, 23 },
    { 0xd7c8c306, 23 },
    { 0xee58c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b8c30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9928c30e, 90 },
    { 0, 0 },
    { 0x6a78c310, 91 },
    { 0x9d28c311, 95 },
    { 0x91c8c311, 43 },
    { 0xfe78c311, 55 },
    { 0x2918c314, 90 },
    { 0x4dc8c315, 98 },
    { 0x3a88c316, 95 },
    { 0xe888c315, 98 },
    { 0xf08c318, 33 },
    { 0, 0 },
    { 0x70a8c31a, 85 },
    { 0, 0 },
    { 0xdcb8c31c, 95 },
    { 0, 0 },
    { 0x85e8c31e, 90 },
    { 0x88c31f, 43 },
    { 0x6118c320, 55 },
    { 0, 0 },
    { 0xc4d8c322, 34 },
    { 0xff78c323, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c327, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7238c32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7698c331, 23 },
    { 0x8f18c331, 43 },
    { 0, 0 },
    { 0x3348c334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8c338, 90 },
    { 0x5848c339, 33 },
    { 0, 0 },
    { 0x4168c33b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4138c341, 23 },
    { 0xb458c341, 76 },
    { 0x88c8c341, 66 },
    { 0x85a8c344, 23 },
    { 0, 0 },
    { 0x7bf8c346, 88 },
    { 0x19f8c347, 90 },
    { 0xa7c8c348, 33 },
    { 0, 0 },
    { 0xb8c8c34a, 77 },
    { 0xda18c34b, 69 },
    { 0xf6f8c34c, 98 },
    { 0x21a8c34d, 33 },
    { 0x5708c34e, 90 },
    { 0xe968c34f, 33 },
    { 0x4e08c350, 76 },
    { 0xe2b8c351, 98 },
    { 0x59e8c351, 61 },
    { 0xfd8c350, 80 },
    { 0x5f68c351, 46 },
    { 0x3ed8c355, 61 },
    { 0xe8f8c356, 76 },
    { 0xa808c34b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98c35a, 84 },
    { 0xb68c35b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8c361, 61 },
    { 0, 0 },
    { 0x3b08c363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928c369, 90 },
    { 0xae78c36a, 23 },
    { 0x1d58c36a, 65 },
    { 0x3778c36c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8378c374, 93 },
    { 0, 0 },
    { 0xe498c376, 95 },
    { 0, 0 },
    { 0x6ec8c378, 90 },
    { 0, 0 },
    { 0xd1f8c37a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8c37e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018c381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788c385, 75 },
    { 0x94d8c385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b18c38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98c38f, 99 },
    { 0x4728c390, 53 },
    { 0xcb98c391, 98 },
    { 0x80e8c390, 33 },
    { 0, 0 },
    { 0x8108c394, 9 },
    { 0x7f58c395, 8 },
    { 0x8ff8c396, 90 },
    { 0x27c8c397, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148c39b, 55 },
    { 0, 0 },
    { 0x8bc8c39d, 91 },
    { 0x588c39d, 33 },
    { 0xfdc8c39d, 91 },
    { 0xeeb8c3a0, 65 },
    { 0xb388c3a1, 61 },
    { 0xd368c39f, 84 },
    { 0xc478c3a3, 33 },
    { 0x2758c3a3, 98 },
    { 0x6978c3a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd38c3ae, 76 },
    { 0, 0 },
    { 0x8ea8c3b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8c3b6, 61 },
    { 0x6828c3b6, 55 },
    { 0xa668c3b8, 87 },
    { 0xd128c3b7, 23 },
    { 0xfeb8c3ba, 76 },
    { 0, 0 },
    { 0x1478c3bc, 98 },
    { 0xc78c3bd, 90 },
    { 0x3728c3bc, 85 },
    { 0xf0d8c3be, 33 },
    { 0xfd28c3c0, 90 },
    { 0x1eb8c3c1, 33 },
    { 0xc58c3c2, 53 },
    { 0x3a38c3c3, 95 },
    { 0x85a8c3c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8c3c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8c3cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68c3cf, 76 },
    { 0, 0 },
    { 0x4218c3d1, 98 },
    { 0xb268c3d2, 23 },
    { 0x70c8c3d3, 33 },
    { 0, 0 },
    { 0xd768c3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a68c3d9, 78 },
    { 0, 0 },
    { 0x6948c3db, 90 },
    { 0x7fc8c3dc, 23 },
    { 0xd8c3db, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec8c3e0, 33 },
    { 0x9d18c3e0, 61 },
    { 0xa948c3e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8c3e5, 90 },
    { 0xf118c3e6, 61 },
    { 0x9f08c3e7, 75 },
    { 0x6c38c3e8, 100 },
    { 0, 0 },
    { 0xe138c3ea, 61 },
    { 0xb578c3eb, 33 },
    { 0, 0 },
    { 0x1358c3ed, 61 },
    { 0, 0 },
    { 0xe468c3ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8c3f2, 90 },
    { 0, 0 },
    { 0x3fa8c3f4, 76 },
    { 0, 0 },
    { 0xd018c3f6, 43 },
    { 0xc208c3f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd88c3fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd648c3ff, 61 },
    { 0xb7c8c400, 81 },
    { 0xd08c401, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8c404, 99 },
    { 0xe428c405, 98 },
    { 0xc098c404, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f8c40e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08c414, 98 },
    { 0xbdf8c414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78c418, 78 },
    { 0x23b8c418, 33 },
    { 0x9338c41a, 76 },
    { 0x4388c418, 95 },
    { 0, 0 },
    { 0x9718c41d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a8c425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9578c428, 76 },
    { 0xe728c429, 81 },
    { 0x4738c429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc8c430, 98 },
    { 0x54b8c431, 95 },
    { 0, 0 },
    { 0xdf78c433, 40 },
    { 0x2408c434, 98 },
    { 0xb238c435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd8c43a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8c440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c447, 34 },
    { 0x7f68c448, 61 },
    { 0xc058c449, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc308c44f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a8c454, 78 },
    { 0x8368c454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b8c45b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158c45f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798c469, 23 },
    { 0x518c469, 90 },
    { 0xfed8c46b, 53 },
    { 0xb168c469, 75 },
    { 0x7f58c46d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9228c471, 77 },
    { 0x1f38c472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc8c475, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338c479, 90 },
    { 0xc428c47a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7178c47d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca8c481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe648c485, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388c488, 78 },
    { 0, 0 },
    { 0x61c8c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d8c48d, 76 },
    { 0x578c48d, 23 },
    { 0x4848c48e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8c493, 98 },
    { 0, 0 },
    { 0x6758c495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb8c499, 93 },
    { 0, 0 },
    { 0x9998c49b, 95 },
    { 0, 0 },
    { 0x2748c49d, 55 },
    { 0, 0 },
    { 0x3b78c49f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc678c4a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc8c4ad, 90 },
    { 0xd1d8c4ae, 23 },
    { 0xb118c4ae, 23 },
    { 0xb2b8c4ad, 55 },
    { 0x5508c4b1, 76 },
    { 0, 0 },
    { 0xe598c4b3, 76 },
    { 0x3848c4b4, 23 },
    { 0xcce8c4b5, 33 },
    { 0xbba8c4b5, 91 },
    { 0, 0 },
    { 0xaa08c4b8, 55 },
    { 0x7508c4b9, 76 },
    { 0x4ac8c4ba, 101 },
    { 0xb3c8c4bb, 98 },
    { 0, 0 },
    { 0xbef8c4bd, 67 },
    { 0xd448c4bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae8c4c3, 61 },
    { 0x72d8c4c3, 90 },
    { 0x6068c4c5, 61 },
    { 0x7e8c4c6, 55 },
    { 0xb0e8c4c7, 90 },
    { 0x6c88c4c8, 61 },
    { 0x6cc8c4c4, 93 },
    { 0xf788c4c3, 99 },
    { 0x7a08c4cb, 84 },
    { 0x1438c4cb, 90 },
    { 0x5d68c4cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa78c4d3, 99 },
    { 0, 0 },
    { 0x4ec8c4d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a38c4d8, 90 },
    { 0xc808c4d9, 33 },
    { 0xcfe8c4da, 46 },
    { 0x9408c4db, 90 },
    { 0xc3f8c4dc, 87 },
    { 0xda08c4dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8c4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8c4e3, 84 },
    { 0, 0 },
    { 0xeea8c4e5, 95 },
    { 0xbe98c4e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798c4eb, 33 },
    { 0x33a8c4ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48c4ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae8c4f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c38c4f8, 75 },
    { 0x8138c4f9, 76 },
    { 0xb228c4f9, 95 },
    { 0x7ed8c4fb, 95 },
    { 0x78b8c4fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab8c500, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d8c505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538c50b, 76 },
    { 0x7558c50b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3758c512, 100 },
    { 0xc638c513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388c516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8c51f, 80 },
    { 0x8ab8c520, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8c523, 91 },
    { 0, 0 },
    { 0xa368c525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc548c52f, 90 },
    { 0x4898c52f, 46 },
    { 0, 0 },
    { 0xe638c532, 91 },
    { 0x31c8c532, 84 },
    { 0x1e8c534, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8c538, 95 },
    { 0, 0 },
    { 0xf6d8c53a, 34 },
    { 0xae88c53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a8c53e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f78c542, 98 },
    { 0x4028c543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478c546, 61 },
    { 0x3f8c547, 95 },
    { 0x7168c547, 33 },
    { 0xbcf8c549, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa438c54c, 100 },
    { 0xb2e8c54d, 77 },
    { 0, 0 },
    { 0xc08c54f, 95 },
    { 0x7be8c54f, 95 },
    { 0, 0 },
    { 0x94a8c552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c555, 90 },
    { 0, 0 },
    { 0xe498c557, 23 },
    { 0x388c558, 98 },
    { 0xdc38c559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868c55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3368c561, 78 },
    { 0x738c562, 33 },
    { 0x3ce8c562, 55 },
    { 0, 0 },
    { 0x9488c565, 95 },
    { 0, 0 },
    { 0xf5c8c567, 23 },
    { 0xf3a8c568, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78c56d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4118c570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98c573, 98 },
    { 0x7d48c574, 90 },
    { 0xcc68c575, 76 },
    { 0xe048c576, 56 },
    { 0xd048c577, 95 },
    { 0, 0 },
    { 0x68e8c579, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d18c580, 34 },
    { 0xe428c581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf78c584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec8c58c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc568c591, 34 },
    { 0xfaf8c591, 33 },
    { 0x3bb8c593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece8c597, 85 },
    { 0x15a8c597, 76 },
    { 0, 0 },
    { 0x6388c59a, 93 },
    { 0x9058c59b, 43 },
    { 0, 0 },
    { 0xc078c59d, 61 },
    { 0, 0 },
    { 0x7418c59f, 55 },
    { 0xb0b8c59f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd8c5a3, 90 },
    { 0xcc68c5a4, 61 },
    { 0x83e8c5a5, 76 },
    { 0x9568c5a5, 61 },
    { 0xf9b8c5a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df8c5b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388c5b7, 90 },
    { 0x6ae8c5b8, 61 },
    { 0, 0 },
    { 0x5208c5ba, 17 },
    { 0xb98c5bb, 33 },
    { 0x7be8c5ba, 66 },
    { 0x1ab8c5ba, 36 },
    { 0xc88c5be, 83 },
    { 0x4278c5bd, 90 },
    { 0x6c18c5bc, 93 },
    { 0x5cc8c5c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838c5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908c5cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8c5d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828c5d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8c5da, 98 },
    { 0x6158c5db, 91 },
    { 0x1ee8c5dc, 95 },
    { 0, 0 },
    { 0xed38c5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a8c5e4, 99 },
    { 0xd38c5e5, 98 },
    { 0, 0 },
    { 0x5f68c5e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c8c5ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8c5f3, 93 },
    { 0, 0 },
    { 0x69b8c5f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b8c5f9, 55 },
    { 0, 0 },
    { 0xc938c5fb, 95 },
    { 0x7d48c5fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce98c600, 95 },
    { 0x2368c601, 95 },
    { 0x7738c602, 85 },
    { 0, 0 },
    { 0xbc8c604, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608c608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58c613, 55 },
    { 0x2968c613, 66 },
    { 0x9198c613, 78 },
    { 0xf878c616, 34 },
    { 0, 0 },
    { 0x5b68c618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528c61d, 23 },
    { 0xf198c61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88c622, 33 },
    { 0, 0 },
    { 0xb1c8c624, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8c630, 43 },
    { 0, 0 },
    { 0x10e8c632, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08c636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf68c63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8c644, 76 },
    { 0x57e8c644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac78c648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6238c64c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc58c652, 69 },
    { 0, 0 },
    { 0x72e8c654, 99 },
    { 0xdc88c655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c659, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb8c65c, 61 },
    { 0xce58c65c, 91 },
    { 0x6878c65e, 33 },
    { 0xf688c65d, 67 },
    { 0x9af8c660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc378c666, 33 },
    { 0x3e88c667, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b8c66a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2568c673, 96 },
    { 0x56d8c674, 78 },
    { 0, 0 },
    { 0xbb08c676, 61 },
    { 0xc4c8c676, 98 },
    { 0xb88c678, 33 },
    { 0xf3b8c677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9598c680, 85 },
    { 0xf708c681, 38 },
    { 0, 0 },
    { 0xae18c683, 23 },
    { 0, 0 },
    { 0x50d8c685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98c68b, 76 },
    { 0x8b8c68c, 33 },
    { 0x6e08c68c, 33 },
    { 0xc978c68e, 95 },
    { 0x82e8c68e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058c699, 33 },
    { 0x2178c69a, 55 },
    { 0x8228c69a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb8c69f, 66 },
    { 0x6358c6a0, 78 },
    { 0, 0 },
    { 0xa48c6a2, 43 },
    { 0, 0 },
    { 0xf408c6a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e78c6ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe138c6b1, 23 },
    { 0xffa8c6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6598c6b8, 61 },
    { 0x9a28c6b9, 23 },
    { 0xa5d8c6ba, 42 },
    { 0xd088c6b8, 90 },
    { 0xbcf8c6bc, 61 },
    { 0xd968c6bd, 17 },
    { 0xfda8c6be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf448c6c2, 43 },
    { 0xc1e8c6c2, 90 },
    { 0xa178c6c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf78c6cb, 44 },
    { 0x51b8c6cc, 66 },
    { 0xcf38c6cd, 55 },
    { 0xd178c6cd, 55 },
    { 0x40d8c6cb, 43 },
    { 0x3878c6cd, 34 },
    { 0x2078c6ce, 55 },
    { 0, 0 },
    { 0x8b48c6d3, 76 },
    { 0x8898c6d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748c6dc, 61 },
    { 0, 0 },
    { 0x17b8c6de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7568c6e5, 85 },
    { 0, 0 },
    { 0xcdb8c6e7, 95 },
    { 0x1a48c6e7, 91 },
    { 0x9e8c6e9, 33 },
    { 0, 0 },
    { 0x38a8c6eb, 76 },
    { 0xdce8c6ec, 23 },
    { 0x4648c6ed, 85 },
    { 0, 0 },
    { 0x7aa8c6ef, 90 },
    { 0x52a8c6f0, 90 },
    { 0xa668c6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28c6f8, 86 },
    { 0xb9a8c6f8, 33 },
    { 0x28d8c6f9, 43 },
    { 0, 0 },
    { 0xc2b8c6fc, 95 },
    { 0x1d8c6fd, 33 },
    { 0x6c28c6fe, 55 },
    { 0x2518c6ff, 61 },
    { 0xfbd8c6ff, 76 },
    { 0, 0 },
    { 0xa728c702, 61 },
    { 0, 0 },
    { 0x1408c704, 55 },
    { 0xfb08c705, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe78c709, 66 },
    { 0xee48c70a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa8c70d, 61 },
    { 0, 0 },
    { 0x5a78c70f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e18c717, 87 },
    { 0x4658c718, 34 },
    { 0x64d8c719, 61 },
    { 0x1188c71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2118c71d, 87 },
    { 0x8108c71e, 95 },
    { 0xb488c71f, 88 },
    { 0xcc38c720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf18c726, 59 },
    { 0x7628c726, 91 },
    { 0, 0 },
    { 0x6c38c729, 90 },
    { 0xc358c72a, 61 },
    { 0x1608c72b, 99 },
    { 0, 0 },
    { 0xbad8c72d, 66 },
    { 0, 0 },
    { 0x9768c72f, 77 },
    { 0x8828c730, 61 },
    { 0xb3b8c731, 34 },
    { 0, 0 },
    { 0x71b8c733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e8c73b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968c73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf158c747, 90 },
    { 0x8438c747, 95 },
    { 0x3638c749, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f88c74f, 77 },
    { 0, 0 },
    { 0xb8b8c751, 34 },
    { 0x9328c751, 46 },
    { 0xc4f8c753, 95 },
    { 0x9d58c754, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a58c759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa828c75d, 95 },
    { 0xa718c75e, 34 },
    { 0xcfd8c75e, 87 },
    { 0, 0 },
    { 0xc278c761, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238c76a, 61 },
    { 0x1778c76b, 90 },
    { 0, 0 },
    { 0xa338c76d, 90 },
    { 0x7a48c76e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f8c772, 90 },
    { 0, 0 },
    { 0x3eb8c774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad8c778, 78 },
    { 0, 0 },
    { 0x3fe8c77a, 87 },
    { 0x5a58c77b, 33 },
    { 0xb288c77c, 76 },
    { 0xb4d8c77d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe118c781, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e8c78a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b8c790, 23 },
    { 0x8a48c790, 97 },
    { 0, 0 },
    { 0xe2d8c793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728c796, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3008c79c, 33 },
    { 0, 0 },
    { 0x90a8c79e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa448c7a2, 90 },
    { 0, 0 },
    { 0x4aa8c7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd758c7a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e8c7ab, 98 },
    { 0x1838c7ac, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2538c7b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b08c7b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa348c7b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e98c7c6, 85 },
    { 0, 0 },
    { 0x6508c7c8, 91 },
    { 0x9478c7c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f8c7cc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78c7d6, 81 },
    { 0, 0 },
    { 0xfe88c7d8, 93 },
    { 0xff28c7d9, 98 },
    { 0xdf58c7d8, 81 },
    { 0, 0 },
    { 0xa3c8c7dc, 23 },
    { 0, 0 },
    { 0x8738c7de, 90 },
    { 0x8558c7df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xced8c7e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28c7e6, 33 },
    { 0x8008c7e7, 81 },
    { 0, 0 },
    { 0x6c58c7e9, 33 },
    { 0x47f8c7e9, 76 },
    { 0, 0 },
    { 0xd8c7ec, 23 },
    { 0x5c68c7ec, 76 },
    { 0xcd28c7ee, 80 },
    { 0x3688c7ec, 80 },
    { 0xd918c7f0, 43 },
    { 0, 0 },
    { 0xfb8c7f2, 61 },
    { 0x9fd8c7f3, 34 },
    { 0x8d18c7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc458c7f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e8c7fa, 46 },
    { 0, 0 },
    { 0x6268c7fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f48c802, 98 },
    { 0xfb38c803, 33 },
    { 0xc168c802, 95 },
    { 0x6de8c803, 23 },
    { 0x3de8c804, 55 },
    { 0xbe88c802, 23 },
    { 0x49d8c808, 95 },
    { 0x4b88c806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7218c80e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac8c811, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338c815, 46 },
    { 0xeaf8c816, 33 },
    { 0xd9e8c817, 46 },
    { 0x51a8c817, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1558c81b, 98 },
    { 0x3e78c81c, 68 },
    { 0x4e8c81c, 46 },
    { 0xdcd8c81c, 100 },
    { 0x8418c81b, 90 },
    { 0, 0 },
    { 0x7618c821, 23 },
    { 0x7ff8c822, 84 },
    { 0xc708c821, 80 },
    { 0x1c28c824, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f8c828, 68 },
    { 0xae68c828, 23 },
    { 0x9cf8c82a, 90 },
    { 0xef48c82b, 95 },
    { 0xa188c82c, 84 },
    { 0x1818c82b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8c834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e8c83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2218c847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18c84f, 23 },
    { 0x5e8c850, 93 },
    { 0, 0 },
    { 0xc3b8c852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c8c856, 61 },
    { 0xbd88c856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18c85d, 95 },
    { 0x1dc8c85e, 33 },
    { 0x77c8c85e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6648c868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed98c86c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f8c874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad88c879, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee38c87d, 100 },
    { 0, 0 },
    { 0x7c8c87f, 4 },
    { 0, 0 },
    { 0xd528c881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18c885, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9578c88a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c88d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8c890, 90 },
    { 0, 0 },
    { 0xc518c892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e38c896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8718c89b, 88 },
    { 0xa5b8c89b, 76 },
    { 0x4a68c89c, 95 },
    { 0x7208c89e, 95 },
    { 0xd4f8c89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3728c8a3, 67 },
    { 0, 0 },
    { 0x7f28c8a5, 78 },
    { 0x4f88c8a5, 61 },
    { 0x7578c8a5, 90 },
    { 0xca58c8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6548c8ac, 61 },
    { 0, 0 },
    { 0x7aa8c8ae, 90 },
    { 0x9bd8c8ae, 23 },
    { 0xa318c8af, 33 },
    { 0x24e8c8b1, 43 },
    { 0x8d58c8b2, 55 },
    { 0x39f8c8b3, 64 },
    { 0xbb48c8b4, 93 },
    { 0xcd08c8b4, 46 },
    { 0x168c8b2, 84 },
    { 0x3e38c8b7, 61 },
    { 0x6528c8b5, 33 },
    { 0x8e78c8b9, 100 },
    { 0x6608c8ba, 33 },
    { 0xdeb8c8bb, 90 },
    { 0x1718c8bc, 100 },
    { 0x5088c8bd, 95 },
    { 0x1838c8bb, 23 },
    { 0x7258c8bd, 76 },
    { 0x1428c8b5, 98 },
    { 0x6e08c8c1, 93 },
    { 0xc4e8c8c2, 96 },
    { 0, 0 },
    { 0x97d8c8c4, 91 },
    { 0xb468c8c5, 90 },
    { 0x48b8c8c4, 61 },
    { 0x86f8c8c6, 90 },
    { 0x8408c8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f8c8d0, 33 },
    { 0, 0 },
    { 0xa4d8c8d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368c8d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb8c8dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8c8e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c8c8ea, 88 },
    { 0, 0 },
    { 0x26f8c8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e8c8ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b8c8fb, 87 },
    { 0x2878c8fc, 61 },
    { 0, 0 },
    { 0x8108c8fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e68c903, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8c907, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78c90a, 67 },
    { 0x1788c90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8258c910, 33 },
    { 0xcb48c910, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b8c915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3718c918, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab8c91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18c91e, 81 },
    { 0xe838c91f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518c926, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb8c929, 23 },
    { 0x7998c92a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828c92f, 43 },
    { 0x3538c930, 46 },
    { 0x81f8c931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6948c934, 90 },
    { 0, 0 },
    { 0x8748c936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8738c93b, 100 },
    { 0x4a68c93c, 76 },
    { 0, 0 },
    { 0x568c93e, 98 },
    { 0x9178c93f, 33 },
    { 0, 0 },
    { 0xe098c941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba8c949, 46 },
    { 0x2688c949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698c951, 33 },
    { 0xe618c951, 95 },
    { 0xc7b8c951, 95 },
    { 0, 0 },
    { 0xb8d8c955, 93 },
    { 0x22c8c956, 43 },
    { 0x5df8c957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2758c95c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a8c962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf8c965, 33 },
    { 0x9ba8c965, 23 },
    { 0, 0 },
    { 0xd668c968, 68 },
    { 0xe1d8c968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f8c96d, 90 },
    { 0, 0 },
    { 0x2198c96f, 33 },
    { 0xa768c96f, 61 },
    { 0, 0 },
    { 0x6648c972, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8c976, 98 },
    { 0x8448c977, 61 },
    { 0xd1d8c978, 61 },
    { 0xa228c977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc818c982, 61 },
    { 0x4f78c983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28c98e, 61 },
    { 0x3768c98f, 51 },
    { 0x89c8c98f, 61 },
    { 0x4dc8c991, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38c996, 45 },
    { 0xca18c996, 33 },
    { 0x8bb8c996, 99 },
    { 0, 0 },
    { 0xda68c99a, 76 },
    { 0, 0 },
    { 0x82a8c99c, 77 },
    { 0x5228c99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9208c9a1, 55 },
    { 0x2d28c9a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428c9a8, 76 },
    { 0xe888c9a9, 78 },
    { 0x90e8c9a9, 61 },
    { 0x4d58c9a8, 61 },
    { 0, 0 },
    { 0xe938c9ad, 55 },
    { 0, 0 },
    { 0x5e38c9af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca38c9b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8c9ba, 61 },
    { 0, 0 },
    { 0xc968c9bc, 76 },
    { 0xea78c9bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1948c9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d8c9cb, 33 },
    { 0x9638c9cc, 85 },
    { 0xf308c9cb, 90 },
    { 0xd508c9cd, 61 },
    { 0, 0 },
    { 0xe538c9d0, 77 },
    { 0, 0 },
    { 0x4fa8c9d2, 90 },
    { 0xda38c9d3, 80 },
    { 0, 0 },
    { 0xd898c9d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f8c9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8c9de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28c9e1, 69 },
    { 0xa88c9e1, 95 },
    { 0xfc8c9e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed58c9ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a8c9f8, 61 },
    { 0x1978c9f9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48c9fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x418c9ff, 61 },
    { 0, 0 },
    { 0xcf88ca01, 64 },
    { 0, 0 },
    { 0xc888ca03, 33 },
    { 0x6f28ca03, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5708ca07, 100 },
    { 0xa108ca07, 95 },
    { 0x8ab8ca09, 55 },
    { 0xb1b8ca07, 90 },
    { 0xaec8ca0a, 55 },
    { 0x9fb8ca08, 23 },
    { 0xea18ca0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b8ca10, 90 },
    { 0, 0 },
    { 0xcc48ca12, 90 },
    { 0x9168ca12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a8ca16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f8ca27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8508ca2f, 66 },
    { 0xd528ca2f, 33 },
    { 0x8ef8ca31, 23 },
    { 0xcf08ca32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3678ca39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438ca41, 55 },
    { 0x7e68ca42, 90 },
    { 0xc9a8ca43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678ca48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa8ca4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a68ca53, 23 },
    { 0, 0 },
    { 0xf468ca55, 90 },
    { 0xab88ca56, 66 },
    { 0xbdb8ca57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8ca5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed8ca61, 90 },
    { 0x92e8ca62, 33 },
    { 0xeb88ca63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6768ca66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d38ca6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858ca6d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4538ca7b, 91 },
    { 0x8648ca7c, 46 },
    { 0, 0 },
    { 0xf448ca7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3038ca81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6598ca84, 98 },
    { 0, 0 },
    { 0x88f8ca86, 93 },
    { 0x6e8ca87, 98 },
    { 0xbd08ca86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a8ca8c, 55 },
    { 0, 0 },
    { 0x8328ca8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e8ca93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8ca97, 61 },
    { 0x1ed8ca98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b38ca9c, 67 },
    { 0, 0 },
    { 0xfc28ca9e, 34 },
    { 0x72b8ca9f, 23 },
    { 0xae58caa0, 34 },
    { 0xe0b8ca9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc618caa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc488caa9, 61 },
    { 0xf508caaa, 81 },
    { 0xd838caab, 90 },
    { 0xc938caac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3308caaf, 66 },
    { 0xfc78cab0, 91 },
    { 0xf3f8cab0, 95 },
    { 0xdc38cab1, 98 },
    { 0, 0 },
    { 0x1318cab4, 61 },
    { 0x7228cab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878caba, 61 },
    { 0, 0 },
    { 0xd1a8cabc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58cabf, 90 },
    { 0, 0 },
    { 0xeda8cac1, 87 },
    { 0, 0 },
    { 0xf308cac3, 83 },
    { 0x5598cac4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08cac9, 90 },
    { 0, 0 },
    { 0x30e8cacb, 61 },
    { 0x30b8cacb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4978cacf, 91 },
    { 0, 0 },
    { 0x20d8cad1, 90 },
    { 0xa788cad2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8cade, 90 },
    { 0xa7a8cade, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff78cae3, 52 },
    { 0x8b38cae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838cae8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8caf1, 90 },
    { 0xc168caf2, 34 },
    { 0xd1f8caf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1358caf6, 95 },
    { 0xf3c8caf6, 61 },
    { 0x2128caf8, 76 },
    { 0, 0 },
    { 0x5108cafa, 77 },
    { 0xef88cafb, 23 },
    { 0xf9c8cafc, 55 },
    { 0x5368cafb, 33 },
    { 0xcf28cafe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8cb04, 61 },
    { 0xb718cb05, 100 },
    { 0x5988cb04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f8cb0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d8cb1b, 61 },
    { 0xa648cb1c, 50 },
    { 0xbb08cb1c, 95 },
    { 0x3bd8cb1e, 90 },
    { 0x8d88cb1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d28cb23, 84 },
    { 0x25b8cb24, 93 },
    { 0, 0 },
    { 0x3138cb26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8cb29, 93 },
    { 0, 0 },
    { 0xb178cb2b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48cb2e, 52 },
    { 0x6cf8cb2f, 23 },
    { 0xd598cb30, 76 },
    { 0xc7b8cb31, 90 },
    { 0x7068cb32, 100 },
    { 0x1b8cb32, 34 },
    { 0, 0 },
    { 0xda08cb35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428cb38, 98 },
    { 0xafc8cb38, 17 },
    { 0xf578cb39, 90 },
    { 0xa648cb3b, 33 },
    { 0x4538cb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a8cb40, 23 },
    { 0x8998cb41, 99 },
    { 0xfc88cb42, 43 },
    { 0xf838cb43, 34 },
    { 0x5418cb44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8cb48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28cb4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe018cb4e, 90 },
    { 0x6048cb4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88cb55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748cb5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28cb5f, 90 },
    { 0xf9c8cb60, 75 },
    { 0, 0 },
    { 0x5ac8cb62, 33 },
    { 0x8d08cb62, 33 },
    { 0xc798cb62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3028cb69, 95 },
    { 0, 0 },
    { 0x4138cb6b, 90 },
    { 0x15f8cb6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7358cb6f, 76 },
    { 0xb4d8cb6f, 34 },
    { 0xdfc8cb71, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d58cb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9198cb7e, 76 },
    { 0x4328cb7f, 91 },
    { 0x83e8cb7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b8cb83, 85 },
    { 0xe428cb83, 33 },
    { 0xbc8cb84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf038cb8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d8cb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df8cb98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd278cb9d, 85 },
    { 0x1268cb9e, 90 },
    { 0, 0 },
    { 0xe68cba0, 95 },
    { 0xf548cba1, 91 },
    { 0xf858cba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec8cba7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608cbaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7788cbae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448cbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2848cbb5, 95 },
    { 0x6088cbb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8cbba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8cbbd, 23 },
    { 0xbc58cbbd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2878cbc1, 98 },
    { 0, 0 },
    { 0xab18cbc3, 33 },
    { 0x6cb8cbc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d18cbc8, 75 },
    { 0x9ec8cbc9, 99 },
    { 0, 0 },
    { 0x6eb8cbcb, 90 },
    { 0xc508cbcc, 98 },
    { 0x4df8cbcd, 61 },
    { 0x93a8cbcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2268cbd1, 46 },
    { 0xa918cbd1, 76 },
    { 0x6c28cbd2, 66 },
    { 0xfac8cbd3, 90 },
    { 0x33a8cbd5, 95 },
    { 0xd2e8cbd5, 99 },
    { 0x8c78cbd7, 23 },
    { 0x5bb8cbd8, 33 },
    { 0x8d68cbd1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd848cbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4768cbe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c48cbe5, 95 },
    { 0xf748cbe6, 93 },
    { 0, 0 },
    { 0x9d68cbe8, 76 },
    { 0xc048cbe9, 23 },
    { 0, 0 },
    { 0x8d08cbeb, 23 },
    { 0xef88cbec, 55 },
    { 0xfa08cbed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8cbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1928cbfb, 61 },
    { 0x8138cbfc, 75 },
    { 0, 0 },
    { 0xc598cbfe, 61 },
    { 0, 0 },
    { 0xf518cc00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c8cc03, 76 },
    { 0xb498cc03, 61 },
    { 0xa88cc03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace8cc12, 76 },
    { 0, 0 },
    { 0xc9a8cc14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1918cc21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6308cc24, 23 },
    { 0, 0 },
    { 0x4278cc26, 95 },
    { 0, 0 },
    { 0x99b8cc28, 61 },
    { 0x6388cc29, 70 },
    { 0x6f68cc2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc8cc30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98cc33, 33 },
    { 0xc998cc34, 91 },
    { 0xa398cc35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf108cc3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd178cc40, 66 },
    { 0x5328cc40, 95 },
    { 0xb2d8cc42, 40 },
    { 0x2ee8cc42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8cc47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8cc4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348cc50, 46 },
    { 0, 0 },
    { 0xe578cc52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d08cc56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c48cc5f, 33 },
    { 0x3488cc60, 95 },
    { 0, 0 },
    { 0xefe8cc62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8cc66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac08cc71, 34 },
    { 0, 0 },
    { 0xae58cc73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca08cc7d, 33 },
    { 0x3e28cc7e, 23 },
    { 0xfc38cc7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28cc82, 90 },
    { 0, 0 },
    { 0x65a8cc84, 93 },
    { 0xae8cc84, 33 },
    { 0, 0 },
    { 0x1af8cc87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd28cc8c, 61 },
    { 0xe408cc8d, 91 },
    { 0x7ec8cc8e, 61 },
    { 0x7068cc8e, 98 },
    { 0, 0 },
    { 0x3bc8cc91, 98 },
    { 0, 0 },
    { 0x6cb8cc93, 43 },
    { 0, 0 },
    { 0x58b8cc95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698cc98, 100 },
    { 0x848cc98, 98 },
    { 0xbcd8cc9a, 98 },
    { 0x9258cc9b, 61 },
    { 0x1418cc9c, 93 },
    { 0x8538cc9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe308cca0, 5 },
    { 0xa328cca1, 100 },
    { 0x8a68cca2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b8cca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c8cca9, 43 },
    { 0xf6a8ccaa, 69 },
    { 0, 0 },
    { 0xfb78ccac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6448ccaf, 38 },
    { 0x5c18ccaf, 95 },
    { 0xf6e8ccb1, 93 },
    { 0xd138ccb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828ccb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe578ccbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad48ccc0, 90 },
    { 0, 0 },
    { 0xa238ccc2, 33 },
    { 0, 0 },
    { 0x12e8ccc4, 87 },
    { 0x38b8ccc5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d68ccc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5138ccce, 72 },
    { 0, 0 },
    { 0x9b18ccd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8ccd4, 66 },
    { 0xf5a8ccd5, 23 },
    { 0, 0 },
    { 0xbf48ccd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98ccdb, 100 },
    { 0x2e38ccdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa68ccdf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8608cce4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf8cce7, 91 },
    { 0x7e18cce7, 76 },
    { 0x8f18cce9, 76 },
    { 0, 0 },
    { 0x5df8cceb, 98 },
    { 0x8af8ccec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a88ccf2, 84 },
    { 0x1a48ccf3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8ccf7, 75 },
    { 0xdfd8ccf7, 98 },
    { 0, 0 },
    { 0x7b68ccfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f8ccff, 55 },
    { 0x20c8cd00, 76 },
    { 0xed8cd00, 98 },
    { 0xf758cd01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8cd05, 66 },
    { 0x2a08cd05, 90 },
    { 0xb808cd07, 46 },
    { 0x2bd8cd08, 66 },
    { 0, 0 },
    { 0x7c88cd0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118cd0d, 97 },
    { 0, 0 },
    { 0x6498cd0f, 91 },
    { 0x6258cd10, 90 },
    { 0x5538cd0f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea38cd15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888cd19, 78 },
    { 0xeeb8cd1a, 33 },
    { 0, 0 },
    { 0x21d8cd1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8718cd27, 23 },
    { 0, 0 },
    { 0x3158cd29, 23 },
    { 0xf708cd29, 55 },
    { 0xeed8cd2b, 75 },
    { 0x2098cd2c, 93 },
    { 0xc5c8cd2c, 61 },
    { 0xa928cd2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f8cd32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc418cd36, 98 },
    { 0x7ad8cd36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa098cd3a, 23 },
    { 0xc058cd3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98cd44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a8cd49, 87 },
    { 0x44a8cd4a, 34 },
    { 0, 0 },
    { 0x9218cd4c, 33 },
    { 0, 0 },
    { 0x3518cd4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a8cd53, 85 },
    { 0xa368cd53, 36 },
    { 0x8208cd54, 99 },
    { 0, 0 },
    { 0x7788cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8cd5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8cd5e, 90 },
    { 0, 0 },
    { 0x7188cd60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8cd65, 23 },
    { 0x5998cd66, 76 },
    { 0xe658cd67, 100 },
    { 0xe6d8cd65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf8cd6e, 61 },
    { 0xfe28cd6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc88cd7d, 55 },
    { 0x8578cd7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3138cd89, 90 },
    { 0, 0 },
    { 0x66d8cd8b, 90 },
    { 0x8348cd8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548cd8f, 23 },
    { 0x4328cd90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe058cd94, 76 },
    { 0, 0 },
    { 0xd638cd96, 33 },
    { 0xc08cd97, 52 },
    { 0, 0 },
    { 0xeda8cd99, 46 },
    { 0xa7c8cd9a, 90 },
    { 0x1a28cd9b, 95 },
    { 0x688cd9b, 33 },
    { 0x70a8cd9c, 69 },
    { 0x1f18cd9e, 46 },
    { 0x67d8cd9a, 46 },
    { 0x6458cd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9278cda3, 93 },
    { 0, 0 },
    { 0x2698cda5, 90 },
    { 0x6c88cda6, 98 },
    { 0x5848cda7, 98 },
    { 0xde88cda8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8cdac, 43 },
    { 0x5988cdad, 98 },
    { 0xdea8cdad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a8cdb5, 93 },
    { 0x5248cdb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c8cdba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a8cdbd, 95 },
    { 0x8be8cdbd, 65 },
    { 0xadb8cdbe, 77 },
    { 0xbd88cdc0, 33 },
    { 0x6698cdbe, 33 },
    { 0, 0 },
    { 0x1418cdc3, 90 },
    { 0x7cc8cdc4, 100 },
    { 0xd998cdc5, 95 },
    { 0, 0 },
    { 0x1558cdc7, 23 },
    { 0, 0 },
    { 0xbc18cdc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88cdcd, 23 },
    { 0x82c8cdce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8cdd2, 23 },
    { 0xe758cdd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1908cdd6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa858cdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa8cddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6268cde5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b8cde9, 23 },
    { 0x2fc8cdea, 91 },
    { 0, 0 },
    { 0xf1a8cdec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8cdef, 66 },
    { 0xc228cdef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d8cdf4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958cdf7, 66 },
    { 0xac88cdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6518ce00, 43 },
    { 0x5c18ce00, 33 },
    { 0x2e98ce01, 88 },
    { 0x2658ce03, 91 },
    { 0xa268ce03, 90 },
    { 0x8ff8ce05, 76 },
    { 0xf0f8ce03, 52 },
    { 0xb8ce07, 93 },
    { 0xe08ce03, 87 },
    { 0xd368ce05, 90 },
    { 0x6d38ce0a, 76 },
    { 0xd648ce0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438ce0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8ce11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528ce14, 66 },
    { 0, 0 },
    { 0xb848ce16, 90 },
    { 0, 0 },
    { 0x3e8ce18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408ce1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe878ce20, 43 },
    { 0xde18ce21, 33 },
    { 0x1a78ce22, 76 },
    { 0xe0e8ce23, 90 },
    { 0x60b8ce24, 90 },
    { 0xed68ce23, 95 },
    { 0x6d98ce25, 61 },
    { 0x65d8ce24, 76 },
    { 0x2f28ce25, 76 },
    { 0x3f28ce24, 90 },
    { 0xddc8ce28, 23 },
    { 0x7cf8ce2b, 46 },
    { 0xbb78ce2c, 99 },
    { 0x4d88ce2b, 33 },
    { 0x4e88ce2e, 91 },
    { 0x3458ce2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa488ce32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8ce36, 90 },
    { 0x3268ce36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc908ce3a, 23 },
    { 0, 0 },
    { 0x8448ce3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88ce45, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898ce48, 90 },
    { 0, 0 },
    { 0xdca8ce4a, 55 },
    { 0, 0 },
    { 0x94e8ce4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f08ce4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568ce53, 77 },
    { 0xf858ce54, 33 },
    { 0xf28ce55, 88 },
    { 0xc378ce56, 61 },
    { 0xf7c8ce56, 90 },
    { 0x7d18ce57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da8ce5e, 90 },
    { 0xae88ce5e, 90 },
    { 0xc878ce5f, 61 },
    { 0xb8e8ce5e, 23 },
    { 0, 0 },
    { 0x8a38ce63, 33 },
    { 0xb428ce63, 34 },
    { 0xc2b8ce65, 85 },
    { 0x8d8ce65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb8ce6a, 90 },
    { 0x7298ce6b, 46 },
    { 0x8668ce6c, 65 },
    { 0xe088ce6d, 95 },
    { 0x2898ce6e, 33 },
    { 0x4b98ce6f, 88 },
    { 0x6008ce70, 90 },
    { 0xf878ce6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa838ce79, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7248ce83, 55 },
    { 0xe08ce84, 91 },
    { 0x89d8ce85, 75 },
    { 0x1d68ce86, 61 },
    { 0x16f8ce87, 90 },
    { 0x1f58ce88, 95 },
    { 0xbfe8ce83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4488ce93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa128ce97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728ce9a, 36 },
    { 0xa088ce9b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc8cea4, 55 },
    { 0, 0 },
    { 0x6c28cea6, 75 },
    { 0xddf8cea7, 78 },
    { 0, 0 },
    { 0xd08cea9, 90 },
    { 0xed8ceaa, 75 },
    { 0, 0 },
    { 0x94a8ceac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f8ceaf, 53 },
    { 0xa6e8ceb0, 99 },
    { 0x3ab8ceb1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe858cebb, 26 },
    { 0xd528cebb, 33 },
    { 0x37e8cebd, 99 },
    { 0xcf78cebe, 59 },
    { 0xceb8cebc, 33 },
    { 0x4678cec0, 55 },
    { 0x4838cebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1448cec4, 52 },
    { 0x2448cec5, 76 },
    { 0, 0 },
    { 0x1d58cec7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb8cecb, 98 },
    { 0xbb78cecc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b8ced2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9878ced5, 90 },
    { 0xef28ced6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a8cedb, 85 },
    { 0x4498cedc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8cedf, 95 },
    { 0x5b28cee0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b8cee3, 34 },
    { 0x4a58cee4, 98 },
    { 0xea78cee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd8ceea, 77 },
    { 0, 0 },
    { 0x4028ceec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8ceef, 85 },
    { 0x4f98cef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8cef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1978cef9, 93 },
    { 0, 0 },
    { 0x8188cefb, 98 },
    { 0xe598cefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58cf01, 76 },
    { 0, 0 },
    { 0xbef8cf03, 90 },
    { 0x3dd8cf04, 48 },
    { 0x62d8cf04, 98 },
    { 0x178cf06, 61 },
    { 0x8988cf07, 98 },
    { 0x848cf06, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd58cf0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a8cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8cf16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a8cf1f, 66 },
    { 0xcb18cf1f, 61 },
    { 0x22b8cf21, 95 },
    { 0, 0 },
    { 0x8b38cf23, 61 },
    { 0xf948cf24, 34 },
    { 0, 0 },
    { 0xde98cf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a8cf29, 65 },
    { 0xf5d8cf29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f28cf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8cf32, 61 },
    { 0, 0 },
    { 0x69c8cf34, 33 },
    { 0xe198cf35, 87 },
    { 0x4748cf36, 23 },
    { 0, 0 },
    { 0x7908cf38, 61 },
    { 0xd1c8cf39, 23 },
    { 0, 0 },
    { 0x1798cf3b, 98 },
    { 0x4038cf3c, 33 },
    { 0x1bb8cf3d, 90 },
    { 0x1fd8cf3e, 90 },
    { 0xc258cf3f, 61 },
    { 0x9908cf3f, 90 },
    { 0x9ba8cf41, 93 },
    { 0x3be8cf3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb8cf47, 66 },
    { 0, 0 },
    { 0x1128cf49, 33 },
    { 0xe4c8cf4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268cf4f, 33 },
    { 0x26c8cf4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4888cf54, 17 },
    { 0, 0 },
    { 0xf0e8cf56, 17 },
    { 0x8fe8cf57, 55 },
    { 0xcb18cf58, 84 },
    { 0xaa98cf58, 33 },
    { 0x5b78cf57, 98 },
    { 0x8d28cf57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5888cf5e, 55 },
    { 0, 0 },
    { 0x7228cf60, 23 },
    { 0x6c38cf61, 33 },
    { 0x4688cf60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d8cf65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e08cf6a, 98 },
    { 0, 0 },
    { 0x4778cf6c, 90 },
    { 0x3578cf6d, 98 },
    { 0x4328cf6d, 52 },
    { 0x478cf6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb728cf79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f48cf7d, 55 },
    { 0xa158cf7e, 67 },
    { 0x9448cf7f, 98 },
    { 0xa2a8cf7f, 99 },
    { 0x7a88cf7f, 93 },
    { 0, 0 },
    { 0xa918cf83, 87 },
    { 0, 0 },
    { 0xe9b8cf85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f78cf88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe38cf8b, 34 },
    { 0x34b8cf8c, 98 },
    { 0xb0a8cf8d, 76 },
    { 0xba28cf8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8cf91, 33 },
    { 0, 0 },
    { 0xca98cf93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf638cf96, 61 },
    { 0xab18cf97, 43 },
    { 0, 0 },
    { 0xfc78cf99, 99 },
    { 0x4028cf99, 40 },
    { 0x35e8cf99, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f8cfa0, 85 },
    { 0x5988cfa0, 23 },
    { 0x7708cfa1, 38 },
    { 0xbf38cfa3, 98 },
    { 0x79d8cfa4, 61 },
    { 0xd8c8cfa4, 33 },
    { 0x888cfa4, 90 },
    { 0xfc48cfa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b8cfac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed08cfb2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4018cfb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d78cfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e8cfc1, 33 },
    { 0x4a58cfc1, 98 },
    { 0xdda8cfc3, 23 },
    { 0x7018cfc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff18cfcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88cfd0, 23 },
    { 0xe108cfd0, 34 },
    { 0x8598cfd2, 98 },
    { 0xe818cfd3, 33 },
    { 0, 0 },
    { 0xfb8cfd5, 34 },
    { 0x5f58cfd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6428cfd9, 33 },
    { 0x1ff8cfda, 90 },
    { 0x9bf8cfdb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef8cfdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a8cfe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e8cfe5, 64 },
    { 0x5b38cfe6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8cfea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e78cff1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8cff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38cff7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b8d000, 34 },
    { 0x9f28d000, 33 },
    { 0, 0 },
    { 0xd4b8d003, 61 },
    { 0xf278d004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4258d00b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48d00e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398d012, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d019, 55 },
    { 0x8118d01a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f98d01d, 78 },
    { 0xd08d01e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2308d021, 61 },
    { 0, 0 },
    { 0x5718d023, 97 },
    { 0x7118d023, 90 },
    { 0xc568d023, 61 },
    { 0x2e88d023, 66 },
    { 0x13a8d027, 43 },
    { 0xacd8d027, 33 },
    { 0xb208d028, 95 },
    { 0x72a8d027, 33 },
    { 0, 0 },
    { 0x4f88d02c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d032, 23 },
    { 0xea18d033, 66 },
    { 0, 0 },
    { 0x47f8d035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e48d039, 87 },
    { 0, 0 },
    { 0xb728d03b, 17 },
    { 0x4e68d03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c8d046, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a88d04d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8678d054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda48d058, 33 },
    { 0x6728d058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8d05c, 90 },
    { 0xa4b8d05c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d8d063, 95 },
    { 0, 0 },
    { 0x34b8d065, 90 },
    { 0xbbb8d066, 93 },
    { 0x9008d067, 84 },
    { 0x2f38d068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b8d06d, 33 },
    { 0xa728d06e, 84 },
    { 0, 0 },
    { 0xa158d070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398d076, 95 },
    { 0x5368d077, 61 },
    { 0x26f8d078, 34 },
    { 0xa1a8d077, 37 },
    { 0, 0 },
    { 0x84c8d07b, 55 },
    { 0, 0 },
    { 0x8718d07d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc328d083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a8d086, 33 },
    { 0, 0 },
    { 0x2c58d088, 23 },
    { 0, 0 },
    { 0xd638d08a, 88 },
    { 0xb8b8d08a, 99 },
    { 0x1428d08a, 61 },
    { 0xd4d8d08d, 55 },
    { 0xa6b8d08e, 55 },
    { 0x34f8d08f, 46 },
    { 0xcc68d08f, 76 },
    { 0x5dd8d08a, 90 },
    { 0xd618d08d, 36 },
    { 0x5ce8d08a, 95 },
    { 0xa538d092, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c8d098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48d09f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad8d0a7, 90 },
    { 0, 0 },
    { 0xb6f8d0a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8d0b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398d0c0, 76 },
    { 0x20f8d0c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e8d0c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf868d0c8, 33 },
    { 0xb328d0c9, 61 },
    { 0x4198d0ca, 75 },
    { 0, 0 },
    { 0xa138d0cc, 61 },
    { 0xaa28d0cc, 90 },
    { 0, 0 },
    { 0xbb68d0cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda78d0d2, 95 },
    { 0, 0 },
    { 0xc608d0d4, 98 },
    { 0x558d0d5, 55 },
    { 0xe898d0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8d0db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818d0df, 43 },
    { 0x7428d0e0, 93 },
    { 0xab38d0e1, 98 },
    { 0, 0 },
    { 0xf928d0e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d8d0eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8d0f0, 33 },
    { 0, 0 },
    { 0x1138d0f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb228d0fa, 55 },
    { 0, 0 },
    { 0xe098d0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c78d102, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958d105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88d108, 90 },
    { 0xd508d109, 78 },
    { 0, 0 },
    { 0x23b8d10b, 92 },
    { 0x1288d10c, 33 },
    { 0, 0 },
    { 0x82e8d10e, 98 },
    { 0, 0 },
    { 0x63b8d110, 23 },
    { 0x2338d111, 77 },
    { 0x8338d112, 33 },
    { 0x5578d113, 17 },
    { 0xd978d112, 95 },
    { 0, 0 },
    { 0x4df8d116, 85 },
    { 0x6e58d117, 33 },
    { 0xb078d118, 100 },
    { 0xa18d119, 90 },
    { 0, 0 },
    { 0xd868d11b, 87 },
    { 0, 0 },
    { 0xe248d11d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6758d121, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718d128, 65 },
    { 0xdd88d129, 90 },
    { 0x3358d129, 95 },
    { 0x51f8d12a, 90 },
    { 0x1f18d12c, 95 },
    { 0x1408d129, 51 },
    { 0x47e8d12e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a58d133, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa638d143, 90 },
    { 0x1ef8d143, 98 },
    { 0xa5c8d145, 95 },
    { 0xaf48d146, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb8d14c, 80 },
    { 0xf098d14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a8d152, 99 },
    { 0x4fa8d153, 78 },
    { 0x878d153, 90 },
    { 0x5528d155, 76 },
    { 0, 0 },
    { 0x6e18d157, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9318d15c, 90 },
    { 0xe038d15d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf8d164, 90 },
    { 0, 0 },
    { 0x40e8d166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc358d16b, 61 },
    { 0x8808d16b, 55 },
    { 0x1838d16b, 90 },
    { 0x6fd8d16b, 43 },
    { 0x1788d16c, 53 },
    { 0, 0 },
    { 0xac18d171, 33 },
    { 0xccd8d172, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc168d177, 76 },
    { 0xc2a8d178, 99 },
    { 0x4c98d179, 46 },
    { 0, 0 },
    { 0x2bb8d17b, 90 },
    { 0x9328d17b, 40 },
    { 0x5188d17d, 90 },
    { 0xcfe8d17e, 90 },
    { 0x428d17b, 90 },
    { 0x57f8d180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8d18b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928d18e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18d193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8d19a, 93 },
    { 0, 0 },
    { 0xf388d19c, 46 },
    { 0x63a8d19d, 43 },
    { 0x9228d19d, 84 },
    { 0, 0 },
    { 0x45e8d1a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9228d1a7, 91 },
    { 0x4e88d1a7, 23 },
    { 0, 0 },
    { 0xfb18d1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38d1ad, 33 },
    { 0, 0 },
    { 0xa588d1af, 64 },
    { 0x5a38d1b0, 34 },
    { 0, 0 },
    { 0xb768d1b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe808d1b6, 95 },
    { 0x7818d1b6, 95 },
    { 0xe4d8d1b8, 40 },
    { 0x2778d1b9, 43 },
    { 0xbd88d1b9, 23 },
    { 0x3398d1ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6418d1be, 76 },
    { 0, 0 },
    { 0xd518d1c0, 34 },
    { 0xfb68d1c1, 61 },
    { 0x7ba8d1c2, 91 },
    { 0x7708d1c3, 67 },
    { 0xa498d1c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708d1c7, 95 },
    { 0x7ad8d1c8, 95 },
    { 0x6298d1c9, 98 },
    { 0x7b48d1ca, 33 },
    { 0x9688d1c9, 66 },
    { 0xd688d1cc, 67 },
    { 0x8248d1c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8238d1d0, 23 },
    { 0, 0 },
    { 0xe5b8d1d2, 61 },
    { 0x2358d1d2, 90 },
    { 0x1098d1d4, 94 },
    { 0x73c8d1d5, 94 },
    { 0x62c8d1d6, 33 },
    { 0xbd28d1d2, 17 },
    { 0, 0 },
    { 0xdab8d1d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf598d1dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b8d1e3, 95 },
    { 0x99a8d1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788d1ea, 91 },
    { 0x6e08d1ea, 98 },
    { 0xce8d1ec, 87 },
    { 0x10c8d1ec, 95 },
    { 0xea38d1ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c48d1f2, 100 },
    { 0x8858d1f2, 90 },
    { 0x7b78d1f4, 33 },
    { 0x9cd8d1f4, 80 },
    { 0xf338d1f5, 68 },
    { 0, 0 },
    { 0xd178d1f8, 55 },
    { 0, 0 },
    { 0xe3a8d1fa, 33 },
    { 0x9918d1fb, 17 },
    { 0, 0 },
    { 0xccf8d1fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda68d204, 46 },
    { 0x23b8d204, 82 },
    { 0x7e48d206, 23 },
    { 0xc4d8d204, 95 },
    { 0, 0 },
    { 0xf8f8d209, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc688d20c, 23 },
    { 0xd468d20d, 33 },
    { 0, 0 },
    { 0xfa78d20f, 66 },
    { 0x4c48d210, 43 },
    { 0, 0 },
    { 0xb2e8d212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4208d215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8d218, 88 },
    { 0x1988d218, 91 },
    { 0x8d28d21a, 33 },
    { 0x28d8d21b, 43 },
    { 0x5708d21b, 98 },
    { 0xc268d219, 34 },
    { 0x188d21e, 17 },
    { 0, 0 },
    { 0x3ab8d220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e8d229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88d233, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8d236, 46 },
    { 0, 0 },
    { 0x3a8d238, 76 },
    { 0x7558d239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8d23c, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a8d242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea8d24f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3598d257, 61 },
    { 0, 0 },
    { 0xdba8d259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88d25d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a8d266, 72 },
    { 0, 0 },
    { 0x8c48d268, 61 },
    { 0x38c8d269, 76 },
    { 0xcf98d268, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb8d26e, 55 },
    { 0x48e8d26e, 90 },
    { 0xf458d270, 95 },
    { 0, 0 },
    { 0x59a8d272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e88d278, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948d27d, 88 },
    { 0, 0 },
    { 0xe58d27f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b08d286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac68d289, 90 },
    { 0x7198d289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb68d28d, 34 },
    { 0xa6c8d28e, 97 },
    { 0x98a8d28e, 23 },
    { 0xf7a8d290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378d294, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d48d29f, 76 },
    { 0x7a38d2a0, 51 },
    { 0x5c8d2a1, 78 },
    { 0, 0 },
    { 0x8ed8d2a3, 55 },
    { 0xb398d2a4, 76 },
    { 0x298d2a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6368d2a8, 88 },
    { 0, 0 },
    { 0x7f78d2aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d8d2ae, 43 },
    { 0xdd48d2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1398d2b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a8d2bd, 70 },
    { 0x4f28d2be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d8d2c3, 23 },
    { 0, 0 },
    { 0xcad8d2c5, 52 },
    { 0x7fc8d2c6, 55 },
    { 0, 0 },
    { 0x20c8d2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438d2cf, 76 },
    { 0, 0 },
    { 0x4b88d2d1, 43 },
    { 0x23d8d2d2, 90 },
    { 0x5608d2d1, 43 },
    { 0x4118d2d3, 23 },
    { 0, 0 },
    { 0x2e98d2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d8d2d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3808d2dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78d2df, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4408d2e3, 90 },
    { 0xd588d2e3, 67 },
    { 0x9f48d2e5, 90 },
    { 0x3d28d2e6, 90 },
    { 0x9528d2e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08d2ed, 95 },
    { 0x9c88d2ee, 33 },
    { 0xeff8d2ef, 98 },
    { 0x2718d2f0, 33 },
    { 0xa58d2ee, 61 },
    { 0, 0 },
    { 0x3258d2f3, 34 },
    { 0x1d28d2f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18d2f8, 90 },
    { 0x298d2f9, 65 },
    { 0x57d8d2fa, 87 },
    { 0x7248d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc8d302, 23 },
    { 0xd198d303, 90 },
    { 0xfac8d303, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8d309, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5818d30d, 99 },
    { 0xa6d8d30d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f78d312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe338d317, 43 },
    { 0x9998d318, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af8d31d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8d321, 33 },
    { 0xaa68d321, 23 },
    { 0x1758d323, 90 },
    { 0xdf78d324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf338d330, 97 },
    { 0x2898d330, 17 },
    { 0x2418d332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98d335, 87 },
    { 0x2a48d336, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8788d33b, 65 },
    { 0, 0 },
    { 0x2e8d33d, 68 },
    { 0, 0 },
    { 0x7c38d33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd88d342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6478d345, 55 },
    { 0, 0 },
    { 0xcf18d347, 95 },
    { 0x8178d347, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c8d34b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x2658d34e, 95 },
    { 0x5158d34e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8d353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf768d357, 33 },
    { 0x6c58d358, 95 },
    { 0, 0 },
    { 0x8078d35a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8d36c, 23 },
    { 0xb138d36d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf8d370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf28d373, 95 },
    { 0x5478d374, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2898d37a, 100 },
    { 0xa438d37b, 78 },
    { 0x2868d37c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2628d37f, 61 },
    { 0x3818d380, 95 },
    { 0xfd18d37f, 61 },
    { 0xf3c8d382, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648d38a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc428d38f, 33 },
    { 0, 0 },
    { 0xfc08d391, 78 },
    { 0x7f88d391, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608d398, 90 },
    { 0x5638d399, 87 },
    { 0xbe98d398, 78 },
    { 0, 0 },
    { 0x44c8d39c, 95 },
    { 0, 0 },
    { 0x37f8d39e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018d3a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e8d3a8, 93 },
    { 0, 0 },
    { 0xdc08d3aa, 33 },
    { 0xbb68d3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a8d3af, 90 },
    { 0, 0 },
    { 0x1398d3b1, 23 },
    { 0xde48d3b1, 95 },
    { 0xa798d3b3, 77 },
    { 0xc08d3b3, 90 },
    { 0x8f38d3b5, 99 },
    { 0xdfa8d3b6, 90 },
    { 0xcc98d3b7, 34 },
    { 0x4bf8d3b5, 90 },
    { 0xd178d3b9, 90 },
    { 0x5d38d3b3, 61 },
    { 0x53c8d3bb, 100 },
    { 0xa438d3bb, 90 },
    { 0x14a8d3bb, 33 },
    { 0x4c18d3bd, 90 },
    { 0x1bd8d3bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2788d3c4, 81 },
    { 0x61e8d3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe868d3c8, 68 },
    { 0, 0 },
    { 0x7618d3ca, 84 },
    { 0x3338d3cb, 66 },
    { 0xe928d3cc, 76 },
    { 0xcc78d3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb148d3d3, 99 },
    { 0, 0 },
    { 0x5958d3d5, 43 },
    { 0, 0 },
    { 0xdda8d3d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7458d3e3, 90 },
    { 0xd558d3e4, 90 },
    { 0x77e8d3e4, 87 },
    { 0xec58d3e4, 55 },
    { 0x2ef8d3e7, 83 },
    { 0x62d8d3e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f8d3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c8d3f6, 33 },
    { 0xa178d3f7, 99 },
    { 0xf718d3f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808d401, 33 },
    { 0x7c28d402, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608d406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b8d40d, 33 },
    { 0, 0 },
    { 0xa3d8d40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7468d415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa948d418, 78 },
    { 0x36c8d418, 55 },
    { 0x4878d419, 33 },
    { 0xbfd8d418, 100 },
    { 0, 0 },
    { 0xbb8d41d, 33 },
    { 0x3b98d41e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8d427, 76 },
    { 0xe7c8d428, 23 },
    { 0x1878d429, 90 },
    { 0x32c8d429, 61 },
    { 0x6668d42b, 33 },
    { 0x1ff8d429, 95 },
    { 0x82e8d427, 95 },
    { 0xfe68d42a, 34 },
    { 0xaa78d42f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf8d435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78d438, 52 },
    { 0, 0 },
    { 0x1978d43a, 76 },
    { 0xff88d43b, 82 },
    { 0xecc8d43c, 90 },
    { 0x27c8d43c, 90 },
    { 0x3f8d43a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a8d444, 98 },
    { 0x5168d445, 55 },
    { 0, 0 },
    { 0x2388d447, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a8d44b, 98 },
    { 0x91f8d44c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948d451, 90 },
    { 0xb98d452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108d455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2968d459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8d45f, 90 },
    { 0xef08d460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6358d463, 90 },
    { 0x50a8d463, 90 },
    { 0xda38d465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418d469, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec38d46d, 33 },
    { 0x1098d46d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8588d471, 33 },
    { 0x7878d471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d47d, 33 },
    { 0xd5b8d47d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd8d487, 90 },
    { 0xf6b8d488, 76 },
    { 0x388d489, 53 },
    { 0, 0 },
    { 0xe168d48b, 46 },
    { 0, 0 },
    { 0x1828d48d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1758d491, 75 },
    { 0, 0 },
    { 0x9948d493, 55 },
    { 0x4798d494, 90 },
    { 0x5048d495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28d499, 61 },
    { 0xf088d499, 90 },
    { 0xe458d49b, 78 },
    { 0, 0 },
    { 0x96a8d49d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a8d4a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4768d4a7, 23 },
    { 0xb838d4a8, 53 },
    { 0, 0 },
    { 0xe2f8d4aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98d4ad, 68 },
    { 0xd1d8d4ae, 23 },
    { 0x398d4ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e8d4b2, 81 },
    { 0xa6a8d4b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8528d4b7, 55 },
    { 0x9fd8d4b8, 90 },
    { 0xcc98d4b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf888d4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8d4c7, 87 },
    { 0xb08d4c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3768d4cc, 76 },
    { 0xb508d4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e8d4d3, 61 },
    { 0x7198d4d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8d4d8, 33 },
    { 0, 0 },
    { 0xb398d4da, 90 },
    { 0x94e8d4db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5358d4e6, 49 },
    { 0xa3f8d4e7, 61 },
    { 0xd8a8d4e6, 76 },
    { 0x7488d4e9, 33 },
    { 0xe738d4e6, 93 },
    { 0x5b58d4e6, 55 },
    { 0xdf88d4ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd288d4f0, 33 },
    { 0x59c8d4f1, 97 },
    { 0xf908d4f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588d4f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8758d4f8, 93 },
    { 0xa928d4f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6468d4fc, 43 },
    { 0x5208d4fd, 52 },
    { 0, 0 },
    { 0x5368d4ff, 93 },
    { 0, 0 },
    { 0x73d8d501, 98 },
    { 0xa528d502, 100 },
    { 0x41c8d502, 43 },
    { 0x5a58d503, 95 },
    { 0, 0 },
    { 0x1898d506, 66 },
    { 0, 0 },
    { 0x3918d508, 47 },
    { 0x5948d509, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8d50c, 95 },
    { 0xd778d50d, 85 },
    { 0, 0 },
    { 0xcfb8d50f, 52 },
    { 0x31b8d510, 84 },
    { 0x7148d511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8d515, 91 },
    { 0x1798d515, 90 },
    { 0xa8d515, 98 },
    { 0, 0 },
    { 0x9cf8d519, 77 },
    { 0, 0 },
    { 0x3a18d51b, 17 },
    { 0x1e28d51c, 98 },
    { 0x14d8d51b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb98d521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a8d527, 81 },
    { 0, 0 },
    { 0xbb68d529, 98 },
    { 0x7968d529, 33 },
    { 0xe738d52a, 90 },
    { 0, 0 },
    { 0xf718d52d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f08d532, 95 },
    { 0xca88d533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb658d53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48d540, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408d543, 76 },
    { 0x1888d543, 87 },
    { 0x7338d544, 34 },
    { 0x5c68d543, 61 },
    { 0xab58d547, 76 },
    { 0x8848d548, 98 },
    { 0x95b8d545, 33 },
    { 0xb338d54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8908d551, 75 },
    { 0x9f58d552, 95 },
    { 0xc2a8d553, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf988d55c, 87 },
    { 0x4818d55d, 95 },
    { 0x5708d55e, 93 },
    { 0x5368d55f, 70 },
    { 0x9168d560, 61 },
    { 0x5e08d561, 23 },
    { 0xb008d562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8d567, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a8d56b, 66 },
    { 0xc3a8d56c, 100 },
    { 0xca98d56d, 61 },
    { 0, 0 },
    { 0x5fd8d56f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c8d57a, 34 },
    { 0x7b8d57b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b88d580, 61 },
    { 0, 0 },
    { 0x5ad8d582, 66 },
    { 0xd3c8d583, 95 },
    { 0x4e08d584, 48 },
    { 0xb778d585, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf48d58b, 33 },
    { 0xcce8d58c, 61 },
    { 0xefc8d58b, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b18d591, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba88d596, 33 },
    { 0, 0 },
    { 0x9c8d598, 33 },
    { 0x6188d599, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc28d59c, 23 },
    { 0xb538d59d, 90 },
    { 0x9e08d59e, 61 },
    { 0, 0 },
    { 0xf908d5a0, 55 },
    { 0x80b8d5a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d5a6, 90 },
    { 0x91b8d5a7, 61 },
    { 0, 0 },
    { 0x99a8d5a9, 90 },
    { 0, 0 },
    { 0x8c88d5ab, 55 },
    { 0, 0 },
    { 0x6be8d5ad, 78 },
    { 0x7cc8d5ad, 68 },
    { 0, 0 },
    { 0xdb8d5b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18d5b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6458d5b8, 100 },
    { 0x4848d5b9, 77 },
    { 0x4328d5ba, 23 },
    { 0, 0 },
    { 0xbd18d5bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa328d5c0, 95 },
    { 0, 0 },
    { 0xca08d5c2, 100 },
    { 0, 0 },
    { 0x99e8d5c4, 34 },
    { 0x208d5c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa698d5c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908d5cd, 43 },
    { 0xaae8d5ce, 76 },
    { 0xacd8d5cf, 90 },
    { 0x3e28d5cd, 76 },
    { 0xb858d5d1, 61 },
    { 0x3698d5d2, 66 },
    { 0x7af8d5d3, 40 },
    { 0xad78d5d1, 99 },
    { 0xfdb8d5d5, 61 },
    { 0x33f8d5d6, 80 },
    { 0x2ab8d5d7, 40 },
    { 0x6938d5d7, 90 },
    { 0xfc8d5d9, 52 },
    { 0xb978d5d9, 95 },
    { 0xc9a8d5db, 33 },
    { 0x4078d5ce, 90 },
    { 0x9b68d5d2, 90 },
    { 0x37e8d5de, 33 },
    { 0x26c8d5de, 98 },
    { 0x38a8d5de, 98 },
    { 0xf768d5e0, 90 },
    { 0xaf08d5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe8d5e6, 78 },
    { 0xa1d8d5e7, 33 },
    { 0xa9c8d5e7, 55 },
    { 0xb1f8d5e9, 90 },
    { 0xa198d5ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c28d5f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd8d5f7, 20 },
    { 0x5a68d5f7, 76 },
    { 0xccc8d5f9, 23 },
    { 0x3678d5f8, 93 },
    { 0x84e8d5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18d605, 34 },
    { 0, 0 },
    { 0xb6d8d607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa8d60a, 55 },
    { 0x9f18d60a, 34 },
    { 0x65f8d60a, 93 },
    { 0xba58d60b, 93 },
    { 0, 0 },
    { 0x1f78d60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f88d616, 33 },
    { 0, 0 },
    { 0xdc18d618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc8d61b, 46 },
    { 0x11e8d61b, 97 },
    { 0x4988d61d, 78 },
    { 0x70a8d61e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708d623, 90 },
    { 0x31d8d624, 95 },
    { 0x3b88d625, 95 },
    { 0x78d624, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28d62a, 33 },
    { 0x95d8d62a, 66 },
    { 0x3fd8d62c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e8d62f, 90 },
    { 0x6418d630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78d635, 43 },
    { 0x4e98d636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8d639, 76 },
    { 0xaa8d639, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158d63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb98d641, 100 },
    { 0x7dd8d641, 87 },
    { 0xf218d642, 76 },
    { 0xa948d643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4468d647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde38d64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3938d64e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58d657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8d65a, 34 },
    { 0x4a58d65b, 55 },
    { 0x8ff8d65b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8d65f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b8d662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b8d665, 66 },
    { 0, 0 },
    { 0xb498d667, 91 },
    { 0x5308d667, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058d66c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd728d679, 61 },
    { 0x3f88d67a, 90 },
    { 0, 0 },
    { 0xb188d67c, 33 },
    { 0, 0 },
    { 0x78d8d67e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa008d684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c8d690, 23 },
    { 0x9888d691, 76 },
    { 0, 0 },
    { 0x7c88d693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8d696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf078d69b, 97 },
    { 0xe5e8d69c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18d69f, 61 },
    { 0x2188d6a0, 61 },
    { 0x95d8d6a1, 23 },
    { 0, 0 },
    { 0x1dd8d6a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8d6a6, 55 },
    { 0xb618d6a7, 93 },
    { 0, 0 },
    { 0xbb28d6a9, 98 },
    { 0xe908d6a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d28d6ad, 90 },
    { 0, 0 },
    { 0x3588d6af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b8d6b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48d6c0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708d6c6, 90 },
    { 0x8d08d6c7, 90 },
    { 0xb878d6c6, 90 },
    { 0, 0 },
    { 0xbe58d6ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb998d6cf, 78 },
    { 0x5488d6d0, 76 },
    { 0x48b8d6d0, 100 },
    { 0x1a58d6d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8d6d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d48d6d9, 34 },
    { 0xcbb8d6d9, 33 },
    { 0, 0 },
    { 0x118d6dc, 85 },
    { 0x8608d6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398d6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c8d6e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb558d6e6, 76 },
    { 0, 0 },
    { 0x7128d6e8, 90 },
    { 0x6548d6e9, 77 },
    { 0xa308d6e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8d6ef, 90 },
    { 0, 0 },
    { 0xcd78d6f1, 23 },
    { 0xd248d6f2, 23 },
    { 0, 0 },
    { 0xa1a8d6f4, 93 },
    { 0xfca8d6f5, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab8d6f8, 33 },
    { 0x3bb8d6f8, 81 },
    { 0xefc8d6fa, 81 },
    { 0x6c08d6f8, 23 },
    { 0x3508d6fc, 95 },
    { 0xcbb8d6fd, 98 },
    { 0x6e88d6fc, 40 },
    { 0xf7a8d6ff, 95 },
    { 0x8368d700, 23 },
    { 0x9928d700, 61 },
    { 0, 0 },
    { 0x3988d703, 76 },
    { 0x7668d703, 90 },
    { 0, 0 },
    { 0xa3d8d706, 95 },
    { 0xa348d707, 98 },
    { 0x4d48d708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808d710, 98 },
    { 0xa6b8d710, 34 },
    { 0x94d8d712, 34 },
    { 0x4578d713, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58d71c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb768d71f, 33 },
    { 0, 0 },
    { 0x18f8d721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d727, 61 },
    { 0xadc8d728, 40 },
    { 0xc708d729, 66 },
    { 0x7b38d72a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98d72d, 84 },
    { 0x2578d72e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa268d731, 23 },
    { 0, 0 },
    { 0xedb8d733, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc728d737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc8d73e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8d742, 55 },
    { 0xfcf8d743, 93 },
    { 0, 0 },
    { 0x5458d745, 33 },
    { 0xc908d746, 23 },
    { 0xc1f8d746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf428d755, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388d75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18d761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6498d765, 43 },
    { 0x87a8d765, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe8d76a, 43 },
    { 0x46b8d76b, 99 },
    { 0x5e38d76a, 95 },
    { 0, 0 },
    { 0x12d8d76e, 55 },
    { 0, 0 },
    { 0xdfa8d770, 61 },
    { 0xb9d8d771, 76 },
    { 0x7a8d772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e8d776, 52 },
    { 0x18c8d777, 91 },
    { 0x50e8d778, 98 },
    { 0, 0 },
    { 0xe6e8d77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b8d77f, 99 },
    { 0x5408d780, 76 },
    { 0x8d68d780, 90 },
    { 0xe508d782, 93 },
    { 0x56e8d783, 95 },
    { 0xd6b8d781, 23 },
    { 0x4108d785, 98 },
    { 0x4c98d786, 95 },
    { 0xfea8d783, 94 },
    { 0xe558d788, 61 },
    { 0x6b18d787, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb18d799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638d7a4, 85 },
    { 0, 0 },
    { 0x1668d7a6, 96 },
    { 0xc6e8d7a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b8d7ad, 55 },
    { 0xa618d7ad, 66 },
    { 0xd838d7af, 34 },
    { 0xb278d7b0, 66 },
    { 0x6ba8d7b1, 66 },
    { 0xa5d8d7b2, 90 },
    { 0, 0 },
    { 0xf408d7b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c28d7b9, 33 },
    { 0, 0 },
    { 0xa2d8d7bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe048d7bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba8d7c5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac98d7c9, 33 },
    { 0x7748d7ca, 52 },
    { 0, 0 },
    { 0xaab8d7cc, 90 },
    { 0x7a58d7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f28d7d1, 91 },
    { 0xccb8d7d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec8d7d5, 90 },
    { 0x72a8d7d6, 33 },
    { 0xc2a8d7d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa28d7dc, 52 },
    { 0, 0 },
    { 0x45f8d7de, 76 },
    { 0xc858d7df, 43 },
    { 0x2ca8d7df, 23 },
    { 0, 0 },
    { 0x1068d7e2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf58d7e6, 95 },
    { 0x9088d7e6, 43 },
    { 0, 0 },
    { 0xbc28d7e9, 98 },
    { 0x7428d7e9, 98 },
    { 0xe7b8d7e9, 95 },
    { 0xd5c8d7ec, 23 },
    { 0x8a58d7ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8248d7f1, 90 },
    { 0xef68d7f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e8d7f7, 100 },
    { 0x7b28d7f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8d7fb, 23 },
    { 0x5168d7fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe318d802, 90 },
    { 0xbfc8d803, 61 },
    { 0, 0 },
    { 0x3ec8d805, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c8d809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8d80d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d8d813, 52 },
    { 0x9f68d814, 84 },
    { 0x2fe8d814, 46 },
    { 0x5768d815, 95 },
    { 0x12a8d817, 98 },
    { 0xabb8d813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1118d81e, 99 },
    { 0xe628d81e, 55 },
    { 0xca68d820, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f8d824, 93 },
    { 0x9d18d824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e8d830, 46 },
    { 0xc8b8d831, 98 },
    { 0xa8f8d832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df8d83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6808d83f, 95 },
    { 0x9d68d840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998d843, 99 },
    { 0x1a18d843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a8d847, 98 },
    { 0, 0 },
    { 0xf518d849, 95 },
    { 0x1058d84a, 66 },
    { 0x1318d849, 61 },
    { 0, 0 },
    { 0x6158d84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5108d850, 85 },
    { 0xd78d851, 78 },
    { 0, 0 },
    { 0xb8f8d853, 67 },
    { 0x3f08d853, 95 },
    { 0, 0 },
    { 0xe7e8d856, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48d85b, 92 },
    { 0xf538d85b, 43 },
    { 0, 0 },
    { 0xf378d85e, 88 },
    { 0, 0 },
    { 0x6898d860, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda18d863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f8d866, 46 },
    { 0xf848d867, 85 },
    { 0, 0 },
    { 0x6bb8d869, 90 },
    { 0, 0 },
    { 0xc068d86b, 85 },
    { 0x8368d86c, 61 },
    { 0x4248d86d, 90 },
    { 0, 0 },
    { 0xe828d86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8d874, 90 },
    { 0x8c18d875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b8d878, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5018d87c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d18d880, 33 },
    { 0x348d881, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf8d884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c08d889, 95 },
    { 0xc08d889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e38d88d, 43 },
    { 0xe8c8d88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf918d893, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2348d897, 84 },
    { 0, 0 },
    { 0xd1b8d899, 76 },
    { 0x6148d89a, 33 },
    { 0xd1a8d89b, 90 },
    { 0xa628d89c, 100 },
    { 0x1d88d89b, 33 },
    { 0x19c8d89d, 34 },
    { 0x6858d89e, 98 },
    { 0x5208d8a0, 100 },
    { 0x5ad8d89e, 100 },
    { 0x5d58d89b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398d8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808d8ac, 87 },
    { 0x37b8d8ad, 78 },
    { 0x7938d8ad, 33 },
    { 0, 0 },
    { 0x4038d8b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858d8b5, 90 },
    { 0xfaa8d8b5, 90 },
    { 0, 0 },
    { 0xbb98d8b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8d8bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd8d8bf, 76 },
    { 0x8638d8c0, 98 },
    { 0x4ed8d8c1, 43 },
    { 0, 0 },
    { 0xe3c8d8c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d8c7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d8ca, 78 },
    { 0, 0 },
    { 0x4928d8cc, 33 },
    { 0x1ea8d8cd, 91 },
    { 0xf608d8cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed68d8d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868d8db, 90 },
    { 0x4548d8dc, 75 },
    { 0, 0 },
    { 0xa738d8de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9808d8e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9238d8ec, 43 },
    { 0, 0 },
    { 0x39f8d8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb008d8f1, 34 },
    { 0xa8a8d8f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2288d8f6, 87 },
    { 0x5958d8f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88d8fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8d8fe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d38d907, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58d90e, 99 },
    { 0xc928d90f, 33 },
    { 0x44f8d910, 90 },
    { 0, 0 },
    { 0x7a58d912, 95 },
    { 0x5b58d913, 95 },
    { 0, 0 },
    { 0x8da8d915, 76 },
    { 0x7988d916, 91 },
    { 0x6388d917, 55 },
    { 0, 0 },
    { 0x2d98d919, 98 },
    { 0x9098d91a, 55 },
    { 0x8ab8d91b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528d923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78d926, 33 },
    { 0x7538d927, 76 },
    { 0xa958d928, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92e, 94 },
    { 0x2098d92f, 33 },
    { 0x218d92e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa678d933, 23 },
    { 0xb518d934, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b8d939, 98 },
    { 0xc878d93a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9338d93e, 43 },
    { 0xe538d93f, 35 },
    { 0x72a8d93f, 76 },
    { 0x9768d941, 34 },
    { 0x33f8d93e, 23 },
    { 0, 0 },
    { 0x7b68d944, 23 },
    { 0xec08d945, 77 },
    { 0, 0 },
    { 0x1e08d947, 87 },
    { 0, 0 },
    { 0x8db8d949, 61 },
    { 0x76b8d94a, 98 },
    { 0, 0 },
    { 0xdb38d94c, 43 },
    { 0x3b38d94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8d950, 61 },
    { 0xcc38d951, 98 },
    { 0x6028d952, 43 },
    { 0x6a28d950, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8948d957, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f8d95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed8d95e, 98 },
    { 0xd858d95f, 95 },
    { 0x5528d95f, 95 },
    { 0xf268d961, 98 },
    { 0xcc88d961, 23 },
    { 0x9c38d962, 34 },
    { 0x4828d963, 90 },
    { 0x3918d962, 75 },
    { 0xf2b8d966, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6408d96b, 90 },
    { 0x40f8d96c, 61 },
    { 0x2178d96d, 90 },
    { 0x3ec8d96e, 76 },
    { 0x6f68d96d, 87 },
    { 0, 0 },
    { 0x9e28d971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818d974, 95 },
    { 0xbb48d975, 85 },
    { 0x7978d976, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a8d97b, 98 },
    { 0x4bc8d97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8d97f, 33 },
    { 0xee18d97f, 84 },
    { 0xde28d981, 93 },
    { 0xfff8d982, 68 },
    { 0x67b8d97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd908d986, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998d98c, 55 },
    { 0x2918d98d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a8d991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6708d994, 61 },
    { 0x4158d995, 85 },
    { 0x2658d995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7148d99e, 26 },
    { 0x68a8d99f, 99 },
    { 0x1368d9a0, 42 },
    { 0xac48d9a0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e8d9a5, 23 },
    { 0, 0 },
    { 0x1878d9a7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d9b0, 23 },
    { 0xbf18d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa88d9c0, 23 },
    { 0x62d8d9c0, 40 },
    { 0, 0 },
    { 0x60b8d9c3, 77 },
    { 0xad08d9c4, 53 },
    { 0xd978d9c4, 23 },
    { 0x5cb8d9c6, 78 },
    { 0x6518d9c5, 95 },
    { 0x3698d9c6, 76 },
    { 0x35d8d9c9, 76 },
    { 0x7768d9ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c58d9cd, 97 },
    { 0x6768d9cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4828d9d2, 95 },
    { 0x7508d9d3, 34 },
    { 0x20c8d9d3, 95 },
    { 0x2168d9d5, 78 },
    { 0, 0 },
    { 0xe768d9d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e8d9da, 95 },
    { 0xd648d9db, 43 },
    { 0, 0 },
    { 0xef98d9dd, 78 },
    { 0x3358d9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28d9e1, 76 },
    { 0xf478d9e2, 88 },
    { 0xf9b8d9e2, 90 },
    { 0xc7b8d9e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d8d9ea, 87 },
    { 0, 0 },
    { 0xb0f8d9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4278d9f1, 90 },
    { 0x4d68d9f2, 90 },
    { 0, 0 },
    { 0xa398d9f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6318d9f7, 46 },
    { 0x7ac8d9f8, 23 },
    { 0x92b8d9f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868d9fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef8da02, 68 },
    { 0, 0 },
    { 0x8fd8da04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee8da09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff88da0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec68da12, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b8da16, 78 },
    { 0x38c8da17, 85 },
    { 0x8608da17, 52 },
    { 0, 0 },
    { 0xd718da1a, 23 },
    { 0x898da1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf8da1e, 17 },
    { 0xd418da1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8da22, 53 },
    { 0xe168da23, 61 },
    { 0x3918da24, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f98da2a, 90 },
    { 0x17e8da2b, 52 },
    { 0x638da2b, 55 },
    { 0x1dd8da2b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a8da30, 98 },
    { 0xca38da30, 61 },
    { 0x52d8da32, 66 },
    { 0, 0 },
    { 0x4e88da34, 33 },
    { 0xb298da34, 61 },
    { 0, 0 },
    { 0x168da37, 43 },
    { 0xdd8da37, 46 },
    { 0xc098da39, 55 },
    { 0xe8da38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd118da40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b8da43, 33 },
    { 0x3a78da43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88da47, 77 },
    { 0xe4a8da48, 84 },
    { 0, 0 },
    { 0xdf88da4a, 66 },
    { 0x7918da4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848da4e, 95 },
    { 0, 0 },
    { 0x49d8da50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888da54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d8da57, 98 },
    { 0x5e28da57, 43 },
    { 0, 0 },
    { 0x7868da5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5088da63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8da68, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda98da73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb88da76, 56 },
    { 0x96b8da77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8da85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f58da8f, 98 },
    { 0xc228da90, 100 },
    { 0x48b8da90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98da94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6298da98, 55 },
    { 0xa4e8da99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7458da9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b8daa1, 90 },
    { 0, 0 },
    { 0x91d8daa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e68daa6, 23 },
    { 0, 0 },
    { 0x2a38daa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8daab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe068dab3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b58dab6, 23 },
    { 0, 0 },
    { 0xa58dab8, 56 },
    { 0x3e18dab9, 98 },
    { 0x81a8dab8, 78 },
    { 0, 0 },
    { 0x2b08dabc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98dac0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea8dac3, 66 },
    { 0xfaf8dac4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3178dac7, 91 },
    { 0x83b8dac8, 43 },
    { 0xb648dac8, 95 },
    { 0xd138daca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b38dace, 95 },
    { 0x50f8dace, 98 },
    { 0x2ca8dace, 93 },
    { 0x8a8dad1, 84 },
    { 0, 0 },
    { 0xb58dad3, 90 },
    { 0x8e98dad4, 77 },
    { 0, 0 },
    { 0x1f78dad6, 85 },
    { 0x3428dad6, 90 },
    { 0, 0 },
    { 0xd018dad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b88dade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3808dae3, 98 },
    { 0xfb78dae4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8dae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448daec, 51 },
    { 0x5e68daed, 52 },
    { 0x4028daee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc88daf1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8daf7, 75 },
    { 0xd3f8daf8, 87 },
    { 0, 0 },
    { 0x98e8dafa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708dafd, 34 },
    { 0x9168dafe, 33 },
    { 0xaf78daff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1518db02, 55 },
    { 0x12b8db03, 17 },
    { 0xea28db03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac48db08, 95 },
    { 0xff78db09, 76 },
    { 0x1bd8db0a, 33 },
    { 0x60b8db0a, 27 },
    { 0, 0 },
    { 0x5758db0d, 34 },
    { 0xe248db0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb8db14, 76 },
    { 0x4a58db15, 85 },
    { 0x34a8db15, 98 },
    { 0x9648db16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68db1a, 33 },
    { 0, 0 },
    { 0x7b28db1c, 33 },
    { 0xe0c8db1c, 23 },
    { 0xd3b8db1e, 61 },
    { 0xb8e8db1f, 92 },
    { 0x8cc8db20, 80 },
    { 0x2838db21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6768db26, 98 },
    { 0x9288db27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398db2b, 78 },
    { 0, 0 },
    { 0xb178db2d, 87 },
    { 0x4698db2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8db32, 90 },
    { 0x8fc8db33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa028db37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818db3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38db40, 90 },
    { 0, 0 },
    { 0xd108db42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb078db45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6388db49, 61 },
    { 0, 0 },
    { 0x8858db4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df8db4e, 61 },
    { 0x9788db4f, 69 },
    { 0x6738db4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df8db55, 90 },
    { 0, 0 },
    { 0xed38db57, 100 },
    { 0x38e8db58, 95 },
    { 0x4e98db57, 34 },
    { 0xed78db5a, 90 },
    { 0x66d8db5a, 95 },
    { 0x9e98db5c, 23 },
    { 0xef78db5d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d28db61, 61 },
    { 0x8b08db62, 33 },
    { 0x24d8db61, 95 },
    { 0x3ec8db61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a8db67, 17 },
    { 0x258db68, 76 },
    { 0, 0 },
    { 0xd318db6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158db6f, 34 },
    { 0x5f48db6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da8db79, 52 },
    { 0, 0 },
    { 0xf248db7b, 43 },
    { 0, 0 },
    { 0xbd08db7d, 84 },
    { 0x198db7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3838db81, 91 },
    { 0x2ed8db82, 61 },
    { 0, 0 },
    { 0xe448db84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db87, 85 },
    { 0, 0 },
    { 0xa598db89, 90 },
    { 0x3408db8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38db94, 98 },
    { 0x92c8db94, 90 },
    { 0xcd88db95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8db9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608db9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd18dba1, 76 },
    { 0, 0 },
    { 0x87d8dba3, 33 },
    { 0xe908dba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c78dba7, 76 },
    { 0x29c8dba7, 93 },
    { 0xb5a8dba7, 34 },
    { 0, 0 },
    { 0xa268dbab, 101 },
    { 0, 0 },
    { 0x8fc8dbad, 90 },
    { 0x5ad8dbae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318dbba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a28dbc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd88dbc3, 33 },
    { 0x3ff8dbc3, 83 },
    { 0xa0e8dbc5, 58 },
    { 0, 0 },
    { 0x2c8dbc7, 93 },
    { 0, 0 },
    { 0xaaf8dbc9, 84 },
    { 0xc8b8dbca, 61 },
    { 0x9b98dbcb, 46 },
    { 0xbd18dbcc, 90 },
    { 0x48d8dbc9, 95 },
    { 0xa778dbce, 93 },
    { 0xedc8dbce, 77 },
    { 0x9e58dbd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f08dbd4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa8dbd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b8dbdc, 33 },
    { 0, 0 },
    { 0xad68dbde, 23 },
    { 0xedb8dbdf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b8dbe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f8dbe9, 90 },
    { 0, 0 },
    { 0x6b68dbeb, 46 },
    { 0, 0 },
    { 0x8d8dbed, 91 },
    { 0xb738dbee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be8dbf1, 81 },
    { 0x77f8dbf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2188dbf8, 90 },
    { 0xc9f8dbf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1988dbfe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1148dc08, 95 },
    { 0, 0 },
    { 0x76e8dc0a, 90 },
    { 0, 0 },
    { 0x8ab8dc0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e8dc11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd8dc15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff68dc1a, 43 },
    { 0xf338dc1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8dc1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58dc26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d08dc2d, 46 },
    { 0xd208dc2d, 76 },
    { 0x65b8dc2e, 90 },
    { 0x1ef8dc2f, 91 },
    { 0, 0 },
    { 0x55a8dc32, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c68dc3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f8dc3f, 43 },
    { 0xbb98dc3f, 34 },
    { 0x8e78dc41, 81 },
    { 0xa958dc42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378dc45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8dc50, 76 },
    { 0xa98dc50, 55 },
    { 0, 0 },
    { 0xb468dc53, 76 },
    { 0xe918dc54, 90 },
    { 0x7cc8dc55, 90 },
    { 0xa798dc56, 43 },
    { 0x8238dc53, 44 },
    { 0x398dc53, 23 },
    { 0x5098dc54, 23 },
    { 0x7448dc5a, 23 },
    { 0xf0c8dc5b, 90 },
    { 0x47c8dc5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6148dc66, 98 },
    { 0, 0 },
    { 0x6618dc68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7448dc6b, 34 },
    { 0, 0 },
    { 0xee58dc6d, 33 },
    { 0xcb78dc6e, 75 },
    { 0x4368dc6f, 61 },
    { 0x3638dc6e, 68 },
    { 0x1f98dc6f, 76 },
    { 0x7ce8dc6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5548dc77, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808dc7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c08dc82, 90 },
    { 0, 0 },
    { 0xf868dc84, 34 },
    { 0x63b8dc85, 46 },
    { 0, 0 },
    { 0x23d8dc87, 90 },
    { 0xf758dc87, 95 },
    { 0xe058dc89, 83 },
    { 0, 0 },
    { 0x4f58dc8b, 95 },
    { 0x46a8dc8c, 33 },
    { 0x3a8dc8d, 33 },
    { 0x9398dc8e, 95 },
    { 0, 0 },
    { 0x95e8dc90, 95 },
    { 0xe6d8dc91, 61 },
    { 0x6f68dc92, 65 },
    { 0x1858dc92, 61 },
    { 0, 0 },
    { 0x7c78dc95, 81 },
    { 0x9438dc96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8dc9b, 98 },
    { 0xaeb8dc9c, 66 },
    { 0, 0 },
    { 0xfc58dc9e, 90 },
    { 0, 0 },
    { 0xb638dca0, 61 },
    { 0xec98dca0, 53 },
    { 0x2ef8dca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8dca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8dcaa, 61 },
    { 0, 0 },
    { 0xbb68dcac, 90 },
    { 0x1fd8dcad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a8dcb4, 46 },
    { 0xdd98dcb4, 23 },
    { 0x7b18dcb6, 90 },
    { 0xae68dcb7, 90 },
    { 0xf1c8dcb8, 55 },
    { 0x4da8dcb9, 95 },
    { 0x2498dcba, 61 },
    { 0, 0 },
    { 0x4328dcbc, 33 },
    { 0x1288dcbc, 33 },
    { 0xc6a8dcbc, 78 },
    { 0x8928dcbf, 66 },
    { 0x63a8dcc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98dcc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28dcce, 76 },
    { 0x8428dccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a8dcd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7278dcd5, 90 },
    { 0, 0 },
    { 0xfd18dcd7, 55 },
    { 0x5cb8dcd8, 93 },
    { 0xbe68dcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd658dcde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d18dce4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688dceb, 88 },
    { 0, 0 },
    { 0xcf38dced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d8dcf1, 61 },
    { 0x6758dcf2, 53 },
    { 0x458dcf1, 98 },
    { 0x1518dcf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb458dcfa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f8dcff, 34 },
    { 0x1948dd00, 90 },
    { 0x3128dd01, 26 },
    { 0x5578dd01, 90 },
    { 0x4d18dd02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a78dd08, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd8dd10, 95 },
    { 0x1c98dd11, 23 },
    { 0xa98dd11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2708dd18, 76 },
    { 0, 0 },
    { 0x8088dd1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8dd1f, 33 },
    { 0x9918dd20, 90 },
    { 0xea58dd21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538dd2d, 23 },
    { 0xb998dd2d, 66 },
    { 0x6868dd2f, 91 },
    { 0x7748dd30, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf248dd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18dd38, 61 },
    { 0, 0 },
    { 0x3348dd3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d98dd40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38dd43, 85 },
    { 0x5fc8dd44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08dd47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b8dd4a, 65 },
    { 0, 0 },
    { 0x1938dd4c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f8dd50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dd53, 55 },
    { 0x2138dd54, 68 },
    { 0xc5d8dd54, 77 },
    { 0xe838dd55, 61 },
    { 0x4cf8dd57, 33 },
    { 0xe5b8dd54, 90 },
    { 0xeae8dd59, 61 },
    { 0x9878dd5a, 78 },
    { 0xab88dd5b, 90 },
    { 0x6188dd5b, 95 },
    { 0x1ed8dd58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b8dd62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3798dd66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318dd6e, 93 },
    { 0x7328dd6e, 76 },
    { 0, 0 },
    { 0xd718dd71, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b8dd79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6718dd7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf138dd81, 78 },
    { 0xa48dd81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908dd85, 33 },
    { 0x698dd85, 33 },
    { 0xeb68dd85, 77 },
    { 0, 0 },
    { 0x2d8dd89, 90 },
    { 0, 0 },
    { 0xa0d8dd8b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7198dd91, 34 },
    { 0, 0 },
    { 0xdaa8dd93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958dd9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6838dda7, 90 },
    { 0x2048dda7, 76 },
    { 0, 0 },
    { 0xeda8ddaa, 34 },
    { 0, 0 },
    { 0xa598ddac, 33 },
    { 0xb0e8ddac, 66 },
    { 0, 0 },
    { 0x53f8ddaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a8ddb3, 66 },
    { 0, 0 },
    { 0xf7e8ddb5, 95 },
    { 0x5428ddb6, 98 },
    { 0xd838ddb7, 61 },
    { 0x94b8ddb8, 98 },
    { 0x8458ddb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f88ddbc, 93 },
    { 0x4a8ddbd, 76 },
    { 0x55e8ddbe, 61 },
    { 0x3768ddbe, 34 },
    { 0x4638ddbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb618ddc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8ddc6, 67 },
    { 0xe458ddc6, 91 },
    { 0x29f8ddc8, 76 },
    { 0, 0 },
    { 0x5508ddca, 99 },
    { 0x74a8ddca, 43 },
    { 0xea18ddcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf28ddcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4378ddd2, 84 },
    { 0x4cc8ddd3, 93 },
    { 0x9958ddd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe728dddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5238dde0, 98 },
    { 0, 0 },
    { 0x4e08dde2, 33 },
    { 0x2768dde3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c8dde6, 83 },
    { 0xb118dde7, 23 },
    { 0x28a8dde8, 33 },
    { 0, 0 },
    { 0xd078ddea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1848ddee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d8ddf1, 33 },
    { 0x4f88ddf2, 40 },
    { 0, 0 },
    { 0x7278ddf4, 23 },
    { 0x2618ddf5, 90 },
    { 0x4398ddf6, 76 },
    { 0, 0 },
    { 0x4e98ddf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ddfc, 69 },
    { 0x4738ddfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278de00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c68de03, 78 },
    { 0, 0 },
    { 0xa5e8de05, 23 },
    { 0, 0 },
    { 0xd1e8de07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf8de0b, 33 },
    { 0x74c8de0c, 61 },
    { 0xda58de0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f08de12, 76 },
    { 0xc388de13, 61 },
    { 0xf3d8de12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8de19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa188de1d, 66 },
    { 0, 0 },
    { 0xff8de1f, 61 },
    { 0, 0 },
    { 0x3dd8de21, 43 },
    { 0x2fe8de22, 87 },
    { 0x6338de23, 91 },
    { 0x87c8de24, 90 },
    { 0xff58de25, 61 },
    { 0xe718de25, 95 },
    { 0, 0 },
    { 0xa2f8de28, 98 },
    { 0, 0 },
    { 0x77a8de2a, 61 },
    { 0xa378de2b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd8de2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d68de39, 90 },
    { 0x8af8de3a, 95 },
    { 0xc388de3b, 33 },
    { 0, 0 },
    { 0xa6d8de3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1948de45, 34 },
    { 0x3b28de45, 33 },
    { 0xa3b8de45, 68 },
    { 0x8768de48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8de4b, 33 },
    { 0x5ba8de4c, 33 },
    { 0, 0 },
    { 0xb428de4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4938de61, 91 },
    { 0xbfa8de62, 99 },
    { 0x3908de62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb18de66, 46 },
    { 0x77e8de67, 78 },
    { 0, 0 },
    { 0x6e08de69, 84 },
    { 0x96a8de69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4888de6d, 98 },
    { 0xb408de6e, 17 },
    { 0x5918de6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58de72, 34 },
    { 0xf058de73, 76 },
    { 0xaa48de72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8de7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd078de81, 55 },
    { 0x58de82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa8de8b, 61 },
    { 0x8998de8c, 33 },
    { 0xcbb8de8b, 17 },
    { 0x7708de8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e8de91, 76 },
    { 0, 0 },
    { 0x7988de93, 33 },
    { 0x47a8de93, 61 },
    { 0xbab8de95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda08de98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d8de9b, 78 },
    { 0xfcd8de9c, 23 },
    { 0x8048de9d, 81 },
    { 0x6468de9e, 43 },
    { 0xf4d8de9f, 46 },
    { 0x5038de9f, 95 },
    { 0, 0 },
    { 0x9398dea2, 93 },
    { 0x3218dea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4828dea7, 66 },
    { 0x8ba8dea8, 46 },
    { 0xf2e8dea7, 55 },
    { 0x9c48dea9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb8deaf, 23 },
    { 0xb658deb0, 55 },
    { 0x268deaf, 90 },
    { 0x4a68deb2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d18deba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee48debe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9598dec2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48dec6, 61 },
    { 0xa208dec7, 95 },
    { 0x43e8dec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1718decb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec28dece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c8ded1, 90 },
    { 0, 0 },
    { 0x1d68ded3, 90 },
    { 0xe1d8ded3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a8ded8, 90 },
    { 0x1048ded8, 96 },
    { 0, 0 },
    { 0x9ee8dedb, 66 },
    { 0x4ce8dedb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638dee0, 82 },
    { 0x2b98dee1, 95 },
    { 0x6028dee2, 90 },
    { 0xd598dee3, 95 },
    { 0xfa38dee4, 98 },
    { 0x9d68dee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c8deec, 95 },
    { 0xb778deed, 23 },
    { 0, 0 },
    { 0x6cd8deef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe28def6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4178def9, 61 },
    { 0, 0 },
    { 0x7948defb, 100 },
    { 0xdf88defc, 97 },
    { 0x5778defc, 90 },
    { 0xe0a8defe, 33 },
    { 0x7d58deff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8df02, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8058df06, 90 },
    { 0xdd78df06, 90 },
    { 0, 0 },
    { 0x4f8df09, 76 },
    { 0x79f8df09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958df0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488df14, 95 },
    { 0x2048df15, 78 },
    { 0, 0 },
    { 0xf198df17, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d48df1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd28df1f, 90 },
    { 0xbbd8df20, 23 },
    { 0xb8e8df21, 99 },
    { 0x6628df1f, 33 },
    { 0, 0 },
    { 0x5a28df24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c68df29, 34 },
    { 0xb338df2a, 33 },
    { 0x16d8df29, 99 },
    { 0x6408df2a, 90 },
    { 0x2bc8df2d, 77 },
    { 0x5378df2e, 55 },
    { 0x2e48df2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8df33, 23 },
    { 0x8e48df34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff8df39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d8df3c, 43 },
    { 0, 0 },
    { 0x5698df3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418df42, 98 },
    { 0x4f68df43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8738df4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a8df4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c8df56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf8df5e, 61 },
    { 0xdcb8df5e, 90 },
    { 0, 0 },
    { 0xa578df61, 61 },
    { 0xa18df62, 61 },
    { 0x69c8df63, 81 },
    { 0xf408df61, 99 },
    { 0, 0 },
    { 0xa078df66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c28df6a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8df74, 98 },
    { 0x84e8df75, 61 },
    { 0x4168df75, 33 },
    { 0xffa8df77, 98 },
    { 0, 0 },
    { 0x4a8df79, 76 },
    { 0, 0 },
    { 0x9d58df7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98df7e, 23 },
    { 0xac38df7f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da8df82, 90 },
    { 0, 0 },
    { 0x25e8df84, 61 },
    { 0, 0 },
    { 0x968df86, 33 },
    { 0, 0 },
    { 0xdb78df88, 88 },
    { 0, 0 },
    { 0x4ae8df8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b8df8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae68df91, 98 },
    { 0, 0 },
    { 0xfce8df93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78df96, 90 },
    { 0x5268df97, 87 },
    { 0xd8a8df96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe68df9b, 55 },
    { 0xb7c8df9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f8df9f, 95 },
    { 0x83c8df9f, 52 },
    { 0x5048df9f, 95 },
    { 0x98a8dfa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd978dfa7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8dfab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2748dfb4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xda18dfb7, 93 },
    { 0, 0 },
    { 0x6098dfb9, 55 },
    { 0xa588dfb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9148dfca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1028dfcd, 23 },
    { 0, 0 },
    { 0x7208dfcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8dfd4, 90 },
    { 0, 0 },
    { 0x6b8dfd6, 90 },
    { 0x9748dfd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c08dfda, 98 },
    { 0x1b08dfdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d8dfe0, 77 },
    { 0, 0 },
    { 0x5488dfe2, 23 },
    { 0xb678dfe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238dfe6, 61 },
    { 0, 0 },
    { 0x84b8dfe8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b88dfec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dfef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb498dffa, 23 },
    { 0x2fe8dffb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8e000, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e8e006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd8e009, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9798e00d, 55 },
    { 0x9a18e00e, 88 },
    { 0, 0 },
    { 0xcb28e010, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb158e014, 61 },
    { 0x9238e015, 76 },
    { 0xa7f8e015, 98 },
    { 0x398e017, 33 },
    { 0x4758e018, 33 },
    { 0x2df8e014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f8e01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc238e020, 33 },
    { 0, 0 },
    { 0xcaf8e022, 52 },
    { 0x76e8e022, 95 },
    { 0xaca8e024, 33 },
    { 0xba98e024, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288e02b, 90 },
    { 0x79f8e02c, 34 },
    { 0, 0 },
    { 0x8768e02e, 64 },
    { 0x4078e02f, 98 },
    { 0, 0 },
    { 0x7408e031, 75 },
    { 0x25a8e032, 23 },
    { 0x2458e033, 91 },
    { 0xead8e032, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac8e038, 95 },
    { 0x1018e039, 90 },
    { 0, 0 },
    { 0xf5f8e03b, 95 },
    { 0, 0 },
    { 0xbbd8e03d, 90 },
    { 0xd558e03e, 18 },
    { 0xd1f8e03f, 93 },
    { 0, 0 },
    { 0x4508e041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8e045, 23 },
    { 0xe478e045, 95 },
    { 0xfe8e045, 8 },
    { 0, 0 },
    { 0x4488e049, 81 },
    { 0x15a8e04a, 90 },
    { 0x9548e04a, 98 },
    { 0xc0f8e04c, 78 },
    { 0x4ab8e049, 33 },
    { 0, 0 },
    { 0x9e38e04f, 95 },
    { 0x3eb8e050, 83 },
    { 0xea78e050, 34 },
    { 0x77d8e052, 61 },
    { 0x7d98e053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8e056, 61 },
    { 0xfe78e057, 23 },
    { 0xc388e057, 76 },
    { 0x5fd8e059, 77 },
    { 0xb278e058, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c8e060, 67 },
    { 0x12c8e061, 76 },
    { 0x7238e062, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e78e066, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c8e06b, 98 },
    { 0xc218e06c, 34 },
    { 0xbf98e06d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9448e073, 61 },
    { 0xa88e073, 23 },
    { 0x9818e073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8e079, 95 },
    { 0x62f8e07a, 61 },
    { 0x3668e079, 33 },
    { 0xa838e07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce98e085, 33 },
    { 0x9948e085, 55 },
    { 0xff8e086, 55 },
    { 0x2398e085, 95 },
    { 0x5c68e088, 43 },
    { 0xfec8e08a, 77 },
    { 0xff38e08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5628e08f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5818e092, 98 },
    { 0x2648e093, 52 },
    { 0x1c48e094, 98 },
    { 0, 0 },
    { 0xa5f8e096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f8e099, 98 },
    { 0xa4a8e09a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938e09d, 78 },
    { 0x9bf8e09e, 61 },
    { 0xf538e09f, 33 },
    { 0x9458e0a0, 84 },
    { 0x6f08e09e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9028e0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef8e0ab, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc688e0ae, 23 },
    { 0x5bb8e0af, 23 },
    { 0, 0 },
    { 0xc8b8e0b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd18e0b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d68e0b7, 33 },
    { 0x1608e0b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58e0bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8e0c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d18e0c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4968e0cd, 90 },
    { 0, 0 },
    { 0xd098e0cf, 95 },
    { 0x2fc8e0cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8e0d7, 61 },
    { 0, 0 },
    { 0x2e68e0d9, 33 },
    { 0x6f38e0da, 33 },
    { 0, 0 },
    { 0x9a88e0dc, 66 },
    { 0x3e68e0dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4008e0e0, 34 },
    { 0, 0 },
    { 0x6bf8e0e2, 61 },
    { 0x718e0e3, 98 },
    { 0x7ac8e0e4, 43 },
    { 0x7668e0e5, 23 },
    { 0x6388e0e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8e0e9, 76 },
    { 0, 0 },
    { 0xc4b8e0eb, 55 },
    { 0xdf8e0ec, 93 },
    { 0, 0 },
    { 0x60d8e0ee, 12 },
    { 0xe7a8e0ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1118e0f2, 76 },
    { 0, 0 },
    { 0x1048e0f4, 85 },
    { 0xf528e0f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca88e0fd, 33 },
    { 0, 0 },
    { 0x9bf8e0ff, 23 },
    { 0, 0 },
    { 0x3818e101, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1338e106, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa8e10b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8e111, 66 },
    { 0xe828e111, 98 },
    { 0x5dc8e113, 61 },
    { 0x6c28e114, 68 },
    { 0x8f78e114, 98 },
    { 0xfe98e115, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d18e119, 90 },
    { 0x89b8e119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078e11e, 93 },
    { 0x7e08e11f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c8e127, 87 },
    { 0x86e8e127, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c98e12c, 96 },
    { 0x6e98e12d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf8e130, 55 },
    { 0x66d8e131, 78 },
    { 0xfa8e131, 90 },
    { 0x5528e133, 66 },
    { 0x3a08e133, 61 },
    { 0x7d78e133, 61 },
    { 0x9538e136, 34 },
    { 0, 0 },
    { 0x2be8e138, 61 },
    { 0xd468e139, 61 },
    { 0xe848e139, 34 },
    { 0, 0 },
    { 0xf6a8e13c, 33 },
    { 0x95e8e13c, 55 },
    { 0, 0 },
    { 0x34c8e13f, 23 },
    { 0, 0 },
    { 0x6ee8e141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9758e148, 23 },
    { 0xf578e149, 98 },
    { 0x6ed8e148, 90 },
    { 0, 0 },
    { 0xe298e14c, 33 },
    { 0x3588e14d, 95 },
    { 0x4bd8e14c, 46 },
    { 0x6608e14f, 99 },
    { 0x9068e14c, 61 },
    { 0x5a08e14c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xca48e154, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb648e159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a8e15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd68e163, 91 },
    { 0x1c18e164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18e16a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b8e16f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d8e176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a58e179, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f8e17c, 52 },
    { 0x4808e17d, 90 },
    { 0x4ce8e17e, 90 },
    { 0x2a78e17f, 76 },
    { 0x6348e180, 38 },
    { 0, 0 },
    { 0x1a28e182, 90 },
    { 0x65d8e182, 98 },
    { 0x9908e184, 87 },
    { 0x92a8e185, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e8e18d, 43 },
    { 0x2bc8e18e, 61 },
    { 0x20f8e18f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a8e192, 75 },
    { 0, 0 },
    { 0x4b38e194, 98 },
    { 0xea28e195, 90 },
    { 0x3568e195, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48e19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b58e1a2, 84 },
    { 0x7fd8e1a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388e1a6, 40 },
    { 0x5f88e1a6, 46 },
    { 0x9398e1a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738e1ac, 98 },
    { 0, 0 },
    { 0x8188e1ae, 98 },
    { 0, 0 },
    { 0x61d8e1b0, 90 },
    { 0x29c8e1b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c8e1b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4688e1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e1c0, 23 },
    { 0x7718e1c1, 61 },
    { 0x1228e1c1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa758e1cf, 23 },
    { 0xb9f8e1d0, 90 },
    { 0x6f8e1cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78e1d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8e1da, 23 },
    { 0, 0 },
    { 0x6de8e1dc, 66 },
    { 0x7e18e1dd, 23 },
    { 0x8c88e1dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638e1e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb378e1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee8e1e8, 84 },
    { 0x2ca8e1e9, 61 },
    { 0xc668e1e8, 90 },
    { 0xdc48e1eb, 33 },
    { 0, 0 },
    { 0x7cf8e1ed, 33 },
    { 0x6288e1ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8868e1f1, 61 },
    { 0, 0 },
    { 0xbe98e1f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea8e1f7, 76 },
    { 0, 0 },
    { 0xb868e1f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7628e1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488e200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288e206, 17 },
    { 0, 0 },
    { 0x1b98e208, 98 },
    { 0xad28e209, 8 },
    { 0x6878e208, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e211, 88 },
    { 0xd6e8e211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a78e215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8e21a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8e21f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda8e222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8e227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8e22f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668e232, 98 },
    { 0x63b8e232, 99 },
    { 0, 0 },
    { 0xe408e235, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8e23b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea8e241, 98 },
    { 0x3b88e242, 90 },
    { 0x7198e243, 33 },
    { 0x368e244, 76 },
    { 0x54d8e245, 98 },
    { 0, 0 },
    { 0x9048e247, 90 },
    { 0, 0 },
    { 0xdc08e249, 61 },
    { 0x73c8e24a, 61 },
    { 0x9918e24b, 76 },
    { 0xfe88e24c, 33 },
    { 0, 0 },
    { 0xf428e24e, 46 },
    { 0xe488e24e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda48e258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8e25b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c18e25f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528e265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8e26c, 95 },
    { 0, 0 },
    { 0xcb78e26e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa8e271, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988e286, 93 },
    { 0xf108e287, 55 },
    { 0, 0 },
    { 0x8b18e289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588e28c, 24 },
    { 0, 0 },
    { 0xa008e28e, 99 },
    { 0x8398e28f, 53 },
    { 0xde78e290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a28e296, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e29d, 95 },
    { 0, 0 },
    { 0x8428e29f, 68 },
    { 0x5908e29f, 98 },
    { 0, 0 },
    { 0x73b8e2a2, 68 },
    { 0x9648e2a3, 95 },
    { 0x6458e2a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5448e2ac, 96 },
    { 0xa9e8e2ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1838e2b0, 61 },
    { 0x7548e2b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8e2b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d8e2c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde18e2c8, 46 },
    { 0xa3c8e2c9, 66 },
    { 0xd6f8e2ca, 23 },
    { 0x3228e2ca, 95 },
    { 0x6528e2cc, 87 },
    { 0, 0 },
    { 0xad78e2ce, 43 },
    { 0x2968e2cf, 34 },
    { 0, 0 },
    { 0x5898e2d1, 100 },
    { 0xf618e2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b8e2d5, 98 },
    { 0xdb18e2d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2da, 33 },
    { 0, 0 },
    { 0x1f08e2dc, 98 },
    { 0, 0 },
    { 0xe7e8e2de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4078e2e2, 34 },
    { 0xf58e2e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8e2e8, 93 },
    { 0x8478e2e9, 33 },
    { 0, 0 },
    { 0x6308e2eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa698e2f3, 99 },
    { 0, 0 },
    { 0xd1a8e2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538e2f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc498e2fb, 98 },
    { 0, 0 },
    { 0x52a8e2fd, 98 },
    { 0x3ce8e2fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6748e303, 101 },
    { 0x87a8e303, 98 },
    { 0x1a58e305, 87 },
    { 0, 0 },
    { 0xfc48e307, 98 },
    { 0x8f28e307, 98 },
    { 0x1d98e308, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d68e30c, 76 },
    { 0xf8e8e30d, 90 },
    { 0x7848e30c, 33 },
    { 0, 0 },
    { 0x2ae8e310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af8e315, 98 },
    { 0x2ef8e316, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8e319, 69 },
    { 0xa8a8e31a, 55 },
    { 0xd328e31a, 95 },
    { 0x4318e31c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd8e31f, 56 },
    { 0xcb48e320, 33 },
    { 0x1ea8e320, 61 },
    { 0xb278e322, 87 },
    { 0xa688e323, 33 },
    { 0xb258e323, 23 },
    { 0xea28e320, 99 },
    { 0xa0c8e323, 95 },
    { 0, 0 },
    { 0x5eb8e328, 77 },
    { 0xd878e328, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e32c, 33 },
    { 0, 0 },
    { 0x1578e32e, 90 },
    { 0xef78e32e, 33 },
    { 0x1cf8e32e, 61 },
    { 0x29d8e331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928e341, 95 },
    { 0xd8e8e342, 33 },
    { 0, 0 },
    { 0xf218e344, 90 },
    { 0, 0 },
    { 0x2648e346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d8e34c, 95 },
    { 0x51e8e34d, 91 },
    { 0x9ea8e34e, 52 },
    { 0, 0 },
    { 0x5a8e350, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1568e353, 91 },
    { 0xdca8e353, 95 },
    { 0xd278e355, 61 },
    { 0x7ab8e353, 46 },
    { 0, 0 },
    { 0x42b8e358, 23 },
    { 0xdef8e359, 43 },
    { 0x48e8e35a, 33 },
    { 0x4558e35a, 95 },
    { 0x9ca8e35c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628e35f, 33 },
    { 0x6808e35f, 95 },
    { 0x9b68e360, 99 },
    { 0x2988e35f, 55 },
    { 0, 0 },
    { 0x1c68e364, 95 },
    { 0, 0 },
    { 0x1168e366, 90 },
    { 0xebf8e367, 46 },
    { 0, 0 },
    { 0xd138e369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf688e36c, 33 },
    { 0x66f8e36d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc198e374, 52 },
    { 0x7028e374, 97 },
    { 0x4938e376, 55 },
    { 0x2d08e375, 90 },
    { 0xbf88e378, 95 },
    { 0, 0 },
    { 0xda58e37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7578e381, 95 },
    { 0xea78e382, 55 },
    { 0x7b28e383, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa788e387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8e38b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d38e38e, 90 },
    { 0x6498e38e, 95 },
    { 0xa518e390, 77 },
    { 0x7458e390, 76 },
    { 0x268e392, 87 },
    { 0xc4f8e393, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588e39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2248e3a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f8e3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d88e3ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2668e3b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e3b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce08e3bb, 78 },
    { 0xc738e3bc, 98 },
    { 0xfbc8e3bc, 33 },
    { 0xe3a8e3be, 66 },
    { 0x61b8e3bf, 23 },
    { 0, 0 },
    { 0xad98e3c1, 61 },
    { 0xb3c8e3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd78e3c5, 61 },
    { 0x1e88e3c5, 91 },
    { 0, 0 },
    { 0xb488e3c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398e3cb, 52 },
    { 0x5908e3cc, 90 },
    { 0xd7b8e3cb, 34 },
    { 0x5818e3ce, 23 },
    { 0x6438e3ce, 23 },
    { 0, 0 },
    { 0xc548e3d1, 95 },
    { 0x1708e3d1, 23 },
    { 0xc338e3d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8328e3d7, 90 },
    { 0x3c88e3d8, 23 },
    { 0xe278e3d9, 95 },
    { 0xf3b8e3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde58e3e2, 61 },
    { 0x4228e3e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc788e3e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068e3ed, 100 },
    { 0xc018e3ee, 66 },
    { 0xc008e3ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1798e3f6, 55 },
    { 0x3388e3f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d8e400, 23 },
    { 0x828e401, 83 },
    { 0x998e402, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab8e405, 23 },
    { 0x93f8e406, 87 },
    { 0xc7c8e405, 23 },
    { 0x9db8e405, 55 },
    { 0x8e98e406, 95 },
    { 0x22d8e408, 44 },
    { 0xbae8e40b, 33 },
    { 0x3198e40b, 98 },
    { 0x7a08e40d, 90 },
    { 0, 0 },
    { 0x6d78e40f, 95 },
    { 0x4bd8e410, 88 },
    { 0xcfc8e411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3378e414, 90 },
    { 0xaf58e414, 34 },
    { 0x38c8e414, 34 },
    { 0, 0 },
    { 0x7118e418, 17 },
    { 0, 0 },
    { 0x8018e41a, 90 },
    { 0x3758e41b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e8e41f, 36 },
    { 0xa4f8e41f, 90 },
    { 0x32c8e421, 40 },
    { 0xc978e422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9538e42e, 36 },
    { 0xb188e42f, 33 },
    { 0, 0 },
    { 0x6378e431, 77 },
    { 0, 0 },
    { 0x61d8e433, 93 },
    { 0, 0 },
    { 0xa978e435, 77 },
    { 0x1ac8e436, 34 },
    { 0, 0 },
    { 0xa238e438, 90 },
    { 0xa28e438, 95 },
    { 0, 0 },
    { 0xa7d8e43b, 90 },
    { 0x8218e43b, 93 },
    { 0x1088e43d, 61 },
    { 0xcf68e43e, 75 },
    { 0xc548e43f, 76 },
    { 0xf0a8e440, 33 },
    { 0xac08e43d, 23 },
    { 0, 0 },
    { 0xd198e443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x858e446, 98 },
    { 0xdbe8e447, 95 },
    { 0x5b78e447, 43 },
    { 0xb5c8e447, 84 },
    { 0xba78e44a, 90 },
    { 0x1f38e44b, 98 },
    { 0x2cd8e44c, 76 },
    { 0x3698e44d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd988e453, 91 },
    { 0, 0 },
    { 0xc7f8e455, 33 },
    { 0x4038e456, 98 },
    { 0xa2d8e457, 88 },
    { 0x96b8e457, 95 },
    { 0xf438e459, 33 },
    { 0x2008e45a, 90 },
    { 0x21a8e45b, 33 },
    { 0, 0 },
    { 0x2998e45d, 33 },
    { 0x3e08e45e, 76 },
    { 0xca78e45d, 95 },
    { 0xf4d8e45f, 33 },
    { 0xfa88e461, 90 },
    { 0, 0 },
    { 0xe658e463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188e466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4368e469, 23 },
    { 0xed58e469, 95 },
    { 0x6828e46b, 95 },
    { 0xeb58e46c, 88 },
    { 0, 0 },
    { 0xec98e46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5158e473, 98 },
    { 0, 0 },
    { 0x7b48e475, 61 },
    { 0, 0 },
    { 0xc578e477, 98 },
    { 0x2fc8e478, 95 },
    { 0x5c88e478, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8e47d, 66 },
    { 0, 0 },
    { 0xe7e8e47f, 98 },
    { 0, 0 },
    { 0x7958e481, 55 },
    { 0xeb58e482, 95 },
    { 0x9e98e483, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8e48a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf708e490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58e493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7198e497, 33 },
    { 0x1618e498, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8e49e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958e4a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b8e4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4348e4af, 76 },
    { 0x79a8e4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab58e4b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc318e4bb, 97 },
    { 0x4668e4bc, 99 },
    { 0x6298e4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88e4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf098e4c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8e4c9, 55 },
    { 0x2d28e4ca, 55 },
    { 0, 0 },
    { 0xd2a8e4cc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e98e4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c8e4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d8e4dc, 33 },
    { 0x3d8e4dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8598e4e0, 98 },
    { 0, 0 },
    { 0x60f8e4e2, 90 },
    { 0x4748e4e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb538e4e6, 61 },
    { 0x1ee8e4e7, 61 },
    { 0x9cf8e4e7, 90 },
    { 0xbc48e4e7, 33 },
    { 0, 0 },
    { 0x4a28e4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e4f3, 40 },
    { 0xc18e4f3, 43 },
    { 0xe408e4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b8e4f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb8e4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e8e502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f8e505, 23 },
    { 0, 0 },
    { 0x2508e507, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8e50a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a98e50e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe428e514, 95 },
    { 0, 0 },
    { 0xd6c8e516, 95 },
    { 0, 0 },
    { 0xbf98e518, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d8e51b, 33 },
    { 0xda18e51c, 85 },
    { 0, 0 },
    { 0x7568e51e, 100 },
    { 0x53a8e51f, 91 },
    { 0xdfb8e51f, 66 },
    { 0xf0a8e521, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc8e524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee8e529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf18e52d, 23 },
    { 0, 0 },
    { 0xe888e52f, 61 },
    { 0x8fe8e530, 87 },
    { 0xfee8e531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7638e534, 100 },
    { 0, 0 },
    { 0x9e58e536, 33 },
    { 0x20a8e537, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e53a, 33 },
    { 0, 0 },
    { 0xa3c8e53c, 46 },
    { 0x8448e53c, 94 },
    { 0, 0 },
    { 0xca28e53f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x258e542, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc8e548, 66 },
    { 0x2d48e548, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8e550, 17 },
    { 0xcc18e551, 61 },
    { 0, 0 },
    { 0x8b78e553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8628e557, 55 },
    { 0, 0 },
    { 0xbea8e559, 33 },
    { 0x6a88e55a, 85 },
    { 0, 0 },
    { 0xa708e55c, 87 },
    { 0x37c8e55d, 90 },
    { 0xa468e55e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b28e561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8048e567, 61 },
    { 0xfc78e568, 95 },
    { 0, 0 },
    { 0x7ed8e56a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608e578, 34 },
    { 0x8bd8e579, 98 },
    { 0, 0 },
    { 0x8928e57b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588e584, 90 },
    { 0x8378e585, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e588, 76 },
    { 0x72b8e589, 68 },
    { 0xa898e588, 66 },
    { 0xacc8e58a, 55 },
    { 0x5678e58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8e591, 91 },
    { 0xe308e592, 55 },
    { 0x5d8e591, 95 },
    { 0, 0 },
    { 0x1598e595, 95 },
    { 0x3e8e595, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f8e599, 33 },
    { 0xe508e59a, 61 },
    { 0, 0 },
    { 0x6b68e59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8e5a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c68e5a3, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f8e5aa, 61 },
    { 0xcaf8e5ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc548e5ae, 77 },
    { 0x5058e5af, 98 },
    { 0x1d88e5b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc98e5b3, 95 },
    { 0x6bd8e5b4, 33 },
    { 0x6858e5b5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc78e5bb, 55 },
    { 0, 0 },
    { 0xfde8e5bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e18e5c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98e5ca, 34 },
    { 0, 0 },
    { 0x4d38e5cc, 90 },
    { 0, 0 },
    { 0x5b8e5ce, 91 },
    { 0x15d8e5cf, 76 },
    { 0, 0 },
    { 0x1338e5d1, 61 },
    { 0x7b38e5d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9558e5d5, 80 },
    { 0x2f18e5d5, 76 },
    { 0, 0 },
    { 0x5d38e5d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7068e5db, 66 },
    { 0, 0 },
    { 0x2728e5dd, 3 },
    { 0, 0 },
    { 0x94f8e5df, 66 },
    { 0xaab8e5df, 34 },
    { 0x8888e5e1, 33 },
    { 0x3198e5e2, 93 },
    { 0x6098e5e2, 76 },
    { 0x9b48e5e4, 43 },
    { 0x6d68e5e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6378e5f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e8e604, 100 },
    { 0xcdd8e605, 98 },
    { 0xc938e604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b98e60c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2028e60f, 90 },
    { 0, 0 },
    { 0xd738e611, 61 },
    { 0, 0 },
    { 0x4ff8e613, 66 },
    { 0xc048e614, 61 },
    { 0xdd28e615, 90 },
    { 0x46f8e616, 55 },
    { 0xa9c8e616, 95 },
    { 0xe3e8e616, 55 },
    { 0x2f68e619, 91 },
    { 0x61a8e61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98e620, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8e623, 90 },
    { 0x3158e624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad8e62f, 33 },
    { 0xfd28e630, 95 },
    { 0xad48e631, 90 },
    { 0x5d8e62f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e636, 33 },
    { 0x5438e636, 43 },
    { 0x7688e636, 43 },
    { 0x2128e639, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3928e63c, 98 },
    { 0x8f48e63d, 17 },
    { 0xa678e63e, 23 },
    { 0xf1a8e63f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb08e642, 95 },
    { 0x7108e643, 33 },
    { 0xc7c8e643, 43 },
    { 0, 0 },
    { 0xcce8e646, 23 },
    { 0, 0 },
    { 0x7ac8e648, 55 },
    { 0xd428e649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d8e64f, 23 },
    { 0x55d8e64f, 76 },
    { 0x65b8e651, 99 },
    { 0, 0 },
    { 0x1b58e653, 33 },
    { 0x2408e654, 90 },
    { 0, 0 },
    { 0x6a58e656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28e659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda38e65d, 34 },
    { 0xf358e65e, 90 },
    { 0x5648e65e, 43 },
    { 0xb9c8e660, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e98e669, 48 },
    { 0x7ff8e669, 23 },
    { 0x79b8e669, 55 },
    { 0, 0 },
    { 0x43b8e66d, 61 },
    { 0x1ff8e66e, 93 },
    { 0, 0 },
    { 0x7988e670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8e675, 95 },
    { 0x7768e676, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a8e67a, 98 },
    { 0x7438e67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb928e67f, 95 },
    { 0, 0 },
    { 0xb8e8e681, 93 },
    { 0x348e682, 90 },
    { 0x2e78e683, 61 },
    { 0xd278e684, 33 },
    { 0x52d8e684, 33 },
    { 0xca38e685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8e68a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9548e68d, 95 },
    { 0, 0 },
    { 0x468e68f, 46 },
    { 0xdd28e690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7958e693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888e696, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2928e699, 49 },
    { 0, 0 },
    { 0x6f08e69b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad38e6a0, 43 },
    { 0xa278e6a1, 33 },
    { 0x2148e6a2, 55 },
    { 0, 0 },
    { 0xcb78e6a4, 99 },
    { 0x6b88e6a4, 93 },
    { 0x25d8e6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d88e6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2328e6ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c38e6b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf048e6b3, 90 },
    { 0x63c8e6b4, 23 },
    { 0x7788e6b5, 74 },
    { 0, 0 },
    { 0x52d8e6b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8e6bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd908e6c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa358e6c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8e6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f8e6cb, 23 },
    { 0, 0 },
    { 0x8bc8e6cd, 40 },
    { 0, 0 },
    { 0xcc08e6cf, 23 },
    { 0x23c8e6cf, 33 },
    { 0, 0 },
    { 0xf718e6d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988e6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc938e6e1, 85 },
    { 0x9248e6e1, 46 },
    { 0x9ea8e6e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888e6e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1928e6f0, 33 },
    { 0xbc8e6f1, 55 },
    { 0x4868e6f0, 34 },
    { 0x8e78e6f3, 84 },
    { 0x5618e6f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4988e6fb, 61 },
    { 0x5ca8e6fc, 93 },
    { 0, 0 },
    { 0xb8e6fe, 91 },
    { 0x8e08e6ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e18e702, 98 },
    { 0xa248e703, 34 },
    { 0x34c8e704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8768e708, 76 },
    { 0x7ac8e709, 69 },
    { 0, 0 },
    { 0x6f48e70b, 91 },
    { 0x1308e70b, 76 },
    { 0xf958e70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728e713, 95 },
    { 0x8b98e713, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee8e717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8e71b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8e720, 95 },
    { 0xb508e721, 90 },
    { 0x3608e720, 84 },
    { 0, 0 },
    { 0xe4d8e724, 61 },
    { 0x74d8e724, 61 },
    { 0x9858e726, 43 },
    { 0x5cd8e727, 65 },
    { 0, 0 },
    { 0x6b38e729, 98 },
    { 0x25a8e72a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88e72d, 99 },
    { 0xa328e72e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b8e73b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638e73f, 90 },
    { 0x88e740, 23 },
    { 0, 0 },
    { 0xe5f8e742, 76 },
    { 0x75d8e743, 17 },
    { 0x3b58e743, 93 },
    { 0, 0 },
    { 0x7258e746, 61 },
    { 0, 0 },
    { 0x4818e748, 66 },
    { 0, 0 },
    { 0x90a8e74a, 90 },
    { 0xea28e74b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa758e74f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1778e752, 55 },
    { 0x9118e753, 61 },
    { 0x61d8e754, 98 },
    { 0xdb8e754, 87 },
    { 0xa358e754, 41 },
    { 0x7658e756, 93 },
    { 0, 0 },
    { 0xb588e759, 98 },
    { 0xb28e75a, 33 },
    { 0x4d78e75a, 84 },
    { 0x5e8e75c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568e762, 98 },
    { 0x91c8e762, 23 },
    { 0, 0 },
    { 0xc238e765, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8e76c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8e76f, 11 },
    { 0, 0 },
    { 0x72d8e771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb8e777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e8e77a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7348e780, 95 },
    { 0xb3a8e781, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e789, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc828e78d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298e791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc78e794, 33 },
    { 0, 0 },
    { 0xed08e796, 87 },
    { 0x30a8e796, 55 },
    { 0, 0 },
    { 0xa3a8e799, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e79c, 90 },
    { 0, 0 },
    { 0xc438e79e, 83 },
    { 0, 0 },
    { 0xbc8e7a0, 38 },
    { 0, 0 },
    { 0x77b8e7a2, 87 },
    { 0x6398e7a3, 81 },
    { 0xd368e7a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308e7a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a8e7ac, 90 },
    { 0xe4d8e7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5908e7b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88e7b5, 23 },
    { 0xb5c8e7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b8e7b9, 90 },
    { 0xb948e7ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8e7bf, 90 },
    { 0, 0 },
    { 0x4628e7c1, 43 },
    { 0, 0 },
    { 0x1f8e7c3, 90 },
    { 0, 0 },
    { 0xd1c8e7c5, 64 },
    { 0x5298e7c6, 90 },
    { 0, 0 },
    { 0x19c8e7c8, 93 },
    { 0xb258e7c9, 76 },
    { 0xa5a8e7c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb8e7cd, 100 },
    { 0, 0 },
    { 0xe958e7cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8e7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c8e7e1, 33 },
    { 0x4978e7e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8e7e6, 98 },
    { 0xf298e7e7, 61 },
    { 0xd588e7e8, 100 },
    { 0x4668e7e9, 87 },
    { 0x2d08e7e7, 95 },
    { 0x1078e7eb, 95 },
    { 0x9d48e7ec, 33 },
    { 0, 0 },
    { 0xeb08e7ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb728e7f2, 61 },
    { 0x5f78e7f2, 88 },
    { 0xec08e7f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa28e7ff, 33 },
    { 0, 0 },
    { 0x24f8e801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd58e804, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d8e80a, 33 },
    { 0xdc58e80b, 46 },
    { 0xd398e80b, 95 },
    { 0x9a78e80d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d48e811, 69 },
    { 0x70c8e812, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8188e819, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e28e81f, 33 },
    { 0x678e81f, 52 },
    { 0x8e98e821, 90 },
    { 0xc088e822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c8e825, 61 },
    { 0xb738e825, 67 },
    { 0x12e8e827, 91 },
    { 0x5d28e828, 23 },
    { 0x2858e829, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa078e830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af8e834, 90 },
    { 0, 0 },
    { 0xf0c8e836, 61 },
    { 0x7468e837, 34 },
    { 0xff58e837, 85 },
    { 0x16a8e838, 33 },
    { 0x61d8e838, 43 },
    { 0x6808e83b, 33 },
    { 0x3d78e83c, 98 },
    { 0, 0 },
    { 0xda88e83e, 55 },
    { 0, 0 },
    { 0x3a78e840, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d8e843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe358e84b, 33 },
    { 0, 0 },
    { 0xc1a8e84d, 67 },
    { 0x5f48e84e, 55 },
    { 0xb188e84d, 85 },
    { 0, 0 },
    { 0x5388e851, 33 },
    { 0xe718e852, 33 },
    { 0x7948e853, 75 },
    { 0xc9b8e854, 80 },
    { 0, 0 },
    { 0x5cf8e856, 95 },
    { 0xb668e857, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa508e85b, 95 },
    { 0x25d8e85c, 78 },
    { 0, 0 },
    { 0x7708e85e, 34 },
    { 0x2cd8e85f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd808e863, 99 },
    { 0xe038e864, 46 },
    { 0x8908e864, 97 },
    { 0, 0 },
    { 0x9fa8e867, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c8e86e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f8e872, 99 },
    { 0x5658e873, 99 },
    { 0x1e88e873, 87 },
    { 0x3698e875, 53 },
    { 0xd6d8e872, 75 },
    { 0xf98e877, 95 },
    { 0, 0 },
    { 0x4f28e879, 53 },
    { 0xec38e87a, 90 },
    { 0x3b58e87b, 98 },
    { 0x7b68e87c, 90 },
    { 0, 0 },
    { 0xef38e87e, 33 },
    { 0xa858e87f, 90 },
    { 0x8808e87e, 98 },
    { 0, 0 },
    { 0xe898e882, 76 },
    { 0, 0 },
    { 0xe6f8e884, 98 },
    { 0xff08e885, 61 },
    { 0, 0 },
    { 0x3828e887, 95 },
    { 0, 0 },
    { 0x5288e889, 68 },
    { 0, 0 },
    { 0x4dc8e88b, 95 },
    { 0x4b58e88c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe698e890, 61 },
    { 0xf9c8e891, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e895, 76 },
    { 0, 0 },
    { 0x65b8e897, 61 },
    { 0x6b18e897, 90 },
    { 0x4178e899, 78 },
    { 0xda28e89a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18e8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf988e8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f88e8b7, 76 },
    { 0xaa68e8b8, 92 },
    { 0, 0 },
    { 0x1b68e8ba, 98 },
    { 0x3798e8ba, 90 },
    { 0x62b8e8bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a8e8c1, 68 },
    { 0x98f8e8c2, 17 },
    { 0xd938e8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8e8cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828e8d0, 74 },
    { 0x4f78e8d0, 90 },
    { 0, 0 },
    { 0x5b38e8d3, 55 },
    { 0, 0 },
    { 0xb378e8d5, 95 },
    { 0xeef8e8d6, 98 },
    { 0, 0 },
    { 0xc158e8d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c28e8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8068e8e2, 100 },
    { 0x2e08e8e3, 90 },
    { 0x5748e8e3, 98 },
    { 0xeaa8e8e5, 34 },
    { 0x4da8e8e6, 98 },
    { 0x9c48e8e6, 61 },
    { 0x8cf8e8e8, 83 },
    { 0xec48e8e4, 34 },
    { 0x4368e8ea, 23 },
    { 0xd028e8ea, 23 },
    { 0xdf98e8e3, 87 },
    { 0x38e8e6, 34 },
    { 0x9fd8e8e9, 88 },
    { 0xf838e8ef, 88 },
    { 0xa418e8ef, 76 },
    { 0, 0 },
    { 0x18a8e8f2, 36 },
    { 0xfae8e8f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c8e8f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa78e8f9, 61 },
    { 0xdfb8e8fa, 23 },
    { 0xc428e8f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc628e8ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d98e907, 95 },
    { 0, 0 },
    { 0x26b8e909, 84 },
    { 0x1408e90a, 69 },
    { 0, 0 },
    { 0x8318e90c, 61 },
    { 0x2258e90c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3968e917, 66 },
    { 0, 0 },
    { 0xa858e919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6408e91d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb848e921, 90 },
    { 0x35f8e922, 33 },
    { 0x1e68e922, 55 },
    { 0x5808e922, 75 },
    { 0, 0 },
    { 0x28e8e926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea98e929, 61 },
    { 0, 0 },
    { 0x22a8e92b, 61 },
    { 0x6438e92c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf8e92f, 43 },
    { 0xb6c8e930, 90 },
    { 0x10c8e930, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98e938, 98 },
    { 0x8ce8e939, 90 },
    { 0xa838e93a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998e940, 33 },
    { 0, 0 },
    { 0x9268e942, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8898e945, 93 },
    { 0xf008e946, 66 },
    { 0, 0 },
    { 0x2a38e948, 93 },
    { 0xc208e949, 25 },
    { 0xb5a8e949, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8e94d, 80 },
    { 0x738e94d, 23 },
    { 0, 0 },
    { 0x5448e950, 46 },
    { 0x7848e950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1068e955, 33 },
    { 0x80f8e955, 33 },
    { 0x2ec8e955, 76 },
    { 0xb528e958, 33 },
    { 0x8128e959, 61 },
    { 0x34d8e95a, 33 },
    { 0xb208e95a, 61 },
    { 0xdbb8e956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6048e95f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf098e962, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b8e966, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e96a, 33 },
    { 0, 0 },
    { 0xadd8e96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f8e972, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08e978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3898e97d, 95 },
    { 0x64f8e97e, 33 },
    { 0xec18e97d, 98 },
    { 0xc938e97d, 90 },
    { 0x6118e981, 97 },
    { 0, 0 },
    { 0xf648e983, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4818e989, 23 },
    { 0xbe68e98a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae58e996, 90 },
    { 0xd318e997, 77 },
    { 0, 0 },
    { 0x1988e999, 23 },
    { 0x2af8e999, 66 },
    { 0x84a8e999, 43 },
    { 0xf578e99b, 98 },
    { 0xe208e99d, 98 },
    { 0x9af8e99e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d68e9a1, 90 },
    { 0x8018e9a2, 88 },
    { 0x4598e9a3, 33 },
    { 0x75b8e9a2, 78 },
    { 0x1f8e9a3, 65 },
    { 0xc5d8e9a2, 33 },
    { 0x87f8e9a7, 61 },
    { 0x1848e9a8, 98 },
    { 0x2a8e9a6, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a8e9ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6168e9b1, 84 },
    { 0x3a58e9b2, 69 },
    { 0xef08e9b2, 55 },
    { 0, 0 },
    { 0x5778e9b5, 33 },
    { 0, 0 },
    { 0x1de8e9b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c38e9bc, 78 },
    { 0x1588e9bd, 95 },
    { 0x2ab8e9bc, 57 },
    { 0xa268e9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb218e9c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a8e9cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868e9d1, 52 },
    { 0x3238e9d1, 61 },
    { 0xf3b8e9d2, 43 },
    { 0, 0 },
    { 0xb9b8e9d5, 99 },
    { 0xbd28e9d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8e9d9, 33 },
    { 0x9878e9da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8e9dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d58e9e0, 95 },
    { 0x7738e9e0, 90 },
    { 0xa2d8e9e2, 93 },
    { 0xe808e9e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18e9e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc528e9ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb358e9f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8938e9f7, 88 },
    { 0, 0 },
    { 0x7158e9f9, 60 },
    { 0xd438e9f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058e9fe, 76 },
    { 0x7d08e9fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e8ea04, 88 },
    { 0x7af8ea05, 68 },
    { 0, 0 },
    { 0x38c8ea07, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58ea0d, 33 },
    { 0xd3a8ea0d, 33 },
    { 0x4848ea0d, 53 },
    { 0x5dc8ea0e, 90 },
    { 0x2588ea11, 30 },
    { 0xf4e8ea11, 91 },
    { 0xf468ea12, 23 },
    { 0xbdb8ea11, 68 },
    { 0xad58ea11, 99 },
    { 0xff98ea11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8ea22, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728ea26, 4 },
    { 0, 0 },
    { 0xeb18ea28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78ea2c, 33 },
    { 0xe7d8ea2d, 85 },
    { 0x3d98ea2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa8ea32, 87 },
    { 0x90d8ea32, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8ea3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c78ea3f, 23 },
    { 0x55d8ea3f, 51 },
    { 0xab18ea3f, 90 },
    { 0x1998ea42, 61 },
    { 0x24d8ea43, 55 },
    { 0xe7b8ea44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628ea48, 66 },
    { 0xe7e8ea49, 61 },
    { 0xdff8ea48, 66 },
    { 0x4fe8ea49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9938ea4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18ea51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738ea58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c8ea5c, 95 },
    { 0xd128ea5d, 93 },
    { 0, 0 },
    { 0xb688ea5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518ea65, 95 },
    { 0, 0 },
    { 0x2d98ea67, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8ea72, 87 },
    { 0x55e8ea73, 95 },
    { 0x628ea74, 61 },
    { 0x3918ea75, 90 },
    { 0x5e58ea76, 33 },
    { 0xe568ea76, 23 },
    { 0xee18ea76, 33 },
    { 0xc168ea79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc628ea81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78ea86, 81 },
    { 0x8d48ea87, 98 },
    { 0xde78ea88, 56 },
    { 0x9128ea89, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab78ea8d, 90 },
    { 0x7258ea8e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb158ea99, 98 },
    { 0xdfb8ea99, 23 },
    { 0, 0 },
    { 0xf718ea9c, 98 },
    { 0, 0 },
    { 0x1648ea9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888eaa2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28eaa7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f98eaab, 90 },
    { 0, 0 },
    { 0x2a68eaad, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88eab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c18eab3, 75 },
    { 0, 0 },
    { 0x9818eab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828eaba, 55 },
    { 0x5378eabb, 98 },
    { 0x1578eabc, 23 },
    { 0x2d38eaba, 87 },
    { 0xedd8eaba, 36 },
    { 0x73c8eabd, 66 },
    { 0x4588eabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8eac5, 90 },
    { 0x9198eac6, 52 },
    { 0x1228eac5, 95 },
    { 0xfe88eac8, 61 },
    { 0x3b78eac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e78eacf, 100 },
    { 0, 0 },
    { 0xdcd8ead1, 23 },
    { 0x74d8ead2, 87 },
    { 0x29a8ead2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a18ead7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d88eadc, 90 },
    { 0, 0 },
    { 0xdfd8eade, 91 },
    { 0xd158eadf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2208eae2, 61 },
    { 0, 0 },
    { 0x73b8eae4, 65 },
    { 0x7b68eae5, 90 },
    { 0xe248eae6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc468eaee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1938eaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee08eaf7, 23 },
    { 0xc4a8eaf8, 76 },
    { 0x2028eaf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d68eb01, 43 },
    { 0, 0 },
    { 0x36b8eb03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748eb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe088eb09, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd8eb0d, 34 },
    { 0xd308eb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618eb13, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158eb19, 98 },
    { 0x3698eb19, 76 },
    { 0x3368eb1b, 46 },
    { 0x8788eb1b, 78 },
    { 0x6b78eb1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb8eb25, 33 },
    { 0, 0 },
    { 0xdc48eb27, 34 },
    { 0xeef8eb27, 19 },
    { 0x27f8eb29, 90 },
    { 0xad18eb2a, 61 },
    { 0, 0 },
    { 0x6978eb2c, 55 },
    { 0, 0 },
    { 0x3988eb2e, 98 },
    { 0x6a38eb2f, 78 },
    { 0x968eb2f, 61 },
    { 0x6c58eb2f, 52 },
    { 0, 0 },
    { 0xd9a8eb33, 33 },
    { 0xd38eb34, 89 },
    { 0xd438eb35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88eb39, 68 },
    { 0, 0 },
    { 0x5c78eb3b, 98 },
    { 0xb868eb3c, 78 },
    { 0xcd68eb3c, 43 },
    { 0, 0 },
    { 0x2518eb3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a8eb43, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb288eb48, 81 },
    { 0xc668eb49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3308eb4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8eb52, 92 },
    { 0xffa8eb53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca98eb57, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3558eb5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xce38eb5e, 43 },
    { 0x8fc8eb5f, 76 },
    { 0xe5e8eb60, 81 },
    { 0xb018eb60, 98 },
    { 0x5758eb62, 34 },
    { 0xc1b8eb63, 91 },
    { 0x2a58eb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a8eb69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8eb70, 76 },
    { 0, 0 },
    { 0xa2e8eb72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac8eb7b, 61 },
    { 0x4f48eb7b, 98 },
    { 0x68d8eb7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8028eb83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08eb8a, 84 },
    { 0x23d8eb8a, 76 },
    { 0x7a88eb8c, 81 },
    { 0x768eb8d, 76 },
    { 0xd098eb8d, 93 },
    { 0xfa98eb8f, 95 },
    { 0xda28eb8c, 33 },
    { 0, 0 },
    { 0x29e8eb92, 90 },
    { 0, 0 },
    { 0xaf58eb94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7308eb97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608eb9e, 95 },
    { 0x68c8eb9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a68eba2, 91 },
    { 0x3ab8eba3, 34 },
    { 0, 0 },
    { 0xa9b8eba5, 90 },
    { 0, 0 },
    { 0xfe48eba7, 77 },
    { 0x2c28eba7, 98 },
    { 0x4c68eba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f38ebaf, 23 },
    { 0x8ee8ebb0, 34 },
    { 0xe028ebb1, 67 },
    { 0xe608ebb2, 87 },
    { 0x4f68ebb3, 78 },
    { 0xa08ebb1, 98 },
    { 0x5098ebb0, 91 },
    { 0x9478ebb6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b68ebb9, 61 },
    { 0, 0 },
    { 0x37a8ebbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e8ebbf, 90 },
    { 0x9ac8ebc0, 90 },
    { 0xc2b8ebc0, 95 },
    { 0x24b8ebc2, 93 },
    { 0x8908ebc3, 61 },
    { 0x4f8ebc2, 90 },
    { 0x7998ebc5, 90 },
    { 0xd978ebc6, 56 },
    { 0xc238ebc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb448ebca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb298ebcd, 98 },
    { 0, 0 },
    { 0xc68ebcf, 98 },
    { 0, 0 },
    { 0x5138ebd1, 17 },
    { 0x7108ebd2, 46 },
    { 0x2ca8ebd3, 23 },
    { 0xb5e8ebd4, 33 },
    { 0x2de8ebd4, 33 },
    { 0x6808ebd5, 99 },
    { 0, 0 },
    { 0x1ee8ebd8, 93 },
    { 0xd8d8ebd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1348ebdc, 90 },
    { 0xfb18ebdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ebe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a88ebe8, 46 },
    { 0, 0 },
    { 0x8968ebea, 33 },
    { 0x5758ebeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb8ebee, 93 },
    { 0xdf98ebee, 98 },
    { 0, 0 },
    { 0x5988ebf1, 98 },
    { 0xb5c8ebf1, 95 },
    { 0, 0 },
    { 0x6888ebf4, 98 },
    { 0xb2f8ebf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8ebf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c8ebfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe608ebff, 46 },
    { 0, 0 },
    { 0x7f18ec01, 23 },
    { 0xbfb8ec01, 59 },
    { 0x3748ec03, 61 },
    { 0, 0 },
    { 0x78a8ec05, 33 },
    { 0x22e8ec06, 68 },
    { 0x6b78ec07, 95 },
    { 0, 0 },
    { 0x2ba8ec09, 46 },
    { 0x23e8ec0a, 90 },
    { 0, 0 },
    { 0x37f8ec0c, 76 },
    { 0, 0 },
    { 0x9f78ec0e, 76 },
    { 0x92f8ec0f, 87 },
    { 0xdd68ec0e, 61 },
    { 0x7938ec0f, 95 },
    { 0x4128ec12, 90 },
    { 0x2528ec0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8ec16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b98ec19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148ec1d, 78 },
    { 0x2ca8ec1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa798ec21, 90 },
    { 0x48f8ec22, 81 },
    { 0xbe38ec23, 98 },
    { 0xb0f8ec24, 90 },
    { 0x26f8ec25, 75 },
    { 0x4778ec26, 90 },
    { 0xc558ec27, 33 },
    { 0x6f18ec27, 84 },
    { 0x6a8ec29, 90 },
    { 0xfe18ec25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f78ec2d, 33 },
    { 0x22d8ec2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff8ec31, 68 },
    { 0x4198ec32, 61 },
    { 0x2d08ec32, 33 },
    { 0, 0 },
    { 0x5578ec35, 98 },
    { 0, 0 },
    { 0x7798ec37, 100 },
    { 0xdfc8ec37, 43 },
    { 0xd3d8ec39, 91 },
    { 0xd1b8ec3a, 83 },
    { 0x6078ec3a, 90 },
    { 0, 0 },
    { 0x7c58ec3d, 53 },
    { 0xed8ec3e, 76 },
    { 0, 0 },
    { 0x32b8ec40, 98 },
    { 0, 0 },
    { 0xc5e8ec42, 55 },
    { 0x3268ec42, 96 },
    { 0xee58ec44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9908ec4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c68ec55, 23 },
    { 0, 0 },
    { 0x7af8ec57, 61 },
    { 0, 0 },
    { 0x2928ec59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48ec5c, 91 },
    { 0x3a98ec5d, 61 },
    { 0x14c8ec5e, 95 },
    { 0x18f8ec5f, 93 },
    { 0xd78ec60, 61 },
    { 0, 0 },
    { 0xbe98ec62, 98 },
    { 0xd1e8ec63, 77 },
    { 0xabc8ec64, 61 },
    { 0xb118ec65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8ec76, 33 },
    { 0, 0 },
    { 0x8938ec78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae8ec7d, 90 },
    { 0x1fd8ec7e, 52 },
    { 0x3f8ec7f, 33 },
    { 0xbb28ec7e, 90 },
    { 0, 0 },
    { 0x48e8ec82, 34 },
    { 0xc828ec83, 90 },
    { 0xf928ec83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38ec88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8ec8b, 95 },
    { 0xed98ec8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8ec92, 61 },
    { 0x1cc8ec93, 55 },
    { 0x5418ec94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f8ec9c, 52 },
    { 0x1ae8ec9d, 90 },
    { 0x8308ec9e, 64 },
    { 0x1998ec9e, 75 },
    { 0xe158ec9c, 95 },
    { 0x7348eca1, 78 },
    { 0xd678eca1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18eca7, 33 },
    { 0, 0 },
    { 0xed88eca9, 93 },
    { 0xec08ecaa, 34 },
    { 0xccb8ecab, 23 },
    { 0x44b8ecaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf48ecb0, 61 },
    { 0xfb8ecb1, 23 },
    { 0xa6d8ecb1, 93 },
    { 0, 0 },
    { 0x7068ecb4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ecb7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7128ecbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828ecc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed8ecc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9568eccd, 90 },
    { 0xad58ecce, 55 },
    { 0xaa88ecce, 95 },
    { 0, 0 },
    { 0x3148ecd1, 33 },
    { 0xd4a8ecd1, 61 },
    { 0x248ecd1, 91 },
    { 0xd8d8ecd3, 55 },
    { 0xd988ecd1, 95 },
    { 0x9f58ecd6, 33 },
    { 0, 0 },
    { 0xd778ecd8, 98 },
    { 0xb468ecd9, 97 },
    { 0x6b68ecda, 77 },
    { 0x8078ecd9, 88 },
    { 0, 0 },
    { 0x2958ecdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98ece1, 33 },
    { 0xa708ece2, 90 },
    { 0xdda8ece3, 33 },
    { 0x5788ece4, 90 },
    { 0x4958ece5, 43 },
    { 0, 0 },
    { 0x2078ece7, 99 },
    { 0x56c8ece8, 76 },
    { 0xd218ece9, 66 },
    { 0x4ac8ece8, 52 },
    { 0x7808ece8, 84 },
    { 0x18a8ecec, 98 },
    { 0x7e78ecec, 55 },
    { 0xc2d8ecec, 98 },
    { 0xfc98ecef, 95 },
    { 0x4a98ecef, 33 },
    { 0x458ecef, 95 },
    { 0xaad8ecec, 87 },
    { 0, 0 },
    { 0x778ecf4, 90 },
    { 0x1cc8ecf5, 91 },
    { 0x948ecf4, 61 },
    { 0xd3c8ecf7, 98 },
    { 0x4368ecf7, 43 },
    { 0, 0 },
    { 0xb118ecfa, 98 },
    { 0x90f8ecfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8528ecfe, 65 },
    { 0x2db8ecff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba8ed07, 33 },
    { 0x1718ed08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd8ed0c, 55 },
    { 0xcbb8ed0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8ed10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f88ed15, 77 },
    { 0, 0 },
    { 0xc018ed17, 98 },
    { 0, 0 },
    { 0x36b8ed19, 95 },
    { 0x1fb8ed1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8ed20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b8ed23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff28ed2c, 33 },
    { 0, 0 },
    { 0x6c28ed2e, 90 },
    { 0, 0 },
    { 0x7618ed30, 90 },
    { 0, 0 },
    { 0xbc98ed32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8ed38, 43 },
    { 0, 0 },
    { 0xa098ed3a, 43 },
    { 0, 0 },
    { 0xf0c8ed3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b38ed40, 34 },
    { 0x6c88ed41, 17 },
    { 0xd468ed41, 33 },
    { 0x1de8ed40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688ed52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d8ed5b, 100 },
    { 0x2d18ed5c, 91 },
    { 0x9b48ed5d, 85 },
    { 0x89f8ed5e, 93 },
    { 0, 0 },
    { 0xc568ed60, 88 },
    { 0, 0 },
    { 0xc1b8ed62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6318ed65, 33 },
    { 0x9b08ed65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2568ed69, 90 },
    { 0, 0 },
    { 0x3f18ed6b, 95 },
    { 0xffc8ed6c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58ed73, 78 },
    { 0x29c8ed74, 46 },
    { 0xe1c8ed74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc18ed7a, 34 },
    { 0xbd68ed7a, 76 },
    { 0, 0 },
    { 0x9b28ed7d, 91 },
    { 0, 0 },
    { 0xa068ed7f, 23 },
    { 0, 0 },
    { 0x2458ed81, 98 },
    { 0, 0 },
    { 0xc1a8ed83, 75 },
    { 0, 0 },
    { 0x5668ed85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c8ed8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b68ed90, 61 },
    { 0, 0 },
    { 0xacc8ed92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c8ed97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4328ed9c, 87 },
    { 0xce08ed9d, 34 },
    { 0x9d48ed9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e68eda4, 46 },
    { 0x3bd8eda5, 100 },
    { 0x53b8eda6, 98 },
    { 0xba08eda7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e28edaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8edae, 66 },
    { 0x3fd8edae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1038edb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa058edbf, 76 },
    { 0x9f08edbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8edc3, 43 },
    { 0x3338edc4, 77 },
    { 0x46c8edc4, 61 },
    { 0xfb18edc6, 81 },
    { 0x4bf8edc7, 33 },
    { 0, 0 },
    { 0x9668edc9, 61 },
    { 0, 0 },
    { 0xe68edcb, 61 },
    { 0x5588edcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad8edd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58edd4, 61 },
    { 0x3e98edd4, 46 },
    { 0x15c8edd4, 55 },
    { 0x6368edd7, 33 },
    { 0xf7e8edd6, 33 },
    { 0x20c8edd6, 43 },
    { 0x5758edda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4348edde, 61 },
    { 0xd928eddf, 90 },
    { 0x2a48eddf, 76 },
    { 0x7008ede1, 34 },
    { 0xc338ede2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8ede9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588eded, 87 },
    { 0x40f8edee, 71 },
    { 0x4868edef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d78edf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db8edf9, 98 },
    { 0xb6f8edfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e08edfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc8ee01, 91 },
    { 0, 0 },
    { 0x3fc8ee03, 33 },
    { 0, 0 },
    { 0xd638ee05, 76 },
    { 0x5e78ee06, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1858ee0d, 91 },
    { 0x5d48ee0d, 52 },
    { 0, 0 },
    { 0x6e08ee10, 90 },
    { 0xd878ee11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be8ee1a, 98 },
    { 0x18d8ee1b, 61 },
    { 0xe6c8ee1c, 95 },
    { 0x7978ee1d, 52 },
    { 0x8798ee1a, 90 },
    { 0, 0 },
    { 0xb778ee20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa468ee23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698ee26, 98 },
    { 0xcdb8ee26, 99 },
    { 0x9618ee28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c8ee31, 93 },
    { 0xbe08ee32, 90 },
    { 0x60b8ee31, 43 },
    { 0x478ee34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb628ee3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a8ee3f, 61 },
    { 0x2be8ee40, 52 },
    { 0x6a8ee40, 23 },
    { 0x7718ee42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688ee45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc8ee4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e8ee51, 84 },
    { 0xbdd8ee51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8ee55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4478ee58, 98 },
    { 0x94d8ee59, 95 },
    { 0x2578ee5a, 78 },
    { 0, 0 },
    { 0x6378ee5c, 61 },
    { 0xa908ee5c, 61 },
    { 0x6fe8ee5d, 87 },
    { 0x7b8ee5c, 55 },
    { 0x5f48ee60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98ee64, 43 },
    { 0x9ab8ee65, 95 },
    { 0x8828ee64, 100 },
    { 0xde38ee67, 61 },
    { 0, 0 },
    { 0x31c8ee69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8ee6c, 33 },
    { 0x2a8ee6d, 76 },
    { 0x7168ee6e, 23 },
    { 0xdf98ee6d, 95 },
    { 0x6a18ee6c, 85 },
    { 0x248ee71, 61 },
    { 0x8f48ee72, 90 },
    { 0x17e8ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2158ee79, 65 },
    { 0xe808ee7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a8ee7d, 99 },
    { 0x6fb8ee7d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc8ee84, 61 },
    { 0, 0 },
    { 0xbee8ee86, 23 },
    { 0xd7e8ee87, 95 },
    { 0xe7d8ee88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e8ee8c, 38 },
    { 0xaaa8ee8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1908ee90, 90 },
    { 0xd548ee90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9798ee97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8ee9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb8eea2, 33 },
    { 0, 0 },
    { 0xe278eea4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48eea7, 33 },
    { 0xeaf8eea8, 61 },
    { 0x5738eea9, 77 },
    { 0xad38eea8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448eeb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e98eeb7, 61 },
    { 0x9be8eeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8eebb, 69 },
    { 0xb238eebc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc378eec0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d78eec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb528eed7, 55 },
    { 0xec88eed7, 90 },
    { 0x3298eed9, 90 },
    { 0xf7e8eeda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6818eedd, 81 },
    { 0, 0 },
    { 0xc118eedf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e8eee7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f8eeec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6788eeef, 96 },
    { 0xf6c8eeef, 84 },
    { 0xd3c8eeef, 43 },
    { 0, 0 },
    { 0xf838eef3, 23 },
    { 0xf378eef4, 95 },
    { 0x3d38eef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f38eef8, 91 },
    { 0x41a8eef9, 91 },
    { 0x4a38eef8, 77 },
    { 0, 0 },
    { 0x10f8eefc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b8ef01, 90 },
    { 0xfff8ef02, 98 },
    { 0, 0 },
    { 0x86a8ef04, 99 },
    { 0xfc58ef05, 98 },
    { 0x4938ef05, 23 },
    { 0xd2a8ef07, 61 },
    { 0xa5b8ef07, 91 },
    { 0, 0 },
    { 0xe898ef0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd518ef0f, 90 },
    { 0x2da8ef10, 55 },
    { 0, 0 },
    { 0x1908ef12, 98 },
    { 0x57d8ef12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad28ef18, 55 },
    { 0, 0 },
    { 0x3318ef1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9268ef1d, 98 },
    { 0, 0 },
    { 0xf708ef1f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b58ef22, 61 },
    { 0x3038ef22, 84 },
    { 0, 0 },
    { 0x7828ef25, 90 },
    { 0, 0 },
    { 0x8eb8ef27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8ef2a, 95 },
    { 0x60a8ef2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe548ef30, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c8ef3b, 75 },
    { 0, 0 },
    { 0x468ef3d, 98 },
    { 0xe978ef3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1888ef43, 78 },
    { 0x4398ef44, 23 },
    { 0x9b18ef45, 61 },
    { 0x5a38ef46, 23 },
    { 0x6868ef47, 84 },
    { 0x788ef48, 23 },
    { 0x1998ef46, 98 },
    { 0x2a88ef4a, 61 },
    { 0xa708ef4a, 90 },
    { 0x64c8ef49, 68 },
    { 0x41c8ef4c, 34 },
    { 0xbff8ef4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8ef52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe318ef57, 90 },
    { 0xf448ef58, 100 },
    { 0xa078ef59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e98ef5c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb18ef60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb48ef6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8ef6e, 90 },
    { 0x1898ef6f, 91 },
    { 0, 0 },
    { 0x9fb8ef71, 95 },
    { 0x2408ef72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28ef76, 33 },
    { 0x7ac8ef77, 68 },
    { 0xb648ef77, 61 },
    { 0x32c8ef76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ef7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b8ef84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38ef89, 99 },
    { 0x6b18ef8a, 100 },
    { 0x1ee8ef8b, 65 },
    { 0xa608ef89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98ef95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa78ef9c, 98 },
    { 0, 0 },
    { 0x8898ef9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48efa2, 93 },
    { 0xe678efa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da8efa6, 76 },
    { 0, 0 },
    { 0x5a38efa8, 76 },
    { 0, 0 },
    { 0xccd8efaa, 34 },
    { 0xea8efab, 76 },
    { 0x7878efab, 78 },
    { 0xf5b8efab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18efb2, 77 },
    { 0xa6b8efb2, 87 },
    { 0xdd28efb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f98efba, 78 },
    { 0x76f8efba, 98 },
    { 0xc088efbc, 76 },
    { 0xc1b8efbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28efc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce8efc6, 98 },
    { 0xf8f8efc7, 99 },
    { 0, 0 },
    { 0x1f88efc9, 100 },
    { 0xfd58efca, 33 },
    { 0x5e58efca, 61 },
    { 0xf6a8efcc, 46 },
    { 0xf68efcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7188efd7, 91 },
    { 0, 0 },
    { 0xdbf8efd9, 33 },
    { 0xf928efda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9858efde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb318efe2, 33 },
    { 0x5d58efe3, 43 },
    { 0x1fb8efe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6818efe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a8efee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e58eff1, 46 },
    { 0xe848eff2, 66 },
    { 0x1038eff2, 76 },
    { 0x6f48eff3, 93 },
    { 0x1798eff5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48effb, 61 },
    { 0xf1d8effb, 95 },
    { 0x8558effb, 93 },
    { 0x3158effe, 55 },
    { 0, 0 },
    { 0xaf18f000, 33 },
    { 0x1f98f000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8f004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa78f009, 33 },
    { 0, 0 },
    { 0x6c48f00b, 33 },
    { 0, 0 },
    { 0x7c28f00d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f011, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc148f015, 33 },
    { 0, 0 },
    { 0x19d8f017, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998f01a, 23 },
    { 0x7a68f01b, 52 },
    { 0, 0 },
    { 0xca08f01d, 33 },
    { 0x62d8f01d, 61 },
    { 0x27b8f01f, 95 },
    { 0xe828f020, 76 },
    { 0x4dc8f020, 98 },
    { 0xa598f020, 46 },
    { 0xaa18f020, 98 },
    { 0xa228f024, 84 },
    { 0xf7f8f01e, 95 },
    { 0xbb08f01d, 23 },
    { 0xe558f025, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588f02c, 65 },
    { 0xd7a8f02d, 81 },
    { 0xff68f02e, 90 },
    { 0x6ac8f02e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8f033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef68f037, 98 },
    { 0xcb88f038, 98 },
    { 0, 0 },
    { 0x2268f03a, 91 },
    { 0x36b8f03b, 73 },
    { 0, 0 },
    { 0x1208f03d, 76 },
    { 0, 0 },
    { 0x61d8f03f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f04c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc8f050, 68 },
    { 0xcbd8f050, 98 },
    { 0x93c8f052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e8f055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9838f058, 88 },
    { 0xe0f8f059, 91 },
    { 0, 0 },
    { 0x7e78f05b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08f061, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8f068, 61 },
    { 0xde38f068, 34 },
    { 0xb78f06a, 61 },
    { 0, 0 },
    { 0x9f88f06c, 76 },
    { 0xf68f06c, 34 },
    { 0x228f06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a38f072, 33 },
    { 0x538f073, 90 },
    { 0x5578f072, 93 },
    { 0xf7a8f075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c8f078, 84 },
    { 0, 0 },
    { 0xf7c8f07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8f083, 90 },
    { 0xbfd8f083, 53 },
    { 0x1d88f085, 61 },
    { 0, 0 },
    { 0x2918f087, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7058f08d, 33 },
    { 0, 0 },
    { 0xf868f08f, 90 },
    { 0xed88f08f, 76 },
    { 0, 0 },
    { 0x5958f092, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7128f097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac8f09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a8f0a1, 34 },
    { 0x9938f0a1, 23 },
    { 0, 0 },
    { 0x1e18f0a4, 90 },
    { 0xae38f0a4, 4 },
    { 0xca98f0a6, 98 },
    { 0, 0 },
    { 0xef58f0a8, 95 },
    { 0, 0 },
    { 0xc6e8f0aa, 34 },
    { 0xef88f0aa, 61 },
    { 0x6618f0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c78f0b1, 55 },
    { 0x8218f0b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a8f0b8, 76 },
    { 0, 0 },
    { 0x488f0ba, 23 },
    { 0x4ea8f0bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98f0c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8f0c4, 33 },
    { 0, 0 },
    { 0x2668f0c6, 95 },
    { 0x2e88f0c7, 23 },
    { 0x2ac8f0c8, 28 },
    { 0x6198f0c9, 46 },
    { 0x248f0c8, 77 },
    { 0xede8f0c7, 33 },
    { 0xdfe8f0cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8f0d0, 61 },
    { 0x1c8f0d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9088f0d6, 52 },
    { 0x9258f0d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28f0dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878f0e5, 23 },
    { 0xa348f0e6, 33 },
    { 0xf668f0e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f0eb, 98 },
    { 0x6428f0ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee68f0f7, 98 },
    { 0x1548f0f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8f0fd, 33 },
    { 0xb0f8f0fe, 76 },
    { 0x6bd8f0ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb688f109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8f10e, 61 },
    { 0x28d8f10f, 61 },
    { 0xee18f110, 76 },
    { 0x27c8f111, 98 },
    { 0x98d8f110, 33 },
    { 0xaaa8f110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc168f11c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f8f121, 33 },
    { 0x92a8f121, 23 },
    { 0x2698f123, 78 },
    { 0x3e8f124, 33 },
    { 0, 0 },
    { 0xdb98f126, 76 },
    { 0, 0 },
    { 0x4398f128, 95 },
    { 0xba48f129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6118f130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18f134, 61 },
    { 0xb858f135, 95 },
    { 0, 0 },
    { 0x1f78f137, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f38f13d, 90 },
    { 0x9f38f13e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d8f144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8f148, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8f14f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f8f156, 33 },
    { 0xc978f157, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe68f15a, 46 },
    { 0, 0 },
    { 0x4ba8f15c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a48f163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8f167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd8f170, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1638f173, 33 },
    { 0x1b58f173, 61 },
    { 0xc458f174, 95 },
    { 0xc4a8f173, 95 },
    { 0x13f8f177, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f8f17a, 66 },
    { 0x12a8f17a, 90 },
    { 0x5588f17a, 34 },
    { 0x8168f17a, 98 },
    { 0xcd8f17e, 90 },
    { 0xcdf8f17e, 91 },
    { 0x1e08f180, 61 },
    { 0x24b8f181, 78 },
    { 0, 0 },
    { 0x5ae8f183, 52 },
    { 0, 0 },
    { 0x18b8f185, 98 },
    { 0x9158f185, 23 },
    { 0x3e58f187, 90 },
    { 0x2798f188, 23 },
    { 0x3238f185, 90 },
    { 0x5c58f187, 98 },
    { 0xa7a8f187, 98 },
    { 0, 0 },
    { 0x18f18d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8f190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e8f19a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18f19d, 75 },
    { 0x9718f19e, 90 },
    { 0x2ae8f19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd8f1a4, 52 },
    { 0x3138f1a5, 33 },
    { 0xb998f1a6, 78 },
    { 0, 0 },
    { 0x79e8f1a8, 90 },
    { 0x3b38f1a9, 61 },
    { 0x4fd8f1a8, 95 },
    { 0, 0 },
    { 0x9a48f1ac, 87 },
    { 0, 0 },
    { 0xb9f8f1ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e8f1b2, 52 },
    { 0x83d8f1b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7318f1b6, 56 },
    { 0x28d8f1b6, 95 },
    { 0xe5a8f1b6, 55 },
    { 0x2198f1b6, 95 },
    { 0, 0 },
    { 0x62e8f1bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2938f1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca8f1c2, 52 },
    { 0, 0 },
    { 0x1008f1c4, 33 },
    { 0xfa98f1c4, 23 },
    { 0x11e8f1c6, 100 },
    { 0x25e8f1c7, 34 },
    { 0x7a8f1c8, 61 },
    { 0xff98f1c9, 95 },
    { 0, 0 },
    { 0xbab8f1cb, 93 },
    { 0, 0 },
    { 0x4f78f1cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538f1d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9378f1d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa8f1d7, 81 },
    { 0xd3e8f1d8, 77 },
    { 0xb588f1d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218f1dd, 46 },
    { 0x5248f1de, 33 },
    { 0x2e78f1df, 34 },
    { 0x3678f1e0, 90 },
    { 0xf818f1e1, 46 },
    { 0xfa38f1de, 90 },
    { 0, 0 },
    { 0xcdb8f1e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8f1e8, 33 },
    { 0xb998f1e9, 65 },
    { 0x9288f1e9, 61 },
    { 0x30b8f1ea, 95 },
    { 0x4d38f1e8, 95 },
    { 0x2438f1ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3748f1f3, 98 },
    { 0xfbb8f1f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd348f1fd, 23 },
    { 0, 0 },
    { 0x2318f1ff, 76 },
    { 0, 0 },
    { 0x58c8f201, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8f205, 78 },
    { 0xe78f206, 61 },
    { 0xce08f205, 98 },
    { 0xde28f208, 23 },
    { 0, 0 },
    { 0x1e88f20a, 76 },
    { 0, 0 },
    { 0x8108f20c, 99 },
    { 0xa0f8f20d, 95 },
    { 0x5aa8f20e, 99 },
    { 0xbc38f20f, 90 },
    { 0x4c68f20e, 78 },
    { 0x23d8f210, 90 },
    { 0x9798f212, 68 },
    { 0xb428f213, 33 },
    { 0x84f8f214, 61 },
    { 0xd58f213, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f21d, 100 },
    { 0x2898f21e, 100 },
    { 0x6f08f21f, 33 },
    { 0xb708f21e, 91 },
    { 0x9c38f221, 33 },
    { 0x5ef8f222, 95 },
    { 0xfff8f223, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa788f227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e18f22c, 17 },
    { 0x5618f22d, 98 },
    { 0x9c58f22d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6488f235, 33 },
    { 0x60f8f236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b58f242, 40 },
    { 0xf378f243, 98 },
    { 0x85c8f243, 34 },
    { 0x4688f245, 90 },
    { 0x5248f246, 90 },
    { 0xa018f245, 33 },
    { 0x7ad8f247, 90 },
    { 0x9a58f246, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd678f24d, 53 },
    { 0x3de8f24e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe838f251, 98 },
    { 0, 0 },
    { 0x77c8f253, 98 },
    { 0, 0 },
    { 0x89e8f255, 95 },
    { 0xb8c8f256, 95 },
    { 0, 0 },
    { 0xa938f258, 98 },
    { 0xb7e8f258, 95 },
    { 0, 0 },
    { 0xb88f25b, 34 },
    { 0x9ad8f25c, 95 },
    { 0x55c8f25d, 90 },
    { 0x9e88f25e, 33 },
    { 0x1a08f25b, 99 },
    { 0x3a88f260, 90 },
    { 0xf788f261, 98 },
    { 0xe738f261, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb28f272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7348f275, 78 },
    { 0x43c8f275, 33 },
    { 0x6768f276, 87 },
    { 0x90e8f278, 84 },
    { 0xe138f279, 36 },
    { 0x8448f27a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a8f27f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8f287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1508f295, 90 },
    { 0xb1e8f295, 55 },
    { 0x8eb8f297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9078f29a, 84 },
    { 0, 0 },
    { 0xb2d8f29c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe308f2a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e8f2a7, 23 },
    { 0x178f2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c8f2ac, 33 },
    { 0x27c8f2ad, 28 },
    { 0, 0 },
    { 0xf908f2af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18f2b3, 33 },
    { 0, 0 },
    { 0x3cd8f2b5, 46 },
    { 0xe058f2b6, 97 },
    { 0xf758f2b7, 88 },
    { 0, 0 },
    { 0x9568f2b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6078f2be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88f2c2, 61 },
    { 0xf8f8f2c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308f2c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f2cb, 90 },
    { 0xed18f2cc, 33 },
    { 0, 0 },
    { 0xfe18f2ce, 23 },
    { 0x4e38f2cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8918f2d3, 99 },
    { 0x1c8f2d4, 75 },
    { 0, 0 },
    { 0x82d8f2d6, 61 },
    { 0x1c18f2d6, 90 },
    { 0x7f68f2d8, 98 },
    { 0x6fb8f2d8, 76 },
    { 0xeee8f2da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb738f2de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4068f2e2, 55 },
    { 0x9958f2e3, 90 },
    { 0x4098f2e4, 90 },
    { 0xe0c8f2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868f2e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718f2ed, 78 },
    { 0x7d08f2ee, 98 },
    { 0x1018f2ef, 33 },
    { 0x9898f2ef, 66 },
    { 0x7b38f2f1, 23 },
    { 0x8978f2f2, 34 },
    { 0x3258f2f3, 61 },
    { 0x8c38f2f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb778f2f7, 33 },
    { 0, 0 },
    { 0x598f2f9, 17 },
    { 0x4a88f2f9, 95 },
    { 0x2ed8f2fb, 85 },
    { 0xb908f2fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8f301, 23 },
    { 0xc938f302, 90 },
    { 0, 0 },
    { 0x5078f304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef98f30a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528f315, 76 },
    { 0x3608f316, 43 },
    { 0x70c8f317, 91 },
    { 0x85a8f318, 23 },
    { 0xdea8f319, 61 },
    { 0x66d8f31a, 23 },
    { 0x2718f31b, 34 },
    { 0xfba8f31a, 76 },
    { 0xb0b8f316, 52 },
    { 0x5fb8f31e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8488f326, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f8f32b, 33 },
    { 0xf478f32c, 100 },
    { 0x3c98f32d, 76 },
    { 0x3028f32c, 98 },
    { 0xe498f32f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba08f334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7938f337, 61 },
    { 0xb2b8f337, 61 },
    { 0, 0 },
    { 0x85e8f33a, 17 },
    { 0, 0 },
    { 0xe648f33c, 61 },
    { 0, 0 },
    { 0x46c8f33e, 91 },
    { 0, 0 },
    { 0x3078f340, 34 },
    { 0, 0 },
    { 0x3ce8f342, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4948f345, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf8f349, 61 },
    { 0x33b8f34a, 78 },
    { 0x4ec8f349, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b8f34f, 33 },
    { 0xb678f34f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518f353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed88f357, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b8f35a, 98 },
    { 0x3f08f35b, 76 },
    { 0xaac8f35b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8f35f, 33 },
    { 0x5ee8f35f, 76 },
    { 0, 0 },
    { 0x67a8f362, 90 },
    { 0x4f48f362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e18f368, 23 },
    { 0, 0 },
    { 0x6138f36a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf138f370, 100 },
    { 0x8c28f371, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff78f377, 90 },
    { 0xe3a8f378, 93 },
    { 0, 0 },
    { 0x54a8f37a, 34 },
    { 0, 0 },
    { 0x4cb8f37c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e8f37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8f386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8008f38f, 98 },
    { 0xc1f8f38f, 97 },
    { 0x9058f391, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8f394, 76 },
    { 0, 0 },
    { 0xcde8f396, 65 },
    { 0, 0 },
    { 0x9428f398, 88 },
    { 0x7cc8f399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58f39e, 98 },
    { 0xf3c8f39f, 23 },
    { 0x1648f3a0, 33 },
    { 0x4888f3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9328f3a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe498f3ae, 61 },
    { 0x9478f3af, 97 },
    { 0x8f08f3af, 33 },
    { 0x4268f3b0, 33 },
    { 0x70e8f3af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e8f3b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848f3bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d8f3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6548f3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f3cb, 90 },
    { 0x2218f3cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe908f3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8398f3db, 23 },
    { 0x7758f3db, 95 },
    { 0x6388f3dd, 98 },
    { 0x9bf8f3de, 43 },
    { 0xea08f3db, 33 },
    { 0xe488f3de, 40 },
    { 0xd6b8f3e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8568f3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a8f3f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a8f3fd, 90 },
    { 0x1ff8f3fd, 98 },
    { 0, 0 },
    { 0x7208f400, 64 },
    { 0, 0 },
    { 0xacb8f402, 23 },
    { 0x4278f403, 98 },
    { 0, 0 },
    { 0x18d8f405, 90 },
    { 0x1588f405, 95 },
    { 0xb9b8f407, 43 },
    { 0x4638f408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8f40c, 53 },
    { 0x8828f40c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8f410, 33 },
    { 0, 0 },
    { 0x4b28f412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f418, 33 },
    { 0xcb88f418, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8f41c, 90 },
    { 0x87b8f41d, 76 },
    { 0x1408f41d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698f422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8f425, 93 },
    { 0x87a8f426, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c98f42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108f42f, 98 },
    { 0x77d8f430, 33 },
    { 0x6e88f431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb38f435, 90 },
    { 0xb0f8f436, 33 },
    { 0x4838f437, 90 },
    { 0xc728f436, 88 },
    { 0xb808f439, 90 },
    { 0x3198f43a, 23 },
    { 0x6758f43a, 75 },
    { 0x24c8f43c, 95 },
    { 0xb628f43c, 90 },
    { 0xdf38f43e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3368f441, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef8f445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508f44b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8f44e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f8f458, 85 },
    { 0x70a8f459, 91 },
    { 0, 0 },
    { 0x5c58f45b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc8f45e, 80 },
    { 0x2fe8f45f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a8f466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c8f46c, 17 },
    { 0x808f46c, 61 },
    { 0x48d8f46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf8f472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff8f475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d28f478, 90 },
    { 0x5e38f479, 98 },
    { 0x2a8f47a, 76 },
    { 0xe5b8f47b, 77 },
    { 0, 0 },
    { 0x30b8f47d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28f485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8f48b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98f48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f38f493, 98 },
    { 0xde18f494, 90 },
    { 0, 0 },
    { 0xb028f496, 95 },
    { 0xd1d8f496, 23 },
    { 0x6c78f496, 33 },
    { 0, 0 },
    { 0xf478f49a, 34 },
    { 0xf6b8f49b, 61 },
    { 0xc88f49c, 98 },
    { 0x1b38f49d, 33 },
    { 0xe108f49c, 85 },
    { 0xe168f49f, 55 },
    { 0x678f4a0, 99 },
    { 0x8518f49f, 66 },
    { 0xd468f49c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318f4aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f4ae, 55 },
    { 0x7d78f4af, 23 },
    { 0x2a78f4b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6008f4b5, 61 },
    { 0, 0 },
    { 0xd818f4b7, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8f4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b8f4bd, 34 },
    { 0, 0 },
    { 0x398f4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a8f4c2, 90 },
    { 0xdc08f4c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8f4ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438f4cd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe468f4d0, 67 },
    { 0x4a48f4d1, 34 },
    { 0x1cd8f4d2, 100 },
    { 0x3db8f4d3, 55 },
    { 0, 0 },
    { 0x4ca8f4d5, 78 },
    { 0xf168f4d6, 90 },
    { 0x3a98f4d7, 38 },
    { 0, 0 },
    { 0x2a28f4d9, 98 },
    { 0x58f8f4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd8f4e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8f4e6, 98 },
    { 0, 0 },
    { 0x4698f4e8, 98 },
    { 0xd268f4e9, 43 },
    { 0xf6d8f4e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8f4f1, 85 },
    { 0x1088f4f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b08f4f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4938f4fa, 33 },
    { 0, 0 },
    { 0xe098f4fc, 23 },
    { 0x9678f4fd, 90 },
    { 0xaad8f4fd, 90 },
    { 0xff58f4ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb8f50b, 85 },
    { 0, 0 },
    { 0xed38f50d, 95 },
    { 0xb4a8f50d, 90 },
    { 0x9588f50e, 61 },
    { 0xec48f50f, 61 },
    { 0x22b8f50e, 93 },
    { 0xc9f8f512, 84 },
    { 0xb328f513, 23 },
    { 0xcef8f512, 95 },
    { 0xe858f50e, 33 },
    { 0x448f510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c8f51b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde18f524, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b8f52a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d8f52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308f531, 85 },
    { 0, 0 },
    { 0xf0a8f533, 23 },
    { 0xf998f533, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3498f539, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f545, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc8f549, 65 },
    { 0x5628f54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8f54e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de8f555, 60 },
    { 0xa8b8f556, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da8f55c, 33 },
    { 0, 0 },
    { 0xf488f55e, 75 },
    { 0x4f28f55f, 33 },
    { 0xe578f560, 23 },
    { 0x8208f55f, 98 },
    { 0x2048f562, 23 },
    { 0x3fb8f562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1318f570, 87 },
    { 0, 0 },
    { 0xac38f572, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8468f576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6228f57d, 17 },
    { 0xc5b8f57d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5838f585, 78 },
    { 0x68c8f585, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3098f58c, 90 },
    { 0xad78f58c, 95 },
    { 0xe8f8f58e, 98 },
    { 0x2d58f58e, 87 },
    { 0, 0 },
    { 0x66a8f591, 98 },
    { 0, 0 },
    { 0x95d8f593, 93 },
    { 0, 0 },
    { 0x4428f595, 76 },
    { 0x1518f596, 33 },
    { 0x9168f597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8f59b, 33 },
    { 0x818f59c, 98 },
    { 0xaa48f59d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f8f5a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa8f5a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d8f5aa, 33 },
    { 0, 0 },
    { 0x6678f5ac, 46 },
    { 0, 0 },
    { 0xeee8f5ae, 61 },
    { 0xe1e8f5af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8f5b3, 99 },
    { 0xc6f8f5b4, 55 },
    { 0xd918f5b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38f5b8, 33 },
    { 0, 0 },
    { 0x1f28f5ba, 98 },
    { 0x7a38f5ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568f5be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508f5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a8f5d0, 84 },
    { 0xb718f5d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8f5d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8f5dd, 85 },
    { 0x6418f5dd, 46 },
    { 0xbde8f5df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f5e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f8f5e5, 76 },
    { 0xc018f5e6, 76 },
    { 0xfe38f5e6, 34 },
    { 0x9178f5e8, 90 },
    { 0x3288f5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1588f5f5, 23 },
    { 0, 0 },
    { 0xae78f5f7, 98 },
    { 0xcd48f5f8, 90 },
    { 0, 0 },
    { 0x5d38f5fa, 95 },
    { 0, 0 },
    { 0xd4a8f5fc, 61 },
    { 0, 0 },
    { 0xa838f5fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea98f601, 85 },
    { 0x7798f601, 98 },
    { 0xbf68f601, 93 },
    { 0, 0 },
    { 0xe878f605, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd568f609, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c88f60f, 33 },
    { 0x9ff8f610, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f614, 36 },
    { 0x38f8f615, 95 },
    { 0xa988f615, 33 },
    { 0, 0 },
    { 0xa0b8f618, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8f627, 98 },
    { 0x8468f628, 76 },
    { 0x7598f629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8f62c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f635, 33 },
    { 0xb498f636, 98 },
    { 0, 0 },
    { 0x4ab8f638, 23 },
    { 0x89d8f639, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b18f642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b8f648, 40 },
    { 0, 0 },
    { 0xe228f64a, 95 },
    { 0, 0 },
    { 0x53e8f64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f08f652, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde78f659, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f8f660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb418f663, 95 },
    { 0, 0 },
    { 0x5248f665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f18f668, 78 },
    { 0xfa38f669, 91 },
    { 0xaba8f669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4228f66d, 90 },
    { 0xec68f66d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5538f673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea8f67b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a8f67f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618f683, 61 },
    { 0, 0 },
    { 0xb1e8f685, 69 },
    { 0x51a8f686, 90 },
    { 0xfa48f686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428f690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb308f696, 61 },
    { 0, 0 },
    { 0x35b8f698, 33 },
    { 0x98c8f699, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8848f6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf178f6a7, 77 },
    { 0, 0 },
    { 0xefc8f6a9, 55 },
    { 0, 0 },
    { 0xa758f6ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd888f6af, 52 },
    { 0, 0 },
    { 0xd1b8f6b1, 98 },
    { 0x18b8f6b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4178f6b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e8f6b9, 98 },
    { 0xf188f6ba, 33 },
    { 0xf338f6b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec8f6bf, 90 },
    { 0x5e18f6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8f6c4, 42 },
    { 0, 0 },
    { 0x2748f6c6, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f6cb, 23 },
    { 0, 0 },
    { 0x628f6cd, 76 },
    { 0, 0 },
    { 0xd828f6cf, 33 },
    { 0x2fc8f6cf, 75 },
    { 0xb708f6cf, 61 },
    { 0, 0 },
    { 0x7cb8f6d3, 66 },
    { 0x5e78f6d3, 60 },
    { 0, 0 },
    { 0x30e8f6d6, 76 },
    { 0x6d88f6d7, 23 },
    { 0x87e8f6d8, 90 },
    { 0x5478f6d9, 33 },
    { 0, 0 },
    { 0xcea8f6db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058f6df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb968f6e3, 55 },
    { 0x4638f6e4, 85 },
    { 0xf1a8f6e5, 34 },
    { 0x8888f6e5, 33 },
    { 0xe818f6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8f6f0, 95 },
    { 0xbf78f6f0, 99 },
    { 0xb2b8f6f0, 43 },
    { 0x7da8f6f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc798f6f7, 34 },
    { 0x87d8f6f8, 33 },
    { 0xece8f6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed8f700, 33 },
    { 0, 0 },
    { 0x5198f702, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d98f707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b8f70c, 61 },
    { 0x47f8f70c, 95 },
    { 0x2388f70e, 90 },
    { 0xc918f70c, 38 },
    { 0xdfe8f710, 55 },
    { 0, 0 },
    { 0x49f8f712, 52 },
    { 0x9c8f713, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a8f716, 55 },
    { 0x23c8f717, 61 },
    { 0xfce8f716, 95 },
    { 0x7e48f718, 95 },
    { 0x3268f71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4678f71e, 98 },
    { 0xede8f71f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b28f722, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc98f725, 90 },
    { 0x9198f725, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b68f72a, 76 },
    { 0xd38f72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c8f72f, 34 },
    { 0x81b8f730, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c8f739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5168f73f, 33 },
    { 0xc578f73f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5888f745, 98 },
    { 0x8dc8f746, 61 },
    { 0, 0 },
    { 0xfa58f748, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8f74b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e8f74e, 34 },
    { 0, 0 },
    { 0x2558f750, 46 },
    { 0x6248f750, 90 },
    { 0xcc18f750, 90 },
    { 0x9ac8f750, 95 },
    { 0xc348f754, 23 },
    { 0x2398f750, 76 },
    { 0x1de8f752, 61 },
    { 0x2248f753, 78 },
    { 0xa9a8f758, 85 },
    { 0, 0 },
    { 0x2e18f75a, 66 },
    { 0x5ec8f75a, 43 },
    { 0x61c8f75c, 87 },
    { 0, 0 },
    { 0xd018f75e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08f766, 33 },
    { 0x6d78f767, 78 },
    { 0x3e08f766, 61 },
    { 0x5b48f769, 90 },
    { 0x18a8f768, 90 },
    { 0x2b8f768, 95 },
    { 0x8f76b, 61 },
    { 0xdab8f76d, 43 },
    { 0xafa8f769, 23 },
    { 0x4c08f768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d8f773, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c78f776, 87 },
    { 0, 0 },
    { 0x44c8f778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd48f77e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8f785, 43 },
    { 0x9a98f786, 33 },
    { 0x8ed8f786, 80 },
    { 0, 0 },
    { 0x8278f789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8f78e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b98f791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e88f797, 17 },
    { 0x7578f798, 33 },
    { 0x2708f799, 90 },
    { 0x48e8f797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928f79e, 90 },
    { 0xb2a8f79f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88f7a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98f7a7, 66 },
    { 0, 0 },
    { 0x3f48f7a9, 91 },
    { 0x1328f7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc8f7b7, 95 },
    { 0xb9f8f7b7, 98 },
    { 0, 0 },
    { 0xb9d8f7ba, 90 },
    { 0, 0 },
    { 0x1c18f7bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f38f7c1, 23 },
    { 0, 0 },
    { 0xea08f7c3, 61 },
    { 0, 0 },
    { 0x1db8f7c5, 52 },
    { 0, 0 },
    { 0x1f58f7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b8f7cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188f7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2628f7d8, 92 },
    { 0, 0 },
    { 0xa268f7da, 85 },
    { 0x2be8f7db, 66 },
    { 0x4b48f7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab28f7e1, 85 },
    { 0x7988f7e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928f7e7, 84 },
    { 0, 0 },
    { 0x9cd8f7e9, 23 },
    { 0x8b38f7e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f7ef, 95 },
    { 0, 0 },
    { 0x3af8f7f1, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd78f7f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a8f7f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438f7fb, 46 },
    { 0, 0 },
    { 0x26b8f7fd, 76 },
    { 0, 0 },
    { 0x6608f7ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea8f803, 99 },
    { 0, 0 },
    { 0x9b38f805, 76 },
    { 0xfed8f806, 33 },
    { 0x8ad8f807, 76 },
    { 0, 0 },
    { 0xe9a8f809, 66 },
    { 0xc2a8f80a, 33 },
    { 0xbb48f80b, 68 },
    { 0, 0 },
    { 0x9338f80d, 55 },
    { 0x9108f80e, 52 },
    { 0xdb88f80e, 61 },
    { 0x1f78f80f, 55 },
    { 0x8378f810, 84 },
    { 0x6e88f812, 76 },
    { 0xb78f813, 33 },
    { 0x1dd8f80d, 90 },
    { 0xa188f812, 100 },
    { 0, 0 },
    { 0xf9f8f817, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8f81a, 77 },
    { 0xb448f81b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5938f820, 43 },
    { 0, 0 },
    { 0xc608f822, 65 },
    { 0, 0 },
    { 0x2378f824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf718f82e, 95 },
    { 0, 0 },
    { 0x3be8f830, 84 },
    { 0x96a8f831, 77 },
    { 0xaf98f831, 33 },
    { 0x3c18f833, 30 },
    { 0, 0 },
    { 0xce78f835, 33 },
    { 0x9268f836, 84 },
    { 0xed88f836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b8f83d, 33 },
    { 0xbef8f83e, 43 },
    { 0x7258f83f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d88f844, 87 },
    { 0x1e48f844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468f84a, 34 },
    { 0, 0 },
    { 0x2738f84c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e48f851, 93 },
    { 0x3538f852, 23 },
    { 0x9b28f852, 95 },
    { 0x7b08f854, 65 },
    { 0x9ad8f854, 95 },
    { 0x83f8f852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1478f85a, 61 },
    { 0, 0 },
    { 0x39a8f85c, 23 },
    { 0x1418f85c, 33 },
    { 0xa3a8f85c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5968f863, 98 },
    { 0, 0 },
    { 0xfcd8f865, 98 },
    { 0x6798f866, 40 },
    { 0x4aa8f866, 95 },
    { 0xe2f8f868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648f86c, 33 },
    { 0, 0 },
    { 0x3808f86e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc38f871, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b08f876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc8f87a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd058f87f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8f887, 88 },
    { 0x3e68f888, 99 },
    { 0, 0 },
    { 0x98f8f88a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588f890, 43 },
    { 0x47f8f890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b88f897, 100 },
    { 0, 0 },
    { 0xc1b8f899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28f89f, 41 },
    { 0x40b8f8a0, 43 },
    { 0xc428f8a0, 46 },
    { 0, 0 },
    { 0x228f8a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438f8a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb8f8b1, 61 },
    { 0x538f8b2, 67 },
    { 0xba08f8b1, 76 },
    { 0xd4c8f8b4, 23 },
    { 0, 0 },
    { 0xec58f8b6, 69 },
    { 0xdf8f8b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368f8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898f8c0, 61 },
    { 0x8dc8f8c0, 98 },
    { 0x22e8f8c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac8f8c5, 93 },
    { 0xe988f8c6, 90 },
    { 0xed88f8c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8f8cd, 90 },
    { 0xad98f8ce, 55 },
    { 0xe88f8cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e18f8d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5538f8db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648f8de, 55 },
    { 0xa058f8de, 98 },
    { 0x92f8f8de, 84 },
    { 0xa338f8e1, 23 },
    { 0x1008f8df, 33 },
    { 0x1fd8f8e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28f8e7, 26 },
    { 0x4d38f8e8, 61 },
    { 0xe1b8f8e7, 61 },
    { 0x8fe8f8e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8988f8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a8f8fd, 100 },
    { 0x35f8f8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de8f902, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868f907, 98 },
    { 0x5918f907, 67 },
    { 0xbb88f909, 33 },
    { 0x41a8f909, 61 },
    { 0x2d18f909, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8418f910, 90 },
    { 0, 0 },
    { 0x4c18f912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe68f915, 95 },
    { 0x3e8f916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba8f919, 100 },
    { 0x82e8f91a, 36 },
    { 0x5c68f91b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce28f921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868f926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc818f92e, 90 },
    { 0x5dc8f92e, 61 },
    { 0x4698f92f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f8f934, 46 },
    { 0x9d68f934, 61 },
    { 0x1ad8f936, 87 },
    { 0x10f8f937, 81 },
    { 0x25f8f936, 93 },
    { 0x1c28f939, 90 },
    { 0x54a8f93a, 19 },
    { 0x83c8f936, 66 },
    { 0xb618f93c, 76 },
    { 0x8988f93d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288f943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f68f948, 88 },
    { 0, 0 },
    { 0xc238f94a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a8f94f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea28f952, 61 },
    { 0, 0 },
    { 0x64b8f954, 47 },
    { 0, 0 },
    { 0x778f956, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648f959, 90 },
    { 0xded8f95a, 46 },
    { 0xeb38f95b, 98 },
    { 0x85f8f95b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa08f961, 90 },
    { 0, 0 },
    { 0xd0b8f963, 23 },
    { 0xee48f964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa708f967, 95 },
    { 0, 0 },
    { 0xff68f969, 76 },
    { 0, 0 },
    { 0x2298f96b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f58f96e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8f973, 23 },
    { 0xcf8f974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2418f983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d28f986, 77 },
    { 0xba58f987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868f98a, 95 },
    { 0x73f8f98b, 33 },
    { 0x7118f98c, 38 },
    { 0x7d08f98a, 99 },
    { 0xcf08f98a, 98 },
    { 0x1728f98d, 34 },
    { 0xd6c8f990, 23 },
    { 0xc068f991, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc638f994, 55 },
    { 0, 0 },
    { 0x87f8f996, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3128f999, 87 },
    { 0x7088f99a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8748f99d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1508f9a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48f9a4, 34 },
    { 0, 0 },
    { 0xcaf8f9a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c8f9a9, 61 },
    { 0x5758f9aa, 98 },
    { 0x12c8f9aa, 76 },
    { 0, 0 },
    { 0xf4c8f9ad, 60 },
    { 0xdbb8f9ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c8f9b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d8f9bf, 33 },
    { 0x3168f9bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff8f9c7, 44 },
    { 0, 0 },
    { 0x8fe8f9c9, 61 },
    { 0x18e8f9ca, 98 },
    { 0x2b88f9ca, 95 },
    { 0, 0 },
    { 0x7298f9cd, 98 },
    { 0x6688f9ce, 76 },
    { 0x7c08f9cd, 93 },
    { 0, 0 },
    { 0x9ee8f9d1, 43 },
    { 0x2db8f9d1, 100 },
    { 0x40c8f9d3, 98 },
    { 0x18b8f9d3, 34 },
    { 0, 0 },
    { 0x4868f9d6, 90 },
    { 0xd468f9d7, 40 },
    { 0x9398f9d8, 23 },
    { 0x5178f9d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d28f9dd, 33 },
    { 0x3428f9de, 55 },
    { 0x7d98f9df, 55 },
    { 0x4218f9dd, 43 },
    { 0xe9d8f9e1, 66 },
    { 0x3868f9e2, 98 },
    { 0x3ac8f9e3, 55 },
    { 0, 0 },
    { 0x4458f9e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648f9e8, 23 },
    { 0, 0 },
    { 0xb998f9ea, 68 },
    { 0, 0 },
    { 0xf268f9ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b8f9f4, 90 },
    { 0x508f9f5, 77 },
    { 0xc958f9f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8f9fa, 95 },
    { 0xc048f9fb, 33 },
    { 0xb308f9fc, 46 },
    { 0xe488f9fc, 34 },
    { 0, 0 },
    { 0x51d8f9ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b8fa03, 95 },
    { 0x3d98fa03, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e8fa08, 90 },
    { 0xcee8fa09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa38fa0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fa11, 81 },
    { 0x11d8fa11, 90 },
    { 0x5be8fa12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1888fa1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f8fa1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f8fa22, 93 },
    { 0x6da8fa23, 90 },
    { 0, 0 },
    { 0x3b88fa25, 78 },
    { 0x4b08fa25, 23 },
    { 0x3ce8fa25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff8fa2b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa438fa2e, 23 },
    { 0, 0 },
    { 0x8e88fa30, 91 },
    { 0, 0 },
    { 0x5ac8fa32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8fa36, 53 },
    { 0x9fe8fa36, 31 },
    { 0xab98fa37, 61 },
    { 0x58a8fa39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428fa3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5308fa42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8fa47, 85 },
    { 0xeb18fa48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc8fa4d, 90 },
    { 0, 0 },
    { 0x46a8fa4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fa52, 23 },
    { 0x58a8fa53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8fa5a, 91 },
    { 0x45a8fa5a, 76 },
    { 0, 0 },
    { 0xf4c8fa5d, 23 },
    { 0x4ea8fa5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88fa61, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf98fa67, 90 },
    { 0x23a8fa68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7418fa6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e38fa74, 99 },
    { 0xace8fa75, 52 },
    { 0x7d8fa76, 78 },
    { 0, 0 },
    { 0x4898fa78, 33 },
    { 0, 0 },
    { 0x9dd8fa7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58fa7e, 95 },
    { 0xf998fa7e, 23 },
    { 0x9c88fa7f, 90 },
    { 0xe948fa7e, 66 },
    { 0x6b8fa82, 34 },
    { 0xfee8fa81, 33 },
    { 0xf3d8fa84, 33 },
    { 0xbf38fa85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef8fa8b, 33 },
    { 0xa198fa8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b58fa92, 76 },
    { 0xef18fa93, 95 },
    { 0xabf8fa93, 61 },
    { 0x7bf8fa94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd18fa99, 87 },
    { 0, 0 },
    { 0x6c98fa9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3538fa9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5858faa2, 17 },
    { 0x6ad8faa3, 33 },
    { 0xdb18faa4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb58faa8, 43 },
    { 0xb558faa9, 94 },
    { 0, 0 },
    { 0xf398faab, 34 },
    { 0x5ea8faac, 61 },
    { 0x1178faad, 95 },
    { 0, 0 },
    { 0xf008faaf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f8fab2, 98 },
    { 0x6638fab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48fab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb058fabd, 66 },
    { 0xef68fabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b68fac4, 64 },
    { 0, 0 },
    { 0xd0a8fac6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d8faca, 61 },
    { 0x6878facb, 61 },
    { 0x4c08facc, 56 },
    { 0x1d98facc, 80 },
    { 0xf908face, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8fad1, 101 },
    { 0x2988fad2, 55 },
    { 0xa8f8fad2, 23 },
    { 0x5ba8fad3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d48fad8, 53 },
    { 0xefa8fad9, 90 },
    { 0x9708fad9, 46 },
    { 0xfde8fadb, 90 },
    { 0x7838fadb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8fae1, 61 },
    { 0x86b8fae1, 33 },
    { 0x58fae3, 95 },
    { 0xcd28fae4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b8fae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af8faed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3098faf2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678faf5, 90 },
    { 0x2668faf6, 33 },
    { 0x7e68faf5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8318fafb, 90 },
    { 0x8208fafc, 87 },
    { 0x7f08fafd, 100 },
    { 0, 0 },
    { 0x6d48faff, 90 },
    { 0, 0 },
    { 0x1df8fb01, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d8fb04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe468fb0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe358fb0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708fb13, 75 },
    { 0, 0 },
    { 0xa8c8fb15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d08fb1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a8fb1e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d48fb22, 43 },
    { 0x4e98fb23, 85 },
    { 0, 0 },
    { 0x4c38fb25, 33 },
    { 0x96e8fb25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce8fb2e, 4 },
    { 0x1168fb2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fb35, 23 },
    { 0xe418fb36, 33 },
    { 0x5dc8fb35, 90 },
    { 0xc768fb38, 90 },
    { 0, 0 },
    { 0x2278fb3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8fb40, 46 },
    { 0x5278fb41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc38fb53, 55 },
    { 0x19d8fb53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf88fb57, 67 },
    { 0x95f8fb58, 33 },
    { 0xab08fb59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fb61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08fb64, 90 },
    { 0xebd8fb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e8fb71, 90 },
    { 0xdb08fb72, 34 },
    { 0x74b8fb73, 98 },
    { 0xca18fb71, 61 },
    { 0x1f28fb75, 55 },
    { 0xa9e8fb75, 90 },
    { 0xdd78fb77, 90 },
    { 0x4118fb76, 33 },
    { 0x3a08fb79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc078fb7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad08fb80, 96 },
    { 0, 0 },
    { 0x5308fb82, 91 },
    { 0xaad8fb83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a78fb87, 90 },
    { 0x3558fb87, 55 },
    { 0xde58fb89, 90 },
    { 0, 0 },
    { 0xa978fb8b, 69 },
    { 0x9938fb8c, 76 },
    { 0x6dc8fb8b, 90 },
    { 0xf458fb8e, 46 },
    { 0xac98fb8d, 69 },
    { 0x2588fb90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e48fb9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3838fba3, 61 },
    { 0x8f58fba3, 33 },
    { 0xc438fba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d8fbad, 67 },
    { 0, 0 },
    { 0xe508fbaf, 61 },
    { 0x7f98fbaf, 90 },
    { 0xdf58fbb1, 90 },
    { 0xcdf8fbb2, 77 },
    { 0xa978fbb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8fbbc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a8fbc0, 33 },
    { 0xf588fbc1, 93 },
    { 0x7f38fbc2, 43 },
    { 0x4418fbc1, 95 },
    { 0xe108fbc0, 95 },
    { 0x2278fbc0, 61 },
    { 0x2798fbc6, 87 },
    { 0xe998fbc7, 33 },
    { 0, 0 },
    { 0xfb28fbc9, 55 },
    { 0x9fb8fbca, 85 },
    { 0x59b8fbcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae18fbce, 52 },
    { 0xe18fbce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b38fbd2, 88 },
    { 0x6d18fbd3, 93 },
    { 0x3158fbd3, 90 },
    { 0xd218fbd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6678fbd8, 61 },
    { 0x518fbd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb348fbdf, 33 },
    { 0xf008fbe0, 61 },
    { 0xa078fbe1, 98 },
    { 0x4528fbdf, 93 },
    { 0xf748fbe3, 36 },
    { 0xd488fbe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e8fbec, 61 },
    { 0x5708fbed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4288fbf0, 85 },
    { 0x9cd8fbf0, 98 },
    { 0x1b8fbf0, 33 },
    { 0, 0 },
    { 0x4cf8fbf4, 61 },
    { 0x1ef8fbf5, 100 },
    { 0x7458fbf6, 33 },
    { 0x55d8fbf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e8fbfb, 76 },
    { 0xdc98fbfb, 33 },
    { 0x178fbfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298fc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928fc09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f08fc11, 33 },
    { 0xf568fc12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8fc15, 98 },
    { 0xda58fc16, 98 },
    { 0x99b8fc17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed8fc1a, 98 },
    { 0, 0 },
    { 0xce18fc1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c8fc24, 99 },
    { 0x2c78fc25, 77 },
    { 0, 0 },
    { 0x2e48fc27, 98 },
    { 0xc408fc28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7328fc2f, 90 },
    { 0x73a8fc30, 55 },
    { 0xe768fc31, 61 },
    { 0xef48fc32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf528fc37, 61 },
    { 0x6a18fc38, 100 },
    { 0xd098fc37, 98 },
    { 0xaff8fc38, 95 },
    { 0, 0 },
    { 0x83e8fc3c, 76 },
    { 0, 0 },
    { 0x5c48fc3e, 76 },
    { 0xfae8fc3f, 98 },
    { 0x4988fc3f, 90 },
    { 0xb658fc41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8fc44, 61 },
    { 0, 0 },
    { 0xf98fc46, 43 },
    { 0xb2f8fc46, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8fc4f, 90 },
    { 0x8e48fc50, 85 },
    { 0xd698fc50, 38 },
    { 0, 0 },
    { 0xdbe8fc53, 85 },
    { 0x6938fc54, 55 },
    { 0x9b88fc54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18fc5b, 67 },
    { 0xbe08fc5c, 69 },
    { 0x4b98fc5d, 23 },
    { 0x2488fc5d, 98 },
    { 0xe998fc5d, 61 },
    { 0x3588fc5b, 93 },
    { 0x43c8fc5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c8fc66, 55 },
    { 0xa368fc67, 33 },
    { 0, 0 },
    { 0x6528fc69, 90 },
    { 0xc5f8fc69, 52 },
    { 0x4228fc6b, 100 },
    { 0x98e8fc6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8fc72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698fc76, 99 },
    { 0x89b8fc77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fc7a, 33 },
    { 0x6da8fc7b, 98 },
    { 0, 0 },
    { 0xf078fc7d, 87 },
    { 0xead8fc7e, 43 },
    { 0x23d8fc7e, 95 },
    { 0xf2f8fc80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58fc84, 33 },
    { 0xc988fc85, 61 },
    { 0x13c8fc85, 95 },
    { 0xc9d8fc86, 55 },
    { 0x6c48fc84, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8fc8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8fc8f, 90 },
    { 0xd7d8fc8f, 55 },
    { 0, 0 },
    { 0xfac8fc92, 95 },
    { 0, 0 },
    { 0xf5e8fc94, 83 },
    { 0x6728fc95, 90 },
    { 0xbbf8fc95, 61 },
    { 0, 0 },
    { 0x61a8fc98, 98 },
    { 0x4248fc98, 78 },
    { 0xba78fc9a, 43 },
    { 0, 0 },
    { 0x9348fc9c, 90 },
    { 0x2048fc9d, 23 },
    { 0xb8a8fc9e, 85 },
    { 0x49f8fc9f, 52 },
    { 0xfc68fca0, 76 },
    { 0x4c28fca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478fcad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee8fcb1, 78 },
    { 0xfbc8fcb1, 90 },
    { 0x82d8fcb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5208fcb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f98fcba, 86 },
    { 0xd508fcba, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae8fcc1, 23 },
    { 0, 0 },
    { 0x4d18fcc3, 61 },
    { 0, 0 },
    { 0xb58fcc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff58fcca, 78 },
    { 0x66f8fccb, 33 },
    { 0xe758fcca, 85 },
    { 0xb958fccb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf8fcd2, 90 },
    { 0x33d8fcd2, 90 },
    { 0xaa18fcd4, 33 },
    { 0xa148fcd4, 23 },
    { 0xf378fcd3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a8fcda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb828fce1, 52 },
    { 0, 0 },
    { 0xe718fce3, 43 },
    { 0, 0 },
    { 0x3198fce5, 93 },
    { 0xbb68fce6, 98 },
    { 0xcef8fce7, 33 },
    { 0xf1a8fce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe8fceb, 61 },
    { 0xe6d8fcec, 34 },
    { 0x4c48fcec, 61 },
    { 0xafa8fcee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348fcf1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf8fcf8, 33 },
    { 0, 0 },
    { 0xdf28fcfa, 98 },
    { 0xdb58fcfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d8fcfe, 69 },
    { 0, 0 },
    { 0x4e08fd00, 76 },
    { 0xd08fd01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4648fd0b, 77 },
    { 0, 0 },
    { 0xb3c8fd0d, 76 },
    { 0xcbc8fd0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc158fd11, 90 },
    { 0xe1d8fd12, 33 },
    { 0xf648fd11, 98 },
    { 0, 0 },
    { 0xc0f8fd15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe388fd19, 100 },
    { 0x84b8fd1a, 95 },
    { 0x9a8fd1b, 67 },
    { 0xe2a8fd1c, 92 },
    { 0, 0 },
    { 0x69f8fd1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b8fd23, 88 },
    { 0, 0 },
    { 0xeb18fd25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a38fd2a, 55 },
    { 0, 0 },
    { 0xf6d8fd2c, 66 },
    { 0x5458fd2d, 90 },
    { 0, 0 },
    { 0x3438fd2f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf348fd3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998fd40, 34 },
    { 0x5e38fd41, 98 },
    { 0, 0 },
    { 0xad68fd43, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e08fd4e, 23 },
    { 0, 0 },
    { 0x59b8fd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca58fd57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6528fd5e, 52 },
    { 0x23e8fd5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab8fd6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d8fd6d, 33 },
    { 0xb848fd6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b8fd71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38fd76, 61 },
    { 0x6108fd77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa8fd7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28fd80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4558fd8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x858fd8d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898fd91, 95 },
    { 0, 0 },
    { 0x18e8fd93, 46 },
    { 0x6818fd94, 65 },
    { 0x9a78fd95, 23 },
    { 0xed8fd95, 23 },
    { 0xbf88fd96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd28fd9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe8fd9d, 90 },
    { 0x3f38fd9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3198fda2, 33 },
    { 0xf98fda3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c68fda7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928fdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb8fdb2, 95 },
    { 0x6ca8fdb3, 76 },
    { 0x12c8fdb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2308fdb8, 90 },
    { 0, 0 },
    { 0xfb88fdba, 90 },
    { 0x1ac8fdba, 76 },
    { 0x37b8fdbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e08fdc0, 61 },
    { 0xb2b8fdc1, 61 },
    { 0xc988fdc0, 95 },
    { 0x9a08fdc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc8fdc9, 61 },
    { 0xc7a8fdca, 33 },
    { 0xf968fdc9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368fdce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a8fdd2, 93 },
    { 0x6528fdd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4628fdd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8fde0, 87 },
    { 0x558fde0, 23 },
    { 0x6cc8fde2, 98 },
    { 0x7638fde3, 33 },
    { 0x73a8fde2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc218fde8, 33 },
    { 0x9a78fde9, 76 },
    { 0x14c8fde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3788fded, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a8fdf0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c8fdf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7558fe02, 55 },
    { 0x8f8fe03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7608fe0a, 55 },
    { 0xa058fe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f88fe0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38fe13, 36 },
    { 0x18a8fe14, 33 },
    { 0x2088fe15, 78 },
    { 0x5818fe13, 65 },
    { 0xbb18fe15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf128fe1a, 33 },
    { 0x2398fe1b, 95 },
    { 0x7dc8fe1c, 33 },
    { 0x4ba8fe1a, 95 },
    { 0x5e38fe1e, 23 },
    { 0x17b8fe1e, 90 },
    { 0x5de8fe20, 61 },
    { 0x75b8fe21, 66 },
    { 0x6b08fe21, 55 },
    { 0x4568fe22, 43 },
    { 0x4488fe24, 55 },
    { 0, 0 },
    { 0x1d98fe26, 98 },
    { 0, 0 },
    { 0x5d58fe28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f88fe2d, 90 },
    { 0x4a18fe2d, 84 },
    { 0xacb8fe2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8fe34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1828fe3d, 99 },
    { 0, 0 },
    { 0xf208fe3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf968fe42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958fe47, 76 },
    { 0xab28fe48, 88 },
    { 0, 0 },
    { 0xdd78fe4a, 84 },
    { 0x3d88fe4b, 23 },
    { 0xe8a8fe4c, 95 },
    { 0x248fe4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18fe50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08fe55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c8fe59, 61 },
    { 0, 0 },
    { 0x7388fe5b, 28 },
    { 0x2258fe5b, 46 },
    { 0x40e8fe5d, 98 },
    { 0xeb78fe5d, 98 },
    { 0, 0 },
    { 0xf928fe60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc8fe67, 34 },
    { 0x8028fe67, 17 },
    { 0xc18fe69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3328fe79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888fe7f, 46 },
    { 0xe8c8fe7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fe85, 76 },
    { 0x8f98fe86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c8fe8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa638fe92, 90 },
    { 0x1358fe93, 55 },
    { 0, 0 },
    { 0x7428fe95, 61 },
    { 0x51b8fe95, 61 },
    { 0xdf08fe97, 52 },
    { 0x15b8fe95, 95 },
    { 0xe4d8fe99, 96 },
    { 0x8bb8fe96, 90 },
    { 0, 0 },
    { 0x5648fe9c, 81 },
    { 0xe408fe9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28fea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2798fea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8fea9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe008fead, 33 },
    { 0, 0 },
    { 0x8368feaf, 95 },
    { 0, 0 },
    { 0x6ae8feb1, 61 },
    { 0xfff8feb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb508feb6, 78 },
    { 0, 0 },
    { 0x9078feb8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb438febc, 66 },
    { 0x95b8febd, 23 },
    { 0xbb78febe, 76 },
    { 0, 0 },
    { 0x3778fec0, 33 },
    { 0, 0 },
    { 0x4618fec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e98fec7, 98 },
    { 0xee38fec8, 33 },
    { 0, 0 },
    { 0x12a8feca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b8fecd, 61 },
    { 0x9378fece, 55 },
    { 0x2e28fecf, 90 },
    { 0xcf58fecf, 46 },
    { 0x5f58fed0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028fed6, 17 },
    { 0xa48fed6, 90 },
    { 0x1b78fed8, 76 },
    { 0x8408fed9, 17 },
    { 0x3fe8feda, 64 },
    { 0x3638fed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b78fee0, 43 },
    { 0xd208fee1, 85 },
    { 0, 0 },
    { 0xb218fee3, 76 },
    { 0x8d18fee4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf858feea, 23 },
    { 0xca48feea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf618fef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x968fef6, 90 },
    { 0x3298fef7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8fefa, 98 },
    { 0, 0 },
    { 0x3738fefc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668ff03, 90 },
    { 0xfd8ff03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa8ff07, 23 },
    { 0, 0 },
    { 0xb428ff09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98ff0c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c8ff12, 98 },
    { 0, 0 },
    { 0x7248ff14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f8ff1a, 55 },
    { 0x8e8ff1b, 23 },
    { 0x37f8ff1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8ff26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb248ff29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af8ff2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe948ff32, 98 },
    { 0x5028ff33, 76 },
    { 0x7ae8ff32, 55 },
    { 0x15b8ff35, 69 },
    { 0xdfc8ff35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb8ff39, 99 },
    { 0xd488ff3a, 99 },
    { 0x85a8ff3b, 100 },
    { 0xf128ff3c, 95 },
    { 0xaae8ff3d, 33 },
    { 0, 0 },
    { 0x7688ff3f, 95 },
    { 0, 0 },
    { 0x4a48ff41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4798ff45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a8ff4c, 98 },
    { 0x9a8ff4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e28ff50, 23 },
    { 0x1638ff51, 61 },
    { 0xa9b8ff50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2598ff5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8ff62, 55 },
    { 0x65b8ff63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a8ff67, 98 },
    { 0x9ea8ff68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8ff70, 76 },
    { 0x14d8ff70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18ff76, 43 },
    { 0x7118ff77, 99 },
    { 0x4bd8ff78, 61 },
    { 0, 0 },
    { 0x8018ff7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a8ff7f, 17 },
    { 0, 0 },
    { 0x16e8ff81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8ff88, 23 },
    { 0x90b8ff88, 59 },
    { 0x3fc8ff8a, 52 },
    { 0x19d8ff8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6188ff90, 95 },
    { 0x2448ff91, 98 },
    { 0x7598ff92, 77 },
    { 0x7c28ff93, 34 },
    { 0x9c88ff91, 76 },
    { 0xe2e8ff91, 94 },
    { 0xa9c8ff96, 95 },
    { 0xd8f8ff90, 85 },
    { 0, 0 },
    { 0xb4d8ff99, 90 },
    { 0, 0 },
    { 0xc88ff9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b88ffa9, 95 },
    { 0xe8f8ffaa, 92 },
    { 0x93e8ffab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7248ffaf, 90 },
    { 0, 0 },
    { 0x4f78ffb1, 85 },
    { 0x7b28ffb1, 33 },
    { 0xa128ffb2, 23 },
    { 0, 0 },
    { 0xd248ffb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc158ffb8, 55 },
    { 0xe4f8ffb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac98ffbc, 95 },
    { 0, 0 },
    { 0x6e28ffbe, 34 },
    { 0xf308ffbf, 61 },
    { 0x3108ffc0, 23 },
    { 0x2338ffc0, 36 },
    { 0x7948ffc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8ffc6, 66 },
    { 0x1b88ffc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc08ffcb, 61 },
    { 0x5898ffcc, 83 },
    { 0, 0 },
    { 0xa6c8ffce, 78 },
    { 0x1d88ffce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8ffd2, 76 },
    { 0xf778ffd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8ffd6, 88 },
    { 0x4b78ffd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5408ffde, 100 },
    { 0x5238ffdf, 67 },
    { 0xf448ffe0, 77 },
    { 0x3838ffe1, 98 },
    { 0, 0 },
    { 0xb488ffe3, 87 },
    { 0x7a8ffe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a8ffe9, 76 },
    { 0xe98ffea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca8ffed, 61 },
    { 0x42f8ffee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a58fff5, 23 },
    { 0, 0 },
    { 0x59d8fff7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xad78fffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8fffe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x70490001, 23 },
    { 0, 0 },
    { 0x4c90003, 76 },
    { 0, 0 },
    { 0xfac90005, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f9000c, 80 },
    { 0xe999000c, 91 },
    { 0x79c9000e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f190011, 61 },
    { 0x68c90012, 93 },
    { 0xe0e90013, 43 },
    { 0x40990013, 55 },
    { 0x7ad90014, 23 },
    { 0x5190016, 95 },
    { 0x5ae90017, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89001a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a9001e, 61 },
    { 0xba79001e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4990022, 81 },
    { 0x2cb90023, 87 },
    { 0x2c590023, 23 },
    { 0xca690023, 33 },
    { 0, 0 },
    { 0x64890027, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x299002a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5659002f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20090032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea190035, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0690039, 90 },
    { 0xa1a9003a, 98 },
    { 0, 0 },
    { 0x2409003c, 76 },
    { 0x1819003d, 91 },
    { 0xa269003c, 90 },
    { 0x46d9003f, 98 },
    { 0xab690040, 52 },
    { 0x24c9003f, 85 },
    { 0xe5a9003f, 90 },
    { 0x48e90043, 33 },
    { 0x46190041, 90 },
    { 0x1b990040, 95 },
    { 0x31d9003e, 55 },
    { 0x18690047, 90 },
    { 0, 0 },
    { 0x40390049, 34 },
    { 0xfa9004a, 90 },
    { 0x9aa90049, 55 },
    { 0, 0 },
    { 0x549004d, 33 },
    { 0, 0 },
    { 0x2ac9004f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9005b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39005f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e290063, 95 },
    { 0xe3590064, 90 },
    { 0x33890064, 90 },
    { 0xe9c90066, 55 },
    { 0x12d90064, 99 },
    { 0xe4390068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a9006d, 33 },
    { 0x13b9006e, 93 },
    { 0xd969006f, 34 },
    { 0, 0 },
    { 0x3bf90071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90074, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b090078, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f9007e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad190082, 33 },
    { 0xaed90083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9990089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a9008c, 52 },
    { 0x7219008d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7790094, 76 },
    { 0xdff90095, 87 },
    { 0x17890095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f9009b, 34 },
    { 0x4339009c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e900a0, 96 },
    { 0xc8d900a1, 90 },
    { 0xd7a900a1, 87 },
    { 0xb22900a0, 99 },
    { 0x2f0900a1, 92 },
    { 0xde8900a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd900ab, 90 },
    { 0x83c900ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb900b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a900b5, 61 },
    { 0xc50900b6, 36 },
    { 0, 0 },
    { 0x90e900b8, 81 },
    { 0x7ee900b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x345900bc, 33 },
    { 0x2b2900bc, 67 },
    { 0x8bb900be, 90 },
    { 0x7df900bf, 33 },
    { 0, 0 },
    { 0xbf4900c1, 43 },
    { 0x49c900c1, 40 },
    { 0x7a900c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x289900c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6900ca, 99 },
    { 0x432900cb, 34 },
    { 0x88d900cc, 98 },
    { 0x9f1900cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284900d2, 66 },
    { 0, 0 },
    { 0x381900d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16900d8, 55 },
    { 0xad7900d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d900dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62900df, 76 },
    { 0x4e8900df, 95 },
    { 0x58a900e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19900e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac900ea, 93 },
    { 0xb9e900eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39900ee, 76 },
    { 0xf2f900ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e900f2, 55 },
    { 0xdf2900f3, 46 },
    { 0, 0 },
    { 0xbb8900f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37900fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc90106, 61 },
    { 0x6d90107, 23 },
    { 0x5d490108, 83 },
    { 0xa4b90109, 98 },
    { 0xe3f9010a, 83 },
    { 0x1aa9010a, 95 },
    { 0xd059010c, 46 },
    { 0x9c49010a, 34 },
    { 0x2629010e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22390112, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd290117, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569011d, 98 },
    { 0xafd9011e, 30 },
    { 0, 0 },
    { 0x4c290120, 93 },
    { 0x19390121, 90 },
    { 0x3ca90121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c90125, 61 },
    { 0, 0 },
    { 0x85890127, 33 },
    { 0x8af90127, 34 },
    { 0xba890128, 95 },
    { 0xe009012a, 33 },
    { 0x60a9012b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e490134, 81 },
    { 0x4c290134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40790138, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4749013d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc390141, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef690149, 90 },
    { 0x8ba90149, 90 },
    { 0xe09014b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99014f, 90 },
    { 0xb8d90150, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16790159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a9015c, 23 },
    { 0xa6b9015c, 23 },
    { 0xe629015c, 81 },
    { 0, 0 },
    { 0x66a90160, 23 },
    { 0x2ad90160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5890166, 33 },
    { 0, 0 },
    { 0x43290168, 95 },
    { 0xbc190168, 91 },
    { 0xd649016a, 55 },
    { 0, 0 },
    { 0x989016c, 95 },
    { 0x4e19016c, 101 },
    { 0xfb39016e, 55 },
    { 0x5929016e, 55 },
    { 0x7a190170, 61 },
    { 0x71490170, 64 },
    { 0x31690170, 61 },
    { 0xd389016e, 93 },
    { 0x31a90174, 33 },
    { 0x30890175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6090178, 33 },
    { 0, 0 },
    { 0x34c9017a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79017d, 95 },
    { 0, 0 },
    { 0x60f9017f, 61 },
    { 0, 0 },
    { 0xad090181, 76 },
    { 0x86c90182, 87 },
    { 0x2e190183, 23 },
    { 0x32590184, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe689018b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7790191, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d90198, 90 },
    { 0, 0 },
    { 0xc989019a, 61 },
    { 0xe2d9019a, 23 },
    { 0xdd29019a, 85 },
    { 0x2069019b, 95 },
    { 0x7919019e, 43 },
    { 0x6b49019c, 23 },
    { 0xdb7901a0, 55 },
    { 0, 0 },
    { 0x59f901a2, 33 },
    { 0xcaa901a2, 17 },
    { 0x344901a2, 95 },
    { 0x2de901a5, 61 },
    { 0xa2b901a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37901ab, 81 },
    { 0xa41901ac, 45 },
    { 0x809901ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7901b0, 99 },
    { 0xe31901b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4901b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710901b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae901bd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6901c3, 69 },
    { 0x49d901c4, 98 },
    { 0x886901c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69901c8, 34 },
    { 0xf7e901c9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f901cd, 85 },
    { 0xc6b901ce, 75 },
    { 0x6be901cd, 23 },
    { 0x39c901d0, 95 },
    { 0xe39901d1, 61 },
    { 0, 0 },
    { 0xafc901d3, 67 },
    { 0, 0 },
    { 0xc4e901d5, 33 },
    { 0x9c7901d5, 98 },
    { 0xf66901d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39901da, 84 },
    { 0, 0 },
    { 0xafa901dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3901e1, 33 },
    { 0x892901e2, 88 },
    { 0xde901e2, 43 },
    { 0x43e901e4, 67 },
    { 0xaad901e3, 61 },
    { 0, 0 },
    { 0x512901e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62901eb, 98 },
    { 0xc1f901ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x995901ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc901ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90202, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46990205, 78 },
    { 0x95590205, 34 },
    { 0x96390207, 91 },
    { 0x3390207, 33 },
    { 0x76590209, 90 },
    { 0x35a9020a, 98 },
    { 0x4509020b, 77 },
    { 0, 0 },
    { 0x3e49020d, 93 },
    { 0x48f9020e, 76 },
    { 0x1099020d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x50590212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9021a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6249021d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e290223, 98 },
    { 0x78390223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae590229, 55 },
    { 0, 0 },
    { 0x8ac9022b, 77 },
    { 0xb659022c, 76 },
    { 0x8669022c, 75 },
    { 0xd619022e, 95 },
    { 0xddc9022e, 23 },
    { 0, 0 },
    { 0xe9990231, 61 },
    { 0x4f590231, 67 },
    { 0x91a90231, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f90236, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b49023e, 46 },
    { 0xde39023e, 87 },
    { 0xb3c9023e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8090243, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa90248, 46 },
    { 0, 0 },
    { 0x3d99024a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68690250, 55 },
    { 0x9c790251, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27190255, 43 },
    { 0xf5990256, 98 },
    { 0x2f990256, 93 },
    { 0, 0 },
    { 0x75690259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa9025d, 34 },
    { 0x8ea9025e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb90261, 66 },
    { 0, 0 },
    { 0x41490263, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f190268, 98 },
    { 0x2b790269, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b9026c, 90 },
    { 0x4ca9026d, 33 },
    { 0x809026e, 95 },
    { 0x9249026f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67690275, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a90279, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6019027c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9027f, 90 },
    { 0xb8990280, 61 },
    { 0, 0 },
    { 0xcfb90282, 98 },
    { 0xcd390283, 94 },
    { 0x86890284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae90287, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de9028b, 33 },
    { 0x9dd9028b, 55 },
    { 0xee39028d, 34 },
    { 0x26f9028e, 78 },
    { 0xb2c9028c, 36 },
    { 0xc1090290, 34 },
    { 0, 0 },
    { 0xf3b90292, 76 },
    { 0, 0 },
    { 0xe5a90294, 99 },
    { 0, 0 },
    { 0xd890296, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc9029a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e902a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949902a4, 33 },
    { 0, 0 },
    { 0x42f902a6, 66 },
    { 0, 0 },
    { 0x974902a8, 61 },
    { 0x1cc902a9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557902b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99902bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303902c1, 91 },
    { 0x7b3902c2, 78 },
    { 0, 0 },
    { 0x59a902c4, 33 },
    { 0, 0 },
    { 0xf02902c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x128902c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7902d3, 33 },
    { 0, 0 },
    { 0x683902d5, 65 },
    { 0xa9c902d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7902d9, 33 },
    { 0, 0 },
    { 0x368902db, 77 },
    { 0x845902dc, 76 },
    { 0, 0 },
    { 0x76c902de, 99 },
    { 0, 0 },
    { 0xe81902e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2902f4, 55 },
    { 0, 0 },
    { 0x6eb902f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc902fa, 67 },
    { 0xb69902fb, 46 },
    { 0x90c902fc, 84 },
    { 0x895902fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a90300, 95 },
    { 0x15d90301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89190304, 61 },
    { 0, 0 },
    { 0x5df90306, 33 },
    { 0xca90306, 33 },
    { 0x37190307, 99 },
    { 0xa1890308, 98 },
    { 0, 0 },
    { 0x169030b, 78 },
    { 0x41c9030c, 67 },
    { 0x5fd9030d, 90 },
    { 0x1fe9030c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69790311, 100 },
    { 0x34b90312, 76 },
    { 0x58690313, 90 },
    { 0xccb90314, 55 },
    { 0xe9890314, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559031a, 81 },
    { 0x31e9031b, 55 },
    { 0x2449031a, 98 },
    { 0x3fc9031c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e90323, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2190328, 23 },
    { 0x79b90329, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e9032c, 55 },
    { 0x9609032d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690330, 85 },
    { 0, 0 },
    { 0xc8c90332, 55 },
    { 0x76f90333, 33 },
    { 0x5ee90334, 93 },
    { 0x8c090335, 98 },
    { 0, 0 },
    { 0x8ce90337, 95 },
    { 0xbbd90338, 95 },
    { 0x21390339, 23 },
    { 0, 0 },
    { 0x75c9033b, 100 },
    { 0x1f9033c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d09033f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd590344, 23 },
    { 0, 0 },
    { 0x7fb90346, 96 },
    { 0xa0290346, 33 },
    { 0x7fc90348, 55 },
    { 0, 0 },
    { 0xdac9034a, 83 },
    { 0x10f9034b, 43 },
    { 0xc7b9034c, 55 },
    { 0x80f9034b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf990350, 23 },
    { 0, 0 },
    { 0x35b90352, 90 },
    { 0, 0 },
    { 0xa8e90354, 87 },
    { 0xda490355, 90 },
    { 0x71090356, 43 },
    { 0xf6490357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789035d, 76 },
    { 0, 0 },
    { 0xfd19035f, 90 },
    { 0xcb390360, 90 },
    { 0x27490361, 23 },
    { 0x71e90362, 98 },
    { 0xb2f9035f, 33 },
    { 0x27c90361, 98 },
    { 0, 0 },
    { 0xc6490366, 76 },
    { 0x84490367, 75 },
    { 0x94a90367, 43 },
    { 0xb6890369, 34 },
    { 0xb1790369, 91 },
    { 0x6289036b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72090373, 43 },
    { 0x2c090374, 94 },
    { 0, 0 },
    { 0x89a90376, 88 },
    { 0x5de90377, 33 },
    { 0, 0 },
    { 0x50390379, 81 },
    { 0x9ab9037a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab9037d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a990383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629038b, 55 },
    { 0, 0 },
    { 0x77c9038d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90390, 43 },
    { 0xe2890391, 43 },
    { 0x9f890391, 70 },
    { 0, 0 },
    { 0xf4b90394, 92 },
    { 0x9b790395, 93 },
    { 0xd1690394, 95 },
    { 0, 0 },
    { 0xd9790398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8669039c, 34 },
    { 0, 0 },
    { 0xfce9039e, 99 },
    { 0xf779039f, 66 },
    { 0x95a9039f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975903a4, 23 },
    { 0xb1e903a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d903a9, 76 },
    { 0x34b903aa, 23 },
    { 0x8a6903ab, 46 },
    { 0xb4e903ac, 87 },
    { 0, 0 },
    { 0x41b903ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746903b4, 95 },
    { 0, 0 },
    { 0xc3d903b6, 88 },
    { 0x18c903b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b903bb, 84 },
    { 0xfd1903bc, 90 },
    { 0xac5903bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa903c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7903c3, 98 },
    { 0, 0 },
    { 0xd60903c5, 90 },
    { 0xe92903c5, 95 },
    { 0x3ce903c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a903cc, 23 },
    { 0xdec903cd, 91 },
    { 0, 0 },
    { 0xed6903cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800903d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af903da, 90 },
    { 0, 0 },
    { 0x4ac903dc, 61 },
    { 0, 0 },
    { 0xfbb903de, 61 },
    { 0x8b4903de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c903e2, 55 },
    { 0, 0 },
    { 0xf5e903e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6903e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24903eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815903ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x155903f2, 55 },
    { 0xbda903f3, 90 },
    { 0xb11903f4, 90 },
    { 0xcf5903f4, 90 },
    { 0x970903f6, 93 },
    { 0x37a903f7, 87 },
    { 0xbe0903f2, 94 },
    { 0x77e903f9, 90 },
    { 0, 0 },
    { 0x5d2903fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a903fe, 91 },
    { 0x131903ff, 95 },
    { 0x2a8903ff, 76 },
    { 0, 0 },
    { 0xdfd90402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9040b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9590410, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72890413, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc169041c, 61 },
    { 0x9a29041d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0590421, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b90425, 65 },
    { 0x5c290426, 52 },
    { 0, 0 },
    { 0xd3190428, 90 },
    { 0x6290429, 85 },
    { 0xc2890429, 66 },
    { 0x2b19042b, 17 },
    { 0x3e29042c, 65 },
    { 0, 0 },
    { 0x6369042e, 76 },
    { 0x9c9042f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43390433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa90439, 77 },
    { 0x12190439, 46 },
    { 0x5c59043a, 61 },
    { 0x6199043c, 67 },
    { 0x55a90439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc90440, 17 },
    { 0x37b90441, 98 },
    { 0x54390441, 61 },
    { 0, 0 },
    { 0xd9590444, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe129044f, 90 },
    { 0x14290450, 30 },
    { 0x43590451, 65 },
    { 0, 0 },
    { 0xe0e90453, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91190459, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc919045e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a790462, 23 },
    { 0x38690462, 90 },
    { 0x8e290464, 81 },
    { 0x93990465, 90 },
    { 0x2dc90462, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86190469, 76 },
    { 0x7a19046a, 85 },
    { 0xab19046b, 84 },
    { 0x3f89046c, 95 },
    { 0x289046c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c90470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7790474, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190478, 33 },
    { 0xe0e90478, 33 },
    { 0xf8d90478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a9047d, 55 },
    { 0x5369047e, 33 },
    { 0, 0 },
    { 0xb5e90480, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1790484, 87 },
    { 0x49990484, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9390488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39048f, 90 },
    { 0, 0 },
    { 0xd0b90491, 98 },
    { 0x2e090492, 98 },
    { 0xc8390491, 61 },
    { 0xd8690491, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad190499, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f89049e, 95 },
    { 0x9f19049e, 91 },
    { 0, 0 },
    { 0xe2e904a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15904ab, 77 },
    { 0xf1d904ab, 76 },
    { 0xf83904ac, 77 },
    { 0xaba904ae, 100 },
    { 0x234904ae, 95 },
    { 0x22c904b0, 98 },
    { 0x887904ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e904b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb904c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b904c8, 61 },
    { 0xe71904c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36904cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06904d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35904db, 61 },
    { 0, 0 },
    { 0x3fc904dd, 90 },
    { 0x313904de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28904e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e904e5, 34 },
    { 0x33b904e6, 98 },
    { 0, 0 },
    { 0x4d3904e8, 95 },
    { 0xba7904e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c904ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8904f2, 65 },
    { 0x39c904f3, 33 },
    { 0, 0 },
    { 0x8b7904f5, 98 },
    { 0xfc6904f5, 61 },
    { 0x8c904f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43904fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c90503, 55 },
    { 0x4af90503, 90 },
    { 0, 0 },
    { 0x54c90506, 46 },
    { 0x92f90507, 81 },
    { 0, 0 },
    { 0xff990509, 99 },
    { 0x6dd90509, 23 },
    { 0, 0 },
    { 0x2d79050c, 90 },
    { 0xdbc9050d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b90516, 98 },
    { 0, 0 },
    { 0xc6d90518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6059051f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d90523, 33 },
    { 0x26a90524, 33 },
    { 0x1e590525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a290528, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269052e, 77 },
    { 0x8df9052e, 90 },
    { 0, 0 },
    { 0x56490531, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e90534, 23 },
    { 0x5fd90534, 95 },
    { 0x74d90536, 43 },
    { 0xe1990537, 90 },
    { 0, 0 },
    { 0x90290539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839053f, 95 },
    { 0x7db9053f, 61 },
    { 0xf589053f, 100 },
    { 0x89490542, 88 },
    { 0x42790541, 61 },
    { 0xc3a90544, 61 },
    { 0xc5290545, 90 },
    { 0xdbe90546, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19054b, 90 },
    { 0x819054c, 46 },
    { 0xbee9054d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78990550, 66 },
    { 0x21890551, 90 },
    { 0x8bd90551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a690556, 61 },
    { 0x6ce90556, 64 },
    { 0x38490558, 98 },
    { 0x9e390559, 98 },
    { 0, 0 },
    { 0x6439055b, 43 },
    { 0, 0 },
    { 0x189055d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c90560, 95 },
    { 0x25c90561, 66 },
    { 0, 0 },
    { 0x78c90563, 73 },
    { 0, 0 },
    { 0xfc390565, 95 },
    { 0, 0 },
    { 0x32790567, 95 },
    { 0xbf390567, 95 },
    { 0x8b890569, 33 },
    { 0x9a290569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc9056f, 61 },
    { 0, 0 },
    { 0xb7090571, 94 },
    { 0xf4c90571, 95 },
    { 0x8f090571, 95 },
    { 0xc1a90574, 75 },
    { 0, 0 },
    { 0x1fd90576, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x14690579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d09057c, 34 },
    { 0, 0 },
    { 0x6ff9057e, 69 },
    { 0x8519057e, 95 },
    { 0x41890580, 98 },
    { 0x3ae9057e, 55 },
    { 0x8dd9057f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4190587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99058a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb9058d, 77 },
    { 0x5199058e, 99 },
    { 0x7709058f, 55 },
    { 0, 0 },
    { 0x9a190591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d90594, 76 },
    { 0xd0b90595, 61 },
    { 0x9af90596, 85 },
    { 0x78090596, 46 },
    { 0x1890597, 99 },
    { 0x11990599, 33 },
    { 0x1b490599, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea905a4, 52 },
    { 0x9ef905a5, 90 },
    { 0xacd905a6, 76 },
    { 0xfe9905a6, 96 },
    { 0xc1905a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4905ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e905b2, 85 },
    { 0xfcc905b2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c905b6, 33 },
    { 0xe0b905b7, 33 },
    { 0x35e905b8, 46 },
    { 0x2ca905b7, 55 },
    { 0x37c905ba, 61 },
    { 0x7d905bb, 85 },
    { 0x58a905bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83905c1, 61 },
    { 0x4ae905c1, 98 },
    { 0, 0 },
    { 0xf88905c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c905c8, 85 },
    { 0x391905c9, 34 },
    { 0x864905c9, 98 },
    { 0, 0 },
    { 0xd90905cc, 91 },
    { 0xbf3905cc, 95 },
    { 0, 0 },
    { 0xb8c905cf, 95 },
    { 0, 0 },
    { 0xaf2905d1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2905e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x810905e3, 61 },
    { 0xa68905e4, 90 },
    { 0x2f1905e5, 87 },
    { 0xcbb905e6, 55 },
    { 0x171905e7, 66 },
    { 0xc8b905e3, 85 },
    { 0x360905e9, 34 },
    { 0x879905e9, 34 },
    { 0x6d1905ea, 40 },
    { 0xd05905e9, 46 },
    { 0x9cd905ec, 34 },
    { 0, 0 },
    { 0x5cd905ef, 34 },
    { 0, 0 },
    { 0x202905f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d905f9, 100 },
    { 0xa95905fa, 33 },
    { 0x372905f9, 61 },
    { 0x6d4905fc, 33 },
    { 0x40a905fc, 33 },
    { 0xee5905fe, 68 },
    { 0x603905fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49790602, 98 },
    { 0xb2290603, 85 },
    { 0x49b90602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c9060e, 98 },
    { 0xd949060f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19290612, 61 },
    { 0, 0 },
    { 0xbb190614, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9061d, 46 },
    { 0xb639061d, 99 },
    { 0x8ae9061f, 77 },
    { 0x9189061d, 38 },
    { 0x41f90621, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de90624, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9062a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3099062e, 95 },
    { 0x7009062f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc190632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc90635, 55 },
    { 0xb0290636, 76 },
    { 0xf0a90636, 55 },
    { 0, 0 },
    { 0x7290639, 34 },
    { 0x6229063a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d90642, 35 },
    { 0, 0 },
    { 0x37690644, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e9064a, 93 },
    { 0x269064b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c99064e, 61 },
    { 0x2999064f, 23 },
    { 0, 0 },
    { 0xb2090651, 95 },
    { 0xf9290652, 23 },
    { 0xd9b90653, 43 },
    { 0xa3590653, 98 },
    { 0xbc390655, 56 },
    { 0x6da90656, 33 },
    { 0x7ca90657, 61 },
    { 0x92190657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e90662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82790668, 98 },
    { 0, 0 },
    { 0x6519066a, 95 },
    { 0, 0 },
    { 0x5a09066c, 66 },
    { 0x7689066d, 61 },
    { 0x3aa9066d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b090672, 90 },
    { 0xbcd90673, 23 },
    { 0x2f590674, 34 },
    { 0, 0 },
    { 0xda390676, 98 },
    { 0x4f290676, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d19067c, 61 },
    { 0xd459067d, 95 },
    { 0xce49067e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b90682, 66 },
    { 0, 0 },
    { 0xf3590684, 99 },
    { 0x71c90685, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae9068c, 92 },
    { 0x4b59068d, 33 },
    { 0x1c29068e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28190694, 23 },
    { 0x94790695, 61 },
    { 0x1f490696, 61 },
    { 0xb7f90697, 90 },
    { 0x94f90698, 90 },
    { 0x6c590695, 90 },
    { 0x7ed9069a, 90 },
    { 0xb769069b, 46 },
    { 0x1e09069c, 43 },
    { 0xb799069c, 90 },
    { 0xef09069e, 95 },
    { 0, 0 },
    { 0x2f1906a0, 99 },
    { 0x795906a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b906a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410906aa, 61 },
    { 0, 0 },
    { 0x476906ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981906b0, 23 },
    { 0xd21906b1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c906b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x102906b7, 34 },
    { 0xaec906b8, 98 },
    { 0x1a3906b9, 90 },
    { 0x6b6906b9, 76 },
    { 0, 0 },
    { 0x449906bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x671906bf, 99 },
    { 0x6bc906bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61906c4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a906c9, 81 },
    { 0x828906c9, 33 },
    { 0xa02906cb, 98 },
    { 0xa8c906ca, 99 },
    { 0xd45906cd, 76 },
    { 0x176906ce, 55 },
    { 0xe58906cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c906d7, 90 },
    { 0xf54906d8, 98 },
    { 0xb27906d9, 33 },
    { 0xbca906da, 69 },
    { 0xaf3906da, 92 },
    { 0xd52906dc, 95 },
    { 0x347906dd, 34 },
    { 0x85d906d8, 90 },
    { 0x523906df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64906e2, 87 },
    { 0, 0 },
    { 0x25b906e4, 34 },
    { 0x8ba906e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x339906e8, 95 },
    { 0xfb3906e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5906f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26906fa, 88 },
    { 0xf51906fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149906ff, 66 },
    { 0xb690700, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e59070a, 23 },
    { 0x8e89070a, 90 },
    { 0, 0 },
    { 0xb569070d, 93 },
    { 0, 0 },
    { 0xb09070f, 46 },
    { 0x5fa90710, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f90714, 55 },
    { 0x8590715, 61 },
    { 0x8ae90715, 43 },
    { 0x64a90717, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859071a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2490721, 55 },
    { 0x6fb90722, 33 },
    { 0xd0a90723, 88 },
    { 0, 0 },
    { 0xb1690725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd079072a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a690730, 33 },
    { 0, 0 },
    { 0xc6390732, 95 },
    { 0x7a590733, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85790737, 95 },
    { 0x4f90737, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe9073f, 90 },
    { 0x12090740, 33 },
    { 0xf4e90740, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16890745, 33 },
    { 0xe1890746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f90749, 55 },
    { 0, 0 },
    { 0x41e9074b, 90 },
    { 0x2be9074c, 98 },
    { 0xee69074c, 76 },
    { 0x2c59074d, 95 },
    { 0xda09074f, 90 },
    { 0x18b90750, 95 },
    { 0x50d90751, 23 },
    { 0x5b690752, 90 },
    { 0x7a79074b, 55 },
    { 0x56890754, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab590758, 100 },
    { 0, 0 },
    { 0xb099075a, 76 },
    { 0x579075b, 98 },
    { 0, 0 },
    { 0x76d9075d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c90760, 33 },
    { 0, 0 },
    { 0x22b90762, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9076b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809076e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28990774, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc90778, 36 },
    { 0, 0 },
    { 0x6749077a, 33 },
    { 0, 0 },
    { 0x42d9077c, 98 },
    { 0xac49077d, 85 },
    { 0xe689077e, 51 },
    { 0xaa59077f, 46 },
    { 0, 0 },
    { 0x42e90781, 76 },
    { 0x96e90782, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c590786, 23 },
    { 0xadd90787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9490791, 43 },
    { 0x27f90792, 43 },
    { 0x66390792, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0990797, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9079a, 34 },
    { 0x73c9079a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209079e, 76 },
    { 0, 0 },
    { 0xd88907a0, 98 },
    { 0xf6f907a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb907a4, 98 },
    { 0, 0 },
    { 0xe42907a6, 33 },
    { 0xbbe907a7, 98 },
    { 0, 0 },
    { 0x8bd907a9, 64 },
    { 0, 0 },
    { 0x754907ab, 90 },
    { 0xddf907ac, 46 },
    { 0x500907ad, 85 },
    { 0x48d907ac, 46 },
    { 0x301907af, 93 },
    { 0x79c907ac, 98 },
    { 0xc32907b0, 23 },
    { 0x7ad907b2, 76 },
    { 0x5b4907b3, 81 },
    { 0xcd907b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6907b7, 33 },
    { 0x4d7907b8, 46 },
    { 0, 0 },
    { 0x959907ba, 95 },
    { 0x97907bb, 43 },
    { 0xa39907bb, 76 },
    { 0xe6d907bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68907c0, 33 },
    { 0, 0 },
    { 0x218907c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x991907c5, 84 },
    { 0xec9907c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9907c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3907d1, 61 },
    { 0x81907d2, 43 },
    { 0x8e0907d3, 33 },
    { 0x475907d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e907d7, 95 },
    { 0, 0 },
    { 0x5da907d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542907e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd907ea, 90 },
    { 0x667907ea, 34 },
    { 0x151907ea, 95 },
    { 0xa24907ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d907f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c907f4, 83 },
    { 0, 0 },
    { 0x18907f6, 33 },
    { 0x9be907f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181907fb, 99 },
    { 0, 0 },
    { 0x8d0907fd, 39 },
    { 0xd20907fe, 61 },
    { 0xe6b907ff, 52 },
    { 0xe5190800, 55 },
    { 0xef390800, 46 },
    { 0xd9c90800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a90805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7990808, 17 },
    { 0xfdb90808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89080d, 23 },
    { 0x5259080d, 33 },
    { 0x7029080f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e90815, 23 },
    { 0x6be90815, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b9081b, 23 },
    { 0, 0 },
    { 0x7ae9081d, 61 },
    { 0x9649081d, 23 },
    { 0, 0 },
    { 0x56490820, 90 },
    { 0, 0 },
    { 0xf1490822, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85990827, 76 },
    { 0, 0 },
    { 0xa4a90829, 77 },
    { 0xd1b9082a, 61 },
    { 0xd8d9082b, 46 },
    { 0x1a39082b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b9082f, 93 },
    { 0x8290830, 33 },
    { 0x36090831, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e090837, 67 },
    { 0xbc090838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9083b, 90 },
    { 0x83c9083b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42090843, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da90846, 90 },
    { 0, 0 },
    { 0xe6f90848, 81 },
    { 0, 0 },
    { 0x7bc9084a, 75 },
    { 0xdd59084a, 34 },
    { 0xf369084c, 80 },
    { 0, 0 },
    { 0x1cf9084e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91890851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71990855, 23 },
    { 0x1e190856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279085c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f390860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde90863, 61 },
    { 0, 0 },
    { 0xbd990865, 61 },
    { 0xe5790866, 23 },
    { 0xf7a90867, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2189086d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x70690870, 98 },
    { 0xd3790870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa690875, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5790878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2269087b, 91 },
    { 0xc3c9087b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17090880, 81 },
    { 0x60e90880, 85 },
    { 0xf2b90882, 99 },
    { 0x1b490883, 98 },
    { 0x82490883, 90 },
    { 0x23f90880, 34 },
    { 0, 0 },
    { 0xff690887, 95 },
    { 0xf090887, 61 },
    { 0xdfd90889, 33 },
    { 0x25490888, 17 },
    { 0x8879088b, 55 },
    { 0xa2f9088c, 98 },
    { 0, 0 },
    { 0x7459088e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b90899, 95 },
    { 0x36d9089a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9509089e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2908a2, 67 },
    { 0, 0 },
    { 0xc27908a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b908ab, 90 },
    { 0x90908ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e908b2, 76 },
    { 0xc15908b3, 100 },
    { 0xbc5908b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec908b8, 34 },
    { 0xe52908b9, 95 },
    { 0, 0 },
    { 0xc2b908bb, 68 },
    { 0, 0 },
    { 0xd85908bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8908c6, 95 },
    { 0x6b0908c7, 61 },
    { 0x3b908c8, 33 },
    { 0x279908c9, 61 },
    { 0x997908ca, 76 },
    { 0xd06908c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775908cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e908d3, 95 },
    { 0x3ab908d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f908dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e908e0, 65 },
    { 0x532908e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0908e4, 90 },
    { 0x23e908e5, 81 },
    { 0x9ee908e6, 61 },
    { 0x8a7908e7, 90 },
    { 0xf6908e8, 91 },
    { 0xe13908e9, 33 },
    { 0xc66908ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e908f0, 98 },
    { 0x56c908f1, 88 },
    { 0x6a0908f2, 90 },
    { 0x7af908f3, 90 },
    { 0x80e908f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7908ff, 56 },
    { 0x11890900, 66 },
    { 0x35690901, 98 },
    { 0x15a90902, 100 },
    { 0x2e790901, 90 },
    { 0xba190904, 88 },
    { 0xef908ff, 61 },
    { 0x5ef90903, 99 },
    { 0, 0 },
    { 0x48890908, 76 },
    { 0x81790908, 87 },
    { 0x2ed9090a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09090f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11390912, 52 },
    { 0x3d890912, 76 },
    { 0xf3190912, 23 },
    { 0x55e90915, 33 },
    { 0, 0 },
    { 0x81790917, 95 },
    { 0xe1e90918, 76 },
    { 0xe8890919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90990920, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e890927, 95 },
    { 0xd6290927, 43 },
    { 0, 0 },
    { 0x9d09092a, 76 },
    { 0xc839092b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b90930, 68 },
    { 0x5c690930, 33 },
    { 0, 0 },
    { 0x4a090933, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b990937, 43 },
    { 0xe0090938, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca9093d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xff190940, 23 },
    { 0xfe890941, 98 },
    { 0x40890942, 33 },
    { 0x63390943, 33 },
    { 0, 0 },
    { 0x25990945, 23 },
    { 0xca190946, 95 },
    { 0, 0 },
    { 0x5090948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9094c, 33 },
    { 0xd119094d, 34 },
    { 0xc1d9094e, 33 },
    { 0xf5f9094f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa490954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e790959, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c9095d, 23 },
    { 0x8519095d, 95 },
    { 0x2289095f, 23 },
    { 0xb179095f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9290963, 34 },
    { 0xae190964, 100 },
    { 0x20290965, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9096b, 23 },
    { 0xa609096b, 52 },
    { 0x9be9096d, 88 },
    { 0, 0 },
    { 0x8f19096f, 51 },
    { 0xd909096f, 90 },
    { 0xa3390971, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce90976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b39097b, 68 },
    { 0xcf29097c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd9097f, 85 },
    { 0x80490980, 98 },
    { 0x39b90981, 98 },
    { 0xcb490982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca90985, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85090989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669098c, 93 },
    { 0xaf79098d, 23 },
    { 0x8cc9098e, 90 },
    { 0x1a99098f, 33 },
    { 0x57a9098e, 85 },
    { 0x6829098f, 64 },
    { 0x9ae9098f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa249099d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86909a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2909a7, 33 },
    { 0x1d2909a7, 46 },
    { 0x4a5909a9, 23 },
    { 0x5a9909aa, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d909ad, 100 },
    { 0x4a4909ae, 67 },
    { 0x241909af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6909b4, 33 },
    { 0, 0 },
    { 0xd8f909b6, 61 },
    { 0xb7b909b6, 76 },
    { 0x5e8909b8, 23 },
    { 0x9c6909b9, 61 },
    { 0xd1909b7, 90 },
    { 0x117909bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62909c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2909c6, 93 },
    { 0x701909c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x229909ca, 77 },
    { 0xbaa909cb, 75 },
    { 0xfe4909ca, 90 },
    { 0x6df909cd, 90 },
    { 0x47f909cd, 52 },
    { 0xe6a909cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df909d3, 90 },
    { 0, 0 },
    { 0xa59909d5, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396909d9, 98 },
    { 0xfb909da, 93 },
    { 0xb28909da, 61 },
    { 0xfb0909dc, 23 },
    { 0x786909dc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac909e1, 66 },
    { 0x835909e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228909e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4909ec, 90 },
    { 0xf5909ed, 55 },
    { 0x95a909ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd909f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41909f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d909fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90090a02, 23 },
    { 0xb8d90a03, 52 },
    { 0, 0 },
    { 0xe8890a05, 23 },
    { 0x36190a06, 98 },
    { 0x50990a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41590a0b, 85 },
    { 0xb2f90a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f590a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b890a13, 91 },
    { 0x19890a14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2190a17, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f690a1b, 87 },
    { 0xe2890a1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e90a22, 34 },
    { 0x75a90a23, 55 },
    { 0x44b90a24, 95 },
    { 0xbcf90a25, 100 },
    { 0, 0 },
    { 0xf2390a27, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd90a2d, 23 },
    { 0, 0 },
    { 0x61f90a2f, 99 },
    { 0xb6190a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d90a37, 77 },
    { 0xc8090a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda90a42, 78 },
    { 0x10190a43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c790a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38390a4d, 61 },
    { 0x43a90a4e, 90 },
    { 0xf5790a4f, 61 },
    { 0x79d90a50, 90 },
    { 0x68990a50, 76 },
    { 0x6ac90a52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d490a59, 90 },
    { 0xfe290a5a, 76 },
    { 0, 0 },
    { 0x93990a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f90a5f, 61 },
    { 0x3090a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c90a64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76590a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf90a6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab290a71, 93 },
    { 0x6ed90a72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b90a76, 61 },
    { 0, 0 },
    { 0x8f590a78, 23 },
    { 0, 0 },
    { 0xe5490a7a, 61 },
    { 0xc6f90a7b, 33 },
    { 0, 0 },
    { 0x86c90a7d, 49 },
    { 0xe7290a7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba90a81, 88 },
    { 0x7f90a81, 61 },
    { 0xf6990a83, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69490a87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70390a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd90a91, 90 },
    { 0xd5190a92, 33 },
    { 0x82b90a93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c90a96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd90a99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74690a9c, 23 },
    { 0xda890a9d, 100 },
    { 0x6dc90a9d, 90 },
    { 0xf7b90a9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d690aa2, 34 },
    { 0xeb690aa2, 90 },
    { 0x71d90aa4, 56 },
    { 0xc7a90aa2, 95 },
    { 0, 0 },
    { 0x4ac90aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a90aab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d790aaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf90ab9, 90 },
    { 0x66290aba, 77 },
    { 0, 0 },
    { 0x89f90abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e90ac0, 43 },
    { 0x18c90ac1, 46 },
    { 0, 0 },
    { 0x5b190ac3, 90 },
    { 0xb1690ac4, 78 },
    { 0x80a90ac5, 98 },
    { 0xb9390ac5, 33 },
    { 0, 0 },
    { 0xd2290ac8, 19 },
    { 0xe3590ac9, 23 },
    { 0x82990ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da90ad0, 33 },
    { 0x57e90ad1, 76 },
    { 0xb0390ad1, 95 },
    { 0xf090ad3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d390ad6, 61 },
    { 0x6990ad7, 100 },
    { 0x8d890ad6, 23 },
    { 0x90290ad8, 33 },
    { 0xf0c90ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab90ade, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a490ae7, 51 },
    { 0x90a90ae8, 61 },
    { 0x8ba90ae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690af1, 90 },
    { 0xc8390af1, 76 },
    { 0xf2090af1, 90 },
    { 0x1fb90af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10790af7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec90afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43490b01, 46 },
    { 0x29990b02, 34 },
    { 0x9090b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb490b07, 33 },
    { 0xcf890b08, 23 },
    { 0x4d190b09, 95 },
    { 0xc3a90b0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec90b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90390b14, 43 },
    { 0, 0 },
    { 0x43c90b16, 61 },
    { 0, 0 },
    { 0xee290b18, 68 },
    { 0x47390b19, 91 },
    { 0x5290b19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a990b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d790b2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf090b33, 90 },
    { 0, 0 },
    { 0x8df90b35, 43 },
    { 0x59090b36, 93 },
    { 0xf6b90b37, 90 },
    { 0xe1590b38, 34 },
    { 0x15790b38, 68 },
    { 0xc6c90b38, 67 },
    { 0x56490b39, 95 },
    { 0x31790b37, 61 },
    { 0, 0 },
    { 0xf5590b3e, 23 },
    { 0xb2190b3f, 82 },
    { 0, 0 },
    { 0x4b990b41, 23 },
    { 0x77590b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b90b47, 34 },
    { 0x25c90b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb90b4b, 33 },
    { 0, 0 },
    { 0xa4b90b4d, 90 },
    { 0x8de90b4e, 52 },
    { 0x78f90b4e, 23 },
    { 0x1bb90b50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d90b54, 98 },
    { 0, 0 },
    { 0x14290b56, 61 },
    { 0x44f90b56, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b90b60, 33 },
    { 0xd1a90b61, 34 },
    { 0x73590b61, 33 },
    { 0, 0 },
    { 0xd4590b64, 90 },
    { 0, 0 },
    { 0xa1590b66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d90b6b, 87 },
    { 0x27090b6c, 17 },
    { 0x61590b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25590b72, 90 },
    { 0xca990b73, 33 },
    { 0xddb90b72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb390b7f, 90 },
    { 0xcec90b80, 55 },
    { 0, 0 },
    { 0x85790b82, 90 },
    { 0, 0 },
    { 0x28790b84, 90 },
    { 0, 0 },
    { 0x36f90b86, 23 },
    { 0x32990b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa90b93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b90b96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6890b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890ba3, 84 },
    { 0, 0 },
    { 0xbd790ba5, 55 },
    { 0, 0 },
    { 0xe1e90ba7, 23 },
    { 0xdbe90ba7, 61 },
    { 0xccd90ba9, 46 },
    { 0x50b90baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15990bb0, 34 },
    { 0x6fd90bb1, 61 },
    { 0, 0 },
    { 0x7e190bb3, 95 },
    { 0x44890bb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d390bb7, 33 },
    { 0x63090bb8, 33 },
    { 0, 0 },
    { 0x8b90bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa90bbe, 76 },
    { 0xf9490bbf, 95 },
    { 0, 0 },
    { 0x59090bc1, 90 },
    { 0x6f790bc2, 55 },
    { 0x82890bc3, 61 },
    { 0x94090bc3, 55 },
    { 0x45a90bc4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f90bc8, 23 },
    { 0xef890bc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3190bcc, 76 },
    { 0x590bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad90bd0, 23 },
    { 0xbb690bd1, 23 },
    { 0, 0 },
    { 0x10090bd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51990be0, 55 },
    { 0xfa790be1, 61 },
    { 0xc6a90be2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e90be8, 95 },
    { 0xc1090be9, 33 },
    { 0x44c90bea, 100 },
    { 0x23790beb, 90 },
    { 0x7aa90bec, 76 },
    { 0x36090bea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee190bf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b90bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c90bff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d90c02, 46 },
    { 0, 0 },
    { 0x43c90c04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e90c08, 76 },
    { 0, 0 },
    { 0xa2490c0a, 91 },
    { 0xc7e90c0b, 17 },
    { 0x3a90c0a, 55 },
    { 0x75390c0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a090c11, 61 },
    { 0xde190c12, 101 },
    { 0xe390c13, 43 },
    { 0x1d090c14, 88 },
    { 0x4f90c15, 23 },
    { 0xb5090c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090c19, 80 },
    { 0x60b90c1a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e90c1d, 96 },
    { 0, 0 },
    { 0x7a190c1f, 17 },
    { 0, 0 },
    { 0x9490c21, 76 },
    { 0x58190c22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a090c26, 90 },
    { 0x5eb90c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f90c2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc90c2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa190c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9590c39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f90c3d, 55 },
    { 0x33e90c3e, 33 },
    { 0xb6d90c3f, 46 },
    { 0x54190c3f, 23 },
    { 0x45190c3e, 95 },
    { 0x5bc90c3e, 81 },
    { 0x4a190c41, 61 },
    { 0x2c590c44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59990c48, 52 },
    { 0x3d790c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14590c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97490c50, 88 },
    { 0, 0 },
    { 0x3bb90c52, 90 },
    { 0xd1b90c52, 67 },
    { 0x61b90c54, 77 },
    { 0xfd390c52, 76 },
    { 0x95790c56, 43 },
    { 0xea190c57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14890c5d, 17 },
    { 0, 0 },
    { 0xc6090c5f, 88 },
    { 0xb7990c60, 33 },
    { 0x4a690c60, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae790c65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26190c69, 87 },
    { 0, 0 },
    { 0xa8390c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce690c71, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b90c74, 95 },
    { 0x9f90c75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3990c78, 33 },
    { 0xb4290c79, 90 },
    { 0x6bd90c7a, 61 },
    { 0x66b90c7b, 95 },
    { 0xfd790c7c, 95 },
    { 0x4a90c7d, 90 },
    { 0x21e90c7c, 99 },
    { 0x7d590c7f, 90 },
    { 0, 0 },
    { 0x5890c81, 61 },
    { 0xc0690c82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x92790c85, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14690c89, 61 },
    { 0x15790c8a, 76 },
    { 0xd190c89, 23 },
    { 0xda490c8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4490c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6790c97, 90 },
    { 0, 0 },
    { 0x84190c99, 33 },
    { 0, 0 },
    { 0xd9390c9b, 81 },
    { 0x92190c9c, 85 },
    { 0x57a90c9d, 67 },
    { 0x2b990c9d, 43 },
    { 0x6d790c9e, 61 },
    { 0x89490ca0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba090ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5690cab, 61 },
    { 0xb1f90cac, 61 },
    { 0, 0 },
    { 0x32f90cae, 33 },
    { 0x98690caf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5690cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8890cba, 85 },
    { 0, 0 },
    { 0xe9790cbc, 61 },
    { 0x19c90cbd, 90 },
    { 0, 0 },
    { 0xb2290cbf, 90 },
    { 0xf3390cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b990cc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c90cc9, 90 },
    { 0, 0 },
    { 0x5cf90ccb, 100 },
    { 0xd2090ccb, 23 },
    { 0x3d290ccb, 93 },
    { 0x41f90ccc, 96 },
    { 0x11c90ccd, 93 },
    { 0x55190cd0, 55 },
    { 0x790cd1, 33 },
    { 0xa090cd2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb590cd5, 33 },
    { 0x55f90cd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa290cdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5490cde, 88 },
    { 0x26990cdf, 95 },
    { 0xfa590ce0, 91 },
    { 0x80f90ce1, 55 },
    { 0xe0390ce2, 53 },
    { 0xe6590ce2, 98 },
    { 0x81b90ce2, 33 },
    { 0x76990ce5, 33 },
    { 0xce190ce6, 88 },
    { 0x1ef90ce7, 46 },
    { 0x9f590ce8, 76 },
    { 0x18690ce9, 77 },
    { 0xa1d90cdf, 9 },
    { 0xb6790ce4, 33 },
    { 0x3ac90cec, 61 },
    { 0x65a90ce5, 95 },
    { 0x7cb90cea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990cf1, 95 },
    { 0, 0 },
    { 0xf4690cf3, 90 },
    { 0, 0 },
    { 0x19390cf5, 77 },
    { 0x56b90cf6, 96 },
    { 0, 0 },
    { 0x5090cf8, 75 },
    { 0, 0 },
    { 0xb0b90cfa, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x61990cfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4590d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d90d07, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c90d0b, 98 },
    { 0x3ff90d0b, 17 },
    { 0x2a790d0d, 78 },
    { 0x50490d0c, 95 },
    { 0x2ed90d0f, 34 },
    { 0x73490d10, 61 },
    { 0x2b190d0f, 95 },
    { 0x34f90d0e, 33 },
    { 0xe0890d13, 33 },
    { 0xf6b90d13, 61 },
    { 0x4fe90d15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7790d19, 98 },
    { 0, 0 },
    { 0xdde90d1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e590d1e, 23 },
    { 0x50090d1f, 61 },
    { 0, 0 },
    { 0xbf190d21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af90d27, 76 },
    { 0xe5f90d28, 95 },
    { 0x15a90d29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c90d2c, 61 },
    { 0xd4590d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d290d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80890d38, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xee090d3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d90d41, 33 },
    { 0x3d990d42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb690d46, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b490d4b, 23 },
    { 0xa4690d4b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1690d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c890d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a90d58, 34 },
    { 0x6790d59, 23 },
    { 0x47790d58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1590d62, 76 },
    { 0xbba90d63, 100 },
    { 0, 0 },
    { 0x9c490d65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb490d70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58490d74, 90 },
    { 0x67490d75, 90 },
    { 0x7c390d76, 90 },
    { 0xf1690d77, 99 },
    { 0, 0 },
    { 0x2e390d79, 49 },
    { 0, 0 },
    { 0xf1690d7b, 98 },
    { 0x2cc90d7c, 23 },
    { 0xdbc90d7d, 95 },
    { 0xee490d7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e90d81, 23 },
    { 0xa5690d82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b990d85, 34 },
    { 0x64190d85, 98 },
    { 0, 0 },
    { 0x77390d88, 46 },
    { 0xec390d88, 95 },
    { 0, 0 },
    { 0xaf290d8b, 88 },
    { 0xa5290d8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb90d90, 97 },
    { 0xa7b90d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29490d94, 90 },
    { 0, 0 },
    { 0xcf890d96, 61 },
    { 0x8fa90d97, 85 },
    { 0, 0 },
    { 0xd2390d99, 90 },
    { 0xd5990d9a, 76 },
    { 0, 0 },
    { 0xddb90d9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57990da2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a90da5, 85 },
    { 0x98590da6, 23 },
    { 0x77290da7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5490dac, 34 },
    { 0xfaf90dad, 23 },
    { 0x72390dae, 52 },
    { 0xbd090daf, 61 },
    { 0x2f890daf, 93 },
    { 0xc8390db1, 40 },
    { 0x6bf90db1, 34 },
    { 0x5fb90db2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee90db6, 23 },
    { 0xb190db6, 98 },
    { 0xf6190db8, 84 },
    { 0x10a90db7, 33 },
    { 0xa4490db8, 33 },
    { 0xd2c90dbb, 100 },
    { 0xf0f90dbc, 23 },
    { 0x5f490dbd, 76 },
    { 0xbeb90db7, 95 },
    { 0x69190dbf, 98 },
    { 0x5890dc0, 87 },
    { 0x5c090dc0, 98 },
    { 0xcbb90dc2, 33 },
    { 0xd8b90dc2, 68 },
    { 0x3f790dc0, 66 },
    { 0xe5e90dc2, 55 },
    { 0xe8390dc6, 84 },
    { 0x5890dbc, 59 },
    { 0x28b90dc8, 34 },
    { 0x8a790dc9, 23 },
    { 0xdac90dc8, 75 },
    { 0xa2b90db8, 55 },
    { 0x87090dc6, 95 },
    { 0xcc090dc8, 90 },
    { 0x30c90dce, 34 },
    { 0xf5690dcf, 46 },
    { 0x88590dc0, 55 },
    { 0xd3190dd1, 23 },
    { 0, 0 },
    { 0xe1590dd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d90dd8, 76 },
    { 0xd9f90dd9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd90dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f90de1, 23 },
    { 0xc6790de2, 90 },
    { 0x78c90de3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2290de6, 87 },
    { 0x4be90de7, 43 },
    { 0x22390de8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a90deb, 81 },
    { 0x23990dec, 66 },
    { 0xdc090dec, 99 },
    { 0x17d90dee, 90 },
    { 0x91e90def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a490df6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79990dfc, 76 },
    { 0x23290dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb90e07, 93 },
    { 0x44090e08, 98 },
    { 0, 0 },
    { 0x8ea90e0a, 81 },
    { 0x67790e0b, 46 },
    { 0xca690e0c, 48 },
    { 0, 0 },
    { 0x7ed90e0e, 85 },
    { 0xbd290e0f, 90 },
    { 0, 0 },
    { 0xa8e90e11, 76 },
    { 0x36f90e12, 23 },
    { 0xba790e12, 76 },
    { 0x55b90e14, 98 },
    { 0x4ab90e13, 84 },
    { 0x21490e12, 53 },
    { 0, 0 },
    { 0x78190e18, 67 },
    { 0x3e90e19, 95 },
    { 0x92290e1a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8890e1e, 64 },
    { 0xc7e90e1e, 61 },
    { 0x89290e20, 34 },
    { 0x45d90e1e, 33 },
    { 0, 0 },
    { 0x37b90e23, 95 },
    { 0x14a90e24, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b990e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7590e2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f90e30, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c90e37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d90e3a, 61 },
    { 0x13390e3b, 96 },
    { 0x3c990e3c, 95 },
    { 0x90e3c, 98 },
    { 0xa3890e3d, 84 },
    { 0x74790e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a90e45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5090e4d, 76 },
    { 0xc2290e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81590e52, 68 },
    { 0, 0 },
    { 0x4dd90e54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51990e59, 98 },
    { 0, 0 },
    { 0x92190e5b, 33 },
    { 0, 0 },
    { 0x35d90e5d, 61 },
    { 0x59790e5e, 88 },
    { 0x78690e5d, 77 },
    { 0x20090e5e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd390e63, 77 },
    { 0xa2790e63, 33 },
    { 0x91990e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b990e69, 98 },
    { 0xb4590e6a, 23 },
    { 0xe3b90e6b, 33 },
    { 0x6b190e6a, 40 },
    { 0x3d090e6c, 24 },
    { 0, 0 },
    { 0xc2c90e6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a090e72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa90e75, 90 },
    { 0xec790e76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f490e7a, 98 },
    { 0x87a90e7b, 93 },
    { 0, 0 },
    { 0x64290e7d, 53 },
    { 0x80c90e7e, 90 },
    { 0x60290e7f, 100 },
    { 0xa90e7d, 61 },
    { 0x1ce90e81, 23 },
    { 0x3da90e82, 76 },
    { 0x75890e7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50490e87, 75 },
    { 0x9e090e88, 43 },
    { 0xcec90e88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92790e8d, 34 },
    { 0, 0 },
    { 0x2f690e8f, 93 },
    { 0xad890e90, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2290e93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a90e97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea090e9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5290ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a90ea4, 43 },
    { 0x76890ea4, 95 },
    { 0, 0 },
    { 0x8090ea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e790eb0, 66 },
    { 0x4f690eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde90eba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4790ec0, 90 },
    { 0, 0 },
    { 0x14d90ec2, 90 },
    { 0, 0 },
    { 0xbd290ec4, 55 },
    { 0xac590ec5, 98 },
    { 0x25590ec6, 98 },
    { 0xe0d90ec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d90eca, 76 },
    { 0x4f990eca, 84 },
    { 0xae190ecc, 91 },
    { 0x12990ecb, 33 },
    { 0, 0 },
    { 0x1b890ecf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76490ed8, 61 },
    { 0xb0290ed9, 77 },
    { 0, 0 },
    { 0xf9d90edb, 43 },
    { 0, 0 },
    { 0x95190edd, 95 },
    { 0xa0b90ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf290ee5, 98 },
    { 0x7fa90ee6, 78 },
    { 0, 0 },
    { 0x34390ee8, 87 },
    { 0x7f690ee9, 61 },
    { 0xfad90ee8, 43 },
    { 0, 0 },
    { 0x5c290eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7690eef, 34 },
    { 0x38c90ef0, 90 },
    { 0x3b690ef1, 95 },
    { 0x86e90ef2, 95 },
    { 0xd6590ef2, 64 },
    { 0x41990ef4, 91 },
    { 0x77290ef4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49090efa, 85 },
    { 0xc1290efb, 76 },
    { 0, 0 },
    { 0x32190efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9190f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd90f04, 33 },
    { 0x5a090f05, 98 },
    { 0x4bf90f06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x48290f09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90f14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe90f19, 75 },
    { 0xd3a90f19, 33 },
    { 0x45790f1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8390f1f, 90 },
    { 0x62490f1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf490f24, 33 },
    { 0x40d90f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad90f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f90f37, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c90f3b, 90 },
    { 0x57290f3c, 46 },
    { 0xb3190f3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa890f44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a090f48, 43 },
    { 0x29c90f49, 33 },
    { 0xd2190f4a, 67 },
    { 0xe3b90f4a, 34 },
    { 0x3ac90f48, 90 },
    { 0x9f790f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde790f50, 33 },
    { 0x4f090f50, 90 },
    { 0xb9290f52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c90f55, 23 },
    { 0x8a090f56, 76 },
    { 0xea790f57, 98 },
    { 0xd7690f58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1590f5c, 76 },
    { 0x2de90f5d, 98 },
    { 0, 0 },
    { 0x49f90f5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d90f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23690f66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c390f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd90f6f, 38 },
    { 0x65090f6f, 61 },
    { 0x96f90f6f, 98 },
    { 0xb9790f72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda90f76, 34 },
    { 0xe6d90f76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df90f7c, 61 },
    { 0, 0 },
    { 0x8390f7e, 23 },
    { 0x94590f7f, 90 },
    { 0x2e290f80, 95 },
    { 0, 0 },
    { 0x60390f82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53290f89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f90f8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51890f91, 76 },
    { 0, 0 },
    { 0xeb990f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46390f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd90f9a, 43 },
    { 0, 0 },
    { 0xffb90f9c, 91 },
    { 0xa1390f9c, 91 },
    { 0, 0 },
    { 0x60c90f9f, 93 },
    { 0, 0 },
    { 0xa6c90fa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2190fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76490fb0, 55 },
    { 0xc4c90fb1, 98 },
    { 0x5ad90fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e90fb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4190fbe, 61 },
    { 0x11590fbe, 55 },
    { 0xcfc90fc0, 98 },
    { 0x65990fc0, 97 },
    { 0x6c190fc2, 40 },
    { 0, 0 },
    { 0x70690fc4, 90 },
    { 0, 0 },
    { 0xbdc90fc6, 23 },
    { 0, 0 },
    { 0x67b90fc8, 78 },
    { 0x9f590fc9, 34 },
    { 0x9c990fca, 90 },
    { 0x14190fcb, 33 },
    { 0x93d90fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b990fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f690fd3, 76 },
    { 0xedb90fd4, 68 },
    { 0xc0190fd4, 90 },
    { 0x3ec90fd5, 61 },
    { 0xb5090fd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e990fda, 52 },
    { 0, 0 },
    { 0xaf390fdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a90fdf, 95 },
    { 0x9f090fe0, 95 },
    { 0xd4c90fe0, 90 },
    { 0x6a90fe0, 34 },
    { 0x1a890fe3, 90 },
    { 0x32190fe3, 33 },
    { 0x2bc90fe5, 61 },
    { 0xb8a90fe6, 76 },
    { 0xfce90fe4, 61 },
    { 0x8a90fe8, 95 },
    { 0xd5a90fe8, 90 },
    { 0xb7590fea, 87 },
    { 0xbd490fdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b90fee, 78 },
    { 0x20990fef, 23 },
    { 0x86d90fee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16790ff3, 90 },
    { 0x6690ff4, 87 },
    { 0, 0 },
    { 0xa6790ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41990ffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f190fff, 61 },
    { 0, 0 },
    { 0x66091001, 95 },
    { 0, 0 },
    { 0xbf591003, 61 },
    { 0xbd891003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b91008, 23 },
    { 0, 0 },
    { 0xedb9100a, 68 },
    { 0x7f49100a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f991014, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189101b, 55 },
    { 0x5279101c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb99101f, 87 },
    { 0x2d491020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2891024, 93 },
    { 0xae891025, 23 },
    { 0x3d491024, 34 },
    { 0xccc91027, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a9102e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5991033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb91037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe769103c, 96 },
    { 0xd209103d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7191040, 88 },
    { 0x20191041, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94191045, 98 },
    { 0x57a91046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1519104b, 99 },
    { 0, 0 },
    { 0xf039104d, 33 },
    { 0, 0 },
    { 0x74b9104f, 33 },
    { 0x2491050, 34 },
    { 0x35691051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1991058, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4191062, 33 },
    { 0, 0 },
    { 0x21891064, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74191068, 61 },
    { 0x84291069, 55 },
    { 0x4539106a, 81 },
    { 0, 0 },
    { 0x48e9106c, 95 },
    { 0xca9106d, 98 },
    { 0x8aa9106c, 90 },
    { 0x3039106f, 95 },
    { 0x2e9106d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde91076, 99 },
    { 0x8fb91076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e69107d, 77 },
    { 0xd249107e, 23 },
    { 0xd99107e, 90 },
    { 0x72f91080, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd91083, 23 },
    { 0xd5c91083, 23 },
    { 0, 0 },
    { 0xe4f91086, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9108a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe91090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf191096, 55 },
    { 0, 0 },
    { 0xdc891098, 33 },
    { 0x99591099, 61 },
    { 0, 0 },
    { 0xa779109b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9109e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682910a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee910a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60910ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2910af, 91 },
    { 0xca5910af, 76 },
    { 0x85d910b1, 90 },
    { 0xc67910b2, 98 },
    { 0x9f2910b1, 98 },
    { 0, 0 },
    { 0x96c910b5, 76 },
    { 0x60e910b6, 40 },
    { 0, 0 },
    { 0xf59910b8, 61 },
    { 0, 0 },
    { 0xff2910ba, 98 },
    { 0xd00910bb, 33 },
    { 0, 0 },
    { 0x90910bd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4910c0, 85 },
    { 0, 0 },
    { 0x6c7910c2, 90 },
    { 0, 0 },
    { 0xcd5910c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71910c9, 75 },
    { 0, 0 },
    { 0x3e910cb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x618910ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9910d3, 76 },
    { 0, 0 },
    { 0x6a4910d5, 33 },
    { 0, 0 },
    { 0x996910d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48910da, 46 },
    { 0x5cb910db, 90 },
    { 0x3bd910da, 93 },
    { 0x834910dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9910e1, 55 },
    { 0xb0e910e2, 61 },
    { 0x980910e3, 78 },
    { 0x715910e2, 34 },
    { 0x2a8910e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f910ee, 55 },
    { 0x884910ef, 98 },
    { 0x762910f0, 95 },
    { 0x49f910f0, 90 },
    { 0x660910f2, 23 },
    { 0x6a7910f3, 46 },
    { 0, 0 },
    { 0x9c1910f5, 90 },
    { 0, 0 },
    { 0xe9a910f7, 95 },
    { 0x519910f8, 90 },
    { 0x1e2910f9, 95 },
    { 0x211910fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6910fd, 95 },
    { 0x1b5910fe, 61 },
    { 0, 0 },
    { 0x9c391100, 77 },
    { 0x2d891100, 84 },
    { 0, 0 },
    { 0x38591103, 34 },
    { 0x26691104, 78 },
    { 0x26991105, 80 },
    { 0x6ab91106, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9110b, 98 },
    { 0, 0 },
    { 0xb7e9110d, 90 },
    { 0, 0 },
    { 0xb819110f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e91116, 36 },
    { 0xaea91116, 95 },
    { 0x8b891116, 61 },
    { 0x22891118, 61 },
    { 0xfee9111a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9111d, 33 },
    { 0x54b9111e, 46 },
    { 0xb2e9111f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb91123, 92 },
    { 0x1a291123, 90 },
    { 0xc0c91123, 91 },
    { 0x58591126, 98 },
    { 0, 0 },
    { 0x3a991128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c9112b, 61 },
    { 0xf7d9112c, 61 },
    { 0x5019112d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26991133, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b29113c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee91142, 90 },
    { 0, 0 },
    { 0x58291144, 76 },
    { 0xe4a91145, 75 },
    { 0xfd691146, 95 },
    { 0xd4f91147, 36 },
    { 0x15b91148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319114b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f791154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9791157, 33 },
    { 0, 0 },
    { 0x43e91159, 23 },
    { 0, 0 },
    { 0xc59115b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f69115f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91163, 87 },
    { 0, 0 },
    { 0x8fa91165, 87 },
    { 0, 0 },
    { 0xf0591167, 93 },
    { 0, 0 },
    { 0xe4691169, 23 },
    { 0, 0 },
    { 0x9359116b, 95 },
    { 0xe929116c, 33 },
    { 0, 0 },
    { 0x9b69116e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8191171, 90 },
    { 0x20691171, 95 },
    { 0, 0 },
    { 0xdf991174, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c9117c, 33 },
    { 0x9029117d, 90 },
    { 0xf279117e, 65 },
    { 0x2fb9117f, 55 },
    { 0x1ce91180, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13091184, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e591188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec19118e, 23 },
    { 0, 0 },
    { 0x58191190, 95 },
    { 0, 0 },
    { 0xb7291192, 94 },
    { 0xff291193, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b91199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b9119c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae911a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc911a5, 90 },
    { 0x98f911a5, 56 },
    { 0, 0 },
    { 0x3ae911a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7911ab, 90 },
    { 0xec6911ac, 33 },
    { 0, 0 },
    { 0xab8911ae, 55 },
    { 0x963911af, 61 },
    { 0xb72911b0, 99 },
    { 0xed1911b1, 61 },
    { 0xfa8911b1, 95 },
    { 0, 0 },
    { 0x59d911b4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6911b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89911bc, 98 },
    { 0, 0 },
    { 0x1ae911be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec911c8, 23 },
    { 0xe0b911c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f911ce, 23 },
    { 0, 0 },
    { 0xa87911d0, 98 },
    { 0xb63911d0, 23 },
    { 0x723911d1, 98 },
    { 0, 0 },
    { 0x1b5911d4, 90 },
    { 0xbc9911d5, 93 },
    { 0x445911d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed911d9, 90 },
    { 0, 0 },
    { 0x52a911db, 43 },
    { 0, 0 },
    { 0x969911dd, 90 },
    { 0xc1a911de, 90 },
    { 0xaad911df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7911e2, 23 },
    { 0x374911e3, 94 },
    { 0, 0 },
    { 0xa4f911e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5911e8, 98 },
    { 0xff3911e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0911ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3911ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c911f3, 61 },
    { 0x952911f4, 90 },
    { 0x84911f3, 26 },
    { 0xbed911f5, 55 },
    { 0x385911f7, 95 },
    { 0xecc911f8, 55 },
    { 0xb70911f9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b91201, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba91207, 90 },
    { 0x8c191208, 87 },
    { 0, 0 },
    { 0xf19120a, 90 },
    { 0x45c9120b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a91211, 90 },
    { 0x5e091212, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9191216, 43 },
    { 0x5f991217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9121a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19121f, 46 },
    { 0x62a9121f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac91226, 32 },
    { 0x70b91227, 65 },
    { 0xc9991227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62891231, 90 },
    { 0x79591231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f591235, 84 },
    { 0x96391236, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79123d, 84 },
    { 0x6909123e, 23 },
    { 0, 0 },
    { 0x34b91240, 91 },
    { 0, 0 },
    { 0x80291242, 78 },
    { 0, 0 },
    { 0x69591244, 96 },
    { 0x9aa91244, 99 },
    { 0xdc291246, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb199124a, 61 },
    { 0xbb29124a, 76 },
    { 0, 0 },
    { 0x9799124d, 84 },
    { 0x33b9124d, 95 },
    { 0x85d9124f, 81 },
    { 0xe9691250, 23 },
    { 0xb129124f, 23 },
    { 0x8a591252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a91259, 90 },
    { 0x2af9125a, 95 },
    { 0x7b09125a, 52 },
    { 0x3629125c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19191262, 98 },
    { 0, 0 },
    { 0xf8791264, 90 },
    { 0, 0 },
    { 0x15f91266, 33 },
    { 0x91d91267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf579126c, 90 },
    { 0xb039126c, 34 },
    { 0xe3f9126c, 61 },
    { 0, 0 },
    { 0xa4391270, 33 },
    { 0xa5991271, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8891275, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca59127d, 85 },
    { 0x6569127e, 34 },
    { 0x7359127e, 90 },
    { 0, 0 },
    { 0x38891281, 95 },
    { 0x46891282, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70491289, 98 },
    { 0xa3e9128a, 46 },
    { 0x9fb9128b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969128f, 61 },
    { 0x2fe91290, 98 },
    { 0xe4391291, 90 },
    { 0x72991292, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b391297, 77 },
    { 0xff891297, 98 },
    { 0x80591299, 76 },
    { 0x89191299, 95 },
    { 0x3529129a, 100 },
    { 0xdac9129c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568912a3, 98 },
    { 0, 0 },
    { 0xd6b912a5, 33 },
    { 0, 0 },
    { 0xea8912a7, 23 },
    { 0xb52912a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab912ab, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772912b0, 100 },
    { 0, 0 },
    { 0xcb0912b2, 34 },
    { 0x2d5912b2, 46 },
    { 0xc6b912b4, 98 },
    { 0x37d912b4, 65 },
    { 0x7a9912b3, 36 },
    { 0xbed912b3, 65 },
    { 0x11d912b3, 95 },
    { 0xc63912b6, 61 },
    { 0xca4912ba, 61 },
    { 0x977912bb, 98 },
    { 0, 0 },
    { 0x32b912bd, 76 },
    { 0, 0 },
    { 0x951912bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa912c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd912c6, 90 },
    { 0, 0 },
    { 0x9b8912c8, 65 },
    { 0x7e4912c9, 95 },
    { 0x20b912c9, 33 },
    { 0, 0 },
    { 0x92a912cc, 95 },
    { 0, 0 },
    { 0x939912ce, 90 },
    { 0xd3f912cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2912d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50912d6, 75 },
    { 0x58b912d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f912da, 61 },
    { 0, 0 },
    { 0xa40912dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb912e1, 23 },
    { 0xc42912e1, 61 },
    { 0x288912e3, 98 },
    { 0x5e6912e3, 85 },
    { 0x7e1912e2, 55 },
    { 0, 0 },
    { 0x74e912e7, 99 },
    { 0, 0 },
    { 0x370912e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc912f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148912fa, 23 },
    { 0, 0 },
    { 0x69c912fc, 95 },
    { 0x643912fd, 61 },
    { 0xe21912fe, 87 },
    { 0x68e912ff, 34 },
    { 0x66e912fe, 90 },
    { 0x740912fe, 40 },
    { 0xbb191302, 66 },
    { 0x5dc91302, 98 },
    { 0xe4391304, 33 },
    { 0, 0 },
    { 0x12c91306, 90 },
    { 0x9ef91306, 23 },
    { 0, 0 },
    { 0x9c591309, 23 },
    { 0xf79130a, 98 },
    { 0x4259130b, 97 },
    { 0xe219130b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb791315, 100 },
    { 0, 0 },
    { 0x84391317, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60591322, 73 },
    { 0xa5391323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc491327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029132d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e191332, 90 },
    { 0x43291333, 99 },
    { 0x19e91334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f91339, 95 },
    { 0x7291339, 95 },
    { 0, 0 },
    { 0x4e89133c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a491340, 87 },
    { 0x43391341, 61 },
    { 0xf3391341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa339134b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec91350, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe39135b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e91364, 98 },
    { 0xe0591364, 98 },
    { 0xa1191366, 38 },
    { 0xe8591367, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc91370, 53 },
    { 0x9891371, 33 },
    { 0x24991372, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce091375, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979137a, 88 },
    { 0x12b9137b, 84 },
    { 0xcfb9137c, 98 },
    { 0xe859137c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a91383, 52 },
    { 0x23791384, 55 },
    { 0x80591385, 61 },
    { 0x5f891386, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c91389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e9138c, 33 },
    { 0x2ed9138c, 55 },
    { 0, 0 },
    { 0x5f99138f, 61 },
    { 0xe759138f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a191393, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc91397, 67 },
    { 0xa9e91398, 90 },
    { 0, 0 },
    { 0xbc9139a, 98 },
    { 0x2ac9139b, 87 },
    { 0xb4a9139b, 98 },
    { 0x2839139d, 95 },
    { 0xc639139a, 95 },
    { 0, 0 },
    { 0xfa5913a0, 84 },
    { 0x9a7913a1, 81 },
    { 0xc7a913a1, 46 },
    { 0xf48913a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278913a7, 90 },
    { 0x92d913a8, 87 },
    { 0x27e913a7, 87 },
    { 0xfb3913a8, 23 },
    { 0xbc8913ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167913b0, 67 },
    { 0, 0 },
    { 0x8ae913b2, 33 },
    { 0xbb1913b3, 23 },
    { 0x891913b4, 61 },
    { 0xdcb913b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191913b9, 77 },
    { 0xf02913ba, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x873913bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3913c0, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b913c4, 55 },
    { 0xbec913c5, 33 },
    { 0xdec913c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9913ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9913d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6913d5, 55 },
    { 0, 0 },
    { 0xc56913d7, 98 },
    { 0, 0 },
    { 0xd97913d9, 98 },
    { 0xfd7913da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb913de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5913e3, 55 },
    { 0, 0 },
    { 0x94913e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c913e9, 23 },
    { 0x100913ea, 66 },
    { 0xb71913eb, 66 },
    { 0xa34913ec, 77 },
    { 0x150913ec, 94 },
    { 0, 0 },
    { 0x944913ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14913f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac913f7, 98 },
    { 0x24d913f7, 99 },
    { 0x3e0913f9, 23 },
    { 0x237913f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4913ff, 61 },
    { 0xc93913ff, 98 },
    { 0x67091400, 95 },
    { 0x457913ff, 52 },
    { 0x1f91403, 43 },
    { 0x9f191400, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x10191407, 95 },
    { 0x49e91408, 98 },
    { 0xa1b91408, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3209140e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e91411, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee91414, 90 },
    { 0x57091415, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b9141b, 90 },
    { 0x7449141c, 43 },
    { 0xdae9141d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x20491420, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d91423, 88 },
    { 0x5fc91424, 99 },
    { 0x8c91424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae91428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d9142e, 61 },
    { 0xd369142e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d91436, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4291439, 98 },
    { 0, 0 },
    { 0x21e9143b, 98 },
    { 0xf219143b, 93 },
    { 0, 0 },
    { 0x3389143e, 90 },
    { 0x78f9143e, 66 },
    { 0xe639143f, 43 },
    { 0x28c91440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d991445, 46 },
    { 0xb7a91446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4899144d, 90 },
    { 0x7519144e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb891457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8949145a, 33 },
    { 0xe9b9145b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb91462, 46 },
    { 0xac491462, 52 },
    { 0x15e91463, 91 },
    { 0x35e91465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29146a, 98 },
    { 0, 0 },
    { 0x1159146c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x599146f, 78 },
    { 0x7eb91470, 43 },
    { 0xbf89146f, 64 },
    { 0, 0 },
    { 0x1a691473, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e591476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a9147b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b291480, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e91485, 91 },
    { 0xc5791485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b91491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7091498, 76 },
    { 0x4a691499, 33 },
    { 0x37191499, 98 },
    { 0xd1a9149b, 90 },
    { 0x1a79149c, 100 },
    { 0, 0 },
    { 0xa9a9149e, 81 },
    { 0x2ec9149f, 98 },
    { 0, 0 },
    { 0x392914a1, 23 },
    { 0x713914a2, 91 },
    { 0x80b914a3, 34 },
    { 0, 0 },
    { 0xf7b914a5, 95 },
    { 0x97e914a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57914ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec914ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220914b4, 23 },
    { 0x9c1914b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7914b8, 34 },
    { 0x87f914b8, 100 },
    { 0, 0 },
    { 0xf30914bb, 61 },
    { 0x56c914bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b914bf, 78 },
    { 0x319914c0, 85 },
    { 0xb49914bf, 61 },
    { 0x627914c2, 65 },
    { 0xb99914c3, 33 },
    { 0x9e6914bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92914c8, 68 },
    { 0xaba914c8, 61 },
    { 0, 0 },
    { 0xb94914cb, 88 },
    { 0, 0 },
    { 0xaa7914cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e914d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9914d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af914dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7914ec, 98 },
    { 0x14e914ec, 33 },
    { 0, 0 },
    { 0xc91914ef, 61 },
    { 0xd1e914f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1914f5, 33 },
    { 0, 0 },
    { 0xe9d914f7, 34 },
    { 0, 0 },
    { 0xeeb914f9, 95 },
    { 0x721914f9, 99 },
    { 0, 0 },
    { 0x48d914fc, 55 },
    { 0xc65914fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc91501, 98 },
    { 0x23491502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c091506, 66 },
    { 0x98491507, 23 },
    { 0x72d91508, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15491510, 101 },
    { 0x65891511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591514, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd91518, 91 },
    { 0xdba91519, 34 },
    { 0xa8591519, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2891523, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9091526, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b791529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef9152e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa91535, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe391539, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f9153c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf859153f, 76 },
    { 0, 0 },
    { 0xbec91541, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7591545, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b91549, 93 },
    { 0x4fd9154a, 90 },
    { 0xf329154a, 36 },
    { 0x33d91549, 90 },
    { 0x9079154b, 98 },
    { 0xe9f9154d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf91556, 55 },
    { 0x49991557, 75 },
    { 0xe991558, 33 },
    { 0x4c191559, 78 },
    { 0xf259155a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c91560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0291566, 19 },
    { 0x96691567, 76 },
    { 0, 0 },
    { 0x2f91569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x299156c, 52 },
    { 0x5f39156d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb91575, 100 },
    { 0x20f91575, 96 },
    { 0x8b391577, 66 },
    { 0x5ae91577, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e9157d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18191585, 33 },
    { 0x33b91586, 87 },
    { 0x1e091587, 33 },
    { 0x82a91588, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9158b, 98 },
    { 0xe449158b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb9158f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9391593, 33 },
    { 0x4f091594, 33 },
    { 0x99091594, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449159b, 85 },
    { 0xab19159b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae915a1, 90 },
    { 0x8e915a2, 46 },
    { 0xf85915a3, 100 },
    { 0, 0 },
    { 0xc86915a5, 98 },
    { 0x9da915a6, 93 },
    { 0x149915a6, 83 },
    { 0x3be915a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332915ac, 68 },
    { 0, 0 },
    { 0x544915ae, 91 },
    { 0, 0 },
    { 0xfbc915b0, 61 },
    { 0x550915b1, 40 },
    { 0x866915b1, 98 },
    { 0x6ab915b3, 95 },
    { 0x5ab915b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9915b9, 88 },
    { 0x4c9915ba, 55 },
    { 0x998915bb, 98 },
    { 0x13a915bb, 23 },
    { 0xbf6915bb, 90 },
    { 0xeb0915bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e915c1, 90 },
    { 0xd90915c2, 45 },
    { 0xb26915c2, 33 },
    { 0xeee915c2, 23 },
    { 0x1fc915c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17915c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41915cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x536915ce, 61 },
    { 0xd92915ce, 98 },
    { 0x986915d0, 34 },
    { 0x57a915d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9915d5, 68 },
    { 0xa81915d5, 33 },
    { 0xd5a915d7, 43 },
    { 0xf5915d8, 90 },
    { 0, 0 },
    { 0x3c9915da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e915e5, 52 },
    { 0xf2a915e5, 87 },
    { 0x765915e5, 90 },
    { 0x6ed915e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662915f2, 67 },
    { 0xfb1915f2, 85 },
    { 0, 0 },
    { 0xf0c915f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08915f8, 61 },
    { 0, 0 },
    { 0xf58915fa, 61 },
    { 0x2c3915fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4491602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61591606, 17 },
    { 0x75691607, 43 },
    { 0xff591608, 84 },
    { 0, 0 },
    { 0x10f9160a, 65 },
    { 0xeee9160b, 53 },
    { 0xa249160c, 65 },
    { 0x6009160d, 33 },
    { 0x59f9160d, 24 },
    { 0x82b9160b, 76 },
    { 0x88891610, 90 },
    { 0x1f791610, 33 },
    { 0, 0 },
    { 0x75e91613, 66 },
    { 0x49b91614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc591618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee9161d, 23 },
    { 0x62f9161e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f91624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809162a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19162e, 66 },
    { 0, 0 },
    { 0x42491630, 98 },
    { 0xd5391631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff91637, 55 },
    { 0x94491638, 69 },
    { 0xf491638, 33 },
    { 0, 0 },
    { 0x3b19163b, 78 },
    { 0xd329163c, 61 },
    { 0x1e29163d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81791642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191647, 84 },
    { 0x88791648, 61 },
    { 0x77d91648, 90 },
    { 0xf639164a, 61 },
    { 0x2a19164b, 98 },
    { 0x1c39164c, 90 },
    { 0xda49164d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf91653, 95 },
    { 0xea991654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e91657, 99 },
    { 0xfe891658, 99 },
    { 0xdd91659, 55 },
    { 0x3c29165a, 78 },
    { 0, 0 },
    { 0x279165c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74991660, 76 },
    { 0x8ef91660, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a49166a, 35 },
    { 0, 0 },
    { 0x9b99166c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3509166f, 76 },
    { 0x77a91670, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1891673, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc191676, 98 },
    { 0, 0 },
    { 0xaab91678, 23 },
    { 0, 0 },
    { 0xbca9167a, 98 },
    { 0, 0 },
    { 0x8cd9167c, 23 },
    { 0x44e9167d, 66 },
    { 0xdab9167d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e91685, 98 },
    { 0xd4c91685, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d79168a, 78 },
    { 0, 0 },
    { 0x40b9168c, 98 },
    { 0xdbc9168c, 98 },
    { 0, 0 },
    { 0xb799168f, 33 },
    { 0x1b891690, 46 },
    { 0x74f91691, 76 },
    { 0xed291692, 33 },
    { 0, 0 },
    { 0xfa691694, 33 },
    { 0x6e291694, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa91698, 43 },
    { 0, 0 },
    { 0x3a19169a, 55 },
    { 0xd989169b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5916a0, 33 },
    { 0x871916a1, 43 },
    { 0xa18916a1, 75 },
    { 0x7da916a3, 33 },
    { 0x506916a4, 98 },
    { 0x251916a5, 97 },
    { 0, 0 },
    { 0x452916a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc916aa, 98 },
    { 0xadf916ab, 42 },
    { 0x5f5916ac, 65 },
    { 0x95c916ad, 98 },
    { 0xc95916ad, 23 },
    { 0xef4916ac, 66 },
    { 0, 0 },
    { 0xe17916b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab916b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a916be, 33 },
    { 0, 0 },
    { 0x1ce916c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568916c6, 76 },
    { 0x8c5916c6, 95 },
    { 0x8ea916c8, 90 },
    { 0xc19916c6, 95 },
    { 0xbef916ca, 85 },
    { 0xe3f916c7, 61 },
    { 0, 0 },
    { 0x6b2916cd, 23 },
    { 0, 0 },
    { 0x8a916cf, 61 },
    { 0x693916d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7916d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e916d6, 55 },
    { 0xf01916d7, 95 },
    { 0, 0 },
    { 0xa8a916d9, 76 },
    { 0, 0 },
    { 0xf67916db, 95 },
    { 0x772916dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7916e2, 34 },
    { 0xcf9916e3, 90 },
    { 0x29916e4, 99 },
    { 0, 0 },
    { 0x181916e6, 33 },
    { 0x65b916e7, 23 },
    { 0x4b8916e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1916eb, 95 },
    { 0x52b916ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34916ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f916f9, 98 },
    { 0xa5f916fa, 33 },
    { 0x735916fb, 23 },
    { 0, 0 },
    { 0xb4e916fd, 84 },
    { 0, 0 },
    { 0xf50916ff, 90 },
    { 0x7ea91700, 61 },
    { 0xb9591700, 95 },
    { 0x70c91702, 87 },
    { 0x9f491703, 46 },
    { 0xbf91704, 55 },
    { 0x7e391704, 90 },
    { 0, 0 },
    { 0xa1591707, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e291717, 33 },
    { 0xbe991718, 40 },
    { 0x73491718, 61 },
    { 0x6a291718, 61 },
    { 0, 0 },
    { 0x11b9171c, 33 },
    { 0x6fc9171d, 98 },
    { 0x58f9171e, 34 },
    { 0xab69171e, 23 },
    { 0x4de9171d, 98 },
    { 0xf7691720, 98 },
    { 0x8e991722, 33 },
    { 0, 0 },
    { 0xa4191724, 33 },
    { 0, 0 },
    { 0x40e91726, 23 },
    { 0x2d391726, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed9172a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a39172e, 84 },
    { 0x98d9172f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc991732, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52491735, 90 },
    { 0x7f391735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d9173e, 46 },
    { 0xe019173f, 33 },
    { 0x2af91740, 43 },
    { 0, 0 },
    { 0xb1791742, 93 },
    { 0, 0 },
    { 0xe3291744, 33 },
    { 0x88291744, 52 },
    { 0xefc91746, 93 },
    { 0, 0 },
    { 0x73491748, 33 },
    { 0, 0 },
    { 0x1e19174a, 98 },
    { 0, 0 },
    { 0x39f9174c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40491756, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47691760, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2891763, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce591769, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e09176e, 91 },
    { 0, 0 },
    { 0x56791770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2491775, 88 },
    { 0x4c391775, 95 },
    { 0xa1791777, 90 },
    { 0, 0 },
    { 0xb8d91779, 90 },
    { 0x8289177a, 98 },
    { 0x42d9177b, 90 },
    { 0x4ef9177b, 93 },
    { 0x9339177d, 33 },
    { 0x9cc9177e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c91782, 33 },
    { 0x9e391782, 76 },
    { 0x18291784, 90 },
    { 0x5bd91782, 90 },
    { 0xdff91786, 33 },
    { 0x4a191786, 65 },
    { 0x41591788, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9178c, 33 },
    { 0x8b9178d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb91794, 23 },
    { 0xea091795, 95 },
    { 0xa4791796, 84 },
    { 0x5c391797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5229179a, 55 },
    { 0, 0 },
    { 0x37c9179c, 34 },
    { 0, 0 },
    { 0x81f9179e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9917a7, 95 },
    { 0x8e3917a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301917ae, 77 },
    { 0, 0 },
    { 0x287917b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc917b6, 66 },
    { 0x207917b7, 98 },
    { 0x410917b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700917bc, 69 },
    { 0, 0 },
    { 0x36a917be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7917c2, 23 },
    { 0xee1917c3, 66 },
    { 0, 0 },
    { 0xbea917c5, 95 },
    { 0xf99917c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36917cc, 61 },
    { 0x252917cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204917d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5917d8, 33 },
    { 0xfc917d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368917e3, 98 },
    { 0, 0 },
    { 0xa08917e5, 42 },
    { 0, 0 },
    { 0xc75917e7, 90 },
    { 0x29f917e7, 99 },
    { 0, 0 },
    { 0xe5b917ea, 61 },
    { 0x2bb917eb, 78 },
    { 0x8f9917ec, 33 },
    { 0x1d2917ea, 33 },
    { 0xf34917ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558917f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91917f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x666917fa, 22 },
    { 0xed3917fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7917fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x22291801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a91805, 95 },
    { 0, 0 },
    { 0xfac91807, 100 },
    { 0x62f91808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb629180b, 55 },
    { 0x7349180c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb91812, 33 },
    { 0xdf291813, 90 },
    { 0, 0 },
    { 0xff191815, 98 },
    { 0x48b91816, 19 },
    { 0, 0 },
    { 0x18b91818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca9181d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091820, 75 },
    { 0, 0 },
    { 0xa9391822, 61 },
    { 0x14091823, 61 },
    { 0x72291822, 76 },
    { 0x8ff91825, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a691829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba9182d, 61 },
    { 0x44a9182e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce91831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca191838, 98 },
    { 0xced91838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789183d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca91843, 95 },
    { 0x76991843, 69 },
    { 0, 0 },
    { 0x14691846, 85 },
    { 0x1fd91847, 61 },
    { 0x8e691848, 19 },
    { 0, 0 },
    { 0x9f09184a, 33 },
    { 0x5be9184b, 61 },
    { 0x52f9184c, 93 },
    { 0x2bb9184c, 90 },
    { 0, 0 },
    { 0xfef9184f, 90 },
    { 0xcdf91850, 61 },
    { 0xf7491850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd91858, 55 },
    { 0x7191859, 78 },
    { 0xba391859, 61 },
    { 0xabf91858, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff9185e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f91861, 33 },
    { 0x5a391862, 23 },
    { 0, 0 },
    { 0xd0691864, 61 },
    { 0x26091865, 98 },
    { 0x54291866, 91 },
    { 0x23b91864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9186a, 87 },
    { 0xc5e9186b, 55 },
    { 0, 0 },
    { 0xe159186d, 98 },
    { 0, 0 },
    { 0x7459186f, 23 },
    { 0x3e891870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca91873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5191877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4769187a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c891880, 23 },
    { 0x23891881, 65 },
    { 0xe2491880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f91885, 52 },
    { 0x74f91886, 17 },
    { 0x57491887, 31 },
    { 0xf6191888, 95 },
    { 0x6d91886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e91890, 76 },
    { 0, 0 },
    { 0x44891892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64491897, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc9189c, 99 },
    { 0x6fd9189c, 95 },
    { 0, 0 },
    { 0xfd79189f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1918a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c918aa, 33 },
    { 0x4bf918ab, 61 },
    { 0xcb9918ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x702918af, 23 },
    { 0x9f0918b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f918b4, 87 },
    { 0x106918b5, 95 },
    { 0xca8918b5, 95 },
    { 0, 0 },
    { 0x414918b8, 98 },
    { 0, 0 },
    { 0x4ae918ba, 77 },
    { 0x7f3918bb, 91 },
    { 0x1c2918ba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0918c0, 33 },
    { 0xf61918c0, 33 },
    { 0x92e918c2, 43 },
    { 0, 0 },
    { 0x5eb918c4, 90 },
    { 0, 0 },
    { 0x503918c6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9918cd, 76 },
    { 0x49918ce, 98 },
    { 0x37a918ce, 76 },
    { 0x12e918d0, 90 },
    { 0, 0 },
    { 0xc9c918d2, 90 },
    { 0x1a4918d3, 95 },
    { 0x743918d4, 78 },
    { 0x56f918d4, 33 },
    { 0, 0 },
    { 0x9ee918d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9918db, 99 },
    { 0, 0 },
    { 0xde918dd, 90 },
    { 0x3b9918de, 33 },
    { 0x88d918df, 93 },
    { 0x7e8918e0, 61 },
    { 0xb86918dd, 90 },
    { 0x170918e2, 90 },
    { 0x710918e2, 98 },
    { 0x8ad918e4, 61 },
    { 0x855918e3, 78 },
    { 0xb63918e6, 46 },
    { 0x1ac918e7, 95 },
    { 0x48b918e8, 100 },
    { 0xdaa918e8, 34 },
    { 0xcfc918e7, 82 },
    { 0x5dd918ea, 78 },
    { 0, 0 },
    { 0x869918ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x84918f0, 85 },
    { 0xf90918f1, 66 },
    { 0xbf1918f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5918f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4918fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb191900, 34 },
    { 0, 0 },
    { 0x49891902, 61 },
    { 0, 0 },
    { 0x9ac91904, 23 },
    { 0x16c91904, 93 },
    { 0, 0 },
    { 0x42a91907, 61 },
    { 0xe7391908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7469190d, 78 },
    { 0x36e9190e, 55 },
    { 0x27a9190f, 90 },
    { 0, 0 },
    { 0x55291911, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09191a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8591920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f791925, 77 },
    { 0xa4d91925, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e9192c, 61 },
    { 0x31f9192d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff491931, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf091936, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9629193a, 90 },
    { 0xd1f9193a, 34 },
    { 0x6a49193b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb9193f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf91942, 17 },
    { 0xb0391942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d91946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa809194b, 87 },
    { 0x319194b, 91 },
    { 0, 0 },
    { 0x2789194e, 87 },
    { 0xf739194e, 98 },
    { 0, 0 },
    { 0xc6791951, 90 },
    { 0x92c91952, 23 },
    { 0, 0 },
    { 0xe6691954, 61 },
    { 0xbcf91955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79195c, 95 },
    { 0xae99195d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58391961, 76 },
    { 0x5b091962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69196b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87791971, 85 },
    { 0x40391972, 85 },
    { 0xaef91973, 90 },
    { 0, 0 },
    { 0x20d91975, 93 },
    { 0xb7c91976, 95 },
    { 0, 0 },
    { 0x23e91978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9197b, 23 },
    { 0xfc39197c, 33 },
    { 0xeef9197d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc91983, 61 },
    { 0x38d91983, 43 },
    { 0x8991985, 23 },
    { 0x51e91986, 95 },
    { 0xee791987, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe839198b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f91990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9391993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7391997, 61 },
    { 0xe9a91997, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9199c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309919a1, 34 },
    { 0xc01919a2, 61 },
    { 0x5ea919a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69919ac, 23 },
    { 0x3db919ad, 95 },
    { 0, 0 },
    { 0xb69919af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54919b9, 83 },
    { 0xf7c919b9, 52 },
    { 0x146919ba, 98 },
    { 0x29f919ba, 61 },
    { 0xf29919ba, 90 },
    { 0x1a4919be, 76 },
    { 0xb8f919bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70919c2, 90 },
    { 0, 0 },
    { 0x51a919c4, 90 },
    { 0xf3a919c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161919c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9919cd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5919d1, 33 },
    { 0x3d9919d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9919d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47919d8, 84 },
    { 0x90d919d9, 61 },
    { 0, 0 },
    { 0x1ce919db, 85 },
    { 0, 0 },
    { 0xcaf919dd, 98 },
    { 0x2f9919dd, 95 },
    { 0xaee919df, 90 },
    { 0xd02919e0, 75 },
    { 0x5d6919e0, 95 },
    { 0x86b919e2, 46 },
    { 0xd20919e1, 17 },
    { 0xc83919e3, 55 },
    { 0x81c919e5, 61 },
    { 0x2d2919e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48919f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2919f3, 90 },
    { 0x472919f4, 55 },
    { 0x97f919f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b919f8, 43 },
    { 0xfbe919f8, 95 },
    { 0, 0 },
    { 0x7c9919fb, 98 },
    { 0x442919fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd919ff, 93 },
    { 0x732919ff, 90 },
    { 0xea691a01, 84 },
    { 0xe3f91a02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa91a09, 77 },
    { 0x37991a09, 61 },
    { 0xa8891a0a, 98 },
    { 0xf7c91a0a, 33 },
    { 0xc2f91a0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52891a10, 33 },
    { 0, 0 },
    { 0x32b91a12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a91a1e, 33 },
    { 0x1ef91a1f, 93 },
    { 0, 0 },
    { 0x63691a21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8991a24, 90 },
    { 0x3bb91a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e591a30, 61 },
    { 0xa5191a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37891a35, 68 },
    { 0x8e491a36, 46 },
    { 0xe2191a36, 85 },
    { 0x4c191a38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c491a40, 87 },
    { 0, 0 },
    { 0x54c91a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2091a45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c91a49, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf791a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3591a4f, 98 },
    { 0xf3191a50, 23 },
    { 0xa4f91a51, 76 },
    { 0x92191a4f, 98 },
    { 0x86791a52, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc91a56, 76 },
    { 0xad491a57, 95 },
    { 0xda291a58, 61 },
    { 0xdd791a58, 43 },
    { 0x33e91a58, 98 },
    { 0x44391a5b, 76 },
    { 0, 0 },
    { 0x20191a5d, 93 },
    { 0x42b91a5e, 43 },
    { 0x51691a5e, 90 },
    { 0x31691a60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5391a63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9291a66, 23 },
    { 0xa1491a67, 52 },
    { 0, 0 },
    { 0x72491a69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df91a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b91a72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2391a77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x75091a7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d91a83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f191a86, 96 },
    { 0, 0 },
    { 0x57591a88, 98 },
    { 0x19291a88, 81 },
    { 0xc3291a8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e91a8f, 93 },
    { 0, 0 },
    { 0xd7491a91, 23 },
    { 0, 0 },
    { 0x71a91a93, 78 },
    { 0x4ec91a94, 33 },
    { 0, 0 },
    { 0xe2191a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa291a99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d91a9c, 89 },
    { 0, 0 },
    { 0x69391a9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7991aa4, 61 },
    { 0, 0 },
    { 0x23791aa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e191aa9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2891aac, 87 },
    { 0x53391aad, 96 },
    { 0, 0 },
    { 0xfeb91aaf, 99 },
    { 0x58091ab0, 61 },
    { 0x62591ab1, 76 },
    { 0x63d91aaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c91ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac91abc, 33 },
    { 0xa8b91abc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0991ac0, 84 },
    { 0, 0 },
    { 0xa4891ac2, 98 },
    { 0x41291ac3, 91 },
    { 0xb2b91ac4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65391ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63691ad4, 87 },
    { 0x59b91ad4, 23 },
    { 0x20991ad6, 90 },
    { 0xd8691ad6, 98 },
    { 0xa0591ad8, 52 },
    { 0x10091ad8, 87 },
    { 0, 0 },
    { 0x9fc91adb, 78 },
    { 0, 0 },
    { 0x50c91add, 95 },
    { 0xcee91ade, 95 },
    { 0x5191add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab91aeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d091aee, 87 },
    { 0x90f91aef, 23 },
    { 0xe0791aef, 90 },
    { 0x8b991aef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49591af8, 77 },
    { 0x16791af9, 23 },
    { 0xa6a91af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed891afd, 66 },
    { 0x86091afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d91b03, 43 },
    { 0, 0 },
    { 0x39091b05, 76 },
    { 0x2991b06, 85 },
    { 0x8c591b07, 90 },
    { 0x38c91b07, 61 },
    { 0, 0 },
    { 0x18091b0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c91b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf391b11, 91 },
    { 0, 0 },
    { 0xb0891b13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2391b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b91b23, 43 },
    { 0, 0 },
    { 0x39d91b25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4591b32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1791b36, 88 },
    { 0, 0 },
    { 0x60291b38, 23 },
    { 0xd0191b39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc91b3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a91b47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae91b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2091b5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2591b61, 76 },
    { 0xfac91b62, 34 },
    { 0xcb791b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54391b6a, 23 },
    { 0x28391b6b, 90 },
    { 0x61891b6c, 99 },
    { 0, 0 },
    { 0xe2491b6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12891b71, 61 },
    { 0x12291b72, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b91b75, 95 },
    { 0, 0 },
    { 0xa0891b77, 78 },
    { 0, 0 },
    { 0x1fa91b79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d91b81, 68 },
    { 0xd791b82, 61 },
    { 0xc7491b83, 95 },
    { 0x29e91b84, 78 },
    { 0xaf291b85, 33 },
    { 0xcbb91b86, 61 },
    { 0x45b91b81, 98 },
    { 0x7ab91b83, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5191b8c, 91 },
    { 0x85091b8d, 23 },
    { 0xda591b8e, 14 },
    { 0x34a91b8d, 52 },
    { 0xb3691b8d, 87 },
    { 0x89791b91, 23 },
    { 0x8d191b8d, 98 },
    { 0x64491b91, 84 },
    { 0xd8391b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5391b9c, 33 },
    { 0xb5491b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89091ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ba3, 33 },
    { 0x40791ba4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f691bab, 93 },
    { 0xa6891bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d91bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20691bb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791bb6, 99 },
    { 0x8ba91bb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f91bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e91bbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96491bc2, 61 },
    { 0, 0 },
    { 0xa3d91bc4, 97 },
    { 0xb3391bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7891bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9091bce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17791bd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa91bd7, 61 },
    { 0xa5c91bd8, 23 },
    { 0xe3591bd9, 46 },
    { 0x11a91bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f91bdd, 65 },
    { 0xe3191bde, 85 },
    { 0, 0 },
    { 0xd5191be0, 90 },
    { 0xfd091be0, 78 },
    { 0, 0 },
    { 0x83991be3, 23 },
    { 0xd2991be4, 95 },
    { 0xfb391be3, 95 },
    { 0x63f91be6, 61 },
    { 0, 0 },
    { 0xf2591be8, 100 },
    { 0xf5191be9, 46 },
    { 0xd7f91be8, 23 },
    { 0x2b091beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf391bee, 23 },
    { 0xaa391bef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93291bf5, 95 },
    { 0x26491bf6, 90 },
    { 0xed391bf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191bfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a91c00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a91c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9091c08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f91c0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53391c0f, 33 },
    { 0x10591c10, 33 },
    { 0xc7e91c11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a991c16, 90 },
    { 0x8a91c16, 99 },
    { 0x68a91c18, 90 },
    { 0xb7091c19, 90 },
    { 0x41891c1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c791c1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e91c29, 91 },
    { 0x7d091c2a, 34 },
    { 0, 0 },
    { 0x8591c2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5291c30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db91c37, 78 },
    { 0xd0b91c38, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55091c3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d291c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f91c42, 33 },
    { 0xd7091c42, 46 },
    { 0xbe091c44, 23 },
    { 0xa791c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0091c52, 34 },
    { 0xced91c53, 33 },
    { 0x7ff91c54, 46 },
    { 0x24a91c55, 100 },
    { 0xae291c56, 95 },
    { 0x3ac91c53, 66 },
    { 0xee291c56, 33 },
    { 0x28791c59, 98 },
    { 0, 0 },
    { 0x9cc91c5b, 90 },
    { 0x5ab91c5b, 34 },
    { 0xeeb91c5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e891c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3691c67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea91c6a, 35 },
    { 0, 0 },
    { 0x23291c6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39491c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d491c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77891c7e, 33 },
    { 0x3a91c7f, 76 },
    { 0x68f91c7f, 98 },
    { 0x64691c81, 95 },
    { 0x9491c82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53191c85, 90 },
    { 0x3d691c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d91c8c, 23 },
    { 0xd8c91c8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d991c90, 97 },
    { 0, 0 },
    { 0x24991c92, 88 },
    { 0, 0 },
    { 0xa5d91c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f91c97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83891c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e491c9e, 46 },
    { 0, 0 },
    { 0xc4891ca0, 90 },
    { 0xd5091ca1, 90 },
    { 0xe3f91ca1, 95 },
    { 0xccf91ca3, 90 },
    { 0x52891ca1, 98 },
    { 0xc6091ca0, 98 },
    { 0, 0 },
    { 0x44e91ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf791caa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41091cae, 55 },
    { 0x1e591caf, 91 },
    { 0, 0 },
    { 0xa6191cb1, 66 },
    { 0x70c91cb2, 98 },
    { 0, 0 },
    { 0x9e891cb4, 90 },
    { 0, 0 },
    { 0xb1591cb6, 76 },
    { 0xe4391cb7, 66 },
    { 0x9291cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab891cbd, 95 },
    { 0, 0 },
    { 0x67791cbf, 95 },
    { 0x4e791cbf, 100 },
    { 0x2db91cbf, 101 },
    { 0xb1691cc2, 87 },
    { 0x91791cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0191cc8, 33 },
    { 0xc6891cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12991ccd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91cd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d91cd7, 90 },
    { 0, 0 },
    { 0x28d91cd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b91cdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e491ce3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37691ce8, 76 },
    { 0, 0 },
    { 0x93491cea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0291cf1, 90 },
    { 0x19891cf2, 55 },
    { 0xdcc91cf1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3891cf7, 90 },
    { 0x91b91cf8, 90 },
    { 0xb1a91cf7, 34 },
    { 0xa0c91cfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d91cff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb091d04, 33 },
    { 0xf8391d05, 95 },
    { 0x55691d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f91d09, 84 },
    { 0, 0 },
    { 0xf0391d0b, 87 },
    { 0x4f391d0c, 23 },
    { 0xac91d0c, 76 },
    { 0x24b91d0c, 23 },
    { 0x4d891d0f, 90 },
    { 0x93b91d0f, 23 },
    { 0xb7591d10, 90 },
    { 0x39291d12, 88 },
    { 0x5f091d12, 76 },
    { 0xac191d0f, 33 },
    { 0x17191d14, 46 },
    { 0x36691d16, 33 },
    { 0xbb691d17, 46 },
    { 0xc6091d17, 76 },
    { 0, 0 },
    { 0xf6a91d1a, 43 },
    { 0xf8191d1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83291d22, 98 },
    { 0, 0 },
    { 0x36a91d24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b91d28, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f91d2f, 90 },
    { 0x67c91d30, 67 },
    { 0xb2d91d31, 98 },
    { 0x36191d31, 34 },
    { 0x18291d33, 35 },
    { 0xd6791d33, 33 },
    { 0x7b191d35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a91d39, 46 },
    { 0x55091d3a, 90 },
    { 0xa4291d3b, 91 },
    { 0xffb91d3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a91d3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f391d42, 87 },
    { 0x8891d43, 55 },
    { 0x9de91d44, 95 },
    { 0x77591d42, 95 },
    { 0x7191d44, 84 },
    { 0x16991d47, 76 },
    { 0x2e191d44, 55 },
    { 0x96991d49, 75 },
    { 0xe7191d4a, 23 },
    { 0x3a891d4b, 61 },
    { 0xdcc91d4b, 52 },
    { 0x8cb91d4a, 76 },
    { 0xc6491d4e, 90 },
    { 0x7a291d4e, 95 },
    { 0x8e891d4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84391d54, 88 },
    { 0x40391d55, 88 },
    { 0x8f791d56, 98 },
    { 0xba391d55, 61 },
    { 0x6c791d58, 67 },
    { 0x5ac91d58, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c91d5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43791d61, 61 },
    { 0xda091d61, 93 },
    { 0x97391d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d91d67, 95 },
    { 0, 0 },
    { 0x81191d69, 61 },
    { 0x38391d69, 33 },
    { 0x3a291d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81391d6e, 23 },
    { 0, 0 },
    { 0xc5491d70, 44 },
    { 0, 0 },
    { 0xe4e91d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d91d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x691d7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891d7f, 90 },
    { 0x37791d80, 90 },
    { 0xb8591d81, 66 },
    { 0x9ee91d82, 46 },
    { 0xe3e91d80, 91 },
    { 0, 0 },
    { 0xdf591d85, 90 },
    { 0x6dc91d86, 76 },
    { 0xb6791d86, 98 },
    { 0xcf191d88, 33 },
    { 0x48491d89, 87 },
    { 0x8df91d8a, 76 },
    { 0xc7491d86, 99 },
    { 0xc8491d88, 61 },
    { 0xa1891d89, 98 },
    { 0x6ef91d8e, 90 },
    { 0xfaf91d8e, 76 },
    { 0xa1991d8e, 90 },
    { 0x8ae91d91, 81 },
    { 0x29b91d87, 95 },
    { 0x73e91d93, 95 },
    { 0xb4791d94, 95 },
    { 0x11291d93, 98 },
    { 0x70791d96, 61 },
    { 0x5cc91d97, 95 },
    { 0xe6291d95, 23 },
    { 0x77e91d8a, 61 },
    { 0x21091d9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce91d9d, 66 },
    { 0, 0 },
    { 0x60e91d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c691da4, 78 },
    { 0, 0 },
    { 0x9de91da6, 34 },
    { 0x6ec91da7, 33 },
    { 0xeaa91da7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b91dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891dae, 43 },
    { 0, 0 },
    { 0xa4991db0, 38 },
    { 0, 0 },
    { 0xd7a91db2, 90 },
    { 0, 0 },
    { 0x1be91db4, 99 },
    { 0x60191db5, 52 },
    { 0x8ae91db5, 23 },
    { 0, 0 },
    { 0x81891db8, 91 },
    { 0xcd591db9, 90 },
    { 0x4da91dba, 33 },
    { 0xa8391dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f691dbf, 61 },
    { 0x59c91dbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e991dc4, 76 },
    { 0x76791dc5, 99 },
    { 0x6d491dc6, 90 },
    { 0x5b891dc5, 91 },
    { 0x16591dc4, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e91dcb, 76 },
    { 0xad991dcc, 34 },
    { 0xbcd91dcb, 33 },
    { 0x6ff91dcd, 34 },
    { 0, 0 },
    { 0xba291dd0, 76 },
    { 0xab691dd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10691dd5, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c091dd9, 61 },
    { 0, 0 },
    { 0xe5f91ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81691dde, 23 },
    { 0, 0 },
    { 0xdab91de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c91de3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f791de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c291dea, 90 },
    { 0, 0 },
    { 0x97a91dec, 55 },
    { 0x7c791ded, 90 },
    { 0x8b791dee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e91df4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e91df8, 33 },
    { 0x92791df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c91dfc, 90 },
    { 0xddb91dfd, 76 },
    { 0x27191dfe, 90 },
    { 0x10591dff, 23 },
    { 0xa4191dfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f91e03, 52 },
    { 0x5f591e03, 34 },
    { 0, 0 },
    { 0x37f91e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e491e09, 78 },
    { 0x2ba91e0a, 55 },
    { 0xb0191e0b, 23 },
    { 0, 0 },
    { 0xcae91e0d, 66 },
    { 0xfba91e0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2391e15, 33 },
    { 0x15591e16, 23 },
    { 0x2a991e17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b591e1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e91e1f, 90 },
    { 0x15391e1f, 90 },
    { 0x22691e1f, 33 },
    { 0x25291e22, 95 },
    { 0xa1391e20, 95 },
    { 0x6991e24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c91e29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8891e2c, 61 },
    { 0x69991e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95991e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77691e35, 98 },
    { 0x5e391e35, 34 },
    { 0xe3591e37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15891e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84891e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc691e4d, 93 },
    { 0xc4991e4d, 95 },
    { 0x3591e4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50991e52, 90 },
    { 0x7a291e53, 56 },
    { 0, 0 },
    { 0xddc91e55, 40 },
    { 0, 0 },
    { 0x7df91e57, 90 },
    { 0, 0 },
    { 0x7c91e59, 95 },
    { 0xb5891e5a, 33 },
    { 0xb3e91e5b, 43 },
    { 0xd8f91e5b, 23 },
    { 0x2f691e5a, 69 },
    { 0xdc391e5b, 93 },
    { 0x57291e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29391e63, 90 },
    { 0xfa491e64, 43 },
    { 0, 0 },
    { 0xb4991e66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a91e69, 98 },
    { 0xd6b91e69, 33 },
    { 0x8f491e6b, 98 },
    { 0x79391e6b, 43 },
    { 0x2ce91e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe091e70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa91e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91e76, 99 },
    { 0, 0 },
    { 0x2f491e78, 66 },
    { 0, 0 },
    { 0x98b91e7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd91e82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e91e85, 90 },
    { 0xe0991e85, 90 },
    { 0x9b491e85, 67 },
    { 0, 0 },
    { 0xcfa91e89, 95 },
    { 0xae591e8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37191e8e, 76 },
    { 0x86791e8e, 55 },
    { 0xafa91e90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab91e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58391e97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75791e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e91e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39791ea1, 85 },
    { 0xb7d91ea1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82091ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a691eac, 90 },
    { 0x2ef91ead, 95 },
    { 0x1ee91eae, 90 },
    { 0xfba91eaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4191eb2, 80 },
    { 0, 0 },
    { 0xb9e91eb4, 46 },
    { 0x4f91eb4, 90 },
    { 0x8ff91eb4, 95 },
    { 0x33391eb7, 90 },
    { 0x7f291eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59691ec0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8391ec7, 61 },
    { 0x70c91ec8, 76 },
    { 0xa0f91ec9, 46 },
    { 0xe5891ec9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41991ed0, 95 },
    { 0x80391ed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e91ed4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff991ed7, 98 },
    { 0x73991ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8591edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc691ee0, 61 },
    { 0, 0 },
    { 0x2cb91ee2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac691ee7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac91eef, 98 },
    { 0x16c91ef0, 43 },
    { 0, 0 },
    { 0x2d591ef2, 85 },
    { 0x38491ef3, 55 },
    { 0, 0 },
    { 0x66791ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12891ef9, 61 },
    { 0, 0 },
    { 0x5b691efb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29891eff, 52 },
    { 0xb4191f00, 61 },
    { 0x83b91eff, 23 },
    { 0xf6391f02, 87 },
    { 0x26791f03, 87 },
    { 0xeae91eff, 55 },
    { 0x42e91f05, 76 },
    { 0x86a91f06, 99 },
    { 0xec491f07, 33 },
    { 0, 0 },
    { 0x57b91f09, 100 },
    { 0, 0 },
    { 0xf8c91f0b, 78 },
    { 0x74b91f0b, 46 },
    { 0xac991f0c, 61 },
    { 0x3a491f0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6791f11, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691f15, 46 },
    { 0x61a91f16, 61 },
    { 0, 0 },
    { 0x4ff91f18, 61 },
    { 0xff391f19, 90 },
    { 0xcbe91f1a, 61 },
    { 0x64991f1b, 90 },
    { 0xdf991f18, 90 },
    { 0x1e891f1d, 75 },
    { 0x86f91f18, 90 },
    { 0x96591f1b, 38 },
    { 0x67291f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1291f24, 33 },
    { 0, 0 },
    { 0x9e191f26, 61 },
    { 0x26191f26, 61 },
    { 0x2df91f28, 95 },
    { 0x28191f26, 33 },
    { 0xe1991f2a, 61 },
    { 0, 0 },
    { 0xa7091f2c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b91f31, 46 },
    { 0x1bd91f32, 61 },
    { 0xdd091f32, 34 },
    { 0xef691f31, 90 },
    { 0x54d91f35, 90 },
    { 0, 0 },
    { 0x2c191f37, 95 },
    { 0, 0 },
    { 0xb2e91f39, 43 },
    { 0xbb791f3a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab391f3d, 43 },
    { 0, 0 },
    { 0xb3091f3f, 90 },
    { 0x79291f40, 91 },
    { 0xc391f3f, 93 },
    { 0x26391f41, 76 },
    { 0x2d691f43, 91 },
    { 0xc7191f40, 55 },
    { 0x16991f45, 78 },
    { 0xf8091f46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf491f49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491f4d, 61 },
    { 0, 0 },
    { 0x57891f4f, 61 },
    { 0, 0 },
    { 0xae791f51, 91 },
    { 0, 0 },
    { 0xda991f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3691f5a, 88 },
    { 0xa3191f5a, 33 },
    { 0xbed91f5a, 33 },
    { 0x2a091f5b, 90 },
    { 0x37191f5e, 90 },
    { 0xaa791f5f, 90 },
    { 0x76091f5e, 90 },
    { 0, 0 },
    { 0xfb191f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3891f68, 90 },
    { 0x2f91f69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891f6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee991f70, 43 },
    { 0x2f91f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a91f75, 61 },
    { 0x62191f75, 65 },
    { 0, 0 },
    { 0x54f91f78, 34 },
    { 0, 0 },
    { 0xd8891f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e91f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c91f87, 98 },
    { 0x43b91f87, 99 },
    { 0, 0 },
    { 0xf0391f8a, 55 },
    { 0, 0 },
    { 0xa1391f8c, 61 },
    { 0xc4091f8c, 23 },
    { 0, 0 },
    { 0xff091f8f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62391f93, 76 },
    { 0x71f91f94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3091f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e91fa2, 90 },
    { 0x61f91fa2, 53 },
    { 0x3a191fa4, 67 },
    { 0x71091fa5, 84 },
    { 0xdf191fa6, 90 },
    { 0xd2791fa7, 61 },
    { 0xe5091fa4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e391fad, 61 },
    { 0xf9391fad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3591fb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b91fbe, 17 },
    { 0, 0 },
    { 0x51091fc0, 90 },
    { 0x35591fc1, 90 },
    { 0x88c91fc2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36291fc6, 61 },
    { 0, 0 },
    { 0x5991fc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46891fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee491fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2391fd8, 33 },
    { 0x19391fd8, 91 },
    { 0xae891fd9, 95 },
    { 0x89a91fda, 46 },
    { 0xea291fdc, 61 },
    { 0x82d91fdc, 33 },
    { 0x86991fdd, 66 },
    { 0x56291fde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f691fe2, 61 },
    { 0x76591fe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d791fe6, 61 },
    { 0, 0 },
    { 0x30791fe8, 97 },
    { 0xc9b91fe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed91fec, 46 },
    { 0x21391fed, 55 },
    { 0x3d791fee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb91ff4, 61 },
    { 0, 0 },
    { 0xf2291ff6, 81 },
    { 0xf5791ff7, 95 },
    { 0xf8091ff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc91ffe, 33 },
    { 0x37391fff, 64 },
    { 0x47192000, 90 },
    { 0x1da92000, 43 },
    { 0, 0 },
    { 0x24b92003, 33 },
    { 0x45892004, 87 },
    { 0xdb692004, 61 },
    { 0xb0992006, 90 },
    { 0x5f692005, 94 },
    { 0x23d92008, 33 },
    { 0x29b92009, 76 },
    { 0, 0 },
    { 0xbc49200b, 52 },
    { 0x2089200c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc92012, 91 },
    { 0xa8992013, 100 },
    { 0x8c192012, 83 },
    { 0xfda92014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1092019, 90 },
    { 0x5239201a, 61 },
    { 0x70e9201a, 96 },
    { 0, 0 },
    { 0x1a79201d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9892020, 91 },
    { 0x35792021, 61 },
    { 0x31e92022, 90 },
    { 0x6da92023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde992026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc079202d, 75 },
    { 0, 0 },
    { 0x1d39202f, 33 },
    { 0x5c292030, 34 },
    { 0x2bb92031, 61 },
    { 0xd0992032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d39203d, 61 },
    { 0xde89203e, 17 },
    { 0x20c9203f, 33 },
    { 0, 0 },
    { 0x65992041, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c292044, 61 },
    { 0x27b92044, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4392049, 52 },
    { 0x5f092049, 77 },
    { 0x90f9204b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33292051, 61 },
    { 0x34892051, 76 },
    { 0xda992053, 76 },
    { 0xbd592053, 98 },
    { 0x8f892053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5192058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe089205b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489206e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e392073, 61 },
    { 0xe8992074, 87 },
    { 0x3ee92075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78492078, 90 },
    { 0x65f92079, 61 },
    { 0xbdc92078, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4592082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c92089, 33 },
    { 0xa3d92089, 77 },
    { 0xb0f9208b, 84 },
    { 0xee69208c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5892091, 33 },
    { 0, 0 },
    { 0xd0e92093, 66 },
    { 0x56c92093, 61 },
    { 0x3b892095, 33 },
    { 0, 0 },
    { 0xdfb92097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a9209f, 33 },
    { 0, 0 },
    { 0x92920a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2920a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967920ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187920be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d920c3, 33 },
    { 0, 0 },
    { 0xf4a920c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12920ca, 90 },
    { 0, 0 },
    { 0xc2d920cc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5920cf, 61 },
    { 0x15920d0, 40 },
    { 0x27c920cf, 46 },
    { 0x9b5920d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78920d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad920d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946920dd, 98 },
    { 0x653920dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x281920e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7920e8, 93 },
    { 0, 0 },
    { 0x88c920ea, 77 },
    { 0x29b920ea, 90 },
    { 0x663920ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36920ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8920f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x514920f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432920ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79492104, 90 },
    { 0xa1592105, 95 },
    { 0, 0 },
    { 0x6d992107, 76 },
    { 0xa7792108, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9210b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb692113, 95 },
    { 0x3b792114, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24792119, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9299211e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d92121, 64 },
    { 0xde392122, 61 },
    { 0xe0d92123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d79212a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9212f, 90 },
    { 0xd8c92130, 23 },
    { 0x86892131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9992134, 95 },
    { 0, 0 },
    { 0x37b92136, 91 },
    { 0x6bd92137, 33 },
    { 0xe3092136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e9213b, 33 },
    { 0x75b9213c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c92142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55992148, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279214b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1392157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989215b, 40 },
    { 0xc1b9215c, 33 },
    { 0x6129215d, 46 },
    { 0x2b59215e, 98 },
    { 0x5289215d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca92162, 76 },
    { 0, 0 },
    { 0x592164, 95 },
    { 0xa5892165, 55 },
    { 0xe4592165, 90 },
    { 0xeb892167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569216a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f9216e, 90 },
    { 0, 0 },
    { 0x4ff92170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c792173, 46 },
    { 0, 0 },
    { 0x1d492175, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f92178, 23 },
    { 0, 0 },
    { 0xa499217a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad09217f, 90 },
    { 0x6af92180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7992192, 90 },
    { 0xe6692193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb92198, 93 },
    { 0x6fe92199, 96 },
    { 0x2c89219a, 76 },
    { 0xda29219a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79219f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712921a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1921a7, 98 },
    { 0x4cd921a8, 34 },
    { 0x5aa921a8, 67 },
    { 0, 0 },
    { 0xe5d921ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54921af, 90 },
    { 0, 0 },
    { 0xaa1921b1, 46 },
    { 0x7fa921b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c921b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865921c3, 95 },
    { 0x970921c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c921c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0921cd, 85 },
    { 0xa2d921ce, 17 },
    { 0xb38921cf, 90 },
    { 0, 0 },
    { 0x7d6921d1, 90 },
    { 0x897921d1, 61 },
    { 0xa08921d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a921d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478921dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83921e1, 76 },
    { 0x871921e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f921e5, 55 },
    { 0, 0 },
    { 0x440921e7, 95 },
    { 0, 0 },
    { 0x8c921e9, 95 },
    { 0, 0 },
    { 0x323921eb, 43 },
    { 0xb97921ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf921ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f921f2, 95 },
    { 0x1a7921f3, 98 },
    { 0x330921f3, 90 },
    { 0xba3921f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a921f9, 34 },
    { 0x9d3921f9, 90 },
    { 0x518921f9, 78 },
    { 0, 0 },
    { 0x4cd921fd, 52 },
    { 0x526921fe, 85 },
    { 0x8b9921fe, 66 },
    { 0xb4192200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e92203, 46 },
    { 0x21b92203, 85 },
    { 0xed092203, 65 },
    { 0x92892206, 23 },
    { 0, 0 },
    { 0x14292208, 76 },
    { 0x38392208, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde59220c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa92213, 76 },
    { 0x40992214, 87 },
    { 0x1ca92215, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4092219, 76 },
    { 0x6e792219, 23 },
    { 0x3d39221b, 91 },
    { 0x9a39221c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54492220, 23 },
    { 0xd8592221, 98 },
    { 0, 0 },
    { 0x30592223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a92226, 75 },
    { 0xc9f92227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9222a, 33 },
    { 0x1ae9222b, 55 },
    { 0x6299222c, 33 },
    { 0x18a9222c, 33 },
    { 0x4a79222c, 61 },
    { 0xf159222f, 33 },
    { 0x73c9222f, 23 },
    { 0xbcf9222f, 97 },
    { 0x74c92232, 67 },
    { 0xae592231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb9223b, 33 },
    { 0x4ae9223b, 98 },
    { 0x6d59223c, 33 },
    { 0xd559223c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b792241, 55 },
    { 0xc6992242, 23 },
    { 0xfe892243, 61 },
    { 0xff892243, 34 },
    { 0x87692245, 90 },
    { 0x5b692246, 52 },
    { 0x9ad92246, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94692252, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42192255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b92259, 90 },
    { 0, 0 },
    { 0x1689225b, 98 },
    { 0, 0 },
    { 0xd669225d, 90 },
    { 0x7399225e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42292266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb9226d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff92271, 23 },
    { 0xfdc92272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d92276, 91 },
    { 0, 0 },
    { 0xfde92278, 99 },
    { 0, 0 },
    { 0x3819227a, 90 },
    { 0xfac9227a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7892280, 95 },
    { 0x33392281, 68 },
    { 0x13592282, 77 },
    { 0x27192282, 85 },
    { 0xc7892280, 61 },
    { 0, 0 },
    { 0xece92286, 98 },
    { 0, 0 },
    { 0xf92288, 95 },
    { 0x21b92288, 93 },
    { 0xb289228a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8392290, 65 },
    { 0x53992291, 90 },
    { 0x33d92292, 55 },
    { 0x53892293, 98 },
    { 0x7a492294, 33 },
    { 0xd4192292, 90 },
    { 0xcda92291, 76 },
    { 0x5c692294, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419229c, 66 },
    { 0, 0 },
    { 0x54f9229e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6922a1, 90 },
    { 0, 0 },
    { 0x723922a3, 90 },
    { 0, 0 },
    { 0x91b922a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc922ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf922af, 99 },
    { 0, 0 },
    { 0x1cd922b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7922b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95922b9, 23 },
    { 0xc2922ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x527922bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9922c1, 95 },
    { 0xd28922c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce922c7, 43 },
    { 0xd922c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72922cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885922cf, 90 },
    { 0x58e922cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5922d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e922d8, 34 },
    { 0xa0b922d8, 68 },
    { 0, 0 },
    { 0xb71922db, 95 },
    { 0x4a9922dc, 91 },
    { 0, 0 },
    { 0xbd4922de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656922e2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920922e7, 81 },
    { 0xfd7922e8, 87 },
    { 0x6ff922e9, 23 },
    { 0x218922ea, 85 },
    { 0x222922eb, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x794922ee, 23 },
    { 0x3b4922ef, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb922f2, 36 },
    { 0x469922f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47922f8, 23 },
    { 0x8b4922f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1922fc, 91 },
    { 0x3c9922fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95292309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9230c, 85 },
    { 0x8b39230c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee92315, 84 },
    { 0, 0 },
    { 0x2892317, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459231a, 90 },
    { 0, 0 },
    { 0x31a9231c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xee89231f, 46 },
    { 0, 0 },
    { 0x19092321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda592324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50792327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f9232b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069232e, 43 },
    { 0x8989232f, 76 },
    { 0x75a9232f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a92333, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692336, 87 },
    { 0x19b92337, 95 },
    { 0xa7192337, 76 },
    { 0, 0 },
    { 0xf009233a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c79233e, 67 },
    { 0x9379233f, 40 },
    { 0x94b92340, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24892344, 83 },
    { 0x53692345, 33 },
    { 0xb1692346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f92349, 70 },
    { 0xd7a9234a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a09234e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f492351, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b92356, 61 },
    { 0x33892357, 61 },
    { 0x85d92358, 61 },
    { 0x97792358, 61 },
    { 0xa3892357, 43 },
    { 0, 0 },
    { 0xf899235c, 61 },
    { 0x5879235d, 95 },
    { 0x3139235e, 98 },
    { 0xa1a9235f, 23 },
    { 0x5ff9235f, 98 },
    { 0xf009235e, 98 },
    { 0xda79235f, 66 },
    { 0x2ea92363, 98 },
    { 0x74592363, 76 },
    { 0xe1992364, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf9236d, 33 },
    { 0xc929236e, 33 },
    { 0, 0 },
    { 0x5a292370, 98 },
    { 0xdb192371, 61 },
    { 0, 0 },
    { 0xd7792373, 33 },
    { 0x65092373, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7692377, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9237a, 95 },
    { 0, 0 },
    { 0xb799237c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b92381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34892384, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8992387, 66 },
    { 0, 0 },
    { 0x71d92389, 33 },
    { 0x6db9238a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1099238f, 92 },
    { 0xb759238f, 90 },
    { 0xcda9238f, 90 },
    { 0xe3f9238f, 61 },
    { 0x24f9238f, 61 },
    { 0x61692392, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba192399, 23 },
    { 0x5a692399, 77 },
    { 0xd369239b, 76 },
    { 0x7759239a, 66 },
    { 0x4609239d, 90 },
    { 0, 0 },
    { 0xd2c9239f, 53 },
    { 0, 0 },
    { 0x2b6923a1, 99 },
    { 0x9a6923a2, 90 },
    { 0x80c923a3, 23 },
    { 0x55c923a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af923aa, 95 },
    { 0x4dd923ab, 95 },
    { 0xd51923ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74923af, 92 },
    { 0x2fb923af, 61 },
    { 0xe6f923af, 75 },
    { 0xd6c923b1, 95 },
    { 0x98b923b3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e923b6, 95 },
    { 0xbb2923b7, 98 },
    { 0x913923b8, 87 },
    { 0x5f4923b6, 90 },
    { 0xcb3923ba, 43 },
    { 0xfb2923bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c923bf, 55 },
    { 0xab923c0, 99 },
    { 0x4d923c1, 55 },
    { 0, 0 },
    { 0x174923c3, 23 },
    { 0x5ef923c3, 75 },
    { 0xe5c923c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf923cc, 95 },
    { 0x7b0923cc, 96 },
    { 0, 0 },
    { 0x54923cf, 96 },
    { 0, 0 },
    { 0x105923d1, 33 },
    { 0x301923d1, 66 },
    { 0x781923d3, 90 },
    { 0x9a7923d4, 55 },
    { 0, 0 },
    { 0x306923d6, 95 },
    { 0x726923d7, 34 },
    { 0x94e923d7, 93 },
    { 0x6e923d9, 33 },
    { 0x659923da, 43 },
    { 0, 0 },
    { 0xd26923dc, 90 },
    { 0x2d0923dd, 55 },
    { 0, 0 },
    { 0x279923df, 81 },
    { 0xa96923e0, 33 },
    { 0x90b923df, 93 },
    { 0, 0 },
    { 0xef8923e3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9923ec, 33 },
    { 0, 0 },
    { 0x8e4923ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c923fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b92403, 66 },
    { 0xfb592404, 33 },
    { 0x9b492403, 23 },
    { 0, 0 },
    { 0x97b92407, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad9240a, 98 },
    { 0x6b29240b, 95 },
    { 0, 0 },
    { 0xa049240d, 98 },
    { 0xb699240e, 33 },
    { 0xd5e9240d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34492413, 95 },
    { 0x95e92413, 95 },
    { 0xdcb92415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7109241b, 90 },
    { 0x659241b, 67 },
    { 0x47b9241d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76392423, 84 },
    { 0, 0 },
    { 0xf9192425, 33 },
    { 0xd0892426, 76 },
    { 0xdc992427, 43 },
    { 0x1be92427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9242d, 33 },
    { 0x8409242e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c92431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae992443, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70192447, 95 },
    { 0xa6392448, 90 },
    { 0xa0c92449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf679244c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a392450, 61 },
    { 0xf492450, 90 },
    { 0xaba92452, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d292455, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff92458, 23 },
    { 0x16492458, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69245d, 78 },
    { 0x1c99245e, 98 },
    { 0x419245e, 95 },
    { 0x5219245f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2992466, 65 },
    { 0, 0 },
    { 0x1fd92468, 33 },
    { 0, 0 },
    { 0x8709246a, 61 },
    { 0xefe9246a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b9246e, 33 },
    { 0, 0 },
    { 0xd2e92470, 23 },
    { 0xbfa92471, 33 },
    { 0xc2392472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62592475, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe592479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92485, 61 },
    { 0, 0 },
    { 0x75d92487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a9248a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9248d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a92491, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4592496, 66 },
    { 0x9bd92496, 33 },
    { 0x64192498, 61 },
    { 0xd6292497, 99 },
    { 0x9ae9249a, 33 },
    { 0x9019249b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5779249f, 33 },
    { 0, 0 },
    { 0xd15924a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae924ac, 65 },
    { 0, 0 },
    { 0x3e8924ae, 23 },
    { 0xa8a924af, 90 },
    { 0, 0 },
    { 0x3a9924b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911924ba, 44 },
    { 0xc2b924bb, 55 },
    { 0xcbe924bc, 99 },
    { 0x199924bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd924c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53924cb, 33 },
    { 0x614924cc, 43 },
    { 0x670924cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed924d2, 98 },
    { 0, 0 },
    { 0xf19924d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315924d9, 33 },
    { 0x117924da, 46 },
    { 0x1bb924da, 95 },
    { 0x377924dc, 78 },
    { 0xda0924d9, 95 },
    { 0x9d9924dd, 58 },
    { 0x460924df, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6924e4, 33 },
    { 0, 0 },
    { 0x495924e6, 23 },
    { 0x38d924e6, 33 },
    { 0xc4a924e8, 61 },
    { 0x7a7924e9, 23 },
    { 0x9d8924e9, 100 },
    { 0xfbc924eb, 61 },
    { 0x4ed924ec, 61 },
    { 0xfaa924ec, 84 },
    { 0, 0 },
    { 0x725924ef, 90 },
    { 0xff5924ef, 95 },
    { 0, 0 },
    { 0x7a3924f2, 76 },
    { 0x189924f3, 61 },
    { 0x186924f4, 93 },
    { 0xbbb924f5, 23 },
    { 0xcfb924f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177924fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5924ff, 91 },
    { 0, 0 },
    { 0x78d92501, 61 },
    { 0xdc792501, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859250a, 34 },
    { 0, 0 },
    { 0xc559250c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe349250f, 55 },
    { 0x4af92510, 99 },
    { 0x6bc92511, 76 },
    { 0xf3292511, 23 },
    { 0x81092513, 90 },
    { 0x2ca92514, 98 },
    { 0x8899250f, 95 },
    { 0xad692510, 81 },
    { 0xa4b92517, 95 },
    { 0xf6b92518, 98 },
    { 0xbda92519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d9251d, 33 },
    { 0xa4e9251e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad92521, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a492525, 23 },
    { 0x61092526, 55 },
    { 0x35b92527, 95 },
    { 0x4fa92528, 90 },
    { 0, 0 },
    { 0xf7f9252a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9252d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7492531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed9253b, 33 },
    { 0xe879253c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47192542, 95 },
    { 0, 0 },
    { 0x22592544, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8892547, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d39254d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd92550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac92553, 90 },
    { 0xe5a92553, 95 },
    { 0xb0a92555, 77 },
    { 0x6be92556, 95 },
    { 0, 0 },
    { 0xefd92558, 61 },
    { 0xea992559, 95 },
    { 0, 0 },
    { 0x2ba9255b, 98 },
    { 0x6859255c, 61 },
    { 0xda39255d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d92561, 95 },
    { 0, 0 },
    { 0x9ca92563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee592566, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8492569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4599256e, 33 },
    { 0x10b9256e, 98 },
    { 0xd7592570, 33 },
    { 0xa2292571, 90 },
    { 0x46c92570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c79257b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9592582, 90 },
    { 0xac92583, 55 },
    { 0, 0 },
    { 0x44292585, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d9258a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9258d, 23 },
    { 0x4ac9258d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70192595, 98 },
    { 0x28e92595, 65 },
    { 0, 0 },
    { 0xfb992598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf099259b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719259e, 99 },
    { 0xae9259e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee925a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e925aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895925b0, 90 },
    { 0x95e925b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9925b7, 44 },
    { 0x8f7925b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135925bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb925c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95925c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d925c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121925cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x972925d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61925d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d925da, 95 },
    { 0xd16925da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b925e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff925e8, 98 },
    { 0x27b925e9, 96 },
    { 0xd51925ea, 46 },
    { 0, 0 },
    { 0x544925ec, 34 },
    { 0, 0 },
    { 0x279925ee, 23 },
    { 0x360925ef, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x318925f2, 44 },
    { 0, 0 },
    { 0x95c925f4, 76 },
    { 0xeba925f4, 93 },
    { 0xcb9925f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6925f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4925fe, 46 },
    { 0, 0 },
    { 0x42892600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d992608, 33 },
    { 0x75492608, 61 },
    { 0x669260a, 67 },
    { 0xe309260a, 90 },
    { 0x88b9260c, 76 },
    { 0x61c92609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd792611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef92614, 76 },
    { 0xbac92615, 78 },
    { 0xa0d92616, 67 },
    { 0x96992616, 90 },
    { 0, 0 },
    { 0x8e992619, 76 },
    { 0x8499261a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09261e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae92625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d92629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a892633, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef92636, 72 },
    { 0, 0 },
    { 0x1b392638, 95 },
    { 0, 0 },
    { 0xc09263a, 52 },
    { 0x9ec9263b, 93 },
    { 0, 0 },
    { 0x2119263d, 98 },
    { 0x6c89263e, 85 },
    { 0xe189263f, 67 },
    { 0x7239263f, 81 },
    { 0xb9a9263e, 34 },
    { 0xcdc92641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d192645, 90 },
    { 0x6b92645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe339264c, 90 },
    { 0x6f29264d, 95 },
    { 0xf669264c, 93 },
    { 0, 0 },
    { 0x9d092650, 61 },
    { 0xd692650, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69592659, 90 },
    { 0x59d9265a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99265f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45892665, 76 },
    { 0xcae92666, 90 },
    { 0, 0 },
    { 0x7f192668, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2692670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd819267b, 34 },
    { 0x6f99267c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12592687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa39268d, 76 },
    { 0xffc9268e, 43 },
    { 0x6129268e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6592692, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f92695, 90 },
    { 0, 0 },
    { 0xde092697, 99 },
    { 0, 0 },
    { 0xbba92699, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999269d, 98 },
    { 0xf859269e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec926a1, 76 },
    { 0, 0 },
    { 0x44a926a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d926a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b926a9, 93 },
    { 0xfb1926aa, 67 },
    { 0xcf9926aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d926b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79926b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82926b8, 66 },
    { 0xc42926b8, 77 },
    { 0xc5b926ba, 99 },
    { 0x39f926ba, 55 },
    { 0x980926bc, 55 },
    { 0, 0 },
    { 0xbb0926be, 17 },
    { 0, 0 },
    { 0x7cf926c0, 33 },
    { 0x292926c0, 84 },
    { 0x637926c1, 61 },
    { 0, 0 },
    { 0x9ce926c4, 61 },
    { 0, 0 },
    { 0xe85926c6, 93 },
    { 0, 0 },
    { 0x62f926c8, 90 },
    { 0x4b926c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3926cc, 43 },
    { 0x813926cc, 64 },
    { 0, 0 },
    { 0x88f926cf, 33 },
    { 0, 0 },
    { 0x41926d1, 98 },
    { 0x5c4926d2, 33 },
    { 0x28f926d2, 85 },
    { 0, 0 },
    { 0xe92926d5, 98 },
    { 0, 0 },
    { 0xa46926d7, 61 },
    { 0xbe4926d7, 61 },
    { 0x985926d9, 76 },
    { 0, 0 },
    { 0x79d926db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d926de, 88 },
    { 0xd97926de, 90 },
    { 0xd08926de, 95 },
    { 0xe55926e1, 90 },
    { 0x20f926e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642926e6, 90 },
    { 0x131926e7, 43 },
    { 0xb7a926e8, 55 },
    { 0, 0 },
    { 0x3c8926ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442926f0, 46 },
    { 0xe0d926f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565926f7, 75 },
    { 0xc52926f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6926fd, 40 },
    { 0xbdf926fd, 33 },
    { 0x820926ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592708, 77 },
    { 0xf7d92709, 90 },
    { 0x4669270a, 98 },
    { 0xdcf92709, 65 },
    { 0x12d9270c, 90 },
    { 0xc029270d, 43 },
    { 0xdf29270e, 98 },
    { 0x8bb9270f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0592715, 23 },
    { 0, 0 },
    { 0xd1392717, 17 },
    { 0xa0d92717, 34 },
    { 0x55e92719, 46 },
    { 0x20692717, 23 },
    { 0, 0 },
    { 0xde29271c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf39271f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b92727, 75 },
    { 0xfa092727, 90 },
    { 0x67292729, 46 },
    { 0x8192728, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79392734, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4092738, 76 },
    { 0, 0 },
    { 0xee89273a, 33 },
    { 0x3859273b, 43 },
    { 0x8089273c, 55 },
    { 0x2419273d, 23 },
    { 0xa609273b, 98 },
    { 0, 0 },
    { 0x51792740, 33 },
    { 0xc4a92740, 82 },
    { 0xbd292741, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3292746, 34 },
    { 0, 0 },
    { 0x7a092748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f9274e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5292752, 65 },
    { 0x7a292753, 61 },
    { 0x16692754, 99 },
    { 0xd3092755, 46 },
    { 0, 0 },
    { 0x25f92757, 88 },
    { 0x55292758, 43 },
    { 0x5cf92757, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a92760, 87 },
    { 0, 0 },
    { 0x63192762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539276a, 93 },
    { 0xc179276b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b792774, 61 },
    { 0x7aa92774, 95 },
    { 0x7ec92776, 88 },
    { 0, 0 },
    { 0xfad92778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b09277d, 98 },
    { 0x1689277e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39892781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48592786, 87 },
    { 0x33c92786, 90 },
    { 0x59e92787, 90 },
    { 0x4a92789, 43 },
    { 0x27f92788, 33 },
    { 0xbef9278b, 33 },
    { 0xf8d92786, 90 },
    { 0xaf49278d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12392797, 66 },
    { 0x39492798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb519279b, 66 },
    { 0xb349279c, 90 },
    { 0x7b79279d, 95 },
    { 0x5559279e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3927a1, 56 },
    { 0x5d5927a2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04927a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475927aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e927ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a927b3, 98 },
    { 0x901927b4, 67 },
    { 0, 0 },
    { 0xc17927b6, 66 },
    { 0xcbd927b7, 90 },
    { 0x326927b7, 46 },
    { 0xbd927b9, 33 },
    { 0, 0 },
    { 0x84f927bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x348927be, 34 },
    { 0, 0 },
    { 0xc77927c0, 99 },
    { 0xb9927c1, 98 },
    { 0xed8927c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a927c5, 61 },
    { 0, 0 },
    { 0x9e5927c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x151927ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816927d1, 3 },
    { 0, 0 },
    { 0x23a927d3, 23 },
    { 0x201927d4, 90 },
    { 0xd29927d4, 55 },
    { 0xcf927d5, 95 },
    { 0x979927d6, 61 },
    { 0x30a927d8, 66 },
    { 0xd1e927d9, 69 },
    { 0xe21927da, 33 },
    { 0xfb7927d5, 70 },
    { 0x463927da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3927df, 90 },
    { 0x494927e0, 90 },
    { 0xe2f927e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x695927e4, 81 },
    { 0xd50927e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9927e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x719927eb, 95 },
    { 0x125927eb, 55 },
    { 0xebf927ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x799927f0, 65 },
    { 0, 0 },
    { 0x9f6927f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x493927f5, 98 },
    { 0x66b927f6, 34 },
    { 0, 0 },
    { 0x29e927f8, 76 },
    { 0xf927f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60592803, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6392808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2899280c, 90 },
    { 0, 0 },
    { 0xa419280e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8092811, 76 },
    { 0x88c92811, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b792816, 46 },
    { 0, 0 },
    { 0x9b692818, 34 },
    { 0x1dc92818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c9281c, 95 },
    { 0x2809281d, 69 },
    { 0xfc69281d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b92822, 93 },
    { 0x1a892822, 98 },
    { 0x7d92824, 77 },
    { 0x6d892825, 61 },
    { 0x792826, 43 },
    { 0x98692823, 33 },
    { 0x3b392828, 43 },
    { 0, 0 },
    { 0xaec9282a, 93 },
    { 0, 0 },
    { 0x5309282c, 68 },
    { 0x1fb9282c, 96 },
    { 0x8959282c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15292833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb9283c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44092840, 61 },
    { 0xfa492840, 23 },
    { 0xcf792841, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992846, 98 },
    { 0, 0 },
    { 0x94992848, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5869284c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84992854, 61 },
    { 0xf8592855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50092858, 33 },
    { 0x76892858, 43 },
    { 0x5739285a, 85 },
    { 0x1c49285a, 98 },
    { 0xa109285c, 33 },
    { 0x7ed9285c, 90 },
    { 0xbcc9285e, 99 },
    { 0x7b69285f, 61 },
    { 0x3d992860, 33 },
    { 0xab292861, 85 },
    { 0, 0 },
    { 0xe6592863, 34 },
    { 0xb292864, 98 },
    { 0xbc592864, 33 },
    { 0xc2192866, 33 },
    { 0x44a92866, 95 },
    { 0xb4492868, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc59286c, 98 },
    { 0xa889286c, 55 },
    { 0x4cb9286d, 34 },
    { 0x64e9286c, 90 },
    { 0x9a292870, 23 },
    { 0x62492871, 43 },
    { 0x39d92871, 90 },
    { 0, 0 },
    { 0x8e392874, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3292877, 80 },
    { 0x27d92878, 98 },
    { 0xd9892879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2092881, 23 },
    { 0, 0 },
    { 0x1d592883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1192886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc9288d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7792891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe192899, 93 },
    { 0xd589289a, 98 },
    { 0x8be9289a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce928a0, 100 },
    { 0, 0 },
    { 0x3d9928a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282928a9, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72928ad, 67 },
    { 0x59a928ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774928b2, 91 },
    { 0xf2e928b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x683928b6, 85 },
    { 0, 0 },
    { 0xa2d928b8, 69 },
    { 0xeaa928b9, 98 },
    { 0, 0 },
    { 0x447928bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6928bf, 23 },
    { 0xdea928c0, 95 },
    { 0x2c7928c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12928c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c928c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876928cf, 95 },
    { 0x1cc928d0, 61 },
    { 0, 0 },
    { 0x553928d2, 90 },
    { 0, 0 },
    { 0xaf0928d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e928d7, 33 },
    { 0xa32928d8, 23 },
    { 0xdad928d7, 61 },
    { 0x78d928da, 55 },
    { 0x2d928db, 95 },
    { 0, 0 },
    { 0x3d0928dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae928e2, 76 },
    { 0x841928e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a928e7, 84 },
    { 0xbc0928e8, 23 },
    { 0x477928e9, 90 },
    { 0x6a4928ea, 46 },
    { 0xe4928e9, 46 },
    { 0x2af928ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de928ef, 55 },
    { 0xd6d928f0, 61 },
    { 0xfd4928f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6928f6, 85 },
    { 0, 0 },
    { 0x593928f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b928fe, 33 },
    { 0x9d3928ff, 67 },
    { 0x2d592900, 61 },
    { 0x752928fe, 95 },
    { 0x46d92900, 18 },
    { 0x6f392903, 55 },
    { 0x8b492900, 98 },
    { 0, 0 },
    { 0xb3992906, 61 },
    { 0x54b92906, 95 },
    { 0, 0 },
    { 0x71192909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3839290c, 98 },
    { 0x8b9290d, 93 },
    { 0x2139290d, 90 },
    { 0, 0 },
    { 0xe1992910, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac92915, 33 },
    { 0xb6992916, 91 },
    { 0x26792916, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9291b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4492921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe592924, 23 },
    { 0x2f992925, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c392928, 98 },
    { 0, 0 },
    { 0x7d99292a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2239293d, 85 },
    { 0xd0e9293e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3992943, 67 },
    { 0x43a92943, 90 },
    { 0, 0 },
    { 0xf0792946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d69294d, 23 },
    { 0x9299294d, 81 },
    { 0, 0 },
    { 0x9d992950, 62 },
    { 0x74c92951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069295a, 38 },
    { 0x5059295b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc19295e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f92962, 61 },
    { 0xc6592962, 95 },
    { 0xbb92964, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39592967, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa9296a, 98 },
    { 0, 0 },
    { 0x119296c, 95 },
    { 0xe989296c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c092973, 61 },
    { 0x8ef92973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b92977, 76 },
    { 0, 0 },
    { 0x76892979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b92988, 61 },
    { 0xd9992989, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8639298d, 95 },
    { 0x4ed9298d, 90 },
    { 0x6ef9298f, 34 },
    { 0x1219298e, 33 },
    { 0x3b69298d, 98 },
    { 0xa492992, 99 },
    { 0, 0 },
    { 0xa3992994, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae492998, 99 },
    { 0xcbc92999, 34 },
    { 0x74f9299a, 61 },
    { 0, 0 },
    { 0x3589299c, 91 },
    { 0, 0 },
    { 0x1fe9299e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe929a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83929a7, 95 },
    { 0, 0 },
    { 0x6d7929a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d929ac, 95 },
    { 0xd16929ac, 23 },
    { 0x979929ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10929b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d929bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469929c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0929c9, 76 },
    { 0x5c929ca, 101 },
    { 0x2a929ca, 90 },
    { 0, 0 },
    { 0xd29929cd, 46 },
    { 0x93929ce, 98 },
    { 0, 0 },
    { 0xdbf929d0, 90 },
    { 0xc89929d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5929d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958929de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972929e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e929ea, 61 },
    { 0, 0 },
    { 0x9d0929ec, 95 },
    { 0x6b6929ed, 87 },
    { 0x35929ed, 98 },
    { 0, 0 },
    { 0x9d929f0, 90 },
    { 0x55f929f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138929f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2929fb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc992a05, 53 },
    { 0x1d692a06, 55 },
    { 0xbd992a07, 33 },
    { 0x5c292a08, 98 },
    { 0, 0 },
    { 0x3dc92a0a, 100 },
    { 0, 0 },
    { 0xf0192a0c, 85 },
    { 0, 0 },
    { 0x71792a0e, 76 },
    { 0, 0 },
    { 0x3bb92a10, 98 },
    { 0x66492a11, 100 },
    { 0x24992a12, 90 },
    { 0x27392a13, 33 },
    { 0x1f92a13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95792a1b, 33 },
    { 0x7a092a1c, 38 },
    { 0x25492a1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc892a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae792a2c, 100 },
    { 0x1b492a2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e92a36, 78 },
    { 0, 0 },
    { 0x96b92a38, 61 },
    { 0xa1a92a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb592a3d, 23 },
    { 0x1d192a3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea92a42, 95 },
    { 0x2b092a43, 77 },
    { 0x24992a42, 23 },
    { 0xea92a45, 66 },
    { 0x70692a43, 99 },
    { 0x15592a42, 99 },
    { 0, 0 },
    { 0x6d592a49, 76 },
    { 0x54192a49, 69 },
    { 0xba492a4b, 95 },
    { 0, 0 },
    { 0x97392a4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c592a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22992a58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c92a5d, 33 },
    { 0xc6492a5e, 95 },
    { 0x99b92a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c92a63, 78 },
    { 0, 0 },
    { 0x59a92a65, 46 },
    { 0x16192a66, 61 },
    { 0, 0 },
    { 0xcf192a68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a92a6b, 43 },
    { 0x26592a6c, 76 },
    { 0xfa592a6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092a71, 90 },
    { 0x17b92a72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7892a79, 84 },
    { 0, 0 },
    { 0x5fd92a7b, 23 },
    { 0x27992a7b, 43 },
    { 0xb1c92a7d, 33 },
    { 0xe7392a7e, 90 },
    { 0x7e792a7f, 46 },
    { 0, 0 },
    { 0x70692a81, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b492a85, 78 },
    { 0xd5992a85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc92a8d, 12 },
    { 0xad292a8e, 90 },
    { 0xf2992a8e, 100 },
    { 0xbc92a90, 76 },
    { 0, 0 },
    { 0x5ad92a92, 23 },
    { 0, 0 },
    { 0xa2b92a94, 33 },
    { 0x42d92a95, 55 },
    { 0xa4f92a96, 33 },
    { 0x3f492a96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72592a9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9792a9e, 91 },
    { 0, 0 },
    { 0x51592aa0, 17 },
    { 0xcf192aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d92aa4, 95 },
    { 0x24392aa5, 91 },
    { 0xcc492aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa92aaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x58892aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24192ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42992ab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef92ac0, 90 },
    { 0, 0 },
    { 0xbb692ac2, 90 },
    { 0x87f92ac3, 93 },
    { 0x73d92ac3, 100 },
    { 0xe8592ac5, 61 },
    { 0x87a92ac2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d92ac9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2392ad2, 68 },
    { 0, 0 },
    { 0x83f92ad4, 17 },
    { 0, 0 },
    { 0x47192ad6, 90 },
    { 0x70f92ad7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73792ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d692ae4, 55 },
    { 0x5b192ae5, 68 },
    { 0, 0 },
    { 0x72292ae7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x63692aea, 99 },
    { 0x3e092aeb, 60 },
    { 0x6192aeb, 23 },
    { 0x28d92aeb, 95 },
    { 0, 0 },
    { 0xca692aef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e092af2, 52 },
    { 0x92492af2, 61 },
    { 0x7db92af2, 93 },
    { 0x45792af5, 23 },
    { 0, 0 },
    { 0x82392af7, 95 },
    { 0x2c492af8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3192afb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe292aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2192b02, 65 },
    { 0, 0 },
    { 0x66b92b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b92b07, 88 },
    { 0x5b792b08, 52 },
    { 0x58e92b07, 81 },
    { 0xfee92b0a, 95 },
    { 0xfca92b0b, 55 },
    { 0, 0 },
    { 0x15392b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e92b10, 95 },
    { 0x7992b11, 100 },
    { 0x5e92b10, 55 },
    { 0, 0 },
    { 0xda092b14, 98 },
    { 0x2e792b15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e92b1c, 23 },
    { 0x60492b1d, 98 },
    { 0x85f92b1d, 95 },
    { 0xca392b1f, 95 },
    { 0x3e992b20, 78 },
    { 0xac492b21, 23 },
    { 0xfd192b1e, 69 },
    { 0x61892b23, 61 },
    { 0x84492b1c, 66 },
    { 0, 0 },
    { 0xea92b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f92b29, 76 },
    { 0, 0 },
    { 0x84392b2b, 67 },
    { 0x35092b2b, 33 },
    { 0x7c392b2d, 66 },
    { 0x9f292b2d, 77 },
    { 0xaf892b2e, 61 },
    { 0x7b392b2d, 90 },
    { 0x31092b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52792b34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe92b38, 34 },
    { 0x1e992b39, 33 },
    { 0x1092b39, 33 },
    { 0x81292b39, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d92b3e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb92b46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x54792b49, 90 },
    { 0x9bc92b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e192b4d, 23 },
    { 0xee492b4d, 66 },
    { 0x68c92b4f, 23 },
    { 0xca492b50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92b53, 66 },
    { 0, 0 },
    { 0x84592b55, 33 },
    { 0x69b92b55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c92b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e92b64, 90 },
    { 0x93892b65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c92b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa492b6f, 81 },
    { 0x55292b70, 85 },
    { 0x5d192b71, 90 },
    { 0, 0 },
    { 0x60b92b73, 90 },
    { 0x31592b73, 69 },
    { 0, 0 },
    { 0x83792b76, 76 },
    { 0xe7b92b76, 43 },
    { 0, 0 },
    { 0xad692b79, 100 },
    { 0x8f592b7a, 93 },
    { 0x9eb92b7b, 95 },
    { 0, 0 },
    { 0x4ca92b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6692b85, 84 },
    { 0xc1292b86, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf892b8a, 78 },
    { 0x5a692b8b, 93 },
    { 0, 0 },
    { 0x64592b8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7892b94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d92b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13892b9d, 23 },
    { 0x97f92b9d, 98 },
    { 0x49f92b9f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b92ba5, 66 },
    { 0x77092ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a792baa, 90 },
    { 0x18892bab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61592bb0, 61 },
    { 0x92492bb1, 95 },
    { 0x15e92bb2, 34 },
    { 0, 0 },
    { 0xbed92bb4, 61 },
    { 0x26892bb4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9292bb9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1892bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8292bbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7292bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9292bcc, 87 },
    { 0, 0 },
    { 0x7a592bce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b92bdb, 33 },
    { 0xe6c92bdb, 61 },
    { 0x1f92bdd, 98 },
    { 0, 0 },
    { 0x5aa92bdf, 61 },
    { 0xd1692bdf, 98 },
    { 0, 0 },
    { 0xe2f92be2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb92be5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x90492be8, 33 },
    { 0x67c92be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa092bed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f92bf0, 95 },
    { 0x84c92bf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2692bf4, 61 },
    { 0xb392bf5, 61 },
    { 0xcae92bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e92bfd, 61 },
    { 0xf0e92bfe, 52 },
    { 0x47392bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec92c04, 87 },
    { 0x4c292c04, 95 },
    { 0x5ae92c06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c92c09, 58 },
    { 0x1a292c0a, 95 },
    { 0xa1e92c09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e92c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e092c13, 95 },
    { 0, 0 },
    { 0xb1392c15, 84 },
    { 0x4cb92c16, 90 },
    { 0x93c92c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36792c1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61792c26, 90 },
    { 0xef592c27, 90 },
    { 0x80192c28, 99 },
    { 0, 0 },
    { 0x34492c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2192c2e, 40 },
    { 0xf7592c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd92c37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17092c3c, 90 },
    { 0xe6292c3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5292c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c92c4a, 83 },
    { 0xd6f92c4b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33392c50, 98 },
    { 0x3c292c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b92c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7192c6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd392c71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d592c75, 61 },
    { 0, 0 },
    { 0xb92c77, 88 },
    { 0xe9692c78, 66 },
    { 0xb3792c79, 83 },
    { 0x21392c79, 76 },
    { 0, 0 },
    { 0x65192c7c, 90 },
    { 0xe4492c7d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b92c80, 78 },
    { 0xdaf92c81, 67 },
    { 0x38792c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec092c89, 66 },
    { 0, 0 },
    { 0xfb992c8b, 22 },
    { 0x7e092c8c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e92c90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b92c93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10892c97, 93 },
    { 0x7bf92c98, 90 },
    { 0, 0 },
    { 0xcb392c9a, 76 },
    { 0, 0 },
    { 0x17a92c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f92ca3, 23 },
    { 0, 0 },
    { 0x46392ca5, 53 },
    { 0x69f92ca6, 95 },
    { 0x3f492ca7, 81 },
    { 0xd8992ca8, 98 },
    { 0xadb92ca9, 95 },
    { 0xa0292ca7, 98 },
    { 0x11b92ca7, 23 },
    { 0xd4592ca6, 96 },
    { 0x77f92ca6, 85 },
    { 0x5fb92cae, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f92cb2, 17 },
    { 0x69a92cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5792cb7, 77 },
    { 0xc6c92cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e92cbb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88692ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78492cd0, 34 },
    { 0x64b92cd1, 87 },
    { 0xd2992cd0, 75 },
    { 0xcae92cd3, 33 },
    { 0xe9192cd4, 33 },
    { 0x12092cd2, 46 },
    { 0x35292cd6, 61 },
    { 0xa4992cd7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3192ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b992ce8, 90 },
    { 0, 0 },
    { 0x61f92cea, 76 },
    { 0, 0 },
    { 0x19492cec, 84 },
    { 0xc0792cec, 33 },
    { 0, 0 },
    { 0xdad92cef, 95 },
    { 0x23592cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36892cf4, 90 },
    { 0, 0 },
    { 0x72b92cf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f292cfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0392d09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc492d0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57992d12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34092d1b, 95 },
    { 0x11192d1c, 90 },
    { 0xd2992d1d, 90 },
    { 0x92c92d1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b92d22, 90 },
    { 0x9d692d23, 69 },
    { 0, 0 },
    { 0xede92d25, 23 },
    { 0xacf92d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde92d29, 90 },
    { 0x5a292d2a, 76 },
    { 0, 0 },
    { 0x84292d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3792d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4692d32, 33 },
    { 0, 0 },
    { 0x69a92d34, 61 },
    { 0x29d92d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63292d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b92d3c, 98 },
    { 0x5fa92d3d, 90 },
    { 0, 0 },
    { 0x78492d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2692d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42892d47, 23 },
    { 0x5ab92d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f892d4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce892d4f, 88 },
    { 0x41292d50, 85 },
    { 0x40192d51, 98 },
    { 0x9f292d52, 95 },
    { 0x3aa92d53, 98 },
    { 0x60e92d53, 90 },
    { 0x10e92d51, 33 },
    { 0xc2c92d56, 76 },
    { 0xad092d57, 53 },
    { 0x7a692d57, 66 },
    { 0x1b092d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb92d5c, 90 },
    { 0x18c92d5d, 90 },
    { 0, 0 },
    { 0x95992d5f, 55 },
    { 0, 0 },
    { 0x65f92d61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da92d64, 61 },
    { 0x85192d65, 90 },
    { 0x1092d65, 34 },
    { 0x2ff92d66, 98 },
    { 0xb1a92d68, 87 },
    { 0, 0 },
    { 0x8d92d6a, 76 },
    { 0x73392d6a, 33 },
    { 0x78a92d6a, 33 },
    { 0x1692d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd92d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d92d76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff892d79, 98 },
    { 0xbb92d7a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb392d81, 61 },
    { 0x8b892d82, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x26892d85, 93 },
    { 0xbd492d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda992d8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e092d93, 52 },
    { 0xe592d94, 95 },
    { 0xcd092d95, 23 },
    { 0xd2c92d94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f392d9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4492d9e, 33 },
    { 0xd1992d9f, 49 },
    { 0xc9492d9e, 90 },
    { 0x55a92da0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78192da8, 90 },
    { 0x23492da8, 33 },
    { 0xe8292daa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef092dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6492db1, 83 },
    { 0x62692db1, 93 },
    { 0x4d392db3, 87 },
    { 0xd1892db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c92db7, 55 },
    { 0x34992db8, 90 },
    { 0xf8f92db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac92dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2592dc6, 33 },
    { 0x17792dc7, 46 },
    { 0x59692dc8, 55 },
    { 0x4ad92dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17692dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb192dd1, 55 },
    { 0xd0b92dd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e92dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74492ddc, 95 },
    { 0xea192ddd, 98 },
    { 0xf8d92dde, 55 },
    { 0xb2692ddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb92de3, 98 },
    { 0xdbb92de4, 61 },
    { 0x7ca92de4, 78 },
    { 0, 0 },
    { 0x73792de7, 33 },
    { 0xb292de8, 93 },
    { 0xbc692de8, 87 },
    { 0x47592de9, 87 },
    { 0, 0 },
    { 0xf6392dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0692def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a92df3, 61 },
    { 0xb9f92df3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95192df7, 61 },
    { 0, 0 },
    { 0x67292df9, 95 },
    { 0x63d92dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f92dfe, 90 },
    { 0, 0 },
    { 0x56992e00, 90 },
    { 0, 0 },
    { 0xdee92e02, 90 },
    { 0, 0 },
    { 0xe5192e04, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc992e09, 34 },
    { 0x2b892e0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec92e0e, 76 },
    { 0xf7492e0f, 46 },
    { 0xf7692e10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77992e13, 61 },
    { 0xa9b92e14, 90 },
    { 0x29492e14, 90 },
    { 0x17492e15, 91 },
    { 0x35e92e17, 33 },
    { 0xe9492e16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f92e1b, 66 },
    { 0, 0 },
    { 0xddd92e1d, 95 },
    { 0xd992e1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab292e22, 61 },
    { 0x9bb92e22, 85 },
    { 0x3d992e24, 85 },
    { 0xeb692e23, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed892e28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85292e2c, 52 },
    { 0, 0 },
    { 0x4a992e2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a092e37, 61 },
    { 0x8b792e37, 62 },
    { 0x86d92e39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b92e3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84792e42, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13292e45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c92e48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0792e50, 90 },
    { 0xa6b92e50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e92e54, 98 },
    { 0xb5692e54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be92e5f, 90 },
    { 0x2ea92e60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692e63, 69 },
    { 0x5d692e63, 76 },
    { 0, 0 },
    { 0xbec92e66, 57 },
    { 0, 0 },
    { 0x41292e68, 76 },
    { 0xe1592e69, 87 },
    { 0, 0 },
    { 0x4ad92e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8992e74, 100 },
    { 0, 0 },
    { 0x62492e76, 95 },
    { 0, 0 },
    { 0xd5892e78, 98 },
    { 0x8a192e79, 98 },
    { 0x8e792e79, 76 },
    { 0x20d92e79, 95 },
    { 0x68c92e7c, 23 },
    { 0x5292e7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e92e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5992e89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66892e8e, 33 },
    { 0, 0 },
    { 0x62692e90, 23 },
    { 0x77892e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d692e96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd92e99, 61 },
    { 0x90792e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67992e9e, 98 },
    { 0x61892e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e92ea3, 61 },
    { 0xf1c92ea4, 55 },
    { 0x75092ea5, 61 },
    { 0x2e792ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6592eaa, 77 },
    { 0xcf892eaa, 75 },
    { 0xf392eaa, 69 },
    { 0x21292ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc92eb3, 33 },
    { 0x3b692eb4, 75 },
    { 0, 0 },
    { 0x30692eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd992ebe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51892ec7, 77 },
    { 0x49592ec8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e92ecf, 90 },
    { 0xbb92ecf, 91 },
    { 0x31192ed1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3892ed6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41692eda, 55 },
    { 0x19a92edb, 98 },
    { 0x56392edc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92ee1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c792ee5, 98 },
    { 0x7cf92ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f92eea, 98 },
    { 0x4c092eeb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73692ef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f92ef4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef92ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56792efa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d92eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed92f04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1892f08, 95 },
    { 0, 0 },
    { 0x82092f0a, 61 },
    { 0, 0 },
    { 0x7b192f0c, 46 },
    { 0x38a92f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee692f11, 90 },
    { 0xcd492f11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a092f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53392f20, 67 },
    { 0, 0 },
    { 0xf6c92f22, 76 },
    { 0, 0 },
    { 0x79492f24, 40 },
    { 0x7f792f25, 90 },
    { 0, 0 },
    { 0x9dd92f27, 33 },
    { 0x4cc92f27, 33 },
    { 0, 0 },
    { 0x9d992f2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d92f2d, 23 },
    { 0, 0 },
    { 0x93792f2f, 90 },
    { 0, 0 },
    { 0x4f892f31, 23 },
    { 0xc0092f32, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea492f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae92f46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b992f49, 23 },
    { 0, 0 },
    { 0x192f4b, 23 },
    { 0x6292f4c, 33 },
    { 0, 0 },
    { 0xb5c92f4e, 66 },
    { 0, 0 },
    { 0xd7e92f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21792f55, 33 },
    { 0x30192f56, 33 },
    { 0, 0 },
    { 0x8b692f58, 98 },
    { 0xfde92f58, 101 },
    { 0x69692f58, 76 },
    { 0, 0 },
    { 0xbeb92f5c, 23 },
    { 0x27492f5c, 77 },
    { 0x60f92f5d, 90 },
    { 0x93392f5c, 34 },
    { 0xaf792f60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e292f66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93492f6e, 52 },
    { 0xc5892f6f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd692f73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f692f76, 98 },
    { 0x7ac92f77, 75 },
    { 0xd5292f78, 98 },
    { 0x86892f76, 93 },
    { 0xafd92f7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9692f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0192f86, 65 },
    { 0, 0 },
    { 0xeb492f88, 90 },
    { 0xfed92f88, 81 },
    { 0x78f92f88, 82 },
    { 0xdb992f8a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf992f95, 52 },
    { 0, 0 },
    { 0x49392f97, 76 },
    { 0x9de92f98, 34 },
    { 0x16392f99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0792f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1992f9f, 91 },
    { 0xca92fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d92fa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a92fad, 77 },
    { 0xd8592fae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd92fb1, 33 },
    { 0x70a92fb1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf92fb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1192fc0, 100 },
    { 0, 0 },
    { 0x4fe92fc2, 61 },
    { 0x67a92fc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f92fd3, 78 },
    { 0xf2492fd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a92fd7, 52 },
    { 0x6e992fd8, 23 },
    { 0, 0 },
    { 0x73492fda, 34 },
    { 0x34892fdb, 65 },
    { 0, 0 },
    { 0xf0b92fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78892fe2, 55 },
    { 0x8f392fe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b092fe6, 95 },
    { 0xc8592fe7, 33 },
    { 0x7092fe6, 61 },
    { 0x9bd92fe9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce92ff6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x26392ff9, 76 },
    { 0xe9392ffa, 61 },
    { 0xd1192ffb, 95 },
    { 0xa3292ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c92fff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea93002, 90 },
    { 0xccf93003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf679300a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d9300d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1093010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf593017, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579301c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6789301f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f693023, 61 },
    { 0x4a593024, 52 },
    { 0x74693025, 98 },
    { 0x25493026, 33 },
    { 0x2d493027, 34 },
    { 0x20a93028, 95 },
    { 0xfd593029, 23 },
    { 0xc5093025, 33 },
    { 0xf979302b, 95 },
    { 0x31393026, 77 },
    { 0xc719302a, 61 },
    { 0xeba9302e, 90 },
    { 0x6de93024, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97993033, 33 },
    { 0, 0 },
    { 0x55493035, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89593038, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939303b, 80 },
    { 0, 0 },
    { 0x6d69303d, 33 },
    { 0xe7c9303d, 66 },
    { 0, 0 },
    { 0x4b993040, 84 },
    { 0x23293041, 76 },
    { 0x9cc93042, 90 },
    { 0xa0993041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993048, 33 },
    { 0, 0 },
    { 0xda9304a, 99 },
    { 0, 0 },
    { 0x4bf9304c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f93050, 90 },
    { 0xdc93051, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e93056, 33 },
    { 0xa7293057, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9305b, 90 },
    { 0xe999305c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc893061, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8893066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789306b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d9307d, 46 },
    { 0, 0 },
    { 0xa089307f, 98 },
    { 0x74a93080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c93083, 34 },
    { 0x34593084, 33 },
    { 0x1b593083, 34 },
    { 0x63793086, 98 },
    { 0xf6893087, 66 },
    { 0x4dd93087, 99 },
    { 0x1bf93088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a293095, 98 },
    { 0xfb793096, 55 },
    { 0, 0 },
    { 0xd5c93098, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6229309b, 23 },
    { 0xa59309c, 33 },
    { 0, 0 },
    { 0xed19309e, 98 },
    { 0x9799309f, 90 },
    { 0x489930a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21930ac, 90 },
    { 0, 0 },
    { 0xdec930ae, 95 },
    { 0x3ab930af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c930b3, 95 },
    { 0xd99930b3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa930b8, 34 },
    { 0xe00930b9, 33 },
    { 0x778930b9, 95 },
    { 0, 0 },
    { 0xd7b930bc, 90 },
    { 0x7b1930bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d930c0, 75 },
    { 0xa86930c1, 90 },
    { 0, 0 },
    { 0x97f930c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3930c6, 75 },
    { 0xc97930c7, 98 },
    { 0xae0930c7, 95 },
    { 0x38d930c7, 95 },
    { 0xb0930ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0930d3, 70 },
    { 0xc9f930d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d930d9, 93 },
    { 0, 0 },
    { 0x7ad930db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b930de, 61 },
    { 0, 0 },
    { 0x2df930e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71930e5, 55 },
    { 0x8c8930e6, 61 },
    { 0xb63930e6, 33 },
    { 0xda6930e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7930eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x255930ee, 67 },
    { 0x311930ef, 90 },
    { 0, 0 },
    { 0x36f930f1, 99 },
    { 0xeca930f2, 84 },
    { 0, 0 },
    { 0xdeb930f4, 61 },
    { 0, 0 },
    { 0x6a2930f6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac930fa, 90 },
    { 0xe60930fb, 91 },
    { 0x4f2930fb, 43 },
    { 0x31d930fb, 66 },
    { 0xc930fe, 33 },
    { 0, 0 },
    { 0x83093100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f493105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f293109, 98 },
    { 0xd279310a, 67 },
    { 0x2d19310a, 17 },
    { 0x60e9310b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e93110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe993120, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17293127, 33 },
    { 0, 0 },
    { 0x90c93129, 15 },
    { 0xdc19312a, 98 },
    { 0xae69312b, 23 },
    { 0, 0 },
    { 0x5599312d, 23 },
    { 0xe549312e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f93131, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2493137, 55 },
    { 0x5a093138, 66 },
    { 0x95093137, 90 },
    { 0x5079313a, 33 },
    { 0, 0 },
    { 0x9a89313c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd169313f, 99 },
    { 0x4029313f, 61 },
    { 0x4c193140, 61 },
    { 0x57f93142, 81 },
    { 0x8de93141, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab93147, 55 },
    { 0x16893148, 61 },
    { 0x73593148, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c9314c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9315b, 66 },
    { 0x2bc9315c, 67 },
    { 0, 0 },
    { 0xb3e9315e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9293164, 61 },
    { 0x40493164, 43 },
    { 0xce593165, 93 },
    { 0, 0 },
    { 0xc4d93168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c79316b, 61 },
    { 0x2449316b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a593171, 95 },
    { 0x79d93172, 55 },
    { 0x19093173, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f693177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f89317f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65693182, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5293187, 95 },
    { 0x27a93188, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819318c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e9318f, 98 },
    { 0x8db9318f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27393194, 23 },
    { 0, 0 },
    { 0x71293196, 85 },
    { 0x3f893197, 34 },
    { 0, 0 },
    { 0x8da93199, 85 },
    { 0xf2693199, 95 },
    { 0x82b9319b, 61 },
    { 0xaa69319b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5931a0, 98 },
    { 0xd06931a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff931a5, 90 },
    { 0, 0 },
    { 0xd2b931a7, 90 },
    { 0x28d931a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab931ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75931ae, 55 },
    { 0, 0 },
    { 0x2c9931b0, 95 },
    { 0xbc9931b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c931b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24931b7, 43 },
    { 0x590931b8, 95 },
    { 0, 0 },
    { 0xd86931ba, 33 },
    { 0x6e931bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49931bf, 34 },
    { 0, 0 },
    { 0x30e931c1, 90 },
    { 0, 0 },
    { 0xd45931c3, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9931c6, 43 },
    { 0xee931c7, 33 },
    { 0x443931c7, 55 },
    { 0x278931c6, 90 },
    { 0x78f931ca, 33 },
    { 0x8a2931cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x819931ce, 80 },
    { 0, 0 },
    { 0xbbe931d0, 85 },
    { 0, 0 },
    { 0xd1c931d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee931df, 33 },
    { 0x8c8931df, 90 },
    { 0x753931e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x644931e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c931e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3931ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a931f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c931f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb931f9, 23 },
    { 0x49d931fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3931ff, 88 },
    { 0x2a593200, 90 },
    { 0x5a693201, 88 },
    { 0, 0 },
    { 0x8b593203, 77 },
    { 0xb3993203, 95 },
    { 0x16593204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9320b, 33 },
    { 0, 0 },
    { 0x1099320d, 87 },
    { 0x7e19320d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2929321a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3993223, 91 },
    { 0, 0 },
    { 0xa6593225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74493229, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19322c, 90 },
    { 0, 0 },
    { 0x6219322e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b93233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a9323d, 90 },
    { 0x1e39323e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4393241, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0293245, 66 },
    { 0, 0 },
    { 0x9c193247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b19324c, 33 },
    { 0x4679324d, 55 },
    { 0x2699324e, 95 },
    { 0xe169324f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef93254, 90 },
    { 0xf1093254, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x12993258, 96 },
    { 0x14793259, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd9325c, 98 },
    { 0x2829325c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3393261, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2093264, 98 },
    { 0x4ab93265, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df93268, 61 },
    { 0xd2d93269, 90 },
    { 0xd4993269, 61 },
    { 0xd649326b, 61 },
    { 0, 0 },
    { 0x84d9326d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a93278, 78 },
    { 0x4d293279, 61 },
    { 0x69c93278, 95 },
    { 0x4ae9327b, 98 },
    { 0, 0 },
    { 0xed49327d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b93281, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169328a, 67 },
    { 0x6c19328b, 66 },
    { 0, 0 },
    { 0x4a9328d, 55 },
    { 0xbf69328e, 34 },
    { 0xabf9328d, 35 },
    { 0, 0 },
    { 0xbe293291, 17 },
    { 0, 0 },
    { 0x73793293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea993299, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91932a1, 90 },
    { 0xad7932a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a932a8, 95 },
    { 0x1a1932a9, 34 },
    { 0, 0 },
    { 0x64d932ab, 76 },
    { 0, 0 },
    { 0xa84932ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b932b0, 95 },
    { 0, 0 },
    { 0x7c0932b2, 98 },
    { 0xd1f932b3, 77 },
    { 0x11d932b3, 23 },
    { 0x1e0932b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13932b8, 34 },
    { 0, 0 },
    { 0xa7c932ba, 76 },
    { 0xf39932bb, 98 },
    { 0x172932bc, 46 },
    { 0xbf932bd, 98 },
    { 0x14b932bc, 98 },
    { 0, 0 },
    { 0xecf932c0, 23 },
    { 0xe6d932c1, 76 },
    { 0xbe6932c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3932c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9932cb, 95 },
    { 0xee9932cb, 77 },
    { 0xed0932cd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb932d0, 61 },
    { 0x397932d0, 56 },
    { 0, 0 },
    { 0x37a932d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61932d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x769932d9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f932dc, 23 },
    { 0x15d932dd, 33 },
    { 0, 0 },
    { 0xfe1932df, 34 },
    { 0, 0 },
    { 0x787932e1, 80 },
    { 0x5f7932e2, 66 },
    { 0x9e2932e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179932ea, 76 },
    { 0, 0 },
    { 0x38e932ec, 23 },
    { 0, 0 },
    { 0x696932ee, 90 },
    { 0x567932ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1932f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3932f6, 34 },
    { 0xde1932f7, 61 },
    { 0xe75932f8, 33 },
    { 0x415932f9, 100 },
    { 0x1d932fa, 52 },
    { 0, 0 },
    { 0x993932fc, 99 },
    { 0xaaf932fd, 100 },
    { 0x637932fe, 33 },
    { 0x4a7932fc, 34 },
    { 0x322932ff, 66 },
    { 0, 0 },
    { 0x50193302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f93306, 52 },
    { 0, 0 },
    { 0xbc593308, 76 },
    { 0xaf493309, 95 },
    { 0, 0 },
    { 0xc9f9330b, 98 },
    { 0xf3e9330b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a93310, 90 },
    { 0, 0 },
    { 0x73793312, 43 },
    { 0xeca93313, 43 },
    { 0xf1293314, 90 },
    { 0x59293315, 34 },
    { 0, 0 },
    { 0x37793317, 91 },
    { 0xf8293318, 33 },
    { 0xead93319, 70 },
    { 0xe9b9331a, 90 },
    { 0, 0 },
    { 0xf849331c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24393324, 98 },
    { 0x72393324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769332d, 43 },
    { 0xba59332e, 55 },
    { 0x3c79332f, 55 },
    { 0, 0 },
    { 0x42193331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7093335, 23 },
    { 0, 0 },
    { 0xd5593337, 98 },
    { 0x4ba93337, 90 },
    { 0xd293337, 90 },
    { 0x53f9333a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77293340, 99 },
    { 0, 0 },
    { 0x73893342, 61 },
    { 0, 0 },
    { 0x90f93344, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x94593347, 90 },
    { 0, 0 },
    { 0x90493349, 90 },
    { 0xefc9334a, 90 },
    { 0xd369334b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e9334e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a993351, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a793357, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb29335b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf339335e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc293366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d93369, 95 },
    { 0, 0 },
    { 0xc009336b, 88 },
    { 0x5589336c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d93372, 9 },
    { 0xb8e93373, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8493379, 40 },
    { 0x85093379, 95 },
    { 0, 0 },
    { 0x8879337c, 43 },
    { 0, 0 },
    { 0xa1e9337e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a93383, 82 },
    { 0x5f293384, 43 },
    { 0x1e693384, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa439338f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b93394, 95 },
    { 0xff893394, 89 },
    { 0x9293396, 33 },
    { 0x1c793396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a9339d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744933a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa933a8, 61 },
    { 0x752933a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e933b0, 78 },
    { 0xcf933b0, 90 },
    { 0xde3933b2, 95 },
    { 0x944933b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c933bd, 90 },
    { 0, 0 },
    { 0x6ea933bf, 75 },
    { 0, 0 },
    { 0x79c933c1, 95 },
    { 0, 0 },
    { 0x384933c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8933c6, 38 },
    { 0x9ee933c7, 55 },
    { 0x8c0933c8, 76 },
    { 0xe933c7, 33 },
    { 0xf78933ca, 61 },
    { 0x17e933c8, 18 },
    { 0x929933cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4933d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693933dd, 46 },
    { 0xa4b933de, 88 },
    { 0, 0 },
    { 0x39f933e0, 90 },
    { 0, 0 },
    { 0x84e933e2, 69 },
    { 0xeb3933e3, 76 },
    { 0, 0 },
    { 0x545933e5, 33 },
    { 0, 0 },
    { 0x1a7933e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c933ef, 90 },
    { 0xc53933f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4933f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2933f9, 55 },
    { 0xc6d933f9, 76 },
    { 0xdd6933fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf993403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93407, 85 },
    { 0x8da93408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4529340c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb9340f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e093415, 33 },
    { 0x72d93416, 67 },
    { 0, 0 },
    { 0x71593418, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe779341f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff93423, 90 },
    { 0x47193424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd969342d, 85 },
    { 0xda09342e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e93432, 66 },
    { 0x4fb93432, 40 },
    { 0x2f293434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb393437, 52 },
    { 0x4ff93438, 76 },
    { 0x34d93439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e9343c, 98 },
    { 0x3389343c, 55 },
    { 0xfbf9343c, 98 },
    { 0xd479343f, 33 },
    { 0xf2e9343f, 76 },
    { 0x49493440, 33 },
    { 0xff493442, 61 },
    { 0x86e93442, 100 },
    { 0xf1a93443, 23 },
    { 0x9c493443, 38 },
    { 0xe293446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb9344e, 93 },
    { 0x9c9344f, 93 },
    { 0x2349344f, 61 },
    { 0x3f693451, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c89345d, 95 },
    { 0x9ec9345d, 33 },
    { 0xd3e9345f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0293465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7709346a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4779346d, 43 },
    { 0xf4f9346e, 90 },
    { 0xc579346f, 66 },
    { 0xc4193470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569347b, 42 },
    { 0xc9d9347b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6299347f, 98 },
    { 0x81993480, 95 },
    { 0x54b93481, 91 },
    { 0x30e93480, 61 },
    { 0x96093482, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc93487, 95 },
    { 0x35593488, 34 },
    { 0xa8a93488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad9348e, 55 },
    { 0x1869348f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b293495, 90 },
    { 0, 0 },
    { 0x81693497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539349c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f934a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461934ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6934b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa934b7, 34 },
    { 0, 0 },
    { 0xc38934b9, 76 },
    { 0x2e3934b9, 33 },
    { 0x922934b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5934c3, 61 },
    { 0, 0 },
    { 0xc93934c5, 43 },
    { 0x3ef934c6, 61 },
    { 0, 0 },
    { 0x4a934c8, 93 },
    { 0x536934c9, 90 },
    { 0x4a0934ca, 34 },
    { 0, 0 },
    { 0x4ea934cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c934cf, 75 },
    { 0, 0 },
    { 0xb1e934d1, 91 },
    { 0, 0 },
    { 0x34d934d3, 95 },
    { 0, 0 },
    { 0x6d934d5, 90 },
    { 0x46d934d6, 34 },
    { 0xf98934d7, 98 },
    { 0xe2b934d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea934db, 61 },
    { 0xf33934db, 61 },
    { 0, 0 },
    { 0x785934de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c934e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4934e5, 38 },
    { 0, 0 },
    { 0x8e3934e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c934ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa934ef, 98 },
    { 0x7b0934f0, 90 },
    { 0xc8934f1, 36 },
    { 0, 0 },
    { 0xdb5934f3, 52 },
    { 0, 0 },
    { 0x90934f5, 64 },
    { 0x5e934f6, 46 },
    { 0, 0 },
    { 0xe04934f8, 52 },
    { 0, 0 },
    { 0xff8934fa, 61 },
    { 0, 0 },
    { 0xe0e934fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6293509, 99 },
    { 0xd39350a, 33 },
    { 0x2979350b, 98 },
    { 0x8ae9350c, 91 },
    { 0xcd39350c, 46 },
    { 0x7af9350d, 95 },
    { 0x9869350a, 23 },
    { 0x8df93509, 76 },
    { 0xfd79350c, 95 },
    { 0x2dd93509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42793515, 95 },
    { 0x63b93516, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b93519, 52 },
    { 0, 0 },
    { 0x9299351b, 55 },
    { 0x3709351c, 76 },
    { 0x7e69351c, 90 },
    { 0, 0 },
    { 0x6b69351f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8293522, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4293527, 36 },
    { 0, 0 },
    { 0xb8e93529, 90 },
    { 0x7ee93529, 64 },
    { 0xfbb9352b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54793531, 95 },
    { 0xa1e93532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f193535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949353b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb329353e, 90 },
    { 0x5659353f, 93 },
    { 0xa79353e, 61 },
    { 0, 0 },
    { 0xa0393542, 81 },
    { 0xe8e93542, 33 },
    { 0xf5893544, 76 },
    { 0x86393544, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b193548, 46 },
    { 0x5f193548, 43 },
    { 0x6fd93548, 43 },
    { 0, 0 },
    { 0x2f49354c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13893551, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50393559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6999355f, 61 },
    { 0xb9393560, 98 },
    { 0xcb193561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27793567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e9356a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4593578, 33 },
    { 0xc3d93578, 34 },
    { 0xddf93579, 55 },
    { 0x8ad9357b, 95 },
    { 0x53b9357b, 34 },
    { 0x6c99357b, 85 },
    { 0x2539357c, 33 },
    { 0xb409357f, 78 },
    { 0x79a9357f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec93586, 95 },
    { 0xbe993587, 90 },
    { 0xac93588, 98 },
    { 0xd893589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd9358e, 84 },
    { 0, 0 },
    { 0x53393590, 90 },
    { 0, 0 },
    { 0x5b193592, 55 },
    { 0, 0 },
    { 0x1a293594, 92 },
    { 0, 0 },
    { 0xc3f93596, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19359a, 55 },
    { 0xb489359b, 90 },
    { 0xa739359b, 33 },
    { 0x5109359d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xede935a0, 98 },
    { 0x6e7935a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce935a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a935ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89935b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee935be, 76 },
    { 0x703935bf, 76 },
    { 0, 0 },
    { 0xa70935c1, 95 },
    { 0xb5b935c1, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8935c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1935ca, 90 },
    { 0x663935ca, 33 },
    { 0xfb7935ca, 33 },
    { 0x21e935cd, 61 },
    { 0xd72935cd, 100 },
    { 0x92935cf, 91 },
    { 0xbae935cd, 76 },
    { 0x4b3935d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x327935d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd935dd, 61 },
    { 0, 0 },
    { 0xa48935df, 66 },
    { 0x459935e0, 61 },
    { 0x9d8935e0, 65 },
    { 0, 0 },
    { 0xb55935e3, 59 },
    { 0, 0 },
    { 0x52f935e5, 95 },
    { 0x59d935e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa935ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x493935f2, 43 },
    { 0, 0 },
    { 0x735935f4, 90 },
    { 0xdf9935f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4935f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b935fd, 90 },
    { 0, 0 },
    { 0xb20935ff, 90 },
    { 0x54c93600, 90 },
    { 0xc2493601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b93607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299360a, 55 },
    { 0, 0 },
    { 0x6699360c, 93 },
    { 0x2209360d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a93613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3049361e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5693622, 100 },
    { 0, 0 },
    { 0xca193624, 90 },
    { 0, 0 },
    { 0xfb093626, 43 },
    { 0xe5a93627, 23 },
    { 0xcc693628, 84 },
    { 0xe93627, 95 },
    { 0x4189362a, 98 },
    { 0x5929362b, 98 },
    { 0x28a9362a, 33 },
    { 0x1a69362d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e193630, 76 },
    { 0xe1893631, 87 },
    { 0xa2593632, 98 },
    { 0x98893630, 23 },
    { 0x5e393634, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d593637, 23 },
    { 0x23393638, 66 },
    { 0xb2193638, 43 },
    { 0x97e93637, 33 },
    { 0xef93639, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd59363e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20293642, 93 },
    { 0xeb993642, 33 },
    { 0x71793644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a93647, 17 },
    { 0xddc93648, 61 },
    { 0xb2f93647, 95 },
    { 0x2889364a, 99 },
    { 0, 0 },
    { 0x4439364c, 77 },
    { 0x6159364d, 90 },
    { 0x5169364e, 90 },
    { 0x2de9364e, 43 },
    { 0xa1e9364e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c293656, 55 },
    { 0, 0 },
    { 0x22493658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c9365b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d93662, 61 },
    { 0xd2093662, 19 },
    { 0, 0 },
    { 0x12593665, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5193668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59366e, 95 },
    { 0x2f9366e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89993674, 93 },
    { 0, 0 },
    { 0x57593676, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f9367a, 90 },
    { 0xba9367b, 47 },
    { 0x1519367b, 98 },
    { 0, 0 },
    { 0x25f9367e, 55 },
    { 0xa789367f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1593683, 55 },
    { 0xc3f93683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec93687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a39368a, 95 },
    { 0x58c9368b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b9368e, 67 },
    { 0x5a79368f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a193694, 52 },
    { 0x85e93694, 23 },
    { 0xfa93695, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319936ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd936ae, 90 },
    { 0x7a5936af, 91 },
    { 0, 0 },
    { 0xec0936b1, 90 },
    { 0xca4936b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806936ba, 93 },
    { 0, 0 },
    { 0xb19936bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a936c3, 93 },
    { 0x7c2936c4, 33 },
    { 0x548936c5, 76 },
    { 0x728936c4, 23 },
    { 0x42936c7, 95 },
    { 0xb28936c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e936cc, 84 },
    { 0x378936cd, 76 },
    { 0xa0b936ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0936d8, 55 },
    { 0x7e2936d9, 95 },
    { 0x1a1936da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc936dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb936e1, 55 },
    { 0x8a9936e1, 90 },
    { 0, 0 },
    { 0xf8d936e4, 66 },
    { 0x1b7936e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d936e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a936ee, 90 },
    { 0x55b936ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35936f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x192936f5, 76 },
    { 0x360936f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73936fa, 33 },
    { 0x888936fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39493709, 33 },
    { 0xd079370a, 76 },
    { 0xb729370b, 76 },
    { 0xc2c9370c, 99 },
    { 0x819370d, 52 },
    { 0xed89370e, 61 },
    { 0, 0 },
    { 0x52b93710, 77 },
    { 0x7e093710, 33 },
    { 0x7d793712, 55 },
    { 0x1893712, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b93716, 33 },
    { 0xfbd93717, 68 },
    { 0xd5c93717, 91 },
    { 0xf4e93719, 90 },
    { 0, 0 },
    { 0x2b49371b, 46 },
    { 0, 0 },
    { 0x3d99371d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d93720, 34 },
    { 0x9fb93720, 90 },
    { 0, 0 },
    { 0xde493723, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc093727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599372e, 66 },
    { 0x7519372f, 90 },
    { 0x7f793730, 23 },
    { 0xf9593731, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x39193734, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22993737, 90 },
    { 0xd9693738, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e9373c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af93743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e93746, 91 },
    { 0, 0 },
    { 0xc6593748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129374b, 90 },
    { 0, 0 },
    { 0x8779374d, 98 },
    { 0, 0 },
    { 0xdaa9374f, 61 },
    { 0x20293750, 100 },
    { 0, 0 },
    { 0x8ef93752, 95 },
    { 0, 0 },
    { 0xc5793754, 56 },
    { 0xfa693754, 98 },
    { 0x13893756, 90 },
    { 0x2fc93757, 98 },
    { 0x48393757, 90 },
    { 0xc4993757, 43 },
    { 0x54e9375a, 33 },
    { 0x2989375b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7493766, 76 },
    { 0xd4993767, 95 },
    { 0x3cf93767, 61 },
    { 0xe393769, 46 },
    { 0x59d93766, 98 },
    { 0x3039376b, 75 },
    { 0x46793769, 76 },
    { 0x94e9376d, 23 },
    { 0xb579376e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43193774, 33 },
    { 0xc793775, 85 },
    { 0x7b593775, 95 },
    { 0xc8d93775, 85 },
    { 0, 0 },
    { 0xb9893779, 55 },
    { 0xede9377a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8519377d, 23 },
    { 0, 0 },
    { 0x9609377f, 33 },
    { 0x73f9377f, 95 },
    { 0, 0 },
    { 0xadc93782, 52 },
    { 0xb993783, 75 },
    { 0x2b193784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc893788, 23 },
    { 0x20393788, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb493792, 90 },
    { 0x82393793, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e9379b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349379e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff937a1, 23 },
    { 0, 0 },
    { 0xb48937a3, 78 },
    { 0xb8a937a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2937a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55937ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4937af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d937b2, 33 },
    { 0, 0 },
    { 0xcb0937b4, 33 },
    { 0, 0 },
    { 0x4aa937b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09937bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f937c2, 78 },
    { 0x2fc937c2, 95 },
    { 0, 0 },
    { 0xb59937c5, 90 },
    { 0x737937c5, 90 },
    { 0xd5c937c7, 87 },
    { 0x472937c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd937cb, 87 },
    { 0x55c937cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39937d0, 61 },
    { 0, 0 },
    { 0xa07937d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b937d6, 98 },
    { 0x4b9937d6, 95 },
    { 0, 0 },
    { 0xc85937d9, 90 },
    { 0x63a937d9, 33 },
    { 0xdea937db, 90 },
    { 0x1d4937dc, 61 },
    { 0, 0 },
    { 0xeb0937de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe937e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856937e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f937ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f937ed, 43 },
    { 0xc3e937ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639937f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x684937fb, 43 },
    { 0x3dd937fb, 76 },
    { 0xb91937fd, 95 },
    { 0xf91937fe, 90 },
    { 0, 0 },
    { 0x82393800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1493804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f93808, 61 },
    { 0xf4a93809, 95 },
    { 0x6b693809, 61 },
    { 0x6ab9380b, 23 },
    { 0xd9b9380c, 61 },
    { 0xf9b9380c, 33 },
    { 0xa0b9380e, 55 },
    { 0x4a29380e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab993812, 46 },
    { 0x34893813, 93 },
    { 0x48293813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f793817, 55 },
    { 0x7fb93817, 61 },
    { 0, 0 },
    { 0x3f39381a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5079381e, 61 },
    { 0, 0 },
    { 0x98b93820, 33 },
    { 0x6a693820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61293832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb93837, 23 },
    { 0, 0 },
    { 0x32793839, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d9383c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb93842, 91 },
    { 0, 0 },
    { 0x1393844, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c893847, 33 },
    { 0x8a593847, 61 },
    { 0x18d93849, 95 },
    { 0xf549384a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce9384d, 23 },
    { 0xe839384e, 76 },
    { 0xce49384d, 81 },
    { 0x2a693850, 76 },
    { 0xcf393851, 55 },
    { 0xa9193851, 90 },
    { 0xe6193851, 75 },
    { 0x17893854, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1793858, 90 },
    { 0x38993858, 61 },
    { 0x56c93859, 87 },
    { 0x7a69385b, 84 },
    { 0x8119385b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1693861, 61 },
    { 0xd2993862, 55 },
    { 0, 0 },
    { 0x63a93864, 23 },
    { 0x40e93865, 88 },
    { 0x32993865, 100 },
    { 0x22293867, 61 },
    { 0xa5093868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9386f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd93873, 33 },
    { 0x6a893874, 100 },
    { 0xcf593874, 67 },
    { 0, 0 },
    { 0xb1993877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719387a, 23 },
    { 0x4b9387b, 38 },
    { 0x9309387c, 98 },
    { 0xad69387b, 76 },
    { 0x6799387e, 77 },
    { 0x4769387f, 76 },
    { 0xde89387e, 95 },
    { 0xb6e93880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda9388a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd679388f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f93896, 98 },
    { 0x1a793896, 61 },
    { 0xb8493898, 67 },
    { 0x29293899, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae9389d, 98 },
    { 0x29b9389e, 23 },
    { 0x7289389f, 68 },
    { 0x1fd9389f, 81 },
    { 0, 0 },
    { 0x297938a2, 61 },
    { 0, 0 },
    { 0x5cc938a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa938a7, 34 },
    { 0x557938a7, 23 },
    { 0xb8a938a9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80938ad, 90 },
    { 0x21a938ae, 90 },
    { 0xb15938ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x340938b2, 96 },
    { 0x900938b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5938b6, 78 },
    { 0x194938b7, 95 },
    { 0, 0 },
    { 0x9b1938b9, 85 },
    { 0x5eb938ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a938c2, 51 },
    { 0x244938c3, 90 },
    { 0x46b938c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d938c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x765938cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe938d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861938d7, 46 },
    { 0x1f5938d8, 46 },
    { 0xb90938d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0938e0, 90 },
    { 0, 0 },
    { 0x53a938e2, 23 },
    { 0, 0 },
    { 0x842938e4, 33 },
    { 0, 0 },
    { 0xbe4938e6, 90 },
    { 0, 0 },
    { 0xf03938e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54938ee, 33 },
    { 0x7f6938ef, 33 },
    { 0x2fa938f0, 87 },
    { 0x212938f1, 33 },
    { 0xd2d938f0, 34 },
    { 0xb01938f3, 43 },
    { 0xb17938ef, 23 },
    { 0, 0 },
    { 0x3d938f6, 33 },
    { 0, 0 },
    { 0xec3938f8, 46 },
    { 0, 0 },
    { 0xba5938fa, 23 },
    { 0x8c938fb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x26938fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26193902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899390b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79390e, 98 },
    { 0, 0 },
    { 0x11393910, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e93918, 66 },
    { 0, 0 },
    { 0x6d89391a, 91 },
    { 0x9bb9391b, 90 },
    { 0x1ae9391b, 34 },
    { 0xd1d9391d, 91 },
    { 0xb369391c, 90 },
    { 0x2f99391b, 87 },
    { 0x4529391f, 76 },
    { 0xab79391f, 78 },
    { 0xf0293922, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e93925, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7493928, 76 },
    { 0x7f693929, 61 },
    { 0xb8393929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca93932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c9393a, 33 },
    { 0, 0 },
    { 0x6399393c, 98 },
    { 0, 0 },
    { 0x1999393e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e93943, 33 },
    { 0, 0 },
    { 0xec093945, 66 },
    { 0x4d393945, 33 },
    { 0x11d93947, 61 },
    { 0x67793947, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449394f, 84 },
    { 0x6593950, 77 },
    { 0xf5393950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a893954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd779395b, 100 },
    { 0x5089395b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c793960, 93 },
    { 0, 0 },
    { 0x2f93962, 90 },
    { 0x96493963, 61 },
    { 0x22c93964, 100 },
    { 0xe7493965, 66 },
    { 0x73993963, 88 },
    { 0x3593966, 33 },
    { 0xf3193965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb49396b, 90 },
    { 0x18e9396c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1293971, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf93974, 99 },
    { 0, 0 },
    { 0x47993976, 33 },
    { 0, 0 },
    { 0x8e793978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a9397e, 88 },
    { 0xdad9397e, 99 },
    { 0x7da93980, 90 },
    { 0x3009397f, 43 },
    { 0x4d693981, 87 },
    { 0, 0 },
    { 0x81093984, 23 },
    { 0, 0 },
    { 0x31d93986, 33 },
    { 0x5e693987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8759398a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b9398d, 52 },
    { 0xa419398d, 65 },
    { 0x3639398e, 43 },
    { 0xafe9398e, 46 },
    { 0x7a293991, 90 },
    { 0, 0 },
    { 0x1c193993, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa793996, 98 },
    { 0x64693997, 23 },
    { 0xea793998, 43 },
    { 0, 0 },
    { 0xd49399a, 33 },
    { 0x4109399b, 56 },
    { 0x6729399a, 61 },
    { 0xaa39399d, 97 },
    { 0x2059399d, 23 },
    { 0x1a69399f, 98 },
    { 0x20d9399f, 55 },
    { 0x4ee9399c, 61 },
    { 0x9cd9399d, 83 },
    { 0x8ff939a3, 99 },
    { 0x327939a3, 95 },
    { 0, 0 },
    { 0x6ff939a6, 33 },
    { 0x36d939a7, 98 },
    { 0x528939a6, 95 },
    { 0xdb3939a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e939ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a939b1, 66 },
    { 0, 0 },
    { 0xeca939b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d939b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372939bd, 61 },
    { 0xee7939bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e939c1, 61 },
    { 0x7d0939c2, 68 },
    { 0x147939c3, 17 },
    { 0, 0 },
    { 0x605939c5, 91 },
    { 0x3f2939c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4939cd, 61 },
    { 0, 0 },
    { 0xeb8939cf, 81 },
    { 0, 0 },
    { 0xf45939d1, 61 },
    { 0x9de939d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8939d6, 61 },
    { 0x863939d6, 98 },
    { 0, 0 },
    { 0x796939d9, 90 },
    { 0x73e939da, 84 },
    { 0, 0 },
    { 0xb23939dc, 90 },
    { 0xee1939dc, 75 },
    { 0, 0 },
    { 0x289939df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0939e3, 33 },
    { 0xa0b939e3, 91 },
    { 0xb4f939e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0939e9, 33 },
    { 0xd3f939e9, 43 },
    { 0xf02939ea, 33 },
    { 0xda1939ec, 61 },
    { 0xca8939ec, 33 },
    { 0x24e939ee, 76 },
    { 0, 0 },
    { 0x1be939f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b939f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6939f6, 98 },
    { 0, 0 },
    { 0x9dd939f8, 61 },
    { 0x127939f8, 33 },
    { 0x691939fa, 23 },
    { 0x634939fb, 61 },
    { 0xc8e939fc, 46 },
    { 0xe30939fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f993a00, 78 },
    { 0x54193a01, 23 },
    { 0, 0 },
    { 0x7493a03, 55 },
    { 0x7a693a04, 43 },
    { 0x1d93a04, 98 },
    { 0xa8193a06, 34 },
    { 0x1c593a07, 90 },
    { 0x38f93a07, 65 },
    { 0xfb793a07, 36 },
    { 0x69993a06, 33 },
    { 0, 0 },
    { 0x35393a0c, 52 },
    { 0x1f893a0c, 98 },
    { 0x99b93a0c, 33 },
    { 0, 0 },
    { 0x8f293a10, 33 },
    { 0xb5993a11, 83 },
    { 0, 0 },
    { 0x18493a13, 90 },
    { 0xcf93a13, 90 },
    { 0x56d93a15, 98 },
    { 0xdb693a16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a93a1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b93a21, 94 },
    { 0xfa293a22, 43 },
    { 0xe2b93a23, 34 },
    { 0x6db93a22, 100 },
    { 0x96b93a21, 90 },
    { 0x92c93a22, 22 },
    { 0xa9593a27, 90 },
    { 0x3e93a27, 91 },
    { 0, 0 },
    { 0x8a93a2a, 23 },
    { 0xeec93a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f593a3c, 23 },
    { 0xc9393a3c, 33 },
    { 0, 0 },
    { 0xc7c93a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75993a42, 90 },
    { 0xd1293a42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf693a47, 33 },
    { 0x57f93a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593a4b, 69 },
    { 0, 0 },
    { 0x40993a4d, 37 },
    { 0, 0 },
    { 0x30693a4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75093a53, 90 },
    { 0xcae93a53, 55 },
    { 0x8a493a55, 98 },
    { 0x93793a55, 33 },
    { 0x2f193a53, 23 },
    { 0xef393a54, 36 },
    { 0x37893a59, 81 },
    { 0x74b93a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e93a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85493a64, 93 },
    { 0, 0 },
    { 0x6e93a66, 90 },
    { 0, 0 },
    { 0xdf793a68, 22 },
    { 0xf2093a69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f93a73, 91 },
    { 0xe5593a73, 43 },
    { 0, 0 },
    { 0x58c93a76, 98 },
    { 0xdf693a76, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d93a7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693a81, 43 },
    { 0, 0 },
    { 0x9e493a83, 33 },
    { 0, 0 },
    { 0xf493a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f393a89, 98 },
    { 0x63f93a89, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b993a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e593a90, 61 },
    { 0xa1493a91, 61 },
    { 0, 0 },
    { 0x6e493a93, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a93a98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c93a9f, 33 },
    { 0, 0 },
    { 0xbdf93aa1, 78 },
    { 0, 0 },
    { 0x95293aa3, 95 },
    { 0xdc093aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8193aa7, 61 },
    { 0xacf93aa8, 34 },
    { 0xc0693aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe193aad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51693ab0, 55 },
    { 0xebe93ab1, 52 },
    { 0, 0 },
    { 0xb2d93ab3, 61 },
    { 0xa7193ab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe893ab9, 46 },
    { 0, 0 },
    { 0x1dc93abb, 34 },
    { 0xcc93abb, 12 },
    { 0, 0 },
    { 0x6cf93abe, 40 },
    { 0x59a93abf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c93ac2, 77 },
    { 0xb8293ac3, 76 },
    { 0, 0 },
    { 0x68493ac5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43293acc, 33 },
    { 0xdd193acd, 23 },
    { 0xe1a93acc, 52 },
    { 0, 0 },
    { 0xa9193ad0, 69 },
    { 0x58e93ad1, 78 },
    { 0xdd93ad2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f893ad5, 23 },
    { 0xcd293ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4293adf, 66 },
    { 0, 0 },
    { 0x98093ae1, 67 },
    { 0xf2d93ae2, 95 },
    { 0xa4593ae1, 33 },
    { 0xba993ae4, 33 },
    { 0xd0f93ae5, 84 },
    { 0xc0893ae4, 90 },
    { 0x72f93ae7, 99 },
    { 0x38f93ae8, 66 },
    { 0x6b093ae9, 75 },
    { 0x25193aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da93aed, 33 },
    { 0x2ec93aee, 98 },
    { 0x98393aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b93af8, 80 },
    { 0, 0 },
    { 0xf2493afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de93afd, 98 },
    { 0xba793afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba93b01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e93b04, 33 },
    { 0, 0 },
    { 0xebb93b06, 33 },
    { 0, 0 },
    { 0xde893b08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b393b0b, 77 },
    { 0xa0c93b0b, 43 },
    { 0x97d93b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d093b10, 95 },
    { 0, 0 },
    { 0xccf93b12, 98 },
    { 0x66793b13, 90 },
    { 0, 0 },
    { 0x73693b15, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2493b18, 34 },
    { 0, 0 },
    { 0xa9893b1a, 52 },
    { 0x69993b1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de93b20, 78 },
    { 0xe393b20, 76 },
    { 0x32893b22, 55 },
    { 0xf5b93b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72993b28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3893b2c, 23 },
    { 0x9d993b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5993b31, 61 },
    { 0x6c693b31, 61 },
    { 0x2e93b33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb93b36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a993b3d, 90 },
    { 0x5fe93b3e, 85 },
    { 0x6a293b3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49393b44, 78 },
    { 0x84293b45, 68 },
    { 0x98593b45, 93 },
    { 0x61893b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42093b4a, 34 },
    { 0x48d93b4a, 33 },
    { 0xd0993b4c, 52 },
    { 0x6a993b4d, 77 },
    { 0x36d93b4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a93b51, 17 },
    { 0xe2b93b52, 76 },
    { 0, 0 },
    { 0x53493b54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e93b57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45893b5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7593b61, 23 },
    { 0x7c593b61, 23 },
    { 0xd293b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41893b6b, 90 },
    { 0x92493b6c, 95 },
    { 0x26c93b6d, 33 },
    { 0x60793b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc593b77, 65 },
    { 0x30493b77, 93 },
    { 0xb7893b79, 33 },
    { 0, 0 },
    { 0xf0e93b7b, 55 },
    { 0x80f93b7c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46893b80, 98 },
    { 0, 0 },
    { 0x3e393b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3093b85, 44 },
    { 0xcdf93b86, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f93b89, 90 },
    { 0x7d893b8a, 91 },
    { 0x59993b89, 33 },
    { 0xb4a93b8c, 34 },
    { 0x84093b89, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c93b90, 99 },
    { 0, 0 },
    { 0x38a93b92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e93b98, 78 },
    { 0xaef93b99, 100 },
    { 0x37d93b9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc93ba0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf93ba8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b93bac, 76 },
    { 0x4f093bad, 23 },
    { 0xfb293bae, 100 },
    { 0x81393baf, 77 },
    { 0x7b93baf, 96 },
    { 0xd8093bb1, 81 },
    { 0x45793bb1, 76 },
    { 0x77b93bb3, 43 },
    { 0x69193bb0, 33 },
    { 0xa3293bb5, 55 },
    { 0, 0 },
    { 0xd7e93bb7, 76 },
    { 0, 0 },
    { 0xa1f93bb9, 83 },
    { 0xe5793bba, 81 },
    { 0x18193bba, 78 },
    { 0x53d93bba, 55 },
    { 0, 0 },
    { 0x3f193bbe, 90 },
    { 0x96b93bbf, 61 },
    { 0xa5d93bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71293bc3, 95 },
    { 0, 0 },
    { 0x8ec93bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33593bc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d093bcb, 78 },
    { 0xeb793bcb, 33 },
    { 0x12093bcc, 87 },
    { 0xb3193bcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad993bd3, 95 },
    { 0, 0 },
    { 0x4193bd5, 66 },
    { 0xa2393bd5, 90 },
    { 0xf9693bd7, 17 },
    { 0x13593bd8, 76 },
    { 0x87c93bd9, 93 },
    { 0, 0 },
    { 0xd6393bdb, 61 },
    { 0x58993bdb, 98 },
    { 0, 0 },
    { 0x90093bde, 95 },
    { 0, 0 },
    { 0x54893be0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e093be4, 76 },
    { 0x70293be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe93bea, 91 },
    { 0xa393beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1493bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b393bf9, 87 },
    { 0, 0 },
    { 0xc0493bfb, 90 },
    { 0x39a93bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e193c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1293c06, 34 },
    { 0x3b593c06, 90 },
    { 0x5ef93c08, 55 },
    { 0xb6293c08, 95 },
    { 0xbd293c0a, 77 },
    { 0x21693c09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c93c13, 33 },
    { 0x3a293c13, 46 },
    { 0xf7693c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d93c1a, 90 },
    { 0x13f93c1a, 90 },
    { 0x2dd93c1c, 61 },
    { 0x4c793c1d, 23 },
    { 0, 0 },
    { 0x93c93c1f, 61 },
    { 0x4c593c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d93c25, 33 },
    { 0x1d793c25, 61 },
    { 0x6d493c27, 90 },
    { 0xf0293c28, 61 },
    { 0x43f93c27, 98 },
    { 0x4ff93c29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9193c34, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40893c39, 98 },
    { 0, 0 },
    { 0xa393c3b, 68 },
    { 0x67c93c3c, 33 },
    { 0xb0093c3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x30393c40, 61 },
    { 0xb1893c41, 90 },
    { 0x5a93c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa93c45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21793c49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0593c4f, 61 },
    { 0xfb993c50, 40 },
    { 0x46693c50, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c93c5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36793c5d, 61 },
    { 0x34093c5e, 68 },
    { 0xeb693c5d, 93 },
    { 0xbe593c5e, 61 },
    { 0x94793c61, 55 },
    { 0x4cd93c61, 66 },
    { 0x4293c5f, 90 },
    { 0xf0093c64, 28 },
    { 0x82193c64, 40 },
    { 0x1d693c66, 33 },
    { 0x3a793c5d, 90 },
    { 0x4e593c68, 78 },
    { 0, 0 },
    { 0x6993c6a, 23 },
    { 0, 0 },
    { 0x7793c6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c293c75, 46 },
    { 0, 0 },
    { 0xfb193c77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49693c7d, 33 },
    { 0x6fe93c7d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe93c82, 61 },
    { 0xe7193c82, 90 },
    { 0, 0 },
    { 0x83f93c85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed993c8e, 23 },
    { 0x6b093c8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f93c97, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73193c9f, 33 },
    { 0x51c93ca0, 90 },
    { 0xa293ca0, 84 },
    { 0x35d93ca2, 90 },
    { 0x4093ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b93ca6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d093cb2, 43 },
    { 0x71393cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf893cb8, 56 },
    { 0x85693cb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3693cbd, 61 },
    { 0, 0 },
    { 0x4293cbf, 33 },
    { 0xb993cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b93cc9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9193ccd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b93cd5, 66 },
    { 0x36093cd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a93ce0, 76 },
    { 0x91f93ce1, 33 },
    { 0xc4d93ce1, 90 },
    { 0xe3293ce1, 61 },
    { 0xc9393ce4, 93 },
    { 0x80393ce5, 90 },
    { 0x52293ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51693ce9, 85 },
    { 0x59c93cea, 76 },
    { 0x31093ceb, 90 },
    { 0x8ff93ce9, 98 },
    { 0x53f93ced, 33 },
    { 0x50493ced, 61 },
    { 0, 0 },
    { 0xe3c93cf0, 61 },
    { 0x39093cf1, 23 },
    { 0xc2693cf0, 84 },
    { 0xe693cf3, 61 },
    { 0xbc493cf4, 98 },
    { 0x7b493cf3, 95 },
    { 0x58b93cf4, 55 },
    { 0, 0 },
    { 0x87893cf8, 59 },
    { 0, 0 },
    { 0xff893cfa, 45 },
    { 0x9fd93cfb, 90 },
    { 0xff993cfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3393cff, 90 },
    { 0, 0 },
    { 0x5993d01, 55 },
    { 0xa1893d02, 90 },
    { 0x52493d01, 23 },
    { 0, 0 },
    { 0x45993d05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e393d08, 55 },
    { 0, 0 },
    { 0x54f93d0a, 33 },
    { 0, 0 },
    { 0x11593d0c, 84 },
    { 0x77893d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa393d10, 33 },
    { 0x5ed93d10, 61 },
    { 0xf3f93d12, 55 },
    { 0x79893d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x57593d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8093d1c, 85 },
    { 0x64093d1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81793d21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42593d25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b93d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa693d2b, 23 },
    { 0xa8993d2c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25493d31, 33 },
    { 0, 0 },
    { 0x50c93d33, 76 },
    { 0x2b793d34, 67 },
    { 0x10a93d34, 99 },
    { 0x8b993d34, 61 },
    { 0xf9993d35, 87 },
    { 0xded93d38, 90 },
    { 0, 0 },
    { 0xc093d3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d93d41, 23 },
    { 0x26093d41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93693d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea93d49, 98 },
    { 0x56c93d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c393d50, 55 },
    { 0x73f93d51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60393d55, 98 },
    { 0x55a93d56, 90 },
    { 0xb2993d56, 66 },
    { 0, 0 },
    { 0x81393d59, 55 },
    { 0x6b93d5a, 66 },
    { 0x91f93d5b, 90 },
    { 0x9c693d5c, 95 },
    { 0xadb93d5d, 76 },
    { 0x5b293d5e, 98 },
    { 0x9c693d5f, 98 },
    { 0xb3593d5f, 90 },
    { 0xf8193d5f, 55 },
    { 0xce693d62, 90 },
    { 0xffb93d62, 89 },
    { 0x7b393d62, 55 },
    { 0x11193d65, 100 },
    { 0x73e93d66, 81 },
    { 0x29493d66, 93 },
    { 0xeec93d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f893d6e, 33 },
    { 0x9c793d6f, 93 },
    { 0x19c93d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b93d7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a93d80, 95 },
    { 0x9eb93d81, 33 },
    { 0x5ac93d82, 51 },
    { 0x43b93d83, 78 },
    { 0x8f393d84, 98 },
    { 0x4d993d84, 76 },
    { 0x7fa93d82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b93d89, 61 },
    { 0, 0 },
    { 0x3c093d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf93d8f, 96 },
    { 0x98093d90, 23 },
    { 0x6a793d90, 95 },
    { 0x2b793d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce93d96, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26393d9c, 40 },
    { 0x60e93d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90393da2, 30 },
    { 0xf1c93da3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1393da7, 75 },
    { 0, 0 },
    { 0x24593da9, 90 },
    { 0x8d893daa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd93db1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e93db4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6993db9, 55 },
    { 0x1293dba, 90 },
    { 0x18593dbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad393dbe, 55 },
    { 0xf4393dbe, 61 },
    { 0, 0 },
    { 0x4ea93dc1, 34 },
    { 0xe4a93dc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a93dc6, 64 },
    { 0, 0 },
    { 0x68993dc8, 90 },
    { 0x6aa93dc9, 90 },
    { 0x40893dca, 61 },
    { 0xb4193dcb, 46 },
    { 0x4d093dcb, 33 },
    { 0x7a893dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c093dd0, 23 },
    { 0x53393dd1, 46 },
    { 0xf2093dd2, 23 },
    { 0xe6693dd2, 33 },
    { 0x75793dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f93dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5793ddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce493de2, 95 },
    { 0xe4d93de3, 90 },
    { 0, 0 },
    { 0xa7193de5, 46 },
    { 0xddb93de5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5193dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab393df6, 34 },
    { 0, 0 },
    { 0x85a93df8, 75 },
    { 0x8f393df9, 44 },
    { 0xb2e93df8, 55 },
    { 0xe3a93dfa, 100 },
    { 0x49093dfc, 90 },
    { 0xee293dfc, 93 },
    { 0x3e293df8, 33 },
    { 0xf1393df9, 23 },
    { 0x45893e00, 95 },
    { 0x4c493e00, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2093e04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33893e0d, 76 },
    { 0xb8493e0e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c893e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6093e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52893e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee93e1f, 38 },
    { 0x48b93e20, 76 },
    { 0x9b193e20, 55 },
    { 0x8b893e22, 98 },
    { 0x21793e22, 90 },
    { 0, 0 },
    { 0xfb693e25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e93e2c, 99 },
    { 0x88e93e2d, 45 },
    { 0, 0 },
    { 0x48293e2f, 49 },
    { 0xbb493e2f, 23 },
    { 0x94793e30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc93e35, 61 },
    { 0x29d93e35, 82 },
    { 0, 0 },
    { 0xd2d93e38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41093e3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66093e44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a93e47, 33 },
    { 0x49a93e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a993e4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c93e53, 60 },
    { 0x7d93e53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a93e58, 33 },
    { 0x92b93e59, 93 },
    { 0, 0 },
    { 0x25093e5b, 87 },
    { 0, 0 },
    { 0xaef93e5d, 98 },
    { 0x8de93e5d, 90 },
    { 0x39e93e5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c593e63, 90 },
    { 0xa4593e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd993e68, 90 },
    { 0x3cb93e69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0093e6e, 95 },
    { 0, 0 },
    { 0xf9893e70, 76 },
    { 0x54793e71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d93e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a193e7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20493e86, 61 },
    { 0x1d493e87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd93e94, 95 },
    { 0x4fc93e94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3293e98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d93e9e, 46 },
    { 0x9de93e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7293ea2, 43 },
    { 0x7d693ea3, 76 },
    { 0, 0 },
    { 0xb9f93ea5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e93eaa, 55 },
    { 0xcca93eaa, 67 },
    { 0, 0 },
    { 0x92793ead, 81 },
    { 0x95893eae, 90 },
    { 0x6cb93eaf, 66 },
    { 0xe6093eb0, 52 },
    { 0x53f93eb0, 24 },
    { 0x38693eb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d93eb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d193eba, 90 },
    { 0x2ef93ebb, 23 },
    { 0x86093ebc, 98 },
    { 0, 0 },
    { 0x44d93ebe, 95 },
    { 0xfcd93ebf, 90 },
    { 0x64193ec0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e693ec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f93ec7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca593ece, 34 },
    { 0x63d93ecf, 65 },
    { 0, 0 },
    { 0x68c93ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d93eda, 90 },
    { 0, 0 },
    { 0x29393edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f93eea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee93eef, 53 },
    { 0xed993ef0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c93ef3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d93ef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a293ef9, 87 },
    { 0xda693ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48293eff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e93f04, 90 },
    { 0xe7293f05, 93 },
    { 0x62893f06, 33 },
    { 0, 0 },
    { 0x12093f08, 34 },
    { 0x74693f08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3893f14, 33 },
    { 0x25393f14, 78 },
    { 0x49793f14, 84 },
    { 0x82893f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b93f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb393f20, 76 },
    { 0x16893f21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3793f24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d693f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x493f2e, 98 },
    { 0xfd393f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e393f33, 23 },
    { 0x9c93f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91893f39, 61 },
    { 0x6f693f3a, 43 },
    { 0x9d393f3b, 78 },
    { 0xb4c93f3c, 33 },
    { 0x11a93f39, 96 },
    { 0x31093f3b, 33 },
    { 0xb3d93f3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab393f43, 55 },
    { 0x63893f44, 34 },
    { 0xd1793f45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0493f48, 95 },
    { 0x7aa93f49, 98 },
    { 0x79093f4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca693f4e, 33 },
    { 0, 0 },
    { 0x28493f50, 61 },
    { 0xf1493f50, 33 },
    { 0xda93f51, 78 },
    { 0x23a93f51, 90 },
    { 0, 0 },
    { 0xfd393f55, 17 },
    { 0, 0 },
    { 0x2b393f57, 90 },
    { 0xffc93f58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef93f60, 69 },
    { 0, 0 },
    { 0x9e893f62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5693f67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c093f6a, 61 },
    { 0xc2593f6b, 61 },
    { 0x6d793f6c, 90 },
    { 0xa5293f6d, 55 },
    { 0x50593f6e, 77 },
    { 0x80193f6a, 33 },
    { 0x25693f6c, 17 },
    { 0x6d293f71, 95 },
    { 0, 0 },
    { 0x67a93f73, 85 },
    { 0x8da93f73, 95 },
    { 0x59993f75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e93f7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f493f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d993f82, 33 },
    { 0xc2193f83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa93f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f93f90, 96 },
    { 0xd9393f91, 85 },
    { 0x65993f90, 90 },
    { 0xf8093f90, 61 },
    { 0xbd593f94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d393f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77693f9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f93fa4, 78 },
    { 0x6f493fa5, 90 },
    { 0x17d93fa5, 34 },
    { 0x44193fa7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0893fac, 92 },
    { 0, 0 },
    { 0x9c193fae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b93fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff593fb5, 77 },
    { 0, 0 },
    { 0xdee93fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf193fbd, 34 },
    { 0xd5993fbd, 43 },
    { 0, 0 },
    { 0xbec93fc0, 55 },
    { 0x9193fc1, 93 },
    { 0x99493fc2, 23 },
    { 0xce293fc2, 95 },
    { 0, 0 },
    { 0x42293fc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e93fc8, 88 },
    { 0, 0 },
    { 0xce493fca, 23 },
    { 0x25c93fcb, 90 },
    { 0, 0 },
    { 0xf0493fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb93fd1, 98 },
    { 0x56e93fd2, 91 },
    { 0x96493fd2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42293fde, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44793fe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1593fe7, 76 },
    { 0x4d093fe8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a93ff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56193ff7, 43 },
    { 0xa7593ff7, 93 },
    { 0xe3093ff8, 34 },
    { 0xd8893ffa, 90 },
    { 0x3eb93ff8, 34 },
    { 0xb2f93ffc, 34 },
    { 0x29793ffd, 28 },
    { 0xb3393ffd, 33 },
    { 0xca093ffe, 33 },
    { 0x6da93ffe, 98 },
    { 0xf1393ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1094007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5959400c, 98 },
    { 0xff59400d, 95 },
    { 0x6959400c, 43 },
    { 0xddb9400f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68894012, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac94015, 34 },
    { 0xce694015, 34 },
    { 0, 0 },
    { 0x6cb94018, 97 },
    { 0x84294018, 85 },
    { 0, 0 },
    { 0xef09401b, 76 },
    { 0x77a9401b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc9401f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d94023, 35 },
    { 0x18094024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b94027, 61 },
    { 0x45d94027, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc49402b, 55 },
    { 0x28e9402c, 43 },
    { 0x4ed9402d, 94 },
    { 0, 0 },
    { 0x3259402f, 84 },
    { 0x65a94030, 61 },
    { 0xc5394031, 61 },
    { 0x88a94030, 95 },
    { 0x99994032, 33 },
    { 0x91a94032, 95 },
    { 0xa994035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5249403d, 55 },
    { 0xd7a9403e, 43 },
    { 0xed59403f, 91 },
    { 0x4f194040, 61 },
    { 0x42294041, 34 },
    { 0, 0 },
    { 0x59094043, 78 },
    { 0, 0 },
    { 0xd6294045, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf9404c, 46 },
    { 0x4c59404d, 93 },
    { 0xd2d9404d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5794051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e94054, 76 },
    { 0x98694055, 76 },
    { 0x21e94055, 99 },
    { 0, 0 },
    { 0xbc194058, 95 },
    { 0x33894058, 66 },
    { 0x54a9405a, 33 },
    { 0xae39405b, 61 },
    { 0x4b9405c, 90 },
    { 0xf429405d, 33 },
    { 0xeeb9405d, 33 },
    { 0x2109405f, 23 },
    { 0xcbf94060, 33 },
    { 0, 0 },
    { 0x77594062, 61 },
    { 0xb2994063, 33 },
    { 0, 0 },
    { 0xdc894065, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9406b, 76 },
    { 0xf4e9406b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf94074, 33 },
    { 0xca894075, 95 },
    { 0x2094074, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c9407a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9408c, 55 },
    { 0, 0 },
    { 0x6539408e, 87 },
    { 0x2859408f, 90 },
    { 0x7569408e, 98 },
    { 0x3b194091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f94094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af94098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc329409b, 43 },
    { 0xdb29409c, 69 },
    { 0x5059409d, 34 },
    { 0x9529409d, 99 },
    { 0xf619409f, 61 },
    { 0x6519409f, 33 },
    { 0x70c940a1, 90 },
    { 0xa8a9409e, 33 },
    { 0x141940a3, 56 },
    { 0xb129409e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46940a7, 90 },
    { 0, 0 },
    { 0xcbc940a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8940ac, 90 },
    { 0xe82940ad, 33 },
    { 0, 0 },
    { 0x8e5940af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f940b9, 40 },
    { 0, 0 },
    { 0x578940bb, 90 },
    { 0x3ba940bc, 55 },
    { 0x271940bd, 33 },
    { 0x9ea940be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39940c1, 23 },
    { 0x327940c2, 90 },
    { 0xdfd940c2, 40 },
    { 0x5a0940c4, 23 },
    { 0, 0 },
    { 0x67e940c6, 55 },
    { 0xceb940c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c940cb, 98 },
    { 0xe30940cc, 33 },
    { 0x50940cd, 80 },
    { 0xa4d940cd, 33 },
    { 0x3c7940cc, 52 },
    { 0xb99940cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xec940d3, 61 },
    { 0x189940d4, 61 },
    { 0xe79940d5, 33 },
    { 0x92a940d5, 95 },
    { 0xa6940d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e940db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1940e1, 61 },
    { 0x2f9940e2, 23 },
    { 0xb62940e1, 24 },
    { 0xafe940e1, 23 },
    { 0xe90940e5, 85 },
    { 0x697940e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac940e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e940ee, 99 },
    { 0, 0 },
    { 0xd1e940f0, 61 },
    { 0x68940f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a940fc, 88 },
    { 0xf2b940fc, 23 },
    { 0, 0 },
    { 0x7b6940ff, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x86594102, 43 },
    { 0, 0 },
    { 0x1b294104, 90 },
    { 0xb294104, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x45494108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31994110, 98 },
    { 0xa194110, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4894114, 61 },
    { 0xcff94114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b9411c, 33 },
    { 0x36e9411d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6794122, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c494129, 61 },
    { 0x91894129, 90 },
    { 0xbee9412a, 93 },
    { 0xbc19412c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe449412f, 95 },
    { 0x31294130, 91 },
    { 0x7dd94131, 23 },
    { 0, 0 },
    { 0x6d794133, 90 },
    { 0xe3494134, 23 },
    { 0xf394135, 33 },
    { 0x55094135, 98 },
    { 0xcfc94137, 95 },
    { 0xa4594137, 33 },
    { 0, 0 },
    { 0x5309413a, 90 },
    { 0x4189413b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed9413e, 90 },
    { 0xab49413f, 17 },
    { 0x7d9413f, 96 },
    { 0x9a69413f, 33 },
    { 0x59a94141, 81 },
    { 0x4b394143, 33 },
    { 0x67994143, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89414c, 33 },
    { 0, 0 },
    { 0x8029414e, 46 },
    { 0xd239414f, 46 },
    { 0x5a94150, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1994153, 83 },
    { 0xebb94154, 98 },
    { 0x79794153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9394159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1394162, 33 },
    { 0xaff94163, 61 },
    { 0x7994164, 93 },
    { 0x97694165, 61 },
    { 0, 0 },
    { 0xe7894167, 84 },
    { 0xf8494167, 33 },
    { 0x76194169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd9416d, 23 },
    { 0, 0 },
    { 0x9949416f, 43 },
    { 0xdd19416f, 91 },
    { 0, 0 },
    { 0x37294172, 95 },
    { 0x83d94173, 60 },
    { 0x7f394174, 90 },
    { 0x50d94173, 55 },
    { 0x23c94175, 34 },
    { 0, 0 },
    { 0xae994178, 23 },
    { 0, 0 },
    { 0x9af9417a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd049417d, 98 },
    { 0, 0 },
    { 0x2209417f, 87 },
    { 0, 0 },
    { 0x17d94181, 40 },
    { 0, 0 },
    { 0xc3394183, 61 },
    { 0xcab94183, 93 },
    { 0x93f94185, 23 },
    { 0x22a94186, 90 },
    { 0x48594183, 91 },
    { 0x97594188, 61 },
    { 0, 0 },
    { 0xd1b9418a, 90 },
    { 0x9549418b, 23 },
    { 0xd279418c, 33 },
    { 0, 0 },
    { 0x6219418e, 96 },
    { 0xe7a9418f, 95 },
    { 0xfcf94190, 61 },
    { 0xe289418e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0194195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55594198, 95 },
    { 0, 0 },
    { 0xfe29419a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17941a4, 90 },
    { 0x85941a5, 90 },
    { 0x8ab941a6, 84 },
    { 0x719941a6, 33 },
    { 0x946941a8, 95 },
    { 0x683941a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69941b0, 90 },
    { 0x75f941b1, 23 },
    { 0x842941b0, 75 },
    { 0x515941b3, 87 },
    { 0x5e6941b4, 56 },
    { 0x3e941b4, 90 },
    { 0x55b941b5, 39 },
    { 0xa95941b4, 75 },
    { 0x903941b4, 95 },
    { 0, 0 },
    { 0x22f941ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52941be, 61 },
    { 0, 0 },
    { 0x7c8941c0, 23 },
    { 0x169941c1, 90 },
    { 0x693941c1, 90 },
    { 0x3a6941c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380941c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268941d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3941de, 68 },
    { 0xbb1941de, 90 },
    { 0, 0 },
    { 0xdd0941e1, 98 },
    { 0, 0 },
    { 0x8c4941e3, 87 },
    { 0x4c7941e4, 76 },
    { 0x55c941e4, 23 },
    { 0xdfa941e6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2941ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157941f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c941f4, 87 },
    { 0xad2941f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1941f9, 76 },
    { 0xae4941fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a894204, 90 },
    { 0x33894204, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a94208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc249420b, 76 },
    { 0x26f9420c, 95 },
    { 0xeb79420c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3894212, 98 },
    { 0xd9d94213, 23 },
    { 0xccd94213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f794218, 33 },
    { 0x53e94219, 77 },
    { 0, 0 },
    { 0xabf9421b, 90 },
    { 0, 0 },
    { 0x5ed9421d, 90 },
    { 0, 0 },
    { 0x70c9421f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd94222, 91 },
    { 0x6d594222, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f94226, 52 },
    { 0x3be94227, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e9422b, 33 },
    { 0x5b99422c, 46 },
    { 0xd359422c, 76 },
    { 0x2a79422b, 94 },
    { 0x22f9422f, 84 },
    { 0x8b594230, 98 },
    { 0xad59422f, 90 },
    { 0xd9b9422c, 95 },
    { 0x1ab94233, 61 },
    { 0xebd9422b, 98 },
    { 0, 0 },
    { 0x9e494236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd94245, 23 },
    { 0xd6b94246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209424d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a994250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee994255, 61 },
    { 0xdb394256, 100 },
    { 0x77394257, 33 },
    { 0x54594258, 95 },
    { 0x2594259, 23 },
    { 0x2959425a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6419425f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8294266, 33 },
    { 0xe4f94267, 90 },
    { 0, 0 },
    { 0x18394269, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf129426d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe94271, 90 },
    { 0x6b394272, 99 },
    { 0, 0 },
    { 0x9e494274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4594278, 44 },
    { 0, 0 },
    { 0xf1c9427a, 61 },
    { 0x5d69427b, 59 },
    { 0x24e9427c, 90 },
    { 0, 0 },
    { 0x58d9427e, 93 },
    { 0xdda9427f, 95 },
    { 0x19694280, 34 },
    { 0x14b9427f, 76 },
    { 0xb6f94282, 77 },
    { 0, 0 },
    { 0x7d794284, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36294288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439428b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e94290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e94296, 67 },
    { 0x7a894296, 98 },
    { 0x11a94298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af9429c, 33 },
    { 0, 0 },
    { 0xa2c9429e, 90 },
    { 0xf89429f, 98 },
    { 0xa90942a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225942a6, 90 },
    { 0x9b1942a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2942ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2942b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6942b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4942c0, 95 },
    { 0x489942c1, 33 },
    { 0x641942c1, 98 },
    { 0xb07942c3, 95 },
    { 0, 0 },
    { 0xf57942c5, 95 },
    { 0x80c942c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00942c9, 33 },
    { 0, 0 },
    { 0x30942cb, 33 },
    { 0, 0 },
    { 0xebe942cd, 43 },
    { 0, 0 },
    { 0xbea942cf, 100 },
    { 0x574942d0, 78 },
    { 0x3c3942d1, 33 },
    { 0x2c5942d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a942d7, 34 },
    { 0xc40942d8, 33 },
    { 0x868942d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x457942dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149942e1, 34 },
    { 0x37942e2, 80 },
    { 0xbe7942e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x305942e6, 34 },
    { 0, 0 },
    { 0x160942e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56942eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4942ef, 91 },
    { 0x3bd942ef, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8942f3, 38 },
    { 0x4f3942f3, 66 },
    { 0xa4942f3, 23 },
    { 0x187942f6, 84 },
    { 0xa80942f6, 61 },
    { 0x9b0942f8, 33 },
    { 0xccd942f5, 98 },
    { 0x5ae942fa, 23 },
    { 0x193942f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f94304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x18394307, 85 },
    { 0x5fe94308, 34 },
    { 0, 0 },
    { 0x4fa9430a, 23 },
    { 0x9c9430a, 43 },
    { 0x1689430c, 34 },
    { 0xf949430c, 55 },
    { 0xbaf9430e, 23 },
    { 0, 0 },
    { 0x1e794310, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70394314, 55 },
    { 0xbd094315, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb494318, 78 },
    { 0x62194318, 33 },
    { 0xad894318, 33 },
    { 0x7279431b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa069431f, 98 },
    { 0, 0 },
    { 0xc9494321, 90 },
    { 0x4f94322, 33 },
    { 0x2494323, 61 },
    { 0x2e594323, 91 },
    { 0x74b94325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62894329, 52 },
    { 0x7f94329, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979432e, 75 },
    { 0xbf69432f, 90 },
    { 0, 0 },
    { 0xd0394331, 33 },
    { 0x13094332, 34 },
    { 0x2094333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b794339, 33 },
    { 0x1bf9433a, 77 },
    { 0, 0 },
    { 0x7159433c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac94340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5394343, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82894349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b94354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9294358, 23 },
    { 0, 0 },
    { 0x4ac9435a, 76 },
    { 0x9f49435a, 46 },
    { 0, 0 },
    { 0x3759435d, 65 },
    { 0x5909435e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e94364, 90 },
    { 0, 0 },
    { 0x67494366, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129436b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x669436e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea94371, 61 },
    { 0xbb894372, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25794377, 71 },
    { 0x9f794378, 34 },
    { 0x7a94377, 61 },
    { 0x40f94377, 46 },
    { 0, 0 },
    { 0xbe99437c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34394382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f194386, 64 },
    { 0xf0194387, 84 },
    { 0, 0 },
    { 0x1c94389, 84 },
    { 0x6009438a, 78 },
    { 0x5d19438b, 33 },
    { 0x26694389, 98 },
    { 0xb219438d, 61 },
    { 0xce29438d, 61 },
    { 0xa189438e, 61 },
    { 0x73f9438c, 92 },
    { 0x4059438c, 33 },
    { 0xdb694392, 76 },
    { 0x37a94393, 23 },
    { 0x14f94392, 88 },
    { 0x5df94395, 76 },
    { 0xfd194393, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8294399, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c19439c, 90 },
    { 0x6079439d, 90 },
    { 0, 0 },
    { 0x51f9439f, 55 },
    { 0x80b943a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f943a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5943a8, 95 },
    { 0, 0 },
    { 0xbe5943aa, 78 },
    { 0xa42943ab, 80 },
    { 0, 0 },
    { 0x363943ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a943b2, 61 },
    { 0xcbf943b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83943be, 66 },
    { 0x9c9943bf, 33 },
    { 0, 0 },
    { 0xc1d943c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8943c7, 90 },
    { 0xa2f943c8, 87 },
    { 0xe1a943c8, 43 },
    { 0xc10943ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4943d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08943d3, 77 },
    { 0xd2f943d4, 55 },
    { 0, 0 },
    { 0x5f943d6, 61 },
    { 0xabb943d7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c943de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x278943e1, 98 },
    { 0, 0 },
    { 0xd0e943e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6943e6, 80 },
    { 0x45d943e7, 81 },
    { 0x8ac943e8, 95 },
    { 0x4ee943e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513943ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370943f3, 90 },
    { 0xba5943f3, 33 },
    { 0xeb3943f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602943fa, 76 },
    { 0x7be943fb, 67 },
    { 0xe99943fc, 23 },
    { 0x9f8943fd, 26 },
    { 0, 0 },
    { 0xb94943ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe479440a, 76 },
    { 0xd399440b, 52 },
    { 0xf29440a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e94414, 33 },
    { 0, 0 },
    { 0x1f694416, 33 },
    { 0xfcb94417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce69441f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f94423, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80994428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6594430, 61 },
    { 0, 0 },
    { 0xd6c94432, 87 },
    { 0, 0 },
    { 0x21f94434, 61 },
    { 0x5d094434, 99 },
    { 0xa6f94436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae9443a, 90 },
    { 0x4a39443b, 33 },
    { 0x3989443c, 78 },
    { 0x5b99443d, 61 },
    { 0, 0 },
    { 0xb709443f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6794449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed19444d, 43 },
    { 0, 0 },
    { 0x1e89444f, 95 },
    { 0x4ae94450, 23 },
    { 0x7df94451, 76 },
    { 0x1fd94452, 61 },
    { 0xa9094453, 33 },
    { 0xa8194453, 93 },
    { 0x28694455, 98 },
    { 0xdda94456, 90 },
    { 0x50d94455, 23 },
    { 0, 0 },
    { 0xb1894459, 61 },
    { 0, 0 },
    { 0x5f09445b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d94460, 34 },
    { 0x8c294461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e94464, 95 },
    { 0xc7794465, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da94468, 90 },
    { 0x8994469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429446e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe194472, 33 },
    { 0, 0 },
    { 0xea494474, 38 },
    { 0, 0 },
    { 0xd3c94476, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5394479, 95 },
    { 0x23a94479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3689447d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b194482, 90 },
    { 0, 0 },
    { 0x3d494484, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4594487, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c394492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49449a, 46 },
    { 0, 0 },
    { 0xaff9449c, 78 },
    { 0x28f9449c, 90 },
    { 0xb689449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f944a2, 84 },
    { 0x6fa944a3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711944ac, 46 },
    { 0, 0 },
    { 0xc8c944ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee944b4, 90 },
    { 0x419944b4, 53 },
    { 0xc47944b4, 52 },
    { 0x5c2944b7, 65 },
    { 0, 0 },
    { 0xd2d944b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52944c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a944c5, 90 },
    { 0x8b5944c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf944ce, 98 },
    { 0x659944cf, 76 },
    { 0xaf8944d0, 33 },
    { 0, 0 },
    { 0xde7944d2, 90 },
    { 0x6d944d2, 95 },
    { 0x21b944d4, 33 },
    { 0, 0 },
    { 0x776944d6, 93 },
    { 0x452944d7, 78 },
    { 0xa1a944d7, 43 },
    { 0x751944d9, 23 },
    { 0xbd944d9, 98 },
    { 0, 0 },
    { 0x9ff944dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd944eb, 90 },
    { 0xefd944ec, 33 },
    { 0xc47944ed, 95 },
    { 0xb7a944ec, 88 },
    { 0x3c4944ef, 61 },
    { 0xc19944f0, 78 },
    { 0xd16944f1, 90 },
    { 0xfbc944f1, 99 },
    { 0, 0 },
    { 0x3f7944f4, 52 },
    { 0, 0 },
    { 0xfe7944f6, 43 },
    { 0, 0 },
    { 0x887944f8, 87 },
    { 0xdcf944f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x539944fc, 55 },
    { 0x5f5944fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd994503, 88 },
    { 0, 0 },
    { 0x5ca94505, 82 },
    { 0xa3c94506, 61 },
    { 0x2fe94507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b39450e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2394512, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5789451a, 33 },
    { 0, 0 },
    { 0xc909451c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3094520, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e94526, 40 },
    { 0xf1594526, 60 },
    { 0, 0 },
    { 0x4d394529, 90 },
    { 0xb9b9452a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc9452f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b094532, 61 },
    { 0x6894533, 90 },
    { 0x7f094534, 90 },
    { 0xe8894532, 100 },
    { 0x72694536, 33 },
    { 0x50994537, 43 },
    { 0, 0 },
    { 0x18c94539, 66 },
    { 0xd969453a, 55 },
    { 0x859453b, 68 },
    { 0xceb94539, 34 },
    { 0x7749453d, 99 },
    { 0, 0 },
    { 0xbe19453f, 33 },
    { 0, 0 },
    { 0xb2094541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a94545, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c94548, 90 },
    { 0, 0 },
    { 0x4469454a, 90 },
    { 0, 0 },
    { 0xdc19454c, 90 },
    { 0, 0 },
    { 0xa5a9454e, 75 },
    { 0x1f19454f, 90 },
    { 0xa19454f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b94555, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b94559, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569455c, 100 },
    { 0, 0 },
    { 0x1419455e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c94563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18094567, 78 },
    { 0x86a94568, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4849456f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d94572, 90 },
    { 0, 0 },
    { 0xa4d94574, 43 },
    { 0x2cf94575, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6894578, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429457d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d994582, 46 },
    { 0x86394583, 66 },
    { 0x90894582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a9458a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc409458d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a94591, 23 },
    { 0x9b694591, 34 },
    { 0x24294591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb94596, 76 },
    { 0x23694596, 61 },
    { 0x58a94596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869459c, 98 },
    { 0, 0 },
    { 0x7249459e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0945ae, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309945b2, 90 },
    { 0xe90945b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c945b8, 98 },
    { 0, 0 },
    { 0xb2945ba, 85 },
    { 0, 0 },
    { 0x563945bc, 66 },
    { 0, 0 },
    { 0xa71945be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc945c1, 100 },
    { 0, 0 },
    { 0xac7945c3, 46 },
    { 0, 0 },
    { 0xeb6945c5, 90 },
    { 0xc48945c5, 61 },
    { 0x177945c7, 66 },
    { 0x23e945c6, 23 },
    { 0x85d945c6, 100 },
    { 0xaa1945c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a945cd, 99 },
    { 0x5b3945ce, 61 },
    { 0xd90945cf, 33 },
    { 0x660945ce, 95 },
    { 0, 0 },
    { 0x1c0945d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a945d5, 90 },
    { 0, 0 },
    { 0x2bf945d7, 98 },
    { 0x939945d8, 77 },
    { 0x44945d8, 90 },
    { 0, 0 },
    { 0x9e945db, 43 },
    { 0x21945dc, 23 },
    { 0, 0 },
    { 0x91f945de, 61 },
    { 0xd14945de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32945e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b945e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d945ec, 61 },
    { 0xfe5945ec, 61 },
    { 0x7d8945ee, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b945f4, 67 },
    { 0xaa945f4, 85 },
    { 0x1b945f5, 90 },
    { 0x12b945f7, 78 },
    { 0x1ae945f8, 76 },
    { 0, 0 },
    { 0x683945fa, 34 },
    { 0x15b945fa, 33 },
    { 0xc27945fc, 90 },
    { 0x23e945fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57494604, 98 },
    { 0x39894604, 90 },
    { 0x13c94606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce494611, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e94614, 81 },
    { 0xe8794615, 98 },
    { 0xa7a94614, 67 },
    { 0xe9e94616, 43 },
    { 0x3a594618, 76 },
    { 0x5d294619, 33 },
    { 0, 0 },
    { 0x93b9461b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16894624, 40 },
    { 0xca294624, 97 },
    { 0, 0 },
    { 0x8f994627, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xac89462a, 78 },
    { 0xa9b9462a, 76 },
    { 0x15d9462c, 55 },
    { 0xf809462c, 93 },
    { 0, 0 },
    { 0xe179462f, 27 },
    { 0, 0 },
    { 0xbe494631, 33 },
    { 0x32e94631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f94636, 97 },
    { 0xa5194637, 34 },
    { 0, 0 },
    { 0x7c394639, 90 },
    { 0xb859463a, 90 },
    { 0x63e94639, 33 },
    { 0x34e9463c, 69 },
    { 0xd7b9463a, 83 },
    { 0x689463b, 90 },
    { 0, 0 },
    { 0xab794640, 61 },
    { 0x53d94641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae894647, 76 },
    { 0x1b994647, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0294650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94694656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c94659, 90 },
    { 0, 0 },
    { 0x55a9465b, 23 },
    { 0, 0 },
    { 0xfbd9465d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x32394660, 76 },
    { 0, 0 },
    { 0x9b994662, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a294666, 66 },
    { 0x85c94667, 33 },
    { 0x3f294668, 90 },
    { 0x94394669, 33 },
    { 0, 0 },
    { 0x5779466b, 75 },
    { 0, 0 },
    { 0x3a29466d, 61 },
    { 0, 0 },
    { 0xcb89466f, 43 },
    { 0xa2194670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d9467a, 98 },
    { 0x19d9467a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf299467e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30094681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1994689, 98 },
    { 0x93494689, 90 },
    { 0, 0 },
    { 0xe169468c, 95 },
    { 0xb8d9468d, 55 },
    { 0x5ca9468c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe94697, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109469b, 99 },
    { 0x84d9469c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41946a0, 33 },
    { 0xd71946a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee946a6, 76 },
    { 0xf37946a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed946aa, 23 },
    { 0, 0 },
    { 0x4946ac, 90 },
    { 0, 0 },
    { 0xaf946ae, 33 },
    { 0x84d946af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864946b4, 33 },
    { 0x6e0946b5, 95 },
    { 0, 0 },
    { 0xa0b946b7, 95 },
    { 0x373946b8, 34 },
    { 0xb38946b9, 34 },
    { 0xb83946ba, 90 },
    { 0x937946bb, 98 },
    { 0x352946ba, 90 },
    { 0, 0 },
    { 0x396946be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e946c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84946cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779946d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f946d4, 88 },
    { 0x605946d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454946d9, 33 },
    { 0xab0946d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3946e2, 98 },
    { 0x52946e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b946e6, 95 },
    { 0, 0 },
    { 0x22a946e8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7946eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257946f0, 90 },
    { 0xdeb946f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a946f4, 91 },
    { 0x65e946f4, 61 },
    { 0x265946f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd946fb, 76 },
    { 0xfdd946fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa94701, 64 },
    { 0, 0 },
    { 0x97994703, 77 },
    { 0x30a94703, 36 },
    { 0xdc294705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89470a, 85 },
    { 0x709470b, 33 },
    { 0xfdd9470c, 91 },
    { 0x3809470d, 56 },
    { 0xdaf9470e, 95 },
    { 0xd8c9470d, 90 },
    { 0x2d59470d, 97 },
    { 0xd269470a, 95 },
    { 0x3319470c, 90 },
    { 0xfbb94711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40594716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a9471a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9471d, 93 },
    { 0, 0 },
    { 0x3849471f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a94728, 90 },
    { 0xd2394729, 36 },
    { 0xe99472a, 55 },
    { 0x5a19472b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb19472f, 76 },
    { 0, 0 },
    { 0xdf994731, 90 },
    { 0xc5394732, 61 },
    { 0, 0 },
    { 0xae794734, 67 },
    { 0xf8b94734, 40 },
    { 0, 0 },
    { 0x6094737, 95 },
    { 0xfbe94738, 76 },
    { 0x55b94739, 61 },
    { 0x8e79473a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf89473f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff194745, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x65194748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab99474c, 33 },
    { 0, 0 },
    { 0x62c9474e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5494752, 33 },
    { 0xf5194753, 34 },
    { 0, 0 },
    { 0xfe094755, 93 },
    { 0, 0 },
    { 0x7294757, 66 },
    { 0xf7e94758, 78 },
    { 0, 0 },
    { 0xe829475a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38794765, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d9476a, 43 },
    { 0, 0 },
    { 0xa859476c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b29476f, 61 },
    { 0x26b94770, 98 },
    { 0, 0 },
    { 0x60494772, 34 },
    { 0x81e94773, 98 },
    { 0, 0 },
    { 0xb3b94775, 34 },
    { 0x6b294776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b29477a, 85 },
    { 0, 0 },
    { 0xa309477c, 98 },
    { 0, 0 },
    { 0x89477e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b94781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb49478c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d94791, 99 },
    { 0xe8b94792, 34 },
    { 0, 0 },
    { 0xa0894794, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c494798, 34 },
    { 0x2cf94798, 61 },
    { 0, 0 },
    { 0x1a9479b, 57 },
    { 0xac49479c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8947a4, 23 },
    { 0xc15947a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44947a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8947ae, 66 },
    { 0x766947af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66947b8, 43 },
    { 0, 0 },
    { 0xc67947ba, 23 },
    { 0xd1c947bb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c947c0, 93 },
    { 0xf02947c1, 90 },
    { 0x6af947c1, 43 },
    { 0, 0 },
    { 0xf04947c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a947c7, 66 },
    { 0x68b947c8, 99 },
    { 0x454947c9, 61 },
    { 0xeb9947ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f947d0, 23 },
    { 0xda2947d1, 46 },
    { 0xd9c947d1, 84 },
    { 0xb8c947d1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda947d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2947d9, 76 },
    { 0, 0 },
    { 0xb98947db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e947e9, 76 },
    { 0xfba947ea, 98 },
    { 0, 0 },
    { 0x62947ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258947f5, 90 },
    { 0x1a2947f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5947fb, 95 },
    { 0xae0947fc, 61 },
    { 0x9f2947fd, 98 },
    { 0xaa0947fe, 99 },
    { 0, 0 },
    { 0x73894800, 61 },
    { 0, 0 },
    { 0x4394802, 33 },
    { 0xfe594802, 8 },
    { 0x10494803, 29 },
    { 0, 0 },
    { 0xa5694806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77094809, 84 },
    { 0xf229480a, 78 },
    { 0x8f29480b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2394812, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca794816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef9481d, 33 },
    { 0x71e9481e, 98 },
    { 0, 0 },
    { 0xbf194820, 90 },
    { 0, 0 },
    { 0xf4194822, 67 },
    { 0xa9294823, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87294827, 98 },
    { 0x7d094828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e9482b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a94830, 76 },
    { 0xc2c94831, 88 },
    { 0x43994832, 76 },
    { 0x19394831, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e94837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a94842, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f59484a, 33 },
    { 0xf079484b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b9484e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac794851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f194855, 76 },
    { 0, 0 },
    { 0xb2794857, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58794860, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb94863, 52 },
    { 0, 0 },
    { 0x99f94865, 76 },
    { 0xf5f94866, 95 },
    { 0xef894865, 98 },
    { 0xdb294865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799486b, 33 },
    { 0x6f59486c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9486f, 72 },
    { 0, 0 },
    { 0x38f94871, 92 },
    { 0, 0 },
    { 0x56694873, 90 },
    { 0x8b394874, 100 },
    { 0xcd594874, 98 },
    { 0xb0d94876, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f9487d, 80 },
    { 0, 0 },
    { 0xcdb9487f, 100 },
    { 0x20094880, 90 },
    { 0, 0 },
    { 0x76c94882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c94886, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c79488a, 23 },
    { 0x94f9488a, 95 },
    { 0xec59488c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58994890, 92 },
    { 0x4ce94890, 17 },
    { 0, 0 },
    { 0x6a894893, 95 },
    { 0x5594893, 90 },
    { 0, 0 },
    { 0x79494896, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529489d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd948a0, 90 },
    { 0xcce948a0, 93 },
    { 0x20e948a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a948a5, 93 },
    { 0x86f948a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52948aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6948ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x280948b1, 76 },
    { 0x787948b2, 98 },
    { 0x2a5948b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87948b7, 75 },
    { 0xcfb948b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb948bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9948be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd948c4, 67 },
    { 0x8c8948c5, 61 },
    { 0, 0 },
    { 0x872948c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200948cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98948d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4948d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x754948d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x646948d9, 33 },
    { 0x195948da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c948df, 64 },
    { 0, 0 },
    { 0xce9948e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132948e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17948ea, 90 },
    { 0x291948eb, 81 },
    { 0, 0 },
    { 0x12c948ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82948f0, 61 },
    { 0, 0 },
    { 0x264948f2, 23 },
    { 0xa12948f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7948f7, 61 },
    { 0x8c3948f7, 23 },
    { 0xd85948f9, 98 },
    { 0x61c948f9, 99 },
    { 0, 0 },
    { 0xaa4948fc, 101 },
    { 0x87c948fd, 23 },
    { 0xd36948fc, 43 },
    { 0xe52948ff, 17 },
    { 0xa7a948ff, 55 },
    { 0x7ae94901, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x75594904, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b394907, 43 },
    { 0xf8e94908, 81 },
    { 0x3eb94909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9490d, 77 },
    { 0xec39490d, 83 },
    { 0xb339490d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc94912, 68 },
    { 0, 0 },
    { 0x4594914, 99 },
    { 0, 0 },
    { 0x2d694916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459491a, 33 },
    { 0x1c39491b, 40 },
    { 0xd99491b, 23 },
    { 0x999491d, 87 },
    { 0, 0 },
    { 0xa089491f, 95 },
    { 0x7c194920, 81 },
    { 0, 0 },
    { 0xd9e94922, 75 },
    { 0x4e394922, 33 },
    { 0, 0 },
    { 0xac094925, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf19492d, 98 },
    { 0, 0 },
    { 0xc4d9492f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74394932, 23 },
    { 0xf194933, 61 },
    { 0x8194934, 65 },
    { 0xb2194933, 55 },
    { 0x1bb94935, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84394939, 96 },
    { 0x8ef9493a, 40 },
    { 0x4a49493b, 93 },
    { 0x3299493c, 55 },
    { 0x88c9493c, 99 },
    { 0x70d9493b, 93 },
    { 0xbef9493c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe199494d, 78 },
    { 0, 0 },
    { 0xa0a9494f, 64 },
    { 0x70194950, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c894953, 76 },
    { 0x3e694953, 55 },
    { 0xcc794954, 61 },
    { 0, 0 },
    { 0xc1294957, 76 },
    { 0xc7194958, 90 },
    { 0, 0 },
    { 0xb269495a, 95 },
    { 0xb1a9495a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9994964, 33 },
    { 0, 0 },
    { 0x41894966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5339496a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d9496e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9694971, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8094975, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b9497a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519497f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c94989, 33 },
    { 0xed79498a, 33 },
    { 0xc8994989, 61 },
    { 0x6b69498b, 98 },
    { 0xc99498c, 33 },
    { 0x5929498e, 55 },
    { 0xeb09498e, 82 },
    { 0xddb9498a, 34 },
    { 0xcdf94991, 43 },
    { 0x80594991, 98 },
    { 0x3cd94993, 33 },
    { 0xe9f94994, 90 },
    { 0xa8394994, 55 },
    { 0x45a94994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d79499c, 90 },
    { 0, 0 },
    { 0xd419499e, 52 },
    { 0x5fa9499f, 33 },
    { 0x831949a0, 33 },
    { 0x497949a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d949a9, 99 },
    { 0x3c949aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28949b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4949b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0949ba, 33 },
    { 0xe23949bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a949be, 93 },
    { 0xe2e949bf, 46 },
    { 0x99a949c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x916949c3, 66 },
    { 0, 0 },
    { 0xb27949c5, 23 },
    { 0xcef949c6, 61 },
    { 0xcb9949c6, 98 },
    { 0x2fb949c8, 85 },
    { 0x7ee949c9, 87 },
    { 0, 0 },
    { 0x5d0949cb, 90 },
    { 0, 0 },
    { 0xcb9949cd, 90 },
    { 0x5c7949ce, 67 },
    { 0, 0 },
    { 0x722949d0, 76 },
    { 0x7a7949d1, 23 },
    { 0, 0 },
    { 0x6f4949d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb949d6, 33 },
    { 0xa45949d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750949da, 76 },
    { 0x150949da, 46 },
    { 0x50e949da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147949e0, 95 },
    { 0x8fc949e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3949e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x283949ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f949ed, 43 },
    { 0x509949ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b949f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79949fe, 33 },
    { 0xa12949ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f94a04, 76 },
    { 0xe2794a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b94a09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21894a0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39994a13, 87 },
    { 0xbd194a14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5394a19, 61 },
    { 0x87e94a19, 90 },
    { 0xd3694a1b, 98 },
    { 0xd6894a1c, 90 },
    { 0xf4694a1c, 98 },
    { 0xcd794a1e, 23 },
    { 0xd9f94a1f, 95 },
    { 0x73494a1e, 95 },
    { 0x44d94a1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f94a25, 95 },
    { 0xaa294a25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef794a29, 33 },
    { 0x1b94a2a, 90 },
    { 0, 0 },
    { 0x3ed94a2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2294a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a94a36, 90 },
    { 0, 0 },
    { 0x8e994a38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae394a3e, 84 },
    { 0xf3694a3f, 95 },
    { 0x8c294a3e, 43 },
    { 0xa7194a3e, 67 },
    { 0x37194a42, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7394a46, 61 },
    { 0x63f94a47, 65 },
    { 0xeb494a48, 95 },
    { 0, 0 },
    { 0x4694a4a, 34 },
    { 0x69594a4b, 78 },
    { 0xeaa94a4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e94a4f, 90 },
    { 0xd8c94a50, 46 },
    { 0, 0 },
    { 0x1c194a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e994a57, 90 },
    { 0, 0 },
    { 0x15094a59, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c94a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b794a60, 33 },
    { 0xcc94a61, 91 },
    { 0, 0 },
    { 0x2ab94a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26294a69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c94a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77794a72, 87 },
    { 0, 0 },
    { 0x9794a74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13694a7b, 55 },
    { 0xbb694a7c, 33 },
    { 0, 0 },
    { 0x11d94a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad94a83, 93 },
    { 0xcf94a84, 33 },
    { 0, 0 },
    { 0xff194a86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63994a8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f94a94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b494a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5094a9f, 33 },
    { 0, 0 },
    { 0xb9c94aa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65894aa6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6294aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b94aac, 43 },
    { 0xfb894aad, 76 },
    { 0x27e94aad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b94ab2, 66 },
    { 0x69794ab3, 46 },
    { 0xd4d94ab4, 33 },
    { 0, 0 },
    { 0xe5294ab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92594aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c94aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a94acf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f494ad2, 76 },
    { 0x25594ad2, 61 },
    { 0x30094ad4, 67 },
    { 0xed994ad3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7394adb, 33 },
    { 0xb3694adc, 95 },
    { 0x78d94add, 69 },
    { 0xded94ade, 95 },
    { 0x16494ade, 34 },
    { 0x71b94ae0, 54 },
    { 0xdb494ae0, 33 },
    { 0x62894ae2, 46 },
    { 0x20794ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41894ae8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c94af1, 34 },
    { 0x37694af2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58594af9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa594b00, 76 },
    { 0xf8594b01, 23 },
    { 0xb1d94b00, 91 },
    { 0x7894b02, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b94b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4594b0f, 33 },
    { 0, 0 },
    { 0xed094b11, 98 },
    { 0, 0 },
    { 0xc6c94b13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af94b16, 90 },
    { 0x23394b17, 23 },
    { 0x86294b18, 93 },
    { 0x31794b19, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df94b1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3594b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b94b26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f694b2b, 76 },
    { 0xfc294b2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf594b30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a94b34, 65 },
    { 0x67394b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9894b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24894b48, 78 },
    { 0, 0 },
    { 0x76994b4a, 95 },
    { 0x35094b4b, 65 },
    { 0, 0 },
    { 0x38094b4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd94b50, 34 },
    { 0xac494b51, 88 },
    { 0x2a194b50, 34 },
    { 0x4fe94b50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84994b58, 61 },
    { 0, 0 },
    { 0x6c94b5a, 61 },
    { 0xdeb94b5b, 55 },
    { 0xfb694b5b, 90 },
    { 0, 0 },
    { 0xcfc94b5e, 95 },
    { 0xdc694b5f, 76 },
    { 0xf7094b60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f494b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28994b69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47394b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41394b75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b7a, 33 },
    { 0x14894b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894b7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b94b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c794b8c, 90 },
    { 0, 0 },
    { 0xcd394b8e, 40 },
    { 0xb2c94b8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f794b93, 33 },
    { 0, 0 },
    { 0x58794b95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e094b99, 34 },
    { 0x3e294b9a, 98 },
    { 0xbdf94b9a, 99 },
    { 0x62a94b9c, 23 },
    { 0xad094b99, 90 },
    { 0, 0 },
    { 0x3d594b9f, 91 },
    { 0xbe394ba0, 90 },
    { 0xa4a94ba1, 90 },
    { 0x68394ba1, 55 },
    { 0x8a194ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f94ba7, 55 },
    { 0x3ac94ba7, 33 },
    { 0x1ae94ba9, 23 },
    { 0xbb994baa, 23 },
    { 0x15b94baa, 87 },
    { 0, 0 },
    { 0x39294bad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81394bb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e94bb5, 55 },
    { 0xe5394bb6, 61 },
    { 0, 0 },
    { 0xe4494bb8, 34 },
    { 0x3cd94bb9, 23 },
    { 0x7c694bb8, 93 },
    { 0, 0 },
    { 0x53e94bbc, 61 },
    { 0x88794bbd, 90 },
    { 0xfcf94bbe, 23 },
    { 0, 0 },
    { 0xa6594bc0, 90 },
    { 0xd8394bc0, 87 },
    { 0x6394bc0, 61 },
    { 0xb9c94bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e494bc6, 85 },
    { 0, 0 },
    { 0xc8e94bc8, 46 },
    { 0xfe394bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa994bcc, 75 },
    { 0x31394bcc, 90 },
    { 0xcc94bce, 33 },
    { 0x9f394bcf, 81 },
    { 0x88194bd0, 55 },
    { 0xf5394bce, 98 },
    { 0x54b94bcc, 55 },
    { 0x2c994bd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba94bd7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf94bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d794bde, 78 },
    { 0xd3794bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c94be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0294be8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c494bf1, 33 },
    { 0xa4c94bf2, 83 },
    { 0xcca94bf1, 34 },
    { 0, 0 },
    { 0xd0694bf5, 98 },
    { 0, 0 },
    { 0x1b294bf7, 33 },
    { 0x85c94bf7, 95 },
    { 0x61b94bf9, 90 },
    { 0x7c94bfa, 61 },
    { 0xc9294bfb, 90 },
    { 0x31c94bfc, 99 },
    { 0x80194bfc, 95 },
    { 0xa6494bfe, 23 },
    { 0xd7994bfd, 61 },
    { 0xe9a94bff, 52 },
    { 0xd0c94bfe, 67 },
    { 0xd8294c02, 91 },
    { 0x7cc94c02, 61 },
    { 0xa1294c03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0294c07, 17 },
    { 0, 0 },
    { 0xbd094c09, 98 },
    { 0, 0 },
    { 0x3f094c0b, 90 },
    { 0x1cc94c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f894c10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd94c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf94c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b894c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99894c21, 76 },
    { 0xc0994c22, 95 },
    { 0x75f94c22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60794c27, 98 },
    { 0, 0 },
    { 0x6b194c29, 84 },
    { 0xd7594c2a, 85 },
    { 0x1c894c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa94c30, 98 },
    { 0xfd694c31, 33 },
    { 0x6de94c32, 93 },
    { 0xa6394c31, 90 },
    { 0, 0 },
    { 0xd9494c35, 33 },
    { 0xcd194c36, 90 },
    { 0x92a94c36, 90 },
    { 0, 0 },
    { 0x5b94c39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b94c3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57194c45, 61 },
    { 0xd5e94c45, 61 },
    { 0x1e094c47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5694c55, 90 },
    { 0xe5194c55, 95 },
    { 0, 0 },
    { 0xc1794c58, 61 },
    { 0x83d94c58, 55 },
    { 0, 0 },
    { 0x98994c5b, 34 },
    { 0xa1094c5c, 33 },
    { 0x7b494c5d, 33 },
    { 0xd194c5e, 84 },
    { 0, 0 },
    { 0x84194c60, 55 },
    { 0x25f94c61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63194c64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd94c6b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de94c6e, 65 },
    { 0x36494c6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33794c73, 99 },
    { 0x8be94c73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb394c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c194c7c, 96 },
    { 0x4e694c7c, 85 },
    { 0xd8294c7e, 99 },
    { 0xea994c7e, 93 },
    { 0x8c694c7d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff594c8a, 61 },
    { 0xaca94c8b, 91 },
    { 0, 0 },
    { 0xf7694c8d, 33 },
    { 0x7e894c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f94c93, 90 },
    { 0x73394c93, 34 },
    { 0, 0 },
    { 0xf8994c96, 95 },
    { 0, 0 },
    { 0x2a294c98, 90 },
    { 0x61294c99, 34 },
    { 0x3794c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead94ca8, 23 },
    { 0x57b94ca9, 43 },
    { 0x8e494caa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd194cb0, 67 },
    { 0, 0 },
    { 0x7e394cb2, 90 },
    { 0xfda94cb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a94cb6, 98 },
    { 0, 0 },
    { 0x2b94cb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13494cbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c94cc3, 34 },
    { 0x8e94cc4, 90 },
    { 0, 0 },
    { 0x56b94cc6, 98 },
    { 0xe0194cc7, 55 },
    { 0, 0 },
    { 0xc1e94cc9, 100 },
    { 0x52094cca, 91 },
    { 0x62294ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47994cd0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e94cd6, 61 },
    { 0x6cb94cd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13394cda, 90 },
    { 0x89c94cdb, 90 },
    { 0, 0 },
    { 0x36194cdd, 97 },
    { 0x19194cde, 91 },
    { 0x43e94cde, 34 },
    { 0x96b94ce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb94ce3, 98 },
    { 0x5db94ce3, 97 },
    { 0x47694ce5, 23 },
    { 0xcff94ce6, 61 },
    { 0, 0 },
    { 0xbc894ce8, 61 },
    { 0, 0 },
    { 0xf3e94cea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa94cf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2594cf7, 40 },
    { 0x6a194cf8, 33 },
    { 0x8d694cf7, 61 },
    { 0, 0 },
    { 0x2d294cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f94cfe, 34 },
    { 0, 0 },
    { 0x39794d00, 95 },
    { 0x53994d00, 55 },
    { 0, 0 },
    { 0xa1e94d03, 95 },
    { 0x1b694d04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d94d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb94d10, 23 },
    { 0x63894d10, 83 },
    { 0x88394d11, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f94d16, 90 },
    { 0xf594d17, 61 },
    { 0, 0 },
    { 0xa1194d19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1294d1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a094d23, 61 },
    { 0, 0 },
    { 0x50294d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a094d28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb994d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f94d36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24494d3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2794d3e, 33 },
    { 0, 0 },
    { 0xdb294d40, 77 },
    { 0xbf194d41, 68 },
    { 0x8d094d42, 90 },
    { 0x8bf94d42, 23 },
    { 0xaac94d44, 84 },
    { 0xbcd94d45, 90 },
    { 0x10194d45, 87 },
    { 0x1ea94d47, 98 },
    { 0x84b94d45, 23 },
    { 0, 0 },
    { 0x32f94d4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b94d56, 95 },
    { 0x27b94d57, 90 },
    { 0x18a94d58, 23 },
    { 0x77794d59, 61 },
    { 0, 0 },
    { 0xa8694d5b, 61 },
    { 0xd6f94d5c, 100 },
    { 0x50294d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf94d64, 78 },
    { 0x8dc94d65, 46 },
    { 0x48294d66, 33 },
    { 0x9a594d64, 46 },
    { 0x4e94d68, 76 },
    { 0xf794d67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb594d6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf294d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3594d77, 61 },
    { 0x57794d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf94d7b, 50 },
    { 0x9c94d7c, 98 },
    { 0, 0 },
    { 0xbc94d7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f94d83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89794d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad94d8d, 90 },
    { 0x4e694d8e, 90 },
    { 0x60e94d8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6894d92, 55 },
    { 0xd4394d92, 98 },
    { 0x84394d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56894d99, 87 },
    { 0, 0 },
    { 0x10494d9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f294d9e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f694da1, 23 },
    { 0x46f94da2, 81 },
    { 0, 0 },
    { 0xb7094da4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91694da8, 23 },
    { 0x6b894da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b094db1, 90 },
    { 0x17294db2, 23 },
    { 0, 0 },
    { 0x26794db4, 52 },
    { 0x6c094db5, 46 },
    { 0x97c94db5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0794dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b94dc1, 90 },
    { 0xefe94dc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef94dc6, 77 },
    { 0x56c94dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34794dcc, 91 },
    { 0, 0 },
    { 0xe5294dce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27494dd5, 23 },
    { 0xff094dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf994ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94294de3, 33 },
    { 0x66994de4, 67 },
    { 0x1d894de3, 23 },
    { 0xc6594de6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d94de9, 34 },
    { 0, 0 },
    { 0xbfa94deb, 46 },
    { 0, 0 },
    { 0xe6c94ded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92594df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b94df3, 90 },
    { 0, 0 },
    { 0x5d194df5, 90 },
    { 0xfe594df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad94dfa, 76 },
    { 0xd1f94dfb, 88 },
    { 0, 0 },
    { 0x5b094dfd, 76 },
    { 0x6ae94dfe, 76 },
    { 0x65094dff, 87 },
    { 0, 0 },
    { 0xc7b94e01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2494e06, 98 },
    { 0xd7494e06, 98 },
    { 0, 0 },
    { 0x1ed94e09, 55 },
    { 0, 0 },
    { 0xfc694e0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c794e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad94e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2894e15, 23 },
    { 0x93694e15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa94e1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa994e22, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a94e25, 33 },
    { 0, 0 },
    { 0x2d994e27, 99 },
    { 0, 0 },
    { 0x2a94e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb394e2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x60694e2f, 43 },
    { 0x8c194e30, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b394e34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f94e38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8094e3b, 78 },
    { 0xf9994e3b, 55 },
    { 0x29b94e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51394e4a, 90 },
    { 0, 0 },
    { 0x32e94e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe94e4f, 90 },
    { 0, 0 },
    { 0xc4294e51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18794e55, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c094e5a, 84 },
    { 0xbde94e5a, 23 },
    { 0, 0 },
    { 0x9cf94e5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4694e66, 90 },
    { 0, 0 },
    { 0x2ab94e68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba94e6b, 33 },
    { 0x65694e6c, 98 },
    { 0, 0 },
    { 0x6a294e6e, 81 },
    { 0, 0 },
    { 0x89894e70, 95 },
    { 0x3ca94e71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90694e7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94894e83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa94e8a, 33 },
    { 0, 0 },
    { 0xd7c94e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c94e90, 93 },
    { 0xb8494e91, 34 },
    { 0, 0 },
    { 0xcef94e93, 77 },
    { 0x4ea94e93, 61 },
    { 0x27594e94, 80 },
    { 0xa9e94e93, 66 },
    { 0, 0 },
    { 0xff394e98, 33 },
    { 0, 0 },
    { 0x7bc94e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8194e9e, 61 },
    { 0xa1b94e9f, 76 },
    { 0x62294ea0, 75 },
    { 0xe4194ea0, 34 },
    { 0x8f994ea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7194ea7, 65 },
    { 0, 0 },
    { 0xc8394ea9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b94eaf, 33 },
    { 0x4494eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff894eb7, 55 },
    { 0xf0794eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c494ebc, 76 },
    { 0x8a394ebd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb94ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1294ec5, 40 },
    { 0x67994ec6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb794ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4994ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5094ed4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0394ed7, 51 },
    { 0xd5594ed7, 56 },
    { 0x69a94ed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f194edc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c94ee2, 34 },
    { 0x22c94ee3, 88 },
    { 0xda94ee2, 46 },
    { 0x25494ee3, 36 },
    { 0x8d094ee6, 61 },
    { 0x73694ee7, 76 },
    { 0x48d94ee8, 88 },
    { 0x7be94ee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e94eec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e94eef, 90 },
    { 0, 0 },
    { 0x2c094ef1, 76 },
    { 0x87794ef2, 84 },
    { 0, 0 },
    { 0x82c94ef4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e694efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f94f06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23194f09, 97 },
    { 0xe3094f09, 80 },
    { 0xa2694f0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc094f0e, 52 },
    { 0, 0 },
    { 0xf9694f10, 23 },
    { 0xe5d94f10, 34 },
    { 0x74294f12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af94f17, 76 },
    { 0x74694f18, 66 },
    { 0x97494f19, 61 },
    { 0x45e94f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a94f1e, 34 },
    { 0, 0 },
    { 0xbbc94f20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19094f29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3894f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a94f3b, 84 },
    { 0, 0 },
    { 0x35f94f3d, 95 },
    { 0x17b94f3d, 93 },
    { 0xff594f3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb894f44, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894f48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34394f56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3094f5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0594f5f, 87 },
    { 0xe9894f5f, 98 },
    { 0, 0 },
    { 0x97894f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7594f6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6494f6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde94f73, 33 },
    { 0xe9194f73, 76 },
    { 0xba394f75, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f94f78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1594f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6694f7e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f94f82, 98 },
    { 0x54394f82, 61 },
    { 0x62b94f84, 23 },
    { 0xf1394f85, 34 },
    { 0xe6a94f84, 90 },
    { 0x89194f85, 76 },
    { 0, 0 },
    { 0x9a694f89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d94f8f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x85194f92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf94f9a, 61 },
    { 0xc9694f9b, 46 },
    { 0xb9b94f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77894f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c94fa2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82994fad, 90 },
    { 0xd6b94fae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae94fb9, 34 },
    { 0, 0 },
    { 0xe3794fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f94fc0, 98 },
    { 0x4cb94fc1, 95 },
    { 0, 0 },
    { 0x88a94fc3, 100 },
    { 0, 0 },
    { 0xb9794fc5, 90 },
    { 0x7b594fc6, 73 },
    { 0x9d794fc7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde94fca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e194fcf, 61 },
    { 0x89994fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f294fd3, 93 },
    { 0xd8e94fd3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e94fd7, 61 },
    { 0, 0 },
    { 0xebc94fd9, 87 },
    { 0, 0 },
    { 0x52b94fdb, 76 },
    { 0xdc594fdc, 33 },
    { 0x8c494fdd, 33 },
    { 0xf7a94fde, 52 },
    { 0xaa794fdd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2394fe2, 98 },
    { 0xead94fe3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f994fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde494fef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f494ff7, 46 },
    { 0x2f194ff8, 59 },
    { 0xacc94ff7, 33 },
    { 0xc8394ffa, 90 },
    { 0x19e94ffb, 99 },
    { 0, 0 },
    { 0x73a94ffd, 61 },
    { 0x52294ffe, 61 },
    { 0xe5194fff, 33 },
    { 0x74995000, 90 },
    { 0x6ac95001, 90 },
    { 0x32694ffd, 95 },
    { 0x11b94ffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d95007, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc95010, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd9501c, 61 },
    { 0xdbb9501d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7395021, 55 },
    { 0x56195022, 55 },
    { 0xad795023, 66 },
    { 0xa2795021, 34 },
    { 0x68395025, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c095028, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb849502d, 61 },
    { 0x89502e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa95032, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7995036, 91 },
    { 0xa1595037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5395042, 23 },
    { 0x58395043, 33 },
    { 0xb4895043, 98 },
    { 0x82795043, 61 },
    { 0, 0 },
    { 0x95195047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec9504a, 23 },
    { 0, 0 },
    { 0x3709504c, 33 },
    { 0xee69504c, 95 },
    { 0, 0 },
    { 0x5609504f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e95055, 23 },
    { 0x32595056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f9505c, 34 },
    { 0x8cb9505c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52895067, 66 },
    { 0x85695068, 76 },
    { 0, 0 },
    { 0xf909506a, 23 },
    { 0xb7b9506a, 65 },
    { 0x2139506c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d95072, 95 },
    { 0, 0 },
    { 0xc0b95074, 98 },
    { 0x8195075, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d95079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5679507c, 90 },
    { 0x86a9507d, 33 },
    { 0xb159507e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xae695081, 95 },
    { 0xa4d95082, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a95085, 95 },
    { 0x7b895085, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09508a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9508f, 76 },
    { 0x5389508f, 95 },
    { 0xa2f95091, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df95098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9509b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161950a0, 61 },
    { 0x68f950a1, 85 },
    { 0x616950a1, 98 },
    { 0x4e950a2, 33 },
    { 0x40f950a4, 98 },
    { 0x41c950a1, 34 },
    { 0x95e950a6, 33 },
    { 0x32950a4, 98 },
    { 0x28950a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7950ab, 33 },
    { 0xdd950ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b950b1, 98 },
    { 0xafe950b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82950b5, 33 },
    { 0xf84950b5, 34 },
    { 0, 0 },
    { 0x9f950b8, 43 },
    { 0xc30950b8, 48 },
    { 0, 0 },
    { 0x84c950bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7950c3, 80 },
    { 0, 0 },
    { 0x22e950c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8950cd, 95 },
    { 0x11950ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1950d1, 43 },
    { 0x75950d2, 33 },
    { 0xb50950d3, 95 },
    { 0x201950d1, 75 },
    { 0x3b8950d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa950da, 98 },
    { 0xab5950db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c950df, 99 },
    { 0, 0 },
    { 0x15b950e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea950e5, 34 },
    { 0xeed950e5, 61 },
    { 0x22a950e7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b950ea, 61 },
    { 0x9f950ea, 66 },
    { 0x822950ec, 46 },
    { 0x20d950ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a950f3, 65 },
    { 0x37d950f3, 90 },
    { 0, 0 },
    { 0xd15950f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c950fb, 4 },
    { 0x951950fc, 8 },
    { 0xa5e950fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f95101, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7295104, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4195107, 90 },
    { 0x92595108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4495113, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda395117, 55 },
    { 0xdab95118, 95 },
    { 0, 0 },
    { 0x1d69511a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd95120, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c95123, 90 },
    { 0x5f795124, 78 },
    { 0x7ef95123, 61 },
    { 0xe4595124, 33 },
    { 0x9d395125, 43 },
    { 0xa9495125, 90 },
    { 0xed895129, 98 },
    { 0x5169512a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9512d, 46 },
    { 0xac29512d, 55 },
    { 0x4569512e, 33 },
    { 0xf3d95130, 81 },
    { 0x5e69512d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e795134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d595138, 76 },
    { 0x99995139, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b9513d, 76 },
    { 0x9139513e, 61 },
    { 0xabe9513f, 90 },
    { 0x1a695140, 17 },
    { 0x73f9513f, 87 },
    { 0x2869513e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b995148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a95151, 61 },
    { 0xa5d95151, 61 },
    { 0x7ec95153, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef95156, 61 },
    { 0xff395156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809515c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34495163, 61 },
    { 0x42e95163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1395167, 33 },
    { 0, 0 },
    { 0x66095169, 77 },
    { 0, 0 },
    { 0x42a9516b, 90 },
    { 0, 0 },
    { 0x4379516d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a95172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf695177, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb9517c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1995180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x13495183, 77 },
    { 0, 0 },
    { 0x65c95185, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2789518e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d295191, 55 },
    { 0x2c495192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99495195, 98 },
    { 0, 0 },
    { 0xd3195197, 46 },
    { 0xf5995198, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299519d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7951a7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x300951aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3951ad, 33 },
    { 0x234951ae, 90 },
    { 0xb9d951af, 95 },
    { 0xb5f951b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29951b3, 23 },
    { 0, 0 },
    { 0xc7951b5, 93 },
    { 0xb46951b6, 33 },
    { 0, 0 },
    { 0xb69951b8, 84 },
    { 0x1c4951b8, 33 },
    { 0xd88951b8, 90 },
    { 0x141951bb, 33 },
    { 0x565951bc, 33 },
    { 0x480951bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7951c6, 91 },
    { 0x1d9951c7, 33 },
    { 0x7d1951c7, 55 },
    { 0, 0 },
    { 0x9d5951ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d951ce, 90 },
    { 0xf4c951cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f951d2, 61 },
    { 0xbb7951d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1951d6, 98 },
    { 0x309951d6, 43 },
    { 0x84e951d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387951dd, 55 },
    { 0x355951dd, 99 },
    { 0x4c951df, 84 },
    { 0xdb3951dd, 43 },
    { 0x2ed951de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d951e6, 90 },
    { 0xfdc951e7, 76 },
    { 0x291951e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736951f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d951f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635951fd, 95 },
    { 0xe16951fd, 95 },
    { 0x3d8951ff, 23 },
    { 0x34795200, 61 },
    { 0x18b95201, 46 },
    { 0x9c595201, 55 },
    { 0xd2a95201, 43 },
    { 0xdf95202, 95 },
    { 0, 0 },
    { 0xbb795206, 55 },
    { 0x43b95207, 93 },
    { 0x9e495207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d9520b, 98 },
    { 0xdbc9520c, 33 },
    { 0, 0 },
    { 0xbaa9520e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85495216, 98 },
    { 0, 0 },
    { 0x58795218, 19 },
    { 0, 0 },
    { 0x2b59521a, 87 },
    { 0x4cb9521a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9521e, 43 },
    { 0, 0 },
    { 0xc7495220, 34 },
    { 0x55b95221, 90 },
    { 0x2395220, 61 },
    { 0x4895221, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xae495226, 78 },
    { 0, 0 },
    { 0x8c995228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8429522b, 93 },
    { 0xc969522c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39522f, 33 },
    { 0, 0 },
    { 0x4f295231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72795236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0095239, 65 },
    { 0xe5e9523a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa509523e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60195244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9524d, 33 },
    { 0xee29524e, 90 },
    { 0x9939524d, 95 },
    { 0, 0 },
    { 0x99995251, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa95254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf429525e, 61 },
    { 0xe599525f, 88 },
    { 0xeca95260, 43 },
    { 0xf495261, 61 },
    { 0, 0 },
    { 0x7ca95263, 61 },
    { 0xf6295264, 93 },
    { 0xd5995263, 52 },
    { 0, 0 },
    { 0x1c95267, 33 },
    { 0x5fe95268, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c9526c, 34 },
    { 0xcac9526d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2695273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e95276, 23 },
    { 0x13e95277, 65 },
    { 0x42d95277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9029527b, 90 },
    { 0, 0 },
    { 0x9869527d, 52 },
    { 0xb769527d, 87 },
    { 0x5b9527d, 87 },
    { 0xcaa95280, 46 },
    { 0x4d195280, 55 },
    { 0x59d95282, 81 },
    { 0xd0195283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3195286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf9528b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9528f, 43 },
    { 0xecf9528f, 90 },
    { 0, 0 },
    { 0x45c95292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21995299, 66 },
    { 0x5f99529a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99529d, 61 },
    { 0, 0 },
    { 0x7d59529f, 61 },
    { 0, 0 },
    { 0xba6952a1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa952a7, 43 },
    { 0x1b7952a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e952ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628952b5, 55 },
    { 0, 0 },
    { 0xd6952b7, 23 },
    { 0xd51952b8, 23 },
    { 0x8d0952b9, 98 },
    { 0, 0 },
    { 0x4c952bb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0952be, 98 },
    { 0x17e952bf, 65 },
    { 0x8be952bf, 99 },
    { 0x82f952c0, 84 },
    { 0xf22952c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a952ca, 76 },
    { 0, 0 },
    { 0x268952cc, 90 },
    { 0x97d952cc, 61 },
    { 0xba1952cc, 55 },
    { 0xae952cf, 34 },
    { 0x294952d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc952d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524952dc, 23 },
    { 0x4b952dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196952e1, 90 },
    { 0x3f2952e2, 76 },
    { 0x754952e2, 55 },
    { 0x83c952e4, 43 },
    { 0x74a952e5, 87 },
    { 0x11b952e4, 23 },
    { 0x158952e7, 91 },
    { 0x4b9952e8, 43 },
    { 0x23f952e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223952f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb952f4, 52 },
    { 0xee6952f5, 40 },
    { 0x564952f6, 33 },
    { 0x78c952f7, 88 },
    { 0xcbe952f4, 90 },
    { 0x66f952f9, 34 },
    { 0x5c5952f7, 100 },
    { 0x89b952fb, 84 },
    { 0x973952fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b95301, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4095305, 95 },
    { 0, 0 },
    { 0xc9095307, 98 },
    { 0x55795308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d95313, 61 },
    { 0xee695313, 61 },
    { 0, 0 },
    { 0x50595316, 17 },
    { 0xc0895317, 90 },
    { 0, 0 },
    { 0xb7c95319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b9531d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c9532e, 53 },
    { 0, 0 },
    { 0x86995330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb95335, 23 },
    { 0x66495336, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df9533a, 33 },
    { 0, 0 },
    { 0x5839533c, 23 },
    { 0, 0 },
    { 0x8449533e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4695344, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64095348, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d89534b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e29534e, 34 },
    { 0x33d9534f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe95354, 33 },
    { 0x49995355, 76 },
    { 0x41195356, 33 },
    { 0x52a95357, 91 },
    { 0xd3c95356, 19 },
    { 0x82b95354, 84 },
    { 0xd219535a, 100 },
    { 0xea99535b, 61 },
    { 0xc1d9535c, 75 },
    { 0xded9535a, 61 },
    { 0x80a95354, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b95363, 55 },
    { 0x61395364, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc795368, 93 },
    { 0x45d95369, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f29536e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4495375, 76 },
    { 0, 0 },
    { 0x64295377, 95 },
    { 0x4eb95377, 90 },
    { 0xec995379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e95385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9538a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c9538f, 87 },
    { 0xfda95390, 33 },
    { 0x4a895390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23595394, 23 },
    { 0x1c695395, 61 },
    { 0xcee95396, 61 },
    { 0x86c95396, 61 },
    { 0, 0 },
    { 0x15195399, 76 },
    { 0x4359539a, 85 },
    { 0x8f09539b, 98 },
    { 0, 0 },
    { 0x6479539d, 84 },
    { 0x20f9539e, 33 },
    { 0x8469539f, 34 },
    { 0x1a39539d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64953a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa953a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405953ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5953af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31953b3, 98 },
    { 0, 0 },
    { 0xb4a953b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53953b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f953bd, 61 },
    { 0, 0 },
    { 0x92953bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935953c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381953cb, 61 },
    { 0x5b953cb, 90 },
    { 0xe21953cd, 33 },
    { 0x42b953ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286953d4, 95 },
    { 0, 0 },
    { 0x941953d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397953dd, 90 },
    { 0x72e953de, 84 },
    { 0, 0 },
    { 0x7b6953e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15953e6, 33 },
    { 0x5d4953e7, 33 },
    { 0, 0 },
    { 0x881953e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f953ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x406953f0, 98 },
    { 0, 0 },
    { 0xe16953f2, 98 },
    { 0xfd8953f3, 22 },
    { 0x1f1953f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0953fc, 23 },
    { 0x53a953fd, 56 },
    { 0, 0 },
    { 0xdc3953ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc595403, 61 },
    { 0, 0 },
    { 0x15395405, 98 },
    { 0x8e295406, 43 },
    { 0xa6295406, 76 },
    { 0x43295407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0895412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a195415, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86595418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7609541c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94495421, 33 },
    { 0xc7f95422, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3095425, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97195428, 61 },
    { 0x2d895429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829542e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b95433, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef95437, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad09543c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd095447, 90 },
    { 0, 0 },
    { 0x92c95449, 61 },
    { 0x76c9544a, 17 },
    { 0x5c49544b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3969544f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17695454, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0595458, 33 },
    { 0, 0 },
    { 0x2189545a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3895464, 61 },
    { 0x20695465, 34 },
    { 0xf2795464, 81 },
    { 0, 0 },
    { 0xbfe95468, 98 },
    { 0xb6a95468, 61 },
    { 0, 0 },
    { 0x469546b, 98 },
    { 0, 0 },
    { 0x89a9546d, 43 },
    { 0x5259546e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d695471, 61 },
    { 0x1dc95472, 76 },
    { 0xb6f95473, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e695477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9547b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a29547f, 84 },
    { 0, 0 },
    { 0xff595481, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba95488, 23 },
    { 0, 0 },
    { 0xdda9548a, 65 },
    { 0xdef9548b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9548e, 33 },
    { 0, 0 },
    { 0xb8795490, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b9549a, 81 },
    { 0xd539549a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349549e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1954a2, 33 },
    { 0, 0 },
    { 0x146954a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e954a7, 81 },
    { 0x277954a8, 84 },
    { 0x988954a9, 46 },
    { 0xf83954aa, 61 },
    { 0x821954a8, 93 },
    { 0x623954a9, 61 },
    { 0, 0 },
    { 0x5bb954ae, 55 },
    { 0xee8954af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x711954b2, 55 },
    { 0x2bc954b2, 98 },
    { 0x25b954b3, 75 },
    { 0, 0 },
    { 0x5a954b6, 100 },
    { 0xf88954b6, 55 },
    { 0, 0 },
    { 0xbe5954b9, 34 },
    { 0x7b5954b9, 98 },
    { 0xbca954bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b954be, 61 },
    { 0x7ee954be, 34 },
    { 0x33b954c0, 75 },
    { 0x7e1954c1, 23 },
    { 0xd8f954c2, 55 },
    { 0x5c9954c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf954c9, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0954cc, 33 },
    { 0x714954cc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b954d0, 100 },
    { 0x83954d1, 97 },
    { 0xe8e954d2, 65 },
    { 0, 0 },
    { 0x2ce954d4, 43 },
    { 0x9f8954d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e954da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc954de, 23 },
    { 0xdc5954de, 75 },
    { 0x730954de, 55 },
    { 0xabd954e1, 88 },
    { 0x14b954de, 66 },
    { 0, 0 },
    { 0x93b954e4, 67 },
    { 0, 0 },
    { 0x827954e6, 43 },
    { 0xe02954e7, 23 },
    { 0xacf954e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36954eb, 61 },
    { 0xd0d954ec, 98 },
    { 0, 0 },
    { 0x4e4954ee, 61 },
    { 0x280954ee, 93 },
    { 0x757954f0, 66 },
    { 0x2b7954f0, 34 },
    { 0, 0 },
    { 0x508954f3, 52 },
    { 0x23b954f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5954f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9954ff, 95 },
    { 0, 0 },
    { 0xc2b95501, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b95506, 33 },
    { 0xaf995507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209550a, 46 },
    { 0x24f9550b, 98 },
    { 0x6509550c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9550f, 33 },
    { 0x7919550f, 78 },
    { 0x73895511, 61 },
    { 0xc4195512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63795516, 34 },
    { 0, 0 },
    { 0xb0c95518, 95 },
    { 0x52695519, 78 },
    { 0, 0 },
    { 0x7759551b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b9551f, 48 },
    { 0x16395520, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed95523, 76 },
    { 0, 0 },
    { 0xa3995525, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed95528, 55 },
    { 0, 0 },
    { 0xb389552a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3895532, 55 },
    { 0xc8295532, 65 },
    { 0, 0 },
    { 0x99395535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59553a, 33 },
    { 0xa6b9553a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d95540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90495544, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2395548, 71 },
    { 0x67895549, 33 },
    { 0, 0 },
    { 0x87d9554b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b95552, 95 },
    { 0x5f395552, 34 },
    { 0x1cb95554, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2495559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26995560, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d95564, 34 },
    { 0, 0 },
    { 0x53995566, 23 },
    { 0, 0 },
    { 0x5e295568, 33 },
    { 0x96695569, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f9556d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52995570, 23 },
    { 0x8c095571, 95 },
    { 0xfaf95572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16195577, 17 },
    { 0x91995578, 98 },
    { 0, 0 },
    { 0x73d9557a, 81 },
    { 0x3da9557a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe9557f, 66 },
    { 0, 0 },
    { 0x50d95581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88095584, 33 },
    { 0, 0 },
    { 0x2f395586, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be9558e, 78 },
    { 0x91b9558e, 91 },
    { 0x6859558e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf59559b, 81 },
    { 0x61b9559b, 78 },
    { 0x8139559c, 90 },
    { 0, 0 },
    { 0x4b99559f, 34 },
    { 0, 0 },
    { 0xafe955a1, 61 },
    { 0x8e6955a2, 90 },
    { 0x3fb955a2, 55 },
    { 0x44f955a1, 98 },
    { 0, 0 },
    { 0x8a1955a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b955ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a955af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6955b3, 33 },
    { 0, 0 },
    { 0xed3955b5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5955b8, 90 },
    { 0x496955b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898955bf, 91 },
    { 0, 0 },
    { 0xf31955c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd955c5, 33 },
    { 0xaaf955c6, 23 },
    { 0xf86955c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa955d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead955de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7955e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06955e4, 52 },
    { 0xb4f955e4, 93 },
    { 0, 0 },
    { 0x80d955e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c955ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7955ed, 87 },
    { 0x132955ed, 61 },
    { 0xfa7955ed, 23 },
    { 0x847955f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37955f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39955f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39955fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39560d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a95614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f995617, 95 },
    { 0, 0 },
    { 0x69795619, 33 },
    { 0x4679561a, 91 },
    { 0x41f9561a, 23 },
    { 0xd0295619, 61 },
    { 0x7319561d, 23 },
    { 0x4a49561e, 33 },
    { 0xc389561f, 55 },
    { 0, 0 },
    { 0x5c95621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c95629, 90 },
    { 0xd679562a, 95 },
    { 0x30d9562b, 85 },
    { 0x56c95629, 66 },
    { 0, 0 },
    { 0x5029562e, 100 },
    { 0x2109562f, 23 },
    { 0xff89562f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56195638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc819563d, 87 },
    { 0, 0 },
    { 0x18c9563f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81695643, 91 },
    { 0, 0 },
    { 0x51a95645, 55 },
    { 0, 0 },
    { 0xf095647, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc9564b, 99 },
    { 0xa2d9564c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4295651, 85 },
    { 0xec295652, 76 },
    { 0xd9995653, 90 },
    { 0, 0 },
    { 0x7a595655, 61 },
    { 0, 0 },
    { 0xcb195657, 93 },
    { 0x12495657, 61 },
    { 0xa5795659, 90 },
    { 0, 0 },
    { 0x3779565b, 33 },
    { 0x2149565c, 38 },
    { 0x1269565b, 61 },
    { 0x4619565e, 61 },
    { 0, 0 },
    { 0x81395660, 40 },
    { 0xd795660, 33 },
    { 0x5fe95662, 67 },
    { 0x5f695661, 90 },
    { 0x52e95664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd095667, 52 },
    { 0, 0 },
    { 0x88995669, 23 },
    { 0x1289566a, 96 },
    { 0xe1e9566a, 33 },
    { 0xbac9566c, 93 },
    { 0, 0 },
    { 0xafb9566e, 55 },
    { 0xa3e9566e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84095675, 61 },
    { 0, 0 },
    { 0x23c95677, 86 },
    { 0x91395677, 33 },
    { 0, 0 },
    { 0xbfc9567a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe679567f, 84 },
    { 0, 0 },
    { 0x53995681, 34 },
    { 0, 0 },
    { 0x52b95683, 90 },
    { 0x92395683, 98 },
    { 0x2c295684, 55 },
    { 0, 0 },
    { 0xc6495687, 67 },
    { 0x5d595688, 46 },
    { 0, 0 },
    { 0x2069568a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f9568d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac95692, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2095695, 55 },
    { 0x1895695, 66 },
    { 0xa2795695, 66 },
    { 0, 0 },
    { 0xbc595699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5956a0, 76 },
    { 0x7c956a1, 84 },
    { 0, 0 },
    { 0x89e956a3, 98 },
    { 0x64956a4, 43 },
    { 0xd9d956a4, 34 },
    { 0x8b3956a4, 80 },
    { 0, 0 },
    { 0xfd5956a8, 76 },
    { 0, 0 },
    { 0x36e956aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9956ae, 23 },
    { 0xf31956ae, 33 },
    { 0x298956af, 55 },
    { 0x86b956ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883956ca, 90 },
    { 0xbc4956cb, 61 },
    { 0x211956cb, 80 },
    { 0xe9956cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e956d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235956d9, 33 },
    { 0, 0 },
    { 0x753956db, 43 },
    { 0xccb956dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53956df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0956e4, 77 },
    { 0xb68956e5, 33 },
    { 0xcee956e6, 95 },
    { 0x602956e6, 98 },
    { 0, 0 },
    { 0xde8956e9, 94 },
    { 0, 0 },
    { 0xb6a956eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe956ef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82956f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9956f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51956ff, 46 },
    { 0xae9956ff, 80 },
    { 0, 0 },
    { 0xe5195702, 33 },
    { 0, 0 },
    { 0x3ec95704, 90 },
    { 0x56295705, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d9570f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d095714, 98 },
    { 0x5a095714, 33 },
    { 0x5bb95716, 88 },
    { 0xf5e95714, 61 },
    { 0xc5895718, 55 },
    { 0x30395714, 46 },
    { 0x9139571a, 23 },
    { 0x47a9571a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48695721, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53795731, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90695736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb95739, 56 },
    { 0x63795739, 61 },
    { 0, 0 },
    { 0x9249573c, 90 },
    { 0xc9f9573c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e95746, 100 },
    { 0, 0 },
    { 0x88995748, 61 },
    { 0, 0 },
    { 0x2189574a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d9574d, 98 },
    { 0xebd9574e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e95752, 95 },
    { 0, 0 },
    { 0x8bf95754, 96 },
    { 0x6d095755, 61 },
    { 0xc4f95755, 33 },
    { 0x94695755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee19575a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba09575d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a95761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5695765, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c95771, 55 },
    { 0x13b95772, 23 },
    { 0xdc295772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23795776, 33 },
    { 0, 0 },
    { 0xf0095778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5019577b, 67 },
    { 0, 0 },
    { 0x9d89577d, 77 },
    { 0xfa99577d, 61 },
    { 0, 0 },
    { 0x2c95780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9695784, 61 },
    { 0, 0 },
    { 0xed295786, 90 },
    { 0x3e695787, 99 },
    { 0xed95788, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9578d, 90 },
    { 0, 0 },
    { 0xb7b9578f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10195796, 33 },
    { 0, 0 },
    { 0x75395798, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29579c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900957a5, 61 },
    { 0x14b957a6, 60 },
    { 0x357957a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf957aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b957af, 55 },
    { 0xa04957b0, 61 },
    { 0x7db957b1, 90 },
    { 0x3d3957b2, 76 },
    { 0x3ee957b0, 98 },
    { 0x49957b3, 83 },
    { 0, 0 },
    { 0xe3957b6, 33 },
    { 0x4b957b7, 61 },
    { 0xb36957b6, 43 },
    { 0x487957b9, 90 },
    { 0x7ee957ba, 90 },
    { 0x1cf957ba, 17 },
    { 0x445957ba, 98 },
    { 0, 0 },
    { 0x9af957be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60957d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995957d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3957d9, 98 },
    { 0, 0 },
    { 0xd72957db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x560957de, 100 },
    { 0x9f957df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b957e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62957e7, 78 },
    { 0x3cf957e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135957ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297957f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81957f8, 91 },
    { 0, 0 },
    { 0x8f0957fa, 33 },
    { 0, 0 },
    { 0x8f9957fc, 78 },
    { 0xeeb957fc, 61 },
    { 0, 0 },
    { 0x8c2957ff, 66 },
    { 0, 0 },
    { 0xaa295801, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21095807, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929580d, 33 },
    { 0x3869580d, 99 },
    { 0x619580d, 23 },
    { 0x68f95810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2495816, 98 },
    { 0x40295816, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff95821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf649582a, 81 },
    { 0, 0 },
    { 0x4689582c, 33 },
    { 0x1879582d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b95830, 34 },
    { 0, 0 },
    { 0x33595832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b95841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee95848, 95 },
    { 0x77895849, 61 },
    { 0x71795848, 55 },
    { 0xfcf9584b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c95850, 23 },
    { 0xe1395851, 23 },
    { 0xb0c95852, 34 },
    { 0x29295852, 90 },
    { 0, 0 },
    { 0x5ee95855, 55 },
    { 0x75a95855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf979585e, 75 },
    { 0x3fd9585f, 66 },
    { 0x6d49585f, 34 },
    { 0x64e95861, 33 },
    { 0x1b795860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df95865, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42295869, 93 },
    { 0xfe39586a, 33 },
    { 0x3839586a, 76 },
    { 0xce69586a, 90 },
    { 0xbae9586d, 78 },
    { 0, 0 },
    { 0x7739586f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe95874, 93 },
    { 0x24195875, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa9587c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f95886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf49588c, 33 },
    { 0x4ef9588d, 98 },
    { 0, 0 },
    { 0xf619588f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85095893, 98 },
    { 0xcb595894, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b95897, 5 },
    { 0x59095897, 61 },
    { 0, 0 },
    { 0x7729589a, 81 },
    { 0x509589b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d958a5, 84 },
    { 0x3fb958a5, 76 },
    { 0xc17958a7, 90 },
    { 0x304958a7, 90 },
    { 0x41b958a7, 23 },
    { 0xd5c958a7, 61 },
    { 0x2b8958ab, 93 },
    { 0x884958ab, 33 },
    { 0x6dd958a5, 78 },
    { 0xc41958ae, 99 },
    { 0x8d958a8, 75 },
    { 0xfb958a7, 8 },
    { 0, 0 },
    { 0x8c5958b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2958b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x860958ba, 33 },
    { 0x3e7958bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a958c0, 23 },
    { 0x896958c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2958c4, 76 },
    { 0xca5958c4, 34 },
    { 0x3fd958c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31958cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7958d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc958d5, 46 },
    { 0xcf8958d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5958da, 93 },
    { 0x316958db, 93 },
    { 0x63a958dc, 40 },
    { 0x932958db, 78 },
    { 0x96b958de, 61 },
    { 0, 0 },
    { 0x25e958e0, 52 },
    { 0x24a958e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6958e5, 67 },
    { 0, 0 },
    { 0xa19958e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4958eb, 36 },
    { 0x5b958ec, 33 },
    { 0xc83958ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29958f2, 33 },
    { 0xdf2958f3, 93 },
    { 0x4ff958f2, 61 },
    { 0x302958f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5958f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06958fd, 33 },
    { 0xd6f958fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee495902, 61 },
    { 0x41095903, 90 },
    { 0xce995903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32195907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29590a, 91 },
    { 0x4919590b, 66 },
    { 0xaee9590b, 33 },
    { 0xd5e9590b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db95910, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde395913, 98 },
    { 0, 0 },
    { 0x42f95915, 90 },
    { 0, 0 },
    { 0x9ec95917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3559591b, 81 },
    { 0x7a19591c, 23 },
    { 0, 0 },
    { 0x2ad9591e, 17 },
    { 0x9fc9591f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2995923, 98 },
    { 0x6bb95924, 61 },
    { 0, 0 },
    { 0xbc795926, 65 },
    { 0xb1e95927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd489592b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c95933, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e95937, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d9593a, 65 },
    { 0x779593b, 87 },
    { 0x27d9593a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee9593f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55795942, 33 },
    { 0, 0 },
    { 0x4c995944, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0295949, 34 },
    { 0x3ab95949, 61 },
    { 0x2b195949, 61 },
    { 0xc6e9594c, 61 },
    { 0x7399594d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c795958, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5839595f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c69596d, 98 },
    { 0, 0 },
    { 0x72e9596f, 34 },
    { 0, 0 },
    { 0xbe395971, 100 },
    { 0x3ef95972, 90 },
    { 0xaab95973, 34 },
    { 0x7a795974, 53 },
    { 0x7c895973, 90 },
    { 0x90995975, 55 },
    { 0, 0 },
    { 0x91795978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2949597b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9597e, 33 },
    { 0x7b79597e, 91 },
    { 0, 0 },
    { 0xe9b95981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df95984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3695989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe629598e, 33 },
    { 0xd369598e, 33 },
    { 0x2839598f, 61 },
    { 0, 0 },
    { 0x7cb95992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c95996, 61 },
    { 0, 0 },
    { 0x1bd95998, 90 },
    { 0x63595999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3799599f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859959a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838959a8, 93 },
    { 0xa4a959a8, 61 },
    { 0xbd1959a8, 34 },
    { 0x8a0959aa, 61 },
    { 0x717959a9, 55 },
    { 0x90959ad, 27 },
    { 0xb23959ac, 95 },
    { 0xd4e959af, 33 },
    { 0xfa9959b0, 76 },
    { 0xc66959af, 33 },
    { 0xf28959b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b959b5, 23 },
    { 0x453959b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72959b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4959bc, 33 },
    { 0, 0 },
    { 0x82e959be, 77 },
    { 0, 0 },
    { 0x391959c0, 23 },
    { 0x58959c0, 43 },
    { 0xc68959c1, 23 },
    { 0xfea959c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf959c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e959c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29959d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487959d8, 91 },
    { 0x468959d9, 76 },
    { 0x16c959d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f959dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f959e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587959e5, 95 },
    { 0, 0 },
    { 0xf86959e7, 98 },
    { 0xbb3959e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94959eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f959f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced959f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x588959fa, 33 },
    { 0, 0 },
    { 0xc87959fc, 93 },
    { 0x1fc959fd, 90 },
    { 0xc24959fe, 91 },
    { 0x46d959fe, 52 },
    { 0, 0 },
    { 0x73495a01, 84 },
    { 0x7e295a02, 90 },
    { 0, 0 },
    { 0xc4f95a04, 98 },
    { 0x9c995a05, 34 },
    { 0x45595a06, 91 },
    { 0, 0 },
    { 0xf1995a08, 66 },
    { 0, 0 },
    { 0x38195a0a, 61 },
    { 0x82c95a0b, 90 },
    { 0x8d395a0c, 90 },
    { 0, 0 },
    { 0xcb295a0e, 61 },
    { 0x3e95a0e, 90 },
    { 0x83b95a10, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e195a16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90895a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25795a21, 56 },
    { 0x46395a22, 95 },
    { 0xfe195a23, 78 },
    { 0, 0 },
    { 0xd0195a25, 33 },
    { 0x55695a26, 33 },
    { 0xdb95a26, 23 },
    { 0x94295a28, 98 },
    { 0xf8895a29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c95a2c, 52 },
    { 0, 0 },
    { 0x89495a2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af95a33, 99 },
    { 0x8a995a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0295a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61495a43, 34 },
    { 0, 0 },
    { 0x39a95a45, 23 },
    { 0, 0 },
    { 0x5b95a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e695a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a95a55, 61 },
    { 0x69e95a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5795a5c, 98 },
    { 0xff895a5c, 55 },
    { 0x4fd95a5e, 64 },
    { 0xf3395a5e, 90 },
    { 0x6e195a60, 43 },
    { 0x13595a61, 61 },
    { 0x5ac95a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e95a66, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf95a6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4595a75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4395a78, 23 },
    { 0, 0 },
    { 0x72c95a7a, 43 },
    { 0xe795a7a, 95 },
    { 0x8f295a7b, 34 },
    { 0xd5495a7b, 55 },
    { 0x3a795a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73895a81, 33 },
    { 0x5d695a82, 34 },
    { 0xa295a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a95a86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67295a8a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d95a8f, 94 },
    { 0, 0 },
    { 0x5fb95a91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36995a95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76595a9e, 33 },
    { 0, 0 },
    { 0xede95aa0, 99 },
    { 0xe0e95aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b95aa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34195ab0, 47 },
    { 0, 0 },
    { 0x46a95ab2, 52 },
    { 0x4a895ab2, 90 },
    { 0x92a95ab4, 90 },
    { 0xd0895ab2, 52 },
    { 0, 0 },
    { 0x6b695ab7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3595ac0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22195ac5, 61 },
    { 0, 0 },
    { 0x5ea95ac7, 91 },
    { 0xf6295ac7, 98 },
    { 0x1d695ac7, 34 },
    { 0xa7795ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a95acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91695ad0, 76 },
    { 0xa4c95ad1, 61 },
    { 0x14495ad2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac195ad5, 76 },
    { 0x17995ad6, 90 },
    { 0x5f695ad6, 98 },
    { 0x44495ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc595ae1, 74 },
    { 0, 0 },
    { 0xaed95ae3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27295af2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xff695af5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50695afd, 23 },
    { 0x34195afd, 43 },
    { 0, 0 },
    { 0xa695b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9295b04, 98 },
    { 0, 0 },
    { 0x1c595b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d695b09, 83 },
    { 0x7b695b09, 98 },
    { 0xcd195b09, 55 },
    { 0x8f995b0c, 90 },
    { 0x8b295b0a, 88 },
    { 0xd1695b0e, 43 },
    { 0, 0 },
    { 0x8fc95b10, 76 },
    { 0xc3d95b10, 90 },
    { 0x29395b11, 93 },
    { 0x8e395b13, 61 },
    { 0x88f95b14, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89995b19, 91 },
    { 0xe0d95b1a, 43 },
    { 0, 0 },
    { 0x10995b1c, 91 },
    { 0x4db95b1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba995b23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6395b27, 95 },
    { 0xdaa95b27, 33 },
    { 0x15b95b29, 33 },
    { 0x1d195b2a, 98 },
    { 0x66195b29, 90 },
    { 0x3b395b2c, 76 },
    { 0x9c095b2d, 23 },
    { 0x87b95b28, 100 },
    { 0xe695b2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f95b34, 77 },
    { 0, 0 },
    { 0xe9595b36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd195b3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c95b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c95b41, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9195b49, 90 },
    { 0x80595b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83795b52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4495b55, 90 },
    { 0x4b095b56, 17 },
    { 0xe6b95b56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c95b5b, 90 },
    { 0, 0 },
    { 0x8cf95b5d, 34 },
    { 0, 0 },
    { 0xdcc95b5f, 100 },
    { 0x99795b60, 65 },
    { 0x9dc95b5f, 33 },
    { 0x59695b5f, 33 },
    { 0xe1695b61, 95 },
    { 0, 0 },
    { 0x73e95b65, 98 },
    { 0xcf195b66, 23 },
    { 0xacc95b67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4295b6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a95b77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f95b7d, 95 },
    { 0x28195b7e, 88 },
    { 0x1b595b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57095b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a95b86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f95b8a, 9 },
    { 0x20595b8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba95b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a395b91, 66 },
    { 0, 0 },
    { 0x26b95b93, 95 },
    { 0xe0995b93, 23 },
    { 0, 0 },
    { 0x47995b96, 75 },
    { 0xe4a95b96, 43 },
    { 0, 0 },
    { 0xdb295b99, 33 },
    { 0x47295b9a, 61 },
    { 0, 0 },
    { 0xbda95b9c, 98 },
    { 0xbb795b9c, 90 },
    { 0x595b9e, 47 },
    { 0, 0 },
    { 0x94995ba0, 98 },
    { 0x30f95ba1, 98 },
    { 0xf8d95ba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0295ba8, 91 },
    { 0xb3e95ba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5295bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6995bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a95bb6, 95 },
    { 0, 0 },
    { 0x4a995bb8, 33 },
    { 0, 0 },
    { 0x30e95bba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38495bc1, 98 },
    { 0x6e095bc1, 95 },
    { 0x65395bc3, 23 },
    { 0xa3f95bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b795bc9, 55 },
    { 0xf9695bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f95bd0, 85 },
    { 0xe4995bd1, 80 },
    { 0x65995bd0, 98 },
    { 0, 0 },
    { 0xc6a95bd4, 33 },
    { 0xb2795bd4, 87 },
    { 0, 0 },
    { 0x4ed95bd7, 76 },
    { 0x1ae95bd8, 85 },
    { 0x5c195bd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f195be1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe395be4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e95be7, 90 },
    { 0, 0 },
    { 0xbc395be9, 67 },
    { 0x4195bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa95bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca95bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b695bfe, 61 },
    { 0x68295bfe, 90 },
    { 0, 0 },
    { 0x79895c01, 95 },
    { 0x16295c01, 39 },
    { 0xa1195c03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc095c08, 23 },
    { 0, 0 },
    { 0x91b95c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b095c0d, 67 },
    { 0xd6695c0e, 48 },
    { 0x9e095c0e, 100 },
    { 0xefd95c0f, 77 },
    { 0x41e95c11, 90 },
    { 0x64d95c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8195c16, 5 },
    { 0x1b795c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a95c1c, 77 },
    { 0, 0 },
    { 0xb3995c1e, 55 },
    { 0xab795c1e, 77 },
    { 0xc1495c20, 67 },
    { 0x5ff95c21, 90 },
    { 0xcd095c22, 67 },
    { 0, 0 },
    { 0x18795c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce95c2a, 77 },
    { 0x2a295c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7495c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53095c3b, 61 },
    { 0x4da95c3b, 95 },
    { 0xf4a95c3c, 90 },
    { 0x89295c3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27995c41, 23 },
    { 0x94b95c41, 98 },
    { 0xecb95c43, 90 },
    { 0x4a295c42, 23 },
    { 0xbe595c45, 61 },
    { 0xd5b95c46, 67 },
    { 0xdf95c45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10695c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0795c52, 33 },
    { 0xb2895c53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a995c56, 99 },
    { 0, 0 },
    { 0xebb95c58, 95 },
    { 0x39f95c59, 90 },
    { 0x78095c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6195c5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb95c65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8395c6e, 33 },
    { 0xedb95c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc95c74, 90 },
    { 0, 0 },
    { 0x79e95c76, 17 },
    { 0xc8095c76, 66 },
    { 0x29995c77, 23 },
    { 0, 0 },
    { 0x92095c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9195c7d, 90 },
    { 0xc795c7e, 33 },
    { 0, 0 },
    { 0xdb695c80, 34 },
    { 0, 0 },
    { 0xf7395c82, 78 },
    { 0x8cf95c82, 90 },
    { 0xdbb95c84, 34 },
    { 0xeae95c84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0095c88, 95 },
    { 0x3195c89, 98 },
    { 0xd6d95c8a, 94 },
    { 0x61895c8a, 33 },
    { 0x5a995c89, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68795c91, 80 },
    { 0x54295c91, 98 },
    { 0x64195c93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f295c96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x80795c99, 33 },
    { 0x6ba95c99, 76 },
    { 0x32b95c9b, 98 },
    { 0xf6995c9c, 98 },
    { 0x9a695c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6995ca0, 77 },
    { 0xc2b95ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4795ca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c95ca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc595cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b395cae, 76 },
    { 0, 0 },
    { 0x4a895cb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69795cb7, 23 },
    { 0x1c695cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12195cbc, 91 },
    { 0x41c95cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e95cc0, 98 },
    { 0x78495cc0, 90 },
    { 0xc5095cc1, 33 },
    { 0x85295cc3, 90 },
    { 0xc6495cc0, 85 },
    { 0x7be95cc4, 90 },
    { 0xf2895cc6, 43 },
    { 0xf9595cc6, 90 },
    { 0x1c595cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed95ccd, 76 },
    { 0, 0 },
    { 0xd95ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d95cd5, 33 },
    { 0x6895cd5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e95cdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5795cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c95ce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6895cf1, 85 },
    { 0, 0 },
    { 0x12a95cf3, 33 },
    { 0, 0 },
    { 0xddb95cf5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x20195cf8, 61 },
    { 0x9e595cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a895cfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23695d0a, 98 },
    { 0, 0 },
    { 0x8e795d0c, 46 },
    { 0xe9d95d0d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4995d13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34995d1d, 76 },
    { 0xf8895d1e, 90 },
    { 0x5d295d1f, 66 },
    { 0xdd995d20, 53 },
    { 0x75d95d20, 66 },
    { 0x78095d22, 81 },
    { 0, 0 },
    { 0x49f95d24, 17 },
    { 0xf0e95d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c95d29, 43 },
    { 0x76595d2a, 64 },
    { 0xeee95d2b, 46 },
    { 0xcb395d2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd95d30, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3295d36, 68 },
    { 0x99495d37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95d3a, 55 },
    { 0, 0 },
    { 0xc9595d3c, 90 },
    { 0x38c95d3c, 23 },
    { 0xa1795d3e, 23 },
    { 0xe8e95d3e, 33 },
    { 0, 0 },
    { 0x96695d41, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f95d44, 39 },
    { 0xca995d45, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a95d4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94195d54, 23 },
    { 0, 0 },
    { 0x13c95d56, 46 },
    { 0x14c95d57, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe95d5d, 61 },
    { 0, 0 },
    { 0x74695d5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b95d67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0895d77, 17 },
    { 0x5a195d78, 84 },
    { 0xf3495d78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe95d7d, 68 },
    { 0x4c395d7e, 23 },
    { 0x89895d7e, 81 },
    { 0xa6595d80, 46 },
    { 0x36695d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf895d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f95d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5695d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68595d95, 76 },
    { 0, 0 },
    { 0x91695d97, 78 },
    { 0xf1795d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84895d9d, 76 },
    { 0, 0 },
    { 0xcec95d9f, 61 },
    { 0x6ea95da0, 55 },
    { 0, 0 },
    { 0xe995da2, 46 },
    { 0xc9795da3, 67 },
    { 0xe3695da4, 76 },
    { 0x1f495da5, 90 },
    { 0xebc95da4, 66 },
    { 0x1c95da5, 95 },
    { 0x4c695da8, 46 },
    { 0, 0 },
    { 0xbf495daa, 61 },
    { 0xcf795dab, 90 },
    { 0x67c95dac, 52 },
    { 0, 0 },
    { 0x2ac95dae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a695dba, 55 },
    { 0xdc995dbb, 76 },
    { 0x2c95dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70095dc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6395dc4, 34 },
    { 0, 0 },
    { 0x5a195dc6, 33 },
    { 0x6495dc7, 61 },
    { 0xe6695dc7, 76 },
    { 0x15995dc6, 52 },
    { 0x10495dc6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6095dcf, 61 },
    { 0x98595dcf, 75 },
    { 0x695dd1, 98 },
    { 0xa6e95dd2, 90 },
    { 0xd4395dcf, 67 },
    { 0, 0 },
    { 0xf7795dd5, 85 },
    { 0xe4395dd6, 46 },
    { 0x60295dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6995ddc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9795de0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5395de7, 98 },
    { 0x4c95de7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc795ded, 52 },
    { 0x8e095dee, 23 },
    { 0xbfb95ded, 75 },
    { 0xfca95ded, 98 },
    { 0, 0 },
    { 0xada95df2, 61 },
    { 0x97195df2, 90 },
    { 0xc5d95df3, 38 },
    { 0, 0 },
    { 0xfeb95df6, 67 },
    { 0, 0 },
    { 0xabe95df8, 95 },
    { 0, 0 },
    { 0x1af95dfa, 61 },
    { 0x77f95dfb, 91 },
    { 0x83195dfb, 95 },
    { 0xec295dfa, 55 },
    { 0xae495dfd, 90 },
    { 0xff795dff, 61 },
    { 0xd6a95e00, 33 },
    { 0, 0 },
    { 0x495e02, 96 },
    { 0x30f95e03, 95 },
    { 0xdbe95e03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e695e09, 55 },
    { 0x6e795e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45495e0d, 84 },
    { 0x14d95e0e, 90 },
    { 0, 0 },
    { 0xc695e10, 61 },
    { 0x21b95e10, 93 },
    { 0, 0 },
    { 0x55695e13, 90 },
    { 0x62495e13, 95 },
    { 0xa8a95e13, 55 },
    { 0xdb895e16, 34 },
    { 0x4a195e17, 90 },
    { 0x29395e18, 90 },
    { 0x88695e18, 90 },
    { 0xd5795e19, 95 },
    { 0x9e195e1b, 61 },
    { 0x5ed95e1b, 95 },
    { 0xd1295e1d, 23 },
    { 0xed995e1e, 38 },
    { 0x12995e1d, 94 },
    { 0x5cd95e20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95e24, 55 },
    { 0xe1495e24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a195e2b, 23 },
    { 0, 0 },
    { 0xb9f95e2d, 23 },
    { 0xa3e95e2e, 76 },
    { 0x6195e2f, 100 },
    { 0x4e095e30, 23 },
    { 0xc5495e30, 95 },
    { 0x95e32, 98 },
    { 0xbb495e33, 84 },
    { 0x8ea95e34, 33 },
    { 0x8ac95e2d, 80 },
    { 0x46495e2d, 95 },
    { 0x90095e37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3295e3a, 46 },
    { 0x61e95e3b, 66 },
    { 0, 0 },
    { 0x33a95e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b95e45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28795e48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12195e4e, 68 },
    { 0, 0 },
    { 0x12995e50, 23 },
    { 0x5a495e51, 35 },
    { 0x17d95e52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55195e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc95e5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb95e60, 34 },
    { 0x70995e61, 77 },
    { 0, 0 },
    { 0x44b95e63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d95e6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1295e6e, 23 },
    { 0x9d395e6f, 33 },
    { 0xb3295e70, 43 },
    { 0xa6395e70, 84 },
    { 0xf5e95e70, 34 },
    { 0xa3895e72, 95 },
    { 0xe1195e73, 62 },
    { 0x3da95e6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37495e7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98995e80, 61 },
    { 0, 0 },
    { 0x38c95e82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8995e85, 90 },
    { 0x65a95e85, 33 },
    { 0, 0 },
    { 0x8e995e88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec95e8b, 33 },
    { 0, 0 },
    { 0x9aa95e8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a595e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a195e96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d95e99, 99 },
    { 0x38b95e99, 55 },
    { 0xcdc95e9a, 66 },
    { 0xd4195e9a, 81 },
    { 0x1095e9d, 87 },
    { 0x58795e9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a95ea1, 61 },
    { 0x94995ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15595ea5, 65 },
    { 0x24e95ea6, 78 },
    { 0, 0 },
    { 0x5a395ea8, 53 },
    { 0, 0 },
    { 0x4895eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd95eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92295eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9595ebb, 33 },
    { 0x99295ebb, 90 },
    { 0xf1b95ebb, 91 },
    { 0x23595ebe, 87 },
    { 0x9c895ebf, 52 },
    { 0xa5e95ec0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c95ec4, 61 },
    { 0x3c395ec5, 33 },
    { 0xc6295ec5, 84 },
    { 0x11595ec7, 76 },
    { 0xc2895ec8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa95ecc, 98 },
    { 0xcce95ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e795ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea95ed3, 87 },
    { 0xa7495ed4, 55 },
    { 0x98395ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae95eda, 97 },
    { 0x4295eda, 33 },
    { 0x89995edc, 67 },
    { 0xe4595edd, 78 },
    { 0x6d595edb, 98 },
    { 0xe0695edf, 90 },
    { 0, 0 },
    { 0xed695ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1895ee7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b495eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f95eef, 93 },
    { 0xea495eef, 90 },
    { 0x19095ef1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d95ef6, 95 },
    { 0xdb695ef6, 98 },
    { 0x9d895ef6, 61 },
    { 0x8fb95ef9, 95 },
    { 0x95a95efa, 98 },
    { 0x5fe95efb, 84 },
    { 0xf9f95efb, 87 },
    { 0xd8995efb, 61 },
    { 0xe2695ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce95f05, 61 },
    { 0x53495f06, 90 },
    { 0x8c495f06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57995f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea95f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed95f14, 91 },
    { 0xa3c95f14, 61 },
    { 0xd6c95f14, 23 },
    { 0x6d795f17, 81 },
    { 0xc4395f14, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68895f1c, 76 },
    { 0x77695f1d, 61 },
    { 0x49195f1d, 61 },
    { 0x3cb95f1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe095f23, 68 },
    { 0x7195f24, 34 },
    { 0xf0895f25, 76 },
    { 0xf6195f26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d95f2f, 98 },
    { 0xb9595f2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25695f34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c95f38, 95 },
    { 0x6f495f39, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b795f3d, 55 },
    { 0x2795f3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22595f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12795f45, 33 },
    { 0x73395f46, 78 },
    { 0x17295f46, 76 },
    { 0x6da95f48, 76 },
    { 0xd9e95f49, 46 },
    { 0x97095f4a, 33 },
    { 0, 0 },
    { 0xb5c95f4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e395f4f, 61 },
    { 0xe4795f50, 84 },
    { 0x5ca95f4f, 33 },
    { 0xca295f50, 90 },
    { 0, 0 },
    { 0xec595f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee195f5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc95f61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c695f65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f95f68, 90 },
    { 0, 0 },
    { 0x89295f6a, 66 },
    { 0xab795f6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2595f71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc895f74, 90 },
    { 0xfb195f75, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec295f7c, 33 },
    { 0, 0 },
    { 0xbce95f7e, 95 },
    { 0, 0 },
    { 0x34e95f80, 93 },
    { 0x3595f81, 61 },
    { 0x33f95f81, 95 },
    { 0xb2c95f83, 95 },
    { 0x14c95f84, 96 },
    { 0, 0 },
    { 0x89e95f86, 90 },
    { 0xfed95f87, 66 },
    { 0, 0 },
    { 0x79995f89, 61 },
    { 0x34595f89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf95f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e895f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40995f9a, 36 },
    { 0x8e895f9b, 98 },
    { 0x74295f9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3595fa1, 95 },
    { 0x7095fa2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e995fa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3895fad, 23 },
    { 0xa0095fae, 88 },
    { 0xa2695faf, 34 },
    { 0, 0 },
    { 0x81895fb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc95fb4, 84 },
    { 0xcf895fb5, 61 },
    { 0xa8195fb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69495fb9, 66 },
    { 0, 0 },
    { 0x4a95fbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e95fbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c95fc2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68595fc9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a395fcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195fd0, 55 },
    { 0, 0 },
    { 0xf0395fd2, 43 },
    { 0x54795fd3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4795fdf, 61 },
    { 0x6a295fe0, 43 },
    { 0, 0 },
    { 0x4cb95fe2, 85 },
    { 0x96595fe3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3295fea, 93 },
    { 0x32895feb, 19 },
    { 0, 0 },
    { 0xbe495fed, 46 },
    { 0x7b895fed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e95ff1, 85 },
    { 0x71295ff1, 95 },
    { 0x95995ff1, 95 },
    { 0x85195ff2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e95ff8, 55 },
    { 0x17595ff8, 78 },
    { 0x37b95ffa, 61 },
    { 0, 0 },
    { 0xb895ffc, 85 },
    { 0xd3395ffd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696001, 67 },
    { 0, 0 },
    { 0x21896003, 55 },
    { 0xc6496003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf296009, 66 },
    { 0xaf49600a, 90 },
    { 0x2e19600a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4496010, 61 },
    { 0x4b96010, 100 },
    { 0x36196012, 76 },
    { 0x33196013, 61 },
    { 0x5af96014, 61 },
    { 0x72b96010, 34 },
    { 0x6a96015, 47 },
    { 0x89496017, 61 },
    { 0, 0 },
    { 0xcb96019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829601d, 61 },
    { 0xa049601e, 61 },
    { 0x2209601f, 76 },
    { 0x71996020, 43 },
    { 0, 0 },
    { 0x19b96022, 98 },
    { 0xdf596022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a99602d, 93 },
    { 0x4269602e, 43 },
    { 0x9789602f, 33 },
    { 0xcbe9602d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c96038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c96047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd9604b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe279604f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b96052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e96055, 90 },
    { 0xa8c96056, 90 },
    { 0x12096057, 75 },
    { 0x9a996058, 95 },
    { 0x71d96057, 34 },
    { 0xba09605a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd829605d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3796064, 33 },
    { 0x28196065, 95 },
    { 0xba396065, 52 },
    { 0xd8e96067, 23 },
    { 0xf0296067, 46 },
    { 0xc3396067, 90 },
    { 0x35a9606a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b9606e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce96072, 43 },
    { 0, 0 },
    { 0x8eb96074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67196079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f396081, 40 },
    { 0, 0 },
    { 0xee296083, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d496088, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b796090, 61 },
    { 0x9bf96090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34696094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e596098, 99 },
    { 0x96696099, 98 },
    { 0x8859609a, 43 },
    { 0, 0 },
    { 0xaec9609c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178960a1, 98 },
    { 0x38c960a2, 87 },
    { 0x2b6960a3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e960a6, 65 },
    { 0xc3e960a7, 66 },
    { 0, 0 },
    { 0x27c960a9, 77 },
    { 0xacb960aa, 55 },
    { 0x767960ab, 34 },
    { 0, 0 },
    { 0xa03960ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7960b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f960b7, 91 },
    { 0x12f960b8, 88 },
    { 0xfbd960b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306960be, 95 },
    { 0x926960bf, 77 },
    { 0x66d960bf, 90 },
    { 0x4e4960be, 98 },
    { 0, 0 },
    { 0x954960c3, 61 },
    { 0x4de960c4, 82 },
    { 0x968960c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97960c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40960ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636960dc, 52 },
    { 0x768960dd, 65 },
    { 0xf3b960dc, 23 },
    { 0xe3960dd, 33 },
    { 0xae8960df, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975960e4, 33 },
    { 0, 0 },
    { 0x777960e6, 95 },
    { 0xac2960e7, 76 },
    { 0xb9c960e7, 76 },
    { 0x6f5960e9, 61 },
    { 0xe99960e8, 100 },
    { 0x1fe960eb, 52 },
    { 0x507960ec, 90 },
    { 0x7c4960ed, 43 },
    { 0x8b7960ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170960f2, 80 },
    { 0x6e4960f3, 83 },
    { 0x3a960f2, 23 },
    { 0x93960f5, 81 },
    { 0xd38960f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74960f9, 90 },
    { 0x3d9960f9, 90 },
    { 0xfe2960fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68960fe, 76 },
    { 0x986960fe, 84 },
    { 0x49596100, 34 },
    { 0x8ed96100, 33 },
    { 0xd5296100, 23 },
    { 0xf2296101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe296107, 84 },
    { 0x51996108, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49610d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df96110, 46 },
    { 0x45296111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c96114, 76 },
    { 0xe5596115, 46 },
    { 0x7b696115, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa859611a, 43 },
    { 0x9389611b, 33 },
    { 0x83a9611b, 46 },
    { 0xba79611c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b96120, 82 },
    { 0, 0 },
    { 0x15496122, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee96126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d96129, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe289612c, 33 },
    { 0xb7f9612d, 95 },
    { 0x2a69612e, 55 },
    { 0, 0 },
    { 0x62a96130, 23 },
    { 0x5e496130, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5996134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c296139, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2919613d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3696141, 78 },
    { 0x5f396142, 95 },
    { 0xb6e96143, 23 },
    { 0x41896143, 85 },
    { 0, 0 },
    { 0xfe596146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f69614a, 33 },
    { 0, 0 },
    { 0x97e9614c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb96153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439615a, 33 },
    { 0x4679615a, 92 },
    { 0x6389615b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32596167, 34 },
    { 0xc5196168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469616f, 61 },
    { 0xcb896170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac96173, 46 },
    { 0xcda96173, 91 },
    { 0xcfc96175, 33 },
    { 0, 0 },
    { 0x5cd96177, 75 },
    { 0, 0 },
    { 0x93b96179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a9617c, 78 },
    { 0x1309617c, 91 },
    { 0xce89617e, 95 },
    { 0x6649617f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x95996182, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7496185, 90 },
    { 0, 0 },
    { 0x4a696187, 76 },
    { 0xb1596188, 34 },
    { 0xfb896188, 61 },
    { 0xeac9618a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef996190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a96193, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5496197, 95 },
    { 0x93096198, 98 },
    { 0x72c96199, 95 },
    { 0, 0 },
    { 0x30c9619b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df9619e, 52 },
    { 0xcdf9619e, 36 },
    { 0x7ab9619e, 66 },
    { 0x998961a1, 99 },
    { 0x10961a2, 61 },
    { 0x78961a2, 98 },
    { 0x73d961a3, 23 },
    { 0xcb7961a0, 33 },
    { 0xa3b961a2, 61 },
    { 0xe72961a0, 98 },
    { 0, 0 },
    { 0x349961a9, 99 },
    { 0, 0 },
    { 0x4ad961ab, 100 },
    { 0x2cd961ab, 76 },
    { 0x4a4961ad, 97 },
    { 0x567961ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff961b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112961b6, 90 },
    { 0, 0 },
    { 0x775961b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6961bc, 66 },
    { 0, 0 },
    { 0xf8b961be, 64 },
    { 0x797961be, 61 },
    { 0, 0 },
    { 0xdc7961c1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d961c6, 90 },
    { 0, 0 },
    { 0x4b4961c8, 98 },
    { 0xdfc961c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984961cd, 55 },
    { 0x61f961ce, 55 },
    { 0xd16961cf, 87 },
    { 0xd48961cf, 33 },
    { 0xe89961d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5961da, 95 },
    { 0xcd6961db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f961df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2961e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923961ea, 95 },
    { 0xddc961eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836961ef, 61 },
    { 0, 0 },
    { 0xe52961f1, 33 },
    { 0x88e961f2, 69 },
    { 0, 0 },
    { 0xc9c961f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d961fa, 76 },
    { 0x926961fb, 34 },
    { 0x865961fc, 93 },
    { 0x89f961fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65596201, 23 },
    { 0xeb496202, 55 },
    { 0xe3d96203, 95 },
    { 0xae496204, 87 },
    { 0x73f96205, 87 },
    { 0xeaf96206, 98 },
    { 0xc3396207, 63 },
    { 0x65196208, 46 },
    { 0x7d896208, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed796210, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796217, 82 },
    { 0xd0196218, 90 },
    { 0x15996218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d89621c, 61 },
    { 0, 0 },
    { 0xe769621e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea596221, 61 },
    { 0xe4896221, 23 },
    { 0, 0 },
    { 0xf5a96224, 76 },
    { 0xb7a96225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae696228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596235, 23 },
    { 0x82a96235, 61 },
    { 0x7ce96237, 90 },
    { 0x49396237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f49623b, 76 },
    { 0xa399623c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1029623f, 23 },
    { 0x45696240, 90 },
    { 0, 0 },
    { 0x46d96242, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d96246, 98 },
    { 0, 0 },
    { 0x2e796248, 98 },
    { 0, 0 },
    { 0x64a9624a, 91 },
    { 0, 0 },
    { 0x1b49624c, 98 },
    { 0x1359624c, 61 },
    { 0, 0 },
    { 0x6f59624f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef96253, 98 },
    { 0x45896254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f196257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff9625c, 99 },
    { 0x2839625c, 90 },
    { 0x4759625c, 90 },
    { 0, 0 },
    { 0x8fa96260, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4996265, 78 },
    { 0x66b96265, 46 },
    { 0xf3996265, 78 },
    { 0, 0 },
    { 0xf8b96269, 90 },
    { 0xf2996269, 78 },
    { 0x4f196269, 90 },
    { 0x13796269, 61 },
    { 0x6279626b, 23 },
    { 0, 0 },
    { 0xd279626f, 68 },
    { 0x1379626f, 40 },
    { 0, 0 },
    { 0x6f596272, 55 },
    { 0, 0 },
    { 0x11896274, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af9627c, 95 },
    { 0xdd69627d, 33 },
    { 0x98a9627e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3396282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed96285, 28 },
    { 0x1dc96286, 91 },
    { 0, 0 },
    { 0x22396288, 88 },
    { 0x61d96289, 90 },
    { 0xe0696289, 95 },
    { 0x7df9628b, 90 },
    { 0x4a496288, 100 },
    { 0x9e79628d, 98 },
    { 0, 0 },
    { 0xf999628f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55796292, 90 },
    { 0x8b096292, 76 },
    { 0xd2796294, 76 },
    { 0xae696295, 90 },
    { 0x52296295, 90 },
    { 0x2cb96296, 33 },
    { 0x66796294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc689629b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea09629f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x136962a2, 49 },
    { 0xf85962a3, 33 },
    { 0, 0 },
    { 0x5c0962a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5962b1, 46 },
    { 0, 0 },
    { 0xb4b962b3, 61 },
    { 0x9ec962b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae962b7, 78 },
    { 0, 0 },
    { 0xcd3962b9, 63 },
    { 0, 0 },
    { 0x69b962bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c962c1, 90 },
    { 0xde962c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589962cb, 66 },
    { 0, 0 },
    { 0x8f6962cd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4962d1, 61 },
    { 0x49e962d2, 76 },
    { 0x80962d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972962d8, 90 },
    { 0, 0 },
    { 0xe1c962da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07962dd, 88 },
    { 0x236962de, 17 },
    { 0xdb7962df, 90 },
    { 0x83b962df, 33 },
    { 0x83962e1, 55 },
    { 0xc22962e2, 98 },
    { 0xf19962e2, 56 },
    { 0xf0a962e4, 43 },
    { 0xc38962e2, 93 },
    { 0, 0 },
    { 0xfaa962e7, 90 },
    { 0xfb6962e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9962eb, 90 },
    { 0xa35962ec, 61 },
    { 0x4c9962ec, 99 },
    { 0, 0 },
    { 0xa1b962ef, 55 },
    { 0xc7d962f0, 99 },
    { 0, 0 },
    { 0x314962f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48962fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07962ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4796302, 11 },
    { 0xcd196303, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7896307, 46 },
    { 0xdd796308, 90 },
    { 0xfab96307, 33 },
    { 0xe0396307, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb9630e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa96311, 33 },
    { 0xd3696312, 98 },
    { 0xc2996313, 61 },
    { 0, 0 },
    { 0xefd96315, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac9631a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3349631f, 67 },
    { 0x49896320, 33 },
    { 0x4579631f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a896324, 52 },
    { 0x2a996325, 23 },
    { 0xc6b96326, 33 },
    { 0, 0 },
    { 0xc5396328, 46 },
    { 0, 0 },
    { 0x3059632a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9632f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae96332, 90 },
    { 0x1c796332, 81 },
    { 0x8fa96332, 55 },
    { 0xf1396335, 3 },
    { 0xdb96334, 90 },
    { 0xea096337, 85 },
    { 0x52e96337, 46 },
    { 0, 0 },
    { 0xcb99633a, 52 },
    { 0x9ba9633a, 85 },
    { 0x1119633c, 43 },
    { 0, 0 },
    { 0x35a9633e, 99 },
    { 0xad79633e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c96345, 81 },
    { 0, 0 },
    { 0xe7496347, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9634d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296352, 23 },
    { 0x82f96353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68896358, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c9635e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b796362, 55 },
    { 0xb9e96363, 46 },
    { 0xebb96364, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e96368, 90 },
    { 0, 0 },
    { 0xb9e9636a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30396373, 34 },
    { 0, 0 },
    { 0x1f796375, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef29637e, 78 },
    { 0x8a89637f, 43 },
    { 0x43e96380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf496385, 33 },
    { 0, 0 },
    { 0x8896387, 61 },
    { 0x51b96388, 81 },
    { 0x7fe96387, 23 },
    { 0x7d396388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98796391, 90 },
    { 0xe3696392, 34 },
    { 0, 0 },
    { 0x83196394, 90 },
    { 0x54d96395, 33 },
    { 0xeb96396, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x81396399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709639e, 34 },
    { 0xb639639f, 90 },
    { 0xf79639e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a963a3, 33 },
    { 0, 0 },
    { 0xe41963a5, 23 },
    { 0xa47963a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457963ad, 76 },
    { 0x53f963ae, 100 },
    { 0x656963ae, 91 },
    { 0, 0 },
    { 0x40c963b1, 61 },
    { 0x62f963b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33963ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da963be, 90 },
    { 0, 0 },
    { 0x4b8963c0, 46 },
    { 0, 0 },
    { 0x357963c2, 46 },
    { 0x49d963c2, 61 },
    { 0xe6b963c4, 61 },
    { 0, 0 },
    { 0x2e0963c6, 43 },
    { 0x569963c6, 95 },
    { 0x417963c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5963cc, 55 },
    { 0, 0 },
    { 0xdaf963ce, 76 },
    { 0, 0 },
    { 0xb67963d0, 90 },
    { 0x234963d0, 91 },
    { 0x1c3963d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d963d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994963d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369963dd, 61 },
    { 0, 0 },
    { 0x2a1963df, 38 },
    { 0, 0 },
    { 0x41f963e1, 76 },
    { 0x3b3963e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896963e6, 75 },
    { 0xd09963e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141963eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f963ee, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3963f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x321963fc, 34 },
    { 0xedc963fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12896400, 67 },
    { 0x7b096401, 90 },
    { 0, 0 },
    { 0xc4c96403, 43 },
    { 0x6f896404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44196409, 95 },
    { 0x7439640a, 67 },
    { 0, 0 },
    { 0x7fe9640c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e96411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6596414, 65 },
    { 0x4f96415, 98 },
    { 0x34e96416, 68 },
    { 0x51d96417, 68 },
    { 0xdf696416, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f9641c, 34 },
    { 0xfeb9641c, 23 },
    { 0xd9b9641c, 100 },
    { 0, 0 },
    { 0x75c96420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee396425, 61 },
    { 0, 0 },
    { 0x90796427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8789642d, 33 },
    { 0xb229642e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa96434, 87 },
    { 0x27396435, 90 },
    { 0x32296435, 91 },
    { 0xa3896437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb319643b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9643e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b96441, 76 },
    { 0x12c96442, 95 },
    { 0xc2396443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99496449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b96450, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc96453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269645a, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09645f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55896462, 34 },
    { 0xf4696462, 87 },
    { 0xd6b96462, 46 },
    { 0xb6496462, 84 },
    { 0x1a596464, 61 },
    { 0, 0 },
    { 0xbf496468, 34 },
    { 0xadf96469, 33 },
    { 0x3089646a, 90 },
    { 0xbee9646a, 90 },
    { 0x3669646b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b496470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9996476, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d896479, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6359647d, 43 },
    { 0x64f9647e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49696481, 33 },
    { 0x9c696481, 33 },
    { 0xb9196482, 90 },
    { 0x8af96481, 76 },
    { 0, 0 },
    { 0x46396486, 33 },
    { 0x63096486, 90 },
    { 0, 0 },
    { 0x74496489, 84 },
    { 0, 0 },
    { 0xe9f9648b, 46 },
    { 0x7ac9648c, 98 },
    { 0, 0 },
    { 0x4039648e, 91 },
    { 0, 0 },
    { 0x94996490, 80 },
    { 0xce696491, 55 },
    { 0, 0 },
    { 0xcc696493, 40 },
    { 0x50696494, 90 },
    { 0xdc596494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7796498, 23 },
    { 0x5f196499, 61 },
    { 0xd1d9649a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5259649e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9964a2, 33 },
    { 0x586964a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f964a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a964ab, 23 },
    { 0xe96964ac, 90 },
    { 0, 0 },
    { 0x35f964ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a964b3, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee964b6, 95 },
    { 0x86964b7, 98 },
    { 0x91c964b8, 34 },
    { 0, 0 },
    { 0x90f964ba, 25 },
    { 0, 0 },
    { 0x5964bc, 78 },
    { 0x333964bd, 76 },
    { 0x995964be, 23 },
    { 0x11964bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6964c7, 90 },
    { 0xd8d964c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8964cb, 33 },
    { 0x69c964cc, 90 },
    { 0x558964cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06964d0, 95 },
    { 0, 0 },
    { 0x3df964d2, 90 },
    { 0xfc5964d3, 87 },
    { 0x836964d4, 90 },
    { 0xc69964d5, 98 },
    { 0x506964d5, 34 },
    { 0, 0 },
    { 0xa09964d8, 40 },
    { 0x48c964d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x634964dc, 46 },
    { 0xa5e964dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d964e3, 17 },
    { 0, 0 },
    { 0x324964e5, 55 },
    { 0, 0 },
    { 0x1bf964e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7964eb, 95 },
    { 0, 0 },
    { 0x142964ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9964f0, 55 },
    { 0x7a4964f0, 100 },
    { 0, 0 },
    { 0xe1d964f3, 33 },
    { 0x92964f4, 83 },
    { 0x28964f5, 61 },
    { 0xe9a964f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18964f9, 61 },
    { 0, 0 },
    { 0x8ff964fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f964ff, 34 },
    { 0xc1896500, 61 },
    { 0xe2e96500, 43 },
    { 0x63496502, 40 },
    { 0xc1b96503, 43 },
    { 0x46a96504, 98 },
    { 0x1f296504, 76 },
    { 0xbec964ff, 90 },
    { 0xef096507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7759650a, 61 },
    { 0x11f9650a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93996511, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3696517, 33 },
    { 0, 0 },
    { 0xde696519, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9651d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc996520, 90 },
    { 0x3b196521, 46 },
    { 0, 0 },
    { 0x31496523, 98 },
    { 0xf1796524, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d96527, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689652a, 33 },
    { 0x2219652b, 43 },
    { 0x41c9652b, 90 },
    { 0x2329652c, 17 },
    { 0x9d39652e, 95 },
    { 0xf299652c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc696533, 85 },
    { 0xb4996534, 93 },
    { 0x36696535, 23 },
    { 0x88996536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4459653e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce696544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99654a, 99 },
    { 0, 0 },
    { 0xdd79654c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60496550, 95 },
    { 0, 0 },
    { 0x33396552, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x70996555, 98 },
    { 0x4396556, 61 },
    { 0, 0 },
    { 0xea996558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa529655d, 61 },
    { 0xd119655e, 43 },
    { 0xeff9655e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53096563, 99 },
    { 0xd4996563, 52 },
    { 0, 0 },
    { 0xbd196566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9656d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea96575, 87 },
    { 0, 0 },
    { 0x29596577, 90 },
    { 0, 0 },
    { 0xe7796579, 93 },
    { 0, 0 },
    { 0xe439657b, 90 },
    { 0x6f49657c, 90 },
    { 0xff39657d, 90 },
    { 0, 0 },
    { 0x2439657f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52096582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33896589, 24 },
    { 0xd5496589, 33 },
    { 0xc319658b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0296592, 95 },
    { 0, 0 },
    { 0x9b996594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6379659b, 90 },
    { 0, 0 },
    { 0x3a69659d, 55 },
    { 0xeeb9659e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4965a4, 33 },
    { 0x28e965a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b965a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816965af, 93 },
    { 0x629965b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43965b4, 61 },
    { 0xd5d965b4, 85 },
    { 0xd9a965b5, 95 },
    { 0x64965b7, 23 },
    { 0, 0 },
    { 0x124965b9, 85 },
    { 0x486965ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a965bf, 87 },
    { 0x4ac965bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5965c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754965ca, 88 },
    { 0x8e1965ca, 23 },
    { 0xf67965cb, 66 },
    { 0xa00965cd, 90 },
    { 0x665965ce, 90 },
    { 0x306965cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39965d5, 90 },
    { 0, 0 },
    { 0x612965d7, 90 },
    { 0, 0 },
    { 0x9c965d9, 43 },
    { 0x530965da, 33 },
    { 0xdce965db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4965df, 90 },
    { 0x139965df, 78 },
    { 0, 0 },
    { 0x4c7965e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca965e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d965e9, 34 },
    { 0x30c965e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf965ed, 90 },
    { 0xad5965ee, 81 },
    { 0, 0 },
    { 0xb7f965f0, 95 },
    { 0xcb9965f0, 77 },
    { 0x46965f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef965f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76965fb, 78 },
    { 0x2b9965fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68796604, 95 },
    { 0, 0 },
    { 0x81396606, 33 },
    { 0xa1f96607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99660d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc96614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b896617, 52 },
    { 0x8c496618, 95 },
    { 0x83896619, 85 },
    { 0x26b9661a, 84 },
    { 0xb8b96619, 61 },
    { 0, 0 },
    { 0x6139661d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b96622, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5996627, 90 },
    { 0x6e496627, 33 },
    { 0x8496629, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e396632, 95 },
    { 0x82196633, 33 },
    { 0xcf596634, 55 },
    { 0xefb96633, 61 },
    { 0x20896634, 55 },
    { 0xb8f96637, 23 },
    { 0x15b96638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe919663d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496642, 90 },
    { 0, 0 },
    { 0x56396644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa809664a, 90 },
    { 0x26b9664b, 23 },
    { 0xb2a9664a, 98 },
    { 0x9ad9664d, 33 },
    { 0, 0 },
    { 0x6519664f, 76 },
    { 0x51996650, 87 },
    { 0, 0 },
    { 0x4d196652, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5396656, 68 },
    { 0x41e96657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e896663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2196669, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84496671, 55 },
    { 0x76b96671, 80 },
    { 0xd7996673, 87 },
    { 0, 0 },
    { 0x18f96675, 43 },
    { 0x2dd96676, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd269667a, 33 },
    { 0xb29667b, 95 },
    { 0x6ae9667c, 95 },
    { 0x18d9667c, 84 },
    { 0x2649667b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a96684, 61 },
    { 0x8db96685, 98 },
    { 0xf1d96686, 95 },
    { 0x28996687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9668a, 23 },
    { 0xbe89668b, 76 },
    { 0xd989668b, 76 },
    { 0x7579668d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38396692, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d96695, 43 },
    { 0x8b896696, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f9669b, 76 },
    { 0, 0 },
    { 0x3c49669d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04966a3, 23 },
    { 0x431966a3, 100 },
    { 0x646966a3, 23 },
    { 0x80d966a6, 85 },
    { 0x740966a7, 23 },
    { 0x2e5966a8, 76 },
    { 0xc52966a8, 17 },
    { 0x35d966a8, 95 },
    { 0xee1966ab, 34 },
    { 0x3e8966a6, 43 },
    { 0x6b4966ab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2966b1, 90 },
    { 0xa4a966b1, 95 },
    { 0x39e966b3, 33 },
    { 0x4ed966b3, 98 },
    { 0, 0 },
    { 0x980966b6, 96 },
    { 0xe5e966b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33966ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66966be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5966c2, 55 },
    { 0xe51966c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866966c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac966ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a966d3, 88 },
    { 0x96b966d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28966da, 91 },
    { 0x404966db, 98 },
    { 0xdcb966da, 34 },
    { 0x18b966dd, 95 },
    { 0xe90966db, 90 },
    { 0x8d7966db, 98 },
    { 0xaf4966dc, 33 },
    { 0xb27966e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88966e6, 85 },
    { 0x752966e7, 95 },
    { 0x60966e8, 98 },
    { 0x468966e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e966ed, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec966f2, 90 },
    { 0xf31966f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711966f9, 90 },
    { 0xb24966f9, 91 },
    { 0, 0 },
    { 0x31966fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30996701, 33 },
    { 0x24896701, 33 },
    { 0, 0 },
    { 0xc2a96704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b796707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd409670e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd996713, 95 },
    { 0x42c96714, 40 },
    { 0, 0 },
    { 0xb2596716, 99 },
    { 0xaa796716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b96720, 93 },
    { 0x64496721, 90 },
    { 0xf8e96722, 23 },
    { 0, 0 },
    { 0x9b796724, 95 },
    { 0, 0 },
    { 0x80896726, 33 },
    { 0x59996727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9672a, 33 },
    { 0xbe79672b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729672e, 90 },
    { 0x69b9672f, 91 },
    { 0x2e89672f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17896734, 38 },
    { 0, 0 },
    { 0xa6996736, 85 },
    { 0xdad96736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9673b, 90 },
    { 0xb829673c, 34 },
    { 0x8a39673d, 99 },
    { 0, 0 },
    { 0xbb19673f, 90 },
    { 0x36696740, 90 },
    { 0xdb096741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c496744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309674b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7996750, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8896754, 90 },
    { 0x26396755, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e96758, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5819675f, 55 },
    { 0xe7e96760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a96766, 43 },
    { 0xb7496766, 33 },
    { 0x65a96768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439676c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf619676f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7796772, 93 },
    { 0x89d96773, 66 },
    { 0x85596774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819677c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26496780, 90 },
    { 0xa9b96781, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56096788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9678c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f96790, 23 },
    { 0x18496791, 23 },
    { 0x29796790, 95 },
    { 0, 0 },
    { 0x51496794, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb649679a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8589679d, 33 },
    { 0, 0 },
    { 0x6869679f, 87 },
    { 0x8589679f, 61 },
    { 0xd40967a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9967a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4967a7, 76 },
    { 0, 0 },
    { 0x456967a9, 66 },
    { 0xb6f967a9, 90 },
    { 0, 0 },
    { 0x534967ac, 34 },
    { 0, 0 },
    { 0xe9f967ae, 76 },
    { 0, 0 },
    { 0x5f8967b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44967bb, 98 },
    { 0xb69967bb, 33 },
    { 0xf07967bb, 33 },
    { 0xd37967be, 33 },
    { 0xd72967bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e967c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51967ca, 66 },
    { 0x90967cb, 33 },
    { 0, 0 },
    { 0x5db967cd, 61 },
    { 0, 0 },
    { 0xe99967cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67967d6, 100 },
    { 0, 0 },
    { 0xc58967d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b967db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206967df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112967e6, 77 },
    { 0xc1f967e7, 98 },
    { 0x5e3967e8, 90 },
    { 0xdb7967e8, 100 },
    { 0x9b0967e7, 90 },
    { 0x732967eb, 66 },
    { 0, 0 },
    { 0x15e967ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834967f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10967f6, 34 },
    { 0xc7e967f7, 23 },
    { 0x5d8967f8, 95 },
    { 0x158967f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c967fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c96800, 95 },
    { 0x99d96801, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa96807, 99 },
    { 0xffc96807, 95 },
    { 0x3c796809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c9680d, 23 },
    { 0xdc39680d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b96813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4096818, 61 },
    { 0xd9796819, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b39681e, 84 },
    { 0, 0 },
    { 0x9aa96820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83996825, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96831, 43 },
    { 0x3a696831, 55 },
    { 0, 0 },
    { 0xa3996834, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b96838, 93 },
    { 0x90096839, 61 },
    { 0xdae96839, 61 },
    { 0x71c9683b, 91 },
    { 0x71b9683b, 40 },
    { 0x5c9683c, 90 },
    { 0x2959683e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21496843, 36 },
    { 0xf1596844, 33 },
    { 0xdc696845, 88 },
    { 0, 0 },
    { 0x5ba96847, 52 },
    { 0x41296847, 98 },
    { 0, 0 },
    { 0x9559684a, 91 },
    { 0x3059684b, 61 },
    { 0x9c89684a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a96850, 77 },
    { 0x6e796851, 33 },
    { 0, 0 },
    { 0x6f396853, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e96862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96596866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319686d, 80 },
    { 0x2189686e, 66 },
    { 0, 0 },
    { 0xa3696870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56796877, 90 },
    { 0xaa196878, 17 },
    { 0, 0 },
    { 0x4569687a, 33 },
    { 0x7ef9687b, 88 },
    { 0x69c9687c, 33 },
    { 0xa389687c, 70 },
    { 0x3c19687e, 90 },
    { 0, 0 },
    { 0x48096880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d96885, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a96889, 55 },
    { 0x3569688a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a96892, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86096895, 98 },
    { 0xfb796895, 23 },
    { 0xe7796897, 33 },
    { 0xd0a96895, 66 },
    { 0, 0 },
    { 0xfa69689a, 95 },
    { 0x9f09689a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59689e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29968a2, 91 },
    { 0, 0 },
    { 0x562968a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6968a8, 77 },
    { 0, 0 },
    { 0x865968aa, 78 },
    { 0x973968ab, 23 },
    { 0, 0 },
    { 0x8e4968ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x197968b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833968b5, 61 },
    { 0x917968b6, 61 },
    { 0xe1a968b7, 23 },
    { 0xd2d968b8, 84 },
    { 0x162968b9, 78 },
    { 0x9d3968ba, 95 },
    { 0, 0 },
    { 0x7c3968bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf968c0, 34 },
    { 0x493968c1, 43 },
    { 0, 0 },
    { 0xe1f968c3, 61 },
    { 0x7aa968c4, 34 },
    { 0, 0 },
    { 0x362968c6, 90 },
    { 0x255968c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536968cc, 91 },
    { 0xce3968cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1968d0, 33 },
    { 0, 0 },
    { 0x86b968d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4968d6, 66 },
    { 0xd98968d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e968df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810968e4, 98 },
    { 0xb65968e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12968ec, 61 },
    { 0xf1f968ec, 17 },
    { 0x7c1968ee, 78 },
    { 0x659968ee, 78 },
    { 0xf77968f0, 90 },
    { 0xbd6968f1, 46 },
    { 0xd5c968f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f968f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x881968f8, 98 },
    { 0x955968f9, 61 },
    { 0, 0 },
    { 0x71e968fb, 69 },
    { 0x9c0968fb, 90 },
    { 0x436968fb, 33 },
    { 0x8e9968fe, 23 },
    { 0, 0 },
    { 0x83796900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21896908, 83 },
    { 0xb9196909, 23 },
    { 0xe589690a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6596911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x35996914, 78 },
    { 0, 0 },
    { 0xbc796916, 98 },
    { 0, 0 },
    { 0x89f96918, 84 },
    { 0, 0 },
    { 0xbc29691a, 55 },
    { 0xe639691b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5496921, 61 },
    { 0xd9596922, 33 },
    { 0xb1296923, 23 },
    { 0x68e96922, 87 },
    { 0x66796925, 34 },
    { 0x12996926, 23 },
    { 0xdb996925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199692b, 46 },
    { 0x2a29692c, 90 },
    { 0xb569692b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29196932, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4196936, 66 },
    { 0xd8e96937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369693a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8539693d, 43 },
    { 0x9a49693e, 98 },
    { 0, 0 },
    { 0xc0296940, 33 },
    { 0, 0 },
    { 0xb0f96942, 55 },
    { 0, 0 },
    { 0x6ea96944, 40 },
    { 0xae896945, 78 },
    { 0x84396945, 65 },
    { 0x9d96947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409694c, 78 },
    { 0, 0 },
    { 0xb249694e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb96954, 23 },
    { 0x85796955, 66 },
    { 0x49496955, 90 },
    { 0, 0 },
    { 0x45c96958, 33 },
    { 0, 0 },
    { 0x96b9695a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70496960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd96969, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d29696f, 66 },
    { 0xa0c96970, 65 },
    { 0x8596971, 40 },
    { 0x8e39696f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4696976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d49697a, 55 },
    { 0x939697a, 55 },
    { 0x54f9697c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d96980, 81 },
    { 0, 0 },
    { 0xd6c96982, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c696989, 93 },
    { 0, 0 },
    { 0x3519698b, 33 },
    { 0, 0 },
    { 0x6a09698d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96994, 61 },
    { 0xa1396995, 61 },
    { 0x8a696995, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8379699b, 33 },
    { 0x2e49699c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9969a5, 61 },
    { 0x665969a6, 55 },
    { 0xb85969a7, 61 },
    { 0x842969a5, 43 },
    { 0xd2969a9, 76 },
    { 0, 0 },
    { 0x8b8969ab, 33 },
    { 0xce969ac, 23 },
    { 0xfef969ab, 46 },
    { 0, 0 },
    { 0xfc7969af, 34 },
    { 0x6ad969b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a969b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918969bc, 33 },
    { 0x8b6969bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x544969c0, 33 },
    { 0x8f6969c0, 43 },
    { 0x1e3969c1, 33 },
    { 0x6c7969c3, 33 },
    { 0xcf5969c4, 90 },
    { 0xfc5969c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1969c8, 55 },
    { 0xe2969c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99969cd, 33 },
    { 0x7a6969ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb969d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870969d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0969d8, 33 },
    { 0x12c969d8, 61 },
    { 0, 0 },
    { 0x4969db, 43 },
    { 0, 0 },
    { 0x7d1969dd, 76 },
    { 0x191969dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1969e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1969ec, 43 },
    { 0x7c4969ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b969f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18969f4, 61 },
    { 0, 0 },
    { 0xead969f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274969fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c996a07, 90 },
    { 0, 0 },
    { 0xfd296a09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c96a16, 90 },
    { 0x9d596a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e496a1c, 98 },
    { 0xec96a1d, 90 },
    { 0x28996a1d, 76 },
    { 0xc3896a1f, 23 },
    { 0x61596a20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd96a23, 33 },
    { 0x55f96a24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac96a29, 33 },
    { 0x5a96a2a, 76 },
    { 0x2e796a2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5796a30, 2 },
    { 0x88e96a31, 91 },
    { 0x16496a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8896a36, 23 },
    { 0x77296a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca896a3c, 87 },
    { 0, 0 },
    { 0xc5696a3e, 55 },
    { 0x22296a3f, 90 },
    { 0x99b96a3e, 95 },
    { 0xf6296a41, 61 },
    { 0xbc296a42, 52 },
    { 0x92a96a3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f96a4e, 83 },
    { 0, 0 },
    { 0x10d96a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf896a59, 4 },
    { 0xf096a5a, 33 },
    { 0x2b296a5a, 33 },
    { 0, 0 },
    { 0xc3396a5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0896a61, 90 },
    { 0, 0 },
    { 0x4d896a63, 84 },
    { 0xd7f96a64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x52896a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e196a6b, 66 },
    { 0x16796a6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b96a6f, 55 },
    { 0x35096a70, 55 },
    { 0, 0 },
    { 0x21396a72, 90 },
    { 0, 0 },
    { 0xb9996a74, 46 },
    { 0xda996a74, 90 },
    { 0x47496a76, 52 },
    { 0xbb096a75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d96a84, 99 },
    { 0, 0 },
    { 0x58a96a86, 67 },
    { 0x5696a87, 64 },
    { 0, 0 },
    { 0x74f96a89, 76 },
    { 0x88096a89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce96a8e, 95 },
    { 0x78f96a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7996a92, 90 },
    { 0x7396a93, 46 },
    { 0x24d96a94, 90 },
    { 0, 0 },
    { 0xba496a96, 61 },
    { 0, 0 },
    { 0x59196a98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xba696a9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd96a9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc96aa4, 90 },
    { 0xe7a96aa5, 90 },
    { 0x5396aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6796aae, 17 },
    { 0, 0 },
    { 0x7d396ab0, 23 },
    { 0xcb096ab1, 90 },
    { 0xb1196ab2, 43 },
    { 0xb3196ab2, 100 },
    { 0x6f196ab4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17796ab8, 34 },
    { 0x22b96ab9, 40 },
    { 0x17996aba, 98 },
    { 0xabb96abb, 90 },
    { 0x2fa96ab9, 61 },
    { 0, 0 },
    { 0x27b96abe, 66 },
    { 0x1da96abf, 98 },
    { 0, 0 },
    { 0x60396ac1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78996ac6, 33 },
    { 0, 0 },
    { 0x23496ac8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab296acc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd96ad1, 23 },
    { 0, 0 },
    { 0xbc496ad3, 43 },
    { 0x71c96ad4, 90 },
    { 0, 0 },
    { 0x75896ad6, 87 },
    { 0x5fa96ad7, 46 },
    { 0x83b96ad8, 55 },
    { 0x8ab96ad8, 46 },
    { 0, 0 },
    { 0xd3496adb, 93 },
    { 0x2c496adc, 55 },
    { 0xeec96adc, 61 },
    { 0x13396ade, 33 },
    { 0, 0 },
    { 0x33996ae0, 61 },
    { 0x89d96ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd696ae4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e196ae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f296aeb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7896af1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55496af4, 46 },
    { 0, 0 },
    { 0x14d96af6, 23 },
    { 0xf8596af7, 95 },
    { 0xb5996af8, 90 },
    { 0x16b96af9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9596afd, 90 },
    { 0, 0 },
    { 0xb896aff, 67 },
    { 0, 0 },
    { 0x78596b01, 75 },
    { 0xd3f96b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c96b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f696b09, 90 },
    { 0, 0 },
    { 0x79596b0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32196b0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8996b13, 33 },
    { 0xc6896b14, 81 },
    { 0x7cf96b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30796b1d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c96b22, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c96b27, 76 },
    { 0x3bd96b28, 23 },
    { 0x48496b29, 87 },
    { 0xe1e96b28, 34 },
    { 0xef396b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74496b36, 90 },
    { 0x8f696b37, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e96b3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c96b40, 61 },
    { 0x1a396b41, 46 },
    { 0x13496b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1996b45, 34 },
    { 0xd2a96b46, 34 },
    { 0x3a596b47, 90 },
    { 0xb4796b48, 61 },
    { 0, 0 },
    { 0xe3596b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d496b4d, 93 },
    { 0, 0 },
    { 0x2d796b4f, 55 },
    { 0x8d696b50, 95 },
    { 0x81396b51, 90 },
    { 0x15d96b50, 90 },
    { 0x39496b53, 98 },
    { 0x6df96b54, 98 },
    { 0xc096b55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee96b59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb96b5e, 55 },
    { 0x80d96b5f, 34 },
    { 0x24996b60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd96b64, 91 },
    { 0x21996b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda096b75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23596b7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a96b7e, 97 },
    { 0xbbd96b7f, 90 },
    { 0x8e996b80, 61 },
    { 0, 0 },
    { 0x35796b82, 76 },
    { 0, 0 },
    { 0xc1b96b84, 36 },
    { 0x88496b84, 33 },
    { 0x2e496b86, 95 },
    { 0, 0 },
    { 0x8db96b88, 61 },
    { 0, 0 },
    { 0x92b96b8a, 90 },
    { 0x90496b8b, 33 },
    { 0, 0 },
    { 0x28e96b8d, 85 },
    { 0xb8c96b8d, 90 },
    { 0x4f296b8f, 61 },
    { 0xb4896b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2296b94, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b796b99, 66 },
    { 0xde96b99, 76 },
    { 0xd9a96b9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b96b9f, 95 },
    { 0xec896ba0, 61 },
    { 0, 0 },
    { 0x7e596ba2, 46 },
    { 0xc1f96ba3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f396ba9, 61 },
    { 0x3fa96baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c96bad, 77 },
    { 0x27996bae, 98 },
    { 0xbe96baf, 43 },
    { 0xda996baf, 33 },
    { 0xaf796bb1, 76 },
    { 0, 0 },
    { 0xe9f96bb3, 23 },
    { 0x19296bb4, 77 },
    { 0x61796bb4, 61 },
    { 0xefb96bb3, 43 },
    { 0x1bd96bb7, 84 },
    { 0, 0 },
    { 0x3ab96bb9, 90 },
    { 0, 0 },
    { 0xf3b96bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696bbe, 66 },
    { 0xf5896bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64996bc2, 90 },
    { 0, 0 },
    { 0x10d96bc4, 53 },
    { 0, 0 },
    { 0x11596bc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d796bcd, 93 },
    { 0x8e896bcd, 66 },
    { 0x6a296bcf, 61 },
    { 0x45896bcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c96bd5, 88 },
    { 0xb6a96bd6, 90 },
    { 0x2e996bd5, 91 },
    { 0, 0 },
    { 0xa896bd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82096be1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xde396be4, 17 },
    { 0, 0 },
    { 0xd4396be6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4396beb, 33 },
    { 0xf7996bec, 90 },
    { 0, 0 },
    { 0x48e96bee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d196bf2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x50996bf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a396bf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69096c03, 89 },
    { 0, 0 },
    { 0x4a696c05, 23 },
    { 0xab796c06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1696c09, 90 },
    { 0, 0 },
    { 0x94c96c0b, 34 },
    { 0xd4996c0c, 94 },
    { 0x93496c0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f096c18, 89 },
    { 0x7e496c19, 23 },
    { 0x57996c18, 95 },
    { 0, 0 },
    { 0xe7096c1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c96c20, 95 },
    { 0x8596c21, 67 },
    { 0, 0 },
    { 0x29496c23, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b096c28, 33 },
    { 0x15e96c28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xae296c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17596c35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f96c3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3396c3e, 93 },
    { 0, 0 },
    { 0x9f996c40, 61 },
    { 0, 0 },
    { 0xf2496c42, 90 },
    { 0xed96c43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb096c47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50496c4f, 55 },
    { 0xd8b96c50, 61 },
    { 0xf8a96c51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c96c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd196c5c, 43 },
    { 0, 0 },
    { 0x5bb96c5e, 33 },
    { 0xcbc96c5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf696c64, 90 },
    { 0xcce96c65, 90 },
    { 0x2af96c66, 95 },
    { 0x5d496c66, 43 },
    { 0x72b96c65, 43 },
    { 0xe8596c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c196c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b696c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db96c73, 95 },
    { 0x1ea96c74, 90 },
    { 0x9d796c75, 64 },
    { 0, 0 },
    { 0x5af96c77, 84 },
    { 0x5a896c78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f996c7d, 55 },
    { 0x38a96c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab296c86, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d96c8b, 68 },
    { 0xb5096c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4996c93, 98 },
    { 0x7f596c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e896c98, 95 },
    { 0, 0 },
    { 0x396c9a, 98 },
    { 0x67c96c9a, 93 },
    { 0x71b96c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e996c9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x72396ca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37396ca6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2896caf, 33 },
    { 0, 0 },
    { 0x98496cb1, 99 },
    { 0xdc296cb1, 87 },
    { 0xebf96cb2, 87 },
    { 0, 0 },
    { 0xef596cb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf296cbb, 33 },
    { 0xac196cbc, 23 },
    { 0xb7f96cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f296cc1, 93 },
    { 0x3e296cc2, 77 },
    { 0xd8196cc3, 95 },
    { 0, 0 },
    { 0xd6b96cc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc096cc9, 90 },
    { 0xf5d96cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93296cd1, 76 },
    { 0x67496cd2, 94 },
    { 0x5ab96cd3, 98 },
    { 0xfea96cd4, 52 },
    { 0xce896cd3, 95 },
    { 0xbb796cd6, 75 },
    { 0x68496cd4, 90 },
    { 0x21596cd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b96cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9696ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe96ce5, 33 },
    { 0x53c96ce6, 47 },
    { 0xb1c96ce7, 78 },
    { 0xba396ce8, 97 },
    { 0xde596ce7, 98 },
    { 0, 0 },
    { 0xecb96ceb, 23 },
    { 0, 0 },
    { 0x4ce96ced, 55 },
    { 0x8ce96cee, 87 },
    { 0xb8396cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf96cf2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea96cf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa96cf9, 55 },
    { 0xb3e96cfa, 34 },
    { 0, 0 },
    { 0x4f496cfc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a96d02, 84 },
    { 0xe8796d03, 23 },
    { 0, 0 },
    { 0x1d896d05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38996d08, 61 },
    { 0xd0496d09, 61 },
    { 0xa0196d09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff96d0e, 98 },
    { 0xffa96d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a396d20, 90 },
    { 0, 0 },
    { 0x73496d22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13396d25, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e96d2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80296d2d, 75 },
    { 0x58c96d2d, 100 },
    { 0x77f96d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee296d32, 58 },
    { 0, 0 },
    { 0xfee96d34, 100 },
    { 0x8f596d35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e96d3b, 90 },
    { 0, 0 },
    { 0x2f96d3d, 76 },
    { 0x22c96d3e, 78 },
    { 0xcf796d3f, 98 },
    { 0x96d96d40, 90 },
    { 0, 0 },
    { 0xb2696d42, 76 },
    { 0, 0 },
    { 0x8ff96d44, 90 },
    { 0xc0596d44, 95 },
    { 0x12496d44, 88 },
    { 0, 0 },
    { 0x27f96d48, 65 },
    { 0, 0 },
    { 0xf7696d4a, 98 },
    { 0xcd96d4b, 34 },
    { 0xa2f96d4c, 33 },
    { 0, 0 },
    { 0xd5296d4e, 98 },
    { 0x78c96d4e, 83 },
    { 0, 0 },
    { 0x49796d51, 34 },
    { 0xd3396d51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f96d55, 33 },
    { 0x72f96d55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c96d59, 95 },
    { 0x296d59, 23 },
    { 0xbbf96d5b, 33 },
    { 0xe096d59, 90 },
    { 0x93796d5a, 95 },
    { 0, 0 },
    { 0xe6d96d5f, 61 },
    { 0x13496d60, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c96d66, 33 },
    { 0, 0 },
    { 0x8f896d68, 95 },
    { 0x28496d68, 68 },
    { 0x77896d6a, 90 },
    { 0x58596d6a, 80 },
    { 0x89f96d6c, 90 },
    { 0x89d96d6c, 61 },
    { 0x9c396d6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b96d75, 78 },
    { 0, 0 },
    { 0x92596d77, 90 },
    { 0xd3496d77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf396d7c, 61 },
    { 0xb0b96d7d, 33 },
    { 0xaa196d7e, 90 },
    { 0x69996d7e, 52 },
    { 0x95796d80, 67 },
    { 0x16d96d81, 99 },
    { 0x24f96d81, 90 },
    { 0x3f596d82, 90 },
    { 0x5dc96d84, 95 },
    { 0x1e196d84, 61 },
    { 0x58896d86, 64 },
    { 0, 0 },
    { 0x86d96d88, 61 },
    { 0xe7596d88, 33 },
    { 0xdf196d8a, 93 },
    { 0xb2296d8a, 61 },
    { 0xd0196d8b, 95 },
    { 0x4a296d8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x18496d90, 93 },
    { 0x7fa96d91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9496d95, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b696d9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94096da1, 61 },
    { 0x4fb96da1, 90 },
    { 0xa1e96da3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f496da9, 98 },
    { 0x20596da9, 61 },
    { 0, 0 },
    { 0x20c96dac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6996db0, 90 },
    { 0x13296db1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d296dbd, 90 },
    { 0x1b496dbd, 85 },
    { 0, 0 },
    { 0xe1a96dc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86596dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d696dca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59396dcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf96dd7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61296ddb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f496de5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2996deb, 23 },
    { 0, 0 },
    { 0xa0a96ded, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7396df1, 68 },
    { 0xa0196df1, 90 },
    { 0x5c696df2, 98 },
    { 0x41296df4, 17 },
    { 0xb0996df4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff96dfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a96e04, 46 },
    { 0x45196e04, 100 },
    { 0x77696e06, 55 },
    { 0xb7c96e07, 90 },
    { 0x87296e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d96e0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c96e10, 93 },
    { 0, 0 },
    { 0xc3196e12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf996e15, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f96e18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a996e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c496e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b96e23, 85 },
    { 0x5fb96e24, 33 },
    { 0, 0 },
    { 0x79996e26, 78 },
    { 0x99b96e27, 23 },
    { 0, 0 },
    { 0x54096e29, 76 },
    { 0x8a796e2a, 90 },
    { 0xbe296e29, 55 },
    { 0xa0196e2a, 98 },
    { 0x60796e2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6296e30, 85 },
    { 0, 0 },
    { 0xbc396e32, 55 },
    { 0xd4796e33, 33 },
    { 0, 0 },
    { 0x2cd96e35, 66 },
    { 0xfd796e35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21796e3a, 34 },
    { 0x9b896e3b, 67 },
    { 0, 0 },
    { 0x14196e3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b696e43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e696e50, 68 },
    { 0xcd196e50, 81 },
    { 0, 0 },
    { 0xb8b96e53, 61 },
    { 0xfb396e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95896e5b, 91 },
    { 0x3a996e5c, 61 },
    { 0x1e196e5c, 90 },
    { 0xc4e96e5e, 52 },
    { 0x1e296e5e, 33 },
    { 0xe0296e60, 66 },
    { 0xad496e60, 67 },
    { 0xc9596e5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b596e6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53596e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d596e79, 46 },
    { 0xd7e96e7a, 33 },
    { 0x54a96e7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e396e81, 67 },
    { 0x1d496e82, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb696e85, 33 },
    { 0x7a896e86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b396e8e, 98 },
    { 0xbec96e8e, 90 },
    { 0, 0 },
    { 0x97a96e91, 33 },
    { 0x60896e91, 93 },
    { 0x61796e93, 46 },
    { 0xbd096e92, 98 },
    { 0, 0 },
    { 0xda396e96, 90 },
    { 0xcf996e96, 99 },
    { 0xda996e98, 90 },
    { 0x3e196e99, 95 },
    { 0x97896e9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c896ea0, 52 },
    { 0x3e596ea0, 46 },
    { 0x3b896ea1, 46 },
    { 0xc3d96ea3, 87 },
    { 0, 0 },
    { 0xe7d96ea5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc96eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d96eb0, 33 },
    { 0x6a596eb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3296eb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a896eb7, 55 },
    { 0x58296eb8, 99 },
    { 0, 0 },
    { 0xf2996eba, 90 },
    { 0xb8896ebb, 95 },
    { 0, 0 },
    { 0x66c96ebd, 33 },
    { 0x23896ebe, 52 },
    { 0x50c96ebf, 55 },
    { 0xa9996ec0, 33 },
    { 0, 0 },
    { 0x12896ec2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6896ec5, 97 },
    { 0, 0 },
    { 0x6a96ec7, 66 },
    { 0, 0 },
    { 0x61196ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e96ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3196ed0, 95 },
    { 0, 0 },
    { 0xfe096ed2, 90 },
    { 0, 0 },
    { 0x2a196ed4, 61 },
    { 0x16a96ed5, 98 },
    { 0, 0 },
    { 0xac096ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6996eda, 33 },
    { 0, 0 },
    { 0x29f96edc, 84 },
    { 0x20796edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1296ee3, 23 },
    { 0x6996ee4, 61 },
    { 0xcd096ee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d96eea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8496ef4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2196ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa96f06, 43 },
    { 0x6cc96f07, 76 },
    { 0xe3f96f08, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a96f0b, 66 },
    { 0x39996f0c, 93 },
    { 0xe9896f0c, 61 },
    { 0xb8396f0b, 43 },
    { 0xf8a96f0f, 69 },
    { 0x23396f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f96f19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a796f1f, 90 },
    { 0, 0 },
    { 0xee96f21, 76 },
    { 0, 0 },
    { 0xda696f23, 95 },
    { 0xec596f24, 33 },
    { 0x28596f25, 61 },
    { 0xc1796f25, 77 },
    { 0x12396f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a96f2d, 90 },
    { 0x46c96f2e, 34 },
    { 0xf6f96f2e, 100 },
    { 0x60f96f2e, 93 },
    { 0x50996f31, 33 },
    { 0xf4e96f31, 90 },
    { 0x20396f33, 46 },
    { 0x19d96f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19496f3a, 90 },
    { 0x95396f3b, 33 },
    { 0xa3b96f3c, 68 },
    { 0, 0 },
    { 0x51696f3e, 90 },
    { 0xf4e96f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1596f44, 95 },
    { 0x4296f45, 61 },
    { 0xe96f45, 33 },
    { 0x61896f44, 93 },
    { 0x2d96f48, 95 },
    { 0x39996f49, 95 },
    { 0xe3a96f4a, 52 },
    { 0x58296f4a, 33 },
    { 0x2c96f4a, 23 },
    { 0xe1496f49, 91 },
    { 0x24496f4d, 98 },
    { 0x4be96f4f, 33 },
    { 0, 0 },
    { 0x32796f51, 55 },
    { 0, 0 },
    { 0x52b96f53, 34 },
    { 0, 0 },
    { 0x1a996f55, 43 },
    { 0x28e96f55, 84 },
    { 0, 0 },
    { 0xeed96f58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24696f5b, 77 },
    { 0xf0396f5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d996f61, 95 },
    { 0x29596f62, 76 },
    { 0xf5a96f63, 76 },
    { 0x5c696f64, 76 },
    { 0x52596f62, 44 },
    { 0x6b196f66, 33 },
    { 0xd7e96f66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61996f6e, 33 },
    { 0xd4196f6f, 76 },
    { 0x42796f70, 61 },
    { 0x6d796f6f, 34 },
    { 0xbf296f72, 55 },
    { 0x28796f73, 77 },
    { 0xa7296f73, 84 },
    { 0x7196f75, 87 },
    { 0xd3f96f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac96f79, 93 },
    { 0x8ec96f7a, 93 },
    { 0, 0 },
    { 0xe9196f7c, 90 },
    { 0x59396f7d, 76 },
    { 0xff496f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f96f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a96f8c, 98 },
    { 0x34d96f8d, 90 },
    { 0x3c696f8d, 87 },
    { 0xda796f8e, 77 },
    { 0xcfe96f8e, 90 },
    { 0, 0 },
    { 0x50a96f92, 34 },
    { 0xc9f96f93, 93 },
    { 0xc6196f92, 33 },
    { 0x6aa96f95, 52 },
    { 0x16096f92, 95 },
    { 0xc796f97, 33 },
    { 0xa3f96f98, 33 },
    { 0xa9b96f99, 61 },
    { 0xde996f9a, 34 },
    { 0x91496f93, 33 },
    { 0xbbc96f98, 33 },
    { 0, 0 },
    { 0x6696f9e, 46 },
    { 0x5ab96f9e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e96fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb96faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce96fad, 95 },
    { 0xcec96fae, 95 },
    { 0xca96faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf96fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc396fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20896fbb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f96fc4, 90 },
    { 0x17396fc4, 33 },
    { 0, 0 },
    { 0x6d196fc7, 87 },
    { 0, 0 },
    { 0x64096fc9, 61 },
    { 0, 0 },
    { 0xc796fcb, 100 },
    { 0xf7396fcc, 78 },
    { 0x7d96fcc, 55 },
    { 0, 0 },
    { 0xc8496fcf, 98 },
    { 0, 0 },
    { 0x7d396fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e96fd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d96fdc, 98 },
    { 0xc8f96fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b96fe1, 17 },
    { 0x60d96fe1, 95 },
    { 0x85a96fe2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8296fe8, 43 },
    { 0x82a96fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f96fee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22496ff1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a96ff6, 68 },
    { 0, 0 },
    { 0xfd596ff8, 52 },
    { 0x98d96ff9, 98 },
    { 0x1a996ff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f696ffd, 43 },
    { 0, 0 },
    { 0xe5d96fff, 34 },
    { 0, 0 },
    { 0x27e97001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d97006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89700b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3897016, 76 },
    { 0x33a97017, 99 },
    { 0x6c297018, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa499701b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc39701f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73797028, 90 },
    { 0, 0 },
    { 0xa099702a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f097037, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b9703c, 80 },
    { 0x7ab9703d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55897040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e197044, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25897048, 38 },
    { 0, 0 },
    { 0xfe79704a, 84 },
    { 0, 0 },
    { 0x2649704c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb97052, 100 },
    { 0x3597053, 92 },
    { 0, 0 },
    { 0x39197055, 33 },
    { 0xd6897056, 90 },
    { 0x6f697057, 55 },
    { 0xb5397058, 78 },
    { 0x5b297058, 95 },
    { 0x8d897057, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e9705e, 98 },
    { 0, 0 },
    { 0xa9597060, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc97068, 76 },
    { 0, 0 },
    { 0x9279706a, 98 },
    { 0, 0 },
    { 0x84e9706c, 90 },
    { 0x68f9706c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6497071, 90 },
    { 0, 0 },
    { 0x6397073, 23 },
    { 0xf1f97074, 43 },
    { 0xaa597073, 90 },
    { 0, 0 },
    { 0x9a797077, 40 },
    { 0, 0 },
    { 0x7bc97079, 87 },
    { 0x3619707a, 43 },
    { 0x21697079, 33 },
    { 0xd7d9707c, 78 },
    { 0x18b9707c, 23 },
    { 0x34f97079, 33 },
    { 0x29b9707c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b697082, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f697086, 69 },
    { 0x19097087, 95 },
    { 0xa7597087, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf399708d, 84 },
    { 0, 0 },
    { 0xc569708f, 61 },
    { 0xff9708f, 90 },
    { 0x8399708f, 95 },
    { 0x379708f, 23 },
    { 0x4d897093, 46 },
    { 0xb6d97094, 90 },
    { 0xb0697095, 95 },
    { 0xe4497096, 90 },
    { 0xf9497096, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b970a9, 36 },
    { 0x8d0970aa, 91 },
    { 0, 0 },
    { 0xedf970ac, 98 },
    { 0, 0 },
    { 0x7af970ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x347970b1, 61 },
    { 0, 0 },
    { 0xf46970b3, 98 },
    { 0xef6970b4, 90 },
    { 0xba6970b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7970b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87970be, 33 },
    { 0, 0 },
    { 0x35b970c0, 61 },
    { 0, 0 },
    { 0x796970c2, 76 },
    { 0x9a0970c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c970c8, 23 },
    { 0, 0 },
    { 0x807970ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101970d0, 90 },
    { 0x2c5970d0, 34 },
    { 0xe18970d2, 33 },
    { 0xd09970d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1970db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5970e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee970e9, 95 },
    { 0, 0 },
    { 0x488970eb, 95 },
    { 0x814970ec, 90 },
    { 0x3e4970ed, 33 },
    { 0x71b970ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a970f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4970f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da970f7, 39 },
    { 0, 0 },
    { 0x45a970f9, 33 },
    { 0x7ed970fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6897100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5397104, 33 },
    { 0xe8197105, 32 },
    { 0, 0 },
    { 0x1b797107, 23 },
    { 0x43697108, 61 },
    { 0, 0 },
    { 0x3079710a, 55 },
    { 0, 0 },
    { 0xe1b9710c, 90 },
    { 0, 0 },
    { 0x8349710e, 33 },
    { 0xbac9710e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f97114, 55 },
    { 0x29b97115, 98 },
    { 0xf3197114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9711c, 40 },
    { 0x6e99711c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d697121, 34 },
    { 0x20797122, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149712a, 33 },
    { 0, 0 },
    { 0x35f9712c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b897131, 34 },
    { 0xac997131, 76 },
    { 0, 0 },
    { 0xd5197134, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b97137, 90 },
    { 0x3c397138, 95 },
    { 0xef897138, 77 },
    { 0x4549713a, 90 },
    { 0xf4d97137, 61 },
    { 0xd249713b, 61 },
    { 0xf589713d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5197141, 93 },
    { 0x1a797142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43897148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559714b, 66 },
    { 0xde59714c, 98 },
    { 0, 0 },
    { 0x9039714e, 33 },
    { 0x6d99714e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb97153, 78 },
    { 0, 0 },
    { 0xba897155, 87 },
    { 0x77697156, 23 },
    { 0x2dd97157, 100 },
    { 0x53497158, 90 },
    { 0, 0 },
    { 0x4c79715a, 55 },
    { 0xdd9715b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a97161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b97166, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569716c, 33 },
    { 0, 0 },
    { 0x8ad9716e, 76 },
    { 0, 0 },
    { 0x6a597170, 76 },
    { 0xea497171, 95 },
    { 0x7f697172, 61 },
    { 0x6c497170, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d097177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2499717a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82497180, 90 },
    { 0x3c197181, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xef597184, 98 },
    { 0, 0 },
    { 0xe3b97186, 61 },
    { 0xc0297187, 76 },
    { 0xfd797188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519718d, 95 },
    { 0xa829718d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e97191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9719d, 23 },
    { 0x28d9719e, 98 },
    { 0x8a39719e, 43 },
    { 0, 0 },
    { 0x943971a1, 78 },
    { 0x24c971a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62971a5, 95 },
    { 0x1eb971a6, 46 },
    { 0, 0 },
    { 0x91e971a8, 66 },
    { 0xda4971a9, 17 },
    { 0x906971a9, 95 },
    { 0xef9971ab, 95 },
    { 0x9ad971ab, 90 },
    { 0x923971ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x606971b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e971b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf971b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc971bb, 61 },
    { 0xf50971bc, 98 },
    { 0x2c8971bd, 34 },
    { 0xcc9971be, 33 },
    { 0x669971bf, 98 },
    { 0xa39971bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7971c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90971c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd971d0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167971d4, 33 },
    { 0x36d971d4, 43 },
    { 0x254971d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c971dd, 90 },
    { 0x372971dd, 34 },
    { 0x5e0971dd, 99 },
    { 0, 0 },
    { 0xb21971e1, 68 },
    { 0, 0 },
    { 0xda7971e3, 90 },
    { 0, 0 },
    { 0x214971e5, 95 },
    { 0, 0 },
    { 0x570971e7, 66 },
    { 0x7fb971e8, 75 },
    { 0, 0 },
    { 0x8ac971ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5971f1, 61 },
    { 0x5d5971f2, 65 },
    { 0xf36971f1, 76 },
    { 0, 0 },
    { 0xf4e971f5, 61 },
    { 0xbf5971f6, 98 },
    { 0x5b4971f7, 34 },
    { 0xab2971f6, 23 },
    { 0xa76971f5, 90 },
    { 0x427971f9, 37 },
    { 0xb53971fb, 33 },
    { 0x503971f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee9720b, 61 },
    { 0x68d9720c, 81 },
    { 0x5839720d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23797210, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17697215, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639721a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e97224, 33 },
    { 0, 0 },
    { 0x29c97226, 78 },
    { 0xb4297226, 33 },
    { 0xc7e97227, 23 },
    { 0xb1297229, 33 },
    { 0xd269722a, 75 },
    { 0xfb69722b, 43 },
    { 0xc4e9722b, 46 },
    { 0, 0 },
    { 0xd659722e, 95 },
    { 0, 0 },
    { 0xc197230, 61 },
    { 0x13d97231, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31297236, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8199723d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea97240, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d97244, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f497247, 93 },
    { 0, 0 },
    { 0xaa97249, 95 },
    { 0x18a9724a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9724e, 98 },
    { 0x3439724f, 88 },
    { 0xccf9724e, 98 },
    { 0xbc597251, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb97254, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9725b, 43 },
    { 0xb659725c, 33 },
    { 0x2249725d, 33 },
    { 0xa5e9725e, 23 },
    { 0xd3d9725f, 43 },
    { 0, 0 },
    { 0x83697261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a9726b, 98 },
    { 0, 0 },
    { 0xdee9726d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b597270, 56 },
    { 0, 0 },
    { 0xa2097272, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54397275, 91 },
    { 0xbea97276, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38097279, 33 },
    { 0xcb69727a, 52 },
    { 0x6f79727b, 23 },
    { 0x7069727b, 95 },
    { 0x7d79727c, 61 },
    { 0xe929727a, 95 },
    { 0xb809727f, 98 },
    { 0x75097280, 46 },
    { 0x75797281, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e97286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8229728a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b9728f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6599729a, 99 },
    { 0, 0 },
    { 0x9c49729c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a39729f, 93 },
    { 0, 0 },
    { 0x467972a1, 34 },
    { 0xf37972a2, 90 },
    { 0xc08972a2, 23 },
    { 0, 0 },
    { 0x801972a5, 95 },
    { 0x30e972a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f972a9, 46 },
    { 0, 0 },
    { 0x2a3972ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea972af, 93 },
    { 0xcc8972af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816972b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68972b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e972bd, 55 },
    { 0xe09972be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c972c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81972ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f972d1, 78 },
    { 0x92f972d1, 65 },
    { 0, 0 },
    { 0xc12972d4, 90 },
    { 0xf70972d5, 88 },
    { 0x786972d4, 23 },
    { 0, 0 },
    { 0x54c972d8, 34 },
    { 0x851972d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c972dc, 58 },
    { 0xe19972dc, 65 },
    { 0x16f972de, 67 },
    { 0, 0 },
    { 0x921972e0, 76 },
    { 0x4ae972e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x807972e4, 84 },
    { 0x8f1972e5, 76 },
    { 0x8b3972e4, 87 },
    { 0xefc972e7, 55 },
    { 0xb1c972e4, 88 },
    { 0x847972e9, 95 },
    { 0xcce972e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52972ef, 95 },
    { 0x277972f0, 95 },
    { 0x67b972f1, 87 },
    { 0x49c972f2, 98 },
    { 0xcc8972f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34972f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a972fc, 55 },
    { 0xd81972fd, 90 },
    { 0x9b5972fd, 55 },
    { 0x152972ff, 46 },
    { 0x782972ff, 91 },
    { 0x2ad972ff, 23 },
    { 0, 0 },
    { 0x24e97303, 98 },
    { 0x6a097303, 90 },
    { 0xda397305, 53 },
    { 0x22497306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5239730a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x289730d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b97314, 61 },
    { 0xef397314, 33 },
    { 0xd3397314, 43 },
    { 0x8e497314, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x659731a, 33 },
    { 0x4d49731b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb697322, 90 },
    { 0x12997323, 77 },
    { 0x83597323, 46 },
    { 0xf0b97325, 90 },
    { 0xf9797326, 95 },
    { 0x34597327, 90 },
    { 0xa7997328, 34 },
    { 0x17d97327, 34 },
    { 0x14397329, 33 },
    { 0x7b79732b, 100 },
    { 0xf869732b, 33 },
    { 0, 0 },
    { 0x9b89732e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a97332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48397338, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed69733f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9734a, 90 },
    { 0x13c9734a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39734e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd97353, 23 },
    { 0x78697354, 33 },
    { 0xc1e97355, 33 },
    { 0x65997355, 52 },
    { 0x47797356, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6319735c, 90 },
    { 0x719735d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34397360, 95 },
    { 0x13197360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a09736a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8319736e, 98 },
    { 0x9439736f, 77 },
    { 0x56597370, 98 },
    { 0xa0d97370, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c9737a, 65 },
    { 0, 0 },
    { 0x6089737c, 55 },
    { 0x65c9737c, 83 },
    { 0x8ee9737c, 17 },
    { 0xfe89737c, 95 },
    { 0x7229737d, 33 },
    { 0x4697381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f397385, 61 },
    { 0xb7997385, 90 },
    { 0x3b597387, 78 },
    { 0x82997386, 90 },
    { 0x10397389, 61 },
    { 0x7297385, 90 },
    { 0, 0 },
    { 0x889738c, 76 },
    { 0x8ae9738d, 33 },
    { 0, 0 },
    { 0x65f9738f, 55 },
    { 0, 0 },
    { 0xd5897391, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64197397, 95 },
    { 0x22a97398, 90 },
    { 0xa0d97398, 23 },
    { 0, 0 },
    { 0xbb79739b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d973a9, 95 },
    { 0x445973a9, 90 },
    { 0xc33973ab, 33 },
    { 0x555973aa, 95 },
    { 0xc10973ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468973b2, 23 },
    { 0xba7973b3, 46 },
    { 0, 0 },
    { 0xca2973b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0973ba, 90 },
    { 0, 0 },
    { 0xbfc973bc, 46 },
    { 0x418973bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d973c4, 90 },
    { 0x513973c5, 23 },
    { 0x291973c4, 98 },
    { 0x395973c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f973cc, 75 },
    { 0x7ed973cd, 93 },
    { 0, 0 },
    { 0x740973cf, 9 },
    { 0xba5973d0, 90 },
    { 0xf32973d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990973d5, 87 },
    { 0, 0 },
    { 0xc08973d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429973df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3973e2, 61 },
    { 0xa45973e3, 90 },
    { 0xda2973e3, 98 },
    { 0x90a973e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a973e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e973ed, 90 },
    { 0xd1f973ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1973f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358973f9, 90 },
    { 0x169973fa, 23 },
    { 0xb02973fb, 98 },
    { 0xce0973fa, 56 },
    { 0, 0 },
    { 0x80f973fe, 95 },
    { 0xa40973ff, 98 },
    { 0x4f397400, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a97404, 95 },
    { 0, 0 },
    { 0xa0097406, 95 },
    { 0x94a97406, 100 },
    { 0xca597406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9740b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf219740f, 23 },
    { 0xa3197410, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88597413, 33 },
    { 0x7c397414, 45 },
    { 0x71e97415, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d9741f, 90 },
    { 0xa97420, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb97427, 61 },
    { 0xe1a97428, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c49742b, 90 },
    { 0xdd69742c, 78 },
    { 0xaf79742d, 81 },
    { 0x58e9742d, 23 },
    { 0xbdf9742b, 55 },
    { 0x40497430, 75 },
    { 0x8a097431, 99 },
    { 0x85197431, 76 },
    { 0xd797433, 94 },
    { 0xc149742d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37997438, 93 },
    { 0xb7797439, 76 },
    { 0x13897439, 90 },
    { 0x1e097438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b97444, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6397447, 34 },
    { 0xa8697448, 90 },
    { 0, 0 },
    { 0xf1e9744a, 76 },
    { 0x9759744a, 61 },
    { 0, 0 },
    { 0x81a9744d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d97450, 55 },
    { 0x8e697451, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1297454, 23 },
    { 0, 0 },
    { 0xadc97456, 90 },
    { 0x3f397457, 75 },
    { 0x6fc97458, 91 },
    { 0, 0 },
    { 0xbfc9745a, 65 },
    { 0x73b9745b, 61 },
    { 0xec99745c, 43 },
    { 0, 0 },
    { 0xfc39745e, 33 },
    { 0x7d9745f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c97463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e197467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c09746c, 34 },
    { 0x98b9746c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3097471, 33 },
    { 0, 0 },
    { 0x71197473, 23 },
    { 0xcc097474, 87 },
    { 0xc7797475, 95 },
    { 0xa9197475, 76 },
    { 0x96b97477, 91 },
    { 0x63e97475, 61 },
    { 0x16997475, 43 },
    { 0x11597476, 43 },
    { 0x5eb9747b, 98 },
    { 0x7ad9747c, 90 },
    { 0xdaa9747d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f97480, 95 },
    { 0xea997480, 69 },
    { 0xfba97482, 90 },
    { 0, 0 },
    { 0x76a97484, 90 },
    { 0xa3e97484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d9748d, 90 },
    { 0x7409748e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0097494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b97497, 93 },
    { 0x42a97497, 100 },
    { 0x64197499, 34 },
    { 0, 0 },
    { 0x1149749b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9749f, 23 },
    { 0xd819749f, 90 },
    { 0x4aa9749f, 87 },
    { 0x649974a2, 64 },
    { 0x72b9749f, 61 },
    { 0x4d69749f, 43 },
    { 0xfdc974a5, 84 },
    { 0xa57974a6, 66 },
    { 0x35f974a4, 95 },
    { 0x539974a8, 98 },
    { 0x3e9974a9, 34 },
    { 0x829974aa, 52 },
    { 0xdef974a8, 53 },
    { 0xcab974ac, 90 },
    { 0xc2e974ad, 61 },
    { 0xaba974a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93974bb, 34 },
    { 0xce8974bc, 43 },
    { 0, 0 },
    { 0xe62974be, 52 },
    { 0xf50974bf, 61 },
    { 0x4b1974bf, 76 },
    { 0x815974be, 95 },
    { 0xc84974c2, 77 },
    { 0x7cd974c0, 33 },
    { 0, 0 },
    { 0x3f8974c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b974cc, 61 },
    { 0x696974cd, 77 },
    { 0, 0 },
    { 0x7cb974cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173974d3, 77 },
    { 0x455974d3, 64 },
    { 0x43974d5, 61 },
    { 0x1cc974d5, 88 },
    { 0x75f974d7, 61 },
    { 0x2b6974d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2974e5, 23 },
    { 0xb35974e5, 90 },
    { 0, 0 },
    { 0x3c974e8, 23 },
    { 0x816974e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b974f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d974f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56974fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee97502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf97507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c9750a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb097513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e97516, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79751f, 33 },
    { 0x53f9751f, 95 },
    { 0x8bc97521, 78 },
    { 0x3c497522, 61 },
    { 0, 0 },
    { 0x32797524, 33 },
    { 0x3e697525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999752c, 97 },
    { 0xaca9752d, 90 },
    { 0xc369752e, 100 },
    { 0x4d9752f, 33 },
    { 0xcba97530, 61 },
    { 0x2d19752d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65197535, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb9753a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb97541, 95 },
    { 0xc8197542, 33 },
    { 0x42e97542, 61 },
    { 0xc7c97542, 90 },
    { 0x18e97542, 33 },
    { 0, 0 },
    { 0x8ee97547, 95 },
    { 0xd6497548, 55 },
    { 0xca297548, 76 },
    { 0, 0 },
    { 0x139754b, 33 },
    { 0, 0 },
    { 0x34b9754d, 76 },
    { 0, 0 },
    { 0xb6a9754f, 34 },
    { 0xd4597550, 90 },
    { 0x7ac97551, 98 },
    { 0xb797550, 33 },
    { 0x83a97553, 100 },
    { 0, 0 },
    { 0xbbd97555, 68 },
    { 0x1ec97556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d97559, 76 },
    { 0x3a79755a, 34 },
    { 0x30897559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89755f, 34 },
    { 0xe9f97560, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e797567, 90 },
    { 0x56197568, 68 },
    { 0xc1197569, 98 },
    { 0xa6597568, 100 },
    { 0xb097568, 23 },
    { 0x33c9756c, 61 },
    { 0xa6b9756c, 76 },
    { 0, 0 },
    { 0x54e9756f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60997574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87197579, 33 },
    { 0x1d89757a, 56 },
    { 0, 0 },
    { 0x3bd9757c, 23 },
    { 0, 0 },
    { 0xb599757e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd197581, 76 },
    { 0, 0 },
    { 0xfe297583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c997587, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e89758a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d9758d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e97592, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62097596, 33 },
    { 0x5ee97597, 33 },
    { 0x20697596, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd975a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f975aa, 98 },
    { 0, 0 },
    { 0xaec975ac, 95 },
    { 0x345975ad, 95 },
    { 0xae4975ae, 63 },
    { 0x650975af, 33 },
    { 0xe6d975b0, 61 },
    { 0x560975af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d975b4, 66 },
    { 0, 0 },
    { 0xe1b975b6, 33 },
    { 0xe75975b7, 33 },
    { 0xbbd975b8, 91 },
    { 0xeef975b7, 33 },
    { 0x33a975ba, 61 },
    { 0x5a8975bb, 61 },
    { 0xfe9975bc, 33 },
    { 0x215975bd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8975c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef975ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29975d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea975d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd975d9, 65 },
    { 0x26975da, 33 },
    { 0, 0 },
    { 0x187975dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x150975df, 99 },
    { 0xa76975e0, 95 },
    { 0xb3b975e0, 96 },
    { 0, 0 },
    { 0xf3a975e3, 90 },
    { 0xe66975e4, 23 },
    { 0x394975e5, 23 },
    { 0x9ea975e6, 98 },
    { 0xb5f975e7, 33 },
    { 0, 0 },
    { 0x7cc975e9, 33 },
    { 0xe01975ea, 84 },
    { 0xa7975ea, 23 },
    { 0, 0 },
    { 0x46f975ed, 90 },
    { 0, 0 },
    { 0x2fd975ef, 84 },
    { 0x7d3975f0, 55 },
    { 0xceb975ef, 33 },
    { 0x5e2975f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf975f8, 23 },
    { 0xb49975f8, 77 },
    { 0, 0 },
    { 0x7e5975fb, 90 },
    { 0x5ca975fc, 90 },
    { 0x18e975fd, 90 },
    { 0x6f9975fe, 55 },
    { 0x695975ff, 90 },
    { 0x518975fd, 23 },
    { 0xba1975fd, 90 },
    { 0x5a4975ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5797606, 98 },
    { 0, 0 },
    { 0xf8697608, 93 },
    { 0xae197609, 34 },
    { 0x4c9760a, 98 },
    { 0x899760b, 66 },
    { 0, 0 },
    { 0x1ee9760d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88697614, 75 },
    { 0, 0 },
    { 0xeb497616, 98 },
    { 0, 0 },
    { 0x16c97618, 61 },
    { 0xa3097618, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739761d, 68 },
    { 0xc3c9761d, 95 },
    { 0xeeb9761f, 90 },
    { 0x3499761f, 98 },
    { 0x14b97621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9979762b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a97634, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x49897637, 81 },
    { 0xb2597638, 98 },
    { 0x55797639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff9763c, 90 },
    { 0x88c9763d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a97644, 66 },
    { 0x5497644, 90 },
    { 0, 0 },
    { 0x96097647, 47 },
    { 0, 0 },
    { 0x7c097649, 93 },
    { 0x9d69764a, 76 },
    { 0x62d9764b, 90 },
    { 0x5f99764c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb219764f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb597652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea97656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1889765a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239765d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e97660, 55 },
    { 0x6a597661, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f97668, 90 },
    { 0xc4e97669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e9766d, 33 },
    { 0x62f9766d, 64 },
    { 0xa979766f, 61 },
    { 0xf1f9766f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa97674, 33 },
    { 0, 0 },
    { 0x5cf97676, 95 },
    { 0xc1a97677, 23 },
    { 0, 0 },
    { 0xf8497679, 55 },
    { 0xed29767a, 95 },
    { 0x1fc9767b, 90 },
    { 0, 0 },
    { 0xe5c9767d, 23 },
    { 0xc869767e, 61 },
    { 0x58a9767f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a9768d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76597694, 90 },
    { 0, 0 },
    { 0x4f897696, 34 },
    { 0xe4997697, 90 },
    { 0xce997696, 99 },
    { 0, 0 },
    { 0x3339769a, 85 },
    { 0, 0 },
    { 0xf619769c, 90 },
    { 0xf1c9769c, 61 },
    { 0xfd29769d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f976a3, 39 },
    { 0xe27976a4, 90 },
    { 0xa03976a4, 98 },
    { 0x7d5976a6, 52 },
    { 0x4a4976a3, 98 },
    { 0x877976a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308976ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785976b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d976b6, 46 },
    { 0, 0 },
    { 0x782976b8, 95 },
    { 0xeb0976b9, 98 },
    { 0, 0 },
    { 0x1c5976bb, 23 },
    { 0, 0 },
    { 0xea976bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314976c3, 23 },
    { 0xee6976c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed976cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9976da, 90 },
    { 0x9d976da, 46 },
    { 0x49d976dc, 61 },
    { 0x215976da, 55 },
    { 0, 0 },
    { 0x352976df, 33 },
    { 0x1d4976e0, 33 },
    { 0xbdb976e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e976e5, 90 },
    { 0x5df976e6, 100 },
    { 0xabc976e7, 61 },
    { 0, 0 },
    { 0x1ac976e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd976ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f976f1, 91 },
    { 0x722976f2, 90 },
    { 0x2c9976f3, 90 },
    { 0x381976f4, 84 },
    { 0x1db976f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98976f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15976ff, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98497703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf889770d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84397712, 23 },
    { 0, 0 },
    { 0x1f997714, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d97717, 95 },
    { 0xf7097717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7839771b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf259771e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe97722, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2597725, 23 },
    { 0x13e97726, 90 },
    { 0xbb997726, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa609772a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae397730, 76 },
    { 0, 0 },
    { 0xc4f97732, 61 },
    { 0, 0 },
    { 0xa6497734, 23 },
    { 0x56997735, 24 },
    { 0x74997735, 95 },
    { 0, 0 },
    { 0x10597738, 61 },
    { 0xf5a97739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19773c, 90 },
    { 0xa879773d, 74 },
    { 0xda09773e, 46 },
    { 0xf129773d, 43 },
    { 0xbb197740, 23 },
    { 0x41b9773e, 61 },
    { 0xaaf97742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd697748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a9774b, 76 },
    { 0xef79774c, 91 },
    { 0x7e29774b, 98 },
    { 0x7ed9774b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4197752, 33 },
    { 0x4e997753, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x47997756, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba697760, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55097766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2589776b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9776f, 90 },
    { 0xbc997770, 48 },
    { 0x62d97770, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0697777, 23 },
    { 0xd4e97777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d9777d, 23 },
    { 0xc0c9777d, 93 },
    { 0xb3c9777d, 61 },
    { 0, 0 },
    { 0xece97781, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb097786, 95 },
    { 0x9c197787, 33 },
    { 0xbf897788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459778b, 90 },
    { 0x3ae9778c, 81 },
    { 0, 0 },
    { 0x7dd9778e, 43 },
    { 0, 0 },
    { 0x60997790, 46 },
    { 0xaaf97790, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b97798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a977a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08977ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420977b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c977b5, 61 },
    { 0, 0 },
    { 0x67c977b7, 90 },
    { 0x355977b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b977bc, 61 },
    { 0x912977bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793977c1, 23 },
    { 0xd67977c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426977c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92977ca, 43 },
    { 0x89b977ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745977d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa977da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46977e5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0977ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c977ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42977f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f977fb, 90 },
    { 0xef3977fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c97800, 75 },
    { 0x5ca97800, 61 },
    { 0, 0 },
    { 0xacc97803, 23 },
    { 0x1ef97804, 93 },
    { 0x5c97803, 55 },
    { 0, 0 },
    { 0x2ef97807, 93 },
    { 0, 0 },
    { 0x4b697809, 87 },
    { 0x9e29780a, 33 },
    { 0xfe597809, 90 },
    { 0x9139780a, 61 },
    { 0xcc797809, 33 },
    { 0x39e9780d, 23 },
    { 0, 0 },
    { 0x1cc97810, 43 },
    { 0, 0 },
    { 0xb4f97812, 23 },
    { 0, 0 },
    { 0x68397814, 61 },
    { 0x3f097815, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb497818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409781b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df97823, 95 },
    { 0xbdb97824, 98 },
    { 0, 0 },
    { 0x2c797826, 61 },
    { 0xbb197826, 34 },
    { 0, 0 },
    { 0x48797829, 33 },
    { 0x88d97829, 61 },
    { 0x9e9782b, 53 },
    { 0x2fe97829, 98 },
    { 0, 0 },
    { 0x7019782e, 75 },
    { 0xbbe9782e, 33 },
    { 0x61897830, 95 },
    { 0xc7097831, 90 },
    { 0xfa897832, 100 },
    { 0x9bb97833, 98 },
    { 0x3a097834, 91 },
    { 0x8329782e, 33 },
    { 0x20a9782e, 61 },
    { 0, 0 },
    { 0x3db97838, 84 },
    { 0x8a997839, 95 },
    { 0x93297838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7797842, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e297847, 98 },
    { 0x2f597848, 61 },
    { 0xeee97849, 91 },
    { 0, 0 },
    { 0xdc49784b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d97850, 87 },
    { 0x2e797851, 61 },
    { 0xefe97851, 95 },
    { 0x21097852, 98 },
    { 0x17897854, 88 },
    { 0xcb297855, 61 },
    { 0xd3897856, 19 },
    { 0x73f97856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa9785b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769785e, 52 },
    { 0, 0 },
    { 0x44897860, 76 },
    { 0xcb197861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac97866, 88 },
    { 0, 0 },
    { 0x35597868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9459786b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15597871, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea97874, 100 },
    { 0, 0 },
    { 0x7db97876, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e97879, 25 },
    { 0xc669787a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b49787d, 67 },
    { 0x9e99787e, 90 },
    { 0xbde9787f, 99 },
    { 0xeb29787e, 33 },
    { 0xdbf97881, 32 },
    { 0x32897882, 99 },
    { 0xfb97883, 90 },
    { 0, 0 },
    { 0x13c97885, 87 },
    { 0x14197886, 98 },
    { 0x46b97885, 23 },
    { 0, 0 },
    { 0x4f097889, 55 },
    { 0, 0 },
    { 0x4ed9788b, 33 },
    { 0, 0 },
    { 0xfd9788d, 33 },
    { 0xe7a9788d, 95 },
    { 0x5e49788f, 23 },
    { 0xd5097890, 83 },
    { 0, 0 },
    { 0x9e197892, 23 },
    { 0x61d97893, 33 },
    { 0, 0 },
    { 0x90797895, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58897899, 69 },
    { 0x52b9789a, 78 },
    { 0xea49789a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce978a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c978ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91978b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4978b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9978ba, 55 },
    { 0x444978ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc978be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11978c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb978cc, 55 },
    { 0x82f978cd, 61 },
    { 0x4c4978ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9978d1, 43 },
    { 0, 0 },
    { 0x4de978d3, 90 },
    { 0x604978d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02978d8, 23 },
    { 0x5ef978d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf978de, 34 },
    { 0xb6f978df, 66 },
    { 0x735978e0, 43 },
    { 0x26e978de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb978e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d978eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41978ef, 95 },
    { 0, 0 },
    { 0xdb4978f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519978f8, 33 },
    { 0x63a978f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53978fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a97902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24997910, 100 },
    { 0xc397910, 90 },
    { 0x10197912, 95 },
    { 0x83997911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75997916, 55 },
    { 0xe8e97916, 33 },
    { 0, 0 },
    { 0xbd597919, 40 },
    { 0x6109791a, 67 },
    { 0, 0 },
    { 0xc29791c, 34 },
    { 0x37b9791d, 66 },
    { 0x8999791d, 61 },
    { 0, 0 },
    { 0xa1097920, 90 },
    { 0x3ca97920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f397924, 33 },
    { 0, 0 },
    { 0xb8897926, 61 },
    { 0xe8e97926, 95 },
    { 0x6fe97927, 98 },
    { 0x58f97926, 98 },
    { 0x4859792a, 83 },
    { 0x40b9792a, 34 },
    { 0, 0 },
    { 0xc729792d, 98 },
    { 0xe8f9792e, 61 },
    { 0, 0 },
    { 0x11b97930, 23 },
    { 0x6bc97931, 61 },
    { 0, 0 },
    { 0xccc97933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af97936, 76 },
    { 0x5e397936, 98 },
    { 0x59797938, 90 },
    { 0xe4397937, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf859793f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f97946, 83 },
    { 0x62497947, 33 },
    { 0xd2a97948, 90 },
    { 0, 0 },
    { 0x7249794a, 55 },
    { 0xa5b9794b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b297951, 23 },
    { 0xfe897952, 76 },
    { 0x7c497952, 76 },
    { 0x5a297954, 61 },
    { 0x79b97955, 76 },
    { 0, 0 },
    { 0x5e497957, 23 },
    { 0x8c097958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c39795d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0997962, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e197965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f97969, 23 },
    { 0x7a49796a, 61 },
    { 0, 0 },
    { 0xfde9796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99796f, 61 },
    { 0x53497970, 43 },
    { 0x7a397970, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4197975, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8699797b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8197983, 33 },
    { 0, 0 },
    { 0x7fa97985, 90 },
    { 0x71b97986, 98 },
    { 0xbf397987, 100 },
    { 0x17397988, 61 },
    { 0x3ab97988, 66 },
    { 0xd2397988, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60997990, 61 },
    { 0xb8d97991, 61 },
    { 0, 0 },
    { 0xf9297993, 23 },
    { 0x5e997994, 61 },
    { 0xb9897995, 23 },
    { 0xa5997995, 98 },
    { 0xe9697997, 61 },
    { 0xffa97998, 33 },
    { 0x9a797999, 90 },
    { 0xd0397999, 23 },
    { 0x9029799b, 68 },
    { 0x5189799b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f9799f, 95 },
    { 0, 0 },
    { 0xd0979a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1979a5, 90 },
    { 0x87b979a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf979ab, 46 },
    { 0x7cc979ac, 43 },
    { 0x268979ad, 93 },
    { 0x5d0979ad, 66 },
    { 0xc5f979af, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30979b5, 95 },
    { 0x995979b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e979b9, 88 },
    { 0x6bb979ba, 78 },
    { 0xdc5979ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b979c0, 94 },
    { 0x191979c1, 84 },
    { 0x1dd979c2, 95 },
    { 0xd92979c2, 23 },
    { 0xcc7979c4, 76 },
    { 0, 0 },
    { 0x1e3979c6, 90 },
    { 0x305979c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57979ca, 33 },
    { 0xa98979cb, 98 },
    { 0x9cc979cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25979d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2979d6, 91 },
    { 0xf06979d7, 90 },
    { 0x353979d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1979e0, 34 },
    { 0x1c5979e1, 69 },
    { 0xf3c979e2, 90 },
    { 0, 0 },
    { 0xd79979e4, 90 },
    { 0xa53979e5, 23 },
    { 0x798979e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e979e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7979ec, 52 },
    { 0, 0 },
    { 0x564979ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x850979f1, 98 },
    { 0xf87979f2, 55 },
    { 0x17f979f3, 43 },
    { 0xebb979f4, 74 },
    { 0x464979f2, 90 },
    { 0, 0 },
    { 0xa92979f7, 70 },
    { 0xbe8979f7, 66 },
    { 0x5ed979f7, 90 },
    { 0, 0 },
    { 0x316979fb, 46 },
    { 0xeb7979fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e997a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d697a03, 55 },
    { 0, 0 },
    { 0x22097a05, 61 },
    { 0x3d997a06, 23 },
    { 0xe8997a06, 76 },
    { 0xc8197a08, 38 },
    { 0x2bd97a09, 76 },
    { 0, 0 },
    { 0xa7097a0b, 98 },
    { 0x51f97a0b, 100 },
    { 0x3b997a0b, 98 },
    { 0xb2197a0e, 90 },
    { 0xa397a0e, 90 },
    { 0x3f297a0f, 98 },
    { 0xddd97a11, 95 },
    { 0, 0 },
    { 0x45997a13, 88 },
    { 0x79097a13, 33 },
    { 0, 0 },
    { 0xd3a97a16, 40 },
    { 0, 0 },
    { 0xc8997a18, 93 },
    { 0, 0 },
    { 0xf6997a1a, 33 },
    { 0, 0 },
    { 0xc8d97a1c, 78 },
    { 0xdbc97a1d, 34 },
    { 0xeea97a1e, 23 },
    { 0xe0097a1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31497a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6397a27, 55 },
    { 0x39497a28, 23 },
    { 0x35497a29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b497a2c, 61 },
    { 0, 0 },
    { 0x38597a2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86797a37, 34 },
    { 0x60897a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76597a3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d97a47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f697a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e97a52, 95 },
    { 0xae97a53, 17 },
    { 0x5c597a53, 23 },
    { 0, 0 },
    { 0xc3197a56, 61 },
    { 0x42f97a57, 55 },
    { 0x8b097a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7397a5b, 95 },
    { 0, 0 },
    { 0xe1097a5d, 95 },
    { 0, 0 },
    { 0xe1697a5f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f97a62, 34 },
    { 0x80497a63, 100 },
    { 0x9ef97a63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85797a67, 84 },
    { 0xd797a68, 77 },
    { 0, 0 },
    { 0x75897a6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b97a6d, 76 },
    { 0x11b97a6e, 98 },
    { 0x4fa97a6f, 90 },
    { 0xae897a70, 78 },
    { 0xb8997a70, 61 },
    { 0x7b797a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e297a77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1997a7b, 66 },
    { 0, 0 },
    { 0x98e97a7d, 61 },
    { 0, 0 },
    { 0xd6b97a7f, 76 },
    { 0, 0 },
    { 0xb6d97a81, 55 },
    { 0, 0 },
    { 0xec197a83, 55 },
    { 0xdfe97a84, 90 },
    { 0, 0 },
    { 0x82297a86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b97a8b, 61 },
    { 0xfbd97a8c, 55 },
    { 0x30397a8c, 95 },
    { 0x2d197a8b, 61 },
    { 0x78c97a8d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a97a92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f97a98, 34 },
    { 0x9e797a98, 33 },
    { 0, 0 },
    { 0x89897a9b, 67 },
    { 0, 0 },
    { 0x50797a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57097aa3, 23 },
    { 0xe0f97aa4, 61 },
    { 0, 0 },
    { 0x46c97aa6, 95 },
    { 0x58f97aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70497aae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd297ab5, 61 },
    { 0, 0 },
    { 0xccd97ab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b97abb, 43 },
    { 0, 0 },
    { 0xf4397abd, 55 },
    { 0x1fe97abd, 61 },
    { 0x21597abd, 96 },
    { 0x13697abf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a97ac4, 33 },
    { 0xe1f97ac5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7697ac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c197ace, 23 },
    { 0x8dd97ace, 23 },
    { 0, 0 },
    { 0xc0797ad1, 69 },
    { 0, 0 },
    { 0xa4a97ad3, 68 },
    { 0xe0797ad4, 91 },
    { 0x46897ad5, 95 },
    { 0x39997ad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f97ad9, 93 },
    { 0xdc297ada, 61 },
    { 0x5f597adb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d97ade, 33 },
    { 0x44f97ade, 23 },
    { 0xcfd97ade, 23 },
    { 0, 0 },
    { 0xce197ae2, 23 },
    { 0xc3497ae3, 98 },
    { 0xe9f97ae3, 17 },
    { 0x9f397ae5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc97ae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f97af1, 98 },
    { 0x48e97af2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73097af8, 93 },
    { 0xe2797af9, 95 },
    { 0x70297af9, 90 },
    { 0x24797afb, 65 },
    { 0xb9b97afc, 19 },
    { 0xd0d97afc, 85 },
    { 0x4fe97afe, 33 },
    { 0xb6597afc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed397b03, 34 },
    { 0x6f97b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce197b07, 95 },
    { 0x5b97b08, 76 },
    { 0x6af97b08, 34 },
    { 0x5e997b07, 95 },
    { 0, 0 },
    { 0x15397b0c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597b11, 87 },
    { 0x12897b12, 68 },
    { 0, 0 },
    { 0x24297b14, 33 },
    { 0x10997b14, 53 },
    { 0xad97b16, 82 },
    { 0, 0 },
    { 0x4af97b18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8597b1c, 61 },
    { 0x84a97b1d, 80 },
    { 0, 0 },
    { 0xc9397b1f, 61 },
    { 0, 0 },
    { 0x1c597b21, 95 },
    { 0, 0 },
    { 0x4be97b23, 66 },
    { 0, 0 },
    { 0xfeb97b25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8597b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea97b2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a697b35, 90 },
    { 0, 0 },
    { 0xd2297b37, 76 },
    { 0xb0897b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c697b3b, 23 },
    { 0x42a97b3c, 90 },
    { 0, 0 },
    { 0x99497b3e, 52 },
    { 0x9a197b3f, 88 },
    { 0xa4897b3f, 98 },
    { 0x33497b41, 34 },
    { 0x2c097b42, 90 },
    { 0xd2797b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a297b46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e97b4a, 90 },
    { 0x6b297b4a, 90 },
    { 0, 0 },
    { 0xf597b4d, 33 },
    { 0x12597b4e, 99 },
    { 0x64f97b4f, 33 },
    { 0, 0 },
    { 0x6b897b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a97b56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a97b59, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9097b5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e97b60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7797b64, 67 },
    { 0x2cc97b65, 77 },
    { 0x16a97b65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3197b6a, 98 },
    { 0, 0 },
    { 0x7cf97b6c, 98 },
    { 0x87697b6c, 66 },
    { 0x12697b6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7497b71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a97b75, 91 },
    { 0xa2497b76, 33 },
    { 0, 0 },
    { 0x4c097b78, 33 },
    { 0x4a497b79, 98 },
    { 0, 0 },
    { 0x5fd97b7b, 61 },
    { 0xae897b7c, 33 },
    { 0, 0 },
    { 0x8c597b7e, 33 },
    { 0x5f297b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb97b82, 76 },
    { 0x69797b83, 91 },
    { 0xabd97b83, 53 },
    { 0, 0 },
    { 0xddd97b86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29597b89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc97b8d, 34 },
    { 0x13197b8d, 90 },
    { 0xc1f97b8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f97b92, 90 },
    { 0x7b297b93, 76 },
    { 0xf4a97b93, 90 },
    { 0x22f97b92, 51 },
    { 0x58597b93, 33 },
    { 0, 0 },
    { 0x4f97b98, 96 },
    { 0xe297b99, 95 },
    { 0xa0d97b9a, 85 },
    { 0xc1c97b9a, 23 },
    { 0x38f97b9c, 88 },
    { 0, 0 },
    { 0x7b597b9e, 93 },
    { 0x95297b9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a97ba4, 99 },
    { 0, 0 },
    { 0xdcb97ba6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497bac, 76 },
    { 0x57c97bac, 95 },
    { 0, 0 },
    { 0xce997baf, 76 },
    { 0, 0 },
    { 0x95d97bb1, 66 },
    { 0x59e97bb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b597bb6, 90 },
    { 0, 0 },
    { 0xe5b97bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f97bc0, 23 },
    { 0, 0 },
    { 0x5a997bc2, 90 },
    { 0xefc97bc3, 85 },
    { 0x62c97bc3, 98 },
    { 0, 0 },
    { 0x48097bc6, 90 },
    { 0x7ca97bc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b97bcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36097bd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45897bd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76797bd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f997bdd, 87 },
    { 0xb3897bdd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c697be1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e97be5, 84 },
    { 0, 0 },
    { 0xae597be7, 95 },
    { 0x16d97be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21297bf0, 61 },
    { 0, 0 },
    { 0x80097bf2, 23 },
    { 0x31097bf3, 40 },
    { 0xf7c97bf3, 61 },
    { 0x38797bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f97bf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0297bfc, 90 },
    { 0x19b97bfc, 33 },
    { 0, 0 },
    { 0x5be97bff, 80 },
    { 0, 0 },
    { 0x53097c01, 33 },
    { 0, 0 },
    { 0xd6297c03, 90 },
    { 0x15a97c03, 33 },
    { 0, 0 },
    { 0xc0197c06, 88 },
    { 0xb3497c07, 98 },
    { 0, 0 },
    { 0xd2e97c09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c97c0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9297c0f, 52 },
    { 0x11e97c10, 90 },
    { 0xc7397c11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d397c18, 23 },
    { 0xb2897c18, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f997c1f, 76 },
    { 0, 0 },
    { 0x89397c21, 61 },
    { 0, 0 },
    { 0x2d097c23, 43 },
    { 0, 0 },
    { 0xc4c97c25, 33 },
    { 0x88997c25, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xac897c29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c97c2f, 61 },
    { 0, 0 },
    { 0xed997c31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de97c3d, 100 },
    { 0, 0 },
    { 0xf0597c3f, 33 },
    { 0, 0 },
    { 0xe3597c41, 33 },
    { 0xd5497c42, 98 },
    { 0x6b797c43, 51 },
    { 0x43797c44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f97c4a, 90 },
    { 0xb6b97c4a, 90 },
    { 0x28b97c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f97c50, 90 },
    { 0xd1697c51, 23 },
    { 0x1fb97c52, 90 },
    { 0x36c97c53, 90 },
    { 0x56a97c53, 94 },
    { 0x87b97c55, 78 },
    { 0x44497c54, 90 },
    { 0x89f97c55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35497c5a, 33 },
    { 0x8b297c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0897c61, 90 },
    { 0x6f197c61, 98 },
    { 0xb6297c63, 23 },
    { 0xbb197c64, 90 },
    { 0x14b97c62, 95 },
    { 0xa3c97c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4497c6b, 46 },
    { 0x22f97c6c, 87 },
    { 0x1dc97c6d, 93 },
    { 0x7be97c6d, 100 },
    { 0xa1a97c6f, 32 },
    { 0x26c97c70, 33 },
    { 0xe0e97c71, 33 },
    { 0x9a497c6d, 95 },
    { 0xcfd97c6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde197c7b, 46 },
    { 0, 0 },
    { 0xbc397c7d, 98 },
    { 0x68497c7e, 61 },
    { 0xfbf97c7f, 33 },
    { 0x43e97c7f, 23 },
    { 0x8db97c81, 23 },
    { 0xf097c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97c86, 67 },
    { 0, 0 },
    { 0x82b97c88, 55 },
    { 0x4fb97c88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x47897c8c, 98 },
    { 0, 0 },
    { 0x7d097c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64397c91, 98 },
    { 0, 0 },
    { 0x24897c93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18597c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a997c99, 61 },
    { 0x52197c9a, 33 },
    { 0x55697c9b, 91 },
    { 0xf7897c9c, 61 },
    { 0x69397c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1197ca0, 99 },
    { 0, 0 },
    { 0x71797ca2, 39 },
    { 0, 0 },
    { 0x12297ca4, 98 },
    { 0, 0 },
    { 0x75297ca6, 90 },
    { 0xed697ca7, 98 },
    { 0x3c097ca6, 46 },
    { 0, 0 },
    { 0xfb297caa, 90 },
    { 0xa4297caa, 90 },
    { 0x2c97cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37397cb0, 61 },
    { 0xe8d97cb0, 93 },
    { 0x59797cb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67297cbc, 34 },
    { 0xbf597cbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e97cc2, 90 },
    { 0, 0 },
    { 0x58997cc4, 53 },
    { 0x83a97cc5, 100 },
    { 0xacc97cc6, 64 },
    { 0x8797cc7, 91 },
    { 0x4e97cc4, 23 },
    { 0xd2697cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a597ccc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc97ccf, 61 },
    { 0x3d497ccf, 61 },
    { 0xf5c97cd1, 85 },
    { 0xdb497cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d97cd7, 90 },
    { 0xcfb97cd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3997cdb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a697ce5, 75 },
    { 0x8f697ce6, 33 },
    { 0x2b497ce6, 33 },
    { 0xbe397ce7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a97ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c97cf2, 43 },
    { 0xdfc97cf3, 33 },
    { 0xfb797cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7397cfd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef97d02, 33 },
    { 0, 0 },
    { 0xfad97d04, 38 },
    { 0x29197d05, 66 },
    { 0x6197d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf97d0c, 34 },
    { 0, 0 },
    { 0x14397d0e, 61 },
    { 0x7b497d0e, 61 },
    { 0x7ef97d10, 93 },
    { 0x2d697d11, 90 },
    { 0x7be97d12, 91 },
    { 0x57797d13, 33 },
    { 0, 0 },
    { 0x72097d15, 95 },
    { 0x96397d16, 77 },
    { 0x62597d17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6297d1b, 76 },
    { 0x3d797d1b, 23 },
    { 0x8bf97d1d, 90 },
    { 0x7497d1e, 66 },
    { 0x76a97d1e, 90 },
    { 0xdde97d20, 33 },
    { 0xdea97d20, 55 },
    { 0x3997d20, 46 },
    { 0xd2c97d1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f497d27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56297d2b, 23 },
    { 0, 0 },
    { 0x85f97d2d, 61 },
    { 0x43e97d2e, 76 },
    { 0, 0 },
    { 0x85597d30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5997d35, 26 },
    { 0x97697d36, 33 },
    { 0x11e97d35, 76 },
    { 0x5a697d36, 55 },
    { 0, 0 },
    { 0xb8d97d3a, 36 },
    { 0x6d697d3b, 90 },
    { 0xd5997d3a, 98 },
    { 0xf8c97d3b, 43 },
    { 0x39497d3c, 23 },
    { 0xf7c97d3f, 61 },
    { 0x59897d3d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d97d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea097d47, 61 },
    { 0, 0 },
    { 0x55c97d49, 61 },
    { 0xac397d49, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0097d51, 75 },
    { 0xb0897d51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c797d55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69997d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b97d62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c697d65, 100 },
    { 0, 0 },
    { 0xfc097d67, 95 },
    { 0, 0 },
    { 0xdb997d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a97d6f, 17 },
    { 0, 0 },
    { 0x1d297d71, 98 },
    { 0x98097d72, 81 },
    { 0, 0 },
    { 0xfe997d74, 91 },
    { 0xae597d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b97d7a, 98 },
    { 0x2b697d7b, 61 },
    { 0x44697d7c, 76 },
    { 0x4ab97d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab97d81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18897d84, 61 },
    { 0x87c97d85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe97d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc97d8b, 84 },
    { 0x45a97d8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1797d94, 43 },
    { 0x62b97d95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e897d99, 61 },
    { 0x20697d9a, 76 },
    { 0, 0 },
    { 0xdde97d9c, 90 },
    { 0, 0 },
    { 0xf7497d9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4097da3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13997da8, 90 },
    { 0xfde97da8, 43 },
    { 0x7b797daa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b397dae, 23 },
    { 0xe0e97daf, 61 },
    { 0x5ac97db0, 99 },
    { 0, 0 },
    { 0x97497db2, 67 },
    { 0xee697db2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe197db8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60797dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc997dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d97dcf, 43 },
    { 0, 0 },
    { 0x7297dd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b97dd7, 42 },
    { 0x1f097dd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e97ddd, 23 },
    { 0xadc97ddd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e97de6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xee97de9, 66 },
    { 0xcae97dea, 61 },
    { 0x34597deb, 66 },
    { 0x73c97dec, 98 },
    { 0x25a97dea, 76 },
    { 0xa4b97dec, 23 },
    { 0x1ea97def, 95 },
    { 0xd7297df0, 33 },
    { 0xe2097df0, 95 },
    { 0x36f97df2, 33 },
    { 0x2ad97df0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa697df6, 98 },
    { 0x38397df7, 90 },
    { 0x23b97df8, 87 },
    { 0xe4197df9, 64 },
    { 0xdfd97df8, 34 },
    { 0x7ce97df7, 100 },
    { 0, 0 },
    { 0x5f97dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed597e04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b997e07, 95 },
    { 0, 0 },
    { 0x80997e09, 61 },
    { 0, 0 },
    { 0xcee97e0b, 90 },
    { 0x49297e0c, 34 },
    { 0xfe797e0b, 82 },
    { 0xe9997e0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b197e17, 33 },
    { 0xd9e97e18, 23 },
    { 0xf8c97e17, 98 },
    { 0, 0 },
    { 0x70697e1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb97e1f, 76 },
    { 0xfdf97e20, 33 },
    { 0xbb497e21, 76 },
    { 0x3c997e22, 85 },
    { 0, 0 },
    { 0xd7097e24, 33 },
    { 0x45097e25, 98 },
    { 0x61597e25, 61 },
    { 0, 0 },
    { 0x2b497e28, 78 },
    { 0x8c97e29, 90 },
    { 0, 0 },
    { 0xf9f97e2b, 61 },
    { 0, 0 },
    { 0x85497e2d, 93 },
    { 0xc897e2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3997e32, 17 },
    { 0, 0 },
    { 0xd9b97e34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c97e37, 90 },
    { 0, 0 },
    { 0xf8d97e39, 55 },
    { 0x77c97e3a, 23 },
    { 0, 0 },
    { 0x90b97e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80797e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89597e42, 34 },
    { 0xe3a97e43, 33 },
    { 0x34297e44, 61 },
    { 0, 0 },
    { 0x17397e46, 98 },
    { 0x9797e46, 33 },
    { 0xa3f97e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9597e4b, 43 },
    { 0x7e197e4b, 43 },
    { 0x61497e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf497e52, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x87297e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef97e58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4697e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd997e5f, 40 },
    { 0x8697e60, 46 },
    { 0, 0 },
    { 0xccc97e62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f297e6d, 51 },
    { 0xa1997e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d897e71, 55 },
    { 0x86b97e71, 90 },
    { 0, 0 },
    { 0xc2e97e74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f97e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c097e7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f97e87, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc997e8a, 45 },
    { 0, 0 },
    { 0x63397e8c, 84 },
    { 0x4fb97e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66397e92, 34 },
    { 0, 0 },
    { 0x36997e94, 48 },
    { 0, 0 },
    { 0x57f97e96, 95 },
    { 0x3da97e97, 52 },
    { 0, 0 },
    { 0xee397e99, 46 },
    { 0x2997e9a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc97e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef297ea1, 33 },
    { 0, 0 },
    { 0x22d97ea3, 23 },
    { 0xba97ea4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f97ea7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4497eac, 66 },
    { 0x3cf97ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55797eb0, 90 },
    { 0, 0 },
    { 0xca397eb2, 23 },
    { 0, 0 },
    { 0x7d397eb4, 100 },
    { 0xad797eb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe697ebd, 78 },
    { 0x86497ebe, 76 },
    { 0xb4197ebf, 98 },
    { 0x14c97ec0, 100 },
    { 0x46d97ec1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d97ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa97ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43897ecb, 17 },
    { 0xbb097ecb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797ed4, 33 },
    { 0, 0 },
    { 0x1ce97ed6, 78 },
    { 0xe6f97ed6, 23 },
    { 0x36f97ed7, 33 },
    { 0x5b297ed6, 98 },
    { 0xe5097eda, 46 },
    { 0x17897ed7, 95 },
    { 0x3bd97eda, 46 },
    { 0x25e97edd, 34 },
    { 0xaf397ede, 23 },
    { 0x4c897ed7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2197ee3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f97ee6, 46 },
    { 0x1eb97ee7, 43 },
    { 0x1ea97ee8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40997eee, 67 },
    { 0x6a597eee, 93 },
    { 0x4a97ef0, 61 },
    { 0xab197ef1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2697ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18397ef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d97f0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c397f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6197f20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23497f23, 33 },
    { 0xdee97f23, 98 },
    { 0, 0 },
    { 0x10297f26, 61 },
    { 0x4c097f27, 100 },
    { 0x38b97f28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b497f2b, 90 },
    { 0x90797f2b, 46 },
    { 0, 0 },
    { 0x8a997f2e, 46 },
    { 0, 0 },
    { 0x9c497f30, 23 },
    { 0x9a397f31, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93397f36, 45 },
    { 0x44f97f36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd97f3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98397f44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e97f47, 23 },
    { 0xac597f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47697f4c, 85 },
    { 0x49797f4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb597f55, 43 },
    { 0xe3697f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee097f5c, 93 },
    { 0x22c97f5d, 75 },
    { 0xee297f5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf97f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a997f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a397f6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d97f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49097f74, 23 },
    { 0x7e597f74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb197f79, 95 },
    { 0x20d97f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb97f80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9197f87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f197f8c, 90 },
    { 0, 0 },
    { 0xc3797f8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d97f9b, 95 },
    { 0x46297f9c, 34 },
    { 0x76b97f9b, 98 },
    { 0, 0 },
    { 0x15b97f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48097fa2, 55 },
    { 0, 0 },
    { 0x83897fa4, 33 },
    { 0x8f497fa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e97fa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5797fac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4597fb1, 88 },
    { 0x4f697fb1, 61 },
    { 0x77097fb1, 43 },
    { 0, 0 },
    { 0x20697fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14397fba, 76 },
    { 0, 0 },
    { 0xb0997fbc, 100 },
    { 0xfbe97fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b97fc2, 33 },
    { 0, 0 },
    { 0x94f97fc4, 69 },
    { 0, 0 },
    { 0xe0897fc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fc9, 61 },
    { 0x38597fca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e97fce, 95 },
    { 0x8af97fce, 68 },
    { 0x2897fd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x12397fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1897fde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb797fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84497fe8, 75 },
    { 0, 0 },
    { 0xe8397fea, 33 },
    { 0x79e97fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d97ff4, 33 },
    { 0x8af97ff5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x99797ff8, 95 },
    { 0, 0 },
    { 0xfa797ffa, 55 },
    { 0xfe897ffb, 98 },
    { 0xd7497ffb, 100 },
    { 0xa597ffd, 23 },
    { 0xf8397ffd, 33 },
    { 0x1cb97fff, 55 },
    { 0, 0 },
    { 0x29498001, 33 },
    { 0, 0 },
    { 0x3d098003, 61 },
    { 0x4b398004, 76 },
    { 0x48c98005, 85 },
    { 0x18298005, 59 },
    { 0x4598007, 61 },
    { 0xe4f98008, 23 },
    { 0xb2d98008, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41198010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee98013, 88 },
    { 0x72998014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d298018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd39801e, 33 },
    { 0xccc9801f, 62 },
    { 0, 0 },
    { 0x72498021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd98029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b9802e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd249803c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa19803f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45998044, 90 },
    { 0x5c98045, 90 },
    { 0xdd698044, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29804d, 66 },
    { 0, 0 },
    { 0xa439804f, 77 },
    { 0, 0 },
    { 0xae798051, 33 },
    { 0, 0 },
    { 0x77f98053, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab798056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9805a, 61 },
    { 0, 0 },
    { 0x8359805c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39805f, 61 },
    { 0xdde9805f, 68 },
    { 0xb7098061, 34 },
    { 0xc1498060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5798065, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9806b, 23 },
    { 0, 0 },
    { 0xa99806d, 98 },
    { 0, 0 },
    { 0xb3b9806f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73298074, 78 },
    { 0xe0298074, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d9807b, 36 },
    { 0, 0 },
    { 0xe9c9807d, 95 },
    { 0xd9e9807e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66898086, 93 },
    { 0x2c98087, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24998091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8798094, 33 },
    { 0, 0 },
    { 0xd6398096, 98 },
    { 0x8ed98096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb99809a, 61 },
    { 0x64e9809a, 98 },
    { 0x3af9809c, 100 },
    { 0x6bb9809b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2980a0, 61 },
    { 0x909980a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49980a4, 95 },
    { 0xdd7980a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13980a8, 90 },
    { 0xac0980a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d980ad, 93 },
    { 0, 0 },
    { 0xf8f980af, 55 },
    { 0, 0 },
    { 0x555980b1, 93 },
    { 0x7cc980b1, 93 },
    { 0xfb9980b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7980ba, 83 },
    { 0, 0 },
    { 0x24f980bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83980c5, 91 },
    { 0, 0 },
    { 0xe6d980c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea980d1, 61 },
    { 0xef8980d2, 87 },
    { 0x1d4980d1, 76 },
    { 0, 0 },
    { 0x2ba980d5, 90 },
    { 0x23f980d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8980db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24980e0, 34 },
    { 0xe63980e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e980e4, 95 },
    { 0, 0 },
    { 0x31d980e6, 33 },
    { 0, 0 },
    { 0x4a1980e8, 33 },
    { 0xb46980e9, 66 },
    { 0xe21980ea, 23 },
    { 0xad7980eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4980ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0980fa, 98 },
    { 0, 0 },
    { 0xdce980fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef398103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a698109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19810c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c98110, 61 },
    { 0, 0 },
    { 0x35f98112, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0098118, 43 },
    { 0xb8c98118, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e9811f, 100 },
    { 0, 0 },
    { 0xe6198121, 34 },
    { 0x7f698122, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d698126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e59812a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079812d, 76 },
    { 0, 0 },
    { 0xe979812f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a98135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4498139, 98 },
    { 0x9c89813a, 84 },
    { 0x79498139, 61 },
    { 0x35c98139, 34 },
    { 0x87c9813d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498143, 55 },
    { 0, 0 },
    { 0x5f98145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b9814d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9098152, 33 },
    { 0xed098153, 46 },
    { 0x28e98153, 87 },
    { 0x1cf98155, 98 },
    { 0, 0 },
    { 0xf6298157, 17 },
    { 0, 0 },
    { 0xd3f98159, 90 },
    { 0, 0 },
    { 0x2549815b, 84 },
    { 0x8a29815c, 98 },
    { 0xc959815c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4798161, 38 },
    { 0x2fc98162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74198166, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46498170, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c98173, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e298177, 61 },
    { 0, 0 },
    { 0x71898179, 90 },
    { 0x39c9817a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb9817f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8698185, 33 },
    { 0x68b98185, 58 },
    { 0x16098187, 75 },
    { 0xc9f98186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae59818b, 90 },
    { 0x6bc9818b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2498190, 98 },
    { 0x18a98191, 91 },
    { 0x4b498191, 90 },
    { 0x90b98190, 84 },
    { 0x19898194, 61 },
    { 0x88098193, 43 },
    { 0x82198196, 98 },
    { 0, 0 },
    { 0xa7798198, 90 },
    { 0xde398198, 88 },
    { 0x5c29819a, 33 },
    { 0xdd39819b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3981a2, 67 },
    { 0xbbf981a3, 95 },
    { 0, 0 },
    { 0x43a981a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5981a8, 66 },
    { 0x4b2981a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac981ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b981b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574981bb, 55 },
    { 0x8a6981bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2981bf, 87 },
    { 0xd28981c0, 78 },
    { 0xdde981c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c981c5, 34 },
    { 0, 0 },
    { 0xf27981c7, 93 },
    { 0, 0 },
    { 0x913981c9, 84 },
    { 0xfb6981ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9981d1, 98 },
    { 0x431981d2, 33 },
    { 0x43d981d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f981d6, 61 },
    { 0x932981d7, 90 },
    { 0x175981d7, 90 },
    { 0x8ed981d8, 95 },
    { 0, 0 },
    { 0xb0c981db, 98 },
    { 0x4c981db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e981e1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62981e4, 17 },
    { 0x12c981e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f981e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d981ec, 55 },
    { 0, 0 },
    { 0x92c981ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3981f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19981f6, 23 },
    { 0xfba981f7, 66 },
    { 0xa61981f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x215981fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a98201, 90 },
    { 0x4d98202, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d298205, 75 },
    { 0, 0 },
    { 0xca898207, 55 },
    { 0, 0 },
    { 0xa0698209, 23 },
    { 0, 0 },
    { 0xa909820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1919820f, 61 },
    { 0, 0 },
    { 0x27798211, 66 },
    { 0xd6998212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27598216, 33 },
    { 0xea698216, 55 },
    { 0, 0 },
    { 0xe5a98219, 90 },
    { 0x6589821a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9821e, 90 },
    { 0, 0 },
    { 0xe1a98220, 55 },
    { 0x7aa98221, 61 },
    { 0xd9d98222, 76 },
    { 0x55198222, 95 },
    { 0x3698224, 76 },
    { 0x7f998221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98229, 61 },
    { 0x4a99822a, 17 },
    { 0x3059822b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15298232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3398236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6259823a, 99 },
    { 0x9389823a, 90 },
    { 0, 0 },
    { 0x1689823d, 93 },
    { 0x1009823d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84498246, 98 },
    { 0x4aa98247, 33 },
    { 0x68098246, 33 },
    { 0x7ca98249, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5169824c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c98251, 51 },
    { 0, 0 },
    { 0x95798253, 76 },
    { 0, 0 },
    { 0x7e198255, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd998258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa349825e, 34 },
    { 0, 0 },
    { 0x73f98260, 90 },
    { 0xb2898261, 99 },
    { 0x83298262, 91 },
    { 0, 0 },
    { 0x89c98264, 76 },
    { 0x14c98264, 34 },
    { 0xd2398266, 10 },
    { 0x1bf98265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c59826c, 95 },
    { 0x6999826d, 80 },
    { 0x9449826e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab98274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9827a, 17 },
    { 0xf1b9827b, 88 },
    { 0x54b9827b, 46 },
    { 0x2fa9827d, 33 },
    { 0xc799827e, 40 },
    { 0xbe49827e, 55 },
    { 0x56698280, 100 },
    { 0x8b9827f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd98288, 34 },
    { 0xb2698288, 90 },
    { 0x1669828a, 76 },
    { 0x5e798288, 66 },
    { 0x7b49828b, 33 },
    { 0, 0 },
    { 0xfb49828e, 55 },
    { 0x7af9828f, 80 },
    { 0xa5598290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb598296, 99 },
    { 0x98b98297, 91 },
    { 0x34298298, 78 },
    { 0xc0d98299, 90 },
    { 0x9bf98296, 90 },
    { 0, 0 },
    { 0x8dd9829c, 98 },
    { 0xa49829d, 90 },
    { 0xa19829e, 78 },
    { 0xb479829e, 55 },
    { 0, 0 },
    { 0x402982a1, 23 },
    { 0x2a2982a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2982ab, 90 },
    { 0, 0 },
    { 0xec2982ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659982b2, 90 },
    { 0xbaa982b3, 61 },
    { 0xa87982b3, 61 },
    { 0x248982b3, 23 },
    { 0xc12982b2, 93 },
    { 0x7b8982b2, 98 },
    { 0x7a982b8, 95 },
    { 0, 0 },
    { 0xbc7982ba, 33 },
    { 0xa66982ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd982be, 90 },
    { 0xa46982bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce982c7, 90 },
    { 0x27a982c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x925982cb, 23 },
    { 0, 0 },
    { 0xaf8982cd, 23 },
    { 0xd60982ce, 81 },
    { 0xf9c982cd, 34 },
    { 0xcb6982ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267982d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100982da, 23 },
    { 0, 0 },
    { 0x934982dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87982df, 98 },
    { 0x2f7982e0, 78 },
    { 0, 0 },
    { 0xadd982e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681982e6, 95 },
    { 0xab8982e7, 23 },
    { 0x87f982e7, 33 },
    { 0x4d3982e9, 34 },
    { 0, 0 },
    { 0x2b0982eb, 84 },
    { 0, 0 },
    { 0xe17982ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4982f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2982f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a982fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd982ff, 61 },
    { 0xa5298300, 55 },
    { 0, 0 },
    { 0x3e198302, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca98305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5939830a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429830f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b898312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e98315, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f9831d, 23 },
    { 0, 0 },
    { 0x4609831f, 98 },
    { 0x89b9831f, 33 },
    { 0x8f29831f, 95 },
    { 0xa4198322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19698326, 23 },
    { 0x8bd98327, 90 },
    { 0x38f98326, 91 },
    { 0xe8a98328, 90 },
    { 0, 0 },
    { 0x87b9832b, 61 },
    { 0xbf19832c, 33 },
    { 0xa859832b, 33 },
    { 0x4499832e, 19 },
    { 0, 0 },
    { 0x4d098330, 23 },
    { 0, 0 },
    { 0x2de98332, 55 },
    { 0, 0 },
    { 0x5a498334, 98 },
    { 0, 0 },
    { 0xe7198336, 65 },
    { 0x2bd98337, 23 },
    { 0x91d98337, 46 },
    { 0xf7798339, 90 },
    { 0xd7a9833a, 90 },
    { 0xb799833a, 43 },
    { 0xe5c9833c, 98 },
    { 0xd569833d, 77 },
    { 0xcf79833e, 69 },
    { 0, 0 },
    { 0xf5b98340, 61 },
    { 0x68c98341, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e98344, 40 },
    { 0xa3e98345, 98 },
    { 0, 0 },
    { 0x10e98347, 33 },
    { 0x3398347, 95 },
    { 0x9698349, 85 },
    { 0x9c398347, 98 },
    { 0x99834b, 52 },
    { 0xe69834c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x109834f, 95 },
    { 0x1709834f, 55 },
    { 0xcaa98350, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77498354, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b98358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc349835d, 55 },
    { 0xe7d9835e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc798361, 55 },
    { 0xaaa98361, 67 },
    { 0x5e798363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6279836a, 76 },
    { 0x3f89836b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49836e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88198374, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9837e, 90 },
    { 0x6f09837e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf498382, 33 },
    { 0x4ec98383, 23 },
    { 0, 0 },
    { 0x1f098385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49598388, 95 },
    { 0xc98389, 61 },
    { 0, 0 },
    { 0x3019838b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b9838e, 90 },
    { 0x3559838e, 95 },
    { 0x67098390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb198394, 77 },
    { 0, 0 },
    { 0xc0198396, 33 },
    { 0x5e498397, 52 },
    { 0xd4a98397, 85 },
    { 0xd5898399, 95 },
    { 0xbfe98398, 33 },
    { 0x91998397, 98 },
    { 0, 0 },
    { 0x7389839d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e983a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f983a4, 76 },
    { 0, 0 },
    { 0x3a1983a6, 98 },
    { 0x825983a7, 85 },
    { 0xb9a983a8, 61 },
    { 0x47f983a9, 55 },
    { 0xf3b983aa, 33 },
    { 0x811983a6, 61 },
    { 0xcad983aa, 90 },
    { 0x585983ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x545983b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a983b3, 76 },
    { 0x4ab983b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea983be, 98 },
    { 0, 0 },
    { 0xd58983c0, 95 },
    { 0x440983c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53983c4, 61 },
    { 0, 0 },
    { 0xc57983c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688983ca, 23 },
    { 0xe1a983ca, 75 },
    { 0xddc983cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e983cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0983d2, 43 },
    { 0, 0 },
    { 0x2e7983d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2983db, 90 },
    { 0, 0 },
    { 0x66c983dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82983e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8983e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0983ee, 33 },
    { 0, 0 },
    { 0x1a2983f0, 95 },
    { 0x7ab983f1, 60 },
    { 0xbb1983f2, 46 },
    { 0xe56983f1, 61 },
    { 0, 0 },
    { 0x862983f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a983fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x13198401, 90 },
    { 0x28298401, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e98407, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d9840a, 87 },
    { 0, 0 },
    { 0xe2e9840c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb98410, 67 },
    { 0x44698411, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe598415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30598418, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f898420, 90 },
    { 0x4c698421, 55 },
    { 0, 0 },
    { 0xa4e98423, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26298427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde79842c, 95 },
    { 0, 0 },
    { 0x4d39842e, 90 },
    { 0x6a69842f, 87 },
    { 0xa429842f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e89843c, 61 },
    { 0xa879843d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca98448, 34 },
    { 0xf7098448, 52 },
    { 0x9c9844a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81898460, 95 },
    { 0, 0 },
    { 0x49f98462, 84 },
    { 0x84a98463, 95 },
    { 0x9d898464, 33 },
    { 0x34198463, 90 },
    { 0x1de98466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39846a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d98471, 17 },
    { 0xc9398472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9698479, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8829847e, 98 },
    { 0xe219847e, 61 },
    { 0x698480, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99098484, 90 },
    { 0, 0 },
    { 0x8dc98486, 61 },
    { 0xe4398486, 23 },
    { 0xc8998486, 34 },
    { 0x41298488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9209848d, 93 },
    { 0xb859848e, 33 },
    { 0, 0 },
    { 0x9ff98490, 33 },
    { 0xd9c98491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14398498, 33 },
    { 0xa4d98498, 90 },
    { 0xac898498, 43 },
    { 0xce39849b, 90 },
    { 0xc4898499, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8984a0, 61 },
    { 0, 0 },
    { 0x8d3984a2, 98 },
    { 0x764984a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1984a7, 49 },
    { 0x845984a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb984b3, 98 },
    { 0x17a984b4, 65 },
    { 0x36f984b5, 33 },
    { 0x7a2984b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6984b9, 65 },
    { 0xf96984ba, 46 },
    { 0xfbc984bb, 61 },
    { 0x92b984bc, 46 },
    { 0x7d7984bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7984c1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d984c5, 55 },
    { 0, 0 },
    { 0x8c9984c7, 43 },
    { 0, 0 },
    { 0xac6984c9, 33 },
    { 0x9c1984c9, 61 },
    { 0, 0 },
    { 0x45984cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f984d0, 90 },
    { 0xec984d1, 61 },
    { 0, 0 },
    { 0x23a984d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x581984d6, 36 },
    { 0x279984d7, 61 },
    { 0xeba984d6, 66 },
    { 0x3b1984d9, 33 },
    { 0x3a5984da, 76 },
    { 0x692984db, 33 },
    { 0x50c984dc, 61 },
    { 0x48b984dc, 98 },
    { 0, 0 },
    { 0xaea984df, 98 },
    { 0x457984e0, 91 },
    { 0x2a6984e1, 61 },
    { 0x6dd984e2, 98 },
    { 0xf45984e3, 23 },
    { 0x9ac984df, 55 },
    { 0x72e984e5, 43 },
    { 0xd62984e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f984e9, 66 },
    { 0, 0 },
    { 0x818984eb, 55 },
    { 0xbea984ec, 90 },
    { 0, 0 },
    { 0x51a984ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53984f1, 90 },
    { 0, 0 },
    { 0x62b984f3, 95 },
    { 0x477984f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x777984f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d984fa, 33 },
    { 0xeab984fb, 90 },
    { 0x364984fc, 61 },
    { 0xe39984fd, 61 },
    { 0, 0 },
    { 0x588984ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b98508, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe569850c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd079850f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf98512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d798516, 95 },
    { 0, 0 },
    { 0x73598518, 90 },
    { 0, 0 },
    { 0xd319851a, 23 },
    { 0x2cb9851a, 87 },
    { 0x6a79851c, 46 },
    { 0, 0 },
    { 0x6299851e, 61 },
    { 0, 0 },
    { 0xd2398520, 33 },
    { 0x98c98521, 23 },
    { 0xb2498520, 68 },
    { 0, 0 },
    { 0x99698524, 76 },
    { 0xc6a98525, 61 },
    { 0, 0 },
    { 0x5ea98527, 46 },
    { 0xec598528, 43 },
    { 0xbc698529, 93 },
    { 0xe5398529, 98 },
    { 0xca09852b, 34 },
    { 0xfbe98529, 55 },
    { 0xec39852d, 34 },
    { 0x49852d, 61 },
    { 0x7ea9852b, 98 },
    { 0x8ae98530, 33 },
    { 0, 0 },
    { 0xb5398532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4998535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf98539, 76 },
    { 0xcbd98539, 61 },
    { 0xa6e9853b, 34 },
    { 0x13a9853a, 100 },
    { 0, 0 },
    { 0x61a9853e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c98543, 23 },
    { 0x1b698543, 95 },
    { 0x79898545, 84 },
    { 0x32e98546, 83 },
    { 0, 0 },
    { 0x85398548, 84 },
    { 0, 0 },
    { 0xcb99854a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1298552, 55 },
    { 0xad098553, 24 },
    { 0, 0 },
    { 0x3ac98555, 61 },
    { 0x62398556, 33 },
    { 0x81d98556, 95 },
    { 0xe5c98558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c698568, 33 },
    { 0x81598569, 98 },
    { 0x5649856a, 78 },
    { 0x1159856b, 90 },
    { 0x14698568, 66 },
    { 0xf2b9856d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0698570, 61 },
    { 0, 0 },
    { 0xf4f98572, 67 },
    { 0x97c98573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e98579, 90 },
    { 0x7fe9857a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9857d, 33 },
    { 0xa949857d, 55 },
    { 0x8d99857e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2098582, 90 },
    { 0, 0 },
    { 0x3698584, 76 },
    { 0x44798585, 95 },
    { 0xf7798585, 23 },
    { 0x3d998587, 78 },
    { 0x9ba98588, 55 },
    { 0x93898589, 95 },
    { 0x26198587, 93 },
    { 0, 0 },
    { 0x2049858c, 99 },
    { 0x4c19858d, 98 },
    { 0x9f29858c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c398591, 100 },
    { 0x2cf98592, 97 },
    { 0x62f98593, 61 },
    { 0, 0 },
    { 0xebf98595, 98 },
    { 0x77e98596, 100 },
    { 0, 0 },
    { 0x5f198598, 61 },
    { 0x2c698598, 94 },
    { 0, 0 },
    { 0xd769859b, 78 },
    { 0x1459859c, 90 },
    { 0xfb59859d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0985a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e985a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e985ab, 68 },
    { 0xcc0985ac, 90 },
    { 0xf89985ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5985b0, 55 },
    { 0x975985b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0985b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687985bb, 91 },
    { 0xba985bc, 95 },
    { 0xbc8985bd, 66 },
    { 0xddd985bd, 76 },
    { 0x62d985bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248985c3, 23 },
    { 0xfaf985c3, 23 },
    { 0xbcc985c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b985c9, 91 },
    { 0x322985ca, 76 },
    { 0x457985ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7985d2, 100 },
    { 0x1ea985d3, 98 },
    { 0, 0 },
    { 0xb26985d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78985d8, 23 },
    { 0, 0 },
    { 0x6c9985da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd985de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f985e3, 37 },
    { 0, 0 },
    { 0x4df985e5, 90 },
    { 0, 0 },
    { 0xb08985e7, 46 },
    { 0xeb985e7, 77 },
    { 0xf5a985e8, 23 },
    { 0x6cc985ea, 33 },
    { 0xe19985eb, 76 },
    { 0x4f2985ec, 33 },
    { 0xc0a985e7, 95 },
    { 0x5d4985ee, 84 },
    { 0x268985ec, 34 },
    { 0x64e985f0, 76 },
    { 0xbf4985f1, 90 },
    { 0x90b985f2, 33 },
    { 0x883985e7, 23 },
    { 0xa51985f4, 81 },
    { 0x665985ec, 98 },
    { 0xfe0985ed, 98 },
    { 0x64c985f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95985fa, 23 },
    { 0x24e985fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5598600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb98606, 93 },
    { 0x44d98607, 43 },
    { 0xeb498608, 90 },
    { 0x77698608, 90 },
    { 0xe1a9860a, 100 },
    { 0x46e98606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3199860e, 76 },
    { 0xbaa9860e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99498612, 93 },
    { 0, 0 },
    { 0xf8d98614, 33 },
    { 0x35798614, 33 },
    { 0x30398616, 28 },
    { 0xbdf98617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2759861b, 91 },
    { 0x6b89861b, 91 },
    { 0x60a9861c, 46 },
    { 0, 0 },
    { 0xe479861f, 98 },
    { 0x85f9861f, 98 },
    { 0x6049861f, 90 },
    { 0, 0 },
    { 0x58798623, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d9862a, 98 },
    { 0x4449862b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe179862f, 33 },
    { 0, 0 },
    { 0xd7598631, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c9863f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298645, 76 },
    { 0x5e998646, 26 },
    { 0xf4298647, 55 },
    { 0x4d98648, 95 },
    { 0x17798649, 33 },
    { 0x45498645, 66 },
    { 0x3c09864b, 76 },
    { 0x1a89864c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e598652, 55 },
    { 0x65698653, 91 },
    { 0, 0 },
    { 0x4ee98655, 90 },
    { 0xcb998656, 77 },
    { 0, 0 },
    { 0x1c698658, 33 },
    { 0xedd98658, 95 },
    { 0xdc49865a, 98 },
    { 0xea398658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba9865e, 43 },
    { 0xe809865f, 33 },
    { 0x83b98660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98667, 61 },
    { 0xf2298667, 76 },
    { 0xd8698667, 31 },
    { 0x1359866a, 100 },
    { 0x43b98667, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29866e, 33 },
    { 0, 0 },
    { 0xe2598670, 23 },
    { 0x46698671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c98674, 90 },
    { 0x7be98675, 95 },
    { 0x26a98676, 23 },
    { 0, 0 },
    { 0xf9698678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7889867b, 33 },
    { 0, 0 },
    { 0x7d79867d, 23 },
    { 0x3f29867e, 76 },
    { 0x6e69867d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b398684, 93 },
    { 0x34c98685, 64 },
    { 0, 0 },
    { 0xb3a98687, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d9868b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1198692, 95 },
    { 0xb2398692, 61 },
    { 0xc3f98692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b98697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba9869a, 33 },
    { 0x9489869b, 23 },
    { 0x8289869a, 90 },
    { 0xe89869a, 61 },
    { 0x9289869a, 93 },
    { 0x929869f, 69 },
    { 0, 0 },
    { 0x4c9986a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5986a5, 90 },
    { 0x3ce986a6, 76 },
    { 0xc0d986a5, 66 },
    { 0x5df986a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e986ab, 23 },
    { 0xbaf986ab, 61 },
    { 0, 0 },
    { 0x783986ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7986b1, 90 },
    { 0x320986b2, 90 },
    { 0xcc5986b3, 76 },
    { 0x5d7986b3, 91 },
    { 0x3f1986b5, 33 },
    { 0xf94986b6, 78 },
    { 0x230986b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37986bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7986c2, 78 },
    { 0xc1d986c3, 78 },
    { 0xd97986c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942986cb, 81 },
    { 0xed7986cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd986d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf986d4, 90 },
    { 0xb61986d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc986d9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc986df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20986e3, 90 },
    { 0xc2f986e3, 55 },
    { 0x471986e3, 66 },
    { 0x577986e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e986ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432986ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6986f6, 43 },
    { 0xd34986f7, 23 },
    { 0x516986f7, 75 },
    { 0x1c1986f8, 90 },
    { 0xc45986fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4498701, 33 },
    { 0, 0 },
    { 0x58f98703, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e9870b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9870e, 96 },
    { 0x6ed9870f, 23 },
    { 0, 0 },
    { 0x3eb98711, 80 },
    { 0x9098712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b98717, 66 },
    { 0xb9798717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9871b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf9871e, 33 },
    { 0xd009871f, 17 },
    { 0x20098720, 90 },
    { 0xe6798720, 95 },
    { 0x9a198720, 61 },
    { 0xb9898722, 95 },
    { 0x23898724, 93 },
    { 0xd0098725, 23 },
    { 0xf6c98726, 90 },
    { 0, 0 },
    { 0x96898728, 34 },
    { 0, 0 },
    { 0x86a9872a, 46 },
    { 0xab19872b, 100 },
    { 0x9629872c, 23 },
    { 0xa89872d, 76 },
    { 0xcff9872e, 77 },
    { 0, 0 },
    { 0x1fc98730, 33 },
    { 0x4a198731, 90 },
    { 0x2f298730, 33 },
    { 0x9a198731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87498739, 95 },
    { 0xeb098739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9873d, 91 },
    { 0x7f79873e, 61 },
    { 0xabd9873f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c98743, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b998748, 84 },
    { 0, 0 },
    { 0xb429874a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449874e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45098752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1798755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82998758, 23 },
    { 0, 0 },
    { 0x2b49875a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9875e, 87 },
    { 0xdd9875f, 90 },
    { 0x65d98760, 34 },
    { 0xc0298761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9876a, 76 },
    { 0, 0 },
    { 0xb4a9876c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30498771, 52 },
    { 0x64198772, 23 },
    { 0x60a98772, 90 },
    { 0xcb698774, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c798778, 100 },
    { 0xc898778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39877c, 52 },
    { 0, 0 },
    { 0xec79877e, 33 },
    { 0xd79877f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15498784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec798788, 90 },
    { 0x4e498788, 95 },
    { 0, 0 },
    { 0xad89878b, 65 },
    { 0x67c9878b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49698790, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc598796, 81 },
    { 0x51e98797, 91 },
    { 0x9a298798, 55 },
    { 0xe9d98797, 33 },
    { 0xdfd98799, 46 },
    { 0, 0 },
    { 0xe89879c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a9879f, 69 },
    { 0xb3b9879f, 43 },
    { 0, 0 },
    { 0x751987a2, 33 },
    { 0x7ef987a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x182987a6, 75 },
    { 0x873987a7, 78 },
    { 0, 0 },
    { 0x5bc987a9, 33 },
    { 0x65e987aa, 76 },
    { 0x2de987ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b987b1, 23 },
    { 0x560987b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279987be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28987c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a987c6, 61 },
    { 0x8e8987c7, 90 },
    { 0, 0 },
    { 0x8cc987c9, 33 },
    { 0, 0 },
    { 0x253987cb, 61 },
    { 0x7aa987cb, 61 },
    { 0x904987cd, 98 },
    { 0, 0 },
    { 0x260987cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41987d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f987dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d987df, 17 },
    { 0x211987e0, 61 },
    { 0x7ed987e1, 91 },
    { 0xf36987df, 23 },
    { 0xb4b987e3, 82 },
    { 0, 0 },
    { 0x8c8987e5, 68 },
    { 0x770987e6, 90 },
    { 0xb74987e5, 76 },
    { 0x621987e5, 24 },
    { 0, 0 },
    { 0xcf3987ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d987ef, 23 },
    { 0x842987ef, 90 },
    { 0, 0 },
    { 0xffe987f2, 90 },
    { 0x4b4987f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f987ff, 33 },
    { 0x62898800, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7098809, 61 },
    { 0x2ef9880a, 98 },
    { 0xf709880b, 64 },
    { 0x33c9880c, 66 },
    { 0x9889880c, 34 },
    { 0x2319880b, 55 },
    { 0x7659880f, 76 },
    { 0xc0398810, 98 },
    { 0x2f498811, 46 },
    { 0x38698810, 55 },
    { 0xa9b9880f, 95 },
    { 0xc1d98814, 75 },
    { 0xe9b98815, 90 },
    { 0, 0 },
    { 0x20298817, 90 },
    { 0x5aa98817, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad9881c, 90 },
    { 0x779881d, 61 },
    { 0xd89881d, 66 },
    { 0xa929881f, 64 },
    { 0, 0 },
    { 0xdff98821, 81 },
    { 0x58e98822, 23 },
    { 0, 0 },
    { 0xd3a98824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5499882d, 61 },
    { 0x3509882d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c198831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81598838, 100 },
    { 0x43298839, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19883d, 33 },
    { 0x3629883e, 33 },
    { 0x7d69883f, 33 },
    { 0x7659883d, 61 },
    { 0x94198841, 98 },
    { 0xd1c98840, 23 },
    { 0, 0 },
    { 0x44c98844, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35598847, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9884b, 33 },
    { 0, 0 },
    { 0x5ea9884d, 98 },
    { 0xf389884d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce98853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e98857, 91 },
    { 0xa5998858, 91 },
    { 0x20c98859, 77 },
    { 0x3d498859, 33 },
    { 0xbc29885b, 33 },
    { 0x22198858, 60 },
    { 0x6889885d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24798863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0598870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2998873, 95 },
    { 0x4c598874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f498877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2089887f, 33 },
    { 0xb4d98880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa98883, 76 },
    { 0, 0 },
    { 0x25f98885, 33 },
    { 0x1c098886, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c49888d, 78 },
    { 0x1889888e, 61 },
    { 0xbd69888f, 78 },
    { 0x86b9888f, 33 },
    { 0xaa898890, 98 },
    { 0x9f798890, 23 },
    { 0x53e98893, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b9889b, 46 },
    { 0x3be9889b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c988ae, 82 },
    { 0x17c988af, 38 },
    { 0x5e1988b0, 98 },
    { 0x79c988ae, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7988b6, 52 },
    { 0x3a9988b7, 83 },
    { 0xb9e988b7, 95 },
    { 0xf6c988b6, 55 },
    { 0, 0 },
    { 0x833988bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582988be, 61 },
    { 0x31e988bf, 67 },
    { 0xee6988c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc988c4, 90 },
    { 0xc55988c5, 90 },
    { 0x7d4988c6, 95 },
    { 0x1ee988c7, 56 },
    { 0, 0 },
    { 0x60988c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33988cc, 46 },
    { 0x10c988cc, 99 },
    { 0x471988cd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392988d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8988d7, 77 },
    { 0x891988d8, 55 },
    { 0, 0 },
    { 0xe66988da, 33 },
    { 0x8c6988db, 46 },
    { 0x18c988db, 98 },
    { 0x96988dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba988e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34988ed, 78 },
    { 0x992988ee, 46 },
    { 0x536988ed, 99 },
    { 0x82a988ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5988f4, 33 },
    { 0xd87988f5, 98 },
    { 0x427988f4, 33 },
    { 0, 0 },
    { 0xadf988f8, 77 },
    { 0xcc2988f9, 66 },
    { 0xb3a988fa, 93 },
    { 0x9bc988fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x497988fe, 34 },
    { 0x792988ff, 61 },
    { 0x10798900, 61 },
    { 0xc5c98900, 17 },
    { 0, 0 },
    { 0x13598903, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf098906, 84 },
    { 0x71e98907, 90 },
    { 0xac298908, 23 },
    { 0x66e98907, 55 },
    { 0x9169890a, 91 },
    { 0, 0 },
    { 0xb559890c, 90 },
    { 0xa799890c, 61 },
    { 0xeda9890e, 33 },
    { 0xa249890f, 98 },
    { 0x3149890e, 33 },
    { 0, 0 },
    { 0xe8e98912, 76 },
    { 0, 0 },
    { 0xd1998914, 48 },
    { 0x58d98914, 66 },
    { 0x8ea98916, 90 },
    { 0x8c998917, 39 },
    { 0xc8898918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc39891d, 99 },
    { 0, 0 },
    { 0xb069891f, 61 },
    { 0, 0 },
    { 0xaf698921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5098924, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94198927, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7829892b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1209892e, 61 },
    { 0xd639892f, 53 },
    { 0xbda98930, 33 },
    { 0x4eb9892e, 55 },
    { 0xc5698932, 61 },
    { 0xdab98932, 23 },
    { 0x6898934, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e79893a, 33 },
    { 0xcd79893b, 93 },
    { 0, 0 },
    { 0x15c9893d, 65 },
    { 0xe0c9893d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4098942, 23 },
    { 0, 0 },
    { 0x11198944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a498947, 98 },
    { 0, 0 },
    { 0x98998949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2179894d, 98 },
    { 0x7669894e, 77 },
    { 0x3d29894f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9298952, 95 },
    { 0xcd398953, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x24898956, 23 },
    { 0xc3b98956, 61 },
    { 0xfa98956, 55 },
    { 0x6da98956, 34 },
    { 0x53498958, 76 },
    { 0x49a98958, 28 },
    { 0, 0 },
    { 0xa7f9895d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21498962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f598968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19896d, 52 },
    { 0xe7d9896e, 90 },
    { 0, 0 },
    { 0xc4798970, 23 },
    { 0xf8098971, 98 },
    { 0xc0d98971, 61 },
    { 0x84398971, 43 },
    { 0x87f98974, 76 },
    { 0x11e98974, 90 },
    { 0, 0 },
    { 0x1aa98977, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1859897a, 76 },
    { 0x899897b, 23 },
    { 0xefa9897b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3498980, 75 },
    { 0, 0 },
    { 0x13898982, 98 },
    { 0xe6698983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9998989, 93 },
    { 0, 0 },
    { 0x5809898b, 34 },
    { 0x979898c, 33 },
    { 0xe309898d, 33 },
    { 0x37f9898e, 66 },
    { 0, 0 },
    { 0x78298990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac98998, 95 },
    { 0x8b898998, 93 },
    { 0xca698999, 75 },
    { 0x1ff9899b, 34 },
    { 0, 0 },
    { 0xf179899d, 77 },
    { 0x6df9899e, 90 },
    { 0x3709899f, 61 },
    { 0, 0 },
    { 0x3c5989a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4989ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338989b8, 81 },
    { 0x4af989b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a989bc, 77 },
    { 0xdaf989bd, 33 },
    { 0, 0 },
    { 0x9fe989bf, 33 },
    { 0, 0 },
    { 0x2de989c1, 98 },
    { 0, 0 },
    { 0x5bd989c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e989c6, 90 },
    { 0x16d989c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0989cc, 61 },
    { 0, 0 },
    { 0xe43989ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2989d1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de989dd, 46 },
    { 0x7ea989dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x486989e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c989e9, 46 },
    { 0x655989ea, 43 },
    { 0xdeb989eb, 98 },
    { 0x413989ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a989f1, 56 },
    { 0xd24989f2, 23 },
    { 0, 0 },
    { 0x2dd989f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930989f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db989fc, 23 },
    { 0x497989fd, 98 },
    { 0x619989fe, 75 },
    { 0x3a1989ff, 95 },
    { 0xeb698a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca98a0b, 98 },
    { 0x2bd98a0c, 61 },
    { 0, 0 },
    { 0xbf998a0e, 33 },
    { 0x1ee98a0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39098a13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44598a1a, 23 },
    { 0, 0 },
    { 0x23e98a1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298a21, 55 },
    { 0xfb598a22, 87 },
    { 0, 0 },
    { 0x33a98a24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef98a2b, 90 },
    { 0xea298a2b, 34 },
    { 0x90098a2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8698a30, 23 },
    { 0xd0e98a31, 23 },
    { 0x29598a32, 95 },
    { 0x18798a30, 43 },
    { 0x22f98a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f098a37, 76 },
    { 0x41598a37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0198a41, 93 },
    { 0x2c798a42, 98 },
    { 0x65898a42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3298a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b198a4b, 77 },
    { 0xd298a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2798a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d498a56, 23 },
    { 0x7d198a56, 90 },
    { 0, 0 },
    { 0x7eb98a59, 95 },
    { 0x5d498a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d998a63, 61 },
    { 0xe6398a63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb98a68, 98 },
    { 0, 0 },
    { 0x87698a6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d998a73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1098a76, 77 },
    { 0x15098a77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df98a7c, 33 },
    { 0x96798a7c, 76 },
    { 0x17c98a7c, 95 },
    { 0x87898a7f, 55 },
    { 0x64f98a80, 23 },
    { 0x3d498a7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b798a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca98a8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98a8e, 94 },
    { 0x28b98a8f, 93 },
    { 0xe2e98a8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d298a93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d798a99, 98 },
    { 0, 0 },
    { 0x14c98a9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33298a9f, 95 },
    { 0x8f798a9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c98aa3, 66 },
    { 0xf6498aa4, 90 },
    { 0x54598aa4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe98aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce98aaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43698ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7798ab5, 98 },
    { 0x6198ab6, 85 },
    { 0x88c98ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a898aba, 33 },
    { 0x73698abb, 43 },
    { 0x4a998abc, 66 },
    { 0x81498abc, 61 },
    { 0x65998abe, 17 },
    { 0, 0 },
    { 0xc7e98ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf198ac4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2098ac9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68998acc, 55 },
    { 0x3b898acc, 61 },
    { 0xa1298acc, 95 },
    { 0x77698ace, 55 },
    { 0xdc398ad0, 75 },
    { 0x71c98ad0, 95 },
    { 0, 0 },
    { 0xda798ad3, 53 },
    { 0x56798ad3, 44 },
    { 0xd5398ad3, 95 },
    { 0x75598ad4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x50998ad9, 34 },
    { 0xebe98ada, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d98ade, 33 },
    { 0xd98ade, 23 },
    { 0x4e798ae0, 66 },
    { 0x10498adf, 46 },
    { 0x9cd98ae1, 19 },
    { 0xef998ae2, 95 },
    { 0, 0 },
    { 0x90298ae5, 66 },
    { 0x2a898ae5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2598aea, 40 },
    { 0x17598aea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3598af2, 90 },
    { 0, 0 },
    { 0x7a998af4, 55 },
    { 0, 0 },
    { 0x30398af6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98afa, 23 },
    { 0x89d98afa, 23 },
    { 0x7cf98afb, 61 },
    { 0, 0 },
    { 0x1d498afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76498b01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf98b04, 33 },
    { 0xab298b05, 95 },
    { 0x57b98b06, 90 },
    { 0, 0 },
    { 0x23498b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17398b0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f98b11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4998b17, 23 },
    { 0xd6f98b18, 69 },
    { 0x30798b19, 98 },
    { 0x9b498b1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a498b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4198b2a, 95 },
    { 0xdfe98b2b, 90 },
    { 0x20d98b2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb198b30, 88 },
    { 0xc3a98b31, 98 },
    { 0, 0 },
    { 0xe1b98b33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b298b37, 90 },
    { 0, 0 },
    { 0xe4e98b39, 90 },
    { 0x18e98b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d298b3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac98b42, 43 },
    { 0xb9e98b43, 98 },
    { 0x6bf98b44, 84 },
    { 0x50b98b45, 98 },
    { 0, 0 },
    { 0x8c398b47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8098b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e98b51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d098b54, 98 },
    { 0x11e98b55, 33 },
    { 0xef198b56, 61 },
    { 0x4f298b57, 90 },
    { 0, 0 },
    { 0xc8e98b59, 81 },
    { 0x3c798b5a, 61 },
    { 0, 0 },
    { 0x1a598b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c98b5f, 95 },
    { 0xe98b60, 90 },
    { 0x3998b61, 91 },
    { 0xdf798b62, 17 },
    { 0x4b598b62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b98b66, 98 },
    { 0x6f898b67, 95 },
    { 0x49a98b68, 38 },
    { 0x2c598b66, 95 },
    { 0x1cc98b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80898b72, 76 },
    { 0, 0 },
    { 0xe7998b74, 23 },
    { 0x76798b75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a198b79, 61 },
    { 0, 0 },
    { 0x1ab98b7b, 95 },
    { 0x35d98b7c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a98b7f, 43 },
    { 0, 0 },
    { 0x91198b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85498b84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba98b87, 61 },
    { 0x9da98b87, 88 },
    { 0x38498b88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f598b90, 23 },
    { 0xf0b98b90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e498b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64798b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb98b9f, 56 },
    { 0xfec98b9f, 84 },
    { 0x9de98ba1, 64 },
    { 0, 0 },
    { 0xb1198ba3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae698ba7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75098bad, 76 },
    { 0xbe698bad, 93 },
    { 0, 0 },
    { 0xbdd98bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53598bb4, 56 },
    { 0x1ce98bb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b98bb9, 55 },
    { 0, 0 },
    { 0x70b98bbb, 99 },
    { 0xf3998bbc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4398bc3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0098bc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b98bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb198bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe98be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75298be4, 33 },
    { 0, 0 },
    { 0x30098be6, 23 },
    { 0, 0 },
    { 0xe7f98be8, 98 },
    { 0, 0 },
    { 0xcb198bea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41198bed, 52 },
    { 0x3fb98bee, 91 },
    { 0xeda98bee, 76 },
    { 0x85598bf0, 99 },
    { 0x1d198bf0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb098bf8, 90 },
    { 0xb5298bf9, 95 },
    { 0x20498bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e398c01, 61 },
    { 0, 0 },
    { 0x68498c03, 33 },
    { 0xd5798c04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f98c08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f98c11, 61 },
    { 0x99698c12, 61 },
    { 0x14098c13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98c1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998c1f, 33 },
    { 0, 0 },
    { 0xf2d98c21, 90 },
    { 0xef098c21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae98c26, 90 },
    { 0x3dc98c27, 61 },
    { 0, 0 },
    { 0xefc98c29, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d98c2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf98c30, 23 },
    { 0x25498c30, 33 },
    { 0x5dd98c32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e98c3e, 61 },
    { 0x51d98c3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7198c43, 98 },
    { 0, 0 },
    { 0x5cb98c45, 75 },
    { 0xe0c98c46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c98c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef498c4e, 87 },
    { 0x4cd98c4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58498c52, 33 },
    { 0, 0 },
    { 0xd3f98c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec898c57, 34 },
    { 0x52698c57, 23 },
    { 0x53d98c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66898c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc98c66, 87 },
    { 0x77898c67, 90 },
    { 0x91c98c67, 95 },
    { 0x51798c69, 95 },
    { 0x62198c69, 66 },
    { 0x9698c6a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xec998c6e, 77 },
    { 0, 0 },
    { 0xac598c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e98c77, 98 },
    { 0x74d98c78, 43 },
    { 0, 0 },
    { 0x34798c7a, 90 },
    { 0xa898c7b, 46 },
    { 0, 0 },
    { 0x53598c7d, 61 },
    { 0xcc798c7e, 76 },
    { 0x51998c7f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5798c85, 95 },
    { 0x4eb98c86, 80 },
    { 0x46398c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1498c8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a698c93, 100 },
    { 0xb2698c94, 93 },
    { 0x4cf98c95, 77 },
    { 0x5f898c93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d98c9c, 61 },
    { 0x7f698c9d, 90 },
    { 0xb0e98c9d, 61 },
    { 0, 0 },
    { 0xad098ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5598ca3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1498ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90498caa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea98caf, 33 },
    { 0x49498cb0, 61 },
    { 0x63a98cb0, 40 },
    { 0x1df98caf, 55 },
    { 0x30b98cb3, 98 },
    { 0xd5d98cb4, 87 },
    { 0x9e998cb5, 33 },
    { 0, 0 },
    { 0x10d98cb7, 75 },
    { 0xfb198cb8, 88 },
    { 0xbee98cb9, 76 },
    { 0x5598cba, 90 },
    { 0x22e98cb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc98cbf, 23 },
    { 0x6a398cc0, 43 },
    { 0xb9a98cc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39898cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60198cc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc98cce, 43 },
    { 0, 0 },
    { 0xe0d98cd0, 23 },
    { 0, 0 },
    { 0xd0d98cd2, 33 },
    { 0x8b798cd3, 76 },
    { 0xf3c98cd4, 23 },
    { 0x27b98cd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe598cda, 95 },
    { 0, 0 },
    { 0xa6a98cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29498cdf, 67 },
    { 0xf7e98ce0, 98 },
    { 0x2cf98ce1, 67 },
    { 0x5c298ce2, 62 },
    { 0x83498ce3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ce6, 78 },
    { 0x57598ce7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f98cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb98cf3, 61 },
    { 0x88c98cf4, 81 },
    { 0x69f98cf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8598cfb, 85 },
    { 0x6ac98cfc, 98 },
    { 0x8a098cfb, 90 },
    { 0xce498cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d98d01, 66 },
    { 0, 0 },
    { 0xae498d03, 46 },
    { 0x65498d04, 33 },
    { 0x4e098d04, 76 },
    { 0xcad98d03, 61 },
    { 0xd8298d07, 99 },
    { 0x30c98d07, 77 },
    { 0xd9b98d06, 33 },
    { 0x5d898d0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3998d0e, 98 },
    { 0x7ac98d0f, 55 },
    { 0, 0 },
    { 0xdc398d11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5298d15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x32098d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e98d1f, 75 },
    { 0x15798d20, 23 },
    { 0xd8398d21, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a98d27, 93 },
    { 0x3d398d28, 46 },
    { 0, 0 },
    { 0xfc398d2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b98d2d, 34 },
    { 0x40598d2e, 77 },
    { 0xf7798d2f, 95 },
    { 0x3c298d30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3598d33, 33 },
    { 0xe4c98d33, 98 },
    { 0xee298d35, 98 },
    { 0x64d98d36, 23 },
    { 0, 0 },
    { 0xba798d38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8498d3b, 69 },
    { 0x92398d3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d98d3f, 95 },
    { 0x1e898d40, 99 },
    { 0x22098d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc498d45, 61 },
    { 0xfed98d45, 90 },
    { 0x98a98d47, 91 },
    { 0xa0e98d47, 33 },
    { 0x1df98d47, 93 },
    { 0x91198d49, 85 },
    { 0x50298d48, 95 },
    { 0x99898d46, 75 },
    { 0xaea98d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6898d55, 90 },
    { 0xee798d55, 81 },
    { 0xafd98d55, 33 },
    { 0xedb98d58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa998d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6998d67, 90 },
    { 0x37798d68, 55 },
    { 0xeec98d69, 23 },
    { 0xcd298d68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb898d71, 46 },
    { 0xa8e98d72, 33 },
    { 0x13798d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78398d76, 33 },
    { 0x29b98d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7898d80, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2698d84, 77 },
    { 0, 0 },
    { 0x2ca98d86, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4898d8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e98d8e, 33 },
    { 0x69998d8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd198d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b198d95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca98d99, 100 },
    { 0x4b998d9a, 84 },
    { 0xc5098d99, 23 },
    { 0x72198d9c, 98 },
    { 0xe8798d9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e98da3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38698da9, 90 },
    { 0, 0 },
    { 0xbcc98dab, 23 },
    { 0x61198dac, 55 },
    { 0x5be98dac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e98db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac98dbf, 76 },
    { 0, 0 },
    { 0x91998dc1, 90 },
    { 0xc9798dc1, 90 },
    { 0xc5798dc3, 77 },
    { 0x61598dc4, 33 },
    { 0xc5a98dc4, 55 },
    { 0xcd998dc3, 90 },
    { 0x8398dc2, 95 },
    { 0x16798dc5, 66 },
    { 0xf9498dc2, 55 },
    { 0, 0 },
    { 0xd9498dcb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5398dce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88398dd8, 40 },
    { 0x6ca98dd8, 99 },
    { 0x31098dda, 61 },
    { 0x69d98ddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6498de0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82198de3, 69 },
    { 0, 0 },
    { 0xc8798de5, 76 },
    { 0x77198de6, 90 },
    { 0x7ea98de6, 90 },
    { 0x23d98de5, 98 },
    { 0, 0 },
    { 0x14298dea, 95 },
    { 0, 0 },
    { 0xf1e98dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a698df5, 61 },
    { 0xeb798df6, 87 },
    { 0xc6e98df6, 77 },
    { 0x95498df5, 55 },
    { 0, 0 },
    { 0xd4398dfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd798dfd, 23 },
    { 0xb6698dfe, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed98e09, 95 },
    { 0xd8898e0a, 90 },
    { 0x15598e0b, 65 },
    { 0x2a698e0a, 98 },
    { 0x78b98e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e698e10, 61 },
    { 0xcfe98e10, 69 },
    { 0x18f98e12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8398e17, 90 },
    { 0, 0 },
    { 0x3ad98e19, 98 },
    { 0, 0 },
    { 0x91f98e1b, 61 },
    { 0x1b598e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc98e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84698e27, 55 },
    { 0x29e98e28, 99 },
    { 0xea798e28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f98e2e, 55 },
    { 0xc1d98e2f, 75 },
    { 0xe0e98e30, 75 },
    { 0x2e998e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34598e35, 33 },
    { 0xf3998e35, 98 },
    { 0, 0 },
    { 0x29998e38, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa98e3e, 34 },
    { 0x14f98e3e, 33 },
    { 0x36298e3e, 87 },
    { 0xdb998e41, 68 },
    { 0xb5698e42, 67 },
    { 0x9ba98e3e, 90 },
    { 0, 0 },
    { 0xcff98e45, 90 },
    { 0, 0 },
    { 0x8f598e47, 78 },
    { 0x51498e48, 55 },
    { 0x93598e49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41698e4c, 23 },
    { 0x898e4c, 33 },
    { 0, 0 },
    { 0x28998e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f98e55, 91 },
    { 0, 0 },
    { 0x2af98e57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae198e5b, 95 },
    { 0x68398e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf098e61, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e98e65, 69 },
    { 0, 0 },
    { 0xbbd98e67, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998e6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda598e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d698e73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e98e78, 34 },
    { 0, 0 },
    { 0x27098e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd398e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf298e8a, 78 },
    { 0x11098e8a, 55 },
    { 0xf8098e8c, 90 },
    { 0x8bb98e8b, 33 },
    { 0x48998e8a, 90 },
    { 0x21898e8c, 98 },
    { 0x41f98e8b, 98 },
    { 0, 0 },
    { 0x32a98e92, 61 },
    { 0, 0 },
    { 0x82b98e94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de98e99, 76 },
    { 0xeea98e9a, 90 },
    { 0, 0 },
    { 0xa0998e9c, 61 },
    { 0, 0 },
    { 0xdea98e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43798ea2, 98 },
    { 0xb8198ea3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c98ea6, 33 },
    { 0x9a98ea7, 90 },
    { 0x5fe98ea8, 99 },
    { 0, 0 },
    { 0x10398eaa, 33 },
    { 0x95c98eab, 98 },
    { 0xaa998eab, 98 },
    { 0x42b98ead, 33 },
    { 0, 0 },
    { 0x1ee98eaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39398eb5, 61 },
    { 0x2e598eb5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd098ebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53198ecb, 90 },
    { 0, 0 },
    { 0x70a98ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a398ed5, 90 },
    { 0xb398ed5, 98 },
    { 0xdde98ed7, 55 },
    { 0xef698ed8, 43 },
    { 0, 0 },
    { 0xa9698eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf398ee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47898ee6, 99 },
    { 0, 0 },
    { 0x61698ee8, 88 },
    { 0x6d898ee9, 55 },
    { 0xab198eea, 91 },
    { 0x29898eea, 33 },
    { 0x5f098ee9, 98 },
    { 0xc4098eed, 33 },
    { 0, 0 },
    { 0x4e598eef, 95 },
    { 0xc9e98ef0, 90 },
    { 0x6ad98ef1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f98ef4, 90 },
    { 0xed098ef4, 33 },
    { 0x28c98ef6, 61 },
    { 0x24f98ef7, 93 },
    { 0xba398ef8, 33 },
    { 0xb3a98ef9, 55 },
    { 0x4ef98efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b098f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af98f0b, 95 },
    { 0xa4498f0c, 61 },
    { 0, 0 },
    { 0x85598f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98f11, 90 },
    { 0, 0 },
    { 0x8ff98f13, 23 },
    { 0, 0 },
    { 0xd2298f15, 69 },
    { 0, 0 },
    { 0x4b498f17, 76 },
    { 0xda198f18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c598f1c, 95 },
    { 0, 0 },
    { 0x23b98f1e, 96 },
    { 0, 0 },
    { 0x72298f20, 95 },
    { 0, 0 },
    { 0x1ed98f22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f098f25, 90 },
    { 0, 0 },
    { 0xb5698f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc198f2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8698f2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa98f31, 33 },
    { 0, 0 },
    { 0x81498f33, 76 },
    { 0x43d98f33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d98f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e98f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba698f45, 100 },
    { 0, 0 },
    { 0x3c698f47, 87 },
    { 0x2e098f48, 61 },
    { 0x6cc98f48, 90 },
    { 0xa6098f4a, 95 },
    { 0xbeb98f48, 98 },
    { 0xa8498f4c, 90 },
    { 0, 0 },
    { 0x20f98f4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c98f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5698f63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d998f6b, 61 },
    { 0x35998f6b, 52 },
    { 0xb0898f6b, 98 },
    { 0, 0 },
    { 0xeee98f6f, 69 },
    { 0, 0 },
    { 0x9dc98f71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e98f75, 90 },
    { 0, 0 },
    { 0xeae98f77, 66 },
    { 0xb7498f78, 46 },
    { 0x12698f77, 95 },
    { 0, 0 },
    { 0x46698f7b, 23 },
    { 0x46498f7b, 97 },
    { 0x56a98f7c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9798f80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe498f84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b98f87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af98f8b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b98f8f, 33 },
    { 0x50898f8f, 43 },
    { 0xe8c98f90, 85 },
    { 0xe7498f92, 43 },
    { 0x94898f90, 98 },
    { 0, 0 },
    { 0xec398f95, 34 },
    { 0x50f98f96, 95 },
    { 0, 0 },
    { 0x85498f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea98f9b, 88 },
    { 0, 0 },
    { 0xc3198f9d, 43 },
    { 0, 0 },
    { 0xbda98f9f, 99 },
    { 0, 0 },
    { 0x8cf98fa1, 98 },
    { 0x1e698fa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0598fa6, 98 },
    { 0, 0 },
    { 0x7bd98fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d98fac, 95 },
    { 0x3ed98fad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad98fb3, 90 },
    { 0, 0 },
    { 0xedc98fb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca398fba, 23 },
    { 0xd3f98fbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad98fc2, 90 },
    { 0, 0 },
    { 0x41998fc4, 23 },
    { 0x10f98fc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56898fc8, 61 },
    { 0, 0 },
    { 0x8c598fca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32598fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c98fd1, 90 },
    { 0xf2798fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40398fd5, 90 },
    { 0xc1998fd5, 90 },
    { 0, 0 },
    { 0x55498fd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x13198fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f398fe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e98fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe098feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df98fef, 98 },
    { 0x97598ff0, 95 },
    { 0x9d398ff1, 90 },
    { 0x99a98ff1, 95 },
    { 0x53e98ff3, 34 },
    { 0x6598ff4, 61 },
    { 0, 0 },
    { 0x1c98ff6, 88 },
    { 0x998ff7, 90 },
    { 0x90e98ff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d298ffb, 90 },
    { 0x15a98ffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d199002, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20199006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac99009, 43 },
    { 0x3f79900a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d999011, 81 },
    { 0, 0 },
    { 0x98599013, 90 },
    { 0x62a99014, 78 },
    { 0x7b799014, 95 },
    { 0x22b99015, 95 },
    { 0x5a299013, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49901b, 95 },
    { 0, 0 },
    { 0x2a79901d, 66 },
    { 0xaed9901e, 61 },
    { 0x41f9901f, 61 },
    { 0x95b99020, 67 },
    { 0x80499021, 97 },
    { 0x6f899020, 84 },
    { 0x27b99023, 98 },
    { 0x3c29901e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93799028, 66 },
    { 0xb8d99029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df9902c, 98 },
    { 0, 0 },
    { 0xeff9902e, 23 },
    { 0xc1e9902f, 36 },
    { 0x64499030, 52 },
    { 0x2269902e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1099038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e79903c, 85 },
    { 0x2539903d, 100 },
    { 0, 0 },
    { 0x9479903f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a99043, 98 },
    { 0, 0 },
    { 0xdcc99045, 68 },
    { 0x56799046, 33 },
    { 0x4d999047, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x129904a, 98 },
    { 0xe549904b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a99056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a599064, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e99067, 99 },
    { 0, 0 },
    { 0x66e99069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa9906e, 64 },
    { 0, 0 },
    { 0x8999070, 23 },
    { 0x11699070, 61 },
    { 0x7cf99071, 61 },
    { 0x2f199073, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a899076, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92199079, 98 },
    { 0xe189907a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66999080, 76 },
    { 0xdee99081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d99084, 90 },
    { 0xa5399085, 95 },
    { 0x72d99086, 90 },
    { 0, 0 },
    { 0x21299088, 33 },
    { 0x7899089, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4379908f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6699095, 85 },
    { 0xd9499096, 90 },
    { 0x24799097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109909b, 98 },
    { 0x2519909c, 43 },
    { 0, 0 },
    { 0x79d9909e, 19 },
    { 0xe7b9909f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8990a5, 33 },
    { 0xc63990a6, 95 },
    { 0x808990a7, 98 },
    { 0, 0 },
    { 0x5b3990a9, 90 },
    { 0, 0 },
    { 0x864990ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870990b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0990b5, 74 },
    { 0, 0 },
    { 0xa03990b7, 40 },
    { 0xf8a990b7, 34 },
    { 0x4fe990b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509990c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x324990c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9990c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1990cd, 67 },
    { 0x63d990ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6990d5, 95 },
    { 0x60f990d6, 23 },
    { 0x53b990d6, 99 },
    { 0, 0 },
    { 0xd3c990d9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0990dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812990e2, 34 },
    { 0x4bf990e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a990e8, 34 },
    { 0x746990e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede990ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f990f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc990f5, 90 },
    { 0x70990f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa990fe, 33 },
    { 0xa4b990ff, 98 },
    { 0x26599100, 33 },
    { 0xe1099101, 90 },
    { 0, 0 },
    { 0xf3e99103, 90 },
    { 0, 0 },
    { 0xd3699105, 95 },
    { 0x80999106, 61 },
    { 0xb6699107, 34 },
    { 0x86a99108, 55 },
    { 0xe6c99108, 33 },
    { 0x729910a, 33 },
    { 0xc109910b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5459910f, 33 },
    { 0xa8e99110, 33 },
    { 0x88299110, 43 },
    { 0x1899111, 61 },
    { 0x24f99113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f99117, 90 },
    { 0xd7f99117, 33 },
    { 0, 0 },
    { 0x56f9911a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd349911d, 33 },
    { 0, 0 },
    { 0x56e9911f, 98 },
    { 0, 0 },
    { 0xd4699121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d099125, 98 },
    { 0x9599126, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9912c, 55 },
    { 0, 0 },
    { 0x26d9912e, 95 },
    { 0, 0 },
    { 0x9f499130, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a999133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a399138, 61 },
    { 0x66199138, 61 },
    { 0x6b79913a, 55 },
    { 0x9eb9913b, 23 },
    { 0x94499138, 98 },
    { 0xffc9913d, 76 },
    { 0xd799913d, 33 },
    { 0x7839913d, 46 },
    { 0x15099140, 90 },
    { 0xe0599140, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c799149, 90 },
    { 0xd829914a, 26 },
    { 0x7589914a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f99150, 33 },
    { 0x1a899151, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c99156, 61 },
    { 0xb3c99156, 23 },
    { 0x2c299158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa629915c, 66 },
    { 0, 0 },
    { 0xa979915e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7899163, 61 },
    { 0, 0 },
    { 0xa1d99165, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f099169, 85 },
    { 0x51999169, 76 },
    { 0, 0 },
    { 0x91f9916c, 61 },
    { 0x49a9916d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8199170, 33 },
    { 0x4de99171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42299174, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f99177, 33 },
    { 0x8f99177, 84 },
    { 0x55699179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9917e, 46 },
    { 0x5139917f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c99182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec599185, 61 },
    { 0x57599186, 76 },
    { 0x7ce99187, 55 },
    { 0, 0 },
    { 0x6a099189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2899190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a99197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1991a3, 41 },
    { 0, 0 },
    { 0x9ac991a5, 67 },
    { 0, 0 },
    { 0x92a991a7, 95 },
    { 0, 0 },
    { 0xaba991a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74991af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e991b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef991ba, 59 },
    { 0x5cb991ba, 91 },
    { 0x24d991bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2991c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67991c3, 98 },
    { 0x7ec991c4, 98 },
    { 0xd86991c5, 34 },
    { 0x974991c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c991ca, 90 },
    { 0x7e991cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6991ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615991d2, 76 },
    { 0x42b991d3, 100 },
    { 0x2dd991d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4991d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46991db, 90 },
    { 0, 0 },
    { 0x603991dd, 76 },
    { 0x4f2991de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x222991e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20991e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e991ec, 61 },
    { 0x75991ed, 75 },
    { 0x86f991ee, 55 },
    { 0, 0 },
    { 0x2ff991f0, 90 },
    { 0x8e5991f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a991f7, 68 },
    { 0xa7d991f8, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c99200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e99203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed999206, 23 },
    { 0x12b99207, 33 },
    { 0xc7299208, 33 },
    { 0xa499207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5299212, 23 },
    { 0x9ae99213, 15 },
    { 0x5b99212, 98 },
    { 0xd7699215, 43 },
    { 0x49399216, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b899219, 87 },
    { 0x2909921a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26199220, 76 },
    { 0, 0 },
    { 0xe2599222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b9922b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7299922f, 61 },
    { 0x7ce99230, 98 },
    { 0xeb99231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2499234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559923d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e999242, 90 },
    { 0x96599243, 56 },
    { 0x45399243, 61 },
    { 0x74299245, 23 },
    { 0x55a99244, 90 },
    { 0, 0 },
    { 0xeff99248, 94 },
    { 0xc3c99249, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b9924c, 38 },
    { 0x2a09924d, 61 },
    { 0x8289924d, 95 },
    { 0xb269924c, 75 },
    { 0x9e199250, 95 },
    { 0x69499250, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b999254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98399258, 65 },
    { 0x68b99258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7599260, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f499263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c99266, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c9926a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9926e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba99277, 46 },
    { 0xe6399278, 61 },
    { 0x6d899279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1179927e, 87 },
    { 0xe039927e, 98 },
    { 0x9f299280, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0299283, 61 },
    { 0x59e99284, 95 },
    { 0x13799283, 98 },
    { 0xf0b99286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429928b, 95 },
    { 0, 0 },
    { 0xf4b9928d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0399291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0299297, 90 },
    { 0xa3e99297, 23 },
    { 0, 0 },
    { 0xbf39929a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7319929f, 33 },
    { 0x9929929f, 33 },
    { 0xa49992a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7992af, 40 },
    { 0xd0f992b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2992b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918992ba, 95 },
    { 0, 0 },
    { 0x11e992bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8992c0, 33 },
    { 0x386992c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80992c7, 98 },
    { 0, 0 },
    { 0xa01992c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a992cd, 33 },
    { 0xe40992ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7992d2, 88 },
    { 0x494992d3, 66 },
    { 0x2fd992d4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d992da, 95 },
    { 0x741992da, 55 },
    { 0x4f6992da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0992df, 95 },
    { 0x727992e0, 23 },
    { 0xdd8992e1, 61 },
    { 0xd0992e1, 33 },
    { 0x15f992e3, 43 },
    { 0, 0 },
    { 0xab0992e5, 34 },
    { 0xe9992e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214992ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x497992ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873992fe, 61 },
    { 0x587992ff, 46 },
    { 0x585992fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40499305, 46 },
    { 0x7dd99305, 95 },
    { 0x9099307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1779930d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa699313, 78 },
    { 0x37999313, 33 },
    { 0xdb499313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8189931a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649931d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5699322, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc99325, 90 },
    { 0x52299326, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f79932b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b19932f, 40 },
    { 0, 0 },
    { 0x31a99331, 61 },
    { 0x23199332, 93 },
    { 0x2a799333, 85 },
    { 0, 0 },
    { 0x8499335, 98 },
    { 0xc9a99335, 100 },
    { 0xccd99335, 33 },
    { 0xcc799338, 61 },
    { 0x89f99339, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b99344, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f99347, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759934a, 84 },
    { 0x919934b, 98 },
    { 0, 0 },
    { 0xa2c9934d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7899350, 61 },
    { 0xdba99350, 95 },
    { 0, 0 },
    { 0xfc399353, 33 },
    { 0, 0 },
    { 0xecb99355, 90 },
    { 0xa9799355, 76 },
    { 0xef199357, 76 },
    { 0xba999355, 76 },
    { 0xc9e99356, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b9935c, 52 },
    { 0x42c9935d, 61 },
    { 0x5469935d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b99366, 93 },
    { 0xa3c99367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489936c, 61 },
    { 0, 0 },
    { 0xf29936e, 98 },
    { 0, 0 },
    { 0xabf99370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae99374, 46 },
    { 0x6a299375, 55 },
    { 0, 0 },
    { 0x54799377, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49937c, 83 },
    { 0, 0 },
    { 0xa089937e, 43 },
    { 0x3e29937f, 95 },
    { 0xcf999380, 99 },
    { 0, 0 },
    { 0xc5b99382, 90 },
    { 0x43699383, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d099386, 34 },
    { 0x20399387, 90 },
    { 0x68299388, 98 },
    { 0x25699389, 61 },
    { 0x5ce9938a, 98 },
    { 0, 0 },
    { 0xb6c9938c, 34 },
    { 0xf3d9938c, 61 },
    { 0, 0 },
    { 0x6979938f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f99393, 93 },
    { 0xbbc99394, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca99399, 46 },
    { 0x9e79939a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9939e, 85 },
    { 0x2859939f, 61 },
    { 0xfea9939f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266993a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406993ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18993af, 23 },
    { 0xee6993b0, 66 },
    { 0, 0 },
    { 0xa84993b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e993b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c993c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa993c6, 91 },
    { 0x3ef993c6, 61 },
    { 0, 0 },
    { 0xf8a993c9, 90 },
    { 0x207993c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4993d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d993d8, 99 },
    { 0x75c993d9, 93 },
    { 0x189993da, 43 },
    { 0, 0 },
    { 0xe91993dc, 33 },
    { 0x4dc993dd, 98 },
    { 0x38f993de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f993e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3993eb, 91 },
    { 0xf32993ec, 61 },
    { 0x19993ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9993f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f993f4, 34 },
    { 0x76e993f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332993f9, 91 },
    { 0, 0 },
    { 0xd14993fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b993ff, 90 },
    { 0x41499400, 78 },
    { 0x49f99401, 33 },
    { 0x28999402, 98 },
    { 0xe5499402, 91 },
    { 0, 0 },
    { 0x71b99405, 99 },
    { 0x80a99406, 66 },
    { 0xdb599405, 33 },
    { 0xf9099408, 23 },
    { 0x89499407, 93 },
    { 0xde89940a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77999413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1899416, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0399419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09941d, 52 },
    { 0x6ca9941e, 43 },
    { 0x2a09941e, 34 },
    { 0, 0 },
    { 0x91799421, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09942e, 33 },
    { 0xa6c9942f, 46 },
    { 0xa149942e, 43 },
    { 0xa4699431, 84 },
    { 0xea399431, 43 },
    { 0, 0 },
    { 0xb0399434, 99 },
    { 0xc7f99434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29943b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9943e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a99441, 61 },
    { 0x8f099442, 90 },
    { 0, 0 },
    { 0x3ac99444, 95 },
    { 0, 0 },
    { 0x51899446, 93 },
    { 0xc9f99447, 33 },
    { 0x21499448, 95 },
    { 0x59599449, 33 },
    { 0x20199448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe589944e, 95 },
    { 0, 0 },
    { 0xc4b99450, 98 },
    { 0xd899451, 53 },
    { 0, 0 },
    { 0x37b99453, 33 },
    { 0x89999453, 90 },
    { 0x74699455, 98 },
    { 0, 0 },
    { 0xc3a99457, 90 },
    { 0, 0 },
    { 0x50e99459, 33 },
    { 0xf229945a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9945d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa99462, 55 },
    { 0x30099463, 17 },
    { 0, 0 },
    { 0x63a99465, 78 },
    { 0x38b99465, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829946a, 43 },
    { 0, 0 },
    { 0x9fc9946c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d99474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31999477, 95 },
    { 0, 0 },
    { 0x3b999479, 23 },
    { 0, 0 },
    { 0xfc09947b, 23 },
    { 0, 0 },
    { 0x45e9947d, 95 },
    { 0x8319947e, 61 },
    { 0xc369947e, 93 },
    { 0xc789947f, 55 },
    { 0xaf899481, 61 },
    { 0x56399482, 61 },
    { 0x83899483, 84 },
    { 0xa3c99482, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d199487, 52 },
    { 0, 0 },
    { 0xf6099489, 33 },
    { 0xa2f9948a, 42 },
    { 0xe419948b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a99490, 98 },
    { 0, 0 },
    { 0xda699492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c199495, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb59949c, 33 },
    { 0x97c9949d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2994a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d994ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb994b0, 65 },
    { 0xcca994b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3994b4, 61 },
    { 0xa9e994b5, 76 },
    { 0x843994b4, 55 },
    { 0, 0 },
    { 0xef994b8, 90 },
    { 0x7c2994b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x388994bc, 33 },
    { 0x9c1994bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4994c2, 55 },
    { 0, 0 },
    { 0xd6f994c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e994c9, 90 },
    { 0, 0 },
    { 0xf78994cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f994d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d994d6, 23 },
    { 0x9df994d6, 23 },
    { 0, 0 },
    { 0x3c2994d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb994dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b994df, 36 },
    { 0x94b994e0, 90 },
    { 0x561994e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb994e4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x897994e7, 93 },
    { 0xf3a994e7, 90 },
    { 0x1994e9, 53 },
    { 0xad1994e8, 43 },
    { 0x6ee994eb, 91 },
    { 0xe8f994ec, 67 },
    { 0x17c994eb, 61 },
    { 0x89e994ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb994f1, 52 },
    { 0x226994f2, 36 },
    { 0xa9d994f3, 91 },
    { 0xcd9994f3, 93 },
    { 0x6d7994f1, 52 },
    { 0xc52994f6, 43 },
    { 0xd7f994f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f994ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad99502, 61 },
    { 0xed999502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9399507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af99510, 77 },
    { 0xf7d99510, 90 },
    { 0xe8e99512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e199516, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39999519, 98 },
    { 0, 0 },
    { 0x2719951b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d99523, 46 },
    { 0x5de99524, 34 },
    { 0x57799523, 33 },
    { 0, 0 },
    { 0x1a699527, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d99530, 33 },
    { 0x8af99531, 34 },
    { 0, 0 },
    { 0x99d99533, 84 },
    { 0x35799533, 90 },
    { 0x1b099535, 55 },
    { 0xc6e99535, 23 },
    { 0x37b99537, 44 },
    { 0x7a599534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b99953d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d99540, 33 },
    { 0xdfc99541, 33 },
    { 0x77199542, 87 },
    { 0x75499540, 33 },
    { 0, 0 },
    { 0xd5d99545, 92 },
    { 0xe8999546, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779954d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c499555, 93 },
    { 0x5b099556, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9955a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4179955d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc099564, 88 },
    { 0xc8599565, 33 },
    { 0x94599565, 90 },
    { 0, 0 },
    { 0x399568, 43 },
    { 0x91a99569, 90 },
    { 0x93299569, 76 },
    { 0xbf399568, 33 },
    { 0x5d39956c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f49956f, 46 },
    { 0, 0 },
    { 0xee099571, 76 },
    { 0xa9899571, 61 },
    { 0x19599572, 34 },
    { 0x79499573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75299577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a79957a, 78 },
    { 0, 0 },
    { 0x819957c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09958a, 95 },
    { 0xcac9958b, 90 },
    { 0x94b9958b, 23 },
    { 0, 0 },
    { 0x56c9958e, 95 },
    { 0, 0 },
    { 0x2fc99590, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb299593, 98 },
    { 0xabc99593, 83 },
    { 0x20699595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8699598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9959b, 98 },
    { 0x7259959b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608995a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55995a6, 99 },
    { 0x1b9995a7, 55 },
    { 0x655995a7, 98 },
    { 0x1ec995a8, 95 },
    { 0xcc1995a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d995ae, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84995b3, 84 },
    { 0xa40995b4, 78 },
    { 0x230995b5, 33 },
    { 0x388995b6, 61 },
    { 0x55995b7, 61 },
    { 0xaab995b8, 93 },
    { 0xc5e995b8, 55 },
    { 0xefb995ba, 46 },
    { 0x5ef995bb, 95 },
    { 0xa8b995b9, 85 },
    { 0xcd995bc, 66 },
    { 0x728995be, 61 },
    { 0xbea995b5, 95 },
    { 0xf5c995b6, 98 },
    { 0, 0 },
    { 0x750995c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d995c7, 76 },
    { 0x212995c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861995ce, 66 },
    { 0xd49995cf, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29995d3, 23 },
    { 0xd29995d4, 87 },
    { 0x339995d3, 98 },
    { 0x188995d6, 96 },
    { 0x545995d4, 75 },
    { 0xec5995d8, 81 },
    { 0xdc5995d8, 93 },
    { 0x239995d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04995e9, 98 },
    { 0x566995e9, 61 },
    { 0x92f995eb, 90 },
    { 0xd2a995eb, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13995ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592995f3, 88 },
    { 0x71e995f3, 95 },
    { 0xdd0995f5, 33 },
    { 0xa4e995f5, 33 },
    { 0, 0 },
    { 0xd28995f8, 95 },
    { 0xc4d995f8, 98 },
    { 0xb12995fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe399600, 75 },
    { 0x83a99601, 55 },
    { 0x5399602, 34 },
    { 0x7cc99601, 38 },
    { 0x41f99604, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd99607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f9960b, 95 },
    { 0x5f89960c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789960f, 90 },
    { 0x91699610, 76 },
    { 0x82e99611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f199614, 95 },
    { 0xca899615, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9961e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e9962c, 98 },
    { 0xe8f9962d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da99633, 43 },
    { 0x2c699634, 23 },
    { 0x59599633, 75 },
    { 0x5dd99636, 93 },
    { 0x32699636, 78 },
    { 0x84599638, 99 },
    { 0xdec99639, 98 },
    { 0xa7d99633, 100 },
    { 0x6919963b, 61 },
    { 0, 0 },
    { 0xa0e9963d, 95 },
    { 0x56d9963e, 76 },
    { 0xad9963f, 93 },
    { 0, 0 },
    { 0x35799641, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9399644, 23 },
    { 0, 0 },
    { 0xb4e99646, 52 },
    { 0x94899646, 33 },
    { 0xbae99647, 61 },
    { 0x6499649, 53 },
    { 0x4849964a, 17 },
    { 0x2329964b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe999652, 98 },
    { 0x2d199653, 23 },
    { 0xdcd99653, 90 },
    { 0x6e899653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0699659, 98 },
    { 0x4ec9965a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f299661, 90 },
    { 0xa1599661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd99665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d9966a, 93 },
    { 0, 0 },
    { 0xfbc9966c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb539966f, 90 },
    { 0xac799670, 61 },
    { 0, 0 },
    { 0x6e399672, 34 },
    { 0x49e99673, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9659967b, 55 },
    { 0, 0 },
    { 0xb979967d, 85 },
    { 0xa5f9967e, 98 },
    { 0x8609967e, 43 },
    { 0xa5299680, 97 },
    { 0xc689967f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d99685, 61 },
    { 0, 0 },
    { 0xadc99687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859968a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf199691, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b99695, 91 },
    { 0x7c599696, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd99699, 34 },
    { 0x9a699699, 33 },
    { 0x81399699, 95 },
    { 0x4a49969a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68996a9, 98 },
    { 0, 0 },
    { 0x8d5996ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae996b1, 55 },
    { 0, 0 },
    { 0xfad996b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf996b8, 61 },
    { 0xfcf996b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x288996bc, 61 },
    { 0, 0 },
    { 0xa4f996be, 95 },
    { 0x99c996bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81996c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb996c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b996ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea996d1, 90 },
    { 0, 0 },
    { 0x4f6996d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c996d7, 76 },
    { 0x8de996d7, 33 },
    { 0xec7996d9, 33 },
    { 0x25d996da, 33 },
    { 0x379996db, 55 },
    { 0x9f7996dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e996e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922996e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e996e8, 78 },
    { 0xc1a996e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76996ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15996f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c996f4, 64 },
    { 0x447996f5, 52 },
    { 0x262996f6, 46 },
    { 0xe51996f7, 33 },
    { 0x3a3996f8, 95 },
    { 0xc68996f5, 23 },
    { 0, 0 },
    { 0x7d7996fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce99701, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x81099704, 88 },
    { 0, 0 },
    { 0x6e799706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4539970a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb9970f, 87 },
    { 0x61499710, 52 },
    { 0x2bf9970f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d9971c, 90 },
    { 0xb599971d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5299721, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc899724, 33 },
    { 0xd6c99725, 33 },
    { 0xb8099726, 91 },
    { 0, 0 },
    { 0x9e999728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1079972b, 95 },
    { 0x1ff9972c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b99730, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22099734, 23 },
    { 0xddf99735, 77 },
    { 0x97d99736, 100 },
    { 0xd7299734, 88 },
    { 0, 0 },
    { 0xdc99739, 61 },
    { 0x13e9973a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919973e, 90 },
    { 0x4c99973f, 33 },
    { 0xef29973f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc99748, 61 },
    { 0xc7999749, 40 },
    { 0x7819974a, 95 },
    { 0, 0 },
    { 0x56f9974c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb9975a, 98 },
    { 0xf629975a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb9975f, 95 },
    { 0, 0 },
    { 0xc1a99761, 100 },
    { 0x52699761, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb899765, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479976b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a9976e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46899775, 76 },
    { 0x7a299776, 99 },
    { 0, 0 },
    { 0x73799778, 65 },
    { 0x67c99779, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe739977c, 90 },
    { 0xe889977c, 90 },
    { 0, 0 },
    { 0x8a29977f, 98 },
    { 0xce999780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2399783, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b9978e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999791, 100 },
    { 0xb099792, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53699797, 61 },
    { 0x8ca99798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec9979d, 95 },
    { 0x9289979d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71997a2, 91 },
    { 0x88b997a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5997a6, 14 },
    { 0x65997a7, 93 },
    { 0x8d7997a7, 33 },
    { 0x962997a8, 90 },
    { 0x984997aa, 84 },
    { 0, 0 },
    { 0x41b997ac, 87 },
    { 0xb6e997ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375997b4, 88 },
    { 0x4c5997b4, 80 },
    { 0xff4997b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5997b9, 53 },
    { 0x8ee997ba, 34 },
    { 0xb87997bb, 33 },
    { 0, 0 },
    { 0x3a8997bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe997c0, 61 },
    { 0xd7f997c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f997c9, 88 },
    { 0x3bd997ca, 98 },
    { 0x949997cb, 34 },
    { 0x56b997cc, 87 },
    { 0x3b7997cd, 85 },
    { 0x420997ce, 23 },
    { 0xc18997cf, 61 },
    { 0x6e1997cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205997d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d997da, 95 },
    { 0x2e997db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e997e3, 23 },
    { 0x43e997e4, 85 },
    { 0xf0a997e5, 34 },
    { 0xf0997e5, 80 },
    { 0, 0 },
    { 0x775997e8, 34 },
    { 0, 0 },
    { 0xe3a997ea, 90 },
    { 0xb2d997ea, 93 },
    { 0, 0 },
    { 0x362997ed, 33 },
    { 0x7f9997ed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691997f5, 95 },
    { 0x45c997f6, 23 },
    { 0x459997f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd997ff, 43 },
    { 0x3ee99800, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2899807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3269980a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64699811, 33 },
    { 0x78799812, 90 },
    { 0x34c99811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4699818, 76 },
    { 0x7e899819, 55 },
    { 0x51799819, 55 },
    { 0x3589981b, 88 },
    { 0xef9981c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46599820, 93 },
    { 0x7ea99820, 85 },
    { 0, 0 },
    { 0x6a899823, 33 },
    { 0, 0 },
    { 0xae799825, 61 },
    { 0x8e999825, 38 },
    { 0xc3399825, 61 },
    { 0xe0299826, 23 },
    { 0xa2599827, 84 },
    { 0, 0 },
    { 0x7d09982b, 43 },
    { 0xfad9982c, 23 },
    { 0, 0 },
    { 0xe1f9982e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb99838, 33 },
    { 0xc7d99839, 46 },
    { 0xa049983a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d99840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b99844, 33 },
    { 0x54b99845, 90 },
    { 0, 0 },
    { 0x4aa99847, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd309984d, 67 },
    { 0, 0 },
    { 0xde59984f, 99 },
    { 0x2819984f, 55 },
    { 0x1ef9984f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb99856, 61 },
    { 0x82b99856, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99985c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58399861, 97 },
    { 0x25a99862, 61 },
    { 0x22999862, 90 },
    { 0x79499863, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23899868, 90 },
    { 0xf7099869, 90 },
    { 0, 0 },
    { 0x7759986b, 66 },
    { 0x9d9986c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d99870, 23 },
    { 0x41299871, 75 },
    { 0, 0 },
    { 0xc5999873, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4699876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a99879, 61 },
    { 0xe0899879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979987d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9299882, 95 },
    { 0xba799882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a399886, 33 },
    { 0, 0 },
    { 0xe6199888, 61 },
    { 0x38b99889, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e59988d, 33 },
    { 0xe3d9988d, 98 },
    { 0x2979988f, 87 },
    { 0, 0 },
    { 0x50b99891, 55 },
    { 0, 0 },
    { 0x93e99893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95099896, 98 },
    { 0xdfb99897, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9989d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0998a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x177998a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x875998a6, 61 },
    { 0xe00998a7, 66 },
    { 0xeac998a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8998ab, 67 },
    { 0, 0 },
    { 0xc79998ad, 90 },
    { 0, 0 },
    { 0x71998af, 95 },
    { 0x3ab998b0, 55 },
    { 0xf08998b0, 98 },
    { 0xc19998b2, 91 },
    { 0x8c8998b3, 58 },
    { 0, 0 },
    { 0x3b998b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a998bc, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8998bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6998c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc998c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52998c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463998d3, 90 },
    { 0x518998d4, 33 },
    { 0xc51998d5, 61 },
    { 0xdee998d6, 66 },
    { 0xc74998d7, 46 },
    { 0x17f998d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71998df, 66 },
    { 0xc5b998e0, 17 },
    { 0xd0998e1, 67 },
    { 0xbb8998e2, 61 },
    { 0xb7a998e3, 52 },
    { 0x2e998df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05998eb, 23 },
    { 0, 0 },
    { 0x859998ed, 95 },
    { 0x648998ed, 61 },
    { 0xa34998ef, 52 },
    { 0, 0 },
    { 0x606998f1, 95 },
    { 0xf00998f2, 61 },
    { 0, 0 },
    { 0x3c9998f4, 90 },
    { 0, 0 },
    { 0x500998f6, 23 },
    { 0x1e5998f6, 17 },
    { 0xfa0998f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x989998fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d998fe, 68 },
    { 0xd41998ff, 95 },
    { 0x9c299900, 23 },
    { 0xc9199901, 98 },
    { 0xf2599902, 98 },
    { 0xfd899903, 76 },
    { 0xf0099901, 34 },
    { 0, 0 },
    { 0x95499906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2999909, 61 },
    { 0x8359990a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719990d, 68 },
    { 0xeb89990d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7599917, 43 },
    { 0x8999918, 78 },
    { 0, 0 },
    { 0x8949991a, 76 },
    { 0x7489991a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419991e, 52 },
    { 0x6009991f, 95 },
    { 0, 0 },
    { 0x80399921, 46 },
    { 0x28b99921, 61 },
    { 0x59899921, 90 },
    { 0, 0 },
    { 0xb2f99925, 33 },
    { 0xc4399925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd9992c, 95 },
    { 0x7d29992d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5899930, 95 },
    { 0x1a99931, 90 },
    { 0x77199932, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19499936, 52 },
    { 0x20199936, 43 },
    { 0x10899938, 90 },
    { 0, 0 },
    { 0xe919993a, 90 },
    { 0, 0 },
    { 0xaa9993c, 90 },
    { 0x57c9993d, 80 },
    { 0x3919993e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64699942, 87 },
    { 0x7799942, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e99949, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd99953, 23 },
    { 0xfa799954, 23 },
    { 0, 0 },
    { 0xf6099956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6809995b, 61 },
    { 0x58a9995c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c699961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d99966, 95 },
    { 0x22c99966, 66 },
    { 0, 0 },
    { 0xc2c99969, 61 },
    { 0x9bc9996a, 66 },
    { 0x5cd9996a, 33 },
    { 0x159996c, 90 },
    { 0xf0599969, 78 },
    { 0x22b99969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b99974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0499977, 99 },
    { 0, 0 },
    { 0x5e699979, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe9997c, 67 },
    { 0x96a9997d, 90 },
    { 0x2379997e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c99986, 33 },
    { 0x34399986, 64 },
    { 0, 0 },
    { 0x4b99989, 55 },
    { 0x3d99998a, 90 },
    { 0x1d399989, 33 },
    { 0xbd19998c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079998f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1899992, 33 },
    { 0, 0 },
    { 0xb2d99994, 40 },
    { 0, 0 },
    { 0x84a99996, 33 },
    { 0x4ef99996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee9999a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf69999d, 33 },
    { 0x1399999e, 55 },
    { 0x4729999f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1999a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19999a8, 99 },
    { 0xcd4999a8, 61 },
    { 0, 0 },
    { 0xe69999ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8999b0, 95 },
    { 0x2c6999b1, 93 },
    { 0, 0 },
    { 0xc16999b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3999b7, 53 },
    { 0xbf2999b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef999bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372999c2, 61 },
    { 0xb70999c3, 76 },
    { 0x43a999c4, 99 },
    { 0x172999c2, 75 },
    { 0xfab999c6, 98 },
    { 0xf72999c7, 23 },
    { 0, 0 },
    { 0x688999c9, 76 },
    { 0xf0999ca, 33 },
    { 0xba5999cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19999cf, 33 },
    { 0xe2b999d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3999d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228999d7, 90 },
    { 0x2f3999d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74999dd, 61 },
    { 0x7af999de, 88 },
    { 0, 0 },
    { 0xaff999e0, 87 },
    { 0, 0 },
    { 0x574999e2, 95 },
    { 0x3c1999e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e999e7, 33 },
    { 0, 0 },
    { 0x632999e9, 84 },
    { 0, 0 },
    { 0xab6999eb, 76 },
    { 0xdee999ec, 99 },
    { 0, 0 },
    { 0x21b999ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e999f4, 76 },
    { 0, 0 },
    { 0xdea999f6, 34 },
    { 0xed9999f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a999ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99a0a, 34 },
    { 0, 0 },
    { 0x57b99a0c, 34 },
    { 0xaf799a0d, 34 },
    { 0xc0499a0e, 55 },
    { 0, 0 },
    { 0x82e99a10, 99 },
    { 0x5f199a10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d399a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40699a18, 33 },
    { 0x54999a19, 33 },
    { 0x3ae99a1a, 95 },
    { 0, 0 },
    { 0x18999a1c, 78 },
    { 0x9c099a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c799a22, 66 },
    { 0, 0 },
    { 0x38899a24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba599a29, 34 },
    { 0, 0 },
    { 0x20599a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0799a32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b899a35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d99a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2899a42, 61 },
    { 0, 0 },
    { 0x35e99a44, 61 },
    { 0x1f899a44, 91 },
    { 0x3dd99a46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d399a49, 90 },
    { 0xbff99a49, 33 },
    { 0xce399a4a, 33 },
    { 0x1a99a4c, 98 },
    { 0xfcd99a4b, 90 },
    { 0x72b99a4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40799a53, 46 },
    { 0x9d199a54, 90 },
    { 0x83499a55, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b99a58, 67 },
    { 0, 0 },
    { 0xb4c99a5a, 17 },
    { 0, 0 },
    { 0x7899a5c, 61 },
    { 0, 0 },
    { 0xd0399a5e, 87 },
    { 0x37999a5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a99a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20199a6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42399a70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc99a73, 33 },
    { 0, 0 },
    { 0xa3299a75, 98 },
    { 0x7c099a76, 90 },
    { 0, 0 },
    { 0x63599a78, 33 },
    { 0, 0 },
    { 0x6c199a7a, 76 },
    { 0x52599a7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef799a80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59699a83, 52 },
    { 0xcee99a84, 17 },
    { 0x6cc99a83, 52 },
    { 0, 0 },
    { 0x98b99a87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7799a8a, 91 },
    { 0, 0 },
    { 0xf9e99a8c, 34 },
    { 0, 0 },
    { 0x82599a8e, 33 },
    { 0, 0 },
    { 0x4a699a90, 52 },
    { 0xc1899a91, 90 },
    { 0x85999a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3699a96, 23 },
    { 0x1d99a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed299a9d, 87 },
    { 0xcfa99a9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f699aa1, 90 },
    { 0x8fd99aa2, 43 },
    { 0, 0 },
    { 0x8b999aa4, 90 },
    { 0xe9199aa4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c399aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef99aab, 78 },
    { 0xbcc99aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd899ab0, 87 },
    { 0x61199ab1, 95 },
    { 0xb7a99ab1, 76 },
    { 0, 0 },
    { 0x1e99ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a99ab8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3699abb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c499ac0, 33 },
    { 0xd5c99ac1, 85 },
    { 0x2bf99ac1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f199ac6, 95 },
    { 0x30999ac7, 84 },
    { 0xd0599ac8, 61 },
    { 0x69b99ac9, 33 },
    { 0, 0 },
    { 0x15d99acb, 95 },
    { 0x64999acb, 43 },
    { 0, 0 },
    { 0x6f299ace, 52 },
    { 0, 0 },
    { 0x18699ad0, 90 },
    { 0x77199ad0, 33 },
    { 0xdfd99ad2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11699ad6, 95 },
    { 0xc4099ad6, 98 },
    { 0x7ee99ad6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d99adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb99ade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d99ae1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30099ae9, 33 },
    { 0xdb99ae9, 33 },
    { 0x75899aea, 76 },
    { 0, 0 },
    { 0x43599aed, 23 },
    { 0xa0499aed, 33 },
    { 0xb499aef, 52 },
    { 0xbd099aed, 40 },
    { 0x51d99af1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93099af4, 68 },
    { 0x49099af4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25899afe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8499b04, 55 },
    { 0xd3099b05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e599b09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f99b0c, 65 },
    { 0x8c999b0c, 55 },
    { 0, 0 },
    { 0x54399b0f, 76 },
    { 0x2a299b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be99b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9899b17, 61 },
    { 0, 0 },
    { 0x7b299b19, 84 },
    { 0x5ce99b19, 61 },
    { 0x97d99b1b, 84 },
    { 0x7cb99b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7999b22, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec399b26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b99b2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f99b31, 23 },
    { 0, 0 },
    { 0x1d799b33, 67 },
    { 0x22b99b34, 33 },
    { 0, 0 },
    { 0xf999b36, 55 },
    { 0x76c99b37, 65 },
    { 0, 0 },
    { 0x36c99b39, 33 },
    { 0, 0 },
    { 0xa99b3b, 55 },
    { 0, 0 },
    { 0xf2b99b3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b99b40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0099b45, 75 },
    { 0xcb799b46, 90 },
    { 0x21999b47, 61 },
    { 0x87e99b48, 90 },
    { 0x92b99b49, 98 },
    { 0x2e099b45, 61 },
    { 0, 0 },
    { 0xf9599b4c, 96 },
    { 0x82a99b4d, 61 },
    { 0x5c599b4d, 23 },
    { 0x22999b4f, 98 },
    { 0xc6099b50, 90 },
    { 0x6a299b51, 99 },
    { 0, 0 },
    { 0x9a299b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e899b5c, 78 },
    { 0x41b99b5c, 33 },
    { 0x7b699b5d, 78 },
    { 0, 0 },
    { 0x12e99b60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f399b65, 99 },
    { 0x23199b65, 43 },
    { 0x11a99b65, 47 },
    { 0x39999b67, 90 },
    { 0, 0 },
    { 0xb0399b6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c99b6d, 90 },
    { 0x33d99b6e, 76 },
    { 0xab99b6f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86999b72, 84 },
    { 0x91499b72, 95 },
    { 0xa8099b74, 90 },
    { 0xdf699b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a99b7c, 34 },
    { 0, 0 },
    { 0xf3b99b7e, 95 },
    { 0x60299b7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c99b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0899b89, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb99b90, 23 },
    { 0xc1c99b91, 46 },
    { 0x35d99b91, 100 },
    { 0, 0 },
    { 0xd1b99b94, 97 },
    { 0, 0 },
    { 0xa6699b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f699b99, 33 },
    { 0x42599b99, 98 },
    { 0, 0 },
    { 0xcea99b9c, 95 },
    { 0x74a99b9d, 33 },
    { 0x12d99b9d, 77 },
    { 0x4b99b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3799ba2, 95 },
    { 0, 0 },
    { 0x62e99ba4, 95 },
    { 0x14999ba4, 91 },
    { 0x7df99ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac99ba9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11299bae, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3499bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10499bb4, 61 },
    { 0x3d099bb5, 76 },
    { 0x1599bb6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c99bb9, 46 },
    { 0xe7899bba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82099bbe, 40 },
    { 0, 0 },
    { 0x62d99bc0, 55 },
    { 0xa4399bc0, 95 },
    { 0xfa799bc0, 23 },
    { 0x82f99bc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a99bc7, 91 },
    { 0x1f099bc7, 95 },
    { 0xe7899bc9, 90 },
    { 0x77199bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee199bcf, 95 },
    { 0, 0 },
    { 0x94a99bd1, 43 },
    { 0xfff99bd1, 23 },
    { 0x6ce99bd3, 66 },
    { 0x8f99bd1, 40 },
    { 0, 0 },
    { 0x32a99bd6, 55 },
    { 0x50899bd7, 93 },
    { 0x3c999bd7, 64 },
    { 0x68899bd7, 98 },
    { 0, 0 },
    { 0x1ff99bdb, 23 },
    { 0, 0 },
    { 0x2b99bdd, 61 },
    { 0x31a99bde, 90 },
    { 0x32199bdf, 98 },
    { 0x71099bde, 87 },
    { 0xdce99be1, 78 },
    { 0xdb699be2, 95 },
    { 0x22499be3, 88 },
    { 0x87699be3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14599bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24299bed, 55 },
    { 0x2f799bee, 55 },
    { 0x4ce99bef, 90 },
    { 0x35e99bf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db99bf3, 56 },
    { 0xe9999bf4, 69 },
    { 0x14b99bf4, 98 },
    { 0x599bf4, 36 },
    { 0x65999bf3, 98 },
    { 0, 0 },
    { 0x1c299bf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59399bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36199c00, 90 },
    { 0xabd99c00, 90 },
    { 0x5b099c02, 98 },
    { 0x61099c03, 90 },
    { 0x81f99c04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf99c09, 23 },
    { 0x94899c0a, 93 },
    { 0x35b99c0b, 33 },
    { 0x7b499c0c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9299c0f, 76 },
    { 0xfb799c0f, 61 },
    { 0, 0 },
    { 0x9fa99c12, 67 },
    { 0x33999c13, 90 },
    { 0xeee99c14, 90 },
    { 0xf6f99c14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf199c19, 23 },
    { 0, 0 },
    { 0x64b99c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c99c23, 23 },
    { 0xab899c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0999c2d, 33 },
    { 0, 0 },
    { 0xb9399c2f, 46 },
    { 0x87c99c2f, 51 },
    { 0x53399c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d699c35, 23 },
    { 0xe2d99c36, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a99c45, 69 },
    { 0, 0 },
    { 0x6f799c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa99c4a, 33 },
    { 0, 0 },
    { 0x1bf99c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0399c50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf399c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e99c59, 90 },
    { 0x74999c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b99c5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6099c62, 55 },
    { 0xb8999c63, 95 },
    { 0x8c699c64, 61 },
    { 0x51499c64, 23 },
    { 0x68399c66, 90 },
    { 0x22599c67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e99c6a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e399c6d, 33 },
    { 0x56499c6e, 43 },
    { 0x8cb99c6f, 43 },
    { 0x2c199c70, 43 },
    { 0x2b199c70, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f599c75, 87 },
    { 0xf5d99c76, 52 },
    { 0x5dd99c76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d99c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73899c7f, 81 },
    { 0x48899c80, 46 },
    { 0x3899c81, 98 },
    { 0, 0 },
    { 0xa9399c83, 61 },
    { 0, 0 },
    { 0x22499c85, 34 },
    { 0, 0 },
    { 0x38899c87, 90 },
    { 0x1c699c88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a99c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48499c8f, 61 },
    { 0, 0 },
    { 0xec899c91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f99c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb299ca0, 61 },
    { 0x98699ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30199ca6, 95 },
    { 0xc7099ca7, 46 },
    { 0x4d699ca7, 98 },
    { 0x71b99ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1599cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98699cba, 76 },
    { 0, 0 },
    { 0x11399cbc, 84 },
    { 0xdc299cbc, 33 },
    { 0x43b99cbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e99cc3, 95 },
    { 0, 0 },
    { 0x31099cc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e99cc8, 98 },
    { 0x83599cc9, 46 },
    { 0x1999cc9, 52 },
    { 0x35b99ccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b99cd0, 87 },
    { 0x35499cd1, 46 },
    { 0x64b99cd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78099cd7, 44 },
    { 0, 0 },
    { 0x45c99cd9, 37 },
    { 0xb0e99cda, 90 },
    { 0x10099cdb, 90 },
    { 0, 0 },
    { 0x2a99cdd, 90 },
    { 0xcb499cdd, 61 },
    { 0xe7199cdf, 90 },
    { 0xd8599ce0, 34 },
    { 0x8b699ce1, 87 },
    { 0x899cde, 95 },
    { 0x91199ce3, 33 },
    { 0x5ca99ce4, 76 },
    { 0, 0 },
    { 0x6ca99ce6, 43 },
    { 0x7fd99ce7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a499ceb, 66 },
    { 0, 0 },
    { 0x46099ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31799cf0, 90 },
    { 0xa5199cf0, 61 },
    { 0x38299cf2, 61 },
    { 0, 0 },
    { 0x48099cf4, 90 },
    { 0, 0 },
    { 0x71d99cf6, 55 },
    { 0xb499cf7, 76 },
    { 0, 0 },
    { 0x48f99cf9, 92 },
    { 0x1e399cfa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c99cfe, 43 },
    { 0x5399cff, 52 },
    { 0xae99d00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b99d03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x79599d06, 90 },
    { 0x20099d06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88799d12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af99d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a99d20, 23 },
    { 0x58999d20, 95 },
    { 0x4aa99d22, 22 },
    { 0xce999d22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1999d28, 61 },
    { 0x30699d28, 90 },
    { 0, 0 },
    { 0xdf599d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b099d2e, 74 },
    { 0x21a99d2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a99d33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a99d3a, 33 },
    { 0xe3d99d3a, 90 },
    { 0xf0299d3b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec99d41, 33 },
    { 0xa599d42, 46 },
    { 0xbf699d43, 90 },
    { 0xf7b99d44, 90 },
    { 0xf9499d43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b99d4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2999d4d, 66 },
    { 0xb7299d4e, 61 },
    { 0xd2d99d4f, 95 },
    { 0xce199d50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b99d54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc99d57, 67 },
    { 0, 0 },
    { 0xa1f99d59, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x95499d5c, 98 },
    { 0, 0 },
    { 0x13199d5e, 99 },
    { 0x9dd99d5f, 40 },
    { 0x41199d60, 95 },
    { 0xc1599d61, 90 },
    { 0xbdc99d62, 90 },
    { 0, 0 },
    { 0x40199d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47199d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb899d6a, 52 },
    { 0x19d99d6a, 75 },
    { 0, 0 },
    { 0x2b599d6d, 36 },
    { 0x39c99d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f99d72, 23 },
    { 0x36799d73, 61 },
    { 0, 0 },
    { 0x3c599d75, 76 },
    { 0xed699d76, 90 },
    { 0x32799d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec99d7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30599d7f, 87 },
    { 0x3ab99d7f, 93 },
    { 0xbd099d81, 66 },
    { 0, 0 },
    { 0xc5a99d83, 61 },
    { 0xb0699d83, 33 },
    { 0, 0 },
    { 0x5df99d86, 55 },
    { 0xf3d99d87, 90 },
    { 0, 0 },
    { 0x67399d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b99d8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9599d95, 98 },
    { 0, 0 },
    { 0x11299d97, 81 },
    { 0x5c699d98, 55 },
    { 0, 0 },
    { 0x67a99d9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d999d9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9299da2, 99 },
    { 0xfba99da3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a999da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d99da9, 100 },
    { 0x8d399daa, 61 },
    { 0xb8099dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca99daf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a99db2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1799db6, 61 },
    { 0x55299db7, 99 },
    { 0, 0 },
    { 0xfe099db9, 23 },
    { 0x4fa99dba, 51 },
    { 0x31599dbb, 66 },
    { 0xd9599dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d499dc6, 95 },
    { 0xd8f99dc7, 43 },
    { 0, 0 },
    { 0xa4599dc9, 33 },
    { 0xfd299dca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae99dcd, 88 },
    { 0, 0 },
    { 0xeb799dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11399dd2, 61 },
    { 0x15c99dd2, 88 },
    { 0x39999dd4, 61 },
    { 0, 0 },
    { 0x4ff99dd6, 90 },
    { 0x88899dd7, 55 },
    { 0x1c99dd8, 61 },
    { 0, 0 },
    { 0x21399dda, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae99dde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b899de2, 33 },
    { 0x30a99de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23599de8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf99dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c99df6, 23 },
    { 0x1b999df7, 61 },
    { 0xe9199df6, 90 },
    { 0, 0 },
    { 0x51199dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13499dfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e399e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db99e09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1999e0c, 34 },
    { 0xd6899e0d, 34 },
    { 0x9d499e0d, 91 },
    { 0, 0 },
    { 0x9ba99e10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78599e13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47799e1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94399e26, 33 },
    { 0x6a199e27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd599e2a, 88 },
    { 0xfbe99e2b, 78 },
    { 0xf0499e2b, 76 },
    { 0, 0 },
    { 0xa8d99e2e, 46 },
    { 0x25999e2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60299e32, 90 },
    { 0x1599e33, 76 },
    { 0xea999e32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d899e37, 90 },
    { 0x4c899e37, 98 },
    { 0xf0299e39, 23 },
    { 0, 0 },
    { 0x6ef99e3b, 90 },
    { 0x16999e3c, 61 },
    { 0xead99e3d, 66 },
    { 0x3ce99e3e, 95 },
    { 0x7af99e3f, 88 },
    { 0x12499e3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b199e45, 33 },
    { 0xda499e45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33099e49, 88 },
    { 0x2e399e49, 85 },
    { 0xf0e99e4a, 98 },
    { 0xa7f99e4c, 23 },
    { 0x199e4c, 91 },
    { 0xc0399e4e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25399e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd599e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e5c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1499e5f, 95 },
    { 0, 0 },
    { 0xee999e61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32099e66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c99e6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60399e72, 88 },
    { 0x8a999e73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69299e7a, 95 },
    { 0x38499e7b, 90 },
    { 0x4d899e7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde599e7f, 84 },
    { 0x34d99e80, 66 },
    { 0x36799e80, 61 },
    { 0x51e99e82, 90 },
    { 0x79799e83, 76 },
    { 0xca199e84, 23 },
    { 0xb8d99e7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b999e88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66299e91, 98 },
    { 0x9d499e92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1499e95, 23 },
    { 0x3399e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0599e9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc99ea3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f299ea7, 76 },
    { 0, 0 },
    { 0xaf99ea9, 78 },
    { 0, 0 },
    { 0xada99eab, 93 },
    { 0x4a799eab, 90 },
    { 0x52899eac, 95 },
    { 0x15099eab, 98 },
    { 0xf0399eab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d99eb5, 34 },
    { 0x7c999eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f99eba, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd099ec1, 90 },
    { 0xef199ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b99ec5, 87 },
    { 0xf1899ec6, 84 },
    { 0xda599ec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf99ed2, 87 },
    { 0xea799ed2, 46 },
    { 0x82299ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a199eda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f99ee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18099eea, 66 },
    { 0, 0 },
    { 0xb1499eec, 23 },
    { 0x95c99eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22899ef0, 17 },
    { 0, 0 },
    { 0x7e899ef2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf99ef9, 64 },
    { 0xf8c99efa, 23 },
    { 0x15399ef9, 98 },
    { 0x51a99efb, 76 },
    { 0xaaa99efd, 87 },
    { 0x92199efc, 67 },
    { 0xd7a99efa, 75 },
    { 0xaa399f00, 23 },
    { 0xb1399efc, 61 },
    { 0x55699efc, 61 },
    { 0xb8d99f03, 33 },
    { 0xdbb99f03, 85 },
    { 0xdd299f05, 95 },
    { 0xb9299f03, 90 },
    { 0xc8c99f04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a299f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb599f0f, 23 },
    { 0xa2299f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb799f15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f599f18, 33 },
    { 0, 0 },
    { 0x21799f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74799f21, 55 },
    { 0x11499f22, 52 },
    { 0xfaa99f23, 34 },
    { 0x9aa99f24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f99f29, 61 },
    { 0, 0 },
    { 0xb5599f2b, 98 },
    { 0x80999f2b, 33 },
    { 0, 0 },
    { 0x8b399f2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75099f37, 61 },
    { 0xbf499f38, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e99f3b, 92 },
    { 0xe1d99f3c, 61 },
    { 0xb5e99f3d, 46 },
    { 0, 0 },
    { 0x34399f3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799f42, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e499f4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf999f4e, 61 },
    { 0, 0 },
    { 0x2fb99f50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97499f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8999f58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb399f5b, 99 },
    { 0x6a799f5c, 33 },
    { 0x71a99f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78299f60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10099f64, 33 },
    { 0, 0 },
    { 0xfce99f66, 23 },
    { 0x3f999f67, 95 },
    { 0x1ce99f68, 64 },
    { 0xf3699f69, 90 },
    { 0xb3499f67, 23 },
    { 0x6fa99f6b, 85 },
    { 0xd9799f6b, 83 },
    { 0x4cd99f69, 84 },
    { 0x81099f69, 98 },
    { 0x68199f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f699f75, 55 },
    { 0x8a899f75, 76 },
    { 0x199f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d99f7a, 38 },
    { 0x6db99f7a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d99f7e, 85 },
    { 0, 0 },
    { 0x65899f80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc99f88, 84 },
    { 0, 0 },
    { 0xd8c99f8a, 34 },
    { 0x5a799f8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a99f91, 87 },
    { 0, 0 },
    { 0xe5199f93, 90 },
    { 0, 0 },
    { 0xe0899f95, 81 },
    { 0x83f99f96, 95 },
    { 0xe1299f97, 78 },
    { 0x1f199f97, 61 },
    { 0x55b99f96, 66 },
    { 0xaff99f9a, 87 },
    { 0xbc899f9b, 76 },
    { 0xd5e99f9c, 76 },
    { 0x64899f9d, 87 },
    { 0xcbe99f9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed99fad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c099fb2, 61 },
    { 0xc7399fb3, 65 },
    { 0x85899fb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37499fb7, 34 },
    { 0x6b99fb7, 93 },
    { 0xa3d99fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e799fbd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96799fc1, 33 },
    { 0xc7999fc2, 84 },
    { 0x83999fc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab99fc6, 90 },
    { 0xd399fc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a99fcb, 88 },
    { 0xad999fcc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a799fd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa99fd5, 77 },
    { 0, 0 },
    { 0xa5e99fd7, 52 },
    { 0, 0 },
    { 0xace99fd9, 61 },
    { 0x8d699fda, 61 },
    { 0, 0 },
    { 0xdc899fdc, 88 },
    { 0x90299fdd, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099fe4, 43 },
    { 0, 0 },
    { 0x5d899fe6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1699fe9, 90 },
    { 0xea099fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12999ff5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5399ffb, 95 },
    { 0xa3f99ffb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd549a004, 23 },
    { 0x7939a005, 90 },
    { 0x1489a006, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a9a00e, 90 },
    { 0x9d19a00f, 36 },
    { 0xa0f9a010, 55 },
    { 0xb849a011, 75 },
    { 0x3a39a012, 33 },
    { 0x25f9a013, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89a016, 77 },
    { 0x7a9a016, 23 },
    { 0xda69a018, 61 },
    { 0x5609a016, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d69a024, 23 },
    { 0xc0a9a025, 33 },
    { 0x3bb9a025, 99 },
    { 0, 0 },
    { 0xfbd9a028, 46 },
    { 0x7389a029, 61 },
    { 0x3469a02a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c09a031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5609a034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d89a039, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129a03c, 100 },
    { 0xb89a03d, 90 },
    { 0, 0 },
    { 0x5689a03f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e9a044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9a050, 90 },
    { 0xdcc9a051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe499a054, 34 },
    { 0, 0 },
    { 0x3339a056, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629a059, 87 },
    { 0xe8e9a05a, 88 },
    { 0x1bc9a05b, 93 },
    { 0x7f09a05b, 36 },
    { 0, 0 },
    { 0x2039a05e, 81 },
    { 0x9669a05e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b9a064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce9a069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1589a072, 76 },
    { 0xf689a073, 66 },
    { 0x7299a073, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9a078, 94 },
    { 0, 0 },
    { 0xf7e9a07a, 55 },
    { 0, 0 },
    { 0xe469a07c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19a07f, 23 },
    { 0, 0 },
    { 0xe0f9a081, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf39a089, 88 },
    { 0, 0 },
    { 0x8fa9a08b, 61 },
    { 0, 0 },
    { 0xdd09a08d, 97 },
    { 0xf8c9a08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f39a092, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc779a095, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899a09b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d9a0a1, 98 },
    { 0x16b9a0a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319a0a5, 55 },
    { 0, 0 },
    { 0x1449a0a7, 53 },
    { 0, 0 },
    { 0x8cc9a0a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc899a0ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829a0b0, 87 },
    { 0xcae9a0b1, 23 },
    { 0xf9a9a0b2, 76 },
    { 0, 0 },
    { 0xc629a0b4, 98 },
    { 0x4fa9a0b4, 91 },
    { 0x7e49a0b4, 90 },
    { 0x50f9a0b7, 33 },
    { 0xd5b9a0b5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c9a0bb, 61 },
    { 0xcfa9a0bb, 98 },
    { 0x8a99a0bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489a0c5, 85 },
    { 0x8fb9a0c6, 90 },
    { 0x6309a0c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df9a0ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9a0d1, 61 },
    { 0, 0 },
    { 0xbe9a0d3, 98 },
    { 0, 0 },
    { 0xf0e9a0d5, 90 },
    { 0, 0 },
    { 0x7c29a0d7, 81 },
    { 0, 0 },
    { 0x7629a0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce19a0de, 9 },
    { 0, 0 },
    { 0xcfe9a0e0, 83 },
    { 0x1a69a0e1, 95 },
    { 0x67f9a0e1, 33 },
    { 0x3f89a0e3, 68 },
    { 0xea19a0e0, 55 },
    { 0x4e69a0e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d9a0e9, 36 },
    { 0xd39a0e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc699a0ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e9a0f2, 98 },
    { 0, 0 },
    { 0x5499a0f4, 95 },
    { 0x26a9a0f4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb849a0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b9a101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef99a106, 43 },
    { 0, 0 },
    { 0x3399a108, 66 },
    { 0x9219a109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9a110, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3649a113, 98 },
    { 0x8099a114, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19a11a, 77 },
    { 0x9629a11a, 61 },
    { 0x9359a11b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49a120, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0x7439a123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee9a128, 90 },
    { 0xd829a129, 88 },
    { 0x14b9a129, 99 },
    { 0x7b59a12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf9a12f, 95 },
    { 0xf2a9a130, 34 },
    { 0xb509a130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769a134, 77 },
    { 0x4d99a135, 87 },
    { 0x259a135, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829a139, 99 },
    { 0, 0 },
    { 0x8bc9a13b, 99 },
    { 0xc809a13c, 90 },
    { 0x43e9a13c, 90 },
    { 0x8ad9a13c, 55 },
    { 0x42a9a13c, 99 },
    { 0, 0 },
    { 0xb049a141, 76 },
    { 0x2479a142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229a145, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299a14a, 90 },
    { 0, 0 },
    { 0x8f59a14c, 76 },
    { 0x6cd9a14d, 94 },
    { 0x46d9a14c, 76 },
    { 0x9329a14f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce9a153, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9a156, 78 },
    { 0x3559a157, 87 },
    { 0xd809a158, 43 },
    { 0x9dc9a157, 61 },
    { 0x9059a158, 99 },
    { 0x8cd9a157, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa659a160, 23 },
    { 0, 0 },
    { 0xe5d9a162, 77 },
    { 0, 0 },
    { 0xb559a164, 93 },
    { 0, 0 },
    { 0x5629a166, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69a16d, 76 },
    { 0x11f9a16e, 87 },
    { 0x2da9a16f, 90 },
    { 0, 0 },
    { 0x2c39a171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4199a178, 91 },
    { 0x7259a179, 33 },
    { 0xe3a9a179, 66 },
    { 0x709a17b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339a183, 61 },
    { 0x1539a184, 55 },
    { 0x6199a184, 61 },
    { 0x9e39a186, 43 },
    { 0xe249a186, 91 },
    { 0x2709a188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429a18d, 65 },
    { 0, 0 },
    { 0xece9a18f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519a192, 76 },
    { 0, 0 },
    { 0xa299a194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc19a197, 23 },
    { 0, 0 },
    { 0x88c9a199, 85 },
    { 0x91c9a19a, 33 },
    { 0, 0 },
    { 0x389a19c, 90 },
    { 0, 0 },
    { 0x6419a19e, 66 },
    { 0x40c9a19e, 17 },
    { 0x6509a1a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa389a1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a1ad, 23 },
    { 0, 0 },
    { 0xfa49a1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb69a1b2, 90 },
    { 0, 0 },
    { 0x2a49a1b4, 33 },
    { 0xa049a1b5, 23 },
    { 0, 0 },
    { 0x9c09a1b7, 93 },
    { 0x67b9a1b7, 95 },
    { 0xf249a1b7, 76 },
    { 0xa599a1ba, 55 },
    { 0x7679a1bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d39a1be, 61 },
    { 0xfd99a1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949a1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe119a1c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7069a1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769a1d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a1d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc279a1dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9a1e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1789a1e4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069a1ec, 25 },
    { 0, 0 },
    { 0xc999a1ee, 90 },
    { 0x39f9a1ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79a1f5, 33 },
    { 0, 0 },
    { 0x63d9a1f7, 66 },
    { 0xa509a1f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769a1fd, 95 },
    { 0x10b9a1fe, 76 },
    { 0xfa69a1ff, 95 },
    { 0xa0c9a200, 52 },
    { 0, 0 },
    { 0x9e39a202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3199a205, 90 },
    { 0x9219a206, 95 },
    { 0xb8f9a207, 46 },
    { 0xbce9a208, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989a20e, 23 },
    { 0x4409a20f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5609a212, 61 },
    { 0, 0 },
    { 0x4cb9a214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9219a217, 43 },
    { 0, 0 },
    { 0x5679a219, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9a21c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79a223, 76 },
    { 0xa609a224, 43 },
    { 0, 0 },
    { 0xbc9a226, 84 },
    { 0x2dd9a226, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc59a22f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde99a232, 61 },
    { 0x7879a232, 23 },
    { 0x9919a234, 49 },
    { 0x8479a233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819a23b, 66 },
    { 0x6729a23c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa879a244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d9a247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a24a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9a24d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9a250, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e9a253, 90 },
    { 0, 0 },
    { 0xc039a255, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b19a25a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4509a25e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9a266, 90 },
    { 0x62e9a267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2069a26c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb089a26f, 95 },
    { 0x71e9a270, 55 },
    { 0xfbb9a271, 95 },
    { 0, 0 },
    { 0xecb9a273, 23 },
    { 0x53d9a273, 98 },
    { 0xaca9a274, 100 },
    { 0x6f09a276, 100 },
    { 0x7449a275, 34 },
    { 0, 0 },
    { 0x9919a279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d9a27c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f9a27f, 98 },
    { 0, 0 },
    { 0xd019a281, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d9a286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd9a289, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d9a28c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a290, 33 },
    { 0xd639a290, 33 },
    { 0, 0 },
    { 0xba69a293, 34 },
    { 0x7a69a293, 55 },
    { 0x28e9a293, 84 },
    { 0xacf9a296, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e9a299, 91 },
    { 0, 0 },
    { 0x2159a29b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e9a29f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9a2a6, 98 },
    { 0, 0 },
    { 0x38a9a2a8, 75 },
    { 0, 0 },
    { 0x27f9a2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539a2b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9a2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x179a2b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9a2c2, 43 },
    { 0, 0 },
    { 0x5a69a2c4, 24 },
    { 0xf189a2c5, 23 },
    { 0x30b9a2c6, 99 },
    { 0xadc9a2c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3469a2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f09a2d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9a2d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b9a2d7, 95 },
    { 0, 0 },
    { 0x8549a2d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f89a2dd, 84 },
    { 0x5409a2de, 34 },
    { 0, 0 },
    { 0x4529a2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9389a2ed, 23 },
    { 0xbd99a2ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f9a2f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc879a2f6, 23 },
    { 0, 0 },
    { 0x1aa9a2f8, 33 },
    { 0x859a2f8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4959a301, 38 },
    { 0, 0 },
    { 0x8a59a303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c9a309, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9a30e, 43 },
    { 0xc079a30f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd79a314, 36 },
    { 0xbbe9a315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a319, 85 },
    { 0, 0 },
    { 0x6539a31b, 23 },
    { 0x4459a31c, 43 },
    { 0, 0 },
    { 0x39e9a31e, 33 },
    { 0x9479a31e, 55 },
    { 0xd059a320, 100 },
    { 0xb829a320, 43 },
    { 0, 0 },
    { 0x2c79a323, 33 },
    { 0x5fb9a324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79a329, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979a32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5979a330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979a334, 81 },
    { 0x7d09a334, 17 },
    { 0x12d9a336, 46 },
    { 0xf6f9a337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9a33a, 23 },
    { 0xae29a33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d9a33e, 23 },
    { 0xb1d9a33f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69a342, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa89a348, 91 },
    { 0x8339a349, 90 },
    { 0x60f9a34a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc09a350, 98 },
    { 0, 0 },
    { 0x4079a352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69a35b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc199a35e, 33 },
    { 0xbc39a35e, 34 },
    { 0x4a29a360, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e99a363, 93 },
    { 0xa899a364, 100 },
    { 0x65d9a365, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d9a368, 95 },
    { 0xab99a368, 95 },
    { 0xe89a368, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc09a36f, 66 },
    { 0, 0 },
    { 0xf299a371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b9a377, 95 },
    { 0x7cf9a378, 87 },
    { 0xa969a379, 90 },
    { 0xea99a37a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e9a37e, 33 },
    { 0xd159a37f, 61 },
    { 0, 0 },
    { 0xffa9a381, 33 },
    { 0xc0e9a382, 61 },
    { 0, 0 },
    { 0x3ee9a384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639a389, 61 },
    { 0, 0 },
    { 0xc279a38b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9a38e, 98 },
    { 0x6aa9a38f, 95 },
    { 0x5e09a38e, 90 },
    { 0x5b89a391, 69 },
    { 0x2109a392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f99a395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec9a39a, 46 },
    { 0x2d99a39a, 61 },
    { 0x6d79a39c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa9a39f, 61 },
    { 0x5b59a3a0, 33 },
    { 0x42b9a3a1, 65 },
    { 0x96d9a3a1, 98 },
    { 0x8e19a3a2, 55 },
    { 0, 0 },
    { 0x5969a3a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9a3ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9439a3af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a49a3b4, 61 },
    { 0, 0 },
    { 0xce79a3b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e9a3b9, 73 },
    { 0, 0 },
    { 0xea39a3bb, 66 },
    { 0x6d19a3bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b9a3c0, 33 },
    { 0x5e39a3c1, 95 },
    { 0x26f9a3c1, 55 },
    { 0x4f59a3c2, 61 },
    { 0x2419a3c2, 90 },
    { 0x4479a3c3, 33 },
    { 0x5829a3c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419a3ce, 98 },
    { 0xf409a3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e9a3d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839a3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb9a3dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a09a3e1, 55 },
    { 0, 0 },
    { 0xa2f9a3e3, 55 },
    { 0xd009a3e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969a3e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f79a3eb, 23 },
    { 0xf249a3ec, 46 },
    { 0, 0 },
    { 0xe269a3ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb69a3f1, 23 },
    { 0xf299a3f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5079a3f5, 95 },
    { 0, 0 },
    { 0x20d9a3f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2849a405, 53 },
    { 0, 0 },
    { 0xb189a407, 55 },
    { 0x48e9a408, 95 },
    { 0x1189a409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c29a417, 76 },
    { 0, 0 },
    { 0x58d9a419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a39a41c, 90 },
    { 0, 0 },
    { 0x4a09a41e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa79a425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a428, 33 },
    { 0, 0 },
    { 0x1439a42a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e9a42e, 55 },
    { 0, 0 },
    { 0x21f9a430, 90 },
    { 0, 0 },
    { 0x189a432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9a43a, 33 },
    { 0xfc29a43a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a9a43f, 95 },
    { 0x3349a43f, 69 },
    { 0x8d19a441, 85 },
    { 0xc149a442, 98 },
    { 0x6e39a442, 36 },
    { 0x5859a444, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab9a44a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d9a44d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa309a450, 90 },
    { 0xead9a451, 52 },
    { 0xa229a452, 90 },
    { 0x1199a453, 99 },
    { 0x25d9a454, 33 },
    { 0, 0 },
    { 0x5a19a456, 19 },
    { 0x34e9a457, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb319a45b, 55 },
    { 0x97f9a45c, 33 },
    { 0xc389a45d, 78 },
    { 0xeee9a45c, 34 },
    { 0x5e09a45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539a466, 90 },
    { 0x2249a466, 90 },
    { 0x1299a468, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea9a46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b89a472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9a478, 23 },
    { 0, 0 },
    { 0xdcc9a47a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d9a485, 61 },
    { 0x4ca9a486, 33 },
    { 0, 0 },
    { 0xf9e9a488, 42 },
    { 0xeb79a489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789a48c, 93 },
    { 0x4a69a48d, 67 },
    { 0xb8b9a48e, 90 },
    { 0x88f9a48f, 61 },
    { 0xd409a48e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89a494, 33 },
    { 0, 0 },
    { 0x9ce9a496, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa979a49d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac9a4a0, 23 },
    { 0x129a4a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d9a4a5, 23 },
    { 0, 0 },
    { 0x8a49a4a7, 87 },
    { 0xb629a4a7, 90 },
    { 0, 0 },
    { 0xba19a4aa, 33 },
    { 0, 0 },
    { 0x8fa9a4ac, 33 },
    { 0x15e9a4ad, 84 },
    { 0x9cd9a4ae, 100 },
    { 0x5e09a4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9a4ba, 89 },
    { 0xce99a4ba, 76 },
    { 0x3ed9a4ba, 90 },
    { 0x49a4ba, 52 },
    { 0x7289a4be, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c9a4c1, 68 },
    { 0x5589a4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc59a4c5, 33 },
    { 0x35f9a4c6, 67 },
    { 0, 0 },
    { 0x7ba9a4c8, 43 },
    { 0x6679a4c8, 67 },
    { 0, 0 },
    { 0xb419a4cb, 76 },
    { 0x7f79a4cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419a4d3, 61 },
    { 0xab59a4d3, 74 },
    { 0xa3c9a4d5, 88 },
    { 0xcce9a4d3, 91 },
    { 0, 0 },
    { 0x3599a4d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9a4de, 76 },
    { 0xd409a4df, 61 },
    { 0xf49a4df, 95 },
    { 0x45e9a4e1, 33 },
    { 0xdbc9a4de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9789a4eb, 99 },
    { 0xc779a4ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409a4ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e99a4f2, 33 },
    { 0x5e99a4f3, 87 },
    { 0xf8f9a4f3, 66 },
    { 0xf439a4f3, 69 },
    { 0x2a99a4f6, 90 },
    { 0, 0 },
    { 0xcb99a4f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b9a4fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c9a500, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e59a505, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3669a50b, 33 },
    { 0, 0 },
    { 0x92c9a50d, 17 },
    { 0x63a9a50d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49a511, 84 },
    { 0, 0 },
    { 0x7d19a513, 98 },
    { 0x3f89a514, 92 },
    { 0x6089a515, 90 },
    { 0, 0 },
    { 0xf899a517, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd9a51d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd9a521, 23 },
    { 0xb2d9a522, 33 },
    { 0, 0 },
    { 0x1a99a524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd9a527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e9a52a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa9a52d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f29a532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb349a535, 23 },
    { 0x4979a535, 90 },
    { 0x56c9a537, 23 },
    { 0x4b99a536, 24 },
    { 0xd1a9a539, 64 },
    { 0x9769a536, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef9a53e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c39a541, 90 },
    { 0xa799a542, 61 },
    { 0x67b9a543, 88 },
    { 0x12b9a544, 95 },
    { 0x2009a542, 90 },
    { 0x98d9a546, 34 },
    { 0x39c9a543, 33 },
    { 0x5879a548, 95 },
    { 0x9929a549, 97 },
    { 0x3129a548, 65 },
    { 0x8f69a54b, 90 },
    { 0x8879a54c, 61 },
    { 0, 0 },
    { 0xf799a54e, 85 },
    { 0xce29a54f, 80 },
    { 0, 0 },
    { 0xcfb9a551, 83 },
    { 0, 0 },
    { 0xa909a553, 85 },
    { 0x1399a553, 95 },
    { 0, 0 },
    { 0xa959a556, 33 },
    { 0xcd69a556, 33 },
    { 0, 0 },
    { 0x1d29a559, 23 },
    { 0xc3a9a55a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6429a55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa049a561, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb489a564, 96 },
    { 0xd7b9a564, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa9a56b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9a56e, 85 },
    { 0, 0 },
    { 0xae39a570, 33 },
    { 0x4499a571, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9a574, 61 },
    { 0, 0 },
    { 0xeea9a576, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9a57a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b9a582, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe849a585, 98 },
    { 0x2759a586, 33 },
    { 0xc5b9a586, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc9a58a, 90 },
    { 0xaca9a58b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89a58e, 52 },
    { 0, 0 },
    { 0x4cf9a590, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f99a593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9a597, 91 },
    { 0x7259a598, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc789a59c, 61 },
    { 0xa479a59c, 52 },
    { 0x7ae9a59c, 46 },
    { 0x9be9a59c, 23 },
    { 0xfb19a5a0, 43 },
    { 0, 0 },
    { 0x3e09a5a2, 66 },
    { 0xe219a5a3, 90 },
    { 0x4779a5a2, 33 },
    { 0, 0 },
    { 0xeef9a5a6, 44 },
    { 0xd429a5a7, 81 },
    { 0, 0 },
    { 0x5eb9a5a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a5ad, 90 },
    { 0x7099a5ae, 33 },
    { 0x9029a5af, 61 },
    { 0x1979a5af, 43 },
    { 0, 0 },
    { 0x9a99a5b2, 95 },
    { 0xf959a5b2, 23 },
    { 0x6839a5b4, 95 },
    { 0xfe79a5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4329a5b9, 90 },
    { 0xbd89a5b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3129a5c2, 61 },
    { 0, 0 },
    { 0x6dd9a5c4, 98 },
    { 0x7c89a5c5, 77 },
    { 0x3409a5c5, 95 },
    { 0, 0 },
    { 0xca19a5c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe589a5cb, 61 },
    { 0x5af9a5cc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39a5d0, 76 },
    { 0, 0 },
    { 0xf6d9a5d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe879a5d9, 90 },
    { 0xbde9a5da, 90 },
    { 0x979a5d9, 55 },
    { 0, 0 },
    { 0xea29a5dd, 63 },
    { 0xa69a5dd, 90 },
    { 0xf2a9a5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29a5e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b09a5e5, 43 },
    { 0x87c9a5e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19a5ea, 90 },
    { 0x99d9a5eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb589a5ef, 68 },
    { 0x4fc9a5f0, 33 },
    { 0xe149a5f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf29a5f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6959a5f9, 61 },
    { 0, 0 },
    { 0x8929a5fb, 95 },
    { 0xfb89a5fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a9a604, 61 },
    { 0, 0 },
    { 0x9a59a606, 33 },
    { 0x7299a607, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1379a60a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9a60d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219a610, 84 },
    { 0x2f19a611, 98 },
    { 0xdea9a611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3859a616, 33 },
    { 0, 0 },
    { 0xcd59a618, 52 },
    { 0xe7c9a619, 66 },
    { 0x6a89a61a, 98 },
    { 0xc4c9a61b, 23 },
    { 0xf089a619, 90 },
    { 0x7d9a61d, 33 },
    { 0xaed9a61c, 46 },
    { 0x80e9a61a, 98 },
    { 0, 0 },
    { 0x8eb9a621, 43 },
    { 0x9699a621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5619a625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f9a62a, 55 },
    { 0xd29a62b, 33 },
    { 0xef29a62c, 33 },
    { 0, 0 },
    { 0xb7a9a62e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a09a631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a637, 23 },
    { 0xf509a638, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529a63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb9a640, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a69a643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99a647, 90 },
    { 0x2509a648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b89a650, 61 },
    { 0xafd9a650, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819a658, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9569a664, 90 },
    { 0, 0 },
    { 0xc5e9a666, 98 },
    { 0xeaf9a667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad9a66f, 74 },
    { 0, 0 },
    { 0x1969a671, 99 },
    { 0xc559a672, 90 },
    { 0x26a9a671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e69a677, 52 },
    { 0x1a09a677, 33 },
    { 0xb169a679, 77 },
    { 0xcb09a677, 23 },
    { 0x4d59a67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a69a67f, 55 },
    { 0x80f9a680, 90 },
    { 0, 0 },
    { 0xcfd9a682, 99 },
    { 0x7569a683, 53 },
    { 0, 0 },
    { 0x6979a685, 91 },
    { 0x9799a685, 99 },
    { 0xb229a687, 90 },
    { 0xc8b9a688, 99 },
    { 0xc939a689, 84 },
    { 0x51e9a68a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a9a68d, 61 },
    { 0, 0 },
    { 0x6319a68f, 98 },
    { 0x1db9a68f, 55 },
    { 0x6ae9a691, 33 },
    { 0, 0 },
    { 0xbbd9a693, 61 },
    { 0x3269a694, 94 },
    { 0xc359a694, 61 },
    { 0x5fd9a694, 100 },
    { 0, 0 },
    { 0x8139a698, 76 },
    { 0xf6c9a699, 46 },
    { 0xab19a69a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b9a6a5, 100 },
    { 0x8d79a6a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29a6ab, 61 },
    { 0x6fe9a6ab, 77 },
    { 0, 0 },
    { 0xbcf9a6ae, 98 },
    { 0xb939a6af, 52 },
    { 0, 0 },
    { 0xcc9a6b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9a6b4, 33 },
    { 0, 0 },
    { 0xeeb9a6b6, 33 },
    { 0xa979a6b6, 33 },
    { 0, 0 },
    { 0xfbb9a6b9, 33 },
    { 0, 0 },
    { 0x9399a6bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629a6be, 33 },
    { 0x1259a6be, 66 },
    { 0, 0 },
    { 0x1589a6c1, 98 },
    { 0x3739a6c1, 98 },
    { 0, 0 },
    { 0xcd49a6c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf99a6c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9a6d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329a6d5, 90 },
    { 0, 0 },
    { 0x27d9a6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9a6da, 52 },
    { 0, 0 },
    { 0xd1b9a6dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a9a6e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a9a6e6, 95 },
    { 0xbdd9a6e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999a6eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249a6f0, 80 },
    { 0, 0 },
    { 0x3389a6f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd109a6f7, 34 },
    { 0xc249a6f7, 95 },
    { 0x9f19a6f7, 33 },
    { 0, 0 },
    { 0xb989a6fb, 76 },
    { 0, 0 },
    { 0xa3d9a6fd, 61 },
    { 0, 0 },
    { 0x16c9a6ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae9a702, 68 },
    { 0, 0 },
    { 0x2009a704, 34 },
    { 0xb989a705, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059a70a, 55 },
    { 0, 0 },
    { 0xa5e9a70c, 33 },
    { 0, 0 },
    { 0xf309a70e, 34 },
    { 0xa6f9a70f, 85 },
    { 0, 0 },
    { 0x1b99a711, 90 },
    { 0, 0 },
    { 0x6819a713, 55 },
    { 0xcae9a714, 67 },
    { 0x2bd9a715, 95 },
    { 0x55b9a716, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1859a71a, 34 },
    { 0x3cb9a71b, 52 },
    { 0xf579a71b, 90 },
    { 0x8259a71d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf889a721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9a726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89a729, 98 },
    { 0, 0 },
    { 0x9709a72b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd9a730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799a735, 67 },
    { 0xdbc9a735, 33 },
    { 0, 0 },
    { 0xf739a738, 23 },
    { 0x1489a738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809a73c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8249a74a, 95 },
    { 0, 0 },
    { 0xab39a74c, 92 },
    { 0x40c9a74c, 23 },
    { 0x8169a74e, 76 },
    { 0xd509a74e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e9a753, 34 },
    { 0x67e9a753, 80 },
    { 0x1629a754, 93 },
    { 0x5139a755, 23 },
    { 0xfc9a755, 56 },
    { 0xb439a756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9a75c, 23 },
    { 0x7749a75c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339a760, 61 },
    { 0x3ce9a760, 36 },
    { 0x5339a761, 33 },
    { 0, 0 },
    { 0xda49a764, 55 },
    { 0x8f39a765, 90 },
    { 0x839a764, 23 },
    { 0, 0 },
    { 0x7769a768, 61 },
    { 0xb659a769, 61 },
    { 0x1629a769, 98 },
    { 0x28e9a769, 17 },
    { 0x4669a76c, 61 },
    { 0, 0 },
    { 0x3749a76e, 98 },
    { 0x3d9a76f, 84 },
    { 0x27a9a770, 75 },
    { 0, 0 },
    { 0x3fb9a772, 85 },
    { 0x5509a773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a9a776, 90 },
    { 0x6e59a777, 69 },
    { 0, 0 },
    { 0x4869a779, 95 },
    { 0xa699a77a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129a77e, 95 },
    { 0x68f9a77f, 100 },
    { 0xb199a780, 36 },
    { 0xe249a780, 33 },
    { 0xdb99a782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e69a785, 61 },
    { 0xd0c9a785, 90 },
    { 0, 0 },
    { 0x9da9a788, 52 },
    { 0, 0 },
    { 0x8e9a78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c39a78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf9a791, 78 },
    { 0, 0 },
    { 0xdc49a793, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e49a797, 34 },
    { 0x6ca9a798, 98 },
    { 0, 0 },
    { 0x3779a79a, 99 },
    { 0xe369a79b, 90 },
    { 0, 0 },
    { 0x78c9a79d, 90 },
    { 0, 0 },
    { 0x8c79a79f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9a7a7, 76 },
    { 0, 0 },
    { 0xa039a7a9, 91 },
    { 0x7dc9a7aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb9a7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729a7b1, 87 },
    { 0x6c39a7b2, 75 },
    { 0xa789a7b2, 55 },
    { 0xac09a7b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd09a7b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f9a7bb, 90 },
    { 0x3789a7bc, 61 },
    { 0xf219a7bc, 81 },
    { 0xfb19a7bd, 84 },
    { 0xef19a7bc, 90 },
    { 0x75e9a7c0, 95 },
    { 0xa4a9a7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9629a7c4, 33 },
    { 0x6d9a7c4, 67 },
    { 0x2f9a7c6, 33 },
    { 0xee59a7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf109a7ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89a7d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89a7d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a9a7dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6869a7e6, 95 },
    { 0, 0 },
    { 0x75d9a7e8, 98 },
    { 0x36b9a7e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe139a7f2, 33 },
    { 0x709a7f2, 68 },
    { 0x5aa9a7f4, 34 },
    { 0x8219a7f5, 76 },
    { 0, 0 },
    { 0xcaf9a7f7, 61 },
    { 0xf439a7f8, 23 },
    { 0x5379a7f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee9a7fe, 34 },
    { 0, 0 },
    { 0xde09a800, 76 },
    { 0xfad9a801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe9a804, 98 },
    { 0x6119a805, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd9a808, 91 },
    { 0, 0 },
    { 0xde59a80a, 67 },
    { 0x4539a80b, 33 },
    { 0x1709a80b, 34 },
    { 0xfb49a80c, 98 },
    { 0x4209a80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa9a814, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819a819, 100 },
    { 0, 0 },
    { 0x3ef9a81b, 43 },
    { 0xd0d9a81c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb59a823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9a826, 90 },
    { 0x1259a826, 61 },
    { 0, 0 },
    { 0xf099a829, 90 },
    { 0x4b59a82a, 61 },
    { 0x3df9a82a, 77 },
    { 0, 0 },
    { 0x4b69a82d, 97 },
    { 0, 0 },
    { 0xf389a82f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9a835, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9a838, 55 },
    { 0x5189a839, 75 },
    { 0x4d49a83a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629a83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259a846, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e89a84e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079a853, 80 },
    { 0, 0 },
    { 0xfdf9a855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e39a85b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5299a861, 43 },
    { 0x9219a862, 46 },
    { 0x8949a861, 33 },
    { 0x4639a864, 67 },
    { 0xbd09a862, 95 },
    { 0, 0 },
    { 0xea69a867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709a86b, 92 },
    { 0x6dd9a86c, 85 },
    { 0xc9c9a86d, 93 },
    { 0xb079a86e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa529a871, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69a874, 23 },
    { 0, 0 },
    { 0x1079a876, 33 },
    { 0, 0 },
    { 0x8509a878, 93 },
    { 0, 0 },
    { 0xde19a87a, 33 },
    { 0xf579a87b, 15 },
    { 0xc639a87a, 95 },
    { 0x7699a87d, 33 },
    { 0xffd9a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef29a881, 52 },
    { 0x3379a882, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9a885, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb139a889, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9a88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad39a894, 43 },
    { 0x6ce9a894, 61 },
    { 0xd9a9a894, 93 },
    { 0x5639a897, 23 },
    { 0xd7e9a897, 76 },
    { 0x8649a898, 56 },
    { 0xdb19a89a, 61 },
    { 0, 0 },
    { 0xae49a89c, 99 },
    { 0, 0 },
    { 0x8319a89e, 100 },
    { 0, 0 },
    { 0x8929a8a0, 46 },
    { 0, 0 },
    { 0x46f9a8a2, 34 },
    { 0xad89a8a2, 38 },
    { 0x7749a8a4, 61 },
    { 0xa399a8a5, 61 },
    { 0x52a9a8a6, 55 },
    { 0xe049a8a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa319a8b1, 90 },
    { 0x1129a8b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8639a8b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2289a8b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9a8bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf39a8c0, 90 },
    { 0x929a8c1, 67 },
    { 0xa799a8c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8659a8c5, 78 },
    { 0xa7e9a8c6, 55 },
    { 0, 0 },
    { 0x74e9a8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a8cb, 90 },
    { 0x5679a8cc, 61 },
    { 0, 0 },
    { 0x50e9a8ce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf9a8d7, 98 },
    { 0x7069a8d8, 61 },
    { 0xbbd9a8d7, 33 },
    { 0x439a8da, 95 },
    { 0x69b9a8da, 33 },
    { 0x6129a8dc, 33 },
    { 0xfed9a8d9, 52 },
    { 0x92b9a8dd, 33 },
    { 0, 0 },
    { 0x8459a8e0, 61 },
    { 0xa739a8e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8489a8e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a9a8e7, 34 },
    { 0x5de9a8e8, 88 },
    { 0x11f9a8e7, 76 },
    { 0x1959a8ea, 76 },
    { 0x2f49a8eb, 55 },
    { 0x9d69a8eb, 66 },
    { 0xb89a8ed, 90 },
    { 0x5889a8ee, 95 },
    { 0x139a8ef, 56 },
    { 0x3679a8ef, 88 },
    { 0xcf89a8f1, 79 },
    { 0xcb49a8ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f79a8fc, 33 },
    { 0xb3e9a8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf979a900, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449a903, 99 },
    { 0, 0 },
    { 0x7e99a905, 95 },
    { 0, 0 },
    { 0xa9f9a907, 23 },
    { 0xd879a908, 95 },
    { 0x7629a908, 23 },
    { 0x2699a908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c29a911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d99a91f, 43 },
    { 0xd129a920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59a924, 90 },
    { 0, 0 },
    { 0x5079a926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5909a92c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a933, 67 },
    { 0x4159a934, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c9a93a, 33 },
    { 0xdcf9a93b, 43 },
    { 0, 0 },
    { 0xf1b9a93d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c9a941, 61 },
    { 0x5009a941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1699a948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9a94b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d9a950, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a9a954, 91 },
    { 0xcda9a955, 77 },
    { 0x5d89a956, 59 },
    { 0xadb9a957, 46 },
    { 0x6c69a956, 33 },
    { 0xf419a959, 65 },
    { 0, 0 },
    { 0x2f09a95b, 76 },
    { 0xb1b9a95b, 55 },
    { 0x37b9a95d, 90 },
    { 0, 0 },
    { 0xf9c9a95f, 93 },
    { 0x8599a960, 46 },
    { 0xda9a95f, 98 },
    { 0xaf09a962, 61 },
    { 0, 0 },
    { 0x69a9a964, 95 },
    { 0x8929a965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c19a968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6569a96b, 76 },
    { 0, 0 },
    { 0xc229a96d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a89a971, 90 },
    { 0xd749a972, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d99a976, 90 },
    { 0x1d99a976, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd9a97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9a97e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c9a983, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd469a988, 43 },
    { 0x8c09a989, 90 },
    { 0x3589a98a, 23 },
    { 0xd379a98b, 55 },
    { 0xc4d9a98c, 43 },
    { 0x15d9a98b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa189a991, 98 },
    { 0, 0 },
    { 0x8a19a993, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c9a998, 78 },
    { 0x7db9a999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb9a99d, 88 },
    { 0, 0 },
    { 0x7de9a99f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1909a9a3, 85 },
    { 0x3b29a9a3, 93 },
    { 0x5ed9a9a5, 34 },
    { 0x4109a9a6, 33 },
    { 0x7ab9a9a7, 95 },
    { 0x2b19a9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf409a9ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe679a9af, 90 },
    { 0, 0 },
    { 0xd929a9b1, 23 },
    { 0, 0 },
    { 0x8719a9b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1389a9b6, 61 },
    { 0, 0 },
    { 0xd779a9b8, 33 },
    { 0x7fa9a9b8, 84 },
    { 0xb899a9ba, 61 },
    { 0xe7f9a9bb, 90 },
    { 0xf6d9a9bc, 33 },
    { 0x3829a9bc, 33 },
    { 0x10c9a9bd, 78 },
    { 0x8fb9a9bd, 61 },
    { 0x4829a9bd, 66 },
    { 0xb569a9bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x379a9c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa719a9cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb499a9d4, 95 },
    { 0xcb29a9d5, 95 },
    { 0x9a89a9d6, 66 },
    { 0xef99a9d6, 87 },
    { 0xe109a9d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf559a9de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8209a9e4, 33 },
    { 0x9d59a9e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b9a9f3, 81 },
    { 0, 0 },
    { 0x7d09a9f5, 61 },
    { 0x3099a9f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc249a9fc, 75 },
    { 0xe649a9fd, 76 },
    { 0xad19a9fe, 83 },
    { 0x5269a9ff, 76 },
    { 0x11a9a9ff, 33 },
    { 0x5ce9aa01, 91 },
    { 0x88f9a9ff, 90 },
    { 0, 0 },
    { 0xc949aa04, 55 },
    { 0xf4a9aa05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9aa08, 61 },
    { 0, 0 },
    { 0xa539aa0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9aa0e, 43 },
    { 0, 0 },
    { 0xfc99aa10, 46 },
    { 0, 0 },
    { 0xebb9aa12, 90 },
    { 0x2bb9aa13, 84 },
    { 0x7f59aa13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569aa19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c9aa1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a9aa20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239aa23, 90 },
    { 0x7429aa23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59aa27, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa69aa2b, 33 },
    { 0x5db9aa2b, 60 },
    { 0, 0 },
    { 0x9c89aa2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e49aa32, 90 },
    { 0xde39aa33, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359aa36, 61 },
    { 0x6c09aa37, 55 },
    { 0x8799aa36, 33 },
    { 0x8b19aa39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29aa42, 43 },
    { 0, 0 },
    { 0x3e69aa44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd69aa47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c39aa4e, 95 },
    { 0, 0 },
    { 0xf109aa50, 53 },
    { 0x2539aa51, 81 },
    { 0xd589aa51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3919aa56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409aa5c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69aa65, 93 },
    { 0x3069aa66, 95 },
    { 0x3f79aa66, 98 },
    { 0, 0 },
    { 0x7c9aa69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109aa6e, 55 },
    { 0x8789aa6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9aa78, 61 },
    { 0, 0 },
    { 0xbe59aa7a, 100 },
    { 0xa49aa7b, 61 },
    { 0xd9a9aa7b, 90 },
    { 0x199aa7b, 34 },
    { 0xdac9aa7d, 98 },
    { 0xa0a9aa7a, 90 },
    { 0, 0 },
    { 0xb789aa81, 33 },
    { 0xc529aa82, 23 },
    { 0x71e9aa83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3429aa87, 66 },
    { 0, 0 },
    { 0x3679aa89, 88 },
    { 0, 0 },
    { 0x9789aa8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899aa8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9aa91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c9aa95, 55 },
    { 0, 0 },
    { 0x9389aa97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb49aa9b, 33 },
    { 0, 0 },
    { 0x1099aa9d, 90 },
    { 0x4bf9aa9e, 61 },
    { 0x2d9aa9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf039aaa3, 98 },
    { 0xbf49aaa4, 46 },
    { 0xa469aaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb79aaa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c9aaae, 98 },
    { 0x87f9aaae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399aab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9aaba, 66 },
    { 0, 0 },
    { 0xdd59aabc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d79aac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe119aac5, 33 },
    { 0xe989aac5, 61 },
    { 0xf7c9aac7, 52 },
    { 0, 0 },
    { 0x1029aac9, 90 },
    { 0xe729aaca, 95 },
    { 0x55a9aacb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb49aacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379aada, 23 },
    { 0x4d19aadb, 61 },
    { 0x8279aadc, 90 },
    { 0, 0 },
    { 0x4c79aade, 66 },
    { 0, 0 },
    { 0xa2d9aae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb09aae3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f9aae6, 23 },
    { 0, 0 },
    { 0xc559aae8, 33 },
    { 0xa3b9aae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2109aaf2, 75 },
    { 0, 0 },
    { 0x8dc9aaf4, 33 },
    { 0, 0 },
    { 0x7599aaf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779aaf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec59aafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa39ab07, 90 },
    { 0x94f9ab08, 100 },
    { 0, 0 },
    { 0x469ab0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf539ab0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f89ab14, 52 },
    { 0x4049ab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d19ab19, 33 },
    { 0, 0 },
    { 0x7779ab1b, 90 },
    { 0, 0 },
    { 0x7eb9ab1d, 43 },
    { 0x79e9ab1e, 34 },
    { 0x8769ab1f, 33 },
    { 0xf219ab20, 22 },
    { 0, 0 },
    { 0x1ba9ab22, 78 },
    { 0, 0 },
    { 0xec49ab24, 61 },
    { 0, 0 },
    { 0x2fd9ab26, 34 },
    { 0xe399ab26, 98 },
    { 0, 0 },
    { 0x3d49ab29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b9ab2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3409ab32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9ab39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1119ab43, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a79ab46, 61 },
    { 0xf469ab47, 78 },
    { 0, 0 },
    { 0xbac9ab49, 61 },
    { 0x66b9ab4a, 61 },
    { 0x9d49ab4a, 55 },
    { 0x7569ab4c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd179ab55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c9ab5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca29ab66, 69 },
    { 0x6b89ab67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ab6a, 43 },
    { 0, 0 },
    { 0xac49ab6c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49ab6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a9ab7b, 94 },
    { 0, 0 },
    { 0x8149ab7d, 90 },
    { 0x9349ab7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c9ab83, 91 },
    { 0, 0 },
    { 0xcda9ab85, 17 },
    { 0x12e9ab85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449ab8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c09ab90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a9ab96, 90 },
    { 0xee19ab97, 91 },
    { 0x1fa9ab98, 78 },
    { 0x99a9ab98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe999ab9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299aba5, 85 },
    { 0xd5a9aba5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb289aba9, 34 },
    { 0x2d59abaa, 76 },
    { 0x2509abab, 90 },
    { 0x32a9abaa, 98 },
    { 0xc509abab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe29abb2, 17 },
    { 0, 0 },
    { 0xe859abb4, 67 },
    { 0xe519abb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9abbc, 52 },
    { 0x8fd9abbd, 98 },
    { 0xb879abbe, 85 },
    { 0x3c49abbe, 33 },
    { 0, 0 },
    { 0x20b9abc1, 46 },
    { 0xb289abc2, 98 },
    { 0x1749abc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa849abc7, 33 },
    { 0x41d9abc7, 98 },
    { 0x7e69abc9, 90 },
    { 0, 0 },
    { 0x9829abcb, 76 },
    { 0, 0 },
    { 0x3599abcd, 76 },
    { 0xf079abce, 82 },
    { 0, 0 },
    { 0x5bc9abd0, 33 },
    { 0, 0 },
    { 0x8959abd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2579abd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9abdc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a9abdf, 33 },
    { 0xb19abdf, 23 },
    { 0xcab9abdf, 90 },
    { 0x1009abe2, 95 },
    { 0x45a9abe2, 90 },
    { 0xe809abe4, 90 },
    { 0, 0 },
    { 0xf5a9abe6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169abe9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee49abec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de9abf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9abf3, 55 },
    { 0, 0 },
    { 0x6969abf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb369abfc, 52 },
    { 0x8859abfc, 90 },
    { 0, 0 },
    { 0x9559abff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3039ac02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c9ac06, 61 },
    { 0, 0 },
    { 0xd6c9ac08, 55 },
    { 0, 0 },
    { 0xcc29ac0a, 61 },
    { 0x2b09ac0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19ac0e, 46 },
    { 0x39a9ac0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d9ac16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4509ac1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a9ac20, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5749ac23, 33 },
    { 0, 0 },
    { 0x46b9ac25, 93 },
    { 0x8c89ac26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b9ac2d, 90 },
    { 0x2469ac2e, 84 },
    { 0, 0 },
    { 0x7359ac30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439ac34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d9ac3d, 61 },
    { 0, 0 },
    { 0xb319ac3f, 90 },
    { 0x1f69ac40, 33 },
    { 0, 0 },
    { 0xd009ac42, 78 },
    { 0x1079ac43, 90 },
    { 0, 0 },
    { 0x95e9ac45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9ac48, 90 },
    { 0xc599ac48, 95 },
    { 0xd4e9ac49, 95 },
    { 0xb0e9ac4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf229ac4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd269ac51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4149ac5c, 98 },
    { 0x1629ac5d, 61 },
    { 0, 0 },
    { 0x62e9ac5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299ac62, 61 },
    { 0x3079ac63, 85 },
    { 0x1c89ac62, 98 },
    { 0, 0 },
    { 0xfe49ac66, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d79ac6a, 46 },
    { 0x9a69ac6b, 52 },
    { 0x13d9ac6b, 87 },
    { 0xa329ac6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa29ac71, 33 },
    { 0x82c9ac72, 61 },
    { 0x2fb9ac72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89ac7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69ac82, 23 },
    { 0, 0 },
    { 0x93a9ac84, 95 },
    { 0xe789ac85, 23 },
    { 0x7a99ac86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9ac8c, 76 },
    { 0xa919ac8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7469ac91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c9ac97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99ac9a, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09ac9d, 87 },
    { 0x5f39ac9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f9aca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c9aca6, 84 },
    { 0x42a9aca6, 43 },
    { 0x5c89aca8, 61 },
    { 0, 0 },
    { 0x3199acaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b9acae, 95 },
    { 0x2349acaf, 90 },
    { 0x2fb9acb0, 55 },
    { 0x9f89acb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a9acb7, 46 },
    { 0, 0 },
    { 0xd3a9acb9, 90 },
    { 0xfd9acb9, 23 },
    { 0xf5a9acba, 98 },
    { 0x4299acbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99acbf, 90 },
    { 0x6e49acc0, 23 },
    { 0xa909acc1, 23 },
    { 0, 0 },
    { 0xc809acc3, 67 },
    { 0, 0 },
    { 0xfa09acc5, 23 },
    { 0, 0 },
    { 0xc29acc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e9accc, 33 },
    { 0x12c9accd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa959acd2, 78 },
    { 0, 0 },
    { 0xa0b9acd4, 91 },
    { 0x3eb9acd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719acd8, 29 },
    { 0x4339acd8, 77 },
    { 0, 0 },
    { 0x1709acdb, 61 },
    { 0x2b59acdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4729ace4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8109ace7, 76 },
    { 0xd119ace8, 45 },
    { 0xa4f9ace9, 81 },
    { 0, 0 },
    { 0x7959aceb, 88 },
    { 0x4549aceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9acf2, 61 },
    { 0, 0 },
    { 0xc759acf4, 23 },
    { 0, 0 },
    { 0x1a49acf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9689acfb, 90 },
    { 0x41e9acfb, 93 },
    { 0, 0 },
    { 0xb439acfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129ad02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7119ad0b, 33 },
    { 0, 0 },
    { 0x8eb9ad0d, 33 },
    { 0, 0 },
    { 0x3e89ad0f, 90 },
    { 0x4cf9ad10, 61 },
    { 0, 0 },
    { 0x1349ad12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f9ad17, 61 },
    { 0x5fa9ad18, 90 },
    { 0, 0 },
    { 0x4969ad1a, 90 },
    { 0x7f39ad1b, 61 },
    { 0x1a39ad1c, 95 },
    { 0xca19ad1a, 58 },
    { 0x7009ad1e, 83 },
    { 0, 0 },
    { 0xaa09ad20, 43 },
    { 0xa809ad21, 90 },
    { 0xd299ad20, 98 },
    { 0xe859ad23, 34 },
    { 0, 0 },
    { 0xf089ad25, 23 },
    { 0, 0 },
    { 0xcbb9ad27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939ad2c, 86 },
    { 0x2e59ad2c, 90 },
    { 0x26a9ad2d, 61 },
    { 0, 0 },
    { 0x29d9ad30, 95 },
    { 0x3249ad30, 61 },
    { 0xcf59ad30, 23 },
    { 0xaad9ad33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9359ad36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf39ad3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e49ad3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7769ad43, 61 },
    { 0xe4c9ad44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a9ad49, 34 },
    { 0x94e9ad49, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba9ad4e, 33 },
    { 0x799ad4e, 76 },
    { 0, 0 },
    { 0x4c89ad51, 99 },
    { 0, 0 },
    { 0x4af9ad53, 90 },
    { 0x43a9ad54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2349ad59, 33 },
    { 0xa709ad5a, 90 },
    { 0x6ef9ad5b, 98 },
    { 0x4749ad5b, 33 },
    { 0x5069ad5d, 77 },
    { 0xe69ad5d, 95 },
    { 0xe859ad5c, 66 },
    { 0x8b69ad60, 23 },
    { 0x3339ad61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee39ad64, 76 },
    { 0xda69ad65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf59ad69, 46 },
    { 0x9569ad6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf9ad6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859ad73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc79ad7b, 95 },
    { 0x9b19ad7b, 90 },
    { 0, 0 },
    { 0x5c39ad7e, 95 },
    { 0x9329ad7e, 93 },
    { 0x949ad80, 76 },
    { 0x8c29ad7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b9ad86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5619ad92, 90 },
    { 0x38a9ad93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9089ad99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489ad9f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9ada3, 99 },
    { 0x3239ada4, 43 },
    { 0xa369ada4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759ada8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d9adba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb409adbd, 93 },
    { 0xc579adbe, 40 },
    { 0x7b79adbf, 90 },
    { 0x8989adbd, 98 },
    { 0x9fc9adc1, 34 },
    { 0x5b99adc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19adc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9adce, 95 },
    { 0xe969adcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229add9, 22 },
    { 0x3519adda, 84 },
    { 0, 0 },
    { 0x5649addc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199addf, 23 },
    { 0xc39ade0, 76 },
    { 0x6fd9ade0, 95 },
    { 0x7619ade2, 84 },
    { 0x90f9ade2, 90 },
    { 0, 0 },
    { 0xd099ade5, 78 },
    { 0x8e99ade5, 34 },
    { 0x7029ade7, 95 },
    { 0x3be9ade8, 98 },
    { 0x7a19ade9, 90 },
    { 0x8389ade9, 34 },
    { 0x87c9ade7, 61 },
    { 0xaae9ade6, 90 },
    { 0, 0 },
    { 0x61f9adee, 34 },
    { 0x6ca9adee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe699adf3, 90 },
    { 0xb179adf4, 95 },
    { 0x4db9adf4, 33 },
    { 0xeb89adf6, 55 },
    { 0x59adf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69adfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249ae00, 76 },
    { 0, 0 },
    { 0xe7d9ae02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb789ae05, 84 },
    { 0, 0 },
    { 0xb7e9ae07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f29ae0a, 61 },
    { 0, 0 },
    { 0xaab9ae0c, 34 },
    { 0xa729ae0d, 81 },
    { 0x5539ae0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd9ae12, 98 },
    { 0x2b19ae13, 90 },
    { 0x5e29ae12, 95 },
    { 0, 0 },
    { 0xff79ae16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5729ae1e, 52 },
    { 0x1029ae1f, 95 },
    { 0xb49ae1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5459ae25, 98 },
    { 0, 0 },
    { 0x2279ae27, 33 },
    { 0, 0 },
    { 0x4ae9ae29, 95 },
    { 0, 0 },
    { 0xd369ae2b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2589ae33, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9ae36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9ae3b, 34 },
    { 0, 0 },
    { 0xff19ae3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec9ae40, 81 },
    { 0x5f29ae41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b79ae46, 90 },
    { 0, 0 },
    { 0x3969ae48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd9ae4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979ae53, 67 },
    { 0, 0 },
    { 0x6439ae55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3989ae66, 85 },
    { 0x2ac9ae67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb9ae6d, 67 },
    { 0x6219ae6e, 76 },
    { 0xe179ae6f, 33 },
    { 0x56c9ae70, 76 },
    { 0x4cb9ae71, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b9ae75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8609ae7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf69ae83, 33 },
    { 0x49b9ae83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd9ae8d, 90 },
    { 0, 0 },
    { 0x1919ae8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a9ae92, 93 },
    { 0, 0 },
    { 0x7779ae94, 98 },
    { 0x23a9ae94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e9ae9c, 23 },
    { 0xc819ae9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa209aea0, 90 },
    { 0x8699aea1, 90 },
    { 0x58e9aea1, 23 },
    { 0, 0 },
    { 0xab59aea4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc719aea9, 90 },
    { 0x509aeaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd49aeae, 52 },
    { 0xd2b9aeaf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9aeb2, 65 },
    { 0xf3e9aeb2, 66 },
    { 0x3059aeb4, 53 },
    { 0x7a59aeb5, 23 },
    { 0x9f29aeb6, 23 },
    { 0xfbe9aeb7, 23 },
    { 0xb769aeb7, 46 },
    { 0xd659aeb8, 55 },
    { 0x5639aeb8, 90 },
    { 0, 0 },
    { 0x2969aebc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8229aec0, 61 },
    { 0x3a59aec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b89aec6, 76 },
    { 0x9409aec7, 87 },
    { 0x7e09aec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9aed4, 98 },
    { 0, 0 },
    { 0x4cb9aed6, 31 },
    { 0, 0 },
    { 0xa449aed8, 76 },
    { 0, 0 },
    { 0xec59aeda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49aedd, 80 },
    { 0, 0 },
    { 0xf599aedf, 90 },
    { 0xc3f9aedf, 17 },
    { 0xf3c9aee1, 52 },
    { 0, 0 },
    { 0x49d9aee3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9799aee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde69aeeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d19aeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489aef2, 66 },
    { 0, 0 },
    { 0x9329aef4, 55 },
    { 0x5699aef4, 95 },
    { 0xacf9aef6, 76 },
    { 0xd6c9aef6, 90 },
    { 0x6179aef6, 23 },
    { 0xa939aef9, 46 },
    { 0, 0 },
    { 0x5a49aefb, 34 },
    { 0, 0 },
    { 0xe9d9aefd, 55 },
    { 0x1989aefe, 33 },
    { 0x5ae9aeff, 61 },
    { 0x4989af00, 23 },
    { 0x9de9af00, 46 },
    { 0xece9af01, 78 },
    { 0x8dd9af03, 64 },
    { 0x4b59af00, 95 },
    { 0x4929af05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99af08, 87 },
    { 0xaeb9af09, 33 },
    { 0x5e79af0a, 61 },
    { 0, 0 },
    { 0x3509af0c, 98 },
    { 0xbf59af0d, 90 },
    { 0xbd39af0d, 69 },
    { 0xb949af0c, 33 },
    { 0, 0 },
    { 0xf719af11, 84 },
    { 0, 0 },
    { 0x6419af13, 34 },
    { 0, 0 },
    { 0xb139af15, 61 },
    { 0xddc9af16, 90 },
    { 0xf6e9af16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa009af1e, 34 },
    { 0xa889af1f, 33 },
    { 0x8c89af20, 78 },
    { 0xe4d9af20, 98 },
    { 0xa8c9af1f, 90 },
    { 0xe869af20, 100 },
    { 0x43a9af1f, 34 },
    { 0x82b9af25, 34 },
    { 0xac19af26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed9af2a, 40 },
    { 0, 0 },
    { 0xbc79af2c, 76 },
    { 0x2609af2d, 33 },
    { 0, 0 },
    { 0xf0b9af2f, 91 },
    { 0xb759af2f, 96 },
    { 0x14c9af31, 97 },
    { 0, 0 },
    { 0xd6d9af33, 90 },
    { 0x7a29af33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439af3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c9af3e, 78 },
    { 0xf7e9af3f, 34 },
    { 0x9ba9af3e, 66 },
    { 0xe409af41, 43 },
    { 0xb599af42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769af49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49af4d, 87 },
    { 0x3989af4d, 40 },
    { 0x2649af4f, 85 },
    { 0x7bb9af50, 95 },
    { 0, 0 },
    { 0xbed9af52, 90 },
    { 0, 0 },
    { 0x4309af54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe909af5a, 77 },
    { 0x8f9af5b, 77 },
    { 0x39a9af5c, 77 },
    { 0x3679af5d, 61 },
    { 0x7da9af5e, 93 },
    { 0xded9af5e, 78 },
    { 0xbf39af60, 23 },
    { 0x4919af5d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269af67, 33 },
    { 0xa4c9af68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b9af6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2969af6f, 90 },
    { 0, 0 },
    { 0x3409af71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9af75, 99 },
    { 0xe8e9af76, 23 },
    { 0xb709af76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d9af7a, 90 },
    { 0, 0 },
    { 0xf639af7c, 33 },
    { 0, 0 },
    { 0x1de9af7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd39af82, 90 },
    { 0, 0 },
    { 0x5389af84, 23 },
    { 0xe579af85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb9af88, 98 },
    { 0, 0 },
    { 0x2ca9af8a, 76 },
    { 0xbc89af8a, 34 },
    { 0x4d99af8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9af91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729af94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf729af97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869afa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a9afa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3499afaa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c59afae, 98 },
    { 0x7e59afae, 84 },
    { 0x6719afb0, 95 },
    { 0, 0 },
    { 0x5459afb2, 67 },
    { 0x9df9afb3, 33 },
    { 0xb269afb4, 36 },
    { 0x1e29afb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9afb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f9afbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece9afc4, 95 },
    { 0, 0 },
    { 0xf949afc6, 98 },
    { 0xa529afc7, 98 },
    { 0x3259afc8, 52 },
    { 0x37c9afc9, 23 },
    { 0x23c9afca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9849afcd, 90 },
    { 0x9a89afcd, 93 },
    { 0, 0 },
    { 0xcb99afd0, 34 },
    { 0xf379afd1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89afd4, 75 },
    { 0, 0 },
    { 0xfca9afd6, 76 },
    { 0x2f9afd7, 34 },
    { 0xa1b9afd8, 95 },
    { 0xd849afd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2479afe0, 99 },
    { 0x4ae9afe0, 95 },
    { 0xb4b9afe2, 33 },
    { 0x29d9afe3, 84 },
    { 0x9c89afe1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b9afe9, 76 },
    { 0x5cb9afea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a09aff0, 98 },
    { 0, 0 },
    { 0x6b09aff2, 87 },
    { 0, 0 },
    { 0x54f9aff4, 90 },
    { 0, 0 },
    { 0x2d59aff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a09affd, 33 },
    { 0xc549affe, 23 },
    { 0xb459afff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa69b003, 44 },
    { 0, 0 },
    { 0xc759b005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf79b008, 90 },
    { 0xa209b009, 33 },
    { 0, 0 },
    { 0x53a9b00b, 39 },
    { 0xda19b00c, 34 },
    { 0xb3a9b00c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b49b014, 40 },
    { 0x98f9b014, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9b01a, 23 },
    { 0x7339b01b, 93 },
    { 0x7769b01a, 90 },
    { 0x97d9b01c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099b021, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b49b026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e79b02d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939b031, 98 },
    { 0, 0 },
    { 0xf1c9b033, 33 },
    { 0, 0 },
    { 0xf429b035, 87 },
    { 0x3219b036, 90 },
    { 0xd029b037, 98 },
    { 0xa619b038, 76 },
    { 0x94e9b038, 52 },
    { 0xc1f9b039, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb9b03e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119b048, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d9b04b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6289b050, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f59b053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf99b057, 78 },
    { 0x20f9b058, 93 },
    { 0x5969b057, 61 },
    { 0xd6a9b05a, 90 },
    { 0x1eb9b05b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f39b062, 61 },
    { 0x60e9b063, 61 },
    { 0x40a9b064, 98 },
    { 0, 0 },
    { 0x68d9b066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa9b069, 88 },
    { 0x2e29b06a, 95 },
    { 0xcd09b06b, 52 },
    { 0x2e19b06b, 84 },
    { 0, 0 },
    { 0x1b39b06e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e9b071, 24 },
    { 0x9809b071, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb9b075, 98 },
    { 0, 0 },
    { 0x3fb9b077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1899b07d, 23 },
    { 0xd439b07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129b082, 90 },
    { 0x18c9b082, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf759b087, 85 },
    { 0x88f9b087, 78 },
    { 0x6859b088, 23 },
    { 0xf419b08a, 33 },
    { 0x7fe9b08a, 61 },
    { 0x3459b08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c59b090, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819b094, 33 },
    { 0xb9e9b095, 61 },
    { 0, 0 },
    { 0x7d39b097, 61 },
    { 0xe669b098, 90 },
    { 0x69c9b099, 61 },
    { 0xb369b09a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb699b09d, 98 },
    { 0x2c29b09e, 98 },
    { 0xa369b09e, 61 },
    { 0x18a9b09d, 33 },
    { 0, 0 },
    { 0x3e49b0a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba9b0a5, 61 },
    { 0x46c9b0a6, 46 },
    { 0xb899b0a7, 95 },
    { 0, 0 },
    { 0xedd9b0a9, 90 },
    { 0, 0 },
    { 0x6ee9b0ab, 100 },
    { 0xd309b0ac, 43 },
    { 0xdb39b0ad, 55 },
    { 0, 0 },
    { 0x7739b0af, 61 },
    { 0x6259b0b0, 98 },
    { 0x73a9b0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd999b0b5, 23 },
    { 0x939b0b5, 33 },
    { 0x7439b0b7, 98 },
    { 0xced9b0b6, 43 },
    { 0, 0 },
    { 0x57d9b0ba, 81 },
    { 0xd709b0bb, 90 },
    { 0xb799b0bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169b0c0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4179b0c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd449b0c8, 33 },
    { 0x7f69b0c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489b0cc, 77 },
    { 0xf6d9b0cd, 95 },
    { 0x3a69b0ce, 19 },
    { 0x72a9b0cc, 23 },
    { 0x6309b0d0, 23 },
    { 0xf5b9b0d1, 61 },
    { 0xe3f9b0d2, 23 },
    { 0x4eb9b0d3, 46 },
    { 0x8db9b0d4, 61 },
    { 0x78d9b0d3, 90 },
    { 0x3589b0d6, 52 },
    { 0xb339b0d5, 55 },
    { 0x81b9b0d5, 95 },
    { 0x2269b0d9, 61 },
    { 0xeb39b0da, 61 },
    { 0x9569b0d9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf9b0de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b39b0e5, 100 },
    { 0x44f9b0e6, 46 },
    { 0x9b19b0e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf69b0ec, 91 },
    { 0, 0 },
    { 0x56b9b0ee, 61 },
    { 0x8279b0ef, 76 },
    { 0x4679b0ee, 66 },
    { 0x6f09b0f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b9b0fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd9b102, 61 },
    { 0, 0 },
    { 0x90b9b104, 61 },
    { 0, 0 },
    { 0xabd9b106, 85 },
    { 0xe7a9b107, 98 },
    { 0xa0b9b108, 46 },
    { 0, 0 },
    { 0xa479b10a, 33 },
    { 0x5aa9b10a, 81 },
    { 0xb7a9b10a, 90 },
    { 0, 0 },
    { 0x2c49b10e, 76 },
    { 0x4e59b10f, 100 },
    { 0xc679b10f, 61 },
    { 0, 0 },
    { 0x489b112, 52 },
    { 0xdcb9b113, 87 },
    { 0xfa09b114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce39b118, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc9b11d, 76 },
    { 0x4319b11d, 33 },
    { 0xa739b11d, 55 },
    { 0xaa79b120, 91 },
    { 0xc569b11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89b124, 61 },
    { 0x4029b125, 34 },
    { 0, 0 },
    { 0x6d99b127, 95 },
    { 0x8989b128, 81 },
    { 0, 0 },
    { 0x3509b12a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb9b12f, 68 },
    { 0xc7a9b130, 33 },
    { 0x959b130, 23 },
    { 0x6d89b12f, 23 },
    { 0xac59b133, 33 },
    { 0x6529b130, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29b138, 33 },
    { 0x8c19b138, 23 },
    { 0x72b9b13a, 34 },
    { 0x97b9b13b, 55 },
    { 0xeda9b13c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529b140, 89 },
    { 0x2079b141, 33 },
    { 0x1a29b141, 34 },
    { 0xdd39b143, 90 },
    { 0xe2f9b143, 52 },
    { 0xdb59b145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6089b148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c69b151, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429b155, 52 },
    { 0xfe9b156, 90 },
    { 0x1359b155, 61 },
    { 0x8c89b158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x159b15b, 33 },
    { 0, 0 },
    { 0xa2c9b15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b162, 90 },
    { 0x42b9b163, 84 },
    { 0x4259b164, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df9b16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba9b16f, 91 },
    { 0xfea9b170, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d09b173, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9159b17b, 75 },
    { 0x84e9b17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f9b183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a9b186, 23 },
    { 0x75c9b187, 76 },
    { 0, 0 },
    { 0xac29b189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9b18c, 17 },
    { 0x30a9b18c, 61 },
    { 0, 0 },
    { 0xa3c9b18f, 95 },
    { 0x1239b190, 90 },
    { 0xa6b9b191, 46 },
    { 0xb3f9b191, 95 },
    { 0, 0 },
    { 0xebf9b194, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe209b197, 52 },
    { 0x4e29b198, 25 },
    { 0xe949b199, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a69b19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089b1a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909b1a6, 90 },
    { 0xd609b1a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b9b1b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e19b1b6, 99 },
    { 0xc7c9b1b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489b1ba, 91 },
    { 0xf7e9b1ba, 55 },
    { 0x7539b1bc, 61 },
    { 0x6239b1ba, 95 },
    { 0, 0 },
    { 0x48f9b1bf, 65 },
    { 0x17a9b1c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c9b1c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe9b1c9, 84 },
    { 0x5329b1ca, 93 },
    { 0, 0 },
    { 0x8419b1cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9889b1d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8449b1d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9b1db, 99 },
    { 0xd019b1dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199b1df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c39b1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9b1e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569b1ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe29b1f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de9b1fa, 61 },
    { 0, 0 },
    { 0xb599b1fc, 98 },
    { 0x5bc9b1fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639b203, 34 },
    { 0xc199b203, 55 },
    { 0xa39b204, 100 },
    { 0xb949b206, 90 },
    { 0xd159b205, 75 },
    { 0x5da9b204, 95 },
    { 0x36e9b207, 46 },
    { 0x7559b204, 61 },
    { 0x10c9b20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3389b210, 55 },
    { 0, 0 },
    { 0x3aa9b212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a9b215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc579b218, 98 },
    { 0xce09b219, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b9b21c, 76 },
    { 0, 0 },
    { 0xf559b21e, 33 },
    { 0, 0 },
    { 0x4739b220, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5359b225, 55 },
    { 0xfb49b226, 96 },
    { 0x7239b227, 55 },
    { 0x5679b228, 33 },
    { 0xd239b226, 98 },
    { 0x5999b22a, 90 },
    { 0xcf09b22a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519b22e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5039b233, 33 },
    { 0x289b233, 93 },
    { 0, 0 },
    { 0x3ee9b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609b239, 95 },
    { 0x4309b23a, 66 },
    { 0x409b23b, 33 },
    { 0, 0 },
    { 0x979b23d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b39b245, 33 },
    { 0, 0 },
    { 0xff79b247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1179b24a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199b24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9b251, 68 },
    { 0x9699b252, 76 },
    { 0x27d9b253, 33 },
    { 0xb419b254, 61 },
    { 0, 0 },
    { 0xe239b256, 61 },
    { 0x51e9b256, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc9b25a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e9b261, 98 },
    { 0, 0 },
    { 0x2dd9b263, 65 },
    { 0xead9b263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb569b269, 90 },
    { 0, 0 },
    { 0xe149b26b, 90 },
    { 0x7419b26b, 33 },
    { 0xc599b26d, 23 },
    { 0, 0 },
    { 0xb7f9b26f, 93 },
    { 0xd629b270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9759b275, 61 },
    { 0xcbb9b275, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649b282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9b285, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9b288, 95 },
    { 0x8d59b289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9b28e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd799b291, 23 },
    { 0x60c9b292, 98 },
    { 0, 0 },
    { 0xa349b294, 33 },
    { 0xfc89b295, 43 },
    { 0x26b9b296, 33 },
    { 0, 0 },
    { 0x3ef9b298, 78 },
    { 0, 0 },
    { 0x35d9b29a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139b2a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179b2a7, 78 },
    { 0x75e9b2a8, 90 },
    { 0xdf29b2a7, 95 },
    { 0x87b9b2aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e9b2af, 55 },
    { 0x7f99b2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a49b2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc39b2b9, 23 },
    { 0x9119b2ba, 61 },
    { 0x3199b2b9, 100 },
    { 0, 0 },
    { 0x53d9b2bd, 90 },
    { 0xc7c9b2bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9b2c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc959b2c6, 36 },
    { 0x9dd9b2c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa369b2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139b2d4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9b2dd, 76 },
    { 0, 0 },
    { 0xe1a9b2df, 43 },
    { 0, 0 },
    { 0x6d19b2e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c99b2e5, 93 },
    { 0, 0 },
    { 0xaa99b2e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e09b2ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae09b2f4, 46 },
    { 0xb149b2f5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb249b2f8, 81 },
    { 0x9bc9b2f8, 33 },
    { 0x6409b2fa, 43 },
    { 0x94f9b2fb, 90 },
    { 0x8719b2f9, 61 },
    { 0x729b2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc439b306, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb9b30b, 43 },
    { 0, 0 },
    { 0xdfa9b30d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5969b312, 52 },
    { 0xd539b313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9b31d, 78 },
    { 0, 0 },
    { 0xab39b31f, 87 },
    { 0, 0 },
    { 0xa689b321, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5789b325, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8829b329, 22 },
    { 0xb849b32a, 55 },
    { 0x4b49b32b, 64 },
    { 0, 0 },
    { 0x96a9b32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a39b331, 90 },
    { 0x5a69b332, 100 },
    { 0x9419b333, 98 },
    { 0x6689b334, 55 },
    { 0x44e9b333, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9b344, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde9b348, 46 },
    { 0, 0 },
    { 0xc589b34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99b34d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29b353, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd9b357, 78 },
    { 0, 0 },
    { 0xea99b359, 33 },
    { 0, 0 },
    { 0xb929b35b, 93 },
    { 0, 0 },
    { 0xcaf9b35d, 81 },
    { 0x5309b35e, 61 },
    { 0, 0 },
    { 0x1bd9b360, 23 },
    { 0xbe9b361, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519b364, 58 },
    { 0x73e9b365, 33 },
    { 0, 0 },
    { 0x7499b367, 61 },
    { 0xdc59b367, 95 },
    { 0xcc49b369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119b36d, 97 },
    { 0x6d19b36e, 55 },
    { 0x7519b36e, 34 },
    { 0x2d9b370, 23 },
    { 0, 0 },
    { 0x4d09b372, 33 },
    { 0xd9c9b372, 34 },
    { 0, 0 },
    { 0x4ea9b375, 55 },
    { 0, 0 },
    { 0x1b99b377, 81 },
    { 0, 0 },
    { 0x1b49b379, 98 },
    { 0x82f9b379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9b37d, 33 },
    { 0x83c9b37d, 98 },
    { 0x2a49b37f, 90 },
    { 0xeb69b380, 23 },
    { 0, 0 },
    { 0xcf49b382, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c99b386, 77 },
    { 0xc0d9b387, 90 },
    { 0, 0 },
    { 0xc5d9b389, 76 },
    { 0x7909b38a, 95 },
    { 0x49b38b, 90 },
    { 0x9549b38c, 95 },
    { 0xe39b38a, 34 },
    { 0x3a79b38e, 61 },
    { 0x4369b38f, 95 },
    { 0x9939b38c, 98 },
    { 0x5bf9b38c, 55 },
    { 0x1eb9b392, 95 },
    { 0x4939b392, 95 },
    { 0xdf59b392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde89b397, 43 },
    { 0, 0 },
    { 0x8089b399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9b3a0, 95 },
    { 0x7049b3a1, 53 },
    { 0, 0 },
    { 0x96f9b3a3, 33 },
    { 0xdb59b3a3, 76 },
    { 0, 0 },
    { 0x55a9b3a6, 95 },
    { 0, 0 },
    { 0x8779b3a8, 65 },
    { 0xcbb9b3a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db9b3af, 17 },
    { 0x5239b3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9659b3b3, 33 },
    { 0xeaa9b3b4, 95 },
    { 0x8919b3b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb549b3b8, 33 },
    { 0, 0 },
    { 0xb6f9b3ba, 33 },
    { 0, 0 },
    { 0x62a9b3bc, 95 },
    { 0xc799b3bd, 90 },
    { 0xf989b3be, 64 },
    { 0, 0 },
    { 0xeae9b3c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59b3c5, 43 },
    { 0, 0 },
    { 0xb639b3c7, 98 },
    { 0x2249b3c7, 23 },
    { 0xb8f9b3c9, 95 },
    { 0x7e89b3ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949b3cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479b3d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9b3d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9b3dc, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee9b3e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c9b3e4, 66 },
    { 0x5b99b3e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe489b3ee, 99 },
    { 0, 0 },
    { 0x1ad9b3f0, 46 },
    { 0x4d69b3f1, 67 },
    { 0x4189b3f2, 43 },
    { 0x3589b3f3, 98 },
    { 0, 0 },
    { 0xcfc9b3f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9b3f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9b401, 33 },
    { 0xe9c9b402, 90 },
    { 0x7489b402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd9b409, 76 },
    { 0xe3c9b40a, 98 },
    { 0xa6d9b40b, 98 },
    { 0, 0 },
    { 0x1579b40d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce9b410, 88 },
    { 0xf419b411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf159b415, 33 },
    { 0xe509b416, 33 },
    { 0, 0 },
    { 0xea99b418, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519b41c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf739b41f, 66 },
    { 0, 0 },
    { 0xf889b421, 69 },
    { 0, 0 },
    { 0x2e9b423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8689b42a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39b42e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369b43d, 95 },
    { 0x6319b43d, 61 },
    { 0x1d39b43e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9b442, 95 },
    { 0, 0 },
    { 0x1a29b444, 61 },
    { 0xfcb9b445, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9b449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf9b44d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9b453, 98 },
    { 0x5a9b454, 53 },
    { 0x4029b455, 84 },
    { 0xf919b456, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea59b45d, 23 },
    { 0xd899b45e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf29b461, 95 },
    { 0x1ef9b462, 33 },
    { 0xd6a9b463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc69b467, 33 },
    { 0, 0 },
    { 0x97c9b469, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209b46d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7449b473, 90 },
    { 0x2829b474, 87 },
    { 0x1259b473, 90 },
    { 0x6a49b476, 95 },
    { 0xe959b476, 90 },
    { 0, 0 },
    { 0x18a9b479, 90 },
    { 0x2a69b479, 78 },
    { 0x7579b47b, 33 },
    { 0, 0 },
    { 0xad69b47d, 98 },
    { 0x3d99b47e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7599b482, 33 },
    { 0xb929b483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7509b488, 33 },
    { 0, 0 },
    { 0x9499b48a, 100 },
    { 0xc909b48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9b48e, 55 },
    { 0x15d9b48f, 23 },
    { 0xc7d9b48e, 66 },
    { 0, 0 },
    { 0x5829b492, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689b497, 81 },
    { 0x43e9b497, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279b49d, 55 },
    { 0x2339b49d, 61 },
    { 0x96a9b49f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9b4a3, 85 },
    { 0x93d9b4a4, 78 },
    { 0xade9b4a5, 61 },
    { 0xef29b4a3, 76 },
    { 0x9e69b4a7, 99 },
    { 0x839b4a8, 23 },
    { 0x2649b4a5, 93 },
    { 0, 0 },
    { 0x8c99b4ab, 33 },
    { 0, 0 },
    { 0x9e39b4ad, 76 },
    { 0, 0 },
    { 0x7c79b4af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba49b4b2, 76 },
    { 0x3299b4b3, 90 },
    { 0xb159b4b3, 87 },
    { 0x7259b4b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9b4ba, 61 },
    { 0x39c9b4ba, 90 },
    { 0x8f59b4ba, 95 },
    { 0, 0 },
    { 0xdbe9b4be, 46 },
    { 0x1e9b4be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c9b4c2, 33 },
    { 0x62c9b4c2, 33 },
    { 0xf479b4c3, 90 },
    { 0xf3d9b4c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa509b4c9, 34 },
    { 0, 0 },
    { 0x2989b4cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b49b4ce, 76 },
    { 0, 0 },
    { 0x68d9b4d0, 90 },
    { 0xad49b4d1, 95 },
    { 0xf229b4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769b4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469b4dd, 76 },
    { 0xa199b4dd, 90 },
    { 0x7e29b4df, 61 },
    { 0xe9b9b4e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339b4e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb519b4e9, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c9b4ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f9b4f2, 98 },
    { 0xdc79b4f3, 33 },
    { 0xc679b4f2, 61 },
    { 0x56b9b4f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d9b4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb929b4fb, 89 },
    { 0x3949b4fc, 90 },
    { 0, 0 },
    { 0x6359b4fe, 95 },
    { 0x6b19b4ff, 83 },
    { 0x3369b500, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf579b503, 90 },
    { 0, 0 },
    { 0x1519b505, 34 },
    { 0, 0 },
    { 0xaca9b507, 85 },
    { 0xac89b507, 98 },
    { 0x7ed9b509, 95 },
    { 0xbd59b509, 52 },
    { 0x5eb9b50a, 33 },
    { 0, 0 },
    { 0x1619b50d, 90 },
    { 0, 0 },
    { 0x659b50f, 61 },
    { 0x719b510, 40 },
    { 0xa3b9b511, 86 },
    { 0xb979b512, 23 },
    { 0x8179b510, 33 },
    { 0x6999b514, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xed29b517, 61 },
    { 0x4aa9b518, 95 },
    { 0x4309b519, 100 },
    { 0x4559b51a, 90 },
    { 0xdb59b51b, 90 },
    { 0xa939b51c, 61 },
    { 0x8049b517, 90 },
    { 0xeef9b51e, 90 },
    { 0xa109b51e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b522, 90 },
    { 0, 0 },
    { 0x5159b524, 64 },
    { 0x559b524, 69 },
    { 0xf629b526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf089b529, 90 },
    { 0, 0 },
    { 0xbec9b52b, 66 },
    { 0x1c89b52c, 90 },
    { 0x81d9b52c, 76 },
    { 0x1ae9b52b, 23 },
    { 0x6519b52f, 90 },
    { 0x59f9b530, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409b538, 90 },
    { 0, 0 },
    { 0xf999b53a, 84 },
    { 0, 0 },
    { 0x1269b53c, 61 },
    { 0xf909b53d, 23 },
    { 0x41f9b53e, 90 },
    { 0x54d9b53f, 95 },
    { 0xcb9b540, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9b544, 76 },
    { 0x71c9b545, 93 },
    { 0x1279b544, 99 },
    { 0x7e29b545, 97 },
    { 0x6649b547, 33 },
    { 0, 0 },
    { 0xbd69b54a, 40 },
    { 0x9129b54b, 61 },
    { 0x319b54c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979b54f, 77 },
    { 0x6d09b54f, 52 },
    { 0, 0 },
    { 0x2b79b552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a9b556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9b559, 90 },
    { 0, 0 },
    { 0xc6e9b55b, 76 },
    { 0x3d69b55c, 67 },
    { 0, 0 },
    { 0x1c99b55e, 44 },
    { 0, 0 },
    { 0x8029b560, 95 },
    { 0xec79b561, 78 },
    { 0, 0 },
    { 0xb449b563, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a9b566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d9b56b, 90 },
    { 0x6249b56b, 65 },
    { 0, 0 },
    { 0xbac9b56e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df9b572, 94 },
    { 0, 0 },
    { 0x5009b574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5729b579, 85 },
    { 0x9b99b57a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9b57f, 78 },
    { 0x3e29b580, 61 },
    { 0x8f39b581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4919b584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd819b58a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c9b58d, 98 },
    { 0, 0 },
    { 0xbb09b58f, 34 },
    { 0xc769b590, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b595, 33 },
    { 0, 0 },
    { 0xe879b597, 77 },
    { 0x409b598, 34 },
    { 0x809b598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e9b59d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf739b5a4, 23 },
    { 0xde9b5a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a9b5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a69b5af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c39b5b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd69b5b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99b5b8, 75 },
    { 0, 0 },
    { 0xdbc9b5ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7969b5bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539b5c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b9b5c5, 90 },
    { 0x3409b5c6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd79b5ca, 98 },
    { 0, 0 },
    { 0x86f9b5cc, 66 },
    { 0, 0 },
    { 0x5ef9b5ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639b5d1, 43 },
    { 0x81a9b5d1, 88 },
    { 0x1909b5d1, 101 },
    { 0x4239b5d4, 99 },
    { 0x49a9b5d2, 95 },
    { 0x4ce9b5d3, 33 },
    { 0, 0 },
    { 0x569b5d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29b5dd, 61 },
    { 0x53b9b5de, 90 },
    { 0x3e29b5dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a9b5e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9b5eb, 98 },
    { 0xb179b5ec, 33 },
    { 0xd499b5ed, 68 },
    { 0x6cb9b5eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9b5f1, 76 },
    { 0, 0 },
    { 0x26e9b5f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc459b5f8, 66 },
    { 0x2d59b5f9, 90 },
    { 0x39d9b5fa, 95 },
    { 0xdfb9b5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b9b600, 91 },
    { 0, 0 },
    { 0x6d89b602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2479b605, 75 },
    { 0x9959b606, 33 },
    { 0, 0 },
    { 0x5079b608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6529b60d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a9b610, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d9b614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839b61f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9b625, 23 },
    { 0xa909b626, 61 },
    { 0x4c49b625, 52 },
    { 0x50b9b628, 98 },
    { 0xf49b628, 75 },
    { 0x4389b629, 23 },
    { 0x9749b628, 75 },
    { 0, 0 },
    { 0xf759b62d, 33 },
    { 0x55e9b62d, 84 },
    { 0xafb9b62f, 77 },
    { 0x1369b630, 87 },
    { 0x2689b631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c9b638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a19b63e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc199b642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4119b649, 100 },
    { 0x9249b649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc369b64d, 81 },
    { 0, 0 },
    { 0x1c9b64f, 90 },
    { 0x529b650, 55 },
    { 0, 0 },
    { 0xdf29b652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc919b655, 34 },
    { 0xb149b656, 90 },
    { 0xe019b657, 98 },
    { 0x7209b658, 98 },
    { 0x54e9b656, 95 },
    { 0xacd9b65a, 90 },
    { 0x1549b656, 76 },
    { 0xc599b65b, 36 },
    { 0xc5b9b655, 98 },
    { 0xac49b65e, 61 },
    { 0, 0 },
    { 0x8c09b660, 90 },
    { 0, 0 },
    { 0x7fb9b662, 23 },
    { 0x4359b663, 90 },
    { 0, 0 },
    { 0x92f9b665, 38 },
    { 0xcc29b665, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd969b66a, 98 },
    { 0xab09b66a, 90 },
    { 0x71b9b66a, 95 },
    { 0x25f9b66d, 26 },
    { 0x4ae9b66d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d9b673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae29b677, 19 },
    { 0, 0 },
    { 0x76e9b679, 80 },
    { 0, 0 },
    { 0x5ba9b67b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd9b686, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc349b68a, 100 },
    { 0, 0 },
    { 0xb6d9b68c, 95 },
    { 0, 0 },
    { 0x3129b68e, 95 },
    { 0x7ed9b68e, 90 },
    { 0x4499b68e, 38 },
    { 0x1aa9b68e, 23 },
    { 0x6cd9b692, 33 },
    { 0x2859b692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec9b696, 93 },
    { 0xab69b697, 97 },
    { 0, 0 },
    { 0xbd99b699, 34 },
    { 0xd2f9b69a, 36 },
    { 0x7489b69a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1729b6a0, 33 },
    { 0x3c99b6a1, 81 },
    { 0xac29b6a2, 33 },
    { 0x7ef9b6a3, 95 },
    { 0x3609b6a4, 23 },
    { 0xbba9b6a5, 95 },
    { 0x2e29b6a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce9b6ad, 51 },
    { 0xfc19b6ad, 93 },
    { 0, 0 },
    { 0x639b6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29b6b3, 98 },
    { 0x9b19b6b3, 90 },
    { 0, 0 },
    { 0x70b9b6b6, 76 },
    { 0xbaa9b6b7, 87 },
    { 0xbd09b6b8, 95 },
    { 0, 0 },
    { 0x6d59b6ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf649b6bd, 93 },
    { 0x6429b6be, 76 },
    { 0x4929b6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89b6c4, 23 },
    { 0, 0 },
    { 0x96e9b6c6, 75 },
    { 0xf259b6c7, 23 },
    { 0xffe9b6c6, 66 },
    { 0xb539b6c7, 55 },
    { 0, 0 },
    { 0x6c29b6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c9b6d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2039b6dc, 34 },
    { 0xe5e9b6dc, 55 },
    { 0x7a79b6de, 43 },
    { 0x3a69b6dd, 66 },
    { 0xbd29b6e0, 66 },
    { 0x3149b6e0, 95 },
    { 0x1309b6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce69b6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39b6ea, 90 },
    { 0, 0 },
    { 0x7479b6ec, 76 },
    { 0x9a29b6ed, 78 },
    { 0, 0 },
    { 0x9799b6ef, 95 },
    { 0, 0 },
    { 0x1189b6f1, 61 },
    { 0xee89b6f2, 100 },
    { 0x26e9b6f3, 93 },
    { 0, 0 },
    { 0x27d9b6f5, 95 },
    { 0x6589b6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d9b6fb, 33 },
    { 0, 0 },
    { 0x7589b6fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa739b703, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b9b70a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf879b70e, 98 },
    { 0xd579b70e, 100 },
    { 0x2f99b710, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669b717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79b71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79b71e, 55 },
    { 0xd479b71f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a9b724, 76 },
    { 0xb039b725, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349b729, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9b72d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd839b731, 34 },
    { 0x1cb9b732, 90 },
    { 0, 0 },
    { 0x809b734, 99 },
    { 0x3fe9b735, 61 },
    { 0xff69b736, 33 },
    { 0xe49b737, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4159b73b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b59b73f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe599b744, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed9b747, 55 },
    { 0xf499b748, 90 },
    { 0x8ae9b749, 90 },
    { 0x7dd9b74a, 90 },
    { 0, 0 },
    { 0x59e9b74c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b74f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b9b756, 94 },
    { 0, 0 },
    { 0x9209b758, 56 },
    { 0x32f9b758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9b75d, 23 },
    { 0x3389b75e, 67 },
    { 0, 0 },
    { 0x6969b760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9b763, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979b76a, 77 },
    { 0, 0 },
    { 0xa5f9b76c, 90 },
    { 0xe389b76d, 90 },
    { 0x8d59b76e, 61 },
    { 0xb6d9b76f, 81 },
    { 0, 0 },
    { 0xac59b771, 76 },
    { 0, 0 },
    { 0xd409b773, 61 },
    { 0x9169b774, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9b777, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c9b77d, 90 },
    { 0x66a9b77e, 84 },
    { 0x4829b77f, 61 },
    { 0xd9b9b780, 76 },
    { 0, 0 },
    { 0x6089b782, 55 },
    { 0, 0 },
    { 0xa349b784, 43 },
    { 0xc1d9b785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c9b788, 90 },
    { 0x81a9b788, 90 },
    { 0x7c99b78a, 98 },
    { 0, 0 },
    { 0xf789b78c, 46 },
    { 0x4f9b78d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9b792, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1369b795, 33 },
    { 0, 0 },
    { 0xb759b797, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299b79b, 33 },
    { 0, 0 },
    { 0x5a39b79d, 43 },
    { 0x1f9b79e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a9b7a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e9b7a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7179b7aa, 98 },
    { 0x19f9b7aa, 33 },
    { 0, 0 },
    { 0x97c9b7ad, 90 },
    { 0x9b19b7ad, 90 },
    { 0x3a79b7af, 33 },
    { 0x6bd9b7b0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f29b7b4, 43 },
    { 0, 0 },
    { 0xf7a9b7b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf9b7b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b49b7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe729b7c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e19b7cb, 90 },
    { 0xb069b7cc, 95 },
    { 0xfe09b7cd, 90 },
    { 0x3b79b7cb, 43 },
    { 0xbec9b7ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f9b7d6, 93 },
    { 0x9ed9b7d6, 84 },
    { 0xc699b7d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4859b7de, 90 },
    { 0x90b9b7de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9b7e5, 61 },
    { 0xd099b7e6, 23 },
    { 0xc999b7e5, 61 },
    { 0x88d9b7e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe09b7ec, 90 },
    { 0, 0 },
    { 0xaf79b7ee, 61 },
    { 0x9f69b7ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9709b7f2, 33 },
    { 0x54e9b7f2, 93 },
    { 0xb2f9b7f4, 76 },
    { 0x64a9b7f4, 33 },
    { 0x50b9b7f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d09b7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f9b800, 90 },
    { 0x7149b801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b49b806, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5309b80f, 77 },
    { 0xb329b80f, 76 },
    { 0xac69b810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9b816, 90 },
    { 0xa2f9b817, 98 },
    { 0x5f79b818, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2089b81c, 43 },
    { 0x8949b81c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6209b822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319b825, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9b829, 33 },
    { 0xb499b82a, 33 },
    { 0x7cf9b82b, 46 },
    { 0xb379b82c, 33 },
    { 0xe879b82d, 76 },
    { 0, 0 },
    { 0x57e9b82f, 78 },
    { 0x1b39b830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1689b833, 90 },
    { 0x8519b833, 84 },
    { 0xce79b835, 99 },
    { 0x56b9b835, 61 },
    { 0, 0 },
    { 0xfe99b838, 61 },
    { 0x4259b838, 90 },
    { 0x7b69b83a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2779b83d, 55 },
    { 0, 0 },
    { 0xf3f9b83f, 90 },
    { 0x6259b83f, 23 },
    { 0xf889b841, 61 },
    { 0x4f89b841, 78 },
    { 0xc199b842, 36 },
    { 0x2669b843, 90 },
    { 0xb999b845, 78 },
    { 0xa899b846, 80 },
    { 0xff09b847, 64 },
    { 0x70f9b847, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c9b84d, 90 },
    { 0, 0 },
    { 0x7d79b84f, 95 },
    { 0x83e9b850, 61 },
    { 0x6369b851, 95 },
    { 0x3129b852, 33 },
    { 0xbbe9b853, 95 },
    { 0x3069b850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9b858, 93 },
    { 0x8329b859, 90 },
    { 0, 0 },
    { 0x6859b85b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b860, 98 },
    { 0x9ae9b861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b39b865, 98 },
    { 0, 0 },
    { 0x9fd9b867, 90 },
    { 0x6f39b868, 46 },
    { 0xbd9b868, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9b86c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9b870, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c9b875, 55 },
    { 0xc9f9b876, 23 },
    { 0xac59b875, 33 },
    { 0xfec9b877, 90 },
    { 0x6989b877, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609b87d, 61 },
    { 0, 0 },
    { 0x6f89b87f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd9b885, 67 },
    { 0x8649b885, 95 },
    { 0, 0 },
    { 0xf529b888, 98 },
    { 0xf689b888, 98 },
    { 0xc519b88a, 61 },
    { 0xcf89b88b, 67 },
    { 0xa2a9b88b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d9b890, 71 },
    { 0x8c19b891, 77 },
    { 0, 0 },
    { 0x1b49b893, 90 },
    { 0xfe39b894, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9b897, 90 },
    { 0, 0 },
    { 0xb2b9b899, 33 },
    { 0x5109b89a, 33 },
    { 0, 0 },
    { 0x2089b89c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9b89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9b8a2, 89 },
    { 0x7479b8a3, 33 },
    { 0x21e9b8a4, 99 },
    { 0xb369b8a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9b8ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9b8b0, 90 },
    { 0xf929b8b0, 61 },
    { 0xaac9b8b2, 77 },
    { 0xedb9b8b3, 87 },
    { 0x1b69b8b4, 85 },
    { 0x4a19b8b5, 61 },
    { 0x4fd9b8b6, 33 },
    { 0x1a49b8b7, 98 },
    { 0, 0 },
    { 0xd259b8b9, 95 },
    { 0, 0 },
    { 0x8839b8bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929b8be, 23 },
    { 0, 0 },
    { 0x6729b8c0, 33 },
    { 0x9da9b8c1, 23 },
    { 0xd299b8c2, 95 },
    { 0, 0 },
    { 0x79a9b8c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd9b8c8, 87 },
    { 0x96b9b8c9, 55 },
    { 0, 0 },
    { 0x7069b8cb, 33 },
    { 0xb7f9b8cc, 77 },
    { 0x7169b8cd, 34 },
    { 0x1589b8cb, 95 },
    { 0xb1e9b8cf, 61 },
    { 0x79a9b8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a49b8d5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9b8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09b8df, 23 },
    { 0x2eb9b8e0, 98 },
    { 0, 0 },
    { 0x1db9b8e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xace9b8e5, 78 },
    { 0x1c99b8e6, 66 },
    { 0xd579b8e7, 67 },
    { 0x5199b8e8, 74 },
    { 0x1c89b8e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39b8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda19b8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd9b8f2, 55 },
    { 0x5a19b8f3, 61 },
    { 0x8f29b8f4, 76 },
    { 0x8e9b8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2619b8fa, 33 },
    { 0x2f89b8fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9b8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7029b902, 95 },
    { 0xe0e9b903, 95 },
    { 0, 0 },
    { 0x9de9b905, 23 },
    { 0x54a9b906, 33 },
    { 0x2709b906, 78 },
    { 0x7869b908, 77 },
    { 0xa0a9b908, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b90f, 33 },
    { 0x48c9b90f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f9b914, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6839b91a, 95 },
    { 0xda79b91a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99b91e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b39b922, 93 },
    { 0x9fb9b923, 83 },
    { 0x2679b923, 98 },
    { 0, 0 },
    { 0x2c89b926, 90 },
    { 0, 0 },
    { 0xf4c9b928, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9b92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9b931, 33 },
    { 0x1259b932, 90 },
    { 0xe119b932, 100 },
    { 0x5529b934, 40 },
    { 0xb909b934, 95 },
    { 0x65c9b933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b93d, 88 },
    { 0xcf09b93e, 43 },
    { 0xc699b93e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269b942, 43 },
    { 0x1fc9b943, 95 },
    { 0x1999b943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269b94a, 23 },
    { 0x1c59b94b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd339b94f, 33 },
    { 0, 0 },
    { 0xe289b951, 87 },
    { 0, 0 },
    { 0xe529b953, 33 },
    { 0x1309b954, 76 },
    { 0xd649b955, 90 },
    { 0xb199b956, 90 },
    { 0xe359b956, 95 },
    { 0xdd29b954, 33 },
    { 0, 0 },
    { 0x6849b95a, 55 },
    { 0x8039b95b, 66 },
    { 0xadc9b95c, 23 },
    { 0, 0 },
    { 0xbf89b95e, 95 },
    { 0x2299b95f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f9b962, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619b966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9b96b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed19b96e, 98 },
    { 0, 0 },
    { 0x17f9b970, 66 },
    { 0xa7c9b971, 61 },
    { 0, 0 },
    { 0xcdd9b973, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3609b976, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9b979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819b97e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2079b98a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1639b98d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479b996, 85 },
    { 0x3309b997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a9b99a, 76 },
    { 0x12b9b99b, 33 },
    { 0xb8e9b99b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc19b9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069b9a7, 95 },
    { 0x87d9b9a7, 98 },
    { 0x3859b9a9, 33 },
    { 0x93c9b9a8, 40 },
    { 0x4dd9b9ab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a9b9b0, 95 },
    { 0x1a09b9b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c89b9b7, 77 },
    { 0x7929b9b8, 90 },
    { 0x4ab9b9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f9b9bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289b9c0, 98 },
    { 0xa0c9b9c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa589b9c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359b9c8, 90 },
    { 0x8759b9c8, 55 },
    { 0x609b9ca, 52 },
    { 0xdb19b9cb, 55 },
    { 0x3f89b9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae19b9d0, 93 },
    { 0, 0 },
    { 0x49d9b9d2, 98 },
    { 0, 0 },
    { 0x28e9b9d4, 17 },
    { 0xdbe9b9d5, 52 },
    { 0x5249b9d6, 95 },
    { 0xccd9b9d7, 61 },
    { 0x4f29b9d4, 33 },
    { 0x3ac9b9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc219b9dc, 76 },
    { 0x25e9b9dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a09b9e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109b9eb, 84 },
    { 0x33d9b9eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc9b9ef, 61 },
    { 0xa139b9f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539b9f4, 87 },
    { 0xb239b9f4, 90 },
    { 0x7639b9f4, 61 },
    { 0, 0 },
    { 0x6f59b9f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d89b9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b9ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759ba03, 90 },
    { 0xe009ba03, 90 },
    { 0xcb79ba05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139ba08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389ba0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79ba12, 76 },
    { 0xba69ba12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c89ba16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec99ba1e, 61 },
    { 0x979ba1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459ba27, 98 },
    { 0, 0 },
    { 0x4a79ba29, 95 },
    { 0x4369ba2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409ba2f, 80 },
    { 0, 0 },
    { 0xe799ba31, 33 },
    { 0, 0 },
    { 0xe959ba33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9ba37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559ba3c, 76 },
    { 0xb789ba3d, 61 },
    { 0, 0 },
    { 0x9fb9ba3f, 99 },
    { 0x18a9ba40, 33 },
    { 0xd3b9ba41, 68 },
    { 0, 0 },
    { 0xc7d9ba43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949ba51, 91 },
    { 0x5f39ba51, 95 },
    { 0, 0 },
    { 0xb1f9ba54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9ba67, 58 },
    { 0x3979ba68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69ba6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd9ba6f, 33 },
    { 0x72b9ba70, 33 },
    { 0x5049ba6f, 33 },
    { 0x1189ba72, 43 },
    { 0, 0 },
    { 0xa749ba74, 33 },
    { 0xbbb9ba75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19ba7a, 33 },
    { 0xe2b9ba7b, 81 },
    { 0x8729ba7c, 61 },
    { 0x7bb9ba7c, 98 },
    { 0x6b39ba7a, 90 },
    { 0x47c9ba7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a59ba83, 40 },
    { 0xcf59ba84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b9ba89, 90 },
    { 0x1489ba8a, 61 },
    { 0xfb59ba89, 33 },
    { 0x2409ba8a, 17 },
    { 0, 0 },
    { 0x9179ba8e, 90 },
    { 0x1479ba8f, 55 },
    { 0, 0 },
    { 0xbd69ba91, 99 },
    { 0, 0 },
    { 0x2149ba93, 55 },
    { 0x5d09ba94, 23 },
    { 0, 0 },
    { 0xe09ba96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b9baa0, 98 },
    { 0x6d9baa1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c39baa7, 93 },
    { 0xc159baa8, 95 },
    { 0xc949baa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa119baaf, 61 },
    { 0, 0 },
    { 0x3e89bab1, 23 },
    { 0x6259bab2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69bab5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf79bab9, 76 },
    { 0xbde9baba, 33 },
    { 0x5a39baba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9babe, 96 },
    { 0, 0 },
    { 0x8049bac0, 88 },
    { 0, 0 },
    { 0x7a99bac2, 87 },
    { 0x13f9bac3, 95 },
    { 0x88f9bac4, 33 },
    { 0x68a9bac4, 95 },
    { 0x3e89bac4, 55 },
    { 0x50e9bac4, 48 },
    { 0xb549bac8, 93 },
    { 0xc8b9bac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569bace, 43 },
    { 0xd199bace, 76 },
    { 0x3139bad0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9bad4, 43 },
    { 0xed9bad5, 95 },
    { 0x34e9bad6, 78 },
    { 0, 0 },
    { 0x4669bad8, 23 },
    { 0x2889bad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a69badd, 52 },
    { 0xba69badd, 91 },
    { 0x7fa9bade, 90 },
    { 0xf019badf, 33 },
    { 0xdb29bae1, 23 },
    { 0xaf29bade, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4549bae8, 90 },
    { 0xf6b9bae9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a9baf3, 43 },
    { 0x63f9baf4, 46 },
    { 0, 0 },
    { 0x2419baf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9bafb, 55 },
    { 0, 0 },
    { 0x519bafd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59bb04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf49bb0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd9bb10, 46 },
    { 0x47e9bb10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39bb17, 52 },
    { 0xc8d9bb17, 90 },
    { 0xde79bb19, 55 },
    { 0, 0 },
    { 0xdef9bb1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19bb1f, 85 },
    { 0, 0 },
    { 0x7639bb21, 67 },
    { 0x1179bb21, 61 },
    { 0x929bb23, 90 },
    { 0xd8b9bb23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd9bb27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f9bb2b, 23 },
    { 0x9459bb2c, 33 },
    { 0x82d9bb2c, 90 },
    { 0xb2c9bb2d, 55 },
    { 0, 0 },
    { 0x36d9bb30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf9bb3a, 33 },
    { 0x4759bb3b, 95 },
    { 0x6d19bb3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b9bb45, 61 },
    { 0, 0 },
    { 0x3559bb47, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9bb4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf089bb4d, 95 },
    { 0, 0 },
    { 0xc9d9bb4f, 98 },
    { 0, 0 },
    { 0xdea9bb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9909bb54, 90 },
    { 0x6649bb54, 99 },
    { 0, 0 },
    { 0xab59bb57, 76 },
    { 0x1ef9bb57, 98 },
    { 0x4b19bb59, 76 },
    { 0xdb99bb5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf379bb5f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89bb62, 76 },
    { 0x23c9bb63, 76 },
    { 0x9c89bb62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b9bb6a, 61 },
    { 0, 0 },
    { 0x7399bb6c, 87 },
    { 0xb539bb6c, 91 },
    { 0, 0 },
    { 0x9d39bb6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29bb73, 88 },
    { 0x7919bb73, 43 },
    { 0x8249bb75, 23 },
    { 0, 0 },
    { 0xfe99bb77, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb429bb7a, 76 },
    { 0xd349bb7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799bb80, 61 },
    { 0x9559bb81, 23 },
    { 0x6c39bb82, 43 },
    { 0x6709bb83, 93 },
    { 0xcce9bb82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6119bb89, 67 },
    { 0x8369bb89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6609bb93, 33 },
    { 0xb7a9bb94, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9bb9a, 23 },
    { 0, 0 },
    { 0x5299bb9c, 34 },
    { 0, 0 },
    { 0x4519bb9e, 61 },
    { 0x9a99bb9e, 33 },
    { 0x7289bb9f, 34 },
    { 0x2c49bba1, 38 },
    { 0xd889bba1, 23 },
    { 0xd4f9bba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c9bba6, 90 },
    { 0x9f69bba6, 87 },
    { 0x1559bba8, 33 },
    { 0x1599bba7, 90 },
    { 0xbbf9bba8, 75 },
    { 0x36d9bbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7359bbb0, 91 },
    { 0x5279bbb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69bbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6699bbb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989bbba, 33 },
    { 0xbf9bbbb, 99 },
    { 0x2f69bbbc, 84 },
    { 0x53f9bbba, 61 },
    { 0x39a9bbbc, 33 },
    { 0xf159bbbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9bbc4, 85 },
    { 0x30c9bbc5, 55 },
    { 0x61a9bbc6, 95 },
    { 0x5c99bbc7, 55 },
    { 0xc4b9bbc6, 33 },
    { 0x7e39bbc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d9bbd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad29bbd7, 55 },
    { 0x63d9bbd8, 61 },
    { 0x639bbd9, 90 },
    { 0, 0 },
    { 0xe0e9bbdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79bbde, 98 },
    { 0x51b9bbdf, 61 },
    { 0x5b99bbe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259bbe3, 33 },
    { 0, 0 },
    { 0xe7a9bbe5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c9bbea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49bbf0, 88 },
    { 0x2479bbf1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x149bbf4, 33 },
    { 0x6619bbf4, 90 },
    { 0, 0 },
    { 0x7c59bbf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e19bbfa, 34 },
    { 0, 0 },
    { 0xce59bbfc, 61 },
    { 0xf889bbfd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e89bc00, 33 },
    { 0, 0 },
    { 0x5bf9bc02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f79bc05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe9bc0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x859bc0d, 33 },
    { 0x2179bc0d, 76 },
    { 0x7e49bc0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89bc13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa299bc1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9bc23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d9bc29, 23 },
    { 0x40e9bc29, 77 },
    { 0x2489bc2b, 33 },
    { 0xe489bc2c, 23 },
    { 0x9169bc29, 61 },
    { 0x29e9bc2e, 64 },
    { 0, 0 },
    { 0xe889bc30, 95 },
    { 0x31f9bc30, 98 },
    { 0xd959bc31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe809bc3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a9bc43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99bc48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939bc4b, 64 },
    { 0x5759bc4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a49bc51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac9bc55, 98 },
    { 0xb1e9bc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe609bc5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3449bc63, 61 },
    { 0, 0 },
    { 0x1b49bc65, 33 },
    { 0x4459bc66, 86 },
    { 0xbe99bc66, 98 },
    { 0x3599bc65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9bc6c, 55 },
    { 0x6b39bc6d, 98 },
    { 0, 0 },
    { 0x8cd9bc6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed9bc74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f9bc77, 85 },
    { 0x8e89bc78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d49bc81, 23 },
    { 0xc19bc82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4419bc86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb9bc89, 61 },
    { 0, 0 },
    { 0x3879bc8b, 93 },
    { 0x4ba9bc8c, 78 },
    { 0xc609bc8d, 56 },
    { 0x8299bc8d, 34 },
    { 0x1fc9bc8f, 46 },
    { 0xd389bc90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d9bc93, 85 },
    { 0x94e9bc93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb339bc97, 46 },
    { 0xf399bc98, 34 },
    { 0x5d99bc97, 90 },
    { 0x9e49bc9a, 61 },
    { 0x9309bc9b, 95 },
    { 0x29d9bc9b, 40 },
    { 0, 0 },
    { 0xa749bc9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c9bca1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b9bca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f9bca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9249bcb8, 81 },
    { 0x5bf9bcb9, 34 },
    { 0, 0 },
    { 0xe869bcbb, 88 },
    { 0xd589bcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9bcbf, 90 },
    { 0, 0 },
    { 0x82f9bcc1, 43 },
    { 0x66a9bcc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1429bcc7, 81 },
    { 0xacc9bcc7, 55 },
    { 0, 0 },
    { 0xf599bcca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe59bccd, 98 },
    { 0xad59bccd, 95 },
    { 0xea19bccf, 98 },
    { 0x4019bcd0, 88 },
    { 0x9599bcd1, 78 },
    { 0x7bc9bcd2, 98 },
    { 0xde99bcd2, 61 },
    { 0x7e59bcd2, 90 },
    { 0x9e49bcd5, 43 },
    { 0x7a79bcd5, 95 },
    { 0xc239bccd, 61 },
    { 0x8ce9bccd, 88 },
    { 0x98f9bcd9, 88 },
    { 0x31b9bcd8, 98 },
    { 0x5c49bcdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39bcde, 81 },
    { 0xf869bcdf, 68 },
    { 0x4d9bcde, 23 },
    { 0xd049bce0, 84 },
    { 0x7c29bce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f19bce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7429bcea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9bcf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee29bcfb, 33 },
    { 0xd559bcfb, 93 },
    { 0x86c9bcfc, 90 },
    { 0x3e69bcfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f39bd01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc739bd09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9bd0f, 33 },
    { 0, 0 },
    { 0x85e9bd11, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9bd16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bd1b, 67 },
    { 0x2d9bd1b, 33 },
    { 0, 0 },
    { 0x9b9bd1e, 98 },
    { 0xefe9bd1f, 93 },
    { 0, 0 },
    { 0x4599bd21, 61 },
    { 0xa199bd21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc559bd25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa389bd28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9bd2b, 98 },
    { 0xd959bd2b, 98 },
    { 0, 0 },
    { 0xdf29bd2e, 66 },
    { 0xddb9bd2f, 23 },
    { 0xb9c9bd30, 95 },
    { 0x55f9bd31, 17 },
    { 0x27b9bd32, 55 },
    { 0, 0 },
    { 0x2dd9bd34, 75 },
    { 0x9ac9bd35, 95 },
    { 0x9249bd35, 93 },
    { 0, 0 },
    { 0x6889bd38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349bd3b, 33 },
    { 0xe189bd3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379bd41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad89bd45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda99bd48, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649bd50, 34 },
    { 0x7519bd51, 33 },
    { 0xb999bd51, 87 },
    { 0xfab9bd53, 33 },
    { 0x4909bd54, 65 },
    { 0x6439bd54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec9bd58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf989bd67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf569bd6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb479bd6f, 33 },
    { 0xbc19bd70, 98 },
    { 0x9499bd70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d9bd75, 98 },
    { 0, 0 },
    { 0x8749bd77, 90 },
    { 0, 0 },
    { 0x7de9bd79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc029bd7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9bd85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f9bd8b, 66 },
    { 0x48c9bd8b, 76 },
    { 0xc959bd8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199bd90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a99bd93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf9bd98, 90 },
    { 0x40d9bd99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae9bd9c, 33 },
    { 0x3fb9bd9d, 76 },
    { 0x3d69bd9d, 98 },
    { 0x87e9bd9f, 99 },
    { 0, 0 },
    { 0x3cf9bda1, 55 },
    { 0xc1f9bda2, 90 },
    { 0x4369bda3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9bda9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669bdaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9bdb2, 96 },
    { 0, 0 },
    { 0x75e9bdb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bdb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf9bdbf, 90 },
    { 0xea09bdc0, 95 },
    { 0x10d9bdc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3279bdc9, 95 },
    { 0xe699bdca, 98 },
    { 0x479bdcb, 76 },
    { 0, 0 },
    { 0xc849bdcd, 33 },
    { 0, 0 },
    { 0x57c9bdcf, 90 },
    { 0, 0 },
    { 0x9a69bdd1, 23 },
    { 0, 0 },
    { 0x94d9bdd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be9bdd9, 53 },
    { 0x40d9bdda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9bde1, 78 },
    { 0x5679bde2, 90 },
    { 0xc6d9bde1, 55 },
    { 0x4009bde4, 33 },
    { 0x93b9bde5, 76 },
    { 0, 0 },
    { 0x2889bde7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9bdea, 17 },
    { 0xb0d9bdeb, 90 },
    { 0, 0 },
    { 0x5419bded, 33 },
    { 0, 0 },
    { 0x999bdef, 61 },
    { 0x7399bdf0, 76 },
    { 0, 0 },
    { 0xba19bdf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a9bdf5, 61 },
    { 0x3699bdf6, 95 },
    { 0x3869bdf7, 97 },
    { 0x9509bdf7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49be04, 90 },
    { 0xe69be05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b69be0f, 90 },
    { 0, 0 },
    { 0xed89be11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309be15, 40 },
    { 0x6d59be16, 78 },
    { 0, 0 },
    { 0xdc69be18, 33 },
    { 0x4b79be18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe289be1c, 76 },
    { 0x5419be1d, 33 },
    { 0x6119be1e, 99 },
    { 0x9699be1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4029be27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2099be2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9be31, 95 },
    { 0x3de9be32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf9be35, 61 },
    { 0x71b9be36, 33 },
    { 0xb59be36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d29be3e, 81 },
    { 0, 0 },
    { 0x68c9be40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf229be45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e9be48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a9be4b, 95 },
    { 0, 0 },
    { 0x24a9be4d, 98 },
    { 0, 0 },
    { 0x71e9be4f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9be52, 88 },
    { 0xb7b9be52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec39be58, 34 },
    { 0, 0 },
    { 0x6789be5a, 90 },
    { 0x9959be5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5569be5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a9be65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e9be68, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc9be6b, 88 },
    { 0x4799be6c, 76 },
    { 0, 0 },
    { 0x4fb9be6e, 23 },
    { 0xe209be6f, 90 },
    { 0x8459be6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c9be73, 17 },
    { 0x5959be74, 95 },
    { 0, 0 },
    { 0x4dc9be76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3079be7a, 98 },
    { 0x2209be7b, 55 },
    { 0xab69be7c, 33 },
    { 0x7db9be7c, 81 },
    { 0x33d9be7a, 33 },
    { 0xac09be7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4879be83, 34 },
    { 0xe309be84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c9be87, 23 },
    { 0, 0 },
    { 0x61e9be89, 55 },
    { 0, 0 },
    { 0x2739be8b, 55 },
    { 0xea19be8c, 90 },
    { 0x7f09be8d, 23 },
    { 0, 0 },
    { 0x9659be8f, 55 },
    { 0xab9be90, 46 },
    { 0x5099be90, 34 },
    { 0xebe9be92, 62 },
    { 0x1b29be93, 43 },
    { 0x1cc9be92, 61 },
    { 0xf039be8f, 99 },
    { 0xe689be96, 100 },
    { 0, 0 },
    { 0xa449be98, 43 },
    { 0, 0 },
    { 0xd4c9be9a, 100 },
    { 0x169be9a, 55 },
    { 0, 0 },
    { 0x3d79be9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac09bea4, 23 },
    { 0x5a29bea5, 76 },
    { 0xd1e9bea6, 23 },
    { 0x6359bea7, 33 },
    { 0xf4f9bea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f9beac, 90 },
    { 0xb329bead, 61 },
    { 0xb9c9beae, 98 },
    { 0x4d19beae, 90 },
    { 0xe5d9beae, 23 },
    { 0, 0 },
    { 0x4559beb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b99beb5, 65 },
    { 0, 0 },
    { 0x6dd9beb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b69bebe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8669bec3, 91 },
    { 0xc809bec4, 52 },
    { 0x5b49bec5, 76 },
    { 0xe639bec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f9becd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d9bed1, 61 },
    { 0, 0 },
    { 0xd439bed3, 93 },
    { 0xa8c9bed3, 84 },
    { 0x2c79bed4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9bed8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa319bee1, 43 },
    { 0x5959bee2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669bee5, 61 },
    { 0, 0 },
    { 0xad99bee7, 33 },
    { 0x9eb9bee7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa549beec, 90 },
    { 0, 0 },
    { 0x86c9beee, 61 },
    { 0, 0 },
    { 0xadc9bef0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca9bef3, 43 },
    { 0xda69bef4, 100 },
    { 0xa919bef5, 46 },
    { 0x8179bef6, 90 },
    { 0xed39bef5, 67 },
    { 0x4a79bef7, 87 },
    { 0x9b29bef9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89bf07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229bf13, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9bf1b, 23 },
    { 0x4609bf1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd249bf20, 90 },
    { 0x30f9bf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb9bf28, 46 },
    { 0, 0 },
    { 0x8979bf2a, 46 },
    { 0, 0 },
    { 0x3149bf2c, 61 },
    { 0xa4f9bf2c, 67 },
    { 0, 0 },
    { 0x5549bf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab9bf38, 90 },
    { 0xc779bf39, 90 },
    { 0xa079bf3a, 46 },
    { 0xe869bf39, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa139bf40, 90 },
    { 0x319bf41, 73 },
    { 0x4249bf41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809bf48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9bf56, 61 },
    { 0, 0 },
    { 0xc709bf58, 93 },
    { 0, 0 },
    { 0xc939bf5a, 90 },
    { 0x8fb9bf5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9bf5e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6619bf64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809bf68, 98 },
    { 0xc179bf69, 43 },
    { 0x3079bf6a, 95 },
    { 0, 0 },
    { 0xf309bf6c, 95 },
    { 0, 0 },
    { 0x8599bf6e, 43 },
    { 0x3dc9bf6f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa9bf76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949bf7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9bf7e, 76 },
    { 0, 0 },
    { 0xfaf9bf80, 61 },
    { 0, 0 },
    { 0x11d9bf82, 34 },
    { 0, 0 },
    { 0x639bf84, 81 },
    { 0, 0 },
    { 0xf0b9bf86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f79bf94, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc59bf98, 95 },
    { 0x8379bf99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a9bf9d, 87 },
    { 0x9529bf9e, 95 },
    { 0x41f9bf9f, 75 },
    { 0x5979bf9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f9bfa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9bfaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c9bfad, 36 },
    { 0x4d49bfae, 33 },
    { 0x689bfad, 91 },
    { 0x4b59bfb0, 67 },
    { 0xfca9bfb1, 46 },
    { 0x8cd9bfb2, 84 },
    { 0x1609bfb1, 55 },
    { 0, 0 },
    { 0xe9d9bfb5, 95 },
    { 0, 0 },
    { 0xf0f9bfb7, 77 },
    { 0x5e89bfb8, 90 },
    { 0, 0 },
    { 0xfd99bfba, 100 },
    { 0xef39bfbb, 90 },
    { 0x7939bfba, 61 },
    { 0x8d39bfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x679bfc0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129bfc3, 61 },
    { 0, 0 },
    { 0xafa9bfc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf389bfc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8039bfce, 61 },
    { 0xb669bfcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad9bfd4, 90 },
    { 0xdd79bfd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb29bfdb, 32 },
    { 0, 0 },
    { 0x4a39bfdd, 90 },
    { 0x8619bfde, 76 },
    { 0x6849bfdd, 46 },
    { 0xb149bfe0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa089bfe6, 98 },
    { 0x8469bfe6, 61 },
    { 0, 0 },
    { 0xaec9bfe9, 33 },
    { 0xfc49bfea, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e79bfed, 90 },
    { 0x5fd9bfee, 46 },
    { 0x2af9bfef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199bff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba9bffc, 78 },
    { 0x5f29bffd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3969c002, 95 },
    { 0, 0 },
    { 0xdd9c004, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9c00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149c010, 67 },
    { 0x9b89c011, 93 },
    { 0, 0 },
    { 0x2b69c013, 91 },
    { 0x97f9c014, 61 },
    { 0, 0 },
    { 0x54d9c016, 95 },
    { 0, 0 },
    { 0xb179c018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c89c022, 61 },
    { 0x3b49c023, 17 },
    { 0x1c39c022, 33 },
    { 0, 0 },
    { 0x8df9c026, 55 },
    { 0x6e59c027, 90 },
    { 0, 0 },
    { 0x4069c029, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b59c02c, 23 },
    { 0, 0 },
    { 0x14c9c02e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a9c033, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee9c03a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629c040, 61 },
    { 0x5d29c041, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329c04a, 34 },
    { 0x2f89c04b, 90 },
    { 0x2509c04c, 95 },
    { 0, 0 },
    { 0x2b39c04e, 90 },
    { 0, 0 },
    { 0x8ed9c050, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf59c053, 68 },
    { 0x8a09c054, 98 },
    { 0, 0 },
    { 0xac09c056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09c05d, 46 },
    { 0x6ba9c05d, 93 },
    { 0x4299c05f, 61 },
    { 0xea49c060, 98 },
    { 0xbcc9c05f, 98 },
    { 0, 0 },
    { 0x15e9c063, 93 },
    { 0x3d59c064, 78 },
    { 0xa2d9c065, 33 },
    { 0, 0 },
    { 0x8bc9c067, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9c06a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6279c06d, 81 },
    { 0xb549c06e, 33 },
    { 0xdd99c06e, 90 },
    { 0xb6e9c06f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f89c075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8539c079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9c07d, 82 },
    { 0xab19c07e, 90 },
    { 0, 0 },
    { 0xbc79c080, 33 },
    { 0x7dd9c080, 80 },
    { 0x8939c082, 46 },
    { 0xb719c083, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec9c08d, 55 },
    { 0x6609c08e, 85 },
    { 0x88f9c08e, 88 },
    { 0x99d9c090, 61 },
    { 0x8e89c090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df9c098, 23 },
    { 0x23e9c098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9c09d, 61 },
    { 0, 0 },
    { 0x2029c09f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439c0a2, 55 },
    { 0x1c09c0a2, 33 },
    { 0x95c9c0a3, 101 },
    { 0, 0 },
    { 0xa249c0a6, 61 },
    { 0xe089c0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed29c0ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b19c0b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2029c0b5, 34 },
    { 0, 0 },
    { 0x46d9c0b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff59c0be, 33 },
    { 0, 0 },
    { 0xbd19c0c0, 95 },
    { 0xcaf9c0c0, 33 },
    { 0xb0e9c0c2, 61 },
    { 0x29d9c0c3, 23 },
    { 0xab19c0c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a9c0c7, 61 },
    { 0, 0 },
    { 0x9b89c0c9, 78 },
    { 0, 0 },
    { 0x4569c0cb, 67 },
    { 0xc0b9c0cc, 33 },
    { 0xf479c0cd, 61 },
    { 0x7319c0cd, 61 },
    { 0x3039c0cf, 76 },
    { 0x9c49c0cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d9c0d3, 33 },
    { 0xda79c0d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee9c0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9c0dc, 55 },
    { 0x6489c0dc, 85 },
    { 0x70a9c0de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29c0e3, 100 },
    { 0x96c9c0e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c09c0ef, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459c0f2, 98 },
    { 0xbf99c0f3, 90 },
    { 0xe629c0f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee9c0f9, 55 },
    { 0xd939c0fa, 46 },
    { 0xb8c9c0fb, 90 },
    { 0x7459c0fc, 55 },
    { 0x6dd9c0fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189c105, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce9c10a, 55 },
    { 0, 0 },
    { 0x4e89c10c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7009c10f, 98 },
    { 0, 0 },
    { 0xdc89c111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf169c114, 76 },
    { 0, 0 },
    { 0x4779c116, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a9c11b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc9c122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9959c125, 93 },
    { 0x5799c125, 76 },
    { 0x4139c127, 52 },
    { 0x55c9c128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c49c12d, 61 },
    { 0xca39c12e, 95 },
    { 0xdd9c12f, 66 },
    { 0x4b29c12e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b9c138, 64 },
    { 0, 0 },
    { 0xbd59c13a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479c13e, 46 },
    { 0x25b9c13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd9c145, 34 },
    { 0xbd59c146, 90 },
    { 0x8069c146, 43 },
    { 0xce79c145, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b9c14c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9c153, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd139c15a, 67 },
    { 0xe799c15b, 90 },
    { 0x789c15c, 90 },
    { 0x93d9c15d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea39c161, 85 },
    { 0x3599c161, 55 },
    { 0xd709c163, 55 },
    { 0x9d89c162, 99 },
    { 0xfb99c165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b9c169, 43 },
    { 0x2c19c16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429c16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd89c171, 61 },
    { 0xb019c171, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x549c175, 61 },
    { 0x6149c175, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f39c180, 85 },
    { 0xce39c180, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c9c189, 33 },
    { 0x5049c189, 100 },
    { 0x3809c189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649c18e, 17 },
    { 0xcf99c18f, 95 },
    { 0xc799c190, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19c19c, 95 },
    { 0xe679c19c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f99c1a6, 76 },
    { 0, 0 },
    { 0x7739c1a8, 23 },
    { 0, 0 },
    { 0xb2b9c1aa, 90 },
    { 0x44e9c1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9c1af, 61 },
    { 0x8bd9c1b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769c1be, 61 },
    { 0xc539c1bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb69c1c7, 93 },
    { 0, 0 },
    { 0xa319c1c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8339c1cc, 98 },
    { 0x40b9c1cd, 85 },
    { 0xc929c1cd, 95 },
    { 0x79f9c1ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa039c1d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd739c1da, 100 },
    { 0xae79c1db, 33 },
    { 0, 0 },
    { 0x2b29c1dd, 90 },
    { 0x6669c1de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa999c1e3, 33 },
    { 0x9a79c1e3, 98 },
    { 0xffd9c1e4, 61 },
    { 0x9099c1e3, 76 },
    { 0xb739c1e7, 90 },
    { 0xe39c1e4, 76 },
    { 0xb479c1e9, 17 },
    { 0xdc39c1e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe519c1f1, 23 },
    { 0x10f9c1f2, 85 },
    { 0xf979c1f1, 46 },
    { 0x3789c1f3, 98 },
    { 0xa6e9c1f5, 23 },
    { 0xc89c1f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca9c1fb, 78 },
    { 0x1559c1fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb9c200, 61 },
    { 0x6889c201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf069c206, 61 },
    { 0x9309c206, 95 },
    { 0x2719c208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad9c20b, 33 },
    { 0, 0 },
    { 0x4319c20d, 78 },
    { 0xcac9c20d, 46 },
    { 0xafe9c20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9c212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e9c215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf749c219, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9949c21d, 23 },
    { 0x42c9c21e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9c222, 55 },
    { 0x1239c222, 61 },
    { 0x7579c222, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9c229, 98 },
    { 0, 0 },
    { 0xada9c22b, 98 },
    { 0xf009c22c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca9c22f, 90 },
    { 0xf4d9c230, 90 },
    { 0xc089c22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09c236, 23 },
    { 0xe969c237, 36 },
    { 0, 0 },
    { 0xbdf9c239, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8529c23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9569c241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd79c244, 33 },
    { 0xcc59c244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9c248, 95 },
    { 0x689c249, 85 },
    { 0, 0 },
    { 0x2819c24b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2689c24e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b9c251, 23 },
    { 0x6209c251, 61 },
    { 0xa749c253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c9c257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c59c25f, 76 },
    { 0, 0 },
    { 0xcca9c261, 23 },
    { 0xc4a9c262, 95 },
    { 0x9669c263, 33 },
    { 0, 0 },
    { 0x5fd9c265, 98 },
    { 0xa239c265, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd9c269, 98 },
    { 0x3a59c26a, 84 },
    { 0xbf39c26b, 61 },
    { 0xf9b9c26b, 61 },
    { 0xa0e9c26c, 95 },
    { 0, 0 },
    { 0xfef9c26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b9c273, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9c276, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7909c27a, 77 },
    { 0, 0 },
    { 0x9a39c27c, 55 },
    { 0x3b9c27d, 91 },
    { 0, 0 },
    { 0xc0b9c27f, 68 },
    { 0xb889c27f, 23 },
    { 0xbdc9c281, 55 },
    { 0x3809c282, 95 },
    { 0, 0 },
    { 0x5809c284, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda19c28a, 90 },
    { 0x4389c28b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b9c290, 55 },
    { 0x1829c291, 90 },
    { 0xca9c292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179c298, 98 },
    { 0xf479c299, 67 },
    { 0xf969c29a, 43 },
    { 0xebb9c29b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca99c2a0, 90 },
    { 0x9b69c2a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d9c2a5, 55 },
    { 0x9059c2a6, 65 },
    { 0, 0 },
    { 0x1b79c2a8, 93 },
    { 0, 0 },
    { 0x1719c2aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d49c2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269c2b8, 81 },
    { 0x5e59c2b9, 34 },
    { 0xfcd9c2b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3369c2bf, 88 },
    { 0xc4c9c2bf, 84 },
    { 0, 0 },
    { 0x4129c2c2, 55 },
    { 0x8a49c2c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4589c2cb, 98 },
    { 0, 0 },
    { 0xb239c2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa89c2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a39c2d6, 52 },
    { 0x68d9c2d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b9c2dc, 33 },
    { 0x7849c2dd, 76 },
    { 0xfa09c2dc, 24 },
    { 0, 0 },
    { 0xfdb9c2e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9c2e3, 23 },
    { 0x4449c2e4, 28 },
    { 0x15d9c2e3, 90 },
    { 0, 0 },
    { 0xc639c2e7, 84 },
    { 0x8809c2e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa299c2eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339c2ef, 98 },
    { 0xb239c2f0, 43 },
    { 0x5789c2f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba9c2f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f09c2f8, 66 },
    { 0x3729c2f8, 33 },
    { 0x2179c2f9, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x8039c2fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3889c301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79c305, 95 },
    { 0, 0 },
    { 0x1629c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9c30a, 90 },
    { 0x649c30a, 43 },
    { 0xec89c30c, 77 },
    { 0x41a9c30d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b9c315, 43 },
    { 0x61c9c316, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a9c31e, 100 },
    { 0x9119c31f, 98 },
    { 0x1c99c31f, 90 },
    { 0, 0 },
    { 0xd769c322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c325, 87 },
    { 0x7d89c326, 77 },
    { 0xfc19c327, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9c330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c9c333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39c338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc019c33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde79c33f, 98 },
    { 0xc859c340, 91 },
    { 0xc6a9c340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4699c344, 65 },
    { 0, 0 },
    { 0xc1f9c346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe769c34e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539c356, 90 },
    { 0xefd9c357, 75 },
    { 0xe2b9c358, 61 },
    { 0xcea9c359, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf709c35d, 33 },
    { 0, 0 },
    { 0x5029c35f, 90 },
    { 0x26b9c360, 33 },
    { 0x48a9c361, 66 },
    { 0x9409c362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9c366, 87 },
    { 0xc9b9c367, 46 },
    { 0x9709c367, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa349c36b, 85 },
    { 0x31f9c36c, 87 },
    { 0x59b9c36c, 91 },
    { 0xe0c9c36e, 34 },
    { 0x6759c36f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff59c375, 91 },
    { 0, 0 },
    { 0x54f9c377, 90 },
    { 0x3c39c378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9309c380, 88 },
    { 0x3bd9c381, 33 },
    { 0xb59c382, 55 },
    { 0x7709c383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299c387, 95 },
    { 0xbca9c387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd669c38b, 34 },
    { 0x5169c38c, 85 },
    { 0xa689c38b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1739c3a0, 98 },
    { 0, 0 },
    { 0xfb29c3a2, 90 },
    { 0xa109c3a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c49c3a6, 43 },
    { 0xda59c3a7, 33 },
    { 0x8529c3a6, 98 },
    { 0, 0 },
    { 0xb259c3aa, 98 },
    { 0, 0 },
    { 0x9899c3ac, 46 },
    { 0xd389c3ad, 100 },
    { 0x9d59c3ae, 33 },
    { 0xbbe9c3ae, 23 },
    { 0xa919c3af, 98 },
    { 0xd9c3ae, 23 },
    { 0xcad9c3ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f9c3b6, 90 },
    { 0, 0 },
    { 0xc469c3b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f9c3bb, 76 },
    { 0x6199c3bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29c3bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e9c3c2, 99 },
    { 0x2d49c3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89c3c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee9c3cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6409c3cf, 61 },
    { 0, 0 },
    { 0xebb9c3d1, 98 },
    { 0xdc79c3d1, 90 },
    { 0, 0 },
    { 0x9179c3d4, 61 },
    { 0, 0 },
    { 0x9089c3d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c69c3dc, 33 },
    { 0x7979c3dd, 23 },
    { 0x629c3de, 95 },
    { 0x90c9c3df, 99 },
    { 0, 0 },
    { 0x5e39c3e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e9c3e4, 43 },
    { 0x2899c3e5, 23 },
    { 0, 0 },
    { 0xbcc9c3e7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9119c3ea, 34 },
    { 0xaa09c3ea, 55 },
    { 0x2029c3ec, 33 },
    { 0xa3b9c3ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f9c3ff, 98 },
    { 0x8c59c400, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419c403, 23 },
    { 0, 0 },
    { 0x9459c405, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc9c40c, 33 },
    { 0, 0 },
    { 0x7389c40e, 87 },
    { 0xf119c40e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc609c414, 90 },
    { 0, 0 },
    { 0xbea9c416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c9c41b, 33 },
    { 0xf159c41c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639c420, 91 },
    { 0xa329c421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4089c424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319c42b, 36 },
    { 0xb559c42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c79c431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc9c434, 95 },
    { 0, 0 },
    { 0x33b9c436, 93 },
    { 0x9b69c437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa9c43b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9c43e, 76 },
    { 0x92a9c43e, 100 },
    { 0x5c39c440, 33 },
    { 0x9799c440, 66 },
    { 0x33d9c442, 55 },
    { 0x5129c443, 23 },
    { 0xc089c443, 34 },
    { 0xf899c442, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e9c448, 76 },
    { 0, 0 },
    { 0x23d9c44a, 52 },
    { 0, 0 },
    { 0x1a99c44c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f9c44f, 23 },
    { 0x4ec9c450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359c454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec9c459, 33 },
    { 0x8019c45a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e9c460, 98 },
    { 0, 0 },
    { 0xe39c462, 52 },
    { 0, 0 },
    { 0x5f39c464, 43 },
    { 0xd739c465, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c9c46b, 76 },
    { 0x3869c46c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8909c46f, 33 },
    { 0xb5b9c470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1059c473, 67 },
    { 0x9039c474, 77 },
    { 0x76c9c475, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d9c478, 95 },
    { 0x5419c479, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b9c486, 90 },
    { 0, 0 },
    { 0xdff9c488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419c48e, 90 },
    { 0x2d79c48f, 66 },
    { 0, 0 },
    { 0x77c9c491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639c499, 90 },
    { 0xcae9c499, 93 },
    { 0x4bd9c49b, 90 },
    { 0x35f9c49c, 36 },
    { 0, 0 },
    { 0x5d49c49e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8029c4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c4a5, 34 },
    { 0xe1e9c4a6, 95 },
    { 0x3ed9c4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669c4aa, 78 },
    { 0, 0 },
    { 0xbcb9c4ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019c4af, 95 },
    { 0x4559c4b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef9c4b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9c4b7, 34 },
    { 0, 0 },
    { 0x5189c4b9, 5 },
    { 0x5ad9c4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd209c4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f9c4c4, 34 },
    { 0, 0 },
    { 0x16b9c4c6, 43 },
    { 0xa769c4c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a29c4ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc9c4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc89c4d3, 90 },
    { 0x1159c4d3, 67 },
    { 0xfb89c4d5, 90 },
    { 0x4da9c4d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8949c4dc, 43 },
    { 0, 0 },
    { 0xa4b9c4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8249c4e3, 34 },
    { 0, 0 },
    { 0x669c4e5, 61 },
    { 0x6fc9c4e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf9c4e9, 95 },
    { 0xc179c4ea, 98 },
    { 0x25b9c4eb, 76 },
    { 0x2d69c4ec, 76 },
    { 0x4c59c4ed, 81 },
    { 0xf549c4ee, 61 },
    { 0, 0 },
    { 0x6959c4f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8279c4f6, 33 },
    { 0x20e9c4f7, 61 },
    { 0x41d9c4f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a89c502, 99 },
    { 0xc199c503, 67 },
    { 0, 0 },
    { 0x4639c505, 23 },
    { 0xc879c506, 23 },
    { 0xbe89c507, 90 },
    { 0xb8f9c505, 43 },
    { 0, 0 },
    { 0x1ec9c50a, 61 },
    { 0xa929c50a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729c50e, 23 },
    { 0, 0 },
    { 0xc8d9c510, 61 },
    { 0, 0 },
    { 0x8099c512, 33 },
    { 0x2e9c512, 100 },
    { 0, 0 },
    { 0x36f9c515, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9409c519, 91 },
    { 0x2be9c51a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139c51f, 61 },
    { 0xb6e9c520, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9c523, 61 },
    { 0x4da9c524, 65 },
    { 0xd579c524, 90 },
    { 0x6dc9c526, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd29c52c, 23 },
    { 0xa359c52d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f9c530, 61 },
    { 0xaa69c531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd09c536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a39c53a, 61 },
    { 0x23d9c53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad89c53f, 90 },
    { 0x35c9c540, 76 },
    { 0x31b9c541, 90 },
    { 0xad29c540, 95 },
    { 0x4299c543, 33 },
    { 0x8449c544, 68 },
    { 0xb259c545, 95 },
    { 0xfff9c545, 95 },
    { 0, 0 },
    { 0x589c548, 23 },
    { 0, 0 },
    { 0xcf99c54a, 55 },
    { 0, 0 },
    { 0x989c54c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19c555, 93 },
    { 0, 0 },
    { 0x7709c557, 77 },
    { 0x2be9c558, 61 },
    { 0, 0 },
    { 0x8059c55a, 61 },
    { 0x3499c55a, 90 },
    { 0x93b9c55c, 90 },
    { 0x5329c55a, 43 },
    { 0, 0 },
    { 0xe3c9c55f, 95 },
    { 0x3339c560, 47 },
    { 0xb269c561, 91 },
    { 0x5a99c561, 46 },
    { 0x31d9c560, 23 },
    { 0x439c564, 97 },
    { 0, 0 },
    { 0x42b9c566, 93 },
    { 0xee39c567, 68 },
    { 0xd6e9c568, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5749c56b, 23 },
    { 0xaab9c56c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f9c575, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e29c578, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd589c57b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b9c585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd29c58c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb9c590, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb89c596, 98 },
    { 0xd659c596, 65 },
    { 0, 0 },
    { 0xc769c599, 34 },
    { 0x3b69c59a, 46 },
    { 0xbdb9c59a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb529c5a1, 34 },
    { 0xe7d9c5a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9c5a6, 95 },
    { 0, 0 },
    { 0xbec9c5a8, 61 },
    { 0x8499c5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149c5ac, 23 },
    { 0x32a9c5ad, 61 },
    { 0xd429c5ae, 46 },
    { 0xd5e9c5ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1289c5b4, 95 },
    { 0xf319c5b5, 33 },
    { 0x7789c5b6, 33 },
    { 0, 0 },
    { 0xc209c5b8, 90 },
    { 0x5f59c5b9, 90 },
    { 0xe419c5b9, 61 },
    { 0xcaa9c5bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c59c5be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e9c5c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499c5c7, 43 },
    { 0x6e39c5c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d89c5ce, 33 },
    { 0x1d59c5cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae9c5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139c5da, 90 },
    { 0, 0 },
    { 0x79b9c5dc, 95 },
    { 0x1339c5dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d9c5e1, 76 },
    { 0x2bc9c5e1, 95 },
    { 0xc279c5e3, 95 },
    { 0xa429c5e4, 43 },
    { 0x6a69c5e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8109c5ea, 43 },
    { 0x6ab9c5ea, 55 },
    { 0, 0 },
    { 0xed19c5ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c5f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c5f3, 78 },
    { 0xdbf9c5f4, 99 },
    { 0x9c9c5f5, 88 },
    { 0x8179c5f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c9c5fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339c5fe, 95 },
    { 0, 0 },
    { 0x8299c600, 55 },
    { 0, 0 },
    { 0xbcb9c602, 34 },
    { 0x7c19c603, 43 },
    { 0xa2c9c604, 66 },
    { 0, 0 },
    { 0xe4d9c606, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd9c60b, 40 },
    { 0xe7f9c60c, 33 },
    { 0xbf79c60d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4629c612, 55 },
    { 0x67c9c613, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9c617, 76 },
    { 0x4be9c618, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9c61b, 94 },
    { 0x59e9c61c, 52 },
    { 0xe109c61d, 90 },
    { 0xbfe9c61b, 61 },
    { 0xa9f9c61d, 67 },
    { 0, 0 },
    { 0xdb69c621, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c59c627, 46 },
    { 0x2409c627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac69c62d, 43 },
    { 0x10a9c62d, 98 },
    { 0xf4c9c62e, 90 },
    { 0, 0 },
    { 0x9919c631, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e9c636, 76 },
    { 0, 0 },
    { 0x9849c638, 90 },
    { 0, 0 },
    { 0x7f99c63a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe539c63d, 61 },
    { 0, 0 },
    { 0x85e9c63f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb879c653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99c65c, 87 },
    { 0xd599c65c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f9c663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb9c666, 95 },
    { 0x6f19c667, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779c673, 90 },
    { 0x5c19c673, 65 },
    { 0xab79c675, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9c67b, 90 },
    { 0, 0 },
    { 0x34e9c67d, 43 },
    { 0xe419c67d, 34 },
    { 0x1059c67d, 44 },
    { 0xd3d9c680, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa639c685, 95 },
    { 0, 0 },
    { 0xd659c687, 34 },
    { 0xa9d9c687, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d9c68c, 90 },
    { 0x3019c68c, 98 },
    { 0xcaf9c68c, 34 },
    { 0, 0 },
    { 0x7b39c690, 61 },
    { 0xb719c691, 66 },
    { 0xee99c692, 23 },
    { 0xb459c693, 81 },
    { 0xf579c694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba9c69a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe499c69e, 90 },
    { 0x7d69c69e, 98 },
    { 0x5bb9c6a0, 90 },
    { 0x5d79c69f, 43 },
    { 0x3509c6a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329c6a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9c6ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b9c6ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2779c6b1, 84 },
    { 0xa189c6b1, 90 },
    { 0xac39c6b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5569c6b8, 93 },
    { 0x1ed9c6b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7339c6bd, 78 },
    { 0x33a9c6bd, 95 },
    { 0xc929c6bf, 33 },
    { 0x7c49c6be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669c6c7, 98 },
    { 0xe0a9c6c8, 23 },
    { 0xe9e9c6c7, 90 },
    { 0xe139c6ca, 76 },
    { 0xbc9c6c7, 61 },
    { 0xe369c6cb, 61 },
    { 0x3c39c6ca, 55 },
    { 0x83c9c6cb, 55 },
    { 0x979c6ce, 95 },
    { 0, 0 },
    { 0xc89c6d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c9c6d7, 95 },
    { 0x7f29c6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949c6dd, 67 },
    { 0xd2c9c6de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5479c6e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639c6e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9c6f0, 100 },
    { 0xe609c6f1, 90 },
    { 0, 0 },
    { 0xd6a9c6f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c9c6f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b9c6fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd879c6fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c09c701, 90 },
    { 0xa899c702, 33 },
    { 0, 0 },
    { 0x2ce9c704, 99 },
    { 0x32c9c705, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa9c70d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf619c710, 90 },
    { 0, 0 },
    { 0x7f69c712, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe029c718, 61 },
    { 0, 0 },
    { 0x1af9c71a, 90 },
    { 0x25e9c71b, 55 },
    { 0x9c59c71b, 95 },
    { 0x6989c71a, 43 },
    { 0xb859c71e, 98 },
    { 0x5869c71e, 90 },
    { 0x7549c71f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19c726, 76 },
    { 0, 0 },
    { 0xcb49c728, 98 },
    { 0, 0 },
    { 0x2d19c72a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d9c72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d9c733, 55 },
    { 0x6dd9c733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd09c737, 61 },
    { 0xc0f9c738, 46 },
    { 0, 0 },
    { 0xa399c73a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9c73d, 93 },
    { 0xc669c73d, 95 },
    { 0xf009c73f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff9c743, 95 },
    { 0xa79c743, 98 },
    { 0xfc29c745, 61 },
    { 0xe349c745, 61 },
    { 0xf799c745, 90 },
    { 0xb1f9c746, 34 },
    { 0, 0 },
    { 0xbb99c74a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29c74e, 90 },
    { 0xee09c74f, 90 },
    { 0x8089c750, 90 },
    { 0xca9c751, 82 },
    { 0xabd9c751, 90 },
    { 0x2d9c752, 90 },
    { 0x5449c752, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209c757, 95 },
    { 0, 0 },
    { 0x9799c759, 75 },
    { 0x9a09c75a, 34 },
    { 0xc3b9c75a, 99 },
    { 0xe729c75c, 90 },
    { 0x2019c759, 33 },
    { 0xce79c75e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89c763, 61 },
    { 0x28f9c764, 23 },
    { 0x8159c764, 93 },
    { 0, 0 },
    { 0x8c19c767, 95 },
    { 0, 0 },
    { 0x9d69c769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e9c771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c778, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2789c77c, 90 },
    { 0, 0 },
    { 0x2f9c77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c9c784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949c787, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9c78a, 83 },
    { 0, 0 },
    { 0xa8a9c78c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc09c78f, 90 },
    { 0, 0 },
    { 0x5649c791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9c794, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc89c798, 85 },
    { 0x4109c799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9c79c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd19c7a0, 98 },
    { 0x8ec9c7a1, 55 },
    { 0, 0 },
    { 0x1bb9c7a3, 33 },
    { 0xc989c7a3, 98 },
    { 0x1c19c7a4, 40 },
    { 0xc689c7a4, 99 },
    { 0xe79c7a6, 69 },
    { 0, 0 },
    { 0xce19c7a9, 67 },
    { 0, 0 },
    { 0x4a69c7ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef9c7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f99c7b4, 99 },
    { 0, 0 },
    { 0xf59c7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c9c7bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd029c7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c9c7cb, 52 },
    { 0, 0 },
    { 0xef79c7cd, 98 },
    { 0xe979c7ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4599c7d4, 33 },
    { 0x8669c7d5, 33 },
    { 0, 0 },
    { 0x389c7d7, 95 },
    { 0xa0f9c7d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99c7df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab9c7e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb9c7e5, 33 },
    { 0xabf9c7e6, 93 },
    { 0x46f9c7e6, 90 },
    { 0xfe39c7e7, 33 },
    { 0xa419c7e9, 23 },
    { 0x4349c7e8, 61 },
    { 0xed9c7eb, 66 },
    { 0, 0 },
    { 0xacf9c7ed, 91 },
    { 0x2b49c7ed, 95 },
    { 0xb0a9c7ef, 99 },
    { 0xd739c7ef, 91 },
    { 0, 0 },
    { 0x2b09c7f2, 95 },
    { 0, 0 },
    { 0xe1f9c7f4, 91 },
    { 0, 0 },
    { 0x29e9c7f6, 33 },
    { 0xaa99c7f6, 93 },
    { 0x27e9c7f7, 93 },
    { 0x7269c7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb29c7fe, 93 },
    { 0xd769c7fe, 33 },
    { 0x3d99c7fe, 61 },
    { 0, 0 },
    { 0x8e99c802, 61 },
    { 0x5499c802, 65 },
    { 0x6789c804, 76 },
    { 0x1809c805, 68 },
    { 0x5f79c805, 90 },
    { 0x51d9c804, 33 },
    { 0xbc09c806, 98 },
    { 0x6199c809, 98 },
    { 0x7609c809, 90 },
    { 0xd049c807, 33 },
    { 0x1649c80c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d9c80f, 91 },
    { 0x8859c810, 76 },
    { 0, 0 },
    { 0x4819c812, 76 },
    { 0, 0 },
    { 0xc089c814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xade9c817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299c81b, 90 },
    { 0xd2d9c81c, 95 },
    { 0xea69c81d, 66 },
    { 0, 0 },
    { 0xd2c9c81f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19c822, 96 },
    { 0xb089c822, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd29c82a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269c82f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a79c832, 23 },
    { 0x34d9c833, 46 },
    { 0x2119c834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9c83a, 98 },
    { 0, 0 },
    { 0xbfa9c83c, 98 },
    { 0x299c83d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799c842, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9c84b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c9c84f, 23 },
    { 0x91b9c84f, 76 },
    { 0, 0 },
    { 0x9529c852, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429c85b, 34 },
    { 0, 0 },
    { 0xb09c85d, 23 },
    { 0xf5c9c85e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c9c862, 46 },
    { 0x6169c863, 98 },
    { 0xb1e9c864, 66 },
    { 0xe729c865, 61 },
    { 0x95d9c866, 85 },
    { 0, 0 },
    { 0x77e9c868, 46 },
    { 0xfbf9c868, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf529c86c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd9c86f, 33 },
    { 0xa289c86f, 65 },
    { 0x1859c871, 55 },
    { 0x17b9c871, 90 },
    { 0xbc89c873, 84 },
    { 0xcc69c873, 78 },
    { 0xe959c875, 98 },
    { 0x7db9c871, 90 },
    { 0x39c9c871, 88 },
    { 0x9d59c873, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3939c87c, 98 },
    { 0, 0 },
    { 0x91f9c87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1129c882, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2889c885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09c889, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9c88d, 77 },
    { 0xd779c88e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe999c893, 34 },
    { 0x5ac9c893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a9c89b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc39c89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf159c8a7, 90 },
    { 0x2299c8a7, 75 },
    { 0, 0 },
    { 0x149c8aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e9c8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff9c8b3, 98 },
    { 0x1e79c8b4, 98 },
    { 0x5839c8b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3299c8c0, 34 },
    { 0x6499c8c0, 61 },
    { 0x35c9c8c2, 93 },
    { 0x55a9c8c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda99c8c7, 99 },
    { 0xa39c8c8, 46 },
    { 0x589c8c9, 66 },
    { 0x2029c8c9, 55 },
    { 0x9a9c8c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09c8d0, 90 },
    { 0x1bd9c8d1, 77 },
    { 0xe509c8d1, 23 },
    { 0x809c8d3, 98 },
    { 0x9889c8d1, 95 },
    { 0x6f59c8d3, 90 },
    { 0x8769c8d1, 95 },
    { 0x8f99c8d3, 77 },
    { 0, 0 },
    { 0x4d39c8d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf009c8df, 95 },
    { 0x32c9c8df, 61 },
    { 0x97e9c8e1, 67 },
    { 0x7079c8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d9c8e7, 99 },
    { 0x48f9c8e8, 78 },
    { 0xea59c8e7, 76 },
    { 0xedc9c8ea, 43 },
    { 0xead9c8ea, 68 },
    { 0xe659c8ec, 33 },
    { 0, 0 },
    { 0xf269c8ee, 93 },
    { 0x56c9c8ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a9c8f6, 43 },
    { 0x7cc9c8f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f9c8fe, 90 },
    { 0xc879c8ff, 23 },
    { 0x3af9c900, 95 },
    { 0, 0 },
    { 0x7169c902, 90 },
    { 0x9c39c902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8049c90a, 90 },
    { 0x7009c90b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c49c90e, 38 },
    { 0x5859c90f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9c91e, 65 },
    { 0xc3c9c91f, 61 },
    { 0, 0 },
    { 0x6e49c921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf9c924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f9c927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d69c92a, 91 },
    { 0xc679c92b, 33 },
    { 0x4189c92c, 95 },
    { 0, 0 },
    { 0xfe69c92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc9c936, 94 },
    { 0xae59c937, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09c93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109c940, 90 },
    { 0, 0 },
    { 0x3f39c942, 75 },
    { 0, 0 },
    { 0x1c99c944, 76 },
    { 0x4b9c944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a9c949, 76 },
    { 0x1639c94a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639c950, 33 },
    { 0x2a39c951, 33 },
    { 0x2159c951, 99 },
    { 0xfac9c952, 90 },
    { 0, 0 },
    { 0xece9c955, 81 },
    { 0x3e09c955, 87 },
    { 0x4fb9c957, 33 },
    { 0xccb9c957, 43 },
    { 0x9da9c959, 61 },
    { 0x6179c958, 33 },
    { 0xa799c959, 61 },
    { 0, 0 },
    { 0x9a09c95d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d9c961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b9c964, 33 },
    { 0x3e09c964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7879c969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab9c96d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7329c973, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9c97c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1789c983, 61 },
    { 0x1629c984, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6939c989, 23 },
    { 0x6d49c98a, 90 },
    { 0, 0 },
    { 0x419c98c, 90 },
    { 0x83b9c98d, 98 },
    { 0x5049c98e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3269c992, 80 },
    { 0xe609c993, 61 },
    { 0x539c994, 33 },
    { 0x6049c995, 90 },
    { 0x4df9c996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa9c999, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b59c9a3, 98 },
    { 0xf999c9a3, 66 },
    { 0xd909c9a5, 98 },
    { 0x8439c9a6, 33 },
    { 0x9809c9a7, 85 },
    { 0, 0 },
    { 0xfe09c9a9, 90 },
    { 0, 0 },
    { 0x3cb9c9ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9c9ae, 61 },
    { 0x5199c9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc9c9b6, 87 },
    { 0xd4d9c9b7, 23 },
    { 0, 0 },
    { 0x5289c9b9, 55 },
    { 0x9ac9c9ba, 17 },
    { 0x4169c9bb, 76 },
    { 0x6569c9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49c9bf, 65 },
    { 0x2cf9c9c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec9c9c7, 76 },
    { 0x7629c9c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b9c9cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459c9d0, 99 },
    { 0xca29c9d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5279c9d5, 99 },
    { 0, 0 },
    { 0x4199c9d7, 95 },
    { 0, 0 },
    { 0xfb89c9d9, 34 },
    { 0x3259c9da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9c9dd, 90 },
    { 0x799c9de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe939c9e2, 46 },
    { 0xc6e9c9e3, 43 },
    { 0, 0 },
    { 0xcda9c9e5, 68 },
    { 0xce89c9e6, 75 },
    { 0x3559c9e5, 33 },
    { 0xa959c9e6, 17 },
    { 0xb159c9e8, 33 },
    { 0xfe59c9ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7529c9ed, 23 },
    { 0x1dc9c9ed, 85 },
    { 0, 0 },
    { 0xc1e9c9f0, 90 },
    { 0xcbe9c9f1, 95 },
    { 0xba9c9f0, 61 },
    { 0x9e59c9f0, 23 },
    { 0xc59c9f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd89c9f7, 46 },
    { 0xf879c9f7, 78 },
    { 0, 0 },
    { 0x8449c9fa, 80 },
    { 0, 0 },
    { 0x8d59c9fc, 76 },
    { 0x4ad9c9fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a89ca05, 95 },
    { 0, 0 },
    { 0x3b49ca07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3789ca0d, 23 },
    { 0, 0 },
    { 0xafb9ca0f, 61 },
    { 0xe059ca0f, 77 },
    { 0x7539ca11, 23 },
    { 0x3af9ca11, 95 },
    { 0xe199ca12, 98 },
    { 0, 0 },
    { 0xc6c9ca15, 93 },
    { 0, 0 },
    { 0x7af9ca17, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9ca1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2609ca24, 91 },
    { 0x6129ca25, 59 },
    { 0, 0 },
    { 0x5f59ca27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99ca2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9ca33, 98 },
    { 0x6d49ca34, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb619ca3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569ca40, 66 },
    { 0x2319ca41, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb569ca48, 61 },
    { 0, 0 },
    { 0x7b49ca4a, 98 },
    { 0x9e19ca4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f9ca4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b09ca52, 43 },
    { 0x8809ca52, 43 },
    { 0, 0 },
    { 0x7889ca55, 8 },
    { 0, 0 },
    { 0x3cb9ca57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b29ca5a, 95 },
    { 0x9029ca5b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89ca5f, 43 },
    { 0, 0 },
    { 0x98d9ca61, 90 },
    { 0xe419ca61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9ca67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209ca6b, 67 },
    { 0xce29ca6c, 33 },
    { 0xb349ca6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649ca74, 78 },
    { 0x18f9ca75, 56 },
    { 0x8a09ca75, 95 },
    { 0, 0 },
    { 0x1a59ca78, 95 },
    { 0x80a9ca78, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d59ca7d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9ca81, 76 },
    { 0xfbf9ca81, 33 },
    { 0x3419ca83, 33 },
    { 0x1609ca82, 95 },
    { 0xa4e9ca85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5029ca88, 84 },
    { 0x33e9ca88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d9ca93, 33 },
    { 0x17f9ca94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe229ca98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59ca9f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9caa4, 33 },
    { 0x5b59caa5, 65 },
    { 0xa129caa4, 61 },
    { 0xc679caa7, 98 },
    { 0xd629caa5, 61 },
    { 0x76e9caa8, 95 },
    { 0xe19caaa, 23 },
    { 0x73a9caa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c9cab0, 90 },
    { 0x4fa9cab1, 33 },
    { 0x4359cab1, 98 },
    { 0, 0 },
    { 0x50d9cab4, 61 },
    { 0, 0 },
    { 0x349cab6, 23 },
    { 0x11d9cab7, 61 },
    { 0xcbb9cab8, 46 },
    { 0x3469cab9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8629cac3, 65 },
    { 0xe399cac4, 94 },
    { 0x9ad9cac5, 23 },
    { 0x33b9cac4, 76 },
    { 0x8c9cac4, 33 },
    { 0, 0 },
    { 0x5af9cac9, 66 },
    { 0x3ef9caca, 17 },
    { 0x1c19caca, 91 },
    { 0, 0 },
    { 0xc2c9cacd, 23 },
    { 0x3209cace, 100 },
    { 0xab19cacf, 95 },
    { 0x4c99cad0, 76 },
    { 0x36c9cace, 90 },
    { 0xcd79cace, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3389cad5, 100 },
    { 0xd6b9cad6, 90 },
    { 0, 0 },
    { 0x8699cad8, 81 },
    { 0xe4a9cad8, 66 },
    { 0, 0 },
    { 0xe459cadb, 76 },
    { 0xbc79cadc, 95 },
    { 0x24a9cadd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9cae1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb489cae4, 33 },
    { 0x2629cae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99caeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4679caee, 44 },
    { 0x4639caee, 90 },
    { 0, 0 },
    { 0x8839caf1, 23 },
    { 0, 0 },
    { 0x4b09caf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f9caf8, 33 },
    { 0, 0 },
    { 0xc39cafa, 66 },
    { 0x70f9cafa, 91 },
    { 0xec69cafb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6319caff, 17 },
    { 0, 0 },
    { 0x1f29cb01, 33 },
    { 0x7cb9cb01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e9cb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4569cb09, 46 },
    { 0xf899cb0a, 66 },
    { 0, 0 },
    { 0xc369cb0c, 90 },
    { 0x4149cb0c, 84 },
    { 0x50a9cb0e, 95 },
    { 0xb839cb0f, 96 },
    { 0x5a59cb10, 95 },
    { 0x2fc9cb10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb769cb1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf09cb2e, 61 },
    { 0x2289cb2f, 98 },
    { 0xa879cb2e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae9cb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb649cb3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c9cb41, 55 },
    { 0, 0 },
    { 0x8029cb43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9cb46, 98 },
    { 0, 0 },
    { 0xac69cb48, 61 },
    { 0xd0b9cb49, 98 },
    { 0x1989cb4a, 55 },
    { 0xaef9cb48, 33 },
    { 0x35f9cb4c, 93 },
    { 0x899cb4d, 95 },
    { 0xb1f9cb4b, 78 },
    { 0xa3f9cb4a, 23 },
    { 0xf7f9cb4a, 99 },
    { 0xc509cb4a, 93 },
    { 0x9fc9cb52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9cb55, 61 },
    { 0x58e9cb56, 23 },
    { 0xe0c9cb57, 90 },
    { 0xfb19cb58, 95 },
    { 0xc4b9cb59, 85 },
    { 0xd4c9cb5a, 98 },
    { 0, 0 },
    { 0xc2e9cb5c, 61 },
    { 0, 0 },
    { 0x3dd9cb5e, 90 },
    { 0, 0 },
    { 0x91c9cb60, 46 },
    { 0xa139cb61, 98 },
    { 0x309cb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29cb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99cb6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b29cb6e, 93 },
    { 0x739cb6e, 46 },
    { 0x77b9cb70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479cb75, 64 },
    { 0x8819cb76, 90 },
    { 0, 0 },
    { 0xfae9cb78, 100 },
    { 0xb529cb78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc749cb7d, 81 },
    { 0x8c9cb7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649cb86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a9cb8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7959cb8d, 90 },
    { 0xf439cb8e, 90 },
    { 0x9ab9cb8f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9cb93, 98 },
    { 0x1409cb93, 88 },
    { 0x209cb95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559cb9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429cb9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a19cba2, 52 },
    { 0, 0 },
    { 0xb819cba4, 90 },
    { 0, 0 },
    { 0x76d9cba6, 46 },
    { 0xcdf9cba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe599cbaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3839cbad, 33 },
    { 0, 0 },
    { 0x4829cbaf, 85 },
    { 0, 0 },
    { 0xf899cbb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9cbb8, 34 },
    { 0xfb19cbb9, 98 },
    { 0x79f9cbba, 84 },
    { 0xa279cbb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b99cbbe, 87 },
    { 0x7729cbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29cbc2, 95 },
    { 0xea99cbc3, 96 },
    { 0x9a59cbc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9cbce, 84 },
    { 0, 0 },
    { 0x5c79cbd0, 33 },
    { 0x8b79cbd0, 55 },
    { 0, 0 },
    { 0x9e59cbd3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9cbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbdc, 75 },
    { 0x7089cbdd, 34 },
    { 0xc009cbdd, 46 },
    { 0x6eb9cbdf, 34 },
    { 0xc809cbe0, 33 },
    { 0x1bb9cbdf, 90 },
    { 0x67e9cbe2, 61 },
    { 0x6769cbdc, 90 },
    { 0, 0 },
    { 0x1429cbe5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5009cbe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa979cbed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe409cbf0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xff39cbf3, 95 },
    { 0xfed9cbf4, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbfe, 75 },
    { 0x8229cbff, 66 },
    { 0xdd09cbff, 17 },
    { 0xf4c9cc01, 46 },
    { 0x3039cc02, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd989cc07, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7559cc10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9cc16, 23 },
    { 0, 0 },
    { 0xfe19cc18, 95 },
    { 0x2139cc18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa579cc1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19cc26, 33 },
    { 0x6dc9cc26, 33 },
    { 0xb889cc27, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd9cc2c, 68 },
    { 0x1289cc2d, 61 },
    { 0x2b39cc2c, 98 },
    { 0x10a9cc2f, 61 },
    { 0x9a49cc2f, 98 },
    { 0xe409cc30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4399cc34, 98 },
    { 0x2009cc35, 61 },
    { 0x8309cc35, 99 },
    { 0xa799cc37, 76 },
    { 0, 0 },
    { 0xa4f9cc39, 52 },
    { 0xd229cc3a, 33 },
    { 0, 0 },
    { 0x8809cc3c, 33 },
    { 0x269cc3d, 76 },
    { 0xe159cc3c, 61 },
    { 0xa8d9cc3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e19cc43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9cc47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab9cc4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf779cc51, 34 },
    { 0, 0 },
    { 0xfd59cc53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc69cc56, 95 },
    { 0xf9c9cc57, 99 },
    { 0x4a39cc58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9589cc5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439cc62, 33 },
    { 0xe9c9cc63, 43 },
    { 0xa449cc63, 61 },
    { 0x12e9cc65, 61 },
    { 0xc849cc66, 90 },
    { 0x7bd9cc67, 91 },
    { 0x3609cc68, 33 },
    { 0x3d29cc67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9cc6c, 93 },
    { 0x8c59cc6d, 33 },
    { 0, 0 },
    { 0xbea9cc6f, 61 },
    { 0x2f69cc6f, 23 },
    { 0xb359cc70, 23 },
    { 0xdbe9cc72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd899cc79, 76 },
    { 0x1999cc79, 61 },
    { 0, 0 },
    { 0x4ab9cc7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9cc80, 98 },
    { 0, 0 },
    { 0xda89cc82, 90 },
    { 0xd7d9cc83, 65 },
    { 0xb029cc84, 98 },
    { 0xe089cc83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c9cc8b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b19cc8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c49cc91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9cc96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b9cc99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579cc9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6299cc9f, 52 },
    { 0x53d9cc9f, 95 },
    { 0x2ef9cca1, 33 },
    { 0xa4a9cca2, 33 },
    { 0x3639cca3, 76 },
    { 0xe989cca3, 61 },
    { 0x4fd9cca1, 98 },
    { 0x9e49cca1, 55 },
    { 0x3929cca7, 65 },
    { 0x14c9cca8, 33 },
    { 0x3fc9cca8, 90 },
    { 0xea89cca9, 99 },
    { 0, 0 },
    { 0xf8f9ccac, 98 },
    { 0x1859ccad, 90 },
    { 0x2e69ccae, 90 },
    { 0, 0 },
    { 0xcc69ccb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649ccb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e19ccb6, 33 },
    { 0x1779ccb6, 61 },
    { 0x1c39ccb8, 67 },
    { 0xc899ccb7, 91 },
    { 0xc829ccb9, 87 },
    { 0xa9d9ccba, 59 },
    { 0x53b9ccb9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6579ccbf, 33 },
    { 0, 0 },
    { 0x1ef9ccc1, 91 },
    { 0xf019ccc1, 61 },
    { 0x5f79ccc3, 90 },
    { 0x6539ccc2, 25 },
    { 0x769ccc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b59ccca, 34 },
    { 0xcf79cccb, 61 },
    { 0xf8e9cccc, 61 },
    { 0x6469cccb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9ccd4, 95 },
    { 0x7d69ccd4, 90 },
    { 0, 0 },
    { 0x19ccd7, 23 },
    { 0xd2d9ccd8, 61 },
    { 0x1439ccd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859ccdf, 90 },
    { 0x4da9cce0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8329cce3, 78 },
    { 0x9169cce4, 55 },
    { 0, 0 },
    { 0xd679cce6, 100 },
    { 0xfc99cce6, 61 },
    { 0xa3c9cce8, 61 },
    { 0x4589cce8, 23 },
    { 0, 0 },
    { 0x8da9cceb, 95 },
    { 0x3829ccec, 34 },
    { 0x1ad9cced, 55 },
    { 0xf359ccee, 95 },
    { 0x6f69ccef, 33 },
    { 0, 0 },
    { 0x7479ccf1, 98 },
    { 0x2d89ccf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec99ccf5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9cd05, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7619cd08, 99 },
    { 0xab39cd09, 83 },
    { 0x44f9cd09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b99cd0f, 90 },
    { 0x5629cd10, 98 },
    { 0x7b59cd11, 76 },
    { 0x2fb9cd12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb879cd19, 100 },
    { 0xba09cd1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a9cd1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba9cd21, 98 },
    { 0x21f9cd22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4279cd26, 83 },
    { 0, 0 },
    { 0xc9e9cd28, 90 },
    { 0xe199cd28, 61 },
    { 0x86f9cd2a, 84 },
    { 0xe3e9cd2b, 23 },
    { 0x1759cd2b, 87 },
    { 0x3669cd2d, 23 },
    { 0xf779cd2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b9cd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f79cd36, 69 },
    { 0x3229cd36, 95 },
    { 0x5879cd38, 34 },
    { 0x3579cd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59cd3c, 43 },
    { 0x2b39cd3d, 23 },
    { 0x2d09cd3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9cd43, 33 },
    { 0x51e9cd43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9cd47, 95 },
    { 0, 0 },
    { 0x4419cd49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859cd4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xece9cd50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d9cd54, 23 },
    { 0, 0 },
    { 0x15c9cd56, 40 },
    { 0x9109cd56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac9cd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd9cd5d, 33 },
    { 0x7189cd5d, 76 },
    { 0x5fe9cd5f, 90 },
    { 0x3ac9cd60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3329cd64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9989cd67, 90 },
    { 0x11f9cd68, 61 },
    { 0x6f09cd68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3169cd6c, 93 },
    { 0x2839cd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d9cd78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab09cd7d, 37 },
    { 0x2539cd7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b9cd82, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4899cd85, 98 },
    { 0x6c89cd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b9cd8a, 91 },
    { 0, 0 },
    { 0x26a9cd8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439cd93, 46 },
    { 0, 0 },
    { 0xa639cd95, 95 },
    { 0x17c9cd96, 87 },
    { 0x34b9cd97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b9cd9b, 93 },
    { 0x1f49cd9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9cd9f, 81 },
    { 0, 0 },
    { 0x7cd9cda1, 34 },
    { 0, 0 },
    { 0x5049cda3, 98 },
    { 0xba29cda4, 34 },
    { 0xe0f9cda5, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9cda9, 90 },
    { 0xa599cda9, 90 },
    { 0x1499cdab, 33 },
    { 0x3a79cdac, 90 },
    { 0x4889cdad, 23 },
    { 0xc479cdab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a9cdb4, 67 },
    { 0x7339cdb4, 66 },
    { 0, 0 },
    { 0x6d99cdb7, 90 },
    { 0xdec9cdb8, 98 },
    { 0xf009cdb9, 33 },
    { 0, 0 },
    { 0xeae9cdbb, 95 },
    { 0, 0 },
    { 0xb7b9cdbd, 33 },
    { 0x509cdbd, 98 },
    { 0, 0 },
    { 0x3889cdc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe799cdc4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b9cdc7, 95 },
    { 0x7799cdc8, 95 },
    { 0xdb59cdc9, 85 },
    { 0x5949cdca, 67 },
    { 0x67b9cdc7, 95 },
    { 0xc8a9cdc9, 85 },
    { 0x3e09cdcd, 90 },
    { 0x3ef9cdce, 23 },
    { 0x4849cdce, 77 },
    { 0x9139cdd0, 95 },
    { 0, 0 },
    { 0x9fc9cdd2, 95 },
    { 0xe6b9cdd3, 98 },
    { 0xe009cdd4, 46 },
    { 0xd849cdd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea99cdd8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8239cde4, 99 },
    { 0, 0 },
    { 0x89f9cde6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c29cdeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9cdf0, 90 },
    { 0x83d9cdf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b69cdf4, 87 },
    { 0xf5b9cdf5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d79cdfa, 22 },
    { 0x99e9cdfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe969ce00, 95 },
    { 0x1d89ce00, 43 },
    { 0, 0 },
    { 0x1fd9ce03, 95 },
    { 0x9329ce04, 90 },
    { 0, 0 },
    { 0xf6d9ce06, 84 },
    { 0xfee9ce07, 84 },
    { 0x2bc9ce07, 76 },
    { 0x509ce09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9ce10, 33 },
    { 0x81c9ce10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299ce16, 76 },
    { 0xc919ce17, 95 },
    { 0x47e9ce17, 36 },
    { 0x8e39ce16, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8159ce23, 33 },
    { 0x9c29ce24, 33 },
    { 0, 0 },
    { 0xb409ce26, 33 },
    { 0x2839ce27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a69ce2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d39ce2e, 98 },
    { 0x1c39ce2f, 61 },
    { 0xba89ce2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce9ce35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e49ce38, 98 },
    { 0x4429ce39, 98 },
    { 0, 0 },
    { 0x98a9ce3b, 98 },
    { 0x9c59ce3b, 61 },
    { 0x1089ce3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189ce40, 76 },
    { 0x68a9ce41, 55 },
    { 0x33a9ce41, 36 },
    { 0x5689ce43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc79ce46, 67 },
    { 0, 0 },
    { 0xd159ce48, 76 },
    { 0xdfa9ce48, 93 },
    { 0xd809ce4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d29ce4e, 33 },
    { 0xb59ce4e, 76 },
    { 0x80b9ce4e, 68 },
    { 0xace9ce4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed49ce54, 78 },
    { 0x1629ce55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a9ce59, 95 },
    { 0x67a9ce5a, 23 },
    { 0xc459ce5a, 93 },
    { 0xb209ce5c, 80 },
    { 0xbeb9ce5d, 76 },
    { 0xcfa9ce5e, 61 },
    { 0x2779ce5f, 23 },
    { 0x74d9ce60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b89ce64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd9ce67, 61 },
    { 0x6649ce68, 78 },
    { 0x2859ce69, 23 },
    { 0x7b39ce69, 33 },
    { 0x7859ce68, 55 },
    { 0xdc29ce6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1189ce6f, 95 },
    { 0, 0 },
    { 0xdc9ce71, 98 },
    { 0, 0 },
    { 0xa89ce73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899ce76, 55 },
    { 0xfa79ce76, 61 },
    { 0x1a29ce77, 90 },
    { 0xcae9ce79, 75 },
    { 0, 0 },
    { 0x5d19ce7b, 98 },
    { 0x3a99ce7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f19ce86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9ce89, 98 },
    { 0xeb9ce8a, 87 },
    { 0x239ce8b, 23 },
    { 0x27a9ce8c, 95 },
    { 0x8019ce8d, 98 },
    { 0, 0 },
    { 0x7679ce8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859ce9d, 98 },
    { 0x41b9ce9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99cea2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d9cea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee59ceaa, 33 },
    { 0x48c9ceab, 34 },
    { 0xa8e9ceac, 77 },
    { 0x6419ceaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb9ceb0, 85 },
    { 0x9649ceb1, 33 },
    { 0xa3a9ceb0, 33 },
    { 0, 0 },
    { 0xa139ceb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9ceb7, 33 },
    { 0x3359ceb7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089cebc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc49cec0, 65 },
    { 0x3399cec1, 100 },
    { 0xc4a9cec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe9cec5, 98 },
    { 0x7f9cec5, 100 },
    { 0xe429cec7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d9ceca, 90 },
    { 0x9d89ceca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe9ced4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad9ced7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519ceda, 93 },
    { 0, 0 },
    { 0x3809cedc, 68 },
    { 0x9bb9cedc, 76 },
    { 0x94e9cede, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f29cee3, 64 },
    { 0xe3d9cee4, 43 },
    { 0x86d9cee5, 80 },
    { 0xb759cee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5299ceea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a9ceee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9cef2, 95 },
    { 0xfea9cef3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b59cefa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb9cefe, 93 },
    { 0x4009ceff, 100 },
    { 0x3619cf00, 90 },
    { 0x3459cf01, 98 },
    { 0xe8d9cf00, 33 },
    { 0xbf89cf01, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e79cf0b, 19 },
    { 0x52d9cf0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5589cf0f, 20 },
    { 0xf59cf0f, 87 },
    { 0, 0 },
    { 0x7389cf12, 99 },
    { 0xe2b9cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a9cf16, 52 },
    { 0x23f9cf16, 90 },
    { 0x2fc9cf16, 76 },
    { 0x9cc9cf19, 23 },
    { 0, 0 },
    { 0x39a9cf1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409cf1e, 99 },
    { 0x899cf1f, 84 },
    { 0x2bc9cf20, 90 },
    { 0, 0 },
    { 0x96f9cf22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6529cf27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a9cf33, 46 },
    { 0x4019cf33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe9cf3a, 61 },
    { 0xa629cf3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99cf3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b99cf48, 77 },
    { 0x2429cf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd9cf52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5519cf5c, 100 },
    { 0x6709cf5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c9cf62, 52 },
    { 0xaec9cf62, 90 },
    { 0x1259cf64, 95 },
    { 0, 0 },
    { 0x41f9cf66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec9cf6e, 77 },
    { 0xba9cf6f, 43 },
    { 0x8089cf70, 23 },
    { 0xb059cf71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb229cf75, 98 },
    { 0x48d9cf76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xead9cf79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69cf7c, 87 },
    { 0, 0 },
    { 0x3029cf7e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9cf83, 76 },
    { 0x68d9cf83, 100 },
    { 0x4559cf85, 95 },
    { 0, 0 },
    { 0xb309cf87, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe19cf8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9cf93, 95 },
    { 0xc279cf94, 46 },
    { 0x949cf95, 88 },
    { 0x55b9cf94, 69 },
    { 0x3f49cf97, 90 },
    { 0, 0 },
    { 0xe4a9cf99, 90 },
    { 0, 0 },
    { 0xc979cf9b, 95 },
    { 0x19d9cf9c, 55 },
    { 0x36a9cf9c, 33 },
    { 0, 0 },
    { 0x7119cf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb9cfa3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef9cfa6, 61 },
    { 0x779cfa7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc869cfaa, 53 },
    { 0xda99cfaa, 61 },
    { 0, 0 },
    { 0x2bb9cfad, 87 },
    { 0, 0 },
    { 0x5c9cfaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c9cfb2, 90 },
    { 0, 0 },
    { 0xece9cfb4, 61 },
    { 0, 0 },
    { 0x4499cfb6, 98 },
    { 0xda69cfb7, 61 },
    { 0x2399cfb8, 91 },
    { 0x8499cfb9, 95 },
    { 0, 0 },
    { 0x4069cfbb, 98 },
    { 0, 0 },
    { 0x9a99cfbd, 98 },
    { 0, 0 },
    { 0x8639cfbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f9cfc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9cfc9, 81 },
    { 0, 0 },
    { 0x5349cfcb, 23 },
    { 0xa2c9cfcc, 95 },
    { 0xdba9cfcd, 52 },
    { 0xc89cfce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b79cfd4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2119cfde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089cfe6, 33 },
    { 0x9319cfe7, 98 },
    { 0, 0 },
    { 0xc9c9cfe9, 90 },
    { 0xb1e9cfea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa9cfed, 76 },
    { 0x9c69cfee, 66 },
    { 0xc4a9cfee, 55 },
    { 0xdb99cff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe349cff3, 100 },
    { 0, 0 },
    { 0xaa69cff5, 90 },
    { 0xc5a9cff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9cffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d000, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf209d005, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259d009, 98 },
    { 0xab9d00a, 80 },
    { 0, 0 },
    { 0x8499d00c, 90 },
    { 0x419d00c, 90 },
    { 0x7a69d00d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9d011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad29d019, 98 },
    { 0xb9b9d01a, 90 },
    { 0x45f9d01b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c39d024, 93 },
    { 0xdcb9d025, 75 },
    { 0x8ce9d026, 77 },
    { 0x93c9d027, 95 },
    { 0x9c89d025, 90 },
    { 0, 0 },
    { 0x1249d02a, 90 },
    { 0, 0 },
    { 0xc59d02c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f9d030, 66 },
    { 0xbdc9d030, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2749d03b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109d043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149d047, 95 },
    { 0, 0 },
    { 0xaa59d049, 34 },
    { 0x53d9d049, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d9d04d, 78 },
    { 0xda49d04d, 55 },
    { 0, 0 },
    { 0x49e9d050, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd729d058, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d05b, 91 },
    { 0xce79d05b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769d060, 85 },
    { 0xa0c9d061, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59d065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff9d06a, 23 },
    { 0x1869d06b, 33 },
    { 0, 0 },
    { 0x5c99d06d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc679d071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea9d077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d07c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99d081, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9d085, 98 },
    { 0x1ef9d086, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c9d089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9d08c, 38 },
    { 0x6699d08d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a9d090, 36 },
    { 0, 0 },
    { 0x659d092, 33 },
    { 0x2ef9d093, 55 },
    { 0x18d9d093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a9d09c, 88 },
    { 0xebd9d09d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b9d0a1, 78 },
    { 0x52d9d0a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd649d0a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc439d0a9, 90 },
    { 0, 0 },
    { 0x3b09d0ab, 46 },
    { 0x91d9d0ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e79d0af, 23 },
    { 0xe819d0b0, 33 },
    { 0, 0 },
    { 0x8879d0b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc9d0b6, 61 },
    { 0xff49d0b7, 98 },
    { 0xca09d0b7, 91 },
    { 0x88a9d0b8, 95 },
    { 0, 0 },
    { 0x56b9d0bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89d0bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d0c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9679d0c7, 33 },
    { 0x1fd9d0c8, 61 },
    { 0x4f89d0c9, 55 },
    { 0, 0 },
    { 0x8089d0cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb99d0d1, 95 },
    { 0xef89d0d1, 46 },
    { 0, 0 },
    { 0x59b9d0d4, 23 },
    { 0, 0 },
    { 0x14c9d0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b9d0db, 34 },
    { 0, 0 },
    { 0x38c9d0dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc509d0e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6389d0e3, 90 },
    { 0xf7b9d0e3, 23 },
    { 0xa259d0e3, 95 },
    { 0x359d0e5, 99 },
    { 0xda39d0e7, 90 },
    { 0x949d0e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99d0f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa089d0f5, 33 },
    { 0xc649d0f6, 93 },
    { 0, 0 },
    { 0x63c9d0f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999d100, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af9d103, 69 },
    { 0x3c39d103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789d108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4329d10d, 90 },
    { 0, 0 },
    { 0xd0e9d10f, 46 },
    { 0, 0 },
    { 0xb0d9d111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d29d115, 90 },
    { 0x6139d116, 46 },
    { 0x9dc9d115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6989d11f, 76 },
    { 0, 0 },
    { 0xdf49d121, 61 },
    { 0x9b99d122, 97 },
    { 0x50c9d123, 61 },
    { 0, 0 },
    { 0x51c9d125, 90 },
    { 0x46f9d125, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9229d12c, 88 },
    { 0, 0 },
    { 0xfd9d12e, 33 },
    { 0x1679d12f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9d132, 78 },
    { 0x6f19d132, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79d136, 33 },
    { 0, 0 },
    { 0xe549d138, 34 },
    { 0xa079d139, 90 },
    { 0xda09d13a, 90 },
    { 0xbd59d13a, 68 },
    { 0xb299d13b, 90 },
    { 0x6929d13d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1419d140, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f9d144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9d147, 97 },
    { 0xc879d147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c69d150, 76 },
    { 0xb109d151, 100 },
    { 0, 0 },
    { 0x2299d153, 90 },
    { 0x47b9d154, 40 },
    { 0, 0 },
    { 0x7389d156, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609d15b, 76 },
    { 0x3fa9d15c, 43 },
    { 0x23f9d15c, 61 },
    { 0x5d89d15e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd089d16b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c59d16f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b9d174, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa629d183, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49d187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd9d18c, 87 },
    { 0xed19d18d, 23 },
    { 0x80a9d18e, 65 },
    { 0, 0 },
    { 0xac59d190, 100 },
    { 0, 0 },
    { 0xc7f9d192, 77 },
    { 0xa669d192, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e9d196, 33 },
    { 0xe369d197, 76 },
    { 0x25f9d197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab09d1a6, 99 },
    { 0x7539d1a6, 76 },
    { 0xc619d1a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49d1ae, 23 },
    { 0, 0 },
    { 0x3d39d1b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa9d1b4, 85 },
    { 0xf3a9d1b5, 65 },
    { 0xa329d1b6, 90 },
    { 0, 0 },
    { 0x5899d1b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede9d1bf, 23 },
    { 0x48a9d1c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d9d1c5, 51 },
    { 0x1419d1c6, 76 },
    { 0xdb99d1c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319d1ca, 95 },
    { 0x7039d1cb, 78 },
    { 0, 0 },
    { 0x9199d1cd, 55 },
    { 0x1119d1ce, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa849d1d2, 76 },
    { 0, 0 },
    { 0x2679d1d4, 98 },
    { 0, 0 },
    { 0x25a9d1d6, 76 },
    { 0, 0 },
    { 0x27f9d1d8, 46 },
    { 0, 0 },
    { 0xa339d1da, 100 },
    { 0x30c9d1da, 84 },
    { 0xebf9d1dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6189d1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f9d1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e9d1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f99d1ed, 69 },
    { 0x2c89d1ee, 48 },
    { 0x10c9d1ef, 95 },
    { 0x8d19d1f0, 33 },
    { 0x1d29d1f1, 23 },
    { 0xa29d1f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199d1f6, 55 },
    { 0x1ef9d1f7, 95 },
    { 0, 0 },
    { 0xad59d1f9, 23 },
    { 0x51e9d1fa, 90 },
    { 0xb2e9d1fb, 90 },
    { 0x6d09d1f9, 95 },
    { 0xd279d1fc, 98 },
    { 0x7b69d1fe, 90 },
    { 0x9b09d1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4659d203, 68 },
    { 0xfb49d204, 34 },
    { 0x7c59d204, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d99d208, 98 },
    { 0x3859d209, 88 },
    { 0xc059d208, 92 },
    { 0xc099d20b, 90 },
    { 0xe699d20c, 55 },
    { 0xdf9d20d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7039d213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff9d216, 90 },
    { 0xbbf9d217, 90 },
    { 0x7599d218, 77 },
    { 0x7219d219, 91 },
    { 0, 0 },
    { 0x4229d21b, 66 },
    { 0x2d09d21b, 95 },
    { 0xa939d21d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d49d221, 55 },
    { 0x1759d222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8579d229, 99 },
    { 0xf009d229, 90 },
    { 0x429d22a, 95 },
    { 0x279d22c, 98 },
    { 0, 0 },
    { 0x4919d22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd699d231, 93 },
    { 0x6339d231, 90 },
    { 0x1cc9d232, 90 },
    { 0xa519d234, 65 },
    { 0x33c9d235, 87 },
    { 0xadc9d236, 76 },
    { 0xf1d9d237, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39d242, 23 },
    { 0x9b29d242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449d248, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799d24b, 33 },
    { 0xc1b9d24c, 76 },
    { 0x5c89d24d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f9d250, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea9d255, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29d259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9d265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269d268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b9d275, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9d27a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9d27d, 98 },
    { 0x9679d27d, 46 },
    { 0x8819d27f, 92 },
    { 0, 0 },
    { 0x8d9d281, 75 },
    { 0xf8f9d282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x589d285, 23 },
    { 0x1a9d286, 90 },
    { 0x6d99d286, 61 },
    { 0xd6f9d288, 33 },
    { 0xfa69d288, 95 },
    { 0xe989d289, 34 },
    { 0xbbb9d288, 33 },
    { 0xea99d285, 23 },
    { 0xdc39d28d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d293, 90 },
    { 0xd4e9d294, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a9d299, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe69d29e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2559d2a5, 90 },
    { 0x3289d2a6, 33 },
    { 0x7da9d2a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2399d2ae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2909d2b3, 55 },
    { 0x30c9d2b4, 61 },
    { 0, 0 },
    { 0x2199d2b6, 78 },
    { 0xe2c9d2b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe89d2bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9d2be, 34 },
    { 0, 0 },
    { 0x4da9d2c0, 33 },
    { 0x9f59d2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f9d2ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419d2cd, 33 },
    { 0xcdf9d2ce, 90 },
    { 0xdb9d2cf, 78 },
    { 0xb349d2cd, 23 },
    { 0xe959d2d1, 100 },
    { 0xac59d2d1, 34 },
    { 0, 0 },
    { 0xb99d2d4, 23 },
    { 0x1d69d2d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299d2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd429d2df, 33 },
    { 0x8659d2df, 83 },
    { 0, 0 },
    { 0xeab9d2e2, 35 },
    { 0x179d2e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49d2e7, 23 },
    { 0xcdb9d2e8, 95 },
    { 0xfbb9d2e8, 80 },
    { 0x63f9d2ea, 78 },
    { 0x1599d2eb, 90 },
    { 0x9469d2ea, 64 },
    { 0x9fc9d2ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe339d2f0, 23 },
    { 0x5199d2f0, 76 },
    { 0, 0 },
    { 0x85a9d2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6329d2f7, 33 },
    { 0x2d49d2f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3289d2fd, 55 },
    { 0x39d9d2fe, 90 },
    { 0x7d59d2ff, 81 },
    { 0xb949d300, 65 },
    { 0x70a9d301, 61 },
    { 0x62b9d300, 98 },
    { 0x7919d2ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d9d306, 95 },
    { 0, 0 },
    { 0x4f49d308, 76 },
    { 0, 0 },
    { 0xc489d30a, 98 },
    { 0, 0 },
    { 0xb529d30c, 66 },
    { 0xe059d30c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd9d312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69d315, 90 },
    { 0x90d9d315, 43 },
    { 0x7679d317, 90 },
    { 0xf459d318, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79d31c, 61 },
    { 0xeea9d31c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6569d327, 61 },
    { 0x3ab9d327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc29d32b, 69 },
    { 0, 0 },
    { 0x8599d32d, 85 },
    { 0xc9d9d32e, 76 },
    { 0xc579d32e, 34 },
    { 0x3419d330, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a9d333, 68 },
    { 0, 0 },
    { 0xbfa9d335, 93 },
    { 0x1cb9d335, 90 },
    { 0xd6a9d337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9d33d, 55 },
    { 0x4ba9d33e, 55 },
    { 0x7329d33f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2609d346, 95 },
    { 0, 0 },
    { 0xcb19d348, 55 },
    { 0x3f89d348, 33 },
    { 0xf9d9d348, 33 },
    { 0x90a9d349, 33 },
    { 0, 0 },
    { 0xe5a9d34d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4739d352, 76 },
    { 0, 0 },
    { 0xd8d9d354, 66 },
    { 0xda59d355, 90 },
    { 0x35d9d356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec9d35b, 43 },
    { 0x9d35b, 90 },
    { 0x10b9d35b, 46 },
    { 0, 0 },
    { 0x7c59d35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9d363, 23 },
    { 0, 0 },
    { 0x26b9d365, 33 },
    { 0xbe29d366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809d36c, 95 },
    { 0, 0 },
    { 0x2229d36e, 52 },
    { 0x339d36f, 90 },
    { 0x2719d36e, 61 },
    { 0x4be9d371, 23 },
    { 0x1749d36e, 61 },
    { 0x4a89d373, 55 },
    { 0x1409d374, 33 },
    { 0x4b69d374, 46 },
    { 0x68d9d374, 61 },
    { 0x26e9d377, 17 },
    { 0xa4b9d378, 98 },
    { 0x9149d376, 90 },
    { 0, 0 },
    { 0x1649d37b, 95 },
    { 0xe7a9d37b, 95 },
    { 0x2d59d37d, 17 },
    { 0x7889d37d, 85 },
    { 0x4ef9d37f, 55 },
    { 0, 0 },
    { 0xb079d381, 55 },
    { 0xf699d382, 69 },
    { 0xc319d382, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea19d386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb9d389, 98 },
    { 0xf199d38a, 33 },
    { 0, 0 },
    { 0x1e19d38c, 33 },
    { 0xcbd9d38d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19d392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb129d39f, 65 },
    { 0x7f79d39f, 77 },
    { 0x97c9d3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc419d3a9, 90 },
    { 0xc519d3aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c9d3b2, 77 },
    { 0x6b49d3b3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e39d3b8, 20 },
    { 0xe749d3b8, 98 },
    { 0, 0 },
    { 0x3c49d3bb, 76 },
    { 0x6999d3bb, 55 },
    { 0x3c49d3bc, 98 },
    { 0xdc69d3be, 33 },
    { 0xb6b9d3bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a19d3c7, 90 },
    { 0x57a9d3c8, 98 },
    { 0x1a49d3c8, 61 },
    { 0x50f9d3ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc29d3ce, 84 },
    { 0x4509d3cf, 33 },
    { 0x4729d3cf, 40 },
    { 0xa9f9d3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3309d3d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6939d3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff9d3e1, 98 },
    { 0x9639d3e2, 76 },
    { 0xf659d3e1, 99 },
    { 0xbb19d3e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7579d3e7, 98 },
    { 0x5a9d3e7, 95 },
    { 0, 0 },
    { 0xc409d3ea, 46 },
    { 0, 0 },
    { 0x7919d3ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd959d3f0, 95 },
    { 0xe4d9d3f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19d3f4, 90 },
    { 0xb89d3f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99d3fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c9d3ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c9d405, 47 },
    { 0xe879d406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb19d40f, 61 },
    { 0x1fd9d410, 93 },
    { 0x2339d410, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819d418, 98 },
    { 0x7c79d418, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299d41c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3889d41f, 46 },
    { 0x1469d41f, 55 },
    { 0x4a09d421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a9d424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89d427, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe99d42b, 33 },
    { 0x1a49d42b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9d430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9d435, 33 },
    { 0x1e59d436, 65 },
    { 0x50f9d435, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2229d43d, 98 },
    { 0xbd49d43e, 90 },
    { 0xd59d43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9d446, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59d44a, 81 },
    { 0, 0 },
    { 0x8829d44c, 23 },
    { 0x9909d44d, 77 },
    { 0x77a9d44c, 43 },
    { 0x7019d44f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb369d457, 76 },
    { 0, 0 },
    { 0xefe9d459, 90 },
    { 0xf6d9d45a, 78 },
    { 0x81c9d45b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad59d460, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f49d472, 55 },
    { 0x6739d473, 61 },
    { 0, 0 },
    { 0x9949d475, 76 },
    { 0, 0 },
    { 0xa219d477, 22 },
    { 0x3ea9d477, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e49d47c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9d484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799d487, 55 },
    { 0, 0 },
    { 0xaf89d489, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b09d48d, 23 },
    { 0xce39d48e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679d492, 95 },
    { 0, 0 },
    { 0x1db9d494, 98 },
    { 0x3159d495, 46 },
    { 0x5e79d494, 75 },
    { 0x72e9d497, 90 },
    { 0x35e9d498, 43 },
    { 0, 0 },
    { 0x6d59d49a, 98 },
    { 0, 0 },
    { 0x2319d49c, 95 },
    { 0, 0 },
    { 0xe369d49e, 98 },
    { 0, 0 },
    { 0xecd9d4a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e99d4a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc89d4a8, 33 },
    { 0x1ca9d4a9, 23 },
    { 0xcf59d4a9, 95 },
    { 0, 0 },
    { 0x7919d4ac, 61 },
    { 0xe509d4ad, 90 },
    { 0x5139d4ae, 68 },
    { 0xe009d4ac, 60 },
    { 0xba89d4ac, 61 },
    { 0x47d9d4af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d59d4b7, 81 },
    { 0xde19d4b7, 66 },
    { 0, 0 },
    { 0x98c9d4ba, 90 },
    { 0xca89d4ba, 98 },
    { 0x159d4bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf969d4bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129d4c2, 98 },
    { 0x60e9d4c3, 76 },
    { 0x6c69d4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349d4c8, 33 },
    { 0, 0 },
    { 0xb9f9d4ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999d4ce, 85 },
    { 0xbd39d4cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639d4d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb9d4d5, 23 },
    { 0x2b49d4d6, 33 },
    { 0xd429d4d6, 55 },
    { 0x5e69d4d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8749d4de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e99d4e4, 23 },
    { 0x7a49d4e5, 84 },
    { 0xb319d4e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39d4e9, 90 },
    { 0, 0 },
    { 0x9a9d4eb, 98 },
    { 0x3e19d4ec, 90 },
    { 0x1a79d4ec, 33 },
    { 0x9329d4ee, 46 },
    { 0xe579d4ef, 100 },
    { 0xeee9d4ec, 23 },
    { 0x5ba9d4f1, 78 },
    { 0xa279d4f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb9d4f5, 90 },
    { 0, 0 },
    { 0xc059d4f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc99d4fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3059d501, 33 },
    { 0xbd9d502, 61 },
    { 0, 0 },
    { 0xd739d504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda9d507, 33 },
    { 0x8839d508, 91 },
    { 0x39d9d508, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d50e, 23 },
    { 0, 0 },
    { 0x57c9d510, 55 },
    { 0x2739d511, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f79d519, 33 },
    { 0, 0 },
    { 0xa789d51b, 90 },
    { 0x6409d51b, 98 },
    { 0xf009d51d, 90 },
    { 0x81d9d51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd089d523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9d529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf519d52d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d9d534, 66 },
    { 0x4f79d535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe09d53b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d541, 91 },
    { 0, 0 },
    { 0xce59d543, 84 },
    { 0, 0 },
    { 0x3319d545, 55 },
    { 0xd349d545, 33 },
    { 0x7d09d547, 76 },
    { 0xb3a9d548, 95 },
    { 0x8609d547, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9d54e, 76 },
    { 0x19b9d54f, 90 },
    { 0xbf59d54f, 98 },
    { 0x6549d551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec59d557, 90 },
    { 0x1bb9d557, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc9d55c, 77 },
    { 0, 0 },
    { 0x33b9d55e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9d561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9d564, 61 },
    { 0, 0 },
    { 0x5949d566, 76 },
    { 0x5489d567, 90 },
    { 0x2079d568, 90 },
    { 0xbac9d569, 23 },
    { 0xe979d56a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c49d575, 56 },
    { 0x3c89d576, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429d579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb9d57e, 33 },
    { 0, 0 },
    { 0x78f9d580, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9d586, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a9d589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79d58d, 33 },
    { 0x1b49d58e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e49d593, 52 },
    { 0x51c9d594, 75 },
    { 0xbc29d595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9d599, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5239d59e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a9d5a1, 61 },
    { 0, 0 },
    { 0x8b89d5a3, 76 },
    { 0x7779d5a4, 33 },
    { 0x31e9d5a5, 76 },
    { 0xc839d5a5, 95 },
    { 0x2759d5a7, 77 },
    { 0x9ff9d5a7, 98 },
    { 0x199d5a8, 61 },
    { 0x9559d5aa, 90 },
    { 0xe2c9d5aa, 95 },
    { 0x4c69d5ac, 93 },
    { 0xc179d5a6, 66 },
    { 0x47e9d5a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9d5b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x939d5b6, 100 },
    { 0xec89d5b6, 55 },
    { 0xce89d5b8, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x3849d5bb, 98 },
    { 0x5a29d5bc, 61 },
    { 0, 0 },
    { 0x6d29d5be, 55 },
    { 0, 0 },
    { 0xdd39d5c0, 34 },
    { 0, 0 },
    { 0x6349d5c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e9d5d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9d5d6, 43 },
    { 0x3ff9d5d7, 84 },
    { 0x5559d5d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9d5db, 33 },
    { 0x2f9d5db, 91 },
    { 0x6eb9d5db, 40 },
    { 0xbae9d5dd, 44 },
    { 0, 0 },
    { 0x4059d5e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d9d5e4, 66 },
    { 0, 0 },
    { 0x69d9d5e6, 95 },
    { 0xfd99d5e7, 99 },
    { 0x2679d5e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9d5eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d5f1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9d5f4, 90 },
    { 0x43c9d5f5, 98 },
    { 0, 0 },
    { 0x31f9d5f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe39d5fb, 96 },
    { 0xa39d5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9d600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759d603, 61 },
    { 0x2639d604, 88 },
    { 0xe6f9d604, 61 },
    { 0xd4a9d603, 90 },
    { 0, 0 },
    { 0x5719d608, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d09d60c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa59d610, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9d614, 23 },
    { 0xcda9d615, 34 },
    { 0xb039d616, 61 },
    { 0x6489d617, 33 },
    { 0x53d9d618, 64 },
    { 0x99d9d619, 61 },
    { 0xaa79d61a, 90 },
    { 0x46e9d61b, 98 },
    { 0x34b9d616, 76 },
    { 0, 0 },
    { 0x4ab9d61e, 76 },
    { 0xc179d61f, 33 },
    { 0, 0 },
    { 0xed29d621, 88 },
    { 0x8889d622, 99 },
    { 0xd239d623, 78 },
    { 0x7ea9d624, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd619d628, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd449d62c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269d62f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e99d635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda69d63c, 84 },
    { 0xb3b9d63c, 95 },
    { 0xa69d63e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079d641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce9d649, 98 },
    { 0x43e9d64a, 90 },
    { 0xd719d64a, 77 },
    { 0x2c49d64c, 61 },
    { 0x60c9d64c, 90 },
    { 0xc309d64e, 33 },
    { 0x50e9d649, 43 },
    { 0xa129d64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe399d653, 46 },
    { 0, 0 },
    { 0xe729d655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b9d658, 33 },
    { 0xce19d659, 95 },
    { 0xcda9d65a, 33 },
    { 0xe799d659, 67 },
    { 0xee59d65c, 95 },
    { 0x2c29d65d, 90 },
    { 0, 0 },
    { 0xcbf9d65f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d89d666, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4029d669, 100 },
    { 0xa799d66a, 33 },
    { 0xba39d66b, 33 },
    { 0xeca9d66c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f9d677, 61 },
    { 0, 0 },
    { 0xbd79d679, 23 },
    { 0xa8d9d67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069d67d, 55 },
    { 0, 0 },
    { 0x9c09d67f, 87 },
    { 0x1d79d680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c9d685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b9d68a, 99 },
    { 0, 0 },
    { 0xd289d68c, 90 },
    { 0xf649d68d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c89d693, 77 },
    { 0, 0 },
    { 0xc769d695, 34 },
    { 0, 0 },
    { 0x4139d697, 23 },
    { 0x9e99d697, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9d69e, 43 },
    { 0, 0 },
    { 0xec9d6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9129d6a4, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd339d6a8, 90 },
    { 0x4459d6a9, 88 },
    { 0xe219d6aa, 76 },
    { 0x36d9d6aa, 34 },
    { 0x5939d6aa, 90 },
    { 0x8229d6ad, 100 },
    { 0x24f9d6ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe49d6b7, 33 },
    { 0x5b49d6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab99d6bc, 61 },
    { 0xc6a9d6bc, 95 },
    { 0x1a49d6be, 43 },
    { 0x9189d6bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5509d6c2, 23 },
    { 0, 0 },
    { 0x9cc9d6c4, 61 },
    { 0x6f59d6c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d59d6c9, 84 },
    { 0x6269d6ca, 55 },
    { 0, 0 },
    { 0x1e29d6cc, 98 },
    { 0x3379d6cd, 91 },
    { 0x1eb9d6cc, 95 },
    { 0x3fc9d6ce, 95 },
    { 0xfe49d6ce, 76 },
    { 0x4739d6d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e9d6d5, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a9d6d8, 23 },
    { 0xafd9d6d9, 33 },
    { 0xb209d6d8, 75 },
    { 0xecf9d6db, 98 },
    { 0, 0 },
    { 0xa849d6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c9d6e7, 34 },
    { 0x3d19d6e7, 90 },
    { 0, 0 },
    { 0x67a9d6ea, 61 },
    { 0x8009d6eb, 34 },
    { 0x2b19d6ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa619d6f3, 61 },
    { 0x52b9d6f3, 98 },
    { 0, 0 },
    { 0xc549d6f6, 55 },
    { 0, 0 },
    { 0x44b9d6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2009d701, 81 },
    { 0x9149d701, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c9d70c, 48 },
    { 0xabc9d70d, 76 },
    { 0, 0 },
    { 0x9c9d70f, 78 },
    { 0, 0 },
    { 0x2d19d711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c9d719, 61 },
    { 0xd8b9d71a, 52 },
    { 0x3389d71b, 93 },
    { 0x72e9d71c, 76 },
    { 0x31f9d71d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a9d722, 55 },
    { 0x60f9d723, 98 },
    { 0xdc29d724, 34 },
    { 0x3c19d725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca79d728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec9d72b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa79d72e, 98 },
    { 0, 0 },
    { 0xfb89d730, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019d734, 99 },
    { 0, 0 },
    { 0x2849d736, 78 },
    { 0, 0 },
    { 0x7009d738, 23 },
    { 0xedb9d739, 85 },
    { 0xaf89d738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f99d73d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d49d741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab9d746, 61 },
    { 0x57e9d747, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa789d74d, 43 },
    { 0xf9a9d74d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e9d752, 90 },
    { 0x6119d753, 34 },
    { 0xea19d753, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa189d758, 99 },
    { 0x34d9d759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5169d75c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629d763, 88 },
    { 0x1ea9d764, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419d768, 55 },
    { 0xb639d768, 33 },
    { 0xe329d76a, 61 },
    { 0xdc99d76a, 87 },
    { 0x44d9d76c, 55 },
    { 0x9809d76d, 43 },
    { 0xdb49d76e, 61 },
    { 0xa109d76f, 76 },
    { 0xb029d76a, 43 },
    { 0xeba9d768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9d777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9d77c, 76 },
    { 0, 0 },
    { 0xcb69d77e, 76 },
    { 0, 0 },
    { 0xf0a9d780, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d9d783, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae9d787, 43 },
    { 0xce59d788, 33 },
    { 0xb689d788, 23 },
    { 0xed99d788, 23 },
    { 0x5419d78b, 95 },
    { 0, 0 },
    { 0xbee9d78d, 23 },
    { 0, 0 },
    { 0xac59d78f, 76 },
    { 0xe6b9d78f, 61 },
    { 0xdb29d791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d796, 100 },
    { 0, 0 },
    { 0xef49d798, 85 },
    { 0, 0 },
    { 0x9559d79a, 33 },
    { 0x9409d79a, 93 },
    { 0, 0 },
    { 0x2559d79d, 43 },
    { 0x2c99d79e, 61 },
    { 0x7379d79f, 93 },
    { 0xd6f9d79f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069d7a3, 95 },
    { 0x1b59d7a4, 33 },
    { 0, 0 },
    { 0x1859d7a6, 91 },
    { 0xb329d7a7, 76 },
    { 0x5539d7a8, 23 },
    { 0, 0 },
    { 0x3279d7aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c9d7b2, 91 },
    { 0x2529d7b2, 91 },
    { 0, 0 },
    { 0x28b9d7b5, 33 },
    { 0, 0 },
    { 0x7169d7b7, 90 },
    { 0, 0 },
    { 0x6819d7b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb979d7be, 95 },
    { 0xed29d7bf, 33 },
    { 0, 0 },
    { 0xd909d7c1, 90 },
    { 0x4e19d7c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa379d7c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf39d7c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e9d7cd, 61 },
    { 0x72f9d7ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4709d7d1, 95 },
    { 0xd799d7d2, 23 },
    { 0x5d59d7d2, 90 },
    { 0xe789d7d1, 93 },
    { 0xd959d7d5, 55 },
    { 0x7a59d7d5, 77 },
    { 0, 0 },
    { 0xc079d7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e09d7db, 53 },
    { 0x2299d7dc, 24 },
    { 0, 0 },
    { 0x8649d7de, 87 },
    { 0xbfc9d7df, 95 },
    { 0x9b9d7e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4999d7e5, 76 },
    { 0, 0 },
    { 0xb6a9d7e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659d7ed, 65 },
    { 0x6b89d7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59d7f4, 23 },
    { 0xd29d7f5, 23 },
    { 0x7d09d7f6, 90 },
    { 0xc779d7f7, 55 },
    { 0x8c59d7f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd599d7fe, 65 },
    { 0, 0 },
    { 0xa6d9d800, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf449d807, 52 },
    { 0x5749d808, 33 },
    { 0x41c9d808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb9d80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69d814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e9d817, 90 },
    { 0, 0 },
    { 0xe349d819, 33 },
    { 0x4e39d81a, 46 },
    { 0, 0 },
    { 0xce9d81c, 77 },
    { 0x5419d81c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd9d820, 66 },
    { 0x55e9d820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e89d827, 85 },
    { 0x20d9d828, 90 },
    { 0xfcc9d829, 51 },
    { 0, 0 },
    { 0xe0b9d82b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c29d830, 83 },
    { 0, 0 },
    { 0x8469d832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9d835, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9589d83b, 33 },
    { 0, 0 },
    { 0xbc59d83d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4659d842, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc469d845, 90 },
    { 0x2e59d846, 84 },
    { 0x2559d847, 80 },
    { 0xfb79d848, 78 },
    { 0x2cd9d845, 61 },
    { 0x1939d848, 95 },
    { 0xa339d847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda49d852, 90 },
    { 0xcef9d853, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329d859, 98 },
    { 0x5b09d85a, 85 },
    { 0xb699d85b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8429d85e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d865, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd379d86d, 90 },
    { 0xcc59d86e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b09d871, 98 },
    { 0, 0 },
    { 0x2ce9d873, 33 },
    { 0x7d09d874, 61 },
    { 0xda49d873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a9d878, 90 },
    { 0x54f9d879, 68 },
    { 0x8c79d878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe279d87d, 68 },
    { 0xfef9d87d, 84 },
    { 0, 0 },
    { 0x5a99d880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4439d889, 33 },
    { 0, 0 },
    { 0xe99d88b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa439d88f, 90 },
    { 0x4159d890, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d59d895, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7539d89b, 61 },
    { 0x1a19d89c, 90 },
    { 0x2079d89c, 90 },
    { 0x4859d89b, 95 },
    { 0, 0 },
    { 0xe0d9d8a0, 90 },
    { 0, 0 },
    { 0xa239d8a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d89d8a8, 88 },
    { 0xe959d8a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf379d8b0, 46 },
    { 0x6e49d8b1, 30 },
    { 0x5139d8b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd99d8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc489d8be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9d8c3, 76 },
    { 0, 0 },
    { 0x6239d8c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb49d8ca, 23 },
    { 0xcb9d8ca, 90 },
    { 0, 0 },
    { 0x3049d8cd, 43 },
    { 0x4309d8ce, 78 },
    { 0, 0 },
    { 0x66e9d8d0, 61 },
    { 0xebf9d8d0, 99 },
    { 0x5399d8d2, 33 },
    { 0, 0 },
    { 0xa609d8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d39d8d8, 61 },
    { 0, 0 },
    { 0xd1f9d8da, 90 },
    { 0xcbf9d8db, 98 },
    { 0xf129d8dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9d8e5, 90 },
    { 0xfb99d8e6, 15 },
    { 0x5339d8e7, 65 },
    { 0x9df9d8e7, 55 },
    { 0xcd29d8e9, 68 },
    { 0x3849d8ea, 90 },
    { 0, 0 },
    { 0x4389d8ec, 99 },
    { 0, 0 },
    { 0xc759d8ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc9d8f3, 33 },
    { 0x23d9d8f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a59d8ff, 78 },
    { 0, 0 },
    { 0xe059d901, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae9d905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd9d909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf629d911, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d9d915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd459d918, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc59d91e, 33 },
    { 0xc29d91f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f39d924, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689d928, 98 },
    { 0x7f39d929, 67 },
    { 0, 0 },
    { 0x9d29d92b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9d932, 23 },
    { 0xd2a9d933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d936, 95 },
    { 0, 0 },
    { 0x8e19d938, 75 },
    { 0, 0 },
    { 0xbb49d93a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a9d93e, 98 },
    { 0, 0 },
    { 0x5fd9d940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9d943, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9d946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89d94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3029d952, 90 },
    { 0, 0 },
    { 0xc4b9d954, 90 },
    { 0xab29d955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069d95a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d9d95d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6949d960, 46 },
    { 0xc319d961, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949d965, 33 },
    { 0x2909d966, 61 },
    { 0, 0 },
    { 0xc359d968, 55 },
    { 0xf89d969, 55 },
    { 0x9ef9d96a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f59d973, 66 },
    { 0x91f9d974, 23 },
    { 0x7699d974, 95 },
    { 0, 0 },
    { 0xad79d977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9009d97a, 23 },
    { 0xd689d97b, 98 },
    { 0, 0 },
    { 0x67c9d97d, 34 },
    { 0, 0 },
    { 0x43d9d97f, 77 },
    { 0xc0c9d980, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac49d984, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829d987, 61 },
    { 0, 0 },
    { 0xc19d989, 90 },
    { 0x4769d98a, 66 },
    { 0x52e9d98b, 55 },
    { 0, 0 },
    { 0xbf09d98d, 23 },
    { 0x2169d98e, 84 },
    { 0, 0 },
    { 0xff49d990, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719d993, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a9d99b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd9d9a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de9d9aa, 76 },
    { 0, 0 },
    { 0x1119d9ac, 26 },
    { 0x7179d9ad, 33 },
    { 0xdfa9d9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8399d9b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9d9b5, 76 },
    { 0x19c9d9b6, 76 },
    { 0, 0 },
    { 0x89d9d9b8, 90 },
    { 0x5bc9d9b9, 98 },
    { 0xd669d9ba, 61 },
    { 0x89a9d9ba, 17 },
    { 0x839d9bc, 90 },
    { 0x41b9d9bc, 33 },
    { 0xc3d9d9bc, 90 },
    { 0x5069d9bf, 43 },
    { 0x6849d9bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199d9c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a19d9cd, 59 },
    { 0x1359d9cd, 90 },
    { 0x8de9d9cf, 33 },
    { 0xebc9d9ce, 33 },
    { 0x54a9d9d1, 93 },
    { 0xa599d9d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa399d9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb59d9da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249d9e2, 90 },
    { 0xbbc9d9e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949d9e8, 67 },
    { 0xb839d9e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199d9f0, 93 },
    { 0, 0 },
    { 0x53f9d9f2, 43 },
    { 0, 0 },
    { 0x5be9d9f4, 76 },
    { 0x5e79d9f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d9f9, 98 },
    { 0x8459d9fa, 34 },
    { 0xcf49d9f9, 43 },
    { 0xbd39d9fc, 95 },
    { 0xf7b9d9fd, 84 },
    { 0, 0 },
    { 0x5e49d9ff, 100 },
    { 0, 0 },
    { 0x5509da01, 93 },
    { 0x3759da02, 98 },
    { 0x8219da03, 55 },
    { 0x21d9da03, 77 },
    { 0, 0 },
    { 0xacf9da06, 90 },
    { 0, 0 },
    { 0x2499da08, 46 },
    { 0x7a79da09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189da0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d79da10, 95 },
    { 0xc529da11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9da14, 55 },
    { 0xb999da14, 23 },
    { 0x8e39da15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f9da1a, 34 },
    { 0, 0 },
    { 0x9cb9da1c, 98 },
    { 0x4b69da1d, 61 },
    { 0xd899da1e, 61 },
    { 0xeb29da1c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a79da24, 98 },
    { 0x6ae9da25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9da2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489da37, 95 },
    { 0, 0 },
    { 0x31d9da39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd9da3d, 76 },
    { 0x4679da3e, 98 },
    { 0x2bd9da3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe799da43, 93 },
    { 0, 0 },
    { 0xf459da45, 27 },
    { 0x8559da46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd69da4a, 96 },
    { 0x5639da4b, 23 },
    { 0xba89da4b, 17 },
    { 0x4929da4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c09da55, 95 },
    { 0x6189da56, 23 },
    { 0x1b49da57, 75 },
    { 0xddb9da57, 90 },
    { 0xaf39da56, 90 },
    { 0xd759da55, 33 },
    { 0x8ea9da5b, 61 },
    { 0x6199da5c, 91 },
    { 0, 0 },
    { 0x3869da5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e9da61, 78 },
    { 0, 0 },
    { 0x4f09da63, 76 },
    { 0xacb9da64, 61 },
    { 0x96a9da64, 23 },
    { 0xa319da63, 99 },
    { 0, 0 },
    { 0x79a9da68, 46 },
    { 0xb0e9da69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9da6d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1159da73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc759da78, 95 },
    { 0x7bc9da79, 98 },
    { 0, 0 },
    { 0xa6f9da7b, 55 },
    { 0, 0 },
    { 0xfea9da7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc49da81, 66 },
    { 0x7049da82, 90 },
    { 0xea89da82, 23 },
    { 0, 0 },
    { 0xb349da85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9da88, 99 },
    { 0xa2a9da89, 23 },
    { 0x5329da89, 33 },
    { 0x8999da8b, 23 },
    { 0x2cc9da8c, 84 },
    { 0, 0 },
    { 0xe069da8e, 23 },
    { 0x10d9da8e, 61 },
    { 0x5a89da90, 55 },
    { 0xfdd9da8f, 68 },
    { 0xd7f9da8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939da99, 52 },
    { 0xffc9da9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de9da9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789daa3, 33 },
    { 0x7d89daa3, 77 },
    { 0, 0 },
    { 0x2459daa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd09daac, 76 },
    { 0x1b59daac, 55 },
    { 0x1b89daad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99dab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb969dab5, 23 },
    { 0x2439dab6, 46 },
    { 0xa2e9dab6, 95 },
    { 0x6129dab8, 61 },
    { 0x8679dab8, 61 },
    { 0xd8a9daba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c9dabf, 53 },
    { 0xaed9dac0, 93 },
    { 0xe749dac1, 95 },
    { 0xd8e9dabf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9dac7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9daca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e9dace, 90 },
    { 0, 0 },
    { 0x5819dad0, 55 },
    { 0x38f9dad1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7139dad4, 43 },
    { 0x2239dad4, 43 },
    { 0xb09dad6, 33 },
    { 0, 0 },
    { 0xf2b9dad8, 34 },
    { 0x2d49dad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3149dae0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd59daeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf9daf2, 87 },
    { 0x2b29daf3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf909daf9, 78 },
    { 0, 0 },
    { 0x8f39dafb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9db02, 76 },
    { 0, 0 },
    { 0x8cb9db04, 61 },
    { 0, 0 },
    { 0x66a9db06, 98 },
    { 0x57f9db07, 90 },
    { 0xf9d9db08, 87 },
    { 0x9969db09, 58 },
    { 0x47d9db07, 66 },
    { 0x2889db0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a69db0e, 98 },
    { 0xaa79db0e, 43 },
    { 0xf8e9db10, 76 },
    { 0, 0 },
    { 0x5ec9db12, 90 },
    { 0, 0 },
    { 0x7a89db14, 91 },
    { 0xd709db15, 90 },
    { 0x3b09db16, 95 },
    { 0x7659db17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab9db1f, 61 },
    { 0x86b9db20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629db27, 93 },
    { 0, 0 },
    { 0xac29db29, 61 },
    { 0, 0 },
    { 0x7539db2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb89db2f, 90 },
    { 0x45e9db30, 61 },
    { 0xf8f9db30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec19db34, 61 },
    { 0, 0 },
    { 0x8d79db36, 61 },
    { 0, 0 },
    { 0x7d99db38, 55 },
    { 0xe369db39, 23 },
    { 0xc8b9db3a, 43 },
    { 0, 0 },
    { 0xe0a9db3c, 43 },
    { 0x70a9db3d, 52 },
    { 0x5da9db3e, 91 },
    { 0xeb79db3e, 98 },
    { 0xcba9db40, 77 },
    { 0x1989db3e, 95 },
    { 0x2529db42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd99db47, 76 },
    { 0x5449db48, 93 },
    { 0x1059db49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e59db4d, 46 },
    { 0, 0 },
    { 0x6869db4f, 68 },
    { 0x1ca9db4f, 93 },
    { 0xd659db4f, 90 },
    { 0x9c79db52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e9db55, 34 },
    { 0xafc9db56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf09db60, 76 },
    { 0xac9db61, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9979db64, 33 },
    { 0, 0 },
    { 0xd059db66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2869db6a, 83 },
    { 0, 0 },
    { 0x5909db6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9db72, 90 },
    { 0x3f59db72, 61 },
    { 0x1ab9db74, 46 },
    { 0, 0 },
    { 0xe799db76, 90 },
    { 0x59f9db77, 23 },
    { 0x5a89db77, 98 },
    { 0xc69db79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9db7d, 90 },
    { 0x9189db7e, 76 },
    { 0x48a9db7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7189db82, 90 },
    { 0xef99db83, 98 },
    { 0x3d29db83, 68 },
    { 0xf329db83, 33 },
    { 0xc5a9db82, 33 },
    { 0x80a9db82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419db95, 90 },
    { 0xf539db95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2359db9a, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29db9d, 90 },
    { 0x2b09db9e, 90 },
    { 0x3c79db9d, 61 },
    { 0xb269dba0, 46 },
    { 0xe149dba1, 76 },
    { 0x5a09dba0, 98 },
    { 0, 0 },
    { 0xa3e9dba4, 90 },
    { 0, 0 },
    { 0x66a9dba6, 95 },
    { 0xe3f9dba7, 61 },
    { 0xa069dba7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d9dbac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6459dbb2, 84 },
    { 0x2d99dbb3, 33 },
    { 0, 0 },
    { 0x86e9dbb5, 33 },
    { 0x1379dbb5, 43 },
    { 0xda9dbb7, 34 },
    { 0x4b39dbb8, 90 },
    { 0x69e9dbb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf789dbbe, 90 },
    { 0x1c99dbbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8349dbc6, 90 },
    { 0x3069dbc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5739dbcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc369dbce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9dbd1, 90 },
    { 0, 0 },
    { 0x7139dbd3, 65 },
    { 0x9b49dbd3, 33 },
    { 0, 0 },
    { 0xf049dbd6, 90 },
    { 0x2b29dbd7, 47 },
    { 0x4ca9dbd8, 98 },
    { 0x9c39dbd8, 33 },
    { 0xaf09dbda, 61 },
    { 0x93b9dbdb, 43 },
    { 0, 0 },
    { 0xf6c9dbdd, 91 },
    { 0x5019dbde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf9dbe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe439dbe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd889dbf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b79dbf6, 55 },
    { 0x419dbf7, 61 },
    { 0x96d9dbf8, 33 },
    { 0x94c9dbf9, 91 },
    { 0x6b9dbf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f9dc04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2259dc07, 94 },
    { 0x76d9dc07, 55 },
    { 0xb569dc07, 98 },
    { 0xb709dc0a, 67 },
    { 0xe029dc0b, 61 },
    { 0x7529dc0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9dc0f, 66 },
    { 0xcfd9dc0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd9dc13, 98 },
    { 0x6019dc13, 64 },
    { 0x1c59dc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e9dc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79dc1d, 33 },
    { 0x5ae9dc1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1359dc21, 98 },
    { 0x55c9dc22, 61 },
    { 0xc8b9dc23, 95 },
    { 0, 0 },
    { 0xf7a9dc25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639dc2d, 23 },
    { 0x2db9dc2d, 45 },
    { 0x20c9dc2e, 61 },
    { 0, 0 },
    { 0x6d19dc31, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859dc36, 77 },
    { 0, 0 },
    { 0xe9e9dc38, 46 },
    { 0xc869dc39, 67 },
    { 0x7009dc3a, 93 },
    { 0x5ab9dc38, 98 },
    { 0, 0 },
    { 0xa659dc3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859dc43, 46 },
    { 0xb5f9dc43, 23 },
    { 0, 0 },
    { 0x5909dc46, 23 },
    { 0x4479dc47, 17 },
    { 0, 0 },
    { 0x4b89dc49, 76 },
    { 0x1fe9dc49, 68 },
    { 0xcb9dc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c9dc4e, 75 },
    { 0x5af9dc4f, 33 },
    { 0x7409dc50, 94 },
    { 0x38f9dc4e, 61 },
    { 0, 0 },
    { 0xd729dc53, 90 },
    { 0x6949dc54, 26 },
    { 0, 0 },
    { 0x3ea9dc56, 61 },
    { 0x6699dc57, 61 },
    { 0x12d9dc58, 61 },
    { 0x5699dc59, 55 },
    { 0xd919dc5a, 33 },
    { 0xfe69dc56, 95 },
    { 0, 0 },
    { 0xd39dc5d, 83 },
    { 0x9239dc5e, 23 },
    { 0x3a69dc5e, 43 },
    { 0x9f49dc60, 61 },
    { 0xf249dc5e, 90 },
    { 0x4919dc5f, 99 },
    { 0x53b9dc63, 66 },
    { 0xea89dc64, 33 },
    { 0xcc89dc65, 55 },
    { 0xc3b9dc5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1659dc6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b49dc74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09dc7d, 91 },
    { 0x1ec9dc7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379dc82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9dc85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d9dc8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee9dc8e, 96 },
    { 0xa0d9dc8f, 61 },
    { 0x57c9dc90, 99 },
    { 0x36f9dc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f9dc98, 61 },
    { 0x7b39dc99, 33 },
    { 0x3789dc9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb9dc9e, 90 },
    { 0xdf99dc9f, 33 },
    { 0x5da9dca0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b59dca3, 23 },
    { 0x3a09dca3, 87 },
    { 0x1609dca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9dcab, 61 },
    { 0x73c9dcab, 90 },
    { 0xf7c9dcab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea9dcb0, 98 },
    { 0x4059dcb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9dcb5, 33 },
    { 0x9969dcb6, 99 },
    { 0x1709dcb6, 55 },
    { 0x7b89dcb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9dcbe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcc2, 55 },
    { 0xfcd9dcc2, 98 },
    { 0x6b9dcc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef9dcc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9dccd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb9dcd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b9dcd8, 33 },
    { 0xc29dcd9, 98 },
    { 0x9049dcda, 96 },
    { 0, 0 },
    { 0x8de9dcdc, 90 },
    { 0x99a9dcdd, 61 },
    { 0xe529dcdd, 100 },
    { 0x4929dcdd, 95 },
    { 0, 0 },
    { 0xfb39dce1, 33 },
    { 0x3389dce2, 34 },
    { 0xfc69dce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409dce7, 95 },
    { 0, 0 },
    { 0x7e79dce9, 43 },
    { 0x5229dce9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc39dced, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dcf5, 42 },
    { 0, 0 },
    { 0x4419dcf7, 52 },
    { 0, 0 },
    { 0xede9dcf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449dcfd, 61 },
    { 0x18b9dcfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de9dd02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc49dd05, 23 },
    { 0x9719dd06, 90 },
    { 0x46c9dd05, 33 },
    { 0xb719dd06, 23 },
    { 0xa6e9dd09, 66 },
    { 0x9919dd06, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9dd0d, 98 },
    { 0, 0 },
    { 0x9b39dd0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9dd14, 93 },
    { 0xdba9dd15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39dd18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d79dd22, 90 },
    { 0x5159dd23, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9dd29, 78 },
    { 0x83a9dd29, 91 },
    { 0x55a9dd2b, 33 },
    { 0xaff9dd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9dd2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309dd33, 33 },
    { 0xd759dd34, 93 },
    { 0x6509dd35, 93 },
    { 0x5da9dd36, 91 },
    { 0x2049dd34, 90 },
    { 0x4619dd38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f29dd3b, 95 },
    { 0, 0 },
    { 0x8189dd3d, 98 },
    { 0, 0 },
    { 0x80a9dd3f, 61 },
    { 0xb859dd3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd69dd43, 90 },
    { 0xf779dd44, 33 },
    { 0xb069dd45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09dd4e, 33 },
    { 0x8789dd4f, 76 },
    { 0xd859dd50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839dd5a, 100 },
    { 0xef39dd5a, 34 },
    { 0, 0 },
    { 0x2199dd5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d19dd61, 61 },
    { 0, 0 },
    { 0x4139dd63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e69dd6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb759dd6e, 91 },
    { 0x3a09dd6f, 52 },
    { 0x7a9dd70, 90 },
    { 0x4679dd71, 23 },
    { 0x30f9dd6e, 33 },
    { 0xa609dd73, 101 },
    { 0xed39dd70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2889dd79, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9dd7c, 33 },
    { 0x8b9dd7c, 33 },
    { 0x3199dd7c, 43 },
    { 0x7a9dd7f, 33 },
    { 0xbc19dd80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf9dd85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9dd88, 91 },
    { 0x179dd88, 90 },
    { 0x4169dd8a, 95 },
    { 0, 0 },
    { 0x21b9dd8c, 43 },
    { 0, 0 },
    { 0xa6b9dd8e, 90 },
    { 0, 0 },
    { 0x4df9dd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b79dd93, 85 },
    { 0x6119dd93, 67 },
    { 0, 0 },
    { 0x2689dd96, 84 },
    { 0, 0 },
    { 0x179dd98, 95 },
    { 0, 0 },
    { 0x11e9dd9a, 78 },
    { 0x27c9dd9b, 76 },
    { 0xafd9dd9c, 95 },
    { 0xcaa9dd9c, 98 },
    { 0, 0 },
    { 0xbb29dd9f, 23 },
    { 0xcab9dda0, 78 },
    { 0, 0 },
    { 0x8869dda2, 93 },
    { 0, 0 },
    { 0x68c9dda4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf49dda8, 95 },
    { 0, 0 },
    { 0xe749ddaa, 78 },
    { 0x8619ddab, 65 },
    { 0x2899ddac, 98 },
    { 0x16c9ddac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509ddb0, 90 },
    { 0, 0 },
    { 0x4409ddb2, 52 },
    { 0x54c9ddb3, 99 },
    { 0x20a9ddb2, 99 },
    { 0xdb99ddb5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19ddba, 90 },
    { 0x7e99ddba, 95 },
    { 0x76b9ddbb, 67 },
    { 0x9899ddbd, 98 },
    { 0, 0 },
    { 0xc0b9ddbf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379ddc4, 23 },
    { 0x81c9ddc4, 93 },
    { 0xf6f9ddc5, 95 },
    { 0xe3f9ddc6, 17 },
    { 0, 0 },
    { 0x54f9ddc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e89ddce, 77 },
    { 0x8019ddce, 52 },
    { 0x8a79ddce, 77 },
    { 0x5e19ddd1, 95 },
    { 0xb829ddd2, 23 },
    { 0x49d9ddd2, 90 },
    { 0xeeb9ddd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99dddc, 76 },
    { 0, 0 },
    { 0x64a9ddde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9ddee, 85 },
    { 0xe8e9ddee, 33 },
    { 0xfbf9ddf0, 100 },
    { 0, 0 },
    { 0xf599ddf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd49ddf8, 92 },
    { 0x1129ddf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9899ddfc, 90 },
    { 0xfe19ddfc, 23 },
    { 0x5079ddfc, 90 },
    { 0x4f89ddfd, 33 },
    { 0, 0 },
    { 0xe599de01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79de04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc099de07, 52 },
    { 0x7339de08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9de0b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49de12, 84 },
    { 0x39d9de12, 33 },
    { 0x2659de12, 64 },
    { 0x1029de15, 61 },
    { 0x409de16, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2399de1a, 61 },
    { 0x2569de1b, 55 },
    { 0x47b9de1c, 99 },
    { 0x96f9de1b, 38 },
    { 0x5ec9de1b, 34 },
    { 0xc549de1f, 95 },
    { 0, 0 },
    { 0x8839de21, 85 },
    { 0x7b49de22, 33 },
    { 0x4239de21, 61 },
    { 0, 0 },
    { 0xabf9de25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9de29, 90 },
    { 0xa869de2a, 55 },
    { 0, 0 },
    { 0x5bd9de2c, 46 },
    { 0x7479de2d, 87 },
    { 0, 0 },
    { 0x6829de2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf449de35, 95 },
    { 0x9d39de36, 23 },
    { 0xaa39de37, 46 },
    { 0x88d9de37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd99de3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce89de3f, 93 },
    { 0xbc79de40, 43 },
    { 0x43b9de41, 23 },
    { 0x3239de42, 93 },
    { 0xba29de43, 23 },
    { 0x83c9de43, 99 },
    { 0x1759de43, 77 },
    { 0x1699de42, 33 },
    { 0xf949de47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb969de4b, 92 },
    { 0x1719de4c, 48 },
    { 0x3999de4d, 98 },
    { 0xca09de4e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe889de51, 67 },
    { 0, 0 },
    { 0xdeb9de53, 23 },
    { 0, 0 },
    { 0x7ce9de55, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679de5a, 61 },
    { 0x1069de5b, 33 },
    { 0, 0 },
    { 0xe869de5d, 33 },
    { 0, 0 },
    { 0x2b69de5f, 87 },
    { 0xb319de60, 95 },
    { 0, 0 },
    { 0x9759de62, 55 },
    { 0x6c09de63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9de68, 23 },
    { 0, 0 },
    { 0xdf69de6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c9de6d, 99 },
    { 0x9dc9de6e, 34 },
    { 0, 0 },
    { 0xdc89de70, 61 },
    { 0, 0 },
    { 0x1699de72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719de76, 34 },
    { 0x6dd9de77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c9de7b, 90 },
    { 0x3479de7c, 88 },
    { 0, 0 },
    { 0x3c99de7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f79de81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d9de84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049de8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc09de98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c69de9f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c39dea4, 95 },
    { 0, 0 },
    { 0x3019dea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69deab, 34 },
    { 0xe799deac, 90 },
    { 0x7789deab, 98 },
    { 0, 0 },
    { 0x2629deaf, 87 },
    { 0x9c19deb0, 23 },
    { 0x10b9deb0, 84 },
    { 0, 0 },
    { 0x6a79deb3, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c9deb6, 61 },
    { 0, 0 },
    { 0x5689deb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca9debd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c9dec2, 61 },
    { 0x4c49dec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69deca, 100 },
    { 0x3be9decb, 61 },
    { 0, 0 },
    { 0xf969decd, 37 },
    { 0x5899dece, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59ded3, 33 },
    { 0, 0 },
    { 0x6209ded5, 75 },
    { 0x7079ded6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c9deda, 33 },
    { 0xef19dedb, 98 },
    { 0, 0 },
    { 0x3469dedd, 23 },
    { 0xa719dede, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4079dee2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5509dee5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39dee8, 99 },
    { 0, 0 },
    { 0x6309deea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf9deee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf659def2, 23 },
    { 0, 0 },
    { 0xaf59def4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e9defa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079deff, 78 },
    { 0xcae9df00, 83 },
    { 0, 0 },
    { 0x82c9df02, 95 },
    { 0, 0 },
    { 0x759df04, 100 },
    { 0xd149df04, 33 },
    { 0x6709df06, 98 },
    { 0xd339df07, 90 },
    { 0xa829df08, 61 },
    { 0x3159df04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069df0c, 99 },
    { 0x18d9df0c, 87 },
    { 0x46c9df0d, 98 },
    { 0xd1f9df0d, 61 },
    { 0, 0 },
    { 0xd0b9df11, 36 },
    { 0x8f79df11, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c9df19, 33 },
    { 0xeb9df19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a39df1f, 90 },
    { 0x77a9df1f, 90 },
    { 0xa6b9df21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9df25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa449df2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f9df30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9df33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129df38, 55 },
    { 0, 0 },
    { 0x7b59df3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819df3d, 88 },
    { 0x919df3d, 61 },
    { 0xcd49df3e, 98 },
    { 0x5e69df40, 85 },
    { 0, 0 },
    { 0xf739df42, 17 },
    { 0, 0 },
    { 0xa869df44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea9df4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1699df50, 61 },
    { 0, 0 },
    { 0x6e39df52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579df58, 34 },
    { 0, 0 },
    { 0xf939df5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a9df5e, 61 },
    { 0, 0 },
    { 0x1c49df60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1149df64, 88 },
    { 0x339df65, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069df68, 33 },
    { 0x20d9df69, 34 },
    { 0, 0 },
    { 0xd1a9df6b, 91 },
    { 0xec9df6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59df6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad39df72, 90 },
    { 0x5e79df72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f9df77, 90 },
    { 0x2cc9df77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509df7b, 46 },
    { 0x95e9df7b, 90 },
    { 0xb169df7b, 33 },
    { 0, 0 },
    { 0x87d9df7f, 90 },
    { 0xc3e9df80, 90 },
    { 0xd1f9df80, 90 },
    { 0xed79df81, 34 },
    { 0, 0 },
    { 0x2f9df84, 66 },
    { 0, 0 },
    { 0xf6b9df86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc9df8c, 76 },
    { 0, 0 },
    { 0x13e9df8e, 33 },
    { 0xaaa9df8f, 23 },
    { 0, 0 },
    { 0x22a9df91, 61 },
    { 0x3989df92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e9df95, 85 },
    { 0, 0 },
    { 0xf899df97, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d49df9a, 101 },
    { 0xacc9df9a, 95 },
    { 0x2f09df9c, 33 },
    { 0xbb39df9c, 55 },
    { 0x1d9df9e, 55 },
    { 0xccb9df9e, 84 },
    { 0xa299dfa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9dfa4, 91 },
    { 0, 0 },
    { 0x3dd9dfa6, 95 },
    { 0, 0 },
    { 0xd6b9dfa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef89dfac, 53 },
    { 0, 0 },
    { 0x8ed9dfae, 99 },
    { 0x1fd9dfaf, 33 },
    { 0x54d9dfb0, 83 },
    { 0xb939dfb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649dfb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9dfbc, 34 },
    { 0xa6b9dfbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f9dfc3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a9dfcc, 43 },
    { 0, 0 },
    { 0xadc9dfce, 76 },
    { 0, 0 },
    { 0xcbd9dfd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129dfd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e9dfd7, 98 },
    { 0, 0 },
    { 0x819dfd9, 93 },
    { 0x8bf9dfd9, 34 },
    { 0xf609dfda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489dfde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a9dfe8, 61 },
    { 0x6d99dfe9, 95 },
    { 0xad69dfe8, 33 },
    { 0, 0 },
    { 0xa099dfec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e9dfef, 90 },
    { 0x8e79dff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9dff6, 90 },
    { 0x4d19dff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b69dffa, 33 },
    { 0, 0 },
    { 0x16e9dffc, 34 },
    { 0, 0 },
    { 0x9ae9dffe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8589e001, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a29e005, 90 },
    { 0xd9d9e006, 93 },
    { 0x489e007, 100 },
    { 0x6e79e005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99e00b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf709e00e, 98 },
    { 0xc719e00f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec79e014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb9e017, 90 },
    { 0, 0 },
    { 0x54f9e019, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99e01c, 55 },
    { 0x8f69e01d, 78 },
    { 0xbfa9e01e, 61 },
    { 0x1579e01f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9e022, 61 },
    { 0xe5c9e023, 78 },
    { 0x4de9e023, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079e02a, 33 },
    { 0, 0 },
    { 0x4549e02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9e034, 52 },
    { 0, 0 },
    { 0x1b99e036, 55 },
    { 0x1ea9e037, 23 },
    { 0x1af9e038, 61 },
    { 0, 0 },
    { 0x7fc9e03a, 34 },
    { 0x19c9e03b, 33 },
    { 0xa569e03c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8169e040, 90 },
    { 0x3339e041, 34 },
    { 0xb2e9e042, 61 },
    { 0x4409e043, 90 },
    { 0, 0 },
    { 0x6a29e045, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa9e049, 46 },
    { 0x63f9e049, 76 },
    { 0, 0 },
    { 0xa9d9e04c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf149e051, 95 },
    { 0x7f69e052, 95 },
    { 0, 0 },
    { 0xfb79e054, 34 },
    { 0x2789e054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa889e059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4159e05e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199e062, 95 },
    { 0x1aa9e062, 84 },
    { 0x74a9e064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b9e067, 87 },
    { 0x6199e068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc659e06b, 95 },
    { 0, 0 },
    { 0x8049e06d, 52 },
    { 0x7869e06e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a89e073, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439e076, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c9e07c, 90 },
    { 0x9ef9e07d, 43 },
    { 0x5a09e07e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9289e082, 55 },
    { 0xa4f9e083, 77 },
    { 0xfcd9e083, 61 },
    { 0xe2e9e085, 90 },
    { 0xb179e084, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb59e08c, 95 },
    { 0, 0 },
    { 0xb529e08e, 67 },
    { 0x94d9e08f, 56 },
    { 0xa39e08f, 76 },
    { 0x1a9e091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a9e096, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb39e099, 66 },
    { 0x7f29e09a, 38 },
    { 0x1229e09b, 23 },
    { 0xb779e09c, 95 },
    { 0xcfe9e09c, 23 },
    { 0, 0 },
    { 0x16c9e09f, 99 },
    { 0x20a9e09f, 23 },
    { 0xef19e0a1, 61 },
    { 0xf0b9e0a2, 33 },
    { 0xe179e0a3, 95 },
    { 0x4969e0a4, 100 },
    { 0xfcd9e0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd09e0ae, 78 },
    { 0xa029e0af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf59e0b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9e0b6, 23 },
    { 0xa379e0b6, 98 },
    { 0x8179e0b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779e0bd, 85 },
    { 0, 0 },
    { 0x2ab9e0bf, 23 },
    { 0x9309e0c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9e0c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc339e0c7, 23 },
    { 0x7239e0c8, 34 },
    { 0x8869e0c9, 90 },
    { 0x81e9e0c9, 95 },
    { 0x7979e0c9, 93 },
    { 0x5949e0cc, 93 },
    { 0xfcb9e0cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59e0d5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f9e0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89e0de, 61 },
    { 0xea09e0df, 33 },
    { 0, 0 },
    { 0x9eb9e0e1, 95 },
    { 0xf3c9e0e1, 98 },
    { 0x6579e0e3, 61 },
    { 0, 0 },
    { 0xe199e0e5, 76 },
    { 0x6819e0e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3169e0eb, 32 },
    { 0, 0 },
    { 0xdf49e0ed, 66 },
    { 0xa829e0ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea9e0f1, 23 },
    { 0xbce9e0f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9949e0f6, 90 },
    { 0xce59e0f6, 76 },
    { 0xd3a9e0f7, 33 },
    { 0x4079e0f9, 33 },
    { 0x1039e0fa, 68 },
    { 0, 0 },
    { 0xb0f9e0fc, 33 },
    { 0xa979e0fd, 85 },
    { 0xb109e0fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2649e102, 93 },
    { 0xa739e102, 90 },
    { 0x6a09e103, 98 },
    { 0x4c39e105, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7729e109, 99 },
    { 0x1e49e10a, 52 },
    { 0xbe99e10b, 66 },
    { 0xcff9e10c, 46 },
    { 0xeca9e10d, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999e110, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1269e115, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca39e11a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6749e11f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff09e123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179e126, 90 },
    { 0, 0 },
    { 0xb159e128, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da9e12b, 33 },
    { 0x1e9e12c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9e130, 66 },
    { 0x1729e131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf309e135, 33 },
    { 0x6f59e135, 61 },
    { 0x3cf9e137, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x269e13a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1499e13d, 23 },
    { 0, 0 },
    { 0xd7d9e13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689e146, 61 },
    { 0xbc69e147, 23 },
    { 0, 0 },
    { 0x5069e149, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929e14d, 23 },
    { 0xdcd9e14e, 90 },
    { 0xe0e9e14f, 34 },
    { 0x6e19e150, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2019e15d, 85 },
    { 0x28d9e15e, 34 },
    { 0xcd49e15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1349e164, 90 },
    { 0xf0f9e164, 97 },
    { 0, 0 },
    { 0x9ad9e167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7339e16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6759e16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d9e174, 84 },
    { 0, 0 },
    { 0x7699e176, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf699e179, 93 },
    { 0x1469e17a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b9e17f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c9e183, 98 },
    { 0xf4e9e184, 17 },
    { 0xe969e185, 55 },
    { 0x7fc9e185, 76 },
    { 0x6a09e187, 23 },
    { 0, 0 },
    { 0xa0d9e189, 61 },
    { 0x4089e18a, 99 },
    { 0x6299e18b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4259e18e, 34 },
    { 0x9de9e18f, 76 },
    { 0x89e190, 33 },
    { 0, 0 },
    { 0xd289e192, 33 },
    { 0, 0 },
    { 0x2af9e194, 33 },
    { 0x65f9e195, 87 },
    { 0x90e9e196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e49e19b, 33 },
    { 0x7369e19b, 33 },
    { 0, 0 },
    { 0x2829e19e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8749e1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f9e1ab, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539e1af, 84 },
    { 0x3ca9e1b0, 33 },
    { 0x7799e1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f09e1b4, 68 },
    { 0xf329e1b5, 43 },
    { 0x5739e1b6, 61 },
    { 0x8829e1b7, 55 },
    { 0xad09e1b8, 90 },
    { 0x1979e1b9, 98 },
    { 0xcac9e1b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f9e1c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269e1c4, 95 },
    { 0x7a9e1c5, 98 },
    { 0xccd9e1c6, 55 },
    { 0, 0 },
    { 0x2c9e1c8, 33 },
    { 0x9e69e1c9, 17 },
    { 0x40f9e1c9, 33 },
    { 0, 0 },
    { 0x64b9e1cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd519e1d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe329e1d7, 33 },
    { 0x57a9e1d8, 95 },
    { 0xde29e1d9, 95 },
    { 0xa9d9e1da, 95 },
    { 0, 0 },
    { 0x74a9e1dc, 98 },
    { 0xcb39e1dd, 90 },
    { 0x7fb9e1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149e1e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe369e1ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8009e1f2, 98 },
    { 0x8a59e1f3, 23 },
    { 0x4f29e1f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629e1fc, 34 },
    { 0x57b9e1fd, 55 },
    { 0x14c9e1fe, 66 },
    { 0xa1d9e1ff, 84 },
    { 0xfe09e200, 26 },
    { 0x94a9e200, 61 },
    { 0x5f79e201, 34 },
    { 0x3f69e202, 90 },
    { 0x6b79e203, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d79e208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8329e20b, 99 },
    { 0, 0 },
    { 0x56a9e20d, 43 },
    { 0, 0 },
    { 0xa599e20f, 23 },
    { 0x5a09e210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c9e214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d9e219, 95 },
    { 0xd329e21a, 33 },
    { 0x1c9e21a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b9e21e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329e226, 61 },
    { 0xf5c9e227, 33 },
    { 0, 0 },
    { 0x66c9e229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd9e22e, 55 },
    { 0x83a9e22f, 90 },
    { 0xc0d9e22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4359e235, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f9e238, 90 },
    { 0, 0 },
    { 0x6829e23a, 76 },
    { 0xefb9e23a, 61 },
    { 0, 0 },
    { 0xac39e23d, 61 },
    { 0, 0 },
    { 0xa199e23f, 66 },
    { 0xa0e9e240, 78 },
    { 0x5a39e240, 66 },
    { 0x47a9e242, 76 },
    { 0x58b9e243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd029e248, 76 },
    { 0xe9e249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc69e24f, 23 },
    { 0xf19e24f, 33 },
    { 0x60b9e251, 17 },
    { 0x3c59e252, 76 },
    { 0, 0 },
    { 0xf3b9e254, 34 },
    { 0xa749e255, 61 },
    { 0x4439e255, 90 },
    { 0xa8a9e257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729e25c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc429e25f, 100 },
    { 0xcaa9e260, 61 },
    { 0x4749e260, 66 },
    { 0xae69e261, 61 },
    { 0xe2e9e263, 34 },
    { 0xe299e25f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259e268, 90 },
    { 0x77d9e269, 90 },
    { 0xbad9e269, 78 },
    { 0x7ab9e26b, 87 },
    { 0x3a79e26c, 95 },
    { 0x3ea9e26c, 61 },
    { 0xe899e26e, 43 },
    { 0xdd59e26d, 55 },
    { 0x6df9e26d, 43 },
    { 0, 0 },
    { 0xf759e272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49e275, 46 },
    { 0, 0 },
    { 0xee39e277, 33 },
    { 0x6ce9e278, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f9e27c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd579e283, 46 },
    { 0x5809e284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259e289, 52 },
    { 0xb429e289, 95 },
    { 0xebe9e289, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf9e28e, 46 },
    { 0x7099e28f, 52 },
    { 0xf119e28f, 31 },
    { 0xe79e28f, 43 },
    { 0, 0 },
    { 0x9d39e293, 34 },
    { 0, 0 },
    { 0x24a9e295, 61 },
    { 0x8179e296, 90 },
    { 0x41e9e297, 90 },
    { 0x4199e296, 46 },
    { 0x4459e297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae99e2a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc029e2a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c09e2aa, 23 },
    { 0xf319e2ab, 76 },
    { 0, 0 },
    { 0xe659e2ad, 33 },
    { 0xdd29e2ae, 98 },
    { 0x1239e2af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d9e2b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8759e2b8, 90 },
    { 0xc9b9e2b8, 98 },
    { 0, 0 },
    { 0x7ca9e2bb, 78 },
    { 0xc479e2bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189e2c0, 23 },
    { 0xcb99e2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b9e2c8, 90 },
    { 0xd1b9e2c8, 98 },
    { 0x3eb9e2ca, 88 },
    { 0xf439e2ca, 91 },
    { 0xb389e2ca, 66 },
    { 0x9ad9e2cb, 33 },
    { 0x1bd9e2ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789e2d2, 98 },
    { 0xa599e2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8869e2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9e2dc, 99 },
    { 0xb9c9e2dd, 35 },
    { 0x47b9e2de, 23 },
    { 0x999e2df, 95 },
    { 0x60d9e2de, 34 },
    { 0x4989e2e1, 84 },
    { 0xdd09e2e2, 91 },
    { 0x6c99e2e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9e2e6, 93 },
    { 0, 0 },
    { 0x719e2e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a9e2ef, 34 },
    { 0, 0 },
    { 0x1539e2f1, 61 },
    { 0xa9f9e2f2, 55 },
    { 0x6c79e2f3, 33 },
    { 0xb949e2f4, 91 },
    { 0, 0 },
    { 0xd599e2f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7329e2ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa479e303, 33 },
    { 0x5849e304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e30a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359e310, 90 },
    { 0xc6c9e310, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459e314, 90 },
    { 0x9d49e315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9e318, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xced9e31b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39e31e, 76 },
    { 0, 0 },
    { 0x6ad9e320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9e324, 90 },
    { 0x8d49e325, 67 },
    { 0, 0 },
    { 0xdb39e327, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc9e32b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99e333, 55 },
    { 0, 0 },
    { 0x2f89e335, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b9e338, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f49e341, 33 },
    { 0x6d19e342, 90 },
    { 0, 0 },
    { 0x4e09e344, 98 },
    { 0, 0 },
    { 0xeaf9e346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019e34e, 61 },
    { 0x4e99e34f, 23 },
    { 0xa3c9e350, 46 },
    { 0xcc99e350, 55 },
    { 0x5e79e352, 40 },
    { 0x5e09e353, 90 },
    { 0xdfc9e354, 90 },
    { 0, 0 },
    { 0xa49e356, 76 },
    { 0xd599e356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9e35b, 78 },
    { 0x43c9e35c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9689e365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3609e375, 23 },
    { 0, 0 },
    { 0xa189e377, 91 },
    { 0, 0 },
    { 0xd5e9e379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca9e37c, 85 },
    { 0x2909e37c, 90 },
    { 0x129e37e, 61 },
    { 0x98c9e37f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d59e38c, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a9e38f, 34 },
    { 0, 0 },
    { 0xcc79e391, 90 },
    { 0x4219e392, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9e396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9879e39f, 36 },
    { 0xcd9e3a0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419e3a4, 85 },
    { 0xc1b9e3a5, 98 },
    { 0x3199e3a6, 90 },
    { 0xbc09e3a6, 61 },
    { 0, 0 },
    { 0x6f39e3a9, 90 },
    { 0xf0c9e3aa, 94 },
    { 0xc819e3a9, 77 },
    { 0x2899e3ac, 100 },
    { 0xae89e3ac, 90 },
    { 0xca09e3ad, 95 },
    { 0x8f79e3ad, 34 },
    { 0x7389e3b0, 33 },
    { 0, 0 },
    { 0xe639e3b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909e3b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9e3b9, 87 },
    { 0, 0 },
    { 0xc179e3bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e09e3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc779e3c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf019e3d2, 46 },
    { 0, 0 },
    { 0x58a9e3d4, 34 },
    { 0x6a99e3d5, 61 },
    { 0x7fd9e3d6, 49 },
    { 0, 0 },
    { 0x2ba9e3d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b9e3df, 40 },
    { 0, 0 },
    { 0x65e9e3e1, 87 },
    { 0xda79e3e2, 93 },
    { 0xecf9e3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9e3eb, 81 },
    { 0xe3d9e3ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b69e3ef, 61 },
    { 0, 0 },
    { 0x8299e3f1, 98 },
    { 0x759e3f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3939e3f9, 90 },
    { 0x5719e3fa, 53 },
    { 0x12f9e3fb, 85 },
    { 0x54f9e3fb, 23 },
    { 0x6be9e3fd, 98 },
    { 0xe49e3fc, 53 },
    { 0, 0 },
    { 0x15d9e400, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a19e403, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e9e407, 33 },
    { 0xaec9e408, 23 },
    { 0x4a09e409, 99 },
    { 0, 0 },
    { 0x3f59e40b, 55 },
    { 0xf6e9e40b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e40f, 75 },
    { 0x5469e40f, 98 },
    { 0, 0 },
    { 0x10c9e412, 94 },
    { 0xee19e412, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49e41a, 55 },
    { 0xa69e41b, 93 },
    { 0xeee9e41c, 98 },
    { 0x6a49e41d, 38 },
    { 0x1569e41e, 33 },
    { 0xc999e41f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e89e424, 90 },
    { 0x4699e425, 98 },
    { 0xfdd9e426, 52 },
    { 0x73e9e427, 95 },
    { 0xf69e425, 23 },
    { 0xb2e9e429, 23 },
    { 0x2d29e429, 17 },
    { 0x23b9e42b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce9e430, 76 },
    { 0xd769e430, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec09e434, 63 },
    { 0x40f9e435, 90 },
    { 0, 0 },
    { 0x3ec9e437, 75 },
    { 0xbfc9e438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9e445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3899e448, 66 },
    { 0x3f69e449, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8609e450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e455, 91 },
    { 0xc59e455, 23 },
    { 0xd359e457, 95 },
    { 0, 0 },
    { 0x2da9e459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec9e45c, 90 },
    { 0x3c09e45c, 88 },
    { 0, 0 },
    { 0xe099e45f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629e464, 95 },
    { 0x4fc9e465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559e46a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9729e46d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d9e472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859e475, 76 },
    { 0, 0 },
    { 0x10d9e477, 33 },
    { 0x6f39e478, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59e47f, 95 },
    { 0, 0 },
    { 0x6b29e481, 43 },
    { 0x3ec9e482, 52 },
    { 0xd0a9e483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9e486, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e29e48b, 52 },
    { 0x11f9e48c, 97 },
    { 0xef99e48c, 61 },
    { 0x32c9e48e, 61 },
    { 0x8e09e48f, 90 },
    { 0xdf99e48d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd119e495, 76 },
    { 0, 0 },
    { 0xb299e497, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb899e49d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c49e4a0, 61 },
    { 0x5c59e4a1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5109e4a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b9e4a9, 90 },
    { 0x4449e4a9, 99 },
    { 0, 0 },
    { 0xfbd9e4ac, 55 },
    { 0, 0 },
    { 0xbe9e4ae, 40 },
    { 0x79b9e4af, 98 },
    { 0x5199e4b0, 43 },
    { 0xe949e4ae, 76 },
    { 0xa6b9e4b2, 23 },
    { 0x6949e4b3, 33 },
    { 0, 0 },
    { 0x52e9e4b5, 78 },
    { 0, 0 },
    { 0x9209e4b7, 43 },
    { 0xb569e4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299e4be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709e4c1, 69 },
    { 0, 0 },
    { 0x4d79e4c3, 90 },
    { 0, 0 },
    { 0x2d69e4c5, 91 },
    { 0, 0 },
    { 0x5c59e4c7, 23 },
    { 0xa7a9e4c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d49e4cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f9e4d1, 76 },
    { 0x109e4d1, 90 },
    { 0xf249e4d1, 99 },
    { 0x4059e4d3, 66 },
    { 0x27d9e4d5, 98 },
    { 0x9759e4d5, 23 },
    { 0x6bd9e4d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa669e4df, 100 },
    { 0x2489e4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e4e6, 34 },
    { 0x63d9e4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9e4eb, 90 },
    { 0xb779e4eb, 52 },
    { 0xb479e4ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6099e4f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9e4f4, 87 },
    { 0, 0 },
    { 0x5549e4f6, 76 },
    { 0x5da9e4f7, 90 },
    { 0, 0 },
    { 0x5af9e4f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab89e501, 83 },
    { 0x1469e501, 61 },
    { 0x39c9e502, 80 },
    { 0x2ef9e502, 17 },
    { 0xdc79e505, 90 },
    { 0x5aa9e506, 90 },
    { 0xb789e507, 61 },
    { 0xc159e508, 100 },
    { 0xa549e502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d9e50c, 95 },
    { 0, 0 },
    { 0x61c9e50e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9e513, 61 },
    { 0, 0 },
    { 0xe519e515, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c19e51a, 95 },
    { 0x8a39e51b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9e51f, 90 },
    { 0, 0 },
    { 0x8279e521, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e9e527, 88 },
    { 0, 0 },
    { 0x219e529, 76 },
    { 0x8549e529, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e530, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b09e533, 99 },
    { 0xdf39e533, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9559e537, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379e53a, 33 },
    { 0xe089e53a, 43 },
    { 0x2409e53b, 86 },
    { 0x6229e53a, 66 },
    { 0x8f69e53d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39e542, 80 },
    { 0x9db9e543, 77 },
    { 0xaf99e544, 100 },
    { 0, 0 },
    { 0x4db9e546, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc569e549, 90 },
    { 0xda09e54a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc9e54e, 95 },
    { 0, 0 },
    { 0x3ca9e550, 84 },
    { 0x20d9e551, 23 },
    { 0xf839e550, 23 },
    { 0x9019e550, 33 },
    { 0x6f89e554, 76 },
    { 0x96c9e555, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb09e55b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa9e561, 98 },
    { 0x6619e562, 55 },
    { 0, 0 },
    { 0x6769e564, 40 },
    { 0x7c49e565, 95 },
    { 0x9129e566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299e56d, 23 },
    { 0x30c9e56e, 90 },
    { 0x4429e56d, 76 },
    { 0xd049e570, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb9e577, 33 },
    { 0x7059e578, 34 },
    { 0x4c9e579, 34 },
    { 0, 0 },
    { 0xb0c9e57b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b9e583, 56 },
    { 0, 0 },
    { 0x1cf9e585, 91 },
    { 0x7a49e586, 61 },
    { 0, 0 },
    { 0x5469e588, 91 },
    { 0x4299e589, 95 },
    { 0, 0 },
    { 0x1ee9e58b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6809e593, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e9e596, 76 },
    { 0xeb49e597, 17 },
    { 0x77c9e598, 98 },
    { 0x2679e599, 52 },
    { 0xfba9e599, 95 },
    { 0x849e598, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f59e59e, 90 },
    { 0x89c9e59f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac9e5a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49e5a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a9e5ab, 23 },
    { 0x9bf9e5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac79e5b6, 69 },
    { 0, 0 },
    { 0x8e9e5b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5989e5bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f9e5be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9e5c1, 98 },
    { 0x6749e5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9e5c6, 33 },
    { 0x2759e5c7, 61 },
    { 0xdd99e5c7, 61 },
    { 0xd859e5c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe449e5cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f89e5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c89e5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9e5db, 90 },
    { 0x5ee9e5dc, 68 },
    { 0x2149e5dc, 33 },
    { 0, 0 },
    { 0x9bb9e5df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x479e5e2, 81 },
    { 0xeda9e5e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe9e5e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9e5ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a9e5f0, 23 },
    { 0, 0 },
    { 0x1149e5f2, 76 },
    { 0xcca9e5f3, 90 },
    { 0, 0 },
    { 0x7bf9e5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb79e601, 93 },
    { 0x7719e602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9e607, 90 },
    { 0, 0 },
    { 0xc0b9e609, 33 },
    { 0x1b9e60a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29e60e, 76 },
    { 0, 0 },
    { 0x2bd9e610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9e614, 23 },
    { 0x3fc9e615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4879e619, 33 },
    { 0x3aa9e61a, 33 },
    { 0x3cd9e61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa339e61e, 90 },
    { 0xd59e61f, 61 },
    { 0x8009e61f, 53 },
    { 0x7cd9e621, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2039e624, 43 },
    { 0xec89e625, 91 },
    { 0x8bb9e624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb069e62a, 90 },
    { 0, 0 },
    { 0x1329e62c, 99 },
    { 0x7f39e62d, 90 },
    { 0, 0 },
    { 0xb269e62f, 95 },
    { 0xe669e630, 98 },
    { 0xce49e62f, 40 },
    { 0xdad9e631, 34 },
    { 0x5299e633, 43 },
    { 0xe39e634, 77 },
    { 0xfaa9e633, 67 },
    { 0xd829e630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4569e63e, 23 },
    { 0xd769e63f, 61 },
    { 0x259e63e, 61 },
    { 0xbc69e63e, 23 },
    { 0x19d9e63e, 61 },
    { 0xc5d9e643, 23 },
    { 0x409e644, 55 },
    { 0x2a39e645, 46 },
    { 0xd7a9e646, 91 },
    { 0xe749e647, 43 },
    { 0, 0 },
    { 0xe059e649, 61 },
    { 0x3279e64a, 95 },
    { 0x55b9e64a, 95 },
    { 0x4499e64a, 90 },
    { 0x1479e64a, 91 },
    { 0x819e649, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479e652, 61 },
    { 0x38f9e653, 33 },
    { 0x4329e654, 33 },
    { 0x2ab9e652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639e65b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9e660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf589e664, 33 },
    { 0x64b9e665, 23 },
    { 0x69e9e665, 87 },
    { 0x7de9e665, 98 },
    { 0xebe9e668, 76 },
    { 0xe4a9e668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc659e675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9e67a, 91 },
    { 0x70f9e67b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9e67e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b9e681, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9e684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3619e68a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9489e68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a29e690, 17 },
    { 0x1ab9e690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6369e696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9e69e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf49e6a4, 3 },
    { 0x29e9e6a5, 95 },
    { 0xf7c9e6a5, 100 },
    { 0xbe9e6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8719e6af, 90 },
    { 0x2989e6b0, 33 },
    { 0xe589e6b0, 93 },
    { 0x1a49e6b2, 77 },
    { 0x4209e6b3, 76 },
    { 0x2b69e6b3, 95 },
    { 0x3fa9e6b4, 90 },
    { 0x42a9e6b6, 91 },
    { 0xe009e6b6, 85 },
    { 0, 0 },
    { 0x9219e6b9, 61 },
    { 0x8479e6b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f9e6be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb9e6c3, 61 },
    { 0xcec9e6c4, 98 },
    { 0x2399e6c5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e19e6c9, 98 },
    { 0xd469e6c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5849e6cd, 82 },
    { 0x6ee9e6ce, 90 },
    { 0x79e9e6cf, 61 },
    { 0, 0 },
    { 0xa99e6d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889e6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c29e6d7, 33 },
    { 0x23f9e6d8, 99 },
    { 0x32b9e6d9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9e6dd, 61 },
    { 0x21c9e6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac29e6e7, 98 },
    { 0xbf29e6e8, 33 },
    { 0, 0 },
    { 0xe4c9e6ea, 61 },
    { 0x88e9e6eb, 90 },
    { 0x1989e6ec, 95 },
    { 0x6869e6ed, 33 },
    { 0x4529e6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d9e6f5, 23 },
    { 0x2a9e6f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa109e6fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc9e6ff, 90 },
    { 0xa909e700, 61 },
    { 0x3e39e700, 55 },
    { 0xa4d9e702, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c49e70a, 61 },
    { 0x4039e70b, 90 },
    { 0, 0 },
    { 0xec29e70d, 46 },
    { 0x38e9e70d, 55 },
    { 0xb709e70e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b89e712, 52 },
    { 0x8919e713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d9e716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5549e71b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e9e71e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f89e722, 33 },
    { 0xb959e723, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a9e72a, 100 },
    { 0x3bf9e72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c9e72f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8809e734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1919e737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b09e73b, 40 },
    { 0, 0 },
    { 0x6c9e73d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8169e740, 98 },
    { 0, 0 },
    { 0x70d9e742, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9e747, 95 },
    { 0xd989e747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9e74b, 81 },
    { 0x1769e74c, 95 },
    { 0xbe59e74d, 55 },
    { 0x8cc9e74e, 95 },
    { 0x3329e74f, 95 },
    { 0x2c59e750, 23 },
    { 0xa379e74e, 95 },
    { 0x1b09e752, 98 },
    { 0xc9b9e753, 90 },
    { 0xf029e754, 66 },
    { 0xd299e74c, 33 },
    { 0xc209e74d, 33 },
    { 0xf649e757, 81 },
    { 0xb179e758, 34 },
    { 0, 0 },
    { 0x7599e75a, 69 },
    { 0, 0 },
    { 0x7d89e75c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9e765, 69 },
    { 0x4419e765, 98 },
    { 0xe7e9e767, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c59e76e, 55 },
    { 0x5df9e76f, 76 },
    { 0, 0 },
    { 0x71a9e771, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b9e776, 90 },
    { 0, 0 },
    { 0x3a89e778, 23 },
    { 0x67c9e779, 43 },
    { 0, 0 },
    { 0x18a9e77b, 23 },
    { 0x8fe9e77c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc359e785, 66 },
    { 0, 0 },
    { 0x2f39e787, 88 },
    { 0x7a9e787, 98 },
    { 0x4cc9e789, 65 },
    { 0xa529e788, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799e790, 80 },
    { 0xb3e9e791, 43 },
    { 0x2b69e791, 23 },
    { 0x98a9e790, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9e7a4, 88 },
    { 0, 0 },
    { 0xb939e7a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9e7ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b49e7b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e7b6, 55 },
    { 0x319e7b7, 95 },
    { 0x5679e7b7, 76 },
    { 0, 0 },
    { 0x5699e7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9e7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f09e7c2, 61 },
    { 0x1ca9e7c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979e7c9, 78 },
    { 0x5739e7c9, 99 },
    { 0xe7f9e7cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab39e7cf, 33 },
    { 0x41b9e7d0, 33 },
    { 0xca89e7d1, 33 },
    { 0, 0 },
    { 0x6419e7d3, 23 },
    { 0x9c69e7d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d09e7dd, 95 },
    { 0xc749e7dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f79e7e2, 33 },
    { 0, 0 },
    { 0x4e09e7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa209e7e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe9e7eb, 23 },
    { 0x1ca9e7ec, 85 },
    { 0x47b9e7eb, 90 },
    { 0x4e79e7ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9e7f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d9e7f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9e7fe, 95 },
    { 0x4519e7fe, 69 },
    { 0x93b9e800, 98 },
    { 0x7319e7ff, 90 },
    { 0x83c9e802, 76 },
    { 0xbcf9e803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e809, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9e80e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9e811, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239e814, 33 },
    { 0x9349e814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649e81a, 33 },
    { 0xc0e9e81b, 33 },
    { 0xcdf9e81c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8769e823, 23 },
    { 0, 0 },
    { 0x9699e825, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf69e828, 33 },
    { 0x849e829, 98 },
    { 0, 0 },
    { 0x7dd9e82b, 46 },
    { 0x3db9e82c, 95 },
    { 0, 0 },
    { 0x389e82e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69e831, 23 },
    { 0x1219e831, 98 },
    { 0x2749e831, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719e837, 93 },
    { 0x7789e837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e09e842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe959e847, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8199e84f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c9e852, 23 },
    { 0xcb9e852, 98 },
    { 0x9e854, 67 },
    { 0x6179e855, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59e85b, 33 },
    { 0x2309e85c, 61 },
    { 0x6c9e85d, 52 },
    { 0, 0 },
    { 0x2ea9e85f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d9e863, 98 },
    { 0x7ba9e864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3299e86c, 99 },
    { 0x5cb9e86d, 81 },
    { 0xfb29e86d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9e873, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1249e879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d29e87e, 33 },
    { 0, 0 },
    { 0x3cd9e880, 90 },
    { 0xa019e881, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3749e886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa029e88c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7749e88f, 76 },
    { 0xc469e890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a69e893, 61 },
    { 0, 0 },
    { 0xdec9e895, 61 },
    { 0xfdc9e896, 99 },
    { 0x4b69e897, 61 },
    { 0xb939e897, 93 },
    { 0x8959e898, 23 },
    { 0x66b9e89a, 95 },
    { 0x5709e898, 61 },
    { 0xafe9e899, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf9e8a6, 40 },
    { 0, 0 },
    { 0xc199e8a8, 23 },
    { 0xaef9e8a9, 88 },
    { 0x3f09e8a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5879e8ae, 95 },
    { 0xba9e8af, 90 },
    { 0, 0 },
    { 0xf999e8b1, 95 },
    { 0x5049e8b2, 23 },
    { 0xaf69e8b2, 61 },
    { 0xebf9e8b4, 75 },
    { 0xb289e8b4, 95 },
    { 0x78c9e8b6, 81 },
    { 0, 0 },
    { 0x56a9e8b8, 78 },
    { 0xa0f9e8b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf929e8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729e8c3, 33 },
    { 0xd859e8c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e9e8ca, 23 },
    { 0, 0 },
    { 0xc89e8cc, 90 },
    { 0xee89e8cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a69e8d0, 88 },
    { 0xaf19e8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49e8d5, 95 },
    { 0x3a89e8d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a49e8da, 61 },
    { 0xf549e8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69e8de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc709e8e1, 52 },
    { 0x5c69e8e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9e8e5, 33 },
    { 0x7b9e8e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a9e8eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e29e8f5, 55 },
    { 0xd6c9e8f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9e8f9, 99 },
    { 0x9c89e8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249e8fd, 91 },
    { 0x4f89e8fe, 95 },
    { 0xfce9e8ff, 17 },
    { 0x65a9e8ff, 90 },
    { 0xc99e901, 76 },
    { 0xd769e901, 33 },
    { 0, 0 },
    { 0xc789e904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb199e907, 61 },
    { 0xe829e907, 100 },
    { 0x8d29e909, 98 },
    { 0, 0 },
    { 0xa919e90b, 61 },
    { 0xdd69e90b, 55 },
    { 0x1929e90c, 61 },
    { 0x98a9e90b, 95 },
    { 0x2d79e90f, 91 },
    { 0xe4a9e90c, 55 },
    { 0x8bb9e911, 76 },
    { 0xe419e912, 100 },
    { 0xbf99e912, 55 },
    { 0xc9a9e912, 90 },
    { 0, 0 },
    { 0xf2d9e916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1249e91d, 95 },
    { 0, 0 },
    { 0xdfc9e91f, 85 },
    { 0, 0 },
    { 0xe789e921, 98 },
    { 0, 0 },
    { 0x7669e923, 76 },
    { 0, 0 },
    { 0x1149e925, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b9e928, 43 },
    { 0, 0 },
    { 0x6959e92a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe949e92e, 23 },
    { 0xe699e92f, 33 },
    { 0x37e9e930, 33 },
    { 0x6e59e931, 34 },
    { 0xc479e931, 90 },
    { 0, 0 },
    { 0x33a9e934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9e937, 51 },
    { 0x1339e938, 76 },
    { 0x6ca9e937, 61 },
    { 0xd599e939, 98 },
    { 0x5eb9e938, 55 },
    { 0xb1d9e93c, 33 },
    { 0xc9e9e93d, 76 },
    { 0x5919e938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f09e94b, 23 },
    { 0xfd19e94b, 98 },
    { 0x5799e94b, 53 },
    { 0x4029e94e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd29e951, 98 },
    { 0x9a69e951, 33 },
    { 0x4c79e951, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d9e957, 76 },
    { 0x5249e958, 90 },
    { 0x3c59e958, 90 },
    { 0xce99e95a, 90 },
    { 0x80a9e95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9359e964, 93 },
    { 0xc0d9e964, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd9e96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd189e96f, 77 },
    { 0, 0 },
    { 0x4979e971, 90 },
    { 0x9999e972, 90 },
    { 0x2379e973, 91 },
    { 0x8d89e973, 83 },
    { 0x3679e975, 23 },
    { 0xcdb9e975, 33 },
    { 0, 0 },
    { 0x4359e978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319e97f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7999e983, 68 },
    { 0xf7e9e983, 33 },
    { 0xbf59e985, 61 },
    { 0x7c29e986, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd9e98e, 77 },
    { 0x1f29e98f, 95 },
    { 0xab19e98e, 90 },
    { 0xc3e9e98e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab19e99d, 93 },
    { 0x5729e99e, 68 },
    { 0x94b9e99e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa9e9a2, 55 },
    { 0, 0 },
    { 0x18c9e9a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2059e9aa, 61 },
    { 0x2849e9ab, 61 },
    { 0xf029e9ac, 84 },
    { 0, 0 },
    { 0x4289e9ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9e9b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499e9b9, 33 },
    { 0xcac9e9ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a9e9c2, 43 },
    { 0, 0 },
    { 0xd139e9c4, 76 },
    { 0, 0 },
    { 0x2f39e9c6, 23 },
    { 0x3439e9c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9e9cb, 91 },
    { 0x35a9e9cc, 55 },
    { 0x5e09e9cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169e9d3, 33 },
    { 0x2849e9d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f89e9da, 91 },
    { 0x6329e9da, 66 },
    { 0xe349e9dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e59e9e5, 98 },
    { 0x6839e9e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9e9ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7199e9f3, 61 },
    { 0x1799e9f4, 77 },
    { 0x2159e9f3, 95 },
    { 0x9769e9f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209e9fd, 98 },
    { 0x99b9e9fe, 66 },
    { 0x1ac9e9ff, 75 },
    { 0x1bf9e9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2459ea04, 65 },
    { 0x9d79ea05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc9ea08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829ea0b, 61 },
    { 0xaa49ea0c, 33 },
    { 0, 0 },
    { 0x9a89ea0e, 46 },
    { 0xc579ea0f, 36 },
    { 0xc749ea0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e39ea1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9ea1e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9ea22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf09ea25, 61 },
    { 0, 0 },
    { 0xe3c9ea27, 43 },
    { 0x1509ea28, 33 },
    { 0, 0 },
    { 0xd619ea2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf259ea2e, 85 },
    { 0x1c79ea2f, 46 },
    { 0xdc39ea30, 55 },
    { 0xf7f9ea2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c79ea36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce09ea3a, 61 },
    { 0xd9e9ea3a, 23 },
    { 0x18a9ea3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459ea3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729ea46, 81 },
    { 0x8149ea46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d59ea4b, 91 },
    { 0x39a9ea4c, 83 },
    { 0xe5f9ea4d, 95 },
    { 0x9019ea4e, 61 },
    { 0xe929ea4e, 90 },
    { 0xd459ea50, 87 },
    { 0xa9a9ea51, 23 },
    { 0x56f9ea52, 33 },
    { 0x2e79ea53, 91 },
    { 0x8d49ea53, 46 },
    { 0x6f89ea55, 61 },
    { 0x79d9ea56, 34 },
    { 0xb729ea52, 55 },
    { 0, 0 },
    { 0xa1f9ea59, 98 },
    { 0xd689ea5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b9ea61, 61 },
    { 0x5229ea61, 95 },
    { 0, 0 },
    { 0xd279ea64, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf9ea67, 90 },
    { 0x25b9ea68, 95 },
    { 0, 0 },
    { 0x64e9ea6a, 95 },
    { 0x44e9ea6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b9ea6f, 76 },
    { 0x7d79ea6f, 98 },
    { 0xa039ea71, 23 },
    { 0x5c69ea6f, 78 },
    { 0, 0 },
    { 0xb3e9ea74, 33 },
    { 0x4659ea75, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f79ea81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e9ea87, 100 },
    { 0xb789ea88, 90 },
    { 0, 0 },
    { 0x46a9ea8a, 55 },
    { 0x9829ea8b, 100 },
    { 0xd0b9ea8c, 76 },
    { 0xe669ea8c, 33 },
    { 0x7099ea8a, 23 },
    { 0x2689ea8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b9ea93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1159ea96, 90 },
    { 0xa159ea96, 76 },
    { 0x7a29ea98, 66 },
    { 0, 0 },
    { 0x72e9ea9a, 76 },
    { 0x7259ea9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa739eaa5, 93 },
    { 0, 0 },
    { 0xa9eaa7, 69 },
    { 0, 0 },
    { 0x4a9eaa9, 76 },
    { 0, 0 },
    { 0x8a99eaab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc769eab3, 99 },
    { 0, 0 },
    { 0x66e9eab5, 95 },
    { 0xd879eab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9eab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c9eabf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe039eac7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8629eacb, 95 },
    { 0x93f9eacc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd409ead3, 90 },
    { 0x35d9ead3, 98 },
    { 0x50a9ead5, 61 },
    { 0, 0 },
    { 0x7a89ead7, 90 },
    { 0, 0 },
    { 0xd229ead9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709eadd, 67 },
    { 0xe19eade, 40 },
    { 0x979eadf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe869eae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9eae7, 49 },
    { 0x78a9eae8, 23 },
    { 0x2d59eae9, 98 },
    { 0, 0 },
    { 0xa569eaeb, 33 },
    { 0xe3d9eaeb, 33 },
    { 0, 0 },
    { 0xb859eaee, 23 },
    { 0x1a9eaee, 98 },
    { 0xd759eaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5479eaf3, 61 },
    { 0x4869eaf3, 61 },
    { 0xbf9eaf4, 23 },
    { 0xdb49eaf6, 88 },
    { 0x9cd9eaf4, 33 },
    { 0, 0 },
    { 0x1d89eaf9, 52 },
    { 0x52c9eafa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9eb00, 89 },
    { 0, 0 },
    { 0x6d89eb02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd9eb05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa699eb08, 90 },
    { 0x2659eb09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4089eb0d, 91 },
    { 0x7a69eb0d, 90 },
    { 0xb7b9eb0f, 98 },
    { 0x6469eb0e, 90 },
    { 0x4cd9eb11, 98 },
    { 0xca59eb12, 43 },
    { 0x21b9eb12, 91 },
    { 0x66f9eb14, 91 },
    { 0x3f79eb15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3919eb1b, 98 },
    { 0xd6b9eb1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c59eb1f, 77 },
    { 0x9a49eb20, 93 },
    { 0x9519eb20, 23 },
    { 0xb7b9eb22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109eb28, 90 },
    { 0xdbe9eb29, 90 },
    { 0xe519eb29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d9eb30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909eb35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b59eb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e9eb3c, 62 },
    { 0xaa49eb3c, 66 },
    { 0x1b29eb3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89eb42, 86 },
    { 0x2d29eb43, 43 },
    { 0x29e9eb43, 95 },
    { 0xad9eb45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d69eb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559eb4f, 61 },
    { 0xaeb9eb4f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9eb57, 85 },
    { 0xab89eb58, 34 },
    { 0, 0 },
    { 0x38a9eb5a, 23 },
    { 0, 0 },
    { 0x75e9eb5c, 90 },
    { 0x84a9eb5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f9eb62, 17 },
    { 0x1039eb63, 95 },
    { 0, 0 },
    { 0xcdd9eb65, 90 },
    { 0xd349eb65, 98 },
    { 0x9329eb66, 61 },
    { 0xad59eb67, 61 },
    { 0, 0 },
    { 0x9e49eb6a, 33 },
    { 0xd089eb6b, 63 },
    { 0x5a09eb6c, 88 },
    { 0x8c29eb6d, 88 },
    { 0x3249eb6a, 34 },
    { 0x7bd9eb6f, 99 },
    { 0x4129eb70, 52 },
    { 0, 0 },
    { 0x4b69eb72, 98 },
    { 0x4da9eb72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559eb77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e9eb7d, 43 },
    { 0xe3b9eb7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf19eb81, 90 },
    { 0xbcb9eb82, 98 },
    { 0x73f9eb82, 90 },
    { 0x59a9eb84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7189eb87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc9eb8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a9eb91, 99 },
    { 0xee9eb91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc679eb98, 78 },
    { 0, 0 },
    { 0x35a9eb9a, 36 },
    { 0xc979eb9b, 34 },
    { 0x61c9eb9c, 98 },
    { 0x7be9eb9d, 61 },
    { 0, 0 },
    { 0x169eb9f, 87 },
    { 0xa099eb9f, 99 },
    { 0xe189eba0, 76 },
    { 0xdbc9eba2, 34 },
    { 0xd139eba3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9ebb0, 95 },
    { 0, 0 },
    { 0x5829ebb2, 33 },
    { 0xd989ebb2, 98 },
    { 0xc419ebb2, 95 },
    { 0, 0 },
    { 0x6219ebb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da9ebb9, 85 },
    { 0, 0 },
    { 0x7d9ebbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb689ebbf, 90 },
    { 0xfb09ebc0, 98 },
    { 0xfcb9ebc1, 55 },
    { 0x3c79ebc0, 32 },
    { 0xb5b9ebc3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ebc8, 90 },
    { 0x8a19ebc9, 52 },
    { 0xc329ebc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59ebcd, 33 },
    { 0xfad9ebce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf989ebd3, 66 },
    { 0x14f9ebd4, 90 },
    { 0xb4c9ebd4, 61 },
    { 0xe879ebd6, 87 },
    { 0xb389ebd5, 85 },
    { 0x1d49ebd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269ebdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449ebe1, 33 },
    { 0, 0 },
    { 0x21c9ebe3, 78 },
    { 0, 0 },
    { 0x8e79ebe5, 87 },
    { 0xe579ebe5, 66 },
    { 0x30e9ebe7, 85 },
    { 0xf239ebe7, 95 },
    { 0x1bb9ebe8, 98 },
    { 0xc529ebea, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9ebee, 33 },
    { 0x3669ebee, 61 },
    { 0xfbd9ebee, 98 },
    { 0x3709ebf1, 61 },
    { 0x6f69ebf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9559ebf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459ebfa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9ebfd, 61 },
    { 0x4109ebfe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9ec05, 33 },
    { 0x9ca9ec06, 55 },
    { 0, 0 },
    { 0x2449ec08, 40 },
    { 0xd889ec09, 23 },
    { 0, 0 },
    { 0xda49ec0b, 33 },
    { 0x7679ec0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439ec11, 43 },
    { 0x57d9ec12, 33 },
    { 0, 0 },
    { 0xf669ec14, 90 },
    { 0x72d9ec14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379ec19, 23 },
    { 0, 0 },
    { 0x7899ec1b, 77 },
    { 0x72a9ec1c, 99 },
    { 0xb9e9ec1d, 95 },
    { 0x16b9ec1e, 23 },
    { 0xf879ec1f, 100 },
    { 0x3a79ec20, 66 },
    { 0x2919ec21, 66 },
    { 0xf819ec22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb9ec29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9ec2d, 33 },
    { 0x8e89ec2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ec31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9249ec36, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc49ec3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad69ec41, 90 },
    { 0, 0 },
    { 0xc3c9ec43, 23 },
    { 0x5999ec43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5989ec48, 98 },
    { 0xff09ec49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec4d, 91 },
    { 0x7f89ec4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b9ec51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f9ec59, 100 },
    { 0, 0 },
    { 0xa289ec5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a9ec60, 34 },
    { 0xe749ec61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209ec66, 99 },
    { 0x8ed9ec66, 23 },
    { 0xe699ec66, 33 },
    { 0xea19ec68, 23 },
    { 0, 0 },
    { 0x9c39ec6b, 69 },
    { 0xa49ec6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc19ec6f, 33 },
    { 0, 0 },
    { 0x9dd9ec71, 33 },
    { 0, 0 },
    { 0x14e9ec73, 91 },
    { 0xbfe9ec74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d69ec77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29ec7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf49ec7e, 80 },
    { 0xa119ec7e, 99 },
    { 0x9309ec7e, 95 },
    { 0xcab9ec7f, 61 },
    { 0xf389ec80, 61 },
    { 0x9f79ec82, 67 },
    { 0x4b49ec84, 33 },
    { 0xca49ec85, 33 },
    { 0, 0 },
    { 0x339ec87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9959ec8c, 55 },
    { 0x269ec8d, 90 },
    { 0xc2d9ec8d, 17 },
    { 0x3659ec8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe149ec92, 43 },
    { 0xbce9ec93, 100 },
    { 0xd299ec94, 100 },
    { 0, 0 },
    { 0x62c9ec96, 33 },
    { 0xb9d9ec96, 34 },
    { 0, 0 },
    { 0xcf99ec99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb119ec9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259ec9f, 46 },
    { 0, 0 },
    { 0x6939eca1, 100 },
    { 0, 0 },
    { 0xe189eca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b9eca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1929eca9, 80 },
    { 0x4099eca9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b29ecae, 61 },
    { 0, 0 },
    { 0x1c09ecb0, 46 },
    { 0xebf9ecb1, 76 },
    { 0, 0 },
    { 0x2369ecb3, 93 },
    { 0x1159ecb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d69ecbb, 23 },
    { 0xd629ecbc, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669ecbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b09ecc2, 90 },
    { 0xf589ecc3, 81 },
    { 0xc499ecc3, 46 },
    { 0xc539ecc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5099ecca, 95 },
    { 0, 0 },
    { 0xb269eccc, 98 },
    { 0x4319eccd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e9ecd5, 75 },
    { 0xbda9ecd6, 95 },
    { 0, 0 },
    { 0x6b29ecd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b99ece0, 65 },
    { 0, 0 },
    { 0x8609ece2, 61 },
    { 0, 0 },
    { 0x2c09ece4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e9eceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6929ecee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9369ecf4, 76 },
    { 0x2fc9ecf5, 59 },
    { 0x3c59ecf6, 76 },
    { 0x4d89ecf6, 61 },
    { 0xc309ecf8, 64 },
    { 0x45f9ecf4, 95 },
    { 0x50e9ecf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd49ed0d, 98 },
    { 0xa239ed0e, 33 },
    { 0xb2c9ed0f, 68 },
    { 0x2c29ed0f, 91 },
    { 0x4a9ed0e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6519ed1a, 75 },
    { 0x2639ed1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db9ed1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc09ed22, 95 },
    { 0xf809ed23, 84 },
    { 0, 0 },
    { 0xca49ed25, 98 },
    { 0xdd9ed25, 90 },
    { 0x3089ed27, 66 },
    { 0x27e9ed25, 55 },
    { 0x34e9ed29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6099ed2c, 95 },
    { 0xeaf9ed2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f9ed31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059ed34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef9ed37, 66 },
    { 0x3d09ed37, 34 },
    { 0, 0 },
    { 0xe3f9ed3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc9ed40, 90 },
    { 0, 0 },
    { 0xf429ed42, 99 },
    { 0xf59ed43, 46 },
    { 0x349ed44, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d9ed4d, 52 },
    { 0x9569ed4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ed53, 61 },
    { 0x6ef9ed54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec09ed57, 34 },
    { 0x2b69ed58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9929ed5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7349ed5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2139ed63, 96 },
    { 0, 0 },
    { 0xf5b9ed65, 23 },
    { 0x8ba9ed66, 55 },
    { 0x7929ed67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c9ed6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d9ed6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa719ed73, 91 },
    { 0xd1d9ed74, 55 },
    { 0x9f59ed75, 33 },
    { 0, 0 },
    { 0xa489ed77, 90 },
    { 0, 0 },
    { 0x7ab9ed79, 55 },
    { 0x7d29ed7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439ed7e, 91 },
    { 0, 0 },
    { 0xb679ed80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb9ed85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5709ed88, 90 },
    { 0x6999ed89, 81 },
    { 0xf799ed8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4479ed90, 87 },
    { 0x5699ed91, 93 },
    { 0xd229ed92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89ed99, 90 },
    { 0xe119ed9a, 91 },
    { 0, 0 },
    { 0x3579ed9c, 52 },
    { 0x2d9ed9d, 61 },
    { 0x1a39ed9e, 23 },
    { 0xb759ed9f, 34 },
    { 0xb719eda0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a29eda9, 78 },
    { 0, 0 },
    { 0xcd39edab, 33 },
    { 0x89a9edac, 90 },
    { 0xee29edad, 34 },
    { 0xdfe9edad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc309edb1, 61 },
    { 0, 0 },
    { 0x5a99edb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9edc6, 33 },
    { 0xc499edc6, 24 },
    { 0x6969edc8, 78 },
    { 0xeb59edc8, 90 },
    { 0x5ce9edc9, 34 },
    { 0xf1c9edcb, 98 },
    { 0xe7e9edcb, 98 },
    { 0xaae9edcb, 76 },
    { 0x3d79edc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9edd2, 81 },
    { 0, 0 },
    { 0xbe59edd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8529edd9, 73 },
    { 0x3b59edd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9ede0, 34 },
    { 0x2219ede0, 51 },
    { 0, 0 },
    { 0x5e49ede3, 76 },
    { 0xd769ede4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1509eded, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe29edf4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3489edf8, 61 },
    { 0x35e9edf9, 61 },
    { 0x5179edfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779ee05, 13 },
    { 0xf689ee06, 33 },
    { 0x3599ee05, 80 },
    { 0x10f9ee08, 34 },
    { 0xef09ee09, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7389ee0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c9ee11, 95 },
    { 0x8ca9ee11, 23 },
    { 0x9ac9ee13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c59ee16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda69ee1a, 98 },
    { 0xd139ee1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea9ee1e, 23 },
    { 0x2af9ee1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb9ee25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939ee2b, 34 },
    { 0x9f09ee2c, 98 },
    { 0x3d59ee2d, 33 },
    { 0xc689ee2b, 34 },
    { 0x6e29ee2d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ee32, 33 },
    { 0, 0 },
    { 0xa699ee34, 90 },
    { 0xb5d9ee35, 23 },
    { 0xea69ee34, 98 },
    { 0, 0 },
    { 0x6119ee38, 98 },
    { 0xe6e9ee39, 55 },
    { 0x4f89ee39, 61 },
    { 0x1f49ee3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4489ee3e, 55 },
    { 0xc389ee3f, 81 },
    { 0x17b9ee40, 81 },
    { 0x76e9ee40, 43 },
    { 0x8b29ee3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19ee4a, 23 },
    { 0x1509ee4b, 75 },
    { 0xd059ee4c, 61 },
    { 0xe8b9ee4d, 90 },
    { 0x2f9ee4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee89ee55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd59ee58, 23 },
    { 0xda79ee59, 34 },
    { 0xd6e9ee5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9ee63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779ee6c, 88 },
    { 0, 0 },
    { 0x7c9ee6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf999ee73, 98 },
    { 0, 0 },
    { 0xca69ee75, 17 },
    { 0, 0 },
    { 0xae49ee77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6539ee7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7909ee80, 83 },
    { 0xb429ee80, 98 },
    { 0xb79ee81, 76 },
    { 0x1e49ee80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59ee8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c9ee8f, 76 },
    { 0xc279ee8f, 65 },
    { 0x2df9ee8f, 33 },
    { 0xea29ee92, 33 },
    { 0xbf79ee8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9ee96, 93 },
    { 0xc509ee97, 52 },
    { 0, 0 },
    { 0x7d19ee99, 85 },
    { 0x6679ee99, 90 },
    { 0xc989ee9b, 33 },
    { 0xe29ee9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea9ee9f, 43 },
    { 0xe19eea0, 98 },
    { 0x8129ee9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c59eea4, 95 },
    { 0, 0 },
    { 0x36c9eea6, 61 },
    { 0, 0 },
    { 0x2149eea8, 46 },
    { 0x2729eea8, 34 },
    { 0, 0 },
    { 0xd8e9eeab, 100 },
    { 0xa479eeab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c49eeb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe329eeb3, 91 },
    { 0, 0 },
    { 0xf979eeb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9eeb8, 90 },
    { 0x2dd9eeb9, 36 },
    { 0x50c9eeb9, 33 },
    { 0xae9eebb, 98 },
    { 0x6ed9eebb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b19eebf, 23 },
    { 0xd2c9eec0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda9eec4, 90 },
    { 0, 0 },
    { 0x3329eec6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c19eec9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb69eecc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f9eecf, 90 },
    { 0x2cd9eed0, 22 },
    { 0xf5c9eed1, 91 },
    { 0x66f9eed2, 81 },
    { 0, 0 },
    { 0xdaf9eed4, 33 },
    { 0xdc39eed5, 33 },
    { 0x749eed6, 35 },
    { 0x3059eed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979eee6, 76 },
    { 0, 0 },
    { 0xf529eee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac9eeed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc9eef1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c9eef6, 83 },
    { 0xb059eef7, 33 },
    { 0x50d9eef7, 23 },
    { 0x87e9eef8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369eefe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939ef01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ef09, 46 },
    { 0, 0 },
    { 0xa3c9ef0b, 78 },
    { 0, 0 },
    { 0xb1e9ef0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c9ef10, 23 },
    { 0x2f19ef11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709ef14, 33 },
    { 0xa0a9ef15, 33 },
    { 0xc389ef16, 67 },
    { 0xdbc9ef16, 95 },
    { 0xc289ef14, 33 },
    { 0x60f9ef18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f39ef1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3529ef22, 55 },
    { 0, 0 },
    { 0xe3f9ef24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa539ef27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed9ef2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829ef2e, 81 },
    { 0x6219ef2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe59ef37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299ef3e, 87 },
    { 0, 0 },
    { 0x3989ef40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd629ef44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6659ef47, 34 },
    { 0x15c9ef48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419ef51, 90 },
    { 0x7ac9ef52, 61 },
    { 0, 0 },
    { 0x81c9ef54, 99 },
    { 0xffb9ef55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9319ef61, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc9ef64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf89ef68, 23 },
    { 0xfc69ef69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9ef6c, 23 },
    { 0xe7d9ef6d, 46 },
    { 0, 0 },
    { 0xfe89ef6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d9ef72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9ef76, 34 },
    { 0x49ef77, 95 },
    { 0xe569ef78, 90 },
    { 0, 0 },
    { 0x3219ef7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199ef82, 84 },
    { 0, 0 },
    { 0xdde9ef84, 95 },
    { 0, 0 },
    { 0x14e9ef86, 99 },
    { 0, 0 },
    { 0x5049ef88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889ef8f, 55 },
    { 0x9839ef8f, 43 },
    { 0x7909ef91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7589ef94, 76 },
    { 0x65e9ef95, 55 },
    { 0xd949ef95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe669efa0, 95 },
    { 0x2409efa1, 40 },
    { 0x489efa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e9efae, 33 },
    { 0xeb09efae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7669efb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe029efb5, 87 },
    { 0, 0 },
    { 0x2499efb7, 91 },
    { 0x5379efb8, 85 },
    { 0x8119efb9, 77 },
    { 0xa1d9efb9, 33 },
    { 0xe849efba, 95 },
    { 0xd0b9efbb, 33 },
    { 0, 0 },
    { 0x27e9efbe, 76 },
    { 0xb069efbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf9efc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b9efc6, 55 },
    { 0x1689efc6, 23 },
    { 0xb779efc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa9efd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9efd4, 33 },
    { 0x65d9efd5, 78 },
    { 0x9b89efd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a49efdc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c9efe7, 98 },
    { 0, 0 },
    { 0x9b89efe9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1149efec, 87 },
    { 0x8449efed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1469eff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a9effb, 98 },
    { 0x4f39effc, 61 },
    { 0xc769effc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8369f000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2769f006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf459f00a, 33 },
    { 0xd899f00a, 23 },
    { 0, 0 },
    { 0x93a9f00d, 55 },
    { 0, 0 },
    { 0x3bb9f00f, 98 },
    { 0x5f49f010, 98 },
    { 0x1ed9f011, 33 },
    { 0, 0 },
    { 0x8fc9f013, 43 },
    { 0, 0 },
    { 0x98e9f015, 98 },
    { 0x46d9f015, 76 },
    { 0xddf9f015, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9939f023, 46 },
    { 0, 0 },
    { 0x5f9f025, 43 },
    { 0, 0 },
    { 0x85a9f027, 33 },
    { 0x55e9f028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c59f02c, 38 },
    { 0x3bb9f02c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349f030, 99 },
    { 0, 0 },
    { 0x8c09f032, 23 },
    { 0, 0 },
    { 0xaa39f034, 23 },
    { 0, 0 },
    { 0x2599f036, 76 },
    { 0x6799f037, 90 },
    { 0x55c9f038, 98 },
    { 0x8a9f039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc479f041, 61 },
    { 0x5bf9f042, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2139f045, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc9f04e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e9f051, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79f054, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9f058, 99 },
    { 0, 0 },
    { 0x3ad9f05a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f9f05d, 61 },
    { 0xa7e9f05e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3119f065, 93 },
    { 0x5ac9f066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac9f06d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339f070, 61 },
    { 0, 0 },
    { 0xdb79f072, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e29f078, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09f07b, 87 },
    { 0x5499f07c, 23 },
    { 0xa989f07d, 15 },
    { 0, 0 },
    { 0x4f69f07f, 61 },
    { 0xf119f080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf519f084, 90 },
    { 0xfc09f084, 55 },
    { 0x61c9f086, 61 },
    { 0x76a9f087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9929f08e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf099f091, 90 },
    { 0x17a9f092, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd159f096, 23 },
    { 0xf389f097, 61 },
    { 0x9a29f098, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6889f09b, 90 },
    { 0x84e9f09c, 95 },
    { 0x269f09b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9f0a2, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf89f0a5, 90 },
    { 0xe5d9f0a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f9f0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a9f0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99f0b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc039f0b7, 55 },
    { 0, 0 },
    { 0x5b9f0b9, 80 },
    { 0xb249f0ba, 43 },
    { 0x43a9f0b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f0c4, 81 },
    { 0x2819f0c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f09f0c9, 99 },
    { 0x7029f0c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b9f0d1, 90 },
    { 0x8e19f0d1, 43 },
    { 0x50a9f0d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd9f0d8, 55 },
    { 0x4729f0d8, 93 },
    { 0xefe9f0da, 66 },
    { 0xf219f0db, 33 },
    { 0xb829f0db, 95 },
    { 0xdaa9f0db, 33 },
    { 0x4ab9f0de, 55 },
    { 0xbe19f0de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869f0e3, 93 },
    { 0x4319f0e4, 34 },
    { 0, 0 },
    { 0x8ba9f0e6, 33 },
    { 0x20c9f0e7, 33 },
    { 0x1a99f0e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba49f0f0, 95 },
    { 0, 0 },
    { 0x9329f0f2, 35 },
    { 0, 0 },
    { 0xceb9f0f4, 98 },
    { 0x8199f0f5, 34 },
    { 0xe39f0f5, 23 },
    { 0xacd9f0f7, 23 },
    { 0x4e9f0f4, 23 },
    { 0x1d89f0f9, 87 },
    { 0x7119f0fa, 98 },
    { 0x88b9f0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9f0ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6259f104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb89f107, 67 },
    { 0x85b9f107, 76 },
    { 0x96a9f109, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b9f10d, 95 },
    { 0x6da9f10e, 33 },
    { 0x829f10f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc319f113, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce9f118, 51 },
    { 0x42d9f119, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19f11d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f99f122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f127, 87 },
    { 0xf8e9f128, 23 },
    { 0x80b9f128, 90 },
    { 0xf6d9f129, 98 },
    { 0xf39f12a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1999f12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9f134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419f13a, 91 },
    { 0xa1e9f13b, 76 },
    { 0, 0 },
    { 0x8229f13d, 100 },
    { 0x59a9f13d, 87 },
    { 0xdf19f13e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9f143, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db9f14a, 95 },
    { 0xe009f14b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2819f14e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6219f153, 55 },
    { 0x2909f153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759f159, 46 },
    { 0x1459f15a, 90 },
    { 0xfa9f15a, 23 },
    { 0x5819f15c, 55 },
    { 0x4569f15b, 77 },
    { 0, 0 },
    { 0xe879f15f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea9f163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca9f168, 90 },
    { 0x8019f169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f9f16d, 34 },
    { 0, 0 },
    { 0xf49f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa379f172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9f176, 95 },
    { 0, 0 },
    { 0x58e9f178, 33 },
    { 0x8759f179, 98 },
    { 0, 0 },
    { 0x7019f17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa229f17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6089f183, 53 },
    { 0xc629f183, 85 },
    { 0x8589f184, 42 },
    { 0xfad9f185, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c9f18a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029f18e, 55 },
    { 0x90c9f18f, 88 },
    { 0xa099f18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff49f196, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c9f199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3859f19c, 91 },
    { 0xf729f19c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759f1a4, 33 },
    { 0x74e9f1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe149f1a8, 43 },
    { 0xcca9f1a9, 91 },
    { 0x82f9f1a8, 34 },
    { 0, 0 },
    { 0x5d19f1ac, 55 },
    { 0xfd29f1ad, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1319f1b7, 95 },
    { 0xa889f1b8, 90 },
    { 0x2c69f1b9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469f1bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e79f1c5, 98 },
    { 0x2439f1c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099f1cd, 17 },
    { 0x6069f1ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc9f1d4, 43 },
    { 0xbe29f1d4, 90 },
    { 0, 0 },
    { 0x41b9f1d7, 98 },
    { 0xb609f1d8, 66 },
    { 0x4ea9f1d8, 76 },
    { 0xd2f9f1d7, 46 },
    { 0, 0 },
    { 0xdc69f1dc, 99 },
    { 0, 0 },
    { 0xd669f1de, 95 },
    { 0, 0 },
    { 0xa19f1e0, 33 },
    { 0x1439f1e1, 90 },
    { 0x86b9f1e2, 33 },
    { 0, 0 },
    { 0xb529f1e4, 55 },
    { 0x4be9f1e5, 58 },
    { 0, 0 },
    { 0xe059f1e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b9f1ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c39f1f2, 46 },
    { 0xf399f1f3, 43 },
    { 0x6639f1f4, 90 },
    { 0xa509f1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f9f1f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9f1fc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe199f200, 98 },
    { 0x3359f201, 55 },
    { 0xfcd9f202, 98 },
    { 0x70e9f203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9f206, 85 },
    { 0xa839f207, 98 },
    { 0, 0 },
    { 0x1749f209, 95 },
    { 0, 0 },
    { 0xc829f20b, 99 },
    { 0xfb19f20c, 81 },
    { 0x9899f20d, 98 },
    { 0xc379f20e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b9f212, 43 },
    { 0x1009f213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39f21d, 61 },
    { 0, 0 },
    { 0xd0b9f21f, 69 },
    { 0xcb19f21f, 61 },
    { 0x9419f21f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69f224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea39f237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f23a, 81 },
    { 0, 0 },
    { 0x67d9f23c, 33 },
    { 0xce29f23d, 33 },
    { 0, 0 },
    { 0x51a9f23f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9f244, 65 },
    { 0, 0 },
    { 0x629f246, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f9f24d, 61 },
    { 0, 0 },
    { 0x12a9f24f, 33 },
    { 0, 0 },
    { 0x9ac9f251, 95 },
    { 0x4ad9f252, 85 },
    { 0, 0 },
    { 0xffa9f254, 81 },
    { 0, 0 },
    { 0xc129f256, 23 },
    { 0x9df9f257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb869f25c, 34 },
    { 0x87c9f25c, 76 },
    { 0x34a9f25d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9f261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a9f268, 93 },
    { 0xe89f269, 33 },
    { 0xc739f26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc469f270, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b9f273, 46 },
    { 0, 0 },
    { 0x17a9f275, 93 },
    { 0, 0 },
    { 0x3a49f277, 33 },
    { 0xc159f278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709f27b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809f27e, 76 },
    { 0xf2f9f27f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9999f288, 90 },
    { 0x26b9f289, 46 },
    { 0, 0 },
    { 0xdb29f28b, 17 },
    { 0x2969f28c, 90 },
    { 0x9119f28b, 76 },
    { 0x4c99f28d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf39f295, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a09f298, 90 },
    { 0x56b9f299, 23 },
    { 0x44a9f298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9f29d, 76 },
    { 0, 0 },
    { 0x84f9f29f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c9f2a6, 23 },
    { 0, 0 },
    { 0xac59f2a8, 85 },
    { 0, 0 },
    { 0xbd49f2aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a69f2ae, 61 },
    { 0, 0 },
    { 0x1e49f2b0, 90 },
    { 0, 0 },
    { 0x4819f2b2, 98 },
    { 0xcf09f2b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b9f2bd, 90 },
    { 0xfa59f2bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9f2c3, 95 },
    { 0x3699f2c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee79f2ca, 90 },
    { 0x8f69f2ca, 43 },
    { 0xb8c9f2cc, 33 },
    { 0xee19f2cd, 17 },
    { 0x8989f2cd, 64 },
    { 0x3e89f2cd, 43 },
    { 0, 0 },
    { 0x6df9f2d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc9f2d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1259f2e3, 61 },
    { 0xf729f2e3, 98 },
    { 0x6819f2e4, 33 },
    { 0x69e9f2e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469f2e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8499f2ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f2f1, 52 },
    { 0x1399f2f1, 98 },
    { 0x8399f2f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c9f2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef9f2fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d9f2fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f9f301, 43 },
    { 0x8399f302, 95 },
    { 0xfb69f302, 76 },
    { 0, 0 },
    { 0x4029f305, 33 },
    { 0x6809f306, 98 },
    { 0xc159f306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e89f30c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419f310, 88 },
    { 0x6a59f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7659f316, 23 },
    { 0x9fc9f317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789f324, 55 },
    { 0x29a9f325, 23 },
    { 0xc5a9f326, 33 },
    { 0x4459f326, 34 },
    { 0x6d49f328, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b9f32f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f9f335, 43 },
    { 0, 0 },
    { 0x4309f337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x249f33a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe109f33e, 52 },
    { 0x31b9f33e, 61 },
    { 0xcc79f33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe229f344, 46 },
    { 0x8659f345, 94 },
    { 0xa379f346, 80 },
    { 0x8ae9f346, 69 },
    { 0x289f347, 48 },
    { 0x6ae9f349, 96 },
    { 0xa969f348, 61 },
    { 0x7b29f344, 31 },
    { 0xc759f34c, 61 },
    { 0x7069f34d, 90 },
    { 0, 0 },
    { 0x11a9f34f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf789f359, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e59f35c, 33 },
    { 0x1219f35c, 33 },
    { 0x9439f35d, 95 },
    { 0x8aa9f35d, 81 },
    { 0x10a9f360, 61 },
    { 0, 0 },
    { 0xfad9f362, 90 },
    { 0x3179f362, 34 },
    { 0x2499f364, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f367, 98 },
    { 0x19c9f367, 33 },
    { 0x599f368, 33 },
    { 0x91d9f36a, 34 },
    { 0xb8a9f36a, 98 },
    { 0xe409f36c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b9f371, 55 },
    { 0x8959f372, 43 },
    { 0xd4d9f372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc999f378, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89f37e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae9f381, 94 },
    { 0, 0 },
    { 0x9e69f383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb359f386, 90 },
    { 0xbbf9f387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe949f38d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469f392, 33 },
    { 0, 0 },
    { 0x5f59f394, 67 },
    { 0xc729f394, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5779f398, 91 },
    { 0x3ad9f399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079f39c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f9f3a2, 64 },
    { 0, 0 },
    { 0x1d59f3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e89f3a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1869f3b0, 80 },
    { 0xf269f3b1, 33 },
    { 0xed49f3b1, 61 },
    { 0x5489f3b1, 38 },
    { 0, 0 },
    { 0x6549f3b5, 90 },
    { 0, 0 },
    { 0x7799f3b7, 96 },
    { 0x6719f3b8, 98 },
    { 0x62f9f3b9, 98 },
    { 0x2509f3ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9f3bf, 46 },
    { 0xac9f3c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2009f3c3, 61 },
    { 0x9e89f3c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb109f3c9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59f3d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5559f3d3, 33 },
    { 0x4dd9f3d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9f3d7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc9f3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb179f3de, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e9f3e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6189f3e5, 98 },
    { 0xfd09f3e5, 78 },
    { 0xe779f3e7, 95 },
    { 0xf8a9f3e7, 98 },
    { 0x9c79f3e9, 84 },
    { 0x5249f3e8, 46 },
    { 0x29d9f3eb, 55 },
    { 0x55f9f3e7, 23 },
    { 0, 0 },
    { 0x2bc9f3ee, 100 },
    { 0, 0 },
    { 0x4f29f3f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e09f3f3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9f3f8, 34 },
    { 0x7029f3f9, 43 },
    { 0xc499f3fa, 95 },
    { 0x8ba9f3fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439f405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb159f40a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b69f40f, 95 },
    { 0, 0 },
    { 0x139f411, 33 },
    { 0x2749f411, 34 },
    { 0xc0c9f413, 55 },
    { 0, 0 },
    { 0x3669f415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f418, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8569f41b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169f420, 91 },
    { 0x2cc9f420, 76 },
    { 0xc7d9f422, 98 },
    { 0x7aa9f423, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829f432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab9f435, 61 },
    { 0, 0 },
    { 0x7329f437, 85 },
    { 0xe2a9f438, 90 },
    { 0x7fb9f439, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c9f441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9f44a, 76 },
    { 0, 0 },
    { 0x9bf9f44c, 90 },
    { 0, 0 },
    { 0x7c69f44e, 81 },
    { 0xdca9f44f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f9f453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129f45a, 98 },
    { 0x3f99f45b, 46 },
    { 0x82b9f45c, 65 },
    { 0xae19f45a, 91 },
    { 0, 0 },
    { 0x26d9f45f, 85 },
    { 0, 0 },
    { 0xc0b9f461, 78 },
    { 0, 0 },
    { 0x9979f463, 67 },
    { 0x7059f464, 33 },
    { 0x8709f464, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e09f468, 91 },
    { 0x48b9f469, 88 },
    { 0xef79f46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c9f46e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59f479, 88 },
    { 0, 0 },
    { 0x1a39f47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b9f47e, 23 },
    { 0xf19f47f, 23 },
    { 0xcfa9f480, 23 },
    { 0, 0 },
    { 0x40c9f482, 90 },
    { 0x9e29f482, 64 },
    { 0xa469f483, 17 },
    { 0xa6e9f485, 67 },
    { 0xd749f486, 33 },
    { 0xfb59f487, 55 },
    { 0x88a9f488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d9f490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7499f493, 61 },
    { 0xa1c9f494, 23 },
    { 0x8c09f495, 98 },
    { 0x169f496, 61 },
    { 0x9309f497, 69 },
    { 0xe0d9f497, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce9f49b, 68 },
    { 0xaa59f49b, 61 },
    { 0x7c09f49b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb759f4a4, 93 },
    { 0x1bf9f4a4, 98 },
    { 0, 0 },
    { 0xe1b9f4a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b9f4b1, 81 },
    { 0x1d89f4b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af9f4b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9f4b8, 90 },
    { 0xc8a9f4b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f9f4bd, 91 },
    { 0x9579f4bd, 55 },
    { 0, 0 },
    { 0x749f4c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789f4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89f4c7, 78 },
    { 0, 0 },
    { 0x2409f4c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9f4cc, 33 },
    { 0x2d79f4cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf89f4d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea9f4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa939f4d8, 91 },
    { 0xbae9f4d9, 90 },
    { 0x5d69f4da, 78 },
    { 0, 0 },
    { 0x7ee9f4dc, 46 },
    { 0x2a09f4dc, 46 },
    { 0xfcd9f4dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9f4e8, 17 },
    { 0xa8c9f4e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389f4ec, 76 },
    { 0x3a29f4ed, 95 },
    { 0xb569f4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda79f4f1, 33 },
    { 0xe59f4f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa139f4f9, 52 },
    { 0x3f9f4f9, 76 },
    { 0xe7f9f4fb, 90 },
    { 0x93d9f4fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c9f502, 95 },
    { 0x209f503, 60 },
    { 0x77b9f504, 100 },
    { 0, 0 },
    { 0xf689f506, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a59f50d, 93 },
    { 0, 0 },
    { 0x58a9f50f, 98 },
    { 0xf3b9f50f, 100 },
    { 0x8539f50f, 43 },
    { 0x94c9f510, 95 },
    { 0xdc89f510, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559f517, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c9f51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c9f522, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9f530, 76 },
    { 0xa049f531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe709f538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f53b, 66 },
    { 0x4609f53c, 90 },
    { 0x8339f53d, 23 },
    { 0, 0 },
    { 0xf1c9f53f, 66 },
    { 0xd9b9f540, 68 },
    { 0xf459f53f, 55 },
    { 0, 0 },
    { 0xd19f543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9739f54a, 90 },
    { 0, 0 },
    { 0x6b29f54c, 100 },
    { 0x19a9f54c, 43 },
    { 0xafb9f54e, 78 },
    { 0xf1b9f54c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c9f556, 40 },
    { 0xe9f557, 23 },
    { 0x6de9f557, 98 },
    { 0, 0 },
    { 0xe4f9f55a, 33 },
    { 0, 0 },
    { 0x9609f55c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9f560, 33 },
    { 0xe849f561, 61 },
    { 0, 0 },
    { 0x3019f563, 95 },
    { 0x1be9f563, 17 },
    { 0x90c9f565, 61 },
    { 0xdec9f566, 23 },
    { 0x7b19f566, 61 },
    { 0x20c9f566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed9f56d, 91 },
    { 0, 0 },
    { 0xc749f56f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa89f573, 95 },
    { 0x7ac9f573, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299f577, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a9f57c, 98 },
    { 0x2759f57d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb9f582, 66 },
    { 0xf999f583, 76 },
    { 0x6589f583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99f58b, 95 },
    { 0xc5e9f58b, 90 },
    { 0x4e89f58d, 34 },
    { 0xc5d9f58e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f594, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda9f59d, 23 },
    { 0x1809f59d, 61 },
    { 0xc3c9f59f, 33 },
    { 0x1b59f5a0, 98 },
    { 0xdad9f5a0, 25 },
    { 0x9059f59d, 33 },
    { 0, 0 },
    { 0xfba9f5a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea9f5af, 23 },
    { 0xbaa9f5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8159f5b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce9f5ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d29f5bd, 99 },
    { 0, 0 },
    { 0x8a79f5bf, 75 },
    { 0xc7f9f5c0, 76 },
    { 0, 0 },
    { 0xcac9f5c2, 90 },
    { 0x2e59f5c2, 98 },
    { 0x12d9f5c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7549f5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9699f5d6, 55 },
    { 0x4399f5d7, 34 },
    { 0x2da9f5d6, 43 },
    { 0x83c9f5d9, 23 },
    { 0x2249f5d9, 33 },
    { 0x5259f5db, 85 },
    { 0xa359f5dc, 34 },
    { 0x1849f5d8, 56 },
    { 0x1ea9f5de, 87 },
    { 0xed9f5db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129f5e3, 99 },
    { 0x2b9f5e4, 95 },
    { 0, 0 },
    { 0xee99f5e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed49f5ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e9f5ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e29f5f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c39f5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9209f5f7, 91 },
    { 0x5c99f5f7, 90 },
    { 0xa679f5f8, 33 },
    { 0xf0b9f5f8, 100 },
    { 0x4029f5f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea29f600, 43 },
    { 0x1d29f600, 77 },
    { 0, 0 },
    { 0xbe49f603, 52 },
    { 0xc3e9f604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde9f607, 61 },
    { 0, 0 },
    { 0x6a49f609, 92 },
    { 0xca69f609, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x359f60d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d69f615, 75 },
    { 0x149f615, 67 },
    { 0x29d9f616, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9f625, 78 },
    { 0xd4b9f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d79f629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b9f632, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3459f63b, 55 },
    { 0x7909f63c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf629f640, 91 },
    { 0x44e9f640, 23 },
    { 0xc069f640, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e9f646, 33 },
    { 0xa889f647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc339f64a, 90 },
    { 0x8169f64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee79f653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f65a, 99 },
    { 0, 0 },
    { 0x6b19f65c, 90 },
    { 0x47a9f65c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9f666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc19f669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac9f66d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049f670, 43 },
    { 0x2279f671, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a9f678, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f9f67b, 88 },
    { 0x7e99f67c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109f680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x129f683, 17 },
    { 0xea39f684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809f687, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7449f68c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be9f690, 23 },
    { 0x9209f691, 85 },
    { 0x9ee9f692, 33 },
    { 0, 0 },
    { 0x7ab9f694, 98 },
    { 0, 0 },
    { 0x4169f696, 66 },
    { 0x1389f696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a9f69b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5229f69f, 55 },
    { 0x3d29f6a0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069f6a5, 55 },
    { 0xffe9f6a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf549f6ab, 52 },
    { 0xf4f9f6ac, 65 },
    { 0, 0 },
    { 0x4999f6ae, 33 },
    { 0x9ae9f6af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f6b4, 33 },
    { 0xe89f6b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4399f6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519f6c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9f6c8, 23 },
    { 0xd079f6c8, 23 },
    { 0x9f6ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9f6cd, 99 },
    { 0xb5e9f6cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499f6d5, 92 },
    { 0x9139f6d6, 90 },
    { 0x63a9f6d6, 76 },
    { 0x3269f6d6, 98 },
    { 0x8509f6d8, 98 },
    { 0x8de9f6da, 90 },
    { 0x4f79f6d6, 33 },
    { 0x7959f6d6, 51 },
    { 0x5439f6db, 78 },
    { 0x8979f6d6, 76 },
    { 0xe7c9f6d5, 91 },
    { 0x13c9f6e0, 55 },
    { 0x10d9f6e1, 98 },
    { 0x8d99f6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe689f6e8, 55 },
    { 0x6bb9f6e9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669f6ee, 33 },
    { 0, 0 },
    { 0xbf9f6f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa9f6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9f6fb, 23 },
    { 0xf149f6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9f6ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2329f704, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb9f70f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8129f715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a9f718, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d9f71b, 87 },
    { 0xfe39f71c, 34 },
    { 0x2de9f71d, 98 },
    { 0xf4f9f71e, 96 },
    { 0x8089f71e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae9f726, 38 },
    { 0xeb09f727, 98 },
    { 0x4b9f727, 90 },
    { 0xd859f729, 87 },
    { 0, 0 },
    { 0xf659f72b, 98 },
    { 0xb8e9f72c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa889f732, 23 },
    { 0x4339f733, 61 },
    { 0x5089f732, 90 },
    { 0x1d49f735, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09f738, 98 },
    { 0xc149f738, 90 },
    { 0x5429f73a, 55 },
    { 0x3299f73b, 46 },
    { 0, 0 },
    { 0xb0a9f73d, 98 },
    { 0x8079f73d, 61 },
    { 0xd519f73f, 76 },
    { 0x13d9f740, 98 },
    { 0x8ac9f73d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9f746, 61 },
    { 0xc629f747, 34 },
    { 0x3259f748, 23 },
    { 0x79e9f749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb719f74e, 65 },
    { 0xc879f74f, 90 },
    { 0x4de9f750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c99f753, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2259f757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d9f75b, 76 },
    { 0, 0 },
    { 0x69f75d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b49f762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9f76a, 46 },
    { 0x37f9f76b, 90 },
    { 0x9c49f76c, 98 },
    { 0, 0 },
    { 0xffc9f76e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef9f772, 61 },
    { 0xc669f773, 33 },
    { 0x6659f773, 43 },
    { 0, 0 },
    { 0x8239f776, 90 },
    { 0x2d59f776, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9f77c, 90 },
    { 0, 0 },
    { 0xf999f77e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db9f783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369f789, 61 },
    { 0xf4b9f78a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339f78f, 61 },
    { 0, 0 },
    { 0xd4f9f791, 81 },
    { 0x63a9f792, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859f798, 95 },
    { 0xaa29f799, 91 },
    { 0xd4b9f799, 55 },
    { 0xfd49f799, 95 },
    { 0xe5f9f799, 76 },
    { 0xbd39f79b, 68 },
    { 0xbfb9f799, 66 },
    { 0xc2d9f79f, 99 },
    { 0x7219f7a0, 90 },
    { 0x78c9f7a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1229f7a4, 66 },
    { 0x1b19f7a5, 34 },
    { 0xc799f7a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f9f7aa, 46 },
    { 0xc019f7ab, 98 },
    { 0xd579f7ac, 98 },
    { 0x38a9f7ab, 95 },
    { 0x3d99f7ae, 48 },
    { 0x7219f7af, 95 },
    { 0x2739f7af, 33 },
    { 0xd979f7ac, 100 },
    { 0xd069f7aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea19f7b5, 76 },
    { 0x3cb9f7b5, 34 },
    { 0, 0 },
    { 0x6469f7b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9f7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c79f7c0, 61 },
    { 0x2ac9f7c0, 81 },
    { 0x4ec9f7c2, 90 },
    { 0x2de9f7c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769f7c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe379f7ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3469f7cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa729f7d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9f7d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e9f7db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3579f7e0, 91 },
    { 0x7999f7e1, 76 },
    { 0xe09f7e2, 52 },
    { 0x6ab9f7e3, 91 },
    { 0xe1b9f7e2, 61 },
    { 0x16c9f7e5, 95 },
    { 0x7e39f7e3, 33 },
    { 0xc89f7e7, 61 },
    { 0x5569f7e3, 95 },
    { 0x6549f7e9, 98 },
    { 0xa619f7e7, 61 },
    { 0xc209f7eb, 78 },
    { 0x3d99f7eb, 95 },
    { 0x5e09f7ed, 67 },
    { 0xee19f7ee, 84 },
    { 0xc209f7ed, 98 },
    { 0x2a29f7ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209f7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809f7fd, 23 },
    { 0xdf29f7fe, 76 },
    { 0, 0 },
    { 0x1e9f800, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa519f805, 33 },
    { 0xb359f805, 94 },
    { 0x2949f807, 84 },
    { 0x22b9f805, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c69f810, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa19f813, 90 },
    { 0xed09f814, 76 },
    { 0x2619f815, 55 },
    { 0x95b9f814, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9f81c, 33 },
    { 0, 0 },
    { 0xb819f81e, 53 },
    { 0xede9f81f, 23 },
    { 0x5e39f81e, 95 },
    { 0xea19f821, 23 },
    { 0, 0 },
    { 0xac69f823, 33 },
    { 0, 0 },
    { 0xe579f825, 93 },
    { 0x51e9f826, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9069f829, 90 },
    { 0x3b29f829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d79f82e, 43 },
    { 0, 0 },
    { 0x1a09f830, 95 },
    { 0xab89f831, 93 },
    { 0x2be9f830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979f835, 66 },
    { 0, 0 },
    { 0x2689f837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e9f842, 98 },
    { 0, 0 },
    { 0x5809f844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1729f84b, 98 },
    { 0x7819f84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f84f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f853, 61 },
    { 0x9909f854, 34 },
    { 0, 0 },
    { 0xad59f856, 98 },
    { 0xe9a9f856, 78 },
    { 0x1de9f858, 56 },
    { 0x9689f859, 98 },
    { 0xff69f858, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9f86a, 55 },
    { 0, 0 },
    { 0x8589f86c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d09f872, 95 },
    { 0, 0 },
    { 0xfea9f874, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f877, 61 },
    { 0, 0 },
    { 0xe2b9f879, 61 },
    { 0, 0 },
    { 0x42e9f87b, 86 },
    { 0x639f87b, 81 },
    { 0x1969f87b, 23 },
    { 0, 0 },
    { 0xc179f87f, 84 },
    { 0x1fc9f87f, 69 },
    { 0x5e59f881, 65 },
    { 0, 0 },
    { 0xe8a9f883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1429f888, 33 },
    { 0xdd89f888, 90 },
    { 0x729f88a, 33 },
    { 0, 0 },
    { 0x5869f88c, 95 },
    { 0x2919f88c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9f892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569f898, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab9f89b, 90 },
    { 0, 0 },
    { 0xa569f89d, 55 },
    { 0xa039f89e, 98 },
    { 0x339f89f, 33 },
    { 0xc1a9f8a0, 91 },
    { 0x149f8a0, 66 },
    { 0xb3a9f89d, 90 },
    { 0xd229f8a3, 90 },
    { 0xf729f8a0, 100 },
    { 0x2549f8a5, 61 },
    { 0, 0 },
    { 0xcd19f8a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb089f8aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a9f8ae, 34 },
    { 0xc379f8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c9f8b4, 61 },
    { 0x7949f8b5, 46 },
    { 0, 0 },
    { 0x8b99f8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d9f8bf, 23 },
    { 0x7799f8c0, 78 },
    { 0x5d09f8c1, 55 },
    { 0xac79f8c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9f8c6, 76 },
    { 0, 0 },
    { 0x7d99f8c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd829f8cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819f8d7, 65 },
    { 0xaa89f8d8, 91 },
    { 0x2769f8d9, 90 },
    { 0xefb9f8d8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829f8de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e29f8e1, 90 },
    { 0x58e9f8e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19f8e5, 81 },
    { 0x2ce9f8e6, 23 },
    { 0, 0 },
    { 0x349f8e8, 98 },
    { 0x2c09f8e9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c9f8ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139f8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a9f8f8, 76 },
    { 0x3bf9f8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5289f900, 98 },
    { 0, 0 },
    { 0xd469f902, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e99f90d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9f911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4099f91b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f89f921, 90 },
    { 0x2299f922, 81 },
    { 0x2e69f923, 81 },
    { 0xccf9f924, 95 },
    { 0xd089f925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3759f931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc179f934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3429f939, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2079f93f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c9f944, 90 },
    { 0x7f99f945, 43 },
    { 0, 0 },
    { 0xb459f947, 23 },
    { 0, 0 },
    { 0xeda9f949, 34 },
    { 0, 0 },
    { 0xfff9f94b, 69 },
    { 0x8a79f94c, 87 },
    { 0x12f9f94c, 61 },
    { 0x8bf9f94c, 43 },
    { 0xea19f94f, 33 },
    { 0xa29f94f, 91 },
    { 0x7ae9f94d, 35 },
    { 0, 0 },
    { 0x6039f953, 33 },
    { 0, 0 },
    { 0x9b29f955, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe899f95b, 40 },
    { 0x6e99f95c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3439f960, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd9f964, 87 },
    { 0x1619f965, 61 },
    { 0x59b9f964, 98 },
    { 0, 0 },
    { 0x8109f968, 55 },
    { 0xd6c9f968, 23 },
    { 0x2389f96a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c9f96e, 61 },
    { 0x22f9f96f, 61 },
    { 0x7049f96f, 33 },
    { 0x8869f96e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c29f976, 76 },
    { 0, 0 },
    { 0x5ec9f978, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a9f97b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c9f985, 55 },
    { 0xdbe9f986, 90 },
    { 0xf419f987, 33 },
    { 0, 0 },
    { 0x6c69f989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299f98c, 93 },
    { 0x6819f98d, 61 },
    { 0, 0 },
    { 0x1749f98f, 61 },
    { 0x1b89f990, 81 },
    { 0xeb9f990, 98 },
    { 0, 0 },
    { 0x91b9f993, 98 },
    { 0x6709f994, 95 },
    { 0, 0 },
    { 0x57d9f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d9f99b, 46 },
    { 0x6a39f99c, 61 },
    { 0x4329f99c, 61 },
    { 0x9309f99c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb59f9a9, 61 },
    { 0x7129f9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1519f9ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e09f9b2, 61 },
    { 0, 0 },
    { 0x7a79f9b4, 76 },
    { 0xda19f9b5, 76 },
    { 0xe819f9b5, 61 },
    { 0x4a69f9b6, 61 },
    { 0x58f9f9b8, 33 },
    { 0x71b9f9b9, 90 },
    { 0x7dc9f9b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19f9be, 87 },
    { 0xdde9f9be, 84 },
    { 0, 0 },
    { 0x449f9c1, 33 },
    { 0, 0 },
    { 0x2419f9c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6859f9ce, 80 },
    { 0x2369f9ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519f9d2, 98 },
    { 0, 0 },
    { 0x5419f9d4, 90 },
    { 0, 0 },
    { 0xea39f9d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4339f9da, 90 },
    { 0, 0 },
    { 0xadf9f9dc, 93 },
    { 0, 0 },
    { 0x9889f9de, 52 },
    { 0x5b69f9df, 99 },
    { 0x39d9f9df, 33 },
    { 0x3119f9e1, 34 },
    { 0, 0 },
    { 0x15b9f9e3, 91 },
    { 0x89f9f9e4, 33 },
    { 0, 0 },
    { 0x9469f9e6, 33 },
    { 0, 0 },
    { 0xa849f9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69f9ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9f9f3, 90 },
    { 0, 0 },
    { 0xb1d9f9f5, 77 },
    { 0x9479f9f6, 46 },
    { 0, 0 },
    { 0x649f9f8, 90 },
    { 0x1329f9f8, 95 },
    { 0, 0 },
    { 0xce39f9fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7439f9ff, 33 },
    { 0x2469fa00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2109fa05, 98 },
    { 0x70c9fa06, 23 },
    { 0x3569fa05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9fa0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c89fa11, 84 },
    { 0x3f89fa11, 99 },
    { 0, 0 },
    { 0x7e99fa14, 95 },
    { 0, 0 },
    { 0xfef9fa16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d9fa1c, 23 },
    { 0xfa09fa1c, 99 },
    { 0, 0 },
    { 0x6ee9fa1f, 90 },
    { 0x3759fa20, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79fa25, 90 },
    { 0x8399fa25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb009fa2b, 90 },
    { 0, 0 },
    { 0x6709fa2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c9fa31, 33 },
    { 0, 0 },
    { 0x7c59fa33, 98 },
    { 0x1f69fa34, 90 },
    { 0, 0 },
    { 0x1829fa36, 23 },
    { 0x1419fa37, 66 },
    { 0x1e79fa37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4019fa3d, 91 },
    { 0x61a9fa3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f9fa41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a9fa4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f9fa52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd9fa57, 33 },
    { 0x93a9fa58, 90 },
    { 0x2a19fa59, 90 },
    { 0x51c9fa5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda39fa61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b79fa64, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e9fa68, 81 },
    { 0x939fa69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4059fa6d, 46 },
    { 0, 0 },
    { 0x9179fa6f, 34 },
    { 0xad99fa6f, 23 },
    { 0x3c79fa71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299fa76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639fa79, 98 },
    { 0, 0 },
    { 0x669fa7b, 33 },
    { 0x9789fa7c, 81 },
    { 0, 0 },
    { 0xd3e9fa7e, 61 },
    { 0, 0 },
    { 0xa999fa80, 23 },
    { 0, 0 },
    { 0xd509fa82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c9fa85, 98 },
    { 0xca9fa86, 87 },
    { 0, 0 },
    { 0x7469fa88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799fa8b, 61 },
    { 0xd989fa8c, 100 },
    { 0xdba9fa8b, 35 },
    { 0x4709fa8e, 28 },
    { 0x3389fa8f, 46 },
    { 0, 0 },
    { 0x69e9fa91, 43 },
    { 0xb8f9fa92, 93 },
    { 0x3d79fa93, 90 },
    { 0xd759fa92, 90 },
    { 0x56a9fa91, 33 },
    { 0, 0 },
    { 0x6ef9fa97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa239fa9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa659faa5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429faae, 33 },
    { 0x9249faaf, 93 },
    { 0xb149fab0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e9fab3, 90 },
    { 0x8959fab4, 88 },
    { 0, 0 },
    { 0x3ef9fab6, 23 },
    { 0xb4b9fab7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf79fabd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99fac6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269fac9, 61 },
    { 0x9f59faca, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b9facf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e9fad6, 91 },
    { 0x4019fad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d9fadb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9399fadf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba9fae4, 87 },
    { 0xddf9fae5, 61 },
    { 0x3b69fae4, 52 },
    { 0x57a9fae5, 76 },
    { 0x9b09fae8, 61 },
    { 0xac99fae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669faed, 49 },
    { 0x7bb9faee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9faf9, 33 },
    { 0x8a79faf9, 56 },
    { 0, 0 },
    { 0xcd39fafc, 90 },
    { 0xc0b9fafd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb799fb00, 34 },
    { 0, 0 },
    { 0x87c9fb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c9fb05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9fb0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa609fb1a, 34 },
    { 0, 0 },
    { 0x3459fb1c, 33 },
    { 0xcd99fb1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a59fb21, 61 },
    { 0x8919fb22, 84 },
    { 0x1089fb23, 98 },
    { 0xaeb9fb22, 83 },
    { 0x5659fb21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a9fb28, 95 },
    { 0xf279fb28, 97 },
    { 0xa259fb28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b9fb2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc599fb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c9fb39, 85 },
    { 0xeee9fb3a, 98 },
    { 0xd169fb39, 95 },
    { 0xada9fb3c, 23 },
    { 0x2b19fb3c, 33 },
    { 0x3349fb3e, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d9fb42, 23 },
    { 0, 0 },
    { 0x7a69fb44, 75 },
    { 0x4149fb44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9fb4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe69fb55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd859fb5f, 84 },
    { 0xfd29fb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9fb6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4729fb70, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619fb74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49fb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9939fb7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6959fb7e, 33 },
    { 0x6b9fb7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a29fb82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509fb87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c9fb8e, 55 },
    { 0x3f59fb8f, 90 },
    { 0, 0 },
    { 0x7a39fb91, 90 },
    { 0xe89fb92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439fb95, 88 },
    { 0x2a79fb96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569fb9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7949fb9f, 93 },
    { 0xc979fba0, 53 },
    { 0xc99fba0, 90 },
    { 0, 0 },
    { 0x679fba3, 90 },
    { 0x1089fba4, 61 },
    { 0x32d9fba5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba9fbb0, 23 },
    { 0xa169fbb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9fbb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59fbb7, 61 },
    { 0x9fc9fbb8, 89 },
    { 0x4189fbb8, 91 },
    { 0x55c9fbba, 61 },
    { 0x569fbb8, 76 },
    { 0xf309fbb7, 33 },
    { 0xbe69fbbd, 78 },
    { 0xe7b9fbbd, 75 },
    { 0x8e29fbbf, 78 },
    { 0, 0 },
    { 0x65d9fbc1, 34 },
    { 0, 0 },
    { 0x69d9fbc3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b9fbca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fbd0, 33 },
    { 0x9d9fbd1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf89fbd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa9fbda, 95 },
    { 0x869fbdb, 55 },
    { 0x5029fbda, 61 },
    { 0x7719fbda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d19fbe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c9fbe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9fbef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99fbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649fbf5, 43 },
    { 0xc579fbf6, 93 },
    { 0x79d9fbf7, 23 },
    { 0xefd9fbf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9fc00, 61 },
    { 0xb489fc00, 90 },
    { 0, 0 },
    { 0xb79fc03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c9fc06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49fc09, 23 },
    { 0xe149fc09, 23 },
    { 0, 0 },
    { 0xd489fc0c, 46 },
    { 0, 0 },
    { 0x1839fc0e, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9fc12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9fc18, 100 },
    { 0xc829fc19, 93 },
    { 0x9469fc1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9fc1d, 88 },
    { 0xe689fc1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e89fc24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b9fc27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79fc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9fc30, 76 },
    { 0x449fc31, 51 },
    { 0x7529fc31, 90 },
    { 0xda79fc33, 90 },
    { 0, 0 },
    { 0xf749fc35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e9fc38, 23 },
    { 0x6d89fc38, 40 },
    { 0xa599fc3a, 90 },
    { 0x8629fc3b, 55 },
    { 0, 0 },
    { 0x53f9fc3d, 55 },
    { 0xb49fc3e, 61 },
    { 0x1659fc3e, 51 },
    { 0x1f79fc40, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7529fc44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79fc4f, 34 },
    { 0, 0 },
    { 0x1d09fc51, 52 },
    { 0x8ac9fc51, 38 },
    { 0, 0 },
    { 0xf279fc54, 90 },
    { 0x5fe9fc55, 33 },
    { 0xfc49fc55, 33 },
    { 0, 0 },
    { 0x4919fc58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e9fc5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9fc62, 87 },
    { 0xfee9fc63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1959fc66, 90 },
    { 0x62d9fc67, 95 },
    { 0xd5f9fc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2849fc6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3179fc70, 95 },
    { 0x199fc71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3019fc75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff9fc78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f79fc7f, 98 },
    { 0x7f69fc80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9fc88, 94 },
    { 0, 0 },
    { 0x8b39fc8a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f9fc95, 76 },
    { 0x1ee9fc95, 95 },
    { 0xbb09fc95, 98 },
    { 0, 0 },
    { 0x2559fc99, 95 },
    { 0, 0 },
    { 0x7929fc9b, 76 },
    { 0x3289fc9c, 34 },
    { 0x35b9fc9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9009fca1, 90 },
    { 0, 0 },
    { 0xc6a9fca3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99fcad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9fcb5, 33 },
    { 0, 0 },
    { 0x649fcb7, 90 },
    { 0x6479fcb8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1129fcbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf19fcbf, 33 },
    { 0xcfd9fcbf, 55 },
    { 0xf729fcc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129fcc4, 95 },
    { 0xf159fcc4, 23 },
    { 0x92d9fcc6, 99 },
    { 0x5f59fcc4, 55 },
    { 0x3c79fcc8, 55 },
    { 0x8dc9fcc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639fcce, 43 },
    { 0xfb79fccf, 33 },
    { 0xcaf9fcd0, 52 },
    { 0xb249fcce, 90 },
    { 0xd289fcd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9fcd5, 76 },
    { 0x3579fcd6, 61 },
    { 0x7029fcd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb29fcdb, 98 },
    { 0x1ed9fcdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb179fcdf, 33 },
    { 0x6e19fcdf, 33 },
    { 0x5c59fcdf, 76 },
    { 0, 0 },
    { 0xfa79fce3, 61 },
    { 0x97f9fce4, 61 },
    { 0x5a49fce4, 46 },
    { 0x7bd9fce5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419fcec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f9fcf1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4999fcf5, 34 },
    { 0, 0 },
    { 0xe649fcf7, 90 },
    { 0x6fc9fcf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd9fcfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc619fd01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9459fd08, 66 },
    { 0x2769fd09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d9fd0c, 55 },
    { 0x44b9fd0d, 95 },
    { 0xb439fd0e, 76 },
    { 0xc819fd0c, 95 },
    { 0x2e49fd10, 84 },
    { 0x8349fd11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a9fd15, 55 },
    { 0x279fd16, 46 },
    { 0x40c9fd17, 78 },
    { 0x73e9fd18, 90 },
    { 0x68c9fd15, 33 },
    { 0xe6a9fd1a, 80 },
    { 0xd9c9fd1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5019fd1e, 61 },
    { 0xa1a9fd1e, 98 },
    { 0x2e19fd20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b19fd24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db9fd2a, 77 },
    { 0, 0 },
    { 0x6269fd2c, 55 },
    { 0, 0 },
    { 0xc69fd2e, 17 },
    { 0x5f79fd2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269fd34, 84 },
    { 0x46c9fd35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029fd3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9fd3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069fd43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb29fd47, 33 },
    { 0x8ee9fd47, 43 },
    { 0x87f9fd49, 23 },
    { 0, 0 },
    { 0x7879fd4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949fd53, 61 },
    { 0, 0 },
    { 0xcc09fd55, 52 },
    { 0x4cd9fd56, 90 },
    { 0xcb09fd57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9fd5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef9fd61, 98 },
    { 0, 0 },
    { 0x5e19fd63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa899fd68, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fd6b, 98 },
    { 0, 0 },
    { 0xd339fd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa749fd70, 33 },
    { 0xb299fd71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809fd79, 90 },
    { 0, 0 },
    { 0x8149fd7b, 61 },
    { 0x6b09fd7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f59fd80, 61 },
    { 0xea19fd81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf69fd87, 55 },
    { 0x93a9fd88, 87 },
    { 0x8e19fd89, 90 },
    { 0x4b09fd88, 65 },
    { 0, 0 },
    { 0x1fe9fd8c, 33 },
    { 0xf379fd8c, 95 },
    { 0x5409fd8e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129fd91, 95 },
    { 0x3f99fd91, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9fd95, 95 },
    { 0, 0 },
    { 0xaf19fd97, 34 },
    { 0, 0 },
    { 0x91c9fd99, 38 },
    { 0x829fd9a, 93 },
    { 0x589fd9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99fd9f, 34 },
    { 0x3f9fd9f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969fda5, 76 },
    { 0, 0 },
    { 0x7fc9fda7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa859fdaa, 34 },
    { 0xc3c9fdab, 75 },
    { 0, 0 },
    { 0x1ac9fdad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba9fdb5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd9fdb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb009fdbc, 78 },
    { 0, 0 },
    { 0x5c69fdbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fdc1, 61 },
    { 0x80f9fdc2, 61 },
    { 0, 0 },
    { 0xa679fdc4, 33 },
    { 0xfb89fdc5, 83 },
    { 0x4c09fdc4, 90 },
    { 0, 0 },
    { 0xd4c9fdc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc379fdcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9fdd1, 33 },
    { 0x2a99fdd2, 66 },
    { 0, 0 },
    { 0x85c9fdd4, 34 },
    { 0xfe49fdd5, 93 },
    { 0xf609fdd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b9fdd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9fdde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b9fde6, 98 },
    { 0, 0 },
    { 0xfc89fde8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c9fdec, 43 },
    { 0xc6d9fdec, 27 },
    { 0x3259fdee, 55 },
    { 0xd889fdef, 90 },
    { 0x3b79fdf0, 90 },
    { 0xcaf9fdef, 90 },
    { 0xf4b9fdf0, 98 },
    { 0xf649fdf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb859fdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1189fdfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99fe00, 69 },
    { 0xa39fe01, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x9259fe04, 76 },
    { 0, 0 },
    { 0x7ac9fe06, 98 },
    { 0x96b9fe06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799fe0a, 98 },
    { 0x4069fe0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f29fe10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad9fe1b, 78 },
    { 0xf09fe1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6979fe1f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c69fe23, 33 },
    { 0x1b9fe24, 95 },
    { 0x599fe25, 61 },
    { 0x15d9fe23, 95 },
    { 0x1389fe27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe619fe2a, 98 },
    { 0x56e9fe2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b69fe34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d9fe3d, 98 },
    { 0, 0 },
    { 0xb2e9fe3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b29fe42, 90 },
    { 0x679fe43, 98 },
    { 0xab79fe44, 93 },
    { 0, 0 },
    { 0x6429fe46, 90 },
    { 0xb1d9fe46, 90 },
    { 0xb449fe48, 78 },
    { 0x8c49fe49, 23 },
    { 0x12c9fe49, 76 },
    { 0, 0 },
    { 0x2ea9fe4c, 76 },
    { 0x86f9fe4c, 55 },
    { 0xf9c9fe4d, 66 },
    { 0xb69fe4c, 75 },
    { 0, 0 },
    { 0x5059fe51, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e9fe56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e9fe5a, 67 },
    { 0, 0 },
    { 0x1f29fe5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49fe5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9fe65, 87 },
    { 0xd59fe66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9fe69, 34 },
    { 0x2b19fe6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f9fe6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9fe77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae89fe7a, 96 },
    { 0, 0 },
    { 0xa4f9fe7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8689fe84, 91 },
    { 0, 0 },
    { 0x9f9fe86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4059fe8b, 17 },
    { 0x6009fe8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9fe8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9fe95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a9fe98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa359fe9b, 33 },
    { 0xc779fe9c, 76 },
    { 0, 0 },
    { 0x7e79fe9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc509fea3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda89fea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c59feab, 33 },
    { 0, 0 },
    { 0x3a9fead, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9feb5, 46 },
    { 0xe1c9feb6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9feb9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc9febc, 46 },
    { 0, 0 },
    { 0x98f9febe, 90 },
    { 0xcc9febf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d9fec5, 46 },
    { 0, 0 },
    { 0x779fec7, 90 },
    { 0, 0 },
    { 0x2b09fec9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf209fecc, 46 },
    { 0x9129fecd, 90 },
    { 0x6c09fecd, 33 },
    { 0xba59fecf, 33 },
    { 0xc9b9fed0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669fed8, 55 },
    { 0, 0 },
    { 0xcd29feda, 100 },
    { 0xf7a9feda, 66 },
    { 0x21f9fedc, 66 },
    { 0xd5d9feda, 66 },
    { 0, 0 },
    { 0x13c9fedf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd539fee2, 23 },
    { 0, 0 },
    { 0x3169fee4, 61 },
    { 0xc9d9fee4, 23 },
    { 0xfa9fee6, 33 },
    { 0xe259fee7, 61 },
    { 0, 0 },
    { 0xfda9fee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8079fef4, 34 },
    { 0xa229fef4, 76 },
    { 0x4a89fef6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fef9, 66 },
    { 0x6e39fef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ff04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e9ff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f39ff0a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4909ff0d, 43 },
    { 0, 0 },
    { 0x41d9ff0f, 87 },
    { 0x13f9ff10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc9ff13, 43 },
    { 0, 0 },
    { 0x4269ff15, 90 },
    { 0x5aa9ff16, 98 },
    { 0x1dd9ff16, 76 },
    { 0, 0 },
    { 0x27e9ff19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a9ff1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce9ff22, 52 },
    { 0, 0 },
    { 0x4819ff24, 55 },
    { 0xbdb9ff25, 33 },
    { 0x5a99ff26, 34 },
    { 0x6779ff26, 93 },
    { 0xded9ff27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979ff2c, 61 },
    { 0xc2f9ff2c, 33 },
    { 0x7939ff2c, 87 },
    { 0x5da9ff2f, 90 },
    { 0, 0 },
    { 0xcea9ff31, 52 },
    { 0xae59ff32, 76 },
    { 0x1a19ff33, 61 },
    { 0x5779ff31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f9ff39, 67 },
    { 0x44c9ff3a, 61 },
    { 0xea39ff3b, 90 },
    { 0xe989ff3c, 65 },
    { 0x4b79ff3c, 91 },
    { 0xb599ff3a, 66 },
    { 0x9d29ff39, 34 },
    { 0xfd59ff40, 61 },
    { 0xccc9ff41, 95 },
    { 0xd859ff42, 84 },
    { 0x9299ff43, 100 },
    { 0x95f9ff44, 43 },
    { 0x8b09ff43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9ff48, 55 },
    { 0x7ea9ff49, 85 },
    { 0xfe99ff4a, 96 },
    { 0x8dd9ff49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769ff4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ff51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc419ff57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b9ff5a, 90 },
    { 0x9ed9ff5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da9ff61, 23 },
    { 0x3c09ff62, 23 },
    { 0xd489ff63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e9ff6f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c19ff72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed9ff75, 87 },
    { 0xaf79ff76, 33 },
    { 0x8fb9ff76, 77 },
    { 0xa7c9ff76, 33 },
    { 0x3a9ff77, 95 },
    { 0x5929ff7a, 33 },
    { 0xd639ff7b, 23 },
    { 0xc129ff7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9ff84, 55 },
    { 0xf79ff85, 55 },
    { 0x7069ff86, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e9ff8a, 33 },
    { 0xfd49ff8b, 46 },
    { 0, 0 },
    { 0x9929ff8d, 76 },
    { 0xa779ff8e, 90 },
    { 0x2e49ff8f, 91 },
    { 0x7129ff8f, 90 },
    { 0x24c9ff8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf99ff96, 78 },
    { 0xa3a9ff96, 23 },
    { 0xfd89ff98, 33 },
    { 0xf309ff99, 34 },
    { 0xc109ff9a, 67 },
    { 0xd1e9ff9a, 90 },
    { 0xbb9ff9c, 61 },
    { 0, 0 },
    { 0xac69ff9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x519ffa1, 91 },
    { 0xb639ffa1, 33 },
    { 0xbf9ffa3, 17 },
    { 0xf6a9ffa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc29ffa8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb069ffab, 33 },
    { 0, 0 },
    { 0x2229ffad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99ffb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce9ffb4, 66 },
    { 0x26f9ffb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e9ffb8, 90 },
    { 0xc959ffb9, 65 },
    { 0xc4a9ffb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4469ffbe, 61 },
    { 0, 0 },
    { 0x7819ffc0, 76 },
    { 0x7d59ffc1, 90 },
    { 0x8b9ffc2, 66 },
    { 0x6089ffc3, 93 },
    { 0xa8e9ffc3, 34 },
    { 0, 0 },
    { 0x5ec9ffc6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d69ffc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc9ffd1, 55 },
    { 0xd3f9ffd2, 87 },
    { 0x1d49ffd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c9ffda, 61 },
    { 0xb69ffdb, 61 },
    { 0xecd9ffdc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4189ffe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069ffe8, 33 },
    { 0xfc9ffe9, 66 },
    { 0x1789ffe8, 67 },
    { 0x2ef9ffe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2759ffef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef9fff7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf19fffb, 95 },
    { 0xb7d9fffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba0000, 60 },
    { 0x49da0000, 43 },
    { 0x595a0001, 6 },
    { 0xd36a0002, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7a0007, 61 },
    { 0x7eaa0008, 33 },
    { 0, 0 },
    { 0x4bda000a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34a000d, 23 },
    { 0xd51a000d, 95 },
    { 0x9e1a000f, 34 },
    { 0x10aa0010, 23 },
    { 0xa7a0010, 61 },
    { 0, 0 },
    { 0x957a0013, 98 },
    { 0xe70a0013, 95 },
    { 0x874a0015, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a0019, 95 },
    { 0x3c6a0019, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a001d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6a0023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x988a0026, 90 },
    { 0x26aa0026, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a002b, 95 },
    { 0, 0 },
    { 0x83ca002d, 85 },
    { 0x846a002d, 98 },
    { 0, 0 },
    { 0x4cda0030, 33 },
    { 0x40aa0031, 84 },
    { 0, 0 },
    { 0xd5fa0033, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a0038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa003d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0a0042, 95 },
    { 0, 0 },
    { 0x836a0044, 61 },
    { 0x43ea0044, 23 },
    { 0xf78a0046, 33 },
    { 0xac8a0047, 46 },
    { 0, 0 },
    { 0x16fa0049, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca004d, 55 },
    { 0x71fa004d, 33 },
    { 0x923a004e, 75 },
    { 0xe64a0050, 52 },
    { 0xca7a004f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fa0057, 51 },
    { 0xf13a0058, 96 },
    { 0xf61a0059, 55 },
    { 0xa0058, 76 },
    { 0x214a0058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedca005f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a0065, 87 },
    { 0xf03a0066, 90 },
    { 0x5c2a0067, 29 },
    { 0, 0 },
    { 0x256a0069, 91 },
    { 0xbcda006a, 90 },
    { 0x750a006a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a0070, 52 },
    { 0x9c0a0071, 46 },
    { 0x873a0070, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a0075, 66 },
    { 0, 0 },
    { 0xd5ca0077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a007a, 91 },
    { 0x1f9a007a, 55 },
    { 0x8baa007c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a0085, 33 },
    { 0x1cda0085, 23 },
    { 0x363a0087, 98 },
    { 0xb6aa0088, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05a008c, 78 },
    { 0, 0 },
    { 0x364a008e, 76 },
    { 0xf8ca008f, 43 },
    { 0x40a008f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a0096, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a00a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea00a3, 33 },
    { 0x53ba00a4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x703a00a7, 85 },
    { 0xa00a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3a00ac, 43 },
    { 0, 0 },
    { 0x839a00ae, 95 },
    { 0xe25a00af, 49 },
    { 0x44ba00b0, 95 },
    { 0xaea00ae, 90 },
    { 0x8dba00b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa00b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba00b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ca00bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a00c0, 23 },
    { 0x419a00c0, 90 },
    { 0, 0 },
    { 0xc4ea00c3, 46 },
    { 0xe1a00c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa00c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ea00ce, 61 },
    { 0x1b3a00cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939a00d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7a00d8, 91 },
    { 0x14ba00d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649a00de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3a00e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x324a00e8, 34 },
    { 0xfb8a00e8, 34 },
    { 0, 0 },
    { 0xdd9a00eb, 90 },
    { 0x749a00ec, 98 },
    { 0xc63a00eb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x694a00f0, 90 },
    { 0xffaa00f0, 98 },
    { 0xe6da00f1, 40 },
    { 0xd34a00f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da00fc, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa0101, 90 },
    { 0xc8ba0101, 95 },
    { 0x734a0103, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba0107, 87 },
    { 0x2b3a0108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a010b, 23 },
    { 0xd91a010c, 90 },
    { 0, 0 },
    { 0xc16a010e, 33 },
    { 0x915a010f, 90 },
    { 0xde9a010e, 61 },
    { 0, 0 },
    { 0x26ea0112, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a0115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba011a, 61 },
    { 0, 0 },
    { 0x6bea011c, 98 },
    { 0xc06a011c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848a0121, 33 },
    { 0xdfca0122, 33 },
    { 0x244a0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85a0126, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9a0129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a012f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a0133, 90 },
    { 0, 0 },
    { 0xda2a0135, 61 },
    { 0x94ea0136, 61 },
    { 0x23aa0136, 95 },
    { 0, 0 },
    { 0xff9a0139, 75 },
    { 0, 0 },
    { 0xd31a013b, 43 },
    { 0, 0 },
    { 0x503a013d, 43 },
    { 0x15ba013d, 66 },
    { 0x3f2a013f, 95 },
    { 0xf7ea013f, 33 },
    { 0xfa1a0140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fca0144, 75 },
    { 0, 0 },
    { 0xd83a0146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8da014c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64aa0150, 67 },
    { 0, 0 },
    { 0xa0ca0152, 93 },
    { 0, 0 },
    { 0x20da0154, 68 },
    { 0x781a0155, 33 },
    { 0xff1a0156, 78 },
    { 0x75fa0154, 98 },
    { 0, 0 },
    { 0x12ca0159, 65 },
    { 0x272a0159, 93 },
    { 0x11da015a, 70 },
    { 0, 0 },
    { 0x1efa015d, 55 },
    { 0x7f0a015e, 95 },
    { 0x89da015f, 61 },
    { 0xaf5a015d, 95 },
    { 0x4b0a015f, 23 },
    { 0xf0ea0162, 61 },
    { 0xb94a0163, 33 },
    { 0x385a0164, 90 },
    { 0, 0 },
    { 0x3a1a0166, 46 },
    { 0xacda0166, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ca016d, 76 },
    { 0xab9a016e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a0176, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ca017b, 78 },
    { 0xbb7a017b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea0180, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5a0183, 78 },
    { 0x6d6a0183, 23 },
    { 0xa39a0183, 67 },
    { 0x85aa0184, 90 },
    { 0x49da0184, 100 },
    { 0x5fca0188, 61 },
    { 0x458a0184, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ca0193, 78 },
    { 0xf03a0194, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400a0198, 95 },
    { 0, 0 },
    { 0x655a019a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802a019e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23a01ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7a01b2, 46 },
    { 0xdd6a01b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a01b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x132a01b9, 76 },
    { 0x267a01ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a01bf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ea01c4, 66 },
    { 0x9b7a01c4, 75 },
    { 0, 0 },
    { 0xd5fa01c7, 84 },
    { 0, 0 },
    { 0xf7ba01c9, 43 },
    { 0xcd3a01ca, 23 },
    { 0, 0 },
    { 0x34aa01cc, 23 },
    { 0x1cca01cc, 95 },
    { 0x998a01cd, 61 },
    { 0x9bda01cf, 87 },
    { 0x28ba01d0, 90 },
    { 0x812a01d1, 23 },
    { 0xfa3a01d2, 23 },
    { 0, 0 },
    { 0xebfa01d4, 84 },
    { 0x1bea01d5, 90 },
    { 0x4c3a01d6, 46 },
    { 0xf00a01d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fa01db, 95 },
    { 0xe24a01dc, 40 },
    { 0, 0 },
    { 0x967a01de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12a01e1, 43 },
    { 0x1f8a01e2, 61 },
    { 0xea0a01e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca01e6, 43 },
    { 0, 0 },
    { 0x265a01e8, 76 },
    { 0x74ea01e8, 100 },
    { 0x4fea01e8, 90 },
    { 0x157a01e9, 61 },
    { 0, 0 },
    { 0xa32a01ed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a01f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a01f4, 33 },
    { 0, 0 },
    { 0xe45a01f6, 100 },
    { 0x747a01f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0aa01fa, 33 },
    { 0, 0 },
    { 0x7daa01fc, 61 },
    { 0, 0 },
    { 0x21ea01fe, 95 },
    { 0, 0 },
    { 0x927a0200, 23 },
    { 0x26fa0201, 61 },
    { 0, 0 },
    { 0x982a0203, 95 },
    { 0x82ca0204, 23 },
    { 0xd7ea0205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a0209, 98 },
    { 0, 0 },
    { 0x8f6a020b, 46 },
    { 0x22da020b, 98 },
    { 0x694a020d, 33 },
    { 0, 0 },
    { 0xf26a020f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998a0213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174a0218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba021c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fa0226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba022d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a0230, 95 },
    { 0x92ba0231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da023f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a0243, 46 },
    { 0, 0 },
    { 0x303a0245, 78 },
    { 0x9bda0246, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a0249, 33 },
    { 0xc8aa024a, 56 },
    { 0xd9ea0249, 23 },
    { 0x40ca024c, 34 },
    { 0x412a024d, 61 },
    { 0xdefa024e, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a0252, 98 },
    { 0x313a0253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9a0256, 99 },
    { 0x373a0257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a025b, 51 },
    { 0x191a025c, 46 },
    { 0, 0 },
    { 0x7baa025e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a0262, 91 },
    { 0x8fea0263, 33 },
    { 0x42fa0264, 61 },
    { 0, 0 },
    { 0x3c4a0266, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76a026a, 98 },
    { 0xb38a026b, 61 },
    { 0xf49a026c, 98 },
    { 0x812a026d, 33 },
    { 0x916a026a, 33 },
    { 0, 0 },
    { 0x58fa0270, 75 },
    { 0xea4a0270, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea0277, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fa027a, 90 },
    { 0, 0 },
    { 0x5ffa027c, 61 },
    { 0x6dea027c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91a0283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33a028c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a028f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca0292, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ba0298, 77 },
    { 0x20a0299, 61 },
    { 0xfc9a0299, 23 },
    { 0x8ea029b, 61 },
    { 0x35ca029c, 33 },
    { 0, 0 },
    { 0x669a029e, 90 },
    { 0, 0 },
    { 0x39da02a0, 78 },
    { 0, 0 },
    { 0xa4ca02a2, 61 },
    { 0xf2ea02a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a02a7, 69 },
    { 0, 0 },
    { 0x724a02a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a02b2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa02b9, 76 },
    { 0xac2a02ba, 34 },
    { 0, 0 },
    { 0x24aa02bc, 90 },
    { 0, 0 },
    { 0xca0a02be, 33 },
    { 0xb7fa02bf, 23 },
    { 0x8e0a02c0, 62 },
    { 0x5cba02c0, 98 },
    { 0x3c3a02c0, 23 },
    { 0x313a02c3, 76 },
    { 0, 0 },
    { 0x676a02c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba02cb, 23 },
    { 0x947a02cc, 33 },
    { 0xb74a02cb, 55 },
    { 0xfa7a02cc, 69 },
    { 0x9e0a02cf, 34 },
    { 0x88aa02cf, 43 },
    { 0x813a02cf, 98 },
    { 0x451a02d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ba02d5, 33 },
    { 0, 0 },
    { 0x5a8a02d7, 46 },
    { 0x4d8a02d7, 61 },
    { 0, 0 },
    { 0x617a02da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caa02dd, 95 },
    { 0, 0 },
    { 0xd43a02df, 34 },
    { 0x679a02e0, 33 },
    { 0x4e2a02e1, 48 },
    { 0xa2ba02e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7a02e5, 43 },
    { 0, 0 },
    { 0xb9a02e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a02ed, 75 },
    { 0xbeea02ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba02f4, 77 },
    { 0xbfda02f5, 66 },
    { 0x76aa02f4, 43 },
    { 0x18aa02f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a02fc, 95 },
    { 0x198a02fd, 98 },
    { 0xb0da02fd, 23 },
    { 0x992a02ff, 61 },
    { 0x18ea0300, 33 },
    { 0x81ca0300, 38 },
    { 0xcf0a0302, 95 },
    { 0xbdca0303, 34 },
    { 0x395a0303, 93 },
    { 0x154a0305, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a030c, 33 },
    { 0x58da030c, 91 },
    { 0xbc1a030e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa0316, 46 },
    { 0x7ffa0316, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a031b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a0322, 76 },
    { 0, 0 },
    { 0xbd9a0324, 99 },
    { 0, 0 },
    { 0x3b4a0326, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a0329, 76 },
    { 0, 0 },
    { 0x827a032b, 33 },
    { 0xeb9a032c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86a0333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca033d, 90 },
    { 0xd40a033e, 43 },
    { 0, 0 },
    { 0x757a0340, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4a0345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba034a, 33 },
    { 0xf90a034b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64a034e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a0352, 59 },
    { 0x3f0a0353, 98 },
    { 0, 0 },
    { 0xe1a0355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a0358, 61 },
    { 0x8f7a0358, 23 },
    { 0xedba035a, 33 },
    { 0x3a5a035b, 66 },
    { 0xbfea035c, 33 },
    { 0xdcda035d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea0363, 87 },
    { 0xad9a0364, 99 },
    { 0x120a0365, 90 },
    { 0x255a0364, 91 },
    { 0x1d5a0367, 61 },
    { 0x32da0367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fa036e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61a0371, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ea0375, 88 },
    { 0, 0 },
    { 0x78ba0377, 33 },
    { 0x39da0378, 33 },
    { 0x693a0379, 23 },
    { 0x186a0379, 95 },
    { 0xe41a037a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a037f, 66 },
    { 0x181a0380, 88 },
    { 0, 0 },
    { 0x12da0382, 61 },
    { 0, 0 },
    { 0x6fca0384, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a0387, 61 },
    { 0x913a0388, 77 },
    { 0xd05a0389, 90 },
    { 0, 0 },
    { 0x5dca038b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a0391, 23 },
    { 0x266a0392, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0396, 90 },
    { 0x265a0396, 84 },
    { 0, 0 },
    { 0x94ba0399, 76 },
    { 0, 0 },
    { 0x313a039b, 85 },
    { 0, 0 },
    { 0x403a039d, 23 },
    { 0xcdfa039e, 17 },
    { 0xf27a039d, 82 },
    { 0, 0 },
    { 0xa7aa03a1, 43 },
    { 0x8dea03a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaa03a5, 90 },
    { 0, 0 },
    { 0x1c8a03a7, 33 },
    { 0x52ba03a8, 33 },
    { 0x592a03a9, 43 },
    { 0xf57a03a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a03ad, 95 },
    { 0x352a03ae, 95 },
    { 0x20ca03ae, 46 },
    { 0x978a03b0, 90 },
    { 0x209a03b1, 76 },
    { 0, 0 },
    { 0x2dba03b3, 77 },
    { 0xbe7a03b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15a03b9, 98 },
    { 0x9cca03ba, 76 },
    { 0, 0 },
    { 0x9cfa03bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a03bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a03c5, 33 },
    { 0, 0 },
    { 0xcafa03c7, 43 },
    { 0x8e8a03c8, 98 },
    { 0x72da03c7, 78 },
    { 0x9e6a03ca, 91 },
    { 0x293a03cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aa03d2, 23 },
    { 0x6d3a03d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27a03d6, 98 },
    { 0x5e7a03d7, 84 },
    { 0x5a5a03d8, 33 },
    { 0x905a03d7, 87 },
    { 0xca03d8, 90 },
    { 0x2e1a03d7, 90 },
    { 0x724a03dc, 90 },
    { 0xb0ba03d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29a03e2, 34 },
    { 0x72ca03e2, 33 },
    { 0xaf4a03e3, 98 },
    { 0x583a03e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53aa03e8, 33 },
    { 0x458a03e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621a03f0, 90 },
    { 0x250a03f1, 98 },
    { 0xebca03f2, 33 },
    { 0xf9da03f1, 80 },
    { 0xb89a03f4, 76 },
    { 0, 0 },
    { 0x264a03f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea03fb, 17 },
    { 0x28a03fc, 34 },
    { 0, 0 },
    { 0xd7ca03fe, 98 },
    { 0xf25a03ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a0403, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a0411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a0415, 61 },
    { 0x190a0416, 66 },
    { 0, 0 },
    { 0x5a4a0418, 96 },
    { 0, 0 },
    { 0xd03a041a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90a041d, 61 },
    { 0xd02a041e, 76 },
    { 0xf3fa041e, 67 },
    { 0x476a0420, 95 },
    { 0x141a041f, 56 },
    { 0x9e6a0422, 61 },
    { 0x50fa0423, 93 },
    { 0xfcba0424, 90 },
    { 0x4ca0424, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758a0429, 23 },
    { 0, 0 },
    { 0xf7a042b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea042f, 23 },
    { 0xfe9a042f, 78 },
    { 0, 0 },
    { 0x45fa0432, 95 },
    { 0, 0 },
    { 0x33ca0434, 95 },
    { 0x94aa0435, 95 },
    { 0, 0 },
    { 0x517a0437, 46 },
    { 0, 0 },
    { 0xde9a0439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bba0445, 90 },
    { 0xdf0a0445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a044c, 87 },
    { 0xd21a044d, 76 },
    { 0x3ea044d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6a0452, 81 },
    { 0xb59a0453, 55 },
    { 0x6d7a0452, 98 },
    { 0xa4ca0454, 74 },
    { 0x6a7a0456, 91 },
    { 0x1cfa0452, 46 },
    { 0x56aa0458, 90 },
    { 0xcca0459, 23 },
    { 0xe7ca0453, 33 },
    { 0x8d9a045b, 33 },
    { 0, 0 },
    { 0x1d2a045d, 23 },
    { 0xeaba045e, 78 },
    { 0x99fa045f, 76 },
    { 0x9fea0460, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8a046e, 93 },
    { 0x1eca046e, 77 },
    { 0x5fa046e, 90 },
    { 0xfbba046f, 90 },
    { 0xf10a046e, 23 },
    { 0x5a4a0472, 33 },
    { 0x592a046e, 43 },
    { 0xaaa0475, 78 },
    { 0x636a0476, 33 },
    { 0x7eba0477, 95 },
    { 0x229a0477, 34 },
    { 0x4f7a0479, 76 },
    { 0x2e1a047a, 33 },
    { 0x652a047b, 61 },
    { 0x565a047a, 90 },
    { 0xceba0479, 98 },
    { 0, 0 },
    { 0x45ba047f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a0483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea0486, 98 },
    { 0xbcca0487, 91 },
    { 0, 0 },
    { 0x4a5a0489, 66 },
    { 0x93a048a, 61 },
    { 0x492a0489, 87 },
    { 0x304a048c, 33 },
    { 0xec5a048d, 76 },
    { 0xe49a048d, 99 },
    { 0x504a048a, 33 },
    { 0x48da0490, 90 },
    { 0x155a0491, 17 },
    { 0x42ea0490, 90 },
    { 0x907a0493, 84 },
    { 0, 0 },
    { 0xb8da0495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabda049b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aea049e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a04a5, 98 },
    { 0, 0 },
    { 0x4e1a04a7, 95 },
    { 0x55ca04a8, 33 },
    { 0xa31a04a7, 84 },
    { 0x4ca04aa, 23 },
    { 0x588a04ab, 55 },
    { 0, 0 },
    { 0x66ca04ad, 90 },
    { 0, 0 },
    { 0xb6ca04af, 61 },
    { 0xf4fa04b0, 100 },
    { 0x371a04b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba04b4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a04b9, 100 },
    { 0x789a04b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a04be, 52 },
    { 0, 0 },
    { 0x6d4a04c0, 23 },
    { 0, 0 },
    { 0xd47a04c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7a04c7, 76 },
    { 0xdbea04c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a04cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a04d0, 95 },
    { 0, 0 },
    { 0xef0a04d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaa04d5, 55 },
    { 0, 0 },
    { 0xf1fa04d7, 23 },
    { 0x330a04d8, 23 },
    { 0, 0 },
    { 0x9b8a04da, 60 },
    { 0x7e0a04db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ea04e1, 33 },
    { 0x294a04e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03a04eb, 34 },
    { 0x727a04ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a04ef, 90 },
    { 0, 0 },
    { 0x199a04f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a04f7, 55 },
    { 0xaf0a04f8, 68 },
    { 0xad1a04f8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694a0500, 90 },
    { 0x9eca0501, 98 },
    { 0xef4a0502, 61 },
    { 0xd30a0503, 76 },
    { 0x86aa0502, 90 },
    { 0x67da0505, 52 },
    { 0xb25a0506, 78 },
    { 0, 0 },
    { 0x416a0508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a050d, 76 },
    { 0xdc2a050e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a0514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a0518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca051b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ea0523, 75 },
    { 0, 0 },
    { 0xe0ba0525, 95 },
    { 0x70fa0526, 95 },
    { 0x935a0526, 66 },
    { 0x202a0525, 76 },
    { 0x591a0527, 98 },
    { 0x38ba0528, 85 },
    { 0xbf6a0525, 76 },
    { 0xc66a0528, 84 },
    { 0x7e3a052d, 90 },
    { 0xf8ba052e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa0533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea0536, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa0540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fa0548, 99 },
    { 0xaeba0548, 90 },
    { 0x5f4a054a, 93 },
    { 0xdea054b, 84 },
    { 0, 0 },
    { 0x7c2a054d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a0550, 90 },
    { 0, 0 },
    { 0xd98a0552, 34 },
    { 0, 0 },
    { 0x7b2a0554, 23 },
    { 0xaa4a0555, 23 },
    { 0x468a0556, 55 },
    { 0x88fa0557, 98 },
    { 0x4a3a0556, 61 },
    { 0xb4ea0555, 55 },
    { 0, 0 },
    { 0xb36a055b, 96 },
    { 0xd68a055b, 67 },
    { 0x1a3a055d, 46 },
    { 0x9f9a055d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cda0564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a0567, 67 },
    { 0, 0 },
    { 0xddfa0569, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309a056e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfa0575, 95 },
    { 0, 0 },
    { 0xfe7a0577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441a057d, 51 },
    { 0x154a057e, 87 },
    { 0, 0 },
    { 0x121a0580, 95 },
    { 0x3c9a0581, 95 },
    { 0, 0 },
    { 0xb28a0583, 76 },
    { 0xa9a0584, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea058c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8da0592, 23 },
    { 0, 0 },
    { 0x4e2a0594, 90 },
    { 0, 0 },
    { 0xfbca0596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708a0599, 61 },
    { 0, 0 },
    { 0x819a059b, 93 },
    { 0x853a059c, 36 },
    { 0x2b3a059d, 87 },
    { 0xfc1a059e, 93 },
    { 0x186a059e, 34 },
    { 0x969a059f, 55 },
    { 0x730a05a1, 43 },
    { 0, 0 },
    { 0x175a05a3, 90 },
    { 0xbeca05a4, 33 },
    { 0, 0 },
    { 0x890a05a6, 90 },
    { 0, 0 },
    { 0xb2ca05a8, 52 },
    { 0, 0 },
    { 0x34fa05aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ea05ae, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a05b1, 33 },
    { 0x2b5a05b2, 76 },
    { 0xcb5a05b3, 98 },
    { 0x441a05b4, 84 },
    { 0xef7a05b5, 52 },
    { 0x43a05b4, 100 },
    { 0x296a05b7, 83 },
    { 0x521a05b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a05bb, 99 },
    { 0xb66a05bc, 33 },
    { 0x642a05bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6a05c0, 90 },
    { 0, 0 },
    { 0xa27a05c2, 76 },
    { 0x8a0a05c2, 43 },
    { 0x476a05c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114a05c8, 34 },
    { 0x311a05c9, 64 },
    { 0x367a05ca, 55 },
    { 0xa71a05c8, 98 },
    { 0xaeda05c9, 83 },
    { 0xfe0a05cd, 43 },
    { 0x651a05ce, 95 },
    { 0xebfa05ce, 33 },
    { 0, 0 },
    { 0xfbea05d1, 36 },
    { 0, 0 },
    { 0xd93a05d3, 23 },
    { 0, 0 },
    { 0x968a05d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870a05df, 90 },
    { 0, 0 },
    { 0x61fa05e1, 23 },
    { 0x63ea05e1, 90 },
    { 0x7b1a05e2, 23 },
    { 0xd40a05e3, 55 },
    { 0x461a05e3, 33 },
    { 0x1dfa05e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20a05ef, 90 },
    { 0xb26a05f0, 52 },
    { 0xb8ba05f1, 90 },
    { 0x75da05f1, 23 },
    { 0x4b8a05f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a05f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x455a05f9, 95 },
    { 0x5e0a05fa, 43 },
    { 0xb37a05fb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a0604, 23 },
    { 0xbcca0605, 61 },
    { 0, 0 },
    { 0xe5da0607, 23 },
    { 0xbc5a0608, 98 },
    { 0xf0da0609, 93 },
    { 0x40fa0607, 98 },
    { 0xae6a060b, 56 },
    { 0, 0 },
    { 0xf9ba060d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32a0610, 23 },
    { 0x39ea0610, 90 },
    { 0x9d7a0610, 95 },
    { 0x38a0613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9a0617, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a061a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea061d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a062a, 46 },
    { 0x677a062b, 87 },
    { 0, 0 },
    { 0xc44a062d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0630, 100 },
    { 0, 0 },
    { 0x56ba0632, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa0637, 91 },
    { 0, 0 },
    { 0xe4aa0639, 90 },
    { 0x3c8a063a, 43 },
    { 0x87fa063a, 76 },
    { 0x72ea063a, 87 },
    { 0xbe2a063d, 90 },
    { 0, 0 },
    { 0x46ea063f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a0642, 85 },
    { 0x73aa0642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ea064e, 69 },
    { 0, 0 },
    { 0x92a0650, 52 },
    { 0xefca0651, 55 },
    { 0x54ba0650, 95 },
    { 0x12ca0650, 46 },
    { 0xdb3a0653, 99 },
    { 0, 0 },
    { 0xd95a0656, 46 },
    { 0, 0 },
    { 0xfe5a0658, 95 },
    { 0xacda0659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aa065c, 61 },
    { 0xeb8a065d, 46 },
    { 0xdc2a065e, 23 },
    { 0x42da065f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ca0669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6a0670, 88 },
    { 0xbb4a0670, 87 },
    { 0x444a0671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bba0675, 61 },
    { 0x587a0676, 8 },
    { 0xbe7a0676, 55 },
    { 0x13ea0678, 23 },
    { 0x2d2a0679, 93 },
    { 0x2a5a0676, 87 },
    { 0xefba067b, 61 },
    { 0x28da067c, 95 },
    { 0, 0 },
    { 0xb48a067e, 85 },
    { 0xe93a067e, 99 },
    { 0xd4a0680, 9 },
    { 0x81ba0681, 34 },
    { 0, 0 },
    { 0x8d9a0683, 93 },
    { 0xa0683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5a068b, 61 },
    { 0x4c5a068b, 34 },
    { 0x33ea068d, 33 },
    { 0, 0 },
    { 0x487a068f, 33 },
    { 0, 0 },
    { 0x6a9a0691, 34 },
    { 0xa3da0692, 61 },
    { 0, 0 },
    { 0xc22a0694, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2a0697, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a069b, 90 },
    { 0, 0 },
    { 0x37ea069d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffa06a1, 100 },
    { 0x90ca06a2, 68 },
    { 0xf4a06a3, 23 },
    { 0xa7fa06a2, 90 },
    { 0x40ca06a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266a06ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a06b1, 61 },
    { 0x8ffa06b2, 66 },
    { 0, 0 },
    { 0x3a3a06b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efa06bc, 81 },
    { 0x247a06bc, 90 },
    { 0x77ba06bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0a06c2, 95 },
    { 0x349a06c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3a06c7, 33 },
    { 0xe54a06c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba06cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843a06d1, 98 },
    { 0x91aa06d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a06df, 33 },
    { 0x760a06e0, 76 },
    { 0, 0 },
    { 0xdf7a06e2, 76 },
    { 0, 0 },
    { 0xe63a06e4, 87 },
    { 0, 0 },
    { 0xa81a06e6, 68 },
    { 0xb24a06e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da06eb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a06ef, 48 },
    { 0xa94a06f0, 61 },
    { 0xcf0a06ef, 99 },
    { 0xf74a06f1, 61 },
    { 0xeb7a06ef, 69 },
    { 0, 0 },
    { 0xefa06f5, 33 },
    { 0x112a06f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fa06fb, 23 },
    { 0, 0 },
    { 0xfb8a06fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a0700, 36 },
    { 0, 0 },
    { 0xc5da0702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fa0706, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95a070b, 43 },
    { 0, 0 },
    { 0xb5fa070d, 90 },
    { 0x9d1a070d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a0712, 52 },
    { 0x6e9a0713, 77 },
    { 0, 0 },
    { 0x6f9a0715, 23 },
    { 0xc24a0715, 81 },
    { 0xd6ea0716, 23 },
    { 0xa3ea0717, 33 },
    { 0x29aa0719, 98 },
    { 0, 0 },
    { 0xa56a071b, 43 },
    { 0, 0 },
    { 0xe0ca071d, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a0720, 95 },
    { 0x11fa0721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x521a0724, 88 },
    { 0x753a0724, 33 },
    { 0, 0 },
    { 0x5eba0727, 33 },
    { 0x3c5a0727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa072d, 76 },
    { 0, 0 },
    { 0xb83a072f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa0734, 76 },
    { 0, 0 },
    { 0x1d1a0736, 96 },
    { 0xc2da0737, 76 },
    { 0x251a0738, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa073c, 78 },
    { 0x1aba073c, 98 },
    { 0x213a073e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aa0744, 19 },
    { 0x7bfa0745, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea0749, 61 },
    { 0x8ea0749, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eda074d, 52 },
    { 0x1dca074e, 90 },
    { 0x22aa074d, 33 },
    { 0xd60a074f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a0754, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da0757, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6da075a, 33 },
    { 0xed7a075a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a0763, 91 },
    { 0x652a0764, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xada076b, 43 },
    { 0xac4a076b, 90 },
    { 0x587a076d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79a0772, 93 },
    { 0x47a0773, 95 },
    { 0, 0 },
    { 0x8c9a0775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa077e, 23 },
    { 0x9cda077f, 33 },
    { 0, 0 },
    { 0xfd6a0781, 64 },
    { 0x18da0782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3a0788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a078b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7a078e, 90 },
    { 0x3eba078f, 98 },
    { 0x1ea078e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a0795, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ca0799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a07a1, 90 },
    { 0, 0 },
    { 0x13ba07a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a07a7, 75 },
    { 0x4a7a07a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20a07ad, 33 },
    { 0xf69a07ae, 90 },
    { 0xc51a07ae, 95 },
    { 0xf3fa07b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a07b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7a07bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ba07bf, 95 },
    { 0x13a07c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a07c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0a07ce, 76 },
    { 0, 0 },
    { 0xb3a07d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a07d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93a07df, 90 },
    { 0x86a07e0, 66 },
    { 0x3e0a07e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a07e7, 90 },
    { 0x713a07e8, 95 },
    { 0, 0 },
    { 0x373a07ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fa07ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea07f8, 91 },
    { 0x287a07f9, 61 },
    { 0x1b7a07f9, 33 },
    { 0, 0 },
    { 0x8f2a07fc, 95 },
    { 0, 0 },
    { 0x9aa07fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45da0802, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a0806, 23 },
    { 0xc2fa0807, 77 },
    { 0xa93a0808, 95 },
    { 0x11ca0809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9a0810, 67 },
    { 0, 0 },
    { 0xfaea0812, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ba0818, 23 },
    { 0x631a0818, 84 },
    { 0, 0 },
    { 0xda9a081b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a081f, 78 },
    { 0xce2a0820, 76 },
    { 0x6e1a0820, 33 },
    { 0xd8ca0822, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeda0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76a082f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4a0834, 99 },
    { 0x1f4a0835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a083c, 64 },
    { 0x200a083c, 33 },
    { 0x518a083e, 90 },
    { 0x364a083f, 95 },
    { 0x1dca083f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2a084a, 95 },
    { 0xcaba084a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa084e, 99 },
    { 0, 0 },
    { 0xdd6a0850, 76 },
    { 0x2b1a0851, 90 },
    { 0x39da0852, 23 },
    { 0x650a0851, 43 },
    { 0xacca0854, 19 },
    { 0x290a0854, 90 },
    { 0x2dba0856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ea0859, 53 },
    { 0x5d1a0859, 23 },
    { 0x30ca0859, 100 },
    { 0x39ea085b, 55 },
    { 0xb4ca085b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a0864, 98 },
    { 0, 0 },
    { 0x595a0866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a086b, 90 },
    { 0x4d0a086b, 55 },
    { 0xb54a086d, 95 },
    { 0, 0 },
    { 0xcc9a086f, 90 },
    { 0x825a0870, 95 },
    { 0x303a0871, 52 },
    { 0x4ca0870, 90 },
    { 0xf10a0873, 34 },
    { 0xf9ba0874, 33 },
    { 0xf03a0875, 98 },
    { 0x40aa0875, 33 },
    { 0x292a0877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a0881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a0885, 34 },
    { 0x755a0886, 93 },
    { 0xeaba0885, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2a088a, 91 },
    { 0, 0 },
    { 0x928a088c, 43 },
    { 0xd89a088d, 95 },
    { 0, 0 },
    { 0xb5da088f, 98 },
    { 0xcc8a0890, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa0893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaa089a, 91 },
    { 0, 0 },
    { 0x467a089c, 90 },
    { 0, 0 },
    { 0x1c0a089e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92da08a2, 90 },
    { 0xf16a08a2, 61 },
    { 0xd3a08a4, 34 },
    { 0x375a08a2, 90 },
    { 0, 0 },
    { 0xe96a08a7, 67 },
    { 0, 0 },
    { 0x72ca08a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2a08b3, 61 },
    { 0x2bca08b3, 52 },
    { 0x14ea08b5, 76 },
    { 0, 0 },
    { 0x4ba08b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a08bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a08c4, 95 },
    { 0, 0 },
    { 0x859a08c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0a08c9, 33 },
    { 0xae7a08ca, 95 },
    { 0x127a08cb, 61 },
    { 0xabda08cc, 87 },
    { 0x276a08ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aca08d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aa08da, 95 },
    { 0x8a0a08db, 55 },
    { 0xff5a08db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aa08e3, 98 },
    { 0, 0 },
    { 0x56da08e5, 95 },
    { 0x87ea08e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31da08ec, 95 },
    { 0, 0 },
    { 0xcd8a08ee, 43 },
    { 0x110a08ef, 61 },
    { 0, 0 },
    { 0xd15a08f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844a08f6, 43 },
    { 0xfc6a08f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946a08fb, 33 },
    { 0x7e2a08fc, 100 },
    { 0, 0 },
    { 0xebfa08fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa0903, 95 },
    { 0, 0 },
    { 0x4a1a0905, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a0909, 75 },
    { 0xe1ba090a, 46 },
    { 0x384a0909, 40 },
    { 0x873a090c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7a0914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dca0918, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15a091b, 98 },
    { 0x95ea091c, 23 },
    { 0, 0 },
    { 0x4a6a091e, 33 },
    { 0xfe9a091f, 95 },
    { 0, 0 },
    { 0x1b4a0921, 95 },
    { 0x624a0922, 33 },
    { 0xee6a0923, 61 },
    { 0x344a0924, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48a092a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63da092d, 88 },
    { 0x6ca092e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754a0932, 55 },
    { 0, 0 },
    { 0xd1da0934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a0938, 95 },
    { 0, 0 },
    { 0x49aa093a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a0943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ea0946, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48a0949, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fa094d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0951, 61 },
    { 0, 0 },
    { 0xad1a0953, 91 },
    { 0xb72a0953, 46 },
    { 0xa32a0953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a0959, 61 },
    { 0x438a095a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04a095d, 33 },
    { 0xe4da095e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a0961, 61 },
    { 0x236a0961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622a0965, 55 },
    { 0x22ba0965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a096b, 90 },
    { 0x6bea096c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0a0974, 55 },
    { 0xceca0975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a0979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba097e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11a0983, 33 },
    { 0x49ca0984, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a0988, 90 },
    { 0x4e9a0988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2a098c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566a0991, 61 },
    { 0, 0 },
    { 0x994a0993, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x601a0996, 100 },
    { 0x770a0997, 55 },
    { 0xd86a0998, 61 },
    { 0x44aa0996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a09a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a09a3, 95 },
    { 0x1c5a09a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08a09a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a09aa, 43 },
    { 0, 0 },
    { 0x3a5a09ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a09af, 98 },
    { 0xaafa09b0, 98 },
    { 0, 0 },
    { 0x7fea09b2, 67 },
    { 0, 0 },
    { 0xbf2a09b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3a09b7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fa09bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ba09c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ea09c7, 23 },
    { 0xa02a09c7, 33 },
    { 0x34aa09c9, 76 },
    { 0xa00a09ca, 77 },
    { 0x4e9a09cb, 23 },
    { 0x22ba09cc, 66 },
    { 0xab6a09c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412a09d9, 33 },
    { 0x6dfa09da, 99 },
    { 0, 0 },
    { 0x351a09dc, 91 },
    { 0xbe5a09dd, 61 },
    { 0x4f8a09dd, 34 },
    { 0x7a4a09dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x213a09e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9a09ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13a09ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ea09f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a09f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a0a00, 94 },
    { 0, 0 },
    { 0x513a0a02, 66 },
    { 0x8e4a0a02, 98 },
    { 0x8fda0a04, 67 },
    { 0xc68a0a05, 95 },
    { 0x227a0a06, 98 },
    { 0x349a0a04, 43 },
    { 0, 0 },
    { 0xa09a0a09, 99 },
    { 0x33fa0a0a, 23 },
    { 0, 0 },
    { 0x9ca0a0c, 33 },
    { 0xae5a0a0d, 78 },
    { 0xf9da0a0d, 90 },
    { 0, 0 },
    { 0xbd5a0a10, 76 },
    { 0x75da0a10, 33 },
    { 0xd11a0a12, 33 },
    { 0xe57a0a13, 55 },
    { 0xaa2a0a11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afa0a19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65da0a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca0a21, 61 },
    { 0, 0 },
    { 0xf56a0a23, 90 },
    { 0, 0 },
    { 0x85ca0a25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405a0a2c, 90 },
    { 0, 0 },
    { 0x412a0a2e, 95 },
    { 0, 0 },
    { 0x60ea0a30, 40 },
    { 0, 0 },
    { 0x160a0a32, 52 },
    { 0, 0 },
    { 0x1b4a0a34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1a0a37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea0a3a, 78 },
    { 0xdb6a0a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fa0a3e, 95 },
    { 0x3bfa0a3f, 17 },
    { 0, 0 },
    { 0x43aa0a41, 93 },
    { 0x3b9a0a41, 52 },
    { 0x7d7a0a43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75da0a49, 76 },
    { 0x9f7a0a4a, 95 },
    { 0, 0 },
    { 0x744a0a4c, 93 },
    { 0xaeea0a4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x408a0a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ca0a54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba0a57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a0a5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a0a61, 99 },
    { 0x711a0a62, 90 },
    { 0xc34a0a61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fda0a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x956a0a6a, 55 },
    { 0, 0 },
    { 0x45aa0a6c, 61 },
    { 0xb04a0a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa0a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3a0a7c, 93 },
    { 0, 0 },
    { 0x9ea0a7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x752a0a81, 95 },
    { 0x79ca0a82, 99 },
    { 0xabaa0a81, 90 },
    { 0xa68a0a84, 91 },
    { 0x726a0a83, 99 },
    { 0, 0 },
    { 0xb63a0a87, 90 },
    { 0, 0 },
    { 0x5d6a0a89, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7a0a8c, 98 },
    { 0x4a4a0a8c, 61 },
    { 0, 0 },
    { 0x949a0a8f, 76 },
    { 0, 0 },
    { 0x6b1a0a91, 77 },
    { 0xbe6a0a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x306a0a95, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a0a99, 46 },
    { 0xf90a0a99, 100 },
    { 0, 0 },
    { 0xb3fa0a9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a0aa0, 90 },
    { 0x9e1a0aa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a0aa6, 91 },
    { 0xde7a0aa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a0aab, 61 },
    { 0xac2a0aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0a0aaf, 90 },
    { 0x62da0aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa0ab4, 52 },
    { 0xcc8a0ab5, 78 },
    { 0x1cda0ab5, 90 },
    { 0x308a0ab7, 90 },
    { 0, 0 },
    { 0x26aa0ab9, 90 },
    { 0x348a0aba, 34 },
    { 0x365a0aba, 98 },
    { 0xa02a0aba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a0ac9, 33 },
    { 0x18a0aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a0ad2, 98 },
    { 0x4ca0ad3, 61 },
    { 0xa8ca0ad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fa0adc, 56 },
    { 0xb7fa0add, 90 },
    { 0, 0 },
    { 0xf6a0adf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x335a0ae2, 98 },
    { 0, 0 },
    { 0xf6da0ae4, 90 },
    { 0x3ada0ae5, 98 },
    { 0, 0 },
    { 0x7c8a0ae7, 99 },
    { 0xca3a0ae8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ba0aeb, 90 },
    { 0xa7da0aec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0af0, 34 },
    { 0, 0 },
    { 0xd2fa0af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97a0af5, 95 },
    { 0x85aa0af6, 55 },
    { 0x61ea0af7, 23 },
    { 0xcea0af7, 33 },
    { 0x61fa0af9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93a0afd, 76 },
    { 0, 0 },
    { 0x2e1a0aff, 66 },
    { 0x960a0aff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963a0b06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca0b0a, 98 },
    { 0xb29a0b0b, 33 },
    { 0, 0 },
    { 0x3e1a0b0d, 99 },
    { 0, 0 },
    { 0x85a0b0f, 33 },
    { 0, 0 },
    { 0xca9a0b11, 99 },
    { 0x74ca0b12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870a0b16, 46 },
    { 0xec1a0b17, 33 },
    { 0x6cfa0b16, 90 },
    { 0x41aa0b19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba0b1f, 100 },
    { 0, 0 },
    { 0xeb5a0b21, 84 },
    { 0x9d8a0b22, 66 },
    { 0, 0 },
    { 0xdc1a0b24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700a0b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a0b31, 61 },
    { 0, 0 },
    { 0xcea0b33, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8a0b37, 90 },
    { 0xd1ba0b38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4a0b3e, 33 },
    { 0, 0 },
    { 0xe22a0b40, 98 },
    { 0xb43a0b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca0b45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a0b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a0b4d, 61 },
    { 0xf60a0b4e, 61 },
    { 0, 0 },
    { 0x93ca0b50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12a0b56, 34 },
    { 0x2b1a0b57, 85 },
    { 0xf5fa0b56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63a0b5c, 95 },
    { 0xf82a0b5c, 95 },
    { 0x155a0b5d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x456a0b61, 33 },
    { 0, 0 },
    { 0x3bca0b63, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a0b77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a0b81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aa0b86, 53 },
    { 0x151a0b87, 78 },
    { 0xd4a0b88, 61 },
    { 0x83aa0b87, 23 },
    { 0x152a0b86, 75 },
    { 0, 0 },
    { 0x342a0b8c, 23 },
    { 0, 0 },
    { 0x3d2a0b8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a0b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca0b98, 90 },
    { 0x9c7a0b99, 98 },
    { 0x239a0b9a, 61 },
    { 0, 0 },
    { 0x393a0b9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a0ba1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a0bab, 61 },
    { 0xda0a0bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a0bb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6a0bba, 55 },
    { 0xe43a0bbb, 90 },
    { 0x1c2a0bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a0bc0, 90 },
    { 0x2aa0bc1, 61 },
    { 0, 0 },
    { 0x7a3a0bc3, 76 },
    { 0, 0 },
    { 0x78da0bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1a0bcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ea0bd5, 34 },
    { 0, 0 },
    { 0xe41a0bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a0bda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dba0be0, 61 },
    { 0, 0 },
    { 0x7fba0be2, 76 },
    { 0, 0 },
    { 0x5c9a0be4, 33 },
    { 0xbf0a0be5, 75 },
    { 0x131a0be6, 76 },
    { 0xc3aa0be7, 98 },
    { 0x687a0be7, 17 },
    { 0xb0aa0be7, 98 },
    { 0, 0 },
    { 0x918a0beb, 84 },
    { 0, 0 },
    { 0xd66a0bed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1a0bf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3a0bf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca0bfc, 23 },
    { 0x9b2a0bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0c00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a0c0a, 46 },
    { 0xc57a0c0a, 90 },
    { 0xd6aa0c0c, 90 },
    { 0xc4aa0c0d, 61 },
    { 0, 0 },
    { 0x1d3a0c0f, 95 },
    { 0xc28a0c10, 90 },
    { 0, 0 },
    { 0x1bfa0c12, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0c15, 90 },
    { 0x6aa0c15, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a0c1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca0c23, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba0c26, 23 },
    { 0x3ffa0c27, 52 },
    { 0x614a0c26, 100 },
    { 0x3f6a0c29, 90 },
    { 0x270a0c2a, 23 },
    { 0x26fa0c28, 98 },
    { 0xb03a0c26, 75 },
    { 0xe75a0c2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea0c35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313a0c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaa0c4e, 17 },
    { 0, 0 },
    { 0xbb1a0c50, 55 },
    { 0xe7fa0c51, 95 },
    { 0x204a0c51, 91 },
    { 0x2cfa0c53, 91 },
    { 0xd92a0c53, 61 },
    { 0x8eca0c51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba0c5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa0c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359a0c66, 94 },
    { 0x99aa0c66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea0c6a, 12 },
    { 0, 0 },
    { 0x70ca0c6c, 95 },
    { 0xaada0c6d, 90 },
    { 0x8b5a0c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942a0c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5a0c76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8a0c7b, 65 },
    { 0, 0 },
    { 0xd47a0c7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0da0c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda0c84, 81 },
    { 0xceea0c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba0c8c, 75 },
    { 0x717a0c8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520a0c91, 66 },
    { 0x7b3a0c92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x468a0c95, 61 },
    { 0, 0 },
    { 0x183a0c97, 90 },
    { 0xaf1a0c98, 93 },
    { 0x681a0c98, 87 },
    { 0x926a0c98, 95 },
    { 0xdada0c9b, 98 },
    { 0xbc8a0c9c, 90 },
    { 0xd1aa0c9c, 98 },
    { 0, 0 },
    { 0x4a9a0c9f, 87 },
    { 0xc15a0ca0, 23 },
    { 0, 0 },
    { 0xb47a0ca2, 74 },
    { 0x7e8a0ca2, 84 },
    { 0x6eca0ca3, 84 },
    { 0xf95a0ca5, 76 },
    { 0, 0 },
    { 0xfc8a0ca7, 90 },
    { 0x788a0ca8, 61 },
    { 0x5d6a0ca9, 76 },
    { 0, 0 },
    { 0x2dea0cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a0cb5, 90 },
    { 0, 0 },
    { 0x2bea0cb7, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a0cba, 76 },
    { 0x46ea0cbb, 90 },
    { 0x574a0cbc, 90 },
    { 0xfe9a0cbd, 93 },
    { 0x2a0cbc, 66 },
    { 0, 0 },
    { 0xde5a0cc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6a0cc3, 61 },
    { 0x394a0cc3, 61 },
    { 0, 0 },
    { 0x97ea0cc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8a0cc9, 83 },
    { 0x513a0cca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a0cd2, 33 },
    { 0x3cea0cd3, 78 },
    { 0xd64a0cd2, 33 },
    { 0x5b5a0cd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0a0cd9, 61 },
    { 0x778a0cd9, 43 },
    { 0, 0 },
    { 0x2efa0cdc, 90 },
    { 0, 0 },
    { 0xaafa0cde, 76 },
    { 0x8f9a0cdf, 95 },
    { 0, 0 },
    { 0x4b1a0ce1, 65 },
    { 0x46da0ce2, 33 },
    { 0, 0 },
    { 0x4b0a0ce4, 93 },
    { 0x62fa0ce5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a0ce8, 90 },
    { 0xb93a0ce8, 65 },
    { 0x625a0cea, 61 },
    { 0xc25a0ceb, 100 },
    { 0xee6a0cea, 23 },
    { 0xf74a0ced, 90 },
    { 0xaf5a0cea, 55 },
    { 0, 0 },
    { 0xdb5a0cf0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3a0cf7, 98 },
    { 0xb98a0cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba0cfb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a0d01, 48 },
    { 0, 0 },
    { 0x649a0d03, 34 },
    { 0x7e3a0d04, 23 },
    { 0xe72a0d04, 98 },
    { 0, 0 },
    { 0x14da0d07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea0d0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea0d0d, 23 },
    { 0x1a8a0d0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a0d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624a0d1e, 90 },
    { 0xac0a0d1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617a0d32, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737a0d39, 76 },
    { 0, 0 },
    { 0x9f4a0d3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a0d3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0d41, 80 },
    { 0xb4ea0d41, 87 },
    { 0xa1aa0d43, 77 },
    { 0x47ea0d43, 95 },
    { 0x530a0d45, 90 },
    { 0x936a0d45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a0d4c, 90 },
    { 0x17da0d4d, 95 },
    { 0xa47a0d4d, 43 },
    { 0x94ba0d4f, 43 },
    { 0x78da0d4f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aa0d53, 36 },
    { 0, 0 },
    { 0x593a0d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fea0d58, 98 },
    { 0x915a0d58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582a0d5d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x764a0d60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x951a0d63, 100 },
    { 0x73a0d64, 92 },
    { 0x96a0d65, 93 },
    { 0xbe7a0d66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a0d6a, 33 },
    { 0x178a0d6b, 61 },
    { 0xcdea0d6a, 43 },
    { 0, 0 },
    { 0x9a0a0d6e, 76 },
    { 0xe91a0d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca0d74, 90 },
    { 0x747a0d75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10a0d79, 95 },
    { 0x22fa0d7a, 33 },
    { 0x69ba0d79, 67 },
    { 0x4cda0d7c, 34 },
    { 0x1efa0d7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a0d81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a0d84, 61 },
    { 0xcbba0d85, 23 },
    { 0x8e7a0d86, 33 },
    { 0x33ea0d86, 90 },
    { 0xc66a0d87, 23 },
    { 0x670a0d89, 76 },
    { 0, 0 },
    { 0xf4a0d8b, 61 },
    { 0x8a0a0d8c, 100 },
    { 0x474a0d8b, 61 },
    { 0x463a0d8c, 33 },
    { 0xf4fa0d8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91a0d96, 87 },
    { 0, 0 },
    { 0x3fa0d98, 77 },
    { 0xa4ca0d99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235a0d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca0da1, 96 },
    { 0, 0 },
    { 0x801a0da3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13a0da8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeea0dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42a0dae, 43 },
    { 0, 0 },
    { 0x564a0db0, 95 },
    { 0xcb8a0db1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a0db7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea0dc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7a0dc6, 38 },
    { 0xaa0a0dc7, 33 },
    { 0x735a0dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0dcb, 43 },
    { 0xe3fa0dcb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedea0dd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a0dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62da0dde, 61 },
    { 0x16aa0ddf, 80 },
    { 0x5e7a0de0, 90 },
    { 0x41fa0de1, 90 },
    { 0xab8a0dde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120a0de8, 76 },
    { 0, 0 },
    { 0xfcba0dea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1a0df0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499a0df9, 77 },
    { 0x65ba0df9, 88 },
    { 0xae7a0df9, 33 },
    { 0x26da0dfc, 34 },
    { 0xf9ba0dfc, 33 },
    { 0x32fa0dfe, 61 },
    { 0x84da0dff, 46 },
    { 0x39ea0e00, 66 },
    { 0xf3fa0e01, 61 },
    { 0x86aa0e02, 43 },
    { 0x360a0e03, 90 },
    { 0xf99a0dff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a0e09, 40 },
    { 0, 0 },
    { 0xfc2a0e0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa0e11, 95 },
    { 0x859a0e12, 76 },
    { 0x441a0e13, 78 },
    { 0x348a0e12, 90 },
    { 0x472a0e15, 23 },
    { 0xf08a0e16, 34 },
    { 0xf7a0e12, 34 },
    { 0, 0 },
    { 0x9fca0e19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a0e1f, 77 },
    { 0x870a0e20, 61 },
    { 0x1c9a0e20, 55 },
    { 0x3e1a0e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a0e26, 34 },
    { 0xe3ea0e26, 46 },
    { 0x1efa0e28, 87 },
    { 0, 0 },
    { 0x16a0e2a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca0e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a0e34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa0e3b, 40 },
    { 0xb8fa0e3c, 90 },
    { 0x2dda0e3d, 33 },
    { 0x216a0e3d, 23 },
    { 0x10ca0e3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x982a0e42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aa0e4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x227a0e50, 91 },
    { 0xb9ea0e51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a0e57, 66 },
    { 0x45ca0e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7a0e5b, 33 },
    { 0xaefa0e5b, 55 },
    { 0xba0e5d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57a0e62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a0e67, 23 },
    { 0x2e5a0e68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a0e6e, 34 },
    { 0x788a0e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a0e72, 46 },
    { 0xee8a0e72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffa0e7d, 98 },
    { 0xb04a0e7e, 68 },
    { 0xa95a0e7f, 95 },
    { 0x74a0e80, 23 },
    { 0x451a0e7d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63a0e84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179a0e8a, 90 },
    { 0x8cea0e8b, 66 },
    { 0x23ea0e8a, 90 },
    { 0xd9aa0e8a, 98 },
    { 0, 0 },
    { 0xfb7a0e8f, 34 },
    { 0x6dda0e90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ea0e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a0e96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea0e9a, 34 },
    { 0x906a0e9b, 45 },
    { 0x2a8a0e9b, 43 },
    { 0xe4a0e9d, 90 },
    { 0xa4a0e9b, 80 },
    { 0, 0 },
    { 0xe45a0ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a0ea3, 55 },
    { 0xdeaa0ea4, 8 },
    { 0, 0 },
    { 0xfa7a0ea6, 93 },
    { 0, 0 },
    { 0xc8ba0ea8, 93 },
    { 0, 0 },
    { 0x490a0eaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efa0eb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6a0ebc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x225a0ebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1a0ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73a0ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa0ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346a0ece, 94 },
    { 0x673a0ece, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a0ed3, 34 },
    { 0, 0 },
    { 0xff3a0ed5, 33 },
    { 0x360a0ed6, 90 },
    { 0x117a0ed6, 90 },
    { 0x101a0ed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5a0edb, 55 },
    { 0, 0 },
    { 0x34ea0edd, 61 },
    { 0, 0 },
    { 0x3b5a0edf, 98 },
    { 0xca5a0ee0, 61 },
    { 0x3bba0ee0, 23 },
    { 0x4b6a0ee2, 55 },
    { 0x2d4a0ee3, 33 },
    { 0xf2ba0ee0, 19 },
    { 0, 0 },
    { 0x1c1a0ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fa0ee9, 90 },
    { 0x1fa0eea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x422a0eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5a0ef3, 93 },
    { 0x488a0ef3, 78 },
    { 0, 0 },
    { 0xb7fa0ef6, 95 },
    { 0, 0 },
    { 0x51ea0ef8, 52 },
    { 0x45ea0ef8, 43 },
    { 0x93fa0efa, 40 },
    { 0x219a0ef8, 98 },
    { 0xda4a0efc, 67 },
    { 0xf97a0efd, 95 },
    { 0x256a0efe, 88 },
    { 0x3cfa0efc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33a0f04, 90 },
    { 0x4d9a0f05, 61 },
    { 0x9c9a0f04, 68 },
    { 0, 0 },
    { 0x87fa0f08, 93 },
    { 0xa24a0f09, 67 },
    { 0x6e0a0f09, 43 },
    { 0xf19a0f0b, 78 },
    { 0x46ba0f0b, 33 },
    { 0xf80a0f0d, 95 },
    { 0x228a0f0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2a0f14, 33 },
    { 0xa50a0f15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89da0f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a0f1e, 91 },
    { 0x622a0f1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ea0f23, 69 },
    { 0x975a0f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaa0f28, 61 },
    { 0x321a0f28, 85 },
    { 0xae7a0f29, 33 },
    { 0, 0 },
    { 0xcc5a0f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1a0f2f, 43 },
    { 0x45ba0f30, 64 },
    { 0x1bba0f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385a0f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa0f38, 90 },
    { 0xa4ea0f39, 23 },
    { 0, 0 },
    { 0xb3aa0f3b, 23 },
    { 0x75ba0f3c, 76 },
    { 0x28fa0f3d, 93 },
    { 0xc18a0f3e, 52 },
    { 0x3afa0f3e, 33 },
    { 0, 0 },
    { 0xb87a0f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea0f44, 95 },
    { 0x45ea0f44, 43 },
    { 0x20aa0f46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4a0f4a, 23 },
    { 0, 0 },
    { 0xaf5a0f4c, 90 },
    { 0x9e9a0f4c, 61 },
    { 0xbda0f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ca0f53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270a0f57, 93 },
    { 0, 0 },
    { 0x13ea0f59, 23 },
    { 0, 0 },
    { 0xabda0f5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa0f5f, 53 },
    { 0xbfea0f5f, 66 },
    { 0x353a0f61, 23 },
    { 0xdd9a0f61, 46 },
    { 0x1b8a0f5f, 98 },
    { 0xca0a0f64, 17 },
    { 0xae1a0f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04a0f6a, 84 },
    { 0x6e4a0f6b, 90 },
    { 0, 0 },
    { 0x4d2a0f6d, 90 },
    { 0xe44a0f6e, 23 },
    { 0xaa5a0f6d, 55 },
    { 0x6b8a0f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dba0f73, 61 },
    { 0x8bea0f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea0f77, 95 },
    { 0x5a3a0f77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea0f7c, 61 },
    { 0x195a0f7d, 95 },
    { 0x331a0f7e, 95 },
    { 0x98ea0f7f, 33 },
    { 0x494a0f7d, 55 },
    { 0x55ea0f7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a0f85, 98 },
    { 0, 0 },
    { 0x1e1a0f87, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749a0f8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a0f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63a0f96, 33 },
    { 0, 0 },
    { 0xc40a0f98, 90 },
    { 0x7ea0f99, 90 },
    { 0x794a0f9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a0f9d, 90 },
    { 0x87ca0f9e, 76 },
    { 0xce4a0f9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a0fa9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150a0fae, 61 },
    { 0xdcba0faf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea0fb5, 90 },
    { 0xfdda0fb6, 98 },
    { 0x59aa0fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a0fba, 90 },
    { 0x49ca0fbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a0fbf, 33 },
    { 0xdbea0fc0, 55 },
    { 0x3a0fc1, 98 },
    { 0x278a0fc2, 98 },
    { 0x584a0fc3, 90 },
    { 0x7aaa0fc0, 88 },
    { 0xd24a0fc5, 78 },
    { 0, 0 },
    { 0xa90a0fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a0fcc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bba0fcf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da0fd5, 90 },
    { 0x86ba0fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2a0fd9, 95 },
    { 0x492a0fda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a0fe0, 87 },
    { 0xd97a0fe1, 33 },
    { 0xd92a0fe2, 77 },
    { 0x55ba0fe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a0fe8, 90 },
    { 0xf77a0fe9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a0ff1, 95 },
    { 0x2c3a0ff1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ea0ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862a0ff9, 15 },
    { 0, 0 },
    { 0xe9aa0ffb, 33 },
    { 0x279a0ffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a1001, 23 },
    { 0x229a1002, 33 },
    { 0, 0 },
    { 0x86da1004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a1007, 46 },
    { 0xbd0a1007, 38 },
    { 0, 0 },
    { 0xa10a100a, 67 },
    { 0x306a100b, 91 },
    { 0xea5a100b, 43 },
    { 0xa59a100d, 91 },
    { 0, 0 },
    { 0x39ea100f, 61 },
    { 0x7f3a1010, 33 },
    { 0, 0 },
    { 0xce9a1012, 95 },
    { 0x9aea1013, 61 },
    { 0x5c7a1014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada101a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a101d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa1021, 34 },
    { 0xce3a1021, 98 },
    { 0, 0 },
    { 0x9d0a1024, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a102c, 98 },
    { 0xfc3a102d, 90 },
    { 0x4c5a102c, 34 },
    { 0x222a102f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37a1032, 61 },
    { 0x7e5a1032, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5a1036, 85 },
    { 0xcfea1037, 98 },
    { 0x68da1038, 98 },
    { 0x3ea1038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a1040, 56 },
    { 0xf84a1040, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea1045, 23 },
    { 0xcb0a1046, 98 },
    { 0x753a1047, 33 },
    { 0x654a1048, 98 },
    { 0x6fda1047, 33 },
    { 0xf85a1045, 95 },
    { 0xd52a104b, 34 },
    { 0xfb4a1046, 33 },
    { 0x36a1045, 100 },
    { 0x1a104e, 95 },
    { 0xd3da104e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1053, 23 },
    { 0x82a1054, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a1057, 95 },
    { 0xd82a1058, 56 },
    { 0, 0 },
    { 0x87da105a, 43 },
    { 0xb22a105a, 33 },
    { 0x396a105b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a1060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146a1065, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a1068, 52 },
    { 0, 0 },
    { 0x897a106a, 90 },
    { 0x29fa106a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a106f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba1072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8a1078, 61 },
    { 0, 0 },
    { 0x820a107a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a107d, 95 },
    { 0, 0 },
    { 0xda5a107f, 34 },
    { 0, 0 },
    { 0xc3a1081, 46 },
    { 0xdd9a1082, 90 },
    { 0, 0 },
    { 0x583a1084, 78 },
    { 0xc21a1084, 90 },
    { 0, 0 },
    { 0x17fa1087, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a108f, 85 },
    { 0xca5a1090, 90 },
    { 0x25fa1090, 69 },
    { 0x1fba108f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a109d, 46 },
    { 0, 0 },
    { 0xef7a109f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a10a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a10a9, 98 },
    { 0x731a10a9, 67 },
    { 0x824a10ab, 81 },
    { 0, 0 },
    { 0xf9a10ad, 33 },
    { 0xa4a10ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7a10b1, 95 },
    { 0, 0 },
    { 0xef3a10b3, 23 },
    { 0xc7ba10b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a10b8, 91 },
    { 0, 0 },
    { 0x297a10ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ca10bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a10c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aa10d1, 33 },
    { 0xf33a10d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa10d7, 61 },
    { 0, 0 },
    { 0xad3a10d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca10de, 34 },
    { 0x285a10df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ba10e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a10e7, 33 },
    { 0xa93a10e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a10ec, 49 },
    { 0xe1fa10ed, 90 },
    { 0xf61a10ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979a10f1, 61 },
    { 0x1fa10f2, 90 },
    { 0xed7a10f3, 81 },
    { 0xa38a10f4, 78 },
    { 0x62ea10f1, 98 },
    { 0x2dca10f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a10fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8a1100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17a1103, 43 },
    { 0x119a1104, 56 },
    { 0xc98a1105, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a1113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a1117, 46 },
    { 0x81ca1118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa111c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca1121, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fa1124, 61 },
    { 0x77ea1124, 67 },
    { 0xbbba1126, 43 },
    { 0, 0 },
    { 0x5eaa1128, 78 },
    { 0, 0 },
    { 0x260a112a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98a112e, 55 },
    { 0xab0a112f, 88 },
    { 0, 0 },
    { 0xe05a1131, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56a1135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a1139, 77 },
    { 0xd4a113a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38a1141, 23 },
    { 0x260a1142, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaa1148, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a114b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1151, 87 },
    { 0, 0 },
    { 0x979a1153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a1158, 98 },
    { 0xa56a1158, 100 },
    { 0x233a1158, 33 },
    { 0, 0 },
    { 0xdc9a115c, 90 },
    { 0xe19a115d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60a1163, 84 },
    { 0, 0 },
    { 0x415a1165, 65 },
    { 0x13fa1166, 98 },
    { 0, 0 },
    { 0x11ca1168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a116d, 95 },
    { 0x901a116e, 90 },
    { 0xe1ba116f, 90 },
    { 0x8fa1170, 98 },
    { 0x843a1170, 34 },
    { 0x26ba1171, 64 },
    { 0xc8ba1172, 23 },
    { 0x66ea1174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca1177, 75 },
    { 0x95ea1178, 67 },
    { 0x6f2a1177, 23 },
    { 0xff4a1178, 85 },
    { 0, 0 },
    { 0x3f4a117c, 33 },
    { 0x4f2a117d, 34 },
    { 0x3d8a117e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6a1181, 75 },
    { 0, 0 },
    { 0xf06a1183, 98 },
    { 0xd22a1184, 33 },
    { 0, 0 },
    { 0xc06a1186, 23 },
    { 0x3fea1187, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ba118d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a1194, 61 },
    { 0, 0 },
    { 0x8aa1196, 90 },
    { 0, 0 },
    { 0x4bda1198, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a119b, 23 },
    { 0x88ba119b, 95 },
    { 0, 0 },
    { 0x27a119e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7a11aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa11ae, 61 },
    { 0x78ba11af, 55 },
    { 0x3b1a11b0, 90 },
    { 0x39ca11b1, 61 },
    { 0, 0 },
    { 0x113a11b3, 61 },
    { 0x61da11b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa11bd, 84 },
    { 0, 0 },
    { 0x803a11bf, 23 },
    { 0x152a11bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883a11c6, 95 },
    { 0, 0 },
    { 0xbc3a11c8, 61 },
    { 0x31aa11c9, 98 },
    { 0xe0ca11c9, 90 },
    { 0x141a11c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80a11d2, 100 },
    { 0x6c4a11d3, 95 },
    { 0, 0 },
    { 0x7b1a11d5, 43 },
    { 0, 0 },
    { 0x15ba11d7, 95 },
    { 0xe96a11d7, 87 },
    { 0x780a11d9, 69 },
    { 0x4f1a11da, 81 },
    { 0, 0 },
    { 0x93a11dc, 23 },
    { 0xb02a11dd, 98 },
    { 0x5f5a11dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x481a11e1, 84 },
    { 0x125a11e2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453a11e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a11e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa11ec, 95 },
    { 0, 0 },
    { 0x67ba11ee, 90 },
    { 0x58fa11ee, 33 },
    { 0xd31a11ef, 95 },
    { 0xa7aa11f1, 90 },
    { 0xc49a11f2, 90 },
    { 0x452a11f3, 68 },
    { 0x5c6a11f4, 93 },
    { 0, 0 },
    { 0xe6da11f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea11fc, 98 },
    { 0x724a11fd, 78 },
    { 0x12ba11fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7a1209, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a120c, 75 },
    { 0, 0 },
    { 0x66da120e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x539a1211, 68 },
    { 0x1d6a1212, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8da1215, 93 },
    { 0, 0 },
    { 0x7ba1217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa121b, 98 },
    { 0x36da121c, 91 },
    { 0xc2ba121d, 55 },
    { 0, 0 },
    { 0x8a8a121f, 52 },
    { 0xc36a121f, 61 },
    { 0, 0 },
    { 0xd03a1222, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13a122b, 19 },
    { 0x9e1a122c, 77 },
    { 0x697a122d, 90 },
    { 0x547a122c, 23 },
    { 0xc87a122f, 87 },
    { 0x46a1230, 23 },
    { 0xa0ca1231, 33 },
    { 0, 0 },
    { 0xc49a1233, 90 },
    { 0, 0 },
    { 0x3eda1235, 95 },
    { 0x852a1236, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea1239, 85 },
    { 0xaf5a1239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fa123f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46a1244, 34 },
    { 0x376a1245, 90 },
    { 0xffca1246, 95 },
    { 0x68aa1246, 33 },
    { 0xbd2a1247, 17 },
    { 0, 0 },
    { 0xaf0a124a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bea124e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a1251, 55 },
    { 0xed5a1251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a1256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa125d, 95 },
    { 0x3eda125e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa1262, 84 },
    { 0x67da1263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a1268, 90 },
    { 0xb17a1269, 66 },
    { 0x93a126a, 36 },
    { 0xe70a126a, 57 },
    { 0xca8a1268, 95 },
    { 0, 0 },
    { 0x4f4a126e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310a1273, 23 },
    { 0, 0 },
    { 0x9faa1275, 95 },
    { 0, 0 },
    { 0x6c9a1277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96a127b, 33 },
    { 0x9c1a127b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fea127f, 76 },
    { 0xd4ca1280, 55 },
    { 0x78da127f, 98 },
    { 0x6b6a1282, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a128b, 34 },
    { 0xdd8a128c, 80 },
    { 0, 0 },
    { 0x350a128e, 84 },
    { 0x694a128f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3a1296, 61 },
    { 0, 0 },
    { 0x6c0a1298, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7a129d, 98 },
    { 0x9a1a129e, 76 },
    { 0x12fa129f, 98 },
    { 0xcf6a129e, 90 },
    { 0xd5ca12a1, 87 },
    { 0, 0 },
    { 0x2b5a12a3, 33 },
    { 0, 0 },
    { 0x82ea12a5, 43 },
    { 0xe2a12a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a12a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a12ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea12bb, 61 },
    { 0xc5ca12bc, 17 },
    { 0xe96a12bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca12c0, 23 },
    { 0x77ba12c1, 61 },
    { 0x35ba12c2, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9a12c7, 33 },
    { 0xcbfa12c8, 34 },
    { 0xbd1a12c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a12cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a12d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aa12d7, 78 },
    { 0, 0 },
    { 0x65da12d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a12e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79a12ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4a12f3, 33 },
    { 0, 0 },
    { 0x84ca12f5, 66 },
    { 0x269a12f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba12f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a1301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeda1306, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aca1317, 90 },
    { 0xd35a1318, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0a131e, 78 },
    { 0x27aa131f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3a1322, 33 },
    { 0x989a1323, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fa1327, 17 },
    { 0, 0 },
    { 0xd73a1329, 55 },
    { 0xb73a1329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545a1331, 33 },
    { 0, 0 },
    { 0xff7a1333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2a1336, 23 },
    { 0xda6a1336, 36 },
    { 0x5cfa1336, 61 },
    { 0xcfba1338, 61 },
    { 0, 0 },
    { 0x41ba133b, 90 },
    { 0xfb4a133b, 61 },
    { 0, 0 },
    { 0x74ea133e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1346, 81 },
    { 0xd2ca1347, 34 },
    { 0xb7aa1346, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ba1352, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a135c, 43 },
    { 0, 0 },
    { 0x526a135e, 34 },
    { 0, 0 },
    { 0x523a1360, 95 },
    { 0x1bea1360, 30 },
    { 0, 0 },
    { 0xab7a1363, 95 },
    { 0xed2a1363, 99 },
    { 0x890a1365, 55 },
    { 0x8c5a1365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34a136c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7a1371, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba1375, 91 },
    { 0, 0 },
    { 0x473a1377, 66 },
    { 0xc7ea1378, 52 },
    { 0xc8ba1378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea137f, 61 },
    { 0xb96a1380, 68 },
    { 0xf5ca1381, 90 },
    { 0x638a1380, 23 },
    { 0xeeda1383, 97 },
    { 0xfeaa1384, 98 },
    { 0xd8ba1384, 55 },
    { 0x1a7a1386, 66 },
    { 0xd1aa1387, 55 },
    { 0x6b8a1386, 100 },
    { 0x4e0a1389, 98 },
    { 0x8eba1386, 34 },
    { 0xfdba1389, 84 },
    { 0, 0 },
    { 0x732a138d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x873a1390, 61 },
    { 0x71ba1390, 75 },
    { 0xcb1a1391, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2a1395, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89a1399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623a139d, 78 },
    { 0x397a139d, 46 },
    { 0xee0a139d, 90 },
    { 0x283a13a0, 80 },
    { 0xae4a13a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da13a4, 75 },
    { 0x76ea13a4, 81 },
    { 0, 0 },
    { 0x55aa13a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94a13aa, 46 },
    { 0x40fa13aa, 90 },
    { 0xbdea13ac, 61 },
    { 0x30da13ac, 76 },
    { 0xb71a13ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12a13b1, 33 },
    { 0x2b1a13b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a13bb, 34 },
    { 0, 0 },
    { 0xdb1a13bd, 61 },
    { 0xdd0a13bd, 90 },
    { 0xe0ea13bf, 98 },
    { 0, 0 },
    { 0x75ca13c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a13c4, 33 },
    { 0x5bfa13c4, 93 },
    { 0, 0 },
    { 0xc7fa13c7, 93 },
    { 0x667a13c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7a13da, 98 },
    { 0xf99a13db, 67 },
    { 0x95ba13dc, 76 },
    { 0xccda13dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a13e1, 43 },
    { 0xe40a13e2, 33 },
    { 0, 0 },
    { 0xd5aa13e4, 95 },
    { 0xd29a13e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13a13ea, 52 },
    { 0xb8fa13eb, 66 },
    { 0, 0 },
    { 0xd22a13ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ba13f5, 98 },
    { 0x2e7a13f6, 33 },
    { 0x830a13f6, 78 },
    { 0x482a13f8, 23 },
    { 0, 0 },
    { 0x1dba13fa, 98 },
    { 0, 0 },
    { 0x2b7a13fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefea1404, 95 },
    { 0, 0 },
    { 0x859a1406, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577a140b, 23 },
    { 0x81ea140c, 98 },
    { 0xd96a140b, 52 },
    { 0, 0 },
    { 0xe58a140f, 46 },
    { 0x71ea140f, 33 },
    { 0x81ba1411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afa1419, 23 },
    { 0, 0 },
    { 0x5c6a141b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a141e, 43 },
    { 0, 0 },
    { 0xeca1420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5a1423, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ca1427, 61 },
    { 0xa8aa1428, 99 },
    { 0, 0 },
    { 0x311a142a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0a1430, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8a1433, 78 },
    { 0, 0 },
    { 0x5b9a1435, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98a1438, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da143c, 34 },
    { 0xd93a143c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a1441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324a144c, 87 },
    { 0x7e2a144d, 76 },
    { 0x2fa144e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3a1454, 38 },
    { 0, 0 },
    { 0x178a1456, 98 },
    { 0, 0 },
    { 0xd1a1458, 33 },
    { 0x3e8a1459, 55 },
    { 0x54da145a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba145d, 55 },
    { 0xd79a145e, 33 },
    { 0x107a145d, 67 },
    { 0x5dea1460, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a1465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca146a, 46 },
    { 0, 0 },
    { 0xcfea146c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7a146f, 98 },
    { 0xb65a1470, 81 },
    { 0, 0 },
    { 0x22aa1472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ba1475, 52 },
    { 0xd0ea1475, 100 },
    { 0x11ca1476, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61a147e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa1481, 33 },
    { 0xfc4a1482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13a1486, 52 },
    { 0x198a1487, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x501a148a, 90 },
    { 0, 0 },
    { 0x3d9a148c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792a1496, 23 },
    { 0xba1497, 93 },
    { 0, 0 },
    { 0x24da1499, 52 },
    { 0xecaa149a, 98 },
    { 0xdfba149b, 55 },
    { 0x163a149c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257a14a0, 23 },
    { 0x482a14a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9a14a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x342a14ab, 19 },
    { 0, 0 },
    { 0x9cea14ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a14b1, 33 },
    { 0x69ba14b2, 55 },
    { 0xeb2a14b3, 90 },
    { 0xd3da14b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0aa14b7, 33 },
    { 0xbc7a14b7, 76 },
    { 0x872a14b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fa14c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fa14c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177a14d4, 23 },
    { 0x96ea14d4, 100 },
    { 0x2e7a14d6, 23 },
    { 0x405a14d6, 23 },
    { 0xf63a14d8, 75 },
    { 0x6c2a14d9, 34 },
    { 0x475a14d8, 67 },
    { 0x6dca14da, 90 },
    { 0x50da14dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aa14e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44da14e3, 52 },
    { 0x266a14e4, 61 },
    { 0xadca14e3, 43 },
    { 0, 0 },
    { 0x91ba14e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaa14eb, 95 },
    { 0xdb4a14ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a14f0, 43 },
    { 0x218a14f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a14f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa14f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51a14ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a1503, 91 },
    { 0, 0 },
    { 0xf5aa1505, 78 },
    { 0x3aba1505, 90 },
    { 0, 0 },
    { 0xd75a1508, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba150e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23a1511, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca1514, 95 },
    { 0x1c4a1515, 23 },
    { 0xb1ca1516, 99 },
    { 0x2c2a1517, 55 },
    { 0x43ca1518, 98 },
    { 0, 0 },
    { 0x44ca151a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2a151e, 95 },
    { 0xcaca151f, 99 },
    { 0xbaca1520, 87 },
    { 0x9aaa1521, 91 },
    { 0x9f3a1521, 33 },
    { 0x458a1520, 34 },
    { 0, 0 },
    { 0x13ba1525, 95 },
    { 0x3e3a1526, 55 },
    { 0, 0 },
    { 0x874a1528, 98 },
    { 0x523a1529, 23 },
    { 0x407a1529, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfa1538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ca153b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9a153e, 55 },
    { 0xfb0a153e, 90 },
    { 0x32ba1540, 61 },
    { 0x64ba1540, 98 },
    { 0x824a1540, 84 },
    { 0xd5ba1543, 100 },
    { 0x884a1544, 98 },
    { 0xd67a1544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a1548, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ea154b, 90 },
    { 0x5a4a154c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea154f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422a1553, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8a1558, 55 },
    { 0xff8a1559, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aa155d, 99 },
    { 0xd62a155e, 61 },
    { 0, 0 },
    { 0x819a1560, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859a1564, 61 },
    { 0xf29a1565, 90 },
    { 0xa85a1566, 33 },
    { 0xb2ea1564, 92 },
    { 0xa75a1568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca156b, 87 },
    { 0xbd4a156c, 17 },
    { 0xfa2a156c, 33 },
    { 0xd00a156b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34a1572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a1575, 33 },
    { 0x66fa1576, 95 },
    { 0, 0 },
    { 0x1efa1578, 90 },
    { 0xc73a1579, 90 },
    { 0x7b7a157a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ca157e, 66 },
    { 0x220a157f, 33 },
    { 0x972a157f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa1584, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ea1588, 90 },
    { 0xf48a1589, 61 },
    { 0x57a158a, 90 },
    { 0xeaa1588, 91 },
    { 0, 0 },
    { 0x2baa158d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a1592, 52 },
    { 0x312a1592, 33 },
    { 0x5bba1594, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a1598, 61 },
    { 0xd2da1599, 66 },
    { 0, 0 },
    { 0xa82a159b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269a15a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca15b3, 61 },
    { 0x195a15b4, 23 },
    { 0x854a15b5, 61 },
    { 0xd41a15b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x805a15b9, 61 },
    { 0xaf9a15ba, 66 },
    { 0x2e3a15bb, 61 },
    { 0xd74a15bb, 61 },
    { 0x434a15bd, 99 },
    { 0xfb4a15bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x594a15c1, 76 },
    { 0x64fa15c2, 43 },
    { 0, 0 },
    { 0xe8ea15c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a15cc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a15d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158a15db, 68 },
    { 0x2e5a15dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6da15e0, 23 },
    { 0xdfba15e1, 33 },
    { 0x8d8a15e2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9a15e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3a15e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca15ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a15ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33a15f5, 67 },
    { 0xe8ba15f5, 55 },
    { 0x384a15f7, 66 },
    { 0xcb7a15f8, 55 },
    { 0, 0 },
    { 0x81aa15fa, 90 },
    { 0, 0 },
    { 0x4c3a15fc, 87 },
    { 0x700a15fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49a1600, 61 },
    { 0xb37a1601, 55 },
    { 0, 0 },
    { 0xceca1603, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa1606, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a160e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa1612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a1616, 96 },
    { 0xd1ba1617, 61 },
    { 0, 0 },
    { 0x1bca1619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47a161e, 33 },
    { 0x51da161e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x132a1622, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a1625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a162e, 95 },
    { 0, 0 },
    { 0x400a1630, 61 },
    { 0x4f2a1630, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddea1636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a163b, 23 },
    { 0x5cea163c, 33 },
    { 0, 0 },
    { 0xbcaa163e, 61 },
    { 0xcaea163f, 23 },
    { 0x400a1640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefa164c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1a164f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2a165a, 67 },
    { 0, 0 },
    { 0xd1da165c, 98 },
    { 0x3d6a165c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a1660, 61 },
    { 0, 0 },
    { 0x5fea1662, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a1665, 93 },
    { 0, 0 },
    { 0x1c4a1667, 68 },
    { 0xe5fa1667, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba166b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca166f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542a1673, 43 },
    { 0xb04a1674, 30 },
    { 0xeb5a1673, 95 },
    { 0, 0 },
    { 0x81ba1677, 44 },
    { 0x318a1677, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x681a167b, 23 },
    { 0, 0 },
    { 0xd15a167d, 23 },
    { 0x85ca167e, 23 },
    { 0xd25a167f, 46 },
    { 0x9ba1680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989a1685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa168a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea168d, 61 },
    { 0xc30a168d, 93 },
    { 0xb5ea168d, 33 },
    { 0, 0 },
    { 0xe4ba1691, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a1694, 61 },
    { 0xec1a1695, 55 },
    { 0xaba1695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a169a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a16a0, 76 },
    { 0xfc3a16a1, 83 },
    { 0xfe7a16a2, 98 },
    { 0, 0 },
    { 0xf4da16a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafa16a7, 90 },
    { 0xde7a16a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaea16ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a16b7, 90 },
    { 0xf83a16b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea16bc, 76 },
    { 0x2d3a16bd, 95 },
    { 0x7e2a16be, 76 },
    { 0x531a16be, 98 },
    { 0xaaa16be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a16c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1a16cc, 77 },
    { 0xa0fa16cd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a16d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a16db, 61 },
    { 0, 0 },
    { 0xf38a16dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a16e5, 98 },
    { 0xb2ba16e6, 61 },
    { 0xddba16e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8a16eb, 76 },
    { 0, 0 },
    { 0x262a16ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9a16f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a16fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a16ff, 23 },
    { 0xbfca1700, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a1704, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a170a, 90 },
    { 0x73ba170b, 90 },
    { 0x134a170c, 98 },
    { 0x49ca170d, 43 },
    { 0x9f5a170b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ba1714, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a1719, 95 },
    { 0xaeca171a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea171d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a1724, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6a1729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ea172f, 98 },
    { 0xe23a1730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ca1734, 61 },
    { 0x431a1734, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a173c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5a1740, 88 },
    { 0xbfa1741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aa1745, 55 },
    { 0xadda1745, 68 },
    { 0xaafa1747, 34 },
    { 0x17ea1748, 92 },
    { 0, 0 },
    { 0xe73a174a, 98 },
    { 0x145a174b, 23 },
    { 0x98ba174b, 80 },
    { 0xa8fa174d, 56 },
    { 0x209a174c, 90 },
    { 0x664a174f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa1752, 99 },
    { 0xeaa1753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926a1757, 77 },
    { 0x6d9a1758, 61 },
    { 0, 0 },
    { 0xf9ea175a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a175f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a1762, 90 },
    { 0, 0 },
    { 0xd6a1764, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a176d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea1770, 61 },
    { 0, 0 },
    { 0x432a1772, 88 },
    { 0x4c3a1773, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a177b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a1787, 90 },
    { 0x8f9a1788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a178d, 90 },
    { 0, 0 },
    { 0x82ca178f, 34 },
    { 0x61fa1790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9a1793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba1796, 76 },
    { 0, 0 },
    { 0xc82a1798, 93 },
    { 0xf66a1798, 43 },
    { 0xd42a179a, 77 },
    { 0, 0 },
    { 0xc93a179c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdda17a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ea17a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40a17aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a17ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a17b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca17b6, 81 },
    { 0x97a17b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8a17bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a17c0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a17c5, 33 },
    { 0x33da17c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a17cf, 90 },
    { 0, 0 },
    { 0xfd6a17d1, 99 },
    { 0, 0 },
    { 0x4b3a17d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba17d6, 95 },
    { 0x43a17d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9a17dc, 55 },
    { 0xf5fa17dd, 61 },
    { 0x9b1a17de, 23 },
    { 0xe32a17df, 91 },
    { 0x5bea17e0, 98 },
    { 0x11ca17e1, 90 },
    { 0xfe1a17e2, 90 },
    { 0x67da17e1, 90 },
    { 0xda5a17df, 77 },
    { 0, 0 },
    { 0x9f5a17e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x395a17e9, 61 },
    { 0x27ea17ea, 66 },
    { 0x1f3a17ea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9a17ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ba17f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66a17f5, 81 },
    { 0xe6ca17f6, 75 },
    { 0x72a17f7, 98 },
    { 0x9dda17f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1a17fb, 95 },
    { 0, 0 },
    { 0xd1a17fd, 55 },
    { 0xd1aa17fd, 55 },
    { 0x306a17fe, 90 },
    { 0xe12a17fe, 93 },
    { 0x395a1801, 33 },
    { 0x777a1802, 33 },
    { 0xa5fa1801, 84 },
    { 0x773a1803, 98 },
    { 0x2bca1805, 78 },
    { 0x22da1802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a1809, 90 },
    { 0, 0 },
    { 0xf48a180b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba180f, 34 },
    { 0xfa2a1810, 65 },
    { 0x946a1810, 91 },
    { 0xf13a1811, 34 },
    { 0x1d2a1813, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa1816, 43 },
    { 0xffba1817, 61 },
    { 0, 0 },
    { 0xa9ea1819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aa181f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcda1829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ca1831, 61 },
    { 0, 0 },
    { 0x9e0a1833, 87 },
    { 0x61fa1834, 36 },
    { 0x286a1833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a1839, 90 },
    { 0, 0 },
    { 0x935a183b, 90 },
    { 0xd9ba183c, 43 },
    { 0x901a183b, 68 },
    { 0x9afa183c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a1844, 90 },
    { 0xb91a1844, 95 },
    { 0x910a1846, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91a1849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbca184d, 43 },
    { 0x65ca184e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a1852, 95 },
    { 0x9b1a1853, 33 },
    { 0xdb0a1854, 23 },
    { 0xba4a1855, 61 },
    { 0x641a1853, 33 },
    { 0xbaaa1857, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6a185b, 85 },
    { 0xd8ea185c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7a185f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aa1866, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaa1872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984a1876, 46 },
    { 0, 0 },
    { 0x302a1878, 93 },
    { 0xe47a1878, 38 },
    { 0x975a187a, 98 },
    { 0xeeba187b, 95 },
    { 0, 0 },
    { 0x73da187d, 61 },
    { 0, 0 },
    { 0xeb6a187f, 33 },
    { 0x699a187f, 34 },
    { 0xe16a1881, 52 },
    { 0x63da187f, 17 },
    { 0x671a1880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa1886, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca188a, 76 },
    { 0x825a188b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0a1893, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bca189b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa189e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a18a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa18ad, 34 },
    { 0, 0 },
    { 0xabfa18af, 95 },
    { 0xd09a18b0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a18b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955a18c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeba18c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967a18cd, 95 },
    { 0, 0 },
    { 0x44ba18cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7a18d5, 52 },
    { 0, 0 },
    { 0x505a18d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea18dc, 90 },
    { 0xe4da18dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3a18e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704a18e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71da18e9, 98 },
    { 0xe2a18e9, 61 },
    { 0xf8ea18eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a18f1, 90 },
    { 0x2f9a18f2, 22 },
    { 0, 0 },
    { 0xd27a18f4, 90 },
    { 0x996a18f5, 43 },
    { 0x8b7a18f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a18f9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfa18fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a1906, 66 },
    { 0, 0 },
    { 0xfa3a1908, 23 },
    { 0x8c2a1908, 76 },
    { 0xe63a190a, 23 },
    { 0x366a190b, 61 },
    { 0xef5a190c, 90 },
    { 0x926a190d, 90 },
    { 0xf00a190e, 43 },
    { 0xd5ea190e, 90 },
    { 0xd12a1910, 61 },
    { 0x1f5a190f, 99 },
    { 0x8d3a190d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70a1916, 91 },
    { 0x951a1916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x985a191a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9a1920, 70 },
    { 0x21ea1921, 61 },
    { 0, 0 },
    { 0x540a1923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a1928, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a192b, 75 },
    { 0, 0 },
    { 0x70fa192d, 61 },
    { 0x7a0a192e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0a1931, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ea1935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaa193d, 33 },
    { 0xf98a193e, 33 },
    { 0, 0 },
    { 0xd85a1940, 90 },
    { 0xd95a1940, 34 },
    { 0, 0 },
    { 0x2fa1943, 95 },
    { 0x391a1943, 33 },
    { 0xcc2a1943, 33 },
    { 0x824a1946, 98 },
    { 0x460a1946, 93 },
    { 0x8e6a1944, 99 },
    { 0x3b7a1949, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x27da194c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ada1952, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1957, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa195b, 76 },
    { 0xa8ea195c, 85 },
    { 0x5d8a195b, 58 },
    { 0x470a195e, 78 },
    { 0xfe5a195f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a1962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a1969, 97 },
    { 0, 0 },
    { 0x6dfa196b, 98 },
    { 0x3bea196c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fa1970, 98 },
    { 0x4bfa1971, 55 },
    { 0, 0 },
    { 0x7a7a1973, 23 },
    { 0, 0 },
    { 0x794a1975, 76 },
    { 0x499a1976, 61 },
    { 0x484a1977, 76 },
    { 0xf3aa1977, 98 },
    { 0, 0 },
    { 0xb95a197a, 33 },
    { 0x9f6a197b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164a197f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa1982, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a1988, 90 },
    { 0x352a1989, 85 },
    { 0x377a198a, 34 },
    { 0, 0 },
    { 0x6cea198c, 76 },
    { 0x911a198c, 33 },
    { 0x3d6a198d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aa1996, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a199b, 23 },
    { 0x6cba199b, 98 },
    { 0, 0 },
    { 0xf7ca199e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7a19a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aba19a7, 33 },
    { 0xaf1a19a8, 75 },
    { 0, 0 },
    { 0x9cea19aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2a19ad, 55 },
    { 0, 0 },
    { 0x331a19af, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa19b3, 33 },
    { 0, 0 },
    { 0x3c8a19b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda19b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a19c0, 61 },
    { 0, 0 },
    { 0xefca19c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527a19c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a19cd, 95 },
    { 0x899a19ce, 40 },
    { 0x982a19ce, 76 },
    { 0x810a19d0, 23 },
    { 0x30ba19cd, 95 },
    { 0x412a19d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x653a19d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda19dd, 85 },
    { 0x5a2a19de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73da19e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x258a19e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea19e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a19ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da19ef, 61 },
    { 0xddba19f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a19f4, 66 },
    { 0x3d2a19f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326a19f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea19fc, 90 },
    { 0, 0 },
    { 0x742a19fe, 61 },
    { 0xc89a19ff, 76 },
    { 0, 0 },
    { 0xdaca1a01, 95 },
    { 0x9f2a1a02, 95 },
    { 0x352a1a01, 55 },
    { 0xb58a1a04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9a1a07, 55 },
    { 0x66ea1a08, 46 },
    { 0, 0 },
    { 0xfdaa1a0a, 76 },
    { 0, 0 },
    { 0xbf4a1a0c, 98 },
    { 0xd66a1a0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da1a22, 55 },
    { 0x547a1a23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ca1a26, 61 },
    { 0x6da1a26, 95 },
    { 0x82a1a27, 61 },
    { 0, 0 },
    { 0xf54a1a2a, 55 },
    { 0xe7ca1a2b, 98 },
    { 0xf3a1a2a, 61 },
    { 0x73ba1a2d, 33 },
    { 0x855a1a2e, 61 },
    { 0, 0 },
    { 0x6f7a1a30, 98 },
    { 0xe83a1a31, 95 },
    { 0x355a1a32, 88 },
    { 0x175a1a32, 77 },
    { 0, 0 },
    { 0xd81a1a35, 99 },
    { 0x3caa1a36, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a1a3a, 96 },
    { 0xecfa1a3b, 98 },
    { 0x756a1a3c, 68 },
    { 0x9fba1a3d, 61 },
    { 0x492a1a3c, 33 },
    { 0, 0 },
    { 0x706a1a40, 99 },
    { 0x3f8a1a40, 23 },
    { 0xcb0a1a40, 90 },
    { 0xd6fa1a43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a1a46, 95 },
    { 0, 0 },
    { 0x274a1a48, 78 },
    { 0x95ca1a49, 93 },
    { 0x490a1a4a, 90 },
    { 0x7aea1a4a, 46 },
    { 0xa86a1a4c, 34 },
    { 0xecfa1a4d, 23 },
    { 0x187a1a4e, 66 },
    { 0xa74a1a4c, 55 },
    { 0, 0 },
    { 0x952a1a51, 43 },
    { 0x84ea1a52, 83 },
    { 0xdada1a52, 70 },
    { 0, 0 },
    { 0x82aa1a55, 98 },
    { 0xf1ca1a56, 33 },
    { 0xad2a1a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2a1a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811a1a67, 28 },
    { 0x32a1a68, 40 },
    { 0, 0 },
    { 0xb67a1a6a, 85 },
    { 0x2a5a1a6a, 90 },
    { 0x68fa1a6c, 34 },
    { 0, 0 },
    { 0x30ea1a6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21a1a71, 88 },
    { 0x3d6a1a72, 52 },
    { 0x146a1a73, 88 },
    { 0xb20a1a74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a1a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4a1a80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a1a83, 91 },
    { 0xc68a1a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ea1a92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a1a97, 90 },
    { 0x24fa1a98, 66 },
    { 0xd01a1a98, 90 },
    { 0x4a3a1a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x348a1a9d, 33 },
    { 0x829a1a9e, 80 },
    { 0, 0 },
    { 0x34ea1aa0, 61 },
    { 0, 0 },
    { 0x208a1aa2, 98 },
    { 0, 0 },
    { 0xc7ea1aa4, 99 },
    { 0x7c0a1aa5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a1ab9, 17 },
    { 0, 0 },
    { 0x221a1abb, 99 },
    { 0xb9aa1abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a1ac0, 98 },
    { 0xce2a1ac1, 98 },
    { 0xc32a1ac1, 46 },
    { 0, 0 },
    { 0xe00a1ac4, 76 },
    { 0x625a1ac4, 90 },
    { 0xed8a1ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbea1acc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa1ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x879a1ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1adc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ca1adf, 90 },
    { 0xdada1ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279a1ae5, 33 },
    { 0xf03a1ae6, 55 },
    { 0xcd8a1ae5, 33 },
    { 0x31ba1ae5, 90 },
    { 0x71aa1ae9, 17 },
    { 0x135a1aea, 98 },
    { 0xdb3a1aea, 80 },
    { 0x156a1aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbba1af0, 99 },
    { 0x60ea1af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca1af6, 33 },
    { 0, 0 },
    { 0x1f3a1af8, 23 },
    { 0x6dda1af9, 95 },
    { 0, 0 },
    { 0x273a1afb, 76 },
    { 0x6e6a1afb, 33 },
    { 0xea4a1afd, 84 },
    { 0xbd4a1afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533a1b04, 84 },
    { 0x6f2a1b05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a1b08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a1b0b, 90 },
    { 0xccaa1b0c, 100 },
    { 0, 0 },
    { 0xf95a1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19a1b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33a1b16, 61 },
    { 0xcc2a1b17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03a1b1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca1b24, 93 },
    { 0, 0 },
    { 0x5d5a1b26, 95 },
    { 0xa66a1b27, 52 },
    { 0x128a1b28, 23 },
    { 0, 0 },
    { 0xea5a1b2a, 34 },
    { 0x924a1b2b, 34 },
    { 0x6ea1b2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a1b30, 19 },
    { 0x50da1b30, 34 },
    { 0, 0 },
    { 0x68aa1b33, 95 },
    { 0, 0 },
    { 0x4aa1b35, 98 },
    { 0, 0 },
    { 0xd3aa1b37, 34 },
    { 0, 0 },
    { 0x78fa1b39, 85 },
    { 0, 0 },
    { 0xf70a1b3b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ea1b3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a1b41, 90 },
    { 0x109a1b42, 67 },
    { 0xd6a1b42, 52 },
    { 0xcd7a1b42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a1b47, 98 },
    { 0x343a1b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ea1b4c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddca1b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda1b58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a1b5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a1b66, 23 },
    { 0x375a1b67, 55 },
    { 0x403a1b67, 58 },
    { 0, 0 },
    { 0x4d1a1b6a, 61 },
    { 0, 0 },
    { 0x1da1b6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a1b72, 67 },
    { 0x2a5a1b73, 36 },
    { 0x81ca1b72, 95 },
    { 0x570a1b73, 82 },
    { 0x261a1b76, 98 },
    { 0x47fa1b77, 100 },
    { 0x2e7a1b78, 99 },
    { 0x294a1b78, 90 },
    { 0, 0 },
    { 0x80fa1b7b, 61 },
    { 0, 0 },
    { 0x9dda1b7d, 23 },
    { 0xec9a1b7e, 46 },
    { 0x451a1b7e, 61 },
    { 0xe79a1b7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba1b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72a1b8c, 90 },
    { 0x6eda1b8d, 98 },
    { 0xe22a1b8c, 33 },
    { 0x8e9a1b8e, 90 },
    { 0xdd6a1b90, 46 },
    { 0x9eaa1b91, 95 },
    { 0x186a1b90, 33 },
    { 0x1b9a1b93, 88 },
    { 0x5c3a1b90, 98 },
    { 0x7c8a1b95, 33 },
    { 0x9aea1b95, 90 },
    { 0xc47a1b8e, 98 },
    { 0xb10a1b8d, 66 },
    { 0, 0 },
    { 0x37a1b9a, 91 },
    { 0xbf1a1b9b, 51 },
    { 0x145a1b9b, 17 },
    { 0x4daa1b9d, 95 },
    { 0, 0 },
    { 0x8a9a1b9f, 8 },
    { 0xe2fa1ba0, 61 },
    { 0xed4a1ba1, 61 },
    { 0, 0 },
    { 0x620a1ba3, 99 },
    { 0xec7a1ba3, 52 },
    { 0x7eba1ba5, 90 },
    { 0xac2a1ba4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ca1ba9, 33 },
    { 0x23a1baa, 90 },
    { 0, 0 },
    { 0xbfba1bac, 61 },
    { 0x46ca1bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba1bb0, 60 },
    { 0, 0 },
    { 0xc3fa1bb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a1bb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca1bbc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a1bbf, 33 },
    { 0, 0 },
    { 0x31ba1bc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa1bcd, 34 },
    { 0xcb1a1bce, 43 },
    { 0xb72a1bcf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a1bd3, 76 },
    { 0, 0 },
    { 0xe0ba1bd5, 80 },
    { 0, 0 },
    { 0x10a1bd7, 19 },
    { 0xb7a1bd8, 23 },
    { 0x641a1bd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6a1bdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da1be1, 90 },
    { 0x242a1be2, 66 },
    { 0xd73a1be3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877a1bea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca1bed, 33 },
    { 0, 0 },
    { 0xdfaa1bef, 52 },
    { 0, 0 },
    { 0x79ea1bf1, 46 },
    { 0, 0 },
    { 0xb24a1bf3, 22 },
    { 0xb09a1bf3, 61 },
    { 0x332a1bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a1bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a1bfe, 46 },
    { 0x52fa1bff, 66 },
    { 0x3c4a1c00, 95 },
    { 0, 0 },
    { 0xa51a1c02, 46 },
    { 0x8a0a1c03, 33 },
    { 0, 0 },
    { 0xf1ea1c05, 17 },
    { 0x658a1c05, 90 },
    { 0x1e8a1c06, 98 },
    { 0xc3fa1c08, 34 },
    { 0xde5a1c08, 90 },
    { 0x228a1c09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a1c0d, 75 },
    { 0, 0 },
    { 0x1a6a1c0f, 100 },
    { 0xb82a1c10, 91 },
    { 0xc69a1c10, 78 },
    { 0x6aaa1c12, 23 },
    { 0xfa3a1c0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91da1c16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfa1c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a1c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a1c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874a1c2a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dca1c2e, 23 },
    { 0, 0 },
    { 0xa2fa1c30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cba1c3a, 90 },
    { 0x33fa1c3b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea1c3e, 33 },
    { 0x39ba1c3e, 61 },
    { 0xe8da1c40, 90 },
    { 0xf95a1c41, 61 },
    { 0x6f7a1c42, 33 },
    { 0x15ca1c42, 66 },
    { 0x798a1c43, 33 },
    { 0xfa0a1c41, 98 },
    { 0xc3da1c46, 98 },
    { 0x754a1c42, 61 },
    { 0x393a1c48, 55 },
    { 0x82a1c49, 98 },
    { 0, 0 },
    { 0x810a1c4b, 90 },
    { 0x76fa1c4c, 34 },
    { 0, 0 },
    { 0xad0a1c4e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fa1c52, 61 },
    { 0x430a1c52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x213a1c56, 90 },
    { 0x7e2a1c57, 34 },
    { 0xb5a1c57, 87 },
    { 0x643a1c59, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afa1c5f, 76 },
    { 0xa2ba1c5f, 43 },
    { 0x509a1c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a1c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a1c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1a1c6c, 97 },
    { 0x91ea1c6d, 23 },
    { 0xe21a1c6e, 87 },
    { 0x4d9a1c6e, 61 },
    { 0x792a1c6c, 90 },
    { 0xf5fa1c71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aa1c75, 43 },
    { 0x304a1c76, 77 },
    { 0x3d9a1c76, 90 },
    { 0xea1a1c77, 17 },
    { 0x8a1c79, 99 },
    { 0xc65a1c7a, 85 },
    { 0xa7fa1c75, 33 },
    { 0xc10a1c7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74a1c80, 98 },
    { 0xb04a1c81, 46 },
    { 0xfe5a1c81, 43 },
    { 0x689a1c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1c86, 90 },
    { 0xf6ea1c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a1c8b, 33 },
    { 0, 0 },
    { 0xfb5a1c8d, 33 },
    { 0x2fca1c8e, 85 },
    { 0x9d6a1c8f, 91 },
    { 0x758a1c8d, 95 },
    { 0, 0 },
    { 0x45da1c92, 99 },
    { 0x8aa1c93, 78 },
    { 0, 0 },
    { 0xfeda1c95, 52 },
    { 0xfd9a1c95, 90 },
    { 0x41ca1c97, 33 },
    { 0, 0 },
    { 0x61ea1c99, 33 },
    { 0xa15a1c9a, 34 },
    { 0, 0 },
    { 0x364a1c9c, 90 },
    { 0, 0 },
    { 0x500a1c9e, 91 },
    { 0x6aea1c9f, 46 },
    { 0x304a1c9f, 93 },
    { 0xe5ba1ca1, 76 },
    { 0xfc0a1c9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890a1ca9, 55 },
    { 0x7efa1ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0a1cae, 23 },
    { 0x402a1caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ca1cb2, 23 },
    { 0xe5ea1cb3, 91 },
    { 0x506a1cb4, 23 },
    { 0xc5ca1cb4, 61 },
    { 0x341a1cb4, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca1cbc, 76 },
    { 0, 0 },
    { 0x888a1cbe, 61 },
    { 0, 0 },
    { 0x32ea1cc0, 61 },
    { 0x2aba1cc1, 61 },
    { 0xd39a1cc2, 23 },
    { 0x4e6a1cc3, 61 },
    { 0x1dba1cc0, 76 },
    { 0x6b8a1cc5, 87 },
    { 0x36aa1cc5, 75 },
    { 0xff9a1cc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7a1cca, 33 },
    { 0x210a1ccb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742a1cd4, 90 },
    { 0x6b5a1cd5, 90 },
    { 0, 0 },
    { 0x977a1cd7, 33 },
    { 0xb0a1cd8, 33 },
    { 0, 0 },
    { 0x86a1cda, 23 },
    { 0, 0 },
    { 0x7bba1cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a1ce0, 87 },
    { 0, 0 },
    { 0x29ca1ce2, 61 },
    { 0x7b4a1ce3, 95 },
    { 0x4b2a1ce4, 81 },
    { 0xfe1a1ce2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x204a1ce8, 23 },
    { 0, 0 },
    { 0x357a1cea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7a1cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2a1cf4, 43 },
    { 0, 0 },
    { 0x67ca1cf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea1cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821a1cff, 100 },
    { 0x6f1a1d00, 34 },
    { 0xf46a1d01, 93 },
    { 0x3cfa1d01, 90 },
    { 0x25ba1d03, 90 },
    { 0x36da1d03, 93 },
    { 0, 0 },
    { 0x5e9a1d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a1d0b, 46 },
    { 0x999a1d0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a1d0f, 46 },
    { 0, 0 },
    { 0xdd6a1d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a1d15, 23 },
    { 0xfd5a1d15, 61 },
    { 0x3bca1d15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43a1d1a, 90 },
    { 0x829a1d1b, 43 },
    { 0x219a1d1c, 46 },
    { 0x110a1d1c, 90 },
    { 0x468a1d1e, 100 },
    { 0x5c2a1d1f, 88 },
    { 0x6eda1d1d, 61 },
    { 0xd31a1d1d, 95 },
    { 0x53ca1d21, 95 },
    { 0xbaaa1d1a, 98 },
    { 0x75ca1d24, 61 },
    { 0, 0 },
    { 0xec5a1d26, 33 },
    { 0, 0 },
    { 0xa39a1d28, 43 },
    { 0x6d3a1d29, 93 },
    { 0xdf0a1d28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a1d2e, 55 },
    { 0x271a1d2f, 95 },
    { 0xbc4a1d2f, 34 },
    { 0xf5aa1d30, 95 },
    { 0, 0 },
    { 0x5dba1d33, 100 },
    { 0, 0 },
    { 0x10ca1d35, 61 },
    { 0, 0 },
    { 0x2e4a1d37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294a1d3f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bea1d44, 93 },
    { 0x387a1d44, 76 },
    { 0, 0 },
    { 0xb63a1d47, 95 },
    { 0xf89a1d48, 98 },
    { 0, 0 },
    { 0x45ba1d4a, 61 },
    { 0xe19a1d4b, 91 },
    { 0x952a1d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ba1d51, 76 },
    { 0xe51a1d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a1d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x902a1d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea1d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580a1d62, 43 },
    { 0x13da1d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a1d69, 33 },
    { 0xe3aa1d6a, 90 },
    { 0, 0 },
    { 0x69fa1d6c, 61 },
    { 0, 0 },
    { 0xb3fa1d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x801a1d71, 76 },
    { 0xae5a1d72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea1d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36a1d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a1d88, 93 },
    { 0x87ba1d89, 61 },
    { 0, 0 },
    { 0xf0ba1d8b, 43 },
    { 0xaaea1d8c, 90 },
    { 0x20ca1d8c, 61 },
    { 0x5aea1d8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da1d91, 53 },
    { 0x839a1d91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a1d95, 43 },
    { 0x56da1d96, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cba1d99, 95 },
    { 0xbbfa1d9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a1d9d, 85 },
    { 0x3f2a1d9e, 90 },
    { 0xda3a1d9e, 98 },
    { 0xe87a1da0, 17 },
    { 0xdd1a1da1, 85 },
    { 0xd89a1da2, 98 },
    { 0x172a1da1, 75 },
    { 0, 0 },
    { 0x2f4a1da5, 33 },
    { 0x61aa1da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x352a1da9, 88 },
    { 0x7a2a1daa, 43 },
    { 0x82aa1dab, 77 },
    { 0x377a1dab, 46 },
    { 0xfc1a1dad, 98 },
    { 0, 0 },
    { 0xfc8a1daf, 46 },
    { 0xccea1db0, 23 },
    { 0xf72a1db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa1db8, 95 },
    { 0x2f1a1db9, 95 },
    { 0xd31a1db8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0a1dbe, 95 },
    { 0xca2a1dbf, 67 },
    { 0x221a1dc0, 61 },
    { 0, 0 },
    { 0x455a1dc2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa1dc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a1dcc, 98 },
    { 0x744a1dcd, 98 },
    { 0x882a1dce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a1dd1, 61 },
    { 0x39da1dd2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ba1dd6, 23 },
    { 0x105a1dd7, 61 },
    { 0x22ca1dd6, 93 },
    { 0, 0 },
    { 0x946a1dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea1dde, 23 },
    { 0x1a8a1dde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea1de2, 23 },
    { 0xb54a1de2, 91 },
    { 0x414a1de2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa1dec, 55 },
    { 0x3c0a1dec, 75 },
    { 0, 0 },
    { 0x9cea1def, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a1df2, 90 },
    { 0x963a1df2, 98 },
    { 0, 0 },
    { 0xb1a1df5, 87 },
    { 0x3d1a1df6, 23 },
    { 0x8e7a1df5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aa1dff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a1e02, 98 },
    { 0xddea1e03, 67 },
    { 0x1d2a1e03, 95 },
    { 0, 0 },
    { 0xaeca1e06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0a1e0a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69a1e12, 90 },
    { 0x403a1e12, 67 },
    { 0xa00a1e14, 23 },
    { 0x4cba1e12, 91 },
    { 0x95da1e16, 33 },
    { 0x822a1e16, 85 },
    { 0xf2fa1e17, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a1e1b, 33 },
    { 0xc0aa1e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba1e20, 76 },
    { 0x9e5a1e21, 23 },
    { 0x95ba1e21, 93 },
    { 0x8f5a1e22, 90 },
    { 0x7c8a1e20, 33 },
    { 0x21ca1e22, 94 },
    { 0x9bba1e26, 61 },
    { 0x41aa1e23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7a1e2d, 61 },
    { 0, 0 },
    { 0x610a1e2f, 61 },
    { 0x2dca1e30, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ea1e33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a1e39, 95 },
    { 0, 0 },
    { 0x4b4a1e3b, 94 },
    { 0xcf0a1e3c, 66 },
    { 0x40ea1e3c, 61 },
    { 0xcfda1e3c, 94 },
    { 0x672a1e3f, 78 },
    { 0xd6aa1e3f, 90 },
    { 0x45a1e3f, 94 },
    { 0x8e8a1e42, 93 },
    { 0x96ca1e42, 66 },
    { 0x337a1e44, 98 },
    { 0x119a1e3f, 85 },
    { 0x5f9a1e3e, 51 },
    { 0, 0 },
    { 0x372a1e48, 33 },
    { 0, 0 },
    { 0xf84a1e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a1e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x731a1e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ea1e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3a1e58, 61 },
    { 0x85aa1e59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2a1e64, 61 },
    { 0xaa4a1e65, 90 },
    { 0x2d6a1e66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41a1e69, 61 },
    { 0xb58a1e6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ea1e6f, 98 },
    { 0xac9a1e6f, 23 },
    { 0x70fa1e6f, 23 },
    { 0xd2ba1e72, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a1e76, 90 },
    { 0xe10a1e76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a1e7a, 52 },
    { 0, 0 },
    { 0x36ba1e7c, 93 },
    { 0xd70a1e7c, 61 },
    { 0x82a1e7e, 90 },
    { 0x1d9a1e7f, 98 },
    { 0xca2a1e7f, 84 },
    { 0x721a1e81, 55 },
    { 0xe2ea1e82, 93 },
    { 0x8dfa1e7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a1e87, 90 },
    { 0xfcca1e87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2a1e8c, 61 },
    { 0x923a1e8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa1e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x146a1e94, 46 },
    { 0x639a1e94, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ba1e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ea1e9e, 55 },
    { 0xd6ca1e9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea1ea4, 84 },
    { 0xfcaa1ea4, 87 },
    { 0xa55a1ea6, 99 },
    { 0x9faa1ea7, 36 },
    { 0x6c1a1ea7, 76 },
    { 0xa6fa1ea7, 75 },
    { 0x1aaa1eaa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02a1eae, 66 },
    { 0x68a1eae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a1eb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba1eb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a1eb9, 61 },
    { 0, 0 },
    { 0xb3ea1ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613a1ebf, 90 },
    { 0x924a1ebf, 44 },
    { 0x90a1ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a1ec8, 78 },
    { 0, 0 },
    { 0x5dba1eca, 23 },
    { 0xf97a1eca, 95 },
    { 0x61fa1ecc, 98 },
    { 0x1d5a1ecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eca1ed4, 84 },
    { 0, 0 },
    { 0x6b4a1ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a1ed9, 78 },
    { 0, 0 },
    { 0xe07a1edb, 90 },
    { 0x3b1a1edb, 33 },
    { 0xd8a1edc, 84 },
    { 0x696a1edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaa1ee2, 85 },
    { 0x41ca1ee3, 95 },
    { 0xb46a1ee4, 61 },
    { 0xab1a1ee4, 98 },
    { 0x997a1ee4, 90 },
    { 0xed5a1ee7, 76 },
    { 0xa0fa1ee7, 84 },
    { 0xae5a1ee7, 61 },
    { 0xafa1ee4, 61 },
    { 0xb32a1eeb, 33 },
    { 0x496a1eec, 91 },
    { 0xbe7a1eec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a1ef4, 98 },
    { 0x30a1ef5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba1ef8, 87 },
    { 0x2d2a1ef9, 23 },
    { 0x511a1efa, 61 },
    { 0x938a1ef9, 43 },
    { 0xee3a1efc, 98 },
    { 0x6a8a1efc, 67 },
    { 0, 0 },
    { 0xeeda1eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a1f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89a1f09, 90 },
    { 0xb4ba1f09, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x531a1f0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a1f12, 69 },
    { 0, 0 },
    { 0xe23a1f14, 95 },
    { 0x44da1f14, 84 },
    { 0, 0 },
    { 0xbd7a1f17, 78 },
    { 0x83fa1f17, 61 },
    { 0, 0 },
    { 0x16ca1f1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda1f1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a1f25, 61 },
    { 0x560a1f26, 52 },
    { 0x2afa1f27, 85 },
    { 0xc2aa1f28, 34 },
    { 0xaeba1f29, 93 },
    { 0x45a1f28, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84a1f2e, 100 },
    { 0x35ba1f2f, 95 },
    { 0x8d1a1f30, 55 },
    { 0xf04a1f2e, 17 },
    { 0x70ea1f30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a1f40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fda1f44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x308a1f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fea1f4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a1f51, 80 },
    { 0x6c7a1f52, 67 },
    { 0x44da1f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425a1f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a1f5b, 33 },
    { 0x4f3a1f5c, 61 },
    { 0, 0 },
    { 0x389a1f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a1f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a1f66, 90 },
    { 0xe0aa1f66, 76 },
    { 0, 0 },
    { 0x509a1f69, 23 },
    { 0x6eba1f6a, 98 },
    { 0x82ca1f6a, 43 },
    { 0xd6aa1f6c, 87 },
    { 0xf4da1f6b, 61 },
    { 0xe98a1f6e, 98 },
    { 0xbb9a1f6e, 99 },
    { 0, 0 },
    { 0x3e0a1f71, 90 },
    { 0, 0 },
    { 0x377a1f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ada1f7b, 76 },
    { 0x64da1f7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4a1f80, 100 },
    { 0x11aa1f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485a1f88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a1f8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x158a1f90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba1f99, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5daa1f9c, 33 },
    { 0x9f7a1f9d, 83 },
    { 0, 0 },
    { 0x20fa1f9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a1fa4, 99 },
    { 0, 0 },
    { 0xd2ba1fa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5a1fb6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a1fb9, 33 },
    { 0, 0 },
    { 0xfcfa1fbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a1fbe, 95 },
    { 0, 0 },
    { 0x99ba1fc0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a1fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80a1fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9a1fcd, 23 },
    { 0x349a1fce, 33 },
    { 0x622a1fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a1fd2, 76 },
    { 0, 0 },
    { 0x816a1fd4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afa1fd8, 46 },
    { 0xd7aa1fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822a1fde, 33 },
    { 0x5f9a1fde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48a1fe3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1a1fe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a1fec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ba1fef, 33 },
    { 0xf00a1ff0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fa1ff4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a1ff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25a1ffe, 77 },
    { 0x7bea1ffe, 61 },
    { 0x9efa1fff, 90 },
    { 0x2bba2001, 95 },
    { 0x2a2002, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384a2006, 23 },
    { 0xde3a2007, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20a200c, 52 },
    { 0x692a200d, 76 },
    { 0x5dca200e, 98 },
    { 0xc47a200f, 94 },
    { 0xac3a2010, 90 },
    { 0xcc8a2011, 90 },
    { 0xc1ca2012, 90 },
    { 0xe79a2012, 93 },
    { 0x7b2a2014, 33 },
    { 0xf10a2015, 98 },
    { 0x214a2016, 95 },
    { 0xefaa200f, 90 },
    { 0x513a2018, 95 },
    { 0x5e7a2019, 65 },
    { 0x32ca201a, 46 },
    { 0xa2fa2018, 55 },
    { 0x1e5a201a, 46 },
    { 0xd3aa201d, 84 },
    { 0xf13a201d, 87 },
    { 0xaa8a200f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492a2024, 91 },
    { 0x8a6a2025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fca2031, 77 },
    { 0xd49a2031, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca2035, 99 },
    { 0x312a2036, 23 },
    { 0x166a2037, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa203b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9da203f, 85 },
    { 0, 0 },
    { 0xf62a2041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca204d, 43 },
    { 0x6a204e, 33 },
    { 0, 0 },
    { 0x5ba2050, 98 },
    { 0xda1a2051, 95 },
    { 0x494a2052, 33 },
    { 0, 0 },
    { 0xb1fa2054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746a2058, 95 },
    { 0xd6a2059, 33 },
    { 0, 0 },
    { 0x670a205b, 90 },
    { 0x611a205c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a2062, 87 },
    { 0x3d4a2063, 55 },
    { 0x780a2064, 33 },
    { 0x1f4a2065, 61 },
    { 0xd08a2066, 76 },
    { 0x1cfa2065, 98 },
    { 0x492a2065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fa206c, 52 },
    { 0x705a206d, 98 },
    { 0x525a206e, 34 },
    { 0x1daa206d, 76 },
    { 0xa97a206d, 100 },
    { 0x1f2a2071, 23 },
    { 0, 0 },
    { 0x93fa2073, 90 },
    { 0x2dea2074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52a2077, 90 },
    { 0xccea2078, 84 },
    { 0, 0 },
    { 0xc9fa207a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a207d, 88 },
    { 0, 0 },
    { 0xa5ea207f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a2082, 55 },
    { 0x444a2082, 34 },
    { 0xff2a2084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ba2088, 95 },
    { 0x86aa2088, 24 },
    { 0x53ea208a, 98 },
    { 0x4a9a208b, 95 },
    { 0xe1ea208c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715a2091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9a2097, 13 },
    { 0, 0 },
    { 0x400a2099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa209e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca20a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07a20ad, 90 },
    { 0x618a20ae, 33 },
    { 0x6e0a20ad, 87 },
    { 0x8b8a20b0, 91 },
    { 0x733a20b1, 66 },
    { 0, 0 },
    { 0xfa3a20b3, 34 },
    { 0xa53a20b3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291a20b9, 78 },
    { 0xf76a20b9, 43 },
    { 0x3a4a20bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909a20c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a20c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a20ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a20ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a20d1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2a20d4, 23 },
    { 0xd78a20d5, 90 },
    { 0x5e3a20d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a20dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5a20e0, 78 },
    { 0, 0 },
    { 0xb18a20e2, 53 },
    { 0x2ba20e3, 46 },
    { 0xa71a20e3, 61 },
    { 0x94ea20e5, 90 },
    { 0xa61a20e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a20e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686a20ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdba20ff, 87 },
    { 0xa90a2100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7a2105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a210b, 98 },
    { 0, 0 },
    { 0x937a210d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a2110, 98 },
    { 0x478a2110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39a2115, 90 },
    { 0xf31a2116, 69 },
    { 0x71ca2117, 33 },
    { 0, 0 },
    { 0xbc5a2119, 76 },
    { 0x66ba211a, 98 },
    { 0x5b7a211a, 23 },
    { 0, 0 },
    { 0x6dda211d, 91 },
    { 0x31a211e, 61 },
    { 0x72ea211d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907a2123, 90 },
    { 0x384a2124, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x530a2127, 90 },
    { 0x4eea2127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a212d, 33 },
    { 0xf45a212e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a2131, 17 },
    { 0xb54a2132, 95 },
    { 0x2daa2133, 67 },
    { 0x87aa2134, 83 },
    { 0xfd3a2134, 98 },
    { 0x9f5a2136, 33 },
    { 0x47a2137, 65 },
    { 0x465a2134, 81 },
    { 0x8e1a2134, 75 },
    { 0xb8a213a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a2142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a214b, 23 },
    { 0xda6a214b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca2150, 55 },
    { 0x145a2151, 55 },
    { 0xedfa2151, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911a2158, 84 },
    { 0x634a2159, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93da215c, 76 },
    { 0, 0 },
    { 0x8a9a215e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a2168, 76 },
    { 0x53da2168, 90 },
    { 0, 0 },
    { 0x6faa216b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ca2171, 91 },
    { 0x5aca2172, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a2175, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290a217a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a217d, 33 },
    { 0x696a217e, 61 },
    { 0xd04a217f, 34 },
    { 0x54a217d, 34 },
    { 0x8cfa2181, 90 },
    { 0x4bea2182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a2186, 23 },
    { 0xea2a2186, 76 },
    { 0x928a2188, 61 },
    { 0x80ea2188, 95 },
    { 0x7b7a2187, 95 },
    { 0, 0 },
    { 0xe14a218c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907a2190, 84 },
    { 0, 0 },
    { 0x865a2192, 90 },
    { 0, 0 },
    { 0xacca2194, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a2197, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fa219c, 90 },
    { 0, 0 },
    { 0xc89a219e, 95 },
    { 0xe21a219e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x648a21a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aa21ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2aa21b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x801a21b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7a21bf, 43 },
    { 0xce6a21bf, 98 },
    { 0x4c0a21c1, 98 },
    { 0x223a21c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfa21c7, 98 },
    { 0xfdfa21c8, 76 },
    { 0x65fa21c9, 91 },
    { 0x48da21ca, 33 },
    { 0xfcfa21cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a21d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a21d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9a21d8, 52 },
    { 0x314a21d9, 98 },
    { 0xb1fa21d8, 85 },
    { 0x8aaa21d9, 98 },
    { 0xf63a21dc, 43 },
    { 0x7d4a21d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a21e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ca21e5, 61 },
    { 0, 0 },
    { 0x170a21e7, 95 },
    { 0x1fca21e8, 76 },
    { 0x85a21e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a21f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877a21f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09a21fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5a2202, 100 },
    { 0x8afa2203, 33 },
    { 0x37ba2204, 90 },
    { 0x179a2202, 90 },
    { 0xf8ba2204, 83 },
    { 0, 0 },
    { 0x7afa2208, 95 },
    { 0, 0 },
    { 0x225a220a, 76 },
    { 0x83ca220b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64a2213, 90 },
    { 0xc3ba2214, 76 },
    { 0xd1da2215, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789a221a, 98 },
    { 0, 0 },
    { 0x1a5a221c, 46 },
    { 0xa76a221d, 61 },
    { 0xcfca221d, 90 },
    { 0x4fda221f, 78 },
    { 0xb1a2220, 95 },
    { 0xd38a221e, 99 },
    { 0x9d4a2222, 36 },
    { 0x69da221f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea222a, 34 },
    { 0, 0 },
    { 0xad8a222c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8a2233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a2239, 23 },
    { 0x2f3a223a, 33 },
    { 0x70ea2239, 61 },
    { 0xadea223c, 76 },
    { 0x5ea2239, 90 },
    { 0xcd2a223e, 33 },
    { 0xbaea223f, 33 },
    { 0xf7ba223f, 33 },
    { 0, 0 },
    { 0x73a2242, 91 },
    { 0x193a2242, 34 },
    { 0x36ba2242, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a224b, 88 },
    { 0x8b9a224b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7da2251, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa2256, 95 },
    { 0, 0 },
    { 0x12ba2258, 67 },
    { 0x1d5a2259, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca225c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dda225f, 52 },
    { 0xc08a2260, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89a2264, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59a2267, 23 },
    { 0xbaba2267, 55 },
    { 0x16ba2269, 77 },
    { 0x9ca226a, 61 },
    { 0x5dba226a, 93 },
    { 0xa00a226b, 61 },
    { 0, 0 },
    { 0x896a226e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x835a2271, 91 },
    { 0, 0 },
    { 0x914a2273, 65 },
    { 0x740a2274, 99 },
    { 0x688a2275, 33 },
    { 0xf0aa2273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6a2279, 23 },
    { 0x4b6a227a, 33 },
    { 0x3c5a227b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ca227e, 55 },
    { 0, 0 },
    { 0x577a2280, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331a2285, 17 },
    { 0x7cba2285, 66 },
    { 0xf3fa2287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eda228a, 34 },
    { 0, 0 },
    { 0x102a228c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cda2292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a2295, 23 },
    { 0, 0 },
    { 0xfe1a2297, 67 },
    { 0x437a2298, 61 },
    { 0, 0 },
    { 0xbd5a229a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751a22a3, 76 },
    { 0x8a5a22a4, 43 },
    { 0x7b9a22a4, 61 },
    { 0x857a22a6, 90 },
    { 0x10fa22a5, 87 },
    { 0x82a22a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924a22ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56a22af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa22b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaa22b6, 87 },
    { 0x40da22b7, 53 },
    { 0xb01a22b8, 98 },
    { 0, 0 },
    { 0xe02a22ba, 95 },
    { 0xc8a22bb, 90 },
    { 0xb06a22bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xecea22bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4a22c5, 88 },
    { 0x764a22c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9a22ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eda22cf, 56 },
    { 0, 0 },
    { 0xa22d1, 67 },
    { 0, 0 },
    { 0x7b8a22d3, 69 },
    { 0xbd9a22d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372a22de, 61 },
    { 0x83a22df, 95 },
    { 0x55fa22e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426a22e6, 23 },
    { 0, 0 },
    { 0x8a0a22e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x614a22eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x843a22ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ea22f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea22f5, 95 },
    { 0x57ba22f5, 61 },
    { 0, 0 },
    { 0x3a3a22f8, 76 },
    { 0x52fa22f8, 61 },
    { 0xfbda22f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a2301, 61 },
    { 0x93da2301, 55 },
    { 0, 0 },
    { 0x2ffa2304, 61 },
    { 0, 0 },
    { 0xf93a2306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a2309, 90 },
    { 0x665a230a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2a230e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba2313, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x87da2316, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afa231a, 55 },
    { 0x5dea231b, 61 },
    { 0x109a231c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a2320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3a2324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46a2334, 55 },
    { 0, 0 },
    { 0xf7aa2336, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa233a, 98 },
    { 0, 0 },
    { 0x217a233c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa2340, 78 },
    { 0x20fa2341, 23 },
    { 0xcbfa2342, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9a2345, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ba2348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fa2358, 61 },
    { 0, 0 },
    { 0x485a235a, 98 },
    { 0xf9fa235b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a235e, 55 },
    { 0xefea235f, 52 },
    { 0x402a2360, 52 },
    { 0x9f9a2361, 90 },
    { 0xb97a2362, 88 },
    { 0xdb4a2363, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a2369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372a236d, 98 },
    { 0xc1da236e, 91 },
    { 0xd92a236f, 23 },
    { 0xba236d, 34 },
    { 0x2b9a236d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aba2376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a2379, 52 },
    { 0x685a237a, 65 },
    { 0xc27a237b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6a2384, 23 },
    { 0, 0 },
    { 0x372a2386, 74 },
    { 0, 0 },
    { 0x939a2388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x828a238b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a238e, 34 },
    { 0x8d1a238f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51a2392, 23 },
    { 0x995a2393, 90 },
    { 0x8dba2393, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea2397, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a239a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36da239e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7a23a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a23a4, 90 },
    { 0x168a23a5, 75 },
    { 0x7f5a23a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48a23ab, 90 },
    { 0, 0 },
    { 0x7ada23ad, 33 },
    { 0xd2aa23ae, 33 },
    { 0, 0 },
    { 0x1b5a23b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa23b8, 67 },
    { 0xd48a23b9, 61 },
    { 0xc3fa23ba, 76 },
    { 0x2b4a23bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a23c3, 87 },
    { 0x416a23c4, 33 },
    { 0, 0 },
    { 0xce0a23c6, 76 },
    { 0x5f6a23c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310a23ce, 90 },
    { 0, 0 },
    { 0x6c9a23d0, 61 },
    { 0, 0 },
    { 0x91ea23d2, 23 },
    { 0xd22a23d3, 51 },
    { 0x17ea23d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a23d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fba23db, 55 },
    { 0, 0 },
    { 0xbe3a23dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x875a23e0, 78 },
    { 0x41ba23e0, 34 },
    { 0x713a23e2, 90 },
    { 0x335a23e3, 23 },
    { 0, 0 },
    { 0x869a23e5, 93 },
    { 0xf46a23e6, 85 },
    { 0x5f2a23e7, 75 },
    { 0x338a23e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71a23eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60a23f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da23f4, 90 },
    { 0x247a23f4, 90 },
    { 0x1eda23f6, 81 },
    { 0x388a23f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794a23fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8a2405, 33 },
    { 0xeaea2406, 55 },
    { 0x1c4a2406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x680a240a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84da2415, 33 },
    { 0x4e2a2415, 99 },
    { 0xbbfa2417, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca241b, 52 },
    { 0xd2ea241c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a241f, 33 },
    { 0x7f4a241f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a2425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a2429, 98 },
    { 0, 0 },
    { 0xe59a242b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x119a242e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55a2433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a2436, 61 },
    { 0x438a2436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a243a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82a2440, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfca2446, 96 },
    { 0x29da2447, 33 },
    { 0xd7fa2446, 85 },
    { 0x74da2449, 61 },
    { 0x51aa2447, 43 },
    { 0xbeda244b, 93 },
    { 0, 0 },
    { 0x7e3a244d, 95 },
    { 0xb62a244e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x593a2451, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75da2454, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03a2457, 88 },
    { 0xb8da2457, 46 },
    { 0xa33a2459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38aa245f, 98 },
    { 0x748a245f, 95 },
    { 0, 0 },
    { 0x58ca2462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a2466, 90 },
    { 0x64ca2466, 95 },
    { 0x7a7a2466, 66 },
    { 0x2b8a2469, 77 },
    { 0x931a2469, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a246d, 55 },
    { 0xa77a246e, 91 },
    { 0, 0 },
    { 0x234a2470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2477, 33 },
    { 0, 0 },
    { 0x46ea2479, 95 },
    { 0xabaa2479, 80 },
    { 0x760a2479, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11da2481, 55 },
    { 0x517a2482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5a2485, 87 },
    { 0x5c0a2485, 40 },
    { 0, 0 },
    { 0xe75a2488, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ca248d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a2494, 46 },
    { 0xdca2494, 33 },
    { 0xa71a2496, 43 },
    { 0xe9ea2496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daa249b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ea249f, 98 },
    { 0x713a249f, 90 },
    { 0, 0 },
    { 0x40a24a2, 90 },
    { 0x81fa24a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43aa24a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13a24ab, 93 },
    { 0x607a24ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495a24b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da24b4, 85 },
    { 0xadca24b5, 78 },
    { 0x47ca24b6, 43 },
    { 0xa95a24b7, 61 },
    { 0xdffa24b8, 61 },
    { 0, 0 },
    { 0x27da24ba, 67 },
    { 0x704a24bb, 34 },
    { 0x6d5a24bc, 100 },
    { 0xfa8a24bd, 94 },
    { 0x8aca24bd, 34 },
    { 0xbf7a24bb, 95 },
    { 0xcc0a24bf, 61 },
    { 0, 0 },
    { 0x2cea24c2, 93 },
    { 0x84aa24c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ca24c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80a24cc, 46 },
    { 0xa8fa24cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba24d1, 55 },
    { 0, 0 },
    { 0x27ba24d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x858a24d6, 61 },
    { 0x4e8a24d7, 38 },
    { 0x764a24d7, 95 },
    { 0xe09a24d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a24dc, 90 },
    { 0x6d2a24dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a24e1, 93 },
    { 0x1b6a24e2, 95 },
    { 0, 0 },
    { 0x3cda24e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a24e7, 99 },
    { 0xa3ba24e7, 78 },
    { 0xe8da24e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01a24ec, 34 },
    { 0x483a24ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a24f0, 76 },
    { 0xdd2a24f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88a24f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x177a24f7, 61 },
    { 0, 0 },
    { 0x6a8a24f9, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5a24fc, 90 },
    { 0x765a24fd, 93 },
    { 0xb50a24fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33a2501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2504, 90 },
    { 0xb45a2504, 90 },
    { 0x454a2504, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779a250a, 61 },
    { 0x47fa250b, 76 },
    { 0x194a250c, 90 },
    { 0, 0 },
    { 0x714a250e, 95 },
    { 0x1dfa250f, 95 },
    { 0x627a2510, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eca2513, 97 },
    { 0x175a2514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721a251a, 33 },
    { 0xa28a251b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a2523, 43 },
    { 0x392a2523, 61 },
    { 0xe06a2523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa2528, 90 },
    { 0, 0 },
    { 0xb0ca252a, 95 },
    { 0, 0 },
    { 0x25da252c, 98 },
    { 0xc42a252d, 99 },
    { 0, 0 },
    { 0x47fa252f, 55 },
    { 0, 0 },
    { 0x9eea2531, 55 },
    { 0xe3ba2532, 33 },
    { 0, 0 },
    { 0xc19a2534, 33 },
    { 0x675a2535, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a2539, 93 },
    { 0xc1ea253a, 99 },
    { 0x44da253b, 55 },
    { 0, 0 },
    { 0x410a253d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a2543, 90 },
    { 0xe06a2544, 55 },
    { 0, 0 },
    { 0xbe1a2546, 55 },
    { 0xb81a2546, 87 },
    { 0, 0 },
    { 0x892a2549, 90 },
    { 0, 0 },
    { 0xbc2a254b, 35 },
    { 0x594a254c, 98 },
    { 0xc4ca254d, 95 },
    { 0x338a254c, 43 },
    { 0x8f6a254f, 90 },
    { 0xa4fa2550, 98 },
    { 0x40aa2551, 79 },
    { 0, 0 },
    { 0xbb6a2553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219a255e, 90 },
    { 0, 0 },
    { 0x551a2560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fa2565, 60 },
    { 0, 0 },
    { 0xd6da2567, 33 },
    { 0, 0 },
    { 0xc2ca2569, 87 },
    { 0xac8a256a, 90 },
    { 0x73ca256b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a256e, 78 },
    { 0x82ea256e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822a2576, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca257e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a2584, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a2587, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca258c, 98 },
    { 0, 0 },
    { 0xa1ca258e, 23 },
    { 0xc3aa258e, 34 },
    { 0, 0 },
    { 0x98ea2591, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3a2594, 66 },
    { 0, 0 },
    { 0xc6fa2596, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea2599, 90 },
    { 0x7a6a259a, 95 },
    { 0xbe5a259a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a259e, 90 },
    { 0xad1a259f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a25a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a25a7, 33 },
    { 0x738a25a8, 90 },
    { 0x4eca25a9, 66 },
    { 0x5d2a25a8, 95 },
    { 0xb0ba25a7, 98 },
    { 0, 0 },
    { 0xb77a25ad, 61 },
    { 0, 0 },
    { 0x52ea25af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a25b3, 95 },
    { 0xd7a25b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaa25be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3a25c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5a25c4, 76 },
    { 0, 0 },
    { 0x981a25c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x377a25c9, 55 },
    { 0, 0 },
    { 0x411a25cb, 77 },
    { 0x706a25cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba25d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118a25d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea25db, 23 },
    { 0x7a7a25dc, 98 },
    { 0xf0ca25dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea25eb, 90 },
    { 0xca6a25eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea25f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a25f3, 17 },
    { 0x343a25f3, 76 },
    { 0, 0 },
    { 0x74da25f6, 95 },
    { 0x8dfa25f6, 91 },
    { 0, 0 },
    { 0x5d1a25f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a25fc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a2605, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x799a2608, 77 },
    { 0x611a2609, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a260e, 55 },
    { 0, 0 },
    { 0x9cda2610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37a2613, 23 },
    { 0xbc5a2613, 61 },
    { 0x442a2614, 61 },
    { 0x715a2616, 90 },
    { 0xebfa2615, 61 },
    { 0x77ba2618, 81 },
    { 0, 0 },
    { 0x918a261a, 61 },
    { 0, 0 },
    { 0x892a261c, 33 },
    { 0, 0 },
    { 0xddaa261e, 46 },
    { 0x1c8a261f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ba2624, 33 },
    { 0xcc9a2625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435a2629, 95 },
    { 0xdb5a262a, 88 },
    { 0x311a2629, 90 },
    { 0x17ba262c, 53 },
    { 0x1daa262d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa2632, 87 },
    { 0x5e5a2633, 84 },
    { 0x2dca2634, 90 },
    { 0, 0 },
    { 0xaa5a2636, 69 },
    { 0, 0 },
    { 0x5e0a2638, 33 },
    { 0xd7fa2638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdba2641, 46 },
    { 0, 0 },
    { 0xec6a2643, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x685a2646, 95 },
    { 0xffca2647, 66 },
    { 0x3eca2648, 61 },
    { 0x855a2647, 76 },
    { 0xe66a2646, 90 },
    { 0, 0 },
    { 0x357a264c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9a264f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ca2652, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2da2655, 33 },
    { 0x491a2655, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ba265d, 61 },
    { 0xacca265d, 95 },
    { 0x7d2a265d, 98 },
    { 0x143a265e, 76 },
    { 0xa265f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fa2664, 95 },
    { 0x5e8a2665, 33 },
    { 0xb9aa2666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2669, 84 },
    { 0xacba266a, 87 },
    { 0x18aa2669, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa2671, 90 },
    { 0xa06a2671, 61 },
    { 0x703a2673, 43 },
    { 0x37ca2673, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x507a2677, 43 },
    { 0, 0 },
    { 0x758a2679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x473a267c, 33 },
    { 0x3b3a267d, 61 },
    { 0xc17a267e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04a2684, 78 },
    { 0xc8aa2685, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a268a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a268f, 76 },
    { 0x5e4a2690, 69 },
    { 0xe84a2690, 76 },
    { 0xdb4a2692, 61 },
    { 0x914a2692, 17 },
    { 0, 0 },
    { 0x21da2695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a2698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a26a3, 78 },
    { 0x32aa26a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95a26b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a26bc, 34 },
    { 0xfaa26bd, 66 },
    { 0, 0 },
    { 0xfe8a26bf, 91 },
    { 0, 0 },
    { 0x168a26c1, 70 },
    { 0x3fea26c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a26c9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a26cc, 61 },
    { 0x5bba26cd, 52 },
    { 0xdb8a26ce, 33 },
    { 0x2b9a26cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4aa26d4, 95 },
    { 0, 0 },
    { 0x45ea26d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13da26dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4a26e0, 98 },
    { 0, 0 },
    { 0x452a26e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85a26e5, 76 },
    { 0xfe9a26e6, 93 },
    { 0, 0 },
    { 0x5b2a26e8, 78 },
    { 0x83aa26e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da26ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a26ef, 55 },
    { 0x331a26f0, 98 },
    { 0x5f2a26ef, 23 },
    { 0x672a26f2, 25 },
    { 0x717a26f1, 43 },
    { 0, 0 },
    { 0xc60a26f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x957a26f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730a26fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a2701, 90 },
    { 0x5da2701, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fa2707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57a270c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a2712, 43 },
    { 0x4aba2713, 98 },
    { 0, 0 },
    { 0xc2ea2715, 90 },
    { 0x79da2715, 23 },
    { 0x242a2715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fa271b, 68 },
    { 0, 0 },
    { 0x1d5a271d, 78 },
    { 0x6b1a271e, 61 },
    { 0, 0 },
    { 0x25da2720, 55 },
    { 0xafaa2721, 61 },
    { 0xca2722, 98 },
    { 0, 0 },
    { 0x797a2724, 76 },
    { 0, 0 },
    { 0xa9ea2726, 90 },
    { 0xf00a2727, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda272b, 90 },
    { 0, 0 },
    { 0xc53a272d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ea2730, 61 },
    { 0x53aa2730, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58aa2735, 87 },
    { 0x112a2736, 99 },
    { 0x836a2737, 66 },
    { 0x11fa2737, 84 },
    { 0x5b0a2739, 90 },
    { 0xf94a273a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a273f, 55 },
    { 0x7f5a273f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28a2745, 88 },
    { 0xaf8a2745, 92 },
    { 0x238a2745, 61 },
    { 0xc60a2748, 34 },
    { 0xa7aa2749, 100 },
    { 0xd63a274a, 100 },
    { 0xe51a2746, 77 },
    { 0xe42a274c, 76 },
    { 0x9f4a274c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x454a2750, 34 },
    { 0x6f2a2751, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a2754, 76 },
    { 0, 0 },
    { 0xdcaa2756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5a2762, 85 },
    { 0xfa4a2762, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaa2766, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a2769, 95 },
    { 0x9da2769, 88 },
    { 0xe43a276b, 76 },
    { 0x27ca276c, 90 },
    { 0xbfa276c, 33 },
    { 0, 0 },
    { 0xb54a276f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39da2773, 40 },
    { 0xe6ba2773, 61 },
    { 0xd7ba2775, 98 },
    { 0x800a2773, 90 },
    { 0xdd6a2775, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aa277c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a277f, 34 },
    { 0xc62a277f, 61 },
    { 0x29fa2781, 95 },
    { 0x6ea2782, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4a2785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619a278b, 77 },
    { 0x37ba278b, 93 },
    { 0, 0 },
    { 0xb2ea278e, 52 },
    { 0, 0 },
    { 0x5dda2790, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea2795, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a2798, 90 },
    { 0xf92a2798, 61 },
    { 0x364a279a, 61 },
    { 0x948a279b, 91 },
    { 0x9c2a279c, 87 },
    { 0xc92a279d, 90 },
    { 0x16ba279c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da27a1, 95 },
    { 0, 0 },
    { 0x5d8a27a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba27a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea27ad, 33 },
    { 0x680a27ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca27b1, 84 },
    { 0x57a27b2, 55 },
    { 0x133a27b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea27bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626a27c0, 90 },
    { 0x607a27c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacda27c6, 80 },
    { 0x807a27c7, 34 },
    { 0, 0 },
    { 0x90ea27c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba27d1, 43 },
    { 0x9aea27d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16da27d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a27d9, 55 },
    { 0x510a27d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a27dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a27e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da27e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a27ec, 84 },
    { 0xfaea27ec, 61 },
    { 0xea6a27ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10a27f2, 52 },
    { 0, 0 },
    { 0x58da27f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a27f8, 76 },
    { 0x6fca27f9, 43 },
    { 0x2b5a27fa, 61 },
    { 0, 0 },
    { 0xaaba27fc, 90 },
    { 0, 0 },
    { 0xcd8a27fe, 43 },
    { 0xd43a27ff, 23 },
    { 0x2f1a27ff, 23 },
    { 0xca2a2801, 84 },
    { 0, 0 },
    { 0x358a2803, 100 },
    { 0x4dda2803, 98 },
    { 0x3eda2805, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942a2809, 76 },
    { 0x6eaa280a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16a280d, 98 },
    { 0xa10a280d, 33 },
    { 0, 0 },
    { 0xc5ca2810, 90 },
    { 0, 0 },
    { 0x6d6a2812, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a2815, 98 },
    { 0, 0 },
    { 0x1c4a2817, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570a281e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12a2821, 76 },
    { 0x864a2822, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x642a2825, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a2829, 55 },
    { 0, 0 },
    { 0x9e7a282b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74a2832, 91 },
    { 0x637a2832, 23 },
    { 0xf8da2834, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a2838, 76 },
    { 0x7d5a2839, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x92da283c, 90 },
    { 0, 0 },
    { 0x94fa283e, 33 },
    { 0xa2da283f, 55 },
    { 0xf44a2840, 61 },
    { 0x409a283f, 34 },
    { 0x6f8a2842, 90 },
    { 0x222a283f, 90 },
    { 0, 0 },
    { 0xf94a2845, 75 },
    { 0x674a2846, 46 },
    { 0x59ba2847, 55 },
    { 0xc64a2848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8a2853, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaea2856, 76 },
    { 0xba2a2856, 90 },
    { 0, 0 },
    { 0x5aaa2859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x453a285c, 76 },
    { 0x7c2a285c, 76 },
    { 0x8dfa285d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a2864, 78 },
    { 0, 0 },
    { 0x9dba2866, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983a286c, 61 },
    { 0xf79a286d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51a287b, 33 },
    { 0xffa287b, 33 },
    { 0x3cba287d, 33 },
    { 0, 0 },
    { 0x666a287f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a2887, 61 },
    { 0x692a2888, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa288e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba2894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23a289a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4da289f, 61 },
    { 0, 0 },
    { 0x6a28a1, 93 },
    { 0x711a28a1, 61 },
    { 0, 0 },
    { 0xb0a28a4, 46 },
    { 0, 0 },
    { 0x2ea28a6, 23 },
    { 0x5e2a28a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a28ac, 23 },
    { 0xabea28ad, 34 },
    { 0xb28a28ae, 66 },
    { 0xe5ca28ae, 33 },
    { 0x104a28b0, 61 },
    { 0xe2ca28ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a28b6, 67 },
    { 0, 0 },
    { 0x494a28b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x346a28bb, 65 },
    { 0xb2ca28bc, 40 },
    { 0, 0 },
    { 0xa1fa28be, 95 },
    { 0, 0 },
    { 0xcfa28c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca28c5, 66 },
    { 0x3c0a28c6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a28ca, 100 },
    { 0, 0 },
    { 0x398a28cc, 52 },
    { 0x6dba28cd, 90 },
    { 0, 0 },
    { 0x4f9a28cf, 90 },
    { 0x300a28d0, 90 },
    { 0xd35a28d1, 87 },
    { 0x999a28d2, 23 },
    { 0xde8a28d3, 90 },
    { 0xa54a28d3, 80 },
    { 0x9f8a28d5, 61 },
    { 0x6a5a28d5, 61 },
    { 0x416a28d3, 66 },
    { 0xaa2a28d8, 90 },
    { 0xc7fa28d4, 93 },
    { 0x4a4a28d1, 76 },
    { 0, 0 },
    { 0x8cca28dc, 98 },
    { 0, 0 },
    { 0x4eea28de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373a28e3, 46 },
    { 0xd0ca28e3, 76 },
    { 0x791a28e5, 61 },
    { 0x836a28e6, 33 },
    { 0x349a28e7, 88 },
    { 0x4bda28e7, 98 },
    { 0xbeba28e7, 55 },
    { 0x20ba28e3, 98 },
    { 0x9e7a28e5, 95 },
    { 0, 0 },
    { 0x75ba28ed, 87 },
    { 0x197a28ee, 55 },
    { 0, 0 },
    { 0xee1a28f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a28f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747a28fb, 17 },
    { 0xb54a28fc, 90 },
    { 0x575a28fd, 95 },
    { 0xa79a28fb, 91 },
    { 0x557a28ff, 90 },
    { 0x275a28fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ea2908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba2919, 5 },
    { 0x4eca291a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8a291d, 95 },
    { 0xf82a291e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a2923, 61 },
    { 0x497a2923, 95 },
    { 0, 0 },
    { 0x910a2926, 33 },
    { 0x784a2926, 100 },
    { 0x611a2928, 37 },
    { 0xfb8a2928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a292c, 84 },
    { 0xf34a292d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2a2932, 99 },
    { 0x7fca2933, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3a2937, 63 },
    { 0x7c2a2938, 90 },
    { 0, 0 },
    { 0xb8aa293a, 23 },
    { 0, 0 },
    { 0xc41a293c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa2944, 55 },
    { 0x4aea2945, 85 },
    { 0x5f5a2946, 33 },
    { 0xa1ea2947, 33 },
    { 0xfd4a2948, 76 },
    { 0x232a2944, 55 },
    { 0, 0 },
    { 0x836a294b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2a294f, 76 },
    { 0xd3fa294f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518a2959, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da2968, 90 },
    { 0, 0 },
    { 0x301a296a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77a296e, 98 },
    { 0x950a296f, 100 },
    { 0x70ea2970, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71a2976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5da297c, 90 },
    { 0, 0 },
    { 0x465a297e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88a2986, 33 },
    { 0, 0 },
    { 0x6a1a2988, 23 },
    { 0x7e5a2989, 100 },
    { 0x8d8a298a, 84 },
    { 0x1f7a2989, 34 },
    { 0x302a298a, 23 },
    { 0, 0 },
    { 0xa62a298e, 90 },
    { 0x5e3a298e, 33 },
    { 0, 0 },
    { 0xf16a2991, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a2996, 93 },
    { 0, 0 },
    { 0xaf4a2998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447a299c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a29a2, 61 },
    { 0xce4a29a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a29a8, 100 },
    { 0, 0 },
    { 0x96aa29aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa29b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a29b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a29bc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97a29c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3da29c3, 43 },
    { 0, 0 },
    { 0x946a29c5, 95 },
    { 0x894a29c6, 55 },
    { 0, 0 },
    { 0x2f8a29c8, 93 },
    { 0x77ba29c9, 89 },
    { 0xcada29c9, 78 },
    { 0xcf8a29cb, 43 },
    { 0x2d5a29cb, 34 },
    { 0x383a29c8, 78 },
    { 0, 0 },
    { 0x6e0a29cf, 88 },
    { 0, 0 },
    { 0x116a29d1, 34 },
    { 0xe1da29d2, 43 },
    { 0x360a29d3, 98 },
    { 0, 0 },
    { 0x9a1a29d5, 33 },
    { 0xc3aa29d6, 84 },
    { 0x15ca29d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2a29dc, 76 },
    { 0xb02a29dd, 95 },
    { 0x996a29de, 61 },
    { 0xd59a29de, 95 },
    { 0x8dda29df, 34 },
    { 0x10ca29e1, 43 },
    { 0xbbfa29e0, 8 },
    { 0x5e6a29dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144a29e8, 93 },
    { 0x3dba29e9, 85 },
    { 0x2bda29ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8a29f3, 95 },
    { 0, 0 },
    { 0x615a29f5, 23 },
    { 0xccca29f6, 78 },
    { 0x793a29f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a29fa, 33 },
    { 0, 0 },
    { 0xb91a29fc, 23 },
    { 0xe1ba29fc, 98 },
    { 0x7d5a29fd, 100 },
    { 0xd7aa29fd, 98 },
    { 0x790a29fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20a2a03, 96 },
    { 0xed7a2a03, 34 },
    { 0x5e4a2a04, 91 },
    { 0, 0 },
    { 0x653a2a07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ea2a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a2a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fa2a11, 90 },
    { 0xdf7a2a11, 98 },
    { 0, 0 },
    { 0xa96a2a14, 33 },
    { 0, 0 },
    { 0x375a2a16, 90 },
    { 0, 0 },
    { 0xdc7a2a18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a2a1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fa2a1f, 61 },
    { 0x32a2a20, 23 },
    { 0x7a6a2a21, 23 },
    { 0x144a2a22, 90 },
    { 0x93ba2a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a2a2c, 66 },
    { 0x12ea2a2d, 91 },
    { 0xc0ba2a2e, 61 },
    { 0xba4a2a2d, 81 },
    { 0xfaea2a30, 33 },
    { 0, 0 },
    { 0x4eca2a32, 95 },
    { 0, 0 },
    { 0x74fa2a34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82a2a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a2a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ba2a42, 52 },
    { 0xa50a2a43, 17 },
    { 0x438a2a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a2a47, 34 },
    { 0x328a2a48, 61 },
    { 0x11ba2a49, 43 },
    { 0x706a2a47, 61 },
    { 0x210a2a49, 93 },
    { 0xf14a2a4c, 84 },
    { 0, 0 },
    { 0xc70a2a4e, 98 },
    { 0x587a2a4f, 35 },
    { 0xe08a2a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba2a54, 23 },
    { 0, 0 },
    { 0x29ba2a56, 95 },
    { 0, 0 },
    { 0x5e5a2a58, 66 },
    { 0x127a2a58, 81 },
    { 0x7dfa2a59, 80 },
    { 0x610a2a5b, 34 },
    { 0, 0 },
    { 0xd92a2a5d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535a2a61, 23 },
    { 0xe67a2a61, 98 },
    { 0x38ea2a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1a2a67, 55 },
    { 0xb01a2a68, 43 },
    { 0, 0 },
    { 0x585a2a6a, 66 },
    { 0, 0 },
    { 0x273a2a6c, 90 },
    { 0x59a2a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aa2a75, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a2a79, 33 },
    { 0, 0 },
    { 0x898a2a7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x700a2a7e, 76 },
    { 0, 0 },
    { 0x9c3a2a80, 91 },
    { 0xadfa2a81, 98 },
    { 0x797a2a82, 56 },
    { 0x214a2a81, 34 },
    { 0x21aa2a84, 99 },
    { 0x5b0a2a85, 90 },
    { 0x4c8a2a84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3a2a89, 90 },
    { 0, 0 },
    { 0xe63a2a8b, 78 },
    { 0, 0 },
    { 0xaea2a8d, 90 },
    { 0x381a2a8e, 98 },
    { 0, 0 },
    { 0xc5ea2a90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaa2a96, 76 },
    { 0x199a2a96, 99 },
    { 0x866a2a98, 90 },
    { 0, 0 },
    { 0xe88a2a9a, 90 },
    { 0, 0 },
    { 0xf10a2a9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x810a2a9f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8da2aab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da2aaf, 85 },
    { 0, 0 },
    { 0x25ba2ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689a2ab5, 95 },
    { 0x4cda2ab6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda2ab9, 23 },
    { 0xf23a2aba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cca2abe, 90 },
    { 0xd35a2abe, 90 },
    { 0x8b2a2ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a2aca, 33 },
    { 0x8fca2acb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fa2ace, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x156a2ad1, 37 },
    { 0, 0 },
    { 0xe23a2ad3, 87 },
    { 0, 0 },
    { 0xf16a2ad5, 33 },
    { 0, 0 },
    { 0x5d6a2ad7, 98 },
    { 0xd57a2ad8, 77 },
    { 0, 0 },
    { 0x200a2ada, 88 },
    { 0x9d7a2adb, 93 },
    { 0x8c3a2adc, 34 },
    { 0x746a2adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82a2ae0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca2ae3, 98 },
    { 0xe56a2ae3, 33 },
    { 0x22da2ae5, 66 },
    { 0x7cea2ae6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca2aeb, 98 },
    { 0xa58a2aeb, 23 },
    { 0xf5da2aed, 76 },
    { 0x2faa2aec, 98 },
    { 0x513a2aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1a2af5, 61 },
    { 0, 0 },
    { 0x90aa2af7, 90 },
    { 0, 0 },
    { 0x551a2af9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a2b00, 23 },
    { 0x3fda2b01, 34 },
    { 0, 0 },
    { 0x1f7a2b03, 65 },
    { 0, 0 },
    { 0x9dba2b05, 46 },
    { 0, 0 },
    { 0x4b8a2b07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dca2b0b, 87 },
    { 0xe55a2b0b, 61 },
    { 0x60ea2b0c, 98 },
    { 0xcc2a2b0e, 87 },
    { 0x92ca2b0e, 98 },
    { 0xf6ba2b10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894a2b15, 99 },
    { 0x67aa2b16, 84 },
    { 0xe90a2b17, 76 },
    { 0xf63a2b18, 23 },
    { 0xae8a2b19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4a2b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba2b23, 64 },
    { 0, 0 },
    { 0xddca2b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbada2b29, 76 },
    { 0, 0 },
    { 0x455a2b2b, 47 },
    { 0xa15a2b2c, 23 },
    { 0, 0 },
    { 0x76da2b2e, 33 },
    { 0x7faa2b2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a2b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a2b3b, 90 },
    { 0x739a2b3c, 55 },
    { 0xb14a2b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23a2b43, 43 },
    { 0xedba2b44, 34 },
    { 0xfd2a2b43, 33 },
    { 0x808a2b46, 97 },
    { 0xa1da2b47, 61 },
    { 0, 0 },
    { 0x81ea2b49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a2b4c, 90 },
    { 0x7b7a2b4c, 61 },
    { 0x715a2b4c, 98 },
    { 0xef7a2b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726a2b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9a2b58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824a2b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109a2b62, 23 },
    { 0xb1ea2b63, 98 },
    { 0xb1ba2b64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca2b69, 23 },
    { 0x2e2a2b6a, 65 },
    { 0xbfa2b6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a2b72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a2b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2b79, 65 },
    { 0x299a2b7a, 90 },
    { 0, 0 },
    { 0xb65a2b7c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a2b85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fca2b8e, 34 },
    { 0x46aa2b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a2b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea2ba0, 23 },
    { 0xbbfa2ba1, 85 },
    { 0, 0 },
    { 0xfd2a2ba3, 95 },
    { 0xd44a2ba3, 17 },
    { 0, 0 },
    { 0x20a2ba6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ba2ba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17a2bae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94a2bb1, 98 },
    { 0xfc5a2bb1, 90 },
    { 0xf8da2bb2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a2bb6, 85 },
    { 0xcfea2bb7, 93 },
    { 0xa30a2bb8, 93 },
    { 0x456a2bb9, 76 },
    { 0xf39a2bba, 61 },
    { 0x3dda2bb9, 96 },
    { 0xac2a2bb7, 90 },
    { 0, 0 },
    { 0x944a2bbe, 66 },
    { 0x61aa2bbf, 99 },
    { 0, 0 },
    { 0xad7a2bc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a2bc6, 61 },
    { 0, 0 },
    { 0x42ea2bc8, 61 },
    { 0, 0 },
    { 0x26fa2bca, 50 },
    { 0xe83a2bca, 23 },
    { 0x5aea2bca, 61 },
    { 0x7bea2bcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811a2bd2, 95 },
    { 0, 0 },
    { 0x6d4a2bd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427a2bd8, 90 },
    { 0, 0 },
    { 0x279a2bda, 23 },
    { 0x1fea2bdb, 61 },
    { 0, 0 },
    { 0xfe9a2bdd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba2be1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322a2be5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa2be9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29a2bf0, 78 },
    { 0xbf3a2bf1, 23 },
    { 0, 0 },
    { 0x86ba2bf3, 33 },
    { 0x856a2bf3, 90 },
    { 0x14a2bf5, 46 },
    { 0x5e3a2bf5, 76 },
    { 0x65ea2bf6, 84 },
    { 0x4d4a2bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa2bfc, 76 },
    { 0xa00a2bfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9a2c03, 33 },
    { 0xddda2c03, 95 },
    { 0x21aa2c05, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfa2c08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbada2c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba2c18, 34 },
    { 0, 0 },
    { 0x71fa2c1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a2c1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14a2c22, 90 },
    { 0x698a2c22, 33 },
    { 0x34ba2c24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca2c27, 88 },
    { 0, 0 },
    { 0xa59a2c29, 90 },
    { 0, 0 },
    { 0x9a6a2c2b, 90 },
    { 0, 0 },
    { 0xdaaa2c2d, 34 },
    { 0xeb8a2c2d, 23 },
    { 0xebca2c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fba2c33, 34 },
    { 0xd16a2c34, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a2c38, 98 },
    { 0xb49a2c38, 75 },
    { 0x50ea2c3a, 90 },
    { 0x9bfa2c3a, 98 },
    { 0x860a2c3c, 66 },
    { 0xa16a2c3c, 90 },
    { 0x21a2c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2c42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0a2c46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca2c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a2c4c, 83 },
    { 0x891a2c4c, 67 },
    { 0, 0 },
    { 0xe4a2c4f, 84 },
    { 0x934a2c50, 98 },
    { 0x3eba2c51, 95 },
    { 0x3cca2c52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da2c56, 33 },
    { 0x1f5a2c56, 61 },
    { 0x646a2c57, 46 },
    { 0x82aa2c58, 95 },
    { 0x4cca2c5a, 95 },
    { 0, 0 },
    { 0xd32a2c5c, 95 },
    { 0x4e8a2c5d, 87 },
    { 0x4a2c5e, 61 },
    { 0x7fa2c5f, 23 },
    { 0, 0 },
    { 0xa79a2c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2c69, 55 },
    { 0x295a2c6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a2c71, 78 },
    { 0x342a2c72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ba2c76, 33 },
    { 0x1cda2c76, 76 },
    { 0x2b0a2c76, 61 },
    { 0x2e8a2c78, 55 },
    { 0xf37a2c7a, 23 },
    { 0xac2a2c7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a2c7e, 34 },
    { 0, 0 },
    { 0xb75a2c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a2c83, 34 },
    { 0xe08a2c83, 100 },
    { 0x2f4a2c85, 61 },
    { 0x12ba2c86, 77 },
    { 0xa55a2c87, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116a2c8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2c91, 98 },
    { 0x735a2c92, 43 },
    { 0x210a2c91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ba2c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa2c9d, 61 },
    { 0xe11a2c9e, 51 },
    { 0x65ea2c9f, 23 },
    { 0xe0ca2ca0, 100 },
    { 0xddea2ca1, 95 },
    { 0, 0 },
    { 0x61aa2ca3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fa2ca7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2caf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a2cb6, 33 },
    { 0x7a8a2cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcba2cbb, 33 },
    { 0xbc5a2cbb, 33 },
    { 0x7c3a2cbb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2a2cc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca2cca, 85 },
    { 0, 0 },
    { 0xe60a2ccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6a2cda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aea2cdd, 90 },
    { 0x56fa2cdd, 33 },
    { 0xe70a2cdf, 98 },
    { 0x36ea2ce0, 98 },
    { 0, 0 },
    { 0xaafa2ce2, 85 },
    { 0x7f9a2ce3, 98 },
    { 0xb5aa2ce4, 90 },
    { 0x44da2ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa2cea, 33 },
    { 0x27da2ceb, 78 },
    { 0xf20a2cea, 98 },
    { 0xa5ca2ced, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada2cf1, 66 },
    { 0x50da2cf2, 61 },
    { 0x5aea2cf3, 61 },
    { 0x3f3a2cf4, 95 },
    { 0x6a8a2cf2, 33 },
    { 0x47a2cf6, 95 },
    { 0, 0 },
    { 0x613a2cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ea2cfb, 100 },
    { 0xd17a2cfc, 66 },
    { 0xa5aa2cfc, 61 },
    { 0xfb2a2cfd, 95 },
    { 0x6e4a2cfc, 33 },
    { 0x577a2cfe, 90 },
    { 0x6faa2d01, 61 },
    { 0xb67a2cfe, 55 },
    { 0xec0a2d03, 55 },
    { 0x20fa2d03, 95 },
    { 0xeeda2d03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca2d0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4a2d12, 67 },
    { 0x682a2d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6a2d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a2d21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8a2d24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7da2d2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652a2d31, 55 },
    { 0x372a2d32, 24 },
    { 0x38fa2d33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x922a2d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a2d3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7a2d41, 90 },
    { 0x637a2d42, 98 },
    { 0x5a5a2d41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a2d4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6a2d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a2d56, 98 },
    { 0x609a2d57, 99 },
    { 0x568a2d58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba2d5c, 90 },
    { 0xdc1a2d5d, 80 },
    { 0xe34a2d5e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9a2d63, 98 },
    { 0x4eba2d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a2d6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea2d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a2d77, 33 },
    { 0, 0 },
    { 0xf42a2d79, 95 },
    { 0, 0 },
    { 0xb88a2d7b, 77 },
    { 0x3bfa2d7c, 46 },
    { 0x72da2d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ca2d91, 98 },
    { 0xd45a2d92, 84 },
    { 0x2efa2d92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9a2d9f, 61 },
    { 0, 0 },
    { 0x20aa2da1, 95 },
    { 0x134a2da1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8a2da5, 99 },
    { 0x46ea2da6, 33 },
    { 0x6d1a2da7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413a2dab, 17 },
    { 0xa06a2dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fa2db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ea2db3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68a2db7, 90 },
    { 0xa6ba2db8, 90 },
    { 0x3c0a2db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a2dbf, 90 },
    { 0x98fa2dc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84a2dc7, 52 },
    { 0, 0 },
    { 0xa3da2dc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca2dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa2dcf, 33 },
    { 0, 0 },
    { 0x3c0a2dd1, 98 },
    { 0xc06a2dd2, 95 },
    { 0x52ea2dd1, 33 },
    { 0x333a2dd4, 78 },
    { 0, 0 },
    { 0xbf8a2dd6, 17 },
    { 0x556a2dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a2ddf, 76 },
    { 0x8cda2de0, 61 },
    { 0x2eca2ddf, 96 },
    { 0, 0 },
    { 0xb65a2de3, 80 },
    { 0xc10a2de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0a2dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba2df1, 61 },
    { 0x9a3a2df1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3a2df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea2df8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2dff, 33 },
    { 0x10ca2e00, 23 },
    { 0x899a2e01, 84 },
    { 0x8e5a2e02, 43 },
    { 0x990a2e01, 23 },
    { 0x41a2e04, 90 },
    { 0, 0 },
    { 0xe17a2e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a2e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48da2e14, 90 },
    { 0, 0 },
    { 0xde9a2e16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705a2e1b, 65 },
    { 0xb6ea2e1b, 33 },
    { 0xd0ba2e1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a2e22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a2e25, 90 },
    { 0xfc8a2e26, 90 },
    { 0, 0 },
    { 0x824a2e28, 76 },
    { 0xacea2e29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0a2e2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fa2e31, 66 },
    { 0xe97a2e32, 78 },
    { 0, 0 },
    { 0xd2fa2e34, 43 },
    { 0, 0 },
    { 0x3cda2e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4a2e3a, 88 },
    { 0x58aa2e3a, 64 },
    { 0x36fa2e3c, 100 },
    { 0xa76a2e3d, 76 },
    { 0x5b8a2e3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a2e42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a2e48, 95 },
    { 0xd36a2e49, 33 },
    { 0xe91a2e48, 61 },
    { 0x41aa2e48, 95 },
    { 0xb83a2e4b, 93 },
    { 0x2a7a2e4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0a2e52, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220a2e5d, 55 },
    { 0x5aca2e5e, 52 },
    { 0x960a2e5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a2e67, 95 },
    { 0x98a2e68, 23 },
    { 0xf19a2e68, 33 },
    { 0x6f5a2e6a, 33 },
    { 0, 0 },
    { 0xbfaa2e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75a2e6f, 84 },
    { 0, 0 },
    { 0x751a2e71, 23 },
    { 0xa78a2e71, 61 },
    { 0, 0 },
    { 0x5dea2e74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95a2e79, 98 },
    { 0xe66a2e7a, 81 },
    { 0xe81a2e7a, 33 },
    { 0x50fa2e7a, 46 },
    { 0xd3fa2e7d, 98 },
    { 0x333a2e7e, 33 },
    { 0xfaba2e7f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a2e83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a2e8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fa2e8d, 90 },
    { 0x155a2e8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a2e92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1a2e99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fa2e9e, 84 },
    { 0xebaa2e9e, 33 },
    { 0, 0 },
    { 0x506a2ea1, 95 },
    { 0, 0 },
    { 0xea0a2ea3, 55 },
    { 0, 0 },
    { 0x827a2ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a2ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66a2eaf, 43 },
    { 0, 0 },
    { 0x537a2eb1, 61 },
    { 0, 0 },
    { 0x1fca2eb3, 100 },
    { 0x5d9a2eb4, 55 },
    { 0x926a2eb5, 78 },
    { 0x80ca2eb6, 33 },
    { 0x475a2eb6, 90 },
    { 0xb9ea2eb8, 23 },
    { 0, 0 },
    { 0x82ea2eba, 90 },
    { 0x81fa2eba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a2ec0, 69 },
    { 0x25ba2ec1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a2ec7, 52 },
    { 0xf7aa2ec8, 76 },
    { 0, 0 },
    { 0x62fa2eca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2a2ed3, 23 },
    { 0, 0 },
    { 0xa7fa2ed5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6a2eda, 52 },
    { 0x750a2edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a2ee0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ba2ee3, 95 },
    { 0, 0 },
    { 0xdf0a2ee5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba2ee9, 23 },
    { 0x497a2eea, 91 },
    { 0xe8ba2eea, 95 },
    { 0xcdea2eec, 55 },
    { 0xbd3a2ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a2ef4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a2ef7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a2efa, 67 },
    { 0, 0 },
    { 0x94ca2efc, 85 },
    { 0xa22a2efd, 34 },
    { 0xeeda2efd, 61 },
    { 0x520a2eff, 33 },
    { 0xb98a2f00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a2f04, 43 },
    { 0xf95a2f05, 95 },
    { 0x774a2f06, 33 },
    { 0x6c3a2f07, 66 },
    { 0xf87a2f08, 82 },
    { 0xbbaa2f09, 90 },
    { 0x173a2f0a, 66 },
    { 0xf3ba2f08, 33 },
    { 0, 0 },
    { 0x102a2f0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a2f13, 33 },
    { 0x72ca2f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a2f1a, 99 },
    { 0, 0 },
    { 0xe2ba2f1c, 33 },
    { 0xb7aa2f1c, 61 },
    { 0xeea2f1c, 23 },
    { 0xa4a2f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa2f24, 45 },
    { 0xce6a2f25, 52 },
    { 0xa4da2f24, 23 },
    { 0x3c8a2f27, 87 },
    { 0xfd0a2f28, 90 },
    { 0x989a2f28, 33 },
    { 0xdba2f28, 90 },
    { 0, 0 },
    { 0xe4da2f2c, 67 },
    { 0x685a2f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a2f30, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a2f38, 77 },
    { 0x977a2f39, 90 },
    { 0, 0 },
    { 0xa41a2f3b, 76 },
    { 0xe19a2f3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271a2f43, 46 },
    { 0x43ca2f43, 98 },
    { 0xfe6a2f45, 98 },
    { 0, 0 },
    { 0x928a2f47, 61 },
    { 0xbb6a2f48, 98 },
    { 0, 0 },
    { 0x929a2f4a, 90 },
    { 0, 0 },
    { 0x34fa2f4c, 93 },
    { 0xf39a2f4d, 98 },
    { 0xe34a2f4e, 33 },
    { 0xb3fa2f4f, 95 },
    { 0xd8ba2f4e, 34 },
    { 0xd9ea2f51, 55 },
    { 0, 0 },
    { 0xa9a2f53, 61 },
    { 0xf6aa2f54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960a2f59, 49 },
    { 0xd8ca2f59, 61 },
    { 0x40fa2f5b, 95 },
    { 0x2c9a2f5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a2f63, 33 },
    { 0x4a8a2f64, 61 },
    { 0x538a2f65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a2f6a, 43 },
    { 0, 0 },
    { 0x273a2f6c, 23 },
    { 0, 0 },
    { 0x41ba2f6e, 65 },
    { 0, 0 },
    { 0x8c4a2f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba2f77, 17 },
    { 0xaa3a2f77, 66 },
    { 0, 0 },
    { 0x899a2f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8a2f7d, 95 },
    { 0, 0 },
    { 0x264a2f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a2f89, 87 },
    { 0xbcba2f8a, 61 },
    { 0, 0 },
    { 0xbb3a2f8c, 23 },
    { 0, 0 },
    { 0x761a2f8e, 61 },
    { 0xb09a2f8f, 90 },
    { 0x841a2f8f, 52 },
    { 0xe06a2f91, 76 },
    { 0xf0ea2f8f, 98 },
    { 0x9dea2f8e, 78 },
    { 0x9c4a2f94, 98 },
    { 0x12ba2f95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a2f9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127a2f9f, 17 },
    { 0, 0 },
    { 0xbeba2fa1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761a2fa8, 44 },
    { 0, 0 },
    { 0x63a2faa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a2fae, 34 },
    { 0x7a4a2faf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76da2fb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3a2fb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9a2fc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81a2fc7, 40 },
    { 0x95ca2fc8, 34 },
    { 0x88aa2fc9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a2fcd, 90 },
    { 0x8dfa2fce, 90 },
    { 0x533a2fcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fa2fd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670a2fdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daa2fe1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca2fe4, 61 },
    { 0xd20a2fe5, 90 },
    { 0xa8ea2fe6, 90 },
    { 0x5e3a2fe5, 90 },
    { 0xbd0a2fe6, 84 },
    { 0, 0 },
    { 0xa21a2fea, 33 },
    { 0xd89a2fea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa2ff0, 61 },
    { 0x9c1a2ff0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a2ff4, 55 },
    { 0x3aa2ff4, 23 },
    { 0xc7aa2ff5, 98 },
    { 0, 0 },
    { 0x9e0a2ff8, 34 },
    { 0, 0 },
    { 0xda6a2ffa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a2ffe, 91 },
    { 0xd53a2fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea3003, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ca300a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a300d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6a3012, 55 },
    { 0xb9ea3013, 33 },
    { 0x8fa3013, 98 },
    { 0, 0 },
    { 0xe79a3016, 90 },
    { 0x5f4a3017, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfa301d, 23 },
    { 0xaba301e, 46 },
    { 0xc0a301f, 90 },
    { 0xaca3020, 34 },
    { 0x2eba3021, 90 },
    { 0, 0 },
    { 0xaa6a3023, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28a3030, 98 },
    { 0, 0 },
    { 0x59da3032, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da303b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a3044, 92 },
    { 0x6b9a3045, 98 },
    { 0x927a3044, 74 },
    { 0, 0 },
    { 0x3b2a3048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a304d, 90 },
    { 0x3b7a304d, 98 },
    { 0xca2a304e, 85 },
    { 0x9a2a304f, 90 },
    { 0x191a3051, 23 },
    { 0x50ea3052, 34 },
    { 0, 0 },
    { 0x876a3054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75a3059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a305f, 33 },
    { 0, 0 },
    { 0xb2ea3061, 90 },
    { 0x20ba3062, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799a306d, 34 },
    { 0x721a306e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa3071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6a307c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5a3081, 93 },
    { 0x71da3082, 33 },
    { 0x5a1a3083, 46 },
    { 0xca9a3082, 84 },
    { 0x32a3085, 33 },
    { 0x51ea3083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7a3089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bea308f, 77 },
    { 0x9a1a308f, 87 },
    { 0xf40a3090, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba3095, 33 },
    { 0x822a3095, 49 },
    { 0x125a3097, 61 },
    { 0x106a3097, 90 },
    { 0xeda3096, 90 },
    { 0xc39a309a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527a309e, 90 },
    { 0x81ea309f, 90 },
    { 0x770a30a0, 55 },
    { 0xe1fa30a1, 55 },
    { 0x19ba30a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafa30a8, 43 },
    { 0, 0 },
    { 0xf46a30aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746a30ae, 95 },
    { 0xd83a30af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aa30b2, 33 },
    { 0, 0 },
    { 0xc50a30b4, 90 },
    { 0x4b4a30b4, 85 },
    { 0x6fba30b6, 55 },
    { 0x954a30b5, 77 },
    { 0x8da30b4, 43 },
    { 0x823a30b8, 61 },
    { 0x5ba30b5, 93 },
    { 0x4cea30bb, 34 },
    { 0, 0 },
    { 0x124a30bd, 76 },
    { 0x1a6a30bd, 95 },
    { 0x33da30bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x501a30c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa30c5, 34 },
    { 0, 0 },
    { 0x90ea30c7, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625a30cd, 76 },
    { 0x688a30cd, 78 },
    { 0x690a30cf, 91 },
    { 0x9d5a30ce, 77 },
    { 0xa20a30d1, 19 },
    { 0x2d3a30d2, 23 },
    { 0x3e5a30cd, 91 },
    { 0xd38a30d4, 67 },
    { 0xcb1a30d4, 66 },
    { 0xdfaa30cd, 98 },
    { 0xd43a30d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482a30db, 34 },
    { 0x39aa30db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a30e1, 91 },
    { 0, 0 },
    { 0xbdda30e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3a30ea, 76 },
    { 0xbfa30ea, 23 },
    { 0xfe0a30eb, 23 },
    { 0, 0 },
    { 0xc12a30ee, 90 },
    { 0x591a30ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da30f2, 61 },
    { 0x5e8a30f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78a30f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aa30fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169a3104, 52 },
    { 0, 0 },
    { 0x10a3106, 93 },
    { 0xa7a3107, 90 },
    { 0x4e1a3108, 84 },
    { 0x13fa3108, 95 },
    { 0, 0 },
    { 0xb70a310b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ca310e, 95 },
    { 0, 0 },
    { 0x16aa3110, 98 },
    { 0, 0 },
    { 0xfdda3112, 95 },
    { 0x99ea3112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690a3117, 99 },
    { 0xb40a3118, 61 },
    { 0x6d1a3119, 90 },
    { 0x7bfa311a, 90 },
    { 0x2b4a3118, 76 },
    { 0x6a2a311c, 33 },
    { 0x91aa3118, 33 },
    { 0, 0 },
    { 0x711a311f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72a312a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeca312e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba3132, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a3137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76a313a, 2 },
    { 0xf5aa313a, 23 },
    { 0, 0 },
    { 0xe42a313d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a3141, 67 },
    { 0xaaa3141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455a3146, 43 },
    { 0x4dea3147, 98 },
    { 0x7d0a3148, 68 },
    { 0, 0 },
    { 0x2e5a314a, 66 },
    { 0, 0 },
    { 0x387a314c, 34 },
    { 0, 0 },
    { 0xd95a314e, 91 },
    { 0x3fca314f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ca3158, 76 },
    { 0x62ca3159, 100 },
    { 0x62ea315a, 52 },
    { 0, 0 },
    { 0x3cfa315c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ba315f, 90 },
    { 0x638a315f, 90 },
    { 0, 0 },
    { 0xe49a3162, 76 },
    { 0x278a3163, 23 },
    { 0xefaa3163, 61 },
    { 0, 0 },
    { 0xe49a3166, 55 },
    { 0x18fa3167, 81 },
    { 0x8dea3168, 78 },
    { 0, 0 },
    { 0x58ba316a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ba316d, 23 },
    { 0, 0 },
    { 0x321a316f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291a3177, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645a317d, 90 },
    { 0, 0 },
    { 0x9eaa317f, 33 },
    { 0x6aa3180, 95 },
    { 0x33ba3181, 33 },
    { 0x196a3182, 90 },
    { 0xda4a3182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eea3188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a318b, 46 },
    { 0x7e7a318c, 61 },
    { 0xf6fa318c, 44 },
    { 0, 0 },
    { 0x874a318f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a3192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716a3199, 95 },
    { 0x3c3a3199, 61 },
    { 0, 0 },
    { 0x59da319c, 76 },
    { 0x74ea319c, 33 },
    { 0, 0 },
    { 0x25ba319f, 84 },
    { 0, 0 },
    { 0xfe2a31a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca31aa, 98 },
    { 0xc7a31ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca31af, 91 },
    { 0xf61a31b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x848a31b3, 95 },
    { 0xb0da31b4, 55 },
    { 0, 0 },
    { 0xbaba31b6, 95 },
    { 0x1f4a31b7, 98 },
    { 0x1aa31b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cea31bc, 90 },
    { 0x682a31bd, 98 },
    { 0, 0 },
    { 0xa60a31bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0a31c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a31c6, 46 },
    { 0xa50a31c7, 75 },
    { 0xf8fa31c8, 85 },
    { 0, 0 },
    { 0x9b8a31ca, 33 },
    { 0x7e3a31cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a31d4, 23 },
    { 0x6da31d5, 61 },
    { 0xe79a31d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddda31d9, 43 },
    { 0xbb5a31da, 91 },
    { 0, 0 },
    { 0x923a31dc, 33 },
    { 0, 0 },
    { 0xf5ba31de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa31e1, 33 },
    { 0xf34a31e2, 55 },
    { 0xf6ea31e2, 43 },
    { 0, 0 },
    { 0x2b7a31e5, 99 },
    { 0xacda31e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562a31eb, 90 },
    { 0x218a31eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a31f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada31fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9a31ff, 98 },
    { 0xe60a3200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa3207, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6a320c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca320f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a3215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0a321d, 66 },
    { 0xf5ca321e, 55 },
    { 0x125a321f, 90 },
    { 0xfb5a321f, 100 },
    { 0x8c4a3221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fa322a, 90 },
    { 0, 0 },
    { 0x236a322c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59a3231, 93 },
    { 0x4ba3232, 55 },
    { 0xdf6a3231, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aca3237, 88 },
    { 0xea7a3237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90a323c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ba3240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4a3243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89a3246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a324b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4a3250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fa3257, 90 },
    { 0xbaca3257, 93 },
    { 0xa6ca3258, 95 },
    { 0xe1fa325a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba3261, 75 },
    { 0x3cfa3261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4a326a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6a326f, 34 },
    { 0x304a3270, 61 },
    { 0x71aa3270, 95 },
    { 0x2e3a3272, 61 },
    { 0xd19a3272, 95 },
    { 0x382a3274, 90 },
    { 0xadca3274, 55 },
    { 0x30ba3276, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea3279, 99 },
    { 0, 0 },
    { 0x886a327b, 76 },
    { 0x446a327b, 98 },
    { 0xd7ca327b, 78 },
    { 0xf86a327d, 93 },
    { 0x244a327d, 95 },
    { 0xad2a327c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99a3285, 76 },
    { 0xd9ea3286, 95 },
    { 0, 0 },
    { 0xdcfa3288, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea328b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803a328f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca3292, 34 },
    { 0x8e9a3292, 66 },
    { 0xd66a3294, 61 },
    { 0, 0 },
    { 0xd4da3296, 95 },
    { 0, 0 },
    { 0x103a3298, 90 },
    { 0x5b1a3299, 78 },
    { 0xeda329a, 66 },
    { 0x277a3299, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314a329f, 34 },
    { 0x923a32a0, 43 },
    { 0x4bfa32a1, 85 },
    { 0xb18a32a0, 33 },
    { 0xf9ea32a0, 34 },
    { 0x5e8a32a4, 76 },
    { 0x9ea32a5, 8 },
    { 0x4a4a32a3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdca32ac, 55 },
    { 0, 0 },
    { 0xccea32ae, 23 },
    { 0xbbba32af, 23 },
    { 0x9ea32ae, 95 },
    { 0, 0 },
    { 0x9b7a32b2, 90 },
    { 0xff6a32b3, 90 },
    { 0xe94a32b4, 66 },
    { 0xfbda32b2, 23 },
    { 0, 0 },
    { 0xb5aa32b7, 67 },
    { 0x842a32b7, 98 },
    { 0x291a32b9, 90 },
    { 0xf64a32b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6a32bd, 46 },
    { 0x56ca32be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa32c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea32c7, 98 },
    { 0xdeda32c8, 90 },
    { 0xaada32c9, 95 },
    { 0, 0 },
    { 0x361a32cb, 68 },
    { 0, 0 },
    { 0x71ca32cd, 87 },
    { 0, 0 },
    { 0x208a32cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977a32d6, 33 },
    { 0xa7fa32d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633a32de, 46 },
    { 0x9d9a32de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaa32e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a32e5, 24 },
    { 0xa1ca32e6, 61 },
    { 0, 0 },
    { 0x418a32e8, 75 },
    { 0, 0 },
    { 0x2eca32ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a32f3, 95 },
    { 0, 0 },
    { 0x32ba32f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328a32f9, 36 },
    { 0xd7ba32fa, 76 },
    { 0xa82a32fb, 55 },
    { 0x1a3a32fc, 61 },
    { 0, 0 },
    { 0x827a32fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a3301, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ca3306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ca3309, 23 },
    { 0, 0 },
    { 0xbdfa330b, 90 },
    { 0x4f1a330c, 98 },
    { 0xe1ba330c, 43 },
    { 0xb0ea330e, 84 },
    { 0x512a330f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cea3313, 17 },
    { 0, 0 },
    { 0x8b0a3315, 98 },
    { 0, 0 },
    { 0xb18a3317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a331d, 61 },
    { 0x8ba331e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca3324, 87 },
    { 0x2faa3324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10a3328, 43 },
    { 0, 0 },
    { 0x343a332a, 84 },
    { 0x59ca332b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4a3330, 95 },
    { 0x550a3331, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faa3343, 61 },
    { 0, 0 },
    { 0x8c9a3345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9a3349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825a334d, 61 },
    { 0xd7fa334e, 69 },
    { 0x798a334e, 87 },
    { 0xcf1a3350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ea3353, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987a3358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634a335c, 52 },
    { 0xb39a335c, 85 },
    { 0xfb6a335c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcea3361, 23 },
    { 0xe24a3361, 90 },
    { 0x394a3362, 17 },
    { 0xb37a3364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a3369, 85 },
    { 0, 0 },
    { 0xb58a336b, 33 },
    { 0x172a336b, 33 },
    { 0x922a336c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a3374, 98 },
    { 0x371a3375, 52 },
    { 0xc0da3376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521a3380, 90 },
    { 0, 0 },
    { 0x9d6a3382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3a3385, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dba338b, 95 },
    { 0x1bda338c, 98 },
    { 0x296a338d, 55 },
    { 0, 0 },
    { 0x3a9a338f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267a3393, 76 },
    { 0x74ca3394, 61 },
    { 0, 0 },
    { 0x77ea3396, 65 },
    { 0x9c8a3397, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecba33a3, 86 },
    { 0, 0 },
    { 0x93ca33a5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7a33ab, 66 },
    { 0xbca33ab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4a33af, 95 },
    { 0xc98a33af, 53 },
    { 0xc1a33b1, 33 },
    { 0, 0 },
    { 0xf2aa33b3, 33 },
    { 0x742a33b4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ba33b7, 78 },
    { 0x506a33b8, 66 },
    { 0, 0 },
    { 0x47a33ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a33c4, 90 },
    { 0x1bba33c5, 23 },
    { 0x89ba33c6, 90 },
    { 0xeb7a33c5, 88 },
    { 0xb49a33c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a33cd, 76 },
    { 0xc95a33cd, 90 },
    { 0xc07a33cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada33d5, 90 },
    { 0x413a33d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0a33d9, 90 },
    { 0, 0 },
    { 0xd54a33db, 90 },
    { 0x405a33dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ba33e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32a33e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea33e7, 91 },
    { 0x6cca33e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da33eb, 98 },
    { 0xe2aa33eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a33f2, 23 },
    { 0x150a33f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da33f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fba33fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ba3402, 59 },
    { 0xc69a3403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bda3406, 52 },
    { 0x3faa3406, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a340a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x999a340d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ba3411, 90 },
    { 0x55ea3412, 90 },
    { 0, 0 },
    { 0xe6ba3414, 23 },
    { 0x32da3414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a3418, 91 },
    { 0, 0 },
    { 0x638a341a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8a3423, 93 },
    { 0x687a3423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba3429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05a3431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846a3436, 48 },
    { 0x47ba3436, 90 },
    { 0, 0 },
    { 0x605a3439, 90 },
    { 0, 0 },
    { 0x306a343b, 61 },
    { 0xa36a343b, 76 },
    { 0x97ea343d, 98 },
    { 0x5dca343e, 90 },
    { 0xb43a343f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x278a3442, 90 },
    { 0x605a3443, 52 },
    { 0, 0 },
    { 0xacea3445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a344f, 34 },
    { 0, 0 },
    { 0x98ea3451, 78 },
    { 0x7c0a3452, 33 },
    { 0, 0 },
    { 0xf23a3454, 43 },
    { 0, 0 },
    { 0x5aba3456, 93 },
    { 0, 0 },
    { 0x18aa3458, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122a345c, 98 },
    { 0, 0 },
    { 0x237a345e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ea3462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627a3468, 33 },
    { 0x63ea3468, 98 },
    { 0xbf3a346a, 95 },
    { 0xd13a3469, 43 },
    { 0xc68a346c, 22 },
    { 0x3f0a346d, 98 },
    { 0x3a2a346e, 66 },
    { 0x5f3a346f, 87 },
    { 0, 0 },
    { 0x4c7a3471, 90 },
    { 0, 0 },
    { 0xc62a3473, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa3479, 55 },
    { 0, 0 },
    { 0xefea347b, 43 },
    { 0, 0 },
    { 0x84da347d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614a3482, 61 },
    { 0x1fea3483, 81 },
    { 0xb58a3483, 87 },
    { 0xbaea3484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a348c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaba348f, 33 },
    { 0, 0 },
    { 0x803a3491, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a3494, 61 },
    { 0, 0 },
    { 0x2fea3496, 64 },
    { 0, 0 },
    { 0x520a3498, 55 },
    { 0, 0 },
    { 0xb2ca349a, 95 },
    { 0x2e5a349b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a34a1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ca34a9, 76 },
    { 0xe2fa34aa, 61 },
    { 0x6a4a34aa, 90 },
    { 0x691a34aa, 90 },
    { 0, 0 },
    { 0x630a34ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa34b1, 95 },
    { 0, 0 },
    { 0xf18a34b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a34bb, 55 },
    { 0x905a34bb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81a34bf, 90 },
    { 0, 0 },
    { 0x51ea34c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba34c4, 76 },
    { 0xabfa34c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a34cf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53a34d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ea34d9, 33 },
    { 0, 0 },
    { 0xe9ca34db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a34de, 61 },
    { 0, 0 },
    { 0x951a34e0, 58 },
    { 0x828a34e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a34e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6a34e9, 55 },
    { 0, 0 },
    { 0xfcba34eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfa34f5, 33 },
    { 0, 0 },
    { 0xf4ba34f7, 33 },
    { 0x23a34f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a34fc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16a3502, 33 },
    { 0x809a3502, 95 },
    { 0, 0 },
    { 0x1aa3505, 61 },
    { 0x790a3506, 75 },
    { 0xc2aa3507, 76 },
    { 0xc6ba3508, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ea350b, 34 },
    { 0x636a350b, 34 },
    { 0xea5a350d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a3511, 33 },
    { 0xc65a3512, 23 },
    { 0, 0 },
    { 0x682a3514, 78 },
    { 0, 0 },
    { 0xc13a3516, 52 },
    { 0xf92a3517, 93 },
    { 0, 0 },
    { 0xde6a3519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aa3520, 95 },
    { 0xf76a3521, 97 },
    { 0x29ea3522, 23 },
    { 0, 0 },
    { 0x9aba3524, 98 },
    { 0x80ba3525, 34 },
    { 0, 0 },
    { 0x125a3527, 98 },
    { 0, 0 },
    { 0xd08a3529, 33 },
    { 0x726a3529, 55 },
    { 0x38da3529, 99 },
    { 0x350a352c, 81 },
    { 0, 0 },
    { 0xdc0a352e, 46 },
    { 0, 0 },
    { 0xfffa3530, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a3535, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda3542, 90 },
    { 0x2f1a3543, 76 },
    { 0x451a3544, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcca3547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da354b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16a354e, 61 },
    { 0x9e1a354e, 90 },
    { 0x602a3550, 43 },
    { 0x263a3551, 98 },
    { 0xc3a3551, 85 },
    { 0x164a3552, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a3557, 90 },
    { 0x35fa3557, 46 },
    { 0x131a3559, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6a355c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509a3562, 33 },
    { 0, 0 },
    { 0xfbfa3564, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92a356c, 91 },
    { 0xc14a356d, 23 },
    { 0xceda356e, 61 },
    { 0, 0 },
    { 0x24da3570, 34 },
    { 0x92da3571, 46 },
    { 0x683a3572, 33 },
    { 0x321a3573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6a3578, 43 },
    { 0x7eea3579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47a357c, 98 },
    { 0, 0 },
    { 0xdd5a357e, 90 },
    { 0, 0 },
    { 0x35ea3580, 66 },
    { 0x90aa3581, 23 },
    { 0x3fa3580, 99 },
    { 0x8f8a3581, 23 },
    { 0x82a3581, 99 },
    { 0x60ea3581, 46 },
    { 0x359a3586, 23 },
    { 0x428a3587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43a358e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x57da3591, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aa3594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a3597, 34 },
    { 0x766a3598, 90 },
    { 0, 0 },
    { 0x880a359a, 93 },
    { 0xf00a359b, 33 },
    { 0xc55a359c, 61 },
    { 0x92ba359d, 76 },
    { 0x225a359d, 61 },
    { 0x8b7a359f, 83 },
    { 0, 0 },
    { 0x669a35a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da35a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4a35ab, 93 },
    { 0x23ba35ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1a35b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462a35b8, 91 },
    { 0xa99a35b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ba35bd, 93 },
    { 0xd71a35be, 88 },
    { 0x332a35be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aca35c6, 23 },
    { 0, 0 },
    { 0x9ea35c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5a35cc, 34 },
    { 0x53fa35cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a35d2, 95 },
    { 0xbaa35d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a35da, 95 },
    { 0, 0 },
    { 0x7bfa35dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aa35e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca35e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174a35eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a35f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaba35f8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aa35fb, 76 },
    { 0x754a35fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a35ff, 61 },
    { 0x4f0a3600, 55 },
    { 0x6c4a3601, 36 },
    { 0, 0 },
    { 0xf45a3603, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a3608, 90 },
    { 0x4eea3609, 95 },
    { 0xeaea360a, 33 },
    { 0xa58a360a, 91 },
    { 0xcf9a360c, 85 },
    { 0x958a360a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddea3610, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62a3614, 23 },
    { 0x89fa3615, 34 },
    { 0xd03a3616, 43 },
    { 0xc77a3616, 85 },
    { 0x321a3614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a361e, 95 },
    { 0, 0 },
    { 0xbbba3620, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fa3623, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8a3626, 46 },
    { 0x625a3627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da362d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59a3630, 33 },
    { 0x225a3631, 43 },
    { 0xd75a3631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ea3638, 23 },
    { 0, 0 },
    { 0xb0a363a, 76 },
    { 0x367a363b, 90 },
    { 0xb52a363c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16a3642, 68 },
    { 0xfc6a3643, 76 },
    { 0xd85a3642, 90 },
    { 0x425a3644, 99 },
    { 0x499a3646, 61 },
    { 0x83ca3643, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a364c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1a3650, 55 },
    { 0x3b5a3650, 43 },
    { 0xef5a3652, 76 },
    { 0, 0 },
    { 0xa7aa3654, 55 },
    { 0, 0 },
    { 0x99aa3656, 78 },
    { 0x7c0a3657, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8a365a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a3661, 75 },
    { 0xfb8a3662, 34 },
    { 0x83a3663, 46 },
    { 0xafa3662, 100 },
    { 0, 0 },
    { 0x360a3666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a3669, 94 },
    { 0, 0 },
    { 0x945a366b, 98 },
    { 0x688a366c, 98 },
    { 0xa9ea366c, 61 },
    { 0, 0 },
    { 0xb23a366f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea3679, 23 },
    { 0xca0a3679, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a367e, 94 },
    { 0, 0 },
    { 0x240a3680, 55 },
    { 0, 0 },
    { 0x616a3682, 33 },
    { 0xd45a3683, 76 },
    { 0xc92a3684, 59 },
    { 0x835a3682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43a368d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fea3691, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aba3696, 33 },
    { 0xb9ba3696, 61 },
    { 0x833a3697, 77 },
    { 0x991a3699, 23 },
    { 0x3dfa369a, 61 },
    { 0x103a369a, 76 },
    { 0, 0 },
    { 0xebba369d, 95 },
    { 0x6b0a369d, 61 },
    { 0x99fa369f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca36a3, 90 },
    { 0xccba36a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a36a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a36aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248a36ae, 64 },
    { 0, 0 },
    { 0x4cea36b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa36b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fa36bb, 41 },
    { 0, 0 },
    { 0xa2ca36bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a36c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a36c3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efa36c6, 33 },
    { 0x3c5a36c7, 61 },
    { 0x5b7a36c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaa36cc, 34 },
    { 0x708a36cd, 93 },
    { 0x67da36cd, 90 },
    { 0x5a4a36cf, 90 },
    { 0x2ffa36d0, 66 },
    { 0x43a36d1, 95 },
    { 0xce5a36d0, 34 },
    { 0xd92a36d3, 90 },
    { 0x22ba36d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ea36d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aa36dc, 23 },
    { 0xb6ca36dc, 43 },
    { 0xda9a36de, 43 },
    { 0x6a3a36df, 61 },
    { 0xb14a36e0, 95 },
    { 0xcf1a36dd, 90 },
    { 0x2aea36de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ba36e8, 98 },
    { 0x7daa36e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a36ec, 55 },
    { 0x5a0a36ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497a36f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aa36fb, 90 },
    { 0xde5a36fc, 52 },
    { 0x3d0a36fd, 61 },
    { 0x6a2a36fd, 90 },
    { 0x993a36ff, 68 },
    { 0xee9a36fd, 98 },
    { 0xe33a3701, 52 },
    { 0, 0 },
    { 0xe21a3703, 55 },
    { 0x9f3a3704, 52 },
    { 0x1cda3703, 95 },
    { 0x7b8a3706, 36 },
    { 0x9f9a3706, 90 },
    { 0x17da3708, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca370b, 95 },
    { 0x42aa370b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6a3711, 61 },
    { 0x320a3711, 33 },
    { 0, 0 },
    { 0x31da3714, 49 },
    { 0x40da3715, 99 },
    { 0xba5a3716, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a371b, 33 },
    { 0, 0 },
    { 0x3b3a371d, 76 },
    { 0xff8a371e, 49 },
    { 0xdfaa371f, 43 },
    { 0x5a4a371d, 55 },
    { 0xc44a371f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a3727, 95 },
    { 0x8a9a3728, 90 },
    { 0xdcba3729, 55 },
    { 0x48a3727, 81 },
    { 0xbd3a372b, 95 },
    { 0xe20a3729, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ba3732, 17 },
    { 0x5eca3732, 85 },
    { 0, 0 },
    { 0x96a3735, 40 },
    { 0xff6a3736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a3739, 33 },
    { 0x364a373a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea373f, 98 },
    { 0x970a3740, 95 },
    { 0x2b3a3741, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba3748, 34 },
    { 0x843a3749, 34 },
    { 0x934a3748, 55 },
    { 0, 0 },
    { 0x9cda374c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a374f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06a3752, 90 },
    { 0x685a3753, 33 },
    { 0x483a3752, 95 },
    { 0x257a3755, 23 },
    { 0, 0 },
    { 0x85ba3757, 91 },
    { 0xc9aa3757, 88 },
    { 0, 0 },
    { 0xe6aa375a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996a375f, 96 },
    { 0x397a375f, 91 },
    { 0xf8ca3760, 43 },
    { 0x7faa3760, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da3765, 61 },
    { 0x9aba3766, 98 },
    { 0x6a9a3765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a376b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a376e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdba3771, 61 },
    { 0x8a2a3772, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01a3777, 95 },
    { 0, 0 },
    { 0xce4a3779, 95 },
    { 0xbc4a3779, 66 },
    { 0xfbaa3779, 61 },
    { 0x2e4a377c, 95 },
    { 0x86ea377d, 91 },
    { 0x553a377e, 66 },
    { 0x841a377d, 68 },
    { 0x8fda3780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aa3783, 43 },
    { 0xfcaa3784, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a3787, 91 },
    { 0, 0 },
    { 0x56da3789, 23 },
    { 0xb6ba378a, 46 },
    { 0, 0 },
    { 0xbf3a378c, 90 },
    { 0x2d8a378c, 33 },
    { 0, 0 },
    { 0x180a378f, 90 },
    { 0xeda3790, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a3793, 33 },
    { 0xd25a3793, 89 },
    { 0xe52a3795, 90 },
    { 0xb28a3796, 36 },
    { 0x7fba3797, 23 },
    { 0xaaba3798, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27a379b, 55 },
    { 0xd7ea379c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a379f, 61 },
    { 0x4b2a37a0, 43 },
    { 0x8a8a379f, 95 },
    { 0x56ba37a2, 95 },
    { 0, 0 },
    { 0x260a37a4, 75 },
    { 0xbe7a37a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a37a8, 17 },
    { 0x751a37a9, 90 },
    { 0, 0 },
    { 0x151a37ab, 55 },
    { 0, 0 },
    { 0x3a1a37ad, 34 },
    { 0xec8a37ae, 90 },
    { 0x8e3a37af, 61 },
    { 0xb79a37b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x352a37b3, 98 },
    { 0x21fa37b4, 61 },
    { 0xed0a37b5, 78 },
    { 0x72ea37b6, 90 },
    { 0x6d4a37b7, 61 },
    { 0x355a37b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650a37bf, 23 },
    { 0x1ada37c0, 76 },
    { 0xc85a37bf, 43 },
    { 0x792a37c2, 94 },
    { 0xa3aa37c3, 34 },
    { 0, 0 },
    { 0xe1a37c5, 46 },
    { 0x1cda37c5, 65 },
    { 0, 0 },
    { 0x67aa37c8, 98 },
    { 0, 0 },
    { 0x845a37ca, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09a37d4, 61 },
    { 0x982a37d5, 34 },
    { 0xb8da37d6, 90 },
    { 0xecfa37d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ba37db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a37df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a37e2, 76 },
    { 0x9bda37e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ca37e6, 100 },
    { 0xbfca37e7, 90 },
    { 0xf33a37e8, 34 },
    { 0xf61a37e9, 17 },
    { 0x6b9a37ea, 98 },
    { 0xa4a37e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252a37ef, 55 },
    { 0x935a37ef, 61 },
    { 0xb0aa37f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fa37f6, 91 },
    { 0x3dba37f7, 87 },
    { 0xcb2a37f8, 38 },
    { 0xf63a37f9, 90 },
    { 0x844a37fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363a37fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39a3801, 76 },
    { 0xdfaa3802, 91 },
    { 0x80ba3801, 33 },
    { 0x851a3804, 90 },
    { 0x8f4a3805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca3809, 100 },
    { 0, 0 },
    { 0x345a380b, 90 },
    { 0x502a380b, 93 },
    { 0xe6aa380c, 98 },
    { 0x9b0a380e, 66 },
    { 0xa94a380f, 95 },
    { 0x1caa3810, 88 },
    { 0x903a3810, 90 },
    { 0, 0 },
    { 0xb2ba3813, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a3827, 33 },
    { 0x1eca3828, 94 },
    { 0x351a3828, 90 },
    { 0x51da382a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3da382d, 33 },
    { 0, 0 },
    { 0xd2ea382f, 90 },
    { 0x4c1a3830, 61 },
    { 0, 0 },
    { 0x99ba3832, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa3838, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10da383b, 83 },
    { 0, 0 },
    { 0xdd2a383d, 95 },
    { 0, 0 },
    { 0x7cda383f, 99 },
    { 0x652a383f, 92 },
    { 0xb2a3840, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a3844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a384a, 66 },
    { 0xbbaa384b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ba385b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9a3864, 33 },
    { 0xedea3864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229a386d, 55 },
    { 0x76fa386d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa3872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49da3875, 90 },
    { 0xd3ba3876, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6da3883, 23 },
    { 0, 0 },
    { 0x39ea3885, 76 },
    { 0x992a3886, 95 },
    { 0, 0 },
    { 0xb96a3888, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefca388b, 93 },
    { 0, 0 },
    { 0x7f8a388d, 99 },
    { 0x200a388e, 64 },
    { 0xa0aa388f, 87 },
    { 0xeea388d, 78 },
    { 0x5e4a3891, 34 },
    { 0, 0 },
    { 0x444a3893, 98 },
    { 0xeada3893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaa389a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba389d, 76 },
    { 0x6e3a389e, 33 },
    { 0x8b6a389f, 34 },
    { 0x963a38a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ea38a7, 91 },
    { 0x820a38a7, 36 },
    { 0x8f2a38a9, 95 },
    { 0xa0da38a8, 38 },
    { 0x730a38a7, 77 },
    { 0, 0 },
    { 0x473a38ad, 33 },
    { 0x4fea38ae, 23 },
    { 0x560a38ae, 99 },
    { 0, 0 },
    { 0xbe2a38b1, 90 },
    { 0, 0 },
    { 0x78a38b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59a38b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca38bc, 98 },
    { 0xf19a38bc, 87 },
    { 0xddfa38be, 33 },
    { 0xc2da38bf, 61 },
    { 0xad9a38c0, 23 },
    { 0xe5ea38bd, 95 },
    { 0, 0 },
    { 0x775a38c3, 78 },
    { 0, 0 },
    { 0x7b1a38c5, 34 },
    { 0x1fca38c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcaa38ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x924a38cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a38d0, 76 },
    { 0x557a38d1, 61 },
    { 0x84aa38d1, 33 },
    { 0x32ea38d2, 76 },
    { 0x2c6a38d3, 98 },
    { 0x4a1a38d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8a38dd, 90 },
    { 0, 0 },
    { 0xe48a38df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39a38e2, 65 },
    { 0, 0 },
    { 0x397a38e4, 98 },
    { 0, 0 },
    { 0x649a38e6, 34 },
    { 0, 0 },
    { 0x11a38e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9da38eb, 93 },
    { 0, 0 },
    { 0xfdfa38ed, 98 },
    { 0x3e3a38ed, 81 },
    { 0, 0 },
    { 0xf1aa38f0, 90 },
    { 0x8c3a38f1, 23 },
    { 0xf41a38f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9a38f5, 33 },
    { 0x7eca38f6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x278a38f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x142a38fc, 55 },
    { 0xc0ba38fd, 33 },
    { 0, 0 },
    { 0x59aa38ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x777a3902, 93 },
    { 0x2afa3903, 33 },
    { 0xcbca3902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2a3909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdea390e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafa3915, 61 },
    { 0xc53a3915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982a391b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a3923, 90 },
    { 0xe6ea3924, 61 },
    { 0x275a3925, 98 },
    { 0, 0 },
    { 0x653a3927, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a392b, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128a3933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2a3938, 90 },
    { 0, 0 },
    { 0x706a393a, 33 },
    { 0x272a393a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0a3946, 23 },
    { 0x16da3947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2a394d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba395a, 59 },
    { 0x299a395b, 34 },
    { 0, 0 },
    { 0xeb1a395d, 33 },
    { 0, 0 },
    { 0x8b5a395f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x602a3962, 33 },
    { 0, 0 },
    { 0x5eca3964, 23 },
    { 0xc46a3965, 46 },
    { 0, 0 },
    { 0x633a3967, 65 },
    { 0x10aa3968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa396f, 87 },
    { 0, 0 },
    { 0x1fea3971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8a3976, 91 },
    { 0xa7ca3977, 34 },
    { 0x9c5a3977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ca397d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ca3983, 95 },
    { 0x3b1a3983, 90 },
    { 0, 0 },
    { 0xc01a3986, 93 },
    { 0x5a6a3986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a398a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6da398f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81aa3992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x430a3995, 94 },
    { 0xbffa3995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a3999, 46 },
    { 0xa28a3999, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a399f, 33 },
    { 0xc23a399f, 23 },
    { 0x9aea399f, 99 },
    { 0xd72a399f, 76 },
    { 0x494a39a0, 12 },
    { 0x465a39a4, 61 },
    { 0xb81a39a1, 78 },
    { 0x6c9a39a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48a39aa, 90 },
    { 0x61a39ab, 95 },
    { 0, 0 },
    { 0x278a39ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8a39b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da39b8, 61 },
    { 0, 0 },
    { 0xb3a39ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9a39c0, 33 },
    { 0, 0 },
    { 0x80fa39c2, 33 },
    { 0x72ba39c2, 34 },
    { 0x2e4a39c4, 55 },
    { 0xa69a39c5, 33 },
    { 0x7d2a39c6, 93 },
    { 0x29da39c6, 3 },
    { 0x725a39c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a39cd, 55 },
    { 0, 0 },
    { 0xdc1a39cf, 34 },
    { 0x3e3a39cf, 88 },
    { 0, 0 },
    { 0x9aba39d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeba39d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44da39db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a39de, 98 },
    { 0x52ca39df, 69 },
    { 0x2caa39e0, 33 },
    { 0, 0 },
    { 0x558a39e2, 23 },
    { 0x3eea39e3, 23 },
    { 0, 0 },
    { 0xca0a39e5, 90 },
    { 0xf8a39e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a39ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ba39f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a39fb, 55 },
    { 0, 0 },
    { 0x622a39fd, 99 },
    { 0xcc6a39fd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3a3a04, 61 },
    { 0xbcaa3a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a3a09, 36 },
    { 0, 0 },
    { 0xc95a3a0b, 91 },
    { 0, 0 },
    { 0x39ea3a0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cca3a10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830a3a15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aa3a19, 76 },
    { 0x7d4a3a19, 76 },
    { 0, 0 },
    { 0x9b4a3a1c, 76 },
    { 0xd29a3a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bda3a23, 33 },
    { 0xacaa3a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69a3a28, 91 },
    { 0x424a3a29, 85 },
    { 0xac6a3a28, 55 },
    { 0, 0 },
    { 0x95da3a2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289a3a33, 98 },
    { 0xfdea3a33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2a3a3b, 90 },
    { 0xbe0a3a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368a3a42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fa3a46, 23 },
    { 0, 0 },
    { 0x1b5a3a48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbea3a4b, 33 },
    { 0xd36a3a4b, 23 },
    { 0xb8ba3a4c, 66 },
    { 0xeb4a3a4e, 98 },
    { 0xbd2a3a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea3a54, 33 },
    { 0x582a3a55, 33 },
    { 0xf7ba3a56, 90 },
    { 0xd23a3a55, 93 },
    { 0x311a3a55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ca3a5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63a3a5f, 90 },
    { 0x798a3a60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a3a64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a3a67, 34 },
    { 0xb5da3a68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3a3a6e, 46 },
    { 0x52a3a6e, 95 },
    { 0, 0 },
    { 0x42ca3a71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da3a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa3a7c, 61 },
    { 0, 0 },
    { 0xc41a3a7e, 90 },
    { 0x7c2a3a7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32a3a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea3a8f, 90 },
    { 0xb04a3a90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735a3a94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01a3a9a, 93 },
    { 0xfdaa3a9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6da3aa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea3aa6, 95 },
    { 0x339a3aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35a3aac, 61 },
    { 0, 0 },
    { 0xaf5a3aae, 43 },
    { 0, 0 },
    { 0x3a7a3ab0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3a3ab7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71a3abc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891a3ac6, 43 },
    { 0x312a3ac7, 87 },
    { 0x765a3ac8, 61 },
    { 0x8f6a3ac8, 95 },
    { 0xcd1a3ac6, 98 },
    { 0, 0 },
    { 0x5a2a3acc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07a3acf, 61 },
    { 0x597a3ad0, 61 },
    { 0x84aa3ad0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a3ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a3ad8, 88 },
    { 0x7c4a3ad9, 34 },
    { 0x107a3ada, 40 },
    { 0x49da3ada, 55 },
    { 0, 0 },
    { 0x289a3add, 98 },
    { 0xf15a3add, 97 },
    { 0x2aaa3add, 33 },
    { 0x7b3a3ae0, 33 },
    { 0x688a3ae1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ba3ae4, 91 },
    { 0xfdba3ae4, 90 },
    { 0xad2a3ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eca3ae9, 90 },
    { 0xe80a3aea, 98 },
    { 0xad4a3ae9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a3af2, 95 },
    { 0, 0 },
    { 0x942a3af4, 92 },
    { 0xbdca3af4, 61 },
    { 0x809a3af6, 66 },
    { 0, 0 },
    { 0x663a3af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440a3afd, 95 },
    { 0xf9a3afd, 90 },
    { 0xd31a3aff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a3b09, 34 },
    { 0x339a3b0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a3b10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0a3b13, 23 },
    { 0xff2a3b13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba3b18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a3b1b, 43 },
    { 0, 0 },
    { 0x8aca3b1d, 23 },
    { 0xcaaa3b1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a3b21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a3b26, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01a3b2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5a3b2d, 90 },
    { 0, 0 },
    { 0x811a3b2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20a3b32, 61 },
    { 0x1e0a3b32, 23 },
    { 0x6dda3b34, 90 },
    { 0x546a3b35, 23 },
    { 0x96ba3b36, 85 },
    { 0xddfa3b37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba3b3c, 91 },
    { 0, 0 },
    { 0x53da3b3e, 61 },
    { 0, 0 },
    { 0x745a3b40, 95 },
    { 0, 0 },
    { 0xebaa3b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29a3b54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4a3b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5a3b5b, 95 },
    { 0xd03a3b5c, 100 },
    { 0x8d6a3b5d, 90 },
    { 0, 0 },
    { 0x698a3b5f, 23 },
    { 0x1fca3b60, 95 },
    { 0, 0 },
    { 0x780a3b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78aa3b6a, 34 },
    { 0xd3ea3b6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36a3b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dea3b79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1da3b7d, 98 },
    { 0x53ea3b7e, 78 },
    { 0x68fa3b7f, 98 },
    { 0, 0 },
    { 0xfdca3b81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a3b87, 33 },
    { 0, 0 },
    { 0x168a3b89, 81 },
    { 0xf76a3b8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aa3b8d, 33 },
    { 0x944a3b8e, 33 },
    { 0x33ca3b8d, 61 },
    { 0x7a4a3b8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a3b93, 90 },
    { 0, 0 },
    { 0xb9aa3b95, 40 },
    { 0, 0 },
    { 0xca7a3b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3a3b9a, 76 },
    { 0xc85a3b9b, 99 },
    { 0x464a3b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fa3ba2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dda3baa, 43 },
    { 0x270a3bab, 99 },
    { 0xc39a3bac, 23 },
    { 0, 0 },
    { 0xe06a3bae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78da3bb5, 33 },
    { 0x210a3bb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ca3bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ca3bc1, 43 },
    { 0x33da3bc1, 90 },
    { 0x115a3bc3, 90 },
    { 0x99ba3bc4, 80 },
    { 0x394a3bc5, 61 },
    { 0xce9a3bc6, 65 },
    { 0x906a3bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1a3bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a3bcd, 35 },
    { 0, 0 },
    { 0xa96a3bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a3bd9, 77 },
    { 0xd93a3bda, 33 },
    { 0xbd1a3bda, 91 },
    { 0xa96a3bdc, 33 },
    { 0, 0 },
    { 0x96fa3bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4a3be2, 98 },
    { 0, 0 },
    { 0x8b9a3be4, 95 },
    { 0x2b8a3be4, 95 },
    { 0xfb9a3be5, 90 },
    { 0x651a3be5, 61 },
    { 0xc0ba3be8, 95 },
    { 0xfa3a3be9, 67 },
    { 0x6e6a3be8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baa3bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa3bf0, 53 },
    { 0, 0 },
    { 0xdc4a3bf2, 33 },
    { 0x98a3bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da3bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ea3c04, 98 },
    { 0x976a3c05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a3c09, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27a3c0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2a3c10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ea3c14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a3c19, 97 },
    { 0x711a3c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37a3c1d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a3c27, 99 },
    { 0x8bea3c28, 95 },
    { 0xd7da3c28, 75 },
    { 0xeb5a3c2a, 33 },
    { 0, 0 },
    { 0xe08a3c2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a3c2f, 95 },
    { 0, 0 },
    { 0x5e7a3c31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3a3c34, 61 },
    { 0x32ba3c35, 43 },
    { 0x6e8a3c34, 90 },
    { 0x9baa3c37, 90 },
    { 0x1c9a3c38, 98 },
    { 0x50a3c39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9a3c3e, 61 },
    { 0x9f5a3c3f, 61 },
    { 0x964a3c40, 23 },
    { 0, 0 },
    { 0x525a3c42, 23 },
    { 0x41a3c42, 98 },
    { 0x55a3c44, 23 },
    { 0xb6fa3c45, 96 },
    { 0x412a3c44, 23 },
    { 0, 0 },
    { 0x1b1a3c48, 36 },
    { 0xd18a3c49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a3c4d, 81 },
    { 0xfbea3c4e, 90 },
    { 0xe72a3c4f, 95 },
    { 0x10ca3c4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea3c53, 66 },
    { 0, 0 },
    { 0x5fba3c55, 95 },
    { 0x5c8a3c56, 90 },
    { 0x5eca3c57, 17 },
    { 0xca1a3c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fa3c5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a3c5f, 33 },
    { 0xbc6a3c5f, 90 },
    { 0xf15a3c61, 67 },
    { 0x3a2a3c62, 95 },
    { 0, 0 },
    { 0xcaea3c64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90a3c67, 33 },
    { 0x4e0a3c68, 55 },
    { 0xd1aa3c69, 33 },
    { 0xa94a3c69, 34 },
    { 0xad4a3c6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675a3c6f, 67 },
    { 0xd62a3c70, 95 },
    { 0, 0 },
    { 0x161a3c72, 33 },
    { 0xfdba3c73, 61 },
    { 0x7c2a3c74, 34 },
    { 0x631a3c75, 33 },
    { 0xee2a3c76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3c7c, 33 },
    { 0x2e0a3c7c, 61 },
    { 0xf3da3c7e, 77 },
    { 0, 0 },
    { 0x303a3c80, 61 },
    { 0, 0 },
    { 0x34da3c82, 33 },
    { 0, 0 },
    { 0x170a3c84, 84 },
    { 0, 0 },
    { 0x6faa3c86, 33 },
    { 0xbefa3c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea3c93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957a3c98, 95 },
    { 0x91fa3c99, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a3c9e, 66 },
    { 0x817a3c9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a3ca5, 23 },
    { 0x9c1a3ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fca3caa, 84 },
    { 0, 0 },
    { 0xfb1a3cac, 61 },
    { 0x5c8a3cad, 90 },
    { 0x209a3cad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a3cb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9a3cbc, 55 },
    { 0x4eaa3cbd, 98 },
    { 0x49a3cbd, 41 },
    { 0x347a3cbd, 40 },
    { 0x337a3cc0, 75 },
    { 0x85fa3cc1, 66 },
    { 0x2ca3cbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa3cc9, 52 },
    { 0xd85a3cc9, 90 },
    { 0x3efa3cca, 23 },
    { 0x978a3cca, 95 },
    { 0x1ca3ccd, 61 },
    { 0x5f2a3cce, 83 },
    { 0, 0 },
    { 0xae4a3cd0, 95 },
    { 0, 0 },
    { 0x8bba3cd2, 23 },
    { 0, 0 },
    { 0x40da3cd4, 33 },
    { 0xc3a3cd4, 66 },
    { 0xb7ea3cd6, 76 },
    { 0xb06a3cd7, 98 },
    { 0xe84a3cd6, 98 },
    { 0x945a3cd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a3cdd, 61 },
    { 0, 0 },
    { 0xcaaa3cdf, 76 },
    { 0, 0 },
    { 0x2c2a3ce1, 95 },
    { 0, 0 },
    { 0xc31a3ce3, 52 },
    { 0, 0 },
    { 0x27ca3ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fa3cfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca3cfd, 95 },
    { 0, 0 },
    { 0x443a3cff, 33 },
    { 0x34da3d00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3d0a, 46 },
    { 0x834a3d0b, 90 },
    { 0xa22a3d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23a3d13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x954a3d16, 33 },
    { 0x125a3d17, 93 },
    { 0xcfea3d18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a3d1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a3d1e, 90 },
    { 0xb00a3d1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5a3d22, 93 },
    { 0x6c7a3d22, 90 },
    { 0xa5fa3d24, 33 },
    { 0x67ea3d24, 61 },
    { 0, 0 },
    { 0xb64a3d27, 33 },
    { 0x9e8a3d27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebda3d2b, 35 },
    { 0x931a3d2b, 98 },
    { 0xa9a3d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a3d31, 93 },
    { 0x7e7a3d32, 76 },
    { 0x5ada3d32, 75 },
    { 0, 0 },
    { 0x8eca3d35, 98 },
    { 0, 0 },
    { 0xd28a3d37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x777a3d3a, 91 },
    { 0xbb2a3d3a, 33 },
    { 0, 0 },
    { 0x5b5a3d3d, 76 },
    { 0x1cba3d3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0a3d4a, 78 },
    { 0xf27a3d4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba3d4e, 88 },
    { 0x8f3a3d4e, 95 },
    { 0, 0 },
    { 0xb2ea3d51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda3d56, 99 },
    { 0x85ca3d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a3d5a, 98 },
    { 0xaeca3d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366a3d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda3d66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ca3d69, 95 },
    { 0, 0 },
    { 0x7aea3d6b, 91 },
    { 0, 0 },
    { 0x8a9a3d6d, 23 },
    { 0x92ca3d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a3d73, 55 },
    { 0, 0 },
    { 0xbb4a3d75, 61 },
    { 0, 0 },
    { 0xaf4a3d77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x641a3d7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x191a3d7d, 23 },
    { 0x506a3d7e, 55 },
    { 0x6b2a3d7f, 87 },
    { 0x78ca3d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a3d83, 90 },
    { 0x7e9a3d84, 33 },
    { 0xddaa3d85, 33 },
    { 0xd26a3d83, 95 },
    { 0x76ca3d87, 90 },
    { 0xccba3d87, 33 },
    { 0x71a3d85, 76 },
    { 0x84ca3d8a, 90 },
    { 0xab9a3d88, 90 },
    { 0, 0 },
    { 0x3eda3d8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a3d90, 61 },
    { 0x274a3d91, 90 },
    { 0x498a3d92, 91 },
    { 0xec2a3d93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a3d96, 43 },
    { 0, 0 },
    { 0xc2ca3d98, 33 },
    { 0x27ba3d99, 87 },
    { 0x6b9a3d9a, 95 },
    { 0xb99a3d9b, 61 },
    { 0, 0 },
    { 0x247a3d9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195a3da2, 46 },
    { 0, 0 },
    { 0xd48a3da4, 91 },
    { 0x349a3da5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14a3dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a3db1, 98 },
    { 0, 0 },
    { 0x61ca3db3, 23 },
    { 0, 0 },
    { 0xad7a3db5, 95 },
    { 0x112a3db6, 90 },
    { 0, 0 },
    { 0x3aba3db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ea3dbd, 23 },
    { 0x6a3a3dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da3dc6, 34 },
    { 0x8a0a3dc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a3dcd, 23 },
    { 0, 0 },
    { 0x995a3dcf, 40 },
    { 0x21a3dcf, 98 },
    { 0xbffa3dd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aa3dd8, 85 },
    { 0x13ca3dd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dca3ddd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba3de2, 95 },
    { 0xf4fa3de3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a3def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca3df2, 90 },
    { 0xab7a3df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3a3df7, 88 },
    { 0x654a3df7, 23 },
    { 0xd35a3df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ba3e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810a3e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13a3e09, 93 },
    { 0xcd5a3e0a, 95 },
    { 0xdf8a3e0b, 91 },
    { 0x13ea3e0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aa3e12, 84 },
    { 0x892a3e13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a3e17, 83 },
    { 0xaa7a3e18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa3e21, 76 },
    { 0, 0 },
    { 0xd4fa3e23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaa3e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa3e2b, 23 },
    { 0, 0 },
    { 0x190a3e2d, 98 },
    { 0x7dea3e2d, 98 },
    { 0x552a3e2f, 23 },
    { 0, 0 },
    { 0xf3ba3e31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a3e38, 95 },
    { 0xc4ba3e39, 61 },
    { 0x89aa3e3a, 52 },
    { 0xfcda3e3a, 66 },
    { 0x39fa3e3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5da3e40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7a3e44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x993a3e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa3e4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1a3e55, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a3e58, 99 },
    { 0x5bca3e58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a3e5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ba3e62, 61 },
    { 0xcb5a3e63, 98 },
    { 0xda5a3e63, 69 },
    { 0x4d5a3e63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93a3e6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da3e6f, 98 },
    { 0x16ea3e70, 90 },
    { 0x6bba3e6f, 95 },
    { 0x51ea3e72, 91 },
    { 0, 0 },
    { 0x49ba3e74, 34 },
    { 0xa4ba3e75, 90 },
    { 0xf71a3e74, 87 },
    { 0x270a3e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0a3e7d, 76 },
    { 0x1b4a3e7d, 52 },
    { 0x6baa3e7f, 52 },
    { 0x1fca3e80, 100 },
    { 0x376a3e81, 46 },
    { 0, 0 },
    { 0x640a3e83, 24 },
    { 0x7daa3e83, 55 },
    { 0xb97a3e83, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aa3e8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a3e8e, 61 },
    { 0x2b9a3e8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8a3e95, 61 },
    { 0x211a3e95, 95 },
    { 0, 0 },
    { 0x231a3e98, 93 },
    { 0xdeda3e99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fa3e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973a3ea1, 87 },
    { 0xa33a3ea2, 90 },
    { 0xa9ca3ea3, 33 },
    { 0xcffa3ea3, 66 },
    { 0x6d7a3ea5, 37 },
    { 0x987a3ea5, 33 },
    { 0, 0 },
    { 0xb9fa3ea8, 66 },
    { 0, 0 },
    { 0xaba3eaa, 66 },
    { 0, 0 },
    { 0x98a3eac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a3eaf, 90 },
    { 0xa71a3eb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fda3eb4, 61 },
    { 0x4d5a3eb5, 90 },
    { 0, 0 },
    { 0xb7aa3eb7, 78 },
    { 0x975a3eb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da3ebb, 66 },
    { 0xbcba3ebc, 61 },
    { 0x8b9a3ebc, 95 },
    { 0x4afa3ebe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eba3ec1, 90 },
    { 0, 0 },
    { 0x76a3ec3, 81 },
    { 0, 0 },
    { 0x6eea3ec5, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53a3ecb, 95 },
    { 0, 0 },
    { 0xca2a3ecd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ea3ed1, 33 },
    { 0, 0 },
    { 0xe33a3ed3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a3ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eda3ee3, 95 },
    { 0xa9ca3ee4, 90 },
    { 0xd22a3ee4, 43 },
    { 0, 0 },
    { 0xb51a3ee7, 90 },
    { 0x5ada3ee7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ea3eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a3ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fa3ef8, 33 },
    { 0x68ba3ef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaa3efc, 76 },
    { 0xab5a3efd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a3f00, 66 },
    { 0, 0 },
    { 0xf56a3f02, 33 },
    { 0, 0 },
    { 0xdf8a3f04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a3f0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x914a3f0d, 33 },
    { 0, 0 },
    { 0x968a3f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4a3f15, 100 },
    { 0, 0 },
    { 0xb15a3f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710a3f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a3f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaada3f25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a3f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ba3f2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x269a3f2e, 66 },
    { 0xa5ca3f2f, 33 },
    { 0xcd9a3f2e, 33 },
    { 0xecba3f31, 90 },
    { 0x6c8a3f31, 76 },
    { 0x889a3f33, 17 },
    { 0x8d7a3f34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4a3f3a, 33 },
    { 0x19fa3f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4a3f3f, 43 },
    { 0xdcca3f3f, 91 },
    { 0xd9a3f40, 90 },
    { 0xf16a3f42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a3f46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3f4a, 94 },
    { 0x9d6a3f4a, 33 },
    { 0x7b1a3f4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffa3f4f, 90 },
    { 0, 0 },
    { 0x88da3f51, 98 },
    { 0x7daa3f51, 55 },
    { 0x432a3f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa3f57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407a3f64, 91 },
    { 0x9e5a3f64, 90 },
    { 0x558a3f66, 76 },
    { 0x891a3f67, 43 },
    { 0, 0 },
    { 0x5e0a3f69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47a3f6d, 61 },
    { 0x1a7a3f6e, 90 },
    { 0x1f5a3f6d, 81 },
    { 0, 0 },
    { 0x638a3f71, 90 },
    { 0, 0 },
    { 0x554a3f73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bda3f78, 95 },
    { 0x68aa3f79, 40 },
    { 0, 0 },
    { 0x891a3f7b, 66 },
    { 0x580a3f7b, 84 },
    { 0, 0 },
    { 0x9f7a3f7e, 33 },
    { 0, 0 },
    { 0x2afa3f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3da3f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a3f88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97da3f97, 90 },
    { 0x868a3f97, 33 },
    { 0x518a3f97, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7a3fa1, 81 },
    { 0x8d4a3fa2, 23 },
    { 0xfc7a3fa3, 84 },
    { 0xceaa3fa1, 87 },
    { 0, 0 },
    { 0x338a3fa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x382a3fa9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6a3fb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a3fb3, 34 },
    { 0x415a3fb3, 33 },
    { 0xc0ea3fb4, 55 },
    { 0xee0a3fb6, 43 },
    { 0xdc8a3fb5, 98 },
    { 0xecaa3fb8, 76 },
    { 0x798a3fb9, 88 },
    { 0x793a3fb3, 90 },
    { 0x5c4a3fbb, 23 },
    { 0x2baa3fbb, 23 },
    { 0, 0 },
    { 0x8f2a3fbe, 33 },
    { 0xcb3a3fbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1a3fc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a3fcc, 90 },
    { 0x18fa3fcd, 33 },
    { 0x644a3fcd, 90 },
    { 0xcbda3fcf, 90 },
    { 0xd20a3fd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cba3fd3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92a3fd7, 98 },
    { 0x271a3fd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164a3fdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ca3fe0, 68 },
    { 0xd8fa3fe1, 33 },
    { 0, 0 },
    { 0x47ba3fe3, 90 },
    { 0xa7ca3fe3, 76 },
    { 0, 0 },
    { 0xafda3fe6, 84 },
    { 0x7fda3fe7, 85 },
    { 0x8bba3fe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eca3feb, 33 },
    { 0x54ba3feb, 33 },
    { 0xd47a3fed, 97 },
    { 0x53ca3fed, 84 },
    { 0, 0 },
    { 0xf7ba3ff0, 34 },
    { 0xcd2a3ff1, 61 },
    { 0x3eaa3ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a3ff5, 33 },
    { 0, 0 },
    { 0xf76a3ff7, 88 },
    { 0x8f0a3ff7, 95 },
    { 0, 0 },
    { 0x3cca3ffa, 67 },
    { 0, 0 },
    { 0x29aa3ffc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9a3fff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x774a4002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90a4009, 95 },
    { 0x9aba400a, 90 },
    { 0x581a400a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aa4010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ba4013, 23 },
    { 0x13da4013, 34 },
    { 0x774a4013, 46 },
    { 0x1a3a4016, 33 },
    { 0x5da4015, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x623a401a, 61 },
    { 0xf77a401b, 23 },
    { 0xb3ca401c, 61 },
    { 0x89aa401d, 99 },
    { 0x6f3a401a, 81 },
    { 0x779a401f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a4024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a4028, 76 },
    { 0, 0 },
    { 0x1baa402a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da402e, 65 },
    { 0x69a402e, 62 },
    { 0xb6ba402f, 90 },
    { 0x9ada402e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4036, 90 },
    { 0xdbda4037, 95 },
    { 0xd2aa4038, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fa403b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ba4042, 95 },
    { 0, 0 },
    { 0x77da4044, 95 },
    { 0xb0fa4044, 98 },
    { 0x504a4046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6a404a, 85 },
    { 0x5ada404a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83da4062, 43 },
    { 0x677a4063, 23 },
    { 0x296a4063, 68 },
    { 0x502a4062, 33 },
    { 0x1c4a4062, 95 },
    { 0xd31a4064, 90 },
    { 0xf00a4062, 100 },
    { 0xfd7a4069, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba406f, 77 },
    { 0xa6a4070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4075, 59 },
    { 0xf8fa4076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a407b, 33 },
    { 0xe92a407b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a407f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca4082, 69 },
    { 0, 0 },
    { 0x99fa4084, 61 },
    { 0x4cfa4085, 98 },
    { 0x18ca4084, 23 },
    { 0, 0 },
    { 0x3aea4088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50a408d, 55 },
    { 0x5d8a408d, 58 },
    { 0xf23a408f, 43 },
    { 0xad3a408f, 95 },
    { 0x16ca4090, 75 },
    { 0x3a1a408f, 55 },
    { 0x105a4093, 90 },
    { 0, 0 },
    { 0x282a4095, 95 },
    { 0xbda4096, 23 },
    { 0, 0 },
    { 0xe23a4098, 98 },
    { 0xa50a4099, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a409e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0a40a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40a8, 61 },
    { 0x9fa40a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563a40b1, 95 },
    { 0xd0a40b1, 98 },
    { 0, 0 },
    { 0xe42a40b4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x883a40b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2a40bb, 76 },
    { 0, 0 },
    { 0xbf2a40bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32a40c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fa40d0, 61 },
    { 0xc15a40d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a40d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca40d8, 34 },
    { 0, 0 },
    { 0x862a40da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46a40e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56a40e4, 90 },
    { 0xa57a40e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0a40e9, 23 },
    { 0x405a40ea, 55 },
    { 0xef4a40ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a40f1, 68 },
    { 0x71aa40f2, 98 },
    { 0, 0 },
    { 0x5f7a40f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763a40ff, 95 },
    { 0, 0 },
    { 0x8d9a4101, 61 },
    { 0x4dda4101, 81 },
    { 0xaca4101, 95 },
    { 0x328a4104, 34 },
    { 0x44ba4105, 66 },
    { 0xb12a4106, 66 },
    { 0xd74a4107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69a410a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a410d, 87 },
    { 0xc3fa410e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51a4111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea4114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6a411a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a411d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca4124, 46 },
    { 0x227a4125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ba4129, 55 },
    { 0x6cba412a, 99 },
    { 0x7a0a412b, 81 },
    { 0x6aea412c, 90 },
    { 0x380a412a, 90 },
    { 0xdb5a412e, 98 },
    { 0, 0 },
    { 0x242a4130, 77 },
    { 0xc3aa4131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aa4134, 52 },
    { 0x114a4135, 34 },
    { 0x8d9a4136, 99 },
    { 0xf4a4137, 55 },
    { 0xd82a4138, 61 },
    { 0x6ea4139, 84 },
    { 0xa8ea413a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78a413e, 98 },
    { 0x9e0a413e, 91 },
    { 0xd0aa413e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a4144, 81 },
    { 0x80ba4145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efa4149, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x129a414c, 90 },
    { 0x282a414d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85a4151, 99 },
    { 0x19ea4152, 87 },
    { 0xe1a4153, 52 },
    { 0, 0 },
    { 0x2d9a4155, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16a415b, 100 },
    { 0, 0 },
    { 0x959a415d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ba4165, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a416c, 93 },
    { 0x8fca416d, 75 },
    { 0xfa7a416e, 23 },
    { 0x127a416d, 95 },
    { 0, 0 },
    { 0x4e0a4171, 61 },
    { 0, 0 },
    { 0xc46a4173, 34 },
    { 0xb57a4174, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a4177, 55 },
    { 0x7c2a4178, 23 },
    { 0, 0 },
    { 0x694a417a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9a4187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea418a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896a418e, 23 },
    { 0x8fda418f, 95 },
    { 0xb1aa4190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4a4193, 80 },
    { 0, 0 },
    { 0x137a4195, 98 },
    { 0x7d7a4196, 95 },
    { 0x87ca4197, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8a41a0, 46 },
    { 0x4a4a41a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a41a6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fa41a9, 75 },
    { 0, 0 },
    { 0x57ca41ab, 23 },
    { 0, 0 },
    { 0x43ba41ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda41b0, 34 },
    { 0, 0 },
    { 0x852a41b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa41b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda41be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a41c3, 23 },
    { 0xc42a41c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a41c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a41ca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a41d3, 38 },
    { 0, 0 },
    { 0x4dca41d5, 61 },
    { 0x721a41d5, 98 },
    { 0x5c4a41d7, 76 },
    { 0x82ea41d7, 55 },
    { 0x541a41d7, 23 },
    { 0xc0ea41d8, 84 },
    { 0xa59a41da, 98 },
    { 0x805a41d8, 46 },
    { 0x9f7a41d9, 76 },
    { 0x960a41d5, 98 },
    { 0x5a2a41df, 55 },
    { 0x283a41e0, 90 },
    { 0x268a41e1, 76 },
    { 0xaf6a41db, 98 },
    { 0xc5ca41e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaea41e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0a41ec, 66 },
    { 0x203a41ed, 61 },
    { 0, 0 },
    { 0xd98a41ef, 95 },
    { 0x6c2a41f0, 23 },
    { 0x5aa41ef, 80 },
    { 0x505a41f2, 98 },
    { 0x7b4a41f2, 81 },
    { 0xafca41ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33a41f9, 81 },
    { 0xb4fa41fa, 40 },
    { 0x93fa41fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39aa4201, 81 },
    { 0x918a4201, 95 },
    { 0, 0 },
    { 0xc5da4204, 34 },
    { 0x800a4204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74a4208, 70 },
    { 0, 0 },
    { 0xdb3a420a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x393a420d, 95 },
    { 0x7f8a420e, 91 },
    { 0x951a420d, 34 },
    { 0xee3a4210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31a4214, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a4219, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6a421c, 17 },
    { 0x5c7a421c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca4225, 61 },
    { 0, 0 },
    { 0xd74a4227, 75 },
    { 0x60da4228, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa422c, 19 },
    { 0x6b6a422c, 33 },
    { 0x9f9a422d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x451a4231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfa4234, 55 },
    { 0x137a4235, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfca423c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca4242, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aa4245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a4249, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca4252, 23 },
    { 0x8e2a4253, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0a4256, 61 },
    { 0x70ea4257, 90 },
    { 0x60aa4258, 23 },
    { 0x4e7a4259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fa4263, 23 },
    { 0xfe6a4264, 97 },
    { 0x648a4265, 90 },
    { 0x9cda4263, 43 },
    { 0x2f5a4267, 61 },
    { 0xd57a4268, 61 },
    { 0x2dda4268, 33 },
    { 0x42da4269, 95 },
    { 0, 0 },
    { 0x14ca426c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aa4270, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a4278, 61 },
    { 0, 0 },
    { 0x41fa427a, 98 },
    { 0x5aba427b, 66 },
    { 0, 0 },
    { 0x72ca427d, 61 },
    { 0x6a7a427e, 75 },
    { 0xad6a427f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68a4284, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a4287, 87 },
    { 0x3a0a4288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffa428d, 100 },
    { 0, 0 },
    { 0x54aa428f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba4293, 61 },
    { 0xd54a4294, 55 },
    { 0x75ea4294, 33 },
    { 0x5d4a4296, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a429c, 23 },
    { 0x978a429d, 100 },
    { 0x33a429d, 43 },
    { 0, 0 },
    { 0x6d8a42a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacda42a3, 100 },
    { 0x5e8a42a4, 33 },
    { 0, 0 },
    { 0x18a42a6, 33 },
    { 0, 0 },
    { 0x590a42a8, 61 },
    { 0, 0 },
    { 0x24ca42aa, 66 },
    { 0x930a42aa, 33 },
    { 0x776a42ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889a42b3, 55 },
    { 0x156a42b4, 55 },
    { 0x378a42b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aa42ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92a42c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da42c3, 43 },
    { 0, 0 },
    { 0x490a42c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefda42ca, 75 },
    { 0x2eba42cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ea42cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa42d5, 61 },
    { 0x848a42d6, 61 },
    { 0x2dca42d5, 34 },
    { 0, 0 },
    { 0x2d7a42d9, 78 },
    { 0x796a42da, 33 },
    { 0, 0 },
    { 0xd47a42dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7a42e2, 98 },
    { 0xf9a42e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998a42e7, 90 },
    { 0, 0 },
    { 0xcbea42e9, 33 },
    { 0, 0 },
    { 0xacba42eb, 40 },
    { 0x629a42eb, 43 },
    { 0x307a42ed, 33 },
    { 0x62fa42ed, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa42f3, 90 },
    { 0x77fa42f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a4300, 23 },
    { 0x90aa4300, 66 },
    { 0xaaa4301, 93 },
    { 0, 0 },
    { 0xf12a4304, 33 },
    { 0x80ba4305, 90 },
    { 0x5c6a4305, 23 },
    { 0x804a4304, 61 },
    { 0x259a4308, 56 },
    { 0xa9aa4305, 95 },
    { 0x317a4305, 90 },
    { 0x3da430b, 81 },
    { 0x428a430b, 33 },
    { 0xb64a430d, 55 },
    { 0x32a4304, 23 },
    { 0, 0 },
    { 0xf04a4310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a4315, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a4319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a4324, 52 },
    { 0, 0 },
    { 0xc69a4326, 90 },
    { 0x790a4327, 95 },
    { 0x625a4328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca432c, 93 },
    { 0xca0a432d, 43 },
    { 0, 0 },
    { 0x93ba432f, 90 },
    { 0xb4ba432f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca4333, 55 },
    { 0x68da4334, 91 },
    { 0xd8da4334, 23 },
    { 0x229a4336, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa4347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ba4356, 52 },
    { 0, 0 },
    { 0xa5aa4358, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa435f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fa4362, 55 },
    { 0x53aa4363, 61 },
    { 0, 0 },
    { 0x49ea4365, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea4368, 95 },
    { 0x79ea4368, 98 },
    { 0xe25a436a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7a4371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a437a, 61 },
    { 0x658a437b, 95 },
    { 0x21ba437c, 61 },
    { 0x804a437d, 17 },
    { 0x4cfa437d, 67 },
    { 0, 0 },
    { 0x4b7a4380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4383, 98 },
    { 0, 0 },
    { 0xc14a4385, 61 },
    { 0x6b1a4386, 23 },
    { 0xafaa4387, 61 },
    { 0x15aa4387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a438c, 61 },
    { 0x324a438d, 23 },
    { 0x4b9a438e, 23 },
    { 0x28aa438f, 95 },
    { 0x2d2a438c, 55 },
    { 0x806a438e, 33 },
    { 0xdd1a438e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x260a4395, 33 },
    { 0xc15a4395, 90 },
    { 0x42fa4395, 43 },
    { 0x8b1a4396, 98 },
    { 0, 0 },
    { 0x2a7a439a, 75 },
    { 0x56da439b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40a439f, 90 },
    { 0, 0 },
    { 0x97da43a1, 34 },
    { 0x540a43a2, 85 },
    { 0x6d1a43a1, 98 },
    { 0, 0 },
    { 0x927a43a5, 88 },
    { 0xb08a43a6, 76 },
    { 0x59aa43a6, 46 },
    { 0xa2ca43a8, 33 },
    { 0, 0 },
    { 0xb7ba43aa, 99 },
    { 0x954a43ab, 52 },
    { 0x34a43ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a43af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a43b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a43b8, 43 },
    { 0x8a3a43b9, 98 },
    { 0, 0 },
    { 0x188a43bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ba43c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7a43c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba43c7, 34 },
    { 0x109a43c8, 61 },
    { 0x3d0a43c9, 31 },
    { 0x16a43ca, 91 },
    { 0xf62a43c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17a43ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846a43d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda43da, 93 },
    { 0, 0 },
    { 0xefa43dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a43e1, 55 },
    { 0x54ba43e2, 55 },
    { 0, 0 },
    { 0x70fa43e4, 23 },
    { 0x18ba43e5, 33 },
    { 0x117a43e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1a43e9, 33 },
    { 0, 0 },
    { 0x417a43eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95a43ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a43f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a43f8, 95 },
    { 0x351a43f9, 87 },
    { 0x717a43fa, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a43fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba4401, 66 },
    { 0xea7a4401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a4407, 90 },
    { 0x2a4a4407, 78 },
    { 0x3f6a4409, 95 },
    { 0x41da440a, 46 },
    { 0, 0 },
    { 0x4b9a440c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaa4410, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a4418, 17 },
    { 0xaf9a4419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234a4420, 87 },
    { 0xd71a4421, 34 },
    { 0x963a4422, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a4426, 87 },
    { 0, 0 },
    { 0xd6fa4428, 61 },
    { 0, 0 },
    { 0x9aea442a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a442d, 33 },
    { 0, 0 },
    { 0x37ca442f, 40 },
    { 0xa9ca442f, 34 },
    { 0xd00a4431, 34 },
    { 0, 0 },
    { 0x3f7a4433, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208a4438, 33 },
    { 0xaa7a4439, 23 },
    { 0, 0 },
    { 0xe5ba443b, 84 },
    { 0x2fca443c, 95 },
    { 0x6d0a443d, 87 },
    { 0xf0ea443e, 95 },
    { 0xe7aa443e, 99 },
    { 0xab3a4440, 61 },
    { 0x6cba4440, 51 },
    { 0x655a4441, 43 },
    { 0x57da4443, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ea4448, 95 },
    { 0, 0 },
    { 0x197a444a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785a444e, 43 },
    { 0, 0 },
    { 0x484a4450, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3a4454, 68 },
    { 0x105a4454, 33 },
    { 0, 0 },
    { 0x404a4457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28a445e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2a4464, 61 },
    { 0xa1da4465, 33 },
    { 0x240a4466, 90 },
    { 0x28ba4467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaa446b, 90 },
    { 0x3e2a446c, 91 },
    { 0x442a446c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285a4476, 95 },
    { 0x5d4a4477, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfba447a, 49 },
    { 0x2e3a447b, 66 },
    { 0x1e9a447c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a4484, 99 },
    { 0, 0 },
    { 0xbeda4486, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1a448e, 84 },
    { 0x4dea448e, 95 },
    { 0x75a448f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4a4493, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fa4497, 95 },
    { 0, 0 },
    { 0xe8ba4499, 76 },
    { 0, 0 },
    { 0x435a449b, 98 },
    { 0x1a1a449c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bea449f, 23 },
    { 0x30aa44a0, 90 },
    { 0, 0 },
    { 0x2daa44a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a44a8, 100 },
    { 0x8c1a44a9, 95 },
    { 0xd55a44a9, 51 },
    { 0x318a44a9, 23 },
    { 0x195a44ac, 76 },
    { 0, 0 },
    { 0xafca44ae, 23 },
    { 0x1a6a44af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607a44b3, 23 },
    { 0, 0 },
    { 0x2caa44b5, 90 },
    { 0x401a44b6, 90 },
    { 0xfb9a44b7, 34 },
    { 0x7faa44b6, 36 },
    { 0x6d1a44b9, 61 },
    { 0, 0 },
    { 0x1bda44bb, 69 },
    { 0x361a44bb, 23 },
    { 0x64fa44bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a44c3, 33 },
    { 0x2d4a44c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca44d1, 33 },
    { 0, 0 },
    { 0xcefa44d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eda44db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a44e3, 78 },
    { 0xc55a44e4, 61 },
    { 0xd62a44e5, 84 },
    { 0x886a44e6, 61 },
    { 0x51da44e5, 98 },
    { 0x4d4a44e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806a44ec, 55 },
    { 0xcf3a44ed, 78 },
    { 0x2aaa44ed, 34 },
    { 0x616a44ee, 33 },
    { 0, 0 },
    { 0x1e6a44f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffba44f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca44f9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfa44fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fa44ff, 90 },
    { 0, 0 },
    { 0x636a4501, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca4505, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea450a, 69 },
    { 0x669a450a, 93 },
    { 0x189a450c, 78 },
    { 0xb5a450a, 61 },
    { 0x3f5a450e, 90 },
    { 0x19ca450f, 90 },
    { 0x543a4510, 87 },
    { 0x4e9a450f, 76 },
    { 0xbeca4512, 33 },
    { 0xc6a4512, 23 },
    { 0xc16a4514, 23 },
    { 0x56ba450c, 90 },
    { 0, 0 },
    { 0x7f4a4517, 90 },
    { 0xf17a4518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9a451b, 90 },
    { 0, 0 },
    { 0xb4ca451d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a4529, 90 },
    { 0, 0 },
    { 0x4eea452b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ca452f, 33 },
    { 0x727a4530, 90 },
    { 0, 0 },
    { 0x6e1a4532, 75 },
    { 0x23da4533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9a4536, 93 },
    { 0x1d0a4536, 90 },
    { 0, 0 },
    { 0x8e9a4539, 76 },
    { 0xc32a453a, 46 },
    { 0x9f4a453a, 55 },
    { 0xe19a453a, 87 },
    { 0x85ca453c, 34 },
    { 0x5fea453c, 33 },
    { 0xaada453f, 87 },
    { 0x3c6a453d, 23 },
    { 0xd71a4541, 17 },
    { 0x6ada4541, 96 },
    { 0x158a4543, 95 },
    { 0x2f0a4543, 95 },
    { 0x7a5a4541, 31 },
    { 0xdbda4546, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a454f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28a4555, 61 },
    { 0xf52a4556, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ba4559, 91 },
    { 0xfb8a455a, 33 },
    { 0x144a455b, 75 },
    { 0, 0 },
    { 0x1f3a455d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2a4560, 61 },
    { 0x40fa4561, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a4566, 61 },
    { 0xa1ea4567, 81 },
    { 0, 0 },
    { 0xc1da4569, 93 },
    { 0, 0 },
    { 0x16aa456b, 95 },
    { 0, 0 },
    { 0xd57a456d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xecca4570, 91 },
    { 0x17a4571, 98 },
    { 0x546a4570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xecda4575, 33 },
    { 0x38ca4576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da4579, 61 },
    { 0x623a4579, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a457f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea4583, 19 },
    { 0xc46a4584, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09a458c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76a458f, 93 },
    { 0x6b4a4590, 98 },
    { 0, 0 },
    { 0x2bfa4592, 61 },
    { 0xfd8a4593, 33 },
    { 0x869a4594, 78 },
    { 0xbf2a4594, 55 },
    { 0x73aa4595, 90 },
    { 0x632a4597, 61 },
    { 0xcb6a4598, 61 },
    { 0x9bda4599, 90 },
    { 0x37ca4598, 98 },
    { 0xfaaa4596, 99 },
    { 0xf3fa459c, 90 },
    { 0x894a4594, 98 },
    { 0x7ba4595, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea45a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a45aa, 91 },
    { 0x1e1a45ab, 95 },
    { 0, 0 },
    { 0xf13a45ad, 90 },
    { 0x8fba45ad, 33 },
    { 0x8cba45af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa45b8, 98 },
    { 0x324a45b8, 33 },
    { 0, 0 },
    { 0x606a45bb, 61 },
    { 0x171a45bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda45bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa45c4, 23 },
    { 0, 0 },
    { 0x6a4a45c6, 23 },
    { 0x305a45c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448a45cb, 95 },
    { 0x378a45cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a45d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971a45e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dca45e4, 76 },
    { 0xf15a45e5, 33 },
    { 0x614a45e6, 23 },
    { 0x604a45e7, 90 },
    { 0, 0 },
    { 0x7eea45e9, 34 },
    { 0x4fba45ea, 66 },
    { 0x965a45eb, 98 },
    { 0xae1a45ea, 76 },
    { 0xc72a45e9, 87 },
    { 0, 0 },
    { 0x8cea45ef, 91 },
    { 0x5aca45ef, 61 },
    { 0x239a45f1, 43 },
    { 0xcaba45f0, 34 },
    { 0xc62a45f3, 55 },
    { 0, 0 },
    { 0x15da45f5, 66 },
    { 0x9eda45f6, 78 },
    { 0x632a45f7, 98 },
    { 0xdfba45f8, 98 },
    { 0, 0 },
    { 0x55a45fa, 90 },
    { 0, 0 },
    { 0xdbba45fc, 90 },
    { 0, 0 },
    { 0xd81a45fe, 23 },
    { 0xe55a45ff, 98 },
    { 0x852a4600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a4603, 90 },
    { 0x256a4604, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ca4607, 98 },
    { 0x508a4608, 93 },
    { 0xa95a4609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa460c, 90 },
    { 0xf42a460d, 90 },
    { 0x479a460e, 90 },
    { 0xa7fa460f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a4613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x886a4616, 90 },
    { 0xfe9a4617, 91 },
    { 0x4e5a4617, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda461b, 80 },
    { 0, 0 },
    { 0x7eaa461d, 97 },
    { 0x27ea461d, 23 },
    { 0xbb4a461d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a4622, 66 },
    { 0x236a4622, 98 },
    { 0x52ca4623, 90 },
    { 0x13a4625, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da462f, 95 },
    { 0x87ea4630, 95 },
    { 0x54ea4631, 19 },
    { 0x76ea4632, 61 },
    { 0x849a4633, 90 },
    { 0xad4a4631, 23 },
    { 0x4aea462f, 99 },
    { 0, 0 },
    { 0x3d2a4637, 95 },
    { 0xd15a4638, 78 },
    { 0xf46a4637, 55 },
    { 0xafda463a, 23 },
    { 0xd72a4638, 95 },
    { 0x55a463c, 33 },
    { 0x58ca463d, 61 },
    { 0x845a463c, 87 },
    { 0xa75a463f, 99 },
    { 0x7b8a4640, 98 },
    { 0xc3ca4641, 88 },
    { 0x73ea4642, 34 },
    { 0x746a4643, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253a4647, 90 },
    { 0xb17a4647, 23 },
    { 0xd84a4649, 55 },
    { 0x247a464a, 55 },
    { 0xef6a4649, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa4650, 23 },
    { 0, 0 },
    { 0x5d6a4652, 61 },
    { 0xdc3a4653, 55 },
    { 0xff0a4654, 33 },
    { 0xcc1a4652, 99 },
    { 0x221a4656, 33 },
    { 0, 0 },
    { 0x2aca4658, 90 },
    { 0xc5aa4659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba465e, 90 },
    { 0xe4ea465f, 33 },
    { 0x402a465e, 82 },
    { 0, 0 },
    { 0x81ca4662, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1a4665, 34 },
    { 0x12ea4666, 55 },
    { 0x760a4667, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3a466a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a4671, 92 },
    { 0x280a4672, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a4675, 61 },
    { 0, 0 },
    { 0xacfa4677, 65 },
    { 0x7a1a4677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a467b, 87 },
    { 0x7e0a467c, 93 },
    { 0x6d3a467b, 90 },
    { 0x2d5a467b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370a4684, 76 },
    { 0x18da4685, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba4688, 33 },
    { 0x909a4689, 61 },
    { 0, 0 },
    { 0x8cfa468b, 88 },
    { 0, 0 },
    { 0x177a468d, 99 },
    { 0, 0 },
    { 0xb8da468f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa4693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afa469b, 100 },
    { 0, 0 },
    { 0x76ca469d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876a46a9, 23 },
    { 0x810a46aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a46b0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a46b3, 98 },
    { 0, 0 },
    { 0x4a3a46b5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a46b8, 76 },
    { 0x9b2a46b8, 40 },
    { 0xbc2a46b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865a46be, 95 },
    { 0, 0 },
    { 0x9eea46c0, 34 },
    { 0, 0 },
    { 0x7a1a46c2, 90 },
    { 0, 0 },
    { 0x7c5a46c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59a46cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfea46d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02a46d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a46d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a46eb, 87 },
    { 0xeeda46ec, 91 },
    { 0xe41a46ed, 98 },
    { 0x2c6a46eb, 61 },
    { 0, 0 },
    { 0x2efa46f0, 84 },
    { 0x406a46f0, 23 },
    { 0x41fa46f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba46f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fca46fc, 95 },
    { 0x132a46fd, 33 },
    { 0x8faa46fd, 34 },
    { 0x4cca46fe, 94 },
    { 0, 0 },
    { 0xf58a4701, 95 },
    { 0xe88a4702, 69 },
    { 0xd32a4703, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a4709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a470e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a471a, 46 },
    { 0x182a471b, 76 },
    { 0, 0 },
    { 0xe8ca471d, 91 },
    { 0, 0 },
    { 0x75aa471f, 98 },
    { 0x765a4720, 43 },
    { 0x648a4720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a4724, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da4728, 34 },
    { 0xcbca4729, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a472c, 84 },
    { 0x48aa472d, 23 },
    { 0, 0 },
    { 0xf22a472f, 33 },
    { 0, 0 },
    { 0xf43a4731, 61 },
    { 0, 0 },
    { 0xac1a4733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49da4737, 76 },
    { 0x62a4737, 33 },
    { 0x5d8a4739, 61 },
    { 0, 0 },
    { 0xd47a473b, 90 },
    { 0x560a473c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94a4740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ba4745, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9a4748, 90 },
    { 0xc8fa4749, 61 },
    { 0xcafa474a, 36 },
    { 0, 0 },
    { 0xc00a474c, 99 },
    { 0, 0 },
    { 0xf5a474e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a4752, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a475c, 61 },
    { 0x361a475d, 61 },
    { 0, 0 },
    { 0xf6da475f, 61 },
    { 0xea7a4760, 67 },
    { 0xbd9a4761, 85 },
    { 0x345a4760, 55 },
    { 0, 0 },
    { 0x229a4764, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796a4769, 23 },
    { 0xfeca476a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6a4773, 85 },
    { 0x596a4773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba4777, 81 },
    { 0xacca4778, 88 },
    { 0xb79a4779, 61 },
    { 0x451a4777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2a477e, 95 },
    { 0x7ba477f, 33 },
    { 0xcb8a4780, 20 },
    { 0, 0 },
    { 0xa4782, 55 },
    { 0, 0 },
    { 0x71aa4784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85a4787, 17 },
    { 0x79da4788, 93 },
    { 0x42ba4789, 55 },
    { 0x75da4789, 55 },
    { 0x344a4787, 100 },
    { 0, 0 },
    { 0x812a478d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2a4790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea4795, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2a479b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2a479e, 48 },
    { 0x8faa479f, 43 },
    { 0xb40a47a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a47a4, 55 },
    { 0, 0 },
    { 0x30da47a6, 55 },
    { 0, 0 },
    { 0x9eba47a8, 33 },
    { 0x1a8a47a9, 34 },
    { 0xc75a47aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a47ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08a47b3, 98 },
    { 0x8c9a47b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ba47b9, 100 },
    { 0, 0 },
    { 0xc21a47bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69a47bf, 61 },
    { 0x198a47c0, 23 },
    { 0x684a47c1, 33 },
    { 0x630a47c2, 40 },
    { 0x309a47c1, 23 },
    { 0x52da47bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x983a47c7, 90 },
    { 0x47da47c8, 75 },
    { 0x906a47c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ba47ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7a47d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a47d5, 100 },
    { 0x5fba47d5, 33 },
    { 0xc89a47d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702a47dc, 68 },
    { 0x98a47dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da47e3, 93 },
    { 0x1b7a47e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa47e7, 66 },
    { 0x5fba47e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a47ec, 33 },
    { 0x6cda47ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a47f3, 65 },
    { 0xc7aa47f4, 43 },
    { 0x76aa47f5, 68 },
    { 0x229a47f6, 95 },
    { 0xecca47f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398a47fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927a4800, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca4803, 95 },
    { 0, 0 },
    { 0xb08a4805, 98 },
    { 0x5cfa4805, 23 },
    { 0, 0 },
    { 0xf77a4808, 67 },
    { 0, 0 },
    { 0x74da480a, 98 },
    { 0, 0 },
    { 0xb23a480c, 98 },
    { 0, 0 },
    { 0xcb9a480e, 91 },
    { 0x360a480e, 76 },
    { 0xe37a4810, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fa4815, 95 },
    { 0x460a4816, 99 },
    { 0xd21a4815, 65 },
    { 0xa28a4818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a481b, 98 },
    { 0x8aaa481b, 33 },
    { 0x902a481d, 100 },
    { 0, 0 },
    { 0xb30a481f, 34 },
    { 0x35da4820, 90 },
    { 0x3d9a4821, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53a4826, 33 },
    { 0, 0 },
    { 0x762a4828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a482b, 61 },
    { 0xa31a482c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a482f, 55 },
    { 0x498a4830, 98 },
    { 0xfeba482f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a483d, 81 },
    { 0, 0 },
    { 0x490a483f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x175a4842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4849, 90 },
    { 0, 0 },
    { 0x184a484b, 77 },
    { 0x3cfa484b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dba484f, 68 },
    { 0x1c6a484f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796a485d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aa4861, 61 },
    { 0xb0a4861, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafa4866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea4869, 23 },
    { 0xc5ca486a, 33 },
    { 0x33da486b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a486e, 100 },
    { 0xce6a486e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x239a4872, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a487d, 66 },
    { 0x8bfa487e, 23 },
    { 0x3eba487f, 76 },
    { 0xba6a4880, 87 },
    { 0x4cea4881, 33 },
    { 0xd78a4882, 23 },
    { 0xcf5a487f, 91 },
    { 0xbdda4884, 33 },
    { 0x510a4885, 23 },
    { 0x26aa4886, 33 },
    { 0xfd5a487e, 23 },
    { 0, 0 },
    { 0xcb3a4889, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56da4890, 76 },
    { 0x99ca4891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a4894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dba489d, 61 },
    { 0xf2ca489e, 90 },
    { 0, 0 },
    { 0x286a48a0, 55 },
    { 0x996a48a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a48a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca48aa, 66 },
    { 0, 0 },
    { 0xa6ea48ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a48b0, 33 },
    { 0xb08a48b0, 61 },
    { 0x779a48b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a48b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa48b9, 23 },
    { 0xbe3a48ba, 61 },
    { 0xa19a48bb, 98 },
    { 0x76aa48bb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496a48c3, 61 },
    { 0, 0 },
    { 0x51a48c5, 95 },
    { 0xdb3a48c6, 98 },
    { 0xf8aa48c5, 81 },
    { 0x5faa48c8, 46 },
    { 0x9dea48c7, 61 },
    { 0xfcaa48c8, 98 },
    { 0, 0 },
    { 0xa89a48cc, 34 },
    { 0xfdea48cd, 23 },
    { 0, 0 },
    { 0x5d9a48cf, 33 },
    { 0x460a48d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150a48d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29a48d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a48dd, 61 },
    { 0, 0 },
    { 0x8f1a48df, 85 },
    { 0x7eea48df, 90 },
    { 0xbe8a48e1, 43 },
    { 0x9f1a48e0, 85 },
    { 0x299a48e3, 61 },
    { 0, 0 },
    { 0x814a48e5, 52 },
    { 0xff7a48e6, 67 },
    { 0x70da48e6, 61 },
    { 0x199a48e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca48ec, 81 },
    { 0x8e9a48ed, 33 },
    { 0x833a48ee, 98 },
    { 0x889a48ef, 34 },
    { 0x5e8a48ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbba48f7, 61 },
    { 0x8cda48f8, 23 },
    { 0xe2a48f9, 90 },
    { 0, 0 },
    { 0x86ba48fb, 61 },
    { 0x2daa48fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39a4901, 95 },
    { 0, 0 },
    { 0xa28a4903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632a490a, 87 },
    { 0x545a490b, 33 },
    { 0, 0 },
    { 0x9cea490d, 66 },
    { 0xc01a490d, 23 },
    { 0x32ca490f, 67 },
    { 0x37ea4910, 90 },
    { 0x2d0a4911, 90 },
    { 0x274a4911, 95 },
    { 0x51ea4913, 76 },
    { 0x172a490f, 90 },
    { 0x38fa490e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5a4919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793a491d, 33 },
    { 0x878a491e, 90 },
    { 0xf19a491f, 95 },
    { 0x86fa491f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5a4925, 90 },
    { 0x13ca4925, 87 },
    { 0x28fa4925, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafa4930, 61 },
    { 0x250a4931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a4935, 46 },
    { 0xcf3a4935, 33 },
    { 0xd6a4936, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da493b, 67 },
    { 0, 0 },
    { 0xbe2a493d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eaa4941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ea4945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4949, 75 },
    { 0x2a2a4949, 81 },
    { 0xd2da494b, 40 },
    { 0x595a494c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a4950, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a4954, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa4957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2a495e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a4962, 55 },
    { 0xafea4962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a496c, 77 },
    { 0, 0 },
    { 0x128a496e, 46 },
    { 0x227a496e, 98 },
    { 0x9da496f, 95 },
    { 0x72ea4971, 43 },
    { 0, 0 },
    { 0xe0da4973, 33 },
    { 0, 0 },
    { 0xb77a4975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fa4979, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a4983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a4989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfa4992, 87 },
    { 0x669a4993, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5a4999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faa499c, 34 },
    { 0x4f7a499d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da49a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72a49ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea49b2, 38 },
    { 0, 0 },
    { 0xeeda49b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a49ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da49be, 69 },
    { 0x216a49bf, 66 },
    { 0, 0 },
    { 0x9ea49c1, 33 },
    { 0x2aaa49c2, 55 },
    { 0x2d5a49c3, 66 },
    { 0xbdfa49c4, 52 },
    { 0, 0 },
    { 0xdd8a49c6, 98 },
    { 0x6a9a49c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a49d0, 33 },
    { 0x260a49d1, 95 },
    { 0x957a49d1, 84 },
    { 0, 0 },
    { 0x9dfa49d4, 61 },
    { 0xe48a49d5, 33 },
    { 0xf7da49d5, 23 },
    { 0x56ba49d7, 67 },
    { 0, 0 },
    { 0x160a49d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba49dc, 21 },
    { 0, 0 },
    { 0x93ba49de, 98 },
    { 0xbcaa49df, 81 },
    { 0, 0 },
    { 0x80da49e1, 77 },
    { 0x73da49e1, 65 },
    { 0x195a49e3, 43 },
    { 0x8cea49e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705a49e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a49f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffea49f3, 85 },
    { 0, 0 },
    { 0x70fa49f5, 23 },
    { 0x100a49f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5a49f9, 66 },
    { 0, 0 },
    { 0x55fa49fb, 67 },
    { 0, 0 },
    { 0x56ca49fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622a4a04, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fa4a07, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9a4a11, 61 },
    { 0xd00a4a12, 91 },
    { 0xeaaa4a13, 46 },
    { 0xb85a4a11, 40 },
    { 0xce2a4a14, 98 },
    { 0x545a4a16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a4a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aa4a21, 23 },
    { 0, 0 },
    { 0x512a4a23, 33 },
    { 0, 0 },
    { 0x49a4a25, 93 },
    { 0xcd1a4a26, 78 },
    { 0, 0 },
    { 0x41ea4a28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a4a2d, 90 },
    { 0x792a4a2e, 23 },
    { 0x488a4a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a4a32, 23 },
    { 0xe89a4a33, 23 },
    { 0xf4da4a33, 99 },
    { 0x33a4a35, 95 },
    { 0, 0 },
    { 0x775a4a37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdca4a3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aea4a3f, 95 },
    { 0x2f7a4a40, 90 },
    { 0xef3a4a41, 81 },
    { 0x7b0a4a42, 90 },
    { 0x38ea4a43, 66 },
    { 0x3d1a4a44, 77 },
    { 0xd4ea4a44, 95 },
    { 0x95da4a40, 91 },
    { 0x17fa4a47, 23 },
    { 0x93ea4a44, 46 },
    { 0, 0 },
    { 0xd08a4a4a, 99 },
    { 0x70fa4a4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x185a4a4e, 65 },
    { 0x1f7a4a4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a4a54, 98 },
    { 0xc50a4a55, 61 },
    { 0xfeba4a55, 95 },
    { 0x450a4a57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a4a5a, 3 },
    { 0, 0 },
    { 0xb20a4a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a4a5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204a4a63, 33 },
    { 0x21fa4a63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01a4a68, 90 },
    { 0x9b8a4a69, 99 },
    { 0xf9a4a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a4a6d, 98 },
    { 0, 0 },
    { 0xd9ea4a6f, 91 },
    { 0, 0 },
    { 0xa2ca4a71, 23 },
    { 0, 0 },
    { 0x82ba4a73, 46 },
    { 0xabfa4a74, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380a4a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645a4a7f, 93 },
    { 0x698a4a80, 90 },
    { 0x9d4a4a80, 46 },
    { 0x781a4a81, 61 },
    { 0x330a4a82, 90 },
    { 0xc8aa4a84, 99 },
    { 0xe70a4a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a4a91, 46 },
    { 0x2b1a4a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ea4a96, 90 },
    { 0x404a4a97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9a4a9a, 52 },
    { 0x89da4a9a, 33 },
    { 0xee4a4a9c, 95 },
    { 0x90ba4a9c, 81 },
    { 0x675a4a9c, 90 },
    { 0x436a4a9a, 95 },
    { 0xa46a4aa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91a4aa3, 68 },
    { 0xf03a4aa3, 90 },
    { 0xf84a4aa5, 97 },
    { 0x827a4aa5, 97 },
    { 0x6a4a4aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca4aab, 98 },
    { 0x504a4aac, 93 },
    { 0x3f2a4aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a4ab1, 97 },
    { 0, 0 },
    { 0x4c7a4ab3, 23 },
    { 0x757a4ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00a4ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8a4aba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a4ac1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca4ac4, 95 },
    { 0xe5a4ac5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a4ac9, 99 },
    { 0xac4a4aca, 100 },
    { 0x627a4aca, 95 },
    { 0xfa4a4acb, 33 },
    { 0xb8ba4acb, 34 },
    { 0xfb0a4ace, 95 },
    { 0x84aa4aca, 98 },
    { 0x930a4ad0, 90 },
    { 0x603a4ad1, 33 },
    { 0, 0 },
    { 0x7b1a4ad3, 98 },
    { 0x6b6a4ad4, 98 },
    { 0, 0 },
    { 0x9fa4ad6, 33 },
    { 0xa75a4ad7, 93 },
    { 0, 0 },
    { 0x9b3a4ad9, 23 },
    { 0xf87a4ada, 98 },
    { 0x508a4ad9, 43 },
    { 0xb07a4adb, 40 },
    { 0x62ca4adc, 93 },
    { 0xa17a4ade, 61 },
    { 0x91da4adb, 33 },
    { 0xc25a4ada, 95 },
    { 0x85ba4ade, 61 },
    { 0x943a4ae1, 61 },
    { 0x376a4ad9, 23 },
    { 0xb1ba4ad9, 93 },
    { 0, 0 },
    { 0x82ca4ae6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ca4aea, 65 },
    { 0, 0 },
    { 0xfc4a4aec, 33 },
    { 0x668a4aec, 61 },
    { 0x7d8a4aee, 43 },
    { 0xb9fa4aef, 95 },
    { 0xf3ca4aee, 61 },
    { 0x276a4aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a4af5, 33 },
    { 0, 0 },
    { 0xf5ea4af7, 95 },
    { 0x7e9a4af8, 33 },
    { 0xa3ca4af9, 76 },
    { 0xdc1a4afa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eba4afd, 98 },
    { 0xcaaa4afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1a4b03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa4b0b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba4b15, 61 },
    { 0, 0 },
    { 0xc3ca4b17, 33 },
    { 0x533a4b18, 97 },
    { 0xe6da4b18, 53 },
    { 0xb52a4b1a, 76 },
    { 0xa4ca4b1a, 81 },
    { 0xdfea4b1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8a4b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a4b25, 90 },
    { 0x64aa4b25, 43 },
    { 0x10fa4b27, 76 },
    { 0, 0 },
    { 0xad8a4b29, 23 },
    { 0, 0 },
    { 0xf45a4b2b, 61 },
    { 0, 0 },
    { 0x209a4b2d, 17 },
    { 0x69aa4b2e, 55 },
    { 0x318a4b2f, 85 },
    { 0xd40a4b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6a4b36, 23 },
    { 0, 0 },
    { 0xa0fa4b38, 61 },
    { 0x6e0a4b39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a4b3d, 76 },
    { 0, 0 },
    { 0x7f0a4b3f, 22 },
    { 0xf8ba4b40, 90 },
    { 0, 0 },
    { 0x2b3a4b42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a4b46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa4b4d, 40 },
    { 0x4aaa4b4d, 98 },
    { 0, 0 },
    { 0x70fa4b50, 100 },
    { 0xc26a4b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a4b57, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a4b5e, 23 },
    { 0, 0 },
    { 0x6f7a4b60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a4b68, 93 },
    { 0xb7da4b69, 90 },
    { 0x370a4b69, 33 },
    { 0x127a4b6b, 98 },
    { 0x515a4b6c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a4b71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea4b74, 84 },
    { 0x27a4b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ba4b7a, 17 },
    { 0, 0 },
    { 0xb76a4b7c, 95 },
    { 0xcb4a4b7d, 90 },
    { 0, 0 },
    { 0xcfaa4b7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4a4b83, 56 },
    { 0x8d6a4b84, 33 },
    { 0x39ba4b85, 90 },
    { 0, 0 },
    { 0x9ca4b87, 68 },
    { 0, 0 },
    { 0x9a9a4b89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a4b8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a4b94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a4b98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a4ba1, 33 },
    { 0x33ba4ba2, 34 },
    { 0x921a4ba3, 39 },
    { 0, 0 },
    { 0xcd2a4ba5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46da4ba9, 68 },
    { 0xf1ea4baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9a4bad, 98 },
    { 0xa22a4bae, 43 },
    { 0x9fa4baf, 90 },
    { 0x59fa4bb0, 98 },
    { 0xa78a4bb1, 84 },
    { 0x731a4bb2, 66 },
    { 0xbaaa4bad, 46 },
    { 0xeb0a4bb4, 90 },
    { 0x9f5a4bb4, 76 },
    { 0x8bca4bb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6a4bbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21a4bbe, 33 },
    { 0x5bea4bbf, 95 },
    { 0xb49a4bc0, 23 },
    { 0xb72a4bc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5a4bc9, 93 },
    { 0x68ca4bca, 98 },
    { 0xbd1a4bca, 100 },
    { 0xc79a4bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a4bd0, 55 },
    { 0xdd4a4bd0, 90 },
    { 0, 0 },
    { 0x78ca4bd3, 90 },
    { 0xee4a4bd4, 88 },
    { 0xde4a4bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aa4bd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50a4bde, 95 },
    { 0x31ea4bdf, 98 },
    { 0, 0 },
    { 0xb33a4be1, 91 },
    { 0, 0 },
    { 0x30ca4be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda4be7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a4bee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11a4bf1, 90 },
    { 0x24fa4bf2, 33 },
    { 0xf7ba4bf2, 90 },
    { 0, 0 },
    { 0x7aaa4bf5, 61 },
    { 0, 0 },
    { 0xfeea4bf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa4bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca4bfd, 61 },
    { 0x3b2a4bfd, 43 },
    { 0, 0 },
    { 0xaf6a4c00, 43 },
    { 0, 0 },
    { 0xfbca4c02, 34 },
    { 0, 0 },
    { 0xc6da4c04, 90 },
    { 0, 0 },
    { 0xa3ea4c06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a4c0c, 76 },
    { 0x992a4c0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea4c10, 98 },
    { 0, 0 },
    { 0x854a4c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a4c18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5a4c1c, 93 },
    { 0, 0 },
    { 0x707a4c1e, 88 },
    { 0x2bba4c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba4c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a4c2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4c2f, 33 },
    { 0, 0 },
    { 0xf46a4c31, 76 },
    { 0, 0 },
    { 0xd97a4c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91a4c38, 76 },
    { 0, 0 },
    { 0x377a4c3a, 61 },
    { 0xe87a4c3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0a4c3e, 90 },
    { 0, 0 },
    { 0x15da4c40, 33 },
    { 0x135a4c41, 66 },
    { 0x47ca4c42, 33 },
    { 0x232a4c40, 46 },
    { 0, 0 },
    { 0x71aa4c45, 75 },
    { 0, 0 },
    { 0x16ba4c47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4a4c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a4c50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5a4c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a4c59, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5a4c5c, 33 },
    { 0xcbfa4c5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a4c60, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fa4c63, 69 },
    { 0xb12a4c64, 84 },
    { 0x31ea4c64, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba4c69, 23 },
    { 0x760a4c6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23a4c73, 98 },
    { 0x8f9a4c74, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa4c79, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeca4c7f, 90 },
    { 0xdaaa4c80, 75 },
    { 0x3e4a4c81, 93 },
    { 0xbd4a4c82, 61 },
    { 0xf04a4c80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda4c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x277a4c8b, 76 },
    { 0, 0 },
    { 0x176a4c8d, 90 },
    { 0, 0 },
    { 0xfaa4c8f, 66 },
    { 0, 0 },
    { 0x477a4c91, 91 },
    { 0xad6a4c92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fa4c95, 78 },
    { 0x7ffa4c95, 34 },
    { 0, 0 },
    { 0xc94a4c98, 68 },
    { 0, 0 },
    { 0x810a4c9a, 33 },
    { 0xb1ea4c9b, 38 },
    { 0xdf9a4c9c, 91 },
    { 0xfe5a4c9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a4ca4, 43 },
    { 0xe40a4ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a4ca8, 85 },
    { 0x40da4ca8, 95 },
    { 0xf12a4ca8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a4cb1, 98 },
    { 0x1a4cb2, 98 },
    { 0x872a4cb1, 33 },
    { 0, 0 },
    { 0x580a4cb5, 33 },
    { 0xb32a4cb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3a4cba, 23 },
    { 0xcdba4cba, 97 },
    { 0x383a4cbc, 87 },
    { 0xdcea4cbc, 33 },
    { 0, 0 },
    { 0xe39a4cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4a4cc6, 33 },
    { 0x590a4cc7, 85 },
    { 0x11da4cc6, 34 },
    { 0, 0 },
    { 0x60fa4cca, 90 },
    { 0, 0 },
    { 0x36ba4ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45a4cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a4cd5, 67 },
    { 0xb91a4cd6, 43 },
    { 0x153a4cd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518a4ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a4ce8, 33 },
    { 0x375a4ce8, 95 },
    { 0x12ca4ce8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55a4cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4cf2, 34 },
    { 0, 0 },
    { 0x9e7a4cf4, 46 },
    { 0x58da4cf5, 93 },
    { 0x753a4cf6, 87 },
    { 0x30da4cf7, 61 },
    { 0xb9fa4cf8, 55 },
    { 0, 0 },
    { 0x23ca4cfa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549a4cfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x168a4d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0a4d08, 61 },
    { 0, 0 },
    { 0x754a4d0a, 45 },
    { 0, 0 },
    { 0xed9a4d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca4d10, 33 },
    { 0x55fa4d11, 90 },
    { 0x78ca4d12, 82 },
    { 0x4fba4d12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4da4d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a4d1d, 61 },
    { 0, 0 },
    { 0x390a4d1f, 93 },
    { 0, 0 },
    { 0x803a4d21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02a4d26, 90 },
    { 0xd0fa4d27, 34 },
    { 0x626a4d28, 23 },
    { 0x28ea4d27, 34 },
    { 0, 0 },
    { 0x48da4d2b, 61 },
    { 0x157a4d2c, 90 },
    { 0xe60a4d2d, 95 },
    { 0x38ba4d2e, 90 },
    { 0x7efa4d2f, 33 },
    { 0x5bea4d2f, 33 },
    { 0x8eda4d2f, 76 },
    { 0xd5fa4d32, 98 },
    { 0x809a4d30, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a4d38, 33 },
    { 0x5c8a4d39, 95 },
    { 0xa54a4d3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39a4d3d, 38 },
    { 0, 0 },
    { 0x503a4d3f, 67 },
    { 0x723a4d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffa4d4a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ea4d4e, 33 },
    { 0xe9a4d4f, 43 },
    { 0xf3ca4d4f, 23 },
    { 0x350a4d50, 95 },
    { 0x961a4d52, 95 },
    { 0xc5ba4d53, 90 },
    { 0xe0ba4d52, 90 },
    { 0xf7fa4d55, 76 },
    { 0xfd6a4d53, 66 },
    { 0x9cda4d56, 34 },
    { 0x502a4d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a4d5c, 84 },
    { 0x8aba4d5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a4d60, 68 },
    { 0, 0 },
    { 0xb32a4d62, 33 },
    { 0, 0 },
    { 0xc72a4d64, 98 },
    { 0x988a4d64, 33 },
    { 0x9e6a4d65, 93 },
    { 0, 0 },
    { 0xec4a4d68, 95 },
    { 0x9a6a4d68, 23 },
    { 0, 0 },
    { 0x9d0a4d6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda4d6f, 68 },
    { 0xf3da4d6f, 23 },
    { 0x891a4d71, 81 },
    { 0xe72a4d72, 76 },
    { 0x30aa4d71, 90 },
    { 0x403a4d73, 23 },
    { 0x6cea4d6f, 90 },
    { 0x9aba4d71, 98 },
    { 0xb8aa4d76, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4d7a, 99 },
    { 0, 0 },
    { 0x1c7a4d7c, 60 },
    { 0xa33a4d7c, 61 },
    { 0x8a7a4d7d, 43 },
    { 0xc91a4d7d, 90 },
    { 0x4c6a4d80, 55 },
    { 0, 0 },
    { 0xff2a4d82, 90 },
    { 0, 0 },
    { 0xe39a4d84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2a4d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba4d8c, 55 },
    { 0xff0a4d8d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a4d90, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a4d93, 78 },
    { 0, 0 },
    { 0x175a4d95, 90 },
    { 0x6e7a4d96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a4d9e, 68 },
    { 0xf43a4d9e, 33 },
    { 0x66ca4d9e, 55 },
    { 0x467a4da1, 84 },
    { 0x5d6a4da2, 100 },
    { 0xf04a4da2, 95 },
    { 0xd23a4da1, 95 },
    { 0xd6fa4da5, 77 },
    { 0x1daa4d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa4da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aea4daf, 33 },
    { 0, 0 },
    { 0x6fea4db1, 95 },
    { 0xaeaa4db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88aa4db8, 76 },
    { 0xa67a4db8, 90 },
    { 0, 0 },
    { 0xc21a4dbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a4dbf, 67 },
    { 0x80ea4dc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cca4dc9, 76 },
    { 0x3dda4dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda4dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5a4dd2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386a4dd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaa4ddd, 93 },
    { 0x745a4ddd, 61 },
    { 0xa0ba4ddf, 61 },
    { 0, 0 },
    { 0x537a4de1, 33 },
    { 0xcf9a4de2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45a4de5, 77 },
    { 0xf32a4de5, 88 },
    { 0x534a4de7, 68 },
    { 0x5dea4de8, 90 },
    { 0, 0 },
    { 0xae8a4dea, 85 },
    { 0x954a4dea, 55 },
    { 0x8a4dec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a4df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x323a4df7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca4dfc, 61 },
    { 0x767a4dfd, 61 },
    { 0xfd5a4dfc, 61 },
    { 0xecfa4dfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x377a4e02, 90 },
    { 0x8d3a4e03, 77 },
    { 0, 0 },
    { 0x3ffa4e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4e0a, 90 },
    { 0x5fda4e0a, 87 },
    { 0xc1ba4e0a, 33 },
    { 0xa97a4e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a4e10, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a4e14, 98 },
    { 0x31ba4e15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53a4e1e, 23 },
    { 0xb08a4e1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55a4e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ea4e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347a4e2d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca4e35, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccda4e3a, 100 },
    { 0x598a4e3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7da4e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82a4e45, 95 },
    { 0xccca4e46, 61 },
    { 0x2d0a4e47, 78 },
    { 0, 0 },
    { 0xd03a4e49, 95 },
    { 0x14ea4e4a, 61 },
    { 0, 0 },
    { 0xaf6a4e4c, 90 },
    { 0x15aa4e4d, 95 },
    { 0xa6ea4e4e, 100 },
    { 0x50da4e4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6a4e62, 33 },
    { 0, 0 },
    { 0xad6a4e64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ba4e6c, 33 },
    { 0, 0 },
    { 0x8e6a4e6e, 23 },
    { 0x3dba4e6f, 55 },
    { 0, 0 },
    { 0x75aa4e71, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a4e75, 33 },
    { 0, 0 },
    { 0xa86a4e77, 34 },
    { 0, 0 },
    { 0x546a4e79, 78 },
    { 0, 0 },
    { 0xaeca4e7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a4e7f, 61 },
    { 0, 0 },
    { 0x8dfa4e81, 76 },
    { 0x2bca4e82, 61 },
    { 0xcf2a4e83, 33 },
    { 0x528a4e83, 33 },
    { 0x606a4e85, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a4e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99a4e93, 46 },
    { 0, 0 },
    { 0xfe1a4e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa4e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcba4e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa4ea0, 98 },
    { 0, 0 },
    { 0x86ea4ea2, 87 },
    { 0, 0 },
    { 0x91da4ea4, 90 },
    { 0x677a4ea4, 33 },
    { 0x5c2a4ea6, 100 },
    { 0xadda4ea6, 90 },
    { 0xe1da4ea8, 86 },
    { 0x4d7a4ea9, 61 },
    { 0xb8fa4eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a4eb1, 76 },
    { 0, 0 },
    { 0x86ba4eb3, 98 },
    { 0xc9a4eb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62a4eba, 98 },
    { 0xcaa4ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733a4ebf, 95 },
    { 0, 0 },
    { 0xd41a4ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6a4ec5, 61 },
    { 0x42fa4ec6, 93 },
    { 0x9a8a4ec7, 98 },
    { 0x685a4ec8, 98 },
    { 0xf8a4ec9, 55 },
    { 0xd74a4eca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecca4ecf, 11 },
    { 0x5c2a4ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475a4ed4, 33 },
    { 0xb04a4ed5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfa4ed8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a4edb, 23 },
    { 0x99fa4edc, 33 },
    { 0x796a4edd, 90 },
    { 0xc06a4ede, 23 },
    { 0xb10a4edf, 61 },
    { 0x5afa4edf, 76 },
    { 0x7c8a4ee0, 75 },
    { 0xd26a4ee2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80a4ee6, 48 },
    { 0x487a4ee7, 55 },
    { 0x461a4ee8, 55 },
    { 0x92ea4ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba4eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354a4ef1, 90 },
    { 0xf73a4ef2, 90 },
    { 0, 0 },
    { 0xbfba4ef4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x994a4ef7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a4efa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ca4f05, 23 },
    { 0x490a4f05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba4f09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fa4f0c, 33 },
    { 0xc71a4f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79da4f12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ba4f16, 98 },
    { 0x46a4f17, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea4f1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfa4f1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0a4f21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa4f24, 67 },
    { 0x8eaa4f25, 76 },
    { 0x4b0a4f26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a4f2b, 90 },
    { 0xc32a4f2b, 61 },
    { 0x4c9a4f2c, 90 },
    { 0x58fa4f2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a4f32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a4f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ba4f3e, 95 },
    { 0, 0 },
    { 0x68ca4f40, 91 },
    { 0, 0 },
    { 0xe96a4f42, 98 },
    { 0, 0 },
    { 0xae2a4f44, 85 },
    { 0x90ba4f44, 53 },
    { 0xd71a4f46, 43 },
    { 0x887a4f44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90a4f4b, 80 },
    { 0xddea4f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6a4f50, 98 },
    { 0x54fa4f50, 23 },
    { 0, 0 },
    { 0xc1aa4f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1a4f58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9a4f5c, 61 },
    { 0xe74a4f5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a4f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x923a4f66, 90 },
    { 0xad6a4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda4f6b, 90 },
    { 0, 0 },
    { 0x478a4f6d, 90 },
    { 0xa5aa4f6e, 33 },
    { 0xc29a4f6e, 98 },
    { 0x80ba4f70, 61 },
    { 0x66fa4f70, 43 },
    { 0x1e3a4f71, 43 },
    { 0x558a4f73, 69 },
    { 0x2d0a4f74, 23 },
    { 0x66ba4f74, 33 },
    { 0x15ba4f72, 98 },
    { 0x371a4f6e, 77 },
    { 0x82ba4f78, 61 },
    { 0, 0 },
    { 0x727a4f7a, 43 },
    { 0x3e7a4f7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a4f89, 34 },
    { 0x436a4f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa4f8d, 43 },
    { 0x967a4f8e, 98 },
    { 0x7c9a4f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba4f95, 95 },
    { 0, 0 },
    { 0xcc4a4f97, 100 },
    { 0xd7fa4f98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a4fa2, 98 },
    { 0xdcda4fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7da4faa, 76 },
    { 0x117a4faa, 95 },
    { 0, 0 },
    { 0x62ca4fad, 55 },
    { 0x1c6a4fae, 61 },
    { 0x20ba4faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467a4fb4, 98 },
    { 0, 0 },
    { 0xdd6a4fb6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca4fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca4fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a4fd3, 87 },
    { 0, 0 },
    { 0xa53a4fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a4fda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a4fe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada4fe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911a4fea, 98 },
    { 0xb0da4feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda4fee, 33 },
    { 0x7f3a4fef, 90 },
    { 0xe3fa4fee, 95 },
    { 0xdeea4ff1, 96 },
    { 0x7ca4ff2, 61 },
    { 0, 0 },
    { 0x24ba4ff4, 90 },
    { 0xca2a4ff5, 55 },
    { 0xf4ba4ff6, 90 },
    { 0xd31a4ff4, 67 },
    { 0x7eca4ff6, 98 },
    { 0x1e8a4ff4, 52 },
    { 0, 0 },
    { 0xe92a4ffb, 76 },
    { 0x176a4ffc, 55 },
    { 0, 0 },
    { 0x747a4ffe, 76 },
    { 0, 0 },
    { 0x9eda5000, 55 },
    { 0xdeba5000, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aea5004, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2a5007, 100 },
    { 0x945a5008, 33 },
    { 0x507a5009, 90 },
    { 0xbeba500a, 95 },
    { 0, 0 },
    { 0x5e6a500c, 33 },
    { 0x240a500d, 61 },
    { 0x594a500d, 90 },
    { 0, 0 },
    { 0x57a5010, 61 },
    { 0xa91a5011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafa5014, 61 },
    { 0, 0 },
    { 0x6f9a5016, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa5019, 46 },
    { 0xae7a501a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a5020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a5023, 23 },
    { 0, 0 },
    { 0x4bda5025, 33 },
    { 0, 0 },
    { 0xc4da5027, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da502c, 33 },
    { 0x10aa502c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ba5030, 33 },
    { 0xec3a5030, 67 },
    { 0xef8a5031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa5035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719a5039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7a5043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x816a5046, 93 },
    { 0xd44a5046, 61 },
    { 0xdeda5048, 33 },
    { 0xc0fa5046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a504d, 61 },
    { 0x275a504d, 85 },
    { 0x46ca504e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63a5053, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aa505b, 34 },
    { 0x611a505c, 36 },
    { 0xb1ba505d, 23 },
    { 0x60da505d, 61 },
    { 0x1a7a505d, 76 },
    { 0x73ca5060, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa5067, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da506a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa506e, 61 },
    { 0, 0 },
    { 0x7a5070, 37 },
    { 0x694a5070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8a5077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba507e, 84 },
    { 0x724a507e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915a5085, 100 },
    { 0x194a5085, 61 },
    { 0xdada5087, 98 },
    { 0x6e2a5088, 33 },
    { 0xfc1a5089, 93 },
    { 0x810a5089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a508d, 87 },
    { 0x730a508e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317a5092, 76 },
    { 0xc13a5092, 61 },
    { 0x92da5094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0a5098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497a509e, 95 },
    { 0x793a509e, 95 },
    { 0xa12a50a0, 91 },
    { 0x10ba50a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea50a4, 95 },
    { 0xb41a50a4, 61 },
    { 0, 0 },
    { 0xf5fa50a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a50ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4a50b1, 61 },
    { 0xe21a50b2, 78 },
    { 0x9f0a50b2, 52 },
    { 0x71da50b2, 43 },
    { 0x52ba50b5, 61 },
    { 0x228a50b4, 98 },
    { 0xcb6a50b2, 90 },
    { 0x1d7a50b8, 99 },
    { 0x6d3a50b4, 90 },
    { 0, 0 },
    { 0x46ca50bb, 76 },
    { 0xea6a50bb, 34 },
    { 0, 0 },
    { 0x10ca50be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a50c2, 95 },
    { 0xd77a50c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a50c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaa50cb, 66 },
    { 0, 0 },
    { 0x512a50cd, 61 },
    { 0, 0 },
    { 0xa53a50cf, 55 },
    { 0x82ea50d0, 95 },
    { 0xd3a50d1, 67 },
    { 0x964a50d1, 34 },
    { 0, 0 },
    { 0x3afa50d4, 98 },
    { 0, 0 },
    { 0x473a50d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a50d9, 29 },
    { 0x124a50da, 46 },
    { 0xb9ea50da, 95 },
    { 0, 0 },
    { 0x380a50dd, 95 },
    { 0, 0 },
    { 0x260a50df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a50e2, 93 },
    { 0x42aa50e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a50ea, 78 },
    { 0xca2a50ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66da50ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba50f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18a50f5, 95 },
    { 0x2b6a50f6, 100 },
    { 0xf0ba50f7, 23 },
    { 0x69fa50f5, 90 },
    { 0, 0 },
    { 0xcd7a50fa, 76 },
    { 0x4dba50fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8a50fe, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca5101, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a5104, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301a5108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da510c, 99 },
    { 0x443a510d, 33 },
    { 0, 0 },
    { 0x480a510f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aa511c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aa511f, 46 },
    { 0x3c0a5120, 98 },
    { 0xe5ea5121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a5127, 98 },
    { 0xafaa5128, 23 },
    { 0, 0 },
    { 0x52da512a, 90 },
    { 0xb43a512b, 64 },
    { 0x1b3a512c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121a5131, 61 },
    { 0xfbca5131, 61 },
    { 0, 0 },
    { 0xbb7a5134, 34 },
    { 0xdf1a5135, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87a513a, 98 },
    { 0, 0 },
    { 0x649a513c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfa5141, 43 },
    { 0x281a5141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171a5145, 90 },
    { 0xfe2a5146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ea5153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefba5157, 76 },
    { 0, 0 },
    { 0x696a5159, 90 },
    { 0, 0 },
    { 0x39ea515b, 85 },
    { 0xc8a515b, 23 },
    { 0, 0 },
    { 0xf96a515e, 90 },
    { 0, 0 },
    { 0x4aca5160, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a5169, 98 },
    { 0x47ea516a, 43 },
    { 0xe5ea516b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2a516f, 95 },
    { 0xc61a5170, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda5173, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca5176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a517b, 55 },
    { 0xfdea517b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17da5180, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48a5183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a5187, 33 },
    { 0x41ba5188, 84 },
    { 0xfeca5188, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa5190, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6a5195, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a519b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14a519f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17a51a2, 90 },
    { 0, 0 },
    { 0x981a51a4, 93 },
    { 0xf34a51a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca51ac, 95 },
    { 0x5c8a51ad, 90 },
    { 0, 0 },
    { 0x61ca51af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a51b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201a51b6, 34 },
    { 0, 0 },
    { 0xccfa51b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca51bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a51c0, 43 },
    { 0xdfba51c0, 33 },
    { 0x303a51c2, 69 },
    { 0, 0 },
    { 0xe67a51c4, 90 },
    { 0, 0 },
    { 0x24a51c6, 88 },
    { 0xa83a51c6, 88 },
    { 0xc5ca51c6, 34 },
    { 0x4faa51c9, 23 },
    { 0x551a51c9, 23 },
    { 0x5f8a51cb, 61 },
    { 0x9f5a51ca, 61 },
    { 0x1eba51cd, 98 },
    { 0xa2fa51cd, 98 },
    { 0x1dba51cf, 91 },
    { 0x5daa51cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0da51d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4aa51da, 23 },
    { 0xc50a51db, 76 },
    { 0, 0 },
    { 0x7da51dd, 61 },
    { 0x5bda51dd, 33 },
    { 0, 0 },
    { 0xde6a51e0, 95 },
    { 0xe5aa51e0, 33 },
    { 0x21a51e2, 101 },
    { 0, 0 },
    { 0x9f3a51e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51e8, 98 },
    { 0, 0 },
    { 0xf36a51ea, 66 },
    { 0xef2a51eb, 99 },
    { 0xc97a51ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bba51f2, 66 },
    { 0xffaa51f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5a51fd, 61 },
    { 0xa32a51fe, 93 },
    { 0x386a51ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffa5202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505a5206, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea520b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca520e, 23 },
    { 0x175a520f, 90 },
    { 0, 0 },
    { 0x3cda5211, 23 },
    { 0xb0ea5211, 98 },
    { 0x57ca5213, 61 },
    { 0xddda5212, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bda5217, 99 },
    { 0, 0 },
    { 0xa41a5219, 23 },
    { 0x281a5219, 33 },
    { 0, 0 },
    { 0xc86a521c, 95 },
    { 0, 0 },
    { 0x813a521e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea5223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x729a5226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ca5229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a522e, 46 },
    { 0x13fa522f, 61 },
    { 0x11ea522e, 33 },
    { 0x224a5231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74a5238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcba523b, 46 },
    { 0xe66a523c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da523f, 76 },
    { 0, 0 },
    { 0x9e0a5241, 61 },
    { 0x848a5242, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa5247, 90 },
    { 0xf65a5248, 98 },
    { 0xa20a5248, 98 },
    { 0x3b5a524a, 93 },
    { 0, 0 },
    { 0xafba524c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72a5250, 90 },
    { 0x2f5a5251, 65 },
    { 0, 0 },
    { 0x82ca5253, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa5256, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ea525d, 65 },
    { 0xab5a525e, 97 },
    { 0xf48a525f, 90 },
    { 0xcb4a5260, 91 },
    { 0x8dba5260, 70 },
    { 0xf43a525e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599a5266, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a526b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a5271, 90 },
    { 0x690a5271, 78 },
    { 0x86ea5273, 95 },
    { 0x8e1a5274, 90 },
    { 0xe2a5275, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51a527c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2a5287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400a528b, 61 },
    { 0xe6ca528c, 84 },
    { 0x261a528d, 33 },
    { 0, 0 },
    { 0x8c6a528f, 76 },
    { 0, 0 },
    { 0x486a5291, 98 },
    { 0x7fa5291, 90 },
    { 0, 0 },
    { 0x14ba5294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a5297, 88 },
    { 0xe07a5298, 61 },
    { 0xc83a5298, 33 },
    { 0x47ea5297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a529e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa52a3, 78 },
    { 0x3c2a52a4, 52 },
    { 0x5d5a52a5, 55 },
    { 0x569a52a5, 93 },
    { 0x7dda52a7, 65 },
    { 0x954a52a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33a52ac, 81 },
    { 0x2bea52ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a52b2, 33 },
    { 0x921a52b3, 33 },
    { 0x6cba52b3, 91 },
    { 0x560a52b3, 23 },
    { 0xa98a52b6, 98 },
    { 0x3b2a52b7, 91 },
    { 0x49aa52b8, 90 },
    { 0, 0 },
    { 0x222a52ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a52c1, 23 },
    { 0x816a52c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24a52c7, 19 },
    { 0x974a52c8, 91 },
    { 0x8afa52c8, 90 },
    { 0x7d0a52ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba52cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44a52d1, 33 },
    { 0xec1a52d2, 33 },
    { 0x9a2a52d3, 33 },
    { 0x236a52d3, 65 },
    { 0x791a52d2, 53 },
    { 0x7aca52d1, 23 },
    { 0, 0 },
    { 0xaa3a52d8, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea52e2, 90 },
    { 0x183a52e3, 35 },
    { 0x6a52e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185a52e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fa52f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa52f4, 23 },
    { 0, 0 },
    { 0xfdfa52f6, 98 },
    { 0x3fda52f6, 55 },
    { 0, 0 },
    { 0xecaa52f9, 23 },
    { 0xc55a52f9, 95 },
    { 0x649a52fb, 61 },
    { 0, 0 },
    { 0xb85a52fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aa5300, 76 },
    { 0, 0 },
    { 0xaa1a5302, 98 },
    { 0, 0 },
    { 0x9cea5304, 53 },
    { 0x8fca5304, 95 },
    { 0x41fa5305, 95 },
    { 0, 0 },
    { 0x54aa5308, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2a530e, 61 },
    { 0xd6ba530f, 55 },
    { 0x5ffa530f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fa5314, 78 },
    { 0x57fa5315, 77 },
    { 0xe7ca5316, 90 },
    { 0x26a5317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5a531b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6a5321, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8a5326, 46 },
    { 0x515a5326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dba532b, 95 },
    { 0x23ca532c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a532f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a5332, 67 },
    { 0xb13a5333, 61 },
    { 0x127a5332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6a5338, 76 },
    { 0, 0 },
    { 0xdcda533a, 85 },
    { 0xd69a533a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fa5341, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ca534c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ba534f, 90 },
    { 0, 0 },
    { 0x687a5351, 52 },
    { 0, 0 },
    { 0x237a5353, 66 },
    { 0x2c2a5354, 85 },
    { 0x7b9a5354, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a535d, 84 },
    { 0xebba535e, 91 },
    { 0x898a535f, 34 },
    { 0x33ca535f, 23 },
    { 0, 0 },
    { 0x665a5362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ea5366, 33 },
    { 0x8e1a5366, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a536c, 43 },
    { 0xc51a536d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5370, 69 },
    { 0xc29a5371, 100 },
    { 0, 0 },
    { 0xc6ca5373, 90 },
    { 0x2bda5374, 87 },
    { 0, 0 },
    { 0xc0fa5376, 100 },
    { 0x69fa5377, 76 },
    { 0xa6fa5378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52a537b, 61 },
    { 0, 0 },
    { 0xb3ba537d, 33 },
    { 0, 0 },
    { 0x6ba537f, 93 },
    { 0, 0 },
    { 0xd0da5381, 95 },
    { 0x7a7a5381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a5385, 61 },
    { 0x2c5a5386, 98 },
    { 0xdb6a5386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2da538d, 53 },
    { 0xad3a538d, 91 },
    { 0x556a538e, 52 },
    { 0x313a5390, 33 },
    { 0x666a538e, 23 },
    { 0xe4a5392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a5398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3a539b, 90 },
    { 0x793a539b, 43 },
    { 0, 0 },
    { 0x442a539e, 98 },
    { 0, 0 },
    { 0x8aea53a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417a53a4, 90 },
    { 0xafba53a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a53a9, 76 },
    { 0x9d7a53aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90a53b2, 90 },
    { 0xb3ea53b3, 34 },
    { 0x359a53b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4a53b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa53c3, 65 },
    { 0x827a53c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a53c8, 93 },
    { 0xe5aa53c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a53ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a53d3, 46 },
    { 0xc30a53d4, 55 },
    { 0xc6da53d3, 78 },
    { 0x8dba53d5, 33 },
    { 0x556a53d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503a53dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda53e0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x402a53e3, 23 },
    { 0, 0 },
    { 0x47ca53e5, 90 },
    { 0, 0 },
    { 0xd06a53e7, 33 },
    { 0xc5fa53e8, 23 },
    { 0, 0 },
    { 0xfc9a53ea, 98 },
    { 0xfd9a53ea, 76 },
    { 0, 0 },
    { 0x54ba53ed, 23 },
    { 0xe92a53ed, 33 },
    { 0xd39a53ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716a53f5, 77 },
    { 0xb34a53f6, 76 },
    { 0xf5ca53f7, 52 },
    { 0xc53a53f8, 90 },
    { 0, 0 },
    { 0xdcca53fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0da53fe, 65 },
    { 0xf64a53ff, 98 },
    { 0xa21a5400, 33 },
    { 0, 0 },
    { 0x270a5402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea540d, 34 },
    { 0, 0 },
    { 0x307a540f, 95 },
    { 0x506a5410, 93 },
    { 0, 0 },
    { 0x729a5412, 93 },
    { 0, 0 },
    { 0xb14a5414, 87 },
    { 0x9b7a5414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5a5418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca541c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a5421, 34 },
    { 0xac4a5421, 98 },
    { 0, 0 },
    { 0xdea5424, 66 },
    { 0x407a5425, 98 },
    { 0x138a5426, 34 },
    { 0x5d9a5424, 77 },
    { 0, 0 },
    { 0x7b0a5429, 90 },
    { 0x251a542a, 95 },
    { 0x31ea542b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448a5430, 55 },
    { 0x475a5431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a5435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1a5438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba543d, 91 },
    { 0, 0 },
    { 0x999a543f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba5444, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692a5449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfa544d, 76 },
    { 0x400a544d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ca5451, 95 },
    { 0x625a5451, 69 },
    { 0x4c7a5453, 40 },
    { 0x2e9a5452, 55 },
    { 0x35a5455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929a545a, 90 },
    { 0x210a545b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56a545e, 61 },
    { 0x29aa545f, 64 },
    { 0, 0 },
    { 0xbf5a5461, 61 },
    { 0x56ba5461, 78 },
    { 0xaba5463, 100 },
    { 0xfaea5463, 98 },
    { 0x5d1a5465, 23 },
    { 0xd1a5463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ba546d, 75 },
    { 0xbeca546d, 93 },
    { 0x3f6a546f, 65 },
    { 0xc99a546d, 76 },
    { 0x135a546d, 76 },
    { 0xa4aa5472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebca547a, 61 },
    { 0, 0 },
    { 0xb09a547c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4da5483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6da5486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a548a, 43 },
    { 0xe87a548b, 33 },
    { 0x3aa548a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca548f, 33 },
    { 0, 0 },
    { 0x837a5491, 61 },
    { 0x50ba5492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5a5496, 61 },
    { 0xce5a5497, 95 },
    { 0, 0 },
    { 0xef5a5499, 66 },
    { 0xa37a5499, 84 },
    { 0x952a549b, 95 },
    { 0xfbaa549a, 82 },
    { 0, 0 },
    { 0x3b3a549e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a54a1, 61 },
    { 0x7b2a54a2, 90 },
    { 0xd86a54a2, 95 },
    { 0x878a54a3, 100 },
    { 0xee5a54a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a54aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa54ae, 46 },
    { 0x97da54af, 61 },
    { 0xc74a54b0, 67 },
    { 0, 0 },
    { 0x2ba54b2, 90 },
    { 0, 0 },
    { 0xa0da54b4, 95 },
    { 0xcc5a54b5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca54bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112a54c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a54ca, 46 },
    { 0x70aa54cb, 51 },
    { 0x120a54cc, 90 },
    { 0x882a54cc, 98 },
    { 0x926a54cd, 46 },
    { 0xadda54cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1a54d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a54d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3a54d9, 66 },
    { 0x317a54d9, 95 },
    { 0x69ea54da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa54df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ca54e8, 90 },
    { 0x598a54e9, 76 },
    { 0, 0 },
    { 0xaf9a54eb, 90 },
    { 0x13fa54ec, 34 },
    { 0x828a54ed, 33 },
    { 0, 0 },
    { 0xae7a54ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7a54f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853a54fa, 23 },
    { 0x234a54fa, 65 },
    { 0xc2a54fc, 68 },
    { 0x660a54fc, 90 },
    { 0x91ea54fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x472a5501, 66 },
    { 0, 0 },
    { 0x9ffa5503, 95 },
    { 0xd72a5503, 90 },
    { 0x7d1a5505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cea5509, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3a550e, 55 },
    { 0x483a550f, 61 },
    { 0, 0 },
    { 0xe5da5511, 33 },
    { 0x318a5512, 33 },
    { 0xe4a5513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52aa5518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a551b, 33 },
    { 0x43da551c, 90 },
    { 0x906a551c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31da5524, 90 },
    { 0x1e1a5525, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba5528, 43 },
    { 0x3bba5528, 90 },
    { 0x18da552a, 99 },
    { 0xc83a5528, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0a552e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x665a5531, 76 },
    { 0, 0 },
    { 0x8a9a5533, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a553d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a554a, 61 },
    { 0xa7ca554b, 65 },
    { 0xfe9a554c, 98 },
    { 0x633a554c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda5550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aa5553, 88 },
    { 0x404a5554, 61 },
    { 0x376a5555, 99 },
    { 0x878a5555, 97 },
    { 0x88ea5556, 23 },
    { 0x99ba5558, 77 },
    { 0x115a5556, 23 },
    { 0xf63a555a, 10 },
    { 0xee5a555a, 91 },
    { 0, 0 },
    { 0x7ba555d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aa5560, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40a5566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561a556a, 52 },
    { 0xd8fa556a, 48 },
    { 0x129a556c, 33 },
    { 0x909a556d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a5570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6a5573, 98 },
    { 0xf73a5573, 23 },
    { 0x52a5575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a5578, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca557c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a5580, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bba5583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eba5586, 81 },
    { 0x35aa5587, 90 },
    { 0, 0 },
    { 0xbeaa5589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aa5590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a559e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fa55a2, 66 },
    { 0, 0 },
    { 0x7dfa55a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a55ab, 49 },
    { 0xbbca55ac, 71 },
    { 0xa97a55ad, 36 },
    { 0xdc4a55ae, 43 },
    { 0, 0 },
    { 0x9baa55b0, 78 },
    { 0x619a55b1, 97 },
    { 0x73a55b2, 76 },
    { 0x1d7a55b0, 90 },
    { 0x9e0a55b4, 33 },
    { 0x19aa55b5, 78 },
    { 0x8fda55b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86a55ba, 23 },
    { 0xbaa55ba, 90 },
    { 0, 0 },
    { 0x833a55bd, 52 },
    { 0x14ba55be, 98 },
    { 0x928a55bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a55c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffa55c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a55cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a55d0, 69 },
    { 0, 0 },
    { 0x3ffa55d2, 90 },
    { 0x944a55d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa55d9, 61 },
    { 0x285a55da, 90 },
    { 0xeda55da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x281a55de, 98 },
    { 0, 0 },
    { 0xb8a55e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bda55e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a55ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a55f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74da55fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a5600, 33 },
    { 0xd09a5601, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5606, 23 },
    { 0xd99a5607, 91 },
    { 0xe84a5608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a560e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172a5616, 61 },
    { 0x427a5617, 77 },
    { 0xe4ca5616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba561d, 90 },
    { 0x385a561e, 78 },
    { 0xc6ba561e, 98 },
    { 0x3f5a5620, 90 },
    { 0x6bfa561e, 91 },
    { 0x2b7a5622, 46 },
    { 0xd55a5623, 100 },
    { 0, 0 },
    { 0x366a5625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a562a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371a562f, 52 },
    { 0xa37a5630, 95 },
    { 0x3c2a5630, 93 },
    { 0x15ba5632, 33 },
    { 0xd5ba5633, 98 },
    { 0xe75a5631, 33 },
    { 0, 0 },
    { 0xb4fa5636, 69 },
    { 0xb76a5637, 33 },
    { 0x159a5638, 55 },
    { 0xc8ba5639, 95 },
    { 0xb07a563a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6a563f, 95 },
    { 0, 0 },
    { 0x137a5641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ca5645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea5648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa564b, 52 },
    { 0x39fa564c, 48 },
    { 0, 0 },
    { 0x184a564e, 76 },
    { 0, 0 },
    { 0x444a5650, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea5654, 90 },
    { 0x9aa5655, 61 },
    { 0x650a5656, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a565b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a565e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a5661, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa5664, 90 },
    { 0x9b3a5664, 95 },
    { 0x1dba5666, 98 },
    { 0x684a5665, 33 },
    { 0x1caa5668, 33 },
    { 0x569a5665, 33 },
    { 0x719a5667, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaa566d, 84 },
    { 0x5d7a566d, 98 },
    { 0x725a566e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a5673, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fa5679, 43 },
    { 0x20da567a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a5683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382a5688, 90 },
    { 0xea5688, 66 },
    { 0, 0 },
    { 0x1a5a568b, 43 },
    { 0, 0 },
    { 0xbd1a568d, 61 },
    { 0x93ca568e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a5691, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5695, 101 },
    { 0x98a5696, 34 },
    { 0x2aa5697, 66 },
    { 0x975a5697, 91 },
    { 0, 0 },
    { 0xd92a569a, 90 },
    { 0, 0 },
    { 0x3c3a569c, 91 },
    { 0, 0 },
    { 0x4eca569e, 55 },
    { 0x569a569f, 61 },
    { 0, 0 },
    { 0xeeca56a1, 55 },
    { 0x135a56a2, 42 },
    { 0xa11a56a2, 23 },
    { 0, 0 },
    { 0xa34a56a5, 90 },
    { 0x167a56a6, 90 },
    { 0xd99a56a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ca56aa, 87 },
    { 0x6dda56ab, 61 },
    { 0x7c0a56aa, 90 },
    { 0x3a1a56aa, 43 },
    { 0xd83a56ae, 91 },
    { 0xd4fa56af, 60 },
    { 0x9bca56ae, 99 },
    { 0xb19a56b1, 61 },
    { 0xd39a56b1, 33 },
    { 0xcf0a56b1, 95 },
    { 0xe44a56b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a56b8, 99 },
    { 0, 0 },
    { 0x91da56ba, 34 },
    { 0, 0 },
    { 0x271a56bc, 43 },
    { 0, 0 },
    { 0x146a56be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398a56c2, 76 },
    { 0, 0 },
    { 0x6b6a56c4, 96 },
    { 0, 0 },
    { 0xfaaa56c6, 61 },
    { 0xb7fa56c7, 90 },
    { 0, 0 },
    { 0x924a56c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a56cd, 90 },
    { 0x4e2a56cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca56d3, 15 },
    { 0, 0 },
    { 0xe13a56d5, 55 },
    { 0xdd8a56d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a56d9, 23 },
    { 0x2d4a56da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239a56df, 46 },
    { 0x50aa56df, 33 },
    { 0x6e4a56e1, 33 },
    { 0xc18a56e2, 98 },
    { 0x32ca56e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fa56e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x272a56eb, 93 },
    { 0xda0a56ec, 46 },
    { 0x514a56ed, 76 },
    { 0xcefa56ee, 33 },
    { 0, 0 },
    { 0xfcea56f0, 61 },
    { 0x8fda56f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a56f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa56fb, 55 },
    { 0x1aa56fc, 90 },
    { 0x625a56fd, 61 },
    { 0xb23a56fe, 61 },
    { 0xef2a56fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ba5706, 34 },
    { 0, 0 },
    { 0x931a5708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a570b, 95 },
    { 0x8bba570c, 95 },
    { 0, 0 },
    { 0x966a570e, 33 },
    { 0, 0 },
    { 0x49fa5710, 55 },
    { 0, 0 },
    { 0x89fa5712, 33 },
    { 0x278a5713, 95 },
    { 0xfcca5713, 61 },
    { 0x833a5712, 55 },
    { 0x9fea5716, 66 },
    { 0x818a5717, 55 },
    { 0xc4aa5718, 43 },
    { 0x968a5719, 55 },
    { 0x323a5718, 68 },
    { 0x17ba571b, 23 },
    { 0x6efa5718, 78 },
    { 0, 0 },
    { 0x19aa571e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba5722, 61 },
    { 0xe54a5723, 23 },
    { 0, 0 },
    { 0x39da5725, 33 },
    { 0x799a5726, 95 },
    { 0x806a5727, 61 },
    { 0xf20a5727, 90 },
    { 0, 0 },
    { 0x652a572a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea572f, 95 },
    { 0x5f6a5730, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa5734, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a5739, 91 },
    { 0x1f3a573a, 90 },
    { 0x30ea573a, 61 },
    { 0x6a1a573c, 76 },
    { 0, 0 },
    { 0xcc2a573e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afa5742, 93 },
    { 0x73aa5743, 93 },
    { 0x430a5744, 95 },
    { 0x99da5744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da5749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a574c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba574f, 98 },
    { 0, 0 },
    { 0xe7a5751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39a5754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5aa575d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da5764, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a5767, 95 },
    { 0, 0 },
    { 0x6afa5769, 88 },
    { 0x122a576a, 23 },
    { 0x38ba576a, 95 },
    { 0x21aa576c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7a5772, 61 },
    { 0x3ada5772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460a577a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5a577d, 61 },
    { 0x75aa577e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5a5784, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea5788, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a5791, 23 },
    { 0, 0 },
    { 0x3bfa5793, 33 },
    { 0x9da5794, 46 },
    { 0x3eda5795, 67 },
    { 0x22da5793, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a579b, 33 },
    { 0x6e2a579b, 61 },
    { 0x8c3a579d, 95 },
    { 0xe8ba579e, 33 },
    { 0xff4a579d, 61 },
    { 0xeaaa57a0, 69 },
    { 0x3f7a57a0, 78 },
    { 0xee1a57a2, 98 },
    { 0x314a57a3, 33 },
    { 0x53da57a2, 33 },
    { 0x85ca57a5, 33 },
    { 0x739a579e, 93 },
    { 0x740a57a7, 61 },
    { 0x788a57a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8a57af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ea57b5, 69 },
    { 0x95fa57b6, 55 },
    { 0x37da57b7, 91 },
    { 0x99ca57b8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a57bc, 61 },
    { 0, 0 },
    { 0x9da57be, 81 },
    { 0xad7a57bf, 81 },
    { 0xfd9a57c0, 95 },
    { 0xd09a57c1, 90 },
    { 0xfaea57c2, 49 },
    { 0x1fba57bf, 76 },
    { 0xe9ba57c3, 65 },
    { 0x448a57c5, 100 },
    { 0xd31a57bf, 84 },
    { 0x88ca57c7, 66 },
    { 0x69fa57c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea57cc, 14 },
    { 0, 0 },
    { 0xf03a57ce, 98 },
    { 0x749a57ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba57d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a57d7, 98 },
    { 0xfc0a57d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafa57de, 23 },
    { 0, 0 },
    { 0x4daa57e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8a57e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca57e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a57ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426a57f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aa57ff, 43 },
    { 0xa87a5800, 33 },
    { 0x4e7a57ff, 90 },
    { 0x776a5802, 95 },
    { 0x9a0a5802, 23 },
    { 0xc82a5803, 90 },
    { 0, 0 },
    { 0xacaa5806, 34 },
    { 0, 0 },
    { 0x1f6a5808, 61 },
    { 0x7aea5809, 90 },
    { 0x463a580a, 90 },
    { 0x1a9a5809, 67 },
    { 0x1d5a580c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a5813, 43 },
    { 0xb57a5814, 77 },
    { 0x724a5814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bca581a, 78 },
    { 0, 0 },
    { 0x6c7a581c, 88 },
    { 0x863a581c, 90 },
    { 0, 0 },
    { 0xdba581f, 55 },
    { 0x688a5820, 84 },
    { 0x8b8a5821, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba582a, 23 },
    { 0xb77a582a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaa582f, 95 },
    { 0, 0 },
    { 0x516a5831, 55 },
    { 0xa56a5831, 91 },
    { 0x5afa5833, 23 },
    { 0x9bea5834, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a5838, 46 },
    { 0x232a5839, 61 },
    { 0xbf7a583a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92a5840, 76 },
    { 0x42ea5841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566a5845, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ba584f, 90 },
    { 0, 0 },
    { 0x2e2a5851, 90 },
    { 0x926a5852, 46 },
    { 0x7b3a5852, 43 },
    { 0x1caa5853, 23 },
    { 0x304a5851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2a5858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a585e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61a5869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8a586d, 95 },
    { 0xac0a586e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246a5873, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x603a5876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765a587a, 91 },
    { 0xe76a587b, 23 },
    { 0x1cea587c, 90 },
    { 0xdf6a587c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da5881, 91 },
    { 0x2f9a5881, 93 },
    { 0x13fa5883, 23 },
    { 0x63ba5881, 98 },
    { 0, 0 },
    { 0x263a5886, 34 },
    { 0x1daa5886, 61 },
    { 0xec7a5887, 95 },
    { 0x946a5889, 76 },
    { 0xc9fa5889, 43 },
    { 0x11da588b, 23 },
    { 0xe3fa588a, 52 },
    { 0xef3a588d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x259a5890, 95 },
    { 0, 0 },
    { 0x411a5892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8a5898, 23 },
    { 0, 0 },
    { 0xa1fa589a, 23 },
    { 0, 0 },
    { 0x8c6a589c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca589f, 92 },
    { 0x29ba58a0, 95 },
    { 0xfca58a1, 34 },
    { 0xec2a58a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a58a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a58af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668a58b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa58b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fa58b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a58bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1a58c1, 95 },
    { 0xdada58c2, 85 },
    { 0xfa8a58c2, 23 },
    { 0x82fa58c2, 95 },
    { 0x882a58c5, 43 },
    { 0, 0 },
    { 0x951a58c7, 43 },
    { 0x6f4a58c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1a58cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a58d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ba58d6, 88 },
    { 0, 0 },
    { 0x6bca58d8, 69 },
    { 0xf4da58d8, 90 },
    { 0xca2a58da, 90 },
    { 0, 0 },
    { 0x548a58dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9a58e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a58e9, 33 },
    { 0xdcfa58ea, 99 },
    { 0x7a8a58eb, 76 },
    { 0xa3fa58ec, 98 },
    { 0x8bea58ed, 98 },
    { 0xc37a58ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72a58f2, 90 },
    { 0x1b3a58f2, 87 },
    { 0x29ba58f2, 55 },
    { 0xa4a58f2, 96 },
    { 0x75a58f6, 90 },
    { 0, 0 },
    { 0xb95a58f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a58fb, 78 },
    { 0, 0 },
    { 0x95fa58fd, 23 },
    { 0, 0 },
    { 0x38fa58ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1a5903, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bda5906, 100 },
    { 0x3c6a5907, 61 },
    { 0x3d0a5907, 33 },
    { 0x3b4a5909, 53 },
    { 0x634a590a, 90 },
    { 0x50ba590b, 66 },
    { 0x32ca590a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ea590f, 72 },
    { 0, 0 },
    { 0x76fa5911, 91 },
    { 0x66ea5911, 78 },
    { 0x8d8a5913, 90 },
    { 0, 0 },
    { 0x30fa5915, 95 },
    { 0xa78a5916, 76 },
    { 0xaeea5915, 90 },
    { 0xa13a5918, 55 },
    { 0xadfa5919, 33 },
    { 0, 0 },
    { 0x72ca591b, 98 },
    { 0, 0 },
    { 0xc86a591d, 90 },
    { 0xaeca591d, 55 },
    { 0xc4a591f, 61 },
    { 0, 0 },
    { 0x980a5921, 55 },
    { 0xe64a5922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x484a5925, 90 },
    { 0xb35a5925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9a5929, 61 },
    { 0, 0 },
    { 0x693a592b, 55 },
    { 0xccba592b, 40 },
    { 0, 0 },
    { 0x307a592e, 99 },
    { 0xa3ea592f, 95 },
    { 0x95da5930, 55 },
    { 0xd7ea5931, 23 },
    { 0x67fa5932, 76 },
    { 0xe39a5930, 90 },
    { 0xccda5934, 95 },
    { 0x618a5935, 61 },
    { 0xf52a5936, 61 },
    { 0x57ea5934, 84 },
    { 0xfb0a5934, 83 },
    { 0x7eda5935, 95 },
    { 0, 0 },
    { 0xa9da593b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788a5940, 33 },
    { 0x16fa5941, 95 },
    { 0xf3aa5942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3a5945, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0a594b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a594e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934a5952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786a5955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fba595b, 33 },
    { 0, 0 },
    { 0x619a595d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41a5962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4a5966, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8a596b, 43 },
    { 0x81fa596c, 98 },
    { 0xb65a596c, 93 },
    { 0, 0 },
    { 0x148a596f, 33 },
    { 0x1e2a5970, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a5977, 81 },
    { 0xba7a5977, 34 },
    { 0x882a5979, 66 },
    { 0x49ca597a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea597d, 23 },
    { 0x9f5a597e, 91 },
    { 0x43ea597f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a5982, 48 },
    { 0, 0 },
    { 0xf4ea5984, 90 },
    { 0xf89a5985, 43 },
    { 0x521a5985, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9a598a, 90 },
    { 0x982a598a, 81 },
    { 0x253a598b, 33 },
    { 0x9b4a598a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a5991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ca5994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49a5997, 85 },
    { 0x103a5998, 33 },
    { 0x5c3a5999, 33 },
    { 0x8c5a5999, 95 },
    { 0, 0 },
    { 0x985a599c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcada599f, 98 },
    { 0xe6ca599f, 65 },
    { 0x66ba59a0, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9a59a4, 76 },
    { 0x259a59a4, 100 },
    { 0xb0da59a4, 95 },
    { 0x1fda59a7, 92 },
    { 0, 0 },
    { 0xa13a59a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a59ac, 90 },
    { 0xbc5a59ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99a59b0, 98 },
    { 0, 0 },
    { 0x2e3a59b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca59b8, 52 },
    { 0, 0 },
    { 0x821a59ba, 55 },
    { 0x37da59ba, 98 },
    { 0xb47a59bc, 90 },
    { 0x8b1a59bd, 34 },
    { 0xfc9a59be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea59c3, 85 },
    { 0xb21a59c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9a59cb, 23 },
    { 0x95ba59cb, 23 },
    { 0x2b6a59cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a59d0, 61 },
    { 0xdb7a59d1, 98 },
    { 0x284a59d2, 55 },
    { 0xb25a59d0, 90 },
    { 0x681a59d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fa59d7, 95 },
    { 0xc3ca59d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a59db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x198a59de, 98 },
    { 0, 0 },
    { 0x9d1a59e0, 98 },
    { 0xa64a59e1, 90 },
    { 0x52ea59e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fa59e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fa59e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa59ef, 99 },
    { 0, 0 },
    { 0x510a59f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853a59fe, 87 },
    { 0x2cda59ff, 33 },
    { 0, 0 },
    { 0xf80a5a01, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ca5a05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2da5a0a, 47 },
    { 0x8cea5a0b, 33 },
    { 0xd19a5a0b, 65 },
    { 0, 0 },
    { 0x378a5a0e, 66 },
    { 0, 0 },
    { 0xeaaa5a10, 61 },
    { 0x82ea5a11, 98 },
    { 0x245a5a11, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35a5a15, 55 },
    { 0xc6ba5a15, 90 },
    { 0, 0 },
    { 0xbfda5a18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a5a1c, 84 },
    { 0, 0 },
    { 0xad5a5a1e, 98 },
    { 0xc35a5a1f, 66 },
    { 0, 0 },
    { 0x89ba5a21, 33 },
    { 0xd46a5a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a5a29, 33 },
    { 0x780a5a2a, 90 },
    { 0xbfba5a2b, 90 },
    { 0, 0 },
    { 0x77da5a2d, 33 },
    { 0x594a5a2e, 61 },
    { 0xdca5a2e, 90 },
    { 0x14a5a30, 98 },
    { 0, 0 },
    { 0x92ea5a32, 69 },
    { 0, 0 },
    { 0x782a5a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaca5a38, 61 },
    { 0, 0 },
    { 0x726a5a3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda5a3d, 23 },
    { 0xca2a5a3e, 95 },
    { 0xd6ba5a3d, 90 },
    { 0xbf3a5a40, 90 },
    { 0, 0 },
    { 0x925a5a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3a5a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fa5a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79a5a54, 36 },
    { 0x620a5a55, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20da5a5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aa5a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869a5a64, 90 },
    { 0xb3ca5a65, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a5a68, 43 },
    { 0x93ca5a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a5a6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a5a70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a5a77, 34 },
    { 0, 0 },
    { 0xe34a5a79, 55 },
    { 0, 0 },
    { 0x6caa5a7b, 90 },
    { 0, 0 },
    { 0x9d1a5a7d, 84 },
    { 0, 0 },
    { 0xfd6a5a7f, 55 },
    { 0x146a5a80, 55 },
    { 0, 0 },
    { 0xb1ba5a82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a5a89, 65 },
    { 0, 0 },
    { 0x8fca5a8b, 61 },
    { 0x694a5a8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da5a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea5aa0, 23 },
    { 0x14a5aa0, 98 },
    { 0x1caa5aa2, 76 },
    { 0x97ca5aa2, 34 },
    { 0xebea5aa2, 90 },
    { 0, 0 },
    { 0x8da5aa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bca5aaa, 98 },
    { 0x16fa5aab, 90 },
    { 0x8ada5aab, 61 },
    { 0x769a5aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42a5ab0, 95 },
    { 0x1dca5ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da5ab5, 33 },
    { 0x3b8a5ab6, 40 },
    { 0, 0 },
    { 0x58ea5ab8, 90 },
    { 0x6bfa5ab8, 87 },
    { 0xe1fa5aba, 95 },
    { 0x825a5ab8, 95 },
    { 0, 0 },
    { 0xa90a5abd, 98 },
    { 0xf2fa5abe, 55 },
    { 0, 0 },
    { 0x23aa5ac0, 99 },
    { 0x868a5ac1, 23 },
    { 0x4b3a5ac1, 33 },
    { 0xac9a5ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a5acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567a5ad1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a5ad4, 61 },
    { 0x999a5ad4, 33 },
    { 0x4ea5ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa5ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25a5add, 76 },
    { 0x315a5ade, 95 },
    { 0x172a5adf, 46 },
    { 0xc35a5ade, 95 },
    { 0xa75a5ae1, 76 },
    { 0x837a5ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x193a5ae5, 4 },
    { 0x20a5ae6, 98 },
    { 0x866a5ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda5aeb, 55 },
    { 0xa66a5aec, 85 },
    { 0x2d5a5aec, 75 },
    { 0xfb0a5aeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a5af5, 34 },
    { 0x931a5af6, 67 },
    { 0x14da5af7, 99 },
    { 0, 0 },
    { 0xf9aa5af9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ca5afe, 55 },
    { 0x1caa5aff, 96 },
    { 0x193a5b00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bba5b03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a5b07, 99 },
    { 0xdd8a5b08, 76 },
    { 0x429a5b09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbea5b15, 28 },
    { 0x811a5b15, 55 },
    { 0, 0 },
    { 0x54a5b18, 85 },
    { 0xd61a5b18, 51 },
    { 0x490a5b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a5b1f, 99 },
    { 0x2d3a5b20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110a5b25, 98 },
    { 0xfda5b25, 76 },
    { 0, 0 },
    { 0x24a5b28, 23 },
    { 0xe51a5b29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8a5b2c, 52 },
    { 0xd1da5b2d, 95 },
    { 0x265a5b2d, 90 },
    { 0xc62a5b2e, 34 },
    { 0, 0 },
    { 0x7d5a5b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa5b34, 91 },
    { 0x340a5b34, 90 },
    { 0x50aa5b36, 93 },
    { 0x9d8a5b37, 90 },
    { 0x6cda5b38, 68 },
    { 0xc8ba5b35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aa5b3c, 100 },
    { 0x35fa5b3c, 87 },
    { 0xa54a5b3e, 76 },
    { 0x6aba5b3f, 19 },
    { 0xceaa5b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda5b45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a5b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70a5b4d, 90 },
    { 0x20a5b4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a5b54, 66 },
    { 0x861a5b55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ba5b5d, 76 },
    { 0x996a5b5e, 40 },
    { 0xa79a5b5f, 61 },
    { 0xbc5a5b60, 73 },
    { 0xf60a5b5d, 43 },
    { 0x4e1a5b5e, 91 },
    { 0x880a5b63, 96 },
    { 0x9aa5b63, 93 },
    { 0xbe7a5b65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ea5b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a5b6b, 55 },
    { 0x794a5b6c, 61 },
    { 0x78ca5b6b, 98 },
    { 0x6c0a5b6e, 23 },
    { 0x595a5b6e, 67 },
    { 0x484a5b70, 33 },
    { 0x55ea5b71, 23 },
    { 0x9bea5b72, 40 },
    { 0x302a5b73, 55 },
    { 0xd2da5b72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8a5b78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbea5b84, 90 },
    { 0, 0 },
    { 0xc35a5b86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cda5b89, 78 },
    { 0, 0 },
    { 0x3c8a5b8b, 93 },
    { 0, 0 },
    { 0x79ba5b8d, 93 },
    { 0x5b8a5b8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5b96, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ada5b99, 95 },
    { 0xdd7a5b9a, 61 },
    { 0x900a5b9b, 33 },
    { 0xa0ea5b9b, 33 },
    { 0xacda5b9a, 98 },
    { 0xa22a5b9e, 90 },
    { 0x8f1a5b9f, 98 },
    { 0xbeaa5ba0, 43 },
    { 0x283a5b9f, 34 },
    { 0, 0 },
    { 0x837a5ba3, 90 },
    { 0x17da5ba4, 99 },
    { 0, 0 },
    { 0xe6aa5ba6, 78 },
    { 0x7bba5ba7, 78 },
    { 0x70ca5ba8, 91 },
    { 0, 0 },
    { 0xf1a5baa, 33 },
    { 0x10ea5bab, 95 },
    { 0xcbba5baa, 84 },
    { 0x443a5bad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bb1, 90 },
    { 0xd05a5bb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea5bb5, 33 },
    { 0x3f8a5bb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91a5bba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aa5bc0, 95 },
    { 0x5d7a5bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a5bc4, 99 },
    { 0xcf2a5bc5, 95 },
    { 0xfbba5bc6, 75 },
    { 0xda7a5bc7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49a5bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5bd2, 61 },
    { 0, 0 },
    { 0xd5ba5bd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a5bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ea5bdd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a5be0, 85 },
    { 0xd39a5be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a5be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a5be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a5bee, 43 },
    { 0xc94a5bef, 76 },
    { 0, 0 },
    { 0xca5a5bf1, 90 },
    { 0xf4a5bf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa5bf5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa5bfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3a5bfd, 57 },
    { 0xc56a5bfd, 93 },
    { 0xa67a5bff, 90 },
    { 0xdbaa5c00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa5c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba5c07, 23 },
    { 0xc1ea5c08, 68 },
    { 0xe96a5c09, 90 },
    { 0xc15a5c07, 23 },
    { 0, 0 },
    { 0xc46a5c0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16da5c0f, 52 },
    { 0xd21a5c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aea5c14, 23 },
    { 0x6aca5c15, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca5c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a5c1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea5c21, 90 },
    { 0xbf1a5c21, 69 },
    { 0x320a5c23, 77 },
    { 0x3aca5c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a5c27, 33 },
    { 0x814a5c28, 98 },
    { 0x1b8a5c29, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1da5c2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27a5c31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454a5c39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca5c3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a5c40, 23 },
    { 0, 0 },
    { 0x4baa5c42, 93 },
    { 0xfbaa5c43, 90 },
    { 0x80ba5c44, 23 },
    { 0x739a5c45, 34 },
    { 0, 0 },
    { 0xbba5c47, 23 },
    { 0xd44a5c48, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a5c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270a5c54, 98 },
    { 0, 0 },
    { 0x701a5c56, 46 },
    { 0xaada5c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ea5c5b, 98 },
    { 0xa2ea5c5c, 23 },
    { 0xf35a5c5d, 39 },
    { 0, 0 },
    { 0x897a5c5f, 72 },
    { 0x67ea5c5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2aa5c6e, 98 },
    { 0, 0 },
    { 0xd9aa5c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a5c75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda5c7f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ba5c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda5c97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeba5c9a, 55 },
    { 0xa51a5c9b, 23 },
    { 0xe19a5c9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffa5ca0, 55 },
    { 0x7b6a5ca1, 61 },
    { 0xeb6a5ca1, 95 },
    { 0x42fa5ca3, 33 },
    { 0x4cfa5ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea5caa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a5cad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3a5cb3, 49 },
    { 0, 0 },
    { 0x8c5a5cb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cca5cbb, 33 },
    { 0x176a5cbb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a5cc1, 33 },
    { 0x593a5cc1, 87 },
    { 0, 0 },
    { 0x86a5cc4, 95 },
    { 0x6bea5cc5, 90 },
    { 0x467a5cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddda5cc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x674a5ccc, 90 },
    { 0xd71a5ccd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29a5cd5, 98 },
    { 0x46ea5cd5, 33 },
    { 0, 0 },
    { 0x33ea5cd8, 98 },
    { 0, 0 },
    { 0x294a5cda, 88 },
    { 0xc6aa5cdb, 67 },
    { 0, 0 },
    { 0x81ca5cdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa5ce0, 77 },
    { 0x95da5ce1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a5ce6, 100 },
    { 0xe3ca5ce7, 24 },
    { 0, 0 },
    { 0x449a5ce9, 23 },
    { 0, 0 },
    { 0xf44a5ceb, 33 },
    { 0x174a5cec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a5cf0, 98 },
    { 0, 0 },
    { 0x22fa5cf2, 96 },
    { 0xa06a5cf2, 84 },
    { 0x5c6a5cf3, 84 },
    { 0xcfaa5cf5, 55 },
    { 0xf43a5cf6, 29 },
    { 0xf68a5cf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3a5cfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31a5d03, 23 },
    { 0xf72a5d04, 90 },
    { 0xc5da5d05, 61 },
    { 0x3bea5d06, 95 },
    { 0xa38a5d07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558a5d0c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a5d0f, 44 },
    { 0x632a5d0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750a5d13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x509a5d16, 23 },
    { 0x433a5d17, 76 },
    { 0, 0 },
    { 0xe9da5d19, 23 },
    { 0x8a2a5d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da5d1e, 75 },
    { 0, 0 },
    { 0xd1da5d20, 90 },
    { 0, 0 },
    { 0xda1a5d22, 33 },
    { 0x87da5d23, 52 },
    { 0x162a5d24, 99 },
    { 0x978a5d25, 61 },
    { 0xda3a5d24, 90 },
    { 0xaaa5d27, 88 },
    { 0x182a5d26, 95 },
    { 0xe99a5d29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612a5d2d, 85 },
    { 0, 0 },
    { 0x462a5d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ca5d34, 93 },
    { 0x7afa5d34, 33 },
    { 0, 0 },
    { 0x4fa5d37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ca5d3b, 93 },
    { 0xf39a5d3b, 93 },
    { 0xeeda5d3d, 76 },
    { 0x26ca5d3e, 66 },
    { 0, 0 },
    { 0xb7ca5d40, 46 },
    { 0xe69a5d40, 55 },
    { 0x372a5d40, 90 },
    { 0x35ea5d43, 95 },
    { 0x5f9a5d44, 23 },
    { 0xcf3a5d45, 61 },
    { 0xbd3a5d46, 95 },
    { 0xda2a5d47, 61 },
    { 0xc58a5d48, 90 },
    { 0x1f7a5d44, 34 },
    { 0x38ea5d4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a5d4d, 95 },
    { 0xec4a5d4d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa5d58, 31 },
    { 0x5f3a5d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a5d5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a5d62, 85 },
    { 0xfbaa5d62, 23 },
    { 0x884a5d63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a5d6a, 84 },
    { 0x8ba5d6b, 76 },
    { 0x692a5d6c, 95 },
    { 0x46a5d6c, 67 },
    { 0x2efa5d6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda5d71, 90 },
    { 0xeb9a5d72, 78 },
    { 0xef3a5d72, 66 },
    { 0, 0 },
    { 0xec8a5d75, 61 },
    { 0, 0 },
    { 0x97ea5d77, 23 },
    { 0x66fa5d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5d7c, 33 },
    { 0x34da5d7d, 91 },
    { 0xc4aa5d7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41a5d84, 87 },
    { 0, 0 },
    { 0x39aa5d86, 90 },
    { 0x7a3a5d87, 90 },
    { 0x2f6a5d86, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57a5d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56da5d94, 77 },
    { 0xa47a5d95, 61 },
    { 0, 0 },
    { 0x7b3a5d97, 33 },
    { 0, 0 },
    { 0x561a5d99, 34 },
    { 0, 0 },
    { 0xbc2a5d9b, 90 },
    { 0x155a5d9b, 99 },
    { 0x9bda5d9b, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964a5da4, 23 },
    { 0xad1a5da4, 43 },
    { 0xafea5da6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a5dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aa5db0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a5db3, 100 },
    { 0, 0 },
    { 0x85a5db5, 77 },
    { 0xf4ca5db5, 76 },
    { 0xd1fa5db7, 95 },
    { 0x2aca5db7, 55 },
    { 0, 0 },
    { 0xb56a5dba, 52 },
    { 0x8a1a5dbb, 61 },
    { 0x26ba5dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x855a5dbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a5dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a5dc9, 61 },
    { 0xea5dca, 88 },
    { 0xe5da5dc9, 23 },
    { 0, 0 },
    { 0xfeaa5dcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa5dd2, 95 },
    { 0x3d0a5dd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a5dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57a5de1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ca5de7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3a5dec, 93 },
    { 0xeefa5dec, 88 },
    { 0, 0 },
    { 0x25fa5def, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaa5df6, 23 },
    { 0x783a5df7, 23 },
    { 0x227a5df8, 98 },
    { 0, 0 },
    { 0x4eca5dfa, 61 },
    { 0xecca5dfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1a5dfe, 84 },
    { 0xa72a5dff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ca5e03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696a5e09, 67 },
    { 0, 0 },
    { 0xb70a5e0b, 95 },
    { 0, 0 },
    { 0xf0ba5e0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a5e13, 84 },
    { 0, 0 },
    { 0xc6fa5e15, 95 },
    { 0xce8a5e16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a5e1b, 76 },
    { 0xab9a5e1b, 81 },
    { 0, 0 },
    { 0xcfba5e1e, 90 },
    { 0x74ca5e1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a5e24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9a5e2d, 98 },
    { 0x291a5e2e, 17 },
    { 0x9a5e2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a5e35, 90 },
    { 0xc56a5e36, 61 },
    { 0x389a5e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532a5e3b, 52 },
    { 0, 0 },
    { 0x1d1a5e3d, 87 },
    { 0x414a5e3e, 33 },
    { 0, 0 },
    { 0xaf9a5e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5e46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a5e49, 23 },
    { 0, 0 },
    { 0x6c7a5e4b, 90 },
    { 0, 0 },
    { 0x70da5e4d, 90 },
    { 0, 0 },
    { 0x17ca5e4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ba5e52, 99 },
    { 0xf2aa5e53, 46 },
    { 0, 0 },
    { 0xd89a5e55, 55 },
    { 0xebaa5e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea5e5a, 95 },
    { 0x3c3a5e5b, 33 },
    { 0xe1ba5e5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa5e61, 95 },
    { 0xd53a5e62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba5e68, 90 },
    { 0x8f4a5e68, 61 },
    { 0xe1ca5e6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a5e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a5e71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dca5e74, 76 },
    { 0x86ca5e75, 87 },
    { 0xb94a5e76, 95 },
    { 0x27ca5e77, 61 },
    { 0xe6ba5e78, 90 },
    { 0x236a5e74, 34 },
    { 0, 0 },
    { 0x6d2a5e7b, 23 },
    { 0x3b4a5e7c, 90 },
    { 0xeb4a5e7d, 100 },
    { 0xddaa5e7e, 70 },
    { 0x6aca5e7f, 43 },
    { 0x5b0a5e7c, 76 },
    { 0, 0 },
    { 0xc96a5e82, 76 },
    { 0, 0 },
    { 0xdbfa5e84, 98 },
    { 0xe29a5e85, 33 },
    { 0xf23a5e86, 38 },
    { 0x6c8a5e84, 77 },
    { 0, 0 },
    { 0x8ffa5e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a5e8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486a5e9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0a5ea3, 46 },
    { 0x118a5ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a5ead, 91 },
    { 0x2e8a5eae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ba5eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37da5eb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ea5ebd, 97 },
    { 0xa63a5ebe, 23 },
    { 0xbb9a5ebf, 90 },
    { 0, 0 },
    { 0x801a5ec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a5ec5, 85 },
    { 0x6bfa5ec6, 61 },
    { 0, 0 },
    { 0x782a5ec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca5ece, 61 },
    { 0, 0 },
    { 0x28ba5ed0, 61 },
    { 0x8ba5ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda5ed7, 76 },
    { 0, 0 },
    { 0x4c9a5ed9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fa5edd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba5ee2, 93 },
    { 0x2da5ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca5ee7, 76 },
    { 0xf54a5ee7, 61 },
    { 0, 0 },
    { 0x2f0a5eea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340a5ef2, 33 },
    { 0xf65a5ef3, 34 },
    { 0xe8ea5ef3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa5efc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eba5eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5f05, 66 },
    { 0xde4a5f06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a5f0a, 90 },
    { 0x118a5f0b, 46 },
    { 0xf12a5f0a, 61 },
    { 0xab4a5f0d, 23 },
    { 0, 0 },
    { 0x5eea5f0f, 66 },
    { 0x87a5f10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea5f17, 61 },
    { 0xd44a5f18, 67 },
    { 0, 0 },
    { 0x590a5f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8a5f1e, 84 },
    { 0x423a5f1f, 90 },
    { 0xa83a5f1e, 95 },
    { 0xd37a5f1e, 81 },
    { 0xaa4a5f22, 68 },
    { 0x2fca5f22, 85 },
    { 0xa4ba5f22, 90 },
    { 0x1b4a5f1f, 90 },
    { 0x86aa5f26, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a5f29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eea5f2f, 90 },
    { 0xef1a5f30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6a5f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157a5f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a5f45, 90 },
    { 0x85da5f45, 55 },
    { 0, 0 },
    { 0x52ba5f48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475a5f51, 90 },
    { 0x31a5f51, 33 },
    { 0, 0 },
    { 0x3bfa5f54, 61 },
    { 0xfc1a5f55, 46 },
    { 0, 0 },
    { 0xa35a5f57, 99 },
    { 0x725a5f58, 98 },
    { 0x322a5f59, 23 },
    { 0xb89a5f58, 95 },
    { 0xb56a5f59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a5f5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28a5f64, 90 },
    { 0x21ea5f65, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9a5f71, 95 },
    { 0xb12a5f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa5f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aa5f7f, 23 },
    { 0, 0 },
    { 0x678a5f81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5f87, 61 },
    { 0x92aa5f87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fa5f92, 93 },
    { 0, 0 },
    { 0xa49a5f94, 98 },
    { 0, 0 },
    { 0x286a5f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4a5f9a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa5f9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4a5fa0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5fa5, 61 },
    { 0x74ca5fa6, 90 },
    { 0x90ea5fa7, 98 },
    { 0x5fea5fa6, 34 },
    { 0, 0 },
    { 0xd47a5faa, 33 },
    { 0xc05a5faa, 78 },
    { 0x2bfa5fac, 84 },
    { 0xd3a5fab, 67 },
    { 0, 0 },
    { 0x34fa5faf, 98 },
    { 0, 0 },
    { 0xdc3a5fb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789a5fb8, 52 },
    { 0xa83a5fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ca5fbf, 33 },
    { 0x4e8a5fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11a5fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760a5fc8, 90 },
    { 0xe3ea5fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547a5fcd, 46 },
    { 0x2f9a5fce, 98 },
    { 0, 0 },
    { 0xff8a5fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda5fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a5fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88aa5fdf, 98 },
    { 0x2a0a5fdf, 98 },
    { 0xb55a5fdf, 77 },
    { 0, 0 },
    { 0xfc6a5fe3, 23 },
    { 0xc2fa5fe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ba5feb, 95 },
    { 0x146a5fec, 99 },
    { 0x955a5fec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a5ff6, 93 },
    { 0, 0 },
    { 0x6f3a5ff8, 90 },
    { 0x8dfa5ff9, 76 },
    { 0xc18a5ffa, 93 },
    { 0x8f2a5ffa, 90 },
    { 0x27fa5ffc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11a6000, 95 },
    { 0, 0 },
    { 0xdd8a6002, 34 },
    { 0xf95a6002, 90 },
    { 0, 0 },
    { 0xb9da6005, 87 },
    { 0xd85a6005, 33 },
    { 0, 0 },
    { 0x9a6008, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03a600e, 87 },
    { 0xf2fa600f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114a6013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa6017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a601b, 33 },
    { 0x7ada601c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a6020, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a6023, 98 },
    { 0xfcca6023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2a602a, 98 },
    { 0x89ca602b, 61 },
    { 0x762a602a, 33 },
    { 0, 0 },
    { 0x424a602e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a6035, 84 },
    { 0x53da6036, 61 },
    { 0x6e0a6035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5a6044, 33 },
    { 0x451a6045, 96 },
    { 0x9aba6045, 85 },
    { 0, 0 },
    { 0xecfa6048, 33 },
    { 0, 0 },
    { 0xa6aa604a, 88 },
    { 0x489a604b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8a604e, 61 },
    { 0, 0 },
    { 0x52fa6050, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330a6054, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eda605a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa605d, 91 },
    { 0x430a605e, 65 },
    { 0, 0 },
    { 0xf6a6060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75a6065, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a6068, 93 },
    { 0xdcfa6069, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5a606d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bba6071, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaa6077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca607f, 90 },
    { 0x46fa6080, 95 },
    { 0x692a607f, 90 },
    { 0x524a6082, 76 },
    { 0xa23a6083, 33 },
    { 0x576a6083, 66 },
    { 0x605a6085, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97a6088, 61 },
    { 0x231a6089, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3a608f, 88 },
    { 0x28fa6090, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca60a4, 99 },
    { 0, 0 },
    { 0x3b9a60a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a60a9, 81 },
    { 0x93fa60a9, 76 },
    { 0, 0 },
    { 0xcd4a60ac, 90 },
    { 0xdb3a60ac, 34 },
    { 0, 0 },
    { 0x3aa60af, 98 },
    { 0x683a60b0, 98 },
    { 0x527a60af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0aa60b5, 61 },
    { 0, 0 },
    { 0x450a60b7, 90 },
    { 0x9d0a60b8, 90 },
    { 0x3afa60b7, 90 },
    { 0, 0 },
    { 0xb1da60bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a60c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ba60c3, 90 },
    { 0x9dfa60c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a60c8, 85 },
    { 0xc63a60c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a60cc, 61 },
    { 0, 0 },
    { 0xa7fa60ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a60d1, 52 },
    { 0, 0 },
    { 0x41aa60d3, 99 },
    { 0, 0 },
    { 0x18aa60d5, 34 },
    { 0x433a60d6, 23 },
    { 0xa3ba60d7, 36 },
    { 0xd3aa60d8, 95 },
    { 0xa9a60d7, 95 },
    { 0xe5fa60da, 33 },
    { 0x93ea60d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6a60df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79aa60e3, 94 },
    { 0xd41a60e3, 61 },
    { 0, 0 },
    { 0x8a0a60e6, 95 },
    { 0x811a60e7, 61 },
    { 0, 0 },
    { 0x1b8a60e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76da60ef, 23 },
    { 0x6f4a60ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x715a60f3, 95 },
    { 0x8fca60f4, 85 },
    { 0x445a60f4, 93 },
    { 0xbeda60f6, 98 },
    { 0, 0 },
    { 0xd4ea60f8, 88 },
    { 0, 0 },
    { 0x5eea60fa, 95 },
    { 0x6ea60fb, 90 },
    { 0x89da60fc, 85 },
    { 0, 0 },
    { 0x331a60fe, 23 },
    { 0x9b7a60ff, 90 },
    { 0, 0 },
    { 0x8daa6101, 99 },
    { 0xb01a6102, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70a610b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a6110, 95 },
    { 0xfe0a6110, 3 },
    { 0xcbba6112, 90 },
    { 0x4fba6113, 23 },
    { 0x3e1a6114, 46 },
    { 0x30a6115, 55 },
    { 0xbe9a6116, 98 },
    { 0x655a6115, 69 },
    { 0x148a6114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fa611d, 66 },
    { 0xea8a611e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a6121, 23 },
    { 0x827a6122, 33 },
    { 0x1f0a6121, 99 },
    { 0xceaa6124, 61 },
    { 0, 0 },
    { 0xecda6126, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a612f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673a6133, 91 },
    { 0, 0 },
    { 0xe19a6135, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dca6138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a613b, 61 },
    { 0x26a613c, 12 },
    { 0, 0 },
    { 0x72ca613e, 66 },
    { 0x1e1a613f, 97 },
    { 0, 0 },
    { 0x720a6141, 90 },
    { 0x5ba6142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a6146, 23 },
    { 0x65da6147, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a614c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa615a, 85 },
    { 0x689a615b, 95 },
    { 0, 0 },
    { 0x759a615d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6160, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa6166, 91 },
    { 0xceba6167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x884a616a, 34 },
    { 0, 0 },
    { 0x235a616c, 33 },
    { 0x594a616d, 91 },
    { 0xeba616e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a6173, 34 },
    { 0, 0 },
    { 0xd5ca6175, 55 },
    { 0xe35a6176, 55 },
    { 0x43fa6176, 98 },
    { 0x892a6177, 98 },
    { 0, 0 },
    { 0xa8a617a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da617e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fba6181, 98 },
    { 0xf35a6181, 46 },
    { 0xf34a6181, 90 },
    { 0x44a6184, 33 },
    { 0, 0 },
    { 0xc23a6186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdea6189, 61 },
    { 0x8f3a6189, 55 },
    { 0, 0 },
    { 0x2f8a618c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a618f, 23 },
    { 0xf54a6190, 90 },
    { 0x94fa6191, 80 },
    { 0x13ca6190, 17 },
    { 0, 0 },
    { 0x466a6194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166a619a, 40 },
    { 0, 0 },
    { 0x493a619c, 93 },
    { 0x80fa619d, 98 },
    { 0, 0 },
    { 0x938a619f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a61a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fa61a7, 66 },
    { 0, 0 },
    { 0x17da61a9, 46 },
    { 0xc55a61a9, 33 },
    { 0x86fa61ab, 91 },
    { 0x6a6a61ac, 90 },
    { 0x3daa61a9, 76 },
    { 0x136a61ae, 90 },
    { 0, 0 },
    { 0xcfca61b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a61ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a61c5, 66 },
    { 0x7b4a61c6, 76 },
    { 0x262a61c7, 43 },
    { 0xcc4a61c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x955a61cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cea61d0, 75 },
    { 0x668a61d0, 90 },
    { 0x9dea61d2, 93 },
    { 0x294a61d2, 61 },
    { 0x377a61d4, 87 },
    { 0xd08a61d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a61db, 98 },
    { 0, 0 },
    { 0x5d7a61dd, 76 },
    { 0xf93a61de, 23 },
    { 0x5a9a61df, 90 },
    { 0x989a61df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902a61e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa61eb, 40 },
    { 0xd05a61ec, 33 },
    { 0, 0 },
    { 0x631a61ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a61f6, 98 },
    { 0, 0 },
    { 0xeb6a61f8, 61 },
    { 0x418a61f9, 87 },
    { 0x63aa61f9, 61 },
    { 0x2baa61fa, 91 },
    { 0x220a61fb, 40 },
    { 0xeb1a61fc, 100 },
    { 0, 0 },
    { 0x4b8a61ff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fa6202, 90 },
    { 0xf34a6202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981a6207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59a620b, 85 },
    { 0x6e7a620b, 61 },
    { 0x6d5a620c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda6216, 80 },
    { 0x354a6216, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a621c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7a621f, 52 },
    { 0x53da6220, 95 },
    { 0x5a8a6221, 90 },
    { 0x773a6222, 61 },
    { 0x4aaa6222, 33 },
    { 0xbc1a6221, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bea6228, 80 },
    { 0xa44a6229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ba622e, 66 },
    { 0, 0 },
    { 0xfa5a6230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea6235, 33 },
    { 0x1eda6236, 87 },
    { 0, 0 },
    { 0xed3a6238, 88 },
    { 0, 0 },
    { 0xb41a623a, 68 },
    { 0xbffa623a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fa623e, 95 },
    { 0xbf9a623e, 43 },
    { 0xd94a623e, 61 },
    { 0x13aa6241, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ca6246, 76 },
    { 0xafba6246, 90 },
    { 0x8cca6246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aa624b, 23 },
    { 0x60aa624b, 61 },
    { 0, 0 },
    { 0x66a624e, 34 },
    { 0x5b8a624f, 61 },
    { 0x809a624e, 23 },
    { 0x23aa6251, 90 },
    { 0x899a6252, 66 },
    { 0, 0 },
    { 0xe6ca6254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9a6257, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77a625c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a625f, 90 },
    { 0x50ca625f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7da6263, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952a6268, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a626b, 99 },
    { 0, 0 },
    { 0xb9da626d, 33 },
    { 0xf94a626e, 23 },
    { 0xf19a626d, 95 },
    { 0x6b6a6270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898a6274, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a6278, 33 },
    { 0, 0 },
    { 0x269a627a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a627e, 98 },
    { 0x346a627f, 67 },
    { 0x16ba6280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a628a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa6293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a629a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457a629e, 43 },
    { 0x967a629f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a62b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a62b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea62b9, 23 },
    { 0x9e9a62b9, 33 },
    { 0x7b1a62b9, 23 },
    { 0, 0 },
    { 0xee3a62bd, 98 },
    { 0, 0 },
    { 0x1d1a62bf, 76 },
    { 0x4d9a62bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4a62c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a62c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5a62cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fa62cf, 46 },
    { 0, 0 },
    { 0xe1ca62d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1a62d4, 61 },
    { 0, 0 },
    { 0x21ba62d6, 61 },
    { 0x74ea62d7, 33 },
    { 0xa0ea62d7, 98 },
    { 0x75fa62d6, 55 },
    { 0xa3da62da, 76 },
    { 0, 0 },
    { 0xabea62dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a62df, 98 },
    { 0x170a62e0, 68 },
    { 0xa9da62e1, 87 },
    { 0x2e3a62e0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65a62ea, 61 },
    { 0xe52a62ea, 23 },
    { 0, 0 },
    { 0x3f3a62ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68a62f7, 95 },
    { 0x35ea62f8, 98 },
    { 0x4e3a62f8, 61 },
    { 0x1f2a62f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a6303, 90 },
    { 0x89fa6304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaa6307, 87 },
    { 0xcf6a6307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a6312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6a631a, 95 },
    { 0xb69a631b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338a6320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a632b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a632e, 90 },
    { 0xb7ca632f, 23 },
    { 0x2da632e, 43 },
    { 0xb02a6331, 33 },
    { 0x4c5a6331, 34 },
    { 0x8a9a6333, 33 },
    { 0, 0 },
    { 0xdf3a6335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a633e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4a6342, 90 },
    { 0, 0 },
    { 0x8d6a6344, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eba6348, 61 },
    { 0, 0 },
    { 0xb04a634a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aa6352, 99 },
    { 0x857a6353, 78 },
    { 0x6a1a6353, 90 },
    { 0, 0 },
    { 0x494a6356, 53 },
    { 0xb0da6357, 61 },
    { 0x628a6357, 46 },
    { 0x8ba6359, 85 },
    { 0, 0 },
    { 0xcfea635b, 23 },
    { 0xbcaa635c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdba6360, 84 },
    { 0xfb8a6361, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105a6365, 23 },
    { 0xa50a6366, 61 },
    { 0x74da6367, 61 },
    { 0xce5a6365, 43 },
    { 0x258a6369, 90 },
    { 0x56ea636a, 52 },
    { 0xe2ca636b, 61 },
    { 0x5f4a636c, 98 },
    { 0x7f3a636d, 90 },
    { 0xf85a6366, 55 },
    { 0x155a636c, 23 },
    { 0xff1a6370, 88 },
    { 0x310a6371, 90 },
    { 0x747a6369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba6375, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa6379, 90 },
    { 0x73ca637a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ba637d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aa6382, 67 },
    { 0xba0a6382, 61 },
    { 0x757a6384, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ba638a, 17 },
    { 0xf10a638b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa638f, 61 },
    { 0, 0 },
    { 0xda6a6391, 90 },
    { 0, 0 },
    { 0x1f4a6393, 91 },
    { 0x83fa6394, 56 },
    { 0xfaea6395, 95 },
    { 0x9a6a6396, 34 },
    { 0x7bfa6397, 76 },
    { 0x76ca6398, 95 },
    { 0x81fa6398, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da639e, 23 },
    { 0xa04a639f, 65 },
    { 0xe9ca639f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a63a3, 81 },
    { 0, 0 },
    { 0xa2fa63a5, 76 },
    { 0x2aa63a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa63ac, 66 },
    { 0xb0fa63ac, 55 },
    { 0xe40a63ac, 43 },
    { 0xf3ba63ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba63b3, 90 },
    { 0x4ffa63b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a63c2, 87 },
    { 0x4e9a63c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ca63c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x541a63ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ca63ce, 95 },
    { 0, 0 },
    { 0x1f2a63d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a63d4, 64 },
    { 0, 0 },
    { 0xd1da63d6, 95 },
    { 0x4efa63d7, 43 },
    { 0, 0 },
    { 0xe35a63d9, 98 },
    { 0, 0 },
    { 0xe1da63db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a63df, 98 },
    { 0x21da63df, 100 },
    { 0x6a2a63e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a63eb, 17 },
    { 0x8ba63ec, 61 },
    { 0x3d1a63ed, 33 },
    { 0xbd0a63ee, 90 },
    { 0xc79a63ed, 61 },
    { 0xe56a63eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a63f3, 85 },
    { 0xee9a63f3, 90 },
    { 0x5dba63f5, 90 },
    { 0, 0 },
    { 0x7bca63f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a63fd, 61 },
    { 0xd1aa63fd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a6402, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709a6407, 95 },
    { 0xdd7a6407, 61 },
    { 0x6c4a6409, 90 },
    { 0x8a3a6409, 90 },
    { 0, 0 },
    { 0x750a640c, 91 },
    { 0x82fa640c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437a6413, 61 },
    { 0xb1a6413, 67 },
    { 0, 0 },
    { 0x2bea6416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cda6419, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa641c, 23 },
    { 0xddba641c, 65 },
    { 0x908a641e, 76 },
    { 0x693a641d, 85 },
    { 0x569a641f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeca6423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca6426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07a642a, 90 },
    { 0x145a642b, 61 },
    { 0x955a642b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a642f, 99 },
    { 0x324a6430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba6435, 33 },
    { 0, 0 },
    { 0x7b4a6437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a643a, 90 },
    { 0, 0 },
    { 0x323a643c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70a644a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7a6450, 55 },
    { 0, 0 },
    { 0xf52a6452, 98 },
    { 0x94a6453, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa645a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5a645d, 95 },
    { 0, 0 },
    { 0xdada645f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a646c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa6472, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a6477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2a647b, 33 },
    { 0xd98a647b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a6480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca6484, 61 },
    { 0, 0 },
    { 0x757a6486, 76 },
    { 0x466a6487, 90 },
    { 0x239a6487, 40 },
    { 0x65a6486, 34 },
    { 0, 0 },
    { 0xfe4a648b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318a648f, 98 },
    { 0xc45a6490, 88 },
    { 0xf57a6491, 33 },
    { 0x4fba648f, 98 },
    { 0xffba6493, 33 },
    { 0x2aa6493, 33 },
    { 0x22ea6494, 64 },
    { 0x6fba6496, 23 },
    { 0xfdda6492, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75a649a, 94 },
    { 0, 0 },
    { 0x5c7a649c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a649f, 90 },
    { 0, 0 },
    { 0xd4ca64a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bea64a9, 93 },
    { 0x151a64a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680a64ae, 55 },
    { 0xaa64af, 98 },
    { 0x3a1a64af, 90 },
    { 0x90ba64b1, 33 },
    { 0x2fba64af, 66 },
    { 0xe0ca64b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda64b6, 23 },
    { 0x69a64b7, 85 },
    { 0x505a64b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a64bd, 76 },
    { 0, 0 },
    { 0x2e0a64bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a64c6, 95 },
    { 0x12aa64c6, 33 },
    { 0, 0 },
    { 0xb9ba64c9, 61 },
    { 0, 0 },
    { 0x7eca64cb, 95 },
    { 0, 0 },
    { 0xe21a64cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efa64d0, 61 },
    { 0x38ba64d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a64d5, 91 },
    { 0, 0 },
    { 0x8bfa64d7, 33 },
    { 0, 0 },
    { 0x9a9a64d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca64dc, 98 },
    { 0x190a64dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baa64e2, 90 },
    { 0x63ca64e3, 90 },
    { 0xea7a64e4, 34 },
    { 0x13a64e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a64e8, 90 },
    { 0x5bea64e9, 76 },
    { 0x4caa64ea, 23 },
    { 0x177a64eb, 77 },
    { 0x875a64ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5a64f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba64f9, 98 },
    { 0, 0 },
    { 0x58ca64fb, 23 },
    { 0xbe6a64fc, 100 },
    { 0x409a64fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa6502, 61 },
    { 0xa68a6502, 98 },
    { 0x166a6503, 61 },
    { 0x41a6502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba6508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa650c, 95 },
    { 0x304a650d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bda6511, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea651e, 23 },
    { 0xb6a651f, 34 },
    { 0, 0 },
    { 0x4c9a6521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a6529, 33 },
    { 0x9cca652a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a652d, 61 },
    { 0, 0 },
    { 0x6efa652f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9a6534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342a653a, 100 },
    { 0, 0 },
    { 0x708a653c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a6544, 95 },
    { 0xb76a6545, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ea6548, 90 },
    { 0, 0 },
    { 0xe8fa654a, 78 },
    { 0x396a654a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a654e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda6552, 98 },
    { 0, 0 },
    { 0xf35a6554, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda655e, 23 },
    { 0x2b8a655f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da6562, 95 },
    { 0xfca6562, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5a6566, 90 },
    { 0x1a9a6566, 23 },
    { 0x219a6568, 76 },
    { 0x48ea6569, 76 },
    { 0x2a0a6566, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca6571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20a6574, 76 },
    { 0x2e0a6575, 43 },
    { 0xcfda6576, 55 },
    { 0xa45a6575, 84 },
    { 0, 0 },
    { 0xce4a6579, 95 },
    { 0, 0 },
    { 0x9a0a657b, 90 },
    { 0, 0 },
    { 0xbb9a657d, 90 },
    { 0x581a657e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa6582, 77 },
    { 0x98a6583, 90 },
    { 0xac7a6584, 90 },
    { 0x305a6582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a658d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36a6594, 91 },
    { 0xee5a6595, 90 },
    { 0x644a6595, 34 },
    { 0x62a6597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a659c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98a65a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a65a4, 75 },
    { 0, 0 },
    { 0x68da65a6, 95 },
    { 0, 0 },
    { 0x955a65a8, 33 },
    { 0, 0 },
    { 0xd58a65aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751a65c1, 33 },
    { 0, 0 },
    { 0xe88a65c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ca65c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a65ca, 90 },
    { 0, 0 },
    { 0x354a65cc, 61 },
    { 0xd4ea65cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda65d0, 98 },
    { 0x382a65d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10a65d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a65d7, 33 },
    { 0x3d5a65d8, 61 },
    { 0x8c7a65d8, 90 },
    { 0xe34a65da, 76 },
    { 0xa77a65da, 96 },
    { 0x29da65db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa65e0, 43 },
    { 0, 0 },
    { 0x255a65e2, 98 },
    { 0x6d3a65e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a65e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a65ed, 23 },
    { 0x7fea65ed, 61 },
    { 0x515a65ed, 55 },
    { 0xab1a65ee, 76 },
    { 0x4d4a65ed, 34 },
    { 0x56ba65f2, 98 },
    { 0xed9a65f3, 23 },
    { 0xc0ea65f2, 76 },
    { 0x85ba65ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaea65f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfa65fc, 53 },
    { 0, 0 },
    { 0xb51a65fe, 85 },
    { 0xf4ca65ff, 33 },
    { 0, 0 },
    { 0xc31a6601, 43 },
    { 0, 0 },
    { 0x9ffa6603, 33 },
    { 0xac6a6604, 43 },
    { 0, 0 },
    { 0x615a6606, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40a6609, 98 },
    { 0, 0 },
    { 0x121a660b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a6612, 85 },
    { 0, 0 },
    { 0xf71a6614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6a6618, 61 },
    { 0, 0 },
    { 0xb68a661a, 95 },
    { 0, 0 },
    { 0x3a5a661c, 52 },
    { 0xf81a661d, 97 },
    { 0xc3fa661c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69a6624, 90 },
    { 0xba6625, 96 },
    { 0x599a6625, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840a662a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89da662e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ea6634, 61 },
    { 0, 0 },
    { 0x934a6636, 88 },
    { 0x7daa6637, 69 },
    { 0x60ca6637, 76 },
    { 0, 0 },
    { 0xba663a, 23 },
    { 0, 0 },
    { 0xa5da663c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19a6640, 76 },
    { 0, 0 },
    { 0x20aa6642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x841a6645, 33 },
    { 0x2d1a6646, 90 },
    { 0xfeca6645, 83 },
    { 0x680a6648, 33 },
    { 0x9d5a6649, 98 },
    { 0x958a664a, 33 },
    { 0, 0 },
    { 0x7aaa664c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98a6651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42a6655, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a665c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95a6661, 61 },
    { 0, 0 },
    { 0x548a6663, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda6669, 90 },
    { 0x39da6669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a666f, 68 },
    { 0x97fa6670, 84 },
    { 0xf2ea6671, 61 },
    { 0, 0 },
    { 0xb89a6673, 76 },
    { 0xf30a6674, 33 },
    { 0xb2fa6675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194a6679, 17 },
    { 0xdd3a667a, 98 },
    { 0x7e8a667b, 33 },
    { 0, 0 },
    { 0xe3aa667d, 61 },
    { 0, 0 },
    { 0xb4ba667f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6a6683, 84 },
    { 0, 0 },
    { 0x4cea6685, 61 },
    { 0xd38a6685, 61 },
    { 0, 0 },
    { 0x57a6688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a668e, 76 },
    { 0x107a668e, 84 },
    { 0xe6da668e, 61 },
    { 0xe7ba6691, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a6694, 55 },
    { 0x31a6695, 98 },
    { 0x6e3a6696, 95 },
    { 0x286a6697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894a669c, 78 },
    { 0x12ca669c, 78 },
    { 0xef9a669e, 93 },
    { 0xa64a669f, 55 },
    { 0xe70a66a0, 85 },
    { 0xde3a669e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a66a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x542a66ab, 97 },
    { 0x37aa66ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa66b0, 99 },
    { 0x905a66b0, 90 },
    { 0xe95a66b1, 87 },
    { 0, 0 },
    { 0xd28a66b4, 90 },
    { 0xdc1a66b4, 95 },
    { 0, 0 },
    { 0xe91a66b7, 95 },
    { 0xee0a66b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264a66be, 61 },
    { 0x1d5a66bf, 66 },
    { 0, 0 },
    { 0xcd8a66c1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8a66c5, 66 },
    { 0xdcda66c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a66c9, 17 },
    { 0x2a2a66c9, 93 },
    { 0x47da66cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca66ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea66d2, 61 },
    { 0x7f9a66d2, 61 },
    { 0xe4ca66d4, 90 },
    { 0x182a66d5, 99 },
    { 0, 0 },
    { 0x48fa66d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcca66dd, 43 },
    { 0xa42a66dd, 95 },
    { 0x5eda66dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x925a66e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecea66e9, 61 },
    { 0x9aa66ea, 61 },
    { 0xcf2a66e9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a66f1, 95 },
    { 0, 0 },
    { 0x42ca66f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a66fa, 23 },
    { 0, 0 },
    { 0xae8a66fc, 23 },
    { 0xffaa66fc, 98 },
    { 0xefba66fe, 84 },
    { 0, 0 },
    { 0x30aa6700, 95 },
    { 0x8cfa6701, 61 },
    { 0x4f2a6702, 95 },
    { 0x502a6703, 87 },
    { 0x31a6701, 34 },
    { 0xf43a6701, 90 },
    { 0x2f9a6706, 81 },
    { 0x60fa6706, 90 },
    { 0xe2fa6708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a670b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea670f, 98 },
    { 0, 0 },
    { 0x463a6711, 68 },
    { 0xe29a6711, 17 },
    { 0x412a6713, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6a6716, 61 },
    { 0x29ca6717, 17 },
    { 0, 0 },
    { 0x827a6719, 52 },
    { 0, 0 },
    { 0x131a671b, 33 },
    { 0xbaaa671c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965a6721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea6727, 90 },
    { 0xd60a6728, 23 },
    { 0x100a6728, 91 },
    { 0x496a672a, 23 },
    { 0xcdea672b, 76 },
    { 0, 0 },
    { 0x46ca672d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3a6732, 61 },
    { 0, 0 },
    { 0xfa2a6734, 66 },
    { 0x343a6735, 95 },
    { 0xb20a6736, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a673b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a673f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a6743, 55 },
    { 0, 0 },
    { 0xb82a6745, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a6748, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fa674b, 23 },
    { 0xbaa674c, 95 },
    { 0, 0 },
    { 0x121a674e, 93 },
    { 0, 0 },
    { 0xbdda6750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcba6758, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4a675f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6a6762, 85 },
    { 0x63da6763, 66 },
    { 0xb5ea6764, 61 },
    { 0xaefa6763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0a6769, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fa676d, 90 },
    { 0x2da676e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47da6772, 88 },
    { 0, 0 },
    { 0x981a6774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a6777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32da677c, 23 },
    { 0x8dfa677d, 93 },
    { 0x338a677e, 33 },
    { 0xf7fa677e, 95 },
    { 0xc62a677e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0a6783, 61 },
    { 0xac6a6783, 67 },
    { 0xc6a6785, 61 },
    { 0, 0 },
    { 0x649a6787, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9a678b, 76 },
    { 0, 0 },
    { 0x5daa678d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea6790, 93 },
    { 0xf12a6791, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21a6794, 93 },
    { 0xfc9a6795, 33 },
    { 0xc4aa6795, 60 },
    { 0x6bea6795, 90 },
    { 0x993a6797, 46 },
    { 0xa94a6799, 97 },
    { 0x5fca679a, 90 },
    { 0, 0 },
    { 0xe8aa679c, 61 },
    { 0, 0 },
    { 0xb78a679e, 52 },
    { 0, 0 },
    { 0x4f7a67a0, 43 },
    { 0x49a67a1, 33 },
    { 0xb60a67a2, 76 },
    { 0x76a67a3, 61 },
    { 0x2b6a67a4, 92 },
    { 0, 0 },
    { 0x863a67a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fa67ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a67b1, 61 },
    { 0x715a67b2, 76 },
    { 0x899a67b2, 33 },
    { 0x7bba67b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaa67b7, 95 },
    { 0x803a67b8, 33 },
    { 0x110a67b8, 33 },
    { 0, 0 },
    { 0x57aa67bb, 76 },
    { 0xd05a67bb, 52 },
    { 0x3f9a67bd, 33 },
    { 0xc72a67be, 95 },
    { 0x6da67bd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a67c2, 33 },
    { 0x4d6a67c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a67cb, 90 },
    { 0xaa1a67cc, 95 },
    { 0, 0 },
    { 0x28a67ce, 91 },
    { 0x141a67cf, 95 },
    { 0x8a6a67d0, 95 },
    { 0xb5aa67d0, 34 },
    { 0x869a67d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7a67d7, 46 },
    { 0xac8a67d8, 90 },
    { 0xca0a67d9, 23 },
    { 0x6eaa67da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219a67df, 77 },
    { 0x97ba67e0, 98 },
    { 0xfc2a67e1, 43 },
    { 0xef6a67df, 97 },
    { 0x421a67e0, 61 },
    { 0xf29a67e1, 82 },
    { 0x71ea67e5, 33 },
    { 0x1bfa67e3, 90 },
    { 0, 0 },
    { 0xbb0a67e8, 61 },
    { 0x51aa67e9, 33 },
    { 0x91da67e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7a67ef, 66 },
    { 0x37a67ef, 87 },
    { 0xfe2a67f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efa67f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6802, 99 },
    { 0, 0 },
    { 0x858a6804, 61 },
    { 0x137a6805, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a680b, 61 },
    { 0, 0 },
    { 0xb22a680d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x747a6810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6816, 46 },
    { 0x67ca6816, 99 },
    { 0xc9ca6818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a681d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82a6829, 95 },
    { 0, 0 },
    { 0x766a682b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9da682e, 61 },
    { 0xa6aa682f, 33 },
    { 0xc6da6830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9a6833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7a6838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceea683e, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ea6844, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fa6849, 76 },
    { 0x946a684a, 34 },
    { 0x906a684a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa684f, 43 },
    { 0xd9aa684f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a6858, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4a685b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cea6860, 90 },
    { 0, 0 },
    { 0x70ba6862, 76 },
    { 0x2f2a6863, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba6869, 95 },
    { 0xa4ba6869, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ba686e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5a6872, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4a6876, 76 },
    { 0, 0 },
    { 0x674a6878, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdda687b, 84 },
    { 0x302a687c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x704a687f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ca6886, 55 },
    { 0, 0 },
    { 0xa1ea6888, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca688c, 90 },
    { 0x5eba688c, 46 },
    { 0x808a688e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722a6893, 90 },
    { 0, 0 },
    { 0x8f6a6895, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ba689f, 23 },
    { 0, 0 },
    { 0xc92a68a1, 77 },
    { 0x76da68a2, 90 },
    { 0xe87a68a3, 61 },
    { 0x2fca68a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601a68af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a68b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392a68b6, 61 },
    { 0xfea68b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a68ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a68c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a68c7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a68cc, 33 },
    { 0, 0 },
    { 0x970a68ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0a68d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba68d6, 98 },
    { 0x31ca68d7, 76 },
    { 0x5d9a68d7, 90 },
    { 0, 0 },
    { 0x375a68da, 61 },
    { 0xd9da68db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806a68df, 98 },
    { 0, 0 },
    { 0xde9a68e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea68e4, 55 },
    { 0xdeba68e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13a68eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aa68fc, 95 },
    { 0xa41a68fd, 99 },
    { 0, 0 },
    { 0x6fba68ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea6902, 80 },
    { 0x2b0a6903, 68 },
    { 0xe7fa6903, 43 },
    { 0xb83a6904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6908, 67 },
    { 0x1c1a6909, 95 },
    { 0xfb6a690a, 90 },
    { 0xaf4a690b, 98 },
    { 0xbaba690a, 99 },
    { 0xa1a6909, 88 },
    { 0x9aaa690e, 23 },
    { 0x4bfa690f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba6912, 38 },
    { 0xddea6913, 100 },
    { 0, 0 },
    { 0x3eba6915, 67 },
    { 0xad1a6916, 33 },
    { 0x7d3a6915, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da6922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53a6929, 43 },
    { 0xdd6a692a, 90 },
    { 0xa8ba692b, 95 },
    { 0x123a692c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5a6930, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a6933, 33 },
    { 0, 0 },
    { 0x2a3a6935, 66 },
    { 0x1fda6936, 23 },
    { 0, 0 },
    { 0xc1ea6938, 98 },
    { 0x8bba6938, 34 },
    { 0xacda693a, 44 },
    { 0x2c1a693b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a6945, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa694a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1a694f, 61 },
    { 0xc5da6950, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaa6958, 33 },
    { 0x7cda6959, 95 },
    { 0xf80a695a, 61 },
    { 0xe5ba6958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ba695f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aa6964, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da6967, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca696b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a696e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2a6971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaba6979, 55 },
    { 0xff3a697a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba697e, 90 },
    { 0xeb4a697f, 52 },
    { 0xb59a6980, 76 },
    { 0xa11a6981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5a6984, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70a6989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9a698d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaa6993, 90 },
    { 0x41ea6993, 33 },
    { 0xacea6995, 33 },
    { 0xc8fa6995, 76 },
    { 0x752a6997, 23 },
    { 0xe45a6998, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2a699f, 88 },
    { 0x67ea69a0, 91 },
    { 0, 0 },
    { 0x4d7a69a2, 75 },
    { 0, 0 },
    { 0x484a69a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea69a7, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a69ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391a69b4, 46 },
    { 0x55fa69b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31a69b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2a69bb, 33 },
    { 0x751a69bc, 43 },
    { 0xa69a69bd, 34 },
    { 0xee2a69bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aa69c5, 95 },
    { 0xe2a69c6, 90 },
    { 0x344a69c5, 80 },
    { 0x625a69c8, 43 },
    { 0x120a69c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a69d0, 90 },
    { 0x193a69d1, 23 },
    { 0xb99a69d0, 61 },
    { 0xb7da69d3, 91 },
    { 0xd1ca69d4, 33 },
    { 0x14ba69d5, 67 },
    { 0xfc2a69d2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a69da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a69df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6a69e7, 34 },
    { 0, 0 },
    { 0xec4a69e9, 95 },
    { 0, 0 },
    { 0x9e6a69eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2a69f0, 100 },
    { 0x313a69f1, 55 },
    { 0xc7da69f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa69f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daa69fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76a69ff, 61 },
    { 0x5f5a6a00, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504a6a06, 67 },
    { 0xadfa6a07, 89 },
    { 0x174a6a08, 77 },
    { 0xd98a6a07, 33 },
    { 0xaa2a6a0a, 98 },
    { 0x788a6a09, 33 },
    { 0xf7a6a0c, 98 },
    { 0xef9a6a0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba6a10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a6a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caa6a19, 33 },
    { 0, 0 },
    { 0x2c4a6a1b, 78 },
    { 0xde4a6a1b, 61 },
    { 0x92aa6a1b, 78 },
    { 0x4fca6a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3a6a23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35a6a2c, 34 },
    { 0xda2a6a2d, 61 },
    { 0xd7a6a2e, 81 },
    { 0x216a6a2f, 34 },
    { 0x1faa6a2f, 98 },
    { 0x176a6a31, 77 },
    { 0xfe5a6a2c, 93 },
    { 0xb96a6a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620a6a37, 23 },
    { 0x940a6a38, 46 },
    { 0xef5a6a38, 98 },
    { 0x9a0a6a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba6a40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a6a43, 68 },
    { 0x1f7a6a44, 23 },
    { 0xd1fa6a45, 34 },
    { 0xb71a6a44, 61 },
    { 0x682a6a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fa6a4b, 61 },
    { 0x805a6a4c, 52 },
    { 0x941a6a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a6a52, 72 },
    { 0, 0 },
    { 0xc74a6a54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81a6a5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3da6a65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a6a69, 91 },
    { 0x405a6a6a, 55 },
    { 0xd7a6a6b, 98 },
    { 0x8bca6a69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a6a70, 43 },
    { 0x3caa6a71, 81 },
    { 0xebca6a72, 43 },
    { 0xd6ea6a73, 61 },
    { 0x1d5a6a73, 33 },
    { 0x94ca6a75, 90 },
    { 0, 0 },
    { 0xb7fa6a77, 33 },
    { 0xe26a6a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a6a7b, 38 },
    { 0xcdba6a7c, 61 },
    { 0x8daa6a7c, 33 },
    { 0x645a6a7e, 61 },
    { 0x81ea6a7f, 101 },
    { 0xb7ea6a7f, 75 },
    { 0xe20a6a7e, 34 },
    { 0xd1fa6a7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a6a88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1da6a8e, 90 },
    { 0x6a4a6a8e, 23 },
    { 0, 0 },
    { 0xdcca6a91, 23 },
    { 0x667a6a92, 76 },
    { 0x51ba6a93, 58 },
    { 0x2a3a6a92, 43 },
    { 0, 0 },
    { 0xd5a6a96, 38 },
    { 0xa02a6a97, 95 },
    { 0, 0 },
    { 0xccaa6a99, 78 },
    { 0, 0 },
    { 0x25ea6a9b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246a6aa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba6aa7, 43 },
    { 0, 0 },
    { 0x1d9a6aa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada6aad, 84 },
    { 0, 0 },
    { 0x5eaa6aaf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5a6ab6, 76 },
    { 0, 0 },
    { 0x369a6ab8, 76 },
    { 0, 0 },
    { 0x4dca6aba, 33 },
    { 0x36ca6abb, 91 },
    { 0x658a6abc, 76 },
    { 0xb4ca6abd, 66 },
    { 0x3aca6abc, 85 },
    { 0xd9ba6abd, 78 },
    { 0x4b2a6abd, 78 },
    { 0x710a6abd, 33 },
    { 0x46ca6abc, 64 },
    { 0, 0 },
    { 0x8f2a6ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x895a6ac7, 55 },
    { 0, 0 },
    { 0xb20a6ac9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3a6ad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa6ad5, 23 },
    { 0x578a6ad5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a6add, 91 },
    { 0, 0 },
    { 0x9d5a6adf, 61 },
    { 0xa40a6ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7a6ae3, 99 },
    { 0x36ca6ae4, 55 },
    { 0x961a6ae4, 95 },
    { 0, 0 },
    { 0x366a6ae7, 90 },
    { 0xc60a6ae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a6aed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5a6af1, 99 },
    { 0, 0 },
    { 0xf16a6af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56a6af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a6afb, 90 },
    { 0, 0 },
    { 0x247a6afd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a6b01, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6b07, 52 },
    { 0xb4fa6b07, 95 },
    { 0, 0 },
    { 0x42da6b0a, 46 },
    { 0, 0 },
    { 0x1a6a6b0c, 90 },
    { 0, 0 },
    { 0xa4da6b0e, 61 },
    { 0, 0 },
    { 0x9da6b10, 66 },
    { 0x519a6b11, 97 },
    { 0xae7a6b10, 67 },
    { 0x680a6b13, 65 },
    { 0xec2a6b10, 33 },
    { 0, 0 },
    { 0x44ca6b16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a6b19, 95 },
    { 0, 0 },
    { 0x17ba6b1b, 95 },
    { 0xe5ca6b1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775a6b2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18a6b2f, 90 },
    { 0x320a6b30, 23 },
    { 0xa1aa6b30, 98 },
    { 0x1a1a6b32, 66 },
    { 0x588a6b30, 23 },
    { 0xd8fa6b33, 98 },
    { 0x2d7a6b35, 23 },
    { 0x33fa6b30, 43 },
    { 0xfe5a6b37, 66 },
    { 0x416a6b35, 93 },
    { 0x5c6a6b39, 84 },
    { 0, 0 },
    { 0x63a6b3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa6b41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7a6b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74a6b49, 61 },
    { 0x7e7a6b4a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7a6b4d, 55 },
    { 0, 0 },
    { 0x77fa6b4f, 98 },
    { 0xd3a6b50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a6b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8a6b57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fa6b5d, 90 },
    { 0x332a6b5e, 98 },
    { 0xc0aa6b5f, 33 },
    { 0x411a6b5d, 66 },
    { 0xe86a6b5d, 43 },
    { 0x8d9a6b62, 61 },
    { 0xe4fa6b5e, 80 },
    { 0x1e7a6b64, 90 },
    { 0xbb1a6b64, 23 },
    { 0xcf1a6b64, 98 },
    { 0x338a6b67, 66 },
    { 0xfca6b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ea6b6b, 61 },
    { 0, 0 },
    { 0xeea6b6d, 93 },
    { 0x17ba6b6e, 90 },
    { 0, 0 },
    { 0x208a6b70, 61 },
    { 0x102a6b71, 90 },
    { 0, 0 },
    { 0xc3ca6b73, 76 },
    { 0x39ba6b73, 33 },
    { 0, 0 },
    { 0x756a6b76, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a6b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4a6b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547a6b8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ea6b8f, 95 },
    { 0xbe8a6b90, 78 },
    { 0xf00a6b91, 55 },
    { 0xeaca6b92, 52 },
    { 0xca3a6b93, 19 },
    { 0x277a6b94, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea6b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402a6b9d, 90 },
    { 0xf32a6b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5a6ba2, 43 },
    { 0, 0 },
    { 0xe64a6ba4, 95 },
    { 0x6efa6ba5, 98 },
    { 0xfaca6ba5, 33 },
    { 0x197a6ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a6bab, 98 },
    { 0x65a6bac, 100 },
    { 0x5c2a6bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a6bb0, 84 },
    { 0xc7aa6bb0, 69 },
    { 0x9baa6bb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a6bb8, 87 },
    { 0, 0 },
    { 0x7e9a6bba, 77 },
    { 0xaefa6bba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a6bbf, 98 },
    { 0x309a6bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a6bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba6bd0, 61 },
    { 0x664a6bd0, 76 },
    { 0xd8da6bd0, 90 },
    { 0x248a6bd1, 90 },
    { 0xe42a6bd4, 80 },
    { 0x23da6bd5, 61 },
    { 0x695a6bd6, 43 },
    { 0xdfba6bd7, 23 },
    { 0xf41a6bd7, 95 },
    { 0x7dba6bd7, 33 },
    { 0xb11a6bd8, 100 },
    { 0xd28a6bdb, 66 },
    { 0x666a6bdc, 49 },
    { 0, 0 },
    { 0xbeba6bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca6be1, 90 },
    { 0, 0 },
    { 0x719a6be3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fa6bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea6bed, 36 },
    { 0, 0 },
    { 0x4cea6bef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea6bf3, 90 },
    { 0x5eaa6bf3, 67 },
    { 0xff5a6bf4, 33 },
    { 0, 0 },
    { 0x1a2a6bf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a6bfd, 33 },
    { 0x67da6bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea6c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda6c0a, 33 },
    { 0x4cba6c0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4a6c0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ca6c19, 76 },
    { 0xa71a6c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ca6c1d, 33 },
    { 0, 0 },
    { 0xfe4a6c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffa6c22, 61 },
    { 0x973a6c23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a6c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9a6c2b, 90 },
    { 0x9d9a6c2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995a6c31, 95 },
    { 0, 0 },
    { 0x62ba6c33, 95 },
    { 0xc6aa6c34, 76 },
    { 0x6b1a6c35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a6c3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca6c3e, 76 },
    { 0x7caa6c3f, 91 },
    { 0x5ada6c40, 90 },
    { 0x9c3a6c41, 61 },
    { 0xbeca6c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea6c47, 76 },
    { 0, 0 },
    { 0x684a6c49, 33 },
    { 0, 0 },
    { 0x854a6c4b, 95 },
    { 0x237a6c4c, 77 },
    { 0x3c7a6c4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77a6c54, 99 },
    { 0, 0 },
    { 0x64ea6c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a6c5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffa6c61, 55 },
    { 0x58a6c62, 77 },
    { 0xde0a6c63, 33 },
    { 0x801a6c64, 78 },
    { 0x3a7a6c65, 55 },
    { 0x2b0a6c66, 85 },
    { 0xa90a6c67, 87 },
    { 0x19a6c67, 90 },
    { 0, 0 },
    { 0xc10a6c6a, 95 },
    { 0xb1a6c6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea6c70, 46 },
    { 0x7a5a6c70, 90 },
    { 0, 0 },
    { 0x2dfa6c73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a6c76, 55 },
    { 0, 0 },
    { 0xb2aa6c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900a6c7d, 32 },
    { 0xbbda6c7e, 33 },
    { 0x355a6c7f, 84 },
    { 0xb9ba6c80, 93 },
    { 0xa7ea6c80, 76 },
    { 0x8eea6c81, 67 },
    { 0xac5a6c83, 55 },
    { 0xfefa6c81, 76 },
    { 0x9a6c80, 46 },
    { 0xbe4a6c82, 95 },
    { 0, 0 },
    { 0xca2a6c88, 33 },
    { 0, 0 },
    { 0xf24a6c8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a6c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3a6c97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dca6c9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56a6c9f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365a6ca9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a6caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x182a6cb2, 33 },
    { 0x2ffa6cb2, 98 },
    { 0xda6cb4, 76 },
    { 0xea5a6cb5, 33 },
    { 0xf8ea6cb3, 55 },
    { 0xd6a6cb3, 23 },
    { 0xf48a6cb8, 17 },
    { 0, 0 },
    { 0x759a6cba, 65 },
    { 0xc0ba6cbb, 98 },
    { 0, 0 },
    { 0xa50a6cbd, 33 },
    { 0xc11a6cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a6cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43a6cc4, 68 },
    { 0x36ea6cc4, 98 },
    { 0xa9aa6cc6, 98 },
    { 0x610a6cc7, 23 },
    { 0, 0 },
    { 0x47da6cc9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a6ccc, 33 },
    { 0x7e3a6ccd, 98 },
    { 0x86aa6ccd, 55 },
    { 0x3aaa6ccc, 77 },
    { 0x811a6ccd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea6cd7, 95 },
    { 0xa60a6cd8, 95 },
    { 0x776a6cd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835a6cde, 61 },
    { 0, 0 },
    { 0xb57a6ce0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0a6ce3, 100 },
    { 0x8eea6ce4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681a6ced, 46 },
    { 0, 0 },
    { 0xc81a6cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2a6cf2, 61 },
    { 0x422a6cf3, 98 },
    { 0xbb0a6cf4, 90 },
    { 0xd04a6cf4, 33 },
    { 0, 0 },
    { 0x719a6cf7, 56 },
    { 0, 0 },
    { 0x74ea6cf9, 76 },
    { 0, 0 },
    { 0xffea6cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688a6cff, 99 },
    { 0xbc5a6d00, 67 },
    { 0x87ca6d01, 43 },
    { 0xb31a6d00, 55 },
    { 0x611a6d03, 52 },
    { 0x281a6d01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a6d07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36a6d14, 98 },
    { 0x5e9a6d15, 90 },
    { 0x2b7a6d16, 23 },
    { 0x93ea6d16, 33 },
    { 0x393a6d18, 81 },
    { 0x357a6d16, 98 },
    { 0x6dba6d1a, 33 },
    { 0x5e1a6d19, 46 },
    { 0xf55a6d1c, 80 },
    { 0xf4ea6d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1a6d20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a6d24, 34 },
    { 0xbfaa6d25, 33 },
    { 0x3cba6d26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebca6d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4a6d2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca6d34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82a6d37, 100 },
    { 0xa80a6d37, 100 },
    { 0xb10a6d38, 90 },
    { 0, 0 },
    { 0x8c6a6d3b, 91 },
    { 0, 0 },
    { 0x4c1a6d3d, 87 },
    { 0x3aaa6d3d, 33 },
    { 0x639a6d3f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a6d45, 33 },
    { 0xe5a6d46, 90 },
    { 0x6f2a6d47, 90 },
    { 0, 0 },
    { 0x5da6d49, 66 },
    { 0x99da6d49, 23 },
    { 0xdf8a6d4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5a6d50, 98 },
    { 0, 0 },
    { 0x9ba6d52, 75 },
    { 0xae5a6d53, 33 },
    { 0x333a6d54, 33 },
    { 0xd7a6d54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca6d5b, 100 },
    { 0x8da6d5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7a6d61, 46 },
    { 0xc14a6d61, 87 },
    { 0, 0 },
    { 0x4eda6d64, 34 },
    { 0x3b8a6d65, 90 },
    { 0x608a6d66, 98 },
    { 0x4d6a6d67, 81 },
    { 0x800a6d68, 93 },
    { 0x36ea6d66, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a6d6e, 100 },
    { 0x527a6d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6a6d75, 23 },
    { 0x60ba6d76, 61 },
    { 0x2fea6d76, 77 },
    { 0xec6a6d75, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a6d7c, 33 },
    { 0x6c8a6d7d, 52 },
    { 0xc4ca6d7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7a6d81, 78 },
    { 0xa7da6d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5a6d8a, 75 },
    { 0, 0 },
    { 0xe0aa6d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda6d90, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x541a6d93, 90 },
    { 0x575a6d94, 76 },
    { 0x696a6d94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a6d99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a6d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7a6da1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ca6dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43a6db4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45a6db8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1a6dbc, 98 },
    { 0xadfa6dbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84a6dc0, 23 },
    { 0x7cda6dc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6a6dc8, 23 },
    { 0x3b0a6dc9, 61 },
    { 0x60fa6dc8, 98 },
    { 0xe4a6dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38a6dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a6dd4, 33 },
    { 0, 0 },
    { 0x801a6dd6, 76 },
    { 0xf6a6dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4a6dda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194a6ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0a6de5, 95 },
    { 0xbdca6de5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a6deb, 95 },
    { 0x7fba6dec, 90 },
    { 0x19aa6ded, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a6df1, 49 },
    { 0x41a6df1, 61 },
    { 0x219a6df1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1a6df6, 55 },
    { 0xce1a6df7, 17 },
    { 0, 0 },
    { 0x205a6df9, 52 },
    { 0xc04a6dfa, 99 },
    { 0x285a6df9, 76 },
    { 0x5b5a6dfc, 90 },
    { 0, 0 },
    { 0xae8a6dfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755a6e02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eba6e0c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a6e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ea6e15, 90 },
    { 0, 0 },
    { 0x7ada6e17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5a6e1a, 33 },
    { 0xba1a6e1a, 76 },
    { 0xca2a6e1c, 61 },
    { 0xc3fa6e1a, 78 },
    { 0xb56a6e1e, 33 },
    { 0x22ba6e1e, 95 },
    { 0x2b0a6e1a, 95 },
    { 0, 0 },
    { 0x3aa6e22, 95 },
    { 0x99aa6e23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcba6e2b, 23 },
    { 0x20ba6e2c, 61 },
    { 0xcd8a6e2d, 17 },
    { 0x6eaa6e2e, 90 },
    { 0xd87a6e2c, 34 },
    { 0xc4ca6e30, 76 },
    { 0, 0 },
    { 0xb37a6e32, 90 },
    { 0, 0 },
    { 0x194a6e34, 34 },
    { 0x267a6e35, 61 },
    { 0, 0 },
    { 0xadfa6e37, 43 },
    { 0, 0 },
    { 0x9bda6e39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea6e3c, 67 },
    { 0x9ada6e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12a6e40, 33 },
    { 0, 0 },
    { 0x3f8a6e42, 17 },
    { 0x392a6e42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91a6e48, 33 },
    { 0, 0 },
    { 0xce0a6e4a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da6e52, 34 },
    { 0x195a6e53, 93 },
    { 0xc7ea6e52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aa6e58, 91 },
    { 0xf75a6e59, 77 },
    { 0, 0 },
    { 0xea9a6e5b, 99 },
    { 0x6d3a6e5b, 40 },
    { 0, 0 },
    { 0x413a6e5e, 46 },
    { 0, 0 },
    { 0xe16a6e60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06a6e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da6e69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aa6e78, 76 },
    { 0x76fa6e79, 33 },
    { 0xdb2a6e7a, 90 },
    { 0x4bea6e7b, 98 },
    { 0, 0 },
    { 0xffba6e7d, 90 },
    { 0x9c4a6e7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fa6e81, 76 },
    { 0xd87a6e82, 61 },
    { 0xd60a6e83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba6e87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188a6e8c, 34 },
    { 0, 0 },
    { 0xc42a6e8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a6e93, 33 },
    { 0x41ca6e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a6e99, 100 },
    { 0x5fa6e9a, 65 },
    { 0xb89a6e99, 76 },
    { 0, 0 },
    { 0xd24a6e9d, 90 },
    { 0, 0 },
    { 0x890a6e9f, 68 },
    { 0x114a6ea0, 33 },
    { 0x403a6ea1, 98 },
    { 0x379a6ea2, 88 },
    { 0xb35a6ea3, 98 },
    { 0, 0 },
    { 0x9e7a6ea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ea6ea8, 23 },
    { 0, 0 },
    { 0xf27a6eaa, 61 },
    { 0x6cca6eab, 61 },
    { 0, 0 },
    { 0x29ba6ead, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x144a6eb0, 90 },
    { 0, 0 },
    { 0x48aa6eb2, 34 },
    { 0, 0 },
    { 0x448a6eb4, 91 },
    { 0, 0 },
    { 0x6e9a6eb6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba6ebb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a6ec2, 90 },
    { 0xe96a6ec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a6ec6, 84 },
    { 0, 0 },
    { 0x8cfa6ec8, 98 },
    { 0xe32a6ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148a6ecf, 90 },
    { 0x8ea6ecf, 61 },
    { 0x2f4a6ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45a6edb, 43 },
    { 0, 0 },
    { 0x40a6edd, 33 },
    { 0xcfea6ede, 55 },
    { 0, 0 },
    { 0x15ea6ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ea6ef0, 95 },
    { 0xb1da6ef1, 55 },
    { 0, 0 },
    { 0xe1fa6ef3, 95 },
    { 0x104a6ef3, 55 },
    { 0xb7aa6ef4, 90 },
    { 0x817a6ef6, 78 },
    { 0x488a6ef5, 97 },
    { 0, 0 },
    { 0x1e0a6ef9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caa6efd, 88 },
    { 0xfb3a6efe, 46 },
    { 0, 0 },
    { 0xbe1a6f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06a6f05, 55 },
    { 0xff1a6f06, 87 },
    { 0x6c2a6f07, 52 },
    { 0xb07a6f07, 90 },
    { 0x763a6f09, 93 },
    { 0x15ea6f0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a6f0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaea6f10, 65 },
    { 0x951a6f11, 95 },
    { 0x756a6f12, 87 },
    { 0, 0 },
    { 0x88a6f14, 98 },
    { 0, 0 },
    { 0xc84a6f16, 77 },
    { 0x70ca6f17, 33 },
    { 0, 0 },
    { 0x5bda6f19, 98 },
    { 0x107a6f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa6f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aa6f21, 95 },
    { 0xb91a6f22, 34 },
    { 0xa44a6f22, 66 },
    { 0x94da6f24, 33 },
    { 0, 0 },
    { 0x1aea6f26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53a6f2f, 95 },
    { 0x93ca6f30, 93 },
    { 0xfa0a6f2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1a6f34, 98 },
    { 0x76da6f35, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9a6f39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cda6f40, 33 },
    { 0xb38a6f41, 90 },
    { 0x588a6f41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f46, 33 },
    { 0xe62a6f47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea6f4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5a6f52, 23 },
    { 0x569a6f52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9da6f5c, 43 },
    { 0, 0 },
    { 0xc15a6f5e, 61 },
    { 0xa25a6f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da6f62, 46 },
    { 0x383a6f63, 76 },
    { 0xbeea6f64, 90 },
    { 0xde0a6f63, 90 },
    { 0, 0 },
    { 0x375a6f67, 43 },
    { 0x784a6f67, 90 },
    { 0x2aba6f69, 53 },
    { 0xa7ca6f68, 61 },
    { 0x51ca6f6b, 90 },
    { 0x128a6f69, 90 },
    { 0x144a6f6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca6f70, 33 },
    { 0xcb0a6f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa6f77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a6f7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa6f7d, 33 },
    { 0x12ea6f7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81a6f81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a6f85, 66 },
    { 0x5afa6f86, 19 },
    { 0x122a6f86, 34 },
    { 0x22ea6f88, 93 },
    { 0xf89a6f88, 23 },
    { 0xf1da6f89, 76 },
    { 0xab5a6f8b, 87 },
    { 0x48aa6f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a6f91, 61 },
    { 0x586a6f92, 95 },
    { 0x6f4a6f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8a6f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa6fa1, 85 },
    { 0xd0a6fa2, 90 },
    { 0x143a6fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa6fa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ca6fab, 90 },
    { 0, 0 },
    { 0x530a6fad, 90 },
    { 0, 0 },
    { 0x478a6faf, 90 },
    { 0, 0 },
    { 0xec2a6fb1, 90 },
    { 0x6b5a6fb2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456a6fb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5a6fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a6fc1, 55 },
    { 0, 0 },
    { 0x575a6fc3, 65 },
    { 0x9eca6fc4, 90 },
    { 0, 0 },
    { 0x4c3a6fc6, 85 },
    { 0, 0 },
    { 0x385a6fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fea6fcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aa6fcf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a6fd2, 100 },
    { 0xd42a6fd3, 36 },
    { 0xa0ba6fd2, 75 },
    { 0, 0 },
    { 0x5d4a6fd6, 90 },
    { 0xb08a6fd7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a6fda, 75 },
    { 0xf7aa6fda, 33 },
    { 0x50ea6fdc, 34 },
    { 0xaca6fdd, 88 },
    { 0xd03a6fde, 43 },
    { 0xf4ba6fdd, 84 },
    { 0x199a6fdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x866a6fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba6fe9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a6ff2, 96 },
    { 0xf80a6ff2, 46 },
    { 0x955a6ff4, 55 },
    { 0x1eba6ff2, 90 },
    { 0x450a6ff5, 67 },
    { 0xa3ea6ff4, 90 },
    { 0x122a6ff8, 98 },
    { 0xf57a6ff9, 52 },
    { 0, 0 },
    { 0xd8a6ffb, 23 },
    { 0, 0 },
    { 0x672a6ffd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405a7001, 90 },
    { 0, 0 },
    { 0x8c5a7003, 61 },
    { 0, 0 },
    { 0xf55a7005, 81 },
    { 0x2aca7006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a700d, 68 },
    { 0x2f7a700d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a7015, 90 },
    { 0xc57a7016, 99 },
    { 0xe0a7017, 61 },
    { 0xb91a7018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a701d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980a7024, 100 },
    { 0xd93a7024, 90 },
    { 0xdd0a7026, 64 },
    { 0xe8aa7027, 66 },
    { 0x4b1a7028, 55 },
    { 0xf9da7027, 91 },
    { 0, 0 },
    { 0x919a702b, 90 },
    { 0, 0 },
    { 0x49da702d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x919a7030, 90 },
    { 0x5b4a7030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a7036, 98 },
    { 0xd43a7036, 23 },
    { 0xab2a7038, 23 },
    { 0x313a7038, 61 },
    { 0x1a5a703a, 90 },
    { 0xdda703a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0a703e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ba704f, 33 },
    { 0x15ca7050, 90 },
    { 0, 0 },
    { 0xd97a7052, 33 },
    { 0x4e6a7052, 90 },
    { 0x99aa7052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcea705b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7060, 98 },
    { 0x7c4a7061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceea7067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ea706e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a7071, 69 },
    { 0x86ca7072, 61 },
    { 0, 0 },
    { 0x5cfa7074, 33 },
    { 0x9eda7075, 55 },
    { 0x7e8a7076, 61 },
    { 0x8b6a7077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a707c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ca707f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea7084, 99 },
    { 0x8d1a7085, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a7088, 33 },
    { 0x9c9a7088, 61 },
    { 0, 0 },
    { 0x2da708b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a708f, 42 },
    { 0x9d5a708f, 23 },
    { 0xdaaa7091, 56 },
    { 0xc1fa7092, 95 },
    { 0, 0 },
    { 0x2e4a7094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3a7098, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09a709e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0a70a3, 33 },
    { 0, 0 },
    { 0x66ea70a5, 98 },
    { 0xd84a70a5, 53 },
    { 0xa02a70a7, 33 },
    { 0, 0 },
    { 0x47da70a9, 46 },
    { 0, 0 },
    { 0x1ffa70ab, 43 },
    { 0xb21a70ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491a70b4, 55 },
    { 0xf46a70b5, 65 },
    { 0x97ca70b6, 61 },
    { 0x406a70b7, 76 },
    { 0xf91a70b8, 76 },
    { 0x2f3a70b9, 46 },
    { 0x1bea70ba, 66 },
    { 0xa2da70bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a70c0, 90 },
    { 0xfeca70c1, 93 },
    { 0xfbea70c2, 43 },
    { 0, 0 },
    { 0xfcba70c4, 90 },
    { 0, 0 },
    { 0x458a70c6, 91 },
    { 0xf35a70c6, 34 },
    { 0x9aaa70c6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60a70cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2a70cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3a70d8, 40 },
    { 0xdeea70d8, 55 },
    { 0xb10a70da, 33 },
    { 0, 0 },
    { 0xad9a70dc, 76 },
    { 0, 0 },
    { 0x52da70de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8a70e4, 55 },
    { 0xaba70e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7a70ea, 95 },
    { 0xe96a70eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a70ee, 98 },
    { 0x331a70ef, 98 },
    { 0x60fa70ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbca70f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75a70f9, 95 },
    { 0xa99a70fa, 98 },
    { 0x2d4a70fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a70fe, 84 },
    { 0x53a70ff, 90 },
    { 0, 0 },
    { 0x990a7101, 90 },
    { 0x8e6a7102, 33 },
    { 0xac6a7103, 23 },
    { 0xcc1a7104, 77 },
    { 0x363a7103, 93 },
    { 0x6c9a7106, 61 },
    { 0, 0 },
    { 0x90aa7108, 33 },
    { 0, 0 },
    { 0x7ada710a, 68 },
    { 0x79da710b, 67 },
    { 0, 0 },
    { 0xca0a710d, 99 },
    { 0x5a1a710d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25a7111, 78 },
    { 0xa70a7112, 46 },
    { 0xce5a7113, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa7116, 93 },
    { 0x1f3a7117, 61 },
    { 0xda7116, 90 },
    { 0xc15a7119, 100 },
    { 0x758a7117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea711f, 76 },
    { 0x93da7120, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0da7123, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8a7126, 98 },
    { 0x392a7126, 61 },
    { 0x2cea7126, 76 },
    { 0, 0 },
    { 0xe61a712a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aa712d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7131, 61 },
    { 0x735a7132, 83 },
    { 0xa57a7133, 46 },
    { 0, 0 },
    { 0xd7aa7135, 76 },
    { 0x9cea7136, 55 },
    { 0, 0 },
    { 0x9bfa7138, 90 },
    { 0xa0da7139, 75 },
    { 0x87aa7139, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a713f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a7148, 34 },
    { 0x1b4a7149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08a714d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda7152, 23 },
    { 0x22a7153, 33 },
    { 0xff7a7154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca7158, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a715e, 33 },
    { 0, 0 },
    { 0x182a7160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a7163, 98 },
    { 0, 0 },
    { 0x778a7165, 66 },
    { 0, 0 },
    { 0x437a7167, 61 },
    { 0xbf7a7168, 33 },
    { 0, 0 },
    { 0xf02a716a, 94 },
    { 0x2dca716b, 96 },
    { 0xd66a716c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fa7171, 95 },
    { 0, 0 },
    { 0xb5ea7173, 17 },
    { 0x62aa7174, 95 },
    { 0x10da7175, 23 },
    { 0xd53a7176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x742a7179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x160a717c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a7181, 76 },
    { 0xe26a7182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7186, 33 },
    { 0xdb2a7187, 90 },
    { 0, 0 },
    { 0xae7a7189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425a718d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca7195, 34 },
    { 0x5caa7196, 90 },
    { 0xceba7197, 83 },
    { 0, 0 },
    { 0xa56a7199, 61 },
    { 0xccfa719a, 34 },
    { 0, 0 },
    { 0xad9a719c, 90 },
    { 0x712a719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690a71a1, 96 },
    { 0xcf0a71a2, 23 },
    { 0, 0 },
    { 0x22ca71a4, 100 },
    { 0xd55a71a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66a71a8, 61 },
    { 0x79ea71a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363a71ad, 53 },
    { 0x6b7a71ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa71b1, 75 },
    { 0, 0 },
    { 0x671a71b3, 83 },
    { 0x32ba71b4, 56 },
    { 0, 0 },
    { 0x23da71b6, 84 },
    { 0, 0 },
    { 0xc6ba71b8, 33 },
    { 0x672a71b9, 95 },
    { 0xe58a71b8, 66 },
    { 0x4e0a71bb, 81 },
    { 0xf3ca71b8, 76 },
    { 0xcb4a71bd, 78 },
    { 0x497a71b8, 23 },
    { 0xd3aa71bf, 88 },
    { 0xe95a71bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1a71c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdca71c6, 77 },
    { 0xa96a71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a71ca, 90 },
    { 0x4b2a71ca, 68 },
    { 0x8bba71cb, 95 },
    { 0, 0 },
    { 0xd44a71ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ca71d1, 33 },
    { 0x50fa71d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fa71d6, 34 },
    { 0, 0 },
    { 0xd85a71d8, 43 },
    { 0x60fa71d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a71dd, 87 },
    { 0x2dfa71dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160a71e4, 33 },
    { 0x9fca71e5, 55 },
    { 0, 0 },
    { 0xed1a71e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a71ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1a71f2, 90 },
    { 0x874a71f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfa71f7, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa71fc, 34 },
    { 0x4e0a71fc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228a7202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fa7205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daa720b, 61 },
    { 0xb44a720b, 76 },
    { 0, 0 },
    { 0x913a720e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ca7213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ba721a, 68 },
    { 0x80ea721b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa7222, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efa7227, 23 },
    { 0x47a7228, 66 },
    { 0xa97a7227, 33 },
    { 0, 0 },
    { 0x4d6a722b, 95 },
    { 0x2c3a722c, 55 },
    { 0, 0 },
    { 0x4a5a722e, 42 },
    { 0, 0 },
    { 0xf6ba7230, 78 },
    { 0xd2ea7231, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76a7235, 91 },
    { 0, 0 },
    { 0xff7a7237, 93 },
    { 0, 0 },
    { 0xb8ba7239, 61 },
    { 0, 0 },
    { 0x399a723b, 53 },
    { 0x203a723c, 95 },
    { 0x6efa723d, 90 },
    { 0x318a723e, 55 },
    { 0xe06a723b, 61 },
    { 0xd21a7240, 87 },
    { 0x258a7241, 40 },
    { 0x123a7240, 84 },
    { 0xfcfa7243, 55 },
    { 0xa84a7241, 100 },
    { 0x755a7245, 90 },
    { 0x57fa7245, 23 },
    { 0, 0 },
    { 0x9b3a7248, 95 },
    { 0xf83a7249, 36 },
    { 0xac4a724a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aa7252, 91 },
    { 0, 0 },
    { 0xa14a7254, 55 },
    { 0, 0 },
    { 0x500a7256, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a725a, 37 },
    { 0xa73a725a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9a7260, 61 },
    { 0, 0 },
    { 0xb71a7262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a7265, 34 },
    { 0, 0 },
    { 0xf05a7267, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bba7274, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da7278, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cda727d, 65 },
    { 0, 0 },
    { 0x2e1a727f, 97 },
    { 0xe1fa7280, 34 },
    { 0xa14a7281, 98 },
    { 0, 0 },
    { 0xc18a7283, 66 },
    { 0x291a7284, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea728b, 34 },
    { 0x8b1a728b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa7290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da7295, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba7298, 53 },
    { 0x4a3a7298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110a729e, 84 },
    { 0x8eda729e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeea72aa, 34 },
    { 0x51ba72aa, 61 },
    { 0x82ca72ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbba72b3, 91 },
    { 0, 0 },
    { 0x807a72b5, 76 },
    { 0, 0 },
    { 0x511a72b7, 91 },
    { 0, 0 },
    { 0x7e7a72b9, 23 },
    { 0xd83a72b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a72bf, 47 },
    { 0, 0 },
    { 0xa99a72c1, 33 },
    { 0, 0 },
    { 0x35ca72c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4a72c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x361a72c9, 33 },
    { 0x253a72ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386a72ce, 90 },
    { 0xea9a72cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a72d7, 61 },
    { 0xe03a72d7, 36 },
    { 0x5a3a72d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba72dc, 90 },
    { 0x124a72dc, 78 },
    { 0x6aba72dc, 61 },
    { 0x6c4a72dc, 61 },
    { 0x53fa72e0, 33 },
    { 0xf60a72df, 76 },
    { 0x738a72dd, 17 },
    { 0x94ca72e3, 95 },
    { 0, 0 },
    { 0x27da72e5, 90 },
    { 0xe13a72e5, 38 },
    { 0xe99a72e7, 78 },
    { 0, 0 },
    { 0x572a72e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a72ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da72f4, 99 },
    { 0x355a72f4, 90 },
    { 0xf44a72f6, 100 },
    { 0x5a9a72f4, 33 },
    { 0xd6ea72f4, 95 },
    { 0xa7ca72f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea72fc, 3 },
    { 0x94ca72fc, 95 },
    { 0, 0 },
    { 0x2c3a72ff, 67 },
    { 0xe71a7300, 61 },
    { 0x563a7301, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a7304, 33 },
    { 0, 0 },
    { 0x6dba7306, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36da730e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a7311, 23 },
    { 0xb7fa7312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca7317, 90 },
    { 0xe6a7318, 88 },
    { 0x6a4a7319, 33 },
    { 0x32da731a, 90 },
    { 0, 0 },
    { 0x96ca731c, 87 },
    { 0, 0 },
    { 0x6ffa731e, 33 },
    { 0x657a731f, 77 },
    { 0x555a731e, 98 },
    { 0xeb6a7321, 23 },
    { 0xb38a7321, 84 },
    { 0, 0 },
    { 0x5faa7324, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a732a, 23 },
    { 0x828a732b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679a7330, 65 },
    { 0xbd7a7331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cba733b, 43 },
    { 0x316a733c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa7347, 61 },
    { 0x800a7348, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa734e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fca7353, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95a7357, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2aa735c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a735f, 90 },
    { 0x94ca735f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aa7364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a7367, 98 },
    { 0x8c4a7368, 23 },
    { 0, 0 },
    { 0x76ea736a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a7373, 95 },
    { 0xfcca7373, 98 },
    { 0x638a7375, 85 },
    { 0, 0 },
    { 0xd04a7377, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470a737b, 23 },
    { 0x9a5a737c, 65 },
    { 0x9b1a737d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea7381, 76 },
    { 0x9e2a7382, 61 },
    { 0, 0 },
    { 0xb73a7384, 33 },
    { 0, 0 },
    { 0xa68a7386, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397a738b, 99 },
    { 0x424a738c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a738f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a7393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3a7397, 95 },
    { 0, 0 },
    { 0xa4ca7399, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a739d, 46 },
    { 0xda4a739e, 77 },
    { 0xcca739f, 98 },
    { 0, 0 },
    { 0xd00a73a1, 67 },
    { 0x41fa73a2, 23 },
    { 0xd58a73a3, 90 },
    { 0, 0 },
    { 0xc42a73a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ea73aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a73b5, 61 },
    { 0, 0 },
    { 0x64aa73b7, 95 },
    { 0x1c2a73b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685a73bb, 23 },
    { 0xf1aa73bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aba73c3, 23 },
    { 0x8baa73c4, 90 },
    { 0x782a73c4, 98 },
    { 0x3c8a73c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a73c9, 93 },
    { 0xf6ca73c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678a73cf, 23 },
    { 0x8f7a73cf, 33 },
    { 0x44ca73d0, 100 },
    { 0, 0 },
    { 0x15da73d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca73da, 33 },
    { 0x88fa73da, 76 },
    { 0x724a73dc, 66 },
    { 0x94a73dd, 98 },
    { 0x70aa73da, 34 },
    { 0, 0 },
    { 0x430a73e0, 78 },
    { 0x6ada73e1, 61 },
    { 0xc75a73e2, 76 },
    { 0x147a73e2, 23 },
    { 0, 0 },
    { 0x973a73e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297a73ed, 61 },
    { 0x1bba73ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5a73f2, 90 },
    { 0, 0 },
    { 0xd0a73f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a73fa, 61 },
    { 0, 0 },
    { 0x850a73fc, 61 },
    { 0xa21a73fd, 36 },
    { 0x407a73fe, 55 },
    { 0x768a73ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7405, 99 },
    { 0x184a7405, 78 },
    { 0x371a7407, 98 },
    { 0x1eda7408, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aa740f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a7412, 33 },
    { 0, 0 },
    { 0x7d6a7414, 55 },
    { 0, 0 },
    { 0x2d0a7416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aa7420, 99 },
    { 0, 0 },
    { 0x55ca7422, 61 },
    { 0x33aa7422, 23 },
    { 0x538a7424, 61 },
    { 0xf4ca7425, 98 },
    { 0xa78a7426, 95 },
    { 0xceca7425, 98 },
    { 0x293a7425, 61 },
    { 0x9d3a7429, 43 },
    { 0xfa9a742a, 77 },
    { 0x184a742a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea7431, 84 },
    { 0, 0 },
    { 0xceaa7433, 95 },
    { 0x690a7434, 61 },
    { 0x15ea7435, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76a7438, 90 },
    { 0xbe8a7439, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633a7440, 61 },
    { 0x9c0a7441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773a7448, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223a744f, 98 },
    { 0, 0 },
    { 0xcaaa7451, 33 },
    { 0x5b7a7452, 33 },
    { 0xd67a7451, 98 },
    { 0x797a7454, 56 },
    { 0x3f4a7455, 64 },
    { 0, 0 },
    { 0x22aa7457, 75 },
    { 0x139a7458, 98 },
    { 0x96ea7458, 78 },
    { 0xec6a745a, 90 },
    { 0x211a745b, 33 },
    { 0x2bda7459, 93 },
    { 0, 0 },
    { 0xaeba745e, 98 },
    { 0x318a745f, 93 },
    { 0xd79a745f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbea7463, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638a7468, 23 },
    { 0xdf0a7469, 93 },
    { 0x4eaa7469, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8a7475, 68 },
    { 0x258a7475, 66 },
    { 0x4b9a7475, 98 },
    { 0x3afa7476, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a747b, 40 },
    { 0x7f1a747b, 43 },
    { 0x17a747b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a7480, 98 },
    { 0x1f1a7480, 43 },
    { 0, 0 },
    { 0xbe8a7483, 95 },
    { 0x7fea7483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefa748e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3a7496, 33 },
    { 0x6cda7497, 23 },
    { 0x587a7497, 76 },
    { 0xdcca7499, 33 },
    { 0, 0 },
    { 0xa39a749b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a74a3, 98 },
    { 0x1a0a74a4, 90 },
    { 0xc78a74a3, 33 },
    { 0x716a74a6, 17 },
    { 0xf30a74a4, 85 },
    { 0x17aa74a8, 55 },
    { 0xc26a74a9, 76 },
    { 0x6e3a74aa, 23 },
    { 0x1eaa74aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39a74b0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ca74b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981a74b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a74c0, 91 },
    { 0, 0 },
    { 0x44ba74c2, 23 },
    { 0, 0 },
    { 0x74da74c4, 52 },
    { 0, 0 },
    { 0xe95a74c6, 97 },
    { 0x323a74c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1a74ca, 78 },
    { 0, 0 },
    { 0x9aca74cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ca74cf, 61 },
    { 0, 0 },
    { 0x972a74d1, 98 },
    { 0xbcda74d1, 34 },
    { 0xe05a74d3, 94 },
    { 0xb43a74d4, 12 },
    { 0xcbfa74d5, 95 },
    { 0x17a74d5, 23 },
    { 0x6aba74d7, 61 },
    { 0, 0 },
    { 0x3aaa74d9, 76 },
    { 0, 0 },
    { 0xdaba74db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a74df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199a74e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7a74e6, 67 },
    { 0xd9ea74e6, 43 },
    { 0x4a3a74e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x973a74eb, 33 },
    { 0xfba74eb, 99 },
    { 0, 0 },
    { 0x837a74ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4a74f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250a7502, 43 },
    { 0x703a7503, 84 },
    { 0xe6a7503, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a7507, 90 },
    { 0xbbaa7508, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x202a750b, 76 },
    { 0x487a750b, 24 },
    { 0, 0 },
    { 0x47da750e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660a751c, 90 },
    { 0x1c8a751d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a7520, 98 },
    { 0x1f0a7521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48a7524, 61 },
    { 0, 0 },
    { 0x654a7526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x344a7529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea752e, 95 },
    { 0x998a752f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dba7532, 61 },
    { 0x9b3a7533, 52 },
    { 0x2aa7533, 90 },
    { 0, 0 },
    { 0xe32a7536, 98 },
    { 0xa87a7536, 33 },
    { 0x94ca7537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ca753b, 33 },
    { 0x8c0a753c, 100 },
    { 0xb69a753b, 98 },
    { 0xc14a753c, 23 },
    { 0x7a2a753f, 90 },
    { 0xb6fa7540, 87 },
    { 0x3a1a7541, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa7544, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a754c, 95 },
    { 0x15ba754d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa7551, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a7554, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x449a7557, 33 },
    { 0x154a7558, 81 },
    { 0xae5a7559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3a755e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a7561, 33 },
    { 0, 0 },
    { 0x110a7563, 93 },
    { 0, 0 },
    { 0x535a7565, 78 },
    { 0x719a7566, 95 },
    { 0x4c4a7566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73da756d, 95 },
    { 0, 0 },
    { 0x8f5a756f, 61 },
    { 0x505a7570, 90 },
    { 0x21aa7571, 46 },
    { 0xffea7570, 34 },
    { 0xf11a7572, 100 },
    { 0x971a7574, 40 },
    { 0x4b8a7574, 23 },
    { 0xc20a756f, 91 },
    { 0xeb6a7577, 77 },
    { 0x599a7578, 77 },
    { 0x265a7572, 33 },
    { 0xfd2a7576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a757f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a7585, 33 },
    { 0xf27a7585, 34 },
    { 0xe05a7587, 77 },
    { 0x6b0a7588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2a758b, 90 },
    { 0x7b7a758c, 93 },
    { 0x2e8a758d, 84 },
    { 0, 0 },
    { 0x77da758f, 95 },
    { 0xc95a7590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa759a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fa75a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa75a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a75a8, 95 },
    { 0xd69a75a9, 55 },
    { 0xc7a75a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a75b4, 90 },
    { 0, 0 },
    { 0x779a75b6, 98 },
    { 0xd5ba75b7, 65 },
    { 0, 0 },
    { 0x874a75b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40a75bf, 98 },
    { 0, 0 },
    { 0x736a75c1, 95 },
    { 0xe48a75c2, 33 },
    { 0x5d1a75c3, 43 },
    { 0x66ba75c4, 34 },
    { 0x57a75c2, 33 },
    { 0xbb3a75c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3a75ca, 99 },
    { 0x369a75cb, 61 },
    { 0x216a75cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca75cf, 34 },
    { 0, 0 },
    { 0xafca75d1, 23 },
    { 0xb7ba75d2, 23 },
    { 0x769a75d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca75d7, 98 },
    { 0xbc3a75d7, 87 },
    { 0, 0 },
    { 0xda2a75da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6a75de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a75e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ba75e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02a75ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1a75ed, 101 },
    { 0xa90a75ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2aa75f3, 33 },
    { 0x584a75f4, 95 },
    { 0, 0 },
    { 0x5b2a75f6, 61 },
    { 0x78a75f6, 43 },
    { 0xe77a75f6, 91 },
    { 0xf12a75f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a75fe, 91 },
    { 0, 0 },
    { 0x3d5a7600, 90 },
    { 0x906a7601, 34 },
    { 0x7fea7602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7605, 61 },
    { 0xbdda7606, 95 },
    { 0xafda7607, 65 },
    { 0x9eca7606, 23 },
    { 0, 0 },
    { 0xe29a760a, 55 },
    { 0x83da760b, 34 },
    { 0x9b5a760c, 36 },
    { 0xfb3a760d, 76 },
    { 0x462a760e, 90 },
    { 0xd71a760f, 61 },
    { 0xb94a7610, 61 },
    { 0xe24a7611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61a7615, 98 },
    { 0, 0 },
    { 0xc76a7617, 90 },
    { 0xaada7618, 23 },
    { 0x61a7618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba761c, 98 },
    { 0xd59a761d, 98 },
    { 0x830a761e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1a7624, 61 },
    { 0x6e1a7625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1a762d, 55 },
    { 0, 0 },
    { 0xc7aa762f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x749a7632, 46 },
    { 0x650a7633, 90 },
    { 0x14ea7634, 84 },
    { 0x5dda7632, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951a7639, 33 },
    { 0x9fda763a, 87 },
    { 0x478a763b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a763e, 61 },
    { 0xd49a763f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a764c, 90 },
    { 0, 0 },
    { 0xd3aa764e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba7654, 85 },
    { 0, 0 },
    { 0xb6ea7656, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a765a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a7661, 65 },
    { 0, 0 },
    { 0xf8ba7663, 23 },
    { 0xe7da7664, 33 },
    { 0xdaaa7665, 67 },
    { 0x502a7666, 90 },
    { 0, 0 },
    { 0x920a7668, 90 },
    { 0xf98a7669, 17 },
    { 0, 0 },
    { 0xa2da766b, 61 },
    { 0xb31a766b, 33 },
    { 0x995a766c, 23 },
    { 0x4e2a766c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa7671, 23 },
    { 0, 0 },
    { 0x58da7673, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a7679, 98 },
    { 0x399a767a, 98 },
    { 0, 0 },
    { 0x1f2a767c, 90 },
    { 0xb87a767c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x170a7680, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ea7684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ea768a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca7691, 23 },
    { 0, 0 },
    { 0x9dea7693, 90 },
    { 0xecda7694, 23 },
    { 0xa5fa7695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea769b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7a76a2, 93 },
    { 0xdaca76a2, 33 },
    { 0, 0 },
    { 0x718a76a5, 55 },
    { 0x90da76a6, 68 },
    { 0xaf6a76a6, 75 },
    { 0, 0 },
    { 0x71da76a9, 91 },
    { 0xd06a76aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7da76b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x268a76ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772a76d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97a76d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a76dd, 85 },
    { 0, 0 },
    { 0xb21a76df, 23 },
    { 0, 0 },
    { 0xbc9a76e1, 87 },
    { 0, 0 },
    { 0x587a76e3, 23 },
    { 0, 0 },
    { 0xa2aa76e5, 33 },
    { 0xa5da76e5, 33 },
    { 0x1a3a76e6, 33 },
    { 0xe52a76e8, 33 },
    { 0xa08a76e9, 61 },
    { 0xf6ba76ea, 95 },
    { 0x95ba76ea, 93 },
    { 0, 0 },
    { 0x9b7a76ed, 38 },
    { 0x46fa76ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faa76f5, 33 },
    { 0, 0 },
    { 0x43a76f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4a76fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807a7701, 100 },
    { 0xad7a7702, 95 },
    { 0, 0 },
    { 0x996a7704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9a7708, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a770b, 95 },
    { 0x88a770c, 33 },
    { 0, 0 },
    { 0xc64a770e, 61 },
    { 0x94aa770f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca7714, 90 },
    { 0, 0 },
    { 0x29ba7716, 33 },
    { 0, 0 },
    { 0x4f6a7718, 95 },
    { 0x710a7719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a771d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0a7720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3a7725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafba7728, 90 },
    { 0x810a7729, 95 },
    { 0xa2ba772a, 97 },
    { 0x81a7728, 52 },
    { 0x691a772c, 78 },
    { 0x6b0a772d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ca7732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a7735, 93 },
    { 0, 0 },
    { 0x431a7737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfa7741, 76 },
    { 0xdc1a7742, 33 },
    { 0x3b2a7741, 60 },
    { 0, 0 },
    { 0x872a7745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4a774d, 34 },
    { 0, 0 },
    { 0xd04a774f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43a7754, 95 },
    { 0, 0 },
    { 0xcf5a7756, 33 },
    { 0x5f0a7756, 61 },
    { 0xb96a7758, 34 },
    { 0x2f3a7758, 46 },
    { 0xc37a775a, 88 },
    { 0x800a775a, 43 },
    { 0x2c3a7759, 47 },
    { 0x45a775a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baa7760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eca7765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7a776b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aa7770, 33 },
    { 0x24aa7771, 96 },
    { 0, 0 },
    { 0x221a7773, 91 },
    { 0x488a7773, 98 },
    { 0, 0 },
    { 0x1a7a7776, 78 },
    { 0, 0 },
    { 0x248a7778, 76 },
    { 0, 0 },
    { 0x97ea777a, 33 },
    { 0xc45a777a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cca777e, 60 },
    { 0x777a777f, 23 },
    { 0x421a7780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x543a7783, 99 },
    { 0xb8ba7784, 33 },
    { 0xeba7785, 33 },
    { 0x3f7a7786, 76 },
    { 0x6e0a7787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9a778a, 43 },
    { 0xbe1a778b, 61 },
    { 0, 0 },
    { 0x240a778d, 98 },
    { 0xa49a778e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a7792, 61 },
    { 0xacaa7792, 33 },
    { 0x4a1a7792, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a7798, 40 },
    { 0, 0 },
    { 0x7eaa779a, 90 },
    { 0, 0 },
    { 0xc85a779c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x533a779f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea77a6, 76 },
    { 0x421a77a7, 76 },
    { 0x948a77a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a77ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba77ae, 58 },
    { 0xd64a77ae, 43 },
    { 0xd8aa77b0, 95 },
    { 0x542a77af, 33 },
    { 0x55ba77b2, 66 },
    { 0, 0 },
    { 0xe78a77b4, 33 },
    { 0, 0 },
    { 0x7aaa77b6, 95 },
    { 0x13ba77b7, 98 },
    { 0xca4a77b7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629a77bc, 84 },
    { 0, 0 },
    { 0xf86a77be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba77c5, 84 },
    { 0xf87a77c6, 52 },
    { 0x456a77c7, 90 },
    { 0xbcfa77c8, 95 },
    { 0x9c6a77c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x893a77cc, 90 },
    { 0, 0 },
    { 0x22aa77ce, 34 },
    { 0x52ea77cf, 100 },
    { 0xbc0a77d0, 66 },
    { 0, 0 },
    { 0x649a77d2, 88 },
    { 0, 0 },
    { 0x55ba77d4, 95 },
    { 0xa40a77d4, 33 },
    { 0x88a77d6, 61 },
    { 0, 0 },
    { 0x3a7a77d8, 33 },
    { 0xfe3a77d9, 55 },
    { 0, 0 },
    { 0xad5a77db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44a77e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fa77e9, 23 },
    { 0x407a77ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca77ed, 61 },
    { 0x681a77ee, 95 },
    { 0xe69a77ef, 55 },
    { 0x81ea77f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a77f4, 87 },
    { 0x376a77f5, 76 },
    { 0, 0 },
    { 0x462a77f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a77fe, 90 },
    { 0xad4a77ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a7802, 80 },
    { 0xa07a7802, 84 },
    { 0xf2a7803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a7808, 55 },
    { 0, 0 },
    { 0x7a1a780a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a7810, 98 },
    { 0, 0 },
    { 0xe00a7812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ba7815, 88 },
    { 0xf8fa7816, 90 },
    { 0xcb6a7817, 78 },
    { 0x8a6a7817, 76 },
    { 0x184a7816, 95 },
    { 0xb5a781a, 90 },
    { 0x263a7816, 95 },
    { 0xefba7816, 81 },
    { 0xd3fa7817, 23 },
    { 0x57a781e, 78 },
    { 0x495a781e, 55 },
    { 0xefa7818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ca7826, 33 },
    { 0, 0 },
    { 0xaf0a7828, 66 },
    { 0xdada7829, 33 },
    { 0xcf5a782a, 66 },
    { 0xcc4a7829, 34 },
    { 0xe7ba782c, 33 },
    { 0, 0 },
    { 0x4d5a782e, 40 },
    { 0x980a782e, 78 },
    { 0xcc7a782e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ca7834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780a783e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fda7848, 34 },
    { 0, 0 },
    { 0x6d0a784a, 90 },
    { 0, 0 },
    { 0x466a784c, 33 },
    { 0x4eaa784c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a7854, 61 },
    { 0xd59a7855, 76 },
    { 0, 0 },
    { 0xec2a7857, 43 },
    { 0x58ea7857, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6a785f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fda7863, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x262a7866, 98 },
    { 0x6c2a7867, 34 },
    { 0xc54a7867, 55 },
    { 0x8dca7866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a786c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7873, 98 },
    { 0x3bda7874, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101a7880, 66 },
    { 0x795a7881, 68 },
    { 0x2da7882, 34 },
    { 0, 0 },
    { 0xa8da7884, 77 },
    { 0xbf3a7885, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a7888, 83 },
    { 0xc49a7889, 91 },
    { 0x4e1a7889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fa788d, 95 },
    { 0, 0 },
    { 0xd97a788f, 75 },
    { 0, 0 },
    { 0x868a7891, 55 },
    { 0x15fa7892, 98 },
    { 0xc66a7893, 90 },
    { 0x196a7894, 100 },
    { 0xca9a7894, 23 },
    { 0x51da7891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a789a, 23 },
    { 0xacfa789a, 23 },
    { 0x998a789a, 100 },
    { 0, 0 },
    { 0xf89a789e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ca78a1, 33 },
    { 0xb5ba78a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a78a7, 43 },
    { 0xf3ba78a7, 33 },
    { 0, 0 },
    { 0xedea78aa, 34 },
    { 0xabea78ab, 23 },
    { 0xea8a78ac, 61 },
    { 0x984a78ab, 100 },
    { 0, 0 },
    { 0x988a78af, 23 },
    { 0x6a0a78b0, 34 },
    { 0, 0 },
    { 0x8b5a78b2, 98 },
    { 0x2f3a78b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a78b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da78bc, 85 },
    { 0x66a78bc, 98 },
    { 0, 0 },
    { 0x7e1a78bf, 55 },
    { 0xeb6a78c0, 98 },
    { 0x403a78c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a78c4, 46 },
    { 0x3efa78c4, 95 },
    { 0x72fa78c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fa78ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85a78cd, 61 },
    { 0, 0 },
    { 0xc30a78cf, 91 },
    { 0, 0 },
    { 0x3e9a78d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a78d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ca78dc, 34 },
    { 0, 0 },
    { 0x4c9a78de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a78e1, 80 },
    { 0, 0 },
    { 0xc5ca78e3, 55 },
    { 0x27fa78e4, 98 },
    { 0xde4a78e5, 87 },
    { 0x581a78e6, 78 },
    { 0x2bfa78e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4a78ea, 95 },
    { 0x62ba78eb, 98 },
    { 0xb67a78ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faa78f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a78f7, 52 },
    { 0x70ca78f8, 95 },
    { 0xdb9a78f9, 91 },
    { 0x4dea78f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea78fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39da7900, 90 },
    { 0x1cda7901, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02a7907, 76 },
    { 0xaa1a7908, 98 },
    { 0xe9ea7909, 78 },
    { 0xfdea7909, 55 },
    { 0, 0 },
    { 0xd2ca790c, 23 },
    { 0x981a790d, 101 },
    { 0, 0 },
    { 0x8fea790f, 61 },
    { 0, 0 },
    { 0xf8ba7911, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389a7918, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa791c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca7920, 61 },
    { 0xceda7920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ca7928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fa792d, 99 },
    { 0, 0 },
    { 0x476a792f, 95 },
    { 0xe67a792f, 52 },
    { 0xc2ea7931, 95 },
    { 0x993a7931, 95 },
    { 0x134a7930, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a7936, 90 },
    { 0x3d5a7937, 23 },
    { 0, 0 },
    { 0x5cfa7939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602a7942, 33 },
    { 0xd2ea7943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a7947, 88 },
    { 0x4c2a7948, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a794d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366a7952, 46 },
    { 0, 0 },
    { 0x155a7954, 33 },
    { 0x7bba7954, 90 },
    { 0xdc8a7955, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba795b, 90 },
    { 0xb5ea795c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01a7961, 83 },
    { 0x7c6a7961, 33 },
    { 0x379a7962, 55 },
    { 0, 0 },
    { 0x391a7965, 38 },
    { 0, 0 },
    { 0x1f1a7967, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdea796a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba796e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a7978, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a797e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fa7981, 33 },
    { 0, 0 },
    { 0x168a7983, 95 },
    { 0, 0 },
    { 0x6f8a7985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a798b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1a798e, 77 },
    { 0, 0 },
    { 0x988a7990, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a799d, 61 },
    { 0xf65a799e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a79a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506a79aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caa79b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396a79b7, 46 },
    { 0xe01a79b8, 98 },
    { 0x7d4a79b7, 33 },
    { 0, 0 },
    { 0x99ea79bb, 91 },
    { 0x782a79bc, 76 },
    { 0x254a79bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3a79c1, 95 },
    { 0, 0 },
    { 0x705a79c3, 87 },
    { 0x3f1a79c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aea79c9, 100 },
    { 0, 0 },
    { 0x39aa79cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa79cf, 33 },
    { 0, 0 },
    { 0x4eba79d1, 85 },
    { 0x2b7a79d1, 33 },
    { 0x5baa79d3, 66 },
    { 0, 0 },
    { 0x224a79d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7a79ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a79f0, 46 },
    { 0x3ca79f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa79f5, 34 },
    { 0x9f3a79f6, 68 },
    { 0x1d0a79f7, 33 },
    { 0xbbca79f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399a79fc, 93 },
    { 0x391a79fc, 33 },
    { 0xdb7a79fe, 23 },
    { 0x5b7a79fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59a7a05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faa7a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a7a16, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a7a1b, 34 },
    { 0x721a7a1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8a7a22, 17 },
    { 0xfada7a22, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa7a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674a7a2c, 99 },
    { 0x640a7a2c, 87 },
    { 0, 0 },
    { 0x50fa7a2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15a7a3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7a41, 76 },
    { 0xd05a7a42, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a7a46, 95 },
    { 0x34fa7a47, 33 },
    { 0, 0 },
    { 0x67a7a49, 90 },
    { 0x23fa7a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a7a4f, 61 },
    { 0, 0 },
    { 0x18ea7a51, 33 },
    { 0xb46a7a52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a7a59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba7a5c, 66 },
    { 0xfdca7a5c, 33 },
    { 0x449a7a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70a7a66, 77 },
    { 0, 0 },
    { 0xcdaa7a68, 46 },
    { 0x5cba7a69, 88 },
    { 0xaada7a68, 93 },
    { 0, 0 },
    { 0xb09a7a6c, 33 },
    { 0xf6a7a6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ba7a70, 100 },
    { 0x959a7a71, 90 },
    { 0x73aa7a72, 95 },
    { 0xd57a7a73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ea7a76, 55 },
    { 0x52ba7a77, 33 },
    { 0, 0 },
    { 0xf2ba7a79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550a7a8b, 84 },
    { 0x6d7a7a8c, 61 },
    { 0x2fda7a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fa7a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340a7a95, 90 },
    { 0x907a7a96, 69 },
    { 0x9b7a7a97, 96 },
    { 0x17ca7a95, 46 },
    { 0xcd3a7a96, 95 },
    { 0x901a7a9a, 98 },
    { 0x95aa7a95, 76 },
    { 0xbbfa7a96, 95 },
    { 0, 0 },
    { 0xc6a7a9e, 56 },
    { 0x17fa7a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a7aa2, 98 },
    { 0, 0 },
    { 0xa14a7aa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66da7aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda7aac, 90 },
    { 0x1c3a7aac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa7ab9, 98 },
    { 0, 0 },
    { 0x368a7abb, 77 },
    { 0x99da7abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca7ac5, 55 },
    { 0x7d8a7ac5, 77 },
    { 0xc6aa7ac6, 76 },
    { 0xed3a7ac8, 23 },
    { 0xae0a7ac5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1a7acc, 52 },
    { 0x502a7acc, 90 },
    { 0, 0 },
    { 0x957a7acf, 95 },
    { 0x600a7acf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea7ad4, 61 },
    { 0, 0 },
    { 0x8e7a7ad6, 90 },
    { 0x1caa7ad7, 33 },
    { 0x9ffa7ad8, 98 },
    { 0x3d6a7ad8, 95 },
    { 0x8bba7ad9, 61 },
    { 0, 0 },
    { 0x286a7adc, 17 },
    { 0x58fa7add, 90 },
    { 0, 0 },
    { 0x727a7adf, 55 },
    { 0x74ca7ae0, 90 },
    { 0x1dda7ae1, 33 },
    { 0x528a7ae2, 38 },
    { 0x5c2a7ae3, 76 },
    { 0x61aa7ae4, 95 },
    { 0x19aa7ae2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49a7aea, 95 },
    { 0xcd6a7aeb, 98 },
    { 0x3c3a7aec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3a7af0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da7afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9a7b01, 68 },
    { 0, 0 },
    { 0x319a7b03, 98 },
    { 0, 0 },
    { 0x321a7b05, 23 },
    { 0xa89a7b06, 95 },
    { 0x812a7b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa7b0b, 90 },
    { 0x912a7b0b, 99 },
    { 0x729a7b0d, 80 },
    { 0xc42a7b0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85a7b12, 76 },
    { 0x742a7b13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a7b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a7b1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a7b1f, 91 },
    { 0, 0 },
    { 0x74aa7b21, 33 },
    { 0x652a7b22, 75 },
    { 0x8b7a7b23, 61 },
    { 0x33ca7b21, 90 },
    { 0x588a7b25, 93 },
    { 0x462a7b26, 93 },
    { 0x3c7a7b22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba7b2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9a7b32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3a7b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86a7b40, 92 },
    { 0xf7a7b41, 90 },
    { 0x5bda7b42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867a7b46, 76 },
    { 0x198a7b46, 46 },
    { 0x4a9a7b48, 96 },
    { 0, 0 },
    { 0xdbea7b4a, 33 },
    { 0, 0 },
    { 0xfa4a7b4c, 33 },
    { 0x34aa7b4d, 88 },
    { 0x2dea7b4d, 100 },
    { 0, 0 },
    { 0x92a7b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974a7b61, 98 },
    { 0x7eda7b62, 33 },
    { 0x847a7b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ca7b66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9a7b69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa7b6e, 77 },
    { 0xcb0a7b6f, 34 },
    { 0, 0 },
    { 0x643a7b71, 38 },
    { 0x3a5a7b72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ea7b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99da7b7e, 98 },
    { 0xd51a7b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95da7b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca7b85, 61 },
    { 0x871a7b86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a7b8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7a7b8d, 33 },
    { 0, 0 },
    { 0xcf1a7b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962a7b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a7b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a7ba1, 76 },
    { 0x602a7ba2, 95 },
    { 0x299a7ba3, 55 },
    { 0, 0 },
    { 0x321a7ba5, 91 },
    { 0x45ea7ba6, 43 },
    { 0x313a7ba6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a7baa, 87 },
    { 0, 0 },
    { 0xda7bac, 33 },
    { 0x908a7bad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9a7bb1, 55 },
    { 0x1bfa7bb2, 75 },
    { 0x1a7a7bb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4a7bb6, 66 },
    { 0, 0 },
    { 0x666a7bb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea7bbc, 84 },
    { 0x44aa7bbc, 55 },
    { 0x273a7bbc, 65 },
    { 0, 0 },
    { 0x984a7bc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aa7bc3, 17 },
    { 0x725a7bc4, 93 },
    { 0xa33a7bc4, 23 },
    { 0xc9ca7bc6, 90 },
    { 0, 0 },
    { 0x4a7bc8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceda7bcf, 61 },
    { 0x711a7bd0, 46 },
    { 0x454a7bcf, 87 },
    { 0xbbaa7bd2, 23 },
    { 0x31aa7bd0, 34 },
    { 0xbbfa7bd4, 17 },
    { 0x87ca7bd4, 77 },
    { 0xea7a7bd6, 33 },
    { 0, 0 },
    { 0x456a7bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7bdb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0a7be4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a7be7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4a7bea, 90 },
    { 0xdc4a7bea, 43 },
    { 0x3d8a7bec, 23 },
    { 0xf78a7bec, 90 },
    { 0x65a7bee, 74 },
    { 0xc2ea7bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a7bf2, 77 },
    { 0x504a7bf2, 98 },
    { 0, 0 },
    { 0x337a7bf5, 38 },
    { 0x1fba7bf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880a7bff, 33 },
    { 0, 0 },
    { 0x2c0a7c01, 98 },
    { 0, 0 },
    { 0xb87a7c03, 38 },
    { 0x721a7c03, 42 },
    { 0, 0 },
    { 0x96ca7c06, 33 },
    { 0, 0 },
    { 0x62fa7c08, 55 },
    { 0xfb2a7c08, 99 },
    { 0, 0 },
    { 0x482a7c0b, 87 },
    { 0, 0 },
    { 0xbb0a7c0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dea7c13, 61 },
    { 0x288a7c14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4a7c19, 55 },
    { 0x4cca7c19, 61 },
    { 0x4afa7c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ea7c1e, 81 },
    { 0xee4a7c1e, 95 },
    { 0x739a7c20, 33 },
    { 0xe91a7c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6a7c27, 76 },
    { 0x5b7a7c28, 91 },
    { 0xb53a7c29, 90 },
    { 0x951a7c28, 95 },
    { 0x765a7c2b, 55 },
    { 0x707a7c2c, 52 },
    { 0x298a7c2b, 68 },
    { 0x9ba7c2e, 40 },
    { 0x2b0a7c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca7c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aa7c35, 87 },
    { 0x4d5a7c36, 23 },
    { 0xc2aa7c35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ca7c3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a7c44, 95 },
    { 0x268a7c45, 87 },
    { 0, 0 },
    { 0x512a7c47, 23 },
    { 0xc2ca7c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688a7c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43a7c57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a7c5d, 76 },
    { 0xa40a7c5e, 33 },
    { 0, 0 },
    { 0xb53a7c60, 87 },
    { 0x351a7c60, 23 },
    { 0x53a7c62, 76 },
    { 0, 0 },
    { 0x5c3a7c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a7c67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17a7c6b, 98 },
    { 0xeaca7c6b, 46 },
    { 0x655a7c6d, 33 },
    { 0x694a7c6d, 90 },
    { 0xb0a7c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baa7c72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca7c76, 98 },
    { 0x423a7c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4a7c7d, 84 },
    { 0, 0 },
    { 0xdd5a7c7f, 33 },
    { 0, 0 },
    { 0xf96a7c81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x216a7c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a7c88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792a7c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1a7c8f, 95 },
    { 0x51a7c90, 98 },
    { 0xfdca7c91, 90 },
    { 0x4b4a7c92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca7c97, 23 },
    { 0, 0 },
    { 0x27a7c99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7c9c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ba7c9f, 98 },
    { 0, 0 },
    { 0x39fa7ca1, 33 },
    { 0x9f4a7ca2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ea7ca6, 90 },
    { 0x545a7ca7, 95 },
    { 0, 0 },
    { 0x757a7ca9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa7cac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a7cb3, 91 },
    { 0x483a7cb3, 90 },
    { 0, 0 },
    { 0x4e3a7cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a7cb9, 99 },
    { 0x26da7cba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a7ccc, 95 },
    { 0x534a7ccd, 18 },
    { 0x738a7ccc, 33 },
    { 0x38ba7cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3a7cd7, 99 },
    { 0xabaa7cd7, 34 },
    { 0, 0 },
    { 0x24ca7cda, 87 },
    { 0, 0 },
    { 0x116a7cdc, 87 },
    { 0x1ada7cdd, 94 },
    { 0, 0 },
    { 0x9d6a7cdf, 33 },
    { 0, 0 },
    { 0x70aa7ce1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a7ce5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fa7cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24a7cef, 77 },
    { 0x742a7cef, 61 },
    { 0xca6a7cf1, 33 },
    { 0x482a7cf1, 91 },
    { 0xdb8a7cf3, 23 },
    { 0x529a7cef, 93 },
    { 0, 0 },
    { 0x5e0a7cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8a7cf9, 95 },
    { 0, 0 },
    { 0xca5a7cfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a7d05, 33 },
    { 0, 0 },
    { 0xbaea7d07, 95 },
    { 0xf8ca7d07, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea7d0b, 98 },
    { 0x18aa7d0b, 17 },
    { 0xb9ea7d0d, 61 },
    { 0, 0 },
    { 0xa7ea7d0f, 95 },
    { 0xb2ba7d10, 33 },
    { 0, 0 },
    { 0x373a7d12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a7d15, 23 },
    { 0xc5aa7d16, 33 },
    { 0x20da7d15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea7d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2a7d1f, 61 },
    { 0, 0 },
    { 0x336a7d21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a7d24, 33 },
    { 0xa8ca7d25, 17 },
    { 0, 0 },
    { 0xc1da7d27, 91 },
    { 0xab5a7d28, 95 },
    { 0x8e5a7d29, 61 },
    { 0, 0 },
    { 0xaf9a7d2b, 61 },
    { 0x5b0a7d2c, 87 },
    { 0xddaa7d2d, 93 },
    { 0, 0 },
    { 0x889a7d2f, 46 },
    { 0x838a7d2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a7d37, 68 },
    { 0, 0 },
    { 0xf51a7d39, 95 },
    { 0x617a7d3a, 67 },
    { 0, 0 },
    { 0xb73a7d3c, 76 },
    { 0x495a7d3d, 43 },
    { 0x25fa7d3e, 96 },
    { 0xb4aa7d3f, 23 },
    { 0, 0 },
    { 0xfcaa7d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcca7d47, 85 },
    { 0, 0 },
    { 0x2bda7d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a7d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9a7d54, 87 },
    { 0x196a7d55, 55 },
    { 0x251a7d56, 99 },
    { 0xc41a7d57, 52 },
    { 0, 0 },
    { 0xd14a7d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a7d5c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a7d5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a7d63, 98 },
    { 0, 0 },
    { 0xcc9a7d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a7d6b, 61 },
    { 0, 0 },
    { 0xc8aa7d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fa7d72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90da7d76, 43 },
    { 0x778a7d76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa7d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a7d81, 69 },
    { 0, 0 },
    { 0xf75a7d83, 88 },
    { 0, 0 },
    { 0xef1a7d85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330a7d8b, 33 },
    { 0xaaaa7d8c, 98 },
    { 0x133a7d8d, 98 },
    { 0x630a7d8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a7d92, 33 },
    { 0x859a7d92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba7d9b, 23 },
    { 0xc7aa7d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x819a7d9f, 43 },
    { 0x11ea7da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba7da3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54da7da7, 98 },
    { 0xa86a7da8, 93 },
    { 0x11ba7da9, 33 },
    { 0, 0 },
    { 0xca6a7dab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87a7db2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x813a7db5, 96 },
    { 0, 0 },
    { 0xfe1a7db7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3a7dba, 23 },
    { 0x615a7dbb, 95 },
    { 0xb1ba7dbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08a7dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ca7dc2, 90 },
    { 0, 0 },
    { 0xdfba7dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a7dc7, 55 },
    { 0, 0 },
    { 0x14ca7dc9, 43 },
    { 0x128a7dc9, 90 },
    { 0x7b1a7dcb, 46 },
    { 0xf4aa7dcc, 90 },
    { 0x130a7dc9, 76 },
    { 0x511a7dce, 95 },
    { 0xa96a7dcf, 95 },
    { 0x2aa7dce, 91 },
    { 0xf4ca7dd0, 84 },
    { 0x192a7dc9, 61 },
    { 0, 0 },
    { 0x49ea7dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba7dd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa7ddd, 33 },
    { 0, 0 },
    { 0xa1da7ddf, 23 },
    { 0, 0 },
    { 0x3b9a7de1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a7de4, 55 },
    { 0x939a7de4, 99 },
    { 0xce2a7de6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259a7deb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea7df7, 33 },
    { 0x91ca7df8, 33 },
    { 0xc89a7df9, 99 },
    { 0, 0 },
    { 0xa00a7dfb, 95 },
    { 0, 0 },
    { 0x43ba7dfd, 90 },
    { 0, 0 },
    { 0xd93a7dff, 90 },
    { 0xaa8a7e00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6a7e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357a7e08, 33 },
    { 0xeb9a7e08, 101 },
    { 0x5c0a7e0a, 95 },
    { 0, 0 },
    { 0xcdea7e0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9a7e0f, 100 },
    { 0, 0 },
    { 0x178a7e11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda7e1b, 97 },
    { 0xb8ca7e1b, 76 },
    { 0xdcfa7e1d, 76 },
    { 0x270a7e1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba7e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa7e34, 90 },
    { 0, 0 },
    { 0xc42a7e36, 97 },
    { 0xeafa7e37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a7e3c, 61 },
    { 0xd97a7e3c, 46 },
    { 0, 0 },
    { 0x6dea7e3f, 43 },
    { 0, 0 },
    { 0x8a1a7e41, 76 },
    { 0xc94a7e42, 36 },
    { 0x222a7e43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ca7e49, 61 },
    { 0xf26a7e4a, 76 },
    { 0, 0 },
    { 0x114a7e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fa7e50, 33 },
    { 0, 0 },
    { 0x399a7e52, 55 },
    { 0, 0 },
    { 0xf94a7e54, 33 },
    { 0xf4aa7e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33a7e5b, 84 },
    { 0, 0 },
    { 0x155a7e5d, 61 },
    { 0x9eda7e5e, 33 },
    { 0, 0 },
    { 0xabea7e60, 90 },
    { 0, 0 },
    { 0xe8da7e62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda7e6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a7e70, 46 },
    { 0xf2ba7e70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90a7e79, 33 },
    { 0x169a7e79, 23 },
    { 0x50da7e7b, 100 },
    { 0x4aaa7e79, 23 },
    { 0xedaa7e7d, 61 },
    { 0xa1ea7e7e, 76 },
    { 0x59da7e7e, 61 },
    { 0x8fba7e7e, 95 },
    { 0xe18a7e80, 61 },
    { 0xd6fa7e7c, 98 },
    { 0xe89a7e83, 100 },
    { 0xa85a7e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a7e8a, 95 },
    { 0x961a7e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7e90, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4a7e94, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a7e98, 23 },
    { 0x6c3a7e99, 90 },
    { 0xc08a7e9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a7e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a7ea4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656a7ea9, 34 },
    { 0, 0 },
    { 0xdbda7eab, 46 },
    { 0x8e2a7eac, 33 },
    { 0, 0 },
    { 0xd69a7eae, 95 },
    { 0, 0 },
    { 0xe45a7eb0, 96 },
    { 0x1a1a7eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0aa7eb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a7eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea7ec0, 90 },
    { 0, 0 },
    { 0x693a7ec2, 23 },
    { 0x5cba7ec3, 33 },
    { 0, 0 },
    { 0x52aa7ec5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a7eca, 23 },
    { 0, 0 },
    { 0x109a7ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549a7ecf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0a7ed3, 40 },
    { 0xa6ca7ed4, 23 },
    { 0, 0 },
    { 0x95ba7ed6, 23 },
    { 0x8d3a7ed6, 33 },
    { 0x437a7ed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6a7edb, 66 },
    { 0xdbea7edc, 98 },
    { 0, 0 },
    { 0xdc7a7ede, 85 },
    { 0, 0 },
    { 0xae6a7ee0, 33 },
    { 0x94ba7ee1, 61 },
    { 0, 0 },
    { 0x9c0a7ee3, 90 },
    { 0, 0 },
    { 0x3fca7ee5, 78 },
    { 0xe63a7ee6, 90 },
    { 0x667a7ee5, 23 },
    { 0x66fa7ee8, 23 },
    { 0x33ea7ee8, 33 },
    { 0, 0 },
    { 0x32ca7eeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8a7ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca7efc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a7f00, 98 },
    { 0x567a7f00, 90 },
    { 0, 0 },
    { 0x216a7f03, 88 },
    { 0x516a7f03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31a7f0b, 91 },
    { 0x4d2a7f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba7f10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a7f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea7f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43aa7f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea7f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x992a7f2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38a7f2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29a7f35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a7f39, 93 },
    { 0x8b1a7f3a, 91 },
    { 0xf8ca7f3b, 66 },
    { 0x98ca7f3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a7f3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da7f45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x761a7f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aba7f4b, 52 },
    { 0xe33a7f4c, 33 },
    { 0x595a7f4d, 90 },
    { 0x75fa7f4e, 55 },
    { 0x6c3a7f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a7f55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a7f5c, 95 },
    { 0x342a7f5c, 75 },
    { 0, 0 },
    { 0xddba7f5f, 77 },
    { 0xee1a7f60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952a7f64, 95 },
    { 0x4c9a7f65, 9 },
    { 0xf81a7f66, 61 },
    { 0x897a7f64, 66 },
    { 0xce6a7f64, 76 },
    { 0x9f5a7f66, 61 },
    { 0x215a7f6a, 95 },
    { 0x77aa7f6b, 95 },
    { 0x4ffa7f6b, 55 },
    { 0xbbda7f64, 43 },
    { 0xc5aa7f6e, 33 },
    { 0x6fca7f6f, 90 },
    { 0x1faa7f70, 90 },
    { 0x601a7f71, 34 },
    { 0xb17a7f6c, 33 },
    { 0x67fa7f73, 95 },
    { 0x616a7f74, 33 },
    { 0x939a7f73, 90 },
    { 0x777a7f67, 61 },
    { 0xdf8a7f68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa7f7d, 100 },
    { 0x197a7f7e, 23 },
    { 0x476a7f7d, 67 },
    { 0xbfba7f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a7f83, 76 },
    { 0x3eba7f84, 61 },
    { 0, 0 },
    { 0xd2ca7f86, 55 },
    { 0, 0 },
    { 0xbf7a7f88, 55 },
    { 0, 0 },
    { 0xbc2a7f8a, 91 },
    { 0, 0 },
    { 0x126a7f8c, 76 },
    { 0xc49a7f8d, 84 },
    { 0x343a7f8e, 90 },
    { 0xacca7f8f, 66 },
    { 0x9f8a7f8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a7f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a7fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fca7fa3, 52 },
    { 0x4d6a7fa4, 23 },
    { 0, 0 },
    { 0x6f4a7fa6, 33 },
    { 0x66ca7fa7, 55 },
    { 0xc1da7fa7, 76 },
    { 0x855a7fa7, 80 },
    { 0x5caa7fa9, 95 },
    { 0, 0 },
    { 0xb7ca7fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea7faf, 61 },
    { 0x351a7faf, 55 },
    { 0, 0 },
    { 0x946a7fb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1a7fb8, 90 },
    { 0x67da7fb9, 55 },
    { 0x544a7fb9, 61 },
    { 0x624a7fb8, 61 },
    { 0x245a7fb9, 55 },
    { 0, 0 },
    { 0xc51a7fbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea7fc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba7fce, 33 },
    { 0xfc7a7fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a7fd5, 61 },
    { 0x166a7fd6, 55 },
    { 0, 0 },
    { 0xcc8a7fd8, 17 },
    { 0, 0 },
    { 0x7c1a7fda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda7fde, 69 },
    { 0xc2ba7fdf, 90 },
    { 0, 0 },
    { 0x43fa7fe1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a7fe6, 33 },
    { 0, 0 },
    { 0x2c1a7fe8, 90 },
    { 0x1b2a7fe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3a7ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a7ff9, 23 },
    { 0, 0 },
    { 0x81aa7ffb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fba8003, 61 },
    { 0x5d9a8004, 34 },
    { 0x1cea8005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928a800b, 53 },
    { 0, 0 },
    { 0x68fa800d, 93 },
    { 0xa6ca800e, 17 },
    { 0xfea800f, 55 },
    { 0x9ca800f, 52 },
    { 0, 0 },
    { 0x354a8012, 23 },
    { 0x60ca8013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daa8016, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ca8020, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba8023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea8026, 96 },
    { 0, 0 },
    { 0x42ea8028, 90 },
    { 0x4b1a8029, 90 },
    { 0xe2fa802a, 55 },
    { 0x5dca8028, 90 },
    { 0x739a802c, 98 },
    { 0x415a802c, 76 },
    { 0x575a802a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123a8032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369a8037, 33 },
    { 0x231a8037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972a803b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba8045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba8048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa804b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fa804e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aa8051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa8056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a805a, 98 },
    { 0x42ca805b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00a8062, 33 },
    { 0x240a8062, 87 },
    { 0, 0 },
    { 0xb61a8065, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa806b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaa806e, 90 },
    { 0xac1a806f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a8074, 76 },
    { 0x5efa8074, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a8079, 88 },
    { 0xdbfa807a, 90 },
    { 0xe8da807b, 90 },
    { 0, 0 },
    { 0x131a807d, 90 },
    { 0xf6ca807e, 55 },
    { 0xafca807d, 17 },
    { 0xe2ea8080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a8083, 23 },
    { 0, 0 },
    { 0x65fa8085, 61 },
    { 0x798a8086, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a8089, 78 },
    { 0x7e5a808a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a8090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda8093, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x613a8096, 34 },
    { 0x316a8096, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19da809c, 61 },
    { 0xe5fa809d, 98 },
    { 0x6baa809e, 95 },
    { 0x8f8a809d, 90 },
    { 0xcc2a80a0, 87 },
    { 0x4a6a80a1, 33 },
    { 0x839a80a1, 48 },
    { 0x3f9a80a3, 33 },
    { 0x635a80a4, 33 },
    { 0xb9fa80a5, 61 },
    { 0x19fa80a6, 34 },
    { 0x73da80a7, 55 },
    { 0x7fca809f, 67 },
    { 0x28ca80a9, 90 },
    { 0x1a6a80aa, 33 },
    { 0x52ba80ab, 95 },
    { 0x4f9a80ab, 23 },
    { 0x3cca80ad, 61 },
    { 0xac9a80a9, 90 },
    { 0, 0 },
    { 0xc49a80b0, 66 },
    { 0xfa2a80b1, 85 },
    { 0x3b6a80b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a80b6, 98 },
    { 0xa2da80b7, 61 },
    { 0xd76a80b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a80c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca80c4, 23 },
    { 0x7e2a80c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfa80c8, 43 },
    { 0xef1a80c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a80ce, 85 },
    { 0, 0 },
    { 0x92fa80d0, 100 },
    { 0xecaa80d0, 61 },
    { 0x41da80d2, 33 },
    { 0xcf0a80d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a80d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a80db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a80df, 95 },
    { 0xdd2a80e0, 98 },
    { 0x118a80e0, 98 },
    { 0xbd3a80e2, 85 },
    { 0x2a5a80e3, 90 },
    { 0, 0 },
    { 0xb53a80e5, 64 },
    { 0, 0 },
    { 0x3c6a80e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58a80ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600a80f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9a80f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a80f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627a80fe, 93 },
    { 0x81ba80ff, 90 },
    { 0x35ba8100, 90 },
    { 0x15ea8101, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ea8104, 67 },
    { 0x7c0a8104, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a8108, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a810e, 61 },
    { 0x189a810f, 93 },
    { 0, 0 },
    { 0x79a8111, 33 },
    { 0x12da8111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca811b, 66 },
    { 0x196a811b, 52 },
    { 0xb7da811d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ba8121, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa8126, 61 },
    { 0xccaa8127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999a812c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca812f, 90 },
    { 0xef5a812f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a8139, 90 },
    { 0x66aa8139, 77 },
    { 0x308a813a, 61 },
    { 0x998a813b, 98 },
    { 0, 0 },
    { 0xa72a813e, 93 },
    { 0x2a813e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589a8145, 61 },
    { 0xd21a8145, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba814a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79a8158, 67 },
    { 0, 0 },
    { 0x272a815a, 90 },
    { 0xfa6a815b, 98 },
    { 0, 0 },
    { 0x603a815d, 23 },
    { 0xd06a815e, 90 },
    { 0, 0 },
    { 0x749a8160, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba816d, 98 },
    { 0, 0 },
    { 0x9fca816f, 34 },
    { 0, 0 },
    { 0xba1a8171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4a8176, 55 },
    { 0x797a8177, 23 },
    { 0xc08a8178, 43 },
    { 0, 0 },
    { 0x1c2a817a, 33 },
    { 0xb05a817b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda8182, 33 },
    { 0x16aa8182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca8186, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427a818a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459a8190, 61 },
    { 0, 0 },
    { 0x970a8192, 55 },
    { 0, 0 },
    { 0xa1fa8194, 61 },
    { 0xa0aa8195, 36 },
    { 0xd80a8196, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79a819d, 78 },
    { 0, 0 },
    { 0x4bba819f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7a81a2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a81a6, 33 },
    { 0xc58a81a7, 98 },
    { 0x5dda81a8, 95 },
    { 0x20ba81a9, 95 },
    { 0xb63a81aa, 61 },
    { 0xf03a81a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaa81ae, 68 },
    { 0xfbba81af, 95 },
    { 0xa67a81b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4a81b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ba81bd, 33 },
    { 0, 0 },
    { 0xa07a81bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x665a81c2, 93 },
    { 0x628a81c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ca81c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41a81cc, 85 },
    { 0x45ba81cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a81d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794a81d6, 61 },
    { 0, 0 },
    { 0x185a81d8, 87 },
    { 0, 0 },
    { 0xd74a81da, 55 },
    { 0xf74a81da, 90 },
    { 0x2dca81dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x992a81df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fa81e4, 98 },
    { 0x297a81e4, 23 },
    { 0x327a81e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a81eb, 61 },
    { 0x48ea81eb, 90 },
    { 0, 0 },
    { 0xc89a81ee, 23 },
    { 0x7d4a81ef, 78 },
    { 0x5fca81ef, 99 },
    { 0xd9da81f1, 61 },
    { 0x661a81f2, 38 },
    { 0xecba81f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a81f6, 61 },
    { 0x2ffa81f7, 90 },
    { 0xbe2a81f7, 76 },
    { 0, 0 },
    { 0x6b4a81fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da8200, 23 },
    { 0x7c6a8201, 40 },
    { 0, 0 },
    { 0x9fca8203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a820e, 98 },
    { 0, 0 },
    { 0x422a8210, 98 },
    { 0x301a8210, 75 },
    { 0x2ba8212, 68 },
    { 0xd3a8212, 59 },
    { 0x615a8214, 33 },
    { 0x566a8215, 52 },
    { 0xfe7a8215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7a821a, 90 },
    { 0x879a821b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a8220, 23 },
    { 0, 0 },
    { 0xdf4a8222, 90 },
    { 0x8bfa8222, 99 },
    { 0xfcaa8222, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a822c, 17 },
    { 0, 0 },
    { 0x30a822e, 34 },
    { 0x1e9a822e, 75 },
    { 0xfe6a8230, 61 },
    { 0xfb3a8230, 61 },
    { 0x3a0a8232, 87 },
    { 0, 0 },
    { 0xea2a8234, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaa8237, 98 },
    { 0xf27a8238, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ba8240, 65 },
    { 0x82ca8241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x218a8244, 66 },
    { 0x3d9a8244, 36 },
    { 0, 0 },
    { 0x4b6a8247, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4a824d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a8252, 96 },
    { 0x397a8253, 33 },
    { 0x1a0a8254, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da825e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a8262, 61 },
    { 0xf1ba8262, 76 },
    { 0, 0 },
    { 0xd94a8265, 46 },
    { 0xf4a8266, 33 },
    { 0xf55a8266, 61 },
    { 0x584a8268, 87 },
    { 0x616a8266, 76 },
    { 0x404a826a, 52 },
    { 0xf22a826a, 93 },
    { 0xc3a826b, 88 },
    { 0, 0 },
    { 0x317a826e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa8272, 75 },
    { 0xec3a8273, 85 },
    { 0xbc3a8272, 66 },
    { 0x4f5a8275, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fca8279, 90 },
    { 0, 0 },
    { 0xfb0a827b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aa827f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a8284, 33 },
    { 0x68fa8285, 100 },
    { 0x21aa8284, 23 },
    { 0x98a8286, 95 },
    { 0x114a8287, 55 },
    { 0x8e6a8289, 80 },
    { 0xb54a828a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaa8290, 43 },
    { 0x424a8291, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a8298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a829d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cba82a5, 90 },
    { 0, 0 },
    { 0x426a82a7, 34 },
    { 0, 0 },
    { 0x7a4a82a9, 33 },
    { 0x54ca82a9, 95 },
    { 0x1aa82aa, 61 },
    { 0x728a82a9, 55 },
    { 0x572a82ad, 66 },
    { 0, 0 },
    { 0xfada82af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda82b2, 90 },
    { 0x789a82b3, 100 },
    { 0x8fea82b3, 90 },
    { 0x698a82b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1a82b8, 95 },
    { 0, 0 },
    { 0xea9a82ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a82c0, 76 },
    { 0x1dda82c1, 43 },
    { 0x76ea82c2, 61 },
    { 0x6cca82c3, 90 },
    { 0, 0 },
    { 0x7da82c5, 95 },
    { 0xba9a82c6, 95 },
    { 0xe04a82c6, 33 },
    { 0x6b4a82c8, 38 },
    { 0x2d2a82c8, 98 },
    { 0xeb2a82ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ca82cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa82d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a82db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a82de, 34 },
    { 0xdc8a82df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ba82e8, 78 },
    { 0x619a82e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x945a82ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752a82f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x153a82f5, 33 },
    { 0xffa82f6, 95 },
    { 0, 0 },
    { 0x10ea82f8, 90 },
    { 0xac1a82f9, 90 },
    { 0, 0 },
    { 0xe93a82fb, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcea8301, 23 },
    { 0, 0 },
    { 0xdf8a8303, 61 },
    { 0x779a8303, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24a830b, 87 },
    { 0, 0 },
    { 0xe89a830d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ea8316, 98 },
    { 0xff2a8317, 43 },
    { 0x402a8318, 93 },
    { 0, 0 },
    { 0x4f9a831a, 61 },
    { 0x9a5a831b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659a8324, 98 },
    { 0, 0 },
    { 0xb86a8326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a832c, 90 },
    { 0x68ea832d, 61 },
    { 0x151a832e, 61 },
    { 0xfb0a832c, 34 },
    { 0xcfaa832f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea8338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97a833c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x670a833f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a8344, 33 },
    { 0, 0 },
    { 0x7c0a8346, 95 },
    { 0x71aa8347, 75 },
    { 0, 0 },
    { 0xb41a8349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a834c, 33 },
    { 0x9daa834d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba8355, 91 },
    { 0, 0 },
    { 0xbea8357, 46 },
    { 0, 0 },
    { 0xad2a8359, 78 },
    { 0, 0 },
    { 0x8b1a835b, 23 },
    { 0x98aa835b, 17 },
    { 0xa9ca835d, 98 },
    { 0xa04a835e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efa8363, 87 },
    { 0xc6a8363, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a836a, 90 },
    { 0, 0 },
    { 0x13aa836c, 23 },
    { 0, 0 },
    { 0xa35a836e, 61 },
    { 0xd97a836f, 33 },
    { 0xcdaa8370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cea8376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x157a8379, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57a837d, 95 },
    { 0x705a837e, 52 },
    { 0x6dfa837d, 99 },
    { 0xd08a8380, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a8383, 76 },
    { 0x192a8384, 51 },
    { 0x92ca8385, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ca8388, 33 },
    { 0, 0 },
    { 0x466a838a, 95 },
    { 0, 0 },
    { 0xf57a838c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aa8391, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa8394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a8397, 61 },
    { 0x804a8398, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da839b, 98 },
    { 0x76aa839c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706a83a3, 90 },
    { 0x6b2a83a3, 95 },
    { 0x177a83a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8a83b1, 23 },
    { 0x886a83b2, 90 },
    { 0xb9fa83b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a83bd, 98 },
    { 0xa31a83be, 35 },
    { 0xec5a83bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4da83c2, 23 },
    { 0x570a83c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a83c6, 33 },
    { 0xc77a83c6, 67 },
    { 0, 0 },
    { 0x28aa83c9, 61 },
    { 0x59ba83ca, 90 },
    { 0x5a8a83ca, 98 },
    { 0, 0 },
    { 0x4bca83cd, 43 },
    { 0x526a83ce, 33 },
    { 0xc22a83cd, 83 },
    { 0x5eea83d0, 23 },
    { 0, 0 },
    { 0xb8a83d2, 95 },
    { 0x517a83d2, 95 },
    { 0x304a83d3, 90 },
    { 0xafda83d5, 95 },
    { 0xfa3a83d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83de, 90 },
    { 0x147a83df, 80 },
    { 0xc78a83e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a83e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea83e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfa83ed, 61 },
    { 0x6a3a83ed, 76 },
    { 0x911a83ef, 23 },
    { 0xb7ea83ee, 33 },
    { 0x1bfa83f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2a83f9, 90 },
    { 0x7bea83fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x702a83fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45a8404, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a840a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca8412, 88 },
    { 0, 0 },
    { 0x9da8414, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8a841a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaca841d, 95 },
    { 0x490a841e, 66 },
    { 0x693a841e, 46 },
    { 0xf12a841e, 99 },
    { 0xee8a841f, 95 },
    { 0xbeca8420, 17 },
    { 0x27ea8423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532a8426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a842c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ba8430, 95 },
    { 0x731a8431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a8434, 33 },
    { 0, 0 },
    { 0x590a8436, 76 },
    { 0xcf1a8437, 33 },
    { 0, 0 },
    { 0x87a8439, 43 },
    { 0, 0 },
    { 0xe37a843b, 61 },
    { 0x740a843b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a8440, 98 },
    { 0xa4a8440, 100 },
    { 0xae9a8440, 23 },
    { 0x1aca8442, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a844c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a8450, 90 },
    { 0xc43a8451, 95 },
    { 0x370a8452, 95 },
    { 0x606a8453, 67 },
    { 0x962a8453, 95 },
    { 0x18fa8452, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba845d, 98 },
    { 0x4bea845e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7a8461, 23 },
    { 0, 0 },
    { 0x58da8463, 77 },
    { 0xafba8463, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2a846b, 52 },
    { 0xbeea846c, 69 },
    { 0xf36a846d, 61 },
    { 0x386a846d, 61 },
    { 0xf51a846f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192a8473, 61 },
    { 0x621a8473, 55 },
    { 0x42a8474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba8478, 77 },
    { 0x640a8478, 98 },
    { 0x396a847a, 95 },
    { 0x1d8a847a, 90 },
    { 0xd54a847c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93a8481, 90 },
    { 0, 0 },
    { 0xb8ca8483, 99 },
    { 0xda5a8483, 93 },
    { 0x967a8484, 93 },
    { 0x869a8486, 2 },
    { 0, 0 },
    { 0xfa7a8488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12da848d, 91 },
    { 0x4fca848e, 23 },
    { 0xdeaa848e, 46 },
    { 0x11ea8490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cca8493, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a8497, 98 },
    { 0x41ea8497, 33 },
    { 0xc50a8498, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93a849c, 87 },
    { 0x82fa849d, 33 },
    { 0xcc1a849c, 84 },
    { 0x78fa849e, 38 },
    { 0xae3a84a0, 66 },
    { 0x95a84a0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa84a5, 61 },
    { 0x8b4a84a6, 83 },
    { 0xe72a84a6, 23 },
    { 0x552a84a8, 23 },
    { 0xbd4a84a6, 61 },
    { 0x3f8a84a6, 95 },
    { 0xd22a84a5, 93 },
    { 0x995a84ac, 90 },
    { 0xda0a84ad, 88 },
    { 0x37ba84ae, 50 },
    { 0x171a84ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3a84b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460a84b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51a84bd, 33 },
    { 0, 0 },
    { 0xcb6a84bf, 34 },
    { 0, 0 },
    { 0xa8ea84c1, 55 },
    { 0x827a84c2, 17 },
    { 0, 0 },
    { 0x22ca84c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a84c9, 61 },
    { 0, 0 },
    { 0x274a84cb, 55 },
    { 0x9e5a84cc, 33 },
    { 0xcc0a84cd, 23 },
    { 0, 0 },
    { 0xc94a84cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a84d3, 34 },
    { 0xfbfa84d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a84da, 88 },
    { 0x5bca84db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaea84e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a84e5, 98 },
    { 0, 0 },
    { 0x7b4a84e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea84ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x323a84ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca84f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736a84f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737a84f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8a84fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea8502, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aa8505, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ca850f, 95 },
    { 0xa6a850f, 90 },
    { 0, 0 },
    { 0x7c3a8512, 98 },
    { 0x6cca8512, 43 },
    { 0x731a8514, 40 },
    { 0x818a8514, 34 },
    { 0x790a8515, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a8520, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1a8526, 55 },
    { 0x16ea8527, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a8530, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca8535, 67 },
    { 0, 0 },
    { 0xb5ba8537, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4a853c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a853f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00a8542, 61 },
    { 0, 0 },
    { 0xc5a8544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa854d, 33 },
    { 0x205a854e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a8553, 33 },
    { 0xd3ba8554, 98 },
    { 0xf29a8553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136a855a, 90 },
    { 0x51a855b, 84 },
    { 0x57a855c, 90 },
    { 0x463a855d, 43 },
    { 0x2dda855d, 90 },
    { 0x3bba855f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ba8569, 55 },
    { 0x4a2a856a, 98 },
    { 0, 0 },
    { 0x9a856c, 61 },
    { 0xf2ca856d, 17 },
    { 0x3a3a856e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a8571, 95 },
    { 0, 0 },
    { 0x88ba8573, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1a857a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72a857e, 68 },
    { 0x3fda857e, 98 },
    { 0x756a8580, 95 },
    { 0, 0 },
    { 0x1d0a8582, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a8585, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba8589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfa858c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a8590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a8596, 90 },
    { 0xcaea8597, 61 },
    { 0x9e2a8598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea859b, 78 },
    { 0, 0 },
    { 0x4eda859d, 77 },
    { 0xba3a859e, 33 },
    { 0xd38a859f, 90 },
    { 0x5d1a859d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea85a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea85b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fa85b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da85be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa85c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea85c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa85cc, 90 },
    { 0x533a85cc, 33 },
    { 0x33ea85ce, 23 },
    { 0x604a85cf, 33 },
    { 0xe76a85cc, 33 },
    { 0x16ba85d1, 85 },
    { 0xe36a85d2, 61 },
    { 0x3bea85d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482a85dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a85e0, 61 },
    { 0x5f5a85e1, 34 },
    { 0, 0 },
    { 0x915a85e3, 33 },
    { 0, 0 },
    { 0x5b5a85e5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a85ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ba85f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x869a85f7, 33 },
    { 0x661a85f8, 69 },
    { 0x560a85f9, 84 },
    { 0, 0 },
    { 0xf43a85fb, 67 },
    { 0xf13a85fb, 95 },
    { 0xa15a85fc, 93 },
    { 0, 0 },
    { 0x1a9a85ff, 55 },
    { 0x410a8600, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbba8603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ada8606, 33 },
    { 0x503a8606, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba860a, 90 },
    { 0x4e0a860b, 34 },
    { 0xf6ba860c, 66 },
    { 0xe80a860d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca8611, 95 },
    { 0xc3a8611, 55 },
    { 0, 0 },
    { 0x135a8614, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fa8617, 84 },
    { 0x32fa8618, 90 },
    { 0xde8a8618, 55 },
    { 0x23fa861a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea861f, 61 },
    { 0xe5aa861f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a8626, 52 },
    { 0xfea8627, 90 },
    { 0x7eca8628, 66 },
    { 0x13aa8629, 90 },
    { 0x470a8629, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a862d, 76 },
    { 0xb3ba862e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a8633, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463a863a, 88 },
    { 0, 0 },
    { 0x7aa863c, 43 },
    { 0x1a4a863c, 90 },
    { 0x34ea863c, 95 },
    { 0x90a863f, 55 },
    { 0x817a863e, 84 },
    { 0x329a8641, 84 },
    { 0x258a8642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3a8646, 61 },
    { 0x80aa8646, 46 },
    { 0xc8a8647, 90 },
    { 0xb39a8649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a864d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a8658, 61 },
    { 0, 0 },
    { 0x60a865a, 84 },
    { 0xc74a865a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7a8661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea8666, 43 },
    { 0x5baa8667, 98 },
    { 0, 0 },
    { 0x34fa8669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cca866c, 52 },
    { 0x90fa866c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfba8671, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a867a, 61 },
    { 0, 0 },
    { 0xc9da867c, 43 },
    { 0x3cba867d, 33 },
    { 0xabba867e, 41 },
    { 0x131a867c, 90 },
    { 0xbaca867f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa8685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca8689, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4a868c, 23 },
    { 0, 0 },
    { 0x763a868e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba8693, 23 },
    { 0x5d4a8694, 84 },
    { 0, 0 },
    { 0x33ca8696, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceca869b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a86a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5a86a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a86af, 33 },
    { 0xb04a86b0, 90 },
    { 0x36a86b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a86b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a86bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9a86bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98da86c2, 33 },
    { 0, 0 },
    { 0x152a86c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a86c7, 90 },
    { 0, 0 },
    { 0x471a86c9, 77 },
    { 0xe6ca86ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a86cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9a86d7, 88 },
    { 0x5c5a86d7, 81 },
    { 0xfd5a86d9, 87 },
    { 0xa30a86d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4a86e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66a86e6, 12 },
    { 0, 0 },
    { 0xe8da86e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a86f4, 76 },
    { 0x567a86f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fa86f8, 33 },
    { 0xeb4a86f9, 94 },
    { 0x7eda86f9, 23 },
    { 0xd5ea86fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da8702, 61 },
    { 0, 0 },
    { 0xd9ca8704, 75 },
    { 0xcd3a8704, 90 },
    { 0, 0 },
    { 0x8baa8707, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da870a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264a870e, 64 },
    { 0x73ea870f, 33 },
    { 0x313a870f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a8717, 61 },
    { 0x993a8717, 34 },
    { 0x9e2a8719, 46 },
    { 0x74aa871a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a871e, 95 },
    { 0, 0 },
    { 0x4c3a8720, 90 },
    { 0x330a8721, 43 },
    { 0x926a8722, 95 },
    { 0x848a8723, 55 },
    { 0xbaa8724, 98 },
    { 0x424a8725, 98 },
    { 0x2fa8726, 95 },
    { 0x510a8726, 95 },
    { 0, 0 },
    { 0x879a8729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a872d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a8732, 90 },
    { 0xe52a8732, 100 },
    { 0x46aa8734, 87 },
    { 0xf57a8734, 23 },
    { 0, 0 },
    { 0xb24a8737, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a873a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a873d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5a8741, 33 },
    { 0xdb9a8741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faa8747, 23 },
    { 0, 0 },
    { 0x202a8749, 43 },
    { 0x93ea874a, 55 },
    { 0x539a8749, 95 },
    { 0x9aea8749, 98 },
    { 0x68ea874d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1a8752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ea8757, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210a875b, 33 },
    { 0x939a875b, 88 },
    { 0x8a0a875b, 34 },
    { 0xa23a875e, 69 },
    { 0x84a875c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0a8763, 55 },
    { 0xad1a8764, 90 },
    { 0xa74a8765, 61 },
    { 0xd2ba8766, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a876d, 90 },
    { 0, 0 },
    { 0x78aa876f, 98 },
    { 0xbf2a8770, 98 },
    { 0, 0 },
    { 0xcb3a8772, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa8778, 90 },
    { 0xc36a8779, 55 },
    { 0, 0 },
    { 0x153a877b, 61 },
    { 0xb6fa877c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faa8781, 85 },
    { 0x776a8781, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a878b, 91 },
    { 0xa82a878c, 33 },
    { 0x9ea878d, 99 },
    { 0x6c6a878b, 90 },
    { 0xd24a878c, 76 },
    { 0x206a878e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da8795, 56 },
    { 0, 0 },
    { 0xaaa8797, 85 },
    { 0x745a8798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa879e, 99 },
    { 0, 0 },
    { 0x137a87a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a87a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba87aa, 78 },
    { 0x6a4a87aa, 95 },
    { 0x578a87ab, 46 },
    { 0x702a87ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a87b1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bea87b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a87b7, 69 },
    { 0, 0 },
    { 0x2bea87b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2a87bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a87c0, 93 },
    { 0xe55a87c0, 23 },
    { 0, 0 },
    { 0xbeea87c3, 88 },
    { 0x251a87c4, 33 },
    { 0x888a87c4, 33 },
    { 0, 0 },
    { 0x5cca87c7, 88 },
    { 0, 0 },
    { 0xb1fa87c9, 33 },
    { 0x27ba87ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa87d0, 90 },
    { 0x2dca87d1, 61 },
    { 0, 0 },
    { 0x28aa87d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a87d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4a87df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dca87e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0a87e8, 95 },
    { 0, 0 },
    { 0xd38a87ea, 69 },
    { 0xc67a87eb, 55 },
    { 0xb8a87ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aa87f2, 98 },
    { 0xb29a87f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa87f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fa87fb, 61 },
    { 0xdb1a87fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0a8805, 90 },
    { 0x376a8806, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da8813, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a8819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aa881d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1a8820, 52 },
    { 0x442a8820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a8824, 61 },
    { 0, 0 },
    { 0x795a8826, 98 },
    { 0x9c4a8827, 61 },
    { 0, 0 },
    { 0x53a8829, 90 },
    { 0xce0a8829, 76 },
    { 0xde9a882b, 95 },
    { 0x64fa882c, 46 },
    { 0xd09a882c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3a8843, 43 },
    { 0x301a8844, 99 },
    { 0xe16a8844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a8848, 77 },
    { 0, 0 },
    { 0xa83a884a, 23 },
    { 0, 0 },
    { 0x9f9a884c, 90 },
    { 0, 0 },
    { 0x14aa884e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dda8851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2a8855, 55 },
    { 0x637a8856, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba8859, 33 },
    { 0x959a885a, 98 },
    { 0x516a8859, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8864, 23 },
    { 0x5e0a8864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aa8868, 33 },
    { 0x7cda8868, 100 },
    { 0xb78a886a, 90 },
    { 0x483a8868, 93 },
    { 0xae7a886c, 55 },
    { 0x675a886d, 90 },
    { 0x360a886e, 33 },
    { 0x7e9a8868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaa8874, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a887b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ea887e, 33 },
    { 0x96a887f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299a8885, 95 },
    { 0x53da8886, 90 },
    { 0x1b5a8885, 33 },
    { 0x31aa8887, 55 },
    { 0x678a8889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872a888d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722a8892, 61 },
    { 0xba6a8893, 90 },
    { 0x856a8894, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a8897, 34 },
    { 0, 0 },
    { 0x818a8899, 61 },
    { 0x28a8899, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a88a1, 95 },
    { 0xf34a88a2, 90 },
    { 0xbb8a88a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca88a6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778a88ad, 91 },
    { 0xdd5a88ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea88b7, 90 },
    { 0, 0 },
    { 0x894a88b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a88bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95da88c2, 55 },
    { 0xf79a88c3, 99 },
    { 0, 0 },
    { 0xb25a88c5, 98 },
    { 0x8fba88c6, 23 },
    { 0x8f2a88c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda88ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a88d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a88d5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3a88d8, 52 },
    { 0x53ea88d8, 68 },
    { 0, 0 },
    { 0x4baa88db, 95 },
    { 0xd3ba88dc, 88 },
    { 0x1d5a88dd, 33 },
    { 0x92da88de, 90 },
    { 0, 0 },
    { 0x510a88e0, 91 },
    { 0x2d1a88e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fa88e5, 23 },
    { 0, 0 },
    { 0xe22a88e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x571a88ea, 93 },
    { 0x9a4a88eb, 100 },
    { 0x734a88ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefa88ef, 78 },
    { 0x78ca88f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a88f6, 83 },
    { 0, 0 },
    { 0xdf1a88f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a88fb, 55 },
    { 0x335a88fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7a8901, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a8904, 90 },
    { 0xaa8a8904, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aa8908, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da890b, 95 },
    { 0, 0 },
    { 0xb54a890d, 55 },
    { 0, 0 },
    { 0x76da890f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda8914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81da8919, 77 },
    { 0xba1a891a, 76 },
    { 0, 0 },
    { 0x665a891c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88a8921, 98 },
    { 0x82ea8922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a8926, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ca892e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x905a8931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa8938, 33 },
    { 0, 0 },
    { 0xecaa893a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192a893d, 61 },
    { 0x86aa893e, 95 },
    { 0x799a893f, 33 },
    { 0x17da8940, 95 },
    { 0, 0 },
    { 0x963a8942, 34 },
    { 0, 0 },
    { 0xb08a8944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da894b, 66 },
    { 0xda0a894c, 23 },
    { 0xc48a894d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242a8951, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1a8955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81a8958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa895c, 30 },
    { 0x89ea895d, 61 },
    { 0xdcca895e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839a8962, 87 },
    { 0x79da8963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a8969, 61 },
    { 0x940a896a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a896f, 68 },
    { 0xafea8970, 93 },
    { 0xfe1a8971, 46 },
    { 0x8fa8972, 76 },
    { 0x334a8973, 23 },
    { 0x118a8971, 55 },
    { 0x8e1a8971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0a897b, 33 },
    { 0, 0 },
    { 0x6b4a897d, 90 },
    { 0, 0 },
    { 0x4e1a897f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89a8988, 23 },
    { 0, 0 },
    { 0x2eba898a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba8990, 61 },
    { 0xaf5a8991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afa8996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aa8999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3a899d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3a89a6, 61 },
    { 0xe06a89a7, 98 },
    { 0x268a89a7, 90 },
    { 0x4d2a89a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a89ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7a89b1, 76 },
    { 0xd86a89b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624a89b9, 33 },
    { 0x5aba89b9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa89bf, 77 },
    { 0xc10a89c0, 100 },
    { 0, 0 },
    { 0x3d4a89c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a89c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ea89ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba89cd, 91 },
    { 0x656a89ce, 85 },
    { 0x510a89cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2a89d5, 33 },
    { 0, 0 },
    { 0x5e2a89d7, 33 },
    { 0x561a89d7, 90 },
    { 0x743a89d9, 76 },
    { 0x329a89d7, 95 },
    { 0x709a89db, 93 },
    { 0xd9aa89dc, 43 },
    { 0xf23a89dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ca89e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a89e5, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffa89e9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca89f0, 61 },
    { 0x41ea89f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fa89f4, 90 },
    { 0xb35a89f4, 95 },
    { 0xb27a89f6, 69 },
    { 0x410a89f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa8a00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fba8a03, 90 },
    { 0, 0 },
    { 0xd9ea8a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5da8a12, 61 },
    { 0xadba8a13, 99 },
    { 0x779a8a14, 95 },
    { 0x48da8a14, 68 },
    { 0x9aca8a16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba8a19, 61 },
    { 0, 0 },
    { 0x537a8a1b, 33 },
    { 0x246a8a1c, 46 },
    { 0xb72a8a1d, 46 },
    { 0x342a8a1e, 68 },
    { 0x16ca8a1e, 95 },
    { 0xfa5a8a20, 98 },
    { 0xba1a8a1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a8a26, 61 },
    { 0x8fda8a27, 53 },
    { 0x3faa8a28, 23 },
    { 0x511a8a28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a8a2c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa8a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79a8a33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420a8a3a, 76 },
    { 0x17ca8a3b, 36 },
    { 0x585a8a3c, 61 },
    { 0x471a8a3d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350a8a41, 61 },
    { 0, 0 },
    { 0xe04a8a43, 53 },
    { 0x85ea8a44, 34 },
    { 0x536a8a43, 95 },
    { 0xaa9a8a46, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a8a4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a8a51, 46 },
    { 0, 0 },
    { 0xc27a8a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778a8a57, 93 },
    { 0, 0 },
    { 0xe60a8a59, 97 },
    { 0, 0 },
    { 0x52fa8a5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x248a8a5e, 78 },
    { 0, 0 },
    { 0xceda8a60, 81 },
    { 0x749a8a60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda8a6a, 90 },
    { 0x4c4a8a6a, 33 },
    { 0x4dea8a6c, 55 },
    { 0x2a3a8a6d, 66 },
    { 0x208a8a6e, 90 },
    { 0x975a8a6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a8a76, 76 },
    { 0x809a8a77, 61 },
    { 0, 0 },
    { 0xebca8a79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa8a7d, 38 },
    { 0, 0 },
    { 0x15da8a7f, 34 },
    { 0, 0 },
    { 0xe36a8a81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a8a89, 99 },
    { 0, 0 },
    { 0x3b9a8a8b, 90 },
    { 0x214a8a8b, 93 },
    { 0xab7a8a8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ca8a91, 61 },
    { 0x641a8a91, 61 },
    { 0x860a8a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a8a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04a8a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ea8a9d, 95 },
    { 0x1eca8a9e, 91 },
    { 0xb3ba8a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca8aa3, 75 },
    { 0x418a8aa4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915a8aa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a8aac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfa8aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa8ab2, 90 },
    { 0x884a8ab3, 98 },
    { 0xa65a8ab4, 55 },
    { 0x144a8ab5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa8abb, 61 },
    { 0, 0 },
    { 0xc84a8abd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a8ac1, 41 },
    { 0x2a6a8ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a8ac9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ca8acc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a8ad0, 99 },
    { 0, 0 },
    { 0x8caa8ad2, 33 },
    { 0xedfa8ad2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7a8ad6, 43 },
    { 0, 0 },
    { 0x159a8ad8, 95 },
    { 0x523a8ad9, 34 },
    { 0x6d5a8ada, 93 },
    { 0xb17a8adb, 33 },
    { 0xdc2a8adb, 33 },
    { 0x775a8adc, 90 },
    { 0xff7a8ad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ba8ae1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892a8ae5, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dda8aec, 34 },
    { 0x52ca8aed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x845a8af0, 55 },
    { 0, 0 },
    { 0x930a8af2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea8af7, 61 },
    { 0xf8ca8af8, 76 },
    { 0x947a8af9, 61 },
    { 0xd92a8af7, 90 },
    { 0, 0 },
    { 0x9aa8afc, 95 },
    { 0xa9fa8afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca8b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47a8b03, 93 },
    { 0, 0 },
    { 0xba6a8b05, 66 },
    { 0, 0 },
    { 0x1d5a8b07, 46 },
    { 0x4c1a8b08, 95 },
    { 0xd9fa8b08, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2a8b12, 23 },
    { 0x174a8b13, 43 },
    { 0, 0 },
    { 0x634a8b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da8b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x896a8b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea8b23, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda8b26, 33 },
    { 0xc09a8b27, 33 },
    { 0, 0 },
    { 0x6fca8b29, 99 },
    { 0, 0 },
    { 0x518a8b2b, 55 },
    { 0, 0 },
    { 0x20ea8b2d, 66 },
    { 0xb0da8b2e, 77 },
    { 0x29da8b2e, 33 },
    { 0xaefa8b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2a8b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8a8b3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a8b42, 46 },
    { 0x8f2a8b43, 34 },
    { 0x78da8b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315a8b48, 93 },
    { 0xb95a8b48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a8b4d, 95 },
    { 0x79ba8b4e, 23 },
    { 0x1d1a8b4e, 95 },
    { 0x198a8b50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaa8b53, 46 },
    { 0x290a8b54, 33 },
    { 0x4aaa8b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a8b5c, 85 },
    { 0xb13a8b5c, 61 },
    { 0, 0 },
    { 0xb33a8b5f, 61 },
    { 0, 0 },
    { 0x231a8b61, 84 },
    { 0x8aca8b62, 33 },
    { 0x64a8b63, 100 },
    { 0x3da8b63, 33 },
    { 0, 0 },
    { 0x2cda8b66, 61 },
    { 0xb88a8b67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffba8b6a, 33 },
    { 0xc3a8b6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eea8b6e, 98 },
    { 0x863a8b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfa8b73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea8b79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3a8b7e, 23 },
    { 0, 0 },
    { 0xce0a8b80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8a8b84, 52 },
    { 0x3eba8b85, 76 },
    { 0x31ea8b86, 98 },
    { 0x87fa8b87, 90 },
    { 0, 0 },
    { 0x9ca8b89, 38 },
    { 0xe2a8b8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bea8b8d, 66 },
    { 0x978a8b8e, 95 },
    { 0x3a1a8b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa8b99, 33 },
    { 0x405a8b9a, 23 },
    { 0, 0 },
    { 0x82da8b9c, 98 },
    { 0xa4da8b9d, 90 },
    { 0xb0da8b9e, 84 },
    { 0xb6ca8b9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea8ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ba8bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdca8bb2, 34 },
    { 0, 0 },
    { 0xf9ea8bb4, 96 },
    { 0x98a8bb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0a8bbb, 77 },
    { 0x552a8bbc, 84 },
    { 0xfdaa8bbd, 69 },
    { 0xd2ea8bbb, 95 },
    { 0x907a8bbe, 76 },
    { 0xdb8a8bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eba8bc5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545a8bcb, 90 },
    { 0xa6da8bcc, 23 },
    { 0x5fca8bcb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888a8bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a8bd8, 34 },
    { 0, 0 },
    { 0xa04a8bda, 95 },
    { 0xb5fa8bda, 33 },
    { 0, 0 },
    { 0x5fa8bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60da8be1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85a8be5, 49 },
    { 0x894a8be6, 78 },
    { 0x6fca8be6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a8bec, 91 },
    { 0, 0 },
    { 0x6cba8bee, 33 },
    { 0x9eda8bef, 98 },
    { 0x9c9a8bf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8a8bf6, 98 },
    { 0, 0 },
    { 0x719a8bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a8bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ba8bfe, 52 },
    { 0xd44a8bff, 66 },
    { 0xc2fa8bfe, 33 },
    { 0x416a8c01, 33 },
    { 0x272a8c02, 95 },
    { 0x11a8c03, 66 },
    { 0x545a8c03, 34 },
    { 0xc5ca8c04, 43 },
    { 0x29aa8c06, 65 },
    { 0x29ca8c06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9da8c0a, 34 },
    { 0x11ea8c0b, 95 },
    { 0x7a4a8c0c, 90 },
    { 0xbd7a8c0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583a8c11, 91 },
    { 0xc6ca8c12, 33 },
    { 0, 0 },
    { 0x6eea8c14, 76 },
    { 0, 0 },
    { 0x6eda8c16, 81 },
    { 0xa17a8c17, 33 },
    { 0, 0 },
    { 0xf5ea8c19, 76 },
    { 0x964a8c19, 90 },
    { 0x502a8c1b, 65 },
    { 0, 0 },
    { 0xe97a8c1d, 90 },
    { 0, 0 },
    { 0x3b4a8c1f, 55 },
    { 0x21a8c20, 23 },
    { 0, 0 },
    { 0x908a8c22, 85 },
    { 0x81ba8c23, 85 },
    { 0x2baa8c23, 61 },
    { 0x47fa8c23, 43 },
    { 0x992a8c26, 61 },
    { 0x5da8c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a8c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7a8c35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4a8c39, 33 },
    { 0x769a8c3a, 43 },
    { 0x2d6a8c3a, 61 },
    { 0xd0ba8c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7a8c3f, 66 },
    { 0, 0 },
    { 0x306a8c41, 23 },
    { 0, 0 },
    { 0x636a8c43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354a8c47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21a8c4e, 90 },
    { 0xfcba8c4e, 95 },
    { 0x99a8c4f, 95 },
    { 0, 0 },
    { 0xa51a8c52, 98 },
    { 0xa12a8c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476a8c59, 66 },
    { 0, 0 },
    { 0xd05a8c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efa8c5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a8c64, 76 },
    { 0, 0 },
    { 0x4a4a8c66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a8c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1a8c79, 95 },
    { 0x74a8c7a, 61 },
    { 0xda8c7a, 23 },
    { 0x76ca8c79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183a8c82, 43 },
    { 0, 0 },
    { 0xc69a8c84, 46 },
    { 0xaa7a8c85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a8c8a, 61 },
    { 0xf63a8c8b, 95 },
    { 0x5a7a8c8c, 82 },
    { 0xc38a8c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45da8c90, 99 },
    { 0x7a3a8c91, 23 },
    { 0xa69a8c92, 90 },
    { 0x2f6a8c90, 93 },
    { 0x3aca8c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caa8c98, 87 },
    { 0xf91a8c98, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5a8ca0, 90 },
    { 0xa43a8ca1, 23 },
    { 0xeea8ca2, 66 },
    { 0xf90a8ca1, 61 },
    { 0xf05a8ca4, 85 },
    { 0xfffa8ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca8ca8, 34 },
    { 0, 0 },
    { 0x680a8caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeda8cb7, 33 },
    { 0x4c5a8cb8, 52 },
    { 0x844a8cb9, 34 },
    { 0xbc9a8cba, 23 },
    { 0x5f4a8cb8, 88 },
    { 0xb49a8cbb, 93 },
    { 0x723a8cb9, 95 },
    { 0x84ba8cbb, 23 },
    { 0, 0 },
    { 0x9cba8cc0, 90 },
    { 0xcaaa8cc1, 95 },
    { 0, 0 },
    { 0xfc1a8cc3, 23 },
    { 0, 0 },
    { 0x6f5a8cc5, 95 },
    { 0x4f3a8cc6, 77 },
    { 0x83fa8cc7, 98 },
    { 0x559a8cc8, 91 },
    { 0x352a8cc5, 66 },
    { 0xc8ea8cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a8cd1, 61 },
    { 0xaa6a8cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4da8cd6, 61 },
    { 0xddca8cd7, 23 },
    { 0xbbfa8cd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a8cdc, 100 },
    { 0x425a8cdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61a8ce1, 33 },
    { 0, 0 },
    { 0x784a8ce3, 91 },
    { 0, 0 },
    { 0xfcba8ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a8ce8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fa8cee, 61 },
    { 0xcd2a8cef, 61 },
    { 0x672a8cee, 34 },
    { 0x875a8cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8a8cf5, 98 },
    { 0, 0 },
    { 0x86fa8cf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a8cfb, 33 },
    { 0, 0 },
    { 0x2c9a8cfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2a8d01, 76 },
    { 0xe0fa8d02, 23 },
    { 0xed8a8d03, 23 },
    { 0xf67a8d04, 90 },
    { 0x877a8d04, 33 },
    { 0x15a8d05, 98 },
    { 0x1b4a8d07, 55 },
    { 0xe77a8d08, 90 },
    { 0xe10a8d06, 98 },
    { 0xfe8a8d02, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfa8d13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a8d16, 90 },
    { 0x28ca8d17, 61 },
    { 0xb14a8d18, 33 },
    { 0x90ba8d18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9a8d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4a8d21, 90 },
    { 0, 0 },
    { 0x6f7a8d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca8d29, 55 },
    { 0, 0 },
    { 0xc4aa8d2b, 84 },
    { 0, 0 },
    { 0x463a8d2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33a8d31, 33 },
    { 0xe83a8d32, 76 },
    { 0x445a8d32, 40 },
    { 0xc9aa8d33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ba8d37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36a8d3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa8d45, 43 },
    { 0, 0 },
    { 0x56ca8d47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51a8d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1a8d4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ca8d52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa8d55, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aa8d58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6a8d5d, 67 },
    { 0, 0 },
    { 0x5f4a8d5f, 91 },
    { 0, 0 },
    { 0x33aa8d61, 100 },
    { 0, 0 },
    { 0xd93a8d63, 98 },
    { 0, 0 },
    { 0x4b1a8d65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8d68, 61 },
    { 0x7a4a8d69, 67 },
    { 0x876a8d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fa8d6e, 95 },
    { 0, 0 },
    { 0x17ca8d70, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aa8d77, 91 },
    { 0xf64a8d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358a8d7d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9a8d84, 23 },
    { 0x59ba8d84, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a8d88, 17 },
    { 0xa7ea8d88, 98 },
    { 0xb1da8d89, 46 },
    { 0x68ba8d8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a8d90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffa8d94, 66 },
    { 0, 0 },
    { 0x36ca8d96, 61 },
    { 0x40ba8d96, 90 },
    { 0, 0 },
    { 0xed4a8d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a8d9e, 98 },
    { 0xa18a8d9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba8da3, 81 },
    { 0x376a8da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a8da8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a8dac, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca8db2, 61 },
    { 0xb1fa8db3, 33 },
    { 0xd1a8db4, 76 },
    { 0, 0 },
    { 0x143a8db6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbea8dbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a8dbf, 90 },
    { 0xb8da8dbf, 33 },
    { 0x934a8dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x260a8dc4, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ca8dd1, 90 },
    { 0x33ba8dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8dd6, 90 },
    { 0x8f1a8dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aa8dda, 33 },
    { 0, 0 },
    { 0x9a8ddc, 95 },
    { 0x6ca8ddd, 61 },
    { 0x1e4a8dde, 33 },
    { 0, 0 },
    { 0x5cea8de0, 93 },
    { 0x8cda8de1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a8de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a8de9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca8dee, 46 },
    { 0x436a8def, 23 },
    { 0, 0 },
    { 0xf0ca8df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a8df5, 23 },
    { 0x1e2a8df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a8dfc, 90 },
    { 0xbeda8dfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fea8e01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8a8e08, 90 },
    { 0x91ca8e09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfa8e14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a8e1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a8e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a8e24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50da8e27, 56 },
    { 0, 0 },
    { 0xbd5a8e29, 23 },
    { 0, 0 },
    { 0xe57a8e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a8e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba8e33, 43 },
    { 0xff1a8e34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x703a8e37, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a8e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7a8e45, 95 },
    { 0x5da8e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a8e4b, 99 },
    { 0xe86a8e4b, 65 },
    { 0xa3ca8e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba8e53, 80 },
    { 0, 0 },
    { 0x7bfa8e55, 46 },
    { 0x380a8e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21a8e5a, 40 },
    { 0, 0 },
    { 0x62ea8e5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceda8e61, 19 },
    { 0xb66a8e62, 91 },
    { 0xc53a8e63, 33 },
    { 0, 0 },
    { 0xaf9a8e65, 55 },
    { 0, 0 },
    { 0x9caa8e67, 67 },
    { 0x20ca8e68, 33 },
    { 0xdffa8e68, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9a8e6f, 98 },
    { 0xc0fa8e70, 61 },
    { 0x6f0a8e70, 61 },
    { 0, 0 },
    { 0xb09a8e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84a8e80, 61 },
    { 0x7d9a8e81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a8e85, 93 },
    { 0xaaca8e86, 33 },
    { 0, 0 },
    { 0x706a8e88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762a8e8d, 55 },
    { 0, 0 },
    { 0x3b5a8e8f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea8e93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a8e96, 95 },
    { 0x888a8e96, 76 },
    { 0xf2aa8e98, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a8e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa8ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a8ea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca8ead, 93 },
    { 0xd54a8eae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa8eb2, 46 },
    { 0x1e1a8eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2a8eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825a8ebb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8ebe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a8ec2, 81 },
    { 0xc09a8ec3, 46 },
    { 0xf8ea8ec4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ca8ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a8ecc, 98 },
    { 0xb6da8ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a8ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a8ed3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cea8ed9, 81 },
    { 0xdc2a8eda, 98 },
    { 0xfd6a8ed9, 46 },
    { 0x1a4a8edc, 98 },
    { 0x75ba8edd, 95 },
    { 0x4bea8edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a8ee6, 78 },
    { 0xc0a8ee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8a8eed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a8ef1, 33 },
    { 0xf56a8ef2, 23 },
    { 0x3faa8ef3, 66 },
    { 0, 0 },
    { 0xafda8ef5, 95 },
    { 0x18fa8ef6, 95 },
    { 0x383a8ef6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da8f00, 23 },
    { 0x2f0a8f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a8f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6a8f0a, 67 },
    { 0xd74a8f0a, 75 },
    { 0, 0 },
    { 0x1ffa8f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeca8f12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a8f17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaa8f1a, 93 },
    { 0x87a8f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fa8f26, 95 },
    { 0x27da8f26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ba8f2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a8f2d, 23 },
    { 0, 0 },
    { 0xcaaa8f2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885a8f34, 95 },
    { 0x873a8f35, 90 },
    { 0xc1a8f36, 23 },
    { 0x7b0a8f35, 95 },
    { 0x942a8f38, 90 },
    { 0xb2da8f36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a8f3c, 93 },
    { 0, 0 },
    { 0x4fa8f3e, 33 },
    { 0x27ca8f3f, 61 },
    { 0x134a8f3e, 98 },
    { 0x6dba8f41, 98 },
    { 0xccfa8f42, 33 },
    { 0x21ea8f41, 23 },
    { 0xa7da8f41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a8f47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a8f4a, 33 },
    { 0x551a8f4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a8f4e, 34 },
    { 0, 0 },
    { 0x2e1a8f50, 55 },
    { 0xb49a8f51, 33 },
    { 0xd4a8f52, 23 },
    { 0x753a8f53, 34 },
    { 0xfe6a8f54, 81 },
    { 0x620a8f55, 33 },
    { 0x630a8f56, 34 },
    { 0, 0 },
    { 0x6bca8f58, 90 },
    { 0x36ca8f58, 61 },
    { 0, 0 },
    { 0x6fea8f5b, 72 },
    { 0x9daa8f5c, 46 },
    { 0x9aea8f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a8f60, 33 },
    { 0x2dda8f61, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fa8f67, 91 },
    { 0x335a8f67, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37a8f6c, 95 },
    { 0x44ba8f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a8f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba8f76, 98 },
    { 0xe19a8f76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa8f7f, 88 },
    { 0, 0 },
    { 0x2a0a8f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa8f87, 23 },
    { 0x381a8f88, 90 },
    { 0x222a8f89, 76 },
    { 0x689a8f8a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9a8f90, 84 },
    { 0, 0 },
    { 0xe3aa8f92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ea8f96, 23 },
    { 0xde0a8f97, 23 },
    { 0x218a8f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca8f9b, 23 },
    { 0, 0 },
    { 0xf83a8f9d, 90 },
    { 0, 0 },
    { 0xe56a8f9f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a8fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570a8fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x797a8fab, 77 },
    { 0x312a8fac, 33 },
    { 0x8bda8fad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8a8fb4, 91 },
    { 0x4dfa8fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8a8fb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fa8fc0, 61 },
    { 0xa3da8fc1, 77 },
    { 0x385a8fc0, 90 },
    { 0xea2a8fc3, 94 },
    { 0x4eba8fc3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a8fcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8fd0, 33 },
    { 0x392a8fd1, 98 },
    { 0x123a8fd2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a8fdf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a8fe7, 34 },
    { 0, 0 },
    { 0x3cba8fe9, 76 },
    { 0xe3fa8fe9, 34 },
    { 0x74ba8feb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa8fee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a8ff3, 38 },
    { 0, 0 },
    { 0xaaea8ff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea8ffa, 42 },
    { 0xe6fa8ffb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a8fff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba9003, 81 },
    { 0, 0 },
    { 0xbd4a9005, 34 },
    { 0xa9ea9005, 81 },
    { 0, 0 },
    { 0x7bba9008, 33 },
    { 0x865a9009, 99 },
    { 0x7f5a900a, 75 },
    { 0x239a900a, 76 },
    { 0x4cda9009, 99 },
    { 0x4f1a9008, 90 },
    { 0x847a900e, 52 },
    { 0, 0 },
    { 0x3b4a9010, 81 },
    { 0xfc4a9011, 65 },
    { 0xf11a9012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3a9018, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a901c, 95 },
    { 0xf86a901c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a9020, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda9024, 86 },
    { 0xda2a9024, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baa9028, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a902f, 33 },
    { 0, 0 },
    { 0xd00a9031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474a9036, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5a9039, 90 },
    { 0x1ada903a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea9042, 65 },
    { 0xe32a9043, 23 },
    { 0x6a5a9043, 95 },
    { 0, 0 },
    { 0xa0fa9046, 95 },
    { 0x1cda9047, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0a904c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258a9055, 52 },
    { 0xd77a9055, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x929a9059, 43 },
    { 0x477a9059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ca905e, 90 },
    { 0, 0 },
    { 0xa37a9060, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a9066, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a906a, 77 },
    { 0, 0 },
    { 0xb3a906c, 34 },
    { 0xb60a906d, 95 },
    { 0x1e6a906e, 98 },
    { 0x6c5a906e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63a9072, 55 },
    { 0x76fa9073, 61 },
    { 0xfb2a9073, 98 },
    { 0, 0 },
    { 0x5a4a9076, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7a907f, 98 },
    { 0x853a907f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aa9086, 33 },
    { 0xe8ea9087, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a908a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a908d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856a9092, 17 },
    { 0x31fa9093, 90 },
    { 0x99da9092, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a9099, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404a90a0, 87 },
    { 0x759a90a1, 23 },
    { 0xf9ea90a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223a90a6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea90aa, 61 },
    { 0x94ba90ab, 33 },
    { 0, 0 },
    { 0xca3a90ad, 90 },
    { 0x971a90ad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a90b1, 88 },
    { 0xccca90b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa90b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14a90bc, 90 },
    { 0x6e1a90bd, 101 },
    { 0xb6ba90be, 40 },
    { 0xc2ba90be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bca90cc, 61 },
    { 0x450a90cd, 24 },
    { 0x72ba90ce, 90 },
    { 0x434a90cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a90d2, 76 },
    { 0x872a90d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a90d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ca90dc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3a90e0, 43 },
    { 0x577a90e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ba90e4, 34 },
    { 0x242a90e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a90e8, 64 },
    { 0x567a90e9, 61 },
    { 0x214a90ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda90ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fa90f1, 81 },
    { 0x327a90f2, 76 },
    { 0x693a90f2, 98 },
    { 0xe2fa90f4, 84 },
    { 0xb50a90f3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a90f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daa90fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa9101, 95 },
    { 0xb69a9102, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0a9106, 88 },
    { 0x171a9107, 43 },
    { 0, 0 },
    { 0xbe3a9109, 33 },
    { 0xb5fa910a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ca910d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a9117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09a911b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba911f, 23 },
    { 0x3e1a911f, 43 },
    { 0x30fa9120, 88 },
    { 0x400a9122, 52 },
    { 0xc00a911f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a9127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba912c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58a912f, 61 },
    { 0x782a912f, 90 },
    { 0, 0 },
    { 0xc03a9132, 33 },
    { 0x3b8a9133, 93 },
    { 0x2f9a9132, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x228a9137, 84 },
    { 0x754a9138, 65 },
    { 0x9f6a9137, 87 },
    { 0xa81a913a, 95 },
    { 0x1b5a913a, 23 },
    { 0, 0 },
    { 0xbc7a913d, 61 },
    { 0xa3da913e, 88 },
    { 0x7a9a913d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4a9142, 95 },
    { 0x364a9142, 61 },
    { 0, 0 },
    { 0xa79a9145, 61 },
    { 0, 0 },
    { 0xd7da9147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aa914d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a9150, 85 },
    { 0, 0 },
    { 0x6e8a9152, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a915b, 78 },
    { 0xc41a915c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a9162, 43 },
    { 0, 0 },
    { 0xf9ca9164, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa9167, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffa916b, 46 },
    { 0, 0 },
    { 0xaa0a916d, 98 },
    { 0x9daa916e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaca9171, 53 },
    { 0x535a9172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa9175, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca9178, 95 },
    { 0x76ea9179, 34 },
    { 0x57a917a, 78 },
    { 0x179a917b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a917f, 91 },
    { 0, 0 },
    { 0x374a9181, 61 },
    { 0xb9ca9182, 95 },
    { 0xd20a9183, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7a918c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4a918f, 23 },
    { 0x1d7a918f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ba9195, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a919e, 90 },
    { 0x704a919e, 67 },
    { 0, 0 },
    { 0xd3ea91a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fea91a6, 99 },
    { 0xacaa91a6, 95 },
    { 0, 0 },
    { 0x258a91a9, 33 },
    { 0, 0 },
    { 0x8e5a91ab, 23 },
    { 0xb2fa91ac, 33 },
    { 0, 0 },
    { 0x567a91ae, 34 },
    { 0xdd1a91ae, 93 },
    { 0, 0 },
    { 0x9dba91b1, 52 },
    { 0, 0 },
    { 0xeeda91b3, 23 },
    { 0x8e3a91b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0a91b8, 98 },
    { 0, 0 },
    { 0x14fa91ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffca91bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7a91c1, 38 },
    { 0xb55a91c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a91c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a91cd, 61 },
    { 0, 0 },
    { 0x7bda91cf, 65 },
    { 0xafca91cf, 61 },
    { 0xa43a91d1, 61 },
    { 0x9d3a91d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ba91d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855a91db, 33 },
    { 0, 0 },
    { 0xf7fa91dd, 61 },
    { 0x5cfa91de, 33 },
    { 0, 0 },
    { 0xc84a91e0, 90 },
    { 0, 0 },
    { 0xc2ca91e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfa91e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ba91ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaa91f8, 98 },
    { 0x46ba91f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488a9200, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa9209, 43 },
    { 0x1dfa920a, 90 },
    { 0x8efa920a, 99 },
    { 0x417a920c, 51 },
    { 0x7bea920c, 23 },
    { 0x5a9209, 33 },
    { 0xe09a920f, 84 },
    { 0x39aa920f, 61 },
    { 0xa3da920d, 23 },
    { 0xceea920f, 52 },
    { 0, 0 },
    { 0x6a9a9214, 69 },
    { 0, 0 },
    { 0x1ca9216, 34 },
    { 0x4d8a9217, 91 },
    { 0xedea9217, 34 },
    { 0x9eaa9219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a9221, 23 },
    { 0xca9a9221, 90 },
    { 0, 0 },
    { 0x445a9224, 52 },
    { 0xcd3a9224, 67 },
    { 0xa22a9224, 40 },
    { 0x37ba9227, 23 },
    { 0xdfba9228, 33 },
    { 0, 0 },
    { 0x78fa922a, 85 },
    { 0x427a922b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da9233, 95 },
    { 0x658a9233, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ba9238, 33 },
    { 0xbdea9239, 33 },
    { 0, 0 },
    { 0x80ba923b, 78 },
    { 0x35ca923b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a9245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a9249, 98 },
    { 0xf45a9249, 23 },
    { 0xcea924b, 52 },
    { 0x75da924c, 77 },
    { 0x580a924c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ea9250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a9259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571a925e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a9262, 53 },
    { 0x808a9262, 61 },
    { 0, 0 },
    { 0xc7fa9265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a926a, 76 },
    { 0, 0 },
    { 0x62da926c, 83 },
    { 0xd47a926d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dea9270, 52 },
    { 0xbe7a9271, 95 },
    { 0x1c8a9272, 90 },
    { 0, 0 },
    { 0xf7ca9274, 43 },
    { 0x959a9275, 31 },
    { 0xa99a9276, 100 },
    { 0x954a9275, 55 },
    { 0xec3a9275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cda9280, 23 },
    { 0x977a9281, 100 },
    { 0x883a9281, 93 },
    { 0x53a9283, 76 },
    { 0x73ea9281, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x184a9287, 98 },
    { 0x81ba9288, 59 },
    { 0xa88a9289, 64 },
    { 0, 0 },
    { 0x548a928b, 23 },
    { 0xbe8a928c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa9290, 84 },
    { 0xc8ba9291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fa9296, 90 },
    { 0x561a9297, 33 },
    { 0x299a9297, 96 },
    { 0x9e1a9299, 91 },
    { 0xf76a929a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba929e, 55 },
    { 0x76aa929f, 67 },
    { 0xd0aa929e, 95 },
    { 0x376a92a1, 66 },
    { 0, 0 },
    { 0x4e0a92a3, 90 },
    { 0x9eea92a4, 61 },
    { 0x2e9a92a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fa92a9, 93 },
    { 0, 0 },
    { 0x2b4a92ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43a92b0, 23 },
    { 0xf1ca92b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7a92b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8a92bb, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea92be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea92c4, 17 },
    { 0x836a92c5, 23 },
    { 0x544a92c5, 90 },
    { 0x109a92c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a92ca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2a92d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8a92d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9a92dc, 99 },
    { 0xc02a92dd, 93 },
    { 0, 0 },
    { 0xb13a92df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcca92e3, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfa92e7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718a92eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eea92ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1a92f7, 98 },
    { 0x3d9a92f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a92fc, 33 },
    { 0xcf5a92fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fa9301, 23 },
    { 0, 0 },
    { 0x559a9303, 90 },
    { 0, 0 },
    { 0x41a9305, 100 },
    { 0x922a9305, 64 },
    { 0, 0 },
    { 0x431a9308, 77 },
    { 0, 0 },
    { 0x527a930a, 46 },
    { 0x7f7a930b, 33 },
    { 0xf07a930c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7a930f, 61 },
    { 0, 0 },
    { 0x108a9311, 84 },
    { 0, 0 },
    { 0x412a9313, 23 },
    { 0xeeaa9314, 90 },
    { 0, 0 },
    { 0xa36a9316, 55 },
    { 0x2c6a9317, 75 },
    { 0, 0 },
    { 0xd06a9319, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840a931d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a9320, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540a9325, 67 },
    { 0x795a9326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9329, 93 },
    { 0, 0 },
    { 0x979a932b, 61 },
    { 0x297a932b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1a932f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a9334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67a933b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74a933e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea9342, 100 },
    { 0x245a9343, 55 },
    { 0x6bba9343, 90 },
    { 0xe72a9345, 23 },
    { 0x705a9346, 91 },
    { 0, 0 },
    { 0x5faa9348, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a9350, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a9354, 98 },
    { 0, 0 },
    { 0x6b6a9356, 95 },
    { 0x5c5a9356, 90 },
    { 0xf8ba9358, 61 },
    { 0, 0 },
    { 0x933a935a, 40 },
    { 0xc38a935a, 23 },
    { 0xa9a935a, 33 },
    { 0xfb8a935c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ba9360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a9368, 33 },
    { 0xd2fa9369, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a9375, 33 },
    { 0, 0 },
    { 0x3d0a9377, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebba937b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ea937f, 61 },
    { 0x75aa937f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a938c, 23 },
    { 0, 0 },
    { 0x639a938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ca9391, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaa9399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da939c, 46 },
    { 0xb3ba939d, 33 },
    { 0xfa2a939e, 61 },
    { 0, 0 },
    { 0xcf7a93a0, 77 },
    { 0x478a93a0, 87 },
    { 0xbea93a2, 33 },
    { 0x7f2a93a3, 93 },
    { 0x3dca93a1, 90 },
    { 0x5c5a93a2, 61 },
    { 0x584a93a6, 95 },
    { 0xb09a93a7, 67 },
    { 0xecfa93a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a93ac, 98 },
    { 0, 0 },
    { 0x587a93ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a93b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca93b9, 76 },
    { 0x97a93ba, 23 },
    { 0x2f2a93ba, 90 },
    { 0xcdfa93bc, 90 },
    { 0x4c7a93bd, 66 },
    { 0xc2fa93be, 97 },
    { 0x430a93be, 23 },
    { 0x4b8a93be, 90 },
    { 0xcf9a93bb, 17 },
    { 0, 0 },
    { 0xe80a93c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594a93cc, 34 },
    { 0xd0aa93cd, 90 },
    { 0x5a2a93cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a93d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a93d4, 23 },
    { 0xddfa93d4, 80 },
    { 0, 0 },
    { 0x135a93d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ea93da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x948a93dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca93e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a93e6, 100 },
    { 0xda0a93e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a93ed, 97 },
    { 0, 0 },
    { 0x9cea93ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a93f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19a93f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a93f9, 32 },
    { 0xac8a93f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85a93fd, 78 },
    { 0xad8a93fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aa9407, 52 },
    { 0xf54a9408, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a940f, 23 },
    { 0, 0 },
    { 0x3eba9411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94a9414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x991a9417, 61 },
    { 0, 0 },
    { 0x51ba9419, 76 },
    { 0x7eca9419, 85 },
    { 0, 0 },
    { 0x13ca941c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a9425, 61 },
    { 0, 0 },
    { 0x579a9427, 23 },
    { 0, 0 },
    { 0x36ea9429, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a9432, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a9436, 81 },
    { 0xf29a9436, 98 },
    { 0xdc5a9436, 95 },
    { 0xceda9438, 95 },
    { 0xd7ea943a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a943d, 90 },
    { 0x746a943d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a9443, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5a944e, 23 },
    { 0, 0 },
    { 0x48aa9450, 84 },
    { 0x7b5a9451, 87 },
    { 0xb2a9450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73a9455, 23 },
    { 0x623a9455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38a9459, 95 },
    { 0x478a945a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a945d, 61 },
    { 0x70da945e, 68 },
    { 0, 0 },
    { 0x214a9460, 90 },
    { 0, 0 },
    { 0x9cea9462, 75 },
    { 0x1b8a9463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a9467, 84 },
    { 0xde3a9468, 95 },
    { 0xc34a9467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365a946d, 33 },
    { 0xbc0a946e, 76 },
    { 0, 0 },
    { 0x455a9470, 33 },
    { 0x810a9470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649a9480, 98 },
    { 0x248a9481, 61 },
    { 0x75da9482, 23 },
    { 0, 0 },
    { 0x58ca9484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a948a, 61 },
    { 0xf44a948b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ca9490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a9496, 95 },
    { 0, 0 },
    { 0xd92a9498, 61 },
    { 0x553a9499, 33 },
    { 0xd9da9499, 61 },
    { 0xddba9498, 98 },
    { 0xd77a949c, 77 },
    { 0x29ba949d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a94a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0a94a8, 85 },
    { 0x26fa94a9, 65 },
    { 0x95a94a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87a94ae, 91 },
    { 0x8a0a94af, 76 },
    { 0xf8da94b0, 77 },
    { 0x8e5a94b0, 76 },
    { 0x645a94b2, 90 },
    { 0xe3ca94b3, 52 },
    { 0xc71a94b4, 66 },
    { 0x9e1a94b3, 76 },
    { 0xefda94b6, 76 },
    { 0x10aa94b7, 46 },
    { 0x521a94af, 84 },
    { 0x31ba94ae, 61 },
    { 0, 0 },
    { 0xcf8a94bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3a94c0, 55 },
    { 0x922a94c1, 34 },
    { 0x546a94c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a94c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591a94ca, 76 },
    { 0x56ba94cb, 55 },
    { 0xc77a94cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a94d5, 90 },
    { 0, 0 },
    { 0x63fa94d7, 78 },
    { 0, 0 },
    { 0xb69a94d9, 95 },
    { 0xcc9a94d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971a94e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aa94e5, 93 },
    { 0, 0 },
    { 0x168a94e7, 95 },
    { 0x60ca94e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994a94ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bca94f0, 61 },
    { 0, 0 },
    { 0xe46a94f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845a94f6, 33 },
    { 0xb51a94f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ba94fa, 33 },
    { 0, 0 },
    { 0xccea94fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a9500, 93 },
    { 0x4eea9500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa9504, 76 },
    { 0x98ba9505, 33 },
    { 0x1c5a9505, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46da950a, 98 },
    { 0xf69a950b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a950e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eca9512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3a9515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950a951a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68a9520, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436a9525, 55 },
    { 0, 0 },
    { 0xc8ca9527, 95 },
    { 0xd10a9528, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa952b, 23 },
    { 0xb44a952c, 33 },
    { 0xd12a952c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0a9530, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a9533, 96 },
    { 0x8d3a9533, 61 },
    { 0x51da9535, 17 },
    { 0x421a9536, 67 },
    { 0x39a9535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2a953e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a9541, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03a9544, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a9547, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x175a954a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a955b, 98 },
    { 0xc1ca955c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x584a955f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788a9564, 66 },
    { 0x205a9565, 68 },
    { 0x34a9566, 23 },
    { 0xc3da9565, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fa956e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fa9574, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5a9577, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ea957f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8a9582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41a9585, 68 },
    { 0xb98a9585, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a958b, 34 },
    { 0x3ea958c, 61 },
    { 0x658a958d, 90 },
    { 0xe90a958d, 23 },
    { 0x6aaa958e, 34 },
    { 0x28fa958d, 55 },
    { 0x658a9591, 90 },
    { 0x44a9592, 34 },
    { 0xd39a958d, 98 },
    { 0xbe5a9594, 90 },
    { 0, 0 },
    { 0x1a3a9596, 93 },
    { 0x60da9597, 23 },
    { 0x9cca9598, 98 },
    { 0x599a9598, 77 },
    { 0x10da959a, 95 },
    { 0x51fa959b, 88 },
    { 0x979a959b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6a959f, 90 },
    { 0xa64a959f, 93 },
    { 0xda4a95a1, 78 },
    { 0x72aa95a2, 90 },
    { 0xb6aa959f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5a95a8, 55 },
    { 0x981a95a8, 98 },
    { 0xab0a95a8, 34 },
    { 0, 0 },
    { 0x9aa95ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a95b4, 91 },
    { 0, 0 },
    { 0xb40a95b6, 33 },
    { 0xbf3a95b7, 46 },
    { 0x1a2a95b8, 98 },
    { 0x77a95b9, 76 },
    { 0x1da95b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98a95bd, 23 },
    { 0x80aa95bd, 34 },
    { 0xf50a95be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffa95c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x368a95c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25a95cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a95d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca95de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7a95e2, 90 },
    { 0x18da95e3, 51 },
    { 0x28ba95e4, 76 },
    { 0x6a4a95e5, 33 },
    { 0xdaba95e5, 76 },
    { 0x193a95e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7a95ea, 52 },
    { 0x959a95eb, 52 },
    { 0x841a95eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34a95fa, 17 },
    { 0, 0 },
    { 0x9a6a95fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3a9609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7a960f, 61 },
    { 0xa0aa9610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6a9615, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x385a9618, 88 },
    { 0x779a9618, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98da961e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x188a9621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7a9624, 77 },
    { 0, 0 },
    { 0x902a9626, 93 },
    { 0, 0 },
    { 0x782a9628, 98 },
    { 0xe5ca9628, 95 },
    { 0xed5a962a, 46 },
    { 0xecfa962a, 95 },
    { 0, 0 },
    { 0x3d6a962d, 93 },
    { 0xa91a962e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x866a9631, 43 },
    { 0x1eea9632, 8 },
    { 0x727a9633, 33 },
    { 0, 0 },
    { 0xcc9a9635, 61 },
    { 0x979a9636, 61 },
    { 0xe10a9637, 40 },
    { 0x6aea9638, 90 },
    { 0x42ea9636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eba9641, 34 },
    { 0xd95a9641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912a964a, 93 },
    { 0x3c5a964b, 33 },
    { 0, 0 },
    { 0x917a964d, 93 },
    { 0xc8ba964d, 61 },
    { 0x596a964f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636a9653, 98 },
    { 0x5b3a9654, 33 },
    { 0, 0 },
    { 0x190a9656, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a965a, 52 },
    { 0, 0 },
    { 0xfb7a965c, 33 },
    { 0, 0 },
    { 0x2e2a965e, 80 },
    { 0x25fa965f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54a9663, 46 },
    { 0, 0 },
    { 0xfd8a9665, 33 },
    { 0xe56a9666, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fa966a, 76 },
    { 0x1eca966b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9a966e, 98 },
    { 0xab8a966f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408a9673, 95 },
    { 0x2e9a9674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a9678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33a967b, 23 },
    { 0, 0 },
    { 0xaaca967d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1a9687, 80 },
    { 0xf1ea9688, 61 },
    { 0xf07a9688, 61 },
    { 0x625a9689, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda968d, 77 },
    { 0x9f7a968e, 90 },
    { 0, 0 },
    { 0x736a9690, 34 },
    { 0x66ba9691, 52 },
    { 0xee5a9691, 33 },
    { 0xd61a9692, 26 },
    { 0x58ca9693, 100 },
    { 0, 0 },
    { 0xf49a9696, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0a9699, 68 },
    { 0, 0 },
    { 0xc21a969b, 61 },
    { 0, 0 },
    { 0x537a969d, 98 },
    { 0x926a969d, 76 },
    { 0xd04a969f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57a96a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a96a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea96bb, 90 },
    { 0xd6ba96bc, 77 },
    { 0x820a96bd, 81 },
    { 0x1c1a96bb, 95 },
    { 0x365a96bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ea96c3, 66 },
    { 0, 0 },
    { 0xa80a96c5, 76 },
    { 0x611a96c6, 98 },
    { 0xe45a96c7, 90 },
    { 0, 0 },
    { 0xe6ca96c9, 90 },
    { 0, 0 },
    { 0xea2a96cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a96ce, 61 },
    { 0, 0 },
    { 0x601a96d0, 66 },
    { 0xf42a96d0, 61 },
    { 0xd92a96d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a96d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x274a96da, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65a96dd, 98 },
    { 0, 0 },
    { 0x9dca96df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3a96e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a96e7, 61 },
    { 0xf16a96e7, 90 },
    { 0x50aa96e9, 90 },
    { 0, 0 },
    { 0xf6fa96eb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a96f0, 33 },
    { 0x44ba96f1, 95 },
    { 0x571a96f1, 98 },
    { 0xe08a96f3, 90 },
    { 0xc73a96f3, 95 },
    { 0x612a96f3, 33 },
    { 0xb9a96f6, 78 },
    { 0x14a96f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x918a96fa, 95 },
    { 0x80da96fa, 95 },
    { 0, 0 },
    { 0xb89a96fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca9700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543a9704, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7a9708, 76 },
    { 0xd72a9709, 76 },
    { 0x4bca9709, 95 },
    { 0xd92a970a, 61 },
    { 0x9a9709, 90 },
    { 0xc9ca970d, 93 },
    { 0, 0 },
    { 0xc18a970f, 43 },
    { 0xa49a970f, 33 },
    { 0xf25a9711, 33 },
    { 0x8e0a9712, 33 },
    { 0x622a9713, 38 },
    { 0xe26a9714, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ca971a, 52 },
    { 0xc72a971a, 87 },
    { 0xa87a971c, 100 },
    { 0x76ea971d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca9720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca9724, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47a9727, 87 },
    { 0xb67a9727, 95 },
    { 0x9ea9729, 76 },
    { 0xbe7a972a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05a972d, 61 },
    { 0, 0 },
    { 0xb2aa972f, 91 },
    { 0x6fda9730, 33 },
    { 0, 0 },
    { 0x31aa9732, 40 },
    { 0x964a9732, 33 },
    { 0x69fa9733, 23 },
    { 0xcfaa9735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a973a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a973d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa9741, 95 },
    { 0xc7ba9742, 91 },
    { 0, 0 },
    { 0x3b1a9744, 34 },
    { 0, 0 },
    { 0xe1ca9746, 90 },
    { 0xdeba9747, 93 },
    { 0x553a9746, 95 },
    { 0x2e1a9747, 52 },
    { 0xfdda974a, 76 },
    { 0xc8fa9747, 43 },
    { 0x86ea974c, 55 },
    { 0x7d0a974d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a9752, 90 },
    { 0x699a9752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aa9759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ba9767, 53 },
    { 0xdb4a9768, 93 },
    { 0x10ba9769, 90 },
    { 0xb76a976a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aa976d, 90 },
    { 0x60da976e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04a9771, 61 },
    { 0x6da9771, 17 },
    { 0xd9a9771, 61 },
    { 0, 0 },
    { 0x5c4a9775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a977a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacea977f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a9782, 91 },
    { 0x82aa9782, 61 },
    { 0x406a9784, 43 },
    { 0x19ca9785, 52 },
    { 0x4c3a9786, 61 },
    { 0xf28a9784, 61 },
    { 0x3aa9785, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca978c, 90 },
    { 0, 0 },
    { 0xb6ea978e, 81 },
    { 0x8a7a978e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a979b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5a97a0, 61 },
    { 0, 0 },
    { 0xdbea97a2, 76 },
    { 0x684a97a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaa97a6, 91 },
    { 0, 0 },
    { 0xee3a97a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ca97ac, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a97b0, 98 },
    { 0x2d9a97b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a97b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a97c3, 98 },
    { 0, 0 },
    { 0x54ea97c5, 84 },
    { 0x981a97c6, 78 },
    { 0x27da97c7, 87 },
    { 0xf06a97c7, 23 },
    { 0x8fa97c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a97cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a97d1, 84 },
    { 0x2f9a97d2, 61 },
    { 0x5e7a97d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a97d7, 23 },
    { 0xd93a97d8, 98 },
    { 0xd58a97d7, 90 },
    { 0x1e0a97d7, 90 },
    { 0x192a97db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x508a97de, 75 },
    { 0x183a97df, 23 },
    { 0x89fa97e0, 90 },
    { 0xaeca97e1, 33 },
    { 0x330a97e1, 33 },
    { 0xcc2a97e3, 91 },
    { 0x3a9a97e4, 81 },
    { 0x758a97e4, 76 },
    { 0, 0 },
    { 0x57ca97e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895a97ed, 84 },
    { 0x4fea97ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a97f1, 23 },
    { 0x4ea97f2, 90 },
    { 0xf56a97f3, 95 },
    { 0x25ba97f4, 23 },
    { 0xb8ca97f5, 33 },
    { 0, 0 },
    { 0x845a97f7, 23 },
    { 0x26aa97f8, 98 },
    { 0x97ca97f9, 43 },
    { 0x639a97fa, 46 },
    { 0xf36a97fb, 100 },
    { 0xc6ea97fc, 76 },
    { 0x8eca97fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a9804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba9808, 84 },
    { 0x69ca9809, 34 },
    { 0, 0 },
    { 0xd5fa980b, 84 },
    { 0x272a980c, 55 },
    { 0x3c5a980b, 95 },
    { 0, 0 },
    { 0x15ea980f, 90 },
    { 0x2eca980f, 98 },
    { 0x360a9811, 90 },
    { 0x409a9811, 99 },
    { 0, 0 },
    { 0xf04a9814, 95 },
    { 0xe24a9814, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba981d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838a9822, 96 },
    { 0x17aa9823, 61 },
    { 0x880a9823, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba982b, 100 },
    { 0x15ba982c, 61 },
    { 0x7f3a982d, 33 },
    { 0xe9fa982e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a9831, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda9834, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x861a9837, 69 },
    { 0x615a9837, 95 },
    { 0xa26a9839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254a983e, 85 },
    { 0x5f8a983f, 66 },
    { 0xc82a9840, 33 },
    { 0x138a9840, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ba9846, 23 },
    { 0xfaaa9847, 90 },
    { 0, 0 },
    { 0x769a9849, 23 },
    { 0x3cfa984a, 34 },
    { 0, 0 },
    { 0xa5fa984c, 90 },
    { 0xb5fa984d, 98 },
    { 0xac5a984d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea9851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda985b, 43 },
    { 0x7c0a985c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a9862, 55 },
    { 0x746a9862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8a9867, 76 },
    { 0x74da9867, 90 },
    { 0x304a9869, 33 },
    { 0x9aba9867, 90 },
    { 0x4b9a986b, 55 },
    { 0xd8aa986c, 34 },
    { 0x10aa986d, 93 },
    { 0x2aa986e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fa9871, 61 },
    { 0, 0 },
    { 0x130a9873, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaa9880, 93 },
    { 0, 0 },
    { 0xc0fa9882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923a9886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14a988e, 91 },
    { 0x8d9a988e, 51 },
    { 0, 0 },
    { 0xca0a9891, 36 },
    { 0xc0a9892, 88 },
    { 0, 0 },
    { 0xaeda9894, 46 },
    { 0x653a9895, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa9898, 23 },
    { 0x3a4a9898, 96 },
    { 0, 0 },
    { 0x15a989b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a989f, 90 },
    { 0x93ea98a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa98a4, 76 },
    { 0x14da98a4, 43 },
    { 0xe6fa98a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca98ad, 23 },
    { 0xd13a98ae, 82 },
    { 0xbb2a98af, 43 },
    { 0x5c8a98ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ca98b3, 98 },
    { 0, 0 },
    { 0x165a98b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7a98bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555a98c3, 23 },
    { 0xd5a98c3, 90 },
    { 0x743a98c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ba98c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87a98cb, 76 },
    { 0xc04a98cc, 55 },
    { 0x49ca98cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a98d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca98d4, 90 },
    { 0xcefa98d4, 98 },
    { 0x9e2a98d6, 33 },
    { 0, 0 },
    { 0xbeba98d8, 90 },
    { 0x1cba98d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca98dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a98df, 76 },
    { 0x6bba98df, 55 },
    { 0xe04a98e1, 17 },
    { 0, 0 },
    { 0x5d6a98e3, 46 },
    { 0x314a98e4, 23 },
    { 0x500a98e4, 68 },
    { 0xba0a98e4, 61 },
    { 0, 0 },
    { 0x47ea98e8, 33 },
    { 0, 0 },
    { 0xa2aa98ea, 61 },
    { 0xe7ba98eb, 33 },
    { 0, 0 },
    { 0xf6aa98ed, 90 },
    { 0, 0 },
    { 0x13ca98ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a98f2, 90 },
    { 0x885a98f3, 77 },
    { 0, 0 },
    { 0xbc0a98f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0a98fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eea9903, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a990a, 43 },
    { 0, 0 },
    { 0xbfba990c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a9911, 95 },
    { 0x347a9912, 90 },
    { 0xd6ba9913, 61 },
    { 0x9d1a9914, 43 },
    { 0x4a8a9913, 33 },
    { 0, 0 },
    { 0x290a9917, 33 },
    { 0xc78a9918, 34 },
    { 0x301a9918, 98 },
    { 0x965a991a, 34 },
    { 0x974a991b, 88 },
    { 0xafca991c, 78 },
    { 0x154a9918, 23 },
    { 0x4aa991e, 23 },
    { 0x1cea9919, 100 },
    { 0x150a9920, 34 },
    { 0, 0 },
    { 0xa86a9922, 23 },
    { 0x9d8a9923, 61 },
    { 0x89fa9923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebea9928, 98 },
    { 0x9d4a9929, 40 },
    { 0x3c8a9929, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77a992d, 98 },
    { 0xe8a992d, 98 },
    { 0xf02a992f, 99 },
    { 0xddca992e, 85 },
    { 0, 0 },
    { 0xc04a9932, 90 },
    { 0xe1aa9933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa993a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ea993d, 90 },
    { 0, 0 },
    { 0xf7da993f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653a9943, 98 },
    { 0xa78a9944, 68 },
    { 0, 0 },
    { 0x73da9946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fa994d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2a9951, 61 },
    { 0xe64a9951, 33 },
    { 0, 0 },
    { 0x3a9a9954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ba9960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608a9969, 8 },
    { 0xc15a9969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x286a996d, 90 },
    { 0x38ea996e, 95 },
    { 0x64ea996f, 23 },
    { 0xc66a996e, 90 },
    { 0x154a9971, 95 },
    { 0x6e8a9971, 77 },
    { 0, 0 },
    { 0xb32a9974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a997a, 98 },
    { 0x7fea997b, 62 },
    { 0x1a6a997b, 43 },
    { 0xac5a997c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea9987, 61 },
    { 0x508a9987, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ada9990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7a9993, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a9997, 67 },
    { 0, 0 },
    { 0x8a4a9999, 55 },
    { 0xfe1a999a, 76 },
    { 0x47ea999b, 33 },
    { 0x8bda999b, 23 },
    { 0x8a4a999d, 87 },
    { 0xd7ea999d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fba99a2, 68 },
    { 0xceea99a2, 55 },
    { 0, 0 },
    { 0x9c8a99a5, 55 },
    { 0xf7aa99a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bca99a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a99b0, 23 },
    { 0x6aea99b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451a99b6, 61 },
    { 0, 0 },
    { 0xa6da99b8, 90 },
    { 0xfa1a99b9, 99 },
    { 0, 0 },
    { 0x7a6a99bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a99be, 90 },
    { 0xbbea99bf, 90 },
    { 0, 0 },
    { 0xbbba99c1, 61 },
    { 0, 0 },
    { 0x2f9a99c3, 90 },
    { 0, 0 },
    { 0x723a99c5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a99c9, 76 },
    { 0x6bba99ca, 93 },
    { 0xcfa99cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a99cf, 34 },
    { 0, 0 },
    { 0x7d6a99d1, 46 },
    { 0xf05a99d1, 23 },
    { 0x373a99d3, 95 },
    { 0xf6a99d4, 61 },
    { 0xa8ca99d5, 87 },
    { 0x5da99d4, 46 },
    { 0x1d4a99d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ca99dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dca99e2, 88 },
    { 0xd06a99e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba99ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a99ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13a99f2, 61 },
    { 0xae5a99f3, 93 },
    { 0xd7da99f2, 93 },
    { 0x524a99f5, 98 },
    { 0x541a99f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bda99fa, 67 },
    { 0, 0 },
    { 0x2e3a99fc, 17 },
    { 0x802a99fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a9a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa9a08, 87 },
    { 0, 0 },
    { 0x798a9a0a, 61 },
    { 0, 0 },
    { 0x68ca9a0c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfa9a0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0a9a13, 61 },
    { 0, 0 },
    { 0x1e2a9a15, 33 },
    { 0, 0 },
    { 0x49a9a17, 33 },
    { 0xaf4a9a17, 98 },
    { 0, 0 },
    { 0x5cda9a1a, 55 },
    { 0xa60a9a1b, 33 },
    { 0, 0 },
    { 0xa4aa9a1d, 95 },
    { 0x48ba9a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05a9a21, 90 },
    { 0, 0 },
    { 0x778a9a23, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a9a27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa9a2e, 34 },
    { 0, 0 },
    { 0xbc0a9a30, 43 },
    { 0x198a9a31, 66 },
    { 0x7f9a9a30, 93 },
    { 0, 0 },
    { 0xe7ea9a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a9a39, 76 },
    { 0xd6ea9a3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a9a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a9a47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a9a4a, 100 },
    { 0xbf0a9a4b, 61 },
    { 0xe54a9a4c, 76 },
    { 0x65da9a4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65a9a51, 61 },
    { 0, 0 },
    { 0xd0da9a53, 76 },
    { 0, 0 },
    { 0xda5a9a55, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a9a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x165a9a5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a9a62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a9a65, 90 },
    { 0xacaa9a65, 90 },
    { 0x27a9a67, 66 },
    { 0x840a9a65, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462a9a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a9a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fa9a73, 98 },
    { 0xda7a9a74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8a9a77, 97 },
    { 0x853a9a78, 98 },
    { 0x7fea9a77, 61 },
    { 0, 0 },
    { 0xf00a9a7b, 23 },
    { 0xeb3a9a7c, 61 },
    { 0x8a2a9a7d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a9a81, 101 },
    { 0xfd8a9a82, 61 },
    { 0x182a9a82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eea9a87, 98 },
    { 0x2f8a9a87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca9a8c, 76 },
    { 0, 0 },
    { 0xd5da9a8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a9a91, 98 },
    { 0xaeba9a92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddba9a98, 34 },
    { 0x6cea9a99, 40 },
    { 0, 0 },
    { 0xf4ba9a9b, 93 },
    { 0x345a9a9c, 99 },
    { 0x20fa9a9d, 55 },
    { 0xa09a9a9c, 76 },
    { 0, 0 },
    { 0x8fba9aa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a9aa3, 9 },
    { 0x9d7a9aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefa9aa7, 100 },
    { 0x13da9aa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fa9aac, 33 },
    { 0x31ca9aad, 77 },
    { 0x65fa9aae, 55 },
    { 0, 0 },
    { 0xe0ea9ab0, 68 },
    { 0xd3a9ab1, 23 },
    { 0, 0 },
    { 0x922a9ab3, 23 },
    { 0, 0 },
    { 0x9f5a9ab5, 66 },
    { 0x7b2a9ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a9ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0a9ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fda9acc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea9acf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda9ad7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a9ade, 68 },
    { 0, 0 },
    { 0x8eca9ae0, 33 },
    { 0xb60a9ae0, 76 },
    { 0x8d5a9ae1, 61 },
    { 0xdcda9ae1, 37 },
    { 0, 0 },
    { 0x6c1a9ae5, 90 },
    { 0x778a9ae6, 95 },
    { 0, 0 },
    { 0x5f9a9ae8, 34 },
    { 0x78ba9ae8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7a9aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06a9af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490a9af9, 55 },
    { 0, 0 },
    { 0xdd9a9afb, 61 },
    { 0xd0da9afc, 33 },
    { 0x973a9afd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aa9b02, 95 },
    { 0xd51a9b03, 43 },
    { 0x614a9b04, 98 },
    { 0xf8da9b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea9b09, 81 },
    { 0xbcda9b09, 78 },
    { 0xd43a9b09, 100 },
    { 0x313a9b09, 40 },
    { 0x46aa9b0c, 95 },
    { 0, 0 },
    { 0xfa1a9b0f, 95 },
    { 0xfa4a9b10, 100 },
    { 0x220a9b0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea9b14, 98 },
    { 0xca1a9b15, 85 },
    { 0xc03a9b15, 46 },
    { 0x96a9b15, 28 },
    { 0, 0 },
    { 0x7da9b19, 61 },
    { 0, 0 },
    { 0xc71a9b1b, 23 },
    { 0, 0 },
    { 0xd9da9b1d, 95 },
    { 0xcf9a9b1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5a9b23, 69 },
    { 0xc82a9b24, 61 },
    { 0x42ea9b24, 24 },
    { 0x4c1a9b26, 90 },
    { 0x7cda9b27, 43 },
    { 0xa5a9b28, 61 },
    { 0x106a9b29, 76 },
    { 0xa33a9b25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aca9b2e, 98 },
    { 0xb67a9b2e, 23 },
    { 0xe5ea9b30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea9b33, 78 },
    { 0x673a9b33, 90 },
    { 0, 0 },
    { 0xba7a9b36, 76 },
    { 0xb58a9b36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1a9b3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9b42, 91 },
    { 0xc47a9b42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a9b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3da9b4a, 4 },
    { 0, 0 },
    { 0x72ea9b4c, 95 },
    { 0xdcca9b4d, 90 },
    { 0, 0 },
    { 0x4efa9b4f, 95 },
    { 0xfcfa9b50, 33 },
    { 0, 0 },
    { 0xa90a9b52, 43 },
    { 0x3d3a9b53, 23 },
    { 0xb57a9b54, 61 },
    { 0xbb5a9b54, 33 },
    { 0x4bea9b56, 95 },
    { 0, 0 },
    { 0x9b9a9b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546a9b5e, 76 },
    { 0x977a9b5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91a9b64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa9b69, 33 },
    { 0x261a9b6a, 46 },
    { 0xac5a9b6b, 55 },
    { 0x921a9b6b, 76 },
    { 0x66a9b6d, 46 },
    { 0xbfea9b6b, 76 },
    { 0x102a9b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4a9b74, 90 },
    { 0, 0 },
    { 0xa7ca9b76, 61 },
    { 0, 0 },
    { 0xdbaa9b78, 93 },
    { 0x2c9a9b79, 93 },
    { 0, 0 },
    { 0x9b4a9b7b, 33 },
    { 0x395a9b7c, 34 },
    { 0xc26a9b7d, 90 },
    { 0x6a7a9b7e, 55 },
    { 0xf01a9b7e, 98 },
    { 0x7e9a9b80, 39 },
    { 0x792a9b7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a9b8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733a9b90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a9b95, 85 },
    { 0x34ba9b96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa9b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ba9b9e, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x370a9ba1, 95 },
    { 0xb6a9ba2, 23 },
    { 0, 0 },
    { 0xf9ca9ba4, 95 },
    { 0x7c7a9ba4, 95 },
    { 0xe05a9ba6, 78 },
    { 0x790a9ba5, 98 },
    { 0xd31a9ba4, 84 },
    { 0x9b3a9ba9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfca9baf, 98 },
    { 0xd12a9bb0, 88 },
    { 0xadfa9bb0, 55 },
    { 0x63ea9bb2, 99 },
    { 0xc4da9bb3, 61 },
    { 0x885a9bb4, 87 },
    { 0xfbca9bb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ba9bb9, 43 },
    { 0x541a9bba, 43 },
    { 0xb3da9bbb, 90 },
    { 0xb23a9bb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaa9bbf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a9bcd, 23 },
    { 0x186a9bce, 61 },
    { 0xf7a9bce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6a9bd2, 43 },
    { 0xce2a9bd3, 52 },
    { 0x1bfa9bd3, 76 },
    { 0xd2fa9bd4, 85 },
    { 0, 0 },
    { 0xc6aa9bd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a9bdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x134a9be2, 90 },
    { 0x7ffa9be3, 61 },
    { 0x880a9be4, 90 },
    { 0xa70a9be5, 85 },
    { 0x9afa9be2, 33 },
    { 0x314a9be7, 95 },
    { 0, 0 },
    { 0x4dca9be9, 95 },
    { 0xe69a9bea, 90 },
    { 0x82ea9bea, 33 },
    { 0x2d3a9bec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda9bf1, 98 },
    { 0, 0 },
    { 0x32da9bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afa9bf6, 95 },
    { 0, 0 },
    { 0xdc3a9bf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9da9bfd, 33 },
    { 0, 0 },
    { 0x36ea9bff, 55 },
    { 0, 0 },
    { 0x203a9c01, 76 },
    { 0, 0 },
    { 0x4cfa9c03, 46 },
    { 0x867a9c04, 33 },
    { 0xbcaa9c05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa9c09, 52 },
    { 0, 0 },
    { 0x2f8a9c0b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a9c11, 46 },
    { 0x2b8a9c12, 83 },
    { 0, 0 },
    { 0x106a9c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536a9c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7a9c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ba9c1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483a9c23, 33 },
    { 0xebda9c23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa9c2c, 33 },
    { 0x682a9c2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7a9c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25a9c37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca9c3a, 87 },
    { 0x9ea9c3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2a9c3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x604a9c41, 76 },
    { 0, 0 },
    { 0xa93a9c43, 91 },
    { 0x123a9c44, 90 },
    { 0x248a9c44, 61 },
    { 0x581a9c46, 93 },
    { 0xb9da9c47, 93 },
    { 0x4b5a9c46, 36 },
    { 0x2c0a9c46, 55 },
    { 0x281a9c47, 69 },
    { 0x2a4a9c4b, 100 },
    { 0xa5a9c48, 60 },
    { 0xf70a9c45, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a9c52, 90 },
    { 0xfcda9c52, 90 },
    { 0, 0 },
    { 0x5d9a9c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576a9c5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a9c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca9c69, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba9c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632a9c73, 55 },
    { 0xb69a9c74, 46 },
    { 0x810a9c74, 40 },
    { 0x485a9c76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a9c79, 61 },
    { 0xa6a9c7a, 93 },
    { 0xb04a9c7b, 61 },
    { 0x21aa9c7b, 33 },
    { 0xa5fa9c7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ea9c82, 85 },
    { 0x8daa9c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1a9c8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2a9c90, 38 },
    { 0xfa0a9c90, 61 },
    { 0, 0 },
    { 0x13fa9c93, 40 },
    { 0, 0 },
    { 0xa31a9c95, 98 },
    { 0, 0 },
    { 0x4fca9c97, 88 },
    { 0x79ea9c97, 90 },
    { 0, 0 },
    { 0xcbda9c9a, 93 },
    { 0x6dda9c9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa9c9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05a9ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3a9ca8, 90 },
    { 0xecea9ca8, 23 },
    { 0, 0 },
    { 0x780a9cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aa9cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a9cb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea9cb4, 98 },
    { 0xd19a9cb4, 95 },
    { 0xfb3a9cb6, 98 },
    { 0x632a9cb7, 33 },
    { 0xb75a9cb6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa9cbc, 52 },
    { 0, 0 },
    { 0xcfca9cbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2a9cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fa9ccc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba9cd1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a9cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ba9cdb, 100 },
    { 0x633a9cdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca9ce3, 61 },
    { 0, 0 },
    { 0x6ca9ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a9ce9, 61 },
    { 0xa61a9ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531a9cef, 33 },
    { 0xa64a9cef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca9cf3, 98 },
    { 0, 0 },
    { 0xc56a9cf5, 76 },
    { 0, 0 },
    { 0xeeea9cf7, 90 },
    { 0xc43a9cf8, 100 },
    { 0x636a9cf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a9cfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641a9d05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa9d09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a9d0c, 64 },
    { 0, 0 },
    { 0x50fa9d0e, 90 },
    { 0, 0 },
    { 0x10ba9d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aa9d18, 33 },
    { 0xe6ca9d18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565a9d1d, 76 },
    { 0x9aa9d1e, 96 },
    { 0xbbda9d1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x922a9d22, 91 },
    { 0x7e4a9d22, 23 },
    { 0x2eba9d23, 33 },
    { 0x503a9d25, 88 },
    { 0xacda9d25, 61 },
    { 0x68ca9d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea9d2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a9d2e, 52 },
    { 0x570a9d2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38a9d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da9d35, 61 },
    { 0x602a9d36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728a9d3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584a9d3f, 90 },
    { 0x781a9d3f, 76 },
    { 0, 0 },
    { 0xba4a9d42, 33 },
    { 0xe36a9d42, 90 },
    { 0, 0 },
    { 0x501a9d45, 95 },
    { 0x714a9d46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2da9d4c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a9d56, 66 },
    { 0, 0 },
    { 0xf29a9d58, 61 },
    { 0xae4a9d59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46a9d5f, 95 },
    { 0xdf6a9d60, 43 },
    { 0x422a9d61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0a9d64, 90 },
    { 0, 0 },
    { 0x264a9d66, 33 },
    { 0xa1da9d67, 66 },
    { 0x51fa9d68, 33 },
    { 0x26ba9d66, 90 },
    { 0x9e8a9d69, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a9d71, 43 },
    { 0xc9ba9d72, 98 },
    { 0x33aa9d73, 87 },
    { 0x94ea9d72, 5 },
    { 0x38ea9d75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea9d7c, 67 },
    { 0x27da9d7d, 95 },
    { 0, 0 },
    { 0xa60a9d7f, 23 },
    { 0xf76a9d80, 67 },
    { 0, 0 },
    { 0xc52a9d82, 66 },
    { 0x5c3a9d83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563a9d89, 95 },
    { 0xe9fa9d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ca9d8e, 66 },
    { 0x968a9d8f, 90 },
    { 0xe7ba9d8e, 93 },
    { 0x61ca9d8f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a9d95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65a9d9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7a9da0, 33 },
    { 0xd1fa9da1, 23 },
    { 0xd7fa9da2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93aa9da8, 55 },
    { 0x359a9da9, 52 },
    { 0x7ea9daa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca9dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1a9db2, 76 },
    { 0xef4a9db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea9db7, 87 },
    { 0xa11a9db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daa9dbc, 23 },
    { 0, 0 },
    { 0xca6a9dbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a9dc1, 96 },
    { 0x9a6a9dc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a9dc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a9dd2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729a9dd6, 90 },
    { 0x405a9dd7, 87 },
    { 0, 0 },
    { 0x87ca9dd9, 56 },
    { 0xe47a9dd9, 33 },
    { 0xb02a9dda, 95 },
    { 0, 0 },
    { 0xa20a9ddd, 98 },
    { 0x5c2a9dde, 33 },
    { 0x11ba9ddf, 95 },
    { 0, 0 },
    { 0x12ea9de1, 33 },
    { 0xf85a9de1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a9de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda9dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a9def, 48 },
    { 0xe36a9def, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40a9df4, 90 },
    { 0x88ca9df5, 23 },
    { 0x27ba9df6, 90 },
    { 0x7a9a9df7, 87 },
    { 0, 0 },
    { 0x522a9df9, 95 },
    { 0, 0 },
    { 0x36ca9dfb, 55 },
    { 0x29a9dfc, 91 },
    { 0x242a9dfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48a9e01, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217a9e06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aa9e0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373a9e12, 98 },
    { 0xa19a9e12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a9e17, 36 },
    { 0x51a9e17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1a9e1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba9e25, 61 },
    { 0x17fa9e26, 98 },
    { 0xaf5a9e25, 43 },
    { 0, 0 },
    { 0x699a9e29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a9e2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ca9e30, 52 },
    { 0xf2aa9e31, 23 },
    { 0xaeba9e31, 76 },
    { 0, 0 },
    { 0xe9fa9e34, 33 },
    { 0, 0 },
    { 0x52ca9e36, 98 },
    { 0x9eba9e36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x800a9e3a, 23 },
    { 0xf10a9e3b, 90 },
    { 0xee2a9e3c, 93 },
    { 0x122a9e3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca9e40, 33 },
    { 0, 0 },
    { 0x313a9e42, 95 },
    { 0x6f3a9e43, 34 },
    { 0xdcfa9e43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca9e48, 87 },
    { 0x5c0a9e48, 66 },
    { 0, 0 },
    { 0xeefa9e4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224a9e52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859a9e57, 23 },
    { 0x5ada9e58, 75 },
    { 0, 0 },
    { 0xbf0a9e5a, 49 },
    { 0x48ba9e5b, 43 },
    { 0x8eca9e5b, 90 },
    { 0xc2ba9e5c, 36 },
    { 0xf33a9e5a, 75 },
    { 0xad3a9e5e, 23 },
    { 0x432a9e5d, 95 },
    { 0, 0 },
    { 0xa2fa9e62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bda9e65, 76 },
    { 0x4c7a9e66, 78 },
    { 0, 0 },
    { 0x87da9e68, 85 },
    { 0xa08a9e68, 90 },
    { 0, 0 },
    { 0xec6a9e6b, 34 },
    { 0, 0 },
    { 0x187a9e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29a9e70, 77 },
    { 0x45da9e71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ba9e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9e78, 90 },
    { 0x7d5a9e79, 83 },
    { 0x639a9e78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26a9e7f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740a9e84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a9e88, 80 },
    { 0x218a9e89, 23 },
    { 0xd05a9e88, 17 },
    { 0xd1fa9e8b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22a9e8e, 61 },
    { 0x8d9a9e8f, 97 },
    { 0x53fa9e90, 93 },
    { 0x3aca9e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x909a9e94, 88 },
    { 0x36aa9e95, 33 },
    { 0xeb0a9e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ea9e9a, 43 },
    { 0, 0 },
    { 0xf22a9e9c, 34 },
    { 0, 0 },
    { 0x7ffa9e9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ba9ea3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458a9eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a9eb4, 98 },
    { 0, 0 },
    { 0xab8a9eb6, 61 },
    { 0xb80a9eb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804a9ebd, 34 },
    { 0xd45a9ebd, 33 },
    { 0x963a9ebf, 61 },
    { 0x694a9ec0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a9ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8a9ec6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a9ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87da9ecf, 61 },
    { 0x9e8a9ed0, 98 },
    { 0xb82a9ed1, 33 },
    { 0, 0 },
    { 0xdf6a9ed3, 34 },
    { 0x2fa9ed3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5a9ed7, 98 },
    { 0xa8ba9ed8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a9edd, 33 },
    { 0xa2da9edd, 67 },
    { 0, 0 },
    { 0x27da9ee0, 43 },
    { 0xa57a9ee1, 33 },
    { 0xc5aa9ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x593a9ee5, 99 },
    { 0xf75a9ee6, 90 },
    { 0x369a9ee7, 43 },
    { 0, 0 },
    { 0xfba9ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6a9ef0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cca9ef9, 17 },
    { 0xeb4a9ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a9eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122a9f04, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ea9f0c, 67 },
    { 0x509a9f0d, 90 },
    { 0, 0 },
    { 0x495a9f0f, 52 },
    { 0x20ca9f10, 61 },
    { 0, 0 },
    { 0xcb6a9f12, 61 },
    { 0x42a9f13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba9f16, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a9f19, 23 },
    { 0, 0 },
    { 0xb76a9f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49a9f1f, 46 },
    { 0, 0 },
    { 0x2bea9f21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a9f2c, 66 },
    { 0x7c3a9f2d, 90 },
    { 0x5f9a9f2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8a9f3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea9f41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ea9f44, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a9f47, 78 },
    { 0xa36a9f48, 90 },
    { 0, 0 },
    { 0xdbea9f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9f4f, 61 },
    { 0, 0 },
    { 0x859a9f51, 90 },
    { 0, 0 },
    { 0x255a9f53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fa9f5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784a9f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a9f62, 85 },
    { 0x376a9f63, 55 },
    { 0xda9a9f64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a9f67, 90 },
    { 0, 0 },
    { 0xa67a9f69, 87 },
    { 0, 0 },
    { 0x1e6a9f6b, 75 },
    { 0xc99a9f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a9f70, 23 },
    { 0x78aa9f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaa9f74, 43 },
    { 0xd87a9f75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x634a9f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x764a9f7b, 52 },
    { 0x811a9f7c, 91 },
    { 0x447a9f7d, 98 },
    { 0x29ba9f7d, 95 },
    { 0, 0 },
    { 0xc96a9f80, 23 },
    { 0, 0 },
    { 0x806a9f82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x316a9f85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195a9f8f, 78 },
    { 0x3d7a9f8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a9f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a9fa1, 95 },
    { 0xe5a9fa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea9fa5, 90 },
    { 0x32fa9fa6, 76 },
    { 0, 0 },
    { 0x7aaa9fa8, 33 },
    { 0xfcfa9fa9, 95 },
    { 0x804a9fa9, 66 },
    { 0x693a9fab, 87 },
    { 0xc20a9fab, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a9fb1, 33 },
    { 0, 0 },
    { 0x6bca9fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6a9fb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x326a9fba, 61 },
    { 0x241a9fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2a9fc4, 65 },
    { 0, 0 },
    { 0xa3a9fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aba9fc9, 23 },
    { 0, 0 },
    { 0x3a8a9fcb, 95 },
    { 0, 0 },
    { 0x8dca9fcd, 33 },
    { 0x754a9fcd, 76 },
    { 0, 0 },
    { 0x7b6a9fd0, 68 },
    { 0xb49a9fd0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a9fd4, 88 },
    { 0x147a9fd4, 34 },
    { 0xeb3a9fd6, 33 },
    { 0x950a9fd7, 43 },
    { 0x196a9fd8, 43 },
    { 0x37aa9fd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a9fdd, 23 },
    { 0, 0 },
    { 0x3bea9fdf, 55 },
    { 0xbf9a9fe0, 90 },
    { 0x2d9a9fdf, 90 },
    { 0x245a9fe2, 33 },
    { 0x57ea9fe2, 23 },
    { 0x691a9fe2, 23 },
    { 0x5a3a9fe5, 61 },
    { 0x9f9a9fe6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a9fec, 90 },
    { 0x9d3a9fed, 91 },
    { 0, 0 },
    { 0x6baa9fef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728a9ff8, 61 },
    { 0, 0 },
    { 0x5c3a9ffa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a9fff, 66 },
    { 0x974aa000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0aa009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6faa00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6aa010, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33aa013, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79aa016, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3aa022, 23 },
    { 0, 0 },
    { 0xb3aaa024, 99 },
    { 0x736aa025, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8aa029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62faa02e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572aa032, 61 },
    { 0xe19aa032, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfaa036, 23 },
    { 0, 0 },
    { 0xc45aa038, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecaa03b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aaa040, 87 },
    { 0x6bbaa041, 17 },
    { 0, 0 },
    { 0x335aa043, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x344aa046, 68 },
    { 0x74eaa047, 98 },
    { 0xcbaaa047, 76 },
    { 0xbfbaa046, 95 },
    { 0xd49aa047, 95 },
    { 0x52eaa04b, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaa04e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935aa053, 34 },
    { 0xdb3aa054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3aa059, 95 },
    { 0xaefaa05a, 33 },
    { 0xa4baa059, 100 },
    { 0, 0 },
    { 0xfefaa05d, 61 },
    { 0xa1aa05e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deaa065, 66 },
    { 0x813aa065, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2aa06a, 61 },
    { 0, 0 },
    { 0xeeeaa06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeaa07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0aa081, 95 },
    { 0, 0 },
    { 0x7eaaa083, 46 },
    { 0x703aa084, 61 },
    { 0x1fdaa085, 61 },
    { 0x6dfaa086, 46 },
    { 0, 0 },
    { 0xd42aa088, 81 },
    { 0, 0 },
    { 0x3bfaa08a, 95 },
    { 0xabbaa08a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538aa08f, 99 },
    { 0, 0 },
    { 0x44faa091, 61 },
    { 0xccaa091, 67 },
    { 0x3eaa091, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3aa096, 61 },
    { 0xfe7aa096, 90 },
    { 0xd35aa097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4aa09b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471aa0a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffaa0a8, 66 },
    { 0, 0 },
    { 0x90aaa0aa, 95 },
    { 0, 0 },
    { 0xa3eaa0ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdaa0b1, 97 },
    { 0, 0 },
    { 0xa11aa0b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aaa0b6, 76 },
    { 0, 0 },
    { 0x136aa0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaaa0bb, 91 },
    { 0, 0 },
    { 0xd2eaa0bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4aa0c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40aa0c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04aa0cd, 76 },
    { 0, 0 },
    { 0x340aa0cf, 90 },
    { 0x10eaa0d0, 23 },
    { 0xe2eaa0d1, 61 },
    { 0xe27aa0d0, 76 },
    { 0x8a2aa0d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5aa0d6, 95 },
    { 0x7c0aa0d7, 33 },
    { 0x9e2aa0d6, 78 },
    { 0, 0 },
    { 0xd0aa0da, 61 },
    { 0x7e4aa0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x453aa0de, 93 },
    { 0, 0 },
    { 0x68eaa0e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660aa0e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8aa0ee, 85 },
    { 0x488aa0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49daa0f4, 90 },
    { 0xaa2aa0f5, 95 },
    { 0xee4aa0f5, 87 },
    { 0xdeaa0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4feaa0fb, 90 },
    { 0, 0 },
    { 0x820aa0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85aa105, 23 },
    { 0x5c5aa106, 77 },
    { 0x9fbaa107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eaa10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2aa114, 43 },
    { 0xca7aa115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfaa11a, 17 },
    { 0, 0 },
    { 0x8f1aa11c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25aa122, 76 },
    { 0xba0aa122, 95 },
    { 0x3d1aa124, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefaa12e, 90 },
    { 0x390aa12f, 65 },
    { 0xd6baa130, 33 },
    { 0x5f2aa12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40faa139, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x981aa13c, 95 },
    { 0x23daa13d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296aa144, 52 },
    { 0x2d9aa145, 90 },
    { 0xcefaa145, 91 },
    { 0, 0 },
    { 0x2b2aa148, 100 },
    { 0x9e4aa149, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1aa14e, 95 },
    { 0, 0 },
    { 0x4b9aa150, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41aa154, 81 },
    { 0x267aa155, 33 },
    { 0xe55aa156, 99 },
    { 0x520aa157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311aa15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2aa15e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06aa162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aa165, 33 },
    { 0, 0 },
    { 0xe4aa167, 68 },
    { 0x83aa168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64faa16f, 90 },
    { 0x5ebaa170, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551aa17a, 93 },
    { 0xc08aa17a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85daa180, 90 },
    { 0x62faa180, 98 },
    { 0x11eaa182, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0aa189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x786aa18c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6aa190, 23 },
    { 0, 0 },
    { 0xef7aa192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399aa198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaaa19d, 76 },
    { 0x23aaa19e, 99 },
    { 0xc57aa19e, 100 },
    { 0xef8aa1a0, 90 },
    { 0, 0 },
    { 0x74baa1a2, 90 },
    { 0xcebaa1a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfaa1a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aaa1ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x815aa1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eaa1b5, 86 },
    { 0x72aaa1b6, 33 },
    { 0x9aaa1b6, 99 },
    { 0, 0 },
    { 0x282aa1b9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374aa1bd, 55 },
    { 0, 0 },
    { 0x188aa1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792aa1c3, 33 },
    { 0xbb1aa1c4, 95 },
    { 0x743aa1c5, 52 },
    { 0xf85aa1c6, 61 },
    { 0, 0 },
    { 0xf00aa1c8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189aa1cd, 55 },
    { 0x2a3aa1ce, 84 },
    { 0x1c9aa1cf, 91 },
    { 0xf03aa1d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08aa1d4, 23 },
    { 0, 0 },
    { 0x35eaa1d6, 46 },
    { 0x52daa1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4aa1dc, 93 },
    { 0xc7daa1dc, 61 },
    { 0, 0 },
    { 0x793aa1df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196aa1e3, 33 },
    { 0x2deaa1e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2aa1ec, 100 },
    { 0, 0 },
    { 0x917aa1ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eaa1f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19aa1fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35aa200, 76 },
    { 0x85aaa201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1aa204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9aa208, 95 },
    { 0xb46aa209, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eaa20d, 33 },
    { 0x8abaa20d, 99 },
    { 0, 0 },
    { 0x7e1aa210, 55 },
    { 0x9a2aa211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daa219, 78 },
    { 0x294aa219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfaa21f, 65 },
    { 0xb27aa220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceaa224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baa22b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64aa241, 61 },
    { 0x667aa241, 93 },
    { 0x532aa241, 23 },
    { 0xe63aa243, 33 },
    { 0xe4faa243, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffaa248, 91 },
    { 0xa23aa248, 33 },
    { 0x3f3aa248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9aa24d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baaa250, 33 },
    { 0, 0 },
    { 0xfd9aa252, 33 },
    { 0x9adaa253, 55 },
    { 0, 0 },
    { 0x44caa255, 90 },
    { 0x76aa256, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46aa25e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15daa264, 90 },
    { 0xf5aaa265, 33 },
    { 0x65daa266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94baa269, 34 },
    { 0xffdaa26a, 46 },
    { 0, 0 },
    { 0x69caa26c, 36 },
    { 0, 0 },
    { 0xd2faa26e, 98 },
    { 0xecaa26f, 90 },
    { 0xd72aa270, 76 },
    { 0x83caa271, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x951aa274, 61 },
    { 0x847aa275, 17 },
    { 0x73aa275, 61 },
    { 0, 0 },
    { 0x83eaa278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7aa27e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504aa285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x838aa288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270aa28f, 23 },
    { 0, 0 },
    { 0xd37aa291, 33 },
    { 0x29aa292, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afaa297, 83 },
    { 0x3f6aa297, 69 },
    { 0xbacaa299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67aa29c, 90 },
    { 0xe14aa29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151aa2a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37aa2a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12daa2aa, 43 },
    { 0x793aa2ab, 95 },
    { 0x979aa2ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x33daa2af, 33 },
    { 0xca8aa2af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdaa2b3, 4 },
    { 0x339aa2b3, 91 },
    { 0xe78aa2b5, 84 },
    { 0xeb0aa2b6, 98 },
    { 0xf00aa2b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4aa2ba, 90 },
    { 0x58daa2ba, 61 },
    { 0x85caa2bc, 68 },
    { 0xe00aa2bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61faa2c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6aa2c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29faa2cc, 66 },
    { 0, 0 },
    { 0x5a3aa2ce, 33 },
    { 0x950aa2ce, 100 },
    { 0x443aa2d0, 33 },
    { 0, 0 },
    { 0xebdaa2d2, 95 },
    { 0, 0 },
    { 0xacaaa2d4, 33 },
    { 0xb38aa2d4, 34 },
    { 0x87aa2d6, 95 },
    { 0x74daa2d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaaa2de, 46 },
    { 0x281aa2df, 33 },
    { 0, 0 },
    { 0xf0daa2e1, 67 },
    { 0, 0 },
    { 0x9a6aa2e3, 23 },
    { 0x25aa2e3, 61 },
    { 0, 0 },
    { 0x13aaa2e6, 98 },
    { 0xa3eaa2e7, 69 },
    { 0x31caa2e8, 95 },
    { 0, 0 },
    { 0x6f8aa2ea, 87 },
    { 0xe2aa2eb, 23 },
    { 0x28daa2eb, 43 },
    { 0, 0 },
    { 0x5e7aa2ee, 93 },
    { 0x7a7aa2ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3aa2f4, 55 },
    { 0, 0 },
    { 0x968aa2f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27aa2ff, 98 },
    { 0x43eaa300, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x726aa303, 43 },
    { 0, 0 },
    { 0x51faa305, 23 },
    { 0, 0 },
    { 0x9d3aa307, 61 },
    { 0, 0 },
    { 0xce9aa309, 34 },
    { 0, 0 },
    { 0x6a8aa30b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8aa310, 61 },
    { 0x6fcaa311, 67 },
    { 0x2d5aa312, 76 },
    { 0x909aa313, 97 },
    { 0x4ecaa313, 23 },
    { 0x43baa315, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83aa318, 93 },
    { 0xd1eaa319, 95 },
    { 0xebcaa319, 34 },
    { 0x842aa31b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacaa321, 61 },
    { 0, 0 },
    { 0x111aa323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaa328, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eaa32d, 90 },
    { 0, 0 },
    { 0x663aa32f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adaa334, 33 },
    { 0x858aa335, 78 },
    { 0x4d9aa335, 61 },
    { 0, 0 },
    { 0xcd1aa338, 98 },
    { 0x855aa339, 23 },
    { 0xed8aa339, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20baa341, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67faa345, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1aa349, 46 },
    { 0xe65aa34a, 90 },
    { 0x907aa34b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4baa34e, 34 },
    { 0x142aa34f, 98 },
    { 0x516aa34f, 83 },
    { 0x473aa351, 90 },
    { 0xcdbaa352, 23 },
    { 0, 0 },
    { 0x204aa354, 61 },
    { 0x596aa355, 61 },
    { 0, 0 },
    { 0x88eaa357, 75 },
    { 0, 0 },
    { 0x483aa359, 98 },
    { 0, 0 },
    { 0x719aa35b, 90 },
    { 0x29caa35c, 33 },
    { 0xf70aa35d, 93 },
    { 0x7d2aa35c, 95 },
    { 0xec6aa35b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5aa364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16caa36c, 93 },
    { 0x936aa36d, 90 },
    { 0x7bcaa36e, 66 },
    { 0, 0 },
    { 0x306aa370, 90 },
    { 0, 0 },
    { 0xd81aa372, 33 },
    { 0x903aa373, 98 },
    { 0xbe1aa373, 61 },
    { 0xd20aa372, 90 },
    { 0x6a7aa376, 33 },
    { 0x9daaa376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7aa37a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6aa37e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1aa381, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180aa385, 43 },
    { 0x9ceaa385, 33 },
    { 0x770aa387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07aa38b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2aa38f, 78 },
    { 0x37daa390, 69 },
    { 0xf18aa391, 33 },
    { 0x724aa392, 23 },
    { 0, 0 },
    { 0x1aaa394, 85 },
    { 0, 0 },
    { 0xaccaa396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51caa3a0, 90 },
    { 0xd8aaa3a1, 23 },
    { 0xdc4aa3a2, 76 },
    { 0x606aa3a0, 75 },
    { 0x217aa3a4, 90 },
    { 0, 0 },
    { 0x168aa3a6, 33 },
    { 0x65caa3a6, 95 },
    { 0, 0 },
    { 0xb5aa3a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948aa3b0, 93 },
    { 0, 0 },
    { 0x991aa3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x864aa3b5, 90 },
    { 0, 0 },
    { 0x596aa3b7, 100 },
    { 0x11aa3b8, 75 },
    { 0, 0 },
    { 0x276aa3ba, 76 },
    { 0xcb9aa3bb, 23 },
    { 0xfd5aa3bc, 23 },
    { 0x60daa3bd, 77 },
    { 0x6bbaa3be, 90 },
    { 0, 0 },
    { 0x2b3aa3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x105aa3c3, 43 },
    { 0xe11aa3c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddaa3c8, 23 },
    { 0xd1daa3c8, 17 },
    { 0xd7daa3ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0aa3ce, 33 },
    { 0x174aa3ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888aa3d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6aa3d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752aa3e2, 95 },
    { 0, 0 },
    { 0xc60aa3e4, 33 },
    { 0x74daa3e5, 55 },
    { 0, 0 },
    { 0x437aa3e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2aa3ea, 76 },
    { 0xf49aa3eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaa3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955aa3f7, 23 },
    { 0x963aa3f7, 34 },
    { 0xc4baa3f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621aa400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eaa403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195aa40b, 77 },
    { 0x943aa40c, 23 },
    { 0xef7aa40c, 23 },
    { 0, 0 },
    { 0x309aa40f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200aa413, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0aa41b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3aa41f, 55 },
    { 0, 0 },
    { 0xd73aa421, 95 },
    { 0, 0 },
    { 0x4c0aa423, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954aa429, 76 },
    { 0x2faa429, 90 },
    { 0, 0 },
    { 0x7e8aa42c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9aa433, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339aa437, 61 },
    { 0xddeaa438, 80 },
    { 0x9a9aa438, 46 },
    { 0xe8faa43a, 46 },
    { 0xc18aa437, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0aa43f, 46 },
    { 0xcd9aa43f, 33 },
    { 0xb1daa441, 38 },
    { 0, 0 },
    { 0x538aa443, 93 },
    { 0x7beaa443, 95 },
    { 0xd2baa443, 55 },
    { 0, 0 },
    { 0x1c0aa447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0aa44e, 90 },
    { 0x8cfaa44f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x818aa452, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x423aa455, 43 },
    { 0xe7faa456, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602aa45a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9aa45f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdaa463, 33 },
    { 0x111aa464, 95 },
    { 0x462aa465, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96aa469, 98 },
    { 0, 0 },
    { 0x663aa46b, 46 },
    { 0x5d5aa46c, 95 },
    { 0x6fcaa46c, 80 },
    { 0xf15aa46b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7aa471, 65 },
    { 0x35faa472, 95 },
    { 0, 0 },
    { 0x4c8aa474, 84 },
    { 0, 0 },
    { 0x771aa476, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6daa486, 66 },
    { 0xe20aa487, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa48f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832aa498, 55 },
    { 0x375aa498, 89 },
    { 0, 0 },
    { 0x2f7aa49b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaaa4a4, 98 },
    { 0xbb6aa4a5, 90 },
    { 0x7a9aa4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250aa4aa, 76 },
    { 0xea7aa4ab, 95 },
    { 0xa9baa4ab, 33 },
    { 0xff4aa4ad, 23 },
    { 0xef4aa4ae, 90 },
    { 0xc69aa4af, 90 },
    { 0xef2aa4ae, 55 },
    { 0, 0 },
    { 0x360aa4b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77baa4b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34aa4bc, 90 },
    { 0xf06aa4bc, 84 },
    { 0x6acaa4be, 90 },
    { 0x23daa4bf, 61 },
    { 0, 0 },
    { 0xc3eaa4c1, 61 },
    { 0x2a5aa4c2, 76 },
    { 0, 0 },
    { 0xc51aa4c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21aa4c9, 55 },
    { 0x6d9aa4ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aaa4d0, 85 },
    { 0, 0 },
    { 0x969aa4d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5aa4d6, 90 },
    { 0x5ddaa4d7, 34 },
    { 0x77faa4d6, 33 },
    { 0x429aa4d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22aa4dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4baa4df, 23 },
    { 0x5f0aa4df, 90 },
    { 0, 0 },
    { 0x2d3aa4e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5aa4e9, 23 },
    { 0xc2faa4ea, 99 },
    { 0, 0 },
    { 0x4f0aa4ec, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42aa4f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdaa4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860aa4f9, 33 },
    { 0xffaa4fa, 33 },
    { 0x55aaa4fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26baa4ff, 43 },
    { 0, 0 },
    { 0x3f2aa501, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aaa505, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eaa508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9aa50d, 90 },
    { 0, 0 },
    { 0xcabaa50f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeceaa51b, 78 },
    { 0x62aa51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daaa520, 55 },
    { 0x283aa521, 52 },
    { 0, 0 },
    { 0x3c1aa523, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdaa52c, 76 },
    { 0, 0 },
    { 0x63daa52e, 61 },
    { 0x479aa52f, 33 },
    { 0x81aaa52f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aa534, 98 },
    { 0x64aaa535, 36 },
    { 0x257aa535, 78 },
    { 0xbf1aa537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x575aa53a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4aa53e, 90 },
    { 0x55caa53f, 66 },
    { 0xba7aa53e, 76 },
    { 0x169aa541, 98 },
    { 0xab0aa541, 23 },
    { 0xbd1aa543, 46 },
    { 0x378aa541, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3aa54c, 93 },
    { 0xfe2aa54c, 66 },
    { 0xa08aa54e, 33 },
    { 0x28aa54f, 98 },
    { 0xe8eaa54c, 55 },
    { 0, 0 },
    { 0xb46aa552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aaaa557, 17 },
    { 0x52caa557, 87 },
    { 0x5caaa559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa55c, 61 },
    { 0x644aa55d, 52 },
    { 0xb84aa55c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55baa563, 100 },
    { 0xddaaa563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7baa567, 68 },
    { 0xf8eaa568, 9 },
    { 0x4f4aa569, 46 },
    { 0x25aaa568, 95 },
    { 0x3e7aa56b, 52 },
    { 0x717aa56c, 76 },
    { 0, 0 },
    { 0xe76aa56e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06aa571, 90 },
    { 0, 0 },
    { 0x5afaa573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25aa577, 90 },
    { 0, 0 },
    { 0xd47aa579, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x24caa57c, 55 },
    { 0x374aa57d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2aa580, 90 },
    { 0x2bfaa581, 33 },
    { 0xe99aa582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebaa58a, 33 },
    { 0x470aa58b, 78 },
    { 0x193aa58c, 33 },
    { 0xccfaa58a, 61 },
    { 0xcd8aa58e, 98 },
    { 0x7b0aa58f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2daa594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21aa598, 95 },
    { 0x311aa599, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x697aa59c, 95 },
    { 0x270aa59c, 98 },
    { 0xd41aa59d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32daa5a1, 33 },
    { 0, 0 },
    { 0x9eaa5a3, 78 },
    { 0x441aa5a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aaa5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x841aa5aa, 55 },
    { 0, 0 },
    { 0xe09aa5ac, 61 },
    { 0, 0 },
    { 0x984aa5ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9aa5b1, 90 },
    { 0x990aa5b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x611aa5b5, 91 },
    { 0xf70aa5b6, 23 },
    { 0x186aa5b5, 34 },
    { 0x30eaa5b8, 55 },
    { 0x170aa5b7, 84 },
    { 0xdd8aa5b8, 23 },
    { 0xfbbaa5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aaa5be, 95 },
    { 0x643aa5bf, 61 },
    { 0xdc0aa5c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6aa5c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edaa5c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93aa5cc, 33 },
    { 0, 0 },
    { 0x6eeaa5ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deaa5d2, 93 },
    { 0x3bcaa5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efaa5dc, 90 },
    { 0x90caa5dd, 90 },
    { 0x412aa5dd, 90 },
    { 0x9dcaa5df, 87 },
    { 0x48aaa5dd, 33 },
    { 0, 0 },
    { 0xf7eaa5e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3aa5e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4aa5eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05aa5f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4aa5f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3aa602, 95 },
    { 0xb27aa603, 55 },
    { 0xdb8aa604, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa607, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73aa60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574aa610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa614, 55 },
    { 0xbc5aa615, 90 },
    { 0xf34aa616, 33 },
    { 0x35aa617, 90 },
    { 0x567aa618, 33 },
    { 0, 0 },
    { 0x77aa61a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa61e, 33 },
    { 0, 0 },
    { 0x84aa620, 55 },
    { 0xde5aa621, 43 },
    { 0x2e4aa620, 95 },
    { 0x870aa623, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aaa626, 33 },
    { 0xd41aa627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8baa62c, 98 },
    { 0x7d3aa62d, 34 },
    { 0x3c4aa62d, 51 },
    { 0xa45aa62f, 33 },
    { 0xae9aa630, 33 },
    { 0x2deaa631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aa639, 93 },
    { 0x6aeaa639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6aa63d, 61 },
    { 0x277aa63e, 90 },
    { 0xa81aa63f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96caa643, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182aa64a, 68 },
    { 0x421aa64b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673aa654, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43faa657, 23 },
    { 0, 0 },
    { 0xe08aa659, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x204aa65c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832aa663, 80 },
    { 0x62eaa664, 33 },
    { 0, 0 },
    { 0xaf4aa666, 95 },
    { 0x9c8aa666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfaa66b, 98 },
    { 0x70eaa66c, 91 },
    { 0xc1baa66b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90aa670, 95 },
    { 0, 0 },
    { 0x53eaa672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaaa675, 90 },
    { 0xffbaa676, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14aa67a, 52 },
    { 0x39eaa67b, 33 },
    { 0x6d0aa67c, 98 },
    { 0, 0 },
    { 0xe68aa67e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x502aa681, 52 },
    { 0, 0 },
    { 0xf6daa683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859aa687, 33 },
    { 0x317aa687, 56 },
    { 0, 0 },
    { 0x18aaa68a, 95 },
    { 0, 0 },
    { 0x91aaa68c, 68 },
    { 0x34aaa68c, 76 },
    { 0x212aa68e, 68 },
    { 0xfb6aa68f, 90 },
    { 0x7f4aa690, 90 },
    { 0x92aa691, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121aa695, 95 },
    { 0xdcaa695, 90 },
    { 0x3b2aa697, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5aa69a, 87 },
    { 0x93aa69a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0aa6ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6aa6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daaa6b6, 23 },
    { 0xd67aa6b7, 66 },
    { 0xb58aa6b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beaa6bc, 33 },
    { 0, 0 },
    { 0xd97aa6be, 55 },
    { 0xa6aa6be, 11 },
    { 0x23aaa6c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x741aa6c3, 61 },
    { 0x249aa6c4, 33 },
    { 0, 0 },
    { 0xeaeaa6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0aa6cf, 77 },
    { 0x9e0aa6cf, 55 },
    { 0x46aaa6d1, 46 },
    { 0xcf9aa6cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7baa6d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17baa6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535aa6e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aaa6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73aa6f4, 78 },
    { 0x778aa6f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13faa6f8, 23 },
    { 0x81faa6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9aa6fc, 99 },
    { 0xd4caa6fc, 38 },
    { 0, 0 },
    { 0x9b1aa6ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12aa704, 23 },
    { 0x667aa705, 34 },
    { 0, 0 },
    { 0x2deaa707, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579aa70b, 34 },
    { 0x8daaa70b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0aa70f, 91 },
    { 0, 0 },
    { 0x136aa711, 98 },
    { 0, 0 },
    { 0x7c6aa713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50faa719, 56 },
    { 0xbfaa71a, 80 },
    { 0x4e8aa71b, 95 },
    { 0xd9caa719, 34 },
    { 0x274aa71d, 38 },
    { 0x8daa71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13aa721, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x472aa724, 40 },
    { 0x4c6aa725, 90 },
    { 0x9ccaa726, 90 },
    { 0x8cbaa727, 100 },
    { 0x97daa726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7aa72c, 96 },
    { 0xf30aa72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1aa730, 34 },
    { 0x456aa730, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99aa735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5aa739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aa73d, 33 },
    { 0xbbcaa73e, 98 },
    { 0x1c5aa73f, 65 },
    { 0x896aa740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73aa743, 52 },
    { 0x95baa743, 95 },
    { 0, 0 },
    { 0x5abaa746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x331aa749, 90 },
    { 0x79eaa74a, 66 },
    { 0x16baa74b, 90 },
    { 0xcffaa74c, 90 },
    { 0, 0 },
    { 0xedfaa74e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03aa756, 33 },
    { 0xe6faa757, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25caa75c, 23 },
    { 0xadaaa75d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1aa760, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71aa763, 23 },
    { 0x8eaa764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9aa769, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa76d, 61 },
    { 0x60aa76d, 95 },
    { 0x37aaa76f, 33 },
    { 0x75aaa770, 95 },
    { 0x6b0aa771, 43 },
    { 0x93faa770, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2aa77a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceaa77d, 98 },
    { 0x35eaa77e, 55 },
    { 0, 0 },
    { 0xfb9aa780, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0daa786, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2aa789, 33 },
    { 0xf20aa789, 95 },
    { 0, 0 },
    { 0x42eaa78c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x340aa78f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaa793, 33 },
    { 0xae6aa793, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9aa799, 99 },
    { 0x215aa79a, 88 },
    { 0x4dcaa79a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adaa79f, 90 },
    { 0, 0 },
    { 0x8d2aa7a1, 52 },
    { 0, 0 },
    { 0x448aa7a3, 33 },
    { 0xff4aa7a4, 78 },
    { 0x1e6aa7a4, 33 },
    { 0, 0 },
    { 0xc88aa7a7, 90 },
    { 0x849aa7a8, 78 },
    { 0x4a1aa7a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfaa7ac, 98 },
    { 0xe62aa7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65daa7b5, 96 },
    { 0, 0 },
    { 0x573aa7b7, 33 },
    { 0xd3aaa7b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa7be, 23 },
    { 0, 0 },
    { 0x824aa7c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x200aa7c3, 33 },
    { 0x348aa7c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4daa7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82aa7cc, 17 },
    { 0x601aa7cc, 90 },
    { 0, 0 },
    { 0x7b0aa7cf, 76 },
    { 0, 0 },
    { 0xa4caa7d1, 76 },
    { 0x1a7aa7d2, 23 },
    { 0x9acaa7d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29aa7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104aa7dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8aa7df, 95 },
    { 0, 0 },
    { 0xcc3aa7e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0aa7e4, 90 },
    { 0x761aa7e4, 43 },
    { 0x5aaaa7e5, 23 },
    { 0x1c3aa7e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2aa7eb, 55 },
    { 0, 0 },
    { 0xde9aa7ed, 55 },
    { 0x2fcaa7ee, 98 },
    { 0, 0 },
    { 0x8f5aa7f0, 66 },
    { 0xe0aa7f0, 98 },
    { 0, 0 },
    { 0x597aa7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaa7f6, 90 },
    { 0x2f7aa7f7, 84 },
    { 0x17faa7f8, 93 },
    { 0xe3faa7f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41aa801, 76 },
    { 0xa1caa802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x130aa805, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8aa80b, 82 },
    { 0x734aa80c, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefaa811, 91 },
    { 0, 0 },
    { 0x248aa813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12aa816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eaa81a, 78 },
    { 0x285aa81a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50aa81f, 95 },
    { 0x6f9aa820, 61 },
    { 0xacdaa821, 90 },
    { 0xd33aa822, 81 },
    { 0xa5baa821, 17 },
    { 0, 0 },
    { 0xfc3aa825, 17 },
    { 0xc0aaa826, 33 },
    { 0x32aaa827, 90 },
    { 0, 0 },
    { 0x487aa829, 76 },
    { 0, 0 },
    { 0x40caa82b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2aa832, 33 },
    { 0, 0 },
    { 0x65daa834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4aa83c, 85 },
    { 0xac5aa83d, 68 },
    { 0x2b5aa83e, 61 },
    { 0, 0 },
    { 0xc8aa840, 84 },
    { 0x957aa840, 43 },
    { 0xdaeaa840, 76 },
    { 0x9d7aa843, 100 },
    { 0xe8faa841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62daa847, 98 },
    { 0, 0 },
    { 0x7a8aa849, 90 },
    { 0x8f6aa84a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09aa84d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629aa856, 95 },
    { 0xe48aa857, 68 },
    { 0x63aaa857, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x348aa85b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95aa85e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54aa863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aa866, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708aa869, 90 },
    { 0xc61aa86a, 83 },
    { 0xddbaa86b, 95 },
    { 0x756aa86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecaa86f, 23 },
    { 0xcfdaa870, 90 },
    { 0, 0 },
    { 0xf6aa872, 55 },
    { 0xd4daa872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaaa878, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5aa87e, 61 },
    { 0x6ffaa87f, 99 },
    { 0x47aaa87f, 55 },
    { 0x900aa881, 23 },
    { 0, 0 },
    { 0xc0baa883, 85 },
    { 0x897aa884, 100 },
    { 0x59baa885, 33 },
    { 0, 0 },
    { 0xc02aa887, 67 },
    { 0, 0 },
    { 0xc18aa889, 23 },
    { 0xf64aa889, 95 },
    { 0xce5aa88a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aeaa88e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7aa892, 55 },
    { 0x754aa893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5aa89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96aa8a1, 23 },
    { 0x8f1aa8a2, 61 },
    { 0xae3aa8a3, 65 },
    { 0xb30aa8a1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2aa8a7, 61 },
    { 0x6b1aa8a8, 17 },
    { 0x70aaa8a7, 90 },
    { 0xbcaa8aa, 61 },
    { 0x827aa8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5aa8b0, 91 },
    { 0, 0 },
    { 0x327aa8b2, 33 },
    { 0, 0 },
    { 0xa9eaa8b4, 46 },
    { 0xdacaa8b5, 93 },
    { 0xcb8aa8b6, 85 },
    { 0x53aaa8b7, 95 },
    { 0, 0 },
    { 0x7caa8b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x583aa8bc, 98 },
    { 0, 0 },
    { 0xc97aa8be, 99 },
    { 0x729aa8be, 100 },
    { 0, 0 },
    { 0x6e7aa8c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eaa8c5, 33 },
    { 0x342aa8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18aa8cb, 46 },
    { 0xd5baa8cc, 61 },
    { 0x93daa8cc, 90 },
    { 0x861aa8cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aa8d1, 22 },
    { 0x22aaa8d2, 65 },
    { 0xf3baa8d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4aa8d9, 51 },
    { 0x396aa8da, 77 },
    { 0xc20aa8d9, 76 },
    { 0xef7aa8da, 23 },
    { 0xf76aa8dd, 98 },
    { 0xaa6aa8de, 43 },
    { 0, 0 },
    { 0xff7aa8e0, 90 },
    { 0x510aa8e1, 43 },
    { 0x656aa8e0, 46 },
    { 0x392aa8e3, 78 },
    { 0, 0 },
    { 0x130aa8e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377aa8e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfaa8ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbaa8f0, 61 },
    { 0, 0 },
    { 0xe51aa8f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbaa8fc, 67 },
    { 0x6faa8fd, 98 },
    { 0xc1aa8fd, 84 },
    { 0x648aa8ff, 95 },
    { 0xa28aa8fe, 61 },
    { 0, 0 },
    { 0xb75aa902, 95 },
    { 0, 0 },
    { 0x7dbaa904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x955aa907, 46 },
    { 0x718aa908, 75 },
    { 0xc89aa909, 78 },
    { 0, 0 },
    { 0xfc4aa90b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97baa913, 76 },
    { 0, 0 },
    { 0xe03aa915, 33 },
    { 0xe71aa916, 55 },
    { 0, 0 },
    { 0x301aa918, 67 },
    { 0xc9caa918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40caa91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6aa91f, 98 },
    { 0xb9aaa91f, 90 },
    { 0, 0 },
    { 0xdccaa922, 76 },
    { 0x961aa923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9aa926, 67 },
    { 0xa1aa927, 61 },
    { 0xec0aa926, 33 },
    { 0xf64aa929, 91 },
    { 0x508aa92a, 90 },
    { 0x7aaa927, 90 },
    { 0x76daa92c, 99 },
    { 0xa1daa92d, 84 },
    { 0xc74aa92e, 34 },
    { 0x55aaa92f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829aa938, 90 },
    { 0, 0 },
    { 0x9e8aa93a, 88 },
    { 0xf04aa93b, 46 },
    { 0xfd0aa93a, 33 },
    { 0x5c6aa93c, 98 },
    { 0, 0 },
    { 0x96eaa93f, 87 },
    { 0x472aa940, 98 },
    { 0xbf5aa941, 23 },
    { 0, 0 },
    { 0xa20aa943, 98 },
    { 0x95daa944, 53 },
    { 0xd8faa945, 61 },
    { 0xe89aa946, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357aa952, 95 },
    { 0, 0 },
    { 0x62aaa954, 68 },
    { 0x605aa954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6aa960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318aa965, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91baa96a, 77 },
    { 0xba1aa96b, 88 },
    { 0xef4aa96c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eaa973, 76 },
    { 0x666aa974, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2aa97d, 98 },
    { 0, 0 },
    { 0x13eaa97f, 61 },
    { 0xe82aa97f, 46 },
    { 0, 0 },
    { 0xfd0aa982, 33 },
    { 0, 0 },
    { 0xbf7aa984, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacaa987, 61 },
    { 0x55faa988, 81 },
    { 0x23aaa988, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11aa98d, 23 },
    { 0x3c5aa98d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21aa993, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2aa999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6aa99c, 61 },
    { 0, 0 },
    { 0xdd1aa99e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8aa9a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717aa9af, 93 },
    { 0x6eaa9af, 90 },
    { 0x854aa9af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42aa9b7, 66 },
    { 0x438aa9b8, 77 },
    { 0x1cbaa9b8, 67 },
    { 0xb9daa9ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96aa9c0, 77 },
    { 0x2eaa9c0, 52 },
    { 0x46daa9c2, 46 },
    { 0x483aa9c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18aa9c6, 95 },
    { 0x25aa9c7, 61 },
    { 0x97faa9c7, 90 },
    { 0, 0 },
    { 0x488aa9ca, 55 },
    { 0, 0 },
    { 0xe43aa9cc, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1aa9d3, 99 },
    { 0x6e6aa9d3, 98 },
    { 0xb90aa9d5, 100 },
    { 0xc41aa9d5, 66 },
    { 0xbeaa9d7, 34 },
    { 0, 0 },
    { 0x619aa9d9, 23 },
    { 0, 0 },
    { 0x6daaa9db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723aa9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966aa9ea, 33 },
    { 0xa83aa9eb, 23 },
    { 0xb23aa9ec, 61 },
    { 0xb83aa9ed, 93 },
    { 0xdbfaa9ee, 34 },
    { 0xeb1aa9ea, 23 },
    { 0, 0 },
    { 0x32faa9f1, 76 },
    { 0x71faa9f1, 67 },
    { 0x2faaa9f1, 61 },
    { 0x9afaa9f4, 90 },
    { 0, 0 },
    { 0x18faa9f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccaa9f9, 23 },
    { 0x21baa9fa, 77 },
    { 0x391aa9fa, 33 },
    { 0x769aa9fc, 90 },
    { 0x9b4aa9fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaaa01, 46 },
    { 0, 0 },
    { 0xa79aaa03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974aaa07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfaaa0d, 61 },
    { 0x34faaa0e, 93 },
    { 0x680aaa0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aaaa14, 23 },
    { 0xec2aaa15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadaaa1a, 61 },
    { 0xcb6aaa1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50aaa1e, 76 },
    { 0xf51aaa1e, 40 },
    { 0xebbaaa1f, 67 },
    { 0x829aaa20, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76aaa26, 40 },
    { 0x39caaa27, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72baaa2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x125aaa31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0aaa35, 61 },
    { 0x74eaaa35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275aaa3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739aaa41, 33 },
    { 0xeaeaaa42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0aaa46, 90 },
    { 0x1fcaaa47, 84 },
    { 0x1dbaaa48, 33 },
    { 0x2a6aaa49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738aaa4d, 33 },
    { 0x737aaa4e, 52 },
    { 0x241aaa4e, 98 },
    { 0, 0 },
    { 0x54baaa51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaaa59, 55 },
    { 0x156aaa5a, 61 },
    { 0, 0 },
    { 0x469aaa5c, 77 },
    { 0x812aaa5c, 84 },
    { 0, 0 },
    { 0x80aaa5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x63baaa62, 43 },
    { 0, 0 },
    { 0x452aaa64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caaa6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbaaa71, 33 },
    { 0xbb7aaa72, 76 },
    { 0xb98aaa73, 90 },
    { 0x51caaa74, 90 },
    { 0x45eaaa75, 34 },
    { 0xfd1aaa73, 38 },
    { 0xedaaaa77, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69baaa7a, 34 },
    { 0x336aaa7b, 78 },
    { 0, 0 },
    { 0xf14aaa7d, 71 },
    { 0x448aaa7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabaaa81, 95 },
    { 0x516aaa82, 36 },
    { 0x60baaa81, 93 },
    { 0, 0 },
    { 0xbadaaa85, 52 },
    { 0x953aaa86, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0aaa89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aaa8c, 98 },
    { 0, 0 },
    { 0x8d4aaa8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86caaa93, 98 },
    { 0x894aaa94, 90 },
    { 0x97baaa93, 98 },
    { 0x331aaa93, 33 },
    { 0, 0 },
    { 0x192aaa98, 61 },
    { 0xaabaaa99, 84 },
    { 0x587aaa9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1aaa9d, 61 },
    { 0, 0 },
    { 0xddbaaa9f, 55 },
    { 0x29caaa9f, 55 },
    { 0, 0 },
    { 0xca6aaaa2, 47 },
    { 0x2b5aaaa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13aaaa7, 90 },
    { 0xa4daaaa8, 77 },
    { 0x36daaaa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aaaaad, 34 },
    { 0x4c3aaaad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8aaab6, 95 },
    { 0xf16aaab7, 33 },
    { 0x7a1aaab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9daaabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faaac2, 52 },
    { 0xe20aaac3, 90 },
    { 0xd1faaac3, 33 },
    { 0x4aaaaac5, 33 },
    { 0x37caaac6, 90 },
    { 0x1afaaac7, 99 },
    { 0, 0 },
    { 0xff6aaac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1aaace, 90 },
    { 0xbeaaace, 96 },
    { 0x4e0aaad0, 52 },
    { 0, 0 },
    { 0x927aaad2, 95 },
    { 0xeaaaaad3, 90 },
    { 0xa1aaad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbaaadb, 55 },
    { 0xbdaaadc, 34 },
    { 0, 0 },
    { 0xb7eaaade, 90 },
    { 0x67aaaadf, 61 },
    { 0x3beaaae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aaae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743aaae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5aaaec, 95 },
    { 0x9ddaaaed, 23 },
    { 0x528aaaed, 75 },
    { 0xf98aaaef, 34 },
    { 0, 0 },
    { 0x92baaaf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9aaaf5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1aaaff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7aab05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53aaab09, 98 },
    { 0, 0 },
    { 0x21aaab0b, 46 },
    { 0, 0 },
    { 0x7b7aab0d, 90 },
    { 0x8cfaab0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01aab12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79faab17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6aab1f, 40 },
    { 0, 0 },
    { 0xf25aab21, 76 },
    { 0x662aab21, 95 },
    { 0, 0 },
    { 0xddaab24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x150aab27, 76 },
    { 0xe41aab27, 52 },
    { 0xc10aab28, 64 },
    { 0x641aab27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x522aab2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20aab34, 90 },
    { 0xa5caab35, 90 },
    { 0xe8daab35, 93 },
    { 0xa91aab36, 93 },
    { 0x882aab38, 95 },
    { 0x3f0aab39, 33 },
    { 0x546aab35, 95 },
    { 0xa69aab3b, 85 },
    { 0, 0 },
    { 0x96aab3d, 55 },
    { 0xd3eaab3d, 61 },
    { 0xc0aaab3f, 98 },
    { 0xcacaab3d, 61 },
    { 0x777aab41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16aab45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eaab49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9faab4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191aab51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658aab57, 51 },
    { 0x24baab58, 65 },
    { 0, 0 },
    { 0xc15aab5a, 34 },
    { 0, 0 },
    { 0x7ccaab5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5aab63, 33 },
    { 0xdf3aab64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4caab67, 61 },
    { 0x593aab68, 95 },
    { 0, 0 },
    { 0x89aaab6a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6baab6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706aab72, 98 },
    { 0x2daaab72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecaab76, 38 },
    { 0, 0 },
    { 0xaedaab78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29aab7b, 23 },
    { 0xd8faab7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x318aab7f, 93 },
    { 0x705aab80, 76 },
    { 0x649aab7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caab86, 78 },
    { 0xd8aab87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7aab8c, 55 },
    { 0xd33aab8d, 100 },
    { 0x401aab8e, 23 },
    { 0xf95aab8e, 90 },
    { 0x7deaab90, 90 },
    { 0x5baaab90, 93 },
    { 0xb4aaab8d, 61 },
    { 0xc6eaab93, 61 },
    { 0xe01aab92, 75 },
    { 0xcf1aab95, 23 },
    { 0x1d5aab95, 91 },
    { 0, 0 },
    { 0x43eaab98, 95 },
    { 0, 0 },
    { 0x1a7aab9a, 76 },
    { 0, 0 },
    { 0x66aab9c, 33 },
    { 0, 0 },
    { 0xb11aab9e, 61 },
    { 0x858aab9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232aaba5, 33 },
    { 0, 0 },
    { 0xacbaaba7, 55 },
    { 0x16aaaba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68aabad, 61 },
    { 0xb15aabad, 52 },
    { 0x4d9aabad, 61 },
    { 0xda7aabb0, 98 },
    { 0x476aabad, 52 },
    { 0x3b9aabb2, 36 },
    { 0x89baabb2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4aabb6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25aabbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81aaabc3, 96 },
    { 0, 0 },
    { 0xd08aabc5, 98 },
    { 0x1f3aabc5, 99 },
    { 0x26aabc7, 90 },
    { 0x3a4aabc8, 43 },
    { 0x679aabc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433aabcd, 100 },
    { 0, 0 },
    { 0x849aabcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49faabd2, 90 },
    { 0xe47aabd3, 34 },
    { 0xe9aabd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09aabd7, 95 },
    { 0x2a7aabd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276aabdd, 61 },
    { 0xe07aabde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3aabe1, 88 },
    { 0xf8aabe1, 33 },
    { 0xcc3aabe3, 34 },
    { 0x2c7aabe3, 61 },
    { 0x559aabe5, 98 },
    { 0x9f3aabe5, 55 },
    { 0, 0 },
    { 0xe3eaabe8, 61 },
    { 0xffdaabe9, 43 },
    { 0x6a2aabea, 98 },
    { 0x974aabeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aabf4, 95 },
    { 0xb08aabf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9aabf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5aabfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82aabff, 90 },
    { 0, 0 },
    { 0x2e2aac01, 90 },
    { 0, 0 },
    { 0x61aaac03, 85 },
    { 0x824aac04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415aac0b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x125aac0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2aac14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aaac17, 98 },
    { 0xea5aac18, 94 },
    { 0xec9aac19, 46 },
    { 0xcaaaac1a, 100 },
    { 0xf0faac1b, 90 },
    { 0x460aac18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeaac1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7baac25, 90 },
    { 0x55faac26, 90 },
    { 0xf57aac25, 94 },
    { 0x8f7aac28, 66 },
    { 0x588aac25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752aac2f, 33 },
    { 0x7bcaac30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aac33, 33 },
    { 0x5deaac34, 98 },
    { 0, 0 },
    { 0x8e3aac36, 95 },
    { 0x4daac37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4aac3a, 61 },
    { 0xae9aac3b, 23 },
    { 0x5a5aac3b, 99 },
    { 0xaf6aac3c, 90 },
    { 0x18aaac3e, 66 },
    { 0, 0 },
    { 0xa4eaac40, 55 },
    { 0x692aac40, 33 },
    { 0x66faac42, 95 },
    { 0x4eeaac43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aaac4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86baac4d, 61 },
    { 0x7bfaac4d, 95 },
    { 0x5f1aac4f, 98 },
    { 0x71faac4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184aac56, 95 },
    { 0x587aac57, 98 },
    { 0, 0 },
    { 0x5a7aac59, 90 },
    { 0xd5daac5a, 85 },
    { 0x195aac59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12aac5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaaac61, 98 },
    { 0xd6faac62, 77 },
    { 0x83eaac63, 40 },
    { 0xbd2aac63, 76 },
    { 0x9bfaac63, 95 },
    { 0xe5faac66, 90 },
    { 0x7f7aac67, 91 },
    { 0x700aac68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53aac6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528aac78, 89 },
    { 0xe50aac79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653aac7e, 33 },
    { 0x232aac7f, 76 },
    { 0, 0 },
    { 0x89baac81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163aac85, 33 },
    { 0x51baac86, 52 },
    { 0xd45aac87, 93 },
    { 0x59faac88, 40 },
    { 0xde1aac89, 43 },
    { 0x88daac86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56caac8d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855aac91, 23 },
    { 0, 0 },
    { 0x4fdaac93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6aac96, 90 },
    { 0x1b6aac96, 84 },
    { 0x261aac98, 98 },
    { 0x485aac99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aac9d, 95 },
    { 0x5baaac9e, 98 },
    { 0, 0 },
    { 0xdddaaca0, 81 },
    { 0, 0 },
    { 0x9ddaaca2, 33 },
    { 0xb72aaca3, 43 },
    { 0, 0 },
    { 0x55aaaca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73daaca8, 34 },
    { 0x568aaca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68aacaf, 93 },
    { 0xc7aaacb0, 61 },
    { 0x8a4aacb0, 33 },
    { 0x14caacaf, 55 },
    { 0, 0 },
    { 0x701aacb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81aacb7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961aacbc, 76 },
    { 0, 0 },
    { 0x393aacbe, 98 },
    { 0x72eaacbe, 98 },
    { 0xaedaacc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaacc4, 95 },
    { 0x987aacc5, 91 },
    { 0xfdeaacc6, 33 },
    { 0x966aacc6, 95 },
    { 0x55aaacc8, 40 },
    { 0xd44aacc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4caaccd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x711aacd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1aacd3, 90 },
    { 0, 0 },
    { 0x4ecaacd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1daacd8, 61 },
    { 0x58faacd9, 33 },
    { 0x5c2aacda, 90 },
    { 0x7b6aacdb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x906aacde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacaace4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbaacea, 34 },
    { 0x756aaceb, 17 },
    { 0, 0 },
    { 0x134aaced, 85 },
    { 0xf19aacee, 93 },
    { 0xa64aacee, 43 },
    { 0xa43aacef, 61 },
    { 0xb1baacef, 66 },
    { 0, 0 },
    { 0xaabaacf3, 95 },
    { 0x589aacf4, 66 },
    { 0x713aacf5, 55 },
    { 0xe1baacf3, 91 },
    { 0xf82aacf3, 33 },
    { 0x785aacf8, 56 },
    { 0x362aacf8, 90 },
    { 0xc2aaacf8, 90 },
    { 0xc0aaacfa, 84 },
    { 0x3b2aacf8, 55 },
    { 0x4d6aacfd, 91 },
    { 0, 0 },
    { 0xf96aacff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635aad05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868aad0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aaad0d, 90 },
    { 0x827aad0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993aad14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfaad18, 66 },
    { 0xea4aad18, 90 },
    { 0xa54aad1a, 76 },
    { 0x469aad1b, 33 },
    { 0x3f9aad1c, 93 },
    { 0xec1aad1c, 66 },
    { 0xee4aad18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aaad21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3aad24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aaad27, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81faad2b, 66 },
    { 0x5d0aad2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7aad31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aad36, 43 },
    { 0x504aad37, 23 },
    { 0xf8aad38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43faad3c, 95 },
    { 0xd50aad3d, 61 },
    { 0xab2aad3e, 33 },
    { 0xd15aad3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2aad43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9aad4a, 90 },
    { 0, 0 },
    { 0x7b1aad4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdaad51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9aad58, 67 },
    { 0x9a8aad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2aad5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754aad63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308aad6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7aad6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3aad74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aad79, 23 },
    { 0xa78aad79, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faad7e, 95 },
    { 0xaadaad7f, 90 },
    { 0x788aad80, 95 },
    { 0xae8aad7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8aad86, 33 },
    { 0x635aad87, 55 },
    { 0xaeeaad87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41daad8b, 33 },
    { 0, 0 },
    { 0x1aeaad8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83aad90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629aada9, 40 },
    { 0xd4aaadaa, 33 },
    { 0x1dcaadab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655aadb2, 96 },
    { 0x722aadb2, 76 },
    { 0x87faadb2, 95 },
    { 0x2bdaadb5, 93 },
    { 0, 0 },
    { 0x5c1aadb7, 98 },
    { 0x1d6aadb8, 55 },
    { 0, 0 },
    { 0xeceaadba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefaadbe, 33 },
    { 0, 0 },
    { 0x94faadc0, 98 },
    { 0xa78aadc1, 61 },
    { 0, 0 },
    { 0xb41aadc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59aadc6, 93 },
    { 0xcf7aadc6, 55 },
    { 0x45baadc8, 23 },
    { 0x5a9aadc7, 34 },
    { 0, 0 },
    { 0x5aeaadcb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edaadce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0aadd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaaadd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935aaddb, 55 },
    { 0xb5baaddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87aade2, 23 },
    { 0, 0 },
    { 0x1d5aade4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60aade8, 77 },
    { 0xecbaade8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10aadec, 23 },
    { 0x8a0aadec, 81 },
    { 0x24aaadec, 33 },
    { 0xe66aadef, 33 },
    { 0xfa5aadf0, 90 },
    { 0x352aadf1, 85 },
    { 0x97caadef, 95 },
    { 0xb62aadf1, 98 },
    { 0x863aadf0, 23 },
    { 0xc1daadf5, 33 },
    { 0x34daadf5, 98 },
    { 0x12eaadf7, 55 },
    { 0xc0aadee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4aadfc, 61 },
    { 0xbeaadfc, 36 },
    { 0x6d0aadfe, 46 },
    { 0x40aadff, 90 },
    { 0x6dbaae00, 90 },
    { 0x74aadff, 85 },
    { 0x804aae01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782aae06, 33 },
    { 0xbdeaae07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704aae0c, 100 },
    { 0, 0 },
    { 0x9f4aae0e, 33 },
    { 0x3afaae0f, 81 },
    { 0xca4aae0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x924aae13, 53 },
    { 0x351aae14, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787aae21, 90 },
    { 0xa5eaae22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39aae2c, 61 },
    { 0xc43aae2c, 61 },
    { 0x4a2aae2e, 33 },
    { 0x4d2aae2f, 34 },
    { 0xe86aae2e, 66 },
    { 0, 0 },
    { 0x4cfaae32, 100 },
    { 0x298aae33, 95 },
    { 0, 0 },
    { 0x9fdaae35, 91 },
    { 0x545aae36, 33 },
    { 0x1aeaae36, 84 },
    { 0, 0 },
    { 0x88aaae39, 90 },
    { 0x7eaae3a, 33 },
    { 0x512aae3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6aae3e, 87 },
    { 0x684aae3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6aae44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23faae47, 97 },
    { 0x376aae47, 90 },
    { 0x459aae47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927aae57, 100 },
    { 0x479aae58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x343aae5b, 61 },
    { 0, 0 },
    { 0x3b0aae5d, 55 },
    { 0x5cbaae5e, 23 },
    { 0, 0 },
    { 0x3f0aae60, 76 },
    { 0x37caae60, 98 },
    { 0x38caae60, 93 },
    { 0x3eaaae63, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcaaae69, 33 },
    { 0x9a3aae6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4aae6d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6aae71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4aae7c, 90 },
    { 0xb31aae7d, 95 },
    { 0x61faae7c, 66 },
    { 0xb80aae7f, 55 },
    { 0, 0 },
    { 0x3f5aae81, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8aae84, 95 },
    { 0xe10aae85, 33 },
    { 0xeeaae85, 85 },
    { 0xcbcaae87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2daae8b, 66 },
    { 0xcaeaae8b, 38 },
    { 0x339aae8c, 95 },
    { 0xfe8aae8e, 98 },
    { 0x3e6aae8f, 43 },
    { 0x5feaae8e, 95 },
    { 0, 0 },
    { 0xfe0aae92, 46 },
    { 0, 0 },
    { 0x9efaae94, 91 },
    { 0, 0 },
    { 0x70daae96, 98 },
    { 0xd08aae97, 52 },
    { 0x131aae98, 87 },
    { 0x5a3aae98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x321aae9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2aaea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefaaea5, 95 },
    { 0xf1faaea6, 46 },
    { 0x305aaea7, 46 },
    { 0x85daaea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9aaeaf, 42 },
    { 0xe46aaeaf, 78 },
    { 0, 0 },
    { 0x264aaeb2, 95 },
    { 0, 0 },
    { 0x51baaeb4, 33 },
    { 0x4e9aaeb5, 90 },
    { 0x3d7aaeb6, 52 },
    { 0x6f7aaeb6, 95 },
    { 0xe6aaaeb7, 33 },
    { 0x740aaeb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784aaec2, 98 },
    { 0, 0 },
    { 0xb2aaec4, 81 },
    { 0x599aaec4, 61 },
    { 0x92baaec4, 95 },
    { 0x533aaec7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63aaecf, 95 },
    { 0x7e4aaed0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93eaaedb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964aaee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x280aaeea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0aaeed, 76 },
    { 0x851aaeee, 99 },
    { 0x6d1aaeef, 33 },
    { 0xb2daaef0, 65 },
    { 0, 0 },
    { 0xacbaaef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16aaef5, 61 },
    { 0x3feaaef6, 77 },
    { 0xdeaaaef7, 66 },
    { 0xef8aaef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adaaefe, 61 },
    { 0, 0 },
    { 0x37caaf00, 98 },
    { 0x8e3aaf00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2aaf04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x376aaf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3aaf0a, 87 },
    { 0xc28aaf0b, 97 },
    { 0x41daaf0c, 76 },
    { 0xe9eaaf0c, 76 },
    { 0x192aaf0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1aaf11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x639aaf14, 85 },
    { 0x953aaf15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29baaf1b, 91 },
    { 0x49faaf1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x336aaf1f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x215aaf22, 23 },
    { 0x784aaf23, 98 },
    { 0, 0 },
    { 0x53eaaf25, 100 },
    { 0xdf0aaf25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72eaaf2b, 33 },
    { 0, 0 },
    { 0xd5eaaf2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aaf30, 98 },
    { 0, 0 },
    { 0x92daaf32, 66 },
    { 0x909aaf33, 95 },
    { 0, 0 },
    { 0x830aaf35, 95 },
    { 0x463aaf36, 81 },
    { 0x231aaf37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781aaf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceaaf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baaf4d, 40 },
    { 0xa96aaf4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afaaf52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13caaf55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caaaf58, 95 },
    { 0, 0 },
    { 0x52aaaf5a, 90 },
    { 0, 0 },
    { 0x749aaf5c, 91 },
    { 0x914aaf5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53aaf68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8aaf6c, 96 },
    { 0x741aaf6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8baaf72, 98 },
    { 0, 0 },
    { 0xcc8aaf74, 61 },
    { 0x440aaf74, 95 },
    { 0, 0 },
    { 0xaadaaf77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65aaf7a, 33 },
    { 0, 0 },
    { 0x399aaf7c, 95 },
    { 0xd8faaf7d, 61 },
    { 0xf72aaf7d, 66 },
    { 0x316aaf7d, 23 },
    { 0, 0 },
    { 0x9d4aaf81, 93 },
    { 0x411aaf82, 53 },
    { 0x9e2aaf83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437aaf89, 88 },
    { 0xa13aaf89, 93 },
    { 0x7a9aaf8b, 88 },
    { 0, 0 },
    { 0x43faaf8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaaf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbaaf97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x729aaf9a, 61 },
    { 0x84daaf9b, 40 },
    { 0, 0 },
    { 0x786aaf9d, 90 },
    { 0x91aaaf9e, 98 },
    { 0, 0 },
    { 0x624aafa0, 33 },
    { 0x5d3aafa1, 95 },
    { 0xdb7aafa2, 46 },
    { 0xf5baafa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbaafaa, 90 },
    { 0x49baafaa, 61 },
    { 0x630aafac, 61 },
    { 0x4b0aafac, 99 },
    { 0, 0 },
    { 0x3ddaafaf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628aafb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadaafbf, 23 },
    { 0x84eaafc0, 84 },
    { 0xfe7aafc1, 55 },
    { 0x33aafc2, 33 },
    { 0xabdaafc1, 84 },
    { 0xcffaafc4, 55 },
    { 0x62daafc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2aaafc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4aafd1, 61 },
    { 0x20faafd2, 85 },
    { 0x296aafd3, 90 },
    { 0xddaafd4, 61 },
    { 0xc9faafd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3aafdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x186aafe0, 61 },
    { 0x884aafe1, 95 },
    { 0x7ffaafe2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaaafe5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304aafe9, 33 },
    { 0x13faafea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61caafed, 95 },
    { 0x764aafee, 61 },
    { 0x2f5aafee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84aaff4, 55 },
    { 0x777aaff5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aab000, 87 },
    { 0x47cab001, 34 },
    { 0, 0 },
    { 0x14aab003, 78 },
    { 0, 0 },
    { 0x1f8ab005, 84 },
    { 0x1b8ab006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ab00a, 96 },
    { 0x99bab00b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ab00e, 33 },
    { 0x16aab00e, 98 },
    { 0xf57ab010, 77 },
    { 0xa6eab010, 55 },
    { 0xb7bab00e, 33 },
    { 0x292ab00e, 23 },
    { 0x51bab012, 23 },
    { 0x833ab015, 90 },
    { 0x937ab015, 78 },
    { 0xb48ab013, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89ab020, 61 },
    { 0xaedab021, 98 },
    { 0xcd5ab022, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eab029, 76 },
    { 0xe82ab02a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5ab02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ab032, 98 },
    { 0x485ab033, 95 },
    { 0x51fab033, 95 },
    { 0xd53ab033, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853ab03a, 76 },
    { 0, 0 },
    { 0x285ab03c, 90 },
    { 0x85bab03d, 76 },
    { 0x2f0ab03e, 27 },
    { 0x383ab03f, 65 },
    { 0x2ceab040, 98 },
    { 0xaefab040, 23 },
    { 0x3eab042, 88 },
    { 0x1eeab043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cab04a, 76 },
    { 0xb62ab04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4ab050, 33 },
    { 0xeddab051, 90 },
    { 0xb5cab052, 90 },
    { 0xbd0ab052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ab056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x516ab059, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ab05c, 93 },
    { 0, 0 },
    { 0xa6ab05e, 80 },
    { 0xdf7ab05e, 23 },
    { 0x8e0ab060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x189ab063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5ab066, 98 },
    { 0x481ab067, 61 },
    { 0xad9ab067, 98 },
    { 0x3a7ab068, 67 },
    { 0x21ab06a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdab06e, 95 },
    { 0, 0 },
    { 0x206ab070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54ab078, 17 },
    { 0xfe9ab079, 95 },
    { 0xa86ab07a, 33 },
    { 0, 0 },
    { 0xe06ab07c, 95 },
    { 0x52fab07d, 85 },
    { 0xd23ab07d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932ab085, 93 },
    { 0x2bcab086, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24ab08a, 46 },
    { 0, 0 },
    { 0x5fcab08c, 90 },
    { 0, 0 },
    { 0x3fdab08e, 76 },
    { 0, 0 },
    { 0xf7ab090, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aab097, 90 },
    { 0x2e6ab098, 90 },
    { 0xa84ab099, 96 },
    { 0xc54ab098, 33 },
    { 0xd15ab099, 95 },
    { 0xb7aab097, 90 },
    { 0xe7aab09d, 43 },
    { 0, 0 },
    { 0xc8bab09f, 68 },
    { 0x4c3ab0a0, 98 },
    { 0x3daab0a0, 98 },
    { 0, 0 },
    { 0xe6bab0a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ab0a6, 46 },
    { 0x578ab0a7, 33 },
    { 0x97fab0a6, 55 },
    { 0x7c0ab0a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410ab0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126ab0bc, 34 },
    { 0x943ab0bd, 78 },
    { 0, 0 },
    { 0xec5ab0bf, 55 },
    { 0xe7eab0c0, 90 },
    { 0x862ab0c1, 95 },
    { 0x842ab0c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32ab0c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab0d4, 46 },
    { 0xc56ab0d5, 76 },
    { 0, 0 },
    { 0x69eab0d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x917ab0da, 95 },
    { 0x20aab0db, 98 },
    { 0, 0 },
    { 0xb7ab0dd, 98 },
    { 0x4e1ab0de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdab0e2, 33 },
    { 0, 0 },
    { 0xe01ab0e4, 33 },
    { 0xe11ab0e4, 95 },
    { 0, 0 },
    { 0x9f1ab0e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6ab0ea, 33 },
    { 0x2edab0eb, 36 },
    { 0xd22ab0ec, 61 },
    { 0xf59ab0ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cab0f0, 80 },
    { 0xf03ab0f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243ab0f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eab0fe, 98 },
    { 0, 0 },
    { 0xe9eab100, 95 },
    { 0, 0 },
    { 0xc06ab102, 55 },
    { 0, 0 },
    { 0x6eab104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bab108, 61 },
    { 0, 0 },
    { 0xc74ab10a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29ab10d, 90 },
    { 0, 0 },
    { 0xc85ab10f, 22 },
    { 0, 0 },
    { 0x5c4ab111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8ab118, 99 },
    { 0x81fab119, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650ab11e, 76 },
    { 0, 0 },
    { 0xebcab120, 61 },
    { 0x36dab121, 61 },
    { 0x698ab121, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122ab128, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ab12b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65ab12e, 61 },
    { 0x8fcab12f, 33 },
    { 0x55aab130, 3 },
    { 0x60eab131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccab134, 91 },
    { 0, 0 },
    { 0xeeaab136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8ab139, 99 },
    { 0xa69ab13a, 23 },
    { 0x7e3ab13b, 90 },
    { 0, 0 },
    { 0xedcab13d, 91 },
    { 0xf4ab13d, 77 },
    { 0xa57ab13d, 55 },
    { 0xbffab13e, 55 },
    { 0xff5ab141, 78 },
    { 0xd34ab13f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dab147, 87 },
    { 0x176ab147, 33 },
    { 0x91eab149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab14f, 61 },
    { 0xa66ab14f, 93 },
    { 0x636ab14f, 55 },
    { 0x96cab152, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ab157, 88 },
    { 0x892ab157, 78 },
    { 0x134ab159, 61 },
    { 0xeb5ab157, 100 },
    { 0x3c6ab157, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bab15e, 33 },
    { 0x9e4ab15e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927ab16a, 98 },
    { 0, 0 },
    { 0x18dab16c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bab173, 23 },
    { 0xeccab174, 61 },
    { 0, 0 },
    { 0x79fab176, 23 },
    { 0, 0 },
    { 0xbfdab178, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbab17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ab17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ab188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bab190, 66 },
    { 0x51ab191, 23 },
    { 0xb53ab190, 61 },
    { 0x2c5ab193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fab19e, 55 },
    { 0xdfcab19e, 98 },
    { 0, 0 },
    { 0xa19ab1a1, 23 },
    { 0x8aab1a2, 90 },
    { 0xa1eab1a2, 76 },
    { 0xdf0ab1a3, 55 },
    { 0x876ab1a1, 95 },
    { 0x4d8ab1a4, 55 },
    { 0xf43ab1a1, 83 },
    { 0x9b1ab1a8, 68 },
    { 0, 0 },
    { 0xc3cab1aa, 34 },
    { 0x7edab1ab, 34 },
    { 0x9bcab1ab, 55 },
    { 0xb2bab1ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379ab1b3, 55 },
    { 0x953ab1b3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ab1ba, 61 },
    { 0, 0 },
    { 0x19dab1bc, 43 },
    { 0xcd0ab1bc, 17 },
    { 0xbd6ab1be, 88 },
    { 0, 0 },
    { 0xf03ab1c0, 52 },
    { 0x6cfab1c0, 76 },
    { 0, 0 },
    { 0xbd6ab1c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9ab1ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecab1d3, 23 },
    { 0, 0 },
    { 0xa87ab1d5, 98 },
    { 0, 0 },
    { 0xb7fab1d7, 61 },
    { 0xeeab1d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5ab1db, 76 },
    { 0x340ab1db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecab1e0, 33 },
    { 0xdccab1e0, 40 },
    { 0x26bab1e2, 55 },
    { 0x558ab1e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dab1e6, 76 },
    { 0x879ab1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2ab1ec, 34 },
    { 0x6c7ab1ed, 95 },
    { 0xe36ab1ed, 95 },
    { 0x688ab1ec, 95 },
    { 0x6a7ab1f0, 76 },
    { 0x1b9ab1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79ab1f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fab1f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab1fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x287ab1ff, 66 },
    { 0, 0 },
    { 0xe6eab201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1ab204, 55 },
    { 0x778ab205, 61 },
    { 0xb5fab205, 90 },
    { 0x2e3ab207, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ab20a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecab211, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eab215, 95 },
    { 0xbe2ab215, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ab21d, 66 },
    { 0x20aab21d, 85 },
    { 0xc8bab21f, 66 },
    { 0x4aeab220, 84 },
    { 0x781ab220, 87 },
    { 0x96cab222, 34 },
    { 0x8f9ab222, 76 },
    { 0, 0 },
    { 0x5e2ab225, 67 },
    { 0x603ab226, 75 },
    { 0x676ab227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdab22a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ab22f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25ab233, 23 },
    { 0x7daab234, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcab23a, 33 },
    { 0x618ab23b, 23 },
    { 0, 0 },
    { 0xb32ab23d, 100 },
    { 0x79ab23e, 33 },
    { 0x2e4ab23d, 90 },
    { 0x3aab240, 91 },
    { 0xe17ab23d, 98 },
    { 0x70fab23d, 90 },
    { 0, 0 },
    { 0x42bab244, 61 },
    { 0, 0 },
    { 0x8c6ab246, 61 },
    { 0, 0 },
    { 0xdaaab248, 77 },
    { 0x63aab249, 98 },
    { 0x81bab24a, 90 },
    { 0x870ab24b, 33 },
    { 0x14eab24c, 33 },
    { 0x5c2ab24c, 90 },
    { 0xd28ab24e, 45 },
    { 0x3f8ab24c, 55 },
    { 0x948ab249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329ab258, 90 },
    { 0, 0 },
    { 0x8e8ab25a, 52 },
    { 0, 0 },
    { 0x757ab25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bab262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ab266, 90 },
    { 0x3d5ab267, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdab26a, 98 },
    { 0xe69ab26b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab26e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ab272, 26 },
    { 0xd90ab273, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0ab27a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3ab27e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ab281, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848ab285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ab289, 46 },
    { 0xc0fab28a, 23 },
    { 0x714ab28b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ab28f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab294, 90 },
    { 0, 0 },
    { 0xf28ab296, 33 },
    { 0x93ab297, 43 },
    { 0x764ab296, 95 },
    { 0xde4ab299, 61 },
    { 0, 0 },
    { 0xa02ab29b, 23 },
    { 0xdb9ab29c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ab2a0, 95 },
    { 0xdc7ab2a1, 90 },
    { 0x377ab2a2, 84 },
    { 0xa71ab2a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77ab2a6, 23 },
    { 0xca6ab2a6, 61 },
    { 0x581ab2a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ab2ab, 98 },
    { 0xe23ab2ac, 61 },
    { 0x70dab2ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eab2b5, 55 },
    { 0xe61ab2b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6ab2b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaab2be, 23 },
    { 0xb07ab2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42ab2c3, 34 },
    { 0xddab2c4, 66 },
    { 0, 0 },
    { 0xea6ab2c6, 90 },
    { 0x842ab2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eab2ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ab2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fab2d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2ab2d6, 88 },
    { 0xad6ab2d7, 65 },
    { 0xaa2ab2d6, 98 },
    { 0xd4eab2d6, 43 },
    { 0x11dab2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x531ab2dd, 100 },
    { 0, 0 },
    { 0x7cbab2df, 90 },
    { 0xc9fab2e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dab2e3, 100 },
    { 0x2dbab2e4, 90 },
    { 0x35bab2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbab2e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477ab2f1, 76 },
    { 0x842ab2f2, 90 },
    { 0x73dab2f3, 90 },
    { 0x728ab2f4, 67 },
    { 0xc59ab2f4, 23 },
    { 0x4bcab2f6, 87 },
    { 0, 0 },
    { 0x959ab2f8, 33 },
    { 0xcf5ab2f9, 68 },
    { 0, 0 },
    { 0xadbab2fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeab2fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0ab303, 95 },
    { 0x2d4ab304, 87 },
    { 0, 0 },
    { 0xa59ab306, 87 },
    { 0xf81ab307, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x838ab30a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934ab312, 55 },
    { 0xc6ab313, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaab317, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ab31a, 98 },
    { 0, 0 },
    { 0xd14ab31c, 95 },
    { 0x29dab31c, 34 },
    { 0x43dab31e, 61 },
    { 0, 0 },
    { 0xa95ab320, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cab324, 61 },
    { 0, 0 },
    { 0x273ab326, 75 },
    { 0x342ab327, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07ab32a, 33 },
    { 0xc86ab32b, 93 },
    { 0xff7ab32c, 23 },
    { 0xfab32d, 98 },
    { 0, 0 },
    { 0xdd2ab32f, 33 },
    { 0x6dbab330, 100 },
    { 0, 0 },
    { 0xf0dab332, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7ab335, 61 },
    { 0xb7cab335, 99 },
    { 0x618ab337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefab33c, 61 },
    { 0xf13ab33d, 90 },
    { 0xe39ab33e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ab341, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ab348, 61 },
    { 0, 0 },
    { 0xe4fab34a, 23 },
    { 0xdbbab34b, 68 },
    { 0xd64ab34c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ab34f, 95 },
    { 0xa60ab350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bab354, 95 },
    { 0, 0 },
    { 0xd56ab356, 61 },
    { 0, 0 },
    { 0x27bab358, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921ab35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deab360, 23 },
    { 0x4fab360, 33 },
    { 0x1c7ab360, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x519ab365, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687ab36b, 90 },
    { 0xf00ab36b, 33 },
    { 0, 0 },
    { 0xa78ab36e, 90 },
    { 0x50bab36e, 76 },
    { 0, 0 },
    { 0x992ab371, 34 },
    { 0x912ab372, 33 },
    { 0xa58ab373, 43 },
    { 0xf8dab373, 23 },
    { 0xc9dab371, 97 },
    { 0x40eab376, 66 },
    { 0, 0 },
    { 0x6afab378, 23 },
    { 0xeab379, 33 },
    { 0, 0 },
    { 0x8e2ab37b, 23 },
    { 0xafdab37c, 61 },
    { 0, 0 },
    { 0x5c5ab37e, 61 },
    { 0, 0 },
    { 0xa77ab380, 61 },
    { 0x8e7ab381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2ab389, 90 },
    { 0, 0 },
    { 0x70dab38b, 55 },
    { 0, 0 },
    { 0x49bab38d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cab391, 76 },
    { 0x3ecab391, 61 },
    { 0xaa7ab391, 76 },
    { 0xbab394, 88 },
    { 0x759ab395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bab39c, 91 },
    { 0x1afab39d, 55 },
    { 0x811ab39e, 46 },
    { 0x71aab39f, 100 },
    { 0, 0 },
    { 0xce9ab3a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bab3a4, 77 },
    { 0, 0 },
    { 0x69fab3a6, 98 },
    { 0x73aab3a7, 90 },
    { 0, 0 },
    { 0x6f2ab3a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9ab3b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9ab3b5, 98 },
    { 0, 0 },
    { 0x707ab3b7, 23 },
    { 0x4c0ab3b8, 92 },
    { 0, 0 },
    { 0x88cab3ba, 23 },
    { 0x7aab3bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ab3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccab3c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaab3c7, 77 },
    { 0xd02ab3c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422ab3ce, 52 },
    { 0x71bab3cf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edab3da, 77 },
    { 0x559ab3db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dab3e4, 93 },
    { 0, 0 },
    { 0xeefab3e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cab3e9, 36 },
    { 0x636ab3e9, 24 },
    { 0x545ab3ea, 55 },
    { 0, 0 },
    { 0xceab3ed, 55 },
    { 0, 0 },
    { 0xbb5ab3ef, 98 },
    { 0x663ab3f0, 46 },
    { 0x813ab3f0, 61 },
    { 0, 0 },
    { 0xe0ab3f3, 61 },
    { 0x94fab3f4, 55 },
    { 0x441ab3f5, 33 },
    { 0xa64ab3f6, 65 },
    { 0xd4eab3f5, 61 },
    { 0x27aab3f8, 23 },
    { 0xac9ab3f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822ab400, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faab404, 90 },
    { 0xb7aab405, 33 },
    { 0x4c3ab404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bab409, 23 },
    { 0xaf7ab40a, 33 },
    { 0x25bab40b, 46 },
    { 0x307ab40c, 98 },
    { 0, 0 },
    { 0x68cab40e, 65 },
    { 0xdbfab40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9ab412, 90 },
    { 0xbbcab413, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47ab417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x667ab41a, 61 },
    { 0, 0 },
    { 0x88dab41c, 83 },
    { 0x826ab41d, 46 },
    { 0xc01ab41d, 98 },
    { 0, 0 },
    { 0x351ab420, 61 },
    { 0xc71ab421, 61 },
    { 0x29dab422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fab425, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdab429, 98 },
    { 0, 0 },
    { 0x5c8ab42b, 99 },
    { 0, 0 },
    { 0x539ab42d, 90 },
    { 0x37bab42e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1ab437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ab43a, 33 },
    { 0, 0 },
    { 0x1fbab43c, 66 },
    { 0xe49ab43c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ab443, 33 },
    { 0, 0 },
    { 0x649ab445, 95 },
    { 0xc8fab446, 33 },
    { 0xee1ab447, 90 },
    { 0x673ab448, 95 },
    { 0x6dab449, 52 },
    { 0x86aab44a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdab451, 23 },
    { 0x900ab452, 33 },
    { 0x361ab451, 100 },
    { 0, 0 },
    { 0xf32ab455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ab458, 61 },
    { 0, 0 },
    { 0xf0fab45a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ab45d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdab463, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ab470, 61 },
    { 0, 0 },
    { 0x172ab472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dab476, 98 },
    { 0x598ab476, 90 },
    { 0x801ab477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63ab47d, 61 },
    { 0x250ab47e, 100 },
    { 0xe5bab47f, 96 },
    { 0xc66ab480, 81 },
    { 0xc5ab481, 34 },
    { 0xf39ab481, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2ab488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ab48d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfab494, 85 },
    { 0xd27ab495, 95 },
    { 0xd3fab496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aab499, 88 },
    { 0xd79ab49a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x755ab49d, 34 },
    { 0xeeab49e, 23 },
    { 0xdb1ab49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aab4a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ab4a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ab4b0, 95 },
    { 0x57dab4b1, 98 },
    { 0, 0 },
    { 0x236ab4b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ab4b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aab4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771ab4c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ab4c8, 61 },
    { 0xeaaab4c9, 90 },
    { 0xc20ab4c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ab4cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ab4d4, 61 },
    { 0x6f7ab4d5, 76 },
    { 0x286ab4d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759ab4da, 61 },
    { 0x231ab4da, 61 },
    { 0xa0ab4db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5ab4e0, 61 },
    { 0x4e6ab4e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ab4e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ab4ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7ab4f0, 23 },
    { 0x610ab4f0, 88 },
    { 0x6e1ab4f2, 23 },
    { 0x7ecab4f3, 33 },
    { 0xdd9ab4f4, 53 },
    { 0x87eab4f4, 95 },
    { 0, 0 },
    { 0x98fab4f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eab4fa, 76 },
    { 0, 0 },
    { 0x2efab4fc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ab4ff, 98 },
    { 0xe21ab4ff, 46 },
    { 0x17fab501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ab504, 23 },
    { 0xc68ab505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4ab50c, 23 },
    { 0x576ab50d, 23 },
    { 0x3beab50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0ab511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bab515, 33 },
    { 0xbdab516, 23 },
    { 0, 0 },
    { 0xa39ab518, 85 },
    { 0, 0 },
    { 0x57ab51a, 33 },
    { 0, 0 },
    { 0x175ab51c, 33 },
    { 0x8adab51c, 33 },
    { 0x80bab51e, 17 },
    { 0x9e1ab51f, 61 },
    { 0, 0 },
    { 0x9d1ab521, 77 },
    { 0x222ab522, 33 },
    { 0xe24ab523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42ab526, 90 },
    { 0x5f7ab527, 33 },
    { 0x2cab526, 33 },
    { 0, 0 },
    { 0x825ab52a, 100 },
    { 0, 0 },
    { 0x67bab52c, 90 },
    { 0xa10ab52d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3ab530, 46 },
    { 0x8fbab530, 34 },
    { 0x12fab530, 23 },
    { 0x2f5ab533, 61 },
    { 0xca3ab531, 61 },
    { 0x555ab533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9ab53a, 67 },
    { 0, 0 },
    { 0xa75ab53c, 95 },
    { 0xfcaab53c, 61 },
    { 0xe9bab53e, 76 },
    { 0xbb8ab53f, 90 },
    { 0xd4dab53f, 34 },
    { 0x6d7ab53c, 23 },
    { 0x844ab53d, 91 },
    { 0xd9eab543, 90 },
    { 0xd37ab544, 34 },
    { 0xe43ab53d, 83 },
    { 0x37bab546, 90 },
    { 0, 0 },
    { 0x20dab548, 33 },
    { 0x2e5ab549, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36ab54c, 55 },
    { 0xd4dab54c, 75 },
    { 0x612ab54e, 98 },
    { 0xfceab54f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aab553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab558, 55 },
    { 0x88aab559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dab565, 80 },
    { 0, 0 },
    { 0xf26ab567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858ab56e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fab572, 33 },
    { 0, 0 },
    { 0x901ab574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bab579, 93 },
    { 0x734ab579, 23 },
    { 0, 0 },
    { 0xbcab57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fab580, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ab583, 46 },
    { 0x567ab584, 34 },
    { 0x2b3ab584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cab588, 90 },
    { 0x4b6ab589, 98 },
    { 0, 0 },
    { 0x6f2ab58b, 43 },
    { 0x10bab58b, 70 },
    { 0x8b2ab58d, 33 },
    { 0x190ab58e, 33 },
    { 0x27dab58f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cab597, 34 },
    { 0, 0 },
    { 0x33bab599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab5a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5ab5a5, 95 },
    { 0x10aab5a6, 90 },
    { 0, 0 },
    { 0xb91ab5a8, 76 },
    { 0, 0 },
    { 0x2cbab5aa, 61 },
    { 0xbb9ab5aa, 33 },
    { 0x592ab5aa, 90 },
    { 0xa68ab5aa, 51 },
    { 0x6c5ab5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ab5b7, 91 },
    { 0xde7ab5b7, 78 },
    { 0, 0 },
    { 0x9b2ab5ba, 88 },
    { 0xa4eab5ba, 90 },
    { 0xf9fab5bc, 76 },
    { 0, 0 },
    { 0xb8dab5be, 90 },
    { 0xd60ab5be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bab5c2, 90 },
    { 0x61cab5c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dab5c6, 90 },
    { 0x540ab5c6, 90 },
    { 0x857ab5c8, 90 },
    { 0xd4fab5c8, 34 },
    { 0x1eab5ca, 52 },
    { 0x314ab5ca, 66 },
    { 0x891ab5ca, 95 },
    { 0xd8cab5cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ab5d2, 33 },
    { 0, 0 },
    { 0xdf1ab5d4, 100 },
    { 0xdedab5d5, 23 },
    { 0, 0 },
    { 0x8aaab5d7, 33 },
    { 0, 0 },
    { 0xe63ab5d9, 66 },
    { 0, 0 },
    { 0x4fdab5db, 84 },
    { 0x690ab5dc, 33 },
    { 0x4b3ab5dd, 76 },
    { 0, 0 },
    { 0xa69ab5df, 33 },
    { 0, 0 },
    { 0x356ab5e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ab5e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faab5eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89ab5ef, 95 },
    { 0xbcaab5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daab5f5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x776ab5f8, 78 },
    { 0xa22ab5f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4ab5fc, 84 },
    { 0, 0 },
    { 0x88bab5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69ab601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ab607, 98 },
    { 0x4afab607, 76 },
    { 0xfc9ab608, 55 },
    { 0x65cab60a, 33 },
    { 0, 0 },
    { 0x542ab60c, 34 },
    { 0, 0 },
    { 0xf52ab60e, 87 },
    { 0x23dab60f, 62 },
    { 0x6e0ab60f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3ab613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acab616, 55 },
    { 0, 0 },
    { 0x6f9ab618, 98 },
    { 0xdf3ab618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ab61e, 36 },
    { 0xda2ab61f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ab622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dab629, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17ab62f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ab637, 98 },
    { 0xc4dab638, 61 },
    { 0x452ab639, 61 },
    { 0, 0 },
    { 0xcfdab63b, 33 },
    { 0x95aab63b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ab63f, 90 },
    { 0x78aab640, 98 },
    { 0x734ab640, 17 },
    { 0x6e1ab641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x400ab645, 17 },
    { 0x472ab645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633ab64a, 23 },
    { 0, 0 },
    { 0xdfdab64c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dab650, 98 },
    { 0, 0 },
    { 0xcab652, 43 },
    { 0x6e3ab653, 100 },
    { 0x9a1ab654, 33 },
    { 0x5f2ab652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250ab65a, 90 },
    { 0, 0 },
    { 0x3eeab65c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1ab65f, 95 },
    { 0xfffab660, 98 },
    { 0, 0 },
    { 0x49aab662, 46 },
    { 0xd40ab663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eab667, 33 },
    { 0x6f1ab668, 61 },
    { 0xa77ab668, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ab66e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71ab672, 90 },
    { 0x532ab672, 90 },
    { 0xb03ab674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dab677, 98 },
    { 0x2ccab678, 55 },
    { 0x8a5ab678, 34 },
    { 0x9edab679, 95 },
    { 0xdd0ab679, 17 },
    { 0xfaab67a, 90 },
    { 0x580ab678, 76 },
    { 0xa35ab67e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412ab684, 76 },
    { 0, 0 },
    { 0x9cfab686, 23 },
    { 0xf41ab686, 55 },
    { 0x1c6ab688, 98 },
    { 0, 0 },
    { 0x22dab68a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaab68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5ab695, 93 },
    { 0xe41ab696, 33 },
    { 0x94cab697, 95 },
    { 0x21fab696, 23 },
    { 0x824ab699, 23 },
    { 0x4d1ab699, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaab69e, 98 },
    { 0x26aab69f, 76 },
    { 0x5aab69e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03ab6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ab6ad, 61 },
    { 0xae1ab6ad, 23 },
    { 0xeceab6af, 52 },
    { 0x47bab6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ab6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfab6c5, 33 },
    { 0, 0 },
    { 0xebbab6c7, 95 },
    { 0x44cab6c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fab6cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ab6d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fab6d5, 23 },
    { 0, 0 },
    { 0x1c6ab6d7, 33 },
    { 0xd17ab6d8, 23 },
    { 0, 0 },
    { 0xcc6ab6da, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdab6e3, 46 },
    { 0, 0 },
    { 0x913ab6e5, 76 },
    { 0xd36ab6e6, 55 },
    { 0x3d6ab6e6, 76 },
    { 0, 0 },
    { 0x12cab6e9, 34 },
    { 0x510ab6e9, 98 },
    { 0x318ab6eb, 61 },
    { 0, 0 },
    { 0x823ab6ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x396ab6f0, 33 },
    { 0x71eab6f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4ab6f4, 67 },
    { 0x10eab6f4, 61 },
    { 0x497ab6f6, 77 },
    { 0x682ab6f7, 61 },
    { 0xaa2ab6f6, 90 },
    { 0x56bab6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4ab6fc, 46 },
    { 0xcedab6fc, 76 },
    { 0, 0 },
    { 0xcfab6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ab704, 87 },
    { 0x9e2ab704, 61 },
    { 0x5b0ab704, 65 },
    { 0x6d3ab707, 90 },
    { 0, 0 },
    { 0x510ab709, 23 },
    { 0x32aab70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ab70e, 69 },
    { 0x15eab70f, 90 },
    { 0x61cab710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56ab71b, 17 },
    { 0x5b9ab71b, 93 },
    { 0x236ab71d, 33 },
    { 0, 0 },
    { 0x76fab71f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdab723, 23 },
    { 0xd78ab723, 52 },
    { 0x158ab725, 17 },
    { 0xa01ab725, 33 },
    { 0xb6eab723, 95 },
    { 0xf7dab728, 43 },
    { 0, 0 },
    { 0xe8aab72a, 95 },
    { 0, 0 },
    { 0xdddab72c, 98 },
    { 0xe75ab72d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aab731, 91 },
    { 0xc3eab732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ab735, 55 },
    { 0, 0 },
    { 0x350ab737, 33 },
    { 0x636ab738, 91 },
    { 0, 0 },
    { 0xd66ab73a, 33 },
    { 0xe62ab73b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3ab73e, 90 },
    { 0x68dab73f, 33 },
    { 0xbe5ab740, 33 },
    { 0, 0 },
    { 0xea1ab742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x802ab745, 98 },
    { 0x528ab746, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fab74a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feab750, 8 },
    { 0x701ab751, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ab754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadab757, 84 },
    { 0, 0 },
    { 0xbfab759, 46 },
    { 0, 0 },
    { 0xcf6ab75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546ab770, 34 },
    { 0xd3fab770, 81 },
    { 0xeb1ab772, 78 },
    { 0, 0 },
    { 0xb2eab774, 67 },
    { 0, 0 },
    { 0xdd6ab776, 93 },
    { 0, 0 },
    { 0xd11ab778, 90 },
    { 0x7e0ab778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676ab77d, 33 },
    { 0x28bab77e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cab786, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ab789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42ab78d, 85 },
    { 0, 0 },
    { 0x50cab78f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716ab792, 91 },
    { 0xd3ab793, 98 },
    { 0, 0 },
    { 0xb5bab795, 33 },
    { 0x669ab796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6ab79a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab79d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cab7a1, 33 },
    { 0, 0 },
    { 0xa5cab7a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ab7a8, 98 },
    { 0x412ab7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x326ab7ac, 85 },
    { 0xbc3ab7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74ab7b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ab7b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962ab7b9, 78 },
    { 0, 0 },
    { 0x900ab7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dab7be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ab7c4, 43 },
    { 0x810ab7c5, 75 },
    { 0x73eab7c6, 23 },
    { 0, 0 },
    { 0x83bab7c8, 33 },
    { 0x249ab7c8, 98 },
    { 0x786ab7ca, 55 },
    { 0xc93ab7cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bab7d0, 95 },
    { 0xa01ab7d0, 90 },
    { 0, 0 },
    { 0x80cab7d3, 84 },
    { 0x234ab7d4, 17 },
    { 0xb72ab7d5, 33 },
    { 0x9e6ab7d3, 23 },
    { 0x862ab7d7, 36 },
    { 0xb8ab7d7, 90 },
    { 0xa4aab7d9, 95 },
    { 0, 0 },
    { 0x225ab7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fab7df, 61 },
    { 0xbc7ab7e0, 91 },
    { 0xfd7ab7e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab7e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abab7ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ab7f3, 43 },
    { 0x8f8ab7f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbab7f8, 23 },
    { 0x1d8ab7f9, 34 },
    { 0, 0 },
    { 0xe44ab7fb, 61 },
    { 0, 0 },
    { 0xa55ab7fd, 61 },
    { 0xe26ab7fe, 61 },
    { 0xf5fab7fe, 61 },
    { 0x526ab800, 90 },
    { 0x7b8ab801, 36 },
    { 0x880ab800, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ab805, 33 },
    { 0, 0 },
    { 0x8bfab807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64ab80c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aab80f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ab816, 98 },
    { 0xf05ab817, 61 },
    { 0x3a1ab818, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dab81f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ab825, 49 },
    { 0x360ab825, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x988ab829, 34 },
    { 0, 0 },
    { 0x81cab82b, 33 },
    { 0, 0 },
    { 0x2e2ab82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bab832, 65 },
    { 0, 0 },
    { 0x7eaab834, 87 },
    { 0x2c2ab835, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fab83a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ab83e, 78 },
    { 0, 0 },
    { 0x565ab840, 61 },
    { 0, 0 },
    { 0xe21ab842, 34 },
    { 0xa16ab843, 55 },
    { 0x3bdab844, 33 },
    { 0x699ab845, 33 },
    { 0x746ab846, 95 },
    { 0x95dab843, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdab84a, 90 },
    { 0x5bbab84b, 76 },
    { 0xa64ab84c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33ab84f, 91 },
    { 0x531ab850, 23 },
    { 0x60dab851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfab854, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x484ab857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2ab85c, 66 },
    { 0xa4eab85d, 52 },
    { 0xe3cab85e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ab861, 43 },
    { 0x9c1ab862, 33 },
    { 0, 0 },
    { 0xb62ab864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adab86c, 33 },
    { 0xd8fab86d, 93 },
    { 0x423ab86d, 67 },
    { 0x8c3ab86d, 23 },
    { 0xa66ab86e, 91 },
    { 0x2ceab870, 23 },
    { 0xa61ab871, 61 },
    { 0, 0 },
    { 0x7bab874, 46 },
    { 0x282ab875, 33 },
    { 0xc9bab875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01ab87d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fab882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276ab889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaab88e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x290ab891, 76 },
    { 0, 0 },
    { 0xe40ab893, 65 },
    { 0x609ab894, 98 },
    { 0x3c6ab893, 90 },
    { 0x1bdab895, 90 },
    { 0xa24ab893, 33 },
    { 0x2c8ab895, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42ab89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ab89e, 33 },
    { 0xe36ab89e, 46 },
    { 0xd50ab8a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fab8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5aab8a9, 43 },
    { 0x7c0ab8aa, 34 },
    { 0x4ceab8ab, 90 },
    { 0, 0 },
    { 0x8d7ab8ad, 61 },
    { 0xdfeab8ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beab8b4, 23 },
    { 0xb1dab8b5, 66 },
    { 0, 0 },
    { 0xd3aab8b7, 66 },
    { 0xb2eab8b8, 65 },
    { 0x125ab8b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bab8bf, 95 },
    { 0xaaaab8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ab8c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698ab8c8, 23 },
    { 0xe85ab8c9, 90 },
    { 0x747ab8ca, 61 },
    { 0x7bdab8cb, 46 },
    { 0x487ab8ca, 90 },
    { 0x7bfab8cd, 61 },
    { 0, 0 },
    { 0x279ab8cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ab8d3, 76 },
    { 0xe78ab8d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986ab8d9, 92 },
    { 0x20bab8da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454ab8df, 23 },
    { 0x2a7ab8e0, 76 },
    { 0, 0 },
    { 0xb8cab8e2, 90 },
    { 0, 0 },
    { 0x566ab8e4, 98 },
    { 0x79aab8e5, 52 },
    { 0xcb3ab8e5, 55 },
    { 0x953ab8e5, 46 },
    { 0, 0 },
    { 0x5c0ab8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ab8ee, 33 },
    { 0x104ab8ee, 23 },
    { 0xab8ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1ab8f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfab8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2ab8fd, 76 },
    { 0xef4ab8fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4ab903, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cab907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cab90b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ab90f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837ab918, 23 },
    { 0, 0 },
    { 0xd1ab91a, 33 },
    { 0x593ab91b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aab91e, 90 },
    { 0x17dab91f, 66 },
    { 0x322ab920, 76 },
    { 0, 0 },
    { 0xdb1ab922, 61 },
    { 0, 0 },
    { 0x3b8ab924, 95 },
    { 0xa0dab924, 91 },
    { 0x862ab926, 85 },
    { 0xda3ab926, 55 },
    { 0xca4ab928, 99 },
    { 0xc77ab929, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ab930, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ab933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeeab938, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bab940, 91 },
    { 0, 0 },
    { 0x6ab942, 61 },
    { 0, 0 },
    { 0x480ab944, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1ab94a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cab94f, 90 },
    { 0xf47ab950, 95 },
    { 0, 0 },
    { 0x43bab952, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686ab957, 90 },
    { 0x9bdab957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cab95b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeab963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ab966, 93 },
    { 0xe62ab967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777ab96e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adab971, 90 },
    { 0x1aaab971, 90 },
    { 0x8f4ab971, 17 },
    { 0xf73ab974, 93 },
    { 0x82fab973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaab97a, 76 },
    { 0, 0 },
    { 0x950ab97c, 61 },
    { 0xfb5ab97d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43ab982, 90 },
    { 0, 0 },
    { 0x4c3ab984, 76 },
    { 0xafab985, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ab98b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fab98e, 61 },
    { 0, 0 },
    { 0xe5cab990, 33 },
    { 0xaeaab990, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ab997, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ab99c, 23 },
    { 0, 0 },
    { 0x4b7ab99e, 100 },
    { 0x795ab99f, 90 },
    { 0x190ab9a0, 99 },
    { 0, 0 },
    { 0x715ab9a2, 84 },
    { 0x4faab9a3, 88 },
    { 0, 0 },
    { 0x86bab9a5, 3 },
    { 0, 0 },
    { 0x4e3ab9a7, 97 },
    { 0, 0 },
    { 0xc6aab9a9, 88 },
    { 0xdcdab9a9, 69 },
    { 0xb47ab9ab, 33 },
    { 0xb84ab9ac, 85 },
    { 0x6fab9ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4ab9b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afab9bd, 98 },
    { 0xb1eab9be, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x249ab9c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcab9ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5ab9cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547ab9d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ab9d5, 67 },
    { 0, 0 },
    { 0x76bab9d7, 90 },
    { 0, 0 },
    { 0xbc6ab9d9, 34 },
    { 0, 0 },
    { 0x971ab9db, 52 },
    { 0xc64ab9dc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cab9e3, 68 },
    { 0, 0 },
    { 0xc1eab9e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ab9e9, 90 },
    { 0xfb7ab9e9, 98 },
    { 0x993ab9eb, 76 },
    { 0, 0 },
    { 0xf41ab9ed, 64 },
    { 0xe43ab9ee, 61 },
    { 0xb24ab9ee, 98 },
    { 0x5d0ab9ee, 76 },
    { 0, 0 },
    { 0xaf0ab9f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6ab9f7, 90 },
    { 0xbd4ab9f8, 33 },
    { 0x38eab9f8, 61 },
    { 0, 0 },
    { 0x29cab9fb, 66 },
    { 0xbc3ab9fc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28aba02, 90 },
    { 0x8c2aba03, 90 },
    { 0, 0 },
    { 0x225aba05, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0aba0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aaba0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaaba12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390aba16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9aba1a, 98 },
    { 0, 0 },
    { 0x623aba1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643aba20, 81 },
    { 0xcbfaba21, 65 },
    { 0x758aba22, 90 },
    { 0xb15aba21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19aba28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57aba2b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65aba30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529aba36, 90 },
    { 0, 0 },
    { 0x14baba38, 95 },
    { 0xf7aba39, 76 },
    { 0x8c0aba3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5caba44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642aba48, 34 },
    { 0, 0 },
    { 0xe02aba4a, 95 },
    { 0xaf7aba4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283aba4f, 95 },
    { 0x200aba50, 61 },
    { 0xe46aba51, 90 },
    { 0x436aba51, 94 },
    { 0xc99aba51, 61 },
    { 0xea8aba54, 33 },
    { 0, 0 },
    { 0x307aba56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0aba5c, 90 },
    { 0xf91aba5d, 61 },
    { 0x407aba5d, 36 },
    { 0xcbdaba5f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47caba6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1faba73, 94 },
    { 0, 0 },
    { 0xee2aba75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9aba7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0aba83, 33 },
    { 0xb8daba84, 33 },
    { 0x5adaba83, 95 },
    { 0xf74aba84, 34 },
    { 0xa96aba87, 77 },
    { 0xf36aba88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90aba8b, 23 },
    { 0x31eaba8c, 33 },
    { 0x86baba8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbaba91, 90 },
    { 0, 0 },
    { 0x732aba93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76aba9f, 23 },
    { 0x2b2abaa0, 90 },
    { 0x283abaa1, 98 },
    { 0x358abaa1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86abaa7, 100 },
    { 0x7e2abaa7, 83 },
    { 0x98dabaa9, 78 },
    { 0x350abaa9, 55 },
    { 0xbb8abaab, 61 },
    { 0xaababaac, 83 },
    { 0x105abaab, 61 },
    { 0xa1fabaa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765abab5, 67 },
    { 0xaf7abab5, 98 },
    { 0x4aaabab7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feababb, 98 },
    { 0xbfeababc, 98 },
    { 0x24cababc, 95 },
    { 0xb40ababe, 98 },
    { 0x4b5ababe, 90 },
    { 0, 0 },
    { 0xf7babac1, 90 },
    { 0x1a7abac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5abac9, 98 },
    { 0x5f4abaca, 56 },
    { 0xc06abaca, 93 },
    { 0, 0 },
    { 0x9faabacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910abad1, 92 },
    { 0x629abad2, 98 },
    { 0xab7abad3, 33 },
    { 0x74cabad4, 77 },
    { 0x79eabad5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4abada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eabadd, 90 },
    { 0x5f6abade, 98 },
    { 0x42fabadf, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2abae4, 76 },
    { 0xdb0abae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5abae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eabaf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41aabafd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4abb09, 33 },
    { 0xb70abb0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fabb0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6abb10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26abb16, 93 },
    { 0, 0 },
    { 0x584abb18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2abb1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cabb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fabb32, 34 },
    { 0x720abb33, 23 },
    { 0x643abb34, 95 },
    { 0xf30abb35, 88 },
    { 0xf3cabb35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595abb41, 55 },
    { 0, 0 },
    { 0x524abb43, 50 },
    { 0xc21abb44, 61 },
    { 0x315abb45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fabb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8abb4e, 23 },
    { 0x790abb4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aabb53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2abb59, 90 },
    { 0xa0eabb5a, 34 },
    { 0x5e5abb5b, 91 },
    { 0xf59abb5c, 90 },
    { 0xa8fabb5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68abb61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41abb65, 93 },
    { 0, 0 },
    { 0xb14abb67, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86abb6b, 23 },
    { 0x856abb6c, 61 },
    { 0x90eabb6d, 61 },
    { 0x9d4abb6e, 61 },
    { 0x14dabb6f, 98 },
    { 0x613abb6e, 34 },
    { 0x476abb71, 81 },
    { 0x1e9abb6f, 98 },
    { 0xf70abb6f, 95 },
    { 0x1ebabb74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83abb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9abb7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4abb85, 85 },
    { 0xbcdabb85, 23 },
    { 0x12cabb86, 76 },
    { 0x699abb88, 96 },
    { 0x187abb89, 92 },
    { 0, 0 },
    { 0xdbdabb8b, 90 },
    { 0x598abb8b, 84 },
    { 0, 0 },
    { 0x9b5abb8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46abb96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902abb9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3abb9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afabba9, 52 },
    { 0, 0 },
    { 0xcc8abbab, 90 },
    { 0xceaabbab, 22 },
    { 0, 0 },
    { 0xb2abbae, 76 },
    { 0, 0 },
    { 0x4c1abbb0, 44 },
    { 0x808abbb1, 46 },
    { 0, 0 },
    { 0x36cabbb3, 33 },
    { 0xea6abbb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2abbba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efabbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4abbc0, 23 },
    { 0xee0abbc0, 61 },
    { 0x784abbc0, 61 },
    { 0x389abbc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dabbcb, 33 },
    { 0x4eeabbcc, 95 },
    { 0x366abbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cabbd1, 90 },
    { 0x37abbd1, 81 },
    { 0, 0 },
    { 0x99fabbd4, 33 },
    { 0, 0 },
    { 0x92fabbd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25abbda, 61 },
    { 0x97cabbdb, 90 },
    { 0xe28abbdb, 43 },
    { 0x926abbdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dabbe0, 93 },
    { 0, 0 },
    { 0xed7abbe2, 64 },
    { 0, 0 },
    { 0x841abbe4, 90 },
    { 0x643abbe5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5abbe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9abbee, 33 },
    { 0xc7dabbee, 43 },
    { 0x51cabbf0, 61 },
    { 0x25dabbf0, 99 },
    { 0x92abbf2, 33 },
    { 0xdf4abbf2, 98 },
    { 0x1a0abbf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47abbfa, 40 },
    { 0, 0 },
    { 0x88babbfc, 90 },
    { 0xa69abbfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeabc01, 95 },
    { 0, 0 },
    { 0xdd1abc03, 76 },
    { 0, 0 },
    { 0xe2fabc05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbabc08, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x723abc0b, 95 },
    { 0, 0 },
    { 0x2d6abc0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745abc16, 61 },
    { 0, 0 },
    { 0xc0abc18, 90 },
    { 0x31babc18, 90 },
    { 0, 0 },
    { 0x505abc1b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eabc1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adabc24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279abc29, 61 },
    { 0x10cabc2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87babc2e, 55 },
    { 0, 0 },
    { 0x771abc30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559abc34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefabc37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54abc3b, 34 },
    { 0xd3abc3c, 33 },
    { 0, 0 },
    { 0x7daabc3e, 90 },
    { 0xb25abc3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3abc48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fabc4c, 95 },
    { 0x2c6abc4d, 76 },
    { 0xb8abc4e, 23 },
    { 0x87aabc4f, 46 },
    { 0, 0 },
    { 0x91dabc51, 90 },
    { 0xf04abc51, 95 },
    { 0x856abc52, 33 },
    { 0, 0 },
    { 0x9f8abc55, 43 },
    { 0xa40abc56, 90 },
    { 0x96dabc57, 61 },
    { 0xc0babc57, 95 },
    { 0xa50abc58, 46 },
    { 0x1fdabc5a, 78 },
    { 0x540abc5b, 91 },
    { 0x243abc5b, 88 },
    { 0x13aabc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eabc64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdabc67, 66 },
    { 0xb9fabc67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2abc6b, 98 },
    { 0xebfabc6c, 90 },
    { 0xabfabc6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fabc73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427abc79, 95 },
    { 0x14babc7a, 90 },
    { 0xaebabc7b, 77 },
    { 0xc83abc7c, 66 },
    { 0x66cabc7a, 61 },
    { 0xedeabc7e, 43 },
    { 0xb32abc7f, 84 },
    { 0x461abc80, 23 },
    { 0x84eabc81, 55 },
    { 0x26fabc82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aabc85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0abc89, 33 },
    { 0, 0 },
    { 0xcbbabc8b, 33 },
    { 0xb32abc8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6abc8f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x984abc92, 84 },
    { 0, 0 },
    { 0x1efabc94, 12 },
    { 0x3d7abc94, 98 },
    { 0xf21abc96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6abc9b, 87 },
    { 0xa70abc9b, 36 },
    { 0x5fdabc9c, 90 },
    { 0x42dabc9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cabca2, 61 },
    { 0x1c6abca3, 93 },
    { 0, 0 },
    { 0xdf2abca5, 92 },
    { 0xf41abca6, 33 },
    { 0xdbabca7, 23 },
    { 0xa67abca7, 98 },
    { 0x74cabca7, 23 },
    { 0xeb6abca7, 95 },
    { 0, 0 },
    { 0xb46abcac, 33 },
    { 0xbb4abcad, 34 },
    { 0x60aabcae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6abcb1, 33 },
    { 0xed4abcb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38abcb5, 66 },
    { 0, 0 },
    { 0xb97abcb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeabcba, 43 },
    { 0, 0 },
    { 0xc35abcbc, 33 },
    { 0x628abcbd, 55 },
    { 0xa26abcbe, 87 },
    { 0x43cabcbc, 43 },
    { 0xeffabcc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecabcc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x767abcc7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefabccf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabcd4, 33 },
    { 0x35babcd4, 98 },
    { 0xc55abcd6, 55 },
    { 0xd3babcd4, 78 },
    { 0xa1abcd6, 55 },
    { 0xe4cabcd9, 76 },
    { 0xb9fabcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368abcde, 52 },
    { 0, 0 },
    { 0x3b3abce0, 23 },
    { 0x3c0abce1, 43 },
    { 0, 0 },
    { 0xed8abce3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fabce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7abced, 76 },
    { 0, 0 },
    { 0x3deabcef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7abcf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0abcfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aabd01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91abd05, 99 },
    { 0xcb1abd06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56abd0f, 61 },
    { 0, 0 },
    { 0x4aeabd11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cabd18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76abd21, 91 },
    { 0x71dabd22, 33 },
    { 0x579abd23, 90 },
    { 0xe48abd22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eabd27, 61 },
    { 0x728abd27, 34 },
    { 0, 0 },
    { 0x4abd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdabd30, 76 },
    { 0, 0 },
    { 0x8c1abd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x999abd35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303abd39, 90 },
    { 0xf90abd39, 23 },
    { 0xa41abd3b, 95 },
    { 0xefabd39, 76 },
    { 0xa72abd3c, 76 },
    { 0xbfabd3c, 77 },
    { 0, 0 },
    { 0xf6eabd40, 83 },
    { 0xc6eabd41, 85 },
    { 0xb2abd42, 95 },
    { 0x9d5abd43, 90 },
    { 0x716abd43, 55 },
    { 0x90abd43, 76 },
    { 0x5e6abd43, 23 },
    { 0, 0 },
    { 0x647abd48, 33 },
    { 0, 0 },
    { 0xddabd4a, 95 },
    { 0, 0 },
    { 0x325abd4c, 34 },
    { 0xd89abd4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3abd52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x77abd55, 98 },
    { 0x80dabd56, 46 },
    { 0xa26abd56, 90 },
    { 0x2bbabd58, 55 },
    { 0xa06abd59, 85 },
    { 0xeceabd5a, 95 },
    { 0xfdabd5a, 43 },
    { 0xdceabd57, 33 },
    { 0x4cfabd5d, 23 },
    { 0x379abd5d, 56 },
    { 0x60eabd58, 61 },
    { 0x483abd5c, 90 },
    { 0x603abd61, 90 },
    { 0x97dabd61, 90 },
    { 0x123abd63, 33 },
    { 0x55abd64, 43 },
    { 0x677abd64, 61 },
    { 0x3aeabd64, 61 },
    { 0xa7cabd67, 69 },
    { 0x6c4abd68, 76 },
    { 0x84fabd63, 90 },
    { 0xef3abd6a, 23 },
    { 0, 0 },
    { 0xd9fabd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3abd71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89abd74, 34 },
    { 0xfecabd75, 94 },
    { 0x401abd76, 100 },
    { 0x58fabd76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cabd7a, 66 },
    { 0, 0 },
    { 0x1c9abd7c, 91 },
    { 0x1c0abd7d, 46 },
    { 0, 0 },
    { 0x134abd7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818abd83, 98 },
    { 0x171abd84, 98 },
    { 0x560abd83, 96 },
    { 0x3b7abd83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dabd8e, 98 },
    { 0, 0 },
    { 0x95babd90, 33 },
    { 0xf7cabd90, 66 },
    { 0, 0 },
    { 0xd3eabd93, 61 },
    { 0xb7cabd94, 90 },
    { 0, 0 },
    { 0x456abd96, 33 },
    { 0x737abd97, 88 },
    { 0x648abd98, 34 },
    { 0, 0 },
    { 0xb96abd9a, 93 },
    { 0x989abd9a, 46 },
    { 0x363abd9b, 55 },
    { 0xb42abd9d, 95 },
    { 0, 0 },
    { 0x42eabd9f, 88 },
    { 0x1adabd9f, 95 },
    { 0xb2babda1, 98 },
    { 0x4caabda2, 98 },
    { 0xacdabda3, 85 },
    { 0, 0 },
    { 0x275abda5, 99 },
    { 0x696abda6, 98 },
    { 0xc6eabda7, 33 },
    { 0, 0 },
    { 0xd04abda9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111abdad, 100 },
    { 0x84babdad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854abdb2, 65 },
    { 0, 0 },
    { 0xe98abdb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x142abdb7, 61 },
    { 0x7d8abdb7, 61 },
    { 0xd5abdb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609abdbe, 95 },
    { 0x44eabdbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55abdc2, 55 },
    { 0, 0 },
    { 0x604abdc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbabdc8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3abdcf, 34 },
    { 0, 0 },
    { 0xf0dabdd1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fabdd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2abdd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fabddf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aabde8, 69 },
    { 0, 0 },
    { 0x515abdea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dabdf0, 34 },
    { 0, 0 },
    { 0x460abdf2, 72 },
    { 0x82fabdf3, 87 },
    { 0xcb5abdf3, 43 },
    { 0x38cabdf5, 99 },
    { 0x5a9abdf3, 55 },
    { 0xf64abdf2, 23 },
    { 0, 0 },
    { 0xe60abdf9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x312abdfc, 66 },
    { 0xdaeabdfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cabe05, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabe08, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85abe0d, 61 },
    { 0xc95abe0e, 33 },
    { 0xc25abe0f, 84 },
    { 0x709abe10, 61 },
    { 0x2b7abe11, 90 },
    { 0x1c7abe12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cabe15, 43 },
    { 0x309abe16, 66 },
    { 0xd6cabe16, 76 },
    { 0xf23abe17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5abe1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880abe23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edabe27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dabe2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844abe34, 17 },
    { 0, 0 },
    { 0x21babe36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaabe39, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eabe3c, 76 },
    { 0x115abe3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88babe40, 61 },
    { 0, 0 },
    { 0x49dabe42, 100 },
    { 0x44eabe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6abe47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3abe4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1abe50, 90 },
    { 0xac9abe51, 33 },
    { 0x2eabe52, 55 },
    { 0x3ffabe53, 61 },
    { 0x82dabe50, 95 },
    { 0x96eabe50, 98 },
    { 0, 0 },
    { 0x2adabe57, 90 },
    { 0x27cabe58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8abe5b, 90 },
    { 0x5caabe5b, 95 },
    { 0, 0 },
    { 0xa2cabe5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x498abe61, 33 },
    { 0x62fabe62, 61 },
    { 0xb87abe61, 90 },
    { 0xb63abe64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563abe6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23abe6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eabe73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4abe7e, 90 },
    { 0x57cabe7f, 61 },
    { 0x1d5abe7e, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639abe84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dabe89, 76 },
    { 0, 0 },
    { 0xab6abe8b, 52 },
    { 0, 0 },
    { 0xae2abe8d, 98 },
    { 0, 0 },
    { 0x2e6abe8f, 90 },
    { 0xd5babe90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dabe94, 66 },
    { 0x7d5abe94, 91 },
    { 0x95abe96, 95 },
    { 0, 0 },
    { 0xc79abe98, 93 },
    { 0xf06abe98, 78 },
    { 0xf4abe99, 61 },
    { 0x353abe98, 23 },
    { 0, 0 },
    { 0x9fbabe9d, 85 },
    { 0, 0 },
    { 0x520abe9f, 61 },
    { 0x299abe9f, 43 },
    { 0x446abea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88abea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97abeaa, 95 },
    { 0x188abeab, 93 },
    { 0x62abeac, 33 },
    { 0xd90abeab, 33 },
    { 0x58cabeae, 95 },
    { 0xcdaabeaa, 61 },
    { 0x30cabeb0, 98 },
    { 0xeaeabeb0, 55 },
    { 0xe95abeb2, 33 },
    { 0, 0 },
    { 0xd06abeb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91babeb8, 52 },
    { 0xe0fabeb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53abebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffabec2, 23 },
    { 0, 0 },
    { 0x9d6abec4, 61 },
    { 0x3b8abec5, 33 },
    { 0xfe7abec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fabecd, 61 },
    { 0x2d5abecd, 55 },
    { 0, 0 },
    { 0x682abed0, 77 },
    { 0, 0 },
    { 0x8b4abed2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abed6, 61 },
    { 0xd49abed7, 95 },
    { 0x83eabed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfabedd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05abee1, 43 },
    { 0xae0abee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463abeec, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927abef3, 95 },
    { 0x1d3abef4, 93 },
    { 0, 0 },
    { 0xf32abef6, 55 },
    { 0, 0 },
    { 0xbbbabef8, 95 },
    { 0x506abef8, 90 },
    { 0x837abefa, 33 },
    { 0x9ffabefa, 90 },
    { 0x993abefc, 43 },
    { 0x494abefb, 98 },
    { 0xe1abefe, 90 },
    { 0xc44abeff, 23 },
    { 0xce0abf00, 23 },
    { 0xb38abf01, 46 },
    { 0xf6fabf02, 93 },
    { 0x61babf03, 33 },
    { 0xa1eabf03, 99 },
    { 0x336abf05, 99 },
    { 0xaa7abf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957abf0b, 84 },
    { 0xe56abf0c, 96 },
    { 0xa98abf0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5abf10, 61 },
    { 0, 0 },
    { 0xeddabf12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3abf18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07abf1e, 81 },
    { 0x5a6abf1e, 90 },
    { 0xffbabf1e, 76 },
    { 0x9d6abf21, 34 },
    { 0, 0 },
    { 0x19abf23, 55 },
    { 0xfdaabf23, 52 },
    { 0, 0 },
    { 0x4edabf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5abf2b, 55 },
    { 0, 0 },
    { 0x140abf2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccabf30, 33 },
    { 0xdbeabf31, 90 },
    { 0x785abf31, 95 },
    { 0x26cabf33, 90 },
    { 0x76babf34, 90 },
    { 0x726abf32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574abf40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420abf4c, 80 },
    { 0xf2abf4d, 33 },
    { 0, 0 },
    { 0x18babf4f, 90 },
    { 0x6c7abf50, 98 },
    { 0, 0 },
    { 0x4ecabf52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeabf57, 64 },
    { 0x15cabf58, 76 },
    { 0, 0 },
    { 0xda1abf5a, 75 },
    { 0x29dabf5b, 9 },
    { 0, 0 },
    { 0x5b7abf5d, 33 },
    { 0x610abf5d, 98 },
    { 0x40babf5f, 76 },
    { 0, 0 },
    { 0x772abf61, 46 },
    { 0x464abf62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77abf67, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cabf6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6abf73, 76 },
    { 0xb8babf74, 91 },
    { 0, 0 },
    { 0xb3babf76, 76 },
    { 0xf98abf76, 95 },
    { 0x99fabf78, 93 },
    { 0x678abf76, 88 },
    { 0x887abf78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0babf7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x962abf81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1abf85, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52abf8b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fabf91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x947abf94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0abf9a, 90 },
    { 0, 0 },
    { 0x178abf9c, 90 },
    { 0x312abf9d, 91 },
    { 0x8cbabf9e, 33 },
    { 0x24eabf9f, 90 },
    { 0x541abf9e, 75 },
    { 0xd0aabfa1, 98 },
    { 0xcf0abfa1, 61 },
    { 0xcfaabfa2, 90 },
    { 0x502abfa2, 33 },
    { 0x15cabf9f, 93 },
    { 0x6bbabfa5, 98 },
    { 0xdbabfa7, 95 },
    { 0x94eabfa7, 12 },
    { 0, 0 },
    { 0x385abfaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124abfb2, 87 },
    { 0x5e6abfb3, 23 },
    { 0xb1dabfb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4aabfb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2abfbc, 43 },
    { 0xaf7abfbd, 85 },
    { 0x2d6abfbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eabfc9, 23 },
    { 0x65babfc9, 23 },
    { 0, 0 },
    { 0xb8eabfcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddabfd0, 98 },
    { 0, 0 },
    { 0xa7fabfd2, 98 },
    { 0, 0 },
    { 0x60abfd4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2abfd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01abfdc, 93 },
    { 0x9f0abfdc, 100 },
    { 0, 0 },
    { 0x85fabfdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cabfe5, 85 },
    { 0x94cabfe5, 90 },
    { 0xfb9abfe7, 23 },
    { 0x99abfe7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6abfee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35abff4, 23 },
    { 0x87dabff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eabffb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aabffe, 33 },
    { 0x8d2abfff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbac003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ac007, 47 },
    { 0x721ac008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ac00b, 66 },
    { 0x4b4ac00c, 90 },
    { 0x71ac00d, 23 },
    { 0x398ac00e, 66 },
    { 0x2f2ac00e, 98 },
    { 0x92dac00f, 33 },
    { 0, 0 },
    { 0xad0ac012, 99 },
    { 0x58aac013, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ac016, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac01a, 61 },
    { 0xdebac01b, 91 },
    { 0x228ac01a, 91 },
    { 0x3f6ac01d, 98 },
    { 0x28fac01d, 90 },
    { 0x3f4ac01f, 34 },
    { 0xbdbac01c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cac023, 23 },
    { 0, 0 },
    { 0x918ac025, 76 },
    { 0x926ac026, 93 },
    { 0, 0 },
    { 0x725ac028, 98 },
    { 0xbb1ac029, 81 },
    { 0x363ac02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ac032, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x855ac035, 52 },
    { 0xd6aac035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554ac03d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afac042, 23 },
    { 0xbedac043, 61 },
    { 0xa17ac044, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889ac04d, 40 },
    { 0xe51ac04e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ac051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816ac055, 87 },
    { 0xcb0ac056, 56 },
    { 0xbb7ac057, 40 },
    { 0xe6dac058, 98 },
    { 0x154ac057, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ac05d, 33 },
    { 0, 0 },
    { 0x968ac05f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bac063, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0ac066, 98 },
    { 0xc2aac066, 23 },
    { 0xed8ac066, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aac06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ac06f, 23 },
    { 0x5c2ac070, 98 },
    { 0xd91ac070, 99 },
    { 0x348ac072, 33 },
    { 0xb2eac073, 90 },
    { 0xa2aac074, 55 },
    { 0xb9dac075, 90 },
    { 0xac1ac076, 36 },
    { 0, 0 },
    { 0x291ac078, 33 },
    { 0, 0 },
    { 0xc05ac07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bac07d, 55 },
    { 0xbcaac07e, 87 },
    { 0, 0 },
    { 0xb9cac080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dac083, 23 },
    { 0x41bac084, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aac089, 78 },
    { 0xfd2ac08a, 93 },
    { 0x3b6ac08a, 98 },
    { 0xc73ac08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fac08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33ac094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679ac098, 34 },
    { 0, 0 },
    { 0xd44ac09a, 98 },
    { 0x43bac09b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15ac0a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6ac0a4, 46 },
    { 0x499ac0a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93ac0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aac0ac, 46 },
    { 0x814ac0ac, 84 },
    { 0, 0 },
    { 0x9d8ac0af, 76 },
    { 0x283ac0b0, 65 },
    { 0x6bdac0b1, 61 },
    { 0xab5ac0b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x538ac0b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bac0ba, 23 },
    { 0x11cac0bb, 34 },
    { 0xb9fac0bb, 23 },
    { 0x3d8ac0bc, 76 },
    { 0x885ac0be, 52 },
    { 0x7d7ac0bf, 33 },
    { 0x479ac0bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bac0c3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac0cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1ac0d0, 34 },
    { 0xb5eac0d1, 90 },
    { 0x572ac0d1, 95 },
    { 0x9d7ac0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40ac0d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ac0d9, 43 },
    { 0x625ac0da, 98 },
    { 0xfe8ac0db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15ac0e4, 98 },
    { 0, 0 },
    { 0x820ac0e6, 40 },
    { 0, 0 },
    { 0x8aac0e8, 84 },
    { 0xb78ac0e9, 52 },
    { 0xd07ac0ea, 98 },
    { 0x265ac0ea, 93 },
    { 0xdebac0e9, 33 },
    { 0x8cac0ec, 96 },
    { 0x7baac0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdac0f1, 43 },
    { 0, 0 },
    { 0xf7eac0f3, 93 },
    { 0, 0 },
    { 0x6dcac0f5, 93 },
    { 0x606ac0f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eac0fd, 23 },
    { 0, 0 },
    { 0xcefac0ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ac102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ac107, 76 },
    { 0x1f2ac108, 67 },
    { 0x993ac109, 76 },
    { 0xe08ac10a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fac111, 33 },
    { 0x684ac112, 76 },
    { 0x17aac113, 98 },
    { 0xd0aac114, 76 },
    { 0x91eac115, 23 },
    { 0, 0 },
    { 0xf28ac117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ac122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69ac12a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffac12d, 33 },
    { 0xf60ac12e, 61 },
    { 0xf98ac12e, 33 },
    { 0x6d4ac130, 67 },
    { 0xa1ac131, 85 },
    { 0x65aac131, 78 },
    { 0x80aac12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9ac137, 76 },
    { 0x3aeac138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bac13d, 76 },
    { 0x409ac13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981ac144, 90 },
    { 0x7eac145, 61 },
    { 0xaeac146, 43 },
    { 0xd1ac146, 91 },
    { 0x3f1ac148, 90 },
    { 0xb60ac149, 90 },
    { 0x676ac14a, 23 },
    { 0, 0 },
    { 0x281ac14c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48ac150, 43 },
    { 0, 0 },
    { 0x120ac152, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188ac156, 95 },
    { 0xe62ac157, 52 },
    { 0x297ac157, 90 },
    { 0xb48ac158, 76 },
    { 0x44cac156, 33 },
    { 0x1d5ac159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ac161, 76 },
    { 0xf87ac161, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbac165, 91 },
    { 0, 0 },
    { 0x95dac167, 61 },
    { 0, 0 },
    { 0xdd7ac169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ac16c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6ac171, 34 },
    { 0, 0 },
    { 0x5ddac173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ac176, 33 },
    { 0xcdfac176, 95 },
    { 0, 0 },
    { 0x60fac179, 90 },
    { 0xeabac179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13ac17d, 23 },
    { 0, 0 },
    { 0x246ac17f, 56 },
    { 0x3b2ac180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcac188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceac191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ac196, 90 },
    { 0xc54ac197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ac19c, 93 },
    { 0x94aac19d, 77 },
    { 0, 0 },
    { 0xb34ac19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dac1a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aac1ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ac1b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bac1b5, 91 },
    { 0x6afac1b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747ac1bd, 61 },
    { 0, 0 },
    { 0xab0ac1bf, 90 },
    { 0, 0 },
    { 0xaaac1c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefac1c4, 81 },
    { 0, 0 },
    { 0x95fac1c6, 55 },
    { 0x3d9ac1c7, 95 },
    { 0xf09ac1c8, 93 },
    { 0, 0 },
    { 0x569ac1ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dac1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9ac1d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aac1db, 34 },
    { 0x1c9ac1dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x701ac1df, 90 },
    { 0, 0 },
    { 0x2abac1e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700ac1e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bac1f0, 43 },
    { 0, 0 },
    { 0x1dfac1f2, 91 },
    { 0x229ac1f3, 61 },
    { 0x45cac1f4, 87 },
    { 0, 0 },
    { 0xa3bac1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eac1fa, 60 },
    { 0xf00ac1fb, 90 },
    { 0, 0 },
    { 0x122ac1fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ac205, 90 },
    { 0xf88ac206, 52 },
    { 0x21bac207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208ac20a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61ac20d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ac210, 76 },
    { 0, 0 },
    { 0xc0bac212, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28ac21b, 61 },
    { 0x83bac21c, 93 },
    { 0x61fac21d, 95 },
    { 0, 0 },
    { 0x77ac21f, 98 },
    { 0x742ac220, 90 },
    { 0x964ac221, 76 },
    { 0, 0 },
    { 0x2aeac223, 76 },
    { 0x10bac224, 90 },
    { 0, 0 },
    { 0x9b2ac226, 55 },
    { 0xa2aac227, 61 },
    { 0x7adac228, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cac22e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539ac239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71eac23e, 33 },
    { 0x1c9ac23f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155ac244, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffac248, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bac24c, 61 },
    { 0xf0cac24d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ac251, 99 },
    { 0x866ac251, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bac257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ac25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeac262, 23 },
    { 0x841ac263, 33 },
    { 0xa91ac264, 87 },
    { 0, 0 },
    { 0x468ac266, 33 },
    { 0xbaaac267, 76 },
    { 0, 0 },
    { 0xb53ac269, 55 },
    { 0xf47ac26a, 90 },
    { 0, 0 },
    { 0xa9eac26c, 23 },
    { 0, 0 },
    { 0xaf8ac26e, 90 },
    { 0x565ac26f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac274, 95 },
    { 0x2f1ac275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24ac278, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21ac285, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ac28b, 33 },
    { 0, 0 },
    { 0x37eac28d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420ac291, 55 },
    { 0, 0 },
    { 0x99eac293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3ac297, 66 },
    { 0, 0 },
    { 0x6bbac299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0ac29c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aac2a1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eac2a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ac2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ac2ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac2af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64ac2b2, 84 },
    { 0, 0 },
    { 0xdc8ac2b4, 23 },
    { 0x86ac2b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aac2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790ac2bb, 90 },
    { 0x8a9ac2bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ac2c4, 23 },
    { 0x3a7ac2c5, 95 },
    { 0xc1eac2c6, 90 },
    { 0x715ac2c7, 90 },
    { 0xbc6ac2c8, 52 },
    { 0xbeaac2c9, 33 },
    { 0xc1dac2ca, 81 },
    { 0, 0 },
    { 0xee4ac2cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ac2d3, 95 },
    { 0, 0 },
    { 0xeb4ac2d5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4ac2d8, 46 },
    { 0, 0 },
    { 0x1f7ac2da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fac2df, 46 },
    { 0, 0 },
    { 0x58cac2e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac2e4, 71 },
    { 0xb23ac2e5, 95 },
    { 0, 0 },
    { 0x15fac2e7, 52 },
    { 0x481ac2e8, 61 },
    { 0, 0 },
    { 0x716ac2ea, 78 },
    { 0x9abac2ea, 23 },
    { 0xcdac2ec, 81 },
    { 0x98ac2ed, 95 },
    { 0xef0ac2ec, 91 },
    { 0xd90ac2eb, 95 },
    { 0, 0 },
    { 0x76fac2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ac2f6, 90 },
    { 0x133ac2f7, 78 },
    { 0xef9ac2f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413ac2fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ac300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518ac304, 66 },
    { 0xe92ac305, 84 },
    { 0, 0 },
    { 0x13fac307, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac30b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6ac313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac319, 23 },
    { 0xc13ac31a, 93 },
    { 0, 0 },
    { 0x925ac31c, 93 },
    { 0xaabac31d, 98 },
    { 0x643ac31e, 95 },
    { 0, 0 },
    { 0xea0ac320, 98 },
    { 0x6f1ac321, 33 },
    { 0xa7fac321, 33 },
    { 0, 0 },
    { 0xd4ac324, 23 },
    { 0xe2fac325, 90 },
    { 0xd8cac325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ac32b, 76 },
    { 0x8a5ac32b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedac32f, 90 },
    { 0xe53ac330, 98 },
    { 0xf9cac331, 98 },
    { 0xa5dac331, 90 },
    { 0, 0 },
    { 0x7fac334, 43 },
    { 0xa06ac335, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffac344, 33 },
    { 0x4abac345, 95 },
    { 0x69dac344, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eac34a, 93 },
    { 0, 0 },
    { 0xd42ac34c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dac350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35ac35d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efac365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1ac368, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0ac373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5ac376, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x839ac379, 33 },
    { 0x20ac37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eac382, 76 },
    { 0, 0 },
    { 0x29cac384, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9ac389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35ac38c, 98 },
    { 0x3f0ac38c, 61 },
    { 0, 0 },
    { 0x5a9ac38f, 88 },
    { 0xcccac38f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ac395, 61 },
    { 0xef3ac395, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82ac39b, 98 },
    { 0xb39ac39c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95aac39f, 33 },
    { 0, 0 },
    { 0xebeac3a1, 40 },
    { 0, 0 },
    { 0x778ac3a3, 23 },
    { 0xed0ac3a4, 55 },
    { 0x8a0ac3a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4ac3bb, 98 },
    { 0xeb5ac3bc, 61 },
    { 0, 0 },
    { 0x781ac3be, 68 },
    { 0xb00ac3bf, 98 },
    { 0xea5ac3c0, 61 },
    { 0, 0 },
    { 0x6e5ac3c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cac3cb, 97 },
    { 0xde8ac3cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5ac3cf, 100 },
    { 0xefac3d0, 33 },
    { 0xfd5ac3cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91ac3d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6ac3dd, 87 },
    { 0, 0 },
    { 0x457ac3df, 23 },
    { 0xc36ac3df, 33 },
    { 0x60cac3e1, 90 },
    { 0x926ac3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fac3e7, 33 },
    { 0x857ac3e8, 23 },
    { 0x8ceac3e8, 90 },
    { 0x3e1ac3ea, 61 },
    { 0x913ac3ea, 87 },
    { 0x919ac3ec, 91 },
    { 0x359ac3ed, 23 },
    { 0x162ac3ed, 19 },
    { 0x52cac3ee, 61 },
    { 0, 0 },
    { 0xbf6ac3f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584ac3f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ac3fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7ac400, 61 },
    { 0, 0 },
    { 0x2cac402, 46 },
    { 0xb6aac403, 33 },
    { 0x8aac403, 88 },
    { 0x4a9ac403, 55 },
    { 0x144ac406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fac40c, 98 },
    { 0x52eac40c, 61 },
    { 0xb39ac40e, 61 },
    { 0xef2ac40e, 75 },
    { 0x39cac40c, 90 },
    { 0xf79ac411, 61 },
    { 0xc41ac412, 58 },
    { 0, 0 },
    { 0xaeac414, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3ac418, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653ac41d, 61 },
    { 0x5c7ac41d, 61 },
    { 0x9a5ac41f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bac422, 55 },
    { 0, 0 },
    { 0x96aac424, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dac427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eac42c, 61 },
    { 0, 0 },
    { 0x954ac42e, 99 },
    { 0, 0 },
    { 0x70fac430, 34 },
    { 0xc86ac431, 78 },
    { 0x8dcac432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cac435, 83 },
    { 0, 0 },
    { 0x8e6ac437, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeac43b, 95 },
    { 0, 0 },
    { 0x8e1ac43d, 76 },
    { 0, 0 },
    { 0xcb9ac43f, 61 },
    { 0x250ac440, 55 },
    { 0x223ac43f, 100 },
    { 0xd24ac43f, 98 },
    { 0x103ac441, 33 },
    { 0xfd6ac444, 93 },
    { 0x1e4ac43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1ac450, 90 },
    { 0x5b2ac451, 17 },
    { 0, 0 },
    { 0xe96ac453, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x968ac456, 76 },
    { 0xe91ac457, 43 },
    { 0x31eac458, 55 },
    { 0xce1ac456, 96 },
    { 0x55fac45a, 91 },
    { 0xfceac456, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225ac461, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ac464, 77 },
    { 0x1eaac465, 33 },
    { 0, 0 },
    { 0x698ac467, 33 },
    { 0, 0 },
    { 0x44eac469, 61 },
    { 0, 0 },
    { 0x309ac46b, 55 },
    { 0xf9dac46c, 33 },
    { 0x172ac46d, 95 },
    { 0xe94ac46e, 55 },
    { 0xdbac46f, 33 },
    { 0x16cac46f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fac476, 23 },
    { 0xb9ac477, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63ac47b, 61 },
    { 0xf47ac47c, 17 },
    { 0, 0 },
    { 0x320ac47e, 91 },
    { 0x72cac47f, 53 },
    { 0x34dac47f, 95 },
    { 0x730ac481, 23 },
    { 0x50eac481, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5ac485, 33 },
    { 0, 0 },
    { 0xc77ac487, 55 },
    { 0xfe2ac487, 33 },
    { 0xe59ac488, 91 },
    { 0xb11ac48a, 33 },
    { 0x942ac488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eac48e, 90 },
    { 0xc31ac48f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feac493, 61 },
    { 0x19dac494, 87 },
    { 0, 0 },
    { 0x9f4ac496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ac49b, 78 },
    { 0x124ac49b, 55 },
    { 0x354ac49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9ac4a2, 76 },
    { 0x9b7ac4a3, 33 },
    { 0x333ac4a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0ac4aa, 33 },
    { 0, 0 },
    { 0x28dac4ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8aac4b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ac4b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7ac4be, 76 },
    { 0xcb9ac4be, 95 },
    { 0xabdac4be, 90 },
    { 0x9b9ac4c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebac4c6, 61 },
    { 0x6d9ac4c7, 46 },
    { 0x810ac4c6, 17 },
    { 0x3c0ac4c9, 33 },
    { 0, 0 },
    { 0x88dac4cb, 81 },
    { 0x57aac4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163ac4d4, 98 },
    { 0x1cfac4d5, 90 },
    { 0, 0 },
    { 0x647ac4d7, 95 },
    { 0xd14ac4d8, 46 },
    { 0x30bac4d8, 61 },
    { 0xa31ac4da, 90 },
    { 0, 0 },
    { 0x1c1ac4dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359ac4e0, 34 },
    { 0x267ac4e1, 95 },
    { 0x82fac4e0, 72 },
    { 0x6a2ac4e3, 55 },
    { 0x37eac4e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ac4f0, 76 },
    { 0x547ac4f1, 90 },
    { 0, 0 },
    { 0xfddac4f3, 55 },
    { 0x4e5ac4f4, 85 },
    { 0xf64ac4f5, 87 },
    { 0, 0 },
    { 0xadcac4f7, 91 },
    { 0, 0 },
    { 0xfafac4f9, 98 },
    { 0, 0 },
    { 0x890ac4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ac4ff, 61 },
    { 0xe78ac500, 90 },
    { 0, 0 },
    { 0x259ac502, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baac507, 23 },
    { 0x54eac508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1ac50d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fac512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bac516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcac51b, 95 },
    { 0, 0 },
    { 0x68bac51d, 55 },
    { 0xbe7ac51e, 12 },
    { 0, 0 },
    { 0x695ac520, 55 },
    { 0x6b3ac521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x879ac524, 90 },
    { 0x53fac525, 66 },
    { 0, 0 },
    { 0xe63ac527, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95ac52e, 55 },
    { 0xbfaac52f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ac533, 90 },
    { 0x69fac533, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bac539, 52 },
    { 0xfdbac539, 101 },
    { 0x469ac539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aac53f, 61 },
    { 0, 0 },
    { 0x93bac541, 100 },
    { 0x6b6ac541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacac54b, 98 },
    { 0x123ac54c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x368ac54f, 99 },
    { 0, 0 },
    { 0xcf5ac551, 95 },
    { 0x2d2ac552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeac556, 46 },
    { 0xee6ac557, 76 },
    { 0x156ac556, 85 },
    { 0, 0 },
    { 0xa55ac55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ac55f, 61 },
    { 0x7a2ac55f, 68 },
    { 0x4f2ac561, 43 },
    { 0, 0 },
    { 0x4dfac563, 85 },
    { 0, 0 },
    { 0xd50ac565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7eac568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ac572, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ac577, 68 },
    { 0xbf7ac578, 95 },
    { 0x43cac577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dac57d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac583, 23 },
    { 0xbaac584, 46 },
    { 0x200ac585, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ac58b, 90 },
    { 0, 0 },
    { 0x87cac58d, 33 },
    { 0x270ac58e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644ac592, 93 },
    { 0x7d7ac593, 23 },
    { 0x9adac594, 34 },
    { 0x1edac595, 91 },
    { 0xcc1ac594, 61 },
    { 0xd98ac593, 46 },
    { 0x203ac598, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704ac59f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abac5a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ac5a6, 95 },
    { 0xfa3ac5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddac5aa, 34 },
    { 0, 0 },
    { 0x492ac5ac, 23 },
    { 0x8f7ac5ad, 33 },
    { 0xc75ac5ae, 66 },
    { 0xe4bac5af, 85 },
    { 0x6cfac5ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140ac5b5, 61 },
    { 0x65ac5b6, 23 },
    { 0xf8fac5b6, 33 },
    { 0x1a2ac5b6, 91 },
    { 0, 0 },
    { 0xc67ac5ba, 80 },
    { 0x9b3ac5bb, 33 },
    { 0x88bac5bc, 23 },
    { 0x83cac5bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eac5c1, 43 },
    { 0, 0 },
    { 0xc77ac5c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ac5d3, 91 },
    { 0, 0 },
    { 0x2c7ac5d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dac5d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417ac5dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137ac5e8, 33 },
    { 0x198ac5e8, 61 },
    { 0x991ac5ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcac5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eac5f1, 23 },
    { 0x828ac5f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871ac5f7, 98 },
    { 0x750ac5f7, 75 },
    { 0, 0 },
    { 0x5cac5fa, 95 },
    { 0, 0 },
    { 0x230ac5fc, 23 },
    { 0x16aac5fd, 33 },
    { 0, 0 },
    { 0xf8cac5ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96ac604, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3ac608, 98 },
    { 0xbcdac608, 23 },
    { 0x4f0ac60a, 55 },
    { 0x70fac60a, 90 },
    { 0x9c3ac60a, 95 },
    { 0xceeac60d, 34 },
    { 0x1faac60e, 95 },
    { 0x192ac60d, 33 },
    { 0x164ac610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac614, 76 },
    { 0x7a9ac614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edac618, 61 },
    { 0xdfcac618, 93 },
    { 0x271ac618, 87 },
    { 0, 0 },
    { 0x7d0ac61c, 23 },
    { 0xde8ac61d, 99 },
    { 0xfbcac61e, 49 },
    { 0, 0 },
    { 0x6f7ac620, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dac624, 33 },
    { 0x3dcac625, 62 },
    { 0x913ac626, 80 },
    { 0x468ac626, 90 },
    { 0x768ac628, 98 },
    { 0, 0 },
    { 0x4abac62a, 33 },
    { 0x375ac62b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ac633, 61 },
    { 0x8a6ac633, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9ac638, 38 },
    { 0, 0 },
    { 0xe9ac63a, 55 },
    { 0, 0 },
    { 0xdf2ac63c, 78 },
    { 0, 0 },
    { 0x78fac63e, 40 },
    { 0x9d1ac63e, 90 },
    { 0x947ac63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30ac643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ac646, 90 },
    { 0, 0 },
    { 0x1bdac648, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ac64b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ac64e, 61 },
    { 0x202ac64e, 53 },
    { 0x258ac64e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3ac654, 78 },
    { 0, 0 },
    { 0xa6aac656, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cac65c, 46 },
    { 0, 0 },
    { 0xe15ac65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eac668, 98 },
    { 0x429ac669, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcac66c, 43 },
    { 0, 0 },
    { 0x60eac66e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x407ac671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558ac675, 46 },
    { 0, 0 },
    { 0xa1dac677, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ac67d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bac685, 66 },
    { 0x57dac685, 61 },
    { 0xee0ac687, 91 },
    { 0x68eac687, 26 },
    { 0xb99ac689, 55 },
    { 0x26dac689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438ac68e, 52 },
    { 0xf00ac68f, 76 },
    { 0x526ac68f, 90 },
    { 0x9a2ac690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39ac694, 23 },
    { 0xaf1ac695, 23 },
    { 0x8f5ac696, 95 },
    { 0x48ac696, 98 },
    { 0, 0 },
    { 0x71ac699, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ac69e, 91 },
    { 0xdf8ac69e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dac6a6, 87 },
    { 0xec7ac6a7, 90 },
    { 0xb93ac6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6ac6b1, 40 },
    { 0x3b2ac6b1, 95 },
    { 0xda0ac6b3, 33 },
    { 0x62eac6b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcac6b8, 78 },
    { 0, 0 },
    { 0xa4ac6ba, 38 },
    { 0x923ac6ba, 66 },
    { 0x363ac6bb, 98 },
    { 0xa6dac6ba, 98 },
    { 0x2e6ac6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964ac6c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abac6c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdac6cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cac6cf, 34 },
    { 0x5f6ac6d0, 90 },
    { 0x5cac6d1, 35 },
    { 0x732ac6d1, 67 },
    { 0x174ac6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fac6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04ac6db, 90 },
    { 0x4d5ac6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5ac6e0, 55 },
    { 0xad0ac6e1, 95 },
    { 0x750ac6e2, 76 },
    { 0x437ac6e3, 51 },
    { 0, 0 },
    { 0xffac6e5, 99 },
    { 0xc19ac6e6, 61 },
    { 0x3baac6e7, 93 },
    { 0x668ac6e6, 90 },
    { 0, 0 },
    { 0xcb4ac6ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2ac6ed, 61 },
    { 0xe41ac6ed, 43 },
    { 0x38eac6ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ac6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7ac6f7, 33 },
    { 0xb93ac6f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bac6fb, 90 },
    { 0x8f8ac6fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ac700, 61 },
    { 0xc3eac701, 95 },
    { 0x105ac702, 90 },
    { 0x15fac700, 33 },
    { 0x713ac704, 81 },
    { 0x9d7ac702, 98 },
    { 0x974ac703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fac70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cac715, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ac718, 34 },
    { 0xfacac718, 53 },
    { 0xfeaac71a, 84 },
    { 0x86dac71b, 33 },
    { 0x8bbac71b, 95 },
    { 0xc86ac71a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199ac723, 55 },
    { 0x43cac723, 90 },
    { 0xcc1ac723, 23 },
    { 0xfb4ac723, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbac72a, 87 },
    { 0, 0 },
    { 0x74bac72c, 95 },
    { 0x1ebac72c, 90 },
    { 0x2c1ac72c, 95 },
    { 0x38bac72f, 23 },
    { 0x58aac72f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51ac733, 61 },
    { 0x6e8ac734, 87 },
    { 0x78ac735, 33 },
    { 0x4caac734, 95 },
    { 0xc0bac737, 95 },
    { 0x85aac738, 95 },
    { 0x8a0ac738, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08ac73c, 90 },
    { 0x83bac73c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1ac741, 61 },
    { 0, 0 },
    { 0xaacac743, 87 },
    { 0, 0 },
    { 0xc8eac745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9ac748, 93 },
    { 0x233ac748, 33 },
    { 0x1edac749, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ac750, 33 },
    { 0xc15ac751, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ac754, 46 },
    { 0xea3ac755, 33 },
    { 0x4b4ac756, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ac759, 23 },
    { 0, 0 },
    { 0xf5ac75b, 67 },
    { 0, 0 },
    { 0x104ac75d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cac762, 34 },
    { 0x3d3ac762, 43 },
    { 0, 0 },
    { 0x463ac765, 61 },
    { 0x366ac766, 100 },
    { 0xba8ac767, 43 },
    { 0, 0 },
    { 0xdceac769, 90 },
    { 0x734ac769, 95 },
    { 0x1ceac76b, 97 },
    { 0x562ac76c, 98 },
    { 0x24eac76b, 76 },
    { 0xcf2ac769, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dac771, 61 },
    { 0, 0 },
    { 0xb93ac773, 77 },
    { 0x98fac774, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ac780, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ac784, 46 },
    { 0, 0 },
    { 0x297ac786, 25 },
    { 0xdb3ac787, 80 },
    { 0x540ac788, 76 },
    { 0x3dcac789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acac78e, 46 },
    { 0, 0 },
    { 0x662ac790, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44ac794, 93 },
    { 0, 0 },
    { 0x732ac796, 46 },
    { 0x1c6ac797, 98 },
    { 0x67fac796, 95 },
    { 0, 0 },
    { 0xd16ac79a, 67 },
    { 0xa56ac79b, 95 },
    { 0x9c6ac79a, 67 },
    { 0, 0 },
    { 0x6ffac79e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281ac7a2, 95 },
    { 0xf07ac7a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ac7a7, 33 },
    { 0, 0 },
    { 0xeb0ac7a9, 76 },
    { 0x53fac7aa, 33 },
    { 0xb8ac7ab, 76 },
    { 0xa18ac7ac, 23 },
    { 0x873ac7aa, 84 },
    { 0x1f6ac7ae, 23 },
    { 0xe05ac7af, 91 },
    { 0xd2eac7b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88ac7bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dac7c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4ac7c5, 95 },
    { 0x269ac7c6, 90 },
    { 0x7d7ac7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5ac7ca, 76 },
    { 0, 0 },
    { 0xd29ac7cc, 95 },
    { 0x3bcac7cd, 61 },
    { 0xdaaac7ce, 33 },
    { 0x28cac7cf, 93 },
    { 0xaa9ac7cf, 76 },
    { 0x76aac7cd, 33 },
    { 0x4beac7d2, 84 },
    { 0x31ac7d3, 33 },
    { 0xc04ac7d2, 91 },
    { 0xeb2ac7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecac7d8, 90 },
    { 0, 0 },
    { 0x6daac7da, 95 },
    { 0x255ac7db, 90 },
    { 0x34fac7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ac7e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cac7e5, 100 },
    { 0x134ac7e6, 17 },
    { 0x4e7ac7e7, 100 },
    { 0xe2aac7e7, 98 },
    { 0xf7bac7e7, 95 },
    { 0x230ac7ea, 55 },
    { 0, 0 },
    { 0x499ac7ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ac7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ac7f7, 43 },
    { 0xe50ac7f8, 80 },
    { 0xb61ac7f8, 98 },
    { 0, 0 },
    { 0x89dac7fb, 91 },
    { 0, 0 },
    { 0xff6ac7fd, 23 },
    { 0x505ac7fd, 86 },
    { 0xe64ac7ff, 98 },
    { 0x77dac800, 23 },
    { 0x1c2ac801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddac804, 90 },
    { 0x57fac804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54ac808, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89ac80b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefac811, 23 },
    { 0xfa1ac812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x752ac815, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfac818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ac81c, 95 },
    { 0xd36ac81d, 98 },
    { 0x76bac81d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bac821, 98 },
    { 0xcfac821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ac825, 76 },
    { 0x26dac825, 76 },
    { 0x64fac826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650ac82d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dac830, 46 },
    { 0xff1ac830, 66 },
    { 0x409ac832, 66 },
    { 0x5a2ac833, 43 },
    { 0xaa6ac832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dac838, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452ac83e, 33 },
    { 0, 0 },
    { 0x80dac840, 33 },
    { 0, 0 },
    { 0x1b1ac842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97ac845, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcac84c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ac850, 34 },
    { 0xab7ac851, 52 },
    { 0x72eac852, 55 },
    { 0x145ac851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ac859, 46 },
    { 0x58fac85a, 61 },
    { 0, 0 },
    { 0xc74ac85c, 33 },
    { 0xce9ac85d, 17 },
    { 0xa80ac85c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ac863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10ac866, 98 },
    { 0x685ac866, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ac86d, 61 },
    { 0xcf1ac86e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bac871, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ac875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fac878, 76 },
    { 0xe15ac879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167ac87e, 23 },
    { 0, 0 },
    { 0x410ac880, 81 },
    { 0x576ac881, 38 },
    { 0x640ac880, 84 },
    { 0x35cac883, 17 },
    { 0xb14ac882, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2ac887, 93 },
    { 0xb15ac887, 43 },
    { 0xb8cac889, 33 },
    { 0, 0 },
    { 0xb44ac88b, 68 },
    { 0xdcbac88b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79ac891, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x407ac894, 90 },
    { 0x8c8ac894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ac898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4ac89c, 98 },
    { 0, 0 },
    { 0xdb9ac89e, 43 },
    { 0xca1ac89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbac8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ac8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ac8b2, 93 },
    { 0x48ac8b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eac8b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33ac8bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bac8bf, 98 },
    { 0xc81ac8bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cac8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0ac8c8, 34 },
    { 0x2ebac8c8, 99 },
    { 0xb5bac8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ac8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532ac8d5, 52 },
    { 0xd77ac8d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedac8dc, 100 },
    { 0, 0 },
    { 0x7eeac8de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ac8e1, 93 },
    { 0, 0 },
    { 0x18ac8e3, 43 },
    { 0xdb2ac8e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379ac8e9, 55 },
    { 0x8a9ac8ea, 77 },
    { 0x8ccac8ea, 46 },
    { 0x8d3ac8eb, 23 },
    { 0xeb9ac8ed, 61 },
    { 0xaedac8ec, 90 },
    { 0x2e1ac8ee, 98 },
    { 0x68bac8f0, 66 },
    { 0x7f8ac8f1, 33 },
    { 0x6c1ac8f2, 33 },
    { 0xafeac8f1, 76 },
    { 0x31dac8ea, 34 },
    { 0xb85ac8f5, 95 },
    { 0xd0aac8f5, 98 },
    { 0x7b8ac8ef, 90 },
    { 0xfd1ac8e9, 61 },
    { 0x23ac8f9, 76 },
    { 0, 0 },
    { 0x218ac8fb, 82 },
    { 0x2fdac8fc, 90 },
    { 0x72ac8fc, 90 },
    { 0xd94ac8fd, 66 },
    { 0, 0 },
    { 0xfe1ac900, 61 },
    { 0, 0 },
    { 0xf8bac902, 98 },
    { 0x7a3ac903, 34 },
    { 0, 0 },
    { 0xbdeac905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac908, 43 },
    { 0, 0 },
    { 0xfd0ac90a, 95 },
    { 0, 0 },
    { 0x6f0ac90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ac910, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495ac915, 33 },
    { 0, 0 },
    { 0xecbac917, 23 },
    { 0xe0aac918, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4ac91b, 90 },
    { 0, 0 },
    { 0x8a1ac91d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376ac923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cac92a, 33 },
    { 0, 0 },
    { 0xa48ac92c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebac936, 91 },
    { 0x289ac937, 23 },
    { 0xfe0ac938, 95 },
    { 0xaaeac939, 61 },
    { 0, 0 },
    { 0x9f9ac93b, 90 },
    { 0x761ac93c, 90 },
    { 0x8c1ac93d, 23 },
    { 0x226ac93d, 33 },
    { 0xd85ac93d, 25 },
    { 0xc82ac940, 68 },
    { 0, 0 },
    { 0xeaaac942, 34 },
    { 0x120ac942, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ac947, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ac94b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2ac950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dac953, 73 },
    { 0x25aac954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cac95a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bac95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42ac960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ac964, 66 },
    { 0x72ac964, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ac96b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eac972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92ac976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13ac979, 23 },
    { 0, 0 },
    { 0x14cac97b, 98 },
    { 0x5dbac97b, 84 },
    { 0x7fbac97c, 61 },
    { 0xea0ac97e, 34 },
    { 0, 0 },
    { 0x77cac980, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94ac983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdac987, 95 },
    { 0xaefac987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3ac98d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4ac990, 33 },
    { 0x926ac991, 34 },
    { 0xd27ac990, 66 },
    { 0x8c0ac993, 23 },
    { 0, 0 },
    { 0xef7ac995, 23 },
    { 0x9ddac996, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bac9a0, 90 },
    { 0x21dac9a0, 23 },
    { 0, 0 },
    { 0x4ac9a3, 67 },
    { 0x20ac9a4, 90 },
    { 0xf80ac9a5, 61 },
    { 0xa9fac9a6, 81 },
    { 0x2a6ac9a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cac9aa, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbac9b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9ac9ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ac9c2, 90 },
    { 0x2bfac9c3, 76 },
    { 0, 0 },
    { 0x3bbac9c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac9c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5ac9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ac9d9, 61 },
    { 0x79aac9da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaac9dd, 61 },
    { 0xd55ac9dd, 76 },
    { 0xe12ac9dd, 19 },
    { 0x714ac9e0, 33 },
    { 0xdb0ac9e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeac9e7, 66 },
    { 0x586ac9e8, 23 },
    { 0x73bac9e7, 84 },
    { 0x39cac9e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x845ac9ed, 90 },
    { 0x596ac9ed, 52 },
    { 0, 0 },
    { 0x416ac9f0, 91 },
    { 0xbefac9f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0ac9f4, 99 },
    { 0x2ceac9f4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6ac9fc, 55 },
    { 0xd02ac9fd, 55 },
    { 0x4a4ac9fc, 100 },
    { 0x223ac9fd, 23 },
    { 0xfc6aca00, 43 },
    { 0, 0 },
    { 0xbaaca02, 33 },
    { 0, 0 },
    { 0x69faca04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29aca08, 46 },
    { 0, 0 },
    { 0x8dbaca0a, 61 },
    { 0x7f6aca0a, 90 },
    { 0, 0 },
    { 0x3fbaca0d, 95 },
    { 0x65baca0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549aca16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56aca1b, 91 },
    { 0xe01aca1c, 61 },
    { 0x1faaca1d, 76 },
    { 0x6c2aca1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x281aca21, 34 },
    { 0x202aca21, 55 },
    { 0, 0 },
    { 0xea5aca24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaca27, 76 },
    { 0xdd0aca28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacaca2b, 23 },
    { 0, 0 },
    { 0x252aca2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcaca30, 68 },
    { 0x35faca30, 76 },
    { 0, 0 },
    { 0x12eaca33, 61 },
    { 0, 0 },
    { 0xcc7aca35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x682aca38, 61 },
    { 0x940aca39, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646aca3e, 33 },
    { 0x6c7aca3f, 34 },
    { 0xd3faca3f, 33 },
    { 0, 0 },
    { 0x322aca42, 90 },
    { 0xba4aca43, 90 },
    { 0x11daca44, 87 },
    { 0xfd0aca44, 65 },
    { 0xc02aca46, 76 },
    { 0xab8aca46, 93 },
    { 0xa51aca48, 61 },
    { 0x48eaca49, 98 },
    { 0x4daca43, 61 },
    { 0x6e6aca48, 76 },
    { 0x1bfaca49, 46 },
    { 0, 0 },
    { 0x9c7aca4e, 55 },
    { 0xcb4aca4e, 52 },
    { 0xd89aca50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aaca53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2aca5b, 84 },
    { 0, 0 },
    { 0x205aca5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715aca62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30aca65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920aca69, 33 },
    { 0x4fdaca6a, 76 },
    { 0xb27aca6b, 98 },
    { 0xc59aca6b, 90 },
    { 0, 0 },
    { 0xd52aca6e, 65 },
    { 0x2d1aca6f, 33 },
    { 0xb7baca70, 91 },
    { 0x192aca71, 33 },
    { 0x2ffaca71, 52 },
    { 0, 0 },
    { 0x43eaca74, 74 },
    { 0xb39aca75, 98 },
    { 0, 0 },
    { 0x3aaca77, 43 },
    { 0x532aca78, 96 },
    { 0x6cdaca77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3aca7c, 98 },
    { 0, 0 },
    { 0x29aca7e, 43 },
    { 0x85baca7f, 33 },
    { 0x211aca7e, 34 },
    { 0xdabaca81, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1aca88, 34 },
    { 0, 0 },
    { 0xb91aca8a, 33 },
    { 0x558aca8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13daca8e, 90 },
    { 0, 0 },
    { 0x3aca90, 55 },
    { 0xe5aca90, 95 },
    { 0x70eaca92, 98 },
    { 0xfa3aca93, 34 },
    { 0x1aaaca93, 55 },
    { 0x5f8aca94, 90 },
    { 0x10aaca96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5aca9d, 34 },
    { 0xb31aca9d, 84 },
    { 0x726aca9f, 76 },
    { 0, 0 },
    { 0xb3bacaa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29acaa6, 61 },
    { 0x521acaa6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93acab1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0acab5, 77 },
    { 0x1c7acab6, 52 },
    { 0xcbfacab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcacac0, 52 },
    { 0x897acac0, 98 },
    { 0xd59acac2, 55 },
    { 0xf5cacac3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2acac6, 76 },
    { 0x1ceacac7, 23 },
    { 0x151acac8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0acad3, 55 },
    { 0xc10acad4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffacad8, 95 },
    { 0, 0 },
    { 0xf45acada, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135acae4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacaeb, 33 },
    { 0x134acaec, 76 },
    { 0, 0 },
    { 0xa5facaee, 100 },
    { 0, 0 },
    { 0x56cacaf0, 84 },
    { 0xa91acaf0, 17 },
    { 0xb0bacaf2, 78 },
    { 0x5fbacaf0, 93 },
    { 0xe77acaf4, 88 },
    { 0x243acaf5, 90 },
    { 0xdb7acaf6, 98 },
    { 0, 0 },
    { 0x711acaf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9acafd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bacb0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2acb10, 91 },
    { 0, 0 },
    { 0x3f9acb12, 23 },
    { 0xc86acb13, 33 },
    { 0x2b5acb13, 95 },
    { 0xb36acb13, 76 },
    { 0x960acb13, 23 },
    { 0xbfdacb17, 90 },
    { 0, 0 },
    { 0x10bacb19, 23 },
    { 0x3c5acb19, 81 },
    { 0, 0 },
    { 0xb68acb1c, 90 },
    { 0, 0 },
    { 0x906acb1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88acb24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477acb29, 95 },
    { 0x964acb2a, 98 },
    { 0, 0 },
    { 0x2aaacb2c, 33 },
    { 0x7ceacb2d, 98 },
    { 0, 0 },
    { 0x380acb2f, 91 },
    { 0, 0 },
    { 0x380acb31, 61 },
    { 0x984acb31, 61 },
    { 0x29facb31, 34 },
    { 0x765acb33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aacb40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaacb4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5acb4d, 43 },
    { 0x82acb4e, 98 },
    { 0x67aacb4f, 33 },
    { 0xabacb50, 61 },
    { 0xe77acb50, 33 },
    { 0x61dacb51, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646acb58, 61 },
    { 0, 0 },
    { 0xd31acb5a, 55 },
    { 0x383acb5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65facb60, 34 },
    { 0, 0 },
    { 0xc98acb62, 23 },
    { 0, 0 },
    { 0xa6bacb64, 76 },
    { 0, 0 },
    { 0xa31acb66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acb70, 95 },
    { 0x401acb70, 100 },
    { 0x93cacb70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaacb76, 46 },
    { 0x91cacb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9acb7b, 90 },
    { 0xf1facb7b, 93 },
    { 0xb37acb7d, 98 },
    { 0, 0 },
    { 0xa1acb7f, 100 },
    { 0xf57acb7f, 99 },
    { 0xb8cacb81, 33 },
    { 0xd94acb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68eacb88, 78 },
    { 0xa23acb88, 76 },
    { 0x1c2acb8a, 36 },
    { 0, 0 },
    { 0x920acb8c, 46 },
    { 0x55facb8d, 90 },
    { 0xe77acb8e, 90 },
    { 0x5f4acb8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bacb95, 98 },
    { 0x294acb96, 93 },
    { 0x7abacb97, 75 },
    { 0x95dacb98, 95 },
    { 0x35acb99, 61 },
    { 0xc81acb9a, 61 },
    { 0x1faacb98, 100 },
    { 0x92cacb96, 84 },
    { 0xb9facb9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6acba0, 76 },
    { 0, 0 },
    { 0x3fdacba2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4acbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593acbae, 38 },
    { 0x4adacbaf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbacbb2, 98 },
    { 0x881acbb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10acbb6, 90 },
    { 0xbd0acbb6, 55 },
    { 0xa25acbb8, 32 },
    { 0x7bbacbb8, 90 },
    { 0x5aacbba, 99 },
    { 0, 0 },
    { 0x64bacbbc, 23 },
    { 0x18cacbbd, 33 },
    { 0x353acbbd, 34 },
    { 0x6d6acbbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380acbc3, 90 },
    { 0x478acbc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0acbc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2acbd2, 46 },
    { 0, 0 },
    { 0xaadacbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eacbd7, 90 },
    { 0x1eaacbd7, 93 },
    { 0xaaaacbd9, 95 },
    { 0x24facbda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56facbdd, 52 },
    { 0x799acbdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9acbe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9acbe5, 99 },
    { 0x15acbe6, 43 },
    { 0, 0 },
    { 0x3d4acbe8, 77 },
    { 0x507acbe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3acbed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0acbf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9acbf5, 23 },
    { 0x6a8acbf5, 98 },
    { 0xbf6acbf7, 38 },
    { 0x977acbf6, 34 },
    { 0x304acbf9, 33 },
    { 0x5d9acbf8, 93 },
    { 0xa53acbfb, 23 },
    { 0xdf9acbfb, 66 },
    { 0, 0 },
    { 0x9eaacbfe, 34 },
    { 0x1f3acbff, 97 },
    { 0, 0 },
    { 0x1f5acc01, 95 },
    { 0x5a1acc02, 90 },
    { 0x1f7acc01, 33 },
    { 0xf24acc04, 66 },
    { 0x47acc04, 43 },
    { 0, 0 },
    { 0xdc8acc07, 76 },
    { 0xa2eacc08, 95 },
    { 0x9beacc08, 95 },
    { 0x36cacc08, 87 },
    { 0xf7facc0b, 52 },
    { 0xd18acc09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68acc0f, 34 },
    { 0xa22acc0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4facc13, 61 },
    { 0x25cacc13, 33 },
    { 0x16aacc14, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadacc1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3acc1e, 55 },
    { 0xc2facc1f, 61 },
    { 0x8deacc20, 90 },
    { 0x272acc21, 17 },
    { 0, 0 },
    { 0x498acc23, 75 },
    { 0xe9acc24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23acc29, 90 },
    { 0x399acc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x940acc2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc30, 91 },
    { 0x6d8acc31, 90 },
    { 0x5f9acc32, 81 },
    { 0x453acc30, 33 },
    { 0xbcaacc34, 88 },
    { 0x27cacc34, 61 },
    { 0, 0 },
    { 0x25bacc37, 33 },
    { 0x165acc38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dacc3c, 98 },
    { 0xb01acc3d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aacc40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aacc46, 33 },
    { 0x860acc47, 61 },
    { 0x905acc46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dacc52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66acc55, 61 },
    { 0xeeeacc56, 95 },
    { 0x62cacc56, 34 },
    { 0xb0cacc57, 95 },
    { 0x87cacc59, 90 },
    { 0, 0 },
    { 0x6facc5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84acc5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeacc63, 61 },
    { 0, 0 },
    { 0x175acc65, 33 },
    { 0x24aacc65, 23 },
    { 0xd48acc66, 90 },
    { 0x4f3acc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01acc6b, 95 },
    { 0xb9cacc6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69acc6f, 33 },
    { 0x6d1acc70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efacc73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcacc77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x685acc7a, 76 },
    { 0xed6acc7a, 61 },
    { 0, 0 },
    { 0x2b4acc7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2acc81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91acc87, 76 },
    { 0x2fbacc87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cacc90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8facc94, 76 },
    { 0xc3facc94, 23 },
    { 0xf54acc96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebacc9c, 90 },
    { 0x9cbacc9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967acca4, 90 },
    { 0, 0 },
    { 0x26acca6, 33 },
    { 0x188acca6, 23 },
    { 0xaf7acca6, 33 },
    { 0x60acca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaccac, 90 },
    { 0x160accac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27baccb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1accb3, 98 },
    { 0, 0 },
    { 0x9efaccb5, 34 },
    { 0xf8daccb5, 23 },
    { 0x6d9accb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaccba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x725accbd, 91 },
    { 0x76aaccbe, 85 },
    { 0xe54accbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4accc7, 90 },
    { 0xb62accc8, 99 },
    { 0x8daaccc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75facccc, 90 },
    { 0, 0 },
    { 0x819accce, 98 },
    { 0, 0 },
    { 0x6d9accd0, 76 },
    { 0x13daccd1, 33 },
    { 0x1f5accd2, 33 },
    { 0, 0 },
    { 0x573accd4, 61 },
    { 0x204accd5, 43 },
    { 0x918accd6, 93 },
    { 0x46caccd7, 61 },
    { 0xba3accd7, 98 },
    { 0xbabaccd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55accdc, 52 },
    { 0xc27accdd, 55 },
    { 0x68aaccde, 95 },
    { 0x51faccdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x954acce2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94baccf0, 46 },
    { 0x7eeaccf0, 98 },
    { 0xaa3accf2, 45 },
    { 0xac3accf3, 55 },
    { 0x83aaccf4, 43 },
    { 0x40faccf4, 23 },
    { 0xcc4accf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9accfa, 78 },
    { 0x8a1accfb, 61 },
    { 0x2f2accfc, 17 },
    { 0, 0 },
    { 0x5d1accfe, 91 },
    { 0, 0 },
    { 0x54cacd00, 55 },
    { 0, 0 },
    { 0x37cacd02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcacd0a, 88 },
    { 0x7aaacd0a, 90 },
    { 0x97eacd0c, 90 },
    { 0, 0 },
    { 0x8bcacd0e, 95 },
    { 0, 0 },
    { 0x6cbacd10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783acd14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaacd1a, 98 },
    { 0, 0 },
    { 0x53facd1c, 61 },
    { 0xe9facd1d, 61 },
    { 0xdd9acd1e, 53 },
    { 0x1d1acd1d, 98 },
    { 0, 0 },
    { 0xb98acd21, 46 },
    { 0xb89acd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eacd26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1acd29, 55 },
    { 0x92cacd2a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7acd35, 100 },
    { 0x33facd36, 78 },
    { 0xd33acd37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edacd3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3acd3e, 90 },
    { 0xbf3acd3e, 67 },
    { 0xbc3acd40, 75 },
    { 0x1d6acd41, 95 },
    { 0x8feacd42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487acd46, 33 },
    { 0x6e3acd46, 34 },
    { 0x21acd48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65acd4b, 34 },
    { 0xf7bacd4c, 90 },
    { 0xdf1acd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bacd52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14facd56, 55 },
    { 0, 0 },
    { 0x180acd58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecacd5b, 52 },
    { 0, 0 },
    { 0xdabacd5d, 99 },
    { 0xfcacd5e, 98 },
    { 0x2dbacd5f, 76 },
    { 0x9c9acd5f, 95 },
    { 0x2c7acd61, 77 },
    { 0xc50acd62, 68 },
    { 0x22acd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15acd68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30acd6b, 33 },
    { 0xb05acd6b, 95 },
    { 0x9e3acd6d, 46 },
    { 0x9aaacd6e, 99 },
    { 0xbdfacd6f, 81 },
    { 0x69acd70, 55 },
    { 0xc9cacd71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669acd76, 23 },
    { 0x27cacd77, 78 },
    { 0, 0 },
    { 0xdc6acd79, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761acd80, 55 },
    { 0xc7acd80, 46 },
    { 0x81bacd82, 55 },
    { 0x950acd83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616acd87, 17 },
    { 0, 0 },
    { 0xe20acd89, 33 },
    { 0x1ecacd8a, 76 },
    { 0xbcdacd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79aacd8f, 91 },
    { 0xc9facd90, 87 },
    { 0xb33acd90, 33 },
    { 0x309acd92, 83 },
    { 0x24bacd93, 52 },
    { 0x424acd90, 46 },
    { 0xb0aacd90, 90 },
    { 0x698acd96, 93 },
    { 0xfc9acd97, 100 },
    { 0x281acd98, 23 },
    { 0, 0 },
    { 0xed3acd9a, 52 },
    { 0x30eacd9a, 67 },
    { 0xaa3acd9c, 99 },
    { 0x520acd9c, 34 },
    { 0x665acd9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eacda3, 76 },
    { 0x345acda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8acda7, 61 },
    { 0, 0 },
    { 0x58eacda9, 33 },
    { 0, 0 },
    { 0xd82acdab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cacdb1, 98 },
    { 0xbc2acdb2, 95 },
    { 0xea7acdb2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eacdb7, 101 },
    { 0xee8acdb7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86facdbf, 33 },
    { 0, 0 },
    { 0x1fcacdc1, 91 },
    { 0, 0 },
    { 0xe34acdc3, 61 },
    { 0x4b0acdc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94acdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadacdd2, 90 },
    { 0x70aacdd3, 95 },
    { 0, 0 },
    { 0xb9aacdd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaacdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356acde0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92acde3, 90 },
    { 0, 0 },
    { 0x1e2acde5, 76 },
    { 0x160acde6, 33 },
    { 0x4daacde5, 69 },
    { 0x3b9acde8, 61 },
    { 0xbd0acde8, 76 },
    { 0x900acdea, 90 },
    { 0x484acdeb, 33 },
    { 0x5e6acdeb, 90 },
    { 0x7b7acded, 90 },
    { 0xb23acdeb, 43 },
    { 0xb1facdef, 61 },
    { 0xb4eacdef, 33 },
    { 0x436acdf1, 90 },
    { 0x73bacdf2, 76 },
    { 0x640acdf2, 46 },
    { 0x5f6acdf3, 95 },
    { 0xb18acdf0, 98 },
    { 0xaf4acdf6, 99 },
    { 0x842acdf7, 95 },
    { 0x783acde8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cacdfb, 55 },
    { 0xc35acdfc, 61 },
    { 0x66acdfc, 23 },
    { 0, 0 },
    { 0xba3acdff, 33 },
    { 0x9faace00, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xbface03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baace06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378ace10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ace15, 98 },
    { 0x764ace16, 75 },
    { 0xa8cace17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ace1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bace22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939ace28, 46 },
    { 0xa4aace28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x852ace2c, 23 },
    { 0x693ace2c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60ace31, 23 },
    { 0xbd5ace31, 78 },
    { 0, 0 },
    { 0xe77ace34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ace38, 95 },
    { 0x4cdace38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccace3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bace44, 42 },
    { 0x465ace45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0ace4a, 38 },
    { 0, 0 },
    { 0xb27ace4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dace52, 23 },
    { 0, 0 },
    { 0xff6ace54, 90 },
    { 0x8c1ace54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cace58, 23 },
    { 0x937ace58, 34 },
    { 0, 0 },
    { 0x9baace5b, 95 },
    { 0xda3ace5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aace60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaace65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23ace68, 23 },
    { 0x168ace68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153ace6e, 91 },
    { 0, 0 },
    { 0x2dcace70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeface73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaace77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844ace7b, 83 },
    { 0xe73ace7b, 34 },
    { 0x6edace7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59face80, 55 },
    { 0x4f7ace81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aace8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80ace8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cace91, 61 },
    { 0x520ace92, 75 },
    { 0x67face93, 100 },
    { 0x13ace93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576ace9a, 33 },
    { 0xa99ace9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5ace9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bacea6, 34 },
    { 0, 0 },
    { 0xa55acea8, 98 },
    { 0, 0 },
    { 0xdf2aceaa, 55 },
    { 0xa56aceab, 61 },
    { 0, 0 },
    { 0xbb4acead, 90 },
    { 0x483aceae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434aceb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116aceba, 23 },
    { 0x3ccacebb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0acebf, 78 },
    { 0x219acec0, 17 },
    { 0x6eeacec0, 90 },
    { 0x973acec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bacec6, 34 },
    { 0, 0 },
    { 0x7d0acec8, 67 },
    { 0xaecacec9, 23 },
    { 0x23cacec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315aced2, 68 },
    { 0xd44aced3, 46 },
    { 0, 0 },
    { 0xfaaced5, 85 },
    { 0xe11aced6, 94 },
    { 0, 0 },
    { 0x665aced8, 87 },
    { 0x3c1aced8, 23 },
    { 0xc89aced8, 98 },
    { 0x86eacedb, 88 },
    { 0x191acedc, 55 },
    { 0xb28acedd, 90 },
    { 0x647aceda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x546acee1, 75 },
    { 0, 0 },
    { 0x6f6acee3, 87 },
    { 0xa80acee4, 33 },
    { 0x2d8acee5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cacee9, 33 },
    { 0x384aceea, 95 },
    { 0x9cfaceea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670acef1, 95 },
    { 0xea1acef2, 76 },
    { 0xe81acef3, 95 },
    { 0, 0 },
    { 0x303acef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9acef8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92acf0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5acf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bacf19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44acf1c, 90 },
    { 0x844acf1c, 43 },
    { 0, 0 },
    { 0x9d8acf1f, 93 },
    { 0, 0 },
    { 0x528acf21, 84 },
    { 0x574acf21, 100 },
    { 0, 0 },
    { 0x46bacf24, 93 },
    { 0x946acf25, 23 },
    { 0xbdcacf26, 90 },
    { 0x773acf27, 17 },
    { 0x2a2acf27, 52 },
    { 0xbabacf29, 75 },
    { 0xa89acf2a, 61 },
    { 0, 0 },
    { 0x212acf2c, 66 },
    { 0xab1acf2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576acf31, 100 },
    { 0x441acf32, 93 },
    { 0xd0bacf33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29facf36, 34 },
    { 0x71eacf36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cacf3b, 66 },
    { 0x465acf3c, 98 },
    { 0xa72acf3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2acf45, 46 },
    { 0x452acf45, 87 },
    { 0xec4acf47, 33 },
    { 0, 0 },
    { 0xd96acf49, 33 },
    { 0xe17acf49, 23 },
    { 0xcb6acf4b, 100 },
    { 0, 0 },
    { 0xe0facf4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4acf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5acf56, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35acf5a, 76 },
    { 0, 0 },
    { 0xc95acf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7acf5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379acf67, 43 },
    { 0, 0 },
    { 0x480acf69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52acf6c, 88 },
    { 0x440acf6d, 34 },
    { 0xdb0acf6e, 87 },
    { 0x491acf6e, 34 },
    { 0xe1dacf6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984acf79, 23 },
    { 0x2a9acf7a, 99 },
    { 0, 0 },
    { 0xd3facf7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eacf80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bacf83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51acf88, 33 },
    { 0, 0 },
    { 0xa44acf8a, 61 },
    { 0, 0 },
    { 0x6e2acf8c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595acf92, 33 },
    { 0x73cacf93, 68 },
    { 0xac0acf94, 33 },
    { 0, 0 },
    { 0x81eacf96, 90 },
    { 0x30acf97, 93 },
    { 0x126acf97, 52 },
    { 0x2d7acf96, 34 },
    { 0, 0 },
    { 0xf33acf9b, 33 },
    { 0, 0 },
    { 0x21aacf9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5acfa1, 98 },
    { 0xd5cacfa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17acfa5, 61 },
    { 0x476acfa6, 90 },
    { 0xde4acfa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9facfaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292acfb1, 90 },
    { 0, 0 },
    { 0xdd0acfb3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68acfba, 46 },
    { 0x6d0acfbb, 77 },
    { 0xe4acfbb, 61 },
    { 0xe2eacfbd, 43 },
    { 0x72facfbc, 100 },
    { 0x77bacfba, 34 },
    { 0xdfacfc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3acfc3, 91 },
    { 0xf5aacfc3, 76 },
    { 0x965acfc5, 23 },
    { 0xc6bacfc5, 33 },
    { 0x7cacfc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beacfca, 23 },
    { 0, 0 },
    { 0x3aacfcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9acfd4, 33 },
    { 0xe02acfd4, 43 },
    { 0x53bacfd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cacfda, 90 },
    { 0x543acfda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5acfe0, 95 },
    { 0x7c8acfe1, 61 },
    { 0x65aacfe1, 23 },
    { 0, 0 },
    { 0xd2dacfe4, 95 },
    { 0x3d6acfe5, 61 },
    { 0x7acacfe6, 95 },
    { 0x1a7acfe7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x589acfea, 94 },
    { 0, 0 },
    { 0x1abacfec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbacfef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bacff3, 23 },
    { 0, 0 },
    { 0xce0acff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacacffb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ad005, 34 },
    { 0x9f7ad006, 98 },
    { 0x8e9ad006, 33 },
    { 0, 0 },
    { 0xf7cad009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9ad012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ead017, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ad01d, 23 },
    { 0xeeead01d, 75 },
    { 0x16fad01e, 85 },
    { 0, 0 },
    { 0xbc7ad021, 85 },
    { 0xe7fad022, 55 },
    { 0x41aad021, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ead026, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0ad02a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30ad02d, 66 },
    { 0x1ecad02d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aad031, 34 },
    { 0xb4ead032, 80 },
    { 0xe44ad033, 90 },
    { 0xad1ad031, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ad039, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ad043, 34 },
    { 0, 0 },
    { 0xc85ad045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450ad049, 76 },
    { 0xbaad049, 98 },
    { 0xa2fad04b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7ad04e, 66 },
    { 0xfbaad04e, 93 },
    { 0, 0 },
    { 0xe40ad051, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbad059, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedad05c, 76 },
    { 0, 0 },
    { 0x5cfad05e, 61 },
    { 0, 0 },
    { 0x5b2ad060, 87 },
    { 0, 0 },
    { 0x5b5ad062, 99 },
    { 0x702ad063, 95 },
    { 0x13fad064, 77 },
    { 0, 0 },
    { 0xaebad066, 43 },
    { 0, 0 },
    { 0x2ddad068, 33 },
    { 0x816ad069, 91 },
    { 0x57ead06a, 76 },
    { 0x8a5ad068, 100 },
    { 0, 0 },
    { 0x1b8ad06d, 33 },
    { 0x35cad06e, 61 },
    { 0x7eaad06f, 85 },
    { 0x846ad06f, 75 },
    { 0xadad071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ad075, 33 },
    { 0x512ad076, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ead083, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ad089, 34 },
    { 0, 0 },
    { 0x9cead08b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ad08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303ad094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ead09b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x351ad09e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cead0a5, 90 },
    { 0, 0 },
    { 0xabcad0a7, 81 },
    { 0xbe9ad0a8, 90 },
    { 0x66cad0a9, 33 },
    { 0x9eead0aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6ad0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad0b8, 23 },
    { 0x2e9ad0b8, 66 },
    { 0xcfad0ba, 61 },
    { 0xa91ad0b8, 80 },
    { 0, 0 },
    { 0xebfad0bd, 23 },
    { 0x7c1ad0be, 33 },
    { 0, 0 },
    { 0xfd8ad0c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3ad0c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ad0c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dad0cc, 80 },
    { 0xcc3ad0cd, 46 },
    { 0, 0 },
    { 0xa95ad0cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ad0d7, 100 },
    { 0x894ad0d8, 84 },
    { 0x75fad0d7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfad0de, 61 },
    { 0x5f6ad0de, 33 },
    { 0x86aad0de, 90 },
    { 0, 0 },
    { 0x292ad0e2, 33 },
    { 0x207ad0e3, 95 },
    { 0x2f9ad0e4, 93 },
    { 0x71cad0e5, 76 },
    { 0xba8ad0e6, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612ad0ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caad0ee, 78 },
    { 0, 0 },
    { 0xc15ad0f0, 99 },
    { 0x8baad0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdad0f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37ad0f7, 76 },
    { 0, 0 },
    { 0x480ad0f9, 100 },
    { 0x878ad0fa, 76 },
    { 0x912ad0f9, 67 },
    { 0, 0 },
    { 0xa08ad0fd, 52 },
    { 0, 0 },
    { 0x801ad0ff, 33 },
    { 0x2ebad100, 66 },
    { 0, 0 },
    { 0xcbad102, 69 },
    { 0, 0 },
    { 0x72dad104, 61 },
    { 0x4baad104, 84 },
    { 0xd9ead106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abad109, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339ad110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x224ad113, 61 },
    { 0xda7ad114, 76 },
    { 0xbebad114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x748ad118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8ad11f, 98 },
    { 0x41dad120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918ad127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95ad12c, 95 },
    { 0, 0 },
    { 0xe0aad12e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ad133, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ead139, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fad13d, 90 },
    { 0x2b6ad13d, 98 },
    { 0x191ad13f, 33 },
    { 0xa62ad140, 65 },
    { 0xa6cad140, 61 },
    { 0xed0ad142, 78 },
    { 0x631ad13f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9ad148, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ad14c, 98 },
    { 0xeb5ad14d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21ad157, 95 },
    { 0x7dbad157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0ad15c, 90 },
    { 0xa19ad15d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604ad161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ad165, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ead168, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cad16e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x599ad171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6ad179, 59 },
    { 0x794ad17a, 90 },
    { 0x499ad17a, 90 },
    { 0x653ad179, 98 },
    { 0x7b0ad17d, 23 },
    { 0xeacad17d, 75 },
    { 0xd63ad17f, 33 },
    { 0x2e0ad17e, 76 },
    { 0xa18ad17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ad184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0ad187, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cad18a, 87 },
    { 0xabfad18a, 33 },
    { 0x9d8ad18c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92ad194, 95 },
    { 0xc32ad195, 23 },
    { 0, 0 },
    { 0x367ad197, 61 },
    { 0xe8cad198, 84 },
    { 0xb72ad197, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aad19e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ad1a4, 90 },
    { 0x2a6ad1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dad1aa, 33 },
    { 0xf99ad1aa, 90 },
    { 0, 0 },
    { 0x322ad1ad, 98 },
    { 0x843ad1ad, 65 },
    { 0x85ead1af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aad1b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7ad1b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21ad1bf, 98 },
    { 0, 0 },
    { 0x8b6ad1c1, 33 },
    { 0, 0 },
    { 0xffad1c3, 90 },
    { 0, 0 },
    { 0xf10ad1c5, 96 },
    { 0x3d5ad1c6, 90 },
    { 0x22cad1c7, 43 },
    { 0xb4fad1c8, 61 },
    { 0x954ad1c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0ad1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x132ad1d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586ad1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efad1e1, 98 },
    { 0xadaad1e1, 61 },
    { 0xfa9ad1e3, 90 },
    { 0, 0 },
    { 0xda2ad1e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5ad1e8, 95 },
    { 0xe2cad1e8, 33 },
    { 0, 0 },
    { 0x3b0ad1eb, 76 },
    { 0xb2dad1eb, 55 },
    { 0x8f4ad1ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522ad1f2, 33 },
    { 0x258ad1f3, 98 },
    { 0, 0 },
    { 0x226ad1f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ead1f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194ad1fc, 95 },
    { 0x3bead1fd, 34 },
    { 0xd72ad1fd, 61 },
    { 0xe6fad1fe, 61 },
    { 0, 0 },
    { 0x4a8ad201, 91 },
    { 0x49cad202, 34 },
    { 0x20ead202, 98 },
    { 0, 0 },
    { 0x4adad205, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ad20a, 34 },
    { 0x544ad20b, 33 },
    { 0, 0 },
    { 0xebcad20d, 33 },
    { 0xbc9ad20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ad212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eead215, 33 },
    { 0x7f3ad215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3ad21f, 85 },
    { 0x273ad220, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dead227, 90 },
    { 0x1afad227, 34 },
    { 0, 0 },
    { 0xca0ad22a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bad22d, 17 },
    { 0, 0 },
    { 0x5e7ad22f, 23 },
    { 0, 0 },
    { 0x434ad231, 76 },
    { 0, 0 },
    { 0xbf4ad233, 69 },
    { 0x97bad233, 46 },
    { 0, 0 },
    { 0xf44ad236, 55 },
    { 0, 0 },
    { 0x3a7ad238, 90 },
    { 0, 0 },
    { 0x46ead23a, 90 },
    { 0x395ad23b, 87 },
    { 0xb55ad23c, 98 },
    { 0xf86ad23c, 76 },
    { 0xf30ad23e, 90 },
    { 0x3bbad23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6ad242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5ad24c, 90 },
    { 0, 0 },
    { 0x7e1ad24e, 33 },
    { 0xd7dad24f, 95 },
    { 0xdb2ad250, 85 },
    { 0x42cad24f, 90 },
    { 0xbffad252, 95 },
    { 0x722ad252, 95 },
    { 0xec6ad254, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ad257, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bead25f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ead266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3ad26d, 76 },
    { 0, 0 },
    { 0x44ead26f, 61 },
    { 0x6a6ad270, 23 },
    { 0xb8cad271, 34 },
    { 0, 0 },
    { 0xb2ead273, 96 },
    { 0x696ad274, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2ad27a, 46 },
    { 0xbc9ad27b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bad27e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56ad286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ad289, 99 },
    { 0x20bad28a, 76 },
    { 0x54ead28b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8ad28e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x768ad291, 61 },
    { 0x93dad292, 66 },
    { 0, 0 },
    { 0xa86ad294, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aad298, 17 },
    { 0xfa0ad299, 61 },
    { 0x82cad298, 33 },
    { 0x6f3ad29b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfad29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbead2a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x833ad2a4, 95 },
    { 0, 0 },
    { 0x1cdad2a6, 33 },
    { 0x771ad2a6, 93 },
    { 0xd4ead2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad2ae, 84 },
    { 0x8e0ad2ae, 100 },
    { 0x228ad2b0, 93 },
    { 0, 0 },
    { 0x9c0ad2b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad2b5, 100 },
    { 0, 0 },
    { 0x141ad2b7, 90 },
    { 0xc24ad2b7, 46 },
    { 0, 0 },
    { 0x11ad2ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dad2c0, 23 },
    { 0, 0 },
    { 0x380ad2c2, 43 },
    { 0xe40ad2c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112ad2c7, 17 },
    { 0xf3bad2c7, 43 },
    { 0, 0 },
    { 0x760ad2ca, 95 },
    { 0xac4ad2cb, 90 },
    { 0x636ad2ca, 95 },
    { 0x33aad2cd, 99 },
    { 0x7ddad2ce, 98 },
    { 0, 0 },
    { 0x872ad2d0, 76 },
    { 0, 0 },
    { 0xea1ad2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ad2d6, 17 },
    { 0x436ad2d7, 42 },
    { 0, 0 },
    { 0x7d0ad2d9, 95 },
    { 0, 0 },
    { 0x66fad2db, 81 },
    { 0x51bad2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adad2e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ad2e3, 61 },
    { 0xb1ead2e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fad2e7, 90 },
    { 0xc93ad2e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6ad2eb, 23 },
    { 0xe5fad2ec, 33 },
    { 0x5c8ad2ed, 23 },
    { 0x129ad2ee, 43 },
    { 0x9c0ad2ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ad2f2, 34 },
    { 0x5f6ad2f3, 33 },
    { 0x7d3ad2f3, 56 },
    { 0xa32ad2f3, 90 },
    { 0x873ad2f3, 96 },
    { 0x9adad2f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ad2fd, 90 },
    { 0xe29ad2fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447ad30a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ad30f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ad313, 81 },
    { 0x1d5ad314, 100 },
    { 0, 0 },
    { 0x753ad316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aad31b, 33 },
    { 0x37ead31c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453ad321, 88 },
    { 0x86cad322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ad331, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ad337, 33 },
    { 0, 0 },
    { 0xfe6ad339, 23 },
    { 0x56ead33a, 33 },
    { 0x2f2ad33b, 99 },
    { 0x3e9ad33a, 87 },
    { 0xe2dad33b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bad340, 65 },
    { 0xe2ead340, 61 },
    { 0x9b7ad342, 33 },
    { 0xe72ad342, 34 },
    { 0x841ad344, 98 },
    { 0x9fcad345, 98 },
    { 0x15ead341, 38 },
    { 0, 0 },
    { 0xaf8ad348, 98 },
    { 0, 0 },
    { 0xcead34a, 76 },
    { 0x94dad34a, 23 },
    { 0, 0 },
    { 0x6cbad34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932ad352, 61 },
    { 0, 0 },
    { 0x7d8ad354, 90 },
    { 0, 0 },
    { 0xccbad356, 90 },
    { 0x48ad357, 61 },
    { 0, 0 },
    { 0xcf4ad359, 61 },
    { 0x820ad35a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aad35d, 33 },
    { 0xd1dad35e, 46 },
    { 0x2d8ad35f, 90 },
    { 0x4dcad360, 52 },
    { 0, 0 },
    { 0x8f3ad362, 76 },
    { 0, 0 },
    { 0x3cbad364, 33 },
    { 0x3efad364, 61 },
    { 0, 0 },
    { 0x2ad367, 90 },
    { 0xafead367, 66 },
    { 0x6abad369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ad36c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ad370, 61 },
    { 0x71ad370, 23 },
    { 0xeffad372, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02ad375, 90 },
    { 0, 0 },
    { 0xc11ad377, 28 },
    { 0, 0 },
    { 0x413ad379, 33 },
    { 0x35fad37a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ad37d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8ad380, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cad386, 23 },
    { 0xd12ad386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ad38b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bad38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862ad398, 61 },
    { 0xb8ad398, 99 },
    { 0, 0 },
    { 0x42aad39b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fad3a0, 33 },
    { 0, 0 },
    { 0x4e9ad3a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aad3a6, 93 },
    { 0x794ad3a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ad3ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212ad3b2, 99 },
    { 0xaccad3b3, 90 },
    { 0, 0 },
    { 0xc3fad3b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174ad3c0, 90 },
    { 0, 0 },
    { 0xb5dad3c2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ad3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58ad3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cad3d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad3db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9ad3e3, 46 },
    { 0, 0 },
    { 0xb45ad3e5, 99 },
    { 0xeb2ad3e5, 33 },
    { 0, 0 },
    { 0x900ad3e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ad3ef, 98 },
    { 0x2c9ad3ef, 98 },
    { 0x304ad3f1, 55 },
    { 0, 0 },
    { 0xd5cad3f3, 90 },
    { 0, 0 },
    { 0xa90ad3f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x632ad3f8, 17 },
    { 0x556ad3f9, 66 },
    { 0xf8dad3fa, 33 },
    { 0x53dad3f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69ad3fe, 66 },
    { 0xa3ead3fe, 61 },
    { 0, 0 },
    { 0xa81ad401, 23 },
    { 0, 0 },
    { 0x1d1ad403, 33 },
    { 0xc6cad404, 61 },
    { 0x5c4ad403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721ad409, 95 },
    { 0xe21ad40a, 76 },
    { 0x749ad40a, 95 },
    { 0x946ad40c, 55 },
    { 0, 0 },
    { 0x798ad40e, 34 },
    { 0xc58ad40f, 90 },
    { 0x805ad40f, 34 },
    { 0xc04ad40f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556ad41a, 23 },
    { 0x1a1ad41b, 17 },
    { 0, 0 },
    { 0x315ad41d, 95 },
    { 0xc9aad41d, 95 },
    { 0xa5bad41d, 93 },
    { 0xb12ad41d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67ad423, 90 },
    { 0x374ad423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383ad42f, 46 },
    { 0, 0 },
    { 0x8edad431, 90 },
    { 0xad3ad431, 23 },
    { 0x781ad433, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baad439, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19ad43d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efad440, 23 },
    { 0xca3ad440, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720ad445, 75 },
    { 0x7ffad446, 47 },
    { 0x1c4ad446, 95 },
    { 0, 0 },
    { 0x560ad449, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798ad450, 90 },
    { 0x112ad451, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2ad457, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ad45a, 61 },
    { 0x844ad45b, 99 },
    { 0, 0 },
    { 0xfd2ad45d, 90 },
    { 0xb31ad45e, 34 },
    { 0x834ad45f, 67 },
    { 0x6e1ad460, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aead464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcad46c, 23 },
    { 0x31ad46d, 95 },
    { 0x65cad46e, 46 },
    { 0x31aad46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dad473, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ad486, 90 },
    { 0xa66ad487, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7ad48c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ad48f, 34 },
    { 0x68ad490, 98 },
    { 0x8b9ad491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ad496, 38 },
    { 0xf89ad497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x841ad49a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fad49e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267ad4a7, 90 },
    { 0xa29ad4a8, 93 },
    { 0x208ad4a8, 97 },
    { 0x49dad4a9, 33 },
    { 0xce2ad4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aad4b3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388ad4b9, 85 },
    { 0x926ad4ba, 61 },
    { 0x442ad4ba, 61 },
    { 0x4aad4bc, 87 },
    { 0x946ad4bd, 85 },
    { 0xc98ad4be, 40 },
    { 0x1c2ad4bd, 43 },
    { 0xd1bad4c0, 98 },
    { 0, 0 },
    { 0x68cad4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdad4c7, 95 },
    { 0, 0 },
    { 0x189ad4c9, 76 },
    { 0x216ad4c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ad4cd, 55 },
    { 0x9c0ad4ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378ad4d2, 76 },
    { 0xffcad4d3, 95 },
    { 0, 0 },
    { 0xf62ad4d5, 33 },
    { 0x561ad4d5, 33 },
    { 0x1abad4d7, 33 },
    { 0x9fead4d8, 78 },
    { 0xa0fad4d5, 33 },
    { 0x581ad4d9, 46 },
    { 0x237ad4db, 95 },
    { 0, 0 },
    { 0x370ad4dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fad4e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831ad4e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bad4ed, 100 },
    { 0xfd8ad4ee, 67 },
    { 0xecfad4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efad4f2, 90 },
    { 0, 0 },
    { 0x50dad4f4, 65 },
    { 0, 0 },
    { 0x941ad4f6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ad4f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dad4fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ad4ff, 75 },
    { 0, 0 },
    { 0xef1ad501, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad509, 33 },
    { 0x80ead50a, 77 },
    { 0x916ad50a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dad512, 90 },
    { 0xf79ad513, 55 },
    { 0x18fad513, 61 },
    { 0x344ad513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40ad518, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6ad523, 90 },
    { 0xff8ad524, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984ad52a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ad52e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39ad531, 88 },
    { 0x7b1ad532, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562ad539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7ad540, 90 },
    { 0xc5bad540, 61 },
    { 0, 0 },
    { 0x32bad543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ead546, 91 },
    { 0xae5ad546, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aad54c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfad54f, 100 },
    { 0xc66ad550, 99 },
    { 0xf42ad54f, 95 },
    { 0x70fad552, 90 },
    { 0x8c5ad553, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafad55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cad560, 33 },
    { 0x4a0ad561, 38 },
    { 0x21bad561, 33 },
    { 0, 0 },
    { 0x530ad564, 55 },
    { 0, 0 },
    { 0x43aad566, 46 },
    { 0, 0 },
    { 0x4b6ad568, 33 },
    { 0x5adad568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x385ad56c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4ad570, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ad578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cad580, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36ad585, 33 },
    { 0x5b3ad585, 90 },
    { 0, 0 },
    { 0xc0fad588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ad58e, 90 },
    { 0x914ad58f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad593, 90 },
    { 0xd3ead594, 43 },
    { 0x337ad595, 56 },
    { 0x2f4ad593, 61 },
    { 0, 0 },
    { 0x402ad598, 52 },
    { 0x94ad598, 90 },
    { 0, 0 },
    { 0xe39ad59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ad5a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fad5ad, 33 },
    { 0, 0 },
    { 0xca0ad5af, 61 },
    { 0xb43ad5b0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694ad5b6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x813ad5b9, 23 },
    { 0, 0 },
    { 0x312ad5bb, 55 },
    { 0x104ad5bc, 95 },
    { 0x9ad5bd, 90 },
    { 0x72dad5be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ad5c7, 100 },
    { 0x4bfad5c7, 90 },
    { 0, 0 },
    { 0x115ad5ca, 19 },
    { 0xcd6ad5ca, 69 },
    { 0, 0 },
    { 0x9b5ad5cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad5d0, 95 },
    { 0x2c8ad5d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dad5d4, 55 },
    { 0, 0 },
    { 0x1baad5d6, 90 },
    { 0xaaaad5d7, 33 },
    { 0x4caad5d6, 95 },
    { 0xeb6ad5d7, 12 },
    { 0, 0 },
    { 0x4f5ad5db, 88 },
    { 0x4c9ad5dc, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80ad5e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9faad5ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x711ad5ef, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad5f5, 90 },
    { 0x777ad5f6, 76 },
    { 0xb22ad5f6, 61 },
    { 0xc7dad5f8, 34 },
    { 0xfb4ad5f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x544ad5fc, 78 },
    { 0x58ead5fc, 52 },
    { 0x8a5ad5fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbad605, 34 },
    { 0, 0 },
    { 0xec8ad607, 78 },
    { 0, 0 },
    { 0x247ad609, 33 },
    { 0xf4bad60a, 61 },
    { 0x8d3ad60b, 61 },
    { 0x803ad60b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ad612, 61 },
    { 0xd47ad612, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baad616, 23 },
    { 0, 0 },
    { 0xc5ad618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ad61b, 59 },
    { 0, 0 },
    { 0x4e4ad61d, 43 },
    { 0, 0 },
    { 0x72aad61f, 100 },
    { 0xacdad620, 64 },
    { 0xb9cad621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddad628, 100 },
    { 0xeadad629, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aad62d, 23 },
    { 0x9f6ad62d, 95 },
    { 0x686ad62f, 94 },
    { 0xffcad62f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ad636, 90 },
    { 0xa05ad637, 90 },
    { 0x4f5ad638, 98 },
    { 0xe90ad639, 77 },
    { 0, 0 },
    { 0xa75ad63b, 98 },
    { 0x8f7ad63c, 90 },
    { 0, 0 },
    { 0x687ad63e, 66 },
    { 0x667ad63e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7ad647, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ad64b, 76 },
    { 0xd7aad64b, 55 },
    { 0, 0 },
    { 0xbfbad64e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682ad658, 23 },
    { 0xd9bad658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8ad65f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ad663, 55 },
    { 0x7fad664, 85 },
    { 0x54fad663, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ad66b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ad66e, 90 },
    { 0, 0 },
    { 0x4afad670, 90 },
    { 0, 0 },
    { 0x63cad672, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfad675, 52 },
    { 0, 0 },
    { 0xe20ad677, 55 },
    { 0, 0 },
    { 0x2f9ad679, 43 },
    { 0x38ead67a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaad67d, 90 },
    { 0, 0 },
    { 0xbf7ad67f, 85 },
    { 0xa11ad680, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1ad685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5ad68b, 98 },
    { 0x7e5ad68c, 90 },
    { 0, 0 },
    { 0x1fdad68e, 76 },
    { 0xe60ad68f, 61 },
    { 0x1f6ad68e, 61 },
    { 0x68cad690, 88 },
    { 0, 0 },
    { 0x2f9ad693, 34 },
    { 0xa9dad694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bad698, 90 },
    { 0xfb1ad698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6ad69f, 98 },
    { 0, 0 },
    { 0x43ead6a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ad6a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5ad6a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95ad6af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dad6b2, 95 },
    { 0x4d6ad6b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ad6b6, 33 },
    { 0, 0 },
    { 0x843ad6b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35ad6c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddad6cd, 90 },
    { 0, 0 },
    { 0x70cad6cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbad6d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ad6d9, 95 },
    { 0x319ad6d9, 95 },
    { 0, 0 },
    { 0xdedad6dc, 93 },
    { 0x296ad6dd, 90 },
    { 0xceaad6dd, 33 },
    { 0x6b2ad6dc, 33 },
    { 0xb37ad6e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adad6e4, 78 },
    { 0, 0 },
    { 0xa48ad6e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237ad6ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ad6ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dad6f4, 76 },
    { 0, 0 },
    { 0x704ad6f6, 90 },
    { 0xeb5ad6f7, 87 },
    { 0xaddad6f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ad6fc, 98 },
    { 0x45aad6fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6ad702, 33 },
    { 0, 0 },
    { 0x7b8ad704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ad709, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dad70c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad710, 95 },
    { 0xf9ad711, 33 },
    { 0xfeead710, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x64aad715, 33 },
    { 0, 0 },
    { 0xaddad717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fead71a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ad71e, 76 },
    { 0, 0 },
    { 0x8aaad720, 23 },
    { 0, 0 },
    { 0x22cad722, 93 },
    { 0x7d9ad722, 33 },
    { 0x88dad724, 87 },
    { 0, 0 },
    { 0xacad726, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148ad72b, 84 },
    { 0x8c2ad72c, 81 },
    { 0xfe7ad72d, 49 },
    { 0, 0 },
    { 0x507ad72f, 23 },
    { 0xf83ad730, 78 },
    { 0xdc0ad731, 99 },
    { 0x5c5ad730, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57ad736, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ad73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ead73f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6ad743, 93 },
    { 0, 0 },
    { 0xb1aad745, 85 },
    { 0x132ad745, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aad749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ad74c, 23 },
    { 0x260ad74c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbad750, 98 },
    { 0x1d7ad750, 55 },
    { 0x73cad750, 87 },
    { 0xfebad751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90ad757, 84 },
    { 0x71cad758, 77 },
    { 0xc0ad759, 23 },
    { 0, 0 },
    { 0xa3ead75b, 81 },
    { 0x8baad75b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dad75f, 61 },
    { 0x7b9ad760, 76 },
    { 0xcbad760, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cad765, 23 },
    { 0, 0 },
    { 0x25ead767, 98 },
    { 0xe17ad768, 33 },
    { 0, 0 },
    { 0x45ead76a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227ad76e, 81 },
    { 0, 0 },
    { 0x298ad770, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ad774, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dad778, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7ad77c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ad781, 90 },
    { 0x143ad782, 61 },
    { 0xe21ad783, 86 },
    { 0x921ad783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afad788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fad791, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x772ad794, 91 },
    { 0x7f5ad794, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acad798, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad79c, 33 },
    { 0, 0 },
    { 0xef3ad79e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ad7a2, 90 },
    { 0x741ad7a3, 61 },
    { 0x30fad7a4, 88 },
    { 0xd4ad7a5, 90 },
    { 0xacbad7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ad7ae, 95 },
    { 0, 0 },
    { 0xbaead7b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ad7b6, 61 },
    { 0x1f3ad7b6, 34 },
    { 0, 0 },
    { 0x8e7ad7b9, 90 },
    { 0, 0 },
    { 0x139ad7bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ad7be, 95 },
    { 0x7d7ad7bf, 95 },
    { 0x865ad7be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0ad7c9, 76 },
    { 0x16cad7c9, 66 },
    { 0, 0 },
    { 0x985ad7cc, 98 },
    { 0xc9fad7cc, 33 },
    { 0xc55ad7cc, 33 },
    { 0, 0 },
    { 0xd27ad7d0, 84 },
    { 0xb6bad7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ad7d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cad7db, 90 },
    { 0x8bfad7dc, 23 },
    { 0x4ebad7dd, 23 },
    { 0x513ad7dd, 95 },
    { 0x51fad7dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ad7e2, 76 },
    { 0x733ad7e3, 67 },
    { 0x6e5ad7e3, 17 },
    { 0x6d8ad7e5, 98 },
    { 0, 0 },
    { 0x344ad7e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bad7eb, 23 },
    { 0x3c6ad7ec, 90 },
    { 0x1faad7ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dad7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ad7f4, 46 },
    { 0x247ad7f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9ad7f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56ad7fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1ad801, 98 },
    { 0xb18ad801, 81 },
    { 0, 0 },
    { 0x632ad804, 46 },
    { 0, 0 },
    { 0xbbdad806, 98 },
    { 0xa7cad807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aad815, 98 },
    { 0, 0 },
    { 0xaf8ad817, 76 },
    { 0x4a1ad818, 33 },
    { 0, 0 },
    { 0xc7dad81a, 33 },
    { 0xce5ad81b, 80 },
    { 0x7efad81c, 46 },
    { 0x50ad81d, 98 },
    { 0x51cad81a, 55 },
    { 0x5e3ad81b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bad823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x145ad826, 95 },
    { 0x902ad827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ad82d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad830, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ad833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cad836, 33 },
    { 0x18ad837, 33 },
    { 0x8f2ad838, 95 },
    { 0, 0 },
    { 0x367ad83a, 98 },
    { 0x36dad83b, 76 },
    { 0x4b5ad83b, 34 },
    { 0xe8ead83d, 23 },
    { 0xa90ad83e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ad841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ad845, 61 },
    { 0xd2cad846, 95 },
    { 0xa07ad847, 33 },
    { 0xe22ad848, 98 },
    { 0x712ad848, 23 },
    { 0, 0 },
    { 0x220ad84b, 90 },
    { 0, 0 },
    { 0xdc0ad84d, 66 },
    { 0, 0 },
    { 0xc42ad84f, 23 },
    { 0, 0 },
    { 0xdd8ad851, 61 },
    { 0, 0 },
    { 0x450ad853, 61 },
    { 0xb5cad854, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fad857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5ad85b, 33 },
    { 0xde3ad85c, 23 },
    { 0xe1bad85c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128ad863, 55 },
    { 0x495ad864, 33 },
    { 0x179ad864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ad869, 90 },
    { 0x7fad86a, 46 },
    { 0, 0 },
    { 0x41bad86c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ad86f, 23 },
    { 0xc50ad86f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x217ad873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29ad876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffad87c, 33 },
    { 0x9ad87d, 9 },
    { 0x606ad87e, 90 },
    { 0x2cad87f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9ad882, 90 },
    { 0x367ad883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6ad887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ad88a, 33 },
    { 0x64dad88b, 66 },
    { 0xe23ad88c, 69 },
    { 0x7f4ad88d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ad890, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edad895, 98 },
    { 0x21ad896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ead89a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2ad89e, 98 },
    { 0x9f1ad89f, 61 },
    { 0, 0 },
    { 0x5fcad8a1, 61 },
    { 0xa08ad8a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235ad8ab, 90 },
    { 0xb9ead8ab, 23 },
    { 0xdd7ad8ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3ad8b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcad8b4, 98 },
    { 0, 0 },
    { 0xd33ad8b6, 61 },
    { 0x358ad8b7, 87 },
    { 0xb6bad8b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cad8bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7ad8bf, 61 },
    { 0xa45ad8c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38ad8c4, 95 },
    { 0xf88ad8c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5ad8c9, 98 },
    { 0xc06ad8ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cad8cf, 84 },
    { 0x2f7ad8d0, 34 },
    { 0, 0 },
    { 0x32cad8d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bad8d5, 78 },
    { 0, 0 },
    { 0xaaaad8d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0ad8e2, 100 },
    { 0x179ad8e3, 33 },
    { 0x399ad8e4, 23 },
    { 0x35ad8e4, 91 },
    { 0xfb6ad8e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ad8f0, 81 },
    { 0xcad8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbad8f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ad8ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fad905, 90 },
    { 0x45dad906, 33 },
    { 0xbcead907, 46 },
    { 0xb6dad908, 99 },
    { 0x273ad905, 23 },
    { 0, 0 },
    { 0xa7ead90b, 76 },
    { 0xbd1ad90c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacad90f, 76 },
    { 0xfbad910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad914, 61 },
    { 0xfcbad914, 98 },
    { 0xf67ad916, 90 },
    { 0x624ad917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ad91a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cad91e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804ad924, 91 },
    { 0xb28ad924, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aad928, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ad92c, 55 },
    { 0x632ad92c, 91 },
    { 0x8ccad92e, 91 },
    { 0, 0 },
    { 0x9e8ad930, 76 },
    { 0x2bcad930, 61 },
    { 0, 0 },
    { 0xf0dad933, 34 },
    { 0, 0 },
    { 0x7bfad935, 23 },
    { 0, 0 },
    { 0x6acad937, 33 },
    { 0xfedad938, 43 },
    { 0x498ad938, 82 },
    { 0xe72ad93a, 95 },
    { 0x6a3ad937, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ad93e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcad942, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccead949, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71ad951, 61 },
    { 0x4d5ad951, 98 },
    { 0x615ad952, 23 },
    { 0x664ad954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad962, 33 },
    { 0x848ad962, 95 },
    { 0, 0 },
    { 0x273ad965, 17 },
    { 0, 0 },
    { 0xa1ad967, 33 },
    { 0xd03ad967, 90 },
    { 0xcfead968, 94 },
    { 0xd8dad96a, 98 },
    { 0, 0 },
    { 0x637ad96c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fad970, 23 },
    { 0xdb2ad970, 67 },
    { 0x597ad971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146ad97a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ad97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ad982, 33 },
    { 0xc15ad983, 95 },
    { 0xe2ad984, 17 },
    { 0xd4ad984, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad988, 90 },
    { 0xe38ad989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ad98e, 100 },
    { 0xdc2ad98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386ad993, 76 },
    { 0x363ad993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9ad999, 46 },
    { 0, 0 },
    { 0xb75ad99b, 90 },
    { 0, 0 },
    { 0x766ad99d, 23 },
    { 0xb8ad99e, 46 },
    { 0xccad99f, 75 },
    { 0xf94ad99e, 17 },
    { 0xc40ad99d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ad9ab, 38 },
    { 0xd6fad9ab, 55 },
    { 0x6d4ad9ad, 78 },
    { 0x2bead9ae, 23 },
    { 0, 0 },
    { 0x14fad9b0, 43 },
    { 0xbdead9b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdad9b7, 33 },
    { 0, 0 },
    { 0xeadad9b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ad9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daad9c4, 76 },
    { 0xa09ad9c4, 61 },
    { 0, 0 },
    { 0x22bad9c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ad9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6ad9ce, 23 },
    { 0x4e1ad9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ad9d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dad9d5, 23 },
    { 0, 0 },
    { 0xe58ad9d7, 55 },
    { 0xb44ad9d8, 98 },
    { 0xb92ad9d9, 95 },
    { 0x8dfad9da, 61 },
    { 0x213ad9db, 85 },
    { 0, 0 },
    { 0x8bcad9dd, 34 },
    { 0xd52ad9de, 93 },
    { 0x9f6ad9de, 52 },
    { 0x55ad9de, 33 },
    { 0, 0 },
    { 0x5b1ad9e2, 90 },
    { 0, 0 },
    { 0x503ad9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0ad9f0, 95 },
    { 0x290ad9f1, 23 },
    { 0, 0 },
    { 0xe81ad9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fad9f7, 95 },
    { 0, 0 },
    { 0x604ad9f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0ad9fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bada00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ada04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efada0a, 23 },
    { 0, 0 },
    { 0x8c6ada0c, 98 },
    { 0, 0 },
    { 0xeaada0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18ada11, 90 },
    { 0xbc8ada11, 38 },
    { 0x700ada12, 43 },
    { 0, 0 },
    { 0xc27ada15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523ada1d, 90 },
    { 0xeb7ada1e, 55 },
    { 0xf28ada1d, 90 },
    { 0xdd7ada1e, 33 },
    { 0xf89ada20, 23 },
    { 0x1c8ada21, 95 },
    { 0x22bada1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6ada26, 88 },
    { 0x52cada27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06ada2b, 33 },
    { 0, 0 },
    { 0x793ada2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9aada30, 68 },
    { 0x66ada31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581ada36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bada3a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x973ada3d, 90 },
    { 0xaa3ada3d, 75 },
    { 0xa23ada3f, 88 },
    { 0x46dada40, 90 },
    { 0, 0 },
    { 0x73dada42, 46 },
    { 0xfabada43, 23 },
    { 0x4d0ada44, 23 },
    { 0x4a6ada45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75ada4c, 84 },
    { 0, 0 },
    { 0xfd5ada4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x28aada51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646ada58, 33 },
    { 0, 0 },
    { 0x273ada5a, 43 },
    { 0x424ada5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aada5e, 95 },
    { 0, 0 },
    { 0xba2ada60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cada6a, 53 },
    { 0xaffada6b, 90 },
    { 0xdb1ada6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aada6f, 23 },
    { 0, 0 },
    { 0x7ffada71, 23 },
    { 0, 0 },
    { 0xe1bada73, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fada76, 90 },
    { 0x1c4ada77, 33 },
    { 0xa9ada77, 33 },
    { 0, 0 },
    { 0xac0ada7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ada7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bada81, 98 },
    { 0x37aada81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ada86, 43 },
    { 0, 0 },
    { 0xb1dada88, 23 },
    { 0, 0 },
    { 0xf3fada8a, 76 },
    { 0, 0 },
    { 0xb63ada8c, 43 },
    { 0xd98ada8d, 61 },
    { 0, 0 },
    { 0x406ada8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381ada93, 43 },
    { 0, 0 },
    { 0xb45ada95, 61 },
    { 0x9c2ada96, 90 },
    { 0xbdfada96, 66 },
    { 0x2b1ada98, 23 },
    { 0x8cdada99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfada9c, 90 },
    { 0x14fada9c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00adaa2, 90 },
    { 0x375adaa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51adaa6, 61 },
    { 0xe5dadaa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0adaae, 95 },
    { 0, 0 },
    { 0xdbbadab0, 33 },
    { 0, 0 },
    { 0xcb6adab2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eadab6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9adabb, 82 },
    { 0xcd2adabc, 95 },
    { 0xe79adabc, 33 },
    { 0x187adabd, 95 },
    { 0, 0 },
    { 0x617adac0, 95 },
    { 0, 0 },
    { 0x8d5adac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adadacb, 23 },
    { 0xe22adacc, 90 },
    { 0x2badacd, 61 },
    { 0x647adace, 91 },
    { 0x9ddadacf, 99 },
    { 0x56badace, 43 },
    { 0x8fadacd, 61 },
    { 0x3d9adad2, 61 },
    { 0, 0 },
    { 0x20cadad4, 61 },
    { 0, 0 },
    { 0x84fadad6, 98 },
    { 0x2aadad7, 98 },
    { 0xa94adad7, 56 },
    { 0, 0 },
    { 0x1d1adada, 95 },
    { 0x422adadb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109adae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66adae3, 81 },
    { 0xe88adae4, 90 },
    { 0xbdbadae5, 98 },
    { 0x3c3adae5, 98 },
    { 0x26adae6, 90 },
    { 0xac2adae8, 61 },
    { 0xd99adae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dadaef, 95 },
    { 0, 0 },
    { 0xdcbadaf1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94adaf4, 88 },
    { 0xc73adaf5, 87 },
    { 0x130adaf5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2adafa, 43 },
    { 0xf09adafb, 23 },
    { 0xcacadafc, 67 },
    { 0x8ecadafd, 23 },
    { 0x307adafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baadb02, 84 },
    { 0xc70adb03, 61 },
    { 0x212adb04, 66 },
    { 0, 0 },
    { 0x588adb06, 78 },
    { 0x29cadb07, 91 },
    { 0xb9cadb08, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aadb12, 78 },
    { 0, 0 },
    { 0xbc2adb14, 38 },
    { 0, 0 },
    { 0x43fadb16, 10 },
    { 0, 0 },
    { 0x26adb18, 90 },
    { 0x5eadb18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeadb20, 33 },
    { 0xd51adb21, 76 },
    { 0xa08adb22, 98 },
    { 0xb81adb23, 65 },
    { 0xcc3adb23, 66 },
    { 0x2f1adb23, 90 },
    { 0, 0 },
    { 0x17eadb27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x178adb2a, 61 },
    { 0xbe7adb2b, 90 },
    { 0xd38adb2b, 33 },
    { 0, 0 },
    { 0x5c1adb2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dadb32, 77 },
    { 0x24cadb33, 87 },
    { 0x4efadb33, 85 },
    { 0x1a7adb32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x679adb38, 34 },
    { 0x38badb39, 61 },
    { 0x587adb38, 61 },
    { 0x422adb3b, 98 },
    { 0, 0 },
    { 0xd25adb3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaadb42, 55 },
    { 0x2c9adb43, 43 },
    { 0xd19adb44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x540adb47, 33 },
    { 0x2ffadb48, 98 },
    { 0xf49adb49, 61 },
    { 0xad3adb4a, 23 },
    { 0x484adb4a, 95 },
    { 0x34adb4c, 78 },
    { 0xb2adb49, 23 },
    { 0xffadb4e, 90 },
    { 0x796adb48, 61 },
    { 0x3c8adb50, 76 },
    { 0x1a5adb51, 90 },
    { 0xc5adb52, 23 },
    { 0x6dbadb51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8adb5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cadb63, 23 },
    { 0xe8aadb64, 33 },
    { 0, 0 },
    { 0xce4adb66, 33 },
    { 0x653adb66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22adb6d, 40 },
    { 0, 0 },
    { 0xa00adb6f, 33 },
    { 0, 0 },
    { 0x266adb71, 52 },
    { 0, 0 },
    { 0x830adb73, 33 },
    { 0, 0 },
    { 0xcfbadb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4adb7c, 34 },
    { 0x646adb7d, 61 },
    { 0xa4adb7e, 61 },
    { 0x757adb7f, 93 },
    { 0xc33adb7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65adb84, 95 },
    { 0, 0 },
    { 0x801adb86, 90 },
    { 0x3f3adb86, 91 },
    { 0x15badb87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746adb91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cadba0, 88 },
    { 0x657adba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aadba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfadba8, 99 },
    { 0xe45adba8, 76 },
    { 0xf09adbaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adadbae, 43 },
    { 0, 0 },
    { 0xd3badbb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7adbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406adbb9, 98 },
    { 0, 0 },
    { 0x2faadbbb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9adbc0, 23 },
    { 0x288adbc1, 43 },
    { 0xe06adbc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29adbc5, 61 },
    { 0x5d8adbc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55adbca, 90 },
    { 0xa25adbca, 55 },
    { 0x54cadbcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0adbd0, 46 },
    { 0x1f9adbd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x982adbd4, 88 },
    { 0xbdfadbd5, 98 },
    { 0x233adbd4, 98 },
    { 0xaaadbd5, 95 },
    { 0, 0 },
    { 0xb7dadbd9, 55 },
    { 0xa9adbda, 61 },
    { 0, 0 },
    { 0x4b4adbdc, 61 },
    { 0, 0 },
    { 0x624adbde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12adbe3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3adbe7, 33 },
    { 0x4c5adbe8, 61 },
    { 0, 0 },
    { 0x2d5adbea, 91 },
    { 0x6d4adbeb, 33 },
    { 0xc14adbea, 61 },
    { 0, 0 },
    { 0x8c7adbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebadbf1, 33 },
    { 0, 0 },
    { 0x2a6adbf3, 76 },
    { 0x9f7adbf4, 95 },
    { 0, 0 },
    { 0x8cbadbf6, 68 },
    { 0xfaaadbf7, 99 },
    { 0x22aadbf7, 93 },
    { 0xf05adbf7, 75 },
    { 0x25eadbfa, 91 },
    { 0x8ffadbfb, 98 },
    { 0xbb2adbf8, 61 },
    { 0x397adbf7, 23 },
    { 0xdcadbfe, 23 },
    { 0x2d5adbff, 81 },
    { 0x579adbf7, 95 },
    { 0, 0 },
    { 0x6dcadc02, 33 },
    { 0x4cdadc03, 81 },
    { 0x638adc03, 33 },
    { 0x497adc04, 90 },
    { 0xd64adc03, 99 },
    { 0, 0 },
    { 0xaefadc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aadc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37adc15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffadc1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574adc1f, 90 },
    { 0, 0 },
    { 0x646adc21, 93 },
    { 0x868adc22, 34 },
    { 0x934adc23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4adc2b, 77 },
    { 0x6b3adc2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84adc30, 90 },
    { 0xedadc30, 98 },
    { 0, 0 },
    { 0x2a6adc33, 90 },
    { 0x9d6adc34, 23 },
    { 0x98fadc34, 95 },
    { 0x6e5adc36, 76 },
    { 0xb0aadc37, 99 },
    { 0x2efadc38, 61 },
    { 0, 0 },
    { 0xf6badc3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6adc3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dadc42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x461adc45, 43 },
    { 0x5a2adc46, 23 },
    { 0, 0 },
    { 0xf72adc48, 95 },
    { 0, 0 },
    { 0x99dadc4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeadc4e, 33 },
    { 0xceadc4f, 98 },
    { 0x5b0adc50, 90 },
    { 0x9e6adc51, 100 },
    { 0x566adc52, 98 },
    { 0x35fadc50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9adc58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaadc5b, 66 },
    { 0xed6adc5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x572adc5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x470adc62, 98 },
    { 0x5eeadc63, 40 },
    { 0x6d0adc64, 61 },
    { 0xb4cadc64, 76 },
    { 0xf69adc62, 76 },
    { 0x9d9adc63, 90 },
    { 0xa5aadc64, 95 },
    { 0, 0 },
    { 0x172adc6a, 46 },
    { 0, 0 },
    { 0x597adc6c, 95 },
    { 0xc32adc6c, 93 },
    { 0, 0 },
    { 0xa4cadc6f, 99 },
    { 0x5c0adc70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50adc74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0adc79, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eadc7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efadc84, 43 },
    { 0x4e1adc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebadc8a, 76 },
    { 0xfedadc8b, 46 },
    { 0, 0 },
    { 0x23badc8d, 23 },
    { 0, 0 },
    { 0x72badc8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eadc98, 33 },
    { 0, 0 },
    { 0x616adc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34adc9d, 91 },
    { 0xd4eadc9e, 34 },
    { 0x20dadc9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7adca7, 90 },
    { 0xdedadca7, 75 },
    { 0xec0adca9, 52 },
    { 0, 0 },
    { 0x3a7adcab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726adcaf, 55 },
    { 0xa6fadcb0, 90 },
    { 0, 0 },
    { 0xd84adcb2, 55 },
    { 0x6cdadcb3, 91 },
    { 0xb74adcb4, 87 },
    { 0x35dadcb4, 95 },
    { 0x25cadcb3, 93 },
    { 0xdc2adcb7, 33 },
    { 0xda3adcb7, 95 },
    { 0xceadcb9, 90 },
    { 0x524adcb7, 95 },
    { 0, 0 },
    { 0x31fadcbc, 61 },
    { 0x392adcbc, 67 },
    { 0x938adcbc, 90 },
    { 0, 0 },
    { 0x939adcc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfadcc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5adcc8, 34 },
    { 0xeecadcc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2adcd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdadcd3, 95 },
    { 0x845adcd4, 33 },
    { 0x79eadcd5, 61 },
    { 0x174adcd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcadcde, 76 },
    { 0, 0 },
    { 0x5a4adce0, 61 },
    { 0x664adce1, 98 },
    { 0x12badce1, 90 },
    { 0, 0 },
    { 0xb5cadce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588adce8, 23 },
    { 0x55dadce9, 91 },
    { 0xedeadcea, 99 },
    { 0x19eadceb, 52 },
    { 0x649adceb, 23 },
    { 0x7c4adcea, 23 },
    { 0, 0 },
    { 0x560adcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x813adcf2, 99 },
    { 0x4b4adcf3, 33 },
    { 0, 0 },
    { 0x4a8adcf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962adcfd, 99 },
    { 0x988adcfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279add05, 90 },
    { 0x6dbadd06, 90 },
    { 0, 0 },
    { 0xf71add08, 67 },
    { 0x30cadd09, 98 },
    { 0xabfadd09, 17 },
    { 0, 0 },
    { 0x225add0c, 23 },
    { 0xd47add0d, 90 },
    { 0, 0 },
    { 0xa4cadd0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecadd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dadd1d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x673add20, 55 },
    { 0xa7cadd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aadd2c, 90 },
    { 0xfdcadd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81add32, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3add37, 95 },
    { 0xcd8add38, 95 },
    { 0xc50add39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74add3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadd40, 66 },
    { 0x99fadd41, 61 },
    { 0x80dadd42, 17 },
    { 0, 0 },
    { 0x30eadd44, 65 },
    { 0xa13add45, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95add48, 64 },
    { 0, 0 },
    { 0xb37add4a, 95 },
    { 0x8e4add4b, 61 },
    { 0xec5add4c, 62 },
    { 0xb53add4a, 23 },
    { 0xb8eadd4d, 90 },
    { 0x9fbadd4f, 61 },
    { 0x195add4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6add53, 95 },
    { 0xa03add54, 95 },
    { 0xe32add55, 33 },
    { 0xca0add54, 33 },
    { 0xb86add55, 95 },
    { 0xa67add56, 33 },
    { 0x563add55, 95 },
    { 0xa28add53, 91 },
    { 0, 0 },
    { 0xa91add5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92add5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x274add62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31add65, 33 },
    { 0xce7add66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfadd69, 61 },
    { 0x731add6a, 76 },
    { 0x927add69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadd6e, 61 },
    { 0, 0 },
    { 0xa45add70, 83 },
    { 0x1eeadd71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227add75, 61 },
    { 0xb8dadd76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dadd7c, 76 },
    { 0, 0 },
    { 0xa4cadd7e, 61 },
    { 0xb73add7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252add85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6add8e, 67 },
    { 0xa4aadd8f, 85 },
    { 0x274add8e, 55 },
    { 0x5b4add8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x486add94, 33 },
    { 0xa34add95, 46 },
    { 0x2d3add95, 94 },
    { 0x184add97, 61 },
    { 0, 0 },
    { 0xde0add99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbadd9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6add9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaadda7, 61 },
    { 0, 0 },
    { 0x184adda9, 23 },
    { 0x1bcaddaa, 34 },
    { 0x3eaaddab, 23 },
    { 0x465addac, 88 },
    { 0x171adda9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x378addb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdaddb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7addb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9addbe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4addc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3addc4, 61 },
    { 0x7b3addc4, 33 },
    { 0x365addc6, 43 },
    { 0x6e4addc7, 61 },
    { 0xa96addc6, 34 },
    { 0xdfaaddc9, 61 },
    { 0, 0 },
    { 0x9d9addcb, 55 },
    { 0, 0 },
    { 0x912addcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfaddd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48addda, 97 },
    { 0x221adddb, 90 },
    { 0, 0 },
    { 0x99adddd, 76 },
    { 0, 0 },
    { 0xb59adddf, 64 },
    { 0, 0 },
    { 0xd36adde1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3addec, 69 },
    { 0, 0 },
    { 0xdbcaddee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x305addf1, 91 },
    { 0, 0 },
    { 0x44addf3, 61 },
    { 0x1b8addf4, 46 },
    { 0xbb7addf5, 55 },
    { 0x443addf5, 61 },
    { 0xaa8addf7, 84 },
    { 0xe05addf4, 98 },
    { 0, 0 },
    { 0x52baddfa, 33 },
    { 0, 0 },
    { 0xce8addfc, 99 },
    { 0xf6baddfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594ade09, 23 },
    { 0xd58ade0a, 43 },
    { 0x54eade0b, 33 },
    { 0x29dade0b, 98 },
    { 0xcc9ade0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ade11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ade14, 76 },
    { 0xa92ade15, 76 },
    { 0x17cade16, 23 },
    { 0x62cade17, 101 },
    { 0x5d2ade18, 33 },
    { 0xc3dade19, 67 },
    { 0x7bbade1a, 55 },
    { 0xfd8ade15, 75 },
    { 0xfdbade14, 43 },
    { 0x180ade18, 23 },
    { 0x349ade1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdade21, 87 },
    { 0xe9ade22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ade2d, 61 },
    { 0x71bade2e, 93 },
    { 0x1aade2e, 23 },
    { 0x742ade30, 90 },
    { 0x58eade30, 61 },
    { 0xb7fade32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964ade36, 61 },
    { 0x4a6ade37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bade3b, 33 },
    { 0x8d0ade3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ade46, 43 },
    { 0xd5dade47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x248ade4a, 80 },
    { 0, 0 },
    { 0x7c7ade4c, 61 },
    { 0x4a5ade4d, 78 },
    { 0x9fcade4c, 95 },
    { 0x7dfade4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x175ade52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cade55, 77 },
    { 0x958ade56, 33 },
    { 0x171ade57, 77 },
    { 0xb55ade55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbade5b, 90 },
    { 0x8bade5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ade61, 90 },
    { 0, 0 },
    { 0xfdcade63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3ade6b, 85 },
    { 0, 0 },
    { 0x9d1ade6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aade70, 61 },
    { 0x722ade70, 95 },
    { 0x64ade70, 75 },
    { 0, 0 },
    { 0x98eade74, 91 },
    { 0x2dade75, 61 },
    { 0x82cade74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebade7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fade7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adade90, 85 },
    { 0xee5ade91, 61 },
    { 0x4d2ade92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432ade96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786ade9e, 90 },
    { 0xe7dade9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3adea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603adea6, 90 },
    { 0xf98adea6, 95 },
    { 0xeeeadea7, 100 },
    { 0xaf5adea9, 24 },
    { 0xc2fadeaa, 95 },
    { 0xa4adeab, 100 },
    { 0x9d5adeac, 55 },
    { 0xa09adeab, 90 },
    { 0x2b8adeae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906adeb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33adeb9, 43 },
    { 0, 0 },
    { 0x919adebb, 33 },
    { 0, 0 },
    { 0xaa5adebd, 33 },
    { 0xf41adebe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07adec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563adec7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x515adeca, 98 },
    { 0x5a1adeca, 100 },
    { 0x1ebadecc, 33 },
    { 0, 0 },
    { 0xc75adece, 61 },
    { 0, 0 },
    { 0x726aded0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x96daded3, 91 },
    { 0xb54aded4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cadeda, 23 },
    { 0xf21adedb, 75 },
    { 0x1c5adedc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7adee3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9adee6, 90 },
    { 0, 0 },
    { 0x4f4adee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7adeec, 23 },
    { 0, 0 },
    { 0xffdadeee, 76 },
    { 0, 0 },
    { 0x474adef0, 55 },
    { 0, 0 },
    { 0xd4eadef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2adef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfadefa, 61 },
    { 0x72eadefb, 96 },
    { 0xf5adefc, 99 },
    { 0xc64adefd, 43 },
    { 0x9eadefe, 98 },
    { 0x4a9adefc, 66 },
    { 0, 0 },
    { 0x6b1adf01, 95 },
    { 0xdf6adf02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0adf08, 90 },
    { 0xd8cadf08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecadf0c, 78 },
    { 0x38fadf0d, 33 },
    { 0x502adf0d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9adf11, 33 },
    { 0x572adf12, 75 },
    { 0x3b8adf12, 81 },
    { 0, 0 },
    { 0x836adf15, 76 },
    { 0, 0 },
    { 0xf39adf17, 22 },
    { 0, 0 },
    { 0xe07adf19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9adf1d, 23 },
    { 0xbe8adf1e, 87 },
    { 0xd6adf1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x576adf22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48adf28, 56 },
    { 0, 0 },
    { 0xa65adf2a, 98 },
    { 0x5dbadf2a, 43 },
    { 0x761adf2c, 61 },
    { 0xc09adf2c, 61 },
    { 0, 0 },
    { 0xfadf2f, 98 },
    { 0xc78adf2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953adf35, 87 },
    { 0xacaadf36, 95 },
    { 0xd07adf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424adf3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x798adf41, 95 },
    { 0x56fadf41, 98 },
    { 0, 0 },
    { 0x80fadf44, 95 },
    { 0xf0badf45, 52 },
    { 0x90badf44, 61 },
    { 0xa60adf47, 23 },
    { 0x980adf48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9adf4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebadf50, 100 },
    { 0xdd9adf51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3adf54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2adf57, 68 },
    { 0x256adf57, 61 },
    { 0x132adf57, 23 },
    { 0x34badf57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9badf5d, 90 },
    { 0, 0 },
    { 0x96aadf5f, 34 },
    { 0x574adf60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604adf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efadf6c, 76 },
    { 0xdc1adf6d, 97 },
    { 0x713adf6d, 90 },
    { 0, 0 },
    { 0x125adf70, 98 },
    { 0x5b7adf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aadf7a, 90 },
    { 0xe50adf7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eadf82, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbadf87, 55 },
    { 0xa72adf88, 90 },
    { 0x80eadf88, 98 },
    { 0x3a7adf8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7adf8d, 75 },
    { 0x336adf8e, 47 },
    { 0x715adf8d, 61 },
    { 0x2faadf90, 12 },
    { 0x685adf91, 61 },
    { 0xf51adf92, 100 },
    { 0x52badf93, 34 },
    { 0x354adf94, 98 },
    { 0xf1cadf94, 90 },
    { 0xc5fadf96, 76 },
    { 0x339adf97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0adf9c, 53 },
    { 0x232adf9c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x733adfa0, 34 },
    { 0, 0 },
    { 0xad6adfa2, 61 },
    { 0xbaadfa3, 95 },
    { 0, 0 },
    { 0xa72adfa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2adfaf, 33 },
    { 0, 0 },
    { 0x440adfb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866adfb6, 61 },
    { 0, 0 },
    { 0x48cadfb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91adfbd, 36 },
    { 0, 0 },
    { 0x1f8adfbf, 61 },
    { 0x898adfc0, 98 },
    { 0, 0 },
    { 0xfdeadfc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56adfc5, 46 },
    { 0x5a0adfc6, 55 },
    { 0x794adfc7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62adfcb, 84 },
    { 0, 0 },
    { 0xecbadfcd, 95 },
    { 0, 0 },
    { 0x18cadfcf, 66 },
    { 0x4a3adfd0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85adfd7, 46 },
    { 0xc0fadfd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6adfdb, 34 },
    { 0x19badfdc, 22 },
    { 0x6d7adfdc, 78 },
    { 0x657adfdc, 33 },
    { 0xce8adfdc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23adfe8, 90 },
    { 0, 0 },
    { 0x1a7adfea, 52 },
    { 0xaddadfeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0adff6, 23 },
    { 0xb99adff7, 100 },
    { 0x5e9adff8, 46 },
    { 0xe62adff8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19adffd, 78 },
    { 0x987adffd, 55 },
    { 0, 0 },
    { 0xfacae000, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ae008, 33 },
    { 0xbfae008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x641ae00c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cae00f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ae012, 81 },
    { 0x814ae013, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceae019, 23 },
    { 0xc21ae01a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ae01d, 90 },
    { 0xa24ae01d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ae021, 99 },
    { 0xb55ae021, 87 },
    { 0x89fae023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bae028, 77 },
    { 0xa68ae029, 43 },
    { 0x517ae02a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ae02e, 100 },
    { 0x2b5ae02e, 91 },
    { 0x6bbae02e, 98 },
    { 0x496ae02e, 90 },
    { 0x5dae02e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dae036, 67 },
    { 0, 0 },
    { 0x2bdae038, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ae03b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeae044, 33 },
    { 0x99ae045, 76 },
    { 0, 0 },
    { 0x79cae047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaae04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x992ae04e, 80 },
    { 0, 0 },
    { 0x875ae050, 95 },
    { 0, 0 },
    { 0x378ae052, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666ae05a, 66 },
    { 0x5eeae05b, 66 },
    { 0, 0 },
    { 0x3f2ae05d, 37 },
    { 0, 0 },
    { 0xb42ae05f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bae062, 93 },
    { 0x52fae062, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ae066, 90 },
    { 0, 0 },
    { 0x76aae068, 34 },
    { 0, 0 },
    { 0x35ae06a, 52 },
    { 0x50fae06b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86eae06f, 91 },
    { 0, 0 },
    { 0x9c7ae071, 88 },
    { 0x2c3ae072, 33 },
    { 0xb7cae071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ae078, 33 },
    { 0, 0 },
    { 0xf10ae07a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae07f, 98 },
    { 0x23aae07f, 98 },
    { 0xeecae081, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9ae084, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eae088, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7ae08c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ae094, 23 },
    { 0, 0 },
    { 0x73dae096, 93 },
    { 0, 0 },
    { 0x203ae098, 33 },
    { 0xadcae099, 61 },
    { 0x6ebae09a, 99 },
    { 0, 0 },
    { 0x2d9ae09c, 90 },
    { 0xf5aae09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae0a1, 34 },
    { 0x675ae0a2, 93 },
    { 0, 0 },
    { 0x60ae0a4, 34 },
    { 0x86aae0a5, 33 },
    { 0x8cbae0a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4ae0a9, 55 },
    { 0x98bae0aa, 88 },
    { 0x1a9ae0ab, 84 },
    { 0x596ae0ac, 98 },
    { 0xa6bae0ad, 88 },
    { 0x888ae0ac, 95 },
    { 0x5d8ae0ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483ae0b3, 61 },
    { 0, 0 },
    { 0x775ae0b5, 61 },
    { 0x680ae0b6, 87 },
    { 0x7b2ae0b7, 34 },
    { 0, 0 },
    { 0x60bae0b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58ae0bc, 66 },
    { 0x3e0ae0bd, 43 },
    { 0xec4ae0bc, 23 },
    { 0xe5dae0bf, 33 },
    { 0x7eeae0bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587ae0ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6ae0d8, 33 },
    { 0xcbbae0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbae0de, 90 },
    { 0xeecae0df, 95 },
    { 0x471ae0de, 67 },
    { 0x352ae0df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ae0e4, 90 },
    { 0xf7ae0e4, 76 },
    { 0xa29ae0e6, 36 },
    { 0x278ae0e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ae0ea, 23 },
    { 0x6afae0eb, 61 },
    { 0x89bae0ec, 43 },
    { 0x304ae0ea, 55 },
    { 0x9bae0ee, 33 },
    { 0x8baae0ec, 33 },
    { 0, 0 },
    { 0x881ae0f1, 98 },
    { 0, 0 },
    { 0x685ae0f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54ae0f8, 90 },
    { 0x769ae0f8, 61 },
    { 0x1cfae0fa, 90 },
    { 0xfd8ae0f9, 61 },
    { 0x276ae0fc, 23 },
    { 0xd14ae0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9ae101, 90 },
    { 0x85cae102, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae107, 36 },
    { 0, 0 },
    { 0x4f5ae109, 66 },
    { 0xf09ae109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfae112, 23 },
    { 0x11bae112, 66 },
    { 0x931ae114, 81 },
    { 0x660ae114, 33 },
    { 0xa2eae115, 95 },
    { 0, 0 },
    { 0x386ae118, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bae128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3ae12b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ae131, 90 },
    { 0, 0 },
    { 0x484ae133, 34 },
    { 0, 0 },
    { 0x421ae135, 34 },
    { 0xdf8ae136, 43 },
    { 0x193ae137, 78 },
    { 0x5e8ae137, 90 },
    { 0x325ae139, 95 },
    { 0x3a5ae137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dae140, 75 },
    { 0xf94ae140, 66 },
    { 0x4beae142, 95 },
    { 0xcc5ae141, 100 },
    { 0x6d2ae144, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4ae147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x961ae14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3ae14d, 33 },
    { 0x404ae14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ae156, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bae15b, 61 },
    { 0xf14ae15c, 98 },
    { 0, 0 },
    { 0xc0dae15e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae161, 81 },
    { 0x73fae162, 98 },
    { 0x9d2ae162, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x683ae166, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ae16b, 76 },
    { 0, 0 },
    { 0x598ae16d, 33 },
    { 0x3b3ae16e, 93 },
    { 0xfbaae16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cae174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebae17f, 23 },
    { 0xc97ae180, 76 },
    { 0xdebae181, 67 },
    { 0x4c7ae181, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae185, 48 },
    { 0, 0 },
    { 0x6f9ae187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fae18a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96ae18e, 33 },
    { 0x391ae18e, 47 },
    { 0xd08ae190, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fae193, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eae196, 84 },
    { 0, 0 },
    { 0x1ffae198, 90 },
    { 0x3a4ae199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854ae19d, 56 },
    { 0x937ae19d, 78 },
    { 0, 0 },
    { 0x49dae1a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ae1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ae1a8, 90 },
    { 0x6c7ae1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ae1ac, 68 },
    { 0x4a6ae1ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbae1b0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90ae1b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecae1b7, 33 },
    { 0xc08ae1b7, 90 },
    { 0xbfdae1b9, 85 },
    { 0x950ae1ba, 61 },
    { 0, 0 },
    { 0xe5bae1bc, 77 },
    { 0x2f4ae1bd, 19 },
    { 0x1acae1be, 90 },
    { 0x67cae1bf, 23 },
    { 0x31fae1c0, 23 },
    { 0xcccae1c1, 90 },
    { 0, 0 },
    { 0x155ae1c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ae1c7, 55 },
    { 0x9dae1c7, 43 },
    { 0xfbdae1c8, 76 },
    { 0, 0 },
    { 0xef7ae1cb, 66 },
    { 0, 0 },
    { 0xb5cae1cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccae1d1, 33 },
    { 0x223ae1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859ae1d6, 95 },
    { 0, 0 },
    { 0x588ae1d8, 33 },
    { 0, 0 },
    { 0xb7ae1da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fae1dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ae1e2, 85 },
    { 0x644ae1e3, 55 },
    { 0xa08ae1e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ae1ea, 33 },
    { 0, 0 },
    { 0xcaaae1ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcae1f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ae1f4, 95 },
    { 0x401ae1f5, 95 },
    { 0x9edae1f6, 98 },
    { 0x665ae1f5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeae1fa, 87 },
    { 0x567ae1fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae1fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bae202, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bae20a, 61 },
    { 0xd9ae20b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae20e, 33 },
    { 0xd5eae20f, 33 },
    { 0xf59ae210, 90 },
    { 0x249ae20e, 95 },
    { 0, 0 },
    { 0xfdaae213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ae216, 78 },
    { 0x9cae217, 23 },
    { 0xeceae216, 90 },
    { 0xa87ae219, 43 },
    { 0xe78ae21a, 90 },
    { 0, 0 },
    { 0xc0bae21c, 90 },
    { 0xbedae21c, 61 },
    { 0x7e4ae21e, 95 },
    { 0x1f9ae21f, 98 },
    { 0x81eae21e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ae223, 90 },
    { 0xc5ae224, 85 },
    { 0xe24ae223, 43 },
    { 0, 0 },
    { 0xe5fae227, 98 },
    { 0xd15ae227, 61 },
    { 0xd67ae227, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ae22d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cae230, 17 },
    { 0, 0 },
    { 0xeb4ae232, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aae236, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16ae239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8ae23c, 67 },
    { 0x43eae23c, 33 },
    { 0, 0 },
    { 0xb4eae23f, 98 },
    { 0, 0 },
    { 0xbecae241, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ae245, 23 },
    { 0xdfaae245, 61 },
    { 0, 0 },
    { 0xa58ae248, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aae24d, 100 },
    { 0x873ae24d, 93 },
    { 0xb1aae24f, 98 },
    { 0xc51ae250, 95 },
    { 0xfa3ae251, 23 },
    { 0, 0 },
    { 0x707ae253, 17 },
    { 0xc8cae254, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66ae257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cae25a, 34 },
    { 0xa7ae25b, 100 },
    { 0x8baae25c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998ae262, 95 },
    { 0, 0 },
    { 0xfa3ae264, 43 },
    { 0x93dae265, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ae26b, 90 },
    { 0, 0 },
    { 0x530ae26d, 90 },
    { 0xb9fae26e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceae278, 85 },
    { 0x135ae279, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ae282, 76 },
    { 0xb51ae283, 93 },
    { 0xebcae282, 100 },
    { 0x7a1ae285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ae28c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46aae28f, 61 },
    { 0, 0 },
    { 0x51cae291, 68 },
    { 0xef6ae292, 93 },
    { 0x701ae293, 88 },
    { 0x8c7ae293, 84 },
    { 0, 0 },
    { 0x895ae296, 61 },
    { 0, 0 },
    { 0x94aae298, 81 },
    { 0x796ae298, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baae29c, 33 },
    { 0x3d3ae29c, 95 },
    { 0xb25ae29d, 95 },
    { 0, 0 },
    { 0x906ae2a0, 34 },
    { 0xdb4ae2a1, 34 },
    { 0x8a9ae2a1, 58 },
    { 0xfcbae2a3, 99 },
    { 0, 0 },
    { 0x2fbae2a5, 88 },
    { 0x9dae2a5, 55 },
    { 0x731ae2a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaae2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ae2af, 90 },
    { 0xd95ae2b0, 43 },
    { 0x450ae2af, 43 },
    { 0xec6ae2b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1ae2b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512ae2bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ae2c0, 65 },
    { 0x4dcae2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcae2d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeae2d3, 90 },
    { 0x3b5ae2d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aae2d9, 67 },
    { 0xf94ae2da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ae2ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11ae2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7ae2f4, 98 },
    { 0x3ecae2f5, 33 },
    { 0xfe6ae2f6, 80 },
    { 0, 0 },
    { 0x137ae2f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20ae2ff, 76 },
    { 0xf69ae300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x248ae303, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae306, 84 },
    { 0xdffae307, 61 },
    { 0x958ae307, 23 },
    { 0x453ae309, 90 },
    { 0, 0 },
    { 0x16ae30b, 52 },
    { 0x532ae30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8ae311, 34 },
    { 0, 0 },
    { 0x3c7ae313, 61 },
    { 0, 0 },
    { 0xd96ae315, 68 },
    { 0, 0 },
    { 0x6e8ae317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bae31c, 77 },
    { 0xac5ae31d, 65 },
    { 0x904ae31e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fae321, 33 },
    { 0xe05ae321, 90 },
    { 0, 0 },
    { 0xc89ae324, 90 },
    { 0x677ae325, 52 },
    { 0, 0 },
    { 0x8b2ae327, 33 },
    { 0, 0 },
    { 0x902ae329, 34 },
    { 0x4aae32a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ae32d, 33 },
    { 0x555ae32d, 90 },
    { 0, 0 },
    { 0xe40ae330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ae334, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ae337, 85 },
    { 0x962ae338, 23 },
    { 0x1beae338, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcae33f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acae342, 90 },
    { 0x68ae342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bae346, 23 },
    { 0x66cae347, 90 },
    { 0xb12ae348, 33 },
    { 0xc5aae349, 93 },
    { 0xfa4ae34a, 98 },
    { 0xbcfae34b, 33 },
    { 0x57cae348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeae34f, 93 },
    { 0, 0 },
    { 0x3d7ae351, 53 },
    { 0x3a8ae351, 76 },
    { 0, 0 },
    { 0x397ae354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ae358, 95 },
    { 0x882ae359, 77 },
    { 0xb1ae35a, 98 },
    { 0xeadae35a, 66 },
    { 0xdeaae35a, 34 },
    { 0x7ebae35d, 100 },
    { 0x8b2ae35b, 33 },
    { 0x64aae35f, 91 },
    { 0x9cbae35f, 76 },
    { 0, 0 },
    { 0x1deae362, 46 },
    { 0x19eae363, 84 },
    { 0xe1bae364, 61 },
    { 0x285ae363, 94 },
    { 0x112ae366, 67 },
    { 0x691ae367, 85 },
    { 0x82fae367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eae36d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bae376, 100 },
    { 0, 0 },
    { 0x1f2ae378, 90 },
    { 0xf02ae379, 98 },
    { 0xe25ae37a, 95 },
    { 0, 0 },
    { 0xf9dae37c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ae37f, 33 },
    { 0xae380, 95 },
    { 0x479ae381, 90 },
    { 0, 0 },
    { 0xa02ae383, 91 },
    { 0x37fae383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683ae389, 75 },
    { 0x22ae38a, 85 },
    { 0x881ae38a, 46 },
    { 0xadaae389, 95 },
    { 0x81eae38d, 33 },
    { 0x7eeae38b, 66 },
    { 0, 0 },
    { 0x370ae390, 66 },
    { 0x540ae391, 90 },
    { 0, 0 },
    { 0x7efae393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6ae399, 77 },
    { 0x889ae39a, 90 },
    { 0xf51ae399, 78 },
    { 0, 0 },
    { 0xa6bae39d, 76 },
    { 0x75dae39d, 23 },
    { 0x69bae39d, 33 },
    { 0xaf9ae3a0, 85 },
    { 0x6dcae3a0, 61 },
    { 0x4e1ae3a2, 46 },
    { 0x220ae3a3, 93 },
    { 0x68ae3a2, 92 },
    { 0x4e4ae3a5, 43 },
    { 0xa7fae3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7ae3ab, 61 },
    { 0xd99ae3ac, 90 },
    { 0xd7aae3ac, 33 },
    { 0xd81ae3ad, 34 },
    { 0xba5ae3af, 46 },
    { 0, 0 },
    { 0x621ae3b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ae3ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ae3c0, 43 },
    { 0, 0 },
    { 0xe0eae3c2, 33 },
    { 0x194ae3c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ae3c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdae3cb, 90 },
    { 0x6c5ae3cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4ae3dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17ae3df, 61 },
    { 0, 0 },
    { 0xd15ae3e1, 61 },
    { 0, 0 },
    { 0x3faae3e3, 93 },
    { 0x9baae3e4, 93 },
    { 0xc9fae3e3, 36 },
    { 0x3e5ae3e6, 55 },
    { 0xeb1ae3e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ae3ea, 76 },
    { 0, 0 },
    { 0xf3dae3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ae3f1, 98 },
    { 0xdd5ae3f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822ae3f6, 76 },
    { 0x333ae3f7, 34 },
    { 0xc19ae3f8, 43 },
    { 0xda0ae3f9, 34 },
    { 0xbb9ae3fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79ae3fd, 90 },
    { 0xc36ae3fe, 78 },
    { 0, 0 },
    { 0x6f8ae400, 46 },
    { 0xe7aae401, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ae404, 33 },
    { 0xb52ae405, 33 },
    { 0x93cae406, 90 },
    { 0xd5eae406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896ae40c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554ae414, 17 },
    { 0, 0 },
    { 0xc92ae416, 78 },
    { 0xc61ae417, 33 },
    { 0xd32ae418, 66 },
    { 0xec5ae419, 23 },
    { 0x9c3ae41a, 99 },
    { 0x4c9ae41b, 95 },
    { 0x4daae41b, 90 },
    { 0xa8aae41c, 33 },
    { 0xa1eae417, 90 },
    { 0xe4ae41d, 23 },
    { 0x8d4ae416, 98 },
    { 0x4ccae417, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908ae428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ae42d, 23 },
    { 0, 0 },
    { 0x3d7ae42f, 61 },
    { 0, 0 },
    { 0x576ae431, 55 },
    { 0x8d9ae432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29ae43a, 98 },
    { 0xb4dae43b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2ae442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341ae447, 91 },
    { 0x9e3ae448, 76 },
    { 0xab0ae449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55ae44f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccae456, 17 },
    { 0x8f2ae457, 33 },
    { 0xbcae458, 33 },
    { 0x3c0ae459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742ae45f, 98 },
    { 0xe9ae460, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370ae464, 33 },
    { 0xaf4ae464, 33 },
    { 0x80aae466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cae46a, 90 },
    { 0xfd5ae46b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6ae472, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fae478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bae482, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fae487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ae48c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969ae495, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367ae49a, 61 },
    { 0x817ae49b, 98 },
    { 0x394ae49b, 66 },
    { 0x6a4ae49d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae4a2, 95 },
    { 0x65fae4a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deae4a6, 46 },
    { 0x627ae4a6, 98 },
    { 0, 0 },
    { 0xd8eae4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcae4ac, 33 },
    { 0x279ae4ac, 84 },
    { 0, 0 },
    { 0xd85ae4af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392ae4b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fae4b7, 46 },
    { 0xea3ae4b8, 42 },
    { 0, 0 },
    { 0x122ae4ba, 93 },
    { 0x1daae4bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ae4c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedae4c8, 33 },
    { 0xad8ae4c8, 46 },
    { 0, 0 },
    { 0x10bae4cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3ae4cf, 34 },
    { 0xf5dae4d0, 98 },
    { 0xaaae4d1, 15 },
    { 0x7daae4d2, 61 },
    { 0, 0 },
    { 0xd73ae4d4, 23 },
    { 0xf93ae4d5, 90 },
    { 0xc1fae4d4, 87 },
    { 0x5ae4d7, 84 },
    { 0xc86ae4d8, 55 },
    { 0x7d2ae4d8, 91 },
    { 0xb9cae4d7, 75 },
    { 0xf1fae4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eae4de, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5ae4e4, 43 },
    { 0xac4ae4e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cae4e8, 98 },
    { 0x2c4ae4e8, 95 },
    { 0, 0 },
    { 0x58aae4eb, 61 },
    { 0, 0 },
    { 0x74aae4ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceae4f1, 23 },
    { 0xc0dae4f1, 95 },
    { 0xd43ae4f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cae4f6, 99 },
    { 0x383ae4f6, 66 },
    { 0, 0 },
    { 0xc03ae4f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaae4ff, 90 },
    { 0x3c5ae4ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5ae505, 61 },
    { 0, 0 },
    { 0xd9fae507, 52 },
    { 0, 0 },
    { 0x64cae509, 97 },
    { 0, 0 },
    { 0x4c3ae50b, 77 },
    { 0x6c8ae50c, 90 },
    { 0x77aae50b, 34 },
    { 0, 0 },
    { 0xfd6ae50f, 61 },
    { 0, 0 },
    { 0x8f7ae511, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fae516, 43 },
    { 0xa1eae517, 61 },
    { 0x4ceae516, 98 },
    { 0xcfae519, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ae51c, 34 },
    { 0x205ae51d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae520, 43 },
    { 0, 0 },
    { 0x1a5ae522, 55 },
    { 0x987ae523, 95 },
    { 0, 0 },
    { 0x5dae525, 95 },
    { 0, 0 },
    { 0xe07ae527, 95 },
    { 0x85eae528, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dae52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fae530, 52 },
    { 0x8fcae531, 93 },
    { 0xe42ae532, 99 },
    { 0, 0 },
    { 0xc5aae534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae537, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aae53a, 43 },
    { 0x366ae53b, 55 },
    { 0x8d7ae53a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x842ae53f, 90 },
    { 0, 0 },
    { 0x9dcae541, 78 },
    { 0x3deae541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fae547, 77 },
    { 0x312ae548, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06ae555, 61 },
    { 0xf7bae556, 98 },
    { 0, 0 },
    { 0x562ae558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eae560, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacae563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4ae566, 99 },
    { 0xf2dae566, 80 },
    { 0, 0 },
    { 0xb75ae569, 59 },
    { 0xa8dae56a, 90 },
    { 0x1c4ae56b, 90 },
    { 0xc55ae56b, 33 },
    { 0x19aae56d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94ae571, 34 },
    { 0x3d9ae572, 33 },
    { 0xdf3ae573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ae57f, 40 },
    { 0x411ae580, 52 },
    { 0xc38ae580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ae58a, 55 },
    { 0x110ae58b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cae592, 90 },
    { 0x36eae592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deae596, 98 },
    { 0x4fbae597, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50ae59b, 61 },
    { 0x859ae59b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8ae59f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ae5a3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ae5a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aae5aa, 17 },
    { 0, 0 },
    { 0x7d7ae5ac, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803ae5b2, 23 },
    { 0, 0 },
    { 0x565ae5b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ae5b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cae5bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feae5c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0ae5cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebae5d0, 61 },
    { 0xc29ae5d1, 90 },
    { 0xfadae5d2, 88 },
    { 0xcfbae5d1, 98 },
    { 0x245ae5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ae5da, 95 },
    { 0xbf4ae5db, 100 },
    { 0xcfeae5dc, 78 },
    { 0x252ae5dd, 76 },
    { 0x6aeae5dd, 61 },
    { 0x85eae5df, 61 },
    { 0, 0 },
    { 0xcd1ae5e1, 90 },
    { 0x822ae5e1, 95 },
    { 0, 0 },
    { 0xe58ae5e4, 46 },
    { 0xaa6ae5e5, 98 },
    { 0x231ae5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592ae5ea, 90 },
    { 0xb32ae5eb, 98 },
    { 0, 0 },
    { 0x3edae5ed, 98 },
    { 0xdf1ae5ed, 90 },
    { 0xa6eae5ed, 78 },
    { 0xd9bae5ef, 95 },
    { 0xbc9ae5f1, 23 },
    { 0xed0ae5f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143ae5ff, 61 },
    { 0xc76ae5ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbae603, 90 },
    { 0x55dae603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fae608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89ae60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefae616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ae61c, 84 },
    { 0x1fcae61d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ae622, 77 },
    { 0x9e0ae622, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bae626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106ae62b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cae62e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bae634, 95 },
    { 0x458ae634, 43 },
    { 0x7c9ae636, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cae641, 93 },
    { 0x623ae642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9ae646, 93 },
    { 0, 0 },
    { 0x10cae648, 46 },
    { 0, 0 },
    { 0xc78ae64a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cae64d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ae65c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ae65f, 33 },
    { 0x460ae660, 93 },
    { 0x7cdae661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35eae66c, 33 },
    { 0, 0 },
    { 0x559ae66e, 61 },
    { 0xae0ae66f, 34 },
    { 0, 0 },
    { 0x6c8ae671, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ae679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeae67e, 30 },
    { 0, 0 },
    { 0x62fae680, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ae683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fae687, 56 },
    { 0x74bae688, 98 },
    { 0, 0 },
    { 0xf4dae68a, 90 },
    { 0, 0 },
    { 0x5b5ae68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aae691, 23 },
    { 0x7fbae691, 90 },
    { 0x2bdae693, 93 },
    { 0x6e9ae693, 95 },
    { 0x270ae695, 87 },
    { 0x234ae696, 87 },
    { 0x972ae694, 75 },
    { 0x26cae698, 95 },
    { 0, 0 },
    { 0xe3ae69a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fae69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430ae6a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ae6a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ae6ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73ae6b2, 55 },
    { 0x724ae6b2, 23 },
    { 0, 0 },
    { 0x713ae6b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662ae6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x244ae6c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfae6c5, 61 },
    { 0x2c9ae6c6, 96 },
    { 0x40eae6c7, 34 },
    { 0x80aae6c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ae6ce, 33 },
    { 0xf7ae6ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaae6d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ae6d7, 90 },
    { 0, 0 },
    { 0x7d0ae6d9, 43 },
    { 0xc94ae6da, 76 },
    { 0x45cae6db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0ae6e4, 90 },
    { 0x339ae6e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952ae6ec, 91 },
    { 0xc65ae6ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ae6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aae6f7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ae6fa, 100 },
    { 0, 0 },
    { 0x3b8ae6fc, 68 },
    { 0xf3fae6fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eae703, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bae709, 76 },
    { 0, 0 },
    { 0x3ddae70b, 66 },
    { 0, 0 },
    { 0x7f8ae70d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dae711, 93 },
    { 0, 0 },
    { 0x5d2ae713, 99 },
    { 0, 0 },
    { 0x139ae715, 65 },
    { 0xe7eae716, 76 },
    { 0xec2ae717, 33 },
    { 0, 0 },
    { 0x815ae719, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47ae71c, 48 },
    { 0, 0 },
    { 0x70aae71e, 90 },
    { 0, 0 },
    { 0xe38ae720, 78 },
    { 0x861ae720, 36 },
    { 0x291ae722, 61 },
    { 0x2daae721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ae727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdae72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdae731, 90 },
    { 0xb95ae732, 98 },
    { 0x25eae732, 98 },
    { 0, 0 },
    { 0x5c1ae735, 90 },
    { 0, 0 },
    { 0x9c8ae737, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfae73a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ae73e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fae744, 46 },
    { 0x7edae745, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2ae748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ae758, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720ae75b, 100 },
    { 0, 0 },
    { 0x898ae75d, 93 },
    { 0xcc7ae75d, 34 },
    { 0x86fae75f, 90 },
    { 0xa0eae75e, 89 },
    { 0, 0 },
    { 0x1dbae762, 61 },
    { 0xa84ae762, 39 },
    { 0xcf2ae764, 33 },
    { 0x82dae765, 93 },
    { 0, 0 },
    { 0x932ae767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546ae76b, 65 },
    { 0, 0 },
    { 0xb7aae76d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06ae770, 66 },
    { 0x2e5ae770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dae774, 55 },
    { 0, 0 },
    { 0x2aeae776, 33 },
    { 0xc8cae777, 77 },
    { 0x3c0ae777, 87 },
    { 0x834ae779, 34 },
    { 0, 0 },
    { 0x934ae77b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaae781, 90 },
    { 0x6afae782, 84 },
    { 0x684ae783, 95 },
    { 0, 0 },
    { 0x4c4ae785, 76 },
    { 0x387ae786, 91 },
    { 0, 0 },
    { 0x3bdae788, 91 },
    { 0x316ae789, 91 },
    { 0xca4ae788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae792, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ae796, 23 },
    { 0xcae797, 76 },
    { 0x845ae796, 93 },
    { 0x8a6ae799, 90 },
    { 0x8d3ae79a, 84 },
    { 0xa15ae79a, 90 },
    { 0xb44ae79c, 95 },
    { 0x41bae79d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ae7a1, 98 },
    { 0, 0 },
    { 0x5f4ae7a3, 76 },
    { 0, 0 },
    { 0xdc3ae7a5, 61 },
    { 0xe45ae7a6, 33 },
    { 0x539ae7a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fae7aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317ae7b0, 78 },
    { 0x59ae7b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654ae7b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcae7b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818ae7be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fae7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ae7c7, 90 },
    { 0xba0ae7c7, 93 },
    { 0x408ae7c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecae7cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeae7d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faae7d9, 23 },
    { 0x9abae7d9, 33 },
    { 0x371ae7db, 52 },
    { 0xd5fae7dc, 66 },
    { 0, 0 },
    { 0xb21ae7de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efae7e5, 93 },
    { 0x761ae7e6, 55 },
    { 0x89dae7e5, 91 },
    { 0xc0fae7e5, 76 },
    { 0x2dcae7e8, 101 },
    { 0x9c7ae7ea, 68 },
    { 0xc1aae7e5, 61 },
    { 0x556ae7e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ae7ef, 67 },
    { 0x620ae7f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412ae7f4, 34 },
    { 0xde4ae7f5, 46 },
    { 0x80fae7f5, 90 },
    { 0, 0 },
    { 0xbcbae7f8, 78 },
    { 0, 0 },
    { 0x204ae7fa, 90 },
    { 0xb72ae7fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ae7ff, 65 },
    { 0, 0 },
    { 0x33aae801, 95 },
    { 0x21aae802, 90 },
    { 0x4faae803, 90 },
    { 0xc6ae803, 43 },
    { 0xf1eae801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253ae811, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2ae814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0ae817, 78 },
    { 0x55dae818, 78 },
    { 0xa0ae819, 34 },
    { 0, 0 },
    { 0x493ae81b, 98 },
    { 0, 0 },
    { 0x192ae81d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bae821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13ae826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548ae82a, 23 },
    { 0xa97ae82b, 90 },
    { 0x915ae82c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37ae830, 33 },
    { 0, 0 },
    { 0x6f8ae832, 78 },
    { 0, 0 },
    { 0x70aae834, 23 },
    { 0, 0 },
    { 0x104ae836, 40 },
    { 0, 0 },
    { 0xdbaae838, 90 },
    { 0x82cae838, 95 },
    { 0x2e9ae838, 23 },
    { 0xa99ae83b, 33 },
    { 0, 0 },
    { 0xed7ae83d, 33 },
    { 0xe2eae83e, 90 },
    { 0x9eeae83f, 33 },
    { 0x6bae83d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ae848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbae84b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ae851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cae856, 46 },
    { 0xbc8ae856, 33 },
    { 0, 0 },
    { 0xba5ae859, 98 },
    { 0xe35ae85a, 61 },
    { 0x99cae85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fae866, 33 },
    { 0, 0 },
    { 0x800ae868, 85 },
    { 0xd72ae869, 90 },
    { 0, 0 },
    { 0x9bae86b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ae872, 33 },
    { 0, 0 },
    { 0x46cae874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98ae877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9ae87a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x515ae87d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eae881, 90 },
    { 0, 0 },
    { 0xb3fae883, 55 },
    { 0, 0 },
    { 0x22bae885, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7ae88a, 34 },
    { 0x50fae88b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae88e, 55 },
    { 0, 0 },
    { 0x616ae890, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbae894, 25 },
    { 0, 0 },
    { 0xcefae896, 23 },
    { 0, 0 },
    { 0x567ae898, 98 },
    { 0x94cae899, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ae8a1, 43 },
    { 0, 0 },
    { 0x3b7ae8a3, 98 },
    { 0xbc9ae8a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bae8a8, 76 },
    { 0xd81ae8a9, 95 },
    { 0, 0 },
    { 0xc60ae8ab, 23 },
    { 0xe16ae8ab, 33 },
    { 0x5a3ae8ad, 61 },
    { 0x18fae8ac, 34 },
    { 0x277ae8ab, 76 },
    { 0, 0 },
    { 0x70eae8b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ae8b5, 90 },
    { 0xbefae8b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612ae8be, 93 },
    { 0, 0 },
    { 0xd50ae8c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ae8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cae8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6ae8ce, 23 },
    { 0x813ae8cf, 98 },
    { 0x930ae8d0, 33 },
    { 0xfb9ae8d1, 34 },
    { 0x2fdae8d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261ae8dd, 23 },
    { 0x6a9ae8dd, 78 },
    { 0x2e4ae8dd, 33 },
    { 0xbcfae8e0, 84 },
    { 0x931ae8e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38ae8e4, 87 },
    { 0x56fae8e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ae8e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae8ec, 87 },
    { 0x732ae8ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5ae8f4, 33 },
    { 0x683ae8f5, 33 },
    { 0x453ae8f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ae8fa, 98 },
    { 0x42fae8fb, 52 },
    { 0xccbae8fb, 23 },
    { 0x38aae8fd, 90 },
    { 0x99bae8fe, 81 },
    { 0, 0 },
    { 0x949ae900, 52 },
    { 0x490ae901, 46 },
    { 0, 0 },
    { 0x607ae903, 33 },
    { 0, 0 },
    { 0x603ae905, 98 },
    { 0xf47ae906, 90 },
    { 0xf5dae907, 90 },
    { 0xf44ae908, 98 },
    { 0x516ae905, 98 },
    { 0xc0eae90a, 55 },
    { 0x96fae90b, 78 },
    { 0xfc6ae90c, 23 },
    { 0, 0 },
    { 0xf37ae90e, 56 },
    { 0xd3cae90f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980ae914, 23 },
    { 0x77fae915, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ae918, 78 },
    { 0xb26ae918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffae91f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cae923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91ae929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742ae92d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ae930, 55 },
    { 0x1bfae931, 33 },
    { 0xab4ae931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7ae939, 90 },
    { 0x174ae93a, 90 },
    { 0x477ae939, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28ae93e, 61 },
    { 0x8aaae93f, 77 },
    { 0xcd4ae93e, 69 },
    { 0x18cae941, 76 },
    { 0x134ae942, 33 },
    { 0x948ae942, 61 },
    { 0x61dae944, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dae947, 90 },
    { 0x88bae948, 90 },
    { 0x277ae948, 23 },
    { 0xd1cae948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1aae953, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeae958, 95 },
    { 0xd2bae958, 90 },
    { 0xd77ae95a, 61 },
    { 0, 0 },
    { 0x2a3ae95c, 90 },
    { 0, 0 },
    { 0xdf1ae95e, 61 },
    { 0, 0 },
    { 0xf81ae960, 33 },
    { 0, 0 },
    { 0x9feae962, 33 },
    { 0xadaae963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9ae968, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aae96c, 23 },
    { 0, 0 },
    { 0x541ae96e, 90 },
    { 0x210ae96f, 29 },
    { 0xcfae96f, 23 },
    { 0x5daae971, 61 },
    { 0xd84ae972, 66 },
    { 0x2dcae973, 46 },
    { 0x553ae96f, 23 },
    { 0x9b5ae975, 98 },
    { 0x27aae971, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8ae97a, 17 },
    { 0xdb5ae97a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06ae985, 76 },
    { 0xf78ae986, 78 },
    { 0, 0 },
    { 0x349ae988, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dae98b, 91 },
    { 0, 0 },
    { 0x6edae98d, 84 },
    { 0xb25ae98d, 95 },
    { 0x524ae98f, 76 },
    { 0x125ae98f, 90 },
    { 0, 0 },
    { 0x8deae992, 23 },
    { 0x81fae993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651ae998, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ae99b, 52 },
    { 0x796ae99b, 34 },
    { 0x5f9ae99d, 43 },
    { 0xbb8ae99e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ae9a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6ae9a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2ae9af, 90 },
    { 0x56fae9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ae9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x589ae9b6, 90 },
    { 0x2dfae9b6, 33 },
    { 0x72eae9b7, 55 },
    { 0xbe5ae9b6, 55 },
    { 0x765ae9ba, 55 },
    { 0x170ae9bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8ae9c1, 90 },
    { 0x235ae9c2, 33 },
    { 0xcd2ae9c2, 95 },
    { 0xd9cae9c4, 76 },
    { 0xce4ae9c5, 61 },
    { 0x6ccae9c4, 98 },
    { 0x81aae9c7, 34 },
    { 0x1ddae9c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cae9cc, 97 },
    { 0x6ceae9cd, 36 },
    { 0, 0 },
    { 0x8f3ae9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807ae9d9, 56 },
    { 0, 0 },
    { 0x383ae9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ae9df, 85 },
    { 0x100ae9e0, 61 },
    { 0x910ae9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eae9e4, 23 },
    { 0x166ae9e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19ae9eb, 98 },
    { 0xa9dae9eb, 90 },
    { 0x75dae9ed, 84 },
    { 0xe67ae9eb, 23 },
    { 0x31cae9ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99ae9f2, 90 },
    { 0, 0 },
    { 0x896ae9f4, 66 },
    { 0x739ae9f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ae9f8, 91 },
    { 0x51cae9f9, 93 },
    { 0x777ae9f9, 95 },
    { 0, 0 },
    { 0x7baae9fc, 40 },
    { 0xeffae9fd, 34 },
    { 0, 0 },
    { 0xfe3ae9ff, 33 },
    { 0xe5eaea00, 98 },
    { 0x9cfae9ff, 61 },
    { 0, 0 },
    { 0x136aea03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccaea06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9aea0f, 43 },
    { 0xed3aea10, 63 },
    { 0x2c5aea11, 67 },
    { 0xc10aea10, 61 },
    { 0x807aea13, 55 },
    { 0x126aea10, 90 },
    { 0xa42aea14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976aea1c, 99 },
    { 0, 0 },
    { 0xa7eaea1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82daea22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeaea2c, 93 },
    { 0x75faea2c, 80 },
    { 0x901aea2e, 98 },
    { 0x205aea2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60aea33, 98 },
    { 0xadbaea34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbaea37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36aea3a, 76 },
    { 0xd5aaea3a, 95 },
    { 0xfc3aea3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209aea40, 43 },
    { 0, 0 },
    { 0x6e1aea42, 90 },
    { 0, 0 },
    { 0xdecaea44, 55 },
    { 0xf0eaea45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269aea4a, 52 },
    { 0x6b1aea4b, 23 },
    { 0xf22aea4a, 75 },
    { 0xb49aea4d, 78 },
    { 0xa15aea4d, 95 },
    { 0x21eaea4f, 95 },
    { 0xba8aea50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60caea55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921aea5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3aea5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3aea64, 91 },
    { 0x9a5aea64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daea71, 98 },
    { 0xfdcaea72, 90 },
    { 0, 0 },
    { 0x45aaea74, 33 },
    { 0x3d6aea74, 33 },
    { 0x8bbaea76, 95 },
    { 0xfbaea74, 61 },
    { 0x5feaea78, 33 },
    { 0x970aea79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38caea7c, 66 },
    { 0x600aea7d, 81 },
    { 0x2afaea7d, 95 },
    { 0x8c4aea7f, 85 },
    { 0x667aea7f, 33 },
    { 0x782aea81, 83 },
    { 0x64daea81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aea88, 78 },
    { 0, 0 },
    { 0xfa2aea8a, 46 },
    { 0, 0 },
    { 0xa07aea8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2aea90, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73caea96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x944aea99, 87 },
    { 0x485aea99, 76 },
    { 0xea7aea9a, 90 },
    { 0xc2faea9c, 23 },
    { 0, 0 },
    { 0x285aea9e, 33 },
    { 0x3efaea9f, 90 },
    { 0x139aea9e, 33 },
    { 0, 0 },
    { 0x855aeaa2, 61 },
    { 0, 0 },
    { 0x567aeaa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9baeaab, 33 },
    { 0x8a0aeaab, 95 },
    { 0xa37aeaad, 88 },
    { 0x568aeaad, 34 },
    { 0x531aeaae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaeab4, 33 },
    { 0x16daeab4, 90 },
    { 0, 0 },
    { 0xbdaaeab7, 33 },
    { 0, 0 },
    { 0x6d6aeab9, 76 },
    { 0x7c8aeaba, 90 },
    { 0xff5aeabb, 94 },
    { 0x65daeaba, 52 },
    { 0xd61aeabd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaaeac1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570aeac5, 33 },
    { 0x3ecaeac6, 99 },
    { 0xbc3aeac7, 98 },
    { 0xb3aeac8, 23 },
    { 0xf0daeac5, 98 },
    { 0xbbcaeaca, 33 },
    { 0, 0 },
    { 0x354aeacc, 33 },
    { 0, 0 },
    { 0x634aeace, 33 },
    { 0x212aeace, 99 },
    { 0x869aead0, 55 },
    { 0x75daead1, 90 },
    { 0x7f3aead2, 95 },
    { 0x361aeace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223aead8, 33 },
    { 0, 0 },
    { 0xbb9aeada, 90 },
    { 0xd69aeadb, 33 },
    { 0x7f8aeadc, 91 },
    { 0, 0 },
    { 0x874aeade, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1aaeae7, 76 },
    { 0, 0 },
    { 0xdcbaeae9, 95 },
    { 0x6e0aeaea, 38 },
    { 0xab1aeaeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756aeaf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdaeaf8, 55 },
    { 0, 0 },
    { 0x895aeafa, 90 },
    { 0x14eaeafb, 23 },
    { 0xfa1aeafa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6aeb00, 55 },
    { 0x98faeb01, 52 },
    { 0xe97aeb02, 34 },
    { 0xd7faeb03, 77 },
    { 0xc5baeb03, 22 },
    { 0x1e7aeb05, 90 },
    { 0x545aeb05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3aeb0e, 30 },
    { 0, 0 },
    { 0xe60aeb10, 55 },
    { 0x397aeb11, 98 },
    { 0x6b4aeb12, 61 },
    { 0, 0 },
    { 0xdf5aeb14, 46 },
    { 0x9fbaeb15, 65 },
    { 0xfb3aeb14, 61 },
    { 0x96aaeb17, 100 },
    { 0xe2eaeb18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8aeb1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9aeb1f, 98 },
    { 0, 0 },
    { 0xf26aeb21, 23 },
    { 0x7c4aeb21, 90 },
    { 0x4aeaeb23, 33 },
    { 0x382aeb24, 90 },
    { 0x2a2aeb24, 33 },
    { 0xfe0aeb23, 90 },
    { 0, 0 },
    { 0xba4aeb28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6baeb2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02aeb2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5aeb35, 98 },
    { 0x303aeb36, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8aeb39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7aeb3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6aeb3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x288aeb42, 90 },
    { 0x259aeb42, 85 },
    { 0x88aaeb44, 33 },
    { 0x89aaeb45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7aeb4b, 90 },
    { 0x3eeaeb4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6aeb50, 19 },
    { 0xa67aeb51, 55 },
    { 0x34aeb52, 61 },
    { 0x146aeb53, 90 },
    { 0x5f0aeb53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823aeb59, 81 },
    { 0x123aeb5a, 33 },
    { 0xd58aeb5b, 33 },
    { 0xe88aeb5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1aeb61, 90 },
    { 0x26aeb62, 61 },
    { 0xd4baeb62, 33 },
    { 0x67baeb64, 93 },
    { 0x668aeb64, 95 },
    { 0xf1aeb63, 33 },
    { 0xc46aeb61, 17 },
    { 0xcbaaeb63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770aeb6d, 90 },
    { 0, 0 },
    { 0x80daeb6f, 43 },
    { 0x5e4aeb6f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20faeb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0aeb79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6caeb81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91faeb85, 90 },
    { 0xdf7aeb86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdaeb8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5aeb90, 95 },
    { 0, 0 },
    { 0xd5aeb92, 90 },
    { 0x1b8aeb93, 46 },
    { 0xd4daeb94, 61 },
    { 0, 0 },
    { 0xbd7aeb96, 91 },
    { 0, 0 },
    { 0x1f9aeb98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238aeb9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1aeb9f, 96 },
    { 0xdeaaeb9f, 95 },
    { 0x683aeba1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x352aeba4, 90 },
    { 0, 0 },
    { 0xe9daeba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74aebaa, 90 },
    { 0xab0aebab, 98 },
    { 0x94aebac, 95 },
    { 0x2f5aebad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1aebb4, 17 },
    { 0x316aebb5, 98 },
    { 0xf7faebb6, 84 },
    { 0xf3eaebb7, 90 },
    { 0x55faebb8, 67 },
    { 0x247aebb9, 96 },
    { 0x2e7aebb8, 95 },
    { 0x9ebaebbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaebc0, 90 },
    { 0, 0 },
    { 0xe33aebc2, 90 },
    { 0xe87aebc2, 34 },
    { 0, 0 },
    { 0x1d2aebc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedaebc9, 93 },
    { 0x3f5aebc9, 75 },
    { 0xecdaebcb, 98 },
    { 0x18faebcc, 90 },
    { 0, 0 },
    { 0xd83aebce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39aebd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffaebd6, 98 },
    { 0xe9aebd7, 61 },
    { 0xe90aebd7, 33 },
    { 0x23aebd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110aebde, 52 },
    { 0, 0 },
    { 0xb50aebe0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7aebe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142aebe7, 98 },
    { 0xd10aebe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcaebed, 61 },
    { 0xb96aebee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95aebf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x401aebf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44aec06, 74 },
    { 0x86aec07, 23 },
    { 0xa04aec08, 23 },
    { 0x509aec09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27daec0c, 95 },
    { 0x7feaec0c, 52 },
    { 0x24eaec0e, 90 },
    { 0x173aec0e, 55 },
    { 0, 0 },
    { 0x23aec11, 23 },
    { 0x6daaec11, 23 },
    { 0, 0 },
    { 0x81daec14, 34 },
    { 0x5eaec15, 90 },
    { 0x3f8aec14, 90 },
    { 0, 0 },
    { 0x817aec18, 95 },
    { 0, 0 },
    { 0x938aec1a, 95 },
    { 0xb5aaec1a, 12 },
    { 0xc2aec1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aaec26, 99 },
    { 0, 0 },
    { 0xb7aaec28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116aec30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baec33, 65 },
    { 0x28faec34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecaec37, 36 },
    { 0x9c7aec38, 96 },
    { 0xa1aaec39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699aec3f, 99 },
    { 0xf64aec40, 55 },
    { 0x69aaec3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eaec44, 23 },
    { 0x937aec45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391aec4a, 87 },
    { 0, 0 },
    { 0x194aec4c, 33 },
    { 0xee8aec4d, 33 },
    { 0x98faec4d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282aec58, 23 },
    { 0x413aec59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2aec5c, 33 },
    { 0xb8aaec5c, 93 },
    { 0x1c1aec5c, 61 },
    { 0x744aec5f, 17 },
    { 0xc4daec5f, 75 },
    { 0x1faaec5e, 98 },
    { 0x61daec62, 33 },
    { 0x4d0aec60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aec67, 66 },
    { 0xfadaec68, 46 },
    { 0, 0 },
    { 0x524aec6a, 93 },
    { 0x27baec6b, 34 },
    { 0, 0 },
    { 0x3b5aec6d, 76 },
    { 0x496aec6e, 77 },
    { 0xfa4aec6f, 9 },
    { 0xfa7aec6d, 96 },
    { 0x464aec71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859aec77, 33 },
    { 0xa63aec77, 61 },
    { 0xd17aec78, 61 },
    { 0, 0 },
    { 0x68eaec7b, 81 },
    { 0xfe1aec7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4aec81, 95 },
    { 0x523aec82, 33 },
    { 0xb0aaec82, 91 },
    { 0xf64aec84, 95 },
    { 0xdc9aec83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aaec88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30aec8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccaec8f, 90 },
    { 0x26baec90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aec93, 43 },
    { 0xf44aec93, 23 },
    { 0, 0 },
    { 0x2e6aec96, 61 },
    { 0x259aec96, 98 },
    { 0x226aec96, 61 },
    { 0xc60aec99, 76 },
    { 0xc92aec99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20aec9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5aeca2, 81 },
    { 0x77daeca2, 87 },
    { 0, 0 },
    { 0xe75aeca5, 55 },
    { 0, 0 },
    { 0x8edaeca7, 33 },
    { 0, 0 },
    { 0x8c0aeca9, 95 },
    { 0x7d6aeca9, 90 },
    { 0xe0daecab, 73 },
    { 0xaefaecab, 65 },
    { 0x8d0aecab, 55 },
    { 0x325aecab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36daecb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81baecb8, 75 },
    { 0, 0 },
    { 0xffaaecba, 17 },
    { 0xc47aecbb, 99 },
    { 0x6f2aecbc, 23 },
    { 0x650aecbd, 69 },
    { 0x173aecbb, 95 },
    { 0x68daecbf, 52 },
    { 0xeeaaecbc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602aecc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943aecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16aecd1, 85 },
    { 0x498aecd2, 23 },
    { 0x916aecd3, 90 },
    { 0x88caecd1, 91 },
    { 0, 0 },
    { 0x834aecd6, 36 },
    { 0xc91aecd7, 33 },
    { 0xc0daecd7, 61 },
    { 0xda0aecd9, 91 },
    { 0x1daecda, 53 },
    { 0x758aecda, 61 },
    { 0, 0 },
    { 0xbeaecdd, 66 },
    { 0xdbaaecde, 33 },
    { 0x83aecdf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4aece2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x25caece5, 68 },
    { 0x988aece6, 33 },
    { 0x957aece7, 61 },
    { 0xde3aece5, 93 },
    { 0xa3daece5, 34 },
    { 0x616aecea, 66 },
    { 0x1d4aece5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x993aecee, 93 },
    { 0x8d4aecee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825aecf5, 95 },
    { 0, 0 },
    { 0x3b2aecf7, 38 },
    { 0, 0 },
    { 0x351aecf9, 95 },
    { 0xa11aecfa, 90 },
    { 0x7dfaecfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9daed05, 33 },
    { 0x703aed06, 90 },
    { 0xf4caed06, 17 },
    { 0x43faed07, 90 },
    { 0x81aed06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355aed0d, 33 },
    { 0xb18aed0e, 95 },
    { 0x144aed0f, 68 },
    { 0x672aed10, 69 },
    { 0xad1aed10, 33 },
    { 0, 0 },
    { 0xb0daed13, 98 },
    { 0, 0 },
    { 0xdf0aed15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2aed1c, 55 },
    { 0, 0 },
    { 0x1d5aed1e, 34 },
    { 0x8faed1e, 34 },
    { 0xf9daed20, 23 },
    { 0xd34aed20, 90 },
    { 0, 0 },
    { 0xd9caed23, 43 },
    { 0xa4aed24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x422aed27, 23 },
    { 0xa64aed28, 33 },
    { 0x9c7aed27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbaed2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9baed30, 77 },
    { 0x51baed31, 90 },
    { 0x51daed30, 61 },
    { 0x41baed33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829aed37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66aed3a, 61 },
    { 0, 0 },
    { 0x615aed3c, 87 },
    { 0x5bcaed3d, 98 },
    { 0xeecaed3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x16aaed41, 90 },
    { 0x311aed42, 61 },
    { 0, 0 },
    { 0x9d3aed44, 98 },
    { 0xf3eaed44, 33 },
    { 0x82eaed45, 87 },
    { 0x35daed45, 90 },
    { 0xc31aed48, 77 },
    { 0x187aed49, 98 },
    { 0x7ddaed4a, 95 },
    { 0x382aed4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98aed4e, 98 },
    { 0, 0 },
    { 0x5f3aed50, 55 },
    { 0, 0 },
    { 0x53caed52, 81 },
    { 0x200aed52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0baed56, 61 },
    { 0x385aed57, 61 },
    { 0xb6baed56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41aed60, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46aed63, 23 },
    { 0, 0 },
    { 0x9d0aed65, 95 },
    { 0, 0 },
    { 0x457aed67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8daed6b, 90 },
    { 0xd8daed6c, 66 },
    { 0xf37aed6d, 78 },
    { 0xcaaaed6c, 33 },
    { 0x6c8aed6d, 43 },
    { 0, 0 },
    { 0x76baed71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1aed75, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4aed7c, 76 },
    { 0xcb1aed7d, 52 },
    { 0, 0 },
    { 0xd32aed7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceaed84, 90 },
    { 0, 0 },
    { 0x3faaed86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdaed89, 66 },
    { 0xd03aed89, 67 },
    { 0, 0 },
    { 0xcf6aed8c, 90 },
    { 0, 0 },
    { 0xfcfaed8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccaed91, 23 },
    { 0, 0 },
    { 0x62eaed93, 34 },
    { 0x1bdaed93, 95 },
    { 0x870aed94, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0baed9a, 90 },
    { 0xaf1aed9b, 55 },
    { 0x8c1aed9c, 98 },
    { 0xa10aed9c, 43 },
    { 0x7c5aed9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8aeda9, 55 },
    { 0xaf2aedaa, 68 },
    { 0x26eaedaa, 61 },
    { 0x5e7aedac, 95 },
    { 0x774aedac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daedb0, 61 },
    { 0, 0 },
    { 0x233aedb2, 90 },
    { 0, 0 },
    { 0xf6baedb4, 61 },
    { 0x8d6aedb5, 78 },
    { 0x9b3aedb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdaedc0, 43 },
    { 0xec0aedc0, 90 },
    { 0, 0 },
    { 0x700aedc3, 61 },
    { 0, 0 },
    { 0x428aedc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7baedc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x521aedcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2aedda, 66 },
    { 0xd3aaedda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcaede0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caede8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eaeded, 98 },
    { 0xb04aeded, 100 },
    { 0, 0 },
    { 0x1b9aedf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645aedf4, 32 },
    { 0x2cdaedf4, 23 },
    { 0, 0 },
    { 0xd9daedf7, 95 },
    { 0xd99aedf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x914aedfb, 68 },
    { 0xe71aedfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64aee00, 95 },
    { 0x884aee01, 90 },
    { 0xbd5aee02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1aee0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00aee0e, 34 },
    { 0x34aee0e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x72baee12, 46 },
    { 0x84aee12, 93 },
    { 0x355aee14, 56 },
    { 0, 0 },
    { 0x114aee16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eaee1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646aee24, 98 },
    { 0x1aee25, 23 },
    { 0x63caee26, 90 },
    { 0, 0 },
    { 0x245aee28, 23 },
    { 0xb06aee29, 43 },
    { 0x3bfaee29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824aee2f, 61 },
    { 0x43caee30, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18faee33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5aee36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4aee3d, 95 },
    { 0, 0 },
    { 0xba7aee3f, 23 },
    { 0, 0 },
    { 0x971aee41, 76 },
    { 0xd28aee42, 98 },
    { 0x146aee43, 61 },
    { 0x9cfaee43, 77 },
    { 0xd5faee43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5caee48, 95 },
    { 0, 0 },
    { 0x4eaaee4a, 64 },
    { 0, 0 },
    { 0xcc2aee4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdaee4f, 33 },
    { 0, 0 },
    { 0xeb5aee51, 77 },
    { 0, 0 },
    { 0x957aee53, 61 },
    { 0, 0 },
    { 0x63faee55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3aee59, 98 },
    { 0, 0 },
    { 0xb82aee5b, 78 },
    { 0, 0 },
    { 0xbdaaee5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0aee64, 76 },
    { 0x3e6aee65, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70aee69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9faee6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aaee71, 90 },
    { 0x4f4aee72, 98 },
    { 0xe62aee72, 99 },
    { 0xfbaaee74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250aee7b, 90 },
    { 0, 0 },
    { 0x5aee7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x810aee80, 33 },
    { 0x324aee80, 61 },
    { 0xbbaee82, 91 },
    { 0x19eaee83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598aee88, 52 },
    { 0, 0 },
    { 0x59caee8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceaee90, 46 },
    { 0x66aee90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34baee9a, 93 },
    { 0x5f0aee9a, 90 },
    { 0xffaaee9c, 33 },
    { 0, 0 },
    { 0x643aee9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34aeea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceaeeaa, 23 },
    { 0x662aeeaa, 55 },
    { 0x71caeeaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53aeeb3, 55 },
    { 0, 0 },
    { 0x111aeeb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baaeeb9, 95 },
    { 0x5faaeeba, 76 },
    { 0, 0 },
    { 0xbe3aeebc, 33 },
    { 0x562aeebd, 51 },
    { 0x718aeebe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7daeec4, 33 },
    { 0x12eaeec5, 89 },
    { 0x49baeec4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceaeed0, 61 },
    { 0, 0 },
    { 0xedcaeed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aaeed7, 93 },
    { 0, 0 },
    { 0x4fdaeed9, 76 },
    { 0x17aeed9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30faeedf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfaeee4, 23 },
    { 0x2cbaeee4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597aeee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652aeeee, 17 },
    { 0x8abaeeee, 43 },
    { 0x53daeeee, 76 },
    { 0, 0 },
    { 0xa7eaeef2, 95 },
    { 0xcf5aeef3, 90 },
    { 0, 0 },
    { 0xc1aeef5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718aeefb, 23 },
    { 0x2efaeefc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78faef00, 98 },
    { 0x190aef01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aef04, 95 },
    { 0, 0 },
    { 0x42faef06, 33 },
    { 0x3fbaef07, 81 },
    { 0x342aef08, 33 },
    { 0, 0 },
    { 0xa9aef0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaef0e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554aef15, 33 },
    { 0x89aef15, 33 },
    { 0, 0 },
    { 0xe76aef18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfaef1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9aef1f, 66 },
    { 0xfcdaef20, 46 },
    { 0x993aef1f, 84 },
    { 0x2cdaef22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0eaef26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50daef2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aaef35, 56 },
    { 0xaa8aef36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x891aef39, 95 },
    { 0, 0 },
    { 0x4ccaef3b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0aef40, 61 },
    { 0x51baef40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeaef48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9aef4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbaef4e, 40 },
    { 0, 0 },
    { 0xa5aef50, 61 },
    { 0x591aef51, 69 },
    { 0, 0 },
    { 0x75daef53, 98 },
    { 0xdefaef53, 33 },
    { 0x2aef53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62aef58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9aef5b, 98 },
    { 0x94baef5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96aef5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199aef65, 43 },
    { 0, 0 },
    { 0x40aaef67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49daef6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcaef6f, 56 },
    { 0, 0 },
    { 0x91eaef71, 53 },
    { 0xd7aef72, 90 },
    { 0xa46aef73, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5aef7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55aef7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eaef8b, 95 },
    { 0, 0 },
    { 0xf2eaef8d, 78 },
    { 0, 0 },
    { 0x846aef8f, 23 },
    { 0xc77aef90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1aef96, 33 },
    { 0x14aef97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aef9e, 93 },
    { 0x6e4aef9e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54caefa5, 84 },
    { 0x16aefa6, 98 },
    { 0, 0 },
    { 0x48aefa8, 100 },
    { 0x88aaefa9, 43 },
    { 0x9cfaefa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29aefad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baefb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6aefb5, 100 },
    { 0xc68aefb6, 33 },
    { 0x77caefb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860aefba, 90 },
    { 0, 0 },
    { 0x59aaefbc, 33 },
    { 0xe19aefbd, 98 },
    { 0x954aefbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeaefc1, 61 },
    { 0x21caefc1, 17 },
    { 0, 0 },
    { 0x729aefc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7aefca, 33 },
    { 0, 0 },
    { 0x9fcaefcc, 85 },
    { 0, 0 },
    { 0x71caefce, 98 },
    { 0, 0 },
    { 0x159aefd0, 90 },
    { 0x64daefd0, 93 },
    { 0x158aefd1, 90 },
    { 0x5bcaefd2, 83 },
    { 0xca3aefd4, 97 },
    { 0xce7aefd4, 33 },
    { 0xd31aefd6, 33 },
    { 0, 0 },
    { 0xb7daefd8, 52 },
    { 0x1b5aefd9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3aefea, 76 },
    { 0, 0 },
    { 0x862aefec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfaeff4, 43 },
    { 0x53caeff5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x576aeff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990aeffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eaefff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6af003, 99 },
    { 0, 0 },
    { 0x1f1af005, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5af009, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258af00e, 23 },
    { 0, 0 },
    { 0x9a4af010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65baf013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478af017, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68baf01b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4af021, 78 },
    { 0xf13af022, 19 },
    { 0xabbaf022, 61 },
    { 0xd14af021, 52 },
    { 0x298af025, 52 },
    { 0xed4af025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1af02b, 33 },
    { 0, 0 },
    { 0xf6aaf02d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4af033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfaf036, 66 },
    { 0, 0 },
    { 0x945af038, 33 },
    { 0, 0 },
    { 0xef0af03a, 98 },
    { 0xcf1af03b, 90 },
    { 0, 0 },
    { 0xcb7af03d, 33 },
    { 0, 0 },
    { 0x69caf03f, 90 },
    { 0xdf0af040, 84 },
    { 0xb9caf03f, 90 },
    { 0x295af041, 23 },
    { 0, 0 },
    { 0x36daf044, 43 },
    { 0x284af044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34af050, 76 },
    { 0x813af050, 90 },
    { 0x582af050, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41af055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989af05e, 90 },
    { 0x7acaf05f, 55 },
    { 0xbfaaf060, 76 },
    { 0xe7aaf05e, 33 },
    { 0xc6af062, 33 },
    { 0, 0 },
    { 0x879af064, 17 },
    { 0x45faf064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x642af068, 98 },
    { 0xe19af069, 91 },
    { 0xaefaf069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82baf06e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65daf072, 53 },
    { 0x197af073, 33 },
    { 0xc84af074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80baf079, 95 },
    { 0x773af07a, 100 },
    { 0xedaf079, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468af080, 66 },
    { 0x1f0af081, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36af084, 95 },
    { 0xfcaaf085, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abaf088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaaf08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7af08f, 33 },
    { 0, 0 },
    { 0x90faf091, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af094, 61 },
    { 0x524af094, 90 },
    { 0x225af096, 33 },
    { 0x70af097, 66 },
    { 0, 0 },
    { 0x1bdaf099, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75af09c, 66 },
    { 0x273af09c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9af0a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eaf0a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16faf0ac, 33 },
    { 0, 0 },
    { 0xd77af0ae, 98 },
    { 0x2f4af0ae, 90 },
    { 0x956af0b0, 78 },
    { 0xd70af0b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eaf0b4, 61 },
    { 0xc88af0b5, 100 },
    { 0x6f7af0b6, 76 },
    { 0, 0 },
    { 0xe11af0b8, 61 },
    { 0xa26af0b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eaf0be, 33 },
    { 0xd90af0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beaf0c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0faf0ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72af0cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aaf0d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x563af0d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43af0d8, 98 },
    { 0x6e7af0d9, 85 },
    { 0x588af0d8, 46 },
    { 0, 0 },
    { 0x512af0dc, 33 },
    { 0x369af0dc, 90 },
    { 0x514af0de, 95 },
    { 0xbaeaf0df, 95 },
    { 0x2b9af0e0, 34 },
    { 0, 0 },
    { 0xc96af0e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdaf0e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90af0e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94af0ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89af0f2, 55 },
    { 0x3a5af0f3, 90 },
    { 0x6daf0f4, 98 },
    { 0xbedaf0f5, 98 },
    { 0x79baf0f6, 96 },
    { 0xde8af0f7, 33 },
    { 0x4d0af0f6, 98 },
    { 0xfa3af0f9, 76 },
    { 0xe0af0fa, 34 },
    { 0xd33af0fa, 87 },
    { 0xdd4af0f4, 66 },
    { 0x692af0f8, 66 },
    { 0x71faf0fe, 93 },
    { 0x36eaf0ff, 95 },
    { 0x790af100, 90 },
    { 0x433af101, 75 },
    { 0xeeeaf101, 82 },
    { 0xdd7af100, 98 },
    { 0x99baf0fd, 93 },
    { 0, 0 },
    { 0x753af106, 61 },
    { 0, 0 },
    { 0xe51af108, 84 },
    { 0xb31af109, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6af10e, 90 },
    { 0xa60af10e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266af113, 61 },
    { 0x746af114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444af118, 76 },
    { 0x177af119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0af11c, 34 },
    { 0, 0 },
    { 0x5ecaf11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4af121, 38 },
    { 0x270af121, 24 },
    { 0xddbaf123, 81 },
    { 0, 0 },
    { 0x816af125, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413af12c, 33 },
    { 0, 0 },
    { 0x4caf12e, 78 },
    { 0xb8aaf12e, 99 },
    { 0x3e6af130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x168af133, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdaf137, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5af13a, 64 },
    { 0x157af13b, 95 },
    { 0xca4af13c, 55 },
    { 0x156af13d, 98 },
    { 0x184af13e, 88 },
    { 0xb74af13f, 55 },
    { 0x4eaaf140, 33 },
    { 0x6ffaf141, 33 },
    { 0x121af142, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aaf148, 75 },
    { 0xa18af149, 61 },
    { 0xa41af148, 95 },
    { 0x43af14b, 52 },
    { 0xfc5af14c, 33 },
    { 0xc70af14d, 55 },
    { 0, 0 },
    { 0x58faf14f, 95 },
    { 0x563af150, 95 },
    { 0, 0 },
    { 0x13baf152, 84 },
    { 0xba1af153, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4af159, 65 },
    { 0x73baf15a, 23 },
    { 0xb57af159, 90 },
    { 0x785af15a, 61 },
    { 0x764af15a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853af165, 66 },
    { 0x575af166, 55 },
    { 0, 0 },
    { 0xfedaf168, 61 },
    { 0xee0af168, 55 },
    { 0, 0 },
    { 0x7ebaf16b, 77 },
    { 0x137af16c, 33 },
    { 0x2b9af16c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf170, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82af178, 61 },
    { 0x4e3af179, 34 },
    { 0x89aaf17a, 33 },
    { 0xc77af17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eaf17e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eaf181, 98 },
    { 0x4d8af182, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42af185, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4af18b, 77 },
    { 0xf06af18c, 90 },
    { 0x2a3af18d, 95 },
    { 0x5d6af18e, 90 },
    { 0, 0 },
    { 0x6eaaf190, 53 },
    { 0xc2af191, 78 },
    { 0x320af191, 100 },
    { 0, 0 },
    { 0xdc8af194, 23 },
    { 0, 0 },
    { 0xed2af196, 33 },
    { 0, 0 },
    { 0x4a1af198, 90 },
    { 0xeb1af199, 76 },
    { 0x60faf19a, 90 },
    { 0, 0 },
    { 0x650af19c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6af1a1, 33 },
    { 0, 0 },
    { 0x4d2af1a3, 51 },
    { 0x82daf1a4, 81 },
    { 0xc2baf1a5, 88 },
    { 0xc76af1a6, 61 },
    { 0, 0 },
    { 0xafaf1a8, 95 },
    { 0xf48af1a8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48baf1ac, 90 },
    { 0x6a6af1ad, 90 },
    { 0, 0 },
    { 0xa23af1af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155af1b4, 55 },
    { 0, 0 },
    { 0x1b1af1b6, 87 },
    { 0x88daf1b7, 46 },
    { 0x3af1b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69af1bc, 101 },
    { 0x3c0af1bc, 90 },
    { 0, 0 },
    { 0xc6caf1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddaf1c3, 98 },
    { 0x266af1c4, 34 },
    { 0, 0 },
    { 0xb09af1c6, 61 },
    { 0x610af1c6, 100 },
    { 0xd06af1c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aaf1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x351af1d0, 61 },
    { 0x734af1d0, 33 },
    { 0, 0 },
    { 0xc9caf1d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1af1d7, 90 },
    { 0, 0 },
    { 0x795af1d9, 43 },
    { 0x4e0af1d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0af1dd, 90 },
    { 0x1f8af1dd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7af1e1, 46 },
    { 0x1c7af1e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90af1e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91af1e9, 52 },
    { 0, 0 },
    { 0x48faf1eb, 33 },
    { 0, 0 },
    { 0xd55af1ed, 19 },
    { 0x3d8af1ee, 93 },
    { 0x758af1ef, 91 },
    { 0, 0 },
    { 0x5daaf1f1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703af1f5, 66 },
    { 0xd2daf1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5af1fd, 56 },
    { 0xdecaf1fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcaf201, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2af204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdaf209, 34 },
    { 0x545af209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baf20f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44af212, 78 },
    { 0xafeaf213, 23 },
    { 0xcc1af212, 90 },
    { 0xc11af214, 95 },
    { 0x4f9af215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749af21a, 67 },
    { 0x8f3af21b, 34 },
    { 0xcf7af21b, 65 },
    { 0xc1faf21d, 34 },
    { 0, 0 },
    { 0x9d6af21f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abaf222, 67 },
    { 0x64faf223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdaf227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aaf232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aaf241, 99 },
    { 0x854af241, 23 },
    { 0, 0 },
    { 0xa85af244, 33 },
    { 0xbd0af244, 99 },
    { 0x658af246, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1af249, 33 },
    { 0xb90af249, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703af252, 69 },
    { 0xf15af252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560af25f, 61 },
    { 0, 0 },
    { 0x3aeaf261, 55 },
    { 0, 0 },
    { 0x16daf263, 99 },
    { 0xfc2af264, 95 },
    { 0x40aaf265, 61 },
    { 0x3a1af266, 76 },
    { 0x1aaaf267, 46 },
    { 0x442af264, 52 },
    { 0xeffaf269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8af26d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7caf271, 93 },
    { 0, 0 },
    { 0x6abaf273, 95 },
    { 0xadfaf274, 76 },
    { 0x22caf275, 61 },
    { 0, 0 },
    { 0x3e1af277, 84 },
    { 0x7e9af277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7af280, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86af283, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70caf287, 84 },
    { 0x554af288, 95 },
    { 0xa65af289, 34 },
    { 0xc45af289, 33 },
    { 0x3a8af289, 98 },
    { 0xc37af28a, 77 },
    { 0, 0 },
    { 0x4fdaf28e, 78 },
    { 0, 0 },
    { 0xde0af290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81caf293, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7af297, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaaf29e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8af2a2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2af2a6, 23 },
    { 0x98af2a6, 23 },
    { 0xab7af2a6, 67 },
    { 0, 0 },
    { 0xe5caf2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23baf2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2af2b0, 43 },
    { 0xea8af2b0, 90 },
    { 0x89daf2b1, 33 },
    { 0x2fcaf2b2, 46 },
    { 0x709af2b3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0af2b7, 23 },
    { 0, 0 },
    { 0x44caf2b9, 23 },
    { 0xbdfaf2ba, 33 },
    { 0, 0 },
    { 0xad8af2bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00af2c0, 33 },
    { 0xa96af2c1, 34 },
    { 0, 0 },
    { 0xc03af2c3, 93 },
    { 0x5cdaf2c4, 90 },
    { 0x16eaf2c3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaaf2cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845af2d6, 61 },
    { 0xd57af2d7, 23 },
    { 0x178af2d8, 98 },
    { 0xb3faf2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6af2e1, 99 },
    { 0x29caf2e2, 87 },
    { 0x430af2e2, 93 },
    { 0x161af2e4, 66 },
    { 0x28faf2e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7caf2e8, 55 },
    { 0x1eaaf2e8, 78 },
    { 0, 0 },
    { 0xc48af2eb, 95 },
    { 0xce0af2eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbaf2ef, 61 },
    { 0x3a6af2ef, 95 },
    { 0, 0 },
    { 0x819af2f2, 95 },
    { 0xeefaf2f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x367af2f6, 33 },
    { 0xa5eaf2f6, 98 },
    { 0x556af2f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9af2ff, 76 },
    { 0, 0 },
    { 0x51baf301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3af30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0baf30e, 34 },
    { 0xe1eaf30f, 77 },
    { 0x8bdaf310, 55 },
    { 0x12caf311, 95 },
    { 0xaf1af312, 90 },
    { 0xed3af312, 98 },
    { 0xcebaf314, 46 },
    { 0x3faaf315, 67 },
    { 0, 0 },
    { 0x5ebaf317, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6faf31b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77baf327, 98 },
    { 0, 0 },
    { 0x93caf329, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26baf330, 23 },
    { 0xae5af331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242af336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859af343, 99 },
    { 0x875af343, 23 },
    { 0xbaeaf345, 61 },
    { 0, 0 },
    { 0x202af347, 98 },
    { 0x2ecaf348, 95 },
    { 0x2a5af349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x917af34c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5faf351, 95 },
    { 0x49aaf352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65af35b, 12 },
    { 0xc9faf35b, 87 },
    { 0x643af35d, 95 },
    { 0, 0 },
    { 0x465af35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6af367, 98 },
    { 0x83aaf368, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x958af36b, 33 },
    { 0x1ccaf36b, 61 },
    { 0, 0 },
    { 0x6dbaf36e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7af371, 34 },
    { 0xfb1af372, 23 },
    { 0x883af373, 55 },
    { 0x131af374, 90 },
    { 0x469af374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78af378, 33 },
    { 0x5e4af379, 98 },
    { 0x248af37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16af37f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24baf384, 90 },
    { 0xd08af385, 23 },
    { 0xeccaf385, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7af38a, 95 },
    { 0x5c9af38b, 66 },
    { 0xb51af38c, 66 },
    { 0x238af38c, 91 },
    { 0xdebaf38e, 46 },
    { 0xad9af38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0faf392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180af397, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x175af39a, 61 },
    { 0, 0 },
    { 0x9c4af39c, 90 },
    { 0, 0 },
    { 0xbd3af39e, 66 },
    { 0x8f3af39f, 61 },
    { 0x2faf3a0, 95 },
    { 0, 0 },
    { 0x736af3a2, 66 },
    { 0, 0 },
    { 0x47daf3a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831af3aa, 61 },
    { 0, 0 },
    { 0xfa8af3ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758af3b3, 90 },
    { 0x81daf3b4, 98 },
    { 0x60aaf3b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9af3b8, 43 },
    { 0, 0 },
    { 0x7aaf3ba, 33 },
    { 0x73daf3bb, 23 },
    { 0x47caf3bc, 98 },
    { 0x29af3bc, 95 },
    { 0x940af3be, 33 },
    { 0, 0 },
    { 0x654af3c0, 55 },
    { 0x52caf3c1, 85 },
    { 0, 0 },
    { 0xc36af3c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0af3c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701af3d1, 95 },
    { 0xf3baf3d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9af3d5, 33 },
    { 0, 0 },
    { 0x5dcaf3d7, 43 },
    { 0, 0 },
    { 0xffbaf3d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfaf3e4, 84 },
    { 0xa26af3e5, 40 },
    { 0x3b7af3e6, 93 },
    { 0x19caf3e7, 88 },
    { 0xcdbaf3e5, 95 },
    { 0x88aaf3e6, 33 },
    { 0x849af3ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583af3ee, 90 },
    { 0, 0 },
    { 0xf89af3f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3af3f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aaf3f7, 33 },
    { 0xed6af3f8, 61 },
    { 0xba4af3f8, 91 },
    { 0, 0 },
    { 0xad5af3fb, 55 },
    { 0x1aaf3fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86faf3ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981af406, 60 },
    { 0xd02af407, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659af40b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787af410, 76 },
    { 0xe40af410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54baf415, 46 },
    { 0x31baf416, 23 },
    { 0x2baf417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7faf41a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deaf41d, 88 },
    { 0, 0 },
    { 0xb1aaf41f, 23 },
    { 0x8c5af420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddaf423, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1af426, 23 },
    { 0x25aaf427, 84 },
    { 0, 0 },
    { 0xc31af429, 76 },
    { 0, 0 },
    { 0xddcaf42b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aaf431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceaf436, 36 },
    { 0x689af437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eaf43e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x442af441, 66 },
    { 0, 0 },
    { 0x177af443, 23 },
    { 0, 0 },
    { 0x710af445, 98 },
    { 0, 0 },
    { 0x3c5af447, 90 },
    { 0x53caf448, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92baf44f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x266af452, 23 },
    { 0xafaf453, 76 },
    { 0, 0 },
    { 0x42daf455, 90 },
    { 0x33af455, 95 },
    { 0, 0 },
    { 0x42daf458, 33 },
    { 0, 0 },
    { 0xd53af45a, 95 },
    { 0, 0 },
    { 0xa16af45c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76daf45f, 65 },
    { 0x6b4af460, 95 },
    { 0, 0 },
    { 0xea8af462, 66 },
    { 0x1b9af463, 23 },
    { 0x1e2af462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840af468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af46e, 23 },
    { 0, 0 },
    { 0x61caf470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5caf476, 95 },
    { 0xf1caf477, 17 },
    { 0x9c2af478, 90 },
    { 0xe1af479, 95 },
    { 0xff2af47a, 43 },
    { 0x28eaf47b, 90 },
    { 0x3e8af478, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602af482, 66 },
    { 0x1a8af482, 84 },
    { 0x525af484, 98 },
    { 0x398af484, 61 },
    { 0, 0 },
    { 0x4abaf487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8af48a, 23 },
    { 0xa1baf48b, 34 },
    { 0xac0af48b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21af48f, 52 },
    { 0xb72af490, 61 },
    { 0x155af491, 23 },
    { 0xc0daf492, 90 },
    { 0x36aaf490, 43 },
    { 0xd79af494, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41caf49a, 95 },
    { 0, 0 },
    { 0x60aaf49c, 34 },
    { 0xf1eaf49d, 40 },
    { 0x5baaf49e, 23 },
    { 0x75af49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9af4a2, 90 },
    { 0xdb2af4a2, 66 },
    { 0xfcdaf4a4, 23 },
    { 0xdebaf4a5, 23 },
    { 0, 0 },
    { 0x810af4a7, 87 },
    { 0, 0 },
    { 0x84daf4a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21af4ac, 23 },
    { 0x5c7af4ad, 93 },
    { 0x4ccaf4ae, 33 },
    { 0x9bcaf4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0af4b5, 90 },
    { 0, 0 },
    { 0x5d4af4b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35caf4ba, 85 },
    { 0, 0 },
    { 0x273af4bc, 33 },
    { 0, 0 },
    { 0xe9baf4be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x351af4c1, 95 },
    { 0, 0 },
    { 0x41caf4c3, 64 },
    { 0xaceaf4c4, 95 },
    { 0x55aaf4c5, 90 },
    { 0x8dfaf4c5, 98 },
    { 0x2e6af4c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66af4cd, 81 },
    { 0x480af4ce, 98 },
    { 0, 0 },
    { 0x1bdaf4d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aaf4d3, 91 },
    { 0, 0 },
    { 0x10caf4d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64af4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7af4dd, 76 },
    { 0x23af4de, 90 },
    { 0x7fcaf4de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6af4e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x927af4e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aaf4ee, 78 },
    { 0xf28af4ef, 90 },
    { 0, 0 },
    { 0x723af4f1, 76 },
    { 0x4b7af4f2, 95 },
    { 0, 0 },
    { 0x692af4f4, 2 },
    { 0, 0 },
    { 0x850af4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcaf4fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50faf4fe, 38 },
    { 0x94af4ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770af504, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x318af516, 100 },
    { 0, 0 },
    { 0xa88af518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x811af51b, 33 },
    { 0, 0 },
    { 0x58aaf51d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3baf523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feaf526, 76 },
    { 0x53aaf526, 33 },
    { 0xbc8af528, 23 },
    { 0xbf6af529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551af52d, 34 },
    { 0, 0 },
    { 0xe76af52f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aaf532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86af538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03af53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3daf542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feaf545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acaf54a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5af54d, 61 },
    { 0, 0 },
    { 0xc6faf54f, 23 },
    { 0, 0 },
    { 0x9dbaf551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66af555, 84 },
    { 0xb5faf556, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10caf55a, 90 },
    { 0, 0 },
    { 0xf8faf55c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504af562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529af567, 90 },
    { 0xa71af568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972af56c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827af570, 61 },
    { 0x23aaf571, 100 },
    { 0xc6eaf571, 43 },
    { 0x3caf573, 23 },
    { 0xd0af573, 98 },
    { 0, 0 },
    { 0x708af576, 23 },
    { 0xe83af577, 46 },
    { 0xb98af578, 90 },
    { 0, 0 },
    { 0x50baf57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70caf57d, 95 },
    { 0x332af57e, 85 },
    { 0x125af57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9af585, 52 },
    { 0xbc8af586, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eaf58d, 84 },
    { 0, 0 },
    { 0x744af58f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174af594, 90 },
    { 0x107af595, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caf598, 23 },
    { 0, 0 },
    { 0x6e9af59a, 46 },
    { 0, 0 },
    { 0xcb9af59c, 96 },
    { 0xa30af59c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552af5a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x362af5a5, 85 },
    { 0, 0 },
    { 0xbd1af5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1af5b0, 23 },
    { 0, 0 },
    { 0xd5caf5b2, 98 },
    { 0x78baf5b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0af5b6, 61 },
    { 0x845af5b6, 55 },
    { 0xbc2af5b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaf5c2, 90 },
    { 0x3b4af5c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807af5c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1af5cb, 90 },
    { 0xc23af5cc, 95 },
    { 0x1edaf5cd, 55 },
    { 0, 0 },
    { 0x86caf5cf, 23 },
    { 0, 0 },
    { 0x3e4af5d1, 84 },
    { 0xb9daf5d1, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8af5d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36af5d9, 90 },
    { 0, 0 },
    { 0x804af5db, 34 },
    { 0x605af5dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4af5e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594af5e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9af5ea, 76 },
    { 0, 0 },
    { 0x21caf5ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971af5f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358af5f8, 93 },
    { 0, 0 },
    { 0x9bcaf5fa, 95 },
    { 0x7d9af5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47caf5ff, 98 },
    { 0x4f6af5ff, 43 },
    { 0xabdaf601, 33 },
    { 0x928af5ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcaf608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01af60d, 90 },
    { 0x39faf60e, 93 },
    { 0, 0 },
    { 0xbc4af610, 90 },
    { 0x532af610, 90 },
    { 0xaadaf610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9af616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474af61c, 33 },
    { 0x65faf61c, 98 },
    { 0xac4af61e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139af623, 61 },
    { 0x49aaf624, 95 },
    { 0x885af625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93af628, 91 },
    { 0x63caf628, 34 },
    { 0x508af62a, 44 },
    { 0xaf2af628, 98 },
    { 0x332af62c, 90 },
    { 0x225af62d, 70 },
    { 0x3dbaf62e, 65 },
    { 0x401af62f, 33 },
    { 0xe84af62a, 61 },
    { 0x35caf630, 46 },
    { 0x5f2af628, 55 },
    { 0x1fbaf633, 85 },
    { 0x855af634, 95 },
    { 0xc7daf633, 61 },
    { 0x1a3af62b, 33 },
    { 0xf12af632, 43 },
    { 0, 0 },
    { 0xbdcaf639, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352af641, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcaf646, 33 },
    { 0x1dfaf647, 46 },
    { 0x937af648, 90 },
    { 0x347af647, 34 },
    { 0xebaf648, 90 },
    { 0xd7faf64a, 98 },
    { 0x5ceaf64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaaf651, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5af654, 55 },
    { 0x161af655, 87 },
    { 0x4c3af655, 98 },
    { 0x150af657, 90 },
    { 0xf8daf658, 34 },
    { 0xc74af657, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcaf663, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eaf667, 65 },
    { 0x971af668, 75 },
    { 0, 0 },
    { 0x2acaf66a, 53 },
    { 0x4e2af66b, 87 },
    { 0x1efaf66c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0af66f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15af672, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaf675, 33 },
    { 0, 0 },
    { 0x590af677, 33 },
    { 0x35daf678, 43 },
    { 0x90daf678, 75 },
    { 0, 0 },
    { 0x2b0af67b, 90 },
    { 0x17eaf67b, 78 },
    { 0, 0 },
    { 0x438af67e, 76 },
    { 0xd2baf67e, 40 },
    { 0, 0 },
    { 0x1eaaf681, 17 },
    { 0, 0 },
    { 0x856af683, 95 },
    { 0, 0 },
    { 0xa1af685, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86af692, 33 },
    { 0x786af692, 98 },
    { 0xe37af694, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aaf69b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af6a2, 33 },
    { 0, 0 },
    { 0x78aaf6a4, 23 },
    { 0xf86af6a5, 95 },
    { 0xfb7af6a6, 38 },
    { 0, 0 },
    { 0x513af6a8, 61 },
    { 0x543af6a9, 90 },
    { 0x626af6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34af6af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfaf6b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aaf6b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53af6c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381af6c4, 76 },
    { 0xab8af6c5, 55 },
    { 0x22daf6c6, 76 },
    { 0xe67af6c6, 77 },
    { 0x1bcaf6c8, 96 },
    { 0, 0 },
    { 0xd97af6ca, 87 },
    { 0xb00af6cb, 43 },
    { 0, 0 },
    { 0xd92af6cd, 34 },
    { 0x691af6ce, 24 },
    { 0x5d7af6ce, 66 },
    { 0, 0 },
    { 0x18caf6d1, 97 },
    { 0xeb6af6d2, 99 },
    { 0, 0 },
    { 0xf69af6d4, 90 },
    { 0x258af6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52daf6d8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x326af6db, 85 },
    { 0x3daf6db, 23 },
    { 0x449af6dc, 77 },
    { 0x5e3af6de, 100 },
    { 0xcccaf6dc, 23 },
    { 0xf18af6dd, 34 },
    { 0xcd2af6e1, 90 },
    { 0x24eaf6e1, 77 },
    { 0x9b9af6e0, 33 },
    { 0, 0 },
    { 0x855af6e5, 55 },
    { 0x6b1af6e6, 33 },
    { 0xa36af6e7, 75 },
    { 0, 0 },
    { 0xf30af6e9, 90 },
    { 0, 0 },
    { 0x517af6eb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0af6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0faf6f5, 90 },
    { 0xac0af6f6, 23 },
    { 0, 0 },
    { 0xc51af6f8, 67 },
    { 0x3d6af6f9, 46 },
    { 0, 0 },
    { 0xd34af6fb, 95 },
    { 0xa24af6fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17af6ff, 80 },
    { 0x834af6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00af705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10af708, 98 },
    { 0, 0 },
    { 0x7a8af70a, 75 },
    { 0xdb1af70a, 99 },
    { 0xf12af70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1af712, 33 },
    { 0, 0 },
    { 0x7e6af714, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7daf719, 23 },
    { 0xfd1af71a, 98 },
    { 0x2bbaf71b, 61 },
    { 0x849af71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4af724, 99 },
    { 0x5faaf724, 33 },
    { 0xc1baf726, 100 },
    { 0x21daf727, 52 },
    { 0x2a3af724, 76 },
    { 0xb15af724, 33 },
    { 0x2eeaf725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15faf733, 95 },
    { 0, 0 },
    { 0x864af735, 61 },
    { 0xc8faf735, 61 },
    { 0x3e1af737, 33 },
    { 0x1a5af736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60af73d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4af741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70baf744, 98 },
    { 0xde3af745, 77 },
    { 0xaffaf745, 61 },
    { 0x4cfaf745, 77 },
    { 0x10eaf748, 23 },
    { 0xf40af747, 98 },
    { 0xf01af746, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeaf74e, 88 },
    { 0x400af74f, 38 },
    { 0xe4eaf74f, 98 },
    { 0, 0 },
    { 0x9cfaf752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92caf759, 88 },
    { 0, 0 },
    { 0x91af75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4faf760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4af764, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73af768, 98 },
    { 0, 0 },
    { 0xd40af76a, 98 },
    { 0x32baf76a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29af772, 66 },
    { 0x542af773, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32af779, 34 },
    { 0x61caf77a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913af77e, 52 },
    { 0x868af77f, 90 },
    { 0, 0 },
    { 0x2bfaf781, 61 },
    { 0x959af781, 33 },
    { 0xe06af782, 87 },
    { 0x4a9af783, 98 },
    { 0, 0 },
    { 0xeb5af786, 56 },
    { 0x6a1af787, 87 },
    { 0x329af787, 95 },
    { 0xe97af788, 76 },
    { 0xcbdaf789, 43 },
    { 0x417af788, 76 },
    { 0x462af78a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09af78f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6baf794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceaf798, 52 },
    { 0xd58af798, 93 },
    { 0x8f6af79a, 38 },
    { 0x80faf79a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1af7a3, 61 },
    { 0x2afaf7a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3af7a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af7b1, 61 },
    { 0, 0 },
    { 0xe75af7b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feaf7b6, 90 },
    { 0x4a5af7b6, 61 },
    { 0x7b1af7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869af7be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4af7c2, 90 },
    { 0, 0 },
    { 0x7b0af7c4, 33 },
    { 0x7dbaf7c5, 23 },
    { 0x1adaf7c6, 61 },
    { 0x4f1af7c7, 90 },
    { 0xb16af7c7, 33 },
    { 0xf2baf7c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276af7cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133af7d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79af7d7, 90 },
    { 0x84caf7d8, 95 },
    { 0x455af7d8, 87 },
    { 0x32af7da, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aaf7de, 76 },
    { 0x9c0af7de, 55 },
    { 0, 0 },
    { 0xd12af7e1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520af7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36af7ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x224af7f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9af7f3, 46 },
    { 0x934af7f4, 98 },
    { 0xe53af7f5, 52 },
    { 0, 0 },
    { 0xb53af7f7, 34 },
    { 0xdefaf7f8, 66 },
    { 0, 0 },
    { 0x198af7fa, 98 },
    { 0xc34af7fb, 93 },
    { 0, 0 },
    { 0xe04af7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704af802, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aaf809, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6af80c, 100 },
    { 0xf35af80c, 90 },
    { 0xa61af80e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5af811, 33 },
    { 0x800af812, 61 },
    { 0x7daaf811, 100 },
    { 0x89caf814, 55 },
    { 0xf1baf815, 61 },
    { 0, 0 },
    { 0x4e2af817, 33 },
    { 0x4b5af817, 33 },
    { 0x53af819, 76 },
    { 0x8c1af81a, 61 },
    { 0xaadaf81b, 17 },
    { 0xfc3af81c, 76 },
    { 0x6afaf81c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53baf820, 55 },
    { 0xfd1af820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfaf824, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aaf82a, 95 },
    { 0xe65af82a, 100 },
    { 0xe54af82c, 55 },
    { 0xbffaf82d, 98 },
    { 0, 0 },
    { 0xe4daf82f, 100 },
    { 0, 0 },
    { 0x80af831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af83b, 33 },
    { 0x8a9af83c, 34 },
    { 0xf3baf83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86daf842, 76 },
    { 0, 0 },
    { 0x1a2af844, 98 },
    { 0x58eaf845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaf84a, 98 },
    { 0x107af84b, 61 },
    { 0x40aaf84c, 98 },
    { 0xd5baf84b, 34 },
    { 0x21eaf84b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22af851, 76 },
    { 0xd22af851, 33 },
    { 0x92eaf853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2faf856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750af85a, 23 },
    { 0xb4caf85b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7af85f, 91 },
    { 0, 0 },
    { 0x730af861, 84 },
    { 0x4f6af862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35daf866, 90 },
    { 0x5bcaf867, 33 },
    { 0x7a2af868, 67 },
    { 0, 0 },
    { 0xa92af86a, 87 },
    { 0, 0 },
    { 0xc81af86c, 98 },
    { 0, 0 },
    { 0x4c0af86e, 98 },
    { 0x85baf86e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadaf872, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdaf87a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36caf87e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefaf883, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6af886, 61 },
    { 0xe33af886, 90 },
    { 0xe2baf888, 33 },
    { 0x460af887, 76 },
    { 0x60af886, 90 },
    { 0, 0 },
    { 0x285af88c, 33 },
    { 0, 0 },
    { 0xaffaf88e, 93 },
    { 0, 0 },
    { 0xcf0af890, 34 },
    { 0x761af891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8af895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8af89b, 33 },
    { 0x8b2af89c, 91 },
    { 0xeafaf89c, 93 },
    { 0, 0 },
    { 0x2f3af89f, 90 },
    { 0x15eaf8a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108af8a7, 26 },
    { 0x1a9af8a8, 90 },
    { 0x884af8a9, 43 },
    { 0xb8af8aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faaf8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9faf8b4, 55 },
    { 0xbb4af8b4, 52 },
    { 0x2c3af8b4, 33 },
    { 0, 0 },
    { 0x4b1af8b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5af8bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16af8c1, 61 },
    { 0, 0 },
    { 0x7ddaf8c3, 98 },
    { 0x21daf8c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eaf8c7, 43 },
    { 0xcc8af8c7, 77 },
    { 0x512af8c9, 95 },
    { 0x4c7af8ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13af8cd, 34 },
    { 0xdebaf8ce, 99 },
    { 0x72aaf8cf, 43 },
    { 0xf5daf8d0, 95 },
    { 0xbb1af8d0, 61 },
    { 0x869af8d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29daf8d8, 33 },
    { 0xb43af8d9, 33 },
    { 0x438af8d9, 90 },
    { 0xd14af8db, 67 },
    { 0x7b1af8d9, 90 },
    { 0xcdcaf8dc, 61 },
    { 0x7eaaf8dc, 81 },
    { 0x7d1af8df, 93 },
    { 0, 0 },
    { 0xb6baf8e1, 43 },
    { 0, 0 },
    { 0x656af8e3, 61 },
    { 0xb72af8e4, 23 },
    { 0x7fdaf8e5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eaf8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caf8f3, 23 },
    { 0x9fdaf8f4, 95 },
    { 0x731af8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaaf8fa, 46 },
    { 0, 0 },
    { 0xcc2af8fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45af8ff, 61 },
    { 0xe88af900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07af904, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6af907, 46 },
    { 0, 0 },
    { 0x17baf909, 55 },
    { 0x190af90a, 46 },
    { 0, 0 },
    { 0x276af90c, 23 },
    { 0xac8af90c, 55 },
    { 0xe46af90e, 90 },
    { 0x6dbaf90f, 91 },
    { 0xb48af910, 91 },
    { 0xc51af911, 52 },
    { 0xea7af910, 64 },
    { 0xabbaf913, 23 },
    { 0xd42af913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4af917, 90 },
    { 0, 0 },
    { 0xa32af919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396af920, 90 },
    { 0, 0 },
    { 0x1e3af922, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x90caf925, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0af929, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0af92d, 90 },
    { 0x22caf92e, 95 },
    { 0x98baf92e, 33 },
    { 0x23faf92f, 99 },
    { 0, 0 },
    { 0xf15af932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eaf93a, 69 },
    { 0xbb0af93a, 23 },
    { 0xb31af93b, 76 },
    { 0, 0 },
    { 0x465af93e, 78 },
    { 0x6d5af93f, 84 },
    { 0x711af940, 90 },
    { 0x1bcaf940, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963af94e, 23 },
    { 0x8a4af94f, 61 },
    { 0x81baf950, 46 },
    { 0, 0 },
    { 0xcfaaf952, 98 },
    { 0x835af952, 99 },
    { 0x1a9af954, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580af959, 23 },
    { 0xf78af959, 90 },
    { 0x6c8af959, 33 },
    { 0x715af95c, 17 },
    { 0xf14af95a, 90 },
    { 0x734af95e, 46 },
    { 0, 0 },
    { 0x95faf960, 23 },
    { 0x700af960, 95 },
    { 0x9c3af960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbaf968, 52 },
    { 0x223af969, 87 },
    { 0xb73af968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1af96e, 90 },
    { 0xffeaf96e, 43 },
    { 0xed6af970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20daf974, 98 },
    { 0xb48af975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45af97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7baf981, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287af986, 52 },
    { 0, 0 },
    { 0x631af988, 67 },
    { 0xbe8af988, 76 },
    { 0x284af98a, 76 },
    { 0, 0 },
    { 0xb33af98c, 98 },
    { 0, 0 },
    { 0x1bdaf98e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadaf993, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12baf996, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5baf9a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x40baf9a9, 61 },
    { 0x778af9a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77af9af, 90 },
    { 0xfacaf9af, 95 },
    { 0x56caf9b1, 99 },
    { 0x7fdaf9b1, 84 },
    { 0, 0 },
    { 0x166af9b4, 76 },
    { 0xa3baf9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0daf9c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742af9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99af9ce, 55 },
    { 0xa54af9cf, 76 },
    { 0x606af9ce, 90 },
    { 0x8feaf9cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af9d6, 34 },
    { 0x7c7af9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45af9da, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x133af9dd, 75 },
    { 0, 0 },
    { 0x960af9df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921af9e2, 77 },
    { 0xb09af9e2, 78 },
    { 0x3d2af9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65caf9ea, 29 },
    { 0x5e7af9eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666af9f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85af9f5, 90 },
    { 0xe9af9f6, 52 },
    { 0x5b9af9f7, 23 },
    { 0xcc8af9f8, 33 },
    { 0xe10af9f6, 90 },
    { 0x40aaf9fa, 23 },
    { 0xd9aaf9f5, 85 },
    { 0, 0 },
    { 0x276af9fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdafa03, 98 },
    { 0, 0 },
    { 0xd42afa05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816afa10, 90 },
    { 0, 0 },
    { 0xdafa12, 90 },
    { 0, 0 },
    { 0x4a2afa14, 33 },
    { 0xb85afa15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5afa1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x173afa1e, 90 },
    { 0, 0 },
    { 0x427afa20, 33 },
    { 0x4d5afa21, 66 },
    { 0x78afa20, 98 },
    { 0xfb5afa23, 61 },
    { 0, 0 },
    { 0xbafa25, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dafa29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06afa2e, 55 },
    { 0x155afa2f, 23 },
    { 0, 0 },
    { 0xe3eafa31, 34 },
    { 0x96cafa32, 76 },
    { 0, 0 },
    { 0x6dbafa34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9afa3b, 61 },
    { 0, 0 },
    { 0xe4cafa3d, 34 },
    { 0x63bafa3e, 93 },
    { 0xa05afa3f, 87 },
    { 0x46cafa3f, 33 },
    { 0, 0 },
    { 0x950afa42, 98 },
    { 0, 0 },
    { 0xfaeafa44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x238afa47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffafa4a, 93 },
    { 0xa37afa4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafa52, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655afa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfafa5f, 61 },
    { 0x6a9afa60, 55 },
    { 0, 0 },
    { 0x9d3afa62, 90 },
    { 0x4b2afa63, 90 },
    { 0, 0 },
    { 0x6fcafa65, 78 },
    { 0x538afa65, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x136afa69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548afa6f, 87 },
    { 0xb3afa70, 76 },
    { 0x496afa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190afa77, 55 },
    { 0xffcafa78, 90 },
    { 0xca5afa78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69afa7c, 91 },
    { 0, 0 },
    { 0xc9bafa7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00afa82, 52 },
    { 0xf88afa82, 90 },
    { 0, 0 },
    { 0x108afa85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14afa8b, 90 },
    { 0x46dafa8c, 66 },
    { 0x6fcafa8d, 93 },
    { 0xd6eafa8d, 76 },
    { 0x7afa8f, 43 },
    { 0xd32afa8d, 33 },
    { 0xd9fafa8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138afa95, 67 },
    { 0xc3bafa96, 98 },
    { 0x8a0afa97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709afa9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2afaa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37afaa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dafaa9, 78 },
    { 0x9e1afaa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdafab1, 52 },
    { 0, 0 },
    { 0xcb2afab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaafab8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6afabb, 61 },
    { 0x9e8afabb, 61 },
    { 0x210afabc, 55 },
    { 0x749afabb, 61 },
    { 0xe38afabf, 90 },
    { 0x8c4afabf, 98 },
    { 0x652afac1, 90 },
    { 0xc11afac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceafac5, 90 },
    { 0x1e0afac6, 33 },
    { 0xd62afac7, 55 },
    { 0, 0 },
    { 0xa2eafac9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44aface, 95 },
    { 0x18aaface, 33 },
    { 0, 0 },
    { 0x900afad1, 61 },
    { 0xeccafad1, 87 },
    { 0xcefafad3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bafad7, 34 },
    { 0, 0 },
    { 0x4bbafad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aafadd, 33 },
    { 0x31dafadd, 33 },
    { 0xddfafadf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5afae2, 46 },
    { 0x540afae3, 23 },
    { 0x8deafae4, 61 },
    { 0x75dafae4, 98 },
    { 0, 0 },
    { 0xaa2afae7, 93 },
    { 0x3d3afae7, 33 },
    { 0x80bafae8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443afaee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96afaf3, 23 },
    { 0, 0 },
    { 0xf34afaf5, 33 },
    { 0, 0 },
    { 0xb7cafaf7, 76 },
    { 0x86afaf8, 23 },
    { 0x4cfafaf9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1afaff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345afb03, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bafb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3afb09, 61 },
    { 0, 0 },
    { 0xdfafb0b, 46 },
    { 0xcd0afb0b, 34 },
    { 0, 0 },
    { 0xfc9afb0e, 61 },
    { 0xd32afb0f, 90 },
    { 0x3fafb0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528afb14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4afb17, 100 },
    { 0xddeafb18, 98 },
    { 0x1e8afb19, 23 },
    { 0x7deafb1a, 34 },
    { 0x65afb1b, 42 },
    { 0x36bafb1c, 33 },
    { 0xf00afb1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dafb20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70afb25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cafb2c, 76 },
    { 0x3ccafb2d, 76 },
    { 0, 0 },
    { 0x21cafb2f, 87 },
    { 0xd5aafb30, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202afb35, 61 },
    { 0, 0 },
    { 0x16dafb37, 33 },
    { 0x5a4afb38, 17 },
    { 0xf48afb39, 34 },
    { 0x6f8afb38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46afb4a, 85 },
    { 0, 0 },
    { 0x14eafb4c, 97 },
    { 0, 0 },
    { 0x147afb4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4afb51, 91 },
    { 0xd2dafb52, 33 },
    { 0x9f2afb52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6afb56, 78 },
    { 0xe02afb57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bafb60, 46 },
    { 0, 0 },
    { 0x2d0afb62, 85 },
    { 0xbf4afb62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43afb6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aafb6f, 78 },
    { 0xd1cafb6f, 98 },
    { 0, 0 },
    { 0x5bfafb72, 100 },
    { 0xdc4afb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19afb78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528afb80, 17 },
    { 0xaceafb81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750afb86, 61 },
    { 0, 0 },
    { 0xd7fafb88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198afb8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7afb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdafb95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995afb9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9afb9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09afba1, 84 },
    { 0x1cbafba2, 100 },
    { 0xf31afba2, 84 },
    { 0x551afba4, 76 },
    { 0x720afba4, 43 },
    { 0xa35afba6, 76 },
    { 0x8e5afba6, 95 },
    { 0x960afba8, 33 },
    { 0xbb1afba9, 98 },
    { 0x999afbaa, 65 },
    { 0x574afbab, 61 },
    { 0x4f9afbac, 90 },
    { 0x836afbac, 34 },
    { 0x4ceafbac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aafbb2, 61 },
    { 0, 0 },
    { 0x201afbb4, 95 },
    { 0x2f2afbb5, 75 },
    { 0xab9afbb6, 95 },
    { 0, 0 },
    { 0x4b8afbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6afbbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aafbbe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x440afbc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332afbca, 95 },
    { 0x82afbcb, 98 },
    { 0xbc6afbcc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4afbd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dafbd6, 75 },
    { 0xff7afbd6, 98 },
    { 0, 0 },
    { 0x30eafbd9, 83 },
    { 0x1a4afbda, 38 },
    { 0xc0aafbdb, 98 },
    { 0x9f5afbd9, 78 },
    { 0x1eafbdc, 90 },
    { 0xc02afbde, 23 },
    { 0xc9aafbdf, 99 },
    { 0, 0 },
    { 0xb84afbe1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78afbe5, 98 },
    { 0, 0 },
    { 0x26cafbe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daafbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687afbf4, 95 },
    { 0xceafbf5, 67 },
    { 0x10dafbf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65afbf9, 67 },
    { 0x53aafbfa, 97 },
    { 0x64fafbfb, 33 },
    { 0xda9afbfc, 81 },
    { 0x1eaafbfb, 90 },
    { 0xdbafbfe, 95 },
    { 0, 0 },
    { 0x283afc00, 90 },
    { 0x95cafc00, 85 },
    { 0x817afc02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84eafc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50afc0f, 94 },
    { 0xf8dafc10, 55 },
    { 0xed5afc11, 91 },
    { 0x879afc11, 90 },
    { 0, 0 },
    { 0x75aafc14, 61 },
    { 0x85fafc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aafc19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6afc1c, 98 },
    { 0x1bdafc1d, 98 },
    { 0xe4dafc1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700afc29, 93 },
    { 0, 0 },
    { 0x230afc2b, 33 },
    { 0xeb1afc2c, 78 },
    { 0xc53afc2d, 43 },
    { 0x620afc2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8afc37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9afc3b, 61 },
    { 0, 0 },
    { 0xd7aafc3d, 98 },
    { 0xd8eafc3e, 91 },
    { 0xf93afc3e, 33 },
    { 0xa0afc40, 76 },
    { 0x457afc41, 46 },
    { 0xd26afc42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fafc45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874afc51, 95 },
    { 0xd37afc52, 90 },
    { 0x2beafc52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dafc57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1afc5b, 91 },
    { 0xb37afc5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bafc61, 95 },
    { 0xaf7afc62, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962afc6e, 77 },
    { 0x3deafc6f, 96 },
    { 0, 0 },
    { 0x75aafc71, 98 },
    { 0, 0 },
    { 0x93dafc73, 33 },
    { 0x40aafc74, 33 },
    { 0, 0 },
    { 0x368afc76, 76 },
    { 0x4bbafc77, 23 },
    { 0x34dafc77, 23 },
    { 0, 0 },
    { 0x8bcafc7a, 76 },
    { 0x2a2afc7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7afc7f, 90 },
    { 0xad2afc7f, 43 },
    { 0x7a8afc81, 76 },
    { 0x20afc82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576afc8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3afc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2afc91, 61 },
    { 0, 0 },
    { 0xda2afc93, 61 },
    { 0x99dafc94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385afc98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745afc9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7afc9f, 78 },
    { 0xe24afca0, 90 },
    { 0, 0 },
    { 0x4d3afca2, 90 },
    { 0x4e3afca3, 52 },
    { 0xf3afca3, 61 },
    { 0, 0 },
    { 0x6bdafca6, 46 },
    { 0x23bafca7, 98 },
    { 0xd09afca7, 75 },
    { 0, 0 },
    { 0x8d4afcaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63afcb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549afcba, 33 },
    { 0x418afcba, 98 },
    { 0xb4aafcbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714afcc2, 76 },
    { 0x8b7afcc3, 90 },
    { 0x13aafcc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bafccb, 95 },
    { 0xc35afccc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deafccf, 78 },
    { 0, 0 },
    { 0xdd8afcd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddafcd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8afcdb, 94 },
    { 0, 0 },
    { 0xe86afcdd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08afce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5afce7, 23 },
    { 0xb1dafce7, 33 },
    { 0x6e8afce9, 55 },
    { 0x5c7afce9, 90 },
    { 0x3f5afceb, 99 },
    { 0, 0 },
    { 0x44aafced, 93 },
    { 0x190afcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92afcf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffafcf4, 61 },
    { 0, 0 },
    { 0x4b2afcf6, 93 },
    { 0, 0 },
    { 0xb0aafcf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x879afcfb, 81 },
    { 0, 0 },
    { 0xabeafcfd, 23 },
    { 0x790afcfe, 90 },
    { 0x644afcff, 61 },
    { 0x757afd00, 61 },
    { 0xb8eafd00, 66 },
    { 0xf83afd02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76afd08, 90 },
    { 0x334afd08, 61 },
    { 0x62afd09, 90 },
    { 0x422afd09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50afd0e, 55 },
    { 0, 0 },
    { 0x303afd10, 43 },
    { 0xbf3afd10, 76 },
    { 0x78afd11, 98 },
    { 0x152afd13, 78 },
    { 0x9f7afd13, 69 },
    { 0x6fdafd15, 98 },
    { 0x174afd13, 69 },
    { 0, 0 },
    { 0x5aeafd18, 33 },
    { 0, 0 },
    { 0xcb5afd1a, 95 },
    { 0, 0 },
    { 0xd88afd1c, 61 },
    { 0x9a2afd1d, 93 },
    { 0x6efafd1d, 76 },
    { 0, 0 },
    { 0x452afd20, 61 },
    { 0, 0 },
    { 0x29fafd22, 78 },
    { 0, 0 },
    { 0x6dbafd24, 90 },
    { 0xd2bafd25, 33 },
    { 0x90cafd26, 98 },
    { 0x217afd27, 90 },
    { 0xf2afd28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767afd2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deafd32, 90 },
    { 0x96eafd33, 76 },
    { 0xf73afd32, 23 },
    { 0xb94afd34, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fafd38, 33 },
    { 0, 0 },
    { 0x9f6afd3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21afd3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fafd42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bafd45, 90 },
    { 0, 0 },
    { 0x5a2afd47, 76 },
    { 0, 0 },
    { 0x83dafd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x640afd4c, 95 },
    { 0, 0 },
    { 0x1fdafd4e, 95 },
    { 0x250afd4e, 43 },
    { 0, 0 },
    { 0x4aeafd51, 43 },
    { 0x54afd52, 64 },
    { 0, 0 },
    { 0x201afd54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49afd5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88afd67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bafd6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cafd6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x848afd72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2afd78, 95 },
    { 0, 0 },
    { 0x938afd7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdafd81, 23 },
    { 0, 0 },
    { 0x803afd83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5afd86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5afd8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bafd8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beafd90, 61 },
    { 0, 0 },
    { 0xb24afd92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70afd96, 46 },
    { 0xa58afd97, 23 },
    { 0xa25afd98, 95 },
    { 0x122afd99, 46 },
    { 0x693afd9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcafd9d, 95 },
    { 0x75eafd9e, 61 },
    { 0x503afd9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96afda3, 87 },
    { 0xccafda3, 95 },
    { 0x701afda3, 61 },
    { 0x45afda6, 90 },
    { 0x2e7afda4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10afdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9afdb1, 100 },
    { 0x992afdb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9afdb5, 61 },
    { 0x5a9afdb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76afdba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeafdbd, 43 },
    { 0x563afdbd, 81 },
    { 0xeaaafdbf, 43 },
    { 0xc09afdc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821afdc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137afdcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afdd7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968afddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2afde0, 90 },
    { 0, 0 },
    { 0x286afde2, 66 },
    { 0xfdafde3, 66 },
    { 0, 0 },
    { 0x7c6afde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dafde8, 61 },
    { 0x6e9afde9, 77 },
    { 0xc65afdea, 87 },
    { 0x18cafde8, 38 },
    { 0x1c9afdec, 38 },
    { 0x1a1afdec, 33 },
    { 0xd61afdee, 46 },
    { 0xb52afded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396afdf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687afdfb, 90 },
    { 0xd37afdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4afe04, 33 },
    { 0, 0 },
    { 0x264afe06, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7afe0f, 23 },
    { 0xd4fafe10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03afe1a, 90 },
    { 0xd64afe1b, 38 },
    { 0, 0 },
    { 0x1e9afe1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aafe21, 90 },
    { 0x962afe21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957afe28, 61 },
    { 0xb93afe28, 90 },
    { 0x387afe28, 90 },
    { 0x86fafe29, 68 },
    { 0x83aafe2b, 101 },
    { 0xa41afe2b, 90 },
    { 0, 0 },
    { 0x943afe2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2afe33, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x388afe36, 95 },
    { 0, 0 },
    { 0xabaafe38, 17 },
    { 0x37cafe39, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeafe3c, 99 },
    { 0x54cafe3d, 93 },
    { 0xb33afe3e, 90 },
    { 0xe05afe3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40afe45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eafe48, 23 },
    { 0xa25afe48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176afe4e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bafe53, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65afe58, 46 },
    { 0x113afe58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aafe5c, 95 },
    { 0xae3afe5d, 98 },
    { 0x3cbafe5e, 95 },
    { 0x1c7afe5c, 90 },
    { 0x183afe60, 76 },
    { 0xec2afe61, 61 },
    { 0x849afe62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66afe6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eafe73, 68 },
    { 0x5f2afe73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8afe78, 61 },
    { 0x3a8afe79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdafe7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cafe80, 100 },
    { 0, 0 },
    { 0x70fafe82, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128afe86, 66 },
    { 0x4bcafe86, 61 },
    { 0, 0 },
    { 0x832afe89, 43 },
    { 0x440afe8a, 90 },
    { 0, 0 },
    { 0x24afe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fafe91, 77 },
    { 0x215afe91, 98 },
    { 0x2c2afe93, 98 },
    { 0x7c0afe93, 91 },
    { 0xec8afe93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dafe98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdafe9b, 46 },
    { 0x797afe9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dafe9f, 33 },
    { 0, 0 },
    { 0xa69afea1, 98 },
    { 0xc05afea2, 95 },
    { 0x7ffafea2, 95 },
    { 0, 0 },
    { 0x3feafea5, 43 },
    { 0x31bafea5, 55 },
    { 0xe0afea5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314afeb0, 98 },
    { 0, 0 },
    { 0xf07afeb2, 23 },
    { 0xa88afeb3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46afec0, 90 },
    { 0x603afec0, 90 },
    { 0xa8afec0, 95 },
    { 0x4c8afec3, 85 },
    { 0x93afec2, 90 },
    { 0x3fcafec3, 84 },
    { 0x93dafec6, 55 },
    { 0xadeafec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120afecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadafed2, 23 },
    { 0x1faafed3, 38 },
    { 0xeecafed4, 34 },
    { 0xb49afed3, 76 },
    { 0x6c6afed6, 23 },
    { 0x474afed6, 23 },
    { 0xb9bafed8, 78 },
    { 0xe9eafed8, 61 },
    { 0x48bafed6, 76 },
    { 0xbbcafedb, 93 },
    { 0x651afedc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecafee4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cafee9, 61 },
    { 0xcb9afee9, 84 },
    { 0xeafeeb, 61 },
    { 0x3c3afeec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aafeef, 67 },
    { 0x7a8afef0, 61 },
    { 0x973afef1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13afef8, 87 },
    { 0xf3bafef8, 76 },
    { 0x8edafef8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543afefe, 98 },
    { 0xb76afeff, 98 },
    { 0x2faff00, 40 },
    { 0, 0 },
    { 0x83daff02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3caff07, 97 },
    { 0, 0 },
    { 0xe54aff09, 33 },
    { 0xb4eaff0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918aff0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aaff13, 90 },
    { 0x409aff14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291aff1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58baff1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faff24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x371aff27, 90 },
    { 0, 0 },
    { 0x488aff29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7caff2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7aff31, 90 },
    { 0, 0 },
    { 0x91aff33, 99 },
    { 0xb6aff33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aff3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7aff4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbaff53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2faff56, 55 },
    { 0x1fdaff57, 65 },
    { 0xf1aaff58, 34 },
    { 0, 0 },
    { 0x2a8aff5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868aff5e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7aff62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daaff66, 52 },
    { 0, 0 },
    { 0x912aff68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacaff6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758aff78, 43 },
    { 0, 0 },
    { 0xceaff7a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedaff7e, 43 },
    { 0x9afaff7f, 33 },
    { 0x41faff80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723aff84, 95 },
    { 0x996aff85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baff8a, 95 },
    { 0, 0 },
    { 0xf08aff8c, 98 },
    { 0xb4eaff8d, 55 },
    { 0, 0 },
    { 0xc9faff8f, 90 },
    { 0, 0 },
    { 0xc66aff91, 66 },
    { 0xa7eaff92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0aff97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aff9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5aff9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0affa0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1affa5, 46 },
    { 0x308affa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x617affa9, 67 },
    { 0, 0 },
    { 0x932affab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaaffaf, 61 },
    { 0xee1affb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42affb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310affb9, 17 },
    { 0xfb8affba, 34 },
    { 0x45baffb9, 90 },
    { 0, 0 },
    { 0x213affbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314affc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14caffc6, 34 },
    { 0, 0 },
    { 0x756affc8, 33 },
    { 0, 0 },
    { 0xd95affca, 90 },
    { 0xcaaffca, 38 },
    { 0x9a6affcc, 34 },
    { 0x504affcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0baffd1, 69 },
    { 0, 0 },
    { 0x717affd3, 95 },
    { 0xb74affd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860affd8, 17 },
    { 0xbc7affd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeaffe1, 90 },
    { 0x87daffe2, 98 },
    { 0x3e8affe3, 53 },
    { 0, 0 },
    { 0xf8caffe5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14affec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33affef, 33 },
    { 0x462affef, 93 },
    { 0x845afff1, 46 },
    { 0x605afff2, 67 },
    { 0xbd8afff3, 76 },
    { 0x9f1afff3, 90 },
    { 0, 0 },
    { 0x2caafff6, 90 },
    { 0x8c0afff6, 34 },
    { 0, 0 },
    { 0x329afff9, 95 },
    { 0, 0 },
    { 0xee7afffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57affff, 40 },
    { 0x7e4b0000, 55 },
    { 0, 0 },
    { 0x1a9b0002, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb0007, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb000b, 46 },
    { 0x656b000c, 61 },
    { 0xe81b000b, 46 },
    { 0x40fb000d, 95 },
    { 0x4d5b000f, 76 },
    { 0, 0 },
    { 0x1dbb0011, 34 },
    { 0xb41b0012, 98 },
    { 0x284b0013, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb0017, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cb001b, 33 },
    { 0x44cb001c, 90 },
    { 0xf22b001c, 90 },
    { 0xd1db001c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0b0023, 90 },
    { 0x265b0023, 67 },
    { 0x8eab0025, 98 },
    { 0, 0 },
    { 0x324b0027, 90 },
    { 0xccfb0027, 34 },
    { 0, 0 },
    { 0x506b002a, 93 },
    { 0x146b002b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb0033, 90 },
    { 0, 0 },
    { 0xab3b0035, 33 },
    { 0x9f8b0036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b0039, 55 },
    { 0xa4bb003a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b0040, 98 },
    { 0x393b0041, 61 },
    { 0, 0 },
    { 0x83cb0043, 95 },
    { 0, 0 },
    { 0xe6cb0045, 17 },
    { 0, 0 },
    { 0x2bdb0047, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b004a, 90 },
    { 0xc61b004b, 61 },
    { 0x44ab004c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99eb004f, 33 },
    { 0x27eb004f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643b0056, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5b005b, 78 },
    { 0, 0 },
    { 0xaaab005d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb0060, 90 },
    { 0x787b0060, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb0064, 81 },
    { 0x83bb0064, 93 },
    { 0x742b0066, 43 },
    { 0xc94b0065, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27b006f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09b0073, 61 },
    { 0x20eb0073, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb0077, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488b007d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fb0083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571b0087, 98 },
    { 0x736b0088, 90 },
    { 0xafeb0087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5b008d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfb0092, 81 },
    { 0x154b0092, 98 },
    { 0x589b0094, 61 },
    { 0x97db0095, 95 },
    { 0, 0 },
    { 0x464b0097, 87 },
    { 0x8f5b0097, 99 },
    { 0, 0 },
    { 0x7c3b009a, 76 },
    { 0x591b009b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bb009e, 98 },
    { 0, 0 },
    { 0xf5b00a0, 95 },
    { 0x5f6b00a1, 43 },
    { 0xc33b00a2, 91 },
    { 0xda8b00a0, 98 },
    { 0x7ccb00a1, 66 },
    { 0x473b00a3, 34 },
    { 0x923b00a1, 61 },
    { 0xd7cb00a1, 90 },
    { 0x9eb00a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x478b00ab, 46 },
    { 0xab0b00ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187b00b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b00b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440b00bc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b00c5, 33 },
    { 0xde3b00c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65b00cb, 90 },
    { 0x604b00cc, 90 },
    { 0, 0 },
    { 0xbd3b00ce, 90 },
    { 0, 0 },
    { 0xd49b00d0, 76 },
    { 0, 0 },
    { 0x504b00d2, 33 },
    { 0, 0 },
    { 0xd30b00d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cab00dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4b00e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3b00e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4b00eb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b00f0, 76 },
    { 0, 0 },
    { 0x596b00f2, 61 },
    { 0x5c2b00f2, 90 },
    { 0xdc8b00f3, 91 },
    { 0x735b00f3, 67 },
    { 0x4b5b00f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576b00fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17b00ff, 43 },
    { 0, 0 },
    { 0xf28b0101, 23 },
    { 0, 0 },
    { 0xe6bb0103, 76 },
    { 0, 0 },
    { 0xab5b0105, 98 },
    { 0xd8db0105, 33 },
    { 0, 0 },
    { 0x195b0108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfeb010b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcb0115, 78 },
    { 0x304b0116, 23 },
    { 0, 0 },
    { 0x79eb0118, 33 },
    { 0, 0 },
    { 0x7db011a, 61 },
    { 0, 0 },
    { 0x110b011c, 23 },
    { 0x60db011d, 95 },
    { 0xa4cb011e, 61 },
    { 0xe7b011c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b0128, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb012b, 33 },
    { 0, 0 },
    { 0xb9db012d, 33 },
    { 0, 0 },
    { 0xd1bb012f, 55 },
    { 0xbacb012f, 33 },
    { 0x10cb012f, 8 },
    { 0, 0 },
    { 0x9bb0133, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b0137, 100 },
    { 0xbd4b0137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x831b013b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb013f, 61 },
    { 0x87ab0140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978b0147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58eb0150, 33 },
    { 0xad1b0151, 98 },
    { 0x9b6b0151, 61 },
    { 0x772b0153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab0158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb015c, 90 },
    { 0, 0 },
    { 0x8bfb015e, 98 },
    { 0x823b015f, 33 },
    { 0xa6cb0160, 98 },
    { 0x45eb0160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccb0165, 33 },
    { 0, 0 },
    { 0xb1cb0167, 55 },
    { 0, 0 },
    { 0xa36b0169, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcb016d, 98 },
    { 0, 0 },
    { 0xeeab016f, 23 },
    { 0x3e0b016f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb0173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb0176, 33 },
    { 0, 0 },
    { 0xe82b0178, 55 },
    { 0xf3db0178, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16b017c, 78 },
    { 0x761b017d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1b0181, 100 },
    { 0xd54b0182, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b0189, 76 },
    { 0x42b0189, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03b0191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9b0199, 85 },
    { 0x533b0199, 98 },
    { 0x434b019a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b019f, 100 },
    { 0x3cbb019f, 66 },
    { 0, 0 },
    { 0x12eb01a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773b01ab, 33 },
    { 0, 0 },
    { 0xbcb01ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb01b0, 55 },
    { 0x174b01b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb01b7, 65 },
    { 0x352b01b7, 22 },
    { 0x9db01b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb01c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb01c5, 76 },
    { 0x571b01c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17b01ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab01d1, 61 },
    { 0x448b01d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1b01e4, 33 },
    { 0xba4b01e4, 61 },
    { 0, 0 },
    { 0x987b01e7, 90 },
    { 0x5cdb01e8, 52 },
    { 0x2c5b01e7, 100 },
    { 0xe03b01ea, 34 },
    { 0xbadb01ea, 23 },
    { 0xf29b01ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb01ef, 65 },
    { 0xf3ab01f0, 90 },
    { 0x515b01ef, 69 },
    { 0, 0 },
    { 0x161b01f3, 67 },
    { 0x18bb01f3, 77 },
    { 0x6bdb01f4, 75 },
    { 0x3b3b01f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb01f9, 68 },
    { 0, 0 },
    { 0x32ab01fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b01ff, 100 },
    { 0xf2cb0200, 90 },
    { 0, 0 },
    { 0xac6b0202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205b0206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdb0209, 99 },
    { 0xd6ab0209, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9b020d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbb0211, 90 },
    { 0x72b0212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fb0216, 23 },
    { 0x5eab0217, 76 },
    { 0xeb1b0218, 55 },
    { 0, 0 },
    { 0x406b021a, 90 },
    { 0xa58b021b, 33 },
    { 0xa09b021b, 76 },
    { 0x60cb021b, 98 },
    { 0, 0 },
    { 0xb50b021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x458b0222, 76 },
    { 0, 0 },
    { 0x7b9b0224, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b022c, 98 },
    { 0, 0 },
    { 0xbceb022e, 53 },
    { 0, 0 },
    { 0x4fcb0230, 76 },
    { 0x670b0231, 23 },
    { 0xd52b0230, 34 },
    { 0xdc7b0233, 96 },
    { 0xabbb0234, 81 },
    { 0xcd1b0233, 91 },
    { 0xf2eb0236, 23 },
    { 0, 0 },
    { 0x50cb0238, 87 },
    { 0x877b0239, 77 },
    { 0, 0 },
    { 0x741b023b, 55 },
    { 0, 0 },
    { 0x1c6b023d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb0245, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eb024b, 33 },
    { 0x252b024b, 90 },
    { 0, 0 },
    { 0x61b024e, 98 },
    { 0, 0 },
    { 0xd9cb0250, 61 },
    { 0, 0 },
    { 0x87eb0252, 33 },
    { 0x4a9b0252, 33 },
    { 0x20eb0253, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb025a, 61 },
    { 0, 0 },
    { 0xb0eb025c, 95 },
    { 0xd36b025d, 61 },
    { 0, 0 },
    { 0x1fcb025f, 90 },
    { 0, 0 },
    { 0xde1b0261, 34 },
    { 0, 0 },
    { 0x589b0263, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261b0269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b026c, 98 },
    { 0x16fb026d, 98 },
    { 0, 0 },
    { 0x526b026f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b0278, 23 },
    { 0, 0 },
    { 0x9b5b027a, 90 },
    { 0xd8cb027b, 33 },
    { 0x262b027a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x409b027f, 55 },
    { 0x61fb027f, 34 },
    { 0xdf1b027f, 33 },
    { 0xb71b027f, 90 },
    { 0, 0 },
    { 0x4c1b0284, 99 },
    { 0xa17b0285, 46 },
    { 0x6adb0284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b028b, 46 },
    { 0, 0 },
    { 0x3b4b028d, 23 },
    { 0, 0 },
    { 0x607b028f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96b0292, 25 },
    { 0xacdb0293, 43 },
    { 0, 0 },
    { 0x2bb0295, 61 },
    { 0xd73b0295, 43 },
    { 0x291b0297, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692b029b, 33 },
    { 0xb80b029c, 90 },
    { 0, 0 },
    { 0x479b029e, 87 },
    { 0xc38b029e, 98 },
    { 0xeefb029e, 69 },
    { 0xac9b02a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ab02a6, 95 },
    { 0, 0 },
    { 0xcf7b02a8, 76 },
    { 0x3fcb02a9, 34 },
    { 0x2a3b02a9, 61 },
    { 0xedb02ab, 95 },
    { 0xffeb02a9, 67 },
    { 0x80db02ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb02b4, 93 },
    { 0x8b5b02b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0b02b8, 61 },
    { 0x38db02b9, 90 },
    { 0xfedb02ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ab02be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b02c6, 98 },
    { 0x351b02c7, 90 },
    { 0x59eb02c7, 23 },
    { 0x717b02c7, 33 },
    { 0x961b02c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb02d1, 23 },
    { 0x586b02d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffb02d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab02db, 23 },
    { 0, 0 },
    { 0x23b02dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b02e9, 75 },
    { 0, 0 },
    { 0xd9db02eb, 99 },
    { 0xa91b02ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8b02f0, 84 },
    { 0xda3b02f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b02f7, 66 },
    { 0, 0 },
    { 0x920b02f9, 61 },
    { 0, 0 },
    { 0x8dcb02fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b02ff, 95 },
    { 0xadb0300, 90 },
    { 0x739b0301, 98 },
    { 0xf1db0301, 77 },
    { 0xde9b0303, 33 },
    { 0x855b0303, 33 },
    { 0x911b0305, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188b0309, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0b030f, 91 },
    { 0xf36b0310, 95 },
    { 0x88fb0311, 90 },
    { 0xce6b0311, 69 },
    { 0x365b0311, 90 },
    { 0x995b0313, 61 },
    { 0x73db030f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb0319, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881b031d, 55 },
    { 0, 0 },
    { 0xdb9b031f, 66 },
    { 0xb2bb0320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82b0323, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db0326, 81 },
    { 0xeccb0326, 98 },
    { 0xe51b0328, 66 },
    { 0xfbab0329, 91 },
    { 0x9e5b032a, 61 },
    { 0xb25b032a, 91 },
    { 0xf4b0328, 55 },
    { 0x2c9b032d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97b0330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb0333, 90 },
    { 0x746b0333, 33 },
    { 0xf40b0333, 61 },
    { 0xd51b0336, 77 },
    { 0xfdab0336, 65 },
    { 0x3e2b0338, 85 },
    { 0, 0 },
    { 0x83b033a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6b033e, 99 },
    { 0x907b033f, 91 },
    { 0, 0 },
    { 0x8fb0341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x198b0344, 98 },
    { 0, 0 },
    { 0xd87b0346, 23 },
    { 0x81cb0346, 90 },
    { 0x25bb0346, 61 },
    { 0xe4db0347, 34 },
    { 0xbb0b0346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b034e, 53 },
    { 0xe16b034f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68b0352, 43 },
    { 0x93fb0352, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b035a, 90 },
    { 0, 0 },
    { 0x1f7b035c, 33 },
    { 0, 0 },
    { 0x86fb035e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b0363, 88 },
    { 0, 0 },
    { 0x955b0365, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96db0369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fb036f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x970b0372, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799b037c, 93 },
    { 0xa83b037c, 98 },
    { 0x3f7b037e, 90 },
    { 0, 0 },
    { 0x7edb0380, 95 },
    { 0xf14b0381, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb0384, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb0388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4b038b, 99 },
    { 0x997b038c, 23 },
    { 0xa73b038d, 85 },
    { 0xc47b038e, 91 },
    { 0, 0 },
    { 0x806b0390, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ab0395, 78 },
    { 0x50db0395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b0399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367b03a5, 40 },
    { 0x638b03a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b03ab, 76 },
    { 0xf09b03ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18db03af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b03b7, 95 },
    { 0xc05b03b8, 61 },
    { 0, 0 },
    { 0xd99b03ba, 34 },
    { 0, 0 },
    { 0xbdb03bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551b03c5, 23 },
    { 0x735b03c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db03c9, 46 },
    { 0xfc6b03c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01b03cf, 33 },
    { 0, 0 },
    { 0xd10b03d1, 66 },
    { 0x687b03d1, 61 },
    { 0xc7eb03d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b03d8, 23 },
    { 0x39fb03d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b03dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21b03e2, 93 },
    { 0x9cab03e3, 76 },
    { 0x28fb03e4, 68 },
    { 0, 0 },
    { 0x6f4b03e6, 90 },
    { 0, 0 },
    { 0xa48b03e8, 33 },
    { 0x55b03e9, 95 },
    { 0, 0 },
    { 0x8acb03eb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b03f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaab03f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db03fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cb03ff, 98 },
    { 0, 0 },
    { 0xf1fb0401, 90 },
    { 0xa1eb0402, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23b0407, 51 },
    { 0, 0 },
    { 0x885b0409, 33 },
    { 0x587b040a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9b041c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6b0425, 95 },
    { 0xb3bb0426, 100 },
    { 0x36eb0426, 100 },
    { 0, 0 },
    { 0xda5b0429, 61 },
    { 0xb36b042a, 68 },
    { 0x454b042b, 34 },
    { 0xbaeb042b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacb0430, 17 },
    { 0x2a6b0430, 55 },
    { 0xf5fb0430, 95 },
    { 0x2ffb0433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b0436, 43 },
    { 0, 0 },
    { 0x648b0438, 98 },
    { 0x16ab0438, 98 },
    { 0x67b0438, 87 },
    { 0xb08b043b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54b043f, 85 },
    { 0x2a2b043f, 95 },
    { 0xd1bb0440, 90 },
    { 0x248b0442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7b0447, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300b044d, 66 },
    { 0xc11b044e, 23 },
    { 0x2e3b044f, 93 },
    { 0xf7cb044d, 90 },
    { 0x2f1b0451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eb0454, 23 },
    { 0xe2db0454, 90 },
    { 0, 0 },
    { 0xb30b0457, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8b045a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b045d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadb0460, 98 },
    { 0x92db0461, 88 },
    { 0x149b0461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb0465, 66 },
    { 0x91db0466, 55 },
    { 0, 0 },
    { 0xd36b0468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb046b, 23 },
    { 0x53fb046b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb0470, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deb0473, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb0479, 61 },
    { 0, 0 },
    { 0xc9cb047b, 33 },
    { 0x9efb047b, 90 },
    { 0xa85b047d, 90 },
    { 0x9b4b047d, 76 },
    { 0x32ab047f, 40 },
    { 0, 0 },
    { 0x54eb0481, 23 },
    { 0, 0 },
    { 0x46db0483, 55 },
    { 0x9a6b0484, 67 },
    { 0, 0 },
    { 0x997b0486, 34 },
    { 0, 0 },
    { 0x27bb0488, 61 },
    { 0xbd6b0489, 90 },
    { 0xfd9b048a, 66 },
    { 0x29eb0488, 95 },
    { 0, 0 },
    { 0xbc2b048d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1deb049b, 46 },
    { 0xaadb049c, 23 },
    { 0x37bb049b, 90 },
    { 0x128b049c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebb04a1, 66 },
    { 0xc83b04a2, 77 },
    { 0, 0 },
    { 0xcefb04a4, 33 },
    { 0x878b04a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552b04a9, 17 },
    { 0x36bb04aa, 98 },
    { 0xd1b04ab, 76 },
    { 0x475b04aa, 78 },
    { 0xcc0b04ad, 46 },
    { 0, 0 },
    { 0x35db04af, 17 },
    { 0x7fdb04af, 95 },
    { 0, 0 },
    { 0x83ab04b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfb04b5, 33 },
    { 0, 0 },
    { 0x528b04b7, 99 },
    { 0x6a4b04b8, 23 },
    { 0xe6ab04b8, 100 },
    { 0xb6eb04ba, 76 },
    { 0x62fb04bb, 23 },
    { 0xf5fb04b7, 90 },
    { 0x343b04bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6b04c3, 33 },
    { 0xf74b04c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb04c9, 33 },
    { 0xe5eb04c9, 23 },
    { 0xd8cb04cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5b04da, 99 },
    { 0x564b04da, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb04e9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b04ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x468b04f0, 23 },
    { 0xa38b04f1, 17 },
    { 0x23eb04f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab04f7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ab0503, 24 },
    { 0x5b7b0504, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x444b0507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x959b050a, 87 },
    { 0, 0 },
    { 0x2abb050c, 23 },
    { 0, 0 },
    { 0x386b050e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcb0514, 90 },
    { 0xde5b0515, 95 },
    { 0x4b6b0516, 69 },
    { 0, 0 },
    { 0x72db0518, 61 },
    { 0x3a0b0519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b051c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb051f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb0527, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x511b052a, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b0534, 98 },
    { 0, 0 },
    { 0x703b0536, 98 },
    { 0x15bb0536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abb053b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452b0540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3b0543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b0546, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b054a, 33 },
    { 0x9b2b054b, 55 },
    { 0, 0 },
    { 0xcceb054d, 33 },
    { 0x3d9b054d, 61 },
    { 0x9c9b054e, 23 },
    { 0, 0 },
    { 0xf21b0551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb0559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b055d, 87 },
    { 0, 0 },
    { 0x29bb055f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb0564, 43 },
    { 0, 0 },
    { 0x930b0566, 61 },
    { 0x11ab0567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b056b, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1b056e, 34 },
    { 0, 0 },
    { 0xc95b0570, 66 },
    { 0xc0eb0571, 55 },
    { 0x4b0b0571, 55 },
    { 0xd36b0570, 33 },
    { 0, 0 },
    { 0x32cb0575, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bb0579, 76 },
    { 0xa8ab057a, 43 },
    { 0x13eb057b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acb057f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb0582, 33 },
    { 0xc9db0583, 85 },
    { 0x54b0583, 66 },
    { 0xccbb0585, 90 },
    { 0x918b0583, 23 },
    { 0x918b0587, 33 },
    { 0xbdeb0587, 65 },
    { 0, 0 },
    { 0xa27b058a, 98 },
    { 0, 0 },
    { 0xbadb058c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b0591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b0598, 61 },
    { 0, 0 },
    { 0xdd6b059a, 98 },
    { 0x6acb059b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b059f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101b05a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b05a9, 84 },
    { 0xe7bb05aa, 90 },
    { 0, 0 },
    { 0xab5b05ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264b05af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217b05ba, 33 },
    { 0x5bcb05bb, 23 },
    { 0, 0 },
    { 0x5bdb05bd, 99 },
    { 0x867b05bd, 90 },
    { 0x7d4b05bf, 23 },
    { 0xa7bb05c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b05c3, 76 },
    { 0x60bb05c4, 33 },
    { 0, 0 },
    { 0xe4fb05c6, 34 },
    { 0x889b05c7, 90 },
    { 0xe01b05c6, 19 },
    { 0x8ebb05c9, 90 },
    { 0x7d5b05c8, 93 },
    { 0x1c0b05cb, 78 },
    { 0x8d4b05c9, 46 },
    { 0x34fb05cd, 95 },
    { 0xebab05ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197b05d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8b05d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x728b05dc, 95 },
    { 0x46eb05dd, 55 },
    { 0xb86b05dd, 67 },
    { 0x6dab05dd, 55 },
    { 0, 0 },
    { 0x323b05e1, 90 },
    { 0xf86b05e2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeb05e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37b05ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4b05f2, 23 },
    { 0, 0 },
    { 0xfe5b05f4, 61 },
    { 0xb3cb05f5, 33 },
    { 0x679b05f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b05fb, 98 },
    { 0x85b05fc, 52 },
    { 0x620b05fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b0601, 95 },
    { 0, 0 },
    { 0xe85b0603, 91 },
    { 0x74ab0604, 80 },
    { 0xfc7b0605, 33 },
    { 0xdabb0606, 33 },
    { 0, 0 },
    { 0x5fb0608, 96 },
    { 0x215b0609, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38b060f, 84 },
    { 0xe78b060f, 33 },
    { 0x1c0b0611, 90 },
    { 0xefdb0612, 61 },
    { 0, 0 },
    { 0xba6b0614, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b061b, 75 },
    { 0, 0 },
    { 0xe24b061d, 95 },
    { 0, 0 },
    { 0xc88b061f, 76 },
    { 0, 0 },
    { 0xc93b0621, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab0624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b0627, 34 },
    { 0x5b7b0628, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49b062e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b0631, 87 },
    { 0x100b0632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b0637, 33 },
    { 0, 0 },
    { 0x5a3b0639, 88 },
    { 0x323b063a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5b063e, 33 },
    { 0x41db063e, 60 },
    { 0x979b0640, 23 },
    { 0xccbb0641, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b0645, 98 },
    { 0, 0 },
    { 0xefdb0647, 55 },
    { 0xc6fb0648, 87 },
    { 0xf0ab0649, 34 },
    { 0xb7bb0649, 80 },
    { 0x4c6b064a, 95 },
    { 0, 0 },
    { 0xe6ab064d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b0651, 90 },
    { 0xae6b0652, 90 },
    { 0x6a9b0653, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b0656, 61 },
    { 0x2ecb0657, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdb065f, 61 },
    { 0xc6ab0660, 34 },
    { 0, 0 },
    { 0x4f6b0662, 55 },
    { 0x350b0663, 67 },
    { 0xdfbb0663, 34 },
    { 0x9d0b0664, 45 },
    { 0xa4fb0666, 95 },
    { 0x8b4b0667, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fb066a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909b066f, 77 },
    { 0xdf6b0670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eb0674, 33 },
    { 0, 0 },
    { 0x5ddb0676, 33 },
    { 0, 0 },
    { 0x2c4b0678, 33 },
    { 0, 0 },
    { 0x993b067a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94b067d, 61 },
    { 0, 0 },
    { 0xda0b067f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b0686, 90 },
    { 0x751b0686, 90 },
    { 0x50b0687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b068e, 55 },
    { 0x841b068e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x606b0692, 84 },
    { 0, 0 },
    { 0xf23b0694, 90 },
    { 0, 0 },
    { 0xa9fb0696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1b0699, 85 },
    { 0x64cb0699, 99 },
    { 0x15cb069b, 84 },
    { 0x93ab069c, 61 },
    { 0x7db069a, 40 },
    { 0x15eb069c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9b06a2, 43 },
    { 0x522b06a3, 55 },
    { 0x202b06a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b06ac, 90 },
    { 0x446b06ad, 95 },
    { 0xb44b06ae, 33 },
    { 0x29b06af, 81 },
    { 0x651b06ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b06b5, 98 },
    { 0x2abb06b5, 53 },
    { 0xe03b06b7, 34 },
    { 0xe33b06b8, 33 },
    { 0x809b06b9, 33 },
    { 0x5a4b06ba, 55 },
    { 0, 0 },
    { 0x27fb06bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b06c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78b06c5, 33 },
    { 0, 0 },
    { 0xba2b06c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5b06cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb06cf, 66 },
    { 0xe1b06cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b06d4, 98 },
    { 0x7d8b06d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab06df, 33 },
    { 0x55db06df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cb06e3, 96 },
    { 0x98bb06e4, 61 },
    { 0x4ab06e4, 90 },
    { 0, 0 },
    { 0x9a1b06e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffb06eb, 61 },
    { 0x452b06eb, 46 },
    { 0xb81b06eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8b06f1, 61 },
    { 0, 0 },
    { 0x129b06f3, 23 },
    { 0, 0 },
    { 0xda4b06f5, 90 },
    { 0xa90b06f5, 84 },
    { 0xb7ab06f7, 76 },
    { 0xabfb06f8, 90 },
    { 0x970b06f9, 90 },
    { 0, 0 },
    { 0xfcdb06fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b0700, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ab0705, 34 },
    { 0xa68b0706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b0709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b070f, 43 },
    { 0xbc0b070f, 38 },
    { 0xa7db0711, 95 },
    { 0, 0 },
    { 0xaa8b0713, 90 },
    { 0xefab0714, 23 },
    { 0x6f3b0715, 76 },
    { 0x679b0716, 76 },
    { 0x9a8b0717, 98 },
    { 0x35eb0717, 23 },
    { 0x429b0719, 38 },
    { 0xaacb0718, 61 },
    { 0xa75b0715, 82 },
    { 0x31eb071c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0725, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b0729, 61 },
    { 0x7cab072a, 90 },
    { 0xfe8b072b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58db0730, 23 },
    { 0x106b0730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b0735, 90 },
    { 0, 0 },
    { 0xfd4b0737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ab073a, 43 },
    { 0, 0 },
    { 0xce8b073c, 90 },
    { 0, 0 },
    { 0x183b073e, 90 },
    { 0x9dbb073f, 61 },
    { 0x780b0740, 33 },
    { 0, 0 },
    { 0xfddb0742, 84 },
    { 0xedbb0743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312b0749, 67 },
    { 0x808b0749, 55 },
    { 0xd5ab074b, 90 },
    { 0x557b074b, 84 },
    { 0xc6ab074d, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7b0750, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b0753, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617b0757, 90 },
    { 0, 0 },
    { 0x2d6b0759, 61 },
    { 0xa7cb075a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b075d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b0760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x284b0763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20b0767, 34 },
    { 0xe32b0768, 33 },
    { 0x511b0769, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8b076d, 61 },
    { 0x3b5b076e, 90 },
    { 0xc11b076f, 33 },
    { 0xf5b076f, 33 },
    { 0xf52b0771, 90 },
    { 0x191b0772, 61 },
    { 0xcb1b0771, 55 },
    { 0xefab0773, 40 },
    { 0x98fb0774, 90 },
    { 0xb62b076f, 61 },
    { 0x75b0771, 86 },
    { 0xc64b0778, 91 },
    { 0x406b0774, 99 },
    { 0x9d9b077a, 84 },
    { 0x22ab0772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2b077e, 61 },
    { 0x884b077f, 65 },
    { 0xd69b0780, 100 },
    { 0, 0 },
    { 0x16ab0782, 23 },
    { 0x2b7b0782, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ab0787, 33 },
    { 0x9f9b0787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb078f, 23 },
    { 0xda8b0790, 61 },
    { 0x964b078f, 55 },
    { 0x66db0792, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41db0796, 90 },
    { 0x60b0797, 52 },
    { 0x83eb0798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb079c, 92 },
    { 0, 0 },
    { 0xb12b079e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83db07a2, 78 },
    { 0xd23b07a3, 55 },
    { 0x460b07a3, 23 },
    { 0x91fb07a2, 95 },
    { 0x4c3b07a3, 93 },
    { 0xb83b07a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab07ac, 90 },
    { 0xb71b07ad, 76 },
    { 0x41b07ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b07b1, 100 },
    { 0x88cb07b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b07b5, 99 },
    { 0, 0 },
    { 0xf0fb07b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb07bd, 90 },
    { 0x8a9b07bd, 76 },
    { 0x4d0b07bf, 33 },
    { 0, 0 },
    { 0xc83b07c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fb07c4, 61 },
    { 0, 0 },
    { 0x686b07c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9db07cc, 23 },
    { 0, 0 },
    { 0x127b07ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b07df, 55 },
    { 0, 0 },
    { 0x59ab07e1, 93 },
    { 0xa26b07e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cb07e8, 90 },
    { 0x2a4b07e8, 90 },
    { 0x555b07ea, 95 },
    { 0xe4bb07ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7b07f0, 77 },
    { 0xa09b07f0, 95 },
    { 0x97ab07f2, 95 },
    { 0xf13b07f3, 90 },
    { 0x503b07f2, 98 },
    { 0x25db07f3, 93 },
    { 0, 0 },
    { 0xf2eb07f7, 90 },
    { 0, 0 },
    { 0xfcdb07f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cb07fc, 55 },
    { 0, 0 },
    { 0xacab07fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6b0801, 33 },
    { 0x7c7b0801, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x522b0805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95b0808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb080b, 23 },
    { 0xbd9b080c, 5 },
    { 0x8fbb080c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b0811, 33 },
    { 0, 0 },
    { 0x113b0813, 33 },
    { 0xf04b0813, 88 },
    { 0x61ab0814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b081a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b081d, 55 },
    { 0x53fb081d, 93 },
    { 0xf21b081f, 95 },
    { 0x8db0820, 55 },
    { 0x89ab0821, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x187b082c, 90 },
    { 0, 0 },
    { 0x11ab082e, 78 },
    { 0, 0 },
    { 0x99eb0830, 17 },
    { 0x2db0831, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75b0837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab083a, 34 },
    { 0x33bb083b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b083f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530b0843, 90 },
    { 0, 0 },
    { 0x66cb0845, 61 },
    { 0, 0 },
    { 0x5bb0847, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8b084d, 76 },
    { 0x209b084e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bb0859, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25b085c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feb0864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab0869, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962b086d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb0871, 84 },
    { 0x221b0872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b0878, 66 },
    { 0xad2b0879, 52 },
    { 0x543b0878, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b0887, 43 },
    { 0x76bb0888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b088c, 61 },
    { 0x518b088c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab0893, 61 },
    { 0xf30b0894, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00b0897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edb08a4, 90 },
    { 0x741b08a4, 23 },
    { 0, 0 },
    { 0x68cb08a7, 59 },
    { 0xe92b08a7, 61 },
    { 0x944b08a9, 45 },
    { 0, 0 },
    { 0x270b08ab, 90 },
    { 0xf91b08ab, 98 },
    { 0xf98b08ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95b08b2, 98 },
    { 0xb77b08b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50db08ba, 33 },
    { 0x871b08ba, 23 },
    { 0xd95b08bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eb08c0, 91 },
    { 0xdb6b08c1, 81 },
    { 0x198b08c1, 98 },
    { 0x748b08c1, 88 },
    { 0x712b08c4, 90 },
    { 0xeccb08c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b08c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91b08ce, 77 },
    { 0x45db08cf, 53 },
    { 0x697b08cf, 61 },
    { 0xfdfb08d0, 78 },
    { 0xaf5b08d2, 90 },
    { 0xd54b08d3, 34 },
    { 0x35fb08ce, 76 },
    { 0x774b08cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b08dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb08e1, 23 },
    { 0, 0 },
    { 0x75eb08e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb08e9, 81 },
    { 0x5d9b08e9, 33 },
    { 0, 0 },
    { 0xd2b08ec, 80 },
    { 0xbbbb08ed, 55 },
    { 0x67bb08ee, 78 },
    { 0x12fb08ee, 33 },
    { 0x17eb08f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b08f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b08f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89b0906, 23 },
    { 0x86fb0906, 93 },
    { 0, 0 },
    { 0xc82b0909, 33 },
    { 0, 0 },
    { 0x569b090b, 90 },
    { 0x4cbb090b, 67 },
    { 0x379b090d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b0910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485b0915, 95 },
    { 0, 0 },
    { 0x68cb0917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b091a, 90 },
    { 0x7a6b091b, 95 },
    { 0, 0 },
    { 0x44ab091d, 90 },
    { 0, 0 },
    { 0xa48b091f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0b0922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456b0926, 23 },
    { 0, 0 },
    { 0x83eb0928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db092b, 34 },
    { 0x173b092c, 78 },
    { 0x77eb092d, 87 },
    { 0xeb0b092c, 91 },
    { 0x287b092b, 91 },
    { 0x338b0930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5b093f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b0944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb094a, 98 },
    { 0x902b094b, 46 },
    { 0xfb3b094c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db0959, 90 },
    { 0xe08b095a, 23 },
    { 0, 0 },
    { 0xcfab095c, 34 },
    { 0, 0 },
    { 0xdbcb095e, 33 },
    { 0x5f0b095e, 43 },
    { 0, 0 },
    { 0x25bb0961, 66 },
    { 0x6c4b0962, 23 },
    { 0x6adb0963, 100 },
    { 0x1c5b0964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbb0967, 61 },
    { 0xa36b0968, 33 },
    { 0xf1db0967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b096e, 43 },
    { 0x24cb096f, 98 },
    { 0x855b0970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bb0979, 95 },
    { 0xbdfb097a, 95 },
    { 0x398b097b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b0987, 95 },
    { 0, 0 },
    { 0x75fb0989, 87 },
    { 0, 0 },
    { 0x6a0b098b, 77 },
    { 0x5eab098c, 90 },
    { 0x816b098d, 76 },
    { 0x8ab098e, 95 },
    { 0xc74b098f, 43 },
    { 0xb96b0990, 61 },
    { 0x368b0990, 81 },
    { 0xc6fb0992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb0996, 61 },
    { 0x186b0997, 33 },
    { 0, 0 },
    { 0x7b2b0999, 33 },
    { 0, 0 },
    { 0x9e3b099b, 77 },
    { 0xc65b099c, 23 },
    { 0xf47b099d, 33 },
    { 0x74ab099e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ab09a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6b09a9, 98 },
    { 0x727b09aa, 55 },
    { 0, 0 },
    { 0x986b09ac, 52 },
    { 0xba0b09ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32b09b0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cb09b4, 43 },
    { 0, 0 },
    { 0xd91b09b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0db09b9, 43 },
    { 0xeedb09ba, 75 },
    { 0, 0 },
    { 0x122b09bc, 90 },
    { 0xeaab09bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b09c0, 99 },
    { 0, 0 },
    { 0xd7db09c2, 98 },
    { 0x2a3b09c3, 78 },
    { 0xf7bb09c4, 95 },
    { 0x8a5b09c5, 90 },
    { 0x335b09c3, 52 },
    { 0, 0 },
    { 0xe34b09c8, 95 },
    { 0x695b09c9, 61 },
    { 0x358b09c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb09ce, 67 },
    { 0, 0 },
    { 0xa05b09d0, 84 },
    { 0xbf3b09d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b09d4, 56 },
    { 0, 0 },
    { 0x9b9b09d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b09d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2b09df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12b09e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db09e8, 98 },
    { 0x717b09e9, 23 },
    { 0xf5b09e9, 93 },
    { 0xb94b09eb, 52 },
    { 0xfa0b09eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db09f1, 61 },
    { 0x9cbb09f2, 40 },
    { 0xea9b09f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbb09f7, 33 },
    { 0x8e5b09f8, 90 },
    { 0x4a8b09f8, 61 },
    { 0xf8eb09fa, 84 },
    { 0x375b09f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb09fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0a01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b0a05, 33 },
    { 0, 0 },
    { 0x3dfb0a07, 67 },
    { 0xe33b0a07, 46 },
    { 0xe79b0a09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b0a0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cb0a13, 98 },
    { 0xd17b0a14, 17 },
    { 0, 0 },
    { 0x5b3b0a16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbb0a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab0a1c, 55 },
    { 0, 0 },
    { 0xc38b0a1e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1b0a22, 61 },
    { 0xacab0a23, 99 },
    { 0xea4b0a23, 33 },
    { 0xdf0b0a23, 99 },
    { 0xa8fb0a25, 90 },
    { 0x9c7b0a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2b0a2c, 90 },
    { 0xd1eb0a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efb0a32, 22 },
    { 0x54ab0a33, 95 },
    { 0x96b0a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deb0a3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1b0a3f, 33 },
    { 0, 0 },
    { 0x3b2b0a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bb0a4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28b0a51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0a56, 98 },
    { 0x511b0a57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5b0a62, 23 },
    { 0, 0 },
    { 0xeafb0a64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2b0a69, 87 },
    { 0, 0 },
    { 0x396b0a6b, 95 },
    { 0x186b0a6c, 55 },
    { 0x281b0a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0a70, 23 },
    { 0x7d2b0a70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x629b0a74, 100 },
    { 0x6feb0a75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b0a79, 23 },
    { 0xb4ab0a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4b0a7e, 66 },
    { 0, 0 },
    { 0xc50b0a80, 65 },
    { 0, 0 },
    { 0xba6b0a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b0a85, 33 },
    { 0x328b0a86, 98 },
    { 0x1f0b0a87, 77 },
    { 0xa40b0a86, 43 },
    { 0x5e4b0a89, 33 },
    { 0, 0 },
    { 0xa19b0a8b, 56 },
    { 0, 0 },
    { 0xf63b0a8d, 55 },
    { 0x624b0a8e, 91 },
    { 0x304b0a8d, 67 },
    { 0xe6b0a8e, 23 },
    { 0x7d6b0a8d, 78 },
    { 0xbacb0a92, 90 },
    { 0, 0 },
    { 0x55eb0a94, 33 },
    { 0xdc1b0a95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb0a9a, 33 },
    { 0xaf5b0a9b, 81 },
    { 0x97b0a9a, 33 },
    { 0x165b0a9d, 98 },
    { 0, 0 },
    { 0xadbb0a9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b0aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b0aaa, 77 },
    { 0x392b0aaa, 33 },
    { 0, 0 },
    { 0xa87b0aad, 75 },
    { 0, 0 },
    { 0xb87b0aaf, 87 },
    { 0, 0 },
    { 0xe53b0ab1, 61 },
    { 0xcb6b0ab1, 34 },
    { 0xa8db0ab3, 95 },
    { 0, 0 },
    { 0x9f3b0ab5, 23 },
    { 0x72cb0ab6, 99 },
    { 0x75eb0ab6, 76 },
    { 0x620b0ab8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4b0abd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0ac1, 90 },
    { 0x88b0ac1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9b0acc, 87 },
    { 0x68cb0acd, 95 },
    { 0x3e4b0acc, 93 },
    { 0xbb9b0ace, 23 },
    { 0x7f2b0ad0, 93 },
    { 0, 0 },
    { 0xc8eb0ad2, 34 },
    { 0xe2bb0ad2, 85 },
    { 0x9deb0ad4, 46 },
    { 0xf5ab0ad2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x56db0ad8, 82 },
    { 0xc16b0ad9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198b0add, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659b0ae1, 61 },
    { 0x6f6b0ae2, 23 },
    { 0x18cb0ae3, 55 },
    { 0x9d6b0ae3, 17 },
    { 0, 0 },
    { 0x38fb0ae6, 76 },
    { 0xb1b0ae7, 98 },
    { 0, 0 },
    { 0x8ebb0ae9, 66 },
    { 0, 0 },
    { 0xb0eb0aeb, 78 },
    { 0x2a8b0aec, 33 },
    { 0x56db0aed, 98 },
    { 0, 0 },
    { 0x98b0aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3b0af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63b0af8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07b0afc, 98 },
    { 0, 0 },
    { 0xef2b0afe, 34 },
    { 0xa10b0aff, 33 },
    { 0x400b0b00, 40 },
    { 0x575b0aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863b0b08, 77 },
    { 0xe73b0b08, 46 },
    { 0, 0 },
    { 0xa3bb0b0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafb0b0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808b0b13, 90 },
    { 0xd42b0b13, 95 },
    { 0xd9eb0b15, 90 },
    { 0xf35b0b16, 95 },
    { 0xadbb0b17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab0b1c, 95 },
    { 0xc6b0b1c, 48 },
    { 0x9d8b0b1d, 100 },
    { 0x8aeb0b1c, 19 },
    { 0x47fb0b1d, 61 },
    { 0xb48b0b1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b0b25, 33 },
    { 0xc08b0b25, 98 },
    { 0, 0 },
    { 0x766b0b28, 43 },
    { 0, 0 },
    { 0xa4eb0b2a, 98 },
    { 0xabbb0b2b, 90 },
    { 0, 0 },
    { 0xe0fb0b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4b0b30, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66b0b3b, 23 },
    { 0x3f3b0b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b0b41, 52 },
    { 0, 0 },
    { 0x8b6b0b43, 61 },
    { 0x578b0b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387b0b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab0b4f, 67 },
    { 0, 0 },
    { 0xdb1b0b51, 23 },
    { 0, 0 },
    { 0x262b0b53, 61 },
    { 0x183b0b54, 61 },
    { 0, 0 },
    { 0x756b0b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db0b5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31b0b60, 87 },
    { 0x20db0b61, 90 },
    { 0xdb2b0b62, 55 },
    { 0xc36b0b63, 76 },
    { 0x9c9b0b64, 34 },
    { 0, 0 },
    { 0x503b0b66, 23 },
    { 0x9c7b0b67, 78 },
    { 0x6f1b0b67, 23 },
    { 0xf83b0b66, 88 },
    { 0x494b0b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eb0b6d, 98 },
    { 0xe2b0b6d, 67 },
    { 0, 0 },
    { 0xadbb0b70, 98 },
    { 0x119b0b71, 78 },
    { 0xbc1b0b70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b0b78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb0b7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb0b80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b0b85, 36 },
    { 0, 0 },
    { 0xc1eb0b87, 33 },
    { 0xba3b0b88, 40 },
    { 0x738b0b89, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ab0b8c, 98 },
    { 0x84cb0b8d, 23 },
    { 0, 0 },
    { 0x712b0b8f, 43 },
    { 0, 0 },
    { 0xa6db0b91, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46b0b98, 61 },
    { 0, 0 },
    { 0x1d6b0b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb0b9d, 76 },
    { 0xab6b0b9e, 90 },
    { 0x867b0b9f, 23 },
    { 0, 0 },
    { 0x241b0ba1, 95 },
    { 0x9c6b0ba1, 76 },
    { 0, 0 },
    { 0x6aeb0ba4, 49 },
    { 0xd74b0ba4, 33 },
    { 0x7e8b0ba6, 98 },
    { 0x92cb0ba5, 61 },
    { 0, 0 },
    { 0x6c4b0ba9, 61 },
    { 0x423b0baa, 65 },
    { 0xbb7b0baa, 23 },
    { 0x9f1b0bac, 34 },
    { 0x9fdb0ba9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0b0bb1, 90 },
    { 0x271b0bb2, 76 },
    { 0x87b0bb3, 52 },
    { 0x16b0bb1, 43 },
    { 0, 0 },
    { 0xe77b0bb6, 34 },
    { 0x6bcb0bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb0bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50b0bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b0bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b0bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b0bcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95b0bd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeb0bd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b0bdc, 95 },
    { 0x81fb0bdd, 75 },
    { 0x475b0bde, 17 },
    { 0, 0 },
    { 0xabab0be0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6b0be3, 43 },
    { 0x73db0be3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b0bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223b0bf1, 23 },
    { 0x5d3b0bf2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x932b0bf5, 91 },
    { 0, 0 },
    { 0xd45b0bf7, 66 },
    { 0, 0 },
    { 0x976b0bf9, 46 },
    { 0xe65b0bfa, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bb0bfd, 17 },
    { 0, 0 },
    { 0x16cb0bff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb0c02, 95 },
    { 0, 0 },
    { 0x55fb0c04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462b0c0b, 33 },
    { 0, 0 },
    { 0x4dcb0c0d, 93 },
    { 0x825b0c0e, 66 },
    { 0xff5b0c0f, 85 },
    { 0x529b0c0f, 55 },
    { 0, 0 },
    { 0x24fb0c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b0c18, 90 },
    { 0x3cfb0c18, 75 },
    { 0x804b0c1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb0c21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fb0c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbb0c35, 61 },
    { 0x79b0c35, 34 },
    { 0, 0 },
    { 0xe57b0c38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb0c41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7b0c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db0c4d, 78 },
    { 0xae7b0c4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46b0c51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526b0c56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b0c5c, 87 },
    { 0, 0 },
    { 0xe07b0c5e, 61 },
    { 0, 0 },
    { 0x55bb0c60, 33 },
    { 0, 0 },
    { 0xf68b0c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82db0c66, 23 },
    { 0, 0 },
    { 0xfa9b0c68, 61 },
    { 0, 0 },
    { 0x1e3b0c6a, 76 },
    { 0x63bb0c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbb0c70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b0c74, 95 },
    { 0, 0 },
    { 0x8a1b0c76, 55 },
    { 0xb3ab0c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb0c7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fb0c82, 67 },
    { 0, 0 },
    { 0xa79b0c84, 90 },
    { 0x60ab0c84, 64 },
    { 0x76bb0c86, 38 },
    { 0, 0 },
    { 0x141b0c88, 90 },
    { 0xf0b0c89, 61 },
    { 0xfd6b0c8a, 91 },
    { 0x2beb0c8b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b0c90, 43 },
    { 0x53db0c91, 68 },
    { 0xd52b0c92, 84 },
    { 0, 0 },
    { 0xc0bb0c94, 52 },
    { 0x79fb0c95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553b0c99, 61 },
    { 0xfe1b0c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b0ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b0ca4, 34 },
    { 0, 0 },
    { 0xd3bb0ca6, 66 },
    { 0, 0 },
    { 0x8db0ca8, 33 },
    { 0x51b0ca9, 36 },
    { 0x8db0caa, 61 },
    { 0xa54b0caa, 46 },
    { 0, 0 },
    { 0x4feb0cad, 61 },
    { 0xfc6b0cae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffb0cb4, 95 },
    { 0, 0 },
    { 0x9f6b0cb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab0cbb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8b0cbf, 99 },
    { 0x70eb0cc0, 76 },
    { 0, 0 },
    { 0x857b0cc2, 33 },
    { 0x87b0cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x579b0cc6, 33 },
    { 0xe2bb0cc7, 61 },
    { 0, 0 },
    { 0xbebb0cc9, 55 },
    { 0x1b9b0cca, 34 },
    { 0x50bb0ccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ab0cd1, 23 },
    { 0x12eb0cd2, 61 },
    { 0x470b0cd3, 90 },
    { 0x922b0cd3, 99 },
    { 0xbebb0cd5, 76 },
    { 0, 0 },
    { 0x25eb0cd7, 23 },
    { 0x73cb0cd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b0cdc, 99 },
    { 0x8c9b0cdd, 98 },
    { 0xc99b0cde, 17 },
    { 0, 0 },
    { 0xcd5b0ce0, 55 },
    { 0x60ab0ce1, 33 },
    { 0xf5bb0ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72b0ce7, 23 },
    { 0x30bb0ce8, 33 },
    { 0x26db0ce8, 55 },
    { 0, 0 },
    { 0xfa8b0ceb, 33 },
    { 0x259b0ceb, 34 },
    { 0xd16b0cec, 77 },
    { 0xc1bb0cee, 95 },
    { 0, 0 },
    { 0x4c8b0cf0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b0cf3, 90 },
    { 0xfc7b0cf3, 61 },
    { 0xbdb0cf5, 61 },
    { 0xb09b0cf5, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb0cf9, 33 },
    { 0xe46b0cf9, 90 },
    { 0xf1cb0cfa, 61 },
    { 0xc1bb0cfb, 76 },
    { 0x697b0cfd, 34 },
    { 0xef5b0cfe, 95 },
    { 0xa2bb0cfe, 99 },
    { 0, 0 },
    { 0x1b4b0d01, 90 },
    { 0xf66b0d02, 93 },
    { 0, 0 },
    { 0x86eb0d04, 67 },
    { 0x841b0d05, 98 },
    { 0xf21b0d06, 95 },
    { 0x750b0d07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b0d0b, 61 },
    { 0, 0 },
    { 0x28db0d0d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b0d10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eb0d13, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b0d1a, 61 },
    { 0x7c8b0d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db0d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b0d22, 84 },
    { 0, 0 },
    { 0x25db0d24, 43 },
    { 0, 0 },
    { 0xf2eb0d26, 90 },
    { 0x527b0d27, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2b0d2a, 76 },
    { 0x161b0d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fb0d2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348b0d37, 98 },
    { 0xfa5b0d38, 90 },
    { 0xd38b0d39, 61 },
    { 0, 0 },
    { 0x6aeb0d3b, 95 },
    { 0, 0 },
    { 0x466b0d3d, 95 },
    { 0xdf3b0d3d, 98 },
    { 0xb2cb0d3f, 90 },
    { 0x93ab0d40, 38 },
    { 0xba3b0d41, 40 },
    { 0x40db0d3f, 75 },
    { 0x23b0d3d, 66 },
    { 0x1b0d44, 55 },
    { 0xf79b0d3d, 95 },
    { 0xeeeb0d3d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcb0d4a, 87 },
    { 0, 0 },
    { 0x685b0d4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51b0d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43b0d57, 17 },
    { 0x780b0d58, 33 },
    { 0x712b0d58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab0d5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41b0d61, 98 },
    { 0, 0 },
    { 0x55fb0d63, 33 },
    { 0, 0 },
    { 0xffb0d65, 61 },
    { 0x3efb0d65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb0d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b0d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3b0d7d, 93 },
    { 0xf45b0d7d, 87 },
    { 0, 0 },
    { 0xa89b0d80, 90 },
    { 0x575b0d80, 91 },
    { 0, 0 },
    { 0xf56b0d83, 98 },
    { 0, 0 },
    { 0xc6cb0d85, 52 },
    { 0, 0 },
    { 0x58ab0d87, 90 },
    { 0x512b0d88, 55 },
    { 0x15ab0d89, 88 },
    { 0, 0 },
    { 0x7c1b0d8b, 91 },
    { 0x4d3b0d8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598b0d91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19b0d94, 76 },
    { 0x377b0d95, 34 },
    { 0, 0 },
    { 0xe2db0d97, 23 },
    { 0, 0 },
    { 0x52ab0d99, 95 },
    { 0, 0 },
    { 0x864b0d9b, 90 },
    { 0, 0 },
    { 0xf66b0d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521b0da2, 33 },
    { 0xffcb0da3, 55 },
    { 0, 0 },
    { 0x3dfb0da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b0daa, 84 },
    { 0, 0 },
    { 0xbbcb0dac, 90 },
    { 0xb3cb0dad, 90 },
    { 0xd60b0dac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab0db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169b0db8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bb0dbd, 66 },
    { 0xe30b0dbe, 33 },
    { 0xd30b0dbf, 61 },
    { 0x1c3b0dc0, 95 },
    { 0, 0 },
    { 0x67eb0dc2, 33 },
    { 0x968b0dc3, 90 },
    { 0, 0 },
    { 0xddeb0dc5, 78 },
    { 0x5fcb0dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab0dc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0dd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddb0dda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b0dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x618b0de1, 33 },
    { 0, 0 },
    { 0x6feb0de3, 93 },
    { 0, 0 },
    { 0xed1b0de5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eb0de9, 98 },
    { 0x46eb0dea, 98 },
    { 0x26cb0dea, 33 },
    { 0x566b0dea, 55 },
    { 0xb7fb0deb, 95 },
    { 0x3c7b0dee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eb0df1, 33 },
    { 0, 0 },
    { 0x310b0df3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b0df6, 67 },
    { 0x985b0df7, 48 },
    { 0x7cdb0df8, 98 },
    { 0x691b0df9, 91 },
    { 0x2bb0df7, 91 },
    { 0xa8b0df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b0e01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ab0e04, 78 },
    { 0xd8fb0e04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4b0e0a, 78 },
    { 0xc15b0e0b, 90 },
    { 0x892b0e0b, 23 },
    { 0x22bb0e0a, 95 },
    { 0x327b0e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4feb0e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb0e16, 34 },
    { 0x5a8b0e16, 61 },
    { 0xce3b0e17, 90 },
    { 0xa16b0e18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276b0e1e, 90 },
    { 0, 0 },
    { 0x9abb0e20, 33 },
    { 0x47db0e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bb0e24, 61 },
    { 0, 0 },
    { 0xf8b0e26, 100 },
    { 0x507b0e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b0e2a, 91 },
    { 0xca1b0e2b, 93 },
    { 0x957b0e2c, 61 },
    { 0x4db0e2c, 23 },
    { 0x9e9b0e2e, 75 },
    { 0, 0 },
    { 0x2d4b0e30, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab0e34, 39 },
    { 0x9b1b0e34, 33 },
    { 0xbc6b0e36, 55 },
    { 0xf40b0e36, 98 },
    { 0x4abb0e38, 46 },
    { 0xf78b0e38, 100 },
    { 0xe1db0e36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4b0e3e, 46 },
    { 0x110b0e3f, 34 },
    { 0x609b0e40, 87 },
    { 0x28b0e41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b0e46, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434b0e51, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67b0e61, 46 },
    { 0, 0 },
    { 0x4f2b0e63, 66 },
    { 0x307b0e64, 81 },
    { 0x793b0e65, 76 },
    { 0x9feb0e63, 23 },
    { 0x8f3b0e65, 75 },
    { 0, 0 },
    { 0xfd1b0e69, 43 },
    { 0, 0 },
    { 0xd19b0e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5b0e6e, 48 },
    { 0, 0 },
    { 0xcfcb0e70, 99 },
    { 0xe7bb0e70, 95 },
    { 0x3abb0e72, 66 },
    { 0x783b0e70, 98 },
    { 0xd10b0e74, 33 },
    { 0xae5b0e75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb0e79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b0e7d, 61 },
    { 0x584b0e7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00b0e81, 90 },
    { 0x3adb0e82, 42 },
    { 0xcdb0e82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb0e86, 43 },
    { 0, 0 },
    { 0x7eab0e88, 33 },
    { 0, 0 },
    { 0xf59b0e8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb0e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x922b0e95, 81 },
    { 0xe35b0e95, 98 },
    { 0, 0 },
    { 0x2b0b0e98, 91 },
    { 0, 0 },
    { 0x87b0e9a, 46 },
    { 0x9c6b0e9b, 98 },
    { 0xd36b0e9c, 98 },
    { 0xff9b0e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b0ea1, 90 },
    { 0x7a4b0ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cb0ea5, 33 },
    { 0x386b0ea5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6b0ea9, 33 },
    { 0x8d6b0eaa, 76 },
    { 0x793b0eab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb0eaf, 33 },
    { 0x6b3b0eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x303b0eb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b0eb9, 33 },
    { 0, 0 },
    { 0x230b0ebb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97b0ec0, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66b0ec4, 99 },
    { 0x6ccb0ec5, 61 },
    { 0, 0 },
    { 0xe7db0ec7, 46 },
    { 0x764b0ec7, 46 },
    { 0x3e4b0ec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x199b0ecc, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945b0ed5, 26 },
    { 0xbf5b0ed6, 98 },
    { 0, 0 },
    { 0x64db0ed8, 81 },
    { 0, 0 },
    { 0x980b0eda, 66 },
    { 0, 0 },
    { 0x7e1b0edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bb0ee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0b0eea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0eee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4b0ef4, 95 },
    { 0x34bb0ef5, 95 },
    { 0xf5b0ef6, 91 },
    { 0xc7db0ef5, 40 },
    { 0xcbab0ef8, 61 },
    { 0, 0 },
    { 0x16fb0efa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b0efe, 61 },
    { 0x484b0efe, 90 },
    { 0, 0 },
    { 0x347b0f01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234b0f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db0f0b, 23 },
    { 0xbe9b0f0b, 43 },
    { 0x603b0f0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b0f12, 76 },
    { 0x79bb0f12, 33 },
    { 0x93eb0f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b0f19, 90 },
    { 0x53ab0f1a, 61 },
    { 0x5ebb0f1b, 93 },
    { 0xa1bb0f1b, 98 },
    { 0x1eb0f1d, 81 },
    { 0x928b0f1e, 66 },
    { 0, 0 },
    { 0xe91b0f20, 90 },
    { 0, 0 },
    { 0x95fb0f22, 88 },
    { 0x54cb0f22, 33 },
    { 0xa2cb0f22, 46 },
    { 0xb8cb0f23, 95 },
    { 0xd1fb0f24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fb0f2d, 95 },
    { 0, 0 },
    { 0xdd3b0f2f, 17 },
    { 0x262b0f30, 50 },
    { 0x8e5b0f31, 87 },
    { 0x7a1b0f2f, 46 },
    { 0, 0 },
    { 0x344b0f34, 46 },
    { 0xb8bb0f35, 65 },
    { 0xa43b0f36, 95 },
    { 0x32eb0f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab0f3a, 95 },
    { 0xd93b0f3b, 33 },
    { 0x954b0f3c, 33 },
    { 0x920b0f3c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b0f40, 23 },
    { 0xa29b0f41, 100 },
    { 0, 0 },
    { 0x46db0f43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b0f4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cb0f4e, 33 },
    { 0xaa0b0f4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb0f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4b0f56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766b0f59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b0f5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b0f61, 76 },
    { 0xcd4b0f62, 98 },
    { 0x645b0f63, 95 },
    { 0, 0 },
    { 0xd0ab0f65, 95 },
    { 0xaa5b0f66, 77 },
    { 0x843b0f66, 61 },
    { 0xaeab0f68, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1b0f6b, 98 },
    { 0x86bb0f6c, 80 },
    { 0xe7bb0f6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0b0f72, 61 },
    { 0xe20b0f72, 90 },
    { 0xeafb0f72, 23 },
    { 0x2b0b0f75, 95 },
    { 0xd82b0f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb0f7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213b0f81, 91 },
    { 0x730b0f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fb0f86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb0f8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b0f91, 88 },
    { 0xb9cb0f92, 33 },
    { 0, 0 },
    { 0x3a1b0f94, 37 },
    { 0x985b0f95, 87 },
    { 0, 0 },
    { 0xe5b0f97, 65 },
    { 0xa45b0f98, 33 },
    { 0, 0 },
    { 0xf50b0f9a, 33 },
    { 0x2d2b0f9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfab0fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cb0fa4, 90 },
    { 0xa0bb0fa4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533b0fb2, 55 },
    { 0, 0 },
    { 0xc41b0fb4, 55 },
    { 0, 0 },
    { 0xa6bb0fb6, 34 },
    { 0xe35b0fb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb0fba, 98 },
    { 0, 0 },
    { 0x573b0fbc, 90 },
    { 0xf02b0fbd, 53 },
    { 0xaa6b0fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b0fc1, 33 },
    { 0xc5cb0fc2, 33 },
    { 0xc11b0fc2, 68 },
    { 0, 0 },
    { 0x133b0fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981b0fc8, 78 },
    { 0x88b0fc9, 52 },
    { 0xca8b0fca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b0fcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94b0fd0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefb0fd5, 90 },
    { 0, 0 },
    { 0x444b0fd7, 33 },
    { 0xbf8b0fd7, 95 },
    { 0xd3ab0fd9, 61 },
    { 0xcbbb0fd9, 90 },
    { 0xb62b0fdb, 90 },
    { 0, 0 },
    { 0xc1bb0fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fb0fe0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb0fec, 90 },
    { 0xdf8b0fed, 42 },
    { 0xda2b0fee, 91 },
    { 0x78eb0fef, 23 },
    { 0x22db0ff0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b0ff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68db0ffc, 88 },
    { 0xbdbb0ffd, 95 },
    { 0, 0 },
    { 0x77eb0fff, 90 },
    { 0x19eb1000, 61 },
    { 0x28bb1000, 87 },
    { 0x9aab0fff, 95 },
    { 0xa77b1000, 34 },
    { 0, 0 },
    { 0x525b1005, 90 },
    { 0xcd3b1006, 95 },
    { 0, 0 },
    { 0x2e8b1008, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aab100b, 97 },
    { 0xc27b100c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb1010, 65 },
    { 0, 0 },
    { 0xb0b1012, 61 },
    { 0, 0 },
    { 0xa48b1014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b1018, 69 },
    { 0x348b1019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ab1022, 88 },
    { 0x238b1022, 98 },
    { 0xb4fb1024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb1027, 91 },
    { 0xebfb1028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b102b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b102e, 98 },
    { 0x977b102f, 61 },
    { 0xd7b102f, 95 },
    { 0x6ecb1030, 52 },
    { 0x856b102f, 61 },
    { 0, 0 },
    { 0xb14b1034, 33 },
    { 0xbedb1035, 37 },
    { 0x69cb1036, 91 },
    { 0x57cb1036, 43 },
    { 0xd23b1038, 33 },
    { 0xccbb1038, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b103d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b1042, 61 },
    { 0x3afb1042, 34 },
    { 0, 0 },
    { 0x7b0b1045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b1049, 90 },
    { 0x1e2b104a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x135b104d, 69 },
    { 0xa97b104e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb1056, 90 },
    { 0, 0 },
    { 0x37eb1058, 99 },
    { 0x72ab1059, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9b105c, 66 },
    { 0x959b105d, 88 },
    { 0, 0 },
    { 0xd05b105f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aab1064, 100 },
    { 0, 0 },
    { 0xca2b1066, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18b1069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b106f, 46 },
    { 0x87bb106f, 90 },
    { 0, 0 },
    { 0x781b1072, 90 },
    { 0x38b1073, 90 },
    { 0x94fb1074, 33 },
    { 0x318b1075, 23 },
    { 0x5cdb1076, 100 },
    { 0x56bb1077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb107d, 55 },
    { 0x51cb107e, 95 },
    { 0xbe5b107f, 61 },
    { 0xe50b1080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fb1085, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb1089, 76 },
    { 0, 0 },
    { 0x17cb108b, 75 },
    { 0x343b108c, 98 },
    { 0x148b108b, 80 },
    { 0x632b108d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b1096, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b109d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90b10a2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b10a9, 95 },
    { 0x3bdb10a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb10ae, 90 },
    { 0x896b10af, 98 },
    { 0, 0 },
    { 0x9c4b10b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b10b4, 55 },
    { 0x301b10b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9b10be, 23 },
    { 0xdecb10bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb10c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786b10c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b10c9, 61 },
    { 0, 0 },
    { 0x203b10cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b10ce, 90 },
    { 0xf1eb10cf, 95 },
    { 0xc3cb10ce, 98 },
    { 0x705b10d1, 43 },
    { 0xa7fb10d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab10d6, 90 },
    { 0, 0 },
    { 0x15bb10d8, 55 },
    { 0xa60b10d9, 23 },
    { 0xce3b10da, 91 },
    { 0xa2eb10da, 33 },
    { 0x4edb10dc, 90 },
    { 0x439b10db, 93 },
    { 0x11ab10dc, 67 },
    { 0x2c8b10de, 33 },
    { 0xb68b10e0, 87 },
    { 0, 0 },
    { 0x68eb10e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145b10ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b10f0, 34 },
    { 0x94ab10f1, 33 },
    { 0x62bb10f1, 61 },
    { 0xd50b10f2, 33 },
    { 0x1dfb10f4, 95 },
    { 0xba7b10f1, 33 },
    { 0x677b10f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fb10fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b1101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eb1106, 98 },
    { 0x3ecb1106, 66 },
    { 0, 0 },
    { 0x96ab1109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b110c, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536b1114, 98 },
    { 0xa3cb1115, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b1118, 33 },
    { 0x4cbb1119, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x266b111c, 93 },
    { 0x772b111d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x868b1120, 33 },
    { 0x425b1120, 91 },
    { 0x5a9b1122, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b1125, 61 },
    { 0, 0 },
    { 0xbc7b1127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8b112e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb1132, 99 },
    { 0xbf5b1133, 61 },
    { 0xca6b1132, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382b113e, 100 },
    { 0x6d1b113f, 33 },
    { 0xcacb113e, 33 },
    { 0x609b113f, 55 },
    { 0xcf4b113e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9b1145, 61 },
    { 0x67eb1146, 55 },
    { 0xfa0b1146, 66 },
    { 0xa62b1148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb114c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb1152, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b1156, 87 },
    { 0, 0 },
    { 0x16ab1158, 95 },
    { 0xde9b1159, 61 },
    { 0x621b1158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0b115f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7b1163, 90 },
    { 0xb1ab1163, 76 },
    { 0x442b1164, 34 },
    { 0, 0 },
    { 0x44cb1167, 61 },
    { 0, 0 },
    { 0xaf4b1169, 23 },
    { 0x652b1169, 95 },
    { 0x6fbb116b, 76 },
    { 0, 0 },
    { 0x740b116d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69b1174, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ab117a, 67 },
    { 0, 0 },
    { 0x332b117c, 90 },
    { 0, 0 },
    { 0xaacb117e, 34 },
    { 0, 0 },
    { 0x6deb1180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b1183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b118c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b1191, 75 },
    { 0xdb7b1192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab1198, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfb119c, 80 },
    { 0x50db119d, 68 },
    { 0x834b119e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb11a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6b11a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab11ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb11b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb11b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b11b9, 84 },
    { 0xd38b11ba, 33 },
    { 0, 0 },
    { 0x10fb11bc, 33 },
    { 0x726b11bd, 98 },
    { 0x314b11bc, 93 },
    { 0xa16b11be, 66 },
    { 0xac7b11c0, 55 },
    { 0, 0 },
    { 0x7b8b11c2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x24db11c5, 93 },
    { 0, 0 },
    { 0x8bab11c7, 98 },
    { 0x677b11c7, 93 },
    { 0, 0 },
    { 0x3eb11ca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b11ce, 33 },
    { 0xc34b11ce, 76 },
    { 0x248b11d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32b11d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9b11da, 68 },
    { 0, 0 },
    { 0xec7b11dc, 33 },
    { 0xbd8b11dd, 61 },
    { 0, 0 },
    { 0xa89b11df, 33 },
    { 0x123b11e0, 95 },
    { 0xd4b11df, 61 },
    { 0x141b11e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecb11e9, 90 },
    { 0x41db11e9, 33 },
    { 0, 0 },
    { 0x680b11ec, 90 },
    { 0xd8eb11ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x660b11f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0b11f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73b11f9, 98 },
    { 0xf10b11fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401b1200, 76 },
    { 0x421b1201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x660b1204, 46 },
    { 0x6f4b1205, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b120b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b1213, 61 },
    { 0x66db1213, 96 },
    { 0, 0 },
    { 0x21b1216, 23 },
    { 0, 0 },
    { 0xd3fb1218, 55 },
    { 0xd99b1219, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06b1223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231b1229, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672b1232, 33 },
    { 0x41cb1233, 76 },
    { 0, 0 },
    { 0x997b1235, 33 },
    { 0, 0 },
    { 0xdaab1237, 98 },
    { 0x25eb1238, 80 },
    { 0x7eab1238, 93 },
    { 0, 0 },
    { 0x61eb123b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb123e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab1241, 17 },
    { 0x8a6b1242, 46 },
    { 0x142b1243, 78 },
    { 0xb58b1243, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x431b1247, 75 },
    { 0x37eb1248, 87 },
    { 0x43db1248, 33 },
    { 0xd36b1247, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522b124d, 23 },
    { 0x36eb124e, 33 },
    { 0xc60b124f, 33 },
    { 0x539b1250, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34b1257, 61 },
    { 0x56fb1258, 98 },
    { 0, 0 },
    { 0x112b125a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb125d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4b1260, 55 },
    { 0x7fb1261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38b1265, 55 },
    { 0xb01b1266, 95 },
    { 0x5fbb1267, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b1275, 23 },
    { 0x3bb1276, 61 },
    { 0x9b1275, 90 },
    { 0, 0 },
    { 0x4bdb1279, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb127c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x555b127f, 90 },
    { 0x13bb127f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b1286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcb128a, 98 },
    { 0x6deb128a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5b1296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35b129a, 67 },
    { 0xb97b129b, 99 },
    { 0x83cb129c, 80 },
    { 0x737b129c, 84 },
    { 0x304b129c, 85 },
    { 0xb5cb129f, 90 },
    { 0x517b12a0, 90 },
    { 0xf1fb12a1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb12a8, 61 },
    { 0, 0 },
    { 0x860b12aa, 93 },
    { 0x4b12ab, 61 },
    { 0xdfeb12ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b12b2, 55 },
    { 0, 0 },
    { 0xd13b12b4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60b12b9, 97 },
    { 0xa70b12ba, 95 },
    { 0xf73b12bb, 99 },
    { 0, 0 },
    { 0x778b12bd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b12c1, 34 },
    { 0, 0 },
    { 0x392b12c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ab12c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7b12cc, 61 },
    { 0, 0 },
    { 0xc0b12ce, 55 },
    { 0xa24b12cf, 55 },
    { 0x807b12cf, 95 },
    { 0, 0 },
    { 0xd1bb12d2, 100 },
    { 0xc38b12d2, 24 },
    { 0xad1b12d4, 33 },
    { 0x675b12d5, 61 },
    { 0xedfb12d6, 33 },
    { 0xbb7b12d6, 99 },
    { 0xcdfb12d8, 43 },
    { 0xb52b12d9, 77 },
    { 0, 0 },
    { 0xe0cb12db, 95 },
    { 0x6ceb12dc, 99 },
    { 0x44fb12dd, 66 },
    { 0xa52b12de, 66 },
    { 0xf1ab12df, 81 },
    { 0x9afb12e0, 61 },
    { 0xb02b12de, 90 },
    { 0, 0 },
    { 0x1a2b12e3, 61 },
    { 0, 0 },
    { 0xe1bb12e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b12ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95b12ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb12f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb12f7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3b12fb, 77 },
    { 0xc42b12fc, 51 },
    { 0x48b12fb, 33 },
    { 0, 0 },
    { 0x143b12ff, 76 },
    { 0, 0 },
    { 0x29bb1301, 90 },
    { 0x83cb1301, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23b1309, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb1312, 68 },
    { 0, 0 },
    { 0xe0ab1314, 33 },
    { 0xa55b1314, 98 },
    { 0, 0 },
    { 0xe13b1317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceb1322, 90 },
    { 0x5d7b1323, 23 },
    { 0x7b5b1324, 61 },
    { 0x839b1325, 98 },
    { 0x6ceb1326, 76 },
    { 0x4db1327, 88 },
    { 0x8c8b1327, 87 },
    { 0xc72b1327, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb132c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6b1332, 91 },
    { 0, 0 },
    { 0xf7bb1334, 90 },
    { 0x887b1335, 95 },
    { 0, 0 },
    { 0x9dfb1337, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb133c, 76 },
    { 0xae3b133d, 76 },
    { 0x8e4b133e, 33 },
    { 0xedfb133f, 88 },
    { 0, 0 },
    { 0x69bb1341, 84 },
    { 0x702b1341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecb1347, 34 },
    { 0x353b1348, 43 },
    { 0x9cfb1349, 90 },
    { 0xcb0b134a, 90 },
    { 0xbccb134a, 90 },
    { 0xa47b134a, 90 },
    { 0x7eb134d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb1351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9b1356, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb135a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9eb1360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb1365, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb1368, 90 },
    { 0, 0 },
    { 0xbfbb136a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acb136d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b1370, 99 },
    { 0x237b1371, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04b137c, 33 },
    { 0x9dfb137c, 23 },
    { 0x549b137e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b1382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5b1387, 76 },
    { 0xa90b1388, 33 },
    { 0xffeb1388, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b138e, 34 },
    { 0, 0 },
    { 0x6edb1390, 33 },
    { 0xda5b1391, 100 },
    { 0xd72b1392, 55 },
    { 0x402b1391, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4b1396, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9b139a, 33 },
    { 0, 0 },
    { 0x7d3b139c, 76 },
    { 0x23db139d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71b13a8, 67 },
    { 0, 0 },
    { 0xa27b13aa, 33 },
    { 0x10eb13aa, 52 },
    { 0x3ceb13ab, 93 },
    { 0xc16b13ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b13b0, 93 },
    { 0x6eab13b1, 99 },
    { 0x2b5b13b2, 75 },
    { 0, 0 },
    { 0x38b13b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94b13b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b13ba, 34 },
    { 0xc2cb13bb, 90 },
    { 0x9c6b13bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603b13c0, 76 },
    { 0x11cb13c1, 23 },
    { 0, 0 },
    { 0x36cb13c3, 55 },
    { 0xc0cb13c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab13c7, 19 },
    { 0x8deb13c8, 12 },
    { 0xce3b13c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b13d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706b13d7, 99 },
    { 0, 0 },
    { 0xae0b13d9, 33 },
    { 0xce3b13da, 98 },
    { 0, 0 },
    { 0xe1cb13dc, 61 },
    { 0, 0 },
    { 0x418b13de, 99 },
    { 0xd3bb13df, 33 },
    { 0x117b13e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18b13e8, 33 },
    { 0xa4fb13e9, 61 },
    { 0xc1bb13ea, 61 },
    { 0x8afb13eb, 91 },
    { 0x7c9b13ea, 46 },
    { 0x4cdb13ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b13f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6b13fb, 34 },
    { 0x50bb13fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3b1402, 90 },
    { 0, 0 },
    { 0x187b1404, 55 },
    { 0, 0 },
    { 0x6b0b1406, 90 },
    { 0, 0 },
    { 0x54fb1408, 90 },
    { 0xb62b1408, 90 },
    { 0xea3b140a, 61 },
    { 0x68ab140a, 69 },
    { 0xe2fb140c, 46 },
    { 0xe82b140a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fb1413, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdab1419, 100 },
    { 0x7f9b141a, 84 },
    { 0x962b141b, 68 },
    { 0xb43b1419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b1422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb1426, 66 },
    { 0xa16b1427, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b142d, 95 },
    { 0x85db142d, 52 },
    { 0x7ab142e, 90 },
    { 0xe71b1430, 33 },
    { 0, 0 },
    { 0x846b1432, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87b1435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b1438, 95 },
    { 0x6d0b1439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ab143e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b1447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b144e, 95 },
    { 0xee0b144f, 93 },
    { 0, 0 },
    { 0xb5fb1451, 95 },
    { 0xfcbb1452, 52 },
    { 0, 0 },
    { 0xeb6b1454, 33 },
    { 0xc05b1455, 90 },
    { 0x6eb1456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb145d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb1460, 93 },
    { 0xb79b1460, 96 },
    { 0xd3db1462, 76 },
    { 0xb0bb1462, 95 },
    { 0x2f2b1462, 61 },
    { 0x5a7b1465, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb146e, 33 },
    { 0xd00b146f, 33 },
    { 0x2b0b1470, 90 },
    { 0x7c4b1470, 90 },
    { 0x9c6b1472, 90 },
    { 0x16eb1473, 95 },
    { 0, 0 },
    { 0xf89b1475, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b1478, 88 },
    { 0x267b1478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dab147d, 34 },
    { 0, 0 },
    { 0xee7b147f, 61 },
    { 0xb5eb1480, 61 },
    { 0x903b1480, 95 },
    { 0xb80b1480, 61 },
    { 0x9bdb1481, 90 },
    { 0x46db1484, 90 },
    { 0xc74b1484, 55 },
    { 0x47cb1486, 33 },
    { 0x37b1487, 85 },
    { 0x3f3b1487, 61 },
    { 0x46bb1487, 98 },
    { 0, 0 },
    { 0xc3fb148b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ab148e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b1493, 95 },
    { 0, 0 },
    { 0x960b1495, 55 },
    { 0, 0 },
    { 0x56ab1497, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b149e, 95 },
    { 0x23cb149f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb14a2, 98 },
    { 0xe42b14a2, 23 },
    { 0, 0 },
    { 0xb92b14a5, 95 },
    { 0x3a1b14a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176b14aa, 34 },
    { 0xda9b14ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b14b3, 15 },
    { 0xc29b14b3, 98 },
    { 0, 0 },
    { 0xcdeb14b6, 81 },
    { 0x4f6b14b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b14bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb14c3, 34 },
    { 0x542b14c4, 55 },
    { 0xc3bb14c4, 98 },
    { 0x9f3b14c6, 66 },
    { 0x655b14c7, 95 },
    { 0x92bb14c8, 95 },
    { 0x534b14c8, 94 },
    { 0x39fb14ca, 100 },
    { 0xc5b14cb, 43 },
    { 0xf91b14c5, 33 },
    { 0, 0 },
    { 0xecab14ce, 23 },
    { 0x8b2b14cf, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b14d3, 33 },
    { 0xbfab14d4, 76 },
    { 0x707b14d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4b14d9, 98 },
    { 0, 0 },
    { 0xe09b14db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b14df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cb14e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b14e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb14f8, 98 },
    { 0, 0 },
    { 0xdc6b14fa, 90 },
    { 0x177b14fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05b1502, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b150b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb150e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbb1515, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0b1518, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb1521, 95 },
    { 0xe4fb1522, 61 },
    { 0x9d6b1523, 85 },
    { 0xd51b1521, 90 },
    { 0xde7b1525, 61 },
    { 0, 0 },
    { 0xcb9b1527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13db152a, 78 },
    { 0, 0 },
    { 0xa49b152c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab1533, 90 },
    { 0, 0 },
    { 0x78eb1535, 33 },
    { 0xbf9b1536, 46 },
    { 0, 0 },
    { 0x558b1538, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eeb153b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b153f, 95 },
    { 0x332b1540, 98 },
    { 0, 0 },
    { 0x675b1542, 95 },
    { 0x475b1543, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29b1548, 83 },
    { 0x193b1549, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db154e, 23 },
    { 0, 0 },
    { 0xed4b1550, 77 },
    { 0, 0 },
    { 0x1cdb1552, 52 },
    { 0xdd5b1552, 61 },
    { 0, 0 },
    { 0x912b1555, 96 },
    { 0xbefb1556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb1559, 78 },
    { 0xfe2b1559, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x356b155d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553b1561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0b1564, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388b1568, 33 },
    { 0, 0 },
    { 0xbbeb156a, 95 },
    { 0x61db156b, 76 },
    { 0xf2bb156c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b1570, 76 },
    { 0xb12b1570, 46 },
    { 0, 0 },
    { 0xa92b1573, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cab1576, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab1579, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cb1580, 23 },
    { 0x868b1581, 33 },
    { 0xa38b1582, 95 },
    { 0x302b1583, 46 },
    { 0, 0 },
    { 0x199b1585, 87 },
    { 0x1c6b1585, 52 },
    { 0x200b1586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b158b, 24 },
    { 0x1bcb158b, 69 },
    { 0x9b9b158c, 40 },
    { 0xee0b158e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b1593, 23 },
    { 0, 0 },
    { 0x4cb1595, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4b159b, 95 },
    { 0x83b159c, 84 },
    { 0xfb1b159c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb15a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb15a8, 95 },
    { 0xbdcb15a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccb15ac, 23 },
    { 0x7c2b15ac, 98 },
    { 0, 0 },
    { 0xbd1b15af, 91 },
    { 0x8cb15b0, 43 },
    { 0xa1cb15b0, 23 },
    { 0xdecb15b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37b15b8, 95 },
    { 0xaa9b15b9, 99 },
    { 0, 0 },
    { 0x6fb15bb, 90 },
    { 0xf8b15bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7b15bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedb15c2, 98 },
    { 0x638b15c2, 43 },
    { 0, 0 },
    { 0xc69b15c5, 91 },
    { 0x33bb15c6, 66 },
    { 0xce0b15c6, 34 },
    { 0, 0 },
    { 0x1cb15c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3b15ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0b15d9, 98 },
    { 0xf10b15da, 90 },
    { 0xb8fb15db, 85 },
    { 0x731b15dc, 95 },
    { 0xbbab15dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb15e5, 75 },
    { 0, 0 },
    { 0x8e3b15e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29b15ea, 48 },
    { 0, 0 },
    { 0xbcfb15ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb15ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0b15f2, 33 },
    { 0x4c1b15f3, 90 },
    { 0xb98b15f4, 61 },
    { 0xeceb15f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfb15f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5b15fc, 90 },
    { 0xcf5b15fc, 95 },
    { 0x47eb15fe, 55 },
    { 0, 0 },
    { 0x531b1600, 34 },
    { 0x343b1601, 84 },
    { 0x9f3b1601, 93 },
    { 0xb42b1603, 90 },
    { 0x4fcb1602, 98 },
    { 0xd95b1605, 52 },
    { 0xa00b1605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77b160e, 61 },
    { 0xa39b160f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabb1612, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658b1616, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb1619, 98 },
    { 0, 0 },
    { 0xd68b161b, 95 },
    { 0, 0 },
    { 0x47cb161d, 52 },
    { 0x28fb161e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2b1624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0b1628, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b162c, 66 },
    { 0, 0 },
    { 0x1dbb162e, 98 },
    { 0, 0 },
    { 0x368b1630, 78 },
    { 0, 0 },
    { 0x344b1632, 95 },
    { 0xf68b1633, 78 },
    { 0x871b1634, 84 },
    { 0xc8fb1633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b1638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7b163b, 61 },
    { 0, 0 },
    { 0x609b163d, 34 },
    { 0x9dcb163e, 99 },
    { 0xbefb163e, 93 },
    { 0xb24b1640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b1648, 88 },
    { 0xfcfb1648, 34 },
    { 0xba7b1649, 84 },
    { 0x53ab164a, 66 },
    { 0x63db164c, 95 },
    { 0x60eb164d, 40 },
    { 0, 0 },
    { 0x4e1b164f, 78 },
    { 0x7cb1650, 23 },
    { 0, 0 },
    { 0xf52b1652, 33 },
    { 0xe7eb1653, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97db1657, 80 },
    { 0x697b1657, 43 },
    { 0xf24b1658, 34 },
    { 0x6a2b1658, 95 },
    { 0, 0 },
    { 0x924b165c, 23 },
    { 0x6a6b165d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb1660, 90 },
    { 0x3ab1661, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb1666, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b166b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264b166f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754b1674, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b167b, 61 },
    { 0, 0 },
    { 0xb34b167d, 97 },
    { 0, 0 },
    { 0xd5fb167f, 90 },
    { 0x153b1680, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b1683, 95 },
    { 0x2f6b1684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991b1689, 90 },
    { 0x911b168a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb168d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fb1695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b1698, 93 },
    { 0x458b1699, 76 },
    { 0xe0b1698, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355b169e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b16a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eb16a8, 33 },
    { 0x735b16a9, 52 },
    { 0x8b4b16a8, 98 },
    { 0xe20b16aa, 52 },
    { 0xc0db16ac, 100 },
    { 0x47eb16ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb16b0, 53 },
    { 0xc8db16b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b16b6, 43 },
    { 0xee8b16b6, 23 },
    { 0, 0 },
    { 0xd9eb16b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfeb16bd, 36 },
    { 0, 0 },
    { 0x7db16bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82b16c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241b16cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b16d4, 55 },
    { 0, 0 },
    { 0x8db16d6, 95 },
    { 0xe4db16d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b16da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bb16dd, 33 },
    { 0xf12b16de, 90 },
    { 0, 0 },
    { 0x885b16e0, 95 },
    { 0xce2b16e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b16e9, 78 },
    { 0x62db16e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eb16ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cab16f1, 85 },
    { 0x7b16f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b16f7, 55 },
    { 0x6f8b16f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7db16ff, 61 },
    { 0x42db16ff, 75 },
    { 0x90eb1700, 95 },
    { 0x1b8b1702, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab1705, 90 },
    { 0, 0 },
    { 0x245b1707, 99 },
    { 0, 0 },
    { 0x7edb1709, 34 },
    { 0x17cb170a, 95 },
    { 0xcfab170a, 23 },
    { 0x422b1709, 85 },
    { 0, 0 },
    { 0xc14b170e, 98 },
    { 0xd55b170f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb1713, 23 },
    { 0xb3b1714, 75 },
    { 0x21eb1714, 90 },
    { 0x4b6b1714, 75 },
    { 0, 0 },
    { 0x329b1718, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fb1722, 91 },
    { 0xb45b1723, 98 },
    { 0, 0 },
    { 0x6d7b1725, 61 },
    { 0x30bb1726, 46 },
    { 0x133b1727, 23 },
    { 0xc5eb1727, 65 },
    { 0, 0 },
    { 0xa75b172a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb172d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43b1733, 55 },
    { 0, 0 },
    { 0xa68b1735, 46 },
    { 0x9ddb1736, 61 },
    { 0x163b1735, 98 },
    { 0xea6b1738, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629b173c, 90 },
    { 0, 0 },
    { 0xb59b173e, 90 },
    { 0x1c6b173f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beb1742, 76 },
    { 0, 0 },
    { 0x50b1744, 23 },
    { 0, 0 },
    { 0x5a3b1746, 67 },
    { 0xa71b1747, 90 },
    { 0x1a3b1748, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b174d, 90 },
    { 0x217b174d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8db1753, 61 },
    { 0x200b1754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb1759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b175c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45b1762, 90 },
    { 0, 0 },
    { 0x311b1764, 81 },
    { 0x5f6b1765, 99 },
    { 0x9b9b1766, 76 },
    { 0xde2b1767, 90 },
    { 0xc39b1766, 61 },
    { 0, 0 },
    { 0xaf9b176a, 68 },
    { 0, 0 },
    { 0x79eb176c, 33 },
    { 0x1cfb176d, 51 },
    { 0xaf2b176d, 98 },
    { 0xd80b176f, 76 },
    { 0x30b176d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb1774, 61 },
    { 0xcdab1775, 23 },
    { 0, 0 },
    { 0xd47b1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b177c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb177f, 52 },
    { 0xdebb1780, 34 },
    { 0x15db177f, 90 },
    { 0x30cb1780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eb1785, 52 },
    { 0x7f6b1785, 61 },
    { 0x4ccb1787, 67 },
    { 0, 0 },
    { 0xf6fb1789, 95 },
    { 0x85db178a, 93 },
    { 0x83b178b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eb1791, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbb1795, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1799, 61 },
    { 0, 0 },
    { 0xe83b179b, 46 },
    { 0, 0 },
    { 0x8ccb179d, 61 },
    { 0xebeb179e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b17a2, 98 },
    { 0x199b17a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751b17a9, 85 },
    { 0, 0 },
    { 0xfab17ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b17b1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb17b9, 81 },
    { 0xcf2b17ba, 23 },
    { 0x35db17b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b17c3, 23 },
    { 0, 0 },
    { 0x8c0b17c5, 90 },
    { 0x43ab17c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x344b17c9, 90 },
    { 0x853b17ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab17cf, 95 },
    { 0x2d8b17d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437b17d5, 91 },
    { 0, 0 },
    { 0x1cab17d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb17dc, 100 },
    { 0xaf0b17dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b17e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x909b17e4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceb17e7, 76 },
    { 0, 0 },
    { 0x88fb17e9, 91 },
    { 0x6f7b17ea, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b17f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92b17f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb17f9, 90 },
    { 0xcb8b17fa, 52 },
    { 0xb2bb17fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b1802, 87 },
    { 0xe95b1803, 61 },
    { 0xf44b1804, 90 },
    { 0x33fb1802, 76 },
    { 0x748b1804, 33 },
    { 0xbfcb1807, 61 },
    { 0x409b1808, 23 },
    { 0xcc9b1809, 46 },
    { 0xeacb1808, 61 },
    { 0x838b1803, 100 },
    { 0x76fb1808, 90 },
    { 0xa45b180d, 55 },
    { 0xd68b180e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9b1811, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b1815, 95 },
    { 0xe63b1815, 90 },
    { 0xc48b1817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab181b, 77 },
    { 0x96db181c, 95 },
    { 0xb13b181d, 87 },
    { 0, 0 },
    { 0x86eb181f, 33 },
    { 0x3b0b181f, 85 },
    { 0, 0 },
    { 0x7e8b1822, 95 },
    { 0xfd0b1823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x294b1826, 55 },
    { 0, 0 },
    { 0x2cb1828, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3b1830, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b1836, 100 },
    { 0x8a5b1837, 90 },
    { 0x7c4b1836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02b183c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb1840, 52 },
    { 0xedb1840, 33 },
    { 0, 0 },
    { 0xdb7b1843, 98 },
    { 0xd80b1843, 80 },
    { 0, 0 },
    { 0x44bb1846, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb184a, 68 },
    { 0x6e6b184a, 75 },
    { 0x18fb184c, 90 },
    { 0xb3bb184c, 75 },
    { 0, 0 },
    { 0x8dfb184f, 95 },
    { 0xf9db1850, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x954b1853, 90 },
    { 0x134b1853, 98 },
    { 0xcadb1855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cb185a, 95 },
    { 0, 0 },
    { 0xf00b185c, 23 },
    { 0x5ccb185d, 40 },
    { 0x7fb185e, 98 },
    { 0xa64b185e, 95 },
    { 0xa89b185e, 33 },
    { 0, 0 },
    { 0x91ab1862, 99 },
    { 0, 0 },
    { 0x80bb1864, 90 },
    { 0, 0 },
    { 0xa12b1866, 61 },
    { 0xb75b1867, 61 },
    { 0x5a2b1866, 23 },
    { 0x78ab1866, 23 },
    { 0, 0 },
    { 0x600b186b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab186e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69b1872, 95 },
    { 0x4aeb1873, 33 },
    { 0, 0 },
    { 0xd2eb1875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b1878, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64b187b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb187e, 96 },
    { 0x22db187e, 90 },
    { 0x1a4b1880, 61 },
    { 0x933b1881, 90 },
    { 0x1bcb1882, 33 },
    { 0x727b1883, 99 },
    { 0x502b1882, 66 },
    { 0x8e6b1885, 95 },
    { 0xccb1886, 34 },
    { 0xcb8b1887, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14b188b, 61 },
    { 0x4c5b188c, 90 },
    { 0, 0 },
    { 0x82b188e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb1892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680b1898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43b189e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1b18a1, 76 },
    { 0x138b18a2, 90 },
    { 0xaa5b18a1, 55 },
    { 0xeceb18a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0b18ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b18b1, 23 },
    { 0x459b18b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66b18b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b18ba, 33 },
    { 0xbcfb18bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcb18bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98b18c4, 90 },
    { 0, 0 },
    { 0x726b18c6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8b18c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60db18cf, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b18d3, 87 },
    { 0, 0 },
    { 0x2edb18d5, 55 },
    { 0x769b18d5, 93 },
    { 0, 0 },
    { 0x1b7b18d8, 98 },
    { 0x6bb18d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9b18de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b18e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2b18e6, 33 },
    { 0xa81b18e7, 93 },
    { 0x90bb18e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bb18f0, 23 },
    { 0x5e7b18f1, 90 },
    { 0xc41b18f0, 33 },
    { 0x2d5b18f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x478b18f6, 84 },
    { 0x71cb18f7, 90 },
    { 0x53fb18f7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b1900, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2b1903, 90 },
    { 0x5eeb1904, 55 },
    { 0, 0 },
    { 0x7b1b1906, 84 },
    { 0x38fb1906, 33 },
    { 0x49ab1906, 95 },
    { 0x2d8b1909, 98 },
    { 0x5c1b190a, 23 },
    { 0x4dab190b, 33 },
    { 0x155b190b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984b1911, 90 },
    { 0xbdab1912, 98 },
    { 0, 0 },
    { 0xe40b1914, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ab191e, 33 },
    { 0x83eb191f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b1923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0b1928, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ab192c, 93 },
    { 0xac9b192c, 90 },
    { 0x1bbb192e, 95 },
    { 0x876b192d, 34 },
    { 0xc9ab1930, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eb1936, 69 },
    { 0, 0 },
    { 0x33fb1938, 100 },
    { 0x46cb1939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70b1941, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b1944, 33 },
    { 0xcf6b1945, 76 },
    { 0x680b1945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3b1949, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6b194c, 99 },
    { 0xd28b194d, 43 },
    { 0xa6ab194e, 23 },
    { 0x8feb194f, 78 },
    { 0, 0 },
    { 0xd2bb1951, 90 },
    { 0x67b1951, 85 },
    { 0x136b1952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572b1957, 95 },
    { 0, 0 },
    { 0x796b1959, 55 },
    { 0x610b195a, 34 },
    { 0xa69b1959, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bb195f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb1962, 52 },
    { 0x94db1963, 55 },
    { 0x44cb1963, 98 },
    { 0xffcb1965, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32b196a, 81 },
    { 0xd38b196b, 67 },
    { 0xd2db196c, 88 },
    { 0xb40b196d, 90 },
    { 0xf95b196e, 61 },
    { 0x601b196e, 52 },
    { 0x891b196d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab1973, 100 },
    { 0x333b1974, 23 },
    { 0x186b1973, 95 },
    { 0x13ab1973, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b1979, 61 },
    { 0x967b197a, 34 },
    { 0xd7db197a, 90 },
    { 0x253b197b, 33 },
    { 0, 0 },
    { 0x6cbb197e, 46 },
    { 0x429b197e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789b1984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cb1987, 64 },
    { 0xe35b1988, 67 },
    { 0x81eb1989, 95 },
    { 0x533b1987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b198d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb1991, 23 },
    { 0, 0 },
    { 0x8a4b1993, 17 },
    { 0x2a8b1994, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab1999, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9b19a4, 90 },
    { 0x8d7b19a5, 88 },
    { 0, 0 },
    { 0x7ab19a7, 95 },
    { 0x7e3b19a8, 46 },
    { 0x2db19a9, 61 },
    { 0, 0 },
    { 0xea9b19ab, 66 },
    { 0x63db19ac, 33 },
    { 0x7e0b19ad, 91 },
    { 0xbf1b19ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db19b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b19b6, 48 },
    { 0x4a2b19b7, 90 },
    { 0, 0 },
    { 0xcd2b19b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b19c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753b19c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb19ca, 80 },
    { 0, 0 },
    { 0xec1b19cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661b19d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62b19d5, 98 },
    { 0x520b19d6, 90 },
    { 0xc57b19d7, 33 },
    { 0xf2eb19d7, 34 },
    { 0xaccb19d7, 81 },
    { 0x5f5b19da, 90 },
    { 0x65fb19d5, 55 },
    { 0x490b19dc, 23 },
    { 0xf9bb19dd, 76 },
    { 0x19db19de, 23 },
    { 0x481b19df, 98 },
    { 0, 0 },
    { 0xe7b19e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b19e4, 43 },
    { 0, 0 },
    { 0xa91b19e6, 23 },
    { 0x7b1b19e7, 87 },
    { 0x7f9b19e8, 66 },
    { 0xb59b19e8, 98 },
    { 0x3d7b19ea, 78 },
    { 0, 0 },
    { 0x893b19ec, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb19f2, 33 },
    { 0xc73b19f3, 90 },
    { 0xca2b19f2, 43 },
    { 0x4efb19f5, 33 },
    { 0x80db19f4, 43 },
    { 0x2c3b19f7, 76 },
    { 0x155b19f8, 78 },
    { 0x2dcb19f7, 61 },
    { 0xc40b19fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b1a05, 77 },
    { 0xfd9b1a06, 33 },
    { 0, 0 },
    { 0x251b1a08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ab1a0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860b1a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b1a17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db1a1a, 90 },
    { 0xe0eb1a1a, 76 },
    { 0xa11b1a1c, 90 },
    { 0, 0 },
    { 0x32bb1a1e, 90 },
    { 0, 0 },
    { 0x62fb1a20, 91 },
    { 0xc52b1a21, 33 },
    { 0x151b1a22, 90 },
    { 0x48eb1a21, 66 },
    { 0, 0 },
    { 0xe3eb1a25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b1a28, 91 },
    { 0xdbdb1a29, 33 },
    { 0x9c1b1a2a, 98 },
    { 0, 0 },
    { 0x2fdb1a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b1a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b1a36, 98 },
    { 0, 0 },
    { 0x1f5b1a38, 95 },
    { 0, 0 },
    { 0xa63b1a3a, 34 },
    { 0xbcab1a3a, 91 },
    { 0xaf5b1a3b, 84 },
    { 0x71fb1a3d, 33 },
    { 0, 0 },
    { 0x7eb1a3f, 99 },
    { 0xf89b1a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x511b1a43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bb1a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b1a4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb1a52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191b1a5a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b1a5f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bb1a65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb1a68, 61 },
    { 0xff7b1a68, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b1a6c, 90 },
    { 0xd8b1a6d, 94 },
    { 0, 0 },
    { 0x62eb1a6f, 98 },
    { 0x6a5b1a6f, 66 },
    { 0xc71b1a71, 66 },
    { 0, 0 },
    { 0x428b1a73, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b1a78, 61 },
    { 0x658b1a78, 61 },
    { 0x43fb1a7a, 43 },
    { 0x8b2b1a7a, 33 },
    { 0x85cb1a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab1a80, 98 },
    { 0, 0 },
    { 0x46b1a82, 61 },
    { 0, 0 },
    { 0x830b1a84, 98 },
    { 0xbe6b1a84, 33 },
    { 0xc1bb1a84, 61 },
    { 0, 0 },
    { 0xa1cb1a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4b1a8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb1a90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acb1a95, 76 },
    { 0xf67b1a95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b1a99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb1a9d, 61 },
    { 0, 0 },
    { 0x69fb1a9f, 90 },
    { 0, 0 },
    { 0x36eb1aa1, 65 },
    { 0x38fb1aa2, 33 },
    { 0, 0 },
    { 0xe5fb1aa4, 23 },
    { 0, 0 },
    { 0xb9b1aa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb1aa9, 93 },
    { 0xbbcb1aaa, 100 },
    { 0x13bb1aab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b1aaf, 61 },
    { 0xbedb1aaf, 33 },
    { 0, 0 },
    { 0xe58b1ab2, 33 },
    { 0, 0 },
    { 0xc5eb1ab4, 55 },
    { 0x2c8b1ab4, 85 },
    { 0xbf9b1ab6, 46 },
    { 0xcc3b1ab7, 68 },
    { 0, 0 },
    { 0x6d1b1ab9, 84 },
    { 0x6f8b1ab9, 83 },
    { 0x30ab1abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7b1abe, 91 },
    { 0xb66b1abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b1ac3, 95 },
    { 0, 0 },
    { 0x751b1ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82b1ac8, 76 },
    { 0xa48b1ac8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b1acd, 91 },
    { 0, 0 },
    { 0xf3cb1acf, 93 },
    { 0, 0 },
    { 0x426b1ad1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb1ad5, 61 },
    { 0xacdb1ad6, 91 },
    { 0, 0 },
    { 0x4c7b1ad8, 93 },
    { 0xaa9b1ad9, 61 },
    { 0, 0 },
    { 0x710b1adb, 78 },
    { 0x351b1adc, 33 },
    { 0x134b1add, 76 },
    { 0x5bbb1add, 98 },
    { 0x4e7b1adf, 95 },
    { 0xf63b1ae0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab1ae5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cb1ae8, 87 },
    { 0, 0 },
    { 0xef6b1aea, 23 },
    { 0x2e0b1aea, 98 },
    { 0x4a1b1aec, 90 },
    { 0x577b1aec, 55 },
    { 0xf34b1aee, 17 },
    { 0x2fb1aea, 98 },
    { 0xe89b1af0, 46 },
    { 0xde9b1af1, 68 },
    { 0, 0 },
    { 0x7f8b1af3, 55 },
    { 0x2a7b1af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab1af7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b1afa, 23 },
    { 0x2c8b1afb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5b1b02, 66 },
    { 0x8d7b1b03, 23 },
    { 0, 0 },
    { 0x892b1b05, 61 },
    { 0, 0 },
    { 0xe94b1b07, 23 },
    { 0x4db1b08, 90 },
    { 0, 0 },
    { 0xd00b1b0a, 23 },
    { 0xf5db1b0b, 77 },
    { 0x11bb1b0c, 33 },
    { 0x29eb1b0d, 90 },
    { 0x7a6b1b0a, 75 },
    { 0xbbb1b0a, 40 },
    { 0x9c8b1b0f, 75 },
    { 0xa51b1b11, 43 },
    { 0x3bab1b12, 61 },
    { 0xa9b1b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb1b1a, 61 },
    { 0xa99b1b1b, 66 },
    { 0xd4ab1b1a, 77 },
    { 0xcd4b1b1a, 40 },
    { 0xe98b1b1e, 98 },
    { 0x928b1b1e, 90 },
    { 0, 0 },
    { 0x78db1b21, 88 },
    { 0xfc8b1b21, 95 },
    { 0, 0 },
    { 0x8e5b1b24, 66 },
    { 0x927b1b24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db1b28, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb1b2e, 46 },
    { 0, 0 },
    { 0x917b1b30, 99 },
    { 0, 0 },
    { 0x65b1b32, 91 },
    { 0, 0 },
    { 0x7b1b34, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb1b3a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb1b3e, 76 },
    { 0, 0 },
    { 0x5cdb1b40, 43 },
    { 0, 0 },
    { 0x9f0b1b42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0b1b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb1b4e, 78 },
    { 0x4ab1b4f, 85 },
    { 0x920b1b50, 87 },
    { 0x2b0b1b51, 61 },
    { 0x349b1b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb1b55, 87 },
    { 0x193b1b56, 52 },
    { 0, 0 },
    { 0x9fb1b58, 61 },
    { 0x949b1b59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b1b5e, 78 },
    { 0, 0 },
    { 0x581b1b60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x581b1b63, 90 },
    { 0xe9fb1b63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b1b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5b1b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b1b71, 66 },
    { 0, 0 },
    { 0x446b1b73, 23 },
    { 0, 0 },
    { 0x45fb1b75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadb1b79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5b1b7e, 52 },
    { 0x8acb1b7f, 33 },
    { 0xe7ab1b80, 23 },
    { 0, 0 },
    { 0xfd6b1b82, 93 },
    { 0x3d3b1b83, 77 },
    { 0xfd0b1b84, 61 },
    { 0x4e8b1b84, 76 },
    { 0, 0 },
    { 0x791b1b87, 88 },
    { 0, 0 },
    { 0x8b5b1b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52db1b8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901b1b97, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b1b9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b1ba0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615b1ba4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb1ba8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb1bae, 90 },
    { 0xdb0b1baf, 43 },
    { 0xc54b1bb0, 46 },
    { 0x4fcb1baf, 46 },
    { 0xaa5b1bb2, 76 },
    { 0x59db1bb3, 77 },
    { 0x710b1bb4, 55 },
    { 0x187b1bb3, 90 },
    { 0xcb5b1bb5, 17 },
    { 0xaefb1bb7, 33 },
    { 0xbe4b1bb8, 67 },
    { 0xd5ab1bb9, 76 },
    { 0x809b1bb1, 23 },
    { 0x140b1bae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb1bbe, 87 },
    { 0, 0 },
    { 0x38fb1bc0, 90 },
    { 0, 0 },
    { 0x59ab1bc2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fb1bc5, 33 },
    { 0xa9db1bc6, 33 },
    { 0, 0 },
    { 0x6b1b1bc8, 23 },
    { 0x8db1bc9, 61 },
    { 0xe8fb1bc9, 43 },
    { 0, 0 },
    { 0x177b1bcc, 84 },
    { 0, 0 },
    { 0xe11b1bce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x655b1bd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcb1bdd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x86db1be0, 95 },
    { 0x61fb1be1, 61 },
    { 0, 0 },
    { 0x24eb1be3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2b1beb, 55 },
    { 0, 0 },
    { 0x9d8b1bed, 98 },
    { 0xec1b1bee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200b1bf6, 99 },
    { 0xe7fb1bf7, 61 },
    { 0xe59b1bf6, 95 },
    { 0xdeab1bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1bfc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1c03, 95 },
    { 0, 0 },
    { 0x957b1c05, 83 },
    { 0x257b1c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22b1c09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b1c0f, 95 },
    { 0x401b1c10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609b1c15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b1c18, 90 },
    { 0x4a2b1c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb1c1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781b1c21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b1c2d, 90 },
    { 0xb2db1c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7b1c31, 37 },
    { 0, 0 },
    { 0x71b1c33, 98 },
    { 0x9e5b1c34, 69 },
    { 0, 0 },
    { 0x75bb1c36, 33 },
    { 0x450b1c36, 23 },
    { 0, 0 },
    { 0x47ab1c39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b1c3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x622b1c40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feb1c45, 43 },
    { 0xbe0b1c46, 93 },
    { 0x17ab1c47, 33 },
    { 0x3dbb1c48, 90 },
    { 0x868b1c48, 46 },
    { 0, 0 },
    { 0x9f9b1c4b, 93 },
    { 0x3c5b1c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bb1c4f, 46 },
    { 0x37fb1c50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b1c53, 91 },
    { 0x12eb1c54, 95 },
    { 0xdddb1c54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb1c5c, 61 },
    { 0xdfbb1c5d, 97 },
    { 0x7e8b1c5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b1c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb1c65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543b1c69, 81 },
    { 0, 0 },
    { 0xbfb1c6b, 66 },
    { 0x378b1c6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52b1c70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b1c75, 90 },
    { 0x6a3b1c76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b1c7b, 33 },
    { 0xf07b1c7c, 61 },
    { 0, 0 },
    { 0x463b1c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296b1c83, 90 },
    { 0x89ab1c84, 93 },
    { 0, 0 },
    { 0x115b1c86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb1c8b, 68 },
    { 0xb1c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b1c92, 33 },
    { 0, 0 },
    { 0x576b1c94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07b1c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7b1ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79b1ca7, 93 },
    { 0xfa0b1ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1db1cb0, 90 },
    { 0x1d5b1cb0, 90 },
    { 0, 0 },
    { 0x5c6b1cb3, 67 },
    { 0xdd3b1cb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb1cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b1cbc, 35 },
    { 0x970b1cbc, 81 },
    { 0xf46b1cbc, 98 },
    { 0xa33b1cbf, 98 },
    { 0x6b9b1cc0, 98 },
    { 0xf3bb1cc1, 33 },
    { 0x67ab1cc0, 78 },
    { 0xe32b1cc3, 95 },
    { 0xf5cb1cc3, 90 },
    { 0x193b1cbe, 76 },
    { 0, 0 },
    { 0x59fb1cc7, 80 },
    { 0, 0 },
    { 0x7f3b1cc9, 61 },
    { 0, 0 },
    { 0x35fb1ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x305b1cce, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fb1cd1, 76 },
    { 0, 0 },
    { 0x2a4b1cd3, 90 },
    { 0x1bbb1cd4, 98 },
    { 0x7fab1cd3, 52 },
    { 0, 0 },
    { 0xc39b1cd7, 87 },
    { 0, 0 },
    { 0x594b1cd9, 93 },
    { 0, 0 },
    { 0xd6b1cdb, 95 },
    { 0xc34b1cdc, 43 },
    { 0, 0 },
    { 0x529b1cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db1ce6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62b1ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b1ced, 76 },
    { 0x728b1ced, 93 },
    { 0, 0 },
    { 0xe35b1cf0, 66 },
    { 0, 0 },
    { 0xa8b1cf2, 68 },
    { 0x477b1cf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b1cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162b1cfd, 87 },
    { 0x4d8b1cfe, 90 },
    { 0x51eb1cff, 90 },
    { 0x626b1cff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b1d04, 90 },
    { 0x9c6b1d04, 95 },
    { 0xcf4b1d06, 90 },
    { 0x5d7b1d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669b1d0f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b1d15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b1d18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1b1d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b1d25, 23 },
    { 0xb27b1d26, 90 },
    { 0, 0 },
    { 0xd23b1d28, 98 },
    { 0, 0 },
    { 0xa9eb1d2a, 67 },
    { 0xe07b1d2a, 90 },
    { 0x84bb1d2c, 81 },
    { 0x577b1d2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b1d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5b1d40, 23 },
    { 0, 0 },
    { 0x3e9b1d42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1b1d46, 23 },
    { 0x998b1d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77b1d4a, 95 },
    { 0, 0 },
    { 0x506b1d4c, 96 },
    { 0x6aab1d4d, 87 },
    { 0xce7b1d4e, 88 },
    { 0x78ab1d4e, 90 },
    { 0x79ab1d4e, 98 },
    { 0x904b1d4c, 33 },
    { 0x7f5b1d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb1d56, 33 },
    { 0x97b1d57, 33 },
    { 0xe59b1d56, 84 },
    { 0x16eb1d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db1d5d, 61 },
    { 0x10cb1d5d, 67 },
    { 0xf71b1d5f, 66 },
    { 0x8afb1d5f, 90 },
    { 0, 0 },
    { 0x326b1d62, 98 },
    { 0xd96b1d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b1d68, 33 },
    { 0x8c4b1d69, 93 },
    { 0, 0 },
    { 0x9f2b1d6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644b1d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86b1d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b1d7b, 81 },
    { 0x6b1b1d7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b1d80, 76 },
    { 0, 0 },
    { 0xbebb1d82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2b1d89, 49 },
    { 0x733b1d89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948b1d90, 61 },
    { 0x9ecb1d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db1d98, 99 },
    { 0, 0 },
    { 0x173b1d9a, 90 },
    { 0x813b1d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b1da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6b1da8, 34 },
    { 0, 0 },
    { 0x8cdb1daa, 90 },
    { 0xf3bb1daa, 78 },
    { 0, 0 },
    { 0xd35b1dad, 99 },
    { 0x41db1dae, 98 },
    { 0x7d0b1daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb1db4, 77 },
    { 0x17cb1db5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cb1dba, 61 },
    { 0xf6cb1dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x188b1dbe, 67 },
    { 0x629b1dbe, 34 },
    { 0x355b1dc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359b1dc9, 33 },
    { 0x62db1dca, 61 },
    { 0, 0 },
    { 0x47b1dcc, 46 },
    { 0xd08b1dcd, 91 },
    { 0x446b1dcd, 76 },
    { 0xcedb1dcf, 33 },
    { 0xbd9b1dcc, 90 },
    { 0x3abb1dd1, 47 },
    { 0x439b1dd2, 34 },
    { 0x355b1dcd, 23 },
    { 0x513b1dd4, 66 },
    { 0x98bb1dcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b1dd9, 90 },
    { 0x802b1dd9, 95 },
    { 0x81b1ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406b1dde, 61 },
    { 0x2fb1dde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb1de4, 95 },
    { 0x7e1b1de5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838b1dee, 75 },
    { 0, 0 },
    { 0xa60b1df0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bb1df6, 33 },
    { 0, 0 },
    { 0x242b1df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7b1dfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b1e00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cb1e04, 90 },
    { 0xe01b1e04, 66 },
    { 0x6f1b1e05, 76 },
    { 0xe50b1e07, 33 },
    { 0x496b1e08, 87 },
    { 0x1f3b1e08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b1e0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfb1e10, 23 },
    { 0xd49b1e11, 33 },
    { 0x59eb1e11, 90 },
    { 0x371b1e11, 84 },
    { 0x452b1e14, 61 },
    { 0x8b3b1e15, 98 },
    { 0xdcdb1e13, 33 },
    { 0xd86b1e17, 33 },
    { 0x68b1e14, 91 },
    { 0xcc4b1e19, 66 },
    { 0xa0cb1e15, 52 },
    { 0xaf5b1e1b, 95 },
    { 0x195b1e15, 76 },
    { 0x6bdb1e1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb1e26, 33 },
    { 0x1a0b1e27, 33 },
    { 0x8f9b1e27, 61 },
    { 0xabfb1e27, 98 },
    { 0xda1b1e2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33b1e38, 75 },
    { 0x356b1e38, 33 },
    { 0x308b1e38, 17 },
    { 0x7bdb1e3b, 95 },
    { 0xdceb1e3b, 56 },
    { 0xcc2b1e39, 90 },
    { 0, 0 },
    { 0x71fb1e3f, 99 },
    { 0x104b1e40, 61 },
    { 0x8f3b1e40, 76 },
    { 0x9ffb1e3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb1e4a, 61 },
    { 0x8b4b1e4b, 61 },
    { 0x68ab1e4c, 93 },
    { 0x19ab1e4c, 46 },
    { 0, 0 },
    { 0xbe3b1e4f, 33 },
    { 0x7e8b1e4f, 95 },
    { 0x99b1e51, 100 },
    { 0x21cb1e51, 90 },
    { 0xfdab1e51, 66 },
    { 0x1b3b1e54, 33 },
    { 0x33b1e54, 76 },
    { 0x4fab1e56, 91 },
    { 0xabfb1e55, 23 },
    { 0, 0 },
    { 0x5d1b1e59, 33 },
    { 0x5aeb1e5a, 90 },
    { 0, 0 },
    { 0x315b1e5c, 98 },
    { 0x353b1e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243b1e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddb1e68, 34 },
    { 0, 0 },
    { 0x9fb1e6a, 76 },
    { 0xb4fb1e6b, 52 },
    { 0, 0 },
    { 0x59bb1e6d, 23 },
    { 0xd20b1e6e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570b1e74, 90 },
    { 0, 0 },
    { 0xeb1b1e76, 76 },
    { 0, 0 },
    { 0x68fb1e78, 33 },
    { 0, 0 },
    { 0x55bb1e7a, 76 },
    { 0x23ab1e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x806b1e7e, 90 },
    { 0, 0 },
    { 0xdcab1e80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b1e84, 78 },
    { 0x73fb1e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b1e8e, 90 },
    { 0, 0 },
    { 0x6aab1e90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ab1e93, 87 },
    { 0xcd7b1e94, 46 },
    { 0, 0 },
    { 0xab5b1e96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb1e9c, 33 },
    { 0x948b1e9c, 90 },
    { 0xccdb1e9c, 90 },
    { 0x4b0b1e9f, 90 },
    { 0x6d9b1e9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7b1eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bb1eb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb1eb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93b1ebf, 52 },
    { 0xaab1ec0, 33 },
    { 0x5d9b1ebf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb1ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ab1ec9, 61 },
    { 0xe20b1eca, 91 },
    { 0x9fb1ec9, 52 },
    { 0xc82b1ecc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb1ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42b1ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571b1ed9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb1edc, 61 },
    { 0x5b5b1edc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bb1ee0, 55 },
    { 0x870b1ee0, 98 },
    { 0x47ab1ee2, 84 },
    { 0x705b1ee3, 90 },
    { 0xa7eb1ee0, 55 },
    { 0x1ab1ee5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ab1ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3b1eee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcb1ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb1ef7, 90 },
    { 0xaceb1ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b1efc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b1f00, 78 },
    { 0x378b1f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cb1f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b1f0b, 95 },
    { 0x1c6b1f0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2b1f14, 90 },
    { 0, 0 },
    { 0x6c5b1f16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab1f1a, 97 },
    { 0x42bb1f1a, 33 },
    { 0x9c9b1f1c, 95 },
    { 0xaab1f1b, 43 },
    { 0, 0 },
    { 0x2a5b1f1f, 87 },
    { 0, 0 },
    { 0x20bb1f21, 95 },
    { 0x8c3b1f22, 98 },
    { 0xdfab1f23, 98 },
    { 0xa17b1f22, 70 },
    { 0x37cb1f25, 17 },
    { 0x233b1f26, 55 },
    { 0x10eb1f22, 93 },
    { 0x686b1f28, 23 },
    { 0xdd3b1f29, 90 },
    { 0x70fb1f2a, 33 },
    { 0xcdb1f2b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b1f2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbb1f35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76b1f38, 95 },
    { 0, 0 },
    { 0xc93b1f3a, 23 },
    { 0xfe8b1f3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x173b1f3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b1f41, 99 },
    { 0x780b1f42, 96 },
    { 0xc38b1f41, 93 },
    { 0xe6cb1f44, 33 },
    { 0xdbeb1f45, 20 },
    { 0x91b1f46, 61 },
    { 0x2d0b1f44, 76 },
    { 0x62b1f48, 98 },
    { 0xba7b1f47, 23 },
    { 0x38fb1f42, 69 },
    { 0x615b1f47, 98 },
    { 0x5f6b1f45, 23 },
    { 0x19eb1f4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131b1f52, 98 },
    { 0x683b1f52, 98 },
    { 0xd98b1f54, 33 },
    { 0, 0 },
    { 0xb81b1f56, 52 },
    { 0x430b1f57, 33 },
    { 0x74cb1f58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892b1f5c, 90 },
    { 0x778b1f5c, 33 },
    { 0, 0 },
    { 0x258b1f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2db1f62, 90 },
    { 0x4ab1f63, 95 },
    { 0x9aeb1f64, 90 },
    { 0x83cb1f65, 90 },
    { 0x9d4b1f65, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b1f6a, 52 },
    { 0x3a7b1f6b, 98 },
    { 0, 0 },
    { 0x9afb1f6d, 95 },
    { 0xab6b1f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78b1f72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bb1f76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b1f7a, 76 },
    { 0x93b1f7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b1f80, 90 },
    { 0x8ebb1f80, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x213b1f84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b1f8a, 67 },
    { 0xfc3b1f8a, 96 },
    { 0xab0b1f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4b1f90, 90 },
    { 0, 0 },
    { 0x4c4b1f92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1f95, 95 },
    { 0xaa4b1f96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x137b1f99, 33 },
    { 0xe08b1f99, 75 },
    { 0xef7b1f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab1fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1faa, 61 },
    { 0, 0 },
    { 0xc20b1fac, 90 },
    { 0x4beb1fac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4b1fb1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2b1fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b1fbe, 90 },
    { 0x5d7b1fbe, 17 },
    { 0x297b1fc0, 95 },
    { 0x2d0b1fc0, 76 },
    { 0, 0 },
    { 0xf07b1fc3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5b1fc6, 90 },
    { 0xa7fb1fc7, 76 },
    { 0xb30b1fc8, 33 },
    { 0, 0 },
    { 0x97b1fca, 34 },
    { 0x34fb1fcb, 33 },
    { 0x644b1fcb, 61 },
    { 0, 0 },
    { 0xe9eb1fce, 23 },
    { 0, 0 },
    { 0xbf4b1fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db1fd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b1fd9, 95 },
    { 0, 0 },
    { 0xb1eb1fdb, 95 },
    { 0x4d8b1fdc, 61 },
    { 0, 0 },
    { 0xd87b1fde, 77 },
    { 0, 0 },
    { 0x478b1fe0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb1fe3, 100 },
    { 0x826b1fe3, 75 },
    { 0xb4eb1fe5, 90 },
    { 0x2eab1fe6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eab1fe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b1fee, 90 },
    { 0xeb5b1fee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b1ff3, 34 },
    { 0, 0 },
    { 0xc8fb1ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1ff8, 90 },
    { 0x68eb1ff9, 78 },
    { 0xaf6b1ff9, 85 },
    { 0xc7b1ffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfb1fff, 93 },
    { 0x294b2000, 55 },
    { 0xfbab2001, 100 },
    { 0xebbb2002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eb2008, 93 },
    { 0xdedb2009, 61 },
    { 0, 0 },
    { 0x970b200b, 80 },
    { 0x74b200b, 98 },
    { 0, 0 },
    { 0x621b200e, 55 },
    { 0, 0 },
    { 0xf48b2010, 90 },
    { 0xa97b2011, 35 },
    { 0, 0 },
    { 0x9b2013, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b2017, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcab201a, 55 },
    { 0xb1eb201b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744b2027, 90 },
    { 0, 0 },
    { 0x5e9b2029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefb202d, 90 },
    { 0x547b202e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b2032, 95 },
    { 0x12b2032, 95 },
    { 0x4ecb2033, 95 },
    { 0, 0 },
    { 0xa14b2036, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb203b, 17 },
    { 0xa4cb203c, 95 },
    { 0xf28b203d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0b2040, 99 },
    { 0xc44b2041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab204a, 98 },
    { 0, 0 },
    { 0x46db204c, 61 },
    { 0x3bbb204d, 90 },
    { 0, 0 },
    { 0x59eb204f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b2054, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb205a, 46 },
    { 0xcc2b205b, 76 },
    { 0x8c9b205a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b205f, 55 },
    { 0, 0 },
    { 0x40cb2061, 53 },
    { 0, 0 },
    { 0x40eb2063, 23 },
    { 0x2ddb2064, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b206b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3b2079, 33 },
    { 0xcbcb207a, 43 },
    { 0x766b207b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ab207f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b2087, 61 },
    { 0, 0 },
    { 0xca7b2089, 90 },
    { 0x656b2089, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b208d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b2093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b2098, 61 },
    { 0, 0 },
    { 0x4ccb209a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb209d, 23 },
    { 0, 0 },
    { 0xdf9b209f, 90 },
    { 0xa3eb20a0, 78 },
    { 0x39bb209f, 33 },
    { 0xa91b20a2, 91 },
    { 0x52fb209f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eb20ad, 91 },
    { 0xd99b20ae, 90 },
    { 0x23bb20ae, 55 },
    { 0x2b7b20ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceb20b7, 98 },
    { 0, 0 },
    { 0x569b20b9, 90 },
    { 0, 0 },
    { 0x735b20bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b20c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb20c5, 90 },
    { 0xe9ab20c6, 68 },
    { 0, 0 },
    { 0x669b20c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcb20ce, 91 },
    { 0xf7db20cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8b20d4, 40 },
    { 0x1d8b20d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fb20d9, 61 },
    { 0, 0 },
    { 0x61fb20db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb20de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32b20e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb20e9, 17 },
    { 0xb92b20ea, 33 },
    { 0xff3b20eb, 61 },
    { 0x172b20ec, 43 },
    { 0x124b20ed, 98 },
    { 0, 0 },
    { 0xd4fb20ef, 99 },
    { 0, 0 },
    { 0x1a5b20f1, 98 },
    { 0, 0 },
    { 0xed9b20f3, 90 },
    { 0x3afb20f3, 98 },
    { 0x8e5b20f3, 91 },
    { 0x6c4b20f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b2101, 87 },
    { 0x8dcb2101, 90 },
    { 0xaafb2103, 17 },
    { 0, 0 },
    { 0x973b2105, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b2108, 33 },
    { 0, 0 },
    { 0x3c3b210a, 67 },
    { 0x41cb210b, 61 },
    { 0x875b210c, 33 },
    { 0, 0 },
    { 0x223b210e, 93 },
    { 0xeecb210e, 90 },
    { 0x433b2110, 78 },
    { 0x6f1b2111, 68 },
    { 0xd0b2112, 46 },
    { 0, 0 },
    { 0xcc9b2114, 81 },
    { 0xd09b2115, 90 },
    { 0x2e4b2116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b211c, 33 },
    { 0x565b211d, 98 },
    { 0xa04b211e, 23 },
    { 0, 0 },
    { 0x39fb2120, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b2127, 23 },
    { 0, 0 },
    { 0xdc4b2129, 66 },
    { 0x760b212a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88b2132, 98 },
    { 0x2e1b2133, 90 },
    { 0x9dcb2134, 33 },
    { 0xd1fb2135, 95 },
    { 0x957b2132, 98 },
    { 0x857b2137, 55 },
    { 0x304b2138, 95 },
    { 0x2c7b2134, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bb2146, 98 },
    { 0, 0 },
    { 0x7b8b2148, 61 },
    { 0, 0 },
    { 0x260b214a, 90 },
    { 0x70ab214b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8b2152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcb2157, 85 },
    { 0xe02b2158, 43 },
    { 0, 0 },
    { 0x423b215a, 34 },
    { 0xc67b215a, 85 },
    { 0, 0 },
    { 0x7e3b215d, 66 },
    { 0, 0 },
    { 0xaf2b215f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b2164, 33 },
    { 0x81db2164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b2169, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fb216c, 77 },
    { 0x2ecb216d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734b2172, 61 },
    { 0, 0 },
    { 0xaeab2174, 90 },
    { 0x967b2175, 43 },
    { 0x52eb2174, 100 },
    { 0x19bb2174, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebb217b, 90 },
    { 0, 0 },
    { 0xc6b217d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34b2181, 91 },
    { 0xb74b2181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db218b, 81 },
    { 0x832b218b, 33 },
    { 0x2bab218b, 77 },
    { 0, 0 },
    { 0x5f4b218f, 55 },
    { 0x53bb2190, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b2196, 76 },
    { 0xeeab2197, 33 },
    { 0xc9fb2196, 95 },
    { 0xf96b2197, 23 },
    { 0xc08b219a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b219e, 23 },
    { 0x979b219f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1b21a3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b21a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b21ac, 99 },
    { 0xd9bb21ad, 85 },
    { 0xf8ab21ae, 90 },
    { 0x89ab21ad, 61 },
    { 0x193b21af, 61 },
    { 0x9feb21b1, 61 },
    { 0xb2bb21ac, 61 },
    { 0xc03b21af, 33 },
    { 0x183b21b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b21b8, 90 },
    { 0xf10b21b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb21bc, 95 },
    { 0, 0 },
    { 0xaf2b21be, 52 },
    { 0, 0 },
    { 0x743b21c0, 98 },
    { 0, 0 },
    { 0xb28b21c2, 88 },
    { 0xc29b21c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b21ce, 33 },
    { 0, 0 },
    { 0xc1db21d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843b21d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7b21d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeab21dc, 23 },
    { 0x53b21dd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db21e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab21e4, 55 },
    { 0x4a9b21e5, 17 },
    { 0x859b21e6, 68 },
    { 0xdedb21e7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69b21eb, 23 },
    { 0x6f5b21ec, 66 },
    { 0, 0 },
    { 0xfe6b21ee, 78 },
    { 0x1eeb21ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffb21f2, 90 },
    { 0xe61b21f3, 55 },
    { 0x229b21f4, 98 },
    { 0xf0cb21f5, 46 },
    { 0, 0 },
    { 0x16db21f7, 34 },
    { 0, 0 },
    { 0x680b21f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b21fe, 95 },
    { 0, 0 },
    { 0x86fb2200, 90 },
    { 0xdadb2200, 56 },
    { 0xf39b2200, 95 },
    { 0xd42b2202, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b2206, 55 },
    { 0x747b2207, 33 },
    { 0, 0 },
    { 0xa9cb2209, 61 },
    { 0xa43b220a, 66 },
    { 0x8edb220a, 95 },
    { 0x40eb220c, 55 },
    { 0xbdbb220c, 61 },
    { 0x7a4b220c, 81 },
    { 0x889b220a, 76 },
    { 0x546b2210, 34 },
    { 0xd96b2211, 61 },
    { 0x8f2b2212, 33 },
    { 0xaf3b2210, 43 },
    { 0x56ab2212, 52 },
    { 0x54fb2210, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbb221a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b221f, 90 },
    { 0x53bb2220, 81 },
    { 0x614b2221, 90 },
    { 0, 0 },
    { 0xf0bb2223, 33 },
    { 0x7a5b2224, 90 },
    { 0xf38b2225, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb2229, 23 },
    { 0xd93b222a, 33 },
    { 0xc81b222b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37b2231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433b2235, 91 },
    { 0, 0 },
    { 0x968b2237, 34 },
    { 0x791b2237, 61 },
    { 0xaf8b2239, 77 },
    { 0x29fb2237, 33 },
    { 0xb22b2238, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94b223e, 87 },
    { 0, 0 },
    { 0x54fb2240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb2244, 91 },
    { 0, 0 },
    { 0x86db2246, 90 },
    { 0x3c4b2247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b224a, 98 },
    { 0xd00b224a, 23 },
    { 0x27b224c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eb2255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bb2260, 99 },
    { 0, 0 },
    { 0x86bb2262, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b2265, 95 },
    { 0x24fb2266, 33 },
    { 0x5fb2265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb226a, 33 },
    { 0, 0 },
    { 0xd30b226c, 61 },
    { 0xbcab226c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b2271, 100 },
    { 0xf6db2272, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb2276, 98 },
    { 0xf4b2277, 52 },
    { 0x869b2277, 46 },
    { 0, 0 },
    { 0xc87b227a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89db227d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb2282, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91b2288, 95 },
    { 0xccab2289, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b2292, 55 },
    { 0x63bb2293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb2299, 55 },
    { 0xf76b2299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eb22a2, 23 },
    { 0x2db22a3, 23 },
    { 0, 0 },
    { 0xbf2b22a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb22a8, 88 },
    { 0xb51b22a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b22ac, 90 },
    { 0x801b22ac, 23 },
    { 0x7b6b22ae, 61 },
    { 0x1c4b22af, 55 },
    { 0, 0 },
    { 0x14db22b1, 61 },
    { 0xc46b22b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171b22b7, 93 },
    { 0, 0 },
    { 0xbcdb22b9, 90 },
    { 0, 0 },
    { 0x421b22bb, 78 },
    { 0xe26b22bc, 98 },
    { 0x728b22bd, 85 },
    { 0xbb7b22bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab22c3, 99 },
    { 0, 0 },
    { 0xfb7b22c5, 33 },
    { 0x2abb22c6, 76 },
    { 0xcf2b22c6, 61 },
    { 0, 0 },
    { 0xe70b22c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176b22cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770b22d1, 99 },
    { 0x7cbb22d2, 33 },
    { 0xbe6b22d3, 93 },
    { 0, 0 },
    { 0x7c3b22d5, 100 },
    { 0x4c9b22d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b22db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b22e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b22e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b22eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b22ee, 66 },
    { 0x321b22ef, 99 },
    { 0, 0 },
    { 0x223b22f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab22f9, 66 },
    { 0x751b22fa, 95 },
    { 0x411b22fa, 43 },
    { 0xfe1b22fc, 23 },
    { 0x2eeb22fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b2304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb2309, 33 },
    { 0x82b2309, 33 },
    { 0x6bfb230b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8db230f, 87 },
    { 0, 0 },
    { 0x5dcb2311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56b2314, 95 },
    { 0xe31b2315, 90 },
    { 0xc76b2315, 33 },
    { 0xac4b2317, 47 },
    { 0xfc3b2318, 33 },
    { 0x825b2316, 76 },
    { 0x2f8b231a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b231f, 93 },
    { 0, 0 },
    { 0xbdeb2321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb2325, 33 },
    { 0xa1bb2326, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ab232d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b2333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb2336, 33 },
    { 0, 0 },
    { 0x631b2338, 88 },
    { 0, 0 },
    { 0x2aeb233a, 90 },
    { 0x174b233b, 23 },
    { 0x65fb233b, 33 },
    { 0, 0 },
    { 0x4a6b233e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bb2342, 34 },
    { 0x468b2343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b2346, 67 },
    { 0x4cb2347, 23 },
    { 0x91db2348, 98 },
    { 0, 0 },
    { 0xea9b234a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b234e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b2351, 85 },
    { 0, 0 },
    { 0x7f6b2353, 43 },
    { 0xe35b2353, 23 },
    { 0x3c9b2355, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58b2359, 33 },
    { 0, 0 },
    { 0xd42b235b, 95 },
    { 0, 0 },
    { 0xb19b235d, 95 },
    { 0x5eab235e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5eb2363, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cb2366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb2369, 43 },
    { 0, 0 },
    { 0x72ab236b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283b2372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbb2375, 98 },
    { 0, 0 },
    { 0xec2b2377, 95 },
    { 0x2c0b2378, 95 },
    { 0x1cb2378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcb237d, 78 },
    { 0x302b237d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3b2381, 84 },
    { 0, 0 },
    { 0xbc6b2383, 61 },
    { 0xd0eb2384, 46 },
    { 0x240b2385, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2b2388, 23 },
    { 0x45eb2389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb238d, 88 },
    { 0xbfeb238d, 90 },
    { 0xb5bb238f, 51 },
    { 0x224b2390, 23 },
    { 0, 0 },
    { 0xe13b2392, 95 },
    { 0xa55b2393, 46 },
    { 0xe8bb2393, 76 },
    { 0x356b2395, 43 },
    { 0x92bb2396, 90 },
    { 0x2ddb2397, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddb239f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfb23a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b23a7, 55 },
    { 0x768b23a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db23af, 55 },
    { 0x5e9b23b0, 90 },
    { 0x737b23b1, 98 },
    { 0x7f0b23b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34b23b9, 98 },
    { 0, 0 },
    { 0xdfb23bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342b23bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39b23c2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5b23c5, 34 },
    { 0x8d8b23c5, 55 },
    { 0x413b23c5, 43 },
    { 0x72eb23c7, 33 },
    { 0x79bb23c9, 85 },
    { 0xa96b23ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9b23d2, 95 },
    { 0, 0 },
    { 0xd12b23d4, 98 },
    { 0xab6b23d5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196b23d9, 33 },
    { 0xaafb23da, 43 },
    { 0x4d8b23db, 61 },
    { 0, 0 },
    { 0x693b23dd, 33 },
    { 0x1ecb23de, 39 },
    { 0, 0 },
    { 0xbeab23e0, 4 },
    { 0, 0 },
    { 0x392b23e2, 78 },
    { 0xb0b23e3, 93 },
    { 0x499b23e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b23e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476b23ed, 85 },
    { 0xa7eb23ee, 52 },
    { 0, 0 },
    { 0xd3ab23f0, 33 },
    { 0x72bb23f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9b23fb, 64 },
    { 0xfffb23fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb23ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b2402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b2409, 98 },
    { 0x67ab240a, 33 },
    { 0x420b240b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46b240f, 98 },
    { 0, 0 },
    { 0xcd3b2411, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390b2416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb241a, 77 },
    { 0xe2b241b, 61 },
    { 0xcf7b241a, 99 },
    { 0, 0 },
    { 0x517b241e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bb2422, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b2429, 90 },
    { 0x490b2429, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b242f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b2435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b243b, 55 },
    { 0xec1b243c, 88 },
    { 0x695b243d, 98 },
    { 0x3bfb243b, 61 },
    { 0xa4eb243e, 90 },
    { 0x59fb2440, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beb2443, 34 },
    { 0xd0b2444, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98db2449, 33 },
    { 0x836b244a, 61 },
    { 0, 0 },
    { 0x96fb244c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3b244f, 91 },
    { 0x17db2450, 76 },
    { 0xa9cb2451, 97 },
    { 0xb39b2452, 85 },
    { 0xe91b2453, 90 },
    { 0x70fb2454, 23 },
    { 0x767b2455, 95 },
    { 0xd8bb2453, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b245b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb2460, 93 },
    { 0, 0 },
    { 0x541b2462, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0b2466, 76 },
    { 0x617b2466, 17 },
    { 0xbafb2468, 93 },
    { 0x846b2469, 34 },
    { 0, 0 },
    { 0xdfeb246b, 2 },
    { 0x836b246c, 23 },
    { 0xcf4b246b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2471, 81 },
    { 0x907b2472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b2475, 95 },
    { 0x905b2476, 67 },
    { 0xbb9b2477, 80 },
    { 0x7eab2477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3b247c, 87 },
    { 0, 0 },
    { 0x3c0b247e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x509b2481, 76 },
    { 0x733b2481, 69 },
    { 0x641b2483, 34 },
    { 0x2f3b2484, 23 },
    { 0x82cb2481, 90 },
    { 0x3a3b2486, 99 },
    { 0xdb3b2486, 78 },
    { 0xe66b2487, 43 },
    { 0xb7ab2486, 95 },
    { 0xdc4b248a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b248d, 33 },
    { 0x8eeb248e, 23 },
    { 0xe0ab248e, 100 },
    { 0xdc4b248e, 80 },
    { 0, 0 },
    { 0x693b2492, 81 },
    { 0x4bbb2493, 90 },
    { 0x128b2494, 90 },
    { 0, 0 },
    { 0xc83b2496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb249c, 52 },
    { 0, 0 },
    { 0x9e1b249e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b24a2, 33 },
    { 0x759b24a3, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9b24a7, 34 },
    { 0xe2fb24a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60b24ac, 94 },
    { 0xa8fb24ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceb24b3, 90 },
    { 0, 0 },
    { 0xcc1b24b5, 98 },
    { 0x5e7b24b6, 71 },
    { 0x319b24b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x609b24ba, 95 },
    { 0xaebb24bb, 95 },
    { 0x37bb24bc, 76 },
    { 0xcbfb24bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b24c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6b24c5, 85 },
    { 0x9deb24c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8b24c9, 17 },
    { 0xa1db24ca, 78 },
    { 0xbc5b24cb, 65 },
    { 0xf7fb24cc, 43 },
    { 0x153b24ca, 33 },
    { 0xfeab24ce, 46 },
    { 0x54eb24cc, 90 },
    { 0x444b24ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eb24d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb24d7, 77 },
    { 0xd18b24d8, 95 },
    { 0xca9b24d7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b24dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab24df, 99 },
    { 0, 0 },
    { 0x9e5b24e1, 46 },
    { 0x8eb24e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b24e6, 23 },
    { 0, 0 },
    { 0x74b24e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x783b24eb, 23 },
    { 0xd27b24ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b24f1, 64 },
    { 0x50fb24f1, 34 },
    { 0xb25b24f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb24f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab24f9, 66 },
    { 0, 0 },
    { 0x7a2b24fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14db2502, 66 },
    { 0x365b2502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fb2506, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345b250a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472b250e, 78 },
    { 0x827b250e, 95 },
    { 0xd5fb250f, 78 },
    { 0, 0 },
    { 0x917b2512, 76 },
    { 0xa55b2513, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375b2518, 98 },
    { 0xca8b2519, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55b251c, 92 },
    { 0, 0 },
    { 0xa97b251e, 87 },
    { 0xea6b251f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7b2523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3b252a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06b252e, 61 },
    { 0x370b252f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb2532, 55 },
    { 0, 0 },
    { 0x5e1b2534, 84 },
    { 0x314b2534, 34 },
    { 0, 0 },
    { 0xb4cb2537, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00b2541, 61 },
    { 0xd99b2541, 33 },
    { 0x16cb2543, 98 },
    { 0xe94b2544, 78 },
    { 0xbaeb2543, 76 },
    { 0x680b2543, 33 },
    { 0xc1eb2547, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b254a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cb254d, 33 },
    { 0x5b2b254e, 33 },
    { 0x580b254e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cb2552, 52 },
    { 0xccbb2553, 90 },
    { 0x72cb2553, 93 },
    { 0xcacb2555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b2558, 68 },
    { 0x9b4b2559, 98 },
    { 0x59bb255a, 43 },
    { 0, 0 },
    { 0x129b255c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdb2561, 95 },
    { 0, 0 },
    { 0x773b2563, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ab2567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb256d, 33 },
    { 0xf69b256e, 91 },
    { 0x165b256f, 23 },
    { 0x2d6b2570, 87 },
    { 0x539b256e, 61 },
    { 0, 0 },
    { 0xb4db2573, 90 },
    { 0x4aeb2573, 52 },
    { 0x16db2574, 61 },
    { 0xc01b2575, 33 },
    { 0, 0 },
    { 0x839b2578, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b257b, 95 },
    { 0, 0 },
    { 0x917b257d, 77 },
    { 0x7b9b257e, 90 },
    { 0x98bb257d, 23 },
    { 0xcabb2580, 65 },
    { 0, 0 },
    { 0xa55b2582, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cb2585, 34 },
    { 0x141b2586, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b258a, 61 },
    { 0xe97b258a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adb258f, 33 },
    { 0xb7ab2590, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x599b2593, 61 },
    { 0x477b2594, 55 },
    { 0x432b2595, 90 },
    { 0, 0 },
    { 0x146b2597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb259b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102b25a1, 90 },
    { 0xf07b25a2, 90 },
    { 0xb01b25a3, 61 },
    { 0x495b25a4, 96 },
    { 0xa0ab25a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cb25a8, 43 },
    { 0x2c5b25a9, 61 },
    { 0x723b25aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x319b25ad, 34 },
    { 0xb9fb25ad, 43 },
    { 0, 0 },
    { 0x7f5b25b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363b25b8, 55 },
    { 0x59b25b9, 85 },
    { 0x486b25b8, 52 },
    { 0x31eb25b8, 95 },
    { 0x61b25bc, 75 },
    { 0x1d0b25bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169b25c2, 66 },
    { 0xae9b25c2, 33 },
    { 0x9b7b25c4, 90 },
    { 0, 0 },
    { 0xae2b25c6, 84 },
    { 0x4dfb25c7, 76 },
    { 0xf46b25c7, 33 },
    { 0, 0 },
    { 0xddb25ca, 61 },
    { 0x691b25ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eab25ce, 52 },
    { 0, 0 },
    { 0xc1cb25d0, 33 },
    { 0x985b25d0, 98 },
    { 0, 0 },
    { 0xbe4b25d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87b25d9, 55 },
    { 0xd58b25d9, 91 },
    { 0, 0 },
    { 0xdcfb25dc, 90 },
    { 0x901b25dd, 55 },
    { 0x17db25de, 94 },
    { 0x904b25df, 90 },
    { 0xeffb25e0, 90 },
    { 0xe26b25e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14b25e4, 77 },
    { 0, 0 },
    { 0x2dcb25e6, 23 },
    { 0xf66b25e6, 98 },
    { 0xc3bb25e8, 47 },
    { 0, 0 },
    { 0xc13b25ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab25ee, 85 },
    { 0, 0 },
    { 0xfa7b25f0, 95 },
    { 0x564b25f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x977b25f4, 95 },
    { 0xb42b25f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bb25f8, 43 },
    { 0xc5db25f8, 23 },
    { 0x85db25fa, 66 },
    { 0xdceb25fa, 43 },
    { 0, 0 },
    { 0x5e5b25fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577b2601, 95 },
    { 0xe0ab2602, 55 },
    { 0, 0 },
    { 0x967b2604, 61 },
    { 0, 0 },
    { 0xbf7b2606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2609, 43 },
    { 0, 0 },
    { 0xbeb260b, 55 },
    { 0x332b260c, 95 },
    { 0, 0 },
    { 0xe83b260e, 61 },
    { 0x748b260f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b261b, 33 },
    { 0x6cbb261c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b2620, 33 },
    { 0xe7db2621, 47 },
    { 0x2b5b2622, 17 },
    { 0xed3b2621, 90 },
    { 0x830b2621, 61 },
    { 0x569b2625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fb2628, 55 },
    { 0x5c1b2629, 23 },
    { 0, 0 },
    { 0x1d9b262b, 80 },
    { 0xea9b262b, 90 },
    { 0x794b262d, 95 },
    { 0x3b4b262e, 61 },
    { 0x397b262f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461b2633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ab2636, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x426b2639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b263c, 33 },
    { 0x4d7b263c, 55 },
    { 0xd42b263e, 23 },
    { 0xe1ab263d, 33 },
    { 0, 0 },
    { 0xfa8b2641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920b2647, 61 },
    { 0xa41b2648, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44b264d, 76 },
    { 0xca8b264e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b2657, 95 },
    { 0xb3ab2657, 95 },
    { 0xe40b2659, 90 },
    { 0, 0 },
    { 0xc2db265b, 61 },
    { 0, 0 },
    { 0x704b265d, 91 },
    { 0x5fbb265d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b2663, 34 },
    { 0, 0 },
    { 0x3a4b2665, 98 },
    { 0, 0 },
    { 0xa30b2667, 98 },
    { 0x62b2667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb266c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b2673, 98 },
    { 0xe74b2673, 100 },
    { 0, 0 },
    { 0x1e5b2676, 98 },
    { 0xc70b2677, 90 },
    { 0, 0 },
    { 0x9cab2679, 23 },
    { 0xc29b2679, 33 },
    { 0x65eb267b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2b2680, 61 },
    { 0, 0 },
    { 0x691b2682, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b268c, 22 },
    { 0xa8cb268c, 99 },
    { 0x551b268e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2b2696, 90 },
    { 0, 0 },
    { 0x10fb2698, 61 },
    { 0, 0 },
    { 0x433b269a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72db269d, 98 },
    { 0x1c0b269e, 90 },
    { 0x698b269f, 90 },
    { 0xfddb269d, 52 },
    { 0x1d9b26a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b26a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8b26ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0b26af, 23 },
    { 0xfdab26b0, 61 },
    { 0x98fb26b1, 99 },
    { 0, 0 },
    { 0x115b26b3, 90 },
    { 0x7e4b26b4, 99 },
    { 0xd1ab26b4, 23 },
    { 0x98cb26b4, 98 },
    { 0xf62b26b7, 90 },
    { 0xdb6b26b6, 52 },
    { 0xb39b26b8, 53 },
    { 0xf68b26b6, 33 },
    { 0x288b26bb, 90 },
    { 0x68bb26ba, 23 },
    { 0x78cb26b3, 95 },
    { 0x8fdb26b7, 99 },
    { 0, 0 },
    { 0x30b26c0, 90 },
    { 0, 0 },
    { 0xb3ab26c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20db26c7, 56 },
    { 0x85ab26c7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cb26d0, 95 },
    { 0xb53b26d0, 61 },
    { 0, 0 },
    { 0x41db26d3, 78 },
    { 0x70ab26d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b26d7, 90 },
    { 0x9db26d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b26de, 78 },
    { 0x53bb26df, 85 },
    { 0x384b26df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67b26e3, 76 },
    { 0xa6ab26e4, 90 },
    { 0x2a7b26e5, 46 },
    { 0x465b26e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248b26ec, 91 },
    { 0, 0 },
    { 0x9b8b26ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0b26f2, 91 },
    { 0xf5b26f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9b26f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdb26fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744b2700, 52 },
    { 0x6f2b2700, 61 },
    { 0x281b2702, 55 },
    { 0x9abb2703, 52 },
    { 0xc17b2704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7b2708, 90 },
    { 0, 0 },
    { 0xa60b270a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb270d, 84 },
    { 0x13db270e, 76 },
    { 0x58db270f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x889b2712, 87 },
    { 0x6cfb2713, 87 },
    { 0x63cb2712, 37 },
    { 0xad8b2715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292b2719, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x957b271c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2b271f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x384b2722, 90 },
    { 0xde3b2723, 84 },
    { 0, 0 },
    { 0xe82b2725, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb2729, 51 },
    { 0x15eb2729, 61 },
    { 0, 0 },
    { 0x8b2b272c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9b2732, 23 },
    { 0, 0 },
    { 0x25eb2734, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62b2738, 90 },
    { 0x461b2738, 100 },
    { 0xb7ab2738, 66 },
    { 0x287b273b, 23 },
    { 0xf5ab273c, 43 },
    { 0x5dcb273c, 76 },
    { 0x5e3b273e, 34 },
    { 0x60fb273d, 76 },
    { 0xaa0b2740, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab2747, 23 },
    { 0, 0 },
    { 0x6e5b2749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb274c, 90 },
    { 0xe0bb274d, 90 },
    { 0x395b274e, 98 },
    { 0x5d3b274f, 95 },
    { 0xa88b2750, 78 },
    { 0, 0 },
    { 0xba7b2752, 87 },
    { 0, 0 },
    { 0x1b4b2754, 33 },
    { 0x53bb2755, 33 },
    { 0, 0 },
    { 0x3f8b2757, 90 },
    { 0xe3eb2757, 84 },
    { 0, 0 },
    { 0x7bbb275a, 61 },
    { 0x528b275b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b275e, 23 },
    { 0, 0 },
    { 0x6ecb2760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bb2763, 23 },
    { 0xf1cb2763, 95 },
    { 0, 0 },
    { 0xe94b2766, 91 },
    { 0x49b2767, 95 },
    { 0x6b8b2768, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b276b, 98 },
    { 0, 0 },
    { 0x91eb276d, 61 },
    { 0xe50b276d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65b2774, 90 },
    { 0xf05b2775, 98 },
    { 0, 0 },
    { 0xb1bb2777, 76 },
    { 0xfbbb2778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54db2780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80b2783, 33 },
    { 0, 0 },
    { 0xc03b2785, 61 },
    { 0xb7eb2786, 33 },
    { 0x1f0b2787, 90 },
    { 0xc2fb2787, 61 },
    { 0, 0 },
    { 0x7ebb278a, 93 },
    { 0x6dbb278b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab278e, 98 },
    { 0x7a0b278f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfb2792, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ab2796, 33 },
    { 0x9aab2797, 98 },
    { 0xd9b2796, 33 },
    { 0x353b2797, 90 },
    { 0, 0 },
    { 0x5ccb279b, 90 },
    { 0xdfab279b, 68 },
    { 0x470b279b, 95 },
    { 0xa38b279e, 34 },
    { 0x8b5b279d, 98 },
    { 0xc6eb279c, 90 },
    { 0xec7b279b, 61 },
    { 0x38bb279c, 43 },
    { 0x23fb27a3, 85 },
    { 0, 0 },
    { 0xf06b27a5, 33 },
    { 0x47cb27a5, 76 },
    { 0xed6b27a5, 33 },
    { 0x47b27a6, 97 },
    { 0x268b27a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b27b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x985b27b6, 95 },
    { 0xaf2b27b7, 78 },
    { 0xb21b27b7, 90 },
    { 0x5f5b27b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb27bd, 23 },
    { 0, 0 },
    { 0xb7bb27bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b27c2, 34 },
    { 0xa2db27c3, 76 },
    { 0xfc1b27c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fb27c9, 91 },
    { 0, 0 },
    { 0xbbeb27cb, 34 },
    { 0x7eeb27cb, 33 },
    { 0x127b27cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b27d0, 69 },
    { 0x517b27d1, 95 },
    { 0xf43b27d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30b27d5, 23 },
    { 0x1fab27d5, 61 },
    { 0x91b27d6, 33 },
    { 0xa33b27d6, 76 },
    { 0xe6cb27d7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1b27de, 55 },
    { 0x87eb27de, 90 },
    { 0xd21b27e0, 61 },
    { 0x3b3b27e0, 33 },
    { 0xe72b27df, 23 },
    { 0, 0 },
    { 0xb85b27e4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb27e7, 90 },
    { 0, 0 },
    { 0x5f0b27e9, 33 },
    { 0x383b27ea, 90 },
    { 0x709b27eb, 33 },
    { 0, 0 },
    { 0xb20b27ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bb27f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b27f7, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205b27fc, 76 },
    { 0xde3b27fd, 18 },
    { 0x9edb27fe, 95 },
    { 0xeb0b27fc, 68 },
    { 0xaedb2800, 61 },
    { 0x9dcb2800, 76 },
    { 0x431b27fe, 33 },
    { 0x4bbb2803, 90 },
    { 0x22ab27fc, 95 },
    { 0, 0 },
    { 0x86db2806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fb280a, 99 },
    { 0xc76b280a, 98 },
    { 0xeeab280c, 99 },
    { 0xeafb280c, 76 },
    { 0xb2b280e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bb2814, 33 },
    { 0xc2bb2814, 98 },
    { 0, 0 },
    { 0x885b2817, 83 },
    { 0x370b2817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bb281b, 33 },
    { 0, 0 },
    { 0x5afb281d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eb2828, 23 },
    { 0xfbb2829, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x803b282c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ab282f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792b2835, 61 },
    { 0xb81b2836, 61 },
    { 0x1cfb2837, 90 },
    { 0, 0 },
    { 0x73cb2839, 23 },
    { 0, 0 },
    { 0x8c0b283b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb2842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb2845, 61 },
    { 0, 0 },
    { 0x588b2847, 95 },
    { 0x6dcb2848, 76 },
    { 0x335b2849, 93 },
    { 0x185b284a, 100 },
    { 0x9c1b284b, 95 },
    { 0xab6b284c, 98 },
    { 0, 0 },
    { 0x278b284e, 95 },
    { 0, 0 },
    { 0xa14b2850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b2853, 83 },
    { 0xe09b2854, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22db2858, 52 },
    { 0x977b2859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b2860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb2864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b2867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beb286e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18b2872, 77 },
    { 0x9ebb2873, 63 },
    { 0x5a9b2874, 78 },
    { 0, 0 },
    { 0x7e5b2876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fb287e, 66 },
    { 0x839b287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365b2882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb2885, 33 },
    { 0, 0 },
    { 0x696b2887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x926b288a, 93 },
    { 0, 0 },
    { 0x6b1b288c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40b288f, 95 },
    { 0x8a9b2890, 90 },
    { 0, 0 },
    { 0x483b2892, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882b289b, 52 },
    { 0x468b289b, 90 },
    { 0xd67b289b, 23 },
    { 0xcc7b289e, 68 },
    { 0xec2b289d, 90 },
    { 0, 0 },
    { 0x491b28a1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdb28a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfeb28ac, 87 },
    { 0x3f5b28ac, 61 },
    { 0x361b28ac, 77 },
    { 0xc4fb28af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb28b3, 34 },
    { 0, 0 },
    { 0xb8ab28b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7b28ba, 61 },
    { 0xe09b28bb, 98 },
    { 0x736b28bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb28c7, 55 },
    { 0, 0 },
    { 0xc23b28c9, 95 },
    { 0, 0 },
    { 0x3f9b28cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efb28d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8b28d5, 93 },
    { 0xa87b28d5, 61 },
    { 0x354b28d6, 90 },
    { 0, 0 },
    { 0x45b28d9, 61 },
    { 0x257b28da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab28dd, 33 },
    { 0x8db28dd, 76 },
    { 0x15db28de, 36 },
    { 0xab2b28de, 65 },
    { 0, 0 },
    { 0x63fb28e2, 95 },
    { 0, 0 },
    { 0xa7cb28e4, 90 },
    { 0x198b28e4, 43 },
    { 0xb3b28e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbb28ea, 99 },
    { 0x842b28eb, 23 },
    { 0xfc1b28ea, 76 },
    { 0x320b28ec, 4 },
    { 0x3c9b28ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b28f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9b2902, 68 },
    { 0x4ab2903, 90 },
    { 0x3c2b2902, 55 },
    { 0, 0 },
    { 0xc55b2906, 33 },
    { 0xa41b2907, 55 },
    { 0xc1bb2907, 85 },
    { 0x725b2908, 90 },
    { 0x518b2906, 33 },
    { 0x88ab290b, 85 },
    { 0x1e1b2908, 90 },
    { 0, 0 },
    { 0xdb0b290e, 90 },
    { 0x202b290f, 39 },
    { 0xb33b2910, 33 },
    { 0x3feb2911, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938b2917, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feb2922, 90 },
    { 0x428b2923, 75 },
    { 0x933b2924, 98 },
    { 0x2eab2923, 34 },
    { 0x577b2926, 43 },
    { 0xa20b2927, 69 },
    { 0, 0 },
    { 0x828b2929, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab2930, 90 },
    { 0xee6b2931, 90 },
    { 0x5f4b2932, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b2936, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb293d, 23 },
    { 0, 0 },
    { 0xba8b293f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7b2944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b294e, 95 },
    { 0, 0 },
    { 0xea4b2950, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189b2954, 23 },
    { 0x6adb2954, 23 },
    { 0x845b2956, 98 },
    { 0xe3ab2957, 90 },
    { 0x65fb2956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6b295b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261b2960, 90 },
    { 0xd60b2961, 98 },
    { 0, 0 },
    { 0xf66b2963, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22b2969, 61 },
    { 0x9eab2969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0b296f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99b2982, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bb298d, 33 },
    { 0, 0 },
    { 0x3c9b298f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b2992, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b2996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8b299a, 98 },
    { 0, 0 },
    { 0x22fb299c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b299f, 87 },
    { 0x453b29a0, 99 },
    { 0, 0 },
    { 0xd02b29a2, 91 },
    { 0x2e2b29a3, 61 },
    { 0x763b29a2, 75 },
    { 0xb4bb29a2, 85 },
    { 0xb84b29a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x761b29a9, 90 },
    { 0x5e4b29a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8b29b0, 84 },
    { 0x732b29b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1b29b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b29bc, 95 },
    { 0x108b29bc, 95 },
    { 0xcf6b29be, 46 },
    { 0x6ccb29be, 90 },
    { 0x2efb29c0, 91 },
    { 0xd03b29c1, 55 },
    { 0xaecb29c1, 23 },
    { 0xf78b29c3, 90 },
    { 0x4c2b29c3, 95 },
    { 0x870b29bd, 95 },
    { 0x2f4b29c3, 95 },
    { 0x57ab29c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb29cb, 23 },
    { 0, 0 },
    { 0x34fb29cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136b29d1, 99 },
    { 0x52eb29d2, 23 },
    { 0xe9db29d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9b29d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b29da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb29e4, 44 },
    { 0x1d5b29e5, 46 },
    { 0, 0 },
    { 0xa1b29e7, 90 },
    { 0xc18b29e8, 34 },
    { 0xb83b29e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fb29ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b29f8, 95 },
    { 0xda3b29f8, 75 },
    { 0xe72b29f9, 43 },
    { 0, 0 },
    { 0xc44b29fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86b29ff, 34 },
    { 0, 0 },
    { 0xd1bb2a01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2b2a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb2a0a, 95 },
    { 0x6fdb2a0b, 84 },
    { 0, 0 },
    { 0x462b2a0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eb2a11, 98 },
    { 0xce2b2a12, 90 },
    { 0x8d7b2a13, 77 },
    { 0xc44b2a12, 76 },
    { 0x940b2a11, 19 },
    { 0xffbb2a16, 55 },
    { 0x818b2a16, 76 },
    { 0, 0 },
    { 0x6afb2a19, 17 },
    { 0, 0 },
    { 0xff3b2a1b, 99 },
    { 0xb2ab2a1c, 76 },
    { 0x6f0b2a1c, 99 },
    { 0xb53b2a1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x291b2a21, 33 },
    { 0x609b2a22, 98 },
    { 0x28cb2a22, 95 },
    { 0x85cb2a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0b2a2a, 34 },
    { 0xd7cb2a2b, 90 },
    { 0x102b2a2c, 90 },
    { 0x86db2a2d, 88 },
    { 0x930b2a2c, 90 },
    { 0xff8b2a2a, 75 },
    { 0, 0 },
    { 0xeceb2a31, 95 },
    { 0, 0 },
    { 0x395b2a33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4db2a37, 61 },
    { 0, 0 },
    { 0x48cb2a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b2a46, 34 },
    { 0xe3db2a46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb2a4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db2a4d, 61 },
    { 0, 0 },
    { 0x9cb2a4f, 98 },
    { 0x448b2a4f, 46 },
    { 0x8db2a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b2a56, 98 },
    { 0x59ab2a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cb2a5c, 90 },
    { 0x274b2a5d, 61 },
    { 0xf44b2a5d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5db2a63, 84 },
    { 0xb85b2a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25eb2a67, 98 },
    { 0x50fb2a68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b2a6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b2a73, 90 },
    { 0, 0 },
    { 0x10cb2a75, 52 },
    { 0, 0 },
    { 0x960b2a77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7b2a81, 33 },
    { 0xcd2b2a82, 33 },
    { 0, 0 },
    { 0x77eb2a84, 23 },
    { 0xf30b2a85, 76 },
    { 0x549b2a85, 33 },
    { 0xc51b2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97b2a8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b2a92, 90 },
    { 0x51b2a92, 61 },
    { 0, 0 },
    { 0x4b8b2a95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bab2a9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7b2a9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157b2aa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1b2aa9, 76 },
    { 0xeeb2aaa, 93 },
    { 0xad2b2aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfb2ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b2ab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b2ab8, 66 },
    { 0xba4b2ab9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4b2abd, 67 },
    { 0x2b8b2abd, 61 },
    { 0xdddb2abf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eb2ac5, 95 },
    { 0x626b2ac6, 61 },
    { 0x709b2ac7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6b2acb, 33 },
    { 0xc60b2acc, 34 },
    { 0x2cbb2acc, 46 },
    { 0x43ab2ace, 33 },
    { 0x845b2acd, 34 },
    { 0x17b2acf, 76 },
    { 0, 0 },
    { 0x8deb2ad2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb2ad8, 93 },
    { 0, 0 },
    { 0x8e6b2ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab2add, 61 },
    { 0, 0 },
    { 0x325b2adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2ae3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b2ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668b2aea, 98 },
    { 0xf9db2aeb, 76 },
    { 0xf1eb2aeb, 61 },
    { 0x1d0b2aeb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cb2af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b2af9, 33 },
    { 0xf1fb2afa, 23 },
    { 0xf6db2af9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb2b00, 66 },
    { 0xd57b2b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x519b2b04, 33 },
    { 0xbecb2b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30b2b09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb2b0f, 98 },
    { 0xa72b2b10, 33 },
    { 0, 0 },
    { 0x2f2b2b12, 81 },
    { 0x9cdb2b13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78b2b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19db2b25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164b2b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43b2b30, 61 },
    { 0xe49b2b31, 90 },
    { 0xe8db2b31, 23 },
    { 0xd4bb2b31, 23 },
    { 0x921b2b32, 75 },
    { 0xf3ab2b32, 66 },
    { 0, 0 },
    { 0x2e8b2b37, 61 },
    { 0, 0 },
    { 0x43cb2b39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x664b2b3c, 43 },
    { 0x4a0b2b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb2b40, 23 },
    { 0, 0 },
    { 0xc68b2b42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2b2b45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb2b49, 98 },
    { 0x311b2b4a, 91 },
    { 0x938b2b4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80b2b51, 33 },
    { 0x28eb2b52, 76 },
    { 0, 0 },
    { 0xe0ab2b54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149b2b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206b2b60, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8b2b64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b2b68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64b2b6e, 76 },
    { 0, 0 },
    { 0x480b2b70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ab2b73, 52 },
    { 0, 0 },
    { 0x4e6b2b75, 33 },
    { 0x62cb2b75, 52 },
    { 0x8c8b2b77, 67 },
    { 0x382b2b78, 100 },
    { 0x8b1b2b79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb2b7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51db2b7f, 76 },
    { 0, 0 },
    { 0x86bb2b81, 43 },
    { 0x98ab2b82, 23 },
    { 0, 0 },
    { 0x8adb2b84, 61 },
    { 0, 0 },
    { 0xabbb2b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869b2b8b, 46 },
    { 0, 0 },
    { 0x15cb2b8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2b92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277b2b96, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebb2b99, 98 },
    { 0xd95b2b9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2b9e, 34 },
    { 0x866b2b9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224b2bad, 93 },
    { 0x5e9b2bae, 61 },
    { 0xee7b2bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b2bb4, 92 },
    { 0, 0 },
    { 0x76ab2bb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfb2bb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7b2bbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bb2bc4, 84 },
    { 0x6b1b2bc5, 33 },
    { 0x62ab2bc4, 98 },
    { 0, 0 },
    { 0xaefb2bc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x471b2bcb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb2bd1, 17 },
    { 0xf9b2bd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2b2bd9, 61 },
    { 0xfb5b2bd9, 95 },
    { 0x437b2bdb, 78 },
    { 0xc16b2bdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fb2bdf, 100 },
    { 0, 0 },
    { 0xb1cb2be1, 93 },
    { 0x662b2be1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x942b2be5, 93 },
    { 0, 0 },
    { 0xd7ab2be7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b2bea, 77 },
    { 0, 0 },
    { 0xecb2bec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b2bf7, 61 },
    { 0xccdb2bf7, 67 },
    { 0xa16b2bf7, 93 },
    { 0, 0 },
    { 0xd4db2bfb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b2c00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bb2c05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0b2c09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2b2c0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841b2c12, 46 },
    { 0, 0 },
    { 0x1d2b2c14, 33 },
    { 0xcedb2c14, 95 },
    { 0, 0 },
    { 0x3e4b2c17, 33 },
    { 0xca7b2c18, 95 },
    { 0, 0 },
    { 0xeaeb2c1a, 40 },
    { 0x24cb2c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb2c1e, 33 },
    { 0xd5cb2c1f, 95 },
    { 0x63ab2c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110b2c24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab2c2a, 23 },
    { 0x36bb2c2a, 34 },
    { 0x3b6b2c2c, 33 },
    { 0xba7b2c2a, 61 },
    { 0, 0 },
    { 0xf65b2c2f, 33 },
    { 0x116b2c30, 61 },
    { 0x902b2c31, 61 },
    { 0x75bb2c2f, 98 },
    { 0x384b2c33, 48 },
    { 0x81fb2c34, 61 },
    { 0x3b2b2c35, 100 },
    { 0xcc9b2c36, 76 },
    { 0xfffb2c37, 43 },
    { 0x8c4b2c38, 55 },
    { 0x6d3b2c39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512b2c3e, 99 },
    { 0, 0 },
    { 0xfafb2c40, 81 },
    { 0x760b2c41, 100 },
    { 0x31eb2c42, 61 },
    { 0x2dcb2c40, 55 },
    { 0x3cb2c43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cb2c54, 80 },
    { 0, 0 },
    { 0x70eb2c56, 61 },
    { 0x6eab2c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b2c5b, 40 },
    { 0x330b2c5c, 33 },
    { 0x2f4b2c5c, 43 },
    { 0xe66b2c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb2c62, 90 },
    { 0xb4cb2c63, 68 },
    { 0x806b2c63, 55 },
    { 0, 0 },
    { 0x237b2c66, 93 },
    { 0xacdb2c66, 78 },
    { 0xf57b2c68, 23 },
    { 0x33eb2c68, 84 },
    { 0, 0 },
    { 0x8f8b2c6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8b2c6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16b2c72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb2c79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb2c7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb2c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfb2c86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bb2c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b2c8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788b2c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20b2ca1, 68 },
    { 0, 0 },
    { 0x5eab2ca3, 23 },
    { 0xe09b2ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ab2cac, 90 },
    { 0x913b2cac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379b2cb1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9b2cb4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b2cb9, 84 },
    { 0xb2eb2cba, 98 },
    { 0x91b2cba, 34 },
    { 0xe4cb2cba, 95 },
    { 0xe06b2cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4b2cc1, 85 },
    { 0, 0 },
    { 0xaafb2cc3, 90 },
    { 0, 0 },
    { 0xdc4b2cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b2cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b2cd0, 85 },
    { 0xdf8b2cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb2cd4, 93 },
    { 0, 0 },
    { 0xbbbb2cd6, 81 },
    { 0x9bab2cd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadb2cdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cb2cdf, 23 },
    { 0xf28b2ce0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2b2ce5, 60 },
    { 0x34b2ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62eb2ceb, 90 },
    { 0, 0 },
    { 0x322b2ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb2cf5, 93 },
    { 0, 0 },
    { 0x9e3b2cf7, 55 },
    { 0x2f2b2cf8, 33 },
    { 0x8fcb2cf9, 61 },
    { 0xe0eb2cf9, 98 },
    { 0, 0 },
    { 0x277b2cfc, 90 },
    { 0, 0 },
    { 0x6b6b2cfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793b2d01, 84 },
    { 0, 0 },
    { 0x36fb2d03, 61 },
    { 0, 0 },
    { 0x7feb2d05, 98 },
    { 0x44db2d05, 90 },
    { 0xfc6b2d05, 61 },
    { 0x7a0b2d08, 55 },
    { 0x985b2d07, 61 },
    { 0xf70b2d07, 61 },
    { 0xae9b2d06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8b2d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b2d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b2d1c, 94 },
    { 0xfc1b2d1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b2d21, 93 },
    { 0, 0 },
    { 0xa8eb2d23, 33 },
    { 0x3e4b2d24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b2d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x802b2d2d, 61 },
    { 0, 0 },
    { 0x87fb2d2f, 99 },
    { 0, 0 },
    { 0xed6b2d31, 23 },
    { 0x404b2d31, 91 },
    { 0, 0 },
    { 0x36eb2d34, 90 },
    { 0x552b2d34, 95 },
    { 0, 0 },
    { 0xabb2d37, 77 },
    { 0x4bcb2d38, 46 },
    { 0xb4bb2d39, 76 },
    { 0x877b2d3a, 33 },
    { 0x6aeb2d3b, 61 },
    { 0x480b2d3c, 91 },
    { 0x90eb2d3a, 76 },
    { 0x730b2d3b, 98 },
    { 0xf70b2d38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffb2d42, 33 },
    { 0x280b2d43, 66 },
    { 0x57ab2d44, 61 },
    { 0x4ecb2d45, 85 },
    { 0x6ceb2d43, 76 },
    { 0x10b2d45, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b2d4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b2d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3b2d57, 100 },
    { 0, 0 },
    { 0x6d3b2d59, 23 },
    { 0x6beb2d5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fb2d5e, 33 },
    { 0xd4cb2d5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb2d68, 91 },
    { 0, 0 },
    { 0x11eb2d6a, 61 },
    { 0xf1db2d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b2d78, 99 },
    { 0xd6eb2d78, 55 },
    { 0x21fb2d7a, 34 },
    { 0, 0 },
    { 0x57bb2d7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65b2d8a, 46 },
    { 0, 0 },
    { 0x9b7b2d8c, 61 },
    { 0x249b2d8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2d92, 90 },
    { 0x624b2d93, 23 },
    { 0x443b2d94, 33 },
    { 0xbceb2d92, 90 },
    { 0xcc0b2d96, 76 },
    { 0x9c1b2d97, 100 },
    { 0xb7bb2d98, 76 },
    { 0, 0 },
    { 0x275b2d9a, 34 },
    { 0x3b0b2d9a, 96 },
    { 0xebeb2d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b2da0, 61 },
    { 0, 0 },
    { 0x1a7b2da2, 34 },
    { 0x6c5b2da3, 90 },
    { 0, 0 },
    { 0x17ab2da5, 61 },
    { 0x3dfb2da5, 95 },
    { 0x168b2da7, 85 },
    { 0, 0 },
    { 0x43ab2da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b2dac, 61 },
    { 0x696b2dac, 90 },
    { 0x959b2dac, 93 },
    { 0x8bb2daf, 76 },
    { 0x272b2daf, 61 },
    { 0x151b2db1, 43 },
    { 0x113b2db2, 34 },
    { 0x33bb2dad, 90 },
    { 0, 0 },
    { 0xe7b2db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503b2db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466b2dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bb2dc4, 33 },
    { 0x54b2dc5, 33 },
    { 0xd59b2dc6, 95 },
    { 0x855b2dc7, 65 },
    { 0xc30b2dc4, 95 },
    { 0xabbb2dc9, 23 },
    { 0xd15b2dca, 33 },
    { 0xd87b2dc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b2dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439b2dd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8b2dda, 61 },
    { 0x8adb2ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b2dde, 91 },
    { 0xa72b2ddf, 61 },
    { 0xb4b2de0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb2de6, 52 },
    { 0x935b2de7, 99 },
    { 0xdc5b2de8, 78 },
    { 0x63cb2de9, 33 },
    { 0, 0 },
    { 0x84b2deb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16b2df0, 93 },
    { 0x197b2df1, 33 },
    { 0x1a3b2df2, 34 },
    { 0xb6fb2df3, 87 },
    { 0, 0 },
    { 0xba1b2df5, 61 },
    { 0, 0 },
    { 0xfddb2df7, 92 },
    { 0x89bb2df8, 99 },
    { 0x3adb2df9, 33 },
    { 0x80b2df8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0b2dfd, 90 },
    { 0, 0 },
    { 0x938b2dff, 61 },
    { 0xfa9b2dff, 76 },
    { 0x7b8b2e01, 85 },
    { 0xed9b2dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x790b2e05, 33 },
    { 0, 0 },
    { 0xdb3b2e07, 55 },
    { 0x31ab2e07, 93 },
    { 0x6c8b2e09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b2e0c, 76 },
    { 0x37eb2e0d, 98 },
    { 0, 0 },
    { 0x52eb2e0f, 61 },
    { 0x654b2e0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdb2e15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b2e1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290b2e23, 93 },
    { 0, 0 },
    { 0xdacb2e25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b2e28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb2e2d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beb2e34, 43 },
    { 0x35eb2e35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b2e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78b2e40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eb2e43, 23 },
    { 0, 0 },
    { 0xc59b2e45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2e49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b2e4c, 90 },
    { 0x2c9b2e4d, 78 },
    { 0x997b2e4e, 33 },
    { 0xb0eb2e4f, 34 },
    { 0xba0b2e4d, 100 },
    { 0x1a8b2e51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eb2e55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db2e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ab2e5d, 95 },
    { 0, 0 },
    { 0x3f3b2e5f, 61 },
    { 0, 0 },
    { 0xc34b2e61, 91 },
    { 0, 0 },
    { 0xebdb2e63, 90 },
    { 0xdb1b2e64, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b2e67, 98 },
    { 0x816b2e67, 98 },
    { 0x320b2e69, 90 },
    { 0x43fb2e6a, 90 },
    { 0x75b2e68, 33 },
    { 0x692b2e6a, 90 },
    { 0x4e0b2e6d, 78 },
    { 0xff3b2e6c, 61 },
    { 0, 0 },
    { 0x815b2e70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb2e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b2e79, 90 },
    { 0xc2fb2e7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfb2e80, 43 },
    { 0x288b2e80, 84 },
    { 0xb96b2e82, 55 },
    { 0x879b2e82, 61 },
    { 0, 0 },
    { 0x190b2e85, 98 },
    { 0x314b2e86, 61 },
    { 0, 0 },
    { 0xf36b2e88, 23 },
    { 0xdc2b2e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912b2e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0db2e90, 33 },
    { 0x176b2e91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb2e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449b2e99, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980b2e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405b2ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5b2ea9, 34 },
    { 0xedeb2eaa, 66 },
    { 0x56db2eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42b2eae, 78 },
    { 0x90db2eaf, 61 },
    { 0xd4db2eaf, 69 },
    { 0x98eb2eb1, 33 },
    { 0xb8bb2eaf, 100 },
    { 0, 0 },
    { 0x7d9b2eb4, 78 },
    { 0, 0 },
    { 0x438b2eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab2ebc, 19 },
    { 0, 0 },
    { 0x948b2ebe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301b2ec2, 34 },
    { 0, 0 },
    { 0xd88b2ec4, 43 },
    { 0x141b2ec4, 95 },
    { 0x68bb2ec6, 39 },
    { 0xf24b2ec6, 76 },
    { 0x2a9b2ec8, 90 },
    { 0x551b2ec8, 95 },
    { 0, 0 },
    { 0x587b2ecb, 93 },
    { 0x29b2ecc, 23 },
    { 0, 0 },
    { 0xa9ab2ece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1b2ed1, 68 },
    { 0x5eeb2ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27b2ed5, 90 },
    { 0x14b2ed5, 91 },
    { 0x3eab2ed7, 94 },
    { 0xa35b2ed8, 99 },
    { 0xcb4b2ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb2edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3b2edf, 55 },
    { 0xc1fb2edf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebb2eec, 33 },
    { 0x67db2eec, 67 },
    { 0xd30b2eec, 86 },
    { 0xb90b2eee, 90 },
    { 0, 0 },
    { 0x701b2ef1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2ef4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfb2ef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986b2efc, 61 },
    { 0xb7ab2efc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1b2f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10b2f03, 98 },
    { 0, 0 },
    { 0xd25b2f05, 76 },
    { 0x447b2f06, 98 },
    { 0x59db2f07, 95 },
    { 0x377b2f08, 23 },
    { 0x737b2f08, 46 },
    { 0x848b2f09, 33 },
    { 0xf0db2f0b, 61 },
    { 0x8b4b2f0c, 87 },
    { 0, 0 },
    { 0x626b2f0e, 23 },
    { 0x1f5b2f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bb2f14, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b2f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b2f1c, 98 },
    { 0, 0 },
    { 0x3a2b2f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39b2f21, 96 },
    { 0x142b2f22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eb2f2b, 100 },
    { 0xab2b2f2c, 81 },
    { 0xca0b2f2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecb2f32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcb2f3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b2f40, 87 },
    { 0x3f8b2f41, 98 },
    { 0xc46b2f41, 23 },
    { 0, 0 },
    { 0xffb2f44, 76 },
    { 0, 0 },
    { 0x673b2f46, 66 },
    { 0xcdcb2f46, 99 },
    { 0xf9ab2f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771b2f4c, 98 },
    { 0, 0 },
    { 0xf9db2f4e, 33 },
    { 0x190b2f4f, 38 },
    { 0, 0 },
    { 0x1b2f51, 40 },
    { 0, 0 },
    { 0x59db2f53, 15 },
    { 0x148b2f54, 95 },
    { 0xea6b2f54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9b2f59, 44 },
    { 0xb04b2f59, 93 },
    { 0xde1b2f5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb2f5e, 95 },
    { 0, 0 },
    { 0x8c2b2f60, 98 },
    { 0, 0 },
    { 0x504b2f62, 93 },
    { 0xdb4b2f63, 61 },
    { 0x222b2f64, 98 },
    { 0, 0 },
    { 0x285b2f66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b2f6f, 33 },
    { 0x150b2f70, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ab2f7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297b2f82, 91 },
    { 0, 0 },
    { 0xa78b2f84, 95 },
    { 0, 0 },
    { 0x3e7b2f86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebb2f89, 34 },
    { 0x29cb2f89, 95 },
    { 0x610b2f8b, 98 },
    { 0xecdb2f89, 95 },
    { 0, 0 },
    { 0xec5b2f8e, 95 },
    { 0x3b1b2f8f, 91 },
    { 0xb6cb2f90, 46 },
    { 0xcdb2f91, 76 },
    { 0, 0 },
    { 0x175b2f93, 90 },
    { 0xc0fb2f94, 23 },
    { 0, 0 },
    { 0x432b2f96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb2f9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2fa1, 33 },
    { 0, 0 },
    { 0xa3fb2fa3, 61 },
    { 0x7acb2fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b2fa7, 61 },
    { 0x961b2fa8, 33 },
    { 0x381b2fa7, 93 },
    { 0, 0 },
    { 0x8d8b2fab, 33 },
    { 0, 0 },
    { 0xc70b2fad, 43 },
    { 0x923b2fae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b2fb1, 90 },
    { 0, 0 },
    { 0xc5eb2fb3, 81 },
    { 0x4f8b2fb4, 65 },
    { 0, 0 },
    { 0x1b5b2fb6, 55 },
    { 0x12cb2fb7, 43 },
    { 0, 0 },
    { 0xd61b2fb9, 23 },
    { 0, 0 },
    { 0xca3b2fbb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b2fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697b2fc1, 95 },
    { 0, 0 },
    { 0x678b2fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59db2fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb2fcb, 78 },
    { 0x61bb2fcb, 90 },
    { 0x36fb2fcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab2fd0, 61 },
    { 0, 0 },
    { 0x574b2fd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66b2fd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4b2fe2, 34 },
    { 0x65cb2fe2, 90 },
    { 0, 0 },
    { 0xc05b2fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db2fef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96eb2ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2ff6, 33 },
    { 0x9d6b2ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b2ffa, 98 },
    { 0x349b2ffa, 23 },
    { 0xe0ab2ffa, 33 },
    { 0x39cb2ffd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b3005, 98 },
    { 0x97bb3006, 99 },
    { 0, 0 },
    { 0x692b3008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeb300c, 55 },
    { 0, 0 },
    { 0x594b300e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748b301b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627b301f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77b3024, 34 },
    { 0, 0 },
    { 0xaf5b3026, 90 },
    { 0xc75b3026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5b302a, 61 },
    { 0x8ccb302a, 95 },
    { 0xb0cb302c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb302f, 90 },
    { 0xdcbb3030, 34 },
    { 0x577b302f, 95 },
    { 0, 0 },
    { 0x3e2b3033, 33 },
    { 0, 0 },
    { 0xb2bb3035, 99 },
    { 0xba5b3036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b303d, 23 },
    { 0, 0 },
    { 0x781b303f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b3044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5b304d, 77 },
    { 0, 0 },
    { 0x133b304f, 23 },
    { 0xed6b3050, 87 },
    { 0x577b3051, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b3054, 95 },
    { 0x179b3055, 33 },
    { 0xb81b3056, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0b3059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb3062, 68 },
    { 0xf40b3063, 93 },
    { 0xb2cb3064, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705b3068, 98 },
    { 0x7c8b3069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8b3088, 61 },
    { 0x5e7b3089, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b308e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b3092, 87 },
    { 0xe59b3093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb3096, 90 },
    { 0xa38b3096, 76 },
    { 0, 0 },
    { 0xe54b3099, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b309d, 78 },
    { 0x470b309e, 23 },
    { 0x3a2b309f, 61 },
    { 0x53cb30a0, 93 },
    { 0x6ab30a1, 33 },
    { 0xf1fb30a1, 61 },
    { 0x76ab30a0, 55 },
    { 0, 0 },
    { 0x565b30a5, 65 },
    { 0x5aab30a6, 43 },
    { 0x844b30a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39b30ab, 98 },
    { 0x2dfb30ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3b30b0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2b30bc, 61 },
    { 0x240b30bd, 36 },
    { 0xda6b30bd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cb30c2, 34 },
    { 0, 0 },
    { 0x124b30c4, 52 },
    { 0, 0 },
    { 0xca9b30c6, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6b30cb, 55 },
    { 0x679b30cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473b30d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cb30d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b30d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b30da, 33 },
    { 0x7bb30da, 90 },
    { 0xa4fb30dc, 61 },
    { 0x5d6b30db, 90 },
    { 0xeb2b30da, 55 },
    { 0xb77b30da, 61 },
    { 0xb01b30df, 61 },
    { 0, 0 },
    { 0x7b6b30e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cb30e6, 95 },
    { 0xb06b30e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b30ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b30ed, 76 },
    { 0x67db30ee, 95 },
    { 0xf6db30ef, 90 },
    { 0x127b30f0, 76 },
    { 0xd5eb30f1, 90 },
    { 0x914b30f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61b30f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bb30f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b30fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eb3102, 55 },
    { 0, 0 },
    { 0x4cdb3104, 90 },
    { 0xed4b3105, 23 },
    { 0, 0 },
    { 0xa47b3107, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b3111, 61 },
    { 0, 0 },
    { 0xfecb3113, 93 },
    { 0xb7b3114, 90 },
    { 0x8e3b3115, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b311a, 95 },
    { 0xac1b311b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b311f, 61 },
    { 0x29db3120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3b3125, 95 },
    { 0, 0 },
    { 0xae6b3127, 84 },
    { 0, 0 },
    { 0x2b6b3129, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9b312f, 52 },
    { 0x669b312f, 33 },
    { 0, 0 },
    { 0xd02b3132, 90 },
    { 0xfd4b3133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1b313b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cb3143, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb3146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebb314b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242b314f, 34 },
    { 0x5ddb314f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1b3153, 95 },
    { 0xd4ab3154, 61 },
    { 0, 0 },
    { 0xd3cb3156, 55 },
    { 0xe55b3157, 90 },
    { 0xdf6b3158, 33 },
    { 0x3f1b3159, 95 },
    { 0x478b315a, 93 },
    { 0xaefb315a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6b315e, 33 },
    { 0x898b315e, 95 },
    { 0x7c6b3160, 81 },
    { 0x8e1b315e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2db3164, 76 },
    { 0x7f9b3165, 94 },
    { 0x433b3166, 76 },
    { 0x2bb3167, 66 },
    { 0x4b7b3168, 33 },
    { 0, 0 },
    { 0x829b316a, 69 },
    { 0x6cdb316a, 99 },
    { 0xd8bb316a, 80 },
    { 0, 0 },
    { 0xbfdb316e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b3178, 33 },
    { 0x38cb3178, 23 },
    { 0xed3b317a, 98 },
    { 0xb0ab317b, 61 },
    { 0x5ab317c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b3180, 90 },
    { 0x405b3181, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eb3184, 90 },
    { 0xbb4b3185, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x193b3188, 78 },
    { 0, 0 },
    { 0xc2db318a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74b318d, 95 },
    { 0, 0 },
    { 0x899b318f, 23 },
    { 0x626b3190, 33 },
    { 0x9cfb318f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b3194, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x512b3197, 61 },
    { 0xf0b3198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb31a0, 90 },
    { 0, 0 },
    { 0x5f5b31a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ab31aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b31b7, 100 },
    { 0xa87b31b8, 99 },
    { 0xbddb31b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0b31be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbb31c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750b31c4, 90 },
    { 0x264b31c5, 61 },
    { 0x6f1b31c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b31cb, 46 },
    { 0, 0 },
    { 0x443b31cd, 34 },
    { 0x66db31ce, 95 },
    { 0xe30b31cf, 33 },
    { 0xc53b31d0, 23 },
    { 0xaadb31d1, 61 },
    { 0, 0 },
    { 0xddfb31d3, 70 },
    { 0, 0 },
    { 0x662b31d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfb31d9, 84 },
    { 0x88eb31da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1deb31e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab31e3, 33 },
    { 0x8b7b31e4, 78 },
    { 0x2e7b31e5, 33 },
    { 0xf84b31e4, 24 },
    { 0xac0b31e7, 66 },
    { 0x3eab31e8, 23 },
    { 0xbeeb31e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bb31ed, 33 },
    { 0x9f4b31ee, 90 },
    { 0, 0 },
    { 0xc3db31f0, 90 },
    { 0x2b0b31f1, 83 },
    { 0xe24b31f2, 78 },
    { 0xce2b31f0, 23 },
    { 0x278b31f4, 43 },
    { 0x76db31f4, 77 },
    { 0xdb7b31f6, 68 },
    { 0xedbb31f7, 90 },
    { 0xbfcb31f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b31fd, 77 },
    { 0xe67b31fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db321a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b321f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b3222, 43 },
    { 0, 0 },
    { 0x611b3224, 56 },
    { 0x662b3224, 23 },
    { 0, 0 },
    { 0x993b3227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eb322a, 33 },
    { 0xc82b322b, 90 },
    { 0x9b6b322b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b322f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbb3234, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb323d, 90 },
    { 0x1e2b323d, 55 },
    { 0xee3b323f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b3246, 55 },
    { 0, 0 },
    { 0x296b3248, 90 },
    { 0x607b3249, 40 },
    { 0xaeb324a, 76 },
    { 0xb9db324b, 95 },
    { 0x343b324c, 61 },
    { 0, 0 },
    { 0xd76b324e, 90 },
    { 0xdb2b324f, 33 },
    { 0, 0 },
    { 0xe15b3251, 95 },
    { 0x581b3252, 61 },
    { 0xa69b3252, 95 },
    { 0xd0ab3254, 98 },
    { 0xbcfb3255, 61 },
    { 0x756b3255, 33 },
    { 0xd4ab3257, 23 },
    { 0x23bb3257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880b3264, 43 },
    { 0x9a9b3265, 66 },
    { 0x90bb3266, 33 },
    { 0x2abb3267, 98 },
    { 0x94fb3266, 90 },
    { 0, 0 },
    { 0xd06b326a, 76 },
    { 0x2fcb326b, 67 },
    { 0xad8b326a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b3270, 13 },
    { 0x1a4b3271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47b327b, 43 },
    { 0, 0 },
    { 0x492b327d, 88 },
    { 0xd80b327d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804b3282, 33 },
    { 0x10eb3283, 61 },
    { 0, 0 },
    { 0xea7b3285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9b3289, 86 },
    { 0x8f0b328a, 61 },
    { 0, 0 },
    { 0x9d3b328c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb3291, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b3295, 90 },
    { 0xf57b3296, 90 },
    { 0x908b3297, 95 },
    { 0x890b3295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x458b329b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb32a3, 90 },
    { 0x79b32a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x983b32a7, 90 },
    { 0, 0 },
    { 0x2a9b32a9, 90 },
    { 0x2afb32aa, 98 },
    { 0, 0 },
    { 0x994b32ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecb32af, 52 },
    { 0x159b32b0, 61 },
    { 0x486b32b1, 61 },
    { 0x528b32b0, 90 },
    { 0xc08b32b3, 34 },
    { 0, 0 },
    { 0x4d6b32b5, 23 },
    { 0x87fb32b6, 98 },
    { 0x12db32b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2db32ba, 52 },
    { 0x32bb32ba, 33 },
    { 0xc6eb32bb, 95 },
    { 0, 0 },
    { 0xc9ab32be, 69 },
    { 0x6abb32bf, 43 },
    { 0, 0 },
    { 0x7aeb32c1, 61 },
    { 0x2c2b32c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72b32c6, 46 },
    { 0x9e5b32c7, 90 },
    { 0, 0 },
    { 0x1dbb32c9, 90 },
    { 0, 0 },
    { 0xbf3b32cb, 76 },
    { 0x61fb32cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9b32cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4b32d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eb32d5, 99 },
    { 0x2f1b32d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x501b32d9, 95 },
    { 0x594b32da, 99 },
    { 0x693b32da, 43 },
    { 0x1d9b32dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796b32e8, 90 },
    { 0xef6b32e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b32f2, 98 },
    { 0, 0 },
    { 0x915b32f4, 77 },
    { 0x638b32f4, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b32f9, 43 },
    { 0, 0 },
    { 0x626b32fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eb32ff, 90 },
    { 0x3beb3300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb3303, 52 },
    { 0x306b3304, 90 },
    { 0x77bb3303, 98 },
    { 0x39db3303, 17 },
    { 0xbf5b3307, 100 },
    { 0x971b3308, 62 },
    { 0, 0 },
    { 0x170b330a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1db330e, 34 },
    { 0xa68b330f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb3318, 90 },
    { 0, 0 },
    { 0x7ddb331a, 36 },
    { 0xb6b331a, 67 },
    { 0, 0 },
    { 0x2b0b331d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17b3321, 98 },
    { 0, 0 },
    { 0x243b3323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bb3326, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b332b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b332f, 93 },
    { 0, 0 },
    { 0xf0cb3331, 34 },
    { 0x36fb3332, 76 },
    { 0xf14b3332, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666b3337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab333b, 23 },
    { 0xe6cb333c, 99 },
    { 0, 0 },
    { 0x248b333e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fb3345, 23 },
    { 0xbd2b3346, 78 },
    { 0x9e3b3347, 61 },
    { 0x5f0b3345, 98 },
    { 0x3c9b3347, 86 },
    { 0xeedb3345, 85 },
    { 0xec1b334b, 95 },
    { 0x98db334b, 33 },
    { 0x395b3346, 85 },
    { 0xb81b334d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb3352, 95 },
    { 0x9beb3353, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3b3356, 93 },
    { 0x97b3357, 88 },
    { 0xd47b3358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdb335b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab335f, 55 },
    { 0x456b3360, 23 },
    { 0, 0 },
    { 0xe7eb3362, 77 },
    { 0xf41b3362, 61 },
    { 0xb4ab3362, 90 },
    { 0x41ab3362, 33 },
    { 0xffcb3363, 33 },
    { 0x727b3367, 43 },
    { 0x63db3367, 90 },
    { 0xe0b3366, 98 },
    { 0xec4b3363, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b3370, 98 },
    { 0x706b3370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33b3376, 77 },
    { 0x89db3376, 95 },
    { 0, 0 },
    { 0xc70b3379, 95 },
    { 0xa87b3379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6b3380, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab3383, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b3387, 68 },
    { 0xa23b3388, 95 },
    { 0, 0 },
    { 0xce3b338a, 88 },
    { 0xea7b338a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b338f, 95 },
    { 0x7b6b3390, 23 },
    { 0xd6fb3391, 88 },
    { 0x8b6b338f, 23 },
    { 0xdd9b3391, 61 },
    { 0x5c7b3393, 90 },
    { 0x441b3395, 93 },
    { 0x93ab3396, 81 },
    { 0xcb4b3395, 90 },
    { 0, 0 },
    { 0x755b3399, 61 },
    { 0xdabb3399, 33 },
    { 0x846b339a, 90 },
    { 0x3f2b339b, 46 },
    { 0x78db339d, 43 },
    { 0x2a7b339e, 33 },
    { 0x3d9b3399, 90 },
    { 0xa6cb33a0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb33a5, 78 },
    { 0x26fb33a6, 52 },
    { 0xe28b33a7, 90 },
    { 0xf62b33a6, 76 },
    { 0x2cdb33a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0b33ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b33b5, 43 },
    { 0, 0 },
    { 0xb96b33b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b33bd, 99 },
    { 0xc4bb33bd, 33 },
    { 0xc05b33be, 23 },
    { 0x2d2b33bf, 67 },
    { 0x140b33c1, 98 },
    { 0xbbdb33be, 61 },
    { 0, 0 },
    { 0xccdb33c4, 33 },
    { 0x44fb33c5, 53 },
    { 0xa46b33c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50b33c9, 43 },
    { 0x3dab33c9, 77 },
    { 0, 0 },
    { 0x95bb33cc, 46 },
    { 0, 0 },
    { 0xa65b33ce, 77 },
    { 0x182b33cf, 91 },
    { 0xabbb33cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b33d5, 53 },
    { 0xa2eb33d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb33d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aab33df, 33 },
    { 0x9cdb33df, 98 },
    { 0xabb33e1, 46 },
    { 0xb65b33e2, 90 },
    { 0xaa7b33df, 61 },
    { 0, 0 },
    { 0xe33b33e5, 61 },
    { 0x9f4b33e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfb33eb, 46 },
    { 0xed6b33eb, 23 },
    { 0x26db33ec, 98 },
    { 0xbc9b33ee, 33 },
    { 0, 0 },
    { 0x684b33f0, 55 },
    { 0xf65b33f1, 23 },
    { 0, 0 },
    { 0x212b33f3, 95 },
    { 0, 0 },
    { 0x632b33f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527b33fb, 90 },
    { 0x43cb33fc, 46 },
    { 0xb5eb33fb, 61 },
    { 0x5e2b33fd, 61 },
    { 0x8abb33ff, 66 },
    { 0xb7db33fd, 61 },
    { 0xaceb33fd, 61 },
    { 0x311b3402, 46 },
    { 0xbc7b33fb, 98 },
    { 0, 0 },
    { 0x1f0b3405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7b3409, 98 },
    { 0, 0 },
    { 0x27bb340b, 61 },
    { 0, 0 },
    { 0xfe8b340d, 95 },
    { 0xd28b340e, 33 },
    { 0x484b340e, 23 },
    { 0xbdfb340d, 61 },
    { 0, 0 },
    { 0xa8fb3412, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b3419, 67 },
    { 0, 0 },
    { 0x6ab341b, 77 },
    { 0, 0 },
    { 0x465b341d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bb3420, 43 },
    { 0xd7bb3421, 66 },
    { 0, 0 },
    { 0x1cb3423, 93 },
    { 0x574b3424, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb3427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4db3430, 66 },
    { 0x581b3431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x828b3434, 93 },
    { 0x209b3435, 43 },
    { 0, 0 },
    { 0x78b3437, 46 },
    { 0x212b3437, 23 },
    { 0x6cbb3437, 23 },
    { 0, 0 },
    { 0x428b343b, 75 },
    { 0x5e7b343b, 99 },
    { 0x9bfb343d, 61 },
    { 0xc01b343e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74b3441, 23 },
    { 0, 0 },
    { 0x9afb3443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3447, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eb344a, 82 },
    { 0xd24b344b, 78 },
    { 0xd06b344c, 93 },
    { 0xfc0b344c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab3451, 100 },
    { 0xed1b3452, 43 },
    { 0x9c4b3453, 98 },
    { 0xf37b3453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b3458, 33 },
    { 0, 0 },
    { 0x1d7b345a, 84 },
    { 0, 0 },
    { 0x7bdb345c, 90 },
    { 0x841b345c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bb3462, 23 },
    { 0, 0 },
    { 0x15db3464, 90 },
    { 0x897b3464, 90 },
    { 0, 0 },
    { 0x3eab3467, 78 },
    { 0xbf8b3468, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98b346d, 100 },
    { 0, 0 },
    { 0xbcb346f, 53 },
    { 0x251b3470, 23 },
    { 0x8cb3471, 90 },
    { 0xba6b3472, 85 },
    { 0xebab3473, 40 },
    { 0xa5bb3474, 90 },
    { 0xc23b3474, 55 },
    { 0, 0 },
    { 0x5e3b3477, 34 },
    { 0x48db3477, 90 },
    { 0x34cb3479, 76 },
    { 0x2abb3478, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceb3483, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b348b, 61 },
    { 0, 0 },
    { 0x742b348d, 76 },
    { 0x59db348e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96b3491, 81 },
    { 0x49db3492, 95 },
    { 0, 0 },
    { 0xbeab3494, 33 },
    { 0xfcab3495, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb3499, 66 },
    { 0xce1b3499, 55 },
    { 0xadbb349a, 46 },
    { 0x219b349b, 61 },
    { 0x2cdb349d, 77 },
    { 0xcdfb349e, 33 },
    { 0x6efb349f, 80 },
    { 0xf39b349f, 76 },
    { 0x539b34a1, 90 },
    { 0x168b34a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ab34a8, 23 },
    { 0x922b34a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96b34ac, 95 },
    { 0xd8b34ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845b34b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2b34bb, 43 },
    { 0, 0 },
    { 0x762b34bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1b34c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4b34c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b34cc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b34cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb34d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcb34d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58b34dc, 78 },
    { 0, 0 },
    { 0x494b34de, 33 },
    { 0x533b34df, 93 },
    { 0xa12b34de, 95 },
    { 0, 0 },
    { 0x6b7b34e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb34e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7b34e8, 52 },
    { 0xc5eb34e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b34ed, 55 },
    { 0, 0 },
    { 0x28cb34ef, 33 },
    { 0x2a8b34f0, 99 },
    { 0x6a9b34ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b34f5, 87 },
    { 0, 0 },
    { 0x611b34f7, 88 },
    { 0xd94b34f7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06b34fb, 90 },
    { 0, 0 },
    { 0x37b34fd, 33 },
    { 0x159b34fe, 61 },
    { 0xf1ab34fe, 87 },
    { 0, 0 },
    { 0xf57b3501, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48b3504, 34 },
    { 0x571b3504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beb3508, 61 },
    { 0xc5ab3508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x433b350c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494b3513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab3517, 90 },
    { 0x9fbb3518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b3520, 99 },
    { 0, 0 },
    { 0x805b3522, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5b352a, 77 },
    { 0x119b352a, 43 },
    { 0x58eb352c, 33 },
    { 0x8bdb352d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705b3535, 68 },
    { 0xc47b3535, 90 },
    { 0, 0 },
    { 0x5eeb3538, 43 },
    { 0x80b3539, 61 },
    { 0, 0 },
    { 0x4f8b353b, 61 },
    { 0x44eb353c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7b3540, 93 },
    { 0x2a7b3541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b3548, 46 },
    { 0xc84b3549, 55 },
    { 0x225b354a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3b354d, 90 },
    { 0x84db354e, 36 },
    { 0x276b354e, 95 },
    { 0xd60b354e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6b3554, 33 },
    { 0, 0 },
    { 0xbf9b3556, 46 },
    { 0x67bb3556, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3b355a, 90 },
    { 0xc1fb355a, 46 },
    { 0x2d9b355a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b3560, 33 },
    { 0x919b3561, 85 },
    { 0x4fbb3561, 75 },
    { 0x10fb3562, 33 },
    { 0x7d2b3564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2b356c, 61 },
    { 0x7afb356d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bb3575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3b3578, 90 },
    { 0, 0 },
    { 0x2a8b357a, 33 },
    { 0x2e2b357b, 90 },
    { 0x229b357b, 95 },
    { 0x329b357c, 100 },
    { 0x437b357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b3581, 75 },
    { 0, 0 },
    { 0x71cb3583, 23 },
    { 0x9cfb3584, 77 },
    { 0xbc5b3584, 90 },
    { 0x200b3584, 66 },
    { 0, 0 },
    { 0x96fb3588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b358b, 46 },
    { 0x72eb358b, 91 },
    { 0x7dcb358c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b3594, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fb3598, 34 },
    { 0, 0 },
    { 0xbddb359a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb359e, 67 },
    { 0x21db359f, 52 },
    { 0x98db35a0, 90 },
    { 0x50cb35a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bb35a5, 23 },
    { 0x46fb35a6, 46 },
    { 0x97ab35a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b35aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b35ae, 88 },
    { 0x611b35af, 23 },
    { 0, 0 },
    { 0x351b35b1, 78 },
    { 0, 0 },
    { 0x25cb35b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eb35b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32b35bd, 90 },
    { 0x1b1b35be, 91 },
    { 0xd23b35bf, 93 },
    { 0xba0b35be, 69 },
    { 0, 0 },
    { 0xe44b35c2, 23 },
    { 0xd79b35c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934b35cb, 90 },
    { 0, 0 },
    { 0x311b35cd, 55 },
    { 0, 0 },
    { 0x3f1b35cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bb35d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61b35d5, 76 },
    { 0xd8b35d5, 33 },
    { 0, 0 },
    { 0x43cb35d8, 98 },
    { 0, 0 },
    { 0x61b35da, 93 },
    { 0xc97b35db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b35de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db35e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65b35e4, 98 },
    { 0x85ab35e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb35ec, 17 },
    { 0, 0 },
    { 0x6fbb35ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236b35fd, 91 },
    { 0x5dcb35fd, 90 },
    { 0xdf8b35ff, 90 },
    { 0x7d6b35fe, 76 },
    { 0xe30b35ff, 98 },
    { 0x5ecb3601, 66 },
    { 0xdafb3603, 93 },
    { 0, 0 },
    { 0x56eb3605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b360a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7b360e, 46 },
    { 0, 0 },
    { 0x9bb3610, 61 },
    { 0x5e1b3611, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb3614, 100 },
    { 0xb1db3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738b3619, 33 },
    { 0, 0 },
    { 0x696b361b, 77 },
    { 0x41eb361b, 43 },
    { 0, 0 },
    { 0x2afb361e, 40 },
    { 0xd4bb361f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60b3622, 17 },
    { 0x3cbb3622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b3627, 23 },
    { 0x401b3628, 52 },
    { 0x616b3627, 98 },
    { 0xf84b362a, 4 },
    { 0x257b3628, 34 },
    { 0xcc6b3627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab3631, 52 },
    { 0, 0 },
    { 0x245b3633, 61 },
    { 0x114b3634, 33 },
    { 0x442b3635, 90 },
    { 0, 0 },
    { 0x74eb3637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90b3640, 33 },
    { 0x3eeb3641, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b3651, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3b365c, 76 },
    { 0xe2b365c, 22 },
    { 0x654b365e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b3661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feb3665, 33 },
    { 0x3e1b3666, 76 },
    { 0x697b3665, 55 },
    { 0, 0 },
    { 0x135b3669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b366f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fb3679, 91 },
    { 0x340b367a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x322b367d, 55 },
    { 0x2f7b367d, 95 },
    { 0, 0 },
    { 0xcb5b3680, 61 },
    { 0x824b3681, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb3684, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b3688, 81 },
    { 0x7dab3688, 34 },
    { 0x6a4b3689, 98 },
    { 0xff2b368b, 55 },
    { 0x6c1b368c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edb3690, 77 },
    { 0xc0eb3691, 93 },
    { 0x5b3692, 61 },
    { 0x779b3691, 23 },
    { 0xd49b3694, 90 },
    { 0x710b3695, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x739b3698, 33 },
    { 0x7dab3699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4b36a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb36a4, 66 },
    { 0, 0 },
    { 0xb40b36a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b36a9, 90 },
    { 0x790b36aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eab36af, 90 },
    { 0xe2bb36b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fb36b8, 90 },
    { 0x606b36b9, 34 },
    { 0xe4eb36ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664b36c0, 81 },
    { 0x25bb36c1, 91 },
    { 0x8a5b36c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb36ce, 84 },
    { 0x756b36cf, 90 },
    { 0, 0 },
    { 0xfe5b36d1, 90 },
    { 0x36db36d1, 95 },
    { 0x1d6b36d3, 95 },
    { 0xe69b36d4, 90 },
    { 0x9edb36d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b36de, 77 },
    { 0x2cdb36df, 55 },
    { 0x26eb36e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b36e5, 33 },
    { 0, 0 },
    { 0x61eb36e7, 90 },
    { 0, 0 },
    { 0x166b36e9, 83 },
    { 0x3f1b36e9, 55 },
    { 0, 0 },
    { 0xbebb36ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cb36f0, 93 },
    { 0x83fb36f1, 61 },
    { 0x87ab36f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12b36fa, 93 },
    { 0, 0 },
    { 0x927b36fc, 34 },
    { 0x638b36fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bb3700, 39 },
    { 0x9a1b3701, 99 },
    { 0, 0 },
    { 0x2f1b3703, 55 },
    { 0, 0 },
    { 0x4db3705, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb3708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b370f, 84 },
    { 0, 0 },
    { 0xb91b3711, 68 },
    { 0xf7db3712, 75 },
    { 0xb68b3711, 98 },
    { 0xbaab3713, 90 },
    { 0x751b3714, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b3718, 85 },
    { 0xa0b3719, 93 },
    { 0, 0 },
    { 0x541b371b, 51 },
    { 0x3a3b371c, 55 },
    { 0xa88b371c, 76 },
    { 0xfcb371b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b3723, 61 },
    { 0, 0 },
    { 0x1deb3725, 55 },
    { 0x6b3b3725, 33 },
    { 0xf2fb3727, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bb372d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db3731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b3737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eb373a, 95 },
    { 0x778b373b, 23 },
    { 0x233b373c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623b3740, 19 },
    { 0xf39b3741, 95 },
    { 0x3d0b3742, 95 },
    { 0x411b3743, 77 },
    { 0x6afb3742, 52 },
    { 0xa68b3745, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b374c, 90 },
    { 0xdfcb374c, 23 },
    { 0, 0 },
    { 0x1f1b374f, 33 },
    { 0xbf4b3750, 61 },
    { 0, 0 },
    { 0x962b3752, 61 },
    { 0xaa3b3753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3b3756, 34 },
    { 0xb7b3757, 23 },
    { 0, 0 },
    { 0x549b3759, 76 },
    { 0xccdb375a, 33 },
    { 0xddbb375b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1b3763, 87 },
    { 0xeafb3763, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb3768, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ab3773, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b377a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b377d, 81 },
    { 0x17cb377d, 98 },
    { 0, 0 },
    { 0xcc0b3780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb3784, 67 },
    { 0x89db3785, 43 },
    { 0x295b3784, 61 },
    { 0x16fb3787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afb3790, 33 },
    { 0x452b3791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b3799, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8b37a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61eb37aa, 98 },
    { 0xc2bb37aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b37ae, 55 },
    { 0, 0 },
    { 0x97db37b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6b37b6, 98 },
    { 0, 0 },
    { 0xa4db37b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b37bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3b37c2, 85 },
    { 0x70b37c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacab37c7, 90 },
    { 0xf20b37c8, 40 },
    { 0x70ab37c9, 95 },
    { 0, 0 },
    { 0x3ecb37cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bb37d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb37d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1b37df, 95 },
    { 0x7c3b37df, 83 },
    { 0x76eb37e1, 92 },
    { 0, 0 },
    { 0x656b37e3, 93 },
    { 0x30eb37e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb37e9, 90 },
    { 0xf6fb37e9, 81 },
    { 0, 0 },
    { 0xcd2b37ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4b37f0, 23 },
    { 0, 0 },
    { 0xdbab37f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beb37f7, 33 },
    { 0x20b37f7, 55 },
    { 0xa8bb37f9, 33 },
    { 0, 0 },
    { 0x95b37fb, 34 },
    { 0x46cb37fc, 76 },
    { 0, 0 },
    { 0x41eb37fe, 90 },
    { 0x12bb37fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebb3802, 65 },
    { 0x9cdb3803, 77 },
    { 0, 0 },
    { 0x43eb3805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b3808, 91 },
    { 0, 0 },
    { 0x6b3b380a, 34 },
    { 0x7fbb380b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2b380e, 49 },
    { 0x2f8b380f, 91 },
    { 0, 0 },
    { 0x9b0b3811, 76 },
    { 0x9deb3812, 95 },
    { 0x94fb3813, 76 },
    { 0xa66b3814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fb3817, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0b381c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b381f, 95 },
    { 0, 0 },
    { 0x840b3821, 67 },
    { 0, 0 },
    { 0x748b3823, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b3826, 90 },
    { 0x513b3827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x924b382a, 23 },
    { 0x47eb382b, 33 },
    { 0xe4fb382c, 90 },
    { 0xf22b382b, 43 },
    { 0, 0 },
    { 0x563b382f, 98 },
    { 0x81bb3830, 90 },
    { 0xbe4b3831, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b3836, 73 },
    { 0xbe0b3837, 33 },
    { 0, 0 },
    { 0x4e3b3839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b383e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b3843, 100 },
    { 0x667b3844, 61 },
    { 0, 0 },
    { 0xf15b3846, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bb3849, 61 },
    { 0, 0 },
    { 0x9dcb384b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eb3851, 61 },
    { 0x479b3852, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b3855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb385a, 98 },
    { 0xe8eb385a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15b385e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2b3863, 90 },
    { 0xda2b3864, 23 },
    { 0x779b3864, 61 },
    { 0x5eab3865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb386e, 52 },
    { 0xffbb386f, 61 },
    { 0, 0 },
    { 0x257b3871, 76 },
    { 0x6dfb3872, 55 },
    { 0x761b3873, 90 },
    { 0xfdb3873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0b3877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cb387a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b387e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0b3883, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b388a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429b388e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39db3893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18db3896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x714b3899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944b38a2, 23 },
    { 0x13db38a3, 95 },
    { 0x423b38a4, 61 },
    { 0xcfdb38a5, 90 },
    { 0xd4cb38a6, 55 },
    { 0, 0 },
    { 0x3adb38a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ab38b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b38b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b38b8, 55 },
    { 0xb77b38b8, 87 },
    { 0xfbab38ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cb38bf, 17 },
    { 0xe77b38c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7b38c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb38c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b38cb, 55 },
    { 0x994b38cc, 95 },
    { 0, 0 },
    { 0x2cb38ce, 33 },
    { 0, 0 },
    { 0xec0b38d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b38d4, 96 },
    { 0x652b38d5, 95 },
    { 0xb99b38d6, 55 },
    { 0x4e6b38d7, 99 },
    { 0x9fb38d7, 90 },
    { 0x365b38d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b38dc, 61 },
    { 0, 0 },
    { 0xf97b38de, 56 },
    { 0x695b38df, 43 },
    { 0, 0 },
    { 0x6efb38e1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30b38e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x256b38e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159b38ec, 23 },
    { 0xf86b38ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698b38f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb38f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x971b38fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b38fe, 95 },
    { 0xb5eb38ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b3904, 93 },
    { 0x82ab3904, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b3909, 78 },
    { 0, 0 },
    { 0x257b390b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b390e, 84 },
    { 0x48db390f, 61 },
    { 0x4c9b390e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b3913, 46 },
    { 0, 0 },
    { 0xe8b3915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb391d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06b3926, 55 },
    { 0x88fb3927, 96 },
    { 0, 0 },
    { 0xe77b3929, 98 },
    { 0xe73b3929, 34 },
    { 0x4cb392b, 55 },
    { 0x12cb392c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ab392f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9b3932, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b3937, 90 },
    { 0x9c9b3938, 93 },
    { 0x8d8b3939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x183b393c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951b3943, 95 },
    { 0x544b3943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b3947, 33 },
    { 0, 0 },
    { 0xe29b3949, 76 },
    { 0, 0 },
    { 0x421b394b, 23 },
    { 0, 0 },
    { 0x2e6b394d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02b3950, 90 },
    { 0x3c1b3951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfb3959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55b395c, 93 },
    { 0xec8b395d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0b3961, 33 },
    { 0xbcb3961, 85 },
    { 0, 0 },
    { 0x7efb3964, 98 },
    { 0, 0 },
    { 0xf92b3966, 33 },
    { 0x28db3966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefb396b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb396e, 61 },
    { 0x85eb396e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84b3977, 61 },
    { 0xb36b3977, 17 },
    { 0, 0 },
    { 0x1f4b397a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4b3981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db3985, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab3988, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db398b, 95 },
    { 0, 0 },
    { 0x7f9b398d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb3991, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b3997, 33 },
    { 0x706b3998, 76 },
    { 0xdd2b3998, 55 },
    { 0, 0 },
    { 0x269b399b, 23 },
    { 0, 0 },
    { 0x6feb399d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cb39a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb39a5, 68 },
    { 0, 0 },
    { 0x3a0b39a7, 100 },
    { 0x7c0b39a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ab39ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96b39b2, 33 },
    { 0, 0 },
    { 0x420b39b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4b39b7, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x285b39ba, 33 },
    { 0x52cb39ba, 55 },
    { 0xee3b39ba, 92 },
    { 0, 0 },
    { 0xa10b39be, 23 },
    { 0xbe1b39be, 90 },
    { 0x3ffb39bf, 23 },
    { 0xfd7b39c1, 23 },
    { 0x1f1b39c2, 43 },
    { 0xc14b39be, 55 },
    { 0xb90b39c2, 33 },
    { 0, 0 },
    { 0x916b39c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb39ca, 98 },
    { 0, 0 },
    { 0x11db39cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b39cf, 95 },
    { 0xff8b39d0, 95 },
    { 0x192b39d1, 95 },
    { 0x101b39d2, 88 },
    { 0xbbfb39d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bab39d8, 98 },
    { 0, 0 },
    { 0x32db39da, 90 },
    { 0xd4cb39db, 23 },
    { 0x607b39dc, 90 },
    { 0x433b39dd, 91 },
    { 0x9e9b39dd, 52 },
    { 0x235b39df, 43 },
    { 0xd2eb39da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b39ed, 38 },
    { 0, 0 },
    { 0x900b39ef, 90 },
    { 0x8e9b39ef, 90 },
    { 0xe6cb39f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb39f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b3a03, 33 },
    { 0, 0 },
    { 0x885b3a05, 76 },
    { 0x8cfb3a05, 61 },
    { 0, 0 },
    { 0xea3b3a08, 33 },
    { 0, 0 },
    { 0x64fb3a0a, 90 },
    { 0xf4fb3a0b, 85 },
    { 0xa49b3a0a, 90 },
    { 0x87cb3a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b3a10, 90 },
    { 0, 0 },
    { 0xd5eb3a12, 90 },
    { 0, 0 },
    { 0x6d0b3a14, 43 },
    { 0xb12b3a15, 69 },
    { 0x6a2b3a16, 46 },
    { 0x10b3a17, 90 },
    { 0xe34b3a14, 34 },
    { 0, 0 },
    { 0x53db3a1a, 98 },
    { 0xa57b3a1b, 52 },
    { 0xb72b3a1b, 90 },
    { 0x4d8b3a1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ab3a23, 90 },
    { 0x4bdb3a24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88b3a27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb3a2c, 76 },
    { 0x84fb3a2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b3a3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bb3a4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b3a50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53b3a53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b3a59, 46 },
    { 0xd81b3a59, 33 },
    { 0, 0 },
    { 0xce0b3a5c, 99 },
    { 0x4aab3a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203b3a63, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70b3a6b, 90 },
    { 0x959b3a6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239b3a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab3a74, 76 },
    { 0x5f4b3a75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b3a7d, 76 },
    { 0, 0 },
    { 0xa8ab3a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b3a85, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b3a88, 43 },
    { 0xe5db3a88, 61 },
    { 0, 0 },
    { 0x53db3a8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2db3a8e, 68 },
    { 0x766b3a8f, 23 },
    { 0x96ab3a90, 69 },
    { 0, 0 },
    { 0x978b3a92, 33 },
    { 0x9feb3a93, 90 },
    { 0x762b3a94, 33 },
    { 0x839b3a94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b3a98, 40 },
    { 0xe4eb3a99, 98 },
    { 0x263b3a9a, 90 },
    { 0xdfbb3a9a, 98 },
    { 0x5adb3a9c, 33 },
    { 0, 0 },
    { 0xe4fb3a9e, 83 },
    { 0xd05b3a9f, 52 },
    { 0xafab3aa0, 76 },
    { 0x2a9b3aa1, 33 },
    { 0x138b3aa2, 93 },
    { 0x746b3aa0, 61 },
    { 0x420b3aa4, 90 },
    { 0x485b3aa4, 33 },
    { 0xea5b3aa6, 33 },
    { 0x5a3b3aa4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cb3aae, 98 },
    { 0x266b3aaf, 76 },
    { 0, 0 },
    { 0xe6ab3ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab3ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96b3abc, 90 },
    { 0x443b3abd, 61 },
    { 0x2b8b3abd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bb3ac2, 61 },
    { 0xc20b3ac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b3ac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b3acb, 85 },
    { 0xa1fb3acc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cb3ad3, 34 },
    { 0xc7eb3ad4, 33 },
    { 0xc82b3ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eb3ad8, 23 },
    { 0x491b3ad9, 23 },
    { 0x486b3ad8, 90 },
    { 0, 0 },
    { 0x235b3adc, 90 },
    { 0xfb3adc, 36 },
    { 0, 0 },
    { 0xe25b3adf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeeb3ae5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb3ae8, 78 },
    { 0, 0 },
    { 0x5cab3aea, 90 },
    { 0xd4b3aeb, 55 },
    { 0x41db3aec, 33 },
    { 0xe34b3aec, 100 },
    { 0xaacb3aee, 95 },
    { 0x6a7b3aef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb3af3, 98 },
    { 0xee7b3af4, 95 },
    { 0x986b3af5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb3afd, 46 },
    { 0x2b8b3afe, 98 },
    { 0x619b3afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337b3b03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284b3b08, 76 },
    { 0x39bb3b08, 33 },
    { 0xb7bb3b0a, 91 },
    { 0xf2cb3b0a, 93 },
    { 0x7bdb3b0c, 76 },
    { 0x680b3b0a, 85 },
    { 0x8c0b3b0e, 61 },
    { 0x7e8b3b0e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8b3b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b3b17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b3b1c, 98 },
    { 0, 0 },
    { 0xa15b3b1e, 33 },
    { 0, 0 },
    { 0xb4ab3b20, 66 },
    { 0xf1ab3b21, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b3b29, 76 },
    { 0, 0 },
    { 0x55bb3b2b, 66 },
    { 0, 0 },
    { 0xb0cb3b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242b3b31, 52 },
    { 0xa50b3b32, 90 },
    { 0, 0 },
    { 0x8edb3b34, 95 },
    { 0xb8db3b34, 91 },
    { 0x7beb3b35, 23 },
    { 0xce5b3b37, 17 },
    { 0xb0ab3b38, 98 },
    { 0x8f3b3b39, 40 },
    { 0xb8cb3b3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33eb3b3e, 78 },
    { 0, 0 },
    { 0x730b3b40, 61 },
    { 0x86db3b41, 23 },
    { 0xf35b3b40, 55 },
    { 0x50b3b43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467b3b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b3b4d, 33 },
    { 0, 0 },
    { 0x65ab3b4f, 95 },
    { 0, 0 },
    { 0x8dfb3b51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb3b5a, 34 },
    { 0x483b3b5b, 90 },
    { 0xe3fb3b5c, 84 },
    { 0x53cb3b5d, 53 },
    { 0x4b0b3b5e, 33 },
    { 0, 0 },
    { 0xd17b3b60, 61 },
    { 0x2fbb3b61, 23 },
    { 0x274b3b62, 84 },
    { 0xef6b3b61, 43 },
    { 0xcaeb3b64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b3b68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb3b73, 90 },
    { 0xf35b3b74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85b3b78, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b3b7b, 36 },
    { 0x564b3b7c, 95 },
    { 0xf61b3b7d, 46 },
    { 0, 0 },
    { 0xb71b3b7f, 88 },
    { 0, 0 },
    { 0x81bb3b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3b8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db3b8e, 76 },
    { 0x11fb3b8e, 33 },
    { 0x9b8b3b8e, 90 },
    { 0xffdb3b91, 85 },
    { 0x95ab3b92, 91 },
    { 0, 0 },
    { 0xac6b3b94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb3b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5b3b9d, 101 },
    { 0x7d8b3b9e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bb3ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab3ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab3bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb3baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03b3bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6b3bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465b3bc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b3bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeb3bd1, 95 },
    { 0xcaeb3bd2, 95 },
    { 0x23db3bd3, 33 },
    { 0x903b3bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b3bd8, 90 },
    { 0, 0 },
    { 0xb03b3bda, 33 },
    { 0x976b3bdb, 52 },
    { 0xeebb3bdc, 100 },
    { 0x576b3bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb3be2, 23 },
    { 0x9e0b3be3, 68 },
    { 0x667b3be4, 98 },
    { 0x371b3be5, 100 },
    { 0xaa9b3be6, 55 },
    { 0xe4fb3be7, 77 },
    { 0x31cb3be6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ab3beb, 61 },
    { 0, 0 },
    { 0x23eb3bed, 43 },
    { 0, 0 },
    { 0x4e5b3bef, 95 },
    { 0x194b3bf0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab3bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4b3bf9, 33 },
    { 0x774b3bfa, 90 },
    { 0, 0 },
    { 0x724b3bfc, 33 },
    { 0, 0 },
    { 0x164b3bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcb3c03, 95 },
    { 0x74db3c04, 90 },
    { 0x5bcb3c04, 23 },
    { 0, 0 },
    { 0xc2cb3c07, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb3c0b, 90 },
    { 0x80db3c0c, 90 },
    { 0x48db3c0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b3c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32b3c13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb3c16, 33 },
    { 0x23db3c17, 67 },
    { 0x550b3c16, 95 },
    { 0x82fb3c17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadb3c1c, 95 },
    { 0x2bab3c1d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875b3c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eb3c2c, 85 },
    { 0, 0 },
    { 0xa26b3c2e, 78 },
    { 0xa68b3c2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb3c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9b3c36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b3c3a, 52 },
    { 0, 0 },
    { 0xdeb3c3c, 90 },
    { 0, 0 },
    { 0x4eeb3c3e, 34 },
    { 0x9b9b3c3e, 33 },
    { 0x419b3c3f, 76 },
    { 0xb91b3c3f, 93 },
    { 0x67eb3c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db3c47, 98 },
    { 0, 0 },
    { 0xfddb3c49, 59 },
    { 0xbd7b3c4a, 65 },
    { 0, 0 },
    { 0xf8fb3c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9b3c53, 76 },
    { 0, 0 },
    { 0x543b3c55, 61 },
    { 0, 0 },
    { 0x2e4b3c57, 90 },
    { 0x666b3c58, 95 },
    { 0xd62b3c59, 33 },
    { 0x72fb3c5a, 52 },
    { 0x1dfb3c5a, 88 },
    { 0x6e4b3c5a, 61 },
    { 0x4b7b3c5d, 61 },
    { 0x541b3c5a, 98 },
    { 0x705b3c5f, 90 },
    { 0x66cb3c57, 76 },
    { 0x87bb3c61, 36 },
    { 0x391b3c62, 95 },
    { 0x34db3c61, 76 },
    { 0xe68b3c59, 98 },
    { 0x253b3c61, 23 },
    { 0x6deb3c66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39b3c6f, 46 },
    { 0x7a3b3c70, 33 },
    { 0x16fb3c71, 67 },
    { 0x7cfb3c71, 33 },
    { 0xb68b3c73, 55 },
    { 0x5c8b3c71, 37 },
    { 0x41ab3c75, 90 },
    { 0xc8fb3c76, 90 },
    { 0xba6b3c76, 43 },
    { 0, 0 },
    { 0xca5b3c79, 24 },
    { 0, 0 },
    { 0xdddb3c7b, 95 },
    { 0, 0 },
    { 0x8afb3c7d, 61 },
    { 0, 0 },
    { 0x5d4b3c7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bb3c82, 23 },
    { 0, 0 },
    { 0xaa2b3c84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedb3c8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffb3c92, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x856b3c95, 67 },
    { 0x4bcb3c96, 40 },
    { 0, 0 },
    { 0xc23b3c98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73b3c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4b3ca6, 23 },
    { 0, 0 },
    { 0x5f0b3ca8, 66 },
    { 0x753b3ca9, 93 },
    { 0x8deb3caa, 95 },
    { 0x2ab3cab, 100 },
    { 0x8a4b3cab, 76 },
    { 0x8bab3cac, 34 },
    { 0x37fb3cae, 46 },
    { 0xee9b3caf, 99 },
    { 0xc62b3cb0, 68 },
    { 0x2f3b3ca9, 55 },
    { 0, 0 },
    { 0xcf9b3cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472b3cbf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4b3cc3, 76 },
    { 0xe0bb3cc3, 66 },
    { 0x727b3cc5, 90 },
    { 0, 0 },
    { 0xd88b3cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b3cca, 76 },
    { 0x7f9b3ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb3cd8, 88 },
    { 0xf26b3cd8, 90 },
    { 0, 0 },
    { 0xd65b3cdb, 61 },
    { 0x972b3cdb, 77 },
    { 0, 0 },
    { 0xbb5b3cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcb3ce2, 78 },
    { 0xb1eb3ce2, 34 },
    { 0, 0 },
    { 0xba4b3ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cb3ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87b3ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb3cf3, 52 },
    { 0x32eb3cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b3d00, 95 },
    { 0, 0 },
    { 0x89eb3d02, 23 },
    { 0x581b3d02, 61 },
    { 0x9e2b3d04, 55 },
    { 0x7deb3d05, 33 },
    { 0xfbb3d06, 33 },
    { 0xef9b3d06, 90 },
    { 0x79b3d06, 55 },
    { 0xb28b3d02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db3d11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b3d1f, 33 },
    { 0x8a5b3d20, 78 },
    { 0, 0 },
    { 0x30fb3d22, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b3d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb3d28, 52 },
    { 0x660b3d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b3d2d, 90 },
    { 0, 0 },
    { 0xb3bb3d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8b3d34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2b3d37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fb3d3d, 33 },
    { 0, 0 },
    { 0x69eb3d3f, 87 },
    { 0xa60b3d40, 95 },
    { 0, 0 },
    { 0x1ebb3d42, 81 },
    { 0xba7b3d42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6b3d46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b3d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5b3d50, 90 },
    { 0x972b3d51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6eb3d57, 90 },
    { 0xadbb3d58, 90 },
    { 0xa44b3d59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb3d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3d5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4b3d66, 91 },
    { 0, 0 },
    { 0x550b3d68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b3d6c, 98 },
    { 0xb87b3d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b3d70, 95 },
    { 0, 0 },
    { 0x864b3d72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5b3d77, 95 },
    { 0x7ddb3d78, 90 },
    { 0, 0 },
    { 0x978b3d7a, 33 },
    { 0xc4cb3d7b, 33 },
    { 0x915b3d7a, 90 },
    { 0, 0 },
    { 0x4b6b3d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419b3d87, 98 },
    { 0xe13b3d87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5b3d8f, 98 },
    { 0x64fb3d90, 78 },
    { 0, 0 },
    { 0x344b3d92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b3d98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ab3d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b3d9e, 33 },
    { 0x6a3b3d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15b3da4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bb3dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfb3db6, 90 },
    { 0xcbcb3db6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b3dc0, 90 },
    { 0x43cb3dc1, 75 },
    { 0xe02b3dc2, 55 },
    { 0x760b3dc3, 33 },
    { 0, 0 },
    { 0xd00b3dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x655b3dc8, 78 },
    { 0x890b3dc9, 23 },
    { 0xa94b3dca, 23 },
    { 0x4bab3dcb, 90 },
    { 0x66db3dc8, 78 },
    { 0x623b3dcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b3dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323b3dd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddb3dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4b3ddc, 55 },
    { 0, 0 },
    { 0xa61b3dde, 65 },
    { 0xc1ab3ddf, 46 },
    { 0xc32b3de0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1b3de3, 90 },
    { 0x95cb3de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ab3de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b3def, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ab3df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb3df8, 66 },
    { 0, 0 },
    { 0xc2eb3dfa, 98 },
    { 0xf53b3dfb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208b3e07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdb3e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2b3e24, 65 },
    { 0x81b3e24, 55 },
    { 0xf67b3e26, 23 },
    { 0, 0 },
    { 0x61b3e28, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b3e2b, 23 },
    { 0x216b3e2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6b3e32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b3e3d, 90 },
    { 0x6ffb3e3d, 84 },
    { 0x686b3e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84db3e42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b3e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccb3e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b3e52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0b3e5e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39b3e64, 100 },
    { 0x148b3e65, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfb3e6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb3e6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb3e75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab3e78, 90 },
    { 0x2cbb3e79, 76 },
    { 0x7cab3e7a, 98 },
    { 0, 0 },
    { 0x9dfb3e7c, 66 },
    { 0, 0 },
    { 0x86ab3e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3e89, 23 },
    { 0xbfcb3e8a, 61 },
    { 0xec7b3e8b, 66 },
    { 0xab3e8a, 76 },
    { 0x1a1b3e8a, 33 },
    { 0xb84b3e8a, 33 },
    { 0xae7b3e8f, 23 },
    { 0xaabb3e8d, 23 },
    { 0xd63b3e91, 90 },
    { 0x84db3e91, 93 },
    { 0x429b3e93, 98 },
    { 0xa93b3e8f, 61 },
    { 0, 0 },
    { 0xb86b3e96, 95 },
    { 0x4b3b3e96, 61 },
    { 0x637b3e98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9b3e9b, 34 },
    { 0xc85b3e9b, 61 },
    { 0x7f3b3e9d, 61 },
    { 0x12bb3e9b, 55 },
    { 0, 0 },
    { 0x251b3ea0, 23 },
    { 0xb11b3ea1, 76 },
    { 0xc38b3ea1, 93 },
    { 0x6f5b3ea2, 33 },
    { 0xe98b3ea1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722b3eaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b3eaf, 76 },
    { 0xf4fb3eb0, 61 },
    { 0xe69b3eb1, 33 },
    { 0xc94b3eb2, 90 },
    { 0xc59b3eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab3ebb, 33 },
    { 0xc8eb3ebc, 81 },
    { 0xb34b3ebd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb3ec4, 93 },
    { 0x9a6b3ec5, 98 },
    { 0xe9ab3ec5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb3ecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b3ed1, 61 },
    { 0xa6b3ed2, 61 },
    { 0x747b3ed2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6b3ed6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x963b3ed9, 76 },
    { 0xad4b3eda, 55 },
    { 0x490b3eda, 95 },
    { 0, 0 },
    { 0xab8b3edd, 95 },
    { 0x841b3ede, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb3ee1, 90 },
    { 0x1f6b3ee1, 90 },
    { 0, 0 },
    { 0x21eb3ee4, 98 },
    { 0, 0 },
    { 0x981b3ee6, 84 },
    { 0x61eb3ee6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb3eef, 78 },
    { 0, 0 },
    { 0x314b3ef1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8b3ef5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b3ef9, 34 },
    { 0, 0 },
    { 0x2ccb3efb, 33 },
    { 0x4c5b3efc, 22 },
    { 0, 0 },
    { 0x65ab3efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7b3f06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b3f0d, 98 },
    { 0x64db3f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb3f11, 33 },
    { 0x222b3f12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bb3f18, 98 },
    { 0x92b3f18, 91 },
    { 0xb79b3f1a, 66 },
    { 0x3db3f19, 23 },
    { 0xf8b3f1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b3f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b3f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b3f2d, 43 },
    { 0x21cb3f2e, 33 },
    { 0x958b3f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x499b3f32, 43 },
    { 0x66cb3f33, 43 },
    { 0, 0 },
    { 0xe5ab3f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7b3f38, 23 },
    { 0, 0 },
    { 0x99cb3f3a, 90 },
    { 0xab5b3f3a, 84 },
    { 0, 0 },
    { 0xe3b3f3d, 91 },
    { 0x473b3f3e, 43 },
    { 0xf5cb3f3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41b3f44, 33 },
    { 0x899b3f45, 101 },
    { 0, 0 },
    { 0xc79b3f47, 33 },
    { 0x2f2b3f48, 90 },
    { 0x7a7b3f49, 33 },
    { 0, 0 },
    { 0x648b3f4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b3f51, 69 },
    { 0x674b3f51, 33 },
    { 0x914b3f53, 48 },
    { 0x3c7b3f53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x338b3f57, 55 },
    { 0x232b3f58, 98 },
    { 0, 0 },
    { 0x757b3f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabb3f5d, 33 },
    { 0, 0 },
    { 0x87bb3f5f, 67 },
    { 0x37bb3f60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eb3f63, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65b3f66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb3f6a, 98 },
    { 0, 0 },
    { 0xe9eb3f6c, 98 },
    { 0x4e8b3f6d, 66 },
    { 0x113b3f6c, 33 },
    { 0x3db3f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db3f76, 76 },
    { 0, 0 },
    { 0x70fb3f78, 84 },
    { 0x470b3f79, 43 },
    { 0, 0 },
    { 0x332b3f7b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffb3f7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26b3f82, 100 },
    { 0xcf5b3f82, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b3f86, 76 },
    { 0x415b3f87, 76 },
    { 0x7abb3f88, 46 },
    { 0x826b3f89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b3f8c, 61 },
    { 0, 0 },
    { 0x412b3f8e, 76 },
    { 0x2aab3f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceb3f92, 76 },
    { 0x2bfb3f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83b3f98, 95 },
    { 0, 0 },
    { 0xcceb3f9a, 90 },
    { 0xf96b3f9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52b3fa1, 23 },
    { 0, 0 },
    { 0x478b3fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbb3fa7, 98 },
    { 0xec1b3fa8, 98 },
    { 0, 0 },
    { 0x335b3faa, 95 },
    { 0x3b3fab, 99 },
    { 0, 0 },
    { 0x2cb3fad, 46 },
    { 0x16eb3fad, 95 },
    { 0xe0fb3fae, 84 },
    { 0, 0 },
    { 0xbcdb3fb1, 17 },
    { 0, 0 },
    { 0xd3db3fb3, 33 },
    { 0, 0 },
    { 0xd98b3fb5, 69 },
    { 0x999b3fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28b3fbb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab3fc0, 95 },
    { 0, 0 },
    { 0xdb5b3fc2, 61 },
    { 0xc94b3fc3, 76 },
    { 0x2eb3fc2, 55 },
    { 0xbc8b3fc5, 43 },
    { 0xbcbb3fc6, 61 },
    { 0xfbcb3fc7, 23 },
    { 0xcd4b3fc7, 43 },
    { 0x926b3fc9, 33 },
    { 0x5f2b3fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8b3fcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b3fd3, 91 },
    { 0x699b3fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50b3fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb3fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dab3fe4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3fe7, 95 },
    { 0x73eb3fe7, 100 },
    { 0, 0 },
    { 0xd79b3fea, 23 },
    { 0xcd9b3feb, 76 },
    { 0xb93b3fea, 84 },
    { 0, 0 },
    { 0xdd1b3fee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b3ff3, 46 },
    { 0x7b4b3ff4, 34 },
    { 0x187b3ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x730b3ff8, 88 },
    { 0x4c7b3ff8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fb3ffd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b4000, 85 },
    { 0, 0 },
    { 0x73ab4002, 95 },
    { 0xf73b4003, 34 },
    { 0x42db4004, 87 },
    { 0, 0 },
    { 0xf99b4006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb4009, 95 },
    { 0x95bb400a, 43 },
    { 0x5ddb400b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffb4012, 99 },
    { 0, 0 },
    { 0xa30b4014, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b4019, 95 },
    { 0xa03b401a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x160b401d, 34 },
    { 0x838b401d, 34 },
    { 0, 0 },
    { 0xf0b4020, 93 },
    { 0x2beb4021, 90 },
    { 0, 0 },
    { 0x1f1b4023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b4026, 95 },
    { 0x175b4027, 93 },
    { 0, 0 },
    { 0x903b4029, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x945b402c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b402f, 23 },
    { 0x135b4030, 23 },
    { 0xbbb402f, 33 },
    { 0x487b4032, 23 },
    { 0xecab4033, 31 },
    { 0x7ddb4030, 90 },
    { 0xd8db4035, 90 },
    { 0x349b402f, 84 },
    { 0, 0 },
    { 0x233b4038, 33 },
    { 0xd31b4038, 29 },
    { 0xa1b403a, 23 },
    { 0, 0 },
    { 0x324b403c, 46 },
    { 0x877b403d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2b4041, 87 },
    { 0x4d6b4041, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2b4045, 91 },
    { 0x3c5b4046, 55 },
    { 0xe64b4047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b404b, 90 },
    { 0x5d5b404b, 75 },
    { 0xff6b404d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b4056, 98 },
    { 0, 0 },
    { 0x414b4058, 33 },
    { 0, 0 },
    { 0x10eb405a, 95 },
    { 0, 0 },
    { 0x8e7b405c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58b4061, 90 },
    { 0xebbb4061, 61 },
    { 0xee9b4063, 61 },
    { 0x650b4061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ceb4068, 78 },
    { 0, 0 },
    { 0xa14b406a, 90 },
    { 0, 0 },
    { 0x142b406c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466b4070, 93 },
    { 0x75ab4071, 90 },
    { 0xd75b4072, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b4078, 95 },
    { 0x386b4079, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b407d, 68 },
    { 0, 0 },
    { 0x28fb407f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb4082, 85 },
    { 0x8a1b4083, 61 },
    { 0x216b4082, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b408a, 55 },
    { 0x121b408b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x978b408e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eb4092, 33 },
    { 0, 0 },
    { 0xfb9b4094, 77 },
    { 0xabdb4095, 78 },
    { 0x12db4096, 83 },
    { 0x187b4095, 33 },
    { 0x261b4098, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb409d, 23 },
    { 0, 0 },
    { 0xe1fb409f, 94 },
    { 0, 0 },
    { 0x29cb40a1, 23 },
    { 0, 0 },
    { 0xca1b40a3, 88 },
    { 0xa75b40a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40ae, 61 },
    { 0xc35b40af, 33 },
    { 0xb50b40b0, 43 },
    { 0x94bb40b1, 90 },
    { 0x4bdb40b0, 98 },
    { 0xaddb40b1, 95 },
    { 0xe5cb40b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b40b9, 76 },
    { 0x343b40ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b40c2, 55 },
    { 0xe4cb40c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b40c7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802b40cd, 61 },
    { 0x1a8b40ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b40d8, 95 },
    { 0, 0 },
    { 0xc97b40da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17b40dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db40e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6b40e3, 98 },
    { 0x52fb40e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0b40e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ab40eb, 61 },
    { 0, 0 },
    { 0xe4b40ed, 95 },
    { 0, 0 },
    { 0x30fb40ef, 81 },
    { 0, 0 },
    { 0xddab40f1, 78 },
    { 0, 0 },
    { 0x724b40f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b40f9, 23 },
    { 0xf4ab40fa, 81 },
    { 0xb42b40fa, 33 },
    { 0xc2cb40fc, 88 },
    { 0x3f6b40fd, 33 },
    { 0x40ab40fd, 90 },
    { 0xbf1b40f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x347b4102, 33 },
    { 0xd03b4102, 90 },
    { 0xd7cb4102, 33 },
    { 0xefb4105, 99 },
    { 0, 0 },
    { 0x64b4107, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b410b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b4111, 90 },
    { 0, 0 },
    { 0x690b4113, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb4117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b411c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db4122, 76 },
    { 0x8b0b4123, 52 },
    { 0xde9b4124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b412b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ab4130, 90 },
    { 0xad2b4131, 69 },
    { 0xcd8b4132, 52 },
    { 0xcaab4133, 34 },
    { 0x6a1b4134, 88 },
    { 0xf5ab4134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb413e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x996b4141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84b4148, 99 },
    { 0x15db4149, 90 },
    { 0xe22b4148, 93 },
    { 0, 0 },
    { 0x26db414c, 90 },
    { 0, 0 },
    { 0xe19b414e, 33 },
    { 0, 0 },
    { 0xe1fb4150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b4155, 87 },
    { 0xcccb4156, 76 },
    { 0, 0 },
    { 0x9d2b4158, 17 },
    { 0x6db4159, 88 },
    { 0x721b4159, 34 },
    { 0x74cb415b, 81 },
    { 0, 0 },
    { 0xae3b415d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cb4161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fb4164, 76 },
    { 0xb97b4165, 90 },
    { 0, 0 },
    { 0x8aab4167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb416d, 33 },
    { 0xbafb416d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90b4173, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934b4179, 33 },
    { 0, 0 },
    { 0xb8b417b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9b4182, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667b4188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b418b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfb418f, 87 },
    { 0, 0 },
    { 0xe77b4191, 34 },
    { 0xa5bb4192, 76 },
    { 0x404b4192, 33 },
    { 0xaabb4194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b419c, 98 },
    { 0xc2eb419d, 53 },
    { 0x2d5b419d, 84 },
    { 0xd2eb419e, 85 },
    { 0x24cb419f, 90 },
    { 0, 0 },
    { 0xb3ab41a2, 76 },
    { 0x5acb41a2, 90 },
    { 0x8f8b41a4, 76 },
    { 0x4f2b41a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89db41a8, 95 },
    { 0x612b41a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b41ae, 77 },
    { 0, 0 },
    { 0x5ccb41b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b41b9, 98 },
    { 0, 0 },
    { 0xdd8b41bb, 100 },
    { 0, 0 },
    { 0xf09b41bd, 91 },
    { 0, 0 },
    { 0x58cb41bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b41c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776b41cb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b41d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb41d7, 90 },
    { 0x83db41d7, 98 },
    { 0x57cb41d9, 34 },
    { 0, 0 },
    { 0x7c4b41db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b41df, 91 },
    { 0xab3b41df, 91 },
    { 0, 0 },
    { 0x77b41e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44b41f1, 33 },
    { 0, 0 },
    { 0x4d9b41f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56b41f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7b41fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b4204, 55 },
    { 0x8eab4205, 65 },
    { 0, 0 },
    { 0x612b4207, 66 },
    { 0xee8b4207, 23 },
    { 0xd8ab4208, 33 },
    { 0xad5b4209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b420d, 90 },
    { 0xad1b420e, 9 },
    { 0x8a9b420d, 55 },
    { 0xb2bb4210, 98 },
    { 0, 0 },
    { 0xd0ab4212, 43 },
    { 0x2a5b4212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1b421e, 78 },
    { 0xefcb421e, 76 },
    { 0x8bcb4220, 98 },
    { 0x242b4220, 98 },
    { 0, 0 },
    { 0x1afb4223, 33 },
    { 0x3a9b4224, 98 },
    { 0x46eb4223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b4228, 23 },
    { 0x62cb4228, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9b4230, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b4234, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4238, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420b423d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373b4242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14b4245, 98 },
    { 0x4b2b4246, 23 },
    { 0x20ab4246, 33 },
    { 0x259b4248, 98 },
    { 0xa42b4246, 23 },
    { 0x1d0b4246, 95 },
    { 0x66db424b, 55 },
    { 0xf45b424b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccb4255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x236b4258, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b425b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540b425f, 33 },
    { 0x3fdb425f, 23 },
    { 0xa64b4260, 34 },
    { 0, 0 },
    { 0xc10b4263, 90 },
    { 0x2bb4263, 34 },
    { 0, 0 },
    { 0xd6db4266, 19 },
    { 0xe76b4267, 33 },
    { 0x948b4267, 23 },
    { 0x39cb4267, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb4271, 99 },
    { 0, 0 },
    { 0xae7b4273, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2b4278, 61 },
    { 0xf2b4279, 33 },
    { 0, 0 },
    { 0xcc7b427b, 81 },
    { 0xbc8b427c, 55 },
    { 0x69cb427d, 100 },
    { 0xdfab427e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4b4282, 61 },
    { 0, 0 },
    { 0x1e7b4284, 98 },
    { 0x397b4285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1b428f, 23 },
    { 0xd87b4290, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ab4293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db4296, 52 },
    { 0, 0 },
    { 0x977b4298, 77 },
    { 0x21db4298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404b42a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddb42a4, 78 },
    { 0x554b42a5, 98 },
    { 0xe33b42a6, 23 },
    { 0x803b42a4, 38 },
    { 0, 0 },
    { 0x6c4b42a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb42b8, 33 },
    { 0, 0 },
    { 0xcb2b42ba, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edb42be, 93 },
    { 0x3edb42bf, 90 },
    { 0x95db42bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446b42c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b42c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb42cc, 93 },
    { 0, 0 },
    { 0x6d6b42ce, 43 },
    { 0, 0 },
    { 0x64cb42d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74b42d6, 81 },
    { 0xcaeb42d6, 76 },
    { 0x366b42d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb42e1, 77 },
    { 0x55ab42e1, 77 },
    { 0x86db42e3, 95 },
    { 0x15bb42e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b42e7, 80 },
    { 0, 0 },
    { 0x7fab42e9, 76 },
    { 0x504b42e9, 61 },
    { 0, 0 },
    { 0x565b42ec, 93 },
    { 0, 0 },
    { 0x89cb42ee, 95 },
    { 0x3cab42ef, 46 },
    { 0xe03b42f0, 90 },
    { 0x719b42f0, 80 },
    { 0x128b42f2, 87 },
    { 0xec0b42f2, 33 },
    { 0x656b42f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b42fa, 75 },
    { 0x3efb42fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afb4307, 87 },
    { 0x452b4307, 23 },
    { 0x364b4309, 53 },
    { 0xdf3b430a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279b430e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26b4311, 90 },
    { 0x516b4312, 33 },
    { 0x52ab4313, 90 },
    { 0xbc1b4314, 33 },
    { 0xa28b4313, 90 },
    { 0xcc4b4316, 33 },
    { 0xebb4315, 95 },
    { 0x1aab4318, 98 },
    { 0, 0 },
    { 0xf5eb431a, 33 },
    { 0xea8b431b, 61 },
    { 0xf70b431b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4b4320, 23 },
    { 0, 0 },
    { 0xbe7b4322, 61 },
    { 0x19cb4323, 98 },
    { 0, 0 },
    { 0x9f3b4325, 33 },
    { 0xb7ab4326, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b432e, 64 },
    { 0x1cbb432f, 66 },
    { 0x4edb432e, 90 },
    { 0xaf7b4331, 76 },
    { 0xf6fb4332, 23 },
    { 0x115b4333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ab4336, 90 },
    { 0x403b4336, 95 },
    { 0x6d3b4338, 61 },
    { 0x6ceb4339, 98 },
    { 0x13cb4338, 90 },
    { 0x7bcb433b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b433e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b4341, 52 },
    { 0, 0 },
    { 0xd69b4343, 33 },
    { 0, 0 },
    { 0x971b4345, 98 },
    { 0xf04b4346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235b434a, 23 },
    { 0xae9b434b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb434f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0b4353, 90 },
    { 0, 0 },
    { 0xfecb4355, 76 },
    { 0, 0 },
    { 0x19cb4357, 90 },
    { 0xc34b4357, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fb435d, 61 },
    { 0xe16b435d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb4361, 33 },
    { 0xe14b4362, 61 },
    { 0x2a5b4361, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76b4366, 23 },
    { 0x18ab4367, 23 },
    { 0xb1bb4368, 99 },
    { 0x634b4369, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0b436c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eb436f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b4373, 84 },
    { 0, 0 },
    { 0x8e5b4375, 43 },
    { 0, 0 },
    { 0xdffb4377, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b437b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb4382, 99 },
    { 0, 0 },
    { 0xd72b4384, 91 },
    { 0xec5b4385, 91 },
    { 0xce2b4385, 88 },
    { 0x4bfb4385, 23 },
    { 0xd3bb4388, 90 },
    { 0, 0 },
    { 0xa1bb438a, 61 },
    { 0, 0 },
    { 0xaab438c, 90 },
    { 0x2bcb438d, 65 },
    { 0x8fdb438e, 68 },
    { 0x4bfb438d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b4394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b4399, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ab439c, 81 },
    { 0x5b7b439d, 48 },
    { 0xebeb439e, 33 },
    { 0x773b439e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb43a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42db43aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b43af, 61 },
    { 0, 0 },
    { 0x736b43b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb43b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b43bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b43bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b43c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549b43c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b43cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fb43d3, 95 },
    { 0x9c3b43d4, 90 },
    { 0xf51b43d5, 65 },
    { 0x2bab43d6, 33 },
    { 0x61ab43d6, 98 },
    { 0x2feb43d8, 23 },
    { 0xdecb43d8, 17 },
    { 0x40bb43d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bb43e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63b43e4, 43 },
    { 0, 0 },
    { 0xe07b43e6, 43 },
    { 0x1e1b43e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b43ec, 88 },
    { 0xa1db43ed, 90 },
    { 0x885b43ec, 76 },
    { 0x1b4b43ef, 61 },
    { 0x12bb43ee, 90 },
    { 0xc41b43f1, 76 },
    { 0x1ffb43f2, 76 },
    { 0xe17b43f3, 98 },
    { 0xf1b43f3, 61 },
    { 0x4c0b43f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233b43fb, 91 },
    { 0, 0 },
    { 0x39cb43fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b4404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab440d, 43 },
    { 0xc55b440e, 88 },
    { 0xfbcb440f, 33 },
    { 0xfb9b440e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b4417, 66 },
    { 0x192b4418, 95 },
    { 0x490b4419, 78 },
    { 0x830b4419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1b441d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x246b4420, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb4424, 77 },
    { 0, 0 },
    { 0x75fb4426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab4429, 90 },
    { 0x9b0b442a, 88 },
    { 0x9f3b442a, 99 },
    { 0xa00b442a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbb4432, 95 },
    { 0x146b4432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefb4438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62db443c, 35 },
    { 0x102b443d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120b4443, 90 },
    { 0x808b4444, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b444e, 90 },
    { 0, 0 },
    { 0x525b4450, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5b4453, 84 },
    { 0x879b4454, 100 },
    { 0x847b4455, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682b445b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab445e, 90 },
    { 0xaf1b445f, 90 },
    { 0x107b4460, 61 },
    { 0xbffb4461, 90 },
    { 0xb6b4462, 95 },
    { 0x7b3b4460, 33 },
    { 0x20b4464, 68 },
    { 0x960b4465, 23 },
    { 0xfd7b4464, 33 },
    { 0x44fb4465, 61 },
    { 0x840b4468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdb446c, 85 },
    { 0x3f4b446d, 90 },
    { 0xbe6b446e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcb4471, 34 },
    { 0xdbcb4471, 33 },
    { 0x187b4473, 33 },
    { 0xa87b4473, 23 },
    { 0x15bb4473, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89b4478, 95 },
    { 0x7d9b4479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4b447f, 61 },
    { 0x3b2b447f, 93 },
    { 0x51b4481, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2b4484, 61 },
    { 0, 0 },
    { 0x184b4486, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146b448e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72db4491, 61 },
    { 0, 0 },
    { 0xfe3b4493, 98 },
    { 0x32db4493, 33 },
    { 0, 0 },
    { 0x7b2b4496, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529b449b, 61 },
    { 0, 0 },
    { 0xf86b449d, 55 },
    { 0x4ccb449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2b44a6, 81 },
    { 0, 0 },
    { 0xac1b44a8, 46 },
    { 0, 0 },
    { 0x7e1b44aa, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddb44ae, 90 },
    { 0xc53b44ae, 91 },
    { 0, 0 },
    { 0x7c3b44b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b44b6, 24 },
    { 0x7db44b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88b44ba, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x606b44bd, 48 },
    { 0, 0 },
    { 0x497b44bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cb44c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729b44c8, 46 },
    { 0, 0 },
    { 0xf2fb44ca, 33 },
    { 0xf9b44cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb44ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b44d3, 34 },
    { 0xc90b44d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5b44dd, 90 },
    { 0xa7db44dd, 55 },
    { 0x5b8b44df, 61 },
    { 0x65b44e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b44e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8b44eb, 96 },
    { 0x128b44ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcb44f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413b44fd, 33 },
    { 0xe75b44fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x540b4501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb450a, 34 },
    { 0xbe3b450a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b4510, 33 },
    { 0xb54b4511, 81 },
    { 0x73eb4512, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b4517, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdb451f, 34 },
    { 0x292b4520, 34 },
    { 0x926b4521, 76 },
    { 0x2a9b4520, 33 },
    { 0x56eb4523, 23 },
    { 0xf07b4524, 90 },
    { 0xd79b4525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7b452a, 90 },
    { 0xe19b452b, 55 },
    { 0, 0 },
    { 0xf05b452d, 33 },
    { 0xb46b452d, 90 },
    { 0x755b452f, 52 },
    { 0, 0 },
    { 0xc82b4531, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cb4536, 55 },
    { 0xe7cb4537, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aeb453b, 61 },
    { 0xfc6b453c, 46 },
    { 0xaf2b453d, 33 },
    { 0x15cb453d, 95 },
    { 0xad3b453f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb4542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb4547, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0eb454d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3b4550, 91 },
    { 0x6cfb4551, 98 },
    { 0x4c9b4552, 23 },
    { 0, 0 },
    { 0xe3cb4554, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b4558, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968b455f, 87 },
    { 0, 0 },
    { 0x295b4561, 66 },
    { 0x9c2b4561, 90 },
    { 0, 0 },
    { 0x9cbb4564, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b4568, 17 },
    { 0xfdeb4568, 83 },
    { 0x5dfb456a, 84 },
    { 0x790b456b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b456f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b4573, 98 },
    { 0xe64b4574, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24b4577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8b4583, 61 },
    { 0, 0 },
    { 0x568b4585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b4588, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb458f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b4592, 23 },
    { 0, 0 },
    { 0x505b4594, 76 },
    { 0xcebb4595, 55 },
    { 0xb20b4596, 93 },
    { 0x421b4597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cb459f, 55 },
    { 0x4d4b45a0, 46 },
    { 0xbcfb45a1, 23 },
    { 0xc8bb45a0, 67 },
    { 0x772b45a3, 88 },
    { 0x50b45a1, 95 },
    { 0x237b45a5, 33 },
    { 0xc3ab45a6, 99 },
    { 0xc64b45a7, 66 },
    { 0x67fb45a8, 98 },
    { 0x7b8b45a3, 95 },
    { 0xf77b45a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fb45af, 33 },
    { 0, 0 },
    { 0x639b45b1, 23 },
    { 0, 0 },
    { 0xc60b45b3, 12 },
    { 0, 0 },
    { 0xa2fb45b5, 52 },
    { 0xc95b45b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b45b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956b45c0, 46 },
    { 0xf64b45c1, 90 },
    { 0x13db45c1, 96 },
    { 0, 0 },
    { 0x1e1b45c4, 61 },
    { 0x305b45c5, 66 },
    { 0x562b45c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b45d3, 46 },
    { 0, 0 },
    { 0xa9db45d5, 43 },
    { 0xb7eb45d5, 61 },
    { 0xb59b45d7, 84 },
    { 0xc59b45d6, 43 },
    { 0, 0 },
    { 0x3cb45da, 46 },
    { 0, 0 },
    { 0x86fb45dc, 90 },
    { 0x82cb45dd, 33 },
    { 0, 0 },
    { 0x612b45df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962b45e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b45e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b45eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b45f0, 55 },
    { 0x807b45f1, 98 },
    { 0x150b45f1, 91 },
    { 0xaa9b45f3, 98 },
    { 0, 0 },
    { 0x91eb45f5, 90 },
    { 0, 0 },
    { 0xcafb45f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ab45fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb4603, 93 },
    { 0xe52b4603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13b4607, 93 },
    { 0xd11b4607, 93 },
    { 0xfa1b4607, 90 },
    { 0xbadb460a, 33 },
    { 0, 0 },
    { 0x4eab460c, 76 },
    { 0x79fb460d, 96 },
    { 0x973b460e, 81 },
    { 0, 0 },
    { 0xa3bb4610, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b4617, 56 },
    { 0x221b4618, 76 },
    { 0xcb6b4618, 77 },
    { 0x813b461a, 23 },
    { 0x652b461b, 61 },
    { 0x39fb461b, 61 },
    { 0xbc4b461b, 33 },
    { 0x681b461a, 34 },
    { 0, 0 },
    { 0x4d1b4620, 100 },
    { 0x6b5b4621, 78 },
    { 0x1d1b4621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb4627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828b462b, 80 },
    { 0, 0 },
    { 0x6afb462d, 98 },
    { 0x659b462e, 34 },
    { 0xc9b462d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554b4635, 61 },
    { 0, 0 },
    { 0xba8b4637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73eb463f, 87 },
    { 0, 0 },
    { 0x1dfb4641, 90 },
    { 0x784b4642, 68 },
    { 0x92eb4642, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b464a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb464d, 76 },
    { 0, 0 },
    { 0x585b464f, 90 },
    { 0x1e5b4650, 23 },
    { 0x51db4651, 43 },
    { 0xa70b4651, 23 },
    { 0x33cb4653, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adb4657, 61 },
    { 0x637b4658, 75 },
    { 0x7bdb4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb465d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x861b4660, 43 },
    { 0, 0 },
    { 0xe10b4662, 46 },
    { 0, 0 },
    { 0xca2b4664, 76 },
    { 0, 0 },
    { 0xe28b4666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x124b4669, 61 },
    { 0x83eb466a, 61 },
    { 0, 0 },
    { 0x763b466c, 33 },
    { 0x4e8b466d, 76 },
    { 0, 0 },
    { 0xb30b466f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150b4674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb4677, 90 },
    { 0, 0 },
    { 0xd8eb4679, 98 },
    { 0xbc0b4679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b467f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12b468a, 98 },
    { 0x305b468a, 91 },
    { 0x123b468c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b468f, 61 },
    { 0x4c4b4690, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cb4694, 23 },
    { 0x5eab4695, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x727b4698, 85 },
    { 0x421b4699, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b46a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfb46a5, 98 },
    { 0x43db46a6, 99 },
    { 0, 0 },
    { 0x5ab46a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b46ab, 90 },
    { 0x2b4b46ac, 55 },
    { 0, 0 },
    { 0xb2db46ae, 48 },
    { 0x13bb46ae, 23 },
    { 0xc02b46ae, 93 },
    { 0xf59b46b1, 61 },
    { 0, 0 },
    { 0x84b46b3, 95 },
    { 0, 0 },
    { 0x99eb46b5, 88 },
    { 0xb42b46b6, 78 },
    { 0, 0 },
    { 0x904b46b8, 34 },
    { 0xe1b46b9, 98 },
    { 0, 0 },
    { 0x9e4b46bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b46bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab46c7, 33 },
    { 0x826b46c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x411b46cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b46ce, 95 },
    { 0x2b3b46ce, 23 },
    { 0x296b46d0, 33 },
    { 0x3f5b46d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489b46d5, 34 },
    { 0x153b46d5, 23 },
    { 0xa9eb46d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b46de, 33 },
    { 0, 0 },
    { 0x5b1b46e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb46e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfab46e9, 90 },
    { 0x34eb46e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b46ed, 46 },
    { 0x124b46ed, 33 },
    { 0, 0 },
    { 0x91cb46f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb46f3, 90 },
    { 0xa8bb46f4, 52 },
    { 0xf79b46f5, 98 },
    { 0, 0 },
    { 0x2e0b46f7, 65 },
    { 0, 0 },
    { 0x9e4b46f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b46fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ab4704, 76 },
    { 0x64fb4704, 95 },
    { 0xe9fb4706, 85 },
    { 0x65fb4707, 51 },
    { 0xe5bb4708, 64 },
    { 0xd79b4709, 17 },
    { 0x56fb470a, 33 },
    { 0xe91b470b, 78 },
    { 0, 0 },
    { 0x4cdb470d, 98 },
    { 0x80db470d, 34 },
    { 0xef8b470e, 51 },
    { 0x4b3b4710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80b4714, 43 },
    { 0, 0 },
    { 0xe93b4716, 61 },
    { 0xaf5b4717, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcb471b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb471e, 77 },
    { 0x3b4b471f, 33 },
    { 0x71db4720, 98 },
    { 0, 0 },
    { 0xef0b4722, 76 },
    { 0x2bb4722, 87 },
    { 0x7a5b4724, 98 },
    { 0x603b4722, 67 },
    { 0, 0 },
    { 0xcd4b4727, 87 },
    { 0xa1db4728, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83b4733, 66 },
    { 0x723b4734, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab4737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb473b, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb4747, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8b474a, 87 },
    { 0, 0 },
    { 0xe07b474c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8b474f, 46 },
    { 0x5a7b4750, 77 },
    { 0xf59b4751, 76 },
    { 0x58bb4751, 95 },
    { 0x2ffb4753, 91 },
    { 0x5eeb4751, 19 },
    { 0xce8b4755, 95 },
    { 0xdb2b4756, 60 },
    { 0xb32b4757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1b475a, 87 },
    { 0x286b475b, 93 },
    { 0x4e5b475a, 52 },
    { 0x218b475c, 43 },
    { 0xf49b475e, 33 },
    { 0x580b475f, 93 },
    { 0xa9cb475e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4770, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b477b, 33 },
    { 0xa0ab477c, 33 },
    { 0x910b477d, 23 },
    { 0xaeb477e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b4781, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab4786, 66 },
    { 0x8d1b4787, 61 },
    { 0xd67b4787, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528b4793, 76 },
    { 0x68eb4794, 90 },
    { 0xfd7b4795, 98 },
    { 0xacb4796, 76 },
    { 0xadab4797, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5db479b, 95 },
    { 0x7a1b479c, 90 },
    { 0x24cb479b, 98 },
    { 0x336b479b, 33 },
    { 0x528b479c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1b47a4, 55 },
    { 0, 0 },
    { 0xea4b47a6, 61 },
    { 0, 0 },
    { 0x8f2b47a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b47ac, 66 },
    { 0xf75b47ad, 95 },
    { 0, 0 },
    { 0x9d9b47af, 90 },
    { 0, 0 },
    { 0xa09b47b1, 81 },
    { 0xdb4b47b1, 61 },
    { 0x582b47b3, 95 },
    { 0x91bb47b3, 33 },
    { 0x1cb47b5, 61 },
    { 0x468b47b5, 98 },
    { 0x733b47b7, 76 },
    { 0x65fb47b7, 99 },
    { 0, 0 },
    { 0xda0b47ba, 101 },
    { 0, 0 },
    { 0x143b47bc, 95 },
    { 0xf17b47bd, 93 },
    { 0, 0 },
    { 0x7e3b47bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0db47c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57b47c6, 98 },
    { 0, 0 },
    { 0x1acb47c8, 90 },
    { 0xfd4b47c9, 33 },
    { 0, 0 },
    { 0x5bbb47cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14b47ce, 52 },
    { 0, 0 },
    { 0xdc6b47d0, 81 },
    { 0xd11b47d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cb47d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b47d9, 90 },
    { 0x178b47da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb47e0, 33 },
    { 0xf3fb47e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb47e6, 66 },
    { 0x6e0b47e6, 90 },
    { 0, 0 },
    { 0x9abb47e9, 34 },
    { 0x58eb47ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x762b47ed, 98 },
    { 0x23db47ee, 99 },
    { 0x852b47ed, 33 },
    { 0xc5b47ed, 98 },
    { 0x3f4b47f1, 93 },
    { 0xa8eb47ee, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb47f6, 61 },
    { 0, 0 },
    { 0x9fbb47f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57b47fc, 43 },
    { 0, 0 },
    { 0xa1b47fe, 90 },
    { 0xd79b47ff, 90 },
    { 0xa4db47ff, 55 },
    { 0x30ab47ff, 93 },
    { 0, 0 },
    { 0x8eb4803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1b4806, 100 },
    { 0x324b4807, 90 },
    { 0x3e4b4808, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7b480b, 68 },
    { 0x9d1b480c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97b4810, 33 },
    { 0xd58b4811, 78 },
    { 0x92b4810, 61 },
    { 0x25b4812, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41b481a, 98 },
    { 0xecdb481a, 23 },
    { 0, 0 },
    { 0x4c2b481d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ab4829, 90 },
    { 0xde4b482a, 85 },
    { 0x201b4829, 38 },
    { 0x354b482c, 93 },
    { 0x5d7b482d, 34 },
    { 0, 0 },
    { 0x7fdb482f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8b4832, 76 },
    { 0x56db4833, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4837, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615b4841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1b4845, 46 },
    { 0xdb9b4846, 90 },
    { 0xde0b4847, 93 },
    { 0, 0 },
    { 0x38eb4849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98b484c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0b484f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fb4852, 34 },
    { 0x3bcb4853, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b4857, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b485a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x738b485d, 90 },
    { 0, 0 },
    { 0xa20b485f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b4868, 98 },
    { 0, 0 },
    { 0x979b486a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b486f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cb4877, 77 },
    { 0x16cb4878, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ab487c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb4881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fb4885, 34 },
    { 0xa04b4886, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b488c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab488f, 61 },
    { 0xe63b4890, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab4897, 33 },
    { 0x399b4898, 90 },
    { 0x803b4897, 33 },
    { 0xc05b4898, 95 },
    { 0xf86b489a, 76 },
    { 0xb49b489c, 61 },
    { 0, 0 },
    { 0xc5b489e, 61 },
    { 0x7e9b489f, 23 },
    { 0xb04b489e, 33 },
    { 0xc40b48a1, 66 },
    { 0, 0 },
    { 0x2a2b48a3, 55 },
    { 0x4f7b48a4, 61 },
    { 0x24eb48a3, 61 },
    { 0x5f4b48a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fb48b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b48b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb48ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b48bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab48c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb48c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4b48cb, 84 },
    { 0xcf7b48cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450b48d0, 33 },
    { 0xea7b48d1, 61 },
    { 0x15db48d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507b48d7, 98 },
    { 0x80cb48d8, 95 },
    { 0, 0 },
    { 0xcdab48da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b48df, 61 },
    { 0, 0 },
    { 0xc1cb48e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b48e4, 61 },
    { 0x293b48e5, 61 },
    { 0, 0 },
    { 0x122b48e7, 33 },
    { 0, 0 },
    { 0x24b48e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb48ed, 93 },
    { 0, 0 },
    { 0x92b48ef, 53 },
    { 0, 0 },
    { 0xdf9b48f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82b48f4, 61 },
    { 0xfd9b48f5, 23 },
    { 0x5f3b48f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b48f9, 33 },
    { 0x6c3b48fa, 95 },
    { 0x23cb48fb, 93 },
    { 0xffeb48fc, 61 },
    { 0xc59b48fd, 68 },
    { 0x21b48fd, 33 },
    { 0x781b48fb, 61 },
    { 0x263b4900, 101 },
    { 0xddb4900, 95 },
    { 0x2a2b4900, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b4907, 90 },
    { 0x1bcb4908, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b490c, 33 },
    { 0x5d5b490d, 98 },
    { 0x6bb490e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bb4912, 33 },
    { 0x83bb4912, 61 },
    { 0xc3db4914, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeab491a, 78 },
    { 0xb34b491a, 33 },
    { 0xd08b491b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b491f, 46 },
    { 0x254b4920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb4924, 76 },
    { 0xee4b4924, 95 },
    { 0x5ddb4924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb4929, 91 },
    { 0x571b4929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b492e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb4931, 61 },
    { 0x495b4931, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb4935, 98 },
    { 0x61cb4935, 90 },
    { 0x7a7b4937, 43 },
    { 0xe5fb4938, 61 },
    { 0x73fb4939, 98 },
    { 0xb39b4939, 86 },
    { 0, 0 },
    { 0x77bb493c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9b4940, 33 },
    { 0x185b4940, 99 },
    { 0xb41b4941, 98 },
    { 0, 0 },
    { 0x295b4944, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb4947, 95 },
    { 0xb60b4948, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3b494d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb4953, 55 },
    { 0x963b4954, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b4957, 76 },
    { 0, 0 },
    { 0x607b4959, 95 },
    { 0x402b495a, 43 },
    { 0, 0 },
    { 0x198b495c, 98 },
    { 0x7cb495c, 55 },
    { 0xd1cb495e, 48 },
    { 0, 0 },
    { 0x947b4960, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x706b4963, 90 },
    { 0xaa7b4964, 43 },
    { 0x6b8b4965, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ab496b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b496e, 55 },
    { 0xba9b496f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x532b4972, 23 },
    { 0, 0 },
    { 0xb86b4974, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42b4977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb4983, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b4986, 94 },
    { 0xd64b4986, 78 },
    { 0x5d2b4988, 100 },
    { 0xc8eb4989, 23 },
    { 0xe0fb4988, 17 },
    { 0xc27b4987, 40 },
    { 0xc4b4989, 67 },
    { 0x24fb498d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb4994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b499a, 31 },
    { 0x86cb499b, 43 },
    { 0xc7fb499b, 55 },
    { 0x2dab499d, 76 },
    { 0x97ab499a, 65 },
    { 0xd37b499d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b49a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b49a5, 98 },
    { 0xd50b49a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51b49ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5b49b6, 33 },
    { 0xbf4b49b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6b49ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b49c6, 33 },
    { 0, 0 },
    { 0x412b49c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b49cd, 33 },
    { 0, 0 },
    { 0x9b2b49cf, 33 },
    { 0x354b49cf, 34 },
    { 0, 0 },
    { 0x72ab49d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9b49da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb49dd, 88 },
    { 0x48fb49dd, 76 },
    { 0x75eb49df, 95 },
    { 0x913b49e0, 100 },
    { 0x364b49de, 95 },
    { 0, 0 },
    { 0x5e5b49e3, 33 },
    { 0x5ccb49e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13b49e9, 53 },
    { 0, 0 },
    { 0x799b49eb, 43 },
    { 0xcbb49ec, 95 },
    { 0xd2cb49ec, 55 },
    { 0xd6db49ee, 90 },
    { 0, 0 },
    { 0xb69b49f0, 90 },
    { 0x455b49f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcb49f6, 98 },
    { 0x565b49f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389b49fc, 98 },
    { 0, 0 },
    { 0xbebb49fe, 23 },
    { 0, 0 },
    { 0xb2b4a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b4a04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x907b4a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acb4a0e, 93 },
    { 0, 0 },
    { 0x76eb4a10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57b4a13, 90 },
    { 0, 0 },
    { 0x5eb4a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cb4a18, 84 },
    { 0, 0 },
    { 0xc03b4a1a, 55 },
    { 0x37db4a1b, 90 },
    { 0, 0 },
    { 0xa07b4a1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5b4a20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab4a24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b4a27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48b4a2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ab4a36, 23 },
    { 0, 0 },
    { 0xf6db4a38, 66 },
    { 0, 0 },
    { 0xf7cb4a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b4a40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x469b4a43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b4a49, 23 },
    { 0xac9b4a49, 90 },
    { 0x6a9b4a4b, 98 },
    { 0xe6cb4a4a, 55 },
    { 0, 0 },
    { 0x8dcb4a4e, 61 },
    { 0, 0 },
    { 0x83bb4a50, 90 },
    { 0xd91b4a51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb4a55, 95 },
    { 0x9bab4a56, 23 },
    { 0xb0fb4a57, 91 },
    { 0x15bb4a58, 23 },
    { 0x476b4a59, 76 },
    { 0, 0 },
    { 0x593b4a5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b4a60, 46 },
    { 0x99fb4a61, 88 },
    { 0x8c0b4a62, 93 },
    { 0x585b4a61, 55 },
    { 0xf33b4a64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b4a67, 23 },
    { 0xb6bb4a68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b4a6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fb4a71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b4a75, 46 },
    { 0, 0 },
    { 0xdfcb4a77, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb4a7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b4a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139b4a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147b4a8f, 61 },
    { 0, 0 },
    { 0xdfcb4a91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5b4a94, 95 },
    { 0, 0 },
    { 0xc3db4a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ab4a9d, 33 },
    { 0, 0 },
    { 0xc6fb4a9f, 97 },
    { 0x3fcb4a9f, 87 },
    { 0xdbeb4aa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b4aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09b4aad, 90 },
    { 0xb3fb4aae, 33 },
    { 0x313b4aaf, 98 },
    { 0x70bb4aaf, 76 },
    { 0xe00b4ab1, 17 },
    { 0xd35b4ab0, 98 },
    { 0xbbab4ab3, 61 },
    { 0x695b4aaf, 23 },
    { 0, 0 },
    { 0xde1b4ab6, 61 },
    { 0, 0 },
    { 0x1b2b4ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0b4abe, 91 },
    { 0x772b4abe, 33 },
    { 0xfabb4ac0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb4ac6, 90 },
    { 0xb65b4ac7, 88 },
    { 0xaaab4ac7, 61 },
    { 0xfa7b4ac9, 52 },
    { 0x6b9b4aca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04b4ad2, 33 },
    { 0x637b4ad3, 95 },
    { 0x263b4ad4, 33 },
    { 0xa76b4ad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b4adc, 90 },
    { 0x97cb4add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b4ae3, 31 },
    { 0, 0 },
    { 0x673b4ae5, 90 },
    { 0xac0b4ae6, 88 },
    { 0, 0 },
    { 0x794b4ae8, 23 },
    { 0xafb4ae8, 33 },
    { 0, 0 },
    { 0xb25b4aeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746b4aef, 17 },
    { 0x7b0b4af0, 43 },
    { 0xbb1b4af1, 61 },
    { 0x7b8b4af1, 95 },
    { 0x1f6b4af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb4afe, 98 },
    { 0, 0 },
    { 0x476b4b00, 90 },
    { 0x870b4b00, 23 },
    { 0x562b4b01, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5b4b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fb4b0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1b4b0f, 61 },
    { 0x297b4b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6b4b1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b4b1d, 95 },
    { 0, 0 },
    { 0x46ab4b1f, 33 },
    { 0x809b4b1f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2b4b26, 88 },
    { 0, 0 },
    { 0xbe0b4b28, 95 },
    { 0x3db4b29, 90 },
    { 0x230b4b2a, 43 },
    { 0, 0 },
    { 0x4f6b4b2c, 33 },
    { 0x338b4b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fb4b33, 34 },
    { 0x746b4b34, 98 },
    { 0x776b4b35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65b4b38, 61 },
    { 0x93ab4b39, 78 },
    { 0x18b4b39, 23 },
    { 0x482b4b3a, 52 },
    { 0, 0 },
    { 0xdcbb4b3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb4b42, 95 },
    { 0, 0 },
    { 0x42fb4b44, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b4b48, 76 },
    { 0, 0 },
    { 0x940b4b4a, 33 },
    { 0, 0 },
    { 0x5c1b4b4c, 43 },
    { 0, 0 },
    { 0x953b4b4e, 99 },
    { 0x134b4b4f, 90 },
    { 0x2d4b4b4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09b4b54, 61 },
    { 0x2f4b4b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb4b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x184b4b5e, 90 },
    { 0xdfb4b5f, 34 },
    { 0x234b4b60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecb4b64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efb4b68, 76 },
    { 0, 0 },
    { 0x40cb4b6a, 55 },
    { 0xc86b4b6b, 84 },
    { 0, 0 },
    { 0xeab4b6d, 66 },
    { 0x2f0b4b6e, 43 },
    { 0x6eeb4b6f, 52 },
    { 0, 0 },
    { 0xb10b4b71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b4b74, 88 },
    { 0xe70b4b74, 33 },
    { 0xcb6b4b74, 98 },
    { 0, 0 },
    { 0x96cb4b78, 80 },
    { 0x187b4b78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ab4b7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b4b84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbb4b8a, 93 },
    { 0x443b4b8a, 76 },
    { 0x7a0b4b8c, 95 },
    { 0x332b4b8d, 95 },
    { 0, 0 },
    { 0xaadb4b8f, 78 },
    { 0xb9cb4b90, 23 },
    { 0, 0 },
    { 0x7c6b4b92, 93 },
    { 0, 0 },
    { 0x9bb4b94, 23 },
    { 0x7acb4b95, 98 },
    { 0x596b4b96, 61 },
    { 0, 0 },
    { 0x10db4b98, 17 },
    { 0x3e9b4b99, 90 },
    { 0, 0 },
    { 0x817b4b9b, 61 },
    { 0x88cb4b9c, 52 },
    { 0, 0 },
    { 0x6d6b4b9e, 43 },
    { 0, 0 },
    { 0xcd4b4ba0, 90 },
    { 0, 0 },
    { 0x403b4ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5b4ba6, 90 },
    { 0xaefb4ba7, 100 },
    { 0x2e0b4ba7, 61 },
    { 0xcf9b4ba6, 90 },
    { 0x315b4baa, 99 },
    { 0x128b4bab, 61 },
    { 0xf2ab4ba7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x687b4baf, 40 },
    { 0x91bb4bb0, 61 },
    { 0x6ddb4bb1, 23 },
    { 0xcf4b4bb2, 33 },
    { 0x5d7b4bb0, 95 },
    { 0, 0 },
    { 0x3dfb4bb5, 98 },
    { 0xc9eb4bb6, 93 },
    { 0, 0 },
    { 0x12b4bb8, 93 },
    { 0, 0 },
    { 0xcebb4bba, 23 },
    { 0xc72b4bbb, 23 },
    { 0, 0 },
    { 0x8d0b4bbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76b4bc0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822b4bc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1b4bcd, 33 },
    { 0, 0 },
    { 0x17cb4bcf, 90 },
    { 0x2b1b4bcf, 90 },
    { 0, 0 },
    { 0xb37b4bd2, 90 },
    { 0, 0 },
    { 0x21b4bd4, 66 },
    { 0xa1ab4bd5, 98 },
    { 0, 0 },
    { 0x37bb4bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50b4bdb, 91 },
    { 0, 0 },
    { 0xc42b4bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b4be0, 67 },
    { 0, 0 },
    { 0x19b4be2, 66 },
    { 0, 0 },
    { 0xfc9b4be4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6b4be8, 98 },
    { 0xa09b4be9, 46 },
    { 0, 0 },
    { 0xc24b4beb, 33 },
    { 0xe3fb4beb, 66 },
    { 0, 0 },
    { 0x8e4b4bee, 93 },
    { 0x81fb4bef, 95 },
    { 0, 0 },
    { 0x4b2b4bf1, 67 },
    { 0x537b4bf2, 95 },
    { 0, 0 },
    { 0xd4b4bf4, 23 },
    { 0, 0 },
    { 0xff7b4bf6, 61 },
    { 0x4ecb4bf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2b4bfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fb4c02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b4c05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb4c08, 33 },
    { 0x56cb4c09, 43 },
    { 0x627b4c0a, 19 },
    { 0x5d9b4c0a, 33 },
    { 0xb7eb4c09, 23 },
    { 0xae0b4c0b, 33 },
    { 0, 0 },
    { 0xb12b4c0f, 23 },
    { 0x8f4b4c10, 61 },
    { 0xc2ab4c11, 23 },
    { 0, 0 },
    { 0x3d3b4c13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fb4c16, 90 },
    { 0xd94b4c17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b4c1e, 61 },
    { 0x8e4b4c1f, 76 },
    { 0, 0 },
    { 0xd81b4c21, 90 },
    { 0x229b4c21, 87 },
    { 0x377b4c23, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7b4c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b4c2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dab4c33, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b4c39, 87 },
    { 0x4fcb4c39, 90 },
    { 0, 0 },
    { 0xf6fb4c3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8b4c40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149b4c47, 90 },
    { 0xa39b4c48, 90 },
    { 0x57eb4c49, 85 },
    { 0x2abb4c4a, 43 },
    { 0, 0 },
    { 0x592b4c4c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b4c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b4c53, 33 },
    { 0x295b4c54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcb4c58, 98 },
    { 0x5a8b4c59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204b4c63, 100 },
    { 0x274b4c64, 55 },
    { 0x7e4b4c64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260b4c69, 5 },
    { 0, 0 },
    { 0x845b4c6b, 90 },
    { 0x35cb4c6c, 95 },
    { 0x5fcb4c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b4c72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b4c84, 68 },
    { 0xfbcb4c84, 23 },
    { 0, 0 },
    { 0x504b4c87, 33 },
    { 0xbb0b4c88, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb4c8c, 23 },
    { 0xe9eb4c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cb4c98, 90 },
    { 0x1b0b4c99, 43 },
    { 0, 0 },
    { 0x49bb4c9b, 68 },
    { 0, 0 },
    { 0x240b4c9d, 90 },
    { 0x58fb4c9e, 33 },
    { 0x79ab4c9d, 81 },
    { 0x621b4c9f, 64 },
    { 0xe75b4c9e, 33 },
    { 0, 0 },
    { 0x599b4ca3, 23 },
    { 0, 0 },
    { 0xc1fb4ca5, 94 },
    { 0xcdeb4ca5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b4caa, 33 },
    { 0, 0 },
    { 0xbd3b4cac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552b4cbd, 52 },
    { 0xf53b4cbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15b4cc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0b4cc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b4cc8, 76 },
    { 0, 0 },
    { 0x334b4cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0b4cd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89b4ce7, 68 },
    { 0x227b4ce8, 90 },
    { 0x2cdb4ce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b4cf2, 98 },
    { 0xc26b4cf2, 33 },
    { 0, 0 },
    { 0xae5b4cf5, 93 },
    { 0, 0 },
    { 0xe8fb4cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5b4cfe, 43 },
    { 0xfa3b4cff, 98 },
    { 0x94db4d00, 33 },
    { 0x345b4d01, 95 },
    { 0x6c0b4d02, 90 },
    { 0, 0 },
    { 0x25ab4d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2b4d09, 13 },
    { 0, 0 },
    { 0xc38b4d0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbb4d11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab4d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cb4d1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b4d1f, 90 },
    { 0x54eb4d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b4d24, 91 },
    { 0xed5b4d24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b4d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ab4d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281b4d3f, 46 },
    { 0x89ab4d3f, 84 },
    { 0x496b4d3f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57b4d44, 33 },
    { 0x16eb4d45, 75 },
    { 0x4f0b4d46, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60b4d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bb4d50, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8b4d55, 55 },
    { 0xd3bb4d55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3b4d59, 62 },
    { 0x9e1b4d5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b4d5d, 90 },
    { 0, 0 },
    { 0x294b4d5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b4d66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1b4d69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffb4d6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b4d73, 76 },
    { 0xffbb4d74, 23 },
    { 0xc38b4d75, 44 },
    { 0x54b4d76, 98 },
    { 0x45eb4d76, 34 },
    { 0x907b4d76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4d7b, 23 },
    { 0xb7fb4d7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cb4d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b4d86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b4d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab4d94, 90 },
    { 0xffdb4d95, 43 },
    { 0xc06b4d95, 61 },
    { 0x11db4d97, 61 },
    { 0xe79b4d98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b4d9f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb4da4, 91 },
    { 0xecbb4da4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fb4dab, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ab4db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612b4db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bb4dbb, 85 },
    { 0xba4b4dbc, 90 },
    { 0x938b4dbd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcb4dc0, 33 },
    { 0, 0 },
    { 0xd69b4dc2, 90 },
    { 0, 0 },
    { 0xddbb4dc4, 46 },
    { 0x361b4dc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431b4dcf, 46 },
    { 0xbd9b4dd0, 52 },
    { 0xe5fb4dcf, 12 },
    { 0xe63b4dd0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9b4dd5, 23 },
    { 0x2a1b4dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b4dd9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670b4dde, 34 },
    { 0xaafb4dde, 64 },
    { 0, 0 },
    { 0x74ab4de1, 43 },
    { 0x36bb4de2, 55 },
    { 0x548b4de3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b4dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4def, 78 },
    { 0, 0 },
    { 0x1e2b4df1, 25 },
    { 0, 0 },
    { 0x518b4df3, 90 },
    { 0, 0 },
    { 0x7d1b4df5, 95 },
    { 0x20cb4df5, 33 },
    { 0x23b4df7, 98 },
    { 0xdbab4df8, 90 },
    { 0x579b4df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b4dfd, 76 },
    { 0x166b4dfe, 34 },
    { 0x840b4dff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75b4e02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520b4e08, 95 },
    { 0, 0 },
    { 0xea7b4e0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7b4e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b4e12, 33 },
    { 0x544b4e13, 90 },
    { 0, 0 },
    { 0xc78b4e15, 46 },
    { 0x634b4e15, 55 },
    { 0x262b4e17, 97 },
    { 0x6eab4e17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8b4e20, 95 },
    { 0, 0 },
    { 0x278b4e22, 43 },
    { 0xd47b4e23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b4e28, 90 },
    { 0, 0 },
    { 0x2ddb4e2a, 66 },
    { 0x8ab4e2a, 95 },
    { 0x57ab4e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7b4e30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb4e33, 52 },
    { 0x4eb4e34, 76 },
    { 0, 0 },
    { 0x1ceb4e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b4e40, 98 },
    { 0, 0 },
    { 0x4f6b4e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab4e47, 85 },
    { 0x49bb4e47, 80 },
    { 0, 0 },
    { 0xcb4b4e4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11db4e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29eb4e50, 93 },
    { 0xe6bb4e51, 43 },
    { 0x46bb4e52, 90 },
    { 0x92fb4e53, 33 },
    { 0x104b4e54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfb4e64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab4e68, 84 },
    { 0x4b3b4e68, 95 },
    { 0xb1ab4e68, 55 },
    { 0x941b4e69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b4e6e, 98 },
    { 0x698b4e6f, 65 },
    { 0x647b4e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x688b4e73, 99 },
    { 0, 0 },
    { 0x707b4e75, 90 },
    { 0x590b4e75, 23 },
    { 0x506b4e77, 90 },
    { 0, 0 },
    { 0xbd9b4e79, 23 },
    { 0x270b4e79, 33 },
    { 0x14fb4e7a, 43 },
    { 0x2ccb4e7a, 90 },
    { 0, 0 },
    { 0x63cb4e7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b4e84, 78 },
    { 0xc6db4e85, 76 },
    { 0, 0 },
    { 0x2b4e87, 98 },
    { 0x679b4e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75b4e8c, 55 },
    { 0x952b4e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb4e90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb4e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3b4e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcb4ea3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b4ea8, 90 },
    { 0x39cb4ea9, 40 },
    { 0x64bb4eaa, 98 },
    { 0, 0 },
    { 0x6bfb4eac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b4eb2, 3 },
    { 0x478b4eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b4eb7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154b4ebb, 61 },
    { 0x9e5b4ebc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3b4ec0, 78 },
    { 0x56fb4ec1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b4ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5b4ece, 52 },
    { 0xb41b4ece, 98 },
    { 0xa55b4ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86b4ed4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b4ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x718b4eda, 101 },
    { 0, 0 },
    { 0xc2ab4edc, 61 },
    { 0xaf4b4edd, 95 },
    { 0xb04b4ede, 33 },
    { 0x5e6b4edf, 61 },
    { 0xd97b4edf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2b4ee7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9b4eeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fb4eef, 90 },
    { 0x7fbb4ef0, 85 },
    { 0xe62b4ef0, 91 },
    { 0x8b4ef0, 75 },
    { 0x621b4ef3, 33 },
    { 0x9f3b4ef4, 94 },
    { 0xc88b4ef5, 53 },
    { 0x14eb4ef6, 91 },
    { 0xbd2b4ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b4efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bb4f02, 98 },
    { 0x708b4f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eb4f06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b4f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b4f0c, 90 },
    { 0x65bb4f0d, 33 },
    { 0xf3b4f0c, 34 },
    { 0, 0 },
    { 0xe1db4f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5b4f13, 95 },
    { 0, 0 },
    { 0xfe2b4f15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eb4f19, 66 },
    { 0, 0 },
    { 0xd4eb4f1b, 61 },
    { 0x922b4f1c, 52 },
    { 0, 0 },
    { 0x531b4f1e, 34 },
    { 0xc4cb4f1f, 90 },
    { 0xe0bb4f1e, 81 },
    { 0xb0bb4f21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b4f24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b4f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beb4f36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0b4f3f, 99 },
    { 0x812b4f3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2b4f46, 23 },
    { 0xe7cb4f47, 93 },
    { 0, 0 },
    { 0xa8cb4f49, 97 },
    { 0x8c3b4f4a, 55 },
    { 0x688b4f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774b4f56, 76 },
    { 0x699b4f56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bb4f60, 53 },
    { 0, 0 },
    { 0x73cb4f62, 61 },
    { 0xeeab4f63, 33 },
    { 0xf1bb4f62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xafeb4f67, 55 },
    { 0x99b4f67, 33 },
    { 0x9f6b4f69, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afb4f71, 95 },
    { 0x8e9b4f72, 52 },
    { 0, 0 },
    { 0x4b9b4f74, 88 },
    { 0x6cdb4f75, 51 },
    { 0xeccb4f76, 55 },
    { 0x458b4f74, 90 },
    { 0, 0 },
    { 0xe42b4f79, 34 },
    { 0xbc4b4f7a, 90 },
    { 0xceeb4f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b4f7e, 93 },
    { 0xdb6b4f7f, 90 },
    { 0x71ab4f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bb4f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9db4f89, 52 },
    { 0, 0 },
    { 0x17eb4f8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb4f8f, 66 },
    { 0xca6b4f8f, 95 },
    { 0xbe6b4f91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab4f95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505b4fa0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb4fab, 34 },
    { 0xa66b4fac, 34 },
    { 0x9f1b4fac, 67 },
    { 0xe7fb4fae, 93 },
    { 0, 0 },
    { 0x984b4fb0, 33 },
    { 0, 0 },
    { 0x6e7b4fb2, 46 },
    { 0x768b4fb2, 90 },
    { 0xb11b4fb4, 95 },
    { 0xefcb4fb3, 90 },
    { 0xb42b4fb6, 55 },
    { 0, 0 },
    { 0xbc7b4fb8, 46 },
    { 0x1fdb4fb8, 23 },
    { 0, 0 },
    { 0x591b4fbb, 98 },
    { 0xdb9b4fbb, 66 },
    { 0x2ab4fbd, 43 },
    { 0xa9cb4fbc, 90 },
    { 0, 0 },
    { 0xa5ab4fc0, 55 },
    { 0x550b4fc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b4fcb, 95 },
    { 0x3e1b4fcb, 90 },
    { 0x69fb4fcc, 33 },
    { 0x6a4b4fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaab4fd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb4fd4, 90 },
    { 0, 0 },
    { 0x576b4fd6, 55 },
    { 0, 0 },
    { 0x318b4fd8, 55 },
    { 0, 0 },
    { 0x7bb4fda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27b4fde, 91 },
    { 0, 0 },
    { 0xfabb4fe0, 100 },
    { 0, 0 },
    { 0xde3b4fe2, 61 },
    { 0xab5b4fe2, 66 },
    { 0xaa6b4fe4, 95 },
    { 0x19db4fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb4fe9, 67 },
    { 0x928b4fea, 99 },
    { 0x12cb4feb, 61 },
    { 0, 0 },
    { 0xe5cb4fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eab4ff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b4ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ab4ffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5b4ffd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b5005, 74 },
    { 0xc2eb5005, 46 },
    { 0xfd6b5007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b500d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b5012, 77 },
    { 0x8e8b5013, 98 },
    { 0x4bab5013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecb5019, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b501c, 55 },
    { 0xa2eb501d, 69 },
    { 0, 0 },
    { 0x898b501f, 76 },
    { 0x4e1b501f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6b5028, 93 },
    { 0x36b5029, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8db502c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b5032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b5039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9b503f, 33 },
    { 0x218b5040, 98 },
    { 0, 0 },
    { 0xa9b5042, 98 },
    { 0x203b5043, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bb5048, 34 },
    { 0x6a7b5049, 61 },
    { 0, 0 },
    { 0x101b504b, 61 },
    { 0xdcab504c, 33 },
    { 0xa44b504b, 33 },
    { 0x154b504e, 46 },
    { 0x3b4b504c, 98 },
    { 0x97fb5050, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb5053, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x348b5056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x676b5059, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05b5060, 77 },
    { 0xa19b5061, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116b5065, 23 },
    { 0xc93b5066, 40 },
    { 0x4a9b5067, 76 },
    { 0xb5ab5066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b506b, 98 },
    { 0, 0 },
    { 0xff5b506d, 43 },
    { 0, 0 },
    { 0xb85b506f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b5081, 91 },
    { 0x6aab5082, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7b5087, 65 },
    { 0, 0 },
    { 0x979b5089, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b508c, 90 },
    { 0xe2fb508d, 99 },
    { 0x54b508d, 100 },
    { 0, 0 },
    { 0x318b5090, 34 },
    { 0x571b5091, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b5098, 77 },
    { 0x620b5099, 34 },
    { 0x7a1b509a, 76 },
    { 0x601b509a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b509f, 97 },
    { 0, 0 },
    { 0x4c5b50a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b50ab, 61 },
    { 0xdd1b50ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963b50b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb50b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb50bd, 98 },
    { 0x568b50bd, 90 },
    { 0x50cb50bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab50c5, 98 },
    { 0x2abb50c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b50cf, 33 },
    { 0x5c6b50cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb50d3, 85 },
    { 0, 0 },
    { 0xc96b50d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583b50e0, 91 },
    { 0, 0 },
    { 0xf31b50e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b50e5, 17 },
    { 0x2bab50e6, 80 },
    { 0x3edb50e7, 46 },
    { 0x61eb50e8, 77 },
    { 0x906b50e8, 61 },
    { 0x880b50ea, 61 },
    { 0x24cb50ea, 98 },
    { 0xa94b50ea, 90 },
    { 0xa15b50ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb50f0, 46 },
    { 0, 0 },
    { 0xf03b50f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493b50f6, 98 },
    { 0x900b50f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb50fb, 61 },
    { 0x90b50fc, 43 },
    { 0, 0 },
    { 0x692b50fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b5107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b510a, 17 },
    { 0x7adb510b, 90 },
    { 0, 0 },
    { 0x70ab510d, 52 },
    { 0, 0 },
    { 0x935b510f, 91 },
    { 0x79b5110, 56 },
    { 0xec0b510f, 23 },
    { 0x717b5112, 90 },
    { 0xb80b5113, 91 },
    { 0x628b5113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb5117, 66 },
    { 0, 0 },
    { 0xd00b5119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9b511d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb5121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459b5126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb512a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b512f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31b5134, 61 },
    { 0, 0 },
    { 0x26fb5136, 99 },
    { 0x752b5137, 33 },
    { 0xfebb5138, 76 },
    { 0xc1b5139, 53 },
    { 0x7afb513a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cb513e, 23 },
    { 0, 0 },
    { 0xbaab5140, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14b514a, 90 },
    { 0x422b514b, 90 },
    { 0x2f0b514c, 90 },
    { 0x820b514b, 78 },
    { 0, 0 },
    { 0xc31b514f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3b5153, 76 },
    { 0x162b5154, 99 },
    { 0x6e6b5153, 93 },
    { 0x604b5156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8b515c, 76 },
    { 0x2a4b515d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb5160, 80 },
    { 0xf92b5161, 23 },
    { 0xc00b5160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5166, 67 },
    { 0xbb0b5167, 75 },
    { 0, 0 },
    { 0x85cb5169, 78 },
    { 0xed2b516a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eb516f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6b5177, 55 },
    { 0x605b5178, 90 },
    { 0x945b5177, 95 },
    { 0xd2eb517a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdb5181, 52 },
    { 0x820b5182, 19 },
    { 0, 0 },
    { 0xc77b5184, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbb5188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db518b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x303b518e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cb5191, 90 },
    { 0, 0 },
    { 0x375b5193, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9b5196, 78 },
    { 0, 0 },
    { 0x225b5198, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304b519d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b51a0, 98 },
    { 0x74ab51a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b51a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb51a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb51b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bb51ba, 94 },
    { 0, 0 },
    { 0xa37b51bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb51c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb51ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb51cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b51d8, 34 },
    { 0xbe2b51d9, 23 },
    { 0x6f0b51da, 90 },
    { 0xaa6b51db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eb51df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab51e6, 61 },
    { 0, 0 },
    { 0xa63b51e8, 98 },
    { 0x6b51e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957b51ed, 78 },
    { 0x58ab51ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6b51f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02b51f6, 34 },
    { 0, 0 },
    { 0xb3b51f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7b51fd, 98 },
    { 0x5aab51fe, 61 },
    { 0xa73b51fd, 53 },
    { 0, 0 },
    { 0xe4ab5201, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33b5207, 85 },
    { 0, 0 },
    { 0x531b5209, 23 },
    { 0xf30b5209, 76 },
    { 0, 0 },
    { 0x5f1b520c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b5211, 33 },
    { 0x5efb5212, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b5218, 90 },
    { 0xfd7b5218, 80 },
    { 0x48bb521a, 95 },
    { 0x11b521b, 88 },
    { 0x48cb521c, 76 },
    { 0x66cb521a, 98 },
    { 0xec2b5218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb5225, 99 },
    { 0xc99b5226, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b5229, 61 },
    { 0, 0 },
    { 0x2a7b522b, 90 },
    { 0x213b522b, 61 },
    { 0xcc3b522d, 76 },
    { 0x57eb522d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33b5231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ab5234, 14 },
    { 0, 0 },
    { 0xdbdb5236, 93 },
    { 0xfc5b5237, 52 },
    { 0x4e6b5238, 23 },
    { 0x878b5237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fb523c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bb5240, 33 },
    { 0x74bb5241, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fb5244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0b5247, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb524a, 61 },
    { 0, 0 },
    { 0x158b524c, 88 },
    { 0, 0 },
    { 0x747b524e, 61 },
    { 0, 0 },
    { 0xc07b5250, 93 },
    { 0, 0 },
    { 0x56eb5252, 99 },
    { 0x686b5253, 55 },
    { 0x56db5253, 84 },
    { 0x9c5b5252, 95 },
    { 0x7ccb5252, 90 },
    { 0xdceb5257, 38 },
    { 0xfaab5256, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bb525d, 76 },
    { 0, 0 },
    { 0xa5ab525f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db5262, 98 },
    { 0x317b5262, 98 },
    { 0x8afb5264, 61 },
    { 0x761b5264, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6b526b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b526e, 33 },
    { 0x961b526f, 87 },
    { 0, 0 },
    { 0xf53b5271, 100 },
    { 0x3cb5271, 61 },
    { 0x6b0b5273, 70 },
    { 0x47fb5274, 23 },
    { 0xd3fb5275, 88 },
    { 0xef1b5274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efb5280, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8b5289, 51 },
    { 0xdadb528a, 95 },
    { 0xeabb528b, 55 },
    { 0x552b528b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eb5290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x274b5293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db5296, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb529b, 93 },
    { 0xb6b529b, 68 },
    { 0x55b529d, 76 },
    { 0x15eb529d, 82 },
    { 0x413b529e, 23 },
    { 0x2f5b52a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b52a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eab52b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b52b7, 43 },
    { 0x655b52b8, 43 },
    { 0xdb3b52b7, 33 },
    { 0xd59b52b7, 76 },
    { 0x34ab52bb, 34 },
    { 0x360b52bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8b52c2, 23 },
    { 0, 0 },
    { 0xa4eb52c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3b52c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250b52cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b52ce, 61 },
    { 0xf8b52ce, 52 },
    { 0x74cb52ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fb52da, 43 },
    { 0xb14b52db, 82 },
    { 0x531b52dc, 85 },
    { 0, 0 },
    { 0xd6bb52de, 61 },
    { 0, 0 },
    { 0xed1b52e0, 65 },
    { 0x744b52e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5b52e8, 100 },
    { 0xf1bb52e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bb52ed, 95 },
    { 0xd57b52ee, 23 },
    { 0x106b52ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b52f2, 61 },
    { 0x126b52f2, 90 },
    { 0xe90b52f3, 33 },
    { 0, 0 },
    { 0x144b52f6, 23 },
    { 0xc99b52f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b52fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b5304, 90 },
    { 0, 0 },
    { 0x60b5306, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb5309, 90 },
    { 0x2dfb530a, 98 },
    { 0x409b5309, 33 },
    { 0x115b530b, 90 },
    { 0, 0 },
    { 0x8cdb530e, 33 },
    { 0x4dfb530f, 33 },
    { 0xd27b530f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92b5313, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecb5316, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b531a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b5323, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b5327, 61 },
    { 0x3d9b5327, 98 },
    { 0x581b5329, 61 },
    { 0x6f2b5328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb5333, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0b5336, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423b533a, 52 },
    { 0, 0 },
    { 0x7f1b533c, 78 },
    { 0, 0 },
    { 0x4f4b533e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x612b5341, 52 },
    { 0x2c3b5342, 61 },
    { 0, 0 },
    { 0x62ab5344, 98 },
    { 0, 0 },
    { 0x7bbb5346, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1b5349, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25b534c, 90 },
    { 0, 0 },
    { 0xb65b534e, 23 },
    { 0x255b534e, 23 },
    { 0x92db534e, 75 },
    { 0x2a8b5351, 93 },
    { 0x577b5352, 65 },
    { 0xa3b5353, 27 },
    { 0x4d1b5352, 76 },
    { 0x56fb5355, 90 },
    { 0x219b5354, 100 },
    { 0x6a6b5357, 95 },
    { 0xa12b5358, 78 },
    { 0x4f1b5359, 76 },
    { 0x4b3b5359, 46 },
    { 0xb2fb5354, 55 },
    { 0xa55b535c, 61 },
    { 0, 0 },
    { 0x460b535e, 90 },
    { 0xd43b535e, 76 },
    { 0, 0 },
    { 0x860b5361, 98 },
    { 0x712b5362, 34 },
    { 0x625b5362, 75 },
    { 0x87eb5364, 98 },
    { 0xb62b5365, 33 },
    { 0, 0 },
    { 0x86eb5367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b536b, 89 },
    { 0x681b536c, 91 },
    { 0xfb5b536b, 98 },
    { 0x47eb536e, 38 },
    { 0, 0 },
    { 0x9c8b5370, 34 },
    { 0x9acb5370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b5374, 61 },
    { 0x84fb5375, 95 },
    { 0x15cb5376, 55 },
    { 0xf43b5377, 78 },
    { 0x63ab5376, 95 },
    { 0xbb0b5379, 23 },
    { 0, 0 },
    { 0xcccb537b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ab538e, 90 },
    { 0x6a5b538f, 34 },
    { 0x3a8b538e, 93 },
    { 0x66b5391, 61 },
    { 0, 0 },
    { 0xa96b5393, 90 },
    { 0x628b5394, 34 },
    { 0, 0 },
    { 0x549b5396, 90 },
    { 0, 0 },
    { 0x642b5398, 61 },
    { 0xefcb5399, 55 },
    { 0x5e7b5399, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b53a3, 33 },
    { 0x964b53a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b53a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b53ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8b53af, 33 },
    { 0xfb6b53b0, 90 },
    { 0x61fb53b1, 90 },
    { 0xc93b53b2, 98 },
    { 0x9e0b53b3, 61 },
    { 0xaab53b4, 90 },
    { 0x103b53b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb53b9, 90 },
    { 0xab5b53b9, 91 },
    { 0x85b53bb, 95 },
    { 0x3f6b53bc, 66 },
    { 0, 0 },
    { 0x791b53be, 76 },
    { 0xfa5b53bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb53c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27db53c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48db53cc, 43 },
    { 0x17eb53cd, 91 },
    { 0x8ccb53cc, 90 },
    { 0x486b53ce, 98 },
    { 0x489b53cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687b53d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233b53db, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eb53e3, 90 },
    { 0x861b53e4, 90 },
    { 0x6ccb53e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb53ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b53f4, 85 },
    { 0xceb53f4, 23 },
    { 0xac8b53f5, 91 },
    { 0x3b7b53f7, 78 },
    { 0x1e3b53f7, 23 },
    { 0xaf4b53f9, 90 },
    { 0x135b53f8, 61 },
    { 0x9c1b53f5, 98 },
    { 0xb35b53fc, 33 },
    { 0xd78b53fc, 76 },
    { 0xdc0b53f8, 43 },
    { 0, 0 },
    { 0x61bb5400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b5406, 90 },
    { 0, 0 },
    { 0xccbb5408, 33 },
    { 0, 0 },
    { 0xe09b540a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806b5412, 93 },
    { 0xb6cb5413, 95 },
    { 0x223b5414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224b541d, 90 },
    { 0, 0 },
    { 0x13bb541f, 98 },
    { 0x19cb5420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeb5424, 55 },
    { 0xe48b5425, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eb542a, 90 },
    { 0xf56b542b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31b542e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb5432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b5438, 46 },
    { 0xb04b5439, 46 },
    { 0xdefb543a, 33 },
    { 0xc69b543a, 34 },
    { 0xe1cb543b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197b5440, 85 },
    { 0xe5eb5441, 33 },
    { 0xdbfb5442, 76 },
    { 0x861b5443, 23 },
    { 0, 0 },
    { 0x8a3b5445, 95 },
    { 0x146b5445, 98 },
    { 0, 0 },
    { 0xb4bb5448, 34 },
    { 0x4cfb5448, 94 },
    { 0xb5fb544a, 23 },
    { 0, 0 },
    { 0x52ab544c, 76 },
    { 0, 0 },
    { 0x1a8b544e, 43 },
    { 0x90bb544f, 98 },
    { 0x33eb544e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cb5453, 91 },
    { 0, 0 },
    { 0x4c1b5455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bb5458, 61 },
    { 0x7feb5459, 91 },
    { 0xd10b545a, 90 },
    { 0x74eb545a, 34 },
    { 0, 0 },
    { 0x61db545d, 23 },
    { 0xf59b545d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826b5463, 67 },
    { 0xac5b5464, 90 },
    { 0x28bb5465, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985b5469, 99 },
    { 0, 0 },
    { 0x5e7b546b, 23 },
    { 0xd2db546b, 61 },
    { 0xc4bb546c, 76 },
    { 0x77cb546e, 76 },
    { 0xbd1b546f, 90 },
    { 0x3bdb546e, 56 },
    { 0xa2ab5471, 33 },
    { 0xabab5471, 65 },
    { 0x888b5473, 33 },
    { 0x368b5474, 61 },
    { 0xb9bb5473, 95 },
    { 0xd40b5472, 52 },
    { 0x104b5477, 90 },
    { 0x2c8b5477, 33 },
    { 0, 0 },
    { 0x73b547a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442b5481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x812b5484, 38 },
    { 0x58bb5485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cb548b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb548e, 99 },
    { 0, 0 },
    { 0xa6bb5490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b5497, 87 },
    { 0, 0 },
    { 0xfc8b5499, 33 },
    { 0, 0 },
    { 0x6e8b549b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b549e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb54a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdb54a7, 61 },
    { 0xd37b54a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9b54ac, 23 },
    { 0x4dbb54ad, 78 },
    { 0, 0 },
    { 0xb56b54af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfb54b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b54bb, 43 },
    { 0x9e5b54bb, 55 },
    { 0, 0 },
    { 0x568b54be, 34 },
    { 0, 0 },
    { 0x1f3b54c0, 84 },
    { 0, 0 },
    { 0x5b1b54c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b54c8, 87 },
    { 0x212b54c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab54cd, 61 },
    { 0xfe0b54ce, 90 },
    { 0xd1fb54cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1b54d3, 91 },
    { 0, 0 },
    { 0xdc2b54d5, 67 },
    { 0x927b54d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b54da, 43 },
    { 0xd8eb54da, 100 },
    { 0x3acb54da, 93 },
    { 0, 0 },
    { 0x526b54de, 61 },
    { 0x821b54df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239b54e5, 33 },
    { 0, 0 },
    { 0x37b54e7, 34 },
    { 0x9b0b54e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3b54eb, 34 },
    { 0x368b54eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681b54f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b54f7, 94 },
    { 0, 0 },
    { 0xc9b54f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6b54fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05b54ff, 95 },
    { 0x60ab5500, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50b5503, 75 },
    { 0x1d6b5504, 23 },
    { 0x4f2b5505, 90 },
    { 0x272b5504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb550c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b5512, 23 },
    { 0xb68b5513, 46 },
    { 0x322b5514, 95 },
    { 0, 0 },
    { 0xe5bb5516, 23 },
    { 0x490b5517, 33 },
    { 0x8c9b5518, 99 },
    { 0x654b5517, 90 },
    { 0xcc6b551a, 23 },
    { 0, 0 },
    { 0xbafb551c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55b5521, 66 },
    { 0xe0bb5522, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb5525, 90 },
    { 0xeb1b5525, 78 },
    { 0, 0 },
    { 0x9a3b5528, 77 },
    { 0xaf7b5529, 69 },
    { 0, 0 },
    { 0xd20b552b, 55 },
    { 0x56cb552c, 55 },
    { 0xe85b552d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5b5532, 88 },
    { 0xf41b5533, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b5536, 95 },
    { 0, 0 },
    { 0xddeb5538, 55 },
    { 0x401b5539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4b553c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653b553f, 67 },
    { 0, 0 },
    { 0x9e1b5541, 98 },
    { 0xa32b5541, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cb554a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb5556, 33 },
    { 0x9e4b5557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b555b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59b5560, 61 },
    { 0, 0 },
    { 0x75fb5562, 40 },
    { 0, 0 },
    { 0xc94b5564, 33 },
    { 0x1b7b5565, 61 },
    { 0x1dfb5565, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4b5569, 100 },
    { 0x60b556a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bb556e, 61 },
    { 0x5f4b556e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9b5572, 88 },
    { 0, 0 },
    { 0x420b5574, 61 },
    { 0, 0 },
    { 0x732b5576, 78 },
    { 0x2c0b5577, 98 },
    { 0x649b5578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9b557b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x475b557e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beb5584, 34 },
    { 0x632b5585, 55 },
    { 0x84eb5585, 55 },
    { 0x319b5587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b558a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750b5591, 95 },
    { 0x754b5592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ab5595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298b559c, 94 },
    { 0xb1fb559d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9b55a3, 33 },
    { 0x1eb55a4, 92 },
    { 0x9a9b55a5, 69 },
    { 0, 0 },
    { 0xc5b55a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ab55ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b55ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb55b6, 90 },
    { 0, 0 },
    { 0x428b55b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542b55bc, 43 },
    { 0, 0 },
    { 0x395b55be, 61 },
    { 0, 0 },
    { 0x1dfb55c0, 34 },
    { 0x382b55c0, 33 },
    { 0xf50b55c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x245b55c5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270b55c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848b55ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb55d6, 90 },
    { 0, 0 },
    { 0xdc0b55d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfb55dc, 61 },
    { 0x3b0b55dd, 17 },
    { 0xd35b55dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b55e4, 76 },
    { 0x2f7b55e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cb55e8, 95 },
    { 0x8cdb55e9, 33 },
    { 0x4e5b55ea, 23 },
    { 0, 0 },
    { 0x275b55ec, 52 },
    { 0xbf5b55ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b55f0, 98 },
    { 0xd28b55f1, 98 },
    { 0x78cb55f1, 76 },
    { 0, 0 },
    { 0x94bb55f4, 93 },
    { 0, 0 },
    { 0x595b55f6, 12 },
    { 0x7c0b55f7, 34 },
    { 0, 0 },
    { 0x293b55f9, 95 },
    { 0x351b55f9, 33 },
    { 0xb3fb55fb, 95 },
    { 0xefeb55fc, 61 },
    { 0xc14b55fd, 23 },
    { 0xbb9b55fd, 56 },
    { 0x61db55fd, 95 },
    { 0x715b55ff, 23 },
    { 0xa59b55fd, 90 },
    { 0, 0 },
    { 0x916b5603, 46 },
    { 0, 0 },
    { 0x362b5605, 67 },
    { 0xff0b5605, 46 },
    { 0, 0 },
    { 0x9c1b5608, 98 },
    { 0x508b5609, 23 },
    { 0x635b560a, 95 },
    { 0xb1fb560a, 61 },
    { 0, 0 },
    { 0x52eb560d, 33 },
    { 0, 0 },
    { 0x2d4b560f, 81 },
    { 0, 0 },
    { 0xabdb5611, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x760b5614, 61 },
    { 0xed5b5615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480b561a, 98 },
    { 0xa7b561a, 33 },
    { 0x303b561c, 95 },
    { 0x40fb561d, 91 },
    { 0xd47b561d, 95 },
    { 0, 0 },
    { 0xf8bb5620, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35b5623, 98 },
    { 0xc0db5624, 33 },
    { 0x618b5625, 55 },
    { 0, 0 },
    { 0xd4ab5627, 33 },
    { 0, 0 },
    { 0xd81b5629, 76 },
    { 0x220b562a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb562d, 33 },
    { 0, 0 },
    { 0x9f5b562f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b5632, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x817b5635, 55 },
    { 0x882b5636, 98 },
    { 0, 0 },
    { 0xf8fb5638, 81 },
    { 0, 0 },
    { 0x24bb563a, 98 },
    { 0xe18b563a, 23 },
    { 0x849b563c, 77 },
    { 0x959b563a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b5640, 61 },
    { 0xb3eb5641, 81 },
    { 0x470b5640, 96 },
    { 0x728b5641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154b564e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab5653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377b5658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eb5661, 84 },
    { 0x576b5662, 76 },
    { 0x36bb5661, 98 },
    { 0xe60b5664, 84 },
    { 0x7b5b5664, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb566c, 93 },
    { 0xe24b566d, 61 },
    { 0xa70b566c, 32 },
    { 0x95bb566c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7db5675, 78 },
    { 0xa41b5675, 98 },
    { 0x558b5677, 90 },
    { 0x429b5677, 87 },
    { 0x76ab5678, 76 },
    { 0xe5cb5675, 61 },
    { 0x3b5b5675, 85 },
    { 0x523b567c, 95 },
    { 0x92fb5678, 90 },
    { 0x7f2b5679, 33 },
    { 0x539b567e, 100 },
    { 0xcfcb5680, 33 },
    { 0, 0 },
    { 0x358b5682, 100 },
    { 0xa52b5683, 90 },
    { 0, 0 },
    { 0xb1fb5685, 90 },
    { 0xe71b5686, 76 },
    { 0xae7b5687, 77 },
    { 0x49cb5686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb568b, 96 },
    { 0, 0 },
    { 0x14fb568d, 95 },
    { 0x44cb568e, 33 },
    { 0xb5b568f, 98 },
    { 0x2d2b5690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4b5694, 76 },
    { 0x7afb5694, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b569d, 98 },
    { 0x193b569e, 70 },
    { 0xd41b569f, 98 },
    { 0, 0 },
    { 0xa1cb56a1, 33 },
    { 0x11cb56a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacab56a6, 68 },
    { 0xf66b56a7, 55 },
    { 0x75db56a8, 33 },
    { 0xb2db56a7, 90 },
    { 0xbdab56a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833b56b0, 33 },
    { 0x985b56b0, 95 },
    { 0x11cb56b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db56b8, 95 },
    { 0x72fb56b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bb56bd, 90 },
    { 0x22fb56bd, 34 },
    { 0x6b0b56be, 90 },
    { 0x497b56c0, 33 },
    { 0, 0 },
    { 0x23cb56c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db56c7, 46 },
    { 0, 0 },
    { 0xf6cb56c9, 93 },
    { 0x3c1b56ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb56d4, 95 },
    { 0xe70b56d4, 33 },
    { 0, 0 },
    { 0x780b56d7, 61 },
    { 0, 0 },
    { 0x155b56d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb56de, 61 },
    { 0x53db56de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19b56e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b56e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab56e9, 61 },
    { 0x292b56ea, 90 },
    { 0x859b56eb, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b56f2, 90 },
    { 0xfa3b56f3, 34 },
    { 0xbd7b56f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b56f7, 43 },
    { 0xa1b56f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772b56fc, 98 },
    { 0, 0 },
    { 0xd3bb56fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b5702, 98 },
    { 0xa57b5703, 33 },
    { 0x332b5704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ab5707, 33 },
    { 0, 0 },
    { 0x2d1b5709, 33 },
    { 0, 0 },
    { 0xb4b570b, 90 },
    { 0x8cb570c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b5711, 76 },
    { 0, 0 },
    { 0x8ffb5713, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x855b5716, 23 },
    { 0x6dfb5716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226b571b, 33 },
    { 0x5ccb571c, 95 },
    { 0xe21b571d, 75 },
    { 0x9e0b571d, 46 },
    { 0xa1b571f, 98 },
    { 0x873b571c, 34 },
    { 0x82ab5721, 55 },
    { 0x792b5722, 33 },
    { 0x584b571b, 33 },
    { 0x6c9b5724, 55 },
    { 0xffdb5725, 23 },
    { 0, 0 },
    { 0x70fb5727, 34 },
    { 0x1e8b5728, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06b572b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b5731, 52 },
    { 0, 0 },
    { 0xc81b5733, 77 },
    { 0xdc6b5733, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb5744, 98 },
    { 0x390b5744, 98 },
    { 0, 0 },
    { 0xa5eb5747, 23 },
    { 0xcbdb5748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b574b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68db574f, 98 },
    { 0x9f2b5750, 66 },
    { 0x521b5751, 23 },
    { 0x22b5752, 34 },
    { 0x8ccb5753, 46 },
    { 0, 0 },
    { 0x2aab5755, 55 },
    { 0xa66b5755, 33 },
    { 0x4aab5757, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bb575b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb575f, 100 },
    { 0xa38b575f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b576a, 90 },
    { 0xc92b576b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ab5770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ab5773, 95 },
    { 0x72bb5773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b577c, 98 },
    { 0x99fb577c, 33 },
    { 0xdd7b577e, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b5783, 76 },
    { 0xd63b5784, 46 },
    { 0, 0 },
    { 0x8fcb5786, 95 },
    { 0xb4b5786, 95 },
    { 0, 0 },
    { 0xa98b5789, 98 },
    { 0x5f3b578a, 64 },
    { 0x4bab578b, 61 },
    { 0, 0 },
    { 0x61b578d, 61 },
    { 0, 0 },
    { 0xbecb578f, 33 },
    { 0x69b5790, 33 },
    { 0x29eb578f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313b5798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x803b579b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54b579f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b57a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379b57ae, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fb57b5, 101 },
    { 0x926b57b5, 98 },
    { 0x17eb57b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb57ba, 76 },
    { 0, 0 },
    { 0x578b57bc, 98 },
    { 0x67ab57bd, 34 },
    { 0xf7ab57be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680b57c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fb57ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83b57d1, 43 },
    { 0xbccb57d1, 34 },
    { 0, 0 },
    { 0xb11b57d4, 77 },
    { 0xd8db57d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb57d9, 90 },
    { 0xaf9b57da, 90 },
    { 0x53db57db, 23 },
    { 0x80fb57dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abb57e3, 95 },
    { 0, 0 },
    { 0x9fdb57e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7b57e9, 34 },
    { 0, 0 },
    { 0xee7b57eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9b57ee, 34 },
    { 0, 0 },
    { 0x3fbb57f0, 90 },
    { 0x2bab57f0, 66 },
    { 0x9ab57f2, 87 },
    { 0x2a1b57f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb57f9, 90 },
    { 0x8cfb57fa, 93 },
    { 0x518b57fa, 76 },
    { 0xae8b57fc, 61 },
    { 0xc88b57fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46b5801, 34 },
    { 0, 0 },
    { 0xdbdb5803, 61 },
    { 0x70fb5804, 76 },
    { 0x704b5805, 23 },
    { 0xf90b5803, 98 },
    { 0x88bb5807, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bb580d, 88 },
    { 0x841b580e, 43 },
    { 0x709b580e, 90 },
    { 0x2b0b5810, 87 },
    { 0, 0 },
    { 0x1c8b5812, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341b5816, 46 },
    { 0xc98b5816, 92 },
    { 0x4a6b5816, 23 },
    { 0x675b5816, 65 },
    { 0x9dab581a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86b581f, 90 },
    { 0xf8ab5820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b582b, 93 },
    { 0, 0 },
    { 0xa81b582d, 61 },
    { 0x739b582e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9b5833, 43 },
    { 0x1deb5834, 75 },
    { 0, 0 },
    { 0x334b5836, 88 },
    { 0xe23b5837, 90 },
    { 0xc39b5836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b583c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b5841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0b5845, 40 },
    { 0, 0 },
    { 0x26fb5847, 61 },
    { 0, 0 },
    { 0xba8b5849, 54 },
    { 0x383b5849, 90 },
    { 0x47b584b, 55 },
    { 0x6abb584c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ab5850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39b5857, 43 },
    { 0x31ab5857, 33 },
    { 0xf41b5858, 34 },
    { 0xdfab585a, 43 },
    { 0x2b8b585b, 95 },
    { 0x378b585a, 61 },
    { 0xd2eb5857, 98 },
    { 0, 0 },
    { 0xc2cb585f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b5862, 95 },
    { 0x2ccb5863, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb5868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dab5870, 33 },
    { 0xef7b5871, 39 },
    { 0xaf4b5870, 34 },
    { 0x3a7b5873, 33 },
    { 0xb33b5874, 58 },
    { 0x4c3b5875, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827b5880, 61 },
    { 0, 0 },
    { 0x49b5882, 33 },
    { 0x7c5b5883, 55 },
    { 0xb2cb5884, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cb5897, 78 },
    { 0xb4fb5898, 84 },
    { 0x852b5899, 33 },
    { 0x555b589a, 91 },
    { 0xe16b589a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fb58a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab58a3, 77 },
    { 0x397b58a3, 100 },
    { 0xd0bb58a5, 61 },
    { 0x9eab58a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b58ac, 61 },
    { 0xa8b58ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b58b0, 81 },
    { 0x3fbb58b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b58bc, 90 },
    { 0, 0 },
    { 0x1f8b58be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbb58c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66b58c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b58cb, 34 },
    { 0x5e7b58cc, 93 },
    { 0x5a6b58cd, 66 },
    { 0, 0 },
    { 0x11ab58cf, 52 },
    { 0, 0 },
    { 0x148b58d1, 33 },
    { 0xcc0b58d2, 33 },
    { 0xcfcb58d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17b58d8, 46 },
    { 0xa24b58d9, 95 },
    { 0xb99b58da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64b58dd, 90 },
    { 0, 0 },
    { 0x812b58df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939b58e9, 52 },
    { 0x611b58e9, 95 },
    { 0xaa3b58eb, 90 },
    { 0x81cb58ec, 61 },
    { 0, 0 },
    { 0x65eb58ee, 90 },
    { 0xf9ab58ef, 55 },
    { 0xb6ab58f0, 61 },
    { 0x3b7b58ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b58f4, 98 },
    { 0x86b58f5, 99 },
    { 0xaf2b58f5, 90 },
    { 0xda3b58f7, 53 },
    { 0xbb58f8, 53 },
    { 0x3eb58f6, 98 },
    { 0x22bb58f4, 46 },
    { 0x77eb58f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb58fe, 55 },
    { 0, 0 },
    { 0xe7cb5900, 33 },
    { 0x958b5901, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deb5905, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b5909, 98 },
    { 0, 0 },
    { 0x52ab590b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb590f, 90 },
    { 0x77eb590f, 40 },
    { 0x908b590f, 61 },
    { 0xa9eb5912, 23 },
    { 0xb81b5913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b5916, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81b591a, 33 },
    { 0xdd2b591b, 43 },
    { 0, 0 },
    { 0xd60b591d, 23 },
    { 0xe6fb591e, 93 },
    { 0xf43b591e, 76 },
    { 0x192b591d, 33 },
    { 0x994b5920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eb5925, 98 },
    { 0xf5ab5926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84b5929, 85 },
    { 0x682b592a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0b592d, 95 },
    { 0xc61b592e, 46 },
    { 0, 0 },
    { 0xe2ab5930, 23 },
    { 0x455b5931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4b5935, 43 },
    { 0x94b5935, 61 },
    { 0x1e6b5936, 61 },
    { 0x2db5938, 98 },
    { 0x83ab5939, 52 },
    { 0x7c2b593a, 77 },
    { 0, 0 },
    { 0xb12b593c, 61 },
    { 0, 0 },
    { 0xd70b593e, 85 },
    { 0x8b593f, 69 },
    { 0xa76b5940, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b5944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeb5950, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbb5953, 98 },
    { 0x647b5954, 40 },
    { 0x702b5955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34b5958, 69 },
    { 0xe00b5959, 90 },
    { 0x100b5959, 68 },
    { 0xa91b595a, 61 },
    { 0x5dbb595c, 90 },
    { 0xa6cb595d, 23 },
    { 0xc50b595b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8b5964, 33 },
    { 0, 0 },
    { 0xf6fb5966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eb596a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6b5970, 95 },
    { 0x22b5970, 52 },
    { 0, 0 },
    { 0x1c8b5973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0b5977, 61 },
    { 0x223b5978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb597c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb597f, 23 },
    { 0x34cb5980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8b5989, 23 },
    { 0, 0 },
    { 0xd6db598b, 90 },
    { 0x752b598b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b598f, 61 },
    { 0xc5db5990, 40 },
    { 0, 0 },
    { 0x709b5992, 61 },
    { 0, 0 },
    { 0x920b5994, 23 },
    { 0, 0 },
    { 0xa90b5996, 88 },
    { 0x909b5996, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deb599a, 99 },
    { 0x5b2b599b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b59a4, 91 },
    { 0x7ceb59a4, 33 },
    { 0, 0 },
    { 0x6e9b59a7, 91 },
    { 0x719b59a8, 87 },
    { 0, 0 },
    { 0x613b59aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aab59b2, 52 },
    { 0x596b59b2, 17 },
    { 0x463b59b4, 84 },
    { 0x2f0b59b5, 66 },
    { 0x589b59b4, 55 },
    { 0x3ecb59b7, 92 },
    { 0, 0 },
    { 0x4d2b59b9, 100 },
    { 0x9a9b59ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x970b59bd, 98 },
    { 0x5a0b59bd, 51 },
    { 0x56fb59bf, 90 },
    { 0x672b59c0, 23 },
    { 0x5c1b59c1, 33 },
    { 0x8b5b59c1, 95 },
    { 0xf6ab59c3, 48 },
    { 0xe5eb59c3, 46 },
    { 0x986b59c3, 23 },
    { 0x3c1b59c3, 90 },
    { 0x67eb59c7, 61 },
    { 0xd7b59c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db59cc, 46 },
    { 0, 0 },
    { 0xb8ab59ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b59d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb59d5, 95 },
    { 0, 0 },
    { 0x742b59d7, 33 },
    { 0xe5cb59d8, 23 },
    { 0x599b59d9, 61 },
    { 0, 0 },
    { 0x399b59db, 100 },
    { 0x7eb59db, 43 },
    { 0, 0 },
    { 0x4cbb59de, 76 },
    { 0, 0 },
    { 0xa6db59e0, 66 },
    { 0xef0b59e0, 33 },
    { 0x29ab59e1, 46 },
    { 0x4cfb59e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9b59e6, 98 },
    { 0, 0 },
    { 0x865b59e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab59ed, 61 },
    { 0, 0 },
    { 0xee5b59ef, 98 },
    { 0x49bb59ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ab59f5, 33 },
    { 0x1b2b59f6, 33 },
    { 0xe9eb59f7, 90 },
    { 0xe7eb59f8, 98 },
    { 0x43bb59f6, 95 },
    { 0xf44b59fa, 98 },
    { 0, 0 },
    { 0x5f7b59fc, 55 },
    { 0, 0 },
    { 0x607b59fe, 78 },
    { 0x43eb59ff, 9 },
    { 0x316b59ff, 53 },
    { 0x6d6b59fe, 43 },
    { 0x353b5a02, 23 },
    { 0xb1eb5a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b5a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167b5a0c, 33 },
    { 0x515b5a0d, 98 },
    { 0xcccb5a0c, 90 },
    { 0xad7b5a0f, 83 },
    { 0x68eb5a0f, 90 },
    { 0x996b5a10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b5a14, 89 },
    { 0x248b5a15, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173b5a19, 78 },
    { 0x6d1b5a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb5a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b5a23, 83 },
    { 0x652b5a23, 43 },
    { 0x214b5a25, 76 },
    { 0xd03b5a26, 100 },
    { 0xba6b5a27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b5a2b, 85 },
    { 0x8b5a2c, 90 },
    { 0x843b5a2d, 98 },
    { 0xfdab5a2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b5a3a, 90 },
    { 0, 0 },
    { 0x697b5a3c, 67 },
    { 0, 0 },
    { 0x1a0b5a3e, 99 },
    { 0xa12b5a3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x344b5a42, 99 },
    { 0x27fb5a42, 61 },
    { 0x3c0b5a43, 87 },
    { 0x55bb5a45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22b5a49, 81 },
    { 0x370b5a4a, 90 },
    { 0x52fb5a4b, 90 },
    { 0, 0 },
    { 0x6d5b5a4d, 67 },
    { 0, 0 },
    { 0x8a0b5a4f, 87 },
    { 0x7dcb5a50, 98 },
    { 0x37b5a51, 84 },
    { 0, 0 },
    { 0x788b5a53, 55 },
    { 0, 0 },
    { 0xb4eb5a55, 96 },
    { 0x8d5b5a55, 61 },
    { 0xd63b5a57, 94 },
    { 0xe96b5a55, 34 },
    { 0xeb5a59, 61 },
    { 0x673b5a59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17b5a5d, 76 },
    { 0, 0 },
    { 0xfe8b5a5f, 98 },
    { 0, 0 },
    { 0x771b5a61, 14 },
    { 0x43ab5a62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ab5a65, 90 },
    { 0x47db5a66, 95 },
    { 0x890b5a66, 91 },
    { 0x6dcb5a68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b5a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ab5a70, 61 },
    { 0x18fb5a71, 79 },
    { 0, 0 },
    { 0xf92b5a73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b5a76, 90 },
    { 0, 0 },
    { 0x17db5a78, 61 },
    { 0x56fb5a78, 90 },
    { 0xeb8b5a7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb5a7e, 43 },
    { 0x20eb5a7f, 92 },
    { 0, 0 },
    { 0xeb9b5a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5a85, 61 },
    { 0x1acb5a86, 84 },
    { 0x461b5a85, 61 },
    { 0, 0 },
    { 0xa14b5a89, 99 },
    { 0xd2db5a89, 95 },
    { 0, 0 },
    { 0x42b5a8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b5a90, 95 },
    { 0, 0 },
    { 0x2fab5a92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6b5a95, 81 },
    { 0, 0 },
    { 0xdbcb5a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2b5a9a, 33 },
    { 0, 0 },
    { 0x5f9b5a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b5a9f, 94 },
    { 0, 0 },
    { 0x130b5aa1, 78 },
    { 0xef4b5aa1, 23 },
    { 0, 0 },
    { 0xffcb5aa4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afb5aa7, 76 },
    { 0xe03b5aa8, 93 },
    { 0x476b5aa9, 23 },
    { 0, 0 },
    { 0xf8fb5aab, 95 },
    { 0, 0 },
    { 0x200b5aad, 78 },
    { 0x465b5aad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b5ab4, 95 },
    { 0x94fb5ab5, 76 },
    { 0x2d9b5ab6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ab5ab9, 61 },
    { 0x5beb5ab9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0b5ac1, 90 },
    { 0xe89b5ac2, 17 },
    { 0x13fb5ac1, 61 },
    { 0x387b5ac4, 78 },
    { 0x549b5ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b5acd, 98 },
    { 0xe7bb5ace, 40 },
    { 0x17bb5acf, 98 },
    { 0x1deb5ace, 61 },
    { 0xc4fb5ad1, 83 },
    { 0x4fab5ad2, 23 },
    { 0x614b5ace, 95 },
    { 0, 0 },
    { 0x100b5ad5, 88 },
    { 0xd58b5ad6, 46 },
    { 0x9b3b5ad7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b5ada, 95 },
    { 0x650b5adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8b5adf, 85 },
    { 0, 0 },
    { 0x623b5ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb5ae4, 52 },
    { 0, 0 },
    { 0x69eb5ae6, 95 },
    { 0, 0 },
    { 0x647b5ae8, 99 },
    { 0, 0 },
    { 0xa4fb5aea, 90 },
    { 0xe37b5aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87b5aee, 55 },
    { 0x516b5aef, 61 },
    { 0xfd0b5af0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b5af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b5afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb5b00, 33 },
    { 0xc09b5b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cb5b04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb5b12, 23 },
    { 0xce9b5b13, 55 },
    { 0xa60b5b14, 40 },
    { 0xbfeb5b15, 55 },
    { 0x1b5b16, 88 },
    { 0x69b5b16, 61 },
    { 0x462b5b18, 43 },
    { 0xc50b5b16, 55 },
    { 0xfd6b5b18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bb5b22, 77 },
    { 0x6feb5b23, 98 },
    { 0x83b5b24, 90 },
    { 0xc18b5b24, 98 },
    { 0xca4b5b26, 98 },
    { 0x429b5b27, 98 },
    { 0xb7eb5b28, 61 },
    { 0x4d0b5b27, 66 },
    { 0x9c3b5b2a, 90 },
    { 0x6d4b5b29, 100 },
    { 0x6eeb5b2b, 90 },
    { 0x866b5b2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab5b30, 65 },
    { 0, 0 },
    { 0x11db5b32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8b5b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5b5b3a, 61 },
    { 0x6f6b5b3b, 43 },
    { 0x96b5b3c, 61 },
    { 0xe17b5b3d, 33 },
    { 0x424b5b3e, 91 },
    { 0x62fb5b3f, 23 },
    { 0x869b5b3c, 36 },
    { 0xd3bb5b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cb5b44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b5b47, 55 },
    { 0xe0cb5b48, 55 },
    { 0, 0 },
    { 0xe16b5b4a, 93 },
    { 0x271b5b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b5b4f, 61 },
    { 0xee3b5b50, 33 },
    { 0, 0 },
    { 0x197b5b52, 90 },
    { 0xf0bb5b53, 34 },
    { 0x915b5b53, 52 },
    { 0, 0 },
    { 0x257b5b56, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb5b65, 90 },
    { 0, 0 },
    { 0x352b5b67, 87 },
    { 0x3c3b5b68, 33 },
    { 0, 0 },
    { 0x1aab5b6a, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feb5b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb5b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47b5b76, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8b5b7a, 61 },
    { 0, 0 },
    { 0x717b5b7c, 23 },
    { 0x33db5b7d, 33 },
    { 0x73b5b7e, 33 },
    { 0xe97b5b7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8b5b83, 43 },
    { 0xf3bb5b83, 33 },
    { 0x947b5b85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5b5b88, 90 },
    { 0x14b5b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb5b8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384b5b99, 95 },
    { 0, 0 },
    { 0x349b5b9b, 43 },
    { 0x490b5b9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab5ba4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cb5ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b5bac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53b5baf, 61 },
    { 0xe4ab5bb0, 34 },
    { 0x836b5bb1, 76 },
    { 0x8d5b5baf, 40 },
    { 0x489b5bb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b5bb7, 100 },
    { 0x2e3b5bb8, 46 },
    { 0, 0 },
    { 0x5b5b5bba, 90 },
    { 0, 0 },
    { 0xad6b5bbc, 90 },
    { 0, 0 },
    { 0x681b5bbe, 78 },
    { 0x7ccb5bbf, 61 },
    { 0, 0 },
    { 0x517b5bc1, 93 },
    { 0, 0 },
    { 0x1e7b5bc3, 33 },
    { 0xeccb5bc3, 23 },
    { 0, 0 },
    { 0xc7b5bc6, 66 },
    { 0xe96b5bc7, 46 },
    { 0x383b5bc8, 34 },
    { 0xd09b5bc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x752b5bcc, 66 },
    { 0xa96b5bcd, 61 },
    { 0x112b5bce, 90 },
    { 0x95fb5bcf, 40 },
    { 0xbf0b5bcf, 90 },
    { 0x4edb5bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb5bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07b5bda, 90 },
    { 0xb54b5bda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb5bde, 98 },
    { 0xa5db5bde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95b5be2, 95 },
    { 0, 0 },
    { 0x3e4b5be4, 90 },
    { 0xdb6b5be5, 76 },
    { 0xc68b5be6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb5bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b5bfb, 34 },
    { 0x669b5bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b5bff, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22b5c0b, 33 },
    { 0x38bb5c0c, 33 },
    { 0x614b5c0d, 76 },
    { 0xcc7b5c0e, 95 },
    { 0, 0 },
    { 0xcbfb5c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7b5c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb5c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2b5c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732b5c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b5c28, 67 },
    { 0x55fb5c29, 87 },
    { 0, 0 },
    { 0x5ebb5c2b, 98 },
    { 0x22bb5c2c, 67 },
    { 0x8d5b5c2d, 53 },
    { 0x750b5c2b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb5c33, 43 },
    { 0x3f6b5c34, 33 },
    { 0xc2bb5c35, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb5c39, 83 },
    { 0, 0 },
    { 0xd45b5c3b, 33 },
    { 0x9d9b5c3c, 23 },
    { 0x737b5c3c, 93 },
    { 0xfecb5c3e, 23 },
    { 0xf34b5c3e, 23 },
    { 0, 0 },
    { 0x284b5c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5c44, 98 },
    { 0x382b5c45, 90 },
    { 0x10b5c44, 90 },
    { 0xd63b5c47, 34 },
    { 0xda9b5c44, 33 },
    { 0xefcb5c49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b5c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab5c52, 42 },
    { 0x2edb5c53, 66 },
    { 0x68db5c52, 98 },
    { 0xea8b5c55, 33 },
    { 0xfa9b5c56, 77 },
    { 0x44ab5c56, 61 },
    { 0x598b5c56, 90 },
    { 0x916b5c59, 95 },
    { 0x386b5c5a, 52 },
    { 0, 0 },
    { 0x819b5c5c, 52 },
    { 0x16cb5c5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b5c60, 61 },
    { 0, 0 },
    { 0x117b5c62, 52 },
    { 0x3a8b5c63, 95 },
    { 0x9ab5c64, 93 },
    { 0x138b5c64, 77 },
    { 0xe65b5c66, 66 },
    { 0xc9bb5c67, 56 },
    { 0x85b5c68, 23 },
    { 0xa5db5c69, 93 },
    { 0xdfcb5c6a, 33 },
    { 0x427b5c67, 33 },
    { 0x386b5c67, 23 },
    { 0, 0 },
    { 0x55cb5c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b5c71, 23 },
    { 0, 0 },
    { 0x3dcb5c73, 23 },
    { 0x4cb5c73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26b5c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791b5c7b, 96 },
    { 0, 0 },
    { 0xaf8b5c7d, 95 },
    { 0, 0 },
    { 0x2aeb5c7f, 23 },
    { 0, 0 },
    { 0x526b5c81, 23 },
    { 0x872b5c82, 61 },
    { 0x59fb5c83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862b5c8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3b5c8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1b5c92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aab5c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331b5c99, 88 },
    { 0, 0 },
    { 0x189b5c9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b5c9f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0b5ca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cb5ca9, 61 },
    { 0x5aab5caa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795b5cb3, 34 },
    { 0, 0 },
    { 0xf42b5cb5, 98 },
    { 0xeb9b5cb5, 61 },
    { 0x249b5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485b5cbb, 76 },
    { 0, 0 },
    { 0xbcb5cbd, 61 },
    { 0xc2fb5cbd, 23 },
    { 0, 0 },
    { 0x567b5cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cb5cc5, 66 },
    { 0xf63b5cc5, 33 },
    { 0xf7bb5cc6, 90 },
    { 0, 0 },
    { 0x8fb5cc9, 100 },
    { 0x615b5cc9, 98 },
    { 0x8f2b5cc9, 87 },
    { 0x344b5cca, 43 },
    { 0x4e5b5ccb, 90 },
    { 0xbe1b5cce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecb5cd2, 76 },
    { 0xd59b5cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdb5cd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3b5cdc, 95 },
    { 0x554b5cdd, 33 },
    { 0, 0 },
    { 0x713b5cdf, 88 },
    { 0, 0 },
    { 0x8cbb5ce1, 97 },
    { 0, 0 },
    { 0x8f7b5ce3, 34 },
    { 0xa8eb5ce4, 68 },
    { 0xce2b5ce5, 52 },
    { 0x1e3b5ce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b5ce9, 23 },
    { 0xf95b5cea, 95 },
    { 0, 0 },
    { 0x250b5cec, 43 },
    { 0x5bdb5ced, 46 },
    { 0x4deb5cee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866b5cf9, 76 },
    { 0x33fb5cfa, 90 },
    { 0x6eb5cf9, 95 },
    { 0x1ceb5cfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b5d00, 76 },
    { 0, 0 },
    { 0xd9ab5d02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b5d08, 33 },
    { 0, 0 },
    { 0xddcb5d0a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab5d13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b5d1d, 85 },
    { 0, 0 },
    { 0x32b5d1f, 95 },
    { 0, 0 },
    { 0x713b5d21, 52 },
    { 0x964b5d21, 53 },
    { 0, 0 },
    { 0x45fb5d24, 83 },
    { 0x8b3b5d24, 76 },
    { 0, 0 },
    { 0xd1db5d27, 100 },
    { 0xf56b5d28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5d2e, 93 },
    { 0xd9ab5d2e, 33 },
    { 0x94b5d30, 23 },
    { 0xdbfb5d31, 87 },
    { 0x5f4b5d2e, 61 },
    { 0x986b5d33, 90 },
    { 0xf58b5d2e, 61 },
    { 0x54ab5d30, 90 },
    { 0xf9b5d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b5d3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b5d3d, 23 },
    { 0xafcb5d3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b5d49, 46 },
    { 0, 0 },
    { 0xc83b5d4b, 33 },
    { 0x614b5d4b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8db5d50, 33 },
    { 0x70fb5d51, 61 },
    { 0, 0 },
    { 0x687b5d53, 90 },
    { 0x13cb5d54, 77 },
    { 0xe62b5d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15b5d58, 88 },
    { 0x642b5d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b5d5c, 90 },
    { 0x376b5d5d, 55 },
    { 0xdaab5d5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db5d65, 64 },
    { 0xd21b5d65, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b5d6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5d70, 95 },
    { 0x949b5d70, 23 },
    { 0xf23b5d72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b5d75, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133b5d79, 61 },
    { 0, 0 },
    { 0x810b5d7b, 78 },
    { 0, 0 },
    { 0x145b5d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b5d85, 90 },
    { 0x9db5d85, 46 },
    { 0xd49b5d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb5d8e, 98 },
    { 0x9abb5d8e, 98 },
    { 0, 0 },
    { 0x279b5d91, 55 },
    { 0, 0 },
    { 0xdb4b5d93, 23 },
    { 0, 0 },
    { 0x184b5d95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18b5d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b5da2, 17 },
    { 0xeeb5da3, 81 },
    { 0, 0 },
    { 0xe4eb5da5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb5dab, 84 },
    { 0x15bb5dab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57b5daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86b5db2, 61 },
    { 0xfbb5db2, 92 },
    { 0x9d5b5db4, 95 },
    { 0x63ab5db2, 40 },
    { 0x85cb5db6, 23 },
    { 0, 0 },
    { 0xf3ab5db8, 90 },
    { 0x92ab5db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x449b5dbc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b5dc8, 90 },
    { 0, 0 },
    { 0xe1db5dca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5b5dcd, 33 },
    { 0x2a7b5dcd, 98 },
    { 0, 0 },
    { 0x451b5dd0, 23 },
    { 0xf06b5dd1, 61 },
    { 0, 0 },
    { 0x287b5dd3, 77 },
    { 0xb4b5dd4, 98 },
    { 0x5f0b5dd3, 75 },
    { 0xd5db5dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641b5dd9, 82 },
    { 0, 0 },
    { 0xc62b5ddb, 99 },
    { 0x703b5ddb, 95 },
    { 0xb75b5ddd, 23 },
    { 0xcc0b5dde, 90 },
    { 0xeffb5dde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x729b5de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b5de5, 66 },
    { 0xa83b5de6, 55 },
    { 0x198b5de7, 98 },
    { 0xca4b5de7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b5deb, 61 },
    { 0x2bcb5deb, 55 },
    { 0xf77b5dec, 95 },
    { 0xc91b5dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefab5df1, 99 },
    { 0, 0 },
    { 0xd04b5df3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bb5e02, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb5e05, 95 },
    { 0x10b5e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b5e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b5e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b5e13, 52 },
    { 0xe07b5e14, 87 },
    { 0x294b5e14, 33 },
    { 0x3db5e13, 61 },
    { 0xdbb5e17, 90 },
    { 0x200b5e14, 90 },
    { 0x107b5e19, 69 },
    { 0xe10b5e1a, 90 },
    { 0x904b5e1b, 38 },
    { 0x208b5e19, 52 },
    { 0xd50b5e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb5e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5e23, 84 },
    { 0, 0 },
    { 0x277b5e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b5e29, 66 },
    { 0x176b5e2a, 98 },
    { 0xbf3b5e29, 33 },
    { 0x208b5e2b, 23 },
    { 0xbf4b5e2b, 90 },
    { 0x4cfb5e2a, 95 },
    { 0x971b5e2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1b5e33, 97 },
    { 0x290b5e34, 68 },
    { 0xf72b5e34, 34 },
    { 0, 0 },
    { 0x7c5b5e37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eb5e3b, 33 },
    { 0xd78b5e3b, 98 },
    { 0x4d8b5e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b5e42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1b5e46, 91 },
    { 0xbddb5e47, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b5e4d, 90 },
    { 0xf94b5e4d, 76 },
    { 0, 0 },
    { 0x86bb5e50, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b5e59, 77 },
    { 0xfd6b5e5a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b5e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb5e6b, 95 },
    { 0x56b5e6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eb5e72, 33 },
    { 0x686b5e72, 55 },
    { 0xa16b5e74, 76 },
    { 0x9f6b5e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b5e78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8b5e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19b5e88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b5e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b5e99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb5e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x733b5e9f, 90 },
    { 0x11fb5ea0, 55 },
    { 0xe2db5ea1, 22 },
    { 0x39ab5ea2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecb5ea5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b5ea9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecb5ead, 23 },
    { 0, 0 },
    { 0x2f2b5eaf, 33 },
    { 0, 0 },
    { 0x2abb5eb1, 96 },
    { 0, 0 },
    { 0x692b5eb3, 95 },
    { 0, 0 },
    { 0x933b5eb5, 36 },
    { 0x18ab5eb6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbb5eb9, 90 },
    { 0x6f8b5eb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b5ebd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61b5ec9, 80 },
    { 0xec6b5eca, 46 },
    { 0x7abb5ecb, 88 },
    { 0x851b5ecb, 66 },
    { 0xa2bb5ecd, 23 },
    { 0xbbbb5eca, 99 },
    { 0, 0 },
    { 0x987b5ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b5ed3, 98 },
    { 0, 0 },
    { 0x5a6b5ed5, 81 },
    { 0, 0 },
    { 0x95b5ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcb5eda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffb5ede, 76 },
    { 0xac3b5edf, 90 },
    { 0x6cfb5ee0, 61 },
    { 0, 0 },
    { 0xeefb5ee2, 98 },
    { 0, 0 },
    { 0x140b5ee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db5ee9, 76 },
    { 0x831b5eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b5ef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ab5ef4, 98 },
    { 0xc1fb5ef4, 55 },
    { 0, 0 },
    { 0x69fb5ef7, 90 },
    { 0xb1ab5ef7, 43 },
    { 0xaa5b5ef9, 90 },
    { 0xbd6b5ef8, 95 },
    { 0, 0 },
    { 0x9cdb5efc, 67 },
    { 0xc1fb5efd, 43 },
    { 0, 0 },
    { 0xdc7b5eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410b5f04, 52 },
    { 0x340b5f05, 52 },
    { 0, 0 },
    { 0x510b5f07, 85 },
    { 0xb65b5f08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafeb5f0b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x395b5f0e, 33 },
    { 0, 0 },
    { 0x589b5f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb5f16, 17 },
    { 0, 0 },
    { 0x400b5f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5b5f1c, 23 },
    { 0x5e9b5f1d, 90 },
    { 0xbcab5f1c, 90 },
    { 0xc7eb5f1f, 33 },
    { 0, 0 },
    { 0xe3b5f21, 95 },
    { 0x7bdb5f22, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6b5f29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642b5f31, 67 },
    { 0, 0 },
    { 0x48bb5f33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b5f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b5f3c, 87 },
    { 0xc1fb5f3c, 98 },
    { 0, 0 },
    { 0x52bb5f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb5f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb5f45, 90 },
    { 0xe59b5f46, 90 },
    { 0xa7cb5f47, 33 },
    { 0, 0 },
    { 0xc26b5f49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b5f51, 88 },
    { 0xce9b5f51, 76 },
    { 0, 0 },
    { 0xa2fb5f54, 23 },
    { 0, 0 },
    { 0xa5eb5f56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b5f59, 61 },
    { 0x78db5f5a, 23 },
    { 0x1c1b5f5b, 78 },
    { 0x319b5f5b, 61 },
    { 0x6d5b5f5a, 90 },
    { 0x5e8b5f5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab5f61, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b5f6e, 33 },
    { 0xc2bb5f6f, 78 },
    { 0xc3fb5f6f, 61 },
    { 0, 0 },
    { 0x59bb5f72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4b5f75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08b5f78, 33 },
    { 0xcbcb5f79, 100 },
    { 0xd79b5f7a, 76 },
    { 0, 0 },
    { 0x71db5f7c, 95 },
    { 0x914b5f7c, 90 },
    { 0x650b5f7c, 90 },
    { 0x239b5f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5f88, 61 },
    { 0xe23b5f88, 95 },
    { 0x63eb5f8a, 55 },
    { 0x652b5f8b, 90 },
    { 0x689b5f8b, 95 },
    { 0x398b5f8c, 98 },
    { 0x479b5f8c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92b5f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x688b5f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b5f99, 61 },
    { 0x51ab5f9a, 91 },
    { 0xbc6b5f9b, 61 },
    { 0x498b5f9c, 61 },
    { 0x5aab5f9c, 61 },
    { 0xf3db5f99, 33 },
    { 0x34fb5f9f, 87 },
    { 0x72eb5fa0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cb5fa7, 77 },
    { 0, 0 },
    { 0x298b5fa9, 33 },
    { 0xf2eb5fa9, 33 },
    { 0x9e3b5faa, 99 },
    { 0, 0 },
    { 0x840b5fad, 61 },
    { 0xe3cb5fae, 98 },
    { 0x5d1b5faf, 69 },
    { 0xe91b5faf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eb5fb4, 87 },
    { 0xe06b5fb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12b5fbb, 67 },
    { 0, 0 },
    { 0x714b5fbd, 98 },
    { 0x5e2b5fbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825b5fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b5fcc, 55 },
    { 0x4d6b5fcc, 23 },
    { 0, 0 },
    { 0xeb0b5fcf, 61 },
    { 0xb4b5fcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b5fd5, 46 },
    { 0, 0 },
    { 0x4dfb5fd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482b5fdc, 33 },
    { 0x1cfb5fdc, 33 },
    { 0x3cab5fde, 76 },
    { 0x44db5fde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b5fe2, 66 },
    { 0x55b5fe3, 98 },
    { 0x21eb5fe2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb5fea, 34 },
    { 0x2dbb5feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b5ff0, 81 },
    { 0x2f2b5ff1, 34 },
    { 0x2f3b5ff1, 33 },
    { 0, 0 },
    { 0x8ddb5ff4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3b5ff9, 36 },
    { 0, 0 },
    { 0xaf6b5ffb, 33 },
    { 0, 0 },
    { 0x1ab5ffd, 55 },
    { 0, 0 },
    { 0xd68b5fff, 90 },
    { 0x20bb6000, 43 },
    { 0x65db6001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db6004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b600b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab6018, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6b601c, 61 },
    { 0, 0 },
    { 0x125b601e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eb6023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ab6028, 95 },
    { 0, 0 },
    { 0x70bb602a, 61 },
    { 0xf5b602b, 98 },
    { 0x7d3b602b, 61 },
    { 0xecdb602a, 34 },
    { 0xd3fb602e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b6031, 93 },
    { 0, 0 },
    { 0x3c2b6033, 33 },
    { 0, 0 },
    { 0x8e5b6035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05b6038, 90 },
    { 0, 0 },
    { 0x958b603a, 98 },
    { 0x718b603b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab603e, 76 },
    { 0x564b603e, 23 },
    { 0, 0 },
    { 0xa3b6041, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136b6046, 33 },
    { 0x56db6046, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b604d, 94 },
    { 0x36ab604e, 78 },
    { 0x822b604e, 33 },
    { 0xbd3b604f, 77 },
    { 0x4dbb6051, 33 },
    { 0, 0 },
    { 0x5d8b6053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b6057, 98 },
    { 0x5a3b6058, 23 },
    { 0, 0 },
    { 0x104b605a, 61 },
    { 0xbe6b605b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b605e, 43 },
    { 0x99cb605f, 33 },
    { 0xf5db6060, 23 },
    { 0x8b4b605f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ab6064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb606b, 76 },
    { 0xe83b606c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95b6073, 76 },
    { 0x8e0b6074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b607b, 78 },
    { 0xaa6b607b, 98 },
    { 0xe8b607c, 90 },
    { 0xdf0b607c, 61 },
    { 0x2dbb607f, 46 },
    { 0x7b607d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b6086, 98 },
    { 0xb2db6087, 93 },
    { 0x53eb6088, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b608f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb6096, 98 },
    { 0x959b6097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b609b, 85 },
    { 0xa61b609b, 81 },
    { 0xe14b609d, 91 },
    { 0xc7ab609e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab60a2, 95 },
    { 0, 0 },
    { 0x25cb60a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cb60ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b60ae, 33 },
    { 0, 0 },
    { 0x501b60b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b60b4, 33 },
    { 0, 0 },
    { 0x7deb60b6, 90 },
    { 0x5c8b60b7, 33 },
    { 0xc34b60b7, 90 },
    { 0, 0 },
    { 0x23b60ba, 91 },
    { 0x699b60bb, 34 },
    { 0x72fb60bb, 55 },
    { 0, 0 },
    { 0x5e2b60be, 23 },
    { 0xf84b60bf, 98 },
    { 0x82cb60c0, 33 },
    { 0x557b60c1, 90 },
    { 0xf07b60c2, 61 },
    { 0x333b60c0, 100 },
    { 0x3c3b60bf, 87 },
    { 0, 0 },
    { 0xac1b60c6, 90 },
    { 0x2f8b60c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcb60ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b60d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfb60d6, 95 },
    { 0, 0 },
    { 0x7deb60d8, 78 },
    { 0x27db60d8, 84 },
    { 0x56bb60da, 67 },
    { 0xc36b60db, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07b60df, 55 },
    { 0, 0 },
    { 0xd25b60e1, 68 },
    { 0x816b60e1, 61 },
    { 0x506b60e1, 98 },
    { 0x74cb60e4, 90 },
    { 0x630b60e5, 61 },
    { 0, 0 },
    { 0x35ab60e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fb60f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9b60f8, 23 },
    { 0xa3cb60f9, 98 },
    { 0x1fab60fa, 94 },
    { 0xef0b60fb, 76 },
    { 0xc39b60fb, 90 },
    { 0, 0 },
    { 0xb80b60fe, 90 },
    { 0x4b1b60ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b6104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b6108, 43 },
    { 0xf5fb6108, 77 },
    { 0, 0 },
    { 0x453b610b, 90 },
    { 0, 0 },
    { 0x704b610d, 40 },
    { 0x922b610d, 61 },
    { 0x82b610e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab6112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86b6118, 33 },
    { 0xe7cb6119, 98 },
    { 0, 0 },
    { 0xfecb611b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6b6120, 66 },
    { 0xb2cb6121, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb6124, 90 },
    { 0, 0 },
    { 0x859b6126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb612a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82db6132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb6136, 95 },
    { 0x262b6137, 98 },
    { 0, 0 },
    { 0xb98b6139, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb613c, 33 },
    { 0x318b613d, 33 },
    { 0xa66b613e, 95 },
    { 0xb09b613e, 87 },
    { 0x242b6140, 23 },
    { 0xbfab6141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64b6144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27b6147, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbb6150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17b6157, 61 },
    { 0, 0 },
    { 0xd9db6159, 98 },
    { 0, 0 },
    { 0x925b615b, 90 },
    { 0xcdeb615b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b615f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb6162, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b6165, 98 },
    { 0x621b6165, 93 },
    { 0xbe4b6165, 77 },
    { 0x83cb6165, 61 },
    { 0, 0 },
    { 0x653b616a, 93 },
    { 0x5d6b616b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3b6172, 78 },
    { 0xc05b6173, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b6177, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdb617b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecb617f, 34 },
    { 0x700b6180, 99 },
    { 0x264b6181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31b6186, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1b6189, 87 },
    { 0, 0 },
    { 0xb40b618b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb618e, 38 },
    { 0x7b5b618e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b619c, 43 },
    { 0, 0 },
    { 0x255b619e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5db61a1, 95 },
    { 0, 0 },
    { 0xf57b61a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b61a7, 95 },
    { 0, 0 },
    { 0x40bb61a9, 90 },
    { 0x23b61aa, 33 },
    { 0xed2b61ab, 34 },
    { 0, 0 },
    { 0x46fb61ad, 55 },
    { 0, 0 },
    { 0xf92b61af, 61 },
    { 0xe02b61af, 76 },
    { 0xdc0b61b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x462b61b4, 34 },
    { 0xd22b61b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x398b61b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fb61bb, 90 },
    { 0xba1b61bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6b61c0, 23 },
    { 0x6f6b61c1, 95 },
    { 0x56bb61c2, 23 },
    { 0x972b61c0, 90 },
    { 0x7b61c1, 78 },
    { 0x380b61c3, 90 },
    { 0xfc9b61c4, 46 },
    { 0xfdb61c3, 88 },
    { 0x234b61c7, 67 },
    { 0x9abb61c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b61cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb61d3, 95 },
    { 0x6aeb61d4, 64 },
    { 0xbb5b61d5, 23 },
    { 0xa2ab61d4, 90 },
    { 0x34db61d3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb61db, 33 },
    { 0, 0 },
    { 0xe92b61dd, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdab61e3, 55 },
    { 0, 0 },
    { 0x41eb61e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bb61e8, 99 },
    { 0xd46b61e9, 90 },
    { 0x2abb61ea, 95 },
    { 0xecb61eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cb61f6, 99 },
    { 0xfd7b61f7, 85 },
    { 0x581b61f7, 66 },
    { 0x689b61f9, 17 },
    { 0xb6b61fa, 98 },
    { 0, 0 },
    { 0x208b61fc, 87 },
    { 0, 0 },
    { 0x626b61fe, 61 },
    { 0, 0 },
    { 0x597b6200, 92 },
    { 0, 0 },
    { 0x89b6202, 33 },
    { 0x99eb6202, 95 },
    { 0xf36b6204, 76 },
    { 0, 0 },
    { 0x7e0b6206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b620c, 93 },
    { 0xe51b620d, 93 },
    { 0xb3fb620d, 90 },
    { 0x453b620e, 75 },
    { 0xe74b6210, 33 },
    { 0, 0 },
    { 0xe1bb6212, 90 },
    { 0xd8eb6213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473b6217, 52 },
    { 0x336b6218, 55 },
    { 0, 0 },
    { 0x649b621a, 91 },
    { 0x88fb621b, 90 },
    { 0xab4b621c, 23 },
    { 0x444b621a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873b6223, 95 },
    { 0, 0 },
    { 0xb0b6225, 34 },
    { 0, 0 },
    { 0xf04b6227, 98 },
    { 0xca2b6228, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5b622d, 61 },
    { 0, 0 },
    { 0x6d7b622f, 33 },
    { 0xb04b6230, 33 },
    { 0xa96b6231, 90 },
    { 0xdbbb622f, 90 },
    { 0x21fb6233, 90 },
    { 0, 0 },
    { 0x295b6235, 98 },
    { 0, 0 },
    { 0x3b5b6237, 24 },
    { 0xc63b6238, 61 },
    { 0x5eeb6239, 85 },
    { 0x157b623a, 55 },
    { 0x683b6239, 66 },
    { 0x8b4b6239, 68 },
    { 0, 0 },
    { 0x6b9b623e, 61 },
    { 0xe77b623f, 33 },
    { 0xdc6b623e, 97 },
    { 0x8cbb6241, 90 },
    { 0x33bb6241, 61 },
    { 0, 0 },
    { 0x5e8b6244, 76 },
    { 0x53cb6245, 93 },
    { 0xae7b6246, 90 },
    { 0x170b6246, 46 },
    { 0, 0 },
    { 0xe9fb6249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1b624c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab6253, 98 },
    { 0xdb5b6254, 99 },
    { 0x406b6253, 61 },
    { 0x4a5b6256, 90 },
    { 0x904b6257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b625a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x500b625d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdb6260, 90 },
    { 0x9f1b6261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5b6264, 33 },
    { 0x46cb6265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b626f, 56 },
    { 0x139b6270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b6275, 90 },
    { 0x497b6276, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1b627a, 76 },
    { 0x481b627b, 98 },
    { 0x5c6b627c, 90 },
    { 0xc57b627c, 93 },
    { 0xf47b627c, 23 },
    { 0x180b627f, 61 },
    { 0x362b6280, 42 },
    { 0, 0 },
    { 0x739b6282, 95 },
    { 0x381b6283, 46 },
    { 0x290b6284, 98 },
    { 0x847b6283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53b6288, 34 },
    { 0x712b6289, 61 },
    { 0xdfcb6289, 43 },
    { 0x3f8b6288, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9b628f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb6296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fab6299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45b629f, 23 },
    { 0x657b62a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b62a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb62ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afb62b9, 33 },
    { 0xf31b62ba, 61 },
    { 0, 0 },
    { 0xbd9b62bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eb62bf, 65 },
    { 0xb3eb62c0, 99 },
    { 0x6bb62c0, 87 },
    { 0x853b62c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502b62c6, 78 },
    { 0x9a2b62c7, 61 },
    { 0x669b62c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b62cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b62cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cb62d2, 33 },
    { 0x986b62d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2b62d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb62d9, 99 },
    { 0, 0 },
    { 0x7aeb62db, 78 },
    { 0x479b62dc, 90 },
    { 0, 0 },
    { 0xa53b62de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b62ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb62ef, 57 },
    { 0x8a6b62ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b62f3, 90 },
    { 0xcbb62f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7b62f8, 90 },
    { 0, 0 },
    { 0xdb6b62fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268b62fe, 23 },
    { 0xe56b62ff, 95 },
    { 0xf87b62fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfb6306, 68 },
    { 0x108b6306, 33 },
    { 0x854b6308, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2b630f, 55 },
    { 0, 0 },
    { 0xcd6b6311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fb6315, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bb6319, 93 },
    { 0x30eb631a, 23 },
    { 0xeab6319, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffb6323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b6326, 95 },
    { 0x5ab6327, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7b632a, 33 },
    { 0x6db632b, 90 },
    { 0xbdb632c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbb6332, 23 },
    { 0x65bb6332, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596b6337, 95 },
    { 0xd58b6338, 23 },
    { 0x62db6339, 33 },
    { 0xf95b633a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb6340, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb6343, 65 },
    { 0x81b6343, 46 },
    { 0x854b6343, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b6349, 61 },
    { 0xc27b634a, 90 },
    { 0xa5cb634b, 76 },
    { 0x865b634c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fb6351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb6355, 90 },
    { 0xa7ab6356, 95 },
    { 0xa42b6356, 33 },
    { 0x2bfb6357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cb635c, 76 },
    { 0x2db635c, 95 },
    { 0xcceb635e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2b6362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beb6369, 55 },
    { 0xf98b636a, 55 },
    { 0, 0 },
    { 0x8e9b636c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b6373, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abb637a, 88 },
    { 0, 0 },
    { 0xd09b637c, 76 },
    { 0xe3eb637d, 61 },
    { 0, 0 },
    { 0x58eb637f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db6382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3b6386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23b638d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24b6391, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b6399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cb639c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db63a1, 76 },
    { 0xb8cb63a2, 33 },
    { 0x1d0b63a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b63a6, 76 },
    { 0, 0 },
    { 0x78eb63a8, 61 },
    { 0, 0 },
    { 0x900b63aa, 33 },
    { 0, 0 },
    { 0xaaab63ac, 23 },
    { 0x379b63ac, 33 },
    { 0x75bb63ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160b63ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b63bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deb63c0, 90 },
    { 0, 0 },
    { 0x587b63c2, 98 },
    { 0xcd6b63c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cb63c8, 23 },
    { 0xaf0b63c9, 94 },
    { 0x4ab63c8, 91 },
    { 0xcf5b63c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6b63ce, 100 },
    { 0x3ab63ce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678b63d9, 34 },
    { 0xb46b63d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecb63dd, 33 },
    { 0, 0 },
    { 0x2deb63df, 61 },
    { 0xd27b63df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977b63eb, 61 },
    { 0x202b63ec, 90 },
    { 0x51b63ed, 61 },
    { 0x73cb63ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eb63f7, 90 },
    { 0xcf5b63f8, 90 },
    { 0x7b8b63f9, 90 },
    { 0, 0 },
    { 0xd54b63fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcb6407, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428b640f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487b6414, 95 },
    { 0x61bb6415, 90 },
    { 0xcfdb6416, 90 },
    { 0x26fb6414, 90 },
    { 0x995b6417, 46 },
    { 0xde3b6419, 95 },
    { 0xbc8b641a, 65 },
    { 0, 0 },
    { 0x76db641c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cb641f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb6422, 22 },
    { 0x9ab6423, 61 },
    { 0x862b6424, 76 },
    { 0x28cb6425, 23 },
    { 0x93b6426, 95 },
    { 0, 0 },
    { 0xb22b6428, 87 },
    { 0x80db6429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813b6430, 95 },
    { 0x21db6430, 24 },
    { 0x13eb6432, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eb6436, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b6439, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab643e, 90 },
    { 0xe6fb643e, 95 },
    { 0x88cb6440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcb6444, 93 },
    { 0x172b6445, 61 },
    { 0x574b6446, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2b6449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8b644d, 98 },
    { 0xfcbb644e, 90 },
    { 0x389b644f, 23 },
    { 0x85db644f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718b6455, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb645a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b645f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1b6463, 33 },
    { 0x80eb6463, 61 },
    { 0x773b6464, 98 },
    { 0x10eb6466, 90 },
    { 0, 0 },
    { 0x41bb6468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb646b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b646e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fb6473, 87 },
    { 0, 0 },
    { 0xeb4b6475, 23 },
    { 0x34fb6476, 55 },
    { 0xfbbb6476, 33 },
    { 0xd9ab6477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7b647e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffb6483, 23 },
    { 0, 0 },
    { 0x778b6485, 34 },
    { 0, 0 },
    { 0x357b6487, 95 },
    { 0x58bb6488, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b648b, 23 },
    { 0, 0 },
    { 0x2f6b648d, 95 },
    { 0x8f6b648e, 34 },
    { 0x4f1b648f, 66 },
    { 0x93b6490, 96 },
    { 0x94cb6491, 33 },
    { 0x1c2b6492, 46 },
    { 0xf20b6491, 91 },
    { 0x68eb6491, 34 },
    { 0x50ab6492, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb649a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab649e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5b64a1, 23 },
    { 0, 0 },
    { 0x436b64a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63b64a8, 90 },
    { 0xe64b64a9, 86 },
    { 0x43b64aa, 90 },
    { 0x107b64ab, 78 },
    { 0x27fb64ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2b64af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3b64b6, 49 },
    { 0x8dfb64b6, 76 },
    { 0x1cbb64b8, 90 },
    { 0, 0 },
    { 0x2a3b64ba, 90 },
    { 0x9dcb64bb, 56 },
    { 0x105b64bb, 78 },
    { 0x11db64bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb64c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45b64c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63b64ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ab64cd, 90 },
    { 0, 0 },
    { 0xc1fb64cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eb64d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b64db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38b64e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb64e6, 81 },
    { 0, 0 },
    { 0xef3b64e8, 90 },
    { 0, 0 },
    { 0x83eb64ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb64ed, 95 },
    { 0x9d7b64ee, 98 },
    { 0xde3b64ed, 61 },
    { 0, 0 },
    { 0x5bab64f1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb64f4, 52 },
    { 0xf9fb64f5, 88 },
    { 0xd91b64f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b64fb, 95 },
    { 0xba1b64fc, 90 },
    { 0xff5b64fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbb6503, 76 },
    { 0x25ab6504, 34 },
    { 0xfe3b6504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb6508, 68 },
    { 0, 0 },
    { 0x88b650a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb6513, 90 },
    { 0, 0 },
    { 0xa87b6515, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8b651d, 98 },
    { 0xa18b651e, 61 },
    { 0x4cbb651f, 99 },
    { 0x828b651d, 61 },
    { 0xcb0b6521, 43 },
    { 0x1a7b6522, 90 },
    { 0, 0 },
    { 0x485b6524, 23 },
    { 0x2e0b6525, 78 },
    { 0, 0 },
    { 0x73fb6527, 99 },
    { 0x3a9b6528, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504b652d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b6530, 33 },
    { 0, 0 },
    { 0x4dab6532, 90 },
    { 0xddb6532, 84 },
    { 0x43db6532, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ab6538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbb653b, 90 },
    { 0xa9bb653c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765b6540, 78 },
    { 0xdb8b6541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb6544, 67 },
    { 0xeedb6545, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfb654b, 33 },
    { 0x558b654c, 76 },
    { 0x15db654d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1b6552, 68 },
    { 0x959b6552, 95 },
    { 0xca4b6554, 33 },
    { 0x7fcb6555, 90 },
    { 0, 0 },
    { 0x2f8b6557, 90 },
    { 0xd6b6558, 100 },
    { 0x47eb6557, 76 },
    { 0xd8b655a, 76 },
    { 0x44eb655b, 84 },
    { 0, 0 },
    { 0x5bcb655d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3b6560, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeb6563, 33 },
    { 0x47bb6564, 68 },
    { 0, 0 },
    { 0xafdb6566, 68 },
    { 0x697b6567, 66 },
    { 0x5e7b6568, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eb656b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb6575, 33 },
    { 0, 0 },
    { 0xb75b6577, 91 },
    { 0x7e1b6578, 23 },
    { 0, 0 },
    { 0xa5bb657a, 91 },
    { 0, 0 },
    { 0x650b657c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x896b657f, 90 },
    { 0xe63b657f, 33 },
    { 0x91db6581, 61 },
    { 0x990b6581, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b6585, 25 },
    { 0, 0 },
    { 0xf5ab6587, 93 },
    { 0x13ab6587, 90 },
    { 0xdd4b6588, 76 },
    { 0, 0 },
    { 0x1cfb658b, 40 },
    { 0x4ceb658c, 90 },
    { 0, 0 },
    { 0xe07b658e, 95 },
    { 0x829b658f, 90 },
    { 0x78fb6590, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4db6594, 55 },
    { 0x308b6595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4db65a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31b65a9, 23 },
    { 0x6d5b65aa, 23 },
    { 0xae2b65aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adb65ae, 98 },
    { 0x6e0b65af, 46 },
    { 0x3b4b65b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b65b4, 100 },
    { 0, 0 },
    { 0x1f9b65b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b65b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b65bd, 46 },
    { 0x978b65be, 61 },
    { 0x403b65bf, 34 },
    { 0x83eb65c0, 90 },
    { 0xd16b65c1, 23 },
    { 0xdab65c0, 55 },
    { 0, 0 },
    { 0xdddb65c4, 99 },
    { 0x7ffb65c4, 98 },
    { 0x1c8b65c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb65cf, 90 },
    { 0xcaeb65cf, 40 },
    { 0x92cb65d1, 90 },
    { 0x13cb65d2, 90 },
    { 0x90db65d2, 76 },
    { 0x43eb65cf, 43 },
    { 0x9b3b65d2, 56 },
    { 0x4eb65d6, 90 },
    { 0, 0 },
    { 0xf9ab65d8, 33 },
    { 0xdb7b65d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b65dd, 90 },
    { 0, 0 },
    { 0x497b65df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab65e2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b65e5, 93 },
    { 0xedfb65e6, 33 },
    { 0x5f5b65e5, 95 },
    { 0, 0 },
    { 0xee6b65e9, 55 },
    { 0x148b65ea, 98 },
    { 0xf85b65eb, 61 },
    { 0x474b65ec, 85 },
    { 0x56bb65ed, 65 },
    { 0x4d2b65ea, 65 },
    { 0x7c3b65ef, 90 },
    { 0xc07b65f0, 95 },
    { 0x5db65f1, 90 },
    { 0x350b65f2, 34 },
    { 0x186b65f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b65f7, 55 },
    { 0, 0 },
    { 0x4c4b65f9, 55 },
    { 0x813b65fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b65ff, 93 },
    { 0x6e7b65ff, 75 },
    { 0x23cb6601, 33 },
    { 0x62ab6602, 61 },
    { 0xd42b6603, 40 },
    { 0xaeab6603, 90 },
    { 0x506b6605, 66 },
    { 0xabb6606, 61 },
    { 0xb58b6607, 95 },
    { 0x138b6606, 33 },
    { 0x65fb6609, 46 },
    { 0xed8b6604, 37 },
    { 0xb66b660b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9b6611, 90 },
    { 0, 0 },
    { 0xc1fb6613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eb6617, 95 },
    { 0x6a9b6618, 81 },
    { 0x286b6619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb6620, 95 },
    { 0xe22b6621, 61 },
    { 0x371b6622, 90 },
    { 0x1c7b6623, 90 },
    { 0x11ab6624, 4 },
    { 0x539b6622, 33 },
    { 0x445b6623, 67 },
    { 0xa1eb6622, 34 },
    { 0x8b8b6626, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35b662e, 91 },
    { 0, 0 },
    { 0xeaab6630, 31 },
    { 0xf1ab6631, 55 },
    { 0x705b6632, 33 },
    { 0x5f2b6631, 33 },
    { 0x8a0b6634, 55 },
    { 0xf67b6635, 98 },
    { 0xa8fb6636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcb663c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6b663f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ab6644, 67 },
    { 0, 0 },
    { 0xb66b6646, 33 },
    { 0xd49b6647, 61 },
    { 0, 0 },
    { 0xd2fb6649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5b664c, 95 },
    { 0x3a7b664d, 61 },
    { 0, 0 },
    { 0x1e1b664f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b6652, 90 },
    { 0xc9db6653, 34 },
    { 0x4ffb6654, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b6657, 78 },
    { 0x3cab6657, 69 },
    { 0x442b6659, 33 },
    { 0xcf5b665a, 90 },
    { 0xdc5b6659, 34 },
    { 0x61db665c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcab6663, 43 },
    { 0xdc9b6664, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b6667, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4b666a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb6674, 61 },
    { 0xdb7b6674, 67 },
    { 0, 0 },
    { 0x955b6677, 85 },
    { 0xf4fb6678, 95 },
    { 0, 0 },
    { 0x62eb667a, 55 },
    { 0xb06b667b, 43 },
    { 0xb11b667b, 33 },
    { 0x3c2b667c, 23 },
    { 0xfeb667d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b6693, 43 },
    { 0x9dab6694, 75 },
    { 0x8c9b6693, 33 },
    { 0xc90b6696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5b669c, 61 },
    { 0, 0 },
    { 0xc52b669e, 52 },
    { 0, 0 },
    { 0x5abb66a0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b66a5, 66 },
    { 0x66fb66a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eb66b1, 93 },
    { 0x129b66b2, 61 },
    { 0xf80b66b3, 91 },
    { 0x4f0b66b1, 55 },
    { 0xff9b66b5, 100 },
    { 0xc6bb66b6, 81 },
    { 0x2c3b66b1, 99 },
    { 0, 0 },
    { 0x31ab66b9, 95 },
    { 0, 0 },
    { 0x79bb66bb, 76 },
    { 0x8a8b66bc, 55 },
    { 0xa6eb66bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357b66c1, 95 },
    { 0, 0 },
    { 0xe1fb66c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0b66cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcb66cf, 98 },
    { 0xd7eb66d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b66d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49b66dc, 23 },
    { 0x7ddb66dd, 93 },
    { 0xf96b66dd, 23 },
    { 0, 0 },
    { 0x647b66e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b66e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377b66e8, 55 },
    { 0, 0 },
    { 0x8bdb66ea, 93 },
    { 0xac1b66eb, 68 },
    { 0, 0 },
    { 0xc4cb66ed, 78 },
    { 0x5b8b66ee, 34 },
    { 0, 0 },
    { 0xdc8b66f0, 76 },
    { 0x5b9b66f0, 95 },
    { 0x2fb66f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb66f6, 76 },
    { 0, 0 },
    { 0x1fbb66f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3b66fe, 23 },
    { 0x724b66ff, 61 },
    { 0x3c9b6700, 90 },
    { 0xa94b66ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b6704, 88 },
    { 0x4c4b6705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700b670a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcab670d, 78 },
    { 0x175b670e, 67 },
    { 0xddfb670f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deb6716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x618b6719, 90 },
    { 0, 0 },
    { 0x670b671b, 99 },
    { 0xfc6b671c, 90 },
    { 0x69b671c, 71 },
    { 0x96ab671e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27b6722, 61 },
    { 0x2b2b6723, 34 },
    { 0, 0 },
    { 0xe0eb6725, 87 },
    { 0xb51b6725, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601b6732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b673b, 84 },
    { 0x492b673b, 99 },
    { 0x5bfb673c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdb6742, 88 },
    { 0xcecb6743, 23 },
    { 0x2b6b6742, 98 },
    { 0xc38b6743, 23 },
    { 0xe1eb6746, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeb674e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x275b6751, 80 },
    { 0x364b6751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b6755, 46 },
    { 0xa3cb6755, 75 },
    { 0, 0 },
    { 0xcdbb6758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127b675c, 34 },
    { 0, 0 },
    { 0x90b675e, 90 },
    { 0xa5bb675e, 61 },
    { 0xc57b6760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ab6766, 98 },
    { 0, 0 },
    { 0x698b6768, 61 },
    { 0, 0 },
    { 0xcfab676a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdb676e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b6775, 33 },
    { 0x86b6775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07b677f, 95 },
    { 0, 0 },
    { 0x3d6b6781, 90 },
    { 0xd25b6782, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb678a, 33 },
    { 0x6ecb678b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cb678f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306b6796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02b679c, 23 },
    { 0x530b679d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b67a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2b67a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8b67a8, 55 },
    { 0x1a7b67a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab67ad, 90 },
    { 0x6eb67ae, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97b67b9, 93 },
    { 0xcb2b67ba, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b67c0, 95 },
    { 0, 0 },
    { 0xa88b67c2, 96 },
    { 0, 0 },
    { 0x56b67c4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x145b67c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53b67cb, 95 },
    { 0xd89b67cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960b67d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eb67d6, 85 },
    { 0x6a3b67d7, 87 },
    { 0xebeb67d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b67df, 46 },
    { 0xcdbb67e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b67e4, 85 },
    { 0, 0 },
    { 0x76ab67e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380b67eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07b67ee, 61 },
    { 0x9b0b67ef, 90 },
    { 0xe76b67f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cb67f3, 23 },
    { 0x78fb67f4, 25 },
    { 0x7b7b67f5, 33 },
    { 0x6eb67f6, 90 },
    { 0xaecb67f7, 95 },
    { 0xa44b67f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b67fc, 90 },
    { 0xbacb67fd, 90 },
    { 0xbecb67fe, 33 },
    { 0x9aab67fe, 90 },
    { 0xa85b6800, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51b6805, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8b680a, 100 },
    { 0, 0 },
    { 0x29b680c, 33 },
    { 0x22bb680d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb6818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abb681b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fb6821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5b6826, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x921b6829, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4b682c, 67 },
    { 0x5e6b682d, 98 },
    { 0x69cb682e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2b6832, 95 },
    { 0x489b6832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb6836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b683a, 33 },
    { 0, 0 },
    { 0xb68b683c, 99 },
    { 0, 0 },
    { 0x918b683e, 95 },
    { 0, 0 },
    { 0x1b8b6840, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b6846, 77 },
    { 0, 0 },
    { 0x5dcb6848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccb684c, 61 },
    { 0xb67b684c, 90 },
    { 0xe55b684e, 95 },
    { 0x824b684c, 34 },
    { 0x4a7b684d, 61 },
    { 0xf40b6851, 98 },
    { 0, 0 },
    { 0xa7ab6853, 82 },
    { 0, 0 },
    { 0x723b6855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b6859, 23 },
    { 0x344b6859, 23 },
    { 0x621b6859, 33 },
    { 0xc0fb685c, 98 },
    { 0x6bab685d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb6860, 34 },
    { 0, 0 },
    { 0x7f9b6862, 90 },
    { 0x5f9b6862, 33 },
    { 0x795b6864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74db686e, 66 },
    { 0x100b686e, 61 },
    { 0, 0 },
    { 0x7a1b6871, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7b6874, 67 },
    { 0x1ccb6875, 98 },
    { 0x6ab6874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb6879, 46 },
    { 0x3bbb687a, 61 },
    { 0x5dfb687b, 98 },
    { 0x646b687c, 33 },
    { 0xb96b687a, 98 },
    { 0x8e1b687b, 43 },
    { 0x2e2b687e, 33 },
    { 0, 0 },
    { 0x41db6881, 98 },
    { 0, 0 },
    { 0xc97b6883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b6887, 17 },
    { 0x38eb6888, 98 },
    { 0x6fb6887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6db688f, 85 },
    { 0xba5b6890, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fb6894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2b6899, 33 },
    { 0x2c5b689a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x698b689d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5b68a1, 93 },
    { 0x59cb68a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b68a9, 76 },
    { 0x661b68aa, 66 },
    { 0, 0 },
    { 0xe4ab68ac, 61 },
    { 0, 0 },
    { 0x15bb68ae, 90 },
    { 0x684b68af, 95 },
    { 0x866b68b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab68b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904b68b7, 61 },
    { 0, 0 },
    { 0xe0ab68b9, 43 },
    { 0, 0 },
    { 0x8f5b68bb, 90 },
    { 0x2dfb68bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db68c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb68c5, 87 },
    { 0x28eb68c5, 33 },
    { 0x9bcb68c7, 95 },
    { 0x72eb68c8, 15 },
    { 0xe8db68c9, 55 },
    { 0x5bbb68ca, 55 },
    { 0, 0 },
    { 0xc4b68cc, 90 },
    { 0x85bb68cd, 61 },
    { 0, 0 },
    { 0x185b68cf, 55 },
    { 0xe1b68d0, 34 },
    { 0xc59b68d1, 61 },
    { 0x7e9b68d2, 61 },
    { 0x950b68d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb68d7, 81 },
    { 0x1cb68d8, 55 },
    { 0x741b68d7, 98 },
    { 0, 0 },
    { 0xf49b68db, 61 },
    { 0xfe9b68dc, 90 },
    { 0xf5cb68dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0b68e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab68e5, 76 },
    { 0xf88b68e6, 23 },
    { 0xe48b68e7, 76 },
    { 0xf8bb68e8, 98 },
    { 0xf8eb68e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8b68ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fb68f6, 90 },
    { 0x419b68f7, 61 },
    { 0x91b68f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bb68fb, 88 },
    { 0x4e1b68fc, 85 },
    { 0x27db68fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bb6901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fb6904, 87 },
    { 0x8aeb6905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x441b6908, 33 },
    { 0xdd1b6909, 34 },
    { 0x57db6908, 91 },
    { 0x212b690a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b6912, 98 },
    { 0, 0 },
    { 0x9c4b6914, 61 },
    { 0xd46b6915, 61 },
    { 0, 0 },
    { 0x2f8b6917, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4db691b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb6921, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab6926, 76 },
    { 0, 0 },
    { 0x13ab6928, 55 },
    { 0, 0 },
    { 0x3bb692a, 61 },
    { 0xdd6b692a, 99 },
    { 0xed5b692b, 90 },
    { 0x2b9b692d, 84 },
    { 0x26fb692c, 95 },
    { 0, 0 },
    { 0xfe0b6930, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x235b6933, 43 },
    { 0x812b6933, 61 },
    { 0xd6cb6935, 84 },
    { 0x38fb6933, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b6939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b6940, 76 },
    { 0x448b6940, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab6944, 61 },
    { 0x87bb6944, 49 },
    { 0, 0 },
    { 0x5d6b6947, 90 },
    { 0x95eb6948, 76 },
    { 0x205b6948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb694f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbb6954, 90 },
    { 0xfeb6955, 78 },
    { 0x3ffb6956, 98 },
    { 0x74db6957, 76 },
    { 0x10b6958, 55 },
    { 0xc69b6955, 34 },
    { 0x9d6b695a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b695e, 33 },
    { 0x2b8b695f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1db6965, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeb696a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5db696d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b6971, 98 },
    { 0, 0 },
    { 0x21cb6973, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183b6977, 61 },
    { 0xfbeb6978, 61 },
    { 0xa22b6979, 75 },
    { 0xdf9b6979, 52 },
    { 0x95db697a, 93 },
    { 0x686b697a, 43 },
    { 0x1fcb6979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1b6981, 52 },
    { 0x6b7b6981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x191b6985, 61 },
    { 0x45db6986, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b698d, 61 },
    { 0xa8eb698e, 23 },
    { 0xf72b698f, 46 },
    { 0xa7ab6990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bb6994, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcb6998, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b699c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x640b699f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b69ae, 95 },
    { 0x5bb69ae, 46 },
    { 0, 0 },
    { 0xba4b69b1, 99 },
    { 0, 0 },
    { 0x7bcb69b3, 23 },
    { 0, 0 },
    { 0xd03b69b5, 100 },
    { 0x9f3b69b5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb69ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8b69bd, 93 },
    { 0xe3cb69be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49b69c1, 75 },
    { 0xc84b69c2, 95 },
    { 0x63b69c3, 61 },
    { 0xbfeb69c4, 98 },
    { 0x87ab69c4, 34 },
    { 0xe5bb69c5, 61 },
    { 0x4ebb69c7, 90 },
    { 0xf18b69c8, 52 },
    { 0x426b69c5, 93 },
    { 0xb1db69c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b69ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b69d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ab69d8, 64 },
    { 0x458b69d9, 23 },
    { 0x964b69da, 87 },
    { 0, 0 },
    { 0x94bb69dc, 34 },
    { 0, 0 },
    { 0xaacb69de, 81 },
    { 0x970b69de, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb69e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb69e6, 33 },
    { 0, 0 },
    { 0xba9b69e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb69eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b69f1, 61 },
    { 0, 0 },
    { 0x6d7b69f3, 76 },
    { 0, 0 },
    { 0xacfb69f5, 93 },
    { 0x6b5b69f5, 98 },
    { 0x7e3b69f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307b69fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79b6a00, 93 },
    { 0x483b6a01, 90 },
    { 0, 0 },
    { 0x31fb6a03, 61 },
    { 0xd88b6a04, 76 },
    { 0, 0 },
    { 0xc8bb6a06, 66 },
    { 0x2cdb6a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41b6a0a, 93 },
    { 0xef7b6a0b, 67 },
    { 0, 0 },
    { 0x744b6a0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8b6a10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663b6a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b6a1b, 87 },
    { 0x61fb6a1b, 33 },
    { 0x82fb6a1d, 90 },
    { 0x30fb6a1d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedb6a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab6a26, 99 },
    { 0x6a4b6a27, 33 },
    { 0, 0 },
    { 0xb74b6a29, 33 },
    { 0x41cb6a2a, 61 },
    { 0x1f1b6a2b, 98 },
    { 0xc59b6a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb6a33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b6a3a, 23 },
    { 0x997b6a3b, 38 },
    { 0x792b6a3b, 67 },
    { 0xd3cb6a3d, 90 },
    { 0x732b6a3e, 23 },
    { 0, 0 },
    { 0xcc0b6a40, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b6a43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb6a47, 95 },
    { 0x87fb6a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb6a4b, 34 },
    { 0, 0 },
    { 0x36eb6a4d, 84 },
    { 0, 0 },
    { 0xd6b6a4f, 93 },
    { 0xd16b6a50, 77 },
    { 0x392b6a51, 23 },
    { 0xbbbb6a52, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904b6a59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4b6a62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bb6a67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb6a6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2b6a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cb6a76, 90 },
    { 0x728b6a77, 61 },
    { 0x9dab6a78, 100 },
    { 0x698b6a78, 87 },
    { 0xfb2b6a7a, 90 },
    { 0x6bab6a7a, 23 },
    { 0xdfab6a79, 55 },
    { 0xa44b6a7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb6a84, 75 },
    { 0, 0 },
    { 0xd03b6a86, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275b6a8b, 61 },
    { 0xa26b6a8c, 55 },
    { 0xff9b6a8d, 100 },
    { 0x255b6a8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb6a93, 90 },
    { 0, 0 },
    { 0x79fb6a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab6a98, 95 },
    { 0xf00b6a99, 61 },
    { 0x28ab6a9a, 90 },
    { 0x9bdb6a9b, 23 },
    { 0, 0 },
    { 0x81b6a9d, 90 },
    { 0x5b3b6a9e, 98 },
    { 0, 0 },
    { 0x874b6aa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b6aa4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab6aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x640b6aad, 76 },
    { 0x65fb6aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b6ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b6ab4, 95 },
    { 0x5bbb6ab5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0eb6ab9, 78 },
    { 0xf74b6ab9, 34 },
    { 0, 0 },
    { 0xd7ab6abc, 90 },
    { 0xc0ab6abd, 33 },
    { 0x1a2b6abe, 66 },
    { 0x1ffb6abc, 95 },
    { 0, 0 },
    { 0x13cb6ac1, 89 },
    { 0xe7b6ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab6ad2, 23 },
    { 0, 0 },
    { 0x407b6ad4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x70db6ad7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bb6ada, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10b6add, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bb6ae2, 68 },
    { 0, 0 },
    { 0x4bfb6ae4, 98 },
    { 0, 0 },
    { 0x4d6b6ae6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b6ae9, 95 },
    { 0, 0 },
    { 0x5c1b6aeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceb6aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636b6af4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bb6afd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b6b03, 90 },
    { 0x38b6b04, 90 },
    { 0, 0 },
    { 0xa7fb6b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb6b0c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fb6b14, 67 },
    { 0x7cbb6b14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663b6b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35eb6b1e, 34 },
    { 0x92b6b1f, 95 },
    { 0xc12b6b20, 61 },
    { 0, 0 },
    { 0x87b6b22, 52 },
    { 0, 0 },
    { 0xfb1b6b24, 61 },
    { 0x21b6b24, 75 },
    { 0x670b6b26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b6b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b6b31, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b6b37, 98 },
    { 0, 0 },
    { 0x478b6b39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edb6b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bb6b41, 90 },
    { 0, 0 },
    { 0xc29b6b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b6b47, 33 },
    { 0x12ab6b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fb6b4c, 98 },
    { 0, 0 },
    { 0xc5eb6b4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb6b51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8b6b54, 90 },
    { 0, 0 },
    { 0xed7b6b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b6b5b, 76 },
    { 0, 0 },
    { 0x653b6b5d, 23 },
    { 0x9d6b6b5e, 98 },
    { 0, 0 },
    { 0x649b6b60, 90 },
    { 0, 0 },
    { 0xb44b6b62, 95 },
    { 0x361b6b63, 95 },
    { 0xfcfb6b64, 85 },
    { 0xbabb6b64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5b6b68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x354b6b6b, 66 },
    { 0, 0 },
    { 0x691b6b6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x132b6b70, 61 },
    { 0xee0b6b71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bb6b75, 46 },
    { 0x523b6b76, 33 },
    { 0xeb5b6b77, 69 },
    { 0, 0 },
    { 0xae1b6b79, 95 },
    { 0x70fb6b7a, 90 },
    { 0x945b6b7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ab6b7e, 90 },
    { 0x22fb6b7f, 33 },
    { 0xf8db6b80, 55 },
    { 0xeb2b6b81, 23 },
    { 0x9cdb6b81, 33 },
    { 0xf2fb6b81, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b6b8e, 46 },
    { 0xc74b6b8f, 64 },
    { 0xf7cb6b8e, 98 },
    { 0, 0 },
    { 0xdfbb6b92, 76 },
    { 0x6e7b6b93, 23 },
    { 0, 0 },
    { 0x7a7b6b95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb6b99, 61 },
    { 0x7d7b6b9a, 52 },
    { 0, 0 },
    { 0xdcb6b9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5b6ba1, 90 },
    { 0x455b6ba2, 91 },
    { 0x493b6ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1b6ba7, 90 },
    { 0x22ab6ba8, 33 },
    { 0x7b1b6ba8, 55 },
    { 0, 0 },
    { 0xc23b6bab, 34 },
    { 0x46ab6bac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b6bb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b6bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb6bb6, 33 },
    { 0, 0 },
    { 0x51cb6bb8, 33 },
    { 0x86ab6bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeb6bbc, 55 },
    { 0, 0 },
    { 0x5f5b6bbe, 84 },
    { 0x277b6bbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882b6bc7, 66 },
    { 0, 0 },
    { 0x8bdb6bc9, 34 },
    { 0x642b6bc9, 34 },
    { 0xd06b6bcb, 90 },
    { 0x8cb6bcc, 34 },
    { 0xc7b6bcd, 76 },
    { 0xcb6b6bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679b6bd2, 55 },
    { 0x444b6bd2, 90 },
    { 0, 0 },
    { 0xc77b6bd5, 98 },
    { 0x775b6bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b6bd9, 33 },
    { 0xd42b6bda, 90 },
    { 0x34b6bd9, 77 },
    { 0x41ab6bdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306b6be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b6be5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b6bee, 90 },
    { 0xc64b6bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb6bf6, 33 },
    { 0x997b6bf7, 98 },
    { 0xe56b6bf8, 43 },
    { 0x1a8b6bf9, 98 },
    { 0, 0 },
    { 0xe61b6bfb, 100 },
    { 0, 0 },
    { 0xb50b6bfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eb6c00, 23 },
    { 0x5beb6c01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eab6c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb6c0b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bb6c0e, 90 },
    { 0x2aab6c0f, 33 },
    { 0x885b6c0f, 55 },
    { 0xbccb6c11, 33 },
    { 0x6fbb6c11, 61 },
    { 0xf10b6c13, 61 },
    { 0x502b6c14, 34 },
    { 0, 0 },
    { 0x541b6c16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab6c1f, 90 },
    { 0x965b6c20, 38 },
    { 0x40ab6c20, 55 },
    { 0x70bb6c22, 33 },
    { 0x2e7b6c22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b6c26, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b6c2b, 33 },
    { 0x6eab6c2c, 90 },
    { 0x1b6b6c2b, 91 },
    { 0x883b6c2e, 76 },
    { 0xcd8b6c2d, 95 },
    { 0x927b6c30, 61 },
    { 0xd42b6c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b6c34, 98 },
    { 0, 0 },
    { 0x6a2b6c36, 78 },
    { 0xf3b6c36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecab6c3b, 61 },
    { 0x843b6c3c, 99 },
    { 0xe2bb6c3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eb6c43, 43 },
    { 0, 0 },
    { 0xa6fb6c45, 34 },
    { 0x49fb6c45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b6c4a, 52 },
    { 0xa46b6c4b, 98 },
    { 0x762b6c4c, 61 },
    { 0x713b6c4d, 33 },
    { 0xf2cb6c4d, 95 },
    { 0xb2ab6c4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3b6c56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x595b6c59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beb6c5e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3b6c64, 76 },
    { 0xe73b6c65, 61 },
    { 0x208b6c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab6c6a, 78 },
    { 0x722b6c6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6c75, 83 },
    { 0x6dbb6c75, 23 },
    { 0xe53b6c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b6c7c, 61 },
    { 0, 0 },
    { 0x2dbb6c7e, 23 },
    { 0x3fcb6c7f, 17 },
    { 0xd8bb6c80, 64 },
    { 0x503b6c81, 95 },
    { 0x67eb6c82, 66 },
    { 0xeb4b6c83, 95 },
    { 0, 0 },
    { 0x18ab6c85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b6c8b, 95 },
    { 0, 0 },
    { 0x6a8b6c8d, 61 },
    { 0xbfeb6c8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8b6c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeeb6c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2db6c9b, 77 },
    { 0x12cb6c9b, 53 },
    { 0xe39b6c9d, 46 },
    { 0xfccb6c9b, 55 },
    { 0, 0 },
    { 0x947b6ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4b6ca3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25b6ca8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b6cac, 90 },
    { 0xf6cb6cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5b6cba, 91 },
    { 0x7ab6cbb, 76 },
    { 0xb3cb6cbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb6cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab6cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b6cc6, 95 },
    { 0xe7db6cc7, 98 },
    { 0x903b6cc8, 76 },
    { 0x933b6cc8, 91 },
    { 0, 0 },
    { 0xf52b6ccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b6cd1, 61 },
    { 0, 0 },
    { 0xe29b6cd3, 95 },
    { 0, 0 },
    { 0x85eb6cd5, 100 },
    { 0xdb9b6cd6, 23 },
    { 0x540b6cd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb6cda, 43 },
    { 0xd3bb6cdb, 61 },
    { 0, 0 },
    { 0x73cb6cdd, 98 },
    { 0x81b6cdd, 42 },
    { 0xbbab6cde, 90 },
    { 0x4f8b6ce0, 98 },
    { 0x715b6ce0, 66 },
    { 0, 0 },
    { 0x5a3b6ce3, 87 },
    { 0x8dbb6ce3, 52 },
    { 0, 0 },
    { 0x8a5b6ce6, 76 },
    { 0x8dfb6ce6, 91 },
    { 0xb40b6ce7, 91 },
    { 0, 0 },
    { 0xc44b6cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b6ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafb6cf4, 61 },
    { 0xe68b6cf5, 90 },
    { 0, 0 },
    { 0x452b6cf7, 23 },
    { 0, 0 },
    { 0xcfdb6cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb6d01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6b6d06, 90 },
    { 0xf51b6d06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0db6d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadb6d13, 95 },
    { 0xb6fb6d13, 76 },
    { 0, 0 },
    { 0x3e6b6d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81b6d1b, 66 },
    { 0xba8b6d1b, 42 },
    { 0, 0 },
    { 0x1ebb6d1e, 93 },
    { 0x179b6d1f, 33 },
    { 0xa96b6d20, 76 },
    { 0x379b6d21, 61 },
    { 0x364b6d20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eb6d26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67b6d29, 95 },
    { 0, 0 },
    { 0x36db6d2b, 95 },
    { 0xb07b6d2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x986b6d2f, 23 },
    { 0xab4b6d30, 100 },
    { 0xe3eb6d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb6d35, 33 },
    { 0, 0 },
    { 0xb30b6d37, 90 },
    { 0, 0 },
    { 0x7ab6d39, 100 },
    { 0x66cb6d3a, 91 },
    { 0x140b6d3a, 55 },
    { 0xd93b6d39, 90 },
    { 0, 0 },
    { 0x465b6d3e, 76 },
    { 0x10bb6d3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75db6d43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1b6d4a, 23 },
    { 0xa46b6d4b, 23 },
    { 0x57b6d4c, 76 },
    { 0, 0 },
    { 0xd6fb6d4e, 76 },
    { 0x286b6d4e, 55 },
    { 0, 0 },
    { 0x52bb6d51, 95 },
    { 0x324b6d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70b6d56, 76 },
    { 0xc96b6d57, 91 },
    { 0x411b6d58, 43 },
    { 0x54cb6d56, 66 },
    { 0x102b6d5a, 61 },
    { 0x320b6d5a, 90 },
    { 0xbd5b6d5b, 76 },
    { 0x7e2b6d5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb6d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab6d6a, 55 },
    { 0xf2cb6d6b, 34 },
    { 0x98b6d6c, 46 },
    { 0x7fab6d6c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b6d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72b6d78, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceb6d7b, 84 },
    { 0x7deb6d7c, 87 },
    { 0, 0 },
    { 0x840b6d7e, 68 },
    { 0x40eb6d7f, 61 },
    { 0xa7eb6d7e, 100 },
    { 0xa94b6d80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0b6d87, 61 },
    { 0xf33b6d88, 34 },
    { 0x8fdb6d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332b6d8f, 77 },
    { 0x875b6d90, 98 },
    { 0, 0 },
    { 0x3b8b6d92, 23 },
    { 0xaefb6d93, 88 },
    { 0x1e1b6d93, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb6d9b, 75 },
    { 0, 0 },
    { 0xf54b6d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93b6da5, 98 },
    { 0, 0 },
    { 0x687b6da7, 98 },
    { 0xd14b6da8, 33 },
    { 0x743b6da8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbb6dae, 61 },
    { 0x20cb6dae, 78 },
    { 0xb69b6db0, 98 },
    { 0x635b6db1, 61 },
    { 0x2e2b6db1, 77 },
    { 0x703b6db2, 66 },
    { 0x4afb6db4, 66 },
    { 0, 0 },
    { 0x16b6db6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb6db9, 34 },
    { 0xe2eb6dba, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b6dbe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6b6dc3, 90 },
    { 0x4cbb6dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b6dc7, 23 },
    { 0x9e9b6dc8, 90 },
    { 0xedeb6dc9, 23 },
    { 0xfe8b6dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb6dd0, 90 },
    { 0xc55b6dd1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb6dd5, 93 },
    { 0x679b6dd5, 90 },
    { 0xb14b6dd6, 95 },
    { 0x77bb6dd8, 33 },
    { 0x938b6dd9, 23 },
    { 0x726b6dda, 75 },
    { 0, 0 },
    { 0x130b6ddc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322b6de2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cb6de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb6de8, 34 },
    { 0x5d4b6de9, 95 },
    { 0, 0 },
    { 0xf8b6deb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab6dee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1b6df1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb6df4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81db6dfa, 90 },
    { 0, 0 },
    { 0xdeab6dfc, 81 },
    { 0xcb6dfd, 53 },
    { 0x5f3b6dfd, 66 },
    { 0xc1b6dff, 76 },
    { 0x406b6e00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681b6e08, 99 },
    { 0, 0 },
    { 0x5b0b6e0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b6e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b6e14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8b6e1a, 34 },
    { 0xe76b6e1b, 66 },
    { 0x243b6e1c, 100 },
    { 0xc8ab6e1d, 69 },
    { 0xbb9b6e1e, 61 },
    { 0xfe2b6e1f, 84 },
    { 0, 0 },
    { 0x6b5b6e21, 90 },
    { 0x2e4b6e22, 75 },
    { 0xd2ab6e22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeb6e29, 95 },
    { 0, 0 },
    { 0x658b6e2b, 66 },
    { 0x936b6e2c, 90 },
    { 0x73fb6e2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6e33, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b6e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcb6e43, 84 },
    { 0, 0 },
    { 0x3b8b6e45, 95 },
    { 0, 0 },
    { 0x27ab6e47, 69 },
    { 0x2d1b6e48, 66 },
    { 0, 0 },
    { 0x634b6e4a, 55 },
    { 0x28ab6e4b, 76 },
    { 0x151b6e4c, 85 },
    { 0x6a8b6e4d, 95 },
    { 0x9a8b6e4d, 34 },
    { 0, 0 },
    { 0xf93b6e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73b6e54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b6e58, 61 },
    { 0, 0 },
    { 0x43eb6e5a, 90 },
    { 0x3b9b6e5b, 88 },
    { 0xc7fb6e5c, 67 },
    { 0x8efb6e5b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b6e63, 76 },
    { 0x2dab6e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x763b6e67, 78 },
    { 0x6afb6e68, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b6e6c, 23 },
    { 0x73eb6e6d, 33 },
    { 0x24cb6e6d, 66 },
    { 0xd2eb6e6f, 34 },
    { 0x966b6e70, 66 },
    { 0xcc3b6e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b6e74, 33 },
    { 0xcedb6e75, 33 },
    { 0x700b6e76, 61 },
    { 0xbdbb6e74, 98 },
    { 0x2b8b6e78, 99 },
    { 0x8f3b6e79, 46 },
    { 0xef0b6e79, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104b6e7f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182b6e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ab6e8b, 95 },
    { 0, 0 },
    { 0x58eb6e8d, 90 },
    { 0x41fb6e8e, 52 },
    { 0xb12b6e8e, 52 },
    { 0x9f1b6e8d, 99 },
    { 0x506b6e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db6e98, 90 },
    { 0x146b6e98, 76 },
    { 0x2d9b6e99, 90 },
    { 0x785b6e9b, 95 },
    { 0, 0 },
    { 0x999b6e9d, 34 },
    { 0xefb6e9e, 34 },
    { 0xcf6b6e9f, 90 },
    { 0x5b5b6ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b6ea4, 34 },
    { 0, 0 },
    { 0x657b6ea6, 34 },
    { 0x403b6ea7, 77 },
    { 0xbffb6ea8, 23 },
    { 0xa27b6ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eb6eb0, 33 },
    { 0xc10b6eb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6eb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b6eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab6ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b6ec6, 33 },
    { 0, 0 },
    { 0x575b6ec8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb6ecc, 34 },
    { 0x205b6ecc, 98 },
    { 0xe0fb6ece, 23 },
    { 0x8cbb6ecf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab6ed5, 90 },
    { 0x15fb6ed6, 33 },
    { 0xd70b6ed7, 98 },
    { 0xfb3b6ed7, 33 },
    { 0xc9ab6ed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b6edc, 33 },
    { 0xc90b6edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x312b6ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb6ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb6ee7, 90 },
    { 0, 0 },
    { 0xab0b6ee9, 90 },
    { 0xa3cb6eea, 95 },
    { 0x33ab6eeb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeab6ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35b6ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fb6efa, 46 },
    { 0x5feb6efb, 23 },
    { 0x5cb6efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5b6f01, 90 },
    { 0, 0 },
    { 0x8c3b6f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b6f0c, 77 },
    { 0x87ab6f0c, 33 },
    { 0xe0db6f0e, 85 },
    { 0xcbab6f0d, 61 },
    { 0x43eb6f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6b6f13, 61 },
    { 0, 0 },
    { 0x43db6f15, 66 },
    { 0x87b6f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0b6f19, 98 },
    { 0, 0 },
    { 0xd36b6f1b, 43 },
    { 0, 0 },
    { 0xde9b6f1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b6f20, 43 },
    { 0x46bb6f21, 33 },
    { 0, 0 },
    { 0x87cb6f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2b6f26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb6f2b, 33 },
    { 0, 0 },
    { 0xce8b6f2d, 33 },
    { 0xe5cb6f2e, 95 },
    { 0, 0 },
    { 0x21ab6f30, 90 },
    { 0, 0 },
    { 0x932b6f32, 33 },
    { 0x36ab6f33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb6f43, 33 },
    { 0xf70b6f44, 33 },
    { 0, 0 },
    { 0x5b6f46, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b6f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31b6f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4eb6f50, 99 },
    { 0, 0 },
    { 0xb3db6f52, 61 },
    { 0, 0 },
    { 0xa60b6f54, 84 },
    { 0, 0 },
    { 0x1ddb6f56, 76 },
    { 0x7deb6f57, 90 },
    { 0x4b0b6f58, 61 },
    { 0x3f0b6f59, 61 },
    { 0xa9cb6f57, 91 },
    { 0x8aab6f5b, 33 },
    { 0xe78b6f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x877b6f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eb6f63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b6f69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2b6f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b6f6f, 98 },
    { 0x3b6b6f70, 90 },
    { 0xb64b6f70, 80 },
    { 0x176b6f72, 99 },
    { 0x154b6f70, 61 },
    { 0x5c3b6f74, 34 },
    { 0x167b6f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b6f78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eeb6f7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2b6f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb6f82, 98 },
    { 0x74eb6f83, 95 },
    { 0, 0 },
    { 0x843b6f85, 99 },
    { 0xefab6f86, 90 },
    { 0, 0 },
    { 0xff3b6f88, 90 },
    { 0x537b6f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6f8d, 34 },
    { 0, 0 },
    { 0x28cb6f8f, 46 },
    { 0xe12b6f8f, 98 },
    { 0xfa5b6f90, 76 },
    { 0, 0 },
    { 0x6c6b6f93, 98 },
    { 0, 0 },
    { 0xecdb6f95, 43 },
    { 0x64fb6f95, 66 },
    { 0, 0 },
    { 0x2f8b6f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b6f9b, 23 },
    { 0, 0 },
    { 0x7f6b6f9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eb6fa0, 61 },
    { 0x7e6b6fa1, 99 },
    { 0xcb0b6fa2, 61 },
    { 0x8a9b6fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fb6fa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eb6fab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204b6fb1, 43 },
    { 0x54b6fb2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b6fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b6fbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7b6fc6, 67 },
    { 0x36db6fc7, 67 },
    { 0x763b6fc6, 91 },
    { 0x161b6fc7, 66 },
    { 0x7f8b6fca, 67 },
    { 0x418b6fca, 75 },
    { 0x9ebb6fcc, 98 },
    { 0x56bb6fca, 90 },
    { 0, 0 },
    { 0x285b6fcf, 90 },
    { 0, 0 },
    { 0xd82b6fd1, 23 },
    { 0x440b6fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2b6fdf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9b6fe7, 33 },
    { 0xcdfb6fe8, 90 },
    { 0x5bfb6fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b6ff0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73b6ff5, 87 },
    { 0, 0 },
    { 0xc37b6ff7, 23 },
    { 0xa92b6ff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db6ffc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb7001, 88 },
    { 0xaf3b7002, 61 },
    { 0x220b7003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b7007, 77 },
    { 0, 0 },
    { 0x3a7b7009, 68 },
    { 0x866b700a, 41 },
    { 0x206b700b, 61 },
    { 0, 0 },
    { 0x96ab700d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb7014, 23 },
    { 0, 0 },
    { 0xd4ab7016, 23 },
    { 0, 0 },
    { 0x9d6b7018, 23 },
    { 0xd90b7019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cb7020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bb7025, 33 },
    { 0xd36b7025, 90 },
    { 0xddfb7025, 61 },
    { 0, 0 },
    { 0x754b7029, 33 },
    { 0x7f6b702a, 93 },
    { 0x4f0b702a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfb7031, 40 },
    { 0xd84b7031, 98 },
    { 0, 0 },
    { 0xb3b7034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32eb7037, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b703b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab703f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab7042, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b7046, 85 },
    { 0xb2ab7046, 90 },
    { 0xe7b7048, 43 },
    { 0x858b7048, 33 },
    { 0xb5eb704a, 34 },
    { 0xdf4b7049, 67 },
    { 0x317b704c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b704f, 28 },
    { 0, 0 },
    { 0x800b7051, 98 },
    { 0x118b7052, 23 },
    { 0x312b7052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ab7058, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb705d, 98 },
    { 0x718b705e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1db7062, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb7065, 33 },
    { 0, 0 },
    { 0x6e3b7067, 46 },
    { 0xed4b7067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb7070, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b7073, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x376b7076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccb7079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cb707c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb7081, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb708a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb708f, 33 },
    { 0, 0 },
    { 0xa54b7091, 61 },
    { 0xc44b7092, 90 },
    { 0xaf4b7093, 95 },
    { 0x69eb7094, 87 },
    { 0xdb1b7094, 90 },
    { 0xd56b7096, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb709e, 91 },
    { 0xe74b709f, 34 },
    { 0, 0 },
    { 0x632b70a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5b70a5, 23 },
    { 0, 0 },
    { 0xdeeb70a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b70af, 23 },
    { 0x9e9b70b0, 75 },
    { 0xd6fb70b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eb70b4, 93 },
    { 0x201b70b4, 55 },
    { 0, 0 },
    { 0xd3b70b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbb70c7, 37 },
    { 0xf8fb70c8, 55 },
    { 0xed3b70c9, 91 },
    { 0x95db70c9, 99 },
    { 0x850b70c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb70d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bb70e5, 90 },
    { 0x223b70e6, 95 },
    { 0xf14b70e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b70eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bb70ee, 87 },
    { 0x72cb70ef, 90 },
    { 0, 0 },
    { 0xdcab70f1, 33 },
    { 0x1feb70f1, 87 },
    { 0x317b70f3, 33 },
    { 0xa1fb70f1, 17 },
    { 0, 0 },
    { 0x681b70f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b70f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bab7102, 93 },
    { 0x5beb7102, 53 },
    { 0, 0 },
    { 0x10eb7105, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b7109, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x557b710c, 61 },
    { 0xc81b710d, 46 },
    { 0xe34b710d, 98 },
    { 0xe67b710f, 61 },
    { 0, 0 },
    { 0xb04b7111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb7119, 87 },
    { 0x51bb7119, 40 },
    { 0xbb6b711a, 76 },
    { 0xe7bb711c, 98 },
    { 0x912b7119, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b7120, 34 },
    { 0x878b7121, 95 },
    { 0xecab7122, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db7129, 89 },
    { 0x220b712a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08b712d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895b7131, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb7138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b713c, 95 },
    { 0x3d6b713d, 90 },
    { 0x692b713e, 90 },
    { 0xfabb713f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdb7148, 43 },
    { 0x651b7149, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb714e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab7154, 67 },
    { 0x2db7155, 95 },
    { 0, 0 },
    { 0xcabb7157, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490b715b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0b7163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eb7167, 61 },
    { 0x7deb7168, 87 },
    { 0xddeb7168, 90 },
    { 0x6f0b716a, 95 },
    { 0xa01b716b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdb716f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab7172, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1b7177, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1b7183, 95 },
    { 0xe8ab7184, 76 },
    { 0x43ab7185, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x805b7188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcb718b, 95 },
    { 0xb79b718c, 98 },
    { 0x2c4b718c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172b719b, 98 },
    { 0x75ab719c, 43 },
    { 0xa0bb719d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebb71a0, 91 },
    { 0, 0 },
    { 0x4f1b71a2, 34 },
    { 0, 0 },
    { 0x684b71a4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0b71a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b71ab, 23 },
    { 0, 0 },
    { 0x126b71ad, 23 },
    { 0x7b3b71ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ab71b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b71b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b71bd, 4 },
    { 0x495b71be, 61 },
    { 0x350b71bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9b71c2, 38 },
    { 0xbb1b71c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb71c6, 93 },
    { 0x4a8b71c6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b71ca, 61 },
    { 0xe96b71cb, 55 },
    { 0xcc3b71cc, 90 },
    { 0, 0 },
    { 0xe1bb71ce, 33 },
    { 0, 0 },
    { 0x128b71d0, 61 },
    { 0x514b71d1, 56 },
    { 0, 0 },
    { 0x8a3b71d3, 93 },
    { 0, 0 },
    { 0x1a5b71d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0b71dc, 61 },
    { 0x490b71dd, 33 },
    { 0x4c4b71de, 95 },
    { 0xdebb71df, 98 },
    { 0, 0 },
    { 0x477b71e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b71eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afb71ef, 55 },
    { 0xe4b71f0, 76 },
    { 0xe85b71f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb71f6, 38 },
    { 0xd97b71f7, 65 },
    { 0x7f2b71f8, 100 },
    { 0x889b71f9, 98 },
    { 0x70ab71f8, 90 },
    { 0x4dab71f6, 33 },
    { 0, 0 },
    { 0x29ab71fd, 66 },
    { 0, 0 },
    { 0x26db71ff, 76 },
    { 0xa4cb7200, 34 },
    { 0, 0 },
    { 0x810b7202, 34 },
    { 0xf34b7202, 46 },
    { 0x156b7204, 66 },
    { 0xf14b7202, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79b720d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ab7210, 61 },
    { 0x95fb7210, 23 },
    { 0x10eb7211, 46 },
    { 0x63db7213, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b7216, 68 },
    { 0, 0 },
    { 0xefbb7218, 90 },
    { 0, 0 },
    { 0x850b721a, 34 },
    { 0, 0 },
    { 0xa49b721c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309b7220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb7223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800b7230, 43 },
    { 0x322b7230, 95 },
    { 0x517b7230, 100 },
    { 0, 0 },
    { 0x8e3b7234, 67 },
    { 0xdedb7235, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b723c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4b723f, 95 },
    { 0xad1b7240, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5b7243, 52 },
    { 0, 0 },
    { 0x83eb7245, 33 },
    { 0xbbcb7245, 61 },
    { 0xf62b7245, 66 },
    { 0xf23b7245, 23 },
    { 0x2d5b7249, 52 },
    { 0x948b724a, 33 },
    { 0xe4fb724a, 17 },
    { 0x6dab724c, 33 },
    { 0x142b724d, 90 },
    { 0, 0 },
    { 0x9efb724f, 95 },
    { 0xeabb7250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb7257, 65 },
    { 0x49fb7258, 61 },
    { 0, 0 },
    { 0x3ffb725a, 90 },
    { 0x62db725b, 78 },
    { 0, 0 },
    { 0x877b725d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bb7260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790b7264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b7269, 61 },
    { 0x1f2b726a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb726d, 67 },
    { 0x33eb726e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fb7271, 33 },
    { 0x255b7272, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb7275, 76 },
    { 0xb57b7276, 98 },
    { 0xeb3b7277, 90 },
    { 0x5a7b7278, 93 },
    { 0xf2ab7278, 98 },
    { 0x36db727a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb727e, 69 },
    { 0, 0 },
    { 0xd8ab7280, 75 },
    { 0, 0 },
    { 0xbfab7282, 49 },
    { 0xcbdb7282, 67 },
    { 0x6b2b7283, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b7288, 78 },
    { 0x154b7289, 93 },
    { 0xf69b7288, 33 },
    { 0x1e2b728b, 90 },
    { 0x5dbb728b, 61 },
    { 0xb0b728b, 61 },
    { 0xbf6b728c, 98 },
    { 0, 0 },
    { 0xdb8b7290, 95 },
    { 0x55eb7290, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eb7299, 33 },
    { 0xd88b7299, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb729e, 84 },
    { 0xf5fb729f, 61 },
    { 0x192b72a0, 90 },
    { 0x540b729e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbab72a4, 43 },
    { 0, 0 },
    { 0x3b2b72a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b72ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb72ae, 61 },
    { 0, 0 },
    { 0x4b2b72b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b72b5, 91 },
    { 0x197b72b6, 85 },
    { 0x153b72b7, 33 },
    { 0xed2b72b8, 55 },
    { 0, 0 },
    { 0x48eb72ba, 61 },
    { 0xf3ab72ba, 34 },
    { 0x48cb72ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1b72c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb72ca, 46 },
    { 0xafdb72cb, 61 },
    { 0x68cb72cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb72d2, 61 },
    { 0, 0 },
    { 0x9abb72d4, 98 },
    { 0, 0 },
    { 0x3dfb72d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x22eb72d9, 23 },
    { 0, 0 },
    { 0xbd3b72db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592b72e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b72e9, 34 },
    { 0xbc7b72ea, 90 },
    { 0, 0 },
    { 0xb17b72ec, 46 },
    { 0x3e6b72ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bb72f0, 95 },
    { 0xd06b72f0, 90 },
    { 0x485b72f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b72f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x295b72fb, 23 },
    { 0, 0 },
    { 0xb7ab72fd, 90 },
    { 0x62fb72fe, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1b7308, 23 },
    { 0xf5b7309, 77 },
    { 0, 0 },
    { 0x6f5b730b, 93 },
    { 0x323b730c, 95 },
    { 0x761b730b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b7315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceb7318, 33 },
    { 0, 0 },
    { 0x4cab731a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddab731f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b7324, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08b7328, 23 },
    { 0, 0 },
    { 0x1ffb732a, 33 },
    { 0, 0 },
    { 0xaf3b732c, 76 },
    { 0xf10b732d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b7330, 90 },
    { 0x2ebb7331, 46 },
    { 0xf89b7332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb7339, 87 },
    { 0x5a2b733a, 100 },
    { 0x61bb733b, 46 },
    { 0x881b733b, 83 },
    { 0x366b7339, 61 },
    { 0xbffb733c, 34 },
    { 0xcc0b7339, 61 },
    { 0x3db7340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54db7349, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b734d, 69 },
    { 0xdd0b734e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bb7352, 36 },
    { 0x25cb7353, 90 },
    { 0x2fcb7354, 33 },
    { 0, 0 },
    { 0xee2b7356, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b735a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eb7362, 90 },
    { 0x4db7363, 95 },
    { 0xc60b7363, 84 },
    { 0, 0 },
    { 0xe51b7366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbb7369, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb736e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb7372, 23 },
    { 0x74eb7372, 99 },
    { 0x5e9b7374, 90 },
    { 0x4c6b7373, 91 },
    { 0x2e8b7376, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdb737a, 98 },
    { 0x96fb737b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782b737f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b7383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3b7388, 99 },
    { 0, 0 },
    { 0x57cb738a, 61 },
    { 0xf6ab738b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841b7393, 52 },
    { 0xca3b7393, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773b739e, 5 },
    { 0, 0 },
    { 0xfb1b73a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b73a5, 66 },
    { 0xbf4b73a5, 34 },
    { 0xb1ab73a7, 33 },
    { 0xd12b73a8, 43 },
    { 0x27db73a7, 100 },
    { 0, 0 },
    { 0xa38b73ab, 61 },
    { 0, 0 },
    { 0x8c4b73ad, 33 },
    { 0x2db73ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00b73b5, 98 },
    { 0x52bb73b6, 90 },
    { 0x511b73b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b73bb, 66 },
    { 0xba4b73bc, 23 },
    { 0xeedb73bd, 23 },
    { 0xc67b73bc, 61 },
    { 0xed2b73bf, 84 },
    { 0x971b73be, 61 },
    { 0xaa1b73c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb73c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb73ca, 61 },
    { 0x99ab73cb, 33 },
    { 0, 0 },
    { 0xcc9b73cd, 90 },
    { 0xd61b73ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0b73d2, 98 },
    { 0, 0 },
    { 0xf9db73d4, 95 },
    { 0xf9ab73d5, 17 },
    { 0xba5b73d4, 55 },
    { 0x9cab73d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b73db, 85 },
    { 0x201b73dc, 61 },
    { 0x7ab73dd, 98 },
    { 0x1fdb73de, 91 },
    { 0xcbbb73dd, 33 },
    { 0xa6cb73e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb73ef, 55 },
    { 0, 0 },
    { 0xbd2b73f1, 61 },
    { 0x1d0b73f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6b73f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53b73fd, 23 },
    { 0x2dcb73fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab7406, 78 },
    { 0xbb0b7407, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb740d, 61 },
    { 0x944b740e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb7411, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3b7414, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb7417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b741b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2db7420, 90 },
    { 0x3a2b7421, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ab7427, 43 },
    { 0xee9b7428, 23 },
    { 0xb2b7429, 90 },
    { 0, 0 },
    { 0xcacb742b, 43 },
    { 0x9f1b742c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500b7431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686b7438, 90 },
    { 0x2f6b7438, 23 },
    { 0, 0 },
    { 0xafeb743b, 95 },
    { 0x966b743c, 95 },
    { 0, 0 },
    { 0xc18b743e, 95 },
    { 0xbe9b743e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db7442, 78 },
    { 0xe5ab7443, 98 },
    { 0xe46b7442, 90 },
    { 0x300b7443, 43 },
    { 0, 0 },
    { 0x5e0b7447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b744a, 23 },
    { 0, 0 },
    { 0x50cb744c, 23 },
    { 0, 0 },
    { 0x1eab744e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b7453, 78 },
    { 0x77cb7453, 95 },
    { 0x74fb7455, 98 },
    { 0x72cb7454, 90 },
    { 0xdb5b7457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447b745b, 23 },
    { 0, 0 },
    { 0x8ccb745d, 77 },
    { 0xc4b745e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb7461, 76 },
    { 0, 0 },
    { 0xe7bb7463, 76 },
    { 0x439b7464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ab7468, 98 },
    { 0x92bb7469, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b746c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x361b746f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03b7478, 23 },
    { 0, 0 },
    { 0x3d4b747a, 33 },
    { 0xef1b747a, 90 },
    { 0x5c0b747c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b7480, 95 },
    { 0xeb7480, 23 },
    { 0x253b7481, 33 },
    { 0x47eb7483, 76 },
    { 0xd7db7484, 56 },
    { 0x391b7484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdb7488, 90 },
    { 0, 0 },
    { 0xc14b748a, 101 },
    { 0xbabb748b, 61 },
    { 0xde2b748c, 69 },
    { 0xa4db748d, 90 },
    { 0x93cb748d, 61 },
    { 0x1a3b748e, 95 },
    { 0xdd5b748d, 98 },
    { 0xd80b748d, 34 },
    { 0xf03b7492, 33 },
    { 0xa92b7493, 23 },
    { 0, 0 },
    { 0xac6b7495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b749b, 100 },
    { 0x228b749b, 33 },
    { 0xc52b749d, 23 },
    { 0x6f4b749d, 61 },
    { 0xf0bb749f, 23 },
    { 0, 0 },
    { 0x21cb74a1, 84 },
    { 0, 0 },
    { 0xb2ab74a3, 23 },
    { 0xaaeb74a3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b74a8, 87 },
    { 0xbb3b74a9, 23 },
    { 0x56fb74aa, 23 },
    { 0x14b74ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6b74b0, 95 },
    { 0xaf6b74b1, 61 },
    { 0, 0 },
    { 0xdd0b74b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb74b6, 98 },
    { 0x777b74b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b74bb, 90 },
    { 0, 0 },
    { 0xe02b74bd, 17 },
    { 0xb47b74bd, 90 },
    { 0x58ab74bd, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b74c8, 61 },
    { 0x61ab74c8, 24 },
    { 0, 0 },
    { 0xdcdb74cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb74d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b74d8, 45 },
    { 0xbc1b74d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b74de, 90 },
    { 0x843b74df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb74eb, 90 },
    { 0x2e8b74ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb74f2, 90 },
    { 0x36db74f2, 23 },
    { 0xf88b74f2, 23 },
    { 0x1d9b74f3, 33 },
    { 0x530b74f6, 76 },
    { 0, 0 },
    { 0x2c2b74f8, 43 },
    { 0x52cb74f9, 90 },
    { 0x9dbb74f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4b74fe, 33 },
    { 0, 0 },
    { 0x148b7500, 33 },
    { 0x181b7500, 23 },
    { 0xed4b7502, 77 },
    { 0, 0 },
    { 0x699b7504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b7511, 100 },
    { 0xdafb7511, 90 },
    { 0xfb1b7511, 33 },
    { 0x620b7514, 61 },
    { 0xab7512, 95 },
    { 0xf48b7515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feb751a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24b751f, 95 },
    { 0x67b751f, 23 },
    { 0xdcab751f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb7528, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8b753a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eb753e, 95 },
    { 0x60b753f, 53 },
    { 0x820b753e, 95 },
    { 0x631b7541, 95 },
    { 0, 0 },
    { 0x190b7543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b754d, 23 },
    { 0x748b754e, 33 },
    { 0x53ab754d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11db7552, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b7557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b755c, 23 },
    { 0x15cb755d, 38 },
    { 0x77b755d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b7561, 34 },
    { 0, 0 },
    { 0x769b7563, 95 },
    { 0x3fab7564, 23 },
    { 0, 0 },
    { 0x465b7566, 56 },
    { 0x86b7567, 93 },
    { 0x127b7566, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bb756b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebb7572, 100 },
    { 0xa68b7572, 34 },
    { 0x85bb7574, 23 },
    { 0, 0 },
    { 0xec0b7576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24b7579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ab757d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19b7583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb758e, 67 },
    { 0x989b758f, 98 },
    { 0x3a3b758f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb7596, 33 },
    { 0xa0cb7597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab759b, 33 },
    { 0x326b759b, 98 },
    { 0xfb9b759d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ab75a2, 81 },
    { 0, 0 },
    { 0x982b75a4, 33 },
    { 0x222b75a5, 33 },
    { 0x8bdb75a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacb75ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5b75b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebb75b5, 76 },
    { 0x18db75b5, 33 },
    { 0xdeb75b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11b75be, 77 },
    { 0xc89b75bf, 68 },
    { 0xc54b75c0, 88 },
    { 0x41fb75bf, 23 },
    { 0xa9db75c2, 87 },
    { 0, 0 },
    { 0x5b9b75c4, 38 },
    { 0x78ab75c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceb75c9, 90 },
    { 0x1db75c9, 99 },
    { 0x619b75c9, 61 },
    { 0, 0 },
    { 0x58bb75cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9b75d2, 93 },
    { 0, 0 },
    { 0x98cb75d4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x385b75d7, 98 },
    { 0x166b75d8, 23 },
    { 0x545b75d9, 99 },
    { 0xd18b75da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb75de, 90 },
    { 0xe15b75df, 55 },
    { 0x492b75df, 95 },
    { 0xb27b75e1, 98 },
    { 0xa01b75e1, 33 },
    { 0xccfb75e3, 94 },
    { 0x4cbb75e2, 43 },
    { 0x8b75e5, 95 },
    { 0, 0 },
    { 0x117b75e7, 67 },
    { 0x57b75e8, 95 },
    { 0, 0 },
    { 0x304b75ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b75ee, 76 },
    { 0x85ab75ef, 85 },
    { 0xc6cb75ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71eb75f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eb75fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb7601, 98 },
    { 0xc90b7601, 69 },
    { 0xf21b7601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79b7606, 83 },
    { 0, 0 },
    { 0x4f4b7608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2b760f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb7613, 17 },
    { 0, 0 },
    { 0xa56b7615, 90 },
    { 0xd2db7616, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b761a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcb761e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409b7622, 90 },
    { 0x143b7623, 46 },
    { 0x176b7623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eb7628, 61 },
    { 0xe27b7629, 43 },
    { 0, 0 },
    { 0xaadb762b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b7638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab7647, 100 },
    { 0, 0 },
    { 0xc5cb7649, 33 },
    { 0xee4b7649, 90 },
    { 0, 0 },
    { 0x52db764c, 43 },
    { 0x585b764c, 61 },
    { 0xc15b764e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b7652, 40 },
    { 0x7a3b7653, 98 },
    { 0x4d5b7654, 95 },
    { 0xed6b7655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ab765c, 98 },
    { 0, 0 },
    { 0x32fb765e, 34 },
    { 0x31ab765e, 90 },
    { 0xf76b765e, 70 },
    { 0xac7b765f, 93 },
    { 0x1b8b7662, 33 },
    { 0x1eeb7662, 17 },
    { 0, 0 },
    { 0x767b7665, 98 },
    { 0x4b9b7666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b7669, 61 },
    { 0x17db766a, 91 },
    { 0x1e9b766b, 43 },
    { 0, 0 },
    { 0x5a2b766d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45db7674, 98 },
    { 0, 0 },
    { 0xe48b7676, 91 },
    { 0xa03b7677, 98 },
    { 0xaf3b7678, 95 },
    { 0x743b7677, 76 },
    { 0, 0 },
    { 0xfbbb767b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62b7681, 77 },
    { 0xd1eb7682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975b7689, 98 },
    { 0xb91b768a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab768d, 55 },
    { 0x7b3b768d, 90 },
    { 0xc76b768d, 95 },
    { 0x8a6b7690, 78 },
    { 0x371b7691, 55 },
    { 0x3cdb7692, 55 },
    { 0x7eb7693, 66 },
    { 0x70cb7693, 98 },
    { 0, 0 },
    { 0x30bb7696, 46 },
    { 0xf0bb7697, 61 },
    { 0xba3b7698, 95 },
    { 0x71ab7699, 66 },
    { 0x8d3b7697, 61 },
    { 0, 0 },
    { 0x710b769c, 23 },
    { 0, 0 },
    { 0x4afb769e, 61 },
    { 0xd0eb769f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cb76a3, 95 },
    { 0x1edb76a4, 96 },
    { 0x9d5b76a5, 23 },
    { 0xb0ab76a3, 90 },
    { 0x652b76a7, 76 },
    { 0x24cb76a7, 40 },
    { 0x989b76a4, 95 },
    { 0xef4b76aa, 33 },
    { 0, 0 },
    { 0x50db76ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eb76b0, 55 },
    { 0, 0 },
    { 0xc7ab76b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21b76b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b76bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6b76c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b76c8, 90 },
    { 0x49fb76c9, 98 },
    { 0x1efb76ca, 23 },
    { 0x3f1b76cb, 40 },
    { 0x278b76cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b76d3, 23 },
    { 0, 0 },
    { 0xa2db76d5, 95 },
    { 0, 0 },
    { 0xcddb76d7, 69 },
    { 0x277b76d8, 33 },
    { 0x76bb76d8, 98 },
    { 0x443b76da, 90 },
    { 0xf9bb76d7, 33 },
    { 0xd4db76db, 23 },
    { 0x98cb76db, 95 },
    { 0x323b76de, 34 },
    { 0x3d2b76df, 61 },
    { 0xe47b76e0, 66 },
    { 0x51eb76e0, 98 },
    { 0x321b76da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb76e8, 52 },
    { 0x3e6b76e8, 34 },
    { 0xf25b76ea, 61 },
    { 0x813b76ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229b76f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863b76f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b76fd, 23 },
    { 0xdf1b76fe, 98 },
    { 0xd77b76fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7703, 33 },
    { 0xec4b7704, 98 },
    { 0, 0 },
    { 0x969b7706, 46 },
    { 0xc63b7707, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b770a, 90 },
    { 0, 0 },
    { 0x5dfb770c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ab770f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b771c, 98 },
    { 0x75bb771d, 76 },
    { 0x9f3b771d, 23 },
    { 0x79eb771e, 76 },
    { 0xebeb7720, 33 },
    { 0xc76b7721, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb7724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x414b7727, 61 },
    { 0xfbb7728, 98 },
    { 0xaabb7729, 34 },
    { 0xcfdb772a, 98 },
    { 0xd6b7729, 34 },
    { 0xfc2b7728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4b7736, 98 },
    { 0, 0 },
    { 0xd25b7738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41b773c, 55 },
    { 0, 0 },
    { 0x2beb773e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x898b7741, 33 },
    { 0x944b7742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab7748, 90 },
    { 0xf22b7748, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fab7752, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7b7756, 95 },
    { 0x8c8b7757, 93 },
    { 0xab7758, 98 },
    { 0xb8fb7757, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x579b7760, 87 },
    { 0, 0 },
    { 0xd0fb7762, 34 },
    { 0x831b7763, 61 },
    { 0x9c9b7762, 33 },
    { 0, 0 },
    { 0x129b7766, 90 },
    { 0x545b7767, 36 },
    { 0xbd9b7767, 33 },
    { 0xc90b7767, 84 },
    { 0x6a7b776a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b776f, 33 },
    { 0x943b776f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edb777a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2b777f, 33 },
    { 0xa91b7780, 33 },
    { 0xa43b7780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3b7785, 75 },
    { 0x744b7786, 55 },
    { 0xe97b7787, 34 },
    { 0x8a8b7788, 23 },
    { 0xf9b7787, 95 },
    { 0xb98b778a, 23 },
    { 0x21db7786, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b778e, 65 },
    { 0x3b2b778e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524b7793, 55 },
    { 0, 0 },
    { 0x51cb7795, 33 },
    { 0xe6ab7795, 55 },
    { 0x833b7797, 23 },
    { 0, 0 },
    { 0xdbab7799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b779f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29b77a4, 61 },
    { 0x819b77a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eb77a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb77ac, 61 },
    { 0xf54b77ac, 61 },
    { 0x9beb77ad, 95 },
    { 0xe43b77ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadb77b4, 23 },
    { 0xee0b77b4, 43 },
    { 0xfa3b77b5, 33 },
    { 0xf6db77b6, 43 },
    { 0x393b77b8, 61 },
    { 0x684b77b9, 67 },
    { 0x821b77b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ab77be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b77c5, 52 },
    { 0xdb4b77c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44b77ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b77ce, 98 },
    { 0xe51b77cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91b77d4, 87 },
    { 0x5e2b77d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab77d8, 76 },
    { 0x43ab77d9, 90 },
    { 0x201b77da, 84 },
    { 0, 0 },
    { 0x234b77dc, 33 },
    { 0x946b77dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b77e4, 55 },
    { 0x602b77e5, 23 },
    { 0x94fb77e5, 93 },
    { 0x26db77e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720b77ed, 65 },
    { 0xc0cb77ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb77f2, 95 },
    { 0xedfb77f3, 93 },
    { 0, 0 },
    { 0x7edb77f5, 96 },
    { 0, 0 },
    { 0xe4cb77f7, 33 },
    { 0x98b77f8, 76 },
    { 0x61ab77f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x774b77fc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304b7800, 77 },
    { 0, 0 },
    { 0x413b7802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1db780a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1b781b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958b781f, 99 },
    { 0, 0 },
    { 0xd41b7821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37db7827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942b7833, 33 },
    { 0, 0 },
    { 0xbe8b7835, 77 },
    { 0xafeb7835, 23 },
    { 0x3a9b7837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8b783b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4b7843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b7847, 33 },
    { 0x9bfb7848, 95 },
    { 0, 0 },
    { 0xb54b784a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b784d, 67 },
    { 0x180b784d, 76 },
    { 0, 0 },
    { 0xa93b7850, 23 },
    { 0x3fab7851, 90 },
    { 0x398b7852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bb7859, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb785d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fb7861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43b7864, 90 },
    { 0, 0 },
    { 0xa2db7866, 76 },
    { 0, 0 },
    { 0x823b7868, 33 },
    { 0x55db7869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab786d, 52 },
    { 0, 0 },
    { 0xcffb786f, 66 },
    { 0xb47b7870, 34 },
    { 0, 0 },
    { 0xe87b7872, 34 },
    { 0xa0ab7872, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86b7877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b787d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b7881, 76 },
    { 0xbb3b7882, 43 },
    { 0, 0 },
    { 0xbd4b7884, 76 },
    { 0, 0 },
    { 0x4afb7886, 33 },
    { 0, 0 },
    { 0x99ab7888, 67 },
    { 0, 0 },
    { 0x878b788a, 76 },
    { 0, 0 },
    { 0xdcdb788c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1db788f, 23 },
    { 0x1f6b7890, 99 },
    { 0x29bb7891, 76 },
    { 0x78fb7892, 43 },
    { 0xbf3b7891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b7896, 33 },
    { 0, 0 },
    { 0xf92b7898, 96 },
    { 0xe64b7898, 90 },
    { 0xf83b789a, 90 },
    { 0xa37b789b, 93 },
    { 0x6db789a, 95 },
    { 0xf7fb789d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fb78a0, 100 },
    { 0xb9cb78a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208b78a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906b78ab, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b78af, 90 },
    { 0x8a2b78b0, 23 },
    { 0x2bfb78b1, 55 },
    { 0xd54b78b1, 95 },
    { 0x95db78b0, 87 },
    { 0xcf7b78b0, 76 },
    { 0, 0 },
    { 0xba4b78b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8b78ba, 90 },
    { 0, 0 },
    { 0xcab78bc, 90 },
    { 0x1e5b78bc, 90 },
    { 0xb31b78bc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262b78c2, 17 },
    { 0x256b78c3, 23 },
    { 0x322b78c4, 61 },
    { 0xa6ab78c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b78ca, 95 },
    { 0x66bb78ca, 43 },
    { 0xf25b78cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb78d0, 46 },
    { 0x31eb78d1, 17 },
    { 0x5d6b78d2, 90 },
    { 0xf44b78d3, 33 },
    { 0x7d3b78d3, 99 },
    { 0, 0 },
    { 0x207b78d6, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b78de, 43 },
    { 0x6e7b78df, 99 },
    { 0x82bb78e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x338b78e3, 85 },
    { 0x800b78e4, 95 },
    { 0x4dbb78e5, 46 },
    { 0, 0 },
    { 0xcf8b78e7, 34 },
    { 0x3d5b78e8, 55 },
    { 0, 0 },
    { 0xc99b78ea, 93 },
    { 0, 0 },
    { 0x31cb78ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b78f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1b78f6, 67 },
    { 0xcf2b78f7, 61 },
    { 0x48bb78f7, 95 },
    { 0xb22b78f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb78fc, 98 },
    { 0x403b78fd, 78 },
    { 0x96eb78fe, 78 },
    { 0x3e7b78fc, 61 },
    { 0xee5b7900, 76 },
    { 0xee9b7901, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebb7904, 90 },
    { 0xcb3b7904, 98 },
    { 0xf13b7906, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5b7909, 61 },
    { 0, 0 },
    { 0x855b790b, 61 },
    { 0xc7cb790c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b7910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eb7913, 55 },
    { 0x4bdb7913, 48 },
    { 0xa6db7914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb7918, 34 },
    { 0, 0 },
    { 0xf9b791a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb7923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cb7926, 66 },
    { 0x7f9b7927, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0b792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80b7930, 95 },
    { 0x794b7931, 95 },
    { 0, 0 },
    { 0xdc8b7933, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb7936, 90 },
    { 0x30eb7936, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab793e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cb7945, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb7948, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cb794b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b794f, 78 },
    { 0x11fb794f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180b7954, 61 },
    { 0, 0 },
    { 0x927b7956, 91 },
    { 0x436b7956, 90 },
    { 0x57fb7957, 98 },
    { 0xb3b7959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b795d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b7961, 90 },
    { 0, 0 },
    { 0x5b5b7963, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb7967, 46 },
    { 0, 0 },
    { 0x8fbb7969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4b7970, 34 },
    { 0, 0 },
    { 0x15eb7972, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3b7976, 91 },
    { 0xddb7977, 78 },
    { 0xc46b7978, 67 },
    { 0x138b7979, 61 },
    { 0, 0 },
    { 0xc83b797b, 100 },
    { 0, 0 },
    { 0x747b797d, 77 },
    { 0xfa2b797e, 33 },
    { 0, 0 },
    { 0x7a9b7980, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73b7984, 77 },
    { 0x84db7985, 43 },
    { 0x8a3b7986, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9db798a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b7993, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bb799a, 76 },
    { 0, 0 },
    { 0x585b799c, 93 },
    { 0, 0 },
    { 0x448b799e, 90 },
    { 0xef3b799f, 95 },
    { 0x743b799e, 95 },
    { 0x6d6b79a1, 95 },
    { 0xdacb79a2, 41 },
    { 0xd81b79a3, 98 },
    { 0x342b79a4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7b79aa, 99 },
    { 0x6dfb79ab, 34 },
    { 0x646b79ab, 95 },
    { 0, 0 },
    { 0xd82b79ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddab79b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3b79b9, 33 },
    { 0x594b79b9, 23 },
    { 0xa37b79b9, 34 },
    { 0x2f6b79bc, 76 },
    { 0xbdcb79bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb79c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdb79c6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126b79d0, 98 },
    { 0xef0b79d1, 80 },
    { 0x569b79d1, 98 },
    { 0x5e3b79d3, 93 },
    { 0x6c1b79d4, 52 },
    { 0xe9b79d4, 90 },
    { 0, 0 },
    { 0xf5cb79d7, 52 },
    { 0xf19b79d7, 61 },
    { 0xa33b79d9, 90 },
    { 0x3d6b79d8, 99 },
    { 0, 0 },
    { 0x63b79dc, 67 },
    { 0xef9b79dd, 23 },
    { 0xd29b79dd, 90 },
    { 0, 0 },
    { 0x88db79e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b79e3, 91 },
    { 0x97fb79e4, 83 },
    { 0x131b79e4, 61 },
    { 0x89ab79e4, 61 },
    { 0x1fab79e5, 23 },
    { 0xf3b79e8, 95 },
    { 0x457b79e9, 90 },
    { 0x3a1b79ea, 87 },
    { 0x501b79e6, 98 },
    { 0x5ab79e6, 76 },
    { 0xc4bb79ed, 61 },
    { 0x200b79ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb79f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b79f6, 95 },
    { 0xf59b79f7, 46 },
    { 0xdf8b79f8, 33 },
    { 0x96b79f9, 23 },
    { 0xff1b79f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b79fd, 33 },
    { 0xbcab79fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ab7a02, 56 },
    { 0x2eb7a02, 95 },
    { 0x27fb7a04, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db7a07, 55 },
    { 0, 0 },
    { 0x80cb7a09, 98 },
    { 0, 0 },
    { 0x4b4b7a0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb7a10, 95 },
    { 0x9f5b7a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6b7a14, 90 },
    { 0xa90b7a15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4b7a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2b7a29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adb7a2e, 90 },
    { 0, 0 },
    { 0x7eeb7a30, 98 },
    { 0x3f7b7a30, 34 },
    { 0xb84b7a32, 77 },
    { 0xa79b7a32, 43 },
    { 0x653b7a34, 95 },
    { 0xcd2b7a32, 90 },
    { 0, 0 },
    { 0x6ceb7a37, 98 },
    { 0x253b7a38, 33 },
    { 0, 0 },
    { 0x90cb7a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b7a3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7db7a40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bb7a45, 98 },
    { 0, 0 },
    { 0x955b7a47, 75 },
    { 0, 0 },
    { 0x8b7a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3b7a4d, 67 },
    { 0x279b7a4e, 23 },
    { 0, 0 },
    { 0x548b7a50, 96 },
    { 0, 0 },
    { 0x6b7a52, 100 },
    { 0x6ccb7a53, 66 },
    { 0, 0 },
    { 0x6ab7a55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b7a5d, 61 },
    { 0x9dfb7a5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab7a62, 95 },
    { 0xa1db7a62, 80 },
    { 0x29b7a64, 90 },
    { 0xa3cb7a65, 98 },
    { 0x408b7a65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccb7a70, 76 },
    { 0, 0 },
    { 0x726b7a72, 98 },
    { 0, 0 },
    { 0x702b7a74, 90 },
    { 0x941b7a74, 33 },
    { 0xb1db7a76, 95 },
    { 0x7b8b7a77, 61 },
    { 0x38db7a78, 55 },
    { 0x61cb7a78, 95 },
    { 0, 0 },
    { 0x283b7a7b, 67 },
    { 0, 0 },
    { 0xb0ab7a7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47b7a81, 34 },
    { 0x553b7a82, 61 },
    { 0x7cfb7a83, 33 },
    { 0, 0 },
    { 0xff1b7a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b7a88, 33 },
    { 0, 0 },
    { 0x50b7a8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbb7a94, 23 },
    { 0xc75b7a95, 76 },
    { 0xd19b7a96, 90 },
    { 0x3ecb7a96, 85 },
    { 0x186b7a98, 100 },
    { 0xbdab7a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb7aa7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670b7aae, 90 },
    { 0x4afb7aae, 95 },
    { 0x1c4b7aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fb7ab6, 98 },
    { 0x125b7ab7, 61 },
    { 0x36ab7ab7, 98 },
    { 0xfbab7ab7, 90 },
    { 0x343b7ab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b7ac4, 90 },
    { 0x333b7ac4, 90 },
    { 0x731b7ac6, 36 },
    { 0xd68b7ac6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7acb, 95 },
    { 0, 0 },
    { 0x846b7acd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b7ad4, 93 },
    { 0x78b7ad4, 95 },
    { 0x574b7ad6, 49 },
    { 0xf94b7ad7, 42 },
    { 0x852b7ad6, 87 },
    { 0, 0 },
    { 0xe57b7ada, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b7ae1, 77 },
    { 0xe7bb7ae2, 55 },
    { 0x85fb7ae3, 66 },
    { 0x867b7ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fb7aef, 61 },
    { 0x3dcb7aef, 55 },
    { 0xea3b7af1, 17 },
    { 0x7a2b7af2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb7af5, 100 },
    { 0x733b7af5, 33 },
    { 0xed0b7af7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39b7afd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cb7b00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fb7b03, 99 },
    { 0xd9fb7b04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacb7b07, 66 },
    { 0xec4b7b08, 68 },
    { 0x72bb7b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1db7b10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b7b16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689b7b1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b7b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb7b25, 33 },
    { 0x845b7b25, 90 },
    { 0x22db7b26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923b7b2c, 95 },
    { 0x561b7b2d, 91 },
    { 0x76b7b2e, 33 },
    { 0x22db7b2f, 46 },
    { 0xab9b7b30, 61 },
    { 0xddeb7b2f, 95 },
    { 0x4aeb7b32, 90 },
    { 0xbb7b31, 23 },
    { 0x5b3b7b34, 90 },
    { 0xc0ab7b34, 43 },
    { 0x716b7b36, 95 },
    { 0x92b7b32, 61 },
    { 0xb1cb7b38, 55 },
    { 0x9c4b7b38, 95 },
    { 0x862b7b3a, 81 },
    { 0x14cb7b3b, 95 },
    { 0xfbbb7b2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab7b42, 23 },
    { 0x651b7b43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9b7b47, 91 },
    { 0xc4b7b48, 90 },
    { 0x1d6b7b49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db7b4d, 23 },
    { 0, 0 },
    { 0x66eb7b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x616b7b52, 78 },
    { 0x344b7b53, 98 },
    { 0, 0 },
    { 0xc89b7b55, 69 },
    { 0, 0 },
    { 0xaf0b7b57, 84 },
    { 0, 0 },
    { 0xda4b7b59, 59 },
    { 0, 0 },
    { 0x5cb7b5b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb7b5e, 78 },
    { 0x262b7b5e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627b7b63, 61 },
    { 0x67fb7b63, 23 },
    { 0xfc8b7b64, 61 },
    { 0xbd1b7b66, 99 },
    { 0xadeb7b67, 95 },
    { 0x5acb7b66, 98 },
    { 0x861b7b69, 87 },
    { 0xa27b7b6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb7b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0b7b77, 43 },
    { 0x673b7b77, 100 },
    { 0, 0 },
    { 0x455b7b7a, 61 },
    { 0xb38b7b7b, 93 },
    { 0, 0 },
    { 0xb79b7b7d, 65 },
    { 0x310b7b7e, 88 },
    { 0xeddb7b7e, 81 },
    { 0xb84b7b7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b7b85, 98 },
    { 0, 0 },
    { 0xa44b7b87, 55 },
    { 0xddab7b87, 52 },
    { 0x5e6b7b88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x218b7b8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x122b7b8f, 95 },
    { 0, 0 },
    { 0x3b9b7b91, 100 },
    { 0x25b7b92, 80 },
    { 0x22cb7b91, 87 },
    { 0x968b7b91, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3b7b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb7ba3, 46 },
    { 0x1ceb7ba4, 33 },
    { 0x801b7ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edb7ba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68b7bac, 66 },
    { 0xc1b7bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb7bb4, 99 },
    { 0x62cb7bb5, 23 },
    { 0xa41b7bb6, 33 },
    { 0xf6bb7bb6, 81 },
    { 0xff6b7bb6, 61 },
    { 0x142b7bb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62b7bbc, 33 },
    { 0x8bb7bbd, 95 },
    { 0x37fb7bbe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fb7bc1, 95 },
    { 0xa6eb7bc2, 23 },
    { 0x328b7bc3, 33 },
    { 0x2d0b7bc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186b7bc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6b7bcd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fb7bd5, 85 },
    { 0xeb2b7bd5, 65 },
    { 0x880b7bd7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ab7bda, 33 },
    { 0, 0 },
    { 0x734b7bdc, 90 },
    { 0x430b7bdd, 23 },
    { 0x2b2b7bde, 55 },
    { 0xe7eb7bde, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb7be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32b7bea, 33 },
    { 0, 0 },
    { 0x4e0b7bec, 90 },
    { 0x481b7bec, 76 },
    { 0x39cb7bee, 33 },
    { 0x83db7bec, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67b7bf3, 90 },
    { 0xafab7bf4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317b7c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24b7c07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db7c0a, 51 },
    { 0xfbeb7c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fb7c0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63b7c12, 61 },
    { 0xbf6b7c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5b7c17, 95 },
    { 0xd91b7c18, 90 },
    { 0x66b7c18, 91 },
    { 0x748b7c18, 43 },
    { 0xc07b7c18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b7c20, 55 },
    { 0xebfb7c21, 95 },
    { 0, 0 },
    { 0xe15b7c23, 34 },
    { 0xa37b7c24, 33 },
    { 0, 0 },
    { 0x13eb7c26, 17 },
    { 0x131b7c27, 95 },
    { 0x49ab7c27, 95 },
    { 0xfb5b7c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b7c2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb7c31, 78 },
    { 0, 0 },
    { 0xf1eb7c33, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452b7c39, 34 },
    { 0, 0 },
    { 0xcd8b7c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b7c42, 98 },
    { 0xe63b7c42, 90 },
    { 0, 0 },
    { 0x301b7c45, 98 },
    { 0x17eb7c45, 23 },
    { 0x95bb7c47, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b7c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b7c4f, 61 },
    { 0x119b7c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23b7c53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b7c56, 65 },
    { 0, 0 },
    { 0xeb1b7c58, 95 },
    { 0x32eb7c59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8b7c5d, 90 },
    { 0x7bfb7c5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bb7c61, 61 },
    { 0x6fcb7c62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b7c69, 34 },
    { 0x32b7c6a, 61 },
    { 0x5a7b7c6a, 67 },
    { 0x8a1b7c6c, 95 },
    { 0x9cbb7c6d, 61 },
    { 0xab6b7c6e, 38 },
    { 0x3eb7c6c, 95 },
    { 0xad5b7c70, 87 },
    { 0x638b7c70, 98 },
    { 0xb3bb7c72, 90 },
    { 0x60db7c73, 34 },
    { 0x566b7c73, 33 },
    { 0x9a3b7c6d, 61 },
    { 0x867b7c76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66db7c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49b7c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381b7c92, 33 },
    { 0x646b7c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fb7c97, 33 },
    { 0x9d4b7c98, 90 },
    { 0xb41b7c99, 93 },
    { 0, 0 },
    { 0xf0db7c9b, 78 },
    { 0xb87b7c9b, 88 },
    { 0x88eb7c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffb7ca4, 98 },
    { 0x547b7ca4, 23 },
    { 0x467b7ca5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb7ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb7cac, 87 },
    { 0xf69b7cad, 61 },
    { 0xc5b7cac, 93 },
    { 0x675b7caf, 33 },
    { 0xd00b7cb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eb7cba, 61 },
    { 0, 0 },
    { 0xe78b7cbc, 95 },
    { 0, 0 },
    { 0xec8b7cbe, 76 },
    { 0x2c6b7cbf, 90 },
    { 0xef6b7cbe, 23 },
    { 0, 0 },
    { 0x90db7cc2, 33 },
    { 0xffcb7cc2, 84 },
    { 0, 0 },
    { 0xf29b7cc5, 93 },
    { 0x50ab7cc6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab7cd4, 34 },
    { 0, 0 },
    { 0x92db7cd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cb7cd9, 84 },
    { 0, 0 },
    { 0xa6ab7cdb, 99 },
    { 0x5f9b7cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab7ce3, 98 },
    { 0x1a9b7ce4, 43 },
    { 0, 0 },
    { 0x7c2b7ce6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2b7cec, 78 },
    { 0, 0 },
    { 0x589b7cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b7cf1, 49 },
    { 0xac5b7cf1, 55 },
    { 0x1bfb7cf3, 33 },
    { 0xe29b7cf2, 61 },
    { 0, 0 },
    { 0xfb9b7cf6, 100 },
    { 0, 0 },
    { 0x14cb7cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eb7d01, 55 },
    { 0xccb7d02, 43 },
    { 0xd1db7d01, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b7d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3b7d09, 95 },
    { 0, 0 },
    { 0x92eb7d0b, 98 },
    { 0, 0 },
    { 0x81cb7d0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b7d10, 33 },
    { 0, 0 },
    { 0x8bfb7d12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb7d15, 23 },
    { 0x548b7d16, 61 },
    { 0x37b7d17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b7d1a, 61 },
    { 0xcab7d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b7d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6b7d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b7d2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bb7d33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfb7d36, 97 },
    { 0, 0 },
    { 0x54b7d38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb7d3b, 33 },
    { 0, 0 },
    { 0xb45b7d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b7d43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b7d47, 76 },
    { 0xd2ab7d48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1b7d4b, 61 },
    { 0x827b7d4c, 68 },
    { 0x689b7d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1b7d51, 61 },
    { 0xfb6b7d52, 77 },
    { 0x496b7d53, 33 },
    { 0xc0b7d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5b7d5a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b7d5e, 48 },
    { 0x4f4b7d5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7d62, 33 },
    { 0x855b7d63, 23 },
    { 0xc19b7d64, 91 },
    { 0x3d2b7d63, 33 },
    { 0x54cb7d66, 61 },
    { 0, 0 },
    { 0xdc0b7d68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7d6d, 76 },
    { 0xb13b7d6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639b7d78, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccb7d7f, 43 },
    { 0x401b7d80, 34 },
    { 0x597b7d80, 61 },
    { 0, 0 },
    { 0xffcb7d83, 85 },
    { 0, 0 },
    { 0x81b7d85, 68 },
    { 0x92bb7d85, 33 },
    { 0xec8b7d85, 99 },
    { 0x510b7d88, 33 },
    { 0x28ab7d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b7d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b7d92, 90 },
    { 0xa6eb7d93, 95 },
    { 0x38fb7d94, 75 },
    { 0, 0 },
    { 0xc12b7d96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1b7d9f, 90 },
    { 0x65b7da0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x948b7da3, 23 },
    { 0, 0 },
    { 0x749b7da5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818b7da9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb7dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbb7daf, 34 },
    { 0x607b7db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7db3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cb7dbe, 95 },
    { 0, 0 },
    { 0xeeeb7dc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b7dc3, 95 },
    { 0x5cdb7dc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36b7dcd, 76 },
    { 0, 0 },
    { 0x187b7dcf, 98 },
    { 0, 0 },
    { 0x55fb7dd1, 34 },
    { 0, 0 },
    { 0xe6cb7dd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbb7dda, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36b7de2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb7de6, 61 },
    { 0x517b7de6, 23 },
    { 0xf95b7de8, 76 },
    { 0xaa6b7de8, 90 },
    { 0x831b7de9, 33 },
    { 0xcb7de9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7def, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb7df2, 23 },
    { 0x78eb7df2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b7df8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab7e01, 90 },
    { 0xb0b7e02, 90 },
    { 0xffab7e01, 17 },
    { 0x6f1b7e04, 93 },
    { 0xf85b7e05, 33 },
    { 0x8eb7e05, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x573b7e09, 90 },
    { 0x3e7b7e09, 99 },
    { 0, 0 },
    { 0xd2cb7e0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7e10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3b7e13, 33 },
    { 0, 0 },
    { 0x7c5b7e15, 93 },
    { 0, 0 },
    { 0x6fdb7e17, 84 },
    { 0xeacb7e17, 23 },
    { 0x593b7e19, 61 },
    { 0, 0 },
    { 0x382b7e1b, 90 },
    { 0, 0 },
    { 0xf9b7e1d, 99 },
    { 0, 0 },
    { 0x89eb7e1f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb7e22, 98 },
    { 0x6a6b7e22, 23 },
    { 0xac8b7e24, 100 },
    { 0xce5b7e25, 6 },
    { 0x9d2b7e24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382b7e2b, 61 },
    { 0xb7cb7e2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b7e2f, 98 },
    { 0, 0 },
    { 0xdd0b7e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b7e37, 99 },
    { 0x668b7e38, 55 },
    { 0xe3bb7e37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab7e3c, 90 },
    { 0, 0 },
    { 0xbefb7e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7e41, 98 },
    { 0, 0 },
    { 0xf4bb7e43, 60 },
    { 0x449b7e44, 90 },
    { 0x54bb7e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b7e4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39db7e50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7e54, 95 },
    { 0, 0 },
    { 0xf93b7e56, 98 },
    { 0x6aeb7e56, 100 },
    { 0xc9bb7e56, 95 },
    { 0x4a7b7e59, 61 },
    { 0x4aeb7e59, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8db7e5e, 23 },
    { 0, 0 },
    { 0xb6eb7e60, 61 },
    { 0, 0 },
    { 0xfc8b7e62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556b7e67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b7e73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb7e79, 90 },
    { 0x4cab7e7a, 68 },
    { 0, 0 },
    { 0xab6b7e7c, 61 },
    { 0, 0 },
    { 0xa9ab7e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb7e81, 99 },
    { 0x3cfb7e82, 66 },
    { 0x9a3b7e83, 84 },
    { 0xed8b7e83, 76 },
    { 0x7d6b7e85, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2b7e89, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361b7e8d, 95 },
    { 0xb77b7e8e, 90 },
    { 0x79db7e8e, 90 },
    { 0xa0fb7e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9b7e93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcb7e98, 52 },
    { 0x8c6b7e99, 98 },
    { 0x307b7e9a, 98 },
    { 0x711b7e9a, 55 },
    { 0x51db7e99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ab7ea1, 61 },
    { 0x5c9b7ea2, 91 },
    { 0x961b7ea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacb7ea6, 81 },
    { 0x758b7ea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eb7eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272b7eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941b7ebc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b7ec6, 87 },
    { 0x963b7ec7, 33 },
    { 0xa2bb7ec8, 52 },
    { 0, 0 },
    { 0x930b7eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6db7ecd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465b7ed1, 90 },
    { 0, 0 },
    { 0x71ab7ed3, 87 },
    { 0x292b7ed4, 34 },
    { 0, 0 },
    { 0x2c1b7ed6, 93 },
    { 0xad4b7ed6, 46 },
    { 0xcb4b7ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4b7edc, 90 },
    { 0xbfb7edd, 91 },
    { 0x45fb7edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b7ee4, 43 },
    { 0xd0fb7ee5, 98 },
    { 0xc9db7ee5, 81 },
    { 0x9e4b7ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb7eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b7eee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86eb7ef8, 90 },
    { 0, 0 },
    { 0x2d1b7efa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fb7f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb7f08, 76 },
    { 0x7a3b7f09, 52 },
    { 0x1d2b7f0a, 98 },
    { 0x943b7f08, 90 },
    { 0, 0 },
    { 0x7e4b7f0d, 95 },
    { 0xbc2b7f0e, 85 },
    { 0x41fb7f0e, 61 },
    { 0, 0 },
    { 0x8f9b7f11, 93 },
    { 0xbebb7f11, 91 },
    { 0, 0 },
    { 0xa91b7f14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107b7f18, 81 },
    { 0, 0 },
    { 0x2b1b7f1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb7f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286b7f22, 96 },
    { 0xb2fb7f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab7f26, 91 },
    { 0, 0 },
    { 0xa81b7f28, 99 },
    { 0xd84b7f29, 33 },
    { 0, 0 },
    { 0xd0bb7f2b, 81 },
    { 0xdf4b7f2b, 95 },
    { 0x40fb7f2d, 66 },
    { 0, 0 },
    { 0xb40b7f2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b7f34, 67 },
    { 0x99db7f35, 90 },
    { 0, 0 },
    { 0xd22b7f37, 33 },
    { 0xaf5b7f37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bb7f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9db7f3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab7f4d, 34 },
    { 0x90fb7f4e, 81 },
    { 0xf4bb7f4e, 87 },
    { 0, 0 },
    { 0xbdbb7f51, 90 },
    { 0, 0 },
    { 0x7a2b7f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b7f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7b7f5b, 61 },
    { 0xecfb7f5b, 98 },
    { 0xa2bb7f5d, 77 },
    { 0xc90b7f5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546b7f64, 43 },
    { 0xb38b7f65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb7f68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ab7f6c, 53 },
    { 0xa2cb7f6d, 95 },
    { 0x5a3b7f6e, 33 },
    { 0xda6b7f6f, 55 },
    { 0x280b7f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fb7f79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb7f7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b7f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cb7f87, 98 },
    { 0x413b7f88, 91 },
    { 0xa9db7f89, 36 },
    { 0x342b7f8a, 61 },
    { 0, 0 },
    { 0x74b7f8c, 90 },
    { 0xeefb7f8d, 55 },
    { 0xe72b7f8c, 23 },
    { 0, 0 },
    { 0xeb5b7f90, 34 },
    { 0xc9b7f91, 98 },
    { 0x844b7f91, 78 },
    { 0x675b7f90, 93 },
    { 0x729b7f90, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb7f9d, 65 },
    { 0x9bb7f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b7fa3, 33 },
    { 0, 0 },
    { 0xe32b7fa5, 93 },
    { 0, 0 },
    { 0xb95b7fa7, 98 },
    { 0x9fdb7fa7, 90 },
    { 0x312b7fa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6b7fb0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cb7fb3, 85 },
    { 0xc1eb7fb4, 23 },
    { 0x48b7fb5, 90 },
    { 0, 0 },
    { 0xa98b7fb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314b7fbb, 23 },
    { 0, 0 },
    { 0x35cb7fbd, 61 },
    { 0, 0 },
    { 0xe94b7fbf, 56 },
    { 0xfa0b7fbf, 33 },
    { 0x5db7fbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b7fc5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b7fc9, 87 },
    { 0xf48b7fc9, 95 },
    { 0x545b7fcb, 43 },
    { 0x8dab7fcc, 55 },
    { 0x1bfb7fcd, 90 },
    { 0, 0 },
    { 0x9b1b7fcf, 66 },
    { 0xd2fb7fd0, 90 },
    { 0x959b7fd1, 38 },
    { 0xe90b7fd2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287b7fd7, 61 },
    { 0x824b7fd7, 98 },
    { 0, 0 },
    { 0xf32b7fda, 23 },
    { 0, 0 },
    { 0x277b7fdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868b7fe6, 90 },
    { 0x616b7fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05b7fee, 75 },
    { 0xe1b7fef, 90 },
    { 0x9f1b7ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6b7ff5, 95 },
    { 0xdd1b7ff6, 33 },
    { 0x950b7ff7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ab7ffa, 95 },
    { 0, 0 },
    { 0x5fab7ffc, 33 },
    { 0xf12b7ffd, 30 },
    { 0x5bcb7ffe, 91 },
    { 0xbdab7fff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0b8005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b8009, 76 },
    { 0, 0 },
    { 0x549b800b, 34 },
    { 0x91b800c, 95 },
    { 0x50bb800d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b8012, 78 },
    { 0, 0 },
    { 0xa74b8014, 90 },
    { 0x139b8015, 61 },
    { 0xfc2b8016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b8019, 33 },
    { 0, 0 },
    { 0x1e5b801b, 23 },
    { 0xfc0b801c, 95 },
    { 0x96cb801d, 93 },
    { 0xc91b801e, 33 },
    { 0xef0b801c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b8022, 23 },
    { 0xdb2b8023, 33 },
    { 0x858b8023, 33 },
    { 0, 0 },
    { 0x698b8026, 34 },
    { 0, 0 },
    { 0x35ab8028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3b802e, 34 },
    { 0, 0 },
    { 0x381b8030, 68 },
    { 0xb51b8031, 90 },
    { 0x1c1b8031, 90 },
    { 0x408b8033, 93 },
    { 0x2a2b8034, 95 },
    { 0x2abb8034, 34 },
    { 0x6d4b8036, 98 },
    { 0x1e8b8036, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572b803c, 75 },
    { 0x1ceb803d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b8041, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b8044, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b8047, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db804c, 98 },
    { 0, 0 },
    { 0x87cb804e, 76 },
    { 0x35b804e, 95 },
    { 0, 0 },
    { 0x85ab8051, 33 },
    { 0, 0 },
    { 0x5a6b8053, 90 },
    { 0, 0 },
    { 0xb4fb8055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb8058, 33 },
    { 0x13eb8059, 23 },
    { 0, 0 },
    { 0x899b805b, 95 },
    { 0x3a9b805b, 61 },
    { 0, 0 },
    { 0x36b805e, 90 },
    { 0x4e9b805f, 99 },
    { 0x3b5b8060, 90 },
    { 0xa73b805f, 43 },
    { 0, 0 },
    { 0xaacb8063, 33 },
    { 0, 0 },
    { 0x572b8065, 61 },
    { 0x39fb8066, 95 },
    { 0x490b8065, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b806b, 93 },
    { 0xba9b806c, 98 },
    { 0x3ecb806c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb807b, 90 },
    { 0x291b807c, 33 },
    { 0, 0 },
    { 0xb2b807e, 91 },
    { 0xf24b807f, 91 },
    { 0xa4eb8080, 55 },
    { 0, 0 },
    { 0x3ab8082, 66 },
    { 0xcc7b8082, 91 },
    { 0x643b8084, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b8088, 87 },
    { 0, 0 },
    { 0xd03b808a, 23 },
    { 0, 0 },
    { 0xd4ab808c, 88 },
    { 0x49ab808c, 23 },
    { 0xafeb808e, 98 },
    { 0x7f2b808f, 66 },
    { 0x24bb8090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b8093, 23 },
    { 0xdabb8094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b8097, 33 },
    { 0x88ab8097, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb809c, 43 },
    { 0x578b809d, 95 },
    { 0x31db809e, 93 },
    { 0x24cb809e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b80a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6b80a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab80a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1b80ad, 84 },
    { 0x7f7b80ad, 33 },
    { 0xcd6b80ae, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab80b4, 81 },
    { 0x8bcb80b5, 23 },
    { 0, 0 },
    { 0xc30b80b7, 27 },
    { 0, 0 },
    { 0x50eb80b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bb80c4, 90 },
    { 0, 0 },
    { 0x7ecb80c6, 23 },
    { 0, 0 },
    { 0x6c0b80c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b80cc, 90 },
    { 0, 0 },
    { 0xa20b80ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb80d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b80d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb80da, 93 },
    { 0, 0 },
    { 0xa6bb80dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b80e0, 82 },
    { 0x95db80e1, 40 },
    { 0, 0 },
    { 0xaa3b80e3, 33 },
    { 0, 0 },
    { 0x80cb80e5, 66 },
    { 0x840b80e6, 34 },
    { 0, 0 },
    { 0x2e5b80e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ab80ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccb80f0, 90 },
    { 0, 0 },
    { 0x30db80f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09b80f5, 38 },
    { 0xf3ab80f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb80fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b8103, 61 },
    { 0x386b8104, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ab8107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedb810a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x178b810d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cb8111, 90 },
    { 0x21fb8112, 34 },
    { 0, 0 },
    { 0x1b4b8114, 23 },
    { 0, 0 },
    { 0x671b8116, 61 },
    { 0x47bb8116, 95 },
    { 0xf78b8118, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb811b, 36 },
    { 0, 0 },
    { 0x5bfb811d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8124, 33 },
    { 0, 0 },
    { 0x676b8126, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557b812a, 90 },
    { 0xdbdb812b, 66 },
    { 0x817b812a, 34 },
    { 0xde6b812a, 23 },
    { 0xc87b812e, 90 },
    { 0xfedb812e, 61 },
    { 0x8a2b8130, 33 },
    { 0xe98b8131, 98 },
    { 0xc5ab8130, 84 },
    { 0, 0 },
    { 0x41ab8134, 55 },
    { 0, 0 },
    { 0xa8cb8136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bb813a, 90 },
    { 0x4aab813b, 33 },
    { 0xe2eb813c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8b813f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb8144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953b8148, 66 },
    { 0xa24b8149, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cb814d, 52 },
    { 0, 0 },
    { 0xfd8b814f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb8152, 91 },
    { 0x4d4b8153, 33 },
    { 0x57bb8152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b815a, 61 },
    { 0xd93b815b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b8162, 33 },
    { 0x614b8162, 90 },
    { 0x402b8164, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b8167, 76 },
    { 0xd6fb8168, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234b816c, 61 },
    { 0x53cb816c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5b8172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeb8177, 90 },
    { 0x8bbb8177, 33 },
    { 0, 0 },
    { 0x6f9b817a, 84 },
    { 0xfa0b817a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b8180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b8184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbb8188, 98 },
    { 0x223b8189, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b818e, 90 },
    { 0, 0 },
    { 0xa3bb8190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb8193, 61 },
    { 0x199b8194, 34 },
    { 0, 0 },
    { 0xf7db8196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb8199, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b819f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x755b81a2, 61 },
    { 0x350b81a2, 17 },
    { 0xd9b81a4, 23 },
    { 0x294b81a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db81ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b81af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b81b3, 90 },
    { 0x23eb81b4, 23 },
    { 0xce8b81b5, 68 },
    { 0x5e4b81b5, 95 },
    { 0x42bb81b4, 61 },
    { 0x343b81b8, 23 },
    { 0xed6b81b5, 76 },
    { 0x883b81ba, 61 },
    { 0x1aeb81b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0b81c0, 91 },
    { 0x818b81c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dab81c4, 66 },
    { 0, 0 },
    { 0x186b81c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab81cb, 98 },
    { 0x687b81cb, 17 },
    { 0x1f5b81cb, 67 },
    { 0, 0 },
    { 0xccab81cf, 95 },
    { 0x94b81cf, 76 },
    { 0x763b81d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeb81db, 91 },
    { 0x2a8b81dc, 95 },
    { 0x62eb81dd, 61 },
    { 0x214b81de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16db81e4, 33 },
    { 0x88fb81e5, 90 },
    { 0x92db81e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb81ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb81fb, 100 },
    { 0x666b81fb, 43 },
    { 0xdab81fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b8202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94db8206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523b820c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x247b820f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb821b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93b821e, 95 },
    { 0x6a5b821f, 46 },
    { 0, 0 },
    { 0xcbdb8221, 93 },
    { 0x8e5b8221, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb8225, 90 },
    { 0xe07b8226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6b822b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb822e, 90 },
    { 0xeddb822e, 85 },
    { 0, 0 },
    { 0xa7bb8231, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbb8234, 33 },
    { 0xd79b8235, 23 },
    { 0xdfdb8236, 52 },
    { 0x906b8236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9b8240, 43 },
    { 0x1c5b8240, 97 },
    { 0x5eb8242, 95 },
    { 0xfc0b8242, 55 },
    { 0xb1eb8244, 64 },
    { 0, 0 },
    { 0xd6cb8246, 33 },
    { 0xd45b8246, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eb824c, 98 },
    { 0x30eb824c, 95 },
    { 0, 0 },
    { 0x500b824f, 34 },
    { 0xf63b8250, 90 },
    { 0xc44b824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439b8256, 98 },
    { 0x5fb8257, 77 },
    { 0, 0 },
    { 0x8eab8259, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b825c, 43 },
    { 0xde9b825d, 55 },
    { 0x7e1b825e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48b8262, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19b826a, 23 },
    { 0xf94b826b, 68 },
    { 0x567b826c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b8270, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eb8274, 84 },
    { 0xfe2b8275, 46 },
    { 0x7a3b8276, 68 },
    { 0x5c1b8276, 95 },
    { 0xbd8b8278, 98 },
    { 0x517b8277, 99 },
    { 0, 0 },
    { 0x66fb827b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb827e, 43 },
    { 0, 0 },
    { 0xf74b8280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373b8284, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903b8288, 78 },
    { 0xf24b8288, 61 },
    { 0, 0 },
    { 0x773b828b, 76 },
    { 0x846b828b, 90 },
    { 0x20eb828c, 99 },
    { 0x93ab828c, 61 },
    { 0xb20b828c, 33 },
    { 0x3e8b8290, 99 },
    { 0x4bbb8291, 76 },
    { 0x603b8292, 90 },
    { 0x702b8293, 75 },
    { 0x4b4b828c, 76 },
    { 0x9d2b828f, 23 },
    { 0, 0 },
    { 0xa50b8297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b829b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87b82a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1b82ac, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689b82b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b82b9, 95 },
    { 0, 0 },
    { 0xba2b82bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b82be, 23 },
    { 0xd8eb82bf, 34 },
    { 0xb9bb82c0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01b82c3, 46 },
    { 0x6ecb82c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0b82ce, 91 },
    { 0, 0 },
    { 0x9e6b82d0, 52 },
    { 0, 0 },
    { 0xab7b82d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b82d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b82db, 90 },
    { 0xe07b82dc, 61 },
    { 0x887b82dd, 98 },
    { 0, 0 },
    { 0xdefb82df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x385b82e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bb82e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b82e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceb82f0, 33 },
    { 0x302b82f0, 34 },
    { 0x69b82f2, 90 },
    { 0, 0 },
    { 0x63bb82f4, 52 },
    { 0x83fb82f5, 90 },
    { 0x6ddb82f6, 61 },
    { 0x3b8b82f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84b82fe, 95 },
    { 0x88b82fe, 76 },
    { 0, 0 },
    { 0xb42b8301, 90 },
    { 0x3e4b8301, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bb8309, 46 },
    { 0, 0 },
    { 0x306b830b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467b8310, 95 },
    { 0xb8cb8311, 23 },
    { 0xb0eb8310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb8316, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb831f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2b832d, 95 },
    { 0xab6b832e, 34 },
    { 0, 0 },
    { 0xa4db8330, 90 },
    { 0x274b8331, 34 },
    { 0, 0 },
    { 0x5a2b8333, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b8338, 81 },
    { 0x5e6b8339, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5b833c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8b8342, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb834b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ab835a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab835d, 23 },
    { 0, 0 },
    { 0xbb9b835f, 91 },
    { 0x2c9b8360, 23 },
    { 0x21db835f, 95 },
    { 0, 0 },
    { 0xa53b8363, 23 },
    { 0x61cb8364, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb8368, 76 },
    { 0x2c9b8368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62b836e, 61 },
    { 0xdf2b836f, 76 },
    { 0, 0 },
    { 0x75bb8371, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcb8375, 33 },
    { 0x509b8376, 76 },
    { 0x9db8377, 90 },
    { 0, 0 },
    { 0xbc6b8379, 76 },
    { 0x3cbb837a, 33 },
    { 0x877b837a, 100 },
    { 0x8c2b837a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b837f, 90 },
    { 0x241b8380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2b8384, 55 },
    { 0xc22b8385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb838c, 87 },
    { 0xd8fb838d, 74 },
    { 0x315b838c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451b8394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35b839a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b839e, 90 },
    { 0xb1cb839f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b83a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4b83a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01b83aa, 34 },
    { 0x85fb83ab, 87 },
    { 0x5ddb83ab, 95 },
    { 0x264b83ad, 90 },
    { 0x52ab83ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1b83b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb83b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb83b9, 43 },
    { 0x902b83ba, 76 },
    { 0xf8ab83ba, 100 },
    { 0xb5ab83bb, 33 },
    { 0, 0 },
    { 0x98bb83be, 84 },
    { 0, 0 },
    { 0xe61b83c0, 56 },
    { 0x78bb83c0, 23 },
    { 0xd10b83c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b83c9, 99 },
    { 0x87cb83ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b83ce, 36 },
    { 0x2d4b83ce, 65 },
    { 0xd21b83d0, 66 },
    { 0, 0 },
    { 0xe0bb83d2, 96 },
    { 0x80eb83d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56b83d7, 59 },
    { 0xcdab83d8, 33 },
    { 0x6edb83d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb83dd, 77 },
    { 0xdb0b83dd, 66 },
    { 0x18ab83df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9b83e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133b83e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb83eb, 81 },
    { 0, 0 },
    { 0x2b9b83ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ab83f1, 33 },
    { 0xe52b83f1, 43 },
    { 0x901b83f3, 91 },
    { 0, 0 },
    { 0xdcbb83f5, 99 },
    { 0x999b83f6, 46 },
    { 0x57db83f6, 34 },
    { 0x975b83f8, 61 },
    { 0, 0 },
    { 0x2f1b83fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b83ff, 80 },
    { 0x3e8b8400, 99 },
    { 0x1c2b83ff, 43 },
    { 0x659b8402, 98 },
    { 0xeeab8400, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b8408, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922b840d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb8411, 56 },
    { 0x8cb8412, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b8417, 34 },
    { 0xeb3b8418, 52 },
    { 0x6b0b8418, 95 },
    { 0xc69b841a, 61 },
    { 0x1b6b8419, 99 },
    { 0xb80b841b, 33 },
    { 0xe31b841b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65b8423, 46 },
    { 0xd96b8424, 61 },
    { 0x60db8423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb842a, 90 },
    { 0x775b842b, 90 },
    { 0xafeb842b, 95 },
    { 0x31cb842c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eb8432, 90 },
    { 0, 0 },
    { 0xb7db8434, 90 },
    { 0, 0 },
    { 0x9d2b8436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3b843c, 95 },
    { 0x934b843d, 90 },
    { 0x4b4b843c, 67 },
    { 0xeaeb843f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3b8442, 90 },
    { 0, 0 },
    { 0x258b8444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b8448, 76 },
    { 0x222b8449, 98 },
    { 0xb9bb844a, 90 },
    { 0xcfeb844b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812b8451, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b8454, 98 },
    { 0, 0 },
    { 0x814b8456, 66 },
    { 0x2b3b8457, 61 },
    { 0x36eb8458, 72 },
    { 0x9f2b8459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb845c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab845f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b8462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b8465, 91 },
    { 0x40eb8465, 89 },
    { 0x967b8467, 96 },
    { 0xed6b8468, 95 },
    { 0xabfb8465, 87 },
    { 0xd66b8468, 61 },
    { 0x73bb8468, 23 },
    { 0, 0 },
    { 0xec7b846d, 33 },
    { 0x9e2b846e, 44 },
    { 0x960b846f, 67 },
    { 0xf05b8470, 76 },
    { 0, 0 },
    { 0xbfdb8472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ab8477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b847a, 77 },
    { 0x95bb847b, 80 },
    { 0, 0 },
    { 0xa36b847d, 76 },
    { 0x9fb847d, 33 },
    { 0x968b847f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b8487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b848c, 95 },
    { 0x16eb848d, 90 },
    { 0xfebb848d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79db8491, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x917b8494, 98 },
    { 0xc0fb8494, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b8498, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5b849e, 84 },
    { 0x956b849e, 61 },
    { 0x934b849e, 93 },
    { 0, 0 },
    { 0x46fb84a2, 95 },
    { 0x70fb84a3, 46 },
    { 0, 0 },
    { 0xbd8b84a5, 90 },
    { 0xd81b84a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb84aa, 98 },
    { 0xebb84aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x358b84ae, 98 },
    { 0xd6db84af, 90 },
    { 0x7a1b84ae, 61 },
    { 0xec9b84b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb84b6, 33 },
    { 0xf4eb84b6, 95 },
    { 0xa7bb84b8, 98 },
    { 0x3dbb84b9, 95 },
    { 0, 0 },
    { 0xb7db84bb, 85 },
    { 0xbefb84bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab84c1, 34 },
    { 0xb6bb84c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bb84ca, 99 },
    { 0xac1b84cb, 9 },
    { 0, 0 },
    { 0xd82b84cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b84d0, 100 },
    { 0x179b84d1, 95 },
    { 0x9a1b84d1, 98 },
    { 0xac8b84d3, 76 },
    { 0, 0 },
    { 0x5c7b84d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab84dd, 55 },
    { 0x73eb84de, 43 },
    { 0, 0 },
    { 0x3efb84e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb84e8, 95 },
    { 0, 0 },
    { 0xd31b84ea, 90 },
    { 0x837b84ea, 43 },
    { 0, 0 },
    { 0x8b9b84ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41b84f0, 75 },
    { 0, 0 },
    { 0x42db84f2, 75 },
    { 0, 0 },
    { 0xe12b84f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413b84fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77b84fe, 91 },
    { 0xac3b84ff, 92 },
    { 0xcccb8500, 98 },
    { 0x630b8500, 81 },
    { 0xcc4b84fe, 34 },
    { 0xd6ab84fe, 46 },
    { 0x902b8504, 91 },
    { 0x8acb8505, 80 },
    { 0xb57b8506, 46 },
    { 0, 0 },
    { 0x3beb8508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb850e, 61 },
    { 0xe66b850f, 23 },
    { 0xbf5b850e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab8514, 98 },
    { 0x725b8515, 98 },
    { 0x2cfb8516, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b8519, 90 },
    { 0, 0 },
    { 0xc1bb851b, 23 },
    { 0x929b851c, 88 },
    { 0x8b8b851c, 76 },
    { 0x9eeb851e, 61 },
    { 0x67eb851e, 98 },
    { 0xb55b8520, 61 },
    { 0x974b8521, 33 },
    { 0x788b8521, 34 },
    { 0xdabb8520, 61 },
    { 0x73eb8524, 34 },
    { 0xc4fb8525, 99 },
    { 0x451b8526, 95 },
    { 0, 0 },
    { 0xafcb8528, 90 },
    { 0x18eb8529, 98 },
    { 0, 0 },
    { 0x9eb852b, 90 },
    { 0x961b852c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b8530, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3b8534, 95 },
    { 0x614b8535, 33 },
    { 0xc77b8536, 61 },
    { 0, 0 },
    { 0x4a6b8538, 95 },
    { 0xba4b8538, 55 },
    { 0, 0 },
    { 0xe56b853b, 61 },
    { 0, 0 },
    { 0x239b853d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb8541, 85 },
    { 0x3b4b8542, 84 },
    { 0, 0 },
    { 0x892b8544, 95 },
    { 0xbd7b8545, 55 },
    { 0xfc5b8544, 98 },
    { 0xa5eb8547, 98 },
    { 0xa29b8548, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cb854c, 43 },
    { 0x508b854c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b8554, 76 },
    { 0x2db8554, 61 },
    { 0, 0 },
    { 0x326b8557, 88 },
    { 0x691b8557, 90 },
    { 0, 0 },
    { 0xc63b855a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7b8565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448b856e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b8573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323b857c, 33 },
    { 0, 0 },
    { 0xb44b857e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317b8582, 67 },
    { 0x352b8583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eb8589, 61 },
    { 0xb1fb8589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab8591, 61 },
    { 0x6adb8592, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b8597, 90 },
    { 0, 0 },
    { 0x977b8599, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab859d, 76 },
    { 0x57fb859e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b85a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327b85ad, 76 },
    { 0, 0 },
    { 0x87db85af, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742b85b3, 98 },
    { 0x25b85b4, 23 },
    { 0xadb85b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb85bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x412b85be, 33 },
    { 0, 0 },
    { 0xed0b85c0, 99 },
    { 0x8b1b85c1, 98 },
    { 0x618b85c2, 98 },
    { 0xb4ab85c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb85c8, 76 },
    { 0, 0 },
    { 0x539b85ca, 88 },
    { 0, 0 },
    { 0xdaeb85cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451b85d7, 95 },
    { 0xa37b85d8, 33 },
    { 0x64eb85d7, 90 },
    { 0, 0 },
    { 0x3c8b85db, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b85de, 33 },
    { 0, 0 },
    { 0xd75b85e0, 76 },
    { 0, 0 },
    { 0xf2eb85e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560b85ea, 23 },
    { 0x412b85eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b85ee, 76 },
    { 0x7e8b85ee, 95 },
    { 0x19b85ee, 95 },
    { 0x597b85f1, 61 },
    { 0x1c9b85f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b85f5, 33 },
    { 0, 0 },
    { 0x93db85f7, 95 },
    { 0x2fb85f8, 33 },
    { 0xa56b85f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb85fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb8602, 100 },
    { 0, 0 },
    { 0xea0b8604, 33 },
    { 0xa63b8605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b860b, 90 },
    { 0xd60b860c, 61 },
    { 0x91b860d, 99 },
    { 0, 0 },
    { 0xc09b860f, 46 },
    { 0x513b8610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bb8613, 90 },
    { 0, 0 },
    { 0xf4cb8615, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0b861a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b861e, 61 },
    { 0x781b861e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b8626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99b862a, 98 },
    { 0x2d3b862b, 78 },
    { 0xdfeb862c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9b8633, 99 },
    { 0xc05b8633, 87 },
    { 0x804b8635, 33 },
    { 0x402b8635, 100 },
    { 0x2c6b8636, 90 },
    { 0x499b8633, 90 },
    { 0x1ceb8639, 65 },
    { 0xa62b863a, 95 },
    { 0xb46b8637, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667b8640, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21b8644, 98 },
    { 0x6afb8644, 61 },
    { 0xedbb8646, 20 },
    { 0xb2bb8646, 23 },
    { 0xe76b8647, 99 },
    { 0xb09b8649, 61 },
    { 0x44eb864a, 67 },
    { 0x89db864b, 90 },
    { 0xda1b864c, 99 },
    { 0xb6cb864d, 92 },
    { 0x84bb864e, 90 },
    { 0xb99b864f, 92 },
    { 0xbabb8650, 67 },
    { 0x288b8650, 93 },
    { 0xe1eb8652, 33 },
    { 0x2f7b8653, 88 },
    { 0xdab8654, 88 },
    { 0xf4b8652, 55 },
    { 0x6c2b8656, 34 },
    { 0x7e4b8657, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db865c, 44 },
    { 0x172b865d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eb866b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1b866f, 61 },
    { 0, 0 },
    { 0x54b8671, 81 },
    { 0, 0 },
    { 0x64bb8673, 75 },
    { 0x1eab8673, 98 },
    { 0x924b8674, 99 },
    { 0xa5eb8675, 90 },
    { 0, 0 },
    { 0xb6cb8678, 100 },
    { 0x79bb8678, 61 },
    { 0x9d3b8678, 95 },
    { 0x451b867b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71b867f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bb8685, 99 },
    { 0x13b8686, 99 },
    { 0, 0 },
    { 0xa94b8688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb868b, 33 },
    { 0x18bb868c, 90 },
    { 0xc19b868d, 95 },
    { 0x9deb868d, 33 },
    { 0xb96b868f, 88 },
    { 0xc3b868f, 23 },
    { 0x690b868e, 61 },
    { 0xcfab8692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb8697, 77 },
    { 0xd9cb8697, 55 },
    { 0x1fb8698, 98 },
    { 0x992b8699, 33 },
    { 0xecdb8697, 90 },
    { 0xd6db869c, 55 },
    { 0x17b8698, 78 },
    { 0, 0 },
    { 0xbb3b869f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b86a2, 90 },
    { 0x955b86a3, 84 },
    { 0x479b86a2, 90 },
    { 0x477b86a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab86a9, 77 },
    { 0, 0 },
    { 0xf16b86ab, 90 },
    { 0xc82b86ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2b86b3, 61 },
    { 0x7bfb86b3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bb86b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175b86bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cb86c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x502b86c5, 33 },
    { 0x5fcb86c6, 34 },
    { 0x189b86c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab86cc, 23 },
    { 0x45cb86cd, 99 },
    { 0x3b0b86cd, 34 },
    { 0xedb86cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb86d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6b86d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b86dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7b86df, 23 },
    { 0xdb3b86df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580b86e4, 98 },
    { 0, 0 },
    { 0x393b86e6, 76 },
    { 0x71bb86e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x751b86ea, 90 },
    { 0x3efb86ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b86ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b86f3, 23 },
    { 0x500b86f3, 95 },
    { 0x7ceb86f5, 61 },
    { 0x243b86f6, 33 },
    { 0x926b86f7, 100 },
    { 0x68db86f7, 61 },
    { 0, 0 },
    { 0xad6b86fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbb8701, 42 },
    { 0x9fcb8701, 77 },
    { 0x2c9b8701, 33 },
    { 0x951b8704, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b8708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8717, 90 },
    { 0xfaeb8718, 33 },
    { 0, 0 },
    { 0x917b871a, 91 },
    { 0x235b871b, 55 },
    { 0, 0 },
    { 0xcd5b871d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbb8722, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4b872a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b872d, 52 },
    { 0x906b872e, 17 },
    { 0xe02b872f, 98 },
    { 0x62bb8730, 33 },
    { 0x5a5b872f, 33 },
    { 0xb11b8731, 34 },
    { 0x2a7b8730, 90 },
    { 0, 0 },
    { 0xdf7b8735, 65 },
    { 0x5b8735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x335b8739, 41 },
    { 0x95eb873a, 33 },
    { 0x32db873b, 76 },
    { 0x4d0b873c, 23 },
    { 0, 0 },
    { 0x5eb873e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08b8741, 81 },
    { 0x394b8741, 81 },
    { 0xd96b8741, 100 },
    { 0xd3eb8744, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52eb874e, 78 },
    { 0x3c5b874f, 95 },
    { 0x685b8750, 33 },
    { 0, 0 },
    { 0x798b8752, 33 },
    { 0, 0 },
    { 0x865b8754, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12b875c, 90 },
    { 0x38bb875c, 33 },
    { 0, 0 },
    { 0x187b875f, 66 },
    { 0x467b8760, 88 },
    { 0xfbeb8761, 46 },
    { 0xb8cb8762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294b876c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffb8770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecb8775, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1b8778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb877c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb877f, 61 },
    { 0xc59b8780, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab8785, 65 },
    { 0x983b8786, 90 },
    { 0x8d3b8787, 95 },
    { 0x27db8785, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398b878d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bb8793, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b8796, 90 },
    { 0xf8eb8796, 34 },
    { 0x6d1b8798, 95 },
    { 0x1eab8799, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b879d, 33 },
    { 0x3fbb879e, 55 },
    { 0xc10b879f, 23 },
    { 0x6fb879e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b87a4, 90 },
    { 0x757b87a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b87a8, 33 },
    { 0x63fb87a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccb87ac, 77 },
    { 0, 0 },
    { 0xae0b87ae, 66 },
    { 0xd3eb87ae, 23 },
    { 0x346b87b0, 34 },
    { 0xa49b87b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b87b6, 23 },
    { 0xf4cb87b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b87bc, 21 },
    { 0xd14b87bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb87c1, 66 },
    { 0, 0 },
    { 0x4dbb87c3, 33 },
    { 0x47b87c3, 33 },
    { 0, 0 },
    { 0x63db87c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15b87ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b87d1, 81 },
    { 0x896b87d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b87d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282b87db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b87de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eb87e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1b87e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b87e9, 52 },
    { 0x146b87e9, 76 },
    { 0xa23b87e9, 90 },
    { 0xc41b87ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffb87f0, 55 },
    { 0xb8bb87f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb87f6, 61 },
    { 0xbe0b87f7, 61 },
    { 0x6deb87f8, 23 },
    { 0x36cb87f9, 61 },
    { 0, 0 },
    { 0xc97b87fb, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b87fe, 93 },
    { 0xaf7b87ff, 93 },
    { 0x8e7b8800, 93 },
    { 0xafcb8801, 38 },
    { 0x313b8801, 61 },
    { 0, 0 },
    { 0xc88b8804, 98 },
    { 0x78eb8805, 55 },
    { 0x2dfb8804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b880a, 99 },
    { 0xd14b880b, 46 },
    { 0x8adb880c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574b8810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b8814, 90 },
    { 0x206b8815, 90 },
    { 0x1cb8816, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fb881a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b881d, 23 },
    { 0xa9cb881e, 90 },
    { 0x6cbb881d, 4 },
    { 0x40b8820, 67 },
    { 0x2eeb8821, 100 },
    { 0, 0 },
    { 0x86db8823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b8826, 61 },
    { 0x3c4b8826, 55 },
    { 0x943b8828, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b882b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddb882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791b8837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db883b, 23 },
    { 0, 0 },
    { 0x427b883d, 95 },
    { 0xd54b883d, 23 },
    { 0xe4cb883e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cb8842, 98 },
    { 0, 0 },
    { 0xeb8b8844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eb8847, 61 },
    { 0, 0 },
    { 0xb2ab8849, 94 },
    { 0, 0 },
    { 0xb4db884b, 55 },
    { 0xbc9b884c, 33 },
    { 0x7b3b884d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdb8850, 36 },
    { 0x131b8851, 90 },
    { 0, 0 },
    { 0xb2cb8853, 34 },
    { 0xe2b8854, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb8858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beb885f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab8864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b8868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eb886b, 23 },
    { 0, 0 },
    { 0x8fdb886d, 61 },
    { 0x9b8b886e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3b8873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1b8876, 33 },
    { 0x926b8877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8b887c, 84 },
    { 0xd70b887c, 23 },
    { 0x432b887d, 61 },
    { 0xb10b887f, 98 },
    { 0, 0 },
    { 0x64b8881, 33 },
    { 0x73eb8882, 23 },
    { 0x7f6b8882, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b8888, 33 },
    { 0x647b8889, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992b888d, 75 },
    { 0, 0 },
    { 0x475b888f, 77 },
    { 0xdcfb888f, 93 },
    { 0xc90b8891, 68 },
    { 0, 0 },
    { 0xcceb8893, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56b8896, 95 },
    { 0x92fb8897, 61 },
    { 0, 0 },
    { 0x1f1b8899, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b88a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b88a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bb88aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb88b7, 98 },
    { 0xfc5b88b8, 99 },
    { 0, 0 },
    { 0xf47b88ba, 76 },
    { 0, 0 },
    { 0xac7b88bc, 91 },
    { 0xee9b88bc, 88 },
    { 0x114b88be, 65 },
    { 0x502b88be, 23 },
    { 0x5c2b88c0, 23 },
    { 0xce9b88be, 23 },
    { 0xc4eb88c2, 33 },
    { 0x67eb88c3, 81 },
    { 0, 0 },
    { 0xbc1b88c5, 55 },
    { 0, 0 },
    { 0x5d1b88c7, 95 },
    { 0x3c5b88c7, 33 },
    { 0x4f4b88c9, 23 },
    { 0, 0 },
    { 0xc27b88cb, 61 },
    { 0, 0 },
    { 0x954b88cd, 61 },
    { 0x5d9b88ce, 38 },
    { 0x9b8b88cd, 90 },
    { 0xbc5b88ce, 85 },
    { 0x38fb88d1, 33 },
    { 0, 0 },
    { 0xdeeb88d3, 98 },
    { 0, 0 },
    { 0xa30b88d5, 43 },
    { 0xdafb88d6, 98 },
    { 0x6a4b88d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b88da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fb88e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0b88e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb88e7, 55 },
    { 0x37fb88e7, 34 },
    { 0x773b88e9, 46 },
    { 0x89eb88ea, 90 },
    { 0, 0 },
    { 0x774b88ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35b88f2, 93 },
    { 0x326b88f3, 85 },
    { 0xba9b88f4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bb88f9, 93 },
    { 0xb24b88fa, 33 },
    { 0xbf7b88fb, 23 },
    { 0, 0 },
    { 0xe74b88fd, 95 },
    { 0, 0 },
    { 0x15bb88ff, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b8905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b8909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3b890d, 52 },
    { 0x402b890e, 33 },
    { 0xb52b890f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cb8914, 34 },
    { 0x9b1b8915, 33 },
    { 0xc7eb8916, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b891b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4b891e, 95 },
    { 0x69cb891f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb8923, 90 },
    { 0x523b8923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b892e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eeb8933, 90 },
    { 0x30b8934, 93 },
    { 0, 0 },
    { 0xe26b8936, 98 },
    { 0, 0 },
    { 0xe69b8938, 23 },
    { 0x242b8939, 61 },
    { 0x7bcb8939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afb893e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcb8943, 67 },
    { 0, 0 },
    { 0xff2b8945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fb894a, 23 },
    { 0xd7eb894b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b894e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955b8952, 61 },
    { 0x26ab8953, 61 },
    { 0xf39b8954, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9b8957, 77 },
    { 0xefb8957, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b895b, 43 },
    { 0, 0 },
    { 0x7f1b895d, 99 },
    { 0xe3eb895e, 98 },
    { 0, 0 },
    { 0xa9fb8960, 95 },
    { 0x42fb8961, 61 },
    { 0x433b8962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0b8966, 78 },
    { 0xc1b8967, 90 },
    { 0xd63b8966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b896b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b896e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645b897e, 91 },
    { 0xa6fb897f, 17 },
    { 0x273b8980, 98 },
    { 0x194b8981, 98 },
    { 0x1c8b8980, 53 },
    { 0xf52b8981, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b8989, 67 },
    { 0x6b6b898a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4eb898d, 100 },
    { 0x773b898e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798b8992, 86 },
    { 0xa34b8993, 99 },
    { 0x260b8994, 76 },
    { 0, 0 },
    { 0xf5cb8996, 33 },
    { 0, 0 },
    { 0xd40b8998, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57b899c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb899f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b89a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b89aa, 33 },
    { 0xcd4b89aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb89ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296b89b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb89b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db89bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b89c8, 33 },
    { 0xb97b89c9, 98 },
    { 0xcd6b89ca, 98 },
    { 0x7bcb89c9, 34 },
    { 0x25b89cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b89cf, 90 },
    { 0, 0 },
    { 0x1a7b89d1, 76 },
    { 0, 0 },
    { 0xb47b89d3, 80 },
    { 0xa00b89d4, 90 },
    { 0xd4cb89d5, 48 },
    { 0x5cfb89d6, 76 },
    { 0xf40b89d6, 33 },
    { 0x2c7b89d8, 2 },
    { 0, 0 },
    { 0xbefb89da, 33 },
    { 0, 0 },
    { 0x2eab89dc, 33 },
    { 0, 0 },
    { 0x61eb89de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdb89e9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11b89ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83db89f0, 33 },
    { 0, 0 },
    { 0x8dcb89f2, 34 },
    { 0x5ab89f3, 90 },
    { 0x171b89f3, 61 },
    { 0, 0 },
    { 0xafcb89f6, 33 },
    { 0, 0 },
    { 0x80bb89f8, 23 },
    { 0x3fb89f9, 67 },
    { 0x59fb89f9, 19 },
    { 0, 0 },
    { 0xc2eb89fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69b8a00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5b8a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365b8a0a, 95 },
    { 0x945b8a0b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b8a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeab8a11, 61 },
    { 0xfccb8a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b8a17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b8a1a, 98 },
    { 0x9a1b8a1b, 46 },
    { 0x62ab8a1c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db8a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdb8a25, 90 },
    { 0x5cdb8a26, 33 },
    { 0xde6b8a26, 90 },
    { 0, 0 },
    { 0x3a7b8a29, 46 },
    { 0x54b8a2a, 9 },
    { 0x531b8a2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0b8a34, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b8a3b, 100 },
    { 0x4a7b8a3b, 90 },
    { 0x83db8a3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b8a45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03b8a4a, 90 },
    { 0xc66b8a4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8a50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0b8a54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ab8a57, 99 },
    { 0, 0 },
    { 0xb7ab8a59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b8a5f, 95 },
    { 0x122b8a60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cb8a67, 84 },
    { 0x744b8a68, 33 },
    { 0xf02b8a69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbb8a74, 23 },
    { 0x451b8a75, 98 },
    { 0x598b8a75, 98 },
    { 0, 0 },
    { 0x435b8a78, 61 },
    { 0xa6ab8a79, 33 },
    { 0, 0 },
    { 0x1ccb8a7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b8a85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55b8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b8a8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b8a91, 78 },
    { 0xce2b8a91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03b8a96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b8a9b, 93 },
    { 0x682b8a9c, 34 },
    { 0x15fb8a9d, 33 },
    { 0x77ab8a9d, 55 },
    { 0, 0 },
    { 0x11cb8aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b8aab, 90 },
    { 0xaa6b8aac, 17 },
    { 0xd18b8aac, 61 },
    { 0xde8b8aac, 61 },
    { 0, 0 },
    { 0x366b8ab0, 95 },
    { 0xbd5b8ab0, 23 },
    { 0x4acb8ab2, 23 },
    { 0xbd2b8ab2, 94 },
    { 0, 0 },
    { 0x42db8ab5, 76 },
    { 0, 0 },
    { 0xcf4b8ab7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab8aba, 81 },
    { 0x5cab8aba, 98 },
    { 0, 0 },
    { 0xf2bb8abd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab8ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab8ac4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1b8ac8, 95 },
    { 0xc6db8ac9, 95 },
    { 0x567b8aca, 66 },
    { 0x4e9b8acb, 90 },
    { 0xf0ab8acb, 34 },
    { 0x5bbb8acd, 100 },
    { 0x436b8aca, 76 },
    { 0x21bb8acf, 90 },
    { 0xf2ab8acf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b8ad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94b8ad6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90b8ade, 61 },
    { 0, 0 },
    { 0xe06b8ae0, 99 },
    { 0xe3db8ae1, 97 },
    { 0xf82b8ae2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256b8ae6, 66 },
    { 0xb45b8ae6, 87 },
    { 0x8d3b8ae7, 85 },
    { 0x1e9b8ae6, 90 },
    { 0, 0 },
    { 0x19bb8aeb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb8aee, 95 },
    { 0, 0 },
    { 0xbd3b8af0, 98 },
    { 0x861b8af0, 33 },
    { 0x7eeb8af2, 90 },
    { 0x174b8af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8b8af7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcb8afd, 91 },
    { 0xc81b8afe, 97 },
    { 0x93ab8afe, 33 },
    { 0, 0 },
    { 0xa77b8b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7b8b07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cb8b0c, 23 },
    { 0x89fb8b0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb8b10, 23 },
    { 0x342b8b11, 90 },
    { 0xb03b8b12, 52 },
    { 0xceb8b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43b8b17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1b8b1b, 61 },
    { 0x3eab8b1b, 93 },
    { 0x7f4b8b1c, 33 },
    { 0xa89b8b1c, 99 },
    { 0xdceb8b1f, 55 },
    { 0xccfb8b20, 95 },
    { 0, 0 },
    { 0x839b8b22, 33 },
    { 0x4b7b8b23, 98 },
    { 0, 0 },
    { 0xee6b8b25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30b8b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357b8b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db8b36, 95 },
    { 0, 0 },
    { 0xe25b8b38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b8b40, 33 },
    { 0, 0 },
    { 0x1afb8b42, 90 },
    { 0x557b8b43, 61 },
    { 0x23db8b42, 61 },
    { 0x2b4b8b45, 76 },
    { 0x4afb8b45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb8b4a, 46 },
    { 0x305b8b4b, 85 },
    { 0x92b8b4c, 66 },
    { 0, 0 },
    { 0xdccb8b4e, 90 },
    { 0, 0 },
    { 0xb42b8b50, 69 },
    { 0, 0 },
    { 0x74db8b52, 33 },
    { 0, 0 },
    { 0xdd3b8b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b8b57, 95 },
    { 0xf5fb8b58, 95 },
    { 0x2f1b8b58, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6b8b5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b8b62, 33 },
    { 0x295b8b63, 90 },
    { 0x92fb8b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb8b6b, 98 },
    { 0, 0 },
    { 0xf22b8b6d, 91 },
    { 0, 0 },
    { 0x187b8b6f, 52 },
    { 0x974b8b6f, 90 },
    { 0x5b6b8b70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7b8b74, 76 },
    { 0x635b8b74, 90 },
    { 0x744b8b76, 95 },
    { 0x814b8b77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb8b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb8b81, 77 },
    { 0, 0 },
    { 0x5d4b8b83, 98 },
    { 0x461b8b83, 90 },
    { 0, 0 },
    { 0x9dcb8b86, 43 },
    { 0x1e8b8b87, 90 },
    { 0, 0 },
    { 0xdeeb8b89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efb8b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ab8b92, 33 },
    { 0, 0 },
    { 0x5acb8b94, 55 },
    { 0, 0 },
    { 0x4d1b8b96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2b8b99, 90 },
    { 0x23db8b9a, 64 },
    { 0x5a1b8b9b, 34 },
    { 0, 0 },
    { 0x83db8b9d, 78 },
    { 0x69b8b9e, 33 },
    { 0, 0 },
    { 0x183b8ba0, 95 },
    { 0x1a9b8ba1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b8ba4, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b8ba7, 43 },
    { 0xaa4b8ba7, 61 },
    { 0x4a9b8ba9, 66 },
    { 0x226b8ba9, 33 },
    { 0x68cb8ba9, 75 },
    { 0x209b8ba7, 52 },
    { 0x871b8ba8, 33 },
    { 0xeeeb8bae, 61 },
    { 0x6fbb8baf, 31 },
    { 0x793b8ba8, 76 },
    { 0x877b8bb1, 69 },
    { 0xbd4b8bb2, 87 },
    { 0xf58b8bb1, 98 },
    { 0xce7b8bb4, 75 },
    { 0x998b8bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9b8bb8, 34 },
    { 0x3b8b8bb9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5db8bbe, 43 },
    { 0x22b8bbf, 87 },
    { 0xf29b8bc0, 91 },
    { 0, 0 },
    { 0x1ceb8bc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaab8bcb, 33 },
    { 0x725b8bcb, 46 },
    { 0, 0 },
    { 0xfeab8bce, 61 },
    { 0xb94b8bcf, 95 },
    { 0x218b8bd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b8bd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb8bdb, 98 },
    { 0, 0 },
    { 0x2f2b8bdd, 90 },
    { 0x3f6b8bdd, 23 },
    { 0x375b8bdf, 90 },
    { 0x9b8b8bdf, 90 },
    { 0xdfb8be1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608b8be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8be9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdb8bec, 61 },
    { 0x81ab8bed, 61 },
    { 0, 0 },
    { 0x9b0b8bef, 61 },
    { 0, 0 },
    { 0x85bb8bf1, 92 },
    { 0xe93b8bf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ab8bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x358b8bf8, 46 },
    { 0x45fb8bf9, 98 },
    { 0x45fb8bfa, 95 },
    { 0, 0 },
    { 0xee1b8bfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db8c00, 61 },
    { 0xa22b8c00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4b8c06, 61 },
    { 0, 0 },
    { 0xc13b8c08, 23 },
    { 0, 0 },
    { 0xcdb8c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866b8c0e, 90 },
    { 0, 0 },
    { 0x7b4b8c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfb8c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2b8c18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x741b8c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb8c22, 23 },
    { 0x492b8c23, 80 },
    { 0xa98b8c23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444b8c2a, 88 },
    { 0x478b8c2a, 66 },
    { 0, 0 },
    { 0xf80b8c2d, 98 },
    { 0x577b8c2d, 46 },
    { 0x4dcb8c2f, 67 },
    { 0, 0 },
    { 0x7c4b8c31, 33 },
    { 0x42bb8c32, 31 },
    { 0x2cfb8c31, 90 },
    { 0, 0 },
    { 0xe3bb8c35, 85 },
    { 0x338b8c35, 84 },
    { 0x97ab8c35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bb8c3c, 55 },
    { 0x4cdb8c3d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab8c41, 23 },
    { 0, 0 },
    { 0x591b8c43, 55 },
    { 0xb2bb8c44, 23 },
    { 0, 0 },
    { 0xd19b8c46, 91 },
    { 0, 0 },
    { 0x67fb8c48, 90 },
    { 0xfadb8c49, 34 },
    { 0, 0 },
    { 0x655b8c4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0b8c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbb8c53, 68 },
    { 0x731b8c54, 33 },
    { 0x25ab8c55, 85 },
    { 0x207b8c55, 53 },
    { 0xd97b8c55, 43 },
    { 0x922b8c58, 35 },
    { 0x6e6b8c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1b8c5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b8c60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b8c68, 61 },
    { 0x84cb8c68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dab8c6d, 98 },
    { 0xdcab8c6e, 33 },
    { 0x603b8c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efb8c73, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b8c7d, 36 },
    { 0, 0 },
    { 0xa74b8c7f, 95 },
    { 0xa73b8c80, 34 },
    { 0xc79b8c81, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59b8c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fb8c92, 61 },
    { 0, 0 },
    { 0x223b8c94, 90 },
    { 0xd1b8c95, 90 },
    { 0x425b8c95, 87 },
    { 0x36cb8c96, 91 },
    { 0xfc4b8c98, 67 },
    { 0x74fb8c98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb8c9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b8ca2, 99 },
    { 0xb89b8ca3, 99 },
    { 0x577b8ca4, 78 },
    { 0, 0 },
    { 0xb0ab8ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab8cad, 61 },
    { 0x44eb8cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64b8cb2, 90 },
    { 0x388b8cb2, 61 },
    { 0x777b8cb2, 23 },
    { 0x48b8cb5, 88 },
    { 0x66ab8cb6, 55 },
    { 0x968b8cb6, 87 },
    { 0x240b8cb7, 67 },
    { 0x3e2b8cb6, 23 },
    { 0x45b8cb7, 85 },
    { 0xb9bb8cba, 61 },
    { 0xbb4b8cbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9db8cc0, 98 },
    { 0, 0 },
    { 0xc7eb8cc2, 33 },
    { 0x845b8cc2, 33 },
    { 0x906b8cc2, 33 },
    { 0x588b8cc5, 61 },
    { 0x8d9b8cc6, 34 },
    { 0xf95b8cc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702b8ccb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x980b8cce, 61 },
    { 0xaeeb8ccf, 98 },
    { 0xfe5b8cd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8cd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8b8cd9, 75 },
    { 0x579b8cda, 98 },
    { 0, 0 },
    { 0xc9fb8cdc, 61 },
    { 0x962b8cdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb8ce0, 53 },
    { 0x935b8ce1, 76 },
    { 0xef7b8ce2, 77 },
    { 0xbc9b8ce3, 61 },
    { 0x8cbb8ce1, 98 },
    { 0xe58b8ce2, 2 },
    { 0x9dab8ce3, 98 },
    { 0x7f7b8ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624b8cec, 90 },
    { 0x769b8ced, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67b8cf1, 90 },
    { 0xbdcb8cf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb8cf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b8cfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb8d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958b8d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb8d0e, 43 },
    { 0, 0 },
    { 0xbeab8d10, 34 },
    { 0, 0 },
    { 0xfd2b8d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2b8d16, 33 },
    { 0, 0 },
    { 0x207b8d18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b8d1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267b8d21, 81 },
    { 0xe1fb8d22, 33 },
    { 0x2bfb8d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x695b8d26, 55 },
    { 0, 0 },
    { 0x44cb8d28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7b8d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab8d2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8d31, 33 },
    { 0x8f0b8d31, 33 },
    { 0x59eb8d31, 90 },
    { 0x3fb8d34, 55 },
    { 0xf3db8d35, 23 },
    { 0x76eb8d35, 46 },
    { 0x27ab8d37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8d3e, 90 },
    { 0x6a8b8d3f, 91 },
    { 0, 0 },
    { 0x73b8d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cab8d47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b8d4a, 23 },
    { 0x8cfb8d4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2db8d4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9b8d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fb8d59, 76 },
    { 0, 0 },
    { 0x27cb8d5b, 61 },
    { 0xc6ab8d5c, 99 },
    { 0xd09b8d5c, 61 },
    { 0x5e3b8d5e, 46 },
    { 0xf77b8d5e, 93 },
    { 0xd64b8d60, 67 },
    { 0x1c3b8d61, 67 },
    { 0x810b8d5f, 33 },
    { 0x333b8d63, 66 },
    { 0xd6ab8d63, 33 },
    { 0, 0 },
    { 0xe9cb8d66, 90 },
    { 0x93bb8d67, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb8d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eab8d6d, 55 },
    { 0xae8b8d6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b8d72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab8d78, 100 },
    { 0xdafb8d79, 23 },
    { 0xab8b8d79, 61 },
    { 0x6f8b8d7b, 95 },
    { 0x3bdb8d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0b8d81, 75 },
    { 0x256b8d82, 65 },
    { 0xd0db8d83, 88 },
    { 0x6e1b8d84, 90 },
    { 0xf7eb8d85, 55 },
    { 0x9fcb8d85, 34 },
    { 0xf59b8d86, 52 },
    { 0x61db8d86, 90 },
    { 0xf09b8d89, 95 },
    { 0x819b8d81, 100 },
    { 0x9b9b8d8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b8d93, 93 },
    { 0xce7b8d94, 76 },
    { 0x371b8d95, 33 },
    { 0x813b8d96, 98 },
    { 0x225b8d97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1b8d9a, 52 },
    { 0, 0 },
    { 0xfd3b8d9c, 34 },
    { 0, 0 },
    { 0xbc7b8d9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbb8da5, 77 },
    { 0xe5eb8da5, 33 },
    { 0xab6b8da7, 95 },
    { 0xe55b8da6, 95 },
    { 0xe5bb8da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35b8dac, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243b8db1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0b8db5, 55 },
    { 0xcbcb8db5, 67 },
    { 0x168b8db7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db8dbd, 100 },
    { 0x473b8dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b8dc6, 34 },
    { 0, 0 },
    { 0x269b8dc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1db8dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b8dd1, 33 },
    { 0x98cb8dd2, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb8dd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b8dda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eeb8dde, 55 },
    { 0, 0 },
    { 0x6e1b8de0, 34 },
    { 0, 0 },
    { 0x8e6b8de2, 33 },
    { 0x8d3b8de3, 87 },
    { 0x762b8de3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b8ded, 95 },
    { 0x6dbb8dee, 78 },
    { 0, 0 },
    { 0x6b2b8df0, 90 },
    { 0, 0 },
    { 0x856b8df2, 88 },
    { 0, 0 },
    { 0xc43b8df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272b8dfd, 76 },
    { 0xda2b8dfe, 34 },
    { 0xa05b8dff, 33 },
    { 0xd9bb8e00, 51 },
    { 0, 0 },
    { 0x627b8e02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b8e05, 98 },
    { 0x567b8e05, 98 },
    { 0x469b8e07, 33 },
    { 0, 0 },
    { 0x86db8e09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb8e0d, 46 },
    { 0xd12b8e0e, 91 },
    { 0xd8b8e0e, 23 },
    { 0, 0 },
    { 0x190b8e11, 61 },
    { 0xf4ab8e12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397b8e1b, 43 },
    { 0x3d4b8e1c, 61 },
    { 0, 0 },
    { 0xfa6b8e1e, 61 },
    { 0x7ceb8e1f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb8e22, 23 },
    { 0, 0 },
    { 0x36b8e24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b8e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb8e2e, 33 },
    { 0x54cb8e2f, 38 },
    { 0x957b8e30, 95 },
    { 0, 0 },
    { 0xc12b8e32, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2b8e38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b8e3c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x194b8e3f, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x824b8e42, 52 },
    { 0xd67b8e43, 100 },
    { 0, 0 },
    { 0xa5eb8e45, 88 },
    { 0, 0 },
    { 0xc80b8e47, 66 },
    { 0xa1fb8e48, 61 },
    { 0x984b8e47, 33 },
    { 0xfd8b8e47, 95 },
    { 0x3bcb8e4b, 93 },
    { 0xd1bb8e4b, 98 },
    { 0xf08b8e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422b8e52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b8e55, 77 },
    { 0, 0 },
    { 0x8acb8e57, 23 },
    { 0xfa0b8e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb8e5b, 95 },
    { 0x2afb8e5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42b8e5f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbb8e63, 23 },
    { 0, 0 },
    { 0x7b2b8e65, 66 },
    { 0, 0 },
    { 0xdb1b8e67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b8e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b8e77, 33 },
    { 0, 0 },
    { 0x8f8b8e79, 56 },
    { 0x11eb8e79, 61 },
    { 0, 0 },
    { 0xf27b8e7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6b8e7f, 90 },
    { 0x25b8e80, 98 },
    { 0x9d0b8e81, 33 },
    { 0, 0 },
    { 0x3e3b8e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eb8e86, 76 },
    { 0xed8b8e87, 34 },
    { 0x973b8e88, 78 },
    { 0x6e5b8e89, 34 },
    { 0, 0 },
    { 0xb27b8e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b8e93, 55 },
    { 0x6ccb8e94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6b8e97, 43 },
    { 0x15fb8e98, 61 },
    { 0x331b8e97, 98 },
    { 0xa3bb8e97, 33 },
    { 0xb3fb8e98, 51 },
    { 0x7a4b8e9c, 33 },
    { 0xec8b8e9b, 90 },
    { 0x2a1b8e9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b8ea1, 98 },
    { 0, 0 },
    { 0x360b8ea3, 52 },
    { 0, 0 },
    { 0x630b8ea5, 61 },
    { 0, 0 },
    { 0xdc8b8ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b8eae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b8eb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db8eb4, 55 },
    { 0xca5b8eb5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x870b8eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5b8ebd, 95 },
    { 0, 0 },
    { 0x416b8ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b8ec2, 43 },
    { 0, 0 },
    { 0x13cb8ec4, 33 },
    { 0xde5b8ec5, 93 },
    { 0x91bb8ec4, 61 },
    { 0x67ab8ec7, 33 },
    { 0x1b6b8ec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662b8ecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b8ecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb8ed2, 34 },
    { 0x151b8ed3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b8edc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5b8ee0, 95 },
    { 0, 0 },
    { 0x750b8ee2, 61 },
    { 0, 0 },
    { 0xa81b8ee4, 61 },
    { 0, 0 },
    { 0x14fb8ee6, 61 },
    { 0x6acb8ee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b8eeb, 84 },
    { 0xda3b8eec, 33 },
    { 0x4d5b8eed, 90 },
    { 0, 0 },
    { 0xcf0b8eef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b8ef6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b8efa, 95 },
    { 0xa51b8efb, 84 },
    { 0xc0cb8efb, 98 },
    { 0x74eb8efa, 33 },
    { 0, 0 },
    { 0xa5cb8eff, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db8f02, 84 },
    { 0, 0 },
    { 0x357b8f04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b8f07, 23 },
    { 0xbbfb8f08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb8f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b8f13, 90 },
    { 0x1fab8f13, 75 },
    { 0xfe4b8f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ab8f1b, 91 },
    { 0x23ab8f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559b8f20, 81 },
    { 0, 0 },
    { 0x2a2b8f22, 61 },
    { 0x66ab8f23, 90 },
    { 0x958b8f24, 34 },
    { 0xf59b8f25, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fb8f29, 85 },
    { 0x6ebb8f29, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0b8f2e, 43 },
    { 0x6d7b8f2f, 98 },
    { 0xa22b8f2f, 90 },
    { 0x1f0b8f30, 98 },
    { 0x345b8f2e, 46 },
    { 0x441b8f33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9eb8f37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b8f3a, 55 },
    { 0xa47b8f3b, 66 },
    { 0x2b8f3c, 61 },
    { 0x66fb8f3b, 99 },
    { 0, 0 },
    { 0xfe0b8f3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b8f47, 88 },
    { 0xa15b8f47, 55 },
    { 0, 0 },
    { 0x4e8b8f4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab8f53, 90 },
    { 0x484b8f54, 78 },
    { 0xdccb8f55, 23 },
    { 0, 0 },
    { 0xc83b8f57, 78 },
    { 0xa04b8f57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb8f5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6b8f61, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b8f67, 55 },
    { 0, 0 },
    { 0x702b8f69, 90 },
    { 0x52b8f6a, 61 },
    { 0, 0 },
    { 0x87b8f6c, 34 },
    { 0, 0 },
    { 0x9fab8f6e, 33 },
    { 0, 0 },
    { 0x83db8f70, 23 },
    { 0, 0 },
    { 0xdfeb8f72, 87 },
    { 0x3d3b8f73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbb8f76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb8f7d, 61 },
    { 0x8e8b8f7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb8f85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb8f88, 61 },
    { 0x8d8b8f89, 85 },
    { 0xbfbb8f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebb8f8f, 98 },
    { 0, 0 },
    { 0x775b8f91, 98 },
    { 0x620b8f92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcb8f98, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b8f9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfb8fa5, 93 },
    { 0xe13b8fa5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fb8fa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fae, 46 },
    { 0x69fb8faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5b8fb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb8fbc, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eb8fc2, 98 },
    { 0, 0 },
    { 0x96cb8fc4, 61 },
    { 0, 0 },
    { 0x237b8fc6, 93 },
    { 0x12fb8fc7, 61 },
    { 0xd31b8fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b8fcc, 90 },
    { 0xc65b8fcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777b8fd5, 46 },
    { 0x98bb8fd5, 55 },
    { 0, 0 },
    { 0x701b8fd8, 33 },
    { 0, 0 },
    { 0x1dfb8fda, 33 },
    { 0x7f2b8fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567b8fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898b8fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51b8fec, 23 },
    { 0, 0 },
    { 0x369b8fee, 98 },
    { 0, 0 },
    { 0xb2cb8ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b8ff3, 46 },
    { 0x5fcb8ff4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5b8ff7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fb9002, 95 },
    { 0, 0 },
    { 0x214b9004, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740b900b, 65 },
    { 0x9f7b900c, 99 },
    { 0xb08b900b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63b9010, 98 },
    { 0, 0 },
    { 0xc73b9012, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b9017, 33 },
    { 0xdbeb9017, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62b901b, 99 },
    { 0, 0 },
    { 0xdbab901d, 90 },
    { 0x3edb901d, 66 },
    { 0xd68b901f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b9023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cb902a, 93 },
    { 0x37fb902a, 90 },
    { 0x20ab902a, 55 },
    { 0, 0 },
    { 0x7c5b902e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb9038, 76 },
    { 0x11fb9038, 76 },
    { 0x2e1b903a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbb9044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b9047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0eb904b, 98 },
    { 0xd1ab904b, 52 },
    { 0, 0 },
    { 0xeb4b904e, 93 },
    { 0x95cb904f, 33 },
    { 0xf00b904f, 95 },
    { 0x54ab9051, 66 },
    { 0xdf4b9052, 33 },
    { 0xe21b9051, 95 },
    { 0xb38b9054, 90 },
    { 0xdccb9055, 98 },
    { 0x98fb9050, 81 },
    { 0x3fcb9057, 78 },
    { 0x2a0b9058, 83 },
    { 0x719b9057, 85 },
    { 0xa8ab905a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330b9060, 90 },
    { 0xdc4b9061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41b9064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67b9067, 23 },
    { 0x29eb9068, 68 },
    { 0, 0 },
    { 0xb9eb906a, 90 },
    { 0x7a8b906b, 99 },
    { 0x180b906c, 61 },
    { 0xe59b906a, 95 },
    { 0xf74b906e, 76 },
    { 0x72eb906f, 33 },
    { 0x163b9070, 43 },
    { 0, 0 },
    { 0x65eb9072, 98 },
    { 0x283b9073, 95 },
    { 0x961b9074, 22 },
    { 0xceb9075, 33 },
    { 0x5bb9075, 95 },
    { 0xe64b9077, 33 },
    { 0xbfdb9077, 61 },
    { 0x702b9079, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eb907c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0b9084, 33 },
    { 0, 0 },
    { 0x381b9086, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b908d, 84 },
    { 0x166b908e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6b9091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917b9095, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b9098, 22 },
    { 0xed4b9098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7b909c, 23 },
    { 0x401b909d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650b90a2, 61 },
    { 0, 0 },
    { 0x500b90a4, 99 },
    { 0, 0 },
    { 0xd74b90a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfb90a9, 78 },
    { 0xf2bb90a9, 84 },
    { 0x9e0b90ab, 90 },
    { 0, 0 },
    { 0xc0db90ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b90b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1b90ba, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eb90c1, 61 },
    { 0xb2ab90c2, 98 },
    { 0xb7b90c1, 90 },
    { 0xc0cb90c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eb90c9, 90 },
    { 0x279b90ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24db90ce, 84 },
    { 0x6d5b90cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3b90d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8b90d7, 81 },
    { 0x3b5b90d7, 100 },
    { 0xa4bb90d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b90dc, 23 },
    { 0x7bab90dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5b90e0, 23 },
    { 0, 0 },
    { 0x976b90e2, 98 },
    { 0x32eb90e3, 84 },
    { 0x898b90e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3b90e7, 75 },
    { 0x765b90e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4b90f1, 98 },
    { 0x7e7b90f1, 61 },
    { 0, 0 },
    { 0xdb6b90f4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ab90f9, 61 },
    { 0xc5ab90fa, 78 },
    { 0xc47b90f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b9101, 61 },
    { 0xd74b9102, 90 },
    { 0x980b9103, 98 },
    { 0x2a7b9102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfb910b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2b9111, 90 },
    { 0x51bb9111, 98 },
    { 0x1e6b9113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31b911a, 61 },
    { 0xf4b911a, 67 },
    { 0x309b911c, 90 },
    { 0x9ccb911d, 76 },
    { 0x81eb911e, 100 },
    { 0x77ab911f, 95 },
    { 0, 0 },
    { 0x4a7b9121, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2b912d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692b9133, 23 },
    { 0, 0 },
    { 0xf7bb9135, 33 },
    { 0xc55b9135, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eb9139, 95 },
    { 0xbe6b913a, 98 },
    { 0, 0 },
    { 0xd10b913c, 95 },
    { 0, 0 },
    { 0xb10b913e, 34 },
    { 0, 0 },
    { 0x682b9140, 90 },
    { 0x464b9141, 99 },
    { 0, 0 },
    { 0x5bab9143, 98 },
    { 0x7bb9144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b914c, 90 },
    { 0x467b914c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1b9158, 98 },
    { 0, 0 },
    { 0x3fcb915a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76b915d, 33 },
    { 0x4c0b915e, 23 },
    { 0x49cb915f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800b9164, 78 },
    { 0x248b9165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92b9168, 95 },
    { 0x9f2b9168, 34 },
    { 0x4a0b916a, 33 },
    { 0x565b916a, 61 },
    { 0xabab9169, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab916f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8b9172, 33 },
    { 0x79b9173, 33 },
    { 0, 0 },
    { 0x162b9175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb9178, 33 },
    { 0, 0 },
    { 0xa5ab917a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110b917f, 61 },
    { 0, 0 },
    { 0xfafb9181, 34 },
    { 0, 0 },
    { 0xbf1b9183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5b9187, 91 },
    { 0, 0 },
    { 0x2f4b9189, 87 },
    { 0x95cb9189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49b9190, 52 },
    { 0xadab9191, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0b9195, 33 },
    { 0xc97b9196, 90 },
    { 0x671b9197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87b919a, 95 },
    { 0xb34b919b, 95 },
    { 0x293b919c, 76 },
    { 0, 0 },
    { 0xd6bb919e, 43 },
    { 0, 0 },
    { 0xf06b91a0, 38 },
    { 0, 0 },
    { 0x10ab91a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895b91a7, 76 },
    { 0xe14b91a7, 46 },
    { 0xd31b91a9, 91 },
    { 0x7a8b91a9, 43 },
    { 0xbfab91ab, 33 },
    { 0x550b91aa, 55 },
    { 0xcfbb91ac, 95 },
    { 0x27cb91ab, 61 },
    { 0x55db91af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6b91b5, 90 },
    { 0xe4b91b6, 53 },
    { 0xb30b91b7, 55 },
    { 0x778b91b8, 38 },
    { 0x655b91b8, 33 },
    { 0, 0 },
    { 0xb11b91bb, 23 },
    { 0xaf0b91bb, 90 },
    { 0, 0 },
    { 0xae8b91be, 61 },
    { 0, 0 },
    { 0x44fb91c0, 91 },
    { 0x76ab91c0, 61 },
    { 0xc81b91c2, 90 },
    { 0x700b91c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab91c9, 90 },
    { 0xbeb91c9, 84 },
    { 0x2e3b91cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x72eb91ce, 61 },
    { 0x435b91cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb91d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b91d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb91db, 61 },
    { 0x5fbb91dc, 34 },
    { 0x4eab91dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceb91e1, 90 },
    { 0x8b8b91e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x677b91e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b91ea, 98 },
    { 0xe80b91ea, 98 },
    { 0, 0 },
    { 0x6a2b91ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22b91f1, 52 },
    { 0x36bb91f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb91f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b91fa, 87 },
    { 0x1b9b91fb, 95 },
    { 0xf74b91fa, 19 },
    { 0, 0 },
    { 0x2e1b91fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b9201, 93 },
    { 0, 0 },
    { 0xc05b9203, 95 },
    { 0xdfeb9204, 23 },
    { 0x291b9203, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29b920c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cb9214, 76 },
    { 0, 0 },
    { 0x594b9216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb921e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x990b9221, 83 },
    { 0, 0 },
    { 0x140b9223, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x347b9226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3b922f, 91 },
    { 0xaf6b9230, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42b9234, 34 },
    { 0, 0 },
    { 0x241b9236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb923a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b923d, 78 },
    { 0x5d7b923e, 76 },
    { 0x6a5b923f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78db9244, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b924e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b9251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b9257, 87 },
    { 0, 0 },
    { 0x787b9259, 90 },
    { 0xec9b925a, 43 },
    { 0x4f7b925b, 23 },
    { 0x5cbb9259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecb9261, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51b9264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617b9269, 91 },
    { 0, 0 },
    { 0x9e8b926b, 53 },
    { 0, 0 },
    { 0x51ab926d, 55 },
    { 0, 0 },
    { 0x298b926f, 68 },
    { 0x2c0b9270, 88 },
    { 0x1a5b9271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79db9275, 95 },
    { 0x860b9276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86b9279, 93 },
    { 0x67ab927a, 33 },
    { 0xe86b927b, 33 },
    { 0xc4bb927b, 23 },
    { 0xadeb927d, 52 },
    { 0xae8b927d, 98 },
    { 0x6e9b927d, 61 },
    { 0xcf8b9280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb9283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9286, 95 },
    { 0, 0 },
    { 0xb5fb9288, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b928b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b928f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171b9293, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7eb929e, 78 },
    { 0xc5ab929f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b92a2, 34 },
    { 0, 0 },
    { 0x866b92a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db92a7, 46 },
    { 0xa53b92a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b92ab, 23 },
    { 0, 0 },
    { 0x326b92ad, 90 },
    { 0, 0 },
    { 0x29b92af, 98 },
    { 0x8beb92b0, 40 },
    { 0x764b92b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b92b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01b92b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb92bc, 43 },
    { 0, 0 },
    { 0xecb92be, 52 },
    { 0xfe1b92bf, 46 },
    { 0xcafb92c0, 43 },
    { 0xbb8b92c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb92c6, 98 },
    { 0, 0 },
    { 0x85fb92c8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20b92cb, 43 },
    { 0x886b92cc, 61 },
    { 0x892b92cc, 23 },
    { 0x25fb92ce, 91 },
    { 0xa4bb92cf, 61 },
    { 0x124b92cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feb92d4, 61 },
    { 0xddcb92d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b92db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7b92de, 84 },
    { 0, 0 },
    { 0xa89b92e0, 95 },
    { 0xc3ab92e1, 76 },
    { 0xea6b92e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17b92e5, 61 },
    { 0, 0 },
    { 0x579b92e7, 61 },
    { 0, 0 },
    { 0x534b92e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ab92f1, 34 },
    { 0x610b92f2, 34 },
    { 0, 0 },
    { 0x1d1b92f4, 34 },
    { 0xcd6b92f4, 40 },
    { 0x33db92f6, 95 },
    { 0x80eb92f5, 90 },
    { 0x297b92f8, 61 },
    { 0x534b92f7, 98 },
    { 0x321b92fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5b92fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b9301, 90 },
    { 0, 0 },
    { 0x98eb9303, 76 },
    { 0xf93b9304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x447b9307, 33 },
    { 0xce9b9307, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bb930b, 61 },
    { 0xcc7b930c, 76 },
    { 0x13fb930b, 91 },
    { 0xdeab930b, 100 },
    { 0xd0bb930f, 33 },
    { 0, 0 },
    { 0x4dcb9311, 95 },
    { 0, 0 },
    { 0x75cb9313, 90 },
    { 0, 0 },
    { 0x7bab9315, 88 },
    { 0xa26b9316, 33 },
    { 0x506b9317, 43 },
    { 0xd08b9318, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46b931b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9322, 61 },
    { 0, 0 },
    { 0xb15b9324, 91 },
    { 0x53db9324, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b932c, 17 },
    { 0x98bb932d, 78 },
    { 0, 0 },
    { 0x609b932f, 23 },
    { 0x1edb9330, 100 },
    { 0xca9b9330, 100 },
    { 0x73b9332, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79b9337, 96 },
    { 0, 0 },
    { 0xa80b9339, 81 },
    { 0x6f4b9339, 23 },
    { 0xeddb933b, 23 },
    { 0xa3cb933a, 34 },
    { 0xb8b933b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab934a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb9354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb935c, 95 },
    { 0xb1eb935d, 93 },
    { 0, 0 },
    { 0x1e6b935f, 90 },
    { 0xd67b9360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b9366, 87 },
    { 0xd39b9367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aab936d, 91 },
    { 0, 0 },
    { 0x22cb936f, 98 },
    { 0x30fb936f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b9373, 95 },
    { 0xc98b9374, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eeb9378, 43 },
    { 0xb7db9379, 34 },
    { 0x30ab9379, 61 },
    { 0xe04b937b, 77 },
    { 0xe5fb937b, 98 },
    { 0x6bdb937a, 81 },
    { 0, 0 },
    { 0x4bfb937f, 23 },
    { 0x49ab9380, 33 },
    { 0x485b9381, 23 },
    { 0x715b9381, 61 },
    { 0x107b9380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88b9387, 95 },
    { 0, 0 },
    { 0x981b9389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb938d, 23 },
    { 0xecdb938e, 95 },
    { 0x9fdb938f, 61 },
    { 0x160b9390, 61 },
    { 0x17fb938f, 61 },
    { 0x5bb9392, 61 },
    { 0x837b9390, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b9398, 98 },
    { 0xc70b9399, 87 },
    { 0xcd7b9399, 83 },
    { 0x31b9399, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b93a1, 76 },
    { 0xa2b93a2, 90 },
    { 0xb30b93a2, 33 },
    { 0, 0 },
    { 0xc4fb93a5, 90 },
    { 0, 0 },
    { 0xe65b93a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab93ab, 76 },
    { 0xa53b93ac, 76 },
    { 0, 0 },
    { 0xdd4b93ae, 95 },
    { 0x2acb93af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1b93b4, 23 },
    { 0, 0 },
    { 0x86b93b6, 34 },
    { 0xca1b93b7, 98 },
    { 0x913b93b7, 23 },
    { 0xc54b93b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebb93bd, 76 },
    { 0, 0 },
    { 0x294b93bf, 90 },
    { 0x9dbb93c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b93c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30b93c8, 84 },
    { 0, 0 },
    { 0x5eb93ca, 76 },
    { 0x3f4b93ca, 99 },
    { 0x2e9b93ca, 93 },
    { 0x5a3b93cd, 98 },
    { 0x189b93ca, 33 },
    { 0x78b93cc, 33 },
    { 0, 0 },
    { 0x59bb93d1, 95 },
    { 0x9eb93d2, 95 },
    { 0x4b4b93d1, 38 },
    { 0xff1b93d4, 76 },
    { 0, 0 },
    { 0x9a3b93d6, 33 },
    { 0, 0 },
    { 0xd8cb93d8, 33 },
    { 0, 0 },
    { 0x22cb93da, 33 },
    { 0, 0 },
    { 0x423b93dc, 61 },
    { 0xc20b93dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbb93e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab93e8, 17 },
    { 0xeebb93e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740b93ee, 98 },
    { 0x961b93ef, 46 },
    { 0x752b93ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b93f4, 34 },
    { 0, 0 },
    { 0x4ab93f6, 95 },
    { 0x959b93f7, 36 },
    { 0xc35b93f8, 90 },
    { 0xafab93f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722b93ff, 33 },
    { 0xbc5b9400, 23 },
    { 0, 0 },
    { 0xe1ab9402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbb9411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab9414, 61 },
    { 0x4a2b9414, 77 },
    { 0xc79b9416, 17 },
    { 0xce1b9414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bab9421, 98 },
    { 0, 0 },
    { 0xe66b9423, 81 },
    { 0xa52b9423, 61 },
    { 0x222b9424, 33 },
    { 0, 0 },
    { 0xbc8b9427, 34 },
    { 0, 0 },
    { 0x73eb9429, 84 },
    { 0x1d8b942a, 33 },
    { 0xacab942a, 34 },
    { 0xdc1b9429, 98 },
    { 0xf9b942d, 98 },
    { 0x844b9429, 100 },
    { 0x7e9b942f, 61 },
    { 0xcdb9429, 98 },
    { 0x5cab9431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b9434, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559b943b, 76 },
    { 0x6acb943b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb943f, 78 },
    { 0, 0 },
    { 0x607b9441, 52 },
    { 0xd5fb9442, 96 },
    { 0x2dab9443, 88 },
    { 0x4c9b9441, 77 },
    { 0xd11b9445, 43 },
    { 0, 0 },
    { 0x5b4b9447, 33 },
    { 0xb8bb9448, 98 },
    { 0x94b9449, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b944c, 98 },
    { 0, 0 },
    { 0x36cb944e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebb9453, 33 },
    { 0, 0 },
    { 0x6d0b9455, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4b9459, 23 },
    { 0xc9fb945a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0b945e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88b9465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b9469, 66 },
    { 0, 0 },
    { 0x72ab946b, 95 },
    { 0x5e3b946c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab946f, 68 },
    { 0xce4b9470, 30 },
    { 0, 0 },
    { 0x970b9472, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bb9478, 91 },
    { 0xb4eb9479, 100 },
    { 0x3d9b947a, 100 },
    { 0xde0b947b, 90 },
    { 0, 0 },
    { 0x258b947d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551b9481, 23 },
    { 0x43b9481, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bb9485, 95 },
    { 0x660b9486, 61 },
    { 0, 0 },
    { 0x7d5b9488, 98 },
    { 0x103b9489, 90 },
    { 0x1e6b948a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0b9496, 90 },
    { 0x902b9496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b949a, 66 },
    { 0xb29b949b, 43 },
    { 0x551b949b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb94a2, 93 },
    { 0x52ab94a3, 98 },
    { 0xca5b94a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ab94a8, 66 },
    { 0x369b94a9, 34 },
    { 0x468b94aa, 27 },
    { 0, 0 },
    { 0xd23b94ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b94b0, 34 },
    { 0, 0 },
    { 0xc30b94b2, 61 },
    { 0x750b94b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eb94b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb94b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0b94bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49b94c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbb94c4, 23 },
    { 0, 0 },
    { 0x6ecb94c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6b94c9, 95 },
    { 0, 0 },
    { 0xd71b94cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b94d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214b94d4, 84 },
    { 0x596b94d4, 43 },
    { 0, 0 },
    { 0x636b94d7, 33 },
    { 0xf4bb94d8, 61 },
    { 0x2a2b94d9, 90 },
    { 0x387b94da, 81 },
    { 0x3d1b94da, 76 },
    { 0x380b94d7, 55 },
    { 0, 0 },
    { 0x82fb94de, 43 },
    { 0, 0 },
    { 0x7dab94e0, 23 },
    { 0x314b94e0, 98 },
    { 0x356b94e2, 61 },
    { 0x638b94e2, 61 },
    { 0x84ab94e1, 95 },
    { 0x2e3b94e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5b94e8, 76 },
    { 0, 0 },
    { 0x9efb94ea, 61 },
    { 0, 0 },
    { 0x1d1b94ec, 90 },
    { 0x680b94ed, 85 },
    { 0x8e8b94ed, 90 },
    { 0, 0 },
    { 0x99db94f0, 85 },
    { 0xc4db94f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb94f4, 88 },
    { 0x9c6b94f4, 84 },
    { 0x45cb94f4, 76 },
    { 0, 0 },
    { 0x601b94f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x741b94fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5b94fe, 76 },
    { 0x2fbb94fe, 65 },
    { 0, 0 },
    { 0xfdcb9501, 95 },
    { 0, 0 },
    { 0x322b9503, 23 },
    { 0, 0 },
    { 0x572b9505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b9508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b9512, 93 },
    { 0x5fab9513, 93 },
    { 0xddeb9514, 98 },
    { 0x5afb9515, 93 },
    { 0xac6b9512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbb951c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edb9520, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5db9525, 61 },
    { 0x8dab9525, 55 },
    { 0x5c4b9527, 93 },
    { 0, 0 },
    { 0x416b9529, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15b9535, 68 },
    { 0x59bb9536, 56 },
    { 0x3a9b9537, 90 },
    { 0, 0 },
    { 0xcc7b9539, 90 },
    { 0xab0b953a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ab9540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b954b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29db954f, 33 },
    { 0x29cb954f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753b9559, 99 },
    { 0x89db955a, 90 },
    { 0x335b9559, 17 },
    { 0, 0 },
    { 0xd22b955d, 81 },
    { 0, 0 },
    { 0x911b955f, 61 },
    { 0, 0 },
    { 0xd82b9561, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b9564, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db9567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3b956b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b9573, 98 },
    { 0xed6b9574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b9578, 88 },
    { 0x506b9579, 95 },
    { 0x75cb9579, 46 },
    { 0xf89b957a, 61 },
    { 0x5bfb957a, 90 },
    { 0x8e4b9578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b9587, 76 },
    { 0x9ceb9587, 90 },
    { 0, 0 },
    { 0xda8b958a, 95 },
    { 0, 0 },
    { 0x77bb958c, 95 },
    { 0x5a4b958d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab9590, 52 },
    { 0, 0 },
    { 0x168b9592, 90 },
    { 0x331b9592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b959a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb959e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb95a1, 33 },
    { 0, 0 },
    { 0x85cb95a3, 90 },
    { 0, 0 },
    { 0x93ab95a5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7b95aa, 36 },
    { 0xd65b95aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb95ae, 90 },
    { 0, 0 },
    { 0xc5cb95b0, 66 },
    { 0, 0 },
    { 0x96fb95b2, 23 },
    { 0x494b95b3, 61 },
    { 0x9ceb95b4, 41 },
    { 0x707b95b3, 76 },
    { 0xf09b95b6, 84 },
    { 0, 0 },
    { 0xf77b95b8, 84 },
    { 0x253b95b9, 61 },
    { 0x3dfb95ba, 33 },
    { 0x1b6b95bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcb95c5, 46 },
    { 0x606b95c6, 91 },
    { 0xe22b95c7, 23 },
    { 0xa50b95c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5b95cc, 95 },
    { 0x790b95cc, 90 },
    { 0, 0 },
    { 0xcebb95cf, 76 },
    { 0xfd6b95d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b95dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb95e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bb95e3, 33 },
    { 0, 0 },
    { 0x37b95e5, 55 },
    { 0, 0 },
    { 0x34b95e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb95ea, 81 },
    { 0x20fb95ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x501b95ee, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0b95f4, 90 },
    { 0xa3db95f5, 52 },
    { 0x965b95f4, 99 },
    { 0x8a0b95f7, 55 },
    { 0x141b95f8, 61 },
    { 0xf1b95f4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fb95fc, 55 },
    { 0xa67b95fc, 95 },
    { 0x92db95fc, 61 },
    { 0x9d6b95fe, 23 },
    { 0, 0 },
    { 0x5c1b9601, 76 },
    { 0x4b9602, 61 },
    { 0, 0 },
    { 0x154b9604, 90 },
    { 0, 0 },
    { 0xe42b9606, 55 },
    { 0xdf7b9607, 33 },
    { 0x542b9608, 76 },
    { 0, 0 },
    { 0xe6ab960a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bb960d, 88 },
    { 0xc68b960e, 33 },
    { 0, 0 },
    { 0xd3fb9610, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b9615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75b9618, 55 },
    { 0, 0 },
    { 0xc08b961a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6b9622, 90 },
    { 0x805b9623, 97 },
    { 0, 0 },
    { 0x697b9625, 84 },
    { 0x2f4b9626, 61 },
    { 0x4ebb9626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfb962c, 66 },
    { 0xaf7b962c, 61 },
    { 0x3b2b962e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fb9634, 91 },
    { 0x98b9635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b9638, 98 },
    { 0xdf8b9638, 77 },
    { 0x131b963a, 23 },
    { 0, 0 },
    { 0x960b963c, 90 },
    { 0x5a4b963d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4b9644, 34 },
    { 0xf80b9645, 61 },
    { 0xd3eb9646, 84 },
    { 0x556b9647, 90 },
    { 0x747b9648, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b964b, 75 },
    { 0, 0 },
    { 0x602b964d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356b9654, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebb965e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b9662, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b9665, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b9668, 23 },
    { 0x44ab9669, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b966d, 90 },
    { 0x3bdb966e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708b9672, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8b9678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb967c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab9685, 98 },
    { 0x18db9686, 90 },
    { 0, 0 },
    { 0x1a4b9688, 55 },
    { 0, 0 },
    { 0x577b968a, 23 },
    { 0xac4b968b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9b9692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab96a4, 43 },
    { 0x574b96a4, 61 },
    { 0, 0 },
    { 0xa0ab96a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb96ab, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb96b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ab96b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb96b6, 98 },
    { 0x4e5b96b7, 78 },
    { 0, 0 },
    { 0xa48b96b9, 46 },
    { 0xf7b96b9, 34 },
    { 0, 0 },
    { 0x9eb96bc, 98 },
    { 0x846b96bd, 98 },
    { 0xc68b96be, 90 },
    { 0xab96bf, 23 },
    { 0x234b96c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb96c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5b96c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b96cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b96d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b96e2, 43 },
    { 0x9f2b96e3, 98 },
    { 0x582b96e4, 96 },
    { 0x8d3b96e2, 61 },
    { 0, 0 },
    { 0xdbab96e7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6b96eb, 52 },
    { 0xbd6b96eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcb96f0, 91 },
    { 0x418b96f0, 33 },
    { 0x276b96f2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931b96f6, 55 },
    { 0x1dbb96f7, 76 },
    { 0xc7db96f8, 91 },
    { 0xc95b96f8, 61 },
    { 0xce2b96fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b96fe, 46 },
    { 0x893b96ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eb9702, 98 },
    { 0x5eb9702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb9707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195b970c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380b9715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfb971a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b9721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229b9727, 91 },
    { 0x4e5b9728, 61 },
    { 0xcd0b9727, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cb9732, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eb973b, 23 },
    { 0x747b973b, 95 },
    { 0xe2fb973b, 61 },
    { 0x4b7b973e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448b9749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb9752, 61 },
    { 0xa1bb9752, 75 },
    { 0, 0 },
    { 0x616b9755, 78 },
    { 0xf61b9756, 77 },
    { 0, 0 },
    { 0x11bb9758, 84 },
    { 0x306b9759, 33 },
    { 0xc0eb975a, 46 },
    { 0, 0 },
    { 0xff5b975c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9760, 98 },
    { 0, 0 },
    { 0x2cb9762, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb976b, 77 },
    { 0, 0 },
    { 0x607b976d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cb9774, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb977b, 43 },
    { 0xf14b977b, 61 },
    { 0x78db977c, 34 },
    { 0xe9db977e, 55 },
    { 0xd9bb977f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb9785, 77 },
    { 0x1b6b9786, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b978a, 46 },
    { 0x4c3b978a, 77 },
    { 0x5adb978b, 34 },
    { 0xa92b978d, 33 },
    { 0x278b978d, 52 },
    { 0xc75b978e, 34 },
    { 0, 0 },
    { 0x46eb9791, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b9794, 23 },
    { 0x6eab9795, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61b9799, 46 },
    { 0x1bfb9799, 33 },
    { 0xf79b979a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4b979f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb97a7, 51 },
    { 0, 0 },
    { 0x265b97a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb97ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b97b4, 61 },
    { 0x8a2b97b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cab97b9, 90 },
    { 0x95bb97ba, 91 },
    { 0x65b97bb, 65 },
    { 0xc55b97bc, 61 },
    { 0, 0 },
    { 0xe31b97be, 40 },
    { 0, 0 },
    { 0x85eb97c0, 33 },
    { 0xac8b97c0, 90 },
    { 0xb03b97c0, 33 },
    { 0x903b97c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb97c7, 90 },
    { 0, 0 },
    { 0xbc0b97c9, 43 },
    { 0x22eb97ca, 61 },
    { 0x1beb97cb, 33 },
    { 0xbe6b97c9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x752b97cf, 33 },
    { 0xb40b97d0, 61 },
    { 0x67db97d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b97d4, 78 },
    { 0x235b97d5, 33 },
    { 0x31bb97d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abb97dc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207b97e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb97e5, 95 },
    { 0xfbbb97e6, 61 },
    { 0x41eb97e6, 61 },
    { 0xd2cb97e5, 19 },
    { 0x426b97e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb97ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b97f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b97f4, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b97ff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bb9803, 90 },
    { 0x50cb9803, 90 },
    { 0x413b9803, 87 },
    { 0x7fb9806, 95 },
    { 0, 0 },
    { 0xa4cb9808, 93 },
    { 0, 0 },
    { 0x9dcb980a, 61 },
    { 0xb8ab980b, 76 },
    { 0x9d5b980b, 65 },
    { 0xe69b980d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b9818, 100 },
    { 0xee1b9818, 23 },
    { 0xce2b981a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddb981e, 33 },
    { 0xf48b981e, 67 },
    { 0x4ddb9820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb9824, 95 },
    { 0x185b9825, 23 },
    { 0xa84b9826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55b9829, 23 },
    { 0, 0 },
    { 0x463b982b, 23 },
    { 0, 0 },
    { 0x9ab982d, 61 },
    { 0x53bb982e, 55 },
    { 0x2dbb982e, 76 },
    { 0xd38b9830, 61 },
    { 0x67bb9830, 33 },
    { 0x7ab982f, 33 },
    { 0x81cb982f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2b9836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981b983e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b9842, 91 },
    { 0x8abb9843, 95 },
    { 0xb8fb9844, 40 },
    { 0xd7cb9845, 67 },
    { 0xb90b9846, 76 },
    { 0xcbeb9846, 34 },
    { 0x584b9846, 56 },
    { 0, 0 },
    { 0x499b984a, 53 },
    { 0, 0 },
    { 0x978b984c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6b9851, 95 },
    { 0xef7b9852, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b9858, 77 },
    { 0x313b9859, 87 },
    { 0x204b9859, 90 },
    { 0x470b985b, 17 },
    { 0xc6ab985a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b985f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9b9863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b9866, 61 },
    { 0xf01b9867, 76 },
    { 0, 0 },
    { 0xaa7b9869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adb986e, 43 },
    { 0xda3b986f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb9872, 61 },
    { 0, 0 },
    { 0x80ab9874, 98 },
    { 0x8a2b9875, 23 },
    { 0, 0 },
    { 0x6c1b9877, 46 },
    { 0x951b9878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb987b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8b9883, 23 },
    { 0x6aeb9883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db9887, 93 },
    { 0x850b9887, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21b988b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb988e, 91 },
    { 0x431b988e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb9894, 98 },
    { 0xd72b9895, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b989a, 61 },
    { 0xe5fb989b, 61 },
    { 0xd47b989c, 90 },
    { 0x16fb989c, 61 },
    { 0x408b989c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378b98a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0b98a6, 90 },
    { 0, 0 },
    { 0xa96b98a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5db98ab, 87 },
    { 0x977b98ab, 33 },
    { 0x7fcb98ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35b98b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901b98b5, 76 },
    { 0xf9db98b6, 77 },
    { 0x47eb98b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ab98bc, 84 },
    { 0xaf8b98bd, 43 },
    { 0x4dcb98be, 77 },
    { 0xa6b98bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573b98c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21b98c9, 95 },
    { 0x4e0b98c9, 90 },
    { 0x2cb98cb, 61 },
    { 0, 0 },
    { 0xc9b98cd, 98 },
    { 0x5dab98cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ab98d5, 23 },
    { 0x4deb98d5, 61 },
    { 0xb48b98d7, 95 },
    { 0x2a8b98d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45b98df, 90 },
    { 0xa8bb98e0, 46 },
    { 0xf3bb98e1, 98 },
    { 0x614b98df, 33 },
    { 0x959b98e3, 77 },
    { 0x2db98e4, 33 },
    { 0x460b98e5, 61 },
    { 0xbcab98e6, 93 },
    { 0x1d5b98e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabb98ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182b98f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab98f8, 90 },
    { 0, 0 },
    { 0x837b98fa, 98 },
    { 0x835b98fa, 95 },
    { 0xd1ab98fc, 95 },
    { 0, 0 },
    { 0xc92b98fe, 91 },
    { 0, 0 },
    { 0x270b9900, 55 },
    { 0xeceb9901, 90 },
    { 0, 0 },
    { 0xfd3b9903, 65 },
    { 0xe92b9904, 34 },
    { 0x4a2b9905, 90 },
    { 0x963b9906, 23 },
    { 0x8e0b9907, 22 },
    { 0x526b9905, 55 },
    { 0xaf6b9909, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb990f, 90 },
    { 0xe5fb990f, 95 },
    { 0xa98b9911, 61 },
    { 0x3f8b9910, 33 },
    { 0x67bb9913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8b9918, 23 },
    { 0, 0 },
    { 0x958b991a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x979b991d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58b9920, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b9924, 55 },
    { 0x1d1b9924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b992a, 33 },
    { 0xc09b992a, 98 },
    { 0x701b992c, 61 },
    { 0x561b992c, 33 },
    { 0xc26b992a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x696b9931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b993b, 90 },
    { 0xf50b993c, 95 },
    { 0xd17b993c, 81 },
    { 0x20bb993c, 93 },
    { 0x4a8b993e, 98 },
    { 0x5dab993c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb9949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb994d, 91 },
    { 0, 0 },
    { 0x2ffb994f, 98 },
    { 0, 0 },
    { 0x415b9951, 43 },
    { 0x2a2b9952, 33 },
    { 0x819b9951, 99 },
    { 0x5bdb9954, 95 },
    { 0xc8cb9955, 33 },
    { 0x21b9956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ab9966, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb9969, 75 },
    { 0x29cb996a, 99 },
    { 0, 0 },
    { 0x5d4b996c, 34 },
    { 0x76cb996d, 100 },
    { 0, 0 },
    { 0x350b996f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b9974, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0b9977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x939b997a, 80 },
    { 0, 0 },
    { 0x554b997c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80b997f, 33 },
    { 0, 0 },
    { 0xb08b9981, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eab9984, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9b998b, 23 },
    { 0x542b998b, 95 },
    { 0x13db998d, 52 },
    { 0x911b998e, 90 },
    { 0, 0 },
    { 0x816b9990, 61 },
    { 0, 0 },
    { 0x94fb9992, 90 },
    { 0, 0 },
    { 0xff6b9994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471b9999, 43 },
    { 0, 0 },
    { 0xe71b999b, 100 },
    { 0xf6eb999c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57b99a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b99a8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3b99ac, 37 },
    { 0x574b99ad, 90 },
    { 0, 0 },
    { 0xa76b99af, 33 },
    { 0, 0 },
    { 0xb51b99b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fb99b6, 23 },
    { 0xb89b99b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887b99c6, 61 },
    { 0, 0 },
    { 0x393b99c8, 90 },
    { 0xf9ab99c8, 95 },
    { 0x6c6b99c9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b99cd, 90 },
    { 0xd60b99ce, 61 },
    { 0x848b99cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fb99d3, 43 },
    { 0x88cb99d4, 77 },
    { 0x3acb99d3, 100 },
    { 0x11eb99d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb99db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b99df, 95 },
    { 0x3b5b99e0, 80 },
    { 0xdf0b99df, 46 },
    { 0x4afb99e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2b99e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b99eb, 98 },
    { 0xf7ab99ec, 66 },
    { 0xc45b99ed, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afb99f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b99f9, 66 },
    { 0, 0 },
    { 0xa57b99fb, 91 },
    { 0x6c6b99fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b9a04, 76 },
    { 0, 0 },
    { 0x949b9a06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b9a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37b9a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbb9a1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b9a22, 76 },
    { 0x53b9a23, 95 },
    { 0xe4db9a24, 64 },
    { 0, 0 },
    { 0x247b9a26, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb9a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b9a31, 66 },
    { 0xe96b9a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bb9a39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2b9a3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00b9a44, 33 },
    { 0, 0 },
    { 0xb7fb9a46, 33 },
    { 0, 0 },
    { 0x10db9a48, 61 },
    { 0, 0 },
    { 0x7d4b9a4a, 98 },
    { 0x26fb9a4b, 66 },
    { 0x5cdb9a4b, 61 },
    { 0, 0 },
    { 0x2beb9a4e, 96 },
    { 0x349b9a4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851b9a53, 23 },
    { 0xd0eb9a54, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db9a59, 100 },
    { 0xbeb9a5a, 33 },
    { 0, 0 },
    { 0x4a6b9a5c, 76 },
    { 0x34db9a5d, 36 },
    { 0x5cb9a5e, 61 },
    { 0x57db9a5f, 95 },
    { 0x60bb9a60, 46 },
    { 0xc05b9a61, 95 },
    { 0, 0 },
    { 0x16b9a63, 98 },
    { 0xf18b9a64, 90 },
    { 0x3e3b9a65, 98 },
    { 0, 0 },
    { 0x388b9a67, 86 },
    { 0xeb9a68, 88 },
    { 0xef2b9a69, 95 },
    { 0x34fb9a6a, 87 },
    { 0x9bdb9a6a, 90 },
    { 0xac6b9a6b, 33 },
    { 0x6d7b9a6d, 61 },
    { 0xdb1b9a6b, 96 },
    { 0xd46b9a6f, 61 },
    { 0xaddb9a70, 36 },
    { 0x2ab9a71, 90 },
    { 0, 0 },
    { 0x5d9b9a73, 75 },
    { 0, 0 },
    { 0x77cb9a75, 34 },
    { 0xc3b9a75, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7b9a79, 33 },
    { 0x62bb9a7a, 100 },
    { 0x9b1b9a7b, 98 },
    { 0x5a8b9a7c, 100 },
    { 0x3fbb9a7c, 87 },
    { 0x2e0b9a7a, 78 },
    { 0xa8eb9a7c, 90 },
    { 0x816b9a80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95b9a8f, 77 },
    { 0xa71b9a8f, 90 },
    { 0x6bbb9a91, 66 },
    { 0xaa1b9a92, 86 },
    { 0x5f3b9a92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2b9a9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb9aa0, 76 },
    { 0xeebb9aa1, 95 },
    { 0x3e8b9aa2, 34 },
    { 0x93db9aa1, 95 },
    { 0xa36b9aa3, 55 },
    { 0xf03b9aa4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afb9aa9, 33 },
    { 0xf6bb9aaa, 52 },
    { 0x19b9aab, 43 },
    { 0x79eb9aac, 33 },
    { 0x692b9aac, 61 },
    { 0, 0 },
    { 0xccbb9aaf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bb9ab2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eab9ab5, 61 },
    { 0, 0 },
    { 0xf55b9ab7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6b9aba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3b9abe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab9ac1, 98 },
    { 0, 0 },
    { 0x2ebb9ac3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645b9ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab9acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab9ad6, 33 },
    { 0, 0 },
    { 0xa2cb9ad8, 61 },
    { 0x3a4b9ad9, 61 },
    { 0xad6b9ad9, 33 },
    { 0, 0 },
    { 0x906b9adc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acb9ae0, 91 },
    { 0xffbb9ae1, 98 },
    { 0xf0b9ae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01b9aea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7b9af1, 95 },
    { 0, 0 },
    { 0x5e3b9af3, 61 },
    { 0x7e2b9af4, 40 },
    { 0, 0 },
    { 0xf29b9af6, 55 },
    { 0x8f9b9af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b9afb, 33 },
    { 0x654b9afb, 90 },
    { 0x7bdb9afd, 85 },
    { 0xa6fb9afb, 90 },
    { 0x4a6b9aff, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb9b02, 77 },
    { 0x287b9b02, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9b0b, 56 },
    { 0, 0 },
    { 0xd63b9b0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab9b13, 90 },
    { 0x2db9b14, 85 },
    { 0x161b9b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b9b1c, 90 },
    { 0xbd4b9b1c, 66 },
    { 0x4c7b9b1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9b9b26, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb9b2a, 90 },
    { 0, 0 },
    { 0xbb2b9b2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9b31, 98 },
    { 0x77db9b32, 40 },
    { 0x38fb9b32, 90 },
    { 0x793b9b34, 83 },
    { 0x2ecb9b34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47b9b3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15b9b3f, 67 },
    { 0, 0 },
    { 0x81bb9b41, 61 },
    { 0x444b9b42, 74 },
    { 0x760b9b42, 66 },
    { 0, 0 },
    { 0x490b9b45, 61 },
    { 0x5b5b9b46, 93 },
    { 0x389b9b47, 90 },
    { 0xc86b9b48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7b9b52, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801b9b56, 65 },
    { 0x2feb9b56, 78 },
    { 0, 0 },
    { 0x757b9b59, 76 },
    { 0, 0 },
    { 0xc1eb9b5b, 98 },
    { 0, 0 },
    { 0x4a0b9b5d, 61 },
    { 0x593b9b5e, 55 },
    { 0, 0 },
    { 0x89b9b60, 91 },
    { 0, 0 },
    { 0x948b9b62, 61 },
    { 0, 0 },
    { 0xb1cb9b64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cb9b69, 95 },
    { 0xac3b9b6a, 33 },
    { 0x797b9b6b, 68 },
    { 0xe21b9b69, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bb9b74, 97 },
    { 0, 0 },
    { 0xeb9b76, 48 },
    { 0x3eeb9b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78b9b7f, 84 },
    { 0xf50b9b7f, 90 },
    { 0x806b9b80, 46 },
    { 0xc57b9b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38b9b8a, 48 },
    { 0x929b9b8b, 90 },
    { 0x6fcb9b8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b9b90, 27 },
    { 0, 0 },
    { 0x511b9b92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b9b9c, 52 },
    { 0xb38b9b9d, 90 },
    { 0xaafb9b9e, 47 },
    { 0x698b9b9e, 46 },
    { 0x39bb9b9d, 93 },
    { 0x36fb9ba1, 33 },
    { 0x13b9ba1, 95 },
    { 0x1bcb9b9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221b9ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab9bb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db9bb9, 33 },
    { 0x245b9bba, 90 },
    { 0x142b9bbb, 33 },
    { 0x92ab9bba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab9bc5, 90 },
    { 0xfa7b9bc5, 61 },
    { 0xdeeb9bc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab9bca, 23 },
    { 0x3a4b9bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611b9bd7, 98 },
    { 0x539b9bd8, 63 },
    { 0x218b9bd7, 91 },
    { 0xddab9bd8, 100 },
    { 0, 0 },
    { 0x73b9bdc, 95 },
    { 0, 0 },
    { 0xae6b9bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b9be3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x801b9be6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b9bea, 76 },
    { 0xfd3b9bea, 33 },
    { 0x3feb9bec, 76 },
    { 0x94cb9beb, 46 },
    { 0x62b9bee, 66 },
    { 0, 0 },
    { 0x5d3b9bf0, 53 },
    { 0xa90b9bf1, 55 },
    { 0, 0 },
    { 0x542b9bf3, 84 },
    { 0x3d4b9bf3, 91 },
    { 0x78fb9bf3, 46 },
    { 0x862b9bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b9bf9, 75 },
    { 0x68ab9bfa, 99 },
    { 0x6f3b9bfa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb9bff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b9c02, 33 },
    { 0x7f3b9c02, 90 },
    { 0x44eb9c04, 80 },
    { 0, 0 },
    { 0xacb9c06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b9c0f, 95 },
    { 0x8d1b9c10, 93 },
    { 0x833b9c0f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28b9c15, 93 },
    { 0xc64b9c16, 61 },
    { 0x399b9c17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb9c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db9c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11b9c21, 33 },
    { 0x76b9c21, 61 },
    { 0x19eb9c22, 78 },
    { 0x198b9c22, 34 },
    { 0xadb9c23, 33 },
    { 0x1f0b9c26, 33 },
    { 0x1b0b9c26, 90 },
    { 0xc18b9c28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4b9c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49b9c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb9c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8b9c3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b9c3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3b9c4e, 55 },
    { 0x4f8b9c4f, 85 },
    { 0, 0 },
    { 0xeb3b9c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290b9c5a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b9c5e, 76 },
    { 0xb5cb9c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb9c67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129b9c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb9c71, 23 },
    { 0xf4db9c71, 23 },
    { 0xf72b9c72, 88 },
    { 0, 0 },
    { 0x5b2b9c75, 33 },
    { 0, 0 },
    { 0xbe3b9c77, 90 },
    { 0x9ceb9c78, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355b9c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9c8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb9c94, 34 },
    { 0x76cb9c95, 52 },
    { 0x51fb9c96, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b9c9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0b9ca2, 55 },
    { 0x47ab9ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb9ca6, 66 },
    { 0, 0 },
    { 0x3aab9ca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71b9cab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2b9cae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb9cb1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b9cb4, 76 },
    { 0, 0 },
    { 0x412b9cb6, 46 },
    { 0xc3db9cb6, 95 },
    { 0x601b9cb8, 61 },
    { 0x9fb9cb9, 52 },
    { 0xd37b9cb9, 40 },
    { 0xefcb9cba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6b9cc0, 95 },
    { 0, 0 },
    { 0xb1b9cc2, 91 },
    { 0x5f3b9cc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb9cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b9ccd, 61 },
    { 0xc0db9ccd, 23 },
    { 0, 0 },
    { 0xf2eb9cd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8db9cd3, 61 },
    { 0xe3eb9cd4, 46 },
    { 0, 0 },
    { 0x47bb9cd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cb9cdd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efb9ce0, 61 },
    { 0xd50b9ce1, 90 },
    { 0, 0 },
    { 0x386b9ce3, 61 },
    { 0xe70b9ce4, 88 },
    { 0xd10b9ce4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b9ceb, 33 },
    { 0x2b0b9cec, 98 },
    { 0x709b9ced, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633b9cf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbb9cf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb9cfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66b9cfd, 67 },
    { 0, 0 },
    { 0x685b9cff, 90 },
    { 0x348b9d00, 95 },
    { 0, 0 },
    { 0xb29b9d02, 34 },
    { 0x146b9d02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630b9d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db9d10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb9d15, 23 },
    { 0xf45b9d15, 55 },
    { 0x1b9d17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563b9d1b, 68 },
    { 0xbd3b9d1b, 52 },
    { 0x2d3b9d1d, 98 },
    { 0xb1eb9d1c, 55 },
    { 0x681b9d1f, 90 },
    { 0xdebb9d20, 43 },
    { 0xdf7b9d21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ab9d24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcb9d2a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ab9d2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b9d30, 23 },
    { 0x225b9d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab9d34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51b9d37, 78 },
    { 0, 0 },
    { 0xefdb9d39, 40 },
    { 0, 0 },
    { 0xb2cb9d3b, 65 },
    { 0x877b9d3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b9d40, 55 },
    { 0xa15b9d41, 23 },
    { 0x78eb9d40, 55 },
    { 0x71bb9d41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226b9d48, 61 },
    { 0x238b9d49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24b9d4c, 95 },
    { 0xad2b9d4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5b9d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57b9d5d, 67 },
    { 0xae7b9d5e, 90 },
    { 0xefdb9d5f, 33 },
    { 0x757b9d60, 61 },
    { 0x444b9d5f, 90 },
    { 0, 0 },
    { 0x9d4b9d63, 61 },
    { 0x4ceb9d63, 61 },
    { 0x7db9d65, 75 },
    { 0, 0 },
    { 0xfa4b9d67, 87 },
    { 0, 0 },
    { 0xdc6b9d69, 99 },
    { 0xee9b9d6a, 91 },
    { 0x8d0b9d6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61b9d6f, 76 },
    { 0xbe7b9d70, 43 },
    { 0xdd5b9d71, 90 },
    { 0x571b9d6f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9d78, 61 },
    { 0xf7fb9d79, 98 },
    { 0xd10b9d7a, 95 },
    { 0x5d9b9d78, 90 },
    { 0, 0 },
    { 0x497b9d7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb9d80, 98 },
    { 0x957b9d80, 17 },
    { 0xa37b9d80, 55 },
    { 0xaa8b9d83, 98 },
    { 0x6e3b9d83, 98 },
    { 0x1fdb9d85, 61 },
    { 0x628b9d86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ab9d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb9d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877b9d92, 100 },
    { 0x297b9d93, 99 },
    { 0x532b9d93, 55 },
    { 0x1eab9d93, 98 },
    { 0x57b9d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eb9d9f, 93 },
    { 0xc4eb9da0, 65 },
    { 0xb01b9da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb9da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9b9dab, 95 },
    { 0x20cb9dab, 33 },
    { 0, 0 },
    { 0x8e4b9dae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab9db2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b9db5, 81 },
    { 0xc3bb9db5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb9db9, 23 },
    { 0xe34b9db9, 55 },
    { 0xb02b9dbb, 90 },
    { 0x250b9dbb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6b9dc1, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab9dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b9dcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb9dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b9dd5, 43 },
    { 0xb89b9dd6, 43 },
    { 0x337b9dd7, 87 },
    { 0x9d8b9dd8, 93 },
    { 0x9ecb9dd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815b9dde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ab9dec, 51 },
    { 0x6a0b9ded, 17 },
    { 0xadbb9dee, 61 },
    { 0xa84b9def, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7b9df6, 61 },
    { 0x620b9df6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x647b9dfa, 33 },
    { 0x82cb9dfb, 85 },
    { 0xfbbb9dfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb9e02, 76 },
    { 0x156b9e03, 33 },
    { 0x468b9e04, 23 },
    { 0x182b9e03, 66 },
    { 0x37ab9e06, 85 },
    { 0x916b9e04, 93 },
    { 0x462b9e08, 76 },
    { 0x402b9e09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1b9e0d, 90 },
    { 0x948b9e0d, 77 },
    { 0xe6bb9e0f, 52 },
    { 0x6ddb9e0d, 17 },
    { 0xa1bb9e11, 23 },
    { 0xae9b9e0e, 76 },
    { 0x6e5b9e13, 61 },
    { 0x4b3b9e14, 43 },
    { 0xa84b9e0d, 76 },
    { 0x832b9e16, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6b9e1a, 33 },
    { 0xbbdb9e1a, 66 },
    { 0xdb4b9e1b, 34 },
    { 0x3abb9e1d, 90 },
    { 0xe56b9e1e, 95 },
    { 0, 0 },
    { 0x10b9e20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b9e23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b9e29, 33 },
    { 0x1ceb9e29, 98 },
    { 0x49db9e2b, 23 },
    { 0xa41b9e2b, 66 },
    { 0xf74b9e2c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00b9e31, 75 },
    { 0x406b9e32, 33 },
    { 0, 0 },
    { 0xb9cb9e34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b9e39, 76 },
    { 0, 0 },
    { 0x105b9e3b, 91 },
    { 0x29bb9e3c, 93 },
    { 0x314b9e3d, 24 },
    { 0x659b9e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb9e41, 100 },
    { 0xcd3b9e42, 34 },
    { 0, 0 },
    { 0xefcb9e44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb9e49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67b9e4d, 17 },
    { 0x6b3b9e4d, 46 },
    { 0, 0 },
    { 0x273b9e50, 90 },
    { 0, 0 },
    { 0xf36b9e52, 43 },
    { 0x4b5b9e52, 64 },
    { 0x56eb9e52, 76 },
    { 0, 0 },
    { 0x81ab9e56, 90 },
    { 0, 0 },
    { 0x7ccb9e58, 33 },
    { 0x555b9e59, 34 },
    { 0xdfb9e59, 90 },
    { 0xbf2b9e5b, 23 },
    { 0x28eb9e5b, 69 },
    { 0xb43b9e5d, 81 },
    { 0x3d1b9e5e, 61 },
    { 0x8c0b9e5f, 91 },
    { 0xd26b9e60, 23 },
    { 0xadfb9e5e, 95 },
    { 0x691b9e62, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b9e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b9e6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb9e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cb9e75, 66 },
    { 0, 0 },
    { 0x54bb9e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b9e7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b9e7f, 98 },
    { 0x4deb9e80, 90 },
    { 0xc69b9e80, 81 },
    { 0x12bb9e80, 87 },
    { 0, 0 },
    { 0xbeb9e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b9e8b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ab9e95, 90 },
    { 0, 0 },
    { 0x25b9e97, 95 },
    { 0x425b9e98, 90 },
    { 0xb32b9e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb9e9e, 76 },
    { 0xd31b9e9e, 33 },
    { 0, 0 },
    { 0x784b9ea1, 95 },
    { 0, 0 },
    { 0x87fb9ea3, 100 },
    { 0x6fdb9ea4, 67 },
    { 0x6eeb9ea4, 93 },
    { 0x2ab9ea4, 23 },
    { 0xe85b9ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10b9eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09b9ead, 61 },
    { 0x1cdb9eae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x799b9eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db9eb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b9ebc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b9ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421b9ec6, 76 },
    { 0xc69b9ec7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4b9ece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb9ed1, 58 },
    { 0xc31b9ed1, 90 },
    { 0x844b9ed3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b9ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x499b9ed9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647b9edd, 61 },
    { 0xd2b9edd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3b9ee1, 61 },
    { 0xcfbb9ee1, 100 },
    { 0xa70b9ee2, 70 },
    { 0xf3bb9ee4, 88 },
    { 0xc5b9ee4, 55 },
    { 0x84fb9ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eb9ee9, 61 },
    { 0, 0 },
    { 0x4dfb9eeb, 33 },
    { 0xb73b9eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8b9eef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfb9ef2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab9efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb9eff, 93 },
    { 0x2a6b9f00, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b9f06, 98 },
    { 0, 0 },
    { 0xc30b9f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbb9f19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeb9f1d, 97 },
    { 0x50db9f1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ab9f22, 99 },
    { 0, 0 },
    { 0x199b9f24, 76 },
    { 0xdf1b9f25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7b9f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb9f30, 55 },
    { 0, 0 },
    { 0xc67b9f32, 64 },
    { 0x2ebb9f32, 78 },
    { 0x86ab9f34, 55 },
    { 0xbab9f34, 90 },
    { 0x81bb9f36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b9f39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50b9f3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1b9f3f, 23 },
    { 0xa29b9f40, 95 },
    { 0x93cb9f3f, 90 },
    { 0xda0b9f42, 90 },
    { 0x40fb9f3f, 17 },
    { 0xf7b9f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b9f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb9f4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb9f55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb9f5a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09b9f5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b9f62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b9f65, 93 },
    { 0x441b9f66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b9f6e, 90 },
    { 0, 0 },
    { 0xc0ab9f70, 48 },
    { 0, 0 },
    { 0x78db9f72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb9f76, 84 },
    { 0, 0 },
    { 0xfcfb9f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b9f7b, 33 },
    { 0xc42b9f7c, 88 },
    { 0xb3fb9f7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52b9f80, 66 },
    { 0x28db9f81, 55 },
    { 0x1c3b9f82, 61 },
    { 0x4d3b9f83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fb9f87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb9f8b, 56 },
    { 0xf7ab9f8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fb9f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb9f97, 55 },
    { 0x290b9f98, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117b9f9e, 95 },
    { 0x607b9f9e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09b9fa5, 90 },
    { 0xc49b9fa6, 80 },
    { 0, 0 },
    { 0xfeb9fa8, 55 },
    { 0xfc8b9fa8, 76 },
    { 0xc79b9faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b9fad, 93 },
    { 0xf0cb9fae, 84 },
    { 0, 0 },
    { 0x41ab9fb0, 95 },
    { 0x7bcb9fb1, 23 },
    { 0x910b9fb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb9fbc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b9fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b9fc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb9fcc, 55 },
    { 0xa76b9fcc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb9fd2, 43 },
    { 0xe71b9fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b9fd6, 66 },
    { 0x556b9fd7, 86 },
    { 0x48db9fd8, 61 },
    { 0x91db9fd9, 78 },
    { 0xf91b9fd8, 34 },
    { 0x277b9fda, 99 },
    { 0x5edb9fdc, 33 },
    { 0x314b9fdc, 90 },
    { 0x697b9fda, 90 },
    { 0x84b9fdb, 61 },
    { 0x1fbb9fdb, 61 },
    { 0x3f1b9fe1, 43 },
    { 0, 0 },
    { 0x2ceb9fe3, 87 },
    { 0xd17b9fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b9feb, 61 },
    { 0x1a8b9feb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b9fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b9ff3, 93 },
    { 0x919b9ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48eb9ff7, 61 },
    { 0, 0 },
    { 0x265b9ff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3b9ffc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bba001, 33 },
    { 0x6aaba002, 98 },
    { 0x38dba003, 96 },
    { 0xd9cba002, 98 },
    { 0, 0 },
    { 0xc80ba006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffba009, 98 },
    { 0x86aba00a, 64 },
    { 0x20bba00a, 98 },
    { 0x38cba00c, 43 },
    { 0x4ba009, 75 },
    { 0x6ecba00e, 33 },
    { 0x19fba00e, 66 },
    { 0, 0 },
    { 0x54aba011, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bba014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cba018, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8ba020, 55 },
    { 0x282ba020, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bba029, 55 },
    { 0, 0 },
    { 0x3eba02b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ba02e, 23 },
    { 0, 0 },
    { 0x71ba030, 61 },
    { 0, 0 },
    { 0x401ba032, 40 },
    { 0, 0 },
    { 0xdf4ba034, 78 },
    { 0xf23ba035, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x540ba038, 43 },
    { 0xfc9ba039, 90 },
    { 0x40fba03a, 61 },
    { 0x73aba03b, 95 },
    { 0, 0 },
    { 0x620ba03d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x431ba040, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cba044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bba048, 34 },
    { 0x3c4ba049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ba04e, 33 },
    { 0, 0 },
    { 0x16eba050, 90 },
    { 0xc92ba051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535ba05d, 55 },
    { 0x48ba05e, 98 },
    { 0x75eba05f, 46 },
    { 0x3faba05f, 95 },
    { 0, 0 },
    { 0x855ba062, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ba065, 93 },
    { 0x3eaba066, 33 },
    { 0xdf4ba067, 98 },
    { 0, 0 },
    { 0x8b1ba069, 98 },
    { 0xfd7ba06a, 93 },
    { 0x4c3ba06a, 82 },
    { 0x6aaba069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dba070, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eba077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602ba07b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973ba081, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3ba086, 85 },
    { 0x5c7ba087, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9ba090, 14 },
    { 0x7ccba091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ba094, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaba098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ba0a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ba0a4, 33 },
    { 0, 0 },
    { 0xc31ba0a6, 52 },
    { 0xe7bba0a7, 95 },
    { 0, 0 },
    { 0xed0ba0a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492ba0af, 52 },
    { 0xc97ba0af, 33 },
    { 0x321ba0b1, 76 },
    { 0xd74ba0b0, 9 },
    { 0, 0 },
    { 0x828ba0b4, 61 },
    { 0x3ceba0b4, 75 },
    { 0, 0 },
    { 0xaf2ba0b7, 33 },
    { 0xbf0ba0b8, 77 },
    { 0xc0cba0b7, 77 },
    { 0x170ba0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ba0bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ba0c4, 90 },
    { 0x46eba0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89ba0cc, 61 },
    { 0, 0 },
    { 0x71eba0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7ba0d4, 33 },
    { 0, 0 },
    { 0x1d5ba0d6, 68 },
    { 0, 0 },
    { 0xc07ba0d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba0df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cba0e3, 91 },
    { 0xfc3ba0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efba0e9, 23 },
    { 0x29dba0ea, 95 },
    { 0x1edba0eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ba0ef, 23 },
    { 0xc20ba0f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaba0f3, 90 },
    { 0x91ba0f4, 61 },
    { 0xa77ba0f4, 98 },
    { 0xfa2ba0f3, 98 },
    { 0xe5dba0f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ba0fe, 33 },
    { 0xa94ba0fe, 33 },
    { 0xaf9ba100, 46 },
    { 0x759ba0fe, 40 },
    { 0xca0ba102, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aba105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eba10b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96eba110, 98 },
    { 0x367ba111, 81 },
    { 0, 0 },
    { 0x3adba113, 95 },
    { 0, 0 },
    { 0xae1ba115, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1ba119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbba11e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eba121, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaba127, 46 },
    { 0xd69ba127, 90 },
    { 0xeba127, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6ba12d, 92 },
    { 0x574ba12e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ba131, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48ba134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76ba139, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fba13c, 78 },
    { 0x2a4ba13d, 92 },
    { 0x450ba13d, 34 },
    { 0xe2aba13c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3ba145, 78 },
    { 0x725ba146, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dba14b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19ba14e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ba153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ba15b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61ba15f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabba167, 33 },
    { 0, 0 },
    { 0x1d7ba169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ba171, 76 },
    { 0, 0 },
    { 0x213ba173, 43 },
    { 0xaccba174, 33 },
    { 0x2c4ba174, 69 },
    { 0x419ba175, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104ba17a, 68 },
    { 0xe8dba17a, 95 },
    { 0xa73ba17c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dba180, 34 },
    { 0xb6cba181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daba184, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ba189, 95 },
    { 0x7bfba18a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ba18f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbba192, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deba198, 19 },
    { 0x955ba198, 61 },
    { 0, 0 },
    { 0xf8ba19b, 95 },
    { 0x5e8ba19c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07ba1a0, 19 },
    { 0, 0 },
    { 0x27bba1a2, 98 },
    { 0, 0 },
    { 0xf1aba1a4, 95 },
    { 0x101ba1a5, 33 },
    { 0xa87ba1a6, 23 },
    { 0xf93ba1a7, 33 },
    { 0xdcba1a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dba1af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba1ba, 23 },
    { 0xc11ba1bb, 100 },
    { 0x33ba1ba, 90 },
    { 0x63ba1bd, 87 },
    { 0xe7cba1be, 33 },
    { 0xb90ba1bf, 61 },
    { 0x55bba1bd, 61 },
    { 0x596ba1c1, 90 },
    { 0xee8ba1c2, 98 },
    { 0x5ebba1c1, 34 },
    { 0, 0 },
    { 0x3edba1c5, 98 },
    { 0, 0 },
    { 0xb47ba1c7, 76 },
    { 0x486ba1c7, 46 },
    { 0xd9ba1c9, 61 },
    { 0xccba1ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba1cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cba1d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ba1d5, 23 },
    { 0xd88ba1d6, 23 },
    { 0x1c8ba1d7, 33 },
    { 0x664ba1d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fba1dc, 95 },
    { 0x632ba1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ba1e1, 95 },
    { 0xae3ba1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ba1e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ba1eb, 66 },
    { 0x42eba1ec, 100 },
    { 0x493ba1eb, 93 },
    { 0x8ffba1ee, 90 },
    { 0, 0 },
    { 0xb00ba1f0, 53 },
    { 0x537ba1f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ba1f4, 95 },
    { 0x1eeba1f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba1f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ba1fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ba201, 90 },
    { 0xb14ba202, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ba20c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ba20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ba212, 99 },
    { 0xd73ba213, 99 },
    { 0xf65ba213, 76 },
    { 0xddbba215, 76 },
    { 0x818ba216, 34 },
    { 0x25bba216, 66 },
    { 0x8a4ba218, 91 },
    { 0x836ba217, 98 },
    { 0x561ba218, 95 },
    { 0xa58ba215, 98 },
    { 0x166ba217, 33 },
    { 0x205ba219, 43 },
    { 0x4d4ba215, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59ba221, 90 },
    { 0x446ba221, 55 },
    { 0x214ba223, 33 },
    { 0x3ccba222, 61 },
    { 0xf62ba225, 33 },
    { 0x70ba225, 76 },
    { 0xaf7ba227, 23 },
    { 0xe9bba222, 83 },
    { 0xdd0ba226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109ba22e, 95 },
    { 0xa76ba22f, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x513ba232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857ba237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865ba23b, 33 },
    { 0, 0 },
    { 0xda5ba23d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba241, 83 },
    { 0, 0 },
    { 0x5f8ba243, 95 },
    { 0x74ba244, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4aba249, 90 },
    { 0x6ccba249, 33 },
    { 0x1cfba24b, 76 },
    { 0xa60ba24c, 33 },
    { 0, 0 },
    { 0xdcdba24e, 69 },
    { 0, 0 },
    { 0xd21ba250, 81 },
    { 0x7e5ba251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16ba254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ba257, 61 },
    { 0x2afba258, 51 },
    { 0xa1fba257, 33 },
    { 0xc11ba25a, 65 },
    { 0x858ba25b, 87 },
    { 0, 0 },
    { 0xab1ba25d, 91 },
    { 0, 0 },
    { 0x56aba25f, 61 },
    { 0x649ba260, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7aba264, 61 },
    { 0x1e9ba264, 87 },
    { 0x373ba266, 52 },
    { 0x5e1ba267, 23 },
    { 0x89bba264, 33 },
    { 0xae4ba269, 90 },
    { 0x3adba26a, 33 },
    { 0, 0 },
    { 0xa31ba26c, 95 },
    { 0x4c1ba26d, 19 },
    { 0xabcba26e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdba27a, 23 },
    { 0x206ba27b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecba27e, 76 },
    { 0xae3ba27e, 93 },
    { 0xc50ba280, 90 },
    { 0xbe0ba281, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19ba284, 68 },
    { 0x9d6ba284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192ba28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bba294, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cba297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bba29a, 55 },
    { 0xee6ba29b, 90 },
    { 0xf88ba29b, 66 },
    { 0, 0 },
    { 0xddfba29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ba2a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2ba2ac, 95 },
    { 0, 0 },
    { 0xc7cba2ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243ba2b2, 66 },
    { 0x7a8ba2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7ba2b8, 43 },
    { 0xe43ba2b8, 19 },
    { 0x2ddba2ba, 23 },
    { 0x796ba2bb, 51 },
    { 0, 0 },
    { 0x532ba2bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ba2c0, 90 },
    { 0xf48ba2c1, 43 },
    { 0x3fba2c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fba2c7, 90 },
    { 0xe28ba2c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838ba2cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5ba2cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fba2d2, 61 },
    { 0, 0 },
    { 0x19dba2d4, 55 },
    { 0xfc1ba2d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacba2dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ba2e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ba2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bba2e9, 46 },
    { 0x48aba2e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121ba2ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ba2f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fba2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940ba300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ba304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376ba30a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7ba30e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaba311, 82 },
    { 0xe80ba311, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09ba316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaba31b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3ba31e, 90 },
    { 0xd7cba31f, 95 },
    { 0xd96ba31e, 76 },
    { 0x1b8ba321, 34 },
    { 0, 0 },
    { 0x6b7ba323, 61 },
    { 0, 0 },
    { 0x11ba325, 33 },
    { 0x7b2ba325, 90 },
    { 0, 0 },
    { 0xf0aba328, 90 },
    { 0xb4cba329, 87 },
    { 0xd4dba32a, 55 },
    { 0x4b6ba32b, 59 },
    { 0xd86ba329, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ba331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aba337, 34 },
    { 0, 0 },
    { 0x6d7ba339, 84 },
    { 0x21eba339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ba33d, 33 },
    { 0x9d0ba33e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fba341, 95 },
    { 0, 0 },
    { 0x401ba343, 33 },
    { 0, 0 },
    { 0xaeeba345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8ba348, 38 },
    { 0, 0 },
    { 0x70bba34a, 38 },
    { 0, 0 },
    { 0xdebba34c, 98 },
    { 0x339ba34c, 76 },
    { 0xb5cba34e, 33 },
    { 0, 0 },
    { 0x83ba350, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ba356, 95 },
    { 0xd5eba357, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ba360, 77 },
    { 0x2e3ba361, 100 },
    { 0, 0 },
    { 0xe1aba363, 55 },
    { 0, 0 },
    { 0xe69ba365, 90 },
    { 0x981ba366, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x876ba369, 90 },
    { 0, 0 },
    { 0x38cba36b, 33 },
    { 0x345ba36c, 76 },
    { 0xa43ba36c, 23 },
    { 0x502ba36d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ba371, 68 },
    { 0x3c4ba372, 76 },
    { 0x27aba371, 55 },
    { 0xab4ba373, 33 },
    { 0x5baba375, 76 },
    { 0xf9fba371, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bba37a, 61 },
    { 0x8faba37a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebba37e, 96 },
    { 0x578ba37f, 61 },
    { 0xcc7ba380, 68 },
    { 0x840ba381, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ba384, 91 },
    { 0xfe9ba385, 95 },
    { 0, 0 },
    { 0x404ba387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ba391, 61 },
    { 0, 0 },
    { 0x83cba393, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ba397, 90 },
    { 0x329ba398, 90 },
    { 0, 0 },
    { 0x44bba39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4ba39d, 93 },
    { 0x122ba39e, 91 },
    { 0xe37ba39d, 33 },
    { 0, 0 },
    { 0x51fba3a1, 61 },
    { 0x785ba3a2, 58 },
    { 0x36eba3a3, 46 },
    { 0, 0 },
    { 0xa84ba3a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ba3aa, 98 },
    { 0x649ba3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x116ba3ae, 34 },
    { 0, 0 },
    { 0x23eba3b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ba3bb, 34 },
    { 0x906ba3bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ba3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ba3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ba3c8, 98 },
    { 0, 0 },
    { 0x382ba3ca, 69 },
    { 0xf9bba3cb, 33 },
    { 0xff3ba3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4ba3d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ba3d3, 85 },
    { 0x115ba3d4, 55 },
    { 0, 0 },
    { 0x9b2ba3d6, 77 },
    { 0x45eba3d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ba3dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bba3e1, 98 },
    { 0x5acba3e1, 95 },
    { 0, 0 },
    { 0xe8fba3e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abba3ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7ba3ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5ba3f2, 61 },
    { 0, 0 },
    { 0x23eba3f4, 90 },
    { 0x1d2ba3f4, 90 },
    { 0xdc5ba3f6, 23 },
    { 0x1d4ba3f7, 55 },
    { 0xbeeba3f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ba401, 34 },
    { 0x286ba402, 90 },
    { 0x1deba402, 95 },
    { 0xec0ba401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aba407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bba40b, 23 },
    { 0, 0 },
    { 0x8d1ba40d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ba412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ba415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5ba418, 23 },
    { 0x15dba419, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ba41d, 90 },
    { 0, 0 },
    { 0x37cba41f, 85 },
    { 0x705ba41f, 99 },
    { 0x77fba421, 68 },
    { 0x19aba422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888ba425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3ba428, 95 },
    { 0x6a0ba429, 94 },
    { 0xc4dba42a, 93 },
    { 0, 0 },
    { 0x7a9ba42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ba437, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bba43d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ba440, 17 },
    { 0xf51ba441, 61 },
    { 0xffbba441, 91 },
    { 0x5b3ba442, 90 },
    { 0xd03ba444, 93 },
    { 0, 0 },
    { 0x258ba446, 43 },
    { 0x6ba447, 98 },
    { 0x854ba448, 33 },
    { 0x12fba449, 23 },
    { 0, 0 },
    { 0x5c9ba44b, 33 },
    { 0x988ba44c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4ba44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdba454, 75 },
    { 0x154ba454, 95 },
    { 0xbbeba455, 55 },
    { 0x2f0ba457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ba45b, 68 },
    { 0, 0 },
    { 0xea1ba45d, 95 },
    { 0, 0 },
    { 0x9b3ba45f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08ba464, 46 },
    { 0xe7eba465, 85 },
    { 0xe35ba466, 55 },
    { 0, 0 },
    { 0xaaba468, 55 },
    { 0x71fba468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ba46c, 61 },
    { 0x1acba46d, 98 },
    { 0x8abba46e, 33 },
    { 0x59ba46d, 76 },
    { 0, 0 },
    { 0xfa3ba471, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ba47b, 99 },
    { 0xc5fba47c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ba483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9ba486, 66 },
    { 0x9c6ba486, 53 },
    { 0xba0ba488, 23 },
    { 0, 0 },
    { 0x824ba48a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ba48d, 55 },
    { 0xe8aba48e, 90 },
    { 0, 0 },
    { 0x5aba490, 84 },
    { 0xac0ba490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ba498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aba49c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ba4a0, 33 },
    { 0, 0 },
    { 0x633ba4a2, 98 },
    { 0xfb1ba4a3, 77 },
    { 0x1d6ba4a4, 77 },
    { 0x74bba4a4, 98 },
    { 0xaabba4a6, 95 },
    { 0x852ba4a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x801ba4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fba4ad, 33 },
    { 0xbc5ba4ae, 66 },
    { 0, 0 },
    { 0x353ba4b0, 33 },
    { 0x26fba4b1, 33 },
    { 0xf25ba4b2, 44 },
    { 0, 0 },
    { 0xf25ba4b4, 55 },
    { 0x5d5ba4b5, 76 },
    { 0, 0 },
    { 0x106ba4b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ba4bc, 100 },
    { 0xd33ba4bd, 90 },
    { 0xef8ba4bc, 83 },
    { 0xf63ba4bf, 90 },
    { 0x9e2ba4bf, 90 },
    { 0, 0 },
    { 0x16eba4c2, 78 },
    { 0xbcdba4c2, 95 },
    { 0x5d5ba4c4, 23 },
    { 0x49eba4c5, 84 },
    { 0xc7bba4c5, 34 },
    { 0x9b3ba4c2, 90 },
    { 0xeeaba4c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaba4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ba4d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba4d5, 76 },
    { 0, 0 },
    { 0x376ba4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8ba4df, 33 },
    { 0x943ba4e0, 43 },
    { 0x97bba4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aba4e4, 23 },
    { 0, 0 },
    { 0x44dba4e6, 97 },
    { 0x538ba4e7, 87 },
    { 0x9bcba4e7, 55 },
    { 0, 0 },
    { 0xb7fba4ea, 23 },
    { 0, 0 },
    { 0xb34ba4ec, 61 },
    { 0xa37ba4ed, 23 },
    { 0x2daba4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878ba4f5, 35 },
    { 0, 0 },
    { 0x5e7ba4f7, 87 },
    { 0xda3ba4f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293ba4fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ba504, 61 },
    { 0xf76ba505, 33 },
    { 0x2ba506, 81 },
    { 0xf2ba507, 17 },
    { 0x8b4ba508, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fba50b, 90 },
    { 0xd64ba50c, 33 },
    { 0x7a3ba50d, 91 },
    { 0xbf0ba50e, 99 },
    { 0xdb6ba50d, 43 },
    { 0x22dba510, 43 },
    { 0x603ba511, 81 },
    { 0xda4ba512, 90 },
    { 0x5aba512, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178ba517, 84 },
    { 0, 0 },
    { 0xf51ba519, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215ba51d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2ba522, 78 },
    { 0x9edba522, 61 },
    { 0x6fcba523, 95 },
    { 0x4a4ba525, 33 },
    { 0xa5ba525, 33 },
    { 0x1d6ba526, 33 },
    { 0xd2eba522, 95 },
    { 0xc71ba529, 61 },
    { 0x37aba52a, 90 },
    { 0x84bba52b, 33 },
    { 0xa49ba52c, 78 },
    { 0xf32ba529, 61 },
    { 0, 0 },
    { 0x3c9ba52f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ba533, 90 },
    { 0x775ba533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ba539, 66 },
    { 0x9b2ba53a, 100 },
    { 0xacfba53b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eba53e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62ba541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15ba546, 33 },
    { 0x978ba547, 23 },
    { 0x99eba547, 61 },
    { 0xc65ba549, 87 },
    { 0x608ba54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fba54d, 55 },
    { 0, 0 },
    { 0xad7ba54f, 33 },
    { 0xed6ba54f, 95 },
    { 0x6e8ba550, 61 },
    { 0, 0 },
    { 0xb07ba553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ba558, 100 },
    { 0xaa0ba559, 97 },
    { 0x1ebba55a, 23 },
    { 0x576ba55a, 33 },
    { 0, 0 },
    { 0x6ba55d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eba563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cba567, 98 },
    { 0, 0 },
    { 0x617ba569, 90 },
    { 0x72dba56a, 93 },
    { 0xb0dba56b, 55 },
    { 0x59ba56c, 90 },
    { 0x4f6ba56c, 98 },
    { 0xe8aba56e, 98 },
    { 0, 0 },
    { 0x942ba570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caba573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320ba57c, 90 },
    { 0x9d4ba57d, 76 },
    { 0, 0 },
    { 0x373ba57f, 23 },
    { 0, 0 },
    { 0xc5dba581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ba584, 66 },
    { 0x106ba585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76ba588, 90 },
    { 0xe0eba589, 95 },
    { 0x612ba589, 98 },
    { 0, 0 },
    { 0x47cba58c, 87 },
    { 0, 0 },
    { 0xef0ba58e, 46 },
    { 0x2b6ba58e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ba594, 90 },
    { 0x924ba595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ba599, 99 },
    { 0x184ba599, 99 },
    { 0, 0 },
    { 0xd01ba59c, 95 },
    { 0, 0 },
    { 0xed1ba59e, 90 },
    { 0xac5ba59e, 33 },
    { 0xa29ba5a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ba5a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5ba5b5, 23 },
    { 0, 0 },
    { 0xc88ba5b7, 98 },
    { 0xb1bba5b8, 90 },
    { 0x7aba5b8, 23 },
    { 0xa4cba5ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ba5be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbba5c3, 98 },
    { 0x9efba5c4, 34 },
    { 0x89cba5c4, 33 },
    { 0xf7cba5c6, 55 },
    { 0, 0 },
    { 0xd22ba5c8, 33 },
    { 0, 0 },
    { 0x393ba5ca, 98 },
    { 0xbba5cb, 90 },
    { 0xf92ba5cc, 98 },
    { 0x2bdba5cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aba5d2, 61 },
    { 0x13fba5d3, 78 },
    { 0xd61ba5d4, 98 },
    { 0xea4ba5d4, 90 },
    { 0x457ba5d6, 76 },
    { 0x9d5ba5d6, 76 },
    { 0xf90ba5d5, 78 },
    { 0, 0 },
    { 0x64dba5da, 93 },
    { 0, 0 },
    { 0xc98ba5dc, 33 },
    { 0x7d3ba5dc, 85 },
    { 0x31fba5de, 33 },
    { 0x2a3ba5dd, 55 },
    { 0, 0 },
    { 0xc26ba5e1, 23 },
    { 0x529ba5e2, 99 },
    { 0xa7aba5e3, 33 },
    { 0xb8cba5e2, 34 },
    { 0, 0 },
    { 0x105ba5e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcba5eb, 87 },
    { 0, 0 },
    { 0x2ddba5ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bba5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287ba5fa, 98 },
    { 0xadfba5fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0ba606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53ba609, 61 },
    { 0x42fba60a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadba611, 77 },
    { 0, 0 },
    { 0x35ba613, 43 },
    { 0, 0 },
    { 0x559ba615, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cba61a, 90 },
    { 0xb50ba61b, 61 },
    { 0x58ba61c, 33 },
    { 0x6a0ba61d, 61 },
    { 0, 0 },
    { 0x5bcba61f, 88 },
    { 0x46fba61f, 33 },
    { 0, 0 },
    { 0x610ba622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292ba625, 55 },
    { 0x6feba626, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x366ba629, 66 },
    { 0x215ba62a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72ba634, 34 },
    { 0x7d8ba635, 90 },
    { 0x919ba636, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32ba63b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x483ba63e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ba642, 55 },
    { 0x895ba642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba648, 81 },
    { 0x2bfba648, 67 },
    { 0xcb8ba64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ba64d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1ba652, 83 },
    { 0x282ba652, 98 },
    { 0, 0 },
    { 0x24bba655, 78 },
    { 0xfbba655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2ba659, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ba660, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ba664, 68 },
    { 0, 0 },
    { 0xb9cba666, 23 },
    { 0x8cbba667, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeba677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddba681, 76 },
    { 0xb51ba682, 23 },
    { 0, 0 },
    { 0x542ba684, 78 },
    { 0x771ba685, 23 },
    { 0x8a7ba685, 61 },
    { 0x8eeba687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ba68a, 53 },
    { 0x535ba68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829ba68f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272ba693, 78 },
    { 0x8d6ba694, 65 },
    { 0, 0 },
    { 0x151ba696, 67 },
    { 0, 0 },
    { 0x457ba698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295ba69c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fba6a0, 98 },
    { 0x5e3ba6a0, 46 },
    { 0xc1aba6a2, 90 },
    { 0xb99ba6a2, 95 },
    { 0xa4dba6a4, 33 },
    { 0, 0 },
    { 0x1bfba6a6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966ba6ac, 90 },
    { 0x128ba6ac, 95 },
    { 0x23aba6ae, 61 },
    { 0x911ba6af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7ba6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcba6bf, 81 },
    { 0, 0 },
    { 0xdb2ba6c1, 95 },
    { 0x3dba6c2, 98 },
    { 0x9feba6c2, 95 },
    { 0, 0 },
    { 0x715ba6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462ba6c9, 33 },
    { 0, 0 },
    { 0x954ba6cb, 42 },
    { 0xc9aba6cc, 98 },
    { 0x958ba6cb, 90 },
    { 0xc3bba6ce, 54 },
    { 0x769ba6ce, 98 },
    { 0xa13ba6d0, 40 },
    { 0, 0 },
    { 0xd1ba6d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eba6d5, 90 },
    { 0, 0 },
    { 0x4efba6d7, 93 },
    { 0, 0 },
    { 0x7b9ba6d9, 78 },
    { 0x934ba6d9, 61 },
    { 0xfabba6da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfba6de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167ba6e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cba6e8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x640ba6eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56ba6fa, 85 },
    { 0x73aba6fb, 34 },
    { 0x4a5ba6fa, 95 },
    { 0, 0 },
    { 0xe4cba6fe, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40ba701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ba704, 76 },
    { 0, 0 },
    { 0xf41ba706, 90 },
    { 0, 0 },
    { 0xd9bba708, 61 },
    { 0x655ba709, 90 },
    { 0x697ba70a, 26 },
    { 0xda1ba70b, 90 },
    { 0, 0 },
    { 0x941ba70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eba710, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ba713, 46 },
    { 0xabcba714, 85 },
    { 0xc6fba713, 76 },
    { 0xd82ba714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fba71b, 33 },
    { 0, 0 },
    { 0xc2dba71d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00ba720, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52ba723, 76 },
    { 0x4a6ba724, 55 },
    { 0x23aba725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ba729, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fba72c, 93 },
    { 0xb06ba72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ba731, 33 },
    { 0xd60ba732, 34 },
    { 0xac5ba733, 33 },
    { 0x43eba733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1ba739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3ba73c, 90 },
    { 0x323ba73d, 98 },
    { 0, 0 },
    { 0x5e0ba73f, 39 },
    { 0xd24ba73f, 95 },
    { 0x592ba741, 98 },
    { 0x63bba73f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cba747, 55 },
    { 0x2ecba748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fba74d, 87 },
    { 0xfa5ba74e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3ba752, 61 },
    { 0, 0 },
    { 0x417ba754, 90 },
    { 0, 0 },
    { 0xcfcba756, 33 },
    { 0x3c7ba756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256ba75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ba763, 23 },
    { 0xd33ba764, 33 },
    { 0xf51ba765, 67 },
    { 0xc8fba766, 65 },
    { 0xf10ba767, 76 },
    { 0xc43ba768, 33 },
    { 0, 0 },
    { 0x230ba76a, 95 },
    { 0xf27ba76b, 95 },
    { 0x401ba76c, 33 },
    { 0, 0 },
    { 0xedba76e, 52 },
    { 0, 0 },
    { 0xd73ba770, 23 },
    { 0x8e9ba771, 17 },
    { 0xf82ba772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138ba779, 23 },
    { 0xf04ba77a, 92 },
    { 0x5beba77a, 66 },
    { 0x38cba77a, 61 },
    { 0x841ba77d, 43 },
    { 0x5deba77e, 23 },
    { 0xffeba77d, 36 },
    { 0, 0 },
    { 0x91bba781, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bba785, 93 },
    { 0xb9eba785, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69ba78d, 23 },
    { 0x35fba78e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bba792, 90 },
    { 0x236ba792, 33 },
    { 0x267ba794, 33 },
    { 0xf18ba795, 46 },
    { 0x8bcba796, 95 },
    { 0xa2fba797, 33 },
    { 0x657ba797, 93 },
    { 0xcc8ba799, 61 },
    { 0x276ba79a, 90 },
    { 0, 0 },
    { 0xe5eba79c, 78 },
    { 0x861ba79d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ba7a1, 90 },
    { 0xcb6ba7a1, 66 },
    { 0, 0 },
    { 0x65bba7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ba7a9, 100 },
    { 0, 0 },
    { 0x3acba7ab, 78 },
    { 0xcdeba7ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29ba7b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfba7b9, 90 },
    { 0x7e8ba7ba, 34 },
    { 0, 0 },
    { 0x3a1ba7bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ba7c1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fba7c4, 98 },
    { 0, 0 },
    { 0xd9dba7c6, 43 },
    { 0x9f8ba7c7, 23 },
    { 0xcfcba7c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ba7cc, 17 },
    { 0, 0 },
    { 0x4a6ba7ce, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x408ba7d1, 23 },
    { 0xf32ba7d1, 98 },
    { 0x2f9ba7d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfba7d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36ba7dc, 90 },
    { 0x807ba7dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9ba7e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ba7e3, 34 },
    { 0x408ba7e3, 23 },
    { 0xbdaba7e5, 97 },
    { 0xceba7e4, 23 },
    { 0x8cba7e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcba7ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2ba7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdba7f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ba7f9, 98 },
    { 0x6a1ba7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868ba7fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51ba803, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eba808, 34 },
    { 0x52aba808, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69eba810, 98 },
    { 0xbe5ba811, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5ba815, 98 },
    { 0x855ba816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eba824, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cba829, 46 },
    { 0xd4bba82a, 93 },
    { 0x663ba82a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18aba832, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fba836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ba839, 91 },
    { 0x484ba839, 23 },
    { 0, 0 },
    { 0x809ba83c, 90 },
    { 0xd1dba83d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ba848, 90 },
    { 0x99dba848, 95 },
    { 0xba3ba848, 95 },
    { 0xfabba84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x487ba84e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ba851, 33 },
    { 0xe09ba851, 55 },
    { 0, 0 },
    { 0xefbba854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ba85a, 67 },
    { 0, 0 },
    { 0xbcbba85c, 66 },
    { 0x418ba85c, 99 },
    { 0x653ba85e, 90 },
    { 0xe92ba85f, 98 },
    { 0, 0 },
    { 0x421ba861, 55 },
    { 0, 0 },
    { 0x23cba863, 23 },
    { 0xe79ba864, 43 },
    { 0, 0 },
    { 0x11aba866, 55 },
    { 0x4b8ba867, 81 },
    { 0xd26ba868, 23 },
    { 0, 0 },
    { 0x36dba86a, 90 },
    { 0xe6ba86a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ba86f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x984ba872, 90 },
    { 0x915ba873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25ba876, 76 },
    { 0x3afba877, 61 },
    { 0, 0 },
    { 0x31bba879, 23 },
    { 0x87dba87a, 93 },
    { 0, 0 },
    { 0x2d7ba87c, 93 },
    { 0xda7ba87c, 55 },
    { 0x8b3ba87e, 87 },
    { 0x359ba87f, 61 },
    { 0, 0 },
    { 0x3ecba881, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dba887, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ba88e, 33 },
    { 0, 0 },
    { 0xf0bba890, 90 },
    { 0, 0 },
    { 0xe44ba892, 17 },
    { 0x677ba893, 76 },
    { 0, 0 },
    { 0x78ba895, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x215ba898, 61 },
    { 0x9c2ba899, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aba89c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1aba8a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdba8a3, 74 },
    { 0, 0 },
    { 0x8e3ba8a5, 76 },
    { 0xc09ba8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0ba8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dba8ae, 93 },
    { 0x9fdba8af, 93 },
    { 0, 0 },
    { 0x547ba8b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62eba8b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cba8b9, 46 },
    { 0xe6eba8ba, 90 },
    { 0xa0bba8ba, 64 },
    { 0, 0 },
    { 0xa8dba8bd, 91 },
    { 0xd53ba8be, 66 },
    { 0xe41ba8bf, 90 },
    { 0xebeba8bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cba8cf, 33 },
    { 0, 0 },
    { 0xdb1ba8d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ba8d4, 46 },
    { 0x44dba8d5, 50 },
    { 0x4f0ba8d5, 48 },
    { 0, 0 },
    { 0x644ba8d8, 61 },
    { 0, 0 },
    { 0x7cba8da, 99 },
    { 0xea1ba8db, 95 },
    { 0xe70ba8dc, 33 },
    { 0xd4aba8dd, 23 },
    { 0x920ba8dd, 95 },
    { 0x2e3ba8df, 99 },
    { 0x906ba8da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7ba8e7, 99 },
    { 0x2eeba8e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdba8f0, 91 },
    { 0xe4bba8f1, 61 },
    { 0x242ba8f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4ba8f5, 90 },
    { 0xd15ba8f6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ba8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ba8fe, 46 },
    { 0x774ba8ff, 33 },
    { 0x9b1ba900, 98 },
    { 0x8ecba900, 95 },
    { 0x9bbba902, 23 },
    { 0x360ba903, 43 },
    { 0xaabba903, 25 },
    { 0xd7ba903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ba909, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ba911, 55 },
    { 0, 0 },
    { 0x24dba913, 88 },
    { 0xd38ba913, 84 },
    { 0x7bfba913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x773ba918, 95 },
    { 0x8b1ba919, 61 },
    { 0xc99ba919, 55 },
    { 0x8cbba91b, 69 },
    { 0x109ba91c, 55 },
    { 0, 0 },
    { 0x6c4ba91e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dba921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baba926, 90 },
    { 0, 0 },
    { 0xebfba928, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x342ba92b, 87 },
    { 0, 0 },
    { 0x6e8ba92d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba934, 33 },
    { 0, 0 },
    { 0xd67ba936, 33 },
    { 0xb61ba937, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623ba93b, 70 },
    { 0, 0 },
    { 0xcfaba93d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ba941, 43 },
    { 0xcb4ba942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x586ba945, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ba94a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cba94f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba952, 95 },
    { 0, 0 },
    { 0x945ba954, 76 },
    { 0xeaaba955, 84 },
    { 0xb8dba956, 61 },
    { 0x7e1ba954, 90 },
    { 0x53dba955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eba95b, 67 },
    { 0x370ba95c, 95 },
    { 0xf9aba95b, 5 },
    { 0x56ba95b, 33 },
    { 0x4baba95f, 43 },
    { 0x599ba95e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dba964, 55 },
    { 0x367ba965, 85 },
    { 0xa7ba966, 87 },
    { 0xf2fba967, 61 },
    { 0, 0 },
    { 0x248ba969, 99 },
    { 0xe66ba969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ba972, 77 },
    { 0xa6fba972, 33 },
    { 0, 0 },
    { 0x906ba975, 34 },
    { 0x5b1ba976, 65 },
    { 0x380ba977, 19 },
    { 0x3a1ba977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fba97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcba97f, 43 },
    { 0xbd0ba97f, 52 },
    { 0x58aba981, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ba988, 90 },
    { 0, 0 },
    { 0x57ba98a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eba993, 23 },
    { 0x9dfba994, 95 },
    { 0xb2cba995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6ba99d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ba9a1, 90 },
    { 0xd43ba9a2, 95 },
    { 0x84eba9a3, 23 },
    { 0, 0 },
    { 0xdadba9a5, 90 },
    { 0, 0 },
    { 0xaedba9a7, 61 },
    { 0x56aba9a7, 24 },
    { 0x938ba9a7, 98 },
    { 0xe55ba9aa, 23 },
    { 0, 0 },
    { 0xb4ba9ac, 33 },
    { 0x170ba9ad, 55 },
    { 0xaa3ba9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ba9b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dba9b9, 61 },
    { 0xc97ba9ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6ba9bf, 95 },
    { 0x54aba9bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2ba9c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17ba9c6, 90 },
    { 0x4d2ba9c6, 96 },
    { 0, 0 },
    { 0x286ba9c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799ba9cd, 34 },
    { 0xb2aba9ce, 90 },
    { 0, 0 },
    { 0xde6ba9d0, 61 },
    { 0xf78ba9d1, 40 },
    { 0x1ecba9d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfba9d7, 61 },
    { 0x79aba9d8, 76 },
    { 0x260ba9d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdba9dc, 33 },
    { 0, 0 },
    { 0x69ba9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcba9e4, 23 },
    { 0xc65ba9e5, 55 },
    { 0, 0 },
    { 0x34ba9e7, 34 },
    { 0xacaba9e8, 91 },
    { 0xf96ba9e8, 76 },
    { 0, 0 },
    { 0xab1ba9eb, 52 },
    { 0xc4fba9ec, 76 },
    { 0xa0dba9ed, 90 },
    { 0x3d3ba9ee, 76 },
    { 0xa98ba9ed, 52 },
    { 0xd52ba9f0, 23 },
    { 0xd8dba9f1, 90 },
    { 0xda0ba9f2, 23 },
    { 0, 0 },
    { 0x9eba9f4, 76 },
    { 0x60aba9f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fba9f8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ba9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162baa01, 90 },
    { 0x4e0baa01, 80 },
    { 0x92dbaa03, 68 },
    { 0xb6cbaa03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbaa0b, 59 },
    { 0, 0 },
    { 0xd4ebaa0d, 47 },
    { 0, 0 },
    { 0xb61baa0f, 69 },
    { 0x81cbaa10, 87 },
    { 0, 0 },
    { 0x7b7baa12, 76 },
    { 0x58baa12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98baa17, 55 },
    { 0, 0 },
    { 0xe55baa19, 90 },
    { 0, 0 },
    { 0xaebbaa1b, 61 },
    { 0, 0 },
    { 0x63cbaa1d, 76 },
    { 0x784baa1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddbaa22, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x173baa25, 98 },
    { 0, 0 },
    { 0x92cbaa27, 85 },
    { 0, 0 },
    { 0xdcbbaa29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x911baa2c, 23 },
    { 0x676baa2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bebaa31, 40 },
    { 0x1eebaa32, 98 },
    { 0x19cbaa33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x844baa36, 90 },
    { 0, 0 },
    { 0x3c5baa38, 98 },
    { 0xd2dbaa39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dbaa3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x648baa42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2baa46, 93 },
    { 0x732baa46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baa4d, 90 },
    { 0xce1baa4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0baa53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412baa58, 76 },
    { 0xdabbaa58, 98 },
    { 0x15baa58, 23 },
    { 0xa0baa5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefbaa5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fbaa63, 52 },
    { 0xa75baa63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefbaa68, 33 },
    { 0, 0 },
    { 0x3adbaa6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dbaa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbaa72, 78 },
    { 0xd9abaa72, 76 },
    { 0x38cbaa74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdbaa7f, 33 },
    { 0xfdfbaa7f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cbaa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47baa8c, 61 },
    { 0, 0 },
    { 0xa5baa8e, 98 },
    { 0x6f9baa8f, 23 },
    { 0xb59baa8f, 33 },
    { 0x9d1baa8e, 76 },
    { 0xc6cbaa92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baa95, 88 },
    { 0x80dbaa96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2baa99, 87 },
    { 0xad3baa9a, 55 },
    { 0xe3cbaa9a, 90 },
    { 0x30ebaa9a, 61 },
    { 0, 0 },
    { 0x5afbaa9e, 76 },
    { 0x295baa9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbaaa2, 87 },
    { 0xc32baaa3, 33 },
    { 0x109baaa4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52baaa7, 61 },
    { 0, 0 },
    { 0x787baaa9, 93 },
    { 0, 0 },
    { 0x919baaab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfbaab0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94baab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbabaab8, 55 },
    { 0, 0 },
    { 0x47dbaaba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7baabd, 43 },
    { 0x6d4baabd, 69 },
    { 0x9fdbaabf, 90 },
    { 0xf71baabf, 23 },
    { 0xe2cbaac1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8baac9, 78 },
    { 0xec8baac9, 90 },
    { 0, 0 },
    { 0x6cebaacc, 90 },
    { 0, 0 },
    { 0x527baace, 55 },
    { 0, 0 },
    { 0xd32baad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1abaad6, 23 },
    { 0x187baad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0baadc, 43 },
    { 0, 0 },
    { 0x24abaade, 33 },
    { 0xcffbaade, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fbaae3, 76 },
    { 0xe97baae4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51baae7, 23 },
    { 0x32dbaae8, 81 },
    { 0x150baae8, 90 },
    { 0x51baaea, 90 },
    { 0xf7cbaaeb, 76 },
    { 0x477baaec, 95 },
    { 0, 0 },
    { 0x9ffbaaee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9baaf5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fbaaf9, 38 },
    { 0x9cbbaaf9, 98 },
    { 0xafdbaafb, 67 },
    { 0x37baafb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87baaff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x153bab02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fbab05, 46 },
    { 0x97ebab06, 95 },
    { 0x2d8bab07, 93 },
    { 0xd84bab08, 91 },
    { 0x6efbab09, 61 },
    { 0, 0 },
    { 0xeffbab0b, 90 },
    { 0, 0 },
    { 0x467bab0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bab12, 95 },
    { 0xb5dbab13, 33 },
    { 0x76fbab14, 33 },
    { 0xd62bab13, 84 },
    { 0x573bab16, 76 },
    { 0, 0 },
    { 0xa78bab18, 99 },
    { 0, 0 },
    { 0x4debab1a, 33 },
    { 0, 0 },
    { 0x3dbab1c, 55 },
    { 0x56fbab1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864bab21, 81 },
    { 0x70bbab21, 75 },
    { 0x66cbab23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bab29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bab2c, 90 },
    { 0x178bab2d, 85 },
    { 0, 0 },
    { 0x615bab2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917bab34, 46 },
    { 0x998bab35, 61 },
    { 0xadabab35, 90 },
    { 0xa3dbab35, 99 },
    { 0xd4bab36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ebab3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddbab42, 76 },
    { 0, 0 },
    { 0x7e3bab44, 23 },
    { 0, 0 },
    { 0x526bab46, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1bab4b, 90 },
    { 0xbe6bab4b, 98 },
    { 0, 0 },
    { 0xac1bab4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ebab52, 23 },
    { 0x563bab53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bab58, 99 },
    { 0x621bab59, 98 },
    { 0, 0 },
    { 0x19bab5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebab5f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14bab65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x501bab68, 99 },
    { 0xc9cbab69, 61 },
    { 0x340bab69, 61 },
    { 0xd66bab6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464bab74, 98 },
    { 0x508bab75, 95 },
    { 0x7adbab74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfbab7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x155bab80, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6bab85, 95 },
    { 0x195bab85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadbab89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678bab96, 93 },
    { 0x657bab97, 100 },
    { 0x116bab98, 98 },
    { 0xdccbab99, 33 },
    { 0x4bdbab98, 23 },
    { 0x430bab96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x821bab9e, 43 },
    { 0x6afbab9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17baba4, 23 },
    { 0xff9baba5, 90 },
    { 0x107baba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fbaba9, 95 },
    { 0, 0 },
    { 0xe35babab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58babaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39babb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x498babb6, 55 },
    { 0, 0 },
    { 0xed7babb8, 23 },
    { 0xe76babb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0babbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91babc1, 95 },
    { 0, 0 },
    { 0xdbbbabc3, 91 },
    { 0x754babc4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aababc7, 76 },
    { 0x112babc8, 84 },
    { 0x3cfbabc8, 98 },
    { 0x856babc9, 33 },
    { 0, 0 },
    { 0x7f3babcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaababd0, 34 },
    { 0, 0 },
    { 0x72ebabd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cbabd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07babd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fbabde, 97 },
    { 0x108babde, 90 },
    { 0xe5babe0, 55 },
    { 0xf82babe1, 84 },
    { 0x8e2babe2, 76 },
    { 0, 0 },
    { 0xf4dbabe4, 33 },
    { 0x266babe5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610babe9, 78 },
    { 0x664babea, 23 },
    { 0xc7ababeb, 33 },
    { 0x39dbabec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaababf0, 90 },
    { 0xa8bbabf0, 36 },
    { 0x1ebbabf0, 33 },
    { 0x92bbabf2, 98 },
    { 0, 0 },
    { 0x2babf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dbabf8, 55 },
    { 0xb89babf9, 90 },
    { 0x6f1babf9, 61 },
    { 0x490babfb, 90 },
    { 0xaadbabfc, 46 },
    { 0xd27babfd, 61 },
    { 0x887babfd, 90 },
    { 0xa28babfb, 61 },
    { 0x243babfe, 90 },
    { 0xe23bac01, 33 },
    { 0x46fbac01, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdbac07, 100 },
    { 0x245bac08, 33 },
    { 0x8bdbac08, 76 },
    { 0, 0 },
    { 0x404bac0b, 52 },
    { 0x8b0bac0c, 23 },
    { 0xfa8bac0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bac11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fbac15, 76 },
    { 0x3ebac15, 87 },
    { 0xd45bac17, 98 },
    { 0x422bac18, 33 },
    { 0x32bbac15, 55 },
    { 0, 0 },
    { 0x439bac1b, 55 },
    { 0xc9bac1b, 61 },
    { 0x635bac1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4bac21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bac27, 56 },
    { 0xa0bac28, 23 },
    { 0, 0 },
    { 0xf4ebac2a, 68 },
    { 0x1f5bac2a, 90 },
    { 0x9f3bac2a, 61 },
    { 0x1d1bac2a, 94 },
    { 0xbebac2a, 61 },
    { 0xfebac2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dbac33, 61 },
    { 0x394bac33, 33 },
    { 0x958bac34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8bac38, 23 },
    { 0x42dbac38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8bac3c, 52 },
    { 0, 0 },
    { 0x1afbac3e, 61 },
    { 0x66bac3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9bac44, 46 },
    { 0xc28bac45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3bac4a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3bac50, 47 },
    { 0xbbac51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0bac57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebac60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bac65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44bac68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bac6e, 33 },
    { 0, 0 },
    { 0xdaabac70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabac77, 31 },
    { 0, 0 },
    { 0xb54bac79, 90 },
    { 0xd60bac79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x741bac7d, 61 },
    { 0xdf4bac7d, 61 },
    { 0x4fbac7f, 23 },
    { 0xf2dbac80, 52 },
    { 0, 0 },
    { 0x48dbac82, 23 },
    { 0xda8bac82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507bac88, 43 },
    { 0xf1abac89, 76 },
    { 0x344bac8a, 61 },
    { 0, 0 },
    { 0xebbbac8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38bac90, 98 },
    { 0x4dbbac91, 58 },
    { 0x85dbac91, 23 },
    { 0x693bac92, 55 },
    { 0, 0 },
    { 0xb9fbac95, 33 },
    { 0xd55bac95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebac9c, 98 },
    { 0xdf1bac9d, 98 },
    { 0x8bdbac9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68baca3, 52 },
    { 0xcc2baca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44baca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3bacab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37bacae, 96 },
    { 0x954bacaf, 17 },
    { 0xf3dbacaf, 33 },
    { 0x498bacb0, 93 },
    { 0x1a7bacb2, 52 },
    { 0x219bacb3, 99 },
    { 0, 0 },
    { 0x872bacb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10bacbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbacc3, 85 },
    { 0x651bacc4, 34 },
    { 0xe3bacc5, 61 },
    { 0x85bacc5, 33 },
    { 0x596bacc5, 88 },
    { 0, 0 },
    { 0x83bacc9, 23 },
    { 0xff0bacc9, 93 },
    { 0xc33bacc9, 33 },
    { 0xa68baccc, 34 },
    { 0, 0 },
    { 0xb26bacce, 33 },
    { 0x42abaccf, 84 },
    { 0xa80baccf, 61 },
    { 0x439bacd0, 90 },
    { 0x8d7bacd2, 61 },
    { 0x9f4bacd1, 23 },
    { 0x49bacd2, 43 },
    { 0, 0 },
    { 0xcbbacd6, 90 },
    { 0x4f6bacd6, 43 },
    { 0x1dabacd8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ebacdd, 23 },
    { 0x6aebacdd, 23 },
    { 0x346bacde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x139bace2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0bace7, 17 },
    { 0xc17bace7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dbaceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fbacef, 61 },
    { 0x16bbacf0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25bacf3, 55 },
    { 0x69dbacf4, 95 },
    { 0xe2cbacf5, 76 },
    { 0, 0 },
    { 0x84abacf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabacfa, 78 },
    { 0xfe1bacfb, 90 },
    { 0x2d4bacfb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bad02, 33 },
    { 0x3b4bad03, 55 },
    { 0, 0 },
    { 0x464bad05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbad0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408bad10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0bad14, 43 },
    { 0xbbcbad15, 61 },
    { 0, 0 },
    { 0x9d0bad17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbad1d, 95 },
    { 0, 0 },
    { 0xbc6bad1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3bad22, 61 },
    { 0xf8ebad22, 61 },
    { 0x93cbad24, 84 },
    { 0, 0 },
    { 0x798bad26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x148bad29, 38 },
    { 0x1d9bad2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5bad2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0bad32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebad36, 75 },
    { 0xbcdbad36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74bad3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fbad42, 95 },
    { 0x239bad42, 61 },
    { 0xe81bad43, 98 },
    { 0xa69bad44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cbad48, 98 },
    { 0, 0 },
    { 0x5b2bad4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4bad4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cbad58, 76 },
    { 0xd9ebad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43bad5f, 93 },
    { 0xba6bad5f, 61 },
    { 0xd91bad5f, 52 },
    { 0, 0 },
    { 0xe6bbad63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dbad67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bad6a, 61 },
    { 0, 0 },
    { 0x529bad6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbad6f, 61 },
    { 0x5febad70, 43 },
    { 0x890bad71, 78 },
    { 0x511bad71, 99 },
    { 0xfb6bad73, 40 },
    { 0x955bad71, 87 },
    { 0x422bad75, 100 },
    { 0xe50bad76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ebad7b, 78 },
    { 0x53ebad7c, 67 },
    { 0xccbbad7b, 46 },
    { 0xf34bad7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8bad82, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bad85, 33 },
    { 0x9a4bad86, 53 },
    { 0xd35bad87, 33 },
    { 0x4fcbad88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ebad8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866bad93, 98 },
    { 0, 0 },
    { 0xdd9bad95, 95 },
    { 0xea6bad95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63abad99, 72 },
    { 0, 0 },
    { 0x208bad9b, 90 },
    { 0x475bad9c, 91 },
    { 0xa3bbad9d, 100 },
    { 0xce7bad9e, 81 },
    { 0xe51bad9e, 91 },
    { 0x688bad9e, 76 },
    { 0xa88bada1, 90 },
    { 0xc67bada2, 90 },
    { 0x118bada1, 33 },
    { 0xe96bada4, 33 },
    { 0xb93bada5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3badab, 90 },
    { 0x951badac, 77 },
    { 0xed6badad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbbadb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbadbd, 92 },
    { 0xb2dbadbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9abadc7, 78 },
    { 0x59fbadc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ebadcd, 93 },
    { 0xcf7badcd, 90 },
    { 0x545badcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fbadd2, 93 },
    { 0x2c7badd3, 95 },
    { 0x705badd4, 98 },
    { 0, 0 },
    { 0xd79badd6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8baddf, 88 },
    { 0, 0 },
    { 0x2cabade1, 33 },
    { 0, 0 },
    { 0x8a2bade3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5bade6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55badf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843badfe, 33 },
    { 0, 0 },
    { 0x476bae00, 90 },
    { 0x11dbae01, 84 },
    { 0x714bae00, 33 },
    { 0xdaebae01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ebae09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ebae0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabae12, 95 },
    { 0, 0 },
    { 0x187bae14, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6bae1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbae1d, 90 },
    { 0x9b7bae1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03bae21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735bae25, 55 },
    { 0, 0 },
    { 0xbd8bae27, 98 },
    { 0x66cbae28, 43 },
    { 0xe48bae29, 90 },
    { 0, 0 },
    { 0xe37bae2b, 33 },
    { 0xaf3bae2b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bae33, 34 },
    { 0xdecbae33, 33 },
    { 0xed2bae35, 98 },
    { 0x9bbbae35, 61 },
    { 0x21bbae34, 98 },
    { 0, 0 },
    { 0xa34bae39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ebae3d, 90 },
    { 0, 0 },
    { 0x255bae3f, 98 },
    { 0x69bbae40, 95 },
    { 0xf72bae41, 95 },
    { 0x682bae41, 23 },
    { 0x457bae42, 100 },
    { 0x687bae44, 23 },
    { 0x47bbae43, 95 },
    { 0x9adbae46, 98 },
    { 0x4acbae47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cbae4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82bae4f, 61 },
    { 0xa1ebae50, 89 },
    { 0, 0 },
    { 0xb16bae52, 98 },
    { 0, 0 },
    { 0xf11bae54, 90 },
    { 0, 0 },
    { 0xf77bae56, 90 },
    { 0, 0 },
    { 0x8e9bae58, 93 },
    { 0xfc8bae59, 23 },
    { 0x441bae59, 61 },
    { 0, 0 },
    { 0x43dbae5c, 17 },
    { 0x69ebae5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57abae61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754bae6a, 98 },
    { 0, 0 },
    { 0x844bae6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808bae77, 99 },
    { 0xa2abae78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bbae7b, 90 },
    { 0xab6bae7c, 76 },
    { 0, 0 },
    { 0x659bae7e, 34 },
    { 0x92bbae7f, 93 },
    { 0, 0 },
    { 0x154bae81, 33 },
    { 0xdfabae81, 90 },
    { 0x809bae83, 33 },
    { 0xd5bae83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cbae89, 23 },
    { 0x1adbae89, 98 },
    { 0x408bae8b, 77 },
    { 0x77abae8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16abae90, 61 },
    { 0, 0 },
    { 0xfdebae92, 88 },
    { 0, 0 },
    { 0x2ebbae94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bbae9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4baea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ebaea9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854baeaf, 43 },
    { 0x6d9baeb0, 23 },
    { 0x6c6baeb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbaeb6, 100 },
    { 0xf54baeb7, 84 },
    { 0xeedbaeb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998baebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdbaec8, 99 },
    { 0x6a6baec9, 98 },
    { 0, 0 },
    { 0x9aebaecb, 81 },
    { 0xc06baecb, 90 },
    { 0, 0 },
    { 0x2ccbaece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbaed3, 61 },
    { 0x715baed3, 66 },
    { 0x30dbaed5, 66 },
    { 0x192baed6, 98 },
    { 0, 0 },
    { 0x959baed8, 46 },
    { 0x462baed8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x999baedc, 95 },
    { 0, 0 },
    { 0x761baede, 78 },
    { 0, 0 },
    { 0x4c6baee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4babaee6, 95 },
    { 0x3a4baee6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37baef4, 90 },
    { 0x42fbaef5, 98 },
    { 0x598baef6, 90 },
    { 0x305baef5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eabaefb, 23 },
    { 0x88bbaefb, 95 },
    { 0xf7abaefd, 55 },
    { 0x225baefd, 66 },
    { 0xbc7baefb, 95 },
    { 0x13bbaefb, 95 },
    { 0x554baf00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77baf07, 90 },
    { 0, 0 },
    { 0xb72baf09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fbaf0e, 90 },
    { 0xfd8baf0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbaf12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868baf17, 78 },
    { 0x66dbaf18, 98 },
    { 0xb69baf17, 67 },
    { 0xa04baf18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x729baf1d, 33 },
    { 0, 0 },
    { 0xc3bbaf1f, 33 },
    { 0x4a9baf20, 34 },
    { 0x248baf20, 23 },
    { 0xfd1baf22, 98 },
    { 0xc7cbaf21, 52 },
    { 0, 0 },
    { 0x86abaf25, 33 },
    { 0, 0 },
    { 0x893baf27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38baf2b, 76 },
    { 0, 0 },
    { 0x1dabaf2d, 78 },
    { 0, 0 },
    { 0x2a8baf2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81abaf33, 95 },
    { 0, 0 },
    { 0xf22baf35, 61 },
    { 0x45abaf36, 90 },
    { 0x1d4baf36, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2baf3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995baf40, 90 },
    { 0, 0 },
    { 0xbf3baf42, 33 },
    { 0xbb2baf43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x949baf46, 99 },
    { 0x22abaf47, 61 },
    { 0, 0 },
    { 0x8d3baf49, 76 },
    { 0, 0 },
    { 0xa10baf4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78abaf4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x256baf52, 33 },
    { 0x1c7baf52, 23 },
    { 0, 0 },
    { 0x97ebaf55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dbaf5a, 98 },
    { 0, 0 },
    { 0xf47baf5c, 90 },
    { 0xd2fbaf5d, 34 },
    { 0xa4abaf5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679baf63, 98 },
    { 0x2c4baf64, 61 },
    { 0x1cebaf63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x964baf68, 81 },
    { 0x86fbaf69, 95 },
    { 0x8b3baf69, 61 },
    { 0xd52baf6b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dbaf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82abaf72, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dbaf76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abaf79, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473baf7e, 46 },
    { 0xaf2baf7f, 66 },
    { 0x30baf80, 66 },
    { 0, 0 },
    { 0xea3baf82, 55 },
    { 0, 0 },
    { 0x157baf84, 34 },
    { 0, 0 },
    { 0x96abaf86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ebaf89, 66 },
    { 0xa01baf89, 91 },
    { 0xe81baf8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30baf96, 90 },
    { 0x10bbaf97, 87 },
    { 0xc61baf97, 95 },
    { 0x62bbaf99, 90 },
    { 0x972baf99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353baf9e, 40 },
    { 0xa22baf9f, 65 },
    { 0x3acbaf9e, 55 },
    { 0xc5cbafa1, 61 },
    { 0xda8bafa2, 76 },
    { 0xba2bafa2, 23 },
    { 0x314bafa4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79bafa9, 98 },
    { 0x49bbafaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebafad, 90 },
    { 0, 0 },
    { 0x9febafaf, 99 },
    { 0x5d5bafb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684bafb9, 76 },
    { 0xfbebafb9, 55 },
    { 0, 0 },
    { 0xd55bafbc, 87 },
    { 0x9fdbafbc, 90 },
    { 0x4bdbafbe, 90 },
    { 0, 0 },
    { 0x55ebafc0, 88 },
    { 0xb06bafc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbafc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbafcb, 98 },
    { 0x2e2bafcb, 23 },
    { 0, 0 },
    { 0xed2bafce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59bafd9, 99 },
    { 0xc30bafd9, 95 },
    { 0x938bafdb, 84 },
    { 0, 0 },
    { 0x8b5bafdd, 23 },
    { 0xf8ebafde, 61 },
    { 0x9b8bafdd, 76 },
    { 0x268bafde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bafe3, 34 },
    { 0, 0 },
    { 0xb48bafe5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbbafeb, 76 },
    { 0x99fbafeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8baff0, 84 },
    { 0x43dbaff1, 67 },
    { 0x2bdbaff2, 52 },
    { 0x1bdbaff3, 55 },
    { 0x5f2baff0, 90 },
    { 0, 0 },
    { 0xc25baff6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dbaff9, 99 },
    { 0x6babaffa, 61 },
    { 0, 0 },
    { 0x2ecbaffc, 23 },
    { 0xe53baffd, 61 },
    { 0xe12baffe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcbb006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67bb009, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0bb00e, 66 },
    { 0x654bb00e, 33 },
    { 0xae5bb00e, 90 },
    { 0x7c7bb011, 99 },
    { 0x1bb00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83bb01a, 43 },
    { 0x1ddbb01b, 90 },
    { 0, 0 },
    { 0xcebbb01d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbb020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587bb025, 90 },
    { 0x46bbb026, 66 },
    { 0xa5bbb026, 93 },
    { 0x30bb025, 61 },
    { 0, 0 },
    { 0x10bbb02a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x649bb02d, 66 },
    { 0xc42bb02d, 34 },
    { 0x825bb02e, 80 },
    { 0, 0 },
    { 0x765bb031, 78 },
    { 0x81dbb031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eebb039, 33 },
    { 0x4d9bb03a, 55 },
    { 0x463bb03a, 66 },
    { 0xca2bb039, 40 },
    { 0, 0 },
    { 0xa43bb03e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77bb047, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9bb04c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4bb051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88bb055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1bb059, 83 },
    { 0x176bb05a, 90 },
    { 0x833bb05b, 33 },
    { 0x59abb05b, 95 },
    { 0xfcbb05b, 95 },
    { 0x28fbb05e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bb061, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffbb06b, 34 },
    { 0xf0cbb06c, 23 },
    { 0x107bb06c, 99 },
    { 0x8abbb06d, 92 },
    { 0x9bcbb06f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bb073, 87 },
    { 0xd9dbb074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x764bb077, 83 },
    { 0, 0 },
    { 0x85ebb079, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efbb08b, 75 },
    { 0x301bb08c, 98 },
    { 0x881bb08c, 34 },
    { 0x868bb08e, 55 },
    { 0xde1bb08d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ebb094, 95 },
    { 0xc73bb095, 33 },
    { 0xe57bb095, 90 },
    { 0xfdfbb096, 98 },
    { 0x470bb098, 84 },
    { 0xa6ebb099, 61 },
    { 0x9aabb099, 33 },
    { 0x8abbb098, 90 },
    { 0xc94bb09c, 43 },
    { 0, 0 },
    { 0x740bb09e, 34 },
    { 0, 0 },
    { 0xe78bb0a0, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbbb0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21bb0ab, 61 },
    { 0x5e8bb0ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbb0b4, 78 },
    { 0xf98bb0b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6bb0be, 98 },
    { 0, 0 },
    { 0xc01bb0c0, 87 },
    { 0xcb6bb0c1, 52 },
    { 0, 0 },
    { 0xdddbb0c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62abb0c7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acbb0ca, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860bb0ce, 98 },
    { 0, 0 },
    { 0xef0bb0d0, 90 },
    { 0xdc0bb0d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7bb0d4, 75 },
    { 0, 0 },
    { 0x286bb0d6, 46 },
    { 0, 0 },
    { 0x392bb0d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211bb0dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632bb0e2, 61 },
    { 0, 0 },
    { 0x709bb0e4, 95 },
    { 0, 0 },
    { 0xc21bb0e6, 98 },
    { 0x442bb0e7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2bb0ef, 34 },
    { 0x9dbbb0ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3abb0f3, 91 },
    { 0x98abb0f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbb0fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bbb102, 65 },
    { 0x863bb103, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bb107, 76 },
    { 0x69ebb108, 36 },
    { 0x3d0bb109, 46 },
    { 0xc3fbb10a, 33 },
    { 0x555bb10a, 66 },
    { 0x86abb109, 98 },
    { 0x547bb108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bebb112, 95 },
    { 0, 0 },
    { 0x2e7bb114, 98 },
    { 0xb85bb114, 84 },
    { 0, 0 },
    { 0x6c6bb117, 55 },
    { 0x19bb118, 93 },
    { 0xde2bb119, 95 },
    { 0xd18bb11a, 34 },
    { 0xc10bb118, 90 },
    { 0xe46bb11c, 43 },
    { 0x4bcbb11d, 90 },
    { 0x48ebb11e, 33 },
    { 0, 0 },
    { 0xc74bb120, 33 },
    { 0, 0 },
    { 0x97bbb122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5bb126, 23 },
    { 0x124bb127, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63bb12d, 34 },
    { 0, 0 },
    { 0x97fbb12f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204bb134, 61 },
    { 0x80fbb135, 33 },
    { 0x74cbb135, 95 },
    { 0xba6bb134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99bb13a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cbb13d, 61 },
    { 0, 0 },
    { 0x561bb13f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdbb142, 33 },
    { 0, 0 },
    { 0x7b7bb144, 43 },
    { 0, 0 },
    { 0x8a0bb146, 34 },
    { 0, 0 },
    { 0x25cbb148, 81 },
    { 0x72ebb149, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbbb14c, 33 },
    { 0x33bb14d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7bb150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5bb153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fbb158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93abb15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcbb161, 47 },
    { 0x4eebb162, 98 },
    { 0xb9dbb163, 98 },
    { 0, 0 },
    { 0x4f8bb165, 33 },
    { 0x29fbb166, 33 },
    { 0x3d2bb167, 95 },
    { 0, 0 },
    { 0x34bb169, 98 },
    { 0x59ebb16a, 95 },
    { 0x2dbb16a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bb16f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270bb172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bb17c, 76 },
    { 0, 0 },
    { 0xaafbb17e, 81 },
    { 0x737bb17f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5bb182, 87 },
    { 0, 0 },
    { 0xd4dbb184, 95 },
    { 0xa2cbb185, 95 },
    { 0xc44bb186, 23 },
    { 0, 0 },
    { 0xe50bb188, 76 },
    { 0, 0 },
    { 0xd87bb18a, 90 },
    { 0x472bb18b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x120bb18e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x978bb191, 46 },
    { 0x766bb191, 90 },
    { 0, 0 },
    { 0x98bb194, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827bb199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83bb19f, 100 },
    { 0, 0 },
    { 0x8abbb1a1, 98 },
    { 0xe23bb1a2, 87 },
    { 0x53abb1a3, 95 },
    { 0xc5bbb1a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87abb1a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bb1ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efbb1b2, 66 },
    { 0xfe1bb1b2, 84 },
    { 0x401bb1b4, 90 },
    { 0x99bbb1b5, 100 },
    { 0, 0 },
    { 0x799bb1b7, 61 },
    { 0x17bb1b7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9bb1c0, 55 },
    { 0x5fcbb1c1, 77 },
    { 0, 0 },
    { 0xf32bb1c3, 90 },
    { 0x8dbb1c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbb1c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5bb1ca, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400bb1d0, 90 },
    { 0xe5bbb1d1, 76 },
    { 0x445bb1d1, 95 },
    { 0, 0 },
    { 0x2c6bb1d4, 98 },
    { 0, 0 },
    { 0x354bb1d6, 95 },
    { 0x2e0bb1d7, 33 },
    { 0x6abbb1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bb1e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xccabb1e3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1bb1e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x457bb1ec, 55 },
    { 0x56cbb1ed, 46 },
    { 0xc9dbb1ee, 91 },
    { 0x73ebb1ef, 67 },
    { 0xde6bb1f0, 65 },
    { 0, 0 },
    { 0xb9bb1f2, 90 },
    { 0, 0 },
    { 0x716bb1f4, 87 },
    { 0x38bbb1f5, 101 },
    { 0x359bb1f5, 97 },
    { 0x44bbb1f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dbb1fa, 23 },
    { 0x6abbb1fa, 33 },
    { 0, 0 },
    { 0x9b9bb1fd, 33 },
    { 0, 0 },
    { 0xa7ebb1ff, 98 },
    { 0, 0 },
    { 0xd69bb201, 61 },
    { 0xeebb202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476bb20b, 23 },
    { 0x598bb20b, 98 },
    { 0x31fbb20d, 61 },
    { 0xd32bb20e, 76 },
    { 0x213bb20f, 55 },
    { 0x1f5bb210, 78 },
    { 0x65fbb210, 61 },
    { 0x93ebb211, 33 },
    { 0, 0 },
    { 0x64bbb214, 76 },
    { 0x87cbb214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810bb219, 76 },
    { 0x864bb21a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cbb21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbb224, 61 },
    { 0x3bebb224, 98 },
    { 0, 0 },
    { 0xd0bbb227, 95 },
    { 0xa98bb228, 52 },
    { 0xc8ebb227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ebb22d, 93 },
    { 0x8a6bb22e, 23 },
    { 0x1b0bb22f, 99 },
    { 0x47fbb230, 23 },
    { 0x16fbb231, 43 },
    { 0x28cbb22f, 99 },
    { 0xbe6bb231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddabb239, 23 },
    { 0x746bb23a, 23 },
    { 0xb45bb23b, 36 },
    { 0x25fbb23c, 76 },
    { 0, 0 },
    { 0x6adbb23e, 46 },
    { 0x7a5bb23f, 76 },
    { 0x84bb240, 77 },
    { 0xc51bb241, 76 },
    { 0, 0 },
    { 0x9c2bb243, 94 },
    { 0x6bcbb244, 100 },
    { 0, 0 },
    { 0x9afbb246, 40 },
    { 0x7c1bb246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dbb24a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cbb250, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bb253, 90 },
    { 0x768bb254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebb257, 91 },
    { 0xebebb258, 44 },
    { 0x719bb257, 43 },
    { 0xdd2bb257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729bb25e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8bb261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bb265, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41bb268, 55 },
    { 0, 0 },
    { 0x51bbb26a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbb270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634bb274, 98 },
    { 0, 0 },
    { 0x92cbb276, 33 },
    { 0x9abbb277, 93 },
    { 0x318bb277, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbb27b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506bb281, 23 },
    { 0xeb1bb282, 95 },
    { 0, 0 },
    { 0x2bb284, 43 },
    { 0x148bb285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370bb289, 98 },
    { 0x4bb28a, 52 },
    { 0xaebbb28b, 98 },
    { 0xadabb28b, 98 },
    { 0x8f6bb28d, 23 },
    { 0, 0 },
    { 0xaa0bb28f, 34 },
    { 0x1f3bb290, 91 },
    { 0x17fbb291, 98 },
    { 0x41bb292, 98 },
    { 0x97ebb293, 23 },
    { 0, 0 },
    { 0xc46bb295, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147bb29c, 78 },
    { 0x820bb29d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2abb2a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19bb2aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb2af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9bb2b7, 98 },
    { 0x11bb2b7, 34 },
    { 0x3f1bb2b9, 67 },
    { 0x1acbb2ba, 87 },
    { 0x76cbb2bb, 61 },
    { 0x2adbb2bb, 23 },
    { 0xe3bbb2bd, 61 },
    { 0xcc4bb2bc, 34 },
    { 0x9b4bb2bf, 98 },
    { 0x68fbb2bb, 100 },
    { 0x58bb2c1, 61 },
    { 0x253bb2c2, 52 },
    { 0xba3bb2bf, 99 },
    { 0x1bbbb2c0, 99 },
    { 0, 0 },
    { 0x388bb2c6, 90 },
    { 0x345bb2c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfbb2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbb2cd, 84 },
    { 0x317bb2cd, 98 },
    { 0xd35bb2cf, 90 },
    { 0xf75bb2cf, 23 },
    { 0, 0 },
    { 0x761bb2d2, 34 },
    { 0x3cdbb2d3, 66 },
    { 0x4e8bb2d4, 91 },
    { 0xcdebb2d3, 95 },
    { 0xb52bb2d6, 66 },
    { 0, 0 },
    { 0xbcdbb2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x743bb2db, 90 },
    { 0xec5bb2db, 99 },
    { 0xfc4bb2db, 2 },
    { 0xf29bb2de, 33 },
    { 0xe94bb2df, 61 },
    { 0x90ebb2de, 76 },
    { 0xecfbb2de, 33 },
    { 0, 0 },
    { 0xc2bbb2e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bb2e7, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0bb2f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bbb2f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bb2f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bb2fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbbb2fd, 85 },
    { 0xfcbb2fe, 85 },
    { 0xd31bb2ff, 90 },
    { 0x1d1bb2ff, 33 },
    { 0xd95bb301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abb304, 100 },
    { 0x780bb304, 33 },
    { 0xf40bb306, 55 },
    { 0xc2fbb307, 61 },
    { 0xb40bb307, 55 },
    { 0, 0 },
    { 0xfaebb30a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254bb30f, 95 },
    { 0xf9ebb310, 90 },
    { 0xab0bb311, 67 },
    { 0x6eebb312, 90 },
    { 0x881bb311, 71 },
    { 0x640bb312, 85 },
    { 0x7b0bb315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bb31d, 98 },
    { 0x212bb31e, 33 },
    { 0, 0 },
    { 0x89cbb320, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19bb324, 72 },
    { 0x238bb324, 98 },
    { 0, 0 },
    { 0xbcbb327, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x501bb32a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbb32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2bb331, 90 },
    { 0, 0 },
    { 0xd3ebb333, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebb336, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ebb33d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734bb347, 67 },
    { 0, 0 },
    { 0xcebbb349, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x693bb34c, 76 },
    { 0x995bb34d, 33 },
    { 0x40bbb34e, 61 },
    { 0xa84bb34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38bb354, 85 },
    { 0, 0 },
    { 0x585bb356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bb359, 95 },
    { 0x474bb35a, 97 },
    { 0x3f3bb35b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bb362, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5bb365, 91 },
    { 0x146bb366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bb369, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bb36d, 76 },
    { 0, 0 },
    { 0x73dbb36f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fbb378, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bb37e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15bb381, 23 },
    { 0, 0 },
    { 0x855bb383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13bb387, 33 },
    { 0xff3bb388, 33 },
    { 0xf33bb388, 90 },
    { 0x8edbb387, 90 },
    { 0xfb5bb387, 55 },
    { 0, 0 },
    { 0x66fbb38d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbb390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2bb398, 61 },
    { 0x974bb399, 90 },
    { 0xeb7bb398, 95 },
    { 0xe6fbb399, 76 },
    { 0x5e7bb39c, 99 },
    { 0x6a0bb39d, 66 },
    { 0xcabb39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbb3a5, 23 },
    { 0x25bb3a6, 98 },
    { 0xe2abb3a5, 33 },
    { 0xae1bb3a6, 55 },
    { 0x3c2bb3a8, 91 },
    { 0x805bb3aa, 23 },
    { 0x807bb3a7, 90 },
    { 0xcccbb3ab, 64 },
    { 0x121bb3ad, 61 },
    { 0xf27bb3ad, 23 },
    { 0x41dbb3ad, 87 },
    { 0x76bb3b0, 61 },
    { 0xc41bb3ad, 95 },
    { 0x7bbb3b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80bb3b7, 95 },
    { 0, 0 },
    { 0x24bb3b9, 76 },
    { 0xe69bb3ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dbb3bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28bb3c7, 34 },
    { 0x791bb3c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x906bb3cb, 55 },
    { 0x870bb3cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0bb3d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724bb3d4, 61 },
    { 0, 0 },
    { 0xd7ebb3d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebb3db, 46 },
    { 0, 0 },
    { 0x8cbbb3dd, 76 },
    { 0, 0 },
    { 0xc89bb3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60bb3e3, 55 },
    { 0x484bb3e4, 43 },
    { 0xb40bb3e5, 61 },
    { 0x7bbbb3e3, 33 },
    { 0x6f4bb3e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43bb3eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ebb3ee, 76 },
    { 0x506bb3ef, 99 },
    { 0x1bdbb3ee, 55 },
    { 0x922bb3f1, 94 },
    { 0x1e4bb3f2, 23 },
    { 0x1f0bb3f1, 61 },
    { 0x127bb3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295bb3fd, 90 },
    { 0x84ebb3fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfbb401, 90 },
    { 0x9bb401, 34 },
    { 0, 0 },
    { 0x20bb404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bb407, 98 },
    { 0xd8bb407, 33 },
    { 0xeebb409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09bb40c, 90 },
    { 0, 0 },
    { 0xbf3bb40e, 34 },
    { 0x69bb40f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368bb414, 34 },
    { 0x534bb414, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69bb418, 68 },
    { 0x338bb419, 90 },
    { 0, 0 },
    { 0x80cbb41b, 98 },
    { 0x12fbb41c, 76 },
    { 0x49abb41d, 101 },
    { 0, 0 },
    { 0xe87bb41f, 55 },
    { 0, 0 },
    { 0x43abb421, 100 },
    { 0xf9bb422, 90 },
    { 0x23dbb422, 52 },
    { 0x957bb421, 34 },
    { 0x5ffbb425, 23 },
    { 0xe00bb424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68abb431, 100 },
    { 0x8f4bb432, 91 },
    { 0x6e1bb433, 81 },
    { 0xaa0bb434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8bb43b, 64 },
    { 0, 0 },
    { 0x86fbb43d, 33 },
    { 0x74bb43e, 33 },
    { 0x8bbb43e, 78 },
    { 0x188bb43f, 98 },
    { 0xd1dbb440, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x289bb444, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fbb44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x224bb44e, 61 },
    { 0xa4abb44e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbb457, 55 },
    { 0x19bbb457, 98 },
    { 0xd6ebb459, 55 },
    { 0x6a4bb458, 95 },
    { 0, 0 },
    { 0x28abb45c, 33 },
    { 0x127bb45d, 76 },
    { 0x9e2bb45d, 46 },
    { 0xef9bb45e, 43 },
    { 0, 0 },
    { 0xa5bb461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ebb468, 34 },
    { 0x293bb469, 43 },
    { 0, 0 },
    { 0x85fbb46b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bb472, 90 },
    { 0, 0 },
    { 0x10fbb474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb47c, 43 },
    { 0x279bb47d, 33 },
    { 0x19bb47e, 76 },
    { 0xd63bb47f, 61 },
    { 0x165bb480, 93 },
    { 0x21bb47d, 76 },
    { 0x3f7bb47f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443bb488, 90 },
    { 0x4d8bb489, 98 },
    { 0x9b6bb48a, 43 },
    { 0x9dfbb489, 43 },
    { 0x26ebb489, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x612bb48f, 34 },
    { 0x7c0bb48f, 61 },
    { 0, 0 },
    { 0x795bb492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb495, 78 },
    { 0, 0 },
    { 0xd60bb497, 98 },
    { 0x1eebb497, 90 },
    { 0x4eebb498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbb49e, 46 },
    { 0xf94bb49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbb4a2, 33 },
    { 0x74ebb4a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62bb4ab, 34 },
    { 0xfddbb4ac, 85 },
    { 0xb59bb4ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305bb4b4, 98 },
    { 0, 0 },
    { 0x127bb4b6, 81 },
    { 0, 0 },
    { 0x38cbb4b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663bb4bd, 90 },
    { 0, 0 },
    { 0x2dbbb4bf, 23 },
    { 0x158bb4bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2bb4c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753bb4c7, 33 },
    { 0x36dbb4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8bb4cb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb4d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602bb4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cbb4da, 87 },
    { 0, 0 },
    { 0x5c1bb4dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9abb4df, 43 },
    { 0, 0 },
    { 0xd2bbb4e1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9bb4e8, 34 },
    { 0x3fabb4e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5bb4ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09bb4f3, 66 },
    { 0, 0 },
    { 0x418bb4f5, 43 },
    { 0, 0 },
    { 0x631bb4f7, 33 },
    { 0x1f6bb4f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580bb4fc, 33 },
    { 0xacebb4fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288bb509, 61 },
    { 0xd4bbb50a, 91 },
    { 0xaccbb50b, 48 },
    { 0x35bbb509, 98 },
    { 0, 0 },
    { 0x54dbb50e, 75 },
    { 0x7f7bb50f, 95 },
    { 0xc27bb510, 100 },
    { 0x392bb511, 76 },
    { 0, 0 },
    { 0xc50bb513, 76 },
    { 0x6f0bb513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfbb520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bbb523, 17 },
    { 0, 0 },
    { 0xe74bb525, 78 },
    { 0x17abb526, 95 },
    { 0, 0 },
    { 0xcfbbb528, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x878bb52b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212bb530, 61 },
    { 0x7c1bb530, 23 },
    { 0x457bb530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4bb535, 95 },
    { 0, 0 },
    { 0x29cbb537, 95 },
    { 0xcabb538, 61 },
    { 0x87fbb539, 90 },
    { 0x3acbb53a, 34 },
    { 0, 0 },
    { 0xa68bb53c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2bb53f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ebb542, 61 },
    { 0x537bb542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dbb546, 33 },
    { 0, 0 },
    { 0x33abb548, 98 },
    { 0x466bb549, 95 },
    { 0, 0 },
    { 0x24bb54b, 93 },
    { 0x97ebb54b, 98 },
    { 0xc60bb54d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52bb552, 98 },
    { 0, 0 },
    { 0x286bb554, 33 },
    { 0x273bb554, 46 },
    { 0xa1abb556, 81 },
    { 0x7a9bb556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bb564, 61 },
    { 0xa21bb565, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x697bb568, 33 },
    { 0, 0 },
    { 0x153bb56a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb56d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dbb573, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5bb578, 69 },
    { 0xfeebb579, 90 },
    { 0xda8bb579, 43 },
    { 0x48bb57a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49bb584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eebb587, 33 },
    { 0x100bb588, 95 },
    { 0x3cbb589, 61 },
    { 0x628bb587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x713bb58d, 61 },
    { 0x37cbb58d, 95 },
    { 0xd79bb58e, 55 },
    { 0x701bb590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbbb594, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71bb598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bb5a3, 75 },
    { 0xf4abb5a4, 76 },
    { 0x99dbb5a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34abb5ab, 33 },
    { 0x937bb5ab, 90 },
    { 0x7acbb5ad, 23 },
    { 0x66bb5ab, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cbb5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98bb5b8, 88 },
    { 0x953bb5b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47bb5bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x191bb5bf, 61 },
    { 0x2b4bb5c0, 33 },
    { 0xf9abb5bf, 75 },
    { 0x911bb5c1, 76 },
    { 0xf32bb5c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bb5cd, 95 },
    { 0x9a4bb5ce, 76 },
    { 0xb8bbb5cf, 65 },
    { 0x90bb5ce, 76 },
    { 0xf01bb5d1, 55 },
    { 0xe42bb5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1bb5d7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0bb5e9, 95 },
    { 0xbfbb5ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727bb5ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74bb5f3, 68 },
    { 0xccbb5f3, 17 },
    { 0, 0 },
    { 0x505bb5f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6bb5f9, 23 },
    { 0x7d8bb5fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279bb601, 88 },
    { 0, 0 },
    { 0x5ffbb603, 33 },
    { 0x61bbb603, 61 },
    { 0x78cbb604, 66 },
    { 0x121bb606, 95 },
    { 0x43bb607, 61 },
    { 0xd77bb608, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbb614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760bb617, 90 },
    { 0x9b0bb617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7bb620, 23 },
    { 0, 0 },
    { 0x396bb622, 68 },
    { 0, 0 },
    { 0x29cbb624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bb629, 33 },
    { 0x9febb629, 98 },
    { 0xb25bb62a, 43 },
    { 0x1b0bb62c, 61 },
    { 0, 0 },
    { 0x960bb62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424bb636, 61 },
    { 0, 0 },
    { 0xc3fbb638, 98 },
    { 0x40abb639, 52 },
    { 0xb3fbb63a, 76 },
    { 0x18ebb639, 100 },
    { 0x6e5bb63c, 90 },
    { 0, 0 },
    { 0xedabb63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0bb641, 76 },
    { 0x70ebb641, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65bb645, 76 },
    { 0x467bb645, 34 },
    { 0, 0 },
    { 0x79abb648, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61bb64f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ebb652, 90 },
    { 0, 0 },
    { 0x7dabb654, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825bb65c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabb660, 77 },
    { 0, 0 },
    { 0xf43bb662, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8bb665, 17 },
    { 0x74fbb666, 61 },
    { 0x381bb666, 95 },
    { 0, 0 },
    { 0x491bb669, 33 },
    { 0xc0bb66a, 33 },
    { 0xa1dbb66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dabb66e, 61 },
    { 0xf95bb66f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6bb674, 23 },
    { 0xa53bb675, 33 },
    { 0x288bb675, 33 },
    { 0x3fbb677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebb67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2bb681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bb684, 84 },
    { 0, 0 },
    { 0x5b2bb686, 33 },
    { 0xa0dbb687, 26 },
    { 0x477bb687, 90 },
    { 0xe4cbb687, 90 },
    { 0xc9ebb687, 69 },
    { 0x200bb68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172bb692, 55 },
    { 0x453bb693, 33 },
    { 0, 0 },
    { 0x1edbb695, 46 },
    { 0x72fbb696, 90 },
    { 0x2cbbb697, 98 },
    { 0xd46bb697, 91 },
    { 0x16dbb697, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9bb69d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bb6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132bb6a6, 90 },
    { 0x2a7bb6a6, 33 },
    { 0x872bb6a7, 76 },
    { 0x76bbb6a9, 33 },
    { 0x2ebbb6aa, 95 },
    { 0x96fbb6aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34bb6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dbb6b6, 61 },
    { 0, 0 },
    { 0x28abb6b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb6be, 90 },
    { 0x4cabb6bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8febb6ca, 98 },
    { 0xf72bb6cb, 98 },
    { 0, 0 },
    { 0x893bb6cd, 75 },
    { 0x10dbb6ce, 61 },
    { 0x42fbb6cf, 81 },
    { 0x8e5bb6ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939bb6d5, 100 },
    { 0x306bb6d6, 46 },
    { 0, 0 },
    { 0xf1fbb6d8, 90 },
    { 0, 0 },
    { 0xd7dbb6da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0abb6df, 90 },
    { 0xf01bb6e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892bb6e5, 91 },
    { 0x1aebb6e6, 38 },
    { 0x872bb6e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2bb6ec, 90 },
    { 0x2c5bb6ed, 90 },
    { 0x44bbb6ed, 33 },
    { 0xd08bb6ef, 33 },
    { 0xcb5bb6ef, 43 },
    { 0xd4cbb6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4bb6f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bb6f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7bb6fb, 61 },
    { 0x5e1bb6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabb6ff, 34 },
    { 0x8a5bb700, 90 },
    { 0x76cbb701, 90 },
    { 0, 0 },
    { 0x48abb703, 99 },
    { 0xf8bb703, 81 },
    { 0x478bb705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6bb70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfbb718, 90 },
    { 0, 0 },
    { 0xd6ebb71a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bb71e, 43 },
    { 0x9ccbb71e, 90 },
    { 0x412bb71f, 90 },
    { 0xc4fbb721, 33 },
    { 0xeacbb722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeebb728, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38bb72c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0bb733, 46 },
    { 0x1e6bb734, 66 },
    { 0xa70bb735, 33 },
    { 0xd6bb735, 90 },
    { 0xfcfbb737, 98 },
    { 0x318bb738, 90 },
    { 0x787bb737, 46 },
    { 0x6d5bb73a, 100 },
    { 0, 0 },
    { 0x5e5bb73c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549bb740, 55 },
    { 0x313bb741, 68 },
    { 0x38dbb741, 98 },
    { 0xb95bb740, 40 },
    { 0x116bb744, 76 },
    { 0xb51bb742, 85 },
    { 0x21fbb746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9bb74c, 98 },
    { 0, 0 },
    { 0x3aebb74e, 90 },
    { 0, 0 },
    { 0xe18bb750, 52 },
    { 0xe56bb751, 84 },
    { 0, 0 },
    { 0xdd2bb753, 34 },
    { 0, 0 },
    { 0x556bb755, 23 },
    { 0x932bb755, 33 },
    { 0x4a1bb757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bb75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedbb764, 68 },
    { 0x13dbb764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62bb768, 84 },
    { 0x1fdbb769, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bb76c, 95 },
    { 0x358bb76c, 76 },
    { 0xe1ebb76e, 90 },
    { 0x7d4bb76f, 40 },
    { 0x408bb76f, 33 },
    { 0x229bb76c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x539bb774, 23 },
    { 0xb63bb775, 76 },
    { 0x66dbb775, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0bb77c, 90 },
    { 0x46dbb77c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dbb781, 55 },
    { 0x2cdbb781, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31bb786, 43 },
    { 0xd5ebb787, 90 },
    { 0, 0 },
    { 0x6dfbb789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6febb78c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecbb794, 98 },
    { 0x6c1bb795, 33 },
    { 0x9abbb795, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73bb799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5bb7a0, 34 },
    { 0x842bb7a1, 98 },
    { 0xd62bb7a2, 52 },
    { 0x11dbb7a2, 67 },
    { 0x815bb7a4, 43 },
    { 0x845bb7a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fabb7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdbb7af, 93 },
    { 0x99bbb7b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x639bb7b3, 90 },
    { 0xf2cbb7b3, 89 },
    { 0x445bb7b5, 99 },
    { 0xbd5bb7b6, 43 },
    { 0, 0 },
    { 0x690bb7b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fbb7bb, 61 },
    { 0x15bb7bc, 55 },
    { 0, 0 },
    { 0x559bb7be, 99 },
    { 0, 0 },
    { 0xafbbb7c0, 90 },
    { 0x825bb7c1, 99 },
    { 0xbcebb7c2, 52 },
    { 0x73bb7c2, 95 },
    { 0, 0 },
    { 0xeadbb7c5, 33 },
    { 0x29cbb7c6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6bb7c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bb7d0, 23 },
    { 0x3fdbb7d0, 85 },
    { 0x8afbb7d0, 33 },
    { 0xf36bb7d2, 61 },
    { 0, 0 },
    { 0xeaebb7d5, 70 },
    { 0, 0 },
    { 0x88ebb7d7, 99 },
    { 0xba6bb7d7, 34 },
    { 0, 0 },
    { 0x127bb7da, 85 },
    { 0xe7dbb7db, 95 },
    { 0, 0 },
    { 0x1d0bb7dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00bb7e4, 23 },
    { 0x453bb7e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdbb7ea, 33 },
    { 0xf00bb7eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bb7f0, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871bb7f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbb7fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3bb7fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fbb800, 61 },
    { 0x10fbb801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4bb807, 66 },
    { 0, 0 },
    { 0xd88bb809, 55 },
    { 0x94cbb809, 95 },
    { 0, 0 },
    { 0xfffbb80c, 55 },
    { 0, 0 },
    { 0xe6ebb80e, 93 },
    { 0xe41bb80e, 68 },
    { 0x890bb810, 90 },
    { 0, 0 },
    { 0x7febb812, 95 },
    { 0, 0 },
    { 0xa62bb814, 33 },
    { 0, 0 },
    { 0x1b2bb816, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303bb81a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bb81e, 23 },
    { 0x47dbb81f, 91 },
    { 0xb34bb81f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55bb828, 78 },
    { 0xb86bb829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2bb82c, 85 },
    { 0, 0 },
    { 0x895bb82e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb833, 95 },
    { 0, 0 },
    { 0x466bb835, 33 },
    { 0xc0bb836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14bb840, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfebb843, 77 },
    { 0x5f1bb843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79bb847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fbb84b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15bb854, 95 },
    { 0xf01bb855, 93 },
    { 0x490bb855, 61 },
    { 0xfe9bb857, 23 },
    { 0x15ebb858, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ebb85e, 90 },
    { 0, 0 },
    { 0x715bb860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafabb865, 78 },
    { 0x5b8bb865, 61 },
    { 0, 0 },
    { 0x654bb868, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8bb86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cbb86f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbb874, 46 },
    { 0xe44bb875, 66 },
    { 0, 0 },
    { 0xd3bb877, 33 },
    { 0, 0 },
    { 0xde1bb879, 91 },
    { 0, 0 },
    { 0xd4abb87b, 66 },
    { 0xfdebb87b, 23 },
    { 0x8d2bb87d, 94 },
    { 0x9a6bb87b, 23 },
    { 0, 0 },
    { 0xb59bb880, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991bb883, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bbb887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64bb88c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edbb890, 95 },
    { 0x3ddbb891, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0bb894, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efbb898, 95 },
    { 0, 0 },
    { 0xbdbbb89a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebb89d, 33 },
    { 0x1b5bb89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cebb8a8, 43 },
    { 0xf55bb8a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebb8af, 52 },
    { 0, 0 },
    { 0x3bcbb8b1, 85 },
    { 0xdbdbb8b2, 90 },
    { 0x542bb8b2, 52 },
    { 0, 0 },
    { 0x7cbbb8b5, 61 },
    { 0, 0 },
    { 0xb9cbb8b7, 90 },
    { 0, 0 },
    { 0xfd5bb8b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dabb8c0, 84 },
    { 0x747bb8c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09bb8c5, 61 },
    { 0x44ebb8c6, 76 },
    { 0, 0 },
    { 0x186bb8c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3abb8ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77bb8d2, 97 },
    { 0xc46bb8d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1bb8d9, 33 },
    { 0, 0 },
    { 0x84bb8db, 55 },
    { 0x78abb8db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbb8e4, 95 },
    { 0xd05bb8e5, 76 },
    { 0x5a2bb8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbebb8eb, 61 },
    { 0x7cabb8ec, 81 },
    { 0x4cabb8ed, 85 },
    { 0, 0 },
    { 0x49abb8ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54bb8f2, 61 },
    { 0x7d7bb8f3, 92 },
    { 0x128bb8f3, 61 },
    { 0x9a9bb8f3, 61 },
    { 0xadbb8f3, 61 },
    { 0x6d8bb8f7, 95 },
    { 0, 0 },
    { 0x27cbb8f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31bb8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5bb90a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x306bb90d, 33 },
    { 0xbcfbb90e, 33 },
    { 0xb1dbb90f, 34 },
    { 0x2ebb910, 93 },
    { 0x77abb911, 90 },
    { 0xef3bb90e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5bb917, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0bb91c, 33 },
    { 0x862bb91d, 23 },
    { 0, 0 },
    { 0xdfebb91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbb923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64abb926, 90 },
    { 0xb98bb927, 61 },
    { 0xa7dbb926, 61 },
    { 0xf42bb928, 46 },
    { 0x29ebb92a, 61 },
    { 0xdb7bb92b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eabb92e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285bb932, 93 },
    { 0x1b9bb932, 33 },
    { 0x228bb934, 61 },
    { 0, 0 },
    { 0xce6bb936, 61 },
    { 0x9e0bb937, 61 },
    { 0x4a5bb937, 86 },
    { 0x5cbbb938, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccbb940, 26 },
    { 0x995bb940, 66 },
    { 0x5e3bb940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842bb946, 36 },
    { 0xac1bb947, 33 },
    { 0, 0 },
    { 0x7f7bb949, 24 },
    { 0xa9bbb949, 87 },
    { 0x90fbb949, 23 },
    { 0x532bb949, 40 },
    { 0x803bb94d, 78 },
    { 0xc0ebb94d, 85 },
    { 0, 0 },
    { 0xb75bb950, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9bb958, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb95c, 33 },
    { 0, 0 },
    { 0x259bb95e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7bb961, 95 },
    { 0x8dbb962, 77 },
    { 0xd33bb963, 76 },
    { 0x8b2bb961, 23 },
    { 0x967bb965, 33 },
    { 0xddebb961, 55 },
    { 0x43ebb967, 90 },
    { 0x18fbb967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80bb970, 23 },
    { 0, 0 },
    { 0x154bb972, 61 },
    { 0xa4abb972, 95 },
    { 0x748bb974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fbb97e, 49 },
    { 0x8afbb97f, 95 },
    { 0, 0 },
    { 0xd00bb981, 87 },
    { 0x474bb982, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbb988, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bb98c, 91 },
    { 0x492bb98c, 66 },
    { 0x54cbb98e, 61 },
    { 0x2c7bb98f, 78 },
    { 0x268bb98f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940bb995, 43 },
    { 0xdbbb996, 90 },
    { 0, 0 },
    { 0x8a2bb998, 33 },
    { 0x9e4bb999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9bb99d, 95 },
    { 0xa9bb99d, 66 },
    { 0xdcebb99f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fbb9a5, 61 },
    { 0xc97bb9a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41bb9a9, 95 },
    { 0x710bb9a9, 75 },
    { 0, 0 },
    { 0x4bbbb9ac, 101 },
    { 0x19cbb9ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7abb9b0, 66 },
    { 0, 0 },
    { 0x2abbb9b2, 61 },
    { 0, 0 },
    { 0x9bdbb9b4, 33 },
    { 0x185bb9b5, 93 },
    { 0, 0 },
    { 0x6b7bb9b7, 55 },
    { 0x440bb9b8, 46 },
    { 0x8b6bb9b9, 23 },
    { 0, 0 },
    { 0x908bb9bb, 93 },
    { 0, 0 },
    { 0xd15bb9bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673bb9c2, 61 },
    { 0xfbfbb9c3, 91 },
    { 0x54ebb9c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fbb9c7, 67 },
    { 0, 0 },
    { 0x58bb9c9, 98 },
    { 0, 0 },
    { 0x248bb9cb, 93 },
    { 0x374bb9cc, 43 },
    { 0x15abb9cb, 87 },
    { 0x6b3bb9cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eebb9d4, 90 },
    { 0xd32bb9d5, 33 },
    { 0, 0 },
    { 0xc7dbb9d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8bb9e0, 78 },
    { 0, 0 },
    { 0x892bb9e2, 90 },
    { 0, 0 },
    { 0xccabb9e4, 23 },
    { 0x201bb9e5, 34 },
    { 0, 0 },
    { 0xff1bb9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bb9eb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852bb9f4, 90 },
    { 0x6bfbb9f5, 96 },
    { 0x78bb9f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbb9fd, 61 },
    { 0x9abbb9fe, 67 },
    { 0xe15bb9fe, 61 },
    { 0x340bba00, 76 },
    { 0xca8bba01, 23 },
    { 0x93cbba00, 76 },
    { 0x2e9bba01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbba0d, 76 },
    { 0x54cbba0e, 93 },
    { 0x56fbba0f, 95 },
    { 0xdfcbba0f, 90 },
    { 0x784bba0f, 95 },
    { 0x3aabba12, 90 },
    { 0x9b6bba0d, 95 },
    { 0, 0 },
    { 0x152bba15, 61 },
    { 0, 0 },
    { 0x898bba17, 99 },
    { 0x2a2bba17, 33 },
    { 0xe9cbba18, 23 },
    { 0x530bba1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37bba1d, 90 },
    { 0x3fbba1e, 66 },
    { 0xa8ebba1e, 33 },
    { 0xd0bbba1d, 69 },
    { 0xac7bba21, 76 },
    { 0xc9dbba21, 46 },
    { 0x67abba23, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7abba28, 91 },
    { 0x7ffbba28, 33 },
    { 0xa8cbba2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebba2f, 34 },
    { 0, 0 },
    { 0xe33bba31, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183bba35, 93 },
    { 0, 0 },
    { 0xa39bba37, 34 },
    { 0x439bba38, 40 },
    { 0x1a7bba39, 34 },
    { 0xb70bba3a, 23 },
    { 0, 0 },
    { 0x923bba3c, 80 },
    { 0, 0 },
    { 0xd5bbba3e, 91 },
    { 0, 0 },
    { 0xd9ebba40, 76 },
    { 0x6d7bba41, 83 },
    { 0x4d2bba42, 33 },
    { 0xb56bba43, 43 },
    { 0x897bba41, 76 },
    { 0x859bba43, 95 },
    { 0x9fdbba46, 34 },
    { 0xc51bba46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6bba4b, 23 },
    { 0, 0 },
    { 0x617bba4d, 90 },
    { 0xe85bba4e, 98 },
    { 0x660bba4f, 55 },
    { 0xb98bba50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bba55, 33 },
    { 0xf0abba56, 90 },
    { 0, 0 },
    { 0xb0bba58, 52 },
    { 0x44abba59, 61 },
    { 0xb05bba5a, 90 },
    { 0xaf9bba5b, 90 },
    { 0xc37bba5b, 33 },
    { 0xd5cbba59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x947bba60, 95 },
    { 0xc5fbba60, 90 },
    { 0xd0ebba61, 98 },
    { 0x9b9bba63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3bba66, 33 },
    { 0xce4bba67, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dbba6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5abba6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606bba72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133bba76, 90 },
    { 0x941bba77, 33 },
    { 0x784bba77, 69 },
    { 0, 0 },
    { 0xe76bba7a, 93 },
    { 0, 0 },
    { 0x1f8bba7c, 33 },
    { 0x479bba7c, 77 },
    { 0, 0 },
    { 0x9c8bba7f, 36 },
    { 0x16cbba80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bba83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4abba87, 78 },
    { 0x4e8bba88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184bba8b, 90 },
    { 0x1bbbba8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdbba91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6bba96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89bba99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bba9d, 77 },
    { 0x706bba9e, 87 },
    { 0x705bba9e, 90 },
    { 0x6e6bba9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabbaa6, 33 },
    { 0, 0 },
    { 0x6f6bbaa8, 67 },
    { 0xf5abbaa9, 33 },
    { 0x7eabbaaa, 91 },
    { 0x6dabbaaa, 61 },
    { 0x43fbbaaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992bbab0, 33 },
    { 0x34abbab1, 61 },
    { 0, 0 },
    { 0x103bbab3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816bbab8, 23 },
    { 0x7b4bbab9, 85 },
    { 0x2dbbbab9, 77 },
    { 0x583bbabb, 33 },
    { 0x97bbbabc, 23 },
    { 0x1d0bbabc, 88 },
    { 0xf2ebbabe, 17 },
    { 0x848bbabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbac2, 61 },
    { 0xf63bbac3, 95 },
    { 0xd79bbac3, 98 },
    { 0xa55bbac5, 33 },
    { 0xc47bbac6, 55 },
    { 0x430bbac7, 98 },
    { 0, 0 },
    { 0x164bbac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fbbacc, 64 },
    { 0xc94bbacd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98bbad0, 65 },
    { 0x4ddbbad1, 34 },
    { 0, 0 },
    { 0xebbbbad3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbbad9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x319bbadc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356bbae0, 61 },
    { 0xbd3bbae1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfbbae7, 61 },
    { 0x33dbbae8, 17 },
    { 0, 0 },
    { 0x8cdbbaea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2bbaf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292bbaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528bbb00, 61 },
    { 0, 0 },
    { 0xaddbbb02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7bbb07, 33 },
    { 0x822bbb08, 55 },
    { 0xca8bbb09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bbbb1c, 90 },
    { 0xe8ebbb1d, 46 },
    { 0xc83bbb1e, 66 },
    { 0x299bbb1f, 61 },
    { 0xf95bbb20, 23 },
    { 0x487bbb1e, 80 },
    { 0xa59bbb22, 78 },
    { 0x10bbb23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbbbb27, 34 },
    { 0x72bbbb28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x159bbb2b, 33 },
    { 0, 0 },
    { 0x61abbb2d, 33 },
    { 0xc05bbb2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99bbb31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cbbb34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbbbb39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efbbb42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bbb47, 98 },
    { 0x7c6bbb48, 98 },
    { 0xd8bbb49, 98 },
    { 0x7dfbbb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dbbb4d, 77 },
    { 0x64dbbb4d, 90 },
    { 0x1dcbbb4d, 61 },
    { 0x5a5bbb50, 95 },
    { 0x60abbb4e, 90 },
    { 0x98fbbb52, 98 },
    { 0x4e6bbb53, 34 },
    { 0, 0 },
    { 0x1d0bbb55, 33 },
    { 0x9fbbb55, 76 },
    { 0xd33bbb56, 23 },
    { 0xf11bbb58, 91 },
    { 0xe01bbb58, 40 },
    { 0x2babbb5a, 90 },
    { 0xb7dbbb57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79bbb5f, 90 },
    { 0, 0 },
    { 0x7bebbb61, 55 },
    { 0x9babbb62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637bbb69, 90 },
    { 0xc7ebbb69, 95 },
    { 0xbe0bbb69, 34 },
    { 0, 0 },
    { 0x339bbb6d, 88 },
    { 0x371bbb6e, 85 },
    { 0xebbbbb6d, 33 },
    { 0, 0 },
    { 0xb0bbbb71, 33 },
    { 0xbcdbbb71, 61 },
    { 0, 0 },
    { 0xf62bbb74, 23 },
    { 0, 0 },
    { 0x9bbbbb76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbbb80, 93 },
    { 0, 0 },
    { 0xc45bbb82, 23 },
    { 0xc95bbb83, 61 },
    { 0x945bbb84, 55 },
    { 0, 0 },
    { 0xb60bbb86, 23 },
    { 0xdb5bbb86, 61 },
    { 0, 0 },
    { 0x832bbb89, 90 },
    { 0, 0 },
    { 0x4b5bbb8b, 98 },
    { 0x4a5bbb8b, 33 },
    { 0xc17bbb8d, 76 },
    { 0x2f3bbb8e, 90 },
    { 0xac5bbb8b, 81 },
    { 0, 0 },
    { 0x15cbbb91, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabbb96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bbbb9d, 29 },
    { 0, 0 },
    { 0x691bbb9f, 95 },
    { 0x7d8bbb9f, 33 },
    { 0xc76bbb9f, 61 },
    { 0x748bbba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75bbba6, 23 },
    { 0xa38bbba7, 52 },
    { 0, 0 },
    { 0x888bbba9, 75 },
    { 0xe6dbbbaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebbbad, 76 },
    { 0xe4bbbad, 90 },
    { 0, 0 },
    { 0xec5bbbb0, 61 },
    { 0x938bbbb0, 55 },
    { 0xafcbbbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9bbbbb, 95 },
    { 0xc42bbbbb, 33 },
    { 0xdc0bbbbc, 90 },
    { 0x8cfbbbbb, 65 },
    { 0x5afbbbbf, 95 },
    { 0x46cbbbc0, 33 },
    { 0x379bbbc1, 52 },
    { 0xa62bbbbd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5bbbc5, 90 },
    { 0, 0 },
    { 0x52cbbbc7, 91 },
    { 0x94dbbbc8, 33 },
    { 0x786bbbc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662bbbcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbbd1, 93 },
    { 0x7bcbbbd2, 93 },
    { 0xb5abbbd3, 66 },
    { 0x825bbbd3, 98 },
    { 0x3dcbbbd3, 34 },
    { 0x7d0bbbd1, 43 },
    { 0, 0 },
    { 0xb4bbbd8, 85 },
    { 0, 0 },
    { 0x3c6bbbda, 98 },
    { 0x7febbbdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bbbe7, 55 },
    { 0, 0 },
    { 0x95abbbe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240bbbed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfbbbf3, 68 },
    { 0x68dbbbf4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbbbf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bbbfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabbc04, 58 },
    { 0xaf4bbc05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09bbc09, 66 },
    { 0, 0 },
    { 0x47fbbc0b, 67 },
    { 0xd14bbc0c, 61 },
    { 0xfebbbc0c, 34 },
    { 0xc0fbbc0e, 90 },
    { 0x925bbc0e, 98 },
    { 0x910bbc10, 33 },
    { 0x341bbc11, 33 },
    { 0, 0 },
    { 0xf78bbc13, 33 },
    { 0x31bbbc13, 23 },
    { 0x23cbbc15, 95 },
    { 0xad2bbc16, 55 },
    { 0, 0 },
    { 0xc90bbc18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fbbc1c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ebbc25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1bbc28, 95 },
    { 0x356bbc29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fbbc2c, 90 },
    { 0, 0 },
    { 0x5c7bbc2e, 95 },
    { 0xd16bbc2f, 65 },
    { 0, 0 },
    { 0x20dbbc31, 77 },
    { 0x286bbc31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dbbc36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0bbc39, 87 },
    { 0xf79bbc3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cbbc3e, 78 },
    { 0x441bbc3f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363bbc44, 66 },
    { 0xfe4bbc45, 85 },
    { 0xe85bbc46, 59 },
    { 0x2e6bbc46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1bbc4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594bbc4f, 46 },
    { 0x3ecbbc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214bbc54, 55 },
    { 0x7c1bbc55, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8bbc59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbc5f, 61 },
    { 0x563bbc60, 61 },
    { 0, 0 },
    { 0x578bbc62, 85 },
    { 0x50fbbc62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ebbc69, 61 },
    { 0, 0 },
    { 0xef4bbc6b, 96 },
    { 0, 0 },
    { 0xc0cbbc6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946bbc72, 90 },
    { 0, 0 },
    { 0x9e7bbc74, 95 },
    { 0, 0 },
    { 0x5acbbc76, 23 },
    { 0, 0 },
    { 0x4b4bbc78, 90 },
    { 0, 0 },
    { 0x997bbc7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8bbc7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbbc84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22bbc8d, 52 },
    { 0xb81bbc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbbc96, 61 },
    { 0xed6bbc97, 23 },
    { 0xc4bbbc98, 33 },
    { 0x6debbc98, 87 },
    { 0xbc3bbc9a, 61 },
    { 0xc04bbc9b, 98 },
    { 0x84abbc97, 23 },
    { 0xc72bbc9d, 33 },
    { 0x679bbc98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5bbca4, 98 },
    { 0, 0 },
    { 0x256bbca6, 95 },
    { 0, 0 },
    { 0xa20bbca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbbcac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ebbcb0, 100 },
    { 0x336bbcb1, 90 },
    { 0x571bbcb1, 95 },
    { 0, 0 },
    { 0x833bbcb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90bbcb7, 100 },
    { 0x5e2bbcb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbbbcbb, 95 },
    { 0x695bbcbc, 93 },
    { 0xb79bbcbd, 52 },
    { 0, 0 },
    { 0xdb2bbcbf, 90 },
    { 0, 0 },
    { 0x75bbbcc1, 73 },
    { 0x5acbbcc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbbcc7, 90 },
    { 0x714bbcc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fbbccc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912bbcd0, 43 },
    { 0xbdfbbcd1, 85 },
    { 0xce6bbcd1, 76 },
    { 0x608bbcd3, 43 },
    { 0xa1fbbcd4, 61 },
    { 0xea8bbcd0, 98 },
    { 0x707bbcd6, 98 },
    { 0x7ebbbcd4, 43 },
    { 0x299bbcd0, 61 },
    { 0x43cbbcd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975bbce0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cbbce3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48bbce6, 95 },
    { 0, 0 },
    { 0xa67bbce8, 78 },
    { 0x917bbce9, 96 },
    { 0x4f8bbce9, 33 },
    { 0, 0 },
    { 0x3bcbbcec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abbcf5, 61 },
    { 0x67dbbcf5, 76 },
    { 0x9b6bbcf5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dbbcfe, 91 },
    { 0, 0 },
    { 0x5debbd00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172bbd05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabbd0a, 87 },
    { 0x2f1bbd0b, 61 },
    { 0x81dbbd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bbbd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bbd1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95bbd28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993bbd2e, 34 },
    { 0x578bbd2f, 61 },
    { 0x23abbd30, 90 },
    { 0x6cdbbd31, 75 },
    { 0xa2fbbd31, 98 },
    { 0xecebbd31, 33 },
    { 0x436bbd34, 76 },
    { 0, 0 },
    { 0x8f3bbd36, 77 },
    { 0x9ddbbd37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527bbd3b, 23 },
    { 0x4e7bbd3b, 95 },
    { 0x15ebbd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbbd42, 17 },
    { 0, 0 },
    { 0x552bbd44, 33 },
    { 0x940bbd44, 98 },
    { 0x257bbd46, 55 },
    { 0, 0 },
    { 0xbb9bbd48, 98 },
    { 0x420bbd49, 34 },
    { 0, 0 },
    { 0x7a0bbd4b, 95 },
    { 0x324bbd4c, 17 },
    { 0, 0 },
    { 0xfabbd4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2bbd53, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x322bbd56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6bbd5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dbbd61, 67 },
    { 0x50ebbd62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0bbd6a, 38 },
    { 0x658bbd6b, 68 },
    { 0x247bbd6c, 99 },
    { 0xaabbbd6c, 17 },
    { 0x2b3bbd6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59bbd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bbd79, 98 },
    { 0xc40bbd7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cabbd7e, 95 },
    { 0, 0 },
    { 0xc4bbd80, 78 },
    { 0xd8bbd81, 91 },
    { 0, 0 },
    { 0xfbebbd83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebbd8d, 99 },
    { 0x469bbd8e, 84 },
    { 0xa84bbd8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bbbd92, 95 },
    { 0, 0 },
    { 0x1debbd94, 67 },
    { 0x8b0bbd95, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8bbda0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69bbdac, 77 },
    { 0xd9cbbdad, 91 },
    { 0x137bbdad, 93 },
    { 0x871bbdad, 90 },
    { 0x243bbdad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcebbdb3, 91 },
    { 0, 0 },
    { 0x45ebbdb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x837bbdb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbbdbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439bbdc0, 61 },
    { 0x24bbbdc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x374bbdc4, 61 },
    { 0xe0dbbdc4, 33 },
    { 0xb11bbdc6, 24 },
    { 0x102bbdc7, 46 },
    { 0, 0 },
    { 0xe92bbdc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69bbdcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304bbdd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180bbdda, 65 },
    { 0xdd6bbdda, 92 },
    { 0xfdbbddb, 96 },
    { 0x877bbddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98bbde0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbbde4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bbdec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeebbdef, 90 },
    { 0xf8dbbdf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x43abbdf3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5bbdfc, 69 },
    { 0x2a4bbdfd, 75 },
    { 0xe38bbdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbe01, 90 },
    { 0x36fbbe02, 34 },
    { 0, 0 },
    { 0xa49bbe04, 95 },
    { 0x922bbe05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ebbe0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bbbe12, 90 },
    { 0x13bbbe12, 33 },
    { 0x693bbe14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x321bbe17, 95 },
    { 0, 0 },
    { 0xdf0bbe19, 98 },
    { 0x571bbe1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bbe25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409bbe2a, 95 },
    { 0, 0 },
    { 0x1e2bbe2c, 23 },
    { 0x5abbbe2d, 33 },
    { 0x369bbe2c, 84 },
    { 0, 0 },
    { 0xcf1bbe30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bbe34, 95 },
    { 0, 0 },
    { 0x244bbe36, 80 },
    { 0x8b0bbe36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ebbe41, 87 },
    { 0x697bbe41, 61 },
    { 0x7b2bbe43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3bbe46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x756bbe49, 34 },
    { 0x486bbe4a, 98 },
    { 0x6b7bbe4a, 95 },
    { 0x18bbe4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28bbe53, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fbbe57, 87 },
    { 0xf29bbe57, 91 },
    { 0x750bbe59, 33 },
    { 0xb0dbbe59, 43 },
    { 0xbc6bbe5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460bbe61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26bbe64, 33 },
    { 0xe85bbe64, 61 },
    { 0x32fbbe66, 84 },
    { 0x969bbe66, 75 },
    { 0x11ebbe66, 95 },
    { 0x307bbe66, 93 },
    { 0, 0 },
    { 0x606bbe6b, 100 },
    { 0x96abbe6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdbbe72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3bbe75, 76 },
    { 0, 0 },
    { 0x5b1bbe77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38bbe7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fbbe7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08bbe80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8bbe89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbbe8d, 55 },
    { 0x965bbe8d, 55 },
    { 0x83bbe8f, 98 },
    { 0x2dbbe8f, 90 },
    { 0x7bbbbe90, 55 },
    { 0xc4cbbe92, 88 },
    { 0x119bbe92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2bbe97, 76 },
    { 0xdb1bbe97, 46 },
    { 0x98bbbe99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5bbe9d, 34 },
    { 0, 0 },
    { 0xa95bbe9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1bbea2, 84 },
    { 0, 0 },
    { 0xc1bbea4, 55 },
    { 0xf00bbea5, 93 },
    { 0, 0 },
    { 0x119bbea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x282bbeaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14bbeaf, 98 },
    { 0xba5bbeb0, 33 },
    { 0xdc2bbeb0, 92 },
    { 0x2c6bbeb2, 46 },
    { 0xd62bbeb1, 23 },
    { 0x8eebbeb2, 34 },
    { 0, 0 },
    { 0x6cdbbeb6, 90 },
    { 0x609bbeb6, 90 },
    { 0x4eabbeb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6bbec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x135bbec6, 98 },
    { 0, 0 },
    { 0xe46bbec8, 23 },
    { 0, 0 },
    { 0x69ebbeca, 61 },
    { 0xe7cbbecb, 23 },
    { 0x463bbecc, 33 },
    { 0xc48bbecb, 23 },
    { 0x8a4bbece, 68 },
    { 0x9adbbecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afbbed3, 90 },
    { 0x663bbed3, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81bbed7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abbbedc, 91 },
    { 0, 0 },
    { 0xfa2bbede, 66 },
    { 0x294bbedf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadbbee7, 66 },
    { 0x179bbee8, 46 },
    { 0xb1ebbee9, 76 },
    { 0x31ebbeea, 91 },
    { 0x857bbeeb, 76 },
    { 0x94ebbeec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8bbeef, 100 },
    { 0, 0 },
    { 0x678bbef1, 43 },
    { 0x894bbef2, 43 },
    { 0xc5bbef3, 76 },
    { 0x369bbef3, 77 },
    { 0, 0 },
    { 0xa07bbef6, 61 },
    { 0x776bbef6, 98 },
    { 0xdecbbef8, 61 },
    { 0xd33bbef9, 76 },
    { 0xf4abbef9, 87 },
    { 0, 0 },
    { 0x24abbefc, 61 },
    { 0, 0 },
    { 0xa7ebbefe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3bbf02, 46 },
    { 0xf63bbf02, 98 },
    { 0x241bbf03, 43 },
    { 0x4bdbbf02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dbbf08, 34 },
    { 0, 0 },
    { 0x206bbf0a, 99 },
    { 0xdebbf0b, 99 },
    { 0, 0 },
    { 0x179bbf0d, 53 },
    { 0, 0 },
    { 0x711bbf0f, 34 },
    { 0xf59bbf0f, 76 },
    { 0xc09bbf11, 67 },
    { 0x7f0bbf12, 23 },
    { 0xe3abbf12, 61 },
    { 0x602bbf14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x219bbf17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2bbf21, 33 },
    { 0xe91bbf22, 33 },
    { 0x1dbbf22, 91 },
    { 0x362bbf22, 33 },
    { 0, 0 },
    { 0xde7bbf26, 19 },
    { 0xe67bbf27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bbf2a, 64 },
    { 0, 0 },
    { 0x758bbf2c, 55 },
    { 0xb91bbf2d, 87 },
    { 0xe60bbf2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18bbf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bbbf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bbf3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dbbf3f, 95 },
    { 0, 0 },
    { 0xb90bbf41, 98 },
    { 0, 0 },
    { 0x9d9bbf43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142bbf47, 33 },
    { 0xd4bbbf48, 61 },
    { 0xb51bbf49, 55 },
    { 0, 0 },
    { 0x82ebbf4b, 65 },
    { 0x9c5bbf4b, 76 },
    { 0x5e4bbf4b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbbbf50, 61 },
    { 0xff1bbf50, 95 },
    { 0x60bbf52, 98 },
    { 0xd68bbf53, 95 },
    { 0xf70bbf53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3bbf58, 61 },
    { 0xe6bbbf58, 33 },
    { 0xfa0bbf59, 100 },
    { 0, 0 },
    { 0x5b8bbf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bbf5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fbbf65, 23 },
    { 0xc0cbbf65, 90 },
    { 0x27bbbf65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcbbf6b, 91 },
    { 0xcadbbf6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6bbf70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728bbf76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bbf7a, 98 },
    { 0, 0 },
    { 0x613bbf7c, 90 },
    { 0xec4bbf7d, 95 },
    { 0x38bbf7e, 61 },
    { 0, 0 },
    { 0x244bbf80, 81 },
    { 0x750bbf80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bbf86, 90 },
    { 0x2cbbbf87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bbf8a, 98 },
    { 0xa0bbf8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1bbf8f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bbbf93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1bbf97, 95 },
    { 0, 0 },
    { 0x912bbf99, 98 },
    { 0x972bbf9a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbbf9d, 93 },
    { 0xb9cbbf9e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cbbfa3, 76 },
    { 0, 0 },
    { 0xbc4bbfa5, 23 },
    { 0x81abbfa6, 95 },
    { 0x514bbfa5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bbfaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2bbfad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bbfb0, 95 },
    { 0xa6abbfb1, 75 },
    { 0, 0 },
    { 0x101bbfb3, 76 },
    { 0x561bbfb4, 91 },
    { 0xc2bbfb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbbfb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1bbfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5bbfc0, 76 },
    { 0x27cbbfc1, 85 },
    { 0x25cbbfc2, 99 },
    { 0x360bbfc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bbfc6, 33 },
    { 0x839bbfc7, 61 },
    { 0, 0 },
    { 0x3a9bbfc9, 43 },
    { 0, 0 },
    { 0xb24bbfcb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0bbfd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x793bbfd5, 90 },
    { 0x758bbfd6, 93 },
    { 0, 0 },
    { 0xbc0bbfd8, 67 },
    { 0x9a0bbfd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbbfdd, 98 },
    { 0x302bbfde, 95 },
    { 0, 0 },
    { 0xe78bbfe0, 97 },
    { 0x76cbbfe0, 55 },
    { 0x683bbfe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5bbfe6, 68 },
    { 0, 0 },
    { 0xb6abbfe8, 55 },
    { 0x427bbfe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392bbfee, 61 },
    { 0x97bbbfee, 98 },
    { 0x60fbbfee, 55 },
    { 0x842bbff1, 61 },
    { 0x16bbbfee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcebbff6, 76 },
    { 0x24bbbff7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fbbffe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00bc001, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x257bc004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbc007, 90 },
    { 0xadfbc008, 95 },
    { 0xf4fbc009, 61 },
    { 0xf00bc00a, 46 },
    { 0xf6bc00b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbc013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6bc016, 97 },
    { 0, 0 },
    { 0xd3cbc018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2bc020, 33 },
    { 0, 0 },
    { 0x375bc022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93bc027, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582bc02b, 23 },
    { 0xc46bc02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2bc034, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4bc038, 23 },
    { 0xc6ebc039, 33 },
    { 0xd49bc03a, 95 },
    { 0, 0 },
    { 0x8b8bc03c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92bc03f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbc042, 33 },
    { 0x469bc043, 34 },
    { 0x3bbbc044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bbc047, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cbc04c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83bc050, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fbc055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47bc058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcbc063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17bc067, 23 },
    { 0xbc9bc068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26bc06f, 34 },
    { 0xde6bc06f, 90 },
    { 0x45dbc071, 77 },
    { 0, 0 },
    { 0x926bc073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04bc076, 23 },
    { 0x250bc077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7bc07e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9bc083, 46 },
    { 0x617bc084, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dbc089, 33 },
    { 0, 0 },
    { 0x163bc08b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbc097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dabc09b, 23 },
    { 0, 0 },
    { 0x8e3bc09d, 55 },
    { 0xad4bc09e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517bc0a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fbc0aa, 55 },
    { 0xdb7bc0aa, 87 },
    { 0xba4bc0aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbc0af, 98 },
    { 0xb1bc0af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc0b4, 90 },
    { 0x61bbc0b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bc0ba, 93 },
    { 0xef0bc0bb, 23 },
    { 0xc11bc0bc, 90 },
    { 0, 0 },
    { 0x6d5bc0be, 99 },
    { 0x3d6bc0be, 33 },
    { 0x425bc0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabc0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfbc0cc, 77 },
    { 0x54bbc0cc, 61 },
    { 0, 0 },
    { 0xff3bc0cf, 55 },
    { 0, 0 },
    { 0xeadbc0d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3bc0d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaebc0dc, 77 },
    { 0, 0 },
    { 0xb68bc0de, 90 },
    { 0xfabbc0de, 81 },
    { 0xacdbc0e0, 33 },
    { 0x77bc0e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc0e5, 52 },
    { 0, 0 },
    { 0x180bc0e7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bc0eb, 98 },
    { 0x2bbbc0ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19bc0f2, 43 },
    { 0xcb7bc0f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x821bc0f6, 23 },
    { 0, 0 },
    { 0x905bc0f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7babc104, 90 },
    { 0xac0bc104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37abc10c, 90 },
    { 0x281bc10d, 90 },
    { 0xee0bc10d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37bc111, 87 },
    { 0x44fbc112, 17 },
    { 0xea8bc112, 23 },
    { 0xe5bbc113, 76 },
    { 0x5a2bc115, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377bc11b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bc129, 81 },
    { 0, 0 },
    { 0xc4bc12b, 61 },
    { 0xb98bc12c, 76 },
    { 0xd80bc12d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbc131, 91 },
    { 0, 0 },
    { 0x65bbc133, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391bc138, 90 },
    { 0, 0 },
    { 0x77fbc13a, 76 },
    { 0x234bc13b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbbc149, 81 },
    { 0x3bebc149, 88 },
    { 0x469bc14a, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4bc14e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4bc153, 23 },
    { 0xc3cbc154, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217bc15e, 90 },
    { 0, 0 },
    { 0xc95bc160, 33 },
    { 0x1bc161, 93 },
    { 0x66bc161, 100 },
    { 0xe45bc163, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x689bc166, 43 },
    { 0, 0 },
    { 0x31abc168, 23 },
    { 0x2adbc168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fbc16d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4bc171, 34 },
    { 0x25fbc171, 74 },
    { 0x85cbc172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bc176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x235bc179, 68 },
    { 0, 0 },
    { 0x103bc17b, 97 },
    { 0, 0 },
    { 0x82fbc17d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9bc186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cbc189, 95 },
    { 0, 0 },
    { 0x8d1bc18b, 42 },
    { 0x534bc18c, 78 },
    { 0, 0 },
    { 0x603bc18e, 95 },
    { 0x321bc18e, 33 },
    { 0x613bc18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34abc194, 61 },
    { 0x8afbc195, 61 },
    { 0xc4cbc195, 90 },
    { 0x1aebc197, 61 },
    { 0, 0 },
    { 0xf39bc199, 66 },
    { 0xc66bc199, 43 },
    { 0x15bc19b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57bc1a3, 23 },
    { 0x444bc1a3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4bc1aa, 95 },
    { 0, 0 },
    { 0x27ebc1ac, 90 },
    { 0x29dbc1ad, 76 },
    { 0, 0 },
    { 0xef4bc1af, 87 },
    { 0x852bc1b0, 84 },
    { 0x2cbbc1af, 76 },
    { 0xb6cbc1af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bbc1b5, 23 },
    { 0x270bc1b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc1b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc1bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8bc1c3, 95 },
    { 0, 0 },
    { 0x5f1bc1c5, 43 },
    { 0, 0 },
    { 0xc07bc1c7, 98 },
    { 0x5fcbc1c8, 33 },
    { 0x88ebc1c9, 91 },
    { 0x8aabc1c9, 95 },
    { 0x533bc1cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fbc1ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc1d2, 85 },
    { 0x7fcbc1d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9bc1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155bc1da, 33 },
    { 0x62ebc1db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07bc1df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bc1e6, 90 },
    { 0xd98bc1e7, 90 },
    { 0x19ebc1e8, 49 },
    { 0xf1cbc1e9, 90 },
    { 0xb2bc1e9, 88 },
    { 0x305bc1e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96bc1f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63bc1f5, 23 },
    { 0xc22bc1f6, 99 },
    { 0, 0 },
    { 0xd8bbc1f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6bc200, 85 },
    { 0, 0 },
    { 0x5a2bc202, 90 },
    { 0xa04bc203, 33 },
    { 0x7e0bc203, 77 },
    { 0, 0 },
    { 0x4e5bc206, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bc20a, 33 },
    { 0xbbdbc20b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3bc20f, 17 },
    { 0xd58bc210, 91 },
    { 0xa43bc210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebc21b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3bc21f, 98 },
    { 0xe16bc220, 78 },
    { 0xb98bc220, 90 },
    { 0xa45bc221, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8bc225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fbc229, 90 },
    { 0x86fbc22a, 23 },
    { 0xc02bc22b, 91 },
    { 0x136bc22c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc230, 95 },
    { 0x852bc231, 55 },
    { 0x7f6bc232, 61 },
    { 0xe16bc232, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccbc236, 99 },
    { 0xa0ebc237, 33 },
    { 0, 0 },
    { 0xb03bc239, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dbc23c, 69 },
    { 0xd90bc23c, 77 },
    { 0, 0 },
    { 0x833bc23f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673bc24b, 76 },
    { 0xa7abc24c, 33 },
    { 0xfabc24d, 34 },
    { 0x5b6bc24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86bc251, 61 },
    { 0x6f9bc251, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x572bc255, 68 },
    { 0x644bc256, 76 },
    { 0x1bebc257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbc25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x202bc25f, 68 },
    { 0, 0 },
    { 0xcbc261, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54bc266, 95 },
    { 0x51abc267, 90 },
    { 0xd74bc268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdbc26e, 34 },
    { 0xe5bbc26e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfbc273, 34 },
    { 0x8c9bc274, 76 },
    { 0x582bc274, 96 },
    { 0, 0 },
    { 0xb94bc277, 76 },
    { 0, 0 },
    { 0x2e1bc279, 95 },
    { 0x603bc27a, 61 },
    { 0xf06bc27b, 64 },
    { 0x92dbc27b, 46 },
    { 0x1fbbc27d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9bc280, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcbc285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81bc288, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebc28b, 67 },
    { 0xad7bc28b, 93 },
    { 0, 0 },
    { 0x5aebc28e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfbc292, 90 },
    { 0xffcbc293, 55 },
    { 0x377bc294, 98 },
    { 0, 0 },
    { 0xb43bc296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1bc299, 90 },
    { 0xe3bbc29a, 61 },
    { 0xe80bc299, 33 },
    { 0xd46bc29a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7bc2a1, 87 },
    { 0x50fbc2a2, 66 },
    { 0xd5cbc2a3, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc2aa, 34 },
    { 0, 0 },
    { 0xa2dbc2ac, 95 },
    { 0xfdfbc2ad, 61 },
    { 0x1e5bc2ac, 90 },
    { 0, 0 },
    { 0xf0bbc2b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bc2b3, 76 },
    { 0, 0 },
    { 0xa7fbc2b5, 98 },
    { 0x4dbc2b5, 90 },
    { 0x687bc2b7, 95 },
    { 0x66fbc2b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905bc2c3, 77 },
    { 0x8b8bc2c4, 90 },
    { 0, 0 },
    { 0x393bc2c6, 49 },
    { 0, 0 },
    { 0xb09bc2c8, 88 },
    { 0x5e8bc2c9, 34 },
    { 0xfdcbc2ca, 36 },
    { 0xe53bc2cb, 98 },
    { 0, 0 },
    { 0x519bc2cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6bc2d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ebc2d9, 23 },
    { 0x473bc2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48bc2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0bc2e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cbc2ec, 78 },
    { 0, 0 },
    { 0x507bc2ee, 99 },
    { 0, 0 },
    { 0x607bc2f0, 55 },
    { 0xdecbc2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7bc2f7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bbc2fd, 90 },
    { 0x1d9bc2fd, 95 },
    { 0xd7bbc2ff, 87 },
    { 0xcadbc300, 88 },
    { 0, 0 },
    { 0x6ddbc302, 101 },
    { 0xcdbbc302, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30bc308, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336bc30c, 98 },
    { 0, 0 },
    { 0xaa8bc30e, 76 },
    { 0, 0 },
    { 0x71fbc310, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2bc318, 90 },
    { 0, 0 },
    { 0x8cebc31a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdbc31d, 33 },
    { 0xda5bc31e, 36 },
    { 0x69abc31e, 90 },
    { 0, 0 },
    { 0x6f8bc321, 87 },
    { 0, 0 },
    { 0x9b0bc323, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1bc32b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abbc32f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97bc338, 98 },
    { 0xe70bc339, 90 },
    { 0x721bc33a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ebc342, 33 },
    { 0xfdabc343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1bc346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdebc34b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dbc34f, 76 },
    { 0, 0 },
    { 0xc2fbc351, 95 },
    { 0x8d3bc352, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc357, 93 },
    { 0x6eabc358, 90 },
    { 0, 0 },
    { 0x1dbc35a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9bc35e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15bc361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4bc36c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebc371, 76 },
    { 0, 0 },
    { 0xa23bc373, 90 },
    { 0x794bc374, 46 },
    { 0x2e7bc374, 90 },
    { 0x7bcbc376, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecbc380, 87 },
    { 0, 0 },
    { 0x154bc382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8bc388, 77 },
    { 0x2a7bc388, 90 },
    { 0, 0 },
    { 0x51bbc38b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4bc392, 78 },
    { 0xbfabc392, 66 },
    { 0x8b6bc394, 4 },
    { 0xd59bc394, 33 },
    { 0, 0 },
    { 0xb16bc397, 33 },
    { 0xa16bc398, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143bc39c, 98 },
    { 0x5d6bc39d, 84 },
    { 0, 0 },
    { 0xe62bc39f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x954bc3a2, 98 },
    { 0, 0 },
    { 0x8e2bc3a4, 98 },
    { 0x3acbc3a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afbc3ac, 90 },
    { 0x79bbc3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534bc3b1, 90 },
    { 0xfc4bc3b2, 55 },
    { 0xd60bc3b3, 23 },
    { 0xb6abc3b3, 34 },
    { 0xaffbc3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x179bc3b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69bc3bc, 33 },
    { 0x94fbc3bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55bc3c4, 76 },
    { 0xe8fbc3c4, 85 },
    { 0x54bc3c6, 90 },
    { 0x2abbc3c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674bc3cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bc3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58bc3d1, 23 },
    { 0, 0 },
    { 0x95dbc3d3, 90 },
    { 0, 0 },
    { 0xb07bc3d5, 98 },
    { 0, 0 },
    { 0xe2abc3d7, 61 },
    { 0x3c8bc3d8, 33 },
    { 0, 0 },
    { 0xf4cbc3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bbc3df, 78 },
    { 0x1fbc3e0, 23 },
    { 0x575bc3e1, 66 },
    { 0x357bc3e2, 55 },
    { 0xad7bc3e2, 76 },
    { 0, 0 },
    { 0xdbdbc3e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49bc3e9, 90 },
    { 0xf78bc3ea, 55 },
    { 0, 0 },
    { 0x56fbc3ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bc3f1, 98 },
    { 0, 0 },
    { 0xd34bc3f3, 55 },
    { 0x998bc3f4, 61 },
    { 0x9f5bc3f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bc3fc, 17 },
    { 0xc95bc3fd, 78 },
    { 0x3d2bc3fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dbc402, 61 },
    { 0x927bc403, 90 },
    { 0xf21bc404, 34 },
    { 0, 0 },
    { 0x502bc406, 78 },
    { 0x512bc406, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abc40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fbc413, 33 },
    { 0x1bc413, 61 },
    { 0x15fbc414, 61 },
    { 0x1a9bc416, 81 },
    { 0x7e6bc417, 46 },
    { 0, 0 },
    { 0x2e9bc419, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421bc41f, 87 },
    { 0x4a4bc420, 90 },
    { 0x517bc421, 23 },
    { 0x19cbc422, 46 },
    { 0xd5cbc41f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc428, 33 },
    { 0x234bc429, 61 },
    { 0x774bc42a, 46 },
    { 0, 0 },
    { 0xbf4bc42c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62bc433, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dbc437, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x818bc43a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fbc440, 101 },
    { 0, 0 },
    { 0xcdcbc442, 34 },
    { 0, 0 },
    { 0x8e7bc444, 33 },
    { 0x250bc444, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bc44b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32bc44e, 98 },
    { 0x90dbc44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523bc455, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdabc458, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791bc45d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862bc461, 23 },
    { 0x3a0bc462, 55 },
    { 0x404bc463, 98 },
    { 0x1aabc464, 33 },
    { 0xf18bc465, 93 },
    { 0xfb8bc464, 55 },
    { 0x5ccbc466, 33 },
    { 0xa75bc461, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41bc46b, 90 },
    { 0, 0 },
    { 0x380bc46d, 61 },
    { 0xb0abc46d, 76 },
    { 0xf86bc46d, 90 },
    { 0x77cbc470, 84 },
    { 0, 0 },
    { 0x51ebc472, 90 },
    { 0x135bc472, 61 },
    { 0x9bcbc474, 23 },
    { 0x5cebc475, 34 },
    { 0x8b9bc475, 90 },
    { 0xde6bc476, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9bc47c, 100 },
    { 0x827bc47d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bc480, 34 },
    { 0, 0 },
    { 0xabcbc482, 95 },
    { 0x7f3bc483, 5 },
    { 0x6d6bc483, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bc48c, 95 },
    { 0, 0 },
    { 0xffebc48e, 61 },
    { 0xd55bc48e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69bc492, 85 },
    { 0xad1bc492, 23 },
    { 0x2dbbc494, 95 },
    { 0xa1fbc492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc4a0, 33 },
    { 0, 0 },
    { 0x625bc4a2, 34 },
    { 0xd7fbc4a3, 95 },
    { 0x957bc4a2, 43 },
    { 0x375bc4a5, 78 },
    { 0, 0 },
    { 0xd0cbc4a7, 90 },
    { 0x78bbc4a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96bc4ae, 95 },
    { 0x610bc4ae, 23 },
    { 0xccbc4b0, 90 },
    { 0, 0 },
    { 0x2c1bc4b2, 76 },
    { 0xfd3bc4b3, 98 },
    { 0x5e4bc4b4, 66 },
    { 0xd02bc4b5, 95 },
    { 0, 0 },
    { 0x6d2bc4b7, 55 },
    { 0xbb5bc4b8, 90 },
    { 0xaadbc4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4bc4be, 90 },
    { 0, 0 },
    { 0x200bc4c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x811bc4c3, 34 },
    { 0x4c6bc4c4, 90 },
    { 0, 0 },
    { 0x71fbc4c6, 99 },
    { 0x710bc4c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1bc4ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867bc4ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bc4d2, 90 },
    { 0, 0 },
    { 0x70bbc4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x679bc4d7, 33 },
    { 0, 0 },
    { 0xc8dbc4d9, 95 },
    { 0x279bc4da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ebc4de, 46 },
    { 0, 0 },
    { 0xbb4bc4e0, 46 },
    { 0xc71bc4e0, 95 },
    { 0, 0 },
    { 0x897bc4e3, 98 },
    { 0x306bc4e3, 95 },
    { 0xf0bc4e5, 90 },
    { 0x95dbc4e6, 99 },
    { 0x4eebc4e6, 34 },
    { 0x445bc4e8, 93 },
    { 0xb5abc4e5, 61 },
    { 0, 0 },
    { 0xa57bc4eb, 37 },
    { 0x5c0bc4eb, 98 },
    { 0xfb8bc4ed, 33 },
    { 0, 0 },
    { 0x820bc4ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x956bc4f2, 90 },
    { 0, 0 },
    { 0xa98bc4f4, 46 },
    { 0xd93bc4f5, 98 },
    { 0x3adbc4f6, 46 },
    { 0xc40bc4f7, 93 },
    { 0xc69bc4f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5bc4fd, 99 },
    { 0x344bc4fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cbc503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bbc506, 61 },
    { 0xf5abc507, 23 },
    { 0xb9abc508, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cbc50c, 98 },
    { 0x727bc50c, 67 },
    { 0, 0 },
    { 0x5a5bc50f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864bc515, 76 },
    { 0, 0 },
    { 0x1d1bc517, 40 },
    { 0x48bc518, 43 },
    { 0xdc1bc519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bc51c, 98 },
    { 0xd18bc51d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bc525, 99 },
    { 0x1a4bc525, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbc529, 95 },
    { 0x3afbc529, 95 },
    { 0x732bc52a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafbc53a, 80 },
    { 0, 0 },
    { 0xe16bc53c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fbc541, 90 },
    { 0x578bc541, 67 },
    { 0x83bc543, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46abc547, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4bc54a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46bc54e, 33 },
    { 0x4edbc54e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446bc553, 76 },
    { 0xae0bc554, 46 },
    { 0xc01bc555, 90 },
    { 0xafebc556, 43 },
    { 0x9ffbc555, 100 },
    { 0x24ebc558, 83 },
    { 0x13fbc558, 46 },
    { 0xf4dbc559, 68 },
    { 0x2dbbc55b, 90 },
    { 0x38fbc559, 33 },
    { 0x920bc55d, 33 },
    { 0xf5bbc55e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1bc562, 51 },
    { 0, 0 },
    { 0x708bc564, 23 },
    { 0, 0 },
    { 0x1d3bc566, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6bc56a, 23 },
    { 0x229bc56a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6bc56e, 90 },
    { 0x5acbc56f, 91 },
    { 0, 0 },
    { 0x9fcbc571, 23 },
    { 0xb56bc571, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1bc576, 55 },
    { 0, 0 },
    { 0xac9bc578, 33 },
    { 0x229bc578, 99 },
    { 0x9ebc57a, 46 },
    { 0, 0 },
    { 0x8a5bc57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bc580, 66 },
    { 0x7aebc581, 98 },
    { 0, 0 },
    { 0x3d2bc583, 76 },
    { 0x397bc584, 85 },
    { 0xe84bc585, 55 },
    { 0x4dfbc585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68bc58e, 100 },
    { 0x7d2bc58f, 34 },
    { 0xc90bc590, 48 },
    { 0xa58bc591, 95 },
    { 0xccdbc592, 83 },
    { 0xa29bc593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc59a, 90 },
    { 0xa21bc59b, 85 },
    { 0x87cbc59c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bc5a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713bc5a4, 33 },
    { 0xc5bbc5a4, 68 },
    { 0, 0 },
    { 0x66abc5a7, 61 },
    { 0xd66bc5a7, 23 },
    { 0xc4fbc5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9bc5ad, 67 },
    { 0xb0fbc5ad, 33 },
    { 0x91abc5ad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919bc5b6, 76 },
    { 0xa79bc5b6, 85 },
    { 0x77bbc5b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcbc5bd, 23 },
    { 0, 0 },
    { 0x12cbc5bf, 99 },
    { 0xdbc5c0, 98 },
    { 0, 0 },
    { 0xa86bc5c2, 95 },
    { 0x2c7bc5c2, 61 },
    { 0, 0 },
    { 0xb36bc5c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aebc5c9, 95 },
    { 0, 0 },
    { 0xe40bc5cb, 61 },
    { 0x33cbc5cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xecebc5cf, 61 },
    { 0, 0 },
    { 0xac6bc5d1, 52 },
    { 0, 0 },
    { 0x64fbc5d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bbc5da, 23 },
    { 0, 0 },
    { 0x843bc5dc, 73 },
    { 0, 0 },
    { 0x1d2bc5de, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799bc5e3, 23 },
    { 0x4aebc5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc5f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636bc5f8, 62 },
    { 0, 0 },
    { 0xdabc5fa, 23 },
    { 0x3cabc5fb, 84 },
    { 0x758bc5fc, 23 },
    { 0x668bc5fd, 83 },
    { 0xd0cbc5fb, 95 },
    { 0x739bc5fe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258bc608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x704bc60b, 43 },
    { 0x817bc60b, 61 },
    { 0xf81bc60d, 66 },
    { 0x115bc60e, 17 },
    { 0xf6bbc60e, 99 },
    { 0xbe3bc610, 23 },
    { 0x28cbc610, 90 },
    { 0xe31bc60d, 90 },
    { 0, 0 },
    { 0xc93bc614, 61 },
    { 0x576bc614, 55 },
    { 0x2abbc616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97bc61c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdbc61f, 52 },
    { 0x3d1bc620, 55 },
    { 0, 0 },
    { 0x573bc622, 43 },
    { 0xe80bc622, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5bc629, 46 },
    { 0, 0 },
    { 0x3e0bc62b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x936bc62e, 61 },
    { 0x700bc62f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110bc635, 90 },
    { 0, 0 },
    { 0x550bc637, 95 },
    { 0x720bc637, 88 },
    { 0, 0 },
    { 0x588bc63a, 58 },
    { 0xb6ebc63b, 100 },
    { 0x95dbc63b, 46 },
    { 0xabbbc63d, 46 },
    { 0xee3bc63b, 55 },
    { 0, 0 },
    { 0xd7ebc640, 43 },
    { 0xaf5bc641, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05bc645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5bc648, 44 },
    { 0x74ebc649, 23 },
    { 0x4c1bc648, 55 },
    { 0x354bc64b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6abc650, 61 },
    { 0, 0 },
    { 0x5e0bc652, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60abc656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeabc659, 61 },
    { 0xe0cbc659, 84 },
    { 0x461bc65a, 78 },
    { 0, 0 },
    { 0x425bc65d, 76 },
    { 0, 0 },
    { 0x8d4bc65f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623bc667, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4bc66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fbc66f, 55 },
    { 0, 0 },
    { 0x1f6bc671, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92bc675, 55 },
    { 0, 0 },
    { 0x15cbc677, 61 },
    { 0xa52bc678, 99 },
    { 0x680bc679, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65abc67f, 90 },
    { 0x574bc680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3bc685, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dbc68a, 23 },
    { 0x206bc68b, 76 },
    { 0x42bc68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1abc690, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7bc698, 98 },
    { 0x8ffbc699, 99 },
    { 0x5e6bc699, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bc6a2, 33 },
    { 0xa5bbc6a3, 90 },
    { 0, 0 },
    { 0x661bc6a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5bc6ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbbc6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9bc6b4, 93 },
    { 0x6babc6b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63bc6bc, 88 },
    { 0x5ffbc6bc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefbc6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x583bc6c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dbc6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9bc6d7, 66 },
    { 0x996bc6d7, 66 },
    { 0x40abc6d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbbc6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc6e1, 95 },
    { 0, 0 },
    { 0xa0bbc6e3, 43 },
    { 0, 0 },
    { 0x9f3bc6e5, 99 },
    { 0, 0 },
    { 0x60dbc6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x348bc6ea, 34 },
    { 0x783bc6eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdbc6f5, 55 },
    { 0x267bc6f6, 93 },
    { 0x8e8bc6f6, 33 },
    { 0x2edbc6f6, 90 },
    { 0, 0 },
    { 0x3b2bc6fa, 61 },
    { 0xf5bc6fb, 23 },
    { 0xcdebc6fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bc700, 95 },
    { 0, 0 },
    { 0xa38bc702, 98 },
    { 0xd1abc703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4bc70a, 34 },
    { 0x5c1bc70b, 90 },
    { 0x364bc70c, 77 },
    { 0, 0 },
    { 0xfa1bc70e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550bc713, 90 },
    { 0, 0 },
    { 0x657bc715, 95 },
    { 0, 0 },
    { 0xb17bc717, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33bc71e, 33 },
    { 0, 0 },
    { 0xa3bc720, 61 },
    { 0x93bc720, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364bc727, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08bc72b, 66 },
    { 0x1b7bc72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648bc730, 93 },
    { 0x3ddbc731, 90 },
    { 0x6cdbc730, 87 },
    { 0xdbebc733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28bc736, 33 },
    { 0xe83bc737, 55 },
    { 0xc92bc738, 90 },
    { 0, 0 },
    { 0x73bc73a, 33 },
    { 0, 0 },
    { 0x960bc73c, 90 },
    { 0x81ebc73c, 61 },
    { 0, 0 },
    { 0x52bbc73f, 33 },
    { 0, 0 },
    { 0x24fbc741, 90 },
    { 0, 0 },
    { 0x51fbc743, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc747, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dbc74a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2bc751, 23 },
    { 0x7fbbc751, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751bc757, 46 },
    { 0, 0 },
    { 0xf61bc759, 98 },
    { 0, 0 },
    { 0x7b0bc75b, 33 },
    { 0x515bc75c, 33 },
    { 0x17bc75d, 23 },
    { 0xe82bc75e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01bc763, 68 },
    { 0x6febc764, 90 },
    { 0x730bc765, 98 },
    { 0xf45bc766, 90 },
    { 0x6f5bc765, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x684bc76a, 90 },
    { 0xe9ebc76a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428bc771, 61 },
    { 0xafbbc772, 64 },
    { 0x56bc773, 66 },
    { 0x976bc774, 95 },
    { 0, 0 },
    { 0xcccbc776, 90 },
    { 0x169bc776, 95 },
    { 0x9ddbc778, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436bc781, 67 },
    { 0, 0 },
    { 0x85fbc783, 90 },
    { 0, 0 },
    { 0xd7cbc785, 33 },
    { 0, 0 },
    { 0x81bc787, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4bc78a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bc792, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x969bc795, 33 },
    { 0xf74bc796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbc79b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85bc7a2, 61 },
    { 0xd78bc7a2, 37 },
    { 0, 0 },
    { 0x820bc7a5, 33 },
    { 0xf14bc7a5, 33 },
    { 0, 0 },
    { 0x55bc7a8, 98 },
    { 0x8f2bc7a9, 90 },
    { 0, 0 },
    { 0xcf6bc7ab, 61 },
    { 0xdabbc7ab, 67 },
    { 0x4b0bc7ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dbc7b1, 23 },
    { 0xf9fbc7b1, 33 },
    { 0x575bc7b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951bc7b7, 61 },
    { 0xce3bc7b8, 95 },
    { 0xaddbc7b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83bc7bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84bc7c3, 61 },
    { 0x21ebc7c4, 43 },
    { 0x5cbc7c5, 76 },
    { 0x471bc7c3, 90 },
    { 0xaa3bc7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698bc7cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99abc7cf, 90 },
    { 0x997bc7d0, 23 },
    { 0, 0 },
    { 0x467bc7d2, 90 },
    { 0x309bc7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454bc7d8, 46 },
    { 0x8f9bc7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cbc7e1, 68 },
    { 0, 0 },
    { 0xe01bc7e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3bc7e6, 95 },
    { 0, 0 },
    { 0x3e7bc7e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bc7eb, 76 },
    { 0x38dbc7ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeabc7f4, 33 },
    { 0, 0 },
    { 0x716bc7f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0bc7fb, 98 },
    { 0x377bc7fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc802, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4bc80a, 93 },
    { 0xca1bc80a, 76 },
    { 0x300bc80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fbc811, 33 },
    { 0, 0 },
    { 0xf12bc813, 38 },
    { 0xba0bc813, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbbc818, 61 },
    { 0xc7bc819, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72bc820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cbc829, 46 },
    { 0, 0 },
    { 0xd57bc82b, 98 },
    { 0xdddbc82b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9bc831, 80 },
    { 0xa36bc832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfbc83c, 98 },
    { 0x5fbc83c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbc840, 43 },
    { 0xe69bc841, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ebc845, 98 },
    { 0x858bc846, 98 },
    { 0x25fbc847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebc84e, 55 },
    { 0xd0dbc84f, 61 },
    { 0x9dfbc850, 61 },
    { 0x152bc851, 46 },
    { 0x2f5bc851, 90 },
    { 0xce3bc84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86bc856, 83 },
    { 0x75cbc857, 98 },
    { 0xf8ebc858, 98 },
    { 0xff4bc858, 98 },
    { 0x254bc859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2bc85e, 87 },
    { 0, 0 },
    { 0xe53bc860, 98 },
    { 0x4febc861, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x686bc864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc86a, 90 },
    { 0x214bc86b, 46 },
    { 0x822bc86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcbc872, 93 },
    { 0, 0 },
    { 0xe33bc874, 66 },
    { 0xe3bbc875, 61 },
    { 0x178bc876, 90 },
    { 0, 0 },
    { 0xe66bc878, 17 },
    { 0xea1bc878, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bbc87e, 90 },
    { 0, 0 },
    { 0x440bc880, 67 },
    { 0x245bc881, 100 },
    { 0x447bc881, 46 },
    { 0xc8abc882, 76 },
    { 0xc8bbc884, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733bc889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bc88f, 100 },
    { 0x149bc88f, 98 },
    { 0, 0 },
    { 0x171bc892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfebc897, 61 },
    { 0x79cbc897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1bc89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8abc8a0, 66 },
    { 0, 0 },
    { 0xd59bc8a2, 98 },
    { 0, 0 },
    { 0xf85bc8a4, 33 },
    { 0, 0 },
    { 0x5b1bc8a6, 95 },
    { 0, 0 },
    { 0xdfdbc8a8, 98 },
    { 0x403bc8a8, 95 },
    { 0x6e0bc8aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcbc8ae, 95 },
    { 0, 0 },
    { 0x4f8bc8b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ebc8b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350bc8bb, 33 },
    { 0, 0 },
    { 0x425bc8bd, 40 },
    { 0xdd1bc8bd, 33 },
    { 0xd25bc8bf, 99 },
    { 0, 0 },
    { 0x6fbc8c1, 61 },
    { 0x770bc8c2, 75 },
    { 0, 0 },
    { 0xb33bc8c4, 61 },
    { 0, 0 },
    { 0x7b0bc8c6, 23 },
    { 0x58bc8c6, 77 },
    { 0, 0 },
    { 0x6ccbc8c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc8d1, 90 },
    { 0x871bc8d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bc8d5, 61 },
    { 0, 0 },
    { 0x12bbc8d7, 78 },
    { 0, 0 },
    { 0x629bc8d9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bc8e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6bc8e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9bc8e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385bc8f4, 61 },
    { 0xb0abc8f5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625bc8fa, 33 },
    { 0, 0 },
    { 0xa5cbc8fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bc900, 99 },
    { 0x482bc901, 95 },
    { 0, 0 },
    { 0x601bc903, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbc90d, 85 },
    { 0xd5ebc90d, 34 },
    { 0x648bc90e, 61 },
    { 0xf1fbc90d, 78 },
    { 0x20abc911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97bc917, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553bc91d, 61 },
    { 0x5a2bc91e, 33 },
    { 0xa92bc91f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebc927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbc92c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67bc931, 90 },
    { 0x1c2bc932, 81 },
    { 0x299bc931, 78 },
    { 0x57ebc934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc938, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x178bc93b, 46 },
    { 0x243bc93c, 75 },
    { 0xaecbc93d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ebc942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbc947, 61 },
    { 0x4e8bc948, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bbc94e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3bc951, 66 },
    { 0xf2ebc952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bbc95b, 90 },
    { 0xd5cbc95c, 23 },
    { 0x38abc95b, 98 },
    { 0xdbabc95e, 76 },
    { 0xb45bc95f, 99 },
    { 0xd08bc960, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ebc966, 100 },
    { 0, 0 },
    { 0x3c2bc968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541bc96c, 76 },
    { 0, 0 },
    { 0x234bc96e, 98 },
    { 0xdd7bc96f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41bc972, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bbc975, 46 },
    { 0, 0 },
    { 0xd32bc977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bc97d, 23 },
    { 0xa3bbc97e, 95 },
    { 0xe0abc97f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24bc986, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbc989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5abc98c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1bc991, 61 },
    { 0xab8bc991, 95 },
    { 0, 0 },
    { 0x2debc994, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc99a, 33 },
    { 0, 0 },
    { 0xbaabc99c, 98 },
    { 0xb3bbc99d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc9a0, 33 },
    { 0x1afbc9a1, 76 },
    { 0xe6bbc9a1, 43 },
    { 0xf4bbc9a3, 85 },
    { 0x324bc9a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afbc9a7, 61 },
    { 0xd1cbc9a8, 43 },
    { 0, 0 },
    { 0x572bc9aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06bc9ad, 33 },
    { 0x3c0bc9ae, 46 },
    { 0xb98bc9af, 33 },
    { 0x781bc9b0, 90 },
    { 0x76dbc9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dbc9b4, 66 },
    { 0x8ebbc9b5, 33 },
    { 0xc8cbc9b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dbc9bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bc9bf, 98 },
    { 0xf1cbc9c0, 52 },
    { 0, 0 },
    { 0x545bc9c2, 98 },
    { 0x67cbc9c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ebc9cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adbc9d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc9d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc9d8, 65 },
    { 0x48bc9d9, 95 },
    { 0xec3bc9da, 78 },
    { 0xebebc9da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490bc9df, 61 },
    { 0x40abc9e0, 88 },
    { 0x3d9bc9e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bc9eb, 75 },
    { 0, 0 },
    { 0x1afbc9ed, 99 },
    { 0xe92bc9ee, 33 },
    { 0xce2bc9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8bc9f4, 55 },
    { 0x81cbc9f4, 65 },
    { 0xf3abc9f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16bc9fd, 55 },
    { 0, 0 },
    { 0xd27bc9ff, 34 },
    { 0, 0 },
    { 0x39abca01, 45 },
    { 0, 0 },
    { 0xd3fbca03, 65 },
    { 0x22bbca03, 23 },
    { 0x5e7bca05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bbca08, 33 },
    { 0, 0 },
    { 0x5cfbca0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cbca18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60bca1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11bca21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07bca25, 100 },
    { 0, 0 },
    { 0xabfbca27, 80 },
    { 0, 0 },
    { 0x791bca29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbca2d, 33 },
    { 0x748bca2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dbca34, 75 },
    { 0, 0 },
    { 0xe5cbca36, 99 },
    { 0xb20bca37, 98 },
    { 0, 0 },
    { 0x46bca39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18bca3f, 100 },
    { 0, 0 },
    { 0x1b4bca41, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65bca45, 61 },
    { 0, 0 },
    { 0xdbbbca47, 43 },
    { 0, 0 },
    { 0xe9dbca49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9bca4c, 61 },
    { 0, 0 },
    { 0xeabca4e, 61 },
    { 0xb65bca4f, 46 },
    { 0xd56bca50, 38 },
    { 0x29cbca50, 33 },
    { 0xaa5bca52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bbca55, 93 },
    { 0xb5bca55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdbca5a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbbca5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415bca62, 33 },
    { 0, 0 },
    { 0x55abca64, 55 },
    { 0, 0 },
    { 0x174bca66, 93 },
    { 0x4b1bca66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7bca6e, 78 },
    { 0, 0 },
    { 0x7b4bca70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adbca74, 91 },
    { 0, 0 },
    { 0x450bca76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bca79, 98 },
    { 0x97ebca79, 38 },
    { 0, 0 },
    { 0xadabca7c, 88 },
    { 0, 0 },
    { 0x60dbca7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76bca81, 58 },
    { 0xeddbca82, 84 },
    { 0xbd9bca83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fbca86, 90 },
    { 0, 0 },
    { 0xb2bca88, 61 },
    { 0, 0 },
    { 0x1ebbca8a, 55 },
    { 0, 0 },
    { 0x5e7bca8c, 76 },
    { 0x878bca8d, 46 },
    { 0, 0 },
    { 0xf6ebca8f, 43 },
    { 0, 0 },
    { 0xb80bca91, 61 },
    { 0x869bca92, 84 },
    { 0xa76bca93, 90 },
    { 0xf0bca91, 95 },
    { 0xb2ebca95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2bca9b, 43 },
    { 0, 0 },
    { 0x24fbca9d, 93 },
    { 0xe19bca9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bbcaa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81bcaaa, 95 },
    { 0, 0 },
    { 0x9cebcaac, 34 },
    { 0, 0 },
    { 0xe54bcaae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742bcab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4bcabd, 33 },
    { 0x76abcabe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcac1, 95 },
    { 0, 0 },
    { 0xe91bcac3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x709bcac6, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510bcacc, 66 },
    { 0x87dbcacc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39abcad0, 98 },
    { 0xe59bcad0, 52 },
    { 0x781bcad2, 61 },
    { 0xe16bcad3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bbcad6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96bcadb, 61 },
    { 0xf0abcadc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7bcae5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434bcaed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bcaf2, 23 },
    { 0, 0 },
    { 0x497bcaf4, 43 },
    { 0x41bcaf5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2bcaf9, 99 },
    { 0, 0 },
    { 0x2bfbcafb, 33 },
    { 0xff6bcafc, 69 },
    { 0, 0 },
    { 0xa79bcafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1bcb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bbcb05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841bcb0b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x472bcb0e, 87 },
    { 0x81dbcb0f, 98 },
    { 0, 0 },
    { 0xbdfbcb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddbcb14, 76 },
    { 0, 0 },
    { 0xa82bcb16, 61 },
    { 0, 0 },
    { 0x4cbbcb18, 81 },
    { 0xd0ebcb18, 23 },
    { 0x6fbbcb18, 33 },
    { 0, 0 },
    { 0xb46bcb1c, 95 },
    { 0xb4bbcb1d, 99 },
    { 0x312bcb1d, 90 },
    { 0, 0 },
    { 0x3fdbcb20, 55 },
    { 0xefebcb21, 93 },
    { 0x453bcb21, 33 },
    { 0x9d9bcb22, 100 },
    { 0, 0 },
    { 0xef0bcb25, 33 },
    { 0x84ebcb26, 33 },
    { 0xbb4bcb26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x505bcb2a, 33 },
    { 0x315bcb2a, 33 },
    { 0, 0 },
    { 0xbd6bcb2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebcb30, 98 },
    { 0, 0 },
    { 0xf60bcb32, 76 },
    { 0xd1ebcb32, 66 },
    { 0, 0 },
    { 0x6eebcb35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cbcb39, 98 },
    { 0x92fbcb39, 17 },
    { 0x3ccbcb39, 91 },
    { 0xe87bcb3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cbcb41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ebcb49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edbcb53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774bcb59, 91 },
    { 0x8debcb5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdbcb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bcb66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207bcb6a, 90 },
    { 0, 0 },
    { 0xed9bcb6c, 43 },
    { 0, 0 },
    { 0xbb7bcb6e, 52 },
    { 0x9c7bcb6f, 33 },
    { 0, 0 },
    { 0x61cbcb71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bcb75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2bcb7d, 91 },
    { 0, 0 },
    { 0x821bcb7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438bcb85, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cbcb8a, 52 },
    { 0x320bcb8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7bcb91, 87 },
    { 0, 0 },
    { 0x65bbcb93, 55 },
    { 0x5c6bcb93, 95 },
    { 0, 0 },
    { 0x673bcb96, 23 },
    { 0, 0 },
    { 0xfd0bcb98, 90 },
    { 0x73bcb99, 66 },
    { 0x37cbcb9a, 80 },
    { 0xadfbcb9a, 23 },
    { 0, 0 },
    { 0x76cbcb9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08bcba6, 43 },
    { 0x13abcba7, 33 },
    { 0xe5fbcba7, 61 },
    { 0xd54bcba9, 76 },
    { 0x407bcba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ebcbaf, 78 },
    { 0, 0 },
    { 0xef6bcbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfbcbb6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ebcbbc, 55 },
    { 0x8afbcbbd, 83 },
    { 0, 0 },
    { 0x260bcbbf, 33 },
    { 0, 0 },
    { 0x2babcbc1, 43 },
    { 0xa91bcbc1, 90 },
    { 0x181bcbc3, 67 },
    { 0x240bcbc1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabcbca, 95 },
    { 0x7f3bcbca, 98 },
    { 0, 0 },
    { 0xc8bbcbcd, 66 },
    { 0xf60bcbce, 99 },
    { 0x716bcbcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bbcbd2, 61 },
    { 0x3ffbcbd2, 99 },
    { 0xec5bcbd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dbcbda, 61 },
    { 0x3ebcbdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7bcbe7, 17 },
    { 0x6eebcbe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0abcbeb, 23 },
    { 0x734bcbec, 76 },
    { 0x65cbcbec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7bcbf1, 99 },
    { 0, 0 },
    { 0xa57bcbf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9abcbfa, 90 },
    { 0, 0 },
    { 0xe80bcbfc, 33 },
    { 0, 0 },
    { 0x601bcbfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bcc01, 76 },
    { 0x776bcc02, 61 },
    { 0xfa2bcc02, 87 },
    { 0x995bcc04, 90 },
    { 0xdc2bcc05, 88 },
    { 0xa0cbcc05, 87 },
    { 0x603bcc01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bcc0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5bcc26, 78 },
    { 0x234bcc27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aebcc2c, 33 },
    { 0xc73bcc2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22bcc30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dbcc36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14bcc39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cebcc3d, 66 },
    { 0x4aabcc3e, 61 },
    { 0x201bcc3e, 90 },
    { 0xae8bcc3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51bcc46, 64 },
    { 0, 0 },
    { 0x6cdbcc48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fbcc4c, 98 },
    { 0x60ebcc4d, 61 },
    { 0x585bcc4e, 33 },
    { 0x388bcc4d, 68 },
    { 0x53dbcc50, 61 },
    { 0x3d7bcc4f, 43 },
    { 0, 0 },
    { 0x7a7bcc53, 55 },
    { 0xffebcc54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721bcc58, 61 },
    { 0xc99bcc59, 90 },
    { 0x337bcc59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2bcc63, 67 },
    { 0x6e8bcc63, 61 },
    { 0, 0 },
    { 0x532bcc66, 61 },
    { 0xde9bcc67, 33 },
    { 0x231bcc67, 90 },
    { 0, 0 },
    { 0xe3fbcc6a, 76 },
    { 0x4ebbcc6b, 88 },
    { 0, 0 },
    { 0x4c6bcc6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bcc71, 17 },
    { 0xf8bbcc72, 23 },
    { 0x147bcc73, 95 },
    { 0x6b9bcc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bcc77, 33 },
    { 0x3c4bcc77, 33 },
    { 0x516bcc77, 61 },
    { 0x54ebcc79, 61 },
    { 0, 0 },
    { 0x71bcc7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbbcc7f, 88 },
    { 0xd8ebcc80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fbcc84, 90 },
    { 0x897bcc85, 23 },
    { 0x442bcc86, 77 },
    { 0, 0 },
    { 0xbc0bcc88, 95 },
    { 0, 0 },
    { 0x74ebcc8a, 98 },
    { 0, 0 },
    { 0x97bcc8c, 36 },
    { 0xa4cbcc8d, 78 },
    { 0xcebbcc8e, 61 },
    { 0, 0 },
    { 0x128bcc90, 98 },
    { 0, 0 },
    { 0xa7bbcc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6bcc96, 61 },
    { 0, 0 },
    { 0xb9cbcc98, 90 },
    { 0x7d6bcc99, 98 },
    { 0x3b5bcc99, 85 },
    { 0, 0 },
    { 0x23bcc9c, 33 },
    { 0x40ebcc9c, 91 },
    { 0x5d8bcc9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcbcca4, 52 },
    { 0x62bcca5, 67 },
    { 0x673bcca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x954bcca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bccac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fbccb0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cbccb4, 95 },
    { 0x882bccb5, 50 },
    { 0x64abccb5, 61 },
    { 0x978bccb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8bccbc, 34 },
    { 0x29abccbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3bccc6, 23 },
    { 0x6fabccc7, 67 },
    { 0, 0 },
    { 0xcb2bccc9, 85 },
    { 0x34dbccca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2bccd4, 90 },
    { 0, 0 },
    { 0xe4ebccd6, 23 },
    { 0xcaabccd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dabcce0, 55 },
    { 0x2e2bcce0, 61 },
    { 0, 0 },
    { 0x60fbcce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677bcce8, 90 },
    { 0x991bcce9, 85 },
    { 0xafbbccea, 33 },
    { 0xe81bcceb, 33 },
    { 0, 0 },
    { 0x650bcced, 90 },
    { 0, 0 },
    { 0x3febccef, 61 },
    { 0, 0 },
    { 0x3debccf1, 66 },
    { 0, 0 },
    { 0xd87bccf3, 99 },
    { 0x2ecbccf3, 34 },
    { 0xc05bccf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2bcd01, 67 },
    { 0x35bcd02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524bcd05, 33 },
    { 0xaa3bcd06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81bcd0d, 33 },
    { 0x830bcd0e, 23 },
    { 0x9eabcd0d, 33 },
    { 0, 0 },
    { 0xca9bcd11, 93 },
    { 0x497bcd11, 76 },
    { 0xd1abcd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4bcd1b, 77 },
    { 0xb0ebcd1c, 38 },
    { 0x9c6bcd1d, 52 },
    { 0xcd6bcd1d, 33 },
    { 0x128bcd1f, 43 },
    { 0x775bcd1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127bcd2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5bcd2e, 95 },
    { 0x476bcd2e, 68 },
    { 0x9c6bcd30, 61 },
    { 0x299bcd31, 61 },
    { 0x22cbcd30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ebcd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0bcd44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97bcd47, 40 },
    { 0xb8ebcd48, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dbcd4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1bcd52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bbcd55, 100 },
    { 0x735bcd56, 55 },
    { 0, 0 },
    { 0x7a1bcd58, 95 },
    { 0, 0 },
    { 0x58ebcd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cbcd5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117bcd65, 88 },
    { 0x2bcd65, 66 },
    { 0xa84bcd67, 61 },
    { 0x5c9bcd67, 33 },
    { 0xf39bcd67, 33 },
    { 0, 0 },
    { 0xe6abcd6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173bcd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x870bcd72, 52 },
    { 0xb4abcd72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x571bcd76, 88 },
    { 0xe10bcd76, 93 },
    { 0x351bcd77, 81 },
    { 0, 0 },
    { 0x204bcd7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689bcd7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefbcd84, 23 },
    { 0, 0 },
    { 0xe59bcd86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0bcd8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bcd8f, 55 },
    { 0xbe9bcd90, 78 },
    { 0x767bcd91, 93 },
    { 0x849bcd91, 84 },
    { 0xae9bcd92, 76 },
    { 0x146bcd93, 61 },
    { 0xb3abcd92, 33 },
    { 0xf99bcd96, 61 },
    { 0x559bcd97, 90 },
    { 0x2edbcd98, 46 },
    { 0x3febcd96, 95 },
    { 0x777bcd92, 46 },
    { 0xbdcbcd9b, 23 },
    { 0xabbbcd9c, 33 },
    { 0x5d9bcd9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x597bcda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1abcdb9, 33 },
    { 0, 0 },
    { 0x269bcdbb, 23 },
    { 0x7b4bcdbb, 98 },
    { 0x182bcdbd, 33 },
    { 0, 0 },
    { 0xe31bcdbf, 34 },
    { 0xaffbcdbf, 98 },
    { 0xdcbcdc1, 52 },
    { 0x4aabcdbf, 34 },
    { 0x91ebcdc3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbcdc6, 37 },
    { 0, 0 },
    { 0xde0bcdc8, 90 },
    { 0xa97bcdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbcdcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2bcdd1, 52 },
    { 0xd3bcdd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcdd7, 68 },
    { 0x14dbcdd7, 90 },
    { 0, 0 },
    { 0xd19bcdda, 95 },
    { 0xae1bcdda, 100 },
    { 0, 0 },
    { 0x65ebcddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bcde0, 99 },
    { 0xe0bbcde1, 98 },
    { 0x54ebcde1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bcde5, 43 },
    { 0xab7bcde6, 90 },
    { 0xf37bcde7, 52 },
    { 0, 0 },
    { 0x33abcde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cbcdfb, 83 },
    { 0xfcfbcdfb, 95 },
    { 0x77bbcdfb, 43 },
    { 0x1c1bcdfc, 95 },
    { 0xac8bcdfe, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabce05, 93 },
    { 0x422bce06, 90 },
    { 0x54ebce06, 90 },
    { 0x3b3bce08, 95 },
    { 0xf5bce07, 80 },
    { 0x1e8bce09, 33 },
    { 0x4c6bce07, 93 },
    { 0x94bce0c, 23 },
    { 0xaa3bce09, 100 },
    { 0xccbbce0e, 100 },
    { 0, 0 },
    { 0x2dabce10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104bce16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3bce19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413bce1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcbce27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bce2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bbce2d, 83 },
    { 0xdedbce2d, 28 },
    { 0, 0 },
    { 0x1a6bce30, 52 },
    { 0x62abce30, 61 },
    { 0x58dbce30, 61 },
    { 0x36fbce33, 43 },
    { 0xef1bce33, 33 },
    { 0x503bce30, 98 },
    { 0x808bce36, 46 },
    { 0x298bce36, 17 },
    { 0, 0 },
    { 0x5fabce39, 93 },
    { 0, 0 },
    { 0x885bce3b, 33 },
    { 0x272bce3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bce42, 33 },
    { 0, 0 },
    { 0x9adbce44, 95 },
    { 0xa4cbce45, 23 },
    { 0xbe4bce44, 61 },
    { 0, 0 },
    { 0x7eebce48, 33 },
    { 0x864bce49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ebce4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0bce52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afbce55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbce5a, 43 },
    { 0, 0 },
    { 0x9febce5c, 23 },
    { 0x204bce5d, 23 },
    { 0x964bce5e, 33 },
    { 0x583bce5c, 98 },
    { 0x91ebce5d, 87 },
    { 0x7bdbce61, 84 },
    { 0x4c0bce5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739bce6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75bce71, 90 },
    { 0x805bce72, 98 },
    { 0xa21bce73, 98 },
    { 0x1d4bce73, 34 },
    { 0xd93bce75, 83 },
    { 0x602bce72, 33 },
    { 0x609bce77, 90 },
    { 0, 0 },
    { 0xd8ebce79, 52 },
    { 0x607bce7a, 34 },
    { 0x633bce7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x707bce7e, 84 },
    { 0, 0 },
    { 0x8b2bce80, 90 },
    { 0, 0 },
    { 0xc4bbce82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583bce86, 23 },
    { 0x94bce86, 95 },
    { 0x2ddbce87, 61 },
    { 0xb8ebce88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53bce8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bce91, 93 },
    { 0, 0 },
    { 0x831bce93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x266bce96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adbce9b, 91 },
    { 0xd9fbce9c, 61 },
    { 0xffebce9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92bcea0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485bcea5, 90 },
    { 0xf63bcea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451bceab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bbceae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5bceb2, 98 },
    { 0xb7fbceb3, 90 },
    { 0x37abceb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x328bceb7, 33 },
    { 0x42cbceb7, 33 },
    { 0, 0 },
    { 0x177bceba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccbcebd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40abcec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9bced0, 90 },
    { 0xe95bced0, 66 },
    { 0, 0 },
    { 0xc98bced3, 61 },
    { 0, 0 },
    { 0xdebced5, 75 },
    { 0xb8abced6, 61 },
    { 0x9abced7, 33 },
    { 0x86abced7, 98 },
    { 0, 0 },
    { 0xa8dbceda, 95 },
    { 0xdeebceda, 61 },
    { 0, 0 },
    { 0x5a2bcedd, 55 },
    { 0x24ebcede, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bcee7, 34 },
    { 0, 0 },
    { 0xcc4bcee9, 76 },
    { 0x71abceea, 61 },
    { 0x548bceea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdbcef0, 93 },
    { 0xb9fbcef1, 55 },
    { 0x1c5bcef0, 99 },
    { 0, 0 },
    { 0x1f6bcef4, 98 },
    { 0, 0 },
    { 0x514bcef6, 90 },
    { 0x326bcef7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913bcefb, 61 },
    { 0xdfabcefb, 98 },
    { 0x99abcefd, 33 },
    { 0x4cbcefe, 95 },
    { 0xe67bceff, 95 },
    { 0, 0 },
    { 0x45dbcf01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9bcf06, 81 },
    { 0, 0 },
    { 0x2cfbcf08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bcf0b, 83 },
    { 0x3e1bcf0b, 95 },
    { 0xfd0bcf0d, 23 },
    { 0xc29bcf0b, 43 },
    { 0x698bcf0f, 90 },
    { 0x506bcf0e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3abcf16, 87 },
    { 0xac0bcf17, 33 },
    { 0xba5bcf17, 56 },
    { 0xa7ebcf19, 90 },
    { 0x219bcf16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479bcf21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676bcf26, 98 },
    { 0, 0 },
    { 0xfebbcf28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebcf2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ebcf2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0bcf32, 46 },
    { 0, 0 },
    { 0xe66bcf34, 69 },
    { 0, 0 },
    { 0x51dbcf36, 81 },
    { 0x9d6bcf36, 23 },
    { 0xeadbcf38, 33 },
    { 0xfaabcf37, 23 },
    { 0, 0 },
    { 0x62ebcf3b, 98 },
    { 0, 0 },
    { 0x89bcf3d, 79 },
    { 0x44bcf3d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cbcf43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2bcf4a, 17 },
    { 0x255bcf4b, 75 },
    { 0x1dabcf4c, 33 },
    { 0x564bcf4c, 95 },
    { 0xb93bcf4e, 43 },
    { 0xd8fbcf4f, 61 },
    { 0, 0 },
    { 0xc1bbcf51, 33 },
    { 0, 0 },
    { 0xebfbcf53, 33 },
    { 0, 0 },
    { 0x189bcf55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36bcf5a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bcf5e, 23 },
    { 0xcb3bcf5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbcf63, 76 },
    { 0xe0dbcf64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837bcf68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43bcf6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12bcf71, 91 },
    { 0, 0 },
    { 0x949bcf73, 85 },
    { 0, 0 },
    { 0xf40bcf75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ebcf7a, 43 },
    { 0x61cbcf7b, 17 },
    { 0x144bcf7b, 93 },
    { 0x359bcf7d, 33 },
    { 0, 0 },
    { 0xd03bcf7f, 90 },
    { 0, 0 },
    { 0x499bcf81, 23 },
    { 0xe66bcf82, 34 },
    { 0xf76bcf83, 33 },
    { 0x327bcf82, 95 },
    { 0xe80bcf85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12bcf8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbcf93, 52 },
    { 0, 0 },
    { 0xe76bcf95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbcf98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38bcf9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0bcfa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918bcfa7, 55 },
    { 0x428bcfa7, 93 },
    { 0, 0 },
    { 0x49ebcfaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3bcfb3, 99 },
    { 0x141bcfb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59bcfb8, 75 },
    { 0x254bcfb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bbcfbc, 66 },
    { 0, 0 },
    { 0xe02bcfbe, 45 },
    { 0x62ebcfbf, 93 },
    { 0xbfbcfc0, 90 },
    { 0x913bcfbe, 33 },
    { 0x6edbcfbe, 95 },
    { 0, 0 },
    { 0x765bcfc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abbcfd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135bcfda, 46 },
    { 0, 0 },
    { 0x5dfbcfdc, 33 },
    { 0x7efbcfdc, 61 },
    { 0x737bcfdc, 61 },
    { 0xdaebcfdf, 61 },
    { 0xb33bcfe0, 7 },
    { 0, 0 },
    { 0xf96bcfe2, 98 },
    { 0x707bcfe3, 23 },
    { 0x779bcfe4, 66 },
    { 0x444bcfe5, 46 },
    { 0x2c3bcfe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebcfe9, 61 },
    { 0xf48bcfe9, 64 },
    { 0x3cdbcfeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cbcff1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13bcff5, 55 },
    { 0x46bcff6, 33 },
    { 0, 0 },
    { 0xfbabcff8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9bcffc, 61 },
    { 0x11bbcffd, 34 },
    { 0, 0 },
    { 0x229bcfff, 23 },
    { 0xd03bd000, 90 },
    { 0x615bd000, 99 },
    { 0, 0 },
    { 0x9c8bd003, 78 },
    { 0, 0 },
    { 0x7c1bd005, 90 },
    { 0xc5dbd006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5debd00c, 98 },
    { 0x669bd00d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ebd010, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edbd020, 80 },
    { 0x1bebd021, 43 },
    { 0x47dbd022, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52bd029, 95 },
    { 0xf00bd02a, 84 },
    { 0xad1bd02b, 38 },
    { 0xeffbd02b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125bd030, 52 },
    { 0x17abd031, 33 },
    { 0xccfbd032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88bd03b, 66 },
    { 0x1b3bd03c, 33 },
    { 0x82bbd03c, 55 },
    { 0x794bd03e, 88 },
    { 0xfdcbd03e, 90 },
    { 0xa72bd040, 76 },
    { 0xa1abd03f, 55 },
    { 0xd48bd040, 66 },
    { 0xd6dbd043, 33 },
    { 0x4bebd03b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396bd04b, 43 },
    { 0xaf1bd04c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16bd051, 98 },
    { 0x960bd052, 61 },
    { 0x2fbbd052, 87 },
    { 0x74cbd054, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x276bd057, 29 },
    { 0xf7dbd058, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725bd062, 88 },
    { 0xdc7bd063, 33 },
    { 0x62dbd064, 92 },
    { 0x66bbd063, 75 },
    { 0, 0 },
    { 0x4fdbd067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8bd06b, 43 },
    { 0xf98bd06c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06bd070, 77 },
    { 0, 0 },
    { 0x97abd072, 33 },
    { 0x4f4bd072, 90 },
    { 0xc05bd074, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9bd079, 17 },
    { 0xe45bd07a, 91 },
    { 0xe65bd07a, 95 },
    { 0x36dbd07c, 23 },
    { 0x1fbbd07d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fbd080, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbd089, 67 },
    { 0x356bd08a, 76 },
    { 0xea8bd089, 61 },
    { 0xc01bd08c, 61 },
    { 0xe54bd089, 33 },
    { 0x7a0bd08e, 76 },
    { 0x33bbd08f, 78 },
    { 0xa1dbd090, 55 },
    { 0x270bd08a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ebd096, 34 },
    { 0x4febd097, 46 },
    { 0x4a7bd098, 23 },
    { 0, 0 },
    { 0x442bd09a, 90 },
    { 0xaffbd09b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7bd0a4, 90 },
    { 0x8bebd0a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd0af, 80 },
    { 0x191bd0b0, 90 },
    { 0x8bcbd0b1, 87 },
    { 0x67bbd0b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfbd0b8, 33 },
    { 0x602bd0b8, 33 },
    { 0x5efbd0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435bd0c0, 77 },
    { 0x454bd0c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dbd0c6, 69 },
    { 0x34bd0c7, 55 },
    { 0xcebd0c6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615bd0cd, 34 },
    { 0x33dbd0ce, 23 },
    { 0, 0 },
    { 0x5b0bd0d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19abd0d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebd0df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988bd0e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbd0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x606bd0eb, 33 },
    { 0xd7fbd0ec, 17 },
    { 0, 0 },
    { 0x81bbd0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadbd0f1, 25 },
    { 0x1d5bd0f2, 93 },
    { 0x5c1bd0f3, 61 },
    { 0xbd0bd0f3, 33 },
    { 0xc09bd0f3, 91 },
    { 0xc1bd0f3, 76 },
    { 0x4ddbd0f7, 90 },
    { 0, 0 },
    { 0x9e7bd0f9, 23 },
    { 0x43cbd0f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16bd0fd, 95 },
    { 0, 0 },
    { 0x194bd0ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddbd103, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bd106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537bd109, 55 },
    { 0, 0 },
    { 0xe91bd10b, 34 },
    { 0, 0 },
    { 0x837bd10d, 90 },
    { 0, 0 },
    { 0x9d7bd10f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805bd115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0bd118, 34 },
    { 0xd08bd119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649bd121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2bd124, 23 },
    { 0xa9fbd125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10bd12e, 90 },
    { 0x984bd12e, 78 },
    { 0xf15bd130, 80 },
    { 0xce1bd130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233bd138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bd13e, 87 },
    { 0x490bd13e, 61 },
    { 0xe6fbd13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4bd143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4bd14d, 90 },
    { 0, 0 },
    { 0x828bd14f, 55 },
    { 0x96dbd150, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x935bd153, 33 },
    { 0, 0 },
    { 0x34bd155, 52 },
    { 0, 0 },
    { 0x576bd157, 23 },
    { 0x55dbd157, 34 },
    { 0xae8bd159, 33 },
    { 0xd68bd15a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bbd165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58bd16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960bd170, 33 },
    { 0, 0 },
    { 0xeaabd172, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bd178, 93 },
    { 0, 0 },
    { 0x1fbd17a, 33 },
    { 0xa67bd17a, 84 },
    { 0xbbbbd17c, 95 },
    { 0xc27bd17a, 90 },
    { 0, 0 },
    { 0x159bd17f, 90 },
    { 0x3c4bd180, 90 },
    { 0, 0 },
    { 0x726bd182, 95 },
    { 0x8d5bd183, 95 },
    { 0xeebbd183, 98 },
    { 0x4e6bd184, 23 },
    { 0x148bd186, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6bd18a, 61 },
    { 0, 0 },
    { 0x6fcbd18c, 68 },
    { 0x899bd18c, 55 },
    { 0x9fbbd18e, 87 },
    { 0x69bd18f, 100 },
    { 0x73bbd18f, 90 },
    { 0, 0 },
    { 0x589bd192, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebd197, 98 },
    { 0, 0 },
    { 0x4c1bd199, 99 },
    { 0x276bd19a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3bd19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x751bd1a0, 77 },
    { 0x786bd1a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596bd1a6, 23 },
    { 0x1e2bd1a7, 23 },
    { 0, 0 },
    { 0xb4bbd1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5bd1af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47bd1b3, 23 },
    { 0x938bd1b4, 34 },
    { 0, 0 },
    { 0x285bd1b6, 75 },
    { 0x2e2bd1b7, 23 },
    { 0, 0 },
    { 0xa67bd1b9, 98 },
    { 0xd15bd1ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54bd1c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868bd1c6, 93 },
    { 0, 0 },
    { 0x9debd1c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebd1cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebd1d4, 61 },
    { 0x8e8bd1d5, 76 },
    { 0xd3dbd1d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cbd1de, 17 },
    { 0x464bd1de, 100 },
    { 0, 0 },
    { 0xa32bd1e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ebd1ed, 61 },
    { 0, 0 },
    { 0x9f1bd1ef, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7bd1f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eebd1f8, 98 },
    { 0, 0 },
    { 0x748bd1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bbd205, 87 },
    { 0, 0 },
    { 0x9cbbd207, 34 },
    { 0xee5bd208, 61 },
    { 0xd6abd209, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37bd20d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983bd213, 98 },
    { 0xa7dbd214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bbd219, 33 },
    { 0, 0 },
    { 0x912bd21b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1bd223, 91 },
    { 0x900bd223, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbd227, 87 },
    { 0xcd0bd228, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2bd22d, 76 },
    { 0x638bd22e, 55 },
    { 0xe6dbd22f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1bd233, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59bd236, 23 },
    { 0x3edbd237, 95 },
    { 0xf20bd238, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cebd23b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbd240, 33 },
    { 0, 0 },
    { 0xc04bd242, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bd245, 98 },
    { 0xa67bd245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbd249, 98 },
    { 0, 0 },
    { 0xabbbd24b, 100 },
    { 0x7b9bd24c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80abd252, 66 },
    { 0x575bd253, 91 },
    { 0, 0 },
    { 0xee3bd255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505bd259, 34 },
    { 0x631bd25a, 17 },
    { 0x173bd25b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x794bd25e, 33 },
    { 0xaeabd25f, 88 },
    { 0, 0 },
    { 0x4cebd261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112bd265, 55 },
    { 0x25bd266, 99 },
    { 0xf6bd267, 17 },
    { 0x21bd267, 68 },
    { 0x469bd268, 66 },
    { 0x80dbd26a, 23 },
    { 0x896bd267, 65 },
    { 0x1e4bd269, 77 },
    { 0xb44bd26c, 34 },
    { 0x692bd26e, 23 },
    { 0x8c7bd26f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dbd272, 33 },
    { 0xe49bd273, 23 },
    { 0, 0 },
    { 0x8b2bd275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560bd27c, 66 },
    { 0, 0 },
    { 0x1cfbd27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bbd28a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92bd296, 46 },
    { 0x3debd296, 90 },
    { 0xf0bbd297, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efbd29b, 91 },
    { 0x8b6bd29c, 98 },
    { 0, 0 },
    { 0xe8bbd29e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1bd2a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbd2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28bd2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ebd2b1, 98 },
    { 0x378bd2b2, 90 },
    { 0xc77bd2b2, 33 },
    { 0, 0 },
    { 0xcfabd2b5, 95 },
    { 0x999bd2b6, 98 },
    { 0, 0 },
    { 0x925bd2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bd2bd, 34 },
    { 0xa3bbd2bd, 90 },
    { 0, 0 },
    { 0x88bd2c0, 23 },
    { 0x813bd2c1, 34 },
    { 0xcf4bd2c2, 43 },
    { 0xbc2bd2c2, 95 },
    { 0xf3abd2c4, 33 },
    { 0x503bd2c4, 93 },
    { 0x37cbd2c5, 33 },
    { 0xd0ebd2c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x777bd2ca, 76 },
    { 0x500bd2cb, 46 },
    { 0x523bd2cb, 90 },
    { 0x544bd2ca, 95 },
    { 0xd04bd2ce, 95 },
    { 0, 0 },
    { 0x28abd2d0, 90 },
    { 0, 0 },
    { 0x30abd2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dbd2d7, 46 },
    { 0xa12bd2d8, 33 },
    { 0, 0 },
    { 0x21abd2da, 46 },
    { 0x5c2bd2da, 46 },
    { 0x5e1bd2dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3bd2e0, 61 },
    { 0x9dcbd2e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcebd2e6, 95 },
    { 0xb17bd2e7, 43 },
    { 0, 0 },
    { 0x6f4bd2e9, 34 },
    { 0x3e8bd2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6bd2f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704bd2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcbd2fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1bd2fe, 95 },
    { 0xc7abd2ff, 52 },
    { 0xcb3bd2ff, 40 },
    { 0x6f7bd2ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3bd305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bbd308, 98 },
    { 0xb6cbd309, 68 },
    { 0xa82bd308, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cbd30d, 61 },
    { 0x73bbd30e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aebd317, 23 },
    { 0, 0 },
    { 0x1d5bd319, 100 },
    { 0xa0ebd31a, 95 },
    { 0x6a8bd31a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0bd31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6bd323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102bd32a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364bd332, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x600bd335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08bd340, 77 },
    { 0x631bd341, 23 },
    { 0xda8bd340, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10bd349, 90 },
    { 0, 0 },
    { 0xa43bd34b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62bd34e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524bd355, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27abd358, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3bd35b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd35f, 90 },
    { 0x49abd360, 33 },
    { 0xb33bd360, 95 },
    { 0x121bd361, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbd367, 98 },
    { 0xa73bd368, 61 },
    { 0xdcdbd368, 40 },
    { 0x8a0bd36a, 93 },
    { 0, 0 },
    { 0x707bd36c, 61 },
    { 0xee9bd36d, 23 },
    { 0xebbbd36e, 55 },
    { 0, 0 },
    { 0x9b9bd370, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9abd37a, 99 },
    { 0xe74bd37b, 60 },
    { 0x8babd37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2bd380, 90 },
    { 0x9f0bd381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522bd384, 90 },
    { 0xad2bd385, 33 },
    { 0xd8bbd385, 33 },
    { 0xc1fbd387, 43 },
    { 0x541bd387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71bd38b, 90 },
    { 0x407bd38c, 47 },
    { 0x234bd38c, 95 },
    { 0x4e7bd38e, 43 },
    { 0x18ebd38f, 34 },
    { 0xc2fbd390, 68 },
    { 0x92ebd38f, 76 },
    { 0, 0 },
    { 0x231bd393, 95 },
    { 0x164bd394, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x152bd397, 33 },
    { 0x4b4bd397, 61 },
    { 0xbd399, 43 },
    { 0, 0 },
    { 0x3fbbd39b, 99 },
    { 0xea9bd39c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddbd3a3, 98 },
    { 0xf0cbd3a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1babd3aa, 61 },
    { 0x9d9bd3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbd3b0, 76 },
    { 0x1b7bd3b1, 23 },
    { 0x868bd3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bbd3b6, 90 },
    { 0x57abd3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161bd3be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bd3c1, 43 },
    { 0, 0 },
    { 0x115bd3c3, 33 },
    { 0xc04bd3c4, 33 },
    { 0x126bd3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bd3cb, 98 },
    { 0x899bd3cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbd3d1, 91 },
    { 0x214bd3d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0bd3d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7bd3dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5bd3ea, 23 },
    { 0x708bd3eb, 98 },
    { 0x661bd3ec, 61 },
    { 0xffcbd3ed, 95 },
    { 0x65abd3ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efbd3f1, 90 },
    { 0, 0 },
    { 0x392bd3f3, 98 },
    { 0, 0 },
    { 0xb38bd3f5, 98 },
    { 0xf0bbd3f6, 55 },
    { 0xcf9bd3f7, 95 },
    { 0x7b3bd3f5, 34 },
    { 0xb00bd3f9, 98 },
    { 0xb46bd3f5, 33 },
    { 0, 0 },
    { 0xc8dbd3fc, 43 },
    { 0, 0 },
    { 0x9debd3fe, 61 },
    { 0x9ebd3ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4bd402, 55 },
    { 0, 0 },
    { 0xd75bd404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389bd408, 33 },
    { 0, 0 },
    { 0x8f4bd40a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5abd40e, 90 },
    { 0xdd2bd40f, 90 },
    { 0x485bd40f, 55 },
    { 0x8bfbd40e, 68 },
    { 0xfdebd412, 90 },
    { 0, 0 },
    { 0x137bd414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbd41e, 90 },
    { 0, 0 },
    { 0x354bd420, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696bd426, 33 },
    { 0xec0bd427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd42f, 90 },
    { 0, 0 },
    { 0xb88bd431, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90bd439, 46 },
    { 0xd38bd439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fbd43d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x341bd440, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bd446, 55 },
    { 0, 0 },
    { 0x301bd448, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13bd457, 46 },
    { 0x4abbd457, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68bd460, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bd463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70abd467, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x610bd46a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebd46e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaabd474, 33 },
    { 0, 0 },
    { 0xd50bd476, 100 },
    { 0xb9dbd477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbd47e, 61 },
    { 0xac0bd47f, 23 },
    { 0x25bbd47f, 90 },
    { 0, 0 },
    { 0xb9fbd482, 33 },
    { 0xd3bd483, 76 },
    { 0xcb8bd484, 95 },
    { 0x14ebd485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccbd494, 90 },
    { 0x2e0bd495, 76 },
    { 0x3e5bd496, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51bd499, 52 },
    { 0x2afbd49a, 32 },
    { 0x803bd499, 98 },
    { 0xa6ebd49c, 61 },
    { 0xfdbd499, 99 },
    { 0x231bd49e, 33 },
    { 0x105bd49f, 75 },
    { 0xb9bd499, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x901bd4a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bd4a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28bd4a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9abd4ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912bd4b8, 90 },
    { 0x6cbbd4b8, 100 },
    { 0xa44bd4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabd4c2, 40 },
    { 0x23cbd4c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1bd4c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5bd4cc, 91 },
    { 0xdd3bd4cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bd4d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cbd4d4, 23 },
    { 0x801bd4d5, 61 },
    { 0xf62bd4d5, 90 },
    { 0xbaebd4d7, 61 },
    { 0xee3bd4d5, 66 },
    { 0xe0cbd4d9, 61 },
    { 0x52ebd4da, 33 },
    { 0x553bd4db, 98 },
    { 0x6ebd4d9, 66 },
    { 0x6fbbd4dd, 91 },
    { 0x30abd4de, 87 },
    { 0x676bd4df, 76 },
    { 0xe96bd4df, 76 },
    { 0xf12bd4e1, 90 },
    { 0, 0 },
    { 0x630bd4e3, 61 },
    { 0, 0 },
    { 0xaf5bd4e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4bd4e8, 91 },
    { 0x971bd4e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x944bd4ec, 33 },
    { 0x46bd4ed, 33 },
    { 0xdf8bd4ee, 90 },
    { 0x51fbd4ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbd4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dbd4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd4f8, 88 },
    { 0, 0 },
    { 0x2f5bd4fa, 23 },
    { 0, 0 },
    { 0x7f7bd4fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49bd502, 23 },
    { 0, 0 },
    { 0x744bd504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bbd507, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49abd50d, 95 },
    { 0xc51bd50e, 91 },
    { 0x467bd50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bd516, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dbd519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3bd51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19bd524, 33 },
    { 0, 0 },
    { 0xc6dbd526, 23 },
    { 0x322bd526, 95 },
    { 0xe5abd527, 66 },
    { 0x743bd526, 93 },
    { 0x7e8bd526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23bd52d, 23 },
    { 0, 0 },
    { 0xa87bd52f, 84 },
    { 0, 0 },
    { 0xe89bd531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6bd535, 33 },
    { 0xc71bd536, 96 },
    { 0, 0 },
    { 0xad6bd538, 90 },
    { 0, 0 },
    { 0x521bd53a, 52 },
    { 0x1a2bd53b, 43 },
    { 0x949bd53c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bd540, 34 },
    { 0x112bd540, 49 },
    { 0x16bd540, 78 },
    { 0xcc7bd542, 95 },
    { 0xdb9bd541, 99 },
    { 0x869bd545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbd549, 90 },
    { 0, 0 },
    { 0xf6ebd54b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77bd54e, 33 },
    { 0x11fbd54f, 78 },
    { 0, 0 },
    { 0x747bd551, 23 },
    { 0x9e3bd552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbd557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cbd55b, 52 },
    { 0x4eebd55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289bd561, 66 },
    { 0, 0 },
    { 0x693bd563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fbd56c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bd572, 91 },
    { 0x48dbd573, 55 },
    { 0xb87bd573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbd57b, 95 },
    { 0x86cbd57b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9abd581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ebd584, 88 },
    { 0, 0 },
    { 0x1b6bd586, 67 },
    { 0x574bd587, 55 },
    { 0, 0 },
    { 0xab3bd589, 77 },
    { 0x83abd589, 84 },
    { 0, 0 },
    { 0x8bd58c, 93 },
    { 0x32bbd58d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2bd593, 52 },
    { 0, 0 },
    { 0xeb0bd595, 93 },
    { 0x66bbd596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dbd599, 66 },
    { 0, 0 },
    { 0xc4ebd59b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaebd5a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cbd5a9, 76 },
    { 0x3dfbd5aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61bd5ae, 77 },
    { 0x723bd5ae, 95 },
    { 0x3c2bd5ae, 76 },
    { 0x980bd5b1, 81 },
    { 0xc7ebd5b2, 90 },
    { 0x571bd5b3, 84 },
    { 0xdbbbd5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191bd5ba, 90 },
    { 0xabebd5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500bd5c1, 99 },
    { 0xbe8bd5c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10bd5c6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1bd5d1, 34 },
    { 0x70bbd5d2, 33 },
    { 0x180bd5d3, 33 },
    { 0x2b4bd5d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dbd5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965bd5e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd5e4, 38 },
    { 0xb3ebd5e5, 43 },
    { 0xb90bd5e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57bd5e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13bd5f0, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30bd5f5, 72 },
    { 0x8e2bd5f6, 34 },
    { 0xbcbbd5f7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbd5fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5bd5fd, 65 },
    { 0x4e0bd5fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95bd603, 98 },
    { 0x4d5bd604, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5bd609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37bd60d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ebd610, 55 },
    { 0xc20bd611, 93 },
    { 0, 0 },
    { 0x4fbbd613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bd61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd61e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4bd622, 66 },
    { 0, 0 },
    { 0xd05bd624, 85 },
    { 0xf15bd625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261bd62a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd62f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353bd633, 95 },
    { 0x9d7bd634, 99 },
    { 0xe63bd634, 33 },
    { 0xe66bd636, 98 },
    { 0x2cabd633, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddbd63a, 81 },
    { 0xe8bd63b, 77 },
    { 0x4f0bd63b, 61 },
    { 0, 0 },
    { 0x118bd63e, 23 },
    { 0x1dcbd63f, 98 },
    { 0x49fbd640, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195bd64c, 90 },
    { 0xecabd64d, 33 },
    { 0xa9fbd64c, 85 },
    { 0xef8bd64f, 78 },
    { 0x11fbd64d, 93 },
    { 0xb48bd651, 61 },
    { 0x805bd651, 84 },
    { 0x3e8bd653, 76 },
    { 0x7debd654, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fbd659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abd65d, 23 },
    { 0, 0 },
    { 0x882bd65f, 23 },
    { 0x961bd65f, 76 },
    { 0, 0 },
    { 0xf53bd662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17bd665, 61 },
    { 0, 0 },
    { 0x3d3bd667, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbbd66b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0abd670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37bd675, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34bd678, 66 },
    { 0x989bd679, 61 },
    { 0, 0 },
    { 0x3d6bd67b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491bd682, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbd686, 76 },
    { 0xbc1bd687, 46 },
    { 0x86abd687, 99 },
    { 0x345bd688, 98 },
    { 0xa9ebd68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd68d, 44 },
    { 0xf98bd68e, 87 },
    { 0x4fbd68f, 99 },
    { 0xbe4bd68f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ebd693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67bd696, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x213bd699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdbd69c, 76 },
    { 0x871bd69d, 23 },
    { 0, 0 },
    { 0xd51bd69f, 100 },
    { 0x46fbd69f, 76 },
    { 0, 0 },
    { 0x826bd6a2, 76 },
    { 0xa7cbd6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fbd6a7, 61 },
    { 0xa25bd6a8, 33 },
    { 0, 0 },
    { 0x5d3bd6aa, 66 },
    { 0xd98bd6aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79bd6af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x438bd6b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x204bd6b5, 61 },
    { 0x3debd6b5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909bd6bc, 77 },
    { 0, 0 },
    { 0xf91bd6be, 61 },
    { 0, 0 },
    { 0x127bd6c0, 76 },
    { 0xa96bd6c1, 55 },
    { 0xfbd6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x244bd6c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bd6ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x474bd6d1, 76 },
    { 0, 0 },
    { 0xee3bd6d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bbd6da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ebd6de, 34 },
    { 0x5ebd6df, 67 },
    { 0, 0 },
    { 0xc43bd6e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cebd6eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9bd6ee, 23 },
    { 0, 0 },
    { 0x52cbd6f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abd6f7, 33 },
    { 0x370bd6f8, 84 },
    { 0xdafbd6f8, 34 },
    { 0x7c1bd6fa, 23 },
    { 0x71bd6fb, 91 },
    { 0xaedbd6fc, 55 },
    { 0x311bd6fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0bd701, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973bd708, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7bd710, 34 },
    { 0, 0 },
    { 0x68fbd712, 23 },
    { 0xb61bd713, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72bd716, 17 },
    { 0, 0 },
    { 0x8d3bd718, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85bd71c, 81 },
    { 0x14fbd71c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461bd723, 78 },
    { 0, 0 },
    { 0x16ebd725, 98 },
    { 0x3aebd725, 61 },
    { 0xbbbbd726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dbd72a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bd72d, 99 },
    { 0x8f0bd72d, 23 },
    { 0, 0 },
    { 0x661bd730, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0bd733, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5febd737, 34 },
    { 0xb56bd738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ebd73e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bd744, 98 },
    { 0x334bd745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43bd74b, 55 },
    { 0xddcbd74c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x140bd74f, 23 },
    { 0, 0 },
    { 0x828bd751, 90 },
    { 0xc01bd752, 88 },
    { 0, 0 },
    { 0xf74bd754, 85 },
    { 0x9f3bd755, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bd758, 95 },
    { 0, 0 },
    { 0x96bd75a, 90 },
    { 0x521bd75b, 98 },
    { 0, 0 },
    { 0xd25bd75d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2bd764, 95 },
    { 0, 0 },
    { 0x74ebd766, 87 },
    { 0, 0 },
    { 0xe0abd768, 76 },
    { 0, 0 },
    { 0xd83bd76a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950bd770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adbd776, 90 },
    { 0, 0 },
    { 0xe3ebd778, 78 },
    { 0, 0 },
    { 0xc93bd77a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10bd780, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39bd783, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9bd788, 55 },
    { 0x28fbd789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124bd791, 46 },
    { 0xb4ebd792, 99 },
    { 0xfb2bd793, 95 },
    { 0xc84bd793, 33 },
    { 0x660bd795, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cbd799, 43 },
    { 0, 0 },
    { 0xa6fbd79b, 23 },
    { 0x560bd79b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bd7ac, 33 },
    { 0x6bbbd7ad, 43 },
    { 0, 0 },
    { 0xcfcbd7af, 78 },
    { 0, 0 },
    { 0x4bcbd7b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6bd7b4, 93 },
    { 0xd21bd7b5, 43 },
    { 0x143bd7b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9bd7c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9bd7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838bd7ca, 98 },
    { 0xe3cbd7ca, 90 },
    { 0, 0 },
    { 0xab2bd7cd, 77 },
    { 0x4d2bd7ce, 61 },
    { 0xea0bd7cd, 95 },
    { 0xd1ebd7cd, 87 },
    { 0xbb7bd7ce, 24 },
    { 0xbe3bd7d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bebd7d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dbd7dc, 43 },
    { 0xbf6bd7dd, 78 },
    { 0, 0 },
    { 0xef4bd7df, 66 },
    { 0x27fbd7e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0bd7e4, 90 },
    { 0x2e1bd7e4, 95 },
    { 0x100bd7e6, 88 },
    { 0x44fbd7e6, 43 },
    { 0xdafbd7e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86bd7eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761bd7ef, 99 },
    { 0x324bd7f0, 64 },
    { 0, 0 },
    { 0x4c9bd7f2, 40 },
    { 0x3bebd7f2, 46 },
    { 0x82bbd7f2, 23 },
    { 0x247bd7f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bd806, 46 },
    { 0, 0 },
    { 0xc17bd808, 90 },
    { 0x433bd809, 61 },
    { 0xfd9bd80a, 33 },
    { 0x6bd809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49bd811, 95 },
    { 0, 0 },
    { 0x8a9bd813, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bbd817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8bd81c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x882bd81f, 67 },
    { 0xe1abd81f, 87 },
    { 0x454bd821, 90 },
    { 0x7cbbd822, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602bd826, 52 },
    { 0xa76bd827, 19 },
    { 0, 0 },
    { 0x7bcbd829, 23 },
    { 0x29ebd82a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12abd830, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x971bd833, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bd837, 23 },
    { 0xd61bd837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccbd83f, 61 },
    { 0, 0 },
    { 0xa23bd841, 33 },
    { 0x2bd842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297bd846, 98 },
    { 0, 0 },
    { 0xa10bd848, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cbd84b, 92 },
    { 0x8adbd84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dbd850, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4bd855, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x332bd858, 61 },
    { 0x614bd858, 95 },
    { 0, 0 },
    { 0x15cbd85b, 98 },
    { 0, 0 },
    { 0x211bd85d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fbd861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564bd865, 33 },
    { 0x526bd866, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5bd869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bd86e, 90 },
    { 0x328bd86f, 61 },
    { 0x14cbd86e, 55 },
    { 0xad7bd871, 55 },
    { 0x7cebd872, 90 },
    { 0x28ebd86e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bd879, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaabd87c, 90 },
    { 0, 0 },
    { 0x4bbd87e, 76 },
    { 0, 0 },
    { 0x627bd880, 66 },
    { 0x84fbd881, 33 },
    { 0xed1bd881, 85 },
    { 0x7c1bd883, 78 },
    { 0, 0 },
    { 0x26dbd885, 66 },
    { 0xdabd885, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bbd88d, 98 },
    { 0, 0 },
    { 0x598bd88f, 61 },
    { 0x7dbbd890, 90 },
    { 0x4e0bd890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fbd896, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1bd899, 43 },
    { 0x219bd89a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbbd89d, 95 },
    { 0x3a7bd89e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bd8a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cbd8af, 91 },
    { 0, 0 },
    { 0x18fbd8b1, 17 },
    { 0xe7bbd8b2, 23 },
    { 0xc79bd8b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bd8b6, 59 },
    { 0, 0 },
    { 0x5fabd8b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecbd8c0, 90 },
    { 0xe6abd8c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbd8c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbd8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909bd8d9, 33 },
    { 0x97dbd8d9, 75 },
    { 0x4f3bd8db, 23 },
    { 0x3b3bd8db, 90 },
    { 0xbbcbd8dc, 84 },
    { 0x134bd8de, 33 },
    { 0x2e4bd8dd, 95 },
    { 0xed6bd8e0, 90 },
    { 0x4b0bd8de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58bd8e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcbd8ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bbd8f1, 100 },
    { 0xe0fbd8f2, 33 },
    { 0xc63bd8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8bd8f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89abd8f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x836bd8fc, 33 },
    { 0, 0 },
    { 0x467bd8fe, 87 },
    { 0xa5fbd8fe, 90 },
    { 0x1b5bd900, 76 },
    { 0, 0 },
    { 0x37bbd902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bd906, 96 },
    { 0xfe5bd907, 61 },
    { 0, 0 },
    { 0x88bd909, 76 },
    { 0xf16bd909, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8bd90e, 76 },
    { 0, 0 },
    { 0x26bbd910, 33 },
    { 0, 0 },
    { 0xa00bd912, 78 },
    { 0, 0 },
    { 0x358bd914, 90 },
    { 0xec7bd915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5bd919, 61 },
    { 0x72abd91a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12bd91f, 56 },
    { 0xf3fbd920, 92 },
    { 0x448bd91f, 95 },
    { 0xda2bd922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adbd925, 36 },
    { 0xdfcbd925, 98 },
    { 0x60abd926, 95 },
    { 0xb2cbd927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656bd92c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3bd932, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cbd939, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cbd93e, 100 },
    { 0x22ebd93e, 33 },
    { 0xb99bd940, 95 },
    { 0x8dbbd940, 61 },
    { 0, 0 },
    { 0x63bbd943, 23 },
    { 0, 0 },
    { 0x82dbd945, 100 },
    { 0xf9abd946, 90 },
    { 0, 0 },
    { 0xe9ebd948, 33 },
    { 0x4acbd948, 23 },
    { 0xe22bd94a, 93 },
    { 0xa9ebd94b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd956, 95 },
    { 0xb14bd957, 76 },
    { 0x2b9bd958, 90 },
    { 0x2f5bd959, 90 },
    { 0, 0 },
    { 0x83fbd95b, 67 },
    { 0x72abd95c, 90 },
    { 0x138bd95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4bd961, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0bd965, 33 },
    { 0x4dabd966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bd96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763bd970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbbd973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbd979, 94 },
    { 0xc68bd97a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2bd980, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbd984, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215bd98a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88bd98d, 64 },
    { 0, 0 },
    { 0x80bd98f, 47 },
    { 0xcc5bd990, 90 },
    { 0xc14bd991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322bd996, 17 },
    { 0, 0 },
    { 0xc2dbd998, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cbd99d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bd9a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4bd9a9, 23 },
    { 0, 0 },
    { 0xe94bd9ab, 55 },
    { 0x86dbd9ac, 98 },
    { 0xd6cbd9ad, 23 },
    { 0x259bd9ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bd9b1, 61 },
    { 0xb1bd9b2, 76 },
    { 0x17ebd9b3, 56 },
    { 0x280bd9b3, 23 },
    { 0x326bd9b5, 90 },
    { 0xafbd9b6, 98 },
    { 0x804bd9b5, 87 },
    { 0x6a9bd9b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bd9bc, 33 },
    { 0x84abd9bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877bd9c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x263bd9c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfbd9cb, 98 },
    { 0x139bd9cc, 85 },
    { 0x642bd9cd, 46 },
    { 0x49fbd9ce, 88 },
    { 0x1bbbd9cb, 33 },
    { 0xbd7bd9d0, 95 },
    { 0x9c7bd9d1, 90 },
    { 0x60fbd9cb, 33 },
    { 0xd2bbd9d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebd9d6, 33 },
    { 0, 0 },
    { 0xdd0bd9d8, 90 },
    { 0, 0 },
    { 0x53bd9da, 53 },
    { 0x5b8bd9db, 55 },
    { 0xfa3bd9dc, 61 },
    { 0, 0 },
    { 0xe41bd9de, 51 },
    { 0x7aabd9df, 66 },
    { 0xa3abd9de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920bd9e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12bd9ec, 95 },
    { 0x79fbd9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9bd9f8, 98 },
    { 0xd44bd9f9, 95 },
    { 0xb6fbd9fa, 90 },
    { 0, 0 },
    { 0xe3cbd9fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bbda00, 90 },
    { 0xbc8bda01, 78 },
    { 0x6d9bda01, 46 },
    { 0xe2abda02, 55 },
    { 0x45bbda02, 90 },
    { 0xfe0bda00, 90 },
    { 0xcf3bda02, 43 },
    { 0x670bda07, 33 },
    { 0, 0 },
    { 0x2febda09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfbda0c, 75 },
    { 0, 0 },
    { 0x81dbda0e, 33 },
    { 0, 0 },
    { 0x5a8bda10, 98 },
    { 0, 0 },
    { 0x3a2bda12, 61 },
    { 0x9ffbda12, 17 },
    { 0x18abda14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3bda18, 93 },
    { 0x9e2bda19, 61 },
    { 0xe45bda1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44bda1e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebda21, 87 },
    { 0x1b2bda22, 90 },
    { 0, 0 },
    { 0x318bda24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x168bda27, 34 },
    { 0x1bdbda28, 66 },
    { 0xd7bbda28, 33 },
    { 0xbd6bda28, 98 },
    { 0x107bda2b, 23 },
    { 0x525bda2c, 95 },
    { 0xf37bda2b, 40 },
    { 0x4fabda2e, 61 },
    { 0xdecbda2f, 87 },
    { 0xdfebda29, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9bda33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7bda3d, 66 },
    { 0xaa7bda3e, 76 },
    { 0x35bbda3e, 33 },
    { 0xd82bda3f, 23 },
    { 0x7a7bda3f, 23 },
    { 0xa56bda3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acbda48, 61 },
    { 0, 0 },
    { 0xf1bbda4a, 23 },
    { 0x198bda4b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bda50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764bda55, 90 },
    { 0, 0 },
    { 0xcf5bda57, 46 },
    { 0x418bda57, 98 },
    { 0x54bbda59, 67 },
    { 0xce2bda5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661bda60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05bda64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811bda6b, 34 },
    { 0xddcbda6c, 43 },
    { 0, 0 },
    { 0x78dbda6e, 95 },
    { 0, 0 },
    { 0xc01bda70, 75 },
    { 0, 0 },
    { 0x342bda72, 40 },
    { 0x77abda72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7bda78, 45 },
    { 0x4f2bda78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bda83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216bda89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497bda8e, 87 },
    { 0x5ecbda8e, 87 },
    { 0x5afbda90, 90 },
    { 0x38bda8e, 78 },
    { 0xb28bda91, 98 },
    { 0xf0dbda93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78bda99, 92 },
    { 0x3eebda9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bda9d, 46 },
    { 0x4fcbda9d, 93 },
    { 0xf1abda9f, 33 },
    { 0xffbbda9d, 33 },
    { 0x89abda9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeabdaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bbdaab, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bdab0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bbdab3, 43 },
    { 0x792bdab3, 76 },
    { 0xcb5bdab3, 44 },
    { 0x676bdab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958bdabb, 69 },
    { 0xf90bdabc, 87 },
    { 0x253bdabc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dbdac0, 100 },
    { 0xbb3bdac1, 33 },
    { 0x43ebdac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7bdac5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02bdac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x907bdacb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddbdace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ebdad2, 95 },
    { 0, 0 },
    { 0x4ebbdad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bdad8, 76 },
    { 0, 0 },
    { 0x1c1bdada, 100 },
    { 0xb9fbdadb, 33 },
    { 0, 0 },
    { 0x7e3bdadd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcebdae4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bdae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496bdaed, 46 },
    { 0, 0 },
    { 0x71abdaef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bdaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bdafc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbdaff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bdb02, 90 },
    { 0, 0 },
    { 0xf72bdb04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5abdb07, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7bdb0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x724bdb0e, 23 },
    { 0, 0 },
    { 0xc42bdb10, 90 },
    { 0, 0 },
    { 0x214bdb12, 95 },
    { 0xe8abdb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1bdb17, 23 },
    { 0xa33bdb17, 98 },
    { 0xf36bdb17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0bdb1d, 33 },
    { 0xae1bdb1d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2abdb21, 36 },
    { 0xb08bdb22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbbdb29, 33 },
    { 0, 0 },
    { 0x836bdb2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6bdb30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bbdb36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x338bdb39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebdb3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bdb40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abdb44, 23 },
    { 0, 0 },
    { 0x4a9bdb46, 95 },
    { 0, 0 },
    { 0x63cbdb48, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaabdb4c, 65 },
    { 0x57abdb4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bbdb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bdb56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dbdb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2bdb60, 90 },
    { 0x3f6bdb61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbdb65, 90 },
    { 0x55bbdb66, 75 },
    { 0xb58bdb65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32bdb6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2bdb70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67bdb7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638bdb86, 80 },
    { 0x7bcbdb87, 61 },
    { 0, 0 },
    { 0x4debdb89, 43 },
    { 0, 0 },
    { 0xf36bdb8b, 90 },
    { 0x650bdb8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197bdb92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43bdb96, 61 },
    { 0x238bdb97, 23 },
    { 0x61ebdb97, 36 },
    { 0, 0 },
    { 0x2fabdb9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264bdba0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ebdba3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173bdbaa, 78 },
    { 0x5fbdbab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47bdbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abbdbb8, 61 },
    { 0x43abdbb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82bdbc0, 93 },
    { 0, 0 },
    { 0x4ecbdbc2, 43 },
    { 0x183bdbc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626bdbc9, 100 },
    { 0x1bbdbca, 55 },
    { 0, 0 },
    { 0x7c9bdbcc, 90 },
    { 0, 0 },
    { 0x747bdbce, 66 },
    { 0, 0 },
    { 0x64fbdbd0, 98 },
    { 0x2e3bdbd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcbdbd8, 61 },
    { 0xd77bdbd9, 23 },
    { 0xf5abdbd9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158bdbe1, 43 },
    { 0xf95bdbe2, 43 },
    { 0x9dcbdbe1, 76 },
    { 0, 0 },
    { 0x638bdbe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bbdbf0, 23 },
    { 0, 0 },
    { 0x200bdbf2, 64 },
    { 0xfe8bdbf3, 67 },
    { 0, 0 },
    { 0xf63bdbf5, 95 },
    { 0x39bdbf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bdbf9, 81 },
    { 0x1b4bdbf9, 90 },
    { 0x75ebdbf9, 95 },
    { 0x12ebdbfc, 23 },
    { 0x154bdbfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bbdc00, 61 },
    { 0, 0 },
    { 0x60bdc02, 91 },
    { 0xf57bdc03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3bdc08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55bdc0b, 90 },
    { 0, 0 },
    { 0x52bbdc0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbdc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddbdc13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8bdc19, 95 },
    { 0x9a2bdc1a, 90 },
    { 0, 0 },
    { 0x358bdc1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fbdc21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cbdc28, 90 },
    { 0xc2dbdc29, 81 },
    { 0x433bdc2a, 78 },
    { 0xa0bdc2b, 84 },
    { 0xa47bdc2b, 31 },
    { 0, 0 },
    { 0xd66bdc2e, 61 },
    { 0xe57bdc2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68bdc32, 33 },
    { 0x8e5bdc33, 90 },
    { 0xb13bdc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64bdc37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x795bdc3a, 93 },
    { 0xeb6bdc3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bdc3e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9bdc41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207bdc47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbbdc4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83bdc51, 61 },
    { 0x71ebdc52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bdc5a, 85 },
    { 0xc0dbdc5a, 78 },
    { 0x44bbdc5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9abdc60, 90 },
    { 0, 0 },
    { 0x8c2bdc62, 33 },
    { 0xdb2bdc62, 85 },
    { 0, 0 },
    { 0xdedbdc65, 55 },
    { 0, 0 },
    { 0x289bdc67, 55 },
    { 0xde6bdc68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5bdc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfbdc72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535bdc76, 33 },
    { 0x51ebdc77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858bdc83, 17 },
    { 0x357bdc84, 95 },
    { 0, 0 },
    { 0x437bdc86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x441bdc89, 46 },
    { 0, 0 },
    { 0xb40bdc8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbbdc8f, 77 },
    { 0x2fcbdc90, 81 },
    { 0xf4cbdc90, 23 },
    { 0, 0 },
    { 0x108bdc93, 99 },
    { 0xe67bdc94, 33 },
    { 0xb0abdc95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bdc98, 91 },
    { 0xa29bdc98, 93 },
    { 0, 0 },
    { 0x363bdc9b, 68 },
    { 0, 0 },
    { 0x10bdc9d, 61 },
    { 0, 0 },
    { 0x9e4bdc9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffbdca2, 33 },
    { 0, 0 },
    { 0x8b6bdca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4bdca7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eabdcaa, 34 },
    { 0xe93bdcab, 95 },
    { 0xef2bdcab, 33 },
    { 0x1a0bdcad, 61 },
    { 0x162bdcac, 66 },
    { 0xe33bdcaf, 90 },
    { 0x418bdcb0, 95 },
    { 0x702bdcb1, 67 },
    { 0x958bdcb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fbdcb5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291bdcb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1bdcbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fbdcc4, 87 },
    { 0, 0 },
    { 0xb64bdcc6, 83 },
    { 0x525bdcc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74bdcca, 46 },
    { 0xab4bdccb, 34 },
    { 0x7e5bdccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bdccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ebdcd5, 33 },
    { 0x9efbdcd6, 76 },
    { 0x586bdcd7, 90 },
    { 0x7b7bdcd8, 91 },
    { 0xd42bdcd6, 67 },
    { 0, 0 },
    { 0x7bbbdcdb, 33 },
    { 0xd54bdcdc, 95 },
    { 0x3afbdcdd, 46 },
    { 0x5f8bdcdb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bbdceb, 33 },
    { 0x107bdcec, 55 },
    { 0x8cbbdced, 98 },
    { 0xe17bdced, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1bdcf3, 33 },
    { 0x900bdcf4, 98 },
    { 0x614bdcf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efbdcf9, 33 },
    { 0, 0 },
    { 0x357bdcfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179bdd02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ebdd05, 67 },
    { 0x73ebdd06, 87 },
    { 0xdc0bdd06, 61 },
    { 0xcedbdd08, 33 },
    { 0xa86bdd09, 34 },
    { 0, 0 },
    { 0xce6bdd0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671bdd11, 33 },
    { 0xdcebdd12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bdd15, 95 },
    { 0, 0 },
    { 0x9f5bdd17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41bdd1c, 33 },
    { 0xe80bdd1d, 90 },
    { 0x8ffbdd1e, 95 },
    { 0x18bdd1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9bdd22, 98 },
    { 0x63abdd23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8bdd27, 98 },
    { 0, 0 },
    { 0x559bdd29, 95 },
    { 0, 0 },
    { 0x819bdd2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bbdd30, 95 },
    { 0x50cbdd31, 61 },
    { 0xf68bdd32, 90 },
    { 0, 0 },
    { 0x6abdd34, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99bdd3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebbdd43, 98 },
    { 0xaeebdd43, 33 },
    { 0, 0 },
    { 0x9f6bdd46, 100 },
    { 0xf47bdd47, 23 },
    { 0, 0 },
    { 0xc5ebdd49, 43 },
    { 0xbe2bdd4a, 23 },
    { 0xd52bdd49, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327bdd51, 77 },
    { 0x7a0bdd52, 91 },
    { 0xdebbdd53, 61 },
    { 0x236bdd54, 40 },
    { 0x3f1bdd55, 76 },
    { 0, 0 },
    { 0x45abdd57, 90 },
    { 0x6e5bdd58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fbdd5f, 33 },
    { 0x791bdd60, 90 },
    { 0xde5bdd60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0bdd6c, 33 },
    { 0xc82bdd6d, 95 },
    { 0xda9bdd6e, 33 },
    { 0xcf7bdd6f, 98 },
    { 0x948bdd70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741bdd75, 55 },
    { 0, 0 },
    { 0xe5bdd77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x544bdd7a, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603bdd7f, 61 },
    { 0x12bbdd7f, 90 },
    { 0, 0 },
    { 0x24cbdd82, 33 },
    { 0x7ddbdd83, 98 },
    { 0xdaabdd83, 23 },
    { 0, 0 },
    { 0x5bbbdd86, 77 },
    { 0x76ebdd87, 98 },
    { 0x5b5bdd88, 90 },
    { 0x159bdd89, 81 },
    { 0x8c1bdd87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53bdd8d, 19 },
    { 0xf7bdd8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82bdd93, 23 },
    { 0, 0 },
    { 0xfaabdd95, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5bdd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x992bdd9c, 100 },
    { 0x24bbdd9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x441bdda0, 61 },
    { 0xcb5bdda0, 19 },
    { 0x3d6bdda2, 90 },
    { 0x16ebdda3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412bdda8, 99 },
    { 0x302bdda8, 65 },
    { 0, 0 },
    { 0x13fbddab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ebddaf, 76 },
    { 0x12abddb0, 87 },
    { 0x929bddb1, 22 },
    { 0xe91bddb2, 33 },
    { 0x4cbddaf, 98 },
    { 0xad2bddb4, 93 },
    { 0x10ebddb3, 95 },
    { 0xa16bddb6, 23 },
    { 0, 0 },
    { 0x28abddb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867bddbc, 93 },
    { 0x7fabddbc, 61 },
    { 0, 0 },
    { 0xce0bddbf, 33 },
    { 0xadcbddbf, 23 },
    { 0, 0 },
    { 0xa3bbddc2, 95 },
    { 0, 0 },
    { 0x36bbddc4, 76 },
    { 0xb2fbddc5, 76 },
    { 0, 0 },
    { 0x5bcbddc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2bddca, 85 },
    { 0xfbbbddcb, 95 },
    { 0xd2dbddcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0bddd0, 61 },
    { 0, 0 },
    { 0xf4bbddd2, 23 },
    { 0x6dfbddd3, 33 },
    { 0x679bddd2, 100 },
    { 0, 0 },
    { 0xdf7bddd6, 90 },
    { 0, 0 },
    { 0xfd3bddd8, 23 },
    { 0x14abddd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12bdddc, 23 },
    { 0, 0 },
    { 0xc86bddde, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bdde1, 90 },
    { 0x35bdde2, 76 },
    { 0, 0 },
    { 0x6f7bdde4, 90 },
    { 0xc55bdde5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3bdde8, 93 },
    { 0xb5bdde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186bdded, 95 },
    { 0x168bddee, 66 },
    { 0xb1cbddef, 23 },
    { 0, 0 },
    { 0x1a8bddf1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0bddf4, 95 },
    { 0xb87bddf5, 61 },
    { 0x5a3bddf6, 90 },
    { 0x64bddf5, 43 },
    { 0, 0 },
    { 0xcc5bddf9, 90 },
    { 0x1e2bddf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7bddfe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57bde05, 90 },
    { 0x39fbde06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdbde0e, 95 },
    { 0xd43bde0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2bde12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bbde16, 46 },
    { 0xf91bde17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510bde1b, 33 },
    { 0x419bde1c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6bde1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327bde23, 95 },
    { 0xe9fbde24, 23 },
    { 0x2c6bde25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20bde2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5bde30, 34 },
    { 0, 0 },
    { 0xfefbde32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7bde35, 98 },
    { 0xb22bde36, 23 },
    { 0xd41bde37, 100 },
    { 0x6b3bde38, 52 },
    { 0, 0 },
    { 0xf31bde3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4bde40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72bde43, 75 },
    { 0x99abde44, 77 },
    { 0x7a0bde45, 23 },
    { 0x86bde45, 76 },
    { 0, 0 },
    { 0x35dbde48, 77 },
    { 0x9debde49, 61 },
    { 0x8ebde48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8bde4f, 76 },
    { 0, 0 },
    { 0xb7dbde51, 84 },
    { 0xdf0bde52, 33 },
    { 0, 0 },
    { 0x52ebde54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cbde59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36abde5c, 66 },
    { 0, 0 },
    { 0xe85bde5e, 33 },
    { 0x7cebde5f, 66 },
    { 0xb1cbde5e, 90 },
    { 0xc62bde60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbde64, 23 },
    { 0, 0 },
    { 0xf7abde66, 33 },
    { 0x412bde67, 81 },
    { 0xbabde68, 61 },
    { 0xcfbbde69, 33 },
    { 0x4a8bde69, 95 },
    { 0x962bde67, 98 },
    { 0xa6bde6c, 90 },
    { 0x707bde67, 98 },
    { 0x71ebde6b, 85 },
    { 0x669bde6f, 7 },
    { 0x591bde70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fbde78, 33 },
    { 0xca5bde79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35bde80, 98 },
    { 0xd25bde80, 93 },
    { 0x2cbbde80, 90 },
    { 0x16abde83, 33 },
    { 0xfcbde84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06bde89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bebde8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0bde95, 90 },
    { 0, 0 },
    { 0x195bde97, 33 },
    { 0x841bde98, 64 },
    { 0x213bde98, 99 },
    { 0x42bde99, 33 },
    { 0x2a7bde97, 95 },
    { 0xfc4bde99, 61 },
    { 0, 0 },
    { 0x179bde9e, 90 },
    { 0x8febde9f, 90 },
    { 0xca9bde9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47bdea4, 34 },
    { 0x830bdea4, 43 },
    { 0, 0 },
    { 0xfb8bdea7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bdeab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bbdeb0, 90 },
    { 0xc88bdeb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cbdeba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0bdebd, 76 },
    { 0xf63bdebd, 39 },
    { 0x43abdebf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5bdec2, 61 },
    { 0xc3ebdec2, 75 },
    { 0x333bdec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbbdec8, 23 },
    { 0x3c4bdec9, 76 },
    { 0x499bdec9, 85 },
    { 0, 0 },
    { 0x5cbbdecc, 77 },
    { 0x1d0bdecc, 61 },
    { 0, 0 },
    { 0x8c7bdecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17abded2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bbded5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cbded8, 33 },
    { 0x813bded9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21bdedd, 52 },
    { 0xf3bdede, 78 },
    { 0, 0 },
    { 0x679bdee0, 98 },
    { 0x84bbdee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bdee7, 43 },
    { 0x7cdbdee7, 64 },
    { 0xedfbdee9, 55 },
    { 0xf36bdeea, 66 },
    { 0xbd7bdeeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445bdeef, 34 },
    { 0xa56bdef0, 23 },
    { 0xa76bdef1, 66 },
    { 0x8d9bdef1, 65 },
    { 0xd7cbdef3, 90 },
    { 0x1b6bdef2, 76 },
    { 0, 0 },
    { 0x2d3bdef6, 100 },
    { 0xf38bdef7, 98 },
    { 0xd56bdef8, 85 },
    { 0, 0 },
    { 0x93abdefa, 61 },
    { 0x668bdefb, 76 },
    { 0x134bdefc, 61 },
    { 0x81cbdefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5bdf01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05bdf0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x466bdf0e, 98 },
    { 0xababdf0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393bdf14, 64 },
    { 0, 0 },
    { 0x85cbdf16, 90 },
    { 0, 0 },
    { 0x64bbdf18, 33 },
    { 0x41ebdf18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81abdf21, 33 },
    { 0x61bbdf21, 61 },
    { 0xdfcbdf23, 33 },
    { 0x290bdf21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bdf27, 98 },
    { 0x10abdf28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ebdf2f, 61 },
    { 0xec8bdf2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4debdf33, 23 },
    { 0x473bdf33, 81 },
    { 0x85bbdf35, 65 },
    { 0xec1bdf34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cebdf3a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332bdf3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ebdf41, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1bdf47, 33 },
    { 0xb01bdf48, 33 },
    { 0, 0 },
    { 0x457bdf4a, 98 },
    { 0, 0 },
    { 0x323bdf4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x338bdf4f, 95 },
    { 0, 0 },
    { 0x9c5bdf51, 98 },
    { 0x4febdf52, 61 },
    { 0x8f2bdf53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3bdf58, 95 },
    { 0, 0 },
    { 0xf8cbdf5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffbdf5e, 85 },
    { 0x884bdf5f, 88 },
    { 0x420bdf60, 76 },
    { 0, 0 },
    { 0x5b7bdf62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bdf65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58bdf68, 76 },
    { 0xd20bdf69, 90 },
    { 0, 0 },
    { 0x9c7bdf6b, 46 },
    { 0xf16bdf6b, 98 },
    { 0x112bdf6d, 85 },
    { 0xb67bdf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfbdf73, 43 },
    { 0x6bbdf74, 93 },
    { 0x170bdf75, 88 },
    { 0xdebdf74, 95 },
    { 0xde1bdf74, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2bdf7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x305bdf7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5bdf84, 95 },
    { 0xe10bdf85, 23 },
    { 0xe6bdf84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426bdf90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36bdf93, 98 },
    { 0x818bdf94, 99 },
    { 0xf86bdf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ebdf9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ebdf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bdfa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931bdfa9, 34 },
    { 0, 0 },
    { 0xb56bdfab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bbdfb2, 54 },
    { 0xefdbdfb3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bdfb6, 66 },
    { 0xf20bdfb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88abdfba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x968bdfbd, 71 },
    { 0x82bdfbe, 98 },
    { 0xaf1bdfbf, 23 },
    { 0, 0 },
    { 0x73abdfc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285bdfcc, 43 },
    { 0xea6bdfcd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbdfd6, 66 },
    { 0x272bdfd6, 76 },
    { 0x628bdfd8, 61 },
    { 0x34dbdfd9, 78 },
    { 0, 0 },
    { 0x3e0bdfdb, 34 },
    { 0xa79bdfdc, 55 },
    { 0x538bdfdc, 99 },
    { 0x4f0bdfde, 95 },
    { 0, 0 },
    { 0xea0bdfe0, 61 },
    { 0xdf0bdfe1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acbdfe4, 93 },
    { 0xdc9bdfe5, 95 },
    { 0, 0 },
    { 0x35dbdfe7, 61 },
    { 0xfebdfe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131bdfed, 90 },
    { 0x80dbdfed, 46 },
    { 0x547bdfef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbbdff3, 46 },
    { 0xe20bdff4, 61 },
    { 0x47bdff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22be000, 93 },
    { 0, 0 },
    { 0x3f3be002, 77 },
    { 0, 0 },
    { 0xa2abe004, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775be009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4be00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacebe011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cbe016, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abe01a, 90 },
    { 0, 0 },
    { 0x622be01c, 90 },
    { 0x42fbe01c, 23 },
    { 0x2a3be01e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432be024, 33 },
    { 0x31abe025, 33 },
    { 0x475be026, 23 },
    { 0xf1cbe025, 55 },
    { 0xd54be024, 61 },
    { 0, 0 },
    { 0xc03be02a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57be030, 52 },
    { 0xb3be031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddbe035, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948be039, 43 },
    { 0x6b8be03a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebe043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865be047, 34 },
    { 0x5c2be047, 55 },
    { 0, 0 },
    { 0x5f6be04a, 100 },
    { 0x781be04b, 23 },
    { 0, 0 },
    { 0xb0ebe04d, 33 },
    { 0x8be04e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbe051, 93 },
    { 0, 0 },
    { 0x69be053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabe05c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83be060, 76 },
    { 0x724be061, 34 },
    { 0, 0 },
    { 0x91bbe063, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afbe067, 100 },
    { 0, 0 },
    { 0x438be069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cbe06e, 95 },
    { 0xf91be06e, 46 },
    { 0, 0 },
    { 0x5b3be071, 91 },
    { 0xfa0be071, 95 },
    { 0x72cbe072, 83 },
    { 0x1fabe074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfbe077, 98 },
    { 0, 0 },
    { 0x410be079, 43 },
    { 0xb83be07a, 98 },
    { 0, 0 },
    { 0xc11be07c, 31 },
    { 0x4b9be07d, 95 },
    { 0, 0 },
    { 0x686be07f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fbe084, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161be08a, 17 },
    { 0x500be08b, 61 },
    { 0x50be08b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdabe091, 77 },
    { 0x69ebe091, 33 },
    { 0, 0 },
    { 0x4f1be094, 33 },
    { 0xbc8be095, 23 },
    { 0xe18be094, 61 },
    { 0xd16be097, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dbe09d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbe0a0, 23 },
    { 0x1bbe0a0, 34 },
    { 0xd42be0a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccbe0a6, 90 },
    { 0x92fbe0a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bbe0ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cbe0b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abe0bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8be0c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98be0c4, 99 },
    { 0, 0 },
    { 0xa73be0c6, 90 },
    { 0xd2ebe0c6, 33 },
    { 0xe36be0c6, 33 },
    { 0x42cbe0c9, 66 },
    { 0xda9be0ca, 90 },
    { 0xebbbe0cb, 34 },
    { 0x225be0cc, 95 },
    { 0x7f9be0c9, 61 },
    { 0x869be0ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56be0d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22be0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfbe0e4, 90 },
    { 0, 0 },
    { 0x4cebe0e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fbe0ea, 33 },
    { 0x7dabe0eb, 90 },
    { 0xb28be0ec, 61 },
    { 0xedebe0ed, 76 },
    { 0x533be0ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929be0f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbbe0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1be101, 33 },
    { 0xb77be102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54be105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74be108, 43 },
    { 0x4dabe109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x787be10c, 90 },
    { 0xb6ebe10c, 23 },
    { 0xdd8be10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x263be111, 33 },
    { 0xc62be112, 90 },
    { 0, 0 },
    { 0xd93be114, 34 },
    { 0x89cbe114, 98 },
    { 0x196be114, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cbe11b, 61 },
    { 0xcfebe11c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbbe11f, 95 },
    { 0x58fbe11f, 76 },
    { 0x17dbe120, 17 },
    { 0xd3be122, 98 },
    { 0x7b4be123, 33 },
    { 0x7eabe124, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35be127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254be12b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61be12e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3be136, 98 },
    { 0x93abe136, 33 },
    { 0xfccbe138, 33 },
    { 0x12be136, 90 },
    { 0xa3cbe13a, 57 },
    { 0x5ddbe139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ebe13f, 43 },
    { 0x1d5be140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacbe145, 98 },
    { 0xdb8be146, 33 },
    { 0x1c5be147, 98 },
    { 0x8e1be148, 76 },
    { 0xd28be148, 61 },
    { 0x12fbe147, 23 },
    { 0x7aebe14b, 99 },
    { 0x3aebe14c, 34 },
    { 0x68ebe14c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488be151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8be154, 67 },
    { 0x6a9be155, 23 },
    { 0x6cebe156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbe15a, 33 },
    { 0xc42be15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8be15f, 33 },
    { 0, 0 },
    { 0xa19be161, 98 },
    { 0, 0 },
    { 0x12bbe163, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccbe166, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x592be169, 53 },
    { 0, 0 },
    { 0xa50be16b, 93 },
    { 0, 0 },
    { 0x7f2be16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887be173, 43 },
    { 0xfe8be173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbe17c, 98 },
    { 0xa9dbe17d, 33 },
    { 0x57dbe17d, 75 },
    { 0xe11be17e, 90 },
    { 0x9fabe17e, 96 },
    { 0xb5cbe17f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x136be184, 43 },
    { 0x310be185, 34 },
    { 0x4cbbe186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x473be189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ebe190, 90 },
    { 0x252be191, 91 },
    { 0xab8be192, 65 },
    { 0xc36be193, 61 },
    { 0x6a4be192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36be197, 76 },
    { 0xf8ebe198, 91 },
    { 0, 0 },
    { 0xe4bbe19a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ebe19d, 90 },
    { 0x542be19d, 53 },
    { 0xf8bbe19f, 23 },
    { 0x19abe1a0, 23 },
    { 0x1ffbe1a1, 98 },
    { 0x199be1a1, 61 },
    { 0x811be1a1, 33 },
    { 0xce8be1a4, 82 },
    { 0x295be1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2be1a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebe1ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169be1bc, 52 },
    { 0x66bbe1bd, 88 },
    { 0xb5fbe1bd, 23 },
    { 0x576be1bd, 43 },
    { 0xa4be1bf, 90 },
    { 0xb1be1c1, 98 },
    { 0xe18be1c2, 93 },
    { 0xe44be1c3, 66 },
    { 0x552be1bc, 34 },
    { 0x5cdbe1bc, 95 },
    { 0xc06be1c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86be1cb, 93 },
    { 0x677be1cb, 95 },
    { 0x158be1cd, 40 },
    { 0, 0 },
    { 0x876be1cf, 98 },
    { 0x99ebe1d0, 43 },
    { 0x5cebe1cf, 84 },
    { 0x204be1d2, 17 },
    { 0x219be1d3, 90 },
    { 0x694be1d3, 61 },
    { 0xdcbbe1d4, 87 },
    { 0x3debe1d6, 76 },
    { 0xa45be1d4, 34 },
    { 0x7c9be1d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145be1dc, 90 },
    { 0x867be1dc, 46 },
    { 0x236be1dc, 61 },
    { 0x602be1df, 61 },
    { 0xd77be1dc, 23 },
    { 0x3a3be1e1, 23 },
    { 0xaddbe1e2, 55 },
    { 0xed8be1e3, 90 },
    { 0xc58be1e4, 61 },
    { 0, 0 },
    { 0xe7bbe1e6, 33 },
    { 0xd05be1e7, 93 },
    { 0x792be1e8, 90 },
    { 0x65cbe1e9, 80 },
    { 0xc81be1e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35be1f3, 33 },
    { 0, 0 },
    { 0x33fbe1f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84be1fd, 33 },
    { 0xd96be1fe, 23 },
    { 0x165be1fd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8be202, 23 },
    { 0xacbe203, 33 },
    { 0x458be204, 33 },
    { 0xd84be205, 46 },
    { 0xf0fbe203, 26 },
    { 0, 0 },
    { 0x8e6be208, 98 },
    { 0x930be209, 43 },
    { 0, 0 },
    { 0x573be20b, 98 },
    { 0, 0 },
    { 0x369be20d, 90 },
    { 0xa0fbe20e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187be212, 90 },
    { 0x692be213, 34 },
    { 0x49fbe212, 99 },
    { 0x38cbe215, 33 },
    { 0x964be216, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbe21a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548be225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32abe228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18be22b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cbe22e, 90 },
    { 0xf0abe22e, 61 },
    { 0xa3cbe230, 100 },
    { 0x136be231, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75be235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefbe238, 78 },
    { 0xeb0be238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fbe23c, 98 },
    { 0xe2abe23d, 61 },
    { 0x39ebe23c, 33 },
    { 0x51be23f, 95 },
    { 0x122be240, 34 },
    { 0xc32be241, 55 },
    { 0x85bbe242, 90 },
    { 0x49dbe243, 33 },
    { 0, 0 },
    { 0x83fbe245, 98 },
    { 0xb3bbe246, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddbe249, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88be24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02be252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0be258, 76 },
    { 0x7aabe258, 91 },
    { 0, 0 },
    { 0x8cbbe25b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bbe260, 52 },
    { 0, 0 },
    { 0x164be262, 93 },
    { 0x54bbe262, 49 },
    { 0, 0 },
    { 0x857be265, 90 },
    { 0, 0 },
    { 0xd36be267, 98 },
    { 0, 0 },
    { 0x65abe269, 67 },
    { 0x389be26a, 93 },
    { 0x687be26b, 33 },
    { 0x431be269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597be271, 55 },
    { 0xf0abe272, 61 },
    { 0, 0 },
    { 0xd59be274, 100 },
    { 0, 0 },
    { 0xe6abe276, 23 },
    { 0, 0 },
    { 0xa46be278, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcbe27b, 33 },
    { 0, 0 },
    { 0xbb6be27d, 88 },
    { 0, 0 },
    { 0xbc0be27f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dbe283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4be288, 91 },
    { 0xac2be288, 90 },
    { 0x7fbbe28a, 90 },
    { 0, 0 },
    { 0xdf7be28c, 90 },
    { 0xe4fbe28d, 64 },
    { 0x94ebe28e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fbe294, 34 },
    { 0x7fabe295, 76 },
    { 0xfacbe295, 61 },
    { 0, 0 },
    { 0x22dbe298, 56 },
    { 0, 0 },
    { 0xbf9be29a, 93 },
    { 0x74fbe29b, 55 },
    { 0, 0 },
    { 0x497be29d, 33 },
    { 0x397be29e, 33 },
    { 0, 0 },
    { 0x510be2a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be2a4, 34 },
    { 0, 0 },
    { 0xbc2be2a6, 50 },
    { 0x50dbe2a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755be2ab, 52 },
    { 0, 0 },
    { 0x89dbe2ad, 65 },
    { 0x2e3be2ae, 95 },
    { 0xf79be2ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3be2b2, 61 },
    { 0x7febe2b2, 61 },
    { 0x32dbe2b4, 92 },
    { 0x7f3be2b2, 34 },
    { 0x2e5be2b3, 95 },
    { 0x43bbe2b2, 90 },
    { 0xcb4be2b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7be2bb, 61 },
    { 0xd09be2bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0abe2c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bbe2c4, 90 },
    { 0, 0 },
    { 0x7c3be2c6, 87 },
    { 0x164be2c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412be2cc, 23 },
    { 0xbc7be2cc, 55 },
    { 0xfebbe2ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636be2d4, 91 },
    { 0xcf4be2d5, 33 },
    { 0x65cbe2d6, 61 },
    { 0x34ebe2d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6abe2db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x741be2de, 33 },
    { 0, 0 },
    { 0x3adbe2e0, 90 },
    { 0xe8ebe2e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dbe2e5, 93 },
    { 0x645be2e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eabe2e9, 34 },
    { 0xdebe2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdbe2f0, 66 },
    { 0x25be2f1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832be2f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90be2f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccbe2fc, 93 },
    { 0, 0 },
    { 0xb66be2fe, 33 },
    { 0, 0 },
    { 0xb78be300, 34 },
    { 0, 0 },
    { 0x348be302, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688be30e, 34 },
    { 0x367be30f, 90 },
    { 0xe40be30e, 33 },
    { 0x1b9be311, 33 },
    { 0xc45be312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6be316, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35be319, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbe31e, 61 },
    { 0, 0 },
    { 0x177be320, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x442be323, 33 },
    { 0x767be324, 95 },
    { 0, 0 },
    { 0x274be326, 101 },
    { 0, 0 },
    { 0x95be328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943be330, 24 },
    { 0xb2cbe331, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8be334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fbe33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95abe342, 34 },
    { 0, 0 },
    { 0xa28be344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103be34e, 91 },
    { 0xe12be34f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29be355, 23 },
    { 0x562be355, 95 },
    { 0x535be355, 33 },
    { 0, 0 },
    { 0xb5dbe359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4be35d, 23 },
    { 0, 0 },
    { 0xa25be35f, 93 },
    { 0xa9be360, 66 },
    { 0xb91be35f, 61 },
    { 0xcbbbe360, 65 },
    { 0, 0 },
    { 0xa29be364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abe368, 90 },
    { 0x61dbe369, 23 },
    { 0x38ebe36a, 90 },
    { 0x66bbe36b, 90 },
    { 0x92ebe368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62be370, 23 },
    { 0, 0 },
    { 0xdf8be372, 43 },
    { 0xf79be373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571be378, 95 },
    { 0x16ebe378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8be37d, 95 },
    { 0, 0 },
    { 0x290be37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740be386, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7be38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbe398, 66 },
    { 0x783be398, 61 },
    { 0x760be39a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97be39e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90be3a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94be3ac, 78 },
    { 0x7c0be3ac, 23 },
    { 0, 0 },
    { 0x904be3af, 87 },
    { 0, 0 },
    { 0xd57be3b1, 61 },
    { 0xaadbe3b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbe3b5, 23 },
    { 0, 0 },
    { 0x5cebe3b7, 76 },
    { 0x8c5be3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abe3bc, 47 },
    { 0x9dfbe3bd, 91 },
    { 0xf0abe3be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dbe3c3, 98 },
    { 0x754be3c4, 98 },
    { 0x9ecbe3c5, 84 },
    { 0x8c4be3c3, 33 },
    { 0xa8abe3c7, 55 },
    { 0x5eebe3c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe3cb, 98 },
    { 0xec8be3cc, 43 },
    { 0xaf5be3cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaabe3d0, 76 },
    { 0xe35be3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x969be3d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3be3d9, 24 },
    { 0x12bbe3da, 95 },
    { 0, 0 },
    { 0xc48be3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbe3df, 33 },
    { 0, 0 },
    { 0x7e9be3e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8be3e9, 98 },
    { 0xc16be3ea, 34 },
    { 0x238be3eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acbe3ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9be3f1, 67 },
    { 0xa9be3f1, 66 },
    { 0, 0 },
    { 0x6bfbe3f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebe3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146be401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0be404, 90 },
    { 0xa3bbe405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3be410, 90 },
    { 0xa71be411, 17 },
    { 0xb7be412, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ebe416, 23 },
    { 0x76cbe416, 93 },
    { 0, 0 },
    { 0xba5be419, 23 },
    { 0, 0 },
    { 0xa2bbe41b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fbe41e, 66 },
    { 0, 0 },
    { 0xd1ebe420, 61 },
    { 0x289be421, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x764be424, 55 },
    { 0, 0 },
    { 0xccbbe426, 81 },
    { 0, 0 },
    { 0xa6bbe428, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5be42d, 23 },
    { 0x7b5be42e, 43 },
    { 0xd0fbe42f, 85 },
    { 0x900be430, 34 },
    { 0x273be431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2be435, 80 },
    { 0x79abe436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87be43a, 84 },
    { 0xf10be43b, 95 },
    { 0xbe9be43a, 34 },
    { 0x9abe43b, 23 },
    { 0, 0 },
    { 0xe4bbe43f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabe443, 90 },
    { 0xd61be443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16abe44c, 87 },
    { 0, 0 },
    { 0x7babe44e, 90 },
    { 0xa92be44f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddbe455, 76 },
    { 0xd18be456, 76 },
    { 0x79abe456, 52 },
    { 0xa5ebe458, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81be45c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dbe462, 86 },
    { 0x70dbe463, 23 },
    { 0xf3abe464, 90 },
    { 0x5b1be465, 33 },
    { 0, 0 },
    { 0x7abe467, 4 },
    { 0x5c6be468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159be46c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65be470, 61 },
    { 0, 0 },
    { 0x588be472, 80 },
    { 0x2e6be473, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13be47d, 98 },
    { 0xd86be47e, 76 },
    { 0, 0 },
    { 0xdd8be480, 95 },
    { 0x7dabe481, 90 },
    { 0, 0 },
    { 0x23ebe483, 34 },
    { 0xaf1be483, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9be489, 47 },
    { 0x8ffbe489, 33 },
    { 0x76be48a, 95 },
    { 0, 0 },
    { 0x3c8be48d, 55 },
    { 0, 0 },
    { 0xb11be48f, 96 },
    { 0x5a8be490, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fbe499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbe49c, 34 },
    { 0x91dbe49d, 52 },
    { 0xd51be49e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x702be4a1, 33 },
    { 0, 0 },
    { 0xb57be4a3, 95 },
    { 0xe5ebe4a3, 66 },
    { 0, 0 },
    { 0x3d2be4a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bebe4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282be4ae, 76 },
    { 0x5debe4ae, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabe4b7, 43 },
    { 0xdf7be4b7, 17 },
    { 0, 0 },
    { 0xd0bbe4ba, 33 },
    { 0, 0 },
    { 0x905be4bc, 95 },
    { 0x721be4bc, 55 },
    { 0xe27be4be, 61 },
    { 0xb01be4bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67abe4c2, 33 },
    { 0x369be4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fbe4c6, 96 },
    { 0xcb3be4c7, 84 },
    { 0x2be4c8, 99 },
    { 0x3b2be4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dbe4cc, 55 },
    { 0x769be4cc, 76 },
    { 0xc99be4cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47be4d3, 98 },
    { 0x697be4d3, 23 },
    { 0x10dbe4d5, 46 },
    { 0, 0 },
    { 0x35ebe4d7, 46 },
    { 0x74abe4d7, 93 },
    { 0, 0 },
    { 0xb17be4da, 34 },
    { 0xcd3be4db, 99 },
    { 0x8afbe4dc, 93 },
    { 0x3f4be4dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48be4e7, 95 },
    { 0x6b9be4e8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4be4eb, 61 },
    { 0xc1dbe4ec, 33 },
    { 0xf21be4ec, 95 },
    { 0xb96be4ed, 95 },
    { 0x25cbe4ee, 98 },
    { 0xb13be4f0, 61 },
    { 0x6dbe4ef, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cebe4f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97be4fa, 83 },
    { 0x7b7be4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcebe500, 78 },
    { 0xd45be500, 61 },
    { 0xbdbe500, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7be505, 61 },
    { 0x83abe506, 76 },
    { 0x5f2be505, 91 },
    { 0, 0 },
    { 0x200be509, 33 },
    { 0x687be50a, 98 },
    { 0x6d5be50b, 55 },
    { 0x576be50c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23be50f, 33 },
    { 0x866be50f, 33 },
    { 0, 0 },
    { 0x7a9be512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebe518, 34 },
    { 0xc4dbe519, 43 },
    { 0, 0 },
    { 0xde3be51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696be521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6be526, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35be529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356be531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46be535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbbe538, 77 },
    { 0xe09be539, 90 },
    { 0x8c5be538, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369be53f, 68 },
    { 0x764be540, 90 },
    { 0, 0 },
    { 0x9abbe542, 33 },
    { 0x286be543, 90 },
    { 0xf05be544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1be547, 23 },
    { 0x340be548, 81 },
    { 0xbbdbe548, 43 },
    { 0xa9ebe547, 61 },
    { 0x71ebe548, 77 },
    { 0x516be54c, 95 },
    { 0xfebbe549, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cbe552, 90 },
    { 0, 0 },
    { 0x607be554, 77 },
    { 0x70fbe555, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe558, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x837be55b, 90 },
    { 0x9a9be55b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4be561, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bbe565, 93 },
    { 0x40dbe566, 61 },
    { 0, 0 },
    { 0x980be568, 76 },
    { 0x14bbe569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395be574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180be57d, 33 },
    { 0x635be57e, 86 },
    { 0xc7cbe57e, 95 },
    { 0, 0 },
    { 0x6d4be581, 33 },
    { 0x4ccbe581, 88 },
    { 0, 0 },
    { 0x787be584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1be589, 95 },
    { 0x104be589, 33 },
    { 0x768be589, 76 },
    { 0xd7bbe58c, 100 },
    { 0x90bbe58d, 98 },
    { 0x657be58c, 33 },
    { 0, 0 },
    { 0xb84be590, 23 },
    { 0x740be590, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x569be594, 46 },
    { 0xc42be594, 61 },
    { 0x163be594, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2be59e, 52 },
    { 0x7e2be59e, 95 },
    { 0xedfbe59e, 100 },
    { 0, 0 },
    { 0xb3bbe5a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05be5a6, 23 },
    { 0, 0 },
    { 0x11cbe5a8, 23 },
    { 0x662be5a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebbe5ac, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddebe5b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9be5bb, 66 },
    { 0xea1be5bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811be5c2, 33 },
    { 0, 0 },
    { 0x61ebe5c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1be5c9, 98 },
    { 0x357be5ca, 61 },
    { 0x5e2be5cb, 76 },
    { 0, 0 },
    { 0x1a2be5cd, 61 },
    { 0xa40be5ce, 23 },
    { 0, 0 },
    { 0x678be5d0, 23 },
    { 0x795be5d0, 88 },
    { 0xdfbe5d2, 61 },
    { 0x3b7be5d0, 90 },
    { 0, 0 },
    { 0x458be5d5, 46 },
    { 0x7d8be5d6, 98 },
    { 0x31bbe5d5, 100 },
    { 0x70fbe5d8, 67 },
    { 0xc53be5d8, 90 },
    { 0xf0dbe5da, 84 },
    { 0x4b5be5da, 93 },
    { 0x85fbe5da, 33 },
    { 0x6f2be5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214be5e3, 33 },
    { 0xf69be5e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75abe5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedbe5ea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbe5ed, 23 },
    { 0x9d2be5ee, 33 },
    { 0, 0 },
    { 0xc2bbe5f0, 61 },
    { 0x40abe5f1, 81 },
    { 0xffbbe5f2, 4 },
    { 0x722be5f3, 90 },
    { 0xcb1be5f2, 61 },
    { 0xa62be5f1, 61 },
    { 0, 0 },
    { 0xd5abe5f7, 88 },
    { 0x329be5f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcabe5fb, 90 },
    { 0, 0 },
    { 0xc2fbe5fd, 98 },
    { 0xe45be5fd, 59 },
    { 0, 0 },
    { 0xeb7be600, 43 },
    { 0x964be601, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabbe609, 90 },
    { 0xa1bbe60a, 95 },
    { 0x493be60b, 76 },
    { 0xfc4be60a, 96 },
    { 0x750be60b, 33 },
    { 0xee8be60e, 34 },
    { 0, 0 },
    { 0x2ebe610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbe613, 81 },
    { 0, 0 },
    { 0x24ebe615, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598be619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63be621, 95 },
    { 0, 0 },
    { 0x899be623, 78 },
    { 0, 0 },
    { 0xa10be625, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be629, 87 },
    { 0, 0 },
    { 0x1d4be62b, 23 },
    { 0x527be62c, 98 },
    { 0x950be62d, 55 },
    { 0xf4be62e, 67 },
    { 0x880be62f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbbe632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80abe63b, 90 },
    { 0x838be63c, 33 },
    { 0x686be63d, 23 },
    { 0xac7be63e, 76 },
    { 0xed2be63f, 76 },
    { 0xa88be63f, 23 },
    { 0xb81be641, 23 },
    { 0xd34be642, 90 },
    { 0, 0 },
    { 0xdcfbe644, 23 },
    { 0x1fabe645, 33 },
    { 0, 0 },
    { 0x757be647, 46 },
    { 0x76cbe648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaebe656, 95 },
    { 0x920be657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be65e, 93 },
    { 0xcc0be65e, 36 },
    { 0, 0 },
    { 0x402be661, 84 },
    { 0, 0 },
    { 0x42cbe663, 55 },
    { 0, 0 },
    { 0x75ebe665, 95 },
    { 0, 0 },
    { 0x4fabe667, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cbe66f, 33 },
    { 0, 0 },
    { 0xae3be671, 33 },
    { 0x18fbe672, 23 },
    { 0x1f0be673, 91 },
    { 0, 0 },
    { 0x9c8be675, 75 },
    { 0x1d8be676, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bbe67a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x926be67d, 93 },
    { 0, 0 },
    { 0x970be67f, 43 },
    { 0, 0 },
    { 0x8adbe681, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36be684, 43 },
    { 0xb32be684, 91 },
    { 0, 0 },
    { 0xa37be687, 100 },
    { 0x646be688, 43 },
    { 0, 0 },
    { 0xe82be68a, 90 },
    { 0x11be68b, 61 },
    { 0x353be68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x822be68f, 61 },
    { 0, 0 },
    { 0xee2be691, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7be698, 52 },
    { 0x5f3be699, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6abe69c, 95 },
    { 0x989be69d, 61 },
    { 0xbccbe69e, 78 },
    { 0xba3be69f, 76 },
    { 0x649be6a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2be6a4, 81 },
    { 0xa21be6a5, 90 },
    { 0x7e7be6a6, 90 },
    { 0xfb2be6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x531be6aa, 23 },
    { 0xfa1be6aa, 61 },
    { 0x5cdbe6aa, 52 },
    { 0x695be6aa, 99 },
    { 0xfefbe6ad, 78 },
    { 0xa46be6ae, 78 },
    { 0x703be6aa, 33 },
    { 0xe29be6ab, 23 },
    { 0xfc1be6ae, 55 },
    { 0x37be6ab, 90 },
    { 0, 0 },
    { 0xc67be6b5, 98 },
    { 0x1abe6b5, 61 },
    { 0xd35be6b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36be6be, 46 },
    { 0xc36be6bf, 56 },
    { 0xbebbe6bf, 76 },
    { 0x91bbe6c1, 91 },
    { 0xc86be6c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be6c6, 87 },
    { 0x745be6c7, 34 },
    { 0x7abe6c8, 61 },
    { 0xe8dbe6c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48abe6cd, 33 },
    { 0, 0 },
    { 0xfacbe6cf, 33 },
    { 0, 0 },
    { 0xcc0be6d1, 95 },
    { 0, 0 },
    { 0xe85be6d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823be6d8, 65 },
    { 0x8c9be6d9, 99 },
    { 0xc61be6da, 61 },
    { 0xa37be6db, 84 },
    { 0x93abe6db, 84 },
    { 0, 0 },
    { 0x28fbe6de, 66 },
    { 0x697be6df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787be6e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31abe6eb, 33 },
    { 0xd63be6ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25be6ef, 90 },
    { 0x320be6f0, 95 },
    { 0xe87be6f1, 66 },
    { 0x604be6f1, 95 },
    { 0xea6be6f3, 33 },
    { 0xaa0be6f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x697be6f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bbe6fa, 23 },
    { 0x958be6fb, 90 },
    { 0xc26be6fa, 90 },
    { 0, 0 },
    { 0xb5cbe6fe, 20 },
    { 0x72cbe6ff, 64 },
    { 0x7c0be700, 95 },
    { 0x566be701, 90 },
    { 0xc05be701, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03be708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30abe712, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe715, 90 },
    { 0, 0 },
    { 0x9e7be717, 61 },
    { 0x677be718, 33 },
    { 0xffebe719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9be71c, 95 },
    { 0xdafbe71c, 34 },
    { 0, 0 },
    { 0xe49be71f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdbe723, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aabe728, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecbe72d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x872be730, 98 },
    { 0x2dbbe730, 90 },
    { 0, 0 },
    { 0x2cbe733, 44 },
    { 0, 0 },
    { 0x458be735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0be738, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fbe74a, 90 },
    { 0xa2fbe74a, 66 },
    { 0x96abe74a, 40 },
    { 0xb1abe74b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be751, 84 },
    { 0x9bcbe752, 78 },
    { 0xf1ebe753, 84 },
    { 0x6a9be754, 90 },
    { 0xfa8be753, 33 },
    { 0xd9dbe756, 46 },
    { 0, 0 },
    { 0x742be758, 34 },
    { 0x4f9be759, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5be761, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12be768, 36 },
    { 0xef4be768, 34 },
    { 0x42ebe768, 93 },
    { 0x6aebe76b, 33 },
    { 0xff2be76c, 98 },
    { 0xaafbe76d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635be773, 78 },
    { 0x50abe774, 77 },
    { 0x6abbe773, 90 },
    { 0xd6fbe775, 33 },
    { 0x7b8be775, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4be77d, 90 },
    { 0x328be77e, 23 },
    { 0x25bbe77f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbe786, 97 },
    { 0xdcebe786, 98 },
    { 0x137be788, 90 },
    { 0x668be789, 90 },
    { 0xfc0be78a, 76 },
    { 0x790be787, 93 },
    { 0xf01be78c, 34 },
    { 0xb16be787, 95 },
    { 0xc51be78a, 66 },
    { 0, 0 },
    { 0xdcdbe790, 90 },
    { 0x851be790, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73abe794, 90 },
    { 0xdf8be795, 98 },
    { 0x7d5be796, 34 },
    { 0x9cabe797, 46 },
    { 0, 0 },
    { 0xbc9be799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26be79e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdbe7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359be7ac, 98 },
    { 0, 0 },
    { 0x306be7ae, 52 },
    { 0x529be7af, 23 },
    { 0xf8abe7b0, 95 },
    { 0x9d5be7b0, 84 },
    { 0xe84be7af, 95 },
    { 0xe98be7af, 93 },
    { 0x724be7b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5be7b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacabe7bf, 95 },
    { 0, 0 },
    { 0x283be7c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5be7c4, 78 },
    { 0, 0 },
    { 0xfa0be7c6, 61 },
    { 0x8c8be7c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebe7ca, 77 },
    { 0, 0 },
    { 0x413be7cc, 34 },
    { 0x331be7cd, 98 },
    { 0x435be7cd, 78 },
    { 0x93fbe7cf, 66 },
    { 0x3be7cc, 76 },
    { 0x274be7cf, 90 },
    { 0x286be7d2, 95 },
    { 0x56dbe7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x250be7d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774be7df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38be7e2, 23 },
    { 0xbc9be7e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115be7ea, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73be7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3be7f2, 33 },
    { 0x32dbe7f3, 43 },
    { 0xd47be7f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602be7f8, 93 },
    { 0, 0 },
    { 0x418be7fa, 46 },
    { 0x600be7fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafbe806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dbe809, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x895be80c, 84 },
    { 0, 0 },
    { 0x5debe80e, 98 },
    { 0xf16be80f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x991be812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8be816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7be81b, 87 },
    { 0xdcbbe81c, 95 },
    { 0x4ccbe81b, 17 },
    { 0x105be81e, 90 },
    { 0xd25be81e, 46 },
    { 0x3cfbe820, 55 },
    { 0, 0 },
    { 0x496be822, 61 },
    { 0, 0 },
    { 0xdc5be824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889be828, 95 },
    { 0xc23be829, 61 },
    { 0x455be82a, 84 },
    { 0x9eebe829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fbe831, 91 },
    { 0xc4be831, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9be83b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dbe842, 98 },
    { 0, 0 },
    { 0xe09be844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddbe84a, 98 },
    { 0, 0 },
    { 0x517be84c, 76 },
    { 0xb5cbe84c, 99 },
    { 0x1f0be84e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40be851, 61 },
    { 0x69bbe851, 66 },
    { 0x2dabe853, 95 },
    { 0xb3cbe854, 95 },
    { 0x30bbe854, 95 },
    { 0x21ebe856, 43 },
    { 0, 0 },
    { 0xe48be858, 90 },
    { 0x6bebe859, 33 },
    { 0xda6be85a, 76 },
    { 0x403be85b, 91 },
    { 0x6c5be85a, 84 },
    { 0, 0 },
    { 0x5cfbe85e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72be867, 67 },
    { 0x28abe868, 77 },
    { 0xf9bbe869, 77 },
    { 0, 0 },
    { 0xad5be86b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80be871, 95 },
    { 0xc8be872, 61 },
    { 0x59be872, 61 },
    { 0x688be872, 34 },
    { 0x1dbbe875, 34 },
    { 0xb90be875, 90 },
    { 0xa1bbe877, 95 },
    { 0x597be876, 61 },
    { 0, 0 },
    { 0xcefbe87a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91be87e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ebe883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645be888, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5be88b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be88f, 61 },
    { 0x4c2be890, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbe895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551be89a, 43 },
    { 0xb1fbe89b, 61 },
    { 0xabbe89c, 98 },
    { 0xf96be89d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648be8a8, 75 },
    { 0xf85be8a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2be8b0, 66 },
    { 0x4c6be8b1, 55 },
    { 0x5f7be8b2, 23 },
    { 0, 0 },
    { 0xae4be8b4, 93 },
    { 0, 0 },
    { 0x87fbe8b6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbe8bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbe8c5, 42 },
    { 0x5be8c6, 96 },
    { 0x80be8c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904be8cb, 90 },
    { 0xf42be8cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be8cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cbe8d2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813be8d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6be8dd, 99 },
    { 0, 0 },
    { 0xdb1be8df, 98 },
    { 0xec4be8df, 87 },
    { 0, 0 },
    { 0xc84be8e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866be8e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x596be8ea, 90 },
    { 0x170be8ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5be8f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183be8fc, 33 },
    { 0x33fbe8fd, 33 },
    { 0, 0 },
    { 0xa6ebe8ff, 95 },
    { 0xbf0be900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dbe905, 61 },
    { 0x744be905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63be909, 93 },
    { 0, 0 },
    { 0xdaabe90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0be910, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbbe914, 89 },
    { 0x3b4be915, 99 },
    { 0x178be915, 33 },
    { 0x313be917, 85 },
    { 0xc6bbe918, 76 },
    { 0x34be919, 84 },
    { 0x153be916, 95 },
    { 0x533be916, 98 },
    { 0x4be91a, 100 },
    { 0xa16be91d, 23 },
    { 0xa01be914, 23 },
    { 0xb6dbe91b, 98 },
    { 0, 0 },
    { 0xebabe921, 43 },
    { 0, 0 },
    { 0xabe923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639be927, 98 },
    { 0xf54be927, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x557be92b, 98 },
    { 0x321be92b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbe92f, 46 },
    { 0x2f7be92f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0be935, 88 },
    { 0xccebe936, 76 },
    { 0x926be935, 61 },
    { 0x66dbe935, 98 },
    { 0x8a7be935, 90 },
    { 0x2adbe935, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ebe93d, 95 },
    { 0x6dcbe93e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x977be941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccbe945, 55 },
    { 0, 0 },
    { 0xb35be947, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67be94a, 84 },
    { 0x8e8be94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x927be94e, 85 },
    { 0, 0 },
    { 0xa3cbe950, 52 },
    { 0, 0 },
    { 0xbbbe952, 98 },
    { 0x7fbe952, 95 },
    { 0xac2be952, 100 },
    { 0xd1cbe953, 46 },
    { 0x856be956, 84 },
    { 0, 0 },
    { 0x8fdbe958, 67 },
    { 0xb46be959, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96be95d, 61 },
    { 0, 0 },
    { 0x193be95f, 99 },
    { 0, 0 },
    { 0x337be961, 88 },
    { 0, 0 },
    { 0x115be963, 95 },
    { 0, 0 },
    { 0xb16be965, 66 },
    { 0x50ebe966, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1be96f, 77 },
    { 0x6a0be970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edbe973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ebe976, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbe97b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cbe97f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91abe986, 66 },
    { 0x7d0be987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4be98b, 61 },
    { 0, 0 },
    { 0xf25be98d, 98 },
    { 0xec9be98d, 85 },
    { 0x8b8be98f, 67 },
    { 0x33ebe98f, 33 },
    { 0x450be991, 23 },
    { 0xfa7be992, 88 },
    { 0xf36be991, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453be997, 33 },
    { 0x9e9be997, 40 },
    { 0x813be999, 34 },
    { 0, 0 },
    { 0x999be99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06be99f, 23 },
    { 0x5c2be9a0, 55 },
    { 0x187be99f, 90 },
    { 0x38dbe9a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efbe9a5, 61 },
    { 0x541be9a5, 34 },
    { 0xb34be9a5, 33 },
    { 0x8cbbe9a8, 23 },
    { 0x8cabe9a9, 68 },
    { 0x5d8be9a8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecbe9ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12be9bd, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70be9c3, 98 },
    { 0xf3ebe9c3, 55 },
    { 0, 0 },
    { 0xe51be9c6, 85 },
    { 0, 0 },
    { 0xffcbe9c8, 33 },
    { 0x69abe9c9, 23 },
    { 0, 0 },
    { 0x437be9cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67abe9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890be9d5, 81 },
    { 0, 0 },
    { 0xcfabe9d7, 90 },
    { 0, 0 },
    { 0xc78be9d9, 90 },
    { 0, 0 },
    { 0x1fbe9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ebe9df, 61 },
    { 0, 0 },
    { 0xf87be9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946be9e5, 61 },
    { 0x43dbe9e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdbe9ee, 88 },
    { 0, 0 },
    { 0x117be9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bbe9f5, 76 },
    { 0, 0 },
    { 0x429be9f7, 87 },
    { 0xe5abe9f8, 33 },
    { 0xceebe9f7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ebe9fd, 33 },
    { 0x34fbe9fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621bea02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469bea07, 95 },
    { 0, 0 },
    { 0xcbdbea09, 95 },
    { 0, 0 },
    { 0xbc8bea0b, 46 },
    { 0x56ebea0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67bea11, 52 },
    { 0x66bbea12, 61 },
    { 0x9dbbea13, 33 },
    { 0xdb6bea13, 61 },
    { 0x6e5bea12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabea19, 95 },
    { 0, 0 },
    { 0xc4cbea1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3bea1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcbea23, 52 },
    { 0, 0 },
    { 0xb41bea25, 90 },
    { 0, 0 },
    { 0x21dbea27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192bea33, 61 },
    { 0xa98bea33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcbea3a, 61 },
    { 0x13dbea3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57bea41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fbea47, 95 },
    { 0xc12bea47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6bea4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bea50, 78 },
    { 0, 0 },
    { 0x53ebea52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dabea55, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0bea5c, 61 },
    { 0x64cbea5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dbea60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740bea66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755bea6b, 66 },
    { 0x294bea6b, 61 },
    { 0xf39bea6d, 98 },
    { 0x689bea6c, 33 },
    { 0xfbabea6c, 85 },
    { 0x42bea6e, 33 },
    { 0x265bea71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bea76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bbea7c, 95 },
    { 0, 0 },
    { 0x1edbea7e, 69 },
    { 0xbe1bea7f, 93 },
    { 0x7e1bea80, 93 },
    { 0x759bea7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fbea85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbea93, 85 },
    { 0xa3dbea93, 90 },
    { 0x554bea95, 87 },
    { 0, 0 },
    { 0x35dbea97, 95 },
    { 0x69dbea98, 90 },
    { 0x4a1bea98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdbea9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6beaa0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbeaa4, 23 },
    { 0xa8ebeaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dabeaa8, 40 },
    { 0x5dbbeaa9, 61 },
    { 0xd51beaa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dbeaae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2beaba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbeabd, 43 },
    { 0x997beabe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8beac1, 95 },
    { 0x8fbeac1, 36 },
    { 0xf6cbeac3, 67 },
    { 0x10cbeac4, 67 },
    { 0xcf5beac4, 99 },
    { 0, 0 },
    { 0xdd2beac7, 61 },
    { 0xafbbeac7, 68 },
    { 0x8f3beac7, 61 },
    { 0xf15beaca, 98 },
    { 0x49dbeaca, 65 },
    { 0x2beacc, 33 },
    { 0xfc0beaca, 55 },
    { 0x48fbeacd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896bead3, 93 },
    { 0x8dbbead3, 90 },
    { 0xa5ebead5, 61 },
    { 0, 0 },
    { 0xfcebead7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x340beada, 46 },
    { 0xfb8beada, 46 },
    { 0xc0cbeada, 91 },
    { 0x128beadd, 77 },
    { 0xf6abeadd, 23 },
    { 0xe57beadf, 33 },
    { 0x22cbeadf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509beae5, 61 },
    { 0xb76beae6, 76 },
    { 0xdc3beae7, 34 },
    { 0xb4ebeae7, 90 },
    { 0, 0 },
    { 0xc15beaea, 23 },
    { 0x144beaeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x110beaee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dbeaf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57beafa, 91 },
    { 0, 0 },
    { 0x998beafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847beb01, 61 },
    { 0xe13beb01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171beb0a, 93 },
    { 0xe68beb0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944beb13, 98 },
    { 0xe0dbeb13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecbeb18, 95 },
    { 0x8afbeb18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896beb23, 33 },
    { 0x653beb23, 93 },
    { 0x203beb23, 61 },
    { 0x9bbeb26, 90 },
    { 0, 0 },
    { 0x916beb28, 33 },
    { 0x631beb29, 52 },
    { 0x7e7beb28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x129beb2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecbeb35, 93 },
    { 0xe05beb35, 33 },
    { 0x117beb36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02beb3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fbeb42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6beb45, 90 },
    { 0xd5ebeb45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdbeb4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccbeb4f, 61 },
    { 0, 0 },
    { 0xb9bbeb51, 46 },
    { 0x556beb52, 76 },
    { 0, 0 },
    { 0x1d3beb54, 76 },
    { 0x5b3beb55, 66 },
    { 0xb3beb56, 90 },
    { 0x125beb54, 98 },
    { 0x906beb58, 33 },
    { 0, 0 },
    { 0xe97beb5a, 34 },
    { 0x6f6beb5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65beb5e, 61 },
    { 0x919beb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831beb65, 90 },
    { 0, 0 },
    { 0xd6abeb67, 33 },
    { 0x973beb68, 43 },
    { 0xac1beb69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bbeb72, 34 },
    { 0x140beb73, 68 },
    { 0x82fbeb74, 61 },
    { 0x1d9beb75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fbeb78, 98 },
    { 0x59fbeb78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beb7f, 95 },
    { 0x4f7beb80, 95 },
    { 0, 0 },
    { 0x185beb82, 95 },
    { 0xbf0beb82, 61 },
    { 0xa3cbeb82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x224beb87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fbeb8b, 43 },
    { 0xa39beb8c, 65 },
    { 0, 0 },
    { 0x17bbeb8e, 34 },
    { 0x31beb8e, 90 },
    { 0x8f7beb90, 61 },
    { 0, 0 },
    { 0x6a1beb92, 98 },
    { 0x5e8beb92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43beb99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bbeba2, 46 },
    { 0x987beba2, 23 },
    { 0, 0 },
    { 0x9d6beba5, 76 },
    { 0x465beba5, 98 },
    { 0x5c8beba7, 78 },
    { 0, 0 },
    { 0x2f4beba9, 93 },
    { 0x8cbebaa, 66 },
    { 0xac7bebab, 98 },
    { 0x8d3bebac, 52 },
    { 0x53bbebad, 95 },
    { 0, 0 },
    { 0xcc2bebaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbebb4, 76 },
    { 0xd6bebb4, 84 },
    { 0x28ebebb5, 33 },
    { 0x14fbebb4, 23 },
    { 0x6f9bebb4, 33 },
    { 0x9ecbebb8, 23 },
    { 0x39bebba, 43 },
    { 0, 0 },
    { 0x537bebbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1bebbf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195bebc3, 90 },
    { 0xba7bebc4, 98 },
    { 0x95fbebc3, 61 },
    { 0, 0 },
    { 0xb32bebc7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fbebd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5bebd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3bebdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddabebdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5bebe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fbebe8, 90 },
    { 0, 0 },
    { 0x191bebea, 43 },
    { 0xf85bebeb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bebee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5bebf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31bebf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ebebfc, 43 },
    { 0x2ddbebfd, 52 },
    { 0x449bebfc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0bec02, 23 },
    { 0x1d2bec03, 66 },
    { 0x32fbec04, 68 },
    { 0x2c9bec05, 61 },
    { 0, 0 },
    { 0xc5ebec07, 33 },
    { 0, 0 },
    { 0xe6cbec09, 52 },
    { 0, 0 },
    { 0x348bec0b, 55 },
    { 0x417bec0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6bec12, 23 },
    { 0xd3bbec13, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x646bec16, 90 },
    { 0, 0 },
    { 0xf82bec18, 33 },
    { 0xdf2bec19, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dbec1d, 55 },
    { 0, 0 },
    { 0x2ebec1f, 98 },
    { 0x2b9bec1f, 52 },
    { 0, 0 },
    { 0x8f5bec22, 87 },
    { 0, 0 },
    { 0x9e2bec24, 90 },
    { 0xa33bec24, 59 },
    { 0x456bec26, 61 },
    { 0x786bec27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dbec34, 98 },
    { 0x72ebec34, 19 },
    { 0xc55bec36, 68 },
    { 0x301bec34, 61 },
    { 0x921bec38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x742bec3b, 98 },
    { 0, 0 },
    { 0x328bec3d, 61 },
    { 0x961bec3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbec42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157bec50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5bec53, 91 },
    { 0, 0 },
    { 0x2eebec55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34bec58, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2bec5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09bec60, 17 },
    { 0x1f6bec61, 84 },
    { 0xaebec61, 98 },
    { 0, 0 },
    { 0x321bec64, 55 },
    { 0, 0 },
    { 0x7e4bec66, 94 },
    { 0xcdfbec67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07bec6c, 80 },
    { 0, 0 },
    { 0x83bec6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbebec74, 34 },
    { 0, 0 },
    { 0xfb3bec76, 34 },
    { 0xdbbbec77, 33 },
    { 0x665bec77, 99 },
    { 0x403bec79, 23 },
    { 0xfbdbec7a, 100 },
    { 0x35cbec79, 23 },
    { 0x6e3bec78, 84 },
    { 0x756bec7d, 76 },
    { 0, 0 },
    { 0x9e2bec7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01bec83, 23 },
    { 0x420bec84, 61 },
    { 0x6d7bec85, 81 },
    { 0xa4ebec85, 33 },
    { 0xe1bec86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9bec8d, 55 },
    { 0xa07bec8d, 33 },
    { 0xe72bec8f, 90 },
    { 0x93cbec90, 84 },
    { 0, 0 },
    { 0x9bebec92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633bec96, 23 },
    { 0, 0 },
    { 0xa8bec98, 61 },
    { 0, 0 },
    { 0x6c5bec9a, 66 },
    { 0, 0 },
    { 0x50dbec9c, 34 },
    { 0x11fbec9c, 90 },
    { 0x9e2bec9e, 85 },
    { 0, 0 },
    { 0x6cfbeca0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dbeca7, 76 },
    { 0, 0 },
    { 0xe03beca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ebecb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462becb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18becb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbecbd, 33 },
    { 0x9f4becbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68becc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ebeccb, 33 },
    { 0x32bbeccc, 90 },
    { 0, 0 },
    { 0xa1ebecce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbecda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbecde, 98 },
    { 0xe69becdf, 90 },
    { 0xa4abece0, 55 },
    { 0x928becdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6febece5, 23 },
    { 0xc61bece6, 33 },
    { 0, 0 },
    { 0xa32bece8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbecec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x737becef, 78 },
    { 0x648becf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1bed02, 95 },
    { 0xdf7bed02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52bed0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbed0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0bed14, 33 },
    { 0x106bed15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0bed18, 34 },
    { 0, 0 },
    { 0x21bed1a, 33 },
    { 0x365bed1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x162bed1e, 43 },
    { 0xbccbed1e, 95 },
    { 0x1fbed20, 33 },
    { 0xeadbed20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0bed2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edbed38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830bed3c, 55 },
    { 0xa82bed3d, 90 },
    { 0xdabbed3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ebed45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806bed49, 34 },
    { 0xdaabed4a, 94 },
    { 0xb03bed49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56bed4e, 93 },
    { 0, 0 },
    { 0xf44bed50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ebed56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbed5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32bed67, 67 },
    { 0, 0 },
    { 0xd12bed69, 33 },
    { 0, 0 },
    { 0x703bed6b, 68 },
    { 0xe63bed6c, 90 },
    { 0x21abed6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4bed70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bed75, 46 },
    { 0, 0 },
    { 0x82abed77, 95 },
    { 0xed0bed78, 90 },
    { 0x3fdbed78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bed7d, 81 },
    { 0x6d6bed7d, 46 },
    { 0, 0 },
    { 0xacabed80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6bed88, 90 },
    { 0xb0ebed89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bed8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72bed92, 43 },
    { 0xa7bed93, 76 },
    { 0x812bed94, 61 },
    { 0xeb4bed95, 77 },
    { 0, 0 },
    { 0x4b2bed97, 91 },
    { 0x1f0bed97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3febed9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dbeda2, 69 },
    { 0x61cbeda2, 78 },
    { 0x9cfbeda3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664beda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26bedac, 42 },
    { 0x8a7bedad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbedb0, 23 },
    { 0, 0 },
    { 0x109bedb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54bedb8, 33 },
    { 0, 0 },
    { 0x252bedba, 90 },
    { 0, 0 },
    { 0xdebbedbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58bedc0, 98 },
    { 0x864bedc0, 61 },
    { 0x342bedc2, 65 },
    { 0x632bedc3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149bedc8, 46 },
    { 0xa20bedc9, 98 },
    { 0xb3bbedca, 87 },
    { 0xf53bedcb, 33 },
    { 0x846bedc8, 98 },
    { 0x457bedcd, 61 },
    { 0xc48bedc8, 33 },
    { 0, 0 },
    { 0x3afbedd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bedd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07bede3, 55 },
    { 0, 0 },
    { 0xff0bede5, 95 },
    { 0x4b1bede6, 23 },
    { 0x8cbede7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310bedeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0bedf0, 53 },
    { 0x517bedf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bbedf5, 67 },
    { 0x872bedf5, 43 },
    { 0xdc6bedf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622bedfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926bee02, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bee08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0bee0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbee10, 95 },
    { 0, 0 },
    { 0xda8bee12, 85 },
    { 0x391bee13, 84 },
    { 0x5edbee12, 78 },
    { 0x4d0bee15, 33 },
    { 0x3e0bee16, 23 },
    { 0xe76bee15, 98 },
    { 0xab1bee18, 43 },
    { 0x70abee17, 90 },
    { 0xad1bee13, 40 },
    { 0xc6ebee17, 34 },
    { 0, 0 },
    { 0x877bee1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ebee21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbee24, 43 },
    { 0xb7bee25, 58 },
    { 0x408bee24, 33 },
    { 0, 0 },
    { 0xc7bbee28, 90 },
    { 0x466bee29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9bee2e, 33 },
    { 0, 0 },
    { 0x395bee30, 90 },
    { 0x6bebee31, 90 },
    { 0, 0 },
    { 0xf68bee33, 90 },
    { 0x5c6bee33, 95 },
    { 0, 0 },
    { 0xcf3bee36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbee39, 95 },
    { 0xeb9bee39, 33 },
    { 0x18bee3b, 23 },
    { 0x5dfbee39, 98 },
    { 0xa5bbee39, 40 },
    { 0xdcbee3e, 76 },
    { 0xde3bee3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03bee42, 98 },
    { 0x15dbee43, 67 },
    { 0xf34bee43, 33 },
    { 0xf19bee45, 90 },
    { 0, 0 },
    { 0xfa3bee47, 98 },
    { 0xcd7bee47, 98 },
    { 0xd0fbee47, 95 },
    { 0, 0 },
    { 0x2f7bee4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388bee50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621bee55, 61 },
    { 0x614bee56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64bee59, 90 },
    { 0x8c5bee5a, 90 },
    { 0x376bee5b, 84 },
    { 0, 0 },
    { 0x6bbbee5d, 33 },
    { 0x90abee5e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8bee65, 98 },
    { 0xaf2bee66, 90 },
    { 0, 0 },
    { 0x1e1bee68, 90 },
    { 0xe4abee69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9bee6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14bee71, 23 },
    { 0xa09bee72, 88 },
    { 0x971bee73, 99 },
    { 0xc4dbee74, 98 },
    { 0xeb2bee71, 90 },
    { 0xdc5bee73, 43 },
    { 0x171bee77, 95 },
    { 0, 0 },
    { 0x425bee79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1bee7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9bee83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fbee8a, 61 },
    { 0, 0 },
    { 0xaa9bee8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561bee91, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783bee95, 17 },
    { 0x799bee96, 90 },
    { 0x3b8bee96, 90 },
    { 0, 0 },
    { 0xeabbee99, 98 },
    { 0xed4bee99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x419bee9d, 67 },
    { 0xdffbee9e, 43 },
    { 0xa4abee9f, 98 },
    { 0xe2cbee9d, 85 },
    { 0xefabee9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfbeea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbeeaf, 88 },
    { 0x3e5beeaf, 88 },
    { 0xfabbeeb0, 33 },
    { 0x3d0beeb2, 61 },
    { 0xae4beeb3, 66 },
    { 0x6acbeeb3, 61 },
    { 0x970beeb3, 43 },
    { 0, 0 },
    { 0x952beeb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebeebe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253beec3, 99 },
    { 0x98ebeec4, 23 },
    { 0x9a7beec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4beec8, 100 },
    { 0, 0 },
    { 0xacabeeca, 78 },
    { 0xd10beeca, 93 },
    { 0xcfbbeecc, 35 },
    { 0x513beecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x627beed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1beed3, 98 },
    { 0x4efbeed4, 95 },
    { 0xfeebeed5, 52 },
    { 0xf4cbeed3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06beed9, 98 },
    { 0x6cdbeed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777beede, 33 },
    { 0xe87beedf, 33 },
    { 0x9c4beee0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x620beee3, 78 },
    { 0x227beee4, 61 },
    { 0xe45beee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cbeeea, 61 },
    { 0x9c7beeea, 68 },
    { 0x883beeea, 76 },
    { 0, 0 },
    { 0x1a1beeee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71beef6, 52 },
    { 0x19fbeef6, 90 },
    { 0x117beef8, 76 },
    { 0x348beef8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bbeefd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfbef03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ebef07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88bef0c, 95 },
    { 0x4dbbef0d, 61 },
    { 0x8cabef0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbef11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609bef1c, 84 },
    { 0, 0 },
    { 0x49cbef1e, 46 },
    { 0x951bef1e, 98 },
    { 0, 0 },
    { 0x175bef21, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bef25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83bef29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21bef2c, 52 },
    { 0x2d7bef2c, 90 },
    { 0, 0 },
    { 0x3b6bef2f, 90 },
    { 0, 0 },
    { 0x777bef31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ebef39, 90 },
    { 0xe19bef3a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5bef3e, 90 },
    { 0x131bef3f, 90 },
    { 0x3c9bef40, 76 },
    { 0, 0 },
    { 0xf5fbef42, 33 },
    { 0, 0 },
    { 0x23dbef44, 69 },
    { 0x769bef45, 84 },
    { 0x960bef44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45abef49, 95 },
    { 0, 0 },
    { 0x40ebef4b, 90 },
    { 0x43fbef4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ebef51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dbef56, 67 },
    { 0x918bef57, 46 },
    { 0x188bef58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3bef5b, 33 },
    { 0, 0 },
    { 0x717bef5d, 99 },
    { 0, 0 },
    { 0x7dbbef5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecbef63, 52 },
    { 0x4dbbef63, 55 },
    { 0x245bef64, 100 },
    { 0, 0 },
    { 0xd0bef67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07bef6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1bef7b, 90 },
    { 0xb15bef7b, 95 },
    { 0x459bef7d, 81 },
    { 0xe25bef7e, 76 },
    { 0x9c9bef7d, 33 },
    { 0x3c3bef80, 33 },
    { 0x858bef7c, 98 },
    { 0xf4bef7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bef85, 98 },
    { 0x1c6bef86, 33 },
    { 0x764bef87, 90 },
    { 0x2debef87, 61 },
    { 0xaf0bef88, 81 },
    { 0, 0 },
    { 0xf0bef8b, 61 },
    { 0xe3bbef8b, 23 },
    { 0x871bef8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304bef93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111bef97, 84 },
    { 0, 0 },
    { 0x49ebef99, 99 },
    { 0, 0 },
    { 0xbe7bef9b, 90 },
    { 0xfc3bef9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3befa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbbefa8, 33 },
    { 0x424befa9, 95 },
    { 0, 0 },
    { 0x51dbefab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabefb0, 76 },
    { 0, 0 },
    { 0xe43befb2, 97 },
    { 0xababefb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbefbb, 33 },
    { 0x316befbb, 55 },
    { 0xfbfbefbd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412befc2, 91 },
    { 0xe04befc3, 100 },
    { 0x1bdbefc3, 85 },
    { 0xf74befc5, 100 },
    { 0x665befc6, 23 },
    { 0x341befc4, 90 },
    { 0x90abefc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddbefcc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498befd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbefd5, 98 },
    { 0, 0 },
    { 0xee7befd7, 78 },
    { 0, 0 },
    { 0x184befd9, 90 },
    { 0xe23befda, 34 },
    { 0x582befda, 100 },
    { 0x663befdc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecbefe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30befe3, 95 },
    { 0x9b6befe4, 87 },
    { 0xb30befe3, 67 },
    { 0, 0 },
    { 0x3c8befe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0befea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fbeff0, 36 },
    { 0, 0 },
    { 0xd2dbeff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21beff7, 79 },
    { 0, 0 },
    { 0x1ebeff9, 23 },
    { 0x679beffa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73beffd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02bf009, 95 },
    { 0xcd0bf00a, 55 },
    { 0xa60bf00b, 87 },
    { 0x330bf00a, 90 },
    { 0x2d7bf00c, 33 },
    { 0, 0 },
    { 0xc8fbf00f, 61 },
    { 0x74dbf010, 33 },
    { 0x4a1bf011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bf014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cbf017, 55 },
    { 0x629bf018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbbf01b, 98 },
    { 0xf1cbf01c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11bf01f, 43 },
    { 0xfc3bf020, 33 },
    { 0x4e0bf020, 77 },
    { 0x6fbf021, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ebf025, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbbf02f, 78 },
    { 0xd4cbf030, 8 },
    { 0xce6bf031, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bf03a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbf03e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810bf046, 84 },
    { 0x470bf047, 90 },
    { 0, 0 },
    { 0xe14bf049, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabf04c, 95 },
    { 0x9f4bf04d, 81 },
    { 0x2d4bf04e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x310bf051, 76 },
    { 0, 0 },
    { 0x25dbf053, 66 },
    { 0x36cbf054, 98 },
    { 0, 0 },
    { 0x8e4bf056, 33 },
    { 0x837bf057, 33 },
    { 0xecbbf058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x608bf05b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bf05e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7bf063, 52 },
    { 0xa76bf064, 87 },
    { 0x714bf065, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabf069, 99 },
    { 0x41cbf069, 90 },
    { 0, 0 },
    { 0x7b4bf06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dbf075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbf078, 67 },
    { 0x933bf079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbf07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5bf080, 95 },
    { 0, 0 },
    { 0xc93bf082, 46 },
    { 0xd0cbf083, 75 },
    { 0xa62bf082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ebf088, 33 },
    { 0, 0 },
    { 0x30bf08a, 55 },
    { 0xf67bf08b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1bf08f, 55 },
    { 0x701bf090, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46bf095, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6bf09c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ebf0a3, 98 },
    { 0xb00bf0a3, 33 },
    { 0, 0 },
    { 0x5f6bf0a6, 90 },
    { 0x345bf0a6, 90 },
    { 0, 0 },
    { 0xeffbf0a9, 55 },
    { 0, 0 },
    { 0xe5cbf0ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabbf0b1, 98 },
    { 0, 0 },
    { 0xa43bf0b3, 23 },
    { 0, 0 },
    { 0x783bf0b5, 87 },
    { 0xed4bf0b6, 90 },
    { 0x120bf0b7, 61 },
    { 0xda9bf0b7, 68 },
    { 0x541bf0b7, 95 },
    { 0x454bf0b6, 90 },
    { 0xf1ebf0b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23bf0be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99bf0c6, 76 },
    { 0x276bf0c6, 98 },
    { 0, 0 },
    { 0xb3fbf0c9, 68 },
    { 0, 0 },
    { 0x6acbf0cb, 90 },
    { 0, 0 },
    { 0xeedbf0cd, 91 },
    { 0x767bf0ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bf0d2, 85 },
    { 0, 0 },
    { 0x963bf0d4, 23 },
    { 0x126bf0d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58bf0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4bf0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8bf0e4, 61 },
    { 0, 0 },
    { 0x2dbbf0e6, 23 },
    { 0x1d6bf0e7, 98 },
    { 0xd34bf0e8, 61 },
    { 0xba4bf0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfbf0ed, 76 },
    { 0x19bf0ee, 76 },
    { 0xc60bf0ef, 61 },
    { 0x741bf0ee, 76 },
    { 0x83fbf0ef, 22 },
    { 0x64ebf0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebf0fa, 55 },
    { 0, 0 },
    { 0x5a8bf0fc, 77 },
    { 0xd1cbf0fd, 52 },
    { 0, 0 },
    { 0x89bbf0ff, 78 },
    { 0x7a3bf100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22bf103, 55 },
    { 0, 0 },
    { 0xa4ebf105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75bf10e, 90 },
    { 0x179bf10f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2bf112, 66 },
    { 0x868bf113, 90 },
    { 0, 0 },
    { 0x4acbf115, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2bf118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0bf11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4bf124, 100 },
    { 0, 0 },
    { 0x16bbf126, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4bf12b, 23 },
    { 0x457bf12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdbf133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bbf138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bbf13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbf141, 98 },
    { 0xb82bf142, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8bf14b, 33 },
    { 0, 0 },
    { 0x3c1bf14d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x122bf150, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bbf154, 95 },
    { 0xb0bbf154, 90 },
    { 0x7debf154, 43 },
    { 0x2f1bf155, 99 },
    { 0x82bf158, 48 },
    { 0xd31bf154, 82 },
    { 0x17bf15a, 36 },
    { 0xdd3bf15b, 33 },
    { 0, 0 },
    { 0x67bbf15d, 90 },
    { 0xd31bf15e, 90 },
    { 0xb91bf15f, 93 },
    { 0xa3bf160, 59 },
    { 0x84bf161, 33 },
    { 0x961bf161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64bf166, 34 },
    { 0xd6fbf166, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326bf16c, 33 },
    { 0, 0 },
    { 0x3eabf16e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bf176, 90 },
    { 0xcedbf177, 97 },
    { 0xfa2bf176, 43 },
    { 0xc0cbf179, 76 },
    { 0xb10bf179, 66 },
    { 0x42fbf179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bf17f, 75 },
    { 0x41cbf180, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dbf18f, 43 },
    { 0x326bf18f, 96 },
    { 0xca8bf191, 34 },
    { 0xd89bf191, 90 },
    { 0x323bf193, 91 },
    { 0xf0fbf193, 98 },
    { 0xeddbf195, 88 },
    { 0xdc7bf194, 53 },
    { 0x1fcbf197, 55 },
    { 0, 0 },
    { 0x447bf199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbf1a2, 17 },
    { 0x4a9bf1a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dbf1a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09bf1ab, 98 },
    { 0, 0 },
    { 0xc4cbf1ad, 99 },
    { 0, 0 },
    { 0x96fbf1af, 33 },
    { 0, 0 },
    { 0xa79bf1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933bf1ba, 43 },
    { 0xbe3bf1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bf1be, 98 },
    { 0x7b7bf1be, 99 },
    { 0x5bbf1bf, 90 },
    { 0xacabf1c1, 55 },
    { 0xd9fbf1c1, 33 },
    { 0x6f0bf1c3, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8bf1c6, 76 },
    { 0x7cebf1c7, 52 },
    { 0x8b1bf1c8, 76 },
    { 0x4debf1c9, 34 },
    { 0x10abf1c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95bf1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9abf1d9, 55 },
    { 0x7b4bf1da, 84 },
    { 0x6e7bf1d9, 61 },
    { 0x503bf1d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcbf1df, 90 },
    { 0, 0 },
    { 0x400bf1e1, 33 },
    { 0x52ebf1e2, 75 },
    { 0, 0 },
    { 0x7f6bf1e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bbf1e7, 95 },
    { 0x645bf1e8, 38 },
    { 0x38dbf1e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75bf1ef, 33 },
    { 0, 0 },
    { 0x886bf1f1, 90 },
    { 0x138bf1f2, 33 },
    { 0, 0 },
    { 0x32bbf1f4, 34 },
    { 0xadbf1f4, 43 },
    { 0x6b0bf1f6, 33 },
    { 0, 0 },
    { 0x397bf1f8, 33 },
    { 0x374bf1f8, 33 },
    { 0x446bf1f9, 90 },
    { 0xbd4bf1fb, 19 },
    { 0x12fbf1fc, 76 },
    { 0, 0 },
    { 0x26fbf1fe, 34 },
    { 0, 0 },
    { 0x3f2bf200, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3babf206, 33 },
    { 0xdc4bf207, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815bf20d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bf210, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bbf213, 33 },
    { 0, 0 },
    { 0xa0ebf215, 61 },
    { 0x41ebf216, 23 },
    { 0xaebf217, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72bf21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbf21e, 33 },
    { 0x710bf21f, 61 },
    { 0, 0 },
    { 0x5f4bf221, 76 },
    { 0xf07bf222, 81 },
    { 0, 0 },
    { 0x4adbf224, 33 },
    { 0xf93bf224, 93 },
    { 0xea2bf226, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64bf22b, 55 },
    { 0xaf2bf22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cbf231, 52 },
    { 0x631bf231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32bf237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ebf23a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x539bf23d, 23 },
    { 0, 0 },
    { 0xb6bf23f, 98 },
    { 0x9e4bf240, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0bf244, 46 },
    { 0x6f4bf245, 76 },
    { 0x52dbf246, 43 },
    { 0x26bbf247, 100 },
    { 0xe08bf247, 33 },
    { 0x161bf244, 67 },
    { 0x518bf245, 98 },
    { 0x9e3bf24b, 33 },
    { 0, 0 },
    { 0xe0bf24d, 90 },
    { 0, 0 },
    { 0x97fbf24f, 17 },
    { 0x790bf250, 67 },
    { 0xd51bf251, 55 },
    { 0xaa6bf251, 95 },
    { 0xa17bf253, 33 },
    { 0, 0 },
    { 0xe0bbf255, 98 },
    { 0x3b5bf256, 61 },
    { 0x7dabf255, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x988bf25a, 17 },
    { 0x69cbf25b, 90 },
    { 0x1c1bf25c, 95 },
    { 0x963bf25d, 93 },
    { 0, 0 },
    { 0xba4bf25f, 84 },
    { 0, 0 },
    { 0x9acbf261, 95 },
    { 0, 0 },
    { 0x55fbf263, 53 },
    { 0x416bf264, 90 },
    { 0x939bf265, 52 },
    { 0xad9bf266, 61 },
    { 0x29fbf267, 95 },
    { 0x58fbf267, 98 },
    { 0xe1ebf266, 37 },
    { 0x8bf263, 90 },
    { 0x38bf26b, 90 },
    { 0x232bf264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0bf26f, 19 },
    { 0xb77bf270, 23 },
    { 0xefbf270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bbf274, 78 },
    { 0xf37bf274, 53 },
    { 0x4a7bf274, 98 },
    { 0xa90bf277, 33 },
    { 0xe61bf275, 33 },
    { 0, 0 },
    { 0xbf8bf27a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bbf27d, 33 },
    { 0x4ebbf27e, 52 },
    { 0x6fbf27d, 90 },
    { 0xe66bf27f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bf284, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17bf28d, 33 },
    { 0, 0 },
    { 0xcaebf28f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6bf292, 23 },
    { 0, 0 },
    { 0xd78bf294, 100 },
    { 0, 0 },
    { 0x75bbf296, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdbf29b, 76 },
    { 0x4b3bf29c, 90 },
    { 0xe52bf29b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fbf2a0, 33 },
    { 0, 0 },
    { 0xc29bf2a2, 96 },
    { 0, 0 },
    { 0x68dbf2a4, 76 },
    { 0x89ebf2a5, 93 },
    { 0xe1dbf2a5, 85 },
    { 0x1a8bf2a5, 98 },
    { 0x1eabf2a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4bf2ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fbf2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3bf2b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9bf2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bf2bf, 55 },
    { 0, 0 },
    { 0xdcabf2c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dbf2c7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bf2cc, 90 },
    { 0x12cbf2cd, 61 },
    { 0x2f6bf2ce, 34 },
    { 0xb7bbf2cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79bf2d7, 67 },
    { 0xd89bf2d8, 98 },
    { 0xf74bf2d9, 46 },
    { 0x4c7bf2da, 61 },
    { 0x603bf2d9, 95 },
    { 0xbbbf2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bbf2e0, 55 },
    { 0x38fbf2e1, 81 },
    { 0x5a7bf2e2, 77 },
    { 0xd64bf2e3, 90 },
    { 0, 0 },
    { 0xf62bf2e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3bf2ed, 91 },
    { 0x8dabf2ed, 76 },
    { 0xe64bf2ef, 23 },
    { 0, 0 },
    { 0xad9bf2f1, 99 },
    { 0, 0 },
    { 0x30abf2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cbf2f6, 95 },
    { 0x2e2bf2f7, 95 },
    { 0, 0 },
    { 0x929bf2f9, 77 },
    { 0x4a1bf2fa, 95 },
    { 0, 0 },
    { 0x2b1bf2fc, 55 },
    { 0x314bf2fc, 17 },
    { 0, 0 },
    { 0x3bbf2ff, 33 },
    { 0x72abf300, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fbf304, 55 },
    { 0xf0ebf305, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425bf30d, 95 },
    { 0x47abf30d, 90 },
    { 0, 0 },
    { 0x22dbf310, 52 },
    { 0x20ebf310, 95 },
    { 0, 0 },
    { 0x26dbf313, 23 },
    { 0x385bf313, 33 },
    { 0, 0 },
    { 0x178bf316, 88 },
    { 0x737bf317, 61 },
    { 0xd35bf318, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbf31f, 90 },
    { 0x820bf320, 61 },
    { 0x8cbf31f, 33 },
    { 0x19bbf31f, 90 },
    { 0, 0 },
    { 0xe1abf324, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abf327, 46 },
    { 0x4dfbf327, 23 },
    { 0x5d6bf327, 80 },
    { 0xe88bf329, 23 },
    { 0x42cbf32b, 61 },
    { 0xb74bf32b, 87 },
    { 0x546bf32d, 77 },
    { 0x8e2bf32d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdbf334, 66 },
    { 0xc60bf335, 34 },
    { 0x514bf336, 95 },
    { 0xdd8bf337, 91 },
    { 0x647bf336, 61 },
    { 0xb35bf339, 98 },
    { 0x5a5bf33a, 84 },
    { 0x74dbf334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fbf33f, 88 },
    { 0xb55bf340, 23 },
    { 0x4b2bf341, 52 },
    { 0x1cebf342, 90 },
    { 0xa5abf33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22bf34f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dbf352, 76 },
    { 0xfbcbf352, 76 },
    { 0, 0 },
    { 0xa79bf355, 46 },
    { 0xca6bf356, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dbf360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20bf363, 90 },
    { 0x8ebf364, 34 },
    { 0x6ebf364, 33 },
    { 0xc82bf363, 52 },
    { 0xa8ebf367, 99 },
    { 0x29bbf368, 90 },
    { 0x2a8bf368, 33 },
    { 0xadebf369, 93 },
    { 0, 0 },
    { 0x370bf36c, 43 },
    { 0, 0 },
    { 0x8b4bf36e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04bf371, 61 },
    { 0xb6dbf372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932bf376, 33 },
    { 0xadbbf377, 46 },
    { 0x22abf378, 33 },
    { 0x505bf376, 95 },
    { 0xb7dbf37a, 40 },
    { 0, 0 },
    { 0xa92bf37c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7bf381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebf388, 95 },
    { 0xabfbf388, 76 },
    { 0, 0 },
    { 0x518bf38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bf395, 33 },
    { 0x9b5bf396, 93 },
    { 0, 0 },
    { 0x57abf398, 99 },
    { 0x4babf399, 85 },
    { 0x3b7bf399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2bf39e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bf3a4, 61 },
    { 0x7abbf3a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93bf3a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715bf3b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06bf3b5, 66 },
    { 0x465bf3b6, 65 },
    { 0x12ebf3b7, 90 },
    { 0xc56bf3b8, 61 },
    { 0, 0 },
    { 0xf99bf3ba, 61 },
    { 0x876bf3ba, 61 },
    { 0, 0 },
    { 0x690bf3bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x478bf3c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fbf3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbf3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548bf3d1, 90 },
    { 0, 0 },
    { 0x8c1bf3d3, 90 },
    { 0xdbcbf3d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x841bf3d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54bf3dc, 84 },
    { 0x50cbf3dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bf3e0, 23 },
    { 0x716bf3e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66bf3e4, 100 },
    { 0xcadbf3e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206bf3e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffbf3ee, 52 },
    { 0, 0 },
    { 0x54abf3f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ebf3f3, 76 },
    { 0, 0 },
    { 0x5a2bf3f5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebf3f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3bf3fb, 23 },
    { 0xd13bf3fc, 98 },
    { 0xe2abf3fc, 93 },
    { 0xbc6bf3fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83bf403, 98 },
    { 0, 0 },
    { 0x6f4bf405, 61 },
    { 0xc6bbf406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2bf409, 90 },
    { 0x49ebf409, 99 },
    { 0xba6bf40b, 90 },
    { 0x523bf40b, 81 },
    { 0xd55bf40d, 76 },
    { 0xab0bf40e, 61 },
    { 0x7f4bf40f, 66 },
    { 0xe3bbf410, 90 },
    { 0xd61bf411, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x667bf414, 90 },
    { 0xad5bf415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92bf419, 93 },
    { 0x635bf419, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670bf423, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fbf427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bf431, 38 },
    { 0x38bbf432, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2bf436, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2bf43f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x465bf442, 61 },
    { 0x2d2bf443, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601bf447, 90 },
    { 0xe4dbf448, 90 },
    { 0x857bf449, 52 },
    { 0xbeebf449, 23 },
    { 0x79fbf44a, 17 },
    { 0x563bf449, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4bf450, 61 },
    { 0xa71bf451, 55 },
    { 0xf91bf450, 61 },
    { 0x765bf452, 67 },
    { 0xbe9bf454, 52 },
    { 0xa7abf455, 61 },
    { 0xe9fbf455, 96 },
    { 0xda8bf455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf45b, 95 },
    { 0x4d6bf45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930bf461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790bf466, 61 },
    { 0, 0 },
    { 0xa6abf468, 95 },
    { 0x821bf469, 33 },
    { 0x8f7bf469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dabf46e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bbf472, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5bf47e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76bf483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bf486, 90 },
    { 0, 0 },
    { 0x70cbf488, 61 },
    { 0xbcfbf489, 76 },
    { 0x409bf48a, 33 },
    { 0xd08bf48a, 98 },
    { 0, 0 },
    { 0x916bf48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92bf490, 33 },
    { 0x375bf490, 33 },
    { 0x4b6bf490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410bf496, 90 },
    { 0, 0 },
    { 0x8a8bf498, 43 },
    { 0xfcabf499, 23 },
    { 0, 0 },
    { 0xe38bf49b, 61 },
    { 0x13ebf49b, 93 },
    { 0xd55bf49d, 52 },
    { 0x509bf49e, 34 },
    { 0xc96bf49e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3bf4a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbf4ac, 76 },
    { 0x8fabf4ad, 61 },
    { 0xf6abf4ae, 55 },
    { 0x1bcbf4ae, 91 },
    { 0x3cebf4b0, 76 },
    { 0x365bf4b1, 84 },
    { 0x649bf4b1, 90 },
    { 0x8c7bf4b3, 77 },
    { 0, 0 },
    { 0x50ebf4b5, 46 },
    { 0, 0 },
    { 0xea1bf4b7, 99 },
    { 0xfebf4b8, 67 },
    { 0x2d8bf4b9, 76 },
    { 0x92fbf4b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bf4bf, 100 },
    { 0, 0 },
    { 0x618bf4c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5bf4c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71bf4c9, 90 },
    { 0x3d0bf4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf4ce, 43 },
    { 0x198bf4ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1bf4d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3bf4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aebf4d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0bf4dc, 23 },
    { 0xf59bf4dc, 76 },
    { 0xafebf4dc, 95 },
    { 0x778bf4df, 95 },
    { 0x2bfbf4de, 95 },
    { 0x8a2bf4e0, 33 },
    { 0x5bfbf4e2, 33 },
    { 0xf53bf4e0, 61 },
    { 0x8d9bf4dd, 61 },
    { 0x4ebbf4e1, 61 },
    { 0xd60bf4e6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bbf4e9, 33 },
    { 0xe55bf4ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbf4ed, 33 },
    { 0xd7bbf4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabf4f1, 99 },
    { 0x255bf4f1, 90 },
    { 0x450bf4f3, 91 },
    { 0, 0 },
    { 0x896bf4f5, 61 },
    { 0, 0 },
    { 0x8a4bf4f7, 77 },
    { 0x948bf4f7, 55 },
    { 0xb23bf4f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02bf4fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647bf502, 80 },
    { 0x13bbf503, 9 },
    { 0x9efbf504, 61 },
    { 0xe96bf504, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1bf508, 43 },
    { 0x4c1bf508, 23 },
    { 0, 0 },
    { 0x3e7bf50b, 81 },
    { 0x6fbf50c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759bf510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bbf514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbf517, 94 },
    { 0, 0 },
    { 0x9a4bf519, 23 },
    { 0xe38bf51a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4bf520, 90 },
    { 0, 0 },
    { 0xee5bf522, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56bf525, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bf532, 46 },
    { 0, 0 },
    { 0x29ebf534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341bf538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01bf53b, 97 },
    { 0, 0 },
    { 0xd78bf53d, 61 },
    { 0xfddbf53e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf541, 90 },
    { 0, 0 },
    { 0x78cbf543, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0bf547, 78 },
    { 0x5e8bf547, 33 },
    { 0, 0 },
    { 0x29ebf54a, 98 },
    { 0xfddbf54b, 66 },
    { 0xacfbf54a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1bf54f, 95 },
    { 0xaa3bf54f, 43 },
    { 0x6f9bf551, 61 },
    { 0x76bf552, 90 },
    { 0x775bf54f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4bf55d, 88 },
    { 0xf54bf55e, 61 },
    { 0x539bf55f, 61 },
    { 0x906bf560, 34 },
    { 0x4a9bf55e, 95 },
    { 0x674bf562, 33 },
    { 0x538bf563, 98 },
    { 0xf33bf564, 90 },
    { 0x758bf563, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87bf569, 90 },
    { 0, 0 },
    { 0xf14bf56b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbbf56e, 34 },
    { 0x91cbf56f, 90 },
    { 0x5ecbf56f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7bf573, 90 },
    { 0xcc2bf573, 66 },
    { 0x2c3bf575, 66 },
    { 0, 0 },
    { 0x794bf577, 33 },
    { 0, 0 },
    { 0xc7abf579, 23 },
    { 0x1f0bf57a, 90 },
    { 0xc46bf57b, 78 },
    { 0x7b6bf57b, 76 },
    { 0xc95bf57d, 93 },
    { 0, 0 },
    { 0xb80bf57f, 95 },
    { 0xba3bf57f, 34 },
    { 0x287bf581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eebf585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7bf589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fbf591, 76 },
    { 0x6a6bf592, 67 },
    { 0xa9bf592, 84 },
    { 0x679bf593, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf597, 84 },
    { 0x926bf598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48bf59d, 76 },
    { 0xab9bf59e, 61 },
    { 0, 0 },
    { 0x146bf5a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664bf5a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbf5a8, 43 },
    { 0, 0 },
    { 0x556bf5aa, 23 },
    { 0x300bf5aa, 46 },
    { 0x6efbf5ac, 91 },
    { 0x123bf5ad, 95 },
    { 0xee1bf5ac, 95 },
    { 0x67cbf5af, 90 },
    { 0x824bf5ae, 95 },
    { 0xf27bf5b1, 23 },
    { 0xc6dbf5b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5bf5b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5bf5bb, 66 },
    { 0, 0 },
    { 0xc33bf5bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4abf5c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abf5c5, 76 },
    { 0, 0 },
    { 0x81bf5c7, 91 },
    { 0, 0 },
    { 0x814bf5c9, 87 },
    { 0x4abbf5c9, 95 },
    { 0x747bf5ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebf5d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cbf5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37bf5db, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ebf5de, 55 },
    { 0xd57bf5df, 61 },
    { 0, 0 },
    { 0xb8bbf5e1, 61 },
    { 0xe3dbf5e1, 91 },
    { 0x8b6bf5e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ebf5e6, 61 },
    { 0x65bf5e7, 61 },
    { 0, 0 },
    { 0xda5bf5e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33bf5ee, 90 },
    { 0x196bf5ee, 43 },
    { 0, 0 },
    { 0xd2bf5f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcbf5f5, 52 },
    { 0xe39bf5f5, 43 },
    { 0x7e3bf5f7, 33 },
    { 0xe1dbf5f7, 93 },
    { 0xc6bf5f9, 23 },
    { 0x4bbf5f9, 70 },
    { 0x177bf5fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabf5ff, 61 },
    { 0xf38bf5ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dbf603, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568bf607, 95 },
    { 0xc2dbf607, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbf60d, 95 },
    { 0, 0 },
    { 0x8a8bf60f, 65 },
    { 0, 0 },
    { 0xebcbf611, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bf616, 40 },
    { 0x768bf617, 33 },
    { 0, 0 },
    { 0x507bf619, 85 },
    { 0x5e5bf61a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1bf61f, 90 },
    { 0, 0 },
    { 0x56cbf621, 85 },
    { 0xc21bf622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bf62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bf632, 33 },
    { 0xde7bf633, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30bf636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x934bf639, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14bf63d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dbf644, 90 },
    { 0, 0 },
    { 0x3fabf646, 61 },
    { 0, 0 },
    { 0x3d9bf648, 87 },
    { 0, 0 },
    { 0xfbbbf64a, 95 },
    { 0x5ecbf64a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737bf64f, 99 },
    { 0x846bf650, 33 },
    { 0xbffbf650, 23 },
    { 0x790bf652, 23 },
    { 0x5c4bf64f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x654bf656, 100 },
    { 0, 0 },
    { 0xf58bf658, 90 },
    { 0x278bf659, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dbf65d, 85 },
    { 0xc24bf65d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cbf663, 90 },
    { 0x763bf663, 87 },
    { 0x501bf665, 85 },
    { 0x21bbf666, 33 },
    { 0xd0abf666, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cebf66d, 23 },
    { 0xf0bbf66e, 90 },
    { 0x21bf66e, 95 },
    { 0x6fcbf66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86bf675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10bf678, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88bf67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bf682, 76 },
    { 0, 0 },
    { 0xbc0bf684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0bf689, 90 },
    { 0xa3dbf689, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cbf68d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebf692, 65 },
    { 0, 0 },
    { 0xf84bf694, 99 },
    { 0x414bf695, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fabf698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87bf69b, 95 },
    { 0x63dbf69b, 80 },
    { 0xeebbf69d, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bf6a6, 34 },
    { 0xdedbf6a7, 23 },
    { 0x6e1bf6a8, 33 },
    { 0x5c7bf6a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1bf6af, 61 },
    { 0, 0 },
    { 0x9b7bf6b1, 17 },
    { 0x3a9bf6b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14abf6b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aabf6c1, 33 },
    { 0x3f5bf6c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x196bf6c5, 84 },
    { 0, 0 },
    { 0x471bf6c7, 91 },
    { 0xd5abf6c7, 37 },
    { 0, 0 },
    { 0xc78bf6ca, 93 },
    { 0, 0 },
    { 0xfabbf6cc, 55 },
    { 0xc40bf6cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfbf6d5, 33 },
    { 0xec3bf6d6, 76 },
    { 0x1c9bf6d7, 23 },
    { 0, 0 },
    { 0xcb9bf6d9, 61 },
    { 0xa27bf6da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbf6e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccbf6e5, 34 },
    { 0x174bf6e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdbf6ef, 76 },
    { 0x10bf6f0, 76 },
    { 0, 0 },
    { 0x8f6bf6f2, 61 },
    { 0x924bf6f3, 80 },
    { 0xd01bf6f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4bf6fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63bf6fd, 61 },
    { 0xfb0bf6fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0bf703, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fbf710, 85 },
    { 0x6b5bf710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514bf716, 66 },
    { 0, 0 },
    { 0xbfbf718, 43 },
    { 0xf68bf719, 98 },
    { 0, 0 },
    { 0x62fbf71b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84bf723, 61 },
    { 0x4d5bf723, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8bf72b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bf72e, 43 },
    { 0, 0 },
    { 0x104bf730, 38 },
    { 0x155bf731, 69 },
    { 0xffabf732, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831bf73e, 93 },
    { 0, 0 },
    { 0x1d3bf740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5bf744, 23 },
    { 0xcdabf744, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0bf752, 53 },
    { 0xc3bf752, 69 },
    { 0x305bf752, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cbf757, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86bf75e, 85 },
    { 0x6cebf75f, 93 },
    { 0xaddbf75f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebbf764, 61 },
    { 0x25dbf764, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fbf76f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3abf772, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x965bf775, 33 },
    { 0, 0 },
    { 0xf4dbf777, 61 },
    { 0x6c6bf778, 90 },
    { 0x4cbbf777, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03bf781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29abf787, 85 },
    { 0, 0 },
    { 0x5b4bf789, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bf78d, 87 },
    { 0xf17bf78e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01bf791, 86 },
    { 0, 0 },
    { 0xc2ebf793, 81 },
    { 0x106bf793, 33 },
    { 0x852bf795, 68 },
    { 0x8ccbf795, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71bf79a, 95 },
    { 0x1d9bf79b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5bf7a3, 90 },
    { 0xbddbf7a3, 98 },
    { 0x2f4bf7a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x136bf7a8, 43 },
    { 0, 0 },
    { 0x592bf7aa, 55 },
    { 0xb70bf7ab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bf7af, 33 },
    { 0x969bf7b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dbf7b5, 46 },
    { 0xebfbf7b5, 84 },
    { 0x150bf7b5, 23 },
    { 0xe1bbf7b8, 80 },
    { 0x860bf7b9, 90 },
    { 0x42fbf7b5, 33 },
    { 0x17bbf7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95bf7c4, 98 },
    { 0xd08bf7c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffbf7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bf7cd, 77 },
    { 0, 0 },
    { 0x7f0bf7cf, 23 },
    { 0x80fbf7d0, 87 },
    { 0x273bf7d1, 87 },
    { 0xc73bf7d0, 76 },
    { 0xf7bbf7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05bf7d6, 90 },
    { 0x4b6bf7d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bbf7dc, 88 },
    { 0x167bf7dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91bf7e4, 23 },
    { 0, 0 },
    { 0x6bf7e6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10abf7ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bf7ef, 23 },
    { 0x83ebf7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29abf7f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbbf7f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3bf7fc, 93 },
    { 0xc34bf7fc, 76 },
    { 0xe85bf7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481bf802, 68 },
    { 0x6cbbf803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57bf806, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x746bf809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404bf80d, 68 },
    { 0x3f2bf80d, 90 },
    { 0x19fbf80f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481bf819, 90 },
    { 0, 0 },
    { 0x288bf81b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cbf823, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bf826, 23 },
    { 0x614bf826, 95 },
    { 0xffebf828, 93 },
    { 0, 0 },
    { 0x109bf82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922bf82e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bf838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bf83b, 98 },
    { 0xcd7bf83c, 61 },
    { 0x589bf83c, 91 },
    { 0x3f5bf83c, 61 },
    { 0x26cbf83e, 98 },
    { 0x4f9bf840, 23 },
    { 0xe94bf841, 85 },
    { 0x897bf842, 100 },
    { 0x8aebf843, 90 },
    { 0x946bf841, 55 },
    { 0x650bf842, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aebf848, 23 },
    { 0xa14bf849, 61 },
    { 0, 0 },
    { 0xd0dbf84b, 90 },
    { 0xa66bf84c, 76 },
    { 0x55bf84d, 33 },
    { 0, 0 },
    { 0xe5abf84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x171bf852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bbf85b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0bf860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbf86c, 85 },
    { 0xfbcbf86c, 55 },
    { 0, 0 },
    { 0xd26bf86f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abf874, 98 },
    { 0x4e3bf875, 34 },
    { 0x563bf875, 76 },
    { 0, 0 },
    { 0x1b2bf878, 8 },
    { 0x336bf878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05bf87c, 17 },
    { 0x51dbf87d, 76 },
    { 0x258bf87e, 98 },
    { 0, 0 },
    { 0x67fbf880, 61 },
    { 0, 0 },
    { 0x19bbf882, 90 },
    { 0, 0 },
    { 0x6edbf884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985bf889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04bf88c, 93 },
    { 0x340bf88d, 61 },
    { 0x26abf88d, 76 },
    { 0, 0 },
    { 0x2b2bf890, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bf895, 77 },
    { 0, 0 },
    { 0xbbbbf897, 100 },
    { 0x70abf898, 91 },
    { 0x50bbf897, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435bf89e, 88 },
    { 0x670bf89f, 55 },
    { 0x815bf8a0, 53 },
    { 0, 0 },
    { 0x364bf8a2, 77 },
    { 0, 0 },
    { 0x5dbbf8a4, 100 },
    { 0x359bf8a4, 88 },
    { 0x3fdbf8a5, 68 },
    { 0xcd2bf8a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9bf8af, 55 },
    { 0x968bf8b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x776bf8b3, 55 },
    { 0, 0 },
    { 0x1a8bf8b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ebf8b8, 85 },
    { 0, 0 },
    { 0x997bf8ba, 76 },
    { 0x42dbf8ba, 76 },
    { 0x7a9bf8bc, 95 },
    { 0x490bf8bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17bf8c3, 90 },
    { 0x183bf8c4, 78 },
    { 0x2a5bf8c5, 52 },
    { 0x2dfbf8c5, 76 },
    { 0x3c9bf8c7, 97 },
    { 0x776bf8c8, 52 },
    { 0x992bf8c9, 93 },
    { 0xfbdbf8c8, 34 },
    { 0xbbfbf8cb, 46 },
    { 0, 0 },
    { 0xe2fbf8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5bf8d4, 78 },
    { 0xbcfbf8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18bf8d8, 55 },
    { 0xa3fbf8d9, 61 },
    { 0xa8cbf8da, 46 },
    { 0x5e5bf8db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbf8e0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ebf8e8, 90 },
    { 0, 0 },
    { 0xfd0bf8ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4bf8ef, 55 },
    { 0x99bbf8ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73abf8f3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8bf8f6, 87 },
    { 0x8c2bf8f7, 34 },
    { 0xd8ebf8f8, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ebf8fe, 60 },
    { 0, 0 },
    { 0x6d6bf900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bf904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37bf90a, 68 },
    { 0xf2dbf90b, 23 },
    { 0x401bf90b, 84 },
    { 0x40bbf90c, 90 },
    { 0x796bf90c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9bf911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748bf91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bf920, 99 },
    { 0x158bf921, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48bf92f, 33 },
    { 0x608bf92f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dbf934, 88 },
    { 0, 0 },
    { 0x4f4bf936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35bf939, 66 },
    { 0x8ebbf939, 81 },
    { 0x56bf93a, 55 },
    { 0, 0 },
    { 0x539bf93d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbf943, 98 },
    { 0x709bf944, 33 },
    { 0xb56bf944, 90 },
    { 0x44abf946, 77 },
    { 0x6c6bf946, 33 },
    { 0x22fbf945, 17 },
    { 0xe1bbf948, 23 },
    { 0x84dbf945, 100 },
    { 0xcc8bf94b, 55 },
    { 0x440bf94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fbf950, 95 },
    { 0x912bf950, 43 },
    { 0x8a3bf951, 33 },
    { 0xbe9bf953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dbf956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bf959, 90 },
    { 0, 0 },
    { 0x98bf95b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bf95f, 61 },
    { 0, 0 },
    { 0x8efbf961, 55 },
    { 0x210bf961, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193bf967, 34 },
    { 0, 0 },
    { 0xfffbf969, 61 },
    { 0x6acbf96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675bf970, 90 },
    { 0x8c2bf971, 23 },
    { 0, 0 },
    { 0xfefbf973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf978, 85 },
    { 0, 0 },
    { 0x730bf97a, 61 },
    { 0xe69bf97b, 67 },
    { 0, 0 },
    { 0xd0dbf97d, 90 },
    { 0, 0 },
    { 0xf45bf97f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44bf986, 34 },
    { 0, 0 },
    { 0x8c6bf988, 33 },
    { 0x826bf989, 55 },
    { 0xd6abf98a, 48 },
    { 0x569bf989, 90 },
    { 0x1efbf98c, 61 },
    { 0, 0 },
    { 0x31fbf98e, 61 },
    { 0x596bf98f, 90 },
    { 0xbc1bf98f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874bf995, 98 },
    { 0, 0 },
    { 0xe5dbf997, 23 },
    { 0x750bf997, 67 },
    { 0, 0 },
    { 0xb20bf99a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565bf99e, 95 },
    { 0x13fbf99e, 43 },
    { 0x26cbf9a0, 78 },
    { 0xf9bbf9a1, 99 },
    { 0, 0 },
    { 0x5abf9a3, 61 },
    { 0, 0 },
    { 0x43dbf9a5, 94 },
    { 0x84ebf9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1bf9a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf9ad, 38 },
    { 0x3debf9ad, 34 },
    { 0x6d1bf9ae, 95 },
    { 0xd9bbf9b0, 76 },
    { 0, 0 },
    { 0x2e4bf9b2, 76 },
    { 0x161bf9b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74bf9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3bf9bb, 48 },
    { 0xfdcbf9bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825bf9c2, 61 },
    { 0xe4fbf9c2, 66 },
    { 0xc2cbf9c4, 78 },
    { 0, 0 },
    { 0xc6bbf9c6, 23 },
    { 0, 0 },
    { 0x335bf9c8, 61 },
    { 0xa11bf9c9, 93 },
    { 0, 0 },
    { 0xd0dbf9cb, 90 },
    { 0, 0 },
    { 0x9b2bf9cd, 55 },
    { 0x161bf9ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1febf9d4, 95 },
    { 0xc37bf9d5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fbf9db, 34 },
    { 0x95bbf9db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766bf9e0, 33 },
    { 0, 0 },
    { 0x9bfbf9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abf9e8, 87 },
    { 0xcb9bf9e8, 34 },
    { 0xacfbf9ea, 61 },
    { 0, 0 },
    { 0xfe6bf9ec, 78 },
    { 0xf9fbf9ed, 90 },
    { 0xbd1bf9ee, 23 },
    { 0, 0 },
    { 0x2bbbf9f0, 90 },
    { 0xd91bf9f1, 23 },
    { 0x8e3bf9f2, 33 },
    { 0, 0 },
    { 0xff2bf9f4, 66 },
    { 0x181bf9f5, 33 },
    { 0xb5ebf9f5, 99 },
    { 0, 0 },
    { 0x169bf9f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8bf9ff, 77 },
    { 0x99fbfa00, 93 },
    { 0x7f8bfa01, 55 },
    { 0x701bfa02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfa0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804bfa11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abfa15, 76 },
    { 0, 0 },
    { 0xefdbfa17, 34 },
    { 0, 0 },
    { 0x529bfa19, 33 },
    { 0xf89bfa1a, 98 },
    { 0x3bfa1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08bfa1e, 66 },
    { 0x623bfa1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65abfa2c, 81 },
    { 0, 0 },
    { 0x20fbfa2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bfa33, 67 },
    { 0x758bfa33, 90 },
    { 0xb8cbfa35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dbfa3e, 33 },
    { 0x9dbfa3e, 33 },
    { 0x1c7bfa3f, 33 },
    { 0xf0fbfa3f, 98 },
    { 0xb2bfa42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13abfa45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6bfa4a, 100 },
    { 0xa2cbfa4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bfa4e, 76 },
    { 0, 0 },
    { 0x6a1bfa50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0bfa53, 90 },
    { 0x161bfa54, 95 },
    { 0x87fbfa55, 98 },
    { 0, 0 },
    { 0x89bfa57, 17 },
    { 0x523bfa57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3bfa5e, 61 },
    { 0xc53bfa5f, 88 },
    { 0, 0 },
    { 0xf02bfa61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbfa68, 93 },
    { 0x521bfa69, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772bfa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535bfa75, 33 },
    { 0x59bfa76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5bfa7b, 52 },
    { 0x395bfa7c, 90 },
    { 0, 0 },
    { 0x93ebfa7e, 52 },
    { 0xa77bfa7f, 95 },
    { 0xe76bfa80, 91 },
    { 0xd76bfa81, 76 },
    { 0, 0 },
    { 0x411bfa83, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bfa8c, 76 },
    { 0xcdbbfa8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940bfa94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbfa97, 34 },
    { 0x313bfa98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efbfa9b, 55 },
    { 0x82abfa9c, 87 },
    { 0xbabbfa9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3bfaa0, 95 },
    { 0x51cbfaa1, 34 },
    { 0x798bfaa2, 95 },
    { 0x720bfaa3, 23 },
    { 0x6c5bfaa4, 33 },
    { 0x2f6bfaa5, 33 },
    { 0, 0 },
    { 0x3a8bfaa7, 76 },
    { 0, 0 },
    { 0x890bfaa9, 69 },
    { 0x49dbfaaa, 76 },
    { 0xc18bfaab, 77 },
    { 0x92abfaac, 46 },
    { 0x989bfaaa, 93 },
    { 0, 0 },
    { 0xdbbbfaaf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dbfabb, 76 },
    { 0x471bfabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1bfabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2bfac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42abfac7, 55 },
    { 0x85cbfac7, 23 },
    { 0x55fbfac7, 98 },
    { 0xe5ebfaca, 34 },
    { 0xaddbfacb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bbfacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9bfad2, 53 },
    { 0, 0 },
    { 0xfa7bfad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175bfad7, 23 },
    { 0, 0 },
    { 0x8f2bfad9, 90 },
    { 0, 0 },
    { 0x50fbfadb, 91 },
    { 0xcc0bfadc, 33 },
    { 0xa93bfadc, 98 },
    { 0x297bfadc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccbfae6, 93 },
    { 0x16cbfae7, 43 },
    { 0xae2bfae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x224bfaeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609bfaef, 90 },
    { 0x440bfaf0, 34 },
    { 0x55cbfaf1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcbfaf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92bfaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4bfb01, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfbfb07, 61 },
    { 0x701bfb07, 98 },
    { 0xd2bbfb07, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbbfb0d, 95 },
    { 0x44fbfb0d, 61 },
    { 0x7e9bfb0f, 90 },
    { 0xee0bfb0d, 98 },
    { 0x61bfb11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2bfb17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97bfb1c, 23 },
    { 0, 0 },
    { 0x4b2bfb1e, 98 },
    { 0, 0 },
    { 0xe7cbfb20, 90 },
    { 0x6eebfb20, 90 },
    { 0x47bfb21, 55 },
    { 0xc62bfb20, 93 },
    { 0x2fdbfb24, 40 },
    { 0xcadbfb25, 95 },
    { 0xa2fbfb26, 52 },
    { 0x511bfb27, 95 },
    { 0x83cbfb28, 43 },
    { 0x9edbfb29, 33 },
    { 0x923bfb26, 34 },
    { 0xc09bfb2a, 61 },
    { 0xc7cbfb27, 98 },
    { 0x3acbfb26, 61 },
    { 0, 0 },
    { 0xd22bfb2f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbfb33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357bfb37, 49 },
    { 0, 0 },
    { 0x84fbfb39, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dbfb3e, 93 },
    { 0x8aabfb3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cbfb43, 33 },
    { 0x1e7bfb43, 61 },
    { 0x9e0bfb45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afbfb50, 90 },
    { 0xbaabfb51, 75 },
    { 0x777bfb52, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8bfb58, 61 },
    { 0, 0 },
    { 0xce6bfb5a, 90 },
    { 0x87cbfb5b, 78 },
    { 0, 0 },
    { 0x37bfb5d, 95 },
    { 0x213bfb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbfb64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42bfb67, 33 },
    { 0xedfbfb68, 23 },
    { 0xeaebfb68, 90 },
    { 0x6d1bfb68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868bfb6e, 46 },
    { 0xdc8bfb6f, 33 },
    { 0x1d7bfb70, 90 },
    { 0, 0 },
    { 0x53bbfb72, 46 },
    { 0xdb4bfb72, 74 },
    { 0x6a2bfb74, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3bfb7a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96bfb7f, 40 },
    { 0x1e0bfb80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69bfb87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bbfb8a, 76 },
    { 0x202bfb8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dbfb8e, 60 },
    { 0, 0 },
    { 0xd6bbfb90, 23 },
    { 0, 0 },
    { 0x39abfb92, 90 },
    { 0x2f5bfb92, 98 },
    { 0x199bfb94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392bfb9f, 98 },
    { 0xf9bbfb9f, 60 },
    { 0, 0 },
    { 0x1fbfba2, 78 },
    { 0xba3bfba3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948bfba7, 34 },
    { 0x393bfba7, 33 },
    { 0x7b5bfba9, 98 },
    { 0x34cbfbaa, 33 },
    { 0x667bfbaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4bfbae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bbfbb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36bfbb6, 95 },
    { 0, 0 },
    { 0x936bfbb8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cbfbbf, 90 },
    { 0, 0 },
    { 0x6b9bfbc1, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18bfbc4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9bfbcd, 98 },
    { 0, 0 },
    { 0x155bfbcf, 43 },
    { 0, 0 },
    { 0xe05bfbd1, 46 },
    { 0x8b8bfbd2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1bfbdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bfbe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbfbf0, 98 },
    { 0x91ebfbf1, 23 },
    { 0x34bbfbf2, 61 },
    { 0xacabfbf3, 87 },
    { 0xf5dbfbf4, 90 },
    { 0xbf6bfbf5, 23 },
    { 0x269bfbf3, 95 },
    { 0xd34bfbf2, 96 },
    { 0xf72bfbf8, 98 },
    { 0x895bfbf8, 76 },
    { 0, 0 },
    { 0xcfabfbfb, 33 },
    { 0, 0 },
    { 0xa4bfbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56bfc00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cbfc04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5febfc0a, 95 },
    { 0x366bfc0b, 23 },
    { 0xd59bfc0b, 98 },
    { 0xa33bfc0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdbfc10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42bfc14, 55 },
    { 0xa22bfc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544bfc1f, 34 },
    { 0x126bfc1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6bfc24, 61 },
    { 0x165bfc25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104bfc2b, 98 },
    { 0xbc2bfc2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x490bfc2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cbfc32, 93 },
    { 0x489bfc32, 65 },
    { 0xceebfc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bfc37, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x726bfc3a, 38 },
    { 0x92cbfc3a, 95 },
    { 0x977bfc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddbfc3f, 90 },
    { 0x351bfc40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5bfc43, 90 },
    { 0x8ddbfc44, 90 },
    { 0x9bcbfc45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797bfc4a, 90 },
    { 0xf55bfc4a, 100 },
    { 0x662bfc4b, 93 },
    { 0, 0 },
    { 0x310bfc4e, 92 },
    { 0x6e5bfc4f, 87 },
    { 0x3d2bfc50, 36 },
    { 0x9ecbfc51, 81 },
    { 0xf5fbfc51, 34 },
    { 0xac7bfc53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67bfc56, 75 },
    { 0x7a1bfc57, 90 },
    { 0x339bfc58, 66 },
    { 0x643bfc59, 98 },
    { 0x6ddbfc5a, 34 },
    { 0x382bfc5a, 76 },
    { 0xfedbfc5a, 23 },
    { 0x7c4bfc59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874bfc62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x153bfc65, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bfc70, 61 },
    { 0xcabbfc71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24bfc76, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfbfc7f, 23 },
    { 0xa38bfc80, 90 },
    { 0xe55bfc81, 61 },
    { 0xe95bfc82, 76 },
    { 0, 0 },
    { 0x3f3bfc84, 76 },
    { 0x743bfc84, 95 },
    { 0xd48bfc86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bfc8c, 87 },
    { 0, 0 },
    { 0xeb4bfc8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597bfc94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730bfc99, 43 },
    { 0x75bbfc9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48bfc9e, 34 },
    { 0, 0 },
    { 0x210bfca0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bfca4, 51 },
    { 0x7c1bfca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88bfcad, 23 },
    { 0, 0 },
    { 0x127bfcaf, 95 },
    { 0xe38bfcaf, 34 },
    { 0x21abfcb1, 33 },
    { 0xe2abfcb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09bfcb8, 100 },
    { 0x441bfcb9, 90 },
    { 0x522bfcba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bfcbf, 92 },
    { 0xed1bfcbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9bfcc7, 98 },
    { 0, 0 },
    { 0xc0cbfcc9, 98 },
    { 0, 0 },
    { 0x776bfccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bfcce, 46 },
    { 0x1c9bfccf, 96 },
    { 0xbf6bfccf, 76 },
    { 0x721bfcd1, 12 },
    { 0x387bfccf, 90 },
    { 0x517bfcd3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbbfcd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbbfcdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdbfce3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178bfce8, 94 },
    { 0xad1bfce9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2bfcec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740bfcf6, 52 },
    { 0xae7bfcf7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01bfcfe, 90 },
    { 0, 0 },
    { 0x150bfd00, 23 },
    { 0x826bfd01, 90 },
    { 0, 0 },
    { 0x163bfd03, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77bfd08, 55 },
    { 0x610bfd09, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99bfd15, 43 },
    { 0, 0 },
    { 0x8cbbfd17, 84 },
    { 0xf78bfd18, 85 },
    { 0x97fbfd19, 87 },
    { 0x3bfbfd18, 61 },
    { 0x572bfd1b, 81 },
    { 0xf4cbfd1a, 17 },
    { 0x9b9bfd1d, 19 },
    { 0, 0 },
    { 0x231bfd1f, 99 },
    { 0x1b9bfd1f, 75 },
    { 0x20abfd20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7febfd25, 61 },
    { 0x878bfd26, 96 },
    { 0, 0 },
    { 0x1d3bfd28, 84 },
    { 0xdbbbfd28, 90 },
    { 0x5aebfd29, 64 },
    { 0xe19bfd2b, 33 },
    { 0xd54bfd2b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52bfd2f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9bfd32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5bfd35, 90 },
    { 0x3fbbfd36, 55 },
    { 0x5cbfd36, 61 },
    { 0xa1fbfd38, 67 },
    { 0x908bfd37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dbfd3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0bfd45, 91 },
    { 0x19bfd46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddbfd49, 76 },
    { 0x966bfd4a, 61 },
    { 0, 0 },
    { 0x330bfd4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0bfd51, 98 },
    { 0x7f8bfd51, 33 },
    { 0xa6dbfd53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dbfd57, 98 },
    { 0, 0 },
    { 0x7e6bfd59, 36 },
    { 0x5f5bfd59, 55 },
    { 0, 0 },
    { 0xd09bfd5c, 78 },
    { 0x40abfd5c, 55 },
    { 0xa54bfd5c, 90 },
    { 0x23bfd5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cbfd64, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fbfd67, 23 },
    { 0, 0 },
    { 0xf3abfd69, 61 },
    { 0x86fbfd6a, 76 },
    { 0x3e0bfd6b, 98 },
    { 0, 0 },
    { 0x298bfd6d, 43 },
    { 0, 0 },
    { 0xc45bfd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4bfd76, 33 },
    { 0xc9bfd76, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6bfd81, 23 },
    { 0x96cbfd82, 52 },
    { 0xbb6bfd83, 61 },
    { 0xf32bfd83, 35 },
    { 0, 0 },
    { 0x3f4bfd86, 98 },
    { 0x42bbfd87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aabfd8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdbfd8f, 85 },
    { 0x2f2bfd90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebfd94, 33 },
    { 0x47abfd95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3bfd99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x813bfd9c, 93 },
    { 0x815bfd9c, 66 },
    { 0xfcbbfd9e, 90 },
    { 0xfe6bfd9f, 61 },
    { 0x456bfda0, 98 },
    { 0xc90bfda1, 95 },
    { 0x410bfda2, 87 },
    { 0x9c9bfd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ebfda8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735bfdac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77bfdb4, 43 },
    { 0, 0 },
    { 0x911bfdb6, 23 },
    { 0x7b1bfdb7, 87 },
    { 0xabbfdb7, 33 },
    { 0xdebfdb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbfdc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766bfdc7, 75 },
    { 0xcaabfdc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9bfdcd, 90 },
    { 0xe67bfdcd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabfdd1, 90 },
    { 0xd66bfdd2, 90 },
    { 0x518bfdd3, 93 },
    { 0xcf8bfdd1, 96 },
    { 0xc3cbfdd5, 43 },
    { 0xd02bfdd5, 43 },
    { 0x12bfdd7, 100 },
    { 0x60fbfdd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bbfddb, 84 },
    { 0x417bfddc, 87 },
    { 0xa73bfddc, 90 },
    { 0xffcbfdde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbfde2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bfde7, 76 },
    { 0, 0 },
    { 0xa62bfde9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbabfdf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44abfdf4, 78 },
    { 0xadfbfdf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbfdf8, 61 },
    { 0x616bfdf8, 43 },
    { 0x831bfdfa, 61 },
    { 0x6b3bfdfb, 90 },
    { 0xe72bfdfc, 90 },
    { 0x566bfdf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80bfe01, 90 },
    { 0x9d9bfe02, 76 },
    { 0x94bbfe02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbfe07, 46 },
    { 0, 0 },
    { 0x949bfe09, 56 },
    { 0, 0 },
    { 0x7febfe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7babfe0e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bfe12, 61 },
    { 0xc60bfe12, 55 },
    { 0xb32bfe13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95bfe1a, 95 },
    { 0, 0 },
    { 0x979bfe1c, 67 },
    { 0x70fbfe1d, 61 },
    { 0x87fbfe1e, 23 },
    { 0x380bfe1e, 65 },
    { 0xfdcbfe1f, 91 },
    { 0, 0 },
    { 0xda4bfe22, 81 },
    { 0x1ccbfe22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69bfe27, 90 },
    { 0x506bfe27, 95 },
    { 0xe12bfe28, 56 },
    { 0xfb7bfe2a, 55 },
    { 0, 0 },
    { 0x65bbfe2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbfe30, 76 },
    { 0xf5ebfe30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917bfe36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcbfe3a, 61 },
    { 0, 0 },
    { 0xb8abfe3c, 61 },
    { 0, 0 },
    { 0x84cbfe3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9bfe45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x808bfe48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26bfe4c, 43 },
    { 0, 0 },
    { 0x78dbfe4e, 33 },
    { 0, 0 },
    { 0xa20bfe50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47bfe59, 64 },
    { 0xc3fbfe5a, 34 },
    { 0xbd8bfe59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834bfe5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41bfe65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbfe6b, 90 },
    { 0, 0 },
    { 0x208bfe6d, 34 },
    { 0x5ecbfe6e, 33 },
    { 0x1f1bfe6f, 33 },
    { 0x17dbfe6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cbfe73, 23 },
    { 0x80fbfe74, 46 },
    { 0xb66bfe75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7bfe78, 99 },
    { 0xe81bfe79, 33 },
    { 0x2e4bfe7a, 90 },
    { 0xdfcbfe79, 85 },
    { 0x41dbfe7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0bfe7f, 99 },
    { 0, 0 },
    { 0x1ccbfe81, 98 },
    { 0, 0 },
    { 0x338bfe83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcbfe86, 95 },
    { 0xfa2bfe86, 33 },
    { 0x309bfe88, 93 },
    { 0, 0 },
    { 0xf1dbfe8a, 90 },
    { 0xbe7bfe8b, 67 },
    { 0x249bfe8b, 75 },
    { 0x309bfe8c, 98 },
    { 0xad7bfe8d, 55 },
    { 0, 0 },
    { 0xb0ebfe90, 61 },
    { 0x5a5bfe91, 90 },
    { 0x4d3bfe92, 33 },
    { 0xa4dbfe93, 33 },
    { 0x703bfe94, 100 },
    { 0x772bfe93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dbfe98, 90 },
    { 0x9b1bfe99, 46 },
    { 0x699bfe99, 98 },
    { 0x92bbfe9b, 23 },
    { 0xdfdbfe9b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbfea1, 61 },
    { 0, 0 },
    { 0xf1cbfea3, 33 },
    { 0xeb9bfea4, 98 },
    { 0x8a1bfea5, 96 },
    { 0, 0 },
    { 0xf81bfea7, 88 },
    { 0, 0 },
    { 0x601bfea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66bfead, 61 },
    { 0x8e9bfeae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64abfeb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4bfeb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dbfebd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796bfec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348bfecb, 34 },
    { 0xb24bfecc, 33 },
    { 0xcdfbfecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428bfed2, 93 },
    { 0xaa1bfed3, 33 },
    { 0xa49bfed2, 90 },
    { 0x642bfed5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8bfedc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242bfee0, 70 },
    { 0, 0 },
    { 0x6aabfee2, 84 },
    { 0, 0 },
    { 0x29abfee4, 55 },
    { 0x602bfee4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6bfee8, 95 },
    { 0, 0 },
    { 0xda9bfeea, 95 },
    { 0xf0bfeeb, 33 },
    { 0x25ebfeeb, 90 },
    { 0xb00bfeed, 66 },
    { 0xd94bfeed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85bfef6, 66 },
    { 0x82cbfef7, 43 },
    { 0, 0 },
    { 0x63ebfef9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26bfefd, 87 },
    { 0, 0 },
    { 0x592bfeff, 80 },
    { 0xc8abfeff, 95 },
    { 0x835bff01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x717bff0a, 39 },
    { 0xe19bff0a, 23 },
    { 0x558bff0b, 77 },
    { 0x86bff0d, 61 },
    { 0x38dbff0a, 23 },
    { 0x525bff0f, 61 },
    { 0xceabff0b, 98 },
    { 0xe74bff11, 17 },
    { 0x922bff12, 33 },
    { 0, 0 },
    { 0xa8cbff14, 34 },
    { 0x4a2bff14, 33 },
    { 0x476bff16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dbff1a, 23 },
    { 0x6e3bff1b, 90 },
    { 0x942bff1c, 76 },
    { 0xd26bff1d, 78 },
    { 0xdf2bff1d, 34 },
    { 0x60dbff1f, 24 },
    { 0xea5bff20, 95 },
    { 0x234bff1b, 33 },
    { 0xde8bff22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacbff2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbff30, 23 },
    { 0x642bff31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffbff36, 33 },
    { 0x841bff36, 61 },
    { 0xe41bff38, 61 },
    { 0xa2dbff39, 76 },
    { 0x3c4bff38, 61 },
    { 0xe20bff39, 68 },
    { 0x337bff39, 33 },
    { 0xefbbff3d, 67 },
    { 0xb47bff3d, 95 },
    { 0x214bff3b, 33 },
    { 0, 0 },
    { 0x5f0bff41, 26 },
    { 0xa72bff42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbbff49, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9bff50, 61 },
    { 0x66bff51, 98 },
    { 0, 0 },
    { 0x985bff53, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabff56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cbff5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ebff5d, 90 },
    { 0x8d5bff5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1bff62, 90 },
    { 0x64abff63, 61 },
    { 0xe59bff64, 90 },
    { 0xab9bff62, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x522bff68, 98 },
    { 0xc8ebff69, 23 },
    { 0xa30bff68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bbff72, 98 },
    { 0, 0 },
    { 0xdb8bff74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bff77, 61 },
    { 0xccfbff78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebbff7c, 23 },
    { 0xbb1bff7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4bff81, 91 },
    { 0, 0 },
    { 0x99bbff83, 34 },
    { 0x7b2bff83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93bff88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbff8c, 95 },
    { 0x874bff8d, 17 },
    { 0xd64bff8e, 55 },
    { 0x9e6bff8f, 90 },
    { 0, 0 },
    { 0x3f5bff91, 34 },
    { 0, 0 },
    { 0x959bff93, 90 },
    { 0x407bff94, 78 },
    { 0, 0 },
    { 0x425bff96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cbff99, 23 },
    { 0xb98bff9a, 86 },
    { 0x3ecbff9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6debffa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138bffaa, 90 },
    { 0xc3abffaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6bffb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1bffbc, 76 },
    { 0, 0 },
    { 0x135bffbe, 88 },
    { 0x74abffbf, 61 },
    { 0x289bffc0, 98 },
    { 0xfffbffbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ebffc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabffc7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebffd4, 87 },
    { 0x39bffd4, 33 },
    { 0xdfebffd4, 61 },
    { 0x5a2bffd7, 90 },
    { 0x419bffd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4bffdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fbffe2, 76 },
    { 0x4f3bffe2, 90 },
    { 0xaa2bffe2, 76 },
    { 0, 0 },
    { 0x67bbffe6, 83 },
    { 0, 0 },
    { 0xa5abffe8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7bfff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bbfff8, 34 },
    { 0xdb8bfff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabfffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc0000, 66 },
    { 0x1b5c0000, 84 },
    { 0xc80c0002, 95 },
    { 0x637c0001, 95 },
    { 0x242c0002, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5c0008, 93 },
    { 0xcd6c0009, 33 },
    { 0, 0 },
    { 0x138c000b, 95 },
    { 0, 0 },
    { 0x6ccc000d, 33 },
    { 0x4f7c000e, 76 },
    { 0xd18c000f, 98 },
    { 0x24ec000e, 77 },
    { 0x614c0011, 23 },
    { 0x160c0011, 99 },
    { 0x220c000e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x361c0016, 96 },
    { 0xcf5c0017, 43 },
    { 0xbc5c0018, 33 },
    { 0xc69c0017, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496c0021, 90 },
    { 0x657c0022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c0026, 100 },
    { 0x753c0026, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c002d, 90 },
    { 0x687c002d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc0033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dac0038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc0049, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac004e, 61 },
    { 0, 0 },
    { 0xf6bc0050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c0062, 23 },
    { 0xa9ec0062, 23 },
    { 0, 0 },
    { 0xb27c0065, 46 },
    { 0xa4fc0066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c0069, 61 },
    { 0, 0 },
    { 0x4a8c006b, 34 },
    { 0xec9c006b, 78 },
    { 0xecbc006d, 46 },
    { 0xd83c006e, 95 },
    { 0xd92c006f, 34 },
    { 0x943c006e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c0073, 43 },
    { 0, 0 },
    { 0xceac0075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c007c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c008c, 90 },
    { 0x1abc008c, 66 },
    { 0, 0 },
    { 0x7ac008f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c0093, 34 },
    { 0x7ddc0093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc009c, 90 },
    { 0, 0 },
    { 0x700c009e, 52 },
    { 0xac3c009f, 34 },
    { 0, 0 },
    { 0x27cc00a1, 69 },
    { 0x88cc00a2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503c00a6, 96 },
    { 0, 0 },
    { 0x5bcc00a8, 100 },
    { 0xc5bc00a8, 47 },
    { 0x398c00a9, 23 },
    { 0x3d4c00a9, 33 },
    { 0x30dc00ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cc00b0, 61 },
    { 0xffcc00b1, 95 },
    { 0x93cc00b1, 90 },
    { 0x48cc00b3, 46 },
    { 0x6d2c00b0, 90 },
    { 0xe57c00b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876c00be, 95 },
    { 0xc03c00be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c00c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6c00ca, 95 },
    { 0x2e8c00cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06c00cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c00d3, 98 },
    { 0, 0 },
    { 0x1ac00d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0c00dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c00e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6c00e5, 61 },
    { 0x3c5c00e6, 52 },
    { 0x198c00e6, 61 },
    { 0x53ec00e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63c00ef, 95 },
    { 0x92dc00f0, 76 },
    { 0x1bac00f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03c00f9, 30 },
    { 0, 0 },
    { 0x84ac00fb, 91 },
    { 0xc25c00fc, 90 },
    { 0x25ac00fd, 23 },
    { 0xc9dc00fc, 100 },
    { 0, 0 },
    { 0xd3bc0100, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eec0106, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c0111, 90 },
    { 0, 0 },
    { 0x1c6c0113, 61 },
    { 0x362c0114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x646c0117, 55 },
    { 0xb47c0117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec011c, 78 },
    { 0, 0 },
    { 0xf27c011e, 46 },
    { 0, 0 },
    { 0x8c0120, 93 },
    { 0, 0 },
    { 0x192c0122, 33 },
    { 0xd62c0123, 23 },
    { 0x362c0123, 11 },
    { 0x6e7c0124, 95 },
    { 0x409c0126, 78 },
    { 0x295c0127, 95 },
    { 0xa1dc0128, 33 },
    { 0x954c0129, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dc012d, 100 },
    { 0x4dcc012e, 90 },
    { 0x199c012e, 55 },
    { 0x791c0130, 76 },
    { 0x52dc012e, 100 },
    { 0xf81c0132, 90 },
    { 0, 0 },
    { 0x72bc0134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cc013a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c0140, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x492c0143, 34 },
    { 0, 0 },
    { 0x4eac0145, 90 },
    { 0xc42c0146, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dac014d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c0150, 87 },
    { 0x3b9c0150, 55 },
    { 0xa3c0152, 33 },
    { 0, 0 },
    { 0x412c0154, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ec015a, 98 },
    { 0xc71c015a, 33 },
    { 0x839c015b, 95 },
    { 0x1b2c015d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc0162, 90 },
    { 0xb27c0162, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc0166, 76 },
    { 0xb50c0167, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bc016b, 98 },
    { 0, 0 },
    { 0xb4fc016d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25c0172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc017c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c0185, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x371c0188, 76 },
    { 0x6e9c0189, 67 },
    { 0xd7dc018a, 43 },
    { 0, 0 },
    { 0x560c018c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c0190, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0193, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516c0197, 90 },
    { 0x28bc0198, 95 },
    { 0xa97c0197, 76 },
    { 0x751c0198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723c01a1, 93 },
    { 0x53c01a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c01a6, 23 },
    { 0, 0 },
    { 0xd8c01a8, 48 },
    { 0, 0 },
    { 0x220c01aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x335c01ad, 98 },
    { 0x28fc01ae, 34 },
    { 0x621c01af, 76 },
    { 0x994c01ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dc01b7, 95 },
    { 0, 0 },
    { 0x510c01b9, 90 },
    { 0, 0 },
    { 0xfd1c01bb, 95 },
    { 0x978c01bb, 67 },
    { 0x9dbc01bd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1c01c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375c01c4, 78 },
    { 0x3f6c01c4, 67 },
    { 0x68cc01c6, 33 },
    { 0xd6ec01c7, 78 },
    { 0x7bc01c4, 69 },
    { 0xea6c01c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfc01d7, 33 },
    { 0x6ec01d8, 95 },
    { 0x31dc01d9, 65 },
    { 0x59bc01da, 33 },
    { 0, 0 },
    { 0x16dc01dc, 43 },
    { 0x459c01dc, 90 },
    { 0, 0 },
    { 0x8bc01df, 95 },
    { 0x11cc01e0, 66 },
    { 0x630c01e1, 93 },
    { 0xc63c01e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c01ff, 75 },
    { 0xae0c0200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbc0203, 98 },
    { 0xb4cc0204, 33 },
    { 0x7c0c0203, 98 },
    { 0x555c0206, 100 },
    { 0xaaac0207, 90 },
    { 0, 0 },
    { 0x733c0209, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fc0210, 98 },
    { 0x15dc0211, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129c0216, 95 },
    { 0x4fec0217, 90 },
    { 0x7b2c0218, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c021f, 90 },
    { 0xd2fc021f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c0223, 90 },
    { 0x5e5c0224, 46 },
    { 0, 0 },
    { 0x208c0226, 46 },
    { 0x995c0227, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc022e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x704c0231, 52 },
    { 0xf4c0232, 36 },
    { 0x468c0233, 76 },
    { 0xd6ec0234, 34 },
    { 0, 0 },
    { 0x325c0236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x932c0239, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c0241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc0247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec024d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfc0251, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeec0255, 61 },
    { 0xb00c0256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c0259, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c0262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39c0268, 90 },
    { 0xaf5c0269, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66c026c, 95 },
    { 0xd9ec026d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc0272, 67 },
    { 0x664c0273, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c0283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544c0288, 98 },
    { 0x8a4c0289, 33 },
    { 0x453c0289, 23 },
    { 0xbe4c028b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c028e, 55 },
    { 0x16ac028f, 90 },
    { 0x43c028e, 95 },
    { 0xc8ec028e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cac0294, 61 },
    { 0x1cbc0294, 93 },
    { 0x897c0295, 90 },
    { 0xc66c0297, 100 },
    { 0xfcc0297, 43 },
    { 0, 0 },
    { 0xf61c029a, 23 },
    { 0xb82c029a, 23 },
    { 0, 0 },
    { 0x547c029d, 46 },
    { 0x3ebc029d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c02a1, 95 },
    { 0, 0 },
    { 0x93dc02a3, 76 },
    { 0x321c02a3, 46 },
    { 0, 0 },
    { 0xd49c02a6, 56 },
    { 0xdc8c02a6, 61 },
    { 0, 0 },
    { 0xd4cc02a9, 34 },
    { 0, 0 },
    { 0xb7bc02ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38c02ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc02b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dc02b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9c02ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c02bd, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9c02c0, 61 },
    { 0x2e4c02c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10c02c4, 76 },
    { 0x3f7c02c5, 95 },
    { 0xb1ec02c5, 90 },
    { 0x5bac02c7, 100 },
    { 0x8b1c02c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c02cb, 61 },
    { 0xb57c02cc, 87 },
    { 0, 0 },
    { 0x992c02ce, 90 },
    { 0, 0 },
    { 0xc3bc02d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c02d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c02d6, 90 },
    { 0x1d6c02d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c02dc, 65 },
    { 0x967c02dd, 61 },
    { 0xd2ac02de, 93 },
    { 0xde8c02df, 61 },
    { 0xebec02df, 66 },
    { 0x19ec02de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62c02e4, 78 },
    { 0, 0 },
    { 0xe80c02e6, 33 },
    { 0x6dac02e7, 90 },
    { 0x491c02e8, 93 },
    { 0xd66c02e9, 55 },
    { 0x5bfc02ea, 59 },
    { 0x322c02e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x396c02ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c02f2, 90 },
    { 0, 0 },
    { 0x2fc02f4, 98 },
    { 0x559c02f4, 67 },
    { 0xf8cc02f6, 33 },
    { 0x26c02f4, 61 },
    { 0x5e9c02f8, 66 },
    { 0x960c02f9, 33 },
    { 0xcacc02f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c02fd, 56 },
    { 0, 0 },
    { 0x5d5c02ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcc0304, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ac030b, 33 },
    { 0, 0 },
    { 0x551c030d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c0313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc0317, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c031a, 34 },
    { 0x59ec031a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6c031e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43c0322, 95 },
    { 0x68ac0322, 61 },
    { 0x687c0324, 76 },
    { 0x3fac0324, 69 },
    { 0x8e8c0326, 23 },
    { 0xba9c0324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfc032b, 40 },
    { 0x737c032c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6c032f, 34 },
    { 0, 0 },
    { 0xb8c0331, 52 },
    { 0x171c0331, 33 },
    { 0xff9c0333, 100 },
    { 0x6d7c0332, 83 },
    { 0, 0 },
    { 0xe0cc0336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c033e, 76 },
    { 0, 0 },
    { 0xccec0340, 76 },
    { 0x4d7c0340, 52 },
    { 0, 0 },
    { 0xe96c0343, 94 },
    { 0x454c0343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c034a, 76 },
    { 0x1b6c034b, 90 },
    { 0x5cec034c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bc0352, 33 },
    { 0x618c0353, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c035d, 34 },
    { 0, 0 },
    { 0x933c035f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c0367, 84 },
    { 0x326c0368, 76 },
    { 0xa4bc0369, 95 },
    { 0x538c0368, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1c036d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842c0371, 52 },
    { 0, 0 },
    { 0x423c0373, 78 },
    { 0, 0 },
    { 0xe00c0375, 90 },
    { 0xa73c0376, 38 },
    { 0x726c0376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac037b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c037e, 95 },
    { 0x456c037f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdc0386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c0389, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44c0392, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fc0399, 23 },
    { 0xe36c0399, 93 },
    { 0xddec039b, 93 },
    { 0x60bc039c, 92 },
    { 0x5f7c039b, 33 },
    { 0x593c039e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c03a6, 23 },
    { 0, 0 },
    { 0x919c03a8, 23 },
    { 0, 0 },
    { 0xaeac03aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3c03ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9c03b1, 33 },
    { 0xaedc03b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6c03b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c03bf, 90 },
    { 0, 0 },
    { 0xa35c03c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c03c4, 43 },
    { 0x1b4c03c5, 67 },
    { 0x51fc03c6, 46 },
    { 0, 0 },
    { 0x54cc03c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825c03cf, 23 },
    { 0, 0 },
    { 0xbf7c03d1, 90 },
    { 0x98ec03d2, 51 },
    { 0xa86c03d2, 33 },
    { 0x518c03d4, 61 },
    { 0, 0 },
    { 0x80ac03d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bc03d9, 23 },
    { 0x3efc03da, 33 },
    { 0xc13c03db, 95 },
    { 0x45ec03dc, 34 },
    { 0x76c03dd, 90 },
    { 0x833c03de, 69 },
    { 0, 0 },
    { 0xe9dc03e0, 23 },
    { 0xf60c03e1, 77 },
    { 0xda6c03e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bc03ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69c03f1, 64 },
    { 0x6a8c03f1, 76 },
    { 0x9f1c03f2, 23 },
    { 0x9bdc03f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac03fa, 84 },
    { 0xa10c03fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c0400, 33 },
    { 0, 0 },
    { 0x316c0402, 90 },
    { 0, 0 },
    { 0xb9ac0404, 34 },
    { 0, 0 },
    { 0x1cec0406, 33 },
    { 0, 0 },
    { 0xe92c0408, 92 },
    { 0xdd1c0409, 34 },
    { 0x26fc040a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac040f, 90 },
    { 0x16ec0410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fc0415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fc0418, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac041b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84c041f, 81 },
    { 0, 0 },
    { 0xa69c0421, 43 },
    { 0, 0 },
    { 0xa2c0423, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c0428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c042b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ac0431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec0435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c0439, 38 },
    { 0x256c043a, 33 },
    { 0, 0 },
    { 0x1dcc043c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c0443, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c0447, 61 },
    { 0, 0 },
    { 0x318c0449, 52 },
    { 0x998c0449, 17 },
    { 0x3eec044b, 66 },
    { 0xc71c044b, 95 },
    { 0xfefc044b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27c0450, 61 },
    { 0x5d4c0451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47c0455, 53 },
    { 0x852c0456, 43 },
    { 0x694c0457, 61 },
    { 0x1acc0457, 23 },
    { 0x4eac0459, 43 },
    { 0x464c0457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9c045e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5c0463, 69 },
    { 0, 0 },
    { 0x52dc0465, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dc0469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4c0470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c0473, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bec047c, 91 },
    { 0xa6fc047d, 100 },
    { 0x189c047e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacac0484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9c0487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477c0491, 98 },
    { 0, 0 },
    { 0xbcbc0493, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c0496, 66 },
    { 0xa3bc0497, 61 },
    { 0xa5ec0497, 91 },
    { 0xa9c0497, 23 },
    { 0xaabc0497, 52 },
    { 0xe66c049a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c049e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c04a3, 55 },
    { 0x329c04a4, 61 },
    { 0xf0ac04a4, 95 },
    { 0, 0 },
    { 0x769c04a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6c04ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ec04ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17c04b2, 33 },
    { 0, 0 },
    { 0xc5bc04b4, 99 },
    { 0x7e4c04b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c04ba, 90 },
    { 0, 0 },
    { 0xb85c04bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c04bf, 76 },
    { 0x456c04bf, 95 },
    { 0xfdbc04c0, 61 },
    { 0x2d2c04c2, 90 },
    { 0x619c04c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfc04c6, 33 },
    { 0, 0 },
    { 0xf1bc04c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ac04cb, 55 },
    { 0x1b5c04cc, 98 },
    { 0xde3c04cd, 96 },
    { 0, 0 },
    { 0x107c04cf, 40 },
    { 0x738c04d0, 90 },
    { 0xe1c04cf, 76 },
    { 0, 0 },
    { 0x33c04d3, 23 },
    { 0xd03c04d3, 23 },
    { 0, 0 },
    { 0x520c04d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8c04de, 43 },
    { 0xf9bc04de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc04e4, 90 },
    { 0x644c04e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c04ea, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dc04ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ec04f9, 90 },
    { 0x9ccc04fa, 76 },
    { 0x30bc04fb, 36 },
    { 0xa89c04fb, 98 },
    { 0xcc6c04fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0c0502, 23 },
    { 0x849c0503, 95 },
    { 0x31ac0503, 34 },
    { 0xd77c0503, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4c0509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5c050d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc0511, 34 },
    { 0, 0 },
    { 0xee0c0513, 64 },
    { 0x4c3c0514, 90 },
    { 0x3d9c0514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c051b, 90 },
    { 0x461c051c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951c0522, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fc0526, 98 },
    { 0, 0 },
    { 0x810c0528, 95 },
    { 0xa1fc0528, 98 },
    { 0xd1ac0528, 61 },
    { 0x936c052b, 61 },
    { 0x21cc052c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac0532, 90 },
    { 0x9e9c0533, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fc0537, 46 },
    { 0, 0 },
    { 0xda2c0539, 90 },
    { 0xd75c0539, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c053d, 33 },
    { 0xff2c053d, 98 },
    { 0x395c053f, 43 },
    { 0, 0 },
    { 0x6e9c0541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2c0545, 55 },
    { 0x1f9c0545, 61 },
    { 0x6ecc0547, 33 },
    { 0xfd6c0548, 61 },
    { 0xdfbc0548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cac054c, 95 },
    { 0xee4c054d, 33 },
    { 0xac2c054e, 76 },
    { 0xc03c054c, 23 },
    { 0x539c0550, 61 },
    { 0x771c0550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c0556, 85 },
    { 0, 0 },
    { 0xbcfc0558, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc055b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c0560, 90 },
    { 0x6f2c0561, 23 },
    { 0x7c9c0561, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c0565, 33 },
    { 0x587c0566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c056b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90c0570, 90 },
    { 0x6b1c0571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dc0577, 90 },
    { 0x56ac0578, 95 },
    { 0x17bc0578, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedc057d, 23 },
    { 0x66bc057e, 33 },
    { 0x370c057f, 61 },
    { 0xec8c0580, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85c0584, 33 },
    { 0, 0 },
    { 0xd32c0586, 33 },
    { 0x6a7c0587, 61 },
    { 0, 0 },
    { 0x5c6c0589, 66 },
    { 0xf85c058a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6c0591, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fc0594, 84 },
    { 0xeabc0595, 66 },
    { 0xd36c0596, 100 },
    { 0x64ac0597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ac059a, 61 },
    { 0xcbac059b, 90 },
    { 0, 0 },
    { 0x74bc059d, 93 },
    { 0x3ebc059e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77c05a9, 33 },
    { 0, 0 },
    { 0x8cc05ab, 95 },
    { 0, 0 },
    { 0xd3ec05ad, 95 },
    { 0x506c05ae, 76 },
    { 0x933c05ad, 23 },
    { 0xbd6c05b0, 77 },
    { 0x45bc05b1, 55 },
    { 0xed2c05b1, 76 },
    { 0x4b3c05ad, 34 },
    { 0xdb6c05b4, 55 },
    { 0x1adc05ad, 77 },
    { 0x3a0c05b5, 99 },
    { 0xc6ac05b7, 85 },
    { 0x49dc05b7, 95 },
    { 0x618c05b9, 90 },
    { 0x8b8c05ba, 61 },
    { 0x966c05b7, 23 },
    { 0x2a0c05bc, 82 },
    { 0x7edc05bc, 76 },
    { 0x6b9c05be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac05c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1c05cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50c05cf, 78 },
    { 0, 0 },
    { 0xf02c05d1, 33 },
    { 0x66dc05d2, 90 },
    { 0xfa0c05d2, 97 },
    { 0, 0 },
    { 0x98ec05d5, 68 },
    { 0xe09c05d5, 95 },
    { 0x2d2c05d7, 34 },
    { 0x8bec05d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c05db, 61 },
    { 0x3a6c05dc, 93 },
    { 0, 0 },
    { 0x285c05de, 61 },
    { 0x73cc05df, 33 },
    { 0xaabc05e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c05e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bc05e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcc05eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afc05f1, 90 },
    { 0x100c05f2, 91 },
    { 0x313c05f1, 61 },
    { 0x26c05f2, 75 },
    { 0x639c05f3, 55 },
    { 0, 0 },
    { 0x83ec05f7, 46 },
    { 0x20ec05f8, 100 },
    { 0x90cc05f9, 66 },
    { 0xbb8c05f9, 33 },
    { 0x616c05fb, 87 },
    { 0, 0 },
    { 0xf8dc05fd, 98 },
    { 0x4a0c05fe, 23 },
    { 0x22fc05ff, 33 },
    { 0x84cc0600, 78 },
    { 0xce8c05fe, 65 },
    { 0x9d4c0602, 23 },
    { 0x9fac0603, 23 },
    { 0x5a2c0604, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c0609, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544c0610, 77 },
    { 0xdaac0610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc0616, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ac061b, 98 },
    { 0xd95c061c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac0620, 61 },
    { 0, 0 },
    { 0xe13c0622, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bc0625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc062a, 23 },
    { 0, 0 },
    { 0x44ec062c, 66 },
    { 0x30bc062c, 98 },
    { 0x33dc062e, 77 },
    { 0xd27c062f, 68 },
    { 0x991c062e, 55 },
    { 0xee2c062d, 69 },
    { 0x1bfc0632, 62 },
    { 0xbccc0631, 95 },
    { 0x7ccc0634, 93 },
    { 0x58fc0634, 93 },
    { 0xe68c0636, 90 },
    { 0xec2c0636, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ec063d, 90 },
    { 0x9dac063e, 99 },
    { 0xca6c063f, 95 },
    { 0x3d3c0640, 97 },
    { 0x944c0641, 76 },
    { 0xa3c0642, 55 },
    { 0xc23c0643, 61 },
    { 0x6ccc0643, 95 },
    { 0x257c0641, 61 },
    { 0x47cc0641, 33 },
    { 0x980c0647, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcc064c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x957c064f, 98 },
    { 0x6cbc0650, 52 },
    { 0xe1fc0650, 23 },
    { 0xcc5c0651, 90 },
    { 0x4a2c0650, 46 },
    { 0xa88c064f, 90 },
    { 0xaebc0655, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c065a, 43 },
    { 0, 0 },
    { 0xa44c065c, 90 },
    { 0xe2ec065d, 90 },
    { 0xfc4c065d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c0663, 80 },
    { 0, 0 },
    { 0xae8c0665, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbc066d, 76 },
    { 0x5eec066e, 77 },
    { 0, 0 },
    { 0xc3c0670, 76 },
    { 0x532c0670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c0676, 46 },
    { 0, 0 },
    { 0x7b8c0678, 76 },
    { 0x1bcc0679, 95 },
    { 0x986c0678, 90 },
    { 0xba2c0678, 43 },
    { 0x523c0679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc067f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bc0683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c0686, 33 },
    { 0xdcfc0687, 95 },
    { 0x658c0688, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc068c, 61 },
    { 0, 0 },
    { 0x669c068e, 98 },
    { 0xb80c068f, 76 },
    { 0x72bc068e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fc0694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2c0699, 98 },
    { 0, 0 },
    { 0x8efc069b, 66 },
    { 0xa56c069c, 98 },
    { 0x30ec069d, 43 },
    { 0x44fc069b, 33 },
    { 0xe38c069b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293c06a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac06a9, 91 },
    { 0, 0 },
    { 0x4fbc06ab, 81 },
    { 0, 0 },
    { 0x18dc06ad, 77 },
    { 0, 0 },
    { 0xbd1c06af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60c06b6, 55 },
    { 0x15dc06b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c06bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x305c06c0, 96 },
    { 0x127c06c1, 34 },
    { 0x9a0c06c2, 23 },
    { 0x908c06c3, 65 },
    { 0xab9c06c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5c06ca, 76 },
    { 0x7afc06cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbc06ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fc06d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c06d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c06df, 95 },
    { 0, 0 },
    { 0x605c06e1, 90 },
    { 0, 0 },
    { 0xc34c06e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ec06e8, 55 },
    { 0x4a8c06e9, 33 },
    { 0x3fdc06e8, 98 },
    { 0x9aec06eb, 33 },
    { 0xc5c06ec, 46 },
    { 0, 0 },
    { 0xe95c06ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c06f9, 93 },
    { 0, 0 },
    { 0x63bc06fb, 46 },
    { 0x7a8c06fc, 98 },
    { 0, 0 },
    { 0x45fc06fe, 61 },
    { 0, 0 },
    { 0x505c0700, 33 },
    { 0x589c0700, 98 },
    { 0x19dc0702, 37 },
    { 0x458c0702, 23 },
    { 0xe23c0701, 95 },
    { 0, 0 },
    { 0x565c0706, 95 },
    { 0, 0 },
    { 0x3acc0708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddc070b, 98 },
    { 0, 0 },
    { 0xeb0c070d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c0711, 90 },
    { 0x5cac0711, 40 },
    { 0, 0 },
    { 0x278c0714, 93 },
    { 0xcd7c0715, 67 },
    { 0x610c0714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c071a, 23 },
    { 0, 0 },
    { 0x8a9c071c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9c071f, 90 },
    { 0x8adc071f, 99 },
    { 0, 0 },
    { 0x52dc0722, 87 },
    { 0x7d1c0722, 85 },
    { 0x567c0724, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00c072a, 93 },
    { 0, 0 },
    { 0xeaac072c, 43 },
    { 0x628c072d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41c0734, 55 },
    { 0xab2c0735, 33 },
    { 0x40ac0736, 55 },
    { 0x24bc0737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c073b, 61 },
    { 0x19dc073c, 100 },
    { 0, 0 },
    { 0xf03c073e, 100 },
    { 0x7bfc073f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc0743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c0748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c074d, 33 },
    { 0xdd9c074e, 81 },
    { 0x5eec074f, 68 },
    { 0x23ac074f, 90 },
    { 0x88dc0751, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ec0758, 100 },
    { 0x398c0759, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93c0760, 93 },
    { 0x553c0760, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c0764, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fec0768, 95 },
    { 0xf59c0769, 23 },
    { 0, 0 },
    { 0xb16c076b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0c0771, 87 },
    { 0, 0 },
    { 0x132c0773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ac0776, 78 },
    { 0x174c0776, 52 },
    { 0x87ac0778, 33 },
    { 0, 0 },
    { 0x63ec077a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543c0787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec078c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9c078f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2c0793, 66 },
    { 0x494c0794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ac0799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8c079c, 91 },
    { 0x75fc079d, 88 },
    { 0, 0 },
    { 0xb42c079f, 90 },
    { 0xacec07a0, 90 },
    { 0xe13c07a0, 36 },
    { 0x382c07a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5c07a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdc07b7, 90 },
    { 0xa9ec07b8, 33 },
    { 0xee4c07b9, 95 },
    { 0x1e1c07ba, 23 },
    { 0x438c07bb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc07be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30c07c4, 46 },
    { 0x218c07c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c07dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x286c07df, 17 },
    { 0x2a1c07e0, 87 },
    { 0, 0 },
    { 0x4efc07e2, 33 },
    { 0, 0 },
    { 0x2c0c07e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac07e7, 33 },
    { 0x944c07e7, 61 },
    { 0x1f3c07e9, 72 },
    { 0, 0 },
    { 0x734c07eb, 77 },
    { 0x9a5c07ec, 66 },
    { 0xd9dc07ec, 95 },
    { 0x168c07ec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cc07f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bc07f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac07fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7c0802, 33 },
    { 0xf86c0803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262c080a, 76 },
    { 0xca1c080b, 90 },
    { 0x4d2c080c, 88 },
    { 0xa7fc080a, 76 },
    { 0, 0 },
    { 0xd97c080f, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c0813, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9c0822, 88 },
    { 0, 0 },
    { 0x77fc0824, 90 },
    { 0, 0 },
    { 0x111c0826, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fc0832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c0835, 90 },
    { 0xc61c0835, 61 },
    { 0x1c3c0836, 98 },
    { 0x777c0836, 70 },
    { 0, 0 },
    { 0x8a7c083a, 23 },
    { 0x3f4c083a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6c083f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc0842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc0846, 85 },
    { 0xcddc0847, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c0852, 90 },
    { 0x18c0853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc0856, 23 },
    { 0xd28c0857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dc085a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6c085d, 90 },
    { 0x8dac085e, 97 },
    { 0x954c085f, 88 },
    { 0xb21c085d, 61 },
    { 0x736c0861, 23 },
    { 0x2cfc085e, 100 },
    { 0x4bfc0863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cac086d, 98 },
    { 0, 0 },
    { 0x2d3c086f, 76 },
    { 0, 0 },
    { 0x217c0871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac0875, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ac087a, 90 },
    { 0x28bc087b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dc087f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6c0888, 90 },
    { 0, 0 },
    { 0x2ac088a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc088d, 23 },
    { 0x37fc088e, 23 },
    { 0x704c088d, 90 },
    { 0x886c088e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c0895, 33 },
    { 0xa1c0896, 85 },
    { 0xe94c0897, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4c089d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c08a3, 78 },
    { 0x2eec08a4, 84 },
    { 0, 0 },
    { 0xb99c08a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88c08ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c08b0, 90 },
    { 0x5d6c08b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc08b4, 90 },
    { 0, 0 },
    { 0x6d3c08b6, 68 },
    { 0, 0 },
    { 0x205c08b8, 33 },
    { 0, 0 },
    { 0x43cc08ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bc08c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1c08cc, 55 },
    { 0x11ac08cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ec08d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc08d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc08da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c08e1, 67 },
    { 0x553c08e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fc08e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x507c08e8, 77 },
    { 0x451c08e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fc08ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c08f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c08fc, 81 },
    { 0x404c08fd, 78 },
    { 0, 0 },
    { 0x177c08ff, 98 },
    { 0, 0 },
    { 0xca3c0901, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c090f, 66 },
    { 0xc58c0910, 90 },
    { 0x40ec0911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27c0914, 98 },
    { 0, 0 },
    { 0x3ebc0916, 33 },
    { 0xcdec0917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c091a, 95 },
    { 0, 0 },
    { 0x7a5c091c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac0921, 61 },
    { 0xf0fc0922, 91 },
    { 0x6c8c0921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x273c0926, 67 },
    { 0x924c0927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c092c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2c0934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc093c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c093f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc0942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194c094a, 66 },
    { 0xd76c094b, 99 },
    { 0xc05c094b, 61 },
    { 0, 0 },
    { 0x25ac094e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc0952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09c0955, 33 },
    { 0x502c0955, 33 },
    { 0x556c0957, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c095f, 83 },
    { 0x531c0960, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec0963, 33 },
    { 0xda8c0964, 90 },
    { 0x229c0965, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904c096b, 33 },
    { 0xfe1c096b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac096f, 17 },
    { 0x337c0970, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedc0973, 61 },
    { 0, 0 },
    { 0xcaec0975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928c097b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x271c097e, 76 },
    { 0x792c097e, 23 },
    { 0xdd0c097e, 34 },
    { 0x90ac0980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c098a, 34 },
    { 0x78dc098b, 91 },
    { 0x7b8c098b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x717c098f, 55 },
    { 0xf03c098f, 46 },
    { 0x399c0991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec0995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42c0998, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fc099d, 34 },
    { 0x330c099e, 78 },
    { 0x17cc099e, 55 },
    { 0x6b4c099e, 76 },
    { 0x8d3c09a1, 61 },
    { 0xc4ac09a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493c09a7, 76 },
    { 0, 0 },
    { 0xcf1c09a9, 17 },
    { 0, 0 },
    { 0x62cc09ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c09af, 95 },
    { 0, 0 },
    { 0xcc2c09b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c09b4, 98 },
    { 0, 0 },
    { 0xd25c09b6, 51 },
    { 0xb4bc09b6, 25 },
    { 0x323c09b8, 23 },
    { 0x333c09b8, 43 },
    { 0x708c09b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x928c09bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec09c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ac09c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac09cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c09cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c09d6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc09d9, 64 },
    { 0xc54c09da, 91 },
    { 0x56ac09db, 55 },
    { 0x826c09dc, 90 },
    { 0x240c09dd, 90 },
    { 0, 0 },
    { 0x4a4c09df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc09e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc09e9, 76 },
    { 0, 0 },
    { 0xaa3c09eb, 90 },
    { 0x3c6c09eb, 76 },
    { 0, 0 },
    { 0x406c09ee, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc09f2, 33 },
    { 0x464c09f3, 44 },
    { 0xd2c09f4, 46 },
    { 0, 0 },
    { 0xd9bc09f6, 76 },
    { 0, 0 },
    { 0x1c5c09f8, 98 },
    { 0xe61c09f9, 95 },
    { 0xfb2c09fa, 95 },
    { 0xe4cc09fa, 90 },
    { 0xfd1c09fa, 17 },
    { 0, 0 },
    { 0x2afc09fe, 68 },
    { 0xa97c09ff, 61 },
    { 0xa7ac0a00, 98 },
    { 0x443c09ff, 90 },
    { 0x6a0c0a01, 87 },
    { 0x2bc0a03, 61 },
    { 0xf3ac0a03, 84 },
    { 0x381c0a03, 90 },
    { 0x980c0a06, 61 },
    { 0xdfac0a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40c0a0e, 99 },
    { 0x7cfc0a0f, 98 },
    { 0x971c0a10, 98 },
    { 0, 0 },
    { 0xcdac0a12, 95 },
    { 0, 0 },
    { 0x2c0a14, 33 },
    { 0, 0 },
    { 0x39c0a16, 87 },
    { 0xda5c0a16, 61 },
    { 0xa15c0a18, 98 },
    { 0xcd2c0a16, 61 },
    { 0xb58c0a18, 34 },
    { 0xef1c0a1b, 76 },
    { 0xffbc0a18, 95 },
    { 0x4fac0a1d, 23 },
    { 0xb7cc0a1e, 61 },
    { 0x21cc0a1e, 33 },
    { 0xcfcc0a1e, 34 },
    { 0x561c0a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984c0a25, 46 },
    { 0xe33c0a25, 61 },
    { 0, 0 },
    { 0x6eec0a28, 33 },
    { 0, 0 },
    { 0xef6c0a2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c0a33, 36 },
    { 0, 0 },
    { 0x3a2c0a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fc0a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c0a40, 76 },
    { 0xe7bc0a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933c0a49, 91 },
    { 0x244c0a49, 23 },
    { 0x3fbc0a4b, 33 },
    { 0xf12c0a4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949c0a58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c0a5b, 76 },
    { 0xeedc0a5c, 34 },
    { 0x1d2c0a5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdec0a62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebc0a66, 61 },
    { 0xc75c0a66, 61 },
    { 0, 0 },
    { 0x987c0a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69c0a71, 61 },
    { 0, 0 },
    { 0x86bc0a73, 95 },
    { 0xeec0a73, 23 },
    { 0x629c0a75, 23 },
    { 0x3b5c0a74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642c0a7b, 100 },
    { 0, 0 },
    { 0xd4ac0a7d, 87 },
    { 0, 0 },
    { 0xab9c0a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8c0a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64c0a8d, 33 },
    { 0xedbc0a8e, 91 },
    { 0x6acc0a8f, 33 },
    { 0x8fc0a90, 28 },
    { 0xe77c0a8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c0a97, 61 },
    { 0x275c0a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c0a9e, 90 },
    { 0x6f7c0a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc0aa9, 99 },
    { 0, 0 },
    { 0x4ecc0aab, 23 },
    { 0x33ec0aac, 23 },
    { 0xea2c0aac, 23 },
    { 0xe26c0aac, 46 },
    { 0x585c0aae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7c0ab6, 61 },
    { 0, 0 },
    { 0x80bc0ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fec0abd, 77 },
    { 0x456c0abd, 23 },
    { 0x36c0abe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82c0ac5, 90 },
    { 0x43ac0ac5, 76 },
    { 0, 0 },
    { 0xddac0ac8, 90 },
    { 0x7a9c0ac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec0acd, 55 },
    { 0xf5fc0ace, 93 },
    { 0x212c0acf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5c0ad2, 90 },
    { 0xc1ec0ad3, 90 },
    { 0xec9c0ad2, 33 },
    { 0x96ac0ad5, 95 },
    { 0x2d7c0ad2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311c0adc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778c0ae0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc0ae4, 41 },
    { 0, 0 },
    { 0x9cc0ae6, 100 },
    { 0, 0 },
    { 0x524c0ae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc0aeb, 90 },
    { 0x6e2c0aec, 33 },
    { 0x35cc0aec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c0af3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edc0af6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44c0afa, 90 },
    { 0, 0 },
    { 0xa29c0afc, 23 },
    { 0, 0 },
    { 0x1d5c0afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c0b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c0b08, 98 },
    { 0x99ac0b09, 96 },
    { 0, 0 },
    { 0xa7ec0b0b, 77 },
    { 0, 0 },
    { 0x75fc0b0d, 33 },
    { 0x5c8c0b0e, 33 },
    { 0, 0 },
    { 0x285c0b10, 33 },
    { 0x1ebc0b11, 46 },
    { 0, 0 },
    { 0x9dcc0b13, 23 },
    { 0, 0 },
    { 0xff2c0b15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12c0b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730c0b1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c0b2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdc0b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ac0b34, 95 },
    { 0, 0 },
    { 0xbd7c0b36, 23 },
    { 0x937c0b37, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbc0b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516c0b42, 66 },
    { 0x11ec0b42, 90 },
    { 0, 0 },
    { 0x6dfc0b45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ac0b49, 46 },
    { 0xeb1c0b49, 90 },
    { 0x1a9c0b4b, 66 },
    { 0x845c0b49, 33 },
    { 0x96c0b4c, 33 },
    { 0xe15c0b4e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c0b58, 34 },
    { 0xd74c0b59, 33 },
    { 0, 0 },
    { 0x60cc0b5b, 90 },
    { 0xcb4c0b5c, 33 },
    { 0x6dbc0b5d, 61 },
    { 0xc02c0b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fc0b61, 23 },
    { 0x4acc0b62, 17 },
    { 0xe22c0b63, 99 },
    { 0, 0 },
    { 0xe12c0b65, 84 },
    { 0xf3dc0b66, 23 },
    { 0x739c0b66, 76 },
    { 0xec6c0b67, 76 },
    { 0xfedc0b68, 99 },
    { 0, 0 },
    { 0xf27c0b6b, 90 },
    { 0x50dc0b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c0b71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ac0b74, 23 },
    { 0, 0 },
    { 0x642c0b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c0b7a, 61 },
    { 0, 0 },
    { 0x378c0b7c, 90 },
    { 0x783c0b7d, 61 },
    { 0xa48c0b7d, 23 },
    { 0xee5c0b7f, 67 },
    { 0xc1dc0b80, 33 },
    { 0xdc7c0b7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c0b87, 66 },
    { 0, 0 },
    { 0x837c0b89, 91 },
    { 0, 0 },
    { 0x5c3c0b8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24c0b8e, 93 },
    { 0x6a3c0b8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c0b93, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c0b9b, 55 },
    { 0x97fc0b9c, 61 },
    { 0x5bcc0b9d, 86 },
    { 0, 0 },
    { 0x29bc0b9f, 66 },
    { 0x5e6c0b9f, 93 },
    { 0xaa6c0ba1, 95 },
    { 0, 0 },
    { 0xdd7c0ba3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x475c0ba6, 43 },
    { 0, 0 },
    { 0x900c0ba8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dc0bac, 95 },
    { 0x44ac0bad, 77 },
    { 0xa80c0bad, 95 },
    { 0x9a0c0bad, 31 },
    { 0x4bdc0bae, 61 },
    { 0, 0 },
    { 0x15dc0bb2, 55 },
    { 0xbf0c0bb2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac0bb7, 40 },
    { 0, 0 },
    { 0xec2c0bb9, 64 },
    { 0xb9ec0bba, 61 },
    { 0x96ac0bbb, 48 },
    { 0xac5c0bbc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c0bc0, 51 },
    { 0x590c0bc0, 33 },
    { 0x7d8c0bc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc0bcd, 43 },
    { 0x73c0bce, 52 },
    { 0, 0 },
    { 0x2ebc0bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4c0bd3, 90 },
    { 0xe9bc0bd4, 52 },
    { 0xac1c0bd5, 46 },
    { 0x2c9c0bd5, 23 },
    { 0xa61c0bd4, 75 },
    { 0x13fc0bd8, 90 },
    { 0x4e0c0bd9, 81 },
    { 0, 0 },
    { 0x85ec0bdb, 42 },
    { 0x8cc0bdc, 83 },
    { 0, 0 },
    { 0x35fc0bde, 90 },
    { 0x6afc0bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5c0be7, 61 },
    { 0x3c6c0be8, 43 },
    { 0x71bc0be9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76c0bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac0bf3, 95 },
    { 0x9cac0bf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c0bf7, 95 },
    { 0x31c0bf8, 95 },
    { 0, 0 },
    { 0x21ec0bfa, 23 },
    { 0x6e3c0bfa, 90 },
    { 0xc02c0bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x947c0bff, 90 },
    { 0, 0 },
    { 0x903c0c01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acc0c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fc0c0b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c0c11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec0c18, 67 },
    { 0xc5bc0c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c0c1f, 90 },
    { 0, 0 },
    { 0x76ac0c21, 33 },
    { 0xfc1c0c22, 90 },
    { 0x610c0c23, 23 },
    { 0x4ebc0c24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336c0c29, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4c0c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c0c32, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c0c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8c0c3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdc0c3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c0c43, 55 },
    { 0x4f4c0c44, 76 },
    { 0xc14c0c44, 93 },
    { 0xd30c0c46, 60 },
    { 0, 0 },
    { 0x738c0c48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c0c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ec0c4f, 76 },
    { 0, 0 },
    { 0x4d1c0c51, 80 },
    { 0x14cc0c52, 100 },
    { 0x59ac0c53, 33 },
    { 0, 0 },
    { 0x438c0c55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6c0c5a, 42 },
    { 0x762c0c5b, 95 },
    { 0, 0 },
    { 0x4e3c0c5d, 23 },
    { 0xf5fc0c5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fc0c63, 90 },
    { 0, 0 },
    { 0x157c0c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c0c68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810c0c6f, 17 },
    { 0, 0 },
    { 0x6f4c0c71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fc0c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec0c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c0c85, 66 },
    { 0, 0 },
    { 0x9e4c0c87, 23 },
    { 0, 0 },
    { 0x55ec0c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc0c8d, 17 },
    { 0x8cbc0c8e, 98 },
    { 0x524c0c8f, 61 },
    { 0x668c0c8f, 38 },
    { 0x4d2c0c91, 94 },
    { 0x693c0c92, 76 },
    { 0x5b3c0c93, 93 },
    { 0x33fc0c94, 98 },
    { 0x9afc0c95, 98 },
    { 0xed8c0c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c0c99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c0ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126c0ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ec0caa, 33 },
    { 0x9cec0caa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c0cb0, 66 },
    { 0, 0 },
    { 0x2f2c0cb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cc0cb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c0cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c0cbe, 90 },
    { 0x23bc0cbe, 68 },
    { 0, 0 },
    { 0x2f4c0cc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c0cc4, 66 },
    { 0x1cac0cc4, 55 },
    { 0x9dcc0cc4, 98 },
    { 0x3cdc0cc4, 99 },
    { 0x353c0cc4, 56 },
    { 0x7dcc0cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ec0cd8, 67 },
    { 0xb3bc0cd8, 61 },
    { 0x1c0c0cd8, 46 },
    { 0, 0 },
    { 0xdbbc0cdc, 46 },
    { 0x919c0cdd, 87 },
    { 0xfd0c0cdd, 23 },
    { 0x6acc0cdc, 76 },
    { 0xae6c0ce0, 55 },
    { 0x383c0ce1, 61 },
    { 0xd56c0ce1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x607c0ce5, 76 },
    { 0xef3c0ce6, 90 },
    { 0x125c0ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x995c0cea, 87 },
    { 0x281c0ceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c0cf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04c0cf6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ec0d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89c0d09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741c0d12, 95 },
    { 0, 0 },
    { 0x701c0d14, 43 },
    { 0x1b2c0d15, 75 },
    { 0x5e0c0d16, 98 },
    { 0x4c0d17, 90 },
    { 0x402c0d18, 98 },
    { 0, 0 },
    { 0xbe4c0d1a, 98 },
    { 0xcf8c0d1b, 17 },
    { 0x36cc0d1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0d1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c0d22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c0d2a, 46 },
    { 0x720c0d2b, 67 },
    { 0x9ec0d2a, 84 },
    { 0x8a4c0d2d, 95 },
    { 0, 0 },
    { 0x5b8c0d2f, 56 },
    { 0xcffc0d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c0d36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4c0d3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c0d3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c0d44, 34 },
    { 0x1a3c0d45, 33 },
    { 0xe77c0d44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3c0d49, 68 },
    { 0xf25c0d49, 66 },
    { 0, 0 },
    { 0x32bc0d4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc0d4f, 90 },
    { 0, 0 },
    { 0x875c0d51, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2c0d56, 61 },
    { 0, 0 },
    { 0x926c0d58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x202c0d5b, 99 },
    { 0x7aac0d5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc0d60, 95 },
    { 0, 0 },
    { 0x419c0d62, 61 },
    { 0x9d7c0d63, 98 },
    { 0xcadc0d64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c0d6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465c0d74, 100 },
    { 0x5e8c0d75, 90 },
    { 0xc0cc0d76, 98 },
    { 0xe74c0d77, 98 },
    { 0x215c0d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c0d7b, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443c0d80, 23 },
    { 0, 0 },
    { 0xd47c0d82, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fac0d85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc0d8c, 95 },
    { 0x5c3c0d8d, 90 },
    { 0, 0 },
    { 0x8d1c0d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149c0d96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c0d9e, 85 },
    { 0x1acc0d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c0da2, 90 },
    { 0xdb7c0da3, 61 },
    { 0x627c0da2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575c0daa, 65 },
    { 0xafbc0dab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c0db1, 90 },
    { 0xd11c0db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ec0dbb, 23 },
    { 0x37fc0dbc, 46 },
    { 0xa62c0dbc, 76 },
    { 0, 0 },
    { 0xedfc0dbf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c0dc2, 93 },
    { 0, 0 },
    { 0xde4c0dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c0dcc, 23 },
    { 0x53ac0dcd, 23 },
    { 0x654c0dce, 68 },
    { 0xf4ac0dce, 34 },
    { 0x90ac0dce, 90 },
    { 0x7f0c0dce, 52 },
    { 0, 0 },
    { 0xe58c0dd3, 23 },
    { 0x81c0dd4, 67 },
    { 0, 0 },
    { 0x694c0dd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec0dd9, 43 },
    { 0, 0 },
    { 0xc20c0ddb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4c0dde, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442c0de2, 61 },
    { 0x549c0de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c0de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c0dec, 61 },
    { 0xe98c0dec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60c0df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05c0df7, 98 },
    { 0, 0 },
    { 0x274c0df9, 90 },
    { 0, 0 },
    { 0x585c0dfb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c0e01, 33 },
    { 0, 0 },
    { 0x99cc0e03, 90 },
    { 0xd24c0e04, 90 },
    { 0, 0 },
    { 0x2eec0e06, 23 },
    { 0x36bc0e07, 90 },
    { 0, 0 },
    { 0x264c0e09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c0e0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ec0e14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac0e1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc0e25, 43 },
    { 0x1fcc0e26, 87 },
    { 0xca4c0e27, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c0e31, 81 },
    { 0xcbcc0e32, 91 },
    { 0xf78c0e32, 52 },
    { 0x9b1c0e31, 24 },
    { 0x5f0c0e33, 33 },
    { 0x88c0e36, 61 },
    { 0xa87c0e36, 33 },
    { 0x28dc0e38, 61 },
    { 0xd80c0e39, 95 },
    { 0x8dec0e34, 84 },
    { 0xf5fc0e3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6c0e41, 23 },
    { 0x51bc0e42, 90 },
    { 0xb0cc0e43, 81 },
    { 0xa5ac0e41, 43 },
    { 0xd7bc0e45, 90 },
    { 0x7f2c0e46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbc0e4c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0e52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c0e5a, 61 },
    { 0x97ac0e5b, 100 },
    { 0x191c0e5b, 23 },
    { 0, 0 },
    { 0x574c0e5e, 55 },
    { 0, 0 },
    { 0x8c7c0e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95c0e65, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8c0e6d, 76 },
    { 0, 0 },
    { 0xb49c0e6f, 33 },
    { 0x855c0e70, 90 },
    { 0x530c0e71, 78 },
    { 0xf1dc0e71, 23 },
    { 0x2b3c0e73, 33 },
    { 0x8c6c0e73, 100 },
    { 0, 0 },
    { 0xa86c0e76, 78 },
    { 0, 0 },
    { 0xb10c0e78, 95 },
    { 0, 0 },
    { 0xca1c0e7a, 98 },
    { 0x14ac0e7b, 23 },
    { 0xeb6c0e7c, 61 },
    { 0, 0 },
    { 0xba3c0e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196c0e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6c0e88, 55 },
    { 0x249c0e88, 33 },
    { 0x469c0e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c0e8f, 34 },
    { 0, 0 },
    { 0x452c0e91, 43 },
    { 0xe8cc0e91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acc0e95, 33 },
    { 0, 0 },
    { 0x31cc0e97, 66 },
    { 0xd3c0e98, 55 },
    { 0x6dbc0e98, 40 },
    { 0xe7fc0e97, 90 },
    { 0x998c0e9b, 61 },
    { 0x43c0e9b, 61 },
    { 0x297c0e9d, 98 },
    { 0x739c0e98, 61 },
    { 0x697c0e9e, 61 },
    { 0x337c0e9f, 90 },
    { 0, 0 },
    { 0xf32c0ea2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c0ea9, 33 },
    { 0x679c0eaa, 61 },
    { 0x5f3c0eab, 95 },
    { 0x631c0eac, 23 },
    { 0xd1c0eac, 55 },
    { 0xd38c0eaa, 98 },
    { 0xfaac0eaf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85c0eb6, 93 },
    { 0, 0 },
    { 0x954c0eb8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68c0ebd, 34 },
    { 0, 0 },
    { 0xb0bc0ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c0ec2, 61 },
    { 0, 0 },
    { 0xcfec0ec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c0ecb, 46 },
    { 0x818c0ecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c0ecf, 78 },
    { 0x89cc0ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191c0ed5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dac0ed8, 95 },
    { 0, 0 },
    { 0x12ac0eda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c0ee4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c0eec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ac0eef, 95 },
    { 0x187c0ef0, 23 },
    { 0x3d4c0ef1, 23 },
    { 0, 0 },
    { 0x259c0ef3, 33 },
    { 0x7ecc0ef4, 90 },
    { 0x5d8c0ef4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bc0ef9, 55 },
    { 0, 0 },
    { 0x49ac0efb, 98 },
    { 0, 0 },
    { 0xc96c0efd, 17 },
    { 0xe11c0efe, 48 },
    { 0x833c0efd, 55 },
    { 0xca9c0f00, 90 },
    { 0x6e0c0eff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ac0f04, 90 },
    { 0, 0 },
    { 0x6c1c0f06, 61 },
    { 0x882c0f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52c0f0e, 34 },
    { 0xe76c0f0f, 90 },
    { 0x168c0f10, 95 },
    { 0, 0 },
    { 0x837c0f12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363c0f18, 61 },
    { 0xa87c0f18, 34 },
    { 0xc74c0f19, 98 },
    { 0x33dc0f1b, 90 },
    { 0xbfbc0f19, 67 },
    { 0x13ac0f18, 95 },
    { 0x43cc0f1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc0f24, 90 },
    { 0, 0 },
    { 0x5ffc0f26, 98 },
    { 0x4c5c0f27, 34 },
    { 0x6c9c0f27, 17 },
    { 0x431c0f29, 43 },
    { 0xa54c0f29, 55 },
    { 0x1aac0f29, 22 },
    { 0xab8c0f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc0f30, 61 },
    { 0xd3fc0f31, 43 },
    { 0x6c4c0f30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c0f37, 99 },
    { 0xdb7c0f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc0f42, 55 },
    { 0x91bc0f43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c0f4c, 52 },
    { 0x475c0f4d, 99 },
    { 0, 0 },
    { 0x31cc0f4f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dc0f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc0f58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c0f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c0f63, 23 },
    { 0xc7bc0f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3c0f6c, 23 },
    { 0, 0 },
    { 0xa61c0f6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53c0f74, 80 },
    { 0x8a2c0f74, 61 },
    { 0x58ec0f76, 76 },
    { 0, 0 },
    { 0xb40c0f78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c0f7f, 48 },
    { 0xcfec0f80, 76 },
    { 0, 0 },
    { 0xcb4c0f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9c0f89, 90 },
    { 0, 0 },
    { 0xd99c0f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afc0f92, 95 },
    { 0, 0 },
    { 0x80c0f94, 66 },
    { 0x983c0f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824c0f99, 67 },
    { 0x1b5c0f9a, 67 },
    { 0xed5c0f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c0f9f, 90 },
    { 0xacac0fa0, 88 },
    { 0xa51c0f9f, 61 },
    { 0x27bc0fa2, 100 },
    { 0, 0 },
    { 0xa2dc0fa4, 33 },
    { 0, 0 },
    { 0x621c0fa6, 93 },
    { 0xbdfc0fa6, 93 },
    { 0, 0 },
    { 0x584c0fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c0fb3, 91 },
    { 0, 0 },
    { 0x4e4c0fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c0fbd, 61 },
    { 0, 0 },
    { 0x29fc0fbf, 98 },
    { 0, 0 },
    { 0x35bc0fc1, 90 },
    { 0xcd8c0fc2, 61 },
    { 0x73ac0fc3, 100 },
    { 0x71ec0fc1, 17 },
    { 0xf66c0fc5, 69 },
    { 0, 0 },
    { 0xe86c0fc7, 91 },
    { 0xca8c0fc8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c0fcc, 23 },
    { 0, 0 },
    { 0x1b6c0fce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33c0fd1, 33 },
    { 0x5b6c0fd1, 34 },
    { 0, 0 },
    { 0xc75c0fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c0fdb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x822c0fde, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec0fe3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19c0fe9, 77 },
    { 0xf73c0fea, 23 },
    { 0x1c8c0feb, 90 },
    { 0x204c0feb, 33 },
    { 0xa77c0fea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dc0ff4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fc0ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c0fff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccc1003, 33 },
    { 0xc80c1004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc1007, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233c100c, 17 },
    { 0, 0 },
    { 0x757c100e, 22 },
    { 0xa07c100f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc1015, 100 },
    { 0x53dc1015, 61 },
    { 0, 0 },
    { 0xdd4c1018, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc1022, 61 },
    { 0x455c1023, 90 },
    { 0x565c1024, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62c102a, 43 },
    { 0x625c102b, 23 },
    { 0xecbc102c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c1030, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bc1033, 17 },
    { 0xd5dc1034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ac103d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8c1043, 61 },
    { 0xa11c1044, 43 },
    { 0xbb3c1045, 98 },
    { 0xc40c1044, 61 },
    { 0xe4c1044, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c104e, 61 },
    { 0xee7c104e, 61 },
    { 0x3b6c1050, 78 },
    { 0x82ec1051, 40 },
    { 0xff5c1052, 61 },
    { 0x60cc1053, 33 },
    { 0x97fc1051, 90 },
    { 0x652c1055, 87 },
    { 0, 0 },
    { 0x80dc1057, 78 },
    { 0x72ec1057, 99 },
    { 0xa1ac1059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c105c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ec1063, 55 },
    { 0x758c1063, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c1068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1071, 90 },
    { 0, 0 },
    { 0x986c1073, 76 },
    { 0x27cc1073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc107c, 98 },
    { 0x758c107d, 90 },
    { 0x1bdc107d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac1085, 43 },
    { 0xff8c1086, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec108b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc108f, 80 },
    { 0xc18c108f, 75 },
    { 0, 0 },
    { 0xee1c1092, 61 },
    { 0x35ec1093, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc1099, 95 },
    { 0x5ecc109a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4c109e, 91 },
    { 0x18ec109f, 59 },
    { 0x9b4c109e, 81 },
    { 0x53ac10a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c10a7, 23 },
    { 0xeb8c10a8, 61 },
    { 0x5b3c10a7, 61 },
    { 0x3e9c10a8, 87 },
    { 0, 0 },
    { 0xf67c10ac, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc10b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c10c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaac10c6, 98 },
    { 0x7cbc10c7, 98 },
    { 0x3f9c10c8, 94 },
    { 0x179c10c9, 90 },
    { 0xa65c10c8, 95 },
    { 0, 0 },
    { 0x287c10cc, 61 },
    { 0, 0 },
    { 0xaffc10ce, 61 },
    { 0, 0 },
    { 0x521c10d0, 52 },
    { 0x5bfc10d1, 91 },
    { 0x41fc10d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697c10d5, 90 },
    { 0x381c10d5, 61 },
    { 0x167c10d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476c10db, 33 },
    { 0x63cc10dc, 33 },
    { 0xb34c10dc, 90 },
    { 0xa42c10de, 95 },
    { 0x44c10dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c10e3, 55 },
    { 0, 0 },
    { 0xbb0c10e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c10ea, 17 },
    { 0, 0 },
    { 0xdbac10ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc10f0, 34 },
    { 0xdf5c10f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421c10f5, 80 },
    { 0xc00c10f5, 90 },
    { 0, 0 },
    { 0x278c10f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c1103, 98 },
    { 0, 0 },
    { 0x8a9c1105, 98 },
    { 0xaeec1106, 90 },
    { 0xfaec1106, 87 },
    { 0xb61c1108, 61 },
    { 0xd41c1109, 52 },
    { 0, 0 },
    { 0xaa3c110b, 23 },
    { 0xf7ac110c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x531c110f, 99 },
    { 0, 0 },
    { 0xae5c1111, 61 },
    { 0x89cc1111, 90 },
    { 0xf39c1113, 76 },
    { 0x184c1111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c111e, 95 },
    { 0x166c111f, 43 },
    { 0xe9cc1120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc1124, 81 },
    { 0x97fc1125, 99 },
    { 0x86c1126, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c112a, 90 },
    { 0xcb4c112b, 61 },
    { 0, 0 },
    { 0xb27c112d, 33 },
    { 0, 0 },
    { 0x8d7c112f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c1134, 33 },
    { 0x72cc1134, 40 },
    { 0x655c1136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712c113a, 33 },
    { 0x480c113b, 55 },
    { 0xb10c113b, 98 },
    { 0xadfc113c, 69 },
    { 0, 0 },
    { 0xbfdc113f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4c1142, 56 },
    { 0x12bc1142, 78 },
    { 0xb0ec1143, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603c1151, 90 },
    { 0xd25c1152, 90 },
    { 0x1c3c1153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc1156, 53 },
    { 0, 0 },
    { 0xb90c1158, 90 },
    { 0xd21c1159, 90 },
    { 0, 0 },
    { 0x962c115b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3c1160, 33 },
    { 0x391c1161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c1166, 61 },
    { 0x174c1166, 33 },
    { 0xec5c1166, 23 },
    { 0x120c1166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fc116d, 43 },
    { 0x6b2c116e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c1177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2c117b, 76 },
    { 0, 0 },
    { 0x34ac117d, 98 },
    { 0, 0 },
    { 0x417c117f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc1182, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121c1186, 55 },
    { 0x744c1186, 99 },
    { 0, 0 },
    { 0xd61c1189, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fc118d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afc1198, 98 },
    { 0xc1198, 98 },
    { 0x104c119a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec119d, 90 },
    { 0x25bc119e, 8 },
    { 0x67dc119f, 61 },
    { 0x809c119f, 95 },
    { 0xf00c11a1, 99 },
    { 0, 0 },
    { 0xc91c11a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc11a7, 33 },
    { 0x6ac11a7, 33 },
    { 0xe5ac11a9, 93 },
    { 0x97ac11aa, 91 },
    { 0x8c9c11a8, 33 },
    { 0xaf6c11ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c11af, 95 },
    { 0x32c11b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x776c11b3, 33 },
    { 0, 0 },
    { 0x232c11b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c11b8, 90 },
    { 0, 0 },
    { 0x18cc11ba, 68 },
    { 0, 0 },
    { 0x60ec11bc, 75 },
    { 0x6c11bd, 23 },
    { 0x405c11be, 98 },
    { 0, 0 },
    { 0x20fc11c0, 76 },
    { 0x141c11c0, 68 },
    { 0xf27c11c0, 23 },
    { 0xd15c11c2, 100 },
    { 0x73ec11c2, 43 },
    { 0xa09c11c5, 95 },
    { 0x5fdc11c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8c11cc, 33 },
    { 0xf36c11cd, 95 },
    { 0x35bc11cd, 33 },
    { 0, 0 },
    { 0x325c11d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974c11d9, 95 },
    { 0x2c4c11da, 85 },
    { 0x9eac11d9, 90 },
    { 0xbd8c11dc, 34 },
    { 0xb29c11dd, 33 },
    { 0x2a0c11d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc11e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcc11f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43c11f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c11f9, 100 },
    { 0x995c11fa, 98 },
    { 0, 0 },
    { 0x29ac11fc, 95 },
    { 0x3fc11fd, 88 },
    { 0xaa4c11fe, 61 },
    { 0xb09c11ff, 90 },
    { 0x4ebc11ff, 23 },
    { 0x410c1200, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dc1205, 23 },
    { 0x755c1205, 100 },
    { 0x160c1206, 88 },
    { 0x6e8c1207, 33 },
    { 0x5d6c1209, 43 },
    { 0x41bc1209, 61 },
    { 0x221c1205, 96 },
    { 0xf32c1209, 61 },
    { 0, 0 },
    { 0xd6ec120e, 76 },
    { 0x539c120f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c1213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ec121d, 95 },
    { 0xb7dc121d, 33 },
    { 0xf31c121f, 33 },
    { 0x48fc1220, 90 },
    { 0xb06c1221, 61 },
    { 0, 0 },
    { 0x36ec1223, 61 },
    { 0x942c1223, 95 },
    { 0xd14c1224, 61 },
    { 0, 0 },
    { 0xadec1227, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311c122d, 55 },
    { 0, 0 },
    { 0x8eac122f, 98 },
    { 0x602c1230, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c1233, 66 },
    { 0xd0ac1233, 95 },
    { 0x9e4c1235, 33 },
    { 0xca3c1236, 61 },
    { 0xcadc1233, 90 },
    { 0x268c1238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98c1240, 81 },
    { 0xe76c1241, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc1247, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cc124a, 33 },
    { 0x704c124a, 55 },
    { 0x96ec124b, 76 },
    { 0xa8dc124d, 91 },
    { 0xbf0c124a, 75 },
    { 0x2ddc124f, 98 },
    { 0xdd0c124f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c1255, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86c1258, 46 },
    { 0xf01c1259, 46 },
    { 0xaddc1259, 61 },
    { 0xfd4c125b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac125f, 84 },
    { 0x8bac125f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c1268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c126d, 95 },
    { 0x231c126d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1c1272, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705c127b, 61 },
    { 0x44bc127c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283c1280, 95 },
    { 0, 0 },
    { 0x480c1282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90c1285, 23 },
    { 0, 0 },
    { 0xd6bc1287, 90 },
    { 0x464c1287, 98 },
    { 0xe75c1289, 77 },
    { 0xf33c128a, 95 },
    { 0xba7c128a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc128e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x169c1291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc1294, 90 },
    { 0, 0 },
    { 0x4bcc1296, 95 },
    { 0x1efc1296, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dc129f, 77 },
    { 0xde0c129f, 61 },
    { 0x1b9c12a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1c12a5, 90 },
    { 0, 0 },
    { 0x698c12a7, 55 },
    { 0xfb5c12a8, 98 },
    { 0xbfec12a8, 23 },
    { 0x87cc12aa, 33 },
    { 0x278c12a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c12b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c12b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436c12c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ec12ce, 81 },
    { 0, 0 },
    { 0x15bc12d0, 33 },
    { 0, 0 },
    { 0xfefc12d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471c12d8, 39 },
    { 0x5a6c12d8, 61 },
    { 0x7eec12da, 76 },
    { 0x352c12d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec12e2, 90 },
    { 0xdf1c12e2, 33 },
    { 0x783c12e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687c12e8, 87 },
    { 0x141c12e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525c12ed, 55 },
    { 0x7b3c12ee, 76 },
    { 0xc0ac12ee, 52 },
    { 0xf1ec12f0, 84 },
    { 0x3aec12f1, 23 },
    { 0xb79c12f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac12f7, 98 },
    { 0x1e8c12f8, 85 },
    { 0x465c12f9, 76 },
    { 0x681c12fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850c1300, 33 },
    { 0x6e0c1301, 33 },
    { 0x420c1302, 61 },
    { 0, 0 },
    { 0x30ec1304, 84 },
    { 0xe19c1304, 99 },
    { 0x644c1306, 90 },
    { 0xfbac1307, 61 },
    { 0x417c1308, 78 },
    { 0x2c1309, 90 },
    { 0xfdcc130a, 93 },
    { 0, 0 },
    { 0x41fc130c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c1312, 61 },
    { 0, 0 },
    { 0x9a6c1314, 84 },
    { 0x58fc1315, 90 },
    { 0x76dc1316, 33 },
    { 0, 0 },
    { 0xca4c1318, 76 },
    { 0x398c1319, 61 },
    { 0, 0 },
    { 0xb61c131b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fc131e, 75 },
    { 0x34bc131f, 68 },
    { 0xce2c1320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc1324, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc132a, 23 },
    { 0xde2c132a, 90 },
    { 0x6dc132c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfc132f, 98 },
    { 0xf3dc1330, 33 },
    { 0, 0 },
    { 0x925c1332, 98 },
    { 0xbb6c1333, 33 },
    { 0x2e5c1334, 90 },
    { 0, 0 },
    { 0xf30c1336, 33 },
    { 0x4b8c1337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ac133a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6c133f, 66 },
    { 0xcccc133f, 23 },
    { 0, 0 },
    { 0xfbc1342, 88 },
    { 0x2cc1343, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821c1349, 34 },
    { 0x49fc1349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc1352, 55 },
    { 0x561c1353, 61 },
    { 0x9ec1354, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80c1357, 61 },
    { 0x788c1357, 46 },
    { 0x7f5c1357, 55 },
    { 0x3a0c135a, 95 },
    { 0, 0 },
    { 0xc39c135c, 90 },
    { 0xa1ec135d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c1360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c1364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c1367, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dc136a, 90 },
    { 0, 0 },
    { 0xc1fc136c, 53 },
    { 0x8a4c136d, 95 },
    { 0x227c136d, 61 },
    { 0xaf9c136f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68c1378, 55 },
    { 0xe14c1378, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bc137c, 88 },
    { 0xfd8c137d, 69 },
    { 0x828c137d, 90 },
    { 0xc56c137c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c1385, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c138d, 34 },
    { 0, 0 },
    { 0xfbdc138f, 61 },
    { 0x6a6c1390, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448c139b, 43 },
    { 0x46ac139c, 61 },
    { 0x43ec139d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dc13a0, 61 },
    { 0xa28c13a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x910c13a4, 98 },
    { 0x2c0c13a5, 99 },
    { 0x45c13a6, 34 },
    { 0x90c13a5, 100 },
    { 0x3cc13a8, 90 },
    { 0xbf5c13a5, 98 },
    { 0x6b4c13a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2c13ad, 23 },
    { 0xa68c13ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ec13b6, 33 },
    { 0x8ffc13b7, 55 },
    { 0x4f6c13b8, 76 },
    { 0xddac13b8, 90 },
    { 0xb68c13b7, 43 },
    { 0xa7ec13bb, 90 },
    { 0xee1c13bb, 43 },
    { 0x45dc13bd, 55 },
    { 0, 0 },
    { 0x921c13bf, 90 },
    { 0xbdcc13bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c13c5, 23 },
    { 0xf65c13c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35c13ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x194c13cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245c13d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x217c13d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c13d7, 90 },
    { 0, 0 },
    { 0x270c13d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec13dc, 99 },
    { 0, 0 },
    { 0xd72c13de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc13e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c13ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdc13f2, 90 },
    { 0xa7ec13f3, 34 },
    { 0xb74c13f4, 52 },
    { 0xd08c13f5, 61 },
    { 0xd14c13f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c13f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c13fc, 90 },
    { 0xecc13fc, 55 },
    { 0x5d7c13fe, 66 },
    { 0x4c3c13ff, 90 },
    { 0x67ac13fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac1408, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc140b, 93 },
    { 0x2ebc140b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c140f, 60 },
    { 0x73ec1410, 33 },
    { 0x783c1410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c1416, 34 },
    { 0x803c1417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x873c141a, 65 },
    { 0x93dc141b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2c141f, 87 },
    { 0x904c1420, 76 },
    { 0x9eec1421, 90 },
    { 0xa48c1422, 61 },
    { 0x72bc1423, 88 },
    { 0, 0 },
    { 0xe20c1425, 33 },
    { 0x58cc1426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05c142b, 98 },
    { 0x96ec142c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac1432, 95 },
    { 0x2b7c1433, 91 },
    { 0, 0 },
    { 0x105c1435, 52 },
    { 0, 0 },
    { 0xfc4c1437, 65 },
    { 0, 0 },
    { 0xd42c1439, 84 },
    { 0, 0 },
    { 0x680c143b, 98 },
    { 0x413c143c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24c143f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504c1445, 90 },
    { 0x8eac1446, 87 },
    { 0xf6fc1447, 38 },
    { 0x2fac1448, 90 },
    { 0xe37c1448, 66 },
    { 0x2a9c144a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c144d, 98 },
    { 0, 0 },
    { 0x7d2c144f, 90 },
    { 0xca0c1450, 85 },
    { 0x670c144f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6c1456, 67 },
    { 0x1a1c1457, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c145f, 90 },
    { 0xc7ac145f, 84 },
    { 0xbfcc1461, 23 },
    { 0, 0 },
    { 0xf4dc1463, 23 },
    { 0xd8cc1464, 90 },
    { 0, 0 },
    { 0x75ec1466, 23 },
    { 0x1c4c1467, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bc146a, 67 },
    { 0x2a1c146b, 23 },
    { 0xdadc146c, 76 },
    { 0x9c9c146d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fc1474, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6c1477, 95 },
    { 0, 0 },
    { 0x2bac1479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c147c, 100 },
    { 0x582c147d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ac1484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545c148f, 100 },
    { 0x8e6c148f, 33 },
    { 0x776c1491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08c149a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9c149e, 69 },
    { 0, 0 },
    { 0x5a7c14a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fc14a5, 55 },
    { 0xf46c14a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfc14af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec14b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x278c14b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc14b9, 66 },
    { 0, 0 },
    { 0x9bfc14bb, 85 },
    { 0x1a6c14bc, 76 },
    { 0x6aec14bb, 91 },
    { 0x11fc14be, 48 },
    { 0xbd1c14bf, 23 },
    { 0x890c14be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58c14c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ec14c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8c14cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c14d3, 95 },
    { 0x25bc14d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc14d7, 90 },
    { 0x92ac14d7, 77 },
    { 0x690c14d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95c14de, 95 },
    { 0xc38c14df, 33 },
    { 0x2f7c14e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c14e5, 90 },
    { 0, 0 },
    { 0x2e2c14e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1c14ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904c14f1, 83 },
    { 0x19ec14f1, 23 },
    { 0x10ac14f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491c14f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c14fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ac14ff, 90 },
    { 0, 0 },
    { 0xcd6c1501, 61 },
    { 0, 0 },
    { 0x751c1503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c1507, 90 },
    { 0x2f0c1508, 55 },
    { 0xea6c1508, 87 },
    { 0, 0 },
    { 0x68dc150b, 33 },
    { 0, 0 },
    { 0xc77c150d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eec1510, 90 },
    { 0x2e0c1510, 61 },
    { 0, 0 },
    { 0x5c9c1513, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1519, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84c1525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc152b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ac152f, 48 },
    { 0xef7c1530, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c1535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac153d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2c1540, 61 },
    { 0x132c1541, 90 },
    { 0, 0 },
    { 0x505c1543, 36 },
    { 0, 0 },
    { 0x69fc1545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91c1548, 97 },
    { 0, 0 },
    { 0x3fdc154a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c154e, 61 },
    { 0xe30c154f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fc1554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c1557, 46 },
    { 0x360c1557, 90 },
    { 0x3b0c1557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c155d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c1561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec1564, 19 },
    { 0xcd5c1565, 76 },
    { 0x430c1566, 33 },
    { 0x39ec1567, 61 },
    { 0xf01c1565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764c156c, 91 },
    { 0xd31c156d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3c1570, 23 },
    { 0x83ac1571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12c1577, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dc157b, 66 },
    { 0, 0 },
    { 0x2bc157d, 43 },
    { 0xe59c157e, 23 },
    { 0xf7c157e, 33 },
    { 0x38fc1580, 83 },
    { 0xb8cc1580, 87 },
    { 0, 0 },
    { 0x9a2c1583, 84 },
    { 0xfebc1584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0c1589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4c158c, 98 },
    { 0, 0 },
    { 0x671c158e, 61 },
    { 0xd06c158e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cc1592, 55 },
    { 0, 0 },
    { 0x288c1594, 61 },
    { 0x582c1595, 90 },
    { 0x3cec1595, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cc1599, 85 },
    { 0, 0 },
    { 0x737c159b, 85 },
    { 0x4d1c159b, 95 },
    { 0x3f9c159d, 33 },
    { 0xb94c159b, 55 },
    { 0x963c159f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7c15a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630c15aa, 90 },
    { 0, 0 },
    { 0xebac15ac, 90 },
    { 0x69dc15ad, 98 },
    { 0xcacc15ae, 46 },
    { 0, 0 },
    { 0x742c15b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x895c15b3, 80 },
    { 0xba4c15b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25c15bc, 90 },
    { 0, 0 },
    { 0x21ac15be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c15c2, 98 },
    { 0x1d2c15c3, 95 },
    { 0, 0 },
    { 0xc91c15c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fc15c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c15cb, 33 },
    { 0xd06c15cc, 23 },
    { 0x2c3c15cd, 90 },
    { 0x78ac15cc, 34 },
    { 0xc0fc15cb, 61 },
    { 0xd89c15d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26c15d3, 33 },
    { 0xafc15d4, 66 },
    { 0x511c15d5, 33 },
    { 0x40ec15d6, 52 },
    { 0x84cc15d6, 33 },
    { 0xe0dc15d6, 61 },
    { 0, 0 },
    { 0x3a8c15da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc15dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc15e0, 34 },
    { 0, 0 },
    { 0x3e9c15e2, 95 },
    { 0, 0 },
    { 0xd37c15e4, 46 },
    { 0x92cc15e5, 98 },
    { 0, 0 },
    { 0xfafc15e7, 87 },
    { 0xe2bc15e8, 23 },
    { 0x49ac15e9, 55 },
    { 0x693c15ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c15ed, 93 },
    { 0x67bc15ee, 66 },
    { 0x6b2c15ee, 43 },
    { 0x371c15ed, 61 },
    { 0, 0 },
    { 0xeeac15f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57c15f5, 61 },
    { 0x79ec15f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c1600, 90 },
    { 0x9d5c1600, 100 },
    { 0x75ac1602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfc1605, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc160f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c1613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bc1617, 69 },
    { 0x49cc1618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ec161b, 98 },
    { 0xd8fc161b, 33 },
    { 0x905c161d, 33 },
    { 0, 0 },
    { 0xb73c161f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc1622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c162a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c1630, 61 },
    { 0x91c1631, 34 },
    { 0, 0 },
    { 0x11cc1633, 90 },
    { 0x9f4c1633, 23 },
    { 0x3c4c1633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac163b, 81 },
    { 0, 0 },
    { 0x4a4c163d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03c1641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c1644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9c1648, 90 },
    { 0xd30c1649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc164c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4c164f, 20 },
    { 0xf78c1650, 23 },
    { 0xef0c1651, 78 },
    { 0x829c1651, 99 },
    { 0x8ac1651, 95 },
    { 0xc79c1650, 93 },
    { 0x69bc1651, 90 },
    { 0xfd1c1656, 99 },
    { 0x29fc1657, 80 },
    { 0, 0 },
    { 0x499c1659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dc1660, 67 },
    { 0x9fac1661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcc1665, 95 },
    { 0x385c1666, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751c166a, 95 },
    { 0, 0 },
    { 0x677c166c, 93 },
    { 0xca8c166c, 90 },
    { 0, 0 },
    { 0x2c8c166f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c1672, 93 },
    { 0x100c1673, 90 },
    { 0x4b0c1674, 90 },
    { 0x4c3c1674, 76 },
    { 0x242c1675, 33 },
    { 0xd92c1675, 55 },
    { 0x2c3c1673, 34 },
    { 0, 0 },
    { 0xcc8c167a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756c1683, 90 },
    { 0, 0 },
    { 0x5bdc1685, 95 },
    { 0xe91c1685, 77 },
    { 0, 0 },
    { 0x11ec1688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100c1693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fc1699, 34 },
    { 0, 0 },
    { 0x3f8c169b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac16a0, 98 },
    { 0xd25c16a1, 67 },
    { 0x96c16a0, 90 },
    { 0, 0 },
    { 0x1dcc16a4, 61 },
    { 0, 0 },
    { 0x2adc16a6, 55 },
    { 0x7d0c16a7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c16ab, 90 },
    { 0x9c7c16ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc16b1, 55 },
    { 0x2a6c16b2, 61 },
    { 0x26bc16b3, 97 },
    { 0xc7fc16b4, 76 },
    { 0x62ac16b3, 33 },
    { 0x7b1c16b6, 90 },
    { 0xfb4c16b1, 61 },
    { 0, 0 },
    { 0x280c16b9, 61 },
    { 0x76cc16ba, 43 },
    { 0x11ac16ba, 98 },
    { 0x9cfc16bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc16c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2c16c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7c16d6, 33 },
    { 0xb8ec16d7, 33 },
    { 0, 0 },
    { 0x485c16d9, 99 },
    { 0, 0 },
    { 0x4c8c16db, 95 },
    { 0x9e4c16dc, 78 },
    { 0x7f5c16db, 33 },
    { 0x8f3c16dc, 17 },
    { 0, 0 },
    { 0xd49c16e0, 61 },
    { 0xbfac16e1, 3 },
    { 0, 0 },
    { 0x510c16e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bc16e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x396c16eb, 33 },
    { 0xf39c16eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9c16ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac16f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac16f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fc1700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdc1706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bc1710, 99 },
    { 0xc58c1711, 76 },
    { 0x543c1711, 33 },
    { 0, 0 },
    { 0x916c1714, 85 },
    { 0x67fc1715, 66 },
    { 0, 0 },
    { 0x918c1717, 97 },
    { 0x720c1718, 67 },
    { 0x31bc1719, 43 },
    { 0xc92c171a, 61 },
    { 0x86cc171b, 61 },
    { 0x19ec171c, 68 },
    { 0x80bc171d, 90 },
    { 0xaec171e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c1723, 33 },
    { 0xe6c1724, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655c172a, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83c172d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347c1733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ec1736, 90 },
    { 0xc92c1736, 76 },
    { 0, 0 },
    { 0xfa9c1739, 95 },
    { 0x8f0c173a, 90 },
    { 0, 0 },
    { 0x79dc173c, 33 },
    { 0x346c173d, 90 },
    { 0, 0 },
    { 0xaddc173f, 43 },
    { 0xa10c173f, 43 },
    { 0x332c173f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37c1744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec174c, 87 },
    { 0xeb5c174d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x296c1750, 46 },
    { 0, 0 },
    { 0x2b7c1752, 98 },
    { 0xb44c1753, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcc1759, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4c175c, 61 },
    { 0x662c175d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6c1766, 90 },
    { 0xcfdc1767, 61 },
    { 0xc7ec1768, 52 },
    { 0xd6c1768, 90 },
    { 0xd01c176a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35c176d, 85 },
    { 0xab9c176d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41c1771, 83 },
    { 0x84c1771, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac1777, 34 },
    { 0x6b3c1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfc177f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599c1783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c1786, 95 },
    { 0x3f7c1786, 33 },
    { 0, 0 },
    { 0x1a6c1789, 87 },
    { 0x6ec178a, 55 },
    { 0, 0 },
    { 0x901c178c, 78 },
    { 0, 0 },
    { 0x994c178e, 61 },
    { 0, 0 },
    { 0x8a4c1790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c1793, 98 },
    { 0xc52c1793, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7c1798, 23 },
    { 0x863c1799, 99 },
    { 0x417c179a, 80 },
    { 0xa00c179b, 33 },
    { 0x525c179c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0c179f, 95 },
    { 0x374c17a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c17a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc17a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583c17b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc17b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96c17ba, 78 },
    { 0xb8dc17bb, 46 },
    { 0, 0 },
    { 0x699c17bd, 34 },
    { 0, 0 },
    { 0xb58c17bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc17c2, 88 },
    { 0xc7c17c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc17c8, 76 },
    { 0x5ec17c8, 76 },
    { 0xe03c17ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176c17ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c17d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c17d8, 23 },
    { 0xefec17d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c17dd, 55 },
    { 0xc99c17dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686c17e4, 33 },
    { 0x2b2c17e4, 98 },
    { 0x183c17e6, 43 },
    { 0x67ac17e7, 23 },
    { 0x5d1c17e8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36c17ee, 33 },
    { 0x4ac17ef, 66 },
    { 0xe04c17f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbac17f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533c17fd, 61 },
    { 0, 0 },
    { 0xf33c17ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x607c1802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fc1806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210c180d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaac181a, 17 },
    { 0xf7bc181a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c1820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafc1829, 33 },
    { 0, 0 },
    { 0xe60c182b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9c1830, 95 },
    { 0x91ac1831, 61 },
    { 0, 0 },
    { 0x37fc1833, 77 },
    { 0x33bc1834, 90 },
    { 0x645c1833, 76 },
    { 0, 0 },
    { 0x450c1837, 55 },
    { 0, 0 },
    { 0x1e8c1839, 90 },
    { 0, 0 },
    { 0x23cc183b, 95 },
    { 0x1e9c183b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c183f, 95 },
    { 0xc4ac183f, 33 },
    { 0x1d0c183f, 33 },
    { 0x810c1842, 93 },
    { 0x92ac1843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c184c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c1850, 52 },
    { 0x850c1851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c1854, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c1858, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161c185d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c1869, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30c186e, 43 },
    { 0xac6c186f, 55 },
    { 0xebbc1870, 77 },
    { 0, 0 },
    { 0x90c1872, 95 },
    { 0x921c1873, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefc1879, 77 },
    { 0x6b6c1879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4c187f, 46 },
    { 0x1a8c187f, 43 },
    { 0x3ec1881, 98 },
    { 0x9f7c187f, 95 },
    { 0xa94c1883, 90 },
    { 0, 0 },
    { 0xceec1885, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126c188a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc1891, 66 },
    { 0xc06c1892, 52 },
    { 0xf75c1891, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5c1896, 87 },
    { 0, 0 },
    { 0x219c1898, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dc189b, 73 },
    { 0x4d2c189c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c18a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c18aa, 69 },
    { 0xcbfc18aa, 76 },
    { 0x855c18ac, 84 },
    { 0x3b7c18ad, 68 },
    { 0x6b8c18ae, 95 },
    { 0xb7bc18ae, 23 },
    { 0x447c18af, 61 },
    { 0xb42c18ae, 23 },
    { 0x2afc18b1, 55 },
    { 0x3dec18b3, 69 },
    { 0, 0 },
    { 0x8a8c18b5, 66 },
    { 0xfb6c18b6, 43 },
    { 0x59cc18b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c18bc, 61 },
    { 0xd21c18bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0c18c0, 78 },
    { 0xa44c18c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c18c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dc18ca, 81 },
    { 0xdb2c18cb, 34 },
    { 0x183c18cb, 61 },
    { 0x87fc18cc, 61 },
    { 0x3d1c18ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc18d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc18d7, 90 },
    { 0x391c18d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5c18db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fc18e1, 61 },
    { 0, 0 },
    { 0x8b6c18e3, 76 },
    { 0x199c18e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52c18e7, 90 },
    { 0xfb2c18e7, 69 },
    { 0x989c18e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c18f0, 33 },
    { 0x90bc18f1, 98 },
    { 0, 0 },
    { 0x977c18f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163c18fa, 90 },
    { 0x613c18fb, 92 },
    { 0x7b7c18fc, 85 },
    { 0, 0 },
    { 0x671c18fe, 90 },
    { 0x300c18fe, 33 },
    { 0xe5ac1900, 95 },
    { 0xd8ac18ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421c1909, 34 },
    { 0x450c190a, 61 },
    { 0xe74c1909, 95 },
    { 0, 0 },
    { 0xfcec190d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61c1912, 33 },
    { 0, 0 },
    { 0xf06c1914, 17 },
    { 0x768c1915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1918, 87 },
    { 0x739c1919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc191f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc1925, 33 },
    { 0xc4c1925, 90 },
    { 0, 0 },
    { 0xc56c1928, 100 },
    { 0xc7bc1928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c192c, 95 },
    { 0, 0 },
    { 0x134c192e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c1931, 96 },
    { 0xe07c1931, 23 },
    { 0x850c1933, 61 },
    { 0, 0 },
    { 0x770c1935, 33 },
    { 0x7d1c1936, 76 },
    { 0x2d3c1937, 90 },
    { 0, 0 },
    { 0x15ec1939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x859c193c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c1944, 95 },
    { 0, 0 },
    { 0x48ac1946, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0c194a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c194d, 95 },
    { 0, 0 },
    { 0x5e5c194f, 33 },
    { 0xb1c1950, 93 },
    { 0xf90c194f, 33 },
    { 0x645c194f, 33 },
    { 0x561c1953, 17 },
    { 0xac8c1953, 61 },
    { 0xd9ec1955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0c1958, 33 },
    { 0xaedc1958, 98 },
    { 0x134c1958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c195e, 61 },
    { 0x272c195e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c1962, 17 },
    { 0x51c1962, 33 },
    { 0, 0 },
    { 0x81bc1965, 17 },
    { 0, 0 },
    { 0x517c1967, 52 },
    { 0x12ec1968, 95 },
    { 0x98dc1968, 43 },
    { 0x9e5c196a, 90 },
    { 0x97dc1968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c197b, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5c197e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x923c1981, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185c1985, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958c198c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac1990, 67 },
    { 0, 0 },
    { 0xe3cc1992, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0c1995, 55 },
    { 0xda0c1996, 33 },
    { 0x27cc1996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c199e, 77 },
    { 0, 0 },
    { 0x162c19a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdc19a6, 90 },
    { 0xbe3c19a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c19b0, 81 },
    { 0, 0 },
    { 0x5b4c19b2, 90 },
    { 0, 0 },
    { 0x2b5c19b4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ac19b8, 55 },
    { 0xd54c19b9, 85 },
    { 0x259c19b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc19bf, 46 },
    { 0x43ec19c0, 91 },
    { 0, 0 },
    { 0xfc7c19c2, 84 },
    { 0, 0 },
    { 0x445c19c4, 76 },
    { 0, 0 },
    { 0x71dc19c6, 81 },
    { 0x81cc19c7, 90 },
    { 0, 0 },
    { 0xbbfc19c9, 61 },
    { 0, 0 },
    { 0xdc2c19cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c19d4, 52 },
    { 0x3ebc19d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bc19da, 99 },
    { 0, 0 },
    { 0x825c19dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bc19df, 34 },
    { 0x4b8c19df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c19e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c19e9, 61 },
    { 0x2b0c19ea, 33 },
    { 0, 0 },
    { 0x735c19ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7c19f4, 90 },
    { 0x574c19f4, 43 },
    { 0x5ffc19f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dc19fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fc1a00, 95 },
    { 0x188c1a01, 81 },
    { 0xc9dc1a02, 90 },
    { 0xc53c1a02, 95 },
    { 0xc93c1a00, 99 },
    { 0xc1ec1a05, 35 },
    { 0xae4c1a01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c1a09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0c1a0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c1a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dc1a16, 69 },
    { 0xaaac1a17, 43 },
    { 0x872c1a17, 95 },
    { 0, 0 },
    { 0x47dc1a1a, 23 },
    { 0, 0 },
    { 0x229c1a1c, 61 },
    { 0x506c1a1d, 90 },
    { 0xe9fc1a1e, 66 },
    { 0x2a5c1a1f, 93 },
    { 0x78cc1a1f, 33 },
    { 0xfc1a1d, 38 },
    { 0x32c1a21, 23 },
    { 0xe8c1a23, 61 },
    { 0, 0 },
    { 0x9b2c1a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c1a2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc1a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc1a32, 33 },
    { 0, 0 },
    { 0x30ec1a34, 4 },
    { 0, 0 },
    { 0xf01c1a36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecc1a3c, 48 },
    { 0x167c1a3d, 23 },
    { 0x783c1a3c, 90 },
    { 0, 0 },
    { 0xf38c1a40, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c1a43, 84 },
    { 0x6fbc1a44, 23 },
    { 0xc80c1a44, 78 },
    { 0x9efc1a45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fc1a54, 61 },
    { 0x389c1a54, 34 },
    { 0xdb2c1a55, 93 },
    { 0xb24c1a57, 98 },
    { 0x5a6c1a56, 55 },
    { 0x6fbc1a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fc1a5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c1a69, 46 },
    { 0xc7ec1a69, 61 },
    { 0xd39c1a6a, 61 },
    { 0xce7c1a6c, 46 },
    { 0x33dc1a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ac1a75, 100 },
    { 0xa7ec1a76, 40 },
    { 0xaac1a76, 95 },
    { 0x27ec1a78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ac1a7d, 90 },
    { 0x9bfc1a7d, 40 },
    { 0x821c1a7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacec1a85, 52 },
    { 0, 0 },
    { 0xe64c1a87, 61 },
    { 0x3fc1a88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dc1a8b, 33 },
    { 0x980c1a8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81c1a8f, 98 },
    { 0, 0 },
    { 0xd99c1a91, 90 },
    { 0, 0 },
    { 0x358c1a93, 55 },
    { 0, 0 },
    { 0xc77c1a95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76c1a99, 46 },
    { 0xfc7c1a9a, 90 },
    { 0, 0 },
    { 0x9a0c1a9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c1a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6c1aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1c1aa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c1aa9, 34 },
    { 0x280c1aaa, 23 },
    { 0x643c1aa9, 34 },
    { 0xae6c1aa9, 99 },
    { 0xd46c1aad, 90 },
    { 0xc1cc1aad, 61 },
    { 0xf2fc1aad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50c1ab3, 66 },
    { 0x391c1ab4, 61 },
    { 0xc43c1ab5, 91 },
    { 0x828c1ab3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359c1abb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec1abe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c1ac3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1ac6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7c1ada, 36 },
    { 0x58c1adb, 95 },
    { 0xddec1adc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c1adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc1ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8c1ae6, 55 },
    { 0xe39c1ae7, 40 },
    { 0xe96c1ae6, 90 },
    { 0, 0 },
    { 0xa3cc1aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc1aed, 90 },
    { 0x825c1aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23c1af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c1af7, 61 },
    { 0xbf8c1af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc1afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c1b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ec1b08, 90 },
    { 0x445c1b09, 91 },
    { 0x7e8c1b0a, 92 },
    { 0x89c1b0b, 87 },
    { 0x64fc1b0b, 90 },
    { 0, 0 },
    { 0x977c1b0e, 76 },
    { 0x2edc1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c1b16, 66 },
    { 0xe11c1b16, 98 },
    { 0x31bc1b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cc1b1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc1b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adc1b29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c1b3b, 43 },
    { 0, 0 },
    { 0x11cc1b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cc1b40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31c1b46, 43 },
    { 0x62bc1b46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0c1b4b, 34 },
    { 0x60ec1b4c, 61 },
    { 0, 0 },
    { 0x50c1b4e, 97 },
    { 0, 0 },
    { 0xeb0c1b50, 33 },
    { 0, 0 },
    { 0xf4bc1b52, 61 },
    { 0x82fc1b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfc1b58, 98 },
    { 0x676c1b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2c1b5c, 34 },
    { 0, 0 },
    { 0x322c1b5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c1b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908c1b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bc1b6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc1b70, 98 },
    { 0, 0 },
    { 0x78ec1b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c1b75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x741c1b78, 33 },
    { 0, 0 },
    { 0x487c1b7a, 98 },
    { 0, 0 },
    { 0x556c1b7c, 34 },
    { 0, 0 },
    { 0x103c1b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bc1b85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c1b88, 52 },
    { 0x348c1b88, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979c1b8f, 95 },
    { 0xc3fc1b90, 90 },
    { 0xc7cc1b90, 84 },
    { 0, 0 },
    { 0x913c1b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c1b97, 95 },
    { 0x24cc1b98, 33 },
    { 0xb4ac1b98, 61 },
    { 0x53ac1b98, 100 },
    { 0xe1ac1b9b, 55 },
    { 0x143c1b99, 61 },
    { 0xdfec1b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cc1ba0, 85 },
    { 0xec7c1ba1, 90 },
    { 0xd8cc1ba2, 76 },
    { 0x19ac1ba3, 91 },
    { 0xc9ec1ba2, 98 },
    { 0xe86c1ba2, 76 },
    { 0, 0 },
    { 0x22ec1ba7, 66 },
    { 0, 0 },
    { 0xd5dc1ba9, 100 },
    { 0x9f2c1baa, 90 },
    { 0xfc8c1bab, 95 },
    { 0xdfbc1bab, 33 },
    { 0xe2fc1bac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafc1bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cc1bb4, 66 },
    { 0, 0 },
    { 0x7f3c1bb6, 61 },
    { 0x165c1bb7, 43 },
    { 0x3b6c1bb8, 23 },
    { 0xce2c1bb8, 98 },
    { 0xd1cc1bb6, 90 },
    { 0xc2fc1bbb, 95 },
    { 0, 0 },
    { 0xa6cc1bbd, 88 },
    { 0x62dc1bbe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c1bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac1bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bc1bd2, 90 },
    { 0, 0 },
    { 0xd3c1bd4, 90 },
    { 0x654c1bd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c1bdf, 23 },
    { 0x32fc1bdf, 80 },
    { 0, 0 },
    { 0x5cac1be2, 90 },
    { 0x53fc1be3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5c1be6, 44 },
    { 0x79fc1be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6c1bf1, 55 },
    { 0, 0 },
    { 0x43bc1bf3, 67 },
    { 0, 0 },
    { 0x4ddc1bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec1bf8, 76 },
    { 0xdeac1bf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc1c00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c1c03, 85 },
    { 0, 0 },
    { 0x2dec1c05, 95 },
    { 0x1edc1c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dc1c09, 52 },
    { 0, 0 },
    { 0x491c1c0b, 46 },
    { 0, 0 },
    { 0x399c1c0d, 99 },
    { 0, 0 },
    { 0xf1bc1c0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55c1c12, 90 },
    { 0xb87c1c12, 33 },
    { 0x891c1c14, 56 },
    { 0x83cc1c15, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8c1c18, 43 },
    { 0x4a0c1c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dc1c1d, 55 },
    { 0x418c1c1e, 98 },
    { 0, 0 },
    { 0xf03c1c20, 23 },
    { 0xf72c1c20, 90 },
    { 0x3b8c1c20, 76 },
    { 0, 0 },
    { 0xa1ec1c24, 98 },
    { 0, 0 },
    { 0x562c1c26, 75 },
    { 0, 0 },
    { 0xfb7c1c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfc1c2e, 23 },
    { 0xddac1c2f, 98 },
    { 0, 0 },
    { 0x65dc1c31, 90 },
    { 0x11c1c32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc1c35, 43 },
    { 0xa10c1c36, 33 },
    { 0, 0 },
    { 0xa53c1c38, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fc1c3b, 95 },
    { 0, 0 },
    { 0x210c1c3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ac1c40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdac1c43, 34 },
    { 0x584c1c43, 100 },
    { 0x69bc1c45, 91 },
    { 0x400c1c46, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bc1c4b, 90 },
    { 0x51c1c4b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8c1c50, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffc1c56, 38 },
    { 0, 0 },
    { 0x673c1c58, 43 },
    { 0x959c1c58, 61 },
    { 0, 0 },
    { 0x468c1c5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708c1c5f, 90 },
    { 0xbe0c1c60, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c1c64, 23 },
    { 0, 0 },
    { 0xac2c1c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9c1c6a, 33 },
    { 0, 0 },
    { 0x6cec1c6c, 90 },
    { 0xb83c1c6d, 78 },
    { 0xeafc1c6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c1c73, 23 },
    { 0x178c1c73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bc1c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac1c7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdc1c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c1c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89c1c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac1c92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec1c95, 61 },
    { 0xd19c1c96, 34 },
    { 0x21dc1c96, 33 },
    { 0xf38c1c95, 43 },
    { 0x74bc1c95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c1c9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x584c1ca0, 61 },
    { 0, 0 },
    { 0x3cc1ca2, 90 },
    { 0, 0 },
    { 0x898c1ca4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292c1ca8, 93 },
    { 0xfc5c1ca9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c1cad, 67 },
    { 0x292c1cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5c1cb4, 96 },
    { 0xd3fc1cb4, 87 },
    { 0x21ac1cb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cc1cba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc1cc2, 33 },
    { 0xbf2c1cc2, 61 },
    { 0, 0 },
    { 0xa4cc1cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c1cc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fc1ccf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bc1cd2, 23 },
    { 0xdcdc1cd3, 66 },
    { 0, 0 },
    { 0x2f8c1cd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bc1cda, 99 },
    { 0x133c1cda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4c1cdf, 76 },
    { 0xe6cc1cdf, 99 },
    { 0, 0 },
    { 0xe69c1ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c1ce6, 53 },
    { 0x8fac1ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c1ceb, 85 },
    { 0xc3dc1cec, 98 },
    { 0xe1dc1cec, 43 },
    { 0x6c2c1cec, 90 },
    { 0xd5ec1cef, 33 },
    { 0x8abc1cef, 52 },
    { 0xee5c1cf1, 46 },
    { 0xd78c1cf2, 65 },
    { 0x7a2c1cee, 33 },
    { 0xf13c1cf4, 98 },
    { 0xda2c1cee, 90 },
    { 0x4b1c1cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec1cf9, 55 },
    { 0x228c1cfa, 95 },
    { 0x1a8c1cfa, 46 },
    { 0, 0 },
    { 0xfa9c1cfd, 98 },
    { 0x38c1cfe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62c1d02, 33 },
    { 0xf4dc1d02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c1d0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cc1d0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c1d17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec1d1d, 76 },
    { 0, 0 },
    { 0x9eec1d1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c1d22, 92 },
    { 0xcc1d23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x449c1d26, 95 },
    { 0xa02c1d27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c1d2b, 69 },
    { 0xebec1d2c, 87 },
    { 0, 0 },
    { 0xf36c1d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38c1d32, 98 },
    { 0, 0 },
    { 0x39cc1d34, 95 },
    { 0xac9c1d34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec1d3e, 46 },
    { 0, 0 },
    { 0x285c1d40, 61 },
    { 0, 0 },
    { 0xc72c1d42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c1d46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ec1d4a, 23 },
    { 0xeafc1d4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6c1d4e, 61 },
    { 0xd84c1d4f, 84 },
    { 0, 0 },
    { 0xfcbc1d51, 33 },
    { 0xae7c1d52, 65 },
    { 0x949c1d53, 99 },
    { 0x91ac1d51, 95 },
    { 0xacac1d55, 95 },
    { 0xc27c1d56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c1d5b, 90 },
    { 0, 0 },
    { 0x64dc1d5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c1d61, 46 },
    { 0xbfec1d62, 98 },
    { 0x9dc1d63, 33 },
    { 0x2e8c1d64, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c1d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dc1d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdc1d71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x788c1d74, 90 },
    { 0x588c1d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c1d7a, 40 },
    { 0, 0 },
    { 0x197c1d7c, 33 },
    { 0x45c1d7d, 76 },
    { 0xa2cc1d7e, 98 },
    { 0, 0 },
    { 0x17ac1d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc1d83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c1d87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x165c1d8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dc1d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c1d95, 34 },
    { 0, 0 },
    { 0x843c1d97, 90 },
    { 0, 0 },
    { 0x7ddc1d99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc1d9d, 33 },
    { 0, 0 },
    { 0x3fdc1d9f, 95 },
    { 0, 0 },
    { 0x17ec1da1, 61 },
    { 0xfd8c1da1, 52 },
    { 0, 0 },
    { 0x4a0c1da4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c1da7, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c1daa, 23 },
    { 0x5c6c1dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8c1db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c1db8, 95 },
    { 0xca4c1db9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5c1dc0, 33 },
    { 0x73ac1dc0, 55 },
    { 0x7bdc1dc2, 67 },
    { 0x6c8c1dc1, 65 },
    { 0x9acc1dc3, 78 },
    { 0, 0 },
    { 0x1e1c1dc6, 99 },
    { 0x6c3c1dc6, 55 },
    { 0x514c1dc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c1dcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c1dd0, 55 },
    { 0x14c1dd1, 61 },
    { 0, 0 },
    { 0xef2c1dd3, 98 },
    { 0, 0 },
    { 0x1bbc1dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0c1dd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509c1ddc, 76 },
    { 0xfa9c1ddc, 61 },
    { 0xeecc1dde, 33 },
    { 0, 0 },
    { 0xbecc1de0, 98 },
    { 0xbf7c1de1, 61 },
    { 0xcbdc1de1, 98 },
    { 0xd0ac1de1, 8 },
    { 0, 0 },
    { 0x4d8c1de5, 33 },
    { 0x59fc1de5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05c1dea, 61 },
    { 0, 0 },
    { 0x467c1dec, 33 },
    { 0x2d3c1dec, 59 },
    { 0xacec1dee, 99 },
    { 0xccec1def, 98 },
    { 0xb9fc1def, 98 },
    { 0xa43c1dec, 95 },
    { 0, 0 },
    { 0x8bcc1df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ec1dfc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c1e00, 98 },
    { 0, 0 },
    { 0x125c1e02, 66 },
    { 0, 0 },
    { 0xcd5c1e04, 55 },
    { 0x5e8c1e05, 51 },
    { 0x588c1e06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3c1e0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fc1e14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c1e1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac1e1d, 95 },
    { 0, 0 },
    { 0x6c0c1e1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cc1e22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5c1e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c1e34, 90 },
    { 0xfb8c1e35, 61 },
    { 0x5d2c1e36, 93 },
    { 0xc7c1e37, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731c1e3e, 33 },
    { 0xdccc1e3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952c1e44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc1e4c, 61 },
    { 0x4f3c1e4c, 90 },
    { 0xbc4c1e4c, 90 },
    { 0, 0 },
    { 0xdcdc1e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9c1e56, 90 },
    { 0, 0 },
    { 0x81bc1e58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c1e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5c1e60, 61 },
    { 0x2dfc1e60, 98 },
    { 0, 0 },
    { 0x5f1c1e63, 90 },
    { 0x3a1c1e64, 33 },
    { 0, 0 },
    { 0xd23c1e66, 95 },
    { 0, 0 },
    { 0x65fc1e68, 23 },
    { 0x832c1e68, 46 },
    { 0xa33c1e6a, 78 },
    { 0xf48c1e68, 76 },
    { 0xaf6c1e68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x197c1e6f, 33 },
    { 0xa3cc1e6f, 23 },
    { 0x2d3c1e71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bc1e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c1e7d, 98 },
    { 0xf8fc1e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ac1e82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5c1e85, 61 },
    { 0x300c1e86, 83 },
    { 0, 0 },
    { 0x8bfc1e88, 33 },
    { 0x85cc1e89, 33 },
    { 0x5f2c1e88, 84 },
    { 0, 0 },
    { 0xca5c1e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ec1e98, 91 },
    { 0x1dac1e99, 98 },
    { 0xab3c1e9a, 55 },
    { 0x4d9c1e9b, 17 },
    { 0x5c4c1e9b, 95 },
    { 0, 0 },
    { 0xe26c1e9e, 68 },
    { 0, 0 },
    { 0x285c1ea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6c1ea5, 33 },
    { 0, 0 },
    { 0xf21c1ea7, 23 },
    { 0x6b3c1ea8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6c1eab, 90 },
    { 0xf8ec1eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfc1eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc1eb4, 52 },
    { 0x8c9c1eb4, 61 },
    { 0x70c1eb4, 76 },
    { 0xe4cc1eb7, 67 },
    { 0x365c1eb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21c1ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfc1ec6, 76 },
    { 0x12fc1ec7, 34 },
    { 0xd3bc1ec7, 17 },
    { 0xe36c1ec9, 76 },
    { 0, 0 },
    { 0x50fc1ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4c1ecf, 94 },
    { 0x1e9c1ecf, 40 },
    { 0x552c1ed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404c1ed5, 61 },
    { 0x2ac1ed6, 99 },
    { 0, 0 },
    { 0xf8cc1ed8, 90 },
    { 0, 0 },
    { 0xbcc1eda, 98 },
    { 0, 0 },
    { 0xc1fc1edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cc1ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c1ee6, 98 },
    { 0x63c1ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c1eea, 84 },
    { 0x938c1eeb, 91 },
    { 0xa3dc1eeb, 61 },
    { 0x715c1eed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c1ef9, 34 },
    { 0x7cec1efa, 55 },
    { 0, 0 },
    { 0x34cc1efc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6c1eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c1f03, 95 },
    { 0x3fbc1f03, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c1f08, 91 },
    { 0x9bec1f09, 61 },
    { 0x450c1f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1f11, 75 },
    { 0x6cfc1f11, 87 },
    { 0xeaac1f11, 46 },
    { 0x2c3c1f13, 33 },
    { 0, 0 },
    { 0x135c1f16, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdc1f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c1f20, 93 },
    { 0xdebc1f21, 82 },
    { 0xf49c1f21, 90 },
    { 0xb68c1f22, 76 },
    { 0x54dc1f24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c1f2a, 33 },
    { 0xf51c1f2b, 33 },
    { 0x3eac1f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c1f2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c1f34, 34 },
    { 0x801c1f34, 43 },
    { 0x52dc1f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dc1f39, 99 },
    { 0, 0 },
    { 0xe7c1f3b, 93 },
    { 0, 0 },
    { 0x87dc1f3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ec1f41, 55 },
    { 0x615c1f42, 65 },
    { 0, 0 },
    { 0x40bc1f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c1f47, 98 },
    { 0xa70c1f47, 33 },
    { 0, 0 },
    { 0xd7ac1f4a, 76 },
    { 0, 0 },
    { 0x776c1f4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79c1f52, 23 },
    { 0xb4c1f53, 36 },
    { 0x56c1f52, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dec1f58, 95 },
    { 0xab8c1f59, 55 },
    { 0x77c1f5a, 34 },
    { 0x982c1f5b, 34 },
    { 0xe06c1f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fc1f64, 61 },
    { 0, 0 },
    { 0xd14c1f66, 98 },
    { 0xb1dc1f67, 78 },
    { 0x782c1f67, 95 },
    { 0x235c1f68, 85 },
    { 0x1bbc1f66, 95 },
    { 0x952c1f6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c1f6e, 34 },
    { 0xca3c1f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c1f73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0c1f7d, 95 },
    { 0xeccc1f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfc1f82, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc1f86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c1f8a, 34 },
    { 0, 0 },
    { 0x4c8c1f8c, 92 },
    { 0x8a5c1f8c, 98 },
    { 0x965c1f8e, 33 },
    { 0x9e8c1f8e, 33 },
    { 0x37c1f8e, 95 },
    { 0, 0 },
    { 0x78fc1f92, 61 },
    { 0x25dc1f93, 95 },
    { 0x66ac1f93, 90 },
    { 0x81bc1f95, 90 },
    { 0x7eec1f96, 46 },
    { 0x297c1f94, 22 },
    { 0x501c1f97, 33 },
    { 0xf9ac1f99, 34 },
    { 0xfd3c1f9a, 78 },
    { 0x118c1f95, 75 },
    { 0x88fc1f9c, 90 },
    { 0xc31c1f9c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ec1fa0, 76 },
    { 0, 0 },
    { 0x68c1fa2, 95 },
    { 0xf88c1fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c1fab, 98 },
    { 0x20bc1fac, 46 },
    { 0, 0 },
    { 0xea1c1fae, 87 },
    { 0xaf7c1faf, 81 },
    { 0, 0 },
    { 0xc37c1fb1, 34 },
    { 0xfeac1fb2, 61 },
    { 0xc97c1fb2, 78 },
    { 0x91fc1fb3, 76 },
    { 0x8afc1fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c1fb8, 48 },
    { 0xc59c1fb9, 99 },
    { 0x18c1fba, 90 },
    { 0, 0 },
    { 0xb60c1fbc, 65 },
    { 0, 0 },
    { 0x188c1fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ac1fc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bc1fc7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fec1fce, 75 },
    { 0, 0 },
    { 0xa6ec1fd0, 61 },
    { 0, 0 },
    { 0xf0dc1fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2c1fd5, 98 },
    { 0, 0 },
    { 0xd2bc1fd7, 23 },
    { 0x63bc1fd8, 61 },
    { 0xf1cc1fd8, 90 },
    { 0x27c1fda, 97 },
    { 0, 0 },
    { 0x4aac1fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cc1fe2, 52 },
    { 0x20fc1fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c1fe6, 95 },
    { 0x2dec1fe7, 90 },
    { 0x156c1fe7, 46 },
    { 0, 0 },
    { 0x12bc1fea, 96 },
    { 0xb3ac1feb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ac1fef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dc1ff2, 91 },
    { 0x3cdc1ff3, 46 },
    { 0xdf5c1ff4, 61 },
    { 0xfbec1ff3, 90 },
    { 0xb01c1ff6, 90 },
    { 0x5ccc1ff7, 76 },
    { 0, 0 },
    { 0xebdc1ff9, 90 },
    { 0xee6c1ffa, 46 },
    { 0x2d5c1ffb, 90 },
    { 0, 0 },
    { 0x99ac1ffd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c2004, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cc200c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffc2012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81c2015, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8c2023, 23 },
    { 0xa84c2024, 90 },
    { 0x420c2024, 46 },
    { 0x2c1c2025, 98 },
    { 0, 0 },
    { 0xc90c2028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c202c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fac2030, 78 },
    { 0x7f8c2031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c2036, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65c203a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc203d, 23 },
    { 0xe79c203e, 96 },
    { 0x4f3c203f, 61 },
    { 0x264c203d, 90 },
    { 0x866c2040, 65 },
    { 0, 0 },
    { 0x578c2043, 61 },
    { 0xc7ec2044, 33 },
    { 0, 0 },
    { 0xe0c2046, 93 },
    { 0, 0 },
    { 0x7f3c2048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec204e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c2053, 23 },
    { 0xa85c2053, 61 },
    { 0x874c2055, 64 },
    { 0, 0 },
    { 0xdcfc2057, 98 },
    { 0, 0 },
    { 0xc43c2059, 93 },
    { 0xdefc205a, 99 },
    { 0, 0 },
    { 0x503c205c, 98 },
    { 0xf65c205d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafc2066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dc2069, 90 },
    { 0, 0 },
    { 0x42dc206b, 90 },
    { 0, 0 },
    { 0x5f7c206d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c2072, 99 },
    { 0x767c2072, 90 },
    { 0x561c2074, 33 },
    { 0, 0 },
    { 0xa9ec2076, 66 },
    { 0, 0 },
    { 0x51dc2078, 84 },
    { 0x64cc2079, 100 },
    { 0x33c207a, 52 },
    { 0x773c207b, 90 },
    { 0xf4cc207c, 61 },
    { 0x443c207c, 85 },
    { 0xcd9c207d, 64 },
    { 0x853c207f, 33 },
    { 0x836c207f, 95 },
    { 0x9e8c2081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7c2084, 66 },
    { 0x288c2085, 95 },
    { 0x1c5c2086, 95 },
    { 0xb7c2087, 76 },
    { 0x415c2086, 90 },
    { 0, 0 },
    { 0xce9c208a, 61 },
    { 0xbfbc208b, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fc208f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c2093, 34 },
    { 0x145c2094, 43 },
    { 0x365c2093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c2099, 91 },
    { 0x820c209a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238c20a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc20a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9c20ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac20b2, 97 },
    { 0, 0 },
    { 0xc67c20b4, 61 },
    { 0, 0 },
    { 0x464c20b6, 43 },
    { 0xe3bc20b7, 95 },
    { 0xd49c20b8, 85 },
    { 0x722c20b9, 99 },
    { 0, 0 },
    { 0x210c20bb, 98 },
    { 0x40ec20bc, 91 },
    { 0xecbc20bc, 61 },
    { 0xb91c20bc, 98 },
    { 0xa05c20bd, 90 },
    { 0x185c20c0, 33 },
    { 0, 0 },
    { 0xbe2c20c2, 95 },
    { 0x110c20c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdec20c7, 87 },
    { 0x353c20c7, 76 },
    { 0x304c20c8, 95 },
    { 0, 0 },
    { 0x883c20cb, 84 },
    { 0, 0 },
    { 0x5eec20cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c20d3, 69 },
    { 0, 0 },
    { 0x442c20d5, 23 },
    { 0, 0 },
    { 0x297c20d7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc20da, 70 },
    { 0x8c7c20db, 61 },
    { 0x5fcc20dc, 100 },
    { 0x38ac20dd, 33 },
    { 0x535c20de, 95 },
    { 0x38ec20da, 95 },
    { 0x3efc20dc, 98 },
    { 0x4b8c20de, 98 },
    { 0, 0 },
    { 0x9a7c20e3, 61 },
    { 0x544c20e4, 93 },
    { 0x80dc20e3, 46 },
    { 0, 0 },
    { 0xd2fc20e7, 55 },
    { 0x1b4c20e8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c20f1, 23 },
    { 0, 0 },
    { 0xa29c20f3, 90 },
    { 0xc7ac20f4, 55 },
    { 0, 0 },
    { 0x5f3c20f6, 61 },
    { 0, 0 },
    { 0xdacc20f8, 95 },
    { 0x598c20f9, 61 },
    { 0, 0 },
    { 0xa04c20fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c20ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78c2109, 90 },
    { 0, 0 },
    { 0x40ec210b, 91 },
    { 0x65fc210c, 33 },
    { 0x315c210d, 95 },
    { 0xddc210b, 34 },
    { 0x8bcc210f, 90 },
    { 0xaa9c210f, 69 },
    { 0x172c2111, 33 },
    { 0xa07c210d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc211c, 90 },
    { 0x832c211d, 52 },
    { 0xddbc211d, 98 },
    { 0, 0 },
    { 0x6a1c2120, 90 },
    { 0xbacc2121, 76 },
    { 0xb45c2122, 61 },
    { 0x15ac2123, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246c2127, 61 },
    { 0, 0 },
    { 0x1eec2129, 90 },
    { 0x2a0c212a, 61 },
    { 0x1a7c212b, 43 },
    { 0x504c212b, 95 },
    { 0xfdec212c, 81 },
    { 0xb4bc212e, 100 },
    { 0, 0 },
    { 0x1adc2130, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5c2133, 73 },
    { 0xa8bc2133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207c213a, 87 },
    { 0, 0 },
    { 0x5a8c213c, 98 },
    { 0xcf7c213d, 98 },
    { 0x8ccc213d, 98 },
    { 0, 0 },
    { 0x735c2140, 43 },
    { 0xf69c2140, 95 },
    { 0, 0 },
    { 0x187c2143, 36 },
    { 0, 0 },
    { 0x95c2145, 74 },
    { 0xa35c2145, 90 },
    { 0x34dc2147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7c214b, 51 },
    { 0xd5ac214c, 95 },
    { 0x2ffc214c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5c2152, 78 },
    { 0xc61c2153, 34 },
    { 0xe7ac2153, 90 },
    { 0xf90c2155, 98 },
    { 0x1cac2155, 66 },
    { 0x139c2155, 46 },
    { 0xb0dc2158, 55 },
    { 0, 0 },
    { 0x98c215a, 95 },
    { 0, 0 },
    { 0xb18c215c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dc215f, 33 },
    { 0xb62c2160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cc2163, 55 },
    { 0, 0 },
    { 0xe9cc2165, 91 },
    { 0x61ec2165, 46 },
    { 0xcc7c2167, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c216a, 52 },
    { 0xb94c216b, 23 },
    { 0xfb9c216c, 100 },
    { 0x213c216c, 95 },
    { 0, 0 },
    { 0x1e6c216f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c2176, 87 },
    { 0xd16c2176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cc217c, 23 },
    { 0x146c217d, 61 },
    { 0xad1c217d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c2182, 98 },
    { 0x351c2183, 34 },
    { 0xa75c2184, 76 },
    { 0xceac2185, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c2189, 46 },
    { 0xd2bc218a, 98 },
    { 0xefc218b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302c218f, 23 },
    { 0, 0 },
    { 0x4fec2191, 17 },
    { 0x3dec2192, 33 },
    { 0xae7c2193, 34 },
    { 0xe9dc2192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26c219c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c21a0, 90 },
    { 0x766c21a1, 85 },
    { 0xb5fc21a1, 17 },
    { 0x637c21a3, 90 },
    { 0xf3ec21a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bc21a8, 91 },
    { 0, 0 },
    { 0xd8dc21aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52c21ad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdc21b0, 55 },
    { 0x337c21b1, 33 },
    { 0x43ac21b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7c21b5, 23 },
    { 0, 0 },
    { 0x8e4c21b7, 43 },
    { 0xe6ec21b8, 23 },
    { 0xcffc21b8, 61 },
    { 0x613c21ba, 90 },
    { 0x5e3c21ba, 76 },
    { 0x219c21b8, 33 },
    { 0x967c21bc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c21c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c21c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecc21c8, 100 },
    { 0xa7dc21c9, 61 },
    { 0x940c21c9, 55 },
    { 0xae0c21ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363c21cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc21d3, 61 },
    { 0, 0 },
    { 0x3cec21d5, 93 },
    { 0, 0 },
    { 0x6e7c21d7, 88 },
    { 0x37c21d8, 90 },
    { 0x924c21d8, 87 },
    { 0x1dc21da, 61 },
    { 0xacec21db, 61 },
    { 0xee0c21dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac21e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bc21e6, 23 },
    { 0xbbbc21e7, 90 },
    { 0x6ddc21e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc21ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ac21f3, 61 },
    { 0, 0 },
    { 0x735c21f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c21f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec21fb, 90 },
    { 0xd48c21fc, 61 },
    { 0xebfc21fd, 98 },
    { 0xe31c21fc, 90 },
    { 0xb26c21fe, 43 },
    { 0xdbc2200, 98 },
    { 0x81ec21fc, 91 },
    { 0, 0 },
    { 0x452c2203, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efc2207, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaac220b, 76 },
    { 0x396c220c, 91 },
    { 0xe07c220d, 33 },
    { 0x78fc220d, 23 },
    { 0, 0 },
    { 0x5dc2210, 61 },
    { 0x953c2210, 43 },
    { 0x6e4c2212, 55 },
    { 0, 0 },
    { 0xffcc2214, 52 },
    { 0x418c2215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c221a, 38 },
    { 0, 0 },
    { 0xbaac221c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc2226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c222a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0c222d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ac2230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c2234, 98 },
    { 0x274c2235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc223f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c2242, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cc2252, 61 },
    { 0x170c2253, 61 },
    { 0xd00c2254, 81 },
    { 0xcb4c2252, 75 },
    { 0, 0 },
    { 0x13bc2257, 23 },
    { 0, 0 },
    { 0xbb4c2259, 61 },
    { 0x8d7c225a, 34 },
    { 0x5e4c225b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac225f, 84 },
    { 0xa2c2260, 100 },
    { 0, 0 },
    { 0x9adc2262, 90 },
    { 0, 0 },
    { 0x702c2264, 33 },
    { 0xdfac2265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec226b, 33 },
    { 0xb58c226c, 55 },
    { 0, 0 },
    { 0xba3c226e, 17 },
    { 0x18ec226e, 55 },
    { 0xe53c226f, 76 },
    { 0x2a3c2271, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c2275, 90 },
    { 0, 0 },
    { 0xe52c2277, 61 },
    { 0, 0 },
    { 0x478c2279, 95 },
    { 0x624c2279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50c227f, 98 },
    { 0xaeec2280, 66 },
    { 0x4f1c2281, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac2284, 33 },
    { 0x5bc2284, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x349c2288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c228d, 33 },
    { 0xe32c228e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c2291, 34 },
    { 0xb0bc2292, 99 },
    { 0x9e9c2293, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563c229a, 84 },
    { 0x7e7c229a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7c22a0, 93 },
    { 0x302c22a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54c22a4, 77 },
    { 0, 0 },
    { 0xf07c22a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbc22a9, 94 },
    { 0, 0 },
    { 0x6f3c22ab, 95 },
    { 0, 0 },
    { 0xca9c22ad, 81 },
    { 0, 0 },
    { 0xd28c22af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cc22b3, 74 },
    { 0x4b8c22b4, 67 },
    { 0, 0 },
    { 0x1fec22b6, 90 },
    { 0xc11c22b6, 98 },
    { 0x66bc22b8, 95 },
    { 0xd02c22b9, 34 },
    { 0xbccc22b9, 46 },
    { 0, 0 },
    { 0x8a8c22bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c22bf, 33 },
    { 0x428c22c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c22c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c22ca, 43 },
    { 0, 0 },
    { 0xe9bc22cc, 61 },
    { 0x602c22cc, 98 },
    { 0x206c22cd, 76 },
    { 0x4a5c22cc, 100 },
    { 0x7fbc22cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c22d4, 93 },
    { 0xd1c22d5, 33 },
    { 0, 0 },
    { 0x781c22d7, 90 },
    { 0, 0 },
    { 0x6b8c22d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ec22e2, 33 },
    { 0x1b9c22e2, 12 },
    { 0xa0ac22e2, 55 },
    { 0x8a2c22e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac22e8, 33 },
    { 0, 0 },
    { 0xc0fc22ea, 85 },
    { 0xfa4c22ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821c22f3, 61 },
    { 0, 0 },
    { 0xccdc22f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c22f8, 61 },
    { 0x8c9c22f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac22fe, 22 },
    { 0x70bc22ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974c2304, 95 },
    { 0, 0 },
    { 0x4a3c2306, 44 },
    { 0, 0 },
    { 0x8c9c2308, 98 },
    { 0x76ac2309, 43 },
    { 0, 0 },
    { 0x513c230b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587c2310, 61 },
    { 0xe05c2310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c2314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c2318, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc231f, 68 },
    { 0x978c2320, 23 },
    { 0x3acc2321, 61 },
    { 0xe3bc2320, 99 },
    { 0x798c2320, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8c2328, 65 },
    { 0x6afc2329, 90 },
    { 0, 0 },
    { 0xf8ec232b, 85 },
    { 0x89bc232c, 75 },
    { 0xb8fc232d, 23 },
    { 0x852c232d, 76 },
    { 0xe82c232c, 95 },
    { 0xa91c232e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cc2337, 55 },
    { 0x50ac2338, 91 },
    { 0x2fac2339, 61 },
    { 0x626c233a, 98 },
    { 0, 0 },
    { 0x191c233c, 55 },
    { 0xf44c233d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13c2345, 33 },
    { 0x327c2345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ac234a, 33 },
    { 0, 0 },
    { 0x1a7c234c, 23 },
    { 0x731c234d, 52 },
    { 0xc00c234d, 88 },
    { 0, 0 },
    { 0xe86c2350, 98 },
    { 0xb60c2350, 11 },
    { 0x11bc2352, 46 },
    { 0xe8cc2352, 61 },
    { 0x16cc2352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82c2358, 46 },
    { 0xf04c2359, 67 },
    { 0x560c235a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ac2361, 61 },
    { 0, 0 },
    { 0x6dfc2363, 33 },
    { 0, 0 },
    { 0x352c2365, 34 },
    { 0x16cc2366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acc236a, 95 },
    { 0, 0 },
    { 0x21c236c, 43 },
    { 0x472c236c, 76 },
    { 0, 0 },
    { 0xa46c236f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec2372, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aac2375, 76 },
    { 0xd22c2375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c2379, 33 },
    { 0, 0 },
    { 0x184c237b, 76 },
    { 0x1d1c237c, 95 },
    { 0x7eac237d, 46 },
    { 0xd8fc237c, 95 },
    { 0x85bc237d, 17 },
    { 0xf09c2380, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7c2384, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6c2387, 91 },
    { 0xaf5c2388, 36 },
    { 0xf77c2389, 61 },
    { 0x7e0c2389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc238f, 98 },
    { 0, 0 },
    { 0x214c2391, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c2397, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bac239b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dc239f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c23a2, 36 },
    { 0, 0 },
    { 0x267c23a4, 99 },
    { 0, 0 },
    { 0xdf0c23a6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c23aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dc23af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec23b4, 55 },
    { 0x4f1c23b4, 78 },
    { 0x20dc23b5, 70 },
    { 0xc06c23b4, 33 },
    { 0xc3c23b8, 81 },
    { 0x45dc23b8, 55 },
    { 0x8d5c23ba, 100 },
    { 0xd72c23b7, 95 },
    { 0x7e0c23b7, 93 },
    { 0, 0 },
    { 0x8bcc23be, 34 },
    { 0x3fdc23be, 34 },
    { 0x23ac23be, 93 },
    { 0x4f4c23c1, 33 },
    { 0, 0 },
    { 0x6fac23c3, 98 },
    { 0, 0 },
    { 0x9b6c23c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ac23ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5c23d4, 61 },
    { 0, 0 },
    { 0x498c23d6, 34 },
    { 0x644c23d6, 95 },
    { 0xcd7c23d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c23db, 34 },
    { 0, 0 },
    { 0xafac23dd, 90 },
    { 0xa5c23de, 99 },
    { 0x5d1c23de, 69 },
    { 0xf29c23e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc23e6, 43 },
    { 0x4d5c23e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162c23ec, 61 },
    { 0xbb4c23ec, 61 },
    { 0xf32c23ee, 77 },
    { 0x81bc23ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc23f2, 61 },
    { 0xf41c23f3, 95 },
    { 0, 0 },
    { 0x2c1c23f5, 85 },
    { 0, 0 },
    { 0xb1dc23f7, 76 },
    { 0xebfc23f7, 66 },
    { 0x163c23f9, 87 },
    { 0xa09c23fa, 85 },
    { 0x99ac23fa, 98 },
    { 0x75cc23f7, 98 },
    { 0xcd6c23fd, 67 },
    { 0xbbfc23fd, 90 },
    { 0, 0 },
    { 0x838c2400, 61 },
    { 0xca2c2401, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2c2404, 17 },
    { 0xf20c2405, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71c240a, 33 },
    { 0x6cdc240b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7c2416, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c2419, 99 },
    { 0, 0 },
    { 0xd95c241b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9c241e, 55 },
    { 0xbe4c241f, 67 },
    { 0x9f1c2420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c2425, 43 },
    { 0x3bcc2426, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c2431, 68 },
    { 0xfd3c2432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c2438, 43 },
    { 0x214c2439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc2442, 23 },
    { 0x7dac2443, 76 },
    { 0xbd2c2442, 98 },
    { 0, 0 },
    { 0xed0c2446, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0c244a, 90 },
    { 0x60ac244b, 33 },
    { 0xe4c244b, 90 },
    { 0, 0 },
    { 0xdf8c244e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82c2452, 69 },
    { 0xe4ec2453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec2456, 55 },
    { 0, 0 },
    { 0x5cc2458, 90 },
    { 0x6d7c2459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7c245c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66c2460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bc2463, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33c246a, 98 },
    { 0, 0 },
    { 0x439c246c, 91 },
    { 0, 0 },
    { 0x4fc246e, 64 },
    { 0xa96c246e, 46 },
    { 0xa9bc2470, 90 },
    { 0x199c2471, 33 },
    { 0, 0 },
    { 0x27dc2473, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc2477, 90 },
    { 0xeedc2478, 34 },
    { 0x188c2479, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x199c247c, 69 },
    { 0, 0 },
    { 0x70fc247e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22c2482, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c2486, 61 },
    { 0x3fcc2486, 65 },
    { 0x53ec2488, 23 },
    { 0x41fc2488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3c248e, 98 },
    { 0, 0 },
    { 0x374c2490, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90c2498, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc249e, 90 },
    { 0x65bc249f, 88 },
    { 0xd4ac249f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc24a3, 76 },
    { 0xa9ac24a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c24a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x408c24aa, 33 },
    { 0x682c24aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc24af, 95 },
    { 0, 0 },
    { 0x396c24b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc24b6, 33 },
    { 0x81ac24b6, 23 },
    { 0xdd5c24b7, 61 },
    { 0xb1c24b7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452c24c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3c24c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fec24cc, 61 },
    { 0xe00c24cd, 87 },
    { 0, 0 },
    { 0xf98c24cf, 33 },
    { 0x6bcc24d0, 23 },
    { 0xd36c24d1, 95 },
    { 0x19dc24d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc24da, 91 },
    { 0x26cc24da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6c24e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x855c24e3, 43 },
    { 0xb86c24e4, 17 },
    { 0, 0 },
    { 0x838c24e6, 33 },
    { 0x932c24e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9c24ea, 98 },
    { 0, 0 },
    { 0xa6ac24ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4c24ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534c24f6, 98 },
    { 0, 0 },
    { 0x409c24f8, 43 },
    { 0, 0 },
    { 0xbd9c24fa, 33 },
    { 0xaabc24fa, 77 },
    { 0x1aec24fc, 65 },
    { 0xfe8c24fa, 46 },
    { 0, 0 },
    { 0x9afc24ff, 84 },
    { 0xe4ac2500, 40 },
    { 0x3f9c2500, 43 },
    { 0x4edc2500, 46 },
    { 0x864c2503, 34 },
    { 0x665c2504, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85c250d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dc2510, 84 },
    { 0xb41c2510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176c2515, 61 },
    { 0xa75c2515, 85 },
    { 0xb1ac2516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c251b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c251f, 9 },
    { 0x19fc2520, 90 },
    { 0, 0 },
    { 0xe67c2522, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c2529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509c252e, 33 },
    { 0x684c252f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc2536, 61 },
    { 0, 0 },
    { 0x956c2538, 33 },
    { 0, 0 },
    { 0x755c253a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dec2542, 99 },
    { 0x847c2542, 61 },
    { 0, 0 },
    { 0x52bc2545, 77 },
    { 0xa63c2546, 93 },
    { 0xc6cc2546, 96 },
    { 0xbd5c2548, 76 },
    { 0xec3c2549, 66 },
    { 0xdeec2549, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x143c254d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c2556, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8c255c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c255f, 87 },
    { 0, 0 },
    { 0x823c2561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bc2565, 33 },
    { 0, 0 },
    { 0xe4dc2567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x562c256a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac2572, 67 },
    { 0, 0 },
    { 0x3cac2574, 66 },
    { 0xc91c2574, 95 },
    { 0, 0 },
    { 0xcfcc2577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac257a, 46 },
    { 0, 0 },
    { 0x3c9c257c, 61 },
    { 0, 0 },
    { 0xb33c257e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0c2584, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c2587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0c258c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263c2593, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712c259a, 91 },
    { 0, 0 },
    { 0xc36c259c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc25a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ac25a5, 8 },
    { 0x63dc25a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc25b0, 98 },
    { 0xc46c25b0, 43 },
    { 0x628c25b2, 43 },
    { 0x38c25b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c25b6, 55 },
    { 0x8bac25b7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc25bb, 98 },
    { 0xef7c25bc, 98 },
    { 0x865c25bb, 55 },
    { 0xdb1c25be, 98 },
    { 0x972c25bd, 87 },
    { 0x18bc25c0, 55 },
    { 0xe09c25bf, 98 },
    { 0xe18c25c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c25c9, 55 },
    { 0xc0c25ca, 90 },
    { 0, 0 },
    { 0xd45c25cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c25d5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac25de, 43 },
    { 0, 0 },
    { 0xfd3c25e0, 90 },
    { 0xd8ec25e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c25e4, 78 },
    { 0x1b6c25e5, 84 },
    { 0x1e2c25e5, 23 },
    { 0, 0 },
    { 0x8b8c25e8, 93 },
    { 0xb4fc25e9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc25ef, 88 },
    { 0xc16c25ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c25f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc25f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc25fd, 55 },
    { 0x31ec25fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c2603, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c2607, 33 },
    { 0x84bc2607, 61 },
    { 0, 0 },
    { 0x9a0c260a, 90 },
    { 0, 0 },
    { 0xc27c260c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffc2610, 83 },
    { 0x57ac2611, 33 },
    { 0xd09c2612, 43 },
    { 0x47cc2612, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c2621, 90 },
    { 0xf3ac2621, 90 },
    { 0xdc6c2623, 97 },
    { 0x8aec2624, 33 },
    { 0xc7bc2624, 33 },
    { 0, 0 },
    { 0xed6c2627, 61 },
    { 0, 0 },
    { 0xdb1c2629, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6c262e, 61 },
    { 0xed6c262e, 61 },
    { 0xe69c262f, 98 },
    { 0xc1fc2631, 98 },
    { 0x1c1c262f, 52 },
    { 0, 0 },
    { 0x8f5c2634, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafc2638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54c263b, 99 },
    { 0xc9ac263c, 55 },
    { 0x16fc263d, 93 },
    { 0x851c263c, 61 },
    { 0x1d5c263f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc2642, 95 },
    { 0xa2ec2643, 84 },
    { 0x727c2644, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x542c2647, 61 },
    { 0, 0 },
    { 0xfafc2649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43c2651, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756c2655, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c265a, 91 },
    { 0x7ac265b, 90 },
    { 0xda1c265c, 87 },
    { 0x477c265d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c2661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97c2664, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc266b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6c266e, 87 },
    { 0xf13c266e, 90 },
    { 0x809c266e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc2673, 61 },
    { 0, 0 },
    { 0x2a2c2675, 85 },
    { 0xc5bc2676, 23 },
    { 0x3d8c2677, 66 },
    { 0x82bc2677, 95 },
    { 0x554c2679, 34 },
    { 0x314c2679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c267e, 95 },
    { 0, 0 },
    { 0xcb3c2680, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c2684, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07c2689, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c268d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec2690, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cc2693, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61c2698, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bc269b, 74 },
    { 0x3a3c269b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c26a2, 78 },
    { 0x601c26a2, 33 },
    { 0x832c26a4, 43 },
    { 0xae8c26a4, 76 },
    { 0x8bcc26a4, 33 },
    { 0xe7bc26a6, 76 },
    { 0x517c26a8, 98 },
    { 0x6a6c26a9, 95 },
    { 0, 0 },
    { 0x223c26ab, 90 },
    { 0x1a8c26ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc26af, 61 },
    { 0xff8c26b0, 55 },
    { 0, 0 },
    { 0x64cc26b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c26b5, 23 },
    { 0, 0 },
    { 0xf2dc26b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x469c26ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc26bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcac26ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ac26d5, 95 },
    { 0xf0cc26d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5c26d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ec26dd, 90 },
    { 0x110c26dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc26e1, 90 },
    { 0x810c26e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8c26e6, 55 },
    { 0, 0 },
    { 0x7edc26e8, 95 },
    { 0x435c26e9, 34 },
    { 0x2bdc26ea, 95 },
    { 0, 0 },
    { 0x443c26ec, 98 },
    { 0x41cc26ed, 68 },
    { 0xec4c26ec, 38 },
    { 0x798c26ef, 61 },
    { 0x5f2c26ec, 98 },
    { 0xa4ec26f1, 89 },
    { 0x8d9c26ec, 95 },
    { 0x478c26f1, 61 },
    { 0xa7c26f0, 95 },
    { 0x181c26f2, 101 },
    { 0xc7cc26ee, 93 },
    { 0x843c26f5, 43 },
    { 0x57bc26ec, 43 },
    { 0x6fcc26f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ac26ff, 36 },
    { 0x190c26ff, 34 },
    { 0x654c26ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc2705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdc2709, 93 },
    { 0x80ac2709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16c270e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3c2712, 60 },
    { 0x854c2713, 96 },
    { 0xd9c2713, 65 },
    { 0x338c2713, 95 },
    { 0x531c2715, 61 },
    { 0x7fbc2717, 98 },
    { 0xe97c2718, 33 },
    { 0xf4ec2719, 95 },
    { 0xfdfc2718, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc271e, 90 },
    { 0x9adc271e, 85 },
    { 0, 0 },
    { 0x5cbc2721, 81 },
    { 0x1fec2721, 33 },
    { 0x2d5c2723, 33 },
    { 0xeeec2724, 68 },
    { 0x6e8c2724, 68 },
    { 0x248c2726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c272f, 95 },
    { 0xe86c2730, 34 },
    { 0x892c2731, 61 },
    { 0xbb1c2730, 17 },
    { 0xa7dc2733, 61 },
    { 0x2c2c2734, 98 },
    { 0xd37c2735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc2739, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c273c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbec2741, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c2744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c2748, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c274e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fc2753, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcc275a, 33 },
    { 0x118c275b, 95 },
    { 0, 0 },
    { 0x9a2c275d, 94 },
    { 0xaa5c275e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54c2769, 98 },
    { 0xdd8c276a, 33 },
    { 0, 0 },
    { 0x9afc276c, 66 },
    { 0xcc8c276c, 78 },
    { 0x3a1c276d, 33 },
    { 0, 0 },
    { 0x8dbc2770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cc2773, 61 },
    { 0xd60c2774, 34 },
    { 0xacdc2775, 78 },
    { 0x917c2775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4c277e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8c2783, 43 },
    { 0x38cc2783, 61 },
    { 0x89ec2783, 95 },
    { 0, 0 },
    { 0x611c2787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccc278d, 68 },
    { 0x591c278d, 37 },
    { 0x920c278f, 34 },
    { 0x692c2790, 52 },
    { 0, 0 },
    { 0xaa8c2792, 90 },
    { 0, 0 },
    { 0x3dac2794, 40 },
    { 0xe7cc2794, 55 },
    { 0x33cc2796, 34 },
    { 0xe8dc2797, 33 },
    { 0, 0 },
    { 0xb3dc2799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87c279c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269c27a3, 76 },
    { 0xd30c27a3, 33 },
    { 0xf90c27a5, 90 },
    { 0x47ec27a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4c27aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dc27ad, 55 },
    { 0x9b7c27ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cc27b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c27b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c27bd, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c27c3, 76 },
    { 0x57dc27c3, 33 },
    { 0, 0 },
    { 0x43c27c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7c27cb, 76 },
    { 0, 0 },
    { 0x2e1c27cd, 93 },
    { 0x9c5c27ce, 33 },
    { 0, 0 },
    { 0x793c27d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc27d7, 95 },
    { 0, 0 },
    { 0xb15c27d9, 84 },
    { 0xeb0c27d9, 99 },
    { 0, 0 },
    { 0x4e6c27dc, 90 },
    { 0xb54c27dc, 78 },
    { 0xe9ac27de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c27e2, 33 },
    { 0xed3c27e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c27e9, 34 },
    { 0x6a1c27ea, 33 },
    { 0, 0 },
    { 0x210c27ec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cec27f4, 90 },
    { 0x4b0c27f4, 91 },
    { 0xbcc27f5, 95 },
    { 0, 0 },
    { 0xa5cc27f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0c27fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369c27ff, 95 },
    { 0xa3dc27ff, 23 },
    { 0, 0 },
    { 0x624c2802, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5c2805, 34 },
    { 0x20ac2805, 91 },
    { 0, 0 },
    { 0x255c2808, 34 },
    { 0, 0 },
    { 0x5a4c280a, 23 },
    { 0xd4dc280b, 47 },
    { 0x124c280c, 88 },
    { 0x115c280d, 33 },
    { 0xbecc280c, 99 },
    { 0, 0 },
    { 0xc0dc2810, 33 },
    { 0x2cec2811, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec2819, 91 },
    { 0x994c281a, 90 },
    { 0x8f0c281b, 23 },
    { 0xae5c281b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c2820, 33 },
    { 0x59cc2821, 61 },
    { 0x188c2821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4c2827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c282b, 66 },
    { 0, 0 },
    { 0xbe8c282d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaac2831, 52 },
    { 0, 0 },
    { 0x465c2833, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c2836, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721c283c, 77 },
    { 0xd91c283c, 76 },
    { 0, 0 },
    { 0xeedc283f, 95 },
    { 0x353c2840, 23 },
    { 0, 0 },
    { 0x23fc2842, 90 },
    { 0xfaec2842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac2859, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x493c285c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c285f, 61 },
    { 0, 0 },
    { 0x25ac2861, 61 },
    { 0x90c2861, 90 },
    { 0xd64c2863, 23 },
    { 0xfd5c2863, 12 },
    { 0, 0 },
    { 0xee9c2866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc286d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2872, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8c2876, 95 },
    { 0x16bc2877, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fc287f, 90 },
    { 0, 0 },
    { 0xb5fc2881, 90 },
    { 0xc70c2882, 33 },
    { 0x7f9c2883, 98 },
    { 0, 0 },
    { 0x85ac2885, 78 },
    { 0x3dac2885, 98 },
    { 0x26ec2887, 33 },
    { 0xe21c2888, 76 },
    { 0xfa9c2888, 33 },
    { 0xa4dc288a, 33 },
    { 0xdb0c2888, 33 },
    { 0xeeac2887, 100 },
    { 0xf50c288c, 90 },
    { 0x741c288e, 90 },
    { 0xf6c2888, 98 },
    { 0, 0 },
    { 0xd5bc2891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39c289a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781c28a0, 95 },
    { 0, 0 },
    { 0x36ac28a2, 23 },
    { 0x3f2c28a2, 98 },
    { 0, 0 },
    { 0xff8c28a5, 55 },
    { 0xbd9c28a5, 85 },
    { 0xafc28a7, 61 },
    { 0, 0 },
    { 0xa04c28a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2c28ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcc28af, 98 },
    { 0, 0 },
    { 0x789c28b1, 46 },
    { 0x2fac28b2, 90 },
    { 0x39c28b3, 46 },
    { 0xb7fc28b4, 93 },
    { 0, 0 },
    { 0xc6bc28b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c28ba, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c28bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c28c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c28cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec28d0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36c28d4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c28d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24c28dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0c28e2, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c28e7, 85 },
    { 0, 0 },
    { 0x238c28e9, 33 },
    { 0xec0c28ea, 23 },
    { 0, 0 },
    { 0x95cc28ec, 22 },
    { 0x63bc28ed, 98 },
    { 0, 0 },
    { 0x6d2c28ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c28f3, 78 },
    { 0xffdc28f4, 78 },
    { 0x714c28f5, 88 },
    { 0, 0 },
    { 0xd1bc28f7, 90 },
    { 0xf81c28f8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ac28fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c28ff, 76 },
    { 0x200c28ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbec2908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc290f, 34 },
    { 0xa03c2910, 92 },
    { 0, 0 },
    { 0x937c2912, 91 },
    { 0, 0 },
    { 0xba1c2914, 61 },
    { 0x5eac2914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc291b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ec2923, 98 },
    { 0x731c2924, 93 },
    { 0x949c2924, 68 },
    { 0x88cc2923, 23 },
    { 0x3c0c2927, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffc292d, 33 },
    { 0xda8c292d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c2934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88c2938, 90 },
    { 0, 0 },
    { 0x464c293a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec293d, 88 },
    { 0xf86c293e, 33 },
    { 0, 0 },
    { 0x207c2940, 52 },
    { 0x123c2940, 61 },
    { 0x610c2942, 76 },
    { 0xd4bc2943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc2948, 90 },
    { 0x6ccc2949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc294c, 61 },
    { 0, 0 },
    { 0xfdfc294e, 76 },
    { 0xc65c294f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6c2956, 61 },
    { 0, 0 },
    { 0x3a6c2958, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197c295d, 100 },
    { 0, 0 },
    { 0x27bc295f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ec2962, 34 },
    { 0xc5bc2962, 98 },
    { 0, 0 },
    { 0xa0bc2965, 17 },
    { 0x75ec2966, 23 },
    { 0xd56c2966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2978, 23 },
    { 0xea8c2979, 46 },
    { 0, 0 },
    { 0xd56c297b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dec297e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79c2983, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224c2988, 95 },
    { 0, 0 },
    { 0x7a8c298a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c298f, 23 },
    { 0x2a1c298f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c2993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x157c2996, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1c299b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc299e, 61 },
    { 0x8cdc299f, 98 },
    { 0x869c299f, 34 },
    { 0, 0 },
    { 0x2dac29a2, 34 },
    { 0xc72c29a3, 33 },
    { 0x774c29a2, 67 },
    { 0x34cc29a3, 87 },
    { 0, 0 },
    { 0xee6c29a7, 33 },
    { 0xaaec29a8, 67 },
    { 0x5c6c29a7, 23 },
    { 0x270c29aa, 96 },
    { 0x2fac29ab, 46 },
    { 0x399c29ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c29b0, 81 },
    { 0, 0 },
    { 0x4ebc29b2, 90 },
    { 0, 0 },
    { 0x46ec29b4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c29b8, 95 },
    { 0x4edc29b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x206c29bc, 33 },
    { 0xc8dc29bc, 85 },
    { 0xa12c29be, 91 },
    { 0, 0 },
    { 0xe6c29c0, 33 },
    { 0x84c29c1, 98 },
    { 0x7ac29c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc29c6, 65 },
    { 0x28cc29c7, 95 },
    { 0xf06c29c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc29cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3c29d2, 99 },
    { 0xbe8c29d2, 33 },
    { 0xab4c29d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x135c29d7, 33 },
    { 0xa0c29d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9c29dc, 95 },
    { 0x9fec29dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbc29e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c29e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2c29ec, 95 },
    { 0xa79c29ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ec29f0, 95 },
    { 0, 0 },
    { 0x94fc29f2, 90 },
    { 0x80fc29f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec29f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adc29fa, 90 },
    { 0, 0 },
    { 0xa4ac29fc, 61 },
    { 0x97ec29fc, 90 },
    { 0xfdcc29fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efc2a03, 90 },
    { 0x273c2a04, 61 },
    { 0xd7cc2a05, 90 },
    { 0xb66c2a03, 99 },
    { 0x9f4c2a07, 33 },
    { 0x138c2a08, 86 },
    { 0xa65c2a05, 75 },
    { 0, 0 },
    { 0x98dc2a0b, 90 },
    { 0x7fbc2a0c, 90 },
    { 0x807c2a0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c2a12, 61 },
    { 0xc2bc2a13, 23 },
    { 0xf61c2a14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fc2a1a, 88 },
    { 0xd62c2a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4c2a1e, 23 },
    { 0x68bc2a1f, 77 },
    { 0x2f4c2a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc2a24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec2a2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ac2a36, 65 },
    { 0x46c2a37, 23 },
    { 0, 0 },
    { 0xac0c2a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c2a3c, 61 },
    { 0xa58c2a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c2a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6c2a48, 61 },
    { 0, 0 },
    { 0x1f3c2a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c2a4f, 98 },
    { 0x174c2a50, 40 },
    { 0x7e3c2a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c2a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c2a63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac2a67, 77 },
    { 0x14c2a68, 78 },
    { 0, 0 },
    { 0x32c2a6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec2a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabac2a72, 98 },
    { 0x30dc2a73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8c2a78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cc2a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc2a80, 100 },
    { 0x114c2a80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afc2a8c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x561c2a8f, 66 },
    { 0x980c2a90, 93 },
    { 0x2e4c2a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bc2a95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac2a99, 78 },
    { 0xbc1c2a9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c2aa8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c2aab, 40 },
    { 0, 0 },
    { 0x8bec2aad, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c2ab2, 90 },
    { 0x8f5c2ab3, 90 },
    { 0xe3ec2ab3, 55 },
    { 0xc55c2ab5, 95 },
    { 0, 0 },
    { 0x6e7c2ab7, 77 },
    { 0xc44c2ab7, 52 },
    { 0, 0 },
    { 0xc6bc2aba, 33 },
    { 0x9abc2abb, 61 },
    { 0xea5c2abb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc2abf, 85 },
    { 0xb96c2ac0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c2acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc2ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2ada, 76 },
    { 0xd89c2adb, 23 },
    { 0, 0 },
    { 0x9fc2add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c2ae2, 71 },
    { 0xeaac2ae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01c2ae8, 34 },
    { 0, 0 },
    { 0x24bc2aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2aed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c2af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec2afc, 43 },
    { 0xde2c2afd, 99 },
    { 0xb53c2afd, 23 },
    { 0x239c2afe, 90 },
    { 0x7f7c2b00, 33 },
    { 0x90dc2b00, 97 },
    { 0x67c2b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c2b07, 95 },
    { 0xe81c2b08, 23 },
    { 0xfb4c2b09, 43 },
    { 0xb99c2b0a, 90 },
    { 0x711c2b0b, 66 },
    { 0, 0 },
    { 0x4c9c2b0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752c2b14, 33 },
    { 0x36cc2b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c2b1c, 34 },
    { 0xaa5c2b1d, 95 },
    { 0x1ffc2b1d, 90 },
    { 0xd4bc2b1f, 33 },
    { 0xed6c2b20, 61 },
    { 0x4dc2b20, 83 },
    { 0, 0 },
    { 0x42ac2b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07c2b27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1c2b2a, 34 },
    { 0, 0 },
    { 0x56bc2b2c, 34 },
    { 0xe69c2b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43c2b30, 95 },
    { 0, 0 },
    { 0x296c2b32, 33 },
    { 0xb6c2b33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c2b36, 90 },
    { 0xc5ac2b36, 61 },
    { 0, 0 },
    { 0x88c2b39, 34 },
    { 0xa4cc2b39, 43 },
    { 0x947c2b3b, 98 },
    { 0, 0 },
    { 0x94ac2b3d, 61 },
    { 0x66ec2b3d, 34 },
    { 0xcdbc2b3f, 23 },
    { 0x2cdc2b40, 26 },
    { 0x23ac2b41, 38 },
    { 0, 0 },
    { 0xc54c2b43, 76 },
    { 0x47fc2b44, 90 },
    { 0x43fc2b45, 93 },
    { 0x6f7c2b45, 95 },
    { 0, 0 },
    { 0x27ac2b48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c2b4b, 33 },
    { 0x16cc2b4c, 96 },
    { 0xff7c2b4d, 29 },
    { 0xe4dc2b4d, 95 },
    { 0x826c2b4f, 55 },
    { 0xf4cc2b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc2b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ec2b5b, 88 },
    { 0xbafc2b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bc2b60, 90 },
    { 0xfc7c2b61, 55 },
    { 0, 0 },
    { 0xe24c2b63, 23 },
    { 0, 0 },
    { 0x677c2b65, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c2b69, 78 },
    { 0xd64c2b6a, 90 },
    { 0xc3fc2b69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2b71, 33 },
    { 0x348c2b72, 61 },
    { 0xcfc2b72, 84 },
    { 0x977c2b74, 98 },
    { 0xde3c2b74, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243c2b79, 95 },
    { 0, 0 },
    { 0xa79c2b7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40c2b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c2b82, 46 },
    { 0xde6c2b82, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ec2b86, 90 },
    { 0, 0 },
    { 0xaf4c2b88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c2b92, 33 },
    { 0x690c2b92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac2b99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ec2ba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c2ba8, 95 },
    { 0xd6ac2ba9, 66 },
    { 0, 0 },
    { 0xa2fc2bab, 98 },
    { 0xb13c2bab, 61 },
    { 0, 0 },
    { 0x1b2c2bae, 90 },
    { 0, 0 },
    { 0x9fac2bb0, 90 },
    { 0x16ec2bb0, 91 },
    { 0xb56c2bb0, 61 },
    { 0x406c2bb0, 34 },
    { 0, 0 },
    { 0xcd8c2bb5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac2bb9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5c2bbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bc2bc1, 95 },
    { 0, 0 },
    { 0x7e6c2bc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c2bc6, 88 },
    { 0x4c8c2bc7, 66 },
    { 0, 0 },
    { 0x165c2bc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228c2bce, 76 },
    { 0xdbac2bce, 55 },
    { 0x3f3c2bce, 47 },
    { 0xe57c2bd1, 55 },
    { 0xcf2c2bd2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c2bd6, 78 },
    { 0, 0 },
    { 0x175c2bd8, 52 },
    { 0xe45c2bd9, 43 },
    { 0x5f4c2bda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c2bdf, 93 },
    { 0x49ac2be0, 82 },
    { 0x5d2c2be0, 66 },
    { 0xa7c2be2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95c2be7, 61 },
    { 0x2e7c2be8, 52 },
    { 0x210c2be9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x728c2bec, 95 },
    { 0x2aec2bed, 66 },
    { 0xdbcc2bee, 69 },
    { 0xd32c2bef, 43 },
    { 0, 0 },
    { 0xbd2c2bf1, 99 },
    { 0xc84c2bf2, 23 },
    { 0x8cbc2bf2, 98 },
    { 0xd19c2bf3, 90 },
    { 0x25bc2bf5, 78 },
    { 0, 0 },
    { 0x9d1c2bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97c2bfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc2bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2c2c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9c2c07, 90 },
    { 0x516c2c07, 61 },
    { 0xad2c2c07, 84 },
    { 0xd77c2c09, 34 },
    { 0, 0 },
    { 0xbe5c2c0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749c2c16, 55 },
    { 0, 0 },
    { 0x234c2c18, 93 },
    { 0xc8fc2c18, 88 },
    { 0x6ac2c1a, 34 },
    { 0x81ec2c1a, 55 },
    { 0x76dc2c1c, 95 },
    { 0, 0 },
    { 0x21cc2c1e, 99 },
    { 0x19dc2c1e, 66 },
    { 0, 0 },
    { 0x3cac2c21, 81 },
    { 0x49ac2c21, 61 },
    { 0, 0 },
    { 0x586c2c24, 95 },
    { 0xc76c2c24, 91 },
    { 0x19bc2c26, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19c2c2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffc2c2f, 95 },
    { 0x59c2c30, 98 },
    { 0, 0 },
    { 0x7fbc2c32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7c2c37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdc2c3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cc2c40, 55 },
    { 0, 0 },
    { 0xf04c2c42, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeac2c48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c2c4b, 90 },
    { 0, 0 },
    { 0xa18c2c4d, 67 },
    { 0x6a2c2c4e, 76 },
    { 0x680c2c4e, 95 },
    { 0xb63c2c4f, 61 },
    { 0xd03c2c51, 61 },
    { 0x8b4c2c52, 23 },
    { 0x2dc2c4f, 100 },
    { 0xfcfc2c4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c2c57, 36 },
    { 0, 0 },
    { 0x63cc2c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ec2c5c, 33 },
    { 0, 0 },
    { 0x7d8c2c5e, 61 },
    { 0x21dc2c5e, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2c65, 61 },
    { 0x455c2c66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fc2c69, 84 },
    { 0, 0 },
    { 0x6c6c2c6b, 33 },
    { 0xbe1c2c6c, 23 },
    { 0x84cc2c6d, 46 },
    { 0xad8c2c6c, 96 },
    { 0x9ecc2c6f, 90 },
    { 0xa1dc2c70, 95 },
    { 0xefcc2c6d, 91 },
    { 0x14dc2c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15c2c77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ec2c7b, 61 },
    { 0, 0 },
    { 0xd05c2c7d, 95 },
    { 0xc46c2c7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89c2c81, 34 },
    { 0, 0 },
    { 0x402c2c83, 34 },
    { 0xad8c2c84, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48c2c87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474c2c8c, 46 },
    { 0x5bcc2c8c, 95 },
    { 0x62fc2c8e, 55 },
    { 0x8a6c2c8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c2c93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c2c9c, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c2c9f, 83 },
    { 0x54ec2ca0, 77 },
    { 0xe97c2c9f, 95 },
    { 0xba7c2ca2, 93 },
    { 0x614c2ca3, 98 },
    { 0xf73c2ca4, 95 },
    { 0x665c2c9f, 95 },
    { 0x523c2ca6, 85 },
    { 0x586c2ca7, 61 },
    { 0xd6c2ca7, 33 },
    { 0xc65c2ca9, 76 },
    { 0xff9c2ca9, 98 },
    { 0x1b2c2ca7, 61 },
    { 0xd16c2cac, 33 },
    { 0xd55c2cad, 33 },
    { 0xfcc2cac, 98 },
    { 0xadec2cae, 95 },
    { 0x8ffc2cb0, 61 },
    { 0x8b9c2ca3, 61 },
    { 0x9aac2cb2, 95 },
    { 0x361c2cb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacc2cb6, 87 },
    { 0x316c2cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbc2cbd, 76 },
    { 0x516c2cbe, 17 },
    { 0xac1c2cbd, 99 },
    { 0x631c2cc0, 66 },
    { 0, 0 },
    { 0xbcac2cc2, 67 },
    { 0x48bc2cc2, 76 },
    { 0x9e2c2cc2, 93 },
    { 0x269c2cc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ac2ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0c2cd1, 98 },
    { 0, 0 },
    { 0x52ec2cd3, 78 },
    { 0x1bac2cd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc2cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c2ce1, 90 },
    { 0, 0 },
    { 0x6b5c2ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c2ce7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51c2cec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccec2cf4, 76 },
    { 0xa51c2cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c2cf8, 90 },
    { 0, 0 },
    { 0x4d2c2cfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2d00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc2d03, 98 },
    { 0x3d1c2d04, 55 },
    { 0xf70c2d05, 33 },
    { 0xc29c2d06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49c2d09, 33 },
    { 0x6b4c2d09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124c2d0e, 61 },
    { 0x214c2d0f, 76 },
    { 0xd74c2d0e, 90 },
    { 0xe03c2d11, 61 },
    { 0, 0 },
    { 0xc2cc2d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c2d16, 98 },
    { 0xcb5c2d17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc2d1a, 23 },
    { 0x783c2d1a, 93 },
    { 0x60ec2d1c, 99 },
    { 0, 0 },
    { 0xc61c2d1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209c2d27, 75 },
    { 0, 0 },
    { 0xb4c2d29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c2d2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9c2d33, 68 },
    { 0x9dbc2d34, 99 },
    { 0, 0 },
    { 0xe4ec2d36, 23 },
    { 0, 0 },
    { 0x615c2d38, 68 },
    { 0x8e1c2d38, 33 },
    { 0x790c2d3a, 61 },
    { 0x855c2d3a, 95 },
    { 0x132c2d3c, 90 },
    { 0x814c2d3c, 90 },
    { 0x9e6c2d3c, 90 },
    { 0xdf7c2d39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c2d43, 23 },
    { 0x58fc2d44, 33 },
    { 0, 0 },
    { 0xc4dc2d46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09c2d4d, 34 },
    { 0xfeec2d4d, 99 },
    { 0xb65c2d4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc2d56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9c2d60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc2d6a, 90 },
    { 0x5dbc2d6a, 98 },
    { 0x9ac2d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4c2d6f, 33 },
    { 0x943c2d6f, 33 },
    { 0x37dc2d70, 61 },
    { 0xc53c2d72, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbc2d79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ec2d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cc2d87, 43 },
    { 0x945c2d88, 66 },
    { 0, 0 },
    { 0x5b7c2d8a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7c2d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebc2d95, 95 },
    { 0x7ecc2d95, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eac2d9b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c2da2, 33 },
    { 0x1c2c2da2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c2da6, 33 },
    { 0x9ac2da6, 33 },
    { 0, 0 },
    { 0xf20c2da9, 33 },
    { 0x397c2da9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2db1, 68 },
    { 0x819c2db2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec2db7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fc2dbc, 66 },
    { 0xa28c2dbc, 76 },
    { 0xf4fc2dbd, 85 },
    { 0xb1ec2dbf, 23 },
    { 0x7e2c2dbf, 52 },
    { 0, 0 },
    { 0x366c2dc2, 61 },
    { 0x1dc2dc2, 33 },
    { 0xcd5c2dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c2dcb, 61 },
    { 0xf33c2dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8c2dd1, 90 },
    { 0, 0 },
    { 0xf6c2dd3, 75 },
    { 0xd0ac2dd4, 55 },
    { 0, 0 },
    { 0xd8bc2dd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdec2de0, 69 },
    { 0xd8ac2de0, 81 },
    { 0xa30c2de2, 96 },
    { 0xf7c2de3, 100 },
    { 0, 0 },
    { 0xa2ac2de5, 23 },
    { 0xb63c2de6, 23 },
    { 0xf4ec2de5, 100 },
    { 0, 0 },
    { 0x48dc2de9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acc2ded, 33 },
    { 0x31c2ded, 23 },
    { 0x1b6c2def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec2df2, 33 },
    { 0xffec2df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bc2df8, 61 },
    { 0, 0 },
    { 0xeebc2dfa, 100 },
    { 0, 0 },
    { 0x5a7c2dfc, 84 },
    { 0x69bc2dfd, 90 },
    { 0x9b9c2dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89c2e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c2e06, 66 },
    { 0, 0 },
    { 0xad4c2e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dc2e0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x867c2e0e, 61 },
    { 0, 0 },
    { 0x35ac2e10, 33 },
    { 0x4b0c2e10, 33 },
    { 0x6f2c2e11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c2e19, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c2e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c2e1f, 61 },
    { 0xf81c2e20, 23 },
    { 0xfdfc2e1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2e24, 46 },
    { 0x1cc2e24, 43 },
    { 0, 0 },
    { 0x8aac2e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fc2e2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dc2e32, 100 },
    { 0xc1c2e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21c2e36, 100 },
    { 0, 0 },
    { 0xceec2e38, 55 },
    { 0xe6bc2e39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fec2e3c, 95 },
    { 0x8c4c2e3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4c2e40, 90 },
    { 0x26bc2e40, 98 },
    { 0, 0 },
    { 0xc1cc2e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7c2e49, 98 },
    { 0x984c2e49, 61 },
    { 0x752c2e4b, 90 },
    { 0xea4c2e4c, 61 },
    { 0xe76c2e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6c2e50, 55 },
    { 0x1dbc2e50, 90 },
    { 0, 0 },
    { 0x43ec2e53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc2e57, 23 },
    { 0, 0 },
    { 0x243c2e59, 93 },
    { 0x42dc2e5a, 76 },
    { 0xbc3c2e5b, 98 },
    { 0x754c2e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c2e61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fc2e64, 90 },
    { 0xdb2c2e64, 32 },
    { 0xf5cc2e66, 23 },
    { 0xa9c2e66, 38 },
    { 0, 0 },
    { 0xa60c2e69, 61 },
    { 0x89bc2e6a, 98 },
    { 0xde5c2e6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c2e6e, 33 },
    { 0x719c2e6f, 23 },
    { 0x19c2e70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c2e74, 52 },
    { 0, 0 },
    { 0xebfc2e76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc2e7b, 55 },
    { 0, 0 },
    { 0xd4fc2e7d, 78 },
    { 0, 0 },
    { 0x8d3c2e7f, 23 },
    { 0x2bec2e80, 34 },
    { 0x6aec2e81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c2e85, 46 },
    { 0xafcc2e85, 61 },
    { 0, 0 },
    { 0xfb2c2e88, 61 },
    { 0xd5cc2e89, 55 },
    { 0xde3c2e8a, 55 },
    { 0x6fcc2e8b, 23 },
    { 0xf84c2e8c, 93 },
    { 0, 0 },
    { 0x4e9c2e8e, 52 },
    { 0x98ac2e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc2e92, 17 },
    { 0, 0 },
    { 0x964c2e94, 46 },
    { 0xa02c2e94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c2e99, 66 },
    { 0xea2c2e99, 33 },
    { 0xe36c2e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c2e9e, 90 },
    { 0, 0 },
    { 0x9f5c2ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500c2ea5, 34 },
    { 0x118c2ea6, 100 },
    { 0xc6dc2ea7, 95 },
    { 0, 0 },
    { 0x7afc2ea9, 48 },
    { 0, 0 },
    { 0x843c2eab, 87 },
    { 0x9b5c2eac, 84 },
    { 0x895c2eac, 23 },
    { 0xc88c2eae, 81 },
    { 0xcf5c2ead, 95 },
    { 0, 0 },
    { 0x341c2eb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bc2eb7, 33 },
    { 0x544c2eb8, 95 },
    { 0xe60c2eb7, 34 },
    { 0, 0 },
    { 0xf2bc2ebb, 88 },
    { 0xa5ac2ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c2ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c2ec3, 61 },
    { 0x425c2ec4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c2ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc2ed1, 34 },
    { 0, 0 },
    { 0x1efc2ed3, 33 },
    { 0x5cfc2ed3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c2ed7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ec2eda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc2ede, 33 },
    { 0xd73c2edf, 95 },
    { 0, 0 },
    { 0xfc0c2ee1, 61 },
    { 0, 0 },
    { 0x1d0c2ee3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1c2ee6, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x427c2ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x520c2eec, 95 },
    { 0x321c2eed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac2ef2, 23 },
    { 0x354c2ef3, 61 },
    { 0x6fdc2ef4, 81 },
    { 0x6abc2ef3, 23 },
    { 0x7dbc2ef6, 95 },
    { 0xa1cc2ef3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59c2efb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c2efe, 33 },
    { 0xb58c2eff, 98 },
    { 0x802c2f00, 34 },
    { 0xa68c2eff, 67 },
    { 0xc71c2f02, 23 },
    { 0xad7c2efe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dc2f0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x279c2f10, 43 },
    { 0xcf2c2f11, 17 },
    { 0x401c2f12, 23 },
    { 0x54cc2f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2f17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2f1d, 33 },
    { 0x59ac2f1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffc2f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c2f2d, 98 },
    { 0xc15c2f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c2f38, 95 },
    { 0xc47c2f39, 61 },
    { 0x32ec2f38, 55 },
    { 0x1a1c2f3b, 95 },
    { 0, 0 },
    { 0x336c2f3d, 61 },
    { 0x490c2f3e, 33 },
    { 0xd53c2f3f, 96 },
    { 0, 0 },
    { 0x277c2f41, 55 },
    { 0, 0 },
    { 0xeeec2f43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697c2f49, 61 },
    { 0xa26c2f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c2f4d, 34 },
    { 0xe65c2f4e, 46 },
    { 0, 0 },
    { 0xf49c2f50, 84 },
    { 0, 0 },
    { 0xc79c2f52, 34 },
    { 0x87ec2f53, 65 },
    { 0x66bc2f54, 84 },
    { 0x106c2f55, 95 },
    { 0x483c2f56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18c2f5a, 98 },
    { 0x64bc2f5b, 61 },
    { 0, 0 },
    { 0x83cc2f5d, 61 },
    { 0, 0 },
    { 0x8fdc2f5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c2f63, 93 },
    { 0, 0 },
    { 0x243c2f65, 55 },
    { 0x91cc2f66, 91 },
    { 0x1ddc2f67, 23 },
    { 0x856c2f67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc2f6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc2f6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x738c2f72, 46 },
    { 0xf34c2f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c2f79, 23 },
    { 0, 0 },
    { 0x343c2f7b, 90 },
    { 0xad4c2f7b, 64 },
    { 0, 0 },
    { 0xb22c2f7e, 23 },
    { 0x9abc2f7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71c2f83, 61 },
    { 0xa45c2f83, 95 },
    { 0xd92c2f85, 99 },
    { 0xe3fc2f86, 66 },
    { 0x6d8c2f87, 61 },
    { 0x8ffc2f83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c2f92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c2f96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ac2f99, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1c2fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83c2fab, 85 },
    { 0x575c2fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68c2fb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c2fb3, 55 },
    { 0, 0 },
    { 0x25ac2fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac2fb8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ac2fbb, 23 },
    { 0xfc9c2fbc, 61 },
    { 0x92ec2fbb, 87 },
    { 0xe8cc2fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cc2fc7, 36 },
    { 0xd9dc2fc7, 61 },
    { 0xaacc2fc9, 90 },
    { 0xc36c2fca, 76 },
    { 0x6ffc2fcb, 76 },
    { 0, 0 },
    { 0x84cc2fcd, 100 },
    { 0xd72c2fce, 98 },
    { 0, 0 },
    { 0xae5c2fd0, 61 },
    { 0, 0 },
    { 0xac0c2fd2, 95 },
    { 0xd0ac2fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c2fd7, 90 },
    { 0, 0 },
    { 0x72dc2fd9, 98 },
    { 0xc46c2fda, 90 },
    { 0x754c2fd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc2fde, 90 },
    { 0, 0 },
    { 0x577c2fe0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c2fe4, 98 },
    { 0x4a7c2fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85c2fee, 34 },
    { 0, 0 },
    { 0xf8bc2ff0, 76 },
    { 0x933c2ff0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59c2ff8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acc2ffe, 34 },
    { 0, 0 },
    { 0xd5bc3000, 61 },
    { 0x45cc3001, 34 },
    { 0xb5cc3002, 61 },
    { 0xf35c3003, 90 },
    { 0x630c3004, 61 },
    { 0, 0 },
    { 0x9a0c3006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec300a, 43 },
    { 0x6ec300b, 46 },
    { 0, 0 },
    { 0xa0c300d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fc3011, 91 },
    { 0xa5dc3011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac3017, 43 },
    { 0x65fc3018, 33 },
    { 0, 0 },
    { 0xe4fc301a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acc301d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dc3024, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20c302a, 90 },
    { 0xefbc302b, 99 },
    { 0, 0 },
    { 0xe45c302d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3c3030, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec3033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fc3037, 98 },
    { 0, 0 },
    { 0x613c3039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecc3043, 61 },
    { 0x968c3044, 67 },
    { 0xacec3043, 98 },
    { 0, 0 },
    { 0xbfdc3047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc3050, 61 },
    { 0x931c3051, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83c3054, 23 },
    { 0, 0 },
    { 0x95cc3056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23c305d, 34 },
    { 0x575c305e, 55 },
    { 0xd1dc305d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1c3068, 78 },
    { 0x80ac3069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c306d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dc3073, 34 },
    { 0, 0 },
    { 0xd3c3075, 43 },
    { 0, 0 },
    { 0xef6c3077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55c3082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74c3089, 23 },
    { 0xae8c308a, 84 },
    { 0, 0 },
    { 0x77cc308c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c3090, 66 },
    { 0x697c3091, 95 },
    { 0x472c3092, 61 },
    { 0xddbc3091, 77 },
    { 0x958c3094, 52 },
    { 0x509c3092, 43 },
    { 0x9fec3096, 90 },
    { 0x312c3097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c309b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc309f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90c30a3, 76 },
    { 0xc91c30a3, 61 },
    { 0, 0 },
    { 0xedbc30a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbc30ac, 84 },
    { 0x874c30ad, 98 },
    { 0xf5c30ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8c30b3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc30b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19c30bb, 76 },
    { 0x432c30bc, 97 },
    { 0x706c30bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bc30c0, 66 },
    { 0xd0ec30c1, 78 },
    { 0xce2c30c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc30d0, 88 },
    { 0, 0 },
    { 0xa89c30d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c30d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adc30e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34c30e7, 61 },
    { 0, 0 },
    { 0x9f5c30e9, 34 },
    { 0xca6c30e9, 40 },
    { 0xf59c30eb, 33 },
    { 0xac9c30ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c30ef, 34 },
    { 0xfdac30f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1c30f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc30f8, 4 },
    { 0xcdcc30f9, 91 },
    { 0xe10c30fa, 43 },
    { 0, 0 },
    { 0xb48c30fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8c30ff, 43 },
    { 0x68dc30ff, 90 },
    { 0, 0 },
    { 0x888c3102, 95 },
    { 0xcffc3103, 34 },
    { 0x908c3104, 67 },
    { 0xb1c3105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec3108, 95 },
    { 0xdb6c3109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3c310c, 66 },
    { 0xc96c310d, 98 },
    { 0x43fc310e, 76 },
    { 0x16ac310e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec3112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c3122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcc3126, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c312f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3c3136, 76 },
    { 0, 0 },
    { 0x69ec3138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc313e, 76 },
    { 0x550c313e, 93 },
    { 0x99bc3140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956c3145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c3148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c314f, 33 },
    { 0x28ec3150, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978c3157, 55 },
    { 0xb74c3158, 91 },
    { 0x6afc3159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fc3161, 95 },
    { 0, 0 },
    { 0xee1c3163, 98 },
    { 0x14bc3164, 33 },
    { 0xff6c3165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79c316d, 36 },
    { 0x804c316e, 61 },
    { 0xc2ec316e, 95 },
    { 0xfd9c316e, 43 },
    { 0xbe4c3171, 52 },
    { 0xc7ec3171, 23 },
    { 0x15cc3170, 23 },
    { 0xbfec316f, 90 },
    { 0x87fc316d, 85 },
    { 0, 0 },
    { 0xf97c3177, 84 },
    { 0x167c3177, 99 },
    { 0xaefc3179, 23 },
    { 0x389c3178, 77 },
    { 0x85dc317b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c317e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc3187, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686c318c, 90 },
    { 0, 0 },
    { 0xa87c318e, 23 },
    { 0, 0 },
    { 0x893c3190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32c3194, 61 },
    { 0xb9cc3194, 61 },
    { 0xa54c3196, 91 },
    { 0x843c3197, 23 },
    { 0xb4dc3197, 90 },
    { 0x958c3199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307c319f, 100 },
    { 0x8f1c31a0, 91 },
    { 0xcbac31a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc31a6, 87 },
    { 0x6cec31a7, 98 },
    { 0x697c31a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c31ac, 23 },
    { 0x1bfc31ad, 61 },
    { 0, 0 },
    { 0xc61c31af, 76 },
    { 0, 0 },
    { 0x330c31b1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0c31b5, 90 },
    { 0x690c31b6, 90 },
    { 0, 0 },
    { 0xcb2c31b8, 90 },
    { 0, 0 },
    { 0x301c31ba, 33 },
    { 0xad2c31bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc31be, 88 },
    { 0x793c31bf, 33 },
    { 0x21c31c0, 43 },
    { 0xf3ec31c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4c31c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eec31c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7c31cc, 33 },
    { 0, 0 },
    { 0xc13c31ce, 53 },
    { 0x195c31ce, 93 },
    { 0, 0 },
    { 0x6ebc31d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0c31d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45c31d7, 95 },
    { 0xddbc31d8, 33 },
    { 0x5bbc31d8, 98 },
    { 0xb12c31da, 23 },
    { 0, 0 },
    { 0xcccc31dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc31e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c31e7, 99 },
    { 0, 0 },
    { 0x1dbc31e9, 95 },
    { 0x1fcc31ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c31ed, 90 },
    { 0xedc31ee, 33 },
    { 0x61ac31ef, 75 },
    { 0x56fc31f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34c31f4, 95 },
    { 0xfa1c31f5, 61 },
    { 0xde2c31f5, 34 },
    { 0xd27c31f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c31fb, 46 },
    { 0xe3c31fb, 33 },
    { 0xfddc31fd, 91 },
    { 0x969c31fe, 36 },
    { 0x836c31fe, 98 },
    { 0, 0 },
    { 0x514c3201, 33 },
    { 0x5f2c3202, 91 },
    { 0x7d6c3202, 90 },
    { 0x8e5c3202, 66 },
    { 0x9cfc3205, 23 },
    { 0x6e2c3206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48c320d, 34 },
    { 0xb53c320d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcc3214, 65 },
    { 0, 0 },
    { 0x710c3216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cc3221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcac3225, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c322a, 36 },
    { 0x777c322a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc322e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fc3232, 90 },
    { 0x150c3233, 61 },
    { 0, 0 },
    { 0x97dc3235, 90 },
    { 0xb7dc3236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4c323a, 90 },
    { 0xf23c323b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94c3240, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c3247, 61 },
    { 0, 0 },
    { 0x950c3249, 76 },
    { 0, 0 },
    { 0x736c324b, 67 },
    { 0x489c324b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356c3250, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ac3257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c325a, 23 },
    { 0xf2cc325a, 90 },
    { 0xb43c325c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c3263, 17 },
    { 0x1d0c3264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797c3271, 90 },
    { 0x926c3272, 98 },
    { 0xd57c3272, 33 },
    { 0x469c3272, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c3278, 33 },
    { 0, 0 },
    { 0x5a2c327a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c327f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c3283, 90 },
    { 0xdc9c3284, 33 },
    { 0x72bc3285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613c3291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec3294, 93 },
    { 0x9c2c3295, 90 },
    { 0xc00c3294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670c329e, 90 },
    { 0xa63c329f, 55 },
    { 0xcf1c329f, 61 },
    { 0, 0 },
    { 0xa08c32a2, 69 },
    { 0xde2c32a3, 33 },
    { 0x7bcc32a4, 23 },
    { 0xf72c32a5, 61 },
    { 0x85ec32a5, 90 },
    { 0x20fc32a7, 65 },
    { 0x329c32a6, 55 },
    { 0, 0 },
    { 0x322c32aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c32b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c32b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2c32b8, 76 },
    { 0xf55c32b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac32bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80c32c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5c32c5, 61 },
    { 0, 0 },
    { 0x928c32c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cc32cc, 33 },
    { 0x380c32cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c32d0, 52 },
    { 0x436c32d0, 101 },
    { 0x9eac32d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7c32d7, 43 },
    { 0, 0 },
    { 0xa70c32d9, 98 },
    { 0xf57c32da, 95 },
    { 0x35dc32db, 55 },
    { 0x8a2c32dc, 33 },
    { 0xb77c32dd, 90 },
    { 0x12bc32de, 90 },
    { 0x839c32df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692c32e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bc32e8, 46 },
    { 0x771c32e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c32ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x797c32f1, 90 },
    { 0xb5fc32f1, 33 },
    { 0, 0 },
    { 0x7d7c32f4, 90 },
    { 0x1a0c32f5, 77 },
    { 0x5ec32f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bc32f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8c32fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc3302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8c3308, 66 },
    { 0, 0 },
    { 0x8bc330a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3310, 84 },
    { 0x489c3311, 34 },
    { 0x7eec3311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fc3318, 23 },
    { 0, 0 },
    { 0x345c331a, 61 },
    { 0x33dc331b, 95 },
    { 0x652c331c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2c3320, 90 },
    { 0x411c3321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9c3324, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cc3327, 78 },
    { 0xc04c3328, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c332b, 76 },
    { 0x6f5c332b, 23 },
    { 0xc4ac332b, 93 },
    { 0xa0cc332e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c3331, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04c3335, 90 },
    { 0, 0 },
    { 0x230c3337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5c333c, 76 },
    { 0x82c333d, 98 },
    { 0, 0 },
    { 0xbd0c333f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911c3343, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56c3349, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498c334d, 95 },
    { 0x34bc334d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac3358, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c335b, 33 },
    { 0xd9ac335c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c3362, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec3366, 61 },
    { 0x132c3367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fc336d, 67 },
    { 0xa5fc336e, 60 },
    { 0xb48c336f, 55 },
    { 0x4fc3370, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99c3373, 52 },
    { 0xb5ac3373, 23 },
    { 0, 0 },
    { 0xa5ec3376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc3379, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc337c, 61 },
    { 0x422c337d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fc3380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c3383, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59c3387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac338c, 33 },
    { 0xe00c338d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c3390, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbc3393, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3c3396, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c3399, 91 },
    { 0x740c3399, 98 },
    { 0, 0 },
    { 0xf83c339c, 33 },
    { 0x2e9c339d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dc33a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c33a7, 33 },
    { 0x1a3c33a8, 17 },
    { 0, 0 },
    { 0x990c33aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c33ae, 22 },
    { 0, 0 },
    { 0x912c33b0, 43 },
    { 0x2e1c33b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c33b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bec33bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c33c0, 76 },
    { 0xcf1c33c1, 95 },
    { 0x34ec33c2, 77 },
    { 0x865c33c3, 33 },
    { 0xbd6c33c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec33c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c33d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694c33d6, 90 },
    { 0, 0 },
    { 0x107c33d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cc33e6, 90 },
    { 0x1d3c33e7, 81 },
    { 0xf20c33e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85c33ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c33ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc33fa, 55 },
    { 0xd82c33fa, 91 },
    { 0x3dfc33fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c3400, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c3404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4c340a, 98 },
    { 0, 0 },
    { 0xa67c340c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689c3410, 95 },
    { 0xb8ec3411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c3415, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x806c3418, 78 },
    { 0, 0 },
    { 0x22cc341a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdc341d, 84 },
    { 0, 0 },
    { 0xa7cc341f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcc3424, 98 },
    { 0xa18c3424, 90 },
    { 0xb6fc3424, 90 },
    { 0, 0 },
    { 0xb72c3428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7c3432, 99 },
    { 0xe49c3433, 34 },
    { 0xb18c3434, 90 },
    { 0xeb1c3432, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c3439, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7c343f, 43 },
    { 0, 0 },
    { 0x732c3441, 33 },
    { 0, 0 },
    { 0xbe1c3443, 90 },
    { 0x4f0c3444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc3448, 61 },
    { 0, 0 },
    { 0xbb0c344a, 78 },
    { 0, 0 },
    { 0x258c344c, 33 },
    { 0xc36c344c, 95 },
    { 0x1e1c344e, 46 },
    { 0xee0c344e, 61 },
    { 0x37bc344f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x177c3453, 61 },
    { 0x2d1c3454, 68 },
    { 0xc43c3455, 33 },
    { 0xb93c3456, 66 },
    { 0xc5fc3456, 61 },
    { 0x5a8c3458, 91 },
    { 0x82c3458, 55 },
    { 0x452c3458, 38 },
    { 0xe5ec3459, 91 },
    { 0xea7c3459, 95 },
    { 0xb00c3454, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68c3463, 23 },
    { 0, 0 },
    { 0x712c3465, 67 },
    { 0x523c3465, 67 },
    { 0x922c3466, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc346a, 95 },
    { 0x393c346b, 61 },
    { 0x22dc346a, 95 },
    { 0, 0 },
    { 0x164c346e, 77 },
    { 0, 0 },
    { 0x8b9c3470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85c3474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01c347b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280c347f, 90 },
    { 0x916c3480, 98 },
    { 0x172c3480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c3484, 95 },
    { 0xc79c3484, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c3488, 78 },
    { 0xcddc3489, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcc348d, 93 },
    { 0, 0 },
    { 0xfe6c348f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c3492, 46 },
    { 0x2fc3492, 23 },
    { 0xf7cc3494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3c349b, 61 },
    { 0xfa5c349b, 61 },
    { 0, 0 },
    { 0x3cc349e, 33 },
    { 0x25c349f, 98 },
    { 0xf5ec34a0, 88 },
    { 0, 0 },
    { 0xc02c34a2, 83 },
    { 0xcdc34a3, 95 },
    { 0, 0 },
    { 0x1fec34a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887c34ab, 40 },
    { 0, 0 },
    { 0xd96c34ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c34b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecc34b4, 52 },
    { 0x70ec34b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5c34bd, 61 },
    { 0xfd2c34be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c34c5, 43 },
    { 0, 0 },
    { 0xdc8c34c7, 85 },
    { 0, 0 },
    { 0xac0c34c9, 43 },
    { 0xb7cc34c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29c34d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc34dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dc34e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecc34e6, 61 },
    { 0xdcfc34e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c34ef, 95 },
    { 0, 0 },
    { 0xf7ac34f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fc34f5, 95 },
    { 0xd22c34f6, 52 },
    { 0xe2bc34f7, 76 },
    { 0x24dc34f7, 33 },
    { 0, 0 },
    { 0x77fc34fa, 77 },
    { 0xfec34fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eec3506, 23 },
    { 0xc4cc3507, 95 },
    { 0xfec3507, 76 },
    { 0x60ec3509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fc350e, 55 },
    { 0xf0bc350e, 17 },
    { 0x2cac3510, 23 },
    { 0xde1c3511, 95 },
    { 0x4cbc3512, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1c3519, 76 },
    { 0, 0 },
    { 0x682c351b, 99 },
    { 0x4c1c351c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc3521, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc3525, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec3528, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4c352b, 99 },
    { 0x96ac352c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcc352f, 95 },
    { 0x9b8c3530, 85 },
    { 0x494c3531, 67 },
    { 0, 0 },
    { 0x637c3533, 85 },
    { 0x8e6c3534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15c3537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c353a, 78 },
    { 0xb7c353b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52c3543, 93 },
    { 0xfd1c3543, 46 },
    { 0x2fbc3545, 90 },
    { 0xd69c3546, 84 },
    { 0xc70c3547, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc354a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadc354e, 33 },
    { 0x80ec354e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3555, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc3559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c355f, 52 },
    { 0, 0 },
    { 0xf8c3561, 76 },
    { 0xb1c3561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44c3568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c356b, 95 },
    { 0xcc6c356c, 61 },
    { 0, 0 },
    { 0x3cfc356e, 90 },
    { 0x61cc356e, 90 },
    { 0xcc6c3570, 33 },
    { 0, 0 },
    { 0x94cc3572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24c3575, 66 },
    { 0x9d2c3576, 90 },
    { 0xf33c3575, 67 },
    { 0x9c5c3576, 34 },
    { 0x15cc3578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x696c3580, 93 },
    { 0xc95c3581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbc3584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfc3588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fc358b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c3596, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443c359a, 95 },
    { 0x695c359b, 90 },
    { 0x92fc359c, 84 },
    { 0x757c359d, 66 },
    { 0x5c7c359e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ac35a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc35ac, 90 },
    { 0x498c35ad, 55 },
    { 0x1f6c35ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c35b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeec35be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3c35c2, 98 },
    { 0x716c35c2, 90 },
    { 0x8c0c35c2, 90 },
    { 0x55c35c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cc35c8, 93 },
    { 0, 0 },
    { 0x561c35ca, 61 },
    { 0x5f2c35cb, 66 },
    { 0x943c35ca, 69 },
    { 0x2f8c35cd, 75 },
    { 0x43fc35cd, 100 },
    { 0, 0 },
    { 0x901c35d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac35d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cc35d7, 90 },
    { 0, 0 },
    { 0x40c35d9, 33 },
    { 0xa0fc35da, 55 },
    { 0xfa9c35db, 90 },
    { 0x450c35dc, 33 },
    { 0xf8dc35db, 55 },
    { 0x920c35db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c35e4, 61 },
    { 0x961c35e4, 90 },
    { 0x505c35e6, 99 },
    { 0, 0 },
    { 0xd1c35e8, 33 },
    { 0x86ac35e9, 33 },
    { 0xa0cc35ea, 43 },
    { 0x294c35eb, 55 },
    { 0x2dbc35e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c35f5, 56 },
    { 0x437c35f6, 95 },
    { 0xe5cc35f6, 98 },
    { 0xb36c35f6, 61 },
    { 0x510c35f5, 76 },
    { 0xbf2c35fa, 84 },
    { 0xa15c35f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c35fe, 65 },
    { 0xa49c35ff, 33 },
    { 0xc43c3600, 23 },
    { 0, 0 },
    { 0x33dc3602, 85 },
    { 0x357c3603, 100 },
    { 0xfd0c3604, 76 },
    { 0xd29c3603, 43 },
    { 0, 0 },
    { 0xd45c3607, 33 },
    { 0xe1cc3608, 87 },
    { 0xf59c3607, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fc360e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fc3611, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cc3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833c3619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x347c361c, 34 },
    { 0, 0 },
    { 0xd8fc361e, 90 },
    { 0, 0 },
    { 0xdc7c3620, 90 },
    { 0x9a8c3621, 76 },
    { 0xc45c3622, 23 },
    { 0xab0c3622, 92 },
    { 0xdecc3624, 52 },
    { 0, 0 },
    { 0xae9c3626, 33 },
    { 0, 0 },
    { 0xf7cc3628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ac362c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fc362f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x624c3632, 98 },
    { 0xccbc3633, 84 },
    { 0xc3dc3634, 78 },
    { 0xcb8c3632, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c363c, 98 },
    { 0xc42c363d, 100 },
    { 0x62ac363e, 90 },
    { 0, 0 },
    { 0x5acc3640, 98 },
    { 0x335c3640, 61 },
    { 0xd24c3641, 66 },
    { 0x248c3643, 99 },
    { 0, 0 },
    { 0x750c3645, 56 },
    { 0x754c3645, 43 },
    { 0, 0 },
    { 0xf49c3648, 100 },
    { 0x8e9c3648, 33 },
    { 0xbf1c364a, 23 },
    { 0, 0 },
    { 0x458c364c, 98 },
    { 0, 0 },
    { 0x3bac364e, 33 },
    { 0x91ec364e, 61 },
    { 0x3c2c3650, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c3653, 90 },
    { 0xae8c3654, 84 },
    { 0, 0 },
    { 0x84ec3656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c365a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c365d, 84 },
    { 0, 0 },
    { 0xf2dc365f, 77 },
    { 0xa02c365f, 66 },
    { 0, 0 },
    { 0x644c3662, 46 },
    { 0x48dc3663, 61 },
    { 0xa34c3664, 23 },
    { 0x8c3c3665, 61 },
    { 0, 0 },
    { 0xd27c3667, 90 },
    { 0, 0 },
    { 0xacec3669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c366f, 43 },
    { 0x61c3670, 76 },
    { 0x797c3670, 90 },
    { 0x3bac3672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ec3676, 33 },
    { 0x375c3677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dac367a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cc368a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc368d, 81 },
    { 0x54c368d, 23 },
    { 0, 0 },
    { 0x861c3690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8c3693, 78 },
    { 0xd14c3694, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547c369b, 76 },
    { 0xc08c369b, 98 },
    { 0x9eec369d, 85 },
    { 0x734c369c, 23 },
    { 0xd1bc369c, 23 },
    { 0x41fc369e, 90 },
    { 0x961c36a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac36a6, 33 },
    { 0, 0 },
    { 0x4d4c36a8, 76 },
    { 0, 0 },
    { 0x484c36aa, 23 },
    { 0, 0 },
    { 0x212c36ac, 61 },
    { 0x807c36ac, 91 },
    { 0x806c36ad, 34 },
    { 0x4edc36af, 61 },
    { 0xdddc36af, 90 },
    { 0, 0 },
    { 0x5ccc36b2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337c36b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac36bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878c36ca, 46 },
    { 0x31dc36cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c36d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dc36d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29c36da, 90 },
    { 0x1c1c36db, 80 },
    { 0x341c36db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c36e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fc36e3, 33 },
    { 0, 0 },
    { 0x46cc36e5, 90 },
    { 0, 0 },
    { 0x79bc36e7, 23 },
    { 0x29dc36e8, 43 },
    { 0, 0 },
    { 0x2f9c36ea, 61 },
    { 0xa91c36eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cc36ee, 33 },
    { 0, 0 },
    { 0x1cfc36f0, 91 },
    { 0, 0 },
    { 0xb30c36f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8c36f5, 61 },
    { 0x365c36f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585c36fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c36ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec3702, 69 },
    { 0xab3c3702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bc3707, 52 },
    { 0, 0 },
    { 0x6a9c3709, 90 },
    { 0, 0 },
    { 0x280c370b, 76 },
    { 0xd47c370c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c370f, 95 },
    { 0xb22c370f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdc3714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfc3718, 90 },
    { 0xbd8c3719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45c3720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717c3728, 56 },
    { 0, 0 },
    { 0xb76c372a, 23 },
    { 0x1b0c372b, 68 },
    { 0x720c372a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc372f, 46 },
    { 0xeb7c3730, 33 },
    { 0x97ec3731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc3734, 95 },
    { 0xf0fc3734, 61 },
    { 0xb2fc3736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc373b, 33 },
    { 0x5a8c373c, 61 },
    { 0, 0 },
    { 0x152c373e, 90 },
    { 0, 0 },
    { 0xe84c3740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8c3748, 33 },
    { 0x660c3749, 61 },
    { 0x9a1c3749, 90 },
    { 0, 0 },
    { 0x663c374c, 33 },
    { 0x9aec374d, 90 },
    { 0xca5c374e, 23 },
    { 0, 0 },
    { 0x460c3750, 23 },
    { 0xa0c3751, 90 },
    { 0xe7c3751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c3755, 95 },
    { 0x99c3755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c375a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec375e, 78 },
    { 0x9b9c375f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77c376c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ec3772, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc3775, 33 },
    { 0x8a5c3776, 33 },
    { 0, 0 },
    { 0xbecc3778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac377e, 66 },
    { 0xb0cc377e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c3784, 93 },
    { 0x7c8c3785, 90 },
    { 0xadac3786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cac378a, 33 },
    { 0xd64c378a, 84 },
    { 0x5a2c378c, 98 },
    { 0x19fc378c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98c3792, 61 },
    { 0x54ec3793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1c3799, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2c379c, 95 },
    { 0, 0 },
    { 0x916c379e, 90 },
    { 0x410c379e, 66 },
    { 0, 0 },
    { 0xec2c37a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ac37a7, 52 },
    { 0x107c37a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x178c37ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51c37ae, 61 },
    { 0xaa4c37ae, 90 },
    { 0xdd5c37ae, 76 },
    { 0x3d1c37b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c37b6, 61 },
    { 0xe8cc37b6, 33 },
    { 0x603c37b8, 46 },
    { 0xef9c37b9, 90 },
    { 0xff8c37ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc37bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c37c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c37c3, 90 },
    { 0x258c37c3, 33 },
    { 0x52cc37c4, 95 },
    { 0x9f0c37c6, 61 },
    { 0x5a3c37c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cc37ca, 78 },
    { 0xd19c37cb, 33 },
    { 0xf2dc37cc, 78 },
    { 0xc4c37cd, 90 },
    { 0, 0 },
    { 0xcd3c37cf, 23 },
    { 0x96ac37d0, 46 },
    { 0, 0 },
    { 0x4a9c37d2, 90 },
    { 0x74dc37d3, 23 },
    { 0x928c37d4, 76 },
    { 0x285c37d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42c37da, 98 },
    { 0xc65c37da, 85 },
    { 0x6b3c37da, 46 },
    { 0, 0 },
    { 0x953c37de, 66 },
    { 0xf14c37df, 90 },
    { 0x4cec37de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0c37e6, 95 },
    { 0x121c37e7, 43 },
    { 0x53dc37e8, 52 },
    { 0xfc3c37e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2c37ed, 93 },
    { 0x25dc37ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc37f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c37fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c3803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c3806, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbc380a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c380d, 76 },
    { 0xfaec380d, 61 },
    { 0, 0 },
    { 0x173c3810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3c3814, 23 },
    { 0x378c3815, 33 },
    { 0xe65c3815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c381c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c3822, 98 },
    { 0x811c3823, 98 },
    { 0x4bec3824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66c3828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec382d, 85 },
    { 0x576c382e, 23 },
    { 0, 0 },
    { 0x3f8c3830, 76 },
    { 0x406c3830, 61 },
    { 0, 0 },
    { 0xa3ec3833, 77 },
    { 0, 0 },
    { 0xfb0c3835, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c3838, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613c383d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9c3841, 33 },
    { 0xcfbc3842, 90 },
    { 0xb2dc3842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846c3847, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c384a, 67 },
    { 0xf1cc384a, 76 },
    { 0xc36c384a, 95 },
    { 0x5a5c384b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354c3852, 98 },
    { 0x7cbc3852, 43 },
    { 0x887c3853, 87 },
    { 0, 0 },
    { 0x289c3856, 90 },
    { 0x826c3857, 23 },
    { 0x3ecc3858, 61 },
    { 0xe48c3859, 95 },
    { 0, 0 },
    { 0x179c385b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec385e, 95 },
    { 0xa99c385f, 61 },
    { 0x3aac385f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16c3863, 81 },
    { 0x47ec3864, 78 },
    { 0xf92c3865, 23 },
    { 0x8eac3864, 33 },
    { 0xa4bc3866, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec386a, 67 },
    { 0, 0 },
    { 0xbeac386c, 23 },
    { 0xc13c386d, 61 },
    { 0, 0 },
    { 0x6b7c386f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c3873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6c3876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bc3879, 23 },
    { 0, 0 },
    { 0x941c387b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc387f, 78 },
    { 0x56ec3880, 61 },
    { 0xb13c3881, 64 },
    { 0x8c3881, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979c3887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223c388b, 61 },
    { 0, 0 },
    { 0x7aec388d, 61 },
    { 0xa6bc388e, 61 },
    { 0xcf8c388e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec3892, 100 },
    { 0, 0 },
    { 0x213c3894, 23 },
    { 0xb92c3895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acc3898, 34 },
    { 0xc02c3899, 61 },
    { 0, 0 },
    { 0x575c389b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c389e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc38a5, 98 },
    { 0, 0 },
    { 0x867c38a7, 23 },
    { 0x10ac38a8, 95 },
    { 0xef8c38a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc38b0, 23 },
    { 0x657c38b1, 90 },
    { 0x8a9c38b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c38b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc38be, 100 },
    { 0x528c38be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ac38c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aac38cd, 93 },
    { 0x66ac38ce, 61 },
    { 0xfacc38cd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79c38d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c38d9, 76 },
    { 0xab3c38da, 95 },
    { 0, 0 },
    { 0x563c38dc, 67 },
    { 0, 0 },
    { 0x278c38de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aac38e2, 55 },
    { 0x565c38e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c38e6, 43 },
    { 0x9bbc38e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abc38ea, 75 },
    { 0, 0 },
    { 0x4d6c38ec, 95 },
    { 0xd5cc38ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c38f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c38f7, 61 },
    { 0, 0 },
    { 0x391c38f9, 98 },
    { 0x46fc38f9, 23 },
    { 0x8aec38fb, 61 },
    { 0x90cc38fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc3900, 55 },
    { 0x2eec3900, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0c3904, 33 },
    { 0, 0 },
    { 0x499c3906, 77 },
    { 0xf90c3907, 61 },
    { 0xeacc3907, 61 },
    { 0xa3cc3909, 53 },
    { 0x3b2c3908, 95 },
    { 0x822c390b, 43 },
    { 0xc48c390b, 61 },
    { 0x141c390c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fec3912, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4c3916, 85 },
    { 0xa92c3917, 23 },
    { 0xf20c3916, 65 },
    { 0xf27c3919, 61 },
    { 0x2cec3919, 61 },
    { 0x964c391b, 98 },
    { 0x449c3919, 55 },
    { 0xa1bc391d, 76 },
    { 0x2ddc391c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2c3921, 33 },
    { 0x25ec3921, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c3926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c392a, 77 },
    { 0, 0 },
    { 0x8c2c392c, 61 },
    { 0xd55c392d, 33 },
    { 0xc1ec392e, 76 },
    { 0x61fc392c, 87 },
    { 0xb79c392e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c3934, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c3939, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c393d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c3940, 23 },
    { 0, 0 },
    { 0x417c3942, 33 },
    { 0x73ac3942, 83 },
    { 0xdf5c3943, 43 },
    { 0x2e3c3944, 75 },
    { 0xa82c3946, 61 },
    { 0x8fbc3947, 52 },
    { 0xe59c3948, 93 },
    { 0, 0 },
    { 0xe0ec394a, 98 },
    { 0x1e0c394b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c394e, 98 },
    { 0, 0 },
    { 0xf4cc3950, 23 },
    { 0x5dbc3950, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768c395b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5c3960, 90 },
    { 0, 0 },
    { 0xd0bc3962, 55 },
    { 0x69cc3963, 98 },
    { 0, 0 },
    { 0xabec3965, 55 },
    { 0, 0 },
    { 0x8b9c3967, 33 },
    { 0x493c3968, 95 },
    { 0xbd6c3968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdc396c, 61 },
    { 0, 0 },
    { 0x1f1c396e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cc3971, 90 },
    { 0, 0 },
    { 0x65cc3973, 90 },
    { 0xd5fc3974, 61 },
    { 0x95bc3973, 91 },
    { 0, 0 },
    { 0x3f7c3977, 90 },
    { 0xbbdc3977, 87 },
    { 0xe66c3979, 75 },
    { 0x9a6c3979, 55 },
    { 0xf1bc397b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642c397f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656c3983, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c3987, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfc398a, 53 },
    { 0, 0 },
    { 0x44c398c, 46 },
    { 0xf45c398d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ac3990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bc3999, 91 },
    { 0x8c4c3999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x545c399d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdac39a5, 99 },
    { 0, 0 },
    { 0xb35c39a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecc39aa, 61 },
    { 0x662c39aa, 76 },
    { 0, 0 },
    { 0x33c39ad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446c39b1, 33 },
    { 0, 0 },
    { 0x5dcc39b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c39c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x645c39c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884c39d0, 68 },
    { 0xf90c39d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc39d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cc39d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x252c39dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ec39e2, 43 },
    { 0xa83c39e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c39e9, 33 },
    { 0xf79c39ea, 23 },
    { 0x4b2c39e9, 96 },
    { 0xc08c39ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc39f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cc39fd, 34 },
    { 0x7a5c39fe, 33 },
    { 0x232c39ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c3a02, 66 },
    { 0xf42c3a03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x183c3a06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edc3a09, 34 },
    { 0, 0 },
    { 0x2dfc3a0b, 23 },
    { 0x19c3a0c, 33 },
    { 0xc68c3a0b, 66 },
    { 0, 0 },
    { 0xfd8c3a0f, 69 },
    { 0x738c3a10, 93 },
    { 0x2b6c3a11, 98 },
    { 0xa41c3a12, 81 },
    { 0xe05c3a13, 95 },
    { 0x1e2c3a13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35c3a17, 61 },
    { 0xd23c3a18, 46 },
    { 0xca5c3a19, 76 },
    { 0x4adc3a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac3a1e, 99 },
    { 0x410c3a1f, 94 },
    { 0x611c3a1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc3a24, 98 },
    { 0xe93c3a25, 90 },
    { 0xc21c3a25, 93 },
    { 0x33cc3a26, 61 },
    { 0xfb1c3a28, 61 },
    { 0x722c3a27, 61 },
    { 0xff2c3a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c3a2d, 78 },
    { 0, 0 },
    { 0xa87c3a2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3c3a37, 34 },
    { 0xd7fc3a37, 98 },
    { 0xabcc3a39, 61 },
    { 0, 0 },
    { 0x961c3a3b, 95 },
    { 0x488c3a3c, 33 },
    { 0x20c3a3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ec3a41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c3a44, 23 },
    { 0xb0dc3a45, 91 },
    { 0x920c3a45, 33 },
    { 0x130c3a46, 33 },
    { 0x3a6c3a47, 61 },
    { 0, 0 },
    { 0x93fc3a4a, 69 },
    { 0x1d7c3a4a, 55 },
    { 0xcc3a4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ac3a53, 98 },
    { 0x186c3a53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8c3a57, 87 },
    { 0, 0 },
    { 0x75ac3a59, 95 },
    { 0xb8c3a5a, 17 },
    { 0xbddc3a59, 43 },
    { 0x5eac3a5c, 23 },
    { 0, 0 },
    { 0xd03c3a5e, 95 },
    { 0x14dc3a5f, 23 },
    { 0x802c3a60, 33 },
    { 0x43fc3a61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edc3a65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5c3a6d, 90 },
    { 0x3c1c3a6e, 37 },
    { 0, 0 },
    { 0x686c3a70, 17 },
    { 0, 0 },
    { 0x799c3a72, 99 },
    { 0x9ac3a73, 33 },
    { 0, 0 },
    { 0xc5fc3a75, 61 },
    { 0x455c3a75, 100 },
    { 0x5b2c3a77, 23 },
    { 0x196c3a78, 33 },
    { 0xbe4c3a79, 61 },
    { 0xcc7c3a7a, 77 },
    { 0x2b7c3a7a, 34 },
    { 0x2dbc3a7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ec3a7f, 55 },
    { 0, 0 },
    { 0xa07c3a81, 67 },
    { 0xeb1c3a81, 33 },
    { 0x1dc3a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac3a86, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c3a89, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c3a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79c3a93, 87 },
    { 0xa81c3a93, 61 },
    { 0x25cc3a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcc3a98, 100 },
    { 0, 0 },
    { 0x33c3a9a, 98 },
    { 0xa36c3a9b, 95 },
    { 0, 0 },
    { 0x794c3a9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c3aa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22c3aa5, 68 },
    { 0x6d8c3aa5, 55 },
    { 0xc2ec3aa7, 66 },
    { 0, 0 },
    { 0x2cac3aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c3aae, 34 },
    { 0x457c3aae, 55 },
    { 0x3b5c3ab0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c3ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3c3aba, 90 },
    { 0, 0 },
    { 0x99bc3abc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795c3ac3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c3aca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82c3ace, 76 },
    { 0x8c8c3ace, 33 },
    { 0x2b3c3ad0, 61 },
    { 0, 0 },
    { 0x799c3ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2c3ad8, 98 },
    { 0x835c3ad9, 95 },
    { 0, 0 },
    { 0xbd6c3adb, 90 },
    { 0x6f6c3adb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c3ae1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c3ae5, 61 },
    { 0xdf0c3ae6, 90 },
    { 0xc68c3ae7, 90 },
    { 0x8d7c3ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc3af1, 76 },
    { 0, 0 },
    { 0xefdc3af3, 90 },
    { 0xbe9c3af4, 61 },
    { 0, 0 },
    { 0xa3ec3af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x709c3af9, 43 },
    { 0xf0fc3afa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2c3afd, 61 },
    { 0xd7fc3afd, 33 },
    { 0, 0 },
    { 0xe5ec3b00, 76 },
    { 0x21ac3b01, 84 },
    { 0, 0 },
    { 0x524c3b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301c3b0a, 95 },
    { 0x70ec3b0b, 46 },
    { 0xcb7c3b0b, 67 },
    { 0, 0 },
    { 0xe91c3b0e, 90 },
    { 0, 0 },
    { 0x73dc3b10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c3b15, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc3b1c, 98 },
    { 0, 0 },
    { 0x4cdc3b1e, 52 },
    { 0x38bc3b1f, 78 },
    { 0x220c3b1f, 33 },
    { 0x7a3c3b21, 33 },
    { 0xe90c3b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42c3b26, 66 },
    { 0xaa0c3b27, 99 },
    { 0x8fac3b26, 43 },
    { 0x886c3b29, 90 },
    { 0x65fc3b2a, 55 },
    { 0, 0 },
    { 0x860c3b2c, 43 },
    { 0, 0 },
    { 0xbf0c3b2e, 61 },
    { 0xde8c3b2e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfec3b34, 65 },
    { 0x591c3b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c3b3c, 90 },
    { 0x80fc3b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774c3b40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3b43, 61 },
    { 0xe16c3b43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffc3b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c3b4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc3b50, 69 },
    { 0xfe4c3b51, 90 },
    { 0x89ec3b51, 84 },
    { 0, 0 },
    { 0x231c3b54, 33 },
    { 0xc12c3b54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81c3b5a, 87 },
    { 0x12ac3b5b, 43 },
    { 0xa44c3b5a, 78 },
    { 0, 0 },
    { 0xc66c3b5e, 95 },
    { 0xaa6c3b5f, 95 },
    { 0, 0 },
    { 0x675c3b61, 23 },
    { 0x34fc3b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc3b65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bc3b69, 85 },
    { 0xd69c3b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c3b6d, 61 },
    { 0, 0 },
    { 0x9e9c3b6f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc3b72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69c3b77, 83 },
    { 0x570c3b78, 67 },
    { 0x2f5c3b78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c3b7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1dc3b81, 67 },
    { 0x577c3b82, 46 },
    { 0xdf4c3b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c3b86, 95 },
    { 0x622c3b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dac3b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617c3b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c3b9c, 61 },
    { 0x59fc3b9d, 98 },
    { 0, 0 },
    { 0x574c3b9f, 96 },
    { 0, 0 },
    { 0x848c3ba1, 23 },
    { 0, 0 },
    { 0x9aec3ba3, 99 },
    { 0x480c3ba4, 76 },
    { 0x67ac3ba5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3bad, 61 },
    { 0x958c3bae, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdc3bb1, 93 },
    { 0x6f4c3bb2, 55 },
    { 0x99ec3bb3, 78 },
    { 0xfcc3bb4, 95 },
    { 0x276c3bb4, 80 },
    { 0x481c3bb6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199c3bbc, 76 },
    { 0x1d5c3bbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ec3bc0, 34 },
    { 0x72c3bc1, 52 },
    { 0, 0 },
    { 0xca7c3bc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec3bcb, 81 },
    { 0x833c3bcc, 90 },
    { 0xda6c3bcd, 61 },
    { 0xb01c3bcd, 90 },
    { 0xb32c3bce, 75 },
    { 0xeabc3bd0, 61 },
    { 0x900c3bd0, 90 },
    { 0xe3dc3bd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97c3bd6, 61 },
    { 0x94ec3bd6, 90 },
    { 0xa9fc3bd6, 78 },
    { 0, 0 },
    { 0xd07c3bda, 43 },
    { 0xdfbc3bda, 90 },
    { 0x496c3bdc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c3be0, 55 },
    { 0, 0 },
    { 0xbcc3be2, 37 },
    { 0x810c3be3, 98 },
    { 0x1d1c3be3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c3be9, 43 },
    { 0, 0 },
    { 0xc61c3beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x117c3bee, 33 },
    { 0x267c3bef, 61 },
    { 0xa46c3bf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938c3bf6, 76 },
    { 0x149c3bf7, 33 },
    { 0x76c3bf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bc3bfb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2c3bfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04c3c01, 90 },
    { 0, 0 },
    { 0x836c3c03, 23 },
    { 0, 0 },
    { 0xdd0c3c05, 61 },
    { 0x145c3c06, 61 },
    { 0xac6c3c06, 93 },
    { 0xf59c3c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57c3c0e, 33 },
    { 0x399c3c0e, 33 },
    { 0x940c3c10, 69 },
    { 0, 0 },
    { 0x29fc3c12, 99 },
    { 0x875c3c13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc3c17, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c3c23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c3c2b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc3c34, 99 },
    { 0x191c3c35, 95 },
    { 0x731c3c36, 93 },
    { 0x62cc3c37, 81 },
    { 0xd9dc3c36, 46 },
    { 0x699c3c34, 33 },
    { 0xe3fc3c3a, 46 },
    { 0xec8c3c3b, 90 },
    { 0x72ec3c3c, 97 },
    { 0xe88c3c3c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac3c41, 91 },
    { 0, 0 },
    { 0x68c3c43, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c3c50, 23 },
    { 0, 0 },
    { 0xdd8c3c52, 91 },
    { 0, 0 },
    { 0x3c2c3c54, 87 },
    { 0, 0 },
    { 0x7fbc3c56, 98 },
    { 0, 0 },
    { 0x3cac3c58, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c3c63, 52 },
    { 0x9bec3c63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ac3c68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0c3c6b, 55 },
    { 0, 0 },
    { 0x764c3c6d, 78 },
    { 0x304c3c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c3c71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54c3c7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cc3c7d, 55 },
    { 0, 0 },
    { 0xb65c3c7f, 90 },
    { 0x431c3c80, 76 },
    { 0x5ccc3c7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c3c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c3c89, 98 },
    { 0x14bc3c8a, 84 },
    { 0x3d4c3c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3c8e, 90 },
    { 0x410c3c8f, 95 },
    { 0xcfac3c90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ac3c93, 76 },
    { 0xeb5c3c94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x919c3c97, 85 },
    { 0x75c3c97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c3ca0, 66 },
    { 0, 0 },
    { 0xae0c3ca2, 90 },
    { 0x8d4c3ca2, 77 },
    { 0xd33c3ca3, 95 },
    { 0x9d2c3ca5, 95 },
    { 0, 0 },
    { 0x65fc3ca7, 87 },
    { 0xa09c3ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfc3cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8c3cb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c3cbe, 100 },
    { 0, 0 },
    { 0xcdfc3cc0, 33 },
    { 0xdfc3cc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc3cc5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7c3cc8, 43 },
    { 0x471c3cc9, 64 },
    { 0, 0 },
    { 0xfeac3ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac3cce, 43 },
    { 0x4a4c3ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c3cd4, 34 },
    { 0, 0 },
    { 0xb96c3cd6, 95 },
    { 0, 0 },
    { 0xad3c3cd8, 95 },
    { 0x45cc3cd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605c3cdd, 61 },
    { 0, 0 },
    { 0xd20c3cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68c3ce3, 83 },
    { 0, 0 },
    { 0x3fac3ce5, 33 },
    { 0xdec3ce6, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c3cec, 43 },
    { 0, 0 },
    { 0x4dc3cee, 75 },
    { 0xfd4c3cef, 55 },
    { 0x1d5c3cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc3cf4, 23 },
    { 0x7b5c3cf4, 67 },
    { 0xb9cc3cf5, 78 },
    { 0xa2dc3cf5, 61 },
    { 0x979c3cf8, 75 },
    { 0xda6c3cf9, 43 },
    { 0x627c3cf9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ec3d00, 95 },
    { 0x4f5c3d01, 61 },
    { 0xc9ec3d02, 33 },
    { 0x414c3d03, 95 },
    { 0x7f6c3d04, 33 },
    { 0xe30c3d03, 61 },
    { 0x2fcc3d03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86c3d09, 90 },
    { 0, 0 },
    { 0xfd1c3d0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c3d11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc3d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc3d19, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc3d24, 61 },
    { 0x74cc3d25, 33 },
    { 0, 0 },
    { 0x58dc3d27, 95 },
    { 0x2ac3d28, 23 },
    { 0x700c3d27, 23 },
    { 0, 0 },
    { 0xfc4c3d2b, 77 },
    { 0x996c3d2b, 93 },
    { 0xc1c3d2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c3d30, 68 },
    { 0x91ec3d31, 90 },
    { 0x79c3d31, 34 },
    { 0x521c3d31, 90 },
    { 0x28bc3d34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bc3d38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c3d3c, 93 },
    { 0, 0 },
    { 0xc51c3d3e, 100 },
    { 0x416c3d3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c3d43, 95 },
    { 0, 0 },
    { 0xe43c3d45, 55 },
    { 0, 0 },
    { 0x442c3d47, 34 },
    { 0, 0 },
    { 0x254c3d49, 90 },
    { 0xa5fc3d4a, 46 },
    { 0x3fbc3d4b, 23 },
    { 0x58ec3d4a, 73 },
    { 0x71dc3d4a, 99 },
    { 0x46fc3d4e, 84 },
    { 0, 0 },
    { 0x2c1c3d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13c3d57, 98 },
    { 0, 0 },
    { 0xcbac3d59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87c3d5d, 87 },
    { 0xfa5c3d5d, 61 },
    { 0, 0 },
    { 0xac4c3d60, 98 },
    { 0x11cc3d61, 72 },
    { 0, 0 },
    { 0xe6ac3d63, 91 },
    { 0x720c3d63, 90 },
    { 0x305c3d64, 81 },
    { 0x9b8c3d66, 61 },
    { 0xfa1c3d67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28c3d6b, 90 },
    { 0x376c3d6c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c3d6f, 34 },
    { 0, 0 },
    { 0x79cc3d71, 46 },
    { 0, 0 },
    { 0x335c3d73, 98 },
    { 0x2d6c3d74, 46 },
    { 0xa1dc3d74, 23 },
    { 0xc52c3d76, 98 },
    { 0x3bfc3d77, 34 },
    { 0x91bc3d75, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec3d7b, 90 },
    { 0, 0 },
    { 0x9cac3d7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150c3d81, 98 },
    { 0x82c3d82, 23 },
    { 0xf8ac3d83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc3d8a, 90 },
    { 0x944c3d8b, 91 },
    { 0x5a2c3d8a, 23 },
    { 0x8e5c3d8a, 95 },
    { 0x32ec3d8e, 93 },
    { 0x31c3d8e, 55 },
    { 0x231c3d8c, 23 },
    { 0xef8c3d8b, 33 },
    { 0x62bc3d8f, 76 },
    { 0x5ac3d93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bc3d96, 52 },
    { 0xaa6c3d97, 76 },
    { 0x1a9c3d96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c3d9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ec3d9f, 17 },
    { 0xcfc3da0, 42 },
    { 0x5eac3da1, 61 },
    { 0x33ec3da2, 84 },
    { 0x37c3da2, 76 },
    { 0x35cc3da2, 61 },
    { 0xb5ac3da3, 53 },
    { 0, 0 },
    { 0x655c3da7, 67 },
    { 0x1d6c3da8, 87 },
    { 0xceec3da8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4c3dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4c3db0, 46 },
    { 0xe8ec3db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dc3db4, 76 },
    { 0xb2dc3db4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac3dbc, 67 },
    { 0xd1ec3dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7c3dc6, 76 },
    { 0x257c3dc7, 90 },
    { 0x84c3dc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93c3dcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c3dcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c3ddd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134c3de3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacec3de8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39c3dec, 17 },
    { 0xdafc3ded, 98 },
    { 0x567c3ded, 98 },
    { 0x8bcc3def, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c3df7, 61 },
    { 0x53c3df8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c3dfb, 98 },
    { 0x285c3dfc, 100 },
    { 0x1ffc3dfc, 95 },
    { 0x3bc3dfc, 55 },
    { 0xe1dc3dfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c3e05, 23 },
    { 0, 0 },
    { 0x369c3e07, 33 },
    { 0, 0 },
    { 0xed7c3e09, 96 },
    { 0x41bc3e0a, 55 },
    { 0xcb2c3e0b, 85 },
    { 0xdd8c3e0c, 55 },
    { 0xabac3e0b, 98 },
    { 0xdbdc3e0e, 95 },
    { 0x752c3e0f, 34 },
    { 0, 0 },
    { 0x27ac3e11, 84 },
    { 0x3bc3e12, 23 },
    { 0xaa4c3e13, 90 },
    { 0x1a1c3e12, 95 },
    { 0xf79c3e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10c3e1d, 98 },
    { 0x92ac3e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c3e22, 43 },
    { 0xf71c3e23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3e29, 80 },
    { 0x5d2c3e2a, 85 },
    { 0xa80c3e2b, 33 },
    { 0, 0 },
    { 0xc25c3e2d, 33 },
    { 0x97c3e2e, 46 },
    { 0x6bfc3e2f, 98 },
    { 0x785c3e30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63c3e35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x374c3e38, 30 },
    { 0x344c3e38, 55 },
    { 0x9ecc3e3a, 23 },
    { 0, 0 },
    { 0x804c3e3c, 65 },
    { 0x518c3e3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x112c3e40, 33 },
    { 0x7cc3e40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649c3e46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63c3e4d, 93 },
    { 0xfbdc3e4e, 33 },
    { 0xb7dc3e4f, 43 },
    { 0x22c3e4f, 93 },
    { 0xcaac3e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edc3e56, 55 },
    { 0, 0 },
    { 0x5b8c3e58, 95 },
    { 0, 0 },
    { 0x1c8c3e5a, 33 },
    { 0x478c3e5b, 90 },
    { 0x579c3e5c, 90 },
    { 0x2a8c3e5d, 33 },
    { 0, 0 },
    { 0x44cc3e5f, 90 },
    { 0x638c3e5f, 90 },
    { 0xd2bc3e61, 90 },
    { 0x146c3e60, 61 },
    { 0xe15c3e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c3e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c3e70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c3e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x238c3e76, 33 },
    { 0xa41c3e76, 35 },
    { 0x284c3e77, 90 },
    { 0, 0 },
    { 0xaaec3e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c3e81, 61 },
    { 0x2dfc3e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c3e87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c3e8c, 90 },
    { 0xf0dc3e8c, 23 },
    { 0x84c3e8e, 34 },
    { 0x46bc3e8f, 95 },
    { 0x7ffc3e8d, 90 },
    { 0, 0 },
    { 0xe8bc3e92, 68 },
    { 0x1d7c3e92, 46 },
    { 0x5ac3e92, 100 },
    { 0, 0 },
    { 0xb37c3e96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc3ea2, 66 },
    { 0xc08c3ea3, 67 },
    { 0x885c3ea3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc3eaf, 23 },
    { 0, 0 },
    { 0xee2c3eb1, 98 },
    { 0xb15c3eb2, 33 },
    { 0xc37c3eb1, 61 },
    { 0x2ddc3eb4, 95 },
    { 0x4b8c3eb5, 23 },
    { 0, 0 },
    { 0x439c3eb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00c3ebf, 76 },
    { 0xb4c3ec0, 33 },
    { 0xb6dc3ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc3ec4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fc3ec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c3ecb, 55 },
    { 0xdac3ecc, 95 },
    { 0x4a1c3ecd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951c3ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bc3ed4, 61 },
    { 0xa8c3ed5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3ed8, 61 },
    { 0x919c3ed9, 77 },
    { 0x4bdc3ed9, 100 },
    { 0, 0 },
    { 0x8fcc3edc, 33 },
    { 0x647c3edc, 65 },
    { 0x91bc3ede, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6c3ee2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac3ee7, 52 },
    { 0x1cfc3ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c3ef1, 66 },
    { 0, 0 },
    { 0x5f1c3ef3, 95 },
    { 0, 0 },
    { 0x3b7c3ef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ac3ef8, 95 },
    { 0xefac3ef9, 61 },
    { 0xc60c3efa, 76 },
    { 0x3eac3ef8, 95 },
    { 0x5d8c3efb, 90 },
    { 0xcb5c3ef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ac3f00, 76 },
    { 0xf04c3f01, 80 },
    { 0x503c3f01, 33 },
    { 0x1dec3f03, 34 },
    { 0x1f8c3f03, 17 },
    { 0xa76c3f01, 81 },
    { 0xf88c3f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eec3f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22c3f0d, 90 },
    { 0xa30c3f0e, 93 },
    { 0x67cc3f0f, 33 },
    { 0x4fdc3f10, 23 },
    { 0x234c3f11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51c3f14, 87 },
    { 0, 0 },
    { 0x664c3f16, 66 },
    { 0, 0 },
    { 0x7d2c3f18, 61 },
    { 0x33c3f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5c3f1f, 43 },
    { 0, 0 },
    { 0xe3ec3f21, 46 },
    { 0x6bdc3f21, 61 },
    { 0x10ac3f21, 34 },
    { 0x91fc3f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x230c3f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac3f2b, 68 },
    { 0x5dac3f2c, 55 },
    { 0x91fc3f2d, 77 },
    { 0x298c3f2d, 90 },
    { 0xc21c3f2f, 33 },
    { 0, 0 },
    { 0x161c3f31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c3f34, 55 },
    { 0x32bc3f34, 98 },
    { 0x977c3f35, 61 },
    { 0, 0 },
    { 0xa6cc3f38, 98 },
    { 0xd01c3f39, 40 },
    { 0x8f8c3f39, 95 },
    { 0x347c3f38, 43 },
    { 0, 0 },
    { 0x408c3f3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4c3f44, 90 },
    { 0x9a3c3f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc3f49, 33 },
    { 0xcb3c3f49, 46 },
    { 0xc6bc3f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192c3f50, 66 },
    { 0x8dcc3f51, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ac3f54, 83 },
    { 0x929c3f55, 81 },
    { 0x1e5c3f56, 98 },
    { 0x27bc3f56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762c3f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c3f64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c3f67, 98 },
    { 0, 0 },
    { 0x355c3f69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc3f70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18c3f73, 95 },
    { 0x7aec3f74, 34 },
    { 0x11dc3f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31c3f7c, 46 },
    { 0xf9cc3f7d, 98 },
    { 0x8f3c3f7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789c3f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc3f87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3f8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec3f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c3f92, 69 },
    { 0, 0 },
    { 0xf39c3f94, 90 },
    { 0xb96c3f95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec3f98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bc3f9c, 61 },
    { 0xfd3c3f9d, 61 },
    { 0x7ddc3f9d, 23 },
    { 0xb54c3f9d, 95 },
    { 0x76dc3f9d, 87 },
    { 0xbedc3fa1, 92 },
    { 0xdac3fa0, 46 },
    { 0xd40c3fa1, 69 },
    { 0xa99c3fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3faa, 93 },
    { 0xee9c3fab, 34 },
    { 0, 0 },
    { 0x9cdc3fad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ec3fb3, 33 },
    { 0xec1c3fb3, 99 },
    { 0, 0 },
    { 0xf10c3fb6, 99 },
    { 0xe40c3fb6, 36 },
    { 0xa82c3fb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc3fbb, 98 },
    { 0x706c3fbc, 90 },
    { 0x838c3fbd, 61 },
    { 0, 0 },
    { 0x29ac3fbf, 46 },
    { 0x465c3fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc3fc9, 76 },
    { 0x1e4c3fca, 95 },
    { 0x87cc3fcb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c3fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbc3fda, 90 },
    { 0xbacc3fdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c3fe4, 90 },
    { 0xadc3fe5, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc3fe8, 33 },
    { 0, 0 },
    { 0x64bc3fea, 66 },
    { 0x634c3fea, 66 },
    { 0x4bdc3fec, 33 },
    { 0x8c3fec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac3ff8, 75 },
    { 0, 0 },
    { 0x7b0c3ffa, 33 },
    { 0, 0 },
    { 0xa24c3ffc, 96 },
    { 0x9bfc3ffd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac4000, 90 },
    { 0x919c4001, 61 },
    { 0x4cbc4002, 90 },
    { 0x44bc4001, 33 },
    { 0x779c4004, 64 },
    { 0, 0 },
    { 0x5e9c4006, 55 },
    { 0, 0 },
    { 0xea0c4008, 93 },
    { 0xa6c4009, 33 },
    { 0x309c4008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c400d, 69 },
    { 0x8e7c400e, 90 },
    { 0x881c400d, 34 },
    { 0, 0 },
    { 0x806c4011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683c4016, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc401d, 93 },
    { 0xd7cc401e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7c4022, 76 },
    { 0, 0 },
    { 0xa97c4024, 52 },
    { 0xce3c4024, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fc402a, 98 },
    { 0, 0 },
    { 0x666c402c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc402f, 67 },
    { 0, 0 },
    { 0x153c4031, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x556c4034, 33 },
    { 0x85dc4035, 40 },
    { 0x11fc4036, 70 },
    { 0x79c4037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216c403b, 76 },
    { 0x598c403b, 43 },
    { 0x978c403d, 23 },
    { 0, 0 },
    { 0x775c403f, 75 },
    { 0, 0 },
    { 0xa01c4041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41c4044, 91 },
    { 0x188c4045, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34c4048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x699c404b, 34 },
    { 0xa76c404c, 66 },
    { 0x259c404d, 68 },
    { 0x71bc404c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c4056, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec4059, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dc4065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9c4068, 95 },
    { 0xe31c4068, 34 },
    { 0x3d7c406a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c406e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabac4074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c4077, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96c407a, 99 },
    { 0xc76c407b, 98 },
    { 0x7f6c407c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x291c407f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2c4082, 95 },
    { 0xff2c4083, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c4087, 66 },
    { 0xba2c4087, 95 },
    { 0x314c4088, 90 },
    { 0x6d6c408a, 55 },
    { 0x660c408a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc4092, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9c4095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ac409b, 76 },
    { 0, 0 },
    { 0x2d3c409d, 90 },
    { 0xd53c409e, 55 },
    { 0xd07c409f, 33 },
    { 0x99c409f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c40a7, 33 },
    { 0xa4cc40a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9c40af, 23 },
    { 0x2cac40b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23c40b4, 34 },
    { 0x133c40b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cac40ba, 66 },
    { 0, 0 },
    { 0xc58c40bc, 23 },
    { 0xdbcc40bd, 98 },
    { 0, 0 },
    { 0x8b6c40bf, 61 },
    { 0xe08c40bf, 33 },
    { 0x669c40c1, 90 },
    { 0, 0 },
    { 0x90bc40c3, 34 },
    { 0xfb1c40c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325c40ca, 77 },
    { 0, 0 },
    { 0x10fc40cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c40d1, 33 },
    { 0x867c40d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c40d6, 17 },
    { 0x9bdc40d7, 78 },
    { 0x55bc40d7, 90 },
    { 0x31fc40d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c40de, 61 },
    { 0, 0 },
    { 0xda3c40e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350c40e5, 76 },
    { 0xa09c40e6, 33 },
    { 0xf63c40e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2c40ec, 90 },
    { 0xcf1c40ec, 90 },
    { 0x3a1c40ed, 34 },
    { 0x3bac40ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2c40f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c40fc, 33 },
    { 0, 0 },
    { 0xa2bc40fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x354c4101, 23 },
    { 0xcbcc4102, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83c410a, 66 },
    { 0xbf4c410b, 90 },
    { 0x7c3c410b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918c4115, 43 },
    { 0, 0 },
    { 0x2fbc4117, 23 },
    { 0x78dc4117, 46 },
    { 0, 0 },
    { 0x29dc411a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1c4121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc4124, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbc4127, 90 },
    { 0x95fc4127, 68 },
    { 0xd67c4129, 90 },
    { 0, 0 },
    { 0xcbcc412b, 61 },
    { 0, 0 },
    { 0x7f1c412d, 75 },
    { 0xae2c412e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdc4133, 88 },
    { 0x1d2c4134, 17 },
    { 0xc56c4135, 90 },
    { 0, 0 },
    { 0x486c4137, 87 },
    { 0xe74c4138, 95 },
    { 0, 0 },
    { 0x896c413a, 61 },
    { 0xd93c413b, 65 },
    { 0xbe5c413c, 98 },
    { 0, 0 },
    { 0xd68c413e, 23 },
    { 0, 0 },
    { 0x638c4140, 97 },
    { 0x801c4141, 95 },
    { 0xe1dc4142, 61 },
    { 0xf9bc4140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dc4147, 100 },
    { 0x67dc4148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76c414b, 33 },
    { 0x537c414c, 76 },
    { 0xe40c414c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c4158, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc415c, 61 },
    { 0xa96c415d, 95 },
    { 0x9a6c415e, 90 },
    { 0, 0 },
    { 0xb5ec4160, 46 },
    { 0x678c4160, 90 },
    { 0, 0 },
    { 0x3c5c4163, 95 },
    { 0xb11c4164, 17 },
    { 0x90dc4164, 55 },
    { 0, 0 },
    { 0xae8c4167, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc416a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4c416f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431c4176, 23 },
    { 0, 0 },
    { 0xe66c4178, 98 },
    { 0xb22c4178, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc417f, 34 },
    { 0, 0 },
    { 0xce8c4181, 61 },
    { 0xce6c4182, 61 },
    { 0, 0 },
    { 0x2f8c4184, 67 },
    { 0x438c4184, 55 },
    { 0x1a7c4184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fc418a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac418e, 68 },
    { 0xd2cc418f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7c4192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11c4197, 46 },
    { 0x8e3c4198, 43 },
    { 0x3a9c4197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ec419c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c41a1, 61 },
    { 0x768c41a2, 23 },
    { 0xbdfc41a3, 90 },
    { 0xed7c41a3, 55 },
    { 0x32fc41a5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x828c41a8, 33 },
    { 0, 0 },
    { 0xe38c41aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4c41ae, 23 },
    { 0, 0 },
    { 0x17ec41b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac41b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc41b9, 23 },
    { 0x31cc41b9, 90 },
    { 0x7efc41bb, 78 },
    { 0xc08c41bc, 76 },
    { 0x237c41ba, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08c41c0, 93 },
    { 0x8c2c41c1, 66 },
    { 0x919c41c2, 95 },
    { 0x4e0c41c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c41c6, 55 },
    { 0x105c41c7, 33 },
    { 0, 0 },
    { 0xfc8c41c9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5c41cd, 61 },
    { 0, 0 },
    { 0xd41c41cf, 95 },
    { 0x1efc41d0, 87 },
    { 0x9bfc41d1, 52 },
    { 0x5b6c41d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3c41d6, 33 },
    { 0x4d1c41d7, 91 },
    { 0x240c41d8, 93 },
    { 0x47dc41d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c41dc, 34 },
    { 0x684c41dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec41e2, 46 },
    { 0, 0 },
    { 0xb42c41e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacc41e9, 85 },
    { 0x8c2c41ea, 17 },
    { 0, 0 },
    { 0x733c41ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353c41f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c41f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cc41fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac41ff, 66 },
    { 0, 0 },
    { 0xa73c4201, 98 },
    { 0, 0 },
    { 0x825c4203, 76 },
    { 0, 0 },
    { 0xde7c4205, 52 },
    { 0x9e2c4206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c420e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edc4213, 33 },
    { 0xd6c4214, 19 },
    { 0xb07c4215, 55 },
    { 0xffdc4214, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc421a, 78 },
    { 0xf03c421a, 77 },
    { 0, 0 },
    { 0x7a2c421d, 33 },
    { 0x220c421e, 65 },
    { 0xab1c421f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61c4222, 95 },
    { 0x651c4223, 23 },
    { 0xf89c4224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6c4228, 95 },
    { 0x49dc4229, 33 },
    { 0xd90c422a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1c422f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac4233, 95 },
    { 0, 0 },
    { 0x12ac4235, 90 },
    { 0, 0 },
    { 0x340c4237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968c4244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38c424a, 76 },
    { 0x19dc424b, 87 },
    { 0x873c424a, 93 },
    { 0xf11c424d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dec4250, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3c4257, 84 },
    { 0, 0 },
    { 0x948c4259, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccc4263, 98 },
    { 0, 0 },
    { 0x291c4265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc4269, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c426c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c426f, 34 },
    { 0x131c4270, 78 },
    { 0x226c4271, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ec4275, 34 },
    { 0xc70c4275, 23 },
    { 0, 0 },
    { 0x3c0c4278, 46 },
    { 0x798c4279, 23 },
    { 0xdac4279, 78 },
    { 0x963c427b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac427e, 33 },
    { 0x25dc427f, 81 },
    { 0, 0 },
    { 0x501c4281, 78 },
    { 0x5a1c4282, 75 },
    { 0x256c4282, 61 },
    { 0xbc1c4282, 98 },
    { 0xb22c4285, 52 },
    { 0x43c4285, 61 },
    { 0xb81c4287, 98 },
    { 0x33fc4288, 95 },
    { 0x21ac4287, 99 },
    { 0x53cc428a, 98 },
    { 0xed8c428b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x164c428e, 61 },
    { 0, 0 },
    { 0xbf3c4290, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec4295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c4298, 88 },
    { 0xf5ac4299, 33 },
    { 0x4ddc429a, 100 },
    { 0x8d8c429b, 91 },
    { 0xd9cc429c, 90 },
    { 0x29fc429d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381c42a4, 61 },
    { 0xeb0c42a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c42a9, 98 },
    { 0x77dc42aa, 75 },
    { 0x3a1c42ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c42b0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bc42b9, 77 },
    { 0xd72c42ba, 90 },
    { 0x879c42bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c42be, 76 },
    { 0, 0 },
    { 0xb73c42c0, 75 },
    { 0x156c42c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efc42c6, 66 },
    { 0, 0 },
    { 0x589c42c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02c42cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ac42ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c42d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c42d4, 34 },
    { 0x4d4c42d5, 100 },
    { 0x7f6c42d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c42d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c42e1, 33 },
    { 0x4c3c42e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ac42e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c42eb, 76 },
    { 0, 0 },
    { 0x6aac42ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c42f0, 33 },
    { 0x5bbc42f1, 98 },
    { 0xef9c42f2, 65 },
    { 0x5cac42f2, 95 },
    { 0xfd1c42f4, 33 },
    { 0, 0 },
    { 0xafec42f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46c42fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x879c42fd, 33 },
    { 0, 0 },
    { 0x442c42ff, 88 },
    { 0xb58c4300, 23 },
    { 0x5a1c4300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c4307, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831c430b, 23 },
    { 0xb9fc430c, 33 },
    { 0x15cc430c, 90 },
    { 0, 0 },
    { 0xef5c430f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dc4313, 84 },
    { 0xe6ec4313, 69 },
    { 0x150c4315, 43 },
    { 0xdfac4314, 95 },
    { 0xfedc4317, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3c431a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4c431d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bc4322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc4328, 43 },
    { 0x1e9c4329, 55 },
    { 0x855c432a, 74 },
    { 0x273c432b, 64 },
    { 0, 0 },
    { 0xb8ac432d, 76 },
    { 0, 0 },
    { 0x76bc432f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fc4332, 19 },
    { 0x7d4c4333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc4336, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bc4340, 95 },
    { 0x422c4341, 76 },
    { 0x266c4341, 23 },
    { 0, 0 },
    { 0x6f6c4344, 88 },
    { 0, 0 },
    { 0xf5fc4346, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c4350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c4355, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c4359, 65 },
    { 0xd82c4359, 61 },
    { 0x87ac435b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c435e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c4364, 90 },
    { 0x941c4365, 23 },
    { 0x2f5c4366, 33 },
    { 0, 0 },
    { 0x4f3c4368, 55 },
    { 0x172c4368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc436e, 95 },
    { 0, 0 },
    { 0x3c2c4370, 23 },
    { 0xacfc4370, 77 },
    { 0x5d4c4371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374c437a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852c437f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c4388, 99 },
    { 0xe24c4389, 99 },
    { 0x1dfc4388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17c438d, 98 },
    { 0x4fcc438e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c4392, 99 },
    { 0x655c4393, 95 },
    { 0xaa5c4394, 76 },
    { 0x7d7c4394, 77 },
    { 0x82fc4395, 23 },
    { 0xb9c4393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337c439c, 90 },
    { 0, 0 },
    { 0x6fac439e, 84 },
    { 0x579c439f, 90 },
    { 0x3e8c439f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4c43a6, 91 },
    { 0xbd6c43a6, 34 },
    { 0x639c43a6, 91 },
    { 0x2f3c43a6, 98 },
    { 0x4a5c43a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c43ad, 95 },
    { 0x913c43ae, 98 },
    { 0x85cc43af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13c43b3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c43b6, 98 },
    { 0, 0 },
    { 0xdbc43b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c43bc, 90 },
    { 0x6adc43bc, 67 },
    { 0x8a3c43be, 90 },
    { 0x2a3c43be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcc43c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88c43cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c43d0, 98 },
    { 0x306c43d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c43d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fc43e0, 85 },
    { 0xdfbc43e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cc43e5, 23 },
    { 0xb95c43e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570c43ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c43f2, 77 },
    { 0x946c43f3, 23 },
    { 0x396c43f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c43fc, 33 },
    { 0, 0 },
    { 0xc90c43fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c4408, 23 },
    { 0, 0 },
    { 0x312c440a, 33 },
    { 0x633c440a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc4411, 23 },
    { 0x4adc4412, 40 },
    { 0x931c4413, 98 },
    { 0x40dc4412, 52 },
    { 0x1e7c4412, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ec4423, 93 },
    { 0xce1c4423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c4429, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960c442f, 88 },
    { 0x333c442f, 100 },
    { 0x9f1c4431, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c4434, 90 },
    { 0xed6c4435, 95 },
    { 0xc19c4435, 61 },
    { 0, 0 },
    { 0x53fc4438, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x777c443b, 33 },
    { 0x111c443b, 76 },
    { 0x355c443d, 33 },
    { 0xfa2c443c, 61 },
    { 0x2f0c443d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac4446, 95 },
    { 0xd65c4446, 43 },
    { 0x9bec4448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c4454, 76 },
    { 0xedec4455, 98 },
    { 0xe9bc4456, 23 },
    { 0xaa8c4457, 67 },
    { 0x1c7c4455, 55 },
    { 0, 0 },
    { 0x9fcc445a, 95 },
    { 0, 0 },
    { 0xb2fc445c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc4460, 76 },
    { 0x4e5c4461, 23 },
    { 0x236c4462, 90 },
    { 0xcfbc4463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ac4468, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736c446b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ec446e, 23 },
    { 0, 0 },
    { 0xa64c4470, 78 },
    { 0x6e9c4471, 90 },
    { 0x125c4471, 93 },
    { 0, 0 },
    { 0xbe0c4474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0c4478, 95 },
    { 0x8c9c4479, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ac447e, 61 },
    { 0x412c447f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c4482, 46 },
    { 0x3f3c4483, 33 },
    { 0x5a3c4483, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bc4489, 76 },
    { 0x71ec448a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c4491, 90 },
    { 0xfbac4491, 61 },
    { 0x9d7c4493, 90 },
    { 0xf32c4494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9c4497, 34 },
    { 0, 0 },
    { 0xfa3c4499, 61 },
    { 0xaccc449a, 84 },
    { 0x2dfc449b, 23 },
    { 0x519c449c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529c44a4, 61 },
    { 0xc92c44a4, 98 },
    { 0x6c6c44a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8c44ad, 98 },
    { 0x690c44ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dc44b2, 55 },
    { 0x43ac44b3, 90 },
    { 0x70fc44b2, 33 },
    { 0xc31c44b5, 95 },
    { 0xadbc44b6, 61 },
    { 0x153c44b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4c44c0, 33 },
    { 0, 0 },
    { 0xb8bc44c2, 61 },
    { 0, 0 },
    { 0x65c44c4, 100 },
    { 0xd3dc44c5, 84 },
    { 0x54ec44c4, 46 },
    { 0x62cc44c7, 96 },
    { 0x85c44c8, 61 },
    { 0x816c44c9, 91 },
    { 0x436c44ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9c44ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac44d2, 33 },
    { 0xe88c44d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fc44d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98c44da, 23 },
    { 0, 0 },
    { 0x156c44dc, 93 },
    { 0x9f6c44dc, 90 },
    { 0, 0 },
    { 0x7e4c44df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ac44e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3c44e6, 23 },
    { 0x90cc44e7, 76 },
    { 0x308c44e8, 90 },
    { 0, 0 },
    { 0x2a1c44ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395c44ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc44f2, 34 },
    { 0, 0 },
    { 0xf33c44f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9c44f9, 33 },
    { 0, 0 },
    { 0xfa8c44fb, 23 },
    { 0, 0 },
    { 0x42cc44fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42c4503, 98 },
    { 0x4dec4503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24c4509, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305c450f, 98 },
    { 0x252c4510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac4515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fac4519, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c451c, 99 },
    { 0xb0fc451c, 33 },
    { 0x7c4c451c, 52 },
    { 0xc1c451f, 17 },
    { 0xee7c4520, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c4526, 55 },
    { 0, 0 },
    { 0x7e7c4528, 43 },
    { 0xbd1c4529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834c452e, 40 },
    { 0, 0 },
    { 0xdedc4530, 34 },
    { 0, 0 },
    { 0x821c4532, 76 },
    { 0, 0 },
    { 0xa53c4534, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc453c, 33 },
    { 0x2eec453d, 32 },
    { 0x9c0c453c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c4544, 46 },
    { 0, 0 },
    { 0xaddc4546, 66 },
    { 0x515c4546, 90 },
    { 0x531c4547, 83 },
    { 0, 0 },
    { 0x99dc454a, 67 },
    { 0, 0 },
    { 0x7b8c454c, 90 },
    { 0x36ac454c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc4557, 99 },
    { 0x8f3c4557, 52 },
    { 0x15bc4557, 23 },
    { 0x171c455a, 61 },
    { 0xe12c4559, 43 },
    { 0xd60c4557, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcc4561, 34 },
    { 0, 0 },
    { 0xf5bc4563, 84 },
    { 0xb4c4564, 61 },
    { 0x84bc4564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38c4568, 86 },
    { 0x223c4569, 33 },
    { 0x491c4568, 84 },
    { 0, 0 },
    { 0x2eac456c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c4570, 33 },
    { 0x84bc4570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891c4575, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6c457d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc4586, 76 },
    { 0x48ec4586, 67 },
    { 0, 0 },
    { 0x3b2c4589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc458d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c4593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc4596, 95 },
    { 0, 0 },
    { 0x712c4598, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c459d, 97 },
    { 0x6dbc459d, 17 },
    { 0x6c4c459f, 66 },
    { 0x7bdc459f, 67 },
    { 0xf1bc459e, 33 },
    { 0x228c45a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec45a6, 76 },
    { 0x564c45a7, 61 },
    { 0x4abc45a8, 76 },
    { 0xbacc45a9, 65 },
    { 0xb8ac45aa, 61 },
    { 0, 0 },
    { 0x426c45ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453c45b0, 78 },
    { 0xcb3c45b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543c45b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ec45b9, 46 },
    { 0x778c45b9, 93 },
    { 0x891c45bb, 34 },
    { 0xffcc45bb, 99 },
    { 0x6b5c45b9, 64 },
    { 0x994c45be, 77 },
    { 0x91bc45bc, 98 },
    { 0x8bcc45b9, 33 },
    { 0xfffc45c1, 84 },
    { 0x48dc45b9, 90 },
    { 0x532c45c3, 23 },
    { 0x4e6c45c3, 95 },
    { 0, 0 },
    { 0x397c45c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec45cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dc45d2, 61 },
    { 0x627c45d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c45d6, 23 },
    { 0xbd3c45d6, 33 },
    { 0xcc1c45d6, 23 },
    { 0xc6bc45d6, 66 },
    { 0, 0 },
    { 0x557c45db, 99 },
    { 0x512c45dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ac45df, 55 },
    { 0, 0 },
    { 0x8e4c45e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc45e5, 98 },
    { 0, 0 },
    { 0xec5c45e7, 52 },
    { 0x2f1c45e8, 95 },
    { 0x724c45e9, 90 },
    { 0, 0 },
    { 0x107c45eb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c45ee, 55 },
    { 0xc2ec45ef, 77 },
    { 0xbe1c45ef, 90 },
    { 0x3bbc45ee, 55 },
    { 0x413c45f2, 87 },
    { 0x88c45ef, 76 },
    { 0, 0 },
    { 0xf6fc45f5, 95 },
    { 0x5d5c45f5, 55 },
    { 0x759c45f5, 61 },
    { 0x82bc45f7, 90 },
    { 0xa3dc45f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ec45fc, 33 },
    { 0, 0 },
    { 0xd5bc45fe, 61 },
    { 0xc11c45fe, 81 },
    { 0x446c4600, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932c4604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dc4607, 90 },
    { 0x18ac4608, 46 },
    { 0xb06c4609, 76 },
    { 0xb6dc4609, 61 },
    { 0, 0 },
    { 0x1e6c460c, 2 },
    { 0x469c460d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c4610, 88 },
    { 0x4c3c4610, 90 },
    { 0x696c4611, 88 },
    { 0, 0 },
    { 0x65ec4614, 52 },
    { 0x42ec4614, 33 },
    { 0x46dc4616, 78 },
    { 0x6d8c4615, 34 },
    { 0x936c4617, 98 },
    { 0xe6dc4614, 23 },
    { 0x6e3c4615, 77 },
    { 0xf96c461b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc461e, 55 },
    { 0x86ec461f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc4625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c462a, 23 },
    { 0x32ac462b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc462e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2c4632, 95 },
    { 0x6dfc4633, 23 },
    { 0, 0 },
    { 0xa45c4635, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5c463f, 55 },
    { 0, 0 },
    { 0x33dc4641, 17 },
    { 0x89dc4642, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebc464b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eac464f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccc4653, 76 },
    { 0x3d7c4653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c465d, 75 },
    { 0xf4ec465d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x692c4661, 98 },
    { 0x2e9c4662, 90 },
    { 0xc12c4663, 90 },
    { 0x6b7c4664, 91 },
    { 0xa2ac4664, 90 },
    { 0x36bc4664, 85 },
    { 0x256c4666, 34 },
    { 0x3d4c4664, 100 },
    { 0xbc5c4669, 84 },
    { 0x7a2c466a, 61 },
    { 0x32ec4664, 100 },
    { 0x8e5c4664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c466f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc4673, 61 },
    { 0xd0c4674, 76 },
    { 0, 0 },
    { 0xa06c4676, 98 },
    { 0x9a7c4677, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac467c, 95 },
    { 0x63ac467c, 95 },
    { 0x111c467e, 23 },
    { 0, 0 },
    { 0xfcdc4680, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6c4683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cc4686, 34 },
    { 0x363c4687, 34 },
    { 0xb0cc4688, 33 },
    { 0xce6c4686, 93 },
    { 0xb4c468a, 61 },
    { 0xf22c468b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x757c468e, 46 },
    { 0, 0 },
    { 0x39dc4690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc4694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc469a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec469e, 90 },
    { 0x93dc469f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504c46a4, 68 },
    { 0xd97c46a4, 43 },
    { 0xe63c46a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c46ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c46b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c46b4, 61 },
    { 0xc17c46b5, 66 },
    { 0, 0 },
    { 0x4bdc46b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc46bd, 95 },
    { 0x629c46bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0c46c3, 87 },
    { 0x3b4c46c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec46ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccc46cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491c46d5, 98 },
    { 0, 0 },
    { 0xbd5c46d7, 33 },
    { 0x1bdc46d7, 68 },
    { 0, 0 },
    { 0x9cc46da, 34 },
    { 0x2f4c46db, 33 },
    { 0, 0 },
    { 0x17ec46dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cc46e3, 34 },
    { 0x969c46e4, 90 },
    { 0, 0 },
    { 0x53fc46e6, 43 },
    { 0x80ec46e7, 66 },
    { 0, 0 },
    { 0x23dc46e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc46ed, 76 },
    { 0x40dc46ee, 90 },
    { 0x9cc46ee, 33 },
    { 0xc80c46f0, 76 },
    { 0xf85c46f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c46f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc46f9, 91 },
    { 0xc4bc46fa, 61 },
    { 0x4a2c46fa, 95 },
    { 0, 0 },
    { 0x8dac46fd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc4702, 95 },
    { 0xe9fc4703, 95 },
    { 0xdbdc4704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930c4708, 77 },
    { 0x1d5c4709, 33 },
    { 0, 0 },
    { 0x821c470b, 96 },
    { 0xae9c470b, 23 },
    { 0xe55c470d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cac4710, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x676c4713, 43 },
    { 0xadec4713, 90 },
    { 0x7ddc4715, 77 },
    { 0x119c4716, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec471b, 90 },
    { 0, 0 },
    { 0xa86c471d, 93 },
    { 0x312c471e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371c4723, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36c4726, 33 },
    { 0x4b4c4726, 93 },
    { 0xe57c4728, 55 },
    { 0xe76c4729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276c4735, 90 },
    { 0xc72c4735, 17 },
    { 0, 0 },
    { 0x42fc4738, 61 },
    { 0, 0 },
    { 0x37cc473a, 34 },
    { 0xf60c473b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc473f, 90 },
    { 0x345c473f, 33 },
    { 0x2acc4741, 61 },
    { 0, 0 },
    { 0x9f1c4743, 95 },
    { 0x84fc4744, 95 },
    { 0xe3fc4743, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabc474d, 33 },
    { 0, 0 },
    { 0xddbc474f, 100 },
    { 0x3d4c4750, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c4754, 23 },
    { 0x4f6c4755, 55 },
    { 0, 0 },
    { 0x895c4757, 33 },
    { 0, 0 },
    { 0xa07c4759, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8c475f, 85 },
    { 0, 0 },
    { 0xc5c4761, 61 },
    { 0, 0 },
    { 0x8d0c4763, 33 },
    { 0, 0 },
    { 0xbd4c4765, 90 },
    { 0x1bac4765, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdc4769, 99 },
    { 0x508c4769, 98 },
    { 0xc4bc476b, 55 },
    { 0, 0 },
    { 0x544c476d, 33 },
    { 0xf9ec476d, 23 },
    { 0xed3c476f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c4774, 33 },
    { 0x46dc4774, 93 },
    { 0xa4c4775, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fc4779, 83 },
    { 0, 0 },
    { 0x81ac477b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cc477e, 23 },
    { 0x33c477f, 98 },
    { 0x4b9c477f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c4784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4788, 90 },
    { 0, 0 },
    { 0xee8c478a, 98 },
    { 0x849c478b, 97 },
    { 0, 0 },
    { 0xa7cc478d, 67 },
    { 0, 0 },
    { 0xab3c478f, 90 },
    { 0xe2fc4790, 95 },
    { 0x1e3c4791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c4798, 43 },
    { 0x6fcc4798, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676c479e, 33 },
    { 0xa94c479e, 23 },
    { 0x145c47a0, 90 },
    { 0x91ec47a1, 99 },
    { 0x6d5c47a2, 97 },
    { 0x548c47a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c47ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x965c47b1, 76 },
    { 0x17ec47b2, 23 },
    { 0xa0fc47b3, 61 },
    { 0x71bc47b1, 95 },
    { 0x777c47b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bc47b8, 33 },
    { 0xd1ec47b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9c47bd, 95 },
    { 0, 0 },
    { 0x1fac47bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3c47c2, 55 },
    { 0x951c47c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78c47c7, 61 },
    { 0x837c47c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c47d2, 90 },
    { 0xfc1c47d3, 34 },
    { 0, 0 },
    { 0x2bac47d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafec47de, 33 },
    { 0x95cc47df, 90 },
    { 0x45ec47df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c47e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98c47e6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c47e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4c47ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8c47ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc47fd, 33 },
    { 0, 0 },
    { 0x69dc47ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9c480a, 61 },
    { 0x231c480b, 33 },
    { 0x2d4c480c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9c4810, 91 },
    { 0x74ec4811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c4819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31c481c, 61 },
    { 0, 0 },
    { 0xe57c481e, 64 },
    { 0x36dc481f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c4823, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c4827, 95 },
    { 0, 0 },
    { 0xd15c4829, 46 },
    { 0, 0 },
    { 0x920c482b, 43 },
    { 0xe96c482b, 34 },
    { 0, 0 },
    { 0x6dcc482e, 61 },
    { 0, 0 },
    { 0xa99c4830, 61 },
    { 0x672c4830, 23 },
    { 0xf54c4830, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adc4838, 87 },
    { 0x663c4839, 90 },
    { 0x85fc4838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c483f, 90 },
    { 0x398c4840, 90 },
    { 0xc82c4840, 23 },
    { 0xf44c4841, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5c4845, 90 },
    { 0, 0 },
    { 0x91dc4847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c484c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcc4853, 33 },
    { 0x31ec4854, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ac485a, 98 },
    { 0x97ac485b, 90 },
    { 0x21bc485b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c4865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c4868, 78 },
    { 0x5d7c4869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac486d, 55 },
    { 0xafdc486d, 61 },
    { 0xd30c486f, 90 },
    { 0x908c4870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec4876, 34 },
    { 0x810c4877, 43 },
    { 0, 0 },
    { 0xc0cc4879, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c487e, 91 },
    { 0, 0 },
    { 0xeb3c4880, 76 },
    { 0xac4881, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66c4889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740c488e, 61 },
    { 0, 0 },
    { 0x18bc4890, 34 },
    { 0x5b4c4891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09c4894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c4898, 90 },
    { 0xac9c4898, 93 },
    { 0x7a8c489a, 61 },
    { 0xfe3c489a, 23 },
    { 0x81bc489c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c48a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846c48a9, 43 },
    { 0x20fc48aa, 55 },
    { 0, 0 },
    { 0x1bec48ac, 75 },
    { 0xf56c48ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380c48b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c48ba, 91 },
    { 0x780c48bb, 55 },
    { 0xe16c48bb, 23 },
    { 0, 0 },
    { 0x695c48be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674c48c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x551c48c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0c48c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc48ce, 61 },
    { 0, 0 },
    { 0x1b9c48d0, 68 },
    { 0xd61c48d1, 52 },
    { 0xc01c48d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8c48d8, 23 },
    { 0x321c48d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c48dc, 66 },
    { 0, 0 },
    { 0xe3c48de, 76 },
    { 0xceac48df, 33 },
    { 0x7d4c48de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c48e4, 55 },
    { 0, 0 },
    { 0x66ac48e6, 34 },
    { 0x49cc48e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9c48ef, 90 },
    { 0x82fc48ef, 23 },
    { 0xf69c48f1, 90 },
    { 0, 0 },
    { 0xd95c48f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc48f6, 43 },
    { 0, 0 },
    { 0xacac48f8, 95 },
    { 0x62ac48f9, 91 },
    { 0xd1ac48f9, 17 },
    { 0, 0 },
    { 0xf99c48fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318c4901, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fc4904, 95 },
    { 0x3a2c4905, 33 },
    { 0x179c4904, 43 },
    { 0xc0ac4904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c490a, 76 },
    { 0x622c490b, 33 },
    { 0, 0 },
    { 0xefc490d, 46 },
    { 0, 0 },
    { 0xabac490f, 90 },
    { 0xbfac490f, 76 },
    { 0xd62c490f, 33 },
    { 0xd91c4912, 23 },
    { 0x4ebc4912, 90 },
    { 0xe23c4914, 34 },
    { 0xf68c4914, 76 },
    { 0x48bc4916, 33 },
    { 0, 0 },
    { 0x53ac4918, 90 },
    { 0, 0 },
    { 0xefcc491a, 95 },
    { 0x6bcc491a, 99 },
    { 0xfc4c491c, 99 },
    { 0x41dc491d, 61 },
    { 0xe44c491e, 90 },
    { 0xa36c491e, 95 },
    { 0x4c4920, 90 },
    { 0x965c4920, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526c492b, 90 },
    { 0, 0 },
    { 0x6c2c492d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bc4934, 17 },
    { 0x3c2c4935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c4939, 93 },
    { 0x4bbc493a, 33 },
    { 0xabcc493b, 24 },
    { 0x55c4939, 85 },
    { 0, 0 },
    { 0x530c493e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c4949, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25c494c, 78 },
    { 0x333c494d, 23 },
    { 0xd5fc494e, 90 },
    { 0x17c494d, 90 },
    { 0xdc7c4950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x698c4953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaec4957, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c495a, 90 },
    { 0, 0 },
    { 0x491c495c, 90 },
    { 0, 0 },
    { 0xa09c495e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacc4961, 90 },
    { 0x80dc4962, 43 },
    { 0xfaac4963, 101 },
    { 0, 0 },
    { 0xe28c4965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0c496a, 76 },
    { 0x487c496a, 90 },
    { 0x29c496c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cc4973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8c4979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc497d, 100 },
    { 0xf43c497d, 95 },
    { 0, 0 },
    { 0x579c4980, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631c4995, 77 },
    { 0, 0 },
    { 0x651c4997, 90 },
    { 0x5c4997, 95 },
    { 0, 0 },
    { 0x192c499a, 52 },
    { 0xd46c499b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fc49a1, 95 },
    { 0xee1c49a2, 46 },
    { 0xaacc49a3, 99 },
    { 0, 0 },
    { 0x4dfc49a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c49a8, 61 },
    { 0xb28c49a9, 95 },
    { 0x85ac49aa, 90 },
    { 0x7e7c49ab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc49af, 23 },
    { 0x586c49af, 33 },
    { 0, 0 },
    { 0x799c49b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c49b8, 66 },
    { 0, 0 },
    { 0xb13c49ba, 61 },
    { 0, 0 },
    { 0xa22c49bc, 99 },
    { 0x2cec49bc, 96 },
    { 0x175c49be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c49c1, 95 },
    { 0, 0 },
    { 0xd79c49c3, 33 },
    { 0, 0 },
    { 0x3dfc49c5, 43 },
    { 0, 0 },
    { 0x411c49c7, 95 },
    { 0x432c49c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520c49ce, 61 },
    { 0x9bcc49ce, 33 },
    { 0x9bfc49d0, 90 },
    { 0x61fc49d1, 55 },
    { 0x331c49d0, 34 },
    { 0x281c49cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ac49d8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bc49e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cc49e4, 69 },
    { 0x2c1c49e5, 98 },
    { 0xd5fc49e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bc49e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc49ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc49f0, 76 },
    { 0x5a2c49f1, 55 },
    { 0x45ac49f0, 23 },
    { 0x352c49f2, 61 },
    { 0, 0 },
    { 0x82cc49f5, 99 },
    { 0xd13c49f5, 61 },
    { 0xc7c49f7, 91 },
    { 0, 0 },
    { 0x5fc49f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cc49fd, 99 },
    { 0x489c49fd, 33 },
    { 0xdddc49fd, 68 },
    { 0, 0 },
    { 0x2d6c4a01, 75 },
    { 0xf9c4a02, 98 },
    { 0x811c4a02, 33 },
    { 0x3bdc4a04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc4a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c4a0f, 87 },
    { 0, 0 },
    { 0x2d8c4a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1c4a17, 46 },
    { 0, 0 },
    { 0xd7c4a19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5c4a1e, 98 },
    { 0, 0 },
    { 0x9a4c4a20, 99 },
    { 0, 0 },
    { 0xff9c4a22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c4a27, 23 },
    { 0, 0 },
    { 0xfb1c4a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x796c4a2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac4a31, 33 },
    { 0, 0 },
    { 0x3e2c4a33, 34 },
    { 0xa44c4a34, 61 },
    { 0x751c4a35, 87 },
    { 0x2d9c4a35, 33 },
    { 0, 0 },
    { 0x4fc4a38, 98 },
    { 0xda7c4a39, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667c4a40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6c4a43, 96 },
    { 0x62ec4a44, 98 },
    { 0, 0 },
    { 0x6f3c4a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfc4a4a, 33 },
    { 0, 0 },
    { 0x8bec4a4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac4a4f, 33 },
    { 0x4dbc4a50, 76 },
    { 0xf25c4a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec4a55, 98 },
    { 0x6f5c4a56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc4a59, 84 },
    { 0x980c4a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dc4a5d, 98 },
    { 0, 0 },
    { 0x62ec4a5f, 33 },
    { 0xb0bc4a60, 23 },
    { 0x58ec4a60, 23 },
    { 0xe54c4a62, 55 },
    { 0xaecc4a62, 61 },
    { 0xd2cc4a64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0c4a68, 55 },
    { 0x3a6c4a68, 34 },
    { 0x82cc4a6a, 90 },
    { 0xf49c4a6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc4a6e, 84 },
    { 0x132c4a6f, 61 },
    { 0x1f9c4a70, 34 },
    { 0xd90c4a70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925c4a75, 23 },
    { 0x142c4a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171c4a7c, 43 },
    { 0, 0 },
    { 0x177c4a7e, 87 },
    { 0xfac4a7e, 90 },
    { 0x974c4a80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c4a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfc4a89, 61 },
    { 0xe0c4a89, 98 },
    { 0x657c4a8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1c4a93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c4a96, 90 },
    { 0, 0 },
    { 0x1bac4a98, 82 },
    { 0xd75c4a99, 66 },
    { 0x195c4a99, 61 },
    { 0x919c4a9b, 52 },
    { 0, 0 },
    { 0x136c4a9d, 93 },
    { 0x391c4a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ec4aa9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc4aac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ec4aaf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4c4ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90c4ab8, 66 },
    { 0, 0 },
    { 0x520c4aba, 61 },
    { 0x701c4aba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c4abf, 46 },
    { 0, 0 },
    { 0xfd1c4ac1, 33 },
    { 0, 0 },
    { 0x3c6c4ac3, 98 },
    { 0x988c4ac4, 66 },
    { 0x626c4ac5, 34 },
    { 0, 0 },
    { 0x66cc4ac7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc4acb, 55 },
    { 0x4c1c4acb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c4acf, 67 },
    { 0xfb2c4ad0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546c4ad5, 98 },
    { 0xfb0c4ad6, 90 },
    { 0x914c4ad7, 33 },
    { 0xee7c4ad6, 98 },
    { 0xb4bc4ad8, 46 },
    { 0xa35c4ada, 34 },
    { 0x5dbc4adb, 90 },
    { 0, 0 },
    { 0xfd8c4add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dc4ae1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999c4ae9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898c4af6, 90 },
    { 0x44ec4af7, 93 },
    { 0x406c4af7, 90 },
    { 0x61ec4af7, 91 },
    { 0xa4bc4af6, 90 },
    { 0xb0dc4afa, 100 },
    { 0x695c4af7, 61 },
    { 0xc83c4afc, 67 },
    { 0x59ac4afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc4b02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82c4b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fc4b11, 78 },
    { 0, 0 },
    { 0xf64c4b13, 61 },
    { 0xab4c4b14, 95 },
    { 0x770c4b15, 43 },
    { 0xd5cc4b16, 100 },
    { 0, 0 },
    { 0xa85c4b18, 97 },
    { 0x1c6c4b19, 76 },
    { 0, 0 },
    { 0xcf5c4b1b, 61 },
    { 0xcac4b1b, 98 },
    { 0xfe2c4b1b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ec4b20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758c4b24, 23 },
    { 0, 0 },
    { 0x20fc4b26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627c4b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec4b34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc4b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486c4b3d, 81 },
    { 0, 0 },
    { 0x57dc4b3f, 34 },
    { 0x356c4b3f, 52 },
    { 0x1d1c4b41, 90 },
    { 0xa2c4b42, 76 },
    { 0x681c4b43, 46 },
    { 0xdc1c4b43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c4b47, 40 },
    { 0, 0 },
    { 0x1dbc4b49, 98 },
    { 0x51dc4b4a, 95 },
    { 0x74ac4b4a, 61 },
    { 0x457c4b4b, 90 },
    { 0xf6ac4b4b, 76 },
    { 0xad7c4b4e, 33 },
    { 0x829c4b4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dc4b52, 91 },
    { 0, 0 },
    { 0x2ebc4b54, 76 },
    { 0, 0 },
    { 0xcbac4b56, 34 },
    { 0, 0 },
    { 0xa50c4b58, 76 },
    { 0xdc6c4b58, 55 },
    { 0x98cc4b5a, 98 },
    { 0x557c4b59, 95 },
    { 0, 0 },
    { 0x572c4b5d, 33 },
    { 0xc58c4b5d, 33 },
    { 0, 0 },
    { 0x17c4b60, 56 },
    { 0x407c4b61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec4b69, 40 },
    { 0xe75c4b6a, 93 },
    { 0x168c4b6b, 87 },
    { 0xd6cc4b69, 33 },
    { 0, 0 },
    { 0xfdc4b6e, 98 },
    { 0x888c4b6e, 61 },
    { 0x2bc4b6e, 23 },
    { 0x936c4b70, 66 },
    { 0x280c4b72, 98 },
    { 0x6cec4b71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fc4b78, 90 },
    { 0xbb5c4b78, 56 },
    { 0x61bc4b7a, 98 },
    { 0x641c4b7b, 33 },
    { 0xa1cc4b7c, 77 },
    { 0xb2fc4b7d, 84 },
    { 0x26ac4b7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45c4b83, 87 },
    { 0, 0 },
    { 0xeaac4b85, 46 },
    { 0x9fdc4b86, 93 },
    { 0xafcc4b87, 86 },
    { 0xa6bc4b88, 23 },
    { 0x8b0c4b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c4b96, 78 },
    { 0xec6c4b97, 90 },
    { 0, 0 },
    { 0x73fc4b99, 61 },
    { 0x964c4b9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c4b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7c4ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382c4ba5, 85 },
    { 0x548c4ba6, 33 },
    { 0x825c4ba6, 61 },
    { 0x4fc4ba8, 98 },
    { 0x32dc4ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ac4bae, 98 },
    { 0xb1bc4baf, 52 },
    { 0xe0dc4bb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ec4bb8, 34 },
    { 0x525c4bb9, 67 },
    { 0xae3c4bb8, 80 },
    { 0, 0 },
    { 0xc28c4bbc, 55 },
    { 0x430c4bbd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dc4bc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0c4bc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac4bc9, 76 },
    { 0xda6c4bca, 84 },
    { 0, 0 },
    { 0xb5cc4bcc, 90 },
    { 0, 0 },
    { 0x899c4bce, 90 },
    { 0xe1ec4bcf, 85 },
    { 0x7fac4bd0, 88 },
    { 0xb2ec4bd0, 95 },
    { 0xa33c4bd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9c4bd5, 96 },
    { 0x42ac4bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7c4bd9, 95 },
    { 0xf54c4bda, 46 },
    { 0xacdc4bdb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28c4be0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc4be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c4beb, 93 },
    { 0xef9c4bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dc4bf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dc4bf4, 76 },
    { 0xa8ac4bf5, 90 },
    { 0x311c4bf6, 88 },
    { 0x318c4bf7, 95 },
    { 0x21fc4bf8, 33 },
    { 0x7fac4bf8, 93 },
    { 0x7c4c4bf8, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc4bff, 77 },
    { 0xd22c4c00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dc4c06, 43 },
    { 0x5acc4c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec4c0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc4c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac4c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bc4c25, 90 },
    { 0x74bc4c25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc4c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fc4c2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc4c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac4c38, 23 },
    { 0x789c4c38, 33 },
    { 0x7cc4c3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc4c3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc4c45, 33 },
    { 0xbc7c4c45, 43 },
    { 0x4bcc4c47, 53 },
    { 0, 0 },
    { 0x703c4c49, 46 },
    { 0x96ec4c49, 46 },
    { 0x77fc4c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc4c4e, 95 },
    { 0, 0 },
    { 0x1efc4c50, 78 },
    { 0, 0 },
    { 0x65fc4c52, 61 },
    { 0xb7ac4c53, 66 },
    { 0, 0 },
    { 0xb50c4c55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acc4c5f, 61 },
    { 0x4b2c4c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25c4c65, 100 },
    { 0, 0 },
    { 0x4aac4c67, 61 },
    { 0xecec4c68, 95 },
    { 0x993c4c69, 90 },
    { 0, 0 },
    { 0xd20c4c6b, 77 },
    { 0x7cbc4c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ec4c6f, 95 },
    { 0x42fc4c70, 61 },
    { 0, 0 },
    { 0x62dc4c72, 23 },
    { 0xfdfc4c72, 33 },
    { 0x85fc4c74, 90 },
    { 0xafc4c73, 91 },
    { 0, 0 },
    { 0x24ec4c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4c4c7a, 95 },
    { 0, 0 },
    { 0x446c4c7c, 61 },
    { 0x505c4c7c, 80 },
    { 0xd23c4c7c, 23 },
    { 0, 0 },
    { 0xfac4c80, 61 },
    { 0x3eec4c81, 46 },
    { 0xe08c4c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876c4c8d, 98 },
    { 0, 0 },
    { 0xb53c4c8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4c4c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c4c9d, 98 },
    { 0x372c4c9e, 91 },
    { 0, 0 },
    { 0xe8bc4ca0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c4cad, 95 },
    { 0x451c4cae, 33 },
    { 0x6ebc4caf, 95 },
    { 0xfecc4cb0, 90 },
    { 0x281c4cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc4cb4, 90 },
    { 0xbb0c4cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc4cc1, 76 },
    { 0x143c4cc1, 98 },
    { 0xc43c4cc2, 99 },
    { 0xff3c4cc4, 33 },
    { 0, 0 },
    { 0x9e6c4cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6c4cc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73c4cce, 36 },
    { 0, 0 },
    { 0xf56c4cd0, 90 },
    { 0xabac4cd0, 90 },
    { 0, 0 },
    { 0xaf6c4cd3, 95 },
    { 0x52dc4cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c4cd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dc4cdf, 33 },
    { 0x1a7c4ce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c4ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0c4ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80c4cf0, 33 },
    { 0xd55c4cf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbc4cf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c4d00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01c4d03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c4d12, 98 },
    { 0x5ddc4d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1c4d16, 61 },
    { 0, 0 },
    { 0x4e4c4d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1c4d1f, 33 },
    { 0xaddc4d1f, 95 },
    { 0, 0 },
    { 0x1cbc4d22, 33 },
    { 0, 0 },
    { 0xcc7c4d24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc4d2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07c4d32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c4d36, 33 },
    { 0x600c4d37, 33 },
    { 0x627c4d38, 87 },
    { 0x95dc4d39, 90 },
    { 0, 0 },
    { 0x1b6c4d3b, 93 },
    { 0x447c4d3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebc4d49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4d4f, 61 },
    { 0, 0 },
    { 0x848c4d51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc4d54, 78 },
    { 0x847c4d55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc4d5c, 23 },
    { 0x134c4d5c, 53 },
    { 0xf2cc4d5e, 34 },
    { 0xebfc4d5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6c4d62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c4d68, 77 },
    { 0, 0 },
    { 0xaa3c4d6a, 95 },
    { 0xdcfc4d6b, 23 },
    { 0xafcc4d6c, 99 },
    { 0x4fc4d6b, 75 },
    { 0, 0 },
    { 0xbe1c4d6f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc4d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c4d78, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc4d80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c4d85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c4d8c, 61 },
    { 0xf02c4d8d, 43 },
    { 0, 0 },
    { 0x821c4d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72c4d93, 34 },
    { 0x1abc4d93, 33 },
    { 0xa5fc4d95, 97 },
    { 0x1c7c4d93, 93 },
    { 0xc1ec4d95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac4d9c, 23 },
    { 0x589c4d9d, 33 },
    { 0, 0 },
    { 0x6aac4d9f, 33 },
    { 0x238c4da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc4da4, 78 },
    { 0xe29c4da5, 90 },
    { 0x67ac4da4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9c4daa, 99 },
    { 0x679c4daa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cc4dae, 43 },
    { 0, 0 },
    { 0x68fc4db0, 61 },
    { 0x20ac4db0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c4db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28c4dba, 84 },
    { 0x1f7c4dbb, 90 },
    { 0x55fc4dbc, 78 },
    { 0xb74c4dbc, 23 },
    { 0x733c4dbe, 61 },
    { 0xb5c4dbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27c4dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ac4dc7, 100 },
    { 0xe81c4dc8, 33 },
    { 0x9aac4dc8, 93 },
    { 0x26ec4dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc4dce, 65 },
    { 0x474c4dcf, 61 },
    { 0, 0 },
    { 0xd8fc4dd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c4dd4, 9 },
    { 0, 0 },
    { 0x20cc4dd6, 33 },
    { 0x1e3c4dd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc4ddb, 33 },
    { 0, 0 },
    { 0xb03c4ddd, 78 },
    { 0x54ac4dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc4de2, 67 },
    { 0xa1bc4de3, 95 },
    { 0x4e3c4de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4de7, 99 },
    { 0x93cc4de8, 100 },
    { 0x71bc4de8, 90 },
    { 0xc6ec4de8, 23 },
    { 0, 0 },
    { 0xbd2c4dec, 17 },
    { 0x6a3c4ded, 43 },
    { 0, 0 },
    { 0x67ec4def, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fc4df7, 43 },
    { 0x1d1c4df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eac4dfb, 61 },
    { 0x366c4dfc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4e01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198c4e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ac4e0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc4e12, 99 },
    { 0xc25c4e13, 38 },
    { 0, 0 },
    { 0x273c4e15, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68c4e1c, 55 },
    { 0xedac4e1d, 98 },
    { 0x6c3c4e1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dc4e25, 55 },
    { 0x6a8c4e26, 98 },
    { 0xfbcc4e27, 55 },
    { 0xbbec4e28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63c4e2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423c4e34, 76 },
    { 0xb95c4e35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c4e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x758c4e3d, 76 },
    { 0xa72c4e3e, 67 },
    { 0xad6c4e3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac4e44, 91 },
    { 0x929c4e45, 66 },
    { 0xad9c4e46, 23 },
    { 0, 0 },
    { 0x576c4e48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bc4e4b, 90 },
    { 0xf29c4e4b, 66 },
    { 0xcdac4e4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c4e51, 56 },
    { 0, 0 },
    { 0x1b6c4e53, 34 },
    { 0, 0 },
    { 0xea2c4e55, 90 },
    { 0, 0 },
    { 0x1c6c4e57, 55 },
    { 0x4f6c4e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c4e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1c4e61, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc4e64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc4e67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420c4e6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc4e6f, 87 },
    { 0xf2cc4e6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c4e73, 23 },
    { 0x109c4e74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53c4e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc4e80, 84 },
    { 0xac4c4e81, 52 },
    { 0, 0 },
    { 0x6c2c4e83, 98 },
    { 0xeb4c4e84, 95 },
    { 0, 0 },
    { 0x357c4e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887c4e8a, 90 },
    { 0xba3c4e8a, 33 },
    { 0x54fc4e8c, 53 },
    { 0x3f2c4e8a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x145c4e90, 46 },
    { 0, 0 },
    { 0xf26c4e92, 85 },
    { 0, 0 },
    { 0x80ec4e94, 90 },
    { 0xe56c4e95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x242c4e98, 33 },
    { 0x455c4e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x515c4e9c, 34 },
    { 0, 0 },
    { 0x29fc4e9e, 90 },
    { 0, 0 },
    { 0x4bdc4ea0, 76 },
    { 0x452c4ea0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c4ea7, 94 },
    { 0, 0 },
    { 0x798c4ea9, 84 },
    { 0xe28c4ea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ac4ead, 99 },
    { 0xe41c4ead, 36 },
    { 0xc43c4eaf, 85 },
    { 0, 0 },
    { 0x343c4eb1, 99 },
    { 0xe97c4eb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200c4ebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddc4ebe, 55 },
    { 0x896c4ebf, 90 },
    { 0, 0 },
    { 0xde2c4ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c4ec7, 98 },
    { 0, 0 },
    { 0xee0c4ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c4ecf, 76 },
    { 0x3d8c4ed0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57c4ed5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc4ed8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41c4ee2, 61 },
    { 0x3b8c4ee3, 76 },
    { 0xe04c4ee4, 98 },
    { 0xb1bc4ee5, 61 },
    { 0xe9cc4ee6, 17 },
    { 0xd73c4ee7, 90 },
    { 0x64ec4ee7, 61 },
    { 0, 0 },
    { 0x6e6c4eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac4eee, 95 },
    { 0xcaac4eef, 68 },
    { 0xe7ec4ef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdc4ef6, 36 },
    { 0x459c4ef7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9c4efa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c4f03, 68 },
    { 0xb58c4f03, 98 },
    { 0xc36c4f05, 23 },
    { 0xe5ec4f06, 33 },
    { 0x77dc4f06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c4f0f, 90 },
    { 0, 0 },
    { 0x3abc4f11, 61 },
    { 0xbf5c4f12, 67 },
    { 0, 0 },
    { 0x9a9c4f14, 85 },
    { 0, 0 },
    { 0xe70c4f16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec4f19, 76 },
    { 0, 0 },
    { 0x848c4f1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c4f1e, 61 },
    { 0xba8c4f1e, 98 },
    { 0, 0 },
    { 0x867c4f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c4f29, 55 },
    { 0x356c4f29, 80 },
    { 0x635c4f2b, 61 },
    { 0x97dc4f2b, 90 },
    { 0xb81c4f2c, 90 },
    { 0xe1ac4f2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591c4f32, 46 },
    { 0x968c4f33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c4f36, 80 },
    { 0, 0 },
    { 0x4fc4f38, 43 },
    { 0xd9cc4f39, 88 },
    { 0, 0 },
    { 0xb24c4f3b, 23 },
    { 0, 0 },
    { 0x8dac4f3d, 61 },
    { 0, 0 },
    { 0x78ac4f3f, 55 },
    { 0xdf7c4f3f, 95 },
    { 0, 0 },
    { 0xd56c4f42, 33 },
    { 0x626c4f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeec4f47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76c4f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x494c4f4f, 70 },
    { 0x8bbc4f50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc4f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ec4f58, 61 },
    { 0x878c4f59, 23 },
    { 0x6bcc4f59, 90 },
    { 0x948c4f5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc4f5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dc4f63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92c4f66, 84 },
    { 0x44c4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc4f6a, 95 },
    { 0x5f5c4f6b, 33 },
    { 0x880c4f6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c4f70, 95 },
    { 0x317c4f70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc4f75, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x133c4f78, 19 },
    { 0, 0 },
    { 0xee4c4f7a, 33 },
    { 0x106c4f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c4f80, 90 },
    { 0xd83c4f81, 95 },
    { 0x31ac4f82, 34 },
    { 0, 0 },
    { 0xd6cc4f84, 43 },
    { 0x857c4f85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c4f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752c4f8d, 77 },
    { 0, 0 },
    { 0x3cc4f8f, 55 },
    { 0x425c4f8f, 80 },
    { 0x715c4f91, 33 },
    { 0x6aec4f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0c4f95, 90 },
    { 0, 0 },
    { 0x63ec4f97, 90 },
    { 0x4bec4f97, 55 },
    { 0xc46c4f99, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc4f9e, 95 },
    { 0x3d2c4f9f, 75 },
    { 0xc34c4fa0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc4fa3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85c4fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51c4fab, 46 },
    { 0xde6c4fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc4fb6, 78 },
    { 0xa8bc4fb7, 45 },
    { 0x808c4fb7, 68 },
    { 0xb73c4fb9, 34 },
    { 0x54dc4fb6, 33 },
    { 0xd96c4fb7, 30 },
    { 0x315c4fbc, 46 },
    { 0x82dc4fbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c4fc1, 61 },
    { 0, 0 },
    { 0x5fac4fc3, 96 },
    { 0x99dc4fc4, 91 },
    { 0, 0 },
    { 0x8c1c4fc6, 78 },
    { 0x91cc4fc6, 78 },
    { 0x66bc4fc8, 23 },
    { 0x435c4fc9, 91 },
    { 0xbd5c4fca, 43 },
    { 0, 0 },
    { 0x5aac4fcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cc4fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195c4fda, 93 },
    { 0x894c4fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c4fde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8c4fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac4fe6, 90 },
    { 0xa8c4fe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bc4fee, 96 },
    { 0x17dc4fef, 46 },
    { 0xb23c4ff0, 33 },
    { 0x980c4ff1, 98 },
    { 0, 0 },
    { 0x6dcc4ff3, 90 },
    { 0x108c4ff4, 90 },
    { 0, 0 },
    { 0x243c4ff6, 83 },
    { 0x1d7c4ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c4ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594c4ffe, 76 },
    { 0x464c4fff, 98 },
    { 0, 0 },
    { 0xd47c5001, 90 },
    { 0, 0 },
    { 0xddac5003, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c500c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c5010, 76 },
    { 0xb09c5011, 55 },
    { 0, 0 },
    { 0x7bfc5013, 90 },
    { 0x20ac5014, 78 },
    { 0x291c5015, 33 },
    { 0xe64c5015, 90 },
    { 0x5c0c5016, 90 },
    { 0, 0 },
    { 0xe33c5019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272c501e, 33 },
    { 0, 0 },
    { 0x9ec5020, 55 },
    { 0, 0 },
    { 0xe0ec5022, 68 },
    { 0x6cdc5023, 95 },
    { 0x893c5023, 61 },
    { 0, 0 },
    { 0x983c5026, 33 },
    { 0x26cc5027, 43 },
    { 0xb8dc5028, 66 },
    { 0, 0 },
    { 0xb9bc502a, 90 },
    { 0x813c502b, 91 },
    { 0x5bbc502c, 23 },
    { 0x6c8c502d, 84 },
    { 0x24dc502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2c5031, 68 },
    { 0x6ec5032, 98 },
    { 0x2d0c5032, 46 },
    { 0xeccc5034, 90 },
    { 0x89dc5035, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c5038, 34 },
    { 0x79c5038, 61 },
    { 0xa32c5038, 75 },
    { 0x2b2c5038, 98 },
    { 0x1e2c5038, 23 },
    { 0xab1c503a, 55 },
    { 0, 0 },
    { 0x8d6c503f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ac5043, 84 },
    { 0x4f0c5044, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7c5047, 91 },
    { 0x8d1c5048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627c5051, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebec5056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec505e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc5061, 48 },
    { 0, 0 },
    { 0x641c5063, 33 },
    { 0, 0 },
    { 0xcb8c5065, 33 },
    { 0x99fc5065, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1c506f, 43 },
    { 0x245c506f, 61 },
    { 0x1aec5071, 90 },
    { 0x24ac5072, 90 },
    { 0x912c5071, 61 },
    { 0xb18c5074, 95 },
    { 0xfd9c5075, 95 },
    { 0x192c506f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94c507e, 56 },
    { 0xfc507f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc5082, 61 },
    { 0xf46c5082, 23 },
    { 0x611c5084, 43 },
    { 0xfbec5084, 77 },
    { 0xa57c5086, 90 },
    { 0, 0 },
    { 0x358c5088, 33 },
    { 0xd11c5088, 61 },
    { 0xac1c508a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5dc508f, 46 },
    { 0x465c5090, 90 },
    { 0x495c5091, 90 },
    { 0xcaec5091, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2c5099, 68 },
    { 0x9d7c5099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c50a2, 93 },
    { 0x105c50a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc50a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c50a9, 17 },
    { 0x9cfc50aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871c50ae, 76 },
    { 0xf57c50af, 90 },
    { 0x9fc50b0, 61 },
    { 0xd9c50ae, 43 },
    { 0x35ac50ae, 95 },
    { 0x442c50b2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c50b9, 34 },
    { 0, 0 },
    { 0x533c50bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c50be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4c50c1, 76 },
    { 0xfd9c50c1, 100 },
    { 0xf5fc50c2, 87 },
    { 0x443c50c2, 90 },
    { 0xe72c50c3, 33 },
    { 0x361c50c6, 43 },
    { 0x901c50c7, 95 },
    { 0, 0 },
    { 0x943c50c9, 87 },
    { 0x1b0c50c9, 65 },
    { 0, 0 },
    { 0xb1c50cc, 61 },
    { 0x2adc50cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c50d6, 98 },
    { 0, 0 },
    { 0xf4dc50d8, 61 },
    { 0xfbac50d9, 34 },
    { 0x275c50da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fec50dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c50e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ac50e4, 34 },
    { 0x21cc50e5, 66 },
    { 0x79cc50e5, 90 },
    { 0, 0 },
    { 0x651c50e8, 67 },
    { 0, 0 },
    { 0xf74c50ea, 76 },
    { 0x338c50ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c50f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c50f8, 79 },
    { 0x25ac50f9, 75 },
    { 0x702c50fa, 76 },
    { 0xe0ec50fb, 77 },
    { 0x4edc50fa, 23 },
    { 0x876c50f8, 84 },
    { 0, 0 },
    { 0xbafc50ff, 95 },
    { 0, 0 },
    { 0x168c5101, 76 },
    { 0x762c5102, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbc5106, 90 },
    { 0xe0dc5107, 95 },
    { 0x24ac5108, 43 },
    { 0x458c5108, 90 },
    { 0x712c510a, 90 },
    { 0x6cdc5108, 90 },
    { 0x544c5109, 46 },
    { 0x508c5106, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x120c5110, 61 },
    { 0x7f4c5111, 55 },
    { 0xe91c5112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc5115, 99 },
    { 0x337c5116, 90 },
    { 0xa5fc5115, 98 },
    { 0x978c5118, 90 },
    { 0, 0 },
    { 0x59ec511a, 78 },
    { 0x597c511b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48c5122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cc5128, 23 },
    { 0x4f1c5129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c512c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c5131, 93 },
    { 0, 0 },
    { 0xa47c5133, 61 },
    { 0xf58c5134, 98 },
    { 0, 0 },
    { 0x904c5136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620c513b, 95 },
    { 0xb5fc513b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48c5144, 61 },
    { 0x301c5144, 33 },
    { 0x241c5146, 98 },
    { 0xf0ac5144, 66 },
    { 0, 0 },
    { 0xe7ec5149, 52 },
    { 0xb47c514a, 68 },
    { 0x288c514a, 98 },
    { 0x4adc514c, 88 },
    { 0, 0 },
    { 0xee8c514e, 90 },
    { 0x363c514e, 43 },
    { 0xcc514e, 23 },
    { 0, 0 },
    { 0xf72c5152, 68 },
    { 0xe31c5153, 91 },
    { 0xc9ac5153, 99 },
    { 0x525c5152, 34 },
    { 0x84fc5156, 84 },
    { 0xf06c5154, 55 },
    { 0xde7c5158, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5c5161, 95 },
    { 0xcafc5161, 90 },
    { 0x556c5163, 91 },
    { 0xcd5c5164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcec5168, 34 },
    { 0x2ec5168, 55 },
    { 0x8e7c5168, 52 },
    { 0xe06c516b, 46 },
    { 0x218c516c, 67 },
    { 0xd92c516a, 23 },
    { 0, 0 },
    { 0x5e9c516f, 98 },
    { 0, 0 },
    { 0xd75c5171, 76 },
    { 0, 0 },
    { 0xccac5173, 34 },
    { 0xbb9c5174, 98 },
    { 0x930c5174, 90 },
    { 0x911c5175, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c517b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c518b, 23 },
    { 0xb8bc518c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dac518f, 90 },
    { 0x70ac5190, 33 },
    { 0xb0fc5190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558c5195, 96 },
    { 0x349c5196, 46 },
    { 0xc3bc5195, 93 },
    { 0, 0 },
    { 0xa10c5199, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61c519d, 61 },
    { 0xb42c519e, 95 },
    { 0x284c519e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798c51a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c51ad, 23 },
    { 0, 0 },
    { 0x25c51af, 52 },
    { 0x62dc51af, 90 },
    { 0x185c51b1, 95 },
    { 0x59c51b1, 61 },
    { 0, 0 },
    { 0x540c51b4, 23 },
    { 0xdcac51b5, 76 },
    { 0x220c51b6, 23 },
    { 0x9d3c51b6, 99 },
    { 0x51cc51b8, 81 },
    { 0x87c51b7, 23 },
    { 0xc2ec51ba, 43 },
    { 0, 0 },
    { 0x5c51bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c51c1, 90 },
    { 0x42dc51c2, 59 },
    { 0xeecc51c3, 55 },
    { 0, 0 },
    { 0xc8cc51c5, 61 },
    { 0x13fc51c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c51ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecac51d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafac51d5, 87 },
    { 0x6b4c51d6, 23 },
    { 0xadcc51d5, 90 },
    { 0x924c51d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x111c51db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3c51e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65c51e9, 67 },
    { 0, 0 },
    { 0xee6c51eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172c51f6, 55 },
    { 0x2e4c51f6, 46 },
    { 0x7ebc51f8, 76 },
    { 0xaa4c51f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc51ff, 61 },
    { 0xaf9c5200, 95 },
    { 0, 0 },
    { 0x68cc5202, 76 },
    { 0x22c5202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c520a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c520d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c5213, 52 },
    { 0xc76c5214, 33 },
    { 0x2a8c5214, 61 },
    { 0x806c5214, 95 },
    { 0x8bec5213, 61 },
    { 0x944c5218, 95 },
    { 0x92c5217, 95 },
    { 0xa2bc521a, 90 },
    { 0x890c521b, 66 },
    { 0xac1c521b, 61 },
    { 0, 0 },
    { 0x849c521e, 90 },
    { 0xa47c521f, 55 },
    { 0xb6dc5220, 76 },
    { 0, 0 },
    { 0x9ecc5222, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc5225, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc522a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fac5231, 98 },
    { 0xe3c5232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5235, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bc523c, 91 },
    { 0, 0 },
    { 0xfe2c523e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3c5241, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc5245, 76 },
    { 0, 0 },
    { 0xedfc5247, 55 },
    { 0x33ac5248, 33 },
    { 0, 0 },
    { 0xb3fc524a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc524e, 95 },
    { 0x16fc524f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cc5252, 23 },
    { 0, 0 },
    { 0x5d7c5254, 99 },
    { 0xebbc5255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c5260, 90 },
    { 0, 0 },
    { 0x14dc5262, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc5265, 23 },
    { 0xf8c5266, 23 },
    { 0, 0 },
    { 0x58dc5268, 33 },
    { 0x579c5269, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc526d, 93 },
    { 0x683c526d, 33 },
    { 0x650c526d, 93 },
    { 0, 0 },
    { 0x37c5271, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c5277, 78 },
    { 0xd8ec5278, 66 },
    { 0xc48c5279, 61 },
    { 0x1e7c527a, 55 },
    { 0x14cc527b, 91 },
    { 0xa23c527c, 61 },
    { 0xf99c527c, 90 },
    { 0x5d6c5278, 98 },
    { 0, 0 },
    { 0xc11c5280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6c5285, 92 },
    { 0x7f9c5285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c528d, 67 },
    { 0xfa4c528e, 61 },
    { 0x2e5c528e, 90 },
    { 0xfaac528f, 72 },
    { 0x4b5c5291, 23 },
    { 0, 0 },
    { 0xe5dc5293, 61 },
    { 0x7cc5294, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644c5299, 33 },
    { 0x6e6c529a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ac529e, 90 },
    { 0x43c529f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c52a5, 61 },
    { 0x87c52a5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c52a9, 33 },
    { 0, 0 },
    { 0xef5c52ab, 85 },
    { 0x7abc52ac, 95 },
    { 0, 0 },
    { 0x1f1c52ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bc52b2, 76 },
    { 0x485c52b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc52b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8c52bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc52c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9c52c3, 17 },
    { 0x206c52c3, 68 },
    { 0, 0 },
    { 0x72dc52c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc52cd, 95 },
    { 0, 0 },
    { 0xdf6c52cf, 61 },
    { 0xe86c52cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x643c52d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec52db, 90 },
    { 0xe51c52db, 33 },
    { 0xbe4c52dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc52e1, 55 },
    { 0x500c52e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c52e6, 98 },
    { 0xd01c52e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acc52eb, 53 },
    { 0, 0 },
    { 0x95fc52ed, 90 },
    { 0xc48c52ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5c52f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c52f5, 33 },
    { 0x28ec52f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc52fa, 90 },
    { 0x301c52fb, 61 },
    { 0x2bec52fc, 49 },
    { 0x575c52fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc5303, 97 },
    { 0x1a6c5304, 33 },
    { 0x105c5305, 90 },
    { 0x93c5306, 81 },
    { 0x8c5307, 98 },
    { 0x19cc5308, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7c530c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ec530f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec5312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c5316, 95 },
    { 0xad0c5317, 81 },
    { 0, 0 },
    { 0xf46c5319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x461c531c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4c5328, 43 },
    { 0x65bc5329, 90 },
    { 0xb56c532a, 95 },
    { 0, 0 },
    { 0x6e0c532c, 66 },
    { 0, 0 },
    { 0xc61c532e, 33 },
    { 0, 0 },
    { 0x3f3c5330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c5333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bc533b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20c533e, 90 },
    { 0, 0 },
    { 0x892c5340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc5343, 33 },
    { 0x413c5343, 95 },
    { 0x372c5345, 33 },
    { 0, 0 },
    { 0x23dc5347, 96 },
    { 0, 0 },
    { 0xb1fc5349, 76 },
    { 0x583c5349, 23 },
    { 0, 0 },
    { 0x582c534c, 90 },
    { 0x1a9c534c, 93 },
    { 0x6a0c534c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9c5353, 100 },
    { 0, 0 },
    { 0x43dc5355, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac535b, 33 },
    { 0x177c535c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x591c535f, 61 },
    { 0xb88c535f, 96 },
    { 0x9c6c535f, 90 },
    { 0xa58c5360, 35 },
    { 0xac5c5363, 39 },
    { 0xad4c5364, 43 },
    { 0xd61c5365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5369, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c536c, 55 },
    { 0x6d3c536d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7c5373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9c537e, 30 },
    { 0, 0 },
    { 0x568c5380, 65 },
    { 0xbe7c5380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc5387, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec538a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c538e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c5391, 98 },
    { 0x663c5392, 33 },
    { 0, 0 },
    { 0x9aec5394, 33 },
    { 0x6e1c5395, 99 },
    { 0x753c5396, 61 },
    { 0xcc8c5397, 33 },
    { 0x7f2c5397, 66 },
    { 0xc1cc5399, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc539c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c53a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c53a3, 61 },
    { 0x5e2c53a4, 34 },
    { 0x9f0c53a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ac53a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bc53ac, 90 },
    { 0, 0 },
    { 0xf5cc53ae, 43 },
    { 0x481c53af, 90 },
    { 0, 0 },
    { 0x11fc53b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c53b5, 43 },
    { 0, 0 },
    { 0x781c53b7, 81 },
    { 0x640c53b8, 85 },
    { 0x113c53b9, 98 },
    { 0, 0 },
    { 0xe2cc53bb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c53be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c53c3, 43 },
    { 0x962c53c3, 87 },
    { 0xfb2c53c3, 90 },
    { 0x2bc53c6, 33 },
    { 0x9a8c53c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bc53ca, 95 },
    { 0x707c53ca, 97 },
    { 0, 0 },
    { 0xbc4c53cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325c53d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc53d5, 85 },
    { 0xa1cc53d5, 61 },
    { 0xa1ec53d5, 98 },
    { 0xd59c53d8, 76 },
    { 0x13fc53d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x341c53dc, 61 },
    { 0x809c53dd, 95 },
    { 0x73c53de, 95 },
    { 0x4a7c53dd, 95 },
    { 0xf08c53e0, 66 },
    { 0xd58c53e1, 77 },
    { 0x6a4c53de, 38 },
    { 0, 0 },
    { 0x7f4c53e4, 78 },
    { 0, 0 },
    { 0xffec53e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2c53ea, 95 },
    { 0, 0 },
    { 0xce5c53ec, 96 },
    { 0xe19c53ed, 74 },
    { 0x8d2c53ee, 61 },
    { 0x48fc53ef, 99 },
    { 0, 0 },
    { 0x4f6c53f1, 88 },
    { 0xeddc53f2, 91 },
    { 0, 0 },
    { 0x9c5c53f4, 90 },
    { 0, 0 },
    { 0xbb1c53f6, 90 },
    { 0xb99c53f7, 33 },
    { 0x722c53f6, 66 },
    { 0x414c53f9, 90 },
    { 0x277c53f7, 55 },
    { 0x808c53f6, 100 },
    { 0x217c53fc, 100 },
    { 0x2fec53f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x536c5400, 94 },
    { 0x59c5401, 46 },
    { 0xc57c5400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c5407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c5411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc5414, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x208c5417, 33 },
    { 0x942c5418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340c541c, 37 },
    { 0x3bcc541d, 90 },
    { 0x681c541d, 61 },
    { 0, 0 },
    { 0x9f1c5420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2c5428, 61 },
    { 0xebfc5428, 78 },
    { 0x93fc5428, 90 },
    { 0x5bcc542b, 100 },
    { 0x7c8c542b, 33 },
    { 0xd1cc5428, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec5430, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c5435, 95 },
    { 0x9a1c5435, 34 },
    { 0x3f5c5437, 90 },
    { 0x5aac5437, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c543e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cc5445, 95 },
    { 0x99cc5445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c544a, 83 },
    { 0, 0 },
    { 0xa73c544c, 33 },
    { 0x8cac544c, 90 },
    { 0xd05c544c, 84 },
    { 0, 0 },
    { 0x3e5c5450, 99 },
    { 0x47ac5451, 88 },
    { 0xa31c5452, 95 },
    { 0x907c5453, 34 },
    { 0, 0 },
    { 0x8e2c5455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bc5458, 69 },
    { 0x84dc5459, 91 },
    { 0x4b3c545a, 46 },
    { 0xc2cc5458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c545e, 87 },
    { 0xc2bc545f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c5465, 66 },
    { 0xe00c5465, 33 },
    { 0x322c5466, 33 },
    { 0x204c5467, 55 },
    { 0, 0 },
    { 0x53ac546a, 66 },
    { 0xb73c546a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64c546e, 95 },
    { 0xb23c546f, 33 },
    { 0x48fc5470, 65 },
    { 0x412c5471, 66 },
    { 0x249c546e, 61 },
    { 0xcd3c5471, 46 },
    { 0xc6cc5474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08c5479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfc547f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264c5483, 55 },
    { 0, 0 },
    { 0x4f9c5485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x208c5488, 85 },
    { 0x5d4c5489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c548e, 61 },
    { 0x9f7c548f, 95 },
    { 0, 0 },
    { 0x3d7c5491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dc549a, 98 },
    { 0xdffc549a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c549e, 33 },
    { 0, 0 },
    { 0xa53c54a0, 40 },
    { 0, 0 },
    { 0xea4c54a2, 93 },
    { 0x869c54a3, 33 },
    { 0x510c54a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c54a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fc54ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cc54b1, 61 },
    { 0xe22c54b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11c54bc, 61 },
    { 0x595c54bd, 23 },
    { 0, 0 },
    { 0x3a7c54bf, 90 },
    { 0x858c54c0, 95 },
    { 0, 0 },
    { 0xe06c54c2, 33 },
    { 0x65ec54c2, 61 },
    { 0, 0 },
    { 0xfe8c54c5, 90 },
    { 0x990c54c5, 90 },
    { 0, 0 },
    { 0x293c54c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebc54cb, 43 },
    { 0, 0 },
    { 0x8dcc54cd, 61 },
    { 0, 0 },
    { 0x143c54cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc54d2, 64 },
    { 0, 0 },
    { 0x105c54d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3c54db, 23 },
    { 0x328c54dc, 66 },
    { 0, 0 },
    { 0x716c54de, 31 },
    { 0xfafc54df, 23 },
    { 0x6bbc54e0, 95 },
    { 0x634c54de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c54e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bc54e9, 98 },
    { 0, 0 },
    { 0x6aac54eb, 88 },
    { 0, 0 },
    { 0x484c54ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775c54f2, 67 },
    { 0x7c1c54f3, 23 },
    { 0x472c54f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9c54f7, 76 },
    { 0x9eec54f8, 95 },
    { 0, 0 },
    { 0xc1ec54fa, 90 },
    { 0, 0 },
    { 0xe8c54fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fc54ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c5505, 93 },
    { 0x97fc5505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4c550f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfc5516, 61 },
    { 0x4f9c5517, 61 },
    { 0, 0 },
    { 0x1c2c5519, 61 },
    { 0, 0 },
    { 0xa2cc551b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6c551e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c5523, 67 },
    { 0xa2ac5524, 33 },
    { 0, 0 },
    { 0xb22c5526, 23 },
    { 0, 0 },
    { 0x75cc5528, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec552e, 64 },
    { 0, 0 },
    { 0xd88c5530, 33 },
    { 0x17cc5531, 98 },
    { 0x462c5532, 61 },
    { 0x1acc5533, 34 },
    { 0x6bc5534, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4c5538, 33 },
    { 0x4eec5539, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2c553c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358c5541, 55 },
    { 0, 0 },
    { 0x8abc5543, 43 },
    { 0, 0 },
    { 0x4e0c5545, 98 },
    { 0xbe6c5546, 90 },
    { 0xb09c5546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342c554c, 90 },
    { 0, 0 },
    { 0x968c554e, 33 },
    { 0xf29c554f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5558, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x975c555b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915c5561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c5565, 34 },
    { 0xb91c5566, 84 },
    { 0xcf6c5567, 52 },
    { 0x2a1c5567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2c556c, 33 },
    { 0, 0 },
    { 0x4dcc556e, 61 },
    { 0x652c556f, 98 },
    { 0x1c3c556e, 98 },
    { 0x799c556f, 99 },
    { 0, 0 },
    { 0x4ecc5573, 98 },
    { 0xc27c5574, 53 },
    { 0, 0 },
    { 0x64ec5576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c557a, 98 },
    { 0, 0 },
    { 0x79ac557c, 90 },
    { 0xf32c557c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac5582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994c5588, 100 },
    { 0, 0 },
    { 0x58ec558a, 91 },
    { 0x329c558a, 66 },
    { 0x713c558a, 90 },
    { 0x327c558d, 33 },
    { 0x6d1c558c, 98 },
    { 0, 0 },
    { 0x367c5590, 61 },
    { 0x990c5591, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cc5594, 33 },
    { 0xb3dc5595, 95 },
    { 0x47ec5596, 34 },
    { 0xbd9c5597, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c55ab, 34 },
    { 0, 0 },
    { 0x394c55ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x190c55b0, 90 },
    { 0xe21c55b0, 90 },
    { 0xcf4c55b2, 33 },
    { 0x3b9c55b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c55b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c55bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892c55c1, 90 },
    { 0xdaec55c2, 61 },
    { 0, 0 },
    { 0x7a1c55c4, 95 },
    { 0xdeec55c5, 55 },
    { 0x384c55c5, 95 },
    { 0xe8fc55c6, 61 },
    { 0x918c55c7, 95 },
    { 0x7bec55c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ec55cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc55d3, 63 },
    { 0, 0 },
    { 0xff0c55d5, 90 },
    { 0, 0 },
    { 0x171c55d7, 33 },
    { 0xe32c55d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c55dc, 100 },
    { 0x850c55dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c55e3, 33 },
    { 0xe9cc55e3, 90 },
    { 0xafec55e5, 90 },
    { 0x166c55e5, 33 },
    { 0x761c55e7, 95 },
    { 0x127c55e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fc55ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699c55f0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc55f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c55fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c55fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c5600, 98 },
    { 0x656c5600, 61 },
    { 0xa2c5601, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ec560b, 43 },
    { 0x2c6c560b, 95 },
    { 0xb41c560d, 95 },
    { 0x8b6c560b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c5616, 80 },
    { 0, 0 },
    { 0xe05c5618, 61 },
    { 0xa3cc5619, 46 },
    { 0xebcc5618, 98 },
    { 0x2a4c561b, 85 },
    { 0x412c5619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dc5621, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c5629, 17 },
    { 0, 0 },
    { 0x322c562b, 77 },
    { 0x9f0c562b, 61 },
    { 0x29c562d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c5632, 67 },
    { 0, 0 },
    { 0x493c5634, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c563e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161c5643, 61 },
    { 0x7c3c5644, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310c564a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c564d, 95 },
    { 0, 0 },
    { 0x883c564f, 61 },
    { 0x2f3c5650, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372c5654, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7c5658, 76 },
    { 0, 0 },
    { 0x3f0c565a, 88 },
    { 0x2e3c565b, 67 },
    { 0x8f0c565b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06c5660, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25c5666, 33 },
    { 0xa8cc5667, 93 },
    { 0x807c5668, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58c5670, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243c5675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x204c5678, 55 },
    { 0, 0 },
    { 0x521c567a, 90 },
    { 0x8d0c567a, 81 },
    { 0xbadc567c, 61 },
    { 0xda9c567d, 91 },
    { 0x4c4c567d, 95 },
    { 0x49dc567f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c5683, 55 },
    { 0x41cc5684, 61 },
    { 0x429c5684, 61 },
    { 0, 0 },
    { 0x7dac5687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ac568b, 90 },
    { 0x669c568c, 52 },
    { 0x4e2c568d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc5697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c569b, 90 },
    { 0x383c569b, 55 },
    { 0x251c569b, 98 },
    { 0x848c569e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1c56a4, 61 },
    { 0, 0 },
    { 0x899c56a6, 76 },
    { 0xac9c56a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813c56ad, 61 },
    { 0, 0 },
    { 0x830c56af, 33 },
    { 0xe46c56af, 90 },
    { 0xe6ec56b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245c56b9, 33 },
    { 0, 0 },
    { 0xc76c56bb, 33 },
    { 0x970c56bb, 90 },
    { 0xe68c56bd, 98 },
    { 0xe41c56bb, 98 },
    { 0x294c56bf, 61 },
    { 0x9a1c56bf, 33 },
    { 0x917c56c1, 98 },
    { 0xbc0c56c2, 23 },
    { 0, 0 },
    { 0xa41c56c4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2c56c8, 23 },
    { 0x9f8c56c9, 76 },
    { 0xafc56c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dc56cd, 33 },
    { 0x1e9c56cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacc56d3, 76 },
    { 0x25bc56d4, 90 },
    { 0xcc6c56d5, 90 },
    { 0x1cac56d6, 98 },
    { 0, 0 },
    { 0xd3cc56d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74c56dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c56e0, 23 },
    { 0, 0 },
    { 0x958c56e2, 93 },
    { 0xd7dc56e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c56e7, 33 },
    { 0x88dc56e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c56eb, 23 },
    { 0xd49c56ec, 61 },
    { 0xab1c56ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ac56f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17c56f5, 98 },
    { 0xf75c56f6, 76 },
    { 0x1d2c56f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadc56ff, 52 },
    { 0x318c5700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c5703, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cc5707, 90 },
    { 0x9b9c5708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c570c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22c570f, 95 },
    { 0x86cc570f, 33 },
    { 0xeecc5711, 43 },
    { 0x768c5711, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c5715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc5718, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac571d, 66 },
    { 0, 0 },
    { 0x402c571f, 95 },
    { 0xf9fc5720, 61 },
    { 0xdc2c5721, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3c572a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8c572e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ac5731, 61 },
    { 0, 0 },
    { 0xe0bc5733, 53 },
    { 0x8cec5734, 33 },
    { 0xbb3c5734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c5738, 34 },
    { 0xd5dc5739, 23 },
    { 0x651c5739, 61 },
    { 0x756c573a, 33 },
    { 0x7d6c573c, 91 },
    { 0x9eac573c, 33 },
    { 0xf43c573e, 91 },
    { 0xe51c5739, 33 },
    { 0x836c5738, 33 },
    { 0xa6ac573d, 53 },
    { 0xf4c5742, 98 },
    { 0xe5dc5738, 95 },
    { 0xb43c5744, 43 },
    { 0xe83c5745, 61 },
    { 0x2f2c5744, 98 },
    { 0xd6dc5744, 76 },
    { 0xc35c5748, 87 },
    { 0, 0 },
    { 0x27bc574a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c574e, 43 },
    { 0x5eec574e, 90 },
    { 0x613c5750, 23 },
    { 0x34ec5751, 55 },
    { 0x11fc5751, 61 },
    { 0x63ec5752, 61 },
    { 0xdd0c5751, 98 },
    { 0xaaac5754, 66 },
    { 0, 0 },
    { 0x9ec5757, 66 },
    { 0x5fac5758, 40 },
    { 0x739c5759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x508c575c, 77 },
    { 0x96dc575c, 84 },
    { 0x89cc575c, 33 },
    { 0x73bc575c, 90 },
    { 0x5f9c575e, 91 },
    { 0, 0 },
    { 0x472c5762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fc5765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc5768, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c5778, 61 },
    { 0x188c5779, 43 },
    { 0x1a7c5779, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6c577d, 43 },
    { 0x4c0c577e, 84 },
    { 0x414c577e, 76 },
    { 0, 0 },
    { 0xf9ec5781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac5787, 55 },
    { 0x47cc5788, 77 },
    { 0x189c5787, 90 },
    { 0, 0 },
    { 0x87cc578b, 68 },
    { 0, 0 },
    { 0xc52c578d, 98 },
    { 0, 0 },
    { 0x59ac578f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x690c5792, 33 },
    { 0x524c5792, 98 },
    { 0, 0 },
    { 0xfaec5795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579c5799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x611c579c, 84 },
    { 0x5a6c579c, 95 },
    { 0x566c579e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c57a2, 23 },
    { 0x66cc57a2, 33 },
    { 0x1f6c57a4, 61 },
    { 0xdadc57a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac57a8, 61 },
    { 0x943c57a9, 90 },
    { 0, 0 },
    { 0xa15c57ab, 78 },
    { 0xa68c57ac, 33 },
    { 0x577c57ad, 96 },
    { 0xfc3c57ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x535c57b1, 55 },
    { 0, 0 },
    { 0xc35c57b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c57b6, 91 },
    { 0, 0 },
    { 0xf21c57b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdc57c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dc57c6, 90 },
    { 0x62cc57c7, 98 },
    { 0x447c57c7, 90 },
    { 0xcac57c8, 33 },
    { 0, 0 },
    { 0x987c57cb, 61 },
    { 0x4a5c57cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3c57cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac57d3, 76 },
    { 0x6b7c57d3, 90 },
    { 0xf1ac57d4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880c57da, 34 },
    { 0xd2ac57db, 52 },
    { 0x82c57dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fac57e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ec57e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc57e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9c57eb, 66 },
    { 0, 0 },
    { 0xa00c57ed, 65 },
    { 0x168c57ee, 90 },
    { 0, 0 },
    { 0x34cc57f0, 98 },
    { 0x767c57f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c57f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0c57f8, 61 },
    { 0x4b0c57f9, 90 },
    { 0xb16c57f9, 46 },
    { 0, 0 },
    { 0x6b3c57fc, 90 },
    { 0xd4fc57fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x258c5800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1c5806, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac580b, 98 },
    { 0x2d0c580c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec5810, 55 },
    { 0, 0 },
    { 0xae1c5812, 40 },
    { 0x6e4c5813, 90 },
    { 0xbd3c5814, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x207c5817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966c581c, 33 },
    { 0, 0 },
    { 0xe74c581e, 95 },
    { 0xf9fc581f, 95 },
    { 0x373c581f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ec5823, 77 },
    { 0x2b0c5823, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9c5828, 55 },
    { 0, 0 },
    { 0xd75c582a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac582e, 46 },
    { 0xd42c582f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33c5833, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cc5836, 76 },
    { 0x9c7c5837, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372c583b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cc5843, 95 },
    { 0xe28c5844, 90 },
    { 0x382c5845, 38 },
    { 0xce5c5846, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600c584d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c5850, 43 },
    { 0, 0 },
    { 0x779c5852, 98 },
    { 0xe0fc5853, 90 },
    { 0x2d1c5853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0c585a, 90 },
    { 0, 0 },
    { 0x324c585c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c5863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c5868, 33 },
    { 0x8d7c5868, 55 },
    { 0, 0 },
    { 0x457c586b, 43 },
    { 0, 0 },
    { 0x427c586d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec5872, 90 },
    { 0xb91c5873, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c5878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c587b, 64 },
    { 0, 0 },
    { 0xec4c587d, 98 },
    { 0, 0 },
    { 0x835c587f, 95 },
    { 0, 0 },
    { 0xe8ac5881, 77 },
    { 0xd8cc5882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7c5885, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc588b, 23 },
    { 0x4f4c588b, 61 },
    { 0x288c588c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc5897, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cc589b, 95 },
    { 0, 0 },
    { 0x5bcc589d, 99 },
    { 0, 0 },
    { 0xbb1c589f, 34 },
    { 0x90ec58a0, 88 },
    { 0xf28c58a1, 78 },
    { 0x6cfc58a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc58a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4c58a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922c58af, 23 },
    { 0x6abc58af, 90 },
    { 0xa6c58b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c58b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bc58b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6c58bb, 33 },
    { 0, 0 },
    { 0x2acc58bd, 100 },
    { 0, 0 },
    { 0xfdc58bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c58c3, 93 },
    { 0, 0 },
    { 0xa54c58c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdc58ca, 61 },
    { 0x9b6c58cb, 90 },
    { 0x5edc58cb, 98 },
    { 0x313c58cc, 90 },
    { 0xd58c58cc, 48 },
    { 0, 0 },
    { 0x6b3c58d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79c58d3, 55 },
    { 0, 0 },
    { 0xe7bc58d5, 90 },
    { 0, 0 },
    { 0x5e4c58d7, 95 },
    { 0x498c58d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cec58de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0c58e4, 77 },
    { 0x757c58e5, 23 },
    { 0x709c58e6, 67 },
    { 0x105c58e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3c58ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c58f4, 33 },
    { 0xf4fc58f5, 99 },
    { 0x27ac58f5, 90 },
    { 0x7edc58f7, 99 },
    { 0xcc3c58f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3c58fd, 38 },
    { 0xe74c58fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8c5903, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc5906, 96 },
    { 0xf05c5907, 33 },
    { 0xa71c5908, 95 },
    { 0xff5c5907, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dac590f, 90 },
    { 0xea3c590f, 43 },
    { 0x197c5910, 33 },
    { 0xf85c5910, 4 },
    { 0xedec5913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc5916, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9c5919, 76 },
    { 0, 0 },
    { 0x3e3c591b, 95 },
    { 0x42bc591c, 98 },
    { 0, 0 },
    { 0x6dbc591e, 99 },
    { 0xbd8c591e, 23 },
    { 0xd32c591e, 65 },
    { 0xe65c5921, 67 },
    { 0x2e9c5920, 80 },
    { 0x42ec5923, 90 },
    { 0x157c5923, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dc5927, 90 },
    { 0x4efc5928, 61 },
    { 0x4efc5929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dc592c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299c5930, 76 },
    { 0, 0 },
    { 0x912c5932, 40 },
    { 0x486c5933, 55 },
    { 0, 0 },
    { 0x9ddc5935, 100 },
    { 0xfc0c5936, 90 },
    { 0x3a0c5937, 61 },
    { 0, 0 },
    { 0x89c5939, 90 },
    { 0xd0bc593a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c5941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74c5944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362c5948, 90 },
    { 0x7e5c5949, 23 },
    { 0xb08c594a, 98 },
    { 0x264c5948, 95 },
    { 0x2aec5948, 36 },
    { 0, 0 },
    { 0x72ac594e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fc5959, 98 },
    { 0x526c595a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc595e, 90 },
    { 0xddbc595f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977c5965, 91 },
    { 0, 0 },
    { 0x667c5967, 55 },
    { 0x7cbc5968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc596d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc5971, 91 },
    { 0xdfdc5972, 78 },
    { 0x524c5971, 43 },
    { 0xe56c5974, 90 },
    { 0xdd8c5975, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c597b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2c597f, 95 },
    { 0x618c5980, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ac5984, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24c5987, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8c598f, 93 },
    { 0x26dc598f, 34 },
    { 0xfafc5991, 33 },
    { 0x7bec5991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc5995, 55 },
    { 0, 0 },
    { 0x975c5997, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c599a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dc59a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2c59a6, 98 },
    { 0, 0 },
    { 0xeefc59a8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c59ac, 94 },
    { 0xed3c59ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cc59b3, 101 },
    { 0x3b4c59b4, 95 },
    { 0xad1c59b5, 55 },
    { 0xfc0c59b5, 47 },
    { 0x5f1c59b7, 90 },
    { 0xca7c59b6, 90 },
    { 0x888c59b4, 95 },
    { 0x953c59b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6c59bf, 90 },
    { 0, 0 },
    { 0x3a6c59c1, 55 },
    { 0x19c59c2, 43 },
    { 0x2a3c59c1, 90 },
    { 0x8ac59c4, 52 },
    { 0xb1ac59c5, 76 },
    { 0xba6c59c6, 98 },
    { 0x66ac59c5, 83 },
    { 0xa1bc59c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13c59cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c59d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c59d7, 34 },
    { 0x60ec59d8, 90 },
    { 0x372c59d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc59dc, 46 },
    { 0x498c59dd, 93 },
    { 0, 0 },
    { 0x99ec59df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c59e2, 100 },
    { 0xb05c59e3, 76 },
    { 0x8e1c59e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc59ec, 75 },
    { 0xb80c59ed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c59f2, 34 },
    { 0xc59c59f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c59fd, 61 },
    { 0xc81c59fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c5a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ac5a07, 85 },
    { 0x3c6c5a07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618c5a14, 34 },
    { 0, 0 },
    { 0xc97c5a16, 55 },
    { 0, 0 },
    { 0x12dc5a18, 43 },
    { 0x4a7c5a18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c5a1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc5a21, 98 },
    { 0x94dc5a22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc5a27, 90 },
    { 0x1efc5a28, 84 },
    { 0xa11c5a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac5a35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c5a39, 23 },
    { 0xdecc5a3a, 90 },
    { 0xa61c5a39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ec5a3f, 33 },
    { 0xf38c5a3f, 95 },
    { 0xf4c5a3f, 98 },
    { 0x344c5a42, 52 },
    { 0x9dac5a43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2c5a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c5a4e, 93 },
    { 0, 0 },
    { 0xf86c5a50, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c5a53, 23 },
    { 0xc2c5a54, 78 },
    { 0x2b6c5a53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1c5a59, 84 },
    { 0x6c5c5a59, 33 },
    { 0x685c5a5a, 99 },
    { 0x76fc5a5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac5a5f, 55 },
    { 0xcf0c5a5f, 90 },
    { 0, 0 },
    { 0xaf2c5a62, 88 },
    { 0x232c5a63, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c5a68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242c5a6c, 40 },
    { 0xabcc5a6d, 33 },
    { 0xd7fc5a6d, 33 },
    { 0xeb4c5a6f, 76 },
    { 0x1acc5a70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c5a73, 61 },
    { 0x7bcc5a73, 99 },
    { 0x27dc5a74, 95 },
    { 0x749c5a76, 33 },
    { 0x7c7c5a75, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c5a7d, 78 },
    { 0x435c5a7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ac5a81, 33 },
    { 0x968c5a82, 24 },
    { 0x53ec5a82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31c5a87, 55 },
    { 0x7aac5a88, 23 },
    { 0xf9ac5a88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c5a8c, 52 },
    { 0x754c5a8c, 84 },
    { 0x3bcc5a8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c5a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac5a96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5a99, 23 },
    { 0x296c5a99, 90 },
    { 0xd15c5a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c5a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c5aa1, 76 },
    { 0x210c5aa1, 64 },
    { 0x795c5aa3, 90 },
    { 0, 0 },
    { 0x641c5aa5, 55 },
    { 0, 0 },
    { 0x9cfc5aa7, 85 },
    { 0xadbc5aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c5aab, 23 },
    { 0x803c5aac, 95 },
    { 0x1dfc5aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769c5ab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec5aba, 33 },
    { 0xe72c5abb, 95 },
    { 0, 0 },
    { 0xa6dc5abd, 33 },
    { 0xd46c5abe, 66 },
    { 0xed1c5abf, 69 },
    { 0, 0 },
    { 0xeafc5ac1, 17 },
    { 0xf30c5ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5ac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5c5acf, 90 },
    { 0, 0 },
    { 0x951c5ad1, 23 },
    { 0x39bc5ad2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74c5ad6, 61 },
    { 0, 0 },
    { 0x8fac5ad8, 67 },
    { 0x689c5ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac5ade, 55 },
    { 0x8dac5adf, 52 },
    { 0xf58c5ae0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acc5ae3, 90 },
    { 0xc21c5ae3, 98 },
    { 0, 0 },
    { 0x7e8c5ae6, 93 },
    { 0x68fc5ae7, 88 },
    { 0x684c5ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5af2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c5aff, 61 },
    { 0x721c5b00, 55 },
    { 0x671c5b01, 23 },
    { 0x820c5b01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c5b06, 33 },
    { 0, 0 },
    { 0x7c0c5b08, 23 },
    { 0, 0 },
    { 0x54dc5b0a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bc5b0e, 46 },
    { 0x91ec5b0f, 93 },
    { 0x879c5b10, 100 },
    { 0x77c5b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc5b14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ac5b17, 78 },
    { 0, 0 },
    { 0x36fc5b19, 34 },
    { 0x2e3c5b19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34c5b1d, 66 },
    { 0xbcac5b1e, 98 },
    { 0xacc5b1f, 93 },
    { 0, 0 },
    { 0xc41c5b21, 90 },
    { 0, 0 },
    { 0xc9c5b23, 90 },
    { 0, 0 },
    { 0x41c5b25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cc5b31, 93 },
    { 0x585c5b32, 33 },
    { 0x73bc5b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbc5b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391c5b40, 95 },
    { 0xa2fc5b41, 95 },
    { 0xd09c5b42, 46 },
    { 0x85dc5b43, 33 },
    { 0x110c5b43, 61 },
    { 0x11c5b45, 23 },
    { 0x218c5b43, 90 },
    { 0xd37c5b47, 34 },
    { 0, 0 },
    { 0x4d8c5b49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65c5b4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63c5b5c, 98 },
    { 0x1f1c5b5d, 90 },
    { 0, 0 },
    { 0x7c2c5b5f, 98 },
    { 0x217c5b5f, 33 },
    { 0x4d6c5b60, 97 },
    { 0x174c5b62, 66 },
    { 0x88fc5b62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341c5b67, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69c5b6a, 66 },
    { 0x314c5b6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x451c5b6e, 90 },
    { 0, 0 },
    { 0x93bc5b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdc5b74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9c5b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c5b80, 61 },
    { 0x9f4c5b81, 61 },
    { 0x523c5b80, 66 },
    { 0x38ec5b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcac5b88, 43 },
    { 0xbedc5b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419c5b8e, 98 },
    { 0x2bbc5b8f, 34 },
    { 0xbafc5b90, 81 },
    { 0, 0 },
    { 0xa59c5b92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c5b96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c5b9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cc5b9e, 55 },
    { 0x289c5b9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5c5ba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbc5ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4c5ba8, 33 },
    { 0x733c5ba9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c5bac, 46 },
    { 0x1cac5bad, 23 },
    { 0, 0 },
    { 0x1ecc5baf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c5bb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67c5bbb, 61 },
    { 0xde3c5bbb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c5bc4, 55 },
    { 0xa6ec5bc4, 90 },
    { 0, 0 },
    { 0xcedc5bc7, 98 },
    { 0x390c5bc8, 61 },
    { 0, 0 },
    { 0x3a0c5bca, 34 },
    { 0x61c5bca, 33 },
    { 0x1a1c5bcc, 60 },
    { 0x1afc5bcd, 98 },
    { 0x9a2c5bce, 76 },
    { 0xb53c5bcd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c5bd8, 23 },
    { 0x807c5bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c5bdc, 40 },
    { 0, 0 },
    { 0x28cc5bde, 24 },
    { 0xe1bc5bdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8c5be2, 95 },
    { 0x4ffc5be3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dec5be6, 93 },
    { 0x222c5be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c5beb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec5bef, 95 },
    { 0x16c5bf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c5bf4, 34 },
    { 0xd9c5bf5, 100 },
    { 0, 0 },
    { 0xb35c5bf7, 55 },
    { 0xd9ac5bf8, 95 },
    { 0, 0 },
    { 0x617c5bfa, 66 },
    { 0xd0cc5bfb, 43 },
    { 0, 0 },
    { 0xd95c5bfd, 33 },
    { 0x7aac5bfe, 90 },
    { 0x989c5bfe, 95 },
    { 0, 0 },
    { 0x4fcc5c01, 80 },
    { 0, 0 },
    { 0x648c5c03, 66 },
    { 0x41bc5c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc5c07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c5c0d, 33 },
    { 0xc8bc5c0e, 90 },
    { 0xb47c5c0d, 66 },
    { 0, 0 },
    { 0x11cc5c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cc5c14, 95 },
    { 0, 0 },
    { 0xb37c5c16, 95 },
    { 0xb41c5c17, 33 },
    { 0x347c5c18, 61 },
    { 0xef2c5c19, 78 },
    { 0xed2c5c19, 95 },
    { 0x2f5c5c17, 90 },
    { 0, 0 },
    { 0xe90c5c1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c5c20, 99 },
    { 0, 0 },
    { 0xc0ac5c22, 98 },
    { 0xa3c5c23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efc5c28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dc5c2d, 93 },
    { 0x861c5c2e, 40 },
    { 0x7f6c5c2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34c5c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc5c37, 23 },
    { 0x81ec5c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x586c5c3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586c5c44, 55 },
    { 0x358c5c44, 91 },
    { 0, 0 },
    { 0x42cc5c47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102c5c4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfc5c54, 95 },
    { 0, 0 },
    { 0x544c5c56, 95 },
    { 0x580c5c57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0c5c5c, 99 },
    { 0x970c5c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1c5c60, 90 },
    { 0xaa0c5c61, 33 },
    { 0, 0 },
    { 0x227c5c63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fc5c6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c5c6e, 88 },
    { 0xa45c5c6f, 98 },
    { 0x5c8c5c70, 61 },
    { 0, 0 },
    { 0x954c5c72, 61 },
    { 0xa9c5c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811c5c7a, 95 },
    { 0xa68c5c7b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bc5c7e, 100 },
    { 0x1c9c5c7e, 76 },
    { 0x68fc5c7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39c5c83, 99 },
    { 0x665c5c83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fc5c87, 61 },
    { 0x7f3c5c88, 34 },
    { 0xb69c5c88, 23 },
    { 0, 0 },
    { 0xcbdc5c8b, 78 },
    { 0x760c5c8b, 95 },
    { 0, 0 },
    { 0x931c5c8e, 95 },
    { 0x7b1c5c8e, 51 },
    { 0, 0 },
    { 0xe3ac5c91, 81 },
    { 0, 0 },
    { 0x74c5c93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239c5c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x105c5c9a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746c5c9f, 93 },
    { 0x1cac5ca0, 76 },
    { 0x32dc5ca1, 61 },
    { 0x7a6c5ca2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edc5ca6, 67 },
    { 0x368c5ca7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c5cac, 98 },
    { 0, 0 },
    { 0x9aec5cae, 95 },
    { 0x42cc5caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dc5cb2, 61 },
    { 0x7aec5cb2, 84 },
    { 0x658c5cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x394c5cb7, 90 },
    { 0x324c5cb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5cbb, 46 },
    { 0x113c5cbc, 46 },
    { 0xa57c5cbc, 23 },
    { 0xc09c5cbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597c5cc2, 90 },
    { 0x612c5cc3, 98 },
    { 0x248c5cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec5cc9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec5ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0c5cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c5cd9, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c5cdd, 61 },
    { 0xa1bc5cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c5ce9, 93 },
    { 0, 0 },
    { 0x7c5ceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec5cef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c5cf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x277c5cf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bec5cfc, 95 },
    { 0x294c5cfd, 95 },
    { 0, 0 },
    { 0xc61c5cff, 90 },
    { 0, 0 },
    { 0x800c5d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c5d06, 34 },
    { 0xa7cc5d07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c5d0b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fc5d0e, 67 },
    { 0xc60c5d0f, 33 },
    { 0, 0 },
    { 0x3d1c5d11, 61 },
    { 0xeebc5d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fc5d1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c5d1e, 90 },
    { 0x710c5d1f, 98 },
    { 0x617c5d20, 33 },
    { 0, 0 },
    { 0xd3fc5d22, 95 },
    { 0xb4cc5d23, 23 },
    { 0x6e8c5d23, 93 },
    { 0x311c5d25, 90 },
    { 0x755c5d26, 33 },
    { 0xb2dc5d27, 17 },
    { 0xcf1c5d26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82c5d2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c5d30, 84 },
    { 0x8ccc5d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4c5d34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c5d38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c5d3d, 33 },
    { 0, 0 },
    { 0xde1c5d3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c5d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839c5d47, 90 },
    { 0x99ac5d48, 96 },
    { 0, 0 },
    { 0x64dc5d4a, 95 },
    { 0x55cc5d4b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c5d4e, 90 },
    { 0x5d8c5d4e, 98 },
    { 0xe1c5d50, 33 },
    { 0xb67c5d51, 43 },
    { 0, 0 },
    { 0x451c5d53, 95 },
    { 0x62ac5d53, 98 },
    { 0, 0 },
    { 0xaac5d56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c5d5b, 61 },
    { 0, 0 },
    { 0x22fc5d5d, 61 },
    { 0xd1ac5d5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc5d62, 53 },
    { 0xf2bc5d63, 98 },
    { 0xc9ec5d63, 98 },
    { 0x801c5d65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2c5d6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c5d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec5d7c, 55 },
    { 0x599c5d7d, 33 },
    { 0xe56c5d7e, 76 },
    { 0xeb5c5d7f, 76 },
    { 0x8e2c5d7c, 95 },
    { 0xc53c5d81, 98 },
    { 0, 0 },
    { 0x822c5d83, 98 },
    { 0, 0 },
    { 0x74c5d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ac5d8c, 17 },
    { 0xc21c5d8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c5d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ac5d96, 90 },
    { 0x21cc5d97, 98 },
    { 0x347c5d98, 23 },
    { 0xc72c5d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc5d9c, 95 },
    { 0xe4dc5d9c, 90 },
    { 0, 0 },
    { 0x5cac5d9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c5da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc5da8, 78 },
    { 0x8a5c5da8, 23 },
    { 0x621c5daa, 33 },
    { 0x606c5dab, 98 },
    { 0xd06c5dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46c5daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac5db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c5db5, 75 },
    { 0x667c5db6, 52 },
    { 0xed7c5db5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64c5dbe, 95 },
    { 0xce1c5dbe, 34 },
    { 0xb34c5dbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c5dc4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c5dc7, 98 },
    { 0, 0 },
    { 0x7f8c5dc9, 46 },
    { 0, 0 },
    { 0x4a5c5dcb, 61 },
    { 0x1bfc5dcc, 98 },
    { 0, 0 },
    { 0x91c5dce, 98 },
    { 0x45fc5dcf, 90 },
    { 0x2d2c5dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c5dd7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59c5ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ec5de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc5def, 61 },
    { 0x645c5df0, 33 },
    { 0x41ec5df1, 65 },
    { 0, 0 },
    { 0xa99c5df3, 98 },
    { 0x144c5df4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c5df8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c5dfb, 61 },
    { 0x56c5dfb, 69 },
    { 0xa46c5dfd, 61 },
    { 0, 0 },
    { 0xf6ac5dff, 61 },
    { 0, 0 },
    { 0x376c5e01, 99 },
    { 0x86c5e02, 98 },
    { 0x3f9c5e01, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec5e0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ac5e11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c5e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c5e17, 34 },
    { 0, 0 },
    { 0x103c5e19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636c5e26, 61 },
    { 0x2b7c5e27, 23 },
    { 0, 0 },
    { 0x63ac5e29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c5e2c, 75 },
    { 0, 0 },
    { 0xe19c5e2e, 34 },
    { 0x2b6c5e2e, 84 },
    { 0x7dc5e30, 76 },
    { 0xfa5c5e30, 23 },
    { 0x843c5e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc5e35, 33 },
    { 0xd7fc5e36, 66 },
    { 0xa76c5e37, 76 },
    { 0, 0 },
    { 0x6e4c5e39, 95 },
    { 0xd73c5e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc5e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bc5e40, 46 },
    { 0, 0 },
    { 0x3eac5e42, 33 },
    { 0x889c5e43, 90 },
    { 0, 0 },
    { 0xc82c5e45, 61 },
    { 0x72c5e45, 34 },
    { 0xf92c5e47, 93 },
    { 0x48bc5e48, 98 },
    { 0, 0 },
    { 0xbeec5e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfc5e51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6c5e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac5e59, 55 },
    { 0x5d6c5e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc5e5d, 46 },
    { 0x617c5e5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c5e62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6c5e66, 95 },
    { 0, 0 },
    { 0x6a7c5e68, 91 },
    { 0, 0 },
    { 0xc99c5e6a, 23 },
    { 0x702c5e6b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cc5e70, 90 },
    { 0xde3c5e70, 98 },
    { 0xdd5c5e72, 23 },
    { 0x52ac5e71, 67 },
    { 0, 0 },
    { 0xde6c5e75, 77 },
    { 0xcf8c5e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac5e7b, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935c5e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59c5e8d, 98 },
    { 0, 0 },
    { 0xfdc5e8f, 55 },
    { 0, 0 },
    { 0x4a9c5e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc5e96, 84 },
    { 0x1bac5e96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ac5ea1, 43 },
    { 0, 0 },
    { 0xc5fc5ea3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc5ea7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x285c5eaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac5ead, 98 },
    { 0, 0 },
    { 0x7cc5eaf, 98 },
    { 0x41ac5eb0, 23 },
    { 0x66ec5eb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fc5eb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804c5ebc, 33 },
    { 0, 0 },
    { 0x5c6c5ebe, 46 },
    { 0x3a4c5ebe, 90 },
    { 0, 0 },
    { 0xf45c5ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x694c5ec4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac5ec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x603c5eca, 100 },
    { 0, 0 },
    { 0x9fbc5ecc, 61 },
    { 0xa33c5ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c5ed2, 90 },
    { 0, 0 },
    { 0xc23c5ed4, 84 },
    { 0xe6ac5ed5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c5ed9, 61 },
    { 0x976c5eda, 61 },
    { 0xaf1c5eda, 95 },
    { 0x73c5edc, 76 },
    { 0x743c5ed9, 95 },
    { 0x148c5ede, 98 },
    { 0xd88c5ede, 33 },
    { 0xcd3c5edf, 61 },
    { 0, 0 },
    { 0xac5c5ee2, 90 },
    { 0x743c5ee2, 44 },
    { 0xe7ec5ee2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc5ee7, 90 },
    { 0, 0 },
    { 0xd56c5ee9, 90 },
    { 0xce7c5eea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc5eee, 98 },
    { 0x962c5eee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dc5ef6, 61 },
    { 0x311c5ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c5efb, 23 },
    { 0xc0fc5efb, 90 },
    { 0x589c5efc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c5f02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5f0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c5f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5c5f12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc5f1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1c5f1f, 43 },
    { 0xfb7c5f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdc5f23, 77 },
    { 0x3f7c5f24, 99 },
    { 0xaebc5f25, 98 },
    { 0, 0 },
    { 0xc80c5f27, 23 },
    { 0xec1c5f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c5f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86c5f32, 66 },
    { 0xaeec5f33, 98 },
    { 0x308c5f34, 78 },
    { 0xc26c5f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c5f38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7c5f3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34c5f48, 55 },
    { 0x29ac5f49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dc5f52, 95 },
    { 0x670c5f53, 95 },
    { 0x34c5f53, 90 },
    { 0, 0 },
    { 0x2abc5f56, 91 },
    { 0x2c9c5f56, 23 },
    { 0, 0 },
    { 0xda5c5f59, 88 },
    { 0xc10c5f5a, 52 },
    { 0xb1ac5f5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccc5f63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4c5f66, 90 },
    { 0x2cbc5f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c5f6a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc5f70, 64 },
    { 0x4bc5f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c5f74, 33 },
    { 0xa74c5f75, 17 },
    { 0xc36c5f76, 85 },
    { 0x533c5f77, 90 },
    { 0x7a2c5f76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc5f7b, 88 },
    { 0x83dc5f7b, 34 },
    { 0x23cc5f7d, 90 },
    { 0xbb1c5f7c, 96 },
    { 0, 0 },
    { 0xd25c5f80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cc5f83, 53 },
    { 0, 0 },
    { 0xe3fc5f85, 46 },
    { 0x89ec5f86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c5f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5c5f90, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9dc5f99, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ac5f9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc5fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x987c5faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c5fb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3c5fb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5fbf, 43 },
    { 0x9dcc5fc0, 99 },
    { 0x148c5fc1, 93 },
    { 0x411c5fc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c5fc9, 90 },
    { 0xe95c5fca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc5fd5, 61 },
    { 0x384c5fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580c5fdb, 100 },
    { 0xa80c5fdc, 90 },
    { 0x24bc5fdc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c5fe1, 77 },
    { 0, 0 },
    { 0x6e8c5fe3, 90 },
    { 0x714c5fe3, 33 },
    { 0xb82c5fe5, 74 },
    { 0xd0dc5fe6, 84 },
    { 0, 0 },
    { 0x896c5fe8, 23 },
    { 0, 0 },
    { 0xfc9c5fea, 98 },
    { 0x924c5feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ec5fef, 76 },
    { 0xa5bc5ff0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbc5ff3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4c5ff8, 33 },
    { 0x5c5ff9, 83 },
    { 0, 0 },
    { 0x3fec5ffb, 23 },
    { 0, 0 },
    { 0x4ebc5ffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adc6007, 66 },
    { 0xbe0c6008, 80 },
    { 0x1e0c6009, 33 },
    { 0x29ac6007, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc600d, 78 },
    { 0x954c600d, 61 },
    { 0xff2c600e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8c6012, 33 },
    { 0, 0 },
    { 0x75fc6014, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec6017, 34 },
    { 0xdd1c6018, 95 },
    { 0x3abc6019, 61 },
    { 0x17dc601a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc601d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378c6022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c6028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ec602c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c6033, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c6042, 61 },
    { 0x9f7c6042, 90 },
    { 0x89dc6044, 36 },
    { 0, 0 },
    { 0x84c6046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2c6049, 94 },
    { 0x247c6049, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c604e, 87 },
    { 0x8d1c604f, 33 },
    { 0x6f8c6050, 90 },
    { 0xe77c604e, 33 },
    { 0xbb4c6052, 98 },
    { 0, 0 },
    { 0xe14c6054, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84c6059, 84 },
    { 0, 0 },
    { 0x737c605b, 84 },
    { 0, 0 },
    { 0x84ec605d, 101 },
    { 0xfd0c605d, 98 },
    { 0, 0 },
    { 0xf30c6060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2c6068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c606e, 84 },
    { 0, 0 },
    { 0x7bc6070, 90 },
    { 0, 0 },
    { 0x27dc6072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x320c6075, 98 },
    { 0, 0 },
    { 0xf22c6077, 90 },
    { 0x1bc6078, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8c607c, 78 },
    { 0x169c607c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8c6080, 98 },
    { 0, 0 },
    { 0x4cec6082, 33 },
    { 0, 0 },
    { 0xb94c6084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cc6088, 61 },
    { 0xcdfc6089, 56 },
    { 0xd04c608a, 56 },
    { 0, 0 },
    { 0xdecc608c, 61 },
    { 0x15ec608d, 33 },
    { 0x2b5c608c, 33 },
    { 0xa84c608d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6c6095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ac609b, 95 },
    { 0x142c609c, 81 },
    { 0x230c609c, 33 },
    { 0x51c609c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2c60a1, 55 },
    { 0xdfcc60a2, 61 },
    { 0, 0 },
    { 0x96c60a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c60ac, 61 },
    { 0x3eac60ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c60b0, 34 },
    { 0x31cc60b0, 67 },
    { 0, 0 },
    { 0xc56c60b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac60b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c60ba, 61 },
    { 0x704c60bb, 52 },
    { 0, 0 },
    { 0xccec60bd, 33 },
    { 0x786c60bd, 91 },
    { 0x24ec60bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414c60c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc60cc, 94 },
    { 0x9cbc60cd, 48 },
    { 0x490c60ce, 98 },
    { 0x9a6c60ce, 76 },
    { 0, 0 },
    { 0xde5c60d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982c60d8, 43 },
    { 0x142c60d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddc60dd, 76 },
    { 0xb5cc60de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02c60e4, 43 },
    { 0xfb9c60e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c60e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c60f2, 100 },
    { 0xd9ac60f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c60f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c60fe, 34 },
    { 0, 0 },
    { 0xadcc6100, 27 },
    { 0xe04c6101, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ac6104, 46 },
    { 0, 0 },
    { 0xe7dc6106, 95 },
    { 0, 0 },
    { 0x891c6108, 95 },
    { 0x992c6109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acc610f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x487c6112, 91 },
    { 0x300c6112, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc6116, 61 },
    { 0, 0 },
    { 0x54c6118, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6120, 61 },
    { 0xca8c6121, 90 },
    { 0xfcfc6120, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24c6125, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc6128, 76 },
    { 0, 0 },
    { 0x1c1c612a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c6135, 33 },
    { 0xb15c6136, 34 },
    { 0xc35c6137, 90 },
    { 0x626c6138, 67 },
    { 0xe69c6137, 88 },
    { 0, 0 },
    { 0xf9dc613b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcc613f, 55 },
    { 0, 0 },
    { 0xf10c6141, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987c6147, 43 },
    { 0xeeec6147, 95 },
    { 0xd5ec6147, 87 },
    { 0, 0 },
    { 0x31ec614b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c614f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bc6152, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ec6155, 66 },
    { 0x154c6156, 98 },
    { 0x7f1c6157, 90 },
    { 0x8f7c6157, 100 },
    { 0x20c6159, 61 },
    { 0x78bc615a, 91 },
    { 0xc4dc615b, 23 },
    { 0xb83c6159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1c6160, 33 },
    { 0, 0 },
    { 0xc9fc6162, 34 },
    { 0xf5c6162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2c6166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cc6169, 90 },
    { 0x5ffc6169, 90 },
    { 0xfbc616b, 41 },
    { 0x2a5c616c, 90 },
    { 0xb3bc616b, 52 },
    { 0xacac616e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52c6171, 43 },
    { 0xd0dc6172, 93 },
    { 0, 0 },
    { 0x279c6174, 90 },
    { 0, 0 },
    { 0xb1ac6176, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fc6179, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15c617c, 90 },
    { 0x84bc617d, 65 },
    { 0x63ac617d, 61 },
    { 0, 0 },
    { 0x595c6180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acc6184, 61 },
    { 0, 0 },
    { 0x488c6186, 95 },
    { 0xd84c6187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99c618a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aec618d, 61 },
    { 0xfdc618e, 90 },
    { 0x2d0c618f, 90 },
    { 0x409c6190, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4c6194, 90 },
    { 0, 0 },
    { 0xd9dc6196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdc619a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c619f, 76 },
    { 0x55bc61a0, 76 },
    { 0, 0 },
    { 0xb33c61a2, 23 },
    { 0x56ec61a2, 61 },
    { 0x782c61a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c61af, 78 },
    { 0, 0 },
    { 0x764c61b1, 91 },
    { 0xcc8c61b1, 88 },
    { 0, 0 },
    { 0x225c61b4, 95 },
    { 0xacdc61b4, 33 },
    { 0, 0 },
    { 0xf86c61b7, 61 },
    { 0x287c61b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6c61be, 55 },
    { 0, 0 },
    { 0x2c5c61c0, 87 },
    { 0, 0 },
    { 0x22c61c2, 84 },
    { 0xfa4c61c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x156c61c6, 23 },
    { 0, 0 },
    { 0xa9cc61c8, 76 },
    { 0xb46c61c8, 95 },
    { 0xe4bc61ca, 76 },
    { 0x2ddc61cb, 85 },
    { 0, 0 },
    { 0xa0dc61cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x896c61d0, 98 },
    { 0x586c61d0, 67 },
    { 0x1d6c61d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871c61d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168c61df, 46 },
    { 0, 0 },
    { 0xf44c61e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc61e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03c61e8, 90 },
    { 0, 0 },
    { 0xcc5c61ea, 61 },
    { 0, 0 },
    { 0xed8c61ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4c61ef, 90 },
    { 0x8ebc61f0, 46 },
    { 0, 0 },
    { 0x44cc61f2, 90 },
    { 0, 0 },
    { 0xb4bc61f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ac61f9, 97 },
    { 0x980c61fa, 34 },
    { 0x799c61fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c6201, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c6204, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x562c6207, 61 },
    { 0x1aac6208, 99 },
    { 0xcdbc6209, 61 },
    { 0xcb0c6209, 79 },
    { 0x6f0c6208, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455c6211, 65 },
    { 0x64dc6211, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4c6217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bc621c, 90 },
    { 0x75ec621d, 90 },
    { 0xb8cc621d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc6225, 98 },
    { 0x15ac6226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c622b, 43 },
    { 0xfb2c622b, 98 },
    { 0xf6bc622c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbac6230, 95 },
    { 0x35cc6230, 99 },
    { 0xc24c6232, 66 },
    { 0x4c8c6232, 99 },
    { 0xf84c6234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c6237, 61 },
    { 0x777c6238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3c623e, 48 },
    { 0x83dc623f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac6242, 61 },
    { 0x1b0c6243, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8c6248, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53c624c, 76 },
    { 0x150c624d, 75 },
    { 0x1b7c624e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9c6252, 100 },
    { 0, 0 },
    { 0x3a1c6254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c625a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc625d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac6262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8c6267, 61 },
    { 0x956c6268, 95 },
    { 0x46dc6269, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c626d, 33 },
    { 0, 0 },
    { 0xf5cc626f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dc6273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896c6278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac6282, 61 },
    { 0, 0 },
    { 0x8edc6284, 33 },
    { 0, 0 },
    { 0x629c6286, 55 },
    { 0x118c6286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ac628a, 52 },
    { 0, 0 },
    { 0x12ac628c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830c6294, 46 },
    { 0x82ec6294, 46 },
    { 0xc72c6296, 95 },
    { 0x7d2c6294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c629f, 91 },
    { 0xef6c62a0, 98 },
    { 0xfabc62a1, 61 },
    { 0x6a1c62a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c62a5, 98 },
    { 0x1ec62a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819c62ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c62ae, 34 },
    { 0, 0 },
    { 0x924c62b0, 93 },
    { 0x952c62b0, 33 },
    { 0x287c62b1, 34 },
    { 0, 0 },
    { 0x98fc62b4, 23 },
    { 0xd1fc62b4, 33 },
    { 0xf51c62b5, 23 },
    { 0xf91c62b6, 76 },
    { 0x956c62b7, 95 },
    { 0xc86c62b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c62bd, 61 },
    { 0x7cc62bd, 23 },
    { 0, 0 },
    { 0x6efc62c0, 33 },
    { 0, 0 },
    { 0xf74c62c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cc62c5, 93 },
    { 0, 0 },
    { 0xeabc62c7, 95 },
    { 0x22ac62c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335c62ce, 95 },
    { 0x55cc62ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0c62d8, 66 },
    { 0x13cc62d9, 87 },
    { 0xd16c62da, 61 },
    { 0x99dc62db, 61 },
    { 0xa4ec62db, 95 },
    { 0x71bc62dd, 61 },
    { 0, 0 },
    { 0x5c6c62df, 95 },
    { 0, 0 },
    { 0xe83c62e1, 94 },
    { 0, 0 },
    { 0x1f7c62e3, 90 },
    { 0, 0 },
    { 0xdd5c62e5, 93 },
    { 0x564c62e5, 23 },
    { 0x318c62e7, 23 },
    { 0, 0 },
    { 0x29c62e9, 90 },
    { 0x211c62ea, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc62ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x366c62f2, 23 },
    { 0x905c62f3, 98 },
    { 0x5d4c62f4, 76 },
    { 0xe81c62f5, 34 },
    { 0xa02c62f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fac62f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9c62fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c6302, 99 },
    { 0, 0 },
    { 0xa61c6304, 95 },
    { 0, 0 },
    { 0x481c6306, 91 },
    { 0xe36c6307, 91 },
    { 0xf77c6307, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c630b, 100 },
    { 0x5bac630b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dc6313, 34 },
    { 0x6c8c6314, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5c6319, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c631e, 98 },
    { 0x297c631f, 77 },
    { 0xcc4c6320, 33 },
    { 0xaafc6320, 90 },
    { 0, 0 },
    { 0x1a0c6323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9c632a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc632d, 23 },
    { 0x4b0c632d, 55 },
    { 0xb94c632d, 34 },
    { 0xfcac6330, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac633f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac6342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c634d, 95 },
    { 0, 0 },
    { 0xcfac634f, 99 },
    { 0xd43c6350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc6353, 85 },
    { 0x18fc6353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc6357, 66 },
    { 0, 0 },
    { 0x71c6359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7c635d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9c6361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bc6367, 78 },
    { 0xa07c6368, 43 },
    { 0x592c6367, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c6370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c6375, 34 },
    { 0, 0 },
    { 0xdd2c6377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc637a, 98 },
    { 0x371c637b, 95 },
    { 0x22ec637b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dc6381, 90 },
    { 0xd32c6382, 88 },
    { 0xdb3c6383, 61 },
    { 0x14bc6382, 46 },
    { 0x2f4c6384, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6c6388, 93 },
    { 0xc54c6389, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc638c, 88 },
    { 0, 0 },
    { 0xe98c638e, 33 },
    { 0x673c638f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dc6395, 98 },
    { 0, 0 },
    { 0xd3dc6397, 46 },
    { 0xc0fc6398, 95 },
    { 0xc22c6398, 34 },
    { 0x26fc6397, 95 },
    { 0xf00c6397, 43 },
    { 0xbe4c639c, 34 },
    { 0x3ccc639d, 93 },
    { 0x5e2c639c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4c63a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fc63a8, 90 },
    { 0xa9fc63a9, 86 },
    { 0, 0 },
    { 0xf28c63ab, 95 },
    { 0x93bc63ac, 98 },
    { 0xb1bc63ad, 90 },
    { 0x1b3c63ae, 76 },
    { 0, 0 },
    { 0x636c63b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c63b5, 67 },
    { 0x532c63b6, 55 },
    { 0xa4ec63b7, 93 },
    { 0x3f1c63b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc63bb, 69 },
    { 0xbf3c63bc, 90 },
    { 0, 0 },
    { 0x324c63be, 90 },
    { 0xa00c63be, 46 },
    { 0, 0 },
    { 0x7aec63c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06c63c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc63cc, 61 },
    { 0x2f3c63cd, 17 },
    { 0x1fbc63ce, 65 },
    { 0xd90c63cf, 33 },
    { 0x2abc63d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc63d3, 33 },
    { 0x120c63d4, 61 },
    { 0x190c63d4, 38 },
    { 0, 0 },
    { 0xcfec63d7, 93 },
    { 0, 0 },
    { 0x8b7c63d9, 91 },
    { 0, 0 },
    { 0xf0cc63db, 43 },
    { 0xc97c63dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc63df, 88 },
    { 0x7c4c63e0, 98 },
    { 0, 0 },
    { 0x939c63e2, 76 },
    { 0x33c63e3, 33 },
    { 0x86c63e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c63e7, 83 },
    { 0xfadc63e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c63eb, 52 },
    { 0x1a3c63ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93c63f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c63f6, 55 },
    { 0xb6ac63f7, 95 },
    { 0x548c63f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c6402, 66 },
    { 0, 0 },
    { 0xbe2c6404, 76 },
    { 0xd27c6404, 87 },
    { 0x11fc6405, 81 },
    { 0xafac6407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c640b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0c640f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8c6416, 33 },
    { 0xd47c6416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c641c, 91 },
    { 0, 0 },
    { 0x12c641e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5c6422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c6425, 95 },
    { 0x360c6425, 55 },
    { 0, 0 },
    { 0xc8ac6428, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c642e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c6433, 78 },
    { 0xc31c6433, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bac6437, 61 },
    { 0xff7c6438, 90 },
    { 0x69c6439, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c643d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c6442, 34 },
    { 0, 0 },
    { 0xddcc6444, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefec644a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c644e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147c6454, 76 },
    { 0x286c6455, 91 },
    { 0x666c6455, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x350c6459, 33 },
    { 0x59cc6459, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c645e, 22 },
    { 0x2fec645f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc6462, 33 },
    { 0xfdbc6462, 76 },
    { 0xa35c6464, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc646a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19c646d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03c6471, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ac6474, 65 },
    { 0xb62c6474, 76 },
    { 0x9e9c6475, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc647c, 55 },
    { 0, 0 },
    { 0x359c647e, 78 },
    { 0, 0 },
    { 0x5f2c6480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc6484, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c6487, 61 },
    { 0x74ac6487, 46 },
    { 0xf87c6488, 90 },
    { 0xc90c6487, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec648d, 65 },
    { 0, 0 },
    { 0x5fbc648f, 66 },
    { 0x431c6490, 43 },
    { 0x920c648f, 36 },
    { 0xdcdc6492, 65 },
    { 0x681c6493, 90 },
    { 0xba0c6492, 98 },
    { 0, 0 },
    { 0xe45c6496, 34 },
    { 0x1fbc6497, 55 },
    { 0xc5ac6498, 34 },
    { 0x3e2c6498, 87 },
    { 0xe67c649a, 90 },
    { 0, 0 },
    { 0x2f9c649c, 33 },
    { 0x58ec649d, 33 },
    { 0x73bc649e, 77 },
    { 0x1bc649f, 43 },
    { 0x848c64a0, 46 },
    { 0xc1bc649c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c64a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c64ad, 84 },
    { 0x54ec64ae, 61 },
    { 0, 0 },
    { 0x75fc64b0, 34 },
    { 0, 0 },
    { 0xc2dc64b2, 61 },
    { 0x50c64b3, 85 },
    { 0xfbcc64b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11c64b7, 61 },
    { 0, 0 },
    { 0x89ac64b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc64bc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x847c64bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aec64c2, 90 },
    { 0xa5ec64c2, 33 },
    { 0xa9fc64c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8c64c7, 90 },
    { 0x3dec64c7, 61 },
    { 0x984c64c7, 61 },
    { 0xf19c64c8, 23 },
    { 0x169c64cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64c64d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c64d6, 90 },
    { 0x9a5c64d7, 90 },
    { 0x846c64d7, 61 },
    { 0, 0 },
    { 0x7f1c64da, 34 },
    { 0, 0 },
    { 0x653c64dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec64e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6c64e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c64eb, 61 },
    { 0, 0 },
    { 0xedac64ed, 98 },
    { 0x23bc64ed, 33 },
    { 0, 0 },
    { 0xdb7c64f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41c64f8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0c6500, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c650a, 98 },
    { 0x702c650a, 34 },
    { 0xadac650a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fc6511, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ac6518, 95 },
    { 0x20ec6518, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc651d, 90 },
    { 0xdbdc651d, 33 },
    { 0x76cc651f, 98 },
    { 0x934c651d, 90 },
    { 0xe1ac651e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ec6526, 46 },
    { 0xfcdc6526, 96 },
    { 0xf56c6526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec652f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cc6532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aec6536, 43 },
    { 0x3ec6537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac653a, 50 },
    { 0x16ac653a, 61 },
    { 0xb60c653b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c653f, 36 },
    { 0x719c6540, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fc6546, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c654a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c654d, 84 },
    { 0, 0 },
    { 0xac9c654f, 43 },
    { 0, 0 },
    { 0xec5c6551, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1c6556, 36 },
    { 0x63fc6557, 55 },
    { 0x132c6558, 33 },
    { 0x4b6c6559, 95 },
    { 0x608c655a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec6564, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c6568, 43 },
    { 0xed0c6569, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfac656f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c6573, 90 },
    { 0x352c6573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc6577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c657a, 91 },
    { 0, 0 },
    { 0xa81c657c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c6583, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec6587, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fc658b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c658e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04c6594, 33 },
    { 0, 0 },
    { 0x2cbc6596, 78 },
    { 0xaddc6597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c659b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbc659f, 90 },
    { 0x400c659f, 61 },
    { 0xe0c65a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c65a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc65a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c65af, 76 },
    { 0xfbfc65b0, 46 },
    { 0x53dc65b0, 95 },
    { 0, 0 },
    { 0x8bfc65b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c65b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c65bd, 33 },
    { 0x259c65be, 55 },
    { 0, 0 },
    { 0x307c65c0, 93 },
    { 0xe01c65c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c65ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92c65cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715c65d5, 90 },
    { 0xd3ec65d6, 80 },
    { 0, 0 },
    { 0xefec65d8, 95 },
    { 0x5d9c65d9, 55 },
    { 0x26ec65da, 52 },
    { 0x187c65db, 93 },
    { 0x5afc65dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x743c65df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c65e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c65e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc65e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50c65f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70c65f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c65fe, 61 },
    { 0x4d7c65ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48c6602, 66 },
    { 0x19ac6602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x746c6606, 98 },
    { 0x52bc6606, 90 },
    { 0x121c6608, 90 },
    { 0x442c6609, 23 },
    { 0xa01c6608, 55 },
    { 0x2d6c6609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc660f, 90 },
    { 0xb9dc6610, 33 },
    { 0, 0 },
    { 0x442c6612, 23 },
    { 0x99bc6613, 33 },
    { 0x5e1c6613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bc6619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c6620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc662e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc6634, 90 },
    { 0xd51c6635, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5c6638, 43 },
    { 0, 0 },
    { 0x183c663a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cc663d, 90 },
    { 0xafac663e, 90 },
    { 0xa9bc663e, 67 },
    { 0x5b2c663d, 17 },
    { 0x143c6641, 76 },
    { 0xdf0c6641, 61 },
    { 0x84fc6643, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8c6648, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c664c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dc6650, 61 },
    { 0, 0 },
    { 0x33fc6652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbc6657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c665b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc665e, 100 },
    { 0x9ffc665f, 99 },
    { 0xd4fc6660, 85 },
    { 0x315c6661, 90 },
    { 0x305c6662, 61 },
    { 0, 0 },
    { 0xc11c6664, 90 },
    { 0, 0 },
    { 0x99fc6666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8c666a, 91 },
    { 0, 0 },
    { 0x7fbc666c, 33 },
    { 0, 0 },
    { 0x7b4c666e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c6673, 61 },
    { 0x4eec6674, 34 },
    { 0x89cc6674, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c667b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cc6682, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc668b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c668e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac6693, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c6698, 61 },
    { 0xc87c6699, 61 },
    { 0, 0 },
    { 0xcbac669b, 34 },
    { 0, 0 },
    { 0x256c669d, 23 },
    { 0xaf3c669e, 55 },
    { 0xd24c669d, 84 },
    { 0, 0 },
    { 0x4e4c66a1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c66a4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c66a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c66ae, 95 },
    { 0xf1bc66af, 61 },
    { 0x83bc66af, 61 },
    { 0xa4c66b1, 34 },
    { 0x5f8c66ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c66b5, 90 },
    { 0x326c66b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32c66bb, 95 },
    { 0xd0fc66bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934c66c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6c66c6, 23 },
    { 0x5f2c66c6, 61 },
    { 0, 0 },
    { 0xa5cc66c9, 76 },
    { 0xc09c66c9, 98 },
    { 0, 0 },
    { 0xf4bc66cc, 95 },
    { 0, 0 },
    { 0xae7c66ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bac66d5, 90 },
    { 0xbd5c66d6, 98 },
    { 0x22bc66d7, 66 },
    { 0x5c9c66d8, 65 },
    { 0xd01c66d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc66e0, 43 },
    { 0x2a0c66e1, 61 },
    { 0xc44c66e1, 100 },
    { 0x7bc66e3, 91 },
    { 0xa4fc66e4, 90 },
    { 0, 0 },
    { 0xc45c66e6, 66 },
    { 0x7dec66e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc66ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05c66ee, 66 },
    { 0xafac66ef, 61 },
    { 0x902c66f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7c66f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc66fb, 46 },
    { 0, 0 },
    { 0xf17c66fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac6704, 33 },
    { 0, 0 },
    { 0xf4ec6706, 95 },
    { 0xd46c6707, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44c670b, 78 },
    { 0x545c670c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fc6710, 33 },
    { 0x43cc6711, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac6715, 85 },
    { 0x80dc6715, 61 },
    { 0, 0 },
    { 0x810c6718, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc671e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c6725, 95 },
    { 0, 0 },
    { 0x90cc6727, 90 },
    { 0x8ccc6727, 55 },
    { 0xe8bc6729, 17 },
    { 0x438c672a, 33 },
    { 0, 0 },
    { 0x647c672c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc6736, 61 },
    { 0x6eac6736, 88 },
    { 0x2e6c6738, 95 },
    { 0xdb3c6739, 34 },
    { 0xfc5c673a, 40 },
    { 0x34dc673b, 61 },
    { 0x952c6738, 66 },
    { 0x3b0c673c, 76 },
    { 0xe2c673e, 95 },
    { 0x5bfc673f, 90 },
    { 0xd53c673f, 90 },
    { 0x790c6741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833c6745, 33 },
    { 0, 0 },
    { 0x87ac6747, 88 },
    { 0, 0 },
    { 0xdf9c6749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6c674e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c6755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c6758, 38 },
    { 0x919c6759, 55 },
    { 0x823c6758, 95 },
    { 0x1a7c6758, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc675f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c6766, 95 },
    { 0xcd3c6767, 90 },
    { 0, 0 },
    { 0x132c6769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac677c, 90 },
    { 0, 0 },
    { 0x94dc677e, 34 },
    { 0, 0 },
    { 0xdd2c6780, 72 },
    { 0, 0 },
    { 0xd9fc6782, 84 },
    { 0x177c6783, 93 },
    { 0x344c6784, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7c6788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2c678c, 33 },
    { 0xe8dc678c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fc6790, 76 },
    { 0xa1bc6790, 46 },
    { 0x645c6792, 61 },
    { 0, 0 },
    { 0x859c6794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6797, 76 },
    { 0, 0 },
    { 0x9a7c6799, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bc679d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac67a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12c67a4, 61 },
    { 0, 0 },
    { 0xf6ec67a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c67ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac67b5, 95 },
    { 0xa99c67b6, 61 },
    { 0xb72c67b7, 93 },
    { 0xe9bc67b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2c67bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bec67c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357c67c4, 64 },
    { 0x725c67c4, 90 },
    { 0xd0c67c4, 34 },
    { 0x2c9c67c7, 95 },
    { 0, 0 },
    { 0xc04c67c9, 93 },
    { 0x993c67ca, 43 },
    { 0xed3c67cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c67ce, 90 },
    { 0, 0 },
    { 0x268c67d0, 81 },
    { 0x2aac67d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c67d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc67de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c67e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c67e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31c67eb, 61 },
    { 0x71c67ec, 95 },
    { 0xc85c67ed, 55 },
    { 0x5adc67ee, 23 },
    { 0x28ac67ef, 34 },
    { 0, 0 },
    { 0xc8fc67f1, 95 },
    { 0, 0 },
    { 0x4b2c67f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bc67f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edc67fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5c6801, 33 },
    { 0x453c6801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c6806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91c6809, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cc6812, 48 },
    { 0x7ec6813, 97 },
    { 0x482c6813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec6818, 43 },
    { 0xe3cc6819, 76 },
    { 0xbf1c681a, 61 },
    { 0x503c681b, 66 },
    { 0, 0 },
    { 0x350c681d, 46 },
    { 0, 0 },
    { 0x808c681f, 85 },
    { 0x661c6820, 55 },
    { 0x6c2c681f, 76 },
    { 0x1bac6822, 61 },
    { 0, 0 },
    { 0xd9dc6824, 43 },
    { 0, 0 },
    { 0xa2cc6826, 65 },
    { 0, 0 },
    { 0xef7c6828, 33 },
    { 0xc4ec6829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71c6831, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265c6837, 98 },
    { 0xeacc6838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cc6843, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527c684e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ac6851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c685c, 65 },
    { 0, 0 },
    { 0x4f5c685e, 95 },
    { 0x49ac685f, 90 },
    { 0xde6c6860, 43 },
    { 0x497c6861, 34 },
    { 0xec2c6861, 90 },
    { 0xe2ac6861, 23 },
    { 0x9ec6864, 23 },
    { 0xa8cc6861, 82 },
    { 0x17dc6865, 33 },
    { 0xda8c6867, 66 },
    { 0x4dc6868, 61 },
    { 0xf0fc6869, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaac686d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x362c6870, 85 },
    { 0x5d2c6871, 61 },
    { 0, 0 },
    { 0x3e1c6873, 100 },
    { 0x4fdc6874, 33 },
    { 0x581c6875, 23 },
    { 0x3e3c6873, 98 },
    { 0x62ac6877, 100 },
    { 0x4c6c6876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c6880, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x977c6883, 55 },
    { 0xb27c6883, 61 },
    { 0, 0 },
    { 0x35fc6886, 53 },
    { 0x1f0c6886, 95 },
    { 0x382c6888, 61 },
    { 0x458c6889, 91 },
    { 0xbd6c688a, 95 },
    { 0, 0 },
    { 0x3a9c688c, 76 },
    { 0, 0 },
    { 0xbc688e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8c6891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c6896, 23 },
    { 0, 0 },
    { 0x56ac6898, 55 },
    { 0, 0 },
    { 0xe06c689a, 95 },
    { 0x35ec689b, 61 },
    { 0xca7c689b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c68a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45c68b0, 95 },
    { 0, 0 },
    { 0x88dc68b2, 61 },
    { 0xbd0c68b2, 76 },
    { 0xa4ac68b4, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c68b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7c68bd, 61 },
    { 0x16fc68be, 23 },
    { 0, 0 },
    { 0xc75c68c0, 76 },
    { 0x94c68c1, 46 },
    { 0, 0 },
    { 0xdd3c68c3, 61 },
    { 0, 0 },
    { 0xbf4c68c5, 78 },
    { 0xc9dc68c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c68cc, 76 },
    { 0xf51c68cd, 90 },
    { 0xf51c68cc, 95 },
    { 0x476c68cd, 37 },
    { 0, 0 },
    { 0xa9ac68d1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c68d4, 90 },
    { 0, 0 },
    { 0xcabc68d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8c68d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbc68dc, 90 },
    { 0, 0 },
    { 0xfa5c68de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4c68e2, 90 },
    { 0, 0 },
    { 0x5c2c68e4, 55 },
    { 0xb02c68e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9c68e9, 55 },
    { 0x53bc68ea, 43 },
    { 0, 0 },
    { 0xfa5c68ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc68f2, 68 },
    { 0xcd2c68f2, 90 },
    { 0, 0 },
    { 0x270c68f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825c68f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc68fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6908, 20 },
    { 0, 0 },
    { 0x45bc690a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c6912, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6c691b, 55 },
    { 0x4cac691c, 98 },
    { 0xd4c691d, 99 },
    { 0x8edc691c, 61 },
    { 0x6a1c691d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9c6924, 95 },
    { 0xaedc6925, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c692b, 90 },
    { 0x465c692c, 66 },
    { 0x3edc692d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17c6932, 55 },
    { 0x550c6933, 33 },
    { 0xfe6c6933, 55 },
    { 0, 0 },
    { 0xf4ac6936, 61 },
    { 0xa37c6937, 69 },
    { 0xf59c6938, 56 },
    { 0, 0 },
    { 0x921c693a, 100 },
    { 0, 0 },
    { 0x4cbc693c, 88 },
    { 0, 0 },
    { 0x1cac693e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c6946, 43 },
    { 0xef5c6946, 98 },
    { 0x307c6948, 93 },
    { 0, 0 },
    { 0xb0ac694a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2c6952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac695d, 100 },
    { 0x79c695e, 88 },
    { 0, 0 },
    { 0x86fc6960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3c6966, 88 },
    { 0, 0 },
    { 0x2cec6968, 33 },
    { 0, 0 },
    { 0x4f5c696a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cc6971, 87 },
    { 0, 0 },
    { 0xbe3c6973, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c6977, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbc6984, 23 },
    { 0x7ccc6984, 95 },
    { 0x718c6986, 90 },
    { 0xc00c6987, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edc698b, 52 },
    { 0xb2cc698c, 91 },
    { 0x574c698b, 95 },
    { 0, 0 },
    { 0x236c698f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ec6992, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168c6999, 55 },
    { 0, 0 },
    { 0x552c699b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc699e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec69a3, 43 },
    { 0, 0 },
    { 0xe4ec69a5, 38 },
    { 0, 0 },
    { 0xf22c69a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec69aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894c69b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c69b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c69bd, 33 },
    { 0xc6bc69be, 55 },
    { 0xfd3c69bf, 61 },
    { 0x90dc69c0, 93 },
    { 0x980c69c1, 90 },
    { 0x6d4c69c1, 98 },
    { 0, 0 },
    { 0xbec69c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c69c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c69cc, 55 },
    { 0, 0 },
    { 0xb17c69ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5c69d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c69d9, 66 },
    { 0xf99c69da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9c69de, 33 },
    { 0xd00c69de, 61 },
    { 0x8f5c69e0, 68 },
    { 0xce4c69e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adc69e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c69ef, 55 },
    { 0x9b2c69f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc69f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c69f9, 23 },
    { 0xb48c69fa, 76 },
    { 0x54ac69fa, 23 },
    { 0x98bc69fc, 87 },
    { 0, 0 },
    { 0xefbc69fe, 76 },
    { 0, 0 },
    { 0xa9c6a00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x297c6a03, 61 },
    { 0x60dc6a04, 61 },
    { 0, 0 },
    { 0x9aac6a06, 100 },
    { 0x420c6a06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac6a0a, 101 },
    { 0x67dc6a0a, 90 },
    { 0xccdc6a0c, 95 },
    { 0xf7ec6a0d, 61 },
    { 0x2eac6a0d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867c6a13, 78 },
    { 0x56c6a14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c6a19, 95 },
    { 0x391c6a1a, 90 },
    { 0xa99c6a19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990c6a1f, 46 },
    { 0x3d6c6a20, 65 },
    { 0xc04c6a20, 76 },
    { 0xb65c6a22, 19 },
    { 0x382c6a23, 98 },
    { 0x401c6a24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c6a2e, 61 },
    { 0xd3dc6a2f, 76 },
    { 0x5a8c6a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263c6a40, 98 },
    { 0xf80c6a40, 34 },
    { 0xa3ac6a42, 43 },
    { 0x50fc6a43, 23 },
    { 0xf46c6a44, 33 },
    { 0xfdcc6a44, 66 },
    { 0xd8cc6a44, 88 },
    { 0x5b4c6a44, 91 },
    { 0x943c6a44, 65 },
    { 0x445c6a49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc6a4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5c6a50, 42 },
    { 0xc73c6a51, 17 },
    { 0xbf4c6a51, 23 },
    { 0x714c6a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cc6a56, 98 },
    { 0x159c6a57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7c6a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5c6a61, 33 },
    { 0xb72c6a62, 95 },
    { 0x906c6a61, 100 },
    { 0, 0 },
    { 0xc73c6a65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c6a6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c6a70, 98 },
    { 0x85bc6a71, 75 },
    { 0xc23c6a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42c6a7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbc6a7d, 95 },
    { 0, 0 },
    { 0x54bc6a7f, 90 },
    { 0, 0 },
    { 0x90c6a81, 98 },
    { 0, 0 },
    { 0xd5dc6a83, 61 },
    { 0x425c6a83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cec6a87, 99 },
    { 0x832c6a88, 85 },
    { 0x703c6a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74c6a8d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0c6a92, 95 },
    { 0, 0 },
    { 0xfd1c6a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac6a97, 95 },
    { 0, 0 },
    { 0x60ec6a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353c6aa3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c6aa6, 93 },
    { 0xe3ac6aa7, 87 },
    { 0xa0fc6aa7, 95 },
    { 0xce0c6aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fc6aaf, 76 },
    { 0xd1bc6ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c6ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c6ab6, 91 },
    { 0x7f6c6ab7, 23 },
    { 0xfa2c6ab8, 61 },
    { 0x874c6ab7, 61 },
    { 0xad2c6aba, 33 },
    { 0xe80c6aba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502c6abf, 23 },
    { 0, 0 },
    { 0xb7cc6ac1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c6ac4, 84 },
    { 0x1f8c6ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ec6ace, 95 },
    { 0, 0 },
    { 0xfdfc6ad0, 46 },
    { 0xc13c6ad1, 90 },
    { 0xeac6ad1, 33 },
    { 0, 0 },
    { 0x8e2c6ad4, 87 },
    { 0, 0 },
    { 0x543c6ad6, 90 },
    { 0xed4c6ad7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efc6add, 100 },
    { 0, 0 },
    { 0x388c6adf, 33 },
    { 0x18c6ae0, 90 },
    { 0x25dc6ae1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffc6ae4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ec6ae9, 90 },
    { 0x8e8c6aea, 98 },
    { 0, 0 },
    { 0xb9dc6aec, 90 },
    { 0, 0 },
    { 0x44c6aee, 76 },
    { 0x4bcc6aef, 76 },
    { 0xad0c6af0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c6af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec6af6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1c6afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c6b00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ac6b09, 34 },
    { 0, 0 },
    { 0x17fc6b0b, 68 },
    { 0x9f7c6b0c, 55 },
    { 0, 0 },
    { 0x127c6b0e, 61 },
    { 0x962c6b0f, 23 },
    { 0x209c6b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c6b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc6b16, 98 },
    { 0x2b2c6b17, 61 },
    { 0xd71c6b18, 34 },
    { 0x99bc6b17, 68 },
    { 0x63ec6b16, 65 },
    { 0x7e9c6b1b, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64c6b1e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc6b21, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fc6b28, 17 },
    { 0x2b1c6b29, 95 },
    { 0x67fc6b28, 78 },
    { 0x16ec6b2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc6b2e, 85 },
    { 0x3d7c6b2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521c6b33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffc6b3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc6b40, 95 },
    { 0xa41c6b41, 90 },
    { 0x85c6b41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89c6b49, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abc6b51, 98 },
    { 0, 0 },
    { 0x2adc6b53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1c6b59, 93 },
    { 0, 0 },
    { 0x266c6b5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc6b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddc6b6c, 77 },
    { 0x9b3c6b6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bc6b72, 23 },
    { 0x3f9c6b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c6b77, 76 },
    { 0x5bec6b77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c6b7d, 23 },
    { 0x3a0c6b7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71c6b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c6b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c6b8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8c6b8f, 85 },
    { 0x97ec6b8f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc6b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x112c6b98, 23 },
    { 0xce2c6b99, 65 },
    { 0x89bc6b99, 91 },
    { 0x7d0c6b9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc6b9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc6ba5, 70 },
    { 0, 0 },
    { 0xa09c6ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c6bb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64c6bb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c6bba, 66 },
    { 0x733c6bba, 33 },
    { 0, 0 },
    { 0x484c6bbd, 33 },
    { 0xeffc6bbe, 76 },
    { 0, 0 },
    { 0x9f2c6bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778c6bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eac6bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c6bd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc6bdc, 91 },
    { 0xc55c6bdc, 55 },
    { 0xd45c6bdc, 75 },
    { 0, 0 },
    { 0x3a0c6be0, 52 },
    { 0, 0 },
    { 0x820c6be2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6c6be6, 33 },
    { 0xf3cc6be7, 77 },
    { 0xceec6be6, 52 },
    { 0x3e3c6be9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc6bed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87c6bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbec6bf5, 23 },
    { 0xb7cc6bf6, 53 },
    { 0x439c6bf6, 95 },
    { 0, 0 },
    { 0x1cbc6bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525c6c00, 61 },
    { 0x7ac6c01, 61 },
    { 0x993c6c02, 61 },
    { 0xce9c6c03, 23 },
    { 0xdeec6c03, 33 },
    { 0x96dc6c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225c6c0a, 23 },
    { 0, 0 },
    { 0x57fc6c0c, 33 },
    { 0x58ec6c0d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4c6c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c6c13, 61 },
    { 0xa2bc6c14, 99 },
    { 0, 0 },
    { 0xbd9c6c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ec6c19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfc6c21, 23 },
    { 0, 0 },
    { 0x3f3c6c23, 78 },
    { 0, 0 },
    { 0xa55c6c25, 61 },
    { 0x693c6c26, 17 },
    { 0, 0 },
    { 0x89fc6c28, 43 },
    { 0x987c6c29, 52 },
    { 0x394c6c2a, 76 },
    { 0xb7c6c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc6c2f, 53 },
    { 0xcc6c30, 91 },
    { 0xbc2c6c31, 55 },
    { 0x3d6c6c30, 78 },
    { 0x26ec6c33, 90 },
    { 0xdadc6c33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9c6c37, 76 },
    { 0xf86c6c37, 61 },
    { 0xef5c6c39, 76 },
    { 0x567c6c3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198c6c40, 76 },
    { 0x52c6c41, 58 },
    { 0x8e5c6c40, 90 },
    { 0, 0 },
    { 0x2e9c6c44, 95 },
    { 0x85cc6c45, 90 },
    { 0, 0 },
    { 0xb26c6c47, 90 },
    { 0xccbc6c48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc6c51, 75 },
    { 0xb9dc6c52, 98 },
    { 0xc5c6c53, 40 },
    { 0x817c6c54, 61 },
    { 0x396c6c54, 23 },
    { 0x95fc6c56, 90 },
    { 0xbb1c6c57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c6c5a, 81 },
    { 0xbecc6c5a, 95 },
    { 0x989c6c5c, 90 },
    { 0xfeac6c5d, 99 },
    { 0x162c6c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8c6c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc6c6e, 95 },
    { 0xc91c6c6f, 33 },
    { 0x2c3c6c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dc6c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc6c7b, 55 },
    { 0x7a0c6c7b, 33 },
    { 0, 0 },
    { 0x106c6c7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c6c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61c6c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3c6c8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c6c90, 55 },
    { 0xa50c6c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dc6c98, 33 },
    { 0x1d1c6c99, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c6c9c, 83 },
    { 0xa91c6c9c, 55 },
    { 0x86fc6c9c, 43 },
    { 0x134c6c9f, 34 },
    { 0x4c3c6ca0, 55 },
    { 0xc2c6ca1, 90 },
    { 0x66bc6c9f, 33 },
    { 0, 0 },
    { 0x623c6ca4, 95 },
    { 0x1ddc6ca5, 94 },
    { 0xdcdc6ca6, 98 },
    { 0xeebc6ca7, 95 },
    { 0x68bc6ca8, 76 },
    { 0x72c6ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec6cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8c6caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec6cb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc6cb8, 75 },
    { 0, 0 },
    { 0x8c2c6cba, 90 },
    { 0xd5ac6cba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46c6cbf, 34 },
    { 0xe33c6cc0, 97 },
    { 0x115c6cc1, 66 },
    { 0x103c6cc2, 33 },
    { 0, 0 },
    { 0xcfdc6cc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1c6ccd, 87 },
    { 0x34fc6cce, 93 },
    { 0, 0 },
    { 0x1dc6cd0, 33 },
    { 0, 0 },
    { 0x273c6cd2, 98 },
    { 0xb1cc6cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeec6cd8, 98 },
    { 0xc1ec6cd8, 61 },
    { 0xba8c6cda, 33 },
    { 0xc32c6cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c6cdf, 92 },
    { 0x2d7c6cdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afc6ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ec6ce8, 78 },
    { 0xc23c6ce9, 55 },
    { 0x2dbc6cea, 95 },
    { 0x47cc6cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c6cf4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacc6cfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fc6cfe, 80 },
    { 0x83ec6cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c6d03, 87 },
    { 0xc4bc6d04, 98 },
    { 0x2c3c6d05, 93 },
    { 0, 0 },
    { 0xe05c6d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c6d0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc6d0e, 91 },
    { 0, 0 },
    { 0x8f3c6d10, 88 },
    { 0x94ac6d10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc6d15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec6d1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393c6d1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ac6d24, 61 },
    { 0, 0 },
    { 0x5bec6d26, 77 },
    { 0xf21c6d27, 84 },
    { 0xb0fc6d26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1c6d2b, 23 },
    { 0x1b6c6d2c, 76 },
    { 0xde9c6d2b, 33 },
    { 0x9dec6d2d, 77 },
    { 0xfa2c6d2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cc6d35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885c6d39, 91 },
    { 0xfb5c6d3a, 96 },
    { 0x30cc6d3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c6d4b, 98 },
    { 0x38ec6d4c, 98 },
    { 0, 0 },
    { 0x362c6d4e, 23 },
    { 0xa22c6d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27c6d54, 95 },
    { 0xe42c6d55, 95 },
    { 0, 0 },
    { 0xfb3c6d57, 33 },
    { 0, 0 },
    { 0x3c6d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1c6d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872c6d62, 98 },
    { 0xd5ac6d63, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406c6d6a, 52 },
    { 0x1c8c6d6a, 93 },
    { 0xa9dc6d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885c6d70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc6d73, 65 },
    { 0, 0 },
    { 0xd7dc6d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521c6d79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec6d85, 98 },
    { 0xec8c6d86, 47 },
    { 0, 0 },
    { 0x535c6d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fc6d8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc6d8f, 40 },
    { 0, 0 },
    { 0x78bc6d91, 61 },
    { 0x6edc6d92, 90 },
    { 0, 0 },
    { 0xd07c6d94, 75 },
    { 0x337c6d94, 43 },
    { 0, 0 },
    { 0x6cac6d97, 46 },
    { 0x3b1c6d97, 100 },
    { 0x5e5c6d99, 33 },
    { 0, 0 },
    { 0xe90c6d9b, 85 },
    { 0x78ec6d9c, 98 },
    { 0x7cbc6d9c, 55 },
    { 0x6bcc6d9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148c6da8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc6dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95c6db0, 90 },
    { 0x407c6db1, 55 },
    { 0x8cc6db2, 43 },
    { 0xbfac6db3, 66 },
    { 0xd42c6db4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc6db7, 99 },
    { 0, 0 },
    { 0x352c6db9, 90 },
    { 0, 0 },
    { 0x9d0c6dbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26c6dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c6dc1, 91 },
    { 0x276c6dc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc6dc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c6dc8, 23 },
    { 0, 0 },
    { 0xc57c6dca, 90 },
    { 0x73ac6dcb, 96 },
    { 0x5ddc6dcc, 67 },
    { 0xcbfc6dcd, 36 },
    { 0x184c6dce, 76 },
    { 0x1e8c6dcf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c6dd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c6dd7, 76 },
    { 0x983c6dd8, 98 },
    { 0xddcc6dd8, 33 },
    { 0x717c6dd8, 61 },
    { 0x3c3c6ddb, 23 },
    { 0x5c6c6dda, 98 },
    { 0x146c6dda, 99 },
    { 0xe33c6dde, 67 },
    { 0x2cac6ddf, 66 },
    { 0, 0 },
    { 0x44fc6de1, 80 },
    { 0x465c6de1, 87 },
    { 0, 0 },
    { 0xfbec6de4, 52 },
    { 0, 0 },
    { 0xca1c6de6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181c6dee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c6df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fc6df8, 48 },
    { 0xf8cc6df9, 61 },
    { 0xc51c6dfa, 34 },
    { 0xacec6dfb, 33 },
    { 0x22ec6dfa, 95 },
    { 0x646c6dfc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c6e02, 33 },
    { 0, 0 },
    { 0xae1c6e04, 61 },
    { 0, 0 },
    { 0xd4ac6e06, 88 },
    { 0x4b8c6e07, 100 },
    { 0x391c6e08, 76 },
    { 0x9a7c6e09, 90 },
    { 0xe78c6e0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c6e0d, 23 },
    { 0xeefc6e0e, 95 },
    { 0xdf8c6e0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc6e12, 17 },
    { 0x272c6e12, 33 },
    { 0x11fc6e14, 90 },
    { 0, 0 },
    { 0xff0c6e16, 67 },
    { 0x3d6c6e16, 75 },
    { 0x522c6e18, 76 },
    { 0xb4dc6e19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14c6e20, 98 },
    { 0, 0 },
    { 0xd4ac6e22, 90 },
    { 0, 0 },
    { 0x146c6e24, 61 },
    { 0xc4dc6e24, 55 },
    { 0x7a6c6e25, 33 },
    { 0x728c6e24, 34 },
    { 0, 0 },
    { 0x3fbc6e29, 69 },
    { 0, 0 },
    { 0xba8c6e2b, 76 },
    { 0x21dc6e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc6e2f, 55 },
    { 0xb3dc6e2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c6e33, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc6e36, 95 },
    { 0xac6c6e37, 23 },
    { 0xa6c6e37, 77 },
    { 0x8e5c6e39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeec6e3c, 90 },
    { 0x92c6e3d, 55 },
    { 0x5ebc6e3e, 90 },
    { 0x539c6e3f, 55 },
    { 0x72dc6e40, 98 },
    { 0x54dc6e3d, 46 },
    { 0x907c6e3e, 23 },
    { 0x952c6e43, 61 },
    { 0xddbc6e42, 23 },
    { 0x906c6e41, 90 },
    { 0x119c6e46, 95 },
    { 0xef8c6e47, 84 },
    { 0xb5dc6e48, 56 },
    { 0x21bc6e49, 33 },
    { 0xea3c6e43, 43 },
    { 0, 0 },
    { 0x5f9c6e4c, 84 },
    { 0x627c6e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bec6e50, 90 },
    { 0xe4cc6e51, 90 },
    { 0, 0 },
    { 0x18cc6e53, 95 },
    { 0, 0 },
    { 0x497c6e55, 78 },
    { 0, 0 },
    { 0xc38c6e57, 34 },
    { 0x302c6e57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcc6e5b, 61 },
    { 0x64fc6e5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c6e62, 55 },
    { 0xfc2c6e63, 67 },
    { 0x5c4c6e63, 90 },
    { 0x6dbc6e65, 76 },
    { 0x584c6e66, 98 },
    { 0x12bc6e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac6e6b, 23 },
    { 0x54ac6e6c, 99 },
    { 0, 0 },
    { 0xe2ac6e6e, 23 },
    { 0xd1dc6e6f, 33 },
    { 0, 0 },
    { 0xf6ac6e71, 75 },
    { 0x770c6e72, 66 },
    { 0xbb7c6e72, 95 },
    { 0, 0 },
    { 0x350c6e75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0c6e78, 46 },
    { 0xc88c6e78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793c6e7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c6e87, 95 },
    { 0, 0 },
    { 0x18cc6e89, 90 },
    { 0xd73c6e8a, 96 },
    { 0x266c6e8a, 99 },
    { 0, 0 },
    { 0x95ec6e8d, 93 },
    { 0x2f9c6e8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c6e94, 23 },
    { 0xa07c6e95, 33 },
    { 0x7c3c6e94, 95 },
    { 0x5c9c6e96, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c6e9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c6e9e, 93 },
    { 0x7e7c6e9f, 90 },
    { 0xd1dc6ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fec6ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648c6eae, 33 },
    { 0xb87c6eaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eec6eb8, 34 },
    { 0x325c6eb9, 87 },
    { 0x715c6eb9, 61 },
    { 0xb39c6ebb, 90 },
    { 0, 0 },
    { 0x58c6ebd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5c6ec0, 33 },
    { 0, 0 },
    { 0xa1fc6ec2, 33 },
    { 0x2d3c6ec3, 95 },
    { 0x6fac6ec3, 95 },
    { 0xf13c6ec5, 33 },
    { 0x433c6ec6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501c6ecd, 46 },
    { 0, 0 },
    { 0xe16c6ecf, 76 },
    { 0, 0 },
    { 0x76bc6ed1, 88 },
    { 0x28dc6ed1, 61 },
    { 0x6a3c6ed3, 76 },
    { 0xc1cc6ed4, 33 },
    { 0, 0 },
    { 0x3a5c6ed6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c6edb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecc6ee0, 61 },
    { 0x62fc6ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c6ee5, 90 },
    { 0, 0 },
    { 0x9b2c6ee7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c6eea, 95 },
    { 0x1dcc6eea, 90 },
    { 0x99ac6eea, 33 },
    { 0, 0 },
    { 0xb95c6eee, 33 },
    { 0x59dc6eee, 91 },
    { 0x548c6ef0, 95 },
    { 0x730c6ef1, 90 },
    { 0xb2ec6ef2, 98 },
    { 0, 0 },
    { 0xffcc6ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c6ef7, 12 },
    { 0xecfc6ef8, 61 },
    { 0xa2ec6ef9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c6efc, 98 },
    { 0, 0 },
    { 0x452c6efe, 98 },
    { 0x1c6efe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc6f02, 76 },
    { 0x2f6c6f02, 90 },
    { 0x6b9c6f03, 67 },
    { 0, 0 },
    { 0xaf9c6f06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c6f09, 90 },
    { 0x3edc6f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bc6f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9c6f11, 46 },
    { 0x773c6f12, 56 },
    { 0x912c6f13, 75 },
    { 0xae9c6f14, 93 },
    { 0x21fc6f14, 61 },
    { 0xe5dc6f13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc6f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882c6f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448c6f2c, 34 },
    { 0x807c6f2d, 90 },
    { 0x181c6f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfc6f32, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6f37, 84 },
    { 0x168c6f38, 95 },
    { 0x1acc6f38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7c6f45, 93 },
    { 0xfccc6f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c6f49, 55 },
    { 0xb94c6f49, 90 },
    { 0x5d6c6f49, 40 },
    { 0x6c1c6f4a, 23 },
    { 0x396c6f4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcc6f53, 34 },
    { 0xf6c6f54, 46 },
    { 0x2b7c6f55, 61 },
    { 0xbc5c6f56, 90 },
    { 0x6c6f57, 95 },
    { 0xd0ac6f54, 33 },
    { 0, 0 },
    { 0x544c6f5a, 23 },
    { 0xb84c6f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c6f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbc6f66, 46 },
    { 0, 0 },
    { 0xa21c6f68, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978c6f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470c6f71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aec6f7a, 85 },
    { 0x5c9c6f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc6f7e, 53 },
    { 0, 0 },
    { 0xb19c6f80, 33 },
    { 0x389c6f80, 55 },
    { 0, 0 },
    { 0xb1fc6f83, 90 },
    { 0x87bc6f83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cec6f87, 95 },
    { 0x6f0c6f87, 43 },
    { 0, 0 },
    { 0xcb2c6f8a, 78 },
    { 0x515c6f8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fc6f90, 84 },
    { 0xb56c6f91, 34 },
    { 0, 0 },
    { 0xed3c6f93, 90 },
    { 0x5b0c6f94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46c6f99, 33 },
    { 0, 0 },
    { 0xd8c6f9b, 43 },
    { 0xe2ec6f9b, 90 },
    { 0, 0 },
    { 0xbfec6f9e, 76 },
    { 0, 0 },
    { 0xb0bc6fa0, 61 },
    { 0xa31c6fa0, 76 },
    { 0xcb0c6fa2, 84 },
    { 0x244c6fa1, 99 },
    { 0x212c6fa4, 95 },
    { 0xd3fc6fa5, 52 },
    { 0x5b3c6fa2, 61 },
    { 0xd1fc6fa7, 90 },
    { 0, 0 },
    { 0x111c6fa9, 34 },
    { 0, 0 },
    { 0xbb3c6fab, 61 },
    { 0x676c6fab, 98 },
    { 0, 0 },
    { 0xaf4c6fae, 98 },
    { 0, 0 },
    { 0x8b9c6fb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c6fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dc6fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664c6fc0, 95 },
    { 0x343c6fc1, 33 },
    { 0x78bc6fc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x806c6fc5, 40 },
    { 0xb4fc6fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dc6fcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cc6fd0, 78 },
    { 0, 0 },
    { 0x282c6fd2, 95 },
    { 0, 0 },
    { 0x291c6fd4, 100 },
    { 0, 0 },
    { 0xcaec6fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c6fdc, 33 },
    { 0xfcc6fdd, 98 },
    { 0xe39c6fdd, 95 },
    { 0x188c6fdf, 87 },
    { 0x3bfc6fe0, 90 },
    { 0, 0 },
    { 0xf53c6fe2, 76 },
    { 0xc43c6fe2, 34 },
    { 0x106c6fe4, 90 },
    { 0x6cbc6fe5, 61 },
    { 0xdabc6fe5, 23 },
    { 0xbbec6fe7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc6fea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c6fef, 43 },
    { 0, 0 },
    { 0xb13c6ff1, 78 },
    { 0x684c6ff2, 98 },
    { 0x4fec6ff2, 90 },
    { 0x871c6ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfc6ffb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c6ffe, 75 },
    { 0x677c6fff, 46 },
    { 0, 0 },
    { 0xd22c7001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfc7005, 46 },
    { 0xe2fc7006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec700d, 98 },
    { 0xa22c700e, 90 },
    { 0x7a4c700d, 90 },
    { 0xf19c700d, 95 },
    { 0xffec7011, 98 },
    { 0x995c7012, 77 },
    { 0x326c700e, 34 },
    { 0xbcac7011, 33 },
    { 0x213c700e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352c7019, 68 },
    { 0, 0 },
    { 0x5ccc701b, 55 },
    { 0x3e4c701c, 90 },
    { 0, 0 },
    { 0xd00c701e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c7021, 78 },
    { 0x43bc7022, 93 },
    { 0xd1c7022, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690c702a, 61 },
    { 0, 0 },
    { 0x806c702c, 33 },
    { 0x521c702d, 34 },
    { 0xcc6c702e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84c703a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c703d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc7040, 33 },
    { 0x78cc7041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56c7046, 23 },
    { 0x332c7047, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x426c704a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc704f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832c7053, 61 },
    { 0xcb8c7053, 90 },
    { 0, 0 },
    { 0x1fcc7056, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc705c, 76 },
    { 0, 0 },
    { 0x459c705e, 33 },
    { 0xd6ec705e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac7062, 43 },
    { 0, 0 },
    { 0xdd7c7064, 43 },
    { 0xbcac7065, 61 },
    { 0x402c7066, 85 },
    { 0x3fcc7067, 33 },
    { 0x4f4c7066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958c706b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c7070, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c7075, 95 },
    { 0xb73c7076, 84 },
    { 0x7cac7077, 23 },
    { 0, 0 },
    { 0x299c7079, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fec707f, 98 },
    { 0xdefc7080, 95 },
    { 0x14c7081, 98 },
    { 0xdcbc7082, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63c708f, 95 },
    { 0, 0 },
    { 0x25fc7091, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77c709c, 98 },
    { 0xc69c709d, 55 },
    { 0x522c709d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x742c70a1, 34 },
    { 0, 0 },
    { 0x903c70a3, 90 },
    { 0xdf0c70a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6c70a7, 90 },
    { 0x999c70a7, 95 },
    { 0xcbac70a9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c70ac, 90 },
    { 0x329c70ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92c70b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c70b9, 61 },
    { 0x430c70ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2c70bd, 46 },
    { 0xf43c70bd, 98 },
    { 0, 0 },
    { 0x39fc70c0, 91 },
    { 0xd9ec70c0, 78 },
    { 0x2e5c70c0, 93 },
    { 0x109c70c2, 90 },
    { 0xae5c70c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c70c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c70cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc70ce, 61 },
    { 0x269c70ce, 76 },
    { 0x43ec70ce, 90 },
    { 0xff3c70d1, 98 },
    { 0xf0fc70d1, 34 },
    { 0x4dec70d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec70d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bc70d9, 98 },
    { 0x9dc70da, 90 },
    { 0x19dc70db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc70df, 90 },
    { 0x9a3c70df, 76 },
    { 0, 0 },
    { 0x216c70e2, 98 },
    { 0, 0 },
    { 0x73bc70e4, 98 },
    { 0, 0 },
    { 0x711c70e6, 55 },
    { 0, 0 },
    { 0xf76c70e8, 34 },
    { 0x922c70e8, 61 },
    { 0xedac70ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x261c70ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0c70f1, 23 },
    { 0xc6dc70f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114c70fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ac70ff, 40 },
    { 0xa48c70ff, 34 },
    { 0x52bc7101, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x388c7104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47c7109, 59 },
    { 0, 0 },
    { 0xaa9c710b, 99 },
    { 0, 0 },
    { 0x315c710d, 80 },
    { 0, 0 },
    { 0x30dc710f, 34 },
    { 0xee7c7110, 84 },
    { 0x5cdc7111, 98 },
    { 0xac4c7110, 61 },
    { 0x74ec7113, 66 },
    { 0x93ec7110, 98 },
    { 0x9a8c7110, 55 },
    { 0xc7ec7112, 90 },
    { 0, 0 },
    { 0xadac7118, 98 },
    { 0, 0 },
    { 0x7e5c711a, 33 },
    { 0, 0 },
    { 0x7c6c711c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec7129, 17 },
    { 0x6d1c712a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c712e, 77 },
    { 0xd53c712f, 23 },
    { 0xe7ec7130, 88 },
    { 0, 0 },
    { 0xa3bc7132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c7137, 34 },
    { 0x8b5c7137, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcc713c, 52 },
    { 0xa69c713c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac7143, 98 },
    { 0xd0fc7143, 95 },
    { 0x5e7c7144, 33 },
    { 0xfcec7146, 61 },
    { 0, 0 },
    { 0xcc0c7148, 61 },
    { 0xae6c7149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c714d, 65 },
    { 0xf50c714d, 98 },
    { 0xf6c714f, 85 },
    { 0, 0 },
    { 0x112c7151, 15 },
    { 0x780c7152, 61 },
    { 0xa21c7153, 34 },
    { 0x287c7154, 33 },
    { 0xd5ac7152, 61 },
    { 0x39ac7151, 90 },
    { 0x248c7153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc715b, 76 },
    { 0xea7c715b, 33 },
    { 0, 0 },
    { 0x233c715e, 23 },
    { 0x272c715f, 33 },
    { 0xadec7160, 87 },
    { 0x91dc7161, 95 },
    { 0xa29c7162, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4c7165, 95 },
    { 0x598c7165, 76 },
    { 0, 0 },
    { 0x8f1c7168, 61 },
    { 0, 0 },
    { 0xbc4c716a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6c716d, 93 },
    { 0xb33c716e, 23 },
    { 0, 0 },
    { 0x82ec7170, 78 },
    { 0x828c7171, 55 },
    { 0x3aec7172, 90 },
    { 0xed2c7173, 34 },
    { 0xea1c7174, 55 },
    { 0, 0 },
    { 0x332c7176, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c7179, 46 },
    { 0x923c717a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c717d, 98 },
    { 0x7f9c717e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbc7182, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cc7185, 61 },
    { 0, 0 },
    { 0x410c7187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51c718a, 85 },
    { 0, 0 },
    { 0xb33c718c, 33 },
    { 0xa58c718d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c7190, 64 },
    { 0x9e3c7190, 34 },
    { 0x863c7192, 33 },
    { 0x3c7193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c7199, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc719e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfc71a3, 66 },
    { 0x74c71a3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefac71a9, 93 },
    { 0, 0 },
    { 0xd91c71ab, 23 },
    { 0, 0 },
    { 0xd2dc71ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cc71b0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ac71b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfc71b9, 95 },
    { 0, 0 },
    { 0x33cc71bb, 33 },
    { 0, 0 },
    { 0x810c71bd, 33 },
    { 0x4d7c71bd, 77 },
    { 0, 0 },
    { 0xe04c71c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135c71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec71cc, 95 },
    { 0x27c71cd, 61 },
    { 0xecfc71cc, 33 },
    { 0xb94c71ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x352c71d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c71d5, 90 },
    { 0x3a3c71d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c71d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64c71dc, 23 },
    { 0x8d7c71dd, 77 },
    { 0, 0 },
    { 0xab6c71df, 33 },
    { 0, 0 },
    { 0x9d3c71e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5c71e9, 55 },
    { 0x6b7c71ea, 55 },
    { 0xa3cc71e9, 23 },
    { 0x54cc71ec, 93 },
    { 0, 0 },
    { 0x779c71ee, 78 },
    { 0, 0 },
    { 0xafdc71f0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c71f3, 90 },
    { 0, 0 },
    { 0x422c71f5, 95 },
    { 0xe3cc71f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760c71fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c71ff, 33 },
    { 0xb5c71ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfec7205, 61 },
    { 0, 0 },
    { 0x31bc7207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2c720b, 100 },
    { 0, 0 },
    { 0x6cac720d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc7213, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4c7217, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c721c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c721f, 95 },
    { 0xabac7220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c7224, 23 },
    { 0x1afc7224, 77 },
    { 0x9d5c7226, 90 },
    { 0xb70c7226, 100 },
    { 0x904c7228, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac722f, 46 },
    { 0x5d9c7230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c7233, 23 },
    { 0, 0 },
    { 0x879c7235, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6c7242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c7245, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc724a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac724d, 27 },
    { 0, 0 },
    { 0x257c724f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcc7254, 55 },
    { 0x77ec7255, 34 },
    { 0x893c7255, 98 },
    { 0x6ffc7255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760c725b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdc725e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc7261, 61 },
    { 0x509c7262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc7267, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebc726b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x405c726e, 60 },
    { 0xf7ec726f, 84 },
    { 0, 0 },
    { 0x221c7271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c7275, 98 },
    { 0xbb8c7276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bc7279, 55 },
    { 0, 0 },
    { 0xc2ec727b, 90 },
    { 0x72cc727c, 67 },
    { 0x96bc727d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753c7285, 61 },
    { 0x1a7c7285, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4c7289, 91 },
    { 0xeccc728a, 34 },
    { 0, 0 },
    { 0x79bc728c, 87 },
    { 0x200c728d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c7291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c7294, 90 },
    { 0x329c7294, 33 },
    { 0x742c7294, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c7299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac729e, 23 },
    { 0xe48c729f, 93 },
    { 0, 0 },
    { 0x35bc72a1, 90 },
    { 0, 0 },
    { 0xea3c72a3, 98 },
    { 0, 0 },
    { 0x7ec72a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9c72a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c72b1, 90 },
    { 0x528c72b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc72b8, 23 },
    { 0xd0cc72b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88c72c1, 33 },
    { 0xe96c72c2, 77 },
    { 0x1f4c72c3, 67 },
    { 0, 0 },
    { 0x6fdc72c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc72c8, 90 },
    { 0x8f6c72c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc72ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eac72d5, 66 },
    { 0x206c72d6, 23 },
    { 0x72c72d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c72dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc72e0, 95 },
    { 0, 0 },
    { 0x553c72e2, 98 },
    { 0x3a2c72e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc72e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c72ec, 46 },
    { 0xd59c72ed, 23 },
    { 0xe8cc72ec, 43 },
    { 0xc8ac72ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc72f3, 81 },
    { 0x79ac72f4, 61 },
    { 0x495c72f4, 23 },
    { 0x252c72f6, 99 },
    { 0x2cbc72f7, 33 },
    { 0, 0 },
    { 0xad1c72f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c72fc, 68 },
    { 0x5c2c72fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399c7306, 23 },
    { 0xac3c7307, 55 },
    { 0, 0 },
    { 0x1afc7309, 95 },
    { 0xef3c730a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c730f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891c7313, 34 },
    { 0xb88c7314, 98 },
    { 0, 0 },
    { 0x94bc7316, 93 },
    { 0x382c7317, 90 },
    { 0x237c7318, 55 },
    { 0, 0 },
    { 0xf0dc731a, 90 },
    { 0, 0 },
    { 0x665c731c, 98 },
    { 0x408c731d, 95 },
    { 0, 0 },
    { 0xab7c731f, 98 },
    { 0x2e1c731f, 78 },
    { 0x800c7321, 93 },
    { 0xb11c7320, 85 },
    { 0x991c7323, 61 },
    { 0x79dc7322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39c7328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c732b, 67 },
    { 0x1d2c732c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3c7330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952c733a, 23 },
    { 0xc82c733a, 98 },
    { 0xcf1c733c, 95 },
    { 0xa24c733d, 23 },
    { 0xe19c733d, 23 },
    { 0, 0 },
    { 0x7e7c7340, 34 },
    { 0, 0 },
    { 0x1cc7342, 61 },
    { 0, 0 },
    { 0x659c7344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17c7347, 33 },
    { 0, 0 },
    { 0x9f4c7349, 33 },
    { 0xc2bc734a, 98 },
    { 0xd92c7349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c7350, 23 },
    { 0xcf7c7351, 67 },
    { 0, 0 },
    { 0x89ac7353, 61 },
    { 0x4bdc7354, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bc735e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c7361, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6c7364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc7368, 95 },
    { 0, 0 },
    { 0xa3dc736a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c736f, 78 },
    { 0x70ac7370, 98 },
    { 0, 0 },
    { 0xe7fc7372, 90 },
    { 0x6eac7373, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ac7376, 23 },
    { 0x969c7376, 55 },
    { 0x37ec7378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc737c, 61 },
    { 0xa42c737d, 90 },
    { 0x5ac737c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bc7382, 33 },
    { 0x504c7383, 34 },
    { 0, 0 },
    { 0x792c7385, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cc738c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc7390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bec7394, 33 },
    { 0x152c7394, 53 },
    { 0x863c7394, 78 },
    { 0x35cc7396, 98 },
    { 0xba5c7398, 33 },
    { 0x8bbc7398, 90 },
    { 0x3b6c739a, 33 },
    { 0x54dc739a, 43 },
    { 0xaf7c739c, 55 },
    { 0, 0 },
    { 0x8dc739e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dc73a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x965c73a6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccc73ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2c73b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc73b3, 43 },
    { 0x5c4c73b3, 90 },
    { 0, 0 },
    { 0x6e2c73b6, 57 },
    { 0x3fac73b7, 90 },
    { 0x280c73b8, 61 },
    { 0x93dc73b9, 95 },
    { 0, 0 },
    { 0x697c73bb, 90 },
    { 0xfd1c73bc, 78 },
    { 0x9f9c73bd, 61 },
    { 0xcacc73be, 61 },
    { 0xc64c73be, 90 },
    { 0, 0 },
    { 0x192c73c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adc73c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc73c8, 90 },
    { 0x7b1c73c8, 23 },
    { 0x905c73c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x841c73cd, 99 },
    { 0xcf3c73cd, 78 },
    { 0x512c73cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc73d6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264c73da, 23 },
    { 0x316c73da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17c73e0, 43 },
    { 0xb64c73e0, 55 },
    { 0x7ec73e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c73ec, 33 },
    { 0x6dbc73ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dc73f2, 91 },
    { 0xde7c73f3, 33 },
    { 0xfb3c73f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9c73f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c73fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbc73fe, 68 },
    { 0x7d9c73ff, 61 },
    { 0xebdc7400, 52 },
    { 0x5e8c73fe, 67 },
    { 0xf1c7401, 100 },
    { 0x4a5c7402, 88 },
    { 0xd81c7404, 69 },
    { 0xc42c7405, 98 },
    { 0, 0 },
    { 0x3bcc7407, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c740a, 76 },
    { 0x145c740b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfc740e, 100 },
    { 0x32c740f, 90 },
    { 0, 0 },
    { 0x3b6c7411, 55 },
    { 0x744c7412, 61 },
    { 0, 0 },
    { 0x577c7414, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456c741a, 95 },
    { 0x901c741b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x284c741e, 78 },
    { 0, 0 },
    { 0xc5ac7420, 40 },
    { 0x601c7421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c7424, 90 },
    { 0xc8cc7425, 17 },
    { 0x373c7425, 34 },
    { 0, 0 },
    { 0xed2c7428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fc742c, 99 },
    { 0x24c742c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188c7432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x377c7435, 90 },
    { 0x281c7436, 95 },
    { 0, 0 },
    { 0x999c7438, 23 },
    { 0, 0 },
    { 0x33dc743a, 33 },
    { 0x90dc743a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c7446, 23 },
    { 0x3efc7447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617c744b, 61 },
    { 0xe85c744b, 85 },
    { 0x6d6c744c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ec7450, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c7454, 95 },
    { 0, 0 },
    { 0xef4c7456, 99 },
    { 0, 0 },
    { 0x202c7458, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3c745b, 33 },
    { 0, 0 },
    { 0x9a4c745d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fc7461, 93 },
    { 0xaf4c7462, 88 },
    { 0xca9c7462, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76c7467, 87 },
    { 0x70c7467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1c746d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dc7471, 23 },
    { 0, 0 },
    { 0x724c7473, 97 },
    { 0xadcc7474, 90 },
    { 0xd8ac7475, 98 },
    { 0, 0 },
    { 0xfffc7477, 43 },
    { 0x229c7478, 91 },
    { 0, 0 },
    { 0xef8c747a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796c7481, 33 },
    { 0xf1cc7481, 98 },
    { 0xc00c7483, 91 },
    { 0xeac7484, 90 },
    { 0, 0 },
    { 0x6b6c7486, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc748a, 90 },
    { 0, 0 },
    { 0x3fcc748c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c7493, 85 },
    { 0xfc7c7493, 85 },
    { 0xbc0c7493, 23 },
    { 0, 0 },
    { 0x8edc7497, 75 },
    { 0x459c7498, 55 },
    { 0xbb1c7499, 98 },
    { 0xc0bc749a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c749e, 33 },
    { 0xca4c749f, 90 },
    { 0xe04c74a0, 95 },
    { 0xcbdc74a1, 76 },
    { 0xf54c74a2, 90 },
    { 0, 0 },
    { 0xc2bc74a4, 98 },
    { 0x573c74a4, 46 },
    { 0x5cfc74a6, 100 },
    { 0, 0 },
    { 0x58ac74a8, 90 },
    { 0x80fc74a9, 93 },
    { 0xca6c74aa, 23 },
    { 0xe84c74a9, 34 },
    { 0, 0 },
    { 0xd88c74ad, 98 },
    { 0xb4fc74ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bc74b9, 81 },
    { 0xf47c74b9, 33 },
    { 0xeecc74b9, 90 },
    { 0x194c74ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fc74c0, 23 },
    { 0x515c74c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2c74ca, 33 },
    { 0x664c74cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bc74ce, 88 },
    { 0x6c4c74ce, 61 },
    { 0x71ec74ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972c74d9, 17 },
    { 0xf36c74da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c74df, 90 },
    { 0xe23c74e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c74e8, 85 },
    { 0xaf4c74e9, 23 },
    { 0x7adc74e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7c74ed, 46 },
    { 0, 0 },
    { 0x13fc74ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ec74f4, 61 },
    { 0, 0 },
    { 0xfb7c74f6, 76 },
    { 0x3c5c74f7, 95 },
    { 0x868c74f8, 91 },
    { 0xe5dc74f7, 78 },
    { 0xe23c74fa, 23 },
    { 0x551c74fb, 46 },
    { 0x784c74fc, 96 },
    { 0xa11c74fd, 68 },
    { 0xcfac74fe, 34 },
    { 0x2bdc74f7, 43 },
    { 0x1e2c7500, 33 },
    { 0xaf8c7501, 23 },
    { 0xf73c74f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c7508, 91 },
    { 0x1e8c7509, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99c750c, 52 },
    { 0x714c750d, 33 },
    { 0xd72c750e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c7511, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c7514, 90 },
    { 0x6fc7514, 43 },
    { 0x70cc7514, 55 },
    { 0, 0 },
    { 0x206c7518, 55 },
    { 0x6fec7519, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c751e, 90 },
    { 0, 0 },
    { 0x356c7520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c7523, 33 },
    { 0x4a0c7524, 93 },
    { 0x5bcc7524, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595c752a, 95 },
    { 0x63fc752b, 91 },
    { 0x19ac752c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3c752f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bc7537, 90 },
    { 0x131c7538, 95 },
    { 0x9f3c7539, 33 },
    { 0, 0 },
    { 0xcacc753b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cc7542, 98 },
    { 0xc88c7543, 61 },
    { 0xab5c7544, 33 },
    { 0x2c0c7543, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dc7550, 78 },
    { 0, 0 },
    { 0xaf7c7552, 76 },
    { 0xd32c7552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ec7558, 95 },
    { 0xe7ec7558, 98 },
    { 0, 0 },
    { 0x109c755b, 98 },
    { 0xa59c755b, 33 },
    { 0, 0 },
    { 0x68ec755e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4c7564, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4c7567, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c7576, 33 },
    { 0x825c7576, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86c757a, 33 },
    { 0, 0 },
    { 0xd8dc757c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x882c757f, 81 },
    { 0, 0 },
    { 0xf46c7581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23c7584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7588, 76 },
    { 0xb79c7589, 36 },
    { 0x7e0c758a, 66 },
    { 0x51ec758b, 75 },
    { 0x618c758c, 76 },
    { 0x40cc758a, 76 },
    { 0xa7c7589, 88 },
    { 0, 0 },
    { 0x19ec7590, 67 },
    { 0, 0 },
    { 0x1a2c7592, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507c7598, 99 },
    { 0x9ec7598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc759c, 33 },
    { 0, 0 },
    { 0x994c759e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fc75a4, 43 },
    { 0x94dc75a5, 17 },
    { 0x442c75a5, 75 },
    { 0, 0 },
    { 0xa51c75a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c75b1, 61 },
    { 0, 0 },
    { 0xa85c75b3, 46 },
    { 0, 0 },
    { 0x8c3c75b5, 100 },
    { 0, 0 },
    { 0x736c75b7, 38 },
    { 0x8dc75b8, 90 },
    { 0x983c75b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fc75bd, 90 },
    { 0, 0 },
    { 0xfaec75bf, 23 },
    { 0, 0 },
    { 0x61fc75c1, 98 },
    { 0xa9fc75c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309c75c7, 98 },
    { 0x8f3c75c8, 95 },
    { 0x183c75c8, 90 },
    { 0x6b9c75c8, 95 },
    { 0, 0 },
    { 0xf40c75cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cc75cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dc75d2, 90 },
    { 0xaa2c75d2, 33 },
    { 0x622c75d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c75d9, 19 },
    { 0xdbcc75da, 88 },
    { 0xbaac75da, 87 },
    { 0xb3ec75db, 23 },
    { 0x1cdc75dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9c75e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0c75e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c75e7, 85 },
    { 0xb3bc75e7, 61 },
    { 0x3eec75e8, 23 },
    { 0xecac75e8, 61 },
    { 0x3c75eb, 23 },
    { 0x76c75ec, 90 },
    { 0, 0 },
    { 0x4f0c75ee, 66 },
    { 0x45dc75ef, 34 },
    { 0xd36c75f0, 99 },
    { 0x5f6c75f1, 98 },
    { 0x79cc75f2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aec75f6, 55 },
    { 0x212c75f7, 90 },
    { 0xa2c75f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315c75ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c7602, 23 },
    { 0x9c6c7603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccc760b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94c760e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c7613, 43 },
    { 0xe35c7613, 91 },
    { 0, 0 },
    { 0xf87c7616, 98 },
    { 0x3f8c7616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec761a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x128c761d, 61 },
    { 0xf66c761e, 76 },
    { 0x9d9c761d, 90 },
    { 0xa2ac7620, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc7623, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac7626, 76 },
    { 0xcb7c7627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7c762a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x555c762d, 76 },
    { 0xb53c762d, 98 },
    { 0xa13c762f, 34 },
    { 0xb2fc762f, 33 },
    { 0xa13c7630, 95 },
    { 0x13bc762d, 23 },
    { 0xdc1c7633, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8c7639, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571c763e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5c7642, 75 },
    { 0xf58c7643, 23 },
    { 0xc7ec7642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7c7648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71c7650, 33 },
    { 0, 0 },
    { 0xd09c7652, 67 },
    { 0xd79c7653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6c7656, 98 },
    { 0x766c7656, 66 },
    { 0x297c7658, 95 },
    { 0x5fbc7659, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fc765e, 90 },
    { 0x937c765f, 90 },
    { 0x857c7660, 78 },
    { 0x80dc7661, 61 },
    { 0, 0 },
    { 0xa44c7663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c7667, 87 },
    { 0xf4c7668, 85 },
    { 0, 0 },
    { 0x27fc766a, 23 },
    { 0x496c766b, 61 },
    { 0, 0 },
    { 0xc2fc766d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac7673, 88 },
    { 0x28ec7674, 33 },
    { 0x2eac7674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bc768e, 55 },
    { 0xa3fc768f, 85 },
    { 0xdd6c7690, 61 },
    { 0xcc1c7690, 68 },
    { 0x4fcc7692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c7695, 99 },
    { 0xa6dc7696, 90 },
    { 0xa0ac7697, 75 },
    { 0x3f6c7697, 77 },
    { 0x773c7699, 61 },
    { 0x9ec769a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc769d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc76a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c76a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c76a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3c76ab, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c76b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9c76b4, 66 },
    { 0xccac76b4, 67 },
    { 0x2d8c76b6, 61 },
    { 0, 0 },
    { 0x817c76b8, 91 },
    { 0x177c76b9, 33 },
    { 0x4c8c76b9, 33 },
    { 0x989c76ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc76be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aac76c3, 95 },
    { 0xd0dc76c3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c76c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41c76cb, 52 },
    { 0x8b5c76cb, 43 },
    { 0xec2c76cd, 98 },
    { 0xdccc76ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc76d1, 56 },
    { 0, 0 },
    { 0xd1cc76d3, 95 },
    { 0xae4c76d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71c76d8, 93 },
    { 0x5dec76d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc76dc, 23 },
    { 0, 0 },
    { 0x1a4c76de, 61 },
    { 0xa32c76de, 43 },
    { 0xb42c76de, 7 },
    { 0xfd1c76e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c76e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10c76e9, 23 },
    { 0x397c76e9, 36 },
    { 0, 0 },
    { 0xe23c76ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x418c76ef, 98 },
    { 0, 0 },
    { 0xcc76f1, 34 },
    { 0, 0 },
    { 0xe03c76f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc76f6, 46 },
    { 0x548c76f6, 68 },
    { 0x58ec76f8, 90 },
    { 0x6c0c76f6, 95 },
    { 0, 0 },
    { 0xc1c76fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88c76fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc7707, 78 },
    { 0x19ec7708, 98 },
    { 0x7adc7707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc770c, 90 },
    { 0xb50c770d, 55 },
    { 0x862c770c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9c7711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319c7716, 90 },
    { 0x4c6c7717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc7721, 84 },
    { 0, 0 },
    { 0x200c7723, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c7728, 46 },
    { 0, 0 },
    { 0x294c772a, 33 },
    { 0x841c772a, 95 },
    { 0, 0 },
    { 0x69fc772d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171c7731, 52 },
    { 0xaa7c7732, 90 },
    { 0xb1fc7733, 34 },
    { 0, 0 },
    { 0x2acc7735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77c773d, 95 },
    { 0, 0 },
    { 0x870c773f, 23 },
    { 0, 0 },
    { 0xb94c7741, 55 },
    { 0, 0 },
    { 0xadc7743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41c774b, 76 },
    { 0, 0 },
    { 0x4c9c774d, 55 },
    { 0xaedc774d, 98 },
    { 0xc27c774d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc7753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50c775b, 98 },
    { 0, 0 },
    { 0x225c775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc7760, 95 },
    { 0x79c7760, 76 },
    { 0xc0bc7762, 73 },
    { 0, 0 },
    { 0xa66c7764, 76 },
    { 0, 0 },
    { 0x8a0c7766, 76 },
    { 0, 0 },
    { 0xec0c7768, 98 },
    { 0xacec7768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c7770, 78 },
    { 0, 0 },
    { 0x8e4c7772, 98 },
    { 0, 0 },
    { 0x581c7774, 98 },
    { 0xbb9c7774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190c7778, 95 },
    { 0x7a3c7779, 90 },
    { 0xc98c7778, 98 },
    { 0xeb1c777b, 61 },
    { 0x5a4c777c, 95 },
    { 0xd0cc777c, 61 },
    { 0x13ac777c, 23 },
    { 0xc1bc777d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05c7782, 23 },
    { 0, 0 },
    { 0xf0dc7784, 46 },
    { 0x786c7785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bec7790, 90 },
    { 0xaa2c7790, 23 },
    { 0, 0 },
    { 0x999c7793, 52 },
    { 0x61fc7794, 61 },
    { 0, 0 },
    { 0xd39c7796, 98 },
    { 0xc88c7797, 61 },
    { 0x808c7797, 90 },
    { 0, 0 },
    { 0x710c779a, 70 },
    { 0, 0 },
    { 0xfb9c779c, 98 },
    { 0x7c779d, 98 },
    { 0xb94c779d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90c77a1, 34 },
    { 0x1c0c77a2, 88 },
    { 0xe50c77a3, 90 },
    { 0xc6c77a4, 23 },
    { 0xb77c77a5, 90 },
    { 0x2c0c77a2, 98 },
    { 0x44ec77a6, 76 },
    { 0x8c2c77a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0c77ab, 52 },
    { 0x160c77ac, 61 },
    { 0x171c77ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccc77b4, 61 },
    { 0, 0 },
    { 0x608c77b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72c77bc, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c77c2, 55 },
    { 0x2dec77c3, 2 },
    { 0x8e2c77c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbc77c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96c77ce, 90 },
    { 0x233c77ce, 76 },
    { 0, 0 },
    { 0x2e8c77d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77d5, 33 },
    { 0xf0fc77d6, 88 },
    { 0xea8c77d6, 85 },
    { 0xd87c77d8, 98 },
    { 0x6b4c77d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc77e0, 55 },
    { 0, 0 },
    { 0xea8c77e2, 34 },
    { 0xfa0c77e3, 78 },
    { 0x115c77e2, 90 },
    { 0x7edc77e2, 23 },
    { 0x9c7c77e6, 90 },
    { 0x355c77e2, 98 },
    { 0x4e6c77e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c77ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3c77f9, 90 },
    { 0x682c77fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4c7800, 23 },
    { 0xb5cc7800, 90 },
    { 0x560c7802, 85 },
    { 0, 0 },
    { 0xe32c7804, 61 },
    { 0, 0 },
    { 0x3e4c7806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc7809, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dac780f, 33 },
    { 0, 0 },
    { 0xe3ac7811, 68 },
    { 0x87ac7811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934c7816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bec7819, 23 },
    { 0x518c781a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4c7828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbc782b, 76 },
    { 0xb9bc782b, 76 },
    { 0, 0 },
    { 0xd3c782e, 34 },
    { 0xbec782f, 38 },
    { 0, 0 },
    { 0x840c7831, 33 },
    { 0, 0 },
    { 0x552c7833, 81 },
    { 0x53fc7834, 61 },
    { 0, 0 },
    { 0x37ac7836, 61 },
    { 0x3acc7836, 61 },
    { 0x708c7837, 61 },
    { 0xaebc7839, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cc783f, 33 },
    { 0x5c5c7840, 98 },
    { 0xf9ac7840, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc7844, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc7847, 87 },
    { 0x5f3c7848, 33 },
    { 0xbb9c7849, 23 },
    { 0x959c784a, 95 },
    { 0, 0 },
    { 0x46ec784c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdc784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fc7854, 100 },
    { 0xa56c7855, 78 },
    { 0x796c7854, 33 },
    { 0xb95c7855, 34 },
    { 0xdf0c7857, 90 },
    { 0, 0 },
    { 0x9a3c785a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c785d, 96 },
    { 0xfb4c785e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346c7864, 98 },
    { 0, 0 },
    { 0xc53c7866, 95 },
    { 0, 0 },
    { 0xf4cc7868, 99 },
    { 0xd4cc7869, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5c786c, 33 },
    { 0x103c786d, 99 },
    { 0x472c786e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6c7874, 95 },
    { 0, 0 },
    { 0x845c7876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c787c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc787f, 43 },
    { 0x506c7880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec7887, 99 },
    { 0xcbec7887, 76 },
    { 0x53ec7889, 95 },
    { 0xda2c788a, 100 },
    { 0xf73c788a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869c788f, 33 },
    { 0x3d1c788f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c7894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc789e, 61 },
    { 0x6efc789f, 33 },
    { 0x2ac789e, 90 },
    { 0xd36c789e, 90 },
    { 0, 0 },
    { 0xe56c78a3, 95 },
    { 0, 0 },
    { 0x19bc78a5, 90 },
    { 0x1bbc78a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bc78aa, 84 },
    { 0x263c78ab, 55 },
    { 0, 0 },
    { 0x9d4c78ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c78b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55c78b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13c78b8, 61 },
    { 0x90ac78b9, 84 },
    { 0xdbec78b9, 23 },
    { 0xe3bc78bb, 84 },
    { 0x723c78bc, 23 },
    { 0xcdac78bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c78c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c78c6, 95 },
    { 0x13ec78c6, 90 },
    { 0x166c78c8, 61 },
    { 0x207c78c8, 55 },
    { 0x515c78ca, 90 },
    { 0x890c78c6, 44 },
    { 0x667c78cc, 76 },
    { 0x2c6c78c9, 33 },
    { 0x815c78ce, 33 },
    { 0xf31c78cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c78d6, 33 },
    { 0x1f4c78d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0c78da, 98 },
    { 0xa14c78db, 91 },
    { 0xe96c78da, 98 },
    { 0x78cc78dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6c78e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c78e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8c78e6, 98 },
    { 0, 0 },
    { 0x3d8c78e8, 33 },
    { 0xa1c78e9, 91 },
    { 0xd10c78ea, 48 },
    { 0x8f7c78ea, 87 },
    { 0, 0 },
    { 0x1cc78ed, 23 },
    { 0, 0 },
    { 0x697c78ef, 23 },
    { 0x26ac78f0, 99 },
    { 0x5b1c78f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac78f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc78fc, 61 },
    { 0, 0 },
    { 0x38dc78fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7904, 98 },
    { 0x7d7c7905, 98 },
    { 0x199c7905, 90 },
    { 0x6adc7907, 81 },
    { 0x966c7906, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90c790b, 68 },
    { 0xe02c790c, 68 },
    { 0xc5ac790c, 55 },
    { 0, 0 },
    { 0xbb4c790f, 86 },
    { 0xdafc7910, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c7914, 76 },
    { 0, 0 },
    { 0xc8ac7916, 98 },
    { 0xc07c7917, 95 },
    { 0xa78c7917, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44dc7922, 61 },
    { 0, 0 },
    { 0xe60c7924, 76 },
    { 0x524c7925, 23 },
    { 0x785c7926, 90 },
    { 0x9d9c7927, 95 },
    { 0xa20c7925, 46 },
    { 0x11ac7929, 17 },
    { 0xdfc792a, 61 },
    { 0x6d7c792b, 89 },
    { 0xdfec792c, 93 },
    { 0x6cbc792b, 98 },
    { 0xcebc792e, 23 },
    { 0xdb1c792f, 95 },
    { 0, 0 },
    { 0x3afc7931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624c7935, 90 },
    { 0, 0 },
    { 0x1d1c7937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c793b, 85 },
    { 0, 0 },
    { 0x190c793d, 78 },
    { 0xe72c793e, 98 },
    { 0xd29c793f, 76 },
    { 0x5a2c793f, 17 },
    { 0x73bc7941, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeac7949, 77 },
    { 0xcd0c7949, 61 },
    { 0xe70c794b, 43 },
    { 0xe45c794c, 95 },
    { 0, 0 },
    { 0xfebc794e, 98 },
    { 0, 0 },
    { 0x754c7950, 46 },
    { 0x8b6c7951, 34 },
    { 0x727c7952, 33 },
    { 0, 0 },
    { 0x4fac7954, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fc795b, 55 },
    { 0x1b9c795c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bec7969, 93 },
    { 0x5d4c7969, 61 },
    { 0, 0 },
    { 0xccbc796c, 23 },
    { 0x213c796d, 76 },
    { 0, 0 },
    { 0xe41c796f, 23 },
    { 0x6e1c7970, 90 },
    { 0, 0 },
    { 0xcec7972, 84 },
    { 0, 0 },
    { 0xb25c7974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c7977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983c797b, 93 },
    { 0x6bc797b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc7980, 23 },
    { 0xcecc7981, 95 },
    { 0xf10c7982, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c7986, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c798b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac798e, 58 },
    { 0x97ec798f, 33 },
    { 0, 0 },
    { 0xc9bc7991, 23 },
    { 0, 0 },
    { 0x7a8c7993, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c7998, 98 },
    { 0x2bec7999, 33 },
    { 0xfddc799a, 95 },
    { 0, 0 },
    { 0xdbc799c, 88 },
    { 0, 0 },
    { 0x718c799e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcc79a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c79aa, 33 },
    { 0xd08c79ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedac79b7, 90 },
    { 0, 0 },
    { 0x9e7c79b9, 90 },
    { 0, 0 },
    { 0xd62c79bb, 95 },
    { 0, 0 },
    { 0x660c79bd, 95 },
    { 0x142c79be, 90 },
    { 0, 0 },
    { 0x121c79c0, 84 },
    { 0, 0 },
    { 0xe90c79c2, 90 },
    { 0xb01c79c2, 33 },
    { 0x91c79c4, 90 },
    { 0x3a4c79c5, 95 },
    { 0xf1c79c6, 23 },
    { 0xf37c79c4, 61 },
    { 0, 0 },
    { 0xc64c79c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ec79cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32c79d0, 90 },
    { 0x93cc79d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec79d4, 99 },
    { 0, 0 },
    { 0x6fec79d6, 76 },
    { 0x9c9c79d6, 88 },
    { 0xa7dc79d6, 33 },
    { 0xfefc79d8, 99 },
    { 0, 0 },
    { 0xffac79db, 95 },
    { 0xb35c79dc, 61 },
    { 0x25cc79dd, 100 },
    { 0xdf8c79dd, 66 },
    { 0xcd7c79dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c79e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabc79ea, 33 },
    { 0, 0 },
    { 0x6ddc79ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cc79f1, 90 },
    { 0, 0 },
    { 0xaa0c79f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac79f7, 95 },
    { 0xebac79f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c7a00, 95 },
    { 0x1a7c7a01, 90 },
    { 0xd46c7a02, 98 },
    { 0xe3ac7a03, 98 },
    { 0xd83c7a04, 23 },
    { 0xdc9c7a02, 83 },
    { 0xa88c7a06, 87 },
    { 0xa5fc7a01, 55 },
    { 0, 0 },
    { 0xc5ac7a09, 43 },
    { 0, 0 },
    { 0x6dec7a0b, 90 },
    { 0x644c7a0b, 23 },
    { 0xa7ac7a0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c7a11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x681c7a14, 34 },
    { 0x801c7a15, 98 },
    { 0xd52c7a16, 78 },
    { 0, 0 },
    { 0x173c7a18, 23 },
    { 0x93ac7a18, 55 },
    { 0xd92c7a18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc7a1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c7a26, 93 },
    { 0x4f2c7a26, 55 },
    { 0, 0 },
    { 0x4e2c7a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x995c7a2c, 98 },
    { 0xc6dc7a2d, 43 },
    { 0xa56c7a2d, 33 },
    { 0x4fec7a2e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462c7a33, 52 },
    { 0x491c7a33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc7a3b, 90 },
    { 0, 0 },
    { 0xa6cc7a3d, 43 },
    { 0, 0 },
    { 0xcfc7a3f, 87 },
    { 0x811c7a40, 90 },
    { 0x2a7c7a41, 76 },
    { 0x93bc7a41, 90 },
    { 0xbaec7a43, 85 },
    { 0xb3cc7a43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c7a47, 61 },
    { 0xf0fc7a47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c7a4e, 46 },
    { 0x7bac7a4f, 34 },
    { 0xccfc7a4e, 55 },
    { 0x475c7a51, 99 },
    { 0x7a8c7a52, 33 },
    { 0xb02c7a53, 33 },
    { 0, 0 },
    { 0x3acc7a55, 95 },
    { 0xae9c7a56, 61 },
    { 0, 0 },
    { 0xecbc7a58, 61 },
    { 0xa91c7a59, 23 },
    { 0x20dc7a59, 95 },
    { 0xacac7a5b, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2c7a5e, 87 },
    { 0, 0 },
    { 0x56fc7a60, 90 },
    { 0x8e1c7a60, 33 },
    { 0, 0 },
    { 0xb89c7a63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c7a6b, 88 },
    { 0xc0ec7a6c, 33 },
    { 0xbbcc7a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4c7a75, 23 },
    { 0, 0 },
    { 0xd90c7a77, 90 },
    { 0xdf8c7a77, 83 },
    { 0, 0 },
    { 0xda9c7a7a, 61 },
    { 0x57bc7a7b, 98 },
    { 0xd71c7a7a, 95 },
    { 0xeebc7a7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c7a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecc7a8e, 33 },
    { 0, 0 },
    { 0x4d2c7a90, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadc7a9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c7a9e, 23 },
    { 0xae5c7a9f, 95 },
    { 0xd14c7aa0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29c7aa4, 90 },
    { 0x43cc7aa5, 61 },
    { 0xa07c7aa4, 33 },
    { 0x9fec7aa7, 52 },
    { 0xb44c7aa5, 23 },
    { 0x9a1c7aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c7ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c7abc, 64 },
    { 0x27dc7abd, 98 },
    { 0x23fc7abe, 56 },
    { 0x3b0c7abf, 91 },
    { 0x492c7ac0, 81 },
    { 0x5e7c7abe, 86 },
    { 0x4d8c7abe, 61 },
    { 0x744c7abf, 91 },
    { 0xc17c7ac2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec7ac7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c7aca, 61 },
    { 0x36ac7aca, 33 },
    { 0xf17c7acc, 23 },
    { 0xa63c7aca, 33 },
    { 0xc56c7ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c7ad1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c7ad4, 77 },
    { 0, 0 },
    { 0x791c7ad6, 67 },
    { 0, 0 },
    { 0x7d0c7ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c7adb, 99 },
    { 0x7dcc7adc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599c7ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c7ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c7aed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c7af0, 84 },
    { 0, 0 },
    { 0x672c7af2, 87 },
    { 0xa08c7af3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c7af7, 90 },
    { 0x8ccc7af7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82c7afb, 33 },
    { 0x64fc7afc, 99 },
    { 0, 0 },
    { 0x47fc7afe, 52 },
    { 0xed0c7aff, 33 },
    { 0x2ccc7aff, 33 },
    { 0x3d2c7b01, 17 },
    { 0x158c7b02, 98 },
    { 0xf7c7b02, 55 },
    { 0x408c7b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac7b08, 23 },
    { 0x7d2c7b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c7b0c, 98 },
    { 0x9acc7b0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x997c7b10, 100 },
    { 0, 0 },
    { 0x6c3c7b12, 33 },
    { 0x83c7b12, 55 },
    { 0xabcc7b14, 98 },
    { 0x96c7b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc7b1b, 61 },
    { 0x1bdc7b1c, 90 },
    { 0x3bdc7b1d, 90 },
    { 0x30c7b1e, 33 },
    { 0xabac7b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac7b22, 34 },
    { 0, 0 },
    { 0xe17c7b24, 23 },
    { 0, 0 },
    { 0xc1ac7b26, 61 },
    { 0xe82c7b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fc7b2a, 34 },
    { 0xb30c7b2b, 52 },
    { 0xfcbc7b2b, 95 },
    { 0xfb3c7b2d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c7b30, 55 },
    { 0x24cc7b31, 99 },
    { 0xa4cc7b32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c7b37, 85 },
    { 0xc0ec7b38, 76 },
    { 0x4f7c7b38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecc7b3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c7b42, 33 },
    { 0xa7cc7b42, 88 },
    { 0, 0 },
    { 0x60fc7b45, 66 },
    { 0, 0 },
    { 0x267c7b47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc7b4b, 93 },
    { 0, 0 },
    { 0x626c7b4d, 81 },
    { 0x2a7c7b4d, 90 },
    { 0, 0 },
    { 0xd1c7b50, 23 },
    { 0xcfac7b50, 66 },
    { 0x781c7b52, 97 },
    { 0xcb9c7b53, 33 },
    { 0x87fc7b52, 55 },
    { 0, 0 },
    { 0x471c7b56, 65 },
    { 0xd37c7b57, 90 },
    { 0x5aac7b58, 77 },
    { 0, 0 },
    { 0xb88c7b5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427c7b5e, 40 },
    { 0x9e3c7b5e, 93 },
    { 0, 0 },
    { 0xf36c7b61, 23 },
    { 0, 0 },
    { 0x76ec7b63, 61 },
    { 0x607c7b64, 19 },
    { 0x6a5c7b63, 33 },
    { 0, 0 },
    { 0xc8ac7b67, 61 },
    { 0x661c7b68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35c7b6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xddac7b6e, 34 },
    { 0, 0 },
    { 0xbbec7b70, 95 },
    { 0, 0 },
    { 0xea3c7b72, 76 },
    { 0, 0 },
    { 0xc4cc7b74, 98 },
    { 0x6e6c7b75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30c7b78, 46 },
    { 0x4ecc7b78, 33 },
    { 0xbd5c7b7a, 52 },
    { 0xd54c7b7b, 77 },
    { 0, 0 },
    { 0xdcdc7b7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c7b87, 55 },
    { 0x30c7b88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c7b8f, 61 },
    { 0xd34c7b90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7c7b95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689c7b9e, 76 },
    { 0, 0 },
    { 0x90c7ba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dc7ba3, 53 },
    { 0, 0 },
    { 0xd74c7ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37c7ba9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x590c7bac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c7bb0, 23 },
    { 0xb9cc7bb1, 23 },
    { 0, 0 },
    { 0xe50c7bb3, 100 },
    { 0xebec7bb4, 33 },
    { 0xfc8c7bb4, 91 },
    { 0x30fc7bb3, 55 },
    { 0xb19c7bb6, 96 },
    { 0x69fc7bb8, 95 },
    { 0, 0 },
    { 0x45c7bba, 23 },
    { 0, 0 },
    { 0x727c7bbc, 55 },
    { 0x8fdc7bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eac7bc6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c7bc9, 43 },
    { 0, 0 },
    { 0x656c7bcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3c7bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1c7bd7, 61 },
    { 0x485c7bd8, 100 },
    { 0x58bc7bd7, 76 },
    { 0xf17c7bda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ac7bea, 91 },
    { 0x67fc7beb, 84 },
    { 0xec7beb, 66 },
    { 0, 0 },
    { 0xc19c7bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9c7bf6, 66 },
    { 0x138c7bf7, 61 },
    { 0, 0 },
    { 0xd44c7bf9, 89 },
    { 0, 0 },
    { 0x4dac7bfb, 55 },
    { 0x10ec7bfc, 85 },
    { 0, 0 },
    { 0x178c7bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc7c03, 91 },
    { 0xd1ec7c04, 48 },
    { 0xe34c7c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec7c09, 61 },
    { 0x8bac7c0a, 52 },
    { 0x4edc7c09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac7c0e, 66 },
    { 0, 0 },
    { 0x51bc7c10, 61 },
    { 0, 0 },
    { 0xe47c7c12, 64 },
    { 0, 0 },
    { 0x395c7c14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869c7c1d, 88 },
    { 0xdec7c1e, 90 },
    { 0x8b5c7c1e, 55 },
    { 0x159c7c1e, 82 },
    { 0x23cc7c21, 95 },
    { 0xf9c7c21, 61 },
    { 0xd4cc7c21, 44 },
    { 0x2c0c7c24, 68 },
    { 0xff1c7c24, 61 },
    { 0x230c7c26, 55 },
    { 0, 0 },
    { 0x753c7c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c7c2b, 61 },
    { 0, 0 },
    { 0x266c7c2d, 87 },
    { 0xfc0c7c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc7c32, 52 },
    { 0, 0 },
    { 0x8cdc7c34, 95 },
    { 0x81ec7c35, 95 },
    { 0xfe5c7c36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2c7c43, 87 },
    { 0x670c7c43, 34 },
    { 0xcb1c7c43, 98 },
    { 0, 0 },
    { 0x143c7c47, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947c7c4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8c7c4f, 99 },
    { 0x21bc7c50, 61 },
    { 0x941c7c51, 98 },
    { 0xe6ac7c52, 61 },
    { 0x18fc7c53, 90 },
    { 0x928c7c53, 67 },
    { 0xd94c7c55, 90 },
    { 0, 0 },
    { 0x83fc7c57, 90 },
    { 0xc2ec7c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x971c7c5b, 95 },
    { 0, 0 },
    { 0x8b8c7c5d, 61 },
    { 0xff3c7c5e, 23 },
    { 0x830c7c5f, 91 },
    { 0x62cc7c60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fc7c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc7c73, 52 },
    { 0x1eec7c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38c7c79, 90 },
    { 0xb49c7c7a, 98 },
    { 0x45ac7c7a, 100 },
    { 0x22ac7c7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc7c7f, 91 },
    { 0, 0 },
    { 0xbb1c7c81, 90 },
    { 0x9dac7c82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fc7c89, 66 },
    { 0xd4c7c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dc7c8e, 55 },
    { 0, 0 },
    { 0xef8c7c90, 55 },
    { 0x909c7c91, 98 },
    { 0xebcc7c91, 85 },
    { 0xab6c7c93, 76 },
    { 0x1eec7c94, 23 },
    { 0x149c7c91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cc7c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585c7ca4, 84 },
    { 0, 0 },
    { 0xd54c7ca6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c7cac, 93 },
    { 0xf25c7cad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c7cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02c7cb8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ac7cc0, 55 },
    { 0x2d0c7cc1, 67 },
    { 0x1b7c7cc2, 95 },
    { 0x5d2c7cc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc7cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508c7ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eac7cd2, 33 },
    { 0x82dc7cd3, 61 },
    { 0, 0 },
    { 0xc1c7cd5, 46 },
    { 0x2fac7cd6, 55 },
    { 0xbb7c7cd5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bc7ce2, 85 },
    { 0x4b3c7ce3, 66 },
    { 0x4dcc7ce4, 99 },
    { 0x917c7ce3, 90 },
    { 0x7fcc7ce6, 68 },
    { 0x4a2c7ce3, 55 },
    { 0xdd6c7ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8c7ceb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9c7cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bc7cf7, 23 },
    { 0, 0 },
    { 0xf45c7cf9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c7cfd, 33 },
    { 0xbdac7cfd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12c7d06, 46 },
    { 0xf46c7d06, 23 },
    { 0, 0 },
    { 0x847c7d09, 33 },
    { 0x18cc7d0a, 46 },
    { 0x3aac7d0a, 76 },
    { 0x863c7d0c, 53 },
    { 0xcc8c7d0d, 33 },
    { 0xa41c7d0c, 90 },
    { 0xcffc7d0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fec7d12, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc7d16, 61 },
    { 0xd1fc7d17, 61 },
    { 0x19cc7d16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c7d1d, 61 },
    { 0xd47c7d1e, 98 },
    { 0, 0 },
    { 0xe6ec7d20, 90 },
    { 0x692c7d20, 66 },
    { 0xa3ac7d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c7d25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9c7d28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9c7d32, 98 },
    { 0x569c7d32, 67 },
    { 0xc6ac7d34, 34 },
    { 0xf52c7d33, 85 },
    { 0x92fc7d36, 78 },
    { 0x835c7d37, 33 },
    { 0x525c7d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c7d3b, 23 },
    { 0x7d2c7d3b, 45 },
    { 0x39dc7d3b, 95 },
    { 0, 0 },
    { 0x1d9c7d3f, 23 },
    { 0xa4ac7d40, 23 },
    { 0x2d8c7d41, 61 },
    { 0xd29c7d42, 61 },
    { 0xe54c7d43, 23 },
    { 0xb3bc7d43, 90 },
    { 0xcbac7d44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c7d4f, 84 },
    { 0x1ddc7d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c7d56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x729c7d59, 33 },
    { 0, 0 },
    { 0x887c7d5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d60, 33 },
    { 0, 0 },
    { 0x579c7d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c7d66, 33 },
    { 0x54ac7d66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2c7d6a, 88 },
    { 0xff4c7d6b, 33 },
    { 0, 0 },
    { 0x89dc7d6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fc7d72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x908c7d75, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c7d78, 61 },
    { 0x83bc7d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5c7d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779c7d80, 100 },
    { 0x432c7d81, 61 },
    { 0x26dc7d80, 98 },
    { 0x7bc7d81, 90 },
    { 0x577c7d84, 33 },
    { 0xd9bc7d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc7d8d, 55 },
    { 0x9a3c7d8e, 67 },
    { 0xe10c7d8f, 93 },
    { 0x622c7d90, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbc7d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c7d97, 85 },
    { 0x97c7d98, 55 },
    { 0xfb2c7d99, 43 },
    { 0x83ec7d99, 99 },
    { 0xf2bc7d99, 88 },
    { 0x197c7d9c, 76 },
    { 0x394c7d9d, 98 },
    { 0x632c7d9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dc7da2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432c7da9, 23 },
    { 0xb5dc7daa, 52 },
    { 0, 0 },
    { 0xe34c7dac, 61 },
    { 0xf80c7dad, 37 },
    { 0x385c7dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c7db3, 76 },
    { 0x31c7db3, 60 },
    { 0xc35c7db5, 84 },
    { 0xb8dc7db6, 91 },
    { 0xabcc7db3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ac7dc0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c7dc5, 76 },
    { 0x3b2c7dc6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ec7dcd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1c7dd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45c7dd6, 55 },
    { 0x6c5c7dd6, 98 },
    { 0x2b2c7dd8, 46 },
    { 0x89bc7dd9, 98 },
    { 0xd1bc7dd6, 68 },
    { 0x587c7dd8, 91 },
    { 0, 0 },
    { 0x512c7ddd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x810c7de0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac7de4, 95 },
    { 0x2cdc7de5, 61 },
    { 0, 0 },
    { 0x2cbc7de7, 77 },
    { 0, 0 },
    { 0xcc3c7de9, 85 },
    { 0xdfec7dea, 55 },
    { 0x215c7deb, 34 },
    { 0xb50c7dec, 98 },
    { 0xf56c7dec, 100 },
    { 0xdf2c7dee, 43 },
    { 0xaf2c7def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c7df2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dc7df6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc7dfc, 68 },
    { 0x392c7dfd, 93 },
    { 0, 0 },
    { 0x1e2c7dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72c7e03, 91 },
    { 0x89fc7e03, 95 },
    { 0x3bc7e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763c7e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ec7e0d, 61 },
    { 0, 0 },
    { 0x20cc7e0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7e13, 83 },
    { 0, 0 },
    { 0xdddc7e15, 61 },
    { 0, 0 },
    { 0x43fc7e17, 23 },
    { 0x25ec7e18, 55 },
    { 0, 0 },
    { 0x7b5c7e1a, 91 },
    { 0x666c7e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c7e20, 76 },
    { 0x2dac7e21, 77 },
    { 0x374c7e21, 78 },
    { 0x97fc7e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbc7e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c7e32, 98 },
    { 0x535c7e33, 55 },
    { 0xbfc7e34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ec7e38, 95 },
    { 0xb8fc7e39, 52 },
    { 0, 0 },
    { 0x155c7e3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc7e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c7e42, 67 },
    { 0, 0 },
    { 0xe87c7e44, 76 },
    { 0xa5fc7e44, 95 },
    { 0x400c7e45, 33 },
    { 0xf4dc7e47, 65 },
    { 0x878c7e46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05c7e59, 98 },
    { 0x6bac7e5a, 95 },
    { 0xd38c7e5a, 66 },
    { 0x699c7e5c, 98 },
    { 0, 0 },
    { 0x116c7e5e, 90 },
    { 0x463c7e5f, 95 },
    { 0, 0 },
    { 0x8bcc7e61, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33c7e66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cec7e69, 33 },
    { 0, 0 },
    { 0xa3bc7e6b, 34 },
    { 0, 0 },
    { 0xc6dc7e6d, 76 },
    { 0, 0 },
    { 0xc7ac7e6f, 23 },
    { 0xa17c7e70, 46 },
    { 0, 0 },
    { 0x95cc7e72, 95 },
    { 0x991c7e73, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88c7e77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94c7e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c7e85, 61 },
    { 0, 0 },
    { 0x6f7c7e87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c7e8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ec7e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2c7e97, 91 },
    { 0xfe9c7e98, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x578c7e9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3c7ea6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c7eaa, 98 },
    { 0xa17c7eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2c7eb1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc7eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fc7eb9, 55 },
    { 0x69c7eba, 98 },
    { 0x643c7ebb, 98 },
    { 0xad8c7ebc, 98 },
    { 0x843c7ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c7ec0, 17 },
    { 0, 0 },
    { 0xa25c7ec2, 46 },
    { 0xc63c7ec2, 88 },
    { 0xa2ec7ec2, 99 },
    { 0x9dfc7ec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c7ec9, 84 },
    { 0, 0 },
    { 0xba5c7ecb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bac7ece, 95 },
    { 0, 0 },
    { 0xb5fc7ed0, 39 },
    { 0x296c7ed0, 93 },
    { 0xad7c7ed1, 90 },
    { 0x32c7ed3, 67 },
    { 0, 0 },
    { 0xb8ec7ed5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408c7edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c7ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c7eea, 61 },
    { 0, 0 },
    { 0x90ec7eec, 90 },
    { 0xa0ac7eed, 93 },
    { 0xd68c7eed, 43 },
    { 0xf29c7eef, 90 },
    { 0x345c7eed, 99 },
    { 0xa6cc7eee, 98 },
    { 0x1f4c7eef, 95 },
    { 0x6a3c7ef3, 80 },
    { 0x796c7ef4, 66 },
    { 0xec1c7ef5, 33 },
    { 0, 0 },
    { 0x803c7ef7, 33 },
    { 0x57fc7ef7, 23 },
    { 0xaf2c7ef9, 99 },
    { 0x519c7efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccc7efe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c7f03, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5c7f07, 23 },
    { 0x210c7f08, 33 },
    { 0, 0 },
    { 0x25fc7f0a, 99 },
    { 0x196c7f0b, 95 },
    { 0x9ddc7f0b, 90 },
    { 0xd2fc7f0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c7f10, 59 },
    { 0xc04c7f11, 61 },
    { 0x91c7f10, 61 },
    { 0x13ac7f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dc7f17, 98 },
    { 0x21ac7f18, 95 },
    { 0, 0 },
    { 0x590c7f1a, 98 },
    { 0x26c7f1b, 90 },
    { 0, 0 },
    { 0x9bc7f1d, 93 },
    { 0x2b8c7f1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c7f21, 61 },
    { 0xaa0c7f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3c7f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393c7f2b, 95 },
    { 0x119c7f2b, 69 },
    { 0xe54c7f2d, 98 },
    { 0x423c7f2b, 95 },
    { 0x6c4c7f2f, 61 },
    { 0x297c7f30, 98 },
    { 0xe99c7f30, 43 },
    { 0x8bec7f32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7f3c, 99 },
    { 0, 0 },
    { 0xaacc7f3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c7f46, 90 },
    { 0xec7c7f46, 52 },
    { 0xff8c7f48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bc7f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6c7f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c7f59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x536c7f5c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c7f5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1c7f64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c7f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c7f72, 33 },
    { 0, 0 },
    { 0xf34c7f74, 90 },
    { 0xa1ac7f75, 23 },
    { 0xd1ac7f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c7f7c, 90 },
    { 0, 0 },
    { 0x38ac7f7e, 81 },
    { 0, 0 },
    { 0x590c7f80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c7f8a, 93 },
    { 0xc9c7f8b, 98 },
    { 0xae7c7f8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c7f94, 87 },
    { 0x64ac7f95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc7f9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ec7f9e, 33 },
    { 0x656c7f9e, 23 },
    { 0x2f3c7fa0, 23 },
    { 0xb48c7f9f, 52 },
    { 0xa24c7fa2, 87 },
    { 0xd65c7fa2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bc7fa6, 83 },
    { 0xcd2c7fa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c7fab, 33 },
    { 0x817c7fab, 98 },
    { 0x30dc7fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7c7fb0, 93 },
    { 0xdfcc7fb1, 90 },
    { 0xa54c7fb2, 55 },
    { 0x488c7fb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c7fb7, 33 },
    { 0x748c7fb8, 98 },
    { 0, 0 },
    { 0x1e5c7fba, 34 },
    { 0, 0 },
    { 0xa85c7fbc, 98 },
    { 0x892c7fbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cec7fc0, 61 },
    { 0, 0 },
    { 0x876c7fc2, 87 },
    { 0x10dc7fc3, 90 },
    { 0x64ac7fc4, 61 },
    { 0xee1c7fc5, 90 },
    { 0x71ac7fc6, 46 },
    { 0x6f0c7fc5, 90 },
    { 0, 0 },
    { 0xd68c7fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c7fcc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc7fd0, 90 },
    { 0xb59c7fd0, 66 },
    { 0, 0 },
    { 0x412c7fd3, 67 },
    { 0, 0 },
    { 0x353c7fd5, 33 },
    { 0x935c7fd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbac7fdf, 61 },
    { 0, 0 },
    { 0x5ecc7fe1, 38 },
    { 0x504c7fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac7fea, 76 },
    { 0, 0 },
    { 0x9d5c7fec, 85 },
    { 0, 0 },
    { 0xc82c7fee, 17 },
    { 0xac7fef, 98 },
    { 0x5afc7fef, 98 },
    { 0xba3c7ff1, 34 },
    { 0x789c7ff2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27c7ff8, 98 },
    { 0, 0 },
    { 0xa75c7ffa, 78 },
    { 0x496c7ffb, 83 },
    { 0x433c7ffb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc8005, 85 },
    { 0x1b6c8006, 68 },
    { 0x818c8006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c800a, 33 },
    { 0, 0 },
    { 0x39bc800c, 95 },
    { 0, 0 },
    { 0xaf7c800e, 95 },
    { 0, 0 },
    { 0x7bec8010, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abc8016, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c801d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c8026, 34 },
    { 0xb72c8026, 95 },
    { 0x67dc8028, 33 },
    { 0, 0 },
    { 0x5fcc802a, 97 },
    { 0, 0 },
    { 0x17cc802c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c8033, 52 },
    { 0, 0 },
    { 0x469c8035, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec8038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cc803c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x414c803f, 66 },
    { 0xb1ac8040, 33 },
    { 0xbd6c8041, 90 },
    { 0x6d0c8040, 67 },
    { 0x22fc8043, 52 },
    { 0x152c803f, 95 },
    { 0x4b5c803f, 61 },
    { 0x932c8046, 95 },
    { 0x13ac8047, 76 },
    { 0x2a7c8044, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eec804b, 61 },
    { 0x8bdc804b, 61 },
    { 0x1b3c804d, 23 },
    { 0x6a7c804e, 43 },
    { 0xbc7c804f, 33 },
    { 0x60dc8050, 90 },
    { 0xac9c8050, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4c8054, 91 },
    { 0x10bc8054, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c8060, 33 },
    { 0x759c8060, 78 },
    { 0xb49c8062, 90 },
    { 0xe63c8061, 90 },
    { 0, 0 },
    { 0xe42c8065, 52 },
    { 0x37c8066, 58 },
    { 0xe64c8066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc806c, 33 },
    { 0xe38c806c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c8075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566c807d, 33 },
    { 0, 0 },
    { 0x34ac807f, 66 },
    { 0, 0 },
    { 0xfddc8081, 61 },
    { 0x92cc8081, 98 },
    { 0xac8083, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebec808f, 95 },
    { 0xbe1c8090, 77 },
    { 0x9cdc8091, 91 },
    { 0xb0fc8092, 95 },
    { 0xf0fc808f, 85 },
    { 0x83cc808f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0c8098, 95 },
    { 0, 0 },
    { 0x6e4c809a, 101 },
    { 0x921c809b, 81 },
    { 0x7abc809c, 53 },
    { 0, 0 },
    { 0x937c809e, 12 },
    { 0, 0 },
    { 0xa86c80a0, 76 },
    { 0x2acc80a1, 43 },
    { 0xb29c80a1, 23 },
    { 0xcefc80a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec80a6, 95 },
    { 0, 0 },
    { 0xec8c80a8, 91 },
    { 0x403c80a9, 75 },
    { 0, 0 },
    { 0x5c5c80ab, 98 },
    { 0x4b9c80ac, 43 },
    { 0x553c80ab, 98 },
    { 0x492c80ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c80b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec80b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c80c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c80ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653c80cf, 99 },
    { 0xaabc80d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec80d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86c80db, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90c80df, 61 },
    { 0xc49c80df, 33 },
    { 0x6ddc80e1, 100 },
    { 0xa92c80e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc80f1, 33 },
    { 0x5a2c80f2, 99 },
    { 0, 0 },
    { 0xfeac80f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34c80f8, 90 },
    { 0x8b5c80f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cc80fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefc80ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c8107, 66 },
    { 0, 0 },
    { 0x8dac8109, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c810f, 52 },
    { 0x600c810f, 76 },
    { 0x250c810f, 98 },
    { 0xe08c8112, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c811a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ec8125, 90 },
    { 0, 0 },
    { 0x493c8127, 95 },
    { 0xfdac8128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5c812b, 61 },
    { 0xb22c812c, 95 },
    { 0x5fcc812c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7c8135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9c813a, 85 },
    { 0x853c813b, 43 },
    { 0x749c813b, 36 },
    { 0x7c5c813b, 93 },
    { 0, 0 },
    { 0x296c813f, 93 },
    { 0, 0 },
    { 0x8ddc8141, 98 },
    { 0, 0 },
    { 0xad0c8143, 90 },
    { 0xdccc8144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0c8149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc814e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c8151, 91 },
    { 0x963c8152, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc8155, 55 },
    { 0x7dbc8155, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c815c, 55 },
    { 0xe80c815d, 92 },
    { 0x4dac815e, 77 },
    { 0x7b4c815f, 76 },
    { 0x9dcc8160, 55 },
    { 0xedbc8160, 23 },
    { 0x286c815f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdc816d, 61 },
    { 0xbf7c816e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8c8172, 61 },
    { 0x1b4c8173, 81 },
    { 0x52ec8173, 76 },
    { 0, 0 },
    { 0x33dc8176, 76 },
    { 0, 0 },
    { 0x779c8178, 98 },
    { 0xc1ec8178, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c817d, 87 },
    { 0xfedc817d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2c8183, 34 },
    { 0xb18c8183, 55 },
    { 0xf5fc8183, 55 },
    { 0xa6dc8186, 78 },
    { 0xc4dc8186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc818a, 33 },
    { 0, 0 },
    { 0x4c0c818c, 43 },
    { 0xf1bc818d, 43 },
    { 0xf52c818d, 43 },
    { 0x5ffc818c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c8192, 78 },
    { 0xf6ac8193, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3c819f, 61 },
    { 0x321c81a0, 55 },
    { 0xefac81a1, 90 },
    { 0x411c81a2, 43 },
    { 0xa3ec81a2, 23 },
    { 0xce1c81a4, 90 },
    { 0x7ddc81a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc81ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc81b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabc81b6, 96 },
    { 0, 0 },
    { 0x71dc81b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88c81bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec81c1, 95 },
    { 0, 0 },
    { 0x2c8c81c3, 98 },
    { 0x504c81c4, 33 },
    { 0x5b0c81c5, 90 },
    { 0, 0 },
    { 0x5f0c81c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bec81cb, 23 },
    { 0x209c81cb, 43 },
    { 0xc69c81cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7c81d1, 61 },
    { 0xc2ec81d2, 25 },
    { 0x316c81d3, 83 },
    { 0x37dc81d4, 98 },
    { 0, 0 },
    { 0x391c81d6, 23 },
    { 0x964c81d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc81df, 85 },
    { 0, 0 },
    { 0x159c81e1, 65 },
    { 0xe29c81e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbc81ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec81ef, 76 },
    { 0, 0 },
    { 0x1e5c81f1, 90 },
    { 0xa59c81f2, 61 },
    { 0x4edc81f1, 61 },
    { 0x852c81f4, 23 },
    { 0xf8bc81f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec8200, 97 },
    { 0x236c8201, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x866c8204, 98 },
    { 0x3fac8204, 55 },
    { 0x4cfc8206, 90 },
    { 0x545c8206, 76 },
    { 0, 0 },
    { 0x72cc8209, 90 },
    { 0, 0 },
    { 0x701c820b, 98 },
    { 0, 0 },
    { 0x80fc820d, 61 },
    { 0xd4dc820d, 93 },
    { 0x7c820f, 61 },
    { 0x10c820f, 93 },
    { 0, 0 },
    { 0x278c8212, 61 },
    { 0, 0 },
    { 0xea6c8214, 90 },
    { 0x109c8215, 66 },
    { 0x51c8214, 55 },
    { 0xf24c8215, 93 },
    { 0x969c8218, 67 },
    { 0x78ec8219, 80 },
    { 0xccec8218, 61 },
    { 0x364c821b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ac8220, 78 },
    { 0x5aec8221, 95 },
    { 0, 0 },
    { 0xec0c8223, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ec8228, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc822f, 68 },
    { 0, 0 },
    { 0x7a9c8231, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6c8237, 55 },
    { 0xe3bc8238, 90 },
    { 0x937c8239, 61 },
    { 0, 0 },
    { 0x45ec823b, 17 },
    { 0xbd1c823c, 95 },
    { 0, 0 },
    { 0xeeec823e, 61 },
    { 0x315c823f, 34 },
    { 0x577c8240, 61 },
    { 0x4e7c8241, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c8246, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9c824c, 76 },
    { 0, 0 },
    { 0x541c824e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8c8252, 76 },
    { 0x5f3c8253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec8257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24c825b, 78 },
    { 0, 0 },
    { 0x9c0c825d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2c8267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bc826f, 100 },
    { 0x93ac8270, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bc8275, 33 },
    { 0x64dc8275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85c827e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5c8283, 95 },
    { 0x8d7c8284, 90 },
    { 0x570c8285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7c8289, 76 },
    { 0xf38c828a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac828d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcc8290, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c8293, 61 },
    { 0x413c8294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c8297, 77 },
    { 0x2bbc8298, 33 },
    { 0xc10c8297, 55 },
    { 0x46ec829a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c82a6, 84 },
    { 0, 0 },
    { 0xd6fc82a8, 46 },
    { 0x6d2c82a9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c82af, 69 },
    { 0x200c82af, 90 },
    { 0x6c1c82b1, 61 },
    { 0x7cac82b1, 46 },
    { 0x66dc82b3, 84 },
    { 0xc28c82b4, 23 },
    { 0x6a4c82b4, 43 },
    { 0x8a9c82b6, 61 },
    { 0, 0 },
    { 0x13c82b8, 33 },
    { 0, 0 },
    { 0x2cec82ba, 23 },
    { 0, 0 },
    { 0xa44c82bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc82c0, 61 },
    { 0x822c82c0, 94 },
    { 0xf0bc82c2, 94 },
    { 0xbecc82c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c82c7, 31 },
    { 0xc65c82c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac82cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c82d3, 55 },
    { 0x348c82d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc82d8, 33 },
    { 0xeb2c82d8, 67 },
    { 0, 0 },
    { 0x74cc82db, 46 },
    { 0, 0 },
    { 0x573c82dd, 91 },
    { 0x5fbc82de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0c82e1, 90 },
    { 0xfbcc82e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c82e9, 52 },
    { 0xb3fc82ea, 76 },
    { 0x889c82ea, 66 },
    { 0xc90c82ea, 76 },
    { 0, 0 },
    { 0x39ec82ee, 17 },
    { 0xaa0c82ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ac82f2, 61 },
    { 0xfbcc82f3, 80 },
    { 0xf4cc82f3, 34 },
    { 0x108c82f5, 46 },
    { 0, 0 },
    { 0x3bc82f7, 42 },
    { 0xd07c82f8, 90 },
    { 0x1a5c82f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x779c82fc, 55 },
    { 0, 0 },
    { 0x80bc82fe, 61 },
    { 0xb35c82ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x401c8302, 98 },
    { 0x9a3c8303, 90 },
    { 0x5c4c8302, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4c8308, 61 },
    { 0xb12c8309, 100 },
    { 0, 0 },
    { 0xd68c830b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc8313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bc8319, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc831d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c8322, 34 },
    { 0xc1c8323, 23 },
    { 0xbb7c8322, 56 },
    { 0x912c8324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c8328, 95 },
    { 0xb6ec8329, 77 },
    { 0x39dc8329, 61 },
    { 0x26bc832b, 34 },
    { 0xcebc832c, 98 },
    { 0xf4bc832d, 90 },
    { 0, 0 },
    { 0x55dc832f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fc8333, 55 },
    { 0, 0 },
    { 0x396c8335, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0c8338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cc833c, 90 },
    { 0xe74c833c, 76 },
    { 0xa63c833e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771c8344, 98 },
    { 0xe84c8344, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c8349, 95 },
    { 0, 0 },
    { 0xfccc834b, 46 },
    { 0x37cc834c, 53 },
    { 0xe1ac834d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514c8351, 95 },
    { 0xb3ec8352, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c8358, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7c835e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c8363, 61 },
    { 0xdf7c8364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac836c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2c8375, 95 },
    { 0xc8ac8376, 33 },
    { 0xa66c8377, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c837b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bc837f, 98 },
    { 0x8c7c8380, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12c8386, 100 },
    { 0, 0 },
    { 0xfa9c8388, 33 },
    { 0, 0 },
    { 0xfd8c838a, 98 },
    { 0xceac838a, 98 },
    { 0, 0 },
    { 0xda5c838d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56c8392, 99 },
    { 0x264c8392, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcc8396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11c839b, 95 },
    { 0xa81c839c, 28 },
    { 0xfaec839c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ac83a4, 23 },
    { 0x2d0c83a4, 90 },
    { 0x722c83a6, 46 },
    { 0x2bec83a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c83ac, 94 },
    { 0x88bc83ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc83b5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c83ba, 90 },
    { 0xf8cc83bb, 91 },
    { 0xe10c83bb, 90 },
    { 0, 0 },
    { 0xdc2c83be, 33 },
    { 0xe30c83bf, 88 },
    { 0, 0 },
    { 0xcd8c83c1, 91 },
    { 0, 0 },
    { 0x11ec83c3, 43 },
    { 0x36fc83c4, 52 },
    { 0xe2fc83c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c83cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abc83d1, 100 },
    { 0xcccc83d1, 33 },
    { 0xc2ac83d1, 95 },
    { 0x6f1c83d4, 23 },
    { 0xdb1c83d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c83de, 76 },
    { 0, 0 },
    { 0x73ec83e0, 85 },
    { 0x856c83e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc83e6, 51 },
    { 0xa29c83e7, 45 },
    { 0x7edc83e8, 66 },
    { 0xfd7c83e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbc83f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec83f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaac83fa, 81 },
    { 0x1dbc83fb, 23 },
    { 0, 0 },
    { 0x76dc83fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c8400, 33 },
    { 0x7c3c8401, 34 },
    { 0, 0 },
    { 0x8c3c8403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847c8408, 95 },
    { 0x1b1c8409, 46 },
    { 0x2ffc840a, 23 },
    { 0xb51c840b, 65 },
    { 0x630c840b, 23 },
    { 0, 0 },
    { 0xd3dc840e, 98 },
    { 0, 0 },
    { 0x95bc8410, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c8415, 90 },
    { 0, 0 },
    { 0x67c8417, 33 },
    { 0xb5bc8418, 98 },
    { 0x771c8419, 98 },
    { 0x6dec841a, 78 },
    { 0x1bc841b, 95 },
    { 0x1d1c841a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5c8421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c8425, 93 },
    { 0xfdfc8425, 85 },
    { 0xd01c8427, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7c842b, 98 },
    { 0, 0 },
    { 0xdcc842d, 55 },
    { 0x7ebc842e, 33 },
    { 0xcbac842f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ec8433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751c8437, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6c843a, 61 },
    { 0, 0 },
    { 0x544c843c, 53 },
    { 0x8afc843d, 75 },
    { 0x8b1c843d, 33 },
    { 0xd92c843f, 17 },
    { 0, 0 },
    { 0xbd6c8441, 66 },
    { 0x11c8442, 40 },
    { 0, 0 },
    { 0x8edc8444, 33 },
    { 0xcdcc8445, 75 },
    { 0, 0 },
    { 0xea3c8447, 43 },
    { 0xfc9c8448, 65 },
    { 0xc9dc8449, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c8452, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc8457, 23 },
    { 0, 0 },
    { 0xf4cc8459, 90 },
    { 0x35ec845a, 80 },
    { 0x604c845b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392c845f, 90 },
    { 0xd16c8460, 98 },
    { 0x21ac8460, 87 },
    { 0x3aec845f, 95 },
    { 0xeac8463, 100 },
    { 0x4e9c8464, 55 },
    { 0, 0 },
    { 0x80fc8466, 33 },
    { 0, 0 },
    { 0xd21c8468, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566c846c, 23 },
    { 0x8bdc846c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc8478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc8480, 93 },
    { 0, 0 },
    { 0x850c8482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28c8486, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c8489, 66 },
    { 0, 0 },
    { 0xbdfc848b, 33 },
    { 0xdccc848b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc848f, 52 },
    { 0x686c848f, 95 },
    { 0, 0 },
    { 0xcf5c8492, 81 },
    { 0x712c8492, 99 },
    { 0xdf4c8494, 90 },
    { 0xb62c8495, 95 },
    { 0xf1c8492, 23 },
    { 0x30bc8497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456c849d, 87 },
    { 0x7eac849e, 99 },
    { 0x367c849f, 87 },
    { 0, 0 },
    { 0x1d7c84a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec84a4, 55 },
    { 0xeb7c84a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9c84a9, 93 },
    { 0, 0 },
    { 0xc9bc84ab, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bc84ae, 81 },
    { 0x343c84ae, 95 },
    { 0, 0 },
    { 0x13ac84b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417c84b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x985c84b9, 84 },
    { 0, 0 },
    { 0x3ecc84bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc84c0, 23 },
    { 0x3a4c84c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc84c4, 95 },
    { 0x76bc84c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79c84ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c84ce, 34 },
    { 0x375c84cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6c84d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3c84d7, 99 },
    { 0x6bec84d8, 33 },
    { 0xe22c84d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bc84dc, 90 },
    { 0xcaec84dd, 38 },
    { 0x37dc84dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c84e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302c84e9, 76 },
    { 0x4c84ea, 100 },
    { 0, 0 },
    { 0xf6fc84ec, 91 },
    { 0xff3c84ec, 61 },
    { 0x751c84ed, 90 },
    { 0, 0 },
    { 0x705c84f0, 95 },
    { 0x23bc84f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3c84f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc84ff, 85 },
    { 0xa75c8500, 90 },
    { 0xacac84ff, 95 },
    { 0xddac8502, 95 },
    { 0xe12c8503, 33 },
    { 0x4b9c8504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aec8507, 46 },
    { 0x5e5c8507, 75 },
    { 0x7eec8509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c850c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aec8511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2c8518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5c851c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589c8520, 90 },
    { 0xc60c8521, 98 },
    { 0xc77c8520, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x379c8525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925c852a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c852d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c8530, 90 },
    { 0x877c8531, 90 },
    { 0, 0 },
    { 0xa25c8533, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c8539, 90 },
    { 0x35bc853a, 90 },
    { 0xe72c853b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485c853f, 61 },
    { 0xcf1c8540, 65 },
    { 0x2e8c8541, 33 },
    { 0x8d1c8541, 61 },
    { 0x103c8542, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968c8548, 33 },
    { 0x1e6c8549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcc854c, 48 },
    { 0, 0 },
    { 0x60dc854e, 90 },
    { 0x55fc854f, 98 },
    { 0xcd5c8550, 55 },
    { 0x16ac8551, 61 },
    { 0x74c8552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94c855b, 23 },
    { 0x695c855c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ec855f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65c8569, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c8573, 43 },
    { 0x2edc8574, 98 },
    { 0xaacc8575, 84 },
    { 0, 0 },
    { 0x12dc8577, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786c857e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfc8584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91c858c, 33 },
    { 0x5e7c858d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c8593, 33 },
    { 0xbbfc8593, 90 },
    { 0x5bfc8595, 34 },
    { 0, 0 },
    { 0x486c8597, 76 },
    { 0xd0ac8598, 34 },
    { 0x1f1c8599, 99 },
    { 0, 0 },
    { 0x120c859b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bc859e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fec85a4, 61 },
    { 0xbabc85a4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25c85aa, 23 },
    { 0xc1ec85ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc85ae, 76 },
    { 0, 0 },
    { 0xbc85b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc85b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcc85bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228c85c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c85c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc85ce, 76 },
    { 0x4afc85ce, 98 },
    { 0, 0 },
    { 0x109c85d1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aec85d5, 23 },
    { 0x6b2c85d6, 84 },
    { 0, 0 },
    { 0xebbc85d8, 34 },
    { 0x54cc85d9, 33 },
    { 0, 0 },
    { 0x18c85db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68c85de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9c85e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50c85e7, 48 },
    { 0x149c85e7, 95 },
    { 0xed5c85e9, 46 },
    { 0xc29c85ea, 95 },
    { 0xa22c85ea, 100 },
    { 0x91fc85e8, 34 },
    { 0x518c85ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc85f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c85f4, 99 },
    { 0xdb5c85f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63c85fc, 98 },
    { 0x806c85fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bc8603, 95 },
    { 0xc34c8603, 95 },
    { 0, 0 },
    { 0x3e8c8606, 33 },
    { 0x2abc8607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775c860d, 46 },
    { 0x6f3c860e, 33 },
    { 0x811c860d, 55 },
    { 0xdbdc8610, 95 },
    { 0x473c8611, 76 },
    { 0, 0 },
    { 0xc14c8613, 90 },
    { 0, 0 },
    { 0xd56c8615, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c8618, 43 },
    { 0x68ac8619, 90 },
    { 0, 0 },
    { 0xcc6c861b, 98 },
    { 0xa37c861b, 23 },
    { 0x8f0c861d, 95 },
    { 0xe40c861b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757c8625, 66 },
    { 0x379c8626, 90 },
    { 0, 0 },
    { 0x98ac8628, 33 },
    { 0xed2c8628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c862f, 98 },
    { 0, 0 },
    { 0xccdc8631, 43 },
    { 0xa2c8631, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcc8637, 95 },
    { 0xdb7c8637, 76 },
    { 0x567c8639, 43 },
    { 0x153c8637, 23 },
    { 0x2dc8638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440c8643, 61 },
    { 0x8ecc8644, 80 },
    { 0x8ecc8645, 90 },
    { 0x90bc8644, 34 },
    { 0x4c5c8646, 91 },
    { 0, 0 },
    { 0xf6fc8649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ac864e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c8653, 76 },
    { 0x57dc8654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c8658, 61 },
    { 0xc98c8658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4c865f, 90 },
    { 0, 0 },
    { 0x422c8661, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fc866d, 46 },
    { 0, 0 },
    { 0xd57c866f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc8674, 90 },
    { 0x3eac8675, 34 },
    { 0, 0 },
    { 0xfcbc8677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c867a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c867f, 23 },
    { 0xca8c8680, 99 },
    { 0xf0ec8680, 33 },
    { 0, 0 },
    { 0x56cc8683, 61 },
    { 0, 0 },
    { 0x88c8685, 33 },
    { 0xfe8c8686, 98 },
    { 0x8bfc8687, 87 },
    { 0xc5bc8685, 33 },
    { 0xacfc8689, 33 },
    { 0xa61c8687, 38 },
    { 0, 0 },
    { 0x106c868c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4c8691, 93 },
    { 0xef1c8692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fc8696, 90 },
    { 0x482c8697, 46 },
    { 0, 0 },
    { 0x5fcc8699, 34 },
    { 0, 0 },
    { 0xaffc869b, 77 },
    { 0, 0 },
    { 0x484c869d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bc86a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec86a3, 76 },
    { 0x33bc86a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c86a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc86b5, 75 },
    { 0xcf2c86b5, 55 },
    { 0, 0 },
    { 0xd6cc86b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c86c2, 95 },
    { 0xb1fc86c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dc86c6, 61 },
    { 0x318c86c7, 93 },
    { 0x812c86c8, 23 },
    { 0, 0 },
    { 0x647c86ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc86cd, 78 },
    { 0x3aec86ce, 88 },
    { 0, 0 },
    { 0x293c86d0, 43 },
    { 0xd9c86d1, 61 },
    { 0x7a6c86d1, 76 },
    { 0x278c86d1, 33 },
    { 0, 0 },
    { 0x5a2c86d5, 43 },
    { 0xcb9c86d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c86db, 52 },
    { 0x56fc86db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30c86df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x858c86e2, 90 },
    { 0x3ecc86e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c86e8, 85 },
    { 0xa65c86e8, 61 },
    { 0x836c86ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0c86ef, 43 },
    { 0xd6dc86f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c86f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c86f8, 98 },
    { 0x6b3c86f9, 33 },
    { 0, 0 },
    { 0xebc86fb, 33 },
    { 0x5a9c86fc, 78 },
    { 0xca9c86fb, 85 },
    { 0x215c86fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8c8704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c8708, 23 },
    { 0x56ec8709, 55 },
    { 0x424c8709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc8710, 23 },
    { 0x865c8711, 68 },
    { 0xcfbc8711, 92 },
    { 0x2c0c8713, 95 },
    { 0xd45c8714, 55 },
    { 0xdec8715, 33 },
    { 0, 0 },
    { 0xd82c8717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c871b, 53 },
    { 0xf99c871c, 85 },
    { 0x7bbc871c, 34 },
    { 0xf96c871c, 61 },
    { 0, 0 },
    { 0xfe3c8720, 17 },
    { 0x80dc8720, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7c8725, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcec872f, 78 },
    { 0xb6dc8730, 98 },
    { 0x41fc8731, 75 },
    { 0, 0 },
    { 0x363c8733, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bc8738, 23 },
    { 0xef0c8739, 33 },
    { 0xae1c8738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375c873e, 76 },
    { 0, 0 },
    { 0xd2bc8740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11c8747, 33 },
    { 0x2e6c8748, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc874b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c8755, 90 },
    { 0, 0 },
    { 0x86ac8757, 23 },
    { 0x5e7c8758, 46 },
    { 0x4b3c8757, 91 },
    { 0x276c8757, 43 },
    { 0xb5bc875b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc875e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733c8761, 87 },
    { 0xc64c8762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bc8765, 90 },
    { 0xd54c8766, 98 },
    { 0xa38c8766, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ec8775, 33 },
    { 0x71bc8775, 46 },
    { 0x2afc8777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c877b, 61 },
    { 0xafac877b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9c8780, 55 },
    { 0xe70c8780, 33 },
    { 0x2d0c8782, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bc8787, 61 },
    { 0x1bbc8788, 33 },
    { 0x625c8789, 90 },
    { 0x62fc8789, 95 },
    { 0x7cbc8789, 90 },
    { 0, 0 },
    { 0xc5bc878d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cc8790, 43 },
    { 0x6c2c8791, 61 },
    { 0x9b6c8792, 55 },
    { 0x38ec8791, 76 },
    { 0x855c8790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c8799, 61 },
    { 0, 0 },
    { 0xa68c879b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cc879e, 33 },
    { 0, 0 },
    { 0x5b5c87a0, 77 },
    { 0x4cec87a1, 34 },
    { 0x82ec87a0, 33 },
    { 0, 0 },
    { 0x135c87a4, 91 },
    { 0x9d2c87a5, 77 },
    { 0xa1c87a4, 95 },
    { 0x8e7c87a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c87b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcc87bb, 61 },
    { 0xd03c87bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec87c1, 77 },
    { 0xe6cc87c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bac87c5, 90 },
    { 0x911c87c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c87c9, 90 },
    { 0, 0 },
    { 0x197c87cb, 57 },
    { 0xb07c87cc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5c87d1, 33 },
    { 0x3cbc87d2, 98 },
    { 0x4dc87d2, 95 },
    { 0x87bc87d2, 34 },
    { 0x768c87d5, 34 },
    { 0xa89c87d6, 95 },
    { 0xf16c87d6, 61 },
    { 0x9c9c87d8, 23 },
    { 0, 0 },
    { 0x40c87da, 96 },
    { 0xa2bc87db, 87 },
    { 0xf7dc87db, 90 },
    { 0x79dc87dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c87e0, 90 },
    { 0x428c87e1, 55 },
    { 0x5e0c87e2, 84 },
    { 0x68c87e2, 61 },
    { 0x491c87e0, 34 },
    { 0x4edc87e5, 78 },
    { 0x898c87e6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x540c87e9, 99 },
    { 0x58fc87ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c87ed, 23 },
    { 0, 0 },
    { 0xd9dc87ef, 33 },
    { 0, 0 },
    { 0xec87f1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc87f4, 68 },
    { 0x1f6c87f5, 43 },
    { 0x97ec87f6, 98 },
    { 0xb4fc87f4, 33 },
    { 0xdcec87f8, 38 },
    { 0x8d2c87f8, 62 },
    { 0xd49c87fa, 52 },
    { 0xbc3c87fa, 98 },
    { 0x280c87fa, 98 },
    { 0x5efc87fd, 33 },
    { 0xe8ec87fe, 93 },
    { 0, 0 },
    { 0x8acc8800, 91 },
    { 0, 0 },
    { 0x911c8802, 55 },
    { 0xb02c8803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1c8806, 99 },
    { 0, 0 },
    { 0x7a9c8808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3c8812, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c881a, 61 },
    { 0, 0 },
    { 0xaa4c881c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c8821, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fc8828, 78 },
    { 0xf0cc8828, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97c882c, 84 },
    { 0xcecc882c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec8832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac8836, 33 },
    { 0xb8bc8836, 34 },
    { 0x4bbc8836, 98 },
    { 0x257c8839, 84 },
    { 0x1ecc883a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc883d, 34 },
    { 0x43fc883e, 33 },
    { 0xef7c883f, 100 },
    { 0x649c8840, 93 },
    { 0xfa1c8840, 95 },
    { 0x9c8c883f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54c8845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec8852, 33 },
    { 0x30ac8852, 95 },
    { 0x57fc8854, 87 },
    { 0, 0 },
    { 0x4c8c8856, 95 },
    { 0xdcec8856, 23 },
    { 0x7b9c8858, 90 },
    { 0x558c8859, 95 },
    { 0, 0 },
    { 0xa14c885b, 90 },
    { 0x92ac885c, 33 },
    { 0, 0 },
    { 0xd97c885e, 43 },
    { 0x672c885f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258c8864, 90 },
    { 0x2e8c8864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677c886b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c8878, 61 },
    { 0x48fc8879, 98 },
    { 0xa42c887a, 90 },
    { 0x49fc8878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c8880, 100 },
    { 0x41cc8880, 90 },
    { 0x43ac8880, 52 },
    { 0, 0 },
    { 0x5e7c8884, 76 },
    { 0x61bc8885, 98 },
    { 0xaa4c8886, 93 },
    { 0xae7c8887, 98 },
    { 0x8c6c8888, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c8890, 90 },
    { 0x570c8890, 99 },
    { 0xef9c8892, 76 },
    { 0xab3c8893, 33 },
    { 0x78cc8894, 23 },
    { 0xe22c8892, 91 },
    { 0xf2fc8893, 33 },
    { 0, 0 },
    { 0x94dc8898, 55 },
    { 0, 0 },
    { 0x841c889a, 76 },
    { 0, 0 },
    { 0x337c889c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982c88a0, 81 },
    { 0x4e3c88a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2c88a4, 46 },
    { 0, 0 },
    { 0x551c88a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128c88ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c88b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bc88b9, 75 },
    { 0xdb5c88b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac88be, 67 },
    { 0xdb0c88bf, 98 },
    { 0x993c88bf, 61 },
    { 0x5d0c88c0, 23 },
    { 0x9c4c88c2, 98 },
    { 0x9d1c88c3, 84 },
    { 0xe6ec88c4, 100 },
    { 0xf8ac88be, 98 },
    { 0x5c9c88c6, 17 },
    { 0xccfc88c7, 33 },
    { 0x76ec88c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0c88ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1c88d6, 23 },
    { 0x586c88d7, 90 },
    { 0x2c7c88d7, 90 },
    { 0xc0dc88d9, 74 },
    { 0x3e9c88da, 90 },
    { 0xb24c88db, 98 },
    { 0xb9bc88dc, 90 },
    { 0x70dc88dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c88e2, 33 },
    { 0x9f6c88e3, 90 },
    { 0, 0 },
    { 0xa2fc88e5, 33 },
    { 0xea4c88e6, 33 },
    { 0x92ac88e5, 100 },
    { 0x725c88e8, 98 },
    { 0, 0 },
    { 0xf0dc88ea, 40 },
    { 0xd9dc88eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715c88ef, 61 },
    { 0x2b2c88f0, 55 },
    { 0, 0 },
    { 0xdacc88f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c88f7, 55 },
    { 0x6bfc88f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75c88fb, 33 },
    { 0xdc4c88fc, 90 },
    { 0x20c88fb, 33 },
    { 0x13cc88fe, 90 },
    { 0, 0 },
    { 0xe07c8900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cc8903, 90 },
    { 0xd54c8903, 95 },
    { 0, 0 },
    { 0xfcec8906, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55c890a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4c890d, 95 },
    { 0xd17c890e, 55 },
    { 0x435c890f, 76 },
    { 0xaa7c890e, 23 },
    { 0, 0 },
    { 0x692c8912, 66 },
    { 0, 0 },
    { 0x20fc8914, 90 },
    { 0xa9c8914, 95 },
    { 0x7f9c8916, 61 },
    { 0x3cec8917, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6c891b, 90 },
    { 0xdddc891b, 55 },
    { 0x96bc891d, 95 },
    { 0x989c891e, 100 },
    { 0xfac891e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c8922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c8925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c892c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8c8936, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9c893b, 95 },
    { 0, 0 },
    { 0x72cc893d, 90 },
    { 0x75ac893e, 96 },
    { 0x4f7c893e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53c8945, 93 },
    { 0, 0 },
    { 0x9c8c8947, 55 },
    { 0x3bfc8948, 98 },
    { 0x67bc8949, 41 },
    { 0, 0 },
    { 0xa35c894b, 33 },
    { 0x12ac894b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dc8950, 90 },
    { 0, 0 },
    { 0x43cc8952, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c8956, 33 },
    { 0x7c9c8956, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738c895e, 81 },
    { 0xdf0c895f, 53 },
    { 0xbbec8960, 90 },
    { 0, 0 },
    { 0x467c8962, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844c8967, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fc896a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c8970, 52 },
    { 0xaddc8971, 98 },
    { 0x5f2c8972, 88 },
    { 0xc82c8971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c8979, 43 },
    { 0x31c897a, 96 },
    { 0xd39c897b, 55 },
    { 0xa3dc897b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c8983, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec898e, 61 },
    { 0x4cac898e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafc8994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c8999, 99 },
    { 0x7b8c8999, 75 },
    { 0x8dbc899b, 80 },
    { 0x5cbc899b, 52 },
    { 0x187c8999, 100 },
    { 0xaffc899e, 55 },
    { 0xf8c899e, 61 },
    { 0x135c89a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac89a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ac89a7, 98 },
    { 0x157c89a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fec89af, 75 },
    { 0, 0 },
    { 0x57dc89b1, 61 },
    { 0x7ac89b2, 75 },
    { 0, 0 },
    { 0x425c89b4, 33 },
    { 0x4acc89b5, 33 },
    { 0x8bdc89b6, 61 },
    { 0x300c89b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c89ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc89be, 44 },
    { 0, 0 },
    { 0x5ac89c0, 34 },
    { 0xba2c89c1, 90 },
    { 0, 0 },
    { 0xa65c89c3, 88 },
    { 0, 0 },
    { 0x150c89c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1c89cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc89d0, 90 },
    { 0x4ec89d1, 95 },
    { 0x94ac89d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c89d6, 33 },
    { 0, 0 },
    { 0xf22c89d8, 98 },
    { 0xebbc89d9, 33 },
    { 0x6f3c89d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc89df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc89e2, 36 },
    { 0x9bec89e3, 23 },
    { 0, 0 },
    { 0x8b2c89e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dc89ed, 93 },
    { 0, 0 },
    { 0x1aac89ef, 77 },
    { 0xe85c89f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ac89f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c89f8, 90 },
    { 0xe66c89f9, 55 },
    { 0, 0 },
    { 0xb28c89fb, 83 },
    { 0x8aac89fb, 76 },
    { 0, 0 },
    { 0x8f8c89fe, 17 },
    { 0, 0 },
    { 0xf90c8a00, 33 },
    { 0xbd5c8a01, 46 },
    { 0xbe2c8a02, 33 },
    { 0, 0 },
    { 0x8b2c8a04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ec8a07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c8a0a, 23 },
    { 0, 0 },
    { 0x673c8a0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc8a0f, 33 },
    { 0xee5c8a10, 66 },
    { 0xc0fc8a10, 90 },
    { 0x895c8a11, 34 },
    { 0x22c8a10, 98 },
    { 0xeb1c8a14, 90 },
    { 0xebec8a15, 33 },
    { 0x623c8a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac8a1c, 76 },
    { 0x7a6c8a1c, 90 },
    { 0, 0 },
    { 0x7e4c8a1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadc8a25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c8a2a, 76 },
    { 0xff2c8a2b, 23 },
    { 0x150c8a2c, 33 },
    { 0xb95c8a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9c8a33, 23 },
    { 0xc6dc8a34, 100 },
    { 0, 0 },
    { 0xb33c8a36, 81 },
    { 0x6d1c8a36, 23 },
    { 0x84cc8a36, 55 },
    { 0x8f1c8a39, 76 },
    { 0, 0 },
    { 0x6cc8a3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1c8a3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c8a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dc8a45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5c8a49, 61 },
    { 0, 0 },
    { 0xaa1c8a4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1c8a51, 95 },
    { 0x837c8a51, 99 },
    { 0x8d9c8a53, 47 },
    { 0xfa1c8a54, 55 },
    { 0x185c8a55, 55 },
    { 0xfcec8a55, 75 },
    { 0xd91c8a53, 17 },
    { 0x8dcc8a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373c8a5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc8a60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bc8a68, 95 },
    { 0x273c8a68, 78 },
    { 0x4a2c8a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc8a6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefc8a71, 36 },
    { 0x2d2c8a72, 33 },
    { 0xcaec8a73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c8a76, 65 },
    { 0x4d2c8a76, 88 },
    { 0xc6cc8a77, 78 },
    { 0xc65c8a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dc8a82, 90 },
    { 0xcdc8a83, 33 },
    { 0xc2c8a84, 76 },
    { 0, 0 },
    { 0x33ec8a86, 95 },
    { 0, 0 },
    { 0xd0dc8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bc8a8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c8a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c8a97, 33 },
    { 0xb9bc8a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5c8aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac8aa4, 61 },
    { 0x4afc8aa5, 23 },
    { 0, 0 },
    { 0x21dc8aa7, 33 },
    { 0x18bc8aa7, 90 },
    { 0xb50c8aa9, 61 },
    { 0, 0 },
    { 0xe71c8aab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x370c8aae, 90 },
    { 0, 0 },
    { 0x434c8ab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ac8ab3, 90 },
    { 0, 0 },
    { 0xf3ac8ab5, 61 },
    { 0xcbfc8ab6, 76 },
    { 0x1d6c8ab7, 66 },
    { 0x7d5c8ab7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284c8abd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481c8ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8ac6, 55 },
    { 0x322c8ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ec8aca, 95 },
    { 0, 0 },
    { 0xc38c8acc, 98 },
    { 0, 0 },
    { 0xd2bc8ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72c8ad3, 98 },
    { 0x58dc8ad4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x518c8ad7, 95 },
    { 0, 0 },
    { 0x3d3c8ad9, 95 },
    { 0, 0 },
    { 0x77fc8adb, 95 },
    { 0, 0 },
    { 0x380c8add, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c8ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x397c8aea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764c8af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccc8af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x219c8af8, 88 },
    { 0xf3bc8af9, 55 },
    { 0x374c8afa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60c8aff, 95 },
    { 0, 0 },
    { 0x714c8b01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70c8b04, 66 },
    { 0x68dc8b05, 34 },
    { 0, 0 },
    { 0xca4c8b07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc8b0b, 65 },
    { 0xd84c8b0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c8b0f, 33 },
    { 0x253c8b10, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1c8b13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc8b18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ac8b1d, 61 },
    { 0x5a9c8b1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac8b25, 65 },
    { 0x935c8b25, 76 },
    { 0x4aac8b27, 61 },
    { 0, 0 },
    { 0x83fc8b29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x893c8b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c8b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c8b34, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8c8b3a, 99 },
    { 0xa6c8b3b, 40 },
    { 0x20bc8b3c, 98 },
    { 0x8bac8b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c8b43, 98 },
    { 0x470c8b43, 90 },
    { 0x360c8b43, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afc8b48, 98 },
    { 0xd3cc8b48, 23 },
    { 0x1e0c8b48, 95 },
    { 0xbf7c8b4b, 55 },
    { 0xa4bc8b4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbc8b4f, 90 },
    { 0, 0 },
    { 0x435c8b51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c8b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ec8b59, 90 },
    { 0x466c8b5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c8b60, 95 },
    { 0x18bc8b61, 93 },
    { 0xa35c8b61, 98 },
    { 0x2ecc8b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x401c8b66, 81 },
    { 0x6c1c8b67, 34 },
    { 0x8cac8b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c8b6c, 78 },
    { 0xb2ac8b6d, 33 },
    { 0, 0 },
    { 0xcddc8b6f, 84 },
    { 0xdbdc8b70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x310c8b73, 98 },
    { 0x6efc8b73, 98 },
    { 0, 0 },
    { 0x7c0c8b76, 91 },
    { 0, 0 },
    { 0xec3c8b78, 66 },
    { 0xd37c8b79, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66c8b7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0c8b7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cc8b83, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c8b86, 55 },
    { 0xf72c8b87, 90 },
    { 0, 0 },
    { 0x52fc8b89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23c8b8c, 98 },
    { 0, 0 },
    { 0x540c8b8e, 85 },
    { 0, 0 },
    { 0x49dc8b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8b93, 61 },
    { 0xc7c8b94, 34 },
    { 0x33dc8b94, 95 },
    { 0x48fc8b94, 94 },
    { 0, 0 },
    { 0x76bc8b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c8b9c, 23 },
    { 0x202c8b9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4c8ba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367c8ba4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c8ba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c8baf, 78 },
    { 0xd81c8bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fc8bb5, 55 },
    { 0x20bc8bb6, 48 },
    { 0x52c8bb7, 88 },
    { 0x84ec8bb8, 81 },
    { 0x97cc8bb6, 55 },
    { 0xf45c8bb8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c8bbe, 61 },
    { 0, 0 },
    { 0x1ecc8bc0, 67 },
    { 0xdc3c8bc0, 90 },
    { 0x3cac8bc2, 33 },
    { 0xac6c8bc3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0c8bce, 34 },
    { 0x5bbc8bcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8bd2, 91 },
    { 0, 0 },
    { 0xcacc8bd4, 33 },
    { 0, 0 },
    { 0x12dc8bd6, 98 },
    { 0xf79c8bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x291c8bda, 85 },
    { 0xb17c8bda, 61 },
    { 0x51c8bdb, 55 },
    { 0xecdc8bdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c8be8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8bec, 17 },
    { 0, 0 },
    { 0x109c8bee, 88 },
    { 0x3c8c8bee, 65 },
    { 0x74bc8bf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5c8bf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4c8bfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c8c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c8c04, 95 },
    { 0x2d3c8c05, 98 },
    { 0xa1ac8c05, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c8c09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839c8c0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c8c11, 67 },
    { 0xcb5c8c12, 91 },
    { 0xf09c8c11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc8c17, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3c8c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c8c21, 33 },
    { 0xa5cc8c22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c8c2b, 61 },
    { 0x7d9c8c2c, 98 },
    { 0, 0 },
    { 0xceac8c2e, 23 },
    { 0x1c5c8c2f, 33 },
    { 0xdb1c8c2e, 95 },
    { 0, 0 },
    { 0xe32c8c32, 43 },
    { 0, 0 },
    { 0xba7c8c34, 76 },
    { 0x6fbc8c35, 43 },
    { 0xfe4c8c36, 95 },
    { 0x6d4c8c37, 77 },
    { 0x318c8c37, 33 },
    { 0x979c8c38, 77 },
    { 0x1c7c8c34, 84 },
    { 0xd81c8c3b, 90 },
    { 0, 0 },
    { 0x792c8c3d, 22 },
    { 0x8f8c8c3e, 99 },
    { 0x57ec8c3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc8c4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x852c8c50, 36 },
    { 0, 0 },
    { 0x261c8c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecc8c56, 33 },
    { 0x949c8c57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec8c5a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc8c60, 90 },
    { 0xe34c8c61, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c8c65, 55 },
    { 0x4b5c8c65, 87 },
    { 0x4a7c8c67, 75 },
    { 0xc37c8c68, 85 },
    { 0xe92c8c69, 91 },
    { 0xb9cc8c68, 90 },
    { 0xe29c8c6b, 64 },
    { 0xeb8c8c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7c8c70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c8c78, 34 },
    { 0xfbac8c79, 98 },
    { 0, 0 },
    { 0xfabc8c7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5c8c7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc8c83, 84 },
    { 0x951c8c84, 17 },
    { 0x1efc8c85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc8c91, 78 },
    { 0x67ec8c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8c96, 2 },
    { 0xec7c8c97, 98 },
    { 0xc68c8c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x269c8c9b, 23 },
    { 0, 0 },
    { 0x3fdc8c9d, 83 },
    { 0x842c8c9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776c8ca1, 38 },
    { 0xe47c8ca1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c8ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818c8ca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac8cac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c8cb1, 23 },
    { 0xa9fc8cb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4c8cb7, 23 },
    { 0, 0 },
    { 0x1c0c8cb9, 63 },
    { 0, 0 },
    { 0x611c8cbb, 66 },
    { 0x19dc8cbb, 55 },
    { 0, 0 },
    { 0x7e9c8cbe, 95 },
    { 0, 0 },
    { 0x926c8cc0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2c8cc3, 76 },
    { 0, 0 },
    { 0xbb4c8cc5, 33 },
    { 0, 0 },
    { 0x5e8c8cc7, 91 },
    { 0, 0 },
    { 0xd24c8cc9, 33 },
    { 0, 0 },
    { 0xe1fc8ccb, 61 },
    { 0x2b2c8ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c8ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c8cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16c8cde, 90 },
    { 0, 0 },
    { 0xaf8c8ce0, 68 },
    { 0, 0 },
    { 0x149c8ce2, 36 },
    { 0, 0 },
    { 0x6e5c8ce4, 95 },
    { 0x485c8ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7c8cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbac8cf3, 43 },
    { 0, 0 },
    { 0x5a9c8cf5, 98 },
    { 0, 0 },
    { 0x56c8cf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6c8cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c8cfe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dc8d06, 90 },
    { 0x264c8d07, 95 },
    { 0, 0 },
    { 0x673c8d09, 93 },
    { 0xe17c8d0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccc8d0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8d10, 98 },
    { 0xeecc8d10, 95 },
    { 0, 0 },
    { 0x8aec8d13, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749c8d18, 93 },
    { 0xf40c8d18, 95 },
    { 0, 0 },
    { 0xdfac8d1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc8d22, 23 },
    { 0, 0 },
    { 0x90bc8d24, 78 },
    { 0xb9cc8d24, 69 },
    { 0x32fc8d24, 95 },
    { 0, 0 },
    { 0xebdc8d28, 43 },
    { 0x79ac8d28, 23 },
    { 0xeb8c8d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efc8d2e, 23 },
    { 0x69bc8d2f, 61 },
    { 0xb05c8d30, 98 },
    { 0x66c8d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaec8d36, 61 },
    { 0, 0 },
    { 0xd39c8d38, 61 },
    { 0xa44c8d39, 95 },
    { 0x36ec8d39, 90 },
    { 0, 0 },
    { 0xfaec8d3c, 90 },
    { 0x512c8d3c, 23 },
    { 0xb5c8d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c8d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc8d45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edc8d4b, 90 },
    { 0x8dec8d4b, 61 },
    { 0xc6cc8d4b, 90 },
    { 0, 0 },
    { 0xd6bc8d4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec8d52, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc8d55, 91 },
    { 0x7ccc8d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec8d59, 78 },
    { 0x48fc8d5a, 69 },
    { 0x738c8d5b, 95 },
    { 0x67fc8d5b, 61 },
    { 0x5fec8d5b, 93 },
    { 0x424c8d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c8d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc8d6e, 98 },
    { 0xc2c8d6e, 33 },
    { 0x716c8d6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x533c8d73, 99 },
    { 0xefc8d74, 77 },
    { 0, 0 },
    { 0xb19c8d76, 98 },
    { 0xb3fc8d76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x788c8d7a, 61 },
    { 0, 0 },
    { 0xc34c8d7c, 34 },
    { 0xf64c8d7d, 23 },
    { 0, 0 },
    { 0x337c8d7f, 34 },
    { 0, 0 },
    { 0xec8d81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919c8d87, 88 },
    { 0xca8c8d88, 33 },
    { 0x61cc8d88, 78 },
    { 0x8c6c8d87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7c8d8f, 61 },
    { 0, 0 },
    { 0x757c8d91, 84 },
    { 0, 0 },
    { 0x661c8d93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c8d96, 95 },
    { 0xa9cc8d97, 23 },
    { 0xf54c8d98, 61 },
    { 0, 0 },
    { 0xeb7c8d9a, 90 },
    { 0, 0 },
    { 0x282c8d9c, 98 },
    { 0x427c8d9d, 61 },
    { 0x55ec8d9e, 90 },
    { 0xc61c8d9e, 99 },
    { 0x7e6c8da0, 61 },
    { 0, 0 },
    { 0x1d7c8da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02c8da9, 95 },
    { 0x43c8daa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dec8dad, 98 },
    { 0, 0 },
    { 0x3f8c8daf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cac8db4, 93 },
    { 0xfc7c8db5, 61 },
    { 0xfc0c8db6, 95 },
    { 0x9c7c8db7, 23 },
    { 0x970c8db5, 90 },
    { 0x8f7c8db9, 23 },
    { 0x789c8db6, 23 },
    { 0, 0 },
    { 0x6c6c8dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471c8dc0, 33 },
    { 0x8c2c8dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c8dc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805c8dd3, 33 },
    { 0xe84c8dd3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8dd8, 77 },
    { 0, 0 },
    { 0xd34c8dda, 76 },
    { 0xd27c8ddb, 76 },
    { 0x90dc8ddc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c8ddf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658c8de4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c8de8, 33 },
    { 0xfb0c8de8, 33 },
    { 0x669c8dea, 95 },
    { 0x279c8dea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fc8df3, 90 },
    { 0, 0 },
    { 0x572c8df5, 93 },
    { 0, 0 },
    { 0x400c8df7, 98 },
    { 0xe68c8df7, 90 },
    { 0x50ec8df9, 23 },
    { 0xcadc8df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x733c8dfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31c8e01, 43 },
    { 0x5f6c8e02, 90 },
    { 0x6ccc8e02, 98 },
    { 0, 0 },
    { 0x43c8e05, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31c8e08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8e0e, 76 },
    { 0, 0 },
    { 0x3bec8e10, 95 },
    { 0x3bcc8e11, 23 },
    { 0xa90c8e10, 23 },
    { 0xf36c8e13, 98 },
    { 0x156c8e14, 98 },
    { 0, 0 },
    { 0xce0c8e16, 95 },
    { 0xbbbc8e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc8e1a, 33 },
    { 0xbeec8e1b, 95 },
    { 0x5a2c8e1c, 34 },
    { 0x516c8e1a, 94 },
    { 0x89fc8e1a, 91 },
    { 0xae8c8e1f, 90 },
    { 0x5b3c8e1d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680c8e24, 98 },
    { 0xa45c8e24, 90 },
    { 0xdbbc8e26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106c8e2a, 66 },
    { 0x1c2c8e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5c8e2f, 46 },
    { 0, 0 },
    { 0x3c5c8e31, 33 },
    { 0, 0 },
    { 0x9e9c8e33, 23 },
    { 0, 0 },
    { 0x9a8c8e35, 31 },
    { 0x2fac8e36, 23 },
    { 0x448c8e36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac8e3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cc8e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7c8e4b, 90 },
    { 0x15bc8e4b, 90 },
    { 0x8e4c8e4d, 99 },
    { 0x2efc8e4c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4c8e52, 52 },
    { 0xf94c8e53, 95 },
    { 0x293c8e54, 77 },
    { 0x74c8e53, 33 },
    { 0xc82c8e56, 90 },
    { 0xacdc8e56, 33 },
    { 0x271c8e56, 98 },
    { 0xcdcc8e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708c8e5f, 98 },
    { 0, 0 },
    { 0x893c8e61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ac8e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620c8e6a, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x137c8e6d, 90 },
    { 0x6dec8e6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc8e71, 93 },
    { 0x431c8e71, 98 },
    { 0x4d0c8e72, 33 },
    { 0xf32c8e74, 68 },
    { 0, 0 },
    { 0xd02c8e76, 34 },
    { 0x9d0c8e76, 95 },
    { 0x787c8e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dc8e7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc8e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c8e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x998c8e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8e90, 43 },
    { 0, 0 },
    { 0x6c8e92, 52 },
    { 0, 0 },
    { 0x7c6c8e94, 93 },
    { 0x44fc8e95, 98 },
    { 0, 0 },
    { 0x45cc8e97, 61 },
    { 0, 0 },
    { 0xae1c8e99, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dc8e9f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc8ea2, 55 },
    { 0, 0 },
    { 0xd50c8ea4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c8ea9, 33 },
    { 0, 0 },
    { 0xf0fc8eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc8eae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26c8eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c8eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x874c8eb9, 84 },
    { 0xc7ac8eba, 43 },
    { 0xacfc8eba, 95 },
    { 0x125c8ebc, 66 },
    { 0x647c8eba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6c8ec3, 95 },
    { 0, 0 },
    { 0x575c8ec5, 100 },
    { 0x4bec8ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c8ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c8ecd, 93 },
    { 0, 0 },
    { 0xd4ac8ecf, 34 },
    { 0x52c8ecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c8ed4, 76 },
    { 0xbdc8ed5, 52 },
    { 0, 0 },
    { 0x937c8ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0c8eda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58c8edd, 100 },
    { 0x360c8ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc8ee4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdac8ee8, 91 },
    { 0x4a5c8ee9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c8eec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923c8ef1, 33 },
    { 0xbfec8ef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9c8ef5, 33 },
    { 0xa23c8ef6, 55 },
    { 0, 0 },
    { 0x167c8ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc8eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceac8f07, 90 },
    { 0, 0 },
    { 0xf1cc8f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec8f0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c8f10, 84 },
    { 0x44cc8f10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ac8f15, 81 },
    { 0x34bc8f16, 23 },
    { 0xfa4c8f17, 95 },
    { 0, 0 },
    { 0x962c8f19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0c8f1e, 99 },
    { 0x856c8f1f, 61 },
    { 0xd8cc8f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c8f23, 53 },
    { 0x752c8f24, 61 },
    { 0x8afc8f25, 90 },
    { 0xa2ec8f24, 31 },
    { 0x22fc8f27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ec8f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bc8f2d, 98 },
    { 0xba3c8f2e, 98 },
    { 0x372c8f2f, 34 },
    { 0, 0 },
    { 0xfc3c8f31, 90 },
    { 0x9f5c8f31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39c8f36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3c8f39, 84 },
    { 0x391c8f3a, 90 },
    { 0, 0 },
    { 0x62bc8f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ec8f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c8f46, 80 },
    { 0x8f3c8f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7c8f4f, 93 },
    { 0, 0 },
    { 0x733c8f51, 90 },
    { 0, 0 },
    { 0xe17c8f53, 95 },
    { 0xe21c8f54, 33 },
    { 0, 0 },
    { 0x25dc8f56, 43 },
    { 0x7e2c8f57, 61 },
    { 0xc4ac8f58, 81 },
    { 0xca0c8f59, 36 },
    { 0x732c8f5a, 23 },
    { 0x2a0c8f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95c8f5f, 43 },
    { 0, 0 },
    { 0x2fdc8f61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dc8f69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c8f6c, 66 },
    { 0, 0 },
    { 0x148c8f6e, 65 },
    { 0xd83c8f6f, 55 },
    { 0x789c8f6f, 53 },
    { 0x69ac8f71, 43 },
    { 0, 0 },
    { 0x24ac8f73, 33 },
    { 0xe7cc8f74, 34 },
    { 0xeecc8f74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73c8f7a, 98 },
    { 0x550c8f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445c8f81, 90 },
    { 0, 0 },
    { 0x5f3c8f83, 23 },
    { 0x457c8f84, 76 },
    { 0, 0 },
    { 0x5bcc8f86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c8f89, 61 },
    { 0, 0 },
    { 0x814c8f8b, 95 },
    { 0, 0 },
    { 0x6b1c8f8d, 90 },
    { 0x32cc8f8e, 78 },
    { 0x551c8f8f, 97 },
    { 0x51ac8f90, 40 },
    { 0x93dc8f8e, 98 },
    { 0, 0 },
    { 0x67cc8f93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8f96, 99 },
    { 0xc3ec8f97, 90 },
    { 0xd38c8f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983c8fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc8fa4, 33 },
    { 0xdcbc8fa5, 33 },
    { 0xa17c8fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dc8fac, 61 },
    { 0x218c8fac, 23 },
    { 0x5ccc8fae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecc8fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dc8fc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5c8fc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dc8fca, 52 },
    { 0, 0 },
    { 0x177c8fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fc8fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ac8fd5, 99 },
    { 0, 0 },
    { 0x250c8fd7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c8fdb, 78 },
    { 0x86bc8fdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33c8fe0, 76 },
    { 0xd6ec8fe0, 85 },
    { 0x691c8fe0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac8fe9, 23 },
    { 0, 0 },
    { 0x6f7c8feb, 23 },
    { 0, 0 },
    { 0xd41c8fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac8ff0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c8ff4, 90 },
    { 0x807c8ff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc8ff9, 68 },
    { 0x65fc8ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dc9001, 87 },
    { 0, 0 },
    { 0xc8ec9003, 43 },
    { 0xa4ac9004, 90 },
    { 0, 0 },
    { 0x57fc9006, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c900a, 52 },
    { 0x92c900b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc900f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc9014, 93 },
    { 0x601c9015, 84 },
    { 0xc15c9016, 33 },
    { 0xe1c9015, 33 },
    { 0xa27c9018, 66 },
    { 0x87fc9017, 90 },
    { 0xa19c9018, 23 },
    { 0x650c901b, 33 },
    { 0x801c9016, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fc9020, 95 },
    { 0, 0 },
    { 0xd13c9022, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c9025, 43 },
    { 0, 0 },
    { 0xa33c9027, 95 },
    { 0x2c7c9028, 100 },
    { 0x367c9029, 61 },
    { 0x660c9028, 61 },
    { 0xdd0c9029, 46 },
    { 0xb61c902c, 95 },
    { 0x641c902b, 33 },
    { 0x79fc902e, 67 },
    { 0x1aac902f, 34 },
    { 0, 0 },
    { 0x2bcc9031, 90 },
    { 0xf9c9032, 55 },
    { 0xae0c9031, 17 },
    { 0xbefc9031, 34 },
    { 0x2bfc9034, 33 },
    { 0, 0 },
    { 0x179c9037, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bc903a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95c9040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c9043, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5c9046, 93 },
    { 0xf12c9047, 35 },
    { 0x5c5c9048, 23 },
    { 0x77bc9049, 61 },
    { 0x451c904a, 98 },
    { 0x67bc9048, 88 },
    { 0, 0 },
    { 0x898c904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97c9052, 98 },
    { 0x9ecc9053, 33 },
    { 0xc82c9054, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc9057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x691c905a, 61 },
    { 0xdb3c905a, 23 },
    { 0x706c905b, 95 },
    { 0, 0 },
    { 0xc05c905e, 87 },
    { 0x70ac905e, 78 },
    { 0, 0 },
    { 0x4d6c9061, 43 },
    { 0x8c8c9061, 23 },
    { 0xa6bc9062, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c9066, 90 },
    { 0x97bc9067, 81 },
    { 0, 0 },
    { 0xb52c9069, 85 },
    { 0x7f2c906a, 98 },
    { 0, 0 },
    { 0x3c5c906c, 61 },
    { 0x19dc906c, 90 },
    { 0x438c906e, 34 },
    { 0x273c906f, 33 },
    { 0x734c9070, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ac9073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c9077, 55 },
    { 0, 0 },
    { 0x44ac9079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c907c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ac9082, 95 },
    { 0x972c9082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3c9088, 90 },
    { 0xffcc9089, 90 },
    { 0x5bc9088, 40 },
    { 0x507c908a, 61 },
    { 0x530c908c, 98 },
    { 0xba8c908d, 26 },
    { 0x941c908d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c9093, 52 },
    { 0x548c9094, 33 },
    { 0xdc5c9095, 33 },
    { 0xe8c9096, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebc90a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc90a6, 61 },
    { 0xefc90a7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc90aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5c90ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c90b0, 91 },
    { 0xd73c90b1, 90 },
    { 0x627c90b2, 88 },
    { 0, 0 },
    { 0xf2bc90b4, 88 },
    { 0, 0 },
    { 0x1ecc90b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c90bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0c90be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc90c3, 33 },
    { 0xe85c90c3, 87 },
    { 0, 0 },
    { 0xd78c90c6, 55 },
    { 0xb5cc90c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c90ca, 43 },
    { 0, 0 },
    { 0x540c90cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60c90cf, 23 },
    { 0, 0 },
    { 0xe5cc90d1, 56 },
    { 0, 0 },
    { 0x277c90d3, 61 },
    { 0x73cc90d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c90d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc90db, 88 },
    { 0x865c90db, 34 },
    { 0xf59c90dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4c90e2, 33 },
    { 0, 0 },
    { 0x481c90e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6c90ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc90ef, 61 },
    { 0x7e9c90f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc90f7, 61 },
    { 0, 0 },
    { 0x595c90f9, 61 },
    { 0, 0 },
    { 0xf6ec90fb, 61 },
    { 0, 0 },
    { 0x78ec90fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x280c9100, 85 },
    { 0, 0 },
    { 0xb57c9102, 61 },
    { 0, 0 },
    { 0x6c2c9104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c9107, 95 },
    { 0x7bbc9107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734c910d, 34 },
    { 0x9c8c910e, 61 },
    { 0x804c910f, 33 },
    { 0xd13c910d, 23 },
    { 0xa33c9111, 85 },
    { 0x9fac9112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcc911b, 90 },
    { 0x485c911c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bc912f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec913b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c9140, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10c9144, 76 },
    { 0xc4bc9144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c9148, 61 },
    { 0x8a0c9148, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fc914c, 93 },
    { 0x779c914d, 90 },
    { 0x71ac914d, 90 },
    { 0x49fc914e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec915f, 85 },
    { 0x29cc915f, 55 },
    { 0x2b4c9161, 78 },
    { 0, 0 },
    { 0x611c9163, 23 },
    { 0xad6c9164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4c9167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaec916c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356c917b, 90 },
    { 0xcdcc917b, 23 },
    { 0x38bc917d, 68 },
    { 0x703c917e, 23 },
    { 0x24ac917f, 33 },
    { 0x3c4c9180, 75 },
    { 0, 0 },
    { 0x7adc9182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fc9187, 95 },
    { 0xbe8c9187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7c918c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c918f, 90 },
    { 0x91c9190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c9193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc9196, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x282c9199, 17 },
    { 0, 0 },
    { 0x814c919b, 100 },
    { 0x4c7c919b, 33 },
    { 0xa6dc919c, 93 },
    { 0xfffc919e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bc91a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c91a5, 66 },
    { 0xcb3c91a6, 90 },
    { 0x4a5c91a7, 61 },
    { 0xb43c91a6, 98 },
    { 0, 0 },
    { 0x1d0c91aa, 55 },
    { 0xf99c91aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ec91b4, 61 },
    { 0x98c91b4, 90 },
    { 0x1c8c91b4, 99 },
    { 0xcf2c91b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88c91ba, 34 },
    { 0, 0 },
    { 0x6f1c91bc, 99 },
    { 0xe51c91bc, 46 },
    { 0x279c91be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00c91c3, 33 },
    { 0xdc3c91c3, 33 },
    { 0x17c91c4, 55 },
    { 0x4c3c91c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c91c9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c91cd, 55 },
    { 0x27ac91ce, 61 },
    { 0x3d6c91cd, 23 },
    { 0x129c91d0, 49 },
    { 0xa66c91d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907c91d7, 81 },
    { 0x285c91d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2c91de, 87 },
    { 0x6d8c91de, 33 },
    { 0x764c91de, 96 },
    { 0, 0 },
    { 0x56cc91e2, 90 },
    { 0x37ac91e3, 61 },
    { 0x96ec91e3, 55 },
    { 0xdcc91e5, 34 },
    { 0x6e6c91e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39c91eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c91ef, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c91f7, 76 },
    { 0, 0 },
    { 0x256c91f9, 52 },
    { 0, 0 },
    { 0xeecc91fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ec9201, 98 },
    { 0x546c9201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c920a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c920d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51c9210, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444c9217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6c921a, 95 },
    { 0, 0 },
    { 0xe79c921c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c9222, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc9227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c922a, 93 },
    { 0xeafc922a, 61 },
    { 0x9cac922b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec922f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c9233, 33 },
    { 0x4f2c9233, 33 },
    { 0x2fac9234, 34 },
    { 0x881c9235, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7c9239, 99 },
    { 0, 0 },
    { 0xee4c923b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbc923e, 61 },
    { 0, 0 },
    { 0x2a0c9240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705c9244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac9247, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c924b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3c9250, 33 },
    { 0, 0 },
    { 0x14dc9252, 46 },
    { 0, 0 },
    { 0xbedc9254, 23 },
    { 0xadbc9255, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc925e, 34 },
    { 0x3ddc925f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c9267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816c926e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c9271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dc9275, 17 },
    { 0x363c9276, 90 },
    { 0, 0 },
    { 0x6c2c9278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c927b, 34 },
    { 0, 0 },
    { 0x509c927d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c9280, 61 },
    { 0xb13c9281, 76 },
    { 0xcec9282, 33 },
    { 0x403c9281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbc928c, 61 },
    { 0x576c928d, 19 },
    { 0x346c928c, 61 },
    { 0xb1dc928f, 33 },
    { 0xb08c928d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac929c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc929f, 23 },
    { 0, 0 },
    { 0x43ec92a1, 95 },
    { 0xf5ec92a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc92a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebc92ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c92af, 90 },
    { 0, 0 },
    { 0x5c0c92b1, 65 },
    { 0xb0ac92b2, 77 },
    { 0x361c92b2, 61 },
    { 0x26bc92b4, 100 },
    { 0, 0 },
    { 0x5eac92b6, 61 },
    { 0xbf2c92b7, 67 },
    { 0, 0 },
    { 0xdaac92b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcc92bd, 23 },
    { 0x646c92be, 61 },
    { 0xea0c92bd, 88 },
    { 0x304c92c0, 23 },
    { 0, 0 },
    { 0xd1dc92c2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709c92c6, 90 },
    { 0x16cc92c7, 91 },
    { 0x9fac92c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c92cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cc92ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c92d3, 90 },
    { 0x897c92d3, 33 },
    { 0x662c92d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c92d8, 33 },
    { 0x584c92d8, 67 },
    { 0, 0 },
    { 0x58dc92db, 95 },
    { 0x642c92db, 23 },
    { 0x80fc92dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x704c92e0, 98 },
    { 0x9bbc92e1, 23 },
    { 0xfafc92e2, 34 },
    { 0xeafc92e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6c92e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c92eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c92ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4c92f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c92fe, 90 },
    { 0x872c92ff, 33 },
    { 0x5ddc9300, 99 },
    { 0x287c9301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c9305, 46 },
    { 0xf57c9305, 43 },
    { 0x5a2c9307, 33 },
    { 0x7c2c9305, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eac930d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfc9313, 90 },
    { 0x7f0c9314, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec9318, 61 },
    { 0x315c9319, 84 },
    { 0xfa4c931a, 61 },
    { 0x5b1c9318, 93 },
    { 0x233c931c, 77 },
    { 0x621c931d, 90 },
    { 0xcf7c931e, 98 },
    { 0x55ec931d, 85 },
    { 0, 0 },
    { 0x2a9c9321, 23 },
    { 0xf2bc9322, 34 },
    { 0, 0 },
    { 0x79ac9324, 98 },
    { 0, 0 },
    { 0x94fc9326, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c932e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202c9334, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbc933b, 90 },
    { 0, 0 },
    { 0x545c933d, 23 },
    { 0x75ac933d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dc9344, 61 },
    { 0xc39c9345, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c934b, 55 },
    { 0xa77c934c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c934f, 34 },
    { 0x579c934f, 69 },
    { 0xa36c9350, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c9357, 43 },
    { 0, 0 },
    { 0x815c9359, 90 },
    { 0, 0 },
    { 0x623c935b, 75 },
    { 0xdc7c935c, 87 },
    { 0x553c935b, 48 },
    { 0x92cc935b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ec9365, 61 },
    { 0, 0 },
    { 0x454c9367, 90 },
    { 0, 0 },
    { 0x70dc9369, 98 },
    { 0x291c936a, 66 },
    { 0xe7cc936b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec936e, 98 },
    { 0x8e9c936f, 85 },
    { 0x291c936e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98c9373, 76 },
    { 0, 0 },
    { 0x18cc9375, 23 },
    { 0x34c9375, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ac937c, 98 },
    { 0, 0 },
    { 0xecec937e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c9381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c9384, 33 },
    { 0x39ac9385, 43 },
    { 0xa80c9386, 66 },
    { 0xa5ec9387, 90 },
    { 0, 0 },
    { 0x45fc9389, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x680c938c, 23 },
    { 0, 0 },
    { 0x236c938e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c9396, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c939a, 61 },
    { 0x43cc939b, 9 },
    { 0x373c939b, 95 },
    { 0x8cac939d, 66 },
    { 0x3b5c939e, 90 },
    { 0xb8ec939f, 46 },
    { 0x938c93a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0c93a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497c93a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc93ab, 33 },
    { 0, 0 },
    { 0xb0ac93ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc93b0, 88 },
    { 0x4c3c93b0, 55 },
    { 0, 0 },
    { 0x317c93b3, 100 },
    { 0xb71c93b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c93bb, 33 },
    { 0, 0 },
    { 0xd76c93bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113c93c7, 55 },
    { 0x6bdc93c8, 95 },
    { 0, 0 },
    { 0xc20c93ca, 95 },
    { 0xe53c93cb, 61 },
    { 0, 0 },
    { 0x8f3c93cd, 95 },
    { 0xc57c93ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cc93d1, 84 },
    { 0, 0 },
    { 0x4c0c93d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c93d7, 95 },
    { 0, 0 },
    { 0x652c93d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c93dc, 43 },
    { 0, 0 },
    { 0xf9fc93de, 33 },
    { 0x73cc93df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c93e2, 91 },
    { 0, 0 },
    { 0xda0c93e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc93e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316c93ef, 46 },
    { 0x378c93f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40c93f5, 33 },
    { 0x5d4c93f6, 23 },
    { 0xba5c93f7, 46 },
    { 0, 0 },
    { 0x380c93f9, 93 },
    { 0x669c93fa, 90 },
    { 0xe47c93fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c9402, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c940c, 33 },
    { 0xa12c940c, 90 },
    { 0, 0 },
    { 0x640c940f, 23 },
    { 0xb62c940f, 98 },
    { 0, 0 },
    { 0xfa4c9412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c9415, 36 },
    { 0, 0 },
    { 0x231c9417, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cc9421, 43 },
    { 0, 0 },
    { 0x343c9423, 76 },
    { 0xc32c9424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13c9428, 95 },
    { 0x508c9429, 90 },
    { 0xe73c942a, 66 },
    { 0xbc3c9428, 17 },
    { 0xed6c942c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x303c942f, 91 },
    { 0, 0 },
    { 0x141c9431, 33 },
    { 0x9ebc9432, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0c9435, 76 },
    { 0xb95c9436, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefc9439, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacc943c, 33 },
    { 0, 0 },
    { 0x994c943e, 33 },
    { 0x209c943f, 53 },
    { 0xb21c943e, 46 },
    { 0xa8cc943e, 95 },
    { 0xbd5c943f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x827c9445, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502c944d, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0c9451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fc9454, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c945d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c9464, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c946c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c9470, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c9476, 90 },
    { 0x845c9477, 95 },
    { 0xb0c9478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x139c947b, 66 },
    { 0x7b2c947c, 99 },
    { 0xf85c947d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665c9482, 23 },
    { 0x283c9483, 91 },
    { 0xad4c9483, 90 },
    { 0x3c9483, 46 },
    { 0x239c9485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5c9489, 55 },
    { 0x149c9489, 100 },
    { 0xe81c9489, 98 },
    { 0x63bc948b, 100 },
    { 0xe61c948a, 66 },
    { 0x751c948c, 91 },
    { 0, 0 },
    { 0x3fec9490, 33 },
    { 0x6f6c9490, 90 },
    { 0x3c8c9490, 76 },
    { 0x856c9490, 76 },
    { 0x9f1c9494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0c9497, 33 },
    { 0, 0 },
    { 0xee6c9499, 55 },
    { 0x10ec949a, 95 },
    { 0, 0 },
    { 0xeeac949c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c94a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebac94a9, 61 },
    { 0xa38c94aa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c94ad, 92 },
    { 0xa4ac94ae, 90 },
    { 0x5f2c94af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c94b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862c94bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ac94c1, 23 },
    { 0xae9c94c2, 95 },
    { 0, 0 },
    { 0x554c94c4, 43 },
    { 0, 0 },
    { 0xb38c94c6, 55 },
    { 0x4c2c94c7, 61 },
    { 0x86c94c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc94cb, 95 },
    { 0xfc7c94cb, 90 },
    { 0xc45c94cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c94d0, 23 },
    { 0, 0 },
    { 0xfebc94d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc94da, 46 },
    { 0xc33c94db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c94e0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x390c94e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebc94e6, 98 },
    { 0xaa7c94e7, 55 },
    { 0xa80c94e8, 33 },
    { 0, 0 },
    { 0x9a4c94ea, 52 },
    { 0x76ac94ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbc94f1, 100 },
    { 0, 0 },
    { 0xbe7c94f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cc94f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dc9501, 95 },
    { 0, 0 },
    { 0x5dc9503, 61 },
    { 0xbb6c9503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c9507, 95 },
    { 0xe74c9508, 66 },
    { 0x25bc9508, 61 },
    { 0x317c9508, 99 },
    { 0x95fc950b, 91 },
    { 0, 0 },
    { 0x5d7c950d, 90 },
    { 0xf9ac950e, 90 },
    { 0x728c950f, 90 },
    { 0, 0 },
    { 0xf24c9511, 98 },
    { 0, 0 },
    { 0x1b1c9513, 95 },
    { 0x4a3c9514, 61 },
    { 0xb02c9514, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7c951d, 61 },
    { 0x3d5c951d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c9524, 55 },
    { 0, 0 },
    { 0xaddc9526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc952a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91c9530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17c953a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7c953e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbc9541, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc9544, 90 },
    { 0, 0 },
    { 0x5a4c9546, 55 },
    { 0x304c9546, 85 },
    { 0x548c9548, 90 },
    { 0x1a9c9549, 46 },
    { 0x1f7c954a, 61 },
    { 0xbc7c954b, 95 },
    { 0, 0 },
    { 0xa99c954d, 76 },
    { 0x536c954e, 90 },
    { 0xeeac954e, 76 },
    { 0x894c9550, 90 },
    { 0, 0 },
    { 0xa7bc9552, 46 },
    { 0x476c9553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc9557, 23 },
    { 0x888c9558, 99 },
    { 0x11c9559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22c9560, 90 },
    { 0, 0 },
    { 0xe1ac9562, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c9566, 100 },
    { 0, 0 },
    { 0xec6c9568, 33 },
    { 0x6f0c9569, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9c956d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75c9570, 34 },
    { 0, 0 },
    { 0x9fdc9572, 90 },
    { 0x651c9572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35c9578, 98 },
    { 0xddbc9578, 46 },
    { 0xf69c9578, 61 },
    { 0x59bc957b, 87 },
    { 0, 0 },
    { 0x9e0c957d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898c9582, 85 },
    { 0x481c9582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c9588, 98 },
    { 0xaeac9589, 61 },
    { 0xe3cc9589, 52 },
    { 0x9bbc9589, 33 },
    { 0x3c9c958b, 87 },
    { 0x3fcc958c, 76 },
    { 0xa93c9588, 53 },
    { 0, 0 },
    { 0xe0ac9590, 91 },
    { 0, 0 },
    { 0xd1dc9592, 33 },
    { 0, 0 },
    { 0x57fc9594, 80 },
    { 0, 0 },
    { 0xfc4c9596, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6c959b, 98 },
    { 0, 0 },
    { 0x272c959d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cc95a2, 61 },
    { 0x54fc95a3, 93 },
    { 0, 0 },
    { 0xb82c95a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c95a8, 95 },
    { 0x785c95a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec95ae, 33 },
    { 0, 0 },
    { 0x48bc95b0, 90 },
    { 0x509c95b1, 95 },
    { 0x2d8c95b2, 87 },
    { 0, 0 },
    { 0x1bac95b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0c95ba, 61 },
    { 0, 0 },
    { 0xa69c95bc, 61 },
    { 0, 0 },
    { 0x796c95be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c95c3, 43 },
    { 0x107c95c4, 69 },
    { 0x41dc95c5, 95 },
    { 0x6e8c95c6, 75 },
    { 0x639c95c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c95cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7c95d6, 81 },
    { 0, 0 },
    { 0xb07c95d8, 61 },
    { 0, 0 },
    { 0xdcac95da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4c95de, 85 },
    { 0, 0 },
    { 0x86bc95e0, 76 },
    { 0, 0 },
    { 0xcbbc95e2, 34 },
    { 0x9ebc95e3, 87 },
    { 0xbe5c95e3, 75 },
    { 0x868c95e4, 66 },
    { 0x64cc95e6, 88 },
    { 0x568c95e7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c95ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774c95ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c95f2, 33 },
    { 0xe5dc95f2, 84 },
    { 0, 0 },
    { 0x90ec95f5, 81 },
    { 0x7c5c95f5, 98 },
    { 0, 0 },
    { 0x3cac95f8, 33 },
    { 0, 0 },
    { 0x6edc95fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9c95fe, 67 },
    { 0xcf5c95ff, 61 },
    { 0xc58c9600, 90 },
    { 0x878c9601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3c9608, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32c960b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15c960e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc9611, 91 },
    { 0xeebc9612, 90 },
    { 0x8c7c9613, 34 },
    { 0x7fdc9613, 84 },
    { 0x1a1c9615, 78 },
    { 0x10bc9612, 61 },
    { 0, 0 },
    { 0xa99c9618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5c961b, 23 },
    { 0xbbec961c, 66 },
    { 0xf3ac961d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x314c9620, 90 },
    { 0x3aec9621, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407c962d, 33 },
    { 0x134c962d, 34 },
    { 0xc19c962f, 98 },
    { 0, 0 },
    { 0xf9dc9631, 33 },
    { 0xbe9c9632, 90 },
    { 0x51c9631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7c9637, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c963a, 33 },
    { 0x38ac963a, 55 },
    { 0, 0 },
    { 0x387c963d, 33 },
    { 0xb1ec963e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96c9642, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc9645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc9648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecac964e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2c9658, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bc9661, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6c9668, 87 },
    { 0x1ddc9668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c9670, 80 },
    { 0x330c9671, 43 },
    { 0xd9fc9670, 95 },
    { 0xa47c9673, 90 },
    { 0, 0 },
    { 0x968c9675, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cc9679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x255c967c, 55 },
    { 0xd28c967d, 52 },
    { 0x18ac967d, 33 },
    { 0xa4fc967f, 88 },
    { 0xe20c967f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cc9683, 98 },
    { 0, 0 },
    { 0x1dc9685, 55 },
    { 0, 0 },
    { 0xb43c9687, 43 },
    { 0xf7c9687, 100 },
    { 0, 0 },
    { 0xfa6c968a, 67 },
    { 0, 0 },
    { 0xf18c968c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c968f, 90 },
    { 0x122c9690, 100 },
    { 0x9ecc9690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc9697, 66 },
    { 0x94cc9698, 17 },
    { 0x800c9699, 69 },
    { 0x885c969a, 34 },
    { 0x2d2c969b, 87 },
    { 0, 0 },
    { 0x733c969d, 43 },
    { 0x21ec969e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2c96a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2c96a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c96ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc96b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc96bf, 33 },
    { 0x8fac96c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227c96c7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c96cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fc96cf, 33 },
    { 0, 0 },
    { 0xc5ec96d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c96e6, 92 },
    { 0, 0 },
    { 0x312c96e8, 52 },
    { 0x408c96e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c96ee, 46 },
    { 0, 0 },
    { 0xa74c96f0, 40 },
    { 0xfd5c96f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x653c96f4, 98 },
    { 0x7edc96f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479c96fa, 55 },
    { 0x815c96fb, 33 },
    { 0x17bc96fc, 80 },
    { 0x4e8c96fd, 78 },
    { 0x425c96fd, 61 },
    { 0x68dc96fd, 33 },
    { 0xe26c96fd, 23 },
    { 0xb1ec9701, 84 },
    { 0x3bbc9702, 23 },
    { 0x42cc9700, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c9707, 77 },
    { 0xa1c9707, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc970e, 98 },
    { 0xaaec970f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c9719, 61 },
    { 0xedac9719, 90 },
    { 0, 0 },
    { 0xdbfc971c, 90 },
    { 0, 0 },
    { 0x66dc971e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836c9724, 61 },
    { 0xb3dc9724, 90 },
    { 0x20ac9726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c972c, 33 },
    { 0, 0 },
    { 0xc19c972e, 33 },
    { 0, 0 },
    { 0x66ac9730, 81 },
    { 0x72ec9730, 61 },
    { 0x850c9732, 93 },
    { 0x335c9730, 38 },
    { 0, 0 },
    { 0x6cec9735, 23 },
    { 0, 0 },
    { 0x5a5c9737, 34 },
    { 0x1fcc9738, 90 },
    { 0, 0 },
    { 0x548c973a, 93 },
    { 0x205c973a, 55 },
    { 0x98fc973a, 33 },
    { 0x4fc973a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bc9741, 43 },
    { 0x7efc9741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24c9745, 88 },
    { 0, 0 },
    { 0x598c9747, 87 },
    { 0x596c9747, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc974d, 33 },
    { 0, 0 },
    { 0x39fc974f, 76 },
    { 0x58ac9750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac9753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ec9758, 78 },
    { 0xb05c9758, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9c9761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4c9768, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ac9773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2c9777, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fc977b, 76 },
    { 0xcb1c977c, 95 },
    { 0, 0 },
    { 0xcc6c977e, 61 },
    { 0x541c977f, 55 },
    { 0, 0 },
    { 0xe0c9781, 85 },
    { 0x429c9782, 90 },
    { 0xa6cc9781, 75 },
    { 0xd79c9782, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c9787, 68 },
    { 0xdcec9788, 23 },
    { 0, 0 },
    { 0x9a4c978a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c978d, 90 },
    { 0x90fc978e, 88 },
    { 0x21bc978f, 92 },
    { 0x48bc9790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c9797, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc979a, 52 },
    { 0x3b6c979a, 43 },
    { 0x1e7c979c, 95 },
    { 0xdc1c979d, 75 },
    { 0x7b0c979e, 33 },
    { 0, 0 },
    { 0xb67c97a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640c97a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2c97aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fc97ad, 61 },
    { 0x2e0c97ad, 23 },
    { 0, 0 },
    { 0xf6bc97b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec97b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307c97b9, 55 },
    { 0x3e9c97ba, 90 },
    { 0xe54c97ba, 33 },
    { 0x669c97bc, 33 },
    { 0x9afc97bd, 23 },
    { 0x2e0c97bc, 33 },
    { 0x1eec97bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cc97c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209c97c8, 76 },
    { 0x10fc97c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62c97d1, 76 },
    { 0xa88c97d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c97d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bc97db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50c97df, 66 },
    { 0xd30c97e0, 90 },
    { 0xa0c97e1, 99 },
    { 0x7a6c97e1, 90 },
    { 0x142c97df, 33 },
    { 0xc75c97e4, 97 },
    { 0x33dc97e5, 55 },
    { 0xc9ec97e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dc97eb, 23 },
    { 0x39ac97ec, 98 },
    { 0xd4cc97eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c97f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x152c97f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c97f9, 23 },
    { 0, 0 },
    { 0x3b5c97fb, 78 },
    { 0xfb0c97fc, 90 },
    { 0x140c97fd, 34 },
    { 0xb77c97fd, 61 },
    { 0xf1bc97fc, 23 },
    { 0x404c97fe, 34 },
    { 0, 0 },
    { 0x76ac9802, 33 },
    { 0x3b2c9803, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac980a, 34 },
    { 0x72ec980a, 87 },
    { 0xd6c980c, 46 },
    { 0x51dc980d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ec981b, 100 },
    { 0xcc981c, 33 },
    { 0x870c981b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933c9821, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7c9824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc9827, 17 },
    { 0xc2bc9828, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c982b, 91 },
    { 0x63ec982c, 33 },
    { 0, 0 },
    { 0x243c982e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c9836, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x235c9839, 23 },
    { 0xbc1c983a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecc983d, 43 },
    { 0x764c983e, 34 },
    { 0xec5c983f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0c9844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549c9847, 84 },
    { 0x651c9848, 33 },
    { 0, 0 },
    { 0xf59c984a, 95 },
    { 0xc38c984a, 95 },
    { 0x5ccc984c, 33 },
    { 0x6b3c984b, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bc9853, 95 },
    { 0x122c9854, 43 },
    { 0x476c9854, 61 },
    { 0x7e9c9856, 95 },
    { 0, 0 },
    { 0x5a3c9858, 95 },
    { 0x5cec9858, 85 },
    { 0, 0 },
    { 0xb2ec985b, 68 },
    { 0x6d9c985c, 77 },
    { 0x629c985c, 78 },
    { 0xd2ec985b, 23 },
    { 0xa3cc985f, 76 },
    { 0x43dc985b, 75 },
    { 0x257c9861, 87 },
    { 0x614c9862, 95 },
    { 0xc45c9860, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc9873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x879c9876, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc987b, 84 },
    { 0x5b4c987c, 98 },
    { 0x17bc987d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c9882, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec9887, 46 },
    { 0x5d7c9887, 52 },
    { 0x3b3c9889, 33 },
    { 0xa3dc9888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafac988d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aac9892, 33 },
    { 0x599c9892, 61 },
    { 0x7e2c9894, 100 },
    { 0x367c9892, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3c989a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcc989d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc98a2, 90 },
    { 0x8d3c98a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62c98a7, 33 },
    { 0xdcac98a8, 66 },
    { 0x97fc98a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c98b4, 88 },
    { 0, 0 },
    { 0x169c98b6, 90 },
    { 0, 0 },
    { 0xe42c98b8, 76 },
    { 0x67dc98b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69c98bd, 90 },
    { 0xb1ec98be, 61 },
    { 0x793c98bf, 43 },
    { 0x820c98c0, 33 },
    { 0, 0 },
    { 0x482c98c2, 90 },
    { 0xc48c98c2, 34 },
    { 0x72fc98c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39c98c7, 61 },
    { 0, 0 },
    { 0xe1cc98c9, 33 },
    { 0x1a3c98c9, 52 },
    { 0x1d9c98cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c98d1, 90 },
    { 0x5dbc98d2, 81 },
    { 0x922c98d3, 91 },
    { 0, 0 },
    { 0x2fcc98d5, 61 },
    { 0x54ec98d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x598c98d9, 33 },
    { 0x5c8c98d9, 76 },
    { 0x7a2c98db, 34 },
    { 0xd08c98db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c98e1, 17 },
    { 0xb6dc98e2, 95 },
    { 0xbc4c98e3, 90 },
    { 0xfc8c98e4, 77 },
    { 0x358c98e5, 61 },
    { 0xeaec98e3, 93 },
    { 0x600c98e3, 84 },
    { 0x972c98e8, 67 },
    { 0x4e3c98e4, 33 },
    { 0xb2dc98ea, 90 },
    { 0xa4fc98e9, 95 },
    { 0xbe1c98e9, 98 },
    { 0x9fdc98e2, 85 },
    { 0xd43c98ee, 98 },
    { 0x9c6c98ef, 33 },
    { 0xfa5c98ec, 33 },
    { 0, 0 },
    { 0x8e2c98f2, 61 },
    { 0, 0 },
    { 0x932c98f4, 98 },
    { 0xafac98f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cc98fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739c9900, 43 },
    { 0, 0 },
    { 0x690c9902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9906, 52 },
    { 0x1d4c9907, 76 },
    { 0xae3c9908, 23 },
    { 0xefac9907, 95 },
    { 0x501c990a, 98 },
    { 0x7b1c990b, 23 },
    { 0x292c990a, 61 },
    { 0x64bc990d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724c9911, 95 },
    { 0x483c9912, 65 },
    { 0, 0 },
    { 0xed4c9914, 99 },
    { 0x2d2c9915, 65 },
    { 0x60ac9916, 65 },
    { 0x7eec9917, 23 },
    { 0xfc4c9915, 33 },
    { 0xb14c9915, 85 },
    { 0xb46c9916, 34 },
    { 0xff7c991b, 61 },
    { 0xfb3c991c, 33 },
    { 0xb3fc9917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ec9922, 61 },
    { 0x5a3c9923, 100 },
    { 0, 0 },
    { 0x9a0c9925, 55 },
    { 0xe5fc9926, 56 },
    { 0x9f2c9927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837c992b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cc992f, 90 },
    { 0x305c9930, 17 },
    { 0xb8bc9931, 61 },
    { 0xdbbc9932, 99 },
    { 0x486c9932, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dc993c, 98 },
    { 0, 0 },
    { 0x6ffc993e, 95 },
    { 0x8fc993e, 33 },
    { 0x2dec993e, 17 },
    { 0, 0 },
    { 0x424c9942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ac9945, 99 },
    { 0, 0 },
    { 0xcd1c9947, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1c994d, 38 },
    { 0xf40c994e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574c9954, 23 },
    { 0x332c9954, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x657c9958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ec995b, 23 },
    { 0x48fc995c, 90 },
    { 0x388c995b, 61 },
    { 0x584c995c, 52 },
    { 0x674c995f, 90 },
    { 0x28ac995f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c9964, 33 },
    { 0, 0 },
    { 0xa14c9966, 61 },
    { 0x5dac9967, 43 },
    { 0xd9c9968, 95 },
    { 0x223c9967, 23 },
    { 0, 0 },
    { 0x781c996b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734c9971, 78 },
    { 0, 0 },
    { 0x497c9973, 52 },
    { 0xca2c9973, 87 },
    { 0x6aec9974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4c9979, 33 },
    { 0xb57c9979, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805c997e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc9981, 33 },
    { 0x63ac9981, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddc9985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bc9988, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c998d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7c9990, 33 },
    { 0xd37c9991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c9995, 55 },
    { 0x4ac9996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fc999a, 43 },
    { 0x3aec999b, 61 },
    { 0x9e4c999b, 33 },
    { 0xd66c999d, 90 },
    { 0xe7cc999e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c99a4, 61 },
    { 0xc1ac99a5, 26 },
    { 0x22bc99a6, 100 },
    { 0xd2ac99a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c99ad, 84 },
    { 0x955c99ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c99b1, 61 },
    { 0x8a2c99b2, 98 },
    { 0, 0 },
    { 0x52bc99b4, 76 },
    { 0, 0 },
    { 0x1f5c99b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c99bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06c99be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3c99c2, 90 },
    { 0x919c99c3, 40 },
    { 0, 0 },
    { 0x2f9c99c5, 78 },
    { 0x155c99c6, 77 },
    { 0xf20c99c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dc99ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c99ce, 93 },
    { 0xf95c99cf, 98 },
    { 0xd80c99d0, 90 },
    { 0x5a6c99d0, 98 },
    { 0xc90c99d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x755c99d5, 93 },
    { 0, 0 },
    { 0xae3c99d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c99de, 90 },
    { 0x823c99df, 90 },
    { 0, 0 },
    { 0x438c99e1, 17 },
    { 0x10ac99e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440c99e7, 98 },
    { 0, 0 },
    { 0x386c99e9, 84 },
    { 0, 0 },
    { 0x639c99eb, 33 },
    { 0, 0 },
    { 0xdd9c99ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c99f6, 66 },
    { 0x7b1c99f6, 95 },
    { 0x113c99f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1c99fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c9a00, 66 },
    { 0x8a0c9a00, 43 },
    { 0x86c9a02, 33 },
    { 0xd27c9a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880c9a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc9a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c9a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bc9a20, 61 },
    { 0x4c1c9a21, 87 },
    { 0x6b1c9a22, 61 },
    { 0x2fac9a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac9a26, 23 },
    { 0x693c9a26, 55 },
    { 0xbe4c9a26, 55 },
    { 0x87cc9a29, 90 },
    { 0, 0 },
    { 0xfb8c9a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c9a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec9a36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15c9a3e, 76 },
    { 0x8d4c9a3f, 98 },
    { 0x53ac9a3f, 23 },
    { 0x162c9a41, 90 },
    { 0xd5cc9a42, 33 },
    { 0x5bcc9a43, 87 },
    { 0x43ec9a43, 34 },
    { 0x2d4c9a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac9a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afc9a50, 91 },
    { 0, 0 },
    { 0xdb2c9a52, 98 },
    { 0, 0 },
    { 0x9d7c9a54, 55 },
    { 0x52fc9a55, 34 },
    { 0, 0 },
    { 0x67ac9a57, 61 },
    { 0x542c9a58, 67 },
    { 0x779c9a57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c9a5e, 33 },
    { 0x289c9a5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fc9a6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c9a6f, 33 },
    { 0x97dc9a70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267c9a77, 33 },
    { 0xbdfc9a78, 61 },
    { 0xdacc9a77, 88 },
    { 0, 0 },
    { 0x546c9a7b, 90 },
    { 0, 0 },
    { 0x5e9c9a7d, 43 },
    { 0x6f3c9a7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc9a82, 66 },
    { 0xb3dc9a82, 61 },
    { 0, 0 },
    { 0x611c9a85, 98 },
    { 0, 0 },
    { 0x18ec9a87, 67 },
    { 0, 0 },
    { 0x5a0c9a89, 23 },
    { 0, 0 },
    { 0x1d3c9a8b, 98 },
    { 0xb83c9a8c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c9a93, 65 },
    { 0xa58c9a94, 33 },
    { 0x24fc9a95, 61 },
    { 0xf2dc9a96, 34 },
    { 0x9acc9a97, 81 },
    { 0xdbdc9a93, 33 },
    { 0x1afc9a99, 95 },
    { 0x5a7c9a9a, 33 },
    { 0x470c9a98, 23 },
    { 0x1b0c9a9c, 99 },
    { 0x520c9a9d, 95 },
    { 0x7c7c9a9c, 95 },
    { 0xbabc9a9f, 90 },
    { 0, 0 },
    { 0x7c5c9aa1, 90 },
    { 0x58c9aa1, 90 },
    { 0xff4c9aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cc9aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc9aa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7c9aae, 98 },
    { 0, 0 },
    { 0x2e5c9ab0, 59 },
    { 0, 0 },
    { 0x3b0c9ab2, 97 },
    { 0x929c9ab3, 95 },
    { 0xf87c9ab4, 96 },
    { 0xb0cc9ab3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc9ab8, 34 },
    { 0xce5c9ab9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c9ac0, 87 },
    { 0, 0 },
    { 0x5e6c9ac2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8c9ac8, 33 },
    { 0x425c9ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1c9acc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ec9ad1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec9ad4, 34 },
    { 0, 0 },
    { 0x207c9ad6, 23 },
    { 0x8afc9ad6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafc9aec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c9af0, 33 },
    { 0xda2c9af1, 90 },
    { 0, 0 },
    { 0x389c9af3, 68 },
    { 0x3f2c9af4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dc9af9, 90 },
    { 0x80dc9afa, 98 },
    { 0x435c9afb, 34 },
    { 0, 0 },
    { 0x82fc9afd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadc9b01, 23 },
    { 0x1ddc9b02, 95 },
    { 0xc54c9b01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c9b07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc9b0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c9b0d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661c9b11, 87 },
    { 0xc2dc9b12, 78 },
    { 0x271c9b11, 61 },
    { 0, 0 },
    { 0xd18c9b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6c9b19, 75 },
    { 0, 0 },
    { 0xe10c9b1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcc9b1f, 33 },
    { 0, 0 },
    { 0x82dc9b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c9b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x644c9b27, 95 },
    { 0, 0 },
    { 0x717c9b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbc9b2e, 90 },
    { 0, 0 },
    { 0xdb7c9b30, 23 },
    { 0, 0 },
    { 0x440c9b32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18c9b39, 34 },
    { 0xe51c9b3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c9b3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870c9b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c9b48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ec9b4b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8c9b4e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c9b51, 76 },
    { 0, 0 },
    { 0xc5bc9b53, 68 },
    { 0x8d4c9b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c9b59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c9b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22c9b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c9b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445c9b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc9b85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c9b8b, 33 },
    { 0, 0 },
    { 0x586c9b8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c9b90, 33 },
    { 0x3c0c9b90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6c9b94, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbc9b97, 76 },
    { 0, 0 },
    { 0x424c9b99, 93 },
    { 0, 0 },
    { 0xf5c9b9b, 55 },
    { 0x6b6c9b9c, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c9ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc9ba4, 33 },
    { 0x348c9ba5, 55 },
    { 0x2ddc9ba6, 87 },
    { 0x3f5c9ba5, 98 },
    { 0xcb0c9ba5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c9bb1, 90 },
    { 0x81c9bb2, 90 },
    { 0, 0 },
    { 0xf54c9bb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x437c9bb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac9bbb, 52 },
    { 0xa6cc9bbc, 33 },
    { 0x4dbc9bbc, 75 },
    { 0x39bc9bbe, 90 },
    { 0x88fc9bbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc9bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc9bc8, 40 },
    { 0xd02c9bc9, 76 },
    { 0, 0 },
    { 0x135c9bcb, 76 },
    { 0, 0 },
    { 0x8c0c9bcd, 75 },
    { 0xde6c9bcd, 61 },
    { 0xe48c9bcd, 81 },
    { 0, 0 },
    { 0x2abc9bd1, 34 },
    { 0x7cfc9bd2, 87 },
    { 0, 0 },
    { 0xe11c9bd4, 94 },
    { 0xac1c9bd5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206c9bdb, 61 },
    { 0, 0 },
    { 0x69cc9bdd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cc9be2, 64 },
    { 0xdf5c9be2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446c9be7, 90 },
    { 0x594c9be8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09c9bef, 23 },
    { 0x238c9bf0, 43 },
    { 0, 0 },
    { 0x144c9bf2, 95 },
    { 0x548c9bf3, 75 },
    { 0x5bdc9bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ec9bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15c9c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c9c05, 100 },
    { 0, 0 },
    { 0xf45c9c07, 87 },
    { 0x54dc9c08, 23 },
    { 0, 0 },
    { 0x999c9c0a, 90 },
    { 0xb25c9c0b, 33 },
    { 0, 0 },
    { 0xe91c9c0d, 91 },
    { 0x983c9c0e, 91 },
    { 0, 0 },
    { 0x194c9c10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15c9c17, 90 },
    { 0, 0 },
    { 0x681c9c19, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c9c1e, 98 },
    { 0xdfbc9c1e, 55 },
    { 0x957c9c20, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc9c23, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c9c29, 98 },
    { 0x47ec9c2a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cc9c2e, 33 },
    { 0x810c9c2e, 23 },
    { 0, 0 },
    { 0x89ac9c31, 94 },
    { 0, 0 },
    { 0xccbc9c33, 90 },
    { 0x15dc9c34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c9c38, 90 },
    { 0xe49c9c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6c9c3e, 33 },
    { 0x113c9c3f, 46 },
    { 0xfdbc9c40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc9c43, 43 },
    { 0xde4c9c44, 90 },
    { 0x9aec9c44, 23 },
    { 0, 0 },
    { 0x6eec9c47, 23 },
    { 0x413c9c47, 52 },
    { 0x571c9c49, 90 },
    { 0x254c9c4a, 87 },
    { 0xa58c9c47, 33 },
    { 0, 0 },
    { 0x34cc9c4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc9c54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c9c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6c9c5c, 47 },
    { 0, 0 },
    { 0xc21c9c5e, 61 },
    { 0, 0 },
    { 0x916c9c60, 85 },
    { 0x3fec9c61, 98 },
    { 0x61c9c62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946c9c69, 68 },
    { 0, 0 },
    { 0xcb8c9c6b, 68 },
    { 0xd4ac9c6b, 85 },
    { 0xed6c9c6b, 34 },
    { 0x93c9c6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acc9c71, 90 },
    { 0x267c9c71, 61 },
    { 0xb59c9c71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c9c76, 33 },
    { 0x8cdc9c77, 93 },
    { 0x17ec9c78, 76 },
    { 0x805c9c77, 61 },
    { 0, 0 },
    { 0xcedc9c7b, 100 },
    { 0, 0 },
    { 0x466c9c7d, 93 },
    { 0x71c9c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bc9c81, 64 },
    { 0x718c9c82, 33 },
    { 0, 0 },
    { 0xbe8c9c84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc9c88, 23 },
    { 0xac2c9c88, 33 },
    { 0, 0 },
    { 0x980c9c8b, 95 },
    { 0xe89c9c8c, 100 },
    { 0, 0 },
    { 0xbc0c9c8e, 90 },
    { 0x6cfc9c8e, 87 },
    { 0, 0 },
    { 0x166c9c91, 34 },
    { 0x58ec9c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c9c98, 61 },
    { 0x3b6c9c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8c9c9c, 55 },
    { 0x22ac9c9d, 23 },
    { 0xa8c9c9e, 55 },
    { 0x53fc9c9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64c9ca2, 33 },
    { 0x9e2c9ca3, 23 },
    { 0x7aec9ca2, 76 },
    { 0x3eac9ca5, 95 },
    { 0xa26c9ca5, 90 },
    { 0x22fc9ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fc9caa, 100 },
    { 0, 0 },
    { 0x2b2c9cac, 69 },
    { 0xcbcc9cad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8c9cb0, 90 },
    { 0x42cc9cb1, 8 },
    { 0xf58c9cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511c9cb8, 33 },
    { 0xdb8c9cb8, 23 },
    { 0, 0 },
    { 0x2cbc9cbb, 93 },
    { 0xa7dc9cbc, 91 },
    { 0, 0 },
    { 0x4ccc9cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x634c9cc1, 83 },
    { 0xd72c9cc1, 61 },
    { 0, 0 },
    { 0xba2c9cc4, 100 },
    { 0x8dec9cc5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ec9cca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48c9ccd, 90 },
    { 0, 0 },
    { 0x916c9ccf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c9cd2, 33 },
    { 0x215c9cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836c9cd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c9cda, 90 },
    { 0, 0 },
    { 0xe8bc9cdc, 90 },
    { 0, 0 },
    { 0xf63c9cde, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ec9ce2, 55 },
    { 0, 0 },
    { 0xb93c9ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffc9cea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0c9cee, 95 },
    { 0xd82c9cef, 76 },
    { 0xfd8c9cf0, 81 },
    { 0x47bc9cf1, 95 },
    { 0x35fc9cf2, 33 },
    { 0x490c9cf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc9cf6, 46 },
    { 0, 0 },
    { 0xeec9cf8, 61 },
    { 0, 0 },
    { 0x4b0c9cfa, 65 },
    { 0x7edc9cfa, 98 },
    { 0x929c9cfc, 90 },
    { 0x771c9cfd, 78 },
    { 0xd3ec9cfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c9d05, 46 },
    { 0, 0 },
    { 0xb53c9d07, 98 },
    { 0xf42c9d08, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dc9d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1c9d10, 90 },
    { 0x844c9d11, 80 },
    { 0x766c9d12, 61 },
    { 0xbe7c9d13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec9d17, 95 },
    { 0x84ac9d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc9d1d, 33 },
    { 0x963c9d1e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc9d21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cc9d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafc9d2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c9d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaac9d31, 66 },
    { 0x158c9d32, 33 },
    { 0xce8c9d33, 98 },
    { 0xb6c9d33, 90 },
    { 0, 0 },
    { 0x7f6c9d36, 61 },
    { 0x40ec9d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c9d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7c9d46, 84 },
    { 0, 0 },
    { 0x5b6c9d48, 84 },
    { 0x3e5c9d48, 61 },
    { 0x88ac9d4a, 61 },
    { 0xc38c9d48, 23 },
    { 0x1aec9d4b, 90 },
    { 0x422c9d4b, 98 },
    { 0xea7c9d4e, 78 },
    { 0xecdc9d4b, 27 },
    { 0, 0 },
    { 0x2e8c9d51, 98 },
    { 0, 0 },
    { 0xc01c9d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615c9d5a, 33 },
    { 0xdc1c9d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c9d60, 78 },
    { 0x82bc9d60, 33 },
    { 0x206c9d61, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac9d69, 90 },
    { 0x9acc9d69, 100 },
    { 0xf4ac9d6b, 33 },
    { 0, 0 },
    { 0x15bc9d6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c9d71, 95 },
    { 0x55ac9d72, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c9d76, 85 },
    { 0, 0 },
    { 0x54cc9d78, 90 },
    { 0x90fc9d79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfc9d82, 43 },
    { 0xfc8c9d82, 95 },
    { 0xa10c9d82, 91 },
    { 0xb6ac9d83, 23 },
    { 0x5dbc9d86, 98 },
    { 0xacac9d82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c9d8a, 100 },
    { 0x58ec9d8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c9d8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2c9d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272c9d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x807c9da0, 23 },
    { 0x774c9da1, 23 },
    { 0, 0 },
    { 0xd80c9da3, 33 },
    { 0x94ec9da3, 55 },
    { 0x3c5c9da4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0c9daa, 90 },
    { 0, 0 },
    { 0x171c9dac, 95 },
    { 0x723c9dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71c9db2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68c9db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37c9dbb, 90 },
    { 0, 0 },
    { 0xa40c9dbd, 33 },
    { 0, 0 },
    { 0xc5dc9dbf, 25 },
    { 0, 0 },
    { 0xeefc9dc1, 46 },
    { 0x2a1c9dc2, 85 },
    { 0x9fdc9dc2, 61 },
    { 0x74ec9dc3, 90 },
    { 0xfdc9dc2, 52 },
    { 0x338c9dc6, 55 },
    { 0xa09c9dc7, 34 },
    { 0, 0 },
    { 0x105c9dc9, 61 },
    { 0xf4cc9dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec9dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc9dd1, 98 },
    { 0xd4bc9dd2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c9dd8, 98 },
    { 0, 0 },
    { 0x702c9dda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9ddf, 76 },
    { 0x30dc9de0, 34 },
    { 0xdbbc9de0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0c9de7, 95 },
    { 0, 0 },
    { 0x6d2c9de9, 93 },
    { 0x749c9de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadc9def, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c9df5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ec9df8, 43 },
    { 0xc52c9df8, 33 },
    { 0xdec9dfa, 98 },
    { 0x825c9dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c9e00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc9e04, 23 },
    { 0, 0 },
    { 0xb53c9e06, 33 },
    { 0x291c9e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebc9e0a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648c9e14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc9e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0c9e1a, 33 },
    { 0x388c9e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fc9e1e, 33 },
    { 0xcc4c9e1e, 67 },
    { 0, 0 },
    { 0x6e1c9e21, 34 },
    { 0xd5cc9e22, 91 },
    { 0, 0 },
    { 0x15c9e24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c9e27, 65 },
    { 0xa76c9e28, 17 },
    { 0xe90c9e28, 75 },
    { 0xfe8c9e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abc9e2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbc9e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dc9e34, 75 },
    { 0xf2bc9e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c9e3d, 23 },
    { 0xebc9e3d, 90 },
    { 0xba6c9e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298c9e44, 75 },
    { 0, 0 },
    { 0x430c9e46, 95 },
    { 0x24ec9e46, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c9e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fc9e50, 93 },
    { 0, 0 },
    { 0xcb7c9e52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc9e55, 76 },
    { 0x333c9e56, 78 },
    { 0, 0 },
    { 0xe85c9e58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c9e5c, 91 },
    { 0x2d5c9e5d, 33 },
    { 0, 0 },
    { 0x668c9e5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fc9e65, 95 },
    { 0x97bc9e66, 90 },
    { 0x160c9e67, 90 },
    { 0x585c9e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2c9e6d, 88 },
    { 0xcf6c9e6d, 33 },
    { 0x90ac9e6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ec9e72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c9e76, 61 },
    { 0xf11c9e77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c9e7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ec9e7e, 90 },
    { 0xa87c9e7f, 76 },
    { 0, 0 },
    { 0xf73c9e81, 43 },
    { 0, 0 },
    { 0x1e0c9e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac9e86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc9e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0c9e8f, 98 },
    { 0xf4bc9e90, 78 },
    { 0xbefc9e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7c9e94, 76 },
    { 0xf7cc9e95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156c9e9a, 95 },
    { 0, 0 },
    { 0x454c9e9c, 76 },
    { 0x84dc9e9d, 76 },
    { 0x1dc9e9e, 34 },
    { 0x447c9e9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815c9ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc9ea7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344c9ead, 51 },
    { 0x3e3c9eae, 95 },
    { 0x256c9eaf, 98 },
    { 0xa21c9ead, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x918c9eb3, 46 },
    { 0xe02c9eb4, 85 },
    { 0x63fc9eb5, 83 },
    { 0x439c9eb6, 76 },
    { 0xdac9eb5, 90 },
    { 0x302c9eb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7c9ec4, 98 },
    { 0x7fcc9ec4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc9ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc9ecc, 90 },
    { 0x484c9ecd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdc9ed0, 33 },
    { 0x4e7c9ed1, 52 },
    { 0, 0 },
    { 0x42ec9ed3, 55 },
    { 0x25bc9ed3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ac9edb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c9ede, 33 },
    { 0x3fc9ede, 38 },
    { 0x3b5c9ede, 76 },
    { 0x509c9ee1, 90 },
    { 0xa9fc9ee2, 34 },
    { 0x659c9ee1, 33 },
    { 0xa88c9ee1, 90 },
    { 0x341c9ee1, 33 },
    { 0xb3c9ee6, 98 },
    { 0x657c9ee4, 38 },
    { 0xcdec9ee8, 90 },
    { 0, 0 },
    { 0x535c9eea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c9ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c9efa, 33 },
    { 0x9f0c9efb, 95 },
    { 0, 0 },
    { 0xd13c9efd, 61 },
    { 0, 0 },
    { 0x465c9eff, 90 },
    { 0x26ac9eff, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ac9f03, 93 },
    { 0x541c9f04, 43 },
    { 0xaa0c9f05, 61 },
    { 0x6b0c9f05, 75 },
    { 0x3d2c9f07, 67 },
    { 0, 0 },
    { 0xa81c9f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654c9f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c9f17, 61 },
    { 0x9e2c9f17, 53 },
    { 0, 0 },
    { 0xff4c9f1a, 95 },
    { 0xdecc9f1a, 33 },
    { 0xaa6c9f1c, 34 },
    { 0x98ac9f1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c9f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbc9f24, 61 },
    { 0, 0 },
    { 0x5c8c9f26, 80 },
    { 0x228c9f27, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c9f3a, 36 },
    { 0, 0 },
    { 0x8bbc9f3c, 55 },
    { 0xc72c9f3c, 95 },
    { 0x355c9f3d, 33 },
    { 0xab2c9f3e, 34 },
    { 0x37fc9f40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bc9f47, 95 },
    { 0x163c9f48, 61 },
    { 0, 0 },
    { 0x96cc9f4a, 95 },
    { 0xc63c9f4a, 98 },
    { 0, 0 },
    { 0xd4ec9f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c9f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebc9f56, 84 },
    { 0xe8ac9f57, 90 },
    { 0x5bbc9f58, 87 },
    { 0xb60c9f58, 55 },
    { 0xf47c9f59, 77 },
    { 0x453c9f59, 33 },
    { 0xfbdc9f5a, 43 },
    { 0xcefc9f5b, 33 },
    { 0, 0 },
    { 0x22cc9f5f, 90 },
    { 0xc93c9f60, 66 },
    { 0x567c9f60, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc9f64, 67 },
    { 0xdc4c9f64, 33 },
    { 0, 0 },
    { 0x4e8c9f67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bc9f6d, 33 },
    { 0, 0 },
    { 0x44dc9f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x229c9f72, 78 },
    { 0xb8fc9f73, 87 },
    { 0xee4c9f73, 23 },
    { 0xc75c9f75, 85 },
    { 0xde0c9f76, 100 },
    { 0x265c9f76, 23 },
    { 0, 0 },
    { 0xd89c9f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c9f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecc9f85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec9f8e, 77 },
    { 0x8f1c9f8e, 99 },
    { 0xc88c9f90, 23 },
    { 0x2cfc9f90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55c9f95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c9f9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23c9f9d, 90 },
    { 0xf1fc9f9e, 61 },
    { 0, 0 },
    { 0x8cec9fa0, 90 },
    { 0, 0 },
    { 0x719c9fa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c9fae, 40 },
    { 0x40ec9fae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ec9fb2, 95 },
    { 0xc41c9fb3, 90 },
    { 0xd1fc9fb4, 98 },
    { 0x626c9fb5, 55 },
    { 0, 0 },
    { 0x7e1c9fb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82c9fba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c9fc1, 34 },
    { 0xefcc9fc2, 33 },
    { 0x88ec9fc2, 91 },
    { 0xfc7c9fc4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72c9fcb, 91 },
    { 0xeb3c9fcc, 88 },
    { 0x31c9fcc, 43 },
    { 0x55cc9fce, 100 },
    { 0xd46c9fcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c9fd2, 95 },
    { 0, 0 },
    { 0xad0c9fd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9fd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9c9fe0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255c9fe9, 100 },
    { 0x870c9fea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x622c9fed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2c9ff0, 43 },
    { 0, 0 },
    { 0xd07c9ff2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330c9ff9, 36 },
    { 0x63ac9ff9, 76 },
    { 0xa7ac9ff9, 34 },
    { 0, 0 },
    { 0x5b3c9ffd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398ca001, 95 },
    { 0xdc3ca002, 75 },
    { 0xfa3ca003, 61 },
    { 0x9d5ca003, 93 },
    { 0, 0 },
    { 0xc9ca006, 93 },
    { 0, 0 },
    { 0x9ecca008, 46 },
    { 0, 0 },
    { 0x438ca00a, 23 },
    { 0xd27ca00b, 52 },
    { 0xafca00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ca00f, 78 },
    { 0, 0 },
    { 0xb69ca011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eca016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15ca019, 34 },
    { 0, 0 },
    { 0xf62ca01b, 23 },
    { 0xd2fca01c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02ca01f, 95 },
    { 0x55cca020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58ca028, 33 },
    { 0, 0 },
    { 0x164ca02a, 95 },
    { 0x807ca02b, 99 },
    { 0x2fcca02b, 77 },
    { 0x98cca02b, 33 },
    { 0x60cca02b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ca037, 90 },
    { 0x942ca037, 61 },
    { 0x1b0ca039, 100 },
    { 0x2d2ca039, 90 },
    { 0, 0 },
    { 0xa93ca03c, 33 },
    { 0x8deca03d, 85 },
    { 0x7fcca03e, 90 },
    { 0x307ca03d, 98 },
    { 0, 0 },
    { 0x8f1ca041, 66 },
    { 0x116ca042, 98 },
    { 0x704ca042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ca049, 43 },
    { 0xfe5ca04a, 23 },
    { 0, 0 },
    { 0x364ca04c, 88 },
    { 0x57ca04d, 61 },
    { 0xa0cca04e, 43 },
    { 0, 0 },
    { 0xecbca050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ca054, 85 },
    { 0x38dca055, 97 },
    { 0x2b2ca055, 23 },
    { 0xad3ca057, 61 },
    { 0, 0 },
    { 0x77eca059, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6ca066, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dca06a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca06f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bca074, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ca07c, 52 },
    { 0, 0 },
    { 0x850ca07e, 46 },
    { 0x1c3ca07f, 23 },
    { 0, 0 },
    { 0x4d8ca081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebca084, 98 },
    { 0x88dca085, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dca089, 40 },
    { 0x532ca08a, 33 },
    { 0xc6fca08b, 43 },
    { 0xc00ca08c, 61 },
    { 0xb6aca08c, 34 },
    { 0xe53ca08a, 46 },
    { 0xb34ca08f, 93 },
    { 0xaf4ca08e, 66 },
    { 0, 0 },
    { 0x3efca092, 87 },
    { 0xcf4ca093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eca097, 61 },
    { 0, 0 },
    { 0xf2bca099, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172ca09d, 90 },
    { 0x10fca09e, 66 },
    { 0, 0 },
    { 0xc1eca0a0, 98 },
    { 0xe2ca0a0, 84 },
    { 0x259ca0a2, 61 },
    { 0x6ebca0a3, 43 },
    { 0x39bca0a4, 33 },
    { 0x4c7ca0a5, 77 },
    { 0xd5eca0a6, 95 },
    { 0xc79ca0a2, 33 },
    { 0x186ca0a3, 43 },
    { 0xb85ca0a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57ca0ad, 77 },
    { 0, 0 },
    { 0x6eca0af, 76 },
    { 0x28fca0b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dca0b7, 78 },
    { 0x4bca0b7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4ca0bd, 84 },
    { 0xf68ca0bd, 98 },
    { 0, 0 },
    { 0x635ca0c0, 76 },
    { 0x52eca0c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaca0cc, 61 },
    { 0xdbcca0cc, 46 },
    { 0x4bbca0ce, 77 },
    { 0, 0 },
    { 0x7baca0d0, 95 },
    { 0x1f2ca0d0, 55 },
    { 0, 0 },
    { 0x45ca0d3, 90 },
    { 0x995ca0d4, 43 },
    { 0, 0 },
    { 0x8b5ca0d6, 23 },
    { 0, 0 },
    { 0xe2bca0d8, 95 },
    { 0, 0 },
    { 0xbf4ca0da, 95 },
    { 0xc3dca0db, 95 },
    { 0, 0 },
    { 0xac8ca0dd, 95 },
    { 0x220ca0de, 33 },
    { 0, 0 },
    { 0x155ca0e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcca0e3, 33 },
    { 0x230ca0e4, 84 },
    { 0, 0 },
    { 0xc94ca0e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877ca0ea, 65 },
    { 0xcafca0ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ca0ee, 91 },
    { 0, 0 },
    { 0xb40ca0f0, 34 },
    { 0xbfdca0f1, 61 },
    { 0, 0 },
    { 0x53eca0f3, 98 },
    { 0, 0 },
    { 0x539ca0f5, 90 },
    { 0xe6dca0f5, 99 },
    { 0xf1aca0f7, 55 },
    { 0xcdcca0f5, 98 },
    { 0, 0 },
    { 0xfebca0fa, 93 },
    { 0xa74ca0fb, 95 },
    { 0x6f2ca0fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cca0ff, 100 },
    { 0, 0 },
    { 0xa99ca101, 88 },
    { 0, 0 },
    { 0x11aca103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81ca106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ca109, 23 },
    { 0xf0fca10a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ca110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ca115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ca11a, 85 },
    { 0, 0 },
    { 0x3f5ca11c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228ca120, 23 },
    { 0, 0 },
    { 0xdd3ca122, 61 },
    { 0, 0 },
    { 0x44bca124, 95 },
    { 0x9b5ca124, 76 },
    { 0x3bfca126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ca129, 95 },
    { 0, 0 },
    { 0xbd1ca12b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ca12e, 88 },
    { 0, 0 },
    { 0x6e3ca130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ca135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafca139, 33 },
    { 0xe1ca13a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bca13e, 23 },
    { 0x3eeca13f, 33 },
    { 0x24eca13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0ca144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaca147, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5ca14a, 98 },
    { 0, 0 },
    { 0x5d0ca14c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ca14f, 93 },
    { 0x1fbca150, 61 },
    { 0x685ca150, 78 },
    { 0x5e8ca150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1ca157, 61 },
    { 0, 0 },
    { 0x9f0ca159, 55 },
    { 0x963ca159, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ca15e, 95 },
    { 0xa21ca15f, 98 },
    { 0x549ca160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aca164, 100 },
    { 0, 0 },
    { 0xd93ca166, 23 },
    { 0x9bdca167, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7ca16a, 61 },
    { 0x251ca16b, 61 },
    { 0x1c7ca16c, 90 },
    { 0, 0 },
    { 0x65eca16e, 33 },
    { 0xe34ca16f, 99 },
    { 0x4d9ca170, 95 },
    { 0x99bca171, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fca17b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfca17e, 95 },
    { 0, 0 },
    { 0xedcca180, 90 },
    { 0, 0 },
    { 0xf49ca182, 98 },
    { 0x7bdca182, 61 },
    { 0, 0 },
    { 0xa35ca185, 66 },
    { 0, 0 },
    { 0xcbeca187, 33 },
    { 0x3aca188, 78 },
    { 0xc5bca189, 95 },
    { 0x4d8ca18a, 90 },
    { 0xa56ca18a, 34 },
    { 0xcedca18c, 65 },
    { 0, 0 },
    { 0xba6ca18e, 23 },
    { 0xdf7ca18e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536ca194, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ca19b, 24 },
    { 0, 0 },
    { 0x957ca19d, 23 },
    { 0xb7dca19d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ca1a2, 91 },
    { 0x2a9ca1a3, 55 },
    { 0xddbca1a4, 33 },
    { 0x4eca1a4, 43 },
    { 0xf51ca1a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4ca1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfca1b3, 76 },
    { 0, 0 },
    { 0x356ca1b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b9, 88 },
    { 0xaf4ca1ba, 66 },
    { 0x28bca1bb, 99 },
    { 0x52eca1bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2ca1c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3ca1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653ca1c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3ca1cb, 90 },
    { 0x68ca1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ca1cf, 23 },
    { 0x723ca1d0, 61 },
    { 0xd9cca1d1, 99 },
    { 0, 0 },
    { 0x4f9ca1d3, 43 },
    { 0x832ca1d4, 98 },
    { 0x240ca1d4, 76 },
    { 0x91bca1d6, 46 },
    { 0x3b9ca1d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5ca1da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6ca1de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ca1e1, 95 },
    { 0, 0 },
    { 0xeb2ca1e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cca1ee, 17 },
    { 0, 0 },
    { 0x46ca1f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ca1f3, 33 },
    { 0x3b9ca1f3, 9 },
    { 0, 0 },
    { 0xe60ca1f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dca1ff, 90 },
    { 0x5f0ca200, 87 },
    { 0xa5aca1ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aca204, 77 },
    { 0, 0 },
    { 0xcedca206, 80 },
    { 0xe6fca207, 33 },
    { 0x620ca208, 95 },
    { 0xf5ca209, 98 },
    { 0xdbeca20a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fca20d, 61 },
    { 0xfa0ca20e, 33 },
    { 0, 0 },
    { 0xa74ca210, 84 },
    { 0xf70ca211, 40 },
    { 0xc12ca212, 95 },
    { 0x1daca213, 98 },
    { 0, 0 },
    { 0x884ca215, 98 },
    { 0x224ca216, 99 },
    { 0x393ca215, 98 },
    { 0x9a7ca218, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dca221, 33 },
    { 0, 0 },
    { 0x190ca223, 52 },
    { 0x7f3ca223, 87 },
    { 0, 0 },
    { 0x69dca226, 90 },
    { 0x861ca227, 90 },
    { 0x9e0ca227, 90 },
    { 0xe35ca227, 95 },
    { 0x752ca22a, 34 },
    { 0, 0 },
    { 0x127ca22c, 17 },
    { 0x65fca22d, 93 },
    { 0xcd3ca22d, 44 },
    { 0xbcfca22d, 76 },
    { 0xd9cca230, 33 },
    { 0x18fca231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ca234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acca239, 90 },
    { 0x998ca23a, 61 },
    { 0, 0 },
    { 0x536ca23c, 61 },
    { 0, 0 },
    { 0xc9dca23e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x737ca241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ca246, 61 },
    { 0x74eca246, 33 },
    { 0x977ca248, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bca24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dca253, 96 },
    { 0xf7fca253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca25b, 90 },
    { 0xc4fca25b, 90 },
    { 0x236ca25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642ca264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ca26d, 84 },
    { 0x54ca26e, 90 },
    { 0, 0 },
    { 0xabbca270, 90 },
    { 0, 0 },
    { 0x68dca272, 100 },
    { 0, 0 },
    { 0x2b7ca274, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ca27b, 34 },
    { 0xd77ca27b, 98 },
    { 0, 0 },
    { 0xbf1ca27e, 36 },
    { 0x11fca27e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcca282, 33 },
    { 0xf35ca282, 95 },
    { 0x527ca284, 61 },
    { 0xf5ca284, 33 },
    { 0xd7aca286, 99 },
    { 0x66aca287, 99 },
    { 0x8a2ca286, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ca28b, 23 },
    { 0xe71ca28c, 90 },
    { 0xd09ca28c, 34 },
    { 0x9acca28e, 67 },
    { 0x30bca28e, 76 },
    { 0xddca28b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cca294, 23 },
    { 0x76eca295, 23 },
    { 0, 0 },
    { 0xc54ca297, 87 },
    { 0xb1cca298, 85 },
    { 0xbfbca299, 33 },
    { 0xc2aca29a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ca2a1, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ca2a6, 76 },
    { 0, 0 },
    { 0x897ca2a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eca2ab, 95 },
    { 0, 0 },
    { 0xb2bca2ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca2b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eca2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baca2b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cca2bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfca2c2, 91 },
    { 0x8b2ca2c3, 55 },
    { 0, 0 },
    { 0x8e6ca2c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eca2c9, 88 },
    { 0xe50ca2c9, 19 },
    { 0, 0 },
    { 0xb8dca2cc, 87 },
    { 0xf1fca2cd, 61 },
    { 0, 0 },
    { 0x338ca2cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08ca2d2, 34 },
    { 0x6a6ca2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ca2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486ca2dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cca2e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ca2e6, 61 },
    { 0x76bca2e7, 34 },
    { 0xec7ca2e6, 98 },
    { 0xd3fca2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1ca2ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eca2f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdca2f6, 95 },
    { 0x40fca2f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ca2fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ca303, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aca307, 76 },
    { 0, 0 },
    { 0xb65ca309, 17 },
    { 0, 0 },
    { 0x75dca30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ca30f, 55 },
    { 0, 0 },
    { 0xb16ca311, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddca314, 91 },
    { 0x490ca314, 84 },
    { 0x381ca314, 61 },
    { 0xe47ca314, 23 },
    { 0x2b4ca318, 23 },
    { 0x18cca317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5ca31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2ca321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ca324, 90 },
    { 0, 0 },
    { 0xcd0ca326, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedca32a, 91 },
    { 0, 0 },
    { 0x23bca32c, 33 },
    { 0, 0 },
    { 0xba8ca32e, 98 },
    { 0xd08ca32e, 67 },
    { 0x7dcca330, 33 },
    { 0x449ca330, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274ca336, 95 },
    { 0x1d4ca337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aca33a, 61 },
    { 0x428ca33a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cca340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x335ca343, 93 },
    { 0xd7cca344, 33 },
    { 0xd4eca345, 87 },
    { 0xf4cca345, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x101ca349, 67 },
    { 0xbc2ca34a, 23 },
    { 0x224ca349, 75 },
    { 0xfa2ca34c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ca34f, 90 },
    { 0xbc5ca350, 61 },
    { 0x2f0ca351, 99 },
    { 0xcc0ca352, 95 },
    { 0xe1dca353, 91 },
    { 0, 0 },
    { 0xbbeca355, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aca35e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x236ca361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eca368, 23 },
    { 0xf0eca369, 17 },
    { 0x1f2ca36a, 65 },
    { 0xcdcca369, 56 },
    { 0x1a3ca36c, 33 },
    { 0xcb9ca36b, 93 },
    { 0, 0 },
    { 0x2d9ca36f, 33 },
    { 0x798ca36f, 46 },
    { 0, 0 },
    { 0x7a0ca372, 84 },
    { 0xa96ca373, 66 },
    { 0, 0 },
    { 0x8d2ca375, 61 },
    { 0, 0 },
    { 0x436ca377, 99 },
    { 0xd94ca378, 99 },
    { 0x70ca379, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2ca37e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677ca387, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcca38a, 95 },
    { 0x66dca38b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cca38f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ca393, 98 },
    { 0, 0 },
    { 0x201ca395, 55 },
    { 0, 0 },
    { 0x74cca397, 98 },
    { 0xbc0ca397, 95 },
    { 0, 0 },
    { 0x51aca39a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aca3a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaca3a3, 33 },
    { 0xb8cca3a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646ca3b0, 85 },
    { 0, 0 },
    { 0xd00ca3b2, 78 },
    { 0, 0 },
    { 0x418ca3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcca3bb, 55 },
    { 0x15bca3bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879ca3c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79ca3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77ca3d2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ca3d7, 23 },
    { 0xf18ca3d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bca3de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cca3e2, 76 },
    { 0xed0ca3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eca3ea, 17 },
    { 0xf20ca3ea, 98 },
    { 0, 0 },
    { 0x106ca3ed, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283ca3f3, 61 },
    { 0, 0 },
    { 0xc70ca3f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8beca3fd, 93 },
    { 0xef1ca3fe, 61 },
    { 0xb9fca3fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fca408, 95 },
    { 0, 0 },
    { 0x5e2ca40a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65ca40f, 95 },
    { 0, 0 },
    { 0x803ca411, 33 },
    { 0, 0 },
    { 0xd55ca413, 33 },
    { 0, 0 },
    { 0x49bca415, 23 },
    { 0x3d9ca416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8ca419, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdca428, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ca42c, 66 },
    { 0x9f1ca42d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31ca431, 90 },
    { 0x29bca432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecca435, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ca438, 61 },
    { 0, 0 },
    { 0xa79ca43a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dca43e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83ca443, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca446, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11ca44f, 91 },
    { 0, 0 },
    { 0x298ca451, 100 },
    { 0, 0 },
    { 0x51eca453, 95 },
    { 0x12fca454, 61 },
    { 0x91dca455, 46 },
    { 0x450ca455, 98 },
    { 0x1c1ca454, 100 },
    { 0xd8ca458, 78 },
    { 0x3feca456, 23 },
    { 0x34ca459, 33 },
    { 0x7e3ca454, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ca460, 100 },
    { 0x8b7ca461, 33 },
    { 0xc77ca462, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aca467, 33 },
    { 0xa63ca468, 33 },
    { 0, 0 },
    { 0xc5fca46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148ca471, 33 },
    { 0, 0 },
    { 0xf34ca473, 92 },
    { 0x5e7ca473, 51 },
    { 0x9bdca475, 46 },
    { 0xeadca476, 46 },
    { 0xc6bca475, 61 },
    { 0xf95ca474, 55 },
    { 0xdc0ca479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8ca47d, 23 },
    { 0, 0 },
    { 0xbceca47f, 53 },
    { 0xc5cca47f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ca485, 76 },
    { 0x64bca486, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ca48a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbca48e, 33 },
    { 0xe00ca48f, 55 },
    { 0xea4ca490, 61 },
    { 0x40bca490, 76 },
    { 0x811ca48e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9ca499, 85 },
    { 0xc13ca499, 90 },
    { 0x320ca49b, 61 },
    { 0x7b6ca49c, 76 },
    { 0x974ca49d, 33 },
    { 0xcb7ca49d, 92 },
    { 0, 0 },
    { 0x1b0ca4a0, 33 },
    { 0x5b5ca4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6ca4a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794ca4ab, 75 },
    { 0, 0 },
    { 0xe9cca4ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eca4b6, 34 },
    { 0x97dca4b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ca4bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ca4c3, 61 },
    { 0xd88ca4c4, 61 },
    { 0, 0 },
    { 0x8b2ca4c6, 90 },
    { 0xfeca4c7, 90 },
    { 0xc5cca4c7, 23 },
    { 0x831ca4c7, 34 },
    { 0x881ca4ca, 66 },
    { 0xf8bca4c6, 99 },
    { 0xa2cca4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5ca4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56ca4d4, 95 },
    { 0, 0 },
    { 0x25ca4d6, 66 },
    { 0xf1ca4d7, 100 },
    { 0x8dcca4d7, 33 },
    { 0xf61ca4d9, 93 },
    { 0x33cca4d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497ca4e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeca4e4, 43 },
    { 0x889ca4e4, 33 },
    { 0x8cca4e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ca4e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623ca4f1, 46 },
    { 0x368ca4f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ca4f6, 78 },
    { 0, 0 },
    { 0x92ca4f8, 33 },
    { 0x9c2ca4f9, 95 },
    { 0, 0 },
    { 0xdbeca4fb, 34 },
    { 0x2d7ca4fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edca500, 61 },
    { 0x98aca501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcca504, 95 },
    { 0xdb4ca504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cca50a, 95 },
    { 0xaf4ca50a, 90 },
    { 0xc2bca50c, 17 },
    { 0, 0 },
    { 0xf81ca50e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9ca511, 61 },
    { 0x48dca511, 90 },
    { 0, 0 },
    { 0xfafca514, 91 },
    { 0, 0 },
    { 0x98dca516, 36 },
    { 0, 0 },
    { 0xcabca518, 90 },
    { 0xaf0ca518, 66 },
    { 0x3d3ca51a, 76 },
    { 0xd52ca51b, 98 },
    { 0, 0 },
    { 0xe82ca51d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fca521, 90 },
    { 0, 0 },
    { 0x126ca523, 66 },
    { 0x88eca524, 33 },
    { 0x745ca525, 61 },
    { 0xc4ca525, 76 },
    { 0xb79ca523, 23 },
    { 0x3ddca525, 76 },
    { 0, 0 },
    { 0xb07ca52a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42ca52d, 61 },
    { 0, 0 },
    { 0xcb7ca52f, 76 },
    { 0xf3cca530, 53 },
    { 0x70cca531, 46 },
    { 0xab4ca532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203ca536, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dca53e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398ca54c, 98 },
    { 0x836ca54d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ca551, 76 },
    { 0xd8eca552, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deca555, 90 },
    { 0, 0 },
    { 0x6fdca557, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x487ca55a, 95 },
    { 0x5c7ca55a, 46 },
    { 0x1eaca55c, 98 },
    { 0xc64ca55d, 90 },
    { 0xbb9ca55e, 87 },
    { 0x60cca55f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27ca569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cca56e, 33 },
    { 0x332ca56e, 55 },
    { 0x960ca56f, 78 },
    { 0xd19ca571, 61 },
    { 0xf13ca570, 61 },
    { 0x25ca572, 61 },
    { 0, 0 },
    { 0x85aca575, 33 },
    { 0x101ca576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251ca57b, 34 },
    { 0x2f6ca57c, 33 },
    { 0, 0 },
    { 0xe18ca57e, 75 },
    { 0x3adca57e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ca588, 31 },
    { 0x320ca589, 33 },
    { 0xdf4ca589, 33 },
    { 0xe3bca58b, 65 },
    { 0xee4ca58c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ca592, 33 },
    { 0, 0 },
    { 0x8b7ca594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfca597, 33 },
    { 0, 0 },
    { 0x57ca599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bca59c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eca59f, 61 },
    { 0x95fca59f, 55 },
    { 0x7e9ca59f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ca5a8, 61 },
    { 0x97ca5a8, 90 },
    { 0, 0 },
    { 0x501ca5ab, 95 },
    { 0x198ca5ac, 61 },
    { 0xc1eca5ad, 76 },
    { 0x64eca5ac, 98 },
    { 0xa72ca5ac, 33 },
    { 0x29ca5b0, 90 },
    { 0x336ca5b1, 43 },
    { 0, 0 },
    { 0xa5bca5b3, 33 },
    { 0x7a4ca5b4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2ca5b8, 34 },
    { 0x119ca5b9, 78 },
    { 0, 0 },
    { 0xf48ca5bb, 68 },
    { 0x39ca5bb, 100 },
    { 0xcb0ca5bd, 76 },
    { 0x153ca5bd, 61 },
    { 0, 0 },
    { 0xf5aca5c0, 56 },
    { 0xc62ca5c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ca5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ca5c8, 99 },
    { 0x31aca5c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadca5d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5ca5d7, 23 },
    { 0x6c3ca5d8, 95 },
    { 0x65cca5d9, 61 },
    { 0x9cca5d9, 76 },
    { 0, 0 },
    { 0x1efca5dc, 33 },
    { 0x2e9ca5dc, 52 },
    { 0xa1dca5de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514ca5e2, 90 },
    { 0, 0 },
    { 0xdfeca5e4, 93 },
    { 0xb29ca5e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbca5e9, 68 },
    { 0x732ca5e9, 33 },
    { 0x470ca5e9, 76 },
    { 0x23eca5ec, 61 },
    { 0x63eca5ed, 95 },
    { 0x69aca5ee, 90 },
    { 0xd51ca5ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ca5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6ca5f5, 46 },
    { 0x9ccca5f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eca603, 76 },
    { 0xf4bca603, 34 },
    { 0, 0 },
    { 0x94eca606, 99 },
    { 0x634ca607, 90 },
    { 0xac1ca606, 33 },
    { 0x4bca609, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceca60d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aca610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ca618, 23 },
    { 0x6c7ca619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46ca61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccca620, 90 },
    { 0x8ddca621, 70 },
    { 0x569ca622, 23 },
    { 0xfa4ca622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53ca626, 55 },
    { 0xcb1ca627, 67 },
    { 0xa84ca628, 90 },
    { 0x4e5ca627, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1ca62c, 94 },
    { 0xb16ca62c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x728ca630, 90 },
    { 0x552ca630, 46 },
    { 0, 0 },
    { 0x4c6ca633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x574ca636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ca639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eca63c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02ca63f, 98 },
    { 0xcadca63f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6ca643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9ca646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ca64b, 99 },
    { 0xe3dca64c, 33 },
    { 0x48bca64d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3ca653, 76 },
    { 0, 0 },
    { 0xbedca655, 66 },
    { 0xec5ca655, 55 },
    { 0x188ca657, 23 },
    { 0x27dca656, 55 },
    { 0xbf9ca659, 95 },
    { 0, 0 },
    { 0xd2dca65b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6ca65f, 90 },
    { 0x629ca65f, 93 },
    { 0xb88ca65f, 83 },
    { 0xf96ca662, 95 },
    { 0xc51ca663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ca667, 33 },
    { 0x2b9ca668, 23 },
    { 0x1e4ca669, 100 },
    { 0x969ca667, 55 },
    { 0xe92ca66b, 100 },
    { 0x9c0ca66c, 40 },
    { 0x337ca66b, 98 },
    { 0x5e5ca66d, 85 },
    { 0xf49ca66a, 23 },
    { 0, 0 },
    { 0x3f6ca671, 61 },
    { 0, 0 },
    { 0xb3aca673, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eca677, 61 },
    { 0xcd5ca677, 66 },
    { 0, 0 },
    { 0xedfca67a, 99 },
    { 0x8afca67b, 33 },
    { 0x7d1ca67a, 99 },
    { 0x73aca67d, 33 },
    { 0xe35ca67d, 98 },
    { 0xd7ca67f, 33 },
    { 0, 0 },
    { 0xc48ca681, 98 },
    { 0, 0 },
    { 0x85cca683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8ca688, 95 },
    { 0x68dca689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7ca68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x288ca68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aca692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ca697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fca69c, 46 },
    { 0xd80ca69d, 67 },
    { 0x82ca69e, 65 },
    { 0xacbca69f, 99 },
    { 0x3e2ca6a0, 23 },
    { 0x40aca6a1, 34 },
    { 0xc00ca6a2, 23 },
    { 0x9c4ca69d, 23 },
    { 0xa63ca6a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ca6a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cca6ab, 90 },
    { 0xf8eca6ab, 43 },
    { 0, 0 },
    { 0x5f9ca6ae, 66 },
    { 0, 0 },
    { 0x574ca6b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ca6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3ca6bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2ca6c1, 77 },
    { 0x5deca6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95ca6c7, 90 },
    { 0x19cca6c8, 36 },
    { 0x4b7ca6c9, 61 },
    { 0x9a9ca6ca, 43 },
    { 0, 0 },
    { 0x29cca6cc, 76 },
    { 0xd64ca6cc, 34 },
    { 0x4e9ca6cc, 90 },
    { 0x3cbca6cd, 61 },
    { 0x628ca6cf, 90 },
    { 0x64aca6d0, 61 },
    { 0xa99ca6d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ca6d6, 90 },
    { 0x85cca6d7, 23 },
    { 0, 0 },
    { 0x3d2ca6d9, 34 },
    { 0x1cbca6da, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ca6dd, 76 },
    { 0, 0 },
    { 0x67bca6df, 98 },
    { 0x83ca6e0, 61 },
    { 0x8acca6e1, 76 },
    { 0x74cca6e2, 10 },
    { 0, 0 },
    { 0x1e0ca6e4, 99 },
    { 0, 0 },
    { 0xab4ca6e6, 83 },
    { 0xd16ca6e6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bca6eb, 90 },
    { 0xa31ca6ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ca6ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9ca6f2, 23 },
    { 0, 0 },
    { 0xdd2ca6f4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ca6fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ca6fe, 60 },
    { 0, 0 },
    { 0xa51ca700, 75 },
    { 0xe35ca701, 33 },
    { 0xc50ca702, 34 },
    { 0xa5cca703, 33 },
    { 0xd75ca704, 23 },
    { 0x4f8ca703, 95 },
    { 0x978ca706, 98 },
    { 0x1ccca700, 38 },
    { 0xb6bca708, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925ca70c, 90 },
    { 0xf07ca70d, 95 },
    { 0xe78ca70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ca712, 34 },
    { 0x817ca713, 93 },
    { 0, 0 },
    { 0x76eca715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ca719, 61 },
    { 0x40dca71a, 68 },
    { 0x1c4ca719, 76 },
    { 0x8c3ca719, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2ca720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeca723, 34 },
    { 0x94bca724, 61 },
    { 0, 0 },
    { 0x4eeca726, 46 },
    { 0, 0 },
    { 0x316ca728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3ca72c, 46 },
    { 0x2cfca72d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00ca731, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaca734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca738, 61 },
    { 0x3fcca738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2ca743, 33 },
    { 0, 0 },
    { 0x757ca745, 61 },
    { 0x5acca746, 55 },
    { 0x685ca745, 69 },
    { 0xc00ca748, 76 },
    { 0xddaca749, 93 },
    { 0x8bcca74a, 23 },
    { 0xdbbca745, 33 },
    { 0x3c4ca74c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dca753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ca75e, 46 },
    { 0xa0bca75f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589ca763, 95 },
    { 0, 0 },
    { 0xc6bca765, 33 },
    { 0x316ca766, 55 },
    { 0x9f9ca767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391ca76a, 43 },
    { 0xf23ca76b, 61 },
    { 0x80ca76c, 43 },
    { 0xcc9ca76d, 90 },
    { 0x392ca76c, 84 },
    { 0, 0 },
    { 0x8fdca770, 61 },
    { 0xd3aca771, 33 },
    { 0x367ca771, 95 },
    { 0x161ca773, 89 },
    { 0x123ca774, 98 },
    { 0x619ca772, 23 },
    { 0x1b4ca776, 98 },
    { 0x5f4ca774, 61 },
    { 0xb1bca778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cca77d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ca780, 90 },
    { 0xafeca781, 84 },
    { 0, 0 },
    { 0x73bca783, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fca787, 67 },
    { 0, 0 },
    { 0xe8dca789, 65 },
    { 0xb32ca78a, 90 },
    { 0x5c4ca78b, 23 },
    { 0xda4ca789, 98 },
    { 0xcf6ca789, 95 },
    { 0, 0 },
    { 0x34aca78f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbca794, 33 },
    { 0, 0 },
    { 0xb8dca796, 55 },
    { 0xf5ca797, 78 },
    { 0xdb7ca798, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bca79f, 23 },
    { 0, 0 },
    { 0x7fca7a1, 61 },
    { 0xffbca7a1, 93 },
    { 0xbfdca7a3, 90 },
    { 0xc8bca7a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeca7a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ca7ac, 33 },
    { 0x610ca7ad, 90 },
    { 0, 0 },
    { 0x106ca7af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ca7b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aca7ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cca7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ca7c3, 34 },
    { 0x82bca7c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ca7c7, 95 },
    { 0xb92ca7c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47ca7cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156ca7d3, 95 },
    { 0, 0 },
    { 0x986ca7d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ca7dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x165ca7e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1dca7e4, 33 },
    { 0xa0bca7e5, 55 },
    { 0x84dca7e4, 90 },
    { 0xc60ca7e7, 61 },
    { 0xd29ca7e8, 90 },
    { 0xaeca7e5, 23 },
    { 0x338ca7ea, 61 },
    { 0x48fca7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aca7ee, 95 },
    { 0xee0ca7ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x144ca7f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcca7f5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89ca7fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cca801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ca804, 76 },
    { 0x1ddca805, 95 },
    { 0xe53ca805, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eca80b, 55 },
    { 0xc26ca80c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ca80f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edca815, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ca818, 66 },
    { 0xbd8ca819, 23 },
    { 0x316ca81a, 61 },
    { 0x5f3ca81b, 97 },
    { 0xe2cca819, 34 },
    { 0x44dca81b, 76 },
    { 0x1acca81a, 29 },
    { 0x419ca81f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca822, 61 },
    { 0, 0 },
    { 0xfbdca824, 33 },
    { 0x1fbca825, 34 },
    { 0x653ca826, 67 },
    { 0xd95ca826, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ca82a, 100 },
    { 0x63dca82a, 76 },
    { 0x886ca82c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafca830, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ca833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ca836, 61 },
    { 0x868ca837, 33 },
    { 0x101ca838, 64 },
    { 0, 0 },
    { 0x38bca83a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75ca841, 95 },
    { 0x350ca841, 52 },
    { 0xf21ca843, 90 },
    { 0x2c7ca844, 90 },
    { 0xef3ca845, 23 },
    { 0xedbca845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5ca84b, 95 },
    { 0x85bca84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bca84f, 76 },
    { 0xa38ca850, 90 },
    { 0, 0 },
    { 0x9f6ca852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4ca855, 61 },
    { 0x25ca855, 33 },
    { 0x3edca855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ca85a, 90 },
    { 0xdf4ca85b, 98 },
    { 0x955ca85a, 17 },
    { 0, 0 },
    { 0x94fca85e, 52 },
    { 0, 0 },
    { 0xa7dca860, 90 },
    { 0x13ca861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ca86c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275ca87a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fca87f, 93 },
    { 0xcfdca880, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdca883, 34 },
    { 0xaa4ca884, 76 },
    { 0, 0 },
    { 0xb9fca886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbca88a, 90 },
    { 0, 0 },
    { 0xb3fca88c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdca893, 61 },
    { 0xb0bca894, 76 },
    { 0x1dbca893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726ca89a, 98 },
    { 0x24aca89b, 61 },
    { 0x754ca89b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebca8a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ca8a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceca8a6, 34 },
    { 0xf46ca8a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbca8b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0ca8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cca8c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3ca8cd, 76 },
    { 0xc5cca8cd, 95 },
    { 0x909ca8cf, 23 },
    { 0, 0 },
    { 0x2d1ca8d1, 61 },
    { 0x1dca8d2, 33 },
    { 0, 0 },
    { 0xd6bca8d4, 98 },
    { 0xb03ca8d4, 61 },
    { 0xa1fca8d6, 98 },
    { 0xb41ca8d7, 33 },
    { 0xca2ca8d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffca8dc, 95 },
    { 0, 0 },
    { 0x7d0ca8de, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dca8e7, 44 },
    { 0, 0 },
    { 0xbbdca8e9, 98 },
    { 0, 0 },
    { 0xdd2ca8eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ca8ef, 76 },
    { 0x661ca8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ca8f3, 23 },
    { 0x6ceca8f4, 34 },
    { 0xdcfca8f4, 95 },
    { 0xe7fca8f6, 33 },
    { 0x997ca8f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28ca8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aca900, 99 },
    { 0, 0 },
    { 0x563ca902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ca907, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ca90a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ca913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aca92d, 61 },
    { 0xf38ca92e, 76 },
    { 0x18ca92e, 34 },
    { 0xa40ca92e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ca935, 90 },
    { 0, 0 },
    { 0x39aca937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afca93c, 90 },
    { 0, 0 },
    { 0x23dca93e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44ca941, 33 },
    { 0x149ca941, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0eca94a, 76 },
    { 0x7fca94a, 76 },
    { 0xc66ca94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80ca950, 90 },
    { 0, 0 },
    { 0x28bca952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5ca956, 90 },
    { 0xcf7ca956, 23 },
    { 0x49dca958, 69 },
    { 0x40fca959, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43ca960, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca964, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5ca96a, 75 },
    { 0x657ca96b, 95 },
    { 0x316ca96c, 61 },
    { 0x610ca96d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aca970, 55 },
    { 0, 0 },
    { 0xe04ca972, 84 },
    { 0x2d6ca972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dca97c, 91 },
    { 0x208ca97d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84ca982, 76 },
    { 0xba7ca982, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eca986, 90 },
    { 0, 0 },
    { 0xcf3ca988, 90 },
    { 0, 0 },
    { 0xdc7ca98a, 91 },
    { 0, 0 },
    { 0xedcca98c, 77 },
    { 0x615ca98d, 38 },
    { 0x312ca98d, 81 },
    { 0x378ca98f, 98 },
    { 0xf92ca98c, 23 },
    { 0x4d5ca98e, 95 },
    { 0x7aeca991, 23 },
    { 0, 0 },
    { 0xcb6ca994, 95 },
    { 0x2e2ca995, 98 },
    { 0xcb5ca994, 67 },
    { 0x33ca997, 90 },
    { 0xb10ca996, 90 },
    { 0xa9bca999, 80 },
    { 0xd51ca99a, 81 },
    { 0, 0 },
    { 0x5afca99c, 33 },
    { 0xed8ca99c, 95 },
    { 0xb7ca99e, 98 },
    { 0x81ca99d, 23 },
    { 0, 0 },
    { 0x87aca9a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ca9a4, 76 },
    { 0x387ca9a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eca9a8, 96 },
    { 0, 0 },
    { 0x90dca9aa, 17 },
    { 0, 0 },
    { 0x8f0ca9ac, 98 },
    { 0, 0 },
    { 0xc2dca9ae, 52 },
    { 0xf2dca9af, 95 },
    { 0x8cdca9b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ca9b7, 55 },
    { 0x466ca9b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bca9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cca9bf, 90 },
    { 0x928ca9bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bca9c4, 76 },
    { 0xe8ca9c4, 98 },
    { 0x830ca9c6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ca9ce, 33 },
    { 0, 0 },
    { 0xb2ca9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcca9d3, 95 },
    { 0, 0 },
    { 0x227ca9d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bca9d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dca9dd, 17 },
    { 0xb12ca9dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dca9e2, 33 },
    { 0x393ca9e3, 77 },
    { 0x67eca9e2, 23 },
    { 0xf74ca9e2, 76 },
    { 0x64eca9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ca9ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5ca9f0, 61 },
    { 0x80aca9f1, 76 },
    { 0xba3ca9f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cca9f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bca9fe, 99 },
    { 0x429ca9ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625caa05, 90 },
    { 0xb44caa06, 90 },
    { 0xb3fcaa06, 93 },
    { 0x724caa08, 93 },
    { 0, 0 },
    { 0x559caa0a, 61 },
    { 0, 0 },
    { 0xd85caa0c, 23 },
    { 0xb4ccaa0c, 98 },
    { 0x878caa0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4caa12, 33 },
    { 0x635caa13, 100 },
    { 0, 0 },
    { 0x8e9caa15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaa18, 61 },
    { 0, 0 },
    { 0xad5caa1a, 65 },
    { 0, 0 },
    { 0x401caa1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4caa1f, 76 },
    { 0x1c7caa20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecaa24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6caa2d, 33 },
    { 0xd72caa2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fcaa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53caa3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374caa45, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bcaa4f, 78 },
    { 0x42dcaa50, 98 },
    { 0x662caa4f, 61 },
    { 0xd9bcaa50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25caa59, 43 },
    { 0xbb9caa5a, 90 },
    { 0, 0 },
    { 0x8d9caa5c, 61 },
    { 0x801caa5c, 36 },
    { 0, 0 },
    { 0x19fcaa5f, 90 },
    { 0, 0 },
    { 0xedbcaa61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6caa67, 87 },
    { 0x505caa67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512caa70, 91 },
    { 0x8decaa71, 33 },
    { 0, 0 },
    { 0xca3caa73, 23 },
    { 0, 0 },
    { 0x7b3caa75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ccaa7b, 100 },
    { 0x5e1caa7c, 33 },
    { 0, 0 },
    { 0xf90caa7e, 33 },
    { 0x31bcaa7f, 34 },
    { 0, 0 },
    { 0x7bacaa81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970caa8a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8acaa90, 61 },
    { 0x6a2caa90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3caa97, 33 },
    { 0x88fcaa97, 98 },
    { 0, 0 },
    { 0xd46caa9a, 90 },
    { 0, 0 },
    { 0x538caa9c, 77 },
    { 0x78caa9d, 61 },
    { 0x93acaa9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dcaaa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fcaaa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0caab0, 46 },
    { 0, 0 },
    { 0xb17caab2, 61 },
    { 0x66ecaab3, 34 },
    { 0x64fcaab3, 34 },
    { 0, 0 },
    { 0x6e8caab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1caabb, 90 },
    { 0xa13caabc, 61 },
    { 0x172caabc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6caaca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfcaace, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124caad4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69caad9, 55 },
    { 0, 0 },
    { 0xa6fcaadb, 87 },
    { 0x8dcaadb, 33 },
    { 0x42bcaadd, 98 },
    { 0xc3ecaade, 90 },
    { 0x665caade, 91 },
    { 0xe88caae0, 46 },
    { 0x92fcaadc, 90 },
    { 0xe85caae2, 78 },
    { 0x2aecaae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1caae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3caaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caaee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467caaf5, 90 },
    { 0xedccaaf6, 12 },
    { 0, 0 },
    { 0xc9bcaaf8, 90 },
    { 0x715caaf9, 34 },
    { 0, 0 },
    { 0x7bccaafb, 90 },
    { 0x8dccaafb, 23 },
    { 0xdd3caafd, 65 },
    { 0x969caafe, 88 },
    { 0x1facaaff, 23 },
    { 0xeeacaafb, 90 },
    { 0xd44cab01, 76 },
    { 0xf27caafb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ccab07, 90 },
    { 0, 0 },
    { 0x247cab09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5cab0d, 95 },
    { 0, 0 },
    { 0xde7cab0f, 23 },
    { 0xef3cab10, 91 },
    { 0xe7fcab11, 36 },
    { 0x2fdcab11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222cab19, 22 },
    { 0, 0 },
    { 0x7fecab1b, 55 },
    { 0x408cab1c, 77 },
    { 0x9a8cab1b, 55 },
    { 0x64ecab1e, 66 },
    { 0x159cab1f, 23 },
    { 0x8a0cab20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132cab24, 67 },
    { 0x7afcab25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcab2a, 61 },
    { 0x51bcab2b, 90 },
    { 0xef5cab2c, 95 },
    { 0xa8ccab2d, 77 },
    { 0x161cab2e, 33 },
    { 0xe69cab2e, 33 },
    { 0x25ecab2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x486cab33, 76 },
    { 0x407cab33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02cab38, 76 },
    { 0xf0bcab39, 33 },
    { 0x384cab3a, 17 },
    { 0xc39cab3b, 90 },
    { 0xf6ccab3a, 98 },
    { 0x80dcab3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ccab43, 77 },
    { 0x3f4cab43, 61 },
    { 0xefccab43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcab49, 98 },
    { 0x4e8cab4a, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4cab50, 33 },
    { 0, 0 },
    { 0xe1fcab52, 23 },
    { 0, 0 },
    { 0xc0dcab54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaccab5c, 61 },
    { 0x2ecab5d, 95 },
    { 0, 0 },
    { 0x38bcab5f, 83 },
    { 0, 0 },
    { 0xca4cab61, 33 },
    { 0x351cab62, 90 },
    { 0x102cab62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03cab66, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecab6b, 23 },
    { 0xc8cab6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecab6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6cab74, 95 },
    { 0xc35cab75, 33 },
    { 0x609cab76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5cab79, 43 },
    { 0, 0 },
    { 0x7d4cab7b, 90 },
    { 0x5e1cab7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cab84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26acab88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcab8d, 93 },
    { 0xa90cab8e, 98 },
    { 0x18ccab8d, 95 },
    { 0, 0 },
    { 0x905cab91, 90 },
    { 0xb27cab92, 17 },
    { 0xb2cab92, 33 },
    { 0xf38cab94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794cab99, 61 },
    { 0xc0ccab99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7caba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacccaba6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62caba9, 61 },
    { 0xd32cabaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634cabaf, 85 },
    { 0x9ecabb0, 78 },
    { 0x8bfcabb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dcabb4, 88 },
    { 0, 0 },
    { 0x722cabb6, 62 },
    { 0, 0 },
    { 0xd89cabb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbcabbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181cabc1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x386cabc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5cabca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1cabcd, 52 },
    { 0, 0 },
    { 0xbe3cabcf, 95 },
    { 0xf85cabd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94cabd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28cabd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133cabe0, 55 },
    { 0x759cabe1, 61 },
    { 0xa5dcabe0, 34 },
    { 0x1dbcabe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dcabe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09cabe9, 55 },
    { 0x811cabe9, 100 },
    { 0xbefcabea, 87 },
    { 0x655cabec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ecabfd, 55 },
    { 0, 0 },
    { 0xb95cabff, 90 },
    { 0xb12cac00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9cac03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3cac08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2cac0e, 33 },
    { 0x6e6cac0e, 61 },
    { 0x91fcac10, 23 },
    { 0xfaecac0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x318cac14, 52 },
    { 0x3cfcac14, 61 },
    { 0xbcbcac16, 46 },
    { 0xdebcac14, 98 },
    { 0x116cac18, 78 },
    { 0x543cac19, 23 },
    { 0, 0 },
    { 0x4f9cac1b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacac1e, 90 },
    { 0xd65cac1e, 61 },
    { 0xfffcac20, 46 },
    { 0, 0 },
    { 0xd7ecac22, 34 },
    { 0, 0 },
    { 0x465cac24, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bcac28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbcac34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbcac37, 77 },
    { 0xbaccac37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2cac3b, 95 },
    { 0xacac3c, 56 },
    { 0x7dacac3c, 98 },
    { 0x360cac3e, 91 },
    { 0xd95cac3f, 98 },
    { 0x7afcac40, 33 },
    { 0xf81cac3c, 95 },
    { 0xec7cac42, 90 },
    { 0x798cac3e, 98 },
    { 0x3d0cac42, 98 },
    { 0x37ecac45, 78 },
    { 0, 0 },
    { 0xe74cac47, 23 },
    { 0xd46cac47, 65 },
    { 0x560cac47, 76 },
    { 0xbc7cac47, 61 },
    { 0xf98cac4b, 90 },
    { 0x168cac4a, 76 },
    { 0, 0 },
    { 0xd15cac4e, 55 },
    { 0x5cecac4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cac54, 98 },
    { 0, 0 },
    { 0x93ccac56, 77 },
    { 0xec0cac56, 90 },
    { 0x328cac56, 33 },
    { 0xa14cac59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dcac62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bcac6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43cac6f, 23 },
    { 0xf42cac70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ccac7b, 34 },
    { 0x229cac7c, 95 },
    { 0, 0 },
    { 0xef0cac7e, 100 },
    { 0x545cac7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbcac85, 75 },
    { 0xbd3cac85, 85 },
    { 0, 0 },
    { 0x18cac88, 90 },
    { 0x904cac89, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cac8e, 33 },
    { 0, 0 },
    { 0xfbdcac90, 46 },
    { 0x73bcac90, 95 },
    { 0x246cac92, 75 },
    { 0x2e4cac93, 36 },
    { 0xdb4cac93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cac97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x478cac9a, 23 },
    { 0, 0 },
    { 0x45ecac9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcac9f, 90 },
    { 0xf77caca0, 95 },
    { 0, 0 },
    { 0x34caca2, 40 },
    { 0, 0 },
    { 0x8ccaca4, 52 },
    { 0, 0 },
    { 0x610caca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecaca9, 90 },
    { 0xe0dcacaa, 76 },
    { 0x7bfcacab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcacae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6cacb1, 61 },
    { 0, 0 },
    { 0xc7acacb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5cacba, 55 },
    { 0x1c5cacba, 33 },
    { 0x3acacba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cacc0, 61 },
    { 0xd84cacc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ccacc4, 95 },
    { 0xe85cacc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fcacd0, 23 },
    { 0x96bcacd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x378cacd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ccacd9, 93 },
    { 0x464cacd9, 78 },
    { 0, 0 },
    { 0xaa3cacdc, 33 },
    { 0xeecacdd, 33 },
    { 0x9b1cacde, 95 },
    { 0xb3acacdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18acace2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4cace5, 76 },
    { 0xa88cace6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69cacf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942cacf5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7cacfd, 76 },
    { 0x53ccacfe, 98 },
    { 0xefacacff, 33 },
    { 0x425cad00, 61 },
    { 0x242cad00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3cad07, 23 },
    { 0x7b9cad08, 90 },
    { 0x40cad09, 23 },
    { 0, 0 },
    { 0xc5fcad0b, 76 },
    { 0xe2ccad0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12cad10, 34 },
    { 0x36cad11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcad15, 55 },
    { 0x3ffcad16, 78 },
    { 0xd3ecad17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cad1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbcad20, 95 },
    { 0x6d2cad21, 97 },
    { 0x38ccad20, 99 },
    { 0x3d5cad23, 90 },
    { 0, 0 },
    { 0x7e9cad25, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5cad2b, 96 },
    { 0x211cad2c, 100 },
    { 0xb63cad2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bccad31, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x331cad34, 100 },
    { 0x473cad34, 90 },
    { 0x74fcad35, 90 },
    { 0x35ccad34, 76 },
    { 0x143cad38, 33 },
    { 0, 0 },
    { 0xab8cad3a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79cad3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64cad43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47cad4a, 65 },
    { 0x26dcad4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4cad4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x671cad52, 33 },
    { 0x189cad53, 55 },
    { 0xf27cad54, 90 },
    { 0, 0 },
    { 0x4f7cad56, 55 },
    { 0x428cad57, 55 },
    { 0, 0 },
    { 0x2bbcad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cad5d, 87 },
    { 0xe8bcad5e, 90 },
    { 0, 0 },
    { 0x3c9cad60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57cad64, 61 },
    { 0x541cad65, 95 },
    { 0xc44cad64, 67 },
    { 0xff8cad67, 52 },
    { 0x1a9cad64, 100 },
    { 0xff6cad64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cad71, 66 },
    { 0, 0 },
    { 0x732cad73, 65 },
    { 0xd3bcad74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0cad7c, 99 },
    { 0xbaacad7d, 33 },
    { 0xf40cad7d, 34 },
    { 0x96dcad7e, 81 },
    { 0x3b2cad80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5cad84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cad8b, 61 },
    { 0x981cad8c, 23 },
    { 0, 0 },
    { 0x7e6cad8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9acad95, 67 },
    { 0xc8ecad95, 66 },
    { 0, 0 },
    { 0x60cad98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0cad9f, 93 },
    { 0x420cada0, 33 },
    { 0xe21cada1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36cada4, 34 },
    { 0x796cada5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bcada8, 93 },
    { 0x67acada9, 33 },
    { 0x481cada9, 33 },
    { 0xccacadab, 98 },
    { 0xcfecadac, 17 },
    { 0xaa7cadac, 55 },
    { 0xdefcadaa, 98 },
    { 0xd1dcadaf, 23 },
    { 0xde1cadaf, 84 },
    { 0x99acadb1, 93 },
    { 0xe2cadb1, 43 },
    { 0x2e9cadb1, 61 },
    { 0, 0 },
    { 0xc66cadb5, 95 },
    { 0, 0 },
    { 0xe3ecadb7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78cadba, 90 },
    { 0x9c2cadba, 23 },
    { 0x635cadbc, 23 },
    { 0x6c5cadbc, 61 },
    { 0x5bdcadbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624cadc2, 65 },
    { 0xbe9cadc3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187cadc9, 55 },
    { 0, 0 },
    { 0x90dcadcb, 42 },
    { 0xa05cadcc, 98 },
    { 0xecfcadcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ccadd3, 95 },
    { 0xfbbcadd3, 90 },
    { 0x577cadd5, 91 },
    { 0xebacadd6, 91 },
    { 0xaccadd7, 23 },
    { 0x743cadd6, 93 },
    { 0x402cadd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11caddc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721cade0, 61 },
    { 0x694cade1, 95 },
    { 0x25ecade0, 95 },
    { 0, 0 },
    { 0x591cade4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4cade9, 33 },
    { 0xeabcadea, 90 },
    { 0xf59cade9, 33 },
    { 0x1cdcadec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fcadf1, 67 },
    { 0xa35cadf2, 98 },
    { 0x967cadf2, 84 },
    { 0, 0 },
    { 0x9bccadf5, 67 },
    { 0, 0 },
    { 0x4e8cadf7, 95 },
    { 0, 0 },
    { 0x2c6cadf9, 95 },
    { 0x72cadfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845cae01, 61 },
    { 0, 0 },
    { 0x3b8cae03, 52 },
    { 0, 0 },
    { 0x9f1cae05, 33 },
    { 0x3bccae05, 76 },
    { 0, 0 },
    { 0x51acae08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5cae0c, 46 },
    { 0, 0 },
    { 0x42ecae0e, 23 },
    { 0xee0cae0e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16cae14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cae18, 33 },
    { 0x5f4cae19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cae1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99cae21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0acae24, 56 },
    { 0x74dcae24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9cae29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4cae2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76cae34, 90 },
    { 0, 0 },
    { 0x71ccae36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ecae3b, 61 },
    { 0x73cae3c, 90 },
    { 0x381cae3b, 95 },
    { 0x14cae3d, 33 },
    { 0x3f2cae3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ccae42, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cae48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6cae4b, 61 },
    { 0xeedcae4b, 90 },
    { 0x6b2cae4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ccae50, 90 },
    { 0x13ecae51, 61 },
    { 0x5b3cae52, 90 },
    { 0xa1cae53, 95 },
    { 0x600cae53, 34 },
    { 0x1f9cae54, 88 },
    { 0x6fecae55, 33 },
    { 0x704cae57, 90 },
    { 0x6a7cae58, 33 },
    { 0, 0 },
    { 0x980cae5a, 81 },
    { 0, 0 },
    { 0x2a1cae5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecae60, 55 },
    { 0x80ccae61, 98 },
    { 0x87ccae61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ccae65, 90 },
    { 0, 0 },
    { 0x34ccae67, 33 },
    { 0xf85cae67, 40 },
    { 0x12cae69, 95 },
    { 0, 0 },
    { 0xa14cae6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8cae73, 33 },
    { 0x455cae74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbcae77, 55 },
    { 0xdbdcae78, 90 },
    { 0x14ecae79, 90 },
    { 0xabfcae7a, 97 },
    { 0, 0 },
    { 0x9cdcae7c, 76 },
    { 0xc1ccae7c, 23 },
    { 0, 0 },
    { 0x6dcae7f, 68 },
    { 0xbabcae80, 65 },
    { 0x7dfcae7f, 76 },
    { 0x9a9cae7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877cae88, 98 },
    { 0x139cae89, 55 },
    { 0x9fecae8a, 78 },
    { 0x2efcae8b, 90 },
    { 0x634cae8c, 90 },
    { 0x85ecae8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x727cae90, 43 },
    { 0x105cae91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dcae94, 91 },
    { 0, 0 },
    { 0x27bcae96, 85 },
    { 0x1e6cae97, 34 },
    { 0x310cae98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dcae9f, 34 },
    { 0, 0 },
    { 0xb9dcaea1, 95 },
    { 0x843caea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x463caea5, 95 },
    { 0, 0 },
    { 0x860caea7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x933caeaa, 55 },
    { 0, 0 },
    { 0x355caeac, 33 },
    { 0xc10caeac, 23 },
    { 0x128caeae, 76 },
    { 0x746caeaf, 84 },
    { 0xc0dcaeb0, 90 },
    { 0, 0 },
    { 0x448caeb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbcaeb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62caebb, 90 },
    { 0xac5caebc, 52 },
    { 0x94caebb, 98 },
    { 0, 0 },
    { 0x1eecaebf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71caec4, 55 },
    { 0xe6bcaec5, 94 },
    { 0xda5caec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55caec9, 92 },
    { 0x7decaeca, 23 },
    { 0x316caeca, 74 },
    { 0, 0 },
    { 0x329caecd, 98 },
    { 0xce2caecd, 33 },
    { 0x170caecf, 68 },
    { 0xfd6caecf, 77 },
    { 0, 0 },
    { 0xfa7caed2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32caed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x505caeda, 33 },
    { 0xb86caedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caede, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1caee1, 93 },
    { 0, 0 },
    { 0x87dcaee3, 85 },
    { 0xb55caee3, 23 },
    { 0xfeccaee5, 55 },
    { 0xd44caee3, 98 },
    { 0xed5caee7, 98 },
    { 0x21ccaee5, 76 },
    { 0x414caee6, 95 },
    { 0x417caeea, 90 },
    { 0xefcaeeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecaeee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677caef6, 65 },
    { 0x3b3caef7, 17 },
    { 0x2bfcaef8, 78 },
    { 0x503caef8, 43 },
    { 0xca0caefa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18caeff, 66 },
    { 0x93ccaf00, 90 },
    { 0, 0 },
    { 0x41caf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1caf0a, 61 },
    { 0x735caf0b, 61 },
    { 0xb11caf0c, 95 },
    { 0x516caf0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdcaf14, 33 },
    { 0x289caf14, 95 },
    { 0, 0 },
    { 0xd1fcaf17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547caf1c, 28 },
    { 0xc4acaf1c, 61 },
    { 0xc48caf1c, 23 },
    { 0xc18caf1f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80caf29, 87 },
    { 0xf42caf2a, 66 },
    { 0x7b4caf2b, 98 },
    { 0x136caf2a, 88 },
    { 0xdf1caf2d, 33 },
    { 0xaeacaf2e, 90 },
    { 0, 0 },
    { 0x8d9caf30, 98 },
    { 0, 0 },
    { 0x6bccaf32, 46 },
    { 0, 0 },
    { 0x520caf34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3caf37, 98 },
    { 0x4f6caf37, 66 },
    { 0x770caf39, 77 },
    { 0xbb0caf39, 55 },
    { 0x6e9caf3b, 42 },
    { 0x53dcaf39, 98 },
    { 0x80dcaf3a, 98 },
    { 0xaf4caf3a, 33 },
    { 0x376caf3f, 90 },
    { 0xfb6caf3f, 98 },
    { 0x7b9caf41, 23 },
    { 0xdadcaf42, 98 },
    { 0x592caf40, 34 },
    { 0xb94caf37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7caf47, 84 },
    { 0x700caf47, 98 },
    { 0x81acaf47, 90 },
    { 0xfc8caf4a, 90 },
    { 0xa2bcaf47, 90 },
    { 0, 0 },
    { 0x1a6caf4d, 33 },
    { 0x2c1caf4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4caf53, 90 },
    { 0xce8caf54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ccaf58, 23 },
    { 0x1e6caf59, 77 },
    { 0, 0 },
    { 0xa2fcaf5b, 61 },
    { 0, 0 },
    { 0xe20caf5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6caf62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ccaf6a, 61 },
    { 0xef4caf6b, 23 },
    { 0, 0 },
    { 0xca3caf6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41caf72, 92 },
    { 0, 0 },
    { 0x731caf74, 90 },
    { 0x75bcaf75, 64 },
    { 0, 0 },
    { 0xcdcaf77, 87 },
    { 0x224caf78, 98 },
    { 0xa46caf79, 61 },
    { 0xb6caf78, 91 },
    { 0, 0 },
    { 0xe0dcaf7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacaf85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6caf88, 48 },
    { 0x8c5caf89, 78 },
    { 0x8eecaf8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddcaf8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccaf91, 90 },
    { 0xb2acaf92, 33 },
    { 0, 0 },
    { 0x33ecaf94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2caf9e, 90 },
    { 0x6d0caf9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x681cafa2, 61 },
    { 0x3c1cafa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5cafa6, 98 },
    { 0x682cafa7, 52 },
    { 0xde4cafa6, 33 },
    { 0x28bcafa9, 67 },
    { 0, 0 },
    { 0xaafcafab, 95 },
    { 0xde2cafac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30cafaf, 66 },
    { 0xab8cafaf, 95 },
    { 0x60ecafb0, 88 },
    { 0, 0 },
    { 0x87dcafb3, 20 },
    { 0xe65cafb3, 91 },
    { 0, 0 },
    { 0x4bdcafb6, 95 },
    { 0, 0 },
    { 0x3b5cafb8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bcafbf, 33 },
    { 0x2a4cafc0, 98 },
    { 0, 0 },
    { 0xcedcafc2, 98 },
    { 0, 0 },
    { 0xf12cafc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528cafc9, 52 },
    { 0x5eacafca, 33 },
    { 0, 0 },
    { 0x7e3cafcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecafdc, 52 },
    { 0xeb1cafdd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x797cafe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaecafe4, 90 },
    { 0x958cafe4, 97 },
    { 0x667cafe6, 90 },
    { 0, 0 },
    { 0xd82cafe8, 98 },
    { 0, 0 },
    { 0x470cafea, 61 },
    { 0xd33cafeb, 33 },
    { 0, 0 },
    { 0xcdfcafed, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196caff6, 87 },
    { 0xfabcaff6, 99 },
    { 0x2f2caff6, 98 },
    { 0, 0 },
    { 0x317caffa, 99 },
    { 0x421caffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23cb004, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3cb00a, 98 },
    { 0, 0 },
    { 0xfffcb00c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7cb011, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508cb016, 76 },
    { 0xb69cb017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541cb022, 90 },
    { 0x447cb022, 93 },
    { 0, 0 },
    { 0xae2cb025, 33 },
    { 0xe2ccb026, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccb02a, 55 },
    { 0xc9fcb02b, 66 },
    { 0x674cb02a, 43 },
    { 0, 0 },
    { 0x6b6cb02e, 61 },
    { 0xa89cb02f, 66 },
    { 0xc4fcb02f, 66 },
    { 0x668cb031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7cb038, 85 },
    { 0, 0 },
    { 0x17fcb03a, 46 },
    { 0x49dcb03a, 90 },
    { 0xa32cb03a, 75 },
    { 0xdffcb03d, 81 },
    { 0x9f0cb03d, 33 },
    { 0xf42cb03f, 84 },
    { 0, 0 },
    { 0x693cb041, 80 },
    { 0xcc6cb042, 97 },
    { 0x1fbcb042, 61 },
    { 0x4bacb042, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x680cb047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dacb04a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638cb052, 90 },
    { 0, 0 },
    { 0x640cb054, 33 },
    { 0x3b5cb055, 55 },
    { 0xcf5cb054, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaacb05b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ccb05e, 90 },
    { 0x4f5cb05e, 90 },
    { 0x767cb060, 61 },
    { 0xeaccb061, 43 },
    { 0x3d7cb060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23cb066, 95 },
    { 0, 0 },
    { 0x715cb068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01cb06b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edcb06f, 61 },
    { 0, 0 },
    { 0x88dcb071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceccb074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcb078, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9cb07b, 67 },
    { 0, 0 },
    { 0x67cb07d, 76 },
    { 0, 0 },
    { 0x6f8cb07f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x780cb082, 99 },
    { 0, 0 },
    { 0x515cb084, 90 },
    { 0x24fcb085, 84 },
    { 0, 0 },
    { 0x989cb087, 90 },
    { 0x380cb087, 35 },
    { 0, 0 },
    { 0xd44cb08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ccb08d, 61 },
    { 0xe70cb08d, 55 },
    { 0x8a1cb08f, 38 },
    { 0x4c9cb08f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x937cb093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4cb096, 67 },
    { 0x6cccb096, 95 },
    { 0, 0 },
    { 0xd0fcb099, 52 },
    { 0x9a8cb099, 98 },
    { 0xcaacb09b, 33 },
    { 0, 0 },
    { 0x732cb09d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cb0a2, 43 },
    { 0x3efcb0a3, 15 },
    { 0xb17cb0a4, 33 },
    { 0x983cb0a2, 98 },
    { 0xe52cb0a6, 91 },
    { 0x6b2cb0a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccb0aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cb0ad, 76 },
    { 0x35ecb0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08cb0bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fcb0bf, 23 },
    { 0x55ccb0bf, 78 },
    { 0x574cb0c1, 33 },
    { 0xe68cb0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x937cb0c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeccb0c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcb0ce, 34 },
    { 0x33dcb0ce, 61 },
    { 0, 0 },
    { 0x27cb0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5cb0d9, 90 },
    { 0x223cb0d9, 66 },
    { 0xb57cb0d9, 61 },
    { 0x57bcb0dc, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb0e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ecb0e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151cb0f6, 98 },
    { 0xd49cb0f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7cb0fa, 68 },
    { 0, 0 },
    { 0x650cb0fc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cb103, 33 },
    { 0, 0 },
    { 0xad1cb105, 76 },
    { 0x66ccb106, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cb10c, 46 },
    { 0xcbcb10d, 76 },
    { 0xe83cb10c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ecb115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5cb11b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cb120, 33 },
    { 0xccfcb121, 34 },
    { 0, 0 },
    { 0x80ecb123, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77cb129, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cb130, 23 },
    { 0, 0 },
    { 0x3ddcb132, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebacb13a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdcb144, 101 },
    { 0xb75cb145, 85 },
    { 0xca9cb146, 95 },
    { 0xf9ecb146, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dcb14a, 66 },
    { 0x4ecb14b, 23 },
    { 0x493cb14c, 66 },
    { 0x434cb14d, 34 },
    { 0, 0 },
    { 0x9c0cb14f, 98 },
    { 0x1cccb150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x240cb153, 90 },
    { 0, 0 },
    { 0x344cb155, 23 },
    { 0, 0 },
    { 0xdbfcb157, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74acb15c, 36 },
    { 0x50ecb15d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0cb160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164cb165, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x900cb168, 68 },
    { 0xbd4cb169, 61 },
    { 0xdf9cb169, 90 },
    { 0xdf1cb169, 33 },
    { 0xe40cb16c, 78 },
    { 0x3dcb169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cb172, 93 },
    { 0xb21cb173, 95 },
    { 0x64acb174, 98 },
    { 0x3b7cb174, 34 },
    { 0xeb6cb172, 66 },
    { 0xcf1cb177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x148cb17a, 52 },
    { 0xfdacb17b, 61 },
    { 0xe57cb17b, 55 },
    { 0x763cb17b, 76 },
    { 0xd10cb17b, 100 },
    { 0x26ecb17f, 77 },
    { 0x7d4cb17d, 84 },
    { 0x9edcb180, 23 },
    { 0x9fbcb182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ccb185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddcb18e, 55 },
    { 0x3d6cb18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdcb193, 93 },
    { 0, 0 },
    { 0xb0cb195, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907cb19c, 61 },
    { 0, 0 },
    { 0x4a8cb19e, 76 },
    { 0xf30cb19e, 43 },
    { 0x872cb1a0, 23 },
    { 0x7aacb1a1, 46 },
    { 0xececb1a2, 78 },
    { 0x429cb1a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7cb1ab, 99 },
    { 0x908cb1ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ccb1af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadcb1b3, 23 },
    { 0x2d0cb1b4, 33 },
    { 0, 0 },
    { 0xbf2cb1b6, 34 },
    { 0x57ecb1b7, 90 },
    { 0x9c2cb1b6, 96 },
    { 0xc64cb1b9, 95 },
    { 0xf81cb1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffacb1c4, 61 },
    { 0xe5ccb1c4, 17 },
    { 0, 0 },
    { 0xc7cb1c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689cb1cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9cb1d3, 76 },
    { 0, 0 },
    { 0x901cb1d5, 55 },
    { 0x452cb1d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcacb1da, 77 },
    { 0xe6ecb1db, 76 },
    { 0x1c5cb1db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cb1e0, 101 },
    { 0xa8dcb1e0, 33 },
    { 0x5dacb1e2, 100 },
    { 0x361cb1e3, 64 },
    { 0x9f8cb1e4, 66 },
    { 0xa14cb1e5, 33 },
    { 0x177cb1e6, 61 },
    { 0x85acb1e0, 90 },
    { 0xb8bcb1e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8cb1ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3cb1f3, 98 },
    { 0, 0 },
    { 0xd65cb1f5, 61 },
    { 0x68dcb1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fcb1fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbccb1fe, 23 },
    { 0x5f0cb1ff, 99 },
    { 0, 0 },
    { 0x477cb201, 38 },
    { 0xa28cb201, 61 },
    { 0x2d4cb201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cb206, 98 },
    { 0xda2cb207, 99 },
    { 0x259cb207, 99 },
    { 0x456cb206, 67 },
    { 0x81cb20a, 99 },
    { 0x23bcb20b, 90 },
    { 0x528cb206, 43 },
    { 0x161cb20d, 34 },
    { 0xa79cb20e, 23 },
    { 0xf90cb20b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4cb215, 93 },
    { 0x781cb215, 87 },
    { 0x45dcb217, 90 },
    { 0, 0 },
    { 0xe6ccb219, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dcb21c, 93 },
    { 0x6d2cb21d, 23 },
    { 0x98cb21e, 90 },
    { 0, 0 },
    { 0x144cb220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12cb226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cb22c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ecb234, 100 },
    { 0x339cb235, 90 },
    { 0, 0 },
    { 0xd4bcb237, 95 },
    { 0x841cb238, 85 },
    { 0xd90cb239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7cb23e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefccb245, 61 },
    { 0x901cb246, 43 },
    { 0x944cb247, 90 },
    { 0x5a7cb248, 66 },
    { 0xb20cb247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8cb251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514cb255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fcb258, 95 },
    { 0, 0 },
    { 0xc54cb25a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fccb25d, 33 },
    { 0x8dbcb25e, 90 },
    { 0xa89cb25f, 95 },
    { 0xa65cb260, 61 },
    { 0x710cb25e, 34 },
    { 0x7accb25d, 75 },
    { 0, 0 },
    { 0xab9cb264, 76 },
    { 0x8dacb264, 91 },
    { 0, 0 },
    { 0x70fcb267, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cccb26a, 33 },
    { 0, 0 },
    { 0x943cb26c, 61 },
    { 0xc5fcb26c, 61 },
    { 0, 0 },
    { 0x7b4cb26f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3cb274, 68 },
    { 0x957cb275, 33 },
    { 0, 0 },
    { 0x60dcb277, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x586cb27a, 43 },
    { 0xb35cb27b, 40 },
    { 0x9edcb27c, 34 },
    { 0, 0 },
    { 0x7aecb27e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23cb282, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cb285, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0acb28a, 23 },
    { 0x3e7cb28b, 98 },
    { 0, 0 },
    { 0xe1acb28d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ecb297, 90 },
    { 0xb34cb298, 66 },
    { 0xa13cb299, 98 },
    { 0x439cb299, 90 },
    { 0x4c2cb29b, 65 },
    { 0x824cb299, 66 },
    { 0x5cb29d, 33 },
    { 0x38bcb29e, 78 },
    { 0, 0 },
    { 0xaaecb2a0, 90 },
    { 0xaeecb2a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75cb2a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdcb2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fcb2b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fcb2ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04cb2be, 61 },
    { 0, 0 },
    { 0xcdecb2c0, 95 },
    { 0xb0cb2c0, 34 },
    { 0, 0 },
    { 0xf44cb2c3, 33 },
    { 0xe28cb2c4, 85 },
    { 0, 0 },
    { 0xcdacb2c6, 51 },
    { 0xd0cb2c7, 90 },
    { 0xf9dcb2c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x822cb2cb, 78 },
    { 0x896cb2cc, 46 },
    { 0x50bcb2cd, 17 },
    { 0x918cb2cd, 91 },
    { 0x279cb2cd, 43 },
    { 0x9adcb2d0, 88 },
    { 0x7bdcb2cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bccb2dc, 17 },
    { 0x884cb2dc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9cb2e1, 90 },
    { 0, 0 },
    { 0xdecb2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1cb2ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcb2ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4cb2f7, 90 },
    { 0xb25cb2f7, 93 },
    { 0, 0 },
    { 0x50fcb2fa, 75 },
    { 0xfa3cb2fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dcb300, 34 },
    { 0x2cdcb301, 66 },
    { 0xdc0cb302, 90 },
    { 0x12cb303, 76 },
    { 0, 0 },
    { 0x383cb305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ccb311, 90 },
    { 0x8f6cb311, 55 },
    { 0x7fdcb313, 61 },
    { 0x77acb313, 98 },
    { 0xa6ecb315, 78 },
    { 0xe4fcb316, 99 },
    { 0, 0 },
    { 0x96ccb318, 69 },
    { 0, 0 },
    { 0x5e0cb31a, 98 },
    { 0xfc2cb31b, 29 },
    { 0xe62cb31b, 95 },
    { 0x834cb31d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fcb321, 78 },
    { 0x6e4cb321, 55 },
    { 0xdfacb323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263cb329, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dcb32c, 33 },
    { 0xd0cb32d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0cb335, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcb33a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bcb33d, 87 },
    { 0xb21cb33e, 78 },
    { 0x17dcb33f, 33 },
    { 0xf95cb33e, 87 },
    { 0xab6cb340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23acb348, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2cb34b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bcb350, 61 },
    { 0xacecb350, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31cb354, 76 },
    { 0xbb7cb355, 91 },
    { 0, 0 },
    { 0xf9bcb357, 90 },
    { 0x81ecb358, 95 },
    { 0, 0 },
    { 0x346cb35a, 98 },
    { 0x8becb35b, 95 },
    { 0xa6dcb35a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xabecb35f, 84 },
    { 0xb47cb35f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bcb364, 90 },
    { 0, 0 },
    { 0x31acb366, 34 },
    { 0x790cb367, 90 },
    { 0xf4bcb368, 67 },
    { 0x5dfcb366, 61 },
    { 0xefacb368, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09cb370, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cb37a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cb37d, 61 },
    { 0x80ccb37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9cb387, 93 },
    { 0x2ebcb388, 23 },
    { 0, 0 },
    { 0xb8fcb38a, 85 },
    { 0x453cb38b, 61 },
    { 0x5a7cb38c, 68 },
    { 0xb44cb38c, 34 },
    { 0x1fcb38e, 91 },
    { 0x7e2cb38f, 23 },
    { 0, 0 },
    { 0x12dcb391, 76 },
    { 0x43acb391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dcb39b, 90 },
    { 0xf68cb39c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22cb3a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163cb3a8, 55 },
    { 0x57cb3a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165cb3b0, 52 },
    { 0, 0 },
    { 0xd56cb3b2, 55 },
    { 0, 0 },
    { 0x4f5cb3b4, 90 },
    { 0xfb7cb3b5, 84 },
    { 0xd85cb3b6, 88 },
    { 0x6d1cb3b7, 61 },
    { 0x553cb3b7, 95 },
    { 0xddcb3b5, 90 },
    { 0x964cb3ba, 90 },
    { 0x6cdcb3b6, 98 },
    { 0x76bcb3bc, 98 },
    { 0x99acb3bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42cb3c0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123cb3c7, 33 },
    { 0x879cb3c8, 99 },
    { 0xa98cb3c7, 90 },
    { 0x4b0cb3c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cb3cd, 93 },
    { 0x5dacb3ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593cb3d4, 76 },
    { 0, 0 },
    { 0xbdbcb3d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5cb3e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952cb3e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cb3ed, 95 },
    { 0x80cb3ee, 34 },
    { 0, 0 },
    { 0xaaccb3f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37cb3f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962cb3f8, 93 },
    { 0, 0 },
    { 0xd3cb3fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9cb3fd, 55 },
    { 0, 0 },
    { 0xeb2cb3ff, 23 },
    { 0, 0 },
    { 0xc89cb401, 61 },
    { 0x84cb401, 84 },
    { 0, 0 },
    { 0xb70cb404, 98 },
    { 0x781cb404, 84 },
    { 0x3edcb405, 98 },
    { 0xa60cb407, 23 },
    { 0x1b7cb407, 99 },
    { 0x742cb409, 34 },
    { 0x542cb409, 90 },
    { 0xe74cb40b, 43 },
    { 0xcb5cb40a, 90 },
    { 0x690cb407, 46 },
    { 0, 0 },
    { 0x9d6cb40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32cb412, 34 },
    { 0xc2fcb412, 34 },
    { 0, 0 },
    { 0x9c9cb415, 61 },
    { 0, 0 },
    { 0xf73cb417, 55 },
    { 0, 0 },
    { 0x3c6cb419, 93 },
    { 0, 0 },
    { 0x29acb41b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5cb41f, 76 },
    { 0, 0 },
    { 0x4f2cb421, 76 },
    { 0x2c0cb422, 39 },
    { 0xa68cb423, 68 },
    { 0, 0 },
    { 0x388cb425, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbcb428, 76 },
    { 0xd78cb429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adcb42d, 28 },
    { 0x773cb42e, 40 },
    { 0x4ddcb42e, 90 },
    { 0, 0 },
    { 0xd30cb431, 55 },
    { 0x7dccb431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fcb435, 95 },
    { 0, 0 },
    { 0x103cb437, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726cb43d, 88 },
    { 0x5b4cb43e, 84 },
    { 0x86dcb43f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb442, 77 },
    { 0, 0 },
    { 0xafecb444, 98 },
    { 0xdbfcb445, 88 },
    { 0, 0 },
    { 0x599cb447, 91 },
    { 0x9bcb448, 23 },
    { 0xb38cb448, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53cb44c, 90 },
    { 0xff6cb44c, 100 },
    { 0x651cb44d, 98 },
    { 0x472cb44f, 46 },
    { 0xa07cb450, 98 },
    { 0xeadcb451, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedcb454, 23 },
    { 0xe17cb455, 84 },
    { 0xeafcb454, 23 },
    { 0, 0 },
    { 0x637cb458, 23 },
    { 0x11ecb459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cb45e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ecb462, 34 },
    { 0xf4ccb462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x949cb466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1cb46d, 33 },
    { 0xf6cb46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7cb471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ecb478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365cb47b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63cb483, 93 },
    { 0x3ccb484, 33 },
    { 0x3acb485, 90 },
    { 0, 0 },
    { 0xd83cb487, 90 },
    { 0, 0 },
    { 0x39bcb489, 90 },
    { 0x4a8cb48a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0cb492, 61 },
    { 0x30cb492, 90 },
    { 0xb94cb494, 95 },
    { 0x29cb495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fcb49f, 34 },
    { 0xf3fcb4a0, 86 },
    { 0x80ecb4a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6cb4a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cb4b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31cb4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdcb4bb, 55 },
    { 0xa85cb4bb, 98 },
    { 0x66cb4bd, 33 },
    { 0, 0 },
    { 0xe6fcb4bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efcb4c3, 90 },
    { 0x976cb4c4, 98 },
    { 0xf42cb4c5, 61 },
    { 0xc4ccb4c6, 33 },
    { 0x536cb4c6, 76 },
    { 0xacbcb4c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cb4cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fcb4cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacb4d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5cb4d9, 98 },
    { 0x728cb4da, 98 },
    { 0x5eecb4da, 55 },
    { 0x817cb4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0cb4e0, 95 },
    { 0, 0 },
    { 0x521cb4e2, 33 },
    { 0x473cb4e3, 55 },
    { 0, 0 },
    { 0xba7cb4e5, 99 },
    { 0, 0 },
    { 0xcdecb4e7, 98 },
    { 0x84dcb4e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2cb4ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecb4f1, 90 },
    { 0x915cb4f1, 95 },
    { 0x41fcb4f3, 23 },
    { 0xc0bcb4f4, 90 },
    { 0x7c4cb4f3, 90 },
    { 0x3e6cb4f4, 76 },
    { 0xadccb4f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1cb4fa, 98 },
    { 0, 0 },
    { 0x51acb4fc, 61 },
    { 0, 0 },
    { 0x42bcb4fe, 100 },
    { 0x31bcb4ff, 90 },
    { 0, 0 },
    { 0x113cb501, 98 },
    { 0x41fcb502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edcb50a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45cb50e, 90 },
    { 0, 0 },
    { 0x834cb510, 90 },
    { 0, 0 },
    { 0xc7fcb512, 93 },
    { 0, 0 },
    { 0x5c5cb514, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb51b, 43 },
    { 0x3f7cb51c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbacb521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01cb529, 95 },
    { 0x41fcb52a, 90 },
    { 0x8bacb52b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cb52e, 46 },
    { 0, 0 },
    { 0x173cb530, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33cb53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4cb53d, 98 },
    { 0, 0 },
    { 0x5afcb53f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cb550, 99 },
    { 0, 0 },
    { 0x206cb552, 95 },
    { 0xab4cb553, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fcb556, 76 },
    { 0, 0 },
    { 0x7c8cb558, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3cb55e, 61 },
    { 0x227cb55f, 34 },
    { 0xed1cb55f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb567, 34 },
    { 0x537cb568, 55 },
    { 0xeddcb569, 90 },
    { 0x822cb569, 67 },
    { 0xa38cb569, 46 },
    { 0x10ecb56c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dcb572, 33 },
    { 0, 0 },
    { 0xb30cb574, 81 },
    { 0, 0 },
    { 0x9fccb576, 65 },
    { 0xd38cb576, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00cb57c, 33 },
    { 0x938cb57d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7cb582, 55 },
    { 0x9a5cb583, 61 },
    { 0x6a5cb583, 33 },
    { 0xacecb585, 84 },
    { 0x3f9cb582, 90 },
    { 0, 0 },
    { 0x8d9cb588, 43 },
    { 0, 0 },
    { 0x209cb58a, 76 },
    { 0, 0 },
    { 0x1b2cb58c, 90 },
    { 0x2d4cb58d, 90 },
    { 0x7e2cb58d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeecb591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768cb599, 98 },
    { 0x2d4cb59a, 90 },
    { 0x1cecb59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7cb5a0, 90 },
    { 0xf99cb5a1, 91 },
    { 0xd5acb5a2, 46 },
    { 0xe26cb5a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eccb5a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adcb5ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x169cb5b0, 33 },
    { 0xf4acb5b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980cb5b6, 90 },
    { 0x918cb5b7, 98 },
    { 0x65fcb5b8, 91 },
    { 0x428cb5b9, 95 },
    { 0xa5ccb5ba, 61 },
    { 0xb7ecb5bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8cb5be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ecb5cd, 23 },
    { 0, 0 },
    { 0xd33cb5cf, 90 },
    { 0, 0 },
    { 0x781cb5d1, 98 },
    { 0, 0 },
    { 0xe2ecb5d3, 90 },
    { 0, 0 },
    { 0x488cb5d5, 98 },
    { 0x61cb5d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb5db, 66 },
    { 0x4acb5db, 61 },
    { 0x379cb5dd, 95 },
    { 0x8d3cb5de, 99 },
    { 0, 0 },
    { 0x2e0cb5e0, 33 },
    { 0xb48cb5e1, 75 },
    { 0xb92cb5e2, 90 },
    { 0x2b3cb5e3, 95 },
    { 0, 0 },
    { 0xdc9cb5e5, 85 },
    { 0x9b8cb5e6, 19 },
    { 0, 0 },
    { 0xd3cb5e8, 97 },
    { 0x3c5cb5e9, 98 },
    { 0x701cb5ea, 33 },
    { 0, 0 },
    { 0xf40cb5ec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ccb5f2, 61 },
    { 0xb42cb5f2, 66 },
    { 0, 0 },
    { 0x1becb5f5, 55 },
    { 0, 0 },
    { 0xe40cb5f7, 93 },
    { 0, 0 },
    { 0x899cb5f9, 65 },
    { 0x54cb5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bcb5ff, 55 },
    { 0, 0 },
    { 0x908cb601, 23 },
    { 0xad1cb602, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63cb607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cb610, 23 },
    { 0xbd8cb610, 90 },
    { 0xbf8cb611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dcb61a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828cb623, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dcb626, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8cb62a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2cb635, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ccb638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88cb63f, 99 },
    { 0xbf7cb63f, 34 },
    { 0, 0 },
    { 0x393cb642, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413cb647, 33 },
    { 0xe12cb648, 75 },
    { 0x858cb649, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512cb653, 69 },
    { 0, 0 },
    { 0x263cb655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425cb65c, 23 },
    { 0xf0bcb65d, 40 },
    { 0xcffcb65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772cb667, 61 },
    { 0xdb7cb667, 61 },
    { 0, 0 },
    { 0xe94cb66a, 90 },
    { 0xd92cb66b, 52 },
    { 0x7facb66c, 98 },
    { 0xc83cb66c, 80 },
    { 0xb92cb66c, 33 },
    { 0x35acb66f, 55 },
    { 0x339cb66b, 91 },
    { 0, 0 },
    { 0x937cb672, 33 },
    { 0x967cb672, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82cb677, 33 },
    { 0x99bcb677, 55 },
    { 0xf9cb679, 95 },
    { 0x7c4cb679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7cb67d, 98 },
    { 0, 0 },
    { 0x2b6cb67f, 34 },
    { 0xadfcb680, 67 },
    { 0x9f2cb681, 87 },
    { 0xf0fcb682, 85 },
    { 0x68bcb682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62cb68b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1cb68f, 76 },
    { 0xa4ecb68f, 61 },
    { 0, 0 },
    { 0x834cb692, 61 },
    { 0x28ecb693, 43 },
    { 0x6aacb694, 90 },
    { 0x16fcb693, 33 },
    { 0xfcbcb695, 90 },
    { 0x695cb696, 61 },
    { 0x96fcb698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2cb69c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ccb6a0, 99 },
    { 0x818cb6a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ecb6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9cb6ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4decb6ba, 76 },
    { 0x422cb6bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cccb6be, 76 },
    { 0x7e6cb6bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6facb6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587cb6cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69cb6d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bcb6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91cb6df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cb6e6, 61 },
    { 0x7e5cb6e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ccb6ea, 66 },
    { 0, 0 },
    { 0xe80cb6ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bccb6f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cb6f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cb6fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861cb701, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6cb706, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2cb70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ecb70d, 98 },
    { 0x641cb70d, 33 },
    { 0x8d5cb70f, 34 },
    { 0x59dcb70d, 87 },
    { 0x79dcb711, 90 },
    { 0xcb2cb70f, 23 },
    { 0x46ccb710, 24 },
    { 0x959cb714, 66 },
    { 0, 0 },
    { 0x7fbcb716, 61 },
    { 0, 0 },
    { 0xfb9cb718, 46 },
    { 0xe84cb718, 98 },
    { 0xa3fcb719, 42 },
    { 0x495cb71b, 95 },
    { 0x50acb71c, 46 },
    { 0x431cb71d, 76 },
    { 0xd2fcb71d, 95 },
    { 0x369cb71f, 76 },
    { 0x9e4cb720, 34 },
    { 0x1cbcb71f, 33 },
    { 0xde3cb720, 33 },
    { 0x968cb71a, 61 },
    { 0, 0 },
    { 0x4f7cb725, 23 },
    { 0x693cb726, 33 },
    { 0xff4cb727, 91 },
    { 0x207cb727, 52 },
    { 0x57ccb727, 76 },
    { 0x3c7cb727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0cb72d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7cb734, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacb73b, 90 },
    { 0x126cb73b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3cb745, 88 },
    { 0x804cb746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8cb74a, 33 },
    { 0x9acb74a, 55 },
    { 0xb7ecb74c, 88 },
    { 0xf1bcb74d, 98 },
    { 0xd5ccb74d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fcb752, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ccb758, 61 },
    { 0xe05cb759, 76 },
    { 0xd63cb75a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x669cb75d, 43 },
    { 0x49fcb75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacccb765, 67 },
    { 0x2e6cb766, 34 },
    { 0x433cb766, 23 },
    { 0xb75cb768, 90 },
    { 0x341cb769, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4cb76d, 78 },
    { 0, 0 },
    { 0x50dcb76f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x481cb772, 95 },
    { 0xdbccb773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x247cb776, 95 },
    { 0x7a0cb777, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x643cb77a, 97 },
    { 0xbb5cb77a, 98 },
    { 0x3e7cb77b, 61 },
    { 0x5b4cb77d, 90 },
    { 0, 0 },
    { 0x518cb77f, 98 },
    { 0x3adcb780, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83cb783, 23 },
    { 0x424cb784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bcb789, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867cb798, 23 },
    { 0, 0 },
    { 0xde6cb79a, 84 },
    { 0, 0 },
    { 0x54ecb79c, 87 },
    { 0, 0 },
    { 0xac8cb79e, 33 },
    { 0x3c1cb79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55cb7a4, 95 },
    { 0, 0 },
    { 0x72ccb7a6, 87 },
    { 0x799cb7a7, 95 },
    { 0x43fcb7a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb7ab, 95 },
    { 0x6b4cb7ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdcb7b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ecb7b8, 23 },
    { 0, 0 },
    { 0x17dcb7ba, 77 },
    { 0, 0 },
    { 0xedcb7bc, 17 },
    { 0xf2fcb7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x994cb7c0, 98 },
    { 0x903cb7c1, 90 },
    { 0x327cb7c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3cb7c8, 90 },
    { 0x323cb7c9, 76 },
    { 0xd32cb7c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bcb7cd, 40 },
    { 0x9edcb7ce, 82 },
    { 0xa3acb7cf, 90 },
    { 0x16dcb7d0, 33 },
    { 0x71fcb7d0, 33 },
    { 0x20fcb7d0, 96 },
    { 0xc81cb7d3, 33 },
    { 0x2bccb7d4, 95 },
    { 0xaeecb7d5, 90 },
    { 0x30cb7d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3cb7da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5cb7df, 90 },
    { 0x41ecb7e0, 98 },
    { 0x391cb7df, 87 },
    { 0, 0 },
    { 0x6ddcb7e3, 90 },
    { 0, 0 },
    { 0xbc8cb7e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2cb7e9, 23 },
    { 0x9e0cb7ea, 33 },
    { 0x27acb7eb, 33 },
    { 0xfaecb7ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x675cb7ef, 61 },
    { 0x859cb7ef, 96 },
    { 0xb9cb7f1, 55 },
    { 0, 0 },
    { 0x2f3cb7f3, 95 },
    { 0x839cb7f3, 76 },
    { 0x24ccb7f5, 33 },
    { 0x55cb7f3, 95 },
    { 0, 0 },
    { 0x817cb7f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7cb7fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649cb7ff, 96 },
    { 0, 0 },
    { 0x235cb801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70cb804, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cb80b, 52 },
    { 0xddfcb80c, 61 },
    { 0x24dcb80d, 46 },
    { 0, 0 },
    { 0x3a1cb80f, 90 },
    { 0xa0bcb810, 33 },
    { 0x970cb811, 100 },
    { 0xd72cb812, 98 },
    { 0xacb813, 88 },
    { 0x1eacb812, 90 },
    { 0x5cccb813, 34 },
    { 0x4ebcb816, 90 },
    { 0xf9bcb817, 85 },
    { 0xde7cb814, 95 },
    { 0x42bcb819, 33 },
    { 0xca2cb81a, 76 },
    { 0x419cb81b, 34 },
    { 0xdfccb819, 25 },
    { 0xa2ccb81d, 55 },
    { 0x1fecb81d, 61 },
    { 0x5b5cb81e, 43 },
    { 0x902cb820, 23 },
    { 0xf38cb81c, 90 },
    { 0x33fcb81e, 98 },
    { 0xe67cb823, 23 },
    { 0, 0 },
    { 0xfa1cb825, 61 },
    { 0x885cb826, 98 },
    { 0, 0 },
    { 0x461cb828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08cb831, 95 },
    { 0xb71cb832, 81 },
    { 0xb9dcb833, 43 },
    { 0x888cb832, 46 },
    { 0x174cb835, 33 },
    { 0, 0 },
    { 0x9accb837, 95 },
    { 0xc02cb837, 23 },
    { 0, 0 },
    { 0xa47cb83a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81cb840, 90 },
    { 0x5b3cb840, 60 },
    { 0xbd6cb842, 95 },
    { 0xc87cb840, 23 },
    { 0xfe4cb843, 90 },
    { 0, 0 },
    { 0xf0ccb846, 88 },
    { 0, 0 },
    { 0xe4ccb848, 90 },
    { 0xc1acb849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0cb84e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddcb856, 98 },
    { 0, 0 },
    { 0x28ecb858, 48 },
    { 0xac7cb859, 90 },
    { 0x863cb85a, 33 },
    { 0, 0 },
    { 0x65cb85c, 98 },
    { 0x3d2cb85d, 61 },
    { 0x682cb85c, 61 },
    { 0x9d4cb85f, 90 },
    { 0x162cb860, 81 },
    { 0xd0fcb861, 93 },
    { 0xf6acb862, 100 },
    { 0x937cb863, 33 },
    { 0x7c0cb860, 90 },
    { 0, 0 },
    { 0xacecb866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fcb86d, 95 },
    { 0, 0 },
    { 0x90bcb86f, 90 },
    { 0x542cb870, 83 },
    { 0xfb3cb871, 85 },
    { 0xdd6cb872, 34 },
    { 0xf86cb871, 84 },
    { 0, 0 },
    { 0xe4bcb875, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8cb879, 66 },
    { 0, 0 },
    { 0xcefcb87b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cb880, 23 },
    { 0x668cb881, 95 },
    { 0x8bacb881, 55 },
    { 0x62acb883, 90 },
    { 0xbe7cb881, 55 },
    { 0xc73cb882, 43 },
    { 0, 0 },
    { 0x1ebcb887, 66 },
    { 0, 0 },
    { 0xa6ecb889, 33 },
    { 0x418cb88a, 90 },
    { 0xab8cb889, 89 },
    { 0, 0 },
    { 0x92ccb88d, 23 },
    { 0x343cb88e, 61 },
    { 0, 0 },
    { 0xfe9cb890, 46 },
    { 0xaa3cb890, 53 },
    { 0xfdbcb892, 98 },
    { 0, 0 },
    { 0x875cb894, 95 },
    { 0xcd4cb895, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dcb898, 88 },
    { 0xd56cb899, 85 },
    { 0x951cb89a, 99 },
    { 0xadecb89b, 46 },
    { 0x7bccb89a, 87 },
    { 0xe1acb89c, 34 },
    { 0x140cb899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7cb8a2, 64 },
    { 0, 0 },
    { 0x88ecb8a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb8a7, 33 },
    { 0, 0 },
    { 0x161cb8a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fcb8ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1cb8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x773cb8b5, 52 },
    { 0xc43cb8b5, 83 },
    { 0x2cacb8b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463cb8c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ecb8c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbcb8cc, 33 },
    { 0, 0 },
    { 0x3d5cb8ce, 95 },
    { 0xccacb8cf, 90 },
    { 0, 0 },
    { 0xf2acb8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6cb8d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731cb8dd, 23 },
    { 0x6b7cb8de, 76 },
    { 0xd0fcb8df, 95 },
    { 0xdc9cb8e0, 61 },
    { 0x48acb8e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12cb8eb, 76 },
    { 0xc6bcb8ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47cb8f1, 67 },
    { 0xb33cb8f1, 34 },
    { 0x2f1cb8f3, 43 },
    { 0, 0 },
    { 0xe0acb8f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bcb8f9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cb8fe, 93 },
    { 0x433cb8ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaacb908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dcb90b, 59 },
    { 0, 0 },
    { 0x733cb90d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x873cb910, 34 },
    { 0, 0 },
    { 0x8ccb912, 84 },
    { 0x605cb913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236cb919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fcb921, 80 },
    { 0x78ecb922, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x962cb925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bcb928, 23 },
    { 0x471cb929, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fcb92e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ecb932, 43 },
    { 0x9a7cb933, 61 },
    { 0x45ccb934, 61 },
    { 0x47ecb932, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ccb93a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdcb93d, 95 },
    { 0, 0 },
    { 0xc80cb93f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520cb943, 81 },
    { 0x41acb944, 99 },
    { 0xa01cb945, 33 },
    { 0x1afcb946, 33 },
    { 0x2b1cb944, 61 },
    { 0x6b4cb946, 61 },
    { 0x33acb949, 43 },
    { 0x7cecb945, 84 },
    { 0xeaecb944, 23 },
    { 0xd7dcb947, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcb94f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cccb952, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ecb958, 95 },
    { 0, 0 },
    { 0x29ccb95a, 23 },
    { 0x184cb95b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacb966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241cb96c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb970, 75 },
    { 0xf62cb971, 95 },
    { 0x3d6cb971, 23 },
    { 0xc7ecb970, 98 },
    { 0, 0 },
    { 0x34ccb975, 90 },
    { 0x163cb975, 33 },
    { 0x9c7cb977, 65 },
    { 0xbbcb978, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adcb97b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ecb97e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x787cb981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06cb989, 98 },
    { 0x117cb98a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3cb990, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8cb993, 91 },
    { 0xa9dcb994, 23 },
    { 0x46dcb995, 66 },
    { 0xfabcb993, 90 },
    { 0x8e9cb997, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6cb99a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fcb99e, 90 },
    { 0x16bcb99f, 98 },
    { 0x54fcb9a0, 90 },
    { 0xeb3cb99f, 76 },
    { 0x86bcb99e, 61 },
    { 0x5cccb99e, 23 },
    { 0xfe1cb9a4, 43 },
    { 0x346cb99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fcb9ab, 33 },
    { 0, 0 },
    { 0x804cb9ad, 99 },
    { 0x877cb9ae, 61 },
    { 0x665cb9af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x245cb9b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2cb9b7, 100 },
    { 0x3becb9b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x116cb9bb, 77 },
    { 0, 0 },
    { 0x15bcb9bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484cb9c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767cb9cf, 23 },
    { 0x5d2cb9cf, 98 },
    { 0, 0 },
    { 0xe92cb9d2, 33 },
    { 0xdc2cb9d3, 33 },
    { 0x9a3cb9d4, 34 },
    { 0x202cb9d5, 98 },
    { 0x13ecb9d5, 61 },
    { 0xe3fcb9d7, 93 },
    { 0x41ccb9d8, 95 },
    { 0x2eacb9d9, 75 },
    { 0xb12cb9d3, 55 },
    { 0x446cb9db, 33 },
    { 0xeb6cb9dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6cb9df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68cb9e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942cb9e9, 90 },
    { 0, 0 },
    { 0xf8cb9eb, 66 },
    { 0, 0 },
    { 0x7afcb9ed, 56 },
    { 0xefcb9ee, 98 },
    { 0, 0 },
    { 0x49bcb9f0, 98 },
    { 0xa87cb9f1, 95 },
    { 0x811cb9f2, 90 },
    { 0xe2ccb9f1, 61 },
    { 0x99bcb9f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8cb9f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4cb9fc, 33 },
    { 0x2cccb9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x272cba00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cba04, 23 },
    { 0, 0 },
    { 0x13fcba06, 61 },
    { 0xa02cba06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bcba0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cba0f, 90 },
    { 0xd90cba0f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598cba15, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ecba19, 46 },
    { 0x4afcba1a, 90 },
    { 0, 0 },
    { 0xe0bcba1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcba20, 55 },
    { 0x1c8cba21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0cba26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fcba29, 98 },
    { 0x88ccba2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9cba2d, 24 },
    { 0xe4dcba2e, 95 },
    { 0x969cba2f, 23 },
    { 0x35cba2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fcba3a, 90 },
    { 0xa87cba3b, 100 },
    { 0xf38cba3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6becba3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15cba42, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83cba45, 100 },
    { 0, 0 },
    { 0xa72cba47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dcba4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecba4e, 23 },
    { 0, 0 },
    { 0xb54cba50, 33 },
    { 0xa2cba51, 66 },
    { 0x879cba52, 99 },
    { 0, 0 },
    { 0x20ecba54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cba57, 33 },
    { 0, 0 },
    { 0xeb2cba59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cba61, 43 },
    { 0xa83cba62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20cba66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x235cba69, 46 },
    { 0, 0 },
    { 0x982cba6b, 33 },
    { 0x86bcba6c, 17 },
    { 0xddecba6d, 61 },
    { 0, 0 },
    { 0x5d9cba6f, 95 },
    { 0xa9ecba70, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ccba74, 23 },
    { 0xa8bcba74, 99 },
    { 0, 0 },
    { 0x58ecba77, 23 },
    { 0xc38cba77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fcba7e, 46 },
    { 0xe0dcba7e, 33 },
    { 0x1f1cba7e, 98 },
    { 0x15ecba81, 93 },
    { 0xc0ecba7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1cba85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7cba88, 33 },
    { 0xb06cba88, 95 },
    { 0xaffcba8a, 43 },
    { 0x73fcba8b, 61 },
    { 0x14ccba88, 69 },
    { 0xa19cba89, 17 },
    { 0x876cba8b, 33 },
    { 0xe6fcba8e, 95 },
    { 0x608cba88, 95 },
    { 0xde3cba8d, 93 },
    { 0xcdccba92, 95 },
    { 0xa37cba8a, 98 },
    { 0x72acba8f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341cba9d, 95 },
    { 0, 0 },
    { 0xb06cba9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccbaa3, 46 },
    { 0x709cbaa3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736cbaa8, 95 },
    { 0xee2cbaa9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bcbaad, 33 },
    { 0, 0 },
    { 0xc16cbaaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fccbab9, 34 },
    { 0, 0 },
    { 0xac9cbabb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638cbac1, 55 },
    { 0, 0 },
    { 0x415cbac3, 90 },
    { 0xe73cbac3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x910cbac7, 93 },
    { 0, 0 },
    { 0xb72cbac9, 99 },
    { 0xb7ccbac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ccbad0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbccbad5, 98 },
    { 0x546cbad6, 98 },
    { 0x9d8cbad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dcbadc, 38 },
    { 0x681cbadd, 34 },
    { 0x39ecbade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4decbae2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fcbae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7cbaea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fcbaed, 98 },
    { 0, 0 },
    { 0x8bdcbaef, 33 },
    { 0x6b4cbaef, 90 },
    { 0x6ddcbaf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cbaf6, 61 },
    { 0, 0 },
    { 0xb9ccbaf8, 46 },
    { 0x855cbaf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470cbb01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9cbb0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x915cbb12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x959cbb15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8cbb18, 33 },
    { 0x840cbb19, 75 },
    { 0, 0 },
    { 0xbd9cbb1b, 23 },
    { 0x68bcbb1b, 85 },
    { 0xebccbb1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594cbb24, 93 },
    { 0x714cbb25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60cbb28, 40 },
    { 0xd4acbb29, 95 },
    { 0xf8ecbb29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956cbb2f, 88 },
    { 0x513cbb2f, 76 },
    { 0xfa2cbb31, 98 },
    { 0x25bcbb31, 46 },
    { 0x5e8cbb33, 87 },
    { 0x52fcbb31, 76 },
    { 0x7c9cbb33, 95 },
    { 0xe2dcbb2f, 76 },
    { 0xb7ccbb37, 90 },
    { 0, 0 },
    { 0x5c9cbb39, 78 },
    { 0x74ccbb3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ecbb3f, 61 },
    { 0x6cbcbb3f, 90 },
    { 0xd4cbb41, 93 },
    { 0, 0 },
    { 0x2bcbb43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cbb46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16cbb4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cbb4f, 90 },
    { 0x502cbb50, 23 },
    { 0xd02cbb51, 52 },
    { 0xf2fcbb52, 87 },
    { 0xb74cbb53, 55 },
    { 0xe2bcbb52, 55 },
    { 0x3d4cbb52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x195cbb58, 69 },
    { 0x199cbb58, 40 },
    { 0x298cbb5a, 33 },
    { 0xae6cbb58, 95 },
    { 0x228cbb5c, 76 },
    { 0x9dcbb5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccbb64, 93 },
    { 0x6aacbb64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52acbb6c, 90 },
    { 0x6c1cbb6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcbb73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dcbb79, 33 },
    { 0x614cbb7a, 17 },
    { 0, 0 },
    { 0xc21cbb7c, 93 },
    { 0xcdbcbb7c, 67 },
    { 0x734cbb7e, 23 },
    { 0x723cbb7f, 90 },
    { 0xec0cbb7f, 55 },
    { 0xf43cbb7d, 90 },
    { 0xe5ccbb82, 23 },
    { 0xa0ccbb80, 99 },
    { 0xb1ecbb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dcbb8b, 90 },
    { 0, 0 },
    { 0x84fcbb8d, 34 },
    { 0x728cbb8e, 24 },
    { 0, 0 },
    { 0x23fcbb90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbcbb9a, 23 },
    { 0, 0 },
    { 0xfb8cbb9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0cbba1, 95 },
    { 0xfdcbba2, 23 },
    { 0xd16cbba2, 34 },
    { 0xea0cbba4, 61 },
    { 0x709cbba5, 55 },
    { 0xe40cbba3, 95 },
    { 0xfe6cbba6, 84 },
    { 0x6b4cbba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06cbbb0, 95 },
    { 0xfe4cbbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2cbbb5, 84 },
    { 0x211cbbb5, 33 },
    { 0, 0 },
    { 0xb23cbbb8, 55 },
    { 0x8f6cbbb8, 34 },
    { 0x632cbbba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88cbbc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1cbbc8, 23 },
    { 0, 0 },
    { 0xb67cbbca, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aacbbcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5cbbd2, 61 },
    { 0x2e8cbbd3, 61 },
    { 0x3b0cbbd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ecbbd7, 90 },
    { 0xa2cbbd8, 33 },
    { 0, 0 },
    { 0xf5fcbbda, 95 },
    { 0x7fbcbbda, 98 },
    { 0xad3cbbda, 98 },
    { 0x18ecbbdd, 85 },
    { 0x897cbbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbbe9, 55 },
    { 0, 0 },
    { 0x87cbbeb, 90 },
    { 0x50acbbec, 97 },
    { 0x1bacbbed, 90 },
    { 0, 0 },
    { 0xec7cbbef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bcbbf2, 95 },
    { 0x6b2cbbf2, 90 },
    { 0, 0 },
    { 0xf84cbbf5, 90 },
    { 0xf38cbbf6, 38 },
    { 0xd21cbbf7, 68 },
    { 0xb5cbbf8, 61 },
    { 0x1fcbbf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfccbc08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4cbc0b, 61 },
    { 0x82fcbc0c, 23 },
    { 0, 0 },
    { 0x738cbc0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ccbc11, 33 },
    { 0x15fcbc12, 93 },
    { 0, 0 },
    { 0x976cbc14, 99 },
    { 0x72ecbc14, 33 },
    { 0xf89cbc16, 34 },
    { 0x3b1cbc15, 90 },
    { 0, 0 },
    { 0x848cbc19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8cbc1c, 92 },
    { 0xf45cbc1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39cbc20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458cbc26, 52 },
    { 0xaccbc27, 81 },
    { 0x9dfcbc28, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561cbc2d, 98 },
    { 0x9cbc2e, 84 },
    { 0x4bbcbc2e, 43 },
    { 0, 0 },
    { 0xd0cbc31, 77 },
    { 0x575cbc31, 90 },
    { 0, 0 },
    { 0x9fccbc34, 61 },
    { 0x47dcbc34, 90 },
    { 0x125cbc36, 91 },
    { 0x9a4cbc37, 43 },
    { 0, 0 },
    { 0x603cbc39, 98 },
    { 0x2eccbc3a, 61 },
    { 0xfeecbc3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1cbc42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bcbc46, 43 },
    { 0xffdcbc46, 90 },
    { 0xa0ecbc46, 61 },
    { 0, 0 },
    { 0xef0cbc4a, 90 },
    { 0x9a0cbc4b, 99 },
    { 0x395cbc4b, 99 },
    { 0x4f8cbc4d, 33 },
    { 0x826cbc4d, 34 },
    { 0xd20cbc4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bcbc55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cbc59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cbc5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0cbc64, 33 },
    { 0, 0 },
    { 0xa6ecbc66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05cbc69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fcbc6d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66cbc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5cbc75, 33 },
    { 0x96ecbc76, 95 },
    { 0x80fcbc76, 76 },
    { 0x7d4cbc78, 85 },
    { 0x45acbc75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772cbc7d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95cbc81, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ecbc88, 95 },
    { 0, 0 },
    { 0x420cbc8a, 100 },
    { 0x15cbc8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x771cbc8e, 33 },
    { 0, 0 },
    { 0xbf7cbc90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bcbc93, 23 },
    { 0, 0 },
    { 0x14fcbc95, 95 },
    { 0xfc8cbc96, 81 },
    { 0xf89cbc97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x813cbc9a, 61 },
    { 0, 0 },
    { 0xf01cbc9c, 33 },
    { 0, 0 },
    { 0xa7dcbc9e, 93 },
    { 0x32fcbc9e, 61 },
    { 0xddcbc9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cbca3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dcbca9, 67 },
    { 0, 0 },
    { 0xeb4cbcab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ccbcb1, 95 },
    { 0x7a3cbcb2, 23 },
    { 0x2b6cbcb3, 98 },
    { 0x614cbcb4, 33 },
    { 0, 0 },
    { 0xda1cbcb6, 73 },
    { 0x20dcbcb6, 90 },
    { 0x959cbcb8, 98 },
    { 0xb34cbcb9, 23 },
    { 0x393cbcb8, 33 },
    { 0, 0 },
    { 0x2e5cbcbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cbcc0, 85 },
    { 0, 0 },
    { 0x8aacbcc2, 61 },
    { 0x75dcbcc3, 53 },
    { 0xacacbcc4, 33 },
    { 0xc30cbcc5, 77 },
    { 0x9afcbcc5, 61 },
    { 0x54dcbcc4, 90 },
    { 0x922cbcc6, 52 },
    { 0x5dbcbcc2, 55 },
    { 0, 0 },
    { 0xa56cbccb, 87 },
    { 0x3edcbccc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447cbcd3, 40 },
    { 0, 0 },
    { 0xe99cbcd5, 61 },
    { 0xaeecbcd5, 98 },
    { 0, 0 },
    { 0xa51cbcd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ecbcde, 95 },
    { 0x94cbcdf, 95 },
    { 0, 0 },
    { 0x135cbce1, 61 },
    { 0xc80cbce2, 77 },
    { 0x745cbce3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cbcee, 90 },
    { 0x6dacbcef, 60 },
    { 0x42ccbcee, 99 },
    { 0, 0 },
    { 0xd66cbcf2, 61 },
    { 0x7a8cbcf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6cbcfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90cbcfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcbd03, 98 },
    { 0, 0 },
    { 0x353cbd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8cbd0c, 67 },
    { 0x8f7cbd0d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3cbd13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9cbd17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcbd1f, 90 },
    { 0x282cbd20, 87 },
    { 0xe48cbd20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccbd24, 76 },
    { 0x80ccbd25, 34 },
    { 0xeacbd26, 98 },
    { 0xba3cbd27, 87 },
    { 0, 0 },
    { 0xc36cbd29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ecbd2c, 34 },
    { 0x53cbd2c, 90 },
    { 0x993cbd2e, 43 },
    { 0x423cbd2d, 55 },
    { 0xf8ecbd2f, 95 },
    { 0xb16cbd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdcbd36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266cbd3d, 90 },
    { 0xc0ecbd3e, 33 },
    { 0xd4cbd3d, 23 },
    { 0x136cbd40, 61 },
    { 0x63cbd41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x749cbd44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8facbd49, 93 },
    { 0xadccbd4a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54cbd4e, 33 },
    { 0x682cbd4f, 91 },
    { 0xbdecbd50, 33 },
    { 0x767cbd51, 76 },
    { 0xe54cbd4f, 61 },
    { 0xb45cbd53, 76 },
    { 0x9ecbd54, 23 },
    { 0xa81cbd55, 61 },
    { 0xf6bcbd55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7cbd5a, 87 },
    { 0x894cbd5b, 33 },
    { 0xd48cbd5c, 99 },
    { 0x689cbd5d, 33 },
    { 0x3bbcbd5a, 98 },
    { 0xfd8cbd5c, 48 },
    { 0x5dacbd60, 33 },
    { 0x356cbd61, 34 },
    { 0x934cbd62, 90 },
    { 0x10ccbd60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dcbd66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adcbd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1cbd75, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9cbd7b, 90 },
    { 0, 0 },
    { 0x747cbd7d, 52 },
    { 0x5c0cbd7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53cbd81, 95 },
    { 0, 0 },
    { 0x278cbd83, 52 },
    { 0, 0 },
    { 0x141cbd85, 61 },
    { 0, 0 },
    { 0x4bfcbd87, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07cbd8a, 46 },
    { 0x1d7cbd8b, 90 },
    { 0x51acbd8c, 95 },
    { 0x43ecbd8d, 76 },
    { 0x588cbd8d, 90 },
    { 0x72acbd8a, 84 },
    { 0xd48cbd90, 68 },
    { 0, 0 },
    { 0xe16cbd92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcbd95, 90 },
    { 0x417cbd95, 95 },
    { 0x88ecbd96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bcbd9d, 85 },
    { 0xd14cbd9d, 76 },
    { 0xd28cbd9d, 90 },
    { 0x1acbd9f, 88 },
    { 0, 0 },
    { 0xdc2cbda2, 23 },
    { 0xfaecbda2, 33 },
    { 0x51cbda3, 91 },
    { 0, 0 },
    { 0x4dbcbda6, 97 },
    { 0x705cbda7, 55 },
    { 0x372cbda8, 91 },
    { 0x8a3cbda8, 33 },
    { 0x280cbda9, 85 },
    { 0, 0 },
    { 0x6a1cbdac, 61 },
    { 0xac1cbdad, 100 },
    { 0x92fcbdac, 33 },
    { 0x919cbdad, 98 },
    { 0x5b0cbdb0, 90 },
    { 0x1e4cbdb1, 66 },
    { 0, 0 },
    { 0xd2fcbdb3, 98 },
    { 0x159cbdb3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fcbdbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bacbdc5, 23 },
    { 0x257cbdc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x892cbdc9, 33 },
    { 0x4efcbdca, 95 },
    { 0xa59cbdcb, 61 },
    { 0, 0 },
    { 0x592cbdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98acbdd1, 95 },
    { 0xc38cbdd1, 61 },
    { 0xadbcbdd3, 98 },
    { 0x887cbdd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470cbdda, 61 },
    { 0x4c6cbdda, 76 },
    { 0xf02cbddc, 61 },
    { 0x74ccbdda, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cbde3, 87 },
    { 0x99cbde4, 23 },
    { 0x49bcbde3, 55 },
    { 0xa97cbde6, 34 },
    { 0xa4dcbde6, 84 },
    { 0xca6cbde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06cbdef, 90 },
    { 0x304cbdef, 43 },
    { 0, 0 },
    { 0x7a6cbdf2, 90 },
    { 0xf13cbdf2, 33 },
    { 0, 0 },
    { 0x72fcbdf5, 61 },
    { 0, 0 },
    { 0x1a6cbdf7, 33 },
    { 0x9e8cbdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3cbdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cbdfe, 33 },
    { 0, 0 },
    { 0x2aecbe00, 55 },
    { 0, 0 },
    { 0xffcbe02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7cbe05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50cbe0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x619cbe0e, 46 },
    { 0x9fbcbe0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397cbe14, 34 },
    { 0x44fcbe15, 90 },
    { 0x4d6cbe15, 90 },
    { 0, 0 },
    { 0x459cbe18, 90 },
    { 0, 0 },
    { 0x99fcbe1a, 87 },
    { 0xe47cbe1a, 33 },
    { 0x9a2cbe1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcbe20, 33 },
    { 0x7c3cbe21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cbe24, 66 },
    { 0x52bcbe24, 76 },
    { 0x4d1cbe26, 61 },
    { 0x55ccbe27, 55 },
    { 0, 0 },
    { 0xb12cbe29, 98 },
    { 0, 0 },
    { 0x685cbe2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cbe30, 69 },
    { 0x367cbe31, 55 },
    { 0, 0 },
    { 0xe21cbe33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324cbe39, 23 },
    { 0, 0 },
    { 0x84acbe3b, 33 },
    { 0xa10cbe3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3cbe43, 100 },
    { 0xf2fcbe44, 23 },
    { 0, 0 },
    { 0x33dcbe46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25cbe4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecbe4d, 55 },
    { 0xb9acbe4e, 98 },
    { 0x2d9cbe4f, 95 },
    { 0x650cbe4d, 90 },
    { 0x29bcbe51, 46 },
    { 0xf7ccbe52, 98 },
    { 0xd92cbe52, 76 },
    { 0xe34cbe54, 55 },
    { 0x7c4cbe53, 87 },
    { 0x677cbe56, 93 },
    { 0x7f1cbe57, 76 },
    { 0xf78cbe57, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x371cbe5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757cbe62, 22 },
    { 0, 0 },
    { 0xba0cbe64, 43 },
    { 0x209cbe64, 78 },
    { 0x632cbe66, 98 },
    { 0, 0 },
    { 0xcecbe68, 23 },
    { 0xa7dcbe69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0cbe6d, 43 },
    { 0x429cbe6d, 98 },
    { 0x8e3cbe6f, 43 },
    { 0x696cbe6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dcbe79, 85 },
    { 0, 0 },
    { 0xb09cbe7b, 98 },
    { 0, 0 },
    { 0xe77cbe7d, 61 },
    { 0xdf2cbe7d, 78 },
    { 0xbe3cbe7f, 90 },
    { 0xec4cbe80, 61 },
    { 0x2a9cbe80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8cbe85, 99 },
    { 0xd1dcbe86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ccbe8c, 33 },
    { 0xc59cbe8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cbe94, 34 },
    { 0x2cecbe94, 34 },
    { 0xc21cbe96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcbe9a, 55 },
    { 0x8bcbe9b, 95 },
    { 0x644cbe9c, 95 },
    { 0xfcdcbe9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cbea4, 99 },
    { 0x4cbea5, 61 },
    { 0x7accbea4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcccbea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5cbeae, 90 },
    { 0x554cbeaf, 87 },
    { 0xbf9cbeaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4cbeb4, 55 },
    { 0x231cbeb5, 88 },
    { 0x8d7cbeb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438cbebd, 98 },
    { 0, 0 },
    { 0xececbebf, 33 },
    { 0xe01cbebf, 93 },
    { 0xb59cbec1, 76 },
    { 0x363cbec2, 61 },
    { 0, 0 },
    { 0xdaccbec4, 98 },
    { 0, 0 },
    { 0xbfcbec6, 90 },
    { 0xb1ecbec6, 33 },
    { 0xa42cbec6, 84 },
    { 0x811cbec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65cbecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20cbed0, 34 },
    { 0x3bacbed1, 63 },
    { 0, 0 },
    { 0xdf3cbed3, 23 },
    { 0x210cbed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223cbed8, 90 },
    { 0xde6cbed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbcbedd, 84 },
    { 0, 0 },
    { 0x3b7cbedf, 55 },
    { 0x488cbedf, 58 },
    { 0xf9cbee1, 33 },
    { 0xe44cbee1, 55 },
    { 0x4c4cbee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50cbee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dcbeec, 98 },
    { 0x8e4cbeed, 87 },
    { 0xe75cbeed, 95 },
    { 0x3c8cbeef, 95 },
    { 0, 0 },
    { 0xda4cbef1, 90 },
    { 0, 0 },
    { 0x734cbef3, 22 },
    { 0x68dcbef4, 23 },
    { 0x1f2cbef5, 23 },
    { 0, 0 },
    { 0x200cbef7, 43 },
    { 0xbd9cbef8, 23 },
    { 0x3bcbef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0cbefc, 85 },
    { 0x6c4cbefd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2becbf00, 98 },
    { 0, 0 },
    { 0xa17cbf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08cbf06, 97 },
    { 0x10acbf07, 23 },
    { 0, 0 },
    { 0xd5ecbf09, 99 },
    { 0x867cbf0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2cbf0e, 52 },
    { 0, 0 },
    { 0x3a5cbf10, 66 },
    { 0xf3ecbf11, 90 },
    { 0, 0 },
    { 0x2bbcbf13, 67 },
    { 0x3f8cbf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eacbf17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11cbf1d, 90 },
    { 0xac2cbf1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833cbf26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5cbf31, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dcbf39, 98 },
    { 0, 0 },
    { 0xcc4cbf3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2cbf3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40cbf4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3cbf52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606cbf57, 81 },
    { 0x6aecbf57, 95 },
    { 0, 0 },
    { 0x8f3cbf5a, 46 },
    { 0x9a5cbf5a, 43 },
    { 0x2e6cbf5c, 33 },
    { 0x4cecbf5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcbf63, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59cbf67, 98 },
    { 0, 0 },
    { 0x45bcbf69, 95 },
    { 0x5b9cbf6a, 33 },
    { 0xae4cbf6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5cbf75, 81 },
    { 0xce7cbf75, 33 },
    { 0xd20cbf77, 23 },
    { 0x60ccbf78, 90 },
    { 0x871cbf78, 55 },
    { 0x2accbf79, 66 },
    { 0, 0 },
    { 0x924cbf7c, 76 },
    { 0x9c0cbf7d, 95 },
    { 0, 0 },
    { 0x763cbf7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346cbf86, 90 },
    { 0x96acbf86, 23 },
    { 0x4bacbf88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cbf8b, 34 },
    { 0, 0 },
    { 0xbbdcbf8d, 92 },
    { 0x108cbf8e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36cbf92, 93 },
    { 0xadcbf93, 55 },
    { 0x4d8cbf94, 92 },
    { 0xf2ccbf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17acbf98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fcbf9b, 90 },
    { 0, 0 },
    { 0x52cbf9d, 34 },
    { 0x60acbf9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4cbfa4, 99 },
    { 0x2aacbfa5, 98 },
    { 0x9eecbfa6, 90 },
    { 0, 0 },
    { 0x4bccbfa8, 93 },
    { 0, 0 },
    { 0xc3fcbfaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbfad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cbfb7, 33 },
    { 0x677cbfb8, 80 },
    { 0, 0 },
    { 0x28fcbfba, 97 },
    { 0x1b2cbfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ccbfbe, 90 },
    { 0, 0 },
    { 0xf6ccbfc0, 61 },
    { 0x1dacbfc1, 55 },
    { 0xb80cbfc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577cbfc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ecbfcc, 34 },
    { 0x10bcbfcc, 98 },
    { 0, 0 },
    { 0x86dcbfcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8cbfd3, 23 },
    { 0, 0 },
    { 0x700cbfd5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cbfdb, 98 },
    { 0, 0 },
    { 0x6e6cbfdd, 52 },
    { 0, 0 },
    { 0x3b8cbfdf, 95 },
    { 0, 0 },
    { 0x90fcbfe1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cbfe9, 43 },
    { 0, 0 },
    { 0x27fcbfeb, 38 },
    { 0, 0 },
    { 0xcf5cbfed, 61 },
    { 0x58acbfee, 90 },
    { 0x7accbfee, 91 },
    { 0xf1fcbfee, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccbff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86cbffa, 61 },
    { 0, 0 },
    { 0xf7cbffc, 90 },
    { 0, 0 },
    { 0x102cbffe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44acc002, 90 },
    { 0x71ecc002, 99 },
    { 0xb7bcc002, 95 },
    { 0x30dcc005, 98 },
    { 0x23bcc006, 40 },
    { 0x6d9cc005, 33 },
    { 0xafecc005, 55 },
    { 0x97ccc009, 90 },
    { 0x5b7cc00a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233cc010, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3cc015, 61 },
    { 0, 0 },
    { 0x40ccc017, 23 },
    { 0x7e7cc017, 34 },
    { 0, 0 },
    { 0x3cc01a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519cc022, 90 },
    { 0xf24cc023, 91 },
    { 0xcb6cc023, 34 },
    { 0x7e6cc025, 23 },
    { 0x633cc025, 85 },
    { 0x372cc027, 77 },
    { 0xa9ccc028, 90 },
    { 0, 0 },
    { 0xf4dcc02a, 78 },
    { 0xac6cc02b, 33 },
    { 0, 0 },
    { 0x87ecc02d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954cc031, 76 },
    { 0x98ecc031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbcc036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cacc03c, 95 },
    { 0xaaccc03d, 91 },
    { 0x2abcc03e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597cc043, 61 },
    { 0x1a6cc044, 95 },
    { 0x1b4cc044, 99 },
    { 0x933cc046, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfcc051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x766cc054, 33 },
    { 0x2b8cc055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fcc059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc05c, 61 },
    { 0, 0 },
    { 0x2cdcc05e, 33 },
    { 0xe0fcc05f, 81 },
    { 0x7cbcc05e, 75 },
    { 0, 0 },
    { 0x3c7cc062, 91 },
    { 0x8cfcc063, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x290cc066, 95 },
    { 0x7a3cc067, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8cc06e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaecc074, 78 },
    { 0x864cc074, 95 },
    { 0xb59cc075, 34 },
    { 0x950cc075, 17 },
    { 0x2afcc075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180cc07e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49cc084, 93 },
    { 0xf74cc085, 85 },
    { 0x7cbcc085, 99 },
    { 0xed2cc086, 98 },
    { 0xf5bcc088, 17 },
    { 0x445cc089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7cc08f, 95 },
    { 0xb0bcc090, 43 },
    { 0x50fcc091, 91 },
    { 0, 0 },
    { 0xea5cc093, 98 },
    { 0x6b5cc094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20cc098, 46 },
    { 0, 0 },
    { 0x7d8cc09a, 78 },
    { 0x40dcc09b, 53 },
    { 0x5facc09b, 100 },
    { 0, 0 },
    { 0xe05cc09e, 98 },
    { 0x41acc09f, 55 },
    { 0x22cc0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dcc0a3, 33 },
    { 0x812cc0a4, 90 },
    { 0x175cc0a5, 61 },
    { 0x109cc0a5, 61 },
    { 0xd78cc0a5, 55 },
    { 0xb45cc0a8, 23 },
    { 0x9eccc0a4, 87 },
    { 0xc78cc0aa, 33 },
    { 0xb52cc0aa, 95 },
    { 0xbafcc0ac, 91 },
    { 0xd25cc0ad, 98 },
    { 0x5e8cc0aa, 98 },
    { 0x2bdcc0af, 98 },
    { 0xd97cc0af, 87 },
    { 0xbc8cc0a6, 76 },
    { 0, 0 },
    { 0xf49cc0b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcc0b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ecc0ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbcc0bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eecc0c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7cc0cf, 95 },
    { 0xc74cc0d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dcc0d7, 90 },
    { 0x89ccc0d8, 66 },
    { 0x72dcc0d9, 61 },
    { 0x15fcc0da, 52 },
    { 0x2efcc0db, 56 },
    { 0x5b3cc0d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbcc0e4, 93 },
    { 0xdc8cc0e5, 69 },
    { 0xa00cc0e6, 76 },
    { 0xefecc0e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fcc0ee, 99 },
    { 0x76ecc0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdcc0f2, 55 },
    { 0xb0ecc0f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4cc0f8, 98 },
    { 0xf61cc0f9, 91 },
    { 0xf5fcc0fa, 23 },
    { 0x104cc0fb, 78 },
    { 0xc06cc0fc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247cc10d, 90 },
    { 0x27cc10e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bcc111, 33 },
    { 0xac9cc112, 90 },
    { 0x877cc113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5cc117, 9 },
    { 0xeccc117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67cc11d, 23 },
    { 0, 0 },
    { 0x308cc11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdcc124, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cc127, 33 },
    { 0x9fccc127, 61 },
    { 0x69acc128, 23 },
    { 0xbcbcc129, 61 },
    { 0x3dacc129, 61 },
    { 0x159cc12c, 33 },
    { 0, 0 },
    { 0x609cc12e, 98 },
    { 0xfb9cc12e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382cc135, 95 },
    { 0, 0 },
    { 0x8c2cc137, 65 },
    { 0, 0 },
    { 0x3a1cc139, 76 },
    { 0x192cc139, 23 },
    { 0x290cc139, 90 },
    { 0, 0 },
    { 0xc74cc13d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70cc142, 90 },
    { 0, 0 },
    { 0xb98cc144, 34 },
    { 0x8f9cc145, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9cc148, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494cc14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61cc155, 23 },
    { 0xf7ccc155, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dcc159, 46 },
    { 0x7efcc15a, 81 },
    { 0x822cc15a, 95 },
    { 0x753cc15a, 23 },
    { 0xc53cc15b, 61 },
    { 0, 0 },
    { 0x8d4cc15f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x663cc162, 68 },
    { 0xe92cc163, 23 },
    { 0, 0 },
    { 0xd75cc165, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edcc168, 76 },
    { 0x4ffcc169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2cc16e, 66 },
    { 0, 0 },
    { 0x73bcc170, 90 },
    { 0x699cc171, 33 },
    { 0x8cccc172, 61 },
    { 0x21fcc173, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eecc179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6cc17d, 23 },
    { 0, 0 },
    { 0xaa0cc17f, 23 },
    { 0x8fbcc17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x117cc183, 98 },
    { 0x98ecc184, 91 },
    { 0, 0 },
    { 0xa04cc186, 91 },
    { 0x9a8cc187, 100 },
    { 0x5f7cc188, 98 },
    { 0, 0 },
    { 0x301cc18a, 66 },
    { 0xb5cc18b, 76 },
    { 0xec7cc18c, 91 },
    { 0xcb0cc18c, 98 },
    { 0x697cc18e, 95 },
    { 0xdebcc18f, 90 },
    { 0x5b2cc18b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35cc198, 95 },
    { 0, 0 },
    { 0x2bccc19a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcc19d, 98 },
    { 0, 0 },
    { 0x801cc19f, 61 },
    { 0, 0 },
    { 0x887cc1a1, 98 },
    { 0, 0 },
    { 0x9d8cc1a3, 93 },
    { 0xa39cc1a4, 38 },
    { 0xd6cc1a5, 33 },
    { 0x8bacc1a3, 47 },
    { 0, 0 },
    { 0x511cc1a8, 52 },
    { 0x3e6cc1a9, 94 },
    { 0x7b9cc1a9, 65 },
    { 0x5afcc1a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137cc1b1, 61 },
    { 0, 0 },
    { 0x53bcc1b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9cc1b6, 95 },
    { 0x570cc1b6, 81 },
    { 0, 0 },
    { 0xbb2cc1b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfcc1bd, 61 },
    { 0xd68cc1be, 23 },
    { 0x468cc1bd, 90 },
    { 0x7a0cc1be, 90 },
    { 0x7e6cc1c1, 23 },
    { 0x436cc1c2, 33 },
    { 0, 0 },
    { 0x7f2cc1c4, 33 },
    { 0xc46cc1c5, 33 },
    { 0x433cc1c6, 33 },
    { 0xc38cc1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175cc1cf, 61 },
    { 0xdd7cc1cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fcc1d5, 98 },
    { 0, 0 },
    { 0x10fcc1d7, 43 },
    { 0x468cc1d8, 43 },
    { 0x445cc1d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ecc1dc, 90 },
    { 0, 0 },
    { 0xbefcc1de, 17 },
    { 0xc13cc1df, 33 },
    { 0xd1fcc1df, 76 },
    { 0, 0 },
    { 0x100cc1e2, 40 },
    { 0x7b3cc1e3, 90 },
    { 0xe6cc1e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485cc1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5cc1eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edcc1f0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bacc1f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fcc1f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffcc201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1cc206, 93 },
    { 0, 0 },
    { 0x2dacc208, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdcc20b, 33 },
    { 0x4ffcc20c, 90 },
    { 0xfbcc20d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609cc211, 93 },
    { 0xf94cc212, 55 },
    { 0x359cc213, 76 },
    { 0xd1acc214, 61 },
    { 0x93dcc213, 41 },
    { 0x7ccc216, 52 },
    { 0x8fccc216, 76 },
    { 0xa75cc218, 55 },
    { 0xb6ecc215, 90 },
    { 0x9f5cc21a, 55 },
    { 0x85acc21b, 78 },
    { 0xdb9cc21c, 91 },
    { 0x2a1cc21d, 55 },
    { 0x9d7cc21e, 53 },
    { 0xa91cc21c, 95 },
    { 0xfd5cc220, 52 },
    { 0xab1cc215, 93 },
    { 0xb40cc21d, 44 },
    { 0x8bfcc215, 90 },
    { 0, 0 },
    { 0x2d6cc225, 95 },
    { 0, 0 },
    { 0x6bccc227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x104cc22a, 23 },
    { 0x65fcc22b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddcc231, 90 },
    { 0, 0 },
    { 0x2f2cc233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47acc239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17cc240, 23 },
    { 0, 0 },
    { 0xf1ecc242, 90 },
    { 0, 0 },
    { 0x99bcc244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623cc251, 90 },
    { 0x873cc252, 61 },
    { 0x54acc252, 90 },
    { 0x530cc253, 98 },
    { 0x7ecc251, 100 },
    { 0x867cc256, 43 },
    { 0x353cc254, 75 },
    { 0x545cc258, 91 },
    { 0x978cc259, 61 },
    { 0, 0 },
    { 0x7adcc25b, 61 },
    { 0, 0 },
    { 0xc34cc25d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9cc260, 23 },
    { 0xeaecc260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ecc264, 95 },
    { 0x1abcc265, 95 },
    { 0x9d8cc266, 90 },
    { 0, 0 },
    { 0xd8acc268, 95 },
    { 0x325cc268, 98 },
    { 0, 0 },
    { 0x872cc26b, 46 },
    { 0, 0 },
    { 0x8c4cc26d, 61 },
    { 0x88dcc26e, 98 },
    { 0xdc5cc26f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912cc273, 33 },
    { 0xfbbcc274, 33 },
    { 0, 0 },
    { 0xc53cc276, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72cc27e, 80 },
    { 0x5eccc27f, 98 },
    { 0x9f8cc280, 23 },
    { 0x7dccc280, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228cc285, 33 },
    { 0xf1ecc285, 33 },
    { 0, 0 },
    { 0xc0ecc288, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x830cc28b, 75 },
    { 0x8efcc28c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cc295, 93 },
    { 0xf2cc296, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc29e, 33 },
    { 0, 0 },
    { 0x4eccc2a0, 61 },
    { 0x8c9cc2a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618cc2ac, 76 },
    { 0xc8ccc2ad, 61 },
    { 0xc12cc2ae, 90 },
    { 0x1c3cc2ad, 90 },
    { 0xd08cc2b0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cc2be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28acc2c5, 23 },
    { 0x28dcc2c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cc2cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0cc2ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8cc2d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14cc2d4, 56 },
    { 0xd23cc2d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178cc2de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cc2e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacc2e6, 46 },
    { 0x954cc2e7, 33 },
    { 0x696cc2e8, 97 },
    { 0xa7ecc2e9, 34 },
    { 0x76cc2e8, 93 },
    { 0, 0 },
    { 0x4fdcc2ec, 88 },
    { 0, 0 },
    { 0xfa3cc2ee, 98 },
    { 0x4e7cc2ee, 33 },
    { 0x840cc2f0, 33 },
    { 0x691cc2f1, 76 },
    { 0xa18cc2ef, 90 },
    { 0xf43cc2f3, 98 },
    { 0x7cfcc2f3, 69 },
    { 0, 0 },
    { 0xe5bcc2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fcc2f9, 33 },
    { 0x915cc2fa, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7cc2fe, 61 },
    { 0x1cbcc2fe, 55 },
    { 0, 0 },
    { 0x7e4cc301, 90 },
    { 0, 0 },
    { 0x9ddcc303, 96 },
    { 0x216cc304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440cc30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67cc312, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x252cc315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bcc318, 95 },
    { 0x95dcc319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23cc31d, 61 },
    { 0x72ecc31d, 66 },
    { 0, 0 },
    { 0x8e5cc320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917cc327, 23 },
    { 0xfe2cc328, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90cc32b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0cc332, 52 },
    { 0x1e9cc333, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ecc337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bcc33c, 90 },
    { 0, 0 },
    { 0x658cc33e, 90 },
    { 0, 0 },
    { 0x9c9cc340, 90 },
    { 0x46bcc340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269cc34a, 33 },
    { 0, 0 },
    { 0x275cc34c, 33 },
    { 0, 0 },
    { 0x685cc34e, 90 },
    { 0x408cc34f, 66 },
    { 0, 0 },
    { 0x7e5cc351, 69 },
    { 0xa4acc351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852cc35e, 34 },
    { 0, 0 },
    { 0xd0bcc360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaccc363, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x428cc366, 46 },
    { 0, 0 },
    { 0xa9ccc368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11cc36b, 61 },
    { 0xdfdcc36c, 23 },
    { 0x877cc36d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3cc370, 17 },
    { 0, 0 },
    { 0x64cc372, 95 },
    { 0xf0bcc372, 59 },
    { 0xc9ccc374, 33 },
    { 0xcbacc375, 93 },
    { 0x83ecc373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7cc379, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3cc37e, 98 },
    { 0xa00cc37e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x224cc382, 95 },
    { 0, 0 },
    { 0xa2acc384, 95 },
    { 0x6f3cc385, 33 },
    { 0x323cc385, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24cc38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4cc38e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1cc393, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cc39a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02cc39f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcc3a4, 55 },
    { 0xe3bcc3a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85cc3a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72cc3ae, 98 },
    { 0x46ccc3ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ccc3b2, 90 },
    { 0, 0 },
    { 0x32ecc3b4, 70 },
    { 0, 0 },
    { 0x531cc3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46cc3b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7cc3c1, 55 },
    { 0x55bcc3c1, 90 },
    { 0x611cc3c3, 89 },
    { 0x1ecc3c4, 91 },
    { 0xe04cc3c4, 98 },
    { 0x536cc3c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cc3ca, 33 },
    { 0xbfacc3ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dcc3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284cc3d9, 46 },
    { 0x3f6cc3da, 76 },
    { 0x948cc3d9, 55 },
    { 0xd1dcc3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5cc3e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cc3e9, 33 },
    { 0, 0 },
    { 0x8e4cc3eb, 55 },
    { 0x9f7cc3ec, 98 },
    { 0x1a6cc3ed, 61 },
    { 0xf42cc3ee, 95 },
    { 0x4d6cc3ef, 23 },
    { 0, 0 },
    { 0xbd2cc3f1, 23 },
    { 0x3f1cc3f1, 95 },
    { 0x620cc3f3, 23 },
    { 0x870cc3f4, 91 },
    { 0x2c2cc3f3, 61 },
    { 0x850cc3f2, 87 },
    { 0xefdcc3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ecc3fb, 86 },
    { 0xb28cc3fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcc403, 17 },
    { 0xd39cc404, 34 },
    { 0x3fccc405, 61 },
    { 0xba7cc406, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04cc409, 95 },
    { 0x6eacc409, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5cc40d, 78 },
    { 0x264cc40e, 76 },
    { 0xca5cc40e, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ecc414, 78 },
    { 0x30ecc414, 90 },
    { 0xcdbcc416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cc419, 55 },
    { 0x936cc41a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84cc41d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1cc420, 33 },
    { 0x796cc421, 61 },
    { 0x310cc422, 23 },
    { 0x845cc422, 23 },
    { 0x76cc424, 33 },
    { 0xb0bcc425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61cc42d, 46 },
    { 0, 0 },
    { 0xba2cc42f, 61 },
    { 0, 0 },
    { 0xf17cc431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcc436, 8 },
    { 0x67ccc437, 23 },
    { 0x592cc438, 76 },
    { 0x927cc438, 34 },
    { 0x924cc43a, 61 },
    { 0, 0 },
    { 0x718cc43c, 33 },
    { 0x7b8cc43c, 3 },
    { 0x900cc43e, 23 },
    { 0xf29cc43e, 99 },
    { 0, 0 },
    { 0x404cc441, 34 },
    { 0x2a7cc442, 98 },
    { 0xcd0cc443, 33 },
    { 0xf41cc442, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc449, 81 },
    { 0xf74cc44a, 90 },
    { 0, 0 },
    { 0xa67cc44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fcc453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4cc456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3cc45b, 34 },
    { 0, 0 },
    { 0xfdbcc45d, 88 },
    { 0xacacc45e, 90 },
    { 0xfa1cc45d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377cc465, 46 },
    { 0xc1bcc465, 56 },
    { 0, 0 },
    { 0x3c0cc468, 43 },
    { 0x612cc469, 17 },
    { 0xdb4cc46a, 78 },
    { 0x5b9cc46a, 90 },
    { 0x98dcc46c, 76 },
    { 0x415cc46b, 33 },
    { 0, 0 },
    { 0x4fcc46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdcc473, 90 },
    { 0x775cc474, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12cc477, 60 },
    { 0, 0 },
    { 0x5e6cc479, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bcc47f, 33 },
    { 0x5ffcc480, 55 },
    { 0, 0 },
    { 0xe5acc482, 23 },
    { 0, 0 },
    { 0xb25cc484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599cc48b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dcc48f, 98 },
    { 0x228cc490, 61 },
    { 0x8eacc491, 23 },
    { 0x7a7cc492, 55 },
    { 0x9c8cc493, 33 },
    { 0x742cc494, 98 },
    { 0x1c6cc495, 91 },
    { 0x958cc496, 98 },
    { 0x380cc493, 43 },
    { 0x4ebcc492, 95 },
    { 0, 0 },
    { 0x3cacc49a, 34 },
    { 0x829cc49a, 33 },
    { 0x4c6cc49c, 66 },
    { 0x1c4cc49d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecc4a9, 88 },
    { 0, 0 },
    { 0xca7cc4ab, 61 },
    { 0x9b8cc4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x427cc4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc4b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eacc4b7, 43 },
    { 0x1b8cc4b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4cc4c1, 93 },
    { 0, 0 },
    { 0xe6dcc4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ecc4ca, 98 },
    { 0x6ccc4cb, 34 },
    { 0xb7bcc4cc, 98 },
    { 0, 0 },
    { 0x2b6cc4ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99cc4d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50cc4d6, 76 },
    { 0x273cc4d7, 84 },
    { 0xfc2cc4d7, 17 },
    { 0x2fccc4d9, 99 },
    { 0x30ccc4da, 77 },
    { 0x59fcc4d9, 81 },
    { 0xf79cc4da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dcc4e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739cc4e7, 55 },
    { 0x845cc4e7, 65 },
    { 0xbcccc4e9, 61 },
    { 0x3c6cc4ea, 33 },
    { 0xd32cc4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bcc4f9, 66 },
    { 0xd5acc4fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858cc4fe, 17 },
    { 0x69fcc4ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655cc503, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1acc506, 90 },
    { 0x5e8cc506, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x914cc50a, 90 },
    { 0x577cc50b, 91 },
    { 0xb70cc50b, 33 },
    { 0x20ecc50b, 98 },
    { 0x432cc50e, 91 },
    { 0, 0 },
    { 0x5edcc510, 48 },
    { 0, 0 },
    { 0x1cacc512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dccc515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604cc523, 90 },
    { 0, 0 },
    { 0x275cc525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cc529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ccc52c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5cc52f, 95 },
    { 0x181cc530, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5cc533, 85 },
    { 0, 0 },
    { 0x4cacc535, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496cc53a, 90 },
    { 0, 0 },
    { 0x609cc53c, 91 },
    { 0xa3ecc53c, 91 },
    { 0, 0 },
    { 0xdd6cc53f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868cc543, 100 },
    { 0xd8bcc543, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47cc54c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02cc550, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cc553, 77 },
    { 0x305cc554, 61 },
    { 0x4cecc553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ecc560, 76 },
    { 0x961cc561, 81 },
    { 0x96bcc560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39cc565, 95 },
    { 0xce5cc566, 33 },
    { 0xf3bcc567, 55 },
    { 0xe0ecc567, 33 },
    { 0x722cc568, 46 },
    { 0x148cc56a, 55 },
    { 0x83acc56b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dcc56e, 55 },
    { 0xb11cc56e, 84 },
    { 0x649cc56e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6cc573, 34 },
    { 0x5c3cc574, 90 },
    { 0x274cc575, 23 },
    { 0xcefcc575, 90 },
    { 0xdaecc577, 90 },
    { 0xff7cc578, 98 },
    { 0, 0 },
    { 0xa0fcc57a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6cc583, 76 },
    { 0xf68cc584, 98 },
    { 0xe2ecc583, 61 },
    { 0, 0 },
    { 0x247cc587, 43 },
    { 0x321cc587, 76 },
    { 0x4e3cc589, 98 },
    { 0x927cc589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cacc58f, 85 },
    { 0x69ecc590, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367cc596, 55 },
    { 0x8f9cc597, 61 },
    { 0, 0 },
    { 0x6a5cc599, 33 },
    { 0xd33cc59a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7acc59d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384cc5a1, 90 },
    { 0, 0 },
    { 0x70dcc5a3, 90 },
    { 0x98bcc5a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4cc5ab, 23 },
    { 0, 0 },
    { 0x8e0cc5ad, 33 },
    { 0, 0 },
    { 0x5cccc5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cc5b3, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336cc5b7, 90 },
    { 0xdb1cc5b7, 55 },
    { 0x1cecc5b8, 23 },
    { 0xb80cc5b9, 33 },
    { 0xfbfcc5bb, 90 },
    { 0xc3dcc5bb, 76 },
    { 0x97fcc5bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc5c1, 90 },
    { 0xa63cc5c2, 90 },
    { 0, 0 },
    { 0xad3cc5c4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dcc5c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16cc5ca, 96 },
    { 0, 0 },
    { 0xdedcc5cc, 90 },
    { 0xe91cc5cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecc5d5, 55 },
    { 0x12dcc5d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3cc5dd, 98 },
    { 0x766cc5de, 90 },
    { 0xf08cc5df, 93 },
    { 0x5efcc5e0, 90 },
    { 0, 0 },
    { 0x70cc5e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cc5e8, 98 },
    { 0x7f5cc5e9, 76 },
    { 0, 0 },
    { 0x9cdcc5eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670cc5f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5cc5fb, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86cc603, 90 },
    { 0, 0 },
    { 0x639cc605, 61 },
    { 0xc97cc606, 98 },
    { 0xd3ccc605, 90 },
    { 0x70fcc608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537cc611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8acc616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249cc61a, 95 },
    { 0, 0 },
    { 0x3fecc61c, 66 },
    { 0, 0 },
    { 0xc04cc61e, 90 },
    { 0x51dcc61e, 90 },
    { 0, 0 },
    { 0x42acc621, 34 },
    { 0, 0 },
    { 0x212cc623, 95 },
    { 0x659cc624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cc627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75cc632, 77 },
    { 0x3cacc633, 61 },
    { 0x960cc633, 34 },
    { 0xf3fcc634, 98 },
    { 0, 0 },
    { 0x13acc637, 80 },
    { 0xa6ecc638, 95 },
    { 0x7e4cc639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dcc63f, 90 },
    { 0, 0 },
    { 0x4bbcc641, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x989cc644, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ecc649, 78 },
    { 0x3aecc64a, 68 },
    { 0xdfcc64b, 61 },
    { 0x4accc64a, 17 },
    { 0x9f4cc64d, 64 },
    { 0xf90cc64e, 23 },
    { 0xbd5cc64f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x742cc652, 76 },
    { 0x8bccc653, 33 },
    { 0, 0 },
    { 0x1a5cc655, 85 },
    { 0xbaecc655, 98 },
    { 0x668cc657, 98 },
    { 0x94dcc657, 76 },
    { 0x5c4cc659, 76 },
    { 0, 0 },
    { 0x7dcc65b, 90 },
    { 0x9bbcc65c, 23 },
    { 0, 0 },
    { 0x2a3cc65e, 55 },
    { 0xd6dcc65e, 95 },
    { 0xe86cc65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ccc664, 23 },
    { 0x518cc664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecc66b, 100 },
    { 0, 0 },
    { 0xbedcc66d, 33 },
    { 0x5e6cc66e, 34 },
    { 0x8b9cc66f, 90 },
    { 0x44cc66f, 91 },
    { 0x340cc671, 77 },
    { 0xb6dcc670, 95 },
    { 0xf21cc671, 55 },
    { 0x93dcc674, 55 },
    { 0xb88cc675, 33 },
    { 0, 0 },
    { 0xf91cc677, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775cc67b, 84 },
    { 0, 0 },
    { 0xa09cc67d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dcc684, 43 },
    { 0, 0 },
    { 0xf8dcc686, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ccc68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5cc68d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cc690, 33 },
    { 0x8eecc691, 90 },
    { 0, 0 },
    { 0x240cc693, 23 },
    { 0x1fcc694, 61 },
    { 0x5a3cc695, 33 },
    { 0xf82cc696, 99 },
    { 0x5fecc696, 46 },
    { 0, 0 },
    { 0xb32cc699, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbcc6a5, 33 },
    { 0, 0 },
    { 0x7f1cc6a7, 55 },
    { 0x174cc6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32cc6ab, 23 },
    { 0xb7bcc6ac, 98 },
    { 0x290cc6ac, 93 },
    { 0x52dcc6ad, 95 },
    { 0xe71cc6ac, 38 },
    { 0, 0 },
    { 0x509cc6b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fcc6b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293cc6bc, 90 },
    { 0, 0 },
    { 0xdb7cc6be, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fcc6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadecc6c9, 23 },
    { 0x102cc6ca, 78 },
    { 0, 0 },
    { 0xb4acc6cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43cc6d0, 95 },
    { 0xa01cc6d1, 33 },
    { 0, 0 },
    { 0xa7cc6d3, 95 },
    { 0xdb9cc6d4, 98 },
    { 0x967cc6d5, 34 },
    { 0x27cc6d3, 90 },
    { 0, 0 },
    { 0x346cc6d8, 61 },
    { 0x28bcc6d9, 99 },
    { 0, 0 },
    { 0xdefcc6db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bcc6e0, 90 },
    { 0x7eecc6e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2cc6e4, 90 },
    { 0, 0 },
    { 0xe40cc6e6, 34 },
    { 0, 0 },
    { 0x152cc6e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12cc6f0, 55 },
    { 0x21cc6f1, 98 },
    { 0x2f3cc6f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x235cc6f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x789cc6f8, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ecc70e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cc716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfcc71c, 100 },
    { 0x6abcc71d, 78 },
    { 0xf3dcc71e, 90 },
    { 0, 0 },
    { 0xecacc720, 23 },
    { 0, 0 },
    { 0xd7ccc722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72acc727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fcc72d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7cc730, 34 },
    { 0, 0 },
    { 0xf69cc732, 55 },
    { 0, 0 },
    { 0x374cc734, 33 },
    { 0x6dfcc735, 23 },
    { 0x7cc735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93cc739, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fcc741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89cc746, 67 },
    { 0x951cc747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1cc74d, 55 },
    { 0x26fcc74e, 17 },
    { 0x3c6cc74e, 95 },
    { 0, 0 },
    { 0xf14cc751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4cc755, 33 },
    { 0x20acc756, 66 },
    { 0x91cc757, 46 },
    { 0x57fcc757, 61 },
    { 0x795cc756, 98 },
    { 0x81ecc758, 34 },
    { 0x953cc759, 93 },
    { 0x307cc75c, 98 },
    { 0, 0 },
    { 0xfc7cc75e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9cc764, 90 },
    { 0, 0 },
    { 0xb3ecc766, 98 },
    { 0x802cc767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fcc76b, 66 },
    { 0x790cc76c, 61 },
    { 0x619cc76d, 98 },
    { 0x23fcc76e, 55 },
    { 0xd23cc76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfcc775, 23 },
    { 0x1e0cc775, 61 },
    { 0, 0 },
    { 0xdafcc778, 23 },
    { 0, 0 },
    { 0x88dcc77a, 87 },
    { 0x48ecc77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96acc786, 61 },
    { 0x5c8cc787, 90 },
    { 0x48acc788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc78f, 47 },
    { 0x1bbcc790, 33 },
    { 0x82ccc791, 55 },
    { 0, 0 },
    { 0xc5ccc793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4cc79c, 33 },
    { 0x5fbcc79d, 61 },
    { 0, 0 },
    { 0x28ccc79f, 85 },
    { 0xc9dcc7a0, 61 },
    { 0x536cc7a1, 75 },
    { 0xdc6cc7a2, 33 },
    { 0x43dcc7a2, 61 },
    { 0x47ecc7a2, 43 },
    { 0x401cc7a5, 90 },
    { 0x1accc79f, 78 },
    { 0xcaacc7a3, 34 },
    { 0x792cc7a8, 35 },
    { 0xe96cc7a9, 34 },
    { 0x7ebcc7aa, 78 },
    { 0, 0 },
    { 0xb4dcc7ac, 90 },
    { 0xf26cc7ad, 33 },
    { 0, 0 },
    { 0x75ecc7af, 33 },
    { 0, 0 },
    { 0xbdbcc7b1, 55 },
    { 0xa5bcc7b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x141cc7b5, 76 },
    { 0x676cc7b5, 95 },
    { 0xccccc7b7, 17 },
    { 0x5becc7b6, 56 },
    { 0x8d1cc7b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337cc7c0, 61 },
    { 0, 0 },
    { 0x35acc7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243cc7c5, 81 },
    { 0, 0 },
    { 0xdf6cc7c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5cc7ca, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc7ce, 90 },
    { 0x53bcc7ce, 61 },
    { 0, 0 },
    { 0xec8cc7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dcc7d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99cc7da, 98 },
    { 0x6d5cc7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55cc7e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13cc7e6, 33 },
    { 0, 0 },
    { 0xd1acc7e8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdcc7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84cc7f5, 43 },
    { 0xf07cc7f5, 95 },
    { 0x6b7cc7f5, 33 },
    { 0, 0 },
    { 0xd96cc7f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62cc802, 77 },
    { 0x76fcc803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaccc807, 34 },
    { 0x6a4cc808, 61 },
    { 0x450cc809, 55 },
    { 0, 0 },
    { 0x661cc80b, 88 },
    { 0, 0 },
    { 0x4c6cc80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4cc812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2cc817, 90 },
    { 0x572cc817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313cc81d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8cc822, 93 },
    { 0, 0 },
    { 0x7fbcc824, 34 },
    { 0x5a1cc825, 78 },
    { 0, 0 },
    { 0x579cc827, 88 },
    { 0xe7ecc828, 67 },
    { 0x13acc828, 90 },
    { 0x429cc828, 98 },
    { 0xaa7cc82b, 61 },
    { 0x384cc82a, 17 },
    { 0xfa3cc827, 33 },
    { 0, 0 },
    { 0x733cc82f, 68 },
    { 0x5a2cc82f, 88 },
    { 0xf09cc830, 91 },
    { 0x29ccc832, 33 },
    { 0x68dcc82f, 77 },
    { 0x58dcc833, 33 },
    { 0xe20cc835, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcc838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8cc83b, 78 },
    { 0, 0 },
    { 0x92ccc83d, 98 },
    { 0x7dccc83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37cc841, 61 },
    { 0x768cc841, 95 },
    { 0xfd3cc843, 65 },
    { 0xb3dcc843, 61 },
    { 0xaeacc844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ccc84a, 98 },
    { 0xdb4cc84a, 78 },
    { 0xda1cc84c, 91 },
    { 0xaedcc84c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37cc852, 33 },
    { 0x64cc853, 80 },
    { 0xe81cc854, 68 },
    { 0x767cc854, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc859, 33 },
    { 0x125cc859, 34 },
    { 0x698cc85b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cc85e, 90 },
    { 0x24fcc85e, 93 },
    { 0x2ecc860, 90 },
    { 0xee9cc861, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54cc865, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcc869, 61 },
    { 0, 0 },
    { 0xb34cc86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeccc86f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498cc873, 66 },
    { 0x21ccc873, 84 },
    { 0, 0 },
    { 0xd70cc876, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9cc879, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedcc87e, 61 },
    { 0, 0 },
    { 0x8e4cc880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697cc883, 77 },
    { 0x53dcc884, 98 },
    { 0x4afcc885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81cc88a, 93 },
    { 0x210cc88a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ccc897, 43 },
    { 0x71acc898, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2cc89c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cc89f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc8a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279cc8a8, 95 },
    { 0x71cc8a8, 98 },
    { 0x552cc8aa, 46 },
    { 0, 0 },
    { 0x92ecc8ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6cc8b0, 76 },
    { 0x3e2cc8b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bcc8b5, 85 },
    { 0xb10cc8b6, 76 },
    { 0xf9ecc8b7, 97 },
    { 0, 0 },
    { 0x8c8cc8b9, 23 },
    { 0x9dccc8ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cc8bf, 91 },
    { 0xbd5cc8c0, 69 },
    { 0x35cc8c1, 43 },
    { 0, 0 },
    { 0x2e3cc8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eccc8cb, 99 },
    { 0x7fecc8cc, 67 },
    { 0xc81cc8cd, 98 },
    { 0, 0 },
    { 0x4e3cc8cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210cc8d5, 23 },
    { 0x8e6cc8d6, 99 },
    { 0xe7ccc8d5, 66 },
    { 0x3d5cc8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcc8dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64cc8e1, 98 },
    { 0x21bcc8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dcc8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bcc8ec, 98 },
    { 0xffbcc8ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7cc8f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edcc8fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bcc905, 93 },
    { 0x54cc906, 76 },
    { 0xeedcc907, 90 },
    { 0xca3cc908, 33 },
    { 0xa21cc907, 55 },
    { 0, 0 },
    { 0x531cc90b, 99 },
    { 0, 0 },
    { 0x36ecc90d, 95 },
    { 0xcd3cc90e, 55 },
    { 0x9a3cc90f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dcc914, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17cc917, 95 },
    { 0, 0 },
    { 0x35acc919, 76 },
    { 0, 0 },
    { 0xb21cc91b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2cc91f, 23 },
    { 0xe2cc920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3cc923, 96 },
    { 0xdc2cc924, 85 },
    { 0xbcc924, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc928, 61 },
    { 0xee1cc929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dcc92e, 23 },
    { 0xc1cc92f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cc935, 55 },
    { 0x436cc936, 94 },
    { 0xe6ecc937, 34 },
    { 0x6f7cc937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cc93b, 33 },
    { 0x7c5cc93b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cc93f, 87 },
    { 0x9a8cc940, 83 },
    { 0, 0 },
    { 0x4facc942, 55 },
    { 0xdadcc943, 33 },
    { 0, 0 },
    { 0x3bbcc945, 95 },
    { 0x928cc946, 90 },
    { 0x304cc947, 64 },
    { 0x646cc946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56acc94e, 78 },
    { 0, 0 },
    { 0xde3cc950, 95 },
    { 0x208cc951, 2 },
    { 0x48ecc951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cc956, 55 },
    { 0x611cc956, 61 },
    { 0, 0 },
    { 0x21bcc959, 23 },
    { 0, 0 },
    { 0xa4ecc95b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657cc95f, 34 },
    { 0x6cdcc960, 43 },
    { 0, 0 },
    { 0x480cc962, 47 },
    { 0x7b7cc963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3accc968, 34 },
    { 0, 0 },
    { 0x9b0cc96a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cc970, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccc973, 90 },
    { 0x338cc974, 97 },
    { 0x98fcc974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dcc979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579cc97d, 90 },
    { 0xebfcc97e, 68 },
    { 0x693cc97e, 33 },
    { 0xf5acc97d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc989, 98 },
    { 0x752cc98a, 61 },
    { 0, 0 },
    { 0xdffcc98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bcc994, 61 },
    { 0x1c0cc995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc999, 55 },
    { 0, 0 },
    { 0x345cc99b, 95 },
    { 0, 0 },
    { 0x89dcc99d, 23 },
    { 0x22fcc99e, 23 },
    { 0, 0 },
    { 0xec0cc9a0, 100 },
    { 0x454cc9a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cc9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880cc9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cc9b4, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cc9be, 96 },
    { 0xe72cc9be, 33 },
    { 0, 0 },
    { 0xc9acc9c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55cc9cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aecc9d1, 34 },
    { 0, 0 },
    { 0xf46cc9d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667cc9de, 93 },
    { 0, 0 },
    { 0x432cc9e0, 23 },
    { 0xb44cc9e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6cc9e4, 95 },
    { 0, 0 },
    { 0x4f3cc9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x272cc9e9, 76 },
    { 0, 0 },
    { 0xac0cc9eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4acc9f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40acc9f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403cca03, 48 },
    { 0xe22cca03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27cca0a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ccca0d, 98 },
    { 0x296cca0e, 43 },
    { 0x178cca0f, 61 },
    { 0x9a4cca0f, 91 },
    { 0, 0 },
    { 0xf8cca12, 95 },
    { 0x11cca12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930cca19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328cca20, 55 },
    { 0xc3ecca21, 76 },
    { 0, 0 },
    { 0x12acca23, 52 },
    { 0xd6ecca23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabacca27, 95 },
    { 0x8a9cca28, 95 },
    { 0x1f0cca27, 23 },
    { 0x708cca2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x557cca2d, 67 },
    { 0x286cca2e, 90 },
    { 0x5e9cca2d, 61 },
    { 0xe8fcca2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cca33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacacca36, 49 },
    { 0x8dacca37, 43 },
    { 0xd53cca38, 76 },
    { 0xf06cca36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874cca3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628cca43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffcca48, 76 },
    { 0x5eacca48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cca4d, 90 },
    { 0x14acca4e, 61 },
    { 0, 0 },
    { 0xaaacca50, 98 },
    { 0, 0 },
    { 0xd72cca52, 95 },
    { 0x583cca53, 81 },
    { 0x9a1cca52, 33 },
    { 0x113cca55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4cca58, 88 },
    { 0x280cca59, 33 },
    { 0xd94cca58, 98 },
    { 0x378cca5b, 49 },
    { 0x755cca5b, 90 },
    { 0, 0 },
    { 0xcd4cca5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acca64, 77 },
    { 0x44acca65, 84 },
    { 0x3c1cca65, 33 },
    { 0x86fcca64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64cca6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x100cca6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dcca71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2cca77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3facca7a, 23 },
    { 0, 0 },
    { 0x6a0cca7c, 91 },
    { 0x49cca7d, 90 },
    { 0xd37cca7e, 69 },
    { 0x3c9cca7f, 98 },
    { 0x74dcca7f, 94 },
    { 0xc86cca81, 34 },
    { 0x53fcca82, 90 },
    { 0x1acca83, 78 },
    { 0x1b7cca84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x703cca87, 95 },
    { 0, 0 },
    { 0xd25cca89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdcca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfcca93, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cca99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccaa1, 43 },
    { 0x76ccaa2, 95 },
    { 0xb9accaa1, 84 },
    { 0x4deccaa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ccaab, 90 },
    { 0x9caccaac, 99 },
    { 0x244ccaab, 55 },
    { 0xeb7ccaae, 85 },
    { 0xceaccaaf, 91 },
    { 0x7faccaad, 23 },
    { 0x9b3ccab1, 34 },
    { 0xb3ccab2, 90 },
    { 0xbaaccab2, 61 },
    { 0x3c6ccab3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ccabc, 95 },
    { 0, 0 },
    { 0xb91ccabe, 93 },
    { 0x15eccabf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbccac3, 23 },
    { 0xabdccac4, 33 },
    { 0, 0 },
    { 0x601ccac6, 33 },
    { 0x71cccac6, 77 },
    { 0x59cccac8, 17 },
    { 0, 0 },
    { 0x13cccaca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ccace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eeccad4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4ccade, 98 },
    { 0, 0 },
    { 0xbb9ccae0, 98 },
    { 0, 0 },
    { 0x778ccae2, 93 },
    { 0xae1ccae3, 76 },
    { 0, 0 },
    { 0xe0dccae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6ccae8, 65 },
    { 0x103ccae9, 23 },
    { 0x6daccae9, 61 },
    { 0x5ebccaeb, 90 },
    { 0x766ccae9, 76 },
    { 0x64eccaeb, 61 },
    { 0x339ccaee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3ccaf1, 87 },
    { 0x576ccaf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276ccaf7, 95 },
    { 0xdedccaf7, 90 },
    { 0x509ccaf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dccafc, 61 },
    { 0x512ccafd, 34 },
    { 0, 0 },
    { 0x16cccaff, 33 },
    { 0, 0 },
    { 0xfdbccb01, 55 },
    { 0, 0 },
    { 0x1deccb03, 90 },
    { 0, 0 },
    { 0x19cccb05, 43 },
    { 0xe9ccb06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93ccb0a, 93 },
    { 0, 0 },
    { 0xe38ccb0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ccb12, 90 },
    { 0, 0 },
    { 0xf40ccb14, 33 },
    { 0xef3ccb15, 77 },
    { 0x572ccb15, 98 },
    { 0xab5ccb15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedccb1d, 33 },
    { 0x2abccb1e, 52 },
    { 0x6eeccb1f, 23 },
    { 0x354ccb20, 23 },
    { 0x69dccb21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dccb27, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ccb2d, 90 },
    { 0xdbcccb2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0ccb32, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8ccb37, 90 },
    { 0xd32ccb38, 33 },
    { 0xa3bccb39, 90 },
    { 0x80dccb3a, 34 },
    { 0xfcbccb39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357ccb42, 93 },
    { 0xad6ccb43, 90 },
    { 0x178ccb43, 61 },
    { 0, 0 },
    { 0x2c6ccb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439ccb4a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18accb54, 23 },
    { 0x512ccb55, 90 },
    { 0xfadccb56, 87 },
    { 0x27bccb55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cccb60, 33 },
    { 0, 0 },
    { 0x9bbccb62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefccb66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x746ccb69, 34 },
    { 0, 0 },
    { 0x3d6ccb6b, 43 },
    { 0x6daccb6c, 81 },
    { 0xf65ccb6d, 98 },
    { 0x715ccb6d, 98 },
    { 0, 0 },
    { 0xbfcccb70, 23 },
    { 0x8e6ccb70, 90 },
    { 0xfecccb72, 75 },
    { 0x28accb73, 61 },
    { 0x11ccb73, 98 },
    { 0xaafccb75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ccb79, 61 },
    { 0, 0 },
    { 0x764ccb7b, 53 },
    { 0, 0 },
    { 0x222ccb7d, 84 },
    { 0x17eccb7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63ccb83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccccb91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ccb96, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2accb9c, 90 },
    { 0xc25ccb9d, 100 },
    { 0x509ccb9c, 34 },
    { 0x6d6ccb9f, 33 },
    { 0x77bccb9f, 87 },
    { 0, 0 },
    { 0xca6ccba2, 34 },
    { 0xffdccba3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cccbab, 75 },
    { 0xe0cccbab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6ccbb0, 93 },
    { 0x546ccbb1, 34 },
    { 0x5e5ccbb1, 23 },
    { 0, 0 },
    { 0x40cccbb4, 61 },
    { 0xb19ccbb5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bccbb9, 93 },
    { 0x875ccbb9, 34 },
    { 0x7d9ccbb9, 95 },
    { 0x1d8ccbb9, 78 },
    { 0x9aeccbbd, 34 },
    { 0x2f5ccbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ccbc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eccbc7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ccbcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x801ccbd0, 61 },
    { 0, 0 },
    { 0x8f2ccbd2, 90 },
    { 0x28accbd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bccbd8, 78 },
    { 0, 0 },
    { 0x666ccbda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51accbde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674ccbe2, 98 },
    { 0x42dccbe3, 33 },
    { 0, 0 },
    { 0x7dfccbe5, 98 },
    { 0xd88ccbe5, 88 },
    { 0xbb8ccbe6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55ccbec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x179ccbef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0ccbf2, 87 },
    { 0xf7ccbf3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ccbf7, 90 },
    { 0xb54ccbf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfccbfb, 93 },
    { 0xec9ccbfc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ccc01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ccc04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caccc0d, 38 },
    { 0xe1dccc0e, 78 },
    { 0, 0 },
    { 0x41eccc10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9accc14, 66 },
    { 0x9a2ccc15, 98 },
    { 0, 0 },
    { 0x7e8ccc17, 95 },
    { 0xfb0ccc17, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0accc1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ccc1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8ccc22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ccc29, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaccc2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x595ccc30, 55 },
    { 0x995ccc30, 98 },
    { 0x3ccc31, 95 },
    { 0, 0 },
    { 0x16ccc34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efccc38, 91 },
    { 0x8adccc39, 23 },
    { 0x32bccc3a, 90 },
    { 0xee0ccc3a, 34 },
    { 0xd67ccc3b, 23 },
    { 0x8f5ccc3d, 90 },
    { 0x6d4ccc3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ccc44, 61 },
    { 0xff1ccc45, 43 },
    { 0x1d7ccc46, 46 },
    { 0, 0 },
    { 0x1cfccc48, 95 },
    { 0x55cccc49, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccc55, 90 },
    { 0xd2bccc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bccc59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2ccc5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadccc62, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ccc66, 90 },
    { 0, 0 },
    { 0x229ccc68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ccc6c, 33 },
    { 0, 0 },
    { 0xb8accc6e, 90 },
    { 0x318ccc6f, 94 },
    { 0x2b5ccc6e, 64 },
    { 0xd74ccc71, 98 },
    { 0xf0fccc71, 33 },
    { 0x1e8ccc73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ccc76, 33 },
    { 0x29dccc76, 33 },
    { 0, 0 },
    { 0xc13ccc79, 95 },
    { 0x5e7ccc7a, 61 },
    { 0, 0 },
    { 0xc4cccc7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4ccc80, 90 },
    { 0xc08ccc81, 33 },
    { 0, 0 },
    { 0x8e5ccc83, 98 },
    { 0x911ccc84, 34 },
    { 0xd8dccc84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6ccc97, 98 },
    { 0x62eccc97, 61 },
    { 0x344ccc98, 98 },
    { 0xbe5ccc99, 90 },
    { 0, 0 },
    { 0xbccccc9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacccca1, 23 },
    { 0x741ccca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6ccca9, 55 },
    { 0xdafccca9, 46 },
    { 0x7c3cccab, 99 },
    { 0x9dbcccac, 95 },
    { 0x39dcccac, 55 },
    { 0x63cccae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1cccb2, 90 },
    { 0, 0 },
    { 0x63bcccb4, 61 },
    { 0, 0 },
    { 0x929cccb6, 55 },
    { 0xee2cccb7, 36 },
    { 0x245cccb8, 61 },
    { 0x82ecccb8, 95 },
    { 0x807cccb9, 99 },
    { 0xf70cccbb, 61 },
    { 0x8aecccb6, 98 },
    { 0xa52cccbd, 90 },
    { 0x13cccbe, 33 },
    { 0x732cccbf, 98 },
    { 0x124cccc0, 90 },
    { 0xd81cccb6, 95 },
    { 0x4cecccc2, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecccc8, 48 },
    { 0, 0 },
    { 0x534cccca, 85 },
    { 0, 0 },
    { 0x51cccccc, 34 },
    { 0, 0 },
    { 0xca0cccce, 81 },
    { 0xc8acccce, 34 },
    { 0, 0 },
    { 0xac7cccd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcccd4, 84 },
    { 0xae2cccd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bcccdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9ccce0, 98 },
    { 0x472ccce1, 91 },
    { 0xb08ccce2, 90 },
    { 0xe0eccce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4ccce7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcccea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ccced, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcccf0, 61 },
    { 0x713cccf1, 90 },
    { 0x69fcccf1, 61 },
    { 0, 0 },
    { 0x661cccf4, 23 },
    { 0x744cccf4, 43 },
    { 0, 0 },
    { 0x398cccf7, 67 },
    { 0, 0 },
    { 0x23acccf9, 95 },
    { 0xd01cccfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8cccfd, 23 },
    { 0, 0 },
    { 0x9bdcccff, 23 },
    { 0xf8accd00, 95 },
    { 0, 0 },
    { 0x189ccd02, 85 },
    { 0xd2dccd03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ccd07, 81 },
    { 0, 0 },
    { 0xe7accd09, 90 },
    { 0, 0 },
    { 0xae4ccd0b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeccd11, 61 },
    { 0xc07ccd12, 95 },
    { 0x838ccd11, 85 },
    { 0, 0 },
    { 0x9a0ccd15, 81 },
    { 0x76fccd16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40ccd1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906ccd21, 61 },
    { 0xcacccd21, 33 },
    { 0, 0 },
    { 0xd3dccd24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ccd27, 23 },
    { 0x683ccd28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ccd2b, 52 },
    { 0x874ccd2c, 52 },
    { 0x294ccd2c, 33 },
    { 0x9b7ccd2d, 88 },
    { 0xec4ccd2b, 90 },
    { 0xcecccd30, 98 },
    { 0, 0 },
    { 0x3ceccd32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ccd36, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0accd3a, 33 },
    { 0x41bccd3b, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fccd43, 33 },
    { 0x579ccd44, 43 },
    { 0xf7cccd44, 98 },
    { 0, 0 },
    { 0xf94ccd47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ccd4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ccd57, 90 },
    { 0x8f6ccd58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580ccd60, 90 },
    { 0xd14ccd60, 33 },
    { 0x1beccd61, 33 },
    { 0xdcfccd63, 78 },
    { 0xaaaccd64, 61 },
    { 0x124ccd65, 98 },
    { 0xe2accd66, 33 },
    { 0, 0 },
    { 0x42dccd68, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cccd6e, 23 },
    { 0xa42ccd6f, 76 },
    { 0xb66ccd6f, 90 },
    { 0x31ccd71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ccd75, 98 },
    { 0x969ccd76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84ccd7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ccd81, 98 },
    { 0x489ccd82, 33 },
    { 0xebbccd83, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ccd87, 90 },
    { 0x6a3ccd87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68ccd8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7ccd8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebccd91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ccd98, 87 },
    { 0xaf2ccd98, 92 },
    { 0x98accd9a, 100 },
    { 0, 0 },
    { 0xe51ccd9c, 77 },
    { 0x877ccd9c, 90 },
    { 0, 0 },
    { 0xabfccd9f, 65 },
    { 0x14dccd9f, 61 },
    { 0, 0 },
    { 0xda5ccda2, 61 },
    { 0xaadccda2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ccdab, 76 },
    { 0x5dccdab, 98 },
    { 0xe78ccdad, 33 },
    { 0x881ccdab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3ccdb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fccdb8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bccdbc, 90 },
    { 0xe0cccdbd, 23 },
    { 0xe9accdbd, 95 },
    { 0xfe7ccdbf, 98 },
    { 0, 0 },
    { 0x124ccdc1, 33 },
    { 0, 0 },
    { 0xf01ccdc3, 72 },
    { 0, 0 },
    { 0xeb6ccdc5, 98 },
    { 0, 0 },
    { 0x283ccdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3ccdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ccdd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dccdd3, 90 },
    { 0x7acccdd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bccddc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ccddf, 90 },
    { 0x6ceccde0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcccde5, 95 },
    { 0xb68ccde5, 61 },
    { 0x99fccde6, 33 },
    { 0xa0cccde8, 95 },
    { 0x9c4ccde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ccdee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ccdf2, 61 },
    { 0, 0 },
    { 0xbadccdf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ccdf9, 40 },
    { 0xf5ccdf9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37cce02, 76 },
    { 0, 0 },
    { 0x700cce04, 61 },
    { 0x3a5cce04, 95 },
    { 0x22cce04, 23 },
    { 0x398cce07, 87 },
    { 0xca6cce08, 90 },
    { 0x603cce09, 98 },
    { 0x673cce08, 33 },
    { 0xa2fcce0b, 67 },
    { 0, 0 },
    { 0x629cce0d, 90 },
    { 0x9cdcce0d, 95 },
    { 0xeb4cce0f, 55 },
    { 0x11cce10, 66 },
    { 0, 0 },
    { 0xd40cce12, 100 },
    { 0x291cce13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafccce16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602cce1b, 95 },
    { 0x671cce1b, 97 },
    { 0, 0 },
    { 0x433cce1e, 33 },
    { 0xa57cce1e, 33 },
    { 0xb0ecce1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dcce24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bcce2d, 91 },
    { 0, 0 },
    { 0xb97cce2f, 43 },
    { 0xa6dcce30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0cce37, 90 },
    { 0x69fcce37, 91 },
    { 0, 0 },
    { 0xb46cce3a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1cce40, 95 },
    { 0x7d2cce41, 23 },
    { 0xc99cce41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82cce47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2cce4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x556cce50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665cce5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffcce62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993cce66, 90 },
    { 0x898cce66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ecce72, 33 },
    { 0x9cccce73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fcce76, 23 },
    { 0xe53cce77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbcce7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bcce7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x516cce80, 99 },
    { 0x27bcce81, 61 },
    { 0xadecce80, 17 },
    { 0, 0 },
    { 0xefdcce84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbacce8a, 33 },
    { 0, 0 },
    { 0x7bcce8c, 95 },
    { 0x58ecce8d, 34 },
    { 0x886cce8e, 76 },
    { 0, 0 },
    { 0xe05cce90, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264cce9a, 90 },
    { 0x134cce9a, 61 },
    { 0, 0 },
    { 0xcf1cce9d, 55 },
    { 0xa6ccce9e, 90 },
    { 0xecacce9f, 61 },
    { 0x24accea0, 90 },
    { 0x16ecce9d, 33 },
    { 0x663ccea2, 90 },
    { 0xb2acce9d, 23 },
    { 0, 0 },
    { 0xe42ccea5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ccea8, 33 },
    { 0x1ddccea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49cceac, 34 },
    { 0, 0 },
    { 0xfe1cceae, 61 },
    { 0x8ffcceae, 78 },
    { 0, 0 },
    { 0x30bcceb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dcceb8, 90 },
    { 0x25bcceb9, 23 },
    { 0, 0 },
    { 0x2f3ccebb, 90 },
    { 0x8c3ccebc, 99 },
    { 0x230ccebc, 97 },
    { 0xe43ccebe, 60 },
    { 0x187ccebf, 90 },
    { 0, 0 },
    { 0xae9ccec1, 61 },
    { 0xb6eccec2, 33 },
    { 0x2daccec1, 43 },
    { 0, 0 },
    { 0x42fccec5, 69 },
    { 0x7b0ccec6, 95 },
    { 0x72eccec5, 61 },
    { 0xe09ccec6, 43 },
    { 0, 0 },
    { 0x175cceca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cccecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31cced2, 46 },
    { 0xc93cced3, 33 },
    { 0, 0 },
    { 0x70acced5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ecced9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x455ccedc, 23 },
    { 0, 0 },
    { 0x567ccede, 46 },
    { 0x65bccede, 23 },
    { 0x318ccede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ccee5, 76 },
    { 0xe8accee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994cceea, 98 },
    { 0xd39cceeb, 90 },
    { 0x5dfcceea, 33 },
    { 0x6d1cceed, 90 },
    { 0x68dcceed, 90 },
    { 0, 0 },
    { 0x227ccef0, 90 },
    { 0x91cccef0, 33 },
    { 0, 0 },
    { 0xaa7ccef3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dccefd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3ccf00, 90 },
    { 0x3dbccf01, 98 },
    { 0xeccf00, 99 },
    { 0xc0ccf03, 23 },
    { 0x4c0ccf02, 61 },
    { 0, 0 },
    { 0xb38ccf06, 33 },
    { 0x3b2ccf07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeccf0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dccf0e, 90 },
    { 0x5b0ccf0f, 77 },
    { 0, 0 },
    { 0x2ccf11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ccf1d, 65 },
    { 0, 0 },
    { 0x73accf1f, 67 },
    { 0xd81ccf1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eccf24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195ccf2a, 61 },
    { 0, 0 },
    { 0x7d3ccf2c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ccf32, 91 },
    { 0x6cbccf32, 33 },
    { 0, 0 },
    { 0x585ccf35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97ccf39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x220ccf3c, 99 },
    { 0, 0 },
    { 0x35fccf3e, 61 },
    { 0, 0 },
    { 0xa78ccf40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ccf47, 87 },
    { 0xa67ccf48, 23 },
    { 0, 0 },
    { 0xd84ccf4a, 33 },
    { 0x384ccf4a, 66 },
    { 0x11ccf4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ccf4f, 77 },
    { 0x906ccf50, 90 },
    { 0, 0 },
    { 0xa02ccf52, 66 },
    { 0xa95ccf53, 66 },
    { 0x5abccf54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dccf58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfccf5b, 67 },
    { 0x7b9ccf5c, 33 },
    { 0x2edccf5b, 95 },
    { 0, 0 },
    { 0xcbeccf5f, 84 },
    { 0xe53ccf60, 55 },
    { 0xe10ccf60, 85 },
    { 0x647ccf62, 95 },
    { 0x6c1ccf63, 95 },
    { 0, 0 },
    { 0x6c1ccf65, 70 },
    { 0xfdccf66, 85 },
    { 0x7aeccf66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ccf6b, 98 },
    { 0x868ccf6c, 23 },
    { 0x74cccf6d, 90 },
    { 0xcf3ccf6c, 94 },
    { 0, 0 },
    { 0xa79ccf70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583ccf74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ccf78, 34 },
    { 0xb1cccf79, 90 },
    { 0x6c0ccf7a, 76 },
    { 0, 0 },
    { 0xae4ccf7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ccf7f, 23 },
    { 0xa42ccf80, 76 },
    { 0x7dbccf81, 17 },
    { 0xf2fccf82, 33 },
    { 0x4ddccf83, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eccf8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542ccf91, 61 },
    { 0x8a1ccf91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bccf99, 90 },
    { 0x1d4ccf9a, 93 },
    { 0x832ccf9b, 61 },
    { 0xc0eccf99, 66 },
    { 0x917ccf9d, 88 },
    { 0xb55ccf9e, 67 },
    { 0x1deccf9b, 33 },
    { 0xd2accf99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ccfa3, 98 },
    { 0x3dccfa4, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ccfa7, 95 },
    { 0, 0 },
    { 0xe5ccfa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ccfac, 95 },
    { 0, 0 },
    { 0x794ccfae, 90 },
    { 0, 0 },
    { 0x698ccfb0, 98 },
    { 0x89cccfb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ccfb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cccfbe, 46 },
    { 0x671ccfbf, 28 },
    { 0xc22ccfbf, 33 },
    { 0xe48ccfc1, 61 },
    { 0xbf8ccfbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6ccfc6, 55 },
    { 0xda5ccfc7, 76 },
    { 0x145ccfc8, 76 },
    { 0xcabccfc9, 33 },
    { 0xb9dccfca, 90 },
    { 0xf1eccfca, 76 },
    { 0xd0eccfcc, 98 },
    { 0xd39ccfc7, 88 },
    { 0x386ccfc9, 98 },
    { 0x13eccfce, 91 },
    { 0x74fccfc7, 76 },
    { 0xdb7ccfc9, 96 },
    { 0x32ccfd2, 55 },
    { 0x1e3ccfd3, 90 },
    { 0x1f2ccfd4, 98 },
    { 0, 0 },
    { 0x5d4ccfd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ccfda, 46 },
    { 0xf57ccfda, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abccfdf, 90 },
    { 0xb1eccfe0, 61 },
    { 0xd9bccfe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bccfe4, 98 },
    { 0x11fccfe5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91ccfe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ccff0, 95 },
    { 0xa92ccff0, 33 },
    { 0, 0 },
    { 0xee8ccff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94ccffb, 75 },
    { 0x851ccffb, 84 },
    { 0xa5fccffd, 91 },
    { 0x78eccffe, 98 },
    { 0x1fbccffe, 90 },
    { 0x352ccfff, 90 },
    { 0xd8dccfff, 90 },
    { 0, 0 },
    { 0xc8acd003, 95 },
    { 0, 0 },
    { 0x57bcd005, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7becd009, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dcd011, 33 },
    { 0x43cd011, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cd017, 76 },
    { 0, 0 },
    { 0x182cd019, 99 },
    { 0x28bcd01a, 98 },
    { 0x58acd019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bcd01e, 98 },
    { 0, 0 },
    { 0x124cd020, 90 },
    { 0xb24cd021, 33 },
    { 0, 0 },
    { 0x477cd023, 90 },
    { 0x38bcd023, 23 },
    { 0xb3dcd023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd028, 33 },
    { 0xf8ecd029, 61 },
    { 0x822cd029, 33 },
    { 0, 0 },
    { 0xed0cd02c, 94 },
    { 0x1dacd02d, 90 },
    { 0xf10cd02d, 17 },
    { 0, 0 },
    { 0x14dcd030, 61 },
    { 0, 0 },
    { 0x194cd032, 98 },
    { 0x9f4cd032, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80cd037, 95 },
    { 0x99acd038, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ecd03b, 93 },
    { 0x3b5cd03c, 91 },
    { 0xaf9cd03c, 88 },
    { 0xfa2cd03d, 95 },
    { 0x7fbcd03f, 81 },
    { 0xb55cd03d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2accd044, 91 },
    { 0x8ccd045, 95 },
    { 0xae9cd046, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdcd049, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cd04d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1cd051, 95 },
    { 0, 0 },
    { 0xadbcd053, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664cd058, 95 },
    { 0xb61cd059, 80 },
    { 0x879cd059, 61 },
    { 0x8f0cd05b, 33 },
    { 0xdb7cd05c, 66 },
    { 0x382cd059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbcd061, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bcd065, 61 },
    { 0, 0 },
    { 0x8b1cd067, 76 },
    { 0xb41cd067, 90 },
    { 0xdd3cd068, 34 },
    { 0xf38cd068, 99 },
    { 0, 0 },
    { 0xd63cd06c, 95 },
    { 0xcecd06c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41cd071, 66 },
    { 0, 0 },
    { 0x5a4cd073, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fccd07b, 90 },
    { 0x12acd07b, 99 },
    { 0, 0 },
    { 0xf98cd07e, 90 },
    { 0, 0 },
    { 0x75ccd080, 55 },
    { 0x39ecd081, 23 },
    { 0x15fcd081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bcd086, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2cd089, 76 },
    { 0x655cd08a, 91 },
    { 0, 0 },
    { 0x15dcd08c, 80 },
    { 0, 0 },
    { 0x991cd08e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2cd09a, 61 },
    { 0xa72cd09b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51acd09e, 96 },
    { 0x559cd09f, 90 },
    { 0xaeacd0a0, 98 },
    { 0xff9cd0a1, 93 },
    { 0, 0 },
    { 0x71cd0a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196cd0ac, 23 },
    { 0xfc8cd0ac, 93 },
    { 0x761cd0ae, 90 },
    { 0xc4dcd0af, 93 },
    { 0xb1fcd0b0, 95 },
    { 0x385cd0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cd0b6, 76 },
    { 0, 0 },
    { 0xad9cd0b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970cd0bc, 99 },
    { 0, 0 },
    { 0xc45cd0be, 95 },
    { 0xa20cd0bf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2cd0c7, 98 },
    { 0x192cd0c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8cd0cc, 76 },
    { 0, 0 },
    { 0xa47cd0ce, 23 },
    { 0x7d0cd0cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cecd0d2, 40 },
    { 0, 0 },
    { 0xdd1cd0d4, 55 },
    { 0, 0 },
    { 0x157cd0d6, 98 },
    { 0, 0 },
    { 0x269cd0d8, 61 },
    { 0, 0 },
    { 0xf73cd0da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cd0e0, 95 },
    { 0xe1bcd0e0, 36 },
    { 0xbabcd0e2, 91 },
    { 0x5bdcd0e2, 61 },
    { 0x8f2cd0e0, 23 },
    { 0, 0 },
    { 0xb6ecd0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacd0ea, 33 },
    { 0x166cd0ea, 23 },
    { 0xd66cd0ec, 23 },
    { 0xa73cd0ea, 61 },
    { 0xbf9cd0ea, 100 },
    { 0, 0 },
    { 0x36cd0f0, 33 },
    { 0, 0 },
    { 0x6facd0f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8cd0f6, 90 },
    { 0xadecd0f6, 61 },
    { 0xf55cd0f8, 33 },
    { 0x66acd0f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9cd108, 95 },
    { 0, 0 },
    { 0xa75cd10a, 61 },
    { 0, 0 },
    { 0x576cd10c, 92 },
    { 0x2e7cd10c, 90 },
    { 0x7bcd10d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5cd114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9cd117, 61 },
    { 0, 0 },
    { 0x784cd119, 61 },
    { 0x1cccd11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0cd11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fcd120, 76 },
    { 0xf21cd121, 33 },
    { 0x2dbcd122, 46 },
    { 0, 0 },
    { 0x938cd124, 98 },
    { 0x7adcd125, 95 },
    { 0xa4acd126, 25 },
    { 0x958cd124, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ccd12a, 90 },
    { 0, 0 },
    { 0x1d0cd12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cd130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328cd134, 95 },
    { 0xb39cd135, 91 },
    { 0xd4bcd136, 95 },
    { 0xbccd135, 46 },
    { 0xf99cd137, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02cd13b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650cd140, 33 },
    { 0, 0 },
    { 0x90cd142, 61 },
    { 0x4cccd143, 23 },
    { 0xc14cd144, 33 },
    { 0x777cd145, 61 },
    { 0x10bcd146, 67 },
    { 0xd82cd147, 76 },
    { 0, 0 },
    { 0xc6fcd149, 75 },
    { 0xd62cd14a, 94 },
    { 0, 0 },
    { 0xd13cd14c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ccd14f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2cd152, 34 },
    { 0x9cacd153, 43 },
    { 0xb2bcd153, 83 },
    { 0x635cd154, 61 },
    { 0x86fcd156, 34 },
    { 0xba7cd157, 78 },
    { 0xd58cd158, 95 },
    { 0x89bcd156, 76 },
    { 0x54ecd159, 100 },
    { 0x72dcd152, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecd161, 75 },
    { 0xf99cd162, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcd169, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cd16d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd170, 95 },
    { 0x80ccd171, 64 },
    { 0xc6fcd172, 90 },
    { 0x3d4cd171, 33 },
    { 0x634cd174, 34 },
    { 0, 0 },
    { 0x9f1cd176, 76 },
    { 0xb99cd176, 98 },
    { 0x8dacd178, 33 },
    { 0x244cd179, 33 },
    { 0, 0 },
    { 0x26bcd17b, 46 },
    { 0x8d1cd17c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4cd181, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edcd184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dcd187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5cd18a, 46 },
    { 0x206cd18b, 61 },
    { 0xf73cd18a, 33 },
    { 0xfefcd18b, 43 },
    { 0x461cd18d, 95 },
    { 0x533cd18f, 61 },
    { 0xef4cd190, 23 },
    { 0, 0 },
    { 0x56ccd192, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1cd197, 90 },
    { 0, 0 },
    { 0x4ffcd199, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fcd1a4, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5cd1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4cd1b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746cd1b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fcd1bd, 33 },
    { 0x65bcd1be, 34 },
    { 0xcbccd1bf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9cd1c2, 23 },
    { 0xccfcd1c3, 85 },
    { 0x8efcd1c4, 80 },
    { 0x110cd1c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808cd1cc, 95 },
    { 0xd1acd1cd, 90 },
    { 0x2b8cd1ce, 93 },
    { 0xffdcd1ce, 23 },
    { 0, 0 },
    { 0x830cd1d1, 61 },
    { 0xa6ecd1d2, 51 },
    { 0xba9cd1d2, 33 },
    { 0xd2bcd1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25cd1d9, 61 },
    { 0x4afcd1da, 84 },
    { 0xf5ecd1db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7cd1df, 78 },
    { 0xadcd1e0, 91 },
    { 0x1a6cd1df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05cd1e7, 90 },
    { 0x6ccd1e8, 99 },
    { 0xa74cd1e7, 67 },
    { 0, 0 },
    { 0x4b9cd1eb, 33 },
    { 0, 0 },
    { 0xbcecd1ed, 33 },
    { 0, 0 },
    { 0xb54cd1ef, 61 },
    { 0xdffcd1f0, 33 },
    { 0xbd3cd1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bcd1f4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48cd1ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94cd202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ecd205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bccd208, 76 },
    { 0x88ecd209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78acd213, 90 },
    { 0xea7cd213, 98 },
    { 0x64fcd215, 23 },
    { 0x738cd216, 43 },
    { 0xf3fcd217, 55 },
    { 0, 0 },
    { 0x98fcd219, 80 },
    { 0, 0 },
    { 0xbffcd21b, 98 },
    { 0x543cd21c, 77 },
    { 0x1d0cd21b, 90 },
    { 0, 0 },
    { 0xb15cd21f, 55 },
    { 0x5b8cd220, 34 },
    { 0xccacd220, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9cd226, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27cd22a, 84 },
    { 0, 0 },
    { 0x4a3cd22c, 61 },
    { 0x80cd22c, 94 },
    { 0, 0 },
    { 0xe8bcd22f, 23 },
    { 0, 0 },
    { 0xb3cd231, 34 },
    { 0x68bcd231, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9cd236, 61 },
    { 0x7b2cd237, 33 },
    { 0x8cfcd238, 90 },
    { 0x789cd238, 98 },
    { 0x2aacd239, 95 },
    { 0x615cd23b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bcd23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366cd248, 98 },
    { 0, 0 },
    { 0x27ecd24a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fcd24d, 61 },
    { 0, 0 },
    { 0x217cd24f, 33 },
    { 0xbccd24f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95cd253, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0cd256, 33 },
    { 0xe25cd257, 87 },
    { 0x6a1cd257, 93 },
    { 0x781cd259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81acd262, 77 },
    { 0xf0acd262, 43 },
    { 0, 0 },
    { 0x43dcd265, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bcd26b, 61 },
    { 0xf87cd26c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142cd270, 46 },
    { 0xf3dcd271, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938cd275, 100 },
    { 0x800cd276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5acd279, 93 },
    { 0, 0 },
    { 0x176cd27b, 66 },
    { 0x3facd27b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72acd283, 33 },
    { 0xa94cd283, 99 },
    { 0, 0 },
    { 0xc64cd286, 33 },
    { 0x98dcd287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7cd28b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x517cd28e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48cd295, 76 },
    { 0, 0 },
    { 0x2eccd297, 98 },
    { 0xb40cd297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfcd2a0, 33 },
    { 0xd5cd2a1, 34 },
    { 0x440cd2a2, 17 },
    { 0xbdcd2a2, 61 },
    { 0xb3acd2a2, 34 },
    { 0x129cd2a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2cd2aa, 98 },
    { 0xff8cd2ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bcd2b0, 93 },
    { 0x711cd2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1cd2b6, 95 },
    { 0xd47cd2b7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcd2c0, 90 },
    { 0x9b6cd2c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19cd2c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633cd2ca, 95 },
    { 0, 0 },
    { 0x938cd2cc, 66 },
    { 0, 0 },
    { 0x8a8cd2ce, 98 },
    { 0, 0 },
    { 0x5ecd2d0, 51 },
    { 0x37bcd2d1, 33 },
    { 0, 0 },
    { 0x507cd2d3, 95 },
    { 0xe98cd2d3, 61 },
    { 0xf88cd2d5, 69 },
    { 0x157cd2d6, 90 },
    { 0xb7cd2d4, 55 },
    { 0x37ecd2d8, 52 },
    { 0xc84cd2d7, 23 },
    { 0xa10cd2d3, 61 },
    { 0x6e0cd2db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8cd2e0, 33 },
    { 0x631cd2e1, 17 },
    { 0, 0 },
    { 0xf1fcd2e3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830cd2e8, 95 },
    { 0x826cd2e9, 36 },
    { 0x30ecd2ea, 68 },
    { 0x875cd2ea, 55 },
    { 0x873cd2ec, 99 },
    { 0x604cd2ea, 55 },
    { 0x770cd2ee, 33 },
    { 0, 0 },
    { 0x5bbcd2f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1acd2f6, 90 },
    { 0, 0 },
    { 0x3bfcd2f8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99cd2fb, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cd2ff, 8 },
    { 0x952cd300, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fccd303, 33 },
    { 0x1efcd303, 61 },
    { 0, 0 },
    { 0xccdcd306, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcd309, 75 },
    { 0x952cd30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffcd30f, 23 },
    { 0x63acd310, 90 },
    { 0xe8fcd311, 34 },
    { 0x4cecd311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595cd319, 76 },
    { 0xe6bcd31a, 95 },
    { 0x287cd31a, 23 },
    { 0x3decd31c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5cd320, 84 },
    { 0x464cd321, 33 },
    { 0, 0 },
    { 0xca2cd323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dcd328, 61 },
    { 0x59ecd328, 33 },
    { 0xe90cd32a, 76 },
    { 0xc41cd32a, 76 },
    { 0x3e5cd32b, 56 },
    { 0xa5cd32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eecd333, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6cd336, 90 },
    { 0, 0 },
    { 0xa38cd338, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7decd33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96cd340, 95 },
    { 0xb12cd341, 95 },
    { 0x97cd342, 67 },
    { 0, 0 },
    { 0x6f7cd344, 61 },
    { 0x35dcd345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cd34b, 65 },
    { 0xc88cd34c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cd351, 61 },
    { 0x2e9cd352, 99 },
    { 0x203cd352, 61 },
    { 0xbe8cd353, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd357, 90 },
    { 0, 0 },
    { 0xd92cd359, 98 },
    { 0, 0 },
    { 0x68ecd35b, 90 },
    { 0, 0 },
    { 0x7f5cd35d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dcd360, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xceacd363, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcd366, 61 },
    { 0x9f1cd367, 61 },
    { 0x650cd368, 85 },
    { 0x443cd368, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315cd36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7cd374, 46 },
    { 0x80ccd375, 78 },
    { 0, 0 },
    { 0x9c9cd377, 66 },
    { 0xf32cd378, 95 },
    { 0xd01cd377, 23 },
    { 0x554cd37a, 33 },
    { 0x816cd37b, 33 },
    { 0, 0 },
    { 0x7d2cd37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532cd381, 52 },
    { 0x7d9cd382, 90 },
    { 0x319cd382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bcd38c, 93 },
    { 0, 0 },
    { 0xc5ccd38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cd391, 61 },
    { 0xe6bcd391, 95 },
    { 0x6e7cd393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41cd397, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617cd39d, 98 },
    { 0, 0 },
    { 0xdc0cd39f, 95 },
    { 0x1a6cd39f, 93 },
    { 0x3e7cd39f, 23 },
    { 0, 0 },
    { 0x9cbcd3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cd3a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bcd3af, 78 },
    { 0, 0 },
    { 0xb68cd3b1, 90 },
    { 0xc7cd3b2, 64 },
    { 0x2a6cd3b1, 52 },
    { 0xecdcd3b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3cd3bd, 34 },
    { 0xeddcd3be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18cd3c5, 33 },
    { 0xd31cd3c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaecd3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ccd3cd, 90 },
    { 0x8c7cd3cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793cd3d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599cd3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10cd3e4, 95 },
    { 0x1d0cd3e5, 23 },
    { 0x619cd3e6, 34 },
    { 0x758cd3e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3acd3ee, 77 },
    { 0x80ecd3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbcd3f7, 98 },
    { 0, 0 },
    { 0xc96cd3f9, 61 },
    { 0xb24cd3fa, 23 },
    { 0x4edcd3fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebcd3fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacd407, 43 },
    { 0xadbcd408, 23 },
    { 0x613cd409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73cd40c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bcd410, 55 },
    { 0x2edcd411, 84 },
    { 0x70dcd411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65acd41d, 87 },
    { 0x95cd41d, 90 },
    { 0x8bfcd41f, 34 },
    { 0xa59cd420, 76 },
    { 0x87ccd421, 98 },
    { 0, 0 },
    { 0xab1cd423, 38 },
    { 0, 0 },
    { 0xd95cd425, 98 },
    { 0x4d1cd426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82cd42d, 61 },
    { 0x157cd42d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x135cd431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ccd434, 33 },
    { 0x291cd435, 90 },
    { 0xadfcd436, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234cd43f, 34 },
    { 0xc9ccd440, 76 },
    { 0, 0 },
    { 0x157cd442, 98 },
    { 0x910cd443, 61 },
    { 0, 0 },
    { 0xf2bcd445, 43 },
    { 0x3c1cd445, 61 },
    { 0xececd447, 34 },
    { 0x8cfcd447, 33 },
    { 0, 0 },
    { 0xc2ccd44a, 55 },
    { 0x48cd44b, 100 },
    { 0, 0 },
    { 0x2fcd44d, 88 },
    { 0x12dcd44e, 61 },
    { 0xf5ccd44f, 97 },
    { 0xb9cd450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaecd453, 33 },
    { 0xab8cd454, 98 },
    { 0x230cd455, 25 },
    { 0xdafcd456, 52 },
    { 0xaf3cd455, 23 },
    { 0x29dcd456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3cd45e, 98 },
    { 0x9ddcd45f, 76 },
    { 0, 0 },
    { 0x9e5cd461, 90 },
    { 0, 0 },
    { 0xf88cd463, 76 },
    { 0x52bcd463, 98 },
    { 0x451cd465, 32 },
    { 0, 0 },
    { 0x732cd467, 34 },
    { 0xff9cd468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05cd472, 33 },
    { 0, 0 },
    { 0xa29cd474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x156cd477, 23 },
    { 0, 0 },
    { 0xd2acd479, 61 },
    { 0x74dcd47a, 75 },
    { 0xb3ecd47b, 33 },
    { 0, 0 },
    { 0x4b8cd47d, 23 },
    { 0xb41cd47e, 95 },
    { 0, 0 },
    { 0xc8ccd480, 46 },
    { 0xe9cd480, 34 },
    { 0xb90cd482, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedcd486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0cd489, 35 },
    { 0xb5cd489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96cd492, 98 },
    { 0, 0 },
    { 0xbd2cd494, 95 },
    { 0x719cd495, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59cd498, 34 },
    { 0xf86cd499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63acd49f, 33 },
    { 0, 0 },
    { 0xc2ecd4a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ccd4a5, 98 },
    { 0x833cd4a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49acd4ac, 56 },
    { 0xec3cd4ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296cd4b2, 43 },
    { 0, 0 },
    { 0xdb2cd4b4, 43 },
    { 0x151cd4b4, 91 },
    { 0x480cd4b6, 87 },
    { 0, 0 },
    { 0x998cd4b8, 88 },
    { 0, 0 },
    { 0xca1cd4ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57cd4c1, 98 },
    { 0x193cd4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bcd4c9, 17 },
    { 0x544cd4ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cd4d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8cd4d4, 61 },
    { 0, 0 },
    { 0x889cd4d6, 95 },
    { 0x695cd4d7, 23 },
    { 0x81ecd4d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9cd4de, 95 },
    { 0, 0 },
    { 0x890cd4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342cd4e4, 40 },
    { 0x868cd4e5, 85 },
    { 0xde2cd4e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacacd4ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcd4f2, 55 },
    { 0xa5fcd4f3, 98 },
    { 0xb39cd4f4, 81 },
    { 0xcb0cd4f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25cd4f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fcd4fe, 90 },
    { 0x6a3cd4ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ccd502, 66 },
    { 0xfefcd503, 34 },
    { 0xd1dcd504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4cd507, 33 },
    { 0x587cd508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefcd50e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fcd513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0cd516, 90 },
    { 0x8a5cd517, 55 },
    { 0x9a2cd518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd51d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03cd52b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37cd52e, 100 },
    { 0, 0 },
    { 0x28ccd530, 90 },
    { 0xe5bcd530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70cd541, 75 },
    { 0x7d4cd542, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x533cd54b, 90 },
    { 0xf2fcd54b, 61 },
    { 0xa21cd54c, 78 },
    { 0xe40cd54b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x751cd551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58cd554, 84 },
    { 0x5d3cd555, 95 },
    { 0, 0 },
    { 0xae7cd557, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbccd55c, 61 },
    { 0x7edcd55c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cd561, 23 },
    { 0x407cd562, 90 },
    { 0x4a6cd563, 95 },
    { 0x174cd561, 95 },
    { 0xb56cd565, 98 },
    { 0xdfcd566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x614cd569, 95 },
    { 0x8a5cd56a, 90 },
    { 0, 0 },
    { 0xf66cd56c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceacd570, 85 },
    { 0x98dcd570, 87 },
    { 0x30dcd571, 46 },
    { 0, 0 },
    { 0x7c8cd574, 43 },
    { 0x851cd574, 34 },
    { 0xd66cd574, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cd582, 87 },
    { 0x6e1cd582, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x889cd586, 91 },
    { 0x424cd586, 43 },
    { 0xd71cd588, 75 },
    { 0, 0 },
    { 0x22ecd58a, 90 },
    { 0xb43cd58b, 91 },
    { 0, 0 },
    { 0xe37cd58d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cd590, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bcd597, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82cd5a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbcd5ae, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5cd5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8cd5bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfcd5bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1cd5c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecd5cc, 77 },
    { 0, 0 },
    { 0xcb2cd5ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbcd5d3, 23 },
    { 0x3dcd5d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31acd5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ecd5dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58cd5e2, 81 },
    { 0x8a6cd5e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310cd5ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805cd5f4, 95 },
    { 0x553cd5f4, 81 },
    { 0, 0 },
    { 0xc94cd5f7, 34 },
    { 0xc89cd5f8, 99 },
    { 0xf54cd5f9, 17 },
    { 0x90cd5f9, 90 },
    { 0, 0 },
    { 0x4eacd5fc, 55 },
    { 0, 0 },
    { 0xd85cd5fe, 17 },
    { 0, 0 },
    { 0x36ecd600, 86 },
    { 0, 0 },
    { 0x7d8cd602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492cd606, 98 },
    { 0x9b3cd607, 55 },
    { 0x990cd608, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfcd60c, 52 },
    { 0x928cd60d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dccd610, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xececd615, 46 },
    { 0xc6acd616, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dcd619, 90 },
    { 0x118cd619, 95 },
    { 0xefdcd61b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd620, 95 },
    { 0x1aecd621, 91 },
    { 0, 0 },
    { 0xc74cd623, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcd628, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cd62d, 23 },
    { 0xc89cd62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cd633, 46 },
    { 0x142cd633, 23 },
    { 0x8f2cd634, 61 },
    { 0xf98cd636, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd639, 61 },
    { 0xb55cd63a, 61 },
    { 0x8f0cd63b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x751cd63e, 61 },
    { 0, 0 },
    { 0x203cd640, 23 },
    { 0x3dcd640, 100 },
    { 0xd9fcd641, 93 },
    { 0x4ebcd643, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600cd647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307cd64f, 43 },
    { 0x216cd64f, 52 },
    { 0x271cd650, 61 },
    { 0xc7bcd64f, 38 },
    { 0x3f6cd653, 90 },
    { 0x8b8cd654, 61 },
    { 0x650cd654, 95 },
    { 0x661cd656, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58cd65e, 23 },
    { 0x48fcd65f, 66 },
    { 0, 0 },
    { 0x6f4cd661, 46 },
    { 0x430cd661, 66 },
    { 0x127cd662, 61 },
    { 0x424cd663, 99 },
    { 0, 0 },
    { 0xe9fcd666, 61 },
    { 0x212cd667, 55 },
    { 0x2e6cd668, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcd66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccd670, 52 },
    { 0xb40cd671, 99 },
    { 0x12acd670, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd67e, 98 },
    { 0xa15cd67f, 65 },
    { 0, 0 },
    { 0xc70cd681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413cd687, 33 },
    { 0x637cd688, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75cd68b, 55 },
    { 0, 0 },
    { 0x376cd68d, 61 },
    { 0x20bcd68e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4cd692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aacd696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28acd699, 66 },
    { 0x8afcd699, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6cd69f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474cd6a4, 90 },
    { 0, 0 },
    { 0x5cbcd6a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffecd6ac, 61 },
    { 0x9a5cd6ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cd6b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624cd6b5, 61 },
    { 0xb6fcd6b6, 87 },
    { 0x16ecd6b6, 76 },
    { 0, 0 },
    { 0xfe6cd6b9, 90 },
    { 0xee9cd6ba, 99 },
    { 0xe60cd6bb, 34 },
    { 0xc87cd6bc, 99 },
    { 0x5c5cd6bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbcd6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9cd6c7, 95 },
    { 0x2cacd6c8, 23 },
    { 0x947cd6c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838cd6cf, 76 },
    { 0x32ecd6d0, 95 },
    { 0, 0 },
    { 0xa8acd6d2, 61 },
    { 0x12dcd6d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7cd6db, 23 },
    { 0x69dcd6db, 98 },
    { 0x18ecd6db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152cd6e9, 90 },
    { 0x526cd6e9, 90 },
    { 0x625cd6ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x593cd6ee, 95 },
    { 0x499cd6ef, 61 },
    { 0x975cd6ef, 85 },
    { 0, 0 },
    { 0x947cd6f2, 61 },
    { 0xe4acd6f3, 98 },
    { 0xddcd6f3, 98 },
    { 0x467cd6f5, 46 },
    { 0x140cd6f4, 46 },
    { 0xaf1cd6f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3cd6fb, 90 },
    { 0xb44cd6fc, 33 },
    { 0xf51cd6fd, 46 },
    { 0x7eacd6fd, 95 },
    { 0xcf3cd6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cd709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20cd70d, 95 },
    { 0x844cd70e, 76 },
    { 0xb4acd70f, 98 },
    { 0xa6ecd710, 90 },
    { 0xa35cd70e, 65 },
    { 0, 0 },
    { 0x1b1cd713, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5cd718, 93 },
    { 0x42ecd718, 33 },
    { 0, 0 },
    { 0xbfcd71b, 61 },
    { 0xb12cd71b, 23 },
    { 0x9fccd71c, 34 },
    { 0xb0fcd71d, 98 },
    { 0xa06cd71b, 75 },
    { 0xbbfcd720, 61 },
    { 0xb77cd720, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bcd731, 52 },
    { 0, 0 },
    { 0xba7cd733, 93 },
    { 0x9becd734, 95 },
    { 0x67ecd733, 100 },
    { 0x47acd736, 22 },
    { 0x56bcd736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ecd73d, 66 },
    { 0x472cd73d, 55 },
    { 0xad2cd73d, 98 },
    { 0, 0 },
    { 0x7f9cd741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5cd744, 67 },
    { 0xc6bcd745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8cd749, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3cd74f, 91 },
    { 0, 0 },
    { 0x62ecd751, 93 },
    { 0xae5cd752, 76 },
    { 0xec3cd752, 90 },
    { 0x492cd754, 33 },
    { 0x30cd755, 46 },
    { 0, 0 },
    { 0x2aacd757, 98 },
    { 0, 0 },
    { 0xf5cd759, 23 },
    { 0, 0 },
    { 0xb03cd75b, 48 },
    { 0xc65cd75c, 66 },
    { 0x92bcd75d, 33 },
    { 0, 0 },
    { 0x129cd75f, 33 },
    { 0xa7dcd760, 33 },
    { 0x7cdcd75f, 95 },
    { 0xedecd762, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2cd767, 55 },
    { 0, 0 },
    { 0x6bbcd769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847cd772, 100 },
    { 0x7a0cd772, 55 },
    { 0xa94cd774, 61 },
    { 0xa02cd775, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcd780, 69 },
    { 0, 0 },
    { 0x2f8cd782, 48 },
    { 0x241cd783, 80 },
    { 0xc73cd783, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745cd788, 93 },
    { 0x97acd788, 61 },
    { 0x1dacd789, 33 },
    { 0xafcd78b, 33 },
    { 0x794cd78c, 61 },
    { 0x566cd788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fcd790, 66 },
    { 0x1adcd790, 90 },
    { 0x400cd792, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcd795, 85 },
    { 0xf43cd796, 90 },
    { 0x782cd797, 33 },
    { 0xe89cd795, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22cd7a3, 95 },
    { 0xffcd7a3, 61 },
    { 0x313cd7a5, 66 },
    { 0x829cd7a4, 90 },
    { 0xc24cd7a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x356cd7aa, 23 },
    { 0x9b1cd7aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cd7b0, 98 },
    { 0, 0 },
    { 0x4e4cd7b2, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cd7bd, 33 },
    { 0, 0 },
    { 0xecd7bf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cd7c5, 98 },
    { 0xaabcd7c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbcd7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60cd7cf, 66 },
    { 0xb22cd7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcd7d3, 55 },
    { 0xce7cd7d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffcd7db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8decd7e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8cd7e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x358cd7e9, 33 },
    { 0xce8cd7ea, 23 },
    { 0, 0 },
    { 0xa4bcd7ec, 55 },
    { 0xd53cd7ec, 43 },
    { 0xc0dcd7ec, 76 },
    { 0, 0 },
    { 0x4acd7f0, 61 },
    { 0x7fecd7f1, 90 },
    { 0, 0 },
    { 0xc82cd7f3, 76 },
    { 0x40fcd7f4, 33 },
    { 0x773cd7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bcd7f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccd801, 52 },
    { 0x1b7cd802, 61 },
    { 0, 0 },
    { 0xbcfcd804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x368cd807, 43 },
    { 0xddecd808, 64 },
    { 0xcd2cd808, 67 },
    { 0xef4cd80a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7cd80d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dcd810, 87 },
    { 0x1accd811, 34 },
    { 0x562cd811, 23 },
    { 0xe68cd813, 93 },
    { 0x566cd814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84cd81a, 33 },
    { 0x732cd81a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd81e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26cd821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33cd824, 99 },
    { 0, 0 },
    { 0xfc1cd826, 68 },
    { 0x3aecd827, 33 },
    { 0xa42cd828, 85 },
    { 0x83ccd827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd82d, 90 },
    { 0x88bcd82e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6acd831, 90 },
    { 0xfc7cd832, 46 },
    { 0xb00cd833, 33 },
    { 0x109cd833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfecd838, 90 },
    { 0x146cd838, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcd83c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x581cd83f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831cd844, 84 },
    { 0xbd7cd844, 33 },
    { 0, 0 },
    { 0x358cd847, 61 },
    { 0xdadcd848, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955cd84c, 90 },
    { 0, 0 },
    { 0x5eecd84e, 61 },
    { 0x674cd84e, 23 },
    { 0xcc5cd84e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447cd858, 23 },
    { 0xa72cd858, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd85c, 61 },
    { 0x78bcd85d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adcd863, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0cd86c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9cd870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8cd87a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47cd87d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfcd882, 90 },
    { 0xf42cd883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dcd886, 98 },
    { 0, 0 },
    { 0xf56cd888, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbcd88b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fcd88f, 43 },
    { 0, 0 },
    { 0xb4ecd891, 78 },
    { 0xf96cd892, 23 },
    { 0x586cd893, 33 },
    { 0, 0 },
    { 0x272cd895, 99 },
    { 0x5c6cd896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x145cd899, 98 },
    { 0x7ffcd899, 23 },
    { 0, 0 },
    { 0x31acd89c, 95 },
    { 0, 0 },
    { 0x62ccd89e, 38 },
    { 0x844cd89f, 98 },
    { 0, 0 },
    { 0x303cd8a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bcd8a5, 95 },
    { 0, 0 },
    { 0x9becd8a7, 23 },
    { 0x54dcd8a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14cd8ac, 95 },
    { 0, 0 },
    { 0xfb9cd8ae, 84 },
    { 0xdaacd8ae, 98 },
    { 0x590cd8ae, 87 },
    { 0xb39cd8b1, 81 },
    { 0, 0 },
    { 0x4cacd8b3, 55 },
    { 0xd20cd8b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949cd8c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x442cd8c6, 33 },
    { 0x8d3cd8c7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14cd8d1, 23 },
    { 0, 0 },
    { 0xe36cd8d3, 23 },
    { 0xee3cd8d4, 69 },
    { 0, 0 },
    { 0x879cd8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fcd8db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cd8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2cd8e2, 90 },
    { 0, 0 },
    { 0x5eecd8e4, 33 },
    { 0, 0 },
    { 0x7fbcd8e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaccd8ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2cd8f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bcd8f7, 23 },
    { 0x3e1cd8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495cd900, 61 },
    { 0, 0 },
    { 0xce6cd902, 99 },
    { 0x387cd903, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abcd907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ecd90a, 61 },
    { 0x5b3cd90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd90f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dcd912, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ccd915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75cd919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x274cd91c, 33 },
    { 0xc15cd91d, 90 },
    { 0x5facd91e, 90 },
    { 0, 0 },
    { 0xfe8cd920, 95 },
    { 0xc7fcd921, 76 },
    { 0xd02cd921, 95 },
    { 0x6abcd923, 78 },
    { 0xb39cd923, 34 },
    { 0x90bcd925, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883cd929, 38 },
    { 0xfa4cd929, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcd931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abcd938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38cd93f, 33 },
    { 0xbfbcd940, 98 },
    { 0xe2cd941, 34 },
    { 0, 0 },
    { 0x7ddcd943, 72 },
    { 0x3facd944, 61 },
    { 0xa4acd943, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cd948, 61 },
    { 0x500cd949, 95 },
    { 0, 0 },
    { 0x680cd94b, 78 },
    { 0xc6acd94b, 61 },
    { 0x34bcd94d, 100 },
    { 0xa7cd94e, 61 },
    { 0x6d6cd94e, 98 },
    { 0x28ecd950, 95 },
    { 0x987cd951, 61 },
    { 0xf47cd952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46cd957, 33 },
    { 0x505cd958, 23 },
    { 0xce2cd957, 43 },
    { 0x900cd95a, 23 },
    { 0, 0 },
    { 0x759cd95c, 23 },
    { 0, 0 },
    { 0xc20cd95e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcd963, 38 },
    { 0xe8cd964, 76 },
    { 0xddccd965, 77 },
    { 0, 0 },
    { 0xe8cd967, 98 },
    { 0x305cd968, 84 },
    { 0x8c5cd969, 98 },
    { 0xde5cd96a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6cd96f, 98 },
    { 0, 0 },
    { 0x7ffcd971, 55 },
    { 0x79bcd971, 61 },
    { 0, 0 },
    { 0x163cd974, 61 },
    { 0x4bccd975, 95 },
    { 0xe95cd975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71acd979, 90 },
    { 0, 0 },
    { 0xfe3cd97b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cd981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dcd984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75cd988, 43 },
    { 0, 0 },
    { 0x5f1cd98a, 93 },
    { 0x22dcd98b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaecd991, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41cd995, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x571cd998, 34 },
    { 0, 0 },
    { 0x791cd99a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48cd99d, 44 },
    { 0, 0 },
    { 0xe3ccd99f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x919cd9a2, 66 },
    { 0, 0 },
    { 0xe6ecd9a4, 90 },
    { 0, 0 },
    { 0x9c8cd9a6, 100 },
    { 0xdb6cd9a7, 87 },
    { 0, 0 },
    { 0x21acd9a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9cd9af, 84 },
    { 0, 0 },
    { 0x8fccd9b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecd9b4, 55 },
    { 0, 0 },
    { 0xf3dcd9b6, 87 },
    { 0x3bfcd9b7, 34 },
    { 0x85bcd9b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27cd9bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68acd9c4, 43 },
    { 0, 0 },
    { 0x888cd9c6, 90 },
    { 0xb37cd9c6, 33 },
    { 0x896cd9c7, 61 },
    { 0, 0 },
    { 0xebfcd9ca, 85 },
    { 0, 0 },
    { 0x52ccd9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564cd9d0, 61 },
    { 0, 0 },
    { 0xd49cd9d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cd9d5, 85 },
    { 0x40bcd9d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebcd9d9, 61 },
    { 0x1ebcd9da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd9de, 33 },
    { 0xa76cd9df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09cd9e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44cd9e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x768cd9ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cd9ed, 23 },
    { 0x15acd9ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3cd9f5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cd9fa, 93 },
    { 0xc81cd9fb, 74 },
    { 0, 0 },
    { 0xc7ccd9fd, 66 },
    { 0xa17cd9fd, 93 },
    { 0x7bccd9ff, 33 },
    { 0x2a8cd9fd, 61 },
    { 0, 0 },
    { 0xcabcda02, 61 },
    { 0x3a5cda03, 51 },
    { 0x809cda03, 98 },
    { 0xf5ecda05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfcda0a, 61 },
    { 0x309cda0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcda0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54cda11, 55 },
    { 0, 0 },
    { 0xdfbcda13, 61 },
    { 0xccda14, 61 },
    { 0, 0 },
    { 0xe7bcda16, 95 },
    { 0, 0 },
    { 0x9fecda18, 33 },
    { 0, 0 },
    { 0x9a7cda1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800cda1e, 77 },
    { 0xebbcda1f, 55 },
    { 0, 0 },
    { 0x227cda21, 46 },
    { 0, 0 },
    { 0x1e0cda23, 23 },
    { 0x609cda23, 61 },
    { 0x5bfcda23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dcda28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x176cda2b, 43 },
    { 0xd70cda2c, 61 },
    { 0x1d7cda2d, 23 },
    { 0, 0 },
    { 0xe56cda2f, 77 },
    { 0x9d1cda2f, 34 },
    { 0xfbbcda31, 33 },
    { 0xdf5cda31, 61 },
    { 0x7e6cda33, 61 },
    { 0x62ccda34, 90 },
    { 0x7d6cda34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5cda38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dcda3f, 76 },
    { 0, 0 },
    { 0xc5dcda41, 24 },
    { 0, 0 },
    { 0xe80cda43, 80 },
    { 0, 0 },
    { 0x3dbcda45, 95 },
    { 0xc2bcda46, 76 },
    { 0x54bcda46, 83 },
    { 0x534cda46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bcda4c, 61 },
    { 0xd35cda4d, 23 },
    { 0, 0 },
    { 0x8fdcda4f, 98 },
    { 0x897cda50, 36 },
    { 0, 0 },
    { 0xf22cda52, 33 },
    { 0xeb0cda53, 61 },
    { 0x70cda53, 90 },
    { 0xd63cda55, 61 },
    { 0, 0 },
    { 0x6c4cda57, 66 },
    { 0x2a2cda58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcda5c, 90 },
    { 0x1c3cda5c, 23 },
    { 0xd4ecda5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2cda61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bcda66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809cda6d, 52 },
    { 0, 0 },
    { 0x9cfcda6f, 90 },
    { 0x2cfcda70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08cda73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415cda7d, 90 },
    { 0xcecda7d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bcda82, 33 },
    { 0xdaccda82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2cda8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11cda8e, 98 },
    { 0, 0 },
    { 0xa4bcda90, 37 },
    { 0x8e3cda91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cda98, 23 },
    { 0xdd6cda99, 23 },
    { 0x711cda98, 93 },
    { 0x7cfcda98, 98 },
    { 0, 0 },
    { 0xadbcda9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2cdaa3, 98 },
    { 0xe15cdaa3, 76 },
    { 0, 0 },
    { 0xbe7cdaa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81cdaac, 61 },
    { 0, 0 },
    { 0xc51cdaae, 33 },
    { 0, 0 },
    { 0xecdab0, 98 },
    { 0x364cdab1, 90 },
    { 0x541cdab1, 61 },
    { 0xfc1cdab3, 55 },
    { 0x29ccdab4, 88 },
    { 0x9f8cdab5, 90 },
    { 0, 0 },
    { 0xb4fcdab7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bcdac3, 90 },
    { 0xd9bcdac3, 90 },
    { 0xb03cdac5, 74 },
    { 0, 0 },
    { 0xa9fcdac7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x255cdaca, 55 },
    { 0x155cdacb, 44 },
    { 0x5eacdacb, 100 },
    { 0x75fcdacd, 33 },
    { 0, 0 },
    { 0xb6cdacf, 98 },
    { 0, 0 },
    { 0xfadcdad1, 33 },
    { 0x872cdad2, 100 },
    { 0x2c0cdad2, 85 },
    { 0, 0 },
    { 0xbcfcdad5, 46 },
    { 0xc24cdad6, 84 },
    { 0x73ccdad6, 100 },
    { 0x42cdad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42cdadc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae6, 90 },
    { 0x612cdae7, 66 },
    { 0x15fcdae7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473cdaec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3cdaf7, 34 },
    { 0x4cccdaf8, 78 },
    { 0x8cacdaf8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cdaff, 81 },
    { 0x4f6cdb00, 91 },
    { 0xc87cdb00, 66 },
    { 0x1ecdb01, 93 },
    { 0x635cdb03, 76 },
    { 0xa25cdb04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ccdb07, 61 },
    { 0x9cdb08, 33 },
    { 0x68bcdb09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccdb10, 61 },
    { 0xbfdcdb10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644cdb1b, 76 },
    { 0, 0 },
    { 0xc32cdb1d, 100 },
    { 0xf0bcdb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ecdb25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ccdb28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cdb2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edcdb32, 61 },
    { 0x370cdb33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cdb3a, 95 },
    { 0xc97cdb3b, 61 },
    { 0, 0 },
    { 0x3d4cdb3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2cdb41, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ecdb44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2cdb47, 23 },
    { 0xf66cdb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903cdb51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508cdb58, 33 },
    { 0xffecdb59, 55 },
    { 0, 0 },
    { 0xdf6cdb5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cdb5e, 100 },
    { 0, 0 },
    { 0xbeccdb60, 87 },
    { 0xa4acdb60, 33 },
    { 0x13ecdb60, 81 },
    { 0x9e1cdb63, 43 },
    { 0xea0cdb63, 90 },
    { 0x61acdb65, 33 },
    { 0x541cdb66, 52 },
    { 0x80acdb66, 33 },
    { 0x733cdb67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcdb6d, 55 },
    { 0xf4acdb6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1cdb71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8cdb75, 33 },
    { 0, 0 },
    { 0xccfcdb77, 90 },
    { 0, 0 },
    { 0x380cdb79, 95 },
    { 0x7ddcdb7a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8cdb7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21cdb83, 86 },
    { 0, 0 },
    { 0xcd6cdb85, 61 },
    { 0x4c5cdb85, 43 },
    { 0xc3fcdb87, 52 },
    { 0x3b3cdb88, 100 },
    { 0x144cdb89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x769cdb8c, 66 },
    { 0xb3fcdb8c, 23 },
    { 0x77ecdb8c, 34 },
    { 0x7d0cdb8f, 66 },
    { 0xcc9cdb8f, 33 },
    { 0xbc3cdb90, 95 },
    { 0x55cdb92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cdb97, 90 },
    { 0xb12cdb98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cdb9b, 99 },
    { 0x95dcdb9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93cdba1, 33 },
    { 0x9abcdba2, 90 },
    { 0x744cdba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cdbac, 55 },
    { 0xcfdcdbad, 55 },
    { 0, 0 },
    { 0x456cdbaf, 61 },
    { 0xea5cdbb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3cdbb3, 90 },
    { 0xd0fcdbb4, 88 },
    { 0xf86cdbb5, 90 },
    { 0x1a5cdbb6, 84 },
    { 0x39ccdbb4, 46 },
    { 0x74ccdbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6cdbc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924cdbcd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08cdbd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382cdbe0, 23 },
    { 0xa7acdbe0, 33 },
    { 0xef7cdbe2, 90 },
    { 0x421cdbe3, 61 },
    { 0x782cdbe2, 69 },
    { 0xc5bcdbe5, 17 },
    { 0x7b8cdbe1, 76 },
    { 0xaebcdbe4, 80 },
    { 0x821cdbe8, 91 },
    { 0x1b5cdbe9, 34 },
    { 0, 0 },
    { 0x6eacdbeb, 40 },
    { 0x90ccdbec, 91 },
    { 0, 0 },
    { 0x131cdbee, 87 },
    { 0x9e8cdbef, 75 },
    { 0x55ccdbef, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9cdbf4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60cdbfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620cdbfe, 46 },
    { 0xba5cdbfe, 34 },
    { 0, 0 },
    { 0xe38cdc01, 93 },
    { 0, 0 },
    { 0xb94cdc03, 61 },
    { 0x177cdc03, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2cdc0c, 98 },
    { 0x5f7cdc0c, 55 },
    { 0, 0 },
    { 0xbe2cdc0f, 67 },
    { 0x3a6cdc10, 33 },
    { 0x89acdc11, 91 },
    { 0x8b1cdc11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x403cdc15, 61 },
    { 0xffcdc16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cdc19, 90 },
    { 0xea2cdc1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aacdc1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9cdc21, 61 },
    { 0xe90cdc21, 33 },
    { 0, 0 },
    { 0xd1acdc24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63cdc27, 22 },
    { 0x989cdc28, 95 },
    { 0, 0 },
    { 0x799cdc2a, 61 },
    { 0xadccdc2b, 84 },
    { 0xa6fcdc2c, 34 },
    { 0x2bfcdc2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ccdc35, 90 },
    { 0, 0 },
    { 0x42acdc37, 23 },
    { 0, 0 },
    { 0x186cdc39, 61 },
    { 0xf5dcdc3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cdc44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3cdc49, 66 },
    { 0xea5cdc49, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccdc52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ecdc59, 99 },
    { 0x1dbcdc5a, 84 },
    { 0x4b7cdc5b, 76 },
    { 0xd5ccdc59, 46 },
    { 0x350cdc5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ccdc63, 46 },
    { 0, 0 },
    { 0xfb8cdc65, 33 },
    { 0, 0 },
    { 0xa30cdc67, 33 },
    { 0xfdfcdc68, 75 },
    { 0x2a0cdc67, 33 },
    { 0xbeacdc6a, 80 },
    { 0x50ccdc6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bcdc6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3cdc71, 90 },
    { 0x7a5cdc71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cdc75, 83 },
    { 0x1dbcdc76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ecdc79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105cdc7f, 23 },
    { 0x428cdc7f, 33 },
    { 0x3fbcdc81, 61 },
    { 0xab0cdc80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cdc88, 43 },
    { 0, 0 },
    { 0xb53cdc8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8cdc8d, 95 },
    { 0, 0 },
    { 0x83ecdc8f, 61 },
    { 0x9cfcdc8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ecdc9e, 91 },
    { 0, 0 },
    { 0x8dccdca0, 61 },
    { 0x518cdca0, 33 },
    { 0x47acdca2, 84 },
    { 0xba5cdca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcdca7, 90 },
    { 0x424cdca8, 55 },
    { 0xd01cdca8, 55 },
    { 0x735cdcaa, 61 },
    { 0xd4dcdcab, 56 },
    { 0xcf7cdcab, 55 },
    { 0xf68cdcad, 95 },
    { 0x2ddcdcae, 90 },
    { 0xa22cdcad, 52 },
    { 0xe40cdca8, 23 },
    { 0xbbdcdcb0, 33 },
    { 0x6c7cdcb2, 81 },
    { 0x949cdcb2, 95 },
    { 0xeeccdcb4, 76 },
    { 0xc6ccdcb1, 43 },
    { 0, 0 },
    { 0xf70cdcb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ccdcbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7cdcc0, 88 },
    { 0xf73cdcc1, 33 },
    { 0x95ecdcc2, 23 },
    { 0x1a0cdcc3, 90 },
    { 0xceacdcc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x710cdcc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacdccd, 61 },
    { 0x59cdccd, 90 },
    { 0xfe2cdcce, 90 },
    { 0, 0 },
    { 0x6e9cdcd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83acdcd5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bcdcd9, 55 },
    { 0xa15cdcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cdce0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ecdce7, 90 },
    { 0, 0 },
    { 0x19ecdce9, 61 },
    { 0, 0 },
    { 0x7fbcdceb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54cdcee, 95 },
    { 0x54bcdcee, 33 },
    { 0, 0 },
    { 0xb3bcdcf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562cdcf5, 98 },
    { 0xb16cdcf6, 95 },
    { 0, 0 },
    { 0xaf1cdcf8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909cdcfc, 93 },
    { 0xd01cdcfd, 91 },
    { 0, 0 },
    { 0x283cdcff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9cdd03, 55 },
    { 0xfe8cdd04, 78 },
    { 0x1a3cdd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cdd08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42cdd0f, 78 },
    { 0x896cdd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222cdd17, 83 },
    { 0, 0 },
    { 0xa42cdd19, 95 },
    { 0x92acdd1a, 90 },
    { 0x434cdd1b, 52 },
    { 0x432cdd1c, 91 },
    { 0xf4ccdd1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45cdd24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871cdd2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62cdd33, 34 },
    { 0xb5bcdd34, 88 },
    { 0x417cdd35, 98 },
    { 0x7e3cdd35, 90 },
    { 0xa40cdd37, 99 },
    { 0x6fcdd38, 95 },
    { 0xa8ecdd38, 33 },
    { 0, 0 },
    { 0xd9ecdd3b, 90 },
    { 0xf32cdd3c, 33 },
    { 0x1a4cdd3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x324cdd40, 46 },
    { 0, 0 },
    { 0x12dcdd42, 78 },
    { 0x27fcdd43, 90 },
    { 0, 0 },
    { 0x39ccdd45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x859cdd48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccdd4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6cdd50, 76 },
    { 0xc7bcdd51, 91 },
    { 0xdc7cdd52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ecdd55, 33 },
    { 0, 0 },
    { 0x452cdd57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdd5b, 34 },
    { 0, 0 },
    { 0xba1cdd5d, 90 },
    { 0x273cdd5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ecdd62, 43 },
    { 0x2abcdd63, 61 },
    { 0x574cdd64, 33 },
    { 0xd5cdd65, 84 },
    { 0xff6cdd66, 90 },
    { 0x541cdd65, 33 },
    { 0xf85cdd65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7cdd6b, 33 },
    { 0x15bcdd6c, 76 },
    { 0x2fdcdd6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8cdd79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcdd7c, 43 },
    { 0, 0 },
    { 0xbb1cdd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1facdd85, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80cdd89, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23cdd90, 98 },
    { 0x4e8cdd91, 81 },
    { 0x88cdd91, 43 },
    { 0xf5ccdd93, 90 },
    { 0x9dccdd92, 51 },
    { 0, 0 },
    { 0xd40cdd96, 95 },
    { 0, 0 },
    { 0x605cdd98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaecdd9e, 95 },
    { 0x20acdd9f, 95 },
    { 0xdfcdda0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcdda4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ccddaa, 23 },
    { 0x169cddaa, 34 },
    { 0x332cddac, 52 },
    { 0x6bccddab, 87 },
    { 0x378cddab, 93 },
    { 0x825cddaf, 76 },
    { 0x5efcddaf, 98 },
    { 0x7bbcddb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90cddb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4cddc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8cddcd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45cddd0, 23 },
    { 0, 0 },
    { 0x77fcddd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3cddd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dcdddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00cdddf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cdde4, 77 },
    { 0xaf7cdde5, 61 },
    { 0, 0 },
    { 0xbc8cdde7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bcddea, 66 },
    { 0, 0 },
    { 0xaf7cddec, 55 },
    { 0, 0 },
    { 0x2dccddee, 65 },
    { 0x32acddef, 98 },
    { 0x355cddef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cddf5, 52 },
    { 0xc6dcddf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121cddf9, 84 },
    { 0xf44cddfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954cde01, 68 },
    { 0xf48cde02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2cde05, 95 },
    { 0xc1acde06, 90 },
    { 0, 0 },
    { 0x47acde08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8cde0b, 67 },
    { 0xde0cde0c, 33 },
    { 0x884cde0b, 61 },
    { 0x16bcde0c, 65 },
    { 0x419cde0f, 85 },
    { 0x4a1cde10, 34 },
    { 0xfe8cde11, 95 },
    { 0x5c2cde12, 52 },
    { 0xa1cde13, 94 },
    { 0x2e4cde13, 68 },
    { 0x21dcde13, 78 },
    { 0xf68cde15, 84 },
    { 0x8bcde14, 75 },
    { 0, 0 },
    { 0xe70cde19, 76 },
    { 0xfc4cde1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6cde22, 90 },
    { 0x829cde22, 90 },
    { 0, 0 },
    { 0xd8acde25, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cde2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388cde2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16cde37, 52 },
    { 0x448cde37, 90 },
    { 0, 0 },
    { 0xa3acde3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fcde43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccde46, 90 },
    { 0, 0 },
    { 0x7afcde48, 23 },
    { 0, 0 },
    { 0x985cde4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cde4d, 87 },
    { 0xd1cde4e, 90 },
    { 0xba8cde4f, 90 },
    { 0xd8bcde50, 34 },
    { 0x153cde51, 80 },
    { 0x1fecde4e, 52 },
    { 0x8b4cde4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82cde57, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aacde5a, 98 },
    { 0, 0 },
    { 0x5e8cde5c, 34 },
    { 0xa3cde5d, 33 },
    { 0x68cde5e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bcde61, 61 },
    { 0x28acde62, 78 },
    { 0x20cde63, 23 },
    { 0x1dfcde64, 100 },
    { 0x1bbcde65, 61 },
    { 0x297cde65, 33 },
    { 0, 0 },
    { 0xb48cde68, 84 },
    { 0, 0 },
    { 0x11bcde6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3cde6e, 96 },
    { 0x547cde6f, 34 },
    { 0xcdbcde70, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ccde78, 90 },
    { 0x618cde78, 77 },
    { 0xb4ecde7a, 90 },
    { 0, 0 },
    { 0x67bcde7c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cde83, 96 },
    { 0, 0 },
    { 0x1decde85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bcde89, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4cde8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4cde91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cde94, 95 },
    { 0x2fcde95, 33 },
    { 0xc5ecde96, 90 },
    { 0xe53cde97, 61 },
    { 0x3f6cde97, 91 },
    { 0xf9ccde99, 98 },
    { 0x6cdcde98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaecde9d, 33 },
    { 0x1bacde9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbcdea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110cdea6, 23 },
    { 0x7ebcdea6, 95 },
    { 0x40dcdea8, 46 },
    { 0x414cdea8, 90 },
    { 0, 0 },
    { 0xecdcdeab, 23 },
    { 0x2a9cdeac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05cdeb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x446cdeb7, 95 },
    { 0xf6dcdeb8, 43 },
    { 0xd37cdeb9, 78 },
    { 0, 0 },
    { 0x11bcdebb, 75 },
    { 0xb30cdebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fcdec0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23cdec3, 33 },
    { 0x9d5cdec3, 33 },
    { 0xfd9cdec5, 34 },
    { 0x715cdec4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ccded0, 90 },
    { 0, 0 },
    { 0xf58cded2, 33 },
    { 0xb39cded2, 37 },
    { 0, 0 },
    { 0x7a4cded5, 90 },
    { 0, 0 },
    { 0x91ecded7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6cdeda, 23 },
    { 0, 0 },
    { 0x77dcdedc, 91 },
    { 0xb52cdedd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x823cdee0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eecdee4, 90 },
    { 0, 0 },
    { 0x28fcdee6, 33 },
    { 0xef8cdee7, 99 },
    { 0, 0 },
    { 0x58bcdee9, 76 },
    { 0xd24cdeea, 77 },
    { 0, 0 },
    { 0x8fbcdeec, 61 },
    { 0, 0 },
    { 0x925cdeee, 23 },
    { 0xfb7cdeef, 90 },
    { 0xfacdef0, 46 },
    { 0, 0 },
    { 0x41cdef2, 76 },
    { 0x927cdef3, 91 },
    { 0x6fccdef3, 61 },
    { 0, 0 },
    { 0x9f7cdef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524cdefd, 34 },
    { 0xca1cdefd, 43 },
    { 0x213cdefd, 34 },
    { 0xd1acdefd, 61 },
    { 0x35dcdf00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192cdf07, 40 },
    { 0, 0 },
    { 0xe99cdf09, 33 },
    { 0, 0 },
    { 0xd41cdf0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cacdf0f, 95 },
    { 0, 0 },
    { 0x480cdf11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5cdf16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecccdf1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2cdf22, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7cdf29, 90 },
    { 0x2dacdf2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6cdf2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11acdf33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1cdf36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ecdf4c, 33 },
    { 0x4a3cdf4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ccdf50, 67 },
    { 0, 0 },
    { 0xf99cdf52, 61 },
    { 0, 0 },
    { 0x211cdf54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7cdf59, 90 },
    { 0x18bcdf5a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ccdf5e, 87 },
    { 0, 0 },
    { 0x358cdf60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5cdf65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41acdf6f, 93 },
    { 0x98dcdf70, 81 },
    { 0xa07cdf71, 99 },
    { 0xcf2cdf72, 95 },
    { 0xb62cdf70, 33 },
    { 0x5e6cdf74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cdf77, 77 },
    { 0, 0 },
    { 0xecfcdf79, 36 },
    { 0, 0 },
    { 0x42dcdf7b, 55 },
    { 0, 0 },
    { 0x21fcdf7d, 91 },
    { 0, 0 },
    { 0xa34cdf7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dcdf83, 90 },
    { 0xf54cdf83, 81 },
    { 0x616cdf84, 98 },
    { 0x881cdf86, 90 },
    { 0x7cdcdf87, 93 },
    { 0, 0 },
    { 0x977cdf89, 66 },
    { 0xd12cdf8a, 43 },
    { 0xfe1cdf8a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbcdf90, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4cdf94, 34 },
    { 0, 0 },
    { 0x4d9cdf96, 33 },
    { 0xc3acdf96, 89 },
    { 0, 0 },
    { 0xc46cdf99, 90 },
    { 0, 0 },
    { 0x4b1cdf9b, 33 },
    { 0xe2acdf9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x761cdf9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fcdfa2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fcdfa6, 34 },
    { 0x297cdfa7, 66 },
    { 0x530cdfa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bcdfab, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acdfae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7cdfb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662cdfbe, 65 },
    { 0, 0 },
    { 0x6f6cdfc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bcdfc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bcdfce, 61 },
    { 0x548cdfcf, 33 },
    { 0, 0 },
    { 0x763cdfd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecdfe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ccdfe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cdfe7, 33 },
    { 0xbedcdfe8, 68 },
    { 0x30acdfe7, 84 },
    { 0xda2cdfe9, 33 },
    { 0x1f6cdfeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3cdfee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4cdff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ccdff5, 55 },
    { 0x9accdff5, 33 },
    { 0x95ccdff6, 65 },
    { 0xa13cdff8, 98 },
    { 0x155cdff9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32acdffc, 52 },
    { 0xe2ecdffc, 98 },
    { 0x341cdffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ce005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ace00d, 76 },
    { 0x2fcce00e, 40 },
    { 0xe9dce00d, 59 },
    { 0xea8ce010, 90 },
    { 0xab9ce010, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x777ce014, 88 },
    { 0xa37ce015, 93 },
    { 0x9cfce016, 23 },
    { 0xa1bce017, 90 },
    { 0x1c2ce014, 61 },
    { 0x5ffce018, 43 },
    { 0x476ce017, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63ce01f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ce023, 33 },
    { 0xa09ce024, 61 },
    { 0x688ce025, 46 },
    { 0xcf9ce026, 34 },
    { 0x68ece027, 68 },
    { 0x5efce024, 98 },
    { 0xce1ce029, 98 },
    { 0xcfbce029, 84 },
    { 0x854ce02a, 55 },
    { 0xdcfce02c, 33 },
    { 0x130ce029, 95 },
    { 0xcbdce02a, 90 },
    { 0, 0 },
    { 0x209ce030, 78 },
    { 0xbcece030, 90 },
    { 0x236ce030, 91 },
    { 0x388ce033, 33 },
    { 0x407ce034, 43 },
    { 0x784ce032, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14ce03b, 98 },
    { 0xc7bce03c, 76 },
    { 0x855ce03c, 95 },
    { 0xc71ce03e, 61 },
    { 0, 0 },
    { 0x5f6ce040, 98 },
    { 0x641ce041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03ce045, 33 },
    { 0x174ce046, 80 },
    { 0x918ce046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddce04c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ece058, 65 },
    { 0, 0 },
    { 0x2a4ce05a, 55 },
    { 0x19cce05a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cce061, 90 },
    { 0x5c8ce062, 87 },
    { 0xdc9ce062, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efce068, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fce079, 93 },
    { 0x662ce079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ce07f, 55 },
    { 0x4acce07f, 93 },
    { 0x72ace081, 76 },
    { 0x80ece082, 98 },
    { 0, 0 },
    { 0x11ece084, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdce088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559ce08c, 23 },
    { 0xb77ce08c, 91 },
    { 0x597ce08c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5ce092, 67 },
    { 0x7f7ce093, 66 },
    { 0x294ce094, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3ce098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ce09b, 76 },
    { 0x299ce09b, 61 },
    { 0xc72ce09d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ce0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8ce0a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccce0aa, 76 },
    { 0, 0 },
    { 0xd9ece0ac, 46 },
    { 0, 0 },
    { 0xf24ce0ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6ce0b5, 53 },
    { 0xc2ce0b6, 33 },
    { 0, 0 },
    { 0xd5bce0b8, 65 },
    { 0x960ce0b8, 87 },
    { 0xb5ace0ba, 43 },
    { 0xbc2ce0bb, 99 },
    { 0xa6ece0ba, 55 },
    { 0x2b4ce0ba, 61 },
    { 0x647ce0bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ce0c1, 93 },
    { 0, 0 },
    { 0x73ace0c3, 33 },
    { 0xfece0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fce0ca, 98 },
    { 0x55bce0cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ce0cf, 23 },
    { 0, 0 },
    { 0x15ece0d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dece0d8, 69 },
    { 0xef5ce0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542ce0dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6face0e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7ce0e3, 91 },
    { 0x9a7ce0e4, 62 },
    { 0, 0 },
    { 0xb2ace0e6, 43 },
    { 0x940ce0e6, 33 },
    { 0, 0 },
    { 0x44ce0e9, 55 },
    { 0x89ce0e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ce0ee, 98 },
    { 0, 0 },
    { 0x69ece0f0, 99 },
    { 0x82ace0f1, 77 },
    { 0x865ce0f1, 53 },
    { 0x4a3ce0f1, 98 },
    { 0, 0 },
    { 0x597ce0f5, 33 },
    { 0x3aace0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fce0fa, 66 },
    { 0, 0 },
    { 0x386ce0fc, 34 },
    { 0, 0 },
    { 0x6b9ce0fe, 90 },
    { 0, 0 },
    { 0xa4dce100, 76 },
    { 0xb11ce100, 90 },
    { 0x8bece102, 85 },
    { 0x41bce103, 98 },
    { 0xce6ce104, 95 },
    { 0xf54ce104, 93 },
    { 0xc77ce106, 61 },
    { 0x2e0ce107, 69 },
    { 0, 0 },
    { 0x1efce109, 90 },
    { 0x183ce10a, 98 },
    { 0, 0 },
    { 0x734ce10c, 66 },
    { 0xe0bce10d, 90 },
    { 0x38bce10d, 90 },
    { 0x2a1ce10f, 34 },
    { 0x20ace110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcce113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ce117, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bce11b, 61 },
    { 0xc33ce11c, 55 },
    { 0, 0 },
    { 0x582ce11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508ce122, 95 },
    { 0, 0 },
    { 0xdb9ce124, 33 },
    { 0x7bcce125, 98 },
    { 0x1d3ce126, 85 },
    { 0x87fce126, 33 },
    { 0, 0 },
    { 0x52dce129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41ce130, 99 },
    { 0x43dce131, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ce134, 61 },
    { 0xb76ce135, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578ce139, 61 },
    { 0xfc8ce139, 93 },
    { 0xb5cce13a, 61 },
    { 0xb76ce139, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dce142, 33 },
    { 0xca8ce143, 90 },
    { 0x340ce144, 90 },
    { 0x6bbce145, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dce14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adce152, 85 },
    { 0x788ce153, 81 },
    { 0x6d2ce154, 91 },
    { 0x1fcce154, 61 },
    { 0x9edce154, 67 },
    { 0, 0 },
    { 0x79dce158, 33 },
    { 0x24bce159, 78 },
    { 0x84fce15a, 99 },
    { 0x354ce15a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ce160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ce164, 61 },
    { 0xacfce165, 81 },
    { 0x90ce165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9ce16a, 61 },
    { 0xe35ce16b, 95 },
    { 0x861ce16b, 87 },
    { 0xf90ce16c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cce170, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8ce177, 52 },
    { 0, 0 },
    { 0xe27ce179, 100 },
    { 0, 0 },
    { 0x335ce17b, 33 },
    { 0, 0 },
    { 0x91ce17d, 78 },
    { 0xec5ce17d, 61 },
    { 0, 0 },
    { 0xc41ce180, 99 },
    { 0x9dcce181, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ce185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ace188, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fce18f, 95 },
    { 0, 0 },
    { 0xbebce191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731ce195, 95 },
    { 0, 0 },
    { 0x15ace197, 66 },
    { 0x539ce198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fce1a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bce1a3, 88 },
    { 0x585ce1a4, 61 },
    { 0xc20ce1a4, 90 },
    { 0x917ce1a4, 33 },
    { 0, 0 },
    { 0x173ce1a8, 36 },
    { 0x725ce1a9, 46 },
    { 0x638ce1aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x816ce1ad, 33 },
    { 0x122ce1ad, 43 },
    { 0x262ce1af, 90 },
    { 0xb1fce1b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ece1b5, 61 },
    { 0x9ece1b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ce1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68ce1bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce1bf, 90 },
    { 0, 0 },
    { 0x565ce1c1, 76 },
    { 0x94bce1c1, 17 },
    { 0x26cce1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cce1c7, 81 },
    { 0, 0 },
    { 0xacece1c9, 87 },
    { 0x82bce1ca, 66 },
    { 0xb49ce1c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502ce1cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ce1d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ce1dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dce1e2, 61 },
    { 0, 0 },
    { 0xaf4ce1e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96ce1e7, 23 },
    { 0x7abce1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1ce1ed, 3 },
    { 0, 0 },
    { 0xadfce1ef, 90 },
    { 0xd43ce1f0, 99 },
    { 0x93bce1f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ce1f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eece1fb, 33 },
    { 0x40ece1fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ace1ff, 90 },
    { 0xb29ce200, 23 },
    { 0x806ce1ff, 33 },
    { 0x5dace202, 38 },
    { 0, 0 },
    { 0xdccce204, 61 },
    { 0, 0 },
    { 0x34bce206, 23 },
    { 0, 0 },
    { 0xc63ce208, 52 },
    { 0xf4cce209, 33 },
    { 0x7acce209, 74 },
    { 0x466ce208, 33 },
    { 0xed6ce209, 33 },
    { 0x954ce20d, 33 },
    { 0x8eece208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ece218, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ce21d, 52 },
    { 0x1a8ce21d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171ce22a, 61 },
    { 0, 0 },
    { 0xeddce22c, 61 },
    { 0x57bce22d, 81 },
    { 0xefbce22d, 43 },
    { 0xc45ce22d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73ce235, 90 },
    { 0xeece236, 33 },
    { 0, 0 },
    { 0x152ce238, 87 },
    { 0, 0 },
    { 0xbb0ce23a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ce243, 81 },
    { 0x14dce243, 33 },
    { 0x2e2ce244, 33 },
    { 0xf28ce245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ce249, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0ce24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ce251, 85 },
    { 0, 0 },
    { 0xe4ace253, 98 },
    { 0, 0 },
    { 0x57dce255, 61 },
    { 0xfd0ce255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ce25b, 99 },
    { 0x6ccce25b, 61 },
    { 0, 0 },
    { 0xdbece25e, 36 },
    { 0x90ece25f, 17 },
    { 0, 0 },
    { 0x635ce261, 55 },
    { 0xe38ce262, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998ce269, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbce26c, 95 },
    { 0xe0bce26d, 94 },
    { 0x878ce26d, 55 },
    { 0x8bfce26f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ce275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09ce27b, 23 },
    { 0x1e3ce27c, 90 },
    { 0x922ce27b, 61 },
    { 0x64ece27e, 52 },
    { 0x638ce27f, 85 },
    { 0xf28ce27f, 46 },
    { 0, 0 },
    { 0x7b4ce282, 43 },
    { 0x107ce283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4ce288, 87 },
    { 0x772ce289, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ece28d, 99 },
    { 0, 0 },
    { 0xdeece28f, 95 },
    { 0, 0 },
    { 0x294ce291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ce294, 33 },
    { 0xdfdce295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fce29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abce2a3, 76 },
    { 0x96ece2a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cce2a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce2af, 78 },
    { 0x162ce2af, 65 },
    { 0x898ce2b1, 56 },
    { 0xe8bce2b2, 84 },
    { 0xb9cce2b3, 88 },
    { 0x99bce2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cce2bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce2c0, 78 },
    { 0x731ce2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcce2cc, 84 },
    { 0xaccce2cd, 40 },
    { 0x36dce2ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ce2d3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ce2d7, 23 },
    { 0xd08ce2d8, 95 },
    { 0x4bcce2d9, 61 },
    { 0x5d1ce2d9, 76 },
    { 0x708ce2d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cce2e3, 23 },
    { 0xdb5ce2e4, 55 },
    { 0x15cce2e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459ce2eb, 55 },
    { 0x6c2ce2ec, 77 },
    { 0, 0 },
    { 0xfcbce2ee, 84 },
    { 0x6ace2ee, 95 },
    { 0x10ce2f0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fce2f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x664ce2fb, 98 },
    { 0xd66ce2fc, 98 },
    { 0x26bce2fc, 90 },
    { 0x2b2ce2fe, 36 },
    { 0xb2fce2ff, 33 },
    { 0xff5ce2fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bce307, 77 },
    { 0xab3ce308, 61 },
    { 0, 0 },
    { 0x5c6ce30a, 76 },
    { 0, 0 },
    { 0x555ce30c, 98 },
    { 0xfbace30d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2ce313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ece316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17ce31e, 76 },
    { 0, 0 },
    { 0x905ce320, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dce323, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adce32b, 34 },
    { 0x658ce32c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ce332, 80 },
    { 0x967ce332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ce337, 33 },
    { 0xa78ce338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bce33b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141ce34b, 93 },
    { 0x3fcce34b, 33 },
    { 0xf4ece34d, 55 },
    { 0xb73ce34e, 91 },
    { 0x28ece34d, 90 },
    { 0, 0 },
    { 0xcd0ce351, 66 },
    { 0x90bce352, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6ce358, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74ce35c, 76 },
    { 0xb18ce35d, 93 },
    { 0x1d6ce35e, 67 },
    { 0xe34ce35d, 42 },
    { 0xc13ce35c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6ce364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ce367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ce370, 84 },
    { 0, 0 },
    { 0xbe5ce372, 76 },
    { 0, 0 },
    { 0x55fce374, 69 },
    { 0x643ce374, 95 },
    { 0x266ce376, 84 },
    { 0x9bfce377, 95 },
    { 0xb67ce378, 61 },
    { 0x6f5ce379, 98 },
    { 0xb06ce37a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ce37d, 43 },
    { 0x511ce37d, 95 },
    { 0, 0 },
    { 0x654ce380, 61 },
    { 0xeace381, 76 },
    { 0, 0 },
    { 0xd50ce383, 78 },
    { 0xeaace383, 88 },
    { 0, 0 },
    { 0xbd6ce386, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ce38a, 90 },
    { 0xdaece38a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60ce38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2ce393, 23 },
    { 0, 0 },
    { 0xdccce395, 43 },
    { 0, 0 },
    { 0xe5ce397, 33 },
    { 0xb3bce398, 77 },
    { 0x84dce398, 90 },
    { 0, 0 },
    { 0x7fece39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dce3a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bace3a6, 90 },
    { 0xdc9ce3a7, 90 },
    { 0x2a9ce3a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974ce3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13ce3b1, 95 },
    { 0, 0 },
    { 0x8ece3b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ce3bd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbece3c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ce3c7, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfce3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ace3ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddce3d5, 95 },
    { 0xfd0ce3d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87ce3de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ce3e1, 77 },
    { 0, 0 },
    { 0x5a0ce3e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1ce3e6, 87 },
    { 0x13dce3e6, 84 },
    { 0xa8bce3e8, 78 },
    { 0x289ce3e8, 90 },
    { 0x78ce3e9, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfce3ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bce3f5, 98 },
    { 0x7d7ce3f6, 90 },
    { 0x950ce3f7, 100 },
    { 0x7f6ce3f5, 33 },
    { 0x81cce3f5, 90 },
    { 0xd27ce3fa, 61 },
    { 0x4ce3f5, 93 },
    { 0, 0 },
    { 0x2fbce3fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73ce401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322ce409, 66 },
    { 0, 0 },
    { 0x739ce40b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabce40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1ce413, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ce41b, 98 },
    { 0xb27ce41c, 98 },
    { 0, 0 },
    { 0x40cce41e, 76 },
    { 0x73ece41f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce42f, 99 },
    { 0x22ace430, 90 },
    { 0x674ce431, 93 },
    { 0x6bce431, 91 },
    { 0, 0 },
    { 0x450ce434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2ce439, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ece43c, 98 },
    { 0, 0 },
    { 0xf26ce43e, 40 },
    { 0x8a7ce43e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ace444, 23 },
    { 0x4e9ce445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bce44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dce44d, 46 },
    { 0xa12ce44e, 75 },
    { 0x491ce44f, 91 },
    { 0x754ce44f, 90 },
    { 0x2d9ce450, 56 },
    { 0x306ce44d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556ce456, 98 },
    { 0xe15ce456, 67 },
    { 0, 0 },
    { 0x9bce459, 52 },
    { 0xdcdce45a, 95 },
    { 0, 0 },
    { 0x64ce45c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fce45f, 23 },
    { 0x63ece460, 66 },
    { 0, 0 },
    { 0xb44ce462, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdface465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fce468, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cce46b, 95 },
    { 0, 0 },
    { 0x4fbce46d, 98 },
    { 0, 0 },
    { 0x91ce46f, 55 },
    { 0, 0 },
    { 0xfcfce471, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bace477, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ce47b, 61 },
    { 0xdc5ce47b, 61 },
    { 0, 0 },
    { 0x787ce47e, 66 },
    { 0, 0 },
    { 0x18ece480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186ce485, 90 },
    { 0, 0 },
    { 0x583ce487, 98 },
    { 0xc3ce488, 76 },
    { 0, 0 },
    { 0x636ce48a, 23 },
    { 0xb79ce48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6ce48e, 61 },
    { 0x357ce48f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388ce495, 33 },
    { 0x8ccce496, 98 },
    { 0, 0 },
    { 0xa5dce498, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ce4a1, 61 },
    { 0xa69ce4a1, 52 },
    { 0x85ace4a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x911ce4a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ace4a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ce4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ce4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ce4b8, 61 },
    { 0, 0 },
    { 0x58bce4ba, 17 },
    { 0xb08ce4bb, 98 },
    { 0, 0 },
    { 0x51ce4bd, 33 },
    { 0xc5ace4bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ce4c2, 95 },
    { 0, 0 },
    { 0xacdce4c4, 43 },
    { 0, 0 },
    { 0xde9ce4c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bce4c9, 90 },
    { 0x2e3ce4ca, 90 },
    { 0x45fce4cb, 98 },
    { 0xe0ace4ca, 33 },
    { 0x257ce4ca, 95 },
    { 0x43bce4ca, 98 },
    { 0x3d2ce4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ce4d3, 23 },
    { 0x429ce4d4, 95 },
    { 0x40ace4d4, 99 },
    { 0, 0 },
    { 0x1e7ce4d7, 23 },
    { 0, 0 },
    { 0x58cce4d9, 76 },
    { 0xc66ce4da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818ce4e2, 84 },
    { 0x4e1ce4e3, 33 },
    { 0x78ece4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ce4e8, 76 },
    { 0x2a9ce4e9, 43 },
    { 0xadace4ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ce4ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ce4f1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x129ce4f4, 33 },
    { 0xfd6ce4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x425ce4f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x453ce4fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ce507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511ce50c, 23 },
    { 0x2d9ce50d, 80 },
    { 0x83dce50e, 90 },
    { 0, 0 },
    { 0xff2ce510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fce514, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x654ce517, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ce51e, 65 },
    { 0, 0 },
    { 0x87ace520, 90 },
    { 0xda2ce520, 88 },
    { 0x4f3ce522, 97 },
    { 0x29bce523, 98 },
    { 0xba8ce521, 76 },
    { 0xc0dce523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ce529, 90 },
    { 0, 0 },
    { 0xdfcce52b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dce52e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dce531, 76 },
    { 0x12ece532, 33 },
    { 0xbe8ce533, 33 },
    { 0x636ce534, 76 },
    { 0xacece533, 95 },
    { 0, 0 },
    { 0x1efce537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688ce53b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2ce541, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ece544, 46 },
    { 0x6b2ce545, 55 },
    { 0x1e5ce546, 69 },
    { 0xb72ce544, 90 },
    { 0, 0 },
    { 0x396ce549, 100 },
    { 0xea3ce54a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ece54e, 95 },
    { 0, 0 },
    { 0xe26ce550, 61 },
    { 0x223ce551, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbace557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ce55a, 90 },
    { 0x96fce55b, 90 },
    { 0, 0 },
    { 0x3d1ce55d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cace562, 61 },
    { 0, 0 },
    { 0x8ddce564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7ce572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x689ce575, 33 },
    { 0xe2ece576, 66 },
    { 0xbd7ce576, 80 },
    { 0x6dbce578, 98 },
    { 0x1a7ce579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6ce57f, 98 },
    { 0x999ce580, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ce586, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffce58b, 34 },
    { 0, 0 },
    { 0x589ce58d, 61 },
    { 0xb7ce58e, 43 },
    { 0xc1ace58f, 85 },
    { 0x8d0ce590, 23 },
    { 0xbf8ce590, 34 },
    { 0x7a7ce592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ce596, 33 },
    { 0x62ace596, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebce59a, 93 },
    { 0, 0 },
    { 0xf86ce59c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327ce5a1, 91 },
    { 0xc7dce5a1, 33 },
    { 0x31bce5a3, 68 },
    { 0x650ce5a4, 46 },
    { 0x184ce5a5, 76 },
    { 0xbeece5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7ce5aa, 66 },
    { 0xf5ace5ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1ce5b0, 43 },
    { 0x453ce5b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbace5b6, 43 },
    { 0, 0 },
    { 0xf10ce5b8, 95 },
    { 0xdcece5b9, 69 },
    { 0xb4fce5b8, 99 },
    { 0x53fce5b8, 98 },
    { 0x9c4ce5b9, 55 },
    { 0, 0 },
    { 0x204ce5be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6ce5c4, 77 },
    { 0xe8cce5c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9ce5c9, 87 },
    { 0x773ce5ca, 34 },
    { 0xdfcce5cb, 33 },
    { 0xe8cce5cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ce5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404ce5d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ce5db, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cce5e0, 90 },
    { 0xa04ce5e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x143ce5e4, 43 },
    { 0x82ace5e5, 23 },
    { 0x2e1ce5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dce5f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ce5f6, 34 },
    { 0xbd8ce5f7, 17 },
    { 0xb93ce5f6, 52 },
    { 0, 0 },
    { 0x378ce5fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2ce600, 90 },
    { 0, 0 },
    { 0xaa3ce602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dce606, 17 },
    { 0xb82ce607, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefece60f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4ce617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fce629, 90 },
    { 0, 0 },
    { 0x75ce62b, 76 },
    { 0, 0 },
    { 0xd4dce62d, 46 },
    { 0, 0 },
    { 0x804ce62f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fce635, 77 },
    { 0x3c1ce636, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ce639, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ce63c, 95 },
    { 0x861ce63c, 67 },
    { 0xbacce63e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ce641, 55 },
    { 0xaa8ce642, 55 },
    { 0x7fdce643, 33 },
    { 0x559ce644, 90 },
    { 0x909ce644, 90 },
    { 0x45ece646, 43 },
    { 0, 0 },
    { 0x6a6ce648, 68 },
    { 0x142ce649, 98 },
    { 0xe1ce64a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ce64d, 90 },
    { 0x5dcce64d, 78 },
    { 0xa8ace64f, 52 },
    { 0, 0 },
    { 0x433ce651, 95 },
    { 0, 0 },
    { 0xcfece653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9ce656, 33 },
    { 0x520ce657, 33 },
    { 0xebfce657, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75ce65d, 65 },
    { 0x753ce65e, 46 },
    { 0, 0 },
    { 0xe71ce660, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645ce664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x524ce667, 90 },
    { 0, 0 },
    { 0xcb0ce669, 77 },
    { 0xdb1ce669, 93 },
    { 0x624ce66b, 98 },
    { 0xb9ace66b, 93 },
    { 0xd0dce66d, 90 },
    { 0, 0 },
    { 0xccbce66f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43ce674, 55 },
    { 0, 0 },
    { 0xf0ece676, 90 },
    { 0x874ce677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11ce67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x921ce67d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bce682, 42 },
    { 0x759ce682, 36 },
    { 0, 0 },
    { 0x67bce685, 34 },
    { 0x280ce685, 95 },
    { 0, 0 },
    { 0x585ce688, 98 },
    { 0xb6bce688, 98 },
    { 0x5c0ce68a, 33 },
    { 0x4fbce689, 91 },
    { 0, 0 },
    { 0xde5ce68d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dce691, 99 },
    { 0x9c4ce692, 90 },
    { 0x6a8ce693, 66 },
    { 0x571ce694, 55 },
    { 0x329ce695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ce69e, 23 },
    { 0xe46ce69e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dce6a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17ce6a6, 84 },
    { 0xaf4ce6a6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bce6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28ce6af, 88 },
    { 0x48ce6b0, 76 },
    { 0x8fcce6b1, 98 },
    { 0x94ece6af, 74 },
    { 0xf32ce6b3, 23 },
    { 0xcbce6b3, 76 },
    { 0x6b6ce6b3, 61 },
    { 0xcf0ce6b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ce6be, 90 },
    { 0x532ce6bf, 66 },
    { 0xceace6be, 34 },
    { 0x880ce6c1, 33 },
    { 0xae0ce6c2, 76 },
    { 0xba8ce6c2, 96 },
    { 0, 0 },
    { 0x3a4ce6c5, 33 },
    { 0, 0 },
    { 0xeebce6c7, 91 },
    { 0x17ece6c8, 61 },
    { 0x2cdce6c9, 98 },
    { 0x8f9ce6c8, 61 },
    { 0x132ce6cb, 100 },
    { 0xf50ce6cc, 76 },
    { 0xe3bce6cd, 33 },
    { 0xa65ce6ce, 61 },
    { 0x4bece6cf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bce6d3, 33 },
    { 0x43ace6d4, 90 },
    { 0xc17ce6d3, 33 },
    { 0x6d4ce6d4, 61 },
    { 0, 0 },
    { 0x820ce6d8, 84 },
    { 0, 0 },
    { 0x831ce6da, 90 },
    { 0x38ece6db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace6df, 95 },
    { 0, 0 },
    { 0x271ce6e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x808ce6e4, 33 },
    { 0x8b2ce6e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x181ce6e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ece6ec, 76 },
    { 0, 0 },
    { 0x7a2ce6ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dce6f7, 90 },
    { 0x8d1ce6f7, 23 },
    { 0, 0 },
    { 0x5f9ce6fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ce6fd, 90 },
    { 0, 0 },
    { 0x686ce6ff, 23 },
    { 0x2c7ce700, 23 },
    { 0x574ce701, 93 },
    { 0, 0 },
    { 0xeaace703, 90 },
    { 0, 0 },
    { 0xcdece705, 90 },
    { 0, 0 },
    { 0x60bce707, 95 },
    { 0xf3ace708, 95 },
    { 0xf89ce708, 78 },
    { 0x21ce709, 90 },
    { 0, 0 },
    { 0x9b8ce70c, 91 },
    { 0xfc1ce70d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901ce711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4ce716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2ce719, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ce71d, 95 },
    { 0xc8fce71e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce721, 33 },
    { 0, 0 },
    { 0xc2bce723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ece727, 46 },
    { 0x5cace728, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09ce72c, 33 },
    { 0xac7ce72d, 90 },
    { 0x272ce72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ce732, 98 },
    { 0x3edce733, 23 },
    { 0, 0 },
    { 0x553ce735, 90 },
    { 0xe03ce736, 61 },
    { 0, 0 },
    { 0xda2ce738, 34 },
    { 0x6bfce739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ce73c, 34 },
    { 0x2dfce73c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cce742, 43 },
    { 0x1adce742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ce747, 68 },
    { 0x78ce748, 23 },
    { 0x965ce749, 95 },
    { 0x575ce749, 95 },
    { 0x6b2ce74b, 43 },
    { 0x61ece74c, 76 },
    { 0x4d6ce74b, 61 },
    { 0xcbfce74c, 17 },
    { 0x4f9ce74a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dce759, 90 },
    { 0x3a9ce759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x511ce75d, 96 },
    { 0xf9ce75d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bce761, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0ce764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15ce76a, 93 },
    { 0, 0 },
    { 0x443ce76c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ace76f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fce774, 95 },
    { 0, 0 },
    { 0xb8ace776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebce779, 34 },
    { 0, 0 },
    { 0xfb2ce77b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ece781, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ce785, 66 },
    { 0, 0 },
    { 0x4d4ce787, 23 },
    { 0x289ce788, 98 },
    { 0, 0 },
    { 0x508ce78a, 98 },
    { 0xcb8ce78a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543ce78f, 46 },
    { 0, 0 },
    { 0x488ce791, 100 },
    { 0x18dce791, 33 },
    { 0x4b7ce792, 83 },
    { 0x65fce794, 98 },
    { 0x66fce794, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bce798, 93 },
    { 0x5c3ce799, 33 },
    { 0x48fce79a, 23 },
    { 0xd9bce798, 61 },
    { 0xf07ce799, 61 },
    { 0, 0 },
    { 0xc2bce79e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ce7a5, 33 },
    { 0x212ce7a6, 23 },
    { 0xc05ce7a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x413ce7aa, 43 },
    { 0xb2fce7ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ce7af, 91 },
    { 0x584ce7b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ace7b3, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ece7b6, 77 },
    { 0, 0 },
    { 0x4f4ce7b8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ce7bb, 55 },
    { 0x651ce7bc, 76 },
    { 0x479ce7bd, 46 },
    { 0x3e1ce7bd, 81 },
    { 0xe60ce7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eace7c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adce7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ce7cb, 90 },
    { 0x72ace7cc, 90 },
    { 0, 0 },
    { 0x5ccce7ce, 34 },
    { 0, 0 },
    { 0xf46ce7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece7d5, 65 },
    { 0, 0 },
    { 0x369ce7d7, 91 },
    { 0xf7ace7d8, 90 },
    { 0x1d6ce7d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0ce7dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adce7e5, 23 },
    { 0xdadce7e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9ce7f1, 95 },
    { 0, 0 },
    { 0x7ace7f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ce7fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ace802, 33 },
    { 0xd5ece802, 33 },
    { 0xe14ce804, 61 },
    { 0x264ce802, 90 },
    { 0, 0 },
    { 0xc40ce807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eece80a, 55 },
    { 0xb87ce80b, 68 },
    { 0xa6ce80b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39ce814, 95 },
    { 0, 0 },
    { 0xcf5ce816, 34 },
    { 0xe6ece817, 90 },
    { 0xeadce818, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ce81d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ce821, 90 },
    { 0, 0 },
    { 0xa98ce823, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130ce827, 61 },
    { 0, 0 },
    { 0x1c6ce829, 34 },
    { 0, 0 },
    { 0xbc6ce82b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fce831, 61 },
    { 0xa7ece832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cce835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce83a, 75 },
    { 0, 0 },
    { 0x18bce83c, 93 },
    { 0, 0 },
    { 0x2b3ce83e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcace841, 100 },
    { 0x781ce841, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62ce84c, 43 },
    { 0, 0 },
    { 0xb6ece84e, 61 },
    { 0, 0 },
    { 0xf66ce850, 34 },
    { 0, 0 },
    { 0xad5ce852, 78 },
    { 0, 0 },
    { 0x620ce854, 95 },
    { 0, 0 },
    { 0x6e4ce856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x249ce859, 23 },
    { 0, 0 },
    { 0xadece85b, 33 },
    { 0x39ce85c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bce85f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229ce863, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dce86a, 90 },
    { 0xab3ce86a, 77 },
    { 0x5f4ce86c, 97 },
    { 0xb45ce86c, 33 },
    { 0, 0 },
    { 0xd0fce86f, 61 },
    { 0x2f1ce86f, 95 },
    { 0, 0 },
    { 0x6ddce872, 87 },
    { 0xf94ce873, 64 },
    { 0x829ce872, 33 },
    { 0xb6cce872, 76 },
    { 0xc2ece876, 82 },
    { 0x3a0ce876, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16ce87a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ace87d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce880, 96 },
    { 0x16ce881, 90 },
    { 0x62fce882, 98 },
    { 0x8fbce883, 95 },
    { 0x6cfce882, 33 },
    { 0x8bce882, 25 },
    { 0xfd0ce882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ce88a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ce88f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960ce893, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cce89a, 88 },
    { 0xcce89b, 88 },
    { 0, 0 },
    { 0xa74ce89d, 23 },
    { 0, 0 },
    { 0x635ce89f, 33 },
    { 0xe3dce8a0, 95 },
    { 0x997ce8a1, 99 },
    { 0x3d8ce8a2, 61 },
    { 0x3d4ce8a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dce8ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ace8ae, 99 },
    { 0xcd8ce8ae, 85 },
    { 0xbdece8af, 98 },
    { 0x918ce8b1, 95 },
    { 0x3fdce8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdce8b7, 43 },
    { 0x7a6ce8b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8ce8bb, 98 },
    { 0, 0 },
    { 0xdb7ce8bd, 33 },
    { 0, 0 },
    { 0xc97ce8bf, 76 },
    { 0x2c6ce8c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfce8c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24ce8c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01ce8d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52ce8db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8ce8de, 33 },
    { 0, 0 },
    { 0xef7ce8e0, 55 },
    { 0xfbdce8e1, 33 },
    { 0xb86ce8e2, 33 },
    { 0x298ce8e3, 75 },
    { 0xf31ce8e2, 98 },
    { 0, 0 },
    { 0xdb0ce8e6, 46 },
    { 0, 0 },
    { 0xd95ce8e8, 23 },
    { 0, 0 },
    { 0xc8ece8ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ce8f6, 61 },
    { 0, 0 },
    { 0x945ce8f8, 40 },
    { 0x64ace8f9, 61 },
    { 0xa05ce8fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfce8ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abce904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6ce907, 33 },
    { 0x6cace907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ce90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ce90f, 95 },
    { 0x7d8ce910, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fce915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dce918, 23 },
    { 0x428ce918, 33 },
    { 0x2d9ce919, 36 },
    { 0xd39ce91b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1ce91e, 93 },
    { 0xc5ace91e, 90 },
    { 0xbc3ce920, 88 },
    { 0x382ce921, 98 },
    { 0, 0 },
    { 0x4dcce923, 46 },
    { 0xa60ce924, 36 },
    { 0x4bce923, 23 },
    { 0x223ce926, 88 },
    { 0x6dace927, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ce92a, 77 },
    { 0x6bbce92b, 98 },
    { 0x279ce92b, 95 },
    { 0, 0 },
    { 0x9aece92e, 39 },
    { 0xf03ce92f, 98 },
    { 0x2a0ce930, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce933, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0ce936, 61 },
    { 0, 0 },
    { 0xca9ce938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fce93b, 96 },
    { 0, 0 },
    { 0x7c8ce93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ce942, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bce94b, 23 },
    { 0xdaece94b, 76 },
    { 0, 0 },
    { 0xc1ce94e, 61 },
    { 0xeefce94e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ce953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30ce958, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ce95e, 33 },
    { 0x607ce95f, 99 },
    { 0x625ce960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bce965, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9ce969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ce96c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadce974, 98 },
    { 0xc43ce975, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ce978, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0ce97f, 61 },
    { 0xe5ece980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbce985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfce990, 33 },
    { 0x59fce991, 33 },
    { 0x10ece992, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ce99f, 33 },
    { 0, 0 },
    { 0xa38ce9a1, 34 },
    { 0, 0 },
    { 0x194ce9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ce9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ce9aa, 61 },
    { 0, 0 },
    { 0x62fce9ac, 98 },
    { 0xe94ce9ad, 61 },
    { 0, 0 },
    { 0xa2ece9af, 52 },
    { 0x4c3ce9b0, 76 },
    { 0xf49ce9b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ce9be, 90 },
    { 0x27bce9bf, 90 },
    { 0xcf8ce9c0, 76 },
    { 0xc92ce9c1, 33 },
    { 0x938ce9c1, 12 },
    { 0xf7dce9c3, 23 },
    { 0xb52ce9c3, 43 },
    { 0x4c5ce9c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8ce9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x619ce9cf, 34 },
    { 0xdce9d0, 76 },
    { 0x1f5ce9d1, 95 },
    { 0x93fce9d0, 34 },
    { 0x802ce9d0, 61 },
    { 0, 0 },
    { 0xd0ece9d5, 87 },
    { 0x3c2ce9d6, 23 },
    { 0x3c4ce9d5, 76 },
    { 0, 0 },
    { 0x5e9ce9d9, 95 },
    { 0, 0 },
    { 0x200ce9db, 90 },
    { 0, 0 },
    { 0xe44ce9dd, 95 },
    { 0, 0 },
    { 0x7a9ce9df, 67 },
    { 0, 0 },
    { 0x909ce9e1, 34 },
    { 0, 0 },
    { 0x8face9e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bce9e6, 34 },
    { 0xb22ce9e7, 90 },
    { 0x391ce9e8, 100 },
    { 0x5d1ce9e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ece9ec, 90 },
    { 0xafece9ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bce9f0, 91 },
    { 0x27ece9f1, 33 },
    { 0x248ce9f1, 90 },
    { 0x69cce9f2, 95 },
    { 0xf73ce9f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ce9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fecea00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dcea04, 70 },
    { 0x694cea05, 43 },
    { 0xaf0cea04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddcea09, 78 },
    { 0, 0 },
    { 0xf52cea0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3cea11, 95 },
    { 0x17acea12, 52 },
    { 0, 0 },
    { 0x9dfcea14, 67 },
    { 0xb49cea14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfcea18, 83 },
    { 0x76ccea18, 55 },
    { 0x28fcea1a, 84 },
    { 0xaa0cea1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cea23, 66 },
    { 0xd7fcea23, 61 },
    { 0xb95cea25, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8cea29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acea2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06cea34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55cea39, 33 },
    { 0xba4cea3a, 88 },
    { 0x7a4cea3a, 95 },
    { 0x8fbcea3b, 90 },
    { 0xe6ccea3d, 76 },
    { 0, 0 },
    { 0xf33cea3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5cea42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3cea46, 33 },
    { 0, 0 },
    { 0xe1dcea48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8cea4c, 34 },
    { 0, 0 },
    { 0xf1acea4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2cea52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcea5a, 40 },
    { 0, 0 },
    { 0x93ccea5c, 99 },
    { 0x427cea5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edcea60, 17 },
    { 0x350cea60, 97 },
    { 0x563cea62, 101 },
    { 0xd3bcea63, 43 },
    { 0x8ccea62, 9 },
    { 0x422cea65, 90 },
    { 0, 0 },
    { 0x963cea67, 55 },
    { 0x1cacea68, 93 },
    { 0xb81cea69, 98 },
    { 0, 0 },
    { 0x7b3cea6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfcea6e, 61 },
    { 0, 0 },
    { 0x73dcea70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cea75, 61 },
    { 0, 0 },
    { 0x7c3cea77, 33 },
    { 0, 0 },
    { 0xb96cea79, 90 },
    { 0xaaacea79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375cea7e, 34 },
    { 0xaeecea7f, 90 },
    { 0xe9bcea7f, 23 },
    { 0, 0 },
    { 0x5b8cea82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x426cea85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7cea89, 46 },
    { 0xfa0cea8a, 95 },
    { 0xd25cea89, 85 },
    { 0xe5ecea8c, 84 },
    { 0x81cea8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0cea90, 23 },
    { 0x3efcea91, 90 },
    { 0x172cea90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fcea96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acea99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92acea9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ceaa0, 23 },
    { 0xbfceaa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bceaa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x748ceaa7, 87 },
    { 0x153ceaa8, 91 },
    { 0xacfceaa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dceaaf, 33 },
    { 0xf7aceab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42ceab4, 80 },
    { 0, 0 },
    { 0xd27ceab6, 38 },
    { 0, 0 },
    { 0x3e9ceab8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dceabc, 90 },
    { 0x452ceabd, 91 },
    { 0x529ceabd, 81 },
    { 0xb14ceabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797cead2, 52 },
    { 0x4dacead2, 98 },
    { 0x50acead4, 90 },
    { 0xae6cead5, 34 },
    { 0xf0ccead6, 90 },
    { 0x48fcead4, 53 },
    { 0x25dcead8, 23 },
    { 0xf8ecead9, 84 },
    { 0x76ecead9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ceadf, 80 },
    { 0x90dceae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2ceae4, 61 },
    { 0xbdcceae5, 61 },
    { 0x655ceae5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84eceaf3, 85 },
    { 0xbb2ceaf4, 46 },
    { 0xbccceaf5, 43 },
    { 0xbeceaf6, 66 },
    { 0xf00ceaf5, 23 },
    { 0x66ceaf5, 55 },
    { 0x567ceaf3, 93 },
    { 0x15bceafa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2eceaff, 23 },
    { 0xb0cceb00, 61 },
    { 0, 0 },
    { 0xb67ceb02, 90 },
    { 0xc93ceb02, 55 },
    { 0x136ceb03, 23 },
    { 0xad0ceb05, 61 },
    { 0x4e6ceb06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11ceb0a, 61 },
    { 0x1ecceb0b, 23 },
    { 0x94aceb0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758ceb10, 75 },
    { 0xe6bceb11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ceb15, 76 },
    { 0, 0 },
    { 0x628ceb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ceb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fceb1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eceb27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfceb2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ceb2e, 33 },
    { 0xfabceb2e, 33 },
    { 0x14bceb2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadceb3a, 61 },
    { 0x83fceb3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7ceb42, 95 },
    { 0x170ceb43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15ceb47, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ceb50, 23 },
    { 0x33dceb50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778ceb55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ceb58, 95 },
    { 0xee4ceb59, 90 },
    { 0x94cceb5a, 76 },
    { 0, 0 },
    { 0x605ceb5c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ceb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bceb76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ceb79, 34 },
    { 0, 0 },
    { 0x429ceb7b, 95 },
    { 0x4eeceb7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bceb80, 88 },
    { 0x1efceb81, 78 },
    { 0x2ebceb82, 33 },
    { 0x90eceb83, 23 },
    { 0x7e1ceb80, 23 },
    { 0xd3eceb85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cceb88, 65 },
    { 0, 0 },
    { 0x6b6ceb8a, 33 },
    { 0x908ceb8b, 98 },
    { 0xb40ceb8c, 90 },
    { 0x786ceb8b, 93 },
    { 0x397ceb8e, 33 },
    { 0, 0 },
    { 0xf1dceb90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ceb97, 75 },
    { 0x532ceb97, 33 },
    { 0x234ceb97, 95 },
    { 0, 0 },
    { 0x45fceb9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ceb9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652ceba9, 76 },
    { 0xe8dceba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfcebae, 34 },
    { 0x927cebaf, 23 },
    { 0xa20cebae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cebb3, 61 },
    { 0x4cdcebb4, 61 },
    { 0xc2ecebb4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63cebb9, 46 },
    { 0, 0 },
    { 0x52ecebbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8cebbe, 90 },
    { 0, 0 },
    { 0x56acebc0, 23 },
    { 0x7abcebc1, 61 },
    { 0x5bcebc2, 61 },
    { 0xe0ccebc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3cebc7, 93 },
    { 0xba5cebc8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfcebcb, 88 },
    { 0xb90cebcc, 76 },
    { 0x93bcebcc, 91 },
    { 0, 0 },
    { 0xc27cebcf, 34 },
    { 0xbbfcebd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfccebd6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dcebdb, 98 },
    { 0, 0 },
    { 0x726cebdd, 61 },
    { 0xd55cebde, 23 },
    { 0xa9fcebdd, 95 },
    { 0x3c1cebdd, 17 },
    { 0xaa5cebe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fcebe7, 43 },
    { 0, 0 },
    { 0xac4cebe9, 76 },
    { 0x6ecebea, 88 },
    { 0, 0 },
    { 0x702cebec, 61 },
    { 0x5dcebed, 90 },
    { 0x631cebee, 61 },
    { 0xbd1cebed, 83 },
    { 0xc74cebf0, 61 },
    { 0x775cebf1, 90 },
    { 0xe4dcebf1, 23 },
    { 0x256cebf1, 52 },
    { 0x88ccebf3, 55 },
    { 0xd0ecebf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdcebfd, 49 },
    { 0x84cebfd, 61 },
    { 0x66dcebfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3becec02, 46 },
    { 0, 0 },
    { 0x95acec04, 33 },
    { 0x92ecec05, 77 },
    { 0x3b5cec06, 64 },
    { 0x93ecec07, 23 },
    { 0xb9acec05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcec0c, 98 },
    { 0x906cec0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ecec12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42cec1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x988cec1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9cec22, 99 },
    { 0, 0 },
    { 0xcabcec24, 95 },
    { 0x548cec25, 98 },
    { 0, 0 },
    { 0x774cec27, 34 },
    { 0x9e5cec27, 33 },
    { 0, 0 },
    { 0xb0bcec2a, 55 },
    { 0, 0 },
    { 0xa01cec2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36cec2f, 96 },
    { 0, 0 },
    { 0x14fcec31, 61 },
    { 0, 0 },
    { 0x684cec33, 66 },
    { 0x186cec33, 34 },
    { 0, 0 },
    { 0x7c6cec36, 99 },
    { 0x7d8cec37, 98 },
    { 0x8b7cec37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323cec3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40cec42, 61 },
    { 0xae4cec43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x788cec46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5cec4e, 64 },
    { 0xc15cec4f, 33 },
    { 0, 0 },
    { 0x925cec51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x877cec54, 88 },
    { 0x516cec55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213cec59, 99 },
    { 0x162cec5a, 87 },
    { 0x2f3cec59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29cec5f, 96 },
    { 0x1dcec60, 68 },
    { 0, 0 },
    { 0xae9cec62, 33 },
    { 0xdbecec63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dcec67, 61 },
    { 0x3a7cec68, 90 },
    { 0, 0 },
    { 0xab4cec6a, 66 },
    { 0xe4ecec6a, 33 },
    { 0x60acec6a, 67 },
    { 0xa64cec6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcec70, 20 },
    { 0x5f2cec70, 90 },
    { 0x290cec71, 90 },
    { 0x8f9cec73, 23 },
    { 0xa0cec73, 76 },
    { 0x4bacec74, 95 },
    { 0x6a1cec72, 87 },
    { 0, 0 },
    { 0x98ecec78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03cec7d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cacec80, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddcec86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4cec89, 90 },
    { 0x4facec89, 33 },
    { 0xaacec8a, 98 },
    { 0x51ccec8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6cec90, 53 },
    { 0xf74cec90, 23 },
    { 0x1c4cec92, 33 },
    { 0x341cec93, 91 },
    { 0x5cecec94, 33 },
    { 0xed3cec95, 61 },
    { 0x7bccec96, 40 },
    { 0x79cec96, 33 },
    { 0, 0 },
    { 0xadccec99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ccec9c, 33 },
    { 0, 0 },
    { 0x417cec9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fceca1, 23 },
    { 0x44dceca1, 75 },
    { 0x779ceca3, 67 },
    { 0x8f5ceca3, 61 },
    { 0x172ceca2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5ceca9, 90 },
    { 0x3e9ceca9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ececad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dcecb2, 98 },
    { 0, 0 },
    { 0x25dcecb4, 67 },
    { 0xd72cecb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dcecb8, 33 },
    { 0x38acecb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x997cecbc, 90 },
    { 0, 0 },
    { 0xb14cecbe, 81 },
    { 0xa57cecbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x108cecc2, 61 },
    { 0x2edcecc2, 98 },
    { 0xf85cecc3, 66 },
    { 0x1cececc5, 90 },
    { 0xd86cecc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cecca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9cecd1, 33 },
    { 0xbbacecd2, 95 },
    { 0xd1acecd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62cecd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26cecd9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6cecdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bcece4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5cece8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aceceb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bcecee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bcecf1, 61 },
    { 0xe4acecf2, 99 },
    { 0xebacecf2, 91 },
    { 0, 0 },
    { 0x85bcecf5, 91 },
    { 0x95ccecf5, 95 },
    { 0xc45cecf5, 61 },
    { 0x708cecf8, 99 },
    { 0x25cecf5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bced02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ced0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bced15, 61 },
    { 0x8bbced16, 76 },
    { 0x1b5ced17, 90 },
    { 0xc09ced18, 61 },
    { 0xcd7ced19, 66 },
    { 0xf32ced1a, 61 },
    { 0, 0 },
    { 0x725ced1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691ced24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ced2a, 23 },
    { 0, 0 },
    { 0xd38ced2c, 33 },
    { 0x258ced2d, 92 },
    { 0xda2ced2d, 36 },
    { 0x6adced2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3ced32, 55 },
    { 0xd1ced32, 78 },
    { 0x9a7ced33, 33 },
    { 0xb33ced34, 98 },
    { 0, 0 },
    { 0xa34ced37, 53 },
    { 0xa5aced37, 43 },
    { 0x9c4ced38, 84 },
    { 0, 0 },
    { 0xef6ced3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aced44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6eced47, 61 },
    { 0xdbced48, 95 },
    { 0, 0 },
    { 0xc5bced4a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ced4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cced51, 23 },
    { 0xff2ced52, 95 },
    { 0x871ced52, 98 },
    { 0x7c2ced52, 95 },
    { 0x664ced55, 98 },
    { 0xd49ced55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ced5b, 87 },
    { 0xe8aced5c, 95 },
    { 0xdf0ced5b, 95 },
    { 0, 0 },
    { 0x396ced5f, 33 },
    { 0xd1aced60, 23 },
    { 0, 0 },
    { 0x6cbced62, 33 },
    { 0, 0 },
    { 0xef2ced64, 90 },
    { 0, 0 },
    { 0x450ced66, 43 },
    { 0xf1fced67, 66 },
    { 0x6f0ced68, 61 },
    { 0x53ced68, 90 },
    { 0x9f0ced6a, 87 },
    { 0xcd9ced69, 90 },
    { 0x22fced6c, 91 },
    { 0x349ced6d, 93 },
    { 0xb30ced6e, 67 },
    { 0, 0 },
    { 0xb60ced70, 52 },
    { 0x13ced70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fced75, 35 },
    { 0xfc3ced76, 66 },
    { 0xbfcced77, 23 },
    { 0xc1ced78, 33 },
    { 0xa19ced78, 93 },
    { 0, 0 },
    { 0x2abced7b, 43 },
    { 0, 0 },
    { 0x482ced7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ced80, 98 },
    { 0x3fced81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ced84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeced8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89ced91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bced94, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ced97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdced9d, 23 },
    { 0xfd0ced9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bceda1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bcedac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06cedb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8cedb6, 23 },
    { 0, 0 },
    { 0xb00cedb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccedbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cedc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebcedc6, 98 },
    { 0xd8cedc6, 84 },
    { 0xe6bcedc7, 99 },
    { 0, 0 },
    { 0xf4ccedca, 33 },
    { 0x81cedca, 33 },
    { 0xbd9cedcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcedcf, 46 },
    { 0x9adcedcf, 55 },
    { 0, 0 },
    { 0x2eccedd2, 89 },
    { 0x768cedd3, 95 },
    { 0x3e3cedd4, 33 },
    { 0xbc4cedd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caceddb, 98 },
    { 0x940ceddc, 34 },
    { 0x966ceddd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353cede1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ecede6, 23 },
    { 0, 0 },
    { 0xb0ccede8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5acedec, 33 },
    { 0, 0 },
    { 0x670cedee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6cedf9, 61 },
    { 0x250cedfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53cedfd, 33 },
    { 0, 0 },
    { 0x8f6cedff, 87 },
    { 0x7fbcee00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcee07, 93 },
    { 0x325cee08, 98 },
    { 0x142cee09, 61 },
    { 0x61ccee0a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3cee0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefcee10, 33 },
    { 0xa20cee11, 93 },
    { 0x674cee12, 43 },
    { 0x2eacee13, 90 },
    { 0, 0 },
    { 0xe88cee15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cecee18, 33 },
    { 0x90ccee19, 98 },
    { 0x6ffcee1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2cee24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89cee29, 43 },
    { 0x50fcee29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x175cee2d, 95 },
    { 0, 0 },
    { 0xabcee2f, 93 },
    { 0x309cee30, 76 },
    { 0xef4cee2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1cee36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155cee3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5cee3d, 34 },
    { 0x85bcee3e, 90 },
    { 0xe7ecee3f, 90 },
    { 0x1e4cee3f, 23 },
    { 0xdbfcee3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3cee47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6cee4a, 95 },
    { 0x845cee4b, 61 },
    { 0xdd9cee4c, 55 },
    { 0xe8bcee4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557cee52, 98 },
    { 0xb2ccee52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0cee57, 34 },
    { 0xc3cee57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ecee62, 34 },
    { 0, 0 },
    { 0x2f4cee64, 90 },
    { 0, 0 },
    { 0xc03cee66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcee69, 99 },
    { 0, 0 },
    { 0x2fbcee6b, 95 },
    { 0xa13cee6c, 90 },
    { 0x1e5cee6d, 76 },
    { 0xac9cee6e, 90 },
    { 0x5fdcee6f, 100 },
    { 0xa4dcee70, 23 },
    { 0xf97cee71, 95 },
    { 0x3b1cee70, 91 },
    { 0x407cee73, 87 },
    { 0xaeecee6d, 61 },
    { 0x4cee75, 52 },
    { 0xcd4cee76, 68 },
    { 0x3e4cee72, 90 },
    { 0x31bcee78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9cee7b, 84 },
    { 0xab0cee7b, 98 },
    { 0x7c2cee7d, 65 },
    { 0x33dcee7e, 33 },
    { 0xc29cee7f, 46 },
    { 0, 0 },
    { 0xeb2cee81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376cee85, 61 },
    { 0xa4fcee86, 90 },
    { 0xcc8cee85, 23 },
    { 0, 0 },
    { 0x20ecee89, 90 },
    { 0, 0 },
    { 0x6efcee8b, 43 },
    { 0xc3fcee8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99cee8f, 33 },
    { 0, 0 },
    { 0x438cee91, 23 },
    { 0, 0 },
    { 0x802cee93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cee96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fecee99, 66 },
    { 0x4a0cee9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fceea4, 61 },
    { 0x2b5ceea5, 90 },
    { 0x2d7ceea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cceeaa, 61 },
    { 0xb7eceeab, 61 },
    { 0x659ceeab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640ceeb2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ceeb7, 34 },
    { 0, 0 },
    { 0x2b1ceeb9, 95 },
    { 0x69ceeb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aceec7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bceecb, 90 },
    { 0xf2dceecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65ceed2, 52 },
    { 0xcefceed2, 75 },
    { 0x68eceed4, 84 },
    { 0x488ceed5, 55 },
    { 0x5b4ceed5, 33 },
    { 0x48cceed7, 46 },
    { 0x582ceed8, 90 },
    { 0x17aceed9, 91 },
    { 0x2dbceed7, 95 },
    { 0x530ceedb, 68 },
    { 0xc8bceedc, 23 },
    { 0x77bceedd, 34 },
    { 0xa4cceedb, 61 },
    { 0x8ddceed9, 90 },
    { 0, 0 },
    { 0x964ceee1, 76 },
    { 0xfc2ceee2, 98 },
    { 0, 0 },
    { 0xaf0ceee4, 95 },
    { 0x9cceee5, 66 },
    { 0, 0 },
    { 0xf57ceee7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ceeea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fceeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61ceef4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cceefa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9ceefe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0cef01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999cef05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aacef0a, 99 },
    { 0x499cef0b, 34 },
    { 0x5ebcef0a, 67 },
    { 0, 0 },
    { 0x24bcef0e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcef11, 84 },
    { 0x43fcef12, 23 },
    { 0x193cef11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcef16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159cef1a, 90 },
    { 0, 0 },
    { 0x74cef1c, 98 },
    { 0x6a7cef1d, 90 },
    { 0x808cef1d, 93 },
    { 0xf7acef1d, 69 },
    { 0xc0cef1d, 61 },
    { 0xe17cef21, 23 },
    { 0, 0 },
    { 0x17bcef23, 100 },
    { 0x920cef24, 90 },
    { 0, 0 },
    { 0xbe2cef26, 23 },
    { 0x6eacef27, 43 },
    { 0x50ccef27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8cef2f, 33 },
    { 0xb8ccef2f, 33 },
    { 0xef1cef31, 33 },
    { 0, 0 },
    { 0xb9cef33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40cef3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116cef3e, 33 },
    { 0xa82cef3f, 66 },
    { 0x5ffcef3e, 55 },
    { 0x9b2cef41, 90 },
    { 0xea7cef42, 61 },
    { 0xa82cef43, 90 },
    { 0x627cef44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3cef47, 95 },
    { 0, 0 },
    { 0xc2dcef49, 76 },
    { 0, 0 },
    { 0x848cef4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492cef4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85cef56, 95 },
    { 0x131cef57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754cef5b, 95 },
    { 0x1cef5b, 95 },
    { 0x856cef5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eecef61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5cef64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cef68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefcef6c, 52 },
    { 0, 0 },
    { 0x955cef6e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x281cef71, 33 },
    { 0, 0 },
    { 0x852cef73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfcef7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x546cef7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbcef83, 43 },
    { 0x1e8cef83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cef87, 61 },
    { 0, 0 },
    { 0xa40cef89, 77 },
    { 0x5fcef8a, 90 },
    { 0, 0 },
    { 0xc9bcef8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x545cef8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cef92, 84 },
    { 0, 0 },
    { 0x3c9cef94, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108cef98, 23 },
    { 0xf97cef99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26cef9e, 23 },
    { 0, 0 },
    { 0x178cefa0, 90 },
    { 0, 0 },
    { 0xd9cefa2, 90 },
    { 0x203cefa3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cefa7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cefb0, 34 },
    { 0x2eecefb0, 23 },
    { 0x649cefb1, 17 },
    { 0x109cefb3, 78 },
    { 0x6bacefb4, 84 },
    { 0x319cefb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbcefb8, 90 },
    { 0, 0 },
    { 0x761cefba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04cefbd, 84 },
    { 0x54bcefbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cefc3, 98 },
    { 0x5c9cefc4, 98 },
    { 0xfa4cefc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bccefc8, 38 },
    { 0x705cefc9, 33 },
    { 0, 0 },
    { 0x189cefcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246cefd6, 43 },
    { 0xb9cefd7, 29 },
    { 0x5f1cefd8, 90 },
    { 0x768cefd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65cefdd, 78 },
    { 0xd29cefde, 90 },
    { 0xb3fcefde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766cefe3, 91 },
    { 0, 0 },
    { 0x4e7cefe5, 37 },
    { 0x114cefe6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cefeb, 93 },
    { 0x48ecefec, 98 },
    { 0x9dccefed, 61 },
    { 0x61acefee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ceff2, 33 },
    { 0x100ceff3, 88 },
    { 0xfc3ceff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cceff7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ceffa, 23 },
    { 0xe95ceffb, 76 },
    { 0, 0 },
    { 0xc27ceffd, 76 },
    { 0x15cceffe, 95 },
    { 0x4abcefff, 34 },
    { 0xe5acefff, 93 },
    { 0xe74cefff, 90 },
    { 0x94dcf002, 87 },
    { 0x1aacf003, 33 },
    { 0, 0 },
    { 0x43dcf005, 95 },
    { 0x19bcf006, 43 },
    { 0x61cf006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270cf00b, 46 },
    { 0x2b3cf00c, 61 },
    { 0x4b4cf00d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92cf018, 95 },
    { 0xb6cf018, 61 },
    { 0, 0 },
    { 0xf5bcf01b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33acf01e, 90 },
    { 0xa13cf01f, 52 },
    { 0x57dcf020, 34 },
    { 0xbf9cf021, 84 },
    { 0xf01cf021, 76 },
    { 0x8d9cf023, 90 },
    { 0xa17cf024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ecf029, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9cf031, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ecf035, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdcf038, 101 },
    { 0, 0 },
    { 0xe53cf03a, 76 },
    { 0, 0 },
    { 0xae8cf03c, 61 },
    { 0xa53cf03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2cf044, 69 },
    { 0x68acf045, 61 },
    { 0, 0 },
    { 0x564cf047, 43 },
    { 0xb2cf048, 94 },
    { 0, 0 },
    { 0x5c2cf04a, 19 },
    { 0, 0 },
    { 0xfdcf04c, 93 },
    { 0x690cf04d, 33 },
    { 0, 0 },
    { 0xa33cf04f, 36 },
    { 0, 0 },
    { 0x61fcf051, 95 },
    { 0, 0 },
    { 0xb08cf053, 23 },
    { 0x94fcf054, 23 },
    { 0x78fcf055, 100 },
    { 0xb50cf056, 23 },
    { 0, 0 },
    { 0xa43cf058, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2cf062, 55 },
    { 0, 0 },
    { 0xd03cf064, 55 },
    { 0x34acf065, 98 },
    { 0, 0 },
    { 0x201cf067, 98 },
    { 0x405cf068, 90 },
    { 0xe39cf067, 91 },
    { 0xde4cf06a, 95 },
    { 0xa8acf06a, 90 },
    { 0, 0 },
    { 0xad0cf06d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdcf072, 23 },
    { 0, 0 },
    { 0xfcccf074, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dcf077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dcf07c, 98 },
    { 0xed6cf07d, 33 },
    { 0, 0 },
    { 0x2cbcf07f, 64 },
    { 0x96acf080, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcf084, 95 },
    { 0x69ccf085, 23 },
    { 0x92bcf085, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54cf08b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9cf094, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eccf09d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758cf0a3, 52 },
    { 0x3f4cf0a4, 100 },
    { 0x4d1cf0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791cf0ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881cf0b2, 83 },
    { 0xf85cf0b2, 98 },
    { 0x889cf0b4, 90 },
    { 0xd70cf0b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x874cf0b8, 23 },
    { 0xf23cf0b9, 95 },
    { 0xf79cf0b9, 100 },
    { 0xff9cf0bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bacf0c7, 92 },
    { 0x45ccf0c8, 98 },
    { 0x2b5cf0c8, 98 },
    { 0, 0 },
    { 0x979cf0cb, 98 },
    { 0, 0 },
    { 0x1d7cf0cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105cf0da, 82 },
    { 0x309cf0db, 23 },
    { 0xbeacf0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787cf0e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06cf0ec, 76 },
    { 0xa4cf0ed, 33 },
    { 0xfc2cf0ee, 53 },
    { 0xae4cf0ef, 55 },
    { 0, 0 },
    { 0x724cf0f1, 87 },
    { 0x1f3cf0f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71cf0f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8cf0fb, 91 },
    { 0x86bcf0fb, 59 },
    { 0x344cf0fd, 98 },
    { 0x3f0cf0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8cf102, 46 },
    { 0, 0 },
    { 0xf84cf104, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bacf110, 96 },
    { 0, 0 },
    { 0x9c5cf112, 91 },
    { 0, 0 },
    { 0xc54cf114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dcf11a, 48 },
    { 0, 0 },
    { 0x562cf11c, 23 },
    { 0xbaacf11d, 52 },
    { 0x679cf11e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575cf123, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cf128, 95 },
    { 0xc7acf129, 78 },
    { 0x3cdcf12a, 23 },
    { 0x234cf12a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07cf132, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ecf136, 56 },
    { 0, 0 },
    { 0x8a3cf138, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4cf13c, 46 },
    { 0x54dcf13c, 95 },
    { 0x77fcf13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06cf143, 34 },
    { 0x98ccf144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373cf14f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecf154, 55 },
    { 0xf7acf155, 77 },
    { 0, 0 },
    { 0x2c2cf157, 76 },
    { 0x7b4cf158, 66 },
    { 0xb79cf159, 61 },
    { 0xab1cf158, 99 },
    { 0x566cf157, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcf15f, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0cf162, 98 },
    { 0x5e9cf162, 46 },
    { 0, 0 },
    { 0xce1cf165, 90 },
    { 0, 0 },
    { 0xf99cf167, 95 },
    { 0x9fecf168, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54cf171, 93 },
    { 0, 0 },
    { 0xcc2cf173, 61 },
    { 0xadbcf173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcf178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ccf17c, 33 },
    { 0x453cf17c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecf182, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1cf186, 98 },
    { 0xd93cf187, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cf18a, 33 },
    { 0, 0 },
    { 0x8f0cf18c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9cf192, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfacf196, 76 },
    { 0xb4bcf197, 75 },
    { 0x9b4cf197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790cf19d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbcf1a0, 23 },
    { 0x337cf1a1, 95 },
    { 0, 0 },
    { 0x33fcf1a3, 95 },
    { 0x966cf1a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbcf1a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03cf1ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585cf1b1, 67 },
    { 0x2d8cf1b2, 33 },
    { 0x841cf1b3, 33 },
    { 0xc42cf1b4, 90 },
    { 0x121cf1b5, 61 },
    { 0xb09cf1b5, 66 },
    { 0xa46cf1b7, 46 },
    { 0, 0 },
    { 0xe20cf1b9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ecf1be, 90 },
    { 0xc54cf1be, 90 },
    { 0xc2ecf1c0, 64 },
    { 0, 0 },
    { 0xdbcf1c2, 90 },
    { 0xdc8cf1c2, 33 },
    { 0xf27cf1c4, 61 },
    { 0x6d0cf1c5, 34 },
    { 0xb8ccf1c4, 90 },
    { 0xf5ecf1c6, 93 },
    { 0xc93cf1c4, 76 },
    { 0x8d9cf1c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96cf1ce, 23 },
    { 0x53acf1ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fcf1d2, 61 },
    { 0x57ccf1d3, 95 },
    { 0, 0 },
    { 0x158cf1d5, 78 },
    { 0, 0 },
    { 0xb9cf1d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90cf1dc, 76 },
    { 0xb09cf1dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeacf1e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8cf1e4, 23 },
    { 0x983cf1e5, 17 },
    { 0xa68cf1e4, 90 },
    { 0x6c7cf1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54cf1ef, 61 },
    { 0, 0 },
    { 0x5efcf1f1, 34 },
    { 0x7b5cf1f1, 52 },
    { 0, 0 },
    { 0x5efcf1f4, 95 },
    { 0xed1cf1f5, 97 },
    { 0, 0 },
    { 0xb9bcf1f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dcf1fc, 61 },
    { 0xd96cf1fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cf200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x570cf203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9cf208, 76 },
    { 0x30acf209, 61 },
    { 0xdd6cf20a, 93 },
    { 0x46bcf20b, 33 },
    { 0x55dcf209, 95 },
    { 0xb55cf20a, 76 },
    { 0xb0ecf20e, 61 },
    { 0xe0fcf20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dcf212, 33 },
    { 0x63fcf213, 95 },
    { 0xf57cf213, 74 },
    { 0x951cf214, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fcf219, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecccf21c, 92 },
    { 0x831cf21d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x878cf220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628cf227, 98 },
    { 0x12acf228, 67 },
    { 0x617cf229, 76 },
    { 0, 0 },
    { 0x598cf22b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x446cf22e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7cf232, 80 },
    { 0xd6fcf232, 55 },
    { 0xb62cf234, 33 },
    { 0x403cf235, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dcf238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5cf23b, 95 },
    { 0x339cf23c, 66 },
    { 0, 0 },
    { 0x31ccf23e, 95 },
    { 0x4becf23f, 33 },
    { 0xeffcf23e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16cf247, 77 },
    { 0, 0 },
    { 0xffdcf249, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fcf24c, 92 },
    { 0x135cf24c, 90 },
    { 0x490cf24e, 33 },
    { 0xb11cf24f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cf256, 95 },
    { 0xa5acf256, 33 },
    { 0, 0 },
    { 0xa7dcf259, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf265, 33 },
    { 0x5c2cf266, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20cf26c, 34 },
    { 0xb71cf26d, 43 },
    { 0xeccf26e, 100 },
    { 0xd72cf26c, 78 },
    { 0xb6ccf270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291cf277, 57 },
    { 0xcb5cf277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0cf27e, 61 },
    { 0x5c3cf27f, 33 },
    { 0x7cdcf280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58cf288, 77 },
    { 0x7e3cf289, 76 },
    { 0x8fccf288, 61 },
    { 0xc6bcf28b, 78 },
    { 0x84ccf28c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acf28f, 61 },
    { 0x70ccf290, 33 },
    { 0, 0 },
    { 0xe55cf292, 76 },
    { 0x581cf292, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8cf299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf29d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x242cf2a0, 87 },
    { 0x4b5cf2a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bcf2a6, 38 },
    { 0x926cf2a6, 23 },
    { 0x551cf2a8, 33 },
    { 0, 0 },
    { 0x3c2cf2aa, 85 },
    { 0x8c0cf2ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8cf2ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86cf2b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3cf2be, 33 },
    { 0x7b7cf2be, 90 },
    { 0xeb5cf2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bcf2c3, 64 },
    { 0x75ccf2c3, 33 },
    { 0x5cdcf2c3, 80 },
    { 0, 0 },
    { 0x4d3cf2c7, 82 },
    { 0x31dcf2c8, 90 },
    { 0xfeacf2c9, 61 },
    { 0x6bacf2c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8acf2d4, 34 },
    { 0, 0 },
    { 0xdd1cf2d6, 91 },
    { 0x698cf2d6, 96 },
    { 0x898cf2d7, 95 },
    { 0x4dacf2d9, 95 },
    { 0x746cf2da, 55 },
    { 0xe6cf2d9, 87 },
    { 0x6e6cf2dc, 77 },
    { 0xa8fcf2dc, 23 },
    { 0, 0 },
    { 0x82acf2df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6cf2e6, 23 },
    { 0x11acf2e7, 55 },
    { 0x103cf2e8, 66 },
    { 0, 0 },
    { 0x24cf2ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8cf2f1, 33 },
    { 0x69ccf2f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffcf2f5, 40 },
    { 0x283cf2f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09cf2fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ecf300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2cf30a, 66 },
    { 0xc0acf30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60cf30f, 90 },
    { 0xf70cf310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf318, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x760cf31b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91cf31e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582cf328, 40 },
    { 0x8e0cf328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ecf32d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fcf337, 61 },
    { 0x3c7cf338, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72cf33d, 43 },
    { 0x560cf33e, 88 },
    { 0x60bcf33e, 95 },
    { 0, 0 },
    { 0x773cf341, 43 },
    { 0x9d6cf342, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abcf346, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffcf34e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cf353, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cf357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5cf35b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fcf360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0cf366, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40cf36b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9cf374, 61 },
    { 0, 0 },
    { 0xa75cf376, 61 },
    { 0xefbcf377, 98 },
    { 0x9f5cf378, 33 },
    { 0x68ccf377, 33 },
    { 0, 0 },
    { 0x208cf37b, 98 },
    { 0xafdcf37c, 43 },
    { 0xe21cf37d, 23 },
    { 0xe13cf37e, 90 },
    { 0x387cf37c, 23 },
    { 0x599cf380, 90 },
    { 0xdc6cf381, 90 },
    { 0x9c6cf382, 90 },
    { 0x41ccf37c, 55 },
    { 0x521cf384, 76 },
    { 0x99acf382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bccf390, 95 },
    { 0, 0 },
    { 0x531cf392, 65 },
    { 0xc5ecf393, 33 },
    { 0, 0 },
    { 0x182cf395, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6cf39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351cf39e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ccf3a1, 33 },
    { 0x4b2cf3a2, 77 },
    { 0, 0 },
    { 0xefacf3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1cf3ac, 33 },
    { 0, 0 },
    { 0x436cf3ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367cf3b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dcf3bf, 43 },
    { 0, 0 },
    { 0x621cf3c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bcf3c7, 55 },
    { 0x564cf3c8, 90 },
    { 0x74acf3c7, 95 },
    { 0x718cf3ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cf3d0, 90 },
    { 0, 0 },
    { 0xa6fcf3d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188cf3da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746cf3de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ccf3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9dcf3ea, 90 },
    { 0x823cf3eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7cf3ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16cf3f4, 99 },
    { 0, 0 },
    { 0xe72cf3f6, 90 },
    { 0x663cf3f7, 81 },
    { 0, 0 },
    { 0x560cf3f9, 90 },
    { 0, 0 },
    { 0xe70cf3fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32cf3ff, 23 },
    { 0xff2cf3ff, 95 },
    { 0, 0 },
    { 0xaf8cf402, 33 },
    { 0, 0 },
    { 0x59ecf404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840cf408, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3cf40c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302cf411, 95 },
    { 0, 0 },
    { 0xf7ecf413, 34 },
    { 0xdcccf414, 90 },
    { 0xe2cf415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9cf419, 61 },
    { 0xc69cf41a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x106cf41d, 79 },
    { 0, 0 },
    { 0xac0cf41f, 98 },
    { 0x446cf41f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533cf426, 96 },
    { 0xfcecf427, 47 },
    { 0xe86cf428, 90 },
    { 0, 0 },
    { 0xfb5cf42a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fcf434, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bcf438, 34 },
    { 0x631cf439, 90 },
    { 0x276cf43a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ecf443, 43 },
    { 0xc00cf444, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0cf449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7cf44d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeecf454, 80 },
    { 0xee0cf455, 23 },
    { 0xf4ecf456, 35 },
    { 0xed5cf456, 98 },
    { 0xfbdcf458, 33 },
    { 0xbd4cf456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3cf45d, 84 },
    { 0x4fccf45e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63cf463, 95 },
    { 0, 0 },
    { 0xdcfcf465, 87 },
    { 0xa73cf465, 95 },
    { 0x2cccf467, 33 },
    { 0, 0 },
    { 0xe1cf469, 84 },
    { 0x545cf46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cf46d, 61 },
    { 0xa21cf46d, 90 },
    { 0x411cf46f, 66 },
    { 0xbb0cf46f, 61 },
    { 0, 0 },
    { 0xadfcf472, 69 },
    { 0xf9fcf472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401cf478, 88 },
    { 0x66cf478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3cf47f, 95 },
    { 0x3afcf480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4cf484, 74 },
    { 0, 0 },
    { 0x54acf486, 43 },
    { 0xbdccf486, 43 },
    { 0, 0 },
    { 0xc07cf489, 75 },
    { 0xa18cf48a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9cf48f, 90 },
    { 0xc7ecf490, 76 },
    { 0xc3fcf491, 69 },
    { 0, 0 },
    { 0xfc7cf493, 98 },
    { 0x2acf494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2cf497, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992cf49f, 98 },
    { 0, 0 },
    { 0xf94cf4a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbacf4a5, 93 },
    { 0, 0 },
    { 0x6a7cf4a7, 98 },
    { 0x6b9cf4a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeccf4ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cf4b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcecf4b4, 66 },
    { 0, 0 },
    { 0x739cf4b6, 95 },
    { 0, 0 },
    { 0xf64cf4b8, 33 },
    { 0xdb0cf4b8, 65 },
    { 0xfacf4b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cf4bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339cf4c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7cf4ca, 33 },
    { 0, 0 },
    { 0xd20cf4cc, 68 },
    { 0x568cf4cd, 33 },
    { 0x49bcf4ce, 95 },
    { 0xc1ecf4cf, 76 },
    { 0x6f5cf4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303cf4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3cf4d7, 90 },
    { 0x369cf4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcf4dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ccf4e4, 98 },
    { 0, 0 },
    { 0x60dcf4e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98cf4ed, 80 },
    { 0, 0 },
    { 0xbd7cf4ef, 52 },
    { 0xdd4cf4f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8cf4f7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4cf4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3acf502, 83 },
    { 0xccf503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bccf50a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aecf50e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3cf511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdccf516, 65 },
    { 0xaf2cf516, 98 },
    { 0, 0 },
    { 0xf0acf519, 95 },
    { 0x763cf519, 78 },
    { 0, 0 },
    { 0x6d3cf51c, 90 },
    { 0, 0 },
    { 0x8bccf51e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ccf52e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76cf531, 61 },
    { 0, 0 },
    { 0x2cbcf533, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40cf536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904cf53f, 99 },
    { 0x3c5cf540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6cf547, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5cf54a, 99 },
    { 0, 0 },
    { 0x385cf54c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fcf550, 66 },
    { 0, 0 },
    { 0xa1dcf552, 52 },
    { 0xd46cf553, 61 },
    { 0x542cf554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fcf558, 91 },
    { 0xe72cf558, 95 },
    { 0x579cf559, 84 },
    { 0xadcf55a, 61 },
    { 0xdb7cf559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0cf55f, 88 },
    { 0xc3acf55f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cf567, 98 },
    { 0xbc4cf567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940cf56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5accf56f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989cf57a, 33 },
    { 0xe3dcf57b, 95 },
    { 0, 0 },
    { 0x75fcf57d, 68 },
    { 0, 0 },
    { 0x489cf57f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ccf582, 83 },
    { 0, 0 },
    { 0x485cf584, 33 },
    { 0xff4cf584, 84 },
    { 0, 0 },
    { 0x11bcf587, 52 },
    { 0xd9bcf588, 66 },
    { 0, 0 },
    { 0xeeecf58a, 17 },
    { 0, 0 },
    { 0xa93cf58c, 67 },
    { 0, 0 },
    { 0x4cacf58e, 90 },
    { 0x65acf58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cf595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cf59a, 33 },
    { 0x8b3cf59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ccf59f, 61 },
    { 0xc79cf5a0, 95 },
    { 0xc11cf59f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126cf5a7, 66 },
    { 0x68cf5a8, 87 },
    { 0xf94cf5a9, 23 },
    { 0x81ecf5a9, 95 },
    { 0x181cf5ab, 46 },
    { 0x87cf5ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cf5b0, 23 },
    { 0x6bccf5b1, 85 },
    { 0, 0 },
    { 0x6f4cf5b3, 23 },
    { 0x185cf5b3, 95 },
    { 0, 0 },
    { 0xd05cf5b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0cf5bb, 99 },
    { 0x539cf5bb, 43 },
    { 0, 0 },
    { 0x7a6cf5be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7cf5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cccf5c5, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dccf5cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4cf5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44cf5d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0cf5db, 52 },
    { 0x85acf5dc, 43 },
    { 0x436cf5dc, 61 },
    { 0x85cf5dd, 34 },
    { 0x42fcf5df, 95 },
    { 0x48bcf5df, 33 },
    { 0xea6cf5e1, 99 },
    { 0x544cf5e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1cf5e5, 98 },
    { 0x474cf5e5, 33 },
    { 0x54bcf5e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750cf5ea, 76 },
    { 0xe70cf5eb, 93 },
    { 0x9a5cf5eb, 90 },
    { 0, 0 },
    { 0xbc1cf5ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484cf5f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cf5f7, 61 },
    { 0xea1cf5f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9cf5fb, 33 },
    { 0xdc6cf5fc, 95 },
    { 0xe65cf5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ccf600, 76 },
    { 0, 0 },
    { 0x1c8cf602, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x127cf605, 23 },
    { 0x904cf606, 98 },
    { 0x68dcf607, 76 },
    { 0xe06cf607, 61 },
    { 0, 0 },
    { 0xbfacf60a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306cf612, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77cf617, 33 },
    { 0xbbecf618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0cf61c, 78 },
    { 0, 0 },
    { 0x800cf61e, 98 },
    { 0x6d3cf61f, 90 },
    { 0, 0 },
    { 0x255cf621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ecf626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440cf62d, 75 },
    { 0, 0 },
    { 0xc34cf62f, 90 },
    { 0xb65cf62f, 34 },
    { 0xeaacf631, 67 },
    { 0x7f2cf632, 90 },
    { 0xaefcf633, 55 },
    { 0x3a3cf62f, 90 },
    { 0xf94cf630, 23 },
    { 0, 0 },
    { 0xae4cf637, 52 },
    { 0x85ccf638, 68 },
    { 0x5a1cf639, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4acf642, 61 },
    { 0x74ecf642, 97 },
    { 0x5fecf644, 23 },
    { 0x4a1cf644, 98 },
    { 0xbc3cf646, 61 },
    { 0, 0 },
    { 0xe25cf648, 99 },
    { 0xe00cf648, 23 },
    { 0x423cf64a, 66 },
    { 0xc9acf64b, 23 },
    { 0x21fcf64a, 34 },
    { 0xb5fcf64a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bcf653, 34 },
    { 0xa45cf654, 33 },
    { 0xf10cf654, 90 },
    { 0xc4acf654, 84 },
    { 0xaebcf657, 90 },
    { 0xaaacf658, 90 },
    { 0, 0 },
    { 0xcffcf65a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12acf65f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84cf668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339cf670, 76 },
    { 0, 0 },
    { 0xcddcf672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bcf679, 68 },
    { 0, 0 },
    { 0x3a4cf67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf67e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ecf681, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cf688, 95 },
    { 0x297cf689, 87 },
    { 0x686cf68a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ecf68d, 95 },
    { 0x2eacf68e, 33 },
    { 0xf06cf68e, 23 },
    { 0x680cf690, 43 },
    { 0xb66cf68e, 90 },
    { 0xbd4cf692, 85 },
    { 0x7efcf692, 36 },
    { 0, 0 },
    { 0xabdcf695, 93 },
    { 0xd2fcf695, 23 },
    { 0xd30cf697, 17 },
    { 0x597cf698, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fcf69c, 68 },
    { 0x2a2cf69c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdcf6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ecf6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccf6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bcf6b7, 91 },
    { 0, 0 },
    { 0x894cf6b9, 95 },
    { 0, 0 },
    { 0xdb5cf6bb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596cf6c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bcf6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16cf6da, 99 },
    { 0x289cf6db, 98 },
    { 0, 0 },
    { 0xf9cf6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cf6e1, 90 },
    { 0x970cf6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905cf6e6, 33 },
    { 0x5fbcf6e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422cf6ec, 68 },
    { 0, 0 },
    { 0x490cf6ee, 67 },
    { 0x409cf6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9cf6f4, 90 },
    { 0x28acf6f5, 61 },
    { 0x9fdcf6f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73cf6f9, 55 },
    { 0x8d8cf6fa, 34 },
    { 0x47fcf6fa, 95 },
    { 0, 0 },
    { 0xd0acf6fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48cf701, 93 },
    { 0, 0 },
    { 0x691cf703, 67 },
    { 0xd06cf703, 52 },
    { 0x1eecf704, 95 },
    { 0xab6cf705, 87 },
    { 0, 0 },
    { 0x789cf708, 61 },
    { 0x69fcf708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bcf70d, 61 },
    { 0x6cacf70d, 34 },
    { 0, 0 },
    { 0x894cf710, 98 },
    { 0x41cf710, 65 },
    { 0, 0 },
    { 0xab3cf713, 95 },
    { 0, 0 },
    { 0x5becf715, 23 },
    { 0x88ecf716, 46 },
    { 0x88dcf716, 81 },
    { 0x61ccf717, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadcf71b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363cf71f, 95 },
    { 0x6b0cf720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541cf724, 90 },
    { 0xef5cf724, 46 },
    { 0xbdbcf725, 95 },
    { 0, 0 },
    { 0x208cf728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x332cf72b, 33 },
    { 0xda7cf72c, 76 },
    { 0, 0 },
    { 0xb4fcf72e, 61 },
    { 0xecbcf72e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ecf733, 85 },
    { 0x8b6cf734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eecf738, 34 },
    { 0x493cf738, 34 },
    { 0, 0 },
    { 0xae0cf73b, 23 },
    { 0xa3ecf73c, 100 },
    { 0xf9acf73c, 99 },
    { 0x522cf73e, 100 },
    { 0x5a8cf73c, 65 },
    { 0x69ecf73b, 95 },
    { 0x766cf73d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7cf744, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcf747, 67 },
    { 0, 0 },
    { 0xd40cf749, 95 },
    { 0x4a8cf749, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcf74d, 52 },
    { 0xc9fcf74e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7cf754, 90 },
    { 0xbc9cf755, 75 },
    { 0x18ecf756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708cf75d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21cf761, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfecf764, 23 },
    { 0x3ddcf765, 55 },
    { 0, 0 },
    { 0xb0fcf767, 61 },
    { 0x40bcf767, 90 },
    { 0, 0 },
    { 0x739cf76a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x839cf76d, 98 },
    { 0x87dcf76e, 66 },
    { 0xe5ccf76f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8cf773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cf776, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdcf77c, 89 },
    { 0, 0 },
    { 0x8d8cf77e, 33 },
    { 0x50bcf77e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efcf782, 95 },
    { 0x56fcf783, 61 },
    { 0x567cf783, 33 },
    { 0xb71cf785, 91 },
    { 0, 0 },
    { 0x6d0cf787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560cf78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96cf793, 78 },
    { 0xdd8cf793, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71cf7a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x607cf7a4, 23 },
    { 0, 0 },
    { 0xeb2cf7a6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cecf7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27cf7ae, 78 },
    { 0xddccf7af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67cf7b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x605cf7b5, 97 },
    { 0x918cf7b5, 80 },
    { 0xe3dcf7b5, 93 },
    { 0, 0 },
    { 0x3eacf7b9, 52 },
    { 0x535cf7b9, 23 },
    { 0, 0 },
    { 0x73dcf7bc, 90 },
    { 0x6f1cf7bc, 90 },
    { 0xb71cf7bd, 95 },
    { 0, 0 },
    { 0xe89cf7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50cf7c4, 98 },
    { 0, 0 },
    { 0x396cf7c6, 43 },
    { 0, 0 },
    { 0xc7acf7c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cf7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afcf7d2, 33 },
    { 0x36ccf7d3, 98 },
    { 0x12acf7d4, 55 },
    { 0x672cf7d5, 91 },
    { 0x7e4cf7d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cf7dd, 43 },
    { 0x310cf7de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afcf7e3, 23 },
    { 0x1a0cf7e4, 95 },
    { 0, 0 },
    { 0x67acf7e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8cf7e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25cf7ec, 93 },
    { 0x20ccf7ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dcf7f0, 90 },
    { 0x23cf7f1, 100 },
    { 0, 0 },
    { 0x659cf7f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x706cf7f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ccf7fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecf801, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62cf809, 91 },
    { 0xa91cf80a, 33 },
    { 0x891cf80a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x707cf80e, 61 },
    { 0xdddcf80f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7cf812, 55 },
    { 0xbe8cf813, 76 },
    { 0, 0 },
    { 0xa9ccf815, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ccf81a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcf81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fcf827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fcf82d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4cf833, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ecf843, 81 },
    { 0, 0 },
    { 0x3eecf845, 33 },
    { 0x751cf846, 90 },
    { 0, 0 },
    { 0x585cf848, 52 },
    { 0xb18cf849, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836cf84d, 100 },
    { 0x935cf84e, 33 },
    { 0x11bcf84f, 98 },
    { 0x888cf84f, 88 },
    { 0, 0 },
    { 0x36dcf852, 34 },
    { 0x944cf852, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dcf860, 95 },
    { 0x671cf860, 43 },
    { 0xf38cf862, 84 },
    { 0x3bacf863, 61 },
    { 0x92ecf862, 33 },
    { 0, 0 },
    { 0x839cf866, 80 },
    { 0, 0 },
    { 0xf1cf868, 23 },
    { 0xc8ecf869, 55 },
    { 0x555cf868, 33 },
    { 0, 0 },
    { 0xe0fcf86c, 34 },
    { 0, 0 },
    { 0x92bcf86e, 95 },
    { 0, 0 },
    { 0x90cf870, 23 },
    { 0x52acf871, 34 },
    { 0x49bcf872, 34 },
    { 0x404cf872, 33 },
    { 0xabdcf873, 85 },
    { 0x899cf873, 87 },
    { 0x50bcf876, 61 },
    { 0x49cf877, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x306cf87a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87cf87f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf883, 84 },
    { 0, 0 },
    { 0x1fccf885, 98 },
    { 0x922cf886, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cecf88b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacecf890, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bcf893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bcf8a1, 98 },
    { 0x48ecf8a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59acf8a7, 33 },
    { 0, 0 },
    { 0x421cf8a9, 90 },
    { 0, 0 },
    { 0xb1dcf8ab, 55 },
    { 0, 0 },
    { 0x2a3cf8ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcf8b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8cf8bd, 100 },
    { 0xbf1cf8be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9cf8c5, 43 },
    { 0xce3cf8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8cf8cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcecf8d2, 33 },
    { 0x6dfcf8d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x784cf8d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fcf8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438cf8de, 90 },
    { 0xcacf8df, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824cf8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dcf8e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82cf8eb, 87 },
    { 0xb80cf8ec, 98 },
    { 0xa11cf8eb, 90 },
    { 0x50dcf8ee, 23 },
    { 0, 0 },
    { 0x250cf8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ccf8f6, 33 },
    { 0x76dcf8f7, 91 },
    { 0xe7cf8f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422cf8fc, 46 },
    { 0xee6cf8fd, 23 },
    { 0x59acf8fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6cf905, 55 },
    { 0xfddcf905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdcf909, 95 },
    { 0x75acf909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cf90d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dcf914, 66 },
    { 0x8b7cf915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8cf918, 78 },
    { 0xcfecf918, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248cf91e, 87 },
    { 0, 0 },
    { 0xda9cf920, 61 },
    { 0x885cf921, 98 },
    { 0x66fcf922, 17 },
    { 0, 0 },
    { 0xc0fcf924, 90 },
    { 0x7d7cf925, 52 },
    { 0x9f7cf926, 88 },
    { 0x36ccf927, 98 },
    { 0x10bcf928, 90 },
    { 0, 0 },
    { 0x21ecf92a, 90 },
    { 0, 0 },
    { 0xffccf92c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437cf931, 95 },
    { 0, 0 },
    { 0x8becf933, 33 },
    { 0, 0 },
    { 0xfa3cf935, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52cf938, 90 },
    { 0, 0 },
    { 0xdecf93a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ecf944, 85 },
    { 0x904cf944, 66 },
    { 0x84acf946, 55 },
    { 0xc7bcf947, 95 },
    { 0xbeccf947, 93 },
    { 0xde0cf949, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6cf953, 34 },
    { 0xbaacf953, 33 },
    { 0x294cf955, 99 },
    { 0x737cf956, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbcf962, 98 },
    { 0x613cf963, 90 },
    { 0xd77cf963, 52 },
    { 0, 0 },
    { 0x6d7cf966, 17 },
    { 0x6e3cf966, 95 },
    { 0xcf8cf967, 46 },
    { 0x22acf969, 93 },
    { 0xcbccf96a, 43 },
    { 0x38ccf96a, 97 },
    { 0x108cf966, 55 },
    { 0xe7dcf966, 84 },
    { 0x2d8cf96e, 55 },
    { 0x171cf96f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6cf975, 34 },
    { 0xe11cf976, 95 },
    { 0, 0 },
    { 0xcc3cf978, 52 },
    { 0x735cf979, 34 },
    { 0x225cf97a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bcf97d, 55 },
    { 0xba5cf97d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfcf986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774cf989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcf98d, 55 },
    { 0xfcacf98e, 33 },
    { 0x22ccf98f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640cf993, 33 },
    { 0xeb4cf994, 33 },
    { 0xae5cf993, 61 },
    { 0x815cf994, 33 },
    { 0, 0 },
    { 0x970cf998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424cf99e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x609cf9a1, 34 },
    { 0xf45cf9a2, 61 },
    { 0xd77cf9a3, 33 },
    { 0xab5cf9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bcf9ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cf9b2, 33 },
    { 0, 0 },
    { 0x778cf9b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eccf9b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591cf9c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x971cf9c6, 99 },
    { 0xf68cf9c7, 33 },
    { 0x8b9cf9c8, 33 },
    { 0x89cf9c8, 87 },
    { 0x663cf9ca, 34 },
    { 0x45bcf9c9, 33 },
    { 0xdfcf9cc, 76 },
    { 0x316cf9ca, 43 },
    { 0x4eecf9cb, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26cf9d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74cf9d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbcf9dc, 23 },
    { 0x87ccf9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87cf9e0, 78 },
    { 0xe8fcf9e0, 61 },
    { 0x999cf9e0, 93 },
    { 0xf7acf9e3, 33 },
    { 0x347cf9e4, 55 },
    { 0, 0 },
    { 0x75bcf9e6, 100 },
    { 0x3efcf9e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350cf9f6, 76 },
    { 0, 0 },
    { 0x298cf9f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x433cf9fb, 55 },
    { 0, 0 },
    { 0x541cf9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bcfa03, 43 },
    { 0x47acfa04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cfa08, 23 },
    { 0, 0 },
    { 0xcffcfa0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cfa0d, 84 },
    { 0x2adcfa0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ecfa13, 88 },
    { 0x592cfa13, 23 },
    { 0x17acfa15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabecfa19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13cfa1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfacfa24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976cfa35, 90 },
    { 0, 0 },
    { 0x88cfa37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2cfa3a, 68 },
    { 0xa1dcfa3b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebcfa40, 23 },
    { 0x795cfa40, 34 },
    { 0, 0 },
    { 0xa81cfa43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fcfa46, 61 },
    { 0xf10cfa47, 98 },
    { 0, 0 },
    { 0xf9cfa49, 66 },
    { 0, 0 },
    { 0x3e7cfa4b, 61 },
    { 0, 0 },
    { 0xc4bcfa4d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbcfa50, 23 },
    { 0xdccfa51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cfa54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3acfa5a, 61 },
    { 0xf0ccfa5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8acfa5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cfa69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fcfa79, 23 },
    { 0, 0 },
    { 0xf7ccfa7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3cfa81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1cfa84, 23 },
    { 0xcfcfa84, 66 },
    { 0xd4bcfa85, 23 },
    { 0, 0 },
    { 0xa78cfa88, 90 },
    { 0xc3ecfa89, 99 },
    { 0x4f7cfa8a, 55 },
    { 0x5a8cfa8b, 97 },
    { 0xb95cfa8b, 90 },
    { 0, 0 },
    { 0xe8cfa8e, 52 },
    { 0x1f1cfa8e, 98 },
    { 0x6e5cfa8e, 80 },
    { 0x643cfa91, 100 },
    { 0x380cfa91, 67 },
    { 0xe67cfa92, 90 },
    { 0xd85cfa93, 43 },
    { 0x9b2cfa95, 90 },
    { 0xb5bcfa96, 34 },
    { 0xbb3cfa92, 61 },
    { 0xb23cfa96, 33 },
    { 0x899cfa99, 61 },
    { 0, 0 },
    { 0xe9dcfa9b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cfaa0, 9 },
    { 0x27dcfaa0, 23 },
    { 0x3e4cfaa2, 23 },
    { 0x971cfaa3, 43 },
    { 0x4c0cfaa4, 34 },
    { 0x9f7cfaa5, 98 },
    { 0x582cfaa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ccfaab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59cfab1, 33 },
    { 0x907cfab1, 33 },
    { 0x6e4cfab3, 17 },
    { 0x1bdcfab1, 76 },
    { 0xc06cfab4, 95 },
    { 0, 0 },
    { 0x289cfab7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8cfabb, 76 },
    { 0x860cfabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292cfabf, 100 },
    { 0, 0 },
    { 0xfa5cfac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44cfac4, 95 },
    { 0xb8fcfac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ecfac8, 33 },
    { 0, 0 },
    { 0x680cfaca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dcfad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17cfad4, 90 },
    { 0, 0 },
    { 0xffdcfad6, 61 },
    { 0, 0 },
    { 0x308cfad8, 91 },
    { 0xf4cfad9, 90 },
    { 0xa1bcfada, 78 },
    { 0xe64cfad8, 23 },
    { 0xdaccfadc, 66 },
    { 0, 0 },
    { 0x6e7cfade, 33 },
    { 0, 0 },
    { 0x9b6cfae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95cfae7, 95 },
    { 0x936cfae7, 67 },
    { 0x699cfae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acfaed, 75 },
    { 0x9cdcfaed, 23 },
    { 0x334cfaed, 23 },
    { 0x7e0cfaf0, 33 },
    { 0, 0 },
    { 0x254cfaf2, 23 },
    { 0, 0 },
    { 0x41ecfaf4, 95 },
    { 0xfbccfaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496cfafa, 23 },
    { 0xc03cfafb, 61 },
    { 0xc7cfafc, 95 },
    { 0, 0 },
    { 0xf13cfafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5cfb04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbacfb08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cfb0b, 84 },
    { 0x83dcfb0c, 23 },
    { 0, 0 },
    { 0x506cfb0e, 61 },
    { 0x88fcfb0e, 33 },
    { 0xd90cfb10, 33 },
    { 0x59bcfb11, 23 },
    { 0, 0 },
    { 0x19acfb13, 33 },
    { 0, 0 },
    { 0x694cfb15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532cfb1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3cfb1e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fcfb22, 33 },
    { 0xd4ccfb23, 66 },
    { 0xb27cfb22, 100 },
    { 0x17ccfb25, 23 },
    { 0x710cfb26, 69 },
    { 0xe7dcfb24, 99 },
    { 0x116cfb22, 69 },
    { 0, 0 },
    { 0x5dbcfb2a, 33 },
    { 0, 0 },
    { 0xbdbcfb2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddcfb30, 66 },
    { 0x160cfb31, 61 },
    { 0, 0 },
    { 0xcc5cfb33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86acfb39, 90 },
    { 0, 0 },
    { 0xc6dcfb3b, 61 },
    { 0x34fcfb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x577cfb3f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6cfb42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6cfb48, 55 },
    { 0, 0 },
    { 0x912cfb4a, 93 },
    { 0, 0 },
    { 0x391cfb4c, 78 },
    { 0xe3ccfb4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2cfb53, 95 },
    { 0x2e8cfb54, 61 },
    { 0xf96cfb54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x745cfb58, 61 },
    { 0x219cfb58, 67 },
    { 0, 0 },
    { 0xf4fcfb5b, 90 },
    { 0x3ebcfb5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cfb61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2cfb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925cfb68, 90 },
    { 0xb56cfb68, 78 },
    { 0xf37cfb68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbcfb6d, 34 },
    { 0x1aecfb6d, 98 },
    { 0xeb3cfb6f, 33 },
    { 0x188cfb70, 91 },
    { 0x466cfb6f, 33 },
    { 0xddcfb72, 60 },
    { 0xa57cfb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfb76, 76 },
    { 0x67acfb77, 75 },
    { 0xfc6cfb78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4cfb7c, 43 },
    { 0, 0 },
    { 0x6dcfb7e, 61 },
    { 0x9f3cfb7e, 87 },
    { 0x883cfb80, 33 },
    { 0, 0 },
    { 0x751cfb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cfb86, 84 },
    { 0xf70cfb87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5acfb8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3cfb91, 67 },
    { 0, 0 },
    { 0xd1cfb93, 76 },
    { 0, 0 },
    { 0xcbacfb95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefcfb9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ecfba1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750cfbac, 98 },
    { 0x1c0cfbac, 33 },
    { 0xd80cfbae, 91 },
    { 0x637cfbaf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940cfbb8, 88 },
    { 0x4e9cfbb9, 23 },
    { 0x41bcfbb9, 90 },
    { 0, 0 },
    { 0x421cfbbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dcfbc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957cfbc7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854cfbcb, 34 },
    { 0x57cfbcb, 95 },
    { 0, 0 },
    { 0xdc2cfbce, 95 },
    { 0, 0 },
    { 0x16dcfbd0, 65 },
    { 0, 0 },
    { 0x6acfbd2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808cfbd9, 52 },
    { 0, 0 },
    { 0xdeacfbdb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ccfbe1, 46 },
    { 0x278cfbe2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cfbe7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39cfbef, 46 },
    { 0x48bcfbf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcfbfa, 61 },
    { 0, 0 },
    { 0x1fcfbfc, 61 },
    { 0xf2ccfbfd, 98 },
    { 0x7c5cfbfc, 88 },
    { 0x653cfbff, 43 },
    { 0xf3ecfbff, 33 },
    { 0x9edcfc01, 61 },
    { 0x793cfbff, 46 },
    { 0x16cfbfc, 95 },
    { 0, 0 },
    { 0x315cfc05, 90 },
    { 0x848cfc06, 34 },
    { 0xe6ccfc06, 90 },
    { 0, 0 },
    { 0xdb6cfc09, 61 },
    { 0x837cfc0a, 33 },
    { 0x5f0cfc0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efcfc0e, 46 },
    { 0x1e1cfc0e, 55 },
    { 0xbc7cfc10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0cfc1a, 34 },
    { 0xef6cfc1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04cfc1e, 34 },
    { 0x6d8cfc1e, 61 },
    { 0xd29cfc1e, 46 },
    { 0xaafcfc21, 23 },
    { 0, 0 },
    { 0x89acfc23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fcfc2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696cfc33, 87 },
    { 0x67ccfc34, 91 },
    { 0x72acfc34, 61 },
    { 0x666cfc36, 34 },
    { 0xa45cfc33, 95 },
    { 0xaffcfc38, 61 },
    { 0xf86cfc33, 33 },
    { 0xfa6cfc39, 55 },
    { 0x97cfc3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x821cfc3e, 61 },
    { 0, 0 },
    { 0x25acfc40, 98 },
    { 0x4fcfc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682cfc44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76acfc49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cfc4c, 23 },
    { 0, 0 },
    { 0xbabcfc4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00cfc60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538cfc64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fcfc68, 23 },
    { 0, 0 },
    { 0xe42cfc6a, 66 },
    { 0x409cfc6b, 33 },
    { 0x8a8cfc6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cfc6f, 90 },
    { 0x35ecfc70, 23 },
    { 0xbf3cfc71, 33 },
    { 0x50acfc71, 61 },
    { 0x12bcfc70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8cfc76, 95 },
    { 0x4d9cfc77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9cfc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fcfc7e, 34 },
    { 0x6b9cfc7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ecfc83, 98 },
    { 0, 0 },
    { 0x396cfc85, 90 },
    { 0xa94cfc86, 33 },
    { 0, 0 },
    { 0xa89cfc88, 33 },
    { 0xbd0cfc88, 76 },
    { 0, 0 },
    { 0x1fdcfc8b, 87 },
    { 0x5e5cfc8c, 44 },
    { 0x577cfc8d, 93 },
    { 0x446cfc8e, 36 },
    { 0x76bcfc8e, 98 },
    { 0x73fcfc90, 90 },
    { 0x4c5cfc90, 95 },
    { 0x91dcfc92, 61 },
    { 0x6a3cfc93, 46 },
    { 0x234cfc92, 90 },
    { 0, 0 },
    { 0xd5ccfc96, 95 },
    { 0x9fcfc96, 98 },
    { 0x4cfcfc97, 95 },
    { 0xd8fcfc99, 61 },
    { 0, 0 },
    { 0x44fcfc9b, 83 },
    { 0, 0 },
    { 0xe58cfc9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28cfca4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcfcaa, 100 },
    { 0xea5cfcab, 33 },
    { 0, 0 },
    { 0x84ecfcad, 95 },
    { 0, 0 },
    { 0x1ccfcaf, 90 },
    { 0, 0 },
    { 0x192cfcb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffcfcb4, 33 },
    { 0, 0 },
    { 0x1c9cfcb6, 52 },
    { 0xd1acfcb6, 100 },
    { 0, 0 },
    { 0xceecfcb9, 98 },
    { 0, 0 },
    { 0x57fcfcbb, 23 },
    { 0x27fcfcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0cfcbf, 61 },
    { 0x56acfcbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404cfcc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8acfccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9cfcd2, 23 },
    { 0x2e5cfcd3, 67 },
    { 0x459cfcd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8accfcd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bcfce2, 90 },
    { 0, 0 },
    { 0x504cfce4, 100 },
    { 0x581cfce5, 61 },
    { 0xaa4cfce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4cfce9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcfced, 76 },
    { 0x580cfced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3cfcf4, 95 },
    { 0, 0 },
    { 0x978cfcf6, 61 },
    { 0xd6fcfcf7, 67 },
    { 0xba5cfcf8, 84 },
    { 0, 0 },
    { 0xbd0cfcfa, 76 },
    { 0x8d8cfcfb, 90 },
    { 0x2b9cfcfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09cfd01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7cfd04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71cfd07, 78 },
    { 0x5e2cfd08, 23 },
    { 0xfeacfd07, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251cfd12, 61 },
    { 0xbd7cfd13, 90 },
    { 0, 0 },
    { 0xbb3cfd15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571cfd1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7cfd24, 99 },
    { 0x982cfd25, 90 },
    { 0, 0 },
    { 0xab6cfd27, 23 },
    { 0x410cfd28, 66 },
    { 0x169cfd28, 93 },
    { 0x1d1cfd2a, 67 },
    { 0x7e8cfd29, 90 },
    { 0xee4cfd2c, 33 },
    { 0xd3cfd2d, 55 },
    { 0x781cfd28, 55 },
    { 0x71dcfd2c, 98 },
    { 0x15fcfd2d, 43 },
    { 0, 0 },
    { 0x5b7cfd32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56cfd39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bcfd3c, 23 },
    { 0x5d1cfd3c, 33 },
    { 0, 0 },
    { 0xa8ccfd3f, 66 },
    { 0xc28cfd3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155cfd45, 61 },
    { 0x54ecfd46, 33 },
    { 0x589cfd47, 76 },
    { 0xc11cfd48, 90 },
    { 0x91cfd45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0cfd4d, 90 },
    { 0xc2acfd4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1decfd51, 33 },
    { 0xd6cfd52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47cfd55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192cfd59, 98 },
    { 0xda6cfd5a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ecfd5f, 95 },
    { 0xdfacfd5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3cfd67, 90 },
    { 0x305cfd67, 95 },
    { 0, 0 },
    { 0xd5dcfd6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccfd70, 33 },
    { 0x23bcfd70, 61 },
    { 0x9d0cfd71, 98 },
    { 0, 0 },
    { 0xef8cfd74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4cfd78, 87 },
    { 0xb8bcfd79, 11 },
    { 0x3eccfd7a, 52 },
    { 0, 0 },
    { 0x848cfd7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910cfd83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307cfd8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dcfd90, 95 },
    { 0x626cfd91, 77 },
    { 0xb72cfd90, 61 },
    { 0xa2dcfd93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bcfd98, 95 },
    { 0xbbbcfd99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77cfd9c, 61 },
    { 0x371cfd9d, 48 },
    { 0xdf0cfd9e, 33 },
    { 0, 0 },
    { 0x9a1cfda0, 33 },
    { 0xe29cfda1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3cfda5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bcfda8, 43 },
    { 0xc94cfda9, 91 },
    { 0xb0bcfdaa, 34 },
    { 0xa21cfdab, 33 },
    { 0xeb1cfdaa, 55 },
    { 0x4cbcfdad, 61 },
    { 0, 0 },
    { 0x737cfdaf, 43 },
    { 0x5c3cfdb0, 55 },
    { 0xf0cfdb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcfdb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x876cfdb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cfdbd, 95 },
    { 0x716cfdbe, 46 },
    { 0x13dcfdbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcfdc2, 33 },
    { 0, 0 },
    { 0xaeecfdc4, 67 },
    { 0xdc3cfdc5, 23 },
    { 0, 0 },
    { 0x613cfdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dcfdca, 77 },
    { 0x55fcfdcb, 90 },
    { 0x8bfcfdcb, 100 },
    { 0xd27cfdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fcfdd0, 33 },
    { 0x5becfdd1, 90 },
    { 0, 0 },
    { 0x527cfdd3, 94 },
    { 0, 0 },
    { 0xd44cfdd5, 33 },
    { 0x82bcfdd6, 23 },
    { 0x8b5cfdd5, 23 },
    { 0xa00cfdd5, 34 },
    { 0xec2cfdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58cfde5, 66 },
    { 0x892cfde6, 61 },
    { 0xf03cfde5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98cfdeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7cfdee, 90 },
    { 0x505cfdef, 90 },
    { 0xe4ecfdef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dcfdf6, 69 },
    { 0x411cfdf6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53cfdfc, 43 },
    { 0x691cfdfc, 91 },
    { 0xf05cfdfe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ccfe05, 95 },
    { 0x9e2cfe05, 34 },
    { 0xec5cfe05, 34 },
    { 0xd67cfe06, 98 },
    { 0, 0 },
    { 0xa61cfe0a, 55 },
    { 0x896cfe0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371cfe0f, 99 },
    { 0xc6fcfe0f, 91 },
    { 0xec9cfe11, 95 },
    { 0, 0 },
    { 0xb42cfe13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4cfe16, 97 },
    { 0x6eccfe17, 34 },
    { 0x8f1cfe18, 76 },
    { 0x563cfe18, 87 },
    { 0, 0 },
    { 0xb8cfe1b, 61 },
    { 0, 0 },
    { 0x2ebcfe1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3cfe22, 33 },
    { 0x1a0cfe22, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52cfe2b, 55 },
    { 0, 0 },
    { 0x4f3cfe2d, 87 },
    { 0xe05cfe2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535cfe38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7cfe3e, 90 },
    { 0, 0 },
    { 0x940cfe40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2cfe45, 23 },
    { 0x795cfe46, 90 },
    { 0x8bcfe47, 55 },
    { 0x87ecfe48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834cfe4f, 40 },
    { 0, 0 },
    { 0x857cfe51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36acfe56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fecfe5e, 90 },
    { 0, 0 },
    { 0xaa4cfe60, 95 },
    { 0x3e8cfe61, 90 },
    { 0xa57cfe62, 98 },
    { 0, 0 },
    { 0x19bcfe64, 90 },
    { 0, 0 },
    { 0x15ecfe66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ecfe6c, 90 },
    { 0, 0 },
    { 0x1e0cfe6e, 81 },
    { 0, 0 },
    { 0xf59cfe70, 90 },
    { 0x206cfe71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606cfe79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x296cfe7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acfe83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36cfe86, 99 },
    { 0, 0 },
    { 0xc1bcfe88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1cfe8f, 90 },
    { 0xc78cfe8f, 90 },
    { 0xae7cfe91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ecfe94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccfe9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05cfe9e, 96 },
    { 0xba3cfe9f, 61 },
    { 0x705cfe9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dcfea4, 90 },
    { 0, 0 },
    { 0x543cfea6, 61 },
    { 0xe4acfea7, 23 },
    { 0x8b4cfea8, 55 },
    { 0x14fcfea9, 78 },
    { 0x7ddcfeaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcfeb7, 76 },
    { 0xc3fcfeb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5cfebd, 87 },
    { 0x30bcfebe, 61 },
    { 0x6edcfebe, 33 },
    { 0, 0 },
    { 0xd87cfec1, 98 },
    { 0x56dcfec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27cfec7, 55 },
    { 0x703cfec8, 34 },
    { 0xe9dcfec7, 33 },
    { 0, 0 },
    { 0x6b4cfecb, 85 },
    { 0x8d1cfecb, 68 },
    { 0x1acfecd, 98 },
    { 0xe14cfece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dcfed1, 46 },
    { 0x20ccfed2, 98 },
    { 0xf4fcfed2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cfed9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2cfedc, 90 },
    { 0x465cfedc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463cfee3, 76 },
    { 0x5e7cfee4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xefecfee7, 98 },
    { 0, 0 },
    { 0x3c2cfee9, 90 },
    { 0x2adcfeea, 92 },
    { 0x5d7cfee9, 33 },
    { 0x198cfeeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8cfef1, 90 },
    { 0, 0 },
    { 0xa4cfef3, 76 },
    { 0xe72cfef4, 43 },
    { 0xeb1cfef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5cfef8, 76 },
    { 0, 0 },
    { 0xf56cfefa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89cfefd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dcff02, 90 },
    { 0x3cacff03, 76 },
    { 0x8d9cff03, 66 },
    { 0x4d3cff05, 76 },
    { 0xb4ecff03, 46 },
    { 0, 0 },
    { 0x4cccff08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ccff0c, 34 },
    { 0x990cff0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1cff12, 98 },
    { 0x844cff13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33cff17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cff1a, 33 },
    { 0, 0 },
    { 0x973cff1c, 76 },
    { 0, 0 },
    { 0xdfcff1e, 100 },
    { 0, 0 },
    { 0xce8cff20, 98 },
    { 0x12bcff21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cff24, 90 },
    { 0xcfecff24, 90 },
    { 0xa91cff26, 61 },
    { 0, 0 },
    { 0x1c8cff28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fcff2b, 95 },
    { 0, 0 },
    { 0x917cff2d, 55 },
    { 0, 0 },
    { 0xba5cff2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcff36, 34 },
    { 0x404cff37, 56 },
    { 0xe27cff38, 91 },
    { 0x777cff38, 100 },
    { 0xf10cff39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28cff3e, 33 },
    { 0x645cff3e, 97 },
    { 0, 0 },
    { 0x474cff41, 90 },
    { 0xa84cff41, 33 },
    { 0x27acff42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcff50, 90 },
    { 0, 0 },
    { 0xaf9cff52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346cff5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93acff5e, 17 },
    { 0xbb6cff5f, 76 },
    { 0xf0cff5f, 23 },
    { 0x420cff61, 98 },
    { 0x1cfcff62, 61 },
    { 0xc3dcff60, 81 },
    { 0, 0 },
    { 0x765cff65, 76 },
    { 0x5a6cff66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cff6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dcff77, 78 },
    { 0, 0 },
    { 0xc8ccff79, 95 },
    { 0x541cff79, 95 },
    { 0, 0 },
    { 0x78ccff7c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x193cff7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353cff84, 100 },
    { 0, 0 },
    { 0xa5dcff86, 40 },
    { 0xa98cff87, 90 },
    { 0xececff86, 95 },
    { 0, 0 },
    { 0x33bcff8a, 81 },
    { 0xb92cff8b, 95 },
    { 0xf04cff8c, 98 },
    { 0xf50cff8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4cff94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x603cff97, 64 },
    { 0x54acff98, 98 },
    { 0, 0 },
    { 0x411cff9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cffa0, 98 },
    { 0xe7bcffa0, 33 },
    { 0x3adcffa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0cffa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fcffa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174cffac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbcffb2, 95 },
    { 0x35bcffb2, 33 },
    { 0x62bcffb3, 76 },
    { 0, 0 },
    { 0x33dcffb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117cffbc, 90 },
    { 0x4cffbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2cffc2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38cffc7, 77 },
    { 0xaa4cffc8, 55 },
    { 0x8eacffc9, 97 },
    { 0x2c1cffc9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcffcf, 99 },
    { 0x2c3cffd0, 98 },
    { 0, 0 },
    { 0xcafcffd2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512cffd6, 23 },
    { 0, 0 },
    { 0x38dcffd8, 93 },
    { 0xc77cffd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cffdd, 61 },
    { 0x859cffdd, 75 },
    { 0x66ccffdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48cffe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17cffe7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3cffec, 95 },
    { 0, 0 },
    { 0xc9cffee, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cfff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cfff8, 93 },
    { 0xdc7cfff9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bcfffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cd0000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bd0004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d0008, 33 },
    { 0xc64d0009, 69 },
    { 0x953d000a, 90 },
    { 0x76fd000b, 95 },
    { 0xaf2d000a, 94 },
    { 0xad4d0009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dd0011, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94d0018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d001b, 98 },
    { 0x6cdd001b, 76 },
    { 0, 0 },
    { 0x856d001e, 98 },
    { 0x19fd001f, 93 },
    { 0x2b5d0020, 66 },
    { 0xfa9d0021, 99 },
    { 0x66ed0021, 92 },
    { 0x3fad0022, 81 },
    { 0x720d0024, 76 },
    { 0x7d0d0023, 33 },
    { 0x167d0026, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd0029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8d002d, 34 },
    { 0, 0 },
    { 0x2e7d002f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1d0033, 43 },
    { 0xa3ed0034, 98 },
    { 0x93d0034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd0039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17d003d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabd0041, 88 },
    { 0xd2d0041, 17 },
    { 0x728d0043, 95 },
    { 0x979d0044, 33 },
    { 0x652d0044, 90 },
    { 0x287d0041, 100 },
    { 0xd71d0047, 34 },
    { 0x718d0047, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d004b, 23 },
    { 0, 0 },
    { 0xd2fd004d, 90 },
    { 0xb68d004d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6d0058, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64d005f, 95 },
    { 0, 0 },
    { 0x545d0061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d0068, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d006b, 33 },
    { 0xa3dd006c, 33 },
    { 0x47dd006d, 98 },
    { 0, 0 },
    { 0xc30d006f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47d0074, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d0078, 98 },
    { 0x26fd0078, 99 },
    { 0x709d007a, 33 },
    { 0x6c2d007b, 43 },
    { 0xac7d007c, 46 },
    { 0x67cd0078, 23 },
    { 0x3dcd007e, 90 },
    { 0xe38d007f, 55 },
    { 0x371d0080, 23 },
    { 0xe3ad0081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d0084, 33 },
    { 0x202d0084, 98 },
    { 0x766d0086, 33 },
    { 0x67d0087, 77 },
    { 0, 0 },
    { 0xd57d0089, 95 },
    { 0, 0 },
    { 0xe07d008b, 36 },
    { 0, 0 },
    { 0x334d008d, 76 },
    { 0, 0 },
    { 0xcbfd008f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d0093, 98 },
    { 0x147d0094, 78 },
    { 0, 0 },
    { 0xd99d0096, 95 },
    { 0, 0 },
    { 0x4d1d0098, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d00a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4d00ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xabad00af, 66 },
    { 0x741d00b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48d00b7, 90 },
    { 0, 0 },
    { 0xd92d00b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6d00bc, 37 },
    { 0xb40d00bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cad00c2, 87 },
    { 0, 0 },
    { 0x36fd00c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d00d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccd00d6, 55 },
    { 0, 0 },
    { 0xac9d00d8, 23 },
    { 0xfedd00d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdd00dd, 77 },
    { 0, 0 },
    { 0x4f2d00df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd00e8, 61 },
    { 0, 0 },
    { 0xe0ad00ea, 92 },
    { 0, 0 },
    { 0xef5d00ec, 61 },
    { 0x2d3d00ec, 99 },
    { 0x86cd00ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d00f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d00fa, 33 },
    { 0, 0 },
    { 0x75d00fc, 23 },
    { 0, 0 },
    { 0x103d00fe, 76 },
    { 0xfc4d00fe, 23 },
    { 0xb3cd0100, 78 },
    { 0x806d0101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d0106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fd010a, 43 },
    { 0, 0 },
    { 0x5e4d010c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x455d010f, 61 },
    { 0, 0 },
    { 0xcc8d0111, 61 },
    { 0xe8bd0112, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7d0116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ed011b, 90 },
    { 0x807d011b, 95 },
    { 0x4c7d011d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319d0124, 95 },
    { 0x445d0124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad0128, 95 },
    { 0x630d0128, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad0130, 76 },
    { 0xb13d0130, 61 },
    { 0xadbd0132, 100 },
    { 0xa91d0130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d0138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d0141, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x213d0144, 90 },
    { 0xd83d0145, 95 },
    { 0xbcdd0146, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x601d0149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5d015b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd0163, 76 },
    { 0, 0 },
    { 0x252d0165, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0168, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d016b, 23 },
    { 0, 0 },
    { 0x81ad016d, 61 },
    { 0xfaad016e, 87 },
    { 0xa15d016d, 98 },
    { 0x6ed016e, 55 },
    { 0xda5d0170, 76 },
    { 0, 0 },
    { 0xf42d0173, 95 },
    { 0, 0 },
    { 0x5b1d0175, 95 },
    { 0x4dcd0176, 78 },
    { 0x62dd0176, 55 },
    { 0xf3bd0176, 76 },
    { 0x670d0179, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d017e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100d0183, 61 },
    { 0x714d0183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d0187, 61 },
    { 0, 0 },
    { 0xad2d0189, 23 },
    { 0xb2ad018a, 98 },
    { 0xa27d018a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d0190, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d0194, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd019b, 33 },
    { 0xc1ed019b, 46 },
    { 0x4ad019c, 90 },
    { 0x30d019e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdd01a5, 23 },
    { 0x511d01a6, 85 },
    { 0xe5d01a5, 95 },
    { 0, 0 },
    { 0x261d01a9, 78 },
    { 0xd4bd01a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd01b2, 95 },
    { 0x90ed01b2, 55 },
    { 0, 0 },
    { 0xc0dd01b5, 100 },
    { 0, 0 },
    { 0xdebd01b7, 61 },
    { 0, 0 },
    { 0x1d4d01b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fd01c1, 98 },
    { 0xd62d01c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d01c5, 48 },
    { 0x33ed01c6, 33 },
    { 0x28cd01c6, 61 },
    { 0xa4ad01c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d01cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594d01cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cd01d2, 90 },
    { 0, 0 },
    { 0x6f4d01d4, 33 },
    { 0x798d01d5, 34 },
    { 0x894d01d6, 61 },
    { 0x35dd01d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dd01de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d01e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6add01e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fd01eb, 78 },
    { 0, 0 },
    { 0x94bd01ed, 46 },
    { 0xb84d01ee, 77 },
    { 0x6b2d01ef, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xafed01f2, 87 },
    { 0xcdfd01f3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28d01ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7d0204, 46 },
    { 0x3c3d0204, 55 },
    { 0x73bd0206, 87 },
    { 0xc6ad0207, 75 },
    { 0x881d0206, 90 },
    { 0xcc7d0205, 85 },
    { 0xf3ad0204, 99 },
    { 0, 0 },
    { 0x6bad020c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffd020f, 78 },
    { 0xb21d020f, 90 },
    { 0, 0 },
    { 0x5f9d0212, 67 },
    { 0x924d0213, 76 },
    { 0, 0 },
    { 0x87ad0215, 98 },
    { 0xb83d0216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cd0219, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d0221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cd0225, 90 },
    { 0, 0 },
    { 0xb1ad0227, 90 },
    { 0xfad0228, 67 },
    { 0, 0 },
    { 0xc32d022a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9d0230, 98 },
    { 0xfedd0231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebd0234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853d0239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd023c, 61 },
    { 0xa1cd023d, 33 },
    { 0x890d023c, 66 },
    { 0x7c7d023c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd0242, 43 },
    { 0x9d2d0243, 55 },
    { 0, 0 },
    { 0x703d0245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4d0249, 23 },
    { 0, 0 },
    { 0x844d024b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ad024e, 98 },
    { 0x3a7d024f, 33 },
    { 0xa3ed024f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d0254, 74 },
    { 0xe04d0255, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4d0258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322d025d, 98 },
    { 0x209d025e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd0264, 78 },
    { 0x7fbd0264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ed0269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d026d, 33 },
    { 0xd31d026e, 23 },
    { 0x7a0d026e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0275, 90 },
    { 0x441d0276, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07d027c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd0280, 95 },
    { 0xa16d0280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bd0285, 55 },
    { 0, 0 },
    { 0xdb2d0287, 55 },
    { 0xcbfd0288, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cd028c, 95 },
    { 0, 0 },
    { 0x14fd028e, 65 },
    { 0xa9ed028f, 90 },
    { 0xe6ad028f, 23 },
    { 0, 0 },
    { 0x58cd0292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88d0295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524d0298, 65 },
    { 0xe19d0298, 76 },
    { 0x6f5d029a, 61 },
    { 0x464d0298, 94 },
    { 0x50bd029c, 46 },
    { 0, 0 },
    { 0xb88d029e, 98 },
    { 0x899d029f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66d02a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fd02a5, 66 },
    { 0x4ead02a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd02ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd02ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00d02b5, 84 },
    { 0, 0 },
    { 0xd65d02b7, 33 },
    { 0x66ed02b8, 90 },
    { 0, 0 },
    { 0x936d02ba, 33 },
    { 0xc53d02bb, 95 },
    { 0xd47d02bc, 90 },
    { 0xd93d02bb, 23 },
    { 0xa25d02be, 52 },
    { 0xe6cd02ba, 90 },
    { 0, 0 },
    { 0xee2d02c1, 23 },
    { 0, 0 },
    { 0x196d02c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecd02c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d02cc, 67 },
    { 0x968d02cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964d02d3, 90 },
    { 0xa6ad02d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7d02d8, 93 },
    { 0x8bdd02d9, 33 },
    { 0xa2bd02da, 61 },
    { 0x645d02da, 95 },
    { 0xc4d02d8, 33 },
    { 0x8dd02dc, 23 },
    { 0xd1fd02de, 33 },
    { 0x92ad02de, 33 },
    { 0x9e4d02df, 95 },
    { 0xfb1d02e1, 52 },
    { 0xc2ad02e0, 23 },
    { 0x8a2d02e3, 55 },
    { 0x1f5d02df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d02ec, 90 },
    { 0, 0 },
    { 0x500d02ee, 95 },
    { 0xbb3d02ef, 95 },
    { 0x9b2d02ee, 99 },
    { 0, 0 },
    { 0xbeed02f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0d02f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd02fa, 67 },
    { 0, 0 },
    { 0x60d02fc, 99 },
    { 0, 0 },
    { 0xcdad02fe, 95 },
    { 0, 0 },
    { 0x2b1d0300, 90 },
    { 0, 0 },
    { 0xf2d0302, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ed030a, 33 },
    { 0, 0 },
    { 0x985d030c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d0310, 91 },
    { 0, 0 },
    { 0xa73d0312, 33 },
    { 0x3fbd0312, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429d0318, 90 },
    { 0x519d0319, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dd031d, 64 },
    { 0x58fd031e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3d0321, 98 },
    { 0, 0 },
    { 0x38bd0323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5d0326, 23 },
    { 0, 0 },
    { 0x7cdd0328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d032d, 46 },
    { 0, 0 },
    { 0x228d032f, 66 },
    { 0, 0 },
    { 0xe05d0331, 61 },
    { 0xcb9d0331, 95 },
    { 0, 0 },
    { 0x1d6d0334, 43 },
    { 0x4e7d0334, 95 },
    { 0xb49d0336, 90 },
    { 0xf91d0337, 33 },
    { 0, 0 },
    { 0x7bad0339, 55 },
    { 0xb0fd033a, 93 },
    { 0xbdad033b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd033e, 76 },
    { 0xb9bd033e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2d0342, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd0347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd034e, 95 },
    { 0, 0 },
    { 0x35dd0350, 86 },
    { 0x8c7d0351, 23 },
    { 0, 0 },
    { 0xe37d0353, 33 },
    { 0, 0 },
    { 0xa6dd0355, 55 },
    { 0x530d0356, 95 },
    { 0x801d0356, 66 },
    { 0xda3d0356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d035b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d0360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7d0364, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26d0367, 65 },
    { 0x374d0368, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0370, 61 },
    { 0x3e9d0371, 93 },
    { 0xe6cd0371, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6d0375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d037b, 55 },
    { 0, 0 },
    { 0xfddd037d, 95 },
    { 0x12ed037d, 95 },
    { 0, 0 },
    { 0x23d0380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d0383, 55 },
    { 0xf72d0384, 23 },
    { 0xc8d0384, 90 },
    { 0xcfed0384, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201d038b, 93 },
    { 0, 0 },
    { 0x125d038d, 33 },
    { 0, 0 },
    { 0x98ad038f, 33 },
    { 0xcadd0390, 100 },
    { 0x24dd0390, 98 },
    { 0xe1fd0392, 90 },
    { 0x7cfd0393, 98 },
    { 0xa85d0394, 100 },
    { 0x95d0394, 95 },
    { 0x8d8d0396, 78 },
    { 0x597d0397, 78 },
    { 0xce0d0396, 61 },
    { 0x7a4d0396, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ed039c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d039f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dd03a2, 77 },
    { 0x1add03a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834d03a8, 93 },
    { 0, 0 },
    { 0x709d03aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41d03b0, 90 },
    { 0xe81d03b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dd03b5, 78 },
    { 0x1cad03b5, 76 },
    { 0xa5cd03b7, 99 },
    { 0x279d03b8, 55 },
    { 0, 0 },
    { 0x73ad03ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d03bf, 80 },
    { 0, 0 },
    { 0x24cd03c1, 23 },
    { 0xb4fd03c2, 43 },
    { 0xf5bd03c3, 23 },
    { 0x45fd03c3, 46 },
    { 0x891d03c5, 98 },
    { 0x4d4d03c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4d03c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3d03d2, 84 },
    { 0, 0 },
    { 0xa6d03d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227d03d8, 31 },
    { 0xe0ad03d9, 33 },
    { 0xed6d03d9, 95 },
    { 0xca6d03db, 91 },
    { 0, 0 },
    { 0x88fd03dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abd03e1, 98 },
    { 0x875d03e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d03e5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaed03ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d03f2, 36 },
    { 0x795d03f3, 43 },
    { 0, 0 },
    { 0x2efd03f5, 87 },
    { 0, 0 },
    { 0x9bbd03f7, 87 },
    { 0x7f4d03f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ed03fc, 38 },
    { 0xd8fd03fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ad0400, 90 },
    { 0x55ed0400, 33 },
    { 0xfa9d0402, 61 },
    { 0x435d0401, 33 },
    { 0xc0d0404, 90 },
    { 0x7e8d0401, 84 },
    { 0xbbcd0401, 99 },
    { 0, 0 },
    { 0xa1dd0408, 99 },
    { 0x748d0408, 90 },
    { 0, 0 },
    { 0x4fcd040b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7d0424, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980d0428, 55 },
    { 0x3bd0429, 90 },
    { 0x565d042a, 66 },
    { 0x7d3d042a, 33 },
    { 0, 0 },
    { 0xdcad042d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccd0430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d0434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd043a, 93 },
    { 0x5b3d043b, 33 },
    { 0x3d4d043c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d043f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d0442, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded0447, 33 },
    { 0x1c5d0448, 43 },
    { 0, 0 },
    { 0xd67d044a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x129d044d, 34 },
    { 0, 0 },
    { 0xa2bd044f, 90 },
    { 0xa02d0450, 78 },
    { 0xdb6d0451, 56 },
    { 0xd56d0450, 23 },
    { 0x24dd0450, 90 },
    { 0x55dd0454, 23 },
    { 0x983d0450, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344d045a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd045d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525d0466, 95 },
    { 0x9ccd0466, 43 },
    { 0, 0 },
    { 0xfa9d0469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1d046f, 61 },
    { 0x3e4d0470, 95 },
    { 0x745d0471, 93 },
    { 0x163d0470, 23 },
    { 0x269d0473, 60 },
    { 0x727d0473, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80d0478, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4d047d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d0483, 23 },
    { 0x125d0484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989d048b, 23 },
    { 0x66ed048c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ced0491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bd0494, 23 },
    { 0x89fd0494, 34 },
    { 0x1b5d0496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d04a0, 55 },
    { 0x92cd04a1, 94 },
    { 0, 0 },
    { 0x366d04a3, 67 },
    { 0xf98d04a4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfd04ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d04b4, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227d04bb, 23 },
    { 0xdded04bb, 95 },
    { 0, 0 },
    { 0xf37d04be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed04c4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x751d04c7, 61 },
    { 0xff4d04c8, 61 },
    { 0x20dd04c9, 99 },
    { 0xbd3d04ca, 95 },
    { 0x9f0d04cb, 78 },
    { 0x44ad04cc, 23 },
    { 0xe75d04cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd04d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd04d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51d04d6, 95 },
    { 0xc8d04d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd04da, 66 },
    { 0x135d04db, 33 },
    { 0x159d04dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad04e0, 84 },
    { 0, 0 },
    { 0xca3d04e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ad04e6, 43 },
    { 0, 0 },
    { 0xfc5d04e8, 98 },
    { 0x89ad04e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad04f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5d04fb, 46 },
    { 0x741d04fc, 100 },
    { 0xb79d04fd, 76 },
    { 0x1bfd04fd, 61 },
    { 0, 0 },
    { 0x60fd0500, 33 },
    { 0x8e8d0500, 90 },
    { 0x938d0502, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd0507, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137d050d, 95 },
    { 0xbaad050d, 67 },
    { 0x67cd050f, 87 },
    { 0, 0 },
    { 0x6e8d0511, 95 },
    { 0x44ed0511, 55 },
    { 0x5b0d0511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed0519, 66 },
    { 0, 0 },
    { 0x749d051b, 33 },
    { 0, 0 },
    { 0xf13d051d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd0521, 33 },
    { 0, 0 },
    { 0x335d0523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd0528, 98 },
    { 0xcecd0529, 75 },
    { 0x9b8d052a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72d0533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ad0536, 98 },
    { 0xc02d0537, 95 },
    { 0x36dd0537, 23 },
    { 0xb36d0539, 76 },
    { 0x752d0539, 46 },
    { 0x2c9d053b, 85 },
    { 0x985d053c, 52 },
    { 0x7f0d053c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503d0541, 61 },
    { 0x534d0541, 87 },
    { 0x22dd0543, 33 },
    { 0x5bbd0541, 33 },
    { 0, 0 },
    { 0x986d0546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872d054c, 78 },
    { 0, 0 },
    { 0x4d2d054e, 23 },
    { 0, 0 },
    { 0x1a9d0550, 33 },
    { 0xfccd0551, 98 },
    { 0x35fd0551, 33 },
    { 0x337d0553, 76 },
    { 0, 0 },
    { 0xccad0555, 84 },
    { 0xee0d0555, 55 },
    { 0, 0 },
    { 0xe55d0558, 46 },
    { 0, 0 },
    { 0x752d055a, 90 },
    { 0xcdd055a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bd0561, 76 },
    { 0xeeed0562, 90 },
    { 0xeb5d0562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd0567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177d056b, 33 },
    { 0xf4dd056c, 95 },
    { 0x835d056d, 91 },
    { 0x361d056e, 66 },
    { 0xcc6d056e, 100 },
    { 0xf56d0570, 90 },
    { 0x5e8d056f, 76 },
    { 0x281d0572, 88 },
    { 0x6cdd0570, 46 },
    { 0, 0 },
    { 0xd5dd0575, 95 },
    { 0xc00d0576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d0579, 33 },
    { 0x527d057a, 33 },
    { 0x4b0d0579, 23 },
    { 0, 0 },
    { 0x33cd057d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512d0583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4d0587, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43d058a, 90 },
    { 0, 0 },
    { 0x270d058c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d058f, 64 },
    { 0x34fd058f, 34 },
    { 0, 0 },
    { 0xa61d0592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5d0597, 100 },
    { 0x519d0598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d059d, 23 },
    { 0, 0 },
    { 0x6bdd059f, 33 },
    { 0x37bd05a0, 23 },
    { 0x82ad05a1, 66 },
    { 0x96d05a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2d05a7, 34 },
    { 0xbc7d05a8, 61 },
    { 0xafd05a9, 76 },
    { 0, 0 },
    { 0x6a0d05ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd05ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd05b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d05b4, 61 },
    { 0, 0 },
    { 0xdb2d05b6, 90 },
    { 0, 0 },
    { 0x3b1d05b8, 93 },
    { 0, 0 },
    { 0x27ed05ba, 33 },
    { 0, 0 },
    { 0xfd5d05bc, 61 },
    { 0, 0 },
    { 0x62d05be, 99 },
    { 0x3e4d05bf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d05c2, 78 },
    { 0x56ad05c3, 76 },
    { 0x9d6d05c4, 23 },
    { 0x69bd05c4, 33 },
    { 0, 0 },
    { 0xc34d05c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1d05ca, 76 },
    { 0x914d05ca, 64 },
    { 0, 0 },
    { 0xc58d05cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfd05d2, 90 },
    { 0, 0 },
    { 0xfbd05d4, 46 },
    { 0, 0 },
    { 0xdfdd05d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502d05da, 77 },
    { 0xa89d05db, 90 },
    { 0xecdd05dc, 98 },
    { 0x5dbd05dd, 87 },
    { 0x59d05dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d05e3, 56 },
    { 0xd48d05e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d05eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ed05ef, 61 },
    { 0xdd05f0, 61 },
    { 0xf15d05f0, 23 },
    { 0xf16d05f0, 55 },
    { 0x76cd05f3, 90 },
    { 0, 0 },
    { 0xf27d05f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad05fa, 90 },
    { 0x3bcd05fa, 55 },
    { 0x546d05fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738d0602, 23 },
    { 0, 0 },
    { 0xb5fd0604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2d0607, 90 },
    { 0x45fd0608, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfd060e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7d0614, 46 },
    { 0xae4d0614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d061c, 33 },
    { 0xb1ad061d, 76 },
    { 0xa0bd061e, 90 },
    { 0x6fed061d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x773d0622, 61 },
    { 0x659d0623, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfd0626, 95 },
    { 0x4d9d0627, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64d062a, 55 },
    { 0x606d062b, 61 },
    { 0xd34d062c, 95 },
    { 0xb2ad062c, 33 },
    { 0x702d062e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d0632, 33 },
    { 0x932d0633, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcd0637, 95 },
    { 0xff6d0638, 77 },
    { 0x848d0639, 34 },
    { 0xb27d063a, 33 },
    { 0x214d063b, 90 },
    { 0xa70d063b, 95 },
    { 0xcedd063a, 33 },
    { 0xd4dd0639, 90 },
    { 0x26ed063f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d0647, 98 },
    { 0xe16d0648, 85 },
    { 0x2f8d0649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd064d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ed0650, 99 },
    { 0xab5d0651, 43 },
    { 0xe44d0650, 95 },
    { 0x5ccd0653, 87 },
    { 0xb08d0654, 33 },
    { 0, 0 },
    { 0x967d0656, 90 },
    { 0, 0 },
    { 0x515d0658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d065b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd065f, 98 },
    { 0, 0 },
    { 0x9f1d0661, 29 },
    { 0x70ed0661, 95 },
    { 0, 0 },
    { 0xc62d0664, 100 },
    { 0x360d0664, 98 },
    { 0xe36d0664, 98 },
    { 0x119d0666, 51 },
    { 0xc47d0668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d066d, 76 },
    { 0, 0 },
    { 0xf02d066f, 23 },
    { 0, 0 },
    { 0xc2fd0671, 76 },
    { 0x5b7d0671, 61 },
    { 0x608d0671, 100 },
    { 0xc1ad0674, 95 },
    { 0xa6ad0675, 78 },
    { 0, 0 },
    { 0x96cd0677, 93 },
    { 0xe3dd0678, 61 },
    { 0xfadd0677, 98 },
    { 0xffd067a, 95 },
    { 0x939d067a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d067e, 90 },
    { 0x37ad067e, 23 },
    { 0, 0 },
    { 0x4b4d0681, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ed0684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaad068a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119d068f, 34 },
    { 0x178d0690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fd0693, 90 },
    { 0, 0 },
    { 0x7eed0695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9d069b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fd069f, 99 },
    { 0, 0 },
    { 0x7f2d06a1, 55 },
    { 0, 0 },
    { 0xb65d06a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d06a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d06b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d06ba, 61 },
    { 0x7e7d06bb, 93 },
    { 0x6ecd06bc, 90 },
    { 0xcb7d06bd, 98 },
    { 0xd4d06be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0d06c4, 76 },
    { 0xb7dd06c4, 98 },
    { 0xeb1d06c6, 98 },
    { 0, 0 },
    { 0x9dfd06c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d06cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d06d5, 90 },
    { 0xeaed06d6, 33 },
    { 0x7f6d06d7, 100 },
    { 0x423d06d8, 23 },
    { 0x692d06d7, 85 },
    { 0, 0 },
    { 0xa26d06db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603d06df, 23 },
    { 0, 0 },
    { 0x886d06e1, 76 },
    { 0, 0 },
    { 0x223d06e3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d06e6, 40 },
    { 0, 0 },
    { 0x483d06e8, 33 },
    { 0xc5bd06e9, 33 },
    { 0x526d06ea, 77 },
    { 0xf5ed06eb, 23 },
    { 0x860d06ec, 52 },
    { 0x39fd06ec, 90 },
    { 0, 0 },
    { 0x2f7d06ef, 46 },
    { 0xfeed06ef, 87 },
    { 0, 0 },
    { 0x3edd06f2, 95 },
    { 0xbddd06f3, 61 },
    { 0xec4d06f4, 78 },
    { 0x188d06f3, 55 },
    { 0, 0 },
    { 0xfbdd06f7, 99 },
    { 0, 0 },
    { 0x3b8d06f9, 23 },
    { 0xc80d06fa, 87 },
    { 0x183d06f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abd06ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188d070a, 33 },
    { 0x979d070b, 76 },
    { 0, 0 },
    { 0x8f8d070d, 84 },
    { 0x12fd070e, 12 },
    { 0xd75d070f, 98 },
    { 0xdddd0710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743d0714, 90 },
    { 0x483d0715, 33 },
    { 0x503d0716, 87 },
    { 0xf07d0717, 90 },
    { 0xe13d0716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d071c, 33 },
    { 0xab4d071d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ed0720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bd072a, 95 },
    { 0x23cd072a, 66 },
    { 0, 0 },
    { 0xc1d072d, 76 },
    { 0x9bfd072e, 77 },
    { 0x6e5d072f, 76 },
    { 0, 0 },
    { 0x69bd0731, 68 },
    { 0, 0 },
    { 0xb8d0733, 76 },
    { 0x2c5d0734, 33 },
    { 0x352d0734, 93 },
    { 0, 0 },
    { 0x738d0737, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d073f, 76 },
    { 0x407d0740, 43 },
    { 0x248d0741, 33 },
    { 0x54ed0742, 23 },
    { 0, 0 },
    { 0x830d0744, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d0749, 61 },
    { 0xa3cd0749, 33 },
    { 0, 0 },
    { 0x63bd074c, 23 },
    { 0, 0 },
    { 0xaa0d074e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d0752, 23 },
    { 0x4f2d0753, 91 },
    { 0x3a3d0753, 95 },
    { 0xd01d0755, 76 },
    { 0, 0 },
    { 0x4d9d0757, 98 },
    { 0, 0 },
    { 0x805d0759, 90 },
    { 0xe3fd0759, 90 },
    { 0, 0 },
    { 0xcdd075c, 75 },
    { 0xbd2d075d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55d0760, 98 },
    { 0x508d0761, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608d0768, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd076e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x266d0771, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87d077a, 33 },
    { 0x72d077a, 53 },
    { 0x35fd077c, 66 },
    { 0x710d077c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cd0784, 91 },
    { 0x867d0785, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99d0788, 61 },
    { 0x85bd0789, 98 },
    { 0x932d0789, 55 },
    { 0xde3d078b, 61 },
    { 0x157d078a, 85 },
    { 0x5d2d078d, 33 },
    { 0xbd3d078a, 99 },
    { 0x9e9d078b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8d0792, 34 },
    { 0x53ad0793, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0798, 98 },
    { 0x4c3d0799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91d079c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d079f, 84 },
    { 0, 0 },
    { 0xae3d07a1, 100 },
    { 0x10d07a1, 93 },
    { 0x810d07a3, 95 },
    { 0x25d07a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d07a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d07ae, 78 },
    { 0x874d07ae, 61 },
    { 0xfa3d07af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d07b5, 98 },
    { 0, 0 },
    { 0x823d07b7, 84 },
    { 0, 0 },
    { 0x1bdd07b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4d07bd, 46 },
    { 0, 0 },
    { 0x487d07bf, 78 },
    { 0x9edd07c0, 33 },
    { 0x592d07c1, 95 },
    { 0x127d07bf, 36 },
    { 0x1b1d07c3, 90 },
    { 0x61bd07c4, 90 },
    { 0xbdcd07c3, 33 },
    { 0xd5cd07c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d07cc, 33 },
    { 0, 0 },
    { 0x86bd07ce, 90 },
    { 0x487d07ce, 33 },
    { 0, 0 },
    { 0x7b6d07d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd07d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d07e1, 43 },
    { 0x1d07e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd07e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66d07ed, 95 },
    { 0x11cd07ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad07fe, 23 },
    { 0xe41d07ff, 84 },
    { 0, 0 },
    { 0x46ad0801, 90 },
    { 0xd0d0802, 85 },
    { 0, 0 },
    { 0x2d2d0804, 33 },
    { 0xf02d0805, 61 },
    { 0xa5dd0806, 33 },
    { 0x4cdd0805, 33 },
    { 0x13fd0808, 98 },
    { 0xf6bd0808, 98 },
    { 0xecfd080a, 95 },
    { 0x3a6d080a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d0813, 95 },
    { 0xffd0813, 98 },
    { 0x71bd0815, 87 },
    { 0, 0 },
    { 0x635d0817, 77 },
    { 0, 0 },
    { 0x5ded0819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed081d, 84 },
    { 0x113d081e, 84 },
    { 0xf2ad081f, 33 },
    { 0x9d6d081d, 90 },
    { 0x661d0821, 43 },
    { 0xe82d0822, 55 },
    { 0xc42d0822, 77 },
    { 0, 0 },
    { 0x78fd0825, 43 },
    { 0x7c0d0825, 98 },
    { 0x588d0827, 68 },
    { 0xb34d0827, 95 },
    { 0x248d0827, 61 },
    { 0x897d082a, 77 },
    { 0x75fd082a, 75 },
    { 0xea5d082c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ad0833, 33 },
    { 0x943d0834, 61 },
    { 0x93cd0834, 99 },
    { 0xb2d0836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5d0839, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd083c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359d0844, 23 },
    { 0x6dfd0844, 91 },
    { 0xc60d0844, 33 },
    { 0x9d5d0845, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ed084a, 98 },
    { 0x3bed084a, 90 },
    { 0, 0 },
    { 0xae2d084d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cd0851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d0855, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8d085a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad086b, 33 },
    { 0xfa3d086b, 65 },
    { 0x132d086c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdd0873, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15d0877, 33 },
    { 0x620d0878, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bd087b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced087e, 66 },
    { 0, 0 },
    { 0x581d0880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0884, 95 },
    { 0xc25d0885, 98 },
    { 0xc05d0884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2d088d, 23 },
    { 0x62cd088e, 76 },
    { 0x1fd088d, 90 },
    { 0x91d0890, 100 },
    { 0x47d0891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x548d0894, 43 },
    { 0, 0 },
    { 0xdeed0896, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689d089a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d08a5, 49 },
    { 0, 0 },
    { 0x37dd08a7, 33 },
    { 0xafdd08a8, 67 },
    { 0xeb7d08a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed08ac, 95 },
    { 0xbe7d08ad, 46 },
    { 0x337d08ae, 65 },
    { 0x1c0d08af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acd08b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670d08bf, 90 },
    { 0x99cd08bf, 68 },
    { 0, 0 },
    { 0x805d08c2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aad08c5, 95 },
    { 0, 0 },
    { 0x8cdd08c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d08cc, 55 },
    { 0xaed08cc, 55 },
    { 0x958d08cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x942d08d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efd08da, 61 },
    { 0xf9cd08db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed08e0, 85 },
    { 0xd8cd08e1, 93 },
    { 0x64ad08e1, 98 },
    { 0xe2cd08e2, 33 },
    { 0xb5ad08e4, 98 },
    { 0x31ed08e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d08e8, 33 },
    { 0x18fd08e8, 95 },
    { 0x421d08e9, 98 },
    { 0x7a3d08eb, 33 },
    { 0, 0 },
    { 0x8f6d08ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749d08f4, 34 },
    { 0x77fd08f5, 95 },
    { 0x978d08f6, 90 },
    { 0x54dd08f6, 76 },
    { 0x3c9d08f8, 33 },
    { 0xfcad08f6, 90 },
    { 0xebed08fa, 53 },
    { 0x607d08fa, 33 },
    { 0xe8ad08f5, 95 },
    { 0xf1dd08f5, 55 },
    { 0xf32d08fe, 78 },
    { 0xaa9d08fe, 95 },
    { 0x33ed08ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06d0903, 61 },
    { 0x7aed0904, 61 },
    { 0xe36d0904, 91 },
    { 0x362d0904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacd0913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5d0917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d091e, 61 },
    { 0x2aad091e, 38 },
    { 0xb7ed091e, 98 },
    { 0x656d0921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cd0924, 99 },
    { 0x715d0925, 40 },
    { 0, 0 },
    { 0x33d0927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d092b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd092f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130d0933, 84 },
    { 0, 0 },
    { 0x484d0935, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d0939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d093e, 78 },
    { 0xef1d093e, 88 },
    { 0x3add0940, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd0943, 23 },
    { 0x15d0944, 100 },
    { 0x624d0945, 100 },
    { 0x8f8d0946, 80 },
    { 0xea5d0947, 87 },
    { 0, 0 },
    { 0xbd3d0949, 72 },
    { 0x6b8d094a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbed094f, 33 },
    { 0x27dd0950, 33 },
    { 0, 0 },
    { 0x449d0952, 34 },
    { 0, 0 },
    { 0x8abd0954, 33 },
    { 0, 0 },
    { 0xebad0956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cd095c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x421d095f, 33 },
    { 0xf8bd0960, 33 },
    { 0, 0 },
    { 0xf7bd0962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dd096b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd096e, 61 },
    { 0x507d096f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ed0972, 98 },
    { 0xaad0973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d0977, 23 },
    { 0xb09d0977, 23 },
    { 0, 0 },
    { 0xb68d097a, 100 },
    { 0x7e7d097a, 87 },
    { 0xec8d097b, 88 },
    { 0, 0 },
    { 0x811d097e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x242d0981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d0988, 61 },
    { 0xfead0988, 61 },
    { 0xd67d0988, 76 },
    { 0x51cd098a, 83 },
    { 0xac4d098c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144d0998, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39d09a3, 90 },
    { 0x2a9d09a4, 93 },
    { 0x373d09a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd09ac, 77 },
    { 0x6e3d09ad, 90 },
    { 0x310d09ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d09b6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd09bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd09c0, 34 },
    { 0x9aed09c1, 33 },
    { 0x5b0d09c2, 67 },
    { 0, 0 },
    { 0xa92d09c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad09c7, 98 },
    { 0xa15d09c7, 76 },
    { 0xd40d09c9, 99 },
    { 0xa5ed09ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ad09d1, 90 },
    { 0xf16d09d1, 93 },
    { 0xe0fd09d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd09d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688d09e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d09e6, 46 },
    { 0x3fed09e6, 61 },
    { 0x39dd09e6, 90 },
    { 0, 0 },
    { 0x482d09ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbd09f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d09f4, 89 },
    { 0xb5dd09f5, 98 },
    { 0x426d09f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d09f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d09fc, 46 },
    { 0xb92d09fd, 66 },
    { 0x6d3d09fd, 55 },
    { 0x9bbd09ff, 66 },
    { 0, 0 },
    { 0x767d0a01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd0a07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad0a0a, 46 },
    { 0, 0 },
    { 0xf2dd0a0c, 90 },
    { 0x157d0a0c, 10 },
    { 0xcd0d0a0e, 61 },
    { 0x327d0a0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5d0a16, 23 },
    { 0x820d0a17, 33 },
    { 0xc43d0a16, 52 },
    { 0x50dd0a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd0a29, 61 },
    { 0, 0 },
    { 0x3b9d0a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570d0a32, 61 },
    { 0x1f4d0a32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d0a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x889d0a3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdd0a40, 90 },
    { 0x466d0a41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8d0a47, 91 },
    { 0x3cd0a48, 23 },
    { 0, 0 },
    { 0x2aad0a4a, 36 },
    { 0, 0 },
    { 0xbbd0a4c, 88 },
    { 0x955d0a4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d0a51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabad0a5a, 23 },
    { 0x858d0a5a, 90 },
    { 0x474d0a5c, 61 },
    { 0x53ed0a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dd0a61, 75 },
    { 0x5dd0a61, 43 },
    { 0xc4cd0a63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d0a6a, 33 },
    { 0xeacd0a6b, 64 },
    { 0x6eed0a6c, 85 },
    { 0xdf7d0a6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd0a72, 88 },
    { 0, 0 },
    { 0xa6ed0a74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0a7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20d0a82, 100 },
    { 0x3cd0a83, 61 },
    { 0xb3cd0a84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885d0a88, 33 },
    { 0xc70d0a89, 36 },
    { 0, 0 },
    { 0xb36d0a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d0a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d0a9c, 93 },
    { 0xf58d0a9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5d0aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad0aa6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d0aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d0ab2, 76 },
    { 0x495d0ab3, 76 },
    { 0x11cd0ab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d0ac4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0d0ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188d0acd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d0ad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4d0adc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8d0adf, 33 },
    { 0x749d0ae0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49d0ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cd0ae8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d0aed, 85 },
    { 0x6a6d0aee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0af1, 95 },
    { 0xd5cd0af2, 23 },
    { 0, 0 },
    { 0x346d0af4, 98 },
    { 0x925d0af5, 83 },
    { 0x44bd0af6, 78 },
    { 0, 0 },
    { 0x11fd0af8, 95 },
    { 0x16dd0af9, 98 },
    { 0x454d0afa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5d0aff, 61 },
    { 0xde1d0aff, 43 },
    { 0, 0 },
    { 0xaead0b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d0b07, 90 },
    { 0, 0 },
    { 0x9ded0b09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d0b0d, 90 },
    { 0x63ad0b0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed0b11, 24 },
    { 0x85bd0b12, 76 },
    { 0x1b1d0b13, 95 },
    { 0, 0 },
    { 0xa15d0b15, 90 },
    { 0, 0 },
    { 0x71ad0b17, 61 },
    { 0xe53d0b18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d0b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10d0b2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ad0b35, 23 },
    { 0xe18d0b36, 90 },
    { 0x7d9d0b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355d0b3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d0b3e, 81 },
    { 0xae0d0b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fd0b43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d0b48, 93 },
    { 0x6a8d0b48, 76 },
    { 0x5c2d0b48, 33 },
    { 0x481d0b48, 23 },
    { 0xff6d0b4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdd0b50, 69 },
    { 0x6d1d0b51, 93 },
    { 0, 0 },
    { 0x70cd0b53, 90 },
    { 0, 0 },
    { 0xf57d0b55, 90 },
    { 0x87ad0b55, 80 },
    { 0x4f9d0b57, 74 },
    { 0x65ad0b56, 34 },
    { 0, 0 },
    { 0x766d0b5a, 33 },
    { 0, 0 },
    { 0xb4cd0b5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ad0b60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58d0b65, 23 },
    { 0x8add0b66, 98 },
    { 0x2dfd0b65, 52 },
    { 0xa50d0b66, 61 },
    { 0xa86d0b67, 52 },
    { 0x895d0b6a, 99 },
    { 0x88fd0b6b, 78 },
    { 0xc05d0b6b, 90 },
    { 0x8d4d0b6c, 90 },
    { 0xd30d0b6d, 76 },
    { 0xe6bd0b6f, 93 },
    { 0xb61d0b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111d0b74, 90 },
    { 0x5d5d0b75, 98 },
    { 0, 0 },
    { 0xb09d0b77, 78 },
    { 0x704d0b78, 52 },
    { 0x66ad0b77, 34 },
    { 0, 0 },
    { 0x8fbd0b7b, 61 },
    { 0xca4d0b7b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d0b86, 90 },
    { 0, 0 },
    { 0x2b0d0b88, 90 },
    { 0x808d0b89, 69 },
    { 0x77fd0b8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46d0b91, 90 },
    { 0x972d0b91, 55 },
    { 0x5a8d0b91, 33 },
    { 0xca1d0b93, 33 },
    { 0xd25d0b95, 87 },
    { 0x1c8d0b95, 99 },
    { 0x3e6d0b92, 34 },
    { 0, 0 },
    { 0xfacd0b99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5d0ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545d0ba6, 99 },
    { 0x9d8d0ba7, 66 },
    { 0, 0 },
    { 0xeb9d0ba9, 78 },
    { 0xd96d0baa, 46 },
    { 0x470d0bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5d0baf, 23 },
    { 0, 0 },
    { 0xe94d0bb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d0bba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8d0bbd, 33 },
    { 0xa8bd0bbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0bc3, 90 },
    { 0, 0 },
    { 0x6e8d0bc5, 52 },
    { 0xcb6d0bc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47d0bcb, 90 },
    { 0x18ed0bcc, 95 },
    { 0, 0 },
    { 0x6ebd0bce, 90 },
    { 0x3ded0bcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ed0bd5, 40 },
    { 0xc18d0bd6, 23 },
    { 0xc1ad0bd5, 99 },
    { 0x184d0bd5, 46 },
    { 0, 0 },
    { 0x8ddd0bda, 43 },
    { 0xd09d0bdb, 61 },
    { 0xb9dd0bdb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dd0be5, 95 },
    { 0x747d0be6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cd0bea, 61 },
    { 0xe40d0beb, 90 },
    { 0xc92d0beb, 98 },
    { 0, 0 },
    { 0x237d0bee, 90 },
    { 0, 0 },
    { 0xb5ad0bf0, 61 },
    { 0x806d0bf1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd0bf4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad0bfb, 22 },
    { 0x727d0bfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47d0bff, 76 },
    { 0xffbd0c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0c03, 90 },
    { 0x5c5d0c04, 33 },
    { 0xb93d0c05, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d0c08, 78 },
    { 0xc91d0c09, 33 },
    { 0x15bd0c0a, 55 },
    { 0, 0 },
    { 0x66cd0c0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d0c10, 46 },
    { 0x107d0c11, 33 },
    { 0, 0 },
    { 0x944d0c13, 77 },
    { 0x6f0d0c14, 38 },
    { 0xb74d0c15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d0c22, 95 },
    { 0x73d0c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d0c26, 78 },
    { 0x364d0c27, 23 },
    { 0xb58d0c26, 98 },
    { 0, 0 },
    { 0xed6d0c2a, 55 },
    { 0x6e0d0c2b, 77 },
    { 0x5d2d0c2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4d0c2f, 61 },
    { 0x25bd0c2f, 33 },
    { 0xca4d0c31, 55 },
    { 0x3a4d0c32, 23 },
    { 0x5ced0c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd0c37, 76 },
    { 0x4f7d0c38, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3d0c3b, 33 },
    { 0xe55d0c3c, 61 },
    { 0x543d0c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d0c42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3d0c49, 84 },
    { 0, 0 },
    { 0x137d0c4b, 90 },
    { 0x6bbd0c4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cd0c51, 76 },
    { 0x410d0c52, 78 },
    { 0xc7ed0c52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d0c56, 61 },
    { 0x12cd0c57, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ded0c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d0c5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad0c62, 34 },
    { 0xe56d0c63, 90 },
    { 0x795d0c62, 55 },
    { 0, 0 },
    { 0xcadd0c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114d0c6a, 91 },
    { 0x457d0c6b, 98 },
    { 0x484d0c6c, 33 },
    { 0x884d0c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ed0c77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0c7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d0c85, 90 },
    { 0, 0 },
    { 0x15dd0c87, 100 },
    { 0x6c5d0c88, 61 },
    { 0x707d0c88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2d0c92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d0c96, 55 },
    { 0xd19d0c96, 55 },
    { 0x44cd0c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d0c9d, 66 },
    { 0xdc6d0c9e, 34 },
    { 0xbb8d0c9f, 61 },
    { 0x1a9d0c9f, 61 },
    { 0x383d0ca0, 93 },
    { 0xebcd0c9e, 99 },
    { 0xea1d0ca3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccd0ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d0cb4, 46 },
    { 0x143d0cb5, 61 },
    { 0x493d0cb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d0cba, 52 },
    { 0xd07d0cba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd0cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3d0cc5, 85 },
    { 0x860d0cc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x236d0cc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded0ccd, 33 },
    { 0xceed0cce, 61 },
    { 0xb57d0ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d0cd6, 23 },
    { 0xaefd0cd7, 33 },
    { 0xdd5d0cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d0cdf, 90 },
    { 0xc7d0ce0, 88 },
    { 0x40d0ce1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9d0ce5, 61 },
    { 0, 0 },
    { 0xc99d0ce7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd0cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dd0cef, 46 },
    { 0x68cd0cf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d0cf6, 90 },
    { 0, 0 },
    { 0xef6d0cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd0cfb, 91 },
    { 0, 0 },
    { 0x7fed0cfd, 61 },
    { 0x68ad0cfe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979d0d05, 46 },
    { 0, 0 },
    { 0x444d0d07, 61 },
    { 0, 0 },
    { 0xfcfd0d09, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad0d11, 33 },
    { 0, 0 },
    { 0x33dd0d13, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efd0d17, 33 },
    { 0x64cd0d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d0d1c, 61 },
    { 0x263d0d1d, 90 },
    { 0x145d0d1e, 76 },
    { 0, 0 },
    { 0x4e4d0d20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d0d23, 67 },
    { 0, 0 },
    { 0xb37d0d25, 84 },
    { 0x64d0d26, 33 },
    { 0x3c6d0d25, 95 },
    { 0, 0 },
    { 0xce6d0d29, 34 },
    { 0xd90d0d2a, 61 },
    { 0x5ded0d2b, 23 },
    { 0x891d0d2c, 91 },
    { 0, 0 },
    { 0x928d0d2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d0d38, 90 },
    { 0, 0 },
    { 0xa68d0d3a, 23 },
    { 0xe6fd0d3b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d0d42, 23 },
    { 0xe4d0d43, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7d0d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd0d4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2d0d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d0d53, 34 },
    { 0x1eed0d54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabd0d5e, 33 },
    { 0, 0 },
    { 0x147d0d60, 90 },
    { 0x5c1d0d60, 69 },
    { 0, 0 },
    { 0x239d0d63, 42 },
    { 0x8c2d0d63, 92 },
    { 0x301d0d63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cd0d6d, 90 },
    { 0x7bed0d6e, 92 },
    { 0xbfcd0d6f, 17 },
    { 0x525d0d6f, 69 },
    { 0x997d0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39d0d78, 76 },
    { 0x73fd0d79, 89 },
    { 0x7bad0d78, 85 },
    { 0, 0 },
    { 0x7bbd0d7c, 98 },
    { 0xafd0d7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857d0d83, 81 },
    { 0xc19d0d84, 99 },
    { 0xde3d0d84, 33 },
    { 0, 0 },
    { 0x2b6d0d87, 36 },
    { 0, 0 },
    { 0x59d0d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d0d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cd0d93, 61 },
    { 0, 0 },
    { 0xcb5d0d95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d0d98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d0d9b, 90 },
    { 0xc25d0d9c, 91 },
    { 0xa98d0d9d, 23 },
    { 0xac9d0d9e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ad0da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cd0da6, 76 },
    { 0, 0 },
    { 0xbfd0da8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d0dab, 76 },
    { 0xcded0dac, 63 },
    { 0x9c4d0dab, 93 },
    { 0x27ed0dac, 90 },
    { 0, 0 },
    { 0x58bd0db0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd0db3, 90 },
    { 0, 0 },
    { 0x8bfd0db5, 99 },
    { 0, 0 },
    { 0x3abd0db7, 90 },
    { 0, 0 },
    { 0xa05d0db9, 66 },
    { 0xe6dd0dba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d0dc2, 87 },
    { 0xf82d0dc3, 77 },
    { 0x6d5d0dc4, 64 },
    { 0xb53d0dc3, 98 },
    { 0xa9d0dc6, 65 },
    { 0xae0d0dc7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cd0dcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ed0dd0, 68 },
    { 0x4bbd0dd1, 23 },
    { 0xe0dd0dd0, 68 },
    { 0x5c5d0dd3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ad0dd6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad0de0, 43 },
    { 0xd7ed0de0, 17 },
    { 0, 0 },
    { 0x7e0d0de3, 90 },
    { 0x77fd0de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dd0de9, 23 },
    { 0x775d0dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d0def, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4d0df2, 67 },
    { 0xd03d0df3, 61 },
    { 0x975d0df4, 90 },
    { 0x639d0df4, 95 },
    { 0xe94d0df2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74d0df9, 61 },
    { 0, 0 },
    { 0x83cd0dfb, 90 },
    { 0xc5ed0dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x397d0dff, 95 },
    { 0, 0 },
    { 0xea8d0e01, 83 },
    { 0x9d6d0e01, 95 },
    { 0, 0 },
    { 0xf34d0e04, 76 },
    { 0xb6bd0e05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9d0e09, 23 },
    { 0x8ad0e0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d0e0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17d0e10, 90 },
    { 0, 0 },
    { 0xca0d0e12, 97 },
    { 0, 0 },
    { 0x68ad0e14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bd0e18, 33 },
    { 0xcad0e18, 23 },
    { 0xb48d0e1a, 75 },
    { 0x240d0e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bd0e1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d0e27, 68 },
    { 0, 0 },
    { 0xcf7d0e29, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d0e2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ad0e32, 64 },
    { 0, 0 },
    { 0x665d0e34, 90 },
    { 0x71cd0e35, 90 },
    { 0xb5d0e36, 93 },
    { 0xa1bd0e35, 55 },
    { 0xc35d0e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38d0e3e, 87 },
    { 0x60fd0e3f, 23 },
    { 0xbeed0e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d0e47, 100 },
    { 0x512d0e47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d0e54, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d0e58, 23 },
    { 0x7b4d0e58, 98 },
    { 0x5d3d0e59, 95 },
    { 0xe46d0e5b, 90 },
    { 0x506d0e5c, 61 },
    { 0xcc4d0e5a, 61 },
    { 0x948d0e5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3d0e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x248d0e64, 33 },
    { 0, 0 },
    { 0x256d0e66, 99 },
    { 0, 0 },
    { 0xc2ad0e68, 61 },
    { 0x55bd0e69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37d0e77, 33 },
    { 0x2ebd0e78, 61 },
    { 0x5e4d0e79, 61 },
    { 0, 0 },
    { 0x7afd0e7b, 61 },
    { 0xbded0e7b, 74 },
    { 0x265d0e7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d0e83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bd0e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd0e8d, 84 },
    { 0xb00d0e8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ad0e94, 52 },
    { 0, 0 },
    { 0xe4fd0e96, 90 },
    { 0xf15d0e97, 90 },
    { 0xf52d0e98, 61 },
    { 0, 0 },
    { 0x7fbd0e9a, 95 },
    { 0xa8dd0e9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cd0e9e, 87 },
    { 0, 0 },
    { 0xf43d0ea0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d0ea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d0ea9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d0eac, 90 },
    { 0x488d0eac, 23 },
    { 0xa9ad0eac, 61 },
    { 0, 0 },
    { 0xb79d0eb0, 34 },
    { 0x104d0eb0, 61 },
    { 0x48ad0eb2, 95 },
    { 0, 0 },
    { 0xb47d0eb4, 23 },
    { 0x8a8d0eb5, 61 },
    { 0x632d0eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d0eb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89d0ebd, 33 },
    { 0x14d0ebe, 90 },
    { 0xb43d0ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d0ec3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccd0ec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd0ecc, 76 },
    { 0xb15d0ecd, 90 },
    { 0x38cd0ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d0ed3, 87 },
    { 0, 0 },
    { 0xc81d0ed5, 99 },
    { 0xe69d0ed5, 65 },
    { 0xe0d0ed7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ed0edd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabd0ee1, 85 },
    { 0, 0 },
    { 0x72cd0ee3, 78 },
    { 0xe5cd0ee4, 87 },
    { 0x9b9d0ee3, 52 },
    { 0x4b9d0ee4, 78 },
    { 0xce4d0ee6, 33 },
    { 0, 0 },
    { 0x4b3d0ee9, 95 },
    { 0, 0 },
    { 0x3d0eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd0ef3, 93 },
    { 0x87dd0ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad0ef8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbd0efb, 61 },
    { 0x509d0efc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd0f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0d0f04, 90 },
    { 0xdaed0f05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd0f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dd0f10, 66 },
    { 0xdb9d0f10, 95 },
    { 0x91d0f12, 90 },
    { 0xac4d0f13, 90 },
    { 0xdedd0f10, 33 },
    { 0xe75d0f15, 33 },
    { 0xecbd0f16, 95 },
    { 0, 0 },
    { 0xf82d0f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1d0f1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x247d0f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d0f25, 76 },
    { 0x238d0f25, 95 },
    { 0x7dd0f25, 90 },
    { 0xda9d0f28, 61 },
    { 0xf04d0f29, 95 },
    { 0xc0bd0f28, 52 },
    { 0x1fbd0f27, 95 },
    { 0xa83d0f29, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d0f30, 64 },
    { 0x873d0f31, 61 },
    { 0, 0 },
    { 0x451d0f33, 90 },
    { 0xe88d0f33, 23 },
    { 0x42fd0f34, 90 },
    { 0x795d0f36, 98 },
    { 0, 0 },
    { 0x8e5d0f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd0f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd0f3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fd0f41, 67 },
    { 0xf6dd0f42, 61 },
    { 0x342d0f43, 33 },
    { 0, 0 },
    { 0x98fd0f45, 90 },
    { 0x591d0f46, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3d0f49, 90 },
    { 0, 0 },
    { 0x446d0f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6d0f4f, 90 },
    { 0xf93d0f50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404d0f54, 61 },
    { 0x7cd0f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd0f5d, 88 },
    { 0, 0 },
    { 0xa79d0f5f, 34 },
    { 0xb55d0f60, 52 },
    { 0, 0 },
    { 0xf1dd0f62, 23 },
    { 0x868d0f62, 61 },
    { 0xcffd0f64, 46 },
    { 0x2ead0f62, 67 },
    { 0, 0 },
    { 0xabbd0f67, 85 },
    { 0x198d0f68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1d0f6b, 61 },
    { 0, 0 },
    { 0x826d0f6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d0f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d0f73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d0f77, 33 },
    { 0x3ead0f78, 90 },
    { 0x3ced0f79, 33 },
    { 0xa25d0f7a, 91 },
    { 0xaccd0f78, 90 },
    { 0x9aed0f7a, 91 },
    { 0x296d0f7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fd0f82, 98 },
    { 0xeccd0f82, 90 },
    { 0x907d0f84, 96 },
    { 0xe6ed0f85, 76 },
    { 0x73fd0f85, 55 },
    { 0xe9bd0f83, 90 },
    { 0xcc7d0f86, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0f8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad0f8e, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cd0f91, 67 },
    { 0x778d0f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ad0f99, 90 },
    { 0x417d0f99, 99 },
    { 0xe05d0f9a, 93 },
    { 0x276d0f9c, 98 },
    { 0x98ed0f9c, 78 },
    { 0x2cfd0f9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x240d0fa1, 100 },
    { 0x212d0fa1, 98 },
    { 0xddad0fa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d0fa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dd0fae, 99 },
    { 0x1d9d0fae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d0fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3d0fb7, 76 },
    { 0, 0 },
    { 0x325d0fb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d0fbc, 61 },
    { 0xc45d0fbd, 90 },
    { 0x397d0fbe, 96 },
    { 0, 0 },
    { 0xb2ad0fc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d0fc8, 93 },
    { 0xeced0fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9d0fd0, 98 },
    { 0xf3bd0fd0, 69 },
    { 0x1fcd0fd2, 39 },
    { 0x3ced0fd3, 34 },
    { 0x6e2d0fd4, 76 },
    { 0x36dd0fd5, 90 },
    { 0xa33d0fd3, 90 },
    { 0, 0 },
    { 0xe2ad0fd8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850d0fdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d0fe1, 83 },
    { 0x2cad0fe2, 90 },
    { 0x40fd0fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd0fe6, 23 },
    { 0xfc5d0fe7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d0fea, 93 },
    { 0, 0 },
    { 0x8b6d0fec, 90 },
    { 0x42bd0fed, 34 },
    { 0, 0 },
    { 0xf7ad0fef, 90 },
    { 0, 0 },
    { 0x19fd0ff1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed0ff4, 52 },
    { 0x8a4d0ff4, 61 },
    { 0, 0 },
    { 0xefdd0ff7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x798d0ffa, 90 },
    { 0xdfbd0ffb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683d0fff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1002, 81 },
    { 0xdded1003, 46 },
    { 0x41cd1004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91d1007, 76 },
    { 0x2ced1008, 52 },
    { 0x564d1007, 61 },
    { 0x51bd1009, 40 },
    { 0x1d2d1007, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bd1010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1d1016, 61 },
    { 0x68fd1017, 88 },
    { 0xfe3d1017, 67 },
    { 0, 0 },
    { 0x2bdd101a, 90 },
    { 0xa70d101a, 61 },
    { 0, 0 },
    { 0x817d101d, 93 },
    { 0x170d101e, 75 },
    { 0xce9d101e, 65 },
    { 0xa0bd101f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953d1029, 76 },
    { 0x826d102a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d1031, 23 },
    { 0, 0 },
    { 0xcccd1033, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d1036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x909d1039, 85 },
    { 0xf0dd103a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd103e, 34 },
    { 0xf1bd103f, 98 },
    { 0, 0 },
    { 0xe46d1041, 98 },
    { 0xf86d1042, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d1045, 93 },
    { 0x8edd1046, 98 },
    { 0xb8ad1047, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ed104b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bd1050, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed1055, 40 },
    { 0x742d1055, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d1063, 76 },
    { 0, 0 },
    { 0x909d1065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d1068, 67 },
    { 0, 0 },
    { 0x4fed106a, 61 },
    { 0, 0 },
    { 0x9d7d106c, 61 },
    { 0xd4fd106d, 67 },
    { 0, 0 },
    { 0x3f6d106f, 46 },
    { 0xcb8d1070, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d1075, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ded1079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd107c, 43 },
    { 0, 0 },
    { 0xb6cd107e, 84 },
    { 0x28bd107e, 90 },
    { 0xbedd1080, 95 },
    { 0x43cd1080, 90 },
    { 0xa36d1082, 33 },
    { 0x3acd1083, 90 },
    { 0xed5d1084, 40 },
    { 0xf85d1084, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51d108c, 87 },
    { 0xa92d108d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d1090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dd1093, 61 },
    { 0xec8d1094, 95 },
    { 0, 0 },
    { 0x9abd1096, 34 },
    { 0, 0 },
    { 0xc58d1098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d109e, 91 },
    { 0x87ed109e, 69 },
    { 0, 0 },
    { 0xe4dd10a1, 90 },
    { 0, 0 },
    { 0xfa2d10a3, 95 },
    { 0xc74d10a4, 61 },
    { 0xfb4d10a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cd10af, 90 },
    { 0x552d10b0, 69 },
    { 0xa34d10af, 85 },
    { 0x9acd10b2, 77 },
    { 0xc13d10b3, 76 },
    { 0x3c2d10b1, 34 },
    { 0, 0 },
    { 0x342d10b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338d10be, 90 },
    { 0xa2fd10be, 90 },
    { 0x510d10be, 61 },
    { 0x14ad10c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d10c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972d10c7, 23 },
    { 0, 0 },
    { 0x489d10c9, 55 },
    { 0xf50d10c9, 81 },
    { 0x708d10cb, 55 },
    { 0x3b4d10c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d10cf, 81 },
    { 0x5ced10d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ed10d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x361d10dc, 93 },
    { 0x688d10dd, 76 },
    { 0, 0 },
    { 0x18fd10df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8d10e4, 61 },
    { 0, 0 },
    { 0xb7fd10e6, 96 },
    { 0x466d10e7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3d10ee, 55 },
    { 0x7a8d10ef, 90 },
    { 0x874d10f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ad10f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cd1100, 61 },
    { 0, 0 },
    { 0xdcbd1102, 46 },
    { 0x2f5d1103, 78 },
    { 0, 0 },
    { 0x67bd1105, 61 },
    { 0xc0dd1106, 77 },
    { 0xba2d1107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d1110, 95 },
    { 0xc52d1111, 61 },
    { 0x81dd1110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d1116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cd1119, 100 },
    { 0x2d7d1119, 95 },
    { 0x7f6d111b, 66 },
    { 0xc7ad111c, 43 },
    { 0, 0 },
    { 0x6aad111e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d1122, 23 },
    { 0xd93d1123, 43 },
    { 0x970d1124, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1127, 34 },
    { 0, 0 },
    { 0xd9dd1129, 90 },
    { 0x2bed112a, 33 },
    { 0xd44d112b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add112e, 46 },
    { 0x583d112f, 52 },
    { 0, 0 },
    { 0xca5d1131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x673d1134, 68 },
    { 0xb37d1135, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed113b, 38 },
    { 0xac8d113b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9d1146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d114a, 76 },
    { 0x98cd114b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd1152, 90 },
    { 0xa35d1152, 61 },
    { 0, 0 },
    { 0xf46d1155, 98 },
    { 0x381d1156, 84 },
    { 0x7f2d1157, 90 },
    { 0x6fad1157, 76 },
    { 0x438d1157, 95 },
    { 0, 0 },
    { 0x1a5d115b, 91 },
    { 0, 0 },
    { 0xd0bd115d, 77 },
    { 0, 0 },
    { 0xdb4d115f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d1165, 76 },
    { 0x335d1166, 98 },
    { 0xb1d1167, 61 },
    { 0x43cd1168, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad116d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd1170, 90 },
    { 0x505d1171, 85 },
    { 0x323d1171, 90 },
    { 0x7e8d1171, 46 },
    { 0xd8ad1173, 90 },
    { 0xc80d1175, 43 },
    { 0, 0 },
    { 0x1f8d1177, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d1183, 23 },
    { 0, 0 },
    { 0xcccd1185, 34 },
    { 0x64ad1186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffd118d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d1191, 77 },
    { 0x5bbd1192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d1197, 43 },
    { 0x4b0d1198, 66 },
    { 0, 0 },
    { 0x8bcd119a, 98 },
    { 0, 0 },
    { 0xd17d119c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d11a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9d11a6, 76 },
    { 0x6dd11a7, 61 },
    { 0x7d7d11a7, 90 },
    { 0x63ad11a9, 34 },
    { 0x7ed11a8, 23 },
    { 0, 0 },
    { 0xa55d11ac, 33 },
    { 0x33bd11ad, 43 },
    { 0x539d11ad, 61 },
    { 0, 0 },
    { 0xfb4d11b0, 95 },
    { 0xc3cd11b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d11bc, 76 },
    { 0, 0 },
    { 0x9d0d11be, 34 },
    { 0x406d11bf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d11c2, 46 },
    { 0xdafd11c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d11ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad11cf, 93 },
    { 0x977d11cf, 95 },
    { 0, 0 },
    { 0x9b5d11d2, 95 },
    { 0x99ad11d3, 65 },
    { 0, 0 },
    { 0xed8d11d5, 66 },
    { 0x30cd11d5, 23 },
    { 0x6a7d11d7, 77 },
    { 0x6d3d11d7, 98 },
    { 0x5aed11d5, 23 },
    { 0x6c1d11d8, 46 },
    { 0xc28d11d5, 98 },
    { 0, 0 },
    { 0x6a7d11dd, 33 },
    { 0xd42d11de, 93 },
    { 0x91bd11df, 90 },
    { 0x3d11df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd11e3, 78 },
    { 0x988d11e4, 17 },
    { 0xae5d11e5, 76 },
    { 0xbdfd11e6, 93 },
    { 0, 0 },
    { 0x6bdd11e8, 61 },
    { 0x5bfd11e8, 76 },
    { 0x62fd11e8, 61 },
    { 0, 0 },
    { 0xfa5d11ec, 98 },
    { 0, 0 },
    { 0xb81d11ee, 98 },
    { 0x51ed11ef, 33 },
    { 0xc1bd11ef, 33 },
    { 0x366d11ee, 76 },
    { 0x54d11f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd11f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35d11f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d11fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d11fe, 91 },
    { 0xf16d11fe, 46 },
    { 0x1f4d1200, 98 },
    { 0xf2fd1201, 90 },
    { 0x9ced11fe, 65 },
    { 0xbf0d1203, 76 },
    { 0x801d1203, 95 },
    { 0xed6d1205, 55 },
    { 0x239d1206, 93 },
    { 0xe4ad1206, 95 },
    { 0xbc8d1208, 98 },
    { 0xbc1d1201, 33 },
    { 0x5c6d120a, 80 },
    { 0xc30d1205, 40 },
    { 0xec7d1201, 76 },
    { 0x35bd120d, 34 },
    { 0x4b0d120c, 95 },
    { 0x3c0d1205, 61 },
    { 0x43d1210, 87 },
    { 0xeeed120c, 90 },
    { 0x490d1205, 66 },
    { 0x38d1207, 90 },
    { 0x220d1214, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ed1219, 46 },
    { 0, 0 },
    { 0x4d2d121b, 61 },
    { 0, 0 },
    { 0x432d121d, 90 },
    { 0x382d121d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32d1221, 61 },
    { 0x2c0d1222, 93 },
    { 0x82dd1222, 55 },
    { 0x45d1224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1d122a, 46 },
    { 0xad8d122b, 86 },
    { 0x763d122c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70d1231, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2d1239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d123d, 23 },
    { 0x328d123e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792d124a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x409d124d, 34 },
    { 0x21dd124e, 98 },
    { 0x9cd124f, 55 },
    { 0xde5d1250, 61 },
    { 0x313d124f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd1256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fd1259, 95 },
    { 0, 0 },
    { 0x16dd125b, 93 },
    { 0xbafd125c, 33 },
    { 0x2c1d125d, 61 },
    { 0xc9d125d, 96 },
    { 0xab3d125d, 90 },
    { 0x5e0d125f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0d1266, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d126c, 98 },
    { 0xed8d126c, 23 },
    { 0xdfed126e, 99 },
    { 0, 0 },
    { 0x511d1270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d1274, 81 },
    { 0, 0 },
    { 0xa78d1276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0d1279, 95 },
    { 0xeb5d127a, 61 },
    { 0xdaad127a, 33 },
    { 0, 0 },
    { 0xc4cd127d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2d1284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd1288, 87 },
    { 0, 0 },
    { 0x263d128a, 23 },
    { 0, 0 },
    { 0x890d128c, 68 },
    { 0x23ed128d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed1295, 99 },
    { 0xe71d1296, 46 },
    { 0, 0 },
    { 0xc87d1298, 23 },
    { 0, 0 },
    { 0x5d0d129a, 95 },
    { 0xa42d129b, 77 },
    { 0, 0 },
    { 0x879d129d, 90 },
    { 0x567d129e, 92 },
    { 0, 0 },
    { 0x5f8d12a0, 95 },
    { 0x33ed12a0, 90 },
    { 0xa15d12a0, 76 },
    { 0x28cd12a0, 52 },
    { 0, 0 },
    { 0x4d0d12a5, 99 },
    { 0x3d4d12a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336d12aa, 61 },
    { 0, 0 },
    { 0x48ad12ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12d12b5, 85 },
    { 0x3c2d12b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d12b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16d12bd, 33 },
    { 0, 0 },
    { 0xa3ed12bf, 100 },
    { 0, 0 },
    { 0x96dd12c1, 98 },
    { 0xb41d12c1, 87 },
    { 0x7d4d12c3, 100 },
    { 0, 0 },
    { 0x2b5d12c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ed12c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd12cb, 46 },
    { 0xceed12cc, 98 },
    { 0xbecd12cc, 33 },
    { 0, 0 },
    { 0x29d12cf, 61 },
    { 0, 0 },
    { 0xf59d12d1, 61 },
    { 0x38fd12d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd12d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d12da, 65 },
    { 0x89fd12db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdd12df, 76 },
    { 0xa6ed12e0, 49 },
    { 0x57bd12e0, 66 },
    { 0xd9bd12df, 61 },
    { 0xca3d12e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd12e7, 61 },
    { 0xc3cd12e8, 98 },
    { 0, 0 },
    { 0xf0d12ea, 93 },
    { 0xefdd12eb, 52 },
    { 0x192d12ec, 95 },
    { 0x6d5d12ed, 61 },
    { 0xed3d12eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3d12f4, 52 },
    { 0x729d12f5, 95 },
    { 0xcf9d12f6, 95 },
    { 0x307d12f7, 33 },
    { 0xd8fd12f8, 76 },
    { 0x794d12f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd12fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd12ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d1308, 23 },
    { 0x7ddd1308, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fd130d, 33 },
    { 0x4fcd130e, 87 },
    { 0x8b5d130f, 55 },
    { 0x28d130d, 61 },
    { 0x42d1311, 76 },
    { 0, 0 },
    { 0xfa2d1313, 90 },
    { 0xcfed1313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75d131d, 83 },
    { 0, 0 },
    { 0xd25d131f, 33 },
    { 0x81cd1320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d1323, 23 },
    { 0x7cfd1324, 98 },
    { 0x3bad1323, 75 },
    { 0x199d1326, 43 },
    { 0x2a3d1327, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebd132c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911d1330, 61 },
    { 0x30fd1330, 33 },
    { 0, 0 },
    { 0x16fd1333, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8d1339, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18d1343, 61 },
    { 0, 0 },
    { 0x952d1345, 61 },
    { 0xab4d1346, 101 },
    { 0x523d1347, 55 },
    { 0xc60d1348, 23 },
    { 0, 0 },
    { 0xb50d134a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d1353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d1357, 90 },
    { 0xfe7d1358, 66 },
    { 0x3ffd1359, 98 },
    { 0x428d1358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d135d, 43 },
    { 0, 0 },
    { 0x21fd135f, 67 },
    { 0, 0 },
    { 0x44ed1361, 78 },
    { 0xc39d1362, 61 },
    { 0x25dd1362, 98 },
    { 0xbfd1362, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afd1368, 90 },
    { 0x6f1d1369, 99 },
    { 0x12ad136a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1371, 61 },
    { 0xd8ad1371, 43 },
    { 0x8dbd1371, 33 },
    { 0, 0 },
    { 0xb89d1375, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd1379, 23 },
    { 0x55ed1379, 43 },
    { 0xee3d1379, 95 },
    { 0, 0 },
    { 0xb66d137d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47d1382, 85 },
    { 0xaffd1383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d1387, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d138d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd1392, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fd1395, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d139d, 95 },
    { 0x837d139e, 98 },
    { 0, 0 },
    { 0x7ed13a0, 87 },
    { 0xcc2d13a1, 76 },
    { 0, 0 },
    { 0x725d13a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ead13a6, 23 },
    { 0, 0 },
    { 0x8afd13a8, 61 },
    { 0x48d13a9, 90 },
    { 0x1ded13aa, 23 },
    { 0, 0 },
    { 0x580d13ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2d13b1, 33 },
    { 0x3add13b1, 90 },
    { 0x823d13b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ead13b7, 90 },
    { 0, 0 },
    { 0x1e9d13b9, 87 },
    { 0x3add13b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7d13bd, 34 },
    { 0xe7bd13be, 66 },
    { 0x9a4d13bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ed13c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d13c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ad13cc, 52 },
    { 0, 0 },
    { 0x20fd13ce, 33 },
    { 0x10dd13cf, 91 },
    { 0, 0 },
    { 0x1d13d1, 33 },
    { 0, 0 },
    { 0x167d13d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x150d13d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ed13db, 90 },
    { 0, 0 },
    { 0x1a8d13dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x231d13e0, 55 },
    { 0x7fd13e0, 33 },
    { 0x42fd13e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800d13e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92d13f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681d13fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead1401, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8d1405, 61 },
    { 0x23bd1406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d140b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bd140e, 95 },
    { 0xe6cd140e, 43 },
    { 0x530d1410, 90 },
    { 0, 0 },
    { 0x722d1412, 90 },
    { 0, 0 },
    { 0xae3d1414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7d1417, 93 },
    { 0xea4d1418, 90 },
    { 0, 0 },
    { 0x7eed141a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198d1421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15d1424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcd1432, 91 },
    { 0, 0 },
    { 0xa54d1434, 63 },
    { 0xdf1d1435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4d143c, 90 },
    { 0x2acd143c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x163d1440, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd1447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cd144e, 100 },
    { 0, 0 },
    { 0xf87d1450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdd1454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd1457, 61 },
    { 0, 0 },
    { 0x37ad1459, 95 },
    { 0xa0d145a, 84 },
    { 0, 0 },
    { 0x89bd145c, 95 },
    { 0xf15d145d, 93 },
    { 0xb38d145e, 91 },
    { 0x299d145f, 33 },
    { 0xdb2d145c, 34 },
    { 0xc73d1461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd1467, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d146a, 61 },
    { 0, 0 },
    { 0x891d146c, 98 },
    { 0, 0 },
    { 0xad4d146e, 23 },
    { 0, 0 },
    { 0x3fcd1470, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad1474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ced147a, 39 },
    { 0x13cd147a, 90 },
    { 0, 0 },
    { 0xc73d147d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8d1480, 33 },
    { 0xa12d1481, 43 },
    { 0, 0 },
    { 0x942d1483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d1486, 33 },
    { 0xf43d1486, 68 },
    { 0x9a4d1487, 49 },
    { 0x647d1489, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60d1490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151d1496, 23 },
    { 0, 0 },
    { 0x3e9d1498, 94 },
    { 0, 0 },
    { 0xd55d149a, 95 },
    { 0xd1d149b, 17 },
    { 0x231d149b, 95 },
    { 0xdcd149d, 90 },
    { 0x738d149e, 23 },
    { 0xf23d149e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7d14a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbad14ac, 93 },
    { 0x888d14ac, 90 },
    { 0x92cd14ae, 23 },
    { 0x825d14af, 90 },
    { 0x90dd14b0, 66 },
    { 0x458d14b1, 46 },
    { 0xcd2d14b0, 95 },
    { 0, 0 },
    { 0xf75d14b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ead14ba, 98 },
    { 0, 0 },
    { 0x463d14bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d14c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d14cb, 95 },
    { 0xaed14cc, 61 },
    { 0xd27d14cc, 99 },
    { 0, 0 },
    { 0x759d14cf, 76 },
    { 0x5d8d14d0, 55 },
    { 0x88d14d1, 61 },
    { 0x185d14d2, 88 },
    { 0x2f6d14d2, 23 },
    { 0x667d14d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79d14d7, 75 },
    { 0xf98d14d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cd14dc, 88 },
    { 0x608d14dc, 87 },
    { 0xf3ad14de, 55 },
    { 0x904d14dc, 98 },
    { 0x596d14e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530d14e9, 33 },
    { 0xe9cd14e9, 81 },
    { 0xcf4d14eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89d14ef, 33 },
    { 0x91ad14f0, 61 },
    { 0xcd7d14f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x323d14f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d14f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962d14fb, 43 },
    { 0x3e9d14fc, 95 },
    { 0x5cfd14fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd1505, 98 },
    { 0xea7d1505, 90 },
    { 0x6d6d1507, 77 },
    { 0x66bd1508, 61 },
    { 0x5ecd1505, 90 },
    { 0xb3d150a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bd150f, 99 },
    { 0x784d1510, 46 },
    { 0x5c5d1511, 90 },
    { 0xdadd150f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507d1516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdd1519, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dd151d, 90 },
    { 0x894d151d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d1522, 33 },
    { 0xed5d1522, 90 },
    { 0x3e4d1524, 87 },
    { 0x930d1525, 23 },
    { 0xfbfd1524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3d1529, 33 },
    { 0xb5bd152a, 77 },
    { 0x1e3d152b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cd1532, 98 },
    { 0xcecd1532, 61 },
    { 0x64cd1533, 95 },
    { 0, 0 },
    { 0xce8d1536, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898d153b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1544, 33 },
    { 0x4b0d1544, 82 },
    { 0xa58d1546, 23 },
    { 0xafbd1547, 42 },
    { 0xeaad1544, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3d154e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d1552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd1557, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8d155b, 98 },
    { 0x363d155b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74d155f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd1563, 61 },
    { 0, 0 },
    { 0xaf0d1565, 33 },
    { 0xad3d1565, 69 },
    { 0x5c8d1567, 93 },
    { 0x2e5d1568, 77 },
    { 0x6f6d1565, 98 },
    { 0xd61d156a, 67 },
    { 0xd4ed1567, 33 },
    { 0, 0 },
    { 0x9c9d156d, 61 },
    { 0, 0 },
    { 0x37ad156f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14d1572, 23 },
    { 0x586d1573, 33 },
    { 0x20ad1572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d1578, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2d157e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775d1588, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd1592, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d1598, 55 },
    { 0x878d1599, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdead159f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed15a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d15a8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fd15ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d15af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240d15c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd15c9, 23 },
    { 0x278d15ca, 61 },
    { 0xe01d15c9, 93 },
    { 0x63ed15cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cd15d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d15d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54d15e1, 34 },
    { 0, 0 },
    { 0x725d15e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38d15e7, 38 },
    { 0x8ecd15e8, 17 },
    { 0xd65d15e9, 90 },
    { 0xefdd15ea, 61 },
    { 0xee2d15e8, 87 },
    { 0xf9cd15ec, 78 },
    { 0, 0 },
    { 0x6ebd15ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149d15f2, 33 },
    { 0x74d15f2, 95 },
    { 0x8b0d15f4, 98 },
    { 0, 0 },
    { 0xf50d15f6, 46 },
    { 0x2a9d15f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67d15fb, 43 },
    { 0, 0 },
    { 0x6d8d15fd, 46 },
    { 0, 0 },
    { 0x1b9d15ff, 90 },
    { 0xbdbd1600, 100 },
    { 0x331d15ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dd1605, 34 },
    { 0xc42d1606, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d160b, 76 },
    { 0, 0 },
    { 0x72ad160d, 90 },
    { 0x77d160e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x159d1611, 98 },
    { 0x147d1612, 91 },
    { 0xb72d1612, 43 },
    { 0x9b5d1614, 34 },
    { 0x889d1615, 98 },
    { 0xe69d1614, 99 },
    { 0x84ed1617, 90 },
    { 0x553d1618, 23 },
    { 0xcc0d1618, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712d1622, 98 },
    { 0xbf1d1622, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d1628, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4d162c, 23 },
    { 0x40ed162c, 84 },
    { 0x5a5d162e, 61 },
    { 0x640d162c, 33 },
    { 0, 0 },
    { 0x740d1631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d163b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d1642, 78 },
    { 0xa36d1642, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223d1648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d164c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cd164f, 55 },
    { 0xf34d1650, 90 },
    { 0xe27d1651, 23 },
    { 0, 0 },
    { 0x82ed1653, 61 },
    { 0xab0d1653, 76 },
    { 0xf64d1655, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d1659, 94 },
    { 0x7f1d165a, 90 },
    { 0xdced165b, 84 },
    { 0, 0 },
    { 0x8fbd165d, 23 },
    { 0, 0 },
    { 0x1e1d165f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed1668, 95 },
    { 0, 0 },
    { 0x3c1d166a, 36 },
    { 0x9f0d166b, 40 },
    { 0x927d166b, 80 },
    { 0xca0d166a, 76 },
    { 0x550d166b, 98 },
    { 0x11dd166d, 55 },
    { 0, 0 },
    { 0x298d1671, 91 },
    { 0x3e6d1671, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27d1675, 44 },
    { 0xa98d1676, 55 },
    { 0, 0 },
    { 0xc3ad1678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1d167b, 90 },
    { 0x5f2d167c, 33 },
    { 0xb0bd167c, 55 },
    { 0x75dd167e, 34 },
    { 0xd40d167f, 98 },
    { 0, 0 },
    { 0x535d1681, 95 },
    { 0x7d6d1682, 33 },
    { 0xbf0d1682, 98 },
    { 0xe24d1684, 55 },
    { 0xc11d1684, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcd168a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dd168e, 61 },
    { 0, 0 },
    { 0x1bed1690, 36 },
    { 0xabad1691, 95 },
    { 0xb96d1692, 61 },
    { 0x326d1693, 90 },
    { 0, 0 },
    { 0xd8ed1695, 33 },
    { 0x2ded1696, 90 },
    { 0x476d1695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482d16a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77d16a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d16ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bd16b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d16b5, 33 },
    { 0, 0 },
    { 0xb97d16b7, 23 },
    { 0x558d16b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d16bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4d16c2, 23 },
    { 0x7fdd16c2, 88 },
    { 0, 0 },
    { 0x485d16c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd16d0, 99 },
    { 0, 0 },
    { 0x363d16d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ed16d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d16d9, 33 },
    { 0xc27d16da, 93 },
    { 0x3cad16d9, 53 },
    { 0xdb9d16dc, 90 },
    { 0, 0 },
    { 0x9c6d16de, 61 },
    { 0x339d16de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead16e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d16eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d16ee, 61 },
    { 0, 0 },
    { 0x21cd16f0, 90 },
    { 0, 0 },
    { 0xf23d16f2, 55 },
    { 0x9e8d16f2, 81 },
    { 0xc32d16f3, 61 },
    { 0, 0 },
    { 0xbb1d16f6, 34 },
    { 0x5c2d16f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d16fb, 87 },
    { 0xeb1d16fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d1702, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cd1705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ad170d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd1711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d1714, 46 },
    { 0xd23d1715, 78 },
    { 0, 0 },
    { 0xeb1d1717, 98 },
    { 0x964d1717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8d171e, 90 },
    { 0, 0 },
    { 0xaf5d1720, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd1727, 98 },
    { 0, 0 },
    { 0x5c0d1729, 96 },
    { 0x74bd172a, 90 },
    { 0, 0 },
    { 0xad5d172c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784d172f, 90 },
    { 0, 0 },
    { 0x6ad1731, 97 },
    { 0x2bdd1731, 75 },
    { 0, 0 },
    { 0xf43d1734, 93 },
    { 0x3bcd1734, 33 },
    { 0x147d1735, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd1739, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9dd173f, 98 },
    { 0xefcd1740, 34 },
    { 0xd45d173f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d1744, 43 },
    { 0, 0 },
    { 0xe9d1746, 52 },
    { 0x271d1747, 34 },
    { 0x68ed1748, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aed174e, 61 },
    { 0xf71d174f, 52 },
    { 0xa52d1750, 90 },
    { 0x942d174f, 66 },
    { 0x7b0d1752, 33 },
    { 0xd29d1753, 34 },
    { 0, 0 },
    { 0x16bd1755, 90 },
    { 0, 0 },
    { 0x2ced1757, 95 },
    { 0xaefd1757, 78 },
    { 0xe42d1759, 66 },
    { 0, 0 },
    { 0xe9d175b, 61 },
    { 0xe62d175c, 46 },
    { 0x6cfd175c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad1760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d1766, 98 },
    { 0xd47d1767, 61 },
    { 0, 0 },
    { 0x589d1769, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85d176c, 98 },
    { 0xd97d176c, 23 },
    { 0x2f4d176c, 88 },
    { 0x981d176d, 43 },
    { 0xffed1770, 98 },
    { 0x95ed1771, 95 },
    { 0xd90d1772, 33 },
    { 0x9eed1772, 101 },
    { 0xa7ad1774, 43 },
    { 0x929d1773, 33 },
    { 0, 0 },
    { 0xff9d1777, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd177c, 33 },
    { 0x5cbd177d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d1780, 61 },
    { 0, 0 },
    { 0x9b6d1782, 100 },
    { 0x363d1783, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136d1788, 96 },
    { 0xef0d1789, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8d1790, 46 },
    { 0xd91d1791, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d1799, 46 },
    { 0xb27d1799, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cd179f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d17a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d17b3, 34 },
    { 0x686d17b3, 23 },
    { 0, 0 },
    { 0x50bd17b6, 53 },
    { 0, 0 },
    { 0x54fd17b8, 84 },
    { 0, 0 },
    { 0x4add17ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9d17c1, 91 },
    { 0x40bd17c1, 55 },
    { 0x130d17c3, 33 },
    { 0x77dd17c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd17c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfd17d0, 46 },
    { 0, 0 },
    { 0x23dd17d2, 95 },
    { 0x8bcd17d2, 95 },
    { 0x12bd17d2, 23 },
    { 0xfd1d17d2, 61 },
    { 0, 0 },
    { 0x860d17d7, 95 },
    { 0x463d17d8, 65 },
    { 0x56d17d9, 46 },
    { 0xdb8d17da, 85 },
    { 0x479d17db, 97 },
    { 0xc19d17d9, 91 },
    { 0xf07d17d9, 43 },
    { 0x2e5d17d7, 98 },
    { 0x799d17d7, 85 },
    { 0x98ed17da, 34 },
    { 0xd07d17e1, 34 },
    { 0x74d17e0, 95 },
    { 0x490d17da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dd17eb, 23 },
    { 0x920d17ec, 87 },
    { 0xb3dd17ec, 23 },
    { 0xc9fd17ed, 33 },
    { 0xbe0d17ef, 66 },
    { 0x1f7d17f0, 17 },
    { 0xc3bd17f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d17f4, 38 },
    { 0, 0 },
    { 0xc21d17f6, 77 },
    { 0xfd4d17f7, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d17fc, 91 },
    { 0xcd8d17fd, 23 },
    { 0xbfed17fe, 101 },
    { 0x5b1d17ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dd1806, 33 },
    { 0x6b8d1807, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cd180a, 95 },
    { 0x7a5d180b, 90 },
    { 0x539d180c, 23 },
    { 0xdc7d180c, 91 },
    { 0xb81d180e, 33 },
    { 0xbcad180a, 90 },
    { 0xaa3d1810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8d1813, 100 },
    { 0x8a7d1814, 43 },
    { 0, 0 },
    { 0x746d1816, 90 },
    { 0xcd2d1816, 95 },
    { 0x53d1818, 95 },
    { 0xf4d1816, 33 },
    { 0, 0 },
    { 0xd55d181b, 33 },
    { 0, 0 },
    { 0x885d181d, 23 },
    { 0x796d181d, 98 },
    { 0x52fd181f, 61 },
    { 0xd59d181e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad1828, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d182f, 56 },
    { 0, 0 },
    { 0xab2d1831, 19 },
    { 0xa08d1832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531d1838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53d183d, 55 },
    { 0x2cbd183d, 95 },
    { 0xd46d183d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d1844, 61 },
    { 0x1c9d1845, 55 },
    { 0xf2bd1846, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d1853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x459d1856, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65d185a, 61 },
    { 0xb6ad185b, 94 },
    { 0x7d2d185c, 55 },
    { 0x3ead185d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918d1868, 76 },
    { 0, 0 },
    { 0x55ed186a, 95 },
    { 0xe22d186a, 98 },
    { 0xe5cd186c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd1870, 98 },
    { 0x4cd1871, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1877, 23 },
    { 0xc4cd1878, 71 },
    { 0xe96d1879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd187c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129d1882, 53 },
    { 0x61cd1882, 61 },
    { 0xbb2d1884, 66 },
    { 0xe7dd1882, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1888, 23 },
    { 0x6bad1888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd1890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd1895, 61 },
    { 0xc8ad1896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad1899, 65 },
    { 0x9b3d189a, 76 },
    { 0, 0 },
    { 0xccbd189c, 78 },
    { 0x3aad189d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d18a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02d18aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccd18b0, 91 },
    { 0x504d18b0, 87 },
    { 0xc9fd18b1, 90 },
    { 0, 0 },
    { 0x652d18b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61d18b8, 34 },
    { 0, 0 },
    { 0x754d18ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406d18c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d18c9, 68 },
    { 0x7ffd18ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x551d18cd, 33 },
    { 0x62cd18cd, 93 },
    { 0, 0 },
    { 0xc45d18d0, 60 },
    { 0xd02d18d1, 98 },
    { 0x59ed18d2, 33 },
    { 0x872d18d3, 61 },
    { 0x70ad18d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7d18d7, 84 },
    { 0x76dd18d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bd18dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9d18e0, 66 },
    { 0, 0 },
    { 0x461d18e2, 33 },
    { 0x43d18e2, 61 },
    { 0xaa8d18e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d18e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459d18ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0d18f7, 34 },
    { 0x506d18f8, 78 },
    { 0xff1d18f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262d18fe, 91 },
    { 0x7add18fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x378d1902, 90 },
    { 0, 0 },
    { 0x5ddd1904, 76 },
    { 0x834d1904, 95 },
    { 0xdb0d1904, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1d1917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666d191b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd1920, 46 },
    { 0xa7cd1921, 99 },
    { 0x938d1922, 52 },
    { 0xc2fd1923, 98 },
    { 0xfefd1923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x964d1927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577d192f, 90 },
    { 0xe68d1930, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfd1935, 61 },
    { 0xccbd1935, 33 },
    { 0xafcd1935, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdd193a, 76 },
    { 0, 0 },
    { 0x889d193c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dd1949, 61 },
    { 0x167d1949, 90 },
    { 0, 0 },
    { 0x959d194c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x739d194f, 90 },
    { 0xd96d1950, 33 },
    { 0x236d194f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594d1955, 98 },
    { 0x51d1955, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0d195c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d1961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d1965, 98 },
    { 0x7fcd1966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd196e, 33 },
    { 0x6e1d196f, 34 },
    { 0x1a5d1970, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528d1977, 23 },
    { 0x93cd1978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d197f, 98 },
    { 0x756d1980, 88 },
    { 0x46ad1981, 77 },
    { 0x543d197f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0d1985, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d198b, 78 },
    { 0, 0 },
    { 0xf9bd198d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172d1996, 76 },
    { 0x664d1997, 23 },
    { 0xbd1998, 33 },
    { 0xee1d1997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d199c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d19a0, 93 },
    { 0xc0ad19a1, 33 },
    { 0xfa7d19a0, 77 },
    { 0xcdfd19a2, 23 },
    { 0x839d19a4, 98 },
    { 0xc26d19a5, 41 },
    { 0x448d19a6, 66 },
    { 0x4a9d19a7, 46 },
    { 0xf6ed19a8, 52 },
    { 0x21fd19a8, 55 },
    { 0x504d19a4, 69 },
    { 0x89d19a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335d19af, 23 },
    { 0x666d19af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d19b3, 33 },
    { 0x199d19b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebd19b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fd19bd, 93 },
    { 0, 0 },
    { 0x64ad19bf, 76 },
    { 0x62ad19c0, 90 },
    { 0xf21d19c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d19c4, 43 },
    { 0xbf7d19c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04d19c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2d19cd, 33 },
    { 0, 0 },
    { 0xb1fd19cf, 98 },
    { 0xdf7d19cf, 61 },
    { 0, 0 },
    { 0xacfd19d2, 95 },
    { 0, 0 },
    { 0x9b2d19d4, 77 },
    { 0x36dd19d4, 91 },
    { 0x223d19d6, 93 },
    { 0x155d19d6, 95 },
    { 0x161d19d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd19e3, 95 },
    { 0xdd3d19e4, 43 },
    { 0x374d19e5, 91 },
    { 0x4d1d19e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d19f0, 61 },
    { 0x6bed19f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbd19f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d19f8, 52 },
    { 0x947d19f8, 76 },
    { 0x1e4d19fa, 90 },
    { 0xaf8d19fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d1a02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175d1a06, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dd1a0e, 34 },
    { 0x7bed1a0f, 95 },
    { 0xbf4d1a0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd1a17, 23 },
    { 0x1bad1a18, 90 },
    { 0, 0 },
    { 0x3b1d1a1a, 98 },
    { 0x14fd1a1b, 76 },
    { 0x1e0d1a1c, 33 },
    { 0, 0 },
    { 0xd1bd1a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07d1a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d1a29, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x268d1a2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d1a2f, 90 },
    { 0, 0 },
    { 0xed9d1a31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad1a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cd1a38, 36 },
    { 0, 0 },
    { 0xde0d1a3a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d1a41, 66 },
    { 0x7e5d1a42, 34 },
    { 0x555d1a42, 75 },
    { 0, 0 },
    { 0x8e2d1a45, 68 },
    { 0xcd8d1a46, 91 },
    { 0, 0 },
    { 0x998d1a48, 78 },
    { 0, 0 },
    { 0x883d1a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebd1a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5d1a54, 76 },
    { 0, 0 },
    { 0x8e5d1a56, 61 },
    { 0x147d1a57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d1a5a, 33 },
    { 0x544d1a5b, 84 },
    { 0, 0 },
    { 0x7cad1a5d, 93 },
    { 0, 0 },
    { 0x20ad1a5f, 42 },
    { 0x2e7d1a5f, 92 },
    { 0x49bd1a5f, 90 },
    { 0x2bd1a60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37d1a69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d1a6c, 61 },
    { 0xa55d1a6d, 61 },
    { 0, 0 },
    { 0xaf7d1a6f, 95 },
    { 0x1e2d1a70, 61 },
    { 0x5fbd1a71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cd1a76, 33 },
    { 0xf97d1a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2d1a7e, 23 },
    { 0x1e4d1a7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4d1a8b, 61 },
    { 0x4b3d1a8b, 84 },
    { 0xa3ed1a8d, 55 },
    { 0x327d1a8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dd1a91, 33 },
    { 0xdb4d1a92, 77 },
    { 0xbdad1a93, 61 },
    { 0xbf8d1a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ed1a9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d1aa9, 90 },
    { 0xa4d1aaa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5d1aae, 95 },
    { 0, 0 },
    { 0xf02d1ab0, 23 },
    { 0x6d8d1ab1, 43 },
    { 0x399d1ab2, 90 },
    { 0x44cd1ab2, 33 },
    { 0x579d1ab2, 40 },
    { 0, 0 },
    { 0xf06d1ab6, 76 },
    { 0xa65d1ab7, 76 },
    { 0x9dad1ab8, 90 },
    { 0, 0 },
    { 0xa92d1aba, 76 },
    { 0, 0 },
    { 0xbb8d1abc, 98 },
    { 0x1b4d1abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acd1ac1, 99 },
    { 0, 0 },
    { 0xe47d1ac3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3d1ac6, 34 },
    { 0, 0 },
    { 0x27ed1ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533d1acc, 70 },
    { 0x2bcd1acc, 67 },
    { 0x5ed1ace, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d1ad5, 93 },
    { 0x203d1ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6d1add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ad1ae6, 90 },
    { 0xb08d1ae7, 66 },
    { 0x922d1ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34d1aee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ed1af1, 90 },
    { 0, 0 },
    { 0xb23d1af3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d1af8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd1afb, 61 },
    { 0x18bd1afb, 61 },
    { 0x5cdd1afd, 46 },
    { 0, 0 },
    { 0xb6fd1aff, 91 },
    { 0, 0 },
    { 0x651d1b01, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fd1b08, 100 },
    { 0xb47d1b09, 76 },
    { 0, 0 },
    { 0x280d1b0b, 98 },
    { 0x823d1b0b, 93 },
    { 0, 0 },
    { 0x371d1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7d1b13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ad1b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26d1b19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ed1b1d, 95 },
    { 0xbe4d1b1e, 33 },
    { 0, 0 },
    { 0xce5d1b20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716d1b27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9d1b2a, 61 },
    { 0xb34d1b2a, 92 },
    { 0xf42d1b2c, 87 },
    { 0x909d1b2d, 90 },
    { 0, 0 },
    { 0x80bd1b2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ed1b32, 90 },
    { 0x581d1b32, 23 },
    { 0xe33d1b34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d1b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849d1b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d1b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d1b45, 91 },
    { 0xa32d1b46, 77 },
    { 0xd2fd1b47, 90 },
    { 0xaead1b46, 98 },
    { 0xeccd1b48, 23 },
    { 0xe6bd1b46, 90 },
    { 0x17d1b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efd1b50, 61 },
    { 0x5cd1b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8d1b54, 83 },
    { 0, 0 },
    { 0x28d1b56, 68 },
    { 0x78fd1b57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d1b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1b62, 61 },
    { 0, 0 },
    { 0x562d1b64, 78 },
    { 0, 0 },
    { 0x5dd1b66, 34 },
    { 0x9a9d1b67, 100 },
    { 0, 0 },
    { 0xe67d1b69, 98 },
    { 0xf15d1b6a, 23 },
    { 0x206d1b6b, 38 },
    { 0xa9d1b6c, 91 },
    { 0, 0 },
    { 0x12fd1b6e, 49 },
    { 0x83d1b6e, 95 },
    { 0, 0 },
    { 0xd2cd1b71, 95 },
    { 0x686d1b71, 76 },
    { 0, 0 },
    { 0xa42d1b74, 61 },
    { 0xae4d1b75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9d1b79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18d1b7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37d1b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d1b85, 61 },
    { 0, 0 },
    { 0x592d1b87, 23 },
    { 0xb32d1b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d1b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d1b91, 98 },
    { 0x5ebd1b91, 61 },
    { 0, 0 },
    { 0xf1ed1b94, 75 },
    { 0xb01d1b95, 33 },
    { 0x9ead1b95, 69 },
    { 0x574d1b95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7d1b9c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d1ba6, 95 },
    { 0, 0 },
    { 0x832d1ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fd1bb0, 76 },
    { 0x94d1bb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93d1bbb, 88 },
    { 0x2cad1bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ed1bbf, 76 },
    { 0x330d1bc0, 75 },
    { 0, 0 },
    { 0x34cd1bc2, 97 },
    { 0x5d3d1bc3, 76 },
    { 0, 0 },
    { 0xca3d1bc5, 33 },
    { 0x344d1bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2d1bca, 84 },
    { 0xe30d1bcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2d1bcf, 91 },
    { 0x61d1bd0, 61 },
    { 0x208d1bd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556d1bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0d1bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d1be4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d1beb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad1bf0, 99 },
    { 0x2abd1bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984d1bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06d1bfd, 85 },
    { 0x30ad1bfe, 76 },
    { 0, 0 },
    { 0x4e6d1c00, 78 },
    { 0xf46d1c00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad1c05, 91 },
    { 0x648d1c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372d1c0c, 87 },
    { 0xf99d1c0c, 98 },
    { 0, 0 },
    { 0xb39d1c0f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd1c12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340d1c28, 45 },
    { 0x81d1c29, 90 },
    { 0x620d1c2a, 90 },
    { 0x3ead1c2b, 46 },
    { 0x8b9d1c2c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977d1c30, 61 },
    { 0x105d1c30, 46 },
    { 0xdbbd1c32, 95 },
    { 0, 0 },
    { 0xe16d1c34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd1c38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd1c3c, 90 },
    { 0x4a9d1c3c, 78 },
    { 0x840d1c3c, 90 },
    { 0xb07d1c3f, 46 },
    { 0xae5d1c40, 23 },
    { 0x95dd1c41, 95 },
    { 0x6d1d1c42, 23 },
    { 0xf5ed1c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3d1c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937d1c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d1c5c, 55 },
    { 0xf87d1c5d, 23 },
    { 0x46ad1c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103d1c6b, 98 },
    { 0x5bad1c6c, 90 },
    { 0xead1c6d, 55 },
    { 0x2c2d1c6e, 90 },
    { 0xd71d1c6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d1c76, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd1c79, 77 },
    { 0x543d1c79, 61 },
    { 0xcd4d1c7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d1c7e, 100 },
    { 0x773d1c7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05d1c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad1c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ad1c8d, 23 },
    { 0xf8ad1c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd1c91, 34 },
    { 0x663d1c91, 95 },
    { 0xda0d1c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x582d1c96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdd1c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4d1ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d1ca6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839d1caa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed1caf, 66 },
    { 0, 0 },
    { 0x759d1cb1, 84 },
    { 0, 0 },
    { 0x150d1cb3, 17 },
    { 0x9dbd1cb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dd1cbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd1cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d1cc5, 93 },
    { 0xbfed1cc6, 81 },
    { 0xc6ed1cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d1cca, 90 },
    { 0, 0 },
    { 0x185d1ccc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288d1cd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278d1cda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485d1ce1, 95 },
    { 0x49ed1ce1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579d1ce7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd1cec, 90 },
    { 0x1f9d1ced, 23 },
    { 0xa76d1cec, 53 },
    { 0x253d1cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x411d1cf2, 92 },
    { 0x15fd1cf3, 100 },
    { 0x142d1cf3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d1cf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78d1d00, 40 },
    { 0xa58d1d01, 23 },
    { 0xabad1d01, 90 },
    { 0x4e4d1d01, 84 },
    { 0, 0 },
    { 0x26d1d05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d1d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d1d10, 46 },
    { 0, 0 },
    { 0xd67d1d12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdd1d17, 22 },
    { 0, 0 },
    { 0x5d9d1d19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded1d1d, 76 },
    { 0xef6d1d1d, 95 },
    { 0x1bed1d1d, 100 },
    { 0x53bd1d1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d1d28, 76 },
    { 0xb05d1d29, 23 },
    { 0xa47d1d28, 33 },
    { 0x60ed1d29, 95 },
    { 0x42ed1d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d1d35, 95 },
    { 0, 0 },
    { 0x3dd1d37, 55 },
    { 0x1f2d1d37, 94 },
    { 0x387d1d38, 33 },
    { 0x3dd1d39, 84 },
    { 0, 0 },
    { 0x4ed1d3c, 17 },
    { 0, 0 },
    { 0xbedd1d3e, 52 },
    { 0, 0 },
    { 0xf2ad1d40, 76 },
    { 0x656d1d41, 65 },
    { 0xc78d1d40, 101 },
    { 0, 0 },
    { 0x9f1d1d44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabd1d4c, 43 },
    { 0xbfbd1d4d, 90 },
    { 0xa61d1d4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ad1d51, 94 },
    { 0, 0 },
    { 0xb40d1d53, 95 },
    { 0xe47d1d54, 76 },
    { 0xb79d1d55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cd1d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d1d5f, 61 },
    { 0x53ed1d5f, 46 },
    { 0x224d1d5f, 36 },
    { 0, 0 },
    { 0xf20d1d63, 51 },
    { 0, 0 },
    { 0xb3bd1d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57d1d6a, 33 },
    { 0x70bd1d6a, 93 },
    { 0xd70d1d6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddd1d73, 66 },
    { 0x5f3d1d73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94d1d78, 61 },
    { 0, 0 },
    { 0x5edd1d7a, 17 },
    { 0x77bd1d7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d1d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118d1d85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d1d8c, 93 },
    { 0x766d1d8d, 55 },
    { 0, 0 },
    { 0xfc1d1d8f, 61 },
    { 0, 0 },
    { 0xe99d1d91, 23 },
    { 0, 0 },
    { 0x748d1d93, 33 },
    { 0, 0 },
    { 0x89dd1d95, 55 },
    { 0, 0 },
    { 0x446d1d97, 66 },
    { 0xde5d1d98, 52 },
    { 0, 0 },
    { 0xb7d1d9a, 95 },
    { 0xc75d1d9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed1da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d1da3, 33 },
    { 0, 0 },
    { 0xcded1da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fd1da9, 61 },
    { 0, 0 },
    { 0x3d0d1dab, 61 },
    { 0xe17d1dac, 34 },
    { 0x4c2d1dad, 55 },
    { 0, 0 },
    { 0x946d1daf, 98 },
    { 0xceed1db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851d1db5, 61 },
    { 0xf4fd1db6, 76 },
    { 0x443d1db7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d1dba, 46 },
    { 0, 0 },
    { 0xf03d1dbc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd1dc0, 90 },
    { 0x997d1dc0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37d1dca, 66 },
    { 0x70ad1dca, 90 },
    { 0, 0 },
    { 0x228d1dcd, 33 },
    { 0x824d1dce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8add1dd2, 99 },
    { 0x8c2d1dd3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd1dd9, 95 },
    { 0x6cbd1dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d1ddd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8d1de8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48d1dec, 23 },
    { 0, 0 },
    { 0x10ed1dee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ad1df2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d1dfa, 41 },
    { 0xd79d1dfa, 90 },
    { 0x609d1dfc, 52 },
    { 0xff9d1dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d1e01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cd1e07, 100 },
    { 0x2fad1e08, 55 },
    { 0x458d1e09, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9d1e14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x233d1e17, 76 },
    { 0x2ed1e18, 34 },
    { 0, 0 },
    { 0x3ead1e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d1e1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58d1e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed1e28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d1e2c, 55 },
    { 0, 0 },
    { 0x60fd1e2e, 95 },
    { 0x607d1e2f, 76 },
    { 0xc25d1e2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d1e33, 91 },
    { 0x585d1e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249d1e3a, 98 },
    { 0x139d1e3b, 75 },
    { 0x69d1e3c, 65 },
    { 0xac6d1e3d, 61 },
    { 0x88dd1e3b, 95 },
    { 0xe78d1e3f, 80 },
    { 0xcdd1e3f, 55 },
    { 0x85fd1e41, 100 },
    { 0xf49d1e3d, 95 },
    { 0xfbed1e43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ed1e57, 90 },
    { 0x66dd1e58, 90 },
    { 0x1c0d1e59, 98 },
    { 0xe39d1e57, 43 },
    { 0xf5fd1e5b, 90 },
    { 0xe0d1e5c, 95 },
    { 0xe10d1e5d, 61 },
    { 0xef8d1e59, 95 },
    { 0x29d1e5a, 46 },
    { 0xf53d1e5c, 33 },
    { 0x77ed1e61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2d1e67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddd1e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd1e75, 66 },
    { 0xa02d1e76, 52 },
    { 0xcaed1e75, 61 },
    { 0, 0 },
    { 0x724d1e79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ad1e7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x977d1e82, 84 },
    { 0x1d9d1e83, 84 },
    { 0xf8ed1e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0d1e88, 90 },
    { 0, 0 },
    { 0xc87d1e8a, 90 },
    { 0x72ad1e8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d1e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962d1e91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d1e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d1e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eed1ea3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x560d1ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fd1eaa, 95 },
    { 0x858d1eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84d1eae, 98 },
    { 0, 0 },
    { 0xdc1d1eb0, 65 },
    { 0, 0 },
    { 0x3add1eb2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d1eb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfd1ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd1ebe, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ed1ec2, 95 },
    { 0, 0 },
    { 0x376d1ec4, 43 },
    { 0, 0 },
    { 0xe58d1ec6, 91 },
    { 0x549d1ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d1ecd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afd1ed4, 23 },
    { 0xa72d1ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacd1eda, 36 },
    { 0, 0 },
    { 0x446d1edc, 43 },
    { 0x855d1edd, 55 },
    { 0x997d1edc, 95 },
    { 0xf2dd1edc, 61 },
    { 0x5dfd1ee0, 61 },
    { 0xcdad1ee0, 81 },
    { 0xf42d1ee1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d1eec, 9 },
    { 0, 0 },
    { 0x187d1eee, 61 },
    { 0xfebd1eef, 36 },
    { 0x329d1ef0, 78 },
    { 0xb39d1ef1, 76 },
    { 0, 0 },
    { 0x5ead1ef3, 98 },
    { 0xe07d1ef4, 93 },
    { 0x14cd1ef5, 33 },
    { 0, 0 },
    { 0xe2d1ef7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd1efe, 90 },
    { 0, 0 },
    { 0xdefd1f00, 95 },
    { 0xe0bd1f01, 66 },
    { 0xff8d1f02, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfd1f06, 68 },
    { 0, 0 },
    { 0xc88d1f08, 90 },
    { 0xfb6d1f09, 87 },
    { 0, 0 },
    { 0x594d1f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d1f11, 38 },
    { 0x9b5d1f12, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d1f17, 98 },
    { 0x2b6d1f18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4d1f1d, 90 },
    { 0x5f7d1f1e, 52 },
    { 0, 0 },
    { 0x464d1f20, 90 },
    { 0x48bd1f21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d1f27, 61 },
    { 0x294d1f27, 76 },
    { 0xb0d1f29, 33 },
    { 0x8add1f2a, 90 },
    { 0x1b9d1f2b, 61 },
    { 0x4dad1f2b, 94 },
    { 0xeefd1f2b, 55 },
    { 0x1b7d1f2b, 61 },
    { 0xadd1f2f, 55 },
    { 0xc5bd1f28, 61 },
    { 0x9bed1f29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d1f34, 34 },
    { 0xc0bd1f35, 68 },
    { 0x640d1f36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cd1f40, 90 },
    { 0xae9d1f41, 90 },
    { 0xddcd1f41, 23 },
    { 0xefbd1f40, 23 },
    { 0x2c3d1f44, 85 },
    { 0xc9d1f45, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abd1f49, 78 },
    { 0, 0 },
    { 0x53ed1f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed1f4f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed1f56, 52 },
    { 0x599d1f56, 33 },
    { 0x9b4d1f58, 76 },
    { 0xfad1f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697d1f5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133d1f63, 68 },
    { 0xaddd1f64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18d1f68, 23 },
    { 0x139d1f69, 46 },
    { 0xe62d1f69, 23 },
    { 0x201d1f6b, 23 },
    { 0, 0 },
    { 0x92fd1f6d, 43 },
    { 0, 0 },
    { 0xfe7d1f6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded1f72, 78 },
    { 0x223d1f73, 100 },
    { 0xa01d1f73, 90 },
    { 0xb3dd1f73, 33 },
    { 0, 0 },
    { 0x54bd1f77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d1f7b, 61 },
    { 0, 0 },
    { 0xbedd1f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cd1f88, 55 },
    { 0, 0 },
    { 0xf64d1f8a, 98 },
    { 0xe14d1f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fd1f91, 90 },
    { 0xc7fd1f91, 95 },
    { 0, 0 },
    { 0x619d1f94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44d1f9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79d1fa0, 33 },
    { 0xbddd1fa0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad1fa5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd1fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585d1fae, 75 },
    { 0x6f8d1faf, 61 },
    { 0x68dd1fb0, 95 },
    { 0, 0 },
    { 0x46bd1fb2, 98 },
    { 0x473d1fb2, 98 },
    { 0x191d1fb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bd1fb7, 98 },
    { 0xa0ed1fb8, 33 },
    { 0x184d1fb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad1fbe, 95 },
    { 0xc2ad1fbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ad1fc5, 23 },
    { 0x2b3d1fc5, 98 },
    { 0, 0 },
    { 0x21dd1fc8, 23 },
    { 0xd31d1fc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408d1fcf, 55 },
    { 0xc70d1fd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffd1fd6, 90 },
    { 0, 0 },
    { 0xe6fd1fd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x970d1fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd1fe1, 61 },
    { 0x205d1fe2, 34 },
    { 0xe3dd1fe3, 85 },
    { 0x86ad1fe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7d1fe9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed1fed, 90 },
    { 0x29ed1fed, 95 },
    { 0xc62d1fef, 90 },
    { 0x514d1ff0, 98 },
    { 0xa9bd1fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8d1ff4, 34 },
    { 0xd35d1ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47d1ff8, 80 },
    { 0xdded1ff9, 93 },
    { 0, 0 },
    { 0x91ad1ffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd1ffe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ad2005, 77 },
    { 0x849d2005, 33 },
    { 0xf6bd2007, 90 },
    { 0x151d2008, 95 },
    { 0x1f7d2009, 93 },
    { 0xd91d2008, 55 },
    { 0x7e6d200a, 90 },
    { 0x1fcd200c, 76 },
    { 0x2b0d200d, 95 },
    { 0xbe8d2008, 95 },
    { 0xf4fd2005, 83 },
    { 0, 0 },
    { 0xa70d2011, 67 },
    { 0, 0 },
    { 0xefad2013, 99 },
    { 0, 0 },
    { 0x46d2015, 95 },
    { 0, 0 },
    { 0x4aed2017, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836d201e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd2028, 95 },
    { 0xc93d2029, 43 },
    { 0xcced2028, 40 },
    { 0, 0 },
    { 0xc33d202c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd202f, 23 },
    { 0, 0 },
    { 0x7a5d2031, 78 },
    { 0xa43d2032, 46 },
    { 0x1cd2032, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d203c, 23 },
    { 0x8d6d203c, 55 },
    { 0, 0 },
    { 0x9acd203f, 98 },
    { 0, 0 },
    { 0xcd2d2041, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd2044, 98 },
    { 0, 0 },
    { 0xb08d2046, 33 },
    { 0x6dad2047, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd204b, 98 },
    { 0, 0 },
    { 0x91ad204d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cd2050, 23 },
    { 0xe60d2050, 98 },
    { 0, 0 },
    { 0xe37d2053, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd2056, 76 },
    { 0x3a1d2057, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd205e, 43 },
    { 0, 0 },
    { 0x12ed2060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d2066, 81 },
    { 0, 0 },
    { 0x97dd2068, 91 },
    { 0x950d2069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0d2071, 78 },
    { 0x797d2071, 91 },
    { 0, 0 },
    { 0x85bd2074, 33 },
    { 0xf8bd2075, 90 },
    { 0xb0cd2075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd207d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d2080, 33 },
    { 0x569d2081, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad2084, 95 },
    { 0x4dbd2084, 85 },
    { 0, 0 },
    { 0x845d2087, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28d208a, 52 },
    { 0x9dfd208b, 93 },
    { 0x4dd208c, 90 },
    { 0x75ed208d, 34 },
    { 0x55dd208e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d2096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7d2099, 43 },
    { 0x7ccd2099, 93 },
    { 0x31ad209b, 76 },
    { 0x11ad209b, 23 },
    { 0x1cd209c, 43 },
    { 0x255d209d, 53 },
    { 0xbc9d209f, 98 },
    { 0x45bd209a, 84 },
    { 0, 0 },
    { 0x208d20a2, 90 },
    { 0, 0 },
    { 0x645d20a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad20a9, 76 },
    { 0x3bcd20a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d20b1, 61 },
    { 0x8b2d20b2, 88 },
    { 0x56ad20b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ed20b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bd20b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad20bd, 95 },
    { 0xaffd20be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd20c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6d20c5, 95 },
    { 0x274d20c5, 95 },
    { 0x3b3d20c7, 98 },
    { 0x68cd20c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bd20cb, 69 },
    { 0x559d20cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebd20d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3d20d6, 61 },
    { 0x5acd20d7, 33 },
    { 0xea5d20d7, 87 },
    { 0, 0 },
    { 0xaaed20da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d20dd, 40 },
    { 0x6b6d20de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd20e4, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d20ec, 23 },
    { 0, 0 },
    { 0x61ad20ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd20f2, 34 },
    { 0, 0 },
    { 0xa72d20f4, 43 },
    { 0x133d20f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dd20fc, 85 },
    { 0xf72d20fc, 90 },
    { 0x18dd20fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd2103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d2107, 90 },
    { 0xa70d2107, 94 },
    { 0xc1d2107, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ed2111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x463d2114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3d2117, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad211f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd2124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ed2127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25d212b, 61 },
    { 0x130d212c, 90 },
    { 0xb01d212c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d2132, 95 },
    { 0xffdd2132, 98 },
    { 0xf47d2134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d213b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d213f, 95 },
    { 0, 0 },
    { 0x95fd2141, 90 },
    { 0, 0 },
    { 0x9dfd2143, 98 },
    { 0x9e9d2144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dd2149, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d214c, 61 },
    { 0xbe6d214d, 55 },
    { 0, 0 },
    { 0x226d214f, 90 },
    { 0x525d2150, 66 },
    { 0x668d2151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d2158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d215b, 17 },
    { 0x92d215c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd2160, 98 },
    { 0x6e5d2161, 61 },
    { 0x890d2162, 67 },
    { 0, 0 },
    { 0x519d2164, 52 },
    { 0x7ead2165, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x287d2168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944d216d, 88 },
    { 0x6d9d216d, 33 },
    { 0x11ed216e, 65 },
    { 0, 0 },
    { 0x510d2171, 55 },
    { 0, 0 },
    { 0x411d2173, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37d2177, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d217a, 33 },
    { 0xca8d217b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ad2185, 90 },
    { 0x11dd2186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d2189, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d2191, 99 },
    { 0, 0 },
    { 0xccad2193, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x416d2196, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d219d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcd21a1, 84 },
    { 0x816d21a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dd21a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d21ae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d21b1, 23 },
    { 0x1ded21b2, 33 },
    { 0xe74d21b3, 61 },
    { 0xc65d21b1, 4 },
    { 0, 0 },
    { 0xabfd21b6, 69 },
    { 0x59d21b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dd21c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18d21c3, 34 },
    { 0xb46d21c4, 61 },
    { 0xf63d21c5, 56 },
    { 0xf2cd21c6, 67 },
    { 0, 0 },
    { 0xdb8d21c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d21cd, 77 },
    { 0x281d21ce, 76 },
    { 0xd06d21cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d21d4, 93 },
    { 0x4f7d21d5, 90 },
    { 0xe59d21d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1d21da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afd21df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaad21e2, 98 },
    { 0x5ecd21e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d21e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d21ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d21f1, 52 },
    { 0xa3ad21f2, 23 },
    { 0x1cad21f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513d21f8, 65 },
    { 0x773d21f8, 90 },
    { 0, 0 },
    { 0x1f8d21fb, 77 },
    { 0xd7ed21fc, 66 },
    { 0xe17d21fd, 17 },
    { 0, 0 },
    { 0x5c5d21ff, 17 },
    { 0x9b9d2200, 91 },
    { 0x56d2201, 17 },
    { 0xa2ed2202, 61 },
    { 0, 0 },
    { 0x714d2204, 90 },
    { 0x580d2204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d220d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3d2212, 90 },
    { 0x7f9d2213, 88 },
    { 0xd37d2212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d2218, 55 },
    { 0x82ed2219, 33 },
    { 0x71ad221a, 61 },
    { 0xd9cd221b, 23 },
    { 0x408d221b, 33 },
    { 0x28ed221d, 90 },
    { 0xeaed221d, 69 },
    { 0x49fd221f, 23 },
    { 0x119d221b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd2223, 61 },
    { 0x5f0d2224, 90 },
    { 0x1e4d2225, 55 },
    { 0x79bd2225, 55 },
    { 0xfd7d2223, 40 },
    { 0, 0 },
    { 0x70ad2229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd222f, 76 },
    { 0x641d2230, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acd2239, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dd223c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd223f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d2243, 34 },
    { 0, 0 },
    { 0xc34d2245, 76 },
    { 0xf21d2246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d2249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed224e, 75 },
    { 0, 0 },
    { 0x52d2250, 90 },
    { 0, 0 },
    { 0x45dd2252, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82d2256, 78 },
    { 0, 0 },
    { 0xc0ed2258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273d225c, 76 },
    { 0x827d225d, 90 },
    { 0xf1ed225e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d2266, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x801d2269, 99 },
    { 0x82fd226a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d226d, 55 },
    { 0xfe0d226e, 55 },
    { 0x8cbd226d, 61 },
    { 0, 0 },
    { 0xe9cd2271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1d2276, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d227b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96d227e, 98 },
    { 0xd6dd227f, 87 },
    { 0x853d2280, 55 },
    { 0xb67d2280, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d2285, 61 },
    { 0x8d0d2285, 95 },
    { 0x481d2285, 61 },
    { 0, 0 },
    { 0x7dad2289, 90 },
    { 0x8ecd2289, 23 },
    { 0x39ed228b, 98 },
    { 0x75dd228c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x562d228f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcd2293, 33 },
    { 0x7d7d2294, 23 },
    { 0xc56d2294, 61 },
    { 0x651d2296, 98 },
    { 0xfe3d2294, 61 },
    { 0, 0 },
    { 0x7afd2299, 76 },
    { 0x596d2299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d229d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138d22a2, 90 },
    { 0x627d22a3, 95 },
    { 0x33ed22a4, 34 },
    { 0xa14d22a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadd22a9, 23 },
    { 0, 0 },
    { 0x3e8d22ab, 23 },
    { 0xdafd22ac, 23 },
    { 0x961d22ad, 93 },
    { 0x2ad22ac, 33 },
    { 0, 0 },
    { 0xccdd22b0, 23 },
    { 0, 0 },
    { 0xf49d22b2, 34 },
    { 0x258d22b2, 34 },
    { 0x854d22b2, 98 },
    { 0xef3d22b5, 34 },
    { 0, 0 },
    { 0x2acd22b7, 33 },
    { 0, 0 },
    { 0x3f3d22b9, 47 },
    { 0x315d22ba, 61 },
    { 0xa6cd22bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925d22c0, 75 },
    { 0xc8d22c1, 76 },
    { 0x257d22c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbd22c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51d22cc, 68 },
    { 0x2a4d22cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ed22d1, 76 },
    { 0x2ddd22d2, 55 },
    { 0xbe1d22d3, 76 },
    { 0xecad22d2, 61 },
    { 0x7f6d22d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x359d22d8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d22e1, 100 },
    { 0, 0 },
    { 0xc77d22e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdd22ea, 65 },
    { 0x61cd22eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd22f1, 76 },
    { 0, 0 },
    { 0x9d3d22f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452d22fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d22ff, 95 },
    { 0, 0 },
    { 0x3d2d2301, 23 },
    { 0x50d2301, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ed2307, 65 },
    { 0xa6ed2307, 95 },
    { 0x702d2308, 43 },
    { 0x8fed230a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x438d230d, 34 },
    { 0x531d230e, 81 },
    { 0x648d230d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bad2315, 61 },
    { 0xbb1d2316, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffd231a, 83 },
    { 0xc68d231b, 61 },
    { 0xffad231c, 55 },
    { 0x244d231b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d2321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551d2325, 43 },
    { 0, 0 },
    { 0x55d2327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfed232c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add232f, 80 },
    { 0x4ecd2330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd2333, 99 },
    { 0xd5dd2334, 90 },
    { 0x56fd2335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d233a, 23 },
    { 0xbf6d233a, 61 },
    { 0x751d233c, 55 },
    { 0xb2fd233c, 23 },
    { 0xbb2d233e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4d2342, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd234b, 90 },
    { 0, 0 },
    { 0xe9d234d, 90 },
    { 0, 0 },
    { 0xc5d234f, 93 },
    { 0x1ecd2350, 61 },
    { 0x395d2351, 98 },
    { 0x1f5d234f, 90 },
    { 0x8c0d2353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fed2358, 90 },
    { 0xc4cd2359, 98 },
    { 0, 0 },
    { 0x81dd235b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d235e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x353d2361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d2366, 73 },
    { 0, 0 },
    { 0x7add2368, 98 },
    { 0xf2bd2369, 34 },
    { 0x2abd2369, 87 },
    { 0, 0 },
    { 0xc90d236c, 46 },
    { 0, 0 },
    { 0x619d236e, 33 },
    { 0xaa0d236e, 42 },
    { 0x4c8d2370, 88 },
    { 0x87ed2370, 46 },
    { 0, 0 },
    { 0x651d2373, 100 },
    { 0x283d2374, 90 },
    { 0, 0 },
    { 0xbdad2376, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecd2379, 99 },
    { 0x7fcd2379, 23 },
    { 0, 0 },
    { 0x703d237c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ed2381, 43 },
    { 0x828d2382, 61 },
    { 0x803d2383, 85 },
    { 0xe2ed2384, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5d238b, 95 },
    { 0xccbd238c, 76 },
    { 0, 0 },
    { 0x97ed238e, 98 },
    { 0xaffd238e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ed2392, 2 },
    { 0, 0 },
    { 0x1b2d2394, 98 },
    { 0x430d2395, 33 },
    { 0x1d2394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d2399, 95 },
    { 0xe83d2399, 90 },
    { 0xf5bd2399, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4d23a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1d23a3, 95 },
    { 0, 0 },
    { 0x45fd23a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d23a9, 95 },
    { 0x25bd23aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d23ae, 33 },
    { 0x9d9d23ae, 33 },
    { 0xccbd23af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd23b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6d23ba, 33 },
    { 0, 0 },
    { 0xa20d23bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453d23c0, 98 },
    { 0, 0 },
    { 0x199d23c2, 33 },
    { 0xe3dd23c2, 60 },
    { 0, 0 },
    { 0x56bd23c5, 23 },
    { 0x742d23c5, 90 },
    { 0, 0 },
    { 0xc16d23c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d23d0, 23 },
    { 0x1bed23d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d23d4, 99 },
    { 0xe7fd23d4, 43 },
    { 0xa2cd23d6, 61 },
    { 0x311d23d7, 46 },
    { 0x808d23d8, 90 },
    { 0xe55d23d8, 61 },
    { 0x538d23d4, 76 },
    { 0x370d23db, 61 },
    { 0x545d23d4, 33 },
    { 0x439d23dd, 98 },
    { 0x4c0d23de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaad23e2, 33 },
    { 0xb62d23e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad23e8, 87 },
    { 0xa92d23e8, 95 },
    { 0x9f5d23ea, 23 },
    { 0xb7fd23eb, 40 },
    { 0, 0 },
    { 0xfcbd23ed, 90 },
    { 0, 0 },
    { 0xad4d23ef, 23 },
    { 0x7e0d23f0, 55 },
    { 0x56fd23ef, 43 },
    { 0, 0 },
    { 0x993d23f3, 61 },
    { 0, 0 },
    { 0x53ed23f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd23f9, 55 },
    { 0xb67d23fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cd2401, 78 },
    { 0x9f5d2402, 100 },
    { 0x396d2403, 90 },
    { 0xe17d2403, 33 },
    { 0x31d2405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7d2408, 90 },
    { 0xa66d2409, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xadad240c, 95 },
    { 0x406d240d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d2413, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x989d2416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fd2419, 23 },
    { 0x80ad241a, 34 },
    { 0x5d7d241b, 90 },
    { 0x67bd241c, 85 },
    { 0, 0 },
    { 0x33d241e, 66 },
    { 0, 0 },
    { 0xb7fd2420, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dd2424, 90 },
    { 0x5dd2425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d242a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9d242d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67d2434, 88 },
    { 0, 0 },
    { 0x2f4d2436, 96 },
    { 0x9fdd2437, 61 },
    { 0x7ddd2436, 61 },
    { 0x666d2439, 35 },
    { 0, 0 },
    { 0x483d243b, 99 },
    { 0xe2ed243c, 90 },
    { 0xfc8d243c, 85 },
    { 0xc7ed243c, 61 },
    { 0x80cd243b, 91 },
    { 0xa91d243e, 55 },
    { 0x136d2440, 52 },
    { 0x6dcd2442, 98 },
    { 0x382d2443, 78 },
    { 0, 0 },
    { 0x1bfd2445, 2 },
    { 0x230d2446, 34 },
    { 0x5cad2446, 90 },
    { 0, 0 },
    { 0x37dd2449, 98 },
    { 0x4f6d244a, 76 },
    { 0, 0 },
    { 0x2f4d244c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915d2451, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d2454, 23 },
    { 0x813d2454, 65 },
    { 0xdb2d2454, 98 },
    { 0x58d2456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d245d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x494d2460, 52 },
    { 0xa4bd2460, 55 },
    { 0xe27d2461, 3 },
    { 0x68fd2463, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300d246a, 55 },
    { 0x24ed246b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bed246f, 100 },
    { 0, 0 },
    { 0x2cd2471, 17 },
    { 0x881d2472, 61 },
    { 0, 0 },
    { 0x96dd2474, 98 },
    { 0xa1dd2475, 90 },
    { 0xb68d2476, 61 },
    { 0x2e8d2474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bd247f, 91 },
    { 0x694d2480, 65 },
    { 0x967d247f, 33 },
    { 0xd86d2482, 55 },
    { 0xc3dd247f, 36 },
    { 0x597d2484, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07d248a, 81 },
    { 0, 0 },
    { 0x3dcd248c, 66 },
    { 0x267d248c, 98 },
    { 0x2f5d248e, 95 },
    { 0x25fd248d, 98 },
    { 0x5d9d248d, 33 },
    { 0x8d5d2491, 53 },
    { 0xd0dd2492, 65 },
    { 0x650d2493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad2496, 85 },
    { 0xaeed2497, 33 },
    { 0x9a9d2496, 33 },
    { 0x23d2499, 34 },
    { 0, 0 },
    { 0x83ed249b, 23 },
    { 0xf8bd249c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688d24a3, 46 },
    { 0, 0 },
    { 0x2fcd24a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d24b2, 23 },
    { 0x1b3d24b3, 43 },
    { 0xafdd24b4, 52 },
    { 0, 0 },
    { 0x823d24b6, 43 },
    { 0, 0 },
    { 0x4c2d24b8, 23 },
    { 0x747d24b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d24be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d24c3, 23 },
    { 0x45bd24c3, 34 },
    { 0x4d8d24c3, 55 },
    { 0x266d24c6, 99 },
    { 0xaed24c7, 33 },
    { 0xc5cd24c8, 34 },
    { 0xe5bd24c7, 100 },
    { 0xd8ed24c8, 76 },
    { 0x236d24cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbd24ce, 95 },
    { 0x25bd24ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d24d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d24d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7d24df, 93 },
    { 0x60ad24e0, 95 },
    { 0x6e4d24df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32d24e4, 40 },
    { 0xf35d24e5, 33 },
    { 0xfa1d24e6, 90 },
    { 0x79ad24e5, 95 },
    { 0xaa6d24e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5d24ec, 61 },
    { 0x5acd24ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d24f0, 98 },
    { 0xb76d24f1, 98 },
    { 0x5aad24f2, 32 },
    { 0xb39d24f3, 43 },
    { 0x960d24f4, 95 },
    { 0xe95d24f4, 33 },
    { 0, 0 },
    { 0x9c4d24f7, 90 },
    { 0, 0 },
    { 0x297d24f9, 33 },
    { 0x84ad24fa, 95 },
    { 0x71d24fb, 23 },
    { 0x38fd24fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d2501, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d250d, 87 },
    { 0xfb2d250d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2512, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd2518, 87 },
    { 0xc60d2519, 43 },
    { 0, 0 },
    { 0xd51d251b, 61 },
    { 0xea8d251c, 61 },
    { 0xd58d251c, 34 },
    { 0x3bfd251e, 90 },
    { 0xca4d251e, 95 },
    { 0xc64d2520, 61 },
    { 0x950d2521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd2526, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d252b, 23 },
    { 0xaf9d252c, 90 },
    { 0x6f4d252d, 90 },
    { 0x6b9d252e, 93 },
    { 0, 0 },
    { 0x81ed2530, 23 },
    { 0xb37d2531, 95 },
    { 0xb8ad2531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d2537, 33 },
    { 0x5d6d2537, 100 },
    { 0x333d2539, 33 },
    { 0, 0 },
    { 0xcad253b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d2543, 96 },
    { 0x5c4d2544, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329d2549, 91 },
    { 0xc1bd254a, 46 },
    { 0, 0 },
    { 0xc61d254c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed254f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd2553, 38 },
    { 0xfb0d2554, 36 },
    { 0xf37d2555, 55 },
    { 0x5f4d2556, 76 },
    { 0x975d2557, 90 },
    { 0x3a3d2558, 38 },
    { 0x8e6d2555, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97d255d, 95 },
    { 0x436d255e, 93 },
    { 0, 0 },
    { 0x6b4d2560, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d2563, 90 },
    { 0, 0 },
    { 0x21fd2565, 93 },
    { 0xa4d2566, 85 },
    { 0, 0 },
    { 0x82dd2568, 43 },
    { 0, 0 },
    { 0x3d9d256a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d2572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d2579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697d257c, 76 },
    { 0, 0 },
    { 0x306d257e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd2582, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd258e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d2593, 52 },
    { 0xf38d2593, 90 },
    { 0x1c7d2595, 68 },
    { 0x492d2596, 98 },
    { 0x128d2597, 76 },
    { 0xda6d2598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d259f, 90 },
    { 0x826d259f, 95 },
    { 0x183d25a1, 23 },
    { 0x48ad25a1, 76 },
    { 0xf8d25a2, 93 },
    { 0x730d25a3, 66 },
    { 0, 0 },
    { 0x491d25a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835d25aa, 84 },
    { 0, 0 },
    { 0x57d25ac, 33 },
    { 0, 0 },
    { 0x895d25ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd25b2, 23 },
    { 0x3c7d25b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad25c5, 83 },
    { 0x50cd25c6, 90 },
    { 0xf40d25c5, 61 },
    { 0xe28d25c8, 33 },
    { 0xbed25c8, 51 },
    { 0xcebd25c6, 61 },
    { 0x602d25cb, 95 },
    { 0xb89d25c9, 56 },
    { 0x76cd25cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd25d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1d25d6, 43 },
    { 0, 0 },
    { 0xab0d25d8, 90 },
    { 0x46dd25d8, 90 },
    { 0, 0 },
    { 0x3fed25db, 84 },
    { 0x673d25dc, 33 },
    { 0xdced25db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dd25e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540d25e4, 95 },
    { 0xd8ad25e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99d25e9, 95 },
    { 0, 0 },
    { 0x751d25eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d25f0, 76 },
    { 0x5ecd25f1, 90 },
    { 0x332d25f1, 23 },
    { 0xad9d25f3, 90 },
    { 0xf4ed25f2, 98 },
    { 0x51cd25f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cd25f8, 90 },
    { 0x923d25f9, 23 },
    { 0x1cfd25f8, 75 },
    { 0x180d25fb, 33 },
    { 0x230d25fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85d2604, 61 },
    { 0xf08d2605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d2609, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cd2611, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383d2621, 99 },
    { 0, 0 },
    { 0x3a7d2623, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45d262b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d262f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bd2632, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd263f, 94 },
    { 0, 0 },
    { 0x1c9d2641, 90 },
    { 0x522d2641, 75 },
    { 0x9bcd2643, 52 },
    { 0x1b7d2644, 34 },
    { 0xa30d2645, 33 },
    { 0xf57d2646, 55 },
    { 0xfc9d2647, 95 },
    { 0xb59d2648, 66 },
    { 0x203d2649, 90 },
    { 0xd7d2642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78d2650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd2656, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38d265b, 81 },
    { 0, 0 },
    { 0x48dd265d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d2660, 46 },
    { 0x677d2661, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2664, 33 },
    { 0x492d2665, 61 },
    { 0, 0 },
    { 0xae9d2667, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd266a, 77 },
    { 0x348d266b, 90 },
    { 0, 0 },
    { 0x8fdd266d, 90 },
    { 0x696d266d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2672, 33 },
    { 0x680d2673, 90 },
    { 0xad8d2674, 67 },
    { 0xc5fd2675, 61 },
    { 0x8d4d2675, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd267d, 33 },
    { 0xbdcd267e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecd2681, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd2685, 85 },
    { 0xb3dd2685, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcced268b, 37 },
    { 0, 0 },
    { 0x198d268d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed2695, 33 },
    { 0xa6ed2696, 33 },
    { 0x7cdd2697, 76 },
    { 0x590d2696, 55 },
    { 0x2c4d2699, 64 },
    { 0x38d2695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dd26a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cd26a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33d26a7, 63 },
    { 0x683d26a8, 90 },
    { 0x447d26a9, 33 },
    { 0x238d26aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dd26b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d26bb, 95 },
    { 0x42ed26bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d26c3, 91 },
    { 0x61ed26c3, 93 },
    { 0, 0 },
    { 0x500d26c6, 52 },
    { 0x688d26c7, 100 },
    { 0x300d26c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3d26cf, 34 },
    { 0, 0 },
    { 0x9ad26d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d26d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98d26da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7d26e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd26e5, 88 },
    { 0, 0 },
    { 0x7dcd26e7, 23 },
    { 0, 0 },
    { 0xa75d26e9, 61 },
    { 0xb13d26ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd26f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d26f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19d26fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x381d26fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333d2708, 33 },
    { 0x88bd2708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ad270c, 87 },
    { 0xcd9d270c, 52 },
    { 0x2a8d270c, 33 },
    { 0x2e6d270f, 34 },
    { 0, 0 },
    { 0xf9d2711, 59 },
    { 0, 0 },
    { 0x77fd2713, 98 },
    { 0x147d2714, 33 },
    { 0xa7d2715, 23 },
    { 0, 0 },
    { 0xaa3d2717, 66 },
    { 0x81fd2717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d271e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcd2722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0d2725, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259d2729, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7d272d, 85 },
    { 0x797d272d, 98 },
    { 0, 0 },
    { 0x618d2730, 87 },
    { 0xd8ed2731, 33 },
    { 0x8e8d2732, 33 },
    { 0xcc8d2732, 95 },
    { 0, 0 },
    { 0x13fd2735, 61 },
    { 0xcd4d2736, 46 },
    { 0x6eed2736, 61 },
    { 0, 0 },
    { 0x9e4d2739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ed273c, 87 },
    { 0xd6fd273d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2740, 23 },
    { 0, 0 },
    { 0x6b5d2742, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2750, 67 },
    { 0, 0 },
    { 0x666d2752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d275c, 55 },
    { 0xb0bd275d, 98 },
    { 0x675d275d, 93 },
    { 0x6b3d275e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8d2763, 52 },
    { 0xd36d2763, 95 },
    { 0, 0 },
    { 0xf24d2766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28d2769, 95 },
    { 0x6b7d2769, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x319d276d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeed2775, 84 },
    { 0x466d2775, 84 },
    { 0x783d2777, 91 },
    { 0x249d2777, 66 },
    { 0x77bd2777, 100 },
    { 0xc42d2778, 90 },
    { 0xc1ad2778, 76 },
    { 0x87fd277c, 33 },
    { 0x1c6d277a, 95 },
    { 0, 0 },
    { 0x8f0d277f, 81 },
    { 0x532d2780, 61 },
    { 0, 0 },
    { 0xc4ad2782, 23 },
    { 0x4dd2783, 90 },
    { 0xba1d2784, 91 },
    { 0x6fcd2785, 43 },
    { 0xfcbd2786, 90 },
    { 0xfb0d2784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bd278a, 84 },
    { 0x36cd278a, 95 },
    { 0, 0 },
    { 0x618d278d, 55 },
    { 0, 0 },
    { 0x698d278f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14d2793, 22 },
    { 0, 0 },
    { 0xc75d2795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d279b, 43 },
    { 0x362d279c, 77 },
    { 0xe28d279d, 87 },
    { 0xc22d279d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fd27a1, 98 },
    { 0xcb3d27a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd27a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d27aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d27ad, 76 },
    { 0x118d27ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fd27b1, 33 },
    { 0x317d27b1, 23 },
    { 0xaa9d27b2, 66 },
    { 0x65bd27b4, 43 },
    { 0, 0 },
    { 0x961d27b6, 55 },
    { 0x617d27b7, 34 },
    { 0, 0 },
    { 0x179d27b9, 76 },
    { 0xb6cd27b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdd27c1, 95 },
    { 0x927d27c2, 98 },
    { 0xefd27c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d27c9, 95 },
    { 0x712d27ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x935d27cd, 90 },
    { 0xe7ad27ce, 90 },
    { 0x176d27cf, 61 },
    { 0x6e5d27cd, 33 },
    { 0xd20d27ce, 95 },
    { 0x96cd27cf, 90 },
    { 0, 0 },
    { 0x611d27d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49d27d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d27df, 100 },
    { 0, 0 },
    { 0x5bad27e1, 46 },
    { 0, 0 },
    { 0x540d27e3, 98 },
    { 0xa87d27e3, 61 },
    { 0xf94d27e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x443d27e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97d27ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431d27f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1d27f4, 43 },
    { 0xa69d27f4, 66 },
    { 0xad3d27f6, 55 },
    { 0xdd5d27f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d27fb, 61 },
    { 0, 0 },
    { 0xb38d27fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d2800, 90 },
    { 0xddd2801, 90 },
    { 0xe6d2801, 33 },
    { 0, 0 },
    { 0xd4d2804, 98 },
    { 0xc51d2805, 23 },
    { 0x165d2806, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x744d2809, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbd280f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebd2817, 34 },
    { 0xc32d2818, 95 },
    { 0x6c6d2819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38d281c, 34 },
    { 0, 0 },
    { 0x916d281e, 46 },
    { 0xa69d281e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd2823, 98 },
    { 0xda7d2823, 77 },
    { 0, 0 },
    { 0x756d2826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eed282c, 40 },
    { 0, 0 },
    { 0x5ffd282e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd2832, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed2835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2d2839, 61 },
    { 0xaead283a, 65 },
    { 0, 0 },
    { 0x613d283c, 52 },
    { 0, 0 },
    { 0xd8d283e, 91 },
    { 0x9b2d283f, 33 },
    { 0xa68d2840, 95 },
    { 0x42ad2841, 33 },
    { 0x4fad283e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d2846, 98 },
    { 0x4e8d2847, 78 },
    { 0xc54d2847, 55 },
    { 0x852d2849, 66 },
    { 0x368d284a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed284e, 33 },
    { 0xa44d284f, 93 },
    { 0xbbd284e, 90 },
    { 0x5dbd2850, 23 },
    { 0x6aad2852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d2856, 52 },
    { 0x600d2856, 93 },
    { 0x910d2856, 98 },
    { 0x47ad2859, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd2865, 87 },
    { 0xdead2865, 90 },
    { 0x23cd2867, 23 },
    { 0xba9d2868, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d286f, 98 },
    { 0, 0 },
    { 0x65ed2871, 78 },
    { 0xe09d2872, 66 },
    { 0xbad2872, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778d287f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x521d2882, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d2885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1d288a, 60 },
    { 0xd81d288a, 61 },
    { 0xea8d288c, 33 },
    { 0, 0 },
    { 0x361d288e, 55 },
    { 0xb0fd288f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed2893, 95 },
    { 0, 0 },
    { 0xfa9d2895, 23 },
    { 0x7bcd2896, 91 },
    { 0, 0 },
    { 0xf77d2898, 87 },
    { 0, 0 },
    { 0x742d289a, 95 },
    { 0x1bfd289b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386d289f, 43 },
    { 0xd6fd28a0, 98 },
    { 0, 0 },
    { 0x851d28a2, 55 },
    { 0x573d28a3, 98 },
    { 0xf87d28a4, 91 },
    { 0, 0 },
    { 0x723d28a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6d28b0, 91 },
    { 0, 0 },
    { 0xce1d28b2, 68 },
    { 0x97bd28b3, 90 },
    { 0xe97d28b4, 93 },
    { 0xf01d28b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d28bb, 52 },
    { 0x2a4d28bc, 61 },
    { 0, 0 },
    { 0x4a0d28be, 77 },
    { 0, 0 },
    { 0xfc1d28c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969d28c5, 95 },
    { 0xa33d28c5, 55 },
    { 0xbcad28c7, 33 },
    { 0xf7dd28c8, 95 },
    { 0x6dbd28c8, 23 },
    { 0x763d28ca, 78 },
    { 0x19ed28c9, 55 },
    { 0xa88d28cc, 55 },
    { 0x1cdd28cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x434d28d0, 33 },
    { 0, 0 },
    { 0x7f1d28d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d28d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd28da, 87 },
    { 0, 0 },
    { 0x6dfd28dc, 55 },
    { 0xd2ed28dc, 63 },
    { 0, 0 },
    { 0x72ed28df, 84 },
    { 0xcf6d28e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d28e5, 61 },
    { 0, 0 },
    { 0xd25d28e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d28ef, 90 },
    { 0xd6ad28f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd28f3, 95 },
    { 0xda5d28f4, 93 },
    { 0xe3ad28f4, 43 },
    { 0, 0 },
    { 0x386d28f7, 33 },
    { 0xf00d28f8, 95 },
    { 0x966d28f9, 91 },
    { 0xb86d28f9, 61 },
    { 0x9a3d28f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed28fe, 36 },
    { 0, 0 },
    { 0xd55d2900, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x297d2903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd2906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd2909, 98 },
    { 0x722d290a, 90 },
    { 0x5ecd2909, 91 },
    { 0x27ed290c, 61 },
    { 0x48dd290d, 95 },
    { 0x510d290c, 93 },
    { 0, 0 },
    { 0x8d2910, 55 },
    { 0xdefd2911, 33 },
    { 0x71cd2912, 23 },
    { 0, 0 },
    { 0x3fbd2914, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ed2919, 78 },
    { 0x477d2919, 99 },
    { 0x61ad2919, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63d2923, 67 },
    { 0xbc5d2924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d2927, 61 },
    { 0xef0d2928, 66 },
    { 0x152d2929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbd292c, 23 },
    { 0, 0 },
    { 0x43ad292e, 46 },
    { 0x863d292f, 95 },
    { 0x2a9d2930, 90 },
    { 0x243d2931, 64 },
    { 0x8a1d2932, 94 },
    { 0x106d2932, 76 },
    { 0xdb1d292e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d2937, 75 },
    { 0xd56d2938, 43 },
    { 0x32d2939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d293c, 23 },
    { 0x563d293d, 33 },
    { 0x728d293e, 61 },
    { 0x18cd293e, 61 },
    { 0x2d3d293c, 23 },
    { 0x358d2941, 33 },
    { 0x8c6d2942, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6d2947, 46 },
    { 0x1d4d2947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad2952, 91 },
    { 0, 0 },
    { 0x282d2954, 66 },
    { 0, 0 },
    { 0x915d2956, 61 },
    { 0, 0 },
    { 0xebed2958, 90 },
    { 0x405d2958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d295e, 95 },
    { 0x361d295e, 78 },
    { 0, 0 },
    { 0x2fad2961, 93 },
    { 0xccad2962, 90 },
    { 0xd3fd2963, 61 },
    { 0, 0 },
    { 0xa22d2965, 55 },
    { 0, 0 },
    { 0xdcad2967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d296a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd296d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed2971, 23 },
    { 0x5f5d2971, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ed2976, 76 },
    { 0xb93d2976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618d297d, 93 },
    { 0xb03d297e, 98 },
    { 0, 0 },
    { 0x4c4d2980, 33 },
    { 0, 0 },
    { 0x20bd2982, 53 },
    { 0x9bed2983, 34 },
    { 0x719d2984, 61 },
    { 0xb6ed2985, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2d298a, 23 },
    { 0xa58d298b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852d2990, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dd2994, 52 },
    { 0, 0 },
    { 0x83dd2996, 95 },
    { 0x5e6d2996, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd299d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371d29a6, 33 },
    { 0xd59d29a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88d29aa, 52 },
    { 0x8e8d29ab, 100 },
    { 0x56ad29aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd29b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed29b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d29b8, 83 },
    { 0, 0 },
    { 0x500d29ba, 76 },
    { 0, 0 },
    { 0x8a6d29bc, 90 },
    { 0x643d29bd, 83 },
    { 0xcd6d29be, 67 },
    { 0xeabd29bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d29c2, 98 },
    { 0xdbdd29c3, 33 },
    { 0x124d29c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd29c7, 33 },
    { 0x917d29c8, 85 },
    { 0x35bd29c7, 98 },
    { 0xbb9d29c8, 46 },
    { 0, 0 },
    { 0xd1fd29cc, 55 },
    { 0xd84d29cc, 33 },
    { 0x29cd29ce, 23 },
    { 0x699d29cc, 55 },
    { 0x848d29ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d29d3, 98 },
    { 0x2b9d29d4, 90 },
    { 0xaead29d3, 93 },
    { 0xd65d29d4, 90 },
    { 0x5dd29d7, 81 },
    { 0, 0 },
    { 0x951d29d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed29e0, 52 },
    { 0x708d29e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d29e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd29e9, 23 },
    { 0xb7cd29e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d29f3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d29f8, 23 },
    { 0, 0 },
    { 0x175d29fa, 85 },
    { 0xe29d29fb, 100 },
    { 0xa13d29fc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d2a00, 95 },
    { 0x15d2a01, 78 },
    { 0x8f6d2a01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58d2a06, 93 },
    { 0xcf0d2a06, 55 },
    { 0, 0 },
    { 0x4b3d2a09, 84 },
    { 0xb65d2a09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ad2a0f, 61 },
    { 0x127d2a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d2a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2d2a17, 96 },
    { 0, 0 },
    { 0xc6ed2a19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d2a22, 23 },
    { 0, 0 },
    { 0x8e0d2a24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2a2b, 38 },
    { 0x91ad2a2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d2a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1d2a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364d2a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d2a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277d2a43, 95 },
    { 0xdc5d2a44, 66 },
    { 0xd87d2a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddd2a48, 61 },
    { 0x42dd2a49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ed2a4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebd2a55, 90 },
    { 0, 0 },
    { 0xd3bd2a57, 90 },
    { 0xc35d2a58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d2a60, 80 },
    { 0, 0 },
    { 0x5dad2a62, 61 },
    { 0xbf3d2a62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cd2a67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66d2a6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92d2a6f, 76 },
    { 0x88ad2a70, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bd2a74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ad2a7f, 43 },
    { 0, 0 },
    { 0x323d2a81, 95 },
    { 0x632d2a82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6d2a85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d2a8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750d2a8f, 93 },
    { 0xfacd2a8f, 95 },
    { 0, 0 },
    { 0x65fd2a92, 34 },
    { 0x4f0d2a93, 23 },
    { 0x4efd2a93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd2a9e, 90 },
    { 0, 0 },
    { 0xad4d2aa0, 66 },
    { 0x6a5d2aa1, 46 },
    { 0, 0 },
    { 0xd40d2aa3, 78 },
    { 0xecbd2aa3, 55 },
    { 0xcd3d2aa5, 33 },
    { 0x33d2aa3, 98 },
    { 0x72ad2aa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d2aad, 33 },
    { 0x995d2aad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9d2ab1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7add2ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82d2aba, 61 },
    { 0x59fd2aba, 23 },
    { 0x437d2abc, 34 },
    { 0xd58d2abd, 33 },
    { 0x536d2abe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bd2ac2, 84 },
    { 0x845d2ac3, 77 },
    { 0xc1bd2ac3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd2ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd2aca, 76 },
    { 0, 0 },
    { 0xfbfd2acc, 90 },
    { 0xc96d2acd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d2ad4, 61 },
    { 0xa70d2ad4, 95 },
    { 0xb83d2ad4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d2ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd2ae7, 93 },
    { 0x79dd2ae8, 90 },
    { 0xc36d2ae8, 90 },
    { 0x982d2aea, 98 },
    { 0, 0 },
    { 0x399d2aec, 17 },
    { 0x594d2aed, 90 },
    { 0x19ad2aee, 90 },
    { 0, 0 },
    { 0x87cd2af0, 99 },
    { 0xd04d2af1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d2af4, 34 },
    { 0x599d2af5, 95 },
    { 0x68bd2af6, 76 },
    { 0xb93d2af5, 97 },
    { 0xb8d2af4, 67 },
    { 0, 0 },
    { 0xcf8d2afa, 61 },
    { 0, 0 },
    { 0x942d2afc, 93 },
    { 0x7f6d2afd, 19 },
    { 0xd5bd2afc, 95 },
    { 0xb9dd2aff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d2b07, 95 },
    { 0xd42d2b08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d2b0d, 93 },
    { 0xdabd2b0d, 55 },
    { 0xab2d2b0f, 43 },
    { 0, 0 },
    { 0x99ed2b11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d2b14, 90 },
    { 0x2b7d2b15, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafd2b19, 52 },
    { 0x1d1d2b19, 95 },
    { 0xb57d2b1b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded2b1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed2b22, 43 },
    { 0x1e3d2b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4d2b28, 98 },
    { 0x7afd2b29, 90 },
    { 0x569d2b29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed2b2d, 96 },
    { 0x2bcd2b2e, 23 },
    { 0xe8d2b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd2b32, 98 },
    { 0x994d2b33, 61 },
    { 0, 0 },
    { 0x33d2b35, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d2b3a, 43 },
    { 0x38ad2b3b, 34 },
    { 0x31dd2b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ed2b40, 77 },
    { 0xc58d2b40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7d2b45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270d2b4b, 99 },
    { 0xefcd2b4c, 90 },
    { 0x751d2b4d, 61 },
    { 0x33ed2b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d2b53, 90 },
    { 0xa50d2b53, 77 },
    { 0, 0 },
    { 0x809d2b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d2b5b, 90 },
    { 0xe66d2b5c, 61 },
    { 0xbb5d2b5c, 61 },
    { 0x383d2b5e, 43 },
    { 0, 0 },
    { 0xfe9d2b60, 33 },
    { 0, 0 },
    { 0x975d2b62, 23 },
    { 0x68ed2b63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aed2b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142d2b6b, 61 },
    { 0xfafd2b6c, 23 },
    { 0xcffd2b6b, 61 },
    { 0x610d2b6d, 90 },
    { 0xd2ed2b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4d2b74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545d2b79, 53 },
    { 0x9d8d2b7a, 23 },
    { 0xc2fd2b7b, 90 },
    { 0, 0 },
    { 0x5f5d2b7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153d2b82, 80 },
    { 0x28bd2b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd2b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bd2b8d, 55 },
    { 0xafbd2b8e, 61 },
    { 0xaaed2b8f, 95 },
    { 0x33fd2b8f, 61 },
    { 0xcefd2b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aad2b95, 90 },
    { 0x2e2d2b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d2b9a, 33 },
    { 0x1ced2b9a, 33 },
    { 0xa52d2b9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d2ba0, 95 },
    { 0x185d2ba1, 85 },
    { 0x5c7d2ba2, 91 },
    { 0x858d2ba3, 98 },
    { 0x6f9d2ba3, 61 },
    { 0x5b7d2ba5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x855d2ba8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed2bac, 33 },
    { 0x9dad2bad, 67 },
    { 0xf8cd2bae, 95 },
    { 0xc7bd2bad, 55 },
    { 0, 0 },
    { 0x5fbd2bb1, 69 },
    { 0xe4cd2bb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483d2bb7, 23 },
    { 0x7e7d2bb8, 93 },
    { 0x3dd2bb8, 55 },
    { 0xe6ad2bb9, 95 },
    { 0, 0 },
    { 0x245d2bbc, 69 },
    { 0xd2ad2bbd, 23 },
    { 0x73bd2bbe, 81 },
    { 0, 0 },
    { 0xd9ed2bc0, 38 },
    { 0x767d2bc1, 90 },
    { 0x176d2bc2, 77 },
    { 0x36ad2bc1, 85 },
    { 0x3b8d2bc3, 56 },
    { 0x223d2bc3, 90 },
    { 0xd6cd2bc6, 33 },
    { 0x2efd2bc2, 61 },
    { 0x72d2bc3, 75 },
    { 0xb35d2bc9, 78 },
    { 0x10cd2bca, 61 },
    { 0, 0 },
    { 0x83ed2bcc, 93 },
    { 0xd8fd2bcd, 55 },
    { 0x6a6d2bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d2bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9d2bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2bdf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddd2beb, 33 },
    { 0x1efd2beb, 80 },
    { 0xe39d2bed, 55 },
    { 0xc6ed2bee, 65 },
    { 0xd25d2bef, 91 },
    { 0x79fd2bef, 100 },
    { 0xcfcd2bf1, 64 },
    { 0x2eed2beb, 90 },
    { 0, 0 },
    { 0x6b4d2bf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d2bfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24d2bff, 33 },
    { 0x614d2c00, 43 },
    { 0x5f0d2c01, 76 },
    { 0xa8ed2c02, 76 },
    { 0xc09d2c02, 61 },
    { 0x72ed2c04, 34 },
    { 0x289d2c05, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d2c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1d2c0c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d2c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd2c18, 84 },
    { 0, 0 },
    { 0x3eed2c1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d2c1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d2c26, 95 },
    { 0x72fd2c27, 34 },
    { 0x20cd2c28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d2c31, 77 },
    { 0, 0 },
    { 0xa9fd2c33, 61 },
    { 0x15ed2c33, 90 },
    { 0, 0 },
    { 0x65bd2c36, 95 },
    { 0x4a7d2c37, 75 },
    { 0x649d2c38, 90 },
    { 0x194d2c39, 55 },
    { 0x246d2c3a, 65 },
    { 0xb31d2c3b, 67 },
    { 0xbebd2c36, 61 },
    { 0, 0 },
    { 0x354d2c3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aad2c42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd2c49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d2c4c, 90 },
    { 0x19cd2c4d, 46 },
    { 0x64ed2c4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0d2c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9d2c57, 95 },
    { 0xe6ad2c58, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8d2c5b, 90 },
    { 0xd01d2c5c, 61 },
    { 0xd20d2c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2d2c61, 33 },
    { 0x3c2d2c62, 34 },
    { 0xfebd2c63, 61 },
    { 0, 0 },
    { 0x76fd2c65, 56 },
    { 0xe36d2c65, 98 },
    { 0xa22d2c67, 33 },
    { 0, 0 },
    { 0x54ed2c69, 100 },
    { 0xe54d2c6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edd2c76, 98 },
    { 0xdd6d2c76, 33 },
    { 0xfbed2c76, 23 },
    { 0x20d2c78, 90 },
    { 0, 0 },
    { 0xa77d2c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08d2c7f, 61 },
    { 0xcf3d2c80, 87 },
    { 0x3b4d2c81, 77 },
    { 0, 0 },
    { 0xf9bd2c83, 34 },
    { 0x70ed2c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbd2c88, 61 },
    { 0, 0 },
    { 0xba3d2c8a, 76 },
    { 0x606d2c8b, 33 },
    { 0, 0 },
    { 0x115d2c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d2c90, 98 },
    { 0x4bfd2c90, 90 },
    { 0x42fd2c92, 61 },
    { 0xfe9d2c93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d2c96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbd2ca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9d2caa, 33 },
    { 0, 0 },
    { 0xf1dd2cac, 78 },
    { 0xaefd2cad, 61 },
    { 0xdccd2cad, 55 },
    { 0x271d2cac, 61 },
    { 0, 0 },
    { 0xc52d2cb1, 55 },
    { 0xc29d2cb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d2cb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8d2cbb, 91 },
    { 0xd99d2cbc, 90 },
    { 0x17d2cbd, 84 },
    { 0, 0 },
    { 0x7d2d2cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bd2cc3, 46 },
    { 0, 0 },
    { 0xa50d2cc5, 75 },
    { 0, 0 },
    { 0x41dd2cc7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x622d2cca, 66 },
    { 0x8afd2cca, 95 },
    { 0xf61d2ccc, 95 },
    { 0x737d2ccc, 61 },
    { 0, 0 },
    { 0x414d2ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbed2cd3, 98 },
    { 0x852d2cd4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d2ce0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcd2ce3, 78 },
    { 0x894d2ce4, 91 },
    { 0x41ad2ce5, 55 },
    { 0xdd1d2ce6, 91 },
    { 0, 0 },
    { 0x633d2ce8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2cec, 93 },
    { 0x86ed2ced, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2d2cf1, 33 },
    { 0, 0 },
    { 0x994d2cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d2cf6, 85 },
    { 0xbf3d2cf7, 78 },
    { 0x86cd2cf7, 55 },
    { 0xff2d2cf9, 23 },
    { 0x3d6d2cfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8d2d01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d2d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50d2d0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432d2d14, 66 },
    { 0x55ad2d15, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82d2d18, 33 },
    { 0xb2ad2d19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96d2d1f, 87 },
    { 0x1e7d2d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ad2d24, 33 },
    { 0x485d2d24, 61 },
    { 0, 0 },
    { 0xced2d27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd2d2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebd2d30, 33 },
    { 0, 0 },
    { 0xcb2d2d32, 76 },
    { 0xabed2d33, 91 },
    { 0x717d2d34, 100 },
    { 0x9e1d2d34, 90 },
    { 0x3b1d2d34, 84 },
    { 0xfefd2d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d2d3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd2d45, 61 },
    { 0x298d2d46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd2d4c, 95 },
    { 0xe8d2d4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678d2d53, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2d58, 95 },
    { 0x2bd2d59, 77 },
    { 0, 0 },
    { 0x7e3d2d5b, 90 },
    { 0, 0 },
    { 0x27cd2d5d, 93 },
    { 0x82bd2d5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93d2d61, 77 },
    { 0x785d2d61, 95 },
    { 0x738d2d63, 43 },
    { 0x677d2d61, 66 },
    { 0x1f7d2d65, 43 },
    { 0xc2fd2d66, 98 },
    { 0xcf2d2d67, 90 },
    { 0x77cd2d68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd2d6b, 33 },
    { 0, 0 },
    { 0x34ed2d6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616d2d76, 93 },
    { 0x9b5d2d77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7d2d7a, 33 },
    { 0x429d2d7b, 76 },
    { 0xfd3d2d7c, 34 },
    { 0xfb6d2d7c, 90 },
    { 0xb43d2d7a, 45 },
    { 0xd45d2d7e, 95 },
    { 0x450d2d7b, 33 },
    { 0xe2cd2d7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78d2d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326d2d89, 2 },
    { 0, 0 },
    { 0xaa5d2d8b, 23 },
    { 0, 0 },
    { 0x11fd2d8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d2d92, 66 },
    { 0, 0 },
    { 0xe72d2d94, 98 },
    { 0xaced2d95, 61 },
    { 0x902d2d96, 61 },
    { 0xa04d2d94, 55 },
    { 0x1efd2d98, 76 },
    { 0xf3bd2d98, 87 },
    { 0xa0d2d99, 23 },
    { 0xcfbd2d9b, 33 },
    { 0x5a5d2d9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed2da0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ed2da3, 55 },
    { 0x9b3d2da4, 33 },
    { 0xc62d2da5, 55 },
    { 0x7d7d2da6, 90 },
    { 0xd49d2da7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d2daa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d2db5, 55 },
    { 0xe0bd2db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d2dba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686d2dbf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4d2dc2, 23 },
    { 0x36bd2dc2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d2dc7, 33 },
    { 0, 0 },
    { 0x418d2dc9, 77 },
    { 0x2bad2dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x589d2dcd, 87 },
    { 0x65fd2dce, 90 },
    { 0xd5d2dcd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d2dd2, 76 },
    { 0, 0 },
    { 0xdcad2dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0d2dda, 90 },
    { 0xc3d2ddb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd2ddf, 96 },
    { 0x144d2ddf, 90 },
    { 0xe84d2de1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32d2de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd2deb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571d2df0, 23 },
    { 0xe3fd2df1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5d2df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ed2df9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d2e00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd2e04, 61 },
    { 0xd1bd2e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedd2e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ad2e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6d2e14, 61 },
    { 0x37ed2e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d2e19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569d2e1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d2e2c, 61 },
    { 0, 0 },
    { 0xa2d2e2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2e31, 33 },
    { 0, 0 },
    { 0x9a6d2e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39d2e3a, 67 },
    { 0xe54d2e3b, 95 },
    { 0x918d2e3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd2e40, 95 },
    { 0x82ad2e41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2d2e46, 95 },
    { 0x237d2e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d2e4a, 34 },
    { 0xbf9d2e4a, 75 },
    { 0, 0 },
    { 0x928d2e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d2e57, 77 },
    { 0, 0 },
    { 0xc88d2e59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x683d2e5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffd2e64, 90 },
    { 0x5d2e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bd2e6b, 33 },
    { 0xbd8d2e6c, 98 },
    { 0, 0 },
    { 0xe2dd2e6e, 53 },
    { 0xfbd2e6f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x207d2e72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d2e78, 99 },
    { 0x3c7d2e78, 55 },
    { 0x8d3d2e7a, 33 },
    { 0x97ed2e7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d2e80, 61 },
    { 0, 0 },
    { 0x6bd2e82, 90 },
    { 0xefdd2e83, 78 },
    { 0x772d2e84, 98 },
    { 0, 0 },
    { 0x37ad2e86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0d2e8a, 98 },
    { 0x4e6d2e8b, 95 },
    { 0xf1ad2e8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d2e90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cd2e93, 95 },
    { 0, 0 },
    { 0x280d2e95, 97 },
    { 0, 0 },
    { 0xb99d2e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d2e9b, 98 },
    { 0x7aed2e9b, 90 },
    { 0xde5d2e9d, 33 },
    { 0x461d2e9b, 40 },
    { 0, 0 },
    { 0xde5d2ea0, 95 },
    { 0x9d9d2ea0, 95 },
    { 0, 0 },
    { 0x7a1d2ea3, 98 },
    { 0x187d2ea4, 90 },
    { 0xad2ea3, 76 },
    { 0x943d2ea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd2eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4d2ead, 46 },
    { 0x431d2ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443d2ebc, 84 },
    { 0xfcd2ebd, 39 },
    { 0xbabd2ebe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bd2ec4, 46 },
    { 0x2a0d2ec5, 90 },
    { 0x6dcd2ec6, 93 },
    { 0, 0 },
    { 0xe6fd2ec8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x658d2ecb, 40 },
    { 0x40d2ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9d2ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d2ed4, 84 },
    { 0x69cd2ed5, 90 },
    { 0xb18d2ed6, 34 },
    { 0x369d2ed7, 87 },
    { 0x151d2ed5, 99 },
    { 0, 0 },
    { 0x24d2eda, 90 },
    { 0, 0 },
    { 0x4b5d2edc, 66 },
    { 0, 0 },
    { 0x293d2ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d2ee6, 52 },
    { 0xb85d2ee7, 43 },
    { 0x1e4d2ee8, 55 },
    { 0x4e2d2ee7, 90 },
    { 0x308d2ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d2ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bd2efa, 33 },
    { 0x432d2efb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddd2efe, 90 },
    { 0xfbd2efe, 61 },
    { 0xadad2f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353d2f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d2f14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd2f17, 55 },
    { 0x5f6d2f18, 100 },
    { 0x437d2f19, 77 },
    { 0x8aed2f18, 34 },
    { 0xf84d2f1b, 90 },
    { 0x8bed2f18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914d2f25, 61 },
    { 0xa98d2f26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd2f2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ed2f30, 55 },
    { 0x8f8d2f30, 34 },
    { 0xc0cd2f30, 55 },
    { 0xfcad2f33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6d2f36, 99 },
    { 0x51d2f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d2f3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d2f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d2f47, 61 },
    { 0x40d2f48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ed2f4f, 90 },
    { 0xd71d2f4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d2f53, 88 },
    { 0x81dd2f54, 76 },
    { 0x987d2f53, 64 },
    { 0x163d2f56, 76 },
    { 0xab6d2f57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d2f5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfd2f5f, 95 },
    { 0x6b3d2f60, 90 },
    { 0xb84d2f60, 98 },
    { 0x827d2f62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fd2f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d2f6c, 61 },
    { 0x81d2f6d, 67 },
    { 0xdafd2f6d, 90 },
    { 0xcf1d2f6f, 61 },
    { 0x7f8d2f70, 98 },
    { 0x3f1d2f71, 90 },
    { 0x9c3d2f70, 76 },
    { 0x822d2f73, 61 },
    { 0x825d2f73, 76 },
    { 0, 0 },
    { 0xda1d2f76, 77 },
    { 0x739d2f76, 55 },
    { 0xa8d2f78, 95 },
    { 0x485d2f78, 61 },
    { 0x213d2f7a, 33 },
    { 0x90cd2f7b, 91 },
    { 0xaf5d2f7a, 90 },
    { 0x7dbd2f7d, 98 },
    { 0x4bdd2f7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d2f82, 99 },
    { 0, 0 },
    { 0x40dd2f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbd2f87, 95 },
    { 0x3cad2f88, 61 },
    { 0x462d2f89, 99 },
    { 0, 0 },
    { 0xf87d2f8b, 33 },
    { 0xd21d2f8b, 33 },
    { 0xf8dd2f8d, 98 },
    { 0x53fd2f8e, 33 },
    { 0, 0 },
    { 0x9b5d2f90, 93 },
    { 0x6e1d2f91, 33 },
    { 0xe45d2f92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bd2f97, 61 },
    { 0, 0 },
    { 0x9d1d2f99, 46 },
    { 0xa8fd2f9a, 33 },
    { 0xec6d2f9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dd2fa0, 90 },
    { 0x571d2fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dd2fa6, 61 },
    { 0x6e2d2fa7, 90 },
    { 0x306d2fa7, 91 },
    { 0, 0 },
    { 0xa17d2faa, 66 },
    { 0xcd6d2faa, 43 },
    { 0x822d2fac, 61 },
    { 0xa27d2fad, 95 },
    { 0xbed2fae, 33 },
    { 0xd82d2faf, 98 },
    { 0x533d2fae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2d2fbb, 33 },
    { 0x2bbd2fbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d2fc2, 90 },
    { 0, 0 },
    { 0x702d2fc4, 33 },
    { 0xdcd2fc4, 90 },
    { 0x110d2fc5, 55 },
    { 0xdbad2fc7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704d2fcf, 23 },
    { 0, 0 },
    { 0x1a0d2fd1, 55 },
    { 0xd22d2fd1, 61 },
    { 0, 0 },
    { 0xeed2fd4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d2fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1d2fe0, 88 },
    { 0, 0 },
    { 0x942d2fe2, 78 },
    { 0, 0 },
    { 0xc32d2fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d2ff1, 75 },
    { 0xa4cd2ff2, 61 },
    { 0xd69d2ff1, 84 },
    { 0x1b0d2ff4, 95 },
    { 0x14cd2ff1, 33 },
    { 0x874d2ff6, 61 },
    { 0xe8bd2ff7, 43 },
    { 0xc09d2ff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d2ffb, 61 },
    { 0x5e0d2ffb, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd2fff, 66 },
    { 0, 0 },
    { 0x5dcd3001, 85 },
    { 0xeb8d3001, 85 },
    { 0, 0 },
    { 0x12cd3004, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd3008, 98 },
    { 0, 0 },
    { 0x2f9d300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7add300e, 95 },
    { 0xc85d300f, 99 },
    { 0x731d3010, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ad3014, 90 },
    { 0x81dd3014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836d3019, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904d301f, 90 },
    { 0, 0 },
    { 0x85ad3021, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04d3024, 76 },
    { 0, 0 },
    { 0x9a9d3026, 98 },
    { 0x2cbd3027, 98 },
    { 0, 0 },
    { 0x7efd3029, 81 },
    { 0xb53d302a, 95 },
    { 0xccdd302a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d302f, 55 },
    { 0, 0 },
    { 0x821d3031, 23 },
    { 0, 0 },
    { 0x5e5d3033, 33 },
    { 0xc88d3034, 91 },
    { 0x6c5d3035, 53 },
    { 0xe04d3035, 75 },
    { 0xfded3034, 100 },
    { 0xb10d3038, 76 },
    { 0x7dad3038, 43 },
    { 0x1e0d303a, 34 },
    { 0x1bd3035, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefd303f, 46 },
    { 0, 0 },
    { 0xd47d3041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd3044, 76 },
    { 0, 0 },
    { 0x5d2d3046, 99 },
    { 0, 0 },
    { 0x7d7d3048, 43 },
    { 0, 0 },
    { 0x858d304a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bd3050, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d3053, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36d3056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d3059, 98 },
    { 0x23cd305a, 75 },
    { 0, 0 },
    { 0x7c2d305c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd305f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466d3063, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8d3066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2d306b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d306e, 81 },
    { 0x7a9d306e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0d3073, 95 },
    { 0x61ad3074, 65 },
    { 0x346d3075, 99 },
    { 0, 0 },
    { 0xa5ad3077, 76 },
    { 0xb41d3078, 76 },
    { 0x199d3079, 33 },
    { 0x4fad3079, 33 },
    { 0x9d7d307b, 76 },
    { 0xfa9d3079, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fd307f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7d3084, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730d3089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd3090, 61 },
    { 0, 0 },
    { 0xc8bd3092, 93 },
    { 0x6a9d3093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5d3096, 91 },
    { 0xfe1d3097, 23 },
    { 0x2a5d3098, 23 },
    { 0x46fd3097, 23 },
    { 0xa12d3098, 76 },
    { 0xb8bd309a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65d30a1, 34 },
    { 0x908d30a2, 76 },
    { 0, 0 },
    { 0x640d30a4, 33 },
    { 0xc74d30a5, 69 },
    { 0x4d8d30a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487d30ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853d30b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d30bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cd30c1, 61 },
    { 0xa12d30c1, 93 },
    { 0x5d2d30c2, 98 },
    { 0x94dd30c4, 98 },
    { 0, 0 },
    { 0x14dd30c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d30cd, 36 },
    { 0xc65d30cd, 90 },
    { 0x890d30cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd30d2, 90 },
    { 0, 0 },
    { 0x78ed30d4, 40 },
    { 0, 0 },
    { 0x50bd30d6, 33 },
    { 0x156d30d7, 23 },
    { 0, 0 },
    { 0x6a1d30d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fd30dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976d30e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d30eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406d30ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d30f6, 87 },
    { 0x776d30f7, 52 },
    { 0xb66d30f8, 84 },
    { 0, 0 },
    { 0x368d30fa, 90 },
    { 0, 0 },
    { 0x841d30fc, 78 },
    { 0xbc1d30fd, 76 },
    { 0x4e9d30fd, 61 },
    { 0x3ccd30fc, 19 },
    { 0x8a6d3100, 23 },
    { 0xd4dd3101, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfd3104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80d3107, 61 },
    { 0x795d3108, 76 },
    { 0x461d3108, 24 },
    { 0x8f5d310a, 51 },
    { 0xe72d310b, 46 },
    { 0x96fd310c, 95 },
    { 0xac8d310c, 76 },
    { 0x438d310a, 33 },
    { 0x6c7d310d, 69 },
    { 0xcdcd3110, 34 },
    { 0, 0 },
    { 0x809d3112, 33 },
    { 0x48fd3113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d311a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d3123, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472d312a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd3130, 61 },
    { 0, 0 },
    { 0xbefd3132, 61 },
    { 0x70cd3133, 98 },
    { 0, 0 },
    { 0x5abd3135, 91 },
    { 0x8edd3135, 90 },
    { 0x442d3137, 67 },
    { 0x114d3138, 33 },
    { 0xf15d3139, 76 },
    { 0xcfed3139, 34 },
    { 0x8edd313b, 98 },
    { 0xd2cd3137, 34 },
    { 0x4a6d313d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ad3142, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ed3145, 43 },
    { 0xc58d3146, 95 },
    { 0x4bed3146, 82 },
    { 0x963d3146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d314c, 23 },
    { 0, 0 },
    { 0x5fad314e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d3151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd3154, 69 },
    { 0x62ad3155, 46 },
    { 0xcbed3154, 43 },
    { 0, 0 },
    { 0xeb1d3158, 33 },
    { 0x6c7d3159, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668d315c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ad3162, 90 },
    { 0, 0 },
    { 0xed3164, 33 },
    { 0xa72d3165, 90 },
    { 0xa7d3165, 95 },
    { 0x7d3166, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d316d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87d3171, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d3174, 98 },
    { 0, 0 },
    { 0x528d3176, 36 },
    { 0x925d3177, 61 },
    { 0x608d3178, 61 },
    { 0, 0 },
    { 0x61fd317a, 90 },
    { 0x4e8d317b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d317e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d3183, 52 },
    { 0, 0 },
    { 0x528d3185, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ded318b, 33 },
    { 0x7c9d318c, 93 },
    { 0x580d318c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9d3194, 55 },
    { 0, 0 },
    { 0xce4d3196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d319c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad31a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25d31a9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d31ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd31b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd31b5, 66 },
    { 0x44ad31b5, 95 },
    { 0xeb9d31b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed31ba, 23 },
    { 0xacdd31bb, 33 },
    { 0x978d31ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4d31c0, 46 },
    { 0, 0 },
    { 0x174d31c2, 40 },
    { 0x749d31c2, 91 },
    { 0x8cdd31c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7d31ce, 95 },
    { 0xda0d31cf, 52 },
    { 0xa0ad31d0, 23 },
    { 0xea7d31d1, 34 },
    { 0x89d31d2, 88 },
    { 0xee9d31d1, 23 },
    { 0xc34d31d1, 34 },
    { 0xd28d31d5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd31dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d31e1, 33 },
    { 0xef9d31e2, 33 },
    { 0, 0 },
    { 0x69fd31e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfd31e9, 23 },
    { 0xd07d31ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2d31ef, 33 },
    { 0, 0 },
    { 0x721d31f1, 77 },
    { 0x2b0d31f2, 93 },
    { 0, 0 },
    { 0x3b1d31f4, 61 },
    { 0, 0 },
    { 0x3b5d31f6, 33 },
    { 0, 0 },
    { 0x5ebd31f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ad31fb, 33 },
    { 0x4f7d31fc, 23 },
    { 0x657d31fd, 87 },
    { 0x438d31fe, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd3201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad3204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6d320b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16d320e, 34 },
    { 0x747d320f, 61 },
    { 0x6acd3210, 23 },
    { 0x8a8d3211, 33 },
    { 0xa90d3212, 95 },
    { 0x7b7d3213, 76 },
    { 0x7bcd3214, 76 },
    { 0, 0 },
    { 0xf6fd3216, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fd3219, 95 },
    { 0, 0 },
    { 0xdf6d321b, 61 },
    { 0xff7d321c, 90 },
    { 0xb2fd321d, 46 },
    { 0xb83d321d, 23 },
    { 0xc05d321f, 33 },
    { 0xd91d3220, 33 },
    { 0x762d3221, 92 },
    { 0xf18d3222, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7d3227, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5d322a, 33 },
    { 0, 0 },
    { 0xb5bd322c, 55 },
    { 0xd7ad322c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd3232, 55 },
    { 0xe2dd3232, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bd3236, 68 },
    { 0xf0fd3236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d323c, 98 },
    { 0xaafd323d, 99 },
    { 0x525d323e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d3241, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad3246, 61 },
    { 0x671d3247, 98 },
    { 0x358d3246, 46 },
    { 0xdcbd3247, 91 },
    { 0xf19d324a, 43 },
    { 0, 0 },
    { 0x61dd324c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1d324f, 61 },
    { 0xa09d3250, 61 },
    { 0x465d3251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362d3255, 93 },
    { 0, 0 },
    { 0xbcbd3257, 76 },
    { 0x30cd3258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39d325d, 33 },
    { 0x368d325e, 61 },
    { 0xdb0d325d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdd326a, 68 },
    { 0, 0 },
    { 0x1f1d326c, 98 },
    { 0x5ddd326d, 33 },
    { 0x3ded326d, 33 },
    { 0xb6bd326f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d3272, 61 },
    { 0xf69d3272, 90 },
    { 0x2a3d3272, 61 },
    { 0x8efd3275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecd327b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bed3283, 95 },
    { 0, 0 },
    { 0x4bbd3285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd3288, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd328c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3290, 95 },
    { 0x52ed3290, 23 },
    { 0x234d3292, 46 },
    { 0x78cd3292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5d3297, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d329d, 93 },
    { 0x1c7d329e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50d32a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3d32a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dad32b1, 98 },
    { 0xa9ad32b2, 55 },
    { 0, 0 },
    { 0x7c9d32b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed32be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d32c2, 90 },
    { 0, 0 },
    { 0x1fdd32c4, 53 },
    { 0xb40d32c5, 98 },
    { 0, 0 },
    { 0x541d32c7, 76 },
    { 0x6fed32c7, 46 },
    { 0, 0 },
    { 0x2dd32ca, 90 },
    { 0x996d32ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd32d2, 17 },
    { 0xa96d32d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89d32d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd32dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99d32df, 90 },
    { 0x4fdd32df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85d32e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d32ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d32ed, 61 },
    { 0x1e7d32ed, 61 },
    { 0x12bd32ed, 36 },
    { 0x4f7d32ef, 17 },
    { 0x89ad32f1, 17 },
    { 0xa5d32f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41d32fb, 95 },
    { 0, 0 },
    { 0x220d32fd, 66 },
    { 0, 0 },
    { 0xa84d32ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x676d3302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dd331b, 66 },
    { 0x4e3d331b, 33 },
    { 0x84d331d, 33 },
    { 0, 0 },
    { 0xad0d331f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d3323, 78 },
    { 0xf83d3324, 55 },
    { 0xdb7d3325, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49d3328, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764d332f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd3333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d3336, 68 },
    { 0xeddd3336, 61 },
    { 0x738d3338, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65d333b, 23 },
    { 0, 0 },
    { 0xbb1d333d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x312d3340, 90 },
    { 0x112d3341, 99 },
    { 0xb1ed3340, 23 },
    { 0x473d3343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0d3347, 98 },
    { 0xa3ad3347, 33 },
    { 0, 0 },
    { 0x76ad334a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21d334d, 33 },
    { 0x60ed334e, 98 },
    { 0, 0 },
    { 0xa09d3350, 61 },
    { 0x609d3351, 33 },
    { 0x180d3352, 53 },
    { 0, 0 },
    { 0x74fd3354, 98 },
    { 0xd53d3355, 43 },
    { 0x91dd3355, 90 },
    { 0x58dd3357, 46 },
    { 0xdabd3355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426d335e, 76 },
    { 0xd6dd335e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0d3362, 85 },
    { 0xf59d3363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead3366, 90 },
    { 0xde9d3367, 34 },
    { 0xb72d3366, 61 },
    { 0, 0 },
    { 0x18d336a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43d336d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6d3370, 34 },
    { 0, 0 },
    { 0xde9d3372, 33 },
    { 0xa19d3373, 90 },
    { 0x291d3374, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd3377, 33 },
    { 0xa1ed3377, 55 },
    { 0, 0 },
    { 0x5bcd337a, 99 },
    { 0x155d337b, 23 },
    { 0x309d337c, 61 },
    { 0x84dd337a, 84 },
    { 0xe46d337a, 95 },
    { 0xf34d337b, 90 },
    { 0, 0 },
    { 0xef8d3381, 55 },
    { 0, 0 },
    { 0x379d3383, 66 },
    { 0xb6d3384, 33 },
    { 0x55bd3384, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d338c, 23 },
    { 0x2e9d338d, 69 },
    { 0xc8ad338d, 93 },
    { 0xaa0d338f, 33 },
    { 0x372d3390, 83 },
    { 0x368d3391, 88 },
    { 0x15fd3390, 23 },
    { 0x2b9d3391, 93 },
    { 0x87cd3394, 87 },
    { 0, 0 },
    { 0xf33d3396, 46 },
    { 0, 0 },
    { 0x9b8d3398, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d339d, 36 },
    { 0, 0 },
    { 0x944d339f, 67 },
    { 0x3ffd33a0, 43 },
    { 0xf84d33a1, 61 },
    { 0, 0 },
    { 0xdf6d33a3, 61 },
    { 0x4f6d33a4, 90 },
    { 0x47fd33a5, 87 },
    { 0x733d33a6, 93 },
    { 0, 0 },
    { 0x28d33a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d33b0, 93 },
    { 0xf64d33b1, 95 },
    { 0x58cd33b2, 66 },
    { 0xdc3d33b1, 33 },
    { 0xe4dd33b4, 78 },
    { 0x13cd33b4, 40 },
    { 0x267d33b1, 33 },
    { 0x2d4d33b7, 87 },
    { 0x6f0d33b8, 34 },
    { 0x577d33b9, 65 },
    { 0xd73d33ba, 43 },
    { 0xa10d33bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd33bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd33c2, 76 },
    { 0, 0 },
    { 0xfb6d33c4, 55 },
    { 0xb0fd33c5, 98 },
    { 0, 0 },
    { 0x415d33c7, 38 },
    { 0xf90d33c7, 23 },
    { 0xa6ed33c9, 93 },
    { 0x816d33c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0d33cd, 88 },
    { 0xcc2d33cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d33d6, 61 },
    { 0, 0 },
    { 0xb84d33d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ad33db, 34 },
    { 0, 0 },
    { 0x4a0d33dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d33e0, 23 },
    { 0x99ad33e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d33e5, 76 },
    { 0x47fd33e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573d33ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed33f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96d33f7, 90 },
    { 0x2e9d33f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd33fc, 90 },
    { 0xbced33fd, 61 },
    { 0, 0 },
    { 0x2dcd33ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d3405, 52 },
    { 0xbfcd3406, 91 },
    { 0xf8fd3407, 98 },
    { 0xc1cd3407, 98 },
    { 0xe7bd3409, 76 },
    { 0xc39d3405, 33 },
    { 0x73d340a, 95 },
    { 0x60dd340c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34d3410, 61 },
    { 0xc1dd3411, 61 },
    { 0xf4cd3412, 33 },
    { 0x699d3411, 93 },
    { 0x497d3414, 100 },
    { 0xeabd3412, 34 },
    { 0x3fd3413, 88 },
    { 0x8b7d3417, 61 },
    { 0x68bd3412, 98 },
    { 0x13ad3419, 87 },
    { 0xf40d341a, 78 },
    { 0xf21d3416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d341e, 23 },
    { 0xba2d341e, 85 },
    { 0, 0 },
    { 0x27bd3421, 23 },
    { 0x7ed3421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254d342c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d342f, 95 },
    { 0x803d3430, 78 },
    { 0, 0 },
    { 0x50bd3432, 90 },
    { 0x80ed3433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbd3437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7d3440, 90 },
    { 0x60d3441, 85 },
    { 0x553d3441, 23 },
    { 0x86ad3440, 43 },
    { 0xc01d3444, 66 },
    { 0x93ed3442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975d344b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d344e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9d3451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed3456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0d345a, 66 },
    { 0xe97d345a, 36 },
    { 0, 0 },
    { 0xcad345d, 75 },
    { 0x659d345e, 93 },
    { 0xc4ed345e, 33 },
    { 0x3a7d3460, 78 },
    { 0, 0 },
    { 0xc70d3462, 66 },
    { 0x111d3463, 90 },
    { 0xadfd3464, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ad346c, 76 },
    { 0x191d346c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4d3471, 65 },
    { 0x209d3471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77d3477, 61 },
    { 0x2cfd3477, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdd347c, 33 },
    { 0xbd3d347d, 90 },
    { 0, 0 },
    { 0xdb1d347f, 76 },
    { 0, 0 },
    { 0xc06d3481, 61 },
    { 0, 0 },
    { 0xa4dd3483, 95 },
    { 0xe9ad3484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d3487, 76 },
    { 0, 0 },
    { 0xaad3489, 87 },
    { 0x56ed348a, 90 },
    { 0xaa0d348b, 61 },
    { 0x9c0d348b, 33 },
    { 0xf64d348d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d3491, 66 },
    { 0x812d3492, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d3498, 88 },
    { 0xb4cd3499, 23 },
    { 0, 0 },
    { 0xb73d349b, 95 },
    { 0, 0 },
    { 0xb76d349d, 34 },
    { 0, 0 },
    { 0x558d349f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fed34a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed34a6, 91 },
    { 0x5a8d34a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd34aa, 98 },
    { 0x7a4d34ab, 52 },
    { 0xf01d34aa, 23 },
    { 0, 0 },
    { 0x67ad34ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd34b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d34b9, 75 },
    { 0, 0 },
    { 0xd3ed34bb, 91 },
    { 0xa6d34bc, 43 },
    { 0, 0 },
    { 0x64d34be, 98 },
    { 0, 0 },
    { 0xef0d34c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d34c5, 23 },
    { 0, 0 },
    { 0x39d34c7, 80 },
    { 0x202d34c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacd34cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ed34cf, 53 },
    { 0, 0 },
    { 0x7b0d34d1, 19 },
    { 0x896d34d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4d34d6, 84 },
    { 0xc7d34d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x116d34da, 61 },
    { 0x538d34db, 33 },
    { 0, 0 },
    { 0x222d34dd, 43 },
    { 0x68ad34de, 33 },
    { 0xa89d34df, 98 },
    { 0xd0dd34de, 40 },
    { 0x5c4d34e1, 23 },
    { 0xde2d34df, 55 },
    { 0xc6dd34e3, 34 },
    { 0x1c6d34e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bd34e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d34f0, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605d34f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebd34f7, 95 },
    { 0, 0 },
    { 0x165d34f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d34fc, 68 },
    { 0, 0 },
    { 0x80fd34fe, 33 },
    { 0xbb1d34ff, 90 },
    { 0xba9d34ff, 23 },
    { 0xbd0d3501, 90 },
    { 0, 0 },
    { 0x30dd3503, 90 },
    { 0xbf5d3504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d350a, 55 },
    { 0, 0 },
    { 0xbd9d350c, 99 },
    { 0x29ad350d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2d3512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d3517, 95 },
    { 0xb2dd3517, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbded351c, 33 },
    { 0, 0 },
    { 0x90dd351e, 61 },
    { 0x6f4d351f, 90 },
    { 0, 0 },
    { 0xf5dd3521, 61 },
    { 0x8e3d3522, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22d3527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dd352c, 98 },
    { 0x35bd352c, 55 },
    { 0x376d352d, 95 },
    { 0x793d352f, 46 },
    { 0, 0 },
    { 0x5fed3531, 91 },
    { 0xc16d3531, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125d353d, 98 },
    { 0xc2d353e, 90 },
    { 0x929d353e, 77 },
    { 0xcbd353e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x312d3543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd3546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d354a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbd3550, 95 },
    { 0x11bd3550, 55 },
    { 0x9d8d3552, 90 },
    { 0x38d3553, 90 },
    { 0, 0 },
    { 0x769d3555, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d355a, 61 },
    { 0x516d355b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d3562, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d3566, 55 },
    { 0, 0 },
    { 0xb60d3568, 84 },
    { 0, 0 },
    { 0x1ecd356a, 78 },
    { 0, 0 },
    { 0x467d356c, 95 },
    { 0, 0 },
    { 0x6a1d356e, 33 },
    { 0x526d356f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d3572, 33 },
    { 0x3a5d3572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd3576, 23 },
    { 0xd01d3577, 61 },
    { 0xe3dd3576, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1d357b, 55 },
    { 0xb28d357b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ed3584, 74 },
    { 0x837d3585, 61 },
    { 0x91ed3584, 61 },
    { 0, 0 },
    { 0x58bd3588, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d358c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebd3591, 61 },
    { 0xc3fd3591, 33 },
    { 0x462d3593, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ad359c, 23 },
    { 0x1d4d359d, 95 },
    { 0x9e2d359e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8d35a3, 34 },
    { 0, 0 },
    { 0xdf3d35a5, 33 },
    { 0xfe0d35a5, 61 },
    { 0x663d35a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcd35b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fd35b9, 87 },
    { 0xc5d35b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d35be, 76 },
    { 0, 0 },
    { 0x118d35c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4d35c5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcd35d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429d35d5, 33 },
    { 0x3c2d35d6, 55 },
    { 0x9a8d35d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d35da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad35de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d35e3, 43 },
    { 0, 0 },
    { 0x5e8d35e5, 85 },
    { 0xd97d35e6, 90 },
    { 0xfcfd35e7, 34 },
    { 0xf09d35e8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60d35ec, 46 },
    { 0xa80d35ed, 90 },
    { 0, 0 },
    { 0xcecd35ef, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69d35f6, 33 },
    { 0xa94d35f6, 61 },
    { 0x95d35f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49d35fc, 93 },
    { 0xfeed35fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7d3600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x740d3603, 23 },
    { 0x674d3604, 23 },
    { 0x7f9d3604, 23 },
    { 0xcecd3604, 95 },
    { 0x646d3606, 55 },
    { 0x381d3608, 90 },
    { 0xd6ed3609, 91 },
    { 0xf13d3609, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecd360e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9d3611, 98 },
    { 0x5c1d3611, 33 },
    { 0, 0 },
    { 0xbf1d3614, 93 },
    { 0x2dad3614, 47 },
    { 0xb7dd3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d361b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fd3622, 61 },
    { 0x137d3623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3628, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1d362e, 34 },
    { 0x102d362f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad3632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196d3637, 95 },
    { 0x7b6d3638, 90 },
    { 0xd4cd3638, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d363c, 76 },
    { 0, 0 },
    { 0x229d363e, 95 },
    { 0xe9ad363f, 93 },
    { 0x38dd3640, 33 },
    { 0x266d3641, 61 },
    { 0x101d3640, 33 },
    { 0xf38d363e, 94 },
    { 0xaa4d3641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9d364a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dd364f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d3653, 23 },
    { 0x9b4d3653, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xecad3657, 98 },
    { 0, 0 },
    { 0x66d3659, 67 },
    { 0xb6fd365a, 55 },
    { 0xe79d365a, 52 },
    { 0x6e0d365c, 52 },
    { 0x851d365b, 66 },
    { 0xd19d365a, 84 },
    { 0, 0 },
    { 0xdc2d3660, 61 },
    { 0x571d3661, 23 },
    { 0x657d3662, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd366d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ad3670, 90 },
    { 0, 0 },
    { 0x9f4d3672, 90 },
    { 0, 0 },
    { 0x927d3674, 61 },
    { 0x8ded3675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d367a, 93 },
    { 0xdc0d367a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d3682, 55 },
    { 0, 0 },
    { 0x727d3684, 33 },
    { 0, 0 },
    { 0x387d3686, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95d3689, 34 },
    { 0xd1cd368a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d368d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd3690, 34 },
    { 0x509d3690, 84 },
    { 0, 0 },
    { 0xdb0d3693, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aed369c, 46 },
    { 0x10d369d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acd36a1, 91 },
    { 0x905d36a1, 90 },
    { 0xc4fd36a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d36a6, 33 },
    { 0, 0 },
    { 0xa1bd36a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d36ab, 23 },
    { 0, 0 },
    { 0x1b0d36ad, 90 },
    { 0x5fdd36ad, 61 },
    { 0xcf6d36af, 95 },
    { 0x17ed36b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87d36b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d36b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418d36bc, 90 },
    { 0x652d36bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493d36c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d36c9, 66 },
    { 0, 0 },
    { 0xa06d36cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bd36ce, 76 },
    { 0x24d36ce, 98 },
    { 0x180d36d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ad36d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d36d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ad36dc, 90 },
    { 0xbed36dd, 33 },
    { 0, 0 },
    { 0xaed36df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fd36e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803d36eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d36ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3d36f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd36f9, 90 },
    { 0, 0 },
    { 0x6f5d36fb, 23 },
    { 0x7cfd36fb, 84 },
    { 0xbf0d36fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd3705, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafad3709, 90 },
    { 0x7efd370a, 61 },
    { 0x508d3709, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40d370e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed3711, 95 },
    { 0, 0 },
    { 0xb25d3713, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x272d3716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dd371a, 53 },
    { 0x604d371a, 23 },
    { 0xdf3d371c, 43 },
    { 0, 0 },
    { 0xb40d371e, 77 },
    { 0x7c2d371f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d3725, 95 },
    { 0, 0 },
    { 0xdffd3727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165d372b, 98 },
    { 0xbaad372c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83d372f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d373c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904d3740, 95 },
    { 0, 0 },
    { 0xc38d3742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x988d3745, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d3748, 23 },
    { 0xaa2d3749, 83 },
    { 0x55ed374a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd374d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ad3752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd3756, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27d3759, 65 },
    { 0x65d375a, 61 },
    { 0x6bcd375a, 38 },
    { 0x77dd375c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0d3761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d3766, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d376a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d376d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d3771, 61 },
    { 0, 0 },
    { 0x787d3773, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd3776, 34 },
    { 0xba6d3777, 33 },
    { 0x7e6d3778, 75 },
    { 0, 0 },
    { 0xce8d377a, 23 },
    { 0, 0 },
    { 0x79d377c, 90 },
    { 0xc67d377d, 23 },
    { 0xced377e, 68 },
    { 0x272d377d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d3788, 98 },
    { 0x355d3789, 43 },
    { 0x230d378a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d378e, 23 },
    { 0, 0 },
    { 0xc78d3790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d3796, 55 },
    { 0x6cd3797, 90 },
    { 0xe37d3798, 76 },
    { 0, 0 },
    { 0x5b4d379a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2d37a7, 95 },
    { 0x571d37a7, 24 },
    { 0x4fd37a9, 78 },
    { 0xd97d37aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944d37af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78d37b2, 77 },
    { 0xe91d37b2, 67 },
    { 0xd04d37b4, 93 },
    { 0, 0 },
    { 0x98ed37b6, 23 },
    { 0x81ed37b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d37bc, 33 },
    { 0x3afd37bd, 98 },
    { 0x8ecd37be, 95 },
    { 0xae4d37bf, 90 },
    { 0x9e7d37bd, 61 },
    { 0xda0d37c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d37c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01d37cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ed37cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad37d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd37da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fd37df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d37ea, 88 },
    { 0x79bd37eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeded37f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bed37f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d37fb, 90 },
    { 0xc3ed37fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d380a, 61 },
    { 0x29bd380b, 80 },
    { 0xf5fd380c, 55 },
    { 0, 0 },
    { 0xa39d380e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3812, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded3815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd3819, 95 },
    { 0, 0 },
    { 0xd1cd381b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d3824, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01d3828, 33 },
    { 0, 0 },
    { 0x8f4d382a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d3835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d383b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d383f, 61 },
    { 0, 0 },
    { 0x23ad3841, 90 },
    { 0xb08d3842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7d3851, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d3856, 80 },
    { 0x217d3856, 55 },
    { 0, 0 },
    { 0x36dd3859, 90 },
    { 0, 0 },
    { 0xf37d385b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d3860, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d3867, 23 },
    { 0x901d3867, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d386d, 23 },
    { 0xd84d386e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd3872, 40 },
    { 0xf91d3872, 61 },
    { 0, 0 },
    { 0x6b0d3875, 90 },
    { 0x5aed3876, 34 },
    { 0x305d3876, 40 },
    { 0xb93d3878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6d387d, 78 },
    { 0x96dd387e, 95 },
    { 0x5b1d387e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3883, 90 },
    { 0x439d3884, 95 },
    { 0x3fbd3885, 33 },
    { 0x641d3886, 76 },
    { 0x6bdd3887, 40 },
    { 0x69fd3887, 90 },
    { 0, 0 },
    { 0x924d388a, 61 },
    { 0xe22d388b, 91 },
    { 0, 0 },
    { 0xc3dd388d, 23 },
    { 0xa85d388d, 95 },
    { 0, 0 },
    { 0x990d3890, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dd3894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dad3898, 100 },
    { 0x8fad3899, 56 },
    { 0, 0 },
    { 0xe54d389b, 98 },
    { 0, 0 },
    { 0x2dfd389d, 90 },
    { 0x3bed389e, 23 },
    { 0xacdd389e, 61 },
    { 0, 0 },
    { 0xf4dd38a1, 46 },
    { 0, 0 },
    { 0x3a9d38a3, 98 },
    { 0xb22d38a4, 98 },
    { 0xa2dd38a5, 46 },
    { 0, 0 },
    { 0x5d9d38a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179d38ae, 33 },
    { 0xd97d38af, 90 },
    { 0x4ddd38b0, 90 },
    { 0x52dd38b0, 95 },
    { 0, 0 },
    { 0x94d38b3, 61 },
    { 0x990d38b4, 90 },
    { 0xea9d38b4, 61 },
    { 0, 0 },
    { 0x695d38b7, 91 },
    { 0x595d38b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bd38bd, 61 },
    { 0, 0 },
    { 0x8cbd38bf, 93 },
    { 0xf70d38bf, 99 },
    { 0, 0 },
    { 0x5bed38c2, 90 },
    { 0xeb1d38c3, 98 },
    { 0, 0 },
    { 0x7ded38c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaed38cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d38d0, 98 },
    { 0x82cd38d1, 61 },
    { 0xb1ad38d1, 33 },
    { 0x354d38d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d38db, 23 },
    { 0, 0 },
    { 0xc52d38dd, 85 },
    { 0, 0 },
    { 0x203d38df, 33 },
    { 0, 0 },
    { 0xcb3d38e1, 61 },
    { 0, 0 },
    { 0x8ad38e3, 95 },
    { 0xed4d38e4, 23 },
    { 0xe81d38e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed38e8, 98 },
    { 0, 0 },
    { 0xa52d38ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffd38ed, 95 },
    { 0xa9cd38ed, 67 },
    { 0x839d38ef, 90 },
    { 0, 0 },
    { 0xfd2d38f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9d38f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d38fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d3900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72d3904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64d390a, 98 },
    { 0, 0 },
    { 0xf53d390c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90d3910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d3913, 76 },
    { 0xf78d3914, 98 },
    { 0xedbd3915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd3918, 33 },
    { 0x218d3918, 23 },
    { 0xb51d391a, 33 },
    { 0x49dd391a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d3926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4d3929, 66 },
    { 0x587d3929, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d392d, 76 },
    { 0x16ed392e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462d3937, 98 },
    { 0xb29d3938, 43 },
    { 0, 0 },
    { 0xcb5d393a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55d393f, 55 },
    { 0xced393f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad3944, 33 },
    { 0x763d3945, 90 },
    { 0xc8d3946, 76 },
    { 0xeb3d3947, 36 },
    { 0x15ed3945, 90 },
    { 0xf9ad3944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d394c, 98 },
    { 0x8aad394d, 95 },
    { 0, 0 },
    { 0xf40d394f, 83 },
    { 0, 0 },
    { 0x3f2d3951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ed3956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed3959, 23 },
    { 0, 0 },
    { 0x327d395b, 33 },
    { 0, 0 },
    { 0x5b9d395d, 87 },
    { 0xacfd395e, 95 },
    { 0x28fd395f, 33 },
    { 0x5bfd395f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297d3967, 75 },
    { 0xcb0d3967, 96 },
    { 0x65dd3969, 33 },
    { 0x334d3969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86d3975, 80 },
    { 0xbf7d3976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ed397a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd397e, 51 },
    { 0x9a6d397f, 76 },
    { 0xfe4d3980, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d3985, 75 },
    { 0xbdd3986, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1d398c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04d398f, 46 },
    { 0x9bfd3990, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ad3994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d399a, 95 },
    { 0, 0 },
    { 0xf94d399c, 76 },
    { 0, 0 },
    { 0xa7bd399e, 98 },
    { 0, 0 },
    { 0x3cad39a0, 61 },
    { 0, 0 },
    { 0x27d39a2, 23 },
    { 0, 0 },
    { 0x63fd39a4, 33 },
    { 0x4e9d39a5, 23 },
    { 0xc43d39a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed39ad, 84 },
    { 0xb06d39ae, 95 },
    { 0xcbfd39ae, 46 },
    { 0x9c6d39ae, 90 },
    { 0, 0 },
    { 0x1cbd39b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769d39bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455d39cb, 33 },
    { 0x796d39cc, 85 },
    { 0xefbd39cd, 33 },
    { 0x866d39cc, 55 },
    { 0, 0 },
    { 0xb9ed39d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6d39d3, 23 },
    { 0, 0 },
    { 0xc2d39d5, 95 },
    { 0, 0 },
    { 0xa3bd39d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27d39dc, 78 },
    { 0x987d39dd, 46 },
    { 0xd8ed39de, 100 },
    { 0x87cd39df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d39e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31d39e5, 88 },
    { 0, 0 },
    { 0x91ed39e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fd39ec, 98 },
    { 0, 0 },
    { 0x246d39ee, 61 },
    { 0, 0 },
    { 0xf5ad39f0, 61 },
    { 0xb54d39f1, 99 },
    { 0x9dd39f0, 66 },
    { 0xf5ed39f3, 23 },
    { 0xd3cd39f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d39fd, 70 },
    { 0xb7cd39fe, 98 },
    { 0, 0 },
    { 0x5bcd3a00, 93 },
    { 0xbabd3a01, 93 },
    { 0x246d3a00, 43 },
    { 0x731d3a03, 95 },
    { 0xeb1d3a04, 77 },
    { 0x25d3a05, 90 },
    { 0x44fd3a01, 90 },
    { 0xf58d3a07, 23 },
    { 0xf72d3a07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d3a0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d3a13, 90 },
    { 0, 0 },
    { 0x74bd3a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5d3a1c, 61 },
    { 0xd99d3a1d, 78 },
    { 0, 0 },
    { 0x400d3a1f, 17 },
    { 0, 0 },
    { 0x499d3a21, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fd3a25, 33 },
    { 0xd7d3a25, 90 },
    { 0xb7ad3a27, 53 },
    { 0, 0 },
    { 0x7d2d3a29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210d3a2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d3a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cd3a33, 95 },
    { 0x85ed3a34, 98 },
    { 0xafcd3a35, 23 },
    { 0xda7d3a33, 90 },
    { 0xe8cd3a37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aad3a3a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cd3a3e, 33 },
    { 0, 0 },
    { 0x50ad3a40, 23 },
    { 0x667d3a41, 100 },
    { 0x589d3a40, 78 },
    { 0xa03d3a43, 91 },
    { 0xfe4d3a43, 98 },
    { 0x2ecd3a45, 84 },
    { 0, 0 },
    { 0xe3fd3a47, 97 },
    { 0, 0 },
    { 0xadbd3a49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69d3a4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132d3a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12d3a58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435d3a61, 90 },
    { 0x707d3a61, 98 },
    { 0x416d3a63, 93 },
    { 0x685d3a64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d3a67, 95 },
    { 0x4a9d3a68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d3a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d3a75, 90 },
    { 0x94dd3a76, 76 },
    { 0x112d3a77, 24 },
    { 0, 0 },
    { 0x21bd3a79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd3a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d3a7f, 99 },
    { 0, 0 },
    { 0x1efd3a81, 91 },
    { 0, 0 },
    { 0x4dbd3a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d3a89, 100 },
    { 0, 0 },
    { 0xe5bd3a8b, 61 },
    { 0x7a8d3a8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fed3a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd3a97, 90 },
    { 0xeb4d3a97, 65 },
    { 0, 0 },
    { 0x244d3a9a, 61 },
    { 0, 0 },
    { 0x630d3a9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x300d3a9f, 75 },
    { 0x1e4d3aa0, 77 },
    { 0x8d1d3aa0, 34 },
    { 0x316d3aa1, 100 },
    { 0xacad3aa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cd3aa6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfd3aaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ad3aad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d3ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29d3ab8, 90 },
    { 0x6e6d3ab8, 98 },
    { 0x71fd3aba, 76 },
    { 0x199d3abb, 93 },
    { 0xd47d3abc, 91 },
    { 0x6d6d3ab9, 95 },
    { 0, 0 },
    { 0x593d3abf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdd3ac2, 90 },
    { 0xab8d3ac3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bad3ac8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd3acf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3d3ad5, 95 },
    { 0x66dd3ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd3ada, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d3add, 86 },
    { 0x98cd3ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadd3ae4, 100 },
    { 0x4afd3ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1d3ae8, 33 },
    { 0x907d3ae9, 66 },
    { 0x40ad3aea, 98 },
    { 0xebfd3aeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d3aee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd3af7, 80 },
    { 0x9f3d3af8, 87 },
    { 0xd3d3af7, 95 },
    { 0, 0 },
    { 0x7a6d3afb, 90 },
    { 0xee0d3afc, 87 },
    { 0x9ddd3afc, 34 },
    { 0, 0 },
    { 0xb4fd3aff, 90 },
    { 0x2ead3b00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d3b04, 61 },
    { 0, 0 },
    { 0x2a9d3b06, 17 },
    { 0x1add3b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d3b0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3b0f, 61 },
    { 0xfa3d3b10, 23 },
    { 0, 0 },
    { 0xd81d3b12, 61 },
    { 0x23bd3b13, 56 },
    { 0, 0 },
    { 0x65bd3b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d3b19, 95 },
    { 0xe4cd3b1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d3b1d, 90 },
    { 0x638d3b1e, 95 },
    { 0xcbcd3b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fad3b24, 98 },
    { 0xad3b25, 61 },
    { 0, 0 },
    { 0x513d3b27, 98 },
    { 0x61d3b28, 88 },
    { 0, 0 },
    { 0x8cad3b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dd3b2d, 98 },
    { 0xfa9d3b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cd3b32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855d3b36, 34 },
    { 0x191d3b37, 33 },
    { 0x75dd3b38, 33 },
    { 0x1f2d3b39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d3b3e, 67 },
    { 0, 0 },
    { 0x4e4d3b40, 90 },
    { 0xc99d3b41, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d3b44, 91 },
    { 0x83bd3b45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35d3b48, 53 },
    { 0x42dd3b49, 95 },
    { 0xe10d3b49, 91 },
    { 0xab1d3b4b, 78 },
    { 0x438d3b4c, 67 },
    { 0xbc6d3b4b, 67 },
    { 0x9a8d3b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270d3b53, 33 },
    { 0xf16d3b54, 95 },
    { 0x662d3b54, 95 },
    { 0xe4cd3b56, 98 },
    { 0xb90d3b57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d3b5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfd3b64, 95 },
    { 0x8f3d3b65, 91 },
    { 0x7c2d3b66, 23 },
    { 0x45bd3b67, 33 },
    { 0xa46d3b68, 95 },
    { 0x56ad3b68, 55 },
    { 0x670d3b6a, 61 },
    { 0xd48d3b6a, 55 },
    { 0x289d3b6c, 33 },
    { 0x3ebd3b6a, 76 },
    { 0xd3cd3b69, 90 },
    { 0x808d3b6b, 95 },
    { 0, 0 },
    { 0x8b7d3b71, 90 },
    { 0xc3ed3b72, 61 },
    { 0x3fad3b73, 61 },
    { 0xb36d3b74, 61 },
    { 0x67fd3b72, 98 },
    { 0, 0 },
    { 0x5add3b77, 95 },
    { 0x3e5d3b78, 90 },
    { 0x443d3b79, 76 },
    { 0xc3d3b77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbd3b7f, 88 },
    { 0xcded3b80, 33 },
    { 0x59dd3b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d3b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277d3b90, 96 },
    { 0xd8bd3b91, 61 },
    { 0, 0 },
    { 0xa8fd3b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d3b9a, 98 },
    { 0x843d3b9b, 90 },
    { 0x95cd3b9a, 90 },
    { 0, 0 },
    { 0xdedd3b9e, 91 },
    { 0, 0 },
    { 0xe28d3ba0, 33 },
    { 0xc7ed3ba0, 33 },
    { 0x2d3ba2, 43 },
    { 0, 0 },
    { 0x37dd3ba4, 34 },
    { 0x6f4d3ba5, 61 },
    { 0xf17d3ba6, 84 },
    { 0, 0 },
    { 0x7c1d3ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd3bab, 46 },
    { 0x348d3bac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9add3baf, 55 },
    { 0, 0 },
    { 0x45cd3bb1, 98 },
    { 0xcdfd3bb2, 23 },
    { 0x73cd3bb3, 76 },
    { 0x6c9d3bb3, 61 },
    { 0xb27d3bb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd3bb8, 33 },
    { 0x71cd3bb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d3bbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x527d3bc0, 33 },
    { 0x15bd3bc0, 23 },
    { 0x54bd3bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d3bc8, 78 },
    { 0x7a5d3bc8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd3bcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d3bd4, 46 },
    { 0, 0 },
    { 0x36bd3bd6, 61 },
    { 0x63d3bd7, 95 },
    { 0x23dd3bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd3be2, 33 },
    { 0x674d3be3, 23 },
    { 0, 0 },
    { 0x371d3be5, 23 },
    { 0xb69d3be6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffd3bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd3bf1, 85 },
    { 0x1cad3bf1, 34 },
    { 0x524d3bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed3bfa, 55 },
    { 0x93ed3bfb, 67 },
    { 0, 0 },
    { 0x780d3bfd, 33 },
    { 0, 0 },
    { 0x71ed3bff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d3c02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bd3c05, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3c0b, 61 },
    { 0, 0 },
    { 0x4ead3c0d, 98 },
    { 0x966d3c0e, 61 },
    { 0, 0 },
    { 0x835d3c10, 87 },
    { 0xc96d3c11, 95 },
    { 0x699d3c12, 55 },
    { 0xfcfd3c13, 43 },
    { 0x61d3c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd3c1a, 77 },
    { 0x6a3d3c1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d3c25, 97 },
    { 0x24ed3c25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02d3c29, 76 },
    { 0x8c0d3c2a, 90 },
    { 0xbb4d3c2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d3c2f, 23 },
    { 0xbad3c2f, 61 },
    { 0x340d3c31, 43 },
    { 0xcd6d3c31, 98 },
    { 0, 0 },
    { 0x2edd3c34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d3c38, 95 },
    { 0xeb3d3c38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd3c3e, 74 },
    { 0xebbd3c3f, 76 },
    { 0xc63d3c3f, 90 },
    { 0, 0 },
    { 0x3ecd3c42, 99 },
    { 0, 0 },
    { 0xdc8d3c44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2d3c4a, 78 },
    { 0x600d3c4b, 34 },
    { 0xb27d3c4c, 61 },
    { 0, 0 },
    { 0xb4bd3c4e, 17 },
    { 0xacad3c4f, 78 },
    { 0x776d3c50, 69 },
    { 0x370d3c4f, 46 },
    { 0x203d3c52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x829d3c55, 93 },
    { 0xce2d3c56, 90 },
    { 0, 0 },
    { 0x4ccd3c58, 77 },
    { 0x493d3c59, 90 },
    { 0x681d3c59, 90 },
    { 0, 0 },
    { 0x2a8d3c5c, 90 },
    { 0x2c4d3c5d, 55 },
    { 0xdc1d3c5e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d3c6a, 61 },
    { 0x714d3c6b, 55 },
    { 0xb99d3c6a, 23 },
    { 0xdbad3c6c, 23 },
    { 0, 0 },
    { 0x349d3c6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71d3c78, 33 },
    { 0x4f4d3c79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cd3c81, 22 },
    { 0xa56d3c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6d3c85, 95 },
    { 0xaded3c85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89d3c8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475d3c8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecd3c91, 95 },
    { 0x1acd3c92, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd3c96, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebd3c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dad3ca3, 33 },
    { 0xcbcd3ca3, 61 },
    { 0x235d3ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ed3cae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fad3cb1, 33 },
    { 0x370d3cb1, 43 },
    { 0x80fd3cb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5d3cb7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddd3cbc, 76 },
    { 0x2c1d3cbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d3cc2, 75 },
    { 0x959d3cc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d3cca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x440d3ccd, 95 },
    { 0xe0cd3ccd, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ed3cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d3cd8, 90 },
    { 0x72dd3cd9, 55 },
    { 0x9a0d3cda, 90 },
    { 0x6c0d3cdb, 23 },
    { 0xbfed3cd9, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05d3ce0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed3ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed3ce8, 98 },
    { 0x449d3ce8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad3cee, 38 },
    { 0xbcfd3cef, 23 },
    { 0x48fd3cf0, 34 },
    { 0x8aad3cf1, 93 },
    { 0xf5ad3cf2, 69 },
    { 0xd41d3cf3, 95 },
    { 0x424d3cf4, 84 },
    { 0xf89d3cf3, 23 },
    { 0x272d3cf6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad3cf9, 55 },
    { 0xa60d3cfa, 55 },
    { 0x249d3cfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6d3cff, 98 },
    { 0, 0 },
    { 0xfa4d3d01, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d3d0a, 61 },
    { 0, 0 },
    { 0x937d3d0c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fd3d10, 95 },
    { 0xca2d3d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57d3d15, 52 },
    { 0x29dd3d15, 75 },
    { 0xe3fd3d16, 33 },
    { 0x8aad3d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83d3d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x980d3d1e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bed3d23, 76 },
    { 0xb5cd3d24, 83 },
    { 0x33fd3d25, 81 },
    { 0x4b2d3d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21d3d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bd3d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd3d35, 78 },
    { 0xf60d3d36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d3d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd3d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931d3d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbd3d4f, 90 },
    { 0x922d3d50, 95 },
    { 0x750d3d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bd3d54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6d3d60, 69 },
    { 0x1a9d3d61, 61 },
    { 0xee0d3d62, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad3d66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ed3d70, 95 },
    { 0, 0 },
    { 0x5c7d3d72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935d3d76, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82d3d83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8d3d89, 43 },
    { 0x53dd3d8a, 95 },
    { 0x3ccd3d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d3d8f, 77 },
    { 0xb24d3d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d3d93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d3d96, 43 },
    { 0, 0 },
    { 0x853d3d98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfd3d9b, 84 },
    { 0, 0 },
    { 0xe5bd3d9d, 23 },
    { 0, 0 },
    { 0x153d3d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dd3daa, 87 },
    { 0x529d3dab, 76 },
    { 0, 0 },
    { 0x7d3d3dad, 33 },
    { 0x6e8d3dae, 65 },
    { 0xaadd3daf, 23 },
    { 0x7cfd3dae, 91 },
    { 0x549d3db1, 78 },
    { 0xcad3db2, 47 },
    { 0xc99d3db3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d3db8, 90 },
    { 0x99d3db9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01d3dbd, 23 },
    { 0xed1d3dbd, 88 },
    { 0, 0 },
    { 0xb3ad3dc0, 34 },
    { 0x612d3dc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad3dc4, 33 },
    { 0xf1d3dc5, 23 },
    { 0xc7bd3dc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x722d3dc9, 61 },
    { 0, 0 },
    { 0xd52d3dcb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341d3dcf, 66 },
    { 0xe93d3dd0, 95 },
    { 0x943d3dd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d3dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55d3ddb, 66 },
    { 0x317d3ddb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d3de0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8d3de9, 83 },
    { 0x70dd3dea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ad3ded, 90 },
    { 0xf3d3dee, 33 },
    { 0x1d7d3def, 98 },
    { 0x2cfd3ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd3df3, 95 },
    { 0xa47d3df4, 67 },
    { 0xb41d3df5, 55 },
    { 0x691d3df4, 95 },
    { 0xf79d3df7, 43 },
    { 0x3d0d3df7, 23 },
    { 0xfb3d3df8, 76 },
    { 0x698d3df5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x498d3dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d3e04, 55 },
    { 0xb25d3e05, 61 },
    { 0xf1fd3e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2d3e0d, 55 },
    { 0x9abd3e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189d3e12, 66 },
    { 0x1a6d3e12, 95 },
    { 0, 0 },
    { 0xdedd3e15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2d3e18, 69 },
    { 0x8b1d3e19, 81 },
    { 0xacd3e18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd3e29, 33 },
    { 0, 0 },
    { 0xb8fd3e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad3e2e, 81 },
    { 0x759d3e2e, 68 },
    { 0x173d3e2e, 90 },
    { 0, 0 },
    { 0x3ad3e32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ad3e35, 23 },
    { 0x69cd3e36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd3e3c, 52 },
    { 0x39fd3e3d, 55 },
    { 0x9c5d3e3e, 43 },
    { 0xd60d3e3f, 98 },
    { 0, 0 },
    { 0x8c6d3e41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x895d3e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d3e48, 95 },
    { 0, 0 },
    { 0x5add3e4a, 42 },
    { 0x104d3e4a, 90 },
    { 0xa68d3e4c, 78 },
    { 0, 0 },
    { 0x83dd3e4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5d3e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d3e5a, 97 },
    { 0x75dd3e5b, 69 },
    { 0xd84d3e5c, 95 },
    { 0xb86d3e5d, 23 },
    { 0x5c4d3e5e, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcad3e61, 90 },
    { 0, 0 },
    { 0x4e9d3e63, 95 },
    { 0x6a5d3e64, 90 },
    { 0x8fed3e65, 91 },
    { 0x52ad3e64, 98 },
    { 0, 0 },
    { 0x982d3e68, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x815d3e6b, 85 },
    { 0, 0 },
    { 0xb29d3e6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8d3e70, 65 },
    { 0x736d3e71, 98 },
    { 0x5b7d3e70, 100 },
    { 0xf53d3e73, 61 },
    { 0x9ced3e74, 33 },
    { 0x4b1d3e74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d3e78, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffd3e7b, 61 },
    { 0xcb3d3e7c, 98 },
    { 0, 0 },
    { 0x528d3e7e, 23 },
    { 0x724d3e7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d3e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x699d3e8b, 34 },
    { 0x5cbd3e8c, 43 },
    { 0x392d3e8d, 66 },
    { 0x10cd3e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d3e92, 33 },
    { 0, 0 },
    { 0xd73d3e94, 87 },
    { 0x9ead3e95, 77 },
    { 0x98cd3e94, 90 },
    { 0, 0 },
    { 0x595d3e98, 43 },
    { 0xd94d3e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9d3e9d, 78 },
    { 0xd7bd3e9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d3ea1, 17 },
    { 0, 0 },
    { 0xeaed3ea3, 61 },
    { 0x975d3ea4, 40 },
    { 0x1c8d3ea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cd3eaa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d3eb7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d3ebc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d3ebf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd3ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3ec7, 75 },
    { 0, 0 },
    { 0xa04d3ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d3ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64d3ed2, 33 },
    { 0x3fad3ed3, 64 },
    { 0x70ad3ed3, 23 },
    { 0x5cd3ed5, 23 },
    { 0x917d3ed2, 23 },
    { 0x984d3ed7, 87 },
    { 0x236d3ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d3edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd3ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed3ee1, 76 },
    { 0xb58d3ee1, 55 },
    { 0x735d3ee3, 55 },
    { 0, 0 },
    { 0xc9d3ee5, 40 },
    { 0, 0 },
    { 0x411d3ee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d3eed, 95 },
    { 0xfced3eee, 88 },
    { 0x2acd3eef, 90 },
    { 0x9e3d3ef0, 100 },
    { 0, 0 },
    { 0x12ad3ef2, 90 },
    { 0x692d3ef2, 87 },
    { 0x585d3ef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x231d3ef7, 21 },
    { 0xaad3ef7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5d3eff, 23 },
    { 0x19fd3f00, 90 },
    { 0x5f4d3f01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7d3f07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd3f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827d3f10, 90 },
    { 0x7d5d3f11, 33 },
    { 0x7d6d3f11, 61 },
    { 0xf39d3f11, 76 },
    { 0xc39d3f14, 99 },
    { 0xdfed3f12, 33 },
    { 0, 0 },
    { 0xf77d3f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd3f1a, 34 },
    { 0x582d3f1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d3f20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6d3f24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d3f2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afd3f32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629d3f37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fd3f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441d3f42, 90 },
    { 0, 0 },
    { 0x989d3f44, 90 },
    { 0, 0 },
    { 0x574d3f46, 46 },
    { 0, 0 },
    { 0x7e5d3f48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d3f4c, 68 },
    { 0xbbdd3f4c, 55 },
    { 0, 0 },
    { 0xd0fd3f4f, 23 },
    { 0x511d3f50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216d3f58, 98 },
    { 0x606d3f58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed3f5d, 23 },
    { 0, 0 },
    { 0x107d3f5f, 85 },
    { 0, 0 },
    { 0xe4bd3f61, 88 },
    { 0x709d3f62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d3f66, 98 },
    { 0x133d3f67, 55 },
    { 0x38fd3f68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d3f6d, 87 },
    { 0x7ead3f6e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530d3f72, 33 },
    { 0, 0 },
    { 0xdfcd3f74, 65 },
    { 0x64fd3f74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483d3f81, 90 },
    { 0x8a7d3f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4d3f88, 37 },
    { 0x9b9d3f89, 76 },
    { 0x94dd3f8a, 93 },
    { 0x8b2d3f8b, 90 },
    { 0xb1bd3f8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433d3f98, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad3faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53d3fb0, 77 },
    { 0xbedd3fb1, 98 },
    { 0x487d3fb2, 90 },
    { 0x139d3fb2, 61 },
    { 0xd88d3fb4, 98 },
    { 0xef5d3fb3, 98 },
    { 0, 0 },
    { 0x60ed3fb7, 61 },
    { 0x49ed3fb8, 95 },
    { 0x99fd3fb7, 33 },
    { 0xfd8d3fba, 69 },
    { 0xae6d3fba, 23 },
    { 0x6f5d3fbc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7d3fc1, 55 },
    { 0, 0 },
    { 0x9f4d3fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad3fcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ed3fd2, 46 },
    { 0x11bd3fd3, 33 },
    { 0x2f4d3fd4, 93 },
    { 0x433d3fd4, 98 },
    { 0x970d3fd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8d3fde, 40 },
    { 0, 0 },
    { 0x1add3fe0, 53 },
    { 0xa4ed3fe1, 33 },
    { 0, 0 },
    { 0x66d3fe3, 46 },
    { 0xd15d3fe4, 95 },
    { 0x6a3d3fe5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160d3feb, 91 },
    { 0xeebd3feb, 95 },
    { 0x6b0d3fec, 98 },
    { 0, 0 },
    { 0xb6ed3fef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ad3ff4, 61 },
    { 0x359d3ff5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536d3ff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5dd3ffe, 43 },
    { 0x75bd3ffe, 61 },
    { 0x58fd3fff, 76 },
    { 0x1b6d4001, 23 },
    { 0, 0 },
    { 0x37fd4003, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd4007, 33 },
    { 0xdd4008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280d400f, 33 },
    { 0x3f6d400f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd4015, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7d401b, 91 },
    { 0x630d401c, 81 },
    { 0x5e2d401d, 98 },
    { 0, 0 },
    { 0x5d3d401f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ed4022, 23 },
    { 0xdd7d4022, 61 },
    { 0x8a0d4023, 17 },
    { 0, 0 },
    { 0xf1fd4026, 55 },
    { 0xb6fd4027, 95 },
    { 0x8f3d4027, 33 },
    { 0xfb4d4029, 90 },
    { 0x19dd4029, 100 },
    { 0x4fd402b, 98 },
    { 0x3cdd402c, 33 },
    { 0x286d402d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ed4030, 51 },
    { 0x70fd4031, 33 },
    { 0xaead4030, 23 },
    { 0xeb2d4033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341d4037, 61 },
    { 0, 0 },
    { 0xae4d4039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d4046, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacd4049, 90 },
    { 0x8f5d404a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d404e, 43 },
    { 0x2fd404e, 91 },
    { 0x8ced404e, 90 },
    { 0x9c3d4050, 43 },
    { 0x5b1d4052, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2d4055, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85d4059, 91 },
    { 0xdcd405a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ed405d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d4060, 61 },
    { 0, 0 },
    { 0xa1fd4062, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ad406b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867d406f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d4073, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d4076, 83 },
    { 0xb0dd4077, 33 },
    { 0x4f4d4077, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd407e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641d4086, 87 },
    { 0, 0 },
    { 0xe85d4088, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d408b, 33 },
    { 0xfc3d408c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482d4091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded4096, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd409c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabd40a2, 52 },
    { 0x552d40a3, 95 },
    { 0x5e7d40a4, 98 },
    { 0x17ed40a5, 23 },
    { 0x6fd40a6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d40aa, 95 },
    { 0x74cd40ab, 23 },
    { 0, 0 },
    { 0xff9d40ad, 61 },
    { 0xfdfd40ae, 33 },
    { 0xf0ed40ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fd40b2, 33 },
    { 0, 0 },
    { 0x15fd40b4, 100 },
    { 0, 0 },
    { 0xd55d40b6, 56 },
    { 0, 0 },
    { 0x896d40b8, 33 },
    { 0xd2cd40b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d40c3, 61 },
    { 0x6d40c3, 52 },
    { 0x29d40c5, 43 },
    { 0x211d40c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887d40ce, 95 },
    { 0x96d40ce, 67 },
    { 0x8d3d40ce, 76 },
    { 0, 0 },
    { 0x6aed40d2, 33 },
    { 0, 0 },
    { 0x597d40d4, 77 },
    { 0x26ad40d4, 33 },
    { 0xb44d40d6, 90 },
    { 0, 0 },
    { 0x6f4d40d8, 90 },
    { 0x85ad40d8, 98 },
    { 0x972d40da, 33 },
    { 0xbaed40d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad40df, 17 },
    { 0xb8ed40df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491d40e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4d40e9, 33 },
    { 0x95d40ea, 61 },
    { 0x290d40ea, 95 },
    { 0xae4d40ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d40f4, 43 },
    { 0x87fd40f5, 90 },
    { 0xdeed40f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad40fa, 76 },
    { 0x14d40fb, 43 },
    { 0x271d40fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d40ff, 90 },
    { 0x20ad40ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad4105, 87 },
    { 0xe64d4106, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8d4109, 90 },
    { 0, 0 },
    { 0x5b8d410b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916d4110, 33 },
    { 0xd0bd4111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ded4116, 34 },
    { 0, 0 },
    { 0xf6fd4118, 33 },
    { 0xca4d4119, 90 },
    { 0x524d411a, 61 },
    { 0xe99d411b, 61 },
    { 0xc52d411b, 90 },
    { 0x4c9d411d, 33 },
    { 0x637d411e, 33 },
    { 0, 0 },
    { 0xaead4120, 98 },
    { 0xd8cd4120, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d4125, 76 },
    { 0xb37d4126, 61 },
    { 0xc4bd4127, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fd4133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fd4136, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72d4139, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10d413d, 61 },
    { 0xacdd413d, 76 },
    { 0x6fbd413d, 61 },
    { 0, 0 },
    { 0x903d4141, 95 },
    { 0xfe8d4142, 93 },
    { 0x93bd4143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574d414d, 33 },
    { 0xc6ed414d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5d4154, 46 },
    { 0x77ad4155, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d4158, 91 },
    { 0x7e1d4159, 76 },
    { 0x379d4158, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad4161, 90 },
    { 0x405d4161, 33 },
    { 0xbebd4161, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdd4166, 85 },
    { 0x9bcd4167, 91 },
    { 0x913d4166, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d416b, 23 },
    { 0xd6ad416c, 76 },
    { 0, 0 },
    { 0xd18d416e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619d4172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41d4175, 52 },
    { 0x2f7d4176, 98 },
    { 0x99cd4177, 23 },
    { 0x80d4178, 43 },
    { 0x3b7d4179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d417c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91d417f, 99 },
    { 0x267d4180, 98 },
    { 0x6d0d4181, 81 },
    { 0x127d4181, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dd4186, 90 },
    { 0x1d8d4187, 23 },
    { 0x44d4188, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d4190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8d4194, 46 },
    { 0x3bfd4195, 33 },
    { 0x4ead4196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d419c, 76 },
    { 0x41ed419d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acd41a0, 80 },
    { 0, 0 },
    { 0xf8fd41a2, 61 },
    { 0xce6d41a3, 87 },
    { 0x191d41a4, 61 },
    { 0xe28d41a5, 46 },
    { 0x946d41a6, 95 },
    { 0x79d41a7, 23 },
    { 0x3ebd41a7, 61 },
    { 0x450d41a3, 55 },
    { 0xd82d41aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed41ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x287d41b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299d41b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d41bb, 99 },
    { 0xad8d41bc, 23 },
    { 0x47fd41bc, 33 },
    { 0x76dd41be, 61 },
    { 0x764d41be, 77 },
    { 0x3e6d41c0, 43 },
    { 0xb7fd41c1, 85 },
    { 0x165d41c2, 43 },
    { 0xafed41c3, 34 },
    { 0x359d41c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d41cf, 61 },
    { 0xe7ad41d0, 90 },
    { 0x6ccd41d1, 61 },
    { 0x764d41d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd41d5, 100 },
    { 0x17ad41d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfd41db, 99 },
    { 0, 0 },
    { 0xa42d41dd, 23 },
    { 0x204d41de, 55 },
    { 0, 0 },
    { 0x99dd41e0, 23 },
    { 0x5c6d41e1, 93 },
    { 0xc8ad41e2, 23 },
    { 0xb4bd41e3, 23 },
    { 0x265d41e4, 95 },
    { 0x8e6d41e5, 23 },
    { 0xb2bd41e6, 68 },
    { 0x349d41e4, 76 },
    { 0x8c3d41e8, 81 },
    { 0xc50d41e8, 85 },
    { 0xb7ed41e8, 98 },
    { 0x854d41e7, 90 },
    { 0, 0 },
    { 0x37ed41ed, 68 },
    { 0x837d41ee, 61 },
    { 0xfbbd41ee, 66 },
    { 0xf2ad41ef, 55 },
    { 0xbced41f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d41f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add41f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad41ff, 66 },
    { 0x904d4200, 76 },
    { 0, 0 },
    { 0x2b0d4202, 87 },
    { 0x863d4203, 87 },
    { 0x10d4204, 88 },
    { 0xc70d4205, 87 },
    { 0xd91d4206, 33 },
    { 0x4e6d4207, 43 },
    { 0x8a1d4208, 46 },
    { 0x2f6d4209, 93 },
    { 0x13bd4207, 98 },
    { 0, 0 },
    { 0x422d420c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd4210, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d4219, 95 },
    { 0, 0 },
    { 0x2cdd421b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bd4220, 87 },
    { 0, 0 },
    { 0x7ddd4222, 34 },
    { 0x1c4d4223, 90 },
    { 0x5a6d4222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aed4228, 23 },
    { 0, 0 },
    { 0x44cd422a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbd422d, 90 },
    { 0x80ad422e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcd4231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d4234, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x478d4237, 43 },
    { 0xab4d4237, 95 },
    { 0xb8ad4238, 76 },
    { 0xbf4d4238, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d423e, 90 },
    { 0x694d423e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bd4242, 93 },
    { 0, 0 },
    { 0xc40d4244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed424a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fd424d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05d425a, 98 },
    { 0, 0 },
    { 0x319d425c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9d4263, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85d4269, 98 },
    { 0x7d3d4269, 87 },
    { 0x995d426b, 61 },
    { 0, 0 },
    { 0x82bd426d, 87 },
    { 0, 0 },
    { 0x4c2d426f, 23 },
    { 0, 0 },
    { 0xdccd4271, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fd4274, 91 },
    { 0x6c2d4274, 95 },
    { 0xab1d4276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5d427a, 90 },
    { 0, 0 },
    { 0x5bcd427c, 87 },
    { 0, 0 },
    { 0xd90d427e, 77 },
    { 0xae3d427f, 33 },
    { 0x527d4280, 43 },
    { 0x75bd4281, 95 },
    { 0x952d4280, 95 },
    { 0, 0 },
    { 0xba8d4284, 75 },
    { 0, 0 },
    { 0x87ed4286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90d428a, 61 },
    { 0x890d428b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630d428f, 95 },
    { 0, 0 },
    { 0xfd0d4291, 23 },
    { 0, 0 },
    { 0xddbd4293, 100 },
    { 0xedad4294, 99 },
    { 0xe04d4295, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4299, 34 },
    { 0, 0 },
    { 0x6b2d429b, 66 },
    { 0x9cad429c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14d429f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d42a8, 81 },
    { 0x694d42a9, 85 },
    { 0, 0 },
    { 0x325d42ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d42ae, 90 },
    { 0xb42d42af, 98 },
    { 0x359d42af, 76 },
    { 0, 0 },
    { 0xf9ed42b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d42b6, 68 },
    { 0xbffd42b7, 61 },
    { 0xca3d42b8, 23 },
    { 0xfead42b8, 33 },
    { 0, 0 },
    { 0xc25d42bb, 95 },
    { 0x93ad42bc, 23 },
    { 0x6e6d42bc, 34 },
    { 0x360d42be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbed42c3, 83 },
    { 0, 0 },
    { 0xa45d42c5, 34 },
    { 0xbdd42c5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965d42ce, 81 },
    { 0x2d9d42ce, 98 },
    { 0xba8d42d0, 66 },
    { 0x9fed42cf, 90 },
    { 0xe21d42d1, 80 },
    { 0x5ad42d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacd42d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d42db, 33 },
    { 0, 0 },
    { 0xe18d42dd, 46 },
    { 0x95d42de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd42e3, 55 },
    { 0, 0 },
    { 0x626d42e5, 61 },
    { 0, 0 },
    { 0x214d42e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dd42ec, 61 },
    { 0, 0 },
    { 0x8d8d42ee, 34 },
    { 0x7b8d42ef, 61 },
    { 0x222d42f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bd42f3, 46 },
    { 0x333d42f4, 90 },
    { 0xf22d42f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ed42f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41d42fc, 33 },
    { 0x212d42fd, 23 },
    { 0, 0 },
    { 0x2c0d42ff, 61 },
    { 0x3c4d4300, 23 },
    { 0xe16d4300, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bd4305, 24 },
    { 0, 0 },
    { 0x8d2d4307, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd430a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9d431c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d4321, 90 },
    { 0xec1d4322, 76 },
    { 0x315d4323, 55 },
    { 0x993d4323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3d4327, 33 },
    { 0, 0 },
    { 0xc0dd4329, 46 },
    { 0xa8dd432a, 76 },
    { 0, 0 },
    { 0x37ad432c, 66 },
    { 0xac2d432c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x529d4330, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2add4334, 43 },
    { 0, 0 },
    { 0x12ad4336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cd433b, 98 },
    { 0, 0 },
    { 0x31d433d, 66 },
    { 0xd3ad433e, 90 },
    { 0x28bd433e, 98 },
    { 0x243d433d, 100 },
    { 0xc5cd433f, 33 },
    { 0x506d433f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddd4345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d4349, 78 },
    { 0xe59d434a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174d434f, 85 },
    { 0x509d434f, 61 },
    { 0x163d4351, 43 },
    { 0x43ed4350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd4356, 98 },
    { 0xc96d4356, 84 },
    { 0, 0 },
    { 0x56ad4359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d435c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142d4362, 95 },
    { 0xbccd4362, 33 },
    { 0x55dd4364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd4367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d436a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed436d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d4374, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d437e, 84 },
    { 0, 0 },
    { 0xfdd4380, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78d4383, 90 },
    { 0x242d4383, 76 },
    { 0, 0 },
    { 0x92ed4386, 88 },
    { 0xd9d4386, 38 },
    { 0xf99d4388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d438c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad4394, 98 },
    { 0x74dd4395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6d4399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add439d, 90 },
    { 0x515d439e, 23 },
    { 0x518d439d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226d43af, 95 },
    { 0xdb1d43b0, 33 },
    { 0x876d43b0, 40 },
    { 0xe41d43b2, 85 },
    { 0, 0 },
    { 0x4b2d43b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1d43b7, 76 },
    { 0, 0 },
    { 0x713d43b9, 43 },
    { 0xc98d43ba, 66 },
    { 0x3bfd43b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d43c5, 98 },
    { 0, 0 },
    { 0xc46d43c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad43ca, 81 },
    { 0x2f5d43cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbad43ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x582d43d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d43d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcd43e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cd43e5, 85 },
    { 0x50bd43e6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4d43ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d43ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bd43f4, 46 },
    { 0x8f1d43f5, 46 },
    { 0x764d43f5, 87 },
    { 0xe47d43f6, 61 },
    { 0x284d43f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdd43fb, 90 },
    { 0xd2ad43fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccd4400, 95 },
    { 0xe12d4400, 33 },
    { 0xf21d4400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8d4405, 78 },
    { 0x39ad4406, 23 },
    { 0x155d4407, 98 },
    { 0xa92d4408, 43 },
    { 0x3cfd4405, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfed440d, 100 },
    { 0x4f3d440d, 98 },
    { 0x5e4d440d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958d4412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523d441a, 95 },
    { 0x6a5d441b, 76 },
    { 0xd72d441b, 90 },
    { 0, 0 },
    { 0x5f8d441e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x119d4421, 19 },
    { 0, 0 },
    { 0x254d4423, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8d4426, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40d442b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ed4432, 66 },
    { 0xfa1d4432, 23 },
    { 0x9b8d4432, 93 },
    { 0x55cd4435, 66 },
    { 0xa06d4436, 61 },
    { 0xf17d4437, 23 },
    { 0x34cd4435, 87 },
    { 0xe37d4439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8d4440, 46 },
    { 0x513d4440, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974d4447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cd444b, 98 },
    { 0x1bed444c, 66 },
    { 0x200d444c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d4453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8d4456, 98 },
    { 0x441d4457, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd445c, 65 },
    { 0xf00d445d, 23 },
    { 0x310d445e, 33 },
    { 0x4f0d445f, 52 },
    { 0x31ed445c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd4463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd4466, 95 },
    { 0, 0 },
    { 0x198d4468, 91 },
    { 0, 0 },
    { 0xf18d446a, 33 },
    { 0xd34d446b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad446e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d4475, 93 },
    { 0xcfad4475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcad4479, 55 },
    { 0xd67d447a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2d447d, 90 },
    { 0x299d447e, 23 },
    { 0x8b2d447e, 95 },
    { 0x833d4480, 93 },
    { 0x1ed4481, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d4485, 55 },
    { 0, 0 },
    { 0x560d4487, 17 },
    { 0, 0 },
    { 0x14d4489, 33 },
    { 0xd1fd448a, 88 },
    { 0x476d448b, 87 },
    { 0x57bd448b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dd448f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd4497, 98 },
    { 0x46ed4498, 23 },
    { 0xfd8d4498, 61 },
    { 0x62dd449a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x508d449d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204d44a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xebed44a6, 66 },
    { 0x90ad44a6, 100 },
    { 0x9bed44a8, 66 },
    { 0xd04d44a6, 95 },
    { 0, 0 },
    { 0xca1d44ab, 90 },
    { 0x3bcd44ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3d44b0, 23 },
    { 0xcf4d44b1, 95 },
    { 0x78d44b1, 77 },
    { 0x4aad44b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ad44b9, 76 },
    { 0x579d44ba, 88 },
    { 0x24cd44b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aad44c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fd44d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x867d44d4, 33 },
    { 0x3d6d44d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8d44d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d44dc, 98 },
    { 0x3fdd44dc, 33 },
    { 0x89cd44dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7d44e3, 55 },
    { 0x514d44e4, 33 },
    { 0x258d44e3, 61 },
    { 0x701d44e6, 90 },
    { 0xbbdd44e7, 46 },
    { 0, 0 },
    { 0x26ad44e9, 95 },
    { 0, 0 },
    { 0xccdd44eb, 55 },
    { 0x391d44ec, 95 },
    { 0xa5d44ec, 61 },
    { 0x7b2d44ed, 84 },
    { 0x92d44ee, 33 },
    { 0x1fd44ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08d44f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d44f9, 34 },
    { 0xee0d44f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d44fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad4501, 100 },
    { 0, 0 },
    { 0x47d4503, 85 },
    { 0xfe2d4504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d4507, 61 },
    { 0x7ddd4508, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd450c, 46 },
    { 0x168d450d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00d4515, 23 },
    { 0xbefd4516, 55 },
    { 0, 0 },
    { 0x5e5d4518, 61 },
    { 0x30dd4518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d451c, 90 },
    { 0x801d451c, 98 },
    { 0xd56d451e, 98 },
    { 0x34ad451f, 90 },
    { 0xbb3d4520, 90 },
    { 0x146d4521, 91 },
    { 0xb78d4521, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x881d4525, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2d452b, 94 },
    { 0x2aed452c, 98 },
    { 0xb7dd452d, 90 },
    { 0x494d452b, 23 },
    { 0x2f9d452f, 98 },
    { 0x668d4530, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899d4537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebd453a, 77 },
    { 0, 0 },
    { 0x2b2d453c, 33 },
    { 0xe8ad453d, 33 },
    { 0x650d453d, 91 },
    { 0xf06d453f, 93 },
    { 0, 0 },
    { 0x945d4541, 95 },
    { 0, 0 },
    { 0x8d9d4543, 90 },
    { 0x3fed4543, 23 },
    { 0xb87d4545, 100 },
    { 0x795d4544, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d454b, 98 },
    { 0, 0 },
    { 0x70bd454d, 36 },
    { 0x45fd454e, 23 },
    { 0x430d454d, 61 },
    { 0xc9cd454f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbed4553, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68d455a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ad455f, 95 },
    { 0xc39d4560, 23 },
    { 0, 0 },
    { 0x81cd4562, 33 },
    { 0x518d4562, 93 },
    { 0x5e4d4564, 91 },
    { 0x9e5d4565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09d4568, 43 },
    { 0x57ad4569, 33 },
    { 0x921d4569, 40 },
    { 0x7b6d456a, 85 },
    { 0xe61d456b, 23 },
    { 0xaf3d456c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894d4572, 91 },
    { 0x82ad4573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd4576, 61 },
    { 0x5ead4576, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad457a, 76 },
    { 0xb0ed457a, 61 },
    { 0x84bd457b, 23 },
    { 0xd2ad457d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4586, 80 },
    { 0xc48d4587, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9d458c, 90 },
    { 0, 0 },
    { 0xaead458e, 90 },
    { 0xdffd458f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d4594, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9d459b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd45a3, 68 },
    { 0xadd45a4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3d45a7, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcd45aa, 43 },
    { 0x50ed45ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d45b1, 33 },
    { 0, 0 },
    { 0xa07d45b3, 90 },
    { 0, 0 },
    { 0xc9fd45b5, 43 },
    { 0x5a3d45b6, 34 },
    { 0x29fd45b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dd45c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bd45c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dd45c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5d45cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7d45cf, 55 },
    { 0x304d45d0, 98 },
    { 0x5efd45d1, 55 },
    { 0xa0dd45d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ad45d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1d45da, 23 },
    { 0xd5ed45da, 98 },
    { 0x1e7d45dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d45df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d45e3, 67 },
    { 0xe40d45e4, 98 },
    { 0x7bfd45e4, 90 },
    { 0x2d8d45e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d45eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bd45ee, 76 },
    { 0xabd45ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbd45f3, 90 },
    { 0, 0 },
    { 0xd10d45f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd45ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9d4606, 23 },
    { 0x37cd4607, 90 },
    { 0xf87d4607, 61 },
    { 0xbfbd4609, 34 },
    { 0, 0 },
    { 0xe6ad460b, 90 },
    { 0, 0 },
    { 0xf47d460d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed4610, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd4614, 23 },
    { 0xa78d4614, 33 },
    { 0x305d4616, 23 },
    { 0x684d4614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8d461a, 98 },
    { 0, 0 },
    { 0x9d7d461c, 43 },
    { 0xcc7d461d, 78 },
    { 0x5a2d461d, 87 },
    { 0x710d461f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d462d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d4633, 93 },
    { 0xad1d4634, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebd463d, 76 },
    { 0xffed463d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d4647, 90 },
    { 0xb6fd4648, 34 },
    { 0xc80d4649, 33 },
    { 0, 0 },
    { 0xeafd464b, 99 },
    { 0, 0 },
    { 0xb13d464d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805d4651, 76 },
    { 0x427d4652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145d4658, 33 },
    { 0xcccd4659, 52 },
    { 0x57ad4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd465f, 4 },
    { 0, 0 },
    { 0x9dad4661, 52 },
    { 0, 0 },
    { 0xd10d4663, 90 },
    { 0, 0 },
    { 0xcb0d4665, 90 },
    { 0, 0 },
    { 0xea3d4667, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd466a, 46 },
    { 0, 0 },
    { 0x7e9d466c, 99 },
    { 0x1f7d466c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d4670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69d4675, 90 },
    { 0, 0 },
    { 0x3c3d4677, 90 },
    { 0x749d4677, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d467d, 55 },
    { 0x4c4d467e, 76 },
    { 0, 0 },
    { 0x885d4680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d4686, 61 },
    { 0, 0 },
    { 0xf4ad4688, 61 },
    { 0, 0 },
    { 0xbcfd468a, 34 },
    { 0x510d468b, 23 },
    { 0, 0 },
    { 0x35cd468d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d4690, 95 },
    { 0x5ccd4691, 33 },
    { 0xd0ed4691, 43 },
    { 0x6bd4692, 84 },
    { 0xe3fd4693, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736d4699, 76 },
    { 0xf52d4699, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d469f, 61 },
    { 0x21dd46a0, 33 },
    { 0x342d469f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd46a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d46a9, 23 },
    { 0, 0 },
    { 0x70dd46ab, 87 },
    { 0x511d46ac, 95 },
    { 0x28cd46ab, 33 },
    { 0x8a7d46ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d46b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ad46c2, 46 },
    { 0x8e4d46c3, 61 },
    { 0, 0 },
    { 0x904d46c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad46c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06d46cc, 80 },
    { 0x5dcd46cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8d46d5, 61 },
    { 0, 0 },
    { 0x581d46d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d46db, 23 },
    { 0, 0 },
    { 0x297d46dd, 55 },
    { 0xaa7d46de, 76 },
    { 0xc29d46df, 55 },
    { 0x8ffd46e0, 90 },
    { 0x752d46e0, 93 },
    { 0x81ad46e2, 95 },
    { 0x7a6d46e2, 33 },
    { 0x269d46e1, 95 },
    { 0x422d46e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd46ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d46ed, 43 },
    { 0, 0 },
    { 0xca5d46ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d46f2, 84 },
    { 0, 0 },
    { 0xd68d46f4, 65 },
    { 0x581d46f5, 9 },
    { 0x5bfd46f5, 61 },
    { 0x930d46f7, 90 },
    { 0, 0 },
    { 0x6ded46f9, 88 },
    { 0, 0 },
    { 0xfccd46fb, 33 },
    { 0x6a4d46fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244d4704, 33 },
    { 0x8ed4704, 95 },
    { 0xed3d4704, 68 },
    { 0xd0fd4707, 61 },
    { 0, 0 },
    { 0x1c5d4709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bd470c, 61 },
    { 0x62ed470d, 34 },
    { 0x3c7d470d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7d4711, 97 },
    { 0xf9fd4712, 95 },
    { 0x5f3d4713, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d4717, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21d471e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d4722, 61 },
    { 0xa25d4722, 95 },
    { 0xcd6d4724, 90 },
    { 0x809d4724, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd472a, 98 },
    { 0xbcd472b, 76 },
    { 0xfe0d472b, 33 },
    { 0, 0 },
    { 0x8a0d472e, 61 },
    { 0, 0 },
    { 0x5b6d4730, 77 },
    { 0, 0 },
    { 0x71ad4732, 77 },
    { 0xc4bd4733, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9d4737, 49 },
    { 0, 0 },
    { 0xf27d4739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd4740, 95 },
    { 0x4dfd4740, 76 },
    { 0xdb2d4742, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bd4747, 84 },
    { 0xf6ad4748, 55 },
    { 0xaf7d4749, 68 },
    { 0xd2ed474a, 55 },
    { 0, 0 },
    { 0x677d474c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d4753, 77 },
    { 0xd71d4754, 87 },
    { 0x478d4755, 90 },
    { 0x758d4754, 61 },
    { 0xaebd4757, 23 },
    { 0xcaed4758, 78 },
    { 0x2add4758, 34 },
    { 0xac2d4759, 98 },
    { 0xb37d475b, 87 },
    { 0xe13d475b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ad4762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5d4766, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbd4769, 33 },
    { 0, 0 },
    { 0xb2ed476b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d476f, 34 },
    { 0x7fad4770, 95 },
    { 0xf3dd4770, 95 },
    { 0xe5cd4772, 76 },
    { 0x572d4773, 33 },
    { 0x112d4773, 98 },
    { 0xdcad476f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x961d4778, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7d477d, 76 },
    { 0, 0 },
    { 0x30dd477f, 88 },
    { 0x66fd4780, 33 },
    { 0x2d1d4781, 68 },
    { 0x55d4782, 91 },
    { 0x2bbd4782, 61 },
    { 0xe77d4782, 23 },
    { 0x5e9d4785, 76 },
    { 0, 0 },
    { 0xb0ed4787, 93 },
    { 0x1a5d4787, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d4790, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd4794, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d4798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73d479c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82d479f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fd47a3, 88 },
    { 0xaf2d47a4, 23 },
    { 0, 0 },
    { 0x5e6d47a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07d47ac, 76 },
    { 0, 0 },
    { 0xb17d47ae, 90 },
    { 0xb15d47af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1add47b2, 90 },
    { 0xd94d47b3, 52 },
    { 0xc36d47b4, 78 },
    { 0x9dd47b5, 61 },
    { 0x9d6d47b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fd47bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d47c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d47cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d47d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d47d5, 77 },
    { 0x2dfd47d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ed47dc, 83 },
    { 0x1b3d47dd, 46 },
    { 0xf14d47dc, 56 },
    { 0xd84d47dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24d47e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x817d47e6, 91 },
    { 0x800d47e7, 90 },
    { 0xd40d47e7, 66 },
    { 0, 0 },
    { 0x51bd47ea, 90 },
    { 0xb25d47eb, 92 },
    { 0xdf0d47ec, 23 },
    { 0x1c6d47ec, 68 },
    { 0x220d47ee, 66 },
    { 0x43ed47ed, 76 },
    { 0xb6fd47ea, 61 },
    { 0x4d8d47f1, 61 },
    { 0x4d0d47f0, 23 },
    { 0x5f7d47f3, 33 },
    { 0, 0 },
    { 0xa3dd47f5, 23 },
    { 0xeb0d47f6, 61 },
    { 0x4ed47f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3d47fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36d47fe, 93 },
    { 0x4c6d47ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3d4803, 55 },
    { 0, 0 },
    { 0x246d4805, 55 },
    { 0, 0 },
    { 0xfd7d4807, 52 },
    { 0x89bd4807, 33 },
    { 0x9ccd4807, 90 },
    { 0x1e1d480a, 46 },
    { 0xcc3d480b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d480e, 61 },
    { 0, 0 },
    { 0x1fdd4810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2d4813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d4816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d481d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd4820, 33 },
    { 0, 0 },
    { 0x54ad4822, 61 },
    { 0, 0 },
    { 0xd08d4824, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540d4828, 93 },
    { 0x78bd4829, 46 },
    { 0x873d482a, 95 },
    { 0xbcd4829, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d482e, 33 },
    { 0x349d482f, 95 },
    { 0x549d482f, 84 },
    { 0x3ccd4830, 33 },
    { 0, 0 },
    { 0x915d4833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd4836, 77 },
    { 0x1dbd4836, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6d4840, 95 },
    { 0, 0 },
    { 0xf50d4842, 95 },
    { 0, 0 },
    { 0x9f5d4844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dd4847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8d484a, 76 },
    { 0x728d484b, 90 },
    { 0xe21d484b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0d484f, 53 },
    { 0x9b5d484f, 68 },
    { 0xe24d4851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d4856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed485a, 96 },
    { 0x174d485a, 55 },
    { 0x2ecd485c, 23 },
    { 0x382d485d, 95 },
    { 0x37cd485d, 61 },
    { 0xcfbd485d, 90 },
    { 0xaccd485e, 33 },
    { 0xc14d485d, 84 },
    { 0xb57d4862, 93 },
    { 0xcc8d4863, 93 },
    { 0, 0 },
    { 0x43ad4865, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d4868, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x641d486b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fd4874, 95 },
    { 0xe7ed4875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ed487e, 33 },
    { 0, 0 },
    { 0x404d4880, 93 },
    { 0xb32d4881, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bd4884, 95 },
    { 0x4ced4885, 61 },
    { 0xfbed4886, 97 },
    { 0xe29d4886, 90 },
    { 0xf9ad4888, 78 },
    { 0xe59d4889, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72d4891, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d4899, 85 },
    { 0xb1d489a, 81 },
    { 0x4ded489b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed489e, 98 },
    { 0x188d489f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd48a2, 61 },
    { 0x5fed48a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4d48a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dd48aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81d48ae, 90 },
    { 0x475d48af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d48b5, 52 },
    { 0x8d9d48b6, 33 },
    { 0xa49d48b7, 34 },
    { 0, 0 },
    { 0x7f5d48b9, 76 },
    { 0, 0 },
    { 0x362d48bb, 90 },
    { 0, 0 },
    { 0x23ad48bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0d48c5, 90 },
    { 0xa22d48c5, 52 },
    { 0x62bd48c5, 91 },
    { 0x537d48c7, 61 },
    { 0x293d48c9, 33 },
    { 0xd61d48c9, 61 },
    { 0x850d48c6, 43 },
    { 0x12ad48c8, 34 },
    { 0xa6fd48cd, 55 },
    { 0x311d48ce, 23 },
    { 0x394d48cc, 98 },
    { 0x6e7d48d0, 93 },
    { 0xfb2d48d1, 52 },
    { 0x70d48d2, 61 },
    { 0x53fd48d2, 55 },
    { 0x764d48d1, 34 },
    { 0x471d48d5, 76 },
    { 0xd9bd48d2, 99 },
    { 0x5b8d48cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd48db, 76 },
    { 0xb5fd48dc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x997d48df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d48e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fd48e8, 90 },
    { 0xf60d48e9, 56 },
    { 0xe1bd48ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d48ed, 33 },
    { 0, 0 },
    { 0x7bd48ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d48f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2d48f8, 90 },
    { 0x3bbd48f9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d48fe, 23 },
    { 0xf6ed48ff, 23 },
    { 0, 0 },
    { 0x6bed4901, 55 },
    { 0x92ad4902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25d4906, 23 },
    { 0xe6fd4907, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7d490b, 90 },
    { 0x9bd490c, 87 },
    { 0, 0 },
    { 0x83dd490e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d4914, 23 },
    { 0xc74d4915, 90 },
    { 0, 0 },
    { 0x1a8d4917, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1add491a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420d4923, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d492d, 61 },
    { 0xe69d492e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29d4939, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdd493f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d4942, 23 },
    { 0, 0 },
    { 0x4a9d4944, 34 },
    { 0x42cd4945, 52 },
    { 0x6e5d4944, 95 },
    { 0x639d4947, 46 },
    { 0, 0 },
    { 0xa5bd4949, 95 },
    { 0x97ed494a, 56 },
    { 0, 0 },
    { 0x122d494c, 95 },
    { 0x259d494c, 87 },
    { 0xdc2d494d, 68 },
    { 0x35dd494f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ed4954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29d495a, 61 },
    { 0, 0 },
    { 0x45fd495c, 61 },
    { 0x446d495d, 61 },
    { 0, 0 },
    { 0xc80d495f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3d496e, 23 },
    { 0xa37d496f, 42 },
    { 0x479d496e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd4973, 43 },
    { 0xa03d4973, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cd4979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365d497e, 23 },
    { 0x505d497f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d4983, 90 },
    { 0xc57d4984, 66 },
    { 0x71dd4985, 91 },
    { 0x826d4986, 87 },
    { 0x676d4986, 33 },
    { 0xd64d4988, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd498e, 98 },
    { 0x94fd498f, 98 },
    { 0xa0ad4990, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaad4997, 28 },
    { 0, 0 },
    { 0x350d4999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d499c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d49a1, 52 },
    { 0x33ed49a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d49a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5d49b4, 85 },
    { 0, 0 },
    { 0x208d49b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819d49ba, 65 },
    { 0xc0ad49ba, 52 },
    { 0xf87d49bb, 34 },
    { 0x7e4d49bd, 33 },
    { 0xf63d49be, 98 },
    { 0x39ed49be, 34 },
    { 0xa15d49bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271d49cc, 68 },
    { 0, 0 },
    { 0x6d6d49ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576d49d3, 34 },
    { 0x574d49d4, 95 },
    { 0, 0 },
    { 0xf68d49d6, 43 },
    { 0, 0 },
    { 0x46cd49d8, 91 },
    { 0x582d49d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548d49e2, 61 },
    { 0x401d49e3, 61 },
    { 0xea7d49e4, 61 },
    { 0, 0 },
    { 0x583d49e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d49ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d49ed, 76 },
    { 0x814d49ee, 23 },
    { 0x459d49ef, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd49f4, 33 },
    { 0xf7cd49f4, 23 },
    { 0x82cd49f4, 98 },
    { 0x466d49f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd49fb, 66 },
    { 0, 0 },
    { 0x329d49fd, 98 },
    { 0x44ed49fd, 77 },
    { 0x846d49fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d4a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78d4a09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d4a13, 55 },
    { 0, 0 },
    { 0x634d4a15, 61 },
    { 0xaded4a16, 90 },
    { 0x281d4a17, 61 },
    { 0x816d4a18, 90 },
    { 0xc27d4a17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x131d4a1c, 76 },
    { 0x94fd4a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d4a2b, 90 },
    { 0x889d4a2c, 23 },
    { 0x5bcd4a2d, 75 },
    { 0x422d4a2d, 91 },
    { 0xaa1d4a2d, 55 },
    { 0x14fd4a2e, 19 },
    { 0xd7bd4a31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487d4a36, 91 },
    { 0x5bfd4a37, 66 },
    { 0x41d4a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d4a43, 55 },
    { 0, 0 },
    { 0xfd3d4a45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd4a4a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dd4a4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fd4a51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6d4a54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bd4a5a, 76 },
    { 0x26fd4a5a, 33 },
    { 0xbb7d4a5c, 93 },
    { 0, 0 },
    { 0x6dcd4a5e, 84 },
    { 0xb67d4a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93d4a6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728d4a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0d4a74, 98 },
    { 0, 0 },
    { 0x76bd4a76, 23 },
    { 0, 0 },
    { 0x7dcd4a78, 55 },
    { 0xf3fd4a79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd4a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd4a88, 90 },
    { 0x8d7d4a88, 55 },
    { 0, 0 },
    { 0x4aed4a8b, 95 },
    { 0x6c6d4a8c, 77 },
    { 0x85cd4a8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427d4a92, 98 },
    { 0xd05d4a93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd4a97, 78 },
    { 0x654d4a97, 80 },
    { 0xe0ad4a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d4a9e, 46 },
    { 0xc38d4a9f, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d4aaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dd4ab2, 98 },
    { 0x131d4ab2, 76 },
    { 0, 0 },
    { 0xcabd4ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d4aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d4ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdd4ac4, 33 },
    { 0x9a1d4ac5, 75 },
    { 0xc74d4ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0d4acb, 23 },
    { 0x53d4acb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0d4ad4, 69 },
    { 0x8ffd4ad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d4ad8, 90 },
    { 0xbf0d4ad9, 55 },
    { 0, 0 },
    { 0x7aed4adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17d4ade, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28d4ae8, 84 },
    { 0xa13d4ae9, 46 },
    { 0, 0 },
    { 0x36d4aeb, 34 },
    { 0x1f0d4aec, 34 },
    { 0xcefd4aed, 23 },
    { 0, 0 },
    { 0xbb4d4aef, 43 },
    { 0, 0 },
    { 0x185d4af1, 99 },
    { 0x259d4af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd4af8, 95 },
    { 0x3c9d4af9, 78 },
    { 0, 0 },
    { 0x61cd4afb, 23 },
    { 0x7f3d4afb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4aff, 43 },
    { 0xa85d4aff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0d4b04, 77 },
    { 0, 0 },
    { 0x87d4b06, 95 },
    { 0x399d4b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d4b0d, 95 },
    { 0x898d4b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad4b11, 77 },
    { 0x5bfd4b12, 99 },
    { 0xe1fd4b13, 61 },
    { 0x53ed4b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d4b19, 33 },
    { 0, 0 },
    { 0xa28d4b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bd4b1f, 49 },
    { 0xfffd4b20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d4b23, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3d4b26, 17 },
    { 0xcbcd4b27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d4b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ed4b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfd4b34, 33 },
    { 0, 0 },
    { 0xa53d4b36, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9d4b3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31d4b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd4b47, 90 },
    { 0xb5ad4b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d4b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d4b4f, 83 },
    { 0, 0 },
    { 0x632d4b51, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d4b57, 69 },
    { 0x721d4b58, 98 },
    { 0x478d4b59, 55 },
    { 0, 0 },
    { 0x15ed4b5b, 34 },
    { 0, 0 },
    { 0x19bd4b5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d4b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d4b68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804d4b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cd4b77, 23 },
    { 0xaded4b78, 33 },
    { 0, 0 },
    { 0xe40d4b7a, 98 },
    { 0x956d4b7a, 84 },
    { 0x7fbd4b7b, 34 },
    { 0x193d4b7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9d4b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d4b83, 81 },
    { 0x484d4b83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fd4b87, 67 },
    { 0x109d4b87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8d4b99, 70 },
    { 0x506d4b9a, 90 },
    { 0x41cd4b9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad4ba2, 33 },
    { 0xecad4ba3, 61 },
    { 0xb7cd4ba4, 67 },
    { 0x34fd4ba5, 90 },
    { 0xa9ad4ba6, 33 },
    { 0, 0 },
    { 0xe3ad4ba8, 90 },
    { 0x742d4ba8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7d4bac, 90 },
    { 0x709d4bac, 55 },
    { 0x70dd4bad, 46 },
    { 0x7c2d4bad, 43 },
    { 0x826d4bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d4bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fd4bba, 90 },
    { 0x6bad4bbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552d4bc2, 76 },
    { 0x377d4bc3, 34 },
    { 0x4cad4bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4d4bc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eed4bcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4bd8, 33 },
    { 0x669d4bd9, 23 },
    { 0x51ed4bda, 85 },
    { 0x2c9d4bdb, 95 },
    { 0xf7ad4bdc, 85 },
    { 0, 0 },
    { 0xfad4bde, 91 },
    { 0x9a2d4bdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcd4be3, 80 },
    { 0, 0 },
    { 0xfd3d4be5, 98 },
    { 0x227d4be5, 23 },
    { 0, 0 },
    { 0xfd7d4be8, 61 },
    { 0, 0 },
    { 0x768d4bea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d4bf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d4bf5, 61 },
    { 0x843d4bf5, 66 },
    { 0x4abd4bf5, 95 },
    { 0x9dbd4bf8, 33 },
    { 0x8b0d4bf8, 33 },
    { 0x65bd4bf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x587d4bfd, 83 },
    { 0, 0 },
    { 0xe1bd4bff, 87 },
    { 0x9fad4c00, 23 },
    { 0x596d4c00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d4c04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd4c09, 90 },
    { 0, 0 },
    { 0x1cdd4c0b, 66 },
    { 0, 0 },
    { 0xe32d4c0d, 61 },
    { 0, 0 },
    { 0x3b7d4c0f, 95 },
    { 0x457d4c10, 95 },
    { 0x1e7d4c11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840d4c19, 90 },
    { 0xd7fd4c1a, 95 },
    { 0x502d4c19, 76 },
    { 0x6b1d4c1c, 38 },
    { 0x74fd4c19, 85 },
    { 0x2e4d4c1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cd4c21, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330d4c26, 69 },
    { 0x5a2d4c26, 90 },
    { 0x24ad4c28, 98 },
    { 0x8dcd4c29, 98 },
    { 0x9d4c27, 55 },
    { 0xdd2d4c28, 70 },
    { 0xec8d4c2c, 95 },
    { 0x8e0d4c26, 90 },
    { 0x6a8d4c2e, 98 },
    { 0x6e2d4c2f, 61 },
    { 0x529d4c30, 88 },
    { 0xa25d4c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d4c34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4c39, 76 },
    { 0x107d4c3a, 95 },
    { 0x3d9d4c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6d4c40, 81 },
    { 0x81cd4c41, 23 },
    { 0x154d4c42, 95 },
    { 0xc1ad4c43, 76 },
    { 0x201d4c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d4c48, 52 },
    { 0x2e8d4c49, 90 },
    { 0x5a1d4c48, 23 },
    { 0xfdcd4c4b, 55 },
    { 0xfdcd4c4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3d4c51, 23 },
    { 0x2e8d4c51, 86 },
    { 0x657d4c53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d4c57, 17 },
    { 0x414d4c57, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd4c5d, 95 },
    { 0, 0 },
    { 0xa51d4c5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d4c62, 90 },
    { 0xa41d4c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d4c73, 48 },
    { 0, 0 },
    { 0x93bd4c75, 90 },
    { 0xdd9d4c75, 98 },
    { 0x622d4c75, 100 },
    { 0xf4ad4c78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d4c7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad4c81, 43 },
    { 0xb58d4c81, 83 },
    { 0xf66d4c83, 90 },
    { 0x51cd4c83, 90 },
    { 0x847d4c85, 95 },
    { 0, 0 },
    { 0xc8fd4c87, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4c8e, 61 },
    { 0xf69d4c8f, 67 },
    { 0x7edd4c8e, 98 },
    { 0xf0ed4c8f, 23 },
    { 0x60cd4c91, 23 },
    { 0, 0 },
    { 0x50d4c94, 98 },
    { 0x9d7d4c95, 95 },
    { 0x4e7d4c95, 95 },
    { 0x10d4c97, 43 },
    { 0x5ebd4c97, 33 },
    { 0x980d4c99, 98 },
    { 0x381d4c9a, 67 },
    { 0x53bd4c9a, 68 },
    { 0xb9cd4c9a, 61 },
    { 0x299d4c97, 100 },
    { 0xd0bd4c9e, 85 },
    { 0, 0 },
    { 0xabdd4ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8d4ca4, 93 },
    { 0, 0 },
    { 0xe6dd4ca6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343d4cab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148d4cb2, 55 },
    { 0xc5dd4cb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd4cb6, 100 },
    { 0x1e8d4cb7, 98 },
    { 0xda7d4cb8, 95 },
    { 0xb3cd4cb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed4cc3, 78 },
    { 0x2ffd4cc4, 88 },
    { 0xe7ad4cc5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dd4cc9, 87 },
    { 0, 0 },
    { 0xd65d4ccb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3d4cd1, 98 },
    { 0x371d4cd2, 24 },
    { 0, 0 },
    { 0x143d4cd4, 23 },
    { 0, 0 },
    { 0x9bdd4cd6, 33 },
    { 0x315d4cd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fd4cda, 23 },
    { 0x245d4cdb, 98 },
    { 0xd2d4cda, 90 },
    { 0, 0 },
    { 0x333d4cde, 98 },
    { 0x41ad4cdf, 17 },
    { 0x2dd4ce0, 93 },
    { 0xc43d4cde, 55 },
    { 0x2a5d4ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed4ce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd4cf0, 33 },
    { 0, 0 },
    { 0x858d4cf2, 34 },
    { 0xd36d4cf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6d4cf8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d4cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d4d06, 91 },
    { 0x8a0d4d06, 90 },
    { 0x80ad4d08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5d4d0c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441d4d12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d4d15, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d4d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d4d1c, 61 },
    { 0x5b5d4d1c, 91 },
    { 0x947d4d1e, 46 },
    { 0xd09d4d1e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3d4d23, 34 },
    { 0, 0 },
    { 0x63ed4d25, 95 },
    { 0xfc2d4d26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x721d4d29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd4d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d4d31, 61 },
    { 0x6b0d4d32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ced4d39, 33 },
    { 0x104d4d3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd4d3f, 90 },
    { 0xaccd4d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263d4d44, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690d4d48, 76 },
    { 0xf93d4d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187d4d50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d4d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd4d58, 33 },
    { 0x28cd4d59, 33 },
    { 0, 0 },
    { 0xf35d4d5b, 90 },
    { 0x826d4d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4d60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcced4d63, 90 },
    { 0x953d4d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87d4d68, 76 },
    { 0xd63d4d69, 46 },
    { 0xba6d4d6a, 78 },
    { 0x483d4d6b, 90 },
    { 0x74fd4d6b, 95 },
    { 0x5c1d4d6d, 84 },
    { 0xa55d4d6d, 93 },
    { 0, 0 },
    { 0xf02d4d70, 33 },
    { 0, 0 },
    { 0x65bd4d72, 52 },
    { 0, 0 },
    { 0x32d4d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d4d78, 33 },
    { 0x935d4d79, 61 },
    { 0xe6cd4d7a, 76 },
    { 0xf47d4d78, 55 },
    { 0xd4ed4d7c, 33 },
    { 0xc6cd4d79, 98 },
    { 0x72fd4d7d, 43 },
    { 0xd6ad4d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d4d84, 33 },
    { 0x3cfd4d84, 43 },
    { 0, 0 },
    { 0xfffd4d87, 93 },
    { 0xf78d4d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d4d8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddd4d90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fd4d93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fd4d98, 23 },
    { 0x6aed4d99, 43 },
    { 0, 0 },
    { 0xd16d4d9b, 90 },
    { 0, 0 },
    { 0xa58d4d9d, 61 },
    { 0xc7dd4d9e, 66 },
    { 0x6c2d4d9e, 91 },
    { 0x65ed4d9e, 98 },
    { 0x491d4da1, 23 },
    { 0, 0 },
    { 0x301d4da3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed4da9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d4dad, 34 },
    { 0x372d4dae, 33 },
    { 0xee5d4daf, 33 },
    { 0x9dcd4db0, 90 },
    { 0x9bed4db0, 87 },
    { 0xae3d4db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428d4db9, 33 },
    { 0, 0 },
    { 0xa7d4dbb, 52 },
    { 0xa9dd4dbc, 65 },
    { 0, 0 },
    { 0xae3d4dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd4dc3, 95 },
    { 0x808d4dc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dad4dc9, 98 },
    { 0xac1d4dca, 33 },
    { 0xf16d4dcb, 61 },
    { 0xa31d4dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd4dd2, 33 },
    { 0, 0 },
    { 0xa67d4dd4, 34 },
    { 0x652d4dd4, 95 },
    { 0, 0 },
    { 0x927d4dd7, 95 },
    { 0x472d4dd8, 23 },
    { 0x3ffd4dd8, 80 },
    { 0xb16d4dda, 90 },
    { 0xa57d4dd7, 98 },
    { 0, 0 },
    { 0xf7dd4ddd, 91 },
    { 0x731d4dde, 87 },
    { 0x26ad4ddf, 33 },
    { 0, 0 },
    { 0xeffd4de1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ed4de6, 23 },
    { 0xb80d4de7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5d4dea, 61 },
    { 0, 0 },
    { 0x11d4dec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d4df6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144d4dfa, 55 },
    { 0xdc0d4dfb, 100 },
    { 0x2e4d4dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55d4dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d4e09, 90 },
    { 0x937d4e09, 87 },
    { 0x322d4e09, 33 },
    { 0x4eed4e0c, 33 },
    { 0xe23d4e0a, 36 },
    { 0x698d4e0e, 98 },
    { 0xa6ad4e0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948d4e16, 98 },
    { 0, 0 },
    { 0x995d4e18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d4e24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d4e29, 90 },
    { 0x942d4e2a, 61 },
    { 0xd69d4e2b, 98 },
    { 0x47bd4e2b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ed4e31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d4e34, 33 },
    { 0x325d4e34, 76 },
    { 0, 0 },
    { 0x462d4e37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17d4e3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279d4e40, 99 },
    { 0x180d4e40, 76 },
    { 0, 0 },
    { 0x5cd4e43, 61 },
    { 0xaf1d4e44, 33 },
    { 0, 0 },
    { 0x60ed4e46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x334d4e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24d4e4d, 99 },
    { 0, 0 },
    { 0x8aad4e4f, 98 },
    { 0, 0 },
    { 0xfe4d4e51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570d4e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd4e5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd4e5f, 46 },
    { 0, 0 },
    { 0x2a4d4e61, 66 },
    { 0, 0 },
    { 0x583d4e63, 90 },
    { 0xd70d4e64, 23 },
    { 0x29bd4e65, 64 },
    { 0x174d4e64, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aed4e69, 95 },
    { 0x71dd4e69, 75 },
    { 0x8b8d4e69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d4e6e, 94 },
    { 0x318d4e6f, 69 },
    { 0xe0ed4e6f, 23 },
    { 0x3b3d4e71, 100 },
    { 0x28ad4e72, 33 },
    { 0x3b8d4e73, 76 },
    { 0x2d4e70, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad4e77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x528d4e7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d4e7f, 23 },
    { 0xe9fd4e80, 90 },
    { 0, 0 },
    { 0xee1d4e82, 95 },
    { 0, 0 },
    { 0xb4cd4e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ed4e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ad4e8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4d4e90, 61 },
    { 0, 0 },
    { 0xa9ed4e92, 33 },
    { 0, 0 },
    { 0xdddd4e94, 98 },
    { 0x813d4e94, 23 },
    { 0, 0 },
    { 0x2aed4e97, 8 },
    { 0x33bd4e98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d4ea0, 95 },
    { 0xae9d4ea1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x125d4ea4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d4eac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7d4eb6, 98 },
    { 0, 0 },
    { 0x621d4eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd4ebd, 90 },
    { 0x6e2d4ebe, 33 },
    { 0xa79d4ebf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d4ec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d4ec5, 90 },
    { 0x8fbd4ec6, 90 },
    { 0x65ed4ec6, 90 },
    { 0xbbad4ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852d4ecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d4ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cd4eda, 90 },
    { 0, 0 },
    { 0xa89d4edc, 98 },
    { 0xf1d4edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd4ee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d4ee7, 90 },
    { 0x5aad4ee8, 46 },
    { 0x111d4ee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d4eef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ed4ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d4efd, 66 },
    { 0, 0 },
    { 0x798d4eff, 93 },
    { 0xf0ad4eff, 23 },
    { 0x5ddd4f01, 34 },
    { 0xe7bd4f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82d4f05, 55 },
    { 0, 0 },
    { 0xa43d4f07, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd4f0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed4f15, 93 },
    { 0x87fd4f16, 23 },
    { 0x63cd4f15, 61 },
    { 0xe17d4f18, 90 },
    { 0x5b7d4f19, 33 },
    { 0xfd6d4f18, 95 },
    { 0xc11d4f16, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522d4f1f, 98 },
    { 0, 0 },
    { 0x83bd4f21, 98 },
    { 0x82cd4f22, 33 },
    { 0x18ed4f22, 33 },
    { 0x609d4f22, 90 },
    { 0xbc3d4f24, 46 },
    { 0x4f0d4f26, 95 },
    { 0, 0 },
    { 0xf37d4f28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970d4f2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x744d4f31, 84 },
    { 0, 0 },
    { 0x90ad4f33, 66 },
    { 0x211d4f33, 43 },
    { 0x371d4f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d4f39, 34 },
    { 0x40ed4f39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd4f3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1d4f41, 90 },
    { 0xc2bd4f42, 77 },
    { 0xd6bd4f42, 93 },
    { 0xdf6d4f42, 34 },
    { 0x813d4f45, 90 },
    { 0x96bd4f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143d4f52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d4f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174d4f5d, 90 },
    { 0, 0 },
    { 0x48ad4f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6d4f65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80d4f70, 61 },
    { 0, 0 },
    { 0xa2cd4f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd4f75, 33 },
    { 0xbb5d4f75, 55 },
    { 0x666d4f77, 96 },
    { 0x2afd4f78, 95 },
    { 0x418d4f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bd4f7c, 95 },
    { 0x40d4f7d, 85 },
    { 0, 0 },
    { 0x36ed4f7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729d4f87, 34 },
    { 0x146d4f87, 95 },
    { 0x1a7d4f89, 99 },
    { 0xdfd4f8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d4f8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628d4f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cd4f96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d4f99, 55 },
    { 0, 0 },
    { 0xe44d4f9b, 33 },
    { 0, 0 },
    { 0x3e3d4f9d, 78 },
    { 0xfe3d4f9e, 46 },
    { 0xbbad4f9f, 90 },
    { 0xc1fd4fa0, 61 },
    { 0xb53d4fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced4fa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d4fad, 90 },
    { 0, 0 },
    { 0x162d4faf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919d4fb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718d4fb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed4fbf, 98 },
    { 0, 0 },
    { 0xa04d4fc1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd4fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd4fd5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5d4fd9, 93 },
    { 0xee0d4fda, 98 },
    { 0x260d4fdb, 99 },
    { 0, 0 },
    { 0x50bd4fdd, 93 },
    { 0, 0 },
    { 0xac0d4fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd4fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4fe9, 90 },
    { 0x365d4fea, 33 },
    { 0, 0 },
    { 0x89ad4fec, 55 },
    { 0xec3d4fed, 61 },
    { 0xc29d4fec, 90 },
    { 0x989d4fef, 23 },
    { 0x97d4fee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ed4ff5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbd4ff8, 99 },
    { 0x517d4ff8, 55 },
    { 0x957d4ffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad4ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d5001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264d5006, 34 },
    { 0xeadd5007, 55 },
    { 0x91dd5008, 43 },
    { 0, 0 },
    { 0x5bed500a, 76 },
    { 0x12dd500b, 46 },
    { 0x356d500c, 93 },
    { 0x555d500c, 23 },
    { 0x1acd500e, 33 },
    { 0, 0 },
    { 0x2dd5010, 98 },
    { 0x6fd5011, 78 },
    { 0x68d5012, 100 },
    { 0xbeed5013, 61 },
    { 0xe28d5014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x892d5017, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x672d501a, 98 },
    { 0xb87d501a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12d501e, 52 },
    { 0x193d501e, 90 },
    { 0, 0 },
    { 0x413d5021, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cd5025, 23 },
    { 0, 0 },
    { 0x67fd5027, 95 },
    { 0, 0 },
    { 0x3fd5029, 61 },
    { 0x5fdd502a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2d5032, 61 },
    { 0, 0 },
    { 0xf8dd5034, 23 },
    { 0x489d5035, 95 },
    { 0, 0 },
    { 0x241d5037, 61 },
    { 0x351d5037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1d503b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6d5041, 55 },
    { 0x6b5d5042, 93 },
    { 0, 0 },
    { 0xde7d5044, 96 },
    { 0, 0 },
    { 0x41fd5046, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad5049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d504d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d5057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ed505a, 33 },
    { 0x583d505b, 95 },
    { 0xd40d505a, 61 },
    { 0x1dcd505d, 23 },
    { 0, 0 },
    { 0xaed505f, 34 },
    { 0, 0 },
    { 0x54fd5061, 52 },
    { 0x7f7d5061, 99 },
    { 0x238d5063, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd5069, 61 },
    { 0x3c7d5069, 100 },
    { 0, 0 },
    { 0xf1cd506c, 65 },
    { 0x5f5d506d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329d5074, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5d5078, 23 },
    { 0x3cd5079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d507c, 23 },
    { 0x35ed507c, 76 },
    { 0xc9ad507c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x369d5081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37d5085, 55 },
    { 0xcc3d5086, 98 },
    { 0, 0 },
    { 0x2bd5088, 33 },
    { 0, 0 },
    { 0x5fbd508a, 98 },
    { 0xae1d508a, 66 },
    { 0, 0 },
    { 0x9f9d508d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d5092, 33 },
    { 0, 0 },
    { 0x469d5094, 33 },
    { 0, 0 },
    { 0xf30d5096, 61 },
    { 0xc04d5097, 76 },
    { 0xa10d5098, 76 },
    { 0x421d5096, 55 },
    { 0xffdd5097, 33 },
    { 0, 0 },
    { 0xa10d509c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dd50a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820d50ac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61d50b3, 55 },
    { 0x3a2d50b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed50b8, 90 },
    { 0, 0 },
    { 0x475d50ba, 33 },
    { 0xf4d50bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x163d50be, 95 },
    { 0xab3d50bf, 65 },
    { 0xca6d50c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1d50c3, 95 },
    { 0, 0 },
    { 0xd9bd50c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x729d50c8, 96 },
    { 0xa1ed50c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bad50cc, 98 },
    { 0xfc7d50cc, 33 },
    { 0x2ddd50ce, 90 },
    { 0x3bbd50cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7d50d4, 43 },
    { 0x886d50d5, 33 },
    { 0xb04d50d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fd50da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d50df, 55 },
    { 0x15cd50e0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ad50e3, 76 },
    { 0, 0 },
    { 0x34ed50e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d50e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2d50f1, 66 },
    { 0x8b4d50f1, 23 },
    { 0x7f9d50f3, 95 },
    { 0xcced50f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885d50f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0d50fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x196d50ff, 43 },
    { 0x5dd5100, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ad5107, 96 },
    { 0x447d5107, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd510e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59d5113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x449d5116, 90 },
    { 0xadcd5116, 69 },
    { 0xb0dd5118, 61 },
    { 0x587d5119, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fd511c, 55 },
    { 0, 0 },
    { 0x142d511e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3d5122, 99 },
    { 0, 0 },
    { 0x215d5124, 42 },
    { 0x25d5124, 61 },
    { 0xf5d5124, 67 },
    { 0x538d5126, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d512c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d5134, 100 },
    { 0xe64d5134, 23 },
    { 0x9c5d5136, 93 },
    { 0, 0 },
    { 0xde2d5138, 40 },
    { 0x1edd5138, 23 },
    { 0x7dbd5138, 68 },
    { 0x8acd513a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d5146, 55 },
    { 0x6d5d5146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ad514a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d514e, 43 },
    { 0xec8d514f, 78 },
    { 0, 0 },
    { 0x9dfd5151, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650d515c, 83 },
    { 0x6c8d515c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfd5168, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdd5171, 55 },
    { 0xdfed5171, 93 },
    { 0xd90d5173, 40 },
    { 0x2b7d5173, 95 },
    { 0x8c7d5172, 61 },
    { 0x472d5171, 90 },
    { 0xdf7d5177, 43 },
    { 0x303d5174, 43 },
    { 0, 0 },
    { 0x831d517a, 98 },
    { 0xc6fd517b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cd5180, 98 },
    { 0x4f4d5180, 90 },
    { 0x882d5181, 33 },
    { 0, 0 },
    { 0xd34d5184, 61 },
    { 0xf8bd5184, 76 },
    { 0x8c4d5186, 37 },
    { 0x6d8d5187, 55 },
    { 0x138d5188, 100 },
    { 0x237d5189, 90 },
    { 0x7a6d518a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886d5190, 61 },
    { 0x55ad5190, 55 },
    { 0xa87d5192, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d5195, 90 },
    { 0x10bd5196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x325d5199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01d519c, 93 },
    { 0, 0 },
    { 0x4e2d519e, 90 },
    { 0x89ed519e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d51a3, 61 },
    { 0, 0 },
    { 0x18dd51a5, 55 },
    { 0x500d51a6, 95 },
    { 0xf21d51a6, 90 },
    { 0, 0 },
    { 0x7f7d51a9, 61 },
    { 0x38ad51aa, 61 },
    { 0xfb3d51aa, 95 },
    { 0x9a5d51ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07d51af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d51b2, 23 },
    { 0x341d51b3, 61 },
    { 0, 0 },
    { 0xe48d51b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad51b9, 23 },
    { 0xa6ad51b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51bf, 61 },
    { 0, 0 },
    { 0xe51d51c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d51c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72d51c7, 93 },
    { 0, 0 },
    { 0xcd3d51c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d51cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cd51cf, 33 },
    { 0xd60d51d0, 33 },
    { 0xdc7d51d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd51d5, 80 },
    { 0x538d51d5, 99 },
    { 0x197d51d7, 95 },
    { 0xdacd51d8, 66 },
    { 0, 0 },
    { 0x96bd51da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810d51de, 98 },
    { 0x188d51df, 95 },
    { 0x18ad51e0, 52 },
    { 0xa59d51df, 98 },
    { 0x1dbd51df, 23 },
    { 0x158d51e1, 98 },
    { 0, 0 },
    { 0x412d51e5, 61 },
    { 0x5cad51e5, 33 },
    { 0x753d51e7, 23 },
    { 0, 0 },
    { 0x72fd51e9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead51f1, 33 },
    { 0xc5cd51f2, 90 },
    { 0xddcd51f3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d51ff, 95 },
    { 0, 0 },
    { 0xd1fd5201, 98 },
    { 0x543d5201, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd5209, 91 },
    { 0xb4ed520a, 90 },
    { 0x36bd520a, 97 },
    { 0x2f8d520c, 90 },
    { 0xba4d520c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597d5211, 46 },
    { 0xc92d5212, 33 },
    { 0, 0 },
    { 0x78ed5214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfd521b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fd521e, 98 },
    { 0xb77d521e, 23 },
    { 0xd4ad5220, 98 },
    { 0x104d5220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd5226, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dd5229, 33 },
    { 0xd54d522a, 33 },
    { 0, 0 },
    { 0x3d6d522c, 55 },
    { 0xc39d522d, 68 },
    { 0x391d522d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d5234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d523a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x481d523d, 90 },
    { 0xdd5d523e, 33 },
    { 0xb36d523f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fd5242, 61 },
    { 0xf28d5243, 76 },
    { 0, 0 },
    { 0xa85d5245, 98 },
    { 0x3f2d5246, 90 },
    { 0, 0 },
    { 0x60cd5248, 77 },
    { 0x67bd5249, 33 },
    { 0xe4ad524a, 61 },
    { 0xbded524b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199d524f, 87 },
    { 0x766d5250, 81 },
    { 0x808d5250, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4d5256, 55 },
    { 0, 0 },
    { 0xe02d5258, 61 },
    { 0xbb2d5259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd525c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813d5261, 98 },
    { 0x2a9d5261, 33 },
    { 0xbf3d5262, 75 },
    { 0, 0 },
    { 0x453d5265, 91 },
    { 0x1aed5265, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd526b, 93 },
    { 0xbe7d526c, 76 },
    { 0xf20d526c, 95 },
    { 0x981d526b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ed5271, 23 },
    { 0xf0ad5272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ed5276, 76 },
    { 0x63cd5276, 90 },
    { 0, 0 },
    { 0x1a8d5279, 34 },
    { 0x19ad527a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bd527d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d5282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22d528a, 66 },
    { 0, 0 },
    { 0x3d2d528c, 33 },
    { 0x8dd528c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ced5292, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bd529b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d52a3, 76 },
    { 0x5a9d52a3, 46 },
    { 0, 0 },
    { 0xa0bd52a6, 98 },
    { 0x479d52a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x742d52aa, 52 },
    { 0xe6d52ab, 34 },
    { 0x93fd52ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884d52b0, 90 },
    { 0xab4d52b1, 61 },
    { 0, 0 },
    { 0xed2d52b3, 78 },
    { 0, 0 },
    { 0x13fd52b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ed52b8, 67 },
    { 0x13dd52b9, 70 },
    { 0x59fd52ba, 40 },
    { 0xef8d52b9, 66 },
    { 0x4e2d52bb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90d52bf, 98 },
    { 0x93fd52c0, 66 },
    { 0x220d52bf, 61 },
    { 0x219d52c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d52cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d52ce, 55 },
    { 0x285d52cf, 95 },
    { 0xbe5d52d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01d52d4, 33 },
    { 0, 0 },
    { 0x4b3d52d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad52dd, 93 },
    { 0xc79d52de, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd52e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d52e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bd52ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0d52ef, 33 },
    { 0x6c4d52ef, 91 },
    { 0x525d52ef, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d52fc, 34 },
    { 0x9dbd52fd, 53 },
    { 0x62d52fe, 61 },
    { 0x86fd52ff, 97 },
    { 0, 0 },
    { 0x909d5301, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7d5304, 33 },
    { 0xd4ed5305, 87 },
    { 0x5c6d5306, 43 },
    { 0x25ad5306, 77 },
    { 0x772d5305, 82 },
    { 0x25d5307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d530c, 98 },
    { 0x804d530d, 90 },
    { 0x673d530e, 67 },
    { 0x4cd530c, 80 },
    { 0, 0 },
    { 0x4dbd5311, 23 },
    { 0, 0 },
    { 0x239d5313, 61 },
    { 0xe95d5313, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957d5318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ad5320, 19 },
    { 0, 0 },
    { 0xf93d5322, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccd5328, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d532b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d532f, 98 },
    { 0x9ecd532f, 100 },
    { 0x2aad5331, 80 },
    { 0x93cd532f, 91 },
    { 0, 0 },
    { 0x6b3d5334, 76 },
    { 0x480d5335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d5338, 23 },
    { 0x2e7d5339, 52 },
    { 0, 0 },
    { 0x7edd533b, 23 },
    { 0x52ed533b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702d5340, 97 },
    { 0, 0 },
    { 0x9bcd5342, 23 },
    { 0xe41d5342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dd534d, 33 },
    { 0, 0 },
    { 0x113d534f, 98 },
    { 0, 0 },
    { 0xf26d5351, 98 },
    { 0, 0 },
    { 0x1b5d5353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d5356, 98 },
    { 0, 0 },
    { 0xfbad5358, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd535c, 90 },
    { 0, 0 },
    { 0x4e8d535e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542d5362, 99 },
    { 0x32fd5363, 68 },
    { 0xe46d5364, 23 },
    { 0x150d5365, 88 },
    { 0xc04d5362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941d536b, 46 },
    { 0x37bd536c, 46 },
    { 0xd15d536b, 98 },
    { 0xf42d536e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd5371, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed5376, 91 },
    { 0xcccd5377, 55 },
    { 0x15ed5377, 95 },
    { 0xf2cd5377, 76 },
    { 0xa2d5378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd537d, 55 },
    { 0, 0 },
    { 0xf5d537f, 55 },
    { 0xacbd5380, 90 },
    { 0x5d4d5381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d5386, 23 },
    { 0x75d5387, 93 },
    { 0, 0 },
    { 0x129d5389, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06d538c, 55 },
    { 0x1d8d538c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd5390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5393, 55 },
    { 0x11d5393, 33 },
    { 0x5a7d5393, 100 },
    { 0x647d5396, 33 },
    { 0x49cd5394, 76 },
    { 0x22cd5398, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6d539c, 85 },
    { 0, 0 },
    { 0x2ecd539e, 23 },
    { 0x94cd539f, 52 },
    { 0x438d53a0, 61 },
    { 0, 0 },
    { 0x3f2d53a2, 99 },
    { 0x328d53a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d53a9, 61 },
    { 0x7c8d53a9, 84 },
    { 0x55dd53ab, 33 },
    { 0xe4bd53ab, 98 },
    { 0, 0 },
    { 0x48ed53ae, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d53b1, 66 },
    { 0xc1ed53b1, 93 },
    { 0xc59d53b2, 46 },
    { 0, 0 },
    { 0xb9bd53b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29d53b8, 23 },
    { 0, 0 },
    { 0x417d53ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2d53bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45d53c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4d53c6, 33 },
    { 0, 0 },
    { 0xa06d53c8, 66 },
    { 0xedcd53c9, 81 },
    { 0x590d53c9, 33 },
    { 0xd94d53cb, 33 },
    { 0x94bd53c9, 80 },
    { 0xb88d53c8, 61 },
    { 0x106d53cb, 80 },
    { 0xa9d53ce, 99 },
    { 0xfa5d53cc, 90 },
    { 0x4c1d53d1, 61 },
    { 0xe62d53d2, 33 },
    { 0x6b0d53d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d53d9, 90 },
    { 0x73cd53da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1d53e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ad53f1, 65 },
    { 0xbdfd53f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d53f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ad5401, 95 },
    { 0, 0 },
    { 0x33cd5403, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974d540a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbd540e, 91 },
    { 0, 0 },
    { 0x878d5410, 98 },
    { 0, 0 },
    { 0xb97d5412, 77 },
    { 0x963d5413, 99 },
    { 0x8fdd5414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2d5417, 95 },
    { 0x130d5417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63d5420, 66 },
    { 0, 0 },
    { 0x5ccd5422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafd5429, 90 },
    { 0xbaed5429, 63 },
    { 0x4d542b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdd542f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27d5432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55d543a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bd5442, 55 },
    { 0x727d5442, 81 },
    { 0x150d5444, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cd5448, 34 },
    { 0x24ad5449, 95 },
    { 0x486d544a, 77 },
    { 0, 0 },
    { 0xcf6d544c, 93 },
    { 0, 0 },
    { 0xae1d544e, 90 },
    { 0xc21d544f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ed5452, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d5455, 33 },
    { 0x968d5456, 88 },
    { 0, 0 },
    { 0x384d5458, 77 },
    { 0xe8ad5458, 23 },
    { 0xbb9d5458, 76 },
    { 0x473d5458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5461, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd5465, 77 },
    { 0x2f9d5465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d546b, 93 },
    { 0x14cd546b, 90 },
    { 0, 0 },
    { 0x42d546e, 99 },
    { 0xcbbd546f, 90 },
    { 0x83cd5470, 78 },
    { 0x716d5471, 93 },
    { 0x89cd5470, 33 },
    { 0xb37d5472, 99 },
    { 0x6c9d546e, 87 },
    { 0xa0dd546f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd547c, 76 },
    { 0xfb0d547d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cd5483, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99d548a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x366d548d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cd5490, 85 },
    { 0xdbdd5490, 55 },
    { 0xd33d5490, 33 },
    { 0, 0 },
    { 0xd4fd5494, 75 },
    { 0x373d5495, 23 },
    { 0xcd5496, 84 },
    { 0, 0 },
    { 0xcf7d5498, 33 },
    { 0xd86d5499, 33 },
    { 0x42bd5498, 61 },
    { 0x214d549b, 98 },
    { 0x846d549c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fad54a1, 98 },
    { 0xda4d54a2, 95 },
    { 0xa07d54a3, 98 },
    { 0x87cd54a1, 95 },
    { 0, 0 },
    { 0xa1ad54a6, 61 },
    { 0xd6d54a7, 91 },
    { 0x83cd54a8, 23 },
    { 0x794d54a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd54b4, 90 },
    { 0x887d54b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedd54b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d54bd, 100 },
    { 0x564d54bd, 98 },
    { 0x138d54bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd54c6, 90 },
    { 0x953d54c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcd54d3, 99 },
    { 0x72bd54d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8d54d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad54da, 90 },
    { 0x32ad54db, 33 },
    { 0, 0 },
    { 0x610d54dd, 52 },
    { 0x489d54de, 38 },
    { 0x30fd54dd, 98 },
    { 0x8dcd54de, 88 },
    { 0xbebd54e1, 61 },
    { 0x707d54e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d54e6, 34 },
    { 0x8cd54e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664d54eb, 43 },
    { 0, 0 },
    { 0x9ecd54ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596d54f1, 90 },
    { 0, 0 },
    { 0x8b9d54f3, 76 },
    { 0, 0 },
    { 0xc2ad54f5, 93 },
    { 0x14bd54f6, 55 },
    { 0x6ed54f7, 64 },
    { 0xe63d54f8, 51 },
    { 0x3b2d54f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58d54fe, 67 },
    { 0, 0 },
    { 0xd2ad5500, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafed5509, 33 },
    { 0x528d550a, 91 },
    { 0x8d4d550b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd550e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d5515, 33 },
    { 0, 0 },
    { 0x535d5517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dd551a, 33 },
    { 0xf2ed551b, 95 },
    { 0, 0 },
    { 0xc5d551d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x180d5520, 61 },
    { 0x6cdd5521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891d5528, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d5530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19d553a, 95 },
    { 0, 0 },
    { 0xab1d553c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd5549, 90 },
    { 0x7c6d554a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9d5551, 95 },
    { 0xe0d5552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3d5556, 76 },
    { 0x91bd5557, 90 },
    { 0, 0 },
    { 0xe5dd5559, 66 },
    { 0xead555a, 8 },
    { 0xecfd5559, 33 },
    { 0, 0 },
    { 0x504d555d, 98 },
    { 0, 0 },
    { 0x124d555f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffd5562, 33 },
    { 0xf75d5563, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d5566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcd556a, 33 },
    { 0xcd5d556b, 98 },
    { 0xa3ed556a, 76 },
    { 0xb96d556d, 67 },
    { 0x541d556c, 95 },
    { 0x331d556f, 33 },
    { 0xc2dd556a, 90 },
    { 0x74cd5571, 78 },
    { 0x115d5572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ad5578, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1d557b, 85 },
    { 0x92dd557c, 68 },
    { 0x2a9d557c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d5582, 23 },
    { 0x1a4d5583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676d558b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad558e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d5592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd5597, 61 },
    { 0x8bfd5598, 96 },
    { 0x838d5598, 52 },
    { 0x273d559a, 55 },
    { 0x1e2d559b, 55 },
    { 0x6bd5597, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330d55a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4d55a7, 93 },
    { 0, 0 },
    { 0xbd3d55a9, 76 },
    { 0x6b3d55aa, 76 },
    { 0, 0 },
    { 0x4c0d55ac, 23 },
    { 0x5e5d55ad, 55 },
    { 0x863d55ae, 61 },
    { 0x15dd55ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06d55b6, 33 },
    { 0xccad55b7, 78 },
    { 0, 0 },
    { 0x38bd55b9, 33 },
    { 0x346d55ba, 52 },
    { 0xd9cd55bb, 23 },
    { 0, 0 },
    { 0xf51d55bd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed55c0, 46 },
    { 0xc7bd55c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d55c4, 55 },
    { 0x391d55c4, 33 },
    { 0x5f0d55c6, 95 },
    { 0x6dcd55c6, 90 },
    { 0xa70d55c7, 23 },
    { 0x296d55c4, 61 },
    { 0x296d55ca, 81 },
    { 0xef4d55ca, 23 },
    { 0, 0 },
    { 0x684d55cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ad55d0, 33 },
    { 0x179d55d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9d55d8, 90 },
    { 0x3d5d55d9, 33 },
    { 0, 0 },
    { 0xa66d55db, 93 },
    { 0xa1fd55dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed55e2, 23 },
    { 0xff0d55e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679d55eb, 98 },
    { 0x6a1d55ec, 43 },
    { 0, 0 },
    { 0xa10d55ee, 33 },
    { 0, 0 },
    { 0x701d55f0, 95 },
    { 0x590d55f1, 98 },
    { 0x7f6d55f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded55f6, 87 },
    { 0, 0 },
    { 0x203d55f8, 95 },
    { 0x7fed55f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ed5601, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493d5606, 80 },
    { 0xa45d5607, 90 },
    { 0, 0 },
    { 0x5b9d5609, 90 },
    { 0, 0 },
    { 0x62bd560b, 78 },
    { 0x937d560c, 98 },
    { 0x4cd560d, 46 },
    { 0x349d560d, 23 },
    { 0x464d560e, 95 },
    { 0, 0 },
    { 0xe79d5611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fad5614, 55 },
    { 0, 0 },
    { 0xad2d5616, 90 },
    { 0, 0 },
    { 0x633d5618, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103d561f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612d5623, 76 },
    { 0, 0 },
    { 0x710d5625, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64d5629, 23 },
    { 0xa51d562a, 98 },
    { 0xd29d562a, 24 },
    { 0x572d5629, 98 },
    { 0, 0 },
    { 0x5f9d562e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x400d5631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2d5635, 33 },
    { 0x48fd5635, 93 },
    { 0, 0 },
    { 0x5bed5638, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x393d563b, 33 },
    { 0x402d563c, 90 },
    { 0x110d563b, 91 },
    { 0xfffd563b, 98 },
    { 0xd8ed563f, 34 },
    { 0x364d5640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd5643, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3d5646, 75 },
    { 0, 0 },
    { 0x9bfd5648, 23 },
    { 0xee6d5648, 90 },
    { 0, 0 },
    { 0xbf4d564b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdd564e, 61 },
    { 0, 0 },
    { 0x8dbd5650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cd5657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bd565a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d5662, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535d5666, 97 },
    { 0, 0 },
    { 0x692d5668, 61 },
    { 0, 0 },
    { 0x82d566a, 90 },
    { 0, 0 },
    { 0x667d566c, 90 },
    { 0x229d566c, 23 },
    { 0xbdcd566d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bd5673, 90 },
    { 0x1b0d5673, 23 },
    { 0x6fad5675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73d567a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501d5680, 52 },
    { 0, 0 },
    { 0x930d5682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5d5685, 90 },
    { 0xa27d5686, 43 },
    { 0xa74d5687, 34 },
    { 0x1c8d5685, 95 },
    { 0xbf2d5689, 90 },
    { 0xbfbd568a, 52 },
    { 0x20ed5689, 69 },
    { 0xd29d5689, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252d5691, 76 },
    { 0, 0 },
    { 0x46cd5693, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d5699, 90 },
    { 0, 0 },
    { 0x820d569b, 90 },
    { 0x156d569c, 95 },
    { 0x888d569d, 46 },
    { 0x530d569e, 98 },
    { 0xa6fd569e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d56a7, 76 },
    { 0, 0 },
    { 0xb6bd56a9, 88 },
    { 0, 0 },
    { 0xdc9d56ab, 95 },
    { 0, 0 },
    { 0x84ad56ad, 33 },
    { 0x703d56ad, 98 },
    { 0, 0 },
    { 0xab1d56b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d56b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d56bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d56c3, 23 },
    { 0, 0 },
    { 0x982d56c5, 91 },
    { 0x956d56c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d56cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532d56da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73d56de, 61 },
    { 0x7e0d56de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d56e2, 100 },
    { 0xd98d56e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8d56e6, 91 },
    { 0, 0 },
    { 0xe87d56e8, 46 },
    { 0, 0 },
    { 0x25dd56ea, 92 },
    { 0, 0 },
    { 0xdbd56ec, 88 },
    { 0x13fd56ed, 23 },
    { 0, 0 },
    { 0x212d56ef, 85 },
    { 0x2ffd56f0, 98 },
    { 0xd54d56f1, 99 },
    { 0x72ad56f2, 23 },
    { 0x712d56f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadd56f7, 78 },
    { 0x9ced56f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded56fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x332d56ff, 55 },
    { 0x11dd56ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd5707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed570a, 95 },
    { 0x3c2d570b, 40 },
    { 0x8c8d570c, 61 },
    { 0, 0 },
    { 0xd11d570e, 61 },
    { 0x6a8d570e, 55 },
    { 0x7d6d5710, 96 },
    { 0, 0 },
    { 0x524d5712, 61 },
    { 0, 0 },
    { 0xe1ad5714, 23 },
    { 0x672d5715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d5718, 40 },
    { 0xb82d5719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd571c, 78 },
    { 0x8ded571c, 95 },
    { 0x60dd571c, 66 },
    { 0, 0 },
    { 0xfd5720, 61 },
    { 0xa6ed5721, 95 },
    { 0xe7fd5722, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed5726, 76 },
    { 0x9e4d5727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d572a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250d572f, 78 },
    { 0x762d5730, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3d573a, 33 },
    { 0x8fad573a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692d573f, 90 },
    { 0x533d5740, 87 },
    { 0xd93d5740, 93 },
    { 0x311d5740, 34 },
    { 0x1ed5741, 100 },
    { 0x786d5744, 90 },
    { 0x832d5741, 65 },
    { 0x5d8d5746, 78 },
    { 0x345d5743, 90 },
    { 0x871d5748, 46 },
    { 0xa71d5749, 88 },
    { 0x948d574a, 33 },
    { 0xc75d574b, 95 },
    { 0x205d5742, 33 },
    { 0x8f2d574a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8d5754, 95 },
    { 0x1cbd5754, 76 },
    { 0x757d5756, 91 },
    { 0x7fad5757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d5765, 76 },
    { 0, 0 },
    { 0x2a6d5767, 68 },
    { 0x48ad5768, 87 },
    { 0x86dd5769, 61 },
    { 0x68cd5768, 34 },
    { 0, 0 },
    { 0xd32d576c, 33 },
    { 0, 0 },
    { 0xfc5d576e, 90 },
    { 0x132d576f, 23 },
    { 0x9a4d5770, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56d5774, 52 },
    { 0xcead5775, 90 },
    { 0xc68d5776, 90 },
    { 0x13fd5777, 36 },
    { 0x407d5777, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92d577e, 75 },
    { 0xaa4d577f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d5782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dd5785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338d5789, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd578c, 90 },
    { 0xda3d578d, 55 },
    { 0x95ed578e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaed5793, 77 },
    { 0x715d5794, 61 },
    { 0x453d5795, 61 },
    { 0, 0 },
    { 0x961d5797, 23 },
    { 0x8add5798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd57a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d57a3, 23 },
    { 0, 0 },
    { 0x28ed57a5, 33 },
    { 0xb96d57a5, 40 },
    { 0xe58d57a5, 93 },
    { 0x243d57a6, 90 },
    { 0x19dd57a9, 76 },
    { 0, 0 },
    { 0xdb5d57ab, 90 },
    { 0x822d57ab, 33 },
    { 0, 0 },
    { 0xd17d57ae, 95 },
    { 0x1a2d57ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fd57b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d57b6, 33 },
    { 0x96ed57b7, 98 },
    { 0, 0 },
    { 0x9c9d57b9, 23 },
    { 0x18fd57b9, 95 },
    { 0x903d57b9, 87 },
    { 0x25dd57bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8d57bf, 44 },
    { 0x56ed57c0, 90 },
    { 0xe3ed57c1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x259d57c4, 81 },
    { 0x208d57c5, 61 },
    { 0, 0 },
    { 0xcdcd57c7, 85 },
    { 0, 0 },
    { 0xf76d57c9, 66 },
    { 0xed8d57ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fd57cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dd57d4, 33 },
    { 0, 0 },
    { 0xefed57d6, 66 },
    { 0xd81d57d7, 33 },
    { 0xdcfd57d7, 23 },
    { 0, 0 },
    { 0xbb9d57da, 93 },
    { 0x745d57db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2d57e0, 55 },
    { 0xd5d57e1, 43 },
    { 0xb42d57e1, 33 },
    { 0xd6dd57e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dd57e6, 100 },
    { 0x121d57e6, 65 },
    { 0, 0 },
    { 0x549d57e9, 90 },
    { 0, 0 },
    { 0xe44d57eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d57ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed57f1, 66 },
    { 0x6d0d57f1, 23 },
    { 0x809d57f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x969d57f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1d57fc, 85 },
    { 0xad1d57fd, 52 },
    { 0xa6d57fd, 75 },
    { 0x688d57ff, 81 },
    { 0x305d57ff, 99 },
    { 0x31dd57ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd5804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acd5807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559d580b, 76 },
    { 0, 0 },
    { 0x6dad580d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648d5813, 66 },
    { 0, 0 },
    { 0x6e0d5815, 61 },
    { 0, 0 },
    { 0xf5dd5817, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d581a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d5822, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d5825, 43 },
    { 0, 0 },
    { 0xf94d5827, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567d582c, 98 },
    { 0x581d582c, 2 },
    { 0, 0 },
    { 0x5cbd582f, 33 },
    { 0, 0 },
    { 0xeb1d5831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd5834, 33 },
    { 0x868d5835, 25 },
    { 0x2f3d5836, 55 },
    { 0, 0 },
    { 0x3bbd5838, 95 },
    { 0, 0 },
    { 0x904d583a, 76 },
    { 0x173d583a, 55 },
    { 0xc7ed583b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611d5840, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbd5848, 76 },
    { 0xeecd5849, 61 },
    { 0x95dd584a, 46 },
    { 0xa52d584b, 33 },
    { 0, 0 },
    { 0xcaad584d, 23 },
    { 0xfd7d584d, 90 },
    { 0, 0 },
    { 0x7aed5850, 98 },
    { 0x326d5851, 33 },
    { 0xbfd5852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd585c, 90 },
    { 0x8e4d585c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed5864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d586a, 64 },
    { 0, 0 },
    { 0xcafd586c, 77 },
    { 0xf7cd586c, 23 },
    { 0xb00d586e, 88 },
    { 0xea7d586e, 95 },
    { 0, 0 },
    { 0xc4d5871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdd5874, 98 },
    { 0xc8cd5875, 23 },
    { 0, 0 },
    { 0xc5fd5877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efd587c, 52 },
    { 0x85fd587d, 95 },
    { 0x2cd587d, 90 },
    { 0, 0 },
    { 0x2a1d5880, 95 },
    { 0x6e6d5881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03d5886, 23 },
    { 0x94cd5887, 90 },
    { 0xb25d5888, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd588d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d5890, 90 },
    { 0xcb5d5891, 90 },
    { 0x28d5890, 55 },
    { 0xe69d5893, 33 },
    { 0x39bd5893, 90 },
    { 0xf98d5895, 90 },
    { 0x57ed5890, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d5899, 91 },
    { 0x285d589a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d589d, 55 },
    { 0, 0 },
    { 0xe5fd589f, 33 },
    { 0x15fd58a0, 33 },
    { 0xcfed58a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d58a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07d58b0, 17 },
    { 0x658d58b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d58b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5d58b8, 90 },
    { 0x378d58b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d58c0, 46 },
    { 0, 0 },
    { 0x1f6d58c2, 100 },
    { 0x2f5d58c3, 78 },
    { 0, 0 },
    { 0x66fd58c5, 23 },
    { 0x6ffd58c6, 61 },
    { 0x966d58c7, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473d58cc, 95 },
    { 0x7bfd58cd, 98 },
    { 0xfc9d58ce, 95 },
    { 0xb18d58cf, 90 },
    { 0x433d58cc, 33 },
    { 0, 0 },
    { 0xb55d58d2, 23 },
    { 0x45bd58d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d58dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd58e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d58e4, 90 },
    { 0x93dd58e5, 98 },
    { 0, 0 },
    { 0x988d58e7, 95 },
    { 0xc63d58e7, 75 },
    { 0, 0 },
    { 0xdbdd58ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eed58ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad58f3, 33 },
    { 0, 0 },
    { 0x5ecd58f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7d58f9, 76 },
    { 0x878d58fa, 95 },
    { 0x490d58fb, 24 },
    { 0x78bd58fb, 88 },
    { 0, 0 },
    { 0x19fd58fe, 33 },
    { 0x335d58ff, 90 },
    { 0, 0 },
    { 0x740d5901, 55 },
    { 0x4edd5902, 55 },
    { 0x912d5903, 55 },
    { 0xb57d5904, 78 },
    { 0xf4bd5902, 33 },
    { 0, 0 },
    { 0x7ddd5907, 76 },
    { 0x6bed5907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77d590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d5910, 43 },
    { 0xb94d5910, 90 },
    { 0x3ddd5911, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3d5915, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcd591d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d5923, 4 },
    { 0xe8ad5924, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d5927, 46 },
    { 0x7cfd5928, 23 },
    { 0x441d5929, 95 },
    { 0xd2ad5929, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713d592f, 66 },
    { 0, 0 },
    { 0x3c9d5931, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d5936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17d593a, 93 },
    { 0x82ed593a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d593e, 76 },
    { 0, 0 },
    { 0xcddd5940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280d5945, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fd594b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391d594f, 99 },
    { 0xa59d5950, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3d5955, 33 },
    { 0, 0 },
    { 0x556d5957, 33 },
    { 0xe26d5958, 61 },
    { 0x198d5959, 95 },
    { 0, 0 },
    { 0x64dd595b, 99 },
    { 0x59dd595b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd5960, 99 },
    { 0x3efd5960, 23 },
    { 0x824d5962, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d5965, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d5969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd596e, 61 },
    { 0x3bdd596f, 76 },
    { 0, 0 },
    { 0xc47d5971, 99 },
    { 0x72dd5972, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dd5978, 93 },
    { 0, 0 },
    { 0x553d597a, 67 },
    { 0xf56d597b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d5980, 23 },
    { 0x3dad5981, 23 },
    { 0xd16d5982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d5987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfd598c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bd5991, 87 },
    { 0xf08d5992, 78 },
    { 0, 0 },
    { 0x838d5994, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d5997, 61 },
    { 0, 0 },
    { 0x96fd5999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacd59a0, 76 },
    { 0x732d59a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd59a8, 61 },
    { 0, 0 },
    { 0x5f9d59aa, 23 },
    { 0x18bd59ab, 100 },
    { 0xe8ed59ab, 34 },
    { 0x152d59ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cd59b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75d59bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdd59c5, 90 },
    { 0x9a2d59c5, 33 },
    { 0, 0 },
    { 0xa29d59c8, 43 },
    { 0, 0 },
    { 0xaa4d59ca, 98 },
    { 0, 0 },
    { 0x35cd59cc, 84 },
    { 0xdfdd59cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd59d7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88d59dc, 95 },
    { 0xb68d59dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d59e0, 61 },
    { 0xd2cd59e1, 76 },
    { 0, 0 },
    { 0xf5d59e3, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed59e6, 33 },
    { 0xd78d59e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cd59ec, 77 },
    { 0, 0 },
    { 0x704d59ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d59f6, 84 },
    { 0, 0 },
    { 0x51cd59f8, 61 },
    { 0xac9d59f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d59fe, 92 },
    { 0xa52d59ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd5a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd5a07, 99 },
    { 0, 0 },
    { 0xfd9d5a09, 43 },
    { 0, 0 },
    { 0x844d5a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d5a0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5d5a16, 77 },
    { 0xd6bd5a16, 51 },
    { 0xb09d5a18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed5a1c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5a21, 55 },
    { 0x84bd5a22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd5a27, 23 },
    { 0xf20d5a27, 43 },
    { 0x7f4d5a29, 90 },
    { 0xa50d5a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a2f, 23 },
    { 0, 0 },
    { 0x586d5a31, 98 },
    { 0, 0 },
    { 0x488d5a33, 78 },
    { 0, 0 },
    { 0x39cd5a35, 84 },
    { 0, 0 },
    { 0x7a3d5a37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d5a3a, 90 },
    { 0xa02d5a3b, 75 },
    { 0, 0 },
    { 0x5cdd5a3d, 87 },
    { 0x30d5a3d, 61 },
    { 0, 0 },
    { 0xa1d5a40, 91 },
    { 0xb47d5a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x419d5a44, 84 },
    { 0x1c6d5a45, 61 },
    { 0xdfed5a46, 95 },
    { 0x760d5a47, 98 },
    { 0x512d5a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abd5a4b, 55 },
    { 0xc1d5a4c, 95 },
    { 0x134d5a4d, 33 },
    { 0x4bad5a4e, 90 },
    { 0x71d5a4f, 75 },
    { 0x729d5a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dd5a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd5a58, 90 },
    { 0xf08d5a59, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfd5a60, 68 },
    { 0x902d5a60, 76 },
    { 0x1b9d5a61, 33 },
    { 0xfd6d5a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fd5a67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9d5a6c, 61 },
    { 0, 0 },
    { 0x8c4d5a6e, 90 },
    { 0x7ddd5a6f, 75 },
    { 0x917d5a6f, 90 },
    { 0x197d5a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8d5a75, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45d5a78, 33 },
    { 0x177d5a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd5a7c, 90 },
    { 0x4ffd5a7c, 76 },
    { 0, 0 },
    { 0x82bd5a7f, 61 },
    { 0xd80d5a7f, 98 },
    { 0xc30d5a81, 46 },
    { 0xf9d5a82, 91 },
    { 0xffcd5a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d5a8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d5a96, 61 },
    { 0x28ed5a97, 33 },
    { 0xc91d5a97, 61 },
    { 0, 0 },
    { 0xdd7d5a9a, 34 },
    { 0x43ed5a9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764d5aa0, 46 },
    { 0xaead5aa0, 80 },
    { 0, 0 },
    { 0xa06d5aa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5aa8, 61 },
    { 0xcf1d5aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d5aac, 75 },
    { 0x58ad5aac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbd5ab6, 99 },
    { 0, 0 },
    { 0x77bd5ab8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccd5abb, 98 },
    { 0xcf6d5abb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad5ac0, 95 },
    { 0x3c4d5ac1, 98 },
    { 0, 0 },
    { 0x4c7d5ac3, 98 },
    { 0xde5d5ac3, 95 },
    { 0xc8bd5ac5, 43 },
    { 0, 0 },
    { 0x1cdd5ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418d5ace, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bed5ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5ad6, 90 },
    { 0, 0 },
    { 0x8edd5ad8, 91 },
    { 0, 0 },
    { 0xf85d5ada, 88 },
    { 0, 0 },
    { 0x444d5adc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdd5ae3, 23 },
    { 0xa20d5ae4, 90 },
    { 0, 0 },
    { 0xc7ed5ae6, 98 },
    { 0xc34d5ae7, 98 },
    { 0x9bed5ae7, 46 },
    { 0x273d5ae7, 76 },
    { 0xce0d5aea, 33 },
    { 0x948d5aeb, 90 },
    { 0xb22d5aec, 85 },
    { 0x4d4d5aec, 95 },
    { 0xf80d5aee, 90 },
    { 0x9bad5aec, 95 },
    { 0xc76d5aec, 34 },
    { 0xe04d5af1, 95 },
    { 0, 0 },
    { 0x266d5af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd5af6, 90 },
    { 0x2e9d5af7, 98 },
    { 0x6d3d5af6, 33 },
    { 0x16dd5af9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d5afd, 55 },
    { 0, 0 },
    { 0xdd5aff, 39 },
    { 0x838d5b00, 76 },
    { 0x8c6d5b00, 55 },
    { 0x8e4d5aff, 25 },
    { 0x1b4d5b03, 61 },
    { 0, 0 },
    { 0x4cd5b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eed5b09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd5b0c, 23 },
    { 0, 0 },
    { 0x116d5b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ed5b19, 61 },
    { 0x997d5b19, 33 },
    { 0x387d5b19, 33 },
    { 0, 0 },
    { 0x2cad5b1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bd5b21, 96 },
    { 0, 0 },
    { 0x5cdd5b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacd5b27, 34 },
    { 0xe07d5b27, 95 },
    { 0xc2dd5b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d5b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bd5b36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbd5b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2d5b40, 99 },
    { 0x513d5b40, 91 },
    { 0xc88d5b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bd5b45, 90 },
    { 0x24dd5b45, 95 },
    { 0xc66d5b47, 81 },
    { 0xcb3d5b48, 33 },
    { 0xc13d5b46, 23 },
    { 0xfbdd5b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded5b4f, 90 },
    { 0x66ad5b4f, 99 },
    { 0x21dd5b50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9d5b54, 85 },
    { 0xaacd5b55, 90 },
    { 0xc57d5b55, 66 },
    { 0x561d5b55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecd5b5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d5b60, 78 },
    { 0x8fbd5b61, 61 },
    { 0x30ed5b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d5b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4d5b68, 76 },
    { 0x8efd5b69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d5b6c, 36 },
    { 0x247d5b6c, 61 },
    { 0xeb7d5b6e, 46 },
    { 0x40fd5b6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d5b72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d5b75, 76 },
    { 0x619d5b76, 95 },
    { 0xecfd5b77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0d5b7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x755d5b82, 33 },
    { 0x14bd5b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d5b8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dd5b93, 33 },
    { 0x47d5b94, 33 },
    { 0x5a4d5b95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d5b99, 90 },
    { 0xf3dd5b9a, 23 },
    { 0xd95d5b9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170d5ba6, 98 },
    { 0x3e9d5ba7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd5bab, 55 },
    { 0xbcbd5bab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d5bb1, 61 },
    { 0x221d5bb1, 95 },
    { 0, 0 },
    { 0xabdd5bb4, 81 },
    { 0xe89d5bb4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d5bbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4d5bc0, 99 },
    { 0xdcfd5bc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857d5bd2, 98 },
    { 0xaa8d5bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d5bd9, 88 },
    { 0xfbad5bda, 98 },
    { 0, 0 },
    { 0x550d5bdc, 85 },
    { 0, 0 },
    { 0x7abd5bde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d5be8, 93 },
    { 0, 0 },
    { 0xf4ad5bea, 33 },
    { 0, 0 },
    { 0xff1d5bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60d5bf2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678d5bf6, 95 },
    { 0x35cd5bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6d5bfa, 90 },
    { 0x9f7d5bfa, 78 },
    { 0xebdd5bfc, 34 },
    { 0x3e8d5bfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38d5c00, 23 },
    { 0x270d5c01, 100 },
    { 0xc52d5c02, 90 },
    { 0xc2d5c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad5c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734d5c09, 61 },
    { 0, 0 },
    { 0x813d5c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cd5c15, 95 },
    { 0x24cd5c15, 90 },
    { 0x294d5c15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d5c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add5c23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ad5c27, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad5c2f, 61 },
    { 0, 0 },
    { 0x2a2d5c31, 66 },
    { 0xb92d5c32, 61 },
    { 0, 0 },
    { 0x3f1d5c34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ad5c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dd5c3c, 61 },
    { 0x13dd5c3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd5c43, 23 },
    { 0, 0 },
    { 0xefdd5c45, 95 },
    { 0x3afd5c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbd5c49, 95 },
    { 0x137d5c49, 90 },
    { 0xf6cd5c4b, 23 },
    { 0x9e5d5c4c, 93 },
    { 0x42bd5c4d, 95 },
    { 0, 0 },
    { 0xe79d5c4f, 84 },
    { 0, 0 },
    { 0x93ad5c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75d5c55, 93 },
    { 0x4a5d5c55, 77 },
    { 0, 0 },
    { 0xd71d5c58, 76 },
    { 0x417d5c58, 90 },
    { 0, 0 },
    { 0x311d5c5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d5c60, 33 },
    { 0xd5ad5c60, 69 },
    { 0, 0 },
    { 0x13bd5c63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6d5c66, 55 },
    { 0, 0 },
    { 0x526d5c68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad5c72, 61 },
    { 0xfc4d5c73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d5c78, 23 },
    { 0x786d5c78, 52 },
    { 0xb79d5c7a, 55 },
    { 0, 0 },
    { 0x7e1d5c7c, 23 },
    { 0x8ead5c7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31d5c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d5c85, 55 },
    { 0xec3d5c86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfad5c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d5c90, 90 },
    { 0xcc2d5c91, 43 },
    { 0x68ad5c90, 98 },
    { 0xfc3d5c90, 98 },
    { 0x2d8d5c93, 98 },
    { 0, 0 },
    { 0xdf4d5c96, 96 },
    { 0xc3cd5c97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad5ca3, 95 },
    { 0xb04d5ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d5ca8, 39 },
    { 0x3ebd5ca9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd5cac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d5cb0, 98 },
    { 0xb84d5cb0, 95 },
    { 0xdafd5cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bd5cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477d5cbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fd5cc0, 90 },
    { 0xc91d5cc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44d5cc5, 76 },
    { 0x2b2d5cc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fd5ccb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x389d5cce, 95 },
    { 0x939d5ccf, 98 },
    { 0x969d5cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d5cd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd5cd8, 76 },
    { 0xde9d5cd9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdd5cdc, 46 },
    { 0x361d5cdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bed5ce0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d5ce9, 33 },
    { 0, 0 },
    { 0x775d5ceb, 84 },
    { 0x207d5cec, 88 },
    { 0x709d5ced, 78 },
    { 0xef6d5cec, 33 },
    { 0, 0 },
    { 0xc77d5cf0, 33 },
    { 0xcedd5cf1, 78 },
    { 0xc9cd5cf2, 43 },
    { 0xa74d5cf1, 90 },
    { 0xaaad5cf4, 61 },
    { 0x4aad5cf3, 23 },
    { 0x3d2d5cf1, 90 },
    { 0xcc5d5cf3, 84 },
    { 0xf61d5cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69d5cfb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d5cff, 61 },
    { 0, 0 },
    { 0x95cd5d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cd5d04, 98 },
    { 0, 0 },
    { 0x391d5d06, 87 },
    { 0xc71d5d07, 61 },
    { 0x208d5d08, 76 },
    { 0, 0 },
    { 0x28ed5d0a, 95 },
    { 0x77ed5d0b, 90 },
    { 0, 0 },
    { 0x13dd5d0d, 33 },
    { 0x4edd5d0e, 76 },
    { 0x7eed5d0f, 95 },
    { 0, 0 },
    { 0xb3ed5d11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd5d1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebd5d23, 95 },
    { 0xed6d5d23, 95 },
    { 0x99fd5d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304d5d2c, 95 },
    { 0, 0 },
    { 0xad0d5d2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad5d31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d5d34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fd5d3c, 90 },
    { 0x8b4d5d3d, 23 },
    { 0x6d5d3d, 61 },
    { 0xf38d5d3f, 90 },
    { 0x6c1d5d40, 55 },
    { 0xf94d5d41, 87 },
    { 0x554d5d41, 90 },
    { 0xaf8d5d40, 34 },
    { 0x302d5d43, 87 },
    { 0xf84d5d45, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdd5d4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d5d51, 66 },
    { 0x9dfd5d51, 55 },
    { 0x6c9d5d53, 34 },
    { 0x797d5d54, 95 },
    { 0xb13d5d55, 66 },
    { 0x7ead5d54, 61 },
    { 0x841d5d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed5d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ed5d5e, 76 },
    { 0, 0 },
    { 0xfbed5d60, 99 },
    { 0x975d5d61, 76 },
    { 0x134d5d61, 93 },
    { 0xf12d5d63, 46 },
    { 0xa23d5d61, 34 },
    { 0x4c3d5d61, 90 },
    { 0x141d5d66, 89 },
    { 0x6d3d5d67, 33 },
    { 0, 0 },
    { 0x798d5d69, 23 },
    { 0, 0 },
    { 0xeded5d6b, 46 },
    { 0x264d5d6c, 61 },
    { 0, 0 },
    { 0x2d1d5d6e, 33 },
    { 0x790d5d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544d5d73, 90 },
    { 0xa5fd5d74, 91 },
    { 0xd66d5d74, 46 },
    { 0x99fd5d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d5d7b, 61 },
    { 0xa3ed5d7c, 46 },
    { 0x1b1d5d7d, 43 },
    { 0x1ead5d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68d5d81, 61 },
    { 0xe27d5d82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd5d9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d5da0, 33 },
    { 0, 0 },
    { 0xf84d5da2, 98 },
    { 0, 0 },
    { 0x3c1d5da4, 76 },
    { 0, 0 },
    { 0x15bd5da6, 34 },
    { 0x99ed5da7, 40 },
    { 0xb8dd5da7, 33 },
    { 0x127d5da6, 98 },
    { 0x660d5daa, 43 },
    { 0, 0 },
    { 0x462d5dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76d5daf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d5db5, 66 },
    { 0xca4d5db6, 55 },
    { 0xedd5db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1d5dc2, 43 },
    { 0, 0 },
    { 0xd4dd5dc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fd5dc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0d5dca, 46 },
    { 0x78dd5dcb, 23 },
    { 0x819d5dcc, 43 },
    { 0x612d5dcc, 98 },
    { 0, 0 },
    { 0xacad5dcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cd5dd3, 95 },
    { 0x771d5dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9d5dd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d5ddb, 33 },
    { 0x346d5ddc, 33 },
    { 0, 0 },
    { 0x5c8d5dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d5de1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0d5dea, 23 },
    { 0, 0 },
    { 0x5b7d5dec, 98 },
    { 0x3b0d5ded, 99 },
    { 0xfefd5dee, 90 },
    { 0x90cd5dee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68d5df4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3d5dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x718d5e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ead5e04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845d5e09, 90 },
    { 0, 0 },
    { 0xa51d5e0b, 75 },
    { 0xa73d5e0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58d5e0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7d5e12, 34 },
    { 0x911d5e13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d5e19, 97 },
    { 0x35cd5e19, 68 },
    { 0xbced5e19, 84 },
    { 0, 0 },
    { 0x766d5e1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x139d5e20, 33 },
    { 0x48d5e21, 23 },
    { 0xa49d5e22, 77 },
    { 0x921d5e23, 46 },
    { 0x79ad5e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed5e28, 66 },
    { 0, 0 },
    { 0x5b2d5e2a, 85 },
    { 0, 0 },
    { 0x70ad5e2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23d5e30, 76 },
    { 0xf71d5e30, 61 },
    { 0, 0 },
    { 0x3f4d5e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eed5e3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06d5e42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d5e47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfd5e4d, 97 },
    { 0xfcd5e4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715d5e56, 61 },
    { 0, 0 },
    { 0xe40d5e58, 90 },
    { 0x712d5e58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279d5e61, 96 },
    { 0xee3d5e61, 43 },
    { 0x495d5e63, 95 },
    { 0x779d5e64, 78 },
    { 0xe6bd5e64, 15 },
    { 0x7cbd5e66, 90 },
    { 0xba6d5e65, 78 },
    { 0x481d5e62, 40 },
    { 0x54fd5e69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfd5e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81d5e6f, 77 },
    { 0, 0 },
    { 0x464d5e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0d5e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d5e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d5e8d, 55 },
    { 0x1fcd5e8d, 61 },
    { 0x376d5e8f, 33 },
    { 0xa1cd5e8f, 46 },
    { 0, 0 },
    { 0x9c8d5e92, 61 },
    { 0x985d5e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d5e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fd5e9c, 55 },
    { 0, 0 },
    { 0x2ed5e9e, 17 },
    { 0, 0 },
    { 0x85ad5ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bd5ea8, 34 },
    { 0, 0 },
    { 0x903d5eaa, 23 },
    { 0x9c3d5eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40d5eaf, 23 },
    { 0, 0 },
    { 0xcfcd5eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad5eb6, 61 },
    { 0, 0 },
    { 0xe07d5eb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cd5ebd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8d5ec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed5ec6, 61 },
    { 0xcc3d5ec7, 95 },
    { 0, 0 },
    { 0xbbbd5ec9, 95 },
    { 0xa0ed5ec9, 90 },
    { 0x776d5eca, 68 },
    { 0xf6cd5ecc, 99 },
    { 0xc6d5ecd, 93 },
    { 0, 0 },
    { 0xc62d5ecf, 43 },
    { 0x20cd5ed0, 90 },
    { 0, 0 },
    { 0x9e7d5ed2, 77 },
    { 0x94ed5ed3, 95 },
    { 0xed1d5ed4, 98 },
    { 0, 0 },
    { 0xe98d5ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d5eda, 33 },
    { 0, 0 },
    { 0xf24d5edc, 33 },
    { 0x9f1d5edd, 82 },
    { 0x2d2d5edd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d5ee2, 33 },
    { 0x2a2d5ee3, 95 },
    { 0xf33d5ee3, 33 },
    { 0, 0 },
    { 0xcaed5ee6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad5ef0, 87 },
    { 0, 0 },
    { 0x711d5ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fd5ef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x705d5efa, 76 },
    { 0x39d5efb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92d5eff, 98 },
    { 0xbf0d5eff, 90 },
    { 0x52cd5f01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3d5f06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd5f0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed5f11, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd5f14, 24 },
    { 0xe45d5f15, 90 },
    { 0x9a2d5f15, 23 },
    { 0x88ed5f17, 17 },
    { 0x8c4d5f18, 95 },
    { 0xa4cd5f19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20d5f1c, 34 },
    { 0xa4dd5f1c, 33 },
    { 0x971d5f1e, 85 },
    { 0x774d5f1f, 93 },
    { 0x5b8d5f20, 43 },
    { 0xc5ed5f21, 91 },
    { 0xd0bd5f22, 96 },
    { 0x4d5d5f1f, 91 },
    { 0xb9dd5f1d, 46 },
    { 0x2d2d5f1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d5f29, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95d5f30, 34 },
    { 0xac9d5f31, 98 },
    { 0x77fd5f30, 43 },
    { 0x5ead5f30, 33 },
    { 0, 0 },
    { 0x501d5f35, 23 },
    { 0xedd5f36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabd5f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d5f3d, 69 },
    { 0xaf8d5f3d, 95 },
    { 0, 0 },
    { 0xa29d5f40, 90 },
    { 0xbc3d5f41, 91 },
    { 0x2d2d5f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d5f46, 33 },
    { 0xef5d5f46, 61 },
    { 0xc6ad5f48, 61 },
    { 0, 0 },
    { 0xdd9d5f4a, 97 },
    { 0xdffd5f4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987d5f4f, 61 },
    { 0x775d5f50, 61 },
    { 0x676d5f50, 85 },
    { 0, 0 },
    { 0x18cd5f53, 33 },
    { 0, 0 },
    { 0x419d5f55, 81 },
    { 0xa9bd5f56, 90 },
    { 0xebcd5f57, 52 },
    { 0x459d5f57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786d5f5c, 99 },
    { 0, 0 },
    { 0xbdd5f5e, 95 },
    { 0, 0 },
    { 0x8a0d5f60, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed5f68, 33 },
    { 0x559d5f69, 34 },
    { 0, 0 },
    { 0x7a8d5f6b, 98 },
    { 0x919d5f6c, 98 },
    { 0x838d5f6d, 55 },
    { 0x7dcd5f6d, 23 },
    { 0xdfcd5f6e, 33 },
    { 0xfd1d5f6e, 87 },
    { 0x271d5f71, 95 },
    { 0x875d5f71, 90 },
    { 0x299d5f70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd5f77, 55 },
    { 0, 0 },
    { 0xc0bd5f79, 67 },
    { 0, 0 },
    { 0x961d5f7b, 81 },
    { 0, 0 },
    { 0xb95d5f7d, 76 },
    { 0, 0 },
    { 0x456d5f7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d5f83, 78 },
    { 0x830d5f83, 98 },
    { 0, 0 },
    { 0xc90d5f86, 33 },
    { 0xd8d5f87, 43 },
    { 0x607d5f88, 33 },
    { 0xdf0d5f88, 61 },
    { 0x301d5f88, 93 },
    { 0, 0 },
    { 0x3c9d5f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20d5f8f, 61 },
    { 0x2c1d5f90, 90 },
    { 0xfd9d5f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d5f95, 78 },
    { 0, 0 },
    { 0x29dd5f97, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d5f9c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38d5f9f, 98 },
    { 0, 0 },
    { 0xb8fd5fa1, 99 },
    { 0xad9d5fa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d5fa7, 33 },
    { 0, 0 },
    { 0xea5d5fa9, 77 },
    { 0xbe9d5fa9, 33 },
    { 0xd1cd5faa, 98 },
    { 0xe5cd5fac, 84 },
    { 0xa32d5fa9, 90 },
    { 0xbfed5faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fad5fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5fb6, 61 },
    { 0, 0 },
    { 0x76ad5fb8, 95 },
    { 0, 0 },
    { 0x891d5fba, 23 },
    { 0xc02d5fba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd5fbe, 95 },
    { 0x5a9d5fbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad5fc3, 52 },
    { 0x528d5fc3, 96 },
    { 0, 0 },
    { 0xb61d5fc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d5fc9, 98 },
    { 0xd4d5fca, 23 },
    { 0x34bd5fcb, 61 },
    { 0, 0 },
    { 0xb0dd5fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7d5fd6, 90 },
    { 0, 0 },
    { 0xe3d5fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed5fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d5fdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd5fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57d5fe8, 95 },
    { 0xa93d5fe9, 91 },
    { 0xe2dd5fe8, 66 },
    { 0x11dd5fe8, 90 },
    { 0x5c1d5fec, 61 },
    { 0xf49d5fec, 90 },
    { 0xce3d5fe9, 88 },
    { 0x248d5feb, 90 },
    { 0xcd3d5ff0, 33 },
    { 0x588d5ff1, 23 },
    { 0x16ad5ff0, 17 },
    { 0xf2ed5fe8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xced5ff6, 34 },
    { 0, 0 },
    { 0x14cd5ff8, 34 },
    { 0x35ed5ff8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d6001, 33 },
    { 0xae4d6002, 95 },
    { 0x7edd6003, 33 },
    { 0x584d6003, 23 },
    { 0xd96d6002, 36 },
    { 0xd93d6006, 46 },
    { 0xb40d6007, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d600c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddd6010, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37d6017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd601c, 55 },
    { 0xcaad601d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d6023, 99 },
    { 0xe10d6023, 93 },
    { 0, 0 },
    { 0x438d6026, 85 },
    { 0x2b5d6027, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbd6035, 94 },
    { 0x4c4d6036, 90 },
    { 0x70bd6037, 23 },
    { 0x65ad6038, 85 },
    { 0xba3d6036, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd603c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcd6040, 43 },
    { 0x73fd6041, 76 },
    { 0xf65d6040, 23 },
    { 0, 0 },
    { 0x94ed6044, 75 },
    { 0x92cd6045, 95 },
    { 0xd9bd6044, 98 },
    { 0, 0 },
    { 0x8f7d6048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d6052, 90 },
    { 0x300d6052, 69 },
    { 0x54fd6053, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d6057, 34 },
    { 0x47bd6057, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd605c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48d6064, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0d606c, 76 },
    { 0, 0 },
    { 0xf01d606e, 23 },
    { 0x7c5d606f, 90 },
    { 0x99cd606f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1d6078, 61 },
    { 0, 0 },
    { 0x9c1d607a, 87 },
    { 0xda7d607b, 76 },
    { 0x901d607c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd6085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924d6089, 93 },
    { 0, 0 },
    { 0xfd7d608b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70d6090, 40 },
    { 0x9bbd6091, 78 },
    { 0x7bad6092, 34 },
    { 0x5f6d6092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3d6098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d609b, 61 },
    { 0x552d609c, 98 },
    { 0x442d609d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd60a0, 61 },
    { 0x326d60a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262d60a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0d60aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d60b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdd60b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6d60ba, 33 },
    { 0x8a1d60bb, 76 },
    { 0xd81d60bc, 98 },
    { 0x1fcd60bc, 95 },
    { 0xe32d60be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9d60c2, 91 },
    { 0x863d60c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dd60c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34d60cd, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914d60d3, 88 },
    { 0, 0 },
    { 0x69d60d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x153d60d8, 34 },
    { 0xe93d60d9, 49 },
    { 0, 0 },
    { 0x350d60db, 98 },
    { 0xb63d60db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7d60e2, 99 },
    { 0xd36d60e3, 61 },
    { 0x2c3d60e4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2d60f6, 43 },
    { 0xaf6d60f7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d60fa, 76 },
    { 0xbc8d60fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565d6101, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd6104, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed610e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dd6113, 90 },
    { 0x479d6113, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392d6118, 90 },
    { 0x30ed6119, 85 },
    { 0, 0 },
    { 0xed9d611b, 23 },
    { 0x5d8d611c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278d6122, 91 },
    { 0x571d6122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed6126, 95 },
    { 0, 0 },
    { 0xaafd6128, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd6130, 23 },
    { 0xa74d6131, 90 },
    { 0xd97d6131, 93 },
    { 0x40cd6132, 98 },
    { 0xc0dd6131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502d6139, 61 },
    { 0, 0 },
    { 0xdebd613b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdd6143, 61 },
    { 0xb97d6143, 95 },
    { 0x2c1d6144, 82 },
    { 0x3e2d6146, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed614f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6d6152, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d6155, 52 },
    { 0, 0 },
    { 0xfe5d6157, 23 },
    { 0x512d6158, 33 },
    { 0x8cad6159, 33 },
    { 0x2f4d615a, 98 },
    { 0xdb2d615b, 78 },
    { 0x26cd615b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dd6160, 90 },
    { 0, 0 },
    { 0x405d6162, 100 },
    { 0xb8cd6162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d616d, 96 },
    { 0x193d616e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d6172, 90 },
    { 0, 0 },
    { 0xa8ad6174, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd6177, 77 },
    { 0x7a8d6178, 34 },
    { 0x308d6179, 91 },
    { 0x628d6178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d617d, 33 },
    { 0x4f6d617e, 34 },
    { 0x9b0d617f, 95 },
    { 0, 0 },
    { 0xb4ed6181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad6186, 23 },
    { 0x9fbd6187, 78 },
    { 0x712d6188, 90 },
    { 0, 0 },
    { 0xfe0d618a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d6191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485d6198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95d619b, 98 },
    { 0x231d619b, 88 },
    { 0x710d619d, 43 },
    { 0x1acd619e, 100 },
    { 0, 0 },
    { 0xc5ad61a0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aed61ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dd61af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd61b2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ed61be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0d61c1, 90 },
    { 0, 0 },
    { 0x9d8d61c3, 33 },
    { 0x5a3d61c3, 98 },
    { 0xa0cd61c5, 90 },
    { 0x2d5d61c4, 34 },
    { 0, 0 },
    { 0x189d61c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d61cb, 23 },
    { 0, 0 },
    { 0x6cad61cd, 66 },
    { 0, 0 },
    { 0x3fbd61cf, 55 },
    { 0x8d61cf, 23 },
    { 0xb51d61cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503d61d6, 76 },
    { 0x50ad61d7, 90 },
    { 0x148d61d8, 34 },
    { 0, 0 },
    { 0x98ad61da, 77 },
    { 0x95d61db, 61 },
    { 0xa13d61db, 76 },
    { 0x8d4d61da, 34 },
    { 0xb1ad61de, 34 },
    { 0xe25d61db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7d61e3, 52 },
    { 0xbaad61e4, 61 },
    { 0xaffd61e5, 78 },
    { 0xbf7d61e6, 90 },
    { 0, 0 },
    { 0x9e1d61e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d61f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add61f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d61f8, 55 },
    { 0, 0 },
    { 0x14cd61fa, 23 },
    { 0, 0 },
    { 0x6e6d61fc, 23 },
    { 0, 0 },
    { 0xeccd61fe, 90 },
    { 0xe65d61fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdd620f, 33 },
    { 0x8cbd6210, 99 },
    { 0x5a9d6211, 76 },
    { 0xc79d620f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad621a, 98 },
    { 0xe77d621b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d621f, 55 },
    { 0, 0 },
    { 0x143d6221, 95 },
    { 0xbad6221, 93 },
    { 0x407d6223, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd6229, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1d622d, 66 },
    { 0x8bfd622e, 77 },
    { 0xf51d622f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dd6238, 23 },
    { 0xd17d6238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0d6240, 55 },
    { 0x338d6241, 68 },
    { 0xd2d6241, 55 },
    { 0x3f5d6243, 23 },
    { 0xf54d6244, 88 },
    { 0xe54d6245, 55 },
    { 0x29ed6244, 95 },
    { 0x9afd6244, 84 },
    { 0x76fd6244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add624d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6d6250, 95 },
    { 0xaabd6251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5d6254, 91 },
    { 0xa5bd6255, 98 },
    { 0x9b7d6256, 61 },
    { 0x10cd6255, 98 },
    { 0xb8d6258, 61 },
    { 0xe7d6258, 91 },
    { 0x467d6256, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d625f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18d6263, 87 },
    { 0x8fbd6263, 66 },
    { 0x8ed6263, 61 },
    { 0x41fd6265, 99 },
    { 0x4dcd6264, 76 },
    { 0, 0 },
    { 0x406d6269, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd626e, 96 },
    { 0x875d626e, 93 },
    { 0, 0 },
    { 0xa2cd6271, 46 },
    { 0x8ecd6272, 95 },
    { 0xdddd6273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d6278, 84 },
    { 0xe9dd6278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d627e, 95 },
    { 0x540d627f, 87 },
    { 0xb72d6280, 67 },
    { 0x966d6281, 55 },
    { 0xd94d6280, 100 },
    { 0x9c2d6283, 90 },
    { 0, 0 },
    { 0x55bd6285, 98 },
    { 0x1c9d6286, 61 },
    { 0x531d6285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbd628c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7d628f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d6292, 61 },
    { 0x18ed6292, 94 },
    { 0xe2fd6293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaad629d, 91 },
    { 0xefbd629e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7d62a1, 95 },
    { 0xad9d62a2, 95 },
    { 0xd96d62a3, 23 },
    { 0x8ebd62a2, 95 },
    { 0x906d62a3, 90 },
    { 0x47ad62a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbd62ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9d62b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd62b4, 70 },
    { 0xb19d62b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8d62ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed62c2, 23 },
    { 0x5b4d62c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dd62c6, 43 },
    { 0xc38d62c6, 61 },
    { 0x5c4d62c8, 33 },
    { 0, 0 },
    { 0xf00d62ca, 90 },
    { 0x4ecd62ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9d62ce, 76 },
    { 0x702d62cf, 67 },
    { 0x8d0d62cf, 97 },
    { 0x956d62d1, 52 },
    { 0, 0 },
    { 0x59fd62d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd62d9, 33 },
    { 0x2e8d62da, 95 },
    { 0x599d62db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d62df, 78 },
    { 0x364d62df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73d62ea, 90 },
    { 0x46d62eb, 64 },
    { 0x1ecd62eb, 98 },
    { 0x952d62ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895d62f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d62fa, 23 },
    { 0xe4fd62fb, 91 },
    { 0, 0 },
    { 0x37ad62fd, 84 },
    { 0xe79d62fe, 33 },
    { 0, 0 },
    { 0x1e6d6300, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401d6306, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d6309, 46 },
    { 0x652d6309, 43 },
    { 0x1eed630b, 77 },
    { 0x7cfd6309, 33 },
    { 0x945d630d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cd6310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d6315, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d6318, 99 },
    { 0xc25d6319, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd631c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97d631f, 66 },
    { 0x500d631f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x373d6323, 33 },
    { 0x9fad6324, 43 },
    { 0, 0 },
    { 0xc3fd6326, 90 },
    { 0, 0 },
    { 0xead6328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d632c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d6330, 36 },
    { 0x704d6330, 23 },
    { 0x27cd6332, 61 },
    { 0, 0 },
    { 0xd82d6334, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fd633c, 76 },
    { 0, 0 },
    { 0x6bed633e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d6343, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d6347, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cd634a, 61 },
    { 0x39ad634a, 80 },
    { 0x667d634c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d634f, 98 },
    { 0x217d6350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d6356, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d635a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d635d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d6364, 98 },
    { 0, 0 },
    { 0x36cd6366, 87 },
    { 0, 0 },
    { 0x464d6368, 33 },
    { 0xfb7d6369, 66 },
    { 0x4f5d636a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd636e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5d6372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d6375, 95 },
    { 0xc89d6376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fd6379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d637c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ed637f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5d6382, 33 },
    { 0x3a7d6383, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2d6386, 78 },
    { 0xd6ad6387, 43 },
    { 0, 0 },
    { 0xb0fd6389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ad638c, 43 },
    { 0, 0 },
    { 0xd74d638e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccd6392, 67 },
    { 0, 0 },
    { 0xe78d6394, 88 },
    { 0xdd2d6395, 55 },
    { 0xc47d6394, 61 },
    { 0x1d4d6397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d639a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0d639e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d63a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7d63a9, 33 },
    { 0xf35d63aa, 90 },
    { 0xcafd63ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbd63b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d63b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67d63b6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x126d63b9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9d63bd, 91 },
    { 0, 0 },
    { 0x2b8d63bf, 23 },
    { 0x6ffd63c0, 98 },
    { 0, 0 },
    { 0xf11d63c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d63ca, 61 },
    { 0, 0 },
    { 0x49fd63cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d63cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d63d4, 23 },
    { 0x1f3d63d5, 95 },
    { 0, 0 },
    { 0xeb5d63d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6d63db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d63df, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7d63e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d63e9, 61 },
    { 0xbd1d63e9, 87 },
    { 0, 0 },
    { 0x7c7d63ec, 34 },
    { 0x82fd63ec, 95 },
    { 0x3e5d63ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611d63f3, 23 },
    { 0, 0 },
    { 0x357d63f5, 81 },
    { 0x4cdd63f5, 46 },
    { 0x837d63f7, 98 },
    { 0xb3ad63f8, 23 },
    { 0x90cd63f9, 68 },
    { 0x96d63f9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88d6406, 55 },
    { 0xdd5d6406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d640a, 90 },
    { 0, 0 },
    { 0x5d3d640c, 95 },
    { 0, 0 },
    { 0x19ed640e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffd6414, 52 },
    { 0x9a0d6415, 98 },
    { 0xb2dd6415, 23 },
    { 0x31dd6417, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd641a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeed641d, 23 },
    { 0x869d641e, 23 },
    { 0xb4d641e, 33 },
    { 0x93fd6420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d6423, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d6429, 93 },
    { 0xa8d6429, 75 },
    { 0x58dd642b, 34 },
    { 0x6b6d642c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8d642f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437d6436, 77 },
    { 0xf7cd6436, 23 },
    { 0x726d6437, 70 },
    { 0x33dd6439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d6442, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded6449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dd6450, 23 },
    { 0x248d6451, 61 },
    { 0x4efd6451, 90 },
    { 0xfc1d6451, 90 },
    { 0x48fd6450, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d645c, 61 },
    { 0, 0 },
    { 0xa29d645e, 33 },
    { 0x916d645f, 93 },
    { 0x591d6460, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd6463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d6468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d6470, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4d6473, 33 },
    { 0x3dd6474, 68 },
    { 0xeb0d6475, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bd6478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618d647c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fd6480, 52 },
    { 0, 0 },
    { 0x94ad6482, 23 },
    { 0xf66d6482, 61 },
    { 0, 0 },
    { 0x116d6485, 98 },
    { 0, 0 },
    { 0x24cd6487, 76 },
    { 0x6fbd6488, 90 },
    { 0x1b1d6488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed648c, 23 },
    { 0, 0 },
    { 0xfccd648e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d6492, 17 },
    { 0x3f0d6492, 17 },
    { 0xe7fd6494, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bd6499, 90 },
    { 0, 0 },
    { 0x48dd649b, 66 },
    { 0x7d9d649c, 61 },
    { 0xa5fd649d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d64a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66d64a6, 98 },
    { 0xa95d64a7, 77 },
    { 0, 0 },
    { 0x861d64a9, 95 },
    { 0x6e5d64aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9d64af, 47 },
    { 0xd5d64b0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6d64b3, 90 },
    { 0x7cd64b3, 23 },
    { 0x4f1d64b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ad64b9, 67 },
    { 0x884d64ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d64be, 78 },
    { 0xadd64bf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ed64c5, 43 },
    { 0x569d64c5, 23 },
    { 0xfbd64c5, 33 },
    { 0xc7cd64c8, 93 },
    { 0xcddd64c9, 33 },
    { 0x3bad64c8, 99 },
    { 0, 0 },
    { 0x85d64cc, 61 },
    { 0x32ad64cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0d64d2, 100 },
    { 0x30fd64d3, 76 },
    { 0xcd2d64d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x811d64d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4d64db, 98 },
    { 0, 0 },
    { 0x399d64dd, 95 },
    { 0xe8ad64dd, 98 },
    { 0x4c2d64dd, 33 },
    { 0xa75d64de, 95 },
    { 0xe44d64df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfd64e8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d64eb, 90 },
    { 0, 0 },
    { 0x60bd64ed, 67 },
    { 0, 0 },
    { 0x2bbd64ef, 67 },
    { 0x9ddd64ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d64f3, 90 },
    { 0x4cdd64f3, 95 },
    { 0, 0 },
    { 0xf20d64f6, 23 },
    { 0, 0 },
    { 0x789d64f8, 78 },
    { 0x6c4d64f9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x111d64fc, 23 },
    { 0x10fd64fd, 61 },
    { 0x3a3d64fe, 34 },
    { 0x967d64ff, 98 },
    { 0x600d64fe, 33 },
    { 0, 0 },
    { 0x579d6502, 95 },
    { 0xb48d6503, 90 },
    { 0, 0 },
    { 0x849d6505, 91 },
    { 0xac1d6506, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed6509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ad650f, 77 },
    { 0x503d650f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bd6516, 66 },
    { 0x93d6517, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd6521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fd652a, 95 },
    { 0xde5d652a, 95 },
    { 0x6a6d652c, 95 },
    { 0x272d652d, 77 },
    { 0x90cd652d, 40 },
    { 0xb64d652c, 61 },
    { 0x4f5d652e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d6533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62d6536, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dd653b, 66 },
    { 0x476d653c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4d6541, 76 },
    { 0x535d6541, 90 },
    { 0xc0d6543, 93 },
    { 0xc69d6542, 98 },
    { 0x5ffd6542, 23 },
    { 0xf1d6546, 95 },
    { 0x118d6546, 91 },
    { 0x403d6548, 98 },
    { 0xfd9d6546, 90 },
    { 0x2d1d654a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53d6551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150d6556, 90 },
    { 0, 0 },
    { 0x24dd6558, 98 },
    { 0, 0 },
    { 0x20bd655a, 83 },
    { 0xebbd655b, 61 },
    { 0x2a9d655b, 33 },
    { 0xfcbd655d, 55 },
    { 0x706d655c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5d6564, 52 },
    { 0x73fd6564, 33 },
    { 0xd9d6566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd6569, 61 },
    { 0xc81d656a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd656e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd6577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7d657a, 59 },
    { 0x8d8d657a, 81 },
    { 0x2d2d657b, 77 },
    { 0x4a3d657d, 23 },
    { 0x302d657a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13d6582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975d658a, 88 },
    { 0x837d658a, 98 },
    { 0xf27d658c, 34 },
    { 0xa3ad658d, 87 },
    { 0xd0fd658c, 67 },
    { 0xb78d658b, 61 },
    { 0xc4bd6590, 95 },
    { 0x9c6d6591, 31 },
    { 0x27d6592, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4d6596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d6599, 43 },
    { 0x3bfd6599, 95 },
    { 0xb4ed659b, 88 },
    { 0xa06d659c, 91 },
    { 0x5a7d659b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bad65a0, 90 },
    { 0x13bd65a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9d65a4, 61 },
    { 0xf30d65a4, 99 },
    { 0x817d65a6, 99 },
    { 0, 0 },
    { 0x5aed65a8, 40 },
    { 0x4dfd65a8, 55 },
    { 0, 0 },
    { 0xc37d65ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d65ae, 23 },
    { 0x1d4d65af, 98 },
    { 0x1a4d65ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ad65b4, 78 },
    { 0xcfdd65b5, 23 },
    { 0x865d65b6, 33 },
    { 0x8f3d65b7, 90 },
    { 0, 0 },
    { 0x876d65b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aad65bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382d65c2, 23 },
    { 0x470d65c3, 100 },
    { 0, 0 },
    { 0xad2d65c5, 88 },
    { 0x5cfd65c6, 98 },
    { 0x5d3d65c6, 61 },
    { 0x52bd65c8, 95 },
    { 0x210d65c8, 98 },
    { 0x8bbd65c5, 61 },
    { 0, 0 },
    { 0x747d65cc, 90 },
    { 0, 0 },
    { 0x9c7d65ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d65d5, 90 },
    { 0x290d65d6, 90 },
    { 0xdc9d65d6, 95 },
    { 0x7c2d65d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d65dd, 61 },
    { 0xb6fd65de, 90 },
    { 0, 0 },
    { 0xbbbd65e0, 61 },
    { 0x1a1d65e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7d65e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399d65eb, 95 },
    { 0x85bd65ec, 85 },
    { 0x34bd65ec, 75 },
    { 0x124d65ee, 91 },
    { 0xab7d65ef, 61 },
    { 0x958d65f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfd65f6, 67 },
    { 0xbbed65f7, 55 },
    { 0, 0 },
    { 0xbfad65f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d65fd, 46 },
    { 0xddd65fd, 33 },
    { 0x3d8d65ff, 90 },
    { 0x5ecd65ff, 95 },
    { 0xeb4d65fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937d6606, 52 },
    { 0, 0 },
    { 0x3dbd6608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6d660e, 61 },
    { 0xc9bd660e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d661a, 36 },
    { 0x211d661b, 34 },
    { 0x314d661a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcd6620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d6629, 55 },
    { 0, 0 },
    { 0x13bd662b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d6632, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726d6637, 34 },
    { 0, 0 },
    { 0x6a8d6639, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28d663d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d6640, 61 },
    { 0x4c1d6641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bd6648, 43 },
    { 0, 0 },
    { 0x575d664a, 101 },
    { 0, 0 },
    { 0x856d664c, 98 },
    { 0xa4bd664d, 95 },
    { 0x197d664c, 93 },
    { 0, 0 },
    { 0xad6650, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dd6653, 90 },
    { 0x6ebd6654, 76 },
    { 0x2b4d6653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ad6659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd665d, 100 },
    { 0xe3ed665e, 84 },
    { 0xdbfd665f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dd6663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dd666b, 84 },
    { 0x8d4d666c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd6671, 34 },
    { 0, 0 },
    { 0xd5d6673, 33 },
    { 0x1a4d6673, 68 },
    { 0x768d6675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd6679, 52 },
    { 0x499d6679, 43 },
    { 0, 0 },
    { 0x31cd667c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d667f, 64 },
    { 0, 0 },
    { 0x1bad6681, 23 },
    { 0xd8d6682, 34 },
    { 0x193d6683, 34 },
    { 0x7c4d6683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd6687, 76 },
    { 0, 0 },
    { 0x5acd6689, 61 },
    { 0x81d668a, 43 },
    { 0x364d6689, 77 },
    { 0x555d668c, 98 },
    { 0xfa5d668c, 43 },
    { 0xad7d6689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed6691, 90 },
    { 0xcc7d6691, 76 },
    { 0x2efd6693, 66 },
    { 0xc39d6694, 90 },
    { 0x524d6695, 17 },
    { 0x681d6695, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced669e, 90 },
    { 0xe53d669f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339d66a4, 55 },
    { 0x233d66a5, 61 },
    { 0xc94d66a6, 98 },
    { 0, 0 },
    { 0xb24d66a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d66ad, 46 },
    { 0, 0 },
    { 0x432d66af, 43 },
    { 0xb36d66b0, 95 },
    { 0, 0 },
    { 0x897d66b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d66b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecd66bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fd66c1, 55 },
    { 0x2a0d66c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d66c5, 66 },
    { 0, 0 },
    { 0x991d66c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d66ce, 23 },
    { 0x20bd66cf, 23 },
    { 0x137d66cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79d66d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98d66dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd66e2, 95 },
    { 0x76bd66e3, 84 },
    { 0x787d66e3, 84 },
    { 0, 0 },
    { 0x90bd66e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd66ec, 23 },
    { 0xd2ad66ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dad66f2, 88 },
    { 0xdf2d66f2, 23 },
    { 0xd69d66f4, 33 },
    { 0, 0 },
    { 0xa5d66f6, 66 },
    { 0x9b8d66f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d66fa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x226d66fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33d6702, 33 },
    { 0x7d4d6703, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d6709, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd670e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d6712, 61 },
    { 0x7ead6713, 34 },
    { 0x6e4d6712, 56 },
    { 0, 0 },
    { 0x112d6716, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d6719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0d671e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x141d6721, 61 },
    { 0, 0 },
    { 0x53cd6723, 55 },
    { 0xc64d6723, 23 },
    { 0x32dd6725, 61 },
    { 0x774d6726, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x313d6729, 87 },
    { 0, 0 },
    { 0xfced672b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdd672e, 90 },
    { 0x6add672e, 61 },
    { 0x3aed672f, 43 },
    { 0xb6d672f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09d6735, 61 },
    { 0, 0 },
    { 0x4bed6737, 90 },
    { 0x4acd6738, 17 },
    { 0xa03d6738, 87 },
    { 0xb33d6738, 24 },
    { 0x2d6737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952d673f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd674d, 98 },
    { 0xc7dd674e, 91 },
    { 0x3b1d674f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x756d6752, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8d6756, 52 },
    { 0xbe5d6757, 74 },
    { 0x1a3d6757, 85 },
    { 0, 0 },
    { 0xb54d675a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95d6769, 90 },
    { 0xa4ad676a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbd6770, 98 },
    { 0x613d6771, 90 },
    { 0x5b6d6771, 33 },
    { 0xadfd6771, 95 },
    { 0x38d6774, 55 },
    { 0, 0 },
    { 0x779d6776, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ed677d, 61 },
    { 0, 0 },
    { 0xb86d677f, 84 },
    { 0x455d6780, 98 },
    { 0, 0 },
    { 0xf3d6782, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d6788, 98 },
    { 0x9f0d6788, 76 },
    { 0x43ed678a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ad678d, 33 },
    { 0xb18d678e, 43 },
    { 0x8c4d678e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8d6792, 43 },
    { 0, 0 },
    { 0xa5ad6794, 46 },
    { 0x709d6795, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xecad6798, 98 },
    { 0, 0 },
    { 0xe48d679a, 75 },
    { 0xe9ed679b, 90 },
    { 0xabdd679b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ed679f, 95 },
    { 0, 0 },
    { 0x3a1d67a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d67a8, 76 },
    { 0x85fd67a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722d67af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5d67b3, 98 },
    { 0x217d67b4, 76 },
    { 0, 0 },
    { 0xb28d67b6, 90 },
    { 0, 0 },
    { 0xd7cd67b8, 17 },
    { 0xf66d67b9, 55 },
    { 0x178d67ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6d67bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d67c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04d67cc, 88 },
    { 0x95dd67cc, 61 },
    { 0x311d67ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d67d1, 33 },
    { 0x1a6d67d2, 33 },
    { 0, 0 },
    { 0xb4ed67d4, 83 },
    { 0x338d67d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2d67d8, 98 },
    { 0xdefd67d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ed67dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd67e5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d67e9, 61 },
    { 0x923d67ea, 91 },
    { 0x8b8d67eb, 46 },
    { 0xfc2d67ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ed67ef, 76 },
    { 0x4a3d67f0, 33 },
    { 0, 0 },
    { 0xce8d67f2, 84 },
    { 0x83ed67f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd67f7, 76 },
    { 0x46bd67f7, 90 },
    { 0, 0 },
    { 0x708d67fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304d67ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d6806, 78 },
    { 0x3fbd6807, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d680a, 90 },
    { 0xd4d680b, 53 },
    { 0xcded680b, 33 },
    { 0xf62d680d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d6812, 61 },
    { 0, 0 },
    { 0x9fd6814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d6819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1d681c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d681f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d6822, 55 },
    { 0xa56d6822, 95 },
    { 0, 0 },
    { 0xce4d6825, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ad682a, 90 },
    { 0x411d682b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d682f, 93 },
    { 0x820d682f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd6833, 84 },
    { 0x8fbd6833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790d6840, 90 },
    { 0, 0 },
    { 0x3d0d6842, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d6845, 93 },
    { 0x2b5d6846, 52 },
    { 0x3d3d6847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d684f, 33 },
    { 0x352d6850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ad6854, 33 },
    { 0x151d6854, 61 },
    { 0xad0d6854, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356d685a, 33 },
    { 0x956d685a, 33 },
    { 0, 0 },
    { 0xd66d685d, 98 },
    { 0xeb7d685d, 75 },
    { 0x676d685f, 55 },
    { 0x32ad685e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6863, 34 },
    { 0, 0 },
    { 0xc42d6865, 98 },
    { 0x455d6865, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cad686a, 23 },
    { 0x615d686a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d6874, 40 },
    { 0x511d6874, 61 },
    { 0x1fbd6876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd6879, 93 },
    { 0, 0 },
    { 0x2afd687b, 42 },
    { 0, 0 },
    { 0xb55d687d, 87 },
    { 0, 0 },
    { 0x139d687f, 90 },
    { 0x20ad687f, 76 },
    { 0xa26d6881, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d6887, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ed688e, 76 },
    { 0x6dcd688f, 100 },
    { 0x25dd688e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acd689b, 66 },
    { 0xa8ad689c, 46 },
    { 0, 0 },
    { 0x185d689e, 98 },
    { 0x771d689f, 33 },
    { 0x427d68a0, 33 },
    { 0xe9ed689f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8d68a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efd68a8, 33 },
    { 0xce7d68a8, 90 },
    { 0x40ad68aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d68b0, 61 },
    { 0x7dcd68b0, 23 },
    { 0xdd1d68b2, 98 },
    { 0x39dd68b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6d68ba, 92 },
    { 0, 0 },
    { 0x924d68bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d68c7, 48 },
    { 0, 0 },
    { 0x84bd68c9, 46 },
    { 0x6f2d68c9, 90 },
    { 0x82d68cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d68ce, 95 },
    { 0x429d68cf, 33 },
    { 0x79ed68d0, 98 },
    { 0, 0 },
    { 0x812d68d2, 33 },
    { 0, 0 },
    { 0x5bad68d4, 99 },
    { 0x9ddd68d4, 94 },
    { 0x264d68d5, 100 },
    { 0, 0 },
    { 0x60bd68d8, 17 },
    { 0x6f9d68d8, 90 },
    { 0xa14d68da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bd68de, 23 },
    { 0x77cd68de, 34 },
    { 0xb81d68e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d68e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd68eb, 61 },
    { 0x5b8d68eb, 43 },
    { 0, 0 },
    { 0x5e2d68ee, 95 },
    { 0, 0 },
    { 0xf69d68f0, 33 },
    { 0x50bd68f1, 23 },
    { 0xf8cd68f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152d68f6, 90 },
    { 0xcaed68f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d68fd, 84 },
    { 0xdf3d68fd, 43 },
    { 0x56ed68ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x655d6902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3d690d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cd6918, 90 },
    { 0xd75d6918, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d691f, 61 },
    { 0xc1d6920, 76 },
    { 0xb7ed6921, 66 },
    { 0, 0 },
    { 0x558d6923, 95 },
    { 0x136d6924, 33 },
    { 0x22fd6925, 93 },
    { 0x876d6925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78d6929, 61 },
    { 0xd79d6929, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d692d, 61 },
    { 0xa02d692d, 76 },
    { 0xd99d692f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d6932, 88 },
    { 0, 0 },
    { 0xd7cd6934, 90 },
    { 0x9e4d6935, 90 },
    { 0xdb1d6936, 55 },
    { 0xa9ad6935, 76 },
    { 0, 0 },
    { 0xfe9d6939, 98 },
    { 0xc60d693a, 43 },
    { 0xc8ad693a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ed693f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d694e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d6952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77d6955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd6958, 81 },
    { 0xdbfd6959, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dd6962, 33 },
    { 0xba8d6963, 61 },
    { 0xc13d6964, 95 },
    { 0x419d6965, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed696f, 23 },
    { 0x6b8d6970, 61 },
    { 0x8fd6970, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd6977, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fed697b, 66 },
    { 0x244d697c, 88 },
    { 0, 0 },
    { 0x277d697e, 23 },
    { 0, 0 },
    { 0x533d6980, 55 },
    { 0x4f3d6981, 23 },
    { 0xb38d6982, 67 },
    { 0xe2fd6983, 76 },
    { 0x3bcd6983, 43 },
    { 0x68d6985, 23 },
    { 0x453d6986, 33 },
    { 0xc5dd6983, 36 },
    { 0x789d6988, 33 },
    { 0, 0 },
    { 0xd0d698a, 61 },
    { 0xb75d698a, 61 },
    { 0xdcad698c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d6990, 61 },
    { 0, 0 },
    { 0xf17d6992, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d6996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcd699a, 78 },
    { 0, 0 },
    { 0x619d699c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90d699f, 33 },
    { 0x4d4d699f, 23 },
    { 0xa45d69a1, 46 },
    { 0, 0 },
    { 0xb4ed69a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed69a8, 90 },
    { 0x8ed69a8, 33 },
    { 0xb3dd69a9, 55 },
    { 0x20cd69ab, 93 },
    { 0x69fd69ab, 90 },
    { 0x7cbd69ad, 23 },
    { 0x24bd69ae, 2 },
    { 0xd0d69ae, 99 },
    { 0xb75d69b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167d69b4, 95 },
    { 0x128d69b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8d69b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622d69bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244d69c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2d69c7, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fd69cb, 90 },
    { 0xaccd69cc, 23 },
    { 0, 0 },
    { 0x287d69ce, 89 },
    { 0, 0 },
    { 0x40d69d0, 33 },
    { 0, 0 },
    { 0x6bad69d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad69d7, 78 },
    { 0xdd2d69d7, 55 },
    { 0x491d69d8, 90 },
    { 0xf6fd69da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd69de, 95 },
    { 0, 0 },
    { 0x210d69e0, 23 },
    { 0, 0 },
    { 0xcf5d69e2, 76 },
    { 0, 0 },
    { 0x656d69e4, 90 },
    { 0, 0 },
    { 0x166d69e6, 99 },
    { 0x8cfd69e7, 95 },
    { 0xf6d69e7, 23 },
    { 0, 0 },
    { 0x8afd69ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d69f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd69f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d69fa, 23 },
    { 0xce0d69fb, 33 },
    { 0x8ddd69fb, 67 },
    { 0, 0 },
    { 0xf5ed69fe, 52 },
    { 0x4b8d69ff, 99 },
    { 0, 0 },
    { 0x5fed6a01, 90 },
    { 0xcfcd6a02, 43 },
    { 0xcf1d6a02, 87 },
    { 0x96ed6a04, 90 },
    { 0xce0d6a05, 61 },
    { 0, 0 },
    { 0x3f0d6a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5d6a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cd6a0f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cd6a12, 76 },
    { 0, 0 },
    { 0x8a0d6a14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5d6a1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d6a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957d6a23, 84 },
    { 0x4e8d6a23, 33 },
    { 0, 0 },
    { 0x5dbd6a26, 100 },
    { 0x998d6a27, 34 },
    { 0x5ad6a28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d6a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6d6a31, 98 },
    { 0x386d6a32, 88 },
    { 0, 0 },
    { 0x68fd6a34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfad6a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74d6a3e, 33 },
    { 0, 0 },
    { 0x3afd6a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09d6a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd6a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23d6a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346d6a50, 98 },
    { 0x9e4d6a51, 81 },
    { 0x3d1d6a52, 95 },
    { 0x7e8d6a53, 90 },
    { 0x90bd6a51, 90 },
    { 0x58ed6a50, 55 },
    { 0xd4d6a56, 95 },
    { 0x419d6a55, 86 },
    { 0, 0 },
    { 0xbb5d6a59, 98 },
    { 0x4add6a59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x533d6a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8d6a66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d6a69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d6a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaad6a73, 95 },
    { 0xa1ed6a74, 78 },
    { 0x88dd6a75, 36 },
    { 0x94ad6a76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd6a80, 33 },
    { 0x224d6a80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3d6a84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d6a88, 90 },
    { 0x956d6a89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8d6a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d6a92, 75 },
    { 0x72ed6a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dd6a96, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d6a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d6aa0, 46 },
    { 0xb79d6aa1, 61 },
    { 0x32ad6aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ed6aa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x704d6aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dd6aac, 76 },
    { 0xd7ad6aad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d6ab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d6ab3, 84 },
    { 0, 0 },
    { 0xa39d6ab5, 76 },
    { 0x41fd6ab6, 33 },
    { 0, 0 },
    { 0x41d6ab8, 61 },
    { 0xd6d6ab9, 68 },
    { 0x216d6aba, 93 },
    { 0x42bd6ab8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ad6abe, 33 },
    { 0, 0 },
    { 0xa21d6ac0, 33 },
    { 0x9f9d6ac1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192d6ac9, 37 },
    { 0x2e8d6aca, 33 },
    { 0x9e4d6aca, 98 },
    { 0x190d6acc, 100 },
    { 0xd4dd6acd, 76 },
    { 0xba9d6ace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6d6ad1, 40 },
    { 0x2c7d6ad1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d6ad7, 90 },
    { 0x40ed6ad8, 76 },
    { 0x1e8d6ad9, 90 },
    { 0x8e9d6ada, 78 },
    { 0xe3ad6ad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd6ade, 76 },
    { 0xd09d6adf, 23 },
    { 0x42bd6ae0, 33 },
    { 0, 0 },
    { 0x66ed6ae2, 76 },
    { 0x4a8d6ae2, 4 },
    { 0x953d6ae3, 33 },
    { 0, 0 },
    { 0x551d6ae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23d6ae9, 66 },
    { 0x992d6ae9, 34 },
    { 0xf06d6aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad6aef, 55 },
    { 0, 0 },
    { 0xd97d6af1, 61 },
    { 0x9dbd6af2, 98 },
    { 0, 0 },
    { 0xe66d6af4, 76 },
    { 0, 0 },
    { 0xadcd6af6, 90 },
    { 0, 0 },
    { 0x6ecd6af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5d6afd, 61 },
    { 0x982d6afe, 90 },
    { 0xd02d6aff, 33 },
    { 0x5c3d6afe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740d6b06, 46 },
    { 0xf74d6b07, 95 },
    { 0x7aed6b08, 76 },
    { 0, 0 },
    { 0xdd0d6b0a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cd6b0d, 88 },
    { 0xdf3d6b0d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ad6b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d6b19, 66 },
    { 0x2c7d6b1a, 61 },
    { 0x81ad6b1b, 90 },
    { 0xdf4d6b19, 76 },
    { 0xed8d6b1d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7d6b20, 84 },
    { 0x9f1d6b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d6b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ed6b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aed6b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x139d6b32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd6b36, 33 },
    { 0x396d6b36, 76 },
    { 0xf85d6b38, 85 },
    { 0xef7d6b38, 80 },
    { 0x81ad6b38, 90 },
    { 0x2a1d6b3b, 40 },
    { 0x515d6b3c, 95 },
    { 0xc3bd6b3c, 43 },
    { 0xf0d6b3b, 61 },
    { 0x578d6b3f, 98 },
    { 0x868d6b40, 98 },
    { 0xc3ed6b41, 91 },
    { 0x9b8d6b40, 93 },
    { 0x6a6d6b3c, 33 },
    { 0x2fd6b40, 34 },
    { 0x2e5d6b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883d6b4a, 81 },
    { 0x78fd6b4b, 93 },
    { 0x781d6b4a, 33 },
    { 0xd04d6b4a, 91 },
    { 0x882d6b4e, 66 },
    { 0xeabd6b4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffd6b55, 98 },
    { 0xd27d6b55, 90 },
    { 0x59ad6b57, 90 },
    { 0x23d6b57, 43 },
    { 0xb65d6b59, 98 },
    { 0xa0ed6b5a, 94 },
    { 0xd7cd6b59, 23 },
    { 0x94cd6b5c, 98 },
    { 0xb58d6b5c, 33 },
    { 0x616d6b5e, 95 },
    { 0xb7cd6b5c, 90 },
    { 0x66bd6b5a, 98 },
    { 0x8fad6b61, 95 },
    { 0xeb7d6b5a, 90 },
    { 0x295d6b58, 99 },
    { 0xb01d6b60, 33 },
    { 0x73ed6b5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd6b6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848d6b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad6b7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d6b7f, 95 },
    { 0x9a1d6b7f, 55 },
    { 0, 0 },
    { 0x2aed6b82, 33 },
    { 0xf48d6b83, 66 },
    { 0xe35d6b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bed6b87, 33 },
    { 0, 0 },
    { 0x2e1d6b89, 23 },
    { 0xa86d6b89, 61 },
    { 0x8b8d6b8b, 90 },
    { 0, 0 },
    { 0xdd2d6b8d, 90 },
    { 0, 0 },
    { 0x3c6d6b8f, 23 },
    { 0xbead6b8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13d6b95, 84 },
    { 0x331d6b96, 90 },
    { 0x7a9d6b97, 98 },
    { 0x579d6b97, 81 },
    { 0x9a2d6b97, 98 },
    { 0, 0 },
    { 0x6f8d6b9b, 81 },
    { 0x61dd6b9b, 75 },
    { 0x651d6b9c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d6ba3, 61 },
    { 0x5a8d6ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d6baa, 90 },
    { 0xfcdd6baa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2d6bae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8d6bb4, 90 },
    { 0xbecd6bb4, 17 },
    { 0xd2fd6bb6, 61 },
    { 0, 0 },
    { 0xb3dd6bb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x102d6bbb, 98 },
    { 0x2edd6bbc, 67 },
    { 0x18bd6bbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafd6bc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d6bc4, 34 },
    { 0, 0 },
    { 0x62fd6bc6, 48 },
    { 0x4f7d6bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952d6bcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cd6bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd6bdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d6bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0d6be2, 95 },
    { 0, 0 },
    { 0x785d6be4, 23 },
    { 0, 0 },
    { 0x1bad6be6, 33 },
    { 0x958d6be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d6bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d6bf1, 90 },
    { 0x8d6bf1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cd6bf5, 90 },
    { 0xcfad6bf5, 61 },
    { 0, 0 },
    { 0xe35d6bf8, 85 },
    { 0, 0 },
    { 0xfafd6bfa, 33 },
    { 0, 0 },
    { 0xc10d6bfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x274d6bff, 46 },
    { 0x90fd6bff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d6c03, 75 },
    { 0x80ad6c03, 66 },
    { 0xcf9d6c05, 33 },
    { 0x629d6c05, 33 },
    { 0x5ded6c07, 90 },
    { 0xc84d6c08, 33 },
    { 0xa40d6c09, 84 },
    { 0x88fd6c09, 33 },
    { 0xe64d6c0b, 43 },
    { 0xf9dd6c08, 34 },
    { 0x423d6c08, 80 },
    { 0x496d6c0e, 95 },
    { 0x19dd6c0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dad6c19, 96 },
    { 0xaf0d6c1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbd6c1e, 76 },
    { 0xadcd6c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d6c22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cd6c30, 90 },
    { 0x4acd6c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d6c38, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ad6c3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d6c43, 90 },
    { 0, 0 },
    { 0xadbd6c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd6c4c, 33 },
    { 0, 0 },
    { 0x45cd6c4e, 61 },
    { 0, 0 },
    { 0x2b8d6c50, 90 },
    { 0, 0 },
    { 0x88ed6c52, 93 },
    { 0xa66d6c53, 43 },
    { 0x319d6c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dd6c57, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d6c5a, 90 },
    { 0xa88d6c5a, 98 },
    { 0x480d6c5c, 99 },
    { 0x5f1d6c5d, 77 },
    { 0x2ebd6c5d, 99 },
    { 0xfebd6c5a, 76 },
    { 0, 0 },
    { 0x6c6d6c61, 98 },
    { 0x804d6c62, 46 },
    { 0x510d6c63, 95 },
    { 0xb42d6c64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d6c67, 91 },
    { 0, 0 },
    { 0x3f7d6c69, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad6c6c, 98 },
    { 0xcedd6c6d, 61 },
    { 0x1ced6c6e, 33 },
    { 0xb27d6c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d6c77, 61 },
    { 0, 0 },
    { 0xeded6c79, 33 },
    { 0x8c7d6c7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05d6c7f, 91 },
    { 0x3c2d6c7f, 23 },
    { 0xc02d6c7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ed6c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4d6c88, 100 },
    { 0xb9cd6c89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cd6c8c, 91 },
    { 0xe16d6c8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd6c92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad6c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d6c9c, 90 },
    { 0xf87d6c9d, 33 },
    { 0xb2ed6c9c, 100 },
    { 0, 0 },
    { 0xb77d6ca0, 95 },
    { 0xe70d6ca1, 23 },
    { 0xf1dd6ca1, 90 },
    { 0xbbdd6ca3, 86 },
    { 0x4c5d6ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6cac, 23 },
    { 0x41fd6cac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad6cb4, 99 },
    { 0, 0 },
    { 0x4b5d6cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d6cbe, 23 },
    { 0xe00d6cbf, 90 },
    { 0xfe2d6cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d6cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bd6cc6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0d6ccb, 61 },
    { 0xb3dd6ccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37d6ccf, 98 },
    { 0xef4d6cd0, 33 },
    { 0x6ed6cd1, 46 },
    { 0x773d6cd2, 23 },
    { 0, 0 },
    { 0x630d6cd4, 99 },
    { 0x787d6cd4, 91 },
    { 0, 0 },
    { 0xce7d6cd7, 33 },
    { 0, 0 },
    { 0x4e4d6cd9, 61 },
    { 0x59ed6cd9, 33 },
    { 0x65fd6cdb, 94 },
    { 0, 0 },
    { 0xa63d6cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310d6ce2, 33 },
    { 0x1e3d6ce3, 95 },
    { 0x622d6ce4, 55 },
    { 0xebad6ce2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x578d6ce8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214d6ced, 95 },
    { 0xbe0d6ced, 98 },
    { 0xbbed6cef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x454d6cf2, 55 },
    { 0x295d6cf3, 95 },
    { 0x8bfd6cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72d6cfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67d6d00, 90 },
    { 0, 0 },
    { 0xd8d6d02, 95 },
    { 0xc5ed6d02, 90 },
    { 0x20ed6d04, 43 },
    { 0xc6bd6d05, 76 },
    { 0xb4bd6d06, 90 },
    { 0xe27d6d06, 99 },
    { 0x725d6d08, 23 },
    { 0xf94d6d09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12d6d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d6d1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6d24, 85 },
    { 0, 0 },
    { 0x28d6d26, 90 },
    { 0, 0 },
    { 0x23dd6d28, 38 },
    { 0x9e6d6d28, 34 },
    { 0, 0 },
    { 0x536d6d2b, 98 },
    { 0, 0 },
    { 0xa12d6d2d, 43 },
    { 0xec5d6d2e, 33 },
    { 0xe18d6d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd6d34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed6d37, 98 },
    { 0x394d6d38, 68 },
    { 0x50bd6d37, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd6d3c, 23 },
    { 0x15ad6d3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d6d41, 26 },
    { 0x220d6d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaded6d47, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd6d50, 98 },
    { 0, 0 },
    { 0xa47d6d52, 23 },
    { 0x8d4d6d52, 40 },
    { 0x89ad6d54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60d6d57, 33 },
    { 0x7b5d6d58, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dd6d5c, 84 },
    { 0, 0 },
    { 0x9a1d6d5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537d6d63, 76 },
    { 0x39d6d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994d6d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad6d71, 99 },
    { 0x4e3d6d72, 33 },
    { 0xdb1d6d72, 95 },
    { 0x935d6d74, 33 },
    { 0x9bad6d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x317d6d78, 34 },
    { 0, 0 },
    { 0xcbdd6d7a, 38 },
    { 0, 0 },
    { 0xf12d6d7c, 98 },
    { 0x465d6d7d, 33 },
    { 0xe2ad6d7d, 93 },
    { 0x648d6d7c, 33 },
    { 0, 0 },
    { 0xe14d6d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddd6d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0d6d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed6d8f, 23 },
    { 0, 0 },
    { 0x450d6d91, 87 },
    { 0, 0 },
    { 0xe59d6d93, 33 },
    { 0x37dd6d93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd6d98, 46 },
    { 0, 0 },
    { 0x9b1d6d9a, 43 },
    { 0x6c4d6d9b, 100 },
    { 0x7b6d6d9b, 90 },
    { 0xe38d6d9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d6da4, 38 },
    { 0xad7d6da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398d6dab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d6db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cd6db3, 78 },
    { 0x885d6db3, 95 },
    { 0x6b7d6db5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d6db8, 84 },
    { 0, 0 },
    { 0x491d6dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dd6dc0, 92 },
    { 0, 0 },
    { 0x822d6dc2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd6dc6, 67 },
    { 0xed2d6dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d6dcf, 90 },
    { 0, 0 },
    { 0x3ccd6dd1, 97 },
    { 0xf67d6dd1, 55 },
    { 0x43cd6dd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad6de1, 95 },
    { 0xb58d6de2, 58 },
    { 0xbbed6de2, 90 },
    { 0xb24d6de4, 43 },
    { 0xce6d6de5, 66 },
    { 0, 0 },
    { 0xcd3d6de7, 61 },
    { 0, 0 },
    { 0x26bd6de9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68d6df6, 98 },
    { 0xa72d6df6, 90 },
    { 0x597d6df8, 91 },
    { 0xe42d6df9, 91 },
    { 0x59d6df7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ad6dfe, 76 },
    { 0x879d6dff, 55 },
    { 0, 0 },
    { 0x46bd6e01, 53 },
    { 0xc06d6e01, 73 },
    { 0, 0 },
    { 0x847d6e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bd6e07, 55 },
    { 0xc19d6e08, 77 },
    { 0x311d6e09, 91 },
    { 0x10cd6e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235d6e11, 95 },
    { 0xf38d6e11, 93 },
    { 0x428d6e12, 33 },
    { 0x3bad6e14, 93 },
    { 0, 0 },
    { 0xbaad6e16, 78 },
    { 0, 0 },
    { 0x5d6e18, 90 },
    { 0, 0 },
    { 0x354d6e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x225d6e1d, 23 },
    { 0, 0 },
    { 0x1c5d6e1f, 34 },
    { 0, 0 },
    { 0x7edd6e21, 90 },
    { 0xc80d6e22, 67 },
    { 0x8dcd6e23, 99 },
    { 0xf54d6e23, 56 },
    { 0, 0 },
    { 0xc2fd6e26, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbd6e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d6e2f, 61 },
    { 0x583d6e2f, 96 },
    { 0x356d6e2f, 95 },
    { 0, 0 },
    { 0x4f3d6e33, 87 },
    { 0, 0 },
    { 0xe80d6e35, 90 },
    { 0, 0 },
    { 0x73dd6e37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd6e3d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bd6e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d6e45, 61 },
    { 0, 0 },
    { 0xf99d6e47, 80 },
    { 0x8a5d6e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56d6e4e, 34 },
    { 0xa17d6e4e, 33 },
    { 0x977d6e50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ad6e54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288d6e5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308d6e63, 23 },
    { 0x241d6e63, 90 },
    { 0xa78d6e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd6e6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd6e6e, 66 },
    { 0xf2fd6e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20d6e72, 81 },
    { 0xb58d6e72, 33 },
    { 0xa04d6e74, 68 },
    { 0x3ded6e75, 88 },
    { 0x93fd6e76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ad6e7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d6e81, 23 },
    { 0xf85d6e82, 61 },
    { 0x5e4d6e83, 90 },
    { 0xef7d6e84, 78 },
    { 0x5efd6e84, 34 },
    { 0x236d6e85, 23 },
    { 0x7b8d6e87, 38 },
    { 0x6b5d6e87, 90 },
    { 0, 0 },
    { 0xa8ad6e8a, 90 },
    { 0x1d4d6e8b, 95 },
    { 0x3fdd6e8c, 36 },
    { 0xb5ed6e8d, 76 },
    { 0x29ad6e8a, 90 },
    { 0x897d6e8a, 46 },
    { 0x579d6e8b, 34 },
    { 0, 0 },
    { 0x22ad6e92, 43 },
    { 0xb42d6e92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d6e9e, 84 },
    { 0, 0 },
    { 0x387d6ea0, 33 },
    { 0x913d6ea1, 33 },
    { 0, 0 },
    { 0x4b3d6ea3, 55 },
    { 0, 0 },
    { 0x5bad6ea5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d6ea8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffd6eab, 55 },
    { 0, 0 },
    { 0x9c3d6ead, 98 },
    { 0xa24d6eae, 23 },
    { 0x24fd6eae, 33 },
    { 0x17d6eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afd6eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd6ebc, 61 },
    { 0x667d6ebd, 34 },
    { 0x562d6ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8d6ec3, 95 },
    { 0x1d8d6ec3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd6ec7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbd6ecb, 95 },
    { 0x15d6ecc, 84 },
    { 0x255d6ecc, 90 },
    { 0xbf3d6ece, 90 },
    { 0xe5ad6ece, 17 },
    { 0, 0 },
    { 0xa52d6ed1, 100 },
    { 0, 0 },
    { 0xa39d6ed3, 77 },
    { 0xd5dd6ed3, 90 },
    { 0x7c9d6ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d6edd, 55 },
    { 0, 0 },
    { 0x935d6edf, 61 },
    { 0x138d6ee0, 91 },
    { 0xb8ed6ee1, 23 },
    { 0x2c3d6ee2, 64 },
    { 0x8dbd6ee3, 90 },
    { 0x941d6ee2, 52 },
    { 0x1e9d6ee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d6ee8, 43 },
    { 0, 0 },
    { 0xac8d6eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d6eed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d6ef1, 90 },
    { 0xba9d6ef2, 23 },
    { 0, 0 },
    { 0x54cd6ef4, 33 },
    { 0, 0 },
    { 0x60dd6ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d6efa, 90 },
    { 0x7e4d6efb, 95 },
    { 0x256d6efa, 33 },
    { 0, 0 },
    { 0xa9bd6efe, 94 },
    { 0x7dcd6eff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ed6f02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d6f0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fd6f18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d6f1f, 85 },
    { 0x6d3d6f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdd6f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5d6f27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f2b, 76 },
    { 0x366d6f2c, 61 },
    { 0x3e5d6f2d, 33 },
    { 0xb71d6f2e, 100 },
    { 0xb1cd6f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbd6f33, 90 },
    { 0x4f0d6f34, 76 },
    { 0xacdd6f35, 43 },
    { 0, 0 },
    { 0x94d6f37, 23 },
    { 0x7add6f38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d6f3f, 61 },
    { 0x8c5d6f40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d6f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7d6f47, 81 },
    { 0x23d6f47, 90 },
    { 0x1e4d6f49, 86 },
    { 0x1e7d6f4a, 90 },
    { 0xfdfd6f4b, 95 },
    { 0x98dd6f4c, 90 },
    { 0, 0 },
    { 0xfdbd6f4e, 93 },
    { 0x395d6f4e, 55 },
    { 0x781d6f50, 75 },
    { 0x24ed6f51, 90 },
    { 0, 0 },
    { 0xddad6f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d6f56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4d6f5e, 90 },
    { 0x2dfd6f5e, 34 },
    { 0, 0 },
    { 0x2aad6f61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3d6f65, 85 },
    { 0, 0 },
    { 0x3ed6f67, 23 },
    { 0xaa2d6f68, 61 },
    { 0, 0 },
    { 0xfa3d6f6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d6f72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d6f75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ad6f78, 90 },
    { 0x9dbd6f79, 33 },
    { 0x29ad6f79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713d6f7e, 34 },
    { 0xd30d6f7f, 33 },
    { 0x4ed6f80, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18d6f89, 34 },
    { 0xd23d6f8a, 46 },
    { 0x83d6f89, 61 },
    { 0x7ced6f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d6f92, 95 },
    { 0, 0 },
    { 0x7dbd6f94, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fd6f98, 95 },
    { 0xaf2d6f99, 61 },
    { 0, 0 },
    { 0x958d6f9b, 90 },
    { 0x13ed6f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d6fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fd6fa7, 87 },
    { 0xc87d6fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d6fab, 95 },
    { 0x2ecd6fac, 23 },
    { 0x456d6fad, 90 },
    { 0, 0 },
    { 0xd80d6faf, 95 },
    { 0, 0 },
    { 0xd94d6fb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ed6fb4, 34 },
    { 0x15d6fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fd6fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d6fbb, 92 },
    { 0xbfbd6fbb, 61 },
    { 0x379d6fbc, 33 },
    { 0x154d6fbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed6fc5, 98 },
    { 0, 0 },
    { 0x9fbd6fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9add6fcc, 98 },
    { 0x534d6fcd, 85 },
    { 0xf90d6fcd, 66 },
    { 0, 0 },
    { 0x3e2d6fd0, 87 },
    { 0, 0 },
    { 0xb1bd6fd2, 95 },
    { 0x56ad6fd2, 91 },
    { 0, 0 },
    { 0x773d6fd5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d6fde, 61 },
    { 0x290d6fdf, 95 },
    { 0x44d6fdf, 76 },
    { 0x12ad6fde, 99 },
    { 0x240d6fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dd6fe7, 85 },
    { 0x94ad6fe7, 55 },
    { 0, 0 },
    { 0x872d6fea, 84 },
    { 0x279d6feb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbd6fee, 23 },
    { 0x68fd6fef, 91 },
    { 0, 0 },
    { 0x5f4d6ff1, 33 },
    { 0x772d6ff2, 95 },
    { 0, 0 },
    { 0xe0fd6ff4, 74 },
    { 0xe35d6ff4, 99 },
    { 0x76dd6ff5, 90 },
    { 0x358d6ff5, 93 },
    { 0x15bd6ff8, 55 },
    { 0, 0 },
    { 0x7d1d6ffa, 61 },
    { 0x75cd6ffb, 17 },
    { 0xbb2d6ffc, 23 },
    { 0x879d6ffb, 64 },
    { 0x813d6ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d7006, 66 },
    { 0, 0 },
    { 0x824d7008, 34 },
    { 0x9afd7009, 90 },
    { 0, 0 },
    { 0x7edd700b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d700f, 95 },
    { 0xb85d7010, 17 },
    { 0xad2d7011, 76 },
    { 0x780d7012, 90 },
    { 0xfb8d7012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66d7019, 98 },
    { 0xeb6d701a, 66 },
    { 0xef9d701a, 98 },
    { 0xcddd7019, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ad701f, 66 },
    { 0, 0 },
    { 0x994d7021, 94 },
    { 0x7fcd7021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cd7031, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6d7036, 95 },
    { 0xefcd7037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20d703d, 66 },
    { 0xa11d703e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcd7041, 95 },
    { 0x9c8d7042, 87 },
    { 0, 0 },
    { 0x640d7044, 90 },
    { 0xef0d7045, 61 },
    { 0, 0 },
    { 0x741d7047, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x495d704a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6d7051, 66 },
    { 0x9bbd7052, 33 },
    { 0x73ed7052, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad705b, 24 },
    { 0, 0 },
    { 0xaf5d705d, 34 },
    { 0, 0 },
    { 0x37cd705f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d7063, 34 },
    { 0, 0 },
    { 0x64d7065, 93 },
    { 0, 0 },
    { 0xa3bd7067, 95 },
    { 0xc45d7068, 55 },
    { 0, 0 },
    { 0x432d706a, 91 },
    { 0, 0 },
    { 0x81bd706c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd7071, 95 },
    { 0xd09d7072, 34 },
    { 0, 0 },
    { 0xa15d7074, 76 },
    { 0x606d7074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad707a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d707d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed7083, 91 },
    { 0xd86d7083, 98 },
    { 0, 0 },
    { 0xe02d7086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26d7092, 17 },
    { 0, 0 },
    { 0xc17d7094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228d7099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad709c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cd709f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd70a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19d70a6, 98 },
    { 0xc16d70a6, 33 },
    { 0xf7dd70a8, 99 },
    { 0x90fd70a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d70b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd70b3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ad70b8, 61 },
    { 0xb00d70b9, 55 },
    { 0xdf2d70ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d70c2, 33 },
    { 0, 0 },
    { 0x3e5d70c4, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d70c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad70cb, 88 },
    { 0x850d70cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed70cf, 90 },
    { 0x577d70d0, 95 },
    { 0, 0 },
    { 0xe7fd70d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d70d6, 33 },
    { 0xbf0d70d7, 90 },
    { 0x7ffd70d6, 81 },
    { 0xb2ed70d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d70dc, 90 },
    { 0x811d70dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbd70e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x532d70e3, 34 },
    { 0xd8cd70e4, 61 },
    { 0xaced70e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fd70ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d70f2, 23 },
    { 0, 0 },
    { 0x653d70f4, 95 },
    { 0x7aad70f4, 95 },
    { 0xe4bd70f6, 61 },
    { 0xc0ed70f6, 33 },
    { 0xb4d70f6, 8 },
    { 0, 0 },
    { 0x1cfd70fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d7100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd7107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d7111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd7116, 33 },
    { 0, 0 },
    { 0x72bd7118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cd711b, 85 },
    { 0xcb8d711c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd7126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd7129, 95 },
    { 0xbaed712a, 93 },
    { 0xbd712a, 52 },
    { 0xd81d712c, 52 },
    { 0xebad712d, 95 },
    { 0, 0 },
    { 0x2d712f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06d7132, 61 },
    { 0x4ebd7133, 84 },
    { 0, 0 },
    { 0x669d7135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebd713b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1d7141, 34 },
    { 0, 0 },
    { 0x649d7143, 90 },
    { 0x115d7144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x927d7147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d714c, 23 },
    { 0xc63d714d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ed7155, 91 },
    { 0xfedd7156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84d715d, 98 },
    { 0, 0 },
    { 0xe9d715f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad7166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d716a, 76 },
    { 0xca5d716b, 76 },
    { 0xe6dd716c, 98 },
    { 0x9e7d716d, 90 },
    { 0xbded716b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15d7174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d7177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dd717c, 90 },
    { 0, 0 },
    { 0x73d717e, 75 },
    { 0xc30d717e, 87 },
    { 0x572d717f, 23 },
    { 0x72cd7181, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d7185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9d718b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cd7190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dd7198, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d719b, 90 },
    { 0xf5fd719c, 33 },
    { 0x7f0d719c, 55 },
    { 0x6b5d719e, 91 },
    { 0xe3fd719f, 90 },
    { 0x17ed71a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0d71a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d71a9, 61 },
    { 0x4b5d71aa, 55 },
    { 0, 0 },
    { 0xba5d71ac, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7d71b0, 98 },
    { 0x19cd71b0, 88 },
    { 0x55dd71b1, 67 },
    { 0x38dd71b3, 55 },
    { 0xe32d71b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd71ba, 95 },
    { 0x2d6d71ba, 95 },
    { 0xf55d71bc, 95 },
    { 0, 0 },
    { 0x4fbd71be, 90 },
    { 0xbd4d71be, 90 },
    { 0x327d71c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd71c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x691d71ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ed71cd, 65 },
    { 0, 0 },
    { 0x272d71cf, 90 },
    { 0x1c3d71d0, 99 },
    { 0, 0 },
    { 0xc13d71d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d71d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd71d8, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d71db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267d71e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd71e3, 33 },
    { 0xb08d71e4, 23 },
    { 0x208d71e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d71e9, 23 },
    { 0xedcd71ea, 100 },
    { 0x60cd71eb, 76 },
    { 0x1dcd71ec, 90 },
    { 0x545d71ed, 43 },
    { 0x640d71ee, 61 },
    { 0x3d2d71ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d71f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78d71f5, 77 },
    { 0, 0 },
    { 0x6fd71f7, 38 },
    { 0xbd7d71f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfd71fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ed71fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301d7204, 77 },
    { 0xf7ed7205, 76 },
    { 0xf20d7206, 100 },
    { 0x52d7205, 43 },
    { 0xe59d7208, 90 },
    { 0x6f0d7209, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d720d, 95 },
    { 0xdb5d720e, 90 },
    { 0x76ad720d, 99 },
    { 0x817d720e, 23 },
    { 0x17bd7210, 33 },
    { 0x813d7212, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbd7215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9d7219, 46 },
    { 0xdeed721a, 46 },
    { 0xaafd721a, 90 },
    { 0xc9dd721b, 61 },
    { 0x543d721d, 84 },
    { 0xe93d721e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dd7222, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfd7225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d722a, 46 },
    { 0xf36d722a, 85 },
    { 0x4dad722c, 76 },
    { 0x3a5d722b, 23 },
    { 0xf0fd722e, 10 },
    { 0xb8ed722d, 43 },
    { 0x9a1d722e, 33 },
    { 0x1ebd722c, 84 },
    { 0xeb0d7232, 78 },
    { 0xc18d722e, 61 },
    { 0xd94d7234, 61 },
    { 0xeefd7234, 61 },
    { 0x8bad722e, 90 },
    { 0x888d7237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d723b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afd723e, 93 },
    { 0xfcd723e, 95 },
    { 0x6c0d723e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d7244, 43 },
    { 0xfcfd7244, 78 },
    { 0x261d7244, 55 },
    { 0x15ad7247, 95 },
    { 0x79d7246, 24 },
    { 0xda7d7249, 33 },
    { 0x516d7247, 55 },
    { 0x8c2d724b, 68 },
    { 0, 0 },
    { 0xaa4d724d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dad7250, 42 },
    { 0xe8cd7250, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5d7256, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced725b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518d725f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5d7262, 61 },
    { 0, 0 },
    { 0xa72d7264, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xebad7267, 33 },
    { 0xab8d7268, 90 },
    { 0x376d7268, 48 },
    { 0, 0 },
    { 0x92bd726b, 61 },
    { 0x63bd726c, 76 },
    { 0x77ed726c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d7271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d7275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1d7278, 93 },
    { 0x82dd7278, 75 },
    { 0, 0 },
    { 0x1acd727b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8d727e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd7281, 95 },
    { 0x16d7282, 95 },
    { 0x721d7283, 84 },
    { 0x596d7281, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2d7289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18d728c, 90 },
    { 0, 0 },
    { 0x842d728e, 95 },
    { 0x38dd728f, 95 },
    { 0x9c1d7290, 96 },
    { 0xc8ad7291, 33 },
    { 0x553d728f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92d7295, 61 },
    { 0x855d7296, 95 },
    { 0, 0 },
    { 0x370d7298, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01d729b, 61 },
    { 0, 0 },
    { 0x891d729d, 91 },
    { 0xa74d729e, 95 },
    { 0x946d729f, 66 },
    { 0x3ffd72a0, 93 },
    { 0, 0 },
    { 0x63cd72a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6d72a9, 36 },
    { 0xcfd72aa, 33 },
    { 0x5aad72ab, 93 },
    { 0xb91d72ab, 88 },
    { 0xe23d72ad, 95 },
    { 0x6d9d72ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7d72b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0d72b7, 33 },
    { 0, 0 },
    { 0x534d72b9, 53 },
    { 0xc55d72ba, 55 },
    { 0, 0 },
    { 0xcdcd72bc, 98 },
    { 0x781d72bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad72c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4d72c7, 80 },
    { 0, 0 },
    { 0xe11d72c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad72ce, 66 },
    { 0xb5ed72ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47d72d4, 98 },
    { 0xcb8d72d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d72d8, 40 },
    { 0x39dd72d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cd72e0, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dd72e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ed72e8, 99 },
    { 0, 0 },
    { 0xe5ad72ea, 90 },
    { 0xe6d72eb, 78 },
    { 0, 0 },
    { 0x550d72ed, 90 },
    { 0, 0 },
    { 0x64bd72ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9d72f3, 98 },
    { 0x4fad72f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad72f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ad72fc, 61 },
    { 0x87bd72fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d7303, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4d7307, 90 },
    { 0xe39d7308, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd730e, 87 },
    { 0, 0 },
    { 0xd07d7310, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd7314, 55 },
    { 0xf76d7314, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x865d7318, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadd731d, 2 },
    { 0x49ad731e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad7322, 43 },
    { 0x814d7323, 46 },
    { 0x917d7322, 61 },
    { 0xf15d7323, 97 },
    { 0xe39d7326, 75 },
    { 0, 0 },
    { 0xb8dd7328, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28d732c, 42 },
    { 0x35cd732d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d7332, 87 },
    { 0, 0 },
    { 0x245d7334, 100 },
    { 0x20d7335, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x391d7338, 61 },
    { 0x5e9d7339, 90 },
    { 0xf32d7338, 61 },
    { 0, 0 },
    { 0x6c2d733c, 76 },
    { 0, 0 },
    { 0x51ed733e, 33 },
    { 0x833d733f, 98 },
    { 0xfd3d7340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81d7346, 23 },
    { 0, 0 },
    { 0xb82d7348, 98 },
    { 0xadfd7348, 23 },
    { 0x152d7349, 98 },
    { 0, 0 },
    { 0x7a5d734c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878d7352, 33 },
    { 0, 0 },
    { 0x1c5d7354, 17 },
    { 0, 0 },
    { 0xa6bd7356, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d7359, 76 },
    { 0x6a5d7359, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2d735d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666d7364, 91 },
    { 0xde0d7365, 90 },
    { 0, 0 },
    { 0x3c9d7367, 98 },
    { 0, 0 },
    { 0xa5cd7369, 95 },
    { 0x551d736a, 64 },
    { 0xd16d736b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd736e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfd7371, 98 },
    { 0, 0 },
    { 0x5dbd7373, 76 },
    { 0x3f7d7373, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31d7380, 66 },
    { 0xeced7381, 75 },
    { 0, 0 },
    { 0x21cd7383, 90 },
    { 0x2bad7384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d7387, 100 },
    { 0x8b8d7387, 95 },
    { 0xc5cd7388, 90 },
    { 0xe5dd738a, 90 },
    { 0x1a2d738b, 55 },
    { 0x7f3d738c, 45 },
    { 0xe8cd738d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3d7391, 23 },
    { 0x68dd7392, 52 },
    { 0x28ed7392, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad7396, 33 },
    { 0xaaad7396, 90 },
    { 0x9dd7397, 95 },
    { 0x466d7399, 95 },
    { 0x6bdd739a, 90 },
    { 0x933d739b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd739f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75d73a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dd73a5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ad73aa, 61 },
    { 0x36dd73ab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8d73ae, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835d73b2, 78 },
    { 0, 0 },
    { 0xbbd73b4, 76 },
    { 0x8cad73b4, 23 },
    { 0xd26d73b6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcd73b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dd73c1, 84 },
    { 0x236d73c1, 61 },
    { 0x49d73c2, 90 },
    { 0, 0 },
    { 0x137d73c5, 61 },
    { 0x8e6d73c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d73ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291d73e4, 23 },
    { 0xa5cd73e5, 76 },
    { 0, 0 },
    { 0xbc8d73e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d73ea, 98 },
    { 0, 0 },
    { 0x4d3d73ec, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8d73ef, 69 },
    { 0x332d73f0, 90 },
    { 0, 0 },
    { 0x264d73f2, 43 },
    { 0x4e2d73f2, 52 },
    { 0x909d73f4, 77 },
    { 0x9a4d73f3, 33 },
    { 0, 0 },
    { 0x571d73f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342d73fd, 98 },
    { 0xd04d73fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6d7401, 33 },
    { 0x892d7402, 88 },
    { 0xbbed7402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71d7406, 33 },
    { 0, 0 },
    { 0x9f4d7408, 55 },
    { 0, 0 },
    { 0x40ed740a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd7410, 98 },
    { 0xa14d7411, 99 },
    { 0x91fd7411, 98 },
    { 0, 0 },
    { 0x6c3d7414, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0d7417, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6d741c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cd7420, 17 },
    { 0, 0 },
    { 0xd0bd7422, 98 },
    { 0, 0 },
    { 0x530d7424, 23 },
    { 0, 0 },
    { 0xe05d7426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd742c, 81 },
    { 0, 0 },
    { 0xbc4d742e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ed7437, 33 },
    { 0x9f9d7438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbd743b, 68 },
    { 0xcd5d743b, 33 },
    { 0, 0 },
    { 0x962d743e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741d7443, 90 },
    { 0x7fcd7444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557d7448, 33 },
    { 0x80ed7448, 95 },
    { 0x56ed744a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded7452, 95 },
    { 0xd62d7452, 76 },
    { 0xa89d7454, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d7457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d745b, 76 },
    { 0xe1fd745c, 33 },
    { 0xb58d745d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28d7461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39d7466, 98 },
    { 0, 0 },
    { 0xb0cd7468, 43 },
    { 0, 0 },
    { 0x51ad746a, 52 },
    { 0x991d746a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd7472, 95 },
    { 0xfeed7472, 66 },
    { 0x924d7474, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7d7477, 95 },
    { 0x956d7478, 33 },
    { 0x7d1d7479, 98 },
    { 0x893d7478, 95 },
    { 0x4e0d7477, 75 },
    { 0xc89d747c, 98 },
    { 0xb02d747d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d7483, 61 },
    { 0x3ffd7483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6d748d, 43 },
    { 0xbdcd748d, 90 },
    { 0xeb5d748f, 55 },
    { 0, 0 },
    { 0xa0ed7491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7495, 95 },
    { 0xd91d7496, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d7499, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dd749c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d749f, 90 },
    { 0, 0 },
    { 0x148d74a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5d74a5, 55 },
    { 0xd2bd74a6, 33 },
    { 0xee9d74a6, 61 },
    { 0, 0 },
    { 0x41fd74a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328d74ae, 90 },
    { 0xa30d74af, 40 },
    { 0x5b4d74b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01d74ba, 95 },
    { 0xbe8d74bb, 90 },
    { 0xed3d74ba, 23 },
    { 0xb27d74bd, 61 },
    { 0xac0d74be, 76 },
    { 0xe2d74bf, 61 },
    { 0x90dd74c0, 33 },
    { 0xae6d74c1, 61 },
    { 0xaa1d74c2, 95 },
    { 0xeccd74c3, 76 },
    { 0xf8d74c4, 95 },
    { 0xad5d74ba, 33 },
    { 0xcf6d74c4, 93 },
    { 0, 0 },
    { 0x12cd74c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x807d74cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362d74d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d74d3, 34 },
    { 0, 0 },
    { 0xb27d74d5, 33 },
    { 0, 0 },
    { 0x4ced74d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d74db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ad74e5, 87 },
    { 0, 0 },
    { 0x5e3d74e7, 43 },
    { 0, 0 },
    { 0xc65d74e9, 85 },
    { 0, 0 },
    { 0x469d74eb, 43 },
    { 0, 0 },
    { 0x1f6d74ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x250d74f0, 90 },
    { 0xa1ed74f0, 61 },
    { 0xbf9d74f2, 98 },
    { 0, 0 },
    { 0x988d74f4, 90 },
    { 0, 0 },
    { 0xa48d74f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dd74f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad7507, 79 },
    { 0xb7bd7508, 65 },
    { 0x1a8d7509, 98 },
    { 0xd86d7509, 33 },
    { 0x10d750b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x751d750e, 61 },
    { 0, 0 },
    { 0xeb5d7510, 61 },
    { 0, 0 },
    { 0x71ad7512, 78 },
    { 0, 0 },
    { 0x7d4d7514, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7d751d, 33 },
    { 0, 0 },
    { 0x4f1d751f, 98 },
    { 0, 0 },
    { 0x9b7d7521, 33 },
    { 0x2b5d7521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x865d7525, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd752e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d7531, 17 },
    { 0x241d7532, 61 },
    { 0x8ad7532, 33 },
    { 0xf48d7534, 23 },
    { 0, 0 },
    { 0xbadd7536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad753d, 98 },
    { 0x82bd753e, 76 },
    { 0, 0 },
    { 0xe50d7540, 88 },
    { 0x6efd7540, 90 },
    { 0, 0 },
    { 0xfa6d7543, 93 },
    { 0x780d7543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375d7548, 95 },
    { 0x93dd7548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd7550, 55 },
    { 0xbe0d7551, 55 },
    { 0, 0 },
    { 0x7d8d7553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79d755b, 78 },
    { 0x94fd755c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d7567, 90 },
    { 0, 0 },
    { 0x8acd7569, 55 },
    { 0xdc7d756a, 90 },
    { 0xdb4d756a, 95 },
    { 0xa0ad756b, 98 },
    { 0, 0 },
    { 0x16ed756e, 90 },
    { 0x388d756f, 67 },
    { 0xa44d7570, 97 },
    { 0x82fd7570, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0d7576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fd757a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4d757f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ed7585, 55 },
    { 0x895d7586, 52 },
    { 0x673d7587, 61 },
    { 0x1aad7588, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x803d758b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd758f, 33 },
    { 0x9efd7590, 55 },
    { 0xeb8d7590, 67 },
    { 0x13cd7592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dd7599, 55 },
    { 0x6c0d759a, 91 },
    { 0xda0d759a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcd759e, 61 },
    { 0, 0 },
    { 0x28d75a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0d75a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d75a9, 76 },
    { 0, 0 },
    { 0xefd75ab, 90 },
    { 0x837d75ac, 87 },
    { 0x2add75ad, 84 },
    { 0, 0 },
    { 0x663d75af, 33 },
    { 0, 0 },
    { 0xcedd75b1, 33 },
    { 0x708d75b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d75b6, 100 },
    { 0x48dd75b7, 33 },
    { 0xa0ad75b8, 95 },
    { 0, 0 },
    { 0x66cd75ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d75bd, 76 },
    { 0x967d75be, 93 },
    { 0, 0 },
    { 0x59ed75c0, 84 },
    { 0, 0 },
    { 0x5fbd75c2, 84 },
    { 0x65bd75c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ad75c6, 76 },
    { 0xc4d75c7, 55 },
    { 0xb03d75c8, 61 },
    { 0xafcd75c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd75cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d75d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d75db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d75e0, 90 },
    { 0x7f1d75e1, 55 },
    { 0xc1ad75e2, 33 },
    { 0x313d75e3, 98 },
    { 0x5bfd75e3, 95 },
    { 0xda6d75e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd75ea, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d75ee, 68 },
    { 0x4b3d75ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13d75f8, 61 },
    { 0x820d75f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d75fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d7607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d760e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175d7613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cd7617, 90 },
    { 0x433d7618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d7625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fd762a, 64 },
    { 0xfb7d762b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bd762e, 90 },
    { 0x8d6d762f, 23 },
    { 0, 0 },
    { 0xf14d7631, 55 },
    { 0, 0 },
    { 0xb4d7633, 98 },
    { 0, 0 },
    { 0x47ad7635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced7638, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd763e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d7642, 55 },
    { 0x515d7642, 34 },
    { 0xde3d7644, 98 },
    { 0x931d7642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d7648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd764d, 55 },
    { 0xc9fd764d, 84 },
    { 0, 0 },
    { 0x1efd7650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d7655, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd7658, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd765b, 61 },
    { 0x4ebd765c, 33 },
    { 0xe6bd765b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd7660, 87 },
    { 0x69ad7660, 90 },
    { 0xb57d7660, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x765d7665, 61 },
    { 0x1dd7666, 43 },
    { 0x9fed7665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd766e, 93 },
    { 0x9d8d766f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0d7675, 61 },
    { 0x308d7675, 34 },
    { 0xa8d7675, 61 },
    { 0xfb0d7678, 69 },
    { 0x40fd7675, 98 },
    { 0x303d767a, 33 },
    { 0xd05d7675, 90 },
    { 0, 0 },
    { 0x28ad767d, 33 },
    { 0xc8cd767e, 90 },
    { 0xdd2d767e, 33 },
    { 0x3d3d7680, 90 },
    { 0xaeed767f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead7684, 90 },
    { 0x3fbd7685, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd7688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcd7690, 78 },
    { 0xc81d7690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38d7694, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d7697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912d769c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dd76a0, 95 },
    { 0, 0 },
    { 0x33dd76a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9d76a6, 68 },
    { 0, 0 },
    { 0xd41d76a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0d76b5, 77 },
    { 0x910d76b6, 33 },
    { 0x369d76b7, 61 },
    { 0x7c8d76b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dd76bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dd76c1, 46 },
    { 0xcd2d76c2, 68 },
    { 0xf57d76c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd76c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d76cf, 99 },
    { 0x967d76d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdced76d5, 64 },
    { 0xd9bd76d6, 66 },
    { 0x322d76d7, 98 },
    { 0, 0 },
    { 0x819d76d9, 93 },
    { 0xf23d76da, 76 },
    { 0xe69d76d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d76de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d76e1, 84 },
    { 0x254d76e2, 87 },
    { 0x16d76e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425d76ed, 33 },
    { 0, 0 },
    { 0xd20d76ef, 61 },
    { 0xf68d76f0, 98 },
    { 0, 0 },
    { 0xa7cd76f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bd76f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fd76fc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7700, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcd7704, 73 },
    { 0x8e7d7705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdd7709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d770d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bd7710, 77 },
    { 0x3b9d7711, 88 },
    { 0x537d7710, 95 },
    { 0, 0 },
    { 0x69dd7714, 67 },
    { 0xcf6d7715, 87 },
    { 0x5aad7714, 33 },
    { 0xbb5d7717, 23 },
    { 0x5ddd7717, 96 },
    { 0xca4d7719, 90 },
    { 0, 0 },
    { 0x975d771b, 33 },
    { 0x80dd771b, 34 },
    { 0, 0 },
    { 0xbe4d771e, 24 },
    { 0, 0 },
    { 0xe35d7720, 77 },
    { 0, 0 },
    { 0x4ed7722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cd7725, 23 },
    { 0x703d7726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead772a, 46 },
    { 0x17ad772a, 90 },
    { 0xba5d772c, 99 },
    { 0x82fd772d, 84 },
    { 0x63d772e, 94 },
    { 0x630d772f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2d7735, 61 },
    { 0, 0 },
    { 0xcbed7737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d7740, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x770d7747, 33 },
    { 0xc5fd7747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5d774c, 95 },
    { 0xcafd774d, 34 },
    { 0x7fbd774d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1d7751, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d7755, 78 },
    { 0xc6fd7755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x889d7759, 61 },
    { 0xfa6d775a, 98 },
    { 0xec9d775b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74d775f, 76 },
    { 0, 0 },
    { 0xc9ad7761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cd7766, 90 },
    { 0x99dd7766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed776b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d776f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cd7772, 23 },
    { 0xd32d7773, 88 },
    { 0x900d7772, 90 },
    { 0x3fad7774, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d7779, 77 },
    { 0x1fed777a, 66 },
    { 0xcb6d777b, 61 },
    { 0x27bd777c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed7781, 84 },
    { 0x18bd7782, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7d7785, 46 },
    { 0x4acd7785, 61 },
    { 0x56ed7785, 55 },
    { 0x394d7788, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfd778b, 61 },
    { 0xd5fd778c, 98 },
    { 0, 0 },
    { 0xd37d778e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d7795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edd779a, 76 },
    { 0x59d779b, 76 },
    { 0x25dd779a, 34 },
    { 0, 0 },
    { 0xfe4d779e, 61 },
    { 0xfbad779f, 95 },
    { 0x262d77a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d77a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38d77a8, 46 },
    { 0x335d77a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d77ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aad77b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1d77b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdd77ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cd77be, 76 },
    { 0xf9dd77bf, 93 },
    { 0, 0 },
    { 0x1a7d77c1, 76 },
    { 0x508d77c2, 90 },
    { 0x822d77c1, 93 },
    { 0xc2dd77c4, 23 },
    { 0xcb5d77c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3d77c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfead77cc, 98 },
    { 0xddad77cd, 91 },
    { 0xb85d77ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7d77d2, 93 },
    { 0x81fd77d3, 53 },
    { 0, 0 },
    { 0xe66d77d5, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d77db, 91 },
    { 0, 0 },
    { 0x858d77dd, 91 },
    { 0x2fad77de, 61 },
    { 0xbebd77df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d77e2, 93 },
    { 0, 0 },
    { 0x733d77e4, 55 },
    { 0xc4dd77e5, 90 },
    { 0xa6bd77e6, 90 },
    { 0x922d77e7, 78 },
    { 0x67bd77e5, 33 },
    { 0xb14d77e9, 90 },
    { 0x848d77ea, 75 },
    { 0xbb1d77eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23d77ee, 52 },
    { 0, 0 },
    { 0xa20d77f0, 55 },
    { 0x1edd77f1, 95 },
    { 0, 0 },
    { 0xbccd77f3, 90 },
    { 0x9b3d77f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9d77f7, 61 },
    { 0xad2d77f8, 2 },
    { 0x723d77f8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd77fd, 100 },
    { 0x97ed77fe, 46 },
    { 0x367d77fd, 99 },
    { 0x103d7800, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72d7804, 98 },
    { 0, 0 },
    { 0x122d7806, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd780b, 76 },
    { 0x502d780c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55d7812, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfd7817, 90 },
    { 0xb78d7818, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3d7820, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7823, 61 },
    { 0x76bd7824, 46 },
    { 0x25dd7825, 76 },
    { 0x5d9d7826, 76 },
    { 0, 0 },
    { 0x9c9d7828, 61 },
    { 0xef9d7829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d782e, 99 },
    { 0x619d782f, 61 },
    { 0xa06d782e, 81 },
    { 0xdf0d7831, 76 },
    { 0xa4cd7832, 81 },
    { 0x805d7833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7d7838, 38 },
    { 0x82dd7839, 43 },
    { 0x796d783a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abd783e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d7845, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd784b, 98 },
    { 0xf55d784b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1d7854, 90 },
    { 0xa60d7855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4d7859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d785c, 61 },
    { 0x7f0d785c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd7862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed7866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bd7869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d786e, 78 },
    { 0xf9fd786f, 95 },
    { 0, 0 },
    { 0x24fd7871, 33 },
    { 0xf15d7872, 33 },
    { 0xc74d7872, 55 },
    { 0x25d7873, 77 },
    { 0x6ffd7874, 75 },
    { 0xa63d7876, 98 },
    { 0x17cd7877, 76 },
    { 0, 0 },
    { 0xd9bd7879, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d787f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9d7883, 91 },
    { 0, 0 },
    { 0x81fd7885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4d788a, 93 },
    { 0x50cd788a, 64 },
    { 0, 0 },
    { 0x324d788d, 55 },
    { 0x6c8d788e, 80 },
    { 0, 0 },
    { 0xdb0d7890, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7894, 90 },
    { 0, 0 },
    { 0x4d0d7896, 55 },
    { 0, 0 },
    { 0xa20d7898, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d789b, 52 },
    { 0, 0 },
    { 0x2bdd789d, 23 },
    { 0x26cd789e, 90 },
    { 0, 0 },
    { 0xbd2d78a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd78a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaed78a8, 33 },
    { 0x54dd78a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd78ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ed78b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bd78b7, 100 },
    { 0, 0 },
    { 0x350d78b9, 61 },
    { 0, 0 },
    { 0x7bed78bb, 23 },
    { 0, 0 },
    { 0x1c2d78bd, 46 },
    { 0xeeed78bd, 33 },
    { 0xc62d78bf, 90 },
    { 0x8f0d78bd, 55 },
    { 0, 0 },
    { 0x7abd78c2, 86 },
    { 0x407d78c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623d78c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd78cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfd78d3, 76 },
    { 0, 0 },
    { 0xfddd78d5, 90 },
    { 0, 0 },
    { 0xf52d78d7, 90 },
    { 0, 0 },
    { 0xf43d78d9, 55 },
    { 0x7d7d78da, 55 },
    { 0, 0 },
    { 0xfadd78dc, 61 },
    { 0xbdbd78dd, 90 },
    { 0x544d78de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03d78e3, 33 },
    { 0x9e4d78e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad78e7, 75 },
    { 0, 0 },
    { 0xb03d78e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615d78ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad78f2, 23 },
    { 0x5ad78f3, 61 },
    { 0xa8ad78f3, 55 },
    { 0xfe4d78f5, 100 },
    { 0xcacd78f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bad78fa, 95 },
    { 0x953d78fa, 85 },
    { 0, 0 },
    { 0xfe4d78fd, 43 },
    { 0x60bd78fe, 66 },
    { 0xc70d78fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11d7903, 61 },
    { 0x342d7903, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d790a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d790d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abd7910, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd791d, 55 },
    { 0xc00d791d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecd7921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2d7928, 83 },
    { 0, 0 },
    { 0x66cd792a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6d792d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d7931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d7934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ed7937, 55 },
    { 0xfdfd7937, 43 },
    { 0, 0 },
    { 0xe6ad793a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd793e, 70 },
    { 0, 0 },
    { 0xfc1d7940, 34 },
    { 0x74ad7940, 61 },
    { 0x375d7942, 23 },
    { 0xeed7943, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316d7947, 23 },
    { 0, 0 },
    { 0x1e0d7949, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x598d794c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad7953, 95 },
    { 0xe3bd7953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d795c, 90 },
    { 0, 0 },
    { 0x565d795e, 55 },
    { 0, 0 },
    { 0x467d7960, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcd7964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cd7968, 65 },
    { 0x57ad7968, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1d796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d7971, 61 },
    { 0, 0 },
    { 0x6dcd7973, 64 },
    { 0xd8fd7974, 43 },
    { 0x5bad7975, 77 },
    { 0, 0 },
    { 0x22ed7977, 34 },
    { 0xff8d7978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd797c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ed7982, 95 },
    { 0x31ed7982, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34d7987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd798b, 46 },
    { 0, 0 },
    { 0x6f8d798d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ed7991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07d7994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cd799a, 90 },
    { 0, 0 },
    { 0x308d799c, 61 },
    { 0x86ed799d, 95 },
    { 0x17ed799e, 95 },
    { 0x982d799f, 61 },
    { 0x3ccd799d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d79a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d79a9, 69 },
    { 0xeddd79aa, 24 },
    { 0xa2ad79aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337d79af, 98 },
    { 0x683d79b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0d79b4, 90 },
    { 0xb81d79b5, 68 },
    { 0xccbd79b6, 98 },
    { 0, 0 },
    { 0x6f6d79b8, 97 },
    { 0x755d79b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd79c1, 93 },
    { 0xb55d79c1, 52 },
    { 0x1e2d79c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03d79c6, 95 },
    { 0xe38d79c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d79d0, 55 },
    { 0x84d79d1, 14 },
    { 0x9bed79d2, 34 },
    { 0x29ad79d3, 84 },
    { 0x3d9d79d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d79d8, 98 },
    { 0x226d79d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05d79dc, 81 },
    { 0xa1ed79dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fd79e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfd79e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826d79ed, 34 },
    { 0xbced79ee, 90 },
    { 0, 0 },
    { 0x672d79f0, 90 },
    { 0x249d79f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d79f8, 34 },
    { 0, 0 },
    { 0xa06d79fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d79ff, 33 },
    { 0xebbd7a00, 61 },
    { 0x216d7a01, 90 },
    { 0x8ed7a02, 91 },
    { 0x1d7d7a01, 61 },
    { 0, 0 },
    { 0x906d7a05, 98 },
    { 0x93d7a05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d7a09, 61 },
    { 0x8b5d7a0a, 87 },
    { 0x614d7a0b, 33 },
    { 0x3ead7a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd7a10, 100 },
    { 0, 0 },
    { 0x45cd7a12, 94 },
    { 0, 0 },
    { 0xa5d7a14, 90 },
    { 0x3aed7a15, 78 },
    { 0x52fd7a15, 38 },
    { 0, 0 },
    { 0x9acd7a18, 23 },
    { 0x3c1d7a19, 98 },
    { 0x895d7a18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d7a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d7a27, 61 },
    { 0xd9cd7a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d7a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4d7a33, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45d7a36, 33 },
    { 0x345d7a37, 90 },
    { 0x2e9d7a37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7a41, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad7a48, 46 },
    { 0, 0 },
    { 0x278d7a4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d7a4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcd7a53, 90 },
    { 0x99d7a54, 52 },
    { 0xc12d7a54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3d7a5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0d7a60, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd7a63, 98 },
    { 0, 0 },
    { 0xf15d7a65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d7a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4d7a6c, 93 },
    { 0xf65d7a6d, 23 },
    { 0xa73d7a6e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437d7a78, 33 },
    { 0, 0 },
    { 0x94fd7a7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94d7a7f, 95 },
    { 0xc62d7a80, 23 },
    { 0x74dd7a80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d7a84, 17 },
    { 0xb8dd7a85, 23 },
    { 0x136d7a84, 19 },
    { 0x3dfd7a84, 99 },
    { 0, 0 },
    { 0xffdd7a89, 76 },
    { 0xb8d7a89, 84 },
    { 0x4c5d7a8a, 88 },
    { 0xb3ed7a8c, 95 },
    { 0x4c8d7a8d, 65 },
    { 0xd65d7a8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edd7a94, 90 },
    { 0x75dd7a94, 33 },
    { 0, 0 },
    { 0x3efd7a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d7a9a, 90 },
    { 0x147d7a9a, 56 },
    { 0xd89d7a9a, 43 },
    { 0xbc3d7a9d, 76 },
    { 0xf8ed7a9b, 90 },
    { 0xcdfd7a9f, 90 },
    { 0x84d7a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38d7aa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308d7aa9, 76 },
    { 0x7ced7aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cd7ab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d7ab4, 87 },
    { 0xe28d7ab4, 95 },
    { 0x279d7ab6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fd7abd, 61 },
    { 0x808d7abe, 84 },
    { 0x4ded7abd, 67 },
    { 0xbc7d7abe, 33 },
    { 0, 0 },
    { 0x3b7d7ac2, 76 },
    { 0x8c0d7ac3, 66 },
    { 0xd6dd7ac3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d7ac8, 76 },
    { 0x57cd7ac8, 55 },
    { 0, 0 },
    { 0xae0d7acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad7ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d7ad3, 33 },
    { 0x4a8d7ad4, 23 },
    { 0x8edd7ad5, 95 },
    { 0xf8dd7ad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45d7ada, 95 },
    { 0x8d7adb, 95 },
    { 0xeb6d7adb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd7ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9d7aeb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6d7aee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bd7af1, 60 },
    { 0x73dd7af2, 95 },
    { 0x85ed7af3, 78 },
    { 0x12bd7af4, 98 },
    { 0x2d8d7af5, 76 },
    { 0xacd7af3, 23 },
    { 0x797d7af7, 98 },
    { 0x4dfd7af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8d7aff, 43 },
    { 0, 0 },
    { 0x1cd7b01, 98 },
    { 0x9e8d7b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x575d7b05, 33 },
    { 0x50dd7b06, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23d7b0c, 55 },
    { 0xc98d7b0d, 68 },
    { 0x40d7b0e, 95 },
    { 0xa0dd7b0f, 53 },
    { 0, 0 },
    { 0x756d7b11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdd7b14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d7b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed7b1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d7b1d, 90 },
    { 0xafcd7b1e, 95 },
    { 0x7f1d7b1e, 66 },
    { 0, 0 },
    { 0xe7bd7b21, 78 },
    { 0xf78d7b21, 77 },
    { 0x3dad7b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed7b27, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x410d7b2a, 98 },
    { 0x388d7b2b, 85 },
    { 0x151d7b2a, 95 },
    { 0x1b6d7b2d, 78 },
    { 0x195d7b2e, 33 },
    { 0x123d7b2c, 66 },
    { 0xf1ad7b2f, 100 },
    { 0xb6dd7b31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x854d7b34, 90 },
    { 0, 0 },
    { 0xf15d7b36, 34 },
    { 0x4abd7b37, 90 },
    { 0, 0 },
    { 0x3d5d7b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9d7b3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d7b42, 93 },
    { 0x4d1d7b43, 61 },
    { 0xfe6d7b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcd7b4c, 61 },
    { 0xf73d7b4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d7b53, 55 },
    { 0x775d7b53, 95 },
    { 0x2d4d7b55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add7b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8d7b5f, 95 },
    { 0x3a6d7b60, 93 },
    { 0xb41d7b61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d7b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293d7b69, 34 },
    { 0, 0 },
    { 0xd15d7b6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273d7b74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682d7b7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd7b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d7b87, 100 },
    { 0xe0ed7b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446d7b96, 95 },
    { 0, 0 },
    { 0x2aad7b98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd7b9b, 43 },
    { 0, 0 },
    { 0xc1dd7b9d, 33 },
    { 0x1c5d7b9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7d7ba5, 55 },
    { 0x291d7ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bd7baf, 85 },
    { 0xc16d7bb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d7bb7, 23 },
    { 0x17ad7bb8, 95 },
    { 0, 0 },
    { 0xc8cd7bba, 77 },
    { 0, 0 },
    { 0x453d7bbc, 76 },
    { 0xb4ed7bbd, 76 },
    { 0, 0 },
    { 0xe1bd7bbf, 66 },
    { 0, 0 },
    { 0xb69d7bc1, 61 },
    { 0xd14d7bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcd7bca, 33 },
    { 0x1d3d7bcb, 90 },
    { 0x881d7bcb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201d7bd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd7bd5, 52 },
    { 0, 0 },
    { 0xfdfd7bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed7bdc, 33 },
    { 0x9e4d7bdd, 90 },
    { 0, 0 },
    { 0x850d7bdf, 93 },
    { 0x3e9d7be0, 95 },
    { 0x159d7be0, 85 },
    { 0x532d7be2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8d7be5, 61 },
    { 0x587d7be5, 23 },
    { 0xab3d7be7, 77 },
    { 0, 0 },
    { 0xda1d7be9, 88 },
    { 0xb78d7be9, 61 },
    { 0xe77d7be9, 46 },
    { 0, 0 },
    { 0x753d7bed, 33 },
    { 0xd98d7bee, 91 },
    { 0, 0 },
    { 0xe63d7bf0, 101 },
    { 0, 0 },
    { 0xe5fd7bf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08d7bf5, 78 },
    { 0x8a1d7bf6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7d7bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcd7bfc, 90 },
    { 0, 0 },
    { 0x566d7bfe, 34 },
    { 0, 0 },
    { 0xdd6d7c00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38d7c03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d7c07, 61 },
    { 0xdccd7c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cd7c10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d7c13, 68 },
    { 0x128d7c14, 77 },
    { 0x2b1d7c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d7c19, 55 },
    { 0x81fd7c1a, 46 },
    { 0x7d0d7c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1d7c21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7c24, 98 },
    { 0x3dcd7c25, 34 },
    { 0x898d7c26, 52 },
    { 0, 0 },
    { 0xcf6d7c28, 98 },
    { 0xb38d7c29, 95 },
    { 0, 0 },
    { 0xa76d7c2b, 23 },
    { 0x108d7c2c, 33 },
    { 0xc1ad7c2c, 95 },
    { 0xf79d7c2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed7c32, 61 },
    { 0x74ad7c33, 52 },
    { 0x7a7d7c33, 55 },
    { 0x3c6d7c35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x856d7c38, 34 },
    { 0xc99d7c39, 52 },
    { 0, 0 },
    { 0x84ed7c3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bd7c42, 98 },
    { 0x7c4d7c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0d7c46, 76 },
    { 0xa4dd7c47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ed7c4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dd7c4f, 88 },
    { 0x602d7c50, 88 },
    { 0x673d7c51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643d7c5d, 76 },
    { 0, 0 },
    { 0x50fd7c5f, 23 },
    { 0xf31d7c5f, 61 },
    { 0xd5d7c61, 91 },
    { 0x45cd7c61, 61 },
    { 0x7abd7c62, 43 },
    { 0xe43d7c61, 55 },
    { 0x31d7c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d7c6d, 76 },
    { 0x34d7c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9d7c72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x242d7c75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cd7c78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d7c7d, 95 },
    { 0xd44d7c7e, 77 },
    { 0xcbcd7c7e, 95 },
    { 0, 0 },
    { 0x687d7c81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd7c8b, 98 },
    { 0, 0 },
    { 0xe5dd7c8d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd7c92, 76 },
    { 0xa71d7c92, 55 },
    { 0xa38d7c94, 95 },
    { 0, 0 },
    { 0x338d7c96, 76 },
    { 0, 0 },
    { 0xfb6d7c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d7c9b, 76 },
    { 0x748d7c9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d7ca1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad7ca6, 55 },
    { 0xf6dd7ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d7cae, 90 },
    { 0xb75d7caf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2d7cb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d7cbb, 61 },
    { 0x6b8d7cbc, 84 },
    { 0x239d7cbc, 34 },
    { 0x495d7cbe, 61 },
    { 0xd1dd7cbe, 76 },
    { 0x54d7cbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd7cc6, 23 },
    { 0x91fd7cc7, 55 },
    { 0x496d7cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0d7ccc, 77 },
    { 0xafad7ccc, 93 },
    { 0, 0 },
    { 0xe00d7ccf, 77 },
    { 0x2d2d7cd0, 52 },
    { 0x4dcd7cd1, 23 },
    { 0xc41d7cd0, 95 },
    { 0xb0cd7cd3, 61 },
    { 0x8ecd7cd1, 61 },
    { 0x7b6d7cd5, 98 },
    { 0x29cd7cd6, 34 },
    { 0x982d7cd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd7cde, 23 },
    { 0x4ffd7cde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d7ce2, 98 },
    { 0xf81d7ce3, 33 },
    { 0x49cd7ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d7ce8, 98 },
    { 0xad6d7ce9, 98 },
    { 0x5a3d7ce8, 95 },
    { 0x352d7ceb, 33 },
    { 0xc9d7cec, 90 },
    { 0xd08d7ced, 85 },
    { 0, 0 },
    { 0xbb7d7cef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ad7cf3, 91 },
    { 0x9d0d7cf4, 76 },
    { 0, 0 },
    { 0x73dd7cf6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d7cfa, 23 },
    { 0x964d7cfb, 90 },
    { 0xe3ed7cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd7cff, 100 },
    { 0xf96d7d00, 33 },
    { 0, 0 },
    { 0xaf6d7d02, 76 },
    { 0xa3fd7d02, 76 },
    { 0x645d7d02, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d7d07, 84 },
    { 0x6fbd7d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6d7d0b, 23 },
    { 0x96bd7d0c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d7d10, 69 },
    { 0, 0 },
    { 0x87cd7d12, 33 },
    { 0xfa4d7d13, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69d7d16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d7d19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d7d1d, 65 },
    { 0x2a0d7d1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeed7d21, 95 },
    { 0x613d7d22, 46 },
    { 0x2cfd7d22, 52 },
    { 0x96ed7d23, 33 },
    { 0x4c4d7d24, 61 },
    { 0x96ad7d26, 90 },
    { 0xe20d7d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced7d2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187d7d30, 75 },
    { 0x72dd7d31, 90 },
    { 0xe7bd7d32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dd7d43, 33 },
    { 0xc62d7d44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bad7d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ed7d4f, 90 },
    { 0x643d7d4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dd7d56, 98 },
    { 0, 0 },
    { 0xd90d7d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69d7d64, 52 },
    { 0x59bd7d64, 95 },
    { 0xa48d7d66, 61 },
    { 0xb61d7d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24d7d72, 90 },
    { 0x890d7d72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd7d78, 43 },
    { 0x26cd7d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149d7d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d7d88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d7d8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7d8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdad7d93, 84 },
    { 0x7cbd7d94, 33 },
    { 0xc2cd7d94, 61 },
    { 0xd22d7d96, 36 },
    { 0x270d7d94, 100 },
    { 0, 0 },
    { 0xdcdd7d99, 33 },
    { 0x6efd7d9a, 95 },
    { 0, 0 },
    { 0x8dd7d9c, 65 },
    { 0xc7dd7d9d, 33 },
    { 0xa0cd7d9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d7da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1d7da7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fd7dab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343d7daf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acd7db4, 20 },
    { 0x6e5d7db4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6d7dba, 96 },
    { 0x7ecd7dbb, 33 },
    { 0x22dd7dbc, 61 },
    { 0x44bd7dbd, 88 },
    { 0x94fd7dbe, 98 },
    { 0xa25d7dbf, 98 },
    { 0xf66d7dc0, 33 },
    { 0, 0 },
    { 0x417d7dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770d7dc6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ad7dc9, 98 },
    { 0, 0 },
    { 0xc8d7dcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd7dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248d7dd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cd7ddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7de2, 33 },
    { 0xfcfd7de3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x598d7de6, 90 },
    { 0x1c8d7de7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fd7dea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bd7df0, 93 },
    { 0xde1d7df1, 34 },
    { 0xe2bd7df1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8d7dfd, 61 },
    { 0, 0 },
    { 0xaed7dff, 33 },
    { 0xd06d7e00, 88 },
    { 0xb55d7e00, 23 },
    { 0, 0 },
    { 0x1c1d7e03, 61 },
    { 0xd83d7e04, 65 },
    { 0xe85d7e05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x335d7e08, 68 },
    { 0xdddd7e08, 33 },
    { 0x99fd7e08, 84 },
    { 0x392d7e09, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfd7e0e, 72 },
    { 0, 0 },
    { 0xf8d7e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9d7e14, 34 },
    { 0x841d7e15, 78 },
    { 0x831d7e15, 61 },
    { 0x9bfd7e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x546d7e1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed7e1e, 90 },
    { 0, 0 },
    { 0xedfd7e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1d7e24, 61 },
    { 0, 0 },
    { 0xbcbd7e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd7e29, 33 },
    { 0, 0 },
    { 0xc19d7e2b, 23 },
    { 0x345d7e2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ed7e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708d7e35, 33 },
    { 0x668d7e36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ad7e39, 98 },
    { 0, 0 },
    { 0xf16d7e3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64d7e3e, 76 },
    { 0, 0 },
    { 0x3ffd7e40, 76 },
    { 0, 0 },
    { 0x4d7d7e42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d7e45, 98 },
    { 0x767d7e46, 90 },
    { 0x332d7e45, 43 },
    { 0, 0 },
    { 0x8f3d7e49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0d7e52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d7e56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2d7e5a, 98 },
    { 0xc77d7e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d7e5e, 81 },
    { 0xfb4d7e5e, 98 },
    { 0xb4dd7e60, 95 },
    { 0xd4fd7e61, 91 },
    { 0x947d7e61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646d7e66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x290d7e69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d7e6c, 61 },
    { 0x65ad7e6d, 33 },
    { 0x5f3d7e6d, 34 },
    { 0x99cd7e6f, 23 },
    { 0xa38d7e70, 43 },
    { 0, 0 },
    { 0x86ad7e72, 95 },
    { 0x62bd7e73, 90 },
    { 0, 0 },
    { 0x477d7e75, 90 },
    { 0x210d7e76, 38 },
    { 0x695d7e75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88d7e7b, 61 },
    { 0, 0 },
    { 0xf46d7e7d, 34 },
    { 0x685d7e7e, 61 },
    { 0x55bd7e7f, 61 },
    { 0xdc0d7e80, 34 },
    { 0xae7d7e7f, 90 },
    { 0xb3bd7e82, 52 },
    { 0x36ed7e83, 98 },
    { 0xfe9d7e84, 90 },
    { 0xc2ed7e84, 76 },
    { 0xebfd7e86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad7e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43d7e9b, 61 },
    { 0xa30d7e9c, 90 },
    { 0xdcad7e9c, 61 },
    { 0x24ad7e9e, 91 },
    { 0x8cdd7e9d, 95 },
    { 0x9ded7ea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d7ea7, 76 },
    { 0x132d7ea8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7eab, 61 },
    { 0x9a6d7eac, 90 },
    { 0, 0 },
    { 0x7aad7eae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd7eb1, 33 },
    { 0x89d7eb2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd7eb5, 97 },
    { 0, 0 },
    { 0x491d7eb7, 78 },
    { 0xf39d7eb8, 33 },
    { 0x4a1d7eb9, 76 },
    { 0xbcfd7eba, 61 },
    { 0x9f0d7ebb, 90 },
    { 0x93ed7ebc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd7ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8d7ec5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d7ecb, 52 },
    { 0x77ad7ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d7ecf, 36 },
    { 0, 0 },
    { 0x2c4d7ed1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d7ed4, 26 },
    { 0xc6cd7ed5, 90 },
    { 0, 0 },
    { 0x668d7ed7, 98 },
    { 0xa73d7ed8, 23 },
    { 0, 0 },
    { 0xeed7eda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8d7edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcd7ee0, 90 },
    { 0, 0 },
    { 0x441d7ee2, 90 },
    { 0x1e4d7ee3, 33 },
    { 0xeefd7ee2, 33 },
    { 0x52d7ee3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dd7ef0, 23 },
    { 0, 0 },
    { 0xf03d7ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7ef9, 84 },
    { 0x89ed7ef9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd7efe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d7f0e, 90 },
    { 0, 0 },
    { 0x2bad7f10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1d7f14, 91 },
    { 0x79fd7f14, 76 },
    { 0xfb2d7f16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0d7f19, 33 },
    { 0x87d7f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d7f20, 33 },
    { 0x23cd7f21, 17 },
    { 0, 0 },
    { 0x6c8d7f23, 64 },
    { 0x6b1d7f23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fd7f27, 98 },
    { 0x128d7f27, 61 },
    { 0xe54d7f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71d7f30, 90 },
    { 0, 0 },
    { 0xf3d7f32, 90 },
    { 0, 0 },
    { 0xda0d7f34, 33 },
    { 0x5fed7f35, 33 },
    { 0xe88d7f36, 55 },
    { 0x67ad7f37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd7f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdd7f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d7f44, 34 },
    { 0, 0 },
    { 0x663d7f46, 61 },
    { 0x4c2d7f46, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dd7f4c, 23 },
    { 0, 0 },
    { 0xcbdd7f4e, 98 },
    { 0, 0 },
    { 0x807d7f50, 55 },
    { 0, 0 },
    { 0x500d7f52, 61 },
    { 0xbc5d7f53, 76 },
    { 0x88cd7f53, 34 },
    { 0, 0 },
    { 0xd34d7f56, 95 },
    { 0x136d7f57, 55 },
    { 0, 0 },
    { 0xa41d7f59, 93 },
    { 0, 0 },
    { 0xdc9d7f5b, 98 },
    { 0x834d7f5c, 23 },
    { 0, 0 },
    { 0x234d7f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d7f61, 23 },
    { 0x2dd7f62, 95 },
    { 0x933d7f61, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d7f6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0d7f74, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d7f7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd7f7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834d7f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d7f8c, 43 },
    { 0xa87d7f8d, 61 },
    { 0, 0 },
    { 0xe23d7f8f, 61 },
    { 0xc84d7f8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd7f94, 95 },
    { 0x4e5d7f94, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddd7f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ad7f9e, 90 },
    { 0, 0 },
    { 0x730d7fa0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3d7fa5, 77 },
    { 0, 0 },
    { 0x41cd7fa7, 98 },
    { 0, 0 },
    { 0x64dd7fa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbd7fad, 34 },
    { 0, 0 },
    { 0xdedd7faf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeead7fb6, 84 },
    { 0x7ccd7fb6, 34 },
    { 0x240d7fb7, 100 },
    { 0x59d7fb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d7fbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed7fc4, 66 },
    { 0x2fed7fc5, 33 },
    { 0xf09d7fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x783d7fc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x552d7fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bd7fd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d7fd7, 33 },
    { 0x5b1d7fd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed7fef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d7ff4, 38 },
    { 0xe01d7ff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x934d7ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bd7ffe, 91 },
    { 0xa26d7fff, 93 },
    { 0x270d8000, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d8005, 33 },
    { 0x747d8006, 84 },
    { 0xd28d8007, 69 },
    { 0, 0 },
    { 0x1bd8009, 85 },
    { 0xe02d800a, 76 },
    { 0x3c6d800a, 61 },
    { 0, 0 },
    { 0x938d800d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd8014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdd8017, 46 },
    { 0xa68d8017, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x562d801b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74d801e, 90 },
    { 0, 0 },
    { 0x43d8020, 52 },
    { 0xd5cd8021, 98 },
    { 0, 0 },
    { 0xa7d8023, 76 },
    { 0x455d8024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cd8028, 23 },
    { 0, 0 },
    { 0x714d802a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471d8030, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd803a, 98 },
    { 0x1eed803b, 90 },
    { 0x999d803b, 93 },
    { 0, 0 },
    { 0x72ed803e, 33 },
    { 0, 0 },
    { 0x500d8040, 46 },
    { 0, 0 },
    { 0x4cbd8042, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d8045, 95 },
    { 0, 0 },
    { 0xdefd8047, 90 },
    { 0x27d8047, 23 },
    { 0x7eed8049, 78 },
    { 0xb1fd8048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacad804d, 90 },
    { 0xa46d804d, 67 },
    { 0xe95d804e, 98 },
    { 0x96ed804f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ad8055, 90 },
    { 0xf2ad8055, 34 },
    { 0, 0 },
    { 0xe17d8058, 61 },
    { 0x348d8059, 36 },
    { 0x51ed8059, 66 },
    { 0, 0 },
    { 0xe9ad805c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d8062, 78 },
    { 0x7bad8063, 98 },
    { 0, 0 },
    { 0x9b4d8065, 23 },
    { 0x8f7d8066, 33 },
    { 0x4e6d8067, 90 },
    { 0x187d8066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d8071, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d8078, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd807b, 75 },
    { 0x308d807c, 90 },
    { 0xc5ad807c, 95 },
    { 0xa8d807e, 33 },
    { 0x9e3d807c, 95 },
    { 0, 0 },
    { 0x307d8081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7d8085, 78 },
    { 0xdc1d8086, 95 },
    { 0, 0 },
    { 0x854d8088, 95 },
    { 0xb8d8089, 23 },
    { 0x942d808a, 55 },
    { 0x8e7d808a, 23 },
    { 0x2c5d808a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfd8091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d80a2, 55 },
    { 0xd05d80a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dd80a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad80a9, 90 },
    { 0, 0 },
    { 0x293d80ab, 68 },
    { 0x69dd80ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x861d80af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880d80b3, 33 },
    { 0xc38d80b4, 67 },
    { 0x91bd80b5, 43 },
    { 0x7ccd80b5, 23 },
    { 0x740d80b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cd80bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d80c0, 77 },
    { 0, 0 },
    { 0x39ed80c2, 65 },
    { 0x4a4d80c3, 68 },
    { 0x61ed80c4, 23 },
    { 0, 0 },
    { 0xd66d80c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84d80cc, 65 },
    { 0xd7bd80cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fd80d9, 76 },
    { 0x3aad80d9, 23 },
    { 0, 0 },
    { 0x918d80dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3d80e2, 52 },
    { 0x9a2d80e3, 95 },
    { 0xaf2d80e4, 76 },
    { 0x5abd80e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad80e9, 90 },
    { 0, 0 },
    { 0xa9d80eb, 23 },
    { 0xed9d80ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d80f1, 66 },
    { 0x10fd80f1, 95 },
    { 0x45fd80f3, 43 },
    { 0x38bd80f4, 23 },
    { 0x94bd80f5, 52 },
    { 0x8a1d80f6, 61 },
    { 0x1ced80f4, 95 },
    { 0xda1d80f8, 90 },
    { 0xad4d80f2, 61 },
    { 0xa2ed80f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cd80fe, 76 },
    { 0, 0 },
    { 0x8dad8100, 91 },
    { 0xc61d8100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dd8111, 78 },
    { 0x71cd8112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x404d8115, 95 },
    { 0, 0 },
    { 0xf4cd8117, 23 },
    { 0, 0 },
    { 0xa75d8119, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd811c, 90 },
    { 0, 0 },
    { 0x252d811e, 33 },
    { 0, 0 },
    { 0xc4fd8120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d8125, 93 },
    { 0, 0 },
    { 0x697d8127, 33 },
    { 0, 0 },
    { 0x245d8129, 40 },
    { 0xcfd8129, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d812d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9d8131, 43 },
    { 0xb0cd8132, 23 },
    { 0x62ed8133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad8138, 23 },
    { 0x7d7d8139, 33 },
    { 0x5edd813a, 84 },
    { 0x650d8139, 90 },
    { 0xa79d813c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ad813f, 61 },
    { 0x927d8140, 90 },
    { 0x6a7d813f, 93 },
    { 0xb08d8142, 33 },
    { 0xf4dd813f, 33 },
    { 0xb32d8144, 91 },
    { 0x268d8144, 33 },
    { 0x9cbd8146, 99 },
    { 0x549d8141, 95 },
    { 0xc79d8148, 87 },
    { 0x9c8d8148, 85 },
    { 0xaa0d8143, 85 },
    { 0, 0 },
    { 0xd91d814c, 91 },
    { 0, 0 },
    { 0x7d2d814e, 33 },
    { 0x7a4d814f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddd8152, 98 },
    { 0xd45d8153, 23 },
    { 0xae2d8154, 95 },
    { 0xff2d8153, 95 },
    { 0, 0 },
    { 0xa30d8157, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfd8161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfd8164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20d8167, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132d816b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdd8174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadd8179, 23 },
    { 0x461d817a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d817d, 55 },
    { 0x199d817d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d8181, 68 },
    { 0x5a3d8181, 34 },
    { 0xd08d8181, 46 },
    { 0xbb9d8184, 61 },
    { 0xdf5d8185, 34 },
    { 0xe95d8186, 84 },
    { 0, 0 },
    { 0x50fd8188, 52 },
    { 0, 0 },
    { 0x3b8d818a, 52 },
    { 0, 0 },
    { 0x76fd818c, 84 },
    { 0, 0 },
    { 0x544d818e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d8191, 76 },
    { 0xcd0d8191, 98 },
    { 0x3c8d8191, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacd8198, 76 },
    { 0xebfd8199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ed81a7, 33 },
    { 0x779d81a7, 33 },
    { 0x157d81a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cd81ac, 33 },
    { 0, 0 },
    { 0x3ecd81ae, 78 },
    { 0xb47d81af, 95 },
    { 0x707d81b0, 33 },
    { 0xc8ed81b1, 98 },
    { 0xa8bd81b0, 23 },
    { 0, 0 },
    { 0x636d81b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dad81b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dd81be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d81c1, 98 },
    { 0x8a2d81c1, 68 },
    { 0, 0 },
    { 0x6f5d81c4, 78 },
    { 0x2ed81c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398d81cb, 93 },
    { 0x70fd81cc, 90 },
    { 0x33ad81cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04d81d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d81d8, 90 },
    { 0x6fdd81d9, 67 },
    { 0x6acd81da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ed81e3, 61 },
    { 0x376d81e4, 55 },
    { 0x347d81e5, 98 },
    { 0, 0 },
    { 0xd12d81e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d81ef, 90 },
    { 0xb79d81f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d81f3, 75 },
    { 0x32d81f4, 34 },
    { 0xf83d81f5, 65 },
    { 0x889d81f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d81f9, 90 },
    { 0, 0 },
    { 0x344d81fb, 98 },
    { 0, 0 },
    { 0xfbed81fd, 67 },
    { 0, 0 },
    { 0x374d81ff, 33 },
    { 0, 0 },
    { 0xae4d8201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdd8205, 78 },
    { 0x226d8205, 90 },
    { 0x921d8207, 33 },
    { 0xbd8205, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332d8210, 98 },
    { 0x50cd8211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ed8218, 33 },
    { 0x799d8218, 23 },
    { 0, 0 },
    { 0xa39d821b, 90 },
    { 0xec1d821c, 85 },
    { 0xdc9d821d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0d8222, 23 },
    { 0xe63d8223, 90 },
    { 0x4a9d8222, 78 },
    { 0x68bd8225, 87 },
    { 0x765d8225, 61 },
    { 0x85d8227, 61 },
    { 0xbdad8227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x843d822b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942d8231, 33 },
    { 0xd1fd8231, 95 },
    { 0x73fd8232, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cd8236, 51 },
    { 0, 0 },
    { 0x8a8d8238, 98 },
    { 0, 0 },
    { 0xcc1d823a, 43 },
    { 0, 0 },
    { 0x5ad823c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d8243, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd8246, 100 },
    { 0, 0 },
    { 0x273d8248, 34 },
    { 0x1f8d8249, 100 },
    { 0xe65d8248, 95 },
    { 0xdb2d8249, 93 },
    { 0xe59d824c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd8251, 61 },
    { 0x245d8252, 90 },
    { 0xab2d8252, 55 },
    { 0x8a9d8254, 33 },
    { 0xe90d8255, 61 },
    { 0x685d8256, 84 },
    { 0xd04d8257, 90 },
    { 0xa3bd8254, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999d825f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d8269, 98 },
    { 0xd17d8269, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad8271, 90 },
    { 0x400d8271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d8276, 85 },
    { 0xd9ed8276, 83 },
    { 0xc13d8278, 100 },
    { 0, 0 },
    { 0x429d827a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619d8280, 90 },
    { 0x21cd8281, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54d8286, 33 },
    { 0, 0 },
    { 0xf97d8288, 90 },
    { 0, 0 },
    { 0xe4ed828a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x538d828d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2d8290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdced8293, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x318d8296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fd829e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d82a6, 83 },
    { 0x356d82a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d82aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9d82b1, 76 },
    { 0, 0 },
    { 0x53fd82b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d82b8, 77 },
    { 0, 0 },
    { 0xafcd82ba, 61 },
    { 0, 0 },
    { 0xea4d82bc, 98 },
    { 0, 0 },
    { 0xb26d82be, 76 },
    { 0xff5d82bf, 87 },
    { 0, 0 },
    { 0x4d1d82c1, 90 },
    { 0xf5bd82c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d82c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dd82cf, 77 },
    { 0x6b6d82cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edd82d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ed82d9, 33 },
    { 0x4ecd82d9, 34 },
    { 0xd45d82db, 85 },
    { 0x27cd82db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02d82df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137d82e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad82e7, 33 },
    { 0, 0 },
    { 0x73cd82e9, 98 },
    { 0xecdd82ea, 55 },
    { 0, 0 },
    { 0x3f1d82ec, 61 },
    { 0xa5cd82ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d82f0, 95 },
    { 0, 0 },
    { 0x6e4d82f2, 76 },
    { 0, 0 },
    { 0xc35d82f4, 55 },
    { 0xbd4d82f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafd82f8, 34 },
    { 0xbe4d82f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cd82fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9d8304, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d8309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d830e, 43 },
    { 0x59bd830f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8317, 93 },
    { 0x92dd8318, 46 },
    { 0, 0 },
    { 0xcefd831a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d831d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d8320, 96 },
    { 0x87ad8321, 55 },
    { 0xa60d8322, 23 },
    { 0, 0 },
    { 0x43d8324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6d8329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d8330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcd8341, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922d8345, 61 },
    { 0, 0 },
    { 0x52dd8347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339d834d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d8352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2d8356, 33 },
    { 0x83bd8356, 98 },
    { 0x613d8358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cd835c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d8362, 76 },
    { 0x1a8d8363, 69 },
    { 0, 0 },
    { 0x8bbd8365, 33 },
    { 0x9a4d8366, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0d8369, 33 },
    { 0xb4cd836a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3d836f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd8374, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d837b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d8380, 87 },
    { 0xa6ed8381, 95 },
    { 0x33ad8382, 61 },
    { 0, 0 },
    { 0x62cd8384, 87 },
    { 0, 0 },
    { 0x483d8386, 87 },
    { 0x195d8386, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76d838a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabd838e, 34 },
    { 0xfd3d838f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fd839c, 34 },
    { 0xa34d839d, 61 },
    { 0x2f9d839e, 23 },
    { 0x8e6d839f, 23 },
    { 0, 0 },
    { 0x77ed83a1, 95 },
    { 0, 0 },
    { 0x305d83a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad83ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d83b1, 55 },
    { 0x274d83b1, 98 },
    { 0x135d83b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424d83bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d83c8, 61 },
    { 0x999d83c9, 76 },
    { 0, 0 },
    { 0xf97d83cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd83d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecd83d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbd83db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d83de, 90 },
    { 0xb53d83df, 80 },
    { 0xd45d83e0, 55 },
    { 0xd42d83e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad83e4, 95 },
    { 0, 0 },
    { 0xfa6d83e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1d83e9, 93 },
    { 0, 0 },
    { 0x752d83eb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ed83f0, 98 },
    { 0x6b3d83f0, 81 },
    { 0x23bd83f2, 77 },
    { 0xf98d83f0, 34 },
    { 0, 0 },
    { 0x55fd83f5, 95 },
    { 0x61fd83f6, 33 },
    { 0xdb2d83f5, 95 },
    { 0xfced83f8, 95 },
    { 0, 0 },
    { 0xa79d83fa, 55 },
    { 0, 0 },
    { 0x2bad83fc, 17 },
    { 0x101d83fd, 23 },
    { 0xc51d83fc, 75 },
    { 0xee4d83ff, 90 },
    { 0, 0 },
    { 0x13ed8401, 98 },
    { 0x7fed8402, 98 },
    { 0, 0 },
    { 0x1fed8404, 95 },
    { 0, 0 },
    { 0xcdcd8406, 33 },
    { 0xcb2d8407, 98 },
    { 0xf3d8408, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafd840f, 90 },
    { 0x750d8410, 84 },
    { 0x3fcd8411, 98 },
    { 0xfddd8411, 34 },
    { 0x8e9d8410, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d841a, 93 },
    { 0xd0ed841a, 90 },
    { 0x2dbd841c, 98 },
    { 0xdd9d841c, 33 },
    { 0xafad841e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd8421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd8425, 98 },
    { 0, 0 },
    { 0xa8fd8427, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36d842a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdd842e, 75 },
    { 0xdd0d842f, 93 },
    { 0x5a0d8430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5d8437, 35 },
    { 0x75ad8437, 95 },
    { 0xb02d8439, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205d8442, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8d8446, 23 },
    { 0x43bd8447, 90 },
    { 0x5d8d8448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7d844c, 61 },
    { 0x808d844d, 98 },
    { 0xf3cd844e, 23 },
    { 0, 0 },
    { 0x486d8450, 88 },
    { 0xf8cd8451, 38 },
    { 0, 0 },
    { 0x949d8453, 38 },
    { 0x1c4d8454, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d8458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fd845e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d8462, 93 },
    { 0, 0 },
    { 0xd7cd8464, 90 },
    { 0x6dbd8465, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfded846b, 88 },
    { 0x8dfd846c, 55 },
    { 0xe7cd846d, 91 },
    { 0x780d846e, 33 },
    { 0x803d846f, 34 },
    { 0xebdd8470, 23 },
    { 0xe2d8471, 61 },
    { 0, 0 },
    { 0xb43d8473, 61 },
    { 0x87bd8474, 33 },
    { 0, 0 },
    { 0xbdbd8476, 69 },
    { 0, 0 },
    { 0x329d8478, 95 },
    { 0x5d2d8479, 88 },
    { 0xaf0d847a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723d847e, 46 },
    { 0xeccd847f, 43 },
    { 0x551d847f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d848e, 78 },
    { 0xd4dd848f, 78 },
    { 0x6e8d848f, 90 },
    { 0x52fd8491, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4d849a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd849d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2d84a0, 98 },
    { 0x88dd84a1, 33 },
    { 0x4d2d84a1, 90 },
    { 0, 0 },
    { 0xabd84a4, 33 },
    { 0, 0 },
    { 0xfbd84a6, 88 },
    { 0, 0 },
    { 0x254d84a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d84af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd84b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad84be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd84c7, 61 },
    { 0, 0 },
    { 0xbbfd84c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d84d1, 61 },
    { 0, 0 },
    { 0x909d84d3, 42 },
    { 0, 0 },
    { 0x38d84d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59d84db, 33 },
    { 0x3e7d84dc, 61 },
    { 0, 0 },
    { 0x4ced84de, 38 },
    { 0, 0 },
    { 0x111d84e0, 61 },
    { 0x8d2d84e1, 33 },
    { 0, 0 },
    { 0x435d84e3, 93 },
    { 0x6aed84e4, 55 },
    { 0xc8cd84e3, 93 },
    { 0, 0 },
    { 0x430d84e7, 61 },
    { 0xbefd84e8, 23 },
    { 0x5e1d84e9, 33 },
    { 0xef0d84e7, 95 },
    { 0xcc1d84e8, 61 },
    { 0x2a2d84ec, 23 },
    { 0x1e3d84ed, 55 },
    { 0xcdad84ee, 23 },
    { 0x947d84ee, 95 },
    { 0x5a8d84e9, 98 },
    { 0x10bd84f1, 91 },
    { 0x39ed84f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d84f5, 90 },
    { 0x271d84f6, 33 },
    { 0x6acd84f7, 76 },
    { 0x76ed84f8, 33 },
    { 0xb59d84f7, 95 },
    { 0x86fd84fa, 61 },
    { 0x2e4d84f6, 87 },
    { 0x107d84fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x789d84ff, 91 },
    { 0, 0 },
    { 0x12fd8501, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d8504, 95 },
    { 0, 0 },
    { 0x277d8506, 95 },
    { 0x8eed8507, 23 },
    { 0xacd8507, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20d850e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d8515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d8518, 17 },
    { 0x972d8519, 61 },
    { 0, 0 },
    { 0x516d851b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d8521, 43 },
    { 0, 0 },
    { 0xb1d8523, 76 },
    { 0, 0 },
    { 0x59bd8525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad852a, 93 },
    { 0x7ed852a, 55 },
    { 0x622d852c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd8533, 77 },
    { 0xd9d8534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fad8538, 98 },
    { 0x83dd8539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd853c, 40 },
    { 0, 0 },
    { 0xd85d853e, 61 },
    { 0xaecd853e, 33 },
    { 0, 0 },
    { 0x739d8541, 61 },
    { 0, 0 },
    { 0xa76d8543, 95 },
    { 0, 0 },
    { 0xb4bd8545, 98 },
    { 0x12d8545, 34 },
    { 0x8a0d8545, 46 },
    { 0xf6d8547, 98 },
    { 0xc30d8549, 24 },
    { 0x363d854a, 46 },
    { 0xfb2d854b, 90 },
    { 0xeecd854a, 90 },
    { 0x40ed854d, 33 },
    { 0x375d8546, 61 },
    { 0x7f4d854f, 90 },
    { 0x27ad8550, 90 },
    { 0x685d8551, 44 },
    { 0xac0d854f, 99 },
    { 0xf90d8548, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ed8559, 93 },
    { 0xf5ed855a, 90 },
    { 0xc6ad8559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d855e, 61 },
    { 0xb05d855f, 96 },
    { 0, 0 },
    { 0xd99d8561, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ed8564, 61 },
    { 0xd9ad8565, 34 },
    { 0x60bd8566, 91 },
    { 0xf1fd8567, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d856e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bd8573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x191d8576, 55 },
    { 0, 0 },
    { 0x488d8578, 76 },
    { 0xe79d8578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dd857c, 61 },
    { 0x575d857c, 95 },
    { 0x9bdd857e, 33 },
    { 0x2e3d857f, 61 },
    { 0xa8fd8580, 66 },
    { 0, 0 },
    { 0x90dd8582, 75 },
    { 0x5f6d8583, 75 },
    { 0xb8d8584, 33 },
    { 0xcfcd8584, 98 },
    { 0xae0d8585, 90 },
    { 0x71ad8587, 90 },
    { 0xfed8587, 23 },
    { 0x1c3d8588, 84 },
    { 0, 0 },
    { 0xc5fd858b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fd8591, 61 },
    { 0, 0 },
    { 0x753d8593, 52 },
    { 0x226d8594, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8599, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3d859e, 61 },
    { 0, 0 },
    { 0x28ad85a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d85a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d85ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97d85b0, 99 },
    { 0, 0 },
    { 0xf2dd85b2, 33 },
    { 0, 0 },
    { 0x242d85b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d85b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d85ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd85c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bad85cb, 52 },
    { 0xb46d85cc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8d85d5, 90 },
    { 0x953d85d6, 33 },
    { 0x391d85d7, 61 },
    { 0x8ccd85d8, 66 },
    { 0x162d85d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd85e1, 77 },
    { 0x5e2d85e2, 34 },
    { 0xa75d85e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d85e6, 67 },
    { 0x245d85e7, 90 },
    { 0, 0 },
    { 0xcf0d85e9, 98 },
    { 0, 0 },
    { 0xb64d85eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd85ef, 43 },
    { 0xe70d85f0, 84 },
    { 0, 0 },
    { 0x97ad85f2, 98 },
    { 0x9fdd85f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1d85f6, 87 },
    { 0xf4ad85f6, 90 },
    { 0xe13d85f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd85fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318d8604, 90 },
    { 0xd16d8604, 55 },
    { 0x9b1d8605, 80 },
    { 0xc3fd8607, 33 },
    { 0xef6d8608, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2d860b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad860e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49d8613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d8616, 33 },
    { 0, 0 },
    { 0xaf6d8618, 90 },
    { 0, 0 },
    { 0x7f4d861a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d8625, 43 },
    { 0xfb1d8626, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d862a, 33 },
    { 0xab2d862b, 95 },
    { 0xb4cd862c, 90 },
    { 0, 0 },
    { 0x746d862e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d8631, 17 },
    { 0, 0 },
    { 0xb69d8633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fd8639, 23 },
    { 0xf89d863a, 81 },
    { 0xb24d8639, 34 },
    { 0x876d863c, 98 },
    { 0x33d863b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacd8640, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ed8643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd864e, 96 },
    { 0, 0 },
    { 0x751d8650, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524d865a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d865d, 33 },
    { 0x66d865d, 40 },
    { 0x814d865d, 61 },
    { 0x8e5d865e, 65 },
    { 0xd53d865f, 43 },
    { 0x728d8661, 55 },
    { 0x1d8663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00d8667, 23 },
    { 0, 0 },
    { 0xf06d8669, 55 },
    { 0x133d866a, 52 },
    { 0xac9d866b, 77 },
    { 0x385d866c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d8672, 98 },
    { 0, 0 },
    { 0x988d8674, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8d8678, 33 },
    { 0, 0 },
    { 0xeecd867a, 23 },
    { 0x6b2d867a, 61 },
    { 0xf15d867c, 46 },
    { 0x82fd867d, 36 },
    { 0x8a5d867e, 78 },
    { 0x7e0d867c, 92 },
    { 0xfcd8680, 46 },
    { 0x184d867c, 43 },
    { 0x673d8680, 23 },
    { 0, 0 },
    { 0xf32d8684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d8688, 55 },
    { 0x49bd8689, 95 },
    { 0x987d868a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbd868f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ad8693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fd8697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550d869e, 90 },
    { 0x18bd869e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d86a3, 55 },
    { 0, 0 },
    { 0xc57d86a5, 40 },
    { 0xdd86a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0d86a9, 100 },
    { 0, 0 },
    { 0x76ed86ab, 33 },
    { 0, 0 },
    { 0xf3fd86ad, 76 },
    { 0x7fcd86ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d86b1, 33 },
    { 0xd5fd86b2, 93 },
    { 0x8fad86b3, 55 },
    { 0x212d86b2, 90 },
    { 0, 0 },
    { 0x85d86b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80d86bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cd86cb, 61 },
    { 0x4aed86cc, 76 },
    { 0x9e7d86cb, 80 },
    { 0x122d86cb, 90 },
    { 0xb59d86cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d86d9, 55 },
    { 0xd86d86da, 67 },
    { 0xc3bd86d9, 95 },
    { 0xb18d86dc, 98 },
    { 0x5fd86dd, 90 },
    { 0x46cd86dc, 48 },
    { 0, 0 },
    { 0xd33d86e0, 34 },
    { 0x956d86e1, 91 },
    { 0x428d86e2, 46 },
    { 0x196d86e3, 23 },
    { 0x161d86e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x453d86e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcd86ea, 95 },
    { 0xc1cd86ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d86ee, 33 },
    { 0xee9d86ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd86f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x631d86f7, 34 },
    { 0, 0 },
    { 0x7d1d86f9, 46 },
    { 0, 0 },
    { 0x3aed86fb, 61 },
    { 0, 0 },
    { 0x427d86fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd8702, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd8705, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d8710, 33 },
    { 0xe73d8710, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d8714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd8719, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dd8720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5d8723, 23 },
    { 0xda0d8724, 95 },
    { 0xde9d8725, 55 },
    { 0x5c0d8725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x567d8729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d872c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d8732, 17 },
    { 0x7d7d8733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d8739, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ed873c, 55 },
    { 0, 0 },
    { 0xa2fd873e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d8741, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d8746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9d874b, 92 },
    { 0xf24d874c, 67 },
    { 0xc9dd874c, 90 },
    { 0xe71d874e, 95 },
    { 0x5f7d874f, 33 },
    { 0, 0 },
    { 0x799d8751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd8754, 33 },
    { 0, 0 },
    { 0xed0d8756, 91 },
    { 0xaeed8757, 34 },
    { 0xf54d8757, 46 },
    { 0xd47d8759, 99 },
    { 0, 0 },
    { 0xa21d875b, 90 },
    { 0x2e8d875c, 55 },
    { 0x75cd875d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32d8764, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d8767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd876b, 90 },
    { 0, 0 },
    { 0x846d876d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446d8778, 90 },
    { 0, 0 },
    { 0xfbfd877a, 95 },
    { 0x4abd877b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1d8781, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdd878a, 23 },
    { 0, 0 },
    { 0x64d878c, 55 },
    { 0xdbbd878d, 61 },
    { 0x169d878e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dd8795, 46 },
    { 0, 0 },
    { 0xa50d8797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad879b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d879e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd87a1, 52 },
    { 0xb56d87a1, 61 },
    { 0, 0 },
    { 0x25ad87a4, 46 },
    { 0xab5d87a5, 61 },
    { 0x59d87a4, 33 },
    { 0x842d87a7, 85 },
    { 0xfbbd87a8, 56 },
    { 0xc0bd87a6, 55 },
    { 0x765d87aa, 77 },
    { 0x410d87ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77d87b0, 25 },
    { 0xd23d87b1, 33 },
    { 0x847d87b2, 51 },
    { 0x5d9d87b3, 90 },
    { 0, 0 },
    { 0xe49d87b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34d87b8, 99 },
    { 0x8bdd87b9, 90 },
    { 0xdf7d87ba, 33 },
    { 0xa11d87bb, 76 },
    { 0x6e3d87bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad87bf, 78 },
    { 0xf3fd87bf, 95 },
    { 0x114d87c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d87c6, 33 },
    { 0x237d87c7, 99 },
    { 0x556d87c8, 95 },
    { 0x278d87c6, 98 },
    { 0xe6ed87ca, 90 },
    { 0x729d87c9, 76 },
    { 0x38fd87cc, 61 },
    { 0x885d87cb, 80 },
    { 0x9d0d87ce, 95 },
    { 0x435d87cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd87d3, 43 },
    { 0xfcd87d4, 85 },
    { 0x6fed87d5, 99 },
    { 0xefbd87d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd87d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4d87dc, 95 },
    { 0xa58d87dd, 61 },
    { 0x1d6d87de, 34 },
    { 0x334d87de, 95 },
    { 0, 0 },
    { 0xfa0d87e1, 95 },
    { 0x3fed87e2, 85 },
    { 0xf2bd87e2, 98 },
    { 0, 0 },
    { 0x68d87e5, 75 },
    { 0, 0 },
    { 0x8add87e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d87ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d87ed, 17 },
    { 0xbbcd87ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9d87f4, 33 },
    { 0, 0 },
    { 0x538d87f6, 90 },
    { 0xdb3d87f7, 76 },
    { 0x70d87f8, 61 },
    { 0xb32d87f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd87fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91d8803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x221d8806, 33 },
    { 0, 0 },
    { 0x453d8808, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0d880b, 23 },
    { 0, 0 },
    { 0x291d880d, 85 },
    { 0, 0 },
    { 0x3a2d880f, 98 },
    { 0xacdd8810, 55 },
    { 0x17bd8811, 61 },
    { 0x430d8812, 55 },
    { 0, 0 },
    { 0x4f4d8814, 43 },
    { 0, 0 },
    { 0x5dad8816, 87 },
    { 0, 0 },
    { 0x73d8818, 90 },
    { 0xe00d8818, 75 },
    { 0, 0 },
    { 0xfad881b, 33 },
    { 0xf9ad881c, 93 },
    { 0x223d881c, 95 },
    { 0xc46d881e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0d8821, 81 },
    { 0xccd8822, 95 },
    { 0x128d8823, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d8828, 67 },
    { 0x81ed8829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8d882e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123d8832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd8842, 98 },
    { 0, 0 },
    { 0xf60d8844, 33 },
    { 0xaf7d8845, 33 },
    { 0xc7ed8846, 95 },
    { 0xdddd8844, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94d884a, 52 },
    { 0, 0 },
    { 0x912d884c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d884f, 66 },
    { 0x532d8850, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237d8854, 94 },
    { 0x2b2d8855, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d8858, 33 },
    { 0x97d8859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27d8860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d8863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8d8868, 80 },
    { 0x661d8869, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4d886c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d8872, 90 },
    { 0x860d8873, 61 },
    { 0, 0 },
    { 0xb9ad8875, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fd8878, 91 },
    { 0, 0 },
    { 0x586d887a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd887e, 46 },
    { 0x7aad887f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaced8883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207d888c, 98 },
    { 0xfdbd888d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d8894, 90 },
    { 0, 0 },
    { 0x954d8896, 90 },
    { 0x493d8896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad889d, 90 },
    { 0x29bd889d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbd88a5, 23 },
    { 0xf10d88a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d88ac, 61 },
    { 0xbedd88ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d88b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84d88b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d88bb, 78 },
    { 0x176d88bc, 61 },
    { 0xa36d88bd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedd88c0, 87 },
    { 0x191d88c0, 61 },
    { 0x5cbd88c2, 43 },
    { 0xaefd88c3, 90 },
    { 0x9ecd88c4, 34 },
    { 0x910d88c0, 90 },
    { 0xd52d88c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x844d88c9, 48 },
    { 0x296d88ca, 90 },
    { 0xd41d88cb, 33 },
    { 0x727d88cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468d88d3, 90 },
    { 0x89bd88d3, 33 },
    { 0x4d2d88d3, 61 },
    { 0x18fd88d6, 76 },
    { 0x709d88d6, 34 },
    { 0xead88d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799d88dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d88e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd88e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8d88f4, 98 },
    { 0x5cfd88f4, 55 },
    { 0, 0 },
    { 0x5d3d88f7, 90 },
    { 0xfa8d88f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcd88ff, 61 },
    { 0xcfbd88ff, 76 },
    { 0x583d8901, 85 },
    { 0x24d8902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cd890a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd890d, 100 },
    { 0x7dbd890e, 88 },
    { 0, 0 },
    { 0xbdbd8910, 66 },
    { 0x107d8910, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d8915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51d8921, 61 },
    { 0, 0 },
    { 0x383d8923, 23 },
    { 0, 0 },
    { 0xf7ad8925, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd892a, 23 },
    { 0x8a8d892b, 46 },
    { 0, 0 },
    { 0x1d9d892d, 33 },
    { 0x7f2d892e, 99 },
    { 0xf9cd892e, 34 },
    { 0, 0 },
    { 0x479d8931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41d893c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d8942, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd8949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2d894f, 95 },
    { 0xf88d8950, 19 },
    { 0x4d6d8951, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9d8954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395d8958, 83 },
    { 0xbbad8959, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448d895d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8961, 90 },
    { 0x347d8962, 33 },
    { 0, 0 },
    { 0xd68d8964, 93 },
    { 0xb38d8965, 77 },
    { 0x29ed8966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d8969, 87 },
    { 0x6bed8969, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d8972, 90 },
    { 0x59bd8973, 44 },
    { 0xdc7d8974, 75 },
    { 0xa57d8975, 52 },
    { 0x24cd8974, 91 },
    { 0xa0ad8973, 23 },
    { 0x5e8d8974, 90 },
    { 0x371d8976, 33 },
    { 0, 0 },
    { 0xd89d897b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87d8980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d8987, 55 },
    { 0xd17d8988, 95 },
    { 0, 0 },
    { 0xc65d898a, 33 },
    { 0x875d898b, 55 },
    { 0x7edd898c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8993, 93 },
    { 0x953d8993, 84 },
    { 0xa5bd8995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbd8999, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddd89a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7d89a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d89aa, 43 },
    { 0x91d89ab, 61 },
    { 0xa21d89ac, 98 },
    { 0, 0 },
    { 0x80cd89ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1d89b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed89b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03d89ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d89c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fd89c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ed89ca, 90 },
    { 0xfdcd89ca, 76 },
    { 0xb7dd89cb, 75 },
    { 0x7d8d89cd, 33 },
    { 0xd3ed89cd, 33 },
    { 0xc59d89cf, 33 },
    { 0, 0 },
    { 0x2fad89d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680d89d7, 33 },
    { 0x42cd89d8, 34 },
    { 0x8d89d9, 90 },
    { 0x20ad89da, 77 },
    { 0x327d89db, 86 },
    { 0xa3d89dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1d89e0, 95 },
    { 0xdbed89e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fd89e9, 33 },
    { 0x7a1d89e9, 81 },
    { 0x25fd89e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03d89ef, 77 },
    { 0x495d89f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd89f5, 61 },
    { 0, 0 },
    { 0x45ad89f7, 91 },
    { 0x808d89f8, 90 },
    { 0x7fd89f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d89ff, 93 },
    { 0, 0 },
    { 0x72ed8a01, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bd8a04, 43 },
    { 0x799d8a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bd8a0a, 95 },
    { 0, 0 },
    { 0x81ed8a0c, 46 },
    { 0, 0 },
    { 0x2d8d8a0e, 95 },
    { 0xd30d8a0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302d8a17, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d8a1b, 90 },
    { 0x529d8a1c, 33 },
    { 0x6c4d8a1d, 43 },
    { 0xb05d8a1e, 76 },
    { 0x529d8a1f, 85 },
    { 0xf6cd8a1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d8a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d8a2b, 90 },
    { 0, 0 },
    { 0x550d8a2d, 55 },
    { 0xcd8a2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0d8a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754d8a35, 88 },
    { 0x19ed8a35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1d8a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dd8a3e, 91 },
    { 0x3b7d8a3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29d8a42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d8a46, 92 },
    { 0x4c5d8a46, 90 },
    { 0xc81d8a46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dd8a4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64d8a57, 90 },
    { 0x267d8a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d8a5e, 23 },
    { 0, 0 },
    { 0x37ad8a60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80d8a65, 43 },
    { 0x21d8a66, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad8a6c, 34 },
    { 0xd47d8a6c, 90 },
    { 0xbdcd8a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0d8a71, 98 },
    { 0x128d8a72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00d8a79, 95 },
    { 0x822d8a79, 46 },
    { 0xa7d8a7b, 46 },
    { 0xac5d8a7c, 76 },
    { 0xbf6d8a7d, 81 },
    { 0x28bd8a7e, 95 },
    { 0, 0 },
    { 0x818d8a80, 95 },
    { 0x4ebd8a81, 66 },
    { 0xcc1d8a82, 87 },
    { 0x6ccd8a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd8a87, 46 },
    { 0xca5d8a88, 33 },
    { 0xb99d8a88, 76 },
    { 0x521d8a89, 76 },
    { 0xedad8a8b, 34 },
    { 0x903d8a8b, 61 },
    { 0x9cdd8a8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bd8a97, 2 },
    { 0x7f2d8a97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd8a9d, 98 },
    { 0xa0fd8a9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8d8aa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed8aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6d8ab4, 33 },
    { 0x36ad8ab5, 91 },
    { 0x33cd8ab6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ead8abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ad8ac1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d8ac4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2d8ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add8ad1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d8ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd8ad8, 23 },
    { 0, 0 },
    { 0xfa8d8ada, 90 },
    { 0, 0 },
    { 0xa88d8adc, 98 },
    { 0, 0 },
    { 0x500d8ade, 55 },
    { 0x15ed8adf, 43 },
    { 0xc5cd8adf, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86d8ae6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ced8ae9, 98 },
    { 0, 0 },
    { 0xbd2d8aeb, 95 },
    { 0x3b0d8aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdd8aef, 98 },
    { 0, 0 },
    { 0x6bed8af1, 45 },
    { 0xc3ed8af2, 76 },
    { 0xd61d8af3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d8af7, 85 },
    { 0xb16d8af7, 100 },
    { 0xdbfd8af9, 33 },
    { 0xf6fd8afa, 34 },
    { 0xac0d8afa, 34 },
    { 0x657d8afa, 61 },
    { 0x5b5d8afc, 90 },
    { 0xdc3d8afa, 88 },
    { 0, 0 },
    { 0xdded8b00, 84 },
    { 0xc10d8b00, 34 },
    { 0, 0 },
    { 0xd27d8b03, 95 },
    { 0, 0 },
    { 0xc70d8b05, 91 },
    { 0xd8ad8b06, 95 },
    { 0, 0 },
    { 0xfc1d8b08, 61 },
    { 0, 0 },
    { 0xdbd8b0a, 90 },
    { 0xe5fd8b0b, 45 },
    { 0, 0 },
    { 0x31d8b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d8b10, 93 },
    { 0xb71d8b11, 52 },
    { 0xa69d8b12, 98 },
    { 0xa06d8b13, 61 },
    { 0x34ed8b13, 78 },
    { 0x53fd8b15, 23 },
    { 0xde0d8b13, 84 },
    { 0x267d8b15, 98 },
    { 0xfbd8b18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32d8b1b, 90 },
    { 0x620d8b1b, 90 },
    { 0, 0 },
    { 0x2b3d8b1e, 61 },
    { 0x817d8b1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92d8b23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9d8b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dd8b2d, 34 },
    { 0xaeed8b2e, 43 },
    { 0, 0 },
    { 0xd41d8b30, 46 },
    { 0xbe9d8b30, 93 },
    { 0x982d8b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d8b35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efd8b3a, 98 },
    { 0x5dfd8b3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd8b3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d8b43, 78 },
    { 0x13dd8b44, 66 },
    { 0xa8bd8b44, 33 },
    { 0, 0 },
    { 0x75ad8b47, 81 },
    { 0xe30d8b48, 98 },
    { 0x7eed8b49, 61 },
    { 0x49ed8b4a, 34 },
    { 0xb7fd8b4b, 76 },
    { 0, 0 },
    { 0xf92d8b4d, 23 },
    { 0x66dd8b4e, 93 },
    { 0x3ced8b4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73d8b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d8b5e, 33 },
    { 0x6bfd8b5f, 98 },
    { 0xafcd8b60, 8 },
    { 0x634d8b61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x210d8b64, 68 },
    { 0, 0 },
    { 0x6efd8b66, 33 },
    { 0x631d8b66, 46 },
    { 0xc06d8b68, 90 },
    { 0x29fd8b68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d8b6d, 90 },
    { 0, 0 },
    { 0x6fbd8b6f, 85 },
    { 0x34fd8b70, 84 },
    { 0x808d8b71, 33 },
    { 0x579d8b71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991d8b77, 33 },
    { 0x65dd8b77, 34 },
    { 0, 0 },
    { 0x998d8b7a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d8b7d, 95 },
    { 0xb9ad8b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd8b86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8b89, 33 },
    { 0, 0 },
    { 0x71dd8b8b, 61 },
    { 0, 0 },
    { 0xe40d8b8d, 34 },
    { 0, 0 },
    { 0x72fd8b8f, 77 },
    { 0xda5d8b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d8b99, 93 },
    { 0xe00d8b99, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad8ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3d8ba7, 93 },
    { 0x994d8ba7, 90 },
    { 0x665d8ba7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ed8bad, 34 },
    { 0x1b6d8bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bd8bb1, 76 },
    { 0x271d8bb2, 66 },
    { 0xd89d8bb2, 61 },
    { 0x799d8bb3, 33 },
    { 0xcadd8bb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9d8bba, 98 },
    { 0x5d9d8bba, 90 },
    { 0x5d8d8bbc, 33 },
    { 0x177d8bbb, 33 },
    { 0, 0 },
    { 0x883d8bbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151d8bc5, 55 },
    { 0, 0 },
    { 0xd16d8bc7, 90 },
    { 0x37ed8bc7, 55 },
    { 0x1d4d8bc9, 23 },
    { 0x894d8bc9, 87 },
    { 0xcb8d8bcb, 90 },
    { 0x8c7d8bcb, 91 },
    { 0xeced8bc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8bd0, 34 },
    { 0x5fcd8bd0, 93 },
    { 0, 0 },
    { 0x4c6d8bd3, 23 },
    { 0, 0 },
    { 0x8dbd8bd5, 90 },
    { 0xf8fd8bd6, 87 },
    { 0x3c0d8bd5, 93 },
    { 0x478d8bd8, 93 },
    { 0x83ad8bd9, 98 },
    { 0x108d8bd9, 95 },
    { 0xb66d8bdb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282d8be2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8be8, 95 },
    { 0x360d8be9, 61 },
    { 0, 0 },
    { 0x12ed8beb, 98 },
    { 0xcc4d8beb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921d8bef, 61 },
    { 0x519d8bf0, 98 },
    { 0xad4d8bf1, 87 },
    { 0, 0 },
    { 0x14ad8bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cd8bf6, 33 },
    { 0x8cdd8bf7, 61 },
    { 0xbbdd8bf8, 81 },
    { 0, 0 },
    { 0x49ad8bfa, 80 },
    { 0xb7dd8bfb, 76 },
    { 0x188d8bfc, 100 },
    { 0, 0 },
    { 0x1ed8bfe, 95 },
    { 0x2dbd8bff, 95 },
    { 0x15ed8bfe, 95 },
    { 0, 0 },
    { 0xc11d8c02, 90 },
    { 0x89d8c03, 23 },
    { 0xf88d8c03, 90 },
    { 0xe76d8c05, 43 },
    { 0xb0d8c06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91d8c09, 34 },
    { 0xd86d8c0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d8c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8d8c15, 23 },
    { 0x322d8c15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d8c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3d8c25, 95 },
    { 0x215d8c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236d8c2c, 90 },
    { 0xb7dd8c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd8c30, 95 },
    { 0x7f3d8c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813d8c35, 76 },
    { 0x150d8c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1d8c39, 23 },
    { 0x29ed8c3a, 95 },
    { 0x2b5d8c3a, 61 },
    { 0, 0 },
    { 0x6e4d8c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d8c42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd8c49, 99 },
    { 0, 0 },
    { 0x8d6d8c4b, 34 },
    { 0x405d8c4c, 34 },
    { 0x64d8c4c, 90 },
    { 0x91fd8c4b, 98 },
    { 0xc28d8c4f, 91 },
    { 0x26ad8c4c, 98 },
    { 0xefbd8c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcd8c55, 23 },
    { 0, 0 },
    { 0xdfcd8c57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd8c5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41d8c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ed8c66, 33 },
    { 0x2ccd8c67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d8c6e, 33 },
    { 0xc4fd8c6e, 84 },
    { 0x2cad8c6f, 61 },
    { 0, 0 },
    { 0x3b2d8c72, 61 },
    { 0, 0 },
    { 0x6fd8c74, 84 },
    { 0, 0 },
    { 0x36cd8c76, 91 },
    { 0, 0 },
    { 0x2d8d8c78, 95 },
    { 0x97fd8c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0d8c7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bd8c86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768d8c8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdd8c90, 93 },
    { 0x376d8c90, 95 },
    { 0x45fd8c90, 84 },
    { 0x78d8c93, 98 },
    { 0, 0 },
    { 0x3d8d8c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bd8c9a, 95 },
    { 0x20bd8c9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x387d8c9e, 61 },
    { 0xbc6d8c9f, 33 },
    { 0xa9dd8c9e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393d8ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383d8cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d8cb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ad8cb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8d8cb8, 61 },
    { 0, 0 },
    { 0xb67d8cba, 23 },
    { 0xa47d8cbb, 90 },
    { 0x46ad8cbb, 55 },
    { 0xeb9d8cbb, 81 },
    { 0x239d8cba, 95 },
    { 0, 0 },
    { 0x5d4d8cc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed8cc5, 88 },
    { 0x78d8cc5, 52 },
    { 0x807d8cc6, 95 },
    { 0xbdcd8cc8, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcd8ccc, 95 },
    { 0, 0 },
    { 0x1ad8cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82d8cd5, 95 },
    { 0, 0 },
    { 0xd6d8cd7, 23 },
    { 0x7f2d8cd7, 75 },
    { 0x32d8cd9, 67 },
    { 0xf78d8cd7, 90 },
    { 0, 0 },
    { 0xafcd8cdc, 23 },
    { 0xc11d8cdd, 23 },
    { 0xbcdd8cde, 90 },
    { 0, 0 },
    { 0x7d2d8ce0, 40 },
    { 0xa9dd8ce1, 98 },
    { 0x3a4d8ce1, 52 },
    { 0x2b0d8ce3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79d8ce7, 93 },
    { 0, 0 },
    { 0x20cd8ce9, 68 },
    { 0xc47d8ce9, 43 },
    { 0xca7d8ceb, 61 },
    { 0xcd8d8cea, 98 },
    { 0xbc5d8cec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d8cf0, 94 },
    { 0, 0 },
    { 0x430d8cf2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd8cf6, 34 },
    { 0x672d8cf7, 23 },
    { 0x8fbd8cf8, 33 },
    { 0, 0 },
    { 0x532d8cfa, 98 },
    { 0xcc0d8cfb, 98 },
    { 0x540d8cfb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd8d03, 95 },
    { 0x88ed8d04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadd8d08, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad8d0b, 90 },
    { 0, 0 },
    { 0xe62d8d0d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd8d16, 99 },
    { 0xabcd8d16, 87 },
    { 0xdeed8d18, 100 },
    { 0x9eed8d19, 90 },
    { 0, 0 },
    { 0xbdcd8d1b, 67 },
    { 0xfa8d8d1c, 61 },
    { 0x825d8d1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d8d22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fd8d27, 90 },
    { 0xabcd8d28, 23 },
    { 0xad8d8d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d8d2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd8d34, 84 },
    { 0, 0 },
    { 0xab4d8d36, 64 },
    { 0x331d8d37, 95 },
    { 0xe58d8d38, 33 },
    { 0x57dd8d37, 33 },
    { 0, 0 },
    { 0x1a6d8d3b, 98 },
    { 0x343d8d3b, 34 },
    { 0x3a8d8d3d, 61 },
    { 0x61bd8d3e, 67 },
    { 0x1a7d8d3e, 98 },
    { 0, 0 },
    { 0x896d8d41, 67 },
    { 0xd3cd8d41, 33 },
    { 0, 0 },
    { 0x361d8d44, 85 },
    { 0xadad8d45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910d8d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8d4f, 98 },
    { 0xccad8d4f, 87 },
    { 0, 0 },
    { 0xff4d8d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d8d56, 98 },
    { 0, 0 },
    { 0x2b8d8d58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7d8d5c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ad8d64, 33 },
    { 0x9b6d8d65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd8d68, 61 },
    { 0x1c4d8d69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8d71, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aed8d74, 46 },
    { 0xd40d8d75, 23 },
    { 0xfdfd8d75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd8d7c, 95 },
    { 0xcd1d8d7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad8d80, 23 },
    { 0x2cfd8d81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313d8d85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d8d88, 98 },
    { 0x232d8d89, 95 },
    { 0xac1d8d8a, 43 },
    { 0, 0 },
    { 0x8e6d8d8c, 23 },
    { 0, 0 },
    { 0xbf5d8d8e, 34 },
    { 0x4a9d8d8f, 46 },
    { 0x95fd8d90, 90 },
    { 0xc2ad8d91, 69 },
    { 0xba3d8d92, 95 },
    { 0, 0 },
    { 0x160d8d94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2d8d97, 23 },
    { 0xa9bd8d98, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63d8d9c, 76 },
    { 0xa0ad8d9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228d8da6, 98 },
    { 0, 0 },
    { 0x145d8da8, 23 },
    { 0x986d8da9, 33 },
    { 0x32fd8daa, 34 },
    { 0xc17d8dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3d8daf, 34 },
    { 0x306d8db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x185d8db3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d8db8, 56 },
    { 0x17bd8db9, 81 },
    { 0x467d8dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5d8dcb, 90 },
    { 0xdc2d8dcc, 61 },
    { 0x962d8dcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d8dd2, 90 },
    { 0, 0 },
    { 0x7c8d8dd4, 40 },
    { 0xded8dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2d8dd8, 61 },
    { 0xd3fd8dd9, 87 },
    { 0x5f3d8dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded8ddd, 90 },
    { 0xf4d8dde, 33 },
    { 0x7b7d8ddf, 76 },
    { 0, 0 },
    { 0x85fd8de1, 65 },
    { 0x749d8de2, 93 },
    { 0, 0 },
    { 0xd1ed8de4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd8de7, 46 },
    { 0x6dfd8de7, 33 },
    { 0xcbbd8de9, 95 },
    { 0, 0 },
    { 0xb0dd8deb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffd8df0, 60 },
    { 0, 0 },
    { 0x29bd8df2, 61 },
    { 0x6aad8df2, 95 },
    { 0xf22d8df4, 95 },
    { 0x3f8d8df2, 101 },
    { 0x65ed8df6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd8dfa, 90 },
    { 0, 0 },
    { 0x46d8dfc, 33 },
    { 0x6aed8dfc, 98 },
    { 0, 0 },
    { 0x2bad8dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cd8e02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9d8e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d8e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cd8e0e, 33 },
    { 0x144d8e0f, 43 },
    { 0xfa5d8e10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed8e16, 92 },
    { 0x73fd8e16, 43 },
    { 0x8cfd8e18, 95 },
    { 0, 0 },
    { 0x321d8e1a, 99 },
    { 0xdd8e1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d8e1e, 69 },
    { 0x528d8e1e, 46 },
    { 0x552d8e1f, 76 },
    { 0xf95d8e21, 99 },
    { 0x654d8e22, 98 },
    { 0xd7ad8e22, 95 },
    { 0, 0 },
    { 0x79fd8e25, 33 },
    { 0xc90d8e26, 43 },
    { 0, 0 },
    { 0x150d8e28, 90 },
    { 0xfbad8e29, 33 },
    { 0xa95d8e28, 90 },
    { 0xe1d8e2a, 91 },
    { 0x58ad8e28, 80 },
    { 0xcb7d8e2d, 61 },
    { 0x29ad8e29, 98 },
    { 0x881d8e2f, 76 },
    { 0x1fbd8e30, 55 },
    { 0xe5dd8e2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bd8e41, 59 },
    { 0, 0 },
    { 0xb4ad8e43, 91 },
    { 0xb6ed8e44, 61 },
    { 0x46fd8e45, 98 },
    { 0, 0 },
    { 0xe1bd8e47, 95 },
    { 0xdcfd8e48, 33 },
    { 0, 0 },
    { 0xde9d8e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d8e4f, 78 },
    { 0x453d8e50, 33 },
    { 0, 0 },
    { 0x38cd8e52, 43 },
    { 0x1acd8e53, 100 },
    { 0xb4dd8e53, 90 },
    { 0x460d8e55, 98 },
    { 0x313d8e55, 61 },
    { 0x467d8e57, 76 },
    { 0xb84d8e57, 66 },
    { 0, 0 },
    { 0x727d8e5a, 87 },
    { 0xb37d8e5a, 87 },
    { 0xcf7d8e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad8e5f, 98 },
    { 0x48fd8e5f, 77 },
    { 0x93bd8e60, 23 },
    { 0, 0 },
    { 0x80fd8e63, 61 },
    { 0x2ddd8e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26d8e68, 91 },
    { 0xc5cd8e69, 61 },
    { 0xd20d8e69, 90 },
    { 0x88cd8e6b, 52 },
    { 0x90dd8e69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d8e6f, 95 },
    { 0xe36d8e6f, 90 },
    { 0x3d4d8e71, 88 },
    { 0x6bdd8e72, 84 },
    { 0xad5d8e71, 98 },
    { 0x33bd8e71, 33 },
    { 0x710d8e70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d8e7c, 97 },
    { 0x158d8e7d, 61 },
    { 0x47d8e7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d8e81, 61 },
    { 0xb36d8e82, 61 },
    { 0x2b8d8e83, 95 },
    { 0, 0 },
    { 0x41dd8e85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ad8e89, 55 },
    { 0, 0 },
    { 0xd23d8e8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2d8e91, 33 },
    { 0x12dd8e91, 95 },
    { 0xeabd8e93, 91 },
    { 0xa20d8e93, 61 },
    { 0xcc6d8e95, 33 },
    { 0xa1fd8e95, 33 },
    { 0, 0 },
    { 0x375d8e98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd8e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fd8ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd8ea3, 100 },
    { 0x4b3d8ea3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd8eab, 68 },
    { 0x1aed8eab, 23 },
    { 0xf1d8eab, 98 },
    { 0, 0 },
    { 0xc42d8eaf, 97 },
    { 0, 0 },
    { 0x79fd8eb1, 40 },
    { 0, 0 },
    { 0x8afd8eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67d8eb6, 33 },
    { 0xd3dd8eb6, 61 },
    { 0x71ed8eb8, 61 },
    { 0, 0 },
    { 0xfd3d8eba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d8ebd, 98 },
    { 0x461d8ebd, 43 },
    { 0x91d8ebf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41d8ec2, 93 },
    { 0, 0 },
    { 0xa6ed8ec4, 61 },
    { 0xd1bd8ec5, 77 },
    { 0x76fd8ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd8eca, 61 },
    { 0x7e6d8ecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x492d8ece, 90 },
    { 0x9b5d8ecf, 98 },
    { 0xaecd8ed0, 78 },
    { 0x8e2d8ed0, 70 },
    { 0x354d8ed0, 33 },
    { 0x33fd8ed3, 84 },
    { 0xb5bd8ece, 88 },
    { 0x203d8ed5, 77 },
    { 0x7add8ed6, 61 },
    { 0x580d8ed7, 76 },
    { 0xcb1d8ed8, 91 },
    { 0xafed8ed8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd8edd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1d8ee0, 52 },
    { 0x8bad8ee1, 90 },
    { 0xf63d8ee1, 33 },
    { 0, 0 },
    { 0xcecd8ee4, 95 },
    { 0x118d8ee5, 61 },
    { 0x4cdd8ee6, 90 },
    { 0xe87d8ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347d8eed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd8ef0, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd8ef3, 95 },
    { 0x47ad8ef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708d8efb, 52 },
    { 0, 0 },
    { 0xf36d8efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dd8f01, 33 },
    { 0, 0 },
    { 0x552d8f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ed8f0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x371d8f0d, 76 },
    { 0x759d8f0e, 76 },
    { 0xc46d8f0f, 90 },
    { 0, 0 },
    { 0x928d8f11, 81 },
    { 0, 0 },
    { 0x872d8f13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939d8f18, 90 },
    { 0x59ed8f19, 95 },
    { 0xc12d8f18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d8f1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd8f26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d8f2a, 84 },
    { 0xb2bd8f2b, 88 },
    { 0xe49d8f2a, 46 },
    { 0xe51d8f2c, 36 },
    { 0xa1ed8f2e, 52 },
    { 0xb79d8f2f, 56 },
    { 0x81cd8f2e, 98 },
    { 0xf30d8f2f, 100 },
    { 0, 0 },
    { 0x2c1d8f33, 85 },
    { 0xacad8f34, 34 },
    { 0xd49d8f35, 84 },
    { 0x5fbd8f35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ad8f3a, 98 },
    { 0x3afd8f3b, 34 },
    { 0xd12d8f3c, 61 },
    { 0, 0 },
    { 0x4c9d8f3e, 98 },
    { 0, 0 },
    { 0x166d8f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6d8f44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd8f49, 61 },
    { 0, 0 },
    { 0x20fd8f4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1d8f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd8f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd8f55, 98 },
    { 0xd9bd8f56, 98 },
    { 0xe18d8f57, 91 },
    { 0, 0 },
    { 0x488d8f59, 52 },
    { 0xa60d8f59, 43 },
    { 0xe0ad8f5b, 33 },
    { 0, 0 },
    { 0x930d8f5d, 95 },
    { 0x5a5d8f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaead8f65, 87 },
    { 0x9fcd8f66, 61 },
    { 0, 0 },
    { 0x1abd8f68, 23 },
    { 0, 0 },
    { 0xff1d8f6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8f6d, 25 },
    { 0, 0 },
    { 0x76dd8f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d8f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0d8f77, 95 },
    { 0, 0 },
    { 0xda4d8f79, 90 },
    { 0x8cad8f79, 90 },
    { 0x3e0d8f7b, 90 },
    { 0x353d8f7c, 33 },
    { 0x23ad8f7a, 61 },
    { 0, 0 },
    { 0xc8ad8f7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dd8f8e, 61 },
    { 0, 0 },
    { 0x801d8f90, 90 },
    { 0x538d8f91, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed8f97, 95 },
    { 0x62fd8f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796d8f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4d8f9f, 33 },
    { 0xb1ed8fa0, 47 },
    { 0xcad8fa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecd8fa6, 33 },
    { 0xbfad8fa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68d8fab, 43 },
    { 0, 0 },
    { 0x170d8fad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5d8fb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed8fb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d8fb7, 76 },
    { 0x87bd8fb7, 61 },
    { 0, 0 },
    { 0x411d8fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354d8fc0, 23 },
    { 0, 0 },
    { 0x7efd8fc2, 55 },
    { 0x368d8fc3, 44 },
    { 0x255d8fc3, 93 },
    { 0x60dd8fc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd8fcf, 66 },
    { 0x4f7d8fd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999d8fdb, 68 },
    { 0, 0 },
    { 0x1f6d8fdd, 34 },
    { 0x58dd8fdd, 23 },
    { 0x16fd8fdf, 56 },
    { 0xe51d8fe0, 23 },
    { 0x58ad8fdf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7d8fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925d8fe8, 90 },
    { 0xf61d8fe9, 61 },
    { 0, 0 },
    { 0xe2dd8feb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dd8ffa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d8ffd, 90 },
    { 0, 0 },
    { 0x589d8fff, 33 },
    { 0, 0 },
    { 0x49d9001, 95 },
    { 0x4a5d9001, 75 },
    { 0x2ead9002, 95 },
    { 0xf5fd9001, 61 },
    { 0x6c5d9005, 91 },
    { 0x203d9006, 91 },
    { 0, 0 },
    { 0xd76d9008, 90 },
    { 0, 0 },
    { 0x9fd900a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ed900e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d9016, 52 },
    { 0x8ead9016, 61 },
    { 0x939d9018, 23 },
    { 0, 0 },
    { 0x652d901a, 93 },
    { 0x771d901b, 34 },
    { 0xb7ad901c, 33 },
    { 0x86d901c, 23 },
    { 0x42dd901d, 55 },
    { 0xc42d901b, 90 },
    { 0xdead901c, 53 },
    { 0, 0 },
    { 0x534d9022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd9026, 33 },
    { 0x63ad9027, 23 },
    { 0x837d9028, 34 },
    { 0xa4ed9027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0d902e, 53 },
    { 0, 0 },
    { 0x82ad9030, 66 },
    { 0x5ead9031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d9036, 91 },
    { 0x22fd9037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d9041, 99 },
    { 0x4d4d9042, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ed9045, 98 },
    { 0x629d9046, 61 },
    { 0, 0 },
    { 0xe26d9048, 95 },
    { 0, 0 },
    { 0xae3d904a, 100 },
    { 0xeb7d904b, 68 },
    { 0, 0 },
    { 0xc1fd904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fd9051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d9058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd905b, 34 },
    { 0x213d905c, 61 },
    { 0, 0 },
    { 0x1fdd905e, 55 },
    { 0x4dfd905f, 99 },
    { 0xa68d9060, 43 },
    { 0x639d9060, 78 },
    { 0, 0 },
    { 0xd79d9063, 61 },
    { 0x893d9063, 61 },
    { 0, 0 },
    { 0x442d9066, 87 },
    { 0, 0 },
    { 0x80cd9068, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d906b, 95 },
    { 0xbad906b, 98 },
    { 0, 0 },
    { 0x5b0d906e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2d9074, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed9078, 98 },
    { 0x99fd9079, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8d907c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd9080, 46 },
    { 0x9a6d9081, 78 },
    { 0x427d9082, 90 },
    { 0, 0 },
    { 0x2d6d9084, 90 },
    { 0xb6fd9084, 76 },
    { 0xb84d9084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786d908a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d908e, 98 },
    { 0xbc5d908e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcd9094, 23 },
    { 0xd05d9095, 33 },
    { 0xd9d9095, 77 },
    { 0xc0dd9095, 33 },
    { 0x733d9095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cad909e, 33 },
    { 0xe38d909f, 95 },
    { 0xd5ed90a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ad90a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84d90a8, 33 },
    { 0xbe0d90a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84d90b2, 90 },
    { 0, 0 },
    { 0xb84d90b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d90bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d90c3, 23 },
    { 0x118d90c4, 64 },
    { 0xbb9d90c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d90c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fd90ce, 91 },
    { 0, 0 },
    { 0xc67d90d0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07d90d5, 98 },
    { 0, 0 },
    { 0xb9ed90d7, 95 },
    { 0x17cd90d8, 96 },
    { 0, 0 },
    { 0xc9fd90da, 52 },
    { 0, 0 },
    { 0x190d90dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d90e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bd90e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d90ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x257d90f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fd90f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabd9100, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d9103, 100 },
    { 0xb6ed9104, 69 },
    { 0xaa7d9105, 95 },
    { 0, 0 },
    { 0x3a6d9107, 93 },
    { 0xc6d9108, 90 },
    { 0x907d9109, 33 },
    { 0x2a0d910a, 90 },
    { 0x9add910b, 33 },
    { 0xdbcd9109, 33 },
    { 0, 0 },
    { 0x577d910e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd9111, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9d9117, 61 },
    { 0xdced9117, 96 },
    { 0xf3fd9119, 40 },
    { 0x523d911a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd911f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2d9123, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d9126, 98 },
    { 0xbf3d9127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0d912c, 33 },
    { 0, 0 },
    { 0x51fd912e, 93 },
    { 0x8afd912f, 97 },
    { 0x4d0d9130, 90 },
    { 0xfd9d9130, 90 },
    { 0x3c6d912e, 90 },
    { 0xb11d9130, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd9137, 46 },
    { 0x650d9137, 90 },
    { 0x26dd9137, 69 },
    { 0, 0 },
    { 0xe05d913b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ed913f, 33 },
    { 0xbedd913f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x936d9143, 76 },
    { 0xf65d9144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x509d9147, 95 },
    { 0x9e9d9148, 23 },
    { 0x237d9149, 95 },
    { 0x2b1d9149, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3d914d, 80 },
    { 0, 0 },
    { 0x879d914f, 98 },
    { 0x6edd9150, 43 },
    { 0x166d9151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4d915f, 61 },
    { 0x7c6d915f, 69 },
    { 0x782d9161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ad9168, 95 },
    { 0, 0 },
    { 0x5fad916a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d9172, 40 },
    { 0xa6ed9173, 68 },
    { 0x7efd9172, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed9178, 93 },
    { 0x64ed9179, 84 },
    { 0x601d917a, 67 },
    { 0x604d9179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bd9180, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd9184, 98 },
    { 0xe97d9185, 90 },
    { 0x424d9186, 33 },
    { 0xeb1d9187, 66 },
    { 0x77fd9184, 34 },
    { 0x5cfd9189, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded918c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9d9190, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7d9194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cd919a, 95 },
    { 0x7fcd919b, 34 },
    { 0x5bdd919c, 34 },
    { 0, 0 },
    { 0xdced919e, 55 },
    { 0, 0 },
    { 0xf67d91a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d91a3, 90 },
    { 0, 0 },
    { 0xb27d91a5, 76 },
    { 0xa36d91a5, 95 },
    { 0xe81d91a7, 23 },
    { 0x36ad91a7, 61 },
    { 0, 0 },
    { 0x97fd91aa, 90 },
    { 0x432d91ab, 90 },
    { 0, 0 },
    { 0xee9d91ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d91b1, 39 },
    { 0xe0dd91b2, 95 },
    { 0xe47d91b3, 23 },
    { 0x5b3d91b1, 46 },
    { 0, 0 },
    { 0x157d91b6, 95 },
    { 0xb66d91b7, 90 },
    { 0, 0 },
    { 0x338d91b9, 61 },
    { 0x282d91ba, 93 },
    { 0x137d91ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fd91c4, 76 },
    { 0x26fd91c5, 95 },
    { 0x7ead91c6, 23 },
    { 0, 0 },
    { 0x1d5d91c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72d91cb, 33 },
    { 0, 0 },
    { 0x205d91cd, 77 },
    { 0xbbbd91ce, 46 },
    { 0x5e4d91ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfd91d3, 43 },
    { 0x3b9d91d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d91d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89d91dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4d91df, 76 },
    { 0x115d91e0, 23 },
    { 0x1f6d91e1, 98 },
    { 0xcdad91e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fd91e9, 43 },
    { 0x43d91ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4d91ee, 34 },
    { 0x3d3d91ef, 46 },
    { 0xedfd91f0, 76 },
    { 0x80ed91f0, 95 },
    { 0x75d91f0, 90 },
    { 0xf04d91f3, 93 },
    { 0, 0 },
    { 0x30d91f5, 23 },
    { 0x999d91f6, 76 },
    { 0xc35d91f5, 95 },
    { 0, 0 },
    { 0xeccd91f9, 98 },
    { 0x5a9d91fa, 98 },
    { 0, 0 },
    { 0xa84d91fc, 61 },
    { 0, 0 },
    { 0x245d91fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd9202, 95 },
    { 0x5f7d9202, 96 },
    { 0xc9bd9203, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd9209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed9212, 98 },
    { 0x7e7d9213, 61 },
    { 0x890d9212, 61 },
    { 0xa4cd9215, 98 },
    { 0xed1d9212, 84 },
    { 0, 0 },
    { 0x627d9218, 91 },
    { 0x9c4d9218, 33 },
    { 0x2aed9219, 33 },
    { 0x719d9219, 78 },
    { 0x246d921c, 67 },
    { 0x880d921c, 23 },
    { 0x5b3d921d, 23 },
    { 0x726d921b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed9223, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dd9226, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04d9235, 95 },
    { 0, 0 },
    { 0xb7fd9237, 99 },
    { 0x302d9238, 95 },
    { 0x403d9239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3d923c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead9242, 55 },
    { 0, 0 },
    { 0x8f1d9244, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed9247, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x191d924a, 34 },
    { 0xabfd924b, 76 },
    { 0x943d924c, 78 },
    { 0x8fad924c, 90 },
    { 0xe60d924c, 55 },
    { 0x6e7d924f, 61 },
    { 0, 0 },
    { 0x41dd9251, 34 },
    { 0x5a0d9252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d9255, 93 },
    { 0xdbbd9255, 55 },
    { 0xa08d9256, 55 },
    { 0xcbdd9258, 95 },
    { 0, 0 },
    { 0xb35d925a, 99 },
    { 0, 0 },
    { 0xe4dd925c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ad925f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfd9268, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871d926e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad9271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9275, 61 },
    { 0x7a6d9276, 90 },
    { 0xf89d9277, 90 },
    { 0x5f6d9278, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bd927e, 99 },
    { 0, 0 },
    { 0xa40d9280, 23 },
    { 0x228d9281, 66 },
    { 0xf37d9282, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed928f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d9296, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd929a, 33 },
    { 0x692d929a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d929e, 90 },
    { 0, 0 },
    { 0x37cd92a0, 93 },
    { 0xbbbd92a0, 34 },
    { 0, 0 },
    { 0x825d92a3, 87 },
    { 0x9fcd92a4, 52 },
    { 0x14dd92a5, 90 },
    { 0, 0 },
    { 0xd76d92a7, 33 },
    { 0xf80d92a8, 23 },
    { 0, 0 },
    { 0x97d92aa, 88 },
    { 0x438d92aa, 61 },
    { 0x860d92aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd92b1, 46 },
    { 0, 0 },
    { 0x26bd92b3, 93 },
    { 0, 0 },
    { 0x5e4d92b5, 43 },
    { 0, 0 },
    { 0xa24d92b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dd92bb, 76 },
    { 0x8fed92bc, 95 },
    { 0, 0 },
    { 0x98ad92be, 88 },
    { 0x270d92be, 90 },
    { 0x9ccd92c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x830d92c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d92c7, 61 },
    { 0x45dd92c8, 33 },
    { 0xeb3d92c8, 95 },
    { 0xf73d92c7, 98 },
    { 0x736d92cb, 93 },
    { 0x722d92cc, 81 },
    { 0x67cd92cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d92d3, 93 },
    { 0x6c1d92d4, 33 },
    { 0x84dd92d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531d92d9, 66 },
    { 0, 0 },
    { 0x82dd92db, 98 },
    { 0, 0 },
    { 0xdc0d92dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd92e4, 77 },
    { 0, 0 },
    { 0x9dbd92e6, 33 },
    { 0, 0 },
    { 0xd3dd92e8, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fd92ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d92ef, 91 },
    { 0x1a5d92f0, 61 },
    { 0x226d92ef, 76 },
    { 0xd10d92f2, 90 },
    { 0x259d92f3, 65 },
    { 0x161d92f2, 90 },
    { 0x8ccd92f3, 33 },
    { 0x3cad92ef, 95 },
    { 0xf9ed92f7, 95 },
    { 0xff1d92f8, 80 },
    { 0xc59d92f9, 91 },
    { 0x4dad92f9, 100 },
    { 0x382d92f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3d9306, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217d930b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d930f, 46 },
    { 0, 0 },
    { 0x611d9311, 76 },
    { 0, 0 },
    { 0x36d9313, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d9318, 33 },
    { 0, 0 },
    { 0x86cd931a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856d931e, 84 },
    { 0xcf0d931e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d9325, 98 },
    { 0, 0 },
    { 0x21dd9327, 55 },
    { 0xc70d9328, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2d932b, 100 },
    { 0x25fd932b, 99 },
    { 0, 0 },
    { 0xda0d932e, 90 },
    { 0xf06d932f, 95 },
    { 0xc34d9330, 95 },
    { 0xac8d9330, 75 },
    { 0, 0 },
    { 0xfbd9333, 23 },
    { 0x451d9333, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d9337, 95 },
    { 0x84ed9338, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafd933c, 61 },
    { 0xb1cd933c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bd9342, 91 },
    { 0x3e7d9342, 23 },
    { 0x667d9344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d9348, 85 },
    { 0xb59d9349, 90 },
    { 0x7bbd9348, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fd934d, 34 },
    { 0x21dd934e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x924d9351, 49 },
    { 0xdc9d9352, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d935a, 61 },
    { 0xdbfd935b, 76 },
    { 0xa96d935a, 23 },
    { 0xfe9d935d, 96 },
    { 0x4f8d935c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d9364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd9369, 33 },
    { 0, 0 },
    { 0x496d936b, 43 },
    { 0x7d5d936c, 90 },
    { 0x178d936d, 33 },
    { 0xfb9d936c, 95 },
    { 0x4a2d936f, 95 },
    { 0xab8d9370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3d9375, 93 },
    { 0xcacd9376, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ad9381, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d9386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9d9392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d939f, 39 },
    { 0xea2d93a0, 23 },
    { 0x800d93a0, 64 },
    { 0, 0 },
    { 0xa62d93a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639d93a9, 46 },
    { 0x626d93a9, 55 },
    { 0x1d6d93ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d93af, 90 },
    { 0x63cd93b0, 98 },
    { 0x7f6d93b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6d93b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919d93b8, 46 },
    { 0x8d93b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d93c1, 34 },
    { 0xcccd93c1, 23 },
    { 0x9ccd93c1, 75 },
    { 0xdb1d93c4, 34 },
    { 0x62ed93c5, 61 },
    { 0xdffd93c6, 83 },
    { 0x779d93c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8d93ca, 95 },
    { 0x1b2d93cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d93ce, 98 },
    { 0xfcbd93cf, 34 },
    { 0, 0 },
    { 0x3d2d93d1, 100 },
    { 0xe81d93d2, 61 },
    { 0xfcfd93d2, 90 },
    { 0x326d93d4, 33 },
    { 0xf61d93d3, 55 },
    { 0, 0 },
    { 0xc31d93d7, 33 },
    { 0xce2d93d8, 88 },
    { 0x49bd93d7, 75 },
    { 0, 0 },
    { 0x571d93db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd93e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd93e7, 23 },
    { 0x989d93e8, 55 },
    { 0x137d93e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ed93f1, 95 },
    { 0x5aad93f2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ed93f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d93fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cd9400, 98 },
    { 0x7ed9400, 98 },
    { 0x990d9402, 76 },
    { 0x473d9402, 98 },
    { 0x1bd9402, 90 },
    { 0, 0 },
    { 0x21d9406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d9409, 98 },
    { 0, 0 },
    { 0x709d940b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ad9411, 80 },
    { 0x39bd9412, 91 },
    { 0, 0 },
    { 0x3e1d9414, 61 },
    { 0xaf1d9415, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdd9422, 76 },
    { 0, 0 },
    { 0x5cd9424, 91 },
    { 0x379d9425, 95 },
    { 0, 0 },
    { 0x425d9427, 17 },
    { 0x8e3d9427, 76 },
    { 0xf5ed9429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bd942c, 77 },
    { 0xb5cd942d, 23 },
    { 0, 0 },
    { 0xc68d942f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d9432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7d9436, 55 },
    { 0, 0 },
    { 0x7efd9438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd943b, 95 },
    { 0, 0 },
    { 0x9f8d943d, 76 },
    { 0x663d943e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730d9446, 61 },
    { 0xee9d9447, 95 },
    { 0xb21d9446, 95 },
    { 0, 0 },
    { 0xb1d944a, 55 },
    { 0x856d944a, 90 },
    { 0x1e8d944c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd9452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233d945b, 78 },
    { 0x158d945c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1d945f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fd9466, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dd946c, 98 },
    { 0x7c6d946d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9d9470, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408d9478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39d9480, 33 },
    { 0x74bd9481, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6d9485, 91 },
    { 0x27cd9486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1d9489, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x388d948c, 46 },
    { 0x6d6d948d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d9490, 81 },
    { 0x787d9490, 99 },
    { 0, 0 },
    { 0xa4cd9493, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d9499, 52 },
    { 0xd0dd9499, 61 },
    { 0, 0 },
    { 0xecbd949c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192d94a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94ab, 69 },
    { 0, 0 },
    { 0x2dd94ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890d94b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d94b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafd94ba, 95 },
    { 0xf1dd94ba, 90 },
    { 0x8f7d94bc, 38 },
    { 0x5c3d94bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4d94c0, 67 },
    { 0x5cdd94c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584d94c5, 33 },
    { 0x338d94c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x376d94c9, 34 },
    { 0x1bdd94ca, 45 },
    { 0, 0 },
    { 0x6a9d94cc, 33 },
    { 0xfa6d94cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171d94d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed94d5, 76 },
    { 0x710d94d6, 91 },
    { 0xc32d94d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d94db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd94e0, 67 },
    { 0x46ad94e1, 17 },
    { 0xb80d94e2, 91 },
    { 0xd85d94e3, 87 },
    { 0, 0 },
    { 0xd0ad94e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d94eb, 23 },
    { 0, 0 },
    { 0xb02d94ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ad94f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd94f3, 61 },
    { 0x970d94f4, 33 },
    { 0, 0 },
    { 0x45ed94f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd94fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecd9505, 33 },
    { 0xc00d9505, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d950f, 23 },
    { 0, 0 },
    { 0xdf0d9511, 61 },
    { 0x5efd9512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfd9515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd951c, 67 },
    { 0x39dd951d, 95 },
    { 0xe0cd951d, 61 },
    { 0x3bad951d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141d9524, 67 },
    { 0xbe7d9525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x948d9528, 33 },
    { 0xc78d9528, 98 },
    { 0xf25d952a, 98 },
    { 0x17dd9528, 43 },
    { 0x1a7d9529, 61 },
    { 0xdad952a, 61 },
    { 0x99dd952e, 23 },
    { 0xa40d9528, 95 },
    { 0xf05d952d, 96 },
    { 0, 0 },
    { 0x14cd9532, 88 },
    { 0, 0 },
    { 0x6fdd9534, 90 },
    { 0x832d9534, 76 },
    { 0xe29d9534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d953e, 99 },
    { 0x223d953f, 55 },
    { 0, 0 },
    { 0x995d9541, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d9544, 81 },
    { 0, 0 },
    { 0xfe4d9546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dd954e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad9553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x448d9556, 33 },
    { 0, 0 },
    { 0x2dbd9558, 55 },
    { 0, 0 },
    { 0xdb0d955a, 98 },
    { 0xafd955b, 90 },
    { 0xe10d955c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedd955f, 55 },
    { 0x104d955f, 46 },
    { 0x689d9560, 43 },
    { 0xc0cd955f, 91 },
    { 0xf0fd955f, 98 },
    { 0xead9564, 93 },
    { 0, 0 },
    { 0xe30d9566, 90 },
    { 0, 0 },
    { 0xdd0d9568, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73d956e, 55 },
    { 0x1dbd956f, 52 },
    { 0x7f6d9570, 76 },
    { 0, 0 },
    { 0x53cd9572, 93 },
    { 0xdeed9573, 90 },
    { 0x27ed9574, 33 },
    { 0x424d9575, 78 },
    { 0x68bd9575, 68 },
    { 0xf99d9575, 90 },
    { 0x3e2d9573, 77 },
    { 0x57bd9579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2d957d, 23 },
    { 0x930d957e, 75 },
    { 0x5b0d957f, 33 },
    { 0x6cd957f, 25 },
    { 0, 0 },
    { 0x60bd9582, 33 },
    { 0x8d7d9582, 33 },
    { 0, 0 },
    { 0x41dd9585, 34 },
    { 0, 0 },
    { 0x76cd9587, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d958a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8d958f, 93 },
    { 0xc03d9590, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad9593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ad9598, 88 },
    { 0x494d9598, 23 },
    { 0x702d959a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fd959f, 90 },
    { 0x1bbd95a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6d95a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d95a9, 61 },
    { 0, 0 },
    { 0x441d95ab, 38 },
    { 0x9e6d95ab, 95 },
    { 0x17ad95ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d95b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70d95bd, 100 },
    { 0, 0 },
    { 0x532d95bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ed95c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d95c6, 33 },
    { 0x2efd95c7, 33 },
    { 0xd1d95c8, 77 },
    { 0, 0 },
    { 0x34cd95ca, 98 },
    { 0x9c4d95cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d95d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48d95d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ed95d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d95de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd95e1, 95 },
    { 0x11cd95e2, 52 },
    { 0, 0 },
    { 0x863d95e4, 95 },
    { 0, 0 },
    { 0x6ced95e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd95ec, 95 },
    { 0xe9d95ed, 93 },
    { 0x49bd95ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d95f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d95f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d95f9, 56 },
    { 0, 0 },
    { 0x690d95fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6d9602, 76 },
    { 0x5c5d9602, 61 },
    { 0xbc3d9602, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3d960c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14d960f, 89 },
    { 0, 0 },
    { 0xc00d9611, 23 },
    { 0xda4d9612, 95 },
    { 0x5b2d9613, 77 },
    { 0, 0 },
    { 0x4dad9615, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd9618, 98 },
    { 0x14ad9619, 33 },
    { 0, 0 },
    { 0x296d961b, 92 },
    { 0x667d961c, 77 },
    { 0x81cd961c, 75 },
    { 0xb80d961e, 61 },
    { 0xc9dd961d, 12 },
    { 0x3f4d9620, 76 },
    { 0x5d4d9621, 76 },
    { 0, 0 },
    { 0xa7ad9623, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d9626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d962c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d962f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133d9636, 76 },
    { 0, 0 },
    { 0x165d9638, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825d9643, 90 },
    { 0, 0 },
    { 0x204d9645, 95 },
    { 0x5ded9646, 33 },
    { 0x4bad9646, 100 },
    { 0x767d9645, 78 },
    { 0xdddd9649, 95 },
    { 0, 0 },
    { 0xeead964b, 90 },
    { 0x35cd964c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674d9650, 36 },
    { 0, 0 },
    { 0x90cd9652, 33 },
    { 0x76d9653, 46 },
    { 0x358d9654, 61 },
    { 0xd03d9652, 78 },
    { 0xb5dd9652, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x650d9659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d965d, 81 },
    { 0xf3d965e, 67 },
    { 0xf08d965e, 52 },
    { 0xa22d9660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1d9663, 46 },
    { 0xd49d9664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dd9669, 87 },
    { 0x224d966a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696d966f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd9672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08d9679, 94 },
    { 0xf88d967a, 77 },
    { 0, 0 },
    { 0xcd3d967c, 95 },
    { 0, 0 },
    { 0x263d967e, 43 },
    { 0, 0 },
    { 0x83d9680, 76 },
    { 0, 0 },
    { 0x260d9682, 61 },
    { 0x1f3d9683, 90 },
    { 0xd42d9682, 34 },
    { 0x430d9685, 95 },
    { 0x804d9686, 99 },
    { 0, 0 },
    { 0xdc5d9688, 61 },
    { 0x8cd9689, 95 },
    { 0x8a4d9689, 33 },
    { 0, 0 },
    { 0x15ed968c, 90 },
    { 0, 0 },
    { 0xf90d968e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd9692, 76 },
    { 0x698d9692, 34 },
    { 0xce8d9694, 33 },
    { 0, 0 },
    { 0x604d9696, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d969b, 90 },
    { 0x7e2d969c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad969f, 77 },
    { 0x6cad96a0, 76 },
    { 0xa2cd96a1, 99 },
    { 0xd6fd969f, 46 },
    { 0, 0 },
    { 0x476d96a4, 76 },
    { 0x184d96a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edd96ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d96b1, 84 },
    { 0xc51d96b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0d96b9, 23 },
    { 0xbe1d96b9, 33 },
    { 0x9add96bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd96c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d96cc, 90 },
    { 0, 0 },
    { 0xe9d96ce, 90 },
    { 0x93ed96ce, 33 },
    { 0, 0 },
    { 0xdcad96d1, 61 },
    { 0xb5d96d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d96d9, 43 },
    { 0x89dd96d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858d96de, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d96e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fd96e7, 90 },
    { 0x680d96e8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675d96f1, 38 },
    { 0x979d96f2, 46 },
    { 0xfc9d96f3, 81 },
    { 0, 0 },
    { 0xfb1d96f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced96f8, 95 },
    { 0x21fd96f9, 85 },
    { 0x452d96fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cd96fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60d9702, 52 },
    { 0, 0 },
    { 0x940d9704, 91 },
    { 0, 0 },
    { 0x11dd9706, 95 },
    { 0, 0 },
    { 0x391d9708, 95 },
    { 0, 0 },
    { 0x47fd970a, 100 },
    { 0, 0 },
    { 0xc8cd970c, 67 },
    { 0x620d970c, 67 },
    { 0x557d970e, 33 },
    { 0xdbad970e, 23 },
    { 0xe8ed970d, 90 },
    { 0x286d9711, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ed9714, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced9719, 98 },
    { 0x87dd971a, 34 },
    { 0x572d971b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d971e, 61 },
    { 0xdbfd971f, 61 },
    { 0x1bbd971f, 33 },
    { 0x48ed9721, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad9724, 33 },
    { 0x4d3d9725, 95 },
    { 0x58d9726, 61 },
    { 0x1b7d9727, 61 },
    { 0x904d9727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7d972b, 98 },
    { 0xf8fd972c, 61 },
    { 0, 0 },
    { 0xa3fd972e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ed973a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x473d973d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1d9740, 33 },
    { 0x304d9741, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed9745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d9752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x412d9755, 77 },
    { 0x194d9756, 52 },
    { 0, 0 },
    { 0xf43d9758, 98 },
    { 0, 0 },
    { 0xf27d975a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18d975e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134d9762, 55 },
    { 0x307d9763, 34 },
    { 0, 0 },
    { 0xaf0d9765, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47d9768, 95 },
    { 0xff7d9768, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d9775, 98 },
    { 0, 0 },
    { 0x6c5d9777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d977b, 76 },
    { 0x98dd977b, 61 },
    { 0x383d977d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dd9783, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7d978a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ed978f, 84 },
    { 0, 0 },
    { 0x13bd9791, 80 },
    { 0xbd2d9792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9798, 61 },
    { 0x63cd9799, 95 },
    { 0x195d979a, 90 },
    { 0xc16d979b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51d97a0, 95 },
    { 0x246d97a1, 68 },
    { 0x45fd97a2, 65 },
    { 0xc6ed97a2, 100 },
    { 0xe3d97a2, 61 },
    { 0x1cfd97a5, 23 },
    { 0x9acd97a5, 90 },
    { 0xcd0d97a7, 55 },
    { 0xa40d97a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedd97ab, 69 },
    { 0x5e5d97ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd97af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0d97b3, 78 },
    { 0x7e0d97b4, 65 },
    { 0x66ad97b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d97ba, 98 },
    { 0xa66d97ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d97bf, 76 },
    { 0xbb6d97c0, 87 },
    { 0x291d97c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd97c4, 93 },
    { 0xad1d97c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d97c9, 90 },
    { 0x8fad97ca, 61 },
    { 0, 0 },
    { 0x6b1d97cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6d97cf, 61 },
    { 0, 0 },
    { 0x98fd97d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaced97d4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d97d8, 55 },
    { 0x2b5d97d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d97e0, 99 },
    { 0x4add97e1, 34 },
    { 0x1a8d97e2, 99 },
    { 0x71d97e3, 83 },
    { 0x45cd97e2, 53 },
    { 0, 0 },
    { 0x7d97e6, 66 },
    { 0, 0 },
    { 0x73d97e8, 61 },
    { 0x684d97e9, 90 },
    { 0x2b7d97e9, 93 },
    { 0, 0 },
    { 0x7cad97ec, 65 },
    { 0x24dd97ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd97f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d97f7, 94 },
    { 0, 0 },
    { 0x5cbd97f9, 91 },
    { 0, 0 },
    { 0x4dbd97fb, 90 },
    { 0x784d97fc, 91 },
    { 0xfdad97fd, 55 },
    { 0xaa7d97fe, 33 },
    { 0, 0 },
    { 0xa5dd9800, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d9803, 95 },
    { 0xb51d9804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x432d9807, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d980a, 34 },
    { 0x7d2d980b, 61 },
    { 0x623d980c, 90 },
    { 0x2a1d980c, 95 },
    { 0, 0 },
    { 0xd01d980f, 33 },
    { 0, 0 },
    { 0xbb0d9811, 90 },
    { 0, 0 },
    { 0x29ed9813, 33 },
    { 0x815d9814, 36 },
    { 0xb50d9815, 43 },
    { 0x1b9d9814, 46 },
    { 0x321d9816, 33 },
    { 0x9bad9818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd981b, 90 },
    { 0, 0 },
    { 0xf20d981d, 55 },
    { 0x3a0d981e, 55 },
    { 0, 0 },
    { 0x9b7d9820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d9829, 76 },
    { 0x431d982a, 93 },
    { 0, 0 },
    { 0x19fd982c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ad9834, 33 },
    { 0xab5d9834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d983a, 100 },
    { 0x95ed983b, 93 },
    { 0, 0 },
    { 0xeb9d983d, 100 },
    { 0xb8cd983e, 91 },
    { 0x78cd983d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fd9843, 90 },
    { 0, 0 },
    { 0xf4ed9845, 33 },
    { 0x56fd9845, 33 },
    { 0, 0 },
    { 0xa2ed9848, 98 },
    { 0, 0 },
    { 0xd4d984a, 23 },
    { 0xb0dd984b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7d9853, 90 },
    { 0, 0 },
    { 0xd21d9855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606d985b, 90 },
    { 0x283d985b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d9861, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d9864, 34 },
    { 0xe75d9865, 98 },
    { 0xcb4d9866, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d986b, 78 },
    { 0xeb4d986c, 46 },
    { 0x922d986d, 61 },
    { 0xa6dd986d, 23 },
    { 0x580d986e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613d9873, 34 },
    { 0x97ed9874, 76 },
    { 0x4bbd9873, 23 },
    { 0x31d9876, 93 },
    { 0xa7d9873, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d987a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2d987e, 96 },
    { 0x423d987e, 23 },
    { 0xfefd9880, 76 },
    { 0x8bfd9881, 61 },
    { 0, 0 },
    { 0xea3d9883, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d9887, 61 },
    { 0, 0 },
    { 0x60ad9889, 23 },
    { 0, 0 },
    { 0x465d988b, 95 },
    { 0, 0 },
    { 0xab3d988d, 77 },
    { 0x230d988e, 14 },
    { 0x256d988f, 33 },
    { 0x52bd988f, 23 },
    { 0x6a4d9891, 99 },
    { 0xbd7d988e, 55 },
    { 0x639d9893, 23 },
    { 0, 0 },
    { 0xc43d9895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd9899, 98 },
    { 0xc00d989a, 89 },
    { 0xe06d989b, 90 },
    { 0x169d9899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fd989f, 91 },
    { 0xe11d989f, 90 },
    { 0x2fdd98a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d98a5, 33 },
    { 0x729d98a6, 80 },
    { 0x515d98a6, 76 },
    { 0x6acd98a8, 23 },
    { 0x445d98a9, 76 },
    { 0, 0 },
    { 0x164d98ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d98b1, 33 },
    { 0, 0 },
    { 0x8bad98b3, 65 },
    { 0x672d98b4, 61 },
    { 0x7b4d98b4, 23 },
    { 0x9dbd98b4, 76 },
    { 0xafcd98b7, 23 },
    { 0x42ed98b8, 95 },
    { 0x413d98b9, 90 },
    { 0xacbd98b6, 46 },
    { 0, 0 },
    { 0xd77d98bc, 66 },
    { 0xe33d98bc, 55 },
    { 0, 0 },
    { 0x72dd98bf, 33 },
    { 0, 0 },
    { 0x857d98c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5d98c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd98ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38d98ce, 95 },
    { 0, 0 },
    { 0x87bd98d0, 95 },
    { 0x2a3d98d1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fd98d9, 91 },
    { 0x197d98d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56d98dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8add98e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6d98e8, 98 },
    { 0x59cd98e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefd98ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddd98f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d98f4, 23 },
    { 0x2e4d98f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d98fc, 98 },
    { 0x114d98fc, 17 },
    { 0xfa1d98fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bad9907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed990d, 76 },
    { 0, 0 },
    { 0x6bfd990f, 95 },
    { 0x97bd9910, 90 },
    { 0, 0 },
    { 0x788d9912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd9917, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd991a, 91 },
    { 0xb7bd991b, 76 },
    { 0x375d991b, 90 },
    { 0xb5d991b, 33 },
    { 0xf02d991e, 33 },
    { 0, 0 },
    { 0xfddd9920, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198d9924, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74d9928, 33 },
    { 0xbbcd9928, 61 },
    { 0xe53d9929, 90 },
    { 0, 0 },
    { 0x96d992c, 61 },
    { 0, 0 },
    { 0x93cd992e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bd9931, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d9936, 90 },
    { 0xfd0d9937, 90 },
    { 0x457d9936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1d993b, 61 },
    { 0xd5ed993b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ad993f, 23 },
    { 0, 0 },
    { 0x149d9941, 33 },
    { 0x997d9941, 95 },
    { 0x694d9942, 98 },
    { 0, 0 },
    { 0xd70d9945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcd994a, 98 },
    { 0xa1dd994a, 8 },
    { 0, 0 },
    { 0x4c8d994d, 76 },
    { 0, 0 },
    { 0xe57d994f, 55 },
    { 0, 0 },
    { 0x5bbd9951, 23 },
    { 0xd6dd9952, 90 },
    { 0x38dd9951, 99 },
    { 0, 0 },
    { 0x828d9955, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40d995a, 98 },
    { 0, 0 },
    { 0x2d3d995c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d995f, 23 },
    { 0, 0 },
    { 0x229d9961, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efd996a, 85 },
    { 0x938d996b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d996f, 13 },
    { 0xbfed996f, 33 },
    { 0x4c2d9971, 100 },
    { 0x12ed9972, 55 },
    { 0x37bd9972, 76 },
    { 0xbcdd9972, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11d9977, 33 },
    { 0, 0 },
    { 0x4b3d9979, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d997e, 69 },
    { 0x6d4d997f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83d9988, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d998b, 23 },
    { 0x9e3d998c, 60 },
    { 0, 0 },
    { 0xb0ed998e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26d9992, 90 },
    { 0x644d9993, 34 },
    { 0x75fd9993, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d999e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572d99a4, 23 },
    { 0, 0 },
    { 0x108d99a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65d99b1, 95 },
    { 0x912d99b2, 55 },
    { 0x2b0d99b3, 68 },
    { 0xf15d99b2, 90 },
    { 0x4dad99b5, 76 },
    { 0x14ed99b6, 43 },
    { 0, 0 },
    { 0xdc7d99b8, 61 },
    { 0, 0 },
    { 0x5a5d99ba, 46 },
    { 0, 0 },
    { 0x12ed99bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dd99bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd99c3, 33 },
    { 0x1a2d99c3, 23 },
    { 0x424d99c4, 93 },
    { 0x824d99c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd99ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d99d2, 101 },
    { 0, 0 },
    { 0xa40d99d4, 52 },
    { 0xceed99d5, 91 },
    { 0x940d99d6, 23 },
    { 0x824d99d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d99da, 98 },
    { 0x14d99db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d99de, 98 },
    { 0, 0 },
    { 0x8b7d99e0, 34 },
    { 0, 0 },
    { 0x735d99e2, 99 },
    { 0x897d99e2, 66 },
    { 0x5efd99e4, 78 },
    { 0x13dd99e5, 33 },
    { 0x7a1d99e4, 43 },
    { 0x9efd99e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ed99ea, 61 },
    { 0, 0 },
    { 0xa11d99ec, 90 },
    { 0xa1d99ed, 23 },
    { 0x16cd99ee, 66 },
    { 0, 0 },
    { 0xd38d99f0, 61 },
    { 0x919d99f1, 42 },
    { 0x74ed99f2, 33 },
    { 0x137d99f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8d99f8, 78 },
    { 0x11ed99f8, 90 },
    { 0xfa3d99f9, 91 },
    { 0xe4dd99fb, 23 },
    { 0x511d99f8, 76 },
    { 0x7cfd99fd, 55 },
    { 0x755d99fb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d9a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d9a09, 61 },
    { 0, 0 },
    { 0x582d9a0b, 55 },
    { 0, 0 },
    { 0x7b2d9a0d, 95 },
    { 0, 0 },
    { 0x34dd9a0f, 23 },
    { 0x3ffd9a0f, 76 },
    { 0xeead9a11, 91 },
    { 0, 0 },
    { 0x264d9a13, 23 },
    { 0xe14d9a14, 66 },
    { 0xa24d9a15, 96 },
    { 0x164d9a14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d9a1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dd9a22, 67 },
    { 0, 0 },
    { 0x6d5d9a24, 77 },
    { 0xeb7d9a25, 97 },
    { 0x441d9a25, 52 },
    { 0xce8d9a27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3d9a2c, 61 },
    { 0x69dd9a2c, 23 },
    { 0xff6d9a2c, 33 },
    { 0xa03d9a2e, 33 },
    { 0x515d9a2f, 61 },
    { 0x7efd9a2d, 66 },
    { 0x101d9a31, 61 },
    { 0x78ed9a31, 98 },
    { 0x5b9d9a32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3d9a3a, 87 },
    { 0, 0 },
    { 0x971d9a3c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d9a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9a42, 76 },
    { 0x488d9a43, 33 },
    { 0, 0 },
    { 0xe73d9a45, 90 },
    { 0xf81d9a45, 81 },
    { 0xb62d9a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dd9a4c, 61 },
    { 0, 0 },
    { 0x645d9a4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cd9a53, 91 },
    { 0x868d9a53, 46 },
    { 0xc95d9a55, 100 },
    { 0x4dfd9a54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401d9a5d, 95 },
    { 0xb2d9a5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd9a63, 76 },
    { 0xe87d9a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2d9a6b, 17 },
    { 0xfb4d9a6b, 34 },
    { 0xdddd9a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d9a74, 34 },
    { 0, 0 },
    { 0xa88d9a76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d9a7d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d9a86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d9a8d, 61 },
    { 0xbccd9a8e, 61 },
    { 0xda2d9a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d9a94, 76 },
    { 0xb27d9a94, 23 },
    { 0, 0 },
    { 0x6d0d9a97, 91 },
    { 0xc8bd9a97, 98 },
    { 0x4a4d9a97, 90 },
    { 0x71cd9a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fd9a9d, 87 },
    { 0x8dd9a9e, 91 },
    { 0x3fbd9a9e, 95 },
    { 0x34bd9aa0, 93 },
    { 0x39ed9aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67d9aa5, 90 },
    { 0xc2ad9aa6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed9aaa, 81 },
    { 0x643d9aaa, 55 },
    { 0xe8fd9aaa, 43 },
    { 0x38cd9aad, 95 },
    { 0x602d9aad, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824d9ab3, 34 },
    { 0xbd8d9ab4, 52 },
    { 0x476d9ab5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad9abc, 98 },
    { 0x2b3d9abd, 53 },
    { 0x672d9abe, 33 },
    { 0x1add9abf, 87 },
    { 0x46ed9ac0, 52 },
    { 0, 0 },
    { 0xa96d9ac2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d9acf, 75 },
    { 0x820d9acf, 76 },
    { 0xb85d9ad1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d9ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dd9ae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56d9ae8, 88 },
    { 0, 0 },
    { 0xeaed9aea, 91 },
    { 0xcd5d9aeb, 23 },
    { 0xe9ed9aec, 66 },
    { 0, 0 },
    { 0xc36d9aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d9af1, 76 },
    { 0x4b2d9af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d9b02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d9b05, 87 },
    { 0x3d5d9b06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d9b0a, 65 },
    { 0x753d9b0b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d9b11, 46 },
    { 0x6fdd9b12, 61 },
    { 0x17ed9b13, 93 },
    { 0x942d9b11, 66 },
    { 0xe9fd9b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85d9b1c, 95 },
    { 0, 0 },
    { 0x666d9b1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d9b21, 46 },
    { 0xb72d9b22, 61 },
    { 0x2c1d9b22, 90 },
    { 0x293d9b22, 61 },
    { 0x74ed9b23, 67 },
    { 0x6a2d9b22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d9b29, 90 },
    { 0, 0 },
    { 0xf1dd9b2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978d9b35, 90 },
    { 0xc71d9b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b3b, 75 },
    { 0xe7dd9b3c, 23 },
    { 0, 0 },
    { 0x6a2d9b3e, 95 },
    { 0x27bd9b3f, 77 },
    { 0x507d9b3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d9b43, 87 },
    { 0xc7ed9b44, 46 },
    { 0, 0 },
    { 0xe8bd9b46, 95 },
    { 0x7c7d9b46, 23 },
    { 0x66ed9b47, 90 },
    { 0x70bd9b49, 87 },
    { 0xcd7d9b4a, 33 },
    { 0x3fcd9b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36d9b56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aed9b5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x690d9b5d, 85 },
    { 0x85fd9b5e, 78 },
    { 0x1afd9b5f, 77 },
    { 0x4c3d9b5f, 61 },
    { 0xc6d9b61, 90 },
    { 0x638d9b5f, 61 },
    { 0x240d9b60, 84 },
    { 0x976d9b64, 95 },
    { 0x9f5d9b65, 34 },
    { 0x997d9b5e, 61 },
    { 0xa15d9b67, 23 },
    { 0xbfdd9b64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dd9b6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3d9b6e, 40 },
    { 0x869d9b6e, 95 },
    { 0xbfed9b70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d9b75, 61 },
    { 0, 0 },
    { 0x5f7d9b77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d9b7e, 23 },
    { 0xf6ed9b7f, 33 },
    { 0xfe3d9b80, 93 },
    { 0xd7ed9b81, 61 },
    { 0x776d9b82, 77 },
    { 0x7e2d9b7e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97d9b86, 76 },
    { 0x6a9d9b87, 41 },
    { 0xf9ed9b87, 98 },
    { 0xd8cd9b88, 90 },
    { 0xffcd9b8a, 98 },
    { 0x7ebd9b87, 90 },
    { 0, 0 },
    { 0x910d9b8d, 33 },
    { 0, 0 },
    { 0x2d8d9b8f, 66 },
    { 0, 0 },
    { 0x374d9b91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bd9b94, 85 },
    { 0, 0 },
    { 0xb92d9b96, 55 },
    { 0x72fd9b96, 58 },
    { 0x370d9b98, 23 },
    { 0x6cbd9b98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181d9b9e, 23 },
    { 0x1dad9b9e, 33 },
    { 0, 0 },
    { 0x746d9ba1, 98 },
    { 0x2ced9ba2, 98 },
    { 0x5ded9ba1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206d9bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d9bb0, 78 },
    { 0xbcd9bb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd9bb4, 52 },
    { 0x91ed9bb5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0d9bc3, 61 },
    { 0x9e7d9bc4, 98 },
    { 0xeead9bc4, 90 },
    { 0x671d9bc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22d9bca, 33 },
    { 0x81bd9bcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d9bcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d9bd8, 61 },
    { 0, 0 },
    { 0xa4ad9bda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd9be0, 66 },
    { 0x8d7d9be1, 76 },
    { 0xdadd9be2, 23 },
    { 0x133d9be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556d9be9, 23 },
    { 0x284d9bea, 93 },
    { 0x86ed9beb, 95 },
    { 0xdbcd9bec, 33 },
    { 0xa59d9bec, 33 },
    { 0xfb0d9bed, 43 },
    { 0xb69d9bef, 90 },
    { 0x6dfd9bf0, 95 },
    { 0, 0 },
    { 0x3a8d9bf2, 23 },
    { 0x49d9bf2, 40 },
    { 0xea6d9bf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d9bf7, 76 },
    { 0, 0 },
    { 0x675d9bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x756d9bfc, 91 },
    { 0xe07d9bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d9c00, 85 },
    { 0xd62d9c00, 95 },
    { 0, 0 },
    { 0x58dd9c03, 90 },
    { 0xff5d9c04, 23 },
    { 0x80dd9c04, 95 },
    { 0, 0 },
    { 0x6bed9c07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655d9c0e, 98 },
    { 0, 0 },
    { 0xa32d9c10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823d9c15, 90 },
    { 0xbfed9c16, 78 },
    { 0x5b9d9c15, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fd9c1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x388d9c1d, 99 },
    { 0, 0 },
    { 0x867d9c1f, 23 },
    { 0xd7fd9c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2d9c25, 23 },
    { 0xe8ad9c25, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d9c29, 55 },
    { 0x218d9c2a, 33 },
    { 0x57ad9c29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d9c2f, 80 },
    { 0x347d9c2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477d9c35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38d9c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735d9c46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257d9c4a, 98 },
    { 0, 0 },
    { 0x232d9c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bd9c52, 87 },
    { 0x2c4d9c53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd9c5b, 98 },
    { 0xa5d9c5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7d9c5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd9c62, 88 },
    { 0x1f4d9c62, 90 },
    { 0xc2fd9c64, 66 },
    { 0x3c4d9c65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d9c6b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfd9c6f, 77 },
    { 0, 0 },
    { 0xc8bd9c71, 52 },
    { 0xe39d9c72, 33 },
    { 0xe59d9c73, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x917d9c76, 53 },
    { 0x1f3d9c77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d9c7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x929d9c7d, 90 },
    { 0x545d9c7e, 91 },
    { 0xa1ad9c7f, 76 },
    { 0, 0 },
    { 0x98ed9c81, 95 },
    { 0x9c5d9c82, 88 },
    { 0x60dd9c81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d9c86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d9c89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d9c8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3d9c91, 61 },
    { 0, 0 },
    { 0x26dd9c93, 95 },
    { 0, 0 },
    { 0xdd5d9c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181d9c99, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x656d9c9c, 78 },
    { 0xa65d9c9d, 33 },
    { 0x1c7d9c9e, 84 },
    { 0x290d9c9c, 96 },
    { 0x52ed9ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ded9ca4, 43 },
    { 0xb94d9ca4, 76 },
    { 0, 0 },
    { 0x1fd9ca7, 34 },
    { 0x2ebd9ca8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23d9cb0, 90 },
    { 0x8f3d9cb1, 67 },
    { 0x9ecd9cb2, 33 },
    { 0xd81d9cb1, 98 },
    { 0xcc1d9cb4, 33 },
    { 0x703d9cb5, 33 },
    { 0x55dd9cb6, 36 },
    { 0x79dd9cb6, 90 },
    { 0x616d9cb4, 23 },
    { 0xb34d9cb4, 90 },
    { 0xfc2d9cb0, 95 },
    { 0, 0 },
    { 0xeb3d9cbc, 33 },
    { 0x53bd9cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71d9cc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3d9cc4, 23 },
    { 0x16cd9cc4, 90 },
    { 0x81dd9cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82d9cca, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bd9cd4, 90 },
    { 0xba8d9cd5, 99 },
    { 0x657d9cd4, 46 },
    { 0x2f6d9cd5, 55 },
    { 0, 0 },
    { 0x14ed9cd9, 90 },
    { 0x478d9cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2d9cdd, 33 },
    { 0xa87d9cde, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15d9ce2, 23 },
    { 0xcdfd9ce2, 90 },
    { 0x5b6d9ce4, 82 },
    { 0x72bd9ce5, 76 },
    { 0x220d9ce6, 93 },
    { 0, 0 },
    { 0xb47d9ce8, 34 },
    { 0xf74d9ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6d9cec, 98 },
    { 0xaf3d9ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d9cf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ad9cf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8d9d02, 23 },
    { 0x303d9d03, 34 },
    { 0x708d9d03, 98 },
    { 0, 0 },
    { 0x92cd9d06, 61 },
    { 0xe52d9d06, 93 },
    { 0xd2ad9d08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0d9d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd9d12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d9d21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd9d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd9d2b, 95 },
    { 0xd9fd9d2c, 95 },
    { 0x597d9d2b, 23 },
    { 0xf95d9d2d, 55 },
    { 0xdacd9d2c, 90 },
    { 0x25fd9d30, 33 },
    { 0x79ad9d2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4d9d39, 98 },
    { 0x4abd9d3a, 36 },
    { 0x7c5d9d3a, 46 },
    { 0, 0 },
    { 0xf88d9d3d, 76 },
    { 0x14d9d3e, 33 },
    { 0, 0 },
    { 0x694d9d40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cd9d46, 55 },
    { 0xdeed9d46, 76 },
    { 0xbaad9d47, 90 },
    { 0x7f7d9d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d9d4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd9d52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed9d58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd9d65, 33 },
    { 0xc09d9d66, 23 },
    { 0, 0 },
    { 0x240d9d68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bed9d6c, 23 },
    { 0xfcd9d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d9d70, 46 },
    { 0x75bd9d71, 90 },
    { 0x35fd9d72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d9d75, 33 },
    { 0, 0 },
    { 0x9e9d9d77, 55 },
    { 0x81ad9d78, 33 },
    { 0x736d9d79, 95 },
    { 0x2d9d9d7a, 33 },
    { 0, 0 },
    { 0x46ed9d7c, 34 },
    { 0, 0 },
    { 0xf15d9d7e, 76 },
    { 0xec9d9d7f, 90 },
    { 0x66ed9d80, 88 },
    { 0x83fd9d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fd9d8b, 61 },
    { 0x222d9d8c, 66 },
    { 0x63fd9d8d, 90 },
    { 0, 0 },
    { 0x7b2d9d8f, 76 },
    { 0x86fd9d90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ad9d94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d9d97, 43 },
    { 0, 0 },
    { 0x96ed9d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176d9d9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232d9da7, 95 },
    { 0x110d9da8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd9dad, 23 },
    { 0xc8dd9dae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35d9db2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3d9dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d9dc0, 46 },
    { 0x951d9dc0, 100 },
    { 0x85fd9dc1, 76 },
    { 0xb11d9dc3, 95 },
    { 0x377d9dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd9dc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d9dcd, 85 },
    { 0, 0 },
    { 0x82fd9dcf, 33 },
    { 0x801d9dcf, 90 },
    { 0x1b3d9dd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bd9dd4, 34 },
    { 0, 0 },
    { 0x82cd9dd6, 93 },
    { 0, 0 },
    { 0x7e7d9dd8, 33 },
    { 0, 0 },
    { 0xf73d9dda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5d9ddf, 33 },
    { 0x670d9de0, 34 },
    { 0x767d9de0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155d9de7, 33 },
    { 0, 0 },
    { 0x9eed9de9, 90 },
    { 0xa6ad9dea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd9df3, 65 },
    { 0x8c9d9df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2d9dfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49d9e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884d9e05, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73d9e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5d9e15, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dd9e1c, 61 },
    { 0, 0 },
    { 0x6aad9e1e, 33 },
    { 0x445d9e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aed9e23, 33 },
    { 0xb1ad9e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd9e27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d9e2d, 85 },
    { 0xdf4d9e2e, 90 },
    { 0x5b2d9e2d, 76 },
    { 0x7d7d9e30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbd9e37, 61 },
    { 0x4f3d9e37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdd9e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0d9e40, 43 },
    { 0x6b3d9e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d9e46, 75 },
    { 0xcadd9e47, 91 },
    { 0, 0 },
    { 0x3b5d9e49, 33 },
    { 0, 0 },
    { 0xc8fd9e4b, 23 },
    { 0xd86d9e4c, 61 },
    { 0xbc2d9e4d, 33 },
    { 0xe8cd9e4e, 55 },
    { 0x42dd9e4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced9e54, 95 },
    { 0x1bdd9e55, 33 },
    { 0x440d9e56, 91 },
    { 0, 0 },
    { 0xe01d9e58, 34 },
    { 0xe95d9e59, 66 },
    { 0xc15d9e5a, 78 },
    { 0x9cad9e5a, 98 },
    { 0x3b7d9e5b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd9e5f, 23 },
    { 0xcb3d9e60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd9e66, 61 },
    { 0, 0 },
    { 0xf1dd9e68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd9e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d9e6f, 83 },
    { 0x15ad9e70, 95 },
    { 0, 0 },
    { 0x52dd9e72, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd9e75, 61 },
    { 0x651d9e76, 33 },
    { 0, 0 },
    { 0xb28d9e78, 23 },
    { 0xb18d9e78, 95 },
    { 0x186d9e78, 98 },
    { 0x67cd9e7b, 100 },
    { 0x537d9e79, 87 },
    { 0, 0 },
    { 0x3dad9e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d9e81, 85 },
    { 0x891d9e81, 84 },
    { 0xd61d9e83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd9e87, 43 },
    { 0x62ed9e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd9e8f, 95 },
    { 0xbf5d9e90, 99 },
    { 0, 0 },
    { 0xf50d9e92, 61 },
    { 0xb1ed9e93, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4d9e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d9ea0, 43 },
    { 0x8c4d9ea1, 52 },
    { 0xffad9ea2, 90 },
    { 0, 0 },
    { 0x25d9ea4, 33 },
    { 0x96d9ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d9ea8, 78 },
    { 0x1bfd9ea9, 97 },
    { 0xded9eaa, 90 },
    { 0x6dad9eaa, 90 },
    { 0, 0 },
    { 0x96fd9ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecd9eb1, 66 },
    { 0xfa8d9eb2, 46 },
    { 0x1fcd9eb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xded9eb6, 34 },
    { 0, 0 },
    { 0x4b8d9eb8, 17 },
    { 0, 0 },
    { 0x25dd9eba, 33 },
    { 0xbc2d9ebb, 90 },
    { 0xa4cd9ebb, 23 },
    { 0xa41d9ebd, 81 },
    { 0xe2d9ebe, 33 },
    { 0x736d9ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677d9ec5, 43 },
    { 0, 0 },
    { 0x6abd9ec7, 90 },
    { 0x4c8d9ec7, 65 },
    { 0x6cd9ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1d9ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35d9ed1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d9ed4, 17 },
    { 0, 0 },
    { 0xdb5d9ed6, 33 },
    { 0x7e1d9ed7, 33 },
    { 0, 0 },
    { 0x1b8d9ed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebd9edf, 55 },
    { 0, 0 },
    { 0x74cd9ee1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd9ee5, 69 },
    { 0, 0 },
    { 0xd4ad9ee7, 65 },
    { 0xa75d9ee8, 90 },
    { 0xdd4d9ee9, 76 },
    { 0x445d9ee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d9ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cd9ef3, 100 },
    { 0, 0 },
    { 0xdced9ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06d9ef8, 95 },
    { 0, 0 },
    { 0x9dbd9efa, 90 },
    { 0xd8bd9efb, 90 },
    { 0x12cd9efb, 34 },
    { 0xe2ed9efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad9f01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fd9f07, 55 },
    { 0, 0 },
    { 0x3b0d9f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcead9f14, 36 },
    { 0, 0 },
    { 0x10ed9f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d9f19, 95 },
    { 0, 0 },
    { 0xed9d9f1b, 61 },
    { 0xd3fd9f1c, 95 },
    { 0x6e9d9f1d, 99 },
    { 0xaced9f1e, 95 },
    { 0x700d9f1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6d9f24, 95 },
    { 0x217d9f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbd9f28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0d9f2f, 23 },
    { 0x440d9f2f, 66 },
    { 0x3c8d9f30, 23 },
    { 0xae8d9f30, 55 },
    { 0x8bcd9f33, 78 },
    { 0x47cd9f33, 34 },
    { 0, 0 },
    { 0xbe3d9f36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d9f3c, 23 },
    { 0, 0 },
    { 0x9a2d9f3e, 91 },
    { 0xf00d9f3f, 98 },
    { 0x4c6d9f40, 90 },
    { 0x89bd9f40, 23 },
    { 0x826d9f40, 23 },
    { 0x347d9f43, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd9f47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1d9f4c, 98 },
    { 0x385d9f4c, 23 },
    { 0xab9d9f4e, 61 },
    { 0xee6d9f4f, 23 },
    { 0x402d9f50, 33 },
    { 0, 0 },
    { 0x9f3d9f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d9f55, 34 },
    { 0x304d9f56, 85 },
    { 0x1bfd9f55, 61 },
    { 0xf34d9f57, 43 },
    { 0x1a9d9f59, 33 },
    { 0xdedd9f59, 87 },
    { 0x208d9f55, 98 },
    { 0x928d9f5c, 43 },
    { 0x6d6d9f5d, 33 },
    { 0x73dd9f5e, 76 },
    { 0x412d9f5e, 95 },
    { 0x20cd9f5f, 46 },
    { 0xb50d9f61, 23 },
    { 0xaded9f62, 90 },
    { 0xf7fd9f63, 66 },
    { 0x7a8d9f62, 52 },
    { 0x36cd9f5e, 23 },
    { 0x6aad9f66, 77 },
    { 0x702d9f65, 23 },
    { 0x434d9f60, 98 },
    { 0xb52d9f63, 95 },
    { 0xfc4d9f6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d9f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825d9f75, 90 },
    { 0, 0 },
    { 0xcf7d9f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762d9f80, 90 },
    { 0, 0 },
    { 0x13dd9f82, 98 },
    { 0xd0cd9f83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x546d9f86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26d9f8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d9f8f, 90 },
    { 0xd8ed9f90, 98 },
    { 0x48fd9f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efd9f94, 95 },
    { 0, 0 },
    { 0x2ccd9f96, 61 },
    { 0, 0 },
    { 0xf48d9f98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71d9f9c, 90 },
    { 0x6cdd9f9c, 99 },
    { 0xaf1d9f9e, 65 },
    { 0xe23d9f9f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824d9fa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d9fa6, 76 },
    { 0xa95d9fa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564d9fab, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead9fb1, 76 },
    { 0, 0 },
    { 0x64fd9fb3, 33 },
    { 0x8c7d9fb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dd9fb8, 33 },
    { 0x6b7d9fb9, 95 },
    { 0x173d9fba, 66 },
    { 0x937d9fbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7d9fc5, 76 },
    { 0, 0 },
    { 0x274d9fc7, 98 },
    { 0, 0 },
    { 0x545d9fc9, 98 },
    { 0x113d9fca, 95 },
    { 0, 0 },
    { 0x2cdd9fcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ad9fd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfd9fd3, 98 },
    { 0x1b9d9fd4, 33 },
    { 0x3ecd9fd5, 77 },
    { 0xe3dd9fd5, 34 },
    { 0xbe0d9fd7, 76 },
    { 0xe11d9fd6, 55 },
    { 0xc6dd9fd6, 90 },
    { 0x524d9fd3, 78 },
    { 0x76fd9fdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d9fde, 90 },
    { 0xbebd9fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d9fe3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d9fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d9fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d9fef, 65 },
    { 0, 0 },
    { 0xe95d9ff1, 77 },
    { 0, 0 },
    { 0xb6fd9ff3, 76 },
    { 0x78ad9ff4, 90 },
    { 0xb84d9ff5, 34 },
    { 0xda0d9ff6, 90 },
    { 0xa83d9ff7, 76 },
    { 0xafad9ff8, 55 },
    { 0x30bd9ff9, 34 },
    { 0x278d9ff4, 23 },
    { 0, 0 },
    { 0x5ebd9ffc, 61 },
    { 0, 0 },
    { 0xded9ffe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877da007, 33 },
    { 0, 0 },
    { 0xdc5da009, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628da00e, 33 },
    { 0, 0 },
    { 0xdc6da010, 99 },
    { 0, 0 },
    { 0xd86da012, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fda015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251da01c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cda026, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73da029, 46 },
    { 0xd09da02a, 61 },
    { 0x9bbda02b, 90 },
    { 0xad9da02a, 98 },
    { 0x277da02b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583da031, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeda037, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26da03a, 33 },
    { 0, 0 },
    { 0xb37da03c, 95 },
    { 0x492da03c, 61 },
    { 0xe4da03e, 43 },
    { 0, 0 },
    { 0x6d5da040, 90 },
    { 0xd43da041, 23 },
    { 0x12cda040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cda047, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcda04c, 33 },
    { 0x12da04d, 95 },
    { 0xdbeda04e, 95 },
    { 0xe48da04f, 91 },
    { 0x52dda04d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcda057, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9da05a, 33 },
    { 0, 0 },
    { 0x49fda05c, 46 },
    { 0xe82da05c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feda060, 55 },
    { 0, 0 },
    { 0xeb6da062, 90 },
    { 0, 0 },
    { 0x9c8da064, 52 },
    { 0xdadda065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38da06a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eda06d, 87 },
    { 0x18fda06e, 90 },
    { 0xce5da06e, 98 },
    { 0, 0 },
    { 0x4f8da071, 46 },
    { 0xef0da071, 83 },
    { 0x61eda073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aada078, 95 },
    { 0, 0 },
    { 0x21bda07a, 23 },
    { 0x7c9da07b, 61 },
    { 0x439da07c, 33 },
    { 0, 0 },
    { 0x43ada07e, 97 },
    { 0, 0 },
    { 0x3c9da080, 33 },
    { 0x3f9da081, 95 },
    { 0xb33da080, 61 },
    { 0, 0 },
    { 0xfc6da084, 33 },
    { 0, 0 },
    { 0xf91da086, 61 },
    { 0, 0 },
    { 0xe31da088, 33 },
    { 0x62bda089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cda08c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fda091, 91 },
    { 0, 0 },
    { 0x47fda093, 76 },
    { 0x4a3da093, 98 },
    { 0x43bda095, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cda09c, 90 },
    { 0, 0 },
    { 0x88fda09e, 76 },
    { 0, 0 },
    { 0x2c5da0a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8da0a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907da0a9, 40 },
    { 0x822da0aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8da0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2da0b4, 53 },
    { 0, 0 },
    { 0x123da0b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65da0b9, 90 },
    { 0xf4fda0ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eda0c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeda0c8, 52 },
    { 0, 0 },
    { 0x473da0ca, 65 },
    { 0, 0 },
    { 0x7ddda0cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4da0d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2da0d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada0da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x783da0dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbda0e1, 90 },
    { 0xd6fda0e2, 33 },
    { 0, 0 },
    { 0x12eda0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccda0e7, 90 },
    { 0, 0 },
    { 0x6c3da0e9, 98 },
    { 0x8beda0ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449da0f0, 23 },
    { 0x47da0f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x491da0f4, 100 },
    { 0xb26da0f5, 98 },
    { 0, 0 },
    { 0x115da0f7, 23 },
    { 0xb6bda0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39da0fe, 96 },
    { 0x91ada0ff, 33 },
    { 0xf45da100, 33 },
    { 0x514da101, 68 },
    { 0x369da102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x608da105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268da109, 65 },
    { 0xd1fda109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfda10d, 33 },
    { 0x824da10e, 98 },
    { 0, 0 },
    { 0x8a3da110, 62 },
    { 0, 0 },
    { 0x9b3da112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fda117, 98 },
    { 0x8a6da118, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabda11e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64da125, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cada128, 95 },
    { 0, 0 },
    { 0xd31da12a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960da131, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bda138, 43 },
    { 0x64ada138, 61 },
    { 0x38da13a, 23 },
    { 0xeeada13b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeda144, 90 },
    { 0xa51da145, 23 },
    { 0, 0 },
    { 0x3cfda147, 20 },
    { 0xbb4da148, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136da14c, 93 },
    { 0x9d3da14d, 52 },
    { 0x74da14e, 34 },
    { 0x847da14f, 85 },
    { 0x666da150, 78 },
    { 0x17fda14f, 61 },
    { 0xd02da14f, 40 },
    { 0x10dda14d, 17 },
    { 0xcedda154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fda158, 66 },
    { 0xaf9da159, 98 },
    { 0, 0 },
    { 0x641da15b, 61 },
    { 0, 0 },
    { 0x3f7da15d, 66 },
    { 0, 0 },
    { 0xf89da15f, 98 },
    { 0xb6eda160, 66 },
    { 0xb0fda161, 33 },
    { 0, 0 },
    { 0x75ada163, 33 },
    { 0x17dda163, 23 },
    { 0x8f4da163, 55 },
    { 0xe5da166, 90 },
    { 0x31bda167, 23 },
    { 0x4aeda165, 55 },
    { 0xb2bda169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43da16c, 34 },
    { 0x172da16d, 61 },
    { 0x566da16e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ada171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feda178, 93 },
    { 0x4c0da178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6da17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bda17f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2da182, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987da186, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cda18f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223da198, 98 },
    { 0, 0 },
    { 0xa5cda19a, 90 },
    { 0, 0 },
    { 0xb3fda19c, 23 },
    { 0x9dfda19c, 76 },
    { 0, 0 },
    { 0xef8da19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda1a4, 55 },
    { 0xb85da1a5, 23 },
    { 0, 0 },
    { 0x119da1a7, 68 },
    { 0xba3da1a7, 43 },
    { 0xff0da1a9, 23 },
    { 0xab0da1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68da1ae, 95 },
    { 0x3acda1af, 95 },
    { 0, 0 },
    { 0x70fda1b1, 76 },
    { 0, 0 },
    { 0xc39da1b3, 23 },
    { 0x77fda1b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda1b9, 33 },
    { 0x43fda1ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eda1bf, 23 },
    { 0x523da1c0, 33 },
    { 0xa86da1bf, 61 },
    { 0, 0 },
    { 0x390da1c3, 91 },
    { 0xcaada1c3, 52 },
    { 0xdceda1c3, 90 },
    { 0x4bda1c5, 90 },
    { 0xf28da1c7, 33 },
    { 0, 0 },
    { 0x38cda1c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341da1cd, 61 },
    { 0, 0 },
    { 0xf3ada1cf, 91 },
    { 0x428da1cf, 98 },
    { 0x438da1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eda1d5, 93 },
    { 0xfddda1d6, 52 },
    { 0, 0 },
    { 0x62eda1d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94da1df, 88 },
    { 0xdf8da1e0, 33 },
    { 0x375da1e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42da1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08da1e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eda1f3, 98 },
    { 0, 0 },
    { 0xf95da1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdda1f9, 90 },
    { 0x2b2da1f9, 31 },
    { 0x6e7da1fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6da1fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990da202, 33 },
    { 0x7f6da202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beda207, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7da211, 88 },
    { 0, 0 },
    { 0xc70da213, 66 },
    { 0x677da213, 61 },
    { 0xb31da213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201da21b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dda220, 53 },
    { 0xda4da220, 23 },
    { 0x928da222, 61 },
    { 0x1b7da223, 90 },
    { 0, 0 },
    { 0xb7bda225, 95 },
    { 0x36da226, 52 },
    { 0xf1cda226, 95 },
    { 0x83da228, 64 },
    { 0, 0 },
    { 0x11bda22a, 97 },
    { 0x60cda22a, 61 },
    { 0x4dcda22b, 95 },
    { 0x759da22d, 101 },
    { 0x813da22e, 76 },
    { 0x20da22f, 76 },
    { 0, 0 },
    { 0x2dfda231, 65 },
    { 0x545da232, 98 },
    { 0x93bda231, 93 },
    { 0x9feda234, 61 },
    { 0x575da235, 98 },
    { 0xbcdda236, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9da239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45da23c, 67 },
    { 0x348da23d, 76 },
    { 0xcfada23d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16da241, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bda244, 90 },
    { 0x89fda244, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ada249, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bda24c, 91 },
    { 0x2edda24c, 95 },
    { 0, 0 },
    { 0xaadda24f, 46 },
    { 0xa6eda250, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fda254, 99 },
    { 0, 0 },
    { 0xd8eda256, 52 },
    { 0, 0 },
    { 0xdfdda258, 33 },
    { 0x77ada258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447da25d, 33 },
    { 0, 0 },
    { 0x9eeda25f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4da265, 43 },
    { 0xa80da266, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddda26a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6da26e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6da276, 98 },
    { 0, 0 },
    { 0xfb5da278, 61 },
    { 0x342da279, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afda27c, 61 },
    { 0xf70da27d, 76 },
    { 0x64bda27e, 90 },
    { 0x27da27f, 61 },
    { 0x4e4da27f, 95 },
    { 0xb7eda280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7da284, 98 },
    { 0xd0dda285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137da289, 23 },
    { 0, 0 },
    { 0xf81da28b, 68 },
    { 0, 0 },
    { 0x94eda28d, 90 },
    { 0x80fda28e, 61 },
    { 0x90eda28f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda293, 55 },
    { 0xba3da293, 23 },
    { 0x591da295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dda299, 23 },
    { 0, 0 },
    { 0x681da29b, 61 },
    { 0xe6da29c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4da2a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6da2a4, 90 },
    { 0, 0 },
    { 0x25bda2a6, 100 },
    { 0xcacda2a7, 33 },
    { 0xb02da2a7, 76 },
    { 0x81fda2a9, 90 },
    { 0, 0 },
    { 0xdecda2ab, 90 },
    { 0x3c8da2ac, 95 },
    { 0x9c5da2ab, 43 },
    { 0xe76da2ae, 95 },
    { 0xbcdda2af, 55 },
    { 0xe4bda2ad, 90 },
    { 0xaecda2b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963da2b9, 34 },
    { 0x2aada2ba, 23 },
    { 0xa8bda2bb, 33 },
    { 0x1dfda2bb, 33 },
    { 0x474da2bd, 90 },
    { 0xab2da2be, 23 },
    { 0x19bda2bd, 33 },
    { 0, 0 },
    { 0xd8cda2c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4da2c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffda2ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5da2cd, 95 },
    { 0x810da2ce, 61 },
    { 0x37dda2cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcda2d5, 93 },
    { 0x21bda2d6, 23 },
    { 0xf6da2d5, 43 },
    { 0, 0 },
    { 0x3afda2d9, 76 },
    { 0xaf4da2d9, 77 },
    { 0xdc6da2d9, 33 },
    { 0x5edda2db, 26 },
    { 0, 0 },
    { 0x6a6da2de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeda2e4, 46 },
    { 0x88eda2e5, 76 },
    { 0, 0 },
    { 0xb7cda2e7, 55 },
    { 0xe6cda2e8, 98 },
    { 0, 0 },
    { 0xcf2da2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887da2ef, 55 },
    { 0x920da2ef, 53 },
    { 0x86fda2f1, 90 },
    { 0xf5eda2f2, 35 },
    { 0x5e6da2f0, 66 },
    { 0, 0 },
    { 0x3dbda2f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8da2fa, 90 },
    { 0x2b1da2fa, 95 },
    { 0x817da2fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6da302, 61 },
    { 0, 0 },
    { 0x84eda304, 52 },
    { 0xbf6da305, 23 },
    { 0x71fda304, 77 },
    { 0x835da306, 33 },
    { 0xa9dda308, 23 },
    { 0x2fbda308, 43 },
    { 0x753da307, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eda30d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0da311, 76 },
    { 0, 0 },
    { 0x1b0da313, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fda316, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234da31c, 90 },
    { 0xf02da31c, 66 },
    { 0x259da31e, 61 },
    { 0x5c7da31c, 90 },
    { 0x6f2da320, 68 },
    { 0x6cda320, 61 },
    { 0xf89da322, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddda328, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8da331, 85 },
    { 0, 0 },
    { 0x34ada333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798da337, 99 },
    { 0x2a0da337, 95 },
    { 0x5c7da337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7da33c, 33 },
    { 0x5d8da33d, 55 },
    { 0x349da33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3da341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecda34a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9deda34d, 61 },
    { 0x221da34d, 98 },
    { 0xab7da34f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5da358, 93 },
    { 0xededa359, 91 },
    { 0x1dda35a, 76 },
    { 0, 0 },
    { 0x9e3da35c, 34 },
    { 0x60ada35d, 96 },
    { 0, 0 },
    { 0x49dda35f, 90 },
    { 0, 0 },
    { 0x211da361, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dda36d, 23 },
    { 0xc99da36e, 34 },
    { 0x83bda36e, 95 },
    { 0x76bda370, 85 },
    { 0xafda370, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379da37c, 68 },
    { 0xeb8da37d, 67 },
    { 0xbfbda37e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39da382, 56 },
    { 0xa40da383, 90 },
    { 0, 0 },
    { 0x959da385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dda38a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5da38d, 55 },
    { 0xf91da38d, 76 },
    { 0x7abda38f, 33 },
    { 0x738da390, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459da394, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeda3a1, 33 },
    { 0, 0 },
    { 0x6eada3a3, 33 },
    { 0xa96da3a4, 76 },
    { 0xf53da3a4, 33 },
    { 0, 0 },
    { 0xd81da3a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4da3ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807da3b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575da3b7, 66 },
    { 0x1cada3b8, 33 },
    { 0x5f3da3b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ada3c5, 90 },
    { 0, 0 },
    { 0x1cada3c7, 61 },
    { 0, 0 },
    { 0xbccda3c9, 90 },
    { 0x7d3da3ca, 61 },
    { 0xafeda3cb, 33 },
    { 0xdc5da3cc, 90 },
    { 0, 0 },
    { 0x458da3ce, 17 },
    { 0x398da3ce, 98 },
    { 0xfcdda3d0, 46 },
    { 0x2bfda3cf, 23 },
    { 0, 0 },
    { 0x426da3d3, 69 },
    { 0, 0 },
    { 0x34fda3d5, 95 },
    { 0x803da3d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fda3dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1da3e2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x904da3e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cada3ec, 61 },
    { 0xadcda3ed, 33 },
    { 0xbb5da3ed, 98 },
    { 0xd98da3ef, 55 },
    { 0xa59da3ed, 91 },
    { 0xff4da3f1, 55 },
    { 0x60dda3f1, 17 },
    { 0x7d8da3f3, 90 },
    { 0xe21da3f3, 93 },
    { 0, 0 },
    { 0x13eda3f6, 61 },
    { 0x64da3f7, 76 },
    { 0x4e2da3f8, 46 },
    { 0, 0 },
    { 0xbfcda3fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x581da3fd, 78 },
    { 0xbf1da3fe, 87 },
    { 0x144da3ff, 33 },
    { 0xc5eda3fe, 90 },
    { 0xeabda401, 61 },
    { 0xf62da402, 23 },
    { 0, 0 },
    { 0x595da404, 95 },
    { 0xd33da404, 23 },
    { 0xc22da406, 33 },
    { 0xf5fda404, 75 },
    { 0, 0 },
    { 0x84ada409, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50da40c, 43 },
    { 0xcd6da40d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cda410, 61 },
    { 0xf4ada411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693da416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfda41e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cda421, 77 },
    { 0x9cada421, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2da425, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99da42b, 33 },
    { 0xf8eda42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bda431, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeda436, 98 },
    { 0, 0 },
    { 0xcc7da438, 46 },
    { 0, 0 },
    { 0xc4dda43a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4da43d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x280da440, 43 },
    { 0x6beda441, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dda447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733da44b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bda450, 23 },
    { 0x235da450, 52 },
    { 0, 0 },
    { 0xdc0da453, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x157da456, 61 },
    { 0x52cda457, 46 },
    { 0x109da457, 34 },
    { 0xc7ada459, 65 },
    { 0x71fda45a, 85 },
    { 0xdf3da45a, 33 },
    { 0x1ada45c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ada461, 33 },
    { 0x2c8da461, 90 },
    { 0xe6bda461, 76 },
    { 0x4f3da464, 61 },
    { 0xcf4da463, 75 },
    { 0x1bdda462, 61 },
    { 0, 0 },
    { 0x70dda468, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x846da46b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72da46e, 98 },
    { 0x725da46f, 61 },
    { 0xae9da470, 23 },
    { 0x4b3da46e, 64 },
    { 0x13cda472, 95 },
    { 0x844da473, 52 },
    { 0, 0 },
    { 0xa1da475, 90 },
    { 0, 0 },
    { 0xfe3da477, 61 },
    { 0xe33da478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7da47b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4da47f, 33 },
    { 0, 0 },
    { 0xd46da481, 28 },
    { 0, 0 },
    { 0x96eda483, 87 },
    { 0xa45da484, 95 },
    { 0xea6da485, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffda48b, 23 },
    { 0x1afda48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3da48f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230da498, 99 },
    { 0, 0 },
    { 0x7f2da49a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x666da49d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fda4a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x533da4a6, 55 },
    { 0, 0 },
    { 0xd4cda4a8, 98 },
    { 0xaddda4a9, 78 },
    { 0xa28da4a8, 65 },
    { 0x355da4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5da4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820da4b6, 95 },
    { 0x510da4b7, 93 },
    { 0xe90da4b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9da4bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da4c3, 90 },
    { 0, 0 },
    { 0x222da4c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dda4c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82da4cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627da4d0, 46 },
    { 0, 0 },
    { 0xa4dda4d2, 98 },
    { 0xb8dda4d3, 87 },
    { 0xd22da4d4, 76 },
    { 0x31fda4d5, 75 },
    { 0xefda4d2, 78 },
    { 0, 0 },
    { 0xf34da4d8, 23 },
    { 0x90fda4d9, 88 },
    { 0, 0 },
    { 0xdc7da4db, 76 },
    { 0x30fda4dc, 99 },
    { 0, 0 },
    { 0xeedda4de, 34 },
    { 0xccdda4df, 99 },
    { 0x5d8da4e0, 90 },
    { 0, 0 },
    { 0x8fda4e2, 32 },
    { 0xdb5da4e3, 61 },
    { 0x7b8da4e4, 89 },
    { 0xd52da4e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4da4eb, 61 },
    { 0xf0ada4eb, 33 },
    { 0, 0 },
    { 0x342da4ee, 95 },
    { 0x9eda4ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8da4fa, 33 },
    { 0xbf0da4fb, 90 },
    { 0x4b7da4fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83da4ff, 33 },
    { 0, 0 },
    { 0xb37da501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfda504, 43 },
    { 0x7edda504, 98 },
    { 0xb06da505, 43 },
    { 0, 0 },
    { 0x466da508, 38 },
    { 0x640da509, 55 },
    { 0xfb8da508, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17da50f, 33 },
    { 0x569da510, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218da517, 95 },
    { 0xd8dda518, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866da51f, 93 },
    { 0, 0 },
    { 0xddbda521, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3da528, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eda52c, 90 },
    { 0, 0 },
    { 0x2f2da52e, 52 },
    { 0xb8bda52f, 88 },
    { 0xe09da52e, 46 },
    { 0x739da52f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x143da534, 61 },
    { 0x807da535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eda53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6da53e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879da542, 77 },
    { 0, 0 },
    { 0xa9cda544, 67 },
    { 0xce3da545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896da549, 66 },
    { 0x91eda54a, 34 },
    { 0x800da54b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5eda554, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213da558, 90 },
    { 0x506da558, 95 },
    { 0, 0 },
    { 0x2b1da55b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6da561, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fda567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eda56b, 85 },
    { 0x7d4da56b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6da570, 87 },
    { 0xcaada571, 76 },
    { 0x187da572, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2da575, 95 },
    { 0xf24da576, 100 },
    { 0xaf3da577, 95 },
    { 0xf64da576, 98 },
    { 0x9d8da576, 99 },
    { 0, 0 },
    { 0xff7da57b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3da57e, 90 },
    { 0x78eda57f, 95 },
    { 0xcbcda580, 55 },
    { 0x89cda581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfada586, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38da591, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280da595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x981da598, 61 },
    { 0x91cda599, 87 },
    { 0x87eda59a, 33 },
    { 0, 0 },
    { 0xa81da59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461da5a1, 78 },
    { 0x5abda5a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1da5ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3da5b2, 98 },
    { 0x746da5b2, 55 },
    { 0xcb5da5b4, 66 },
    { 0xddcda5b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268da5b9, 90 },
    { 0x32eda5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68da5c2, 55 },
    { 0, 0 },
    { 0x762da5c4, 96 },
    { 0xa5cda5c4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9da5d2, 100 },
    { 0x2c6da5d2, 34 },
    { 0x997da5d4, 52 },
    { 0xb76da5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0da5db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5e0, 33 },
    { 0x5a5da5e1, 77 },
    { 0xfe8da5e1, 23 },
    { 0x182da5e0, 98 },
    { 0x2fda5e1, 93 },
    { 0xfe6da5e4, 33 },
    { 0xba2da5e6, 61 },
    { 0xe32da5e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ada5ef, 43 },
    { 0x542da5f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71da5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582da5fb, 84 },
    { 0x6aada5fc, 76 },
    { 0x392da5fc, 90 },
    { 0xefbda5fe, 55 },
    { 0, 0 },
    { 0x5e4da600, 93 },
    { 0, 0 },
    { 0x2c1da602, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0da605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4da60a, 53 },
    { 0, 0 },
    { 0x309da60c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716da60f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fda616, 85 },
    { 0xa57da617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55eda61b, 76 },
    { 0x444da61b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x936da61f, 34 },
    { 0x204da620, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eda628, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4da62e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ada634, 77 },
    { 0xd31da635, 82 },
    { 0, 0 },
    { 0x194da637, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6da63c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fda640, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58da644, 78 },
    { 0x4b7da644, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72da648, 46 },
    { 0x64da649, 53 },
    { 0x8fcda648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bada64e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812da653, 66 },
    { 0xe61da653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9da65b, 55 },
    { 0, 0 },
    { 0xbbda65d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3da661, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05da665, 76 },
    { 0, 0 },
    { 0xe07da667, 61 },
    { 0xa69da668, 88 },
    { 0xbd7da669, 87 },
    { 0x362da668, 95 },
    { 0x743da66b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffada675, 98 },
    { 0x37ada676, 93 },
    { 0x858da675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3da67d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cda680, 55 },
    { 0, 0 },
    { 0x9b3da682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785da687, 61 },
    { 0xc6dda688, 90 },
    { 0xb85da688, 43 },
    { 0, 0 },
    { 0xef9da68b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86da68f, 90 },
    { 0x881da68f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aada699, 91 },
    { 0xc60da69a, 76 },
    { 0x1d3da69a, 61 },
    { 0x176da69c, 33 },
    { 0x953da69d, 40 },
    { 0, 0 },
    { 0xe30da69f, 33 },
    { 0x90fda69f, 33 },
    { 0x99dda6a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228da6a8, 95 },
    { 0x89da6a8, 95 },
    { 0, 0 },
    { 0x272da6ab, 66 },
    { 0x427da6ac, 84 },
    { 0x933da6ac, 88 },
    { 0x212da6ae, 61 },
    { 0xe8eda6af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fda6b2, 98 },
    { 0x426da6b3, 34 },
    { 0x1d3da6b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989da6be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ada6c1, 23 },
    { 0x963da6c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda6c6, 90 },
    { 0x23bda6c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ada6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8da6cf, 90 },
    { 0, 0 },
    { 0xc39da6d1, 64 },
    { 0x85bda6d1, 90 },
    { 0, 0 },
    { 0x7bada6d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fda6d9, 43 },
    { 0x7ceda6da, 87 },
    { 0x544da6db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3da6e1, 91 },
    { 0x7a2da6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69da6e5, 77 },
    { 0, 0 },
    { 0xd58da6e7, 87 },
    { 0x472da6e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ada6f2, 33 },
    { 0x519da6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434da6f8, 34 },
    { 0x41ada6f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2da6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dda702, 23 },
    { 0x4a0da703, 34 },
    { 0x86ada702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9da708, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787da70e, 55 },
    { 0xa1da70f, 52 },
    { 0x51bda710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfda713, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbda718, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200da71c, 46 },
    { 0xa8bda71d, 46 },
    { 0x8d1da71c, 84 },
    { 0xa0bda71c, 95 },
    { 0xa07da720, 23 },
    { 0xedda721, 98 },
    { 0x12eda722, 46 },
    { 0x950da723, 17 },
    { 0xa7eda721, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16da728, 77 },
    { 0, 0 },
    { 0x31cda72a, 46 },
    { 0x2c0da72b, 98 },
    { 0x5deda72b, 95 },
    { 0xce4da72c, 85 },
    { 0xae8da72e, 46 },
    { 0x635da72f, 33 },
    { 0x87eda72c, 95 },
    { 0, 0 },
    { 0x7cda732, 34 },
    { 0xd4eda733, 99 },
    { 0x889da734, 76 },
    { 0x685da734, 43 },
    { 0x6eada736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169da73c, 90 },
    { 0, 0 },
    { 0x10cda73e, 99 },
    { 0x9d6da73e, 23 },
    { 0x9d5da740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8da743, 90 },
    { 0x5b9da743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105da749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1da750, 61 },
    { 0xa7da751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2da754, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcda75b, 87 },
    { 0x9d2da75c, 95 },
    { 0, 0 },
    { 0xd0ada75e, 76 },
    { 0, 0 },
    { 0xee3da760, 84 },
    { 0, 0 },
    { 0xeb7da762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cada765, 33 },
    { 0, 0 },
    { 0xafbda767, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da76c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cda76f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60da773, 90 },
    { 0xad9da773, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cda777, 95 },
    { 0, 0 },
    { 0x736da779, 78 },
    { 0xe8dda77a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3da77e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427da787, 81 },
    { 0xa82da787, 23 },
    { 0xadcda789, 65 },
    { 0, 0 },
    { 0xa8cda78b, 66 },
    { 0x683da78c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bda790, 23 },
    { 0x4fada790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5da794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadda797, 23 },
    { 0xb42da797, 43 },
    { 0x45ada798, 95 },
    { 0x190da799, 34 },
    { 0, 0 },
    { 0x82bda79c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eda7a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfda7aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8da7ae, 100 },
    { 0xb8bda7af, 67 },
    { 0x4cda7af, 91 },
    { 0, 0 },
    { 0x958da7b2, 55 },
    { 0xcdfda7b2, 23 },
    { 0, 0 },
    { 0x7e7da7b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bda7b9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dda7be, 95 },
    { 0, 0 },
    { 0x2f7da7c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cda7c7, 84 },
    { 0x8beda7c8, 76 },
    { 0xd32da7c7, 61 },
    { 0, 0 },
    { 0xd19da7cb, 83 },
    { 0, 0 },
    { 0x722da7cd, 90 },
    { 0, 0 },
    { 0x159da7cf, 61 },
    { 0x733da7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6da7d4, 95 },
    { 0x4c5da7d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cda7d8, 91 },
    { 0x99cda7d8, 61 },
    { 0, 0 },
    { 0xc65da7db, 93 },
    { 0x770da7dc, 9 },
    { 0x813da7dd, 43 },
    { 0, 0 },
    { 0x9c1da7df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cada7e3, 90 },
    { 0, 0 },
    { 0xbf9da7e5, 33 },
    { 0xfc8da7e6, 99 },
    { 0x50bda7e7, 76 },
    { 0x49ada7e6, 98 },
    { 0, 0 },
    { 0x483da7ea, 91 },
    { 0xbd5da7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50da7f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7da7fb, 33 },
    { 0, 0 },
    { 0x938da7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fda800, 90 },
    { 0xa4dda801, 95 },
    { 0, 0 },
    { 0x610da803, 95 },
    { 0xffcda804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfda807, 90 },
    { 0x6bdda808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90da80b, 67 },
    { 0x748da80c, 61 },
    { 0, 0 },
    { 0x4c1da80e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4da811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fda815, 76 },
    { 0xa5eda816, 23 },
    { 0x250da815, 95 },
    { 0, 0 },
    { 0x684da819, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615da81f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298da824, 76 },
    { 0x909da825, 23 },
    { 0x8ebda825, 31 },
    { 0xe65da827, 95 },
    { 0xc8ada828, 61 },
    { 0x48da828, 98 },
    { 0xd0eda82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fda82f, 33 },
    { 0x27bda82f, 101 },
    { 0, 0 },
    { 0xa2fda832, 95 },
    { 0, 0 },
    { 0xdb9da834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78da839, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171da83d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12da840, 98 },
    { 0xaecda841, 95 },
    { 0xaa8da842, 99 },
    { 0xed6da843, 34 },
    { 0x4feda844, 76 },
    { 0x3dada841, 61 },
    { 0xafcda842, 33 },
    { 0xf53da847, 61 },
    { 0x360da848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x631da84b, 98 },
    { 0xf5dda84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339da850, 61 },
    { 0xa02da850, 93 },
    { 0, 0 },
    { 0x94dda853, 33 },
    { 0x9bbda854, 66 },
    { 0x1b5da855, 68 },
    { 0, 0 },
    { 0x9d3da857, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6da85b, 90 },
    { 0, 0 },
    { 0x791da85d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x753da860, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9da864, 34 },
    { 0xf0ada865, 91 },
    { 0xfd9da865, 93 },
    { 0x54da867, 55 },
    { 0xc40da868, 34 },
    { 0x2f6da865, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fda86d, 55 },
    { 0xc23da86e, 43 },
    { 0x543da86e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5da872, 76 },
    { 0x4c0da872, 61 },
    { 0xa8bda874, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2da87c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adda880, 95 },
    { 0xc03da881, 61 },
    { 0, 0 },
    { 0x366da883, 55 },
    { 0, 0 },
    { 0xb58da885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84da88f, 33 },
    { 0xb43da88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1da893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcda897, 33 },
    { 0xf34da897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ada89b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afda89e, 90 },
    { 0xf58da89e, 33 },
    { 0xe43da89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6da8a5, 51 },
    { 0xd92da8a5, 23 },
    { 0x1b2da8a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ada8aa, 23 },
    { 0x209da8ab, 34 },
    { 0, 0 },
    { 0x1feda8ad, 33 },
    { 0, 0 },
    { 0xcabda8af, 61 },
    { 0x893da8b0, 33 },
    { 0xf44da8b1, 90 },
    { 0x70bda8b1, 91 },
    { 0x6ebda8b1, 33 },
    { 0xe94da8b2, 90 },
    { 0x640da8b4, 61 },
    { 0x9fdda8b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1da8b9, 61 },
    { 0, 0 },
    { 0x4d6da8bb, 76 },
    { 0, 0 },
    { 0xde5da8bd, 61 },
    { 0, 0 },
    { 0x5e9da8bf, 91 },
    { 0, 0 },
    { 0xdd4da8c1, 90 },
    { 0x467da8c2, 100 },
    { 0x60eda8c2, 33 },
    { 0x2bfda8c4, 68 },
    { 0x63ada8c2, 33 },
    { 0x76eda8c6, 61 },
    { 0x4f9da8c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822da8cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeda8d8, 34 },
    { 0xef1da8d9, 88 },
    { 0x2afda8da, 40 },
    { 0x46ada8db, 95 },
    { 0xe5ada8dc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ada8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84da8e6, 33 },
    { 0x279da8e7, 95 },
    { 0xe70da8e7, 52 },
    { 0xc6bda8e9, 90 },
    { 0, 0 },
    { 0xbda8eb, 87 },
    { 0x617da8ec, 23 },
    { 0xbd2da8ed, 33 },
    { 0, 0 },
    { 0x1feda8ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15da8f6, 52 },
    { 0xafada8f6, 43 },
    { 0x848da8f6, 82 },
    { 0xe1fda8f7, 34 },
    { 0, 0 },
    { 0x26ada8fb, 76 },
    { 0xeccda8fc, 34 },
    { 0xbd9da8fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeda904, 23 },
    { 0x7cda905, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x223da908, 76 },
    { 0, 0 },
    { 0x2fbda90a, 90 },
    { 0, 0 },
    { 0xd45da90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6da913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0da917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada91a, 55 },
    { 0x44fda91b, 34 },
    { 0x49ada91b, 67 },
    { 0xf23da91d, 95 },
    { 0x30bda91d, 95 },
    { 0x1d7da91a, 23 },
    { 0x9d6da91a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bda924, 85 },
    { 0x828da924, 98 },
    { 0x37ada926, 93 },
    { 0, 0 },
    { 0x628da928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8da92b, 80 },
    { 0xc87da92c, 93 },
    { 0xfd6da92d, 46 },
    { 0x505da92b, 91 },
    { 0, 0 },
    { 0xc0bda930, 17 },
    { 0, 0 },
    { 0x462da932, 77 },
    { 0, 0 },
    { 0xca1da934, 78 },
    { 0x31dda934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9da939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedada945, 61 },
    { 0xf56da946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810da94a, 90 },
    { 0x91cda94a, 61 },
    { 0, 0 },
    { 0x296da94d, 91 },
    { 0, 0 },
    { 0x5eada94f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56da952, 90 },
    { 0, 0 },
    { 0x159da954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x530da957, 78 },
    { 0x8bcda957, 61 },
    { 0, 0 },
    { 0x447da95a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8da960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585da966, 80 },
    { 0, 0 },
    { 0x6edda968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x241da96b, 88 },
    { 0, 0 },
    { 0xc73da96d, 95 },
    { 0x67eda96e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bda973, 67 },
    { 0x3ddda973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56da977, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98da97c, 33 },
    { 0x7d4da97d, 46 },
    { 0x298da97c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0da983, 43 },
    { 0xc08da984, 61 },
    { 0x4f3da983, 55 },
    { 0x472da986, 98 },
    { 0xc6eda987, 98 },
    { 0x801da988, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dda98b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597da991, 68 },
    { 0x2edda991, 33 },
    { 0x592da992, 66 },
    { 0, 0 },
    { 0x3dfda995, 43 },
    { 0xcda996, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131da99b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6da9ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53da9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fda9b5, 98 },
    { 0xf0ada9b6, 43 },
    { 0xabada9b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684da9be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387da9c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1da9ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2da9cd, 43 },
    { 0xcada9ce, 98 },
    { 0x899da9cf, 78 },
    { 0, 0 },
    { 0x89cda9d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1da9d9, 93 },
    { 0, 0 },
    { 0xbb1da9db, 23 },
    { 0, 0 },
    { 0x1cbda9dd, 34 },
    { 0, 0 },
    { 0xbd5da9df, 33 },
    { 0x89dda9e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790da9e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13da9ec, 61 },
    { 0xabada9ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11da9f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394da9f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23da9f7, 91 },
    { 0x89eda9f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0da9ff, 91 },
    { 0xe68daa00, 90 },
    { 0x7a2da9ff, 67 },
    { 0, 0 },
    { 0x616daa03, 90 },
    { 0x4c9daa04, 61 },
    { 0, 0 },
    { 0x8f7daa06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69daa0c, 67 },
    { 0, 0 },
    { 0xdd0daa0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62daa11, 43 },
    { 0, 0 },
    { 0xcb9daa13, 90 },
    { 0x6b9daa14, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07daa17, 61 },
    { 0x8f7daa18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241daa1e, 33 },
    { 0x2cedaa1e, 33 },
    { 0, 0 },
    { 0x108daa21, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935daa27, 76 },
    { 0x5f9daa28, 43 },
    { 0xe02daa28, 90 },
    { 0x29edaa29, 98 },
    { 0x6c1daa2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6daa31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46daa35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ddaa38, 98 },
    { 0, 0 },
    { 0x264daa3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813daa41, 23 },
    { 0xfe1daa42, 99 },
    { 0x8e5daa43, 76 },
    { 0xbeddaa42, 55 },
    { 0, 0 },
    { 0xf78daa46, 93 },
    { 0, 0 },
    { 0x462daa48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69adaa4c, 93 },
    { 0x558daa4d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bdaa50, 33 },
    { 0, 0 },
    { 0x276daa52, 23 },
    { 0xb21daa52, 33 },
    { 0, 0 },
    { 0x62edaa55, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66adaa5c, 61 },
    { 0, 0 },
    { 0x7ebdaa5e, 46 },
    { 0, 0 },
    { 0xd4edaa60, 75 },
    { 0x1f7daa60, 95 },
    { 0xa98daa62, 23 },
    { 0x9cbdaa63, 34 },
    { 0, 0 },
    { 0xae0daa65, 52 },
    { 0x53daa66, 90 },
    { 0, 0 },
    { 0xcffdaa68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bdaa6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5daa6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2edaa71, 95 },
    { 0xb00daa72, 52 },
    { 0, 0 },
    { 0x8dedaa74, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdaa77, 90 },
    { 0, 0 },
    { 0xfa8daa79, 46 },
    { 0, 0 },
    { 0xa59daa7b, 55 },
    { 0, 0 },
    { 0x5b4daa7d, 95 },
    { 0x915daa7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3daa84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77daa87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfdaa8a, 40 },
    { 0, 0 },
    { 0xfaedaa8c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4daa90, 33 },
    { 0xbc2daa91, 33 },
    { 0, 0 },
    { 0x639daa93, 33 },
    { 0x62daa94, 97 },
    { 0x51cdaa94, 95 },
    { 0x2fddaa96, 23 },
    { 0xa1adaa97, 61 },
    { 0x5cadaa98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45daa9f, 90 },
    { 0x81daaa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ddaaa8, 76 },
    { 0x9b5daaa9, 46 },
    { 0x8f7daaaa, 81 },
    { 0xf41daaa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdaab0, 90 },
    { 0x314daab0, 23 },
    { 0x465daab0, 33 },
    { 0x17ddaab1, 98 },
    { 0, 0 },
    { 0xdf9daab5, 95 },
    { 0xd8edaab5, 90 },
    { 0xd35daab5, 91 },
    { 0, 0 },
    { 0xa55daab9, 23 },
    { 0xa1cdaab9, 46 },
    { 0xe28daabb, 90 },
    { 0xd29daabb, 43 },
    { 0, 0 },
    { 0x97edaabe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85daac1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x275daac4, 90 },
    { 0x843daac4, 93 },
    { 0x4f8daac6, 61 },
    { 0xe47daac4, 51 },
    { 0x3eddaac8, 90 },
    { 0xe2fdaac7, 61 },
    { 0xcb8daac4, 55 },
    { 0x2a5daaca, 33 },
    { 0, 0 },
    { 0x777daacd, 91 },
    { 0x83daacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730daad2, 61 },
    { 0xd85daad3, 33 },
    { 0xbfedaad2, 33 },
    { 0x637daad5, 90 },
    { 0x831daad5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191daadc, 98 },
    { 0, 0 },
    { 0xf89daade, 66 },
    { 0xed1daadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x914daae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9daaed, 90 },
    { 0xe4ddaaed, 61 },
    { 0xcb1daaef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4daaf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94daaf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699dab04, 43 },
    { 0x3c3dab05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45dab08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6adab0d, 46 },
    { 0x1fcdab0e, 34 },
    { 0x98cdab0e, 95 },
    { 0x9f8dab0f, 93 },
    { 0x538dab11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0dab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908dab1b, 49 },
    { 0x334dab1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x853dab1f, 33 },
    { 0xd72dab20, 33 },
    { 0xff5dab1f, 76 },
    { 0xbbedab1f, 61 },
    { 0x1cbdab1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dab26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bdab2b, 95 },
    { 0x6a1dab2b, 61 },
    { 0x1b1dab2d, 88 },
    { 0x9afdab2e, 95 },
    { 0x488dab2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86edab32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fdab36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527dab3a, 23 },
    { 0x35adab3a, 95 },
    { 0, 0 },
    { 0xa20dab3d, 81 },
    { 0x239dab3e, 23 },
    { 0x89cdab3f, 98 },
    { 0x1cbdab3d, 17 },
    { 0x2dadab3e, 51 },
    { 0x74edab42, 61 },
    { 0xdfdab42, 23 },
    { 0x37bdab42, 66 },
    { 0, 0 },
    { 0x913dab46, 95 },
    { 0, 0 },
    { 0xecddab48, 93 },
    { 0xa4edab49, 61 },
    { 0x723dab48, 33 },
    { 0, 0 },
    { 0x4fcdab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3dab4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7dab54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cdab57, 90 },
    { 0, 0 },
    { 0xfd4dab59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2dab5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15adab61, 43 },
    { 0x4cbdab62, 98 },
    { 0, 0 },
    { 0x30dab64, 84 },
    { 0x62edab65, 90 },
    { 0, 0 },
    { 0xfc2dab67, 33 },
    { 0, 0 },
    { 0x688dab69, 61 },
    { 0xf4bdab6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfdab6d, 46 },
    { 0, 0 },
    { 0x564dab6f, 61 },
    { 0, 0 },
    { 0x1badab71, 93 },
    { 0, 0 },
    { 0x952dab73, 90 },
    { 0, 0 },
    { 0x817dab75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8edab78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0dab7b, 23 },
    { 0x972dab7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4badab80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6dab83, 61 },
    { 0x859dab83, 93 },
    { 0xc6bdab85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cdab88, 33 },
    { 0xe6ddab88, 61 },
    { 0xf71dab88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdab8d, 23 },
    { 0x1ebdab8e, 61 },
    { 0, 0 },
    { 0x9f4dab90, 61 },
    { 0x14edab91, 90 },
    { 0xce9dab92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26dab95, 87 },
    { 0x7b7dab96, 98 },
    { 0, 0 },
    { 0x401dab98, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdab9f, 98 },
    { 0x914daba0, 88 },
    { 0, 0 },
    { 0xcd0daba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97daba6, 61 },
    { 0, 0 },
    { 0x395daba8, 55 },
    { 0, 0 },
    { 0x697dabaa, 61 },
    { 0xb90dabab, 91 },
    { 0, 0 },
    { 0xd1bdabad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82dabb2, 61 },
    { 0x82adabb2, 23 },
    { 0, 0 },
    { 0x454dabb5, 95 },
    { 0, 0 },
    { 0xf3adabb7, 33 },
    { 0xbfbdabb8, 95 },
    { 0, 0 },
    { 0x2c3dabba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801dabbf, 61 },
    { 0, 0 },
    { 0x60fdabc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74dabc5, 95 },
    { 0x490dabc6, 66 },
    { 0, 0 },
    { 0x528dabc8, 76 },
    { 0xf6cdabc9, 33 },
    { 0xb63dabca, 68 },
    { 0x72adabca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bdabce, 23 },
    { 0x329dabcf, 52 },
    { 0xbb2dabcf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453dabd8, 61 },
    { 0xb41dabd9, 33 },
    { 0xa8ddabda, 98 },
    { 0x62dabd8, 76 },
    { 0, 0 },
    { 0x3a9dabdd, 34 },
    { 0x228dabdd, 61 },
    { 0x525dabdf, 80 },
    { 0xe16dabe0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3dabe5, 91 },
    { 0, 0 },
    { 0x450dabe7, 61 },
    { 0xd94dabe8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x920dabeb, 85 },
    { 0, 0 },
    { 0xb47dabed, 93 },
    { 0xdd6dabee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bdabf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8dabf4, 23 },
    { 0xfa1dabf4, 91 },
    { 0x40dabf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdabf9, 98 },
    { 0, 0 },
    { 0x755dabfb, 33 },
    { 0x980dabfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cdac01, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dac07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x695dac0a, 76 },
    { 0x4ffdac0b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3dac10, 97 },
    { 0x570dac10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x879dac14, 23 },
    { 0, 0 },
    { 0xe50dac16, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dac1b, 68 },
    { 0x2dadac1c, 76 },
    { 0, 0 },
    { 0xfdddac1e, 61 },
    { 0xf0cdac1f, 46 },
    { 0, 0 },
    { 0x5afdac21, 98 },
    { 0xa2fdac22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadac26, 66 },
    { 0x65adac27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x889dac2a, 36 },
    { 0xb8bdac2b, 55 },
    { 0, 0 },
    { 0x930dac2d, 78 },
    { 0x153dac2e, 60 },
    { 0, 0 },
    { 0xf70dac30, 61 },
    { 0x678dac31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdddac35, 93 },
    { 0xfa9dac36, 100 },
    { 0, 0 },
    { 0x599dac38, 90 },
    { 0x8a6dac39, 76 },
    { 0, 0 },
    { 0xb3bdac3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbdac3e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bdac43, 55 },
    { 0x193dac43, 98 },
    { 0xf4fdac44, 99 },
    { 0x749dac46, 95 },
    { 0xaf6dac43, 98 },
    { 0x10adac44, 99 },
    { 0x3d5dac49, 65 },
    { 0, 0 },
    { 0xe48dac4b, 61 },
    { 0xf31dac4b, 55 },
    { 0, 0 },
    { 0x113dac4e, 95 },
    { 0, 0 },
    { 0x79bdac50, 68 },
    { 0xec2dac51, 61 },
    { 0, 0 },
    { 0xd85dac53, 61 },
    { 0x3b0dac53, 68 },
    { 0x9d5dac55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedac5f, 23 },
    { 0, 0 },
    { 0x8edac61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660dac67, 100 },
    { 0xfd7dac68, 84 },
    { 0xbe8dac69, 76 },
    { 0xd5cdac6a, 33 },
    { 0xf99dac67, 95 },
    { 0, 0 },
    { 0xaadac6d, 95 },
    { 0, 0 },
    { 0x170dac6f, 34 },
    { 0, 0 },
    { 0xbbcdac71, 46 },
    { 0, 0 },
    { 0xf4cdac73, 55 },
    { 0xb95dac73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4dac78, 61 },
    { 0, 0 },
    { 0x1dedac7a, 95 },
    { 0x28dac7a, 33 },
    { 0x771dac7c, 23 },
    { 0xf94dac7c, 67 },
    { 0x545dac7e, 43 },
    { 0x9f3dac7f, 94 },
    { 0, 0 },
    { 0xe1adac81, 98 },
    { 0x7b4dac82, 46 },
    { 0x707dac83, 33 },
    { 0x71cdac81, 95 },
    { 0xa82dac83, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dac88, 61 },
    { 0x5fcdac89, 93 },
    { 0xdd5dac89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4dac8d, 95 },
    { 0x8e9dac8e, 59 },
    { 0x34ddac8f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647dac95, 93 },
    { 0x5c2dac96, 100 },
    { 0, 0 },
    { 0xd89dac98, 46 },
    { 0xef9dac98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addac9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3daca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x840daca6, 52 },
    { 0x372daca6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdacad, 33 },
    { 0xd38dacad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21dacb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1dacb7, 53 },
    { 0xdc4dacb8, 98 },
    { 0x311dacb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fdacbc, 33 },
    { 0x40cdacbd, 33 },
    { 0xa1adacbd, 61 },
    { 0xcd7dacbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398dacca, 75 },
    { 0xd69daccb, 95 },
    { 0xd9daccb, 90 },
    { 0x11bdaccc, 61 },
    { 0x44bdaccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57edacd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948dace1, 64 },
    { 0x292dace2, 98 },
    { 0x453dace2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddace7, 61 },
    { 0, 0 },
    { 0xa8fdace9, 23 },
    { 0xeddacea, 23 },
    { 0xd89dacea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164dacf2, 43 },
    { 0x19ddacf2, 46 },
    { 0x73fdacf4, 76 },
    { 0x26edacf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327dacfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fddad03, 33 },
    { 0, 0 },
    { 0xbe7dad05, 55 },
    { 0x403dad06, 81 },
    { 0xce2dad07, 76 },
    { 0xb62dad08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbdad0c, 98 },
    { 0x244dad0d, 90 },
    { 0xd83dad0c, 93 },
    { 0x32ddad0f, 95 },
    { 0x65dad0f, 98 },
    { 0x719dad11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2dad16, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1adad1e, 67 },
    { 0x4fcdad1e, 99 },
    { 0x815dad1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30dad27, 75 },
    { 0x6dddad28, 33 },
    { 0, 0 },
    { 0x8badad2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2dad2d, 33 },
    { 0xfeadad2e, 90 },
    { 0, 0 },
    { 0x14cdad30, 90 },
    { 0x161dad31, 43 },
    { 0xd07dad32, 76 },
    { 0x51cdad31, 33 },
    { 0x9ddad30, 43 },
    { 0xfa5dad35, 95 },
    { 0, 0 },
    { 0xc91dad37, 76 },
    { 0, 0 },
    { 0x773dad39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aedad3d, 99 },
    { 0x75cdad3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x686dad41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7dad44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afdad4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35dad50, 23 },
    { 0xed6dad50, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964dad57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ddad5d, 61 },
    { 0x992dad5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573dad65, 33 },
    { 0x3e8dad65, 76 },
    { 0, 0 },
    { 0xb16dad68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985dad6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51edad6e, 77 },
    { 0x5d4dad6f, 98 },
    { 0xad4dad6e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77edad77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cddad7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28dad7d, 98 },
    { 0x5fcdad7e, 90 },
    { 0x5b4dad7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dad89, 33 },
    { 0xd1fdad8a, 78 },
    { 0xe51dad8a, 78 },
    { 0, 0 },
    { 0xcfadad8d, 23 },
    { 0xbdddad8e, 61 },
    { 0x17dad8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88dad93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602dad99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4dad9f, 43 },
    { 0, 0 },
    { 0xd1cdada1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ddada4, 90 },
    { 0, 0 },
    { 0xa98dada6, 34 },
    { 0x597dada7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bdadaa, 23 },
    { 0xf82dadab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x127dadae, 88 },
    { 0, 0 },
    { 0x25bdadb0, 90 },
    { 0x518dadb1, 55 },
    { 0xc6bdadb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1dadb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dadb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcdadbc, 61 },
    { 0x83adadbc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fedadc2, 66 },
    { 0, 0 },
    { 0x3e6dadc4, 90 },
    { 0, 0 },
    { 0x76cdadc6, 23 },
    { 0x234dadc7, 25 },
    { 0x1dbdadc8, 76 },
    { 0x8e2dadc8, 93 },
    { 0x4c4dadca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95edadd8, 76 },
    { 0, 0 },
    { 0x58adadda, 23 },
    { 0, 0 },
    { 0xbc1daddc, 87 },
    { 0xd9adaddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbdade3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6dade9, 66 },
    { 0xbebdadea, 100 },
    { 0x6dadade9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01dadf1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdadf4, 100 },
    { 0, 0 },
    { 0x5d4dadf6, 61 },
    { 0x3a1dadf7, 61 },
    { 0x539dadf8, 98 },
    { 0x29bdadf9, 95 },
    { 0, 0 },
    { 0xae5dadfb, 90 },
    { 0xc2bdadfc, 34 },
    { 0x2c8dadfd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fdae02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38dae05, 90 },
    { 0x82cdae06, 90 },
    { 0, 0 },
    { 0xebbdae08, 90 },
    { 0x223dae09, 95 },
    { 0x3d7dae09, 95 },
    { 0x9b8dae09, 95 },
    { 0x439dae08, 90 },
    { 0, 0 },
    { 0x672dae0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111dae12, 94 },
    { 0xa57dae13, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63edae18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x627dae1b, 33 },
    { 0x232dae1b, 46 },
    { 0xc3adae1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfdae21, 66 },
    { 0, 0 },
    { 0x8b7dae23, 33 },
    { 0, 0 },
    { 0xa3bdae25, 98 },
    { 0xa5edae25, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbdae2b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dae33, 90 },
    { 0xe48dae34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2dae39, 61 },
    { 0, 0 },
    { 0x888dae3b, 61 },
    { 0x2aedae3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49dae41, 75 },
    { 0xd95dae42, 77 },
    { 0, 0 },
    { 0x5fadae44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cdae49, 52 },
    { 0x4bdae4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46dae52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187dae58, 65 },
    { 0, 0 },
    { 0xa24dae5a, 61 },
    { 0xf1fdae5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461dae61, 61 },
    { 0, 0 },
    { 0x4dddae63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9dae69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6dae6d, 90 },
    { 0x596dae6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x825dae71, 77 },
    { 0xb3dae72, 85 },
    { 0xaaadae71, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaedae76, 91 },
    { 0xa85dae76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badae7d, 33 },
    { 0xafddae7d, 33 },
    { 0x5fedae7e, 77 },
    { 0xc6cdae7f, 34 },
    { 0, 0 },
    { 0x637dae82, 98 },
    { 0xe60dae83, 90 },
    { 0, 0 },
    { 0x5f7dae85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadae88, 90 },
    { 0, 0 },
    { 0x28dae8a, 33 },
    { 0x5c0dae8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddae8e, 61 },
    { 0, 0 },
    { 0x9adae90, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afdae93, 78 },
    { 0x540dae94, 77 },
    { 0xccadae94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ddae9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82daea1, 17 },
    { 0xbccdaea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878daea9, 43 },
    { 0x2f4daeaa, 33 },
    { 0xa1bdaea9, 76 },
    { 0x213daeac, 95 },
    { 0x29bdaead, 97 },
    { 0x362daeae, 76 },
    { 0x401daea9, 93 },
    { 0xd2fdaeb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edaeb4, 90 },
    { 0, 0 },
    { 0x3cadaeb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89daeba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5daebd, 98 },
    { 0x71fdaebe, 76 },
    { 0x53fdaebf, 36 },
    { 0x9abdaebe, 93 },
    { 0x9d4daebe, 95 },
    { 0, 0 },
    { 0x2aadaec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0daec9, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00daecf, 98 },
    { 0x565daed0, 40 },
    { 0xcbdaecf, 98 },
    { 0, 0 },
    { 0xfffdaed3, 23 },
    { 0, 0 },
    { 0xdf5daed5, 75 },
    { 0x8f6daed6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fdaeda, 90 },
    { 0x424daedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fdaedf, 55 },
    { 0, 0 },
    { 0xf9daee1, 34 },
    { 0, 0 },
    { 0x8f9daee3, 90 },
    { 0, 0 },
    { 0x30ddaee5, 95 },
    { 0xcbedaee6, 98 },
    { 0xceedaee7, 86 },
    { 0xba1daee6, 33 },
    { 0x861daee9, 75 },
    { 0x8abdaee9, 33 },
    { 0x8dddaeeb, 55 },
    { 0xa6ddaeea, 93 },
    { 0x5d4daee6, 56 },
    { 0x688daeee, 95 },
    { 0x49cdaeef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34daef7, 90 },
    { 0xd04daef7, 34 },
    { 0x68bdaef7, 33 },
    { 0, 0 },
    { 0xb84daefb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcdaf00, 55 },
    { 0xf35daf01, 55 },
    { 0xce0daf01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cdaf06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9daf0b, 23 },
    { 0x24daf0c, 66 },
    { 0xf8bdaf0c, 76 },
    { 0x7bdaf0b, 67 },
    { 0xea2daf0f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4daf12, 95 },
    { 0xb87daf12, 81 },
    { 0x1d0daf13, 90 },
    { 0x3adaf15, 61 },
    { 0, 0 },
    { 0xd01daf17, 93 },
    { 0xa6fdaf18, 34 },
    { 0, 0 },
    { 0x1fcdaf1a, 67 },
    { 0, 0 },
    { 0xe60daf1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32daf21, 43 },
    { 0xd11daf22, 90 },
    { 0xffddaf22, 93 },
    { 0x687daf21, 34 },
    { 0x88bdaf25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158daf2a, 98 },
    { 0xd10daf2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11edaf30, 23 },
    { 0x232daf30, 90 },
    { 0x829daf30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129daf36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbedaf40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2daf49, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02daf4d, 98 },
    { 0, 0 },
    { 0x5aedaf4f, 61 },
    { 0x6bcdaf50, 33 },
    { 0xb2fdaf51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08daf55, 55 },
    { 0x7addaf55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ddaf5a, 33 },
    { 0x612daf5a, 76 },
    { 0, 0 },
    { 0x70fdaf5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fdaf60, 76 },
    { 0x286daf61, 96 },
    { 0xeefdaf60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bdaf65, 90 },
    { 0x758daf66, 34 },
    { 0xe47daf65, 55 },
    { 0, 0 },
    { 0xab2daf69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0daf70, 33 },
    { 0, 0 },
    { 0x156daf72, 66 },
    { 0x110daf73, 23 },
    { 0x555daf74, 67 },
    { 0, 0 },
    { 0xeb9daf76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14daf7b, 17 },
    { 0, 0 },
    { 0x990daf7d, 65 },
    { 0, 0 },
    { 0xec2daf7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67daf85, 34 },
    { 0, 0 },
    { 0x92cdaf87, 55 },
    { 0x82bdaf88, 23 },
    { 0x1e1daf89, 17 },
    { 0x381daf8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5daf93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedaf9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abdafa5, 43 },
    { 0x660dafa6, 98 },
    { 0x53adafa7, 99 },
    { 0x83dafa6, 43 },
    { 0xf44dafa9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdafb2, 23 },
    { 0, 0 },
    { 0x2c8dafb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210dafb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4dafc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x966dafc5, 99 },
    { 0, 0 },
    { 0xdc4dafc7, 88 },
    { 0x11ddafc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999dafcd, 98 },
    { 0xdc0dafcd, 98 },
    { 0xd91dafcf, 78 },
    { 0xcd3dafce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fdafd3, 33 },
    { 0x555dafd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bdafd7, 90 },
    { 0x18dafd8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ddafdf, 23 },
    { 0x56bdafe0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x783dafe3, 98 },
    { 0x752dafe4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdafea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cdafed, 23 },
    { 0, 0 },
    { 0x612dafef, 61 },
    { 0x108daff0, 99 },
    { 0, 0 },
    { 0x60edaff2, 98 },
    { 0x785daff2, 90 },
    { 0xedcdaff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x772daff7, 81 },
    { 0x331daff7, 76 },
    { 0xb3edaff9, 43 },
    { 0x5ffdaff7, 95 },
    { 0xd2edaffb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28daffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91db002, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9db005, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30db009, 93 },
    { 0x8dfdb00a, 67 },
    { 0xecddb00a, 17 },
    { 0x695db00c, 33 },
    { 0x23edb00d, 90 },
    { 0x6ccdb00e, 61 },
    { 0x5a6db00c, 78 },
    { 0xae9db00c, 90 },
    { 0x97ddb00e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eadb01d, 98 },
    { 0x2bedb01e, 23 },
    { 0x3e1db01f, 65 },
    { 0xd89db01f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbdb023, 90 },
    { 0xfd2db023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ddb027, 68 },
    { 0x32edb027, 33 },
    { 0x5fbdb027, 83 },
    { 0xa70db028, 98 },
    { 0, 0 },
    { 0x5c7db02c, 90 },
    { 0x94cdb02c, 88 },
    { 0x399db02e, 98 },
    { 0x7bdb02d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9db032, 98 },
    { 0xa71db033, 94 },
    { 0x1dedb033, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6db038, 90 },
    { 0xaf4db039, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efdb040, 84 },
    { 0xff2db041, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57db044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cdb057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db05a, 23 },
    { 0, 0 },
    { 0xbd9db05c, 23 },
    { 0x49cdb05d, 33 },
    { 0xffedb05e, 33 },
    { 0xeebdb05f, 33 },
    { 0x3a6db060, 76 },
    { 0x8f7db061, 43 },
    { 0, 0 },
    { 0x32db063, 76 },
    { 0x780db064, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2db06d, 80 },
    { 0, 0 },
    { 0x4abdb06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99db076, 33 },
    { 0x936db077, 34 },
    { 0x234db076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cdb07b, 23 },
    { 0xe61db07c, 3 },
    { 0xb45db07d, 90 },
    { 0, 0 },
    { 0x970db07f, 46 },
    { 0x4adb07f, 90 },
    { 0xf11db07f, 98 },
    { 0xbb9db080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165db088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89db08b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebdb08f, 46 },
    { 0xdadb090, 61 },
    { 0, 0 },
    { 0x37db092, 61 },
    { 0x940db092, 69 },
    { 0x41ddb094, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35db098, 59 },
    { 0x2a6db099, 33 },
    { 0x23bdb099, 24 },
    { 0x4f9db09b, 33 },
    { 0x416db09b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ddb0a2, 90 },
    { 0xee8db0a3, 90 },
    { 0, 0 },
    { 0xd5fdb0a5, 98 },
    { 0x4f5db0a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fadb0aa, 33 },
    { 0x24edb0ab, 33 },
    { 0, 0 },
    { 0xbecdb0ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abdb0b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fdb0b5, 77 },
    { 0x674db0b6, 43 },
    { 0x75fdb0b6, 33 },
    { 0, 0 },
    { 0xc83db0b9, 66 },
    { 0x933db0ba, 68 },
    { 0x5a7db0b9, 33 },
    { 0xc0fdb0bc, 23 },
    { 0x36fdb0b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdb0c2, 34 },
    { 0, 0 },
    { 0x6e4db0c4, 33 },
    { 0xb1cdb0c4, 43 },
    { 0xc2db0c6, 98 },
    { 0xc34db0c6, 90 },
    { 0, 0 },
    { 0x857db0c9, 61 },
    { 0xaf1db0c9, 100 },
    { 0x246db0ca, 78 },
    { 0x9dcdb0cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7db0cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410db0d4, 61 },
    { 0, 0 },
    { 0x2bfdb0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2db0e0, 17 },
    { 0xfeddb0e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ddb0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ddb0e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63db0ec, 90 },
    { 0x716db0ed, 90 },
    { 0x672db0ec, 52 },
    { 0xcefdb0ef, 52 },
    { 0, 0 },
    { 0xc68db0f1, 98 },
    { 0, 0 },
    { 0xae5db0f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0db0f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98db103, 91 },
    { 0, 0 },
    { 0x1b3db105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12db108, 46 },
    { 0, 0 },
    { 0xd10db10a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42adb10e, 90 },
    { 0, 0 },
    { 0x2b1db110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6db11b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3db121, 65 },
    { 0xa50db122, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edb125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdb129, 77 },
    { 0xf93db12a, 33 },
    { 0xfddb12b, 78 },
    { 0x1edb12b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7db12f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fdb133, 98 },
    { 0x28cdb134, 94 },
    { 0x6efdb135, 33 },
    { 0x4dadb136, 33 },
    { 0x8a7db137, 85 },
    { 0xb19db134, 76 },
    { 0x334db139, 90 },
    { 0x72edb13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb13d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7db140, 33 },
    { 0, 0 },
    { 0x2b0db142, 78 },
    { 0x354db143, 52 },
    { 0x1b8db144, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bdb148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x492db14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x867db14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00db153, 98 },
    { 0x271db154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06db158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cdb15c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdb15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x859db162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284db166, 77 },
    { 0x475db166, 34 },
    { 0, 0 },
    { 0x571db169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2db170, 33 },
    { 0, 0 },
    { 0x9cadb172, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x967db175, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1db179, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aedb17c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00db180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fadb183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1db188, 61 },
    { 0x635db189, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db18c, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5db191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19db196, 43 },
    { 0, 0 },
    { 0x338db198, 94 },
    { 0, 0 },
    { 0x984db19a, 77 },
    { 0xd81db19a, 90 },
    { 0x235db19c, 98 },
    { 0xa3cdb19d, 61 },
    { 0xc07db19d, 90 },
    { 0x848db19d, 95 },
    { 0x2d2db1a0, 34 },
    { 0xf82db1a1, 55 },
    { 0, 0 },
    { 0xa74db1a3, 61 },
    { 0, 0 },
    { 0x901db1a5, 93 },
    { 0, 0 },
    { 0x6cbdb1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fddb1ab, 61 },
    { 0x380db1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71edb1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518db1b6, 87 },
    { 0, 0 },
    { 0xd1edb1b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafdb1bb, 98 },
    { 0xb4ddb1bc, 95 },
    { 0x734db1bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x634db1c0, 61 },
    { 0xab6db1c1, 55 },
    { 0, 0 },
    { 0xc81db1c3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cdb1c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcdb1cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ddb1cf, 61 },
    { 0x889db1d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49adb1d7, 84 },
    { 0x3d6db1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28db1e2, 23 },
    { 0, 0 },
    { 0x8eedb1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72edb1e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805db1ee, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6db1f8, 67 },
    { 0xa0bdb1f9, 52 },
    { 0xce2db1fa, 76 },
    { 0, 0 },
    { 0x7bdb1fc, 95 },
    { 0x415db1fd, 23 },
    { 0xc01db1fe, 33 },
    { 0, 0 },
    { 0x8aadb200, 90 },
    { 0xe66db201, 33 },
    { 0x390db201, 46 },
    { 0, 0 },
    { 0x83cdb204, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4db209, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bddb20f, 61 },
    { 0xf29db210, 23 },
    { 0xa2fdb20f, 90 },
    { 0x66db210, 67 },
    { 0, 0 },
    { 0x85edb214, 55 },
    { 0xc7cdb214, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x594db218, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfdb229, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34db22c, 51 },
    { 0xbf8db22c, 55 },
    { 0, 0 },
    { 0xed6db22f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bdb232, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fdb237, 90 },
    { 0, 0 },
    { 0xbeddb239, 61 },
    { 0x61db23a, 33 },
    { 0xb7cdb23b, 77 },
    { 0xef1db23b, 33 },
    { 0, 0 },
    { 0x40adb23e, 61 },
    { 0xf34db23e, 98 },
    { 0x1edb23e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdb244, 66 },
    { 0, 0 },
    { 0x9eedb246, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddb249, 95 },
    { 0, 0 },
    { 0xfc9db24b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedddb253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ddb256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7db259, 81 },
    { 0x64db25a, 90 },
    { 0, 0 },
    { 0x3b5db25c, 61 },
    { 0x7db25d, 98 },
    { 0xc5edb25e, 98 },
    { 0x277db25e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345db265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1edb269, 76 },
    { 0xb53db269, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6edb26f, 33 },
    { 0, 0 },
    { 0x83ddb271, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2db275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84adb27c, 34 },
    { 0x5a0db27d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bdb284, 90 },
    { 0xc75db284, 61 },
    { 0x339db286, 93 },
    { 0xe7db287, 76 },
    { 0x8a1db288, 23 },
    { 0xe0bdb289, 81 },
    { 0x3f0db288, 90 },
    { 0x9dddb289, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ddb296, 98 },
    { 0, 0 },
    { 0x4efdb298, 34 },
    { 0x104db298, 33 },
    { 0x763db298, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16db29d, 100 },
    { 0, 0 },
    { 0x52bdb29f, 98 },
    { 0xba3db29f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504db2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cdb2ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdb2b0, 33 },
    { 0, 0 },
    { 0x73bdb2b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddb2b5, 46 },
    { 0, 0 },
    { 0xec4db2b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6db2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6db2c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692db2cc, 33 },
    { 0x1b1db2cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94db2d1, 52 },
    { 0xed5db2d1, 98 },
    { 0x33db2d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82db2d9, 61 },
    { 0, 0 },
    { 0xd91db2db, 95 },
    { 0, 0 },
    { 0x778db2dd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15db2e0, 33 },
    { 0x25fdb2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59adb2ec, 98 },
    { 0x157db2ed, 64 },
    { 0x477db2ec, 91 },
    { 0xb0adb2ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7db2f2, 85 },
    { 0, 0 },
    { 0x602db2f4, 95 },
    { 0x3b6db2f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x201db2f8, 17 },
    { 0xc44db2f9, 33 },
    { 0xe72db2fa, 93 },
    { 0x406db2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badb301, 95 },
    { 0x8aadb302, 99 },
    { 0x98adb301, 33 },
    { 0x41fdb304, 33 },
    { 0x5ddb304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503db30b, 90 },
    { 0x576db30c, 95 },
    { 0, 0 },
    { 0x18edb30e, 17 },
    { 0x690db30e, 90 },
    { 0xeb9db30f, 33 },
    { 0x42edb30f, 61 },
    { 0x18edb312, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45db315, 98 },
    { 0x9e1db316, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db319, 17 },
    { 0, 0 },
    { 0x196db31b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87db323, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bdb32c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01db332, 33 },
    { 0x751db333, 34 },
    { 0x654db334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fdb337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21db33a, 69 },
    { 0x7d8db33b, 67 },
    { 0x16adb33c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27db341, 61 },
    { 0x21db341, 90 },
    { 0x940db343, 52 },
    { 0x25edb344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cddb347, 66 },
    { 0xdb4db348, 66 },
    { 0x7eedb348, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28db34c, 100 },
    { 0x417db34c, 77 },
    { 0x70cdb34e, 98 },
    { 0, 0 },
    { 0x927db350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5db354, 33 },
    { 0xc0edb355, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fdb35d, 61 },
    { 0x46adb35d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5db366, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83db36e, 90 },
    { 0, 0 },
    { 0x895db370, 99 },
    { 0xbb1db371, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cdb374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bdb377, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360db37d, 90 },
    { 0x425db37e, 93 },
    { 0, 0 },
    { 0x3edb380, 61 },
    { 0, 0 },
    { 0x55fdb382, 66 },
    { 0, 0 },
    { 0xe6cdb384, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9db38c, 23 },
    { 0xd50db38d, 95 },
    { 0xa4adb38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0db391, 61 },
    { 0x3cedb391, 55 },
    { 0x697db393, 76 },
    { 0, 0 },
    { 0x4bdb395, 84 },
    { 0x45adb395, 93 },
    { 0x90edb395, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ddb39d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7db3a1, 98 },
    { 0x2c3db3a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73db3a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fdb3a9, 78 },
    { 0xe54db3a9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991db3ae, 67 },
    { 0x523db3af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8db3b7, 66 },
    { 0x9fadb3b8, 61 },
    { 0, 0 },
    { 0x959db3ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183db3bf, 33 },
    { 0x7d3db3bf, 85 },
    { 0, 0 },
    { 0xb5db3c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ddb3c5, 87 },
    { 0xf40db3c6, 90 },
    { 0x36edb3c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682db3cd, 95 },
    { 0x5c9db3ce, 98 },
    { 0x4abdb3ce, 43 },
    { 0x895db3d0, 98 },
    { 0xad4db3d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x857db3d4, 61 },
    { 0, 0 },
    { 0x6b7db3d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffdb3d9, 100 },
    { 0xe9db3da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81db3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0db3f0, 98 },
    { 0xacdb3f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fadb3fb, 66 },
    { 0x575db3fc, 98 },
    { 0xa90db3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x497db400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eddb403, 46 },
    { 0xe57db404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29db409, 61 },
    { 0xa5cdb409, 85 },
    { 0xeb8db40b, 55 },
    { 0x91cdb40b, 12 },
    { 0xbaadb40d, 61 },
    { 0x11db40e, 23 },
    { 0x94bdb40d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50db413, 33 },
    { 0x9c2db413, 61 },
    { 0xbe1db415, 23 },
    { 0xe86db414, 75 },
    { 0xd16db417, 23 },
    { 0xca7db418, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6db420, 99 },
    { 0x588db421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ddb428, 87 },
    { 0x5b9db428, 87 },
    { 0xc8edb429, 91 },
    { 0, 0 },
    { 0xec1db42c, 76 },
    { 0xa03db42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487db432, 66 },
    { 0xf7adb433, 95 },
    { 0, 0 },
    { 0x3a0db435, 77 },
    { 0xd59db436, 84 },
    { 0xdafdb437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23db443, 60 },
    { 0x664db444, 88 },
    { 0, 0 },
    { 0x670db446, 61 },
    { 0, 0 },
    { 0x597db448, 23 },
    { 0x5d8db449, 23 },
    { 0, 0 },
    { 0x3fadb44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79edb44e, 95 },
    { 0x788db44e, 67 },
    { 0, 0 },
    { 0x73cdb451, 76 },
    { 0x1e5db452, 91 },
    { 0x56fdb452, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4db456, 91 },
    { 0xebddb457, 81 },
    { 0x6b4db457, 33 },
    { 0, 0 },
    { 0x17edb45a, 90 },
    { 0x102db45b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9db45f, 90 },
    { 0xb12db45f, 90 },
    { 0x446db461, 84 },
    { 0, 0 },
    { 0xc5fdb463, 61 },
    { 0x57cdb463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdb469, 95 },
    { 0x10fdb46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04db46d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x353db470, 34 },
    { 0x271db471, 90 },
    { 0x55ddb472, 93 },
    { 0, 0 },
    { 0x3d2db474, 17 },
    { 0x68bdb474, 55 },
    { 0x9e8db476, 61 },
    { 0x1badb477, 88 },
    { 0, 0 },
    { 0xa87db479, 98 },
    { 0xf27db47a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x488db47d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401db482, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7db485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dedb488, 95 },
    { 0x5f2db488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415db491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6db496, 90 },
    { 0xb54db497, 8 },
    { 0x966db496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91db49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9db4aa, 90 },
    { 0xac6db4aa, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdb4ae, 88 },
    { 0xdebdb4ae, 95 },
    { 0x5e2db4b0, 61 },
    { 0x5b8db4b0, 90 },
    { 0, 0 },
    { 0xccedb4b3, 33 },
    { 0, 0 },
    { 0x7b9db4b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4db4ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3edb4bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eddb4c3, 95 },
    { 0xd09db4c4, 80 },
    { 0, 0 },
    { 0xcf1db4c6, 76 },
    { 0x1a6db4c6, 95 },
    { 0, 0 },
    { 0xd34db4c9, 55 },
    { 0x718db4ca, 90 },
    { 0xcb2db4cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245db4d6, 95 },
    { 0x547db4d7, 90 },
    { 0, 0 },
    { 0x488db4d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601db4dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbddb4e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63db4ea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4db4ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6db4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362db4f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527db4fb, 23 },
    { 0, 0 },
    { 0x8eadb4fd, 33 },
    { 0xdd2db4fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9addb507, 34 },
    { 0x6b8db507, 55 },
    { 0x262db509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478db510, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335db51b, 43 },
    { 0x4addb51c, 33 },
    { 0xa48db51b, 23 },
    { 0x906db51e, 95 },
    { 0, 0 },
    { 0x996db520, 91 },
    { 0x99adb521, 23 },
    { 0x388db522, 24 },
    { 0, 0 },
    { 0x723db524, 95 },
    { 0x405db525, 66 },
    { 0xe3ddb524, 98 },
    { 0x9c4db526, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcdb52a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x257db52d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1db534, 61 },
    { 0xb6edb534, 76 },
    { 0, 0 },
    { 0xb05db537, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50db53a, 55 },
    { 0x8aadb53b, 43 },
    { 0, 0 },
    { 0x624db53d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05db541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42adb548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfadb54d, 88 },
    { 0, 0 },
    { 0xe58db54f, 90 },
    { 0, 0 },
    { 0x742db551, 76 },
    { 0x207db552, 99 },
    { 0x932db552, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7db55b, 55 },
    { 0, 0 },
    { 0x6a5db55d, 93 },
    { 0x819db55d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbdb564, 40 },
    { 0x2dedb565, 98 },
    { 0x35ddb566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcdb56a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52db56e, 34 },
    { 0x36ddb56f, 23 },
    { 0x65edb56f, 98 },
    { 0x60edb56f, 67 },
    { 0x7f9db572, 91 },
    { 0x79edb572, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8db57a, 90 },
    { 0x7cddb57b, 36 },
    { 0x332db57a, 43 },
    { 0x3a1db57d, 81 },
    { 0xd9bdb57e, 90 },
    { 0x65adb57d, 33 },
    { 0xf85db57e, 90 },
    { 0, 0 },
    { 0xb9fdb582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddb588, 33 },
    { 0xf14db589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbdb58c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137db590, 23 },
    { 0x262db590, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x283db594, 46 },
    { 0xb97db594, 23 },
    { 0xb3bdb596, 95 },
    { 0, 0 },
    { 0x2cbdb598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1adb5a9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdb5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cedb5b2, 67 },
    { 0xfe4db5b3, 78 },
    { 0x9c0db5b3, 91 },
    { 0, 0 },
    { 0xd0fdb5b6, 65 },
    { 0xa00db5b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7addb5ba, 90 },
    { 0x42db5bb, 98 },
    { 0xe05db5bc, 23 },
    { 0xd18db5bc, 61 },
    { 0xbd8db5be, 61 },
    { 0x70fdb5be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8db5c5, 23 },
    { 0, 0 },
    { 0xf44db5c7, 65 },
    { 0, 0 },
    { 0x903db5c9, 46 },
    { 0x406db5c9, 48 },
    { 0, 0 },
    { 0xa40db5cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x758db5cf, 78 },
    { 0, 0 },
    { 0x372db5d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdb5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fdb5d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83db5de, 81 },
    { 0xd6edb5df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929db5e3, 61 },
    { 0x71db5e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb5e7, 33 },
    { 0xd63db5e7, 75 },
    { 0xba2db5e7, 75 },
    { 0x979db5e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76db5ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601db5f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efdb5f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0db603, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4db607, 90 },
    { 0x586db607, 95 },
    { 0x281db609, 76 },
    { 0x585db60a, 93 },
    { 0x799db60b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ddb60e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296db612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36db615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0db619, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fdb61e, 33 },
    { 0x36cdb61f, 52 },
    { 0x6d3db61f, 23 },
    { 0, 0 },
    { 0x9d8db622, 55 },
    { 0xac6db622, 91 },
    { 0, 0 },
    { 0xa7adb625, 90 },
    { 0x318db626, 75 },
    { 0, 0 },
    { 0xb84db628, 100 },
    { 0x27db629, 90 },
    { 0xa03db629, 33 },
    { 0x574db629, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0db62e, 85 },
    { 0xfa7db62e, 90 },
    { 0x12ddb62e, 76 },
    { 0, 0 },
    { 0xb12db632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0db63a, 33 },
    { 0, 0 },
    { 0x771db63c, 87 },
    { 0xcf2db63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db640, 100 },
    { 0x5dddb641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb645, 66 },
    { 0x8e0db646, 91 },
    { 0xc48db647, 67 },
    { 0xd8adb646, 95 },
    { 0xdd8db649, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0db64f, 23 },
    { 0xaa5db650, 90 },
    { 0, 0 },
    { 0xa1edb652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492db65b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3db65f, 76 },
    { 0, 0 },
    { 0x1cadb661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabdb66c, 61 },
    { 0, 0 },
    { 0x9a2db66e, 46 },
    { 0xddbdb66f, 87 },
    { 0x527db66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdb678, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddb67b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5db680, 95 },
    { 0, 0 },
    { 0xbbbdb682, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66db686, 95 },
    { 0x8d5db687, 43 },
    { 0xa44db688, 80 },
    { 0, 0 },
    { 0xbf1db68a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556db68e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7db691, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6adb695, 90 },
    { 0x3c0db696, 76 },
    { 0x9d1db696, 43 },
    { 0x7d4db698, 94 },
    { 0xdddb698, 46 },
    { 0xa31db69a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556db6a2, 90 },
    { 0x4b3db6a3, 61 },
    { 0x34edb6a2, 95 },
    { 0x1d9db6a5, 93 },
    { 0x68bdb6a6, 90 },
    { 0xfa3db6a4, 55 },
    { 0, 0 },
    { 0xf6adb6a9, 90 },
    { 0, 0 },
    { 0xc9edb6ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319db6af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cddb6b2, 90 },
    { 0, 0 },
    { 0x17adb6b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5db6b7, 95 },
    { 0, 0 },
    { 0x473db6b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133db6c0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb6c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6db6c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdb6d2, 95 },
    { 0x3bfdb6d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbdb6d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5db6de, 33 },
    { 0xb6cdb6df, 91 },
    { 0x102db6df, 61 },
    { 0x6bddb6e1, 66 },
    { 0x771db6e0, 33 },
    { 0, 0 },
    { 0x1fddb6e4, 100 },
    { 0, 0 },
    { 0x478db6e6, 90 },
    { 0xcd8db6e7, 23 },
    { 0x9ecdb6e7, 23 },
    { 0xb5fdb6e7, 95 },
    { 0x6c1db6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bdb6ed, 33 },
    { 0x589db6ed, 61 },
    { 0xd98db6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bdb6f2, 90 },
    { 0, 0 },
    { 0xd5adb6f4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55adb6f8, 98 },
    { 0, 0 },
    { 0xd61db6fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cdb700, 76 },
    { 0, 0 },
    { 0xcc4db702, 90 },
    { 0xeb4db703, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201db70b, 33 },
    { 0, 0 },
    { 0xf20db70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdb710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6db715, 93 },
    { 0xd75db716, 88 },
    { 0x66edb715, 17 },
    { 0xf9fdb718, 56 },
    { 0, 0 },
    { 0x4c9db71a, 85 },
    { 0x8fbdb71b, 91 },
    { 0x243db71b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cdb72a, 95 },
    { 0, 0 },
    { 0x68ddb72c, 76 },
    { 0x96edb72d, 76 },
    { 0x666db72e, 40 },
    { 0x728db72f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fdb737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4adb73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60adb73d, 77 },
    { 0, 0 },
    { 0x568db73f, 33 },
    { 0x34db740, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddb743, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614db747, 84 },
    { 0xf63db748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859db74e, 23 },
    { 0, 0 },
    { 0x8db750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250db753, 85 },
    { 0xde9db754, 98 },
    { 0xe0db755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75db75a, 52 },
    { 0x522db75b, 91 },
    { 0x89fdb75c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x806db75f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2db76c, 81 },
    { 0x101db76c, 52 },
    { 0x5cdb76e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968db772, 43 },
    { 0x1e2db773, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921db779, 98 },
    { 0x953db77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4db77f, 90 },
    { 0, 0 },
    { 0x289db781, 23 },
    { 0x409db782, 90 },
    { 0xdc1db783, 23 },
    { 0x974db784, 100 },
    { 0xc07db784, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cdb78c, 84 },
    { 0x7b2db78d, 38 },
    { 0, 0 },
    { 0xd9db78f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fdb794, 97 },
    { 0x4b2db794, 75 },
    { 0, 0 },
    { 0xf5edb797, 23 },
    { 0x5e7db798, 43 },
    { 0x62bdb798, 90 },
    { 0x528db79a, 55 },
    { 0xcfddb797, 43 },
    { 0x99bdb79c, 99 },
    { 0xc18db79d, 93 },
    { 0xd22db79e, 76 },
    { 0xe92db79a, 90 },
    { 0xafdb79f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72db7a3, 90 },
    { 0x19ddb7a4, 23 },
    { 0x274db7a5, 68 },
    { 0x2bfdb7a3, 55 },
    { 0x5c5db7a7, 61 },
    { 0x6edb7a8, 88 },
    { 0xa3bdb7a5, 61 },
    { 0, 0 },
    { 0x9dddb7ab, 76 },
    { 0x324db7ac, 46 },
    { 0x238db7ab, 76 },
    { 0xa0fdb7ab, 76 },
    { 0xcbddb7ab, 66 },
    { 0x69cdb7af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2db7b6, 55 },
    { 0x81ddb7b7, 61 },
    { 0x407db7b8, 76 },
    { 0x5fcdb7b8, 90 },
    { 0xb44db7b8, 61 },
    { 0x3e1db7ba, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x79adb7be, 75 },
    { 0x4ebdb7bf, 90 },
    { 0x10cdb7c0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7db7c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdb7cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9db7cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53db7d2, 23 },
    { 0xc83db7d3, 90 },
    { 0x7a4db7d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcdb7da, 90 },
    { 0x655db7da, 36 },
    { 0x316db7da, 84 },
    { 0xcbfdb7dd, 90 },
    { 0xa83db7de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdb7e6, 55 },
    { 0x54db7e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cadb7ea, 33 },
    { 0xa6cdb7ea, 67 },
    { 0, 0 },
    { 0x26fdb7ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240db7f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fedb7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44db7fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28db800, 33 },
    { 0x764db801, 84 },
    { 0, 0 },
    { 0x6badb803, 92 },
    { 0x666db803, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66db809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdb811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb81d, 33 },
    { 0x6ffdb81e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x141db821, 23 },
    { 0xe0fdb822, 90 },
    { 0x425db823, 90 },
    { 0x6f4db823, 34 },
    { 0x776db824, 87 },
    { 0xeedb826, 34 },
    { 0, 0 },
    { 0xdc8db828, 87 },
    { 0xa91db829, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dadb82d, 33 },
    { 0x6b6db82e, 23 },
    { 0xdaddb82e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5db833, 99 },
    { 0xbfdb834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702db83c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700db844, 48 },
    { 0xd66db845, 23 },
    { 0x721db846, 90 },
    { 0xb3bdb845, 55 },
    { 0x2d9db848, 95 },
    { 0x3d7db845, 55 },
    { 0xcc7db849, 101 },
    { 0x38db848, 91 },
    { 0x322db84b, 64 },
    { 0, 0 },
    { 0x1fdb84e, 90 },
    { 0x1f2db84f, 91 },
    { 0xbc5db850, 91 },
    { 0xc29db851, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabdb854, 23 },
    { 0x24edb855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedb858, 99 },
    { 0xc21db859, 61 },
    { 0x1aadb85a, 33 },
    { 0, 0 },
    { 0x994db85c, 99 },
    { 0x54adb85d, 95 },
    { 0xa5edb85e, 84 },
    { 0x75bdb85f, 95 },
    { 0x576db860, 46 },
    { 0x726db860, 56 },
    { 0x656db862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bdb865, 90 },
    { 0x77db866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddb872, 98 },
    { 0, 0 },
    { 0x168db874, 98 },
    { 0x81fdb875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbdb879, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855db88e, 98 },
    { 0xedbdb88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb893, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0edb896, 87 },
    { 0xb69db897, 23 },
    { 0, 0 },
    { 0x486db899, 55 },
    { 0, 0 },
    { 0xfcbdb89b, 87 },
    { 0xfa6db89c, 90 },
    { 0x55edb89c, 61 },
    { 0x7d6db89d, 95 },
    { 0x455db89f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cdb8a4, 67 },
    { 0, 0 },
    { 0xda2db8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacdb8a9, 67 },
    { 0xf1edb8a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cdb8af, 90 },
    { 0xfacdb8b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376db8bb, 80 },
    { 0x3addb8bc, 61 },
    { 0xc86db8bc, 65 },
    { 0, 0 },
    { 0x11adb8bf, 33 },
    { 0xa8db8c0, 75 },
    { 0, 0 },
    { 0xefadb8c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x37adb8c5, 95 },
    { 0xbcddb8c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfdb8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffdb8cd, 34 },
    { 0xbe5db8ce, 66 },
    { 0x340db8cf, 95 },
    { 0xbe3db8cf, 90 },
    { 0x78ddb8cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4db8d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4db8e4, 23 },
    { 0, 0 },
    { 0x343db8e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbdb8eb, 23 },
    { 0, 0 },
    { 0x466db8ed, 84 },
    { 0x31ddb8ed, 33 },
    { 0xe57db8ef, 98 },
    { 0x4d0db8f0, 76 },
    { 0x57ddb8f0, 94 },
    { 0x8c2db8f1, 55 },
    { 0xd2db8f1, 23 },
    { 0xcf0db8f4, 100 },
    { 0, 0 },
    { 0x98db8f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcdb8f9, 55 },
    { 0, 0 },
    { 0x68ddb8fb, 33 },
    { 0x671db8fb, 99 },
    { 0, 0 },
    { 0x86cdb8fe, 95 },
    { 0xfcddb8ff, 33 },
    { 0, 0 },
    { 0xb32db901, 85 },
    { 0xe95db902, 100 },
    { 0, 0 },
    { 0x2eedb904, 99 },
    { 0x2b4db905, 87 },
    { 0x821db906, 46 },
    { 0x94ddb906, 87 },
    { 0, 0 },
    { 0x9cbdb909, 40 },
    { 0, 0 },
    { 0x810db90b, 78 },
    { 0xa92db90b, 84 },
    { 0x921db90d, 98 },
    { 0x9c8db90c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8db914, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edb91d, 95 },
    { 0xe2adb91e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ddb921, 98 },
    { 0x13ddb922, 55 },
    { 0x773db921, 96 },
    { 0x957db923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x990db927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3db92a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9db92e, 23 },
    { 0x4bfdb92e, 43 },
    { 0x8b2db92e, 76 },
    { 0x65adb92e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23db93a, 23 },
    { 0xfa8db93b, 99 },
    { 0x27db93a, 90 },
    { 0x2dadb93c, 46 },
    { 0x1f5db93e, 23 },
    { 0x28edb93f, 53 },
    { 0xbc4db940, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbdb946, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db949, 95 },
    { 0x1edb94a, 33 },
    { 0xafddb94b, 4 },
    { 0xb4db94a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20db950, 95 },
    { 0, 0 },
    { 0xb88db952, 76 },
    { 0x15db953, 91 },
    { 0xae2db952, 90 },
    { 0x37cdb955, 90 },
    { 0x8c0db952, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fdb959, 90 },
    { 0, 0 },
    { 0x84adb95b, 33 },
    { 0, 0 },
    { 0x5c0db95d, 75 },
    { 0, 0 },
    { 0xae9db95f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7db965, 66 },
    { 0xd00db966, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fdb96d, 90 },
    { 0, 0 },
    { 0x1d6db96f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0db972, 33 },
    { 0x1a1db973, 43 },
    { 0xd67db974, 67 },
    { 0x25bdb975, 95 },
    { 0x690db976, 68 },
    { 0x9dbdb972, 23 },
    { 0x43bdb978, 95 },
    { 0xab5db978, 80 },
    { 0xd9cdb97a, 46 },
    { 0x915db979, 25 },
    { 0x1a2db97c, 98 },
    { 0, 0 },
    { 0x640db97e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdb981, 95 },
    { 0xafddb982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98db987, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484db99c, 95 },
    { 0, 0 },
    { 0x56bdb99e, 75 },
    { 0, 0 },
    { 0x247db9a0, 46 },
    { 0xcb7db9a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57db9a4, 46 },
    { 0xd08db9a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecadb9b3, 90 },
    { 0xddbdb9b4, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83edb9b9, 95 },
    { 0x3ffdb9ba, 78 },
    { 0xde8db9bb, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862db9c0, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74db9c4, 100 },
    { 0xac1db9c5, 87 },
    { 0, 0 },
    { 0xa61db9c7, 65 },
    { 0xe50db9c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64db9d2, 23 },
    { 0xdaddb9d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdb9d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fdb9d9, 17 },
    { 0x15fdb9d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20db9dd, 33 },
    { 0xb5fdb9de, 61 },
    { 0, 0 },
    { 0x1a6db9e0, 95 },
    { 0xabcdb9e1, 66 },
    { 0x121db9e2, 91 },
    { 0, 0 },
    { 0xa41db9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9db9e7, 80 },
    { 0x824db9e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fadb9eb, 90 },
    { 0xe2bdb9ec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8adb9ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71db9f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91edb9fe, 34 },
    { 0x9f0db9ff, 93 },
    { 0x687dba00, 61 },
    { 0, 0 },
    { 0xc43dba02, 33 },
    { 0x7cfdba03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecdba07, 23 },
    { 0x77adba07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8dba0b, 85 },
    { 0x1dbdba0c, 100 },
    { 0x9ecdba0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409dba11, 95 },
    { 0, 0 },
    { 0x9d4dba13, 61 },
    { 0x5a2dba14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bdba17, 23 },
    { 0xa98dba18, 90 },
    { 0xc2adba17, 33 },
    { 0xe17dba18, 94 },
    { 0xcbcdba1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4adba21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bdba25, 90 },
    { 0x22edba25, 61 },
    { 0, 0 },
    { 0xfd3dba28, 90 },
    { 0xf66dba29, 84 },
    { 0xdaadba2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cdba2d, 68 },
    { 0, 0 },
    { 0x59cdba2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8dba32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdba36, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x390dba39, 76 },
    { 0, 0 },
    { 0x39fdba3b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99dba3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddba42, 95 },
    { 0x40cdba42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dba46, 33 },
    { 0x2f3dba47, 98 },
    { 0xcf8dba47, 90 },
    { 0xc3ddba49, 76 },
    { 0x1bedba48, 34 },
    { 0xf5fdba49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9dba4e, 78 },
    { 0x566dba4f, 55 },
    { 0x2ebdba4e, 43 },
    { 0x4b0dba51, 75 },
    { 0x6a5dba52, 23 },
    { 0x152dba53, 99 },
    { 0x4b6dba54, 33 },
    { 0xec5dba55, 61 },
    { 0x9b3dba56, 43 },
    { 0x6b9dba53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2dba5c, 95 },
    { 0x992dba5c, 87 },
    { 0x777dba5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4dba61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49dba66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dba69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdba6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3dba75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdba79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cadba7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731dba81, 90 },
    { 0xdcfdba82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901dba89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7dba8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6addba91, 53 },
    { 0xfe6dba92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3adba95, 90 },
    { 0xeaddba96, 46 },
    { 0xf71dba97, 77 },
    { 0x2b2dba97, 33 },
    { 0x7e1dba99, 66 },
    { 0x961dba98, 23 },
    { 0x456dba97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765dbaa1, 52 },
    { 0xc1adbaa2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1edbaa5, 61 },
    { 0, 0 },
    { 0x23fdbaa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810dbaaf, 76 },
    { 0, 0 },
    { 0x4d3dbab1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69dbab6, 52 },
    { 0x80adbab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76edbaba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ddbabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdbac0, 17 },
    { 0xcb6dbac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5dbac5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x602dbac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffdbacc, 100 },
    { 0x987dbacd, 17 },
    { 0xb1edbacd, 23 },
    { 0xcbadbacd, 93 },
    { 0, 0 },
    { 0xabdbad1, 61 },
    { 0xf73dbad2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ddbad7, 43 },
    { 0, 0 },
    { 0xa60dbad9, 98 },
    { 0, 0 },
    { 0x3c8dbadb, 55 },
    { 0x93dbadc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42dbadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdbae4, 88 },
    { 0xd42dbae4, 90 },
    { 0xd78dbae5, 95 },
    { 0x472dbae6, 76 },
    { 0x617dbae8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cdbaf0, 85 },
    { 0xfa0dbaf0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1addbaf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dbaf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945dbafc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3edbaff, 98 },
    { 0x123dbb00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25dbb07, 93 },
    { 0, 0 },
    { 0xaf1dbb09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3adbb11, 68 },
    { 0xc37dbb11, 95 },
    { 0x68fdbb11, 95 },
    { 0xccddbb12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1dbb18, 33 },
    { 0, 0 },
    { 0xf61dbb1a, 95 },
    { 0xd2cdbb1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edbb1e, 56 },
    { 0x116dbb1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dbb26, 93 },
    { 0x6d9dbb27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedbb2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538dbb31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abdbb34, 93 },
    { 0, 0 },
    { 0x39adbb36, 65 },
    { 0, 0 },
    { 0x986dbb38, 33 },
    { 0x3c4dbb39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11dbb3c, 68 },
    { 0x55dbb3d, 23 },
    { 0x71ddbb3d, 98 },
    { 0x708dbb3f, 91 },
    { 0xd4edbb40, 36 },
    { 0x1efdbb41, 43 },
    { 0, 0 },
    { 0xd71dbb43, 66 },
    { 0, 0 },
    { 0x13edbb45, 55 },
    { 0, 0 },
    { 0x483dbb47, 76 },
    { 0xf42dbb48, 61 },
    { 0xf1adbb49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbdbb4c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdbb52, 76 },
    { 0, 0 },
    { 0x35cdbb54, 61 },
    { 0, 0 },
    { 0x180dbb56, 23 },
    { 0xf8cdbb57, 90 },
    { 0x698dbb58, 87 },
    { 0, 0 },
    { 0x9e2dbb5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08dbb63, 93 },
    { 0x9a4dbb63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6dbb67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fdbb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fdbb6f, 90 },
    { 0x950dbb70, 90 },
    { 0xd4bdbb71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ddbb76, 34 },
    { 0, 0 },
    { 0x6a7dbb78, 33 },
    { 0x6dbb79, 93 },
    { 0xc22dbb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9dbb81, 61 },
    { 0, 0 },
    { 0xa51dbb83, 88 },
    { 0, 0 },
    { 0x1d9dbb85, 43 },
    { 0x621dbb86, 23 },
    { 0, 0 },
    { 0xa25dbb88, 82 },
    { 0, 0 },
    { 0x620dbb8a, 23 },
    { 0xd6adbb8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5dbb90, 55 },
    { 0xcd8dbb91, 23 },
    { 0x9bdbb91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefedbb9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212dbba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x472dbba5, 99 },
    { 0, 0 },
    { 0x992dbba7, 76 },
    { 0x9c3dbba7, 84 },
    { 0, 0 },
    { 0x5e3dbbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868dbbaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460dbbb5, 23 },
    { 0x20bdbbb6, 61 },
    { 0xb73dbbb7, 95 },
    { 0, 0 },
    { 0x66ddbbb9, 66 },
    { 0x6dfdbbb9, 25 },
    { 0, 0 },
    { 0x620dbbbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499dbbc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784dbbc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cdbbcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4dbbd3, 77 },
    { 0xc72dbbd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x963dbbd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82adbbdf, 90 },
    { 0, 0 },
    { 0x9b0dbbe1, 95 },
    { 0, 0 },
    { 0x623dbbe3, 33 },
    { 0xd69dbbe3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x416dbbe7, 33 },
    { 0x371dbbe8, 68 },
    { 0x7f1dbbe8, 55 },
    { 0x538dbbe9, 99 },
    { 0xbddbbe7, 84 },
    { 0x4b4dbbec, 36 },
    { 0x96dbbed, 43 },
    { 0xc96dbbe9, 40 },
    { 0x276dbbea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32dbbf4, 68 },
    { 0x2b5dbbf5, 23 },
    { 0x8d8dbbf6, 34 },
    { 0xf38dbbf7, 85 },
    { 0x5e2dbbf4, 99 },
    { 0x4abdbbf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcdbbfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816dbc00, 61 },
    { 0, 0 },
    { 0x6a4dbc02, 77 },
    { 0, 0 },
    { 0x548dbc04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dbc0b, 98 },
    { 0x343dbc0c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451dbc12, 92 },
    { 0x273dbc13, 99 },
    { 0x1f0dbc13, 23 },
    { 0x18bdbc15, 76 },
    { 0x1e8dbc16, 33 },
    { 0x846dbc17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dbc1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625dbc20, 76 },
    { 0x280dbc20, 34 },
    { 0x249dbc21, 78 },
    { 0x850dbc21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6dbc27, 33 },
    { 0xe26dbc27, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edbc2c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x823dbc2f, 76 },
    { 0, 0 },
    { 0x990dbc31, 90 },
    { 0, 0 },
    { 0x9f8dbc33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3badbc38, 96 },
    { 0, 0 },
    { 0x960dbc3a, 93 },
    { 0x80cdbc3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386dbc41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336dbc46, 23 },
    { 0xf85dbc46, 61 },
    { 0x2c7dbc48, 87 },
    { 0xa4edbc49, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58adbc4e, 34 },
    { 0xd7dbc4f, 33 },
    { 0xebadbc4e, 43 },
    { 0x263dbc51, 52 },
    { 0, 0 },
    { 0x206dbc53, 80 },
    { 0x1dfdbc53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2adbc65, 99 },
    { 0xdaddbc65, 95 },
    { 0, 0 },
    { 0x65fdbc68, 95 },
    { 0xe9bdbc68, 66 },
    { 0xe34dbc69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1dbc6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36dbc75, 90 },
    { 0, 0 },
    { 0x9f3dbc77, 33 },
    { 0, 0 },
    { 0xdf2dbc79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x887dbc7c, 76 },
    { 0, 0 },
    { 0xa4bdbc7e, 77 },
    { 0x69cdbc7f, 33 },
    { 0x861dbc80, 81 },
    { 0x48edbc81, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2dbc8c, 76 },
    { 0x160dbc8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcdbc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938dbc99, 96 },
    { 0x2addbc9a, 76 },
    { 0x57dbc99, 55 },
    { 0x86adbc9b, 46 },
    { 0xc6cdbc9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadbca3, 95 },
    { 0xec3dbca3, 98 },
    { 0x706dbca5, 61 },
    { 0x912dbca6, 61 },
    { 0, 0 },
    { 0x6b9dbca8, 33 },
    { 0xc45dbca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cdbcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985dbcb4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabdbcbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cddbcbe, 95 },
    { 0xe09dbcbe, 46 },
    { 0xcddbcc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77dbcc3, 61 },
    { 0x8c3dbcc4, 90 },
    { 0, 0 },
    { 0x672dbcc6, 34 },
    { 0x53cdbcc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fdbcd1, 55 },
    { 0x4aadbcd1, 93 },
    { 0x61edbcd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7dbcd9, 34 },
    { 0x5ddbcda, 85 },
    { 0, 0 },
    { 0xf57dbcdc, 85 },
    { 0x93cdbcdd, 93 },
    { 0x8d8dbcde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450dbce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17dbceb, 94 },
    { 0, 0 },
    { 0x7b3dbced, 78 },
    { 0x51edbced, 95 },
    { 0x814dbced, 23 },
    { 0xa42dbcee, 100 },
    { 0xaf9dbcf1, 23 },
    { 0x4ddbcef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56edbcf6, 33 },
    { 0x147dbcf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddbcfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bdbcfd, 91 },
    { 0, 0 },
    { 0xce0dbcff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffdbd04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fdbd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2dbd10, 84 },
    { 0xe2bdbd10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cdbd15, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddbd1a, 36 },
    { 0x4dbdbd1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48dbd1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cdbd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cadbd29, 91 },
    { 0x60adbd2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73dbd2d, 91 },
    { 0x8cdbd2e, 47 },
    { 0, 0 },
    { 0x4ccdbd30, 33 },
    { 0xd98dbd31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eadbd34, 90 },
    { 0xbdcdbd34, 61 },
    { 0x752dbd36, 76 },
    { 0x660dbd37, 66 },
    { 0x86adbd38, 23 },
    { 0, 0 },
    { 0x452dbd3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30dbd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dbd42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x660dbd45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4dbd4b, 99 },
    { 0, 0 },
    { 0x1e5dbd4d, 95 },
    { 0x389dbd4e, 43 },
    { 0x344dbd4f, 33 },
    { 0xfefdbd50, 95 },
    { 0, 0 },
    { 0xd8fdbd52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdbd58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bdbd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ddbd66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9dbd69, 55 },
    { 0, 0 },
    { 0x383dbd6b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbddbd70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38dbd7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239dbd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afdbd81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4dbd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86adbd8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995dbd90, 90 },
    { 0xa32dbd90, 90 },
    { 0, 0 },
    { 0xf23dbd93, 61 },
    { 0xedbdbd93, 65 },
    { 0xeaadbd95, 98 },
    { 0, 0 },
    { 0x329dbd97, 68 },
    { 0xdeedbd97, 61 },
    { 0x3c2dbd98, 61 },
    { 0x989dbd9a, 33 },
    { 0x6f6dbd9b, 34 },
    { 0x968dbd9c, 52 },
    { 0x3a9dbd9d, 23 },
    { 0xb4fdbd9a, 17 },
    { 0xdc3dbd9f, 33 },
    { 0x69adbd9f, 90 },
    { 0, 0 },
    { 0xb9bdbda2, 61 },
    { 0x81cdbda3, 33 },
    { 0x597dbda3, 61 },
    { 0x62edbda2, 99 },
    { 0, 0 },
    { 0xce8dbda7, 33 },
    { 0xc60dbda8, 91 },
    { 0xc44dbda9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808dbdb5, 43 },
    { 0x658dbdb5, 61 },
    { 0, 0 },
    { 0xa65dbdb8, 33 },
    { 0, 0 },
    { 0x92ddbdba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9dbdbd, 95 },
    { 0x7bddbdbd, 43 },
    { 0x37edbdbe, 98 },
    { 0xd62dbdbe, 95 },
    { 0xbffdbdbd, 93 },
    { 0x7d9dbdc2, 88 },
    { 0x32fdbdc2, 55 },
    { 0xe02dbdc3, 90 },
    { 0x6acdbdc5, 90 },
    { 0, 0 },
    { 0xf73dbdc7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2dbdcb, 98 },
    { 0x3dedbdcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43edbdcf, 33 },
    { 0, 0 },
    { 0x952dbdd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bddbdd6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8dbdda, 34 },
    { 0, 0 },
    { 0x5e8dbddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58adbde0, 23 },
    { 0, 0 },
    { 0x1bdbde2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dbdea, 84 },
    { 0, 0 },
    { 0x292dbdec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79adbdf0, 33 },
    { 0x6c1dbdf0, 100 },
    { 0x288dbdf1, 98 },
    { 0xbdedbdf3, 90 },
    { 0x863dbdf0, 90 },
    { 0x239dbdf1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8dbdf8, 83 },
    { 0x423dbdf8, 85 },
    { 0, 0 },
    { 0xa65dbdfb, 95 },
    { 0x57fdbdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x853dbdff, 93 },
    { 0x71adbe00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4dbe04, 87 },
    { 0, 0 },
    { 0xdbdbe06, 98 },
    { 0x5ffdbe07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffdbe0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94edbe0d, 46 },
    { 0, 0 },
    { 0xacfdbe0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459dbe1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddbe23, 23 },
    { 0, 0 },
    { 0xdc5dbe25, 43 },
    { 0x7e7dbe26, 23 },
    { 0x18cdbe26, 34 },
    { 0x755dbe28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x154dbe2b, 76 },
    { 0, 0 },
    { 0x527dbe2d, 90 },
    { 0x817dbe2e, 90 },
    { 0, 0 },
    { 0x31bdbe30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3dbe35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aedbe38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bdbe3b, 33 },
    { 0x8d4dbe3c, 40 },
    { 0x49adbe3b, 99 },
    { 0xd7bdbe3e, 61 },
    { 0, 0 },
    { 0x6d9dbe40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4dbe48, 90 },
    { 0, 0 },
    { 0xcc0dbe4a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40edbe52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50dbe58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7dbe5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x693dbe5e, 34 },
    { 0, 0 },
    { 0x872dbe60, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fdbe64, 33 },
    { 0xc2adbe65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cdbe6d, 90 },
    { 0x141dbe6d, 33 },
    { 0xa32dbe6f, 76 },
    { 0x5addbe6f, 23 },
    { 0x7d8dbe71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dbe76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46edbe79, 84 },
    { 0x6c1dbe7a, 87 },
    { 0, 0 },
    { 0x98bdbe7c, 95 },
    { 0, 0 },
    { 0x588dbe7e, 90 },
    { 0x9b9dbe7f, 61 },
    { 0xe31dbe80, 90 },
    { 0xb5cdbe81, 99 },
    { 0xfc1dbe82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49dbe89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabddbe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8dbe93, 98 },
    { 0xd6ddbe94, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708dbe99, 56 },
    { 0x9c9dbe9a, 33 },
    { 0x7a5dbe9b, 90 },
    { 0xd0fdbe9c, 100 },
    { 0xb40dbe9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193dbea1, 33 },
    { 0x716dbea2, 23 },
    { 0, 0 },
    { 0x152dbea4, 33 },
    { 0x3a4dbea5, 76 },
    { 0x52bdbea5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34dbeaa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dbeb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0dbeb6, 46 },
    { 0x32adbeb7, 90 },
    { 0xe24dbeb8, 93 },
    { 0x1b4dbeb9, 90 },
    { 0x185dbeb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bdbebe, 65 },
    { 0x9d5dbebf, 85 },
    { 0, 0 },
    { 0x40adbec1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66dbec9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabddbecf, 40 },
    { 0, 0 },
    { 0x144dbed1, 96 },
    { 0, 0 },
    { 0xf67dbed3, 93 },
    { 0, 0 },
    { 0x9c7dbed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x231dbed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434dbedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcdbeeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x554dbeee, 98 },
    { 0x938dbeee, 76 },
    { 0xe1dbef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafedbef3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437dbefb, 88 },
    { 0xc0bdbefc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3dbeff, 34 },
    { 0, 0 },
    { 0x8d6dbf01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12dbf04, 96 },
    { 0x608dbf04, 36 },
    { 0, 0 },
    { 0x8d6dbf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fdbf0d, 93 },
    { 0, 0 },
    { 0x683dbf0f, 100 },
    { 0, 0 },
    { 0xa34dbf11, 95 },
    { 0x383dbf12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1dbf17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ddbf1b, 65 },
    { 0, 0 },
    { 0x97dbf1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaedbf20, 67 },
    { 0x4eddbf21, 95 },
    { 0x1c8dbf21, 38 },
    { 0, 0 },
    { 0x670dbf24, 43 },
    { 0, 0 },
    { 0xe80dbf26, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dadbf29, 92 },
    { 0x47ddbf2a, 98 },
    { 0x878dbf2a, 85 },
    { 0xc8bdbf2c, 95 },
    { 0x1efdbf2d, 85 },
    { 0x8dbdbf2d, 66 },
    { 0x7b6dbf2f, 85 },
    { 0x250dbf29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645dbf34, 76 },
    { 0x9d6dbf35, 99 },
    { 0x3acdbf36, 90 },
    { 0x8c0dbf37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x126dbf3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91dbf3f, 98 },
    { 0, 0 },
    { 0xe1dbf41, 95 },
    { 0, 0 },
    { 0x2f3dbf43, 99 },
    { 0, 0 },
    { 0x924dbf45, 61 },
    { 0x36bdbf46, 85 },
    { 0x66bdbf47, 52 },
    { 0x25edbf48, 33 },
    { 0, 0 },
    { 0x7c7dbf4a, 77 },
    { 0xf1fdbf4a, 53 },
    { 0xf19dbf4b, 61 },
    { 0x96adbf4b, 61 },
    { 0xfd4dbf4d, 90 },
    { 0x8f8dbf4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8dbf52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5dbf55, 38 },
    { 0x607dbf56, 76 },
    { 0, 0 },
    { 0xd04dbf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166dbf5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737dbf61, 95 },
    { 0xa04dbf62, 77 },
    { 0, 0 },
    { 0xf8cdbf64, 61 },
    { 0x3f0dbf65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28dbf68, 23 },
    { 0x365dbf68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadbf6c, 84 },
    { 0xe7dbf6d, 33 },
    { 0, 0 },
    { 0xa22dbf6f, 98 },
    { 0x9b1dbf70, 95 },
    { 0x42cdbf71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cdbf76, 85 },
    { 0, 0 },
    { 0xc6fdbf78, 98 },
    { 0x70edbf79, 76 },
    { 0xe75dbf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bdbf7d, 98 },
    { 0, 0 },
    { 0x830dbf7f, 90 },
    { 0x465dbf7f, 68 },
    { 0xe01dbf81, 100 },
    { 0x860dbf82, 90 },
    { 0, 0 },
    { 0xbe2dbf84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17dbf8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1dbf90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddbf99, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x703dbf9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62dbf9f, 61 },
    { 0, 0 },
    { 0x6bfdbfa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fdbfa6, 90 },
    { 0, 0 },
    { 0x5ccdbfa8, 85 },
    { 0x915dbfa9, 76 },
    { 0x8e1dbfaa, 23 },
    { 0x6addbfaa, 78 },
    { 0, 0 },
    { 0xe91dbfad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3dbfb7, 33 },
    { 0xf15dbfb8, 98 },
    { 0x19bdbfb9, 55 },
    { 0xadedbfba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76dbfbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x864dbfc0, 76 },
    { 0, 0 },
    { 0x459dbfc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadbfc9, 23 },
    { 0x8c3dbfca, 55 },
    { 0, 0 },
    { 0xaa8dbfcc, 55 },
    { 0, 0 },
    { 0x484dbfce, 96 },
    { 0, 0 },
    { 0x3e8dbfd0, 90 },
    { 0, 0 },
    { 0xb2ddbfd2, 76 },
    { 0, 0 },
    { 0xf42dbfd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0dbfd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfdbfe2, 78 },
    { 0x30dbfe3, 34 },
    { 0x5d7dbfe2, 98 },
    { 0x5c5dbfe5, 98 },
    { 0xf05dbfe6, 66 },
    { 0x28dbfe5, 43 },
    { 0x54cdbfe8, 98 },
    { 0xdb0dbfe9, 90 },
    { 0x8dddbfea, 95 },
    { 0xed9dbfe9, 90 },
    { 0x5b5dbfec, 90 },
    { 0xa2fdbfed, 99 },
    { 0x94adbfed, 23 },
    { 0xcf7dbfee, 100 },
    { 0x74cdbfed, 61 },
    { 0xb13dbfee, 98 },
    { 0x560dbff2, 61 },
    { 0x2c3dbff3, 90 },
    { 0x630dbff4, 78 },
    { 0xf7adbfee, 23 },
    { 0x3e0dbff0, 34 },
    { 0xc32dbff7, 90 },
    { 0x8e3dbff2, 98 },
    { 0xa65dbfee, 65 },
    { 0xa56dbff3, 99 },
    { 0x7d6dbffb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x100dbffe, 95 },
    { 0xc42dbffe, 95 },
    { 0xf90dbffe, 46 },
    { 0x703dc001, 43 },
    { 0x2fcdc002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2adc006, 90 },
    { 0xb88dc006, 55 },
    { 0x268dc007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bdc00f, 87 },
    { 0, 0 },
    { 0xe59dc011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8dc014, 55 },
    { 0, 0 },
    { 0x244dc016, 95 },
    { 0, 0 },
    { 0xb70dc018, 43 },
    { 0xd40dc019, 33 },
    { 0x663dc018, 90 },
    { 0, 0 },
    { 0x614dc01c, 90 },
    { 0xc87dc01d, 61 },
    { 0x7bfdc01e, 98 },
    { 0x5f8dc01f, 22 },
    { 0x801dc01f, 61 },
    { 0x57ddc021, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0dc027, 95 },
    { 0x6eadc027, 67 },
    { 0x1f2dc028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefdc02d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6dc030, 66 },
    { 0xef3dc031, 90 },
    { 0, 0 },
    { 0x23ddc033, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00dc036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdc039, 98 },
    { 0, 0 },
    { 0xacfdc03b, 75 },
    { 0, 0 },
    { 0x709dc03d, 55 },
    { 0x76edc03e, 90 },
    { 0, 0 },
    { 0x31adc040, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124dc048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc04b, 75 },
    { 0, 0 },
    { 0xcb5dc04d, 61 },
    { 0x946dc04e, 90 },
    { 0x613dc04f, 98 },
    { 0, 0 },
    { 0x93cdc051, 98 },
    { 0xa05dc052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cdc057, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17adc05d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1dc060, 33 },
    { 0xd1bdc061, 61 },
    { 0, 0 },
    { 0xb3dc063, 76 },
    { 0xe48dc064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0dc067, 95 },
    { 0xa1ddc068, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9dc06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4dc073, 33 },
    { 0, 0 },
    { 0x4c5dc075, 87 },
    { 0xe6edc075, 90 },
    { 0xf15dc077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bdc07b, 98 },
    { 0x32dc07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401dc087, 34 },
    { 0xab1dc087, 76 },
    { 0x256dc089, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc08d, 33 },
    { 0, 0 },
    { 0x28ddc08f, 66 },
    { 0x9ebdc090, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dc093, 23 },
    { 0, 0 },
    { 0x7d3dc095, 90 },
    { 0xce4dc096, 100 },
    { 0x3a3dc096, 76 },
    { 0, 0 },
    { 0xf2ddc099, 43 },
    { 0x3c3dc09a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476dc09f, 87 },
    { 0xfd4dc0a0, 33 },
    { 0xdefdc09f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cdc0a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0dc0b0, 85 },
    { 0xec5dc0b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06dc0b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91dc0b7, 65 },
    { 0x7fbdc0b8, 34 },
    { 0xc7cdc0b9, 61 },
    { 0, 0 },
    { 0xa52dc0bb, 95 },
    { 0x9f1dc0bc, 23 },
    { 0x37bdc0bb, 23 },
    { 0x1c3dc0bb, 33 },
    { 0xeeedc0bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc0c5, 23 },
    { 0x70ddc0c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x558dc0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5dc0cc, 87 },
    { 0x73dc0cc, 61 },
    { 0xd05dc0cc, 23 },
    { 0x77dc0cc, 55 },
    { 0xd9ddc0d0, 90 },
    { 0x486dc0d1, 95 },
    { 0x810dc0d1, 78 },
    { 0, 0 },
    { 0x1f0dc0d4, 93 },
    { 0x285dc0d4, 33 },
    { 0xe6ddc0d6, 61 },
    { 0xd78dc0d7, 95 },
    { 0xb9dc0d7, 33 },
    { 0x47fdc0d9, 77 },
    { 0xc90dc0d4, 76 },
    { 0x4f1dc0da, 96 },
    { 0x205dc0db, 98 },
    { 0x334dc0dd, 66 },
    { 0xe71dc0dd, 43 },
    { 0x3bdc0dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3dc0e2, 90 },
    { 0, 0 },
    { 0x2b2dc0e4, 33 },
    { 0xdaedc0e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44adc0e8, 94 },
    { 0x249dc0e9, 95 },
    { 0x21fdc0ea, 31 },
    { 0xccedc0e8, 61 },
    { 0xc72dc0ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90dc0f2, 90 },
    { 0xb04dc0f3, 96 },
    { 0x785dc0f4, 88 },
    { 0x718dc0f5, 90 },
    { 0, 0 },
    { 0xc2edc0f7, 76 },
    { 0, 0 },
    { 0xb70dc0f9, 100 },
    { 0, 0 },
    { 0x5c4dc0fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddc0fe, 61 },
    { 0xda2dc0ff, 96 },
    { 0x36ddc0ff, 90 },
    { 0xf69dc0ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bdc109, 33 },
    { 0, 0 },
    { 0xdf5dc10b, 23 },
    { 0x35adc10c, 90 },
    { 0x85cdc10d, 61 },
    { 0xb89dc10e, 33 },
    { 0xc8cdc10e, 55 },
    { 0x861dc10f, 69 },
    { 0xf96dc10b, 93 },
    { 0x862dc112, 98 },
    { 0x1d7dc113, 34 },
    { 0xc79dc110, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3dc118, 95 },
    { 0, 0 },
    { 0xb40dc11a, 90 },
    { 0, 0 },
    { 0x931dc11c, 61 },
    { 0xfcddc11c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ddc120, 95 },
    { 0, 0 },
    { 0x1dddc122, 33 },
    { 0x208dc123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbedc136, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78edc139, 55 },
    { 0x2c9dc139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x203dc13d, 52 },
    { 0x61dc13e, 98 },
    { 0, 0 },
    { 0xc6fdc140, 33 },
    { 0x6f9dc140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fedc146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5dc14b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cdc14e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x890dc151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859dc15a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x105dc15d, 90 },
    { 0xa22dc15d, 93 },
    { 0x4a5dc15f, 90 },
    { 0x556dc160, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744dc165, 17 },
    { 0x9cddc166, 95 },
    { 0xbf4dc167, 68 },
    { 0x314dc166, 34 },
    { 0, 0 },
    { 0x6e3dc16a, 61 },
    { 0x1b6dc16b, 90 },
    { 0, 0 },
    { 0x6cddc16d, 23 },
    { 0x2bcdc16e, 90 },
    { 0x3d1dc16e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33adc172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10adc176, 33 },
    { 0xd0cdc177, 91 },
    { 0xc5edc178, 90 },
    { 0x139dc179, 34 },
    { 0xceedc17a, 98 },
    { 0x145dc178, 85 },
    { 0x8c8dc17c, 95 },
    { 0xe8fdc17c, 33 },
    { 0x540dc17e, 90 },
    { 0xe14dc17f, 84 },
    { 0x83edc17f, 23 },
    { 0x7bcdc17e, 33 },
    { 0x99fdc180, 43 },
    { 0xae8dc183, 33 },
    { 0xfefdc184, 98 },
    { 0x2c0dc183, 61 },
    { 0xa78dc186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3dc18b, 23 },
    { 0xcd4dc18b, 90 },
    { 0, 0 },
    { 0xed4dc18e, 91 },
    { 0xde6dc18f, 61 },
    { 0x643dc190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1dc19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87dc1a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xafddc1a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45dc1a7, 90 },
    { 0x6a7dc1a8, 85 },
    { 0x777dc1a9, 61 },
    { 0x7dddc1a7, 33 },
    { 0x527dc1ab, 90 },
    { 0xdddc1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdc1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0dc1b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80edc1b8, 90 },
    { 0xa89dc1b9, 34 },
    { 0x8f0dc1ba, 95 },
    { 0, 0 },
    { 0x48cdc1bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2dc1bf, 95 },
    { 0xeedc1c0, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1dc1c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dc1cd, 75 },
    { 0, 0 },
    { 0xe92dc1cf, 55 },
    { 0, 0 },
    { 0xa0dc1d1, 33 },
    { 0xed1dc1d2, 46 },
    { 0xa18dc1d3, 46 },
    { 0x567dc1d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bdc1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648dc1e0, 23 },
    { 0x67adc1e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdc1e7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fdc1eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fedc1ee, 90 },
    { 0, 0 },
    { 0x5e8dc1f0, 61 },
    { 0x7a9dc1f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fdc1f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x463dc1f7, 95 },
    { 0, 0 },
    { 0x17fdc1f9, 98 },
    { 0x4addc1f9, 65 },
    { 0xe8edc1fa, 98 },
    { 0, 0 },
    { 0xda6dc1fd, 34 },
    { 0x49adc1fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230dc204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2dc207, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1dc20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1dc211, 61 },
    { 0x6d6dc211, 90 },
    { 0x818dc213, 52 },
    { 0x93bdc213, 61 },
    { 0x8a8dc214, 98 },
    { 0x187dc214, 91 },
    { 0x69ddc214, 98 },
    { 0x99bdc215, 76 },
    { 0x165dc218, 95 },
    { 0x7eadc21a, 43 },
    { 0x4f7dc215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fddc21f, 90 },
    { 0, 0 },
    { 0x359dc221, 33 },
    { 0x1a1dc222, 95 },
    { 0x9dbdc223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4dc226, 66 },
    { 0x1dddc227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffadc22a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03dc22e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cdc232, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4dc236, 46 },
    { 0x546dc237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2edc243, 98 },
    { 0, 0 },
    { 0xd5bdc245, 17 },
    { 0, 0 },
    { 0xf41dc247, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cdc254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4dc25b, 33 },
    { 0, 0 },
    { 0x63ddc25d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942dc26d, 75 },
    { 0x54adc26e, 23 },
    { 0, 0 },
    { 0x58fdc270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424dc277, 43 },
    { 0x40fdc278, 95 },
    { 0x170dc277, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecdc280, 95 },
    { 0x481dc281, 85 },
    { 0x2a6dc282, 95 },
    { 0x71cdc283, 76 },
    { 0xc41dc283, 98 },
    { 0x87fdc283, 61 },
    { 0xbfedc286, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x963dc289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8dc291, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdc294, 90 },
    { 0, 0 },
    { 0xaeedc296, 52 },
    { 0xf71dc296, 95 },
    { 0x255dc297, 61 },
    { 0xe6dc297, 33 },
    { 0, 0 },
    { 0x3d5dc29b, 90 },
    { 0xcf0dc29c, 66 },
    { 0x13cdc29d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedc2a0, 93 },
    { 0x573dc2a1, 90 },
    { 0x77fdc2a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130dc2a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616dc2ab, 90 },
    { 0xc12dc2ac, 76 },
    { 0x752dc2ad, 68 },
    { 0xb06dc2ac, 76 },
    { 0x4eadc2ae, 43 },
    { 0x9d0dc2ac, 93 },
    { 0x110dc2b0, 98 },
    { 0x232dc2b2, 46 },
    { 0x14cdc2b2, 91 },
    { 0x352dc2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948dc2b9, 84 },
    { 0, 0 },
    { 0x72bdc2bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dc2bf, 55 },
    { 0xb86dc2c0, 61 },
    { 0, 0 },
    { 0xf03dc2c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7dc2c9, 61 },
    { 0, 0 },
    { 0x3d5dc2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecdc2ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77edc2d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cdc2d9, 46 },
    { 0x6d6dc2da, 33 },
    { 0xd95dc2db, 34 },
    { 0x789dc2d9, 98 },
    { 0x891dc2dd, 61 },
    { 0x2e9dc2de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdc2e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x365dc2e4, 67 },
    { 0x219dc2e4, 93 },
    { 0x1bbdc2e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24dc2ed, 68 },
    { 0xf1dc2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929dc2f2, 61 },
    { 0, 0 },
    { 0x831dc2f4, 55 },
    { 0, 0 },
    { 0x804dc2f6, 95 },
    { 0xf82dc2f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913dc2ff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dc303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1dc312, 23 },
    { 0xf49dc313, 76 },
    { 0x8fddc312, 33 },
    { 0, 0 },
    { 0xf24dc316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bdc31a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9dc31d, 23 },
    { 0x7b4dc31e, 87 },
    { 0, 0 },
    { 0xf1fdc320, 23 },
    { 0, 0 },
    { 0x2f4dc322, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ddc329, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdedc32f, 65 },
    { 0xf34dc330, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badc333, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5dc336, 55 },
    { 0xe5bdc337, 23 },
    { 0xf09dc337, 77 },
    { 0x11dc339, 55 },
    { 0, 0 },
    { 0x5c1dc33b, 90 },
    { 0, 0 },
    { 0xda6dc33d, 33 },
    { 0x545dc33e, 46 },
    { 0, 0 },
    { 0x24adc340, 93 },
    { 0, 0 },
    { 0x4eedc342, 98 },
    { 0xf1ddc342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535dc347, 90 },
    { 0, 0 },
    { 0x344dc349, 55 },
    { 0xf18dc349, 98 },
    { 0x1fddc34b, 66 },
    { 0x207dc34b, 91 },
    { 0xb4adc34b, 23 },
    { 0x660dc34e, 98 },
    { 0x8b6dc34f, 61 },
    { 0x733dc34f, 46 },
    { 0x1fcdc34c, 12 },
    { 0xbd8dc352, 95 },
    { 0x5dfdc34b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29dc357, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70dc360, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4dc365, 81 },
    { 0x3dadc365, 47 },
    { 0xadbdc367, 33 },
    { 0x9bedc368, 95 },
    { 0x580dc369, 78 },
    { 0x11fdc369, 61 },
    { 0, 0 },
    { 0x222dc36c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93dc372, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120dc378, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79dc37e, 33 },
    { 0x553dc37f, 23 },
    { 0xcb8dc37f, 23 },
    { 0x638dc381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7dc387, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4dc38a, 67 },
    { 0x85edc38a, 61 },
    { 0x214dc38a, 55 },
    { 0x2cfdc38d, 61 },
    { 0, 0 },
    { 0xcb7dc38f, 23 },
    { 0x37fdc38f, 23 },
    { 0, 0 },
    { 0x398dc392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04dc395, 23 },
    { 0x686dc396, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7dc39c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466dc3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1dc3a8, 95 },
    { 0, 0 },
    { 0x8efdc3aa, 34 },
    { 0x426dc3aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cdc3ae, 61 },
    { 0x33ddc3ae, 98 },
    { 0xde4dc3af, 33 },
    { 0x6a2dc3b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cdc3b8, 33 },
    { 0x681dc3b9, 76 },
    { 0xea2dc3b8, 94 },
    { 0x685dc3bb, 52 },
    { 0x214dc3b8, 46 },
    { 0x62fdc3b8, 100 },
    { 0, 0 },
    { 0x1b9dc3bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ddc3cc, 69 },
    { 0xd24dc3cd, 36 },
    { 0x4aedc3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dc3d4, 93 },
    { 0, 0 },
    { 0x3e3dc3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbdc3de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc3e1, 98 },
    { 0x989dc3e2, 61 },
    { 0x496dc3e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cdc3e6, 76 },
    { 0xc99dc3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ddc3ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cdc3f0, 81 },
    { 0x5fddc3f0, 90 },
    { 0xfeadc3f2, 95 },
    { 0x567dc3f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470dc3f7, 66 },
    { 0xa88dc3f8, 76 },
    { 0x743dc3f9, 90 },
    { 0xa13dc3f9, 61 },
    { 0xdd0dc3f9, 33 },
    { 0, 0 },
    { 0x5e1dc3fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdc402, 61 },
    { 0x158dc403, 90 },
    { 0, 0 },
    { 0xb28dc405, 90 },
    { 0x432dc406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586dc40c, 23 },
    { 0x665dc40c, 43 },
    { 0, 0 },
    { 0x3d9dc40f, 55 },
    { 0, 0 },
    { 0x3a0dc411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbdc419, 56 },
    { 0x46bdc41a, 17 },
    { 0x67edc41b, 95 },
    { 0xac1dc41a, 90 },
    { 0xa83dc41d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3dc421, 81 },
    { 0, 0 },
    { 0xc42dc423, 33 },
    { 0, 0 },
    { 0xe86dc425, 23 },
    { 0x591dc426, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bdc42a, 76 },
    { 0x15dc42b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3dc42f, 88 },
    { 0x743dc430, 76 },
    { 0x7a4dc431, 90 },
    { 0, 0 },
    { 0x67adc433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbdc436, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdc43e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49edc441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6dc444, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488dc44e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199dc453, 90 },
    { 0, 0 },
    { 0x8b4dc455, 33 },
    { 0, 0 },
    { 0x42dc457, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611dc45b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1dc45e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25dc462, 98 },
    { 0, 0 },
    { 0x3bfdc464, 90 },
    { 0xbb9dc464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edc468, 90 },
    { 0xd76dc469, 95 },
    { 0x72dc46a, 76 },
    { 0xa8cdc46a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fdc46f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1dc472, 98 },
    { 0x6e7dc473, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cadc477, 98 },
    { 0xbc9dc478, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51dc482, 52 },
    { 0xfe6dc482, 87 },
    { 0, 0 },
    { 0xc40dc485, 42 },
    { 0x8eedc486, 98 },
    { 0xacfdc485, 99 },
    { 0xf60dc487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6dc48c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedc490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabedc494, 76 },
    { 0xb19dc495, 90 },
    { 0, 0 },
    { 0x58bdc497, 66 },
    { 0x726dc498, 23 },
    { 0xc6dc499, 55 },
    { 0, 0 },
    { 0x1eadc49b, 61 },
    { 0xb9cdc49b, 98 },
    { 0, 0 },
    { 0xa07dc49e, 23 },
    { 0x37edc49f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efdc4a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dc4a6, 88 },
    { 0xf17dc4a6, 77 },
    { 0x70fdc4a8, 95 },
    { 0x954dc4a9, 98 },
    { 0x3dadc4aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dc4ad, 76 },
    { 0x7d9dc4ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cadc4b2, 90 },
    { 0xafbdc4b3, 69 },
    { 0x72fdc4b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ddc4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233dc4c3, 61 },
    { 0xbb0dc4c3, 77 },
    { 0xd2dc4c5, 55 },
    { 0x5fbdc4c5, 98 },
    { 0xcf4dc4c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8dc4ca, 66 },
    { 0, 0 },
    { 0xfd7dc4cc, 40 },
    { 0x6efdc4cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bdc4d3, 90 },
    { 0xd9cdc4d4, 33 },
    { 0x9bbdc4d3, 90 },
    { 0xa5edc4d5, 78 },
    { 0x407dc4d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00dc4e3, 87 },
    { 0x4f2dc4e3, 46 },
    { 0, 0 },
    { 0xd53dc4e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addc4ec, 95 },
    { 0x92adc4ed, 23 },
    { 0x8b8dc4ee, 55 },
    { 0x1eadc4ed, 61 },
    { 0xa13dc4ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85dc4f3, 56 },
    { 0x8f9dc4f4, 43 },
    { 0x287dc4f3, 95 },
    { 0x28dc4f5, 55 },
    { 0x47dc4f7, 61 },
    { 0x8e5dc4f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddc4fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccdc4ff, 95 },
    { 0x49edc4ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809dc504, 85 },
    { 0x688dc504, 98 },
    { 0x883dc506, 90 },
    { 0x2c5dc507, 23 },
    { 0xe05dc507, 98 },
    { 0x430dc509, 17 },
    { 0xb6bdc506, 77 },
    { 0xaebdc50b, 90 },
    { 0x190dc50b, 98 },
    { 0x657dc50d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0dc511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98dc517, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216dc51b, 61 },
    { 0, 0 },
    { 0xe7fdc51d, 91 },
    { 0, 0 },
    { 0xaa8dc51f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fdc529, 63 },
    { 0x2fbdc52a, 23 },
    { 0, 0 },
    { 0x91fdc52c, 23 },
    { 0x93adc52d, 23 },
    { 0x371dc52c, 98 },
    { 0, 0 },
    { 0xd82dc530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcdc537, 61 },
    { 0, 0 },
    { 0xac6dc539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55dc53e, 100 },
    { 0, 0 },
    { 0x90adc540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdc543, 65 },
    { 0xd9dc544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acdc548, 90 },
    { 0x6fedc549, 66 },
    { 0xa80dc549, 23 },
    { 0x4dadc54b, 90 },
    { 0x40cdc548, 90 },
    { 0xc93dc549, 55 },
    { 0x591dc54e, 76 },
    { 0xc00dc54f, 90 },
    { 0xf93dc54f, 23 },
    { 0x86bdc54e, 90 },
    { 0x120dc54d, 75 },
    { 0x5dcdc553, 78 },
    { 0x738dc54d, 55 },
    { 0x532dc54c, 23 },
    { 0x280dc552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fdc55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dc569, 81 },
    { 0xb8bdc569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdc56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94dc571, 34 },
    { 0x96edc572, 55 },
    { 0xdf6dc573, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cdc576, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5dc57a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50dc57e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdc58a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8dc58d, 33 },
    { 0x46ddc58e, 74 },
    { 0x9d4dc58f, 65 },
    { 0xc38dc590, 100 },
    { 0xc6dc590, 85 },
    { 0xf40dc592, 52 },
    { 0x9dedc58d, 98 },
    { 0xfa6dc594, 90 },
    { 0x8f4dc594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25edc598, 61 },
    { 0, 0 },
    { 0x437dc59a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49dc59d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27dc5a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4dc5a3, 90 },
    { 0, 0 },
    { 0xc36dc5a5, 61 },
    { 0xfb3dc5a6, 90 },
    { 0xf60dc5a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9edc5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2dc5ae, 98 },
    { 0, 0 },
    { 0x575dc5b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29dc5b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9dc5b6, 90 },
    { 0xb5cdc5b6, 81 },
    { 0x344dc5b8, 61 },
    { 0xa0fdc5b8, 43 },
    { 0, 0 },
    { 0xff8dc5bb, 98 },
    { 0x6abdc5bc, 34 },
    { 0, 0 },
    { 0xa23dc5be, 23 },
    { 0xbe1dc5bf, 61 },
    { 0x2e2dc5be, 17 },
    { 0xfe7dc5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fedc5c4, 67 },
    { 0x106dc5c4, 34 },
    { 0, 0 },
    { 0xe3dc5c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bdc5cb, 100 },
    { 0x555dc5cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebdc5d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7dc5dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dddc5e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58dc5e3, 46 },
    { 0, 0 },
    { 0xc07dc5e5, 90 },
    { 0x7d8dc5e6, 43 },
    { 0x173dc5e6, 34 },
    { 0, 0 },
    { 0x10dc5e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1dc5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500dc5f2, 33 },
    { 0, 0 },
    { 0x562dc5f4, 34 },
    { 0xaf2dc5f5, 100 },
    { 0xe58dc5f6, 95 },
    { 0x55adc5f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4dc5fb, 33 },
    { 0, 0 },
    { 0x499dc5fd, 64 },
    { 0x273dc5fe, 23 },
    { 0x541dc5ff, 23 },
    { 0x132dc600, 67 },
    { 0x3dcdc601, 90 },
    { 0x9bddc5fd, 95 },
    { 0, 0 },
    { 0xc72dc604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdc60a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0edc612, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55dc617, 78 },
    { 0x25fdc618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc61d, 95 },
    { 0x139dc61e, 52 },
    { 0x181dc61f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8dc622, 34 },
    { 0xaccdc623, 90 },
    { 0x3eedc624, 95 },
    { 0x9b7dc625, 76 },
    { 0, 0 },
    { 0x2f3dc627, 61 },
    { 0xd5fdc628, 23 },
    { 0x6a8dc629, 98 },
    { 0x7b7dc62a, 87 },
    { 0x4dadc629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dc631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcdc635, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7dc638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37dc63b, 17 },
    { 0xc76dc63c, 90 },
    { 0xfbadc63c, 95 },
    { 0, 0 },
    { 0x776dc63f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7dc644, 43 },
    { 0x8fddc645, 43 },
    { 0x6d2dc644, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bedc64d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9dc657, 90 },
    { 0x9ddc658, 100 },
    { 0x68ddc659, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226dc65e, 95 },
    { 0x1eddc65f, 33 },
    { 0, 0 },
    { 0xdddc661, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdc66b, 95 },
    { 0, 0 },
    { 0x64bdc66d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467dc671, 33 },
    { 0x169dc672, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2dc678, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40dc67c, 98 },
    { 0, 0 },
    { 0x715dc67e, 98 },
    { 0x5aedc67f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdc684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668dc68b, 33 },
    { 0x76cdc68b, 95 },
    { 0xea4dc68d, 98 },
    { 0x232dc68e, 33 },
    { 0, 0 },
    { 0x885dc690, 33 },
    { 0x48cdc691, 66 },
    { 0xba3dc692, 91 },
    { 0x8f5dc692, 100 },
    { 0x27bdc693, 78 },
    { 0x526dc695, 90 },
    { 0xd36dc694, 90 },
    { 0x1cddc697, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfdc69a, 66 },
    { 0x51bdc69b, 90 },
    { 0, 0 },
    { 0x17bdc69d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x883dc6a0, 90 },
    { 0xce4dc6a1, 91 },
    { 0xf79dc6a1, 97 },
    { 0x68dc6a1, 87 },
    { 0xcb5dc6a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589dc6a8, 34 },
    { 0, 0 },
    { 0x32cdc6aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dc6ae, 34 },
    { 0x22fdc6af, 66 },
    { 0xc2adc6b0, 33 },
    { 0x68bdc6b1, 90 },
    { 0xca6dc6b2, 61 },
    { 0x883dc6b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc6b7, 98 },
    { 0xe94dc6b7, 33 },
    { 0, 0 },
    { 0x19adc6ba, 33 },
    { 0, 0 },
    { 0xb49dc6bc, 76 },
    { 0x353dc6bc, 78 },
    { 0x46dc6be, 61 },
    { 0x362dc6bd, 66 },
    { 0x115dc6bc, 43 },
    { 0, 0 },
    { 0xc97dc6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70dc6c7, 61 },
    { 0x2f4dc6c8, 61 },
    { 0, 0 },
    { 0xe8cdc6ca, 17 },
    { 0xa99dc6cb, 98 },
    { 0xbd3dc6cb, 81 },
    { 0, 0 },
    { 0xf72dc6ce, 33 },
    { 0x27ddc6cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1dc6d6, 46 },
    { 0xb1fdc6d7, 46 },
    { 0xa67dc6d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dc6dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dc6df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x805dc6e2, 77 },
    { 0xd3edc6e2, 66 },
    { 0x236dc6e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2dc6e7, 87 },
    { 0, 0 },
    { 0xa07dc6e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241dc6ed, 61 },
    { 0x1adc6ee, 33 },
    { 0x258dc6ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fdc6f2, 90 },
    { 0xd74dc6f3, 33 },
    { 0x32fdc6f4, 100 },
    { 0x90edc6f2, 23 },
    { 0x57edc6f6, 17 },
    { 0x406dc6f7, 61 },
    { 0x1e3dc6f8, 93 },
    { 0x339dc6f6, 90 },
    { 0, 0 },
    { 0xe4ddc6fb, 90 },
    { 0, 0 },
    { 0x6c0dc6fd, 78 },
    { 0x879dc6fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866dc70a, 95 },
    { 0xfbdc70b, 3 },
    { 0, 0 },
    { 0x184dc70d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115dc712, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x79edc715, 66 },
    { 0xc82dc716, 33 },
    { 0xc65dc717, 98 },
    { 0x111dc717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfdc71b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bdc71f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x945dc722, 61 },
    { 0xe09dc722, 91 },
    { 0x28dc724, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bdc728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827dc72f, 90 },
    { 0, 0 },
    { 0x4f2dc731, 34 },
    { 0x705dc732, 55 },
    { 0, 0 },
    { 0xadcdc734, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc737, 78 },
    { 0x847dc738, 43 },
    { 0x3c5dc739, 33 },
    { 0xa18dc73a, 61 },
    { 0xb6cdc73b, 90 },
    { 0x9fadc73c, 65 },
    { 0x367dc73c, 93 },
    { 0, 0 },
    { 0xe56dc73f, 88 },
    { 0x62edc73f, 61 },
    { 0x7f9dc740, 66 },
    { 0x6b0dc742, 34 },
    { 0x3dddc742, 88 },
    { 0, 0 },
    { 0x74dc745, 91 },
    { 0, 0 },
    { 0x619dc747, 33 },
    { 0xd99dc748, 61 },
    { 0x3a7dc749, 90 },
    { 0, 0 },
    { 0x5ccdc74b, 98 },
    { 0, 0 },
    { 0x87fdc74d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722dc751, 86 },
    { 0, 0 },
    { 0x5f3dc753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x629dc756, 34 },
    { 0, 0 },
    { 0x445dc758, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5dc75d, 88 },
    { 0, 0 },
    { 0x64adc75f, 66 },
    { 0xe1ddc75f, 95 },
    { 0x943dc761, 43 },
    { 0x160dc762, 33 },
    { 0x82cdc75f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdc766, 90 },
    { 0x5d7dc767, 55 },
    { 0x773dc768, 76 },
    { 0, 0 },
    { 0x68ddc76a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddc76d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81edc771, 33 },
    { 0x9c2dc772, 40 },
    { 0x88cdc771, 77 },
    { 0x800dc774, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc778, 34 },
    { 0, 0 },
    { 0xa33dc77a, 98 },
    { 0xd5ddc77a, 90 },
    { 0, 0 },
    { 0xa0adc77d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5dc783, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ddc78c, 90 },
    { 0xf2bdc78c, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedc791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25dc796, 90 },
    { 0, 0 },
    { 0x8fddc798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584dc79c, 96 },
    { 0, 0 },
    { 0x110dc79e, 96 },
    { 0xe58dc79f, 52 },
    { 0, 0 },
    { 0x1eedc7a1, 90 },
    { 0xa61dc7a2, 92 },
    { 0x682dc7a2, 76 },
    { 0x414dc7a4, 78 },
    { 0xe59dc7a5, 90 },
    { 0x51ddc7a5, 77 },
    { 0xf80dc7a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2dc7ac, 90 },
    { 0x40dc7ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bdc7b3, 33 },
    { 0, 0 },
    { 0x59edc7b5, 93 },
    { 0, 0 },
    { 0xf34dc7b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936dc7c1, 61 },
    { 0x924dc7c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90adc7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfdc7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bdc7d4, 87 },
    { 0xa97dc7d4, 95 },
    { 0xc13dc7d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x338dc7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x311dc7dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99dc7e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ddc7e5, 98 },
    { 0xb0cdc7e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdc7ea, 33 },
    { 0, 0 },
    { 0x2d2dc7ec, 34 },
    { 0x3c2dc7ed, 33 },
    { 0xb09dc7ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdc7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89adc7f5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607dc7fa, 33 },
    { 0x818dc7fb, 66 },
    { 0x27bdc7fc, 78 },
    { 0x1ddc7fd, 76 },
    { 0xf5bdc7fe, 81 },
    { 0x4b2dc7fe, 61 },
    { 0xa16dc7fb, 61 },
    { 0x6a7dc7ff, 80 },
    { 0xfb7dc802, 90 },
    { 0, 0 },
    { 0x858dc804, 75 },
    { 0xd5bdc804, 98 },
    { 0xc06dc805, 95 },
    { 0x7cbdc807, 61 },
    { 0x46ddc805, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x626dc80b, 43 },
    { 0xe6cdc80c, 85 },
    { 0xb97dc80d, 95 },
    { 0x881dc80e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2addc812, 61 },
    { 0, 0 },
    { 0xce8dc814, 58 },
    { 0xc99dc814, 90 },
    { 0xf91dc814, 95 },
    { 0, 0 },
    { 0x554dc818, 90 },
    { 0, 0 },
    { 0x9a6dc81a, 98 },
    { 0xfc7dc81a, 90 },
    { 0xe8fdc81b, 90 },
    { 0x258dc81d, 76 },
    { 0x622dc81e, 33 },
    { 0x703dc81a, 75 },
    { 0xd32dc81f, 65 },
    { 0, 0 },
    { 0x1a2dc822, 33 },
    { 0, 0 },
    { 0xc25dc824, 78 },
    { 0x722dc825, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfdc82d, 67 },
    { 0, 0 },
    { 0x4fdc82f, 55 },
    { 0x6dfdc830, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efdc833, 23 },
    { 0x9cfdc833, 23 },
    { 0x585dc835, 95 },
    { 0x135dc836, 46 },
    { 0xbcedc836, 55 },
    { 0x23edc835, 34 },
    { 0x849dc839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7dc83e, 55 },
    { 0x870dc83f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603dc843, 55 },
    { 0, 0 },
    { 0x87adc845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02dc84c, 87 },
    { 0x60edc84d, 23 },
    { 0x54ddc84e, 33 },
    { 0x573dc84d, 95 },
    { 0xbb5dc84e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679dc855, 61 },
    { 0x729dc856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696dc85a, 76 },
    { 0x8e5dc85a, 100 },
    { 0xdb4dc85c, 98 },
    { 0x922dc85d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x510dc860, 42 },
    { 0xf27dc861, 61 },
    { 0, 0 },
    { 0x73cdc863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fddc866, 76 },
    { 0x26edc867, 61 },
    { 0xbe2dc868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x554dc86b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8dc86f, 96 },
    { 0x994dc870, 99 },
    { 0, 0 },
    { 0x9dddc872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdedc877, 90 },
    { 0x7bedc878, 75 },
    { 0x26dc879, 69 },
    { 0xecdc879, 33 },
    { 0xe73dc87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x427dc87e, 78 },
    { 0xacddc87e, 93 },
    { 0x9e6dc87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aadc884, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaddc887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x425dc88a, 88 },
    { 0xdd9dc88a, 90 },
    { 0, 0 },
    { 0x6b4dc88d, 90 },
    { 0x6f9dc88e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766dc894, 76 },
    { 0x6f5dc895, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6adc898, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46dc89e, 93 },
    { 0xe16dc89f, 67 },
    { 0x967dc89f, 33 },
    { 0xf2bdc89e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140dc8a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691dc8ab, 90 },
    { 0x969dc8ab, 98 },
    { 0, 0 },
    { 0x9cadc8ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586dc8b3, 80 },
    { 0x75adc8b3, 99 },
    { 0, 0 },
    { 0x562dc8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3dc8b9, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dc8bc, 87 },
    { 0x8cbdc8bc, 23 },
    { 0x482dc8be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dc8c4, 95 },
    { 0, 0 },
    { 0xc02dc8c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4edc8d0, 95 },
    { 0, 0 },
    { 0x7d0dc8d2, 66 },
    { 0xad3dc8d3, 43 },
    { 0, 0 },
    { 0x1e9dc8d5, 23 },
    { 0x4f4dc8d5, 23 },
    { 0x5f8dc8d6, 33 },
    { 0x226dc8d8, 90 },
    { 0xccddc8d9, 90 },
    { 0xa4adc8d8, 66 },
    { 0x92edc8d7, 23 },
    { 0x7d4dc8d8, 36 },
    { 0x39fdc8da, 33 },
    { 0, 0 },
    { 0x290dc8df, 98 },
    { 0x4f1dc8df, 87 },
    { 0xe58dc8e1, 34 },
    { 0, 0 },
    { 0x69adc8e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaadc8ed, 78 },
    { 0xecdc8ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1addc8f1, 33 },
    { 0, 0 },
    { 0x342dc8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcadc902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649dc907, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfdc90b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x473dc90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc91a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140dc91e, 34 },
    { 0, 0 },
    { 0x277dc920, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718dc924, 76 },
    { 0, 0 },
    { 0x968dc926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0dc92a, 77 },
    { 0x94bdc92a, 98 },
    { 0xb22dc92c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17dc933, 33 },
    { 0xeeedc933, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ddc937, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9edc93a, 84 },
    { 0, 0 },
    { 0x687dc93c, 90 },
    { 0x4c1dc93d, 90 },
    { 0x8cbdc93e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abdc944, 100 },
    { 0x111dc945, 61 },
    { 0x195dc945, 67 },
    { 0x394dc944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaedc94b, 65 },
    { 0x4dadc94c, 34 },
    { 0x4a3dc94b, 98 },
    { 0xaf1dc94e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdc952, 61 },
    { 0xb8fdc953, 55 },
    { 0x676dc954, 91 },
    { 0xaaadc952, 90 },
    { 0x4a1dc956, 84 },
    { 0xfb4dc957, 55 },
    { 0xbeddc958, 100 },
    { 0xbc5dc957, 33 },
    { 0, 0 },
    { 0x167dc95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dddc962, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8adc968, 23 },
    { 0xd00dc968, 95 },
    { 0xe71dc96a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ddc96e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aedc974, 48 },
    { 0xf64dc975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fddc97b, 34 },
    { 0xcfbdc97b, 90 },
    { 0x127dc97d, 66 },
    { 0xbbedc97e, 61 },
    { 0x74ddc97f, 89 },
    { 0, 0 },
    { 0x63fdc981, 33 },
    { 0x7f5dc982, 90 },
    { 0xb9dc982, 55 },
    { 0x374dc982, 34 },
    { 0, 0 },
    { 0xda0dc986, 52 },
    { 0x9f8dc986, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaedc98c, 33 },
    { 0, 0 },
    { 0x96adc98e, 61 },
    { 0, 0 },
    { 0x9dc990, 43 },
    { 0xbd5dc991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaedc995, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66edc999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85dc99c, 76 },
    { 0x627dc99d, 87 },
    { 0xc03dc99e, 78 },
    { 0x8c3dc99e, 34 },
    { 0, 0 },
    { 0x2e0dc9a1, 52 },
    { 0x38bdc9a2, 90 },
    { 0x15adc9a2, 61 },
    { 0, 0 },
    { 0x378dc9a5, 60 },
    { 0, 0 },
    { 0x8e6dc9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x255dc9aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69edc9ad, 55 },
    { 0x2b1dc9ae, 61 },
    { 0x3a3dc9af, 90 },
    { 0, 0 },
    { 0xba7dc9b1, 33 },
    { 0xa27dc9b2, 93 },
    { 0xff0dc9b3, 67 },
    { 0x7dedc9b4, 23 },
    { 0x25bdc9b5, 95 },
    { 0xb28dc9b6, 46 },
    { 0xa88dc9b7, 33 },
    { 0xf28dc9b8, 90 },
    { 0, 0 },
    { 0x5b1dc9ba, 91 },
    { 0x8f9dc9bb, 95 },
    { 0x403dc9bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64dc9c2, 61 },
    { 0x424dc9c2, 61 },
    { 0xd7ddc9c3, 43 },
    { 0, 0 },
    { 0x35edc9c6, 98 },
    { 0xf95dc9c7, 23 },
    { 0xc18dc9c6, 90 },
    { 0, 0 },
    { 0xdf3dc9ca, 90 },
    { 0xb11dc9cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bdc9ce, 95 },
    { 0xfd0dc9ce, 95 },
    { 0xc3ddc9d0, 75 },
    { 0, 0 },
    { 0x84bdc9d2, 96 },
    { 0x7c6dc9d3, 55 },
    { 0x671dc9d4, 97 },
    { 0xbbddc9d5, 90 },
    { 0xd2dc9d5, 91 },
    { 0x29ddc9d7, 93 },
    { 0, 0 },
    { 0x8adc9d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6dc9de, 61 },
    { 0xc69dc9df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cdc9e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddc9ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2dc9f4, 51 },
    { 0x4b6dc9f5, 95 },
    { 0x3c0dc9f5, 69 },
    { 0x860dc9f7, 87 },
    { 0xa6cdc9f4, 61 },
    { 0xcfcdc9f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53dc9fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7dc9ff, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1dca03, 95 },
    { 0x55cdca04, 93 },
    { 0xeefdca05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dca13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25dca1c, 90 },
    { 0x200dca1c, 91 },
    { 0x8c2dca1c, 33 },
    { 0, 0 },
    { 0x8d5dca20, 33 },
    { 0, 0 },
    { 0x37ddca22, 92 },
    { 0xe98dca22, 88 },
    { 0, 0 },
    { 0x9e9dca25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0dca29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1dca2d, 90 },
    { 0xf33dca2e, 33 },
    { 0x9d3dca2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ddca36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdca3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7dca45, 82 },
    { 0, 0 },
    { 0x91edca47, 61 },
    { 0x6cadca48, 81 },
    { 0, 0 },
    { 0x298dca4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddca4d, 52 },
    { 0xb04dca4e, 61 },
    { 0, 0 },
    { 0x9b5dca50, 98 },
    { 0x942dca51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dca5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edca5f, 95 },
    { 0x94bdca60, 84 },
    { 0, 0 },
    { 0x1f7dca62, 43 },
    { 0xd15dca63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241dca68, 98 },
    { 0, 0 },
    { 0xa9edca6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0dca77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dca7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9dca7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffadca83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88dca87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fedca8d, 76 },
    { 0x701dca8d, 67 },
    { 0xfcfdca8d, 55 },
    { 0, 0 },
    { 0xe6adca91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dca94, 98 },
    { 0xabddca95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edca9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bdca9e, 84 },
    { 0x47bdca9f, 64 },
    { 0x49dca9f, 46 },
    { 0xc62dca9f, 95 },
    { 0, 0 },
    { 0x2e3dcaa3, 76 },
    { 0, 0 },
    { 0xac2dcaa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306dcaaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x599dcaad, 90 },
    { 0, 0 },
    { 0xf8bdcaaf, 33 },
    { 0, 0 },
    { 0x2a2dcab1, 34 },
    { 0x7b5dcab2, 90 },
    { 0x2fedcab3, 98 },
    { 0, 0 },
    { 0xc01dcab5, 34 },
    { 0x1bedcab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9dcab9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ddcabd, 84 },
    { 0x2b0dcabe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97dcac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bdcac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3dcac8, 33 },
    { 0xb4dcac9, 76 },
    { 0xa99dcaca, 41 },
    { 0xc90dcac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64dcad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fdcad9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bdcae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ddcae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4dcaf2, 43 },
    { 0x56dcaf3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cdcaf6, 78 },
    { 0x25adcaf7, 93 },
    { 0xa34dcaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ddcafc, 33 },
    { 0, 0 },
    { 0x203dcafe, 84 },
    { 0, 0 },
    { 0xab1dcb00, 34 },
    { 0x465dcb01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acdcb05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46edcb08, 63 },
    { 0x9dcdcb09, 72 },
    { 0, 0 },
    { 0x20bdcb0b, 76 },
    { 0x866dcb0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bddcb0f, 85 },
    { 0, 0 },
    { 0xc78dcb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343dcb1c, 55 },
    { 0xe23dcb1d, 46 },
    { 0, 0 },
    { 0xf93dcb1f, 78 },
    { 0x4b9dcb1f, 75 },
    { 0x893dcb21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892dcb25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdcb2a, 23 },
    { 0x437dcb2b, 61 },
    { 0x415dcb2c, 98 },
    { 0xbe6dcb2d, 84 },
    { 0x520dcb2b, 90 },
    { 0, 0 },
    { 0x67cdcb30, 89 },
    { 0x368dcb30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dcb39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ddcb40, 95 },
    { 0, 0 },
    { 0xce3dcb42, 95 },
    { 0x20bdcb43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fedcb48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbdcb4b, 34 },
    { 0xafcdcb4c, 93 },
    { 0xe0edcb4d, 76 },
    { 0, 0 },
    { 0x3c3dcb4f, 33 },
    { 0xe49dcb50, 33 },
    { 0x864dcb4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10dcb56, 52 },
    { 0x36dcb57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1dcb5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dcb5f, 33 },
    { 0x21edcb5f, 34 },
    { 0xfe7dcb61, 90 },
    { 0xa7dcb62, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddcb68, 33 },
    { 0x8c2dcb69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95dcb6f, 33 },
    { 0x867dcb70, 81 },
    { 0x644dcb70, 98 },
    { 0xb12dcb70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3dcb77, 33 },
    { 0x113dcb78, 90 },
    { 0x3edcb77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482dcb7d, 95 },
    { 0, 0 },
    { 0x344dcb7f, 33 },
    { 0xf74dcb80, 98 },
    { 0x8d5dcb81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8dcb84, 90 },
    { 0, 0 },
    { 0x297dcb86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9adcb89, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bdcb8e, 33 },
    { 0, 0 },
    { 0xa5dcb90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467dcb9a, 33 },
    { 0x931dcb9a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dcba3, 95 },
    { 0x244dcba3, 61 },
    { 0xb67dcba5, 93 },
    { 0x515dcba4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6dcbad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8adcbb0, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cdcbbc, 76 },
    { 0xb1bdcbbd, 33 },
    { 0x162dcbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969dcbc3, 90 },
    { 0xcb5dcbc3, 34 },
    { 0xa48dcbc5, 34 },
    { 0xe08dcbc5, 75 },
    { 0, 0 },
    { 0xdc8dcbc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x751dcbcb, 33 },
    { 0xb65dcbcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24edcbcf, 65 },
    { 0xca9dcbcf, 95 },
    { 0xea3dcbd1, 55 },
    { 0x1dbdcbd2, 23 },
    { 0x99edcbd1, 80 },
    { 0xc5fdcbd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119dcbdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7dcbe4, 98 },
    { 0xe9adcbe5, 55 },
    { 0xa1dcbe5, 61 },
    { 0x9bddcbe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dcbea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7dcbf4, 67 },
    { 0, 0 },
    { 0x841dcbf6, 75 },
    { 0xe5dcbf7, 76 },
    { 0x9fbdcbf7, 95 },
    { 0xe7bdcbf7, 98 },
    { 0xc40dcbfa, 100 },
    { 0xbc7dcbf6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ddcbfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4dcc04, 88 },
    { 0xd48dcc05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588dcc0c, 81 },
    { 0xb75dcc0c, 91 },
    { 0x5d4dcc0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x381dcc11, 98 },
    { 0, 0 },
    { 0x1a3dcc13, 43 },
    { 0x263dcc14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dcc18, 91 },
    { 0, 0 },
    { 0xf2ddcc1a, 90 },
    { 0xcd2dcc1b, 98 },
    { 0, 0 },
    { 0xd23dcc1d, 61 },
    { 0x814dcc1e, 80 },
    { 0x367dcc1e, 76 },
    { 0x5e8dcc1e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78edcc24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6dcc27, 46 },
    { 0xb8cdcc28, 65 },
    { 0x7cedcc29, 23 },
    { 0x9ccdcc2a, 90 },
    { 0xcfcdcc29, 17 },
    { 0, 0 },
    { 0xe40dcc2d, 52 },
    { 0xa5adcc2e, 91 },
    { 0, 0 },
    { 0x937dcc30, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984dcc3a, 61 },
    { 0x11ddcc3b, 33 },
    { 0x98edcc3b, 98 },
    { 0xf4bdcc3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0adcc49, 23 },
    { 0xae6dcc4a, 90 },
    { 0x8c6dcc4a, 100 },
    { 0, 0 },
    { 0x80cdcc4d, 61 },
    { 0, 0 },
    { 0xadfdcc4f, 64 },
    { 0xe24dcc4f, 65 },
    { 0xcaddcc51, 33 },
    { 0x152dcc51, 76 },
    { 0x32cdcc51, 100 },
    { 0x12dcc54, 95 },
    { 0xfdcdcc55, 55 },
    { 0x197dcc53, 66 },
    { 0x97fdcc57, 90 },
    { 0xeedcc55, 98 },
    { 0xab0dcc58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fdcc5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59dcc65, 43 },
    { 0x3f1dcc65, 75 },
    { 0xc6adcc65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521dcc6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438dcc73, 34 },
    { 0, 0 },
    { 0xa3edcc75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62dcc7f, 33 },
    { 0x109dcc80, 66 },
    { 0xe44dcc81, 43 },
    { 0x89adcc82, 95 },
    { 0x933dcc81, 55 },
    { 0x721dcc84, 61 },
    { 0x9e0dcc85, 95 },
    { 0x7b5dcc86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dcc8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49dcc8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534dcc93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6adcc99, 78 },
    { 0, 0 },
    { 0xf2fdcc9b, 33 },
    { 0x852dcc9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ddcca1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4dcca6, 52 },
    { 0, 0 },
    { 0x45cdcca8, 55 },
    { 0, 0 },
    { 0xa2bdccaa, 61 },
    { 0x376dccaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6dccb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x373dccba, 93 },
    { 0xc9fdccbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88dccbe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76edccc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64dccc5, 61 },
    { 0xcedccc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907dcccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5adccce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x264dccd1, 95 },
    { 0xbedccd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebdccda, 82 },
    { 0x11edccdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x996dccde, 52 },
    { 0xbf6dccdf, 23 },
    { 0x9b3dcce0, 43 },
    { 0x8f0dccdf, 23 },
    { 0xcdadcce2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83adcced, 61 },
    { 0, 0 },
    { 0x382dccef, 36 },
    { 0xd75dccf0, 33 },
    { 0, 0 },
    { 0x67edccf2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cedccf9, 67 },
    { 0xa6cdccfa, 93 },
    { 0xc4adccfa, 46 },
    { 0xd7bdccfc, 23 },
    { 0, 0 },
    { 0xc2ddccfe, 55 },
    { 0xbfedccfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdcd08, 98 },
    { 0x4e4dcd08, 95 },
    { 0x5e8dcd0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5dcd15, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dcd1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dcd1e, 97 },
    { 0, 0 },
    { 0x221dcd20, 52 },
    { 0xb5edcd21, 23 },
    { 0xa17dcd21, 90 },
    { 0x958dcd20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4dcd27, 76 },
    { 0, 0 },
    { 0xefedcd29, 34 },
    { 0x167dcd2a, 33 },
    { 0x459dcd2b, 100 },
    { 0, 0 },
    { 0xfc0dcd2d, 23 },
    { 0xe4bdcd2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dcd32, 23 },
    { 0, 0 },
    { 0x32ddcd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcdcd37, 66 },
    { 0x79fdcd37, 90 },
    { 0, 0 },
    { 0x93edcd3a, 88 },
    { 0x999dcd3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfdcd3f, 99 },
    { 0xe00dcd40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x966dcd43, 98 },
    { 0x792dcd44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47dcd4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402dcd50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddcd54, 23 },
    { 0xfb8dcd55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5edcd58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacdcd5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdcd62, 76 },
    { 0, 0 },
    { 0x583dcd64, 23 },
    { 0, 0 },
    { 0x97edcd66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9dcd69, 76 },
    { 0, 0 },
    { 0xbc2dcd6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57adcd70, 33 },
    { 0, 0 },
    { 0x5a9dcd72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dcd79, 46 },
    { 0xba1dcd7a, 90 },
    { 0x111dcd7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7dcd7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaedcd81, 67 },
    { 0, 0 },
    { 0xe7edcd83, 76 },
    { 0x706dcd84, 33 },
    { 0x4cadcd84, 89 },
    { 0xd06dcd86, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15edcd8e, 23 },
    { 0xf2fdcd8e, 33 },
    { 0xd73dcd8e, 98 },
    { 0xbd2dcd91, 46 },
    { 0x217dcd91, 90 },
    { 0x978dcd93, 33 },
    { 0x6fddcd93, 34 },
    { 0x140dcd95, 33 },
    { 0x26ddcd96, 23 },
    { 0x852dcd95, 33 },
    { 0x81fdcd92, 98 },
    { 0x8c0dcd99, 98 },
    { 0x4cdcd9a, 17 },
    { 0xaebdcd93, 55 },
    { 0, 0 },
    { 0x27fdcd9d, 85 },
    { 0, 0 },
    { 0x534dcd9f, 98 },
    { 0x669dcda0, 88 },
    { 0, 0 },
    { 0x21ddcda2, 100 },
    { 0x5a9dcda3, 55 },
    { 0x8ffdcda4, 90 },
    { 0x1bfdcda4, 90 },
    { 0x279dcda4, 90 },
    { 0xccfdcda7, 52 },
    { 0x28ddcda7, 90 },
    { 0xd76dcda6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536dcdad, 33 },
    { 0, 0 },
    { 0x1d8dcdaf, 90 },
    { 0x547dcdaf, 43 },
    { 0, 0 },
    { 0xea1dcdb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0dcdbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14dcdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ddcdc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3adcdcf, 33 },
    { 0, 0 },
    { 0x7fbdcdd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eedcdd6, 90 },
    { 0, 0 },
    { 0x960dcdd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23dcddc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1dcde1, 93 },
    { 0, 0 },
    { 0x962dcde3, 33 },
    { 0xa69dcde4, 61 },
    { 0x4b7dcde5, 90 },
    { 0xcafdcde6, 61 },
    { 0xdafdcde5, 98 },
    { 0xb6ddcde8, 100 },
    { 0, 0 },
    { 0x8a0dcdea, 90 },
    { 0, 0 },
    { 0x3dadcdec, 66 },
    { 0x57adcdec, 90 },
    { 0xd3edcdee, 95 },
    { 0x211dcded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fdcdf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ddcdf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3dcdfa, 23 },
    { 0x894dcdfa, 98 },
    { 0, 0 },
    { 0x661dcdfd, 98 },
    { 0xaccdcdfe, 100 },
    { 0x949dcdfe, 9 },
    { 0, 0 },
    { 0xe8bdce01, 90 },
    { 0x439dce02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fdce0a, 33 },
    { 0xd4edce0b, 46 },
    { 0x5e3dce0a, 90 },
    { 0x190dce0d, 95 },
    { 0x963dce0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342dce12, 61 },
    { 0x703dce13, 90 },
    { 0, 0 },
    { 0x3d6dce15, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9dce19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9dce1d, 46 },
    { 0x7cbdce1e, 63 },
    { 0, 0 },
    { 0xa43dce20, 34 },
    { 0x9a9dce20, 67 },
    { 0, 0 },
    { 0x63edce23, 61 },
    { 0x88fdce24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdce2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7dce36, 88 },
    { 0, 0 },
    { 0x489dce38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576dce42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806dce46, 55 },
    { 0x254dce47, 52 },
    { 0, 0 },
    { 0x7bddce49, 76 },
    { 0xe0fdce4a, 90 },
    { 0, 0 },
    { 0xacedce4c, 84 },
    { 0x5c8dce4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdce50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10dce53, 76 },
    { 0x19adce54, 23 },
    { 0x505dce53, 37 },
    { 0x81cdce55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45dce5a, 98 },
    { 0x569dce5b, 46 },
    { 0, 0 },
    { 0x30fdce5d, 34 },
    { 0, 0 },
    { 0x6fbdce5f, 33 },
    { 0x1e9dce5f, 33 },
    { 0x8b2dce5f, 84 },
    { 0x759dce62, 46 },
    { 0, 0 },
    { 0x8aadce64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93dce69, 34 },
    { 0xb66dce6a, 33 },
    { 0x2d5dce69, 76 },
    { 0, 0 },
    { 0x385dce6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105dce72, 90 },
    { 0x2dbdce73, 90 },
    { 0xaa9dce73, 34 },
    { 0xd1adce75, 65 },
    { 0, 0 },
    { 0x16fdce77, 61 },
    { 0x413dce78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dce7c, 85 },
    { 0xe22dce7d, 52 },
    { 0x9f0dce7e, 61 },
    { 0x8a5dce7f, 90 },
    { 0, 0 },
    { 0xd26dce81, 99 },
    { 0x605dce82, 61 },
    { 0x990dce83, 46 },
    { 0x943dce84, 33 },
    { 0xcaedce85, 98 },
    { 0x875dce84, 84 },
    { 0xdffdce84, 90 },
    { 0xf5fdce83, 99 },
    { 0x821dce87, 55 },
    { 0xa28dce8a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdce8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edce93, 90 },
    { 0xf06dce93, 33 },
    { 0x3dcdce95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dce99, 33 },
    { 0x849dce9a, 99 },
    { 0xda0dce9b, 17 },
    { 0x6fdce9b, 33 },
    { 0x513dce9a, 100 },
    { 0xe2ddce9b, 55 },
    { 0, 0 },
    { 0x82dcea0, 76 },
    { 0, 0 },
    { 0x458dcea2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6dcea8, 33 },
    { 0x33edcea8, 98 },
    { 0x1d6dcea9, 98 },
    { 0xe7edcea9, 34 },
    { 0x3e2dcea8, 55 },
    { 0x29bdcea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415dceb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cdceb4, 61 },
    { 0, 0 },
    { 0x8d6dceb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1dcebc, 33 },
    { 0xbb6dcebc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0dcec1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bdcec7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecdcecc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1dced0, 98 },
    { 0xc25dced1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdced9, 98 },
    { 0x165dceda, 43 },
    { 0x652dcedb, 23 },
    { 0xb7bdcedb, 76 },
    { 0, 0 },
    { 0x1c7dcede, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522dcee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fdcee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1dceef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x213dcef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bdcef7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06dcefa, 43 },
    { 0, 0 },
    { 0x66adcefc, 77 },
    { 0, 0 },
    { 0x954dcefe, 90 },
    { 0x36cdceff, 34 },
    { 0xbacdcefe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dcf07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77dcf0c, 33 },
    { 0xf2edcf0d, 98 },
    { 0xfaddcf0e, 76 },
    { 0xa71dcf0f, 46 },
    { 0xf45dcf10, 100 },
    { 0x89cdcf0c, 34 },
    { 0xed5dcf12, 98 },
    { 0xcf2dcf13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadcf1f, 75 },
    { 0xa33dcf20, 52 },
    { 0x62cdcf21, 23 },
    { 0, 0 },
    { 0x515dcf23, 67 },
    { 0xbe5dcf24, 99 },
    { 0xecedcf25, 90 },
    { 0x4f5dcf25, 43 },
    { 0x95adcf25, 95 },
    { 0, 0 },
    { 0x673dcf29, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1dcf2d, 76 },
    { 0, 0 },
    { 0x447dcf2f, 33 },
    { 0, 0 },
    { 0x494dcf31, 43 },
    { 0, 0 },
    { 0x16cdcf33, 76 },
    { 0xaacdcf34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dcf37, 76 },
    { 0, 0 },
    { 0xb66dcf39, 87 },
    { 0, 0 },
    { 0xa77dcf3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361dcf43, 55 },
    { 0x8b9dcf44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdadcf49, 34 },
    { 0x333dcf49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286dcf4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274dcf55, 52 },
    { 0, 0 },
    { 0x2c2dcf57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4dcf5a, 76 },
    { 0xc9bdcf5b, 43 },
    { 0xec9dcf5c, 98 },
    { 0, 0 },
    { 0x8dadcf5e, 99 },
    { 0x6fdcf5e, 84 },
    { 0x29cdcf60, 98 },
    { 0xb6adcf61, 98 },
    { 0x570dcf62, 33 },
    { 0x255dcf5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0dcf69, 61 },
    { 0xf9dcf69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x576dcf6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8dcf73, 84 },
    { 0x33ddcf74, 66 },
    { 0, 0 },
    { 0x233dcf76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6dcf79, 90 },
    { 0x265dcf7a, 23 },
    { 0x6b2dcf7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaedcf81, 52 },
    { 0x5a9dcf82, 87 },
    { 0xfd5dcf83, 61 },
    { 0x707dcf84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7dcf88, 23 },
    { 0, 0 },
    { 0x75cdcf8a, 76 },
    { 0, 0 },
    { 0x48cdcf8c, 98 },
    { 0xe29dcf8c, 90 },
    { 0, 0 },
    { 0x8ccdcf8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dcf92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfdcf95, 98 },
    { 0x15ddcf96, 87 },
    { 0, 0 },
    { 0xb3ddcf98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdadcf9e, 55 },
    { 0x79ddcf9f, 23 },
    { 0, 0 },
    { 0x376dcfa1, 43 },
    { 0x9f3dcfa2, 93 },
    { 0, 0 },
    { 0xefedcfa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcdcfa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedcfad, 34 },
    { 0x29fdcfae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7dcfb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193dcfba, 23 },
    { 0x4bddcfbb, 90 },
    { 0xe2edcfbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98adcfc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9dcfd0, 52 },
    { 0xc7edcfd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9dcfd5, 61 },
    { 0xacddcfd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecdcfdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553dcfdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cadcfe2, 34 },
    { 0x408dcfe2, 33 },
    { 0, 0 },
    { 0x959dcfe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7dcfe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1edcfee, 34 },
    { 0x477dcfef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcddcff4, 98 },
    { 0, 0 },
    { 0x6c5dcff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3dcffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdd004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdd00a, 91 },
    { 0x426dd00a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434dd013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dd017, 33 },
    { 0x419dd018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ddd020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fdd025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18dd02c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541dd031, 33 },
    { 0xf8dd032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x169dd035, 34 },
    { 0xc20dd035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5dd03a, 90 },
    { 0xa20dd03b, 55 },
    { 0, 0 },
    { 0x4fdd03d, 95 },
    { 0x65ddd03d, 76 },
    { 0xdbadd03f, 97 },
    { 0x1d2dd03f, 84 },
    { 0xf2dd041, 43 },
    { 0xcc4dd03e, 95 },
    { 0, 0 },
    { 0xb44dd044, 23 },
    { 0x586dd045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6dd048, 98 },
    { 0, 0 },
    { 0x3d9dd04a, 98 },
    { 0x45bdd04b, 33 },
    { 0x3bbdd04b, 95 },
    { 0x1dd04d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fedd057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x679dd05a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dd062, 95 },
    { 0x209dd062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734dd068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd06c, 66 },
    { 0, 0 },
    { 0xc2edd06e, 23 },
    { 0x636dd06f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd072, 61 },
    { 0, 0 },
    { 0x7f1dd074, 33 },
    { 0x50ddd074, 61 },
    { 0x7e4dd075, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edd080, 33 },
    { 0, 0 },
    { 0x86dd082, 76 },
    { 0x150dd083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317dd089, 55 },
    { 0x5dbdd089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fdd08e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd095, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8dd099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cdd09f, 55 },
    { 0x4cddd0a0, 55 },
    { 0x6a5dd0a1, 76 },
    { 0, 0 },
    { 0xb05dd0a3, 90 },
    { 0xb23dd0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x538dd0a7, 77 },
    { 0x9b8dd0a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x735dd0ab, 52 },
    { 0xa92dd0ab, 61 },
    { 0xfa5dd0ab, 76 },
    { 0x5e5dd0ae, 33 },
    { 0x7e1dd0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cdd0b6, 61 },
    { 0x569dd0b6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65dd0bd, 23 },
    { 0x15add0bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722dd0c4, 23 },
    { 0x1dfdd0c5, 76 },
    { 0xb25dd0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271dd0ca, 61 },
    { 0xeb1dd0ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52dd0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51dd0d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1dd0d9, 43 },
    { 0, 0 },
    { 0xee2dd0db, 92 },
    { 0x256dd0db, 23 },
    { 0xf39dd0dd, 76 },
    { 0, 0 },
    { 0xb7add0df, 98 },
    { 0x200dd0e0, 100 },
    { 0x1afdd0e1, 98 },
    { 0xbd2dd0e1, 33 },
    { 0x12fdd0e3, 61 },
    { 0x698dd0e1, 90 },
    { 0x24ddd0e1, 43 },
    { 0, 0 },
    { 0xaf5dd0e7, 34 },
    { 0, 0 },
    { 0x846dd0e9, 87 },
    { 0xd84dd0ea, 46 },
    { 0x35dd0e9, 93 },
    { 0xa43dd0eb, 90 },
    { 0xf25dd0ed, 33 },
    { 0x24bdd0ea, 99 },
    { 0, 0 },
    { 0x610dd0f0, 9 },
    { 0xcc0dd0f1, 95 },
    { 0xe01dd0f1, 98 },
    { 0, 0 },
    { 0x1fcdd0f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51dd0f8, 72 },
    { 0x698dd0f9, 75 },
    { 0xa24dd0fa, 95 },
    { 0x1c6dd0fb, 23 },
    { 0x50cdd0fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dd102, 33 },
    { 0, 0 },
    { 0xa74dd104, 95 },
    { 0x6dadd104, 98 },
    { 0x5d5dd105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67dd10a, 99 },
    { 0x2e5dd10b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fdd119, 91 },
    { 0xddfdd11a, 81 },
    { 0x3e3dd119, 34 },
    { 0x52edd11b, 67 },
    { 0, 0 },
    { 0x119dd11e, 100 },
    { 0, 0 },
    { 0x7ddd120, 100 },
    { 0x94fdd121, 55 },
    { 0x214dd120, 98 },
    { 0x89ddd123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fdd12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd130, 66 },
    { 0x53fdd131, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbdd138, 91 },
    { 0xd59dd138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd13c, 90 },
    { 0xb43dd13d, 90 },
    { 0xb93dd13e, 98 },
    { 0x2f1dd13f, 95 },
    { 0x12add13f, 61 },
    { 0xb4edd141, 23 },
    { 0x45fdd142, 40 },
    { 0x93add141, 23 },
    { 0xafadd142, 66 },
    { 0, 0 },
    { 0x799dd146, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9dd152, 23 },
    { 0x979dd153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x457dd156, 46 },
    { 0x2c4dd157, 100 },
    { 0, 0 },
    { 0xd68dd159, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91dd15c, 33 },
    { 0x963dd15d, 55 },
    { 0x47cdd15e, 55 },
    { 0xfabdd15f, 81 },
    { 0xe36dd15e, 34 },
    { 0x399dd160, 38 },
    { 0x136dd15d, 36 },
    { 0xc04dd163, 85 },
    { 0, 0 },
    { 0x1b4dd165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8dd168, 72 },
    { 0, 0 },
    { 0xc54dd16a, 34 },
    { 0x746dd16b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcdd170, 99 },
    { 0xf90dd170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ddd178, 81 },
    { 0x8d3dd178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecdd17d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6dd180, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18dd186, 33 },
    { 0, 0 },
    { 0x678dd188, 85 },
    { 0, 0 },
    { 0xa27dd18a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7dd18f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadd192, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7dd195, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dd198, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddd19f, 98 },
    { 0, 0 },
    { 0xb5cdd1a1, 33 },
    { 0xfd4dd1a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x640dd1a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880dd1ab, 100 },
    { 0, 0 },
    { 0x434dd1ad, 78 },
    { 0x75bdd1ae, 98 },
    { 0, 0 },
    { 0xeeddd1b0, 33 },
    { 0xf64dd1b1, 66 },
    { 0xa88dd1b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53dd1b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310dd1b9, 85 },
    { 0, 0 },
    { 0x31fdd1bb, 33 },
    { 0, 0 },
    { 0x7abdd1bd, 100 },
    { 0xcc6dd1bd, 90 },
    { 0xf10dd1bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331dd1c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x755dd1c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133dd1cd, 90 },
    { 0x79bdd1ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75edd1d1, 98 },
    { 0xea6dd1d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ddd1d5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ddd1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ddd1e4, 76 },
    { 0x242dd1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4dd1f0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efdd1f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfdd1fb, 67 },
    { 0, 0 },
    { 0x686dd1fd, 33 },
    { 0, 0 },
    { 0x75edd1ff, 78 },
    { 0x8dddd200, 61 },
    { 0xa3fdd200, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6dd20a, 84 },
    { 0xf46dd20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928dd211, 95 },
    { 0, 0 },
    { 0xe03dd213, 47 },
    { 0, 0 },
    { 0x7a0dd215, 23 },
    { 0x5c0dd215, 85 },
    { 0xcb3dd215, 90 },
    { 0x500dd218, 34 },
    { 0, 0 },
    { 0x3ddd21a, 34 },
    { 0x373dd21b, 91 },
    { 0xd1add21a, 93 },
    { 0x41edd21d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8addd220, 61 },
    { 0, 0 },
    { 0x5ddd222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606dd22a, 98 },
    { 0xffddd22a, 91 },
    { 0x21edd22c, 77 },
    { 0xadedd22c, 93 },
    { 0x292dd22a, 33 },
    { 0xf4fdd22f, 43 },
    { 0xc64dd230, 47 },
    { 0x9aadd231, 43 },
    { 0x8eedd232, 55 },
    { 0x7fcdd233, 23 },
    { 0xbb8dd234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3dd237, 23 },
    { 0xdf1dd238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7dd241, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edd244, 33 },
    { 0x3b5dd245, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92dd248, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15dd24c, 75 },
    { 0, 0 },
    { 0x7cbdd24e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cdd251, 61 },
    { 0xed4dd251, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08dd255, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857dd259, 90 },
    { 0x598dd25a, 90 },
    { 0xc8fdd25a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x882dd25e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8dd261, 23 },
    { 0x879dd261, 90 },
    { 0xdbcdd261, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ddd26c, 98 },
    { 0, 0 },
    { 0xf43dd26e, 61 },
    { 0x36cdd26f, 95 },
    { 0x2f2dd270, 97 },
    { 0xfa0dd270, 76 },
    { 0xa70dd271, 95 },
    { 0x8addd272, 98 },
    { 0xc40dd26e, 55 },
    { 0xb49dd275, 93 },
    { 0x26fdd270, 90 },
    { 0xf55dd274, 66 },
    { 0x8e8dd278, 61 },
    { 0x6b1dd278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x674dd27c, 90 },
    { 0xe6fdd27d, 90 },
    { 0xebfdd27e, 33 },
    { 0x4a2dd27f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2dd286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218dd28b, 55 },
    { 0, 0 },
    { 0x5dd28d, 52 },
    { 0xa78dd28e, 46 },
    { 0xd85dd28e, 76 },
    { 0x7a6dd28e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34dd294, 61 },
    { 0x380dd294, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ddd298, 90 },
    { 0, 0 },
    { 0x4aedd29a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15dd29d, 33 },
    { 0, 0 },
    { 0xc05dd29f, 33 },
    { 0x210dd29f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16dd2a3, 23 },
    { 0x653dd2a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719dd2ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1dd2b0, 43 },
    { 0x5dadd2b0, 33 },
    { 0, 0 },
    { 0x58bdd2b3, 98 },
    { 0x71ddd2b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85dd2b7, 33 },
    { 0xcf4dd2b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22dd2bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdd2c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fadd2c8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74dd2d9, 61 },
    { 0x566dd2d9, 34 },
    { 0xdedd2db, 64 },
    { 0xb4cdd2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1dd2e1, 23 },
    { 0xa3edd2e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x103dd2e5, 66 },
    { 0x501dd2e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x240dd2e9, 90 },
    { 0xb33dd2ea, 98 },
    { 0x620dd2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21dd2ee, 33 },
    { 0xa27dd2ee, 90 },
    { 0, 0 },
    { 0x9f6dd2f1, 33 },
    { 0, 0 },
    { 0x9fcdd2f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x95add2f6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1dd2f9, 33 },
    { 0x83bdd2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19dd2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0dd301, 24 },
    { 0, 0 },
    { 0xe7bdd303, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fdd306, 33 },
    { 0, 0 },
    { 0x1c9dd308, 76 },
    { 0, 0 },
    { 0x2c7dd30a, 93 },
    { 0xb8add30b, 33 },
    { 0x506dd30c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedd312, 76 },
    { 0x82bdd313, 65 },
    { 0xe8edd314, 90 },
    { 0x4c0dd315, 61 },
    { 0x2dadd315, 93 },
    { 0x2cedd314, 78 },
    { 0xd14dd318, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dd31f, 38 },
    { 0x2bfdd320, 99 },
    { 0, 0 },
    { 0x174dd322, 88 },
    { 0x426dd323, 87 },
    { 0, 0 },
    { 0x6dadd325, 34 },
    { 0, 0 },
    { 0xcdadd327, 33 },
    { 0x314dd327, 65 },
    { 0x95fdd329, 38 },
    { 0x22add32a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0dd330, 85 },
    { 0x580dd331, 23 },
    { 0, 0 },
    { 0xbc0dd333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ddd33d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5dd344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ddd349, 90 },
    { 0x2cbdd349, 23 },
    { 0x151dd34a, 33 },
    { 0x8cdd34c, 90 },
    { 0x882dd34c, 34 },
    { 0xc7add34e, 33 },
    { 0xbf4dd34f, 59 },
    { 0xe48dd34e, 90 },
    { 0, 0 },
    { 0xe79dd352, 99 },
    { 0, 0 },
    { 0xbb3dd354, 98 },
    { 0x185dd355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dd35a, 90 },
    { 0xebfdd35b, 33 },
    { 0x8a8dd35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x905dd35f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237dd363, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619dd368, 90 },
    { 0x41fdd369, 95 },
    { 0xd44dd36a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dd36f, 76 },
    { 0xe6dd370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7dd373, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cdd377, 61 },
    { 0, 0 },
    { 0x2a8dd379, 23 },
    { 0x964dd37a, 61 },
    { 0x942dd37b, 78 },
    { 0x49edd379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddd37f, 90 },
    { 0x178dd37f, 91 },
    { 0x9d0dd381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667dd38b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ddd391, 78 },
    { 0, 0 },
    { 0xa47dd393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10dd396, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd39b, 34 },
    { 0xad1dd39c, 93 },
    { 0, 0 },
    { 0x27edd39e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdd3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557dd3b2, 23 },
    { 0, 0 },
    { 0xa0fdd3b4, 61 },
    { 0xad2dd3b5, 23 },
    { 0, 0 },
    { 0x9cedd3b7, 90 },
    { 0x8dbdd3b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ddd3bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24dd3c0, 61 },
    { 0x85ddd3c0, 55 },
    { 0x22bdd3c0, 61 },
    { 0, 0 },
    { 0x8aedd3c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x696dd3c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1dd3cc, 90 },
    { 0xe4edd3cc, 33 },
    { 0, 0 },
    { 0xcdadd3cf, 88 },
    { 0x814dd3cf, 100 },
    { 0x7b3dd3d0, 24 },
    { 0xacdd3d2, 55 },
    { 0, 0 },
    { 0x127dd3d4, 76 },
    { 0xfbddd3d5, 90 },
    { 0x85add3d6, 34 },
    { 0x6f9dd3d7, 88 },
    { 0x914dd3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0dd3db, 61 },
    { 0, 0 },
    { 0xedd3dd, 98 },
    { 0, 0 },
    { 0x152dd3df, 68 },
    { 0xe4fdd3df, 33 },
    { 0, 0 },
    { 0xf2ddd3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dd3e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7dd3e8, 40 },
    { 0x83ddd3e8, 23 },
    { 0, 0 },
    { 0x98ddd3eb, 99 },
    { 0x354dd3ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddd3f2, 95 },
    { 0x521dd3f3, 90 },
    { 0xe70dd3f4, 87 },
    { 0x5dddd3f3, 68 },
    { 0xf18dd3f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dd3fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fddd3fe, 46 },
    { 0xd2ddd3ff, 46 },
    { 0xca8dd3fe, 61 },
    { 0, 0 },
    { 0x27ddd402, 33 },
    { 0, 0 },
    { 0xad5dd404, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65dd407, 33 },
    { 0x743dd408, 91 },
    { 0xbebdd409, 61 },
    { 0x8aedd409, 90 },
    { 0x633dd40b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ddd40e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131dd415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86dd418, 76 },
    { 0xf18dd419, 61 },
    { 0xb0dd41a, 33 },
    { 0x926dd418, 61 },
    { 0x34ddd418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x778dd41f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63dd425, 100 },
    { 0x447dd426, 90 },
    { 0x492dd427, 55 },
    { 0xb48dd428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332dd42c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8dd432, 84 },
    { 0x3e1dd432, 68 },
    { 0x646dd432, 33 },
    { 0x861dd435, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ddd438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202dd43c, 43 },
    { 0x434dd43c, 90 },
    { 0x8b0dd43e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd443, 43 },
    { 0, 0 },
    { 0xdd445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63dd44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325dd452, 98 },
    { 0xee6dd452, 95 },
    { 0x388dd454, 55 },
    { 0x366dd455, 61 },
    { 0, 0 },
    { 0x3a8dd457, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ddd45e, 23 },
    { 0, 0 },
    { 0x1dfdd460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6dd463, 85 },
    { 0xcf1dd463, 95 },
    { 0xabbdd465, 98 },
    { 0, 0 },
    { 0x45bdd467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cddd46a, 77 },
    { 0, 0 },
    { 0x73bdd46c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabdd471, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7dd474, 90 },
    { 0x5dcdd475, 90 },
    { 0, 0 },
    { 0x2e2dd477, 46 },
    { 0x80fdd478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dd482, 99 },
    { 0xd2add483, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287dd488, 34 },
    { 0, 0 },
    { 0x676dd48a, 55 },
    { 0x156dd48b, 78 },
    { 0x28cdd48c, 66 },
    { 0, 0 },
    { 0x52add48e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x840dd491, 98 },
    { 0, 0 },
    { 0xddcdd493, 90 },
    { 0, 0 },
    { 0xd01dd495, 4 },
    { 0x5d6dd496, 91 },
    { 0x6b1dd497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0dd49e, 34 },
    { 0x88cdd49f, 90 },
    { 0xe77dd4a0, 98 },
    { 0x49bdd4a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cdd4a7, 43 },
    { 0, 0 },
    { 0x9d4dd4a9, 90 },
    { 0xdc9dd4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440dd4b0, 52 },
    { 0x902dd4b1, 95 },
    { 0xf69dd4b2, 90 },
    { 0xaeedd4b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cedd4b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09dd4bc, 55 },
    { 0, 0 },
    { 0x1cadd4be, 78 },
    { 0, 0 },
    { 0x8ebdd4c0, 46 },
    { 0xf2cdd4c0, 93 },
    { 0xbefdd4c2, 87 },
    { 0x386dd4c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdd4cb, 90 },
    { 0xb0edd4cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbdd4d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aedd4d7, 78 },
    { 0xc64dd4d8, 76 },
    { 0xbadd4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162dd4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50add4e4, 81 },
    { 0x2bbdd4e5, 61 },
    { 0xb1fdd4e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5edd4e9, 61 },
    { 0, 0 },
    { 0x3e2dd4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebdd4ee, 23 },
    { 0x271dd4ef, 76 },
    { 0x71dd4f0, 98 },
    { 0x228dd4f0, 33 },
    { 0, 0 },
    { 0xe96dd4f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98dd4f6, 95 },
    { 0x2b5dd4f7, 23 },
    { 0x4a2dd4f8, 93 },
    { 0, 0 },
    { 0x2dd4fa, 95 },
    { 0x837dd4fb, 52 },
    { 0x1acdd4fa, 23 },
    { 0xb2dd4fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20dd506, 46 },
    { 0x4f7dd507, 9 },
    { 0, 0 },
    { 0xcf3dd509, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31dd50e, 90 },
    { 0x94add50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69dd516, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174dd51b, 61 },
    { 0x89edd51b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16edd525, 95 },
    { 0xdaedd526, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edd530, 84 },
    { 0xb9fdd530, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cdd534, 90 },
    { 0, 0 },
    { 0xc3bdd536, 95 },
    { 0, 0 },
    { 0x786dd538, 33 },
    { 0x6fddd538, 33 },
    { 0x55cdd538, 34 },
    { 0x2a4dd53b, 98 },
    { 0x2d7dd53b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4dd540, 43 },
    { 0x552dd541, 95 },
    { 0, 0 },
    { 0x811dd543, 95 },
    { 0x18ddd543, 34 },
    { 0xd41dd543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4dd548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63dd54e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0dd551, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51dd556, 55 },
    { 0x6dfdd557, 98 },
    { 0xf89dd557, 90 },
    { 0x61dd556, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68dd55f, 95 },
    { 0, 0 },
    { 0xb6add561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ddd566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afdd56c, 95 },
    { 0x615dd56d, 67 },
    { 0x351dd56e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0dd575, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926dd580, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fdd585, 33 },
    { 0, 0 },
    { 0x3d1dd587, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30edd58c, 61 },
    { 0, 0 },
    { 0xd5fdd58e, 47 },
    { 0x538dd58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadddd594, 76 },
    { 0x803dd595, 58 },
    { 0x901dd596, 61 },
    { 0x421dd597, 95 },
    { 0xa84dd595, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12dd59c, 61 },
    { 0xa44dd59d, 43 },
    { 0x444dd59d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fdd5a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edd5a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x253dd5a7, 90 },
    { 0x833dd5a7, 23 },
    { 0, 0 },
    { 0xac1dd5aa, 85 },
    { 0xadddd5aa, 87 },
    { 0x9cedd5ac, 76 },
    { 0x288dd5ab, 91 },
    { 0xcd5dd5ae, 76 },
    { 0, 0 },
    { 0x19cdd5b0, 61 },
    { 0, 0 },
    { 0xf9add5b2, 75 },
    { 0xd51dd5b2, 98 },
    { 0x49dd5b4, 33 },
    { 0x2c1dd5b4, 61 },
    { 0xf4bdd5b6, 90 },
    { 0x66ddd5b5, 43 },
    { 0x40cdd5b5, 76 },
    { 0x670dd5b9, 33 },
    { 0x68cdd5b7, 90 },
    { 0x3e1dd5bb, 23 },
    { 0x4d7dd5bc, 88 },
    { 0xf34dd5b9, 84 },
    { 0x6f3dd5b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08dd5c3, 66 },
    { 0xfe8dd5c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336dd5cc, 33 },
    { 0x6c1dd5cd, 90 },
    { 0, 0 },
    { 0x7b5dd5cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cddd5d4, 90 },
    { 0x9f3dd5d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x540dd5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bdd5df, 98 },
    { 0x748dd5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x164dd5e3, 76 },
    { 0x299dd5e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x907dd5e7, 52 },
    { 0x28fdd5e8, 43 },
    { 0x4eedd5e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ddd5ed, 76 },
    { 0, 0 },
    { 0x2d4dd5ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2dd5f2, 90 },
    { 0xd01dd5f2, 96 },
    { 0xea2dd5f2, 78 },
    { 0x9c6dd5f5, 76 },
    { 0, 0 },
    { 0x357dd5f7, 90 },
    { 0x927dd5f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133dd5fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9dd605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44add608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6dd60b, 34 },
    { 0x93edd60c, 55 },
    { 0x7e6dd60b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dd610, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1dd617, 84 },
    { 0x48add618, 98 },
    { 0x67fdd618, 33 },
    { 0x76add61a, 55 },
    { 0x8a2dd61b, 90 },
    { 0, 0 },
    { 0xe38dd61d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dd621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dd624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4dd62c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fdd62f, 33 },
    { 0, 0 },
    { 0x5cadd631, 46 },
    { 0xd73dd631, 50 },
    { 0xf57dd631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54dd638, 34 },
    { 0x70fdd639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593dd63d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdd646, 33 },
    { 0x57ddd647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0add64a, 95 },
    { 0, 0 },
    { 0x8fadd64c, 90 },
    { 0, 0 },
    { 0xcb5dd64e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3dd653, 33 },
    { 0x8d9dd654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadddd659, 69 },
    { 0x62cdd65a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7dd660, 8 },
    { 0, 0 },
    { 0x714dd662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92dd669, 66 },
    { 0xa7cdd669, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd66d, 81 },
    { 0x742dd66d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84dd671, 90 },
    { 0x1dadd672, 61 },
    { 0x5cedd671, 34 },
    { 0xeeedd673, 76 },
    { 0xe93dd675, 53 },
    { 0xc2bdd676, 43 },
    { 0xb2bdd676, 61 },
    { 0x628dd675, 90 },
    { 0, 0 },
    { 0x6d5dd67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282dd67d, 75 },
    { 0x1c1dd67d, 84 },
    { 0, 0 },
    { 0x32bdd680, 95 },
    { 0, 0 },
    { 0x4a1dd682, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fdd686, 61 },
    { 0, 0 },
    { 0xfc0dd688, 76 },
    { 0, 0 },
    { 0xb12dd68a, 78 },
    { 0x33cdd68a, 78 },
    { 0x3acdd68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190dd690, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04dd693, 52 },
    { 0x233dd693, 33 },
    { 0, 0 },
    { 0xf6bdd696, 90 },
    { 0xb8edd697, 98 },
    { 0xfdcdd697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cdd69b, 90 },
    { 0x73edd69b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x785dd69f, 55 },
    { 0, 0 },
    { 0x498dd6a1, 61 },
    { 0x2e6dd6a2, 52 },
    { 0xf94dd6a1, 98 },
    { 0, 0 },
    { 0xa14dd6a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dedd6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24dd6ad, 33 },
    { 0xbe1dd6ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159dd6b4, 83 },
    { 0xd94dd6b4, 67 },
    { 0x773dd6b6, 33 },
    { 0x9f7dd6b6, 23 },
    { 0x9fadd6b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549dd6bd, 92 },
    { 0x16dd6be, 33 },
    { 0x34ddd6bf, 43 },
    { 0, 0 },
    { 0x58ddd6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfdd6c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9dd6cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83dd6d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ddd6da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdd6dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2dd6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310dd6ee, 90 },
    { 0, 0 },
    { 0x1b1dd6f0, 84 },
    { 0x2efdd6f1, 46 },
    { 0x998dd6f1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916dd6f8, 43 },
    { 0x1badd6f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fdd6fc, 98 },
    { 0xf37dd6fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4dd701, 76 },
    { 0xedbdd702, 17 },
    { 0xaf6dd703, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdd709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcdd70c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dd711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edd715, 99 },
    { 0, 0 },
    { 0xd05dd717, 46 },
    { 0x82ddd717, 98 },
    { 0, 0 },
    { 0x23bdd71a, 90 },
    { 0, 0 },
    { 0x61dd71c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fdd723, 87 },
    { 0x607dd724, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524dd72c, 95 },
    { 0x66add72d, 33 },
    { 0x672dd72d, 98 },
    { 0xc71dd72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cdd732, 33 },
    { 0, 0 },
    { 0x8dcdd734, 43 },
    { 0x2b1dd734, 66 },
    { 0x8fcdd736, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8dd73d, 98 },
    { 0, 0 },
    { 0xfaadd73f, 23 },
    { 0x4d6dd740, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208dd749, 77 },
    { 0, 0 },
    { 0x8b9dd74b, 90 },
    { 0, 0 },
    { 0x636dd74d, 90 },
    { 0xb8edd74e, 43 },
    { 0x479dd74e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59add754, 85 },
    { 0, 0 },
    { 0xab1dd756, 61 },
    { 0, 0 },
    { 0x5f0dd758, 61 },
    { 0xd8edd758, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5dd763, 90 },
    { 0x67cdd763, 95 },
    { 0x310dd765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd769, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x352dd76c, 61 },
    { 0, 0 },
    { 0x243dd76e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fdd771, 23 },
    { 0xc59dd771, 98 },
    { 0, 0 },
    { 0xd33dd774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbadd778, 68 },
    { 0x4ccdd779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fdd77c, 61 },
    { 0xd8ddd77d, 98 },
    { 0x61cdd77e, 33 },
    { 0, 0 },
    { 0x31fdd780, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fdd786, 23 },
    { 0x241dd787, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9dd78a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bdd78d, 76 },
    { 0xe79dd78d, 33 },
    { 0x815dd78f, 61 },
    { 0xfa1dd790, 91 },
    { 0xddfdd790, 61 },
    { 0xecdd792, 87 },
    { 0x67dd792, 93 },
    { 0x798dd790, 90 },
    { 0x2f4dd793, 23 },
    { 0x76edd790, 46 },
    { 0xe0dd793, 55 },
    { 0x19ddd798, 93 },
    { 0xdacdd792, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cdd79d, 90 },
    { 0, 0 },
    { 0x3ecdd79f, 98 },
    { 0xfc8dd7a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dd7a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bdd7a7, 90 },
    { 0, 0 },
    { 0xc39dd7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddd7ae, 36 },
    { 0xa58dd7af, 90 },
    { 0xde2dd7af, 55 },
    { 0x69dd7b1, 98 },
    { 0xc3cdd7b2, 61 },
    { 0x4d0dd7b2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10add7b8, 52 },
    { 0xdc9dd7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44dd7bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabdd7c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9dd7c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185dd7ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ddd7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60dd7d0, 78 },
    { 0x169dd7d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadd7d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x637dd7d9, 17 },
    { 0, 0 },
    { 0x4a4dd7db, 33 },
    { 0xecfdd7db, 91 },
    { 0xa9add7dd, 97 },
    { 0x39add7dd, 33 },
    { 0x8dddd7df, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cdd7e5, 90 },
    { 0x17edd7e6, 43 },
    { 0xddbdd7e7, 23 },
    { 0xb87dd7e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469dd7ec, 61 },
    { 0xba2dd7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfddd7f6, 34 },
    { 0x241dd7f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0dd7fd, 78 },
    { 0xaf6dd7fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fdd806, 90 },
    { 0x26bdd806, 87 },
    { 0xdebdd806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdd80c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5dd813, 95 },
    { 0x211dd814, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd817, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2dd81e, 95 },
    { 0xfe2dd81e, 44 },
    { 0xfa6dd820, 55 },
    { 0x23fdd820, 95 },
    { 0x3ffdd822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409dd827, 33 },
    { 0, 0 },
    { 0xf1bdd829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x265dd82c, 94 },
    { 0x2cedd82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cddd830, 33 },
    { 0x4badd831, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6dd835, 46 },
    { 0xfb8dd836, 91 },
    { 0xa58dd835, 34 },
    { 0x3eadd835, 93 },
    { 0x417dd838, 90 },
    { 0x28dd83a, 66 },
    { 0x907dd83a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9dd843, 98 },
    { 0xfb1dd844, 98 },
    { 0x720dd844, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9dd848, 90 },
    { 0, 0 },
    { 0x492dd84a, 95 },
    { 0x141dd84a, 83 },
    { 0, 0 },
    { 0x509dd84d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45dd856, 46 },
    { 0, 0 },
    { 0x4afdd858, 90 },
    { 0xdabdd859, 95 },
    { 0xd4ddd858, 8 },
    { 0, 0 },
    { 0x615dd85c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bdd860, 90 },
    { 0, 0 },
    { 0x6a5dd862, 67 },
    { 0xfb3dd862, 98 },
    { 0x51fdd864, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06dd867, 33 },
    { 0xc3fdd868, 81 },
    { 0x5f6dd869, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfdd86e, 46 },
    { 0xa4ddd86f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781dd87a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3dd87e, 40 },
    { 0x43cdd87f, 87 },
    { 0x76dd880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bdd883, 33 },
    { 0xfdfdd884, 90 },
    { 0x374dd885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x863dd888, 95 },
    { 0x396dd889, 33 },
    { 0xb67dd88a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdd88e, 61 },
    { 0xfc2dd88e, 98 },
    { 0xc51dd890, 90 },
    { 0x98add890, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ddd895, 33 },
    { 0x3e8dd895, 33 },
    { 0, 0 },
    { 0xe83dd898, 90 },
    { 0x65add899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81dd89c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30dd8a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724dd8b0, 51 },
    { 0x395dd8b0, 33 },
    { 0xe1bdd8b1, 93 },
    { 0x313dd8b0, 98 },
    { 0xf4fdd8b4, 76 },
    { 0x51add8b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cadd8b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6dd8be, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9dd8c4, 90 },
    { 0, 0 },
    { 0x502dd8c6, 33 },
    { 0x5e1dd8c7, 53 },
    { 0x89cdd8c6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8dd8cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94dd8cf, 90 },
    { 0x689dd8d0, 95 },
    { 0xadd8d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39dd8d4, 23 },
    { 0, 0 },
    { 0xa30dd8d6, 95 },
    { 0x593dd8d7, 61 },
    { 0, 0 },
    { 0xc9add8d9, 95 },
    { 0x335dd8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x615dd8dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5dd8e5, 61 },
    { 0x2ccdd8e6, 95 },
    { 0xbaadd8e7, 34 },
    { 0xfeddd8e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157dd8ed, 46 },
    { 0xc0cdd8ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663dd8f8, 90 },
    { 0x46cdd8f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0dd8fd, 95 },
    { 0x7a7dd8fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4dd902, 61 },
    { 0x2b3dd903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bdd90f, 93 },
    { 0, 0 },
    { 0xcbcdd911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3dd915, 65 },
    { 0xb5add915, 40 },
    { 0x514dd915, 98 },
    { 0xfbddd917, 90 },
    { 0x9e9dd919, 61 },
    { 0x8c6dd919, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1dd921, 49 },
    { 0, 0 },
    { 0x214dd923, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8dd927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738dd92a, 33 },
    { 0x6f1dd92a, 33 },
    { 0x673dd92c, 33 },
    { 0x525dd92c, 90 },
    { 0x5eedd92c, 34 },
    { 0x3fbdd92c, 91 },
    { 0x21fdd930, 95 },
    { 0x40dd92f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81dd934, 90 },
    { 0xd27dd934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35dd939, 90 },
    { 0x64cdd93a, 99 },
    { 0x5e1dd93b, 98 },
    { 0x2f5dd93b, 90 },
    { 0x73ddd93d, 95 },
    { 0x6ccdd93d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1dd943, 90 },
    { 0x721dd944, 90 },
    { 0, 0 },
    { 0xf08dd946, 55 },
    { 0x296dd946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910dd955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fdd958, 98 },
    { 0xf99dd959, 93 },
    { 0, 0 },
    { 0xcf7dd95b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3dd95f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39dd962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57dd965, 34 },
    { 0, 0 },
    { 0x802dd967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x456dd96a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480dd973, 81 },
    { 0xd7ddd973, 23 },
    { 0xf08dd975, 85 },
    { 0xc7edd975, 90 },
    { 0x5dddd974, 81 },
    { 0x9eddd978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0edd97e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5dd98a, 23 },
    { 0xe0add98b, 95 },
    { 0x819dd98b, 95 },
    { 0xbd5dd98c, 95 },
    { 0x37fdd98e, 61 },
    { 0xfc3dd98f, 95 },
    { 0, 0 },
    { 0x5bcdd991, 93 },
    { 0x6fbdd992, 55 },
    { 0, 0 },
    { 0x8c5dd994, 99 },
    { 0x3ecdd995, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0dd999, 46 },
    { 0x9d4dd999, 34 },
    { 0, 0 },
    { 0xf6fdd99c, 33 },
    { 0x579dd99c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0dd9a7, 98 },
    { 0x347dd9a8, 23 },
    { 0xbd9dd9a9, 76 },
    { 0x8aadd9aa, 37 },
    { 0x808dd9a7, 98 },
    { 0, 0 },
    { 0x261dd9ad, 23 },
    { 0x506dd9ae, 61 },
    { 0, 0 },
    { 0xb67dd9b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5dd9b7, 52 },
    { 0x485dd9b8, 66 },
    { 0xffadd9b9, 66 },
    { 0x1f9dd9b9, 95 },
    { 0, 0 },
    { 0xccfdd9bc, 98 },
    { 0x144dd9bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dd9cc, 90 },
    { 0, 0 },
    { 0x24edd9ce, 98 },
    { 0x30cdd9ce, 34 },
    { 0, 0 },
    { 0x913dd9d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dd9d4, 90 },
    { 0, 0 },
    { 0xf29dd9d6, 90 },
    { 0xea3dd9d6, 68 },
    { 0xddedd9d8, 33 },
    { 0x3b4dd9d9, 33 },
    { 0x298dd9da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0edd9dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5dd9e0, 84 },
    { 0xc54dd9e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16dd9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9dd9ef, 33 },
    { 0x283dd9f0, 33 },
    { 0xf85dd9f1, 90 },
    { 0xbc7dd9f1, 90 },
    { 0xb28dd9f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdd9f6, 92 },
    { 0, 0 },
    { 0x5a0dd9f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2dd9fe, 33 },
    { 0x702dd9ff, 55 },
    { 0xfc3dd9ff, 95 },
    { 0xa99dd9ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1dda05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbdda09, 90 },
    { 0x904dda0a, 77 },
    { 0, 0 },
    { 0xa60dda0c, 78 },
    { 0x775dda0c, 55 },
    { 0xbe6dda0e, 95 },
    { 0x3f4dda0e, 55 },
    { 0xfabdda0d, 55 },
    { 0x14edda0d, 90 },
    { 0xe23dda0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdda19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bdda1c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535dda20, 46 },
    { 0, 0 },
    { 0x14adda22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7dda29, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8dda2f, 33 },
    { 0xbfcdda30, 67 },
    { 0xdc7dda30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cdda39, 33 },
    { 0x6b6dda3a, 78 },
    { 0xca0dda3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490dda44, 90 },
    { 0x857dda45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x643dda48, 99 },
    { 0, 0 },
    { 0xe5dda4a, 33 },
    { 0x49adda4a, 95 },
    { 0x2a8dda4c, 87 },
    { 0x4c4dda4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91dda52, 33 },
    { 0, 0 },
    { 0x363dda54, 87 },
    { 0, 0 },
    { 0x82bdda56, 99 },
    { 0xcd1dda57, 90 },
    { 0, 0 },
    { 0xcbddda59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331dda5f, 76 },
    { 0xa5dda60, 33 },
    { 0xb5edda61, 78 },
    { 0xb22dda60, 84 },
    { 0, 0 },
    { 0xd31dda64, 90 },
    { 0xd16dda64, 33 },
    { 0xf56dda65, 55 },
    { 0, 0 },
    { 0xe40dda68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5dda6d, 23 },
    { 0, 0 },
    { 0xeeddda6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35dda73, 61 },
    { 0, 0 },
    { 0x35edda75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54dda79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x826dda7c, 68 },
    { 0, 0 },
    { 0xcabdda7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933dda82, 23 },
    { 0xd9bdda82, 95 },
    { 0x895dda84, 93 },
    { 0xc08dda84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573dda97, 17 },
    { 0x674dda97, 95 },
    { 0xf91dda99, 55 },
    { 0x46edda98, 95 },
    { 0x9c0dda99, 46 },
    { 0x5e9dda9b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4ddaa1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48ddaa6, 61 },
    { 0xa47ddaa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57ddaab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8ddab2, 76 },
    { 0xa49ddab3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cddab6, 100 },
    { 0x7e0ddab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fddabb, 76 },
    { 0xd6cddabb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3ddac3, 61 },
    { 0, 0 },
    { 0xac1ddac5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaddac8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcddacf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1ddad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1eddadd, 9 },
    { 0x6c7ddadd, 23 },
    { 0, 0 },
    { 0x5b6ddae0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fddae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ddaec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x182ddaef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ddaf2, 98 },
    { 0xdf4ddaf3, 33 },
    { 0, 0 },
    { 0xa42ddaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9ddaf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ddb00, 52 },
    { 0xbdcddb01, 91 },
    { 0x478ddb02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ddb08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ddb0b, 90 },
    { 0x789ddb0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdddb11, 90 },
    { 0, 0 },
    { 0xd4fddb13, 78 },
    { 0, 0 },
    { 0xb54ddb15, 43 },
    { 0xbbddb16, 91 },
    { 0xc31ddb17, 90 },
    { 0, 0 },
    { 0x196ddb19, 33 },
    { 0x14addb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cddb1d, 66 },
    { 0x5fdddb1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dddb21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfddb24, 85 },
    { 0, 0 },
    { 0x2e0ddb26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442ddb2b, 61 },
    { 0x9ebddb2c, 98 },
    { 0xd57ddb2d, 87 },
    { 0, 0 },
    { 0x648ddb2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eddb33, 99 },
    { 0xb73ddb34, 33 },
    { 0, 0 },
    { 0x54bddb36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ddb3b, 76 },
    { 0x790ddb3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddb40, 76 },
    { 0x4aeddb40, 93 },
    { 0x395ddb42, 23 },
    { 0xbf7ddb42, 98 },
    { 0x363ddb44, 98 },
    { 0x913ddb44, 99 },
    { 0x4feddb44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6ddb51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0ddb56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ddb5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fddb5e, 23 },
    { 0, 0 },
    { 0xa6addb60, 61 },
    { 0, 0 },
    { 0x218ddb62, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08ddb68, 36 },
    { 0x160ddb69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbddb71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bddb75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fddb7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ddb86, 43 },
    { 0x616ddb87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbddb8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adddb8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fddb95, 95 },
    { 0x872ddb96, 33 },
    { 0xe79ddb97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bddb9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afddb9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dddba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ddba5, 90 },
    { 0, 0 },
    { 0xc89ddba7, 76 },
    { 0, 0 },
    { 0xba0ddba9, 76 },
    { 0x5f1ddba9, 98 },
    { 0xfdcddba9, 33 },
    { 0xe8fddbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cddbb2, 52 },
    { 0x38dddbb2, 43 },
    { 0x2a1ddbb4, 95 },
    { 0x499ddbb5, 33 },
    { 0xafddbb6, 33 },
    { 0xfc4ddbb7, 43 },
    { 0x843ddbb8, 43 },
    { 0, 0 },
    { 0xe08ddbba, 61 },
    { 0xe4dddbbb, 23 },
    { 0xfbaddbbc, 52 },
    { 0x95fddbbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ddbc0, 61 },
    { 0xf65ddbc0, 99 },
    { 0x36bddbc2, 95 },
    { 0xc24ddbc3, 88 },
    { 0x4d4ddbc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fddbc8, 87 },
    { 0xf4addbc9, 95 },
    { 0xc81ddbca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cddbcf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974ddbda, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ddbde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ddbe9, 61 },
    { 0xaafddbea, 90 },
    { 0, 0 },
    { 0xb54ddbec, 77 },
    { 0x361ddbed, 33 },
    { 0, 0 },
    { 0x8cdddbef, 61 },
    { 0, 0 },
    { 0xae7ddbf1, 55 },
    { 0xd2cddbf2, 81 },
    { 0xe2fddbf2, 91 },
    { 0xbd6ddbf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12ddbf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bddbfc, 66 },
    { 0, 0 },
    { 0xb2dddbfe, 99 },
    { 0x41cddbff, 76 },
    { 0x72addc00, 61 },
    { 0xcd4ddc01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ddc08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09ddc0b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4addc0f, 23 },
    { 0x1a2ddc10, 93 },
    { 0x888ddc10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840ddc18, 93 },
    { 0, 0 },
    { 0xaa7ddc1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ddc1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fddc22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20ddc2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3ddc2e, 90 },
    { 0xbc3ddc2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6ddc36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9addc39, 55 },
    { 0, 0 },
    { 0xbc8ddc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422ddc3f, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaddc44, 23 },
    { 0x1c5ddc45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ddc49, 23 },
    { 0x7a2ddc4a, 23 },
    { 0x9bfddc4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ddc4f, 81 },
    { 0x14ddc50, 76 },
    { 0x2fcddc50, 64 },
    { 0xf5bddc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1ddc55, 66 },
    { 0xdc3ddc56, 33 },
    { 0x956ddc57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abddc5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ddc5e, 95 },
    { 0, 0 },
    { 0x244ddc60, 90 },
    { 0xfdeddc61, 90 },
    { 0, 0 },
    { 0x56eddc63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ddc66, 90 },
    { 0, 0 },
    { 0x9d3ddc68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefddc6c, 95 },
    { 0xb6cddc6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5ddc70, 61 },
    { 0xdf0ddc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ddc75, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ddc79, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fddc7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799ddc87, 23 },
    { 0xf7ddc87, 34 },
    { 0x6afddc89, 33 },
    { 0x6aeddc8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cddc8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eddc91, 98 },
    { 0, 0 },
    { 0x56dddc93, 100 },
    { 0x2dddc94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ddc9a, 52 },
    { 0xae7ddc9b, 98 },
    { 0x3a0ddc9c, 76 },
    { 0, 0 },
    { 0xfe3ddc9e, 98 },
    { 0x7fcddc9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261ddca5, 95 },
    { 0x35cddca6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8ddcaa, 65 },
    { 0xa3addcab, 43 },
    { 0x3a5ddcac, 33 },
    { 0xec9ddcad, 55 },
    { 0, 0 },
    { 0x638ddcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503ddcb3, 95 },
    { 0x881ddcb4, 95 },
    { 0x519ddcb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ddcbb, 55 },
    { 0, 0 },
    { 0x782ddcbd, 55 },
    { 0xcddddcbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381ddcc6, 23 },
    { 0, 0 },
    { 0xf22ddcc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6ddccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ddcd2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ddcdd, 33 },
    { 0x956ddcde, 66 },
    { 0x201ddcde, 23 },
    { 0, 0 },
    { 0xf98ddce1, 33 },
    { 0x270ddce2, 43 },
    { 0, 0 },
    { 0x8aeddce4, 90 },
    { 0xbe3ddce5, 90 },
    { 0xff0ddce6, 61 },
    { 0x7c0ddce5, 40 },
    { 0x6c4ddce4, 88 },
    { 0x510ddce6, 55 },
    { 0x5ebddcea, 95 },
    { 0xb63ddce9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6ddcf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8ddcf4, 88 },
    { 0xd53ddcf4, 90 },
    { 0xa7bddcf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02ddcfe, 98 },
    { 0x73fddcfe, 34 },
    { 0x574ddd00, 33 },
    { 0xca5ddd01, 52 },
    { 0x458ddd01, 76 },
    { 0xeceddd02, 61 },
    { 0x97bddd03, 76 },
    { 0xc6dddd01, 76 },
    { 0x37eddd05, 95 },
    { 0x64dddd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3ddd0e, 85 },
    { 0x636ddd0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fddd14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ddd17, 17 },
    { 0xac7ddd17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eddd20, 98 },
    { 0, 0 },
    { 0x4afddd22, 95 },
    { 0x6f0ddd23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cddd28, 61 },
    { 0x3e2ddd28, 78 },
    { 0, 0 },
    { 0xf5cddd2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67ddd2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ddd33, 4 },
    { 0x23bddd34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2ddd37, 78 },
    { 0x2dbddd38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890ddd3e, 68 },
    { 0, 0 },
    { 0x420ddd40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdddd48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x269ddd4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ddd4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dddd55, 33 },
    { 0, 0 },
    { 0xda7ddd57, 23 },
    { 0x10addd58, 23 },
    { 0xac7ddd59, 61 },
    { 0xe29ddd5a, 66 },
    { 0xd9ddd59, 34 },
    { 0x27eddd5b, 61 },
    { 0x7a2ddd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70ddd60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffddd66, 76 },
    { 0x171ddd67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ddd73, 23 },
    { 0x517ddd74, 90 },
    { 0x77fddd75, 61 },
    { 0xe6cddd76, 99 },
    { 0x627ddd77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dddd7b, 61 },
    { 0x5c9ddd7c, 23 },
    { 0xe8eddd7c, 34 },
    { 0, 0 },
    { 0x66eddd7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ddd82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24ddd85, 90 },
    { 0, 0 },
    { 0xdabddd87, 76 },
    { 0, 0 },
    { 0x336ddd89, 52 },
    { 0x574ddd8a, 52 },
    { 0x4e1ddd89, 52 },
    { 0x8ceddd8c, 90 },
    { 0, 0 },
    { 0xd26ddd8e, 33 },
    { 0xdb0ddd8e, 98 },
    { 0x9bcddd90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4ddd97, 83 },
    { 0x387ddd98, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ddd9d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bddda3, 95 },
    { 0x208ddda4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fddda9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5dddaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdddb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0adddb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8dddba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4dddbe, 61 },
    { 0, 0 },
    { 0xd87dddc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35adddc5, 76 },
    { 0x1bbdddc5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8edddca, 87 },
    { 0xba3dddca, 66 },
    { 0x95adddcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fdddcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512dddd6, 90 },
    { 0x6e9dddd7, 61 },
    { 0xf96dddd8, 85 },
    { 0x424dddd9, 33 },
    { 0xc4bdddda, 87 },
    { 0xc08ddddb, 23 },
    { 0xa6bddddc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3ddddf, 23 },
    { 0x887ddddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cddde3, 71 },
    { 0x112ddde4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ddde7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6dddeb, 76 },
    { 0x91dddec, 77 },
    { 0xa2fddded, 90 },
    { 0xb50dddeb, 98 },
    { 0xa95ddded, 90 },
    { 0, 0 },
    { 0xa27dddf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35dddf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bdddf8, 87 },
    { 0x9adddf9, 66 },
    { 0xa55dddf8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bdddfd, 95 },
    { 0, 0 },
    { 0x685dddff, 61 },
    { 0x6b5dde00, 61 },
    { 0x389dde01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7dde06, 93 },
    { 0xe6fdde06, 61 },
    { 0, 0 },
    { 0xfbcdde09, 98 },
    { 0xb7fdde0a, 33 },
    { 0x8e2dde09, 93 },
    { 0xa94dde09, 100 },
    { 0x354dde0d, 52 },
    { 0xab9dde0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70adde17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fdde1a, 95 },
    { 0xccedde1b, 95 },
    { 0x54cdde1c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdde23, 93 },
    { 0x44ddde24, 81 },
    { 0x27dde25, 93 },
    { 0xa4edde25, 61 },
    { 0, 0 },
    { 0xedde28, 88 },
    { 0x457dde29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bdde2d, 98 },
    { 0x617dde2e, 98 },
    { 0, 0 },
    { 0xfcdde30, 61 },
    { 0x98dde30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1dde38, 90 },
    { 0x89edde39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bddde3c, 98 },
    { 0, 0 },
    { 0xe50dde3e, 23 },
    { 0x14adde3f, 46 },
    { 0xbfedde3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1badde47, 33 },
    { 0xd7adde47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25dde4e, 98 },
    { 0, 0 },
    { 0x22cdde50, 34 },
    { 0x1e7dde50, 91 },
    { 0, 0 },
    { 0xdaadde53, 61 },
    { 0x163dde53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cadde5b, 23 },
    { 0, 0 },
    { 0x4fdde5d, 55 },
    { 0x57adde5e, 75 },
    { 0xf48dde5f, 33 },
    { 0x824dde5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ddde64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdde68, 77 },
    { 0x5ecdde68, 95 },
    { 0, 0 },
    { 0x5b5dde6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dde6e, 55 },
    { 0, 0 },
    { 0x4ccdde70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdde76, 6 },
    { 0xa71dde77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde7c, 61 },
    { 0x913dde7c, 33 },
    { 0, 0 },
    { 0xfbedde7f, 23 },
    { 0, 0 },
    { 0x589dde81, 76 },
    { 0x1f1dde81, 78 },
    { 0xb1bdde82, 90 },
    { 0x31bdde84, 98 },
    { 0xd70dde81, 64 },
    { 0x581dde85, 90 },
    { 0x510dde87, 90 },
    { 0xb66dde88, 98 },
    { 0x443dde81, 90 },
    { 0xaeddde84, 90 },
    { 0xad3dde87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7dde8e, 86 },
    { 0x6efdde8f, 99 },
    { 0xff5dde90, 100 },
    { 0xff6dde8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826dde9b, 34 },
    { 0, 0 },
    { 0xb76dde9d, 95 },
    { 0x338dde9d, 61 },
    { 0, 0 },
    { 0x2bdddea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ddea3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81addeab, 23 },
    { 0, 0 },
    { 0x513ddead, 76 },
    { 0x9aeddeae, 95 },
    { 0xc70ddeae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ddeba, 95 },
    { 0xc17ddeba, 99 },
    { 0x73dddebc, 98 },
    { 0, 0 },
    { 0x37fddebe, 38 },
    { 0xfb8ddebf, 33 },
    { 0x1daddec0, 91 },
    { 0xb97ddec1, 88 },
    { 0x864ddec1, 43 },
    { 0xf81ddec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5ddec7, 55 },
    { 0, 0 },
    { 0x190ddec9, 90 },
    { 0x68cddec9, 87 },
    { 0x7c6ddec9, 34 },
    { 0x7a7ddecc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdddedb, 33 },
    { 0x99fddedc, 76 },
    { 0x99bddedb, 93 },
    { 0xe8addedb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80ddeea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dddef1, 95 },
    { 0x896ddef2, 40 },
    { 0x66fddef3, 92 },
    { 0x439ddef3, 98 },
    { 0, 0 },
    { 0x678ddef6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eddefe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaddf02, 95 },
    { 0x89ddf03, 91 },
    { 0xcffddf04, 78 },
    { 0, 0 },
    { 0xb69ddf06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ddf0a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddf10, 61 },
    { 0x6f3ddf10, 33 },
    { 0x297ddf12, 88 },
    { 0x6e4ddf13, 98 },
    { 0x780ddf14, 93 },
    { 0xc7eddf15, 53 },
    { 0xd4fddf14, 43 },
    { 0, 0 },
    { 0x16bddf18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ddf1c, 33 },
    { 0, 0 },
    { 0x1afddf1e, 95 },
    { 0x10eddf1f, 95 },
    { 0x95fddf20, 95 },
    { 0, 0 },
    { 0xc1bddf22, 34 },
    { 0, 0 },
    { 0x6bfddf24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ddf2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ddf2e, 55 },
    { 0, 0 },
    { 0xb4fddf30, 47 },
    { 0, 0 },
    { 0xa2dddf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ddf36, 33 },
    { 0, 0 },
    { 0x701ddf38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5addf3d, 61 },
    { 0xd48ddf3e, 78 },
    { 0x238ddf3f, 90 },
    { 0xcaeddf3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224ddf44, 55 },
    { 0xf5ddf44, 33 },
    { 0xf70ddf44, 33 },
    { 0xcdaddf47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddf4e, 90 },
    { 0, 0 },
    { 0xd92ddf50, 91 },
    { 0x19ddf50, 52 },
    { 0, 0 },
    { 0x1e8ddf53, 68 },
    { 0, 0 },
    { 0x17cddf55, 97 },
    { 0x14dddf56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ddf59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ddf60, 55 },
    { 0x95bddf61, 33 },
    { 0, 0 },
    { 0xa47ddf63, 98 },
    { 0, 0 },
    { 0x725ddf65, 34 },
    { 0, 0 },
    { 0x5e2ddf67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddddf6a, 68 },
    { 0xf00ddf6a, 98 },
    { 0x337ddf6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ddf70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0ddf77, 61 },
    { 0x62cddf77, 90 },
    { 0xad5ddf79, 23 },
    { 0, 0 },
    { 0xc8eddf7b, 61 },
    { 0x1f0ddf7b, 76 },
    { 0, 0 },
    { 0x1d8ddf7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ddf81, 67 },
    { 0xd05ddf82, 55 },
    { 0, 0 },
    { 0xc11ddf84, 46 },
    { 0xfddf85, 52 },
    { 0x8a9ddf84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ddf8a, 33 },
    { 0xdf8ddf8b, 98 },
    { 0xc10ddf8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x594ddf8f, 99 },
    { 0x1a0ddf90, 46 },
    { 0xe3fddf90, 90 },
    { 0x545ddf92, 55 },
    { 0, 0 },
    { 0x996ddf94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3ddf97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feddfa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ddfa9, 100 },
    { 0x2bddfaa, 61 },
    { 0x452ddfaa, 61 },
    { 0x4a3ddfac, 66 },
    { 0xe58ddfad, 95 },
    { 0xf74ddfaa, 61 },
    { 0x602ddfa9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ddfb4, 52 },
    { 0x379ddfb5, 33 },
    { 0, 0 },
    { 0x93ddfb7, 95 },
    { 0x430ddfb8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ddfc0, 76 },
    { 0x9dcddfc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ddfc8, 55 },
    { 0, 0 },
    { 0x631ddfca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73ddfcf, 46 },
    { 0x2d9ddfd0, 34 },
    { 0x47fddfd1, 93 },
    { 0, 0 },
    { 0x92dddfd3, 95 },
    { 0x525ddfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ddfd8, 61 },
    { 0x1cddfd9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ddfe0, 98 },
    { 0x104ddfe1, 95 },
    { 0, 0 },
    { 0xda3ddfe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eddfe6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635ddfed, 84 },
    { 0xb4eddfee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19ddff1, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ddff4, 33 },
    { 0x660ddff5, 90 },
    { 0, 0 },
    { 0x71addff7, 23 },
    { 0, 0 },
    { 0xb94ddff9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02ddffe, 76 },
    { 0, 0 },
    { 0x64cde000, 91 },
    { 0xc2bde000, 76 },
    { 0xe3dde001, 33 },
    { 0x7dede003, 88 },
    { 0, 0 },
    { 0x440de005, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdde00a, 78 },
    { 0x2dcde00b, 98 },
    { 0xa5bde00b, 55 },
    { 0x55ade00a, 100 },
    { 0x61dde00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1de011, 76 },
    { 0x78de012, 53 },
    { 0, 0 },
    { 0x859de014, 95 },
    { 0xa26de014, 95 },
    { 0xe71de014, 90 },
    { 0, 0 },
    { 0x5a6de018, 46 },
    { 0xe17de018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2de01d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fde021, 93 },
    { 0xdd9de022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694de026, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dde029, 55 },
    { 0x113de02a, 61 },
    { 0x8cde02b, 90 },
    { 0x46cde02c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dde033, 88 },
    { 0, 0 },
    { 0x2a8de035, 91 },
    { 0x7a0de036, 34 },
    { 0, 0 },
    { 0x8cde038, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffde03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de043, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453de04a, 38 },
    { 0, 0 },
    { 0x793de04c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bde052, 100 },
    { 0x132de053, 23 },
    { 0x740de054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dde057, 55 },
    { 0x7f7de058, 83 },
    { 0x169de059, 91 },
    { 0x1dade05a, 66 },
    { 0xf14de05a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59de06c, 93 },
    { 0xf60de06d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0de070, 44 },
    { 0, 0 },
    { 0xebade072, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7de075, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83de078, 66 },
    { 0, 0 },
    { 0x5dfde07a, 34 },
    { 0, 0 },
    { 0x6eade07c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ede07f, 61 },
    { 0xeeade080, 90 },
    { 0x6b6de081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ade086, 95 },
    { 0x7c3de087, 61 },
    { 0, 0 },
    { 0x453de089, 23 },
    { 0x116de089, 80 },
    { 0, 0 },
    { 0xf83de08c, 80 },
    { 0, 0 },
    { 0x38bde08e, 90 },
    { 0, 0 },
    { 0xadcde090, 77 },
    { 0x42de090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x523de094, 90 },
    { 0, 0 },
    { 0xc06de096, 34 },
    { 0, 0 },
    { 0xade098, 33 },
    { 0x27dde099, 55 },
    { 0, 0 },
    { 0xa0cde09b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ade09e, 61 },
    { 0, 0 },
    { 0x8ede0a0, 90 },
    { 0x749de0a1, 95 },
    { 0x42cde0a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3de0a5, 61 },
    { 0xc19de0a5, 95 },
    { 0, 0 },
    { 0x457de0a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48de0ab, 76 },
    { 0xb56de0ac, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393de0b0, 90 },
    { 0, 0 },
    { 0x1aede0b2, 23 },
    { 0xc26de0b3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457de0b9, 61 },
    { 0xa9ade0ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ede0c4, 61 },
    { 0xe9ade0c5, 90 },
    { 0x13fde0c4, 61 },
    { 0xe60de0c5, 33 },
    { 0x619de0c7, 55 },
    { 0x3c3de0c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5de0ce, 61 },
    { 0x19fde0ce, 98 },
    { 0, 0 },
    { 0xbbade0d1, 90 },
    { 0xbc3de0d2, 33 },
    { 0x22ade0d2, 23 },
    { 0x5f5de0d4, 55 },
    { 0x1c7de0d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x715de0d8, 86 },
    { 0x7e8de0d8, 34 },
    { 0xbcdde0d9, 61 },
    { 0, 0 },
    { 0x1edde0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27de0e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849de0e8, 33 },
    { 0x3bde0e9, 61 },
    { 0xe42de0e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1de0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3de0f5, 91 },
    { 0, 0 },
    { 0x66ade0f7, 98 },
    { 0xd6dde0f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de0fb, 43 },
    { 0, 0 },
    { 0x7e6de0fd, 33 },
    { 0x3dede0fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7de104, 90 },
    { 0x21de104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81de10a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x608de10d, 76 },
    { 0x918de10e, 33 },
    { 0x7a1de10f, 90 },
    { 0x12dde10f, 98 },
    { 0x528de111, 61 },
    { 0xea2de111, 66 },
    { 0x40dde110, 76 },
    { 0xc9bde10e, 55 },
    { 0x19bde115, 33 },
    { 0, 0 },
    { 0x3ffde117, 52 },
    { 0, 0 },
    { 0xd3ade119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ade121, 99 },
    { 0x656de122, 90 },
    { 0, 0 },
    { 0xecde124, 90 },
    { 0x9fcde125, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1de128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6de12c, 34 },
    { 0xdb3de12d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7de131, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ade134, 98 },
    { 0x8a4de135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ade13b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6de142, 43 },
    { 0x73ede142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829de148, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ede14b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9de14f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51de156, 98 },
    { 0x964de157, 67 },
    { 0x9aede156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cde15b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fde15e, 90 },
    { 0xb39de15f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8de16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4de170, 55 },
    { 0x5f3de170, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7de175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390de17f, 84 },
    { 0xddfde180, 93 },
    { 0, 0 },
    { 0x42dde182, 23 },
    { 0x8b5de183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319de187, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544de18c, 90 },
    { 0x62fde18d, 78 },
    { 0, 0 },
    { 0xf2dde18f, 23 },
    { 0xf1fde18f, 100 },
    { 0xea9de191, 81 },
    { 0, 0 },
    { 0xbcfde193, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420de198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742de19e, 61 },
    { 0xf2fde19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbde1a4, 88 },
    { 0x2e7de1a5, 34 },
    { 0x296de1a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dde1a9, 62 },
    { 0x5cfde1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fde1ad, 85 },
    { 0x6d4de1ae, 77 },
    { 0xfdde1af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79de1b2, 81 },
    { 0xe0bde1b3, 95 },
    { 0x19ede1b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798de1be, 95 },
    { 0xb1ede1bf, 95 },
    { 0xd1ede1c0, 77 },
    { 0xc97de1c0, 90 },
    { 0xb9dde1be, 75 },
    { 0xcd5de1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641de1c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dde1cd, 99 },
    { 0xf29de1ce, 61 },
    { 0xa43de1cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9de1d3, 76 },
    { 0xde0de1d3, 43 },
    { 0xe51de1d3, 99 },
    { 0, 0 },
    { 0xef4de1d7, 43 },
    { 0, 0 },
    { 0xe3ade1d9, 91 },
    { 0x95cde1da, 96 },
    { 0, 0 },
    { 0xbcede1dc, 43 },
    { 0xb69de1dd, 61 },
    { 0xce1de1de, 23 },
    { 0x102de1de, 34 },
    { 0x452de1e0, 33 },
    { 0xe3de1e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414de1e8, 33 },
    { 0, 0 },
    { 0xf7ede1ea, 43 },
    { 0x6dcde1eb, 34 },
    { 0, 0 },
    { 0xa17de1ed, 98 },
    { 0, 0 },
    { 0xdb6de1ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcde1f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ade1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348de202, 33 },
    { 0x958de203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdde206, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13de212, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fde217, 23 },
    { 0xf04de217, 100 },
    { 0x88ede219, 61 },
    { 0x78fde21a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x438de21d, 98 },
    { 0x5c2de21e, 34 },
    { 0, 0 },
    { 0x54ade220, 23 },
    { 0x4f2de220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791de22e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6de231, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfde235, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bde23c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2de240, 68 },
    { 0xa3ede241, 23 },
    { 0x134de242, 55 },
    { 0x95dde240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23de24c, 87 },
    { 0, 0 },
    { 0x72ade24e, 65 },
    { 0x616de24e, 61 },
    { 0, 0 },
    { 0x6bcde251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81de259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cde25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dde25f, 90 },
    { 0, 0 },
    { 0x968de261, 95 },
    { 0x635de261, 76 },
    { 0, 0 },
    { 0xe54de264, 23 },
    { 0x250de264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5de26b, 87 },
    { 0x8f7de26c, 23 },
    { 0, 0 },
    { 0x128de26e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24de271, 100 },
    { 0, 0 },
    { 0x7ffde273, 88 },
    { 0x8b5de274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4de278, 33 },
    { 0xd28de279, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecde27c, 66 },
    { 0xcc7de27d, 55 },
    { 0, 0 },
    { 0x2e1de27f, 55 },
    { 0x211de280, 88 },
    { 0x5b2de281, 98 },
    { 0, 0 },
    { 0x55ade283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ede287, 33 },
    { 0xad6de287, 90 },
    { 0xd08de289, 93 },
    { 0, 0 },
    { 0xc8ede28b, 90 },
    { 0, 0 },
    { 0xf45de28d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfde296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5de29a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687de29e, 55 },
    { 0, 0 },
    { 0xda0de2a0, 23 },
    { 0xf37de2a0, 46 },
    { 0xde1de2a2, 76 },
    { 0x818de2a3, 78 },
    { 0x839de2a4, 61 },
    { 0x9edde2a5, 93 },
    { 0, 0 },
    { 0xfe8de2a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954de2ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62de2b2, 80 },
    { 0, 0 },
    { 0xf60de2b4, 43 },
    { 0, 0 },
    { 0x195de2b6, 61 },
    { 0x9d7de2b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03de2bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cde2be, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dde2c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83de2c4, 95 },
    { 0, 0 },
    { 0x6bde2c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1de2cc, 90 },
    { 0x2ecde2cd, 85 },
    { 0xf59de2ce, 85 },
    { 0x3c4de2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30de2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dde2d9, 33 },
    { 0x2cfde2da, 98 },
    { 0x3efde2da, 76 },
    { 0x773de2d9, 33 },
    { 0xdcdde2dd, 61 },
    { 0xcade2d9, 95 },
    { 0, 0 },
    { 0x137de2e0, 65 },
    { 0xd15de2e0, 87 },
    { 0, 0 },
    { 0xa09de2e3, 34 },
    { 0, 0 },
    { 0x84ade2e5, 91 },
    { 0x8a4de2e6, 98 },
    { 0xbffde2e5, 61 },
    { 0x4c4de2e8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacde2eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dde2ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afde2f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ede2f8, 61 },
    { 0, 0 },
    { 0x741de2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68de2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fede300, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0de305, 52 },
    { 0x455de306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8de309, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6de30c, 76 },
    { 0x9e5de30c, 76 },
    { 0, 0 },
    { 0x4fbde30f, 61 },
    { 0xbe2de310, 100 },
    { 0, 0 },
    { 0x57cde312, 46 },
    { 0x907de313, 90 },
    { 0x697de314, 84 },
    { 0x304de312, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x844de318, 99 },
    { 0xe2cde319, 98 },
    { 0, 0 },
    { 0xd9cde31b, 34 },
    { 0xa98de31c, 23 },
    { 0, 0 },
    { 0x2bcde31e, 90 },
    { 0x63de31f, 17 },
    { 0x8a0de31e, 23 },
    { 0xce7de321, 87 },
    { 0x600de31e, 61 },
    { 0, 0 },
    { 0x90ede324, 95 },
    { 0, 0 },
    { 0xf81de326, 84 },
    { 0xda4de327, 32 },
    { 0xa0ade328, 33 },
    { 0, 0 },
    { 0xb26de32a, 61 },
    { 0, 0 },
    { 0x70ade32c, 23 },
    { 0x425de32c, 66 },
    { 0x67de32e, 34 },
    { 0xdffde32f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575de336, 90 },
    { 0x1f8de337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3de33a, 90 },
    { 0x2abde33a, 98 },
    { 0x2bede33c, 66 },
    { 0x8bde33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0de340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767de349, 55 },
    { 0x904de34a, 33 },
    { 0xc60de34b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690de350, 98 },
    { 0, 0 },
    { 0x934de352, 78 },
    { 0xd9bde352, 99 },
    { 0x7dcde354, 34 },
    { 0x914de355, 33 },
    { 0, 0 },
    { 0x18bde357, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62de35a, 55 },
    { 0xa7bde35b, 33 },
    { 0, 0 },
    { 0x1f8de35d, 33 },
    { 0x6f5de35e, 95 },
    { 0, 0 },
    { 0x38dde360, 87 },
    { 0x25ede361, 23 },
    { 0xc78de361, 34 },
    { 0x520de363, 66 },
    { 0x23cde364, 61 },
    { 0xf1de363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fde368, 95 },
    { 0, 0 },
    { 0xdf4de36a, 90 },
    { 0, 0 },
    { 0xdecde36c, 90 },
    { 0x365de36d, 90 },
    { 0xbc5de36e, 90 },
    { 0xb5de36f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbede376, 90 },
    { 0x999de377, 61 },
    { 0x136de378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64de37b, 93 },
    { 0, 0 },
    { 0xdde37d, 61 },
    { 0xc77de37e, 90 },
    { 0, 0 },
    { 0x63ade380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de388, 76 },
    { 0xf93de388, 76 },
    { 0, 0 },
    { 0x609de38b, 33 },
    { 0xb6ede38b, 90 },
    { 0xb0bde38c, 43 },
    { 0x3d9de38e, 98 },
    { 0xc5cde38b, 33 },
    { 0xe78de38b, 33 },
    { 0x2d2de391, 52 },
    { 0x28fde392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecde395, 78 },
    { 0, 0 },
    { 0x9acde397, 98 },
    { 0x67bde398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7de3a1, 98 },
    { 0, 0 },
    { 0x81de3a3, 95 },
    { 0x73cde3a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282de3aa, 61 },
    { 0, 0 },
    { 0x80bde3ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cde3b4, 61 },
    { 0xa7de3b4, 90 },
    { 0, 0 },
    { 0xff6de3b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956de3bb, 90 },
    { 0x2a9de3bb, 95 },
    { 0xcf7de3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3de3c0, 81 },
    { 0xbc5de3c0, 61 },
    { 0x602de3c0, 64 },
    { 0x742de3c3, 61 },
    { 0x4bede3c3, 95 },
    { 0, 0 },
    { 0x287de3c6, 95 },
    { 0x564de3c6, 75 },
    { 0xfe9de3c8, 84 },
    { 0, 0 },
    { 0x478de3ca, 95 },
    { 0, 0 },
    { 0xb7ede3cc, 98 },
    { 0, 0 },
    { 0xad1de3ce, 90 },
    { 0xc22de3ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7de3d4, 91 },
    { 0xb5fde3d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9de3d9, 43 },
    { 0x650de3da, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4de3df, 33 },
    { 0xf73de3e0, 66 },
    { 0x6bfde3e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x438de3e4, 95 },
    { 0xefade3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ede3e8, 95 },
    { 0x7dbde3e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ade3ec, 34 },
    { 0, 0 },
    { 0x4dcde3ee, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ede3f1, 95 },
    { 0, 0 },
    { 0x8f0de3f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x297de3f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0de3f9, 98 },
    { 0, 0 },
    { 0xc86de3fb, 76 },
    { 0xbeade3fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0de3ff, 43 },
    { 0x5b8de400, 98 },
    { 0x214de401, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6de406, 66 },
    { 0xe02de407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366de40c, 95 },
    { 0, 0 },
    { 0xedde40e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14de412, 76 },
    { 0, 0 },
    { 0xefde414, 76 },
    { 0, 0 },
    { 0x740de416, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bde419, 95 },
    { 0x7edde41a, 33 },
    { 0, 0 },
    { 0x881de41c, 52 },
    { 0xc87de41d, 61 },
    { 0x299de41c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fade421, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635de425, 55 },
    { 0xd7cde426, 61 },
    { 0xe1fde426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3de42c, 90 },
    { 0x5fde42d, 76 },
    { 0xb22de42e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dde433, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fde436, 95 },
    { 0, 0 },
    { 0xddfde438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20de444, 52 },
    { 0x25ade444, 87 },
    { 0, 0 },
    { 0x371de447, 33 },
    { 0x1abde447, 90 },
    { 0xb40de448, 23 },
    { 0xc0ede44a, 40 },
    { 0, 0 },
    { 0x1e7de44c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8de450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5de453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fde456, 33 },
    { 0x1eede456, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93de45e, 69 },
    { 0xae7de45f, 90 },
    { 0, 0 },
    { 0xb44de461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35de467, 33 },
    { 0x98ade468, 90 },
    { 0xfc3de469, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2de46d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cde471, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1de474, 56 },
    { 0, 0 },
    { 0xcd1de476, 61 },
    { 0xe6dde477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73de47a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41de47d, 52 },
    { 0x9c0de47e, 66 },
    { 0x934de47f, 95 },
    { 0x272de47e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518de48c, 23 },
    { 0x415de48d, 95 },
    { 0, 0 },
    { 0xf47de48f, 90 },
    { 0, 0 },
    { 0xc35de491, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0de497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdde4a3, 23 },
    { 0, 0 },
    { 0x1a6de4a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de4aa, 55 },
    { 0xec6de4aa, 23 },
    { 0, 0 },
    { 0x8a0de4ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fde4b6, 59 },
    { 0xad9de4b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382de4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1de4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520de4c7, 46 },
    { 0x665de4c8, 23 },
    { 0x66bde4c8, 43 },
    { 0x931de4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dde4ce, 83 },
    { 0x207de4ce, 23 },
    { 0, 0 },
    { 0x918de4d1, 33 },
    { 0x725de4d2, 61 },
    { 0xd78de4d2, 85 },
    { 0x642de4d4, 33 },
    { 0x21fde4d5, 61 },
    { 0x4b0de4d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73de4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafade4dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eede4e4, 33 },
    { 0, 0 },
    { 0xd00de4e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10de4eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1de4f1, 88 },
    { 0x745de4f1, 91 },
    { 0x9ebde4f3, 75 },
    { 0xa34de4f4, 77 },
    { 0xe09de4f4, 68 },
    { 0, 0 },
    { 0xa06de4f7, 87 },
    { 0xe8dde4f8, 78 },
    { 0x2c7de4f9, 100 },
    { 0, 0 },
    { 0x36cde4fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebde4ff, 96 },
    { 0x7f1de4ff, 95 },
    { 0, 0 },
    { 0x467de502, 33 },
    { 0x795de503, 69 },
    { 0x220de502, 61 },
    { 0xf25de504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ade509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333de50d, 61 },
    { 0, 0 },
    { 0xdb9de50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfde512, 90 },
    { 0, 0 },
    { 0xaa3de514, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ede518, 52 },
    { 0x470de519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de51f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6de523, 93 },
    { 0xfc4de524, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1de528, 33 },
    { 0, 0 },
    { 0xab8de52a, 23 },
    { 0, 0 },
    { 0x78dde52c, 43 },
    { 0x669de52d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ede531, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bde53b, 66 },
    { 0, 0 },
    { 0x9f4de53d, 52 },
    { 0, 0 },
    { 0x543de53f, 98 },
    { 0, 0 },
    { 0x52ede541, 61 },
    { 0xa0fde542, 33 },
    { 0, 0 },
    { 0x482de544, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bde54c, 61 },
    { 0xdc6de54d, 23 },
    { 0x1b2de54e, 78 },
    { 0xdedde54f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fde552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477de557, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6de55a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532de561, 87 },
    { 0x55dde562, 33 },
    { 0x815de563, 88 },
    { 0x86bde563, 75 },
    { 0x2bbde561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedde568, 98 },
    { 0x705de569, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fde56d, 90 },
    { 0x87dde56d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66de571, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34de578, 98 },
    { 0xa2cde578, 95 },
    { 0x1b2de57a, 90 },
    { 0xa7de57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37de57e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26de582, 43 },
    { 0, 0 },
    { 0xa67de584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfde589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97de58c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621de591, 76 },
    { 0, 0 },
    { 0x7f4de593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cde59a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7de5a2, 90 },
    { 0, 0 },
    { 0x8c0de5a4, 93 },
    { 0xed9de5a5, 61 },
    { 0x114de5a4, 61 },
    { 0, 0 },
    { 0x43ede5a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479de5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607de5b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x553de5b9, 99 },
    { 0x73dde5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cde5bd, 67 },
    { 0, 0 },
    { 0x713de5bf, 38 },
    { 0xde0de5bf, 98 },
    { 0x484de5c1, 23 },
    { 0xb81de5bf, 65 },
    { 0xe11de5c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254de5c8, 98 },
    { 0x3b7de5c9, 93 },
    { 0xede5c9, 99 },
    { 0x6e8de5cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5de5d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4de5d3, 43 },
    { 0, 0 },
    { 0x26ade5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8de5dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3de5e3, 95 },
    { 0, 0 },
    { 0x632de5e5, 66 },
    { 0xcb2de5e6, 61 },
    { 0, 0 },
    { 0x43cde5e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde5ee, 23 },
    { 0x46de5ef, 34 },
    { 0x315de5f0, 66 },
    { 0, 0 },
    { 0x352de5f2, 33 },
    { 0x2b7de5f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfde5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fede5ff, 67 },
    { 0x502de600, 23 },
    { 0xd0de600, 61 },
    { 0xfa5de602, 23 },
    { 0x2a8de603, 81 },
    { 0xb23de603, 90 },
    { 0xd0de603, 95 },
    { 0xbf5de606, 98 },
    { 0xa7bde607, 46 },
    { 0, 0 },
    { 0x6b2de609, 47 },
    { 0xf84de60a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ede60e, 93 },
    { 0x5adde60f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0de612, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5de616, 61 },
    { 0xfc0de616, 23 },
    { 0xb24de618, 55 },
    { 0, 0 },
    { 0x9de61a, 76 },
    { 0, 0 },
    { 0xb3ade61c, 61 },
    { 0x2fcde61c, 84 },
    { 0xec7de61e, 95 },
    { 0, 0 },
    { 0x114de620, 80 },
    { 0xddde621, 87 },
    { 0x27fde621, 33 },
    { 0xc9ade620, 23 },
    { 0, 0 },
    { 0x2d0de625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988de629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ede62d, 76 },
    { 0x972de62e, 98 },
    { 0x2c9de62f, 61 },
    { 0x71cde62f, 98 },
    { 0xc9fde62e, 33 },
    { 0x194de632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4de63a, 98 },
    { 0xb86de63a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39de640, 33 },
    { 0x597de641, 97 },
    { 0x397de642, 33 },
    { 0, 0 },
    { 0xe8bde644, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cde64b, 33 },
    { 0xa5cde64c, 90 },
    { 0, 0 },
    { 0x158de64e, 90 },
    { 0, 0 },
    { 0xc24de650, 90 },
    { 0xe93de650, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321de65e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1de663, 23 },
    { 0, 0 },
    { 0x417de665, 91 },
    { 0x399de666, 95 },
    { 0, 0 },
    { 0xda7de668, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaade66c, 33 },
    { 0, 0 },
    { 0x13ede66e, 95 },
    { 0, 0 },
    { 0x359de670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50de673, 100 },
    { 0, 0 },
    { 0x823de675, 61 },
    { 0xc7dde675, 34 },
    { 0x72de677, 95 },
    { 0x453de678, 55 },
    { 0, 0 },
    { 0x490de67a, 61 },
    { 0xe50de67b, 55 },
    { 0xd0ade67b, 100 },
    { 0xe23de67a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fde682, 91 },
    { 0xbcfde683, 34 },
    { 0xedde682, 55 },
    { 0xc6dde685, 76 },
    { 0, 0 },
    { 0x8de687, 77 },
    { 0, 0 },
    { 0xebede689, 84 },
    { 0x58fde68a, 55 },
    { 0, 0 },
    { 0xf46de68c, 76 },
    { 0xd5ede68d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5de690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ede693, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ade69d, 93 },
    { 0, 0 },
    { 0xce0de69f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bde6a3, 47 },
    { 0xed2de6a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede6a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fade6ac, 90 },
    { 0xc44de6ad, 90 },
    { 0x6a8de6ae, 98 },
    { 0x905de6ae, 61 },
    { 0x99ede6b0, 46 },
    { 0x130de6b0, 91 },
    { 0, 0 },
    { 0x2eede6b3, 33 },
    { 0xb8cde6b4, 90 },
    { 0x338de6b5, 43 },
    { 0x8b9de6b4, 99 },
    { 0xbbde6b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74de6bc, 98 },
    { 0xdcfde6bc, 84 },
    { 0xb6de6be, 90 },
    { 0x1f5de6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x319de6c2, 98 },
    { 0x9d8de6c2, 95 },
    { 0x15ade6c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cde6c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeede6ca, 33 },
    { 0x545de6ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadde6ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x267de6d1, 23 },
    { 0x1bfde6d1, 66 },
    { 0x572de6d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67de6d8, 31 },
    { 0x11cde6d9, 100 },
    { 0, 0 },
    { 0x6a2de6db, 33 },
    { 0x158de6dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99de6e3, 90 },
    { 0x295de6e4, 34 },
    { 0xbf4de6e5, 61 },
    { 0x7e4de6e6, 85 },
    { 0x8a8de6e7, 61 },
    { 0xefede6e8, 23 },
    { 0xbd2de6e7, 95 },
    { 0xb94de6e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cde6fb, 90 },
    { 0, 0 },
    { 0xbb6de6fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adde700, 98 },
    { 0x87bde701, 61 },
    { 0, 0 },
    { 0xbc2de703, 61 },
    { 0xc7de704, 90 },
    { 0, 0 },
    { 0xe31de706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x772de709, 90 },
    { 0xf4bde709, 61 },
    { 0x1d1de70b, 55 },
    { 0xf60de70c, 33 },
    { 0x9acde70d, 34 },
    { 0x474de70e, 84 },
    { 0x34de70e, 23 },
    { 0x6edde710, 98 },
    { 0, 0 },
    { 0xb52de712, 17 },
    { 0xb89de712, 64 },
    { 0, 0 },
    { 0x57bde715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfde718, 33 },
    { 0x92dde719, 90 },
    { 0, 0 },
    { 0x469de71b, 76 },
    { 0, 0 },
    { 0xacbde71d, 61 },
    { 0x971de71d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1de721, 90 },
    { 0, 0 },
    { 0x27ade723, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352de72e, 76 },
    { 0xb02de72f, 43 },
    { 0xbf9de72f, 90 },
    { 0, 0 },
    { 0x81cde732, 98 },
    { 0x126de733, 98 },
    { 0xc07de734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173de738, 98 },
    { 0xa0de739, 33 },
    { 0x5d5de73a, 52 },
    { 0x6c6de73a, 33 },
    { 0x214de73c, 76 },
    { 0xd86de73b, 33 },
    { 0, 0 },
    { 0x39ede73f, 82 },
    { 0xf6de740, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737de749, 90 },
    { 0x487de74a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258de750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477de754, 33 },
    { 0x66cde754, 98 },
    { 0xb6fde756, 55 },
    { 0x428de757, 33 },
    { 0x74fde758, 43 },
    { 0x6dbde758, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505de75f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146de76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fde76e, 78 },
    { 0x2dcde76e, 90 },
    { 0x7a4de770, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de775, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ede77a, 76 },
    { 0xf2bde77a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7de77e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442de781, 23 },
    { 0, 0 },
    { 0xebade783, 95 },
    { 0x9fdde783, 33 },
    { 0, 0 },
    { 0xd41de786, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1de78a, 66 },
    { 0, 0 },
    { 0xd9bde78c, 61 },
    { 0x206de78d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376de793, 53 },
    { 0x143de793, 68 },
    { 0, 0 },
    { 0xe8ade796, 95 },
    { 0x38bde796, 33 },
    { 0x4adde798, 99 },
    { 0xfa0de799, 67 },
    { 0xe8ade79a, 95 },
    { 0xec8de79a, 34 },
    { 0xadbde79c, 36 },
    { 0xf72de79d, 33 },
    { 0x92cde79e, 98 },
    { 0xd2de79f, 87 },
    { 0xb97de7a0, 84 },
    { 0, 0 },
    { 0xce7de7a2, 43 },
    { 0x315de7a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1de7a6, 66 },
    { 0, 0 },
    { 0x31fde7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ade7b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ede7b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61de7bb, 33 },
    { 0x88ede7bc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdde7c0, 34 },
    { 0, 0 },
    { 0x20dde7c2, 40 },
    { 0xefbde7c3, 46 },
    { 0, 0 },
    { 0xaf2de7c5, 64 },
    { 0xd90de7c6, 99 },
    { 0x842de7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfde7ca, 33 },
    { 0x655de7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fde7cf, 95 },
    { 0, 0 },
    { 0xdc0de7d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694de7dd, 98 },
    { 0xe37de7de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8de7e8, 95 },
    { 0, 0 },
    { 0x8b2de7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7de7f1, 91 },
    { 0, 0 },
    { 0x8cde7f3, 43 },
    { 0x7c9de7f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20de7f7, 68 },
    { 0xe43de7f8, 55 },
    { 0x853de7f8, 61 },
    { 0, 0 },
    { 0x91fde7fb, 23 },
    { 0x732de7fb, 66 },
    { 0xb0ede7fc, 33 },
    { 0xfeede7fc, 23 },
    { 0x9b8de7ff, 9 },
    { 0xf62de7fe, 90 },
    { 0x34ede801, 91 },
    { 0xcbde802, 90 },
    { 0x542de803, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0de80a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2de80f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707de819, 96 },
    { 0, 0 },
    { 0xd0dde81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8de81f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdede828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x825de82b, 23 },
    { 0, 0 },
    { 0x6d9de82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9de836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ede83a, 52 },
    { 0xfa8de83a, 34 },
    { 0xb79de83c, 55 },
    { 0x8bfde83d, 66 },
    { 0x199de83d, 99 },
    { 0x6a8de83f, 42 },
    { 0xbde840, 76 },
    { 0, 0 },
    { 0x1a0de842, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ade847, 23 },
    { 0xfbbde848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ade84b, 33 },
    { 0x7e2de84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804de852, 61 },
    { 0, 0 },
    { 0x124de854, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcde85c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9de860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1de865, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5de869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de86d, 95 },
    { 0x4b2de86e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade872, 95 },
    { 0xb49de873, 61 },
    { 0xc14de873, 34 },
    { 0, 0 },
    { 0x97ede876, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebde882, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcde88a, 95 },
    { 0x98fde88a, 23 },
    { 0x100de88a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37de893, 90 },
    { 0xf50de893, 33 },
    { 0x9c7de895, 97 },
    { 0x986de893, 43 },
    { 0xaa8de897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12de89b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6de89e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ade8a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345de8ab, 98 },
    { 0, 0 },
    { 0x42ade8ad, 95 },
    { 0xe6bde8ae, 75 },
    { 0xb99de8af, 46 },
    { 0x217de8b0, 52 },
    { 0xfa6de8b1, 43 },
    { 0, 0 },
    { 0x138de8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfde8b9, 61 },
    { 0x674de8ba, 95 },
    { 0x96ade8bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ade8be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbede8c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2de8c7, 77 },
    { 0, 0 },
    { 0xdb2de8c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de8cc, 76 },
    { 0, 0 },
    { 0x58bde8ce, 85 },
    { 0x536de8ce, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfde8d3, 99 },
    { 0, 0 },
    { 0x26cde8d5, 98 },
    { 0, 0 },
    { 0x3abde8d7, 69 },
    { 0x4b2de8d7, 33 },
    { 0x57fde8d9, 55 },
    { 0x3de8da, 46 },
    { 0xa0fde8d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8de8df, 76 },
    { 0x3bfde8df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eede8e7, 76 },
    { 0xb60de8e7, 81 },
    { 0x207de8e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bde8ec, 33 },
    { 0x79dde8ed, 43 },
    { 0x6a1de8ec, 23 },
    { 0x47de8ef, 95 },
    { 0, 0 },
    { 0xb3ede8f1, 44 },
    { 0x50de8f2, 33 },
    { 0, 0 },
    { 0xc1dde8f4, 91 },
    { 0xcacde8f5, 69 },
    { 0x4c6de8f6, 23 },
    { 0x5d0de8f4, 98 },
    { 0, 0 },
    { 0x98dde8f9, 23 },
    { 0x6dade8f9, 95 },
    { 0, 0 },
    { 0x3cfde8fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ade902, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x809de905, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x696de908, 90 },
    { 0x334de909, 95 },
    { 0x759de90a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ede911, 76 },
    { 0x8b9de912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade917, 90 },
    { 0xa54de918, 55 },
    { 0x8a8de918, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412de91d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ade923, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527de927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29de92b, 100 },
    { 0x910de92c, 43 },
    { 0x280de92d, 61 },
    { 0x40bde92e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x327de931, 43 },
    { 0xa37de932, 98 },
    { 0x368de932, 61 },
    { 0xa87de934, 98 },
    { 0xaa0de934, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26de93b, 99 },
    { 0x273de93b, 34 },
    { 0, 0 },
    { 0x652de93e, 46 },
    { 0, 0 },
    { 0xff2de940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164de944, 95 },
    { 0x1aede945, 87 },
    { 0x8e2de945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ede94c, 23 },
    { 0x68ede94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56de951, 33 },
    { 0x20dde951, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bde957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeede95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ede961, 61 },
    { 0x24de962, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5de967, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889de96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abde972, 17 },
    { 0x938de973, 90 },
    { 0x653de974, 23 },
    { 0x92fde975, 95 },
    { 0xc18de976, 90 },
    { 0x190de976, 23 },
    { 0xfacde978, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eade97b, 90 },
    { 0, 0 },
    { 0x2edde97d, 61 },
    { 0xaa0de97e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256de982, 33 },
    { 0xc09de983, 76 },
    { 0x5c3de984, 90 },
    { 0x307de985, 34 },
    { 0, 0 },
    { 0x503de987, 98 },
    { 0, 0 },
    { 0xa28de989, 4 },
    { 0xe22de98a, 23 },
    { 0x295de98b, 23 },
    { 0xb57de98a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39de98f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de997, 61 },
    { 0xd5ade998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97de99b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28de99f, 95 },
    { 0x13cde99f, 61 },
    { 0x49ade9a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06de9a6, 76 },
    { 0, 0 },
    { 0x569de9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdde9ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcde9ae, 55 },
    { 0, 0 },
    { 0x590de9b0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ade9b3, 95 },
    { 0x410de9b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3de9b8, 95 },
    { 0xbe8de9b8, 99 },
    { 0, 0 },
    { 0xe4ede9bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13de9be, 33 },
    { 0xe3ade9be, 33 },
    { 0x388de9c0, 81 },
    { 0xfdade9c1, 33 },
    { 0, 0 },
    { 0x98de9c3, 95 },
    { 0x9f0de9c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccde9c7, 93 },
    { 0, 0 },
    { 0x555de9c9, 23 },
    { 0xa93de9ca, 62 },
    { 0xdf4de9c9, 76 },
    { 0x376de9cc, 23 },
    { 0, 0 },
    { 0x97dde9ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108de9d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1de9d7, 75 },
    { 0xa56de9d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145de9de, 98 },
    { 0, 0 },
    { 0x84dde9e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63de9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de9ec, 100 },
    { 0x1b9de9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2de9f1, 55 },
    { 0xf79de9f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9de9f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3de9f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bde9fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1adea00, 33 },
    { 0xdb5dea00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1dea04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6dea07, 78 },
    { 0xacdea07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3badea10, 33 },
    { 0, 0 },
    { 0xb2bdea12, 93 },
    { 0, 0 },
    { 0x525dea14, 23 },
    { 0x84ddea15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5dea1a, 98 },
    { 0xf64dea1b, 33 },
    { 0x7cbdea1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dea1f, 55 },
    { 0x32dea1f, 23 },
    { 0x414dea1f, 23 },
    { 0, 0 },
    { 0xa93dea23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931dea28, 87 },
    { 0, 0 },
    { 0xda1dea2a, 90 },
    { 0xbf6dea2a, 90 },
    { 0xff6dea2c, 96 },
    { 0x379dea2c, 98 },
    { 0x4d3dea2c, 100 },
    { 0x434dea2b, 96 },
    { 0x3aedea2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea33, 80 },
    { 0x601dea33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbdea39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940dea3d, 34 },
    { 0x649dea3d, 17 },
    { 0x7d8dea3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63edea49, 91 },
    { 0, 0 },
    { 0xdc0dea4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3dea51, 100 },
    { 0, 0 },
    { 0x312dea53, 93 },
    { 0, 0 },
    { 0x195dea55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fdea5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dea61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddea65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685dea68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dea6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3dea71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223dea76, 81 },
    { 0x718dea76, 33 },
    { 0x6b3dea78, 98 },
    { 0x9b8dea79, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80edea7f, 90 },
    { 0, 0 },
    { 0x431dea81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1dea84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338dea8a, 34 },
    { 0xa12dea8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9dea8f, 33 },
    { 0xb59dea8f, 82 },
    { 0x1ffdea91, 52 },
    { 0xd66dea92, 83 },
    { 0xb27dea91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3dea96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ddea9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8dea9d, 76 },
    { 0, 0 },
    { 0x75edea9f, 98 },
    { 0xbdea9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4deaa8, 90 },
    { 0xfcbdeaa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4deaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273deab3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7deab6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6deabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x863deabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7deac1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x462deac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8deac9, 98 },
    { 0x248deac9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89deacd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cdead8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedeadf, 23 },
    { 0xa49deadf, 67 },
    { 0xcdfdeadf, 17 },
    { 0x1dbdeae2, 55 },
    { 0, 0 },
    { 0xc73deae4, 23 },
    { 0xe3deae5, 78 },
    { 0xd8adeae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ddeafd, 46 },
    { 0, 0 },
    { 0x67cdeaff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ddeb04, 85 },
    { 0xe61deb04, 55 },
    { 0xd3adeb05, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506deb11, 33 },
    { 0x7adeb11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762deb16, 33 },
    { 0, 0 },
    { 0x182deb18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79deb1c, 95 },
    { 0x839deb1d, 98 },
    { 0, 0 },
    { 0xee2deb1f, 23 },
    { 0xf27deb20, 95 },
    { 0x120deb21, 87 },
    { 0, 0 },
    { 0x7dadeb23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bdeb26, 95 },
    { 0xfe0deb27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedeb2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdddeb32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdeb35, 61 },
    { 0, 0 },
    { 0x342deb37, 40 },
    { 0x99fdeb37, 23 },
    { 0x1d5deb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3deb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510deb40, 95 },
    { 0x19deb41, 90 },
    { 0, 0 },
    { 0x333deb43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bdeb47, 93 },
    { 0x5d9deb48, 49 },
    { 0xbffdeb49, 90 },
    { 0, 0 },
    { 0x82cdeb4b, 83 },
    { 0xb27deb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abdeb50, 33 },
    { 0, 0 },
    { 0x26bdeb52, 55 },
    { 0xa23deb52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1deb56, 76 },
    { 0xa56deb56, 66 },
    { 0x3b9deb58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579deb5e, 33 },
    { 0x639deb5f, 61 },
    { 0xc02deb60, 99 },
    { 0x4e7deb61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774deb67, 55 },
    { 0, 0 },
    { 0xf43deb69, 55 },
    { 0x18fdeb6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64deb6d, 99 },
    { 0, 0 },
    { 0x7afdeb6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98deb77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ddeb7b, 34 },
    { 0, 0 },
    { 0xbccdeb7d, 61 },
    { 0x520deb7e, 90 },
    { 0x4a9deb7d, 99 },
    { 0x677deb7d, 91 },
    { 0xf4adeb81, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394deb85, 33 },
    { 0x496deb86, 85 },
    { 0x180deb86, 98 },
    { 0xf58deb85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2deb91, 93 },
    { 0xbfddeb91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23deb9c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0deba2, 55 },
    { 0, 0 },
    { 0x1c2deba4, 23 },
    { 0x531deba4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabdeba8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1badebab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53adebaf, 33 },
    { 0x8ffdebaf, 100 },
    { 0x830debaf, 33 },
    { 0xc67debb2, 90 },
    { 0x60cdebb3, 98 },
    { 0xdb6debb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77debba, 88 },
    { 0xb70debba, 46 },
    { 0x9c6debbc, 75 },
    { 0, 0 },
    { 0x39ddebbe, 23 },
    { 0, 0 },
    { 0x5cfdebc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x293debc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcdebc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9debcc, 66 },
    { 0, 0 },
    { 0x77edebce, 61 },
    { 0x267debce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33debd2, 85 },
    { 0, 0 },
    { 0xb13debd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2adebda, 99 },
    { 0x83debdb, 40 },
    { 0, 0 },
    { 0x3b1debdd, 98 },
    { 0x5c5debde, 61 },
    { 0xdb0debdf, 90 },
    { 0x74cdebdd, 43 },
    { 0xe4bdebde, 33 },
    { 0xa2edebe0, 99 },
    { 0xe7ddebe0, 84 },
    { 0x790debe4, 90 },
    { 0x90bdebe5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1debe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69debec, 75 },
    { 0xdebec, 98 },
    { 0, 0 },
    { 0xb23debef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732debf6, 76 },
    { 0xea1debf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684debfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15debfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74adec06, 98 },
    { 0, 0 },
    { 0xb46dec08, 90 },
    { 0, 0 },
    { 0x287dec0a, 52 },
    { 0x3d5dec0b, 98 },
    { 0, 0 },
    { 0x826dec0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x258dec10, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5addec13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4edec1c, 23 },
    { 0xa74dec1c, 91 },
    { 0x1a0dec1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbdec21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bdec24, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efdec28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4dec30, 95 },
    { 0, 0 },
    { 0xb7adec32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8dec38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7dec3b, 75 },
    { 0, 0 },
    { 0x945dec3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7dec40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4dec44, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34dec47, 93 },
    { 0x5dec48, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0dec4e, 23 },
    { 0xa09dec4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcdec53, 90 },
    { 0xd4fdec54, 90 },
    { 0x40edec53, 87 },
    { 0x82ddec55, 90 },
    { 0x6d8dec54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322dec5d, 91 },
    { 0, 0 },
    { 0xe55dec5f, 94 },
    { 0xddddec60, 76 },
    { 0x719dec61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78dec65, 90 },
    { 0xa64dec65, 56 },
    { 0xa5cdec67, 90 },
    { 0x4f6dec67, 23 },
    { 0, 0 },
    { 0x2e0dec6a, 93 },
    { 0, 0 },
    { 0xad9dec6c, 61 },
    { 0x8bcdec6d, 33 },
    { 0xf9dec6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fddec72, 78 },
    { 0xad4dec72, 101 },
    { 0xd05dec72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdec79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x371dec7c, 33 },
    { 0x878dec7d, 76 },
    { 0xecdec7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffdec82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbedec87, 23 },
    { 0xa7fdec88, 93 },
    { 0xc7fdec87, 46 },
    { 0, 0 },
    { 0xd8dec8b, 90 },
    { 0, 0 },
    { 0x7d2dec8d, 23 },
    { 0x5bcdec8e, 81 },
    { 0x68ddec8f, 23 },
    { 0x7addec8f, 17 },
    { 0xd26dec91, 33 },
    { 0x2c1dec8f, 98 },
    { 0x4fcdec92, 23 },
    { 0xf43dec94, 91 },
    { 0x2e8dec94, 90 },
    { 0x477dec96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ddec99, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7dec9e, 91 },
    { 0x975dec9f, 77 },
    { 0x7fddeca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddeca3, 90 },
    { 0xc6ddeca4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fdecb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddecb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2decbd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7decc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x199deccb, 86 },
    { 0, 0 },
    { 0x67deccd, 33 },
    { 0, 0 },
    { 0xa9adeccf, 61 },
    { 0xb81decd0, 43 },
    { 0, 0 },
    { 0x32fdecd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7decd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660decdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edece2, 52 },
    { 0, 0 },
    { 0xf80dece4, 33 },
    { 0, 0 },
    { 0x684dece6, 23 },
    { 0x854dece7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ddecf5, 61 },
    { 0x9d8decf6, 61 },
    { 0xdb2decf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8decfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7ded00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ded03, 23 },
    { 0, 0 },
    { 0x842ded05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7ded0c, 55 },
    { 0x1f2ded0d, 85 },
    { 0xaacded0e, 66 },
    { 0xd0bded0f, 90 },
    { 0x1fdded0f, 90 },
    { 0x3bdded0e, 98 },
    { 0xfe0ded12, 46 },
    { 0x6ffded0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ded19, 76 },
    { 0x909ded1a, 67 },
    { 0xe27ded1a, 99 },
    { 0xdc9ded19, 77 },
    { 0xbbfded1d, 46 },
    { 0x1e8ded1d, 100 },
    { 0xd2bded1c, 34 },
    { 0x705ded1a, 95 },
    { 0xd2ded1d, 93 },
    { 0x423ded1d, 43 },
    { 0x310ded1b, 81 },
    { 0, 0 },
    { 0xcd1ded25, 43 },
    { 0x836ded26, 33 },
    { 0, 0 },
    { 0x633ded28, 76 },
    { 0xcb1ded29, 98 },
    { 0xf50ded2a, 95 },
    { 0x714ded2b, 90 },
    { 0x33fded2b, 55 },
    { 0xc83ded2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aded36, 98 },
    { 0x33fded37, 90 },
    { 0xa66ded38, 46 },
    { 0x564ded39, 100 },
    { 0xe7fded38, 95 },
    { 0x41dded3b, 76 },
    { 0x499ded3c, 95 },
    { 0x226ded3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ded44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cded49, 76 },
    { 0x758ded4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bded4e, 33 },
    { 0, 0 },
    { 0x8edded50, 33 },
    { 0xa57ded51, 52 },
    { 0, 0 },
    { 0x63dded53, 95 },
    { 0xe30ded53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ded57, 61 },
    { 0, 0 },
    { 0x75fded59, 95 },
    { 0xef5ded5a, 61 },
    { 0xdaaded5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3ded5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eded62, 23 },
    { 0xa43ded63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73ded66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cded6c, 95 },
    { 0xa05ded6c, 98 },
    { 0xbdfded6e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9ded72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ded79, 33 },
    { 0xda5ded79, 99 },
    { 0xe9fded79, 90 },
    { 0, 0 },
    { 0x908ded7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x786ded80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ded83, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ded87, 77 },
    { 0, 0 },
    { 0x53dded89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86ded8d, 23 },
    { 0x9e9ded8e, 60 },
    { 0xa9dded8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cded97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992ded9f, 66 },
    { 0x396deda0, 61 },
    { 0xfb8deda1, 33 },
    { 0xd3ededa2, 33 },
    { 0x12adeda1, 23 },
    { 0xa27deda1, 98 },
    { 0x994deda2, 61 },
    { 0x3b1deda6, 61 },
    { 0, 0 },
    { 0x9afdeda8, 87 },
    { 0, 0 },
    { 0x231dedaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84dedad, 91 },
    { 0x30bdedae, 55 },
    { 0, 0 },
    { 0x420dedb0, 90 },
    { 0x4d6dedb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944dedb5, 61 },
    { 0xcf0dedb6, 61 },
    { 0, 0 },
    { 0x49ddedb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6dedbd, 61 },
    { 0x8d0dedbe, 90 },
    { 0, 0 },
    { 0x2c0dedc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3dedc4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884dedcb, 23 },
    { 0xe53dedcc, 90 },
    { 0xc25dedcc, 33 },
    { 0x99ededcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83dedd1, 55 },
    { 0x706dedd2, 95 },
    { 0, 0 },
    { 0xa1dedd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ededd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bededdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38adeddf, 76 },
    { 0, 0 },
    { 0xf7cdede1, 61 },
    { 0xf2ddede2, 33 },
    { 0, 0 },
    { 0xb3bdede4, 33 },
    { 0x843dede5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727dede9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97dedef, 33 },
    { 0x17cdedef, 33 },
    { 0, 0 },
    { 0xf68dedf2, 84 },
    { 0xe1dedf3, 55 },
    { 0x823dedf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcdedf8, 98 },
    { 0x6addedf9, 33 },
    { 0xb8fdedfa, 90 },
    { 0xc30dedfb, 46 },
    { 0x969dedfb, 56 },
    { 0xf0bdedfa, 98 },
    { 0xdebdedfe, 75 },
    { 0, 0 },
    { 0xcf0dee00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2dee04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cedee0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0dee16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2dee19, 61 },
    { 0xdafdee19, 98 },
    { 0x6eddee1b, 33 },
    { 0x3bdee19, 99 },
    { 0, 0 },
    { 0xc6ddee1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782dee22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24dee25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5dee2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3dee2f, 65 },
    { 0xf08dee30, 33 },
    { 0, 0 },
    { 0xd7adee32, 59 },
    { 0xddbdee33, 23 },
    { 0xb56dee33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bdee37, 23 },
    { 0, 0 },
    { 0x39ddee39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cdee3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dee41, 91 },
    { 0xadedee42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245dee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dee4e, 45 },
    { 0, 0 },
    { 0x71ddee50, 34 },
    { 0x51ddee51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664dee55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09dee5c, 99 },
    { 0x3e5dee5d, 90 },
    { 0, 0 },
    { 0xd61dee5f, 95 },
    { 0xba1dee60, 91 },
    { 0xe2bdee61, 90 },
    { 0x315dee5f, 76 },
    { 0, 0 },
    { 0x832dee64, 23 },
    { 0xc6fdee65, 90 },
    { 0x80bdee66, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adee6a, 100 },
    { 0, 0 },
    { 0xefedee6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaadee74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8dee77, 43 },
    { 0x8a8dee77, 34 },
    { 0, 0 },
    { 0xbcddee7a, 61 },
    { 0, 0 },
    { 0x1bddee7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630dee87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608dee8c, 68 },
    { 0, 0 },
    { 0xdddee8e, 95 },
    { 0, 0 },
    { 0x80edee90, 23 },
    { 0x21edee91, 17 },
    { 0x445dee92, 43 },
    { 0, 0 },
    { 0xe19dee94, 76 },
    { 0, 0 },
    { 0x5a0dee96, 61 },
    { 0xefadee97, 78 },
    { 0xeddee96, 61 },
    { 0xcbbdee99, 61 },
    { 0, 0 },
    { 0x47adee9b, 98 },
    { 0x30adee9b, 46 },
    { 0x4aedee9c, 34 },
    { 0, 0 },
    { 0x514dee9f, 34 },
    { 0x7bfdeea0, 33 },
    { 0, 0 },
    { 0x4c4deea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bdeea5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449deeac, 67 },
    { 0, 0 },
    { 0xa70deeae, 98 },
    { 0x470deeae, 34 },
    { 0xad4deeae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86deeb5, 93 },
    { 0xb5adeeb6, 34 },
    { 0xfa2deeb7, 91 },
    { 0, 0 },
    { 0x245deeb9, 95 },
    { 0, 0 },
    { 0x5bddeebb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66deebe, 61 },
    { 0, 0 },
    { 0x221deec0, 67 },
    { 0, 0 },
    { 0x3d5deec2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ddeec7, 75 },
    { 0x395deec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdeecb, 33 },
    { 0xe21deecb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16deed0, 90 },
    { 0xba9deed1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5deed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833deeda, 61 },
    { 0xf39deedb, 43 },
    { 0, 0 },
    { 0x98fdeedd, 36 },
    { 0x69ddeede, 23 },
    { 0x1bddeede, 55 },
    { 0xcebdeede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddddeee6, 52 },
    { 0, 0 },
    { 0xfa1deee8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7deeeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2deef2, 33 },
    { 0, 0 },
    { 0xd7bdeef4, 76 },
    { 0, 0 },
    { 0x8d9deef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddeefa, 91 },
    { 0, 0 },
    { 0x19adeefc, 23 },
    { 0x470deefc, 95 },
    { 0x1f1deefe, 17 },
    { 0xe2bdeeff, 100 },
    { 0xeb5def00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697def07, 23 },
    { 0x323def08, 88 },
    { 0xd8fdef09, 33 },
    { 0, 0 },
    { 0x5afdef0b, 55 },
    { 0xffbdef0b, 95 },
    { 0x45fdef0d, 98 },
    { 0x759def0e, 81 },
    { 0x3fbdef0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11def1b, 33 },
    { 0x523def1c, 77 },
    { 0, 0 },
    { 0x2b8def1e, 77 },
    { 0x41adef1f, 87 },
    { 0x845def20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88adef24, 65 },
    { 0x5b7def25, 33 },
    { 0, 0 },
    { 0x5aadef27, 43 },
    { 0xbcfdef28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467def2d, 88 },
    { 0xb81def2e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3def31, 76 },
    { 0x3d7def32, 90 },
    { 0x3bdef33, 95 },
    { 0xde9def34, 98 },
    { 0x6e8def31, 33 },
    { 0, 0 },
    { 0xd1bdef37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbdef43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71adef49, 69 },
    { 0, 0 },
    { 0xd76def4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6def4f, 61 },
    { 0, 0 },
    { 0x73edef51, 23 },
    { 0xe38def51, 98 },
    { 0x437def53, 76 },
    { 0x6b4def52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fdef57, 90 },
    { 0, 0 },
    { 0x485def59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcdef5e, 60 },
    { 0, 0 },
    { 0x9a7def60, 65 },
    { 0x72adef61, 33 },
    { 0xb84def62, 100 },
    { 0x588def63, 55 },
    { 0, 0 },
    { 0xfa1def65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eadef69, 40 },
    { 0x3c3def6a, 23 },
    { 0xc80def6b, 90 },
    { 0x71cdef6b, 80 },
    { 0x50ddef6a, 75 },
    { 0xda7def6c, 90 },
    { 0x77fdef6f, 55 },
    { 0x4f7def6f, 43 },
    { 0x49ddef71, 43 },
    { 0x8b7def72, 76 },
    { 0xdc4def73, 61 },
    { 0xd55def73, 98 },
    { 0x5bddef75, 68 },
    { 0xa01def76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x962def79, 33 },
    { 0x1bcdef79, 90 },
    { 0x2e7def7b, 95 },
    { 0, 0 },
    { 0xf35def7d, 23 },
    { 0, 0 },
    { 0xe36def7f, 38 },
    { 0, 0 },
    { 0x7c9def81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622def84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddef8a, 90 },
    { 0x5b9def8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26adef92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8def96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafdef9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bdefa1, 98 },
    { 0x73edefa2, 95 },
    { 0, 0 },
    { 0x303defa4, 33 },
    { 0x930defa5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdefa9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519defad, 61 },
    { 0, 0 },
    { 0x98edefaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35defb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x245defb5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x10defb8, 66 },
    { 0, 0 },
    { 0x61defba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945defc1, 33 },
    { 0x433defc2, 52 },
    { 0xc8cdefc2, 91 },
    { 0x291defc3, 66 },
    { 0x513defc2, 55 },
    { 0x835defc6, 66 },
    { 0xd00defc6, 23 },
    { 0x637defc2, 76 },
    { 0xf4edefc9, 34 },
    { 0x80cdefca, 84 },
    { 0xb38defcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930defd3, 77 },
    { 0x698defd4, 46 },
    { 0, 0 },
    { 0x6dadefd6, 33 },
    { 0xc7edefd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x653defda, 96 },
    { 0x8d5defdb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadefe0, 100 },
    { 0, 0 },
    { 0x54defe2, 98 },
    { 0x9c1defe3, 33 },
    { 0xd00defe3, 40 },
    { 0, 0 },
    { 0x4badefe6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcdefe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365deff4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x529deff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62deffe, 76 },
    { 0xa35defff, 95 },
    { 0, 0 },
    { 0x79df001, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831df007, 97 },
    { 0xf61df008, 90 },
    { 0x3d0df009, 61 },
    { 0xe56df00a, 90 },
    { 0, 0 },
    { 0x7e2df00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896df023, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdf02a, 32 },
    { 0xe99df02b, 100 },
    { 0, 0 },
    { 0x3dcdf02d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258df032, 98 },
    { 0x20ddf033, 100 },
    { 0xe4edf034, 66 },
    { 0, 0 },
    { 0x7c8df036, 33 },
    { 0x82ddf037, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02df03c, 95 },
    { 0x83edf03c, 19 },
    { 0xe07df03e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55edf041, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6df045, 23 },
    { 0x9edf046, 61 },
    { 0x48ddf046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6df051, 55 },
    { 0xc8edf052, 17 },
    { 0xe8cdf053, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cdf058, 61 },
    { 0x1bdf058, 95 },
    { 0, 0 },
    { 0xd11df05b, 91 },
    { 0xba9df05b, 69 },
    { 0x14bdf05d, 68 },
    { 0xeabdf05c, 78 },
    { 0xd99df05e, 47 },
    { 0x5b8df060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98edf067, 61 },
    { 0x2d3df068, 76 },
    { 0x1aadf069, 23 },
    { 0x80cdf067, 33 },
    { 0xf0cdf069, 61 },
    { 0x1c1df06a, 77 },
    { 0, 0 },
    { 0x1c5df06e, 52 },
    { 0, 0 },
    { 0xdebdf070, 95 },
    { 0, 0 },
    { 0xebddf072, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x852df075, 98 },
    { 0xeadf076, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2df07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58df07e, 76 },
    { 0x1aadf07f, 95 },
    { 0x8b5df07f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840df084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8df089, 90 },
    { 0x2b7df089, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1df08f, 85 },
    { 0, 0 },
    { 0xf56df091, 23 },
    { 0x2a2df092, 91 },
    { 0xf30df093, 33 },
    { 0x3b9df094, 90 },
    { 0x7d5df095, 90 },
    { 0xaa8df094, 90 },
    { 0, 0 },
    { 0x50df098, 90 },
    { 0, 0 },
    { 0x2b8df09a, 23 },
    { 0, 0 },
    { 0x263df09c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7df09f, 46 },
    { 0x4aadf09f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810df0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9df0ab, 61 },
    { 0x7e8df0ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3df0b2, 80 },
    { 0xcfddf0b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bdf0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29df0b9, 23 },
    { 0, 0 },
    { 0x3c6df0bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2df0c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acdf0ca, 23 },
    { 0xcc3df0cb, 100 },
    { 0, 0 },
    { 0x646df0cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9df0d5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbddf0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8df0de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x808df0e1, 98 },
    { 0x2fadf0e2, 17 },
    { 0xae0df0e2, 33 },
    { 0xbaadf0e4, 93 },
    { 0x1fdf0e4, 61 },
    { 0xa4ddf0e6, 78 },
    { 0x1f1df0e6, 93 },
    { 0x84edf0e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abdf0ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x620df0ef, 34 },
    { 0x246df0f0, 43 },
    { 0x478df0f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cdf0f5, 42 },
    { 0x208df0f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5df0fe, 77 },
    { 0xf21df0fe, 66 },
    { 0xd16df100, 78 },
    { 0x310df100, 61 },
    { 0x733df102, 55 },
    { 0, 0 },
    { 0x7e6df104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4df10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcadf110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdf114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695df11b, 46 },
    { 0, 0 },
    { 0xb81df11d, 76 },
    { 0x2c6df11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01df122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x182df125, 77 },
    { 0, 0 },
    { 0x99cdf127, 44 },
    { 0xf31df128, 55 },
    { 0x34fdf129, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45adf12d, 95 },
    { 0x89fdf12e, 23 },
    { 0, 0 },
    { 0xfe2df130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x384df133, 46 },
    { 0x15ddf133, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530df13c, 61 },
    { 0x933df13d, 61 },
    { 0x1dbdf13e, 98 },
    { 0xe39df13f, 34 },
    { 0x7d3df13d, 34 },
    { 0xa48df141, 34 },
    { 0x5cedf13f, 90 },
    { 0x86cdf143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdf149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5df156, 23 },
    { 0xad8df157, 78 },
    { 0xa9fdf157, 98 },
    { 0, 0 },
    { 0xbe9df15a, 93 },
    { 0x565df15b, 76 },
    { 0xbb5df15a, 95 },
    { 0xbfbdf15d, 33 },
    { 0xe8adf15d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cdf161, 23 },
    { 0x3c6df162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425df166, 90 },
    { 0, 0 },
    { 0x343df168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fdf16b, 61 },
    { 0, 0 },
    { 0x23fdf16d, 100 },
    { 0, 0 },
    { 0x11fdf16f, 46 },
    { 0, 0 },
    { 0x323df171, 95 },
    { 0, 0 },
    { 0x385df173, 90 },
    { 0, 0 },
    { 0x8f5df175, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801df17e, 33 },
    { 0xc59df17f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832df184, 23 },
    { 0x7cedf185, 33 },
    { 0, 0 },
    { 0xfe0df187, 99 },
    { 0xb90df188, 95 },
    { 0x1d1df189, 90 },
    { 0, 0 },
    { 0x2efdf18b, 91 },
    { 0, 0 },
    { 0xedfdf18d, 23 },
    { 0, 0 },
    { 0xe07df18f, 93 },
    { 0, 0 },
    { 0x4a9df191, 33 },
    { 0x8a1df192, 17 },
    { 0, 0 },
    { 0xa81df194, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18df199, 98 },
    { 0x231df19a, 61 },
    { 0, 0 },
    { 0xcfbdf19c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787df19f, 90 },
    { 0x2aedf1a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4df1a4, 81 },
    { 0, 0 },
    { 0x78cdf1a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7df1aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bdf1ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdf1b1, 55 },
    { 0x479df1b2, 95 },
    { 0x758df1b1, 90 },
    { 0, 0 },
    { 0xf99df1b5, 61 },
    { 0xc82df1b6, 33 },
    { 0x8dddf1b7, 90 },
    { 0xe43df1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadf1bb, 98 },
    { 0x6badf1bc, 34 },
    { 0xdb6df1bd, 34 },
    { 0, 0 },
    { 0x2e5df1bf, 33 },
    { 0x2cfdf1c0, 66 },
    { 0x41adf1bf, 33 },
    { 0xfeadf1c2, 90 },
    { 0xcfcdf1bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddf1c6, 77 },
    { 0x363df1c7, 23 },
    { 0, 0 },
    { 0x405df1c9, 95 },
    { 0xa7fdf1c9, 93 },
    { 0x8d6df1c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1df1ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6df1d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08df1d6, 33 },
    { 0, 0 },
    { 0xa65df1d8, 77 },
    { 0, 0 },
    { 0xd09df1da, 90 },
    { 0x9adf1db, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffdf1df, 55 },
    { 0x4f0df1df, 23 },
    { 0x7dfdf1e1, 53 },
    { 0xccfdf1df, 61 },
    { 0x665df1e3, 95 },
    { 0xf8bdf1e4, 61 },
    { 0xc0fdf1e5, 90 },
    { 0xcc6df1e6, 55 },
    { 0x697df1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5df1ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4df1f5, 87 },
    { 0x786df1f6, 99 },
    { 0x21edf1f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4df1fd, 98 },
    { 0, 0 },
    { 0x7b0df1ff, 100 },
    { 0, 0 },
    { 0x25adf201, 90 },
    { 0x8f6df202, 90 },
    { 0xf84df202, 76 },
    { 0x41adf204, 43 },
    { 0x1c3df205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329df20a, 91 },
    { 0xfc4df20b, 98 },
    { 0, 0 },
    { 0xaafdf20d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2df210, 43 },
    { 0xc4adf211, 56 },
    { 0x3a5df211, 61 },
    { 0, 0 },
    { 0x2ebdf214, 95 },
    { 0, 0 },
    { 0x20edf216, 76 },
    { 0xaeddf217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x571df21a, 34 },
    { 0x465df21b, 95 },
    { 0x415df21c, 61 },
    { 0x24cdf21d, 95 },
    { 0x974df21e, 55 },
    { 0, 0 },
    { 0x62ddf220, 33 },
    { 0x79ddf221, 87 },
    { 0x604df222, 66 },
    { 0x395df223, 23 },
    { 0, 0 },
    { 0x118df225, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7df22a, 80 },
    { 0x8aedf22a, 23 },
    { 0x6e7df22c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afdf231, 33 },
    { 0xfa8df232, 100 },
    { 0xe69df233, 61 },
    { 0x873df231, 23 },
    { 0xca8df234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34df239, 33 },
    { 0x98adf23a, 85 },
    { 0xa4adf239, 95 },
    { 0x803df23a, 95 },
    { 0, 0 },
    { 0xf86df23e, 93 },
    { 0xb30df23f, 95 },
    { 0x559df23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccdf243, 43 },
    { 0x290df244, 66 },
    { 0x453df243, 85 },
    { 0x160df244, 95 },
    { 0, 0 },
    { 0x151df248, 75 },
    { 0xac1df249, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32df24d, 33 },
    { 0xe04df24d, 95 },
    { 0x681df24f, 33 },
    { 0x2dddf24e, 33 },
    { 0xd6df251, 55 },
    { 0xb65df252, 19 },
    { 0xaa8df253, 84 },
    { 0x76adf24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45df25a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x573df25d, 90 },
    { 0x51cdf25d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0df261, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df264, 17 },
    { 0xeffdf265, 43 },
    { 0x6edf264, 33 },
    { 0, 0 },
    { 0xcb1df268, 40 },
    { 0x286df268, 99 },
    { 0x550df26a, 90 },
    { 0x6b1df26b, 33 },
    { 0xa34df26b, 61 },
    { 0, 0 },
    { 0x43ddf26e, 90 },
    { 0xbc5df26f, 98 },
    { 0xb5df270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1df277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf27e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fdf282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdf288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbdf28b, 61 },
    { 0, 0 },
    { 0x46bdf28d, 83 },
    { 0, 0 },
    { 0x20cdf28f, 99 },
    { 0, 0 },
    { 0x2e7df291, 76 },
    { 0x583df292, 52 },
    { 0xbbddf293, 95 },
    { 0, 0 },
    { 0x90cdf295, 63 },
    { 0, 0 },
    { 0x3f4df297, 98 },
    { 0x782df298, 81 },
    { 0x76bdf298, 33 },
    { 0x267df29a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4df29e, 66 },
    { 0x2dedf29e, 78 },
    { 0xef9df29f, 90 },
    { 0, 0 },
    { 0xe05df2a2, 61 },
    { 0x21adf2a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3df2a9, 55 },
    { 0x3a6df2aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95df2ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1edf2b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cdf2b7, 76 },
    { 0x99edf2b8, 55 },
    { 0x6f0df2b9, 23 },
    { 0, 0 },
    { 0xd6bdf2bb, 23 },
    { 0, 0 },
    { 0x870df2bd, 61 },
    { 0x358df2be, 34 },
    { 0, 0 },
    { 0x14edf2c0, 33 },
    { 0x800df2c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4df2c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cdf2d1, 98 },
    { 0x5ecdf2d1, 23 },
    { 0x83df2d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfadf2d6, 33 },
    { 0x952df2d7, 84 },
    { 0x7abdf2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadf2db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391df2df, 90 },
    { 0, 0 },
    { 0xb0df2e1, 46 },
    { 0x6d9df2e1, 90 },
    { 0x6f8df2e3, 91 },
    { 0xad2df2e4, 43 },
    { 0xf5bdf2e5, 91 },
    { 0x753df2e6, 91 },
    { 0, 0 },
    { 0x8f7df2e8, 98 },
    { 0xc66df2e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x708df2ec, 61 },
    { 0, 0 },
    { 0xd33df2ee, 95 },
    { 0, 0 },
    { 0x803df2f0, 95 },
    { 0xe9fdf2f1, 90 },
    { 0, 0 },
    { 0x74ddf2f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1df2f9, 33 },
    { 0, 0 },
    { 0x621df2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8edf301, 61 },
    { 0x89ddf301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eddf305, 23 },
    { 0xc7adf306, 76 },
    { 0, 0 },
    { 0xce3df308, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0df30b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edf30f, 76 },
    { 0x7aedf310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bdf313, 34 },
    { 0x4d0df314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1df317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ddf31a, 90 },
    { 0x863df31a, 90 },
    { 0xffcdf31b, 33 },
    { 0x520df31d, 90 },
    { 0x731df31a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cdf327, 98 },
    { 0, 0 },
    { 0xf87df329, 87 },
    { 0xe0bdf32a, 61 },
    { 0xd29df32b, 33 },
    { 0x897df32c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fdf32f, 77 },
    { 0x15df330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53df333, 76 },
    { 0, 0 },
    { 0xa31df335, 33 },
    { 0xa8df336, 33 },
    { 0, 0 },
    { 0xe62df338, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36df33b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0df340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42df343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cdf346, 55 },
    { 0x5e0df346, 61 },
    { 0x6f8df348, 33 },
    { 0x129df347, 95 },
    { 0, 0 },
    { 0x387df34b, 95 },
    { 0, 0 },
    { 0x4d8df34d, 46 },
    { 0, 0 },
    { 0xf40df34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ddf358, 52 },
    { 0, 0 },
    { 0x133df35a, 90 },
    { 0x709df35a, 33 },
    { 0xe0ddf35b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343df360, 90 },
    { 0x9fddf361, 33 },
    { 0, 0 },
    { 0x21ddf363, 55 },
    { 0x8a1df364, 98 },
    { 0, 0 },
    { 0xdb0df366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332df36a, 87 },
    { 0, 0 },
    { 0x6e4df36c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadf370, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240df374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19df377, 75 },
    { 0xaedf378, 98 },
    { 0, 0 },
    { 0x8b1df37a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53df37d, 95 },
    { 0, 0 },
    { 0xf28df37f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf382, 81 },
    { 0xce1df382, 90 },
    { 0x3bddf384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1df389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1df38d, 77 },
    { 0, 0 },
    { 0xb3cdf38f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943df393, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedf39a, 33 },
    { 0x610df39a, 84 },
    { 0x9e3df39c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8df3a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10df3a9, 61 },
    { 0, 0 },
    { 0x712df3ab, 90 },
    { 0x811df3ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bdf3b1, 65 },
    { 0xcb1df3b2, 33 },
    { 0xe26df3b2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6df3b6, 97 },
    { 0, 0 },
    { 0x8a0df3b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdf3bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdf3c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08df3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2df3cb, 90 },
    { 0x345df3cb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208df3d1, 61 },
    { 0xc52df3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806df3d6, 88 },
    { 0xa8edf3d7, 95 },
    { 0xe14df3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51df3dd, 90 },
    { 0, 0 },
    { 0x26df3df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35df3e3, 46 },
    { 0, 0 },
    { 0xd9df3e5, 93 },
    { 0xa13df3e5, 33 },
    { 0x929df3e7, 55 },
    { 0xef3df3e6, 90 },
    { 0x895df3e9, 69 },
    { 0x6a4df3ea, 95 },
    { 0, 0 },
    { 0xc44df3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1adf3ef, 33 },
    { 0, 0 },
    { 0xdc9df3f1, 33 },
    { 0x5dadf3f2, 50 },
    { 0x97fdf3f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49df3f6, 95 },
    { 0xcfadf3f7, 55 },
    { 0x567df3f8, 77 },
    { 0x993df3f8, 61 },
    { 0x64bdf3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fdf405, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88df40d, 99 },
    { 0, 0 },
    { 0x2b3df40f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56df412, 84 },
    { 0, 0 },
    { 0xf33df414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33df418, 99 },
    { 0xe18df418, 76 },
    { 0xdffdf419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fdf425, 93 },
    { 0, 0 },
    { 0xf82df427, 23 },
    { 0x48df428, 26 },
    { 0x68adf428, 61 },
    { 0x2cddf428, 90 },
    { 0, 0 },
    { 0xc23df42c, 98 },
    { 0, 0 },
    { 0x85ddf42e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7edf432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58edf435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ddf43c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63df440, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdedf443, 65 },
    { 0x5d4df443, 55 },
    { 0x622df445, 91 },
    { 0x7b1df446, 23 },
    { 0x5aadf447, 87 },
    { 0x9a3df443, 90 },
    { 0xa2cdf447, 98 },
    { 0x7e8df44a, 40 },
    { 0x9ccdf44a, 66 },
    { 0x96cdf44c, 33 },
    { 0xc3cdf44c, 76 },
    { 0x992df44d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0df451, 84 },
    { 0xdfdf451, 98 },
    { 0x544df452, 98 },
    { 0x2e3df451, 90 },
    { 0xe22df453, 55 },
    { 0x5ecdf454, 66 },
    { 0x8e2df452, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2df460, 98 },
    { 0, 0 },
    { 0x8a5df462, 99 },
    { 0, 0 },
    { 0x649df464, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdedf468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fdf46d, 98 },
    { 0x60df46e, 98 },
    { 0x8b4df46e, 61 },
    { 0xdecdf46f, 90 },
    { 0x1c8df46e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdf474, 95 },
    { 0xa6edf474, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ddf478, 17 },
    { 0, 0 },
    { 0x1c1df47a, 90 },
    { 0xd47df47b, 34 },
    { 0x166df47c, 78 },
    { 0x94edf47c, 61 },
    { 0x383df47d, 23 },
    { 0, 0 },
    { 0x45ddf480, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcdf488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28df48e, 68 },
    { 0x9ffdf48e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4df496, 95 },
    { 0x78ddf497, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fedf49b, 93 },
    { 0, 0 },
    { 0xfa5df49d, 76 },
    { 0, 0 },
    { 0x537df49f, 88 },
    { 0x288df4a0, 55 },
    { 0x6f5df4a0, 100 },
    { 0x4ccdf4a1, 61 },
    { 0x30edf4a0, 95 },
    { 0xa7ddf4a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6df4a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7df4b0, 91 },
    { 0x4f8df4b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addf4b5, 61 },
    { 0xe05df4b5, 52 },
    { 0xa8fdf4b7, 69 },
    { 0x5a2df4b6, 93 },
    { 0, 0 },
    { 0xc72df4ba, 23 },
    { 0x24df4bb, 65 },
    { 0xfd1df4bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bdf4bf, 90 },
    { 0x581df4bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4df4c9, 95 },
    { 0xcacdf4ca, 90 },
    { 0xd76df4cb, 91 },
    { 0, 0 },
    { 0xfdbdf4cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68df4d0, 49 },
    { 0x2e6df4d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bdf4d4, 76 },
    { 0x337df4d4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578df4d9, 33 },
    { 0x5ecdf4da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166df4e0, 78 },
    { 0x9cfdf4e0, 34 },
    { 0xc60df4e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x456df4e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x994df4e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcdf4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35edf4f2, 90 },
    { 0x3d2df4f2, 68 },
    { 0, 0 },
    { 0xf05df4f5, 100 },
    { 0, 0 },
    { 0xd6adf4f7, 95 },
    { 0, 0 },
    { 0xc3adf4f9, 88 },
    { 0xf9bdf4fa, 65 },
    { 0x3b6df4fa, 34 },
    { 0, 0 },
    { 0x1cdf4fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cadf500, 55 },
    { 0, 0 },
    { 0x3fddf502, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edf507, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fdf50c, 78 },
    { 0x51bdf50d, 91 },
    { 0x734df50e, 99 },
    { 0x122df50e, 61 },
    { 0xebdf50d, 36 },
    { 0x586df511, 90 },
    { 0xae3df50c, 61 },
    { 0x914df50c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26df516, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7df519, 91 },
    { 0x7d4df519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x196df51d, 99 },
    { 0, 0 },
    { 0xb8fdf51f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6df528, 52 },
    { 0xd6fdf529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1df52d, 90 },
    { 0xf25df52d, 76 },
    { 0xeb6df52f, 76 },
    { 0, 0 },
    { 0xcf4df531, 17 },
    { 0x764df531, 81 },
    { 0, 0 },
    { 0x619df534, 33 },
    { 0x213df535, 98 },
    { 0xa55df536, 33 },
    { 0, 0 },
    { 0xed9df538, 52 },
    { 0, 0 },
    { 0xdfbdf53a, 78 },
    { 0, 0 },
    { 0x652df53c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154df543, 84 },
    { 0x780df544, 91 },
    { 0xcedf545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2df54e, 84 },
    { 0, 0 },
    { 0x829df550, 98 },
    { 0x426df551, 61 },
    { 0x7f1df550, 33 },
    { 0x2c7df553, 98 },
    { 0xc9adf554, 98 },
    { 0xc19df554, 33 },
    { 0x79cdf556, 100 },
    { 0x10ddf553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfdf55c, 76 },
    { 0x6f4df55d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bdf564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42df569, 33 },
    { 0x82adf56a, 93 },
    { 0, 0 },
    { 0x79bdf56c, 33 },
    { 0x8dedf56d, 75 },
    { 0xf02df56c, 95 },
    { 0x27ddf56f, 33 },
    { 0x6c2df56f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bdf574, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345df578, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7adf57c, 90 },
    { 0xe97df57d, 33 },
    { 0x3f4df57d, 61 },
    { 0x281df57f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdf583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90df587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bdf58a, 90 },
    { 0, 0 },
    { 0x9bedf58c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5edf58f, 76 },
    { 0x2c1df58f, 46 },
    { 0xe24df591, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59adf594, 55 },
    { 0x8bcdf595, 76 },
    { 0x99df595, 95 },
    { 0xddedf597, 99 },
    { 0xee5df598, 76 },
    { 0x7d3df599, 90 },
    { 0xd74df59a, 61 },
    { 0x101df594, 61 },
    { 0x14edf59c, 33 },
    { 0x72ddf59d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282df5a0, 61 },
    { 0xbabdf5a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x365df5a4, 61 },
    { 0x841df5a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbdf5aa, 90 },
    { 0x1dfdf5aa, 33 },
    { 0x3f2df5ac, 95 },
    { 0x5bedf5aa, 23 },
    { 0, 0 },
    { 0xc58df5af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985df5b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8df5b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45df5ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fdf5c4, 99 },
    { 0x24df5c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628df5ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabdf5d4, 82 },
    { 0x6d4df5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eadf5d9, 88 },
    { 0x2cfdf5da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bdf5de, 55 },
    { 0, 0 },
    { 0x878df5e0, 90 },
    { 0x76bdf5e1, 46 },
    { 0x7dadf5e1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60edf5e8, 43 },
    { 0, 0 },
    { 0x781df5ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffdf5ed, 53 },
    { 0x30df5ee, 34 },
    { 0x9eadf5ef, 33 },
    { 0x5e9df5f0, 49 },
    { 0x791df5f0, 98 },
    { 0x1fadf5f2, 90 },
    { 0xfe9df5ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cdf5f6, 33 },
    { 0x821df5f6, 33 },
    { 0xbe4df5f8, 34 },
    { 0x65fdf5f9, 61 },
    { 0, 0 },
    { 0xac3df5fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x625df5fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x813df601, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcdf606, 46 },
    { 0, 0 },
    { 0x4c2df608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bdf60d, 68 },
    { 0x149df60e, 90 },
    { 0x4c7df60f, 36 },
    { 0xb0adf60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230df615, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334df619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdf61e, 66 },
    { 0xea5df61e, 34 },
    { 0xe74df61e, 61 },
    { 0x8f7df621, 33 },
    { 0xe67df622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebdf627, 95 },
    { 0, 0 },
    { 0xeafdf629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffdf62c, 33 },
    { 0x755df62c, 55 },
    { 0x858df62e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdf631, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4df638, 43 },
    { 0x6f2df639, 33 },
    { 0, 0 },
    { 0x1ccdf63b, 90 },
    { 0x387df63b, 76 },
    { 0x715df63b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748df641, 33 },
    { 0x222df642, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818df648, 23 },
    { 0x5df649, 78 },
    { 0, 0 },
    { 0x727df64b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825df64e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15df652, 33 },
    { 0x90adf652, 90 },
    { 0xc05df654, 90 },
    { 0, 0 },
    { 0xec9df656, 76 },
    { 0xbe0df657, 90 },
    { 0xcfcdf656, 46 },
    { 0, 0 },
    { 0x9bcdf65a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x658df65d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df663, 46 },
    { 0x7b8df664, 90 },
    { 0x328df665, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5df66a, 61 },
    { 0xeb3df66b, 69 },
    { 0xa3ddf66c, 76 },
    { 0xf87df66d, 61 },
    { 0x11df66e, 55 },
    { 0x8fadf66f, 23 },
    { 0xb2df66c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddf675, 55 },
    { 0x63edf676, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0adf67a, 17 },
    { 0x5d4df67b, 99 },
    { 0xb5ddf67b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0df680, 49 },
    { 0xd5edf681, 61 },
    { 0xbb7df682, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83df686, 81 },
    { 0xe7fdf686, 59 },
    { 0, 0 },
    { 0xbd4df689, 61 },
    { 0x1fcdf68a, 78 },
    { 0x2e9df68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8df691, 90 },
    { 0x5d5df692, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418df698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36df69d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fdf6a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15df6ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192df6b0, 75 },
    { 0x543df6b1, 23 },
    { 0x88ddf6b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1df6b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00df6bb, 32 },
    { 0, 0 },
    { 0xb37df6bd, 55 },
    { 0xd5df6be, 77 },
    { 0xe05df6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1df6c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ddf6c8, 33 },
    { 0xec7df6c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afdf6cd, 99 },
    { 0, 0 },
    { 0xa2edf6cf, 88 },
    { 0, 0 },
    { 0x718df6d1, 33 },
    { 0xa03df6d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcdf6d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x844df6dc, 80 },
    { 0, 0 },
    { 0x6f2df6de, 61 },
    { 0xfb1df6df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2df6e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78adf6ed, 90 },
    { 0xf8adf6ee, 88 },
    { 0, 0 },
    { 0xb19df6f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3df6f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fdf6f6, 61 },
    { 0x9fddf6f7, 95 },
    { 0, 0 },
    { 0x391df6f9, 100 },
    { 0xb72df6f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x132df6fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadf702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x818df705, 61 },
    { 0, 0 },
    { 0xa76df707, 87 },
    { 0x7e5df707, 90 },
    { 0xb4bdf709, 61 },
    { 0xfabdf70a, 90 },
    { 0x6c6df70a, 67 },
    { 0x855df70c, 23 },
    { 0xf06df708, 95 },
    { 0xd4fdf70b, 66 },
    { 0x2dddf709, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9df719, 93 },
    { 0, 0 },
    { 0xc1bdf71b, 33 },
    { 0xc87df71c, 95 },
    { 0x73edf71c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19df723, 23 },
    { 0x1eadf723, 76 },
    { 0xc22df725, 98 },
    { 0x1cadf725, 33 },
    { 0xafedf725, 61 },
    { 0, 0 },
    { 0x205df729, 80 },
    { 0x34df729, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2df730, 23 },
    { 0, 0 },
    { 0x831df732, 46 },
    { 0x228df733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fdf737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cedf73e, 91 },
    { 0x19df73e, 95 },
    { 0x17bdf740, 33 },
    { 0x138df73e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3df746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bdf74c, 61 },
    { 0, 0 },
    { 0xf4ddf74e, 90 },
    { 0xb0fdf74e, 90 },
    { 0, 0 },
    { 0x48edf751, 76 },
    { 0x96ddf752, 78 },
    { 0x575df751, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5df756, 91 },
    { 0xebbdf757, 90 },
    { 0x70df757, 82 },
    { 0x84ddf759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbdf75f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6df764, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cadf767, 98 },
    { 0x608df767, 34 },
    { 0x610df769, 99 },
    { 0xe8df768, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fdf774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cdf778, 55 },
    { 0xd94df779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3df77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2df781, 84 },
    { 0x876df782, 95 },
    { 0xa3df782, 23 },
    { 0xd62df781, 38 },
    { 0x886df781, 33 },
    { 0, 0 },
    { 0x392df787, 46 },
    { 0xc33df788, 34 },
    { 0xf5edf789, 90 },
    { 0x3f5df789, 33 },
    { 0x333df787, 91 },
    { 0, 0 },
    { 0x1e8df78d, 55 },
    { 0x4addf78e, 93 },
    { 0xa3bdf78f, 76 },
    { 0x571df790, 95 },
    { 0xd9bdf791, 98 },
    { 0xf0bdf78e, 90 },
    { 0x1f0df78f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7df797, 61 },
    { 0xeecdf797, 95 },
    { 0x3f5df798, 93 },
    { 0xb5fdf79a, 90 },
    { 0, 0 },
    { 0xe4bdf79c, 93 },
    { 0, 0 },
    { 0xeffdf79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fdf7a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8df7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71df7b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4df7b6, 90 },
    { 0, 0 },
    { 0x160df7b8, 81 },
    { 0x757df7b9, 33 },
    { 0, 0 },
    { 0xe0bdf7bb, 46 },
    { 0x9dfdf7bc, 61 },
    { 0xfe4df7bd, 95 },
    { 0xc58df7be, 33 },
    { 0x28adf7bf, 23 },
    { 0x942df7c0, 55 },
    { 0x42adf7bb, 61 },
    { 0x572df7c2, 78 },
    { 0x24bdf7c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732df7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fdf7ca, 90 },
    { 0x932df7cb, 80 },
    { 0xbbfdf7cc, 100 },
    { 0xecadf7cd, 33 },
    { 0x1bdf7ce, 61 },
    { 0x2dadf7cf, 64 },
    { 0x9f3df7d0, 93 },
    { 0x2c0df7d1, 67 },
    { 0xeb0df7d2, 98 },
    { 0x838df7d0, 61 },
    { 0xf26df7d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58df7d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fddf7da, 91 },
    { 0xbb0df7db, 100 },
    { 0xf17df7dc, 55 },
    { 0, 0 },
    { 0x8fadf7de, 23 },
    { 0x398df7de, 46 },
    { 0, 0 },
    { 0xfaedf7e1, 66 },
    { 0xb1edf7e2, 33 },
    { 0x542df7e3, 43 },
    { 0x66ddf7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4df7e8, 61 },
    { 0xadcdf7e9, 76 },
    { 0, 0 },
    { 0x88edf7eb, 75 },
    { 0xd24df7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576df7f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0df7f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0edf7f7, 46 },
    { 0, 0 },
    { 0xa00df7f9, 52 },
    { 0xa6edf7f9, 90 },
    { 0, 0 },
    { 0xa09df7fc, 61 },
    { 0x114df7fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687df801, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x855df804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3df80b, 95 },
    { 0xc96df80c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x726df80f, 95 },
    { 0x58cdf810, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849df815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cddf818, 67 },
    { 0x38df819, 61 },
    { 0x61df81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3edf820, 84 },
    { 0xf24df821, 88 },
    { 0xff7df821, 33 },
    { 0x1dedf820, 33 },
    { 0xc7df824, 90 },
    { 0x4d2df821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2df828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67adf82c, 33 },
    { 0x4b8df82d, 43 },
    { 0, 0 },
    { 0x9ecdf82f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645df835, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52df838, 43 },
    { 0, 0 },
    { 0xbcedf83a, 55 },
    { 0x750df83a, 23 },
    { 0x2cedf83c, 43 },
    { 0xf0fdf83b, 95 },
    { 0xed0df83e, 46 },
    { 0xb82df83f, 46 },
    { 0xe1edf83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2df843, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3df848, 91 },
    { 0x5dcdf849, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfdf84e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbdf851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x183df854, 95 },
    { 0x536df855, 33 },
    { 0x5cedf855, 33 },
    { 0xbfdf857, 77 },
    { 0x9bedf857, 98 },
    { 0x1d6df854, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96df85e, 90 },
    { 0, 0 },
    { 0xbdcdf860, 95 },
    { 0, 0 },
    { 0x662df862, 95 },
    { 0x21bdf862, 95 },
    { 0x746df864, 33 },
    { 0, 0 },
    { 0x8dcdf866, 78 },
    { 0, 0 },
    { 0xd42df868, 33 },
    { 0x4cdf868, 66 },
    { 0x33ddf86a, 76 },
    { 0xd4fdf86a, 33 },
    { 0xc02df86c, 76 },
    { 0x2e5df86a, 23 },
    { 0xde9df868, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8df872, 23 },
    { 0xd99df873, 23 },
    { 0x45edf873, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbdf879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683df87f, 43 },
    { 0x6b4df880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf883, 23 },
    { 0, 0 },
    { 0xbe3df885, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fdf890, 90 },
    { 0x633df891, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0df89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268df8a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2df8a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4df8aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4df8ae, 51 },
    { 0x261df8af, 84 },
    { 0x8ddf8ae, 90 },
    { 0x29cdf8b1, 33 },
    { 0x7cdf8b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102df8b6, 55 },
    { 0, 0 },
    { 0x7d7df8b8, 98 },
    { 0, 0 },
    { 0x507df8ba, 90 },
    { 0x2aadf8bb, 81 },
    { 0xf6adf8bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdf8c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778df8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2badf8ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6df8d4, 90 },
    { 0x372df8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aadf8d8, 33 },
    { 0xaf9df8d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322df8dd, 83 },
    { 0x841df8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cdf8e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5df8e4, 88 },
    { 0x130df8e4, 65 },
    { 0xc3cdf8e4, 99 },
    { 0xb53df8e7, 52 },
    { 0xf31df8e7, 23 },
    { 0x607df8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92df8f0, 61 },
    { 0xba7df8f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5df8f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51df8f7, 65 },
    { 0x8acdf8f7, 90 },
    { 0x79df8f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1df8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67df902, 69 },
    { 0, 0 },
    { 0xabddf904, 95 },
    { 0x349df905, 95 },
    { 0, 0 },
    { 0x6dcdf907, 90 },
    { 0xcd1df908, 90 },
    { 0, 0 },
    { 0x6bedf90a, 98 },
    { 0, 0 },
    { 0x900df90c, 98 },
    { 0x824df90d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df918, 33 },
    { 0, 0 },
    { 0x25df91a, 90 },
    { 0, 0 },
    { 0xf5edf91c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5df92d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fdf935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686df93e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3df943, 84 },
    { 0x7b8df944, 67 },
    { 0, 0 },
    { 0x8cbdf946, 78 },
    { 0, 0 },
    { 0xb86df948, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8df94b, 61 },
    { 0, 0 },
    { 0xa5edf94d, 100 },
    { 0, 0 },
    { 0x45bdf94f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15df955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdf95a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bdf95d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df963, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ddf966, 43 },
    { 0xd9adf966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428df96c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fdf96f, 52 },
    { 0x7afdf970, 68 },
    { 0x451df971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2df975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ddf97d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730df981, 33 },
    { 0x301df982, 90 },
    { 0xa2edf981, 33 },
    { 0x953df984, 33 },
    { 0x8b1df985, 61 },
    { 0xc63df985, 87 },
    { 0xa81df985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65df98b, 90 },
    { 0x289df98c, 35 },
    { 0xbeedf98d, 97 },
    { 0xe0adf98e, 23 },
    { 0xf7bdf98f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffdf999, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88df99e, 61 },
    { 0, 0 },
    { 0x3a8df9a0, 87 },
    { 0, 0 },
    { 0x52df9a2, 33 },
    { 0x28df9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x923df9a6, 61 },
    { 0xf70df9a7, 61 },
    { 0x1b9df9a7, 101 },
    { 0x760df9a6, 55 },
    { 0x3d7df9a7, 34 },
    { 0x8a6df9a7, 95 },
    { 0x827df9ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7df9b3, 76 },
    { 0, 0 },
    { 0x15bdf9b5, 98 },
    { 0x455df9b6, 23 },
    { 0xdf0df9b5, 100 },
    { 0xfd1df9b7, 99 },
    { 0, 0 },
    { 0x679df9ba, 90 },
    { 0x554df9ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3df9be, 61 },
    { 0xe33df9be, 23 },
    { 0xbb0df9bf, 76 },
    { 0x2f3df9c1, 88 },
    { 0x4c8df9c2, 98 },
    { 0x96cdf9c3, 95 },
    { 0xffcdf9c4, 43 },
    { 0x55adf9c4, 65 },
    { 0x4eddf9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30df9c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ddf9cf, 90 },
    { 0, 0 },
    { 0x793df9d1, 61 },
    { 0x6acdf9d1, 76 },
    { 0xfa7df9d3, 33 },
    { 0, 0 },
    { 0xbfdf9d5, 77 },
    { 0xc0ddf9d6, 95 },
    { 0xdfedf9d7, 52 },
    { 0x254df9d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6df9db, 61 },
    { 0, 0 },
    { 0x788df9dd, 98 },
    { 0x23fdf9de, 33 },
    { 0xc0df9dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455df9e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6df9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88df9f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bdf9f6, 23 },
    { 0, 0 },
    { 0x7a7df9f8, 33 },
    { 0xd8ddf9f8, 61 },
    { 0x295df9f8, 61 },
    { 0, 0 },
    { 0xeefdf9fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdfa00, 98 },
    { 0x256dfa01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9dfa05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fdfa08, 96 },
    { 0xf88dfa09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2dfa11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810dfa17, 88 },
    { 0xc03dfa18, 61 },
    { 0x979dfa18, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63dfa1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x430dfa20, 43 },
    { 0x1d0dfa20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ddfa24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199dfa28, 33 },
    { 0, 0 },
    { 0xdc5dfa2a, 90 },
    { 0x953dfa2a, 40 },
    { 0xc36dfa2c, 99 },
    { 0x8fdfa2c, 33 },
    { 0x117dfa2d, 95 },
    { 0x9acdfa2f, 87 },
    { 0x40ddfa2f, 34 },
    { 0xce4dfa2e, 17 },
    { 0x60cdfa2a, 33 },
    { 0, 0 },
    { 0x3fedfa34, 61 },
    { 0x5b5dfa35, 23 },
    { 0xbeadfa36, 95 },
    { 0x62cdfa37, 61 },
    { 0xc62dfa38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdfa41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801dfa4d, 65 },
    { 0x6cddfa4e, 99 },
    { 0x8fedfa4f, 98 },
    { 0x1ccdfa50, 87 },
    { 0x536dfa4d, 43 },
    { 0, 0 },
    { 0x9b2dfa53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132dfa59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8dfa5c, 52 },
    { 0, 0 },
    { 0xf06dfa5e, 85 },
    { 0, 0 },
    { 0x978dfa60, 33 },
    { 0x6b0dfa61, 93 },
    { 0x17adfa62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eddfa6b, 98 },
    { 0, 0 },
    { 0x29cdfa6d, 95 },
    { 0, 0 },
    { 0x3d3dfa6f, 85 },
    { 0xd7dfa70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x272dfa73, 91 },
    { 0x4efdfa74, 61 },
    { 0xc51dfa74, 40 },
    { 0xab5dfa73, 84 },
    { 0x172dfa77, 90 },
    { 0, 0 },
    { 0xe73dfa79, 43 },
    { 0x30edfa7a, 90 },
    { 0xf3fdfa7a, 98 },
    { 0, 0 },
    { 0xefadfa7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x203dfa80, 90 },
    { 0, 0 },
    { 0x9a9dfa82, 98 },
    { 0x151dfa82, 66 },
    { 0xd6fdfa84, 90 },
    { 0, 0 },
    { 0xda0dfa86, 98 },
    { 0x717dfa86, 90 },
    { 0x107dfa86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdfa8b, 23 },
    { 0x8e3dfa8b, 90 },
    { 0, 0 },
    { 0xd4bdfa8e, 55 },
    { 0xdbdfa8f, 87 },
    { 0xc0edfa8f, 61 },
    { 0xcecdfa8e, 99 },
    { 0, 0 },
    { 0x968dfa93, 53 },
    { 0, 0 },
    { 0xa46dfa95, 76 },
    { 0x574dfa96, 87 },
    { 0, 0 },
    { 0xd8fdfa98, 23 },
    { 0x34bdfa98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x221dfa9c, 23 },
    { 0x2a5dfa9d, 23 },
    { 0x698dfa9e, 75 },
    { 0xc8dfa9f, 23 },
    { 0xc3adfa9c, 61 },
    { 0x604dfaa1, 67 },
    { 0x59fdfa9f, 34 },
    { 0xa00dfaa3, 46 },
    { 0x2fcdfaa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcddfaa7, 23 },
    { 0, 0 },
    { 0x85cdfaa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99edfaac, 98 },
    { 0xd67dfaac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x306dfab0, 55 },
    { 0x3aedfab1, 43 },
    { 0, 0 },
    { 0xb08dfab3, 90 },
    { 0x7a0dfab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x814dfab7, 90 },
    { 0x32edfab8, 23 },
    { 0xb61dfab9, 84 },
    { 0xa3dfaba, 34 },
    { 0xc4fdfabb, 40 },
    { 0, 0 },
    { 0xddddfabd, 61 },
    { 0x5b8dfabe, 93 },
    { 0xfbedfabf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5dfac9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ddfad0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedfad5, 33 },
    { 0x4e9dfad5, 67 },
    { 0xa46dfad7, 100 },
    { 0, 0 },
    { 0xf01dfad9, 98 },
    { 0x716dfada, 61 },
    { 0x637dfadb, 61 },
    { 0xfc4dfadb, 95 },
    { 0x578dfadc, 92 },
    { 0xebcdfade, 33 },
    { 0x6aadfadf, 33 },
    { 0xcdfade, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecdfae6, 95 },
    { 0x64cdfae7, 99 },
    { 0xcf2dfae8, 75 },
    { 0x5dedfae9, 90 },
    { 0x8f8dfae9, 100 },
    { 0x9c0dfaeb, 98 },
    { 0x412dfae6, 33 },
    { 0x79cdfaed, 56 },
    { 0x21adfaed, 88 },
    { 0xa4edfaed, 95 },
    { 0x67dfaf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9dfaf5, 98 },
    { 0xbf3dfaf5, 69 },
    { 0, 0 },
    { 0xd0bdfaf8, 36 },
    { 0xb09dfaf9, 65 },
    { 0, 0 },
    { 0x120dfafb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21dfb02, 66 },
    { 0x273dfb03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311dfb0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5dfb12, 34 },
    { 0xab3dfb13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9dfb16, 33 },
    { 0, 0 },
    { 0x5fdfb18, 23 },
    { 0xe37dfb19, 90 },
    { 0x614dfb1a, 61 },
    { 0xe44dfb18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72dfb23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175dfb26, 66 },
    { 0x650dfb26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267dfb2b, 76 },
    { 0, 0 },
    { 0x7aedfb2d, 33 },
    { 0x40ddfb2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4dfb34, 46 },
    { 0x7abdfb34, 95 },
    { 0x99fdfb36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdfb39, 65 },
    { 0x34edfb3a, 90 },
    { 0x59cdfb3b, 95 },
    { 0x46cdfb3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebddfb40, 98 },
    { 0, 0 },
    { 0x587dfb42, 34 },
    { 0xa87dfb42, 33 },
    { 0xb29dfb44, 33 },
    { 0x899dfb45, 90 },
    { 0xadddfb43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84adfb4d, 91 },
    { 0xb08dfb4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50dfb52, 78 },
    { 0xd3fdfb53, 61 },
    { 0x5d0dfb53, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fdfb5a, 90 },
    { 0, 0 },
    { 0xb1ddfb5c, 33 },
    { 0x76ddfb5c, 43 },
    { 0xac8dfb5d, 40 },
    { 0xc5adfb5e, 61 },
    { 0x619dfb60, 69 },
    { 0, 0 },
    { 0x93fdfb62, 43 },
    { 0x159dfb63, 33 },
    { 0x810dfb63, 61 },
    { 0x233dfb65, 61 },
    { 0, 0 },
    { 0xe37dfb67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7adfb6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dfb73, 61 },
    { 0xdccdfb74, 76 },
    { 0xe15dfb74, 23 },
    { 0x2d1dfb76, 77 },
    { 0x1fddfb73, 61 },
    { 0xbc4dfb78, 43 },
    { 0x837dfb79, 84 },
    { 0xb73dfb73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfb84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcdfb87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dfb8a, 33 },
    { 0xc49dfb8b, 93 },
    { 0xd88dfb8a, 23 },
    { 0xd9edfb8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x601dfb90, 95 },
    { 0, 0 },
    { 0x527dfb92, 95 },
    { 0x72dfb93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8dfb96, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fdfb9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dfb9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cdfba1, 34 },
    { 0, 0 },
    { 0x5d2dfba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655dfba7, 91 },
    { 0xa5bdfba8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695dfbae, 46 },
    { 0x5a1dfbaf, 46 },
    { 0x909dfbb0, 43 },
    { 0xedadfbb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5dfbb9, 46 },
    { 0x69bdfbb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5dfbc0, 95 },
    { 0x8bdfbc1, 90 },
    { 0xd55dfbc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbadfbd3, 66 },
    { 0xaadfbd4, 76 },
    { 0xd96dfbd5, 95 },
    { 0xd97dfbd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6dfbdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875dfbe3, 55 },
    { 0, 0 },
    { 0xfc8dfbe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924dfbf0, 95 },
    { 0xe89dfbf0, 43 },
    { 0x362dfbf2, 98 },
    { 0, 0 },
    { 0x770dfbf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acdfbf7, 76 },
    { 0x22fdfbf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3dfbfb, 75 },
    { 0x752dfbfc, 53 },
    { 0x14dfbfc, 87 },
    { 0, 0 },
    { 0x1d5dfbff, 34 },
    { 0x2ddfc00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dfc03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cdfc0a, 52 },
    { 0xb31dfc0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81dfc13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858dfc18, 33 },
    { 0x586dfc18, 78 },
    { 0xd51dfc1a, 55 },
    { 0, 0 },
    { 0x2bfdfc1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dfc24, 90 },
    { 0, 0 },
    { 0x33fdfc26, 46 },
    { 0x9d5dfc27, 40 },
    { 0x61dfc28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74dfc2d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83dfc31, 95 },
    { 0, 0 },
    { 0xfe9dfc33, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdfc36, 91 },
    { 0x1a8dfc37, 61 },
    { 0xcd6dfc38, 95 },
    { 0x1eddfc39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291dfc40, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dfc43, 55 },
    { 0, 0 },
    { 0x17ddfc45, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dfc4a, 95 },
    { 0x1efdfc4b, 33 },
    { 0, 0 },
    { 0x64bdfc4d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6dfc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2dfc53, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24adfc59, 23 },
    { 0x749dfc59, 91 },
    { 0xe97dfc5a, 61 },
    { 0xe01dfc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fdfc61, 23 },
    { 0x1d8dfc62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dfc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37dfc69, 98 },
    { 0x34edfc6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaddfc71, 93 },
    { 0, 0 },
    { 0x961dfc73, 97 },
    { 0x4f0dfc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1adfc77, 95 },
    { 0x5b2dfc78, 61 },
    { 0xd00dfc79, 76 },
    { 0x5a1dfc78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477dfc7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03dfc82, 46 },
    { 0xa36dfc83, 61 },
    { 0x3a0dfc84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cdfc87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fdfc8a, 34 },
    { 0, 0 },
    { 0x765dfc8c, 98 },
    { 0xea3dfc8d, 61 },
    { 0, 0 },
    { 0x8d1dfc8f, 98 },
    { 0xccddfc90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3adfc93, 90 },
    { 0, 0 },
    { 0xa38dfc95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fdfca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8dfca9, 98 },
    { 0xad4dfcaa, 98 },
    { 0x368dfcab, 76 },
    { 0xc41dfcab, 40 },
    { 0xe75dfcab, 26 },
    { 0xfdddfcae, 97 },
    { 0x42cdfcaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddfcb9, 84 },
    { 0x212dfcb9, 90 },
    { 0x901dfcbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dfcbe, 61 },
    { 0, 0 },
    { 0x5e6dfcc0, 95 },
    { 0xccfdfcc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0dfcc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcedfcc7, 100 },
    { 0x3dcdfcc8, 33 },
    { 0x147dfcc9, 23 },
    { 0x910dfcca, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ddfcd0, 52 },
    { 0xb9cdfcd0, 84 },
    { 0x429dfcd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dfcd7, 61 },
    { 0x2f6dfcd8, 95 },
    { 0x6e5dfcd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4dfcdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50dfce6, 78 },
    { 0x897dfce7, 95 },
    { 0, 0 },
    { 0x235dfce9, 90 },
    { 0x20adfcea, 93 },
    { 0x92dfcea, 95 },
    { 0, 0 },
    { 0x615dfced, 90 },
    { 0x76bdfcee, 33 },
    { 0x134dfced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ddfcf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248dfcf7, 90 },
    { 0xf4edfcf8, 61 },
    { 0xb37dfcf7, 98 },
    { 0x5d0dfcf7, 34 },
    { 0xc72dfcfb, 95 },
    { 0x7bddfcfc, 33 },
    { 0, 0 },
    { 0xf7cdfcfe, 23 },
    { 0, 0 },
    { 0x6b1dfd00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46dfd09, 61 },
    { 0x5dedfd0a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777dfd10, 67 },
    { 0, 0 },
    { 0xad6dfd12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x930dfd15, 17 },
    { 0, 0 },
    { 0x80edfd17, 55 },
    { 0xf8adfd17, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dfd1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706dfd1f, 33 },
    { 0xa7cdfd20, 76 },
    { 0xcb8dfd21, 98 },
    { 0x915dfd22, 78 },
    { 0, 0 },
    { 0x89edfd24, 46 },
    { 0xfeddfd24, 42 },
    { 0xbaadfd26, 84 },
    { 0x860dfd26, 98 },
    { 0x23edfd27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfd2b, 43 },
    { 0xb5adfd2c, 84 },
    { 0xdd4dfd2d, 98 },
    { 0x9b8dfd2e, 46 },
    { 0, 0 },
    { 0x49ddfd30, 34 },
    { 0, 0 },
    { 0xfafdfd32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfd36, 33 },
    { 0, 0 },
    { 0x34adfd38, 61 },
    { 0, 0 },
    { 0x879dfd3a, 23 },
    { 0xfeadfd3b, 61 },
    { 0x4b3dfd3c, 91 },
    { 0xfb1dfd3a, 33 },
    { 0xf66dfd3e, 55 },
    { 0x9b5dfd3a, 95 },
    { 0x17edfd3b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fddfd44, 33 },
    { 0x3a9dfd45, 100 },
    { 0xd03dfd46, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedfd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0dfd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561dfd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594dfd5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdfd60, 84 },
    { 0x395dfd61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486dfd65, 33 },
    { 0x931dfd65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47adfd69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dddfd75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddfd7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dfd7d, 61 },
    { 0, 0 },
    { 0x344dfd7f, 95 },
    { 0, 0 },
    { 0x705dfd81, 95 },
    { 0x76edfd82, 17 },
    { 0x2a7dfd82, 55 },
    { 0, 0 },
    { 0x1e2dfd85, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4dfd88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32dfd8b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20dfd92, 90 },
    { 0xcfdfd92, 100 },
    { 0, 0 },
    { 0x277dfd95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaddfd9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fdfda1, 90 },
    { 0x6fdfda2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160dfdaa, 98 },
    { 0xf68dfdaa, 58 },
    { 0xbdcdfdaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bdfdb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952dfdb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x895dfdb9, 33 },
    { 0x9aedfdba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaadfdbf, 52 },
    { 0x5e5dfdbf, 61 },
    { 0x603dfdbf, 33 },
    { 0, 0 },
    { 0xce8dfdc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493dfdc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cdfdcb, 34 },
    { 0x335dfdcc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23adfdd6, 23 },
    { 0x60edfdd7, 33 },
    { 0, 0 },
    { 0x9d5dfdd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6edfddc, 33 },
    { 0xf9edfddd, 23 },
    { 0, 0 },
    { 0xcfcdfddf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fdfde4, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfde7, 46 },
    { 0x311dfde8, 93 },
    { 0xcf3dfde9, 95 },
    { 0xdcfdfdea, 55 },
    { 0x4e1dfde7, 65 },
    { 0xac6dfdec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116dfdf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630dfdf9, 23 },
    { 0, 0 },
    { 0xd6ddfdfb, 95 },
    { 0x52bdfdfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668dfe03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dfe08, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bdfe0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178dfe0f, 61 },
    { 0, 0 },
    { 0x760dfe11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddfe15, 33 },
    { 0x98dfe16, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bdfe1a, 61 },
    { 0xcd8dfe1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94dfe22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x640dfe25, 23 },
    { 0, 0 },
    { 0xea4dfe27, 90 },
    { 0x874dfe27, 83 },
    { 0x62fdfe29, 23 },
    { 0x4e7dfe2a, 90 },
    { 0x1a5dfe28, 33 },
    { 0xdb0dfe2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690dfe32, 33 },
    { 0x189dfe33, 90 },
    { 0xe67dfe34, 61 },
    { 0xc2bdfe34, 95 },
    { 0x310dfe34, 76 },
    { 0xccbdfe34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324dfe3c, 46 },
    { 0x323dfe3d, 98 },
    { 0xc9bdfe3e, 40 },
    { 0x1a1dfe3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496dfe45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dfe48, 23 },
    { 0x3a1dfe49, 90 },
    { 0x4c9dfe4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42edfe50, 95 },
    { 0xe67dfe50, 77 },
    { 0xf31dfe50, 61 },
    { 0x8d9dfe53, 33 },
    { 0xe23dfe53, 95 },
    { 0x999dfe55, 90 },
    { 0x738dfe56, 77 },
    { 0xab7dfe55, 66 },
    { 0x9bdfe58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x519dfe5b, 90 },
    { 0xd71dfe5c, 23 },
    { 0xf84dfe5d, 61 },
    { 0, 0 },
    { 0x37edfe5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4edfe62, 55 },
    { 0x7dedfe63, 61 },
    { 0x46ddfe64, 84 },
    { 0xd6ddfe63, 90 },
    { 0, 0 },
    { 0x470dfe67, 55 },
    { 0, 0 },
    { 0x225dfe69, 55 },
    { 0x24bdfe6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bdfe6d, 61 },
    { 0xefdfe6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82edfe72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdfe75, 90 },
    { 0xdbddfe75, 33 },
    { 0x2f7dfe77, 87 },
    { 0x829dfe76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dfe7b, 33 },
    { 0, 0 },
    { 0x213dfe7d, 43 },
    { 0xecbdfe7e, 61 },
    { 0xe7bdfe7f, 78 },
    { 0xdb0dfe80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0dfe83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaadfe87, 90 },
    { 0x471dfe88, 66 },
    { 0xe10dfe87, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09dfe8c, 90 },
    { 0, 0 },
    { 0x48dfe8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdfe95, 52 },
    { 0, 0 },
    { 0xce9dfe97, 84 },
    { 0x8c5dfe98, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fadfea2, 67 },
    { 0, 0 },
    { 0xc60dfea4, 95 },
    { 0xbd6dfea5, 95 },
    { 0x2a5dfea4, 93 },
    { 0xfcfdfea7, 98 },
    { 0x9e3dfea8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbdfeae, 68 },
    { 0, 0 },
    { 0xef5dfeb0, 33 },
    { 0, 0 },
    { 0x21edfeb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6dfebe, 90 },
    { 0, 0 },
    { 0x4f2dfec0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200dfec7, 87 },
    { 0xa10dfec7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90dfece, 76 },
    { 0x3f2dfecf, 61 },
    { 0, 0 },
    { 0x9a9dfed1, 46 },
    { 0, 0 },
    { 0x483dfed3, 95 },
    { 0xf5bdfed4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeadfed8, 23 },
    { 0xbefdfed9, 61 },
    { 0, 0 },
    { 0x911dfedb, 24 },
    { 0, 0 },
    { 0x385dfedd, 43 },
    { 0x950dfede, 95 },
    { 0x33cdfedf, 98 },
    { 0xe9dfedd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3dfeea, 98 },
    { 0, 0 },
    { 0xe3ddfeec, 61 },
    { 0, 0 },
    { 0x928dfeee, 37 },
    { 0, 0 },
    { 0xaf2dfef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdfef7, 90 },
    { 0xf54dfef8, 98 },
    { 0x5b0dfef8, 96 },
    { 0xf85dfefa, 76 },
    { 0, 0 },
    { 0xeebdfefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x643dfeff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cdff05, 81 },
    { 0xe5cdff06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6dff0b, 95 },
    { 0x392dff0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x407dff0f, 93 },
    { 0x1fadff10, 76 },
    { 0xd24dff0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dff17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5dff1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4dff20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0dff28, 34 },
    { 0xaa4dff29, 88 },
    { 0x380dff2a, 90 },
    { 0x148dff2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85adff34, 46 },
    { 0, 0 },
    { 0x52adff36, 98 },
    { 0xddfdff37, 78 },
    { 0x55fdff38, 85 },
    { 0xf19dff39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08dff3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bdff43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dff47, 33 },
    { 0, 0 },
    { 0x12cdff49, 90 },
    { 0, 0 },
    { 0xafadff4b, 90 },
    { 0xef1dff4c, 76 },
    { 0, 0 },
    { 0xd02dff4e, 85 },
    { 0xd8fdff4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2dff52, 99 },
    { 0x7f4dff52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20dff57, 78 },
    { 0xab7dff57, 17 },
    { 0x72edff59, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafdff5c, 61 },
    { 0, 0 },
    { 0xa84dff5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadff66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04dff69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99dff6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dff72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdff7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65edff7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1dff81, 95 },
    { 0x7badff81, 98 },
    { 0x2b3dff83, 23 },
    { 0x667dff83, 95 },
    { 0, 0 },
    { 0x550dff86, 33 },
    { 0x996dff87, 85 },
    { 0x615dff87, 90 },
    { 0, 0 },
    { 0x406dff8a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823dff91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54edff94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddff9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22dff9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fdffa4, 98 },
    { 0x1c5dffa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777dffab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cdffb0, 33 },
    { 0xa5fdffb0, 33 },
    { 0, 0 },
    { 0xab0dffb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45dffb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634dffbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cdffbe, 55 },
    { 0x3f2dffbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x817dffc2, 95 },
    { 0x952dffc3, 85 },
    { 0xe39dffc2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x318dffc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aedffca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549dffcf, 95 },
    { 0x615dffd0, 34 },
    { 0x9bedffd0, 43 },
    { 0, 0 },
    { 0x6c1dffd3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2dffd6, 23 },
    { 0x15bdffd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9dffde, 66 },
    { 0x226dffdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212dffe5, 80 },
    { 0x92bdffe5, 55 },
    { 0, 0 },
    { 0xa5bdffe8, 61 },
    { 0xffedffe9, 98 },
    { 0xebadffe9, 68 },
    { 0x1a6dffea, 46 },
    { 0x14dffec, 33 },
    { 0x9bddffed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80dfffc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45edffff, 95 },
    { 0, 0 },
    { 0xf57e0001, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e0005, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517e0009, 98 },
    { 0xf64e0009, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xface0010, 90 },
    { 0, 0 },
    { 0x1d1e0012, 33 },
    { 0, 0 },
    { 0xc83e0014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x968e0017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e001b, 61 },
    { 0, 0 },
    { 0x1dfe001d, 95 },
    { 0xe74e001d, 33 },
    { 0x29e001f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9e002b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e002f, 52 },
    { 0xf8ae002f, 84 },
    { 0xba6e002f, 90 },
    { 0xfc2e0030, 90 },
    { 0x1e7e0030, 98 },
    { 0xab9e0032, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51e003c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae003f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebe0042, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e0047, 40 },
    { 0, 0 },
    { 0x824e0049, 78 },
    { 0xfc7e004a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae0052, 36 },
    { 0, 0 },
    { 0xfaae0054, 23 },
    { 0x6cae0055, 61 },
    { 0xeece0054, 23 },
    { 0x1e5e0057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2e005a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce005e, 34 },
    { 0x5dde005f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898e0063, 78 },
    { 0xb5ae0064, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e0069, 77 },
    { 0x2c3e006a, 78 },
    { 0x559e006b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e0074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37e0077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ee007a, 33 },
    { 0x434e007b, 61 },
    { 0xda8e007b, 96 },
    { 0, 0 },
    { 0x92fe007e, 33 },
    { 0x11ce007f, 34 },
    { 0, 0 },
    { 0x617e0081, 100 },
    { 0xb4ce0081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55e0087, 23 },
    { 0, 0 },
    { 0x646e0089, 55 },
    { 0x5dce0089, 76 },
    { 0, 0 },
    { 0x461e008c, 91 },
    { 0x6c0e008c, 95 },
    { 0xa42e008e, 33 },
    { 0xe1be008f, 43 },
    { 0x8afe0090, 90 },
    { 0x67e0091, 98 },
    { 0x17de0092, 85 },
    { 0xe01e0093, 43 },
    { 0xac5e008e, 65 },
    { 0x35de0095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35e009c, 95 },
    { 0x544e009d, 26 },
    { 0x492e009e, 23 },
    { 0xe11e009d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8e00a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723e00ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9e00b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e00b5, 90 },
    { 0x291e00b5, 95 },
    { 0x15ae00b7, 43 },
    { 0x762e00b8, 55 },
    { 0xa4be00b9, 23 },
    { 0, 0 },
    { 0xff1e00bb, 33 },
    { 0, 0 },
    { 0xef4e00bd, 23 },
    { 0, 0 },
    { 0x98ee00bf, 46 },
    { 0x28ae00bf, 67 },
    { 0x740e00bf, 93 },
    { 0x757e00c2, 98 },
    { 0xa07e00c0, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27de00cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e00d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eae00d7, 98 },
    { 0x7c5e00d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce00db, 93 },
    { 0xb03e00dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de00df, 23 },
    { 0x249e00e0, 33 },
    { 0, 0 },
    { 0x51ee00e2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49e00e5, 34 },
    { 0xb48e00e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e00ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfece00ef, 90 },
    { 0x873e00f0, 98 },
    { 0xc73e00f1, 95 },
    { 0x3d2e00f2, 95 },
    { 0x8fee00f1, 23 },
    { 0x9d6e00f1, 23 },
    { 0x4bde00f0, 23 },
    { 0, 0 },
    { 0xa62e00f7, 90 },
    { 0xb39e00f8, 93 },
    { 0xb3ee00f9, 61 },
    { 0x618e00f9, 43 },
    { 0xb7e00fb, 23 },
    { 0x675e00fb, 50 },
    { 0, 0 },
    { 0x858e00fe, 98 },
    { 0, 0 },
    { 0x83ce0100, 61 },
    { 0, 0 },
    { 0xc19e0102, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee0107, 99 },
    { 0x7b9e0108, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e010b, 55 },
    { 0x8c6e010b, 33 },
    { 0, 0 },
    { 0x7dee010e, 43 },
    { 0, 0 },
    { 0x894e0110, 33 },
    { 0x4cfe0110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ce0114, 34 },
    { 0, 0 },
    { 0x5d7e0116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee0119, 98 },
    { 0, 0 },
    { 0x41de011b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe011f, 80 },
    { 0x727e0120, 67 },
    { 0x43ee0121, 33 },
    { 0, 0 },
    { 0x839e0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x454e0126, 46 },
    { 0, 0 },
    { 0x107e0128, 95 },
    { 0x498e0129, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e012c, 33 },
    { 0x889e012d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e0130, 65 },
    { 0x2bae0131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x303e0134, 69 },
    { 0x4b6e0135, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de013b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce014d, 61 },
    { 0x4abe014d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcae0153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de0158, 95 },
    { 0x2c0e0159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5de015c, 55 },
    { 0xaa0e015d, 66 },
    { 0xf91e015d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce0161, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce0166, 76 },
    { 0x5bee0167, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ce0171, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee0174, 34 },
    { 0, 0 },
    { 0x8f0e0176, 91 },
    { 0x9d2e0177, 90 },
    { 0x65e0176, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce017b, 98 },
    { 0x96ce017c, 98 },
    { 0xc2be017b, 95 },
    { 0x7a3e017c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6e0181, 61 },
    { 0x9b8e0181, 52 },
    { 0xf35e0183, 33 },
    { 0x5e4e0184, 98 },
    { 0xdc7e0184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e0188, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480e018e, 99 },
    { 0xd3de018e, 98 },
    { 0, 0 },
    { 0x42ee0191, 33 },
    { 0x665e0192, 55 },
    { 0xac4e0192, 95 },
    { 0, 0 },
    { 0xf63e0195, 23 },
    { 0x954e0196, 76 },
    { 0, 0 },
    { 0xb03e0198, 84 },
    { 0x945e0199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e019d, 66 },
    { 0xf90e019e, 84 },
    { 0x860e019f, 19 },
    { 0xe45e01a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e01a3, 33 },
    { 0, 0 },
    { 0xba0e01a5, 77 },
    { 0, 0 },
    { 0xc29e01a7, 56 },
    { 0x63de01a7, 33 },
    { 0x9bfe01a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ce01ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafae01b2, 55 },
    { 0, 0 },
    { 0xb5ee01b4, 90 },
    { 0xb03e01b5, 98 },
    { 0x5a4e01b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7e01bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4e01c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e01c9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be01d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe01d6, 68 },
    { 0x580e01d7, 61 },
    { 0x9fae01d8, 61 },
    { 0x319e01d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589e01dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11be01e0, 61 },
    { 0x33be01e1, 85 },
    { 0x57fe01e1, 23 },
    { 0, 0 },
    { 0xad8e01e4, 46 },
    { 0x16ee01e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe01eb, 87 },
    { 0, 0 },
    { 0x869e01ed, 36 },
    { 0xd02e01ee, 95 },
    { 0, 0 },
    { 0x928e01f0, 98 },
    { 0, 0 },
    { 0x17ce01f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886e01f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e01f9, 93 },
    { 0x664e01fa, 23 },
    { 0xb48e01fb, 81 },
    { 0x2abe01fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e01ff, 76 },
    { 0xb34e0200, 76 },
    { 0x98de0201, 99 },
    { 0x855e01ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e0206, 43 },
    { 0, 0 },
    { 0x6c1e0208, 90 },
    { 0, 0 },
    { 0x10ae020a, 78 },
    { 0x7a4e020b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e0211, 98 },
    { 0, 0 },
    { 0x389e0213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdce0216, 90 },
    { 0, 0 },
    { 0x5b5e0218, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e021c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0e0220, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939e0224, 76 },
    { 0x8a5e0225, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacee0229, 61 },
    { 0, 0 },
    { 0xa49e022b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8e022e, 61 },
    { 0x49ee022f, 34 },
    { 0xceae0230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdce023b, 55 },
    { 0x9bae023c, 33 },
    { 0x249e023d, 67 },
    { 0x805e023d, 90 },
    { 0x545e023d, 99 },
    { 0, 0 },
    { 0x6dee0241, 90 },
    { 0x8b4e0242, 78 },
    { 0x8f4e0241, 23 },
    { 0xd71e0244, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e024d, 90 },
    { 0x5c3e024e, 95 },
    { 0, 0 },
    { 0xcc4e0250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ae0257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fe025a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e0260, 95 },
    { 0, 0 },
    { 0xf82e0262, 101 },
    { 0x744e0263, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e0270, 68 },
    { 0xa93e0271, 87 },
    { 0xe0ce0272, 99 },
    { 0, 0 },
    { 0xede0274, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735e0278, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ae027c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e0284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9be028b, 33 },
    { 0, 0 },
    { 0xa7de028d, 76 },
    { 0xa6e028e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x221e0291, 61 },
    { 0xc5be0292, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e0296, 98 },
    { 0, 0 },
    { 0x8dde0298, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e029b, 90 },
    { 0xc0ce029b, 23 },
    { 0xc2fe029c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee02a3, 87 },
    { 0x48ae02a4, 95 },
    { 0xf5e02a5, 65 },
    { 0, 0 },
    { 0xab2e02a7, 33 },
    { 0, 0 },
    { 0x678e02a9, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be02ae, 96 },
    { 0, 0 },
    { 0x790e02b0, 76 },
    { 0x739e02b1, 33 },
    { 0xe04e02b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cde02b5, 90 },
    { 0xd0ae02b6, 40 },
    { 0x89fe02b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e02ba, 55 },
    { 0xfcbe02bb, 95 },
    { 0, 0 },
    { 0x7bce02bd, 85 },
    { 0, 0 },
    { 0xd4ee02bf, 76 },
    { 0xd0ce02bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2e02c5, 40 },
    { 0x290e02c5, 100 },
    { 0x392e02c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbe02cb, 95 },
    { 0xedde02cc, 88 },
    { 0xdc7e02cd, 61 },
    { 0xa85e02cc, 67 },
    { 0x88ae02cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e02d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbe02d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e02db, 91 },
    { 0xde7e02dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707e02e3, 78 },
    { 0x8e0e02e3, 90 },
    { 0, 0 },
    { 0xbde02e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce02ee, 98 },
    { 0, 0 },
    { 0xa2ae02f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2e02f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce02f7, 33 },
    { 0, 0 },
    { 0x175e02f9, 34 },
    { 0x94de02fa, 85 },
    { 0xaf2e02f9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ae0301, 93 },
    { 0x767e0302, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e030d, 76 },
    { 0xd31e030e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e0314, 45 },
    { 0xfd6e0314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4be0318, 91 },
    { 0x7d1e0319, 61 },
    { 0, 0 },
    { 0xcbde031b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3de0320, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ace0325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48be0329, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbee032d, 55 },
    { 0x657e032e, 76 },
    { 0xece032e, 44 },
    { 0, 0 },
    { 0x904e0331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5e0335, 33 },
    { 0, 0 },
    { 0x712e0337, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e033a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ee033e, 95 },
    { 0x3f9e033e, 55 },
    { 0x29be033f, 23 },
    { 0x569e033f, 55 },
    { 0, 0 },
    { 0xef0e0343, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87be0347, 46 },
    { 0, 0 },
    { 0xa58e0349, 98 },
    { 0x976e034a, 68 },
    { 0xde034a, 61 },
    { 0x5b4e034c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3e034f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece0352, 61 },
    { 0, 0 },
    { 0xa20e0354, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ae0359, 61 },
    { 0, 0 },
    { 0x74fe035b, 61 },
    { 0xb31e035c, 90 },
    { 0x60de035c, 90 },
    { 0x830e035b, 90 },
    { 0x498e035f, 23 },
    { 0x47de0360, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e0364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce0368, 23 },
    { 0x978e0369, 87 },
    { 0, 0 },
    { 0xd5ae036b, 98 },
    { 0, 0 },
    { 0xf7ce036d, 61 },
    { 0, 0 },
    { 0x619e036f, 33 },
    { 0xab3e0370, 43 },
    { 0, 0 },
    { 0xca0e0372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e0376, 43 },
    { 0xe6be0377, 66 },
    { 0x80fe0378, 34 },
    { 0x4e5e0378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ee0380, 61 },
    { 0xda3e0380, 23 },
    { 0xcede0380, 76 },
    { 0xd57e0381, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0386, 98 },
    { 0, 0 },
    { 0x2fae0388, 23 },
    { 0xb52e0389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4e038c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x758e038f, 99 },
    { 0xf44e0390, 90 },
    { 0x109e038f, 53 },
    { 0x377e0392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae0396, 78 },
    { 0x1ebe0396, 67 },
    { 0xa9ee0398, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e039c, 68 },
    { 0x66ee039d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fe03a0, 61 },
    { 0, 0 },
    { 0xefe03a2, 43 },
    { 0, 0 },
    { 0xeae03a4, 66 },
    { 0xbdce03a5, 46 },
    { 0, 0 },
    { 0xbc0e03a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e03ad, 98 },
    { 0xb24e03ae, 84 },
    { 0xd4de03ae, 90 },
    { 0x2c5e03af, 23 },
    { 0x874e03b1, 90 },
    { 0xd17e03b1, 61 },
    { 0, 0 },
    { 0x656e03b4, 52 },
    { 0, 0 },
    { 0x46be03b6, 23 },
    { 0xdfe03b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fe03c3, 98 },
    { 0xdbde03c4, 76 },
    { 0x3be03c3, 23 },
    { 0x265e03c6, 95 },
    { 0xa95e03c7, 98 },
    { 0, 0 },
    { 0x167e03c9, 100 },
    { 0xfc7e03ca, 78 },
    { 0xefee03cb, 76 },
    { 0x524e03cc, 90 },
    { 0x819e03c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee03d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4e03e2, 90 },
    { 0, 0 },
    { 0xa20e03e4, 83 },
    { 0x1c0e03e5, 61 },
    { 0xaede03e5, 34 },
    { 0, 0 },
    { 0x10de03e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809e03ec, 33 },
    { 0x948e03ed, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e03f3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4e03f6, 76 },
    { 0xcabe03f7, 93 },
    { 0, 0 },
    { 0xe81e03f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7e0401, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ede0406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e040c, 76 },
    { 0x61ae040d, 34 },
    { 0x5d6e040d, 90 },
    { 0x9c8e040f, 93 },
    { 0x58ce0410, 90 },
    { 0x38be0411, 46 },
    { 0xaade0410, 43 },
    { 0x287e0410, 98 },
    { 0x79be0414, 90 },
    { 0x2a2e0411, 61 },
    { 0, 0 },
    { 0xd2be0417, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe041c, 90 },
    { 0, 0 },
    { 0xfd8e041e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ae0421, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ae0426, 98 },
    { 0, 0 },
    { 0xafbe0428, 90 },
    { 0xc4de0429, 33 },
    { 0x67ee042a, 90 },
    { 0x6d6e042a, 93 },
    { 0xff1e042c, 33 },
    { 0x2fde042c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe0432, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e0436, 90 },
    { 0x220e0437, 55 },
    { 0, 0 },
    { 0x15be0439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796e043e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fee0441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e0446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae044b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e0455, 55 },
    { 0x334e0456, 76 },
    { 0xb98e0457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fe045a, 90 },
    { 0xf94e045b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fe0460, 100 },
    { 0xa1be0461, 61 },
    { 0x303e0462, 17 },
    { 0, 0 },
    { 0x4a1e0464, 98 },
    { 0x5aae0464, 81 },
    { 0xa26e0466, 61 },
    { 0xa8ee0466, 93 },
    { 0x1dce0467, 78 },
    { 0xb14e0467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ae046d, 90 },
    { 0xb43e046e, 93 },
    { 0, 0 },
    { 0x68fe0470, 61 },
    { 0x78ae0471, 95 },
    { 0xaeae0470, 34 },
    { 0x91be0472, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2e047b, 93 },
    { 0x768e047c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ae0484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e0487, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52e048b, 84 },
    { 0xface048b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e048f, 99 },
    { 0xf84e048f, 33 },
    { 0x701e0490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951e0498, 76 },
    { 0x881e0498, 90 },
    { 0xc5de049a, 95 },
    { 0x51fe049a, 93 },
    { 0xe9ce049c, 61 },
    { 0x217e049c, 98 },
    { 0x4f1e049e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e04a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe04a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9e04a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358e04b2, 90 },
    { 0xbf8e04b3, 98 },
    { 0, 0 },
    { 0x34e04b5, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e04bf, 46 },
    { 0, 0 },
    { 0x6e3e04c1, 61 },
    { 0x250e04c2, 90 },
    { 0, 0 },
    { 0xb63e04c4, 91 },
    { 0xcf2e04c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fce04cb, 33 },
    { 0x76ce04cc, 23 },
    { 0, 0 },
    { 0xc6ee04ce, 98 },
    { 0x74be04ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1e04d4, 76 },
    { 0, 0 },
    { 0xc2ee04d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41e04d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e04dc, 84 },
    { 0xe67e04dd, 95 },
    { 0, 0 },
    { 0x99ee04df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ae04e2, 33 },
    { 0, 0 },
    { 0x9fae04e4, 55 },
    { 0x24fe04e4, 95 },
    { 0x6f1e04e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e04ea, 23 },
    { 0x311e04eb, 77 },
    { 0xc3e04eb, 34 },
    { 0, 0 },
    { 0xcbee04ee, 33 },
    { 0, 0 },
    { 0xbf5e04f0, 43 },
    { 0, 0 },
    { 0x208e04f2, 95 },
    { 0x88ee04f3, 33 },
    { 0xb20e04f4, 61 },
    { 0x890e04f2, 66 },
    { 0x6ae04f6, 34 },
    { 0, 0 },
    { 0x39de04f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e0500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e050a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9e050d, 76 },
    { 0xbb0e050d, 61 },
    { 0x21de050f, 33 },
    { 0, 0 },
    { 0x6b2e0511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2de0517, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae051c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce0523, 23 },
    { 0, 0 },
    { 0xb49e0525, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348e0529, 12 },
    { 0, 0 },
    { 0x60ae052b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4e052e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e0531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe0538, 33 },
    { 0x967e0539, 60 },
    { 0x974e0538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe053d, 33 },
    { 0xfe4e053e, 90 },
    { 0xd4ae053f, 68 },
    { 0, 0 },
    { 0xc6be0541, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760e0544, 23 },
    { 0, 0 },
    { 0x6f4e0546, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cee054a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfe0551, 23 },
    { 0x7aee0552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868e0556, 17 },
    { 0, 0 },
    { 0xbe2e0558, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384e055e, 33 },
    { 0, 0 },
    { 0xec7e0560, 34 },
    { 0x65ce0560, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e0567, 95 },
    { 0x8a5e0568, 55 },
    { 0x93ce0569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9e0571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e0578, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1e057e, 84 },
    { 0x701e057f, 98 },
    { 0, 0 },
    { 0xa28e0581, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ae0587, 80 },
    { 0, 0 },
    { 0x5ace0589, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152e058f, 76 },
    { 0x594e058f, 90 },
    { 0x16de0591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862e0598, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e059b, 61 },
    { 0x3dde059c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde059f, 23 },
    { 0xc65e05a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475e05a4, 23 },
    { 0xa0e05a5, 33 },
    { 0, 0 },
    { 0x3a5e05a7, 46 },
    { 0xd07e05a7, 69 },
    { 0xb73e05a7, 90 },
    { 0x5b3e05a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae05b2, 43 },
    { 0x4bde05b3, 98 },
    { 0, 0 },
    { 0x4d0e05b5, 98 },
    { 0xd95e05b6, 98 },
    { 0xcdae05b7, 61 },
    { 0xb24e05b8, 99 },
    { 0xd57e05b6, 55 },
    { 0, 0 },
    { 0x8fee05bb, 99 },
    { 0x273e05bc, 76 },
    { 0x7abe05bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9e05c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e05c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe05cc, 55 },
    { 0, 0 },
    { 0x4d0e05ce, 85 },
    { 0x408e05cf, 66 },
    { 0x2fae05ce, 96 },
    { 0xc6ce05cf, 33 },
    { 0xd96e05d2, 88 },
    { 0x29e05d2, 76 },
    { 0x674e05d1, 46 },
    { 0, 0 },
    { 0xf70e05d6, 95 },
    { 0x2c4e05d7, 93 },
    { 0xc36e05d8, 66 },
    { 0xb5ce05d9, 52 },
    { 0x20ee05d9, 90 },
    { 0, 0 },
    { 0xabde05dc, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ace05e0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae05e3, 46 },
    { 0, 0 },
    { 0x989e05e5, 95 },
    { 0x7e7e05e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5e05ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e05f0, 55 },
    { 0x9a3e05f0, 90 },
    { 0xc59e05f2, 23 },
    { 0x741e05f3, 61 },
    { 0xfb4e05f2, 55 },
    { 0, 0 },
    { 0x3aae05f6, 90 },
    { 0, 0 },
    { 0x534e05f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ace05fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de05ff, 95 },
    { 0xebce05ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x993e0603, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363e0607, 90 },
    { 0xede0608, 17 },
    { 0x680e0607, 76 },
    { 0x880e060a, 90 },
    { 0x3a4e060b, 61 },
    { 0xb88e060c, 93 },
    { 0x481e0607, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2e0610, 87 },
    { 0xcbae0610, 95 },
    { 0x2cfe0612, 52 },
    { 0x9d0e0613, 98 },
    { 0x6c9e0614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe0617, 61 },
    { 0xee5e0618, 90 },
    { 0xc48e0619, 85 },
    { 0x5a6e061a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e061f, 95 },
    { 0x346e061f, 66 },
    { 0x774e0620, 95 },
    { 0xf7ae0621, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e0628, 34 },
    { 0x9fde0628, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce062e, 33 },
    { 0xf65e062f, 61 },
    { 0x4c4e0630, 90 },
    { 0xc52e0630, 33 },
    { 0x947e0632, 90 },
    { 0x81de0631, 95 },
    { 0xf4ee0634, 36 },
    { 0xe54e0633, 90 },
    { 0x752e062e, 100 },
    { 0xa95e0637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0e063d, 87 },
    { 0xa5be063e, 43 },
    { 0xfc2e063f, 61 },
    { 0, 0 },
    { 0xe6ee0641, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e0644, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ee0647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ae064d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dee0653, 51 },
    { 0xbd0e0654, 61 },
    { 0x93e0655, 61 },
    { 0x9a6e0653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe065a, 33 },
    { 0, 0 },
    { 0xf8de065c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581e0661, 84 },
    { 0x7de0662, 90 },
    { 0x7d7e0661, 90 },
    { 0xde5e0661, 98 },
    { 0xc62e0661, 33 },
    { 0x3a8e0664, 90 },
    { 0x130e0665, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e066c, 90 },
    { 0xf49e066c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896e0673, 61 },
    { 0x5d4e0674, 43 },
    { 0xd48e0673, 96 },
    { 0xb48e0676, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de0679, 66 },
    { 0x154e067a, 91 },
    { 0x922e067a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ce0682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ee0688, 34 },
    { 0x348e0689, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e068c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1e068f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e0696, 90 },
    { 0x590e0697, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726e06a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de06a7, 23 },
    { 0x535e06a8, 67 },
    { 0x410e06a7, 90 },
    { 0xe99e06aa, 90 },
    { 0, 0 },
    { 0x96be06ac, 95 },
    { 0xff3e06ad, 33 },
    { 0x138e06ac, 90 },
    { 0xa6ae06af, 52 },
    { 0x2bee06af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78e06b9, 52 },
    { 0, 0 },
    { 0x91de06bb, 33 },
    { 0x21ce06bb, 61 },
    { 0xcf8e06bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522e06c2, 98 },
    { 0, 0 },
    { 0xd2ae06c4, 98 },
    { 0, 0 },
    { 0xe7ae06c6, 47 },
    { 0xbf5e06c7, 23 },
    { 0xd51e06c8, 95 },
    { 0xc33e06c7, 52 },
    { 0xd75e06c9, 23 },
    { 0, 0 },
    { 0xc4e06cc, 66 },
    { 0x6a9e06cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e06d2, 77 },
    { 0x438e06d3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de06d7, 90 },
    { 0, 0 },
    { 0x45ce06d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x584e06dc, 98 },
    { 0xd64e06dd, 61 },
    { 0xba1e06dc, 40 },
    { 0, 0 },
    { 0x118e06e0, 85 },
    { 0, 0 },
    { 0xea5e06e2, 78 },
    { 0x9be06e3, 23 },
    { 0, 0 },
    { 0x1ece06e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x295e06e8, 84 },
    { 0, 0 },
    { 0xefee06ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e06ed, 90 },
    { 0x23ce06ed, 59 },
    { 0x371e06ed, 100 },
    { 0xbece06f0, 23 },
    { 0x3dbe06f1, 66 },
    { 0x54ee06f1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0e06f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e06f8, 43 },
    { 0xc73e06f8, 33 },
    { 0xb8de06fa, 90 },
    { 0xf0de06f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9e0702, 76 },
    { 0x2be0703, 17 },
    { 0x370e0702, 66 },
    { 0x659e0704, 61 },
    { 0, 0 },
    { 0xbf2e0707, 93 },
    { 0xec7e0707, 46 },
    { 0x9dae0709, 33 },
    { 0x938e070a, 95 },
    { 0x744e0708, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe0710, 33 },
    { 0, 0 },
    { 0x969e0712, 33 },
    { 0xf2be0713, 87 },
    { 0, 0 },
    { 0xc4ae0715, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de0718, 99 },
    { 0, 0 },
    { 0x955e071a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e0723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e0729, 33 },
    { 0, 0 },
    { 0xa40e072b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88e072e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5e0731, 90 },
    { 0xad4e0731, 98 },
    { 0x4cbe0733, 98 },
    { 0x9efe0734, 98 },
    { 0, 0 },
    { 0xabe0736, 38 },
    { 0x8ace0737, 90 },
    { 0x32e0738, 83 },
    { 0xe6be0737, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471e073d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814e0742, 46 },
    { 0x72ee0743, 61 },
    { 0x4a5e0744, 33 },
    { 0xf77e0743, 67 },
    { 0x94e0746, 95 },
    { 0x99fe0746, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e074a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e074e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48e0751, 88 },
    { 0x19de0752, 87 },
    { 0x10ee0752, 90 },
    { 0x3aae0752, 23 },
    { 0x4d4e0755, 93 },
    { 0x351e0752, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be0759, 90 },
    { 0x63fe0759, 90 },
    { 0, 0 },
    { 0x6c1e075c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0de0760, 88 },
    { 0, 0 },
    { 0x7e9e0762, 98 },
    { 0x6ade0763, 23 },
    { 0x5afe0764, 99 },
    { 0x3cfe0764, 98 },
    { 0x159e0766, 66 },
    { 0x97e0764, 55 },
    { 0x4bfe0767, 61 },
    { 0xfafe0769, 33 },
    { 0xfde0769, 61 },
    { 0, 0 },
    { 0x4c2e076c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e0773, 34 },
    { 0, 0 },
    { 0xac6e0775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae0779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e0780, 33 },
    { 0x89de0781, 46 },
    { 0xd98e0782, 90 },
    { 0xf12e0782, 95 },
    { 0, 0 },
    { 0xbbde0785, 55 },
    { 0x995e0786, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0e078c, 17 },
    { 0xdb0e078d, 33 },
    { 0x6fae078e, 90 },
    { 0, 0 },
    { 0x7e3e0790, 90 },
    { 0xa66e0791, 81 },
    { 0, 0 },
    { 0x104e0793, 23 },
    { 0xea4e0794, 91 },
    { 0, 0 },
    { 0x6dae0796, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e079f, 81 },
    { 0xde9e07a0, 76 },
    { 0x605e079f, 101 },
    { 0x6a2e07a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e07a7, 33 },
    { 0, 0 },
    { 0x916e07a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade07ae, 40 },
    { 0xd2de07ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ee07b2, 95 },
    { 0x5e3e07b3, 88 },
    { 0xb17e07b2, 78 },
    { 0x7b4e07b5, 90 },
    { 0x6f4e07b5, 34 },
    { 0xc0fe07b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e07ba, 91 },
    { 0xeb9e07ba, 90 },
    { 0xdcfe07bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x754e07bf, 61 },
    { 0, 0 },
    { 0x64ae07c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e07c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e07ce, 93 },
    { 0, 0 },
    { 0x8b2e07d0, 23 },
    { 0xed1e07d1, 78 },
    { 0x496e07d0, 76 },
    { 0, 0 },
    { 0x1ffe07d4, 90 },
    { 0, 0 },
    { 0x8e4e07d6, 95 },
    { 0x962e07d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e07da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7e07de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ae07e1, 61 },
    { 0, 0 },
    { 0x5b0e07e3, 90 },
    { 0x25de07e4, 38 },
    { 0xf36e07e5, 99 },
    { 0, 0 },
    { 0xa54e07e7, 91 },
    { 0x16ae07e8, 90 },
    { 0xa33e07e8, 43 },
    { 0x8f2e07e7, 67 },
    { 0xba1e07eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de07f1, 90 },
    { 0xb4e07f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ce07f9, 98 },
    { 0xb2ee07fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be0801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e0804, 61 },
    { 0xa6be0805, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de0808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689e080e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8e0811, 23 },
    { 0x196e0812, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fe081b, 55 },
    { 0, 0 },
    { 0x428e081d, 23 },
    { 0x2ffe081e, 93 },
    { 0xec3e081f, 95 },
    { 0x132e081f, 98 },
    { 0, 0 },
    { 0x4dce0822, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926e0828, 55 },
    { 0, 0 },
    { 0xec5e082a, 61 },
    { 0, 0 },
    { 0x7abe082c, 34 },
    { 0, 0 },
    { 0xa4ee082e, 52 },
    { 0xf7ae082e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619e0834, 46 },
    { 0, 0 },
    { 0x362e0836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e083a, 61 },
    { 0x979e083b, 34 },
    { 0, 0 },
    { 0xde1e083d, 90 },
    { 0, 0 },
    { 0xd38e083f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156e0845, 90 },
    { 0x330e0846, 93 },
    { 0xf0fe0846, 55 },
    { 0xfa7e0848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e0851, 23 },
    { 0xad7e0852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36e0858, 33 },
    { 0, 0 },
    { 0xc9ce085a, 43 },
    { 0x9f8e085b, 23 },
    { 0x44ee085b, 61 },
    { 0x817e085d, 61 },
    { 0xbbe085d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7e0862, 61 },
    { 0, 0 },
    { 0x2b8e0864, 77 },
    { 0x379e0865, 61 },
    { 0, 0 },
    { 0x9e6e0867, 43 },
    { 0, 0 },
    { 0x202e0869, 69 },
    { 0xf3ee086a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e0872, 98 },
    { 0x9e7e0872, 90 },
    { 0xd60e0872, 23 },
    { 0, 0 },
    { 0xdfce0876, 77 },
    { 0xbbfe0877, 33 },
    { 0x132e0878, 98 },
    { 0x1b9e0876, 33 },
    { 0x40ee087a, 78 },
    { 0x151e087b, 23 },
    { 0x8ee087b, 90 },
    { 0x41ae087d, 36 },
    { 0, 0 },
    { 0x624e087f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e0885, 95 },
    { 0, 0 },
    { 0x67ce0887, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69e088a, 90 },
    { 0, 0 },
    { 0xd07e088c, 90 },
    { 0x372e088d, 99 },
    { 0xf2fe088e, 95 },
    { 0xf6fe088e, 34 },
    { 0, 0 },
    { 0x14ee0891, 55 },
    { 0x46fe0892, 55 },
    { 0xf8de0893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e0896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977e089a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47be089e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7e08a3, 61 },
    { 0, 0 },
    { 0xe27e08a5, 66 },
    { 0, 0 },
    { 0xee9e08a7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe08ad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e08b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4e08b3, 66 },
    { 0xe62e08b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254e08bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4e08c0, 84 },
    { 0, 0 },
    { 0xc21e08c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ace08c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e08ce, 87 },
    { 0x366e08ce, 83 },
    { 0x587e08cf, 23 },
    { 0xf6ee08d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x187e08d4, 34 },
    { 0, 0 },
    { 0x594e08d6, 33 },
    { 0x448e08d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e08dc, 77 },
    { 0x754e08dd, 93 },
    { 0, 0 },
    { 0xae0e08df, 33 },
    { 0, 0 },
    { 0xee7e08e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de08e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e08e8, 87 },
    { 0x588e08e9, 61 },
    { 0x757e08ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e08f3, 33 },
    { 0x9b2e08f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x794e08f7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e08fb, 61 },
    { 0, 0 },
    { 0x28e08fd, 66 },
    { 0x3a2e08fd, 33 },
    { 0x977e08ff, 90 },
    { 0x65ce0900, 90 },
    { 0x76ee08fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e0907, 90 },
    { 0, 0 },
    { 0xed5e0909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fae090d, 33 },
    { 0x7e9e090e, 23 },
    { 0xe67e090f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0912, 55 },
    { 0, 0 },
    { 0xebe0914, 28 },
    { 0, 0 },
    { 0x21de0916, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xace0919, 43 },
    { 0xb9ee0919, 33 },
    { 0x100e0919, 76 },
    { 0xa01e091c, 61 },
    { 0xf0be091d, 95 },
    { 0x577e091e, 23 },
    { 0x311e091f, 34 },
    { 0xeb9e0920, 90 },
    { 0x70fe0921, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e0925, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e092b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e092f, 98 },
    { 0x208e0930, 93 },
    { 0xbf6e0931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde0937, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8be093a, 100 },
    { 0x8dce093b, 66 },
    { 0xa38e093c, 95 },
    { 0x332e093d, 90 },
    { 0xc83e093e, 55 },
    { 0x93e093f, 61 },
    { 0xcc2e0940, 23 },
    { 0x4b9e0941, 98 },
    { 0xbe7e0941, 33 },
    { 0, 0 },
    { 0xdb7e0944, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee094d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe0951, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae0957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e095d, 95 },
    { 0, 0 },
    { 0x174e095f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e0966, 81 },
    { 0x320e0967, 33 },
    { 0x533e0968, 33 },
    { 0, 0 },
    { 0xfa2e096a, 66 },
    { 0, 0 },
    { 0x549e096c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be0970, 61 },
    { 0, 0 },
    { 0xb24e0972, 34 },
    { 0, 0 },
    { 0x653e0974, 66 },
    { 0xe11e0974, 33 },
    { 0, 0 },
    { 0xa2be0977, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee097a, 33 },
    { 0x99be097b, 98 },
    { 0, 0 },
    { 0xd59e097d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243e0981, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbe0989, 99 },
    { 0x747e0989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1e0991, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4e0995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ae099c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce09a0, 55 },
    { 0x617e09a0, 83 },
    { 0x2ade09a2, 33 },
    { 0x97be09a3, 95 },
    { 0xb94e09a3, 46 },
    { 0xf37e09a2, 76 },
    { 0x557e09a5, 78 },
    { 0xb3ee09a7, 84 },
    { 0x63ee09a8, 34 },
    { 0x449e09a9, 33 },
    { 0, 0 },
    { 0x7dde09ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38e09af, 98 },
    { 0xf20e09af, 78 },
    { 0x645e09b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9be09b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ae09bb, 98 },
    { 0xcb6e09bc, 46 },
    { 0x4aee09bd, 98 },
    { 0x469e09bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ee09c2, 55 },
    { 0xecae09c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e09c8, 90 },
    { 0x20fe09c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ae09d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645e09d6, 95 },
    { 0, 0 },
    { 0x346e09d8, 34 },
    { 0x23be09d9, 23 },
    { 0, 0 },
    { 0x263e09db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869e09e3, 95 },
    { 0, 0 },
    { 0x158e09e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63e09e8, 55 },
    { 0x56be09e9, 23 },
    { 0x3d8e09ea, 66 },
    { 0xdae09eb, 91 },
    { 0x27ee09ea, 96 },
    { 0xa12e09ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5e09f2, 95 },
    { 0xdf0e09f3, 90 },
    { 0x8b2e09f2, 52 },
    { 0xed8e09f3, 95 },
    { 0x481e09f6, 90 },
    { 0, 0 },
    { 0xbbce09f8, 36 },
    { 0x1e2e09f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae09fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e0a03, 98 },
    { 0x1bbe0a04, 98 },
    { 0xefae0a03, 98 },
    { 0x5b9e0a04, 84 },
    { 0x6c0e0a03, 93 },
    { 0, 0 },
    { 0x212e0a09, 98 },
    { 0x9efe0a0a, 61 },
    { 0, 0 },
    { 0xfe4e0a0c, 90 },
    { 0x425e0a0d, 55 },
    { 0xea9e0a0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0a12, 43 },
    { 0x7b4e0a13, 98 },
    { 0x9ede0a14, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de0a19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e0a1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6e0a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x690e0a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x957e0a27, 98 },
    { 0x711e0a28, 95 },
    { 0x748e0a28, 93 },
    { 0xab0e0a28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbe0a2d, 85 },
    { 0x3b3e0a2e, 98 },
    { 0xc8ae0a2f, 93 },
    { 0xaf4e0a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfe0a36, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92e0a39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace0a3d, 23 },
    { 0xa6ae0a3e, 52 },
    { 0x395e0a3e, 46 },
    { 0x52ae0a40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ae0a43, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0a46, 99 },
    { 0xeace0a47, 75 },
    { 0xf9de0a48, 61 },
    { 0x4d3e0a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e0a4c, 33 },
    { 0x431e0a4d, 61 },
    { 0x177e0a4e, 33 },
    { 0xbb1e0a4e, 55 },
    { 0xa45e0a4e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0e0a56, 87 },
    { 0xe64e0a56, 90 },
    { 0x364e0a56, 61 },
    { 0, 0 },
    { 0xc64e0a5a, 34 },
    { 0xa3be0a5b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e0a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be0a62, 46 },
    { 0xae7e0a63, 43 },
    { 0, 0 },
    { 0x34ae0a65, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e0a69, 61 },
    { 0, 0 },
    { 0xccce0a6b, 55 },
    { 0xa0ee0a6c, 23 },
    { 0x263e0a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577e0a71, 90 },
    { 0, 0 },
    { 0x353e0a73, 81 },
    { 0xd8ae0a74, 34 },
    { 0, 0 },
    { 0xee0a76, 55 },
    { 0xf82e0a77, 33 },
    { 0x2de0a78, 76 },
    { 0x955e0a79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e0a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4e0a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce0a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3be0a88, 90 },
    { 0x34de0a88, 95 },
    { 0x66fe0a88, 23 },
    { 0xff4e0a8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e0a92, 33 },
    { 0xdcee0a92, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e0a96, 99 },
    { 0, 0 },
    { 0x979e0a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ce0a9d, 90 },
    { 0, 0 },
    { 0x344e0a9f, 98 },
    { 0, 0 },
    { 0x735e0aa1, 94 },
    { 0x73de0aa2, 98 },
    { 0, 0 },
    { 0xeb4e0aa4, 61 },
    { 0x88ee0aa5, 90 },
    { 0x1dbe0aa6, 33 },
    { 0x2fbe0aa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0aaa, 64 },
    { 0x5a1e0aaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e0ab1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fce0ab5, 61 },
    { 0x5c9e0ab5, 93 },
    { 0x251e0ab5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e0abb, 66 },
    { 0xe6fe0abc, 33 },
    { 0xf68e0abb, 33 },
    { 0, 0 },
    { 0x46de0abf, 61 },
    { 0, 0 },
    { 0x33e0ac1, 34 },
    { 0xb2e0ac1, 33 },
    { 0xe2fe0ac3, 90 },
    { 0x85ce0ac3, 91 },
    { 0x484e0ac3, 85 },
    { 0xb81e0ac6, 90 },
    { 0, 0 },
    { 0xc7be0ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ce0acb, 69 },
    { 0xfabe0acb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ee0ad3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444e0adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e0adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e0ae4, 33 },
    { 0x9e4e0ae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be0ae8, 81 },
    { 0, 0 },
    { 0xbe6e0aea, 34 },
    { 0x5cce0aeb, 90 },
    { 0x86ae0aec, 61 },
    { 0x3f8e0aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e0af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4e0af7, 90 },
    { 0x81e0af8, 34 },
    { 0x823e0af8, 33 },
    { 0xbd3e0afa, 85 },
    { 0x7d5e0af7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71e0b00, 95 },
    { 0x164e0b00, 23 },
    { 0x43be0b01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ae0b06, 78 },
    { 0x7b8e0b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e0b14, 88 },
    { 0xecbe0b14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8e0b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce0b1f, 55 },
    { 0, 0 },
    { 0x718e0b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813e0b2b, 90 },
    { 0, 0 },
    { 0xae3e0b2d, 90 },
    { 0x1fde0b2e, 98 },
    { 0x544e0b2f, 23 },
    { 0x227e0b30, 61 },
    { 0x496e0b31, 61 },
    { 0x9bfe0b31, 61 },
    { 0xd73e0b33, 66 },
    { 0x70be0b34, 95 },
    { 0x66ae0b34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e0b39, 91 },
    { 0x1d2e0b3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee0b3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e0b40, 100 },
    { 0xbd7e0b41, 23 },
    { 0, 0 },
    { 0xfeae0b43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eae0b47, 93 },
    { 0xa49e0b48, 98 },
    { 0xf6be0b49, 60 },
    { 0xcd9e0b4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835e0b4f, 95 },
    { 0, 0 },
    { 0x270e0b51, 55 },
    { 0x3e9e0b52, 61 },
    { 0, 0 },
    { 0x947e0b54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be0b58, 75 },
    { 0x3c6e0b59, 78 },
    { 0xf21e0b5a, 91 },
    { 0xb53e0b59, 23 },
    { 0xca8e0b5a, 33 },
    { 0xfeae0b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebe0b62, 33 },
    { 0xb36e0b62, 43 },
    { 0x3ce0b64, 81 },
    { 0x6fde0b65, 34 },
    { 0, 0 },
    { 0xf39e0b67, 87 },
    { 0x82fe0b67, 46 },
    { 0x53e0b67, 33 },
    { 0x952e0b67, 55 },
    { 0, 0 },
    { 0x26ce0b6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615e0b72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfe0b7a, 90 },
    { 0xcf7e0b7b, 95 },
    { 0xc8ae0b7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae0b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe0b85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48e0b88, 98 },
    { 0, 0 },
    { 0xe35e0b8a, 98 },
    { 0, 0 },
    { 0xfbfe0b8c, 84 },
    { 0x7d0e0b8d, 55 },
    { 0xcf6e0b8e, 52 },
    { 0, 0 },
    { 0xefee0b90, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde0b97, 55 },
    { 0, 0 },
    { 0xc8ae0b99, 95 },
    { 0x2fe0b9a, 77 },
    { 0x9fe0b9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e0ba3, 23 },
    { 0x75ee0ba3, 98 },
    { 0x8eee0ba5, 93 },
    { 0xaf8e0ba6, 98 },
    { 0x854e0ba6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffee0bb1, 87 },
    { 0, 0 },
    { 0xc8ce0bb3, 78 },
    { 0xf64e0bb4, 61 },
    { 0xc1ee0bb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e0bc2, 90 },
    { 0x43fe0bc3, 67 },
    { 0, 0 },
    { 0xa05e0bc5, 76 },
    { 0, 0 },
    { 0xbd2e0bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6e0bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e0bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e0bd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde0bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fe0bd9, 61 },
    { 0x8d4e0bda, 33 },
    { 0, 0 },
    { 0x20ce0bdc, 87 },
    { 0xef0e0bdd, 95 },
    { 0x2f8e0bde, 33 },
    { 0x14e0bdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7e0be5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0be0be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffe0bec, 46 },
    { 0x7dee0bed, 55 },
    { 0x176e0bec, 53 },
    { 0, 0 },
    { 0xc8ae0bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x786e0bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e0bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4e0bfa, 61 },
    { 0x336e0bfa, 84 },
    { 0, 0 },
    { 0xf20e0bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e0c01, 33 },
    { 0, 0 },
    { 0x608e0c03, 76 },
    { 0x7bde0c03, 90 },
    { 0, 0 },
    { 0x1d5e0c06, 76 },
    { 0x76be0c06, 98 },
    { 0x90ee0c07, 34 },
    { 0x98ce0c09, 86 },
    { 0x603e0c0a, 69 },
    { 0xcf9e0c09, 85 },
    { 0xcfae0c0c, 43 },
    { 0xe74e0c0c, 90 },
    { 0x5dae0c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6be0c16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfe0c1d, 93 },
    { 0x430e0c1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0c21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x486e0c24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6e0c28, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ee0c2e, 94 },
    { 0xa9be0c2e, 33 },
    { 0, 0 },
    { 0xdd8e0c31, 98 },
    { 0, 0 },
    { 0x1c6e0c33, 91 },
    { 0xf76e0c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e0c38, 33 },
    { 0, 0 },
    { 0xe8de0c3a, 90 },
    { 0x55ae0c3b, 61 },
    { 0xd5ae0c3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8e0c3f, 55 },
    { 0x7f5e0c40, 95 },
    { 0, 0 },
    { 0x83fe0c42, 61 },
    { 0x2dee0c43, 33 },
    { 0x683e0c44, 95 },
    { 0x7a8e0c45, 90 },
    { 0xbfee0c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e0c49, 90 },
    { 0x95ee0c4a, 93 },
    { 0x87ae0c4a, 98 },
    { 0x7c8e0c4c, 98 },
    { 0xc9ee0c4d, 33 },
    { 0x981e0c4d, 55 },
    { 0xedbe0c4f, 87 },
    { 0x700e0c4f, 90 },
    { 0x349e0c4f, 90 },
    { 0xa90e0c4f, 23 },
    { 0x35be0c4d, 61 },
    { 0x4b8e0c54, 52 },
    { 0x8c8e0c55, 43 },
    { 0x32ee0c55, 23 },
    { 0, 0 },
    { 0xce8e0c58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5e0c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x849e0c61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x869e0c64, 78 },
    { 0x87ce0c65, 43 },
    { 0xaf0e0c65, 61 },
    { 0xe3be0c67, 23 },
    { 0, 0 },
    { 0xb4ce0c69, 98 },
    { 0x775e0c6a, 40 },
    { 0x6ee0c69, 90 },
    { 0x79fe0c6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaee0c6f, 95 },
    { 0x6f8e0c70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608e0c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce0c78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12e0c7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde0c7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x668e0c81, 17 },
    { 0x97de0c82, 23 },
    { 0x764e0c83, 46 },
    { 0x5b9e0c83, 23 },
    { 0x7cae0c84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73e0c89, 87 },
    { 0x609e0c8a, 95 },
    { 0x307e0c8b, 98 },
    { 0, 0 },
    { 0xbfbe0c8d, 98 },
    { 0, 0 },
    { 0x60ae0c8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x313e0c92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce0c98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e0c9b, 50 },
    { 0, 0 },
    { 0x54de0c9d, 76 },
    { 0, 0 },
    { 0xa03e0c9f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x381e0ca2, 95 },
    { 0x394e0ca3, 100 },
    { 0xd37e0ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ce0ca7, 23 },
    { 0x440e0ca7, 90 },
    { 0xf45e0ca7, 61 },
    { 0x714e0caa, 52 },
    { 0x340e0cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ce0cb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558e0cb9, 78 },
    { 0, 0 },
    { 0x3cfe0cbb, 95 },
    { 0, 0 },
    { 0xbeae0cbd, 95 },
    { 0, 0 },
    { 0xad4e0cbf, 55 },
    { 0x6a4e0cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fe0cc4, 95 },
    { 0x12ee0cc5, 95 },
    { 0xba6e0cc6, 95 },
    { 0xfe8e0cc7, 98 },
    { 0xd8ae0cc8, 91 },
    { 0xd03e0cc8, 68 },
    { 0xca8e0cca, 76 },
    { 0x95fe0cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae0cd8, 90 },
    { 0x568e0cd9, 90 },
    { 0x9ee0cda, 61 },
    { 0x9d9e0cda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e0cde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e0ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae0ce9, 99 },
    { 0x92ee0cea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1e0ced, 90 },
    { 0xb2ee0cee, 95 },
    { 0xfb4e0cef, 33 },
    { 0xc3ee0cf0, 66 },
    { 0, 0 },
    { 0x5c4e0cf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07e0cf6, 87 },
    { 0, 0 },
    { 0xf21e0cf8, 23 },
    { 0x7dae0cf9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e0cfd, 98 },
    { 0x699e0cfe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e0d01, 34 },
    { 0xd69e0d02, 61 },
    { 0, 0 },
    { 0xc50e0d04, 64 },
    { 0, 0 },
    { 0x8c3e0d06, 95 },
    { 0x3ace0d07, 43 },
    { 0xa2fe0d07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e0d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde0d13, 33 },
    { 0xddee0d14, 90 },
    { 0x2c6e0d15, 61 },
    { 0x44be0d16, 55 },
    { 0x1a9e0d16, 34 },
    { 0, 0 },
    { 0xb14e0d19, 78 },
    { 0xc0be0d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e0d1d, 33 },
    { 0xffde0d1e, 33 },
    { 0x45ce0d1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e0d22, 95 },
    { 0xbdbe0d23, 84 },
    { 0, 0 },
    { 0xe5fe0d25, 90 },
    { 0x520e0d26, 100 },
    { 0x42ce0d27, 61 },
    { 0xd70e0d28, 78 },
    { 0xeb2e0d29, 76 },
    { 0x1b4e0d26, 61 },
    { 0x7ede0d2b, 61 },
    { 0, 0 },
    { 0xa8de0d2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605e0d31, 33 },
    { 0, 0 },
    { 0xbe0d33, 93 },
    { 0, 0 },
    { 0xe35e0d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de0d3c, 98 },
    { 0x4b6e0d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e0d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe0d48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04e0d4b, 76 },
    { 0x4b4e0d4c, 61 },
    { 0xd7ae0d4d, 61 },
    { 0, 0 },
    { 0x8fe0d4f, 85 },
    { 0xc03e0d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61e0d53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x453e0d56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4be0d5a, 61 },
    { 0, 0 },
    { 0xc3ae0d5c, 90 },
    { 0x18ce0d5d, 43 },
    { 0xa16e0d5c, 33 },
    { 0x1dfe0d5f, 23 },
    { 0x34ce0d60, 85 },
    { 0x51ce0d61, 91 },
    { 0xbdde0d61, 33 },
    { 0, 0 },
    { 0xd4ae0d64, 95 },
    { 0xdc6e0d65, 17 },
    { 0x608e0d66, 84 },
    { 0, 0 },
    { 0x5a8e0d68, 24 },
    { 0xed1e0d69, 93 },
    { 0, 0 },
    { 0x507e0d6b, 34 },
    { 0xb65e0d6c, 99 },
    { 0x4ebe0d6c, 84 },
    { 0x1c7e0d6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ee0d72, 99 },
    { 0, 0 },
    { 0x51ee0d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35e0d78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18de0d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e0d80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae0d85, 90 },
    { 0, 0 },
    { 0x9c0e0d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae0d8a, 69 },
    { 0, 0 },
    { 0x794e0d8c, 98 },
    { 0x937e0d8d, 43 },
    { 0x56ae0d8e, 90 },
    { 0x36ce0d8d, 90 },
    { 0xc71e0d8f, 43 },
    { 0x8f4e0d8f, 61 },
    { 0x7f3e0d92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfce0d96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e0d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e0d9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e0da7, 99 },
    { 0x6a6e0da7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e0dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397e0db2, 90 },
    { 0xca8e0db3, 33 },
    { 0xf7e0db3, 87 },
    { 0xe4ae0db5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e0dba, 43 },
    { 0xfcae0dba, 34 },
    { 0, 0 },
    { 0x952e0dbd, 33 },
    { 0x927e0dbd, 90 },
    { 0xccde0dbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fce0dc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46e0dcd, 93 },
    { 0x15fe0dce, 84 },
    { 0x2a2e0dce, 36 },
    { 0xfb0e0dd0, 90 },
    { 0x132e0dcd, 76 },
    { 0x35de0dd2, 76 },
    { 0, 0 },
    { 0xd88e0dd4, 87 },
    { 0x7f3e0dd4, 95 },
    { 0xee4e0dd6, 34 },
    { 0x54fe0dd5, 90 },
    { 0x282e0dd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be0ddd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0de4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e0de7, 75 },
    { 0xbcee0de8, 87 },
    { 0xa6e0de7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424e0df0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307e0df7, 93 },
    { 0, 0 },
    { 0xe6e0df9, 95 },
    { 0, 0 },
    { 0x93e0dfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85e0dff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9e0e03, 84 },
    { 0xd7ae0e03, 33 },
    { 0x68e0e05, 23 },
    { 0x885e0e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e0e09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fce0e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35e0e16, 69 },
    { 0x103e0e17, 90 },
    { 0xf00e0e17, 90 },
    { 0x15ae0e19, 33 },
    { 0x7d6e0e19, 55 },
    { 0xb8ae0e1b, 61 },
    { 0xb1ce0e1c, 34 },
    { 0xc6be0e1a, 90 },
    { 0x6b5e0e1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0be0e22, 36 },
    { 0xe69e0e22, 33 },
    { 0, 0 },
    { 0xff7e0e25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e0e28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee0e32, 33 },
    { 0x78be0e32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe0e3d, 33 },
    { 0x24de0e3e, 90 },
    { 0, 0 },
    { 0xec0e0e40, 36 },
    { 0, 0 },
    { 0xd95e0e42, 98 },
    { 0x229e0e42, 61 },
    { 0x3b3e0e44, 43 },
    { 0x166e0e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ee0e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1e0e4f, 55 },
    { 0, 0 },
    { 0x4a3e0e51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28e0e55, 95 },
    { 0x54ae0e55, 23 },
    { 0xc12e0e57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe0e5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe0e5e, 93 },
    { 0x41ce0e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ae0e62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e0e66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0e6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be0e73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae0e76, 90 },
    { 0, 0 },
    { 0xfc3e0e78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689e0e7e, 98 },
    { 0, 0 },
    { 0xb52e0e80, 61 },
    { 0x4c4e0e80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae0e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e0e8d, 66 },
    { 0xb2e0e8d, 76 },
    { 0x940e0e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cde0e94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e0e98, 34 },
    { 0xe81e0e98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ee0e9c, 98 },
    { 0x360e0e9d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1e0ea0, 43 },
    { 0xb1ce0ea1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e0ea5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ae0eaa, 78 },
    { 0xaade0eab, 90 },
    { 0xfd3e0eac, 23 },
    { 0x84be0eac, 61 },
    { 0x9d2e0ead, 91 },
    { 0xfd9e0eaf, 61 },
    { 0x92ae0eb0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ae0eb3, 78 },
    { 0x6afe0eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e0eb7, 43 },
    { 0xdfae0eb8, 61 },
    { 0x979e0eb9, 90 },
    { 0x2ede0eba, 52 },
    { 0, 0 },
    { 0x3d1e0ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3e0ebf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e0ec4, 68 },
    { 0, 0 },
    { 0xce2e0ec6, 95 },
    { 0xbb5e0ec7, 90 },
    { 0, 0 },
    { 0x5f4e0ec9, 90 },
    { 0x800e0eca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee0ece, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e0ed4, 77 },
    { 0x96ce0ed4, 98 },
    { 0, 0 },
    { 0x1fe0ed7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e0eda, 34 },
    { 0x648e0edb, 61 },
    { 0, 0 },
    { 0x5ce0edd, 61 },
    { 0x357e0ede, 68 },
    { 0, 0 },
    { 0x7b7e0ee0, 98 },
    { 0x643e0ee1, 85 },
    { 0, 0 },
    { 0x882e0ee3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de0eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e0ef1, 66 },
    { 0x86de0ef2, 33 },
    { 0x385e0ef2, 95 },
    { 0x943e0ef4, 33 },
    { 0x545e0ef4, 95 },
    { 0x612e0ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe0efc, 23 },
    { 0x3d1e0efc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ce0f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06e0f09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49be0f0c, 90 },
    { 0xd45e0f0c, 61 },
    { 0, 0 },
    { 0xae3e0f0f, 98 },
    { 0xd54e0f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee0f13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e0f18, 76 },
    { 0, 0 },
    { 0x117e0f1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe0f1e, 90 },
    { 0x5e8e0f1e, 23 },
    { 0xd73e0f20, 33 },
    { 0, 0 },
    { 0x553e0f22, 33 },
    { 0xcede0f22, 98 },
    { 0x408e0f23, 34 },
    { 0x570e0f25, 34 },
    { 0, 0 },
    { 0xd78e0f27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebe0f2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae0f30, 88 },
    { 0xeefe0f31, 69 },
    { 0, 0 },
    { 0xe35e0f33, 77 },
    { 0, 0 },
    { 0xcbee0f35, 90 },
    { 0, 0 },
    { 0xfede0f37, 33 },
    { 0xebee0f38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe0f3b, 98 },
    { 0, 0 },
    { 0x438e0f3d, 78 },
    { 0x47ce0f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394e0f42, 90 },
    { 0xdb6e0f42, 68 },
    { 0x6fce0f44, 98 },
    { 0x597e0f42, 90 },
    { 0xe8e0f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e0f4c, 23 },
    { 0xb7fe0f4d, 34 },
    { 0, 0 },
    { 0xc10e0f4f, 98 },
    { 0, 0 },
    { 0xc05e0f51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e0f56, 33 },
    { 0x1b1e0f56, 85 },
    { 0x2c2e0f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e0f5b, 33 },
    { 0x2b7e0f5b, 93 },
    { 0x999e0f5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce0f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e0f67, 95 },
    { 0x7c8e0f67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbede0f71, 93 },
    { 0xef0e0f71, 67 },
    { 0, 0 },
    { 0x1b6e0f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7e0f80, 61 },
    { 0x99e0f80, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103e0f8d, 90 },
    { 0x844e0f8e, 46 },
    { 0x420e0f8f, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e0f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9e0f9c, 98 },
    { 0, 0 },
    { 0xebce0f9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e0faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9be0fb0, 97 },
    { 0xe29e0fb0, 98 },
    { 0x4aee0fb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee0fb5, 98 },
    { 0x18be0fb5, 76 },
    { 0xa66e0fb7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280e0fc2, 60 },
    { 0xfe9e0fc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81e0fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe0fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e0fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177e0fd5, 33 },
    { 0x9e0e0fd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39e0fda, 33 },
    { 0, 0 },
    { 0xd5ee0fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e0fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741e0fe8, 95 },
    { 0xcabe0fe9, 33 },
    { 0, 0 },
    { 0x39ce0feb, 35 },
    { 0x348e0feb, 33 },
    { 0x2c3e0fec, 43 },
    { 0, 0 },
    { 0xc95e0fef, 43 },
    { 0, 0 },
    { 0x934e0ff1, 61 },
    { 0xc9fe0ff2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12e0ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e0ff8, 67 },
    { 0x999e0ff9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84e0fff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20e1002, 90 },
    { 0, 0 },
    { 0xfd5e1004, 75 },
    { 0, 0 },
    { 0xa9fe1006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e1009, 100 },
    { 0x279e1009, 71 },
    { 0x292e100b, 61 },
    { 0xfa3e100b, 95 },
    { 0x150e100d, 55 },
    { 0x1d2e100e, 93 },
    { 0xf81e100c, 98 },
    { 0xb0de1010, 98 },
    { 0xa15e1010, 81 },
    { 0, 0 },
    { 0x223e1013, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511e1017, 98 },
    { 0, 0 },
    { 0x485e1019, 77 },
    { 0x6aee101a, 98 },
    { 0x240e101b, 85 },
    { 0x848e1019, 75 },
    { 0, 0 },
    { 0xdcfe101e, 38 },
    { 0xe38e101f, 98 },
    { 0x748e1020, 23 },
    { 0x461e101e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520e1025, 98 },
    { 0x42de1025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fe102c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6e1030, 90 },
    { 0xd3de1031, 95 },
    { 0, 0 },
    { 0x650e1033, 76 },
    { 0xcaae1034, 61 },
    { 0x4ece1035, 99 },
    { 0x9e2e1036, 95 },
    { 0x9a6e1037, 98 },
    { 0x2f5e1038, 35 },
    { 0xe5ce1037, 90 },
    { 0xee0e1036, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581e1044, 23 },
    { 0x52be1044, 43 },
    { 0x3bde1044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e104e, 33 },
    { 0x735e104e, 61 },
    { 0, 0 },
    { 0x1ae1051, 55 },
    { 0, 0 },
    { 0xa02e1053, 95 },
    { 0xafe1053, 34 },
    { 0x90de1054, 23 },
    { 0x4d9e1056, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7e105c, 33 },
    { 0x114e105d, 88 },
    { 0xa62e105e, 90 },
    { 0xd70e105d, 61 },
    { 0xd10e1060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e1065, 66 },
    { 0x42de1066, 61 },
    { 0x77ce1067, 76 },
    { 0x904e1065, 46 },
    { 0xfc7e1066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e1070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x897e1073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e1076, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0e107c, 77 },
    { 0x8b7e107c, 23 },
    { 0x777e107e, 99 },
    { 0xc53e107f, 90 },
    { 0xbc6e1080, 98 },
    { 0, 0 },
    { 0xfb2e1082, 33 },
    { 0xe43e1082, 34 },
    { 0x5cae1084, 76 },
    { 0xa0fe1085, 78 },
    { 0x78fe1086, 61 },
    { 0xbc7e1083, 36 },
    { 0x739e1088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e108c, 75 },
    { 0x5fe108d, 76 },
    { 0, 0 },
    { 0x9fae108f, 95 },
    { 0x3d1e1090, 61 },
    { 0, 0 },
    { 0xb7e1092, 55 },
    { 0x711e1093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfe1099, 90 },
    { 0x2b3e109a, 61 },
    { 0x405e109b, 98 },
    { 0x731e109b, 91 },
    { 0xa5de109a, 76 },
    { 0x3ede109b, 61 },
    { 0xbf6e109e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e10a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66e10a9, 46 },
    { 0x85ae10aa, 34 },
    { 0x64de10aa, 98 },
    { 0, 0 },
    { 0x651e10ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e10b5, 66 },
    { 0xf80e10b6, 90 },
    { 0x904e10b7, 100 },
    { 0xbb5e10b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x121e10bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e10c4, 90 },
    { 0, 0 },
    { 0x4a6e10c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fe10c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7e10cc, 33 },
    { 0x82be10cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ce10d3, 67 },
    { 0, 0 },
    { 0xbade10d5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e10db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583e10e2, 46 },
    { 0, 0 },
    { 0x35be10e4, 90 },
    { 0xcd4e10e5, 95 },
    { 0x684e10e6, 61 },
    { 0xd0de10e7, 81 },
    { 0x67fe10e8, 98 },
    { 0x11e10e9, 95 },
    { 0xa25e10e9, 90 },
    { 0xf34e10e9, 68 },
    { 0x963e10eb, 76 },
    { 0x770e10ed, 42 },
    { 0xfdce10ee, 33 },
    { 0xf16e10e5, 43 },
    { 0x76be10f0, 61 },
    { 0x4d4e10f0, 34 },
    { 0xb2be10f2, 66 },
    { 0x495e10ee, 33 },
    { 0x7aae10f2, 61 },
    { 0x7c9e10f5, 61 },
    { 0xf2e10f6, 33 },
    { 0x29ae10f6, 33 },
    { 0, 0 },
    { 0x2e1e10f9, 33 },
    { 0xda4e10fa, 95 },
    { 0, 0 },
    { 0x64de10fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fae10ff, 33 },
    { 0, 0 },
    { 0x12ce1101, 61 },
    { 0, 0 },
    { 0x5d2e1103, 33 },
    { 0x1a2e1103, 95 },
    { 0, 0 },
    { 0x1e9e1106, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60e110d, 33 },
    { 0x3ede110d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865e1118, 96 },
    { 0x941e1118, 90 },
    { 0x7c7e1119, 34 },
    { 0xa77e1118, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae1123, 90 },
    { 0xd75e1124, 93 },
    { 0x6f2e1124, 55 },
    { 0x623e1126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de1129, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e112f, 95 },
    { 0, 0 },
    { 0x27ae1131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aae1138, 76 },
    { 0xc9ee1138, 95 },
    { 0xe84e113a, 55 },
    { 0x75ee1138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fe113e, 23 },
    { 0xec6e113f, 61 },
    { 0x53ce113f, 86 },
    { 0xc3ee1141, 23 },
    { 0x6fe1140, 33 },
    { 0, 0 },
    { 0x767e1144, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce1149, 75 },
    { 0xc58e114a, 43 },
    { 0xc54e114a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde114f, 55 },
    { 0xcabe1150, 61 },
    { 0x386e114f, 90 },
    { 0xfb9e1152, 33 },
    { 0xf10e1153, 64 },
    { 0, 0 },
    { 0x761e1155, 91 },
    { 0xb07e1156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e115b, 52 },
    { 0xd21e115b, 34 },
    { 0x5f6e115c, 61 },
    { 0, 0 },
    { 0xe79e115f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42e1163, 33 },
    { 0xd2e1163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e1168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56e116c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174e1171, 43 },
    { 0x523e1172, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e1179, 90 },
    { 0xef5e117a, 91 },
    { 0xb89e117a, 76 },
    { 0x18ee117c, 90 },
    { 0xf7fe117c, 23 },
    { 0x74de117a, 55 },
    { 0x17ee117f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fce118e, 61 },
    { 0x59be118f, 90 },
    { 0xf0fe118e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee119a, 93 },
    { 0xa99e119b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe119e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7be11a2, 90 },
    { 0xcc4e11a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e11a7, 95 },
    { 0x138e11a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeae11ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72e11b1, 23 },
    { 0, 0 },
    { 0xff4e11b3, 98 },
    { 0x125e11b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de11b8, 33 },
    { 0x64ee11b9, 75 },
    { 0xac2e11ba, 61 },
    { 0xe53e11ba, 95 },
    { 0x8d5e11bc, 12 },
    { 0, 0 },
    { 0xc7fe11be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e11c1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992e11c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391e11cb, 23 },
    { 0, 0 },
    { 0xbcfe11cd, 64 },
    { 0x188e11ce, 76 },
    { 0xca9e11ce, 90 },
    { 0xb4ce11d0, 55 },
    { 0, 0 },
    { 0x207e11d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73be11d6, 34 },
    { 0xa47e11d6, 93 },
    { 0x308e11d6, 98 },
    { 0xb81e11d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116e11de, 95 },
    { 0xbdce11df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35e11e4, 33 },
    { 0x158e11e5, 33 },
    { 0x2fce11e4, 95 },
    { 0, 0 },
    { 0x98be11e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe11ee, 78 },
    { 0, 0 },
    { 0x249e11f0, 75 },
    { 0xac4e11f0, 90 },
    { 0x84ee11f2, 22 },
    { 0xb2be11f0, 90 },
    { 0, 0 },
    { 0xba4e11f5, 61 },
    { 0x57de11f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e11fa, 98 },
    { 0, 0 },
    { 0x3c8e11fc, 90 },
    { 0, 0 },
    { 0xcb4e11fe, 55 },
    { 0xedde11ff, 90 },
    { 0xef3e11ff, 61 },
    { 0, 0 },
    { 0x10ce1202, 66 },
    { 0, 0 },
    { 0x230e1204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cce1207, 43 },
    { 0x43ee1207, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e121c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e121f, 76 },
    { 0, 0 },
    { 0xee0e1221, 90 },
    { 0, 0 },
    { 0x297e1223, 65 },
    { 0xc43e1224, 95 },
    { 0, 0 },
    { 0xde8e1226, 43 },
    { 0, 0 },
    { 0xc26e1228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e122b, 65 },
    { 0x6e5e122c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17e122f, 68 },
    { 0, 0 },
    { 0x25de1231, 95 },
    { 0, 0 },
    { 0x2eae1233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ee1240, 88 },
    { 0xe66e1240, 84 },
    { 0x5b1e1242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fe1248, 90 },
    { 0xb21e1249, 84 },
    { 0xea9e1248, 61 },
    { 0x5f0e124b, 87 },
    { 0xc53e1248, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ae124f, 43 },
    { 0, 0 },
    { 0xa3ae1251, 23 },
    { 0xbf1e1252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ce1256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38e1259, 34 },
    { 0xf31e1259, 33 },
    { 0x33ae1259, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e125f, 98 },
    { 0xc8ae1260, 95 },
    { 0x10ee1261, 34 },
    { 0x1bfe1262, 90 },
    { 0x2c1e125f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73e1268, 23 },
    { 0xcc6e1268, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae126c, 33 },
    { 0x196e126d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e1270, 61 },
    { 0xc76e1270, 90 },
    { 0x2fde1272, 23 },
    { 0x682e1272, 95 },
    { 0xdd9e1272, 24 },
    { 0xc8ee1275, 23 },
    { 0, 0 },
    { 0x60ee1277, 23 },
    { 0xf3e1278, 43 },
    { 0, 0 },
    { 0xffde127a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x893e127d, 76 },
    { 0xd72e127e, 34 },
    { 0xfa9e127d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867e1283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce1286, 40 },
    { 0x45e1287, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be128b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ce1290, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7e1295, 90 },
    { 0xd3ae1296, 95 },
    { 0xf89e1297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8e129a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede129e, 33 },
    { 0x9e6e129f, 98 },
    { 0, 0 },
    { 0x37ee12a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdde12a4, 55 },
    { 0x81ee12a5, 98 },
    { 0xcebe12a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe12b5, 43 },
    { 0x241e12b6, 23 },
    { 0x19ce12b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214e12c4, 55 },
    { 0xe18e12c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2e12c8, 98 },
    { 0x702e12c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71e12cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e12cf, 95 },
    { 0xa4ce12d0, 23 },
    { 0x9b3e12d1, 65 },
    { 0x5bce12d2, 90 },
    { 0x6a4e12d3, 87 },
    { 0x4cee12d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee12d9, 90 },
    { 0, 0 },
    { 0xca6e12db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6e12e1, 67 },
    { 0xa13e12e1, 36 },
    { 0x950e12e1, 33 },
    { 0, 0 },
    { 0x7a5e12e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dae12eb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae12ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce12f6, 99 },
    { 0, 0 },
    { 0x884e12f8, 61 },
    { 0, 0 },
    { 0x761e12fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e12fe, 90 },
    { 0, 0 },
    { 0xe34e1300, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e1306, 23 },
    { 0xbbe1307, 100 },
    { 0x99ce1308, 61 },
    { 0x38ce1309, 90 },
    { 0xc5be130a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e130e, 90 },
    { 0, 0 },
    { 0x650e1310, 32 },
    { 0x51ee1311, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e1314, 95 },
    { 0x626e1314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x209e1318, 77 },
    { 0xe18e1319, 76 },
    { 0x48fe1319, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e131d, 55 },
    { 0xf7e131e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7e1323, 33 },
    { 0, 0 },
    { 0xe77e1325, 90 },
    { 0x3c0e1325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fe132c, 90 },
    { 0, 0 },
    { 0x53fe132e, 67 },
    { 0xf0ce132e, 61 },
    { 0xdebe132e, 99 },
    { 0, 0 },
    { 0x3afe1332, 90 },
    { 0x8cee1333, 61 },
    { 0, 0 },
    { 0x183e1335, 68 },
    { 0xc61e1336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9e133b, 88 },
    { 0x482e133c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ae133f, 98 },
    { 0xeabe1340, 95 },
    { 0, 0 },
    { 0x4abe1342, 82 },
    { 0x12ee1343, 98 },
    { 0x344e1342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81de1348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25e134b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x489e134e, 95 },
    { 0x29e134f, 46 },
    { 0x5fe134f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeee1353, 46 },
    { 0, 0 },
    { 0x53de1355, 93 },
    { 0xf0be1356, 33 },
    { 0xd0ee1357, 90 },
    { 0, 0 },
    { 0xf34e1359, 23 },
    { 0xd69e135a, 84 },
    { 0x87fe135b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17e135f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e1362, 23 },
    { 0xac4e1362, 23 },
    { 0, 0 },
    { 0x52ae1365, 91 },
    { 0x67fe1366, 85 },
    { 0x81ce1366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cee136e, 56 },
    { 0xb75e136f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7e1372, 33 },
    { 0x937e1373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654e137e, 85 },
    { 0x1fe137f, 81 },
    { 0xcc5e1380, 37 },
    { 0xb84e137f, 33 },
    { 0x108e1382, 96 },
    { 0xc78e1383, 76 },
    { 0x524e1384, 91 },
    { 0xfb8e1385, 95 },
    { 0x157e1382, 46 },
    { 0x939e1381, 93 },
    { 0xb44e1388, 90 },
    { 0xe0e1389, 51 },
    { 0x1dbe138a, 23 },
    { 0xd79e138b, 81 },
    { 0xcce1389, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe1390, 76 },
    { 0xfa0e1391, 33 },
    { 0xa01e1392, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efe1396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98e139a, 23 },
    { 0x69de139a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x376e139e, 78 },
    { 0xab0e139f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe13aa, 34 },
    { 0, 0 },
    { 0x6d2e13ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfe13b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7e13b7, 95 },
    { 0xda1e13b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeae13bc, 23 },
    { 0x57ee13bc, 98 },
    { 0x87e13be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e13c2, 33 },
    { 0xfb8e13c3, 61 },
    { 0xd7ee13c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fe13c7, 99 },
    { 0xfe5e13c8, 12 },
    { 0x4c6e13c9, 55 },
    { 0, 0 },
    { 0x604e13cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ce13d3, 66 },
    { 0x972e13d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e13d8, 76 },
    { 0xf45e13d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9e13e2, 23 },
    { 0x125e13e2, 90 },
    { 0x584e13e4, 77 },
    { 0x2fee13e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9e13e8, 90 },
    { 0, 0 },
    { 0xf34e13ea, 76 },
    { 0, 0 },
    { 0xdc5e13ec, 61 },
    { 0, 0 },
    { 0x9dde13ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8be13f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6e13f8, 61 },
    { 0x7ece13f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84e1404, 66 },
    { 0x31ce1405, 87 },
    { 0xe1be1405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21e1409, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3e140c, 55 },
    { 0x428e140d, 55 },
    { 0x29ae140d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e1414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e141a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95e1421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e1428, 55 },
    { 0, 0 },
    { 0x3cce142a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e1432, 95 },
    { 0x75e1433, 78 },
    { 0xc9ee1434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x791e1437, 61 },
    { 0x5e4e1438, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ae143d, 95 },
    { 0x839e143d, 33 },
    { 0xe5de143f, 33 },
    { 0x506e143d, 91 },
    { 0x9e0e143d, 17 },
    { 0, 0 },
    { 0x75e1443, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16e1449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9e144c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32e1452, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658e1460, 93 },
    { 0x2c9e1461, 33 },
    { 0x33ae1462, 61 },
    { 0x241e1463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e1466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e146c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efe1474, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x568e1477, 87 },
    { 0xb94e1477, 23 },
    { 0xd51e1478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2e147c, 78 },
    { 0x3dde147d, 55 },
    { 0, 0 },
    { 0x642e147f, 68 },
    { 0x65ae1480, 52 },
    { 0, 0 },
    { 0xc95e1482, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e1486, 33 },
    { 0, 0 },
    { 0x332e1488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441e1493, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222e1499, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0be149e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ee14a3, 90 },
    { 0x4ce14a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ae14a7, 43 },
    { 0xebfe14a8, 23 },
    { 0xc26e14a9, 75 },
    { 0x55fe14a8, 55 },
    { 0xb67e14ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e14ae, 90 },
    { 0, 0 },
    { 0xb4ce14b0, 95 },
    { 0x71ce14b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e14b6, 90 },
    { 0x4b4e14b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290e14bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3e14c2, 23 },
    { 0x60ce14c3, 23 },
    { 0xa58e14c4, 76 },
    { 0xaf8e14c5, 98 },
    { 0xf1ae14c2, 95 },
    { 0x1f4e14c2, 61 },
    { 0x948e14c3, 23 },
    { 0, 0 },
    { 0x4c3e14ca, 61 },
    { 0x1b1e14cb, 98 },
    { 0xa07e14cb, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ce14d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78be14d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fe14df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe14e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e14ee, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x61de14f1, 33 },
    { 0x6fee14f1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3e14f7, 33 },
    { 0, 0 },
    { 0x4d8e14f9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ce14fd, 33 },
    { 0x687e14fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bee1503, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fe1508, 90 },
    { 0xdcde1508, 88 },
    { 0x242e1508, 75 },
    { 0x910e150b, 52 },
    { 0x24e150c, 93 },
    { 0x2a8e150c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e1511, 100 },
    { 0x269e1512, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1515, 88 },
    { 0x9a4e1516, 43 },
    { 0, 0 },
    { 0xb4ee1518, 33 },
    { 0xc06e1518, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54e151e, 33 },
    { 0x157e151f, 98 },
    { 0x94e151f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe1526, 90 },
    { 0xd47e1526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e1535, 98 },
    { 0x557e1535, 95 },
    { 0x959e1537, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ae153b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ede153e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53e1541, 91 },
    { 0x8fae1542, 99 },
    { 0xa19e1541, 83 },
    { 0x21ae1543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fe1548, 98 },
    { 0xb13e1549, 91 },
    { 0xab4e154a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7e154d, 40 },
    { 0, 0 },
    { 0x681e154f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49e1552, 93 },
    { 0xf33e1553, 77 },
    { 0x96e1552, 98 },
    { 0x390e1555, 90 },
    { 0xfb0e1552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e155d, 95 },
    { 0, 0 },
    { 0x577e155f, 76 },
    { 0x5eae155f, 90 },
    { 0x9e1560, 23 },
    { 0, 0 },
    { 0x247e1563, 33 },
    { 0xc8fe1563, 90 },
    { 0x106e1565, 40 },
    { 0, 0 },
    { 0xb3ce1567, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e156b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e156e, 95 },
    { 0xf13e156f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde1576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e1579, 97 },
    { 0x84ae1579, 61 },
    { 0xe03e1579, 23 },
    { 0, 0 },
    { 0x52e157d, 84 },
    { 0, 0 },
    { 0x2ece157f, 98 },
    { 0x6bae157f, 61 },
    { 0xfe4e1581, 61 },
    { 0, 0 },
    { 0xd57e1583, 61 },
    { 0, 0 },
    { 0x29ae1585, 90 },
    { 0xaa2e1586, 96 },
    { 0x370e1585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ade158a, 34 },
    { 0, 0 },
    { 0xe64e158c, 95 },
    { 0x461e158c, 85 },
    { 0, 0 },
    { 0x6bae158f, 95 },
    { 0xe9ae158f, 43 },
    { 0xfe6e1591, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe1597, 76 },
    { 0xf8fe1597, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee15a2, 33 },
    { 0xf74e15a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1e15a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21e15a9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e15b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476e15b7, 33 },
    { 0, 0 },
    { 0x9a3e15b9, 23 },
    { 0x2a3e15ba, 95 },
    { 0, 0 },
    { 0xad6e15bc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89e15bf, 98 },
    { 0x73de15bf, 95 },
    { 0xed4e15c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe15c4, 33 },
    { 0xfe1e15c5, 90 },
    { 0x17fe15c6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754e15ce, 33 },
    { 0x64ce15cf, 76 },
    { 0x27ae15d0, 58 },
    { 0x110e15d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e15d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e15db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e15e2, 84 },
    { 0xdf1e15e2, 36 },
    { 0, 0 },
    { 0x58fe15e5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ee15e9, 33 },
    { 0xb20e15ea, 95 },
    { 0x381e15eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560e15f0, 95 },
    { 0x1bae15f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15e15fc, 84 },
    { 0, 0 },
    { 0x136e15fe, 98 },
    { 0, 0 },
    { 0x9dae1600, 91 },
    { 0x532e1601, 34 },
    { 0, 0 },
    { 0xfc0e1603, 90 },
    { 0xaf1e1604, 90 },
    { 0xb65e1605, 46 },
    { 0x7b6e1604, 23 },
    { 0x813e1606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe160b, 55 },
    { 0x227e160c, 90 },
    { 0xeb4e160d, 93 },
    { 0x948e160d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee1613, 23 },
    { 0xd15e1614, 90 },
    { 0xec7e1614, 38 },
    { 0, 0 },
    { 0x838e1617, 33 },
    { 0, 0 },
    { 0x4aae1619, 55 },
    { 0x2cfe161a, 23 },
    { 0, 0 },
    { 0x4f9e161c, 54 },
    { 0, 0 },
    { 0x5fee161e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4e1621, 95 },
    { 0, 0 },
    { 0xaf2e1623, 98 },
    { 0, 0 },
    { 0xc9ee1625, 90 },
    { 0, 0 },
    { 0x25ce1627, 23 },
    { 0, 0 },
    { 0x8e1e1629, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ee162f, 76 },
    { 0, 0 },
    { 0x92be1631, 90 },
    { 0xbb8e1632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x139e1635, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e163a, 34 },
    { 0, 0 },
    { 0x5fde163c, 90 },
    { 0xb8de163c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee1646, 91 },
    { 0, 0 },
    { 0xccee1648, 68 },
    { 0x7ffe1649, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fee1650, 35 },
    { 0x724e1650, 99 },
    { 0, 0 },
    { 0x482e1653, 76 },
    { 0, 0 },
    { 0xf54e1655, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8e1659, 33 },
    { 0xef6e165a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579e165e, 34 },
    { 0, 0 },
    { 0x5c8e1660, 23 },
    { 0, 0 },
    { 0xb18e1662, 100 },
    { 0, 0 },
    { 0xf10e1664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x985e1667, 91 },
    { 0x148e1667, 61 },
    { 0, 0 },
    { 0x333e166a, 33 },
    { 0x7cae166b, 61 },
    { 0xa6de166c, 78 },
    { 0xa0be166b, 90 },
    { 0xca5e166d, 46 },
    { 0xc67e166f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e1672, 55 },
    { 0, 0 },
    { 0x93ae1674, 2 },
    { 0x4f8e1675, 33 },
    { 0x70be1676, 95 },
    { 0x2aae1677, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86e1681, 55 },
    { 0, 0 },
    { 0xb8ae1683, 43 },
    { 0x42fe1684, 34 },
    { 0x286e1685, 88 },
    { 0x33be1685, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x663e1689, 76 },
    { 0, 0 },
    { 0x513e168b, 98 },
    { 0xede168b, 99 },
    { 0x9f6e168b, 61 },
    { 0, 0 },
    { 0x5d6e168f, 34 },
    { 0x848e1690, 99 },
    { 0x1c3e1691, 95 },
    { 0, 0 },
    { 0xf72e1693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739e1697, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e169c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e16a2, 90 },
    { 0, 0 },
    { 0x983e16a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe16a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ee16ab, 95 },
    { 0, 0 },
    { 0xdc9e16ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79e16b2, 90 },
    { 0x9cde16b3, 33 },
    { 0x509e16b4, 23 },
    { 0x942e16b4, 90 },
    { 0xbbbe16b6, 87 },
    { 0xc4fe16b6, 90 },
    { 0x862e16b8, 46 },
    { 0x81ce16b8, 67 },
    { 0, 0 },
    { 0x1fde16bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6e16be, 55 },
    { 0xde8e16bf, 40 },
    { 0xf69e16be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30e16c3, 95 },
    { 0x236e16c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be16cb, 76 },
    { 0x7c4e16cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x275e16cf, 84 },
    { 0x852e16cf, 90 },
    { 0, 0 },
    { 0x80de16d2, 68 },
    { 0x50ee16d3, 61 },
    { 0xb7be16d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae16d7, 91 },
    { 0xd6e16d8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe16db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1e16df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6e16e5, 61 },
    { 0, 0 },
    { 0x27be16e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e16ea, 61 },
    { 0, 0 },
    { 0xa54e16ec, 78 },
    { 0, 0 },
    { 0x6c7e16ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce16f1, 76 },
    { 0xe1e16f2, 55 },
    { 0x9cae16f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4e16f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x16de16fb, 81 },
    { 0x6f5e16fc, 23 },
    { 0x5e16fd, 90 },
    { 0xf95e16fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae1701, 84 },
    { 0, 0 },
    { 0xb75e1703, 78 },
    { 0x773e1704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5e1707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce1715, 90 },
    { 0, 0 },
    { 0xa9fe1717, 61 },
    { 0x946e1718, 33 },
    { 0x5c0e1719, 90 },
    { 0xff8e171a, 90 },
    { 0xcd5e171b, 90 },
    { 0xe7be171b, 23 },
    { 0x8c1e171c, 76 },
    { 0x421e1719, 84 },
    { 0x955e171f, 98 },
    { 0x849e1720, 99 },
    { 0xf30e171f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e1725, 69 },
    { 0xcf6e1726, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e172a, 84 },
    { 0xb2ae172b, 90 },
    { 0x8ebe172c, 76 },
    { 0, 0 },
    { 0xc1de172e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773e1732, 23 },
    { 0x64de1733, 95 },
    { 0xa5ce1734, 33 },
    { 0x491e1732, 90 },
    { 0xda0e1736, 34 },
    { 0xe0ee1737, 47 },
    { 0xb48e1738, 91 },
    { 0xd60e1738, 91 },
    { 0x500e1738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6e1740, 46 },
    { 0x2aae1741, 84 },
    { 0, 0 },
    { 0xe43e1743, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae1746, 95 },
    { 0x594e1746, 17 },
    { 0, 0 },
    { 0xebde1749, 98 },
    { 0x84be174a, 33 },
    { 0xd25e174a, 78 },
    { 0x6fde174c, 61 },
    { 0xeb3e174a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe1753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946e1757, 66 },
    { 0, 0 },
    { 0x8afe1759, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974e1762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e1767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9e176a, 61 },
    { 0x9cde176a, 67 },
    { 0x1fee176b, 95 },
    { 0x768e176c, 33 },
    { 0x181e176b, 95 },
    { 0xefbe176a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x676e1772, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de1779, 100 },
    { 0x6e2e177a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09e177e, 90 },
    { 0x7cce177f, 23 },
    { 0, 0 },
    { 0x7e5e1781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebe1785, 86 },
    { 0x5afe1786, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69de178a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e1793, 23 },
    { 0, 0 },
    { 0x913e1795, 76 },
    { 0, 0 },
    { 0x36ee1797, 61 },
    { 0xe75e1797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e17a1, 77 },
    { 0x158e17a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ce17a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x425e17a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9e17ae, 33 },
    { 0x5c4e17af, 23 },
    { 0x803e17b0, 90 },
    { 0x20fe17b0, 33 },
    { 0x91be17b2, 23 },
    { 0x440e17b3, 76 },
    { 0x40ce17b4, 90 },
    { 0xf70e17ae, 43 },
    { 0x9eae17b0, 44 },
    { 0, 0 },
    { 0x70ae17b8, 93 },
    { 0, 0 },
    { 0xfc7e17ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe17c3, 97 },
    { 0x991e17c4, 55 },
    { 0x5c5e17c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde17c9, 43 },
    { 0xbce17ca, 99 },
    { 0x6f0e17cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59e17d1, 66 },
    { 0x482e17d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5e17d8, 95 },
    { 0, 0 },
    { 0xc5e17da, 80 },
    { 0x2d0e17db, 90 },
    { 0x8fae17dc, 46 },
    { 0xd69e17db, 87 },
    { 0xc13e17da, 33 },
    { 0x3fbe17dd, 43 },
    { 0xc0fe17de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317e17e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x764e17e7, 78 },
    { 0x8f4e17e8, 99 },
    { 0, 0 },
    { 0x8f3e17ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e17ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de17f0, 85 },
    { 0x852e17f1, 43 },
    { 0x623e17f0, 95 },
    { 0x4cee17f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e17fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ade1804, 61 },
    { 0x63de1804, 93 },
    { 0xa57e1806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe180a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cce1810, 46 },
    { 0xb07e1811, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4e1815, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e181b, 17 },
    { 0xd53e181b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5e1825, 33 },
    { 0xd21e1826, 98 },
    { 0xbc8e1826, 84 },
    { 0xd83e1828, 87 },
    { 0, 0 },
    { 0x273e182a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47e182e, 90 },
    { 0, 0 },
    { 0x975e1830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288e1837, 33 },
    { 0x18ee1837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22be183e, 61 },
    { 0x59e183f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33e1845, 61 },
    { 0, 0 },
    { 0x1fce1847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e184e, 61 },
    { 0x183e184f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ce1855, 61 },
    { 0, 0 },
    { 0x35ce1857, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae1860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e1865, 81 },
    { 0x1ce1866, 95 },
    { 0x5fbe1866, 61 },
    { 0, 0 },
    { 0x339e1869, 98 },
    { 0x3d8e186a, 90 },
    { 0x7fce186a, 67 },
    { 0xdbde1869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x259e186f, 34 },
    { 0, 0 },
    { 0x76ae1871, 98 },
    { 0xd1be1872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e187f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae1884, 90 },
    { 0xcc6e1885, 76 },
    { 0xd4e1884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e1890, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ee1897, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee189a, 90 },
    { 0xa3ee189b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2e189f, 46 },
    { 0xca4e189f, 23 },
    { 0, 0 },
    { 0x16e18a2, 61 },
    { 0x887e18a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ce18ac, 40 },
    { 0x912e18ad, 90 },
    { 0x290e18ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e18b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871e18bd, 33 },
    { 0xf02e18bd, 43 },
    { 0x7bde18bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce18c2, 66 },
    { 0x482e18c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce18c9, 95 },
    { 0xa8fe18ca, 55 },
    { 0x1dce18cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe18d2, 81 },
    { 0, 0 },
    { 0x740e18d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828e18d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87be18dc, 34 },
    { 0x674e18dd, 93 },
    { 0xa4be18de, 23 },
    { 0, 0 },
    { 0xbb4e18e0, 90 },
    { 0x6d5e18e0, 66 },
    { 0xd2ee18e2, 77 },
    { 0, 0 },
    { 0xd0fe18e4, 66 },
    { 0x932e18e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bce18ea, 95 },
    { 0, 0 },
    { 0x293e18ec, 78 },
    { 0, 0 },
    { 0x2cae18ee, 17 },
    { 0xc31e18ef, 93 },
    { 0x199e18f0, 55 },
    { 0xf25e18f0, 22 },
    { 0, 0 },
    { 0xabde18f3, 61 },
    { 0xe42e18f4, 77 },
    { 0xbfe18f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e18f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e18fb, 34 },
    { 0xb5fe18fc, 52 },
    { 0x207e18fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x872e1900, 90 },
    { 0, 0 },
    { 0xe7de1902, 33 },
    { 0xedfe1903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e1906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e190a, 55 },
    { 0xf8ae190b, 46 },
    { 0xfaae190a, 76 },
    { 0, 0 },
    { 0x10ae190e, 55 },
    { 0x215e190f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e1912, 23 },
    { 0x397e1913, 76 },
    { 0xeee1913, 78 },
    { 0, 0 },
    { 0xb28e1916, 90 },
    { 0, 0 },
    { 0xfe8e1918, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09e1921, 88 },
    { 0x386e1921, 55 },
    { 0x84be1923, 34 },
    { 0x300e1923, 46 },
    { 0xc21e1925, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x594e1928, 93 },
    { 0x6ace1928, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae192d, 23 },
    { 0, 0 },
    { 0xde5e192f, 65 },
    { 0x137e1930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2de1934, 99 },
    { 0x280e1935, 33 },
    { 0xdabe1936, 46 },
    { 0xa26e1937, 90 },
    { 0, 0 },
    { 0x15fe1939, 81 },
    { 0, 0 },
    { 0x18be193b, 66 },
    { 0xc14e193c, 100 },
    { 0x42fe193c, 95 },
    { 0xb82e193e, 33 },
    { 0, 0 },
    { 0x38be1940, 98 },
    { 0x886e1941, 55 },
    { 0x75de1942, 52 },
    { 0x100e1940, 34 },
    { 0x343e1944, 95 },
    { 0x726e1945, 33 },
    { 0x508e1941, 99 },
    { 0xcade1940, 33 },
    { 0xf6ce1948, 23 },
    { 0xeeee1948, 84 },
    { 0x5cfe1942, 95 },
    { 0xd14e1942, 76 },
    { 0xdcee1941, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e194f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be1952, 97 },
    { 0x9dee1953, 33 },
    { 0xefae1953, 95 },
    { 0xacae1954, 33 },
    { 0, 0 },
    { 0x40ee1957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63e195c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51e1961, 81 },
    { 0x692e1961, 33 },
    { 0xf12e1963, 78 },
    { 0xe95e1964, 52 },
    { 0xe5be1963, 87 },
    { 0x32be1964, 95 },
    { 0x66e1965, 23 },
    { 0x632e1968, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x197e196b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae1970, 95 },
    { 0, 0 },
    { 0xa43e1972, 55 },
    { 0, 0 },
    { 0x935e1974, 46 },
    { 0xaf1e1974, 61 },
    { 0x4ee1975, 43 },
    { 0, 0 },
    { 0xcd1e1978, 46 },
    { 0x58be1979, 17 },
    { 0x158e1979, 90 },
    { 0x952e197a, 67 },
    { 0, 0 },
    { 0x3d6e197d, 33 },
    { 0x296e197e, 33 },
    { 0xbb9e197f, 68 },
    { 0, 0 },
    { 0x7b6e1981, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ce1984, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde1987, 66 },
    { 0x36ce1988, 68 },
    { 0x107e1989, 95 },
    { 0x77ce1987, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592e198e, 90 },
    { 0x946e198e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bce199b, 34 },
    { 0, 0 },
    { 0x9ce199d, 98 },
    { 0x903e199e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607e19a4, 46 },
    { 0x5cde19a4, 61 },
    { 0x402e19a6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e19ae, 90 },
    { 0x1ae19af, 69 },
    { 0xfae19b0, 90 },
    { 0xf9ce19b1, 99 },
    { 0x3e6e19b2, 95 },
    { 0, 0 },
    { 0xe02e19b4, 85 },
    { 0x67ae19b5, 75 },
    { 0x86ae19b5, 33 },
    { 0x9e6e19b7, 23 },
    { 0x12de19b8, 61 },
    { 0xa58e19b9, 95 },
    { 0xb45e19b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be19bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x758e19c0, 95 },
    { 0xc2be19c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79e19c7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e19cb, 78 },
    { 0, 0 },
    { 0x5e5e19cd, 98 },
    { 0, 0 },
    { 0xe63e19cf, 98 },
    { 0, 0 },
    { 0xd9de19d1, 33 },
    { 0x208e19d2, 55 },
    { 0x467e19d3, 90 },
    { 0x57be19d4, 90 },
    { 0, 0 },
    { 0x4b4e19d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e19db, 23 },
    { 0x8cfe19dc, 98 },
    { 0x752e19dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be19e2, 98 },
    { 0x817e19e3, 77 },
    { 0, 0 },
    { 0x636e19e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x405e19e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e19eb, 23 },
    { 0x7b7e19ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e19ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee19f2, 61 },
    { 0x80be19f3, 95 },
    { 0x32fe19f2, 99 },
    { 0x5f5e19f5, 78 },
    { 0, 0 },
    { 0x55e19f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e19fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae1a04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e1a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e1a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ce1a14, 90 },
    { 0x560e1a15, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97e1a18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e1a1d, 93 },
    { 0, 0 },
    { 0xf9be1a1f, 23 },
    { 0, 0 },
    { 0xa64e1a21, 100 },
    { 0, 0 },
    { 0x9dae1a23, 33 },
    { 0x232e1a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce1a2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550e1a30, 90 },
    { 0x262e1a31, 55 },
    { 0xf42e1a32, 98 },
    { 0, 0 },
    { 0xd84e1a34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de1a3d, 84 },
    { 0xea7e1a3e, 76 },
    { 0x48fe1a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e1a46, 61 },
    { 0, 0 },
    { 0x493e1a48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde1a56, 90 },
    { 0, 0 },
    { 0xf9ae1a58, 99 },
    { 0xd7de1a58, 98 },
    { 0xb14e1a5a, 85 },
    { 0x6a0e1a5a, 95 },
    { 0x91ee1a5c, 33 },
    { 0x89be1a5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2e1a64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e1a73, 46 },
    { 0xfbde1a74, 90 },
    { 0xa2e1a73, 93 },
    { 0xeffe1a76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607e1a7d, 61 },
    { 0x3a6e1a7e, 95 },
    { 0xc1ce1a7d, 55 },
    { 0x676e1a80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e1a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95be1a86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e1a8a, 61 },
    { 0, 0 },
    { 0x667e1a8c, 61 },
    { 0x721e1a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e1a91, 33 },
    { 0x1b0e1a92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6e1a95, 61 },
    { 0x35de1a96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33be1a9e, 55 },
    { 0, 0 },
    { 0xba4e1aa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83de1aa5, 33 },
    { 0x215e1aa6, 43 },
    { 0, 0 },
    { 0x2a7e1aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ae1aab, 87 },
    { 0x2fae1aac, 43 },
    { 0xad6e1aab, 61 },
    { 0xe41e1aae, 33 },
    { 0, 0 },
    { 0x819e1ab0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e1ab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe1ab7, 56 },
    { 0xcf4e1ab8, 46 },
    { 0x2ade1ab9, 61 },
    { 0x4d6e1ab8, 66 },
    { 0x97ce1ab8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e1abe, 66 },
    { 0x305e1abf, 90 },
    { 0, 0 },
    { 0xc4ce1ac1, 93 },
    { 0, 0 },
    { 0x548e1ac3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ae1ac6, 55 },
    { 0xffce1ac6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x808e1aca, 23 },
    { 0xbebe1acb, 98 },
    { 0x4b6e1acb, 33 },
    { 0x4b9e1acd, 61 },
    { 0xbc8e1ace, 76 },
    { 0xf7ae1acf, 90 },
    { 0, 0 },
    { 0xd94e1ad1, 61 },
    { 0xd27e1ad2, 90 },
    { 0xbb4e1ad3, 98 },
    { 0xfd5e1ad3, 33 },
    { 0, 0 },
    { 0xa2ae1ad6, 95 },
    { 0, 0 },
    { 0xe04e1ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28be1ade, 36 },
    { 0x7f4e1ade, 46 },
    { 0x9e6e1ae0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aae1ae5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace1aea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3be1aed, 34 },
    { 0x5d4e1aee, 38 },
    { 0x3cce1aef, 61 },
    { 0, 0 },
    { 0xa02e1af1, 23 },
    { 0, 0 },
    { 0x18de1af3, 43 },
    { 0xbace1af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e1afa, 76 },
    { 0x145e1afa, 43 },
    { 0, 0 },
    { 0xdc7e1afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57e1b02, 96 },
    { 0, 0 },
    { 0xbf7e1b04, 33 },
    { 0x75e1b05, 23 },
    { 0xb3ce1b04, 23 },
    { 0xeeae1b07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebe1b0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e1b14, 33 },
    { 0xe28e1b14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fe1b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0e1b20, 52 },
    { 0x1cfe1b20, 46 },
    { 0, 0 },
    { 0xc95e1b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ce1b28, 61 },
    { 0xf26e1b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75e1b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2e1b2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5e1b32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e1b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e1b3a, 34 },
    { 0, 0 },
    { 0x879e1b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43e1b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10e1b44, 95 },
    { 0x31e1b45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ce1b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadee1b4c, 90 },
    { 0x53ae1b4c, 80 },
    { 0x5b8e1b4e, 90 },
    { 0, 0 },
    { 0x35ce1b50, 84 },
    { 0x270e1b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeae1b55, 61 },
    { 0x508e1b56, 93 },
    { 0x8cde1b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fe1b5f, 36 },
    { 0, 0 },
    { 0x42ae1b61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53e1b68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be1b6b, 99 },
    { 0, 0 },
    { 0xa89e1b6d, 52 },
    { 0, 0 },
    { 0xf65e1b6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e1b73, 55 },
    { 0x2b6e1b73, 95 },
    { 0xe24e1b73, 46 },
    { 0x706e1b76, 98 },
    { 0xf21e1b77, 90 },
    { 0, 0 },
    { 0xd6e1b79, 23 },
    { 0, 0 },
    { 0xb74e1b7b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58e1b82, 90 },
    { 0x233e1b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedae1b87, 55 },
    { 0xe83e1b88, 67 },
    { 0x9a1e1b89, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e1b8d, 95 },
    { 0, 0 },
    { 0xb37e1b8f, 90 },
    { 0x7a9e1b8f, 33 },
    { 0xf8fe1b8f, 67 },
    { 0x365e1b92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e1b95, 61 },
    { 0xc4e1b96, 34 },
    { 0x2d3e1b97, 33 },
    { 0x944e1b98, 90 },
    { 0xdd4e1b95, 99 },
    { 0xe97e1b9a, 95 },
    { 0x6f2e1b9b, 90 },
    { 0x4c8e1b9c, 98 },
    { 0x13ae1b9a, 95 },
    { 0x73e1b9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e1ba8, 23 },
    { 0x4e7e1ba8, 23 },
    { 0, 0 },
    { 0xee8e1bab, 98 },
    { 0, 0 },
    { 0xf83e1bad, 77 },
    { 0x388e1bad, 98 },
    { 0x84de1bae, 61 },
    { 0xa20e1bb0, 33 },
    { 0x361e1bb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e1bb4, 33 },
    { 0, 0 },
    { 0x9bbe1bb6, 23 },
    { 0xc44e1bb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e1bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79e1bc6, 81 },
    { 0xc46e1bc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ae1bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e1bcd, 98 },
    { 0xf52e1bce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e1bd1, 98 },
    { 0xae9e1bd2, 87 },
    { 0xd49e1bd3, 23 },
    { 0, 0 },
    { 0xbb7e1bd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e1bd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53e1bdc, 87 },
    { 0x9f9e1bdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6e1be1, 34 },
    { 0xab3e1be2, 23 },
    { 0x6fce1be3, 90 },
    { 0, 0 },
    { 0xd71e1be5, 61 },
    { 0, 0 },
    { 0xb90e1be7, 55 },
    { 0x5b2e1be8, 98 },
    { 0x854e1be9, 90 },
    { 0x56ce1be7, 38 },
    { 0, 0 },
    { 0x8dee1bec, 91 },
    { 0xae7e1bec, 55 },
    { 0x16ae1bec, 100 },
    { 0, 0 },
    { 0x953e1bf0, 90 },
    { 0x978e1bf0, 38 },
    { 0x81fe1bf0, 46 },
    { 0x145e1bf3, 95 },
    { 0x415e1bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fe1bf7, 55 },
    { 0, 0 },
    { 0xc82e1bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e1bfc, 23 },
    { 0x51ae1bfc, 55 },
    { 0xb8ae1bfd, 23 },
    { 0xfefe1bff, 55 },
    { 0x68ae1bfd, 23 },
    { 0x31ae1c01, 95 },
    { 0x7cbe1c00, 95 },
    { 0, 0 },
    { 0xa32e1c04, 33 },
    { 0x37de1c04, 33 },
    { 0x1be1c06, 67 },
    { 0, 0 },
    { 0xec6e1c08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6e1c0c, 17 },
    { 0x96ae1c0c, 88 },
    { 0x3bfe1c0e, 85 },
    { 0, 0 },
    { 0xc3fe1c10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1e1c18, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae1c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07e1c25, 100 },
    { 0xa23e1c26, 40 },
    { 0x89de1c26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243e1c2c, 90 },
    { 0xe8fe1c2d, 67 },
    { 0x725e1c2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fe1c33, 98 },
    { 0x98fe1c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e1c38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ade1c3b, 90 },
    { 0x138e1c3c, 46 },
    { 0x251e1c3d, 85 },
    { 0x9b2e1c3c, 61 },
    { 0x136e1c3e, 99 },
    { 0xb43e1c40, 33 },
    { 0x3e9e1c40, 88 },
    { 0x38de1c3e, 55 },
    { 0x388e1c43, 33 },
    { 0xd2ee1c44, 55 },
    { 0x464e1c45, 95 },
    { 0x9c1e1c46, 55 },
    { 0xce2e1c45, 98 },
    { 0, 0 },
    { 0x96de1c49, 95 },
    { 0, 0 },
    { 0x4e1c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee1c50, 69 },
    { 0, 0 },
    { 0xc89e1c52, 61 },
    { 0xf23e1c53, 23 },
    { 0x9a9e1c54, 98 },
    { 0x661e1c54, 95 },
    { 0, 0 },
    { 0xd7de1c57, 43 },
    { 0x3d6e1c58, 99 },
    { 0xbaae1c57, 90 },
    { 0x597e1c5a, 61 },
    { 0xb75e1c5a, 33 },
    { 0x315e1c58, 76 },
    { 0x26ae1c5a, 90 },
    { 0xdabe1c5e, 17 },
    { 0x289e1c5e, 80 },
    { 0x8bde1c60, 34 },
    { 0x3ae1c61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de1c67, 96 },
    { 0xfe1c68, 101 },
    { 0x91be1c67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1c6e, 76 },
    { 0, 0 },
    { 0x73be1c70, 40 },
    { 0xe2ee1c71, 33 },
    { 0xcffe1c72, 65 },
    { 0x361e1c71, 98 },
    { 0x779e1c74, 61 },
    { 0x398e1c74, 98 },
    { 0x73ae1c74, 33 },
    { 0, 0 },
    { 0x580e1c78, 55 },
    { 0xf59e1c79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07e1c83, 99 },
    { 0, 0 },
    { 0x4d7e1c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1c8a, 23 },
    { 0x1e0e1c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78be1c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79e1c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eee1c98, 87 },
    { 0, 0 },
    { 0x736e1c9a, 90 },
    { 0x6bfe1c9a, 61 },
    { 0x49e1c9c, 95 },
    { 0xf44e1c9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8e1ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159e1ca8, 66 },
    { 0, 0 },
    { 0xf0ee1caa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e1cb0, 12 },
    { 0x39ce1cb1, 23 },
    { 0xa2e1cb1, 55 },
    { 0x690e1cb3, 23 },
    { 0x8f9e1cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de1cba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e1cbf, 61 },
    { 0xabfe1cbf, 90 },
    { 0x2cae1cc1, 43 },
    { 0xcf3e1cc2, 52 },
    { 0x729e1cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e1cc7, 76 },
    { 0, 0 },
    { 0xa93e1cc9, 23 },
    { 0x15be1cca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37e1ccd, 98 },
    { 0, 0 },
    { 0x3ee1ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e1cd3, 98 },
    { 0x3d1e1cd3, 77 },
    { 0x8aae1cd3, 90 },
    { 0, 0 },
    { 0xd5fe1cd7, 33 },
    { 0xbae1cd8, 76 },
    { 0x607e1cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88de1cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1e1ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e1ce9, 33 },
    { 0x836e1cea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aae1ced, 91 },
    { 0xee8e1cee, 46 },
    { 0, 0 },
    { 0x777e1cf0, 61 },
    { 0x9bde1cf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be1cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77e1cfb, 33 },
    { 0x6f7e1cfc, 93 },
    { 0x1eee1cfd, 90 },
    { 0xd6fe1cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80e1d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe1d0c, 95 },
    { 0xf75e1d0d, 76 },
    { 0xacfe1d0e, 98 },
    { 0xd13e1d0f, 61 },
    { 0x9f6e1d10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47e1d15, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624e1d1b, 23 },
    { 0, 0 },
    { 0x9c1e1d1d, 90 },
    { 0x1a2e1d1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15e1d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e1d2c, 78 },
    { 0, 0 },
    { 0x818e1d2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6e1d37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65e1d3a, 95 },
    { 0xf5e1d3b, 66 },
    { 0xfc6e1d3a, 33 },
    { 0xbd1e1d3d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551e1d41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69e1d50, 34 },
    { 0x2c3e1d51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ae1d54, 34 },
    { 0, 0 },
    { 0xaaee1d56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebe1d5a, 33 },
    { 0, 0 },
    { 0x69ae1d5c, 90 },
    { 0xd44e1d5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156e1d61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce1d65, 95 },
    { 0, 0 },
    { 0x696e1d67, 85 },
    { 0x7fce1d67, 66 },
    { 0x7e0e1d67, 76 },
    { 0x19ce1d67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44e1d74, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5de1d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713e1d80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ae1d83, 61 },
    { 0, 0 },
    { 0x74be1d85, 98 },
    { 0x70de1d85, 23 },
    { 0x333e1d87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847e1d8c, 90 },
    { 0, 0 },
    { 0xaafe1d8e, 90 },
    { 0, 0 },
    { 0xa01e1d90, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58e1d93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e1d97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce1da3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5e1da7, 23 },
    { 0xcf6e1da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabce1db1, 61 },
    { 0xc47e1db2, 33 },
    { 0x439e1db2, 100 },
    { 0, 0 },
    { 0xa55e1db5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abe1db9, 95 },
    { 0x988e1db9, 98 },
    { 0x329e1dba, 99 },
    { 0, 0 },
    { 0x15ae1dbd, 23 },
    { 0, 0 },
    { 0x8a8e1dbf, 93 },
    { 0xc94e1dc0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade1dc6, 90 },
    { 0x4e1dc6, 61 },
    { 0, 0 },
    { 0x294e1dc9, 94 },
    { 0xdf0e1dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e1dce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e1dd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e1ddd, 90 },
    { 0x586e1dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ce1de1, 85 },
    { 0x9fee1de2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e1de6, 98 },
    { 0x52fe1de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbe1dec, 33 },
    { 0x5aae1ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3e1df4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x859e1df7, 33 },
    { 0, 0 },
    { 0xbcbe1df9, 61 },
    { 0, 0 },
    { 0xb69e1dfb, 33 },
    { 0, 0 },
    { 0x46fe1dfd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e1e03, 61 },
    { 0x70be1e04, 9 },
    { 0x9d2e1e04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae1e09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ce1e0f, 23 },
    { 0xadae1e0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8e1e16, 100 },
    { 0x7afe1e17, 61 },
    { 0x965e1e18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbe1e24, 90 },
    { 0xb77e1e25, 90 },
    { 0xaece1e26, 61 },
    { 0x929e1e27, 95 },
    { 0xad3e1e28, 90 },
    { 0x54fe1e29, 90 },
    { 0x893e1e27, 61 },
    { 0x74be1e25, 95 },
    { 0xf25e1e2c, 95 },
    { 0x572e1e2d, 61 },
    { 0x30e1e2e, 84 },
    { 0x382e1e2d, 34 },
    { 0xa2ee1e29, 76 },
    { 0x767e1e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e1e34, 33 },
    { 0x28e1e34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x459e1e38, 98 },
    { 0x33fe1e39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee1e43, 61 },
    { 0x7fe1e43, 98 },
    { 0x433e1e44, 22 },
    { 0x3ede1e46, 61 },
    { 0, 0 },
    { 0x316e1e48, 61 },
    { 0, 0 },
    { 0x33ae1e4a, 53 },
    { 0x62fe1e4a, 23 },
    { 0x9f7e1e4c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fe1e50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e1e56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfe1e59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x352e1e5c, 23 },
    { 0, 0 },
    { 0x917e1e5e, 98 },
    { 0x74ae1e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e1e65, 33 },
    { 0x16ae1e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4e1e73, 93 },
    { 0xa81e1e74, 90 },
    { 0, 0 },
    { 0xe79e1e76, 95 },
    { 0x282e1e76, 98 },
    { 0, 0 },
    { 0xd8de1e79, 97 },
    { 0x3dce1e79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee1e86, 90 },
    { 0xbd7e1e86, 87 },
    { 0x4c8e1e88, 90 },
    { 0x590e1e86, 23 },
    { 0x14ee1e86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437e1e8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de1e98, 90 },
    { 0, 0 },
    { 0x26fe1e9a, 43 },
    { 0x940e1e9b, 81 },
    { 0x449e1e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41e1ea0, 88 },
    { 0x312e1ea0, 93 },
    { 0x451e1ea0, 55 },
    { 0xcb2e1ea3, 78 },
    { 0, 0 },
    { 0xad6e1ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12de1eac, 93 },
    { 0x9eae1eac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ece1eb0, 90 },
    { 0x733e1eb1, 33 },
    { 0x10fe1eb2, 43 },
    { 0xb88e1eb3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e1eba, 76 },
    { 0x4d2e1eba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6e1ebe, 96 },
    { 0xb3be1ebe, 85 },
    { 0, 0 },
    { 0xa73e1ec1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e1ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ce1ecb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee1ecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be1ed3, 93 },
    { 0xcc0e1ed4, 95 },
    { 0x8bce1ed5, 95 },
    { 0, 0 },
    { 0x982e1ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce1edd, 95 },
    { 0x9dce1ede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e1ee1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de1ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492e1eef, 34 },
    { 0, 0 },
    { 0x9f0e1ef1, 90 },
    { 0, 0 },
    { 0x268e1ef3, 61 },
    { 0, 0 },
    { 0x18ee1ef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be1f03, 90 },
    { 0x5a8e1f03, 43 },
    { 0xcd0e1f05, 99 },
    { 0xb55e1f06, 89 },
    { 0xe91e1f06, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade1f0a, 33 },
    { 0xd65e1f0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ee1f10, 43 },
    { 0, 0 },
    { 0xb12e1f12, 76 },
    { 0, 0 },
    { 0xeaee1f14, 85 },
    { 0x99e1f15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae1f1a, 34 },
    { 0x665e1f1b, 33 },
    { 0xe90e1f1a, 38 },
    { 0x2a4e1f1d, 98 },
    { 0, 0 },
    { 0x66ce1f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ee1f22, 52 },
    { 0xa64e1f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fe1f26, 95 },
    { 0x2ade1f27, 98 },
    { 0xe87e1f28, 33 },
    { 0x798e1f26, 23 },
    { 0xf2e1f2a, 90 },
    { 0xcf0e1f2b, 95 },
    { 0, 0 },
    { 0x7f0e1f2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725e1f31, 34 },
    { 0, 0 },
    { 0x1ebe1f33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e1f3b, 43 },
    { 0x576e1f3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e1f3f, 100 },
    { 0x7bde1f3f, 34 },
    { 0xa4fe1f41, 55 },
    { 0x4e2e1f41, 99 },
    { 0x500e1f43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x426e1f46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e1f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x628e1f4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ce1f51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e1f57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1e1f5e, 95 },
    { 0x32e1f5f, 61 },
    { 0, 0 },
    { 0x83fe1f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1e1f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde1f67, 75 },
    { 0x7fae1f68, 90 },
    { 0, 0 },
    { 0x50e1f6a, 61 },
    { 0, 0 },
    { 0x252e1f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e1f6f, 33 },
    { 0x3e3e1f70, 76 },
    { 0xba4e1f6f, 91 },
    { 0x738e1f70, 92 },
    { 0x7c5e1f71, 61 },
    { 0x97ae1f74, 93 },
    { 0xbfce1f73, 33 },
    { 0x99ae1f72, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ae1f7a, 67 },
    { 0, 0 },
    { 0x40be1f7c, 84 },
    { 0xe08e1f7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0e1f80, 23 },
    { 0, 0 },
    { 0x653e1f82, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235e1f88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde1f8c, 23 },
    { 0x80de1f8d, 95 },
    { 0, 0 },
    { 0x8fe1f8f, 34 },
    { 0xcefe1f90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efe1f93, 52 },
    { 0x72be1f94, 46 },
    { 0xfe1e1f93, 98 },
    { 0x72e1f93, 90 },
    { 0xda5e1f95, 78 },
    { 0, 0 },
    { 0x59fe1f99, 84 },
    { 0, 0 },
    { 0x12fe1f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e1f9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x836e1fa2, 90 },
    { 0x319e1fa3, 33 },
    { 0x9abe1fa4, 43 },
    { 0xadae1fa4, 78 },
    { 0x4b4e1fa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494e1faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97e1fb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6e1fb5, 23 },
    { 0x463e1fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e1fb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bee1fbc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e1fc3, 66 },
    { 0, 0 },
    { 0x6eee1fc5, 81 },
    { 0x340e1fc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5e1fc9, 68 },
    { 0, 0 },
    { 0x684e1fcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee1fd2, 33 },
    { 0, 0 },
    { 0x264e1fd4, 43 },
    { 0xac7e1fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3e1fd9, 90 },
    { 0x526e1fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e1fe0, 84 },
    { 0x3a6e1fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e1fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834e1ff1, 55 },
    { 0x470e1ff1, 43 },
    { 0x4a2e1ff2, 61 },
    { 0, 0 },
    { 0x5fde1ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7e1ffb, 90 },
    { 0, 0 },
    { 0x248e1ffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfe2006, 98 },
    { 0xde7e2006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce200a, 91 },
    { 0x5aae200a, 33 },
    { 0, 0 },
    { 0x25ee200d, 100 },
    { 0xbdbe200e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64e2011, 95 },
    { 0x6fee2012, 90 },
    { 0xe7ee2011, 61 },
    { 0x1c7e2014, 93 },
    { 0, 0 },
    { 0x781e2016, 95 },
    { 0xf0de2017, 61 },
    { 0x4e4e2018, 96 },
    { 0xcbee2019, 76 },
    { 0x9ace2018, 90 },
    { 0xe8fe201b, 43 },
    { 0x8cce201a, 61 },
    { 0x988e201b, 90 },
    { 0xd5ae201e, 61 },
    { 0x903e201e, 85 },
    { 0, 0 },
    { 0x7d1e2021, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaae2027, 90 },
    { 0x175e2027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e2030, 95 },
    { 0x43e2031, 66 },
    { 0x7ede2032, 98 },
    { 0x7f4e2033, 98 },
    { 0xb53e2033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe203b, 95 },
    { 0x746e203b, 61 },
    { 0, 0 },
    { 0x970e203e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be204a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e204f, 95 },
    { 0xb0de2050, 98 },
    { 0, 0 },
    { 0x102e2052, 61 },
    { 0xac0e2052, 61 },
    { 0, 0 },
    { 0x443e2055, 89 },
    { 0x364e2056, 78 },
    { 0x983e2055, 33 },
    { 0x324e2058, 61 },
    { 0, 0 },
    { 0x212e205a, 61 },
    { 0x9ebe205a, 61 },
    { 0x480e205c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e2063, 33 },
    { 0x1d6e2064, 88 },
    { 0x699e2064, 61 },
    { 0x73e2066, 85 },
    { 0, 0 },
    { 0xe70e2068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522e206b, 65 },
    { 0xb6ce206c, 17 },
    { 0x705e206b, 90 },
    { 0x4bbe206b, 93 },
    { 0xd1fe206f, 33 },
    { 0, 0 },
    { 0x934e2071, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e207e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58e2084, 43 },
    { 0xa12e2085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674e208a, 33 },
    { 0x70e208b, 67 },
    { 0, 0 },
    { 0x30de208d, 90 },
    { 0x8d2e208e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fe2093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfde209a, 46 },
    { 0x52e209b, 33 },
    { 0x99de209a, 87 },
    { 0xc05e209d, 33 },
    { 0x70ee209e, 76 },
    { 0, 0 },
    { 0x4e6e20a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x981e20a3, 33 },
    { 0, 0 },
    { 0xc54e20a5, 95 },
    { 0x88de20a6, 43 },
    { 0x743e20a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce20ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ce20af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e20b2, 99 },
    { 0x470e20b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705e20b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6e20c5, 91 },
    { 0, 0 },
    { 0xc3be20c7, 90 },
    { 0, 0 },
    { 0x83be20c9, 55 },
    { 0, 0 },
    { 0xf22e20cb, 93 },
    { 0, 0 },
    { 0x6cbe20cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde20d0, 95 },
    { 0, 0 },
    { 0x7c4e20d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe20d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ae20dc, 61 },
    { 0x910e20dd, 85 },
    { 0xb2de20dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589e20e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9e20e7, 76 },
    { 0x933e20e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9e20eb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e20ee, 37 },
    { 0x961e20ee, 61 },
    { 0xd13e20f0, 97 },
    { 0x66ae20f1, 95 },
    { 0x4aee20f2, 23 },
    { 0xd2e20f3, 43 },
    { 0x9ae20f4, 66 },
    { 0x73be20f1, 43 },
    { 0x7d8e20f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de20f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47e20fd, 23 },
    { 0x366e20fe, 61 },
    { 0xa70e20fd, 34 },
    { 0x2bae2100, 46 },
    { 0x520e2100, 93 },
    { 0xb6ae2102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2e2107, 46 },
    { 0, 0 },
    { 0xdafe2109, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e2110, 90 },
    { 0x3f1e2110, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfe2118, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x346e211b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e211e, 33 },
    { 0xd69e211f, 33 },
    { 0x4f3e2120, 55 },
    { 0x7a1e211f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e2127, 33 },
    { 0xc2ce2128, 61 },
    { 0x8e7e2127, 23 },
    { 0x73ae212a, 98 },
    { 0x63ee212b, 40 },
    { 0xe07e212b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e212f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe2134, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdee213a, 76 },
    { 0, 0 },
    { 0xa5de213c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e213f, 33 },
    { 0xdfee2140, 65 },
    { 0x5aee2140, 61 },
    { 0, 0 },
    { 0x738e2143, 33 },
    { 0xa22e2144, 98 },
    { 0xa88e2145, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13e2149, 46 },
    { 0xc9ae214a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8e214d, 40 },
    { 0x37ae214d, 99 },
    { 0x97de214f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2153, 99 },
    { 0x831e2153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cde2159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e215c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283e2163, 61 },
    { 0x378e2163, 33 },
    { 0xe76e2165, 76 },
    { 0x566e2166, 33 },
    { 0x2d2e2164, 55 },
    { 0, 0 },
    { 0x5d7e2169, 33 },
    { 0, 0 },
    { 0x801e216b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e2170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2e2176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e217c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e2181, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407e2188, 84 },
    { 0, 0 },
    { 0x504e218a, 33 },
    { 0xd01e218b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ee218e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65e2191, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e2194, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820e2198, 43 },
    { 0x5e3e2199, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e21a2, 23 },
    { 0xbc1e21a3, 95 },
    { 0x2a3e21a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e21a8, 90 },
    { 0, 0 },
    { 0x53ee21aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e21ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52be21b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe21b8, 43 },
    { 0x8a4e21b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184e21c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458e21c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e21cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4e21d2, 66 },
    { 0x500e21d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e21d8, 23 },
    { 0, 0 },
    { 0x6f4e21da, 61 },
    { 0x56e21db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbe21e6, 95 },
    { 0xf75e21e7, 36 },
    { 0x9b8e21e8, 61 },
    { 0x351e21e9, 23 },
    { 0xc3be21e9, 100 },
    { 0x760e21e8, 91 },
    { 0xe89e21ec, 78 },
    { 0, 0 },
    { 0xf11e21ee, 23 },
    { 0x73ee21ee, 98 },
    { 0, 0 },
    { 0xee9e21f1, 43 },
    { 0, 0 },
    { 0xd11e21f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e21f8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e21fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ade21fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbe2202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e2209, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71e220f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be221a, 33 },
    { 0x608e221a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfe221f, 61 },
    { 0x3b0e2220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ae2226, 78 },
    { 0x11e2226, 90 },
    { 0x5d8e2228, 33 },
    { 0x9d4e2229, 90 },
    { 0x2ede2229, 23 },
    { 0xf86e222a, 61 },
    { 0xe16e222c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91e222f, 90 },
    { 0x60ae222f, 100 },
    { 0xc9ae2231, 17 },
    { 0, 0 },
    { 0xf91e2233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67be2236, 93 },
    { 0x17fe2237, 33 },
    { 0x269e2238, 76 },
    { 0x739e2238, 33 },
    { 0x7bae2239, 61 },
    { 0, 0 },
    { 0x989e223c, 61 },
    { 0, 0 },
    { 0x720e223e, 23 },
    { 0, 0 },
    { 0xab6e2240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x420e2243, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751e2247, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e224a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e224e, 33 },
    { 0x61de224e, 95 },
    { 0xc8be2250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919e2254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710e225c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e226a, 23 },
    { 0, 0 },
    { 0x9f3e226c, 84 },
    { 0, 0 },
    { 0xf69e226e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e2271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9e2276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3e227a, 40 },
    { 0x278e227a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e227e, 100 },
    { 0, 0 },
    { 0x43de2280, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e2288, 61 },
    { 0, 0 },
    { 0xed2e228a, 33 },
    { 0x8e4e228b, 33 },
    { 0, 0 },
    { 0x1b1e228d, 99 },
    { 0, 0 },
    { 0x168e228f, 78 },
    { 0x902e2290, 95 },
    { 0, 0 },
    { 0xc21e2292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee2296, 43 },
    { 0xbdfe2296, 23 },
    { 0x2dde2297, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e229f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ee22a2, 98 },
    { 0x361e22a3, 69 },
    { 0x3a1e22a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e22a8, 61 },
    { 0x83de22a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3e22ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e22b1, 34 },
    { 0xd94e22b2, 61 },
    { 0, 0 },
    { 0x4f2e22b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde22b8, 68 },
    { 0xc8ee22b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be22bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe22c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce22c5, 33 },
    { 0, 0 },
    { 0x523e22c7, 53 },
    { 0x85fe22c8, 67 },
    { 0x282e22c9, 90 },
    { 0, 0 },
    { 0x20ae22cb, 99 },
    { 0xbc4e22cc, 76 },
    { 0, 0 },
    { 0xd2e22ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbae22d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee22d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709e22e1, 85 },
    { 0, 0 },
    { 0x8ebe22e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41e22e6, 98 },
    { 0x899e22e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae22ea, 33 },
    { 0, 0 },
    { 0x20be22ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e22f2, 66 },
    { 0x4b9e22f3, 43 },
    { 0x71e22f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e22f8, 17 },
    { 0, 0 },
    { 0xe1ce22fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e2300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be2304, 34 },
    { 0xef1e2304, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4e230a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3e2310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e2314, 99 },
    { 0xe73e2315, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89de2318, 34 },
    { 0xd4fe2319, 90 },
    { 0xcd4e231a, 33 },
    { 0x3b5e231a, 85 },
    { 0, 0 },
    { 0xcd5e231d, 84 },
    { 0xeb8e231d, 90 },
    { 0xc43e231e, 95 },
    { 0, 0 },
    { 0x73ae2321, 64 },
    { 0x718e2322, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e2325, 93 },
    { 0x9d0e2326, 98 },
    { 0x9e5e2326, 33 },
    { 0x44ce2328, 91 },
    { 0, 0 },
    { 0x2bfe232a, 55 },
    { 0xb55e232b, 90 },
    { 0x6b5e232c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade2336, 93 },
    { 0x8e5e2337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefe233a, 95 },
    { 0, 0 },
    { 0x33fe233c, 55 },
    { 0, 0 },
    { 0x47ce233e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e2342, 98 },
    { 0x4e2343, 17 },
    { 0xe17e2344, 91 },
    { 0xd94e2345, 39 },
    { 0, 0 },
    { 0x2c4e2347, 33 },
    { 0x5c6e2348, 23 },
    { 0, 0 },
    { 0x786e234a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564e2353, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fe2359, 33 },
    { 0xa3ee235a, 95 },
    { 0xdb3e235b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae235f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1e2366, 76 },
    { 0x25ae2366, 33 },
    { 0xb81e2368, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9e236e, 53 },
    { 0, 0 },
    { 0x759e2370, 66 },
    { 0, 0 },
    { 0xf3e2372, 98 },
    { 0, 0 },
    { 0x511e2374, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe2379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679e2382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e238b, 76 },
    { 0xc4de238b, 34 },
    { 0x3e8e238d, 61 },
    { 0xdcbe238c, 98 },
    { 0, 0 },
    { 0x27be2390, 93 },
    { 0xcce2390, 33 },
    { 0xbd3e2392, 61 },
    { 0x5ffe2391, 55 },
    { 0x64ae2393, 89 },
    { 0, 0 },
    { 0xc37e2396, 95 },
    { 0xfcce2396, 76 },
    { 0x60be2396, 98 },
    { 0, 0 },
    { 0x75ae239a, 95 },
    { 0x1cfe239b, 65 },
    { 0, 0 },
    { 0x71ee239d, 88 },
    { 0xc91e239e, 33 },
    { 0x9a4e239e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade23a5, 43 },
    { 0, 0 },
    { 0xae1e23a7, 91 },
    { 0xebfe23a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e23ac, 95 },
    { 0, 0 },
    { 0x9e7e23ae, 55 },
    { 0xc81e23af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e23b2, 77 },
    { 0, 0 },
    { 0x472e23b4, 33 },
    { 0xcfde23b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dce23b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40e23bd, 75 },
    { 0, 0 },
    { 0x9bce23bf, 90 },
    { 0, 0 },
    { 0xf93e23c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9e23c5, 95 },
    { 0x587e23c6, 76 },
    { 0x317e23c7, 90 },
    { 0x49de23c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fe23ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce23d1, 95 },
    { 0xbd7e23d2, 93 },
    { 0x6aae23d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5de23d6, 76 },
    { 0xf66e23d6, 33 },
    { 0x51ce23d6, 55 },
    { 0x1e0e23d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be23e2, 76 },
    { 0, 0 },
    { 0xfafe23e4, 80 },
    { 0xe3de23e4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e23e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de23ef, 61 },
    { 0x8cae23f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7e23f3, 98 },
    { 0x8ade23f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ee23fb, 60 },
    { 0x59fe23fc, 23 },
    { 0xa78e23fb, 66 },
    { 0x87fe23fe, 90 },
    { 0x6c6e23fe, 55 },
    { 0x3a9e2400, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e2403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e2406, 43 },
    { 0, 0 },
    { 0xb3ce2408, 98 },
    { 0x307e2409, 28 },
    { 0xa70e2409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e2411, 55 },
    { 0x9bbe2412, 76 },
    { 0x998e2413, 23 },
    { 0, 0 },
    { 0x8cae2415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e2418, 34 },
    { 0xfade2419, 93 },
    { 0x6f7e241a, 90 },
    { 0x163e241b, 90 },
    { 0, 0 },
    { 0x676e241d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x316e2420, 55 },
    { 0x68e2421, 46 },
    { 0xf50e2422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5e2425, 43 },
    { 0, 0 },
    { 0x4f8e2427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e242d, 46 },
    { 0x5fae242d, 67 },
    { 0xbbfe242d, 90 },
    { 0x7b9e2430, 87 },
    { 0xb9de2431, 52 },
    { 0x7f7e2431, 34 },
    { 0xd14e2433, 98 },
    { 0xa1de2434, 46 },
    { 0x6d3e2435, 84 },
    { 0x4d3e2432, 17 },
    { 0xa41e2437, 93 },
    { 0, 0 },
    { 0xad5e2439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cae2441, 88 },
    { 0, 0 },
    { 0xd6be2443, 17 },
    { 0x952e2443, 76 },
    { 0x79de2443, 75 },
    { 0, 0 },
    { 0x868e2447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de244a, 47 },
    { 0, 0 },
    { 0x42ae244c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaee2450, 90 },
    { 0x7cfe2450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cde2455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be2459, 34 },
    { 0x6b4e245a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e245e, 66 },
    { 0, 0 },
    { 0x824e2460, 33 },
    { 0xf97e2461, 67 },
    { 0x720e2461, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e2466, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe246b, 95 },
    { 0, 0 },
    { 0x593e246d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1e2471, 95 },
    { 0x85ae2472, 85 },
    { 0xb75e2473, 95 },
    { 0, 0 },
    { 0x669e2475, 76 },
    { 0xb07e2476, 90 },
    { 0xb5ce2476, 61 },
    { 0x24ee2478, 76 },
    { 0, 0 },
    { 0x9abe247a, 66 },
    { 0, 0 },
    { 0x52e247c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dee2483, 23 },
    { 0x763e2484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e2487, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ce248b, 61 },
    { 0x3f2e248c, 93 },
    { 0, 0 },
    { 0xb5be248e, 43 },
    { 0xf06e248e, 76 },
    { 0x1c4e2490, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e2493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de2498, 90 },
    { 0xf8be2499, 77 },
    { 0, 0 },
    { 0x764e249b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbe249e, 99 },
    { 0, 0 },
    { 0xc7ae24a0, 61 },
    { 0xe5be24a1, 5 },
    { 0, 0 },
    { 0x48ae24a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e24a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbee24ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ce24b0, 78 },
    { 0x677e24b1, 34 },
    { 0, 0 },
    { 0x38e24b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbe24b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfade24bc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647e24c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e24c6, 87 },
    { 0xe6ee24c7, 76 },
    { 0x6dbe24c8, 68 },
    { 0, 0 },
    { 0x566e24ca, 92 },
    { 0x932e24cb, 23 },
    { 0xec1e24cc, 99 },
    { 0x441e24cd, 90 },
    { 0xab5e24ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6e24d2, 33 },
    { 0, 0 },
    { 0x16ee24d4, 76 },
    { 0xb17e24d4, 68 },
    { 0xdbde24d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135e24db, 76 },
    { 0xabfe24db, 90 },
    { 0x9a6e24dd, 23 },
    { 0, 0 },
    { 0x24ae24df, 99 },
    { 0, 0 },
    { 0x2afe24e1, 55 },
    { 0, 0 },
    { 0x1f1e24e3, 81 },
    { 0, 0 },
    { 0x821e24e5, 90 },
    { 0, 0 },
    { 0xfeae24e7, 61 },
    { 0x7c2e24e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x560e24eb, 76 },
    { 0, 0 },
    { 0xb1ae24ed, 61 },
    { 0, 0 },
    { 0x45ce24ef, 8 },
    { 0, 0 },
    { 0xa99e24f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cce24f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfe24f9, 33 },
    { 0x3a8e24fa, 90 },
    { 0, 0 },
    { 0xe8e24fc, 23 },
    { 0xe37e24fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ee2501, 93 },
    { 0x114e2502, 76 },
    { 0, 0 },
    { 0xe09e2504, 33 },
    { 0xcd1e2504, 95 },
    { 0x6b7e2506, 33 },
    { 0xd52e2507, 55 },
    { 0x4f6e2508, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e250b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe2514, 95 },
    { 0xa9ae2515, 88 },
    { 0xf76e2516, 89 },
    { 0x79de2517, 91 },
    { 0x53e2516, 35 },
    { 0xfd2e2519, 52 },
    { 0x754e251a, 87 },
    { 0xf85e2515, 33 },
    { 0x548e251c, 33 },
    { 0xe74e251d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ee2524, 61 },
    { 0xc8de2525, 91 },
    { 0x479e2526, 84 },
    { 0x92ee2527, 78 },
    { 0xfd9e2528, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x315e252b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ee2536, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e253c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330e2542, 97 },
    { 0, 0 },
    { 0x412e2544, 99 },
    { 0, 0 },
    { 0xdb4e2546, 33 },
    { 0xfd8e2547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abe254e, 90 },
    { 0x7ee254f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97e2552, 90 },
    { 0x6bae2552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce255a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ae255f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280e256d, 98 },
    { 0, 0 },
    { 0xc4ce256f, 90 },
    { 0, 0 },
    { 0x504e2571, 95 },
    { 0x4ece2571, 33 },
    { 0, 0 },
    { 0x959e2574, 33 },
    { 0, 0 },
    { 0x6ebe2576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e2579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae257f, 91 },
    { 0x790e257f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e2586, 23 },
    { 0x459e2587, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ce258b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e258f, 61 },
    { 0, 0 },
    { 0xb0e2591, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ce2595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace259d, 55 },
    { 0xf54e259d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e25a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9e25ad, 23 },
    { 0x4aae25ae, 61 },
    { 0xba7e25af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409e25b5, 52 },
    { 0xd2fe25b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e25b9, 90 },
    { 0, 0 },
    { 0xc20e25bb, 85 },
    { 0xcbee25bc, 43 },
    { 0, 0 },
    { 0xd1e25be, 52 },
    { 0x1b7e25be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de25c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48de25ca, 87 },
    { 0, 0 },
    { 0x737e25cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e25d1, 43 },
    { 0x858e25d2, 23 },
    { 0x3e4e25d1, 33 },
    { 0, 0 },
    { 0xcafe25d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e25d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9e25e2, 33 },
    { 0xf0ee25e3, 90 },
    { 0, 0 },
    { 0x575e25e5, 17 },
    { 0x27e25e6, 78 },
    { 0x781e25e6, 90 },
    { 0, 0 },
    { 0x927e25e9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3e25ec, 67 },
    { 0xe50e25ed, 95 },
    { 0xb8ce25ee, 91 },
    { 0x500e25ed, 23 },
    { 0xd4ae25f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebe25f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be25f9, 33 },
    { 0xffee25fa, 87 },
    { 0x89e25fb, 23 },
    { 0x714e25f9, 95 },
    { 0x58be25fd, 61 },
    { 0, 0 },
    { 0x8dae25ff, 55 },
    { 0x1ece25ff, 43 },
    { 0, 0 },
    { 0xecee2602, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11e2605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e2608, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e260f, 99 },
    { 0x1a3e260f, 33 },
    { 0xe92e2611, 33 },
    { 0x765e260f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9e2615, 23 },
    { 0, 0 },
    { 0x323e2617, 90 },
    { 0xd6fe2618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61e261d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e2627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe262a, 40 },
    { 0xf85e262b, 80 },
    { 0x7c5e262c, 66 },
    { 0xbb3e262d, 33 },
    { 0x34ee262e, 93 },
    { 0xcc8e262d, 76 },
    { 0, 0 },
    { 0x8f1e2631, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76e2634, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e2637, 66 },
    { 0, 0 },
    { 0xf93e2639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5e263c, 33 },
    { 0x3e9e263c, 68 },
    { 0, 0 },
    { 0x620e263f, 33 },
    { 0x6a8e2640, 90 },
    { 0x7d0e2641, 93 },
    { 0x646e2641, 46 },
    { 0, 0 },
    { 0xcb0e2644, 34 },
    { 0, 0 },
    { 0xf6be2646, 23 },
    { 0x6c0e2646, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee264b, 66 },
    { 0, 0 },
    { 0x210e264d, 23 },
    { 0xbbe264e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97e2654, 33 },
    { 0, 0 },
    { 0x819e2656, 95 },
    { 0, 0 },
    { 0x8d0e2658, 87 },
    { 0x592e2658, 99 },
    { 0xfc4e2659, 87 },
    { 0x40ee2658, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e265e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e2662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde2665, 61 },
    { 0, 0 },
    { 0xbe9e2667, 34 },
    { 0x6fee2668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254e266d, 23 },
    { 0, 0 },
    { 0xd6ae266f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48e2672, 100 },
    { 0x7efe2672, 33 },
    { 0x355e2674, 98 },
    { 0x367e2675, 55 },
    { 0xddbe2676, 95 },
    { 0xaee2674, 90 },
    { 0xcc3e2677, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224e267d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86e2682, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e2686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce268b, 95 },
    { 0, 0 },
    { 0x353e268d, 90 },
    { 0xf56e268d, 90 },
    { 0x6a1e268e, 34 },
    { 0x730e2690, 43 },
    { 0x98fe2691, 33 },
    { 0, 0 },
    { 0x8a9e2693, 33 },
    { 0xa3ce2694, 65 },
    { 0x41de2695, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e2698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624e269e, 75 },
    { 0x6ce269f, 98 },
    { 0x885e269e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e26a3, 66 },
    { 0x3fbe26a3, 61 },
    { 0xf19e26a5, 90 },
    { 0xbc1e26a5, 33 },
    { 0x39ee26a7, 23 },
    { 0xac6e26a4, 98 },
    { 0xc6ae26a5, 67 },
    { 0xc11e26aa, 77 },
    { 0x1fbe26aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845e26af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e26b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53e26bb, 95 },
    { 0xe01e26bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702e26c1, 61 },
    { 0xb00e26c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e26c6, 61 },
    { 0, 0 },
    { 0xf67e26c8, 61 },
    { 0, 0 },
    { 0x301e26ca, 46 },
    { 0x711e26ca, 23 },
    { 0xebde26ca, 55 },
    { 0xf7be26cd, 99 },
    { 0, 0 },
    { 0x941e26cf, 90 },
    { 0, 0 },
    { 0x659e26d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e26d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae26d8, 99 },
    { 0x6e9e26d9, 98 },
    { 0, 0 },
    { 0x854e26db, 95 },
    { 0, 0 },
    { 0x347e26dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e26e3, 98 },
    { 0xcb1e26e4, 55 },
    { 0xeece26e4, 98 },
    { 0xd9ee26e6, 61 },
    { 0x1fe26e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e26eb, 90 },
    { 0xf4de26ec, 76 },
    { 0xa15e26ec, 90 },
    { 0x719e26ee, 66 },
    { 0x451e26ed, 67 },
    { 0xec9e26ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee26f6, 56 },
    { 0x402e26f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e26ff, 90 },
    { 0x381e26ff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19e2709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0e270c, 90 },
    { 0x9c2e270d, 90 },
    { 0x42de270d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x317e2711, 23 },
    { 0xd9ce2712, 98 },
    { 0x47ae2713, 95 },
    { 0x18e2713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e2717, 100 },
    { 0x5a3e2718, 101 },
    { 0x37ae2719, 61 },
    { 0xbc1e271a, 69 },
    { 0, 0 },
    { 0x61ee271c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e271f, 95 },
    { 0xa4ee2720, 90 },
    { 0, 0 },
    { 0x84de2722, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e2725, 90 },
    { 0, 0 },
    { 0xa62e2727, 90 },
    { 0x5b8e2727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbe2731, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7e2734, 23 },
    { 0xebde2734, 90 },
    { 0x295e2734, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8e273b, 61 },
    { 0xcede273c, 46 },
    { 0x968e273d, 33 },
    { 0x859e273e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3de2741, 17 },
    { 0x4eee2741, 76 },
    { 0x97ee2743, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14e2746, 100 },
    { 0xe2be2747, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbe274c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e2751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e2755, 93 },
    { 0x46e2756, 37 },
    { 0xaa5e2757, 34 },
    { 0xed3e2758, 67 },
    { 0xa95e2759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172e275f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2e2762, 76 },
    { 0, 0 },
    { 0x921e2764, 98 },
    { 0x71e2765, 77 },
    { 0x8ffe2766, 76 },
    { 0x1b0e2767, 33 },
    { 0xaa1e2767, 61 },
    { 0x354e2768, 23 },
    { 0x2f4e276a, 61 },
    { 0xf47e276b, 36 },
    { 0x443e276c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e2773, 88 },
    { 0x6fbe2774, 34 },
    { 0x942e2775, 65 },
    { 0x128e2775, 98 },
    { 0xb57e2774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9e277b, 90 },
    { 0xbebe277c, 34 },
    { 0xaace277b, 52 },
    { 0x163e277e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e2782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ce2785, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e2789, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e278c, 45 },
    { 0x6e0e278c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2de2790, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685e2795, 23 },
    { 0xd2de2795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e2799, 61 },
    { 0xfbbe279a, 66 },
    { 0x27e279b, 33 },
    { 0x699e279c, 90 },
    { 0, 0 },
    { 0x780e279e, 90 },
    { 0, 0 },
    { 0x8bbe27a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00e27a4, 61 },
    { 0xc6ee27a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756e27ac, 66 },
    { 0, 0 },
    { 0x707e27ae, 84 },
    { 0xe58e27ae, 40 },
    { 0xb4ce27af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e27b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be27bf, 55 },
    { 0, 0 },
    { 0x786e27c1, 90 },
    { 0x6ade27c2, 61 },
    { 0x418e27c3, 33 },
    { 0, 0 },
    { 0x68be27c5, 90 },
    { 0xbcbe27c5, 90 },
    { 0, 0 },
    { 0xa5ae27c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e27cb, 34 },
    { 0, 0 },
    { 0x6b2e27cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e27d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788e27d6, 43 },
    { 0x2dde27d7, 61 },
    { 0x5eee27d7, 33 },
    { 0, 0 },
    { 0xce2e27da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e27dd, 87 },
    { 0x76be27dd, 66 },
    { 0xfb8e27df, 90 },
    { 0x895e27e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abe27e3, 90 },
    { 0x82fe27e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53e27e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65e27eb, 98 },
    { 0x4e9e27ec, 85 },
    { 0x8b9e27ec, 96 },
    { 0x49fe27ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e27f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee2800, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e2803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2806, 67 },
    { 0x609e2807, 83 },
    { 0, 0 },
    { 0x6a1e2809, 84 },
    { 0x2bbe2809, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443e280e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e2819, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbae281c, 33 },
    { 0, 0 },
    { 0xf3e281e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14e2822, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e2827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5e282a, 100 },
    { 0, 0 },
    { 0xed0e282c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe282f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ce2834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce2838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4e283e, 91 },
    { 0x5d9e283e, 90 },
    { 0x441e2840, 81 },
    { 0xdbe2840, 43 },
    { 0xe82e2840, 65 },
    { 0xaae2843, 66 },
    { 0xc8ee2844, 98 },
    { 0xf7ce2845, 95 },
    { 0xf57e2842, 96 },
    { 0x6dae2842, 98 },
    { 0, 0 },
    { 0x48ee2849, 23 },
    { 0x2d1e2849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128e284e, 90 },
    { 0, 0 },
    { 0x11de2850, 61 },
    { 0xc25e2851, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae2854, 97 },
    { 0xf5ee2854, 95 },
    { 0xf1ae2856, 84 },
    { 0x33ae2855, 90 },
    { 0xc2ce2858, 90 },
    { 0x7d3e2859, 90 },
    { 0x942e285a, 61 },
    { 0, 0 },
    { 0xa7ae285c, 90 },
    { 0xd85e285c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e2860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ce2865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eee286c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fe2872, 84 },
    { 0x7e2e2873, 33 },
    { 0x155e2873, 98 },
    { 0xaf1e2875, 96 },
    { 0x99be2876, 77 },
    { 0xea2e2877, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7e287a, 43 },
    { 0xbb1e287a, 61 },
    { 0x179e287b, 95 },
    { 0x7a9e287a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e2883, 81 },
    { 0x517e2884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ae288a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fde288e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230e2892, 23 },
    { 0x34be2892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93e2896, 33 },
    { 0x4b5e2897, 52 },
    { 0x5cae2897, 23 },
    { 0x904e2898, 95 },
    { 0x9a9e2899, 87 },
    { 0xb7be289b, 61 },
    { 0x5a1e289b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57de28a0, 82 },
    { 0, 0 },
    { 0x417e28a2, 43 },
    { 0x1fde28a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ce28a6, 90 },
    { 0, 0 },
    { 0xab2e28a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e28ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e28b0, 95 },
    { 0xe0fe28b0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fde28b4, 98 },
    { 0, 0 },
    { 0x1f3e28b6, 33 },
    { 0x538e28b7, 33 },
    { 0, 0 },
    { 0x8fde28b9, 61 },
    { 0xf33e28ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e28bd, 91 },
    { 0x701e28be, 87 },
    { 0x4f4e28be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dde28c6, 98 },
    { 0xf86e28c6, 90 },
    { 0, 0 },
    { 0x168e28c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e28cc, 95 },
    { 0x59de28cd, 91 },
    { 0x49e28ce, 91 },
    { 0x9eae28cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e28d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x114e28d6, 61 },
    { 0xa1ce28d7, 90 },
    { 0x379e28d7, 90 },
    { 0xf0ce28d9, 77 },
    { 0x4ece28da, 91 },
    { 0x270e28db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e28df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e28e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe28e9, 98 },
    { 0x543e28ea, 90 },
    { 0, 0 },
    { 0x232e28ec, 61 },
    { 0x68e28ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0de28f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0e28f3, 49 },
    { 0x328e28f4, 55 },
    { 0x559e28f5, 33 },
    { 0x658e28f6, 61 },
    { 0xa77e28f7, 46 },
    { 0x95fe28f5, 90 },
    { 0x5e2e28f7, 46 },
    { 0xd12e28fa, 61 },
    { 0xfe3e28fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ce28fe, 36 },
    { 0x21be28ff, 90 },
    { 0xc1fe2900, 75 },
    { 0, 0 },
    { 0xc38e2902, 24 },
    { 0x3cfe2903, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2906, 61 },
    { 0x687e2906, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e290a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7e2915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e291f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3e2923, 95 },
    { 0x240e2924, 96 },
    { 0x604e2925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cae2929, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476e292e, 61 },
    { 0xfd5e292f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ee2934, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56e2938, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e293b, 23 },
    { 0x564e293b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57e293f, 87 },
    { 0, 0 },
    { 0x388e2941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3e2948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6e294e, 93 },
    { 0, 0 },
    { 0xd8e2950, 90 },
    { 0, 0 },
    { 0x37ce2952, 96 },
    { 0, 0 },
    { 0x8c6e2954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee2957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42e295d, 98 },
    { 0x89ae295e, 23 },
    { 0x927e295e, 33 },
    { 0x3e1e2960, 61 },
    { 0xf49e2961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e2965, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fe296f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94e2975, 23 },
    { 0x38be2976, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e297a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57e2985, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72e2989, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37e298c, 23 },
    { 0, 0 },
    { 0x9cee298e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe2991, 23 },
    { 0xa26e2992, 95 },
    { 0x904e2991, 84 },
    { 0, 0 },
    { 0x857e2995, 23 },
    { 0, 0 },
    { 0xb8be2997, 95 },
    { 0x39be2997, 61 },
    { 0x57fe2998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e299d, 33 },
    { 0x3f1e299d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fe29a7, 23 },
    { 0x98ce29a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e29ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2e29b0, 93 },
    { 0x11de29b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59e29b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963e29bc, 33 },
    { 0xf03e29bd, 67 },
    { 0x4c3e29bd, 95 },
    { 0, 0 },
    { 0xce3e29c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294e29c4, 34 },
    { 0x579e29c5, 90 },
    { 0xf14e29c4, 98 },
    { 0, 0 },
    { 0xe5ae29c8, 61 },
    { 0xafae29c9, 23 },
    { 0xe63e29ca, 76 },
    { 0x5b4e29cb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee29cf, 90 },
    { 0x17ce29cf, 78 },
    { 0x365e29d1, 98 },
    { 0xcd0e29d2, 55 },
    { 0, 0 },
    { 0x9b4e29d4, 46 },
    { 0xeb5e29d5, 23 },
    { 0, 0 },
    { 0xd72e29d7, 33 },
    { 0xc90e29d7, 93 },
    { 0xb4be29d9, 61 },
    { 0x344e29da, 98 },
    { 0xc13e29db, 93 },
    { 0x78de29dc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee29e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987e29e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e29e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862e29ec, 23 },
    { 0, 0 },
    { 0x661e29ee, 34 },
    { 0xdb4e29ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e29f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e29f9, 95 },
    { 0, 0 },
    { 0xb15e29fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e29fe, 23 },
    { 0x514e29fe, 90 },
    { 0x8c2e2a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ae2a04, 17 },
    { 0xb88e2a05, 19 },
    { 0x2eae2a06, 76 },
    { 0x787e2a07, 98 },
    { 0x7c1e2a04, 93 },
    { 0x65ce2a09, 78 },
    { 0x25de2a04, 95 },
    { 0x908e2a08, 55 },
    { 0x1be2a0c, 23 },
    { 0, 0 },
    { 0x266e2a0e, 90 },
    { 0x114e2a0f, 87 },
    { 0x165e2a0e, 90 },
    { 0xeb6e2a0f, 46 },
    { 0xd8ae2a0e, 55 },
    { 0xdb1e2a10, 95 },
    { 0x4bbe2a13, 76 },
    { 0x37fe2a15, 95 },
    { 0, 0 },
    { 0x38ee2a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce2a1f, 90 },
    { 0xde4e2a20, 98 },
    { 0x665e2a21, 61 },
    { 0x605e2a22, 66 },
    { 0xddfe2a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efe2a27, 61 },
    { 0, 0 },
    { 0x396e2a29, 46 },
    { 0x94ce2a2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be2a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e2a34, 46 },
    { 0xbd7e2a35, 68 },
    { 0x151e2a35, 55 },
    { 0x25ce2a34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfe2a3f, 95 },
    { 0x89ae2a40, 55 },
    { 0xd3e2a40, 65 },
    { 0, 0 },
    { 0xc7be2a43, 61 },
    { 0xde8e2a44, 98 },
    { 0x799e2a45, 91 },
    { 0xd15e2a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeee2a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe2a4e, 90 },
    { 0xfbbe2a4e, 90 },
    { 0x73ee2a50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e2a54, 98 },
    { 0x897e2a55, 61 },
    { 0xe9de2a56, 76 },
    { 0xee7e2a57, 33 },
    { 0xf0ee2a57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e2a5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe2a60, 33 },
    { 0, 0 },
    { 0x656e2a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5e2a6b, 43 },
    { 0xa27e2a6c, 100 },
    { 0x6d3e2a6b, 33 },
    { 0xde0e2a6d, 90 },
    { 0x521e2a6f, 34 },
    { 0xf94e2a6f, 94 },
    { 0, 0 },
    { 0xfe9e2a72, 100 },
    { 0xbdee2a72, 90 },
    { 0x645e2a74, 90 },
    { 0x8b5e2a73, 90 },
    { 0x414e2a74, 98 },
    { 0x931e2a77, 90 },
    { 0xb31e2a77, 33 },
    { 0, 0 },
    { 0x5f4e2a7a, 23 },
    { 0, 0 },
    { 0x917e2a7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece2a8b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3e2a8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe2a93, 90 },
    { 0xbe7e2a94, 90 },
    { 0x484e2a95, 95 },
    { 0, 0 },
    { 0x8fbe2a97, 76 },
    { 0x23e2a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade2a9c, 90 },
    { 0, 0 },
    { 0xbece2a9e, 90 },
    { 0, 0 },
    { 0xf14e2aa0, 61 },
    { 0x1f1e2aa1, 46 },
    { 0x71be2aa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91be2aa7, 78 },
    { 0x632e2aa7, 90 },
    { 0xc76e2aa9, 100 },
    { 0, 0 },
    { 0x103e2aab, 90 },
    { 0x98fe2aab, 67 },
    { 0x9ace2aad, 90 },
    { 0, 0 },
    { 0xf94e2aaf, 43 },
    { 0x57ce2ab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ee2ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e2ab6, 69 },
    { 0x77be2ab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e2ac0, 61 },
    { 0, 0 },
    { 0xaafe2ac2, 43 },
    { 0, 0 },
    { 0x3c3e2ac4, 40 },
    { 0, 0 },
    { 0x7bce2ac6, 34 },
    { 0, 0 },
    { 0x2d3e2ac8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686e2acd, 61 },
    { 0x87ee2ace, 34 },
    { 0x210e2acf, 34 },
    { 0, 0 },
    { 0x468e2ad1, 61 },
    { 0x478e2ad2, 23 },
    { 0x211e2ad3, 99 },
    { 0xb32e2ad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38be2add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x154e2ae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e2ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9e2ae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e2aec, 95 },
    { 0xab0e2aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81e2af0, 33 },
    { 0x328e2af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2b01, 34 },
    { 0, 0 },
    { 0x406e2b03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddee2b06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567e2b0f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751e2b14, 76 },
    { 0xd5fe2b14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e2b1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87e2b20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40de2b24, 99 },
    { 0x182e2b24, 23 },
    { 0x228e2b25, 67 },
    { 0x57ae2b27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e2b31, 76 },
    { 0, 0 },
    { 0x768e2b33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe2b36, 83 },
    { 0x11e2b37, 34 },
    { 0x6d8e2b38, 23 },
    { 0x9e6e2b38, 90 },
    { 0x7f7e2b3a, 84 },
    { 0xeece2b3a, 90 },
    { 0, 0 },
    { 0x5f1e2b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dde2b44, 95 },
    { 0x7e8e2b45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e2b4b, 23 },
    { 0x4ebe2b4b, 84 },
    { 0x91ce2b4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977e2b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe2b57, 90 },
    { 0xe8ee2b58, 93 },
    { 0xd26e2b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ce2b5c, 81 },
    { 0xf9ae2b5d, 81 },
    { 0x27be2b5c, 98 },
    { 0x44ee2b5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e2b62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ee2b65, 90 },
    { 0, 0 },
    { 0x93e2b67, 90 },
    { 0x68be2b68, 85 },
    { 0xecee2b68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9e2b70, 8 },
    { 0, 0 },
    { 0x73be2b72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608e2b79, 95 },
    { 0, 0 },
    { 0x727e2b7b, 90 },
    { 0xf82e2b7c, 98 },
    { 0, 0 },
    { 0x76ce2b7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2b82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31e2b88, 33 },
    { 0x3fbe2b89, 23 },
    { 0, 0 },
    { 0x86fe2b8b, 23 },
    { 0x9fae2b8c, 90 },
    { 0x472e2b8d, 68 },
    { 0xffbe2b8e, 69 },
    { 0x821e2b8f, 95 },
    { 0x55de2b8d, 98 },
    { 0xf18e2b91, 33 },
    { 0x513e2b8b, 95 },
    { 0x30ee2b93, 88 },
    { 0x6b9e2b93, 23 },
    { 0, 0 },
    { 0x504e2b96, 98 },
    { 0x449e2b96, 33 },
    { 0xa2be2b98, 33 },
    { 0xc30e2b98, 61 },
    { 0x9cce2b9a, 34 },
    { 0x2dbe2b9b, 66 },
    { 0x231e2b9c, 61 },
    { 0xb04e2b9d, 95 },
    { 0xdd2e2b9e, 43 },
    { 0x587e2b9f, 33 },
    { 0x383e2ba0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9be2ba9, 99 },
    { 0x137e2baa, 43 },
    { 0, 0 },
    { 0x17fe2bac, 90 },
    { 0x4be2bad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e2bb0, 34 },
    { 0, 0 },
    { 0xf41e2bb2, 34 },
    { 0, 0 },
    { 0x89de2bb4, 33 },
    { 0x24ce2bb5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e2bb9, 55 },
    { 0, 0 },
    { 0x521e2bbb, 46 },
    { 0x576e2bbc, 61 },
    { 0x752e2bbc, 61 },
    { 0xc0ce2bbe, 101 },
    { 0x99ce2bbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e2bc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fe2bca, 18 },
    { 0, 0 },
    { 0xb7de2bcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361e2bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbce2be2, 61 },
    { 0xaee2be3, 96 },
    { 0, 0 },
    { 0x3f0e2be5, 98 },
    { 0x523e2be5, 33 },
    { 0x1ade2be6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de2bf2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52de2bf9, 23 },
    { 0x142e2bfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de2c00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65de2c0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe2c0d, 95 },
    { 0x727e2c0d, 95 },
    { 0x869e2c0f, 76 },
    { 0x2f4e2c10, 43 },
    { 0xdbee2c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61e2c1a, 90 },
    { 0, 0 },
    { 0x88ee2c1c, 90 },
    { 0, 0 },
    { 0x91ce2c1e, 98 },
    { 0xb1de2c1f, 100 },
    { 0x83ce2c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe2c27, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2e2c2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28e2c33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592e2c38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56e2c3b, 61 },
    { 0xeeae2c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6e2c40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7e2c48, 33 },
    { 0xd5be2c49, 95 },
    { 0x31e2c4a, 90 },
    { 0x3cae2c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2c50, 17 },
    { 0x1e4e2c51, 61 },
    { 0x147e2c50, 33 },
    { 0x28ce2c52, 61 },
    { 0, 0 },
    { 0x585e2c55, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3e2c58, 76 },
    { 0x637e2c58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e2c5c, 85 },
    { 0, 0 },
    { 0xf2ee2c5e, 34 },
    { 0, 0 },
    { 0xb10e2c60, 81 },
    { 0xb3de2c61, 52 },
    { 0xbae2c62, 90 },
    { 0xf05e2c62, 43 },
    { 0x961e2c64, 99 },
    { 0x9f2e2c63, 23 },
    { 0x9fee2c66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e2c6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42e2c6f, 88 },
    { 0x6ee2c70, 90 },
    { 0x844e2c71, 42 },
    { 0xb6ae2c72, 66 },
    { 0x73ce2c6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4e2c77, 76 },
    { 0xad2e2c78, 84 },
    { 0x506e2c78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e2c7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aee2c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fde2c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e2c89, 43 },
    { 0x194e2c8a, 61 },
    { 0x93de2c8b, 55 },
    { 0x225e2c8a, 75 },
    { 0, 0 },
    { 0xd5ae2c8e, 67 },
    { 0x49de2c8f, 34 },
    { 0x23ae2c8f, 43 },
    { 0xaaae2c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496e2c95, 76 },
    { 0x49be2c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ae2c99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e2c9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae2ca3, 98 },
    { 0, 0 },
    { 0x712e2ca5, 55 },
    { 0x488e2ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee2cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5e2cb3, 90 },
    { 0xb3e2cb3, 33 },
    { 0x7c9e2cb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e2cb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e2cc1, 23 },
    { 0xe59e2cc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52be2cc5, 90 },
    { 0xa54e2cc5, 98 },
    { 0x405e2cc5, 43 },
    { 0xd31e2cc6, 90 },
    { 0, 0 },
    { 0xf6ae2cca, 76 },
    { 0x627e2ccb, 90 },
    { 0xa6de2ccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe2cda, 98 },
    { 0xcd1e2cdb, 61 },
    { 0xb66e2cdb, 90 },
    { 0, 0 },
    { 0x711e2cde, 95 },
    { 0, 0 },
    { 0x328e2ce0, 98 },
    { 0, 0 },
    { 0xf08e2ce2, 23 },
    { 0, 0 },
    { 0x183e2ce4, 90 },
    { 0x707e2ce4, 99 },
    { 0, 0 },
    { 0xe1e2ce7, 95 },
    { 0x1cfe2ce8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390e2ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2cf4, 61 },
    { 0, 0 },
    { 0x3fe2cf6, 33 },
    { 0xaabe2cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e2cfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde2d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78e2d0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e2d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27e2d20, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615e2d26, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e2d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e2d35, 91 },
    { 0xcb8e2d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce2d39, 90 },
    { 0x33fe2d3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2d3e, 77 },
    { 0x239e2d3e, 99 },
    { 0, 0 },
    { 0xd9ee2d41, 23 },
    { 0xce0e2d42, 95 },
    { 0xfe5e2d41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8e2d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e2d4b, 33 },
    { 0, 0 },
    { 0xca2e2d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe2d53, 98 },
    { 0, 0 },
    { 0xb50e2d55, 90 },
    { 0x69ce2d56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x487e2d59, 90 },
    { 0, 0 },
    { 0xfbce2d5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ace2d62, 98 },
    { 0x4fae2d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e2d68, 52 },
    { 0, 0 },
    { 0xa7de2d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x221e2d6d, 91 },
    { 0, 0 },
    { 0xad1e2d6f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e2d79, 23 },
    { 0x73e2d79, 80 },
    { 0, 0 },
    { 0xe4ae2d7c, 100 },
    { 0xa7ae2d7d, 33 },
    { 0xdfbe2d7d, 33 },
    { 0x43ee2d7f, 90 },
    { 0xd56e2d7f, 90 },
    { 0xbfe2d80, 90 },
    { 0x632e2d82, 58 },
    { 0xd6ce2d83, 61 },
    { 0xa03e2d84, 78 },
    { 0x62ce2d83, 23 },
    { 0xf51e2d86, 93 },
    { 0xe5ae2d86, 33 },
    { 0x544e2d87, 95 },
    { 0x604e2d89, 34 },
    { 0x448e2d83, 38 },
    { 0xc4fe2d8b, 34 },
    { 0x9a3e2d8c, 52 },
    { 0x764e2d8b, 36 },
    { 0x73ce2d8d, 61 },
    { 0x2e0e2d80, 43 },
    { 0x1c5e2d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e2d95, 33 },
    { 0xc3ce2d96, 33 },
    { 0xe92e2d96, 43 },
    { 0x8c0e2d97, 34 },
    { 0x4a2e2d99, 87 },
    { 0xec2e2d97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6e2d9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03e2da3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e2da7, 76 },
    { 0x126e2da8, 90 },
    { 0xbd4e2da7, 99 },
    { 0xbf1e2daa, 84 },
    { 0xeece2daa, 34 },
    { 0, 0 },
    { 0x1c4e2dad, 66 },
    { 0xba3e2dae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7be2db1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543e2db5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ce2dbd, 87 },
    { 0xc13e2dbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e2dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e2dcd, 55 },
    { 0, 0 },
    { 0xdee2dcf, 98 },
    { 0x39fe2dcf, 33 },
    { 0xcd4e2dd1, 65 },
    { 0, 0 },
    { 0x6f9e2dd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e2dd9, 33 },
    { 0x649e2dda, 46 },
    { 0x9aee2ddb, 77 },
    { 0, 0 },
    { 0x40ae2ddd, 90 },
    { 0, 0 },
    { 0x473e2ddf, 95 },
    { 0x843e2de0, 56 },
    { 0xd19e2de0, 61 },
    { 0x36ae2de2, 90 },
    { 0x7ade2ddf, 55 },
    { 0x7be2de1, 46 },
    { 0, 0 },
    { 0x5d4e2de6, 23 },
    { 0x19fe2de6, 46 },
    { 0xa6ae2de7, 67 },
    { 0x370e2de7, 61 },
    { 0xd9be2de9, 95 },
    { 0, 0 },
    { 0x300e2dec, 91 },
    { 0x7dee2ded, 87 },
    { 0x9cde2ded, 98 },
    { 0x2fde2def, 87 },
    { 0x385e2dec, 99 },
    { 0, 0 },
    { 0x2b0e2df2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e2df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e2dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ae2e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ce2e06, 99 },
    { 0, 0 },
    { 0x26ee2e08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e2e0e, 61 },
    { 0xabde2e0f, 33 },
    { 0x4fe2e0e, 90 },
    { 0xe19e2e0e, 90 },
    { 0x37e2e10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e2e16, 90 },
    { 0xf82e2e17, 98 },
    { 0x3b1e2e17, 87 },
    { 0x4d3e2e19, 94 },
    { 0x4abe2e1a, 90 },
    { 0xf96e2e1a, 98 },
    { 0x73ee2e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313e2e25, 90 },
    { 0, 0 },
    { 0xd32e2e27, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce2e2f, 76 },
    { 0x943e2e30, 90 },
    { 0x9b3e2e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x952e2e34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e2e3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e2e41, 93 },
    { 0xea6e2e42, 87 },
    { 0x7b7e2e43, 96 },
    { 0, 0 },
    { 0x681e2e45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8e2e48, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe2e57, 23 },
    { 0xebe2e58, 33 },
    { 0x284e2e59, 43 },
    { 0x627e2e5a, 23 },
    { 0x13e2e59, 85 },
    { 0x30ee2e5b, 80 },
    { 0xe7be2e5b, 67 },
    { 0x41ae2e5e, 61 },
    { 0xf57e2e5a, 95 },
    { 0xa98e2e60, 96 },
    { 0xe7ae2e61, 76 },
    { 0x503e2e61, 98 },
    { 0x6e0e2e60, 66 },
    { 0x80ae2e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe2e67, 43 },
    { 0x941e2e68, 55 },
    { 0xa60e2e69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e2e6c, 76 },
    { 0, 0 },
    { 0xfc8e2e6e, 61 },
    { 0x8e8e2e6f, 61 },
    { 0x965e2e6e, 69 },
    { 0, 0 },
    { 0xc53e2e72, 2 },
    { 0, 0 },
    { 0x142e2e74, 43 },
    { 0x54be2e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aee2e7e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e2e82, 33 },
    { 0xaf7e2e82, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e2e86, 33 },
    { 0, 0 },
    { 0xf74e2e88, 84 },
    { 0x504e2e88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fe2e8d, 33 },
    { 0, 0 },
    { 0x669e2e8f, 76 },
    { 0x3a4e2e8f, 46 },
    { 0xfd7e2e90, 61 },
    { 0xf76e2e92, 33 },
    { 0x37fe2e8f, 90 },
    { 0xbc6e2e94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2e98, 47 },
    { 0x2e7e2e99, 95 },
    { 0, 0 },
    { 0x1e2e2e9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310e2ea1, 23 },
    { 0x350e2ea2, 95 },
    { 0xa9fe2ea3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6e2eaa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8e2eae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712e2eb5, 23 },
    { 0x568e2eb6, 90 },
    { 0xb70e2eb7, 95 },
    { 0xc30e2eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e2ec0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39e2ec5, 90 },
    { 0xe1be2ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fae2ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e2ecc, 33 },
    { 0x8a7e2ecc, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e2ed2, 43 },
    { 0x69ce2ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e2eda, 94 },
    { 0, 0 },
    { 0x20fe2edc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41e2ee0, 76 },
    { 0, 0 },
    { 0xba0e2ee2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeee2ee5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1e2ee9, 23 },
    { 0x4d7e2eea, 85 },
    { 0, 0 },
    { 0xd10e2eec, 93 },
    { 0x5cfe2eed, 85 },
    { 0xadee2eed, 33 },
    { 0xb01e2eef, 46 },
    { 0xfbde2ef0, 34 },
    { 0x47e2eee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59e2ef7, 98 },
    { 0, 0 },
    { 0xd97e2ef9, 98 },
    { 0, 0 },
    { 0xd96e2efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ece2efe, 68 },
    { 0xe90e2eff, 76 },
    { 0x5c4e2eff, 95 },
    { 0x409e2f01, 90 },
    { 0, 0 },
    { 0xb0e2f03, 93 },
    { 0, 0 },
    { 0x148e2f05, 60 },
    { 0, 0 },
    { 0xc75e2f07, 52 },
    { 0x1e0e2f08, 55 },
    { 0x4c1e2f07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04e2f0e, 77 },
    { 0xdf5e2f0f, 33 },
    { 0x49fe2f10, 34 },
    { 0xaace2f11, 80 },
    { 0x46de2f12, 90 },
    { 0xb9ce2f13, 95 },
    { 0x960e2f13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7e2f1a, 34 },
    { 0, 0 },
    { 0xb21e2f1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e2f20, 90 },
    { 0xa50e2f21, 91 },
    { 0x174e2f22, 96 },
    { 0x87be2f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26e2f27, 33 },
    { 0x3dee2f27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ae2f34, 23 },
    { 0x9d0e2f35, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be2f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x375e2f3d, 67 },
    { 0x747e2f3e, 76 },
    { 0xe1ee2f3f, 51 },
    { 0x186e2f3d, 23 },
    { 0xce7e2f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94e2f47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2e2f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ce2f52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31de2f58, 61 },
    { 0x28ae2f59, 23 },
    { 0x5ace2f5a, 90 },
    { 0x649e2f58, 98 },
    { 0, 0 },
    { 0x279e2f5d, 91 },
    { 0, 0 },
    { 0x7aee2f5f, 61 },
    { 0xb8fe2f5f, 66 },
    { 0, 0 },
    { 0xd11e2f62, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bae2f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3e2f76, 78 },
    { 0xf7ce2f76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafde2f7c, 98 },
    { 0x595e2f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28e2f84, 95 },
    { 0x422e2f85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5e2f8c, 61 },
    { 0, 0 },
    { 0xd06e2f8e, 76 },
    { 0x3a4e2f8f, 76 },
    { 0xfcee2f90, 64 },
    { 0x894e2f91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e2f94, 81 },
    { 0x112e2f95, 33 },
    { 0, 0 },
    { 0x735e2f97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e2f9d, 98 },
    { 0xf80e2f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfce2fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be2fb3, 23 },
    { 0, 0 },
    { 0x1dce2fb5, 48 },
    { 0x430e2fb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e2fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ce2fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de2fc3, 90 },
    { 0xc98e2fc4, 52 },
    { 0x666e2fc5, 78 },
    { 0x1dce2fc4, 90 },
    { 0x6ade2fc4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ce2fcf, 34 },
    { 0, 0 },
    { 0x328e2fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x405e2fd4, 36 },
    { 0xec9e2fd5, 93 },
    { 0xd82e2fd6, 91 },
    { 0xcf5e2fd7, 40 },
    { 0x2f8e2fd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349e2fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce2fe3, 55 },
    { 0x467e2fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dde2fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74e2ff0, 23 },
    { 0x195e2ff1, 96 },
    { 0x2b8e2ff2, 90 },
    { 0x9b4e2ff1, 33 },
    { 0, 0 },
    { 0x330e2ff5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685e3006, 76 },
    { 0xa4ee3006, 33 },
    { 0x287e3008, 66 },
    { 0, 0 },
    { 0xb16e300a, 74 },
    { 0xe76e300b, 43 },
    { 0xf56e300c, 61 },
    { 0xe6e300c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102e301c, 75 },
    { 0, 0 },
    { 0xe7ae301e, 68 },
    { 0x7a1e301f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15e3022, 23 },
    { 0xda3e3023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade3027, 67 },
    { 0, 0 },
    { 0x101e3029, 93 },
    { 0, 0 },
    { 0x92ae302b, 33 },
    { 0x8f2e302c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fe3032, 23 },
    { 0xfc9e3032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe3036, 88 },
    { 0xa01e3036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2e303e, 33 },
    { 0, 0 },
    { 0xb20e3040, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e3047, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e304c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e304f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de3056, 90 },
    { 0, 0 },
    { 0x73ee3058, 90 },
    { 0x7fce3058, 94 },
    { 0x14ce305a, 90 },
    { 0x76de305b, 67 },
    { 0xb78e3059, 90 },
    { 0, 0 },
    { 0x6e4e305e, 88 },
    { 0xc76e305f, 87 },
    { 0, 0 },
    { 0x755e3061, 23 },
    { 0x9f0e3062, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eee3068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e306c, 33 },
    { 0xa2ce306c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee3071, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce3075, 38 },
    { 0, 0 },
    { 0x8d2e3077, 34 },
    { 0x8c4e3078, 95 },
    { 0x350e3078, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58be307f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590e3084, 98 },
    { 0, 0 },
    { 0xf3fe3086, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676e3091, 66 },
    { 0x440e3092, 61 },
    { 0xbdae3091, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce3097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee30a7, 61 },
    { 0x9ace30a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e30af, 33 },
    { 0xb61e30b0, 61 },
    { 0x15e30b1, 77 },
    { 0xe58e30b0, 98 },
    { 0x4b6e30b3, 53 },
    { 0, 0 },
    { 0xcf9e30b5, 90 },
    { 0, 0 },
    { 0x837e30b7, 33 },
    { 0x6fde30b8, 55 },
    { 0xae7e30b9, 90 },
    { 0xdeee30ba, 37 },
    { 0x682e30ba, 90 },
    { 0x7f2e30ba, 61 },
    { 0x3e2e30bc, 61 },
    { 0, 0 },
    { 0xc2de30bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726e30c8, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae30cc, 43 },
    { 0xa4ae30cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e30d0, 80 },
    { 0x998e30d1, 95 },
    { 0x8a5e30d2, 23 },
    { 0x5bae30d3, 52 },
    { 0x3b9e30d2, 90 },
    { 0x3aee30d2, 33 },
    { 0, 0 },
    { 0x7f2e30d7, 33 },
    { 0x879e30d7, 95 },
    { 0, 0 },
    { 0xbd4e30da, 61 },
    { 0x6cbe30da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8e30df, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee30e2, 93 },
    { 0x7dde30e3, 90 },
    { 0xa6de30e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce30ea, 33 },
    { 0x32ee30eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e30ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ee30f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee30f8, 33 },
    { 0xd8de30f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99e30fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e3101, 55 },
    { 0, 0 },
    { 0xe3ce3103, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe3108, 61 },
    { 0xca5e3109, 76 },
    { 0xbd1e3108, 68 },
    { 0xd8e310b, 43 },
    { 0xed7e310b, 77 },
    { 0, 0 },
    { 0x869e310e, 46 },
    { 0xc99e310e, 90 },
    { 0, 0 },
    { 0x603e3111, 43 },
    { 0x701e3112, 95 },
    { 0, 0 },
    { 0x295e3114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8e311b, 33 },
    { 0xc9ce311c, 90 },
    { 0, 0 },
    { 0x416e311e, 66 },
    { 0, 0 },
    { 0xd9de3120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe3124, 61 },
    { 0x182e3125, 95 },
    { 0, 0 },
    { 0xb11e3127, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e312a, 34 },
    { 0, 0 },
    { 0x72ae312c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e3133, 91 },
    { 0x6d4e3134, 23 },
    { 0, 0 },
    { 0x192e3136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6e3139, 76 },
    { 0, 0 },
    { 0x36ae313b, 95 },
    { 0, 0 },
    { 0x519e313d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace3142, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162e3148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ae314c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x422e314f, 100 },
    { 0, 0 },
    { 0x935e3151, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee3154, 61 },
    { 0, 0 },
    { 0xffbe3156, 95 },
    { 0xe93e3157, 93 },
    { 0x57be3158, 90 },
    { 0x3f0e3158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de315f, 61 },
    { 0, 0 },
    { 0x68e3161, 95 },
    { 0, 0 },
    { 0xebbe3163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ae3166, 90 },
    { 0x6d0e3166, 76 },
    { 0x3aae3168, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e316b, 90 },
    { 0xe28e316c, 98 },
    { 0xad9e316c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3172, 61 },
    { 0, 0 },
    { 0xdb2e3174, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fde3177, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecce317b, 34 },
    { 0xea1e317c, 99 },
    { 0, 0 },
    { 0x3b8e317e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994e3182, 33 },
    { 0, 0 },
    { 0x2f7e3184, 66 },
    { 0x92de3185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8e318a, 98 },
    { 0, 0 },
    { 0x146e318c, 76 },
    { 0, 0 },
    { 0xc84e318e, 100 },
    { 0x71ee318f, 90 },
    { 0x362e3190, 95 },
    { 0x12be3190, 46 },
    { 0x656e3190, 23 },
    { 0x9cce318e, 76 },
    { 0x555e3194, 95 },
    { 0x7a6e3194, 61 },
    { 0x2fce3191, 95 },
    { 0xf2be3193, 90 },
    { 0, 0 },
    { 0xb1ae3199, 67 },
    { 0x278e319a, 98 },
    { 0x321e319b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9e31a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e31a6, 43 },
    { 0x786e31a7, 77 },
    { 0xf6be31a8, 92 },
    { 0xa62e31a7, 61 },
    { 0, 0 },
    { 0xe76e31ab, 66 },
    { 0x7b4e31ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ae31b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be31b6, 61 },
    { 0x8de31b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89be31bc, 61 },
    { 0x315e31bd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e31c6, 33 },
    { 0x72ae31c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fe31cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e31ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ce31da, 43 },
    { 0xdbce31db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e31df, 100 },
    { 0, 0 },
    { 0xcdfe31e1, 34 },
    { 0x97fe31e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce31e5, 61 },
    { 0, 0 },
    { 0xd0e31e7, 98 },
    { 0x3bfe31e7, 61 },
    { 0xa39e31e8, 33 },
    { 0, 0 },
    { 0x13ee31eb, 66 },
    { 0x759e31eb, 61 },
    { 0x799e31ed, 55 },
    { 0x5c7e31ed, 95 },
    { 0xe2ee31ee, 55 },
    { 0x82e31eb, 100 },
    { 0x640e31f1, 69 },
    { 0x5ee31f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fe31f5, 84 },
    { 0, 0 },
    { 0x216e31f7, 61 },
    { 0xf0fe31f8, 23 },
    { 0x302e31f9, 68 },
    { 0x6a3e31f7, 40 },
    { 0, 0 },
    { 0xf63e31fc, 90 },
    { 0x7ebe31fd, 76 },
    { 0x42ee31fe, 95 },
    { 0xda0e31fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3207, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e320a, 33 },
    { 0, 0 },
    { 0x428e320c, 61 },
    { 0x245e320d, 93 },
    { 0xe36e320e, 23 },
    { 0xe52e320e, 84 },
    { 0x750e320e, 95 },
    { 0x109e3211, 88 },
    { 0xe64e3212, 43 },
    { 0, 0 },
    { 0x349e3214, 52 },
    { 0x7cce3214, 40 },
    { 0xf9ee3216, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7e321a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e3220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e3223, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xabde3226, 46 },
    { 0x220e3227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e322b, 33 },
    { 0x201e322c, 23 },
    { 0xc70e322d, 93 },
    { 0, 0 },
    { 0xb19e322f, 66 },
    { 0x2cbe3230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e323d, 34 },
    { 0, 0 },
    { 0x9cae323f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebe3244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2be3248, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbe3250, 88 },
    { 0xf64e3250, 94 },
    { 0, 0 },
    { 0x78fe3253, 60 },
    { 0x97fe3254, 98 },
    { 0, 0 },
    { 0xa62e3256, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e325b, 90 },
    { 0, 0 },
    { 0xc16e325d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e3261, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05e3264, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5e3268, 95 },
    { 0x33ae3268, 61 },
    { 0xae8e326a, 76 },
    { 0x568e326a, 85 },
    { 0x80ce326c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e3270, 99 },
    { 0x1dae3271, 93 },
    { 0, 0 },
    { 0xdae3273, 95 },
    { 0x3d1e3273, 84 },
    { 0, 0 },
    { 0xf95e3276, 33 },
    { 0, 0 },
    { 0xf4ce3278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe3280, 76 },
    { 0x31e3281, 87 },
    { 0xf29e3281, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ede3287, 90 },
    { 0xd61e3288, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce328d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee3291, 95 },
    { 0xf02e3291, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e3295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ae329b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6be329f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20be32a5, 33 },
    { 0, 0 },
    { 0x445e32a7, 33 },
    { 0xa67e32a8, 81 },
    { 0x1c7e32a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8e32b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19de32b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861e32c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae32ca, 33 },
    { 0x782e32cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e32d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade32e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e32e9, 95 },
    { 0, 0 },
    { 0x5f4e32eb, 90 },
    { 0x558e32ec, 46 },
    { 0, 0 },
    { 0x1e9e32ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998e32f2, 17 },
    { 0x562e32f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e32fb, 93 },
    { 0xc2ce32fb, 84 },
    { 0xa6be32fd, 61 },
    { 0x13e32fe, 90 },
    { 0, 0 },
    { 0x5d2e3300, 40 },
    { 0x69ae3301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fe3304, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30e330b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e330e, 93 },
    { 0x1b0e330e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe3314, 80 },
    { 0xefee3315, 4 },
    { 0, 0 },
    { 0x90ce3317, 93 },
    { 0x516e3317, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cee331b, 23 },
    { 0x977e331b, 98 },
    { 0xea3e331d, 76 },
    { 0, 0 },
    { 0xecce331f, 61 },
    { 0x84ce331f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be3329, 91 },
    { 0xe5ee332a, 76 },
    { 0x5a5e332b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e332f, 33 },
    { 0, 0 },
    { 0xea9e3331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e3335, 17 },
    { 0, 0 },
    { 0x470e3337, 95 },
    { 0, 0 },
    { 0x504e3339, 43 },
    { 0xce6e3339, 55 },
    { 0xe16e3339, 93 },
    { 0, 0 },
    { 0xa8ae333d, 61 },
    { 0x8dae333e, 23 },
    { 0xdafe333e, 98 },
    { 0xdf6e333e, 33 },
    { 0xb3de3341, 61 },
    { 0, 0 },
    { 0xa88e3343, 34 },
    { 0xc51e3344, 98 },
    { 0x8fde3345, 23 },
    { 0xc23e3346, 95 },
    { 0, 0 },
    { 0xa89e3348, 85 },
    { 0xadee3349, 56 },
    { 0x489e3349, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7e334d, 60 },
    { 0xa5fe334e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e3352, 33 },
    { 0x5a7e3353, 61 },
    { 0x4ae3352, 55 },
    { 0x76fe3354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae335b, 80 },
    { 0, 0 },
    { 0x2b1e335d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29de3360, 76 },
    { 0, 0 },
    { 0xb5e3362, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e3369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bee336e, 95 },
    { 0xeb9e336e, 95 },
    { 0x8a8e336f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4e3374, 61 },
    { 0x14e3375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e3379, 93 },
    { 0, 0 },
    { 0x251e337b, 33 },
    { 0x851e337c, 66 },
    { 0, 0 },
    { 0xd7be337e, 61 },
    { 0x6dae337e, 23 },
    { 0xfdde3380, 23 },
    { 0x490e337e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fe3387, 90 },
    { 0x73ce3387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41e338b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e3392, 23 },
    { 0, 0 },
    { 0x86fe3394, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce339b, 33 },
    { 0x5d6e339b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3de339f, 33 },
    { 0xd20e33a0, 97 },
    { 0x56fe33a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x294e33a4, 95 },
    { 0xa08e33a5, 61 },
    { 0x7bce33a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6e33ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9e33b3, 33 },
    { 0x118e33b4, 95 },
    { 0x35fe33b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e33b8, 76 },
    { 0xe39e33b8, 33 },
    { 0x494e33ba, 61 },
    { 0, 0 },
    { 0x9b2e33bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68e33c1, 90 },
    { 0x2f8e33c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe33c7, 33 },
    { 0, 0 },
    { 0x395e33c9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e33ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e33d2, 43 },
    { 0, 0 },
    { 0x690e33d4, 67 },
    { 0x4cfe33d5, 61 },
    { 0xb7ce33d6, 90 },
    { 0x50ce33d7, 66 },
    { 0x8f1e33d8, 33 },
    { 0x686e33d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e33dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58be33e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e33e5, 61 },
    { 0x3c0e33e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e33e9, 98 },
    { 0, 0 },
    { 0x734e33eb, 61 },
    { 0xfd4e33eb, 95 },
    { 0x1bae33ed, 61 },
    { 0xa17e33ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde33f2, 84 },
    { 0x2d1e33f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e33f9, 33 },
    { 0, 0 },
    { 0xf3ee33fb, 75 },
    { 0x544e33fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e33ff, 33 },
    { 0xce6e3400, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3404, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939e340e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e3411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5e341a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce3422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64e342a, 46 },
    { 0, 0 },
    { 0x9f8e342c, 69 },
    { 0xa47e342d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64e3434, 99 },
    { 0xc19e3434, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4e3438, 61 },
    { 0x30e3439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0e3441, 93 },
    { 0x1c2e3442, 91 },
    { 0x39de3441, 33 },
    { 0x2f8e3441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e3447, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7e344b, 61 },
    { 0x494e344c, 76 },
    { 0xb5fe344d, 90 },
    { 0x457e344e, 76 },
    { 0xa3de344c, 43 },
    { 0, 0 },
    { 0xe2be3451, 81 },
    { 0xe9de3451, 23 },
    { 0xbd6e3453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202e3458, 85 },
    { 0, 0 },
    { 0x387e345a, 66 },
    { 0xfe1e345b, 95 },
    { 0x390e345c, 33 },
    { 0xdabe345d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1e3461, 55 },
    { 0xd4ee3461, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2e3469, 90 },
    { 0, 0 },
    { 0x3d5e346b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e346f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e3472, 90 },
    { 0xebae3473, 90 },
    { 0x37de3474, 64 },
    { 0x9cfe3473, 78 },
    { 0, 0 },
    { 0xefe3477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703e347e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ce3481, 95 },
    { 0x4c5e3482, 23 },
    { 0xdcfe3483, 98 },
    { 0, 0 },
    { 0xbebe3485, 23 },
    { 0, 0 },
    { 0x3fee3487, 95 },
    { 0xbb6e3488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1e348b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdee3491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2e3494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e3498, 84 },
    { 0xff1e3499, 95 },
    { 0xecbe349a, 55 },
    { 0x912e349b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e34a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae34ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e34af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7e34b3, 23 },
    { 0, 0 },
    { 0x25be34b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65e34b8, 33 },
    { 0xff9e34b9, 61 },
    { 0, 0 },
    { 0xe14e34bb, 34 },
    { 0xc7ce34bb, 34 },
    { 0x381e34bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4e34c7, 78 },
    { 0, 0 },
    { 0x780e34c9, 77 },
    { 0x3d5e34c9, 90 },
    { 0x1be34c9, 33 },
    { 0x9f1e34cc, 76 },
    { 0xfa0e34cc, 61 },
    { 0x3e0e34cc, 66 },
    { 0, 0 },
    { 0xb87e34d0, 61 },
    { 0xd05e34d0, 98 },
    { 0x9a2e34d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ade34da, 68 },
    { 0x2c9e34db, 78 },
    { 0, 0 },
    { 0xbd6e34dd, 90 },
    { 0x1ece34de, 95 },
    { 0x1e2e34df, 90 },
    { 0x6c0e34e0, 61 },
    { 0x236e34e1, 95 },
    { 0x78e34e0, 52 },
    { 0x629e34e3, 46 },
    { 0, 0 },
    { 0x1d7e34e5, 90 },
    { 0x1a3e34e5, 33 },
    { 0x7a7e34e6, 98 },
    { 0xf3e34e5, 93 },
    { 0xe1e34e5, 84 },
    { 0x7ebe34e8, 100 },
    { 0xe00e34eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e34ee, 69 },
    { 0xe17e34ef, 90 },
    { 0, 0 },
    { 0xdffe34f1, 87 },
    { 0xed7e34f1, 91 },
    { 0x755e34f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eae34f6, 66 },
    { 0x7c6e34f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaede34fb, 90 },
    { 0, 0 },
    { 0xc4ce34fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3501, 93 },
    { 0x8ee3501, 90 },
    { 0x6c9e3503, 55 },
    { 0xd8fe3504, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e3508, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e350b, 95 },
    { 0xa64e350c, 81 },
    { 0x103e350b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcee3512, 98 },
    { 0x9a8e3512, 90 },
    { 0, 0 },
    { 0xdcee3515, 46 },
    { 0x7d2e3516, 53 },
    { 0x35de3515, 100 },
    { 0xe62e3515, 90 },
    { 0xa27e3519, 33 },
    { 0x185e351a, 33 },
    { 0x90fe351a, 61 },
    { 0xb64e351c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60e3520, 90 },
    { 0, 0 },
    { 0x28ae3522, 67 },
    { 0, 0 },
    { 0x921e3524, 23 },
    { 0xe03e3524, 86 },
    { 0x362e3525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee3529, 98 },
    { 0, 0 },
    { 0x11ae352b, 61 },
    { 0xf9be352c, 88 },
    { 0x814e352c, 33 },
    { 0xe72e352c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e3531, 48 },
    { 0, 0 },
    { 0x687e3533, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e3536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6e3539, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffe353d, 23 },
    { 0x58ce353e, 66 },
    { 0x7d4e353e, 98 },
    { 0, 0 },
    { 0xe6be3541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be3545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590e3548, 17 },
    { 0, 0 },
    { 0x786e354a, 69 },
    { 0, 0 },
    { 0x663e354c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e354f, 88 },
    { 0x44ee3550, 34 },
    { 0xff1e3551, 93 },
    { 0x803e3552, 98 },
    { 0x4d0e3552, 95 },
    { 0x3fde3554, 33 },
    { 0x550e3553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e3559, 33 },
    { 0x7cce3559, 23 },
    { 0, 0 },
    { 0x2de355c, 33 },
    { 0x162e355c, 33 },
    { 0xf19e355e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1e356d, 23 },
    { 0, 0 },
    { 0x63be356f, 90 },
    { 0x5ade3570, 90 },
    { 0x467e3570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e3576, 33 },
    { 0x181e3577, 33 },
    { 0, 0 },
    { 0xcc3e3579, 84 },
    { 0xdf9e357a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfe357d, 23 },
    { 0x9c8e357e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce3581, 61 },
    { 0x2e4e3581, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e3589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e358d, 76 },
    { 0xcbae358d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de359c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e359f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fe35a3, 33 },
    { 0x74fe35a4, 23 },
    { 0xec6e35a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x891e35a8, 55 },
    { 0, 0 },
    { 0x4a5e35aa, 23 },
    { 0x916e35aa, 23 },
    { 0xa2ae35ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e35b4, 65 },
    { 0xe10e35b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ee35ba, 78 },
    { 0, 0 },
    { 0x732e35bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee35bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbe35c2, 90 },
    { 0x41de35c3, 76 },
    { 0xadee35c3, 55 },
    { 0xb11e35c5, 61 },
    { 0xf51e35c6, 37 },
    { 0x5abe35c5, 90 },
    { 0xadde35c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe35ce, 33 },
    { 0xefae35cf, 33 },
    { 0, 0 },
    { 0x11de35d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe35d4, 84 },
    { 0x49de35d5, 98 },
    { 0, 0 },
    { 0xdc8e35d7, 77 },
    { 0x9c6e35d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e35db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee35e0, 76 },
    { 0x320e35e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e35eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78e35ee, 67 },
    { 0, 0 },
    { 0xee0e35f0, 33 },
    { 0x6ebe35f1, 33 },
    { 0x296e35f2, 85 },
    { 0x9e5e35f2, 76 },
    { 0x2fe35f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ede35f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e35fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48be3603, 61 },
    { 0x5cae3604, 98 },
    { 0, 0 },
    { 0xa1ae3606, 33 },
    { 0, 0 },
    { 0x400e3608, 90 },
    { 0xd72e3609, 85 },
    { 0xf4ae3608, 67 },
    { 0x7fce3609, 36 },
    { 0xe30e360c, 61 },
    { 0x89ce360d, 61 },
    { 0xc7e360e, 95 },
    { 0xe58e360e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x404e3612, 66 },
    { 0xefe3612, 36 },
    { 0x12fe3613, 61 },
    { 0, 0 },
    { 0xa04e3616, 98 },
    { 0x58ce3617, 76 },
    { 0xc63e3617, 17 },
    { 0xb99e3619, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792e3620, 61 },
    { 0x319e3621, 33 },
    { 0x2dde3621, 99 },
    { 0x2efe3623, 76 },
    { 0, 0 },
    { 0x3cae3625, 78 },
    { 0x1fee3626, 34 },
    { 0x653e3625, 66 },
    { 0x82ee3628, 34 },
    { 0xac3e3628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfe362f, 19 },
    { 0, 0 },
    { 0x296e3631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79e3634, 51 },
    { 0, 0 },
    { 0xbdce3636, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffe363b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e3644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fe3648, 36 },
    { 0x33ae3649, 90 },
    { 0xf07e364a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34e364d, 93 },
    { 0x79ae364e, 90 },
    { 0x712e364e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e3652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111e3656, 33 },
    { 0x8f5e3657, 34 },
    { 0, 0 },
    { 0xd84e3659, 90 },
    { 0xb93e365a, 90 },
    { 0x6a0e3659, 43 },
    { 0, 0 },
    { 0x747e365d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e3661, 67 },
    { 0, 0 },
    { 0xd3ee3663, 95 },
    { 0xdb0e3664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bde3667, 34 },
    { 0, 0 },
    { 0xfece3669, 43 },
    { 0xfdce366a, 90 },
    { 0xac2e366a, 95 },
    { 0x398e366a, 34 },
    { 0x995e366d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e3673, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540e367c, 87 },
    { 0xb5e367c, 33 },
    { 0xf7ee367e, 38 },
    { 0xec5e367f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee3682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe3685, 55 },
    { 0, 0 },
    { 0x7eae3687, 52 },
    { 0x7bde3688, 61 },
    { 0x23ce3688, 67 },
    { 0xd31e3689, 78 },
    { 0xd51e3687, 76 },
    { 0xdcce368b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5e3690, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e3694, 33 },
    { 0xbf2e3695, 23 },
    { 0x67be3694, 98 },
    { 0x444e3697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e36a4, 31 },
    { 0x932e36a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71e36aa, 93 },
    { 0x5dce36ab, 55 },
    { 0x525e36ac, 46 },
    { 0x3c1e36ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7e36b1, 61 },
    { 0xbb9e36b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7e36b7, 23 },
    { 0x9c5e36b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1e36bc, 77 },
    { 0xbcce36bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e36c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae36ca, 34 },
    { 0, 0 },
    { 0xa8be36cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce36d0, 67 },
    { 0x64e36d0, 43 },
    { 0, 0 },
    { 0x21ce36d3, 98 },
    { 0, 0 },
    { 0x65ce36d5, 81 },
    { 0x1c7e36d5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eee36e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e36e3, 98 },
    { 0xb8fe36e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e36e7, 34 },
    { 0, 0 },
    { 0x8d7e36e9, 93 },
    { 0x16be36ea, 93 },
    { 0xde0e36eb, 17 },
    { 0, 0 },
    { 0x7b4e36ed, 61 },
    { 0, 0 },
    { 0x672e36ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e36f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e36f7, 61 },
    { 0x3d2e36f8, 33 },
    { 0, 0 },
    { 0xbc5e36fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e36fd, 88 },
    { 0x2eae36fe, 98 },
    { 0x645e36ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbece3702, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76e3705, 90 },
    { 0x766e3706, 77 },
    { 0xdcae3707, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8e370b, 52 },
    { 0x4c1e370c, 90 },
    { 0xb92e370d, 100 },
    { 0x16e370e, 76 },
    { 0x38fe370f, 90 },
    { 0x4ebe370c, 33 },
    { 0xf6be3711, 17 },
    { 0x1ede3712, 91 },
    { 0xbf3e3712, 76 },
    { 0x245e3711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ce3717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6be371d, 85 },
    { 0xbcbe371e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e3721, 33 },
    { 0xe39e3722, 90 },
    { 0xbd6e3723, 93 },
    { 0x28ae3723, 61 },
    { 0x623e3725, 61 },
    { 0x356e3723, 76 },
    { 0xa00e3726, 34 },
    { 0xbf3e3722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce372b, 33 },
    { 0x50ae372c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465e3731, 55 },
    { 0x3f8e3732, 84 },
    { 0xfede3731, 55 },
    { 0xbffe3734, 98 },
    { 0, 0 },
    { 0xfe4e3736, 33 },
    { 0x1d6e3736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x819e373a, 33 },
    { 0x293e373b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e3741, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60e374b, 98 },
    { 0, 0 },
    { 0xc9fe374d, 76 },
    { 0, 0 },
    { 0xa00e374f, 90 },
    { 0x99de3750, 84 },
    { 0x657e3751, 61 },
    { 0xb76e3752, 33 },
    { 0, 0 },
    { 0x458e3754, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3be3765, 43 },
    { 0, 0 },
    { 0x55fe3767, 33 },
    { 0x7a4e3767, 23 },
    { 0xf47e3769, 76 },
    { 0x88fe376a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307e376e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3e3771, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e3775, 66 },
    { 0x6bee3776, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae3779, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e377c, 98 },
    { 0x71be377d, 34 },
    { 0xa29e377d, 81 },
    { 0x63ee377f, 61 },
    { 0x922e377f, 76 },
    { 0xbe2e377f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe3789, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43e378f, 98 },
    { 0xdcbe3790, 52 },
    { 0x6d1e3791, 66 },
    { 0xd58e3792, 33 },
    { 0x75ce3793, 84 },
    { 0x131e3794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7e379f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246e37a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e37ab, 53 },
    { 0x180e37ac, 91 },
    { 0, 0 },
    { 0xb31e37ae, 90 },
    { 0xaa7e37af, 61 },
    { 0x553e37b0, 100 },
    { 0x343e37b0, 33 },
    { 0x8c4e37b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7e37b9, 90 },
    { 0x23ee37ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13e37bd, 95 },
    { 0, 0 },
    { 0xd85e37bf, 33 },
    { 0xa01e37c0, 90 },
    { 0, 0 },
    { 0xe54e37c2, 100 },
    { 0, 0 },
    { 0xb53e37c4, 88 },
    { 0, 0 },
    { 0xdc8e37c6, 90 },
    { 0xe27e37c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e37cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2e37d0, 90 },
    { 0x4a1e37d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e37da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e37df, 85 },
    { 0x85be37e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ce37e4, 95 },
    { 0, 0 },
    { 0x5e4e37e6, 33 },
    { 0x60fe37e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279e37eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e37ee, 95 },
    { 0x836e37ef, 95 },
    { 0xe66e37ef, 67 },
    { 0x373e37ee, 95 },
    { 0xf96e37f2, 98 },
    { 0xcece37f0, 90 },
    { 0x3a6e37f0, 23 },
    { 0x5c5e37f4, 36 },
    { 0xd5fe37f0, 95 },
    { 0, 0 },
    { 0x511e37f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e37fb, 46 },
    { 0, 0 },
    { 0xb73e37fd, 17 },
    { 0xdc9e37fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e3803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e3806, 95 },
    { 0x1c4e3807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e380a, 47 },
    { 0, 0 },
    { 0x920e380c, 61 },
    { 0xc7ce380d, 23 },
    { 0x4d3e380e, 66 },
    { 0xb5de380f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0e3812, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e3816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535e381f, 61 },
    { 0x568e3820, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549e3824, 34 },
    { 0xa0ae3825, 33 },
    { 0x74ae3825, 76 },
    { 0x8f9e3827, 98 },
    { 0x687e3828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5e3830, 61 },
    { 0, 0 },
    { 0xa7be3832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee3838, 23 },
    { 0x6e2e3839, 33 },
    { 0xae7e3839, 34 },
    { 0x43de3839, 98 },
    { 0, 0 },
    { 0x3ade383d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74e3840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e3844, 95 },
    { 0xeefe3845, 23 },
    { 0, 0 },
    { 0xa4ae3847, 46 },
    { 0xa70e3848, 91 },
    { 0xc3ee3848, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744e384e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854e3855, 55 },
    { 0xf5be3856, 90 },
    { 0, 0 },
    { 0x25ee3858, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e385e, 46 },
    { 0x722e385e, 93 },
    { 0, 0 },
    { 0x65be3861, 77 },
    { 0x169e3862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e3866, 90 },
    { 0x618e3867, 90 },
    { 0, 0 },
    { 0x309e3869, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e386c, 98 },
    { 0x3b6e386c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70e3870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be3876, 65 },
    { 0x8d4e3877, 95 },
    { 0, 0 },
    { 0x342e3879, 95 },
    { 0, 0 },
    { 0xa43e387b, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e387e, 46 },
    { 0xb27e387f, 87 },
    { 0x92de387f, 91 },
    { 0x23be387f, 23 },
    { 0xbfce3882, 66 },
    { 0x370e3883, 43 },
    { 0x167e3884, 75 },
    { 0x933e3885, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbe388c, 33 },
    { 0x193e388d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6e3891, 61 },
    { 0x299e3892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9e3897, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9e38a2, 95 },
    { 0, 0 },
    { 0x8a6e38a4, 23 },
    { 0, 0 },
    { 0x913e38a6, 34 },
    { 0xfb4e38a7, 43 },
    { 0x12fe38a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e38ac, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96e38af, 61 },
    { 0x95ce38b0, 23 },
    { 0, 0 },
    { 0x4fee38b2, 23 },
    { 0x2c6e38b3, 91 },
    { 0x4aee38b2, 76 },
    { 0x68ce38b5, 93 },
    { 0, 0 },
    { 0x4cfe38b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffe38bb, 95 },
    { 0xa41e38bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5e38bf, 90 },
    { 0, 0 },
    { 0x14ce38c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdde38c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fe38cf, 75 },
    { 0, 0 },
    { 0xbe0e38d1, 90 },
    { 0, 0 },
    { 0x928e38d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae38da, 90 },
    { 0, 0 },
    { 0xdcbe38dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe38df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9e38e3, 98 },
    { 0xf04e38e4, 90 },
    { 0, 0 },
    { 0x908e38e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0e38eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede38f2, 98 },
    { 0x1efe38f3, 23 },
    { 0x136e38f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e38fd, 46 },
    { 0, 0 },
    { 0x12e38ff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66be3905, 81 },
    { 0x9f2e3906, 76 },
    { 0xa51e3906, 90 },
    { 0x7c8e3908, 90 },
    { 0x64ae3906, 58 },
    { 0xe8ce390a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7e3911, 61 },
    { 0xb34e3912, 93 },
    { 0x2f5e3913, 91 },
    { 0x5f1e3914, 61 },
    { 0xd53e3915, 33 },
    { 0xaee3916, 90 },
    { 0x6b5e3911, 75 },
    { 0, 0 },
    { 0x736e3919, 38 },
    { 0, 0 },
    { 0x8afe391b, 95 },
    { 0x7a2e391c, 95 },
    { 0, 0 },
    { 0x9ece391e, 76 },
    { 0x16e391e, 90 },
    { 0x118e3920, 84 },
    { 0x757e3920, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2e3925, 87 },
    { 0xe82e3926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ae392a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19e392d, 90 },
    { 0x30be392e, 52 },
    { 0, 0 },
    { 0x439e3930, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713e3934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce3937, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7e393c, 58 },
    { 0xe69e393d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67e3941, 43 },
    { 0xe81e3942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e3946, 90 },
    { 0xe33e3947, 98 },
    { 0xabee3947, 90 },
    { 0x57e3948, 90 },
    { 0x371e394a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e394e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x244e3951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bce3955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e395c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1e3968, 33 },
    { 0, 0 },
    { 0x56ce396a, 76 },
    { 0, 0 },
    { 0x8e4e396c, 76 },
    { 0x98be396c, 68 },
    { 0xf94e396d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3972, 23 },
    { 0, 0 },
    { 0xfbee3974, 43 },
    { 0x2b0e3975, 85 },
    { 0x586e3974, 100 },
    { 0x25de3974, 95 },
    { 0, 0 },
    { 0x482e3979, 98 },
    { 0, 0 },
    { 0xfdae397b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcee3981, 95 },
    { 0x3c1e3982, 61 },
    { 0, 0 },
    { 0xfe9e3984, 78 },
    { 0x860e3984, 98 },
    { 0x163e3984, 36 },
    { 0, 0 },
    { 0xcfbe3988, 97 },
    { 0x8f3e3989, 33 },
    { 0, 0 },
    { 0x758e398b, 90 },
    { 0x82de398c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e3992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e3998, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce399c, 46 },
    { 0xa94e399d, 52 },
    { 0x5b0e399e, 95 },
    { 0, 0 },
    { 0x59e39a0, 87 },
    { 0x8a9e39a1, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48e39a7, 23 },
    { 0xe28e39a7, 90 },
    { 0, 0 },
    { 0x4f0e39aa, 23 },
    { 0x256e39aa, 80 },
    { 0, 0 },
    { 0x30be39ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fe39b0, 68 },
    { 0x28de39b0, 93 },
    { 0xc5e39b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe39b5, 81 },
    { 0x25ce39b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcae39bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721e39c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ae39c7, 96 },
    { 0xd2fe39c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ae39cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e39d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe39d3, 98 },
    { 0xa5ee39d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78de39d9, 33 },
    { 0x7ebe39da, 99 },
    { 0x984e39d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e39e1, 90 },
    { 0x3dce39e1, 46 },
    { 0x16ce39e2, 93 },
    { 0xb03e39e4, 90 },
    { 0, 0 },
    { 0x566e39e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e39ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69e39f6, 90 },
    { 0x14be39f6, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de39fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824e3a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e3a0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e3a13, 90 },
    { 0, 0 },
    { 0x4e5e3a15, 36 },
    { 0xe96e3a16, 77 },
    { 0x19ae3a17, 95 },
    { 0x775e3a18, 33 },
    { 0x9bbe3a19, 90 },
    { 0, 0 },
    { 0x5f9e3a1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e3a20, 98 },
    { 0x9e2e3a21, 90 },
    { 0, 0 },
    { 0x1e5e3a23, 65 },
    { 0xbb5e3a23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89e3a27, 98 },
    { 0xf1ae3a28, 90 },
    { 0x8e4e3a29, 90 },
    { 0x159e3a28, 61 },
    { 0x7d6e3a28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e3a2e, 8 },
    { 0, 0 },
    { 0x6b0e3a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66e3a37, 61 },
    { 0, 0 },
    { 0xa8ee3a39, 61 },
    { 0x61ce3a3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe3a3d, 90 },
    { 0xa36e3a3e, 95 },
    { 0xd0fe3a3f, 85 },
    { 0xe95e3a40, 61 },
    { 0x531e3a40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee3a46, 46 },
    { 0, 0 },
    { 0x90ee3a48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5e3a4c, 91 },
    { 0, 0 },
    { 0x178e3a4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a51, 61 },
    { 0x9abe3a51, 93 },
    { 0x165e3a53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e3a57, 90 },
    { 0x6f0e3a58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce3a5b, 61 },
    { 0, 0 },
    { 0xcf4e3a5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e3a61, 33 },
    { 0x893e3a62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e3a65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e3a6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938e3a70, 46 },
    { 0x5a1e3a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee3a77, 33 },
    { 0xe0ee3a78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e3a7d, 33 },
    { 0x663e3a7d, 33 },
    { 0, 0 },
    { 0x9efe3a80, 76 },
    { 0xc9ae3a80, 52 },
    { 0, 0 },
    { 0x1abe3a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44de3a86, 78 },
    { 0x2a7e3a87, 98 },
    { 0x48ce3a86, 95 },
    { 0, 0 },
    { 0xd39e3a8a, 34 },
    { 0x377e3a8b, 81 },
    { 0x360e3a8c, 90 },
    { 0x38ae3a8a, 33 },
    { 0x9ae3a8e, 68 },
    { 0x130e3a8d, 23 },
    { 0, 0 },
    { 0x530e3a91, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6e3a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ae3a99, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x395e3a9c, 23 },
    { 0x940e3a9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ce3aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e3aa3, 91 },
    { 0x90ce3aa4, 23 },
    { 0xdfbe3aa3, 33 },
    { 0x36e3aa3, 90 },
    { 0x954e3aa3, 90 },
    { 0x9f0e3aa8, 76 },
    { 0x4ade3aa9, 23 },
    { 0, 0 },
    { 0x1ece3aab, 65 },
    { 0x8b6e3aac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fce3ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee3ab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ee3ac0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ade3ac3, 76 },
    { 0, 0 },
    { 0x54de3ac5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92be3ac9, 46 },
    { 0x7e8e3aca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbe3ace, 23 },
    { 0xbb3e3acf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bce3ad2, 15 },
    { 0x472e3ad3, 61 },
    { 0xb3ee3ad4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fe3ad8, 27 },
    { 0xd6ae3ad8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ae3ae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9de3ae4, 55 },
    { 0x23fe3ae5, 61 },
    { 0xe1e3ae6, 23 },
    { 0x859e3ae7, 55 },
    { 0xe6ee3ae8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae3aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538e3af4, 88 },
    { 0, 0 },
    { 0x228e3af6, 61 },
    { 0xae1e3af7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3afc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56e3b01, 52 },
    { 0xf04e3b02, 90 },
    { 0xcf1e3b03, 72 },
    { 0, 0 },
    { 0xdf2e3b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336e3b0d, 100 },
    { 0, 0 },
    { 0xe3e3b0f, 43 },
    { 0x162e3b10, 33 },
    { 0x7b4e3b11, 77 },
    { 0xe64e3b12, 93 },
    { 0, 0 },
    { 0x198e3b14, 33 },
    { 0xb0ee3b14, 61 },
    { 0, 0 },
    { 0x93fe3b17, 33 },
    { 0x7dfe3b17, 93 },
    { 0xf14e3b18, 91 },
    { 0xcd2e3b1a, 33 },
    { 0x697e3b1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3b21, 95 },
    { 0x5b0e3b21, 95 },
    { 0, 0 },
    { 0x377e3b24, 33 },
    { 0, 0 },
    { 0x9f9e3b26, 33 },
    { 0x14fe3b27, 90 },
    { 0x2f7e3b28, 33 },
    { 0xe2de3b28, 17 },
    { 0x23fe3b2a, 61 },
    { 0xabde3b2b, 23 },
    { 0xa07e3b2c, 90 },
    { 0x4e3b2a, 34 },
    { 0xf69e3b2d, 90 },
    { 0, 0 },
    { 0x372e3b30, 93 },
    { 0xbb3e3b31, 23 },
    { 0x9d5e3b32, 95 },
    { 0xd50e3b33, 61 },
    { 0, 0 },
    { 0x75de3b35, 68 },
    { 0x9e5e3b36, 38 },
    { 0xcf7e3b35, 91 },
    { 0xd45e3b38, 61 },
    { 0, 0 },
    { 0x7dce3b3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e3b3e, 90 },
    { 0x207e3b3f, 95 },
    { 0x701e3b40, 23 },
    { 0x966e3b40, 67 },
    { 0xaffe3b42, 76 },
    { 0x1d1e3b43, 33 },
    { 0xfd8e3b44, 95 },
    { 0xc9ee3b44, 55 },
    { 0x553e3b44, 98 },
    { 0x39fe3b47, 76 },
    { 0xeb4e3b43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8e3b51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77e3b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557e3b58, 77 },
    { 0x84de3b59, 23 },
    { 0x157e3b59, 90 },
    { 0xdf4e3b5b, 33 },
    { 0xc15e3b5c, 33 },
    { 0xf6ce3b59, 93 },
    { 0xd95e3b5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e3b66, 65 },
    { 0, 0 },
    { 0x34fe3b68, 61 },
    { 0, 0 },
    { 0xaece3b6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee3b6e, 33 },
    { 0, 0 },
    { 0xfc7e3b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe3b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e3b77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bce3b7f, 76 },
    { 0, 0 },
    { 0x6f3e3b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x371e3b84, 33 },
    { 0xc07e3b84, 43 },
    { 0x9e0e3b86, 34 },
    { 0x4e9e3b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e3b8d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34e3b97, 61 },
    { 0xf5ae3b98, 34 },
    { 0, 0 },
    { 0xdb6e3b9a, 43 },
    { 0, 0 },
    { 0x9fde3b9c, 55 },
    { 0x790e3b9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ade3ba2, 76 },
    { 0x884e3ba3, 61 },
    { 0xd1ae3ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4e3ba8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aae3bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e3bbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59e3bbe, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdace3bc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e3bcb, 77 },
    { 0xee3e3bcc, 33 },
    { 0x842e3bcc, 66 },
    { 0, 0 },
    { 0x2b3e3bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25e3bd6, 93 },
    { 0, 0 },
    { 0x79de3bd8, 99 },
    { 0, 0 },
    { 0xc9e3bda, 33 },
    { 0xabe3bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5e3bdf, 87 },
    { 0xeafe3bdf, 90 },
    { 0x659e3bdf, 23 },
    { 0x74e3be2, 55 },
    { 0, 0 },
    { 0x60fe3be4, 87 },
    { 0x6e5e3be5, 61 },
    { 0xe9ee3be4, 93 },
    { 0x5e2e3be6, 52 },
    { 0, 0 },
    { 0x195e3be9, 55 },
    { 0x7b2e3be9, 90 },
    { 0x8f2e3beb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe3bee, 69 },
    { 0x8fbe3bef, 95 },
    { 0xcece3bf0, 77 },
    { 0x5a8e3bef, 90 },
    { 0, 0 },
    { 0xd9fe3bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0e3bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce3bfb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e3bfe, 76 },
    { 0x888e3bfe, 61 },
    { 0x75fe3c00, 68 },
    { 0x868e3c01, 77 },
    { 0, 0 },
    { 0x7c5e3c03, 90 },
    { 0x11ce3c04, 76 },
    { 0, 0 },
    { 0xa71e3c06, 61 },
    { 0x1f0e3c07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e3c0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326e3c1a, 33 },
    { 0x142e3c1b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4e3c20, 85 },
    { 0x3d1e3c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e3c25, 90 },
    { 0x80ee3c26, 76 },
    { 0xa12e3c26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e3c2b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e3c2f, 55 },
    { 0xbf0e3c30, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42e3c35, 55 },
    { 0, 0 },
    { 0xcdbe3c37, 23 },
    { 0xde5e3c38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87e3c40, 23 },
    { 0xfe8e3c40, 94 },
    { 0xb1be3c42, 81 },
    { 0, 0 },
    { 0x676e3c44, 55 },
    { 0xf64e3c45, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ce3c4b, 91 },
    { 0x369e3c4c, 61 },
    { 0x45e3c4d, 41 },
    { 0x772e3c4d, 23 },
    { 0xb53e3c4e, 61 },
    { 0xfd3e3c4e, 90 },
    { 0x113e3c51, 23 },
    { 0x20ce3c52, 61 },
    { 0x254e3c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee3c56, 90 },
    { 0x8bce3c57, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ee3c5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0e3c65, 61 },
    { 0x7be3c66, 95 },
    { 0, 0 },
    { 0x9fde3c68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e3c6c, 33 },
    { 0xa6ae3c6d, 90 },
    { 0x5dde3c6e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be3c71, 95 },
    { 0xf61e3c72, 34 },
    { 0, 0 },
    { 0xad8e3c74, 61 },
    { 0xdbfe3c75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e3c7a, 98 },
    { 0xfdbe3c7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ce3c7e, 55 },
    { 0xecde3c7f, 61 },
    { 0x3bde3c7e, 46 },
    { 0x507e3c7e, 99 },
    { 0x941e3c7f, 61 },
    { 0x2c5e3c83, 90 },
    { 0xc64e3c7f, 33 },
    { 0, 0 },
    { 0x84fe3c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e3c90, 33 },
    { 0x139e3c90, 90 },
    { 0xe3ae3c90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ce3c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fe3c9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe3c9e, 93 },
    { 0x58ee3c9f, 76 },
    { 0xb94e3ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e3ca4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe3ca9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27e3cae, 91 },
    { 0x653e3caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ae3cb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e3cb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13be3cbc, 78 },
    { 0, 0 },
    { 0x7bae3cbe, 33 },
    { 0x7f6e3cbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x603e3cc2, 46 },
    { 0x6ce3cc3, 55 },
    { 0x850e3cc3, 53 },
    { 0x1a3e3cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee3cd0, 95 },
    { 0xc6fe3cd1, 61 },
    { 0, 0 },
    { 0x1abe3cd3, 52 },
    { 0xdcee3cd4, 33 },
    { 0x57de3cd4, 100 },
    { 0, 0 },
    { 0x8c7e3cd7, 33 },
    { 0xa7be3cd8, 95 },
    { 0, 0 },
    { 0x5f2e3cda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ae3cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x850e3ce1, 46 },
    { 0, 0 },
    { 0xc5ce3ce3, 100 },
    { 0xca6e3ce4, 98 },
    { 0xefe3ce5, 25 },
    { 0x2f3e3ce6, 33 },
    { 0xc2de3ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9de3cec, 95 },
    { 0x507e3ced, 23 },
    { 0x68ee3cec, 67 },
    { 0xfdfe3cef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e3cf3, 23 },
    { 0x60ce3cf3, 100 },
    { 0xa57e3cf5, 98 },
    { 0x44e3cf3, 91 },
    { 0x70e3cf7, 43 },
    { 0x48ae3cf8, 61 },
    { 0x1d4e3cf9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x157e3cfc, 92 },
    { 0x2bee3cfd, 76 },
    { 0xa5fe3cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be3d01, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3e3d05, 23 },
    { 0x4bbe3d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e3d09, 61 },
    { 0xb9ae3d09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3e3d0e, 88 },
    { 0x881e3d0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e3d14, 90 },
    { 0x630e3d14, 98 },
    { 0x150e3d16, 90 },
    { 0xbc5e3d14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae3d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702e3d21, 98 },
    { 0, 0 },
    { 0x4dee3d23, 91 },
    { 0xb83e3d24, 61 },
    { 0xa52e3d23, 90 },
    { 0, 0 },
    { 0xf93e3d27, 90 },
    { 0xfc1e3d28, 98 },
    { 0x6c3e3d28, 61 },
    { 0, 0 },
    { 0xc0ce3d2b, 99 },
    { 0, 0 },
    { 0x5c6e3d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be3d30, 76 },
    { 0xeb3e3d31, 34 },
    { 0x1fe3d30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1e3d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404e3d3b, 84 },
    { 0x451e3d3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e3d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e3d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3d4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6de3d59, 31 },
    { 0, 0 },
    { 0x57e3d5b, 46 },
    { 0x80e3d5b, 52 },
    { 0, 0 },
    { 0xba0e3d5e, 78 },
    { 0x828e3d5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddce3d62, 46 },
    { 0xf96e3d62, 95 },
    { 0, 0 },
    { 0xebce3d65, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06e3d68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973e3d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4e3d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e3d76, 61 },
    { 0xc7ee3d77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499e3d7e, 76 },
    { 0, 0 },
    { 0x66ee3d80, 61 },
    { 0x40e3d81, 97 },
    { 0, 0 },
    { 0x4cce3d83, 33 },
    { 0, 0 },
    { 0x241e3d85, 61 },
    { 0xc57e3d85, 76 },
    { 0xb92e3d86, 23 },
    { 0x311e3d88, 98 },
    { 0x182e3d89, 72 },
    { 0xec1e3d8a, 46 },
    { 0x6b8e3d8a, 100 },
    { 0, 0 },
    { 0x63ce3d8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56be3d93, 98 },
    { 0, 0 },
    { 0xea9e3d95, 33 },
    { 0x71fe3d96, 95 },
    { 0x98e3d97, 23 },
    { 0x3cae3d97, 55 },
    { 0x25fe3d95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e3d9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ee3da1, 88 },
    { 0xf55e3da2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffe3da8, 33 },
    { 0x83ee3da9, 61 },
    { 0x218e3daa, 33 },
    { 0x71be3dab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbae3db3, 95 },
    { 0, 0 },
    { 0xb9e3db5, 33 },
    { 0xc42e3db6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0e3dba, 90 },
    { 0x606e3dbb, 33 },
    { 0x86de3dba, 90 },
    { 0x3e5e3dbb, 90 },
    { 0, 0 },
    { 0x518e3dbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e3dc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e3dca, 68 },
    { 0, 0 },
    { 0xc9de3dcc, 90 },
    { 0x351e3dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04e3dd6, 61 },
    { 0xb3ae3dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fe3ddd, 46 },
    { 0, 0 },
    { 0x555e3ddf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7de3de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13e3df0, 77 },
    { 0, 0 },
    { 0x979e3df2, 98 },
    { 0x268e3df2, 43 },
    { 0x470e3df3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1e3df7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e3dfb, 95 },
    { 0x868e3dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafe3e01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2e3e09, 43 },
    { 0, 0 },
    { 0xa44e3e0b, 91 },
    { 0xbebe3e0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e3e0f, 61 },
    { 0, 0 },
    { 0xbc5e3e11, 85 },
    { 0, 0 },
    { 0xfb3e3e13, 34 },
    { 0x402e3e13, 76 },
    { 0xcf7e3e15, 23 },
    { 0xd6ae3e14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e3e1a, 55 },
    { 0x7a3e3e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be3e20, 42 },
    { 0x33ee3e21, 93 },
    { 0x326e3e22, 90 },
    { 0, 0 },
    { 0xf9e3e24, 33 },
    { 0xbd5e3e25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eee3e28, 98 },
    { 0x5f4e3e28, 33 },
    { 0x79fe3e2a, 23 },
    { 0x5ebe3e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce3e30, 23 },
    { 0, 0 },
    { 0x393e3e32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9e3e35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e3e38, 90 },
    { 0xe2ee3e39, 23 },
    { 0x4dae3e3a, 61 },
    { 0x6b0e3e3b, 90 },
    { 0, 0 },
    { 0xeb2e3e3d, 64 },
    { 0xcd0e3e3e, 33 },
    { 0x3c9e3e3e, 100 },
    { 0x4f6e3e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee3e43, 66 },
    { 0xe09e3e44, 23 },
    { 0, 0 },
    { 0xa4e3e46, 34 },
    { 0xc76e3e47, 33 },
    { 0x8c4e3e48, 90 },
    { 0xb9e3e48, 90 },
    { 0xcabe3e49, 88 },
    { 0x5e3e3e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29e3e4e, 98 },
    { 0x7b1e3e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769e3e53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e3e56, 95 },
    { 0x4bde3e57, 70 },
    { 0x3a2e3e57, 23 },
    { 0x635e3e59, 95 },
    { 0xbdae3e5a, 55 },
    { 0, 0 },
    { 0xa25e3e5c, 78 },
    { 0xa5ee3e5d, 90 },
    { 0, 0 },
    { 0x205e3e5f, 98 },
    { 0xfd1e3e60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e3e63, 90 },
    { 0x50ce3e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e3e67, 95 },
    { 0xe64e3e68, 90 },
    { 0x24e3e68, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e3e6f, 23 },
    { 0x33de3e70, 61 },
    { 0xdd4e3e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e3e77, 33 },
    { 0xe36e3e78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77e3e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3e3e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ee3e92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e3e95, 90 },
    { 0xe37e3e96, 85 },
    { 0xe1ae3e96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce3e9c, 98 },
    { 0xcd1e3e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de3ea0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3ea3, 87 },
    { 0x24ae3ea3, 46 },
    { 0xf85e3ea5, 33 },
    { 0xa1be3ea5, 61 },
    { 0xf54e3ea7, 55 },
    { 0xc3fe3ea4, 55 },
    { 0xe5ce3ea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5e3eae, 23 },
    { 0x5a2e3eaf, 83 },
    { 0x917e3eaf, 95 },
    { 0x9a1e3eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712e3eb7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2e3ebc, 90 },
    { 0x793e3ebd, 69 },
    { 0, 0 },
    { 0xce1e3ebf, 33 },
    { 0xc8ae3ec0, 23 },
    { 0, 0 },
    { 0x89de3ec2, 98 },
    { 0x345e3ec3, 61 },
    { 0x7d9e3ec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144e3ec9, 99 },
    { 0x58ee3ec9, 75 },
    { 0, 0 },
    { 0x1b6e3ecc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee3ecf, 93 },
    { 0xb25e3ecf, 23 },
    { 0, 0 },
    { 0x2fe3ed2, 100 },
    { 0xda8e3ed3, 90 },
    { 0, 0 },
    { 0x524e3ed5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae3edf, 34 },
    { 0x968e3ee0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03e3ee8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773e3eed, 88 },
    { 0x38e3eee, 90 },
    { 0, 0 },
    { 0x39fe3ef0, 99 },
    { 0xe0ce3ef1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afe3ef8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31e3efb, 95 },
    { 0x1d8e3efb, 95 },
    { 0, 0 },
    { 0xc5fe3efe, 46 },
    { 0x102e3eff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de3f02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e3f05, 95 },
    { 0xe3e3f05, 34 },
    { 0x10e3f07, 23 },
    { 0xa47e3f08, 99 },
    { 0x941e3f07, 23 },
    { 0x57be3f0a, 98 },
    { 0x241e3f0a, 33 },
    { 0x2f5e3f0a, 98 },
    { 0x40ae3f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x228e3f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ee3f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e3f17, 61 },
    { 0, 0 },
    { 0x3f8e3f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe3f1f, 33 },
    { 0x12be3f20, 23 },
    { 0x6afe3f1f, 43 },
    { 0xe39e3f1f, 85 },
    { 0, 0 },
    { 0xd40e3f24, 76 },
    { 0x41ee3f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee3f29, 100 },
    { 0, 0 },
    { 0xd8de3f2b, 90 },
    { 0, 0 },
    { 0xba6e3f2d, 23 },
    { 0xc04e3f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45e3f33, 98 },
    { 0xacae3f34, 23 },
    { 0x1bbe3f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e3f42, 55 },
    { 0xe2e3f43, 65 },
    { 0, 0 },
    { 0xdc7e3f45, 98 },
    { 0, 0 },
    { 0x85fe3f47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4e3f4b, 76 },
    { 0, 0 },
    { 0x68e3f4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8e3f51, 76 },
    { 0, 0 },
    { 0x4dbe3f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278e3f59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e3f5c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe3f63, 61 },
    { 0x4b6e3f64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e3f67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298e3f6b, 52 },
    { 0, 0 },
    { 0x122e3f6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fe3f70, 87 },
    { 0x4d8e3f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834e3f75, 90 },
    { 0x916e3f76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876e3f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e3f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e3f84, 40 },
    { 0, 0 },
    { 0xcfce3f86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901e3f8a, 33 },
    { 0, 0 },
    { 0x6a1e3f8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932e3f90, 98 },
    { 0x8c2e3f90, 23 },
    { 0, 0 },
    { 0xee3f93, 33 },
    { 0, 0 },
    { 0xd93e3f95, 91 },
    { 0x6ece3f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01e3f9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae3fa0, 90 },
    { 0x1b7e3fa0, 93 },
    { 0xe22e3fa0, 55 },
    { 0x132e3fa3, 95 },
    { 0x71fe3fa2, 90 },
    { 0xddce3fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee3fa9, 33 },
    { 0xb04e3faa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be3fad, 78 },
    { 0, 0 },
    { 0xffe3faf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e3fb3, 100 },
    { 0xcdee3fb4, 49 },
    { 0x777e3fb4, 76 },
    { 0x33ae3fb5, 76 },
    { 0x6ede3fb6, 66 },
    { 0x6e5e3fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4e3fbf, 33 },
    { 0, 0 },
    { 0xf50e3fc1, 95 },
    { 0, 0 },
    { 0x67ce3fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be3fc9, 85 },
    { 0x4bbe3fca, 61 },
    { 0xcdce3fca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de3fce, 70 },
    { 0xfc9e3fcf, 33 },
    { 0xa7ce3fce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e3fd4, 34 },
    { 0xa55e3fd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e3fda, 55 },
    { 0xfece3fdb, 33 },
    { 0x5b1e3fdb, 23 },
    { 0x25de3fdd, 9 },
    { 0, 0 },
    { 0x7b3e3fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be3fe8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3ff0, 40 },
    { 0, 0 },
    { 0xaa5e3ff2, 65 },
    { 0xe1ce3ff2, 61 },
    { 0xd2ce3ff4, 23 },
    { 0, 0 },
    { 0x90be3ff6, 88 },
    { 0x140e3ff7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ae3ffb, 52 },
    { 0xf94e3ffb, 46 },
    { 0xdd1e3ffd, 78 },
    { 0xf3ce3ffe, 93 },
    { 0x7fae3fff, 33 },
    { 0x8dde3fff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e4008, 90 },
    { 0, 0 },
    { 0x5e400a, 34 },
    { 0, 0 },
    { 0x295e400c, 90 },
    { 0x5cde400d, 98 },
    { 0x81fe400e, 98 },
    { 0xfafe400d, 33 },
    { 0xd9ee4010, 90 },
    { 0xe75e4010, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0e4014, 90 },
    { 0, 0 },
    { 0x513e4016, 66 },
    { 0, 0 },
    { 0xa1ee4018, 90 },
    { 0xe35e4018, 76 },
    { 0x8ece4019, 95 },
    { 0, 0 },
    { 0x77be401c, 34 },
    { 0x76fe401d, 95 },
    { 0x1abe401e, 34 },
    { 0x70be401e, 90 },
    { 0x647e401c, 69 },
    { 0xf14e401d, 93 },
    { 0x162e4022, 43 },
    { 0xde7e401e, 40 },
    { 0, 0 },
    { 0x2dee4025, 56 },
    { 0, 0 },
    { 0xd46e4027, 93 },
    { 0x333e4028, 85 },
    { 0xed2e4028, 90 },
    { 0x43e4029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee402e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ee4034, 33 },
    { 0x1e2e4034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e4038, 98 },
    { 0, 0 },
    { 0x880e403a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762e4041, 66 },
    { 0x195e4042, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be4046, 61 },
    { 0xed3e4047, 75 },
    { 0, 0 },
    { 0xba4e4049, 23 },
    { 0x294e4049, 23 },
    { 0x545e4049, 43 },
    { 0x652e404c, 78 },
    { 0xf81e404c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e4050, 23 },
    { 0x8d2e4051, 98 },
    { 0x201e4052, 88 },
    { 0xb5de4051, 81 },
    { 0x206e4054, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x387e4057, 60 },
    { 0x1a2e4058, 90 },
    { 0, 0 },
    { 0xbc9e405a, 90 },
    { 0x1cae405b, 98 },
    { 0, 0 },
    { 0x410e405d, 96 },
    { 0x444e405e, 40 },
    { 0xcf3e405e, 55 },
    { 0xdf2e405e, 95 },
    { 0x187e405e, 17 },
    { 0xcc6e405e, 55 },
    { 0, 0 },
    { 0xa66e4064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505e406b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6e406e, 90 },
    { 0xc2ce406f, 66 },
    { 0x91ae406f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ee4073, 87 },
    { 0xbae4074, 17 },
    { 0x97ce4073, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ee4079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e407c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce407f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae4085, 81 },
    { 0, 0 },
    { 0x726e4087, 61 },
    { 0x16fe4088, 33 },
    { 0x785e4089, 61 },
    { 0x404e4087, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e408e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12e4092, 98 },
    { 0x111e4093, 55 },
    { 0xa57e4094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36e4098, 61 },
    { 0xf1de4098, 90 },
    { 0xd1de409a, 61 },
    { 0, 0 },
    { 0x982e409c, 98 },
    { 0x95ae409d, 23 },
    { 0xe92e409e, 23 },
    { 0x786e409d, 61 },
    { 0x5dfe40a0, 55 },
    { 0xe6fe40a1, 61 },
    { 0x1ee40a1, 61 },
    { 0xa93e40a2, 43 },
    { 0xf9ee40a4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e40aa, 33 },
    { 0xdd8e40aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279e40b2, 34 },
    { 0xd88e40b2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94be40bb, 46 },
    { 0, 0 },
    { 0xf6ee40bd, 61 },
    { 0, 0 },
    { 0x56ae40bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e40c3, 34 },
    { 0xa1fe40c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e40cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804e40cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ee40d2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de40da, 91 },
    { 0xa3ae40db, 86 },
    { 0x879e40da, 55 },
    { 0x8f5e40db, 75 },
    { 0, 0 },
    { 0xdcee40df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3e40e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107e40e7, 34 },
    { 0, 0 },
    { 0xaf9e40e9, 23 },
    { 0x49e40ea, 95 },
    { 0xb31e40ea, 33 },
    { 0x2e5e40ec, 69 },
    { 0, 0 },
    { 0x665e40ee, 90 },
    { 0xde0e40ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ae40f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x794e40f6, 91 },
    { 0, 0 },
    { 0x7dce40f8, 98 },
    { 0, 0 },
    { 0xf02e40fa, 95 },
    { 0x112e40fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e4106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee410c, 90 },
    { 0, 0 },
    { 0x8f2e410e, 90 },
    { 0xc7ae410e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e4115, 87 },
    { 0x61be4115, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce411b, 67 },
    { 0xf32e411c, 78 },
    { 0x556e411c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e4121, 98 },
    { 0, 0 },
    { 0xe6be4123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x530e4126, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcee4139, 43 },
    { 0, 0 },
    { 0x7a9e413b, 98 },
    { 0x182e413b, 33 },
    { 0, 0 },
    { 0xfaee413e, 78 },
    { 0x840e413e, 91 },
    { 0xb05e413f, 46 },
    { 0x8a4e4141, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5e4147, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fde414e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e4153, 55 },
    { 0, 0 },
    { 0x607e4155, 33 },
    { 0, 0 },
    { 0xbe7e4157, 61 },
    { 0xbf4e4157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e415b, 95 },
    { 0x10e415b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7e4165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e4169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e416d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fde4172, 23 },
    { 0x9a0e4173, 84 },
    { 0x7e5e4173, 34 },
    { 0x5f7e4175, 90 },
    { 0xa61e4176, 76 },
    { 0xb84e4175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x170e417a, 95 },
    { 0x627e417b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849e4180, 90 },
    { 0xf0de4181, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e4184, 33 },
    { 0x4b3e4185, 95 },
    { 0x2d0e4186, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fe418f, 23 },
    { 0, 0 },
    { 0x9a8e4191, 87 },
    { 0, 0 },
    { 0xbf0e4193, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e4198, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x747e419b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efe41a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e41a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218e41ac, 64 },
    { 0xa9fe41ad, 87 },
    { 0x28ae41ad, 46 },
    { 0x46ae41ae, 66 },
    { 0xea4e41b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990e41b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e41c6, 12 },
    { 0xd4e41c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ce41cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de41d3, 33 },
    { 0x193e41d4, 23 },
    { 0xc10e41d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x326e41d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfe41db, 33 },
    { 0xf08e41dc, 33 },
    { 0xa66e41dd, 61 },
    { 0x132e41de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15e41e1, 100 },
    { 0x51de41e2, 78 },
    { 0x20fe41e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e41e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63e41ee, 52 },
    { 0x650e41ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e41f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6e41f5, 61 },
    { 0x728e41f5, 34 },
    { 0xeede41f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0e41fb, 43 },
    { 0, 0 },
    { 0x2dde41fd, 76 },
    { 0, 0 },
    { 0xebe41ff, 33 },
    { 0x802e41ff, 85 },
    { 0x96ee41ff, 21 },
    { 0xa89e4202, 61 },
    { 0, 0 },
    { 0x805e4204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627e420f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4212, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efe4215, 61 },
    { 0xc71e4216, 93 },
    { 0x1c4e4217, 75 },
    { 0, 0 },
    { 0xd3ce4219, 95 },
    { 0xeefe421a, 90 },
    { 0xc46e421b, 85 },
    { 0x681e421a, 33 },
    { 0, 0 },
    { 0xfdfe421e, 33 },
    { 0xb40e421e, 66 },
    { 0x3e9e421e, 23 },
    { 0x389e421e, 91 },
    { 0, 0 },
    { 0xb61e4223, 98 },
    { 0, 0 },
    { 0xe5e4225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccee422e, 76 },
    { 0xd3ae422e, 95 },
    { 0xc6fe422e, 36 },
    { 0x283e4231, 95 },
    { 0x700e4232, 33 },
    { 0xf15e4231, 33 },
    { 0x7cae4230, 66 },
    { 0x8a2e4231, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dae423a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e4240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459e4244, 46 },
    { 0x6ffe4244, 61 },
    { 0, 0 },
    { 0x30ce4247, 95 },
    { 0x1ede4248, 77 },
    { 0xa22e4249, 33 },
    { 0x38ee4249, 67 },
    { 0x2e5e424a, 77 },
    { 0x1cee424c, 76 },
    { 0, 0 },
    { 0xd50e424e, 40 },
    { 0, 0 },
    { 0x953e4250, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731e4258, 61 },
    { 0x5ebe4258, 98 },
    { 0xeb0e425a, 90 },
    { 0xa82e425b, 76 },
    { 0xb14e425b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e4261, 46 },
    { 0, 0 },
    { 0x5f3e4263, 67 },
    { 0xc45e4264, 36 },
    { 0x645e4265, 61 },
    { 0x942e4265, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48e426c, 90 },
    { 0x1d7e426c, 23 },
    { 0x4f2e426c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1e4272, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07e4275, 23 },
    { 0xf61e4275, 77 },
    { 0xab0e4275, 23 },
    { 0xd2e4278, 61 },
    { 0xfaee4276, 91 },
    { 0x568e427a, 77 },
    { 0xe7ce427b, 90 },
    { 0x54ee427c, 40 },
    { 0x27e427d, 23 },
    { 0x7c7e427e, 46 },
    { 0x9b1e427f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28be4283, 52 },
    { 0, 0 },
    { 0x179e4285, 98 },
    { 0, 0 },
    { 0x4c5e4287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efe428d, 23 },
    { 0, 0 },
    { 0xe9fe428f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e4292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e429c, 68 },
    { 0xf01e429d, 95 },
    { 0x396e429e, 90 },
    { 0x45fe429f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416e42a7, 17 },
    { 0, 0 },
    { 0x39fe42a9, 23 },
    { 0x2f1e42a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9e42ad, 76 },
    { 0x637e42ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe42b2, 23 },
    { 0, 0 },
    { 0xdd9e42b4, 52 },
    { 0xb48e42b5, 34 },
    { 0xfae42b6, 76 },
    { 0, 0 },
    { 0xac4e42b8, 34 },
    { 0, 0 },
    { 0x7e7e42ba, 37 },
    { 0, 0 },
    { 0xc8ae42bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e42c5, 90 },
    { 0xbe7e42c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ee42cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0e42cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bce42d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121e42db, 61 },
    { 0x898e42db, 84 },
    { 0x8d5e42dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaee42e3, 33 },
    { 0, 0 },
    { 0x413e42e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ee42ed, 90 },
    { 0xf1ae42ee, 61 },
    { 0xa0ae42ef, 93 },
    { 0xd60e42ef, 100 },
    { 0xd44e42ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3e42f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e42fc, 23 },
    { 0xb1ce42fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720e4300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce4308, 61 },
    { 0xe3de4308, 91 },
    { 0, 0 },
    { 0x22de430b, 95 },
    { 0, 0 },
    { 0xb1de430d, 61 },
    { 0xcf7e430e, 95 },
    { 0x7ebe430f, 33 },
    { 0, 0 },
    { 0x18be4311, 66 },
    { 0, 0 },
    { 0xf11e4313, 90 },
    { 0xb37e4313, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e4319, 61 },
    { 0xf04e4319, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93e4328, 23 },
    { 0, 0 },
    { 0x3a2e432a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8e4333, 95 },
    { 0, 0 },
    { 0xb1fe4335, 66 },
    { 0, 0 },
    { 0xfcfe4337, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e433c, 40 },
    { 0, 0 },
    { 0x27fe433e, 88 },
    { 0xf58e433e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226e4345, 98 },
    { 0x584e4346, 90 },
    { 0x753e4345, 90 },
    { 0x524e4348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e434b, 76 },
    { 0xdbae434c, 33 },
    { 0x5fae434d, 33 },
    { 0xc89e434c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ae4352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3e4355, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88e435c, 90 },
    { 0x8cee435d, 34 },
    { 0xc1ee435e, 34 },
    { 0x164e435f, 61 },
    { 0xe8e4360, 23 },
    { 0x488e4361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e436e, 46 },
    { 0, 0 },
    { 0xce2e4370, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ae4374, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e4378, 98 },
    { 0x741e4379, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e437f, 90 },
    { 0x296e437f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce4385, 34 },
    { 0, 0 },
    { 0x3cbe4387, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ae438a, 33 },
    { 0, 0 },
    { 0x40be438c, 43 },
    { 0, 0 },
    { 0x4dde438e, 34 },
    { 0x43de438f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde4392, 40 },
    { 0x95fe4393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6e439b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e439e, 84 },
    { 0, 0 },
    { 0xd4ee43a0, 87 },
    { 0, 0 },
    { 0x1eae43a2, 55 },
    { 0x22ce43a3, 81 },
    { 0x808e43a3, 90 },
    { 0x8ade43a5, 33 },
    { 0x3cbe43a3, 33 },
    { 0x5efe43a7, 90 },
    { 0xf42e43a8, 90 },
    { 0x71e43a6, 90 },
    { 0x90be43aa, 90 },
    { 0xae43ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e43ae, 90 },
    { 0, 0 },
    { 0xcf0e43b0, 99 },
    { 0, 0 },
    { 0x9e6e43b2, 33 },
    { 0xb79e43b3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ae43b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ae43ba, 95 },
    { 0, 0 },
    { 0x200e43bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e43bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e43c3, 98 },
    { 0xbdde43c4, 98 },
    { 0x5f2e43c3, 61 },
    { 0xc57e43c6, 68 },
    { 0xb57e43c6, 98 },
    { 0x78ce43c8, 90 },
    { 0xfffe43c9, 90 },
    { 0x6a9e43c3, 34 },
    { 0xfbfe43cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336e43cf, 90 },
    { 0x58ae43d0, 90 },
    { 0xbb5e43d1, 90 },
    { 0x44e43d2, 61 },
    { 0, 0 },
    { 0x107e43d4, 84 },
    { 0, 0 },
    { 0x5be43d6, 66 },
    { 0, 0 },
    { 0x30ae43d8, 95 },
    { 0, 0 },
    { 0xcebe43da, 90 },
    { 0xb7fe43db, 23 },
    { 0xadde43dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997e43e0, 61 },
    { 0, 0 },
    { 0xac7e43e2, 78 },
    { 0, 0 },
    { 0x1ede43e4, 90 },
    { 0xe12e43e4, 90 },
    { 0xdd9e43e6, 75 },
    { 0x3f4e43e7, 66 },
    { 0x755e43e7, 95 },
    { 0, 0 },
    { 0x24e43ea, 90 },
    { 0, 0 },
    { 0x6dee43ec, 61 },
    { 0x5a1e43ec, 90 },
    { 0, 0 },
    { 0x524e43ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ae43f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bee43f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e43fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7e43fd, 95 },
    { 0xe0ce43fd, 53 },
    { 0x600e43fd, 43 },
    { 0x877e43ff, 93 },
    { 0, 0 },
    { 0x2c5e4402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aae4405, 76 },
    { 0xb3de4405, 98 },
    { 0x302e4407, 61 },
    { 0, 0 },
    { 0xb04e4409, 23 },
    { 0x6be440a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e440e, 52 },
    { 0, 0 },
    { 0x45be4410, 98 },
    { 0x3a0e4410, 98 },
    { 0, 0 },
    { 0xda0e4413, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e4416, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace441c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe4421, 33 },
    { 0xc49e4422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234e4429, 98 },
    { 0xe34e442a, 90 },
    { 0, 0 },
    { 0xe78e442c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5e4431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90e4434, 98 },
    { 0xcafe4435, 90 },
    { 0, 0 },
    { 0x3dde4437, 90 },
    { 0x1ce4438, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e443e, 61 },
    { 0, 0 },
    { 0x2f7e4440, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fe4447, 95 },
    { 0, 0 },
    { 0x20e4449, 95 },
    { 0x687e4449, 69 },
    { 0x627e444b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e4452, 78 },
    { 0x236e4452, 77 },
    { 0, 0 },
    { 0x66fe4455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be445b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x197e445e, 90 },
    { 0xc15e445f, 95 },
    { 0xf98e445f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3e4463, 38 },
    { 0x1a5e4464, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde4467, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e446a, 70 },
    { 0xc48e446b, 33 },
    { 0, 0 },
    { 0x30ce446d, 52 },
    { 0xc87e446e, 95 },
    { 0xf33e446f, 95 },
    { 0x1c6e4470, 76 },
    { 0x369e4471, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ce4475, 100 },
    { 0x6e0e4476, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae4479, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e448c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e4491, 45 },
    { 0, 0 },
    { 0xaf4e4493, 94 },
    { 0, 0 },
    { 0xfc7e4495, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee449f, 95 },
    { 0, 0 },
    { 0x157e44a1, 95 },
    { 0x91de44a2, 40 },
    { 0, 0 },
    { 0x283e44a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656e44a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e44b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6e44b4, 23 },
    { 0x490e44b4, 76 },
    { 0, 0 },
    { 0xf74e44b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ee44ba, 90 },
    { 0x60ce44ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cae44be, 98 },
    { 0, 0 },
    { 0x731e44c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35be44c5, 23 },
    { 0x5e5e44c6, 95 },
    { 0x8b3e44c7, 23 },
    { 0, 0 },
    { 0x881e44c9, 88 },
    { 0xcbe44c9, 81 },
    { 0, 0 },
    { 0xd75e44cc, 90 },
    { 0x599e44cd, 23 },
    { 0x787e44ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e44d1, 33 },
    { 0x4ce44d1, 90 },
    { 0, 0 },
    { 0x413e44d4, 95 },
    { 0, 0 },
    { 0xd4de44d6, 90 },
    { 0x63ce44d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee44da, 33 },
    { 0, 0 },
    { 0x23ce44dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ae44e0, 95 },
    { 0x48ce44e0, 93 },
    { 0x511e44e2, 40 },
    { 0x354e44e2, 98 },
    { 0, 0 },
    { 0x41de44e5, 55 },
    { 0xf70e44e6, 39 },
    { 0x898e44e7, 43 },
    { 0, 0 },
    { 0x719e44e9, 52 },
    { 0xbe0e44e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242e44f1, 83 },
    { 0x44ae44f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de44f6, 98 },
    { 0, 0 },
    { 0x358e44f8, 99 },
    { 0xd3ce44f9, 98 },
    { 0, 0 },
    { 0x67ee44fb, 95 },
    { 0, 0 },
    { 0xf5ce44fd, 93 },
    { 0x7b3e44fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e4501, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe4511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ee4514, 46 },
    { 0xb3ee4515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e4518, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963e451e, 90 },
    { 0, 0 },
    { 0xbc6e4520, 95 },
    { 0, 0 },
    { 0x9bce4522, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e4528, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee452c, 90 },
    { 0, 0 },
    { 0xb93e452e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e4533, 65 },
    { 0, 0 },
    { 0x82e4535, 77 },
    { 0x8e1e4536, 76 },
    { 0x930e4537, 66 },
    { 0x888e4538, 33 },
    { 0, 0 },
    { 0x5dfe453a, 46 },
    { 0, 0 },
    { 0xfbee453c, 33 },
    { 0x4f5e453c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e4540, 17 },
    { 0x879e4541, 98 },
    { 0, 0 },
    { 0xba3e4543, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41e454a, 91 },
    { 0x2ce454b, 33 },
    { 0x4dae454b, 52 },
    { 0xb75e454d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee4550, 78 },
    { 0x964e4550, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce4557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e455c, 46 },
    { 0, 0 },
    { 0xba5e455e, 90 },
    { 0x1c7e455f, 95 },
    { 0xd1ee4560, 33 },
    { 0x4a3e4561, 81 },
    { 0x13be4561, 33 },
    { 0xb2e4563, 90 },
    { 0xc2be4560, 95 },
    { 0x938e4565, 43 },
    { 0x562e4566, 78 },
    { 0xba3e4561, 33 },
    { 0x4fce4568, 61 },
    { 0xea7e4569, 66 },
    { 0x2b5e456a, 23 },
    { 0x4b1e455e, 87 },
    { 0x5e4e4561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x777e456f, 98 },
    { 0xd2ae456f, 95 },
    { 0, 0 },
    { 0x1b9e4572, 33 },
    { 0, 0 },
    { 0xc11e4574, 90 },
    { 0, 0 },
    { 0xf68e4576, 90 },
    { 0x5c0e4577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbe457a, 91 },
    { 0xb7e457b, 90 },
    { 0x602e457c, 34 },
    { 0, 0 },
    { 0x93be457e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e4581, 90 },
    { 0x5bbe4582, 90 },
    { 0x840e4581, 90 },
    { 0xa9be4584, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e458c, 33 },
    { 0x4dfe458d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e4594, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be45a4, 90 },
    { 0xd8de45a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e45a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e45ad, 33 },
    { 0, 0 },
    { 0x974e45af, 100 },
    { 0x854e45af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7e45b4, 98 },
    { 0, 0 },
    { 0xa5de45b6, 90 },
    { 0x6f3e45b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e45be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e45cc, 90 },
    { 0xa43e45cd, 61 },
    { 0, 0 },
    { 0x72be45cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731e45d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520e45dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40e45e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e45e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542e45e7, 90 },
    { 0x636e45e8, 78 },
    { 0, 0 },
    { 0xef5e45ea, 34 },
    { 0xf12e45eb, 55 },
    { 0x8efe45ec, 99 },
    { 0x8a1e45ec, 95 },
    { 0x1bbe45ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae45f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde45f6, 46 },
    { 0x3fde45f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95be45fb, 33 },
    { 0x18ce45fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e45ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e4602, 61 },
    { 0x561e4603, 95 },
    { 0, 0 },
    { 0x8b1e4605, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fde4608, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e460c, 23 },
    { 0, 0 },
    { 0xb31e460e, 23 },
    { 0x82de460f, 33 },
    { 0x666e4610, 17 },
    { 0xfede4611, 33 },
    { 0x262e4610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5e4617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e461d, 43 },
    { 0xb8ae461e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103e4626, 91 },
    { 0xd25e4627, 46 },
    { 0xc47e4627, 66 },
    { 0x743e4628, 84 },
    { 0x3d7e4629, 61 },
    { 0, 0 },
    { 0x428e462c, 46 },
    { 0x522e462c, 33 },
    { 0xf2e462e, 95 },
    { 0, 0 },
    { 0x3cae4630, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14e4633, 98 },
    { 0xe34e4633, 43 },
    { 0, 0 },
    { 0x8cfe4636, 95 },
    { 0, 0 },
    { 0x810e4638, 34 },
    { 0, 0 },
    { 0xbd6e463a, 91 },
    { 0x9c0e463b, 76 },
    { 0xbf6e463c, 33 },
    { 0x5e0e463d, 95 },
    { 0x324e463e, 99 },
    { 0x5b3e463f, 80 },
    { 0xf32e4640, 98 },
    { 0, 0 },
    { 0x5e7e4642, 46 },
    { 0xca1e4643, 61 },
    { 0xd76e4644, 90 },
    { 0x8afe4645, 91 },
    { 0, 0 },
    { 0xa1ae4647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1e464b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x778e464e, 33 },
    { 0xe6fe464f, 66 },
    { 0, 0 },
    { 0xb7e4651, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5de4654, 61 },
    { 0xb98e4654, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e4661, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90de466a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce466d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e4671, 23 },
    { 0xbcde4672, 98 },
    { 0x4a8e4672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de4677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69be467a, 46 },
    { 0x140e467b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206e4681, 33 },
    { 0x432e4681, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bee4686, 17 },
    { 0x7a8e4686, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce468b, 61 },
    { 0xa16e468c, 95 },
    { 0, 0 },
    { 0x6efe468e, 33 },
    { 0x110e468e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae4693, 43 },
    { 0x144e4694, 55 },
    { 0x852e4694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7e469d, 95 },
    { 0, 0 },
    { 0x95de469f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8e46a2, 90 },
    { 0xb4de46a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6e46a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbe46ac, 99 },
    { 0, 0 },
    { 0x7c4e46ae, 61 },
    { 0x8a9e46af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffe46b5, 61 },
    { 0x1cce46b6, 33 },
    { 0, 0 },
    { 0xbeee46b8, 87 },
    { 0, 0 },
    { 0xaa1e46ba, 67 },
    { 0x586e46bb, 52 },
    { 0, 0 },
    { 0xffde46bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345e46c1, 84 },
    { 0, 0 },
    { 0xccfe46c3, 61 },
    { 0, 0 },
    { 0xd75e46c5, 33 },
    { 0, 0 },
    { 0x634e46c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e46ce, 78 },
    { 0x36ee46cf, 33 },
    { 0x7bae46d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddce46d6, 98 },
    { 0, 0 },
    { 0x4e7e46d8, 90 },
    { 0, 0 },
    { 0xe31e46da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33e46dd, 34 },
    { 0, 0 },
    { 0x86e46df, 34 },
    { 0, 0 },
    { 0x8bee46e1, 95 },
    { 0x46e46e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ae46f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe46f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e46f8, 98 },
    { 0xfede46f8, 5 },
    { 0, 0 },
    { 0x7dee46fb, 95 },
    { 0, 0 },
    { 0xd02e46fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fae4703, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5e470b, 90 },
    { 0x79ae470c, 99 },
    { 0xdeee470d, 66 },
    { 0x59ae470e, 90 },
    { 0xe5de470f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e4715, 68 },
    { 0x51be4715, 93 },
    { 0, 0 },
    { 0x8c8e4718, 90 },
    { 0x8fee4719, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ee471d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e4727, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x274e472a, 95 },
    { 0, 0 },
    { 0xacae472c, 33 },
    { 0x917e472d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede4731, 67 },
    { 0, 0 },
    { 0x5fe4733, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9e4738, 33 },
    { 0, 0 },
    { 0x36e473a, 95 },
    { 0x290e473a, 17 },
    { 0, 0 },
    { 0xffce473d, 98 },
    { 0x510e473e, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de4743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e4746, 66 },
    { 0, 0 },
    { 0x8d6e4748, 43 },
    { 0x5b5e4749, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe474c, 90 },
    { 0, 0 },
    { 0x238e474e, 67 },
    { 0x50ce474f, 33 },
    { 0xc7ce4750, 33 },
    { 0x737e474e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ce4756, 76 },
    { 0xf7ce4757, 90 },
    { 0x28fe4758, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e475c, 90 },
    { 0x8f9e475c, 43 },
    { 0x54ae475e, 61 },
    { 0xf27e475f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84e4762, 55 },
    { 0x4dee4763, 46 },
    { 0xeaee4763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6e476d, 98 },
    { 0xe76e476e, 90 },
    { 0xd32e476f, 90 },
    { 0xe78e476f, 90 },
    { 0xd08e4770, 23 },
    { 0x90de476f, 95 },
    { 0x541e4773, 98 },
    { 0x391e476e, 55 },
    { 0x852e4775, 43 },
    { 0xd40e4776, 33 },
    { 0x81be4771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ae477a, 98 },
    { 0x5e4e477a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fe477f, 61 },
    { 0xa4fe4780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e4784, 90 },
    { 0, 0 },
    { 0x2e8e4786, 52 },
    { 0, 0 },
    { 0x1d2e4788, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274e478f, 46 },
    { 0x1efe4790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e4796, 33 },
    { 0xb51e4797, 33 },
    { 0x579e4798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ce479b, 95 },
    { 0x78de479b, 98 },
    { 0xcd7e479d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe47a0, 34 },
    { 0, 0 },
    { 0x4a1e47a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22e47ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e47c1, 95 },
    { 0x32ce47c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e47c7, 100 },
    { 0, 0 },
    { 0x45ee47c9, 93 },
    { 0x94be47ca, 90 },
    { 0xca7e47cb, 78 },
    { 0, 0 },
    { 0x83de47cd, 98 },
    { 0xf76e47ce, 33 },
    { 0xbd5e47ce, 34 },
    { 0x2d4e47ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e47d8, 23 },
    { 0, 0 },
    { 0x8dce47da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be47e0, 90 },
    { 0x131e47e1, 90 },
    { 0x841e47e2, 55 },
    { 0xec7e47e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e47e9, 49 },
    { 0xbd1e47ea, 76 },
    { 0x636e47e9, 43 },
    { 0, 0 },
    { 0xd9ae47ed, 93 },
    { 0x60be47ee, 55 },
    { 0xbe3e47ef, 85 },
    { 0x331e47f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe47f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327e47ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e480e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59e4812, 33 },
    { 0x564e4813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e4819, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x896e481c, 95 },
    { 0x94ce481d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0e4826, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443e482b, 78 },
    { 0, 0 },
    { 0x362e482d, 90 },
    { 0x29ce482e, 98 },
    { 0x644e482d, 90 },
    { 0xc68e4830, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40e4834, 95 },
    { 0, 0 },
    { 0x5a3e4836, 34 },
    { 0x519e4837, 95 },
    { 0x547e4836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e4841, 98 },
    { 0x67ee4841, 36 },
    { 0, 0 },
    { 0xc57e4844, 98 },
    { 0x9f8e4845, 66 },
    { 0xe7ce4846, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefde4849, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1e484e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25e4852, 61 },
    { 0xa38e4853, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e4857, 95 },
    { 0x700e4857, 46 },
    { 0x81fe4858, 33 },
    { 0, 0 },
    { 0x29be485b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x359e485e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bce4861, 91 },
    { 0x45fe4862, 98 },
    { 0x15be4862, 61 },
    { 0xaa1e4864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88e486a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e486f, 56 },
    { 0, 0 },
    { 0x34ee4871, 76 },
    { 0x12be4872, 98 },
    { 0x9fce4873, 90 },
    { 0, 0 },
    { 0x34de4875, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23de4879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4e487c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e4880, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be4884, 101 },
    { 0, 0 },
    { 0xb30e4886, 61 },
    { 0x820e4887, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22e488c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e4891, 98 },
    { 0xcd4e4892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e4895, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e4899, 61 },
    { 0x8dfe4899, 95 },
    { 0x3c8e489b, 76 },
    { 0, 0 },
    { 0xe95e489d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e48a7, 23 },
    { 0x411e48a7, 34 },
    { 0x17fe48a9, 87 },
    { 0x374e48a9, 90 },
    { 0x3c7e48a8, 95 },
    { 0x9ace48ac, 34 },
    { 0x9eae48ad, 23 },
    { 0xed5e48ae, 90 },
    { 0x495e48af, 55 },
    { 0, 0 },
    { 0x4f9e48b1, 76 },
    { 0, 0 },
    { 0x104e48b3, 78 },
    { 0xe63e48b4, 95 },
    { 0xb71e48b5, 34 },
    { 0, 0 },
    { 0xf77e48b7, 55 },
    { 0, 0 },
    { 0xc16e48b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35e48bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05e48c7, 91 },
    { 0xabee48c8, 33 },
    { 0xf4ce48c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207e48ce, 55 },
    { 0xa3fe48ce, 61 },
    { 0xd3de48d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe48db, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e48df, 98 },
    { 0x734e48e0, 61 },
    { 0x40e48e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46e48e6, 61 },
    { 0x539e48e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6e48ec, 95 },
    { 0, 0 },
    { 0xcf0e48ee, 14 },
    { 0, 0 },
    { 0x72ae48f0, 75 },
    { 0, 0 },
    { 0xb82e48f2, 98 },
    { 0, 0 },
    { 0xabde48f4, 17 },
    { 0x857e48f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e48f9, 99 },
    { 0xffce48fa, 64 },
    { 0xba8e48fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90be48ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95e4907, 100 },
    { 0x673e4907, 90 },
    { 0x8aae4907, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e4911, 33 },
    { 0, 0 },
    { 0xc8ce4913, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbe4916, 52 },
    { 0x824e4917, 68 },
    { 0x530e4918, 23 },
    { 0x259e4917, 90 },
    { 0, 0 },
    { 0x1afe491b, 61 },
    { 0x2bde491b, 95 },
    { 0xdd8e491d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be4921, 33 },
    { 0xe31e4922, 98 },
    { 0xb43e4923, 23 },
    { 0, 0 },
    { 0xd8ce4925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e4929, 55 },
    { 0x900e4929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e4933, 100 },
    { 0x745e4934, 52 },
    { 0xf34e4935, 95 },
    { 0x265e4933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e493a, 67 },
    { 0, 0 },
    { 0xf32e493c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ee493f, 61 },
    { 0x62ae4940, 95 },
    { 0, 0 },
    { 0xe4ae4942, 23 },
    { 0x425e4942, 93 },
    { 0, 0 },
    { 0xf3fe4945, 36 },
    { 0, 0 },
    { 0x875e4947, 90 },
    { 0xe00e4947, 90 },
    { 0x68de4949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ae494d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe4957, 94 },
    { 0xe81e4957, 34 },
    { 0xd91e4958, 98 },
    { 0, 0 },
    { 0x897e495b, 78 },
    { 0x115e495c, 23 },
    { 0, 0 },
    { 0x484e495e, 33 },
    { 0xf83e495f, 55 },
    { 0x696e495e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e4964, 46 },
    { 0x780e4965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e4968, 61 },
    { 0, 0 },
    { 0xeae496a, 61 },
    { 0, 0 },
    { 0x599e496c, 61 },
    { 0xe5ce496c, 90 },
    { 0, 0 },
    { 0xa71e496f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683e4975, 90 },
    { 0x70fe4976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e497d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ce4980, 93 },
    { 0xf7be4980, 55 },
    { 0x695e4982, 33 },
    { 0x707e4983, 23 },
    { 0x1b3e4983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7e4989, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e498d, 76 },
    { 0x55de498e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e4993, 98 },
    { 0x71de4994, 33 },
    { 0x4fce4993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ce499c, 43 },
    { 0, 0 },
    { 0x614e499e, 65 },
    { 0xa78e499f, 23 },
    { 0x52ce49a0, 90 },
    { 0xfaae49a0, 61 },
    { 0xbdae49a1, 95 },
    { 0, 0 },
    { 0xc7e49a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e49a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eae49ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83de49ae, 43 },
    { 0x2ffe49ae, 67 },
    { 0x9aee49b0, 33 },
    { 0x6bae49b1, 40 },
    { 0x85e49b2, 77 },
    { 0xbd6e49b0, 93 },
    { 0x67be49b4, 97 },
    { 0, 0 },
    { 0x2c2e49b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7e49b9, 52 },
    { 0x331e49ba, 90 },
    { 0, 0 },
    { 0x2c7e49bc, 33 },
    { 0xed6e49bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe49c0, 98 },
    { 0x609e49c0, 95 },
    { 0, 0 },
    { 0xc71e49c3, 98 },
    { 0x658e49c4, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440e49cb, 87 },
    { 0, 0 },
    { 0x41be49cd, 61 },
    { 0x1e5e49cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x248e49d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29e49d6, 76 },
    { 0xbade49d7, 61 },
    { 0x764e49d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15be49df, 61 },
    { 0, 0 },
    { 0x635e49e1, 90 },
    { 0x73ce49e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ae49e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce49eb, 23 },
    { 0x6dfe49ec, 98 },
    { 0x2dae49ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e49f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1e49f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ede49f8, 33 },
    { 0xb58e49f9, 95 },
    { 0xe6ce49fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde49fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bce4a00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be4a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ee4a06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e4a0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41e4a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ee4a15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fe4a1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54e4a23, 90 },
    { 0xfdce4a24, 23 },
    { 0x121e4a24, 46 },
    { 0xb7de4a26, 52 },
    { 0xa89e4a27, 90 },
    { 0, 0 },
    { 0x271e4a29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e4a3b, 90 },
    { 0, 0 },
    { 0x938e4a3d, 33 },
    { 0, 0 },
    { 0x90de4a3f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fe4a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2e4a48, 55 },
    { 0x5de4a48, 95 },
    { 0x5b0e4a4a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e4a4d, 90 },
    { 0x6a1e4a4e, 98 },
    { 0, 0 },
    { 0x886e4a50, 23 },
    { 0, 0 },
    { 0x2c8e4a52, 43 },
    { 0, 0 },
    { 0x4bde4a54, 78 },
    { 0x46ee4a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237e4a60, 33 },
    { 0xb75e4a61, 90 },
    { 0x303e4a62, 77 },
    { 0xa7fe4a63, 33 },
    { 0x4dfe4a64, 52 },
    { 0xf82e4a65, 33 },
    { 0xb19e4a65, 76 },
    { 0xd36e4a66, 61 },
    { 0x502e4a63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2be4a6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439e4a71, 77 },
    { 0x3b7e4a71, 96 },
    { 0x771e4a72, 95 },
    { 0xb15e4a74, 33 },
    { 0xdf8e4a75, 95 },
    { 0xa4ae4a76, 52 },
    { 0x54e4a77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x667e4a7a, 83 },
    { 0, 0 },
    { 0xc24e4a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fe4a84, 89 },
    { 0x124e4a84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3e4a89, 84 },
    { 0xa5fe4a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62de4a8f, 33 },
    { 0x85e4a90, 23 },
    { 0x259e4a90, 33 },
    { 0xeb5e4a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19e4a96, 61 },
    { 0x607e4a97, 83 },
    { 0xcc8e4a98, 80 },
    { 0xb97e4a99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186e4a9e, 87 },
    { 0xa7be4a9f, 33 },
    { 0, 0 },
    { 0x412e4aa1, 100 },
    { 0xe61e4aa2, 64 },
    { 0, 0 },
    { 0x8b5e4aa4, 61 },
    { 0xd27e4aa4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9e4aaa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ce4aaf, 95 },
    { 0x166e4aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e4ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e4abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8e4ac0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4ac4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabe4acd, 34 },
    { 0x4a8e4ace, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ce4ad1, 76 },
    { 0xe62e4ad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24e4ad5, 23 },
    { 0x3fee4ad6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae4adb, 55 },
    { 0x29ee4adb, 33 },
    { 0x125e4adb, 61 },
    { 0xaf5e4ade, 93 },
    { 0x228e4adf, 55 },
    { 0x6c5e4adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4ae6, 61 },
    { 0xa57e4ae7, 93 },
    { 0x376e4ae7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fde4aec, 90 },
    { 0, 0 },
    { 0x736e4aee, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0e4af1, 95 },
    { 0xdce4af2, 45 },
    { 0xe4ae4af2, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77e4af6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e4afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641e4b01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e4b08, 84 },
    { 0xedce4b09, 55 },
    { 0, 0 },
    { 0x9c0e4b0b, 98 },
    { 0xeee4b0c, 98 },
    { 0xedae4b0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e4b12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe4b16, 98 },
    { 0xf4ae4b17, 33 },
    { 0x908e4b18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace4b1b, 61 },
    { 0xe93e4b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7e4b25, 34 },
    { 0x11ae4b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e4b2b, 34 },
    { 0x6c4e4b2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce4b2f, 33 },
    { 0xba3e4b30, 61 },
    { 0x9e5e4b31, 33 },
    { 0xcee4b31, 98 },
    { 0, 0 },
    { 0xa0de4b34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4b37, 34 },
    { 0x7b5e4b38, 52 },
    { 0xc8ce4b38, 33 },
    { 0x256e4b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece4b3e, 61 },
    { 0, 0 },
    { 0xe2de4b40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x447e4b43, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e4b4c, 36 },
    { 0x53de4b4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8e4b50, 98 },
    { 0xbd9e4b50, 90 },
    { 0, 0 },
    { 0xd1fe4b53, 43 },
    { 0x153e4b54, 90 },
    { 0x825e4b55, 95 },
    { 0, 0 },
    { 0x3e0e4b57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e4b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fe4b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfe4b63, 95 },
    { 0x299e4b64, 67 },
    { 0x539e4b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32be4b6a, 33 },
    { 0xb6ee4b6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e4b71, 90 },
    { 0xb91e4b72, 66 },
    { 0x22fe4b73, 90 },
    { 0x6b5e4b74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91e4b79, 46 },
    { 0xe87e4b7a, 95 },
    { 0xae3e4b7a, 98 },
    { 0, 0 },
    { 0x4f4e4b7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe4b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436e4b8a, 95 },
    { 0x279e4b8b, 43 },
    { 0x647e4b8c, 87 },
    { 0xd81e4b8d, 76 },
    { 0x6fde4b8c, 90 },
    { 0x56e4b8f, 87 },
    { 0xdfce4b90, 95 },
    { 0x63fe4b91, 36 },
    { 0xf97e4b8f, 55 },
    { 0x21ee4b8a, 81 },
    { 0x296e4b94, 95 },
    { 0, 0 },
    { 0xc9e4b96, 90 },
    { 0xf28e4b96, 90 },
    { 0x162e4b98, 95 },
    { 0x9b9e4b99, 93 },
    { 0xe4fe4b9a, 68 },
    { 0, 0 },
    { 0xd97e4b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ce4ba0, 43 },
    { 0x16de4ba1, 88 },
    { 0, 0 },
    { 0x3ece4ba3, 61 },
    { 0xa56e4ba4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633e4baa, 97 },
    { 0, 0 },
    { 0x2bce4bac, 33 },
    { 0xae9e4bac, 98 },
    { 0xb2e4bae, 90 },
    { 0x8bae4bad, 52 },
    { 0, 0 },
    { 0xc07e4bb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece4bba, 55 },
    { 0x381e4bbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e4bbe, 88 },
    { 0x23be4bbf, 55 },
    { 0x383e4bc0, 55 },
    { 0, 0 },
    { 0xe05e4bc2, 23 },
    { 0x96be4bc3, 90 },
    { 0x1be4bc4, 76 },
    { 0x627e4bc5, 33 },
    { 0x5e7e4bc5, 65 },
    { 0, 0 },
    { 0xf9ae4bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de4bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce4bd2, 23 },
    { 0x687e4bd3, 33 },
    { 0x1d7e4bd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e4bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5e4bdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fe4be3, 23 },
    { 0x553e4be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65e4be8, 95 },
    { 0xcabe4be9, 81 },
    { 0, 0 },
    { 0xf35e4beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09e4bef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e4bf2, 4 },
    { 0x23de4bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae4bf9, 90 },
    { 0, 0 },
    { 0xacde4bfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e4c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10e4c03, 90 },
    { 0x2c2e4c04, 90 },
    { 0xea1e4c05, 43 },
    { 0x555e4c04, 90 },
    { 0x6f4e4c05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae4c0a, 91 },
    { 0, 0 },
    { 0xcc3e4c0c, 91 },
    { 0, 0 },
    { 0xcd3e4c0e, 98 },
    { 0xcfee4c0f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8e4c19, 82 },
    { 0xa3fe4c19, 88 },
    { 0x7c6e4c19, 23 },
    { 0xb02e4c1c, 100 },
    { 0xbc3e4c1c, 87 },
    { 0x9bbe4c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x782e4c21, 33 },
    { 0, 0 },
    { 0x234e4c23, 34 },
    { 0xf9ee4c24, 33 },
    { 0x8ede4c25, 99 },
    { 0xcebe4c25, 76 },
    { 0x372e4c25, 90 },
    { 0x247e4c28, 67 },
    { 0x1e5e4c29, 90 },
    { 0xa8ee4c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dee4c2d, 40 },
    { 0x3b6e4c2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9e4c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e4c36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356e4c3a, 48 },
    { 0x88be4c3b, 43 },
    { 0x8b4e4c3c, 90 },
    { 0x906e4c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e4c46, 95 },
    { 0, 0 },
    { 0xc5ee4c48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfce4c4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aee4c55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4c58, 90 },
    { 0x2dde4c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75be4c5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1e4c63, 61 },
    { 0x125e4c63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871e4c6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e4c6e, 17 },
    { 0xbdae4c6f, 34 },
    { 0xfe3e4c6e, 90 },
    { 0, 0 },
    { 0x17be4c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe4c75, 61 },
    { 0x342e4c76, 90 },
    { 0xe24e4c77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19be4c7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26e4c7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdae4c83, 61 },
    { 0x217e4c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefe4c87, 99 },
    { 0x1de4c87, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8e4c8f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce4c93, 8 },
    { 0x254e4c94, 77 },
    { 0, 0 },
    { 0x786e4c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e4c99, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835e4c9f, 81 },
    { 0x7aae4c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce4ca3, 61 },
    { 0x243e4ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afe4caa, 65 },
    { 0, 0 },
    { 0xfece4cac, 95 },
    { 0, 0 },
    { 0xa00e4cae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e4cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e4cb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x474e4cb7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e4cba, 90 },
    { 0, 0 },
    { 0xd11e4cbc, 95 },
    { 0x495e4cbd, 23 },
    { 0, 0 },
    { 0xd83e4cbf, 46 },
    { 0xec1e4cbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e4cc4, 33 },
    { 0x5dde4cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e4cce, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e4cd1, 92 },
    { 0x209e4cd2, 98 },
    { 0x44de4cd1, 80 },
    { 0xfb3e4cd4, 36 },
    { 0x9d8e4cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e4ce2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e4ce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7e4ceb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e4cef, 76 },
    { 0xbbde4cf0, 91 },
    { 0xd7e4cef, 93 },
    { 0x46ee4cf2, 95 },
    { 0x41ee4cf3, 90 },
    { 0, 0 },
    { 0xd28e4cf5, 33 },
    { 0xbc5e4cf6, 43 },
    { 0x329e4cf6, 87 },
    { 0, 0 },
    { 0x63ce4cf9, 90 },
    { 0, 0 },
    { 0xb4e4cfb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e4cfe, 61 },
    { 0xf4ce4cfe, 61 },
    { 0x373e4d00, 84 },
    { 0xd9e4d01, 99 },
    { 0, 0 },
    { 0x469e4d03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473e4d07, 99 },
    { 0, 0 },
    { 0x71ee4d09, 55 },
    { 0, 0 },
    { 0xc3e4d0b, 61 },
    { 0, 0 },
    { 0x1d2e4d0d, 43 },
    { 0, 0 },
    { 0x20be4d0f, 34 },
    { 0x264e4d10, 88 },
    { 0x3bbe4d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e4d18, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05e4d1e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbe4d23, 98 },
    { 0x91fe4d24, 43 },
    { 0xe53e4d24, 76 },
    { 0x790e4d25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50e4d29, 98 },
    { 0x54be4d2a, 93 },
    { 0, 0 },
    { 0xbffe4d2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912e4d33, 76 },
    { 0xdb8e4d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e4d38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2e4d3c, 98 },
    { 0, 0 },
    { 0x3f9e4d3e, 36 },
    { 0x987e4d3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124e4d44, 93 },
    { 0x5ffe4d45, 90 },
    { 0, 0 },
    { 0xe02e4d47, 95 },
    { 0, 0 },
    { 0xcc9e4d49, 23 },
    { 0x77ce4d4a, 98 },
    { 0x432e4d4b, 61 },
    { 0xc56e4d4c, 23 },
    { 0, 0 },
    { 0x932e4d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce4d51, 98 },
    { 0, 0 },
    { 0xb15e4d53, 87 },
    { 0x8dae4d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e4d57, 66 },
    { 0, 0 },
    { 0xdb3e4d59, 91 },
    { 0x645e4d5a, 55 },
    { 0x3c8e4d5b, 33 },
    { 0, 0 },
    { 0xd93e4d5d, 46 },
    { 0x7a3e4d5e, 46 },
    { 0x44e4d5e, 91 },
    { 0x7c5e4d5f, 90 },
    { 0, 0 },
    { 0xc88e4d62, 95 },
    { 0xa26e4d63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae4d6d, 68 },
    { 0xeeee4d6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebe4d71, 34 },
    { 0x87fe4d72, 34 },
    { 0xcd8e4d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e4d76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4e4d7a, 94 },
    { 0xf73e4d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208e4d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e4d84, 66 },
    { 0xb5ae4d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e4d88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e4d93, 90 },
    { 0x207e4d94, 61 },
    { 0, 0 },
    { 0x721e4d96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5e4d9a, 90 },
    { 0, 0 },
    { 0x65fe4d9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x333e4d9f, 61 },
    { 0x902e4da0, 93 },
    { 0, 0 },
    { 0x5be4da2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5e4da6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32e4da9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de4daf, 33 },
    { 0x1f3e4db0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e4db4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce4db7, 61 },
    { 0x714e4db7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840e4dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de4dc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69e4dc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade4dc8, 67 },
    { 0x3f9e4dc9, 90 },
    { 0, 0 },
    { 0x162e4dcb, 95 },
    { 0x43be4dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e4dcf, 43 },
    { 0x88ee4dcf, 33 },
    { 0x6a8e4dd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dee4dd6, 99 },
    { 0x2aae4dd7, 43 },
    { 0x7cfe4dd8, 81 },
    { 0xd4de4dd8, 34 },
    { 0xdc8e4dda, 87 },
    { 0x18ce4dd8, 23 },
    { 0x8fe4dd7, 34 },
    { 0x91de4dd9, 90 },
    { 0xa9de4dde, 43 },
    { 0x8dde4ddf, 68 },
    { 0xb9de4ddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e4de8, 61 },
    { 0xa2e4de9, 46 },
    { 0xb84e4dea, 46 },
    { 0x460e4deb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4e4df1, 98 },
    { 0, 0 },
    { 0x2dfe4df3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e4df6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314e4dfa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bae4dfd, 55 },
    { 0xeb0e4dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33e4e04, 95 },
    { 0xe08e4e05, 87 },
    { 0xb04e4e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce4e0b, 90 },
    { 0x6cfe4e0b, 61 },
    { 0x6b9e4e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce4e11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dee4e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e4e1c, 17 },
    { 0xa18e4e1d, 95 },
    { 0xfe9e4e1e, 65 },
    { 0, 0 },
    { 0xe9fe4e20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25e4e23, 93 },
    { 0, 0 },
    { 0xd29e4e25, 61 },
    { 0x93ee4e26, 46 },
    { 0xc59e4e27, 95 },
    { 0x65ae4e28, 53 },
    { 0xd67e4e29, 78 },
    { 0x8f1e4e27, 95 },
    { 0xd83e4e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e4e30, 61 },
    { 0x93fe4e30, 84 },
    { 0x507e4e31, 46 },
    { 0x5e6e4e33, 61 },
    { 0x5e6e4e34, 52 },
    { 0x5cfe4e33, 55 },
    { 0, 0 },
    { 0x9e5e4e37, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260e4e3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ce4e3f, 33 },
    { 0xe52e4e3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95e4e4e, 90 },
    { 0x324e4e4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e4e52, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce4e58, 52 },
    { 0x720e4e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790e4e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625e4e60, 81 },
    { 0x294e4e61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e4e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e4e69, 46 },
    { 0xcaae4e6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10be4e70, 33 },
    { 0xb83e4e71, 23 },
    { 0xb01e4e72, 17 },
    { 0x6a8e4e70, 90 },
    { 0xffe4e74, 61 },
    { 0x3f9e4e72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59de4e78, 46 },
    { 0x760e4e78, 33 },
    { 0x547e4e79, 33 },
    { 0, 0 },
    { 0xc01e4e7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbe4e7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe4e82, 33 },
    { 0xe50e4e82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113e4e88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce4e8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8e4e91, 90 },
    { 0x32ce4e92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x600e4e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810e4e9d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e4ea1, 65 },
    { 0, 0 },
    { 0xd0ee4ea3, 46 },
    { 0xed3e4ea3, 61 },
    { 0, 0 },
    { 0x213e4ea6, 99 },
    { 0, 0 },
    { 0x517e4ea8, 88 },
    { 0, 0 },
    { 0xa76e4eaa, 69 },
    { 0, 0 },
    { 0xc29e4eac, 85 },
    { 0, 0 },
    { 0x4cae4eae, 33 },
    { 0x8a5e4eaf, 98 },
    { 0xd44e4eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cae4eb6, 85 },
    { 0xf8de4eb7, 65 },
    { 0xf8ee4eb8, 34 },
    { 0x918e4eb9, 23 },
    { 0xbb3e4eba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ae4ebd, 23 },
    { 0x38be4ebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1e4ec1, 96 },
    { 0xb3ee4ec1, 33 },
    { 0x7a8e4ec3, 98 },
    { 0x16be4ec4, 90 },
    { 0x108e4ec1, 61 },
    { 0xbdde4ec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e4ecc, 55 },
    { 0xdcee4ecc, 69 },
    { 0x487e4ece, 33 },
    { 0xc41e4ecf, 90 },
    { 0x6f0e4ecf, 33 },
    { 0xd8fe4ecc, 33 },
    { 0x880e4ed1, 33 },
    { 0x50de4ecd, 43 },
    { 0xed4e4ed4, 90 },
    { 0x7b8e4ed5, 68 },
    { 0x2a7e4ed6, 23 },
    { 0x197e4ed6, 76 },
    { 0, 0 },
    { 0x773e4ed9, 95 },
    { 0x9c7e4ed9, 33 },
    { 0x2a3e4edb, 87 },
    { 0x323e4edc, 23 },
    { 0x560e4edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dde4ee0, 100 },
    { 0x9fbe4ee0, 23 },
    { 0xe97e4ee2, 90 },
    { 0x2e0e4ee1, 95 },
    { 0x5abe4ee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ee4ee8, 76 },
    { 0x6c4e4ee9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ae4eee, 93 },
    { 0, 0 },
    { 0x90ae4ef0, 43 },
    { 0xd4e4ef1, 23 },
    { 0x867e4ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be4ef5, 61 },
    { 0, 0 },
    { 0xc46e4ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38e4efa, 84 },
    { 0xbfae4efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ee4efe, 95 },
    { 0x9bee4efe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8e4f0c, 67 },
    { 0xa19e4f0d, 33 },
    { 0x4dee4f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefce4f12, 98 },
    { 0, 0 },
    { 0x48e4f14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9e4f17, 90 },
    { 0, 0 },
    { 0xb8ce4f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3de4f1e, 23 },
    { 0x75ae4f1f, 81 },
    { 0xd68e4f1f, 95 },
    { 0x94be4f21, 98 },
    { 0, 0 },
    { 0x1dbe4f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cae4f2a, 34 },
    { 0x3aae4f2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e4f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde4f3b, 90 },
    { 0x259e4f3c, 78 },
    { 0x2eee4f3c, 61 },
    { 0x834e4f3e, 76 },
    { 0xcdce4f3c, 33 },
    { 0, 0 },
    { 0x40ae4f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e4f44, 90 },
    { 0x126e4f45, 46 },
    { 0x1a0e4f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e4f4b, 23 },
    { 0, 0 },
    { 0x1dae4f4d, 55 },
    { 0, 0 },
    { 0x7d0e4f4f, 99 },
    { 0x3a9e4f50, 66 },
    { 0xcdbe4f4f, 65 },
    { 0x450e4f52, 55 },
    { 0xa8ee4f51, 43 },
    { 0xa3de4f54, 52 },
    { 0, 0 },
    { 0x23fe4f56, 95 },
    { 0, 0 },
    { 0x58be4f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x885e4f5b, 47 },
    { 0x316e4f5b, 91 },
    { 0x98e4f5d, 23 },
    { 0x852e4f5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e4f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7e4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce4f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e4f70, 95 },
    { 0, 0 },
    { 0x885e4f72, 36 },
    { 0, 0 },
    { 0xba1e4f74, 61 },
    { 0, 0 },
    { 0x99be4f76, 84 },
    { 0, 0 },
    { 0x286e4f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e4f7c, 34 },
    { 0xe4fe4f7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42e4f80, 33 },
    { 0xf31e4f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2e4f84, 83 },
    { 0, 0 },
    { 0x8b9e4f86, 46 },
    { 0, 0 },
    { 0xf29e4f88, 99 },
    { 0x2cce4f88, 55 },
    { 0x4ae4f8a, 90 },
    { 0, 0 },
    { 0xd0ce4f8c, 95 },
    { 0, 0 },
    { 0xe90e4f8e, 90 },
    { 0xbf5e4f8f, 90 },
    { 0x56de4f90, 79 },
    { 0, 0 },
    { 0x86ce4f92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x107e4f95, 33 },
    { 0x8a6e4f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e4f99, 43 },
    { 0x899e4f9a, 98 },
    { 0, 0 },
    { 0x61e4f9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ce4fa1, 61 },
    { 0, 0 },
    { 0xfa9e4fa3, 90 },
    { 0xc4ae4fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x677e4fa7, 93 },
    { 0x72be4fa8, 61 },
    { 0x639e4fa9, 93 },
    { 0x271e4fa9, 90 },
    { 0xb20e4fab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748e4fb0, 12 },
    { 0x211e4fb1, 90 },
    { 0xece4fb0, 95 },
    { 0xaa6e4fb3, 33 },
    { 0, 0 },
    { 0x89fe4fb5, 95 },
    { 0, 0 },
    { 0xe47e4fb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ae4fbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be4fc2, 95 },
    { 0x59ce4fc3, 55 },
    { 0, 0 },
    { 0x9e4fc5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ee4fcd, 34 },
    { 0xfafe4fce, 80 },
    { 0x20e4fcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ce4fd2, 55 },
    { 0, 0 },
    { 0xaeee4fd4, 90 },
    { 0xc8de4fd5, 95 },
    { 0xee0e4fd4, 80 },
    { 0x407e4fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56be4fdc, 23 },
    { 0x79fe4fdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372e4fe4, 78 },
    { 0xc2fe4fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24be4fea, 33 },
    { 0x52fe4feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350e4ff0, 90 },
    { 0xdb3e4ff0, 93 },
    { 0x86e4ff2, 90 },
    { 0xa87e4ff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ff7, 33 },
    { 0x910e4ff8, 78 },
    { 0x763e4ff9, 76 },
    { 0x1f7e4ffa, 46 },
    { 0x62be4ffb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x464e4ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e5001, 45 },
    { 0xe7ce5002, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fe500b, 65 },
    { 0, 0 },
    { 0xf0ae500d, 61 },
    { 0x77de500e, 61 },
    { 0, 0 },
    { 0xe65e5010, 46 },
    { 0, 0 },
    { 0xa1ee5012, 90 },
    { 0, 0 },
    { 0x9c0e5014, 100 },
    { 0, 0 },
    { 0x1d0e5016, 33 },
    { 0, 0 },
    { 0xdbbe5018, 65 },
    { 0x4d7e5018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e501c, 43 },
    { 0, 0 },
    { 0x21fe501e, 53 },
    { 0x83ce501f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee5026, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x786e5029, 46 },
    { 0x663e502a, 61 },
    { 0x103e502a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be5031, 93 },
    { 0x8d6e5031, 55 },
    { 0xbf2e5032, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71e5036, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e503a, 52 },
    { 0x734e503a, 85 },
    { 0, 0 },
    { 0x890e503d, 90 },
    { 0, 0 },
    { 0x557e503f, 33 },
    { 0x8d7e5040, 23 },
    { 0xaefe5040, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee5044, 33 },
    { 0xbe5e5044, 90 },
    { 0, 0 },
    { 0x2d2e5047, 98 },
    { 0, 0 },
    { 0x59be5049, 88 },
    { 0, 0 },
    { 0x364e504b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3e5058, 52 },
    { 0, 0 },
    { 0xeede505a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e505d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678e5062, 93 },
    { 0xa76e5062, 95 },
    { 0x39e5064, 68 },
    { 0xa76e5064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e5069, 90 },
    { 0x404e5069, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4e506d, 88 },
    { 0, 0 },
    { 0x10e506f, 68 },
    { 0x19ae5070, 42 },
    { 0xee1e5071, 77 },
    { 0xd46e5072, 96 },
    { 0, 0 },
    { 0x925e5074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88e5080, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e5084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5e5087, 98 },
    { 0xc11e5088, 83 },
    { 0xad0e5088, 90 },
    { 0xdc4e508a, 95 },
    { 0xf55e508a, 95 },
    { 0x192e5087, 43 },
    { 0x1e5e508a, 23 },
    { 0x88e508e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae5092, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ee5097, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cee509b, 23 },
    { 0xb20e509c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae509f, 43 },
    { 0x5c6e50a0, 33 },
    { 0x1d3e50a0, 33 },
    { 0x6ebe509f, 61 },
    { 0, 0 },
    { 0xef1e50a4, 33 },
    { 0xb4be50a5, 66 },
    { 0x899e50a6, 90 },
    { 0x832e50a7, 90 },
    { 0xa31e50a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481e50ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae50af, 61 },
    { 0xd38e50b0, 69 },
    { 0xf9ae50b1, 78 },
    { 0x6bee50b1, 33 },
    { 0x1bee50b2, 67 },
    { 0x70ae50b0, 90 },
    { 0, 0 },
    { 0x6f4e50b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0e50b9, 75 },
    { 0x276e50ba, 23 },
    { 0xff2e50b9, 90 },
    { 0x48ee50bb, 90 },
    { 0, 0 },
    { 0x3a5e50be, 33 },
    { 0xea7e50be, 61 },
    { 0x64ae50c0, 90 },
    { 0, 0 },
    { 0xd84e50c2, 55 },
    { 0xe5ae50c3, 93 },
    { 0xd3fe50c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce50c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaae50cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424e50d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741e50d6, 43 },
    { 0x356e50d6, 36 },
    { 0, 0 },
    { 0x263e50d9, 95 },
    { 0, 0 },
    { 0x973e50db, 34 },
    { 0, 0 },
    { 0xb85e50dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9e50e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be50e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e50f2, 33 },
    { 0x63fe50f3, 98 },
    { 0x22e50f4, 66 },
    { 0xf98e50f5, 76 },
    { 0x3c6e50f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e50fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164e5103, 91 },
    { 0, 0 },
    { 0x97de5105, 67 },
    { 0xbcae5106, 33 },
    { 0xc0fe5107, 61 },
    { 0x864e5108, 23 },
    { 0, 0 },
    { 0x569e510a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee5111, 98 },
    { 0x4ffe5111, 66 },
    { 0, 0 },
    { 0xddfe5114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be5117, 90 },
    { 0x452e5118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fe5121, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x39de5124, 78 },
    { 0xd29e5124, 23 },
    { 0xd67e5125, 61 },
    { 0x438e5127, 55 },
    { 0xbd9e5128, 90 },
    { 0x88ee5128, 55 },
    { 0x865e5129, 61 },
    { 0xd97e512b, 55 },
    { 0x96be5128, 90 },
    { 0, 0 },
    { 0x342e512e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2e5131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e5134, 91 },
    { 0x8e2e5135, 88 },
    { 0xa2ce5136, 91 },
    { 0, 0 },
    { 0x2e6e5138, 90 },
    { 0xc7de5139, 78 },
    { 0x7dee5139, 65 },
    { 0xce6e5139, 61 },
    { 0x88fe5139, 90 },
    { 0x3fae513b, 23 },
    { 0x886e5139, 43 },
    { 0xac7e513f, 46 },
    { 0x297e5139, 33 },
    { 0x8d2e5141, 98 },
    { 0x9b0e5142, 76 },
    { 0xd19e5142, 61 },
    { 0x53ce5144, 23 },
    { 0x285e5144, 100 },
    { 0, 0 },
    { 0x35e5147, 90 },
    { 0xd77e5148, 78 },
    { 0x5efe5147, 95 },
    { 0xb52e514a, 23 },
    { 0x13ee514b, 91 },
    { 0x584e5148, 95 },
    { 0x5dae514c, 43 },
    { 0, 0 },
    { 0x89de514f, 95 },
    { 0x9a5e5150, 33 },
    { 0, 0 },
    { 0xf18e5152, 38 },
    { 0x23ce5153, 100 },
    { 0, 0 },
    { 0x5e1e5155, 95 },
    { 0x753e5156, 90 },
    { 0xb42e5156, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e515a, 23 },
    { 0x22be515a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee515e, 87 },
    { 0x9ffe515f, 66 },
    { 0x9eae5160, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6e5165, 90 },
    { 0x250e5166, 95 },
    { 0x15ae5166, 76 },
    { 0, 0 },
    { 0xf0e5169, 62 },
    { 0x1be516a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1e516d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e5172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0e5179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdace517c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662e5180, 98 },
    { 0, 0 },
    { 0x332e5182, 69 },
    { 0x840e5183, 99 },
    { 0xbc1e5183, 99 },
    { 0xbfae5185, 90 },
    { 0x1ade5186, 33 },
    { 0xea2e5182, 33 },
    { 0xb14e5188, 67 },
    { 0xc58e5189, 98 },
    { 0xa96e518a, 85 },
    { 0, 0 },
    { 0x81ae518c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9e518f, 98 },
    { 0xe3ae518f, 33 },
    { 0x768e5191, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e519b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e519e, 36 },
    { 0x4bee519f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8e51a8, 46 },
    { 0x39de51a9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861e51ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae51b4, 99 },
    { 0x302e51b5, 23 },
    { 0, 0 },
    { 0x3e6e51b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ae51bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17e51c2, 55 },
    { 0xdf2e51c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07e51c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e51ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6e51d1, 85 },
    { 0, 0 },
    { 0xe0fe51d3, 23 },
    { 0x22ee51d4, 52 },
    { 0, 0 },
    { 0x9c9e51d6, 93 },
    { 0x92be51d6, 85 },
    { 0, 0 },
    { 0xadbe51d9, 90 },
    { 0xf2ee51da, 64 },
    { 0xe31e51d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde51de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde51e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541e51e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ce51ea, 90 },
    { 0x344e51ea, 90 },
    { 0xa6de51ea, 95 },
    { 0, 0 },
    { 0xd7fe51ee, 34 },
    { 0x51ee51ef, 61 },
    { 0, 0 },
    { 0xc57e51f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee51f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e51fa, 61 },
    { 0x384e51fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafe51fe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65be5204, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e520b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208e5211, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30e5214, 33 },
    { 0x119e5215, 98 },
    { 0xa9e5216, 55 },
    { 0, 0 },
    { 0xbf1e5218, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ee521b, 95 },
    { 0, 0 },
    { 0xfaee521d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2e5220, 67 },
    { 0, 0 },
    { 0xe11e5222, 61 },
    { 0x89fe5223, 33 },
    { 0xb6de5224, 90 },
    { 0x99ae5225, 87 },
    { 0x3e4e5224, 90 },
    { 0x2ace5224, 95 },
    { 0x8d3e5223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e522d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e5232, 33 },
    { 0xbf6e5232, 76 },
    { 0, 0 },
    { 0xb86e5235, 55 },
    { 0x2b9e5236, 95 },
    { 0, 0 },
    { 0x20fe5238, 43 },
    { 0x3d9e5239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x525e523c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171e523f, 93 },
    { 0x14e5240, 99 },
    { 0xa15e523f, 33 },
    { 0x6e2e5242, 49 },
    { 0, 0 },
    { 0xb4ae5244, 33 },
    { 0x115e5244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaae5248, 87 },
    { 0x76ee5248, 95 },
    { 0x269e524a, 17 },
    { 0, 0 },
    { 0x886e524c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee524f, 33 },
    { 0xe2ce5250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66e5253, 55 },
    { 0x7ece5254, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe525b, 98 },
    { 0x6a4e525b, 76 },
    { 0x871e525d, 84 },
    { 0xc5e525d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e5261, 90 },
    { 0xf62e5261, 23 },
    { 0x5eae5262, 81 },
    { 0x449e5262, 61 },
    { 0, 0 },
    { 0x16ce5266, 33 },
    { 0, 0 },
    { 0x684e5268, 61 },
    { 0xa2ae5269, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e526e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ae5274, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36e5277, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120e527d, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e5288, 95 },
    { 0x986e5288, 46 },
    { 0, 0 },
    { 0x71be528b, 87 },
    { 0x47ce528c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae5295, 90 },
    { 0xb16e5296, 90 },
    { 0, 0 },
    { 0x626e5298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee529f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffe52a3, 98 },
    { 0, 0 },
    { 0x112e52a5, 36 },
    { 0x998e52a6, 46 },
    { 0x41be52a7, 76 },
    { 0x498e52a8, 100 },
    { 0x68e52a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ee52ac, 99 },
    { 0xee6e52ad, 55 },
    { 0x3cee52ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56e52b2, 33 },
    { 0x579e52b3, 78 },
    { 0, 0 },
    { 0xb52e52b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e52b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ece52bc, 46 },
    { 0xcfe52bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe52c2, 33 },
    { 0x256e52c3, 69 },
    { 0x11ce52c3, 61 },
    { 0xe1be52c2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede52c9, 61 },
    { 0xc45e52ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8e52cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5e52d2, 99 },
    { 0xf3fe52d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114e52d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ce52df, 91 },
    { 0x773e52e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944e52e6, 33 },
    { 0xcb7e52e7, 33 },
    { 0x137e52e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e52ec, 92 },
    { 0x854e52ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e52f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae52f7, 77 },
    { 0x280e52f7, 34 },
    { 0x91ee52f7, 90 },
    { 0, 0 },
    { 0x665e52fb, 61 },
    { 0x16be52fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e5304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aee5308, 64 },
    { 0xf67e5309, 76 },
    { 0xa0fe5308, 33 },
    { 0x6e7e530b, 61 },
    { 0x42e530b, 95 },
    { 0x13e530d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e5314, 40 },
    { 0xc89e5315, 95 },
    { 0x70ae5316, 34 },
    { 0xa4ce5316, 90 },
    { 0x362e5315, 40 },
    { 0x3a9e5319, 43 },
    { 0xe37e531a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe531e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e5321, 61 },
    { 0xd57e5321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e532c, 76 },
    { 0x9f2e532c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e5333, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e5336, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de533a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e533e, 69 },
    { 0xd22e533f, 33 },
    { 0x206e5340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e5347, 23 },
    { 0xd38e5348, 98 },
    { 0, 0 },
    { 0xbd3e534a, 55 },
    { 0x459e534b, 90 },
    { 0x944e534b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948e5350, 23 },
    { 0x48ae5350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce5354, 90 },
    { 0, 0 },
    { 0x44e5356, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5359, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x706e535c, 23 },
    { 0, 0 },
    { 0xfe1e535e, 98 },
    { 0x985e535f, 33 },
    { 0x649e5360, 90 },
    { 0x681e5361, 23 },
    { 0, 0 },
    { 0x7ede5363, 77 },
    { 0x715e5363, 90 },
    { 0, 0 },
    { 0xf73e5366, 99 },
    { 0x1be5367, 61 },
    { 0xd4be5368, 76 },
    { 0x988e5369, 61 },
    { 0x791e536a, 61 },
    { 0, 0 },
    { 0x93be536c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae5370, 33 },
    { 0, 0 },
    { 0x1b7e5372, 61 },
    { 0xb68e5373, 61 },
    { 0, 0 },
    { 0xac7e5375, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9be537a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39e537f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e5382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e5385, 65 },
    { 0xd73e5386, 33 },
    { 0, 0 },
    { 0xf29e5388, 85 },
    { 0x19e5388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee5391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16e5396, 93 },
    { 0xa0de5397, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e539d, 33 },
    { 0x98ee539e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3e53a2, 101 },
    { 0x93ce53a2, 91 },
    { 0x425e53a4, 23 },
    { 0xce8e53a5, 61 },
    { 0x466e53a3, 43 },
    { 0xcd3e53a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31e53aa, 90 },
    { 0x822e53aa, 51 },
    { 0xd48e53ac, 61 },
    { 0x308e53ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e53b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9e53ba, 34 },
    { 0, 0 },
    { 0x668e53bc, 93 },
    { 0, 0 },
    { 0xa97e53be, 95 },
    { 0, 0 },
    { 0x76ce53c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7e53c6, 43 },
    { 0xdb4e53c6, 23 },
    { 0x532e53c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe53cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeae53d3, 39 },
    { 0x410e53d3, 85 },
    { 0xe22e53d5, 55 },
    { 0x11be53d5, 90 },
    { 0x72e53d4, 36 },
    { 0x7e8e53d5, 34 },
    { 0x9b2e53d9, 90 },
    { 0x9cce53d9, 34 },
    { 0xe1fe53da, 34 },
    { 0x4fde53dc, 52 },
    { 0xb6ae53d8, 95 },
    { 0x39ae53de, 77 },
    { 0xaa4e53de, 76 },
    { 0xabee53de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ee53e8, 52 },
    { 0, 0 },
    { 0x1cbe53ea, 61 },
    { 0x524e53eb, 52 },
    { 0xb38e53ec, 98 },
    { 0xa52e53ea, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e53f0, 90 },
    { 0, 0 },
    { 0xab4e53f2, 47 },
    { 0x56be53f2, 33 },
    { 0x74fe53f3, 33 },
    { 0, 0 },
    { 0x524e53f6, 17 },
    { 0x559e53f7, 90 },
    { 0x1efe53f8, 66 },
    { 0xb26e53f9, 85 },
    { 0x3fce53f7, 95 },
    { 0x320e53fa, 33 },
    { 0xf9be53fc, 60 },
    { 0x8c0e53fb, 95 },
    { 0x22ce53f7, 67 },
    { 0x708e53f8, 34 },
    { 0x679e5400, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae5406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730e540b, 99 },
    { 0x5fce540c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1e540f, 23 },
    { 0x6dae540f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x570e5413, 90 },
    { 0xca2e5413, 55 },
    { 0, 0 },
    { 0x191e5416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5419, 33 },
    { 0x69be541a, 98 },
    { 0x29de541b, 61 },
    { 0x7c7e541c, 46 },
    { 0xcafe541d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x637e5420, 90 },
    { 0, 0 },
    { 0xb9ce5422, 33 },
    { 0x5ffe5423, 90 },
    { 0x44ae5424, 77 },
    { 0xa07e5424, 43 },
    { 0x63e5426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ee542c, 95 },
    { 0, 0 },
    { 0xcd7e542e, 95 },
    { 0x166e542e, 98 },
    { 0x7cee542f, 55 },
    { 0, 0 },
    { 0x8e1e5432, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ee5435, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de5439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49e543d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be5440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efe5449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e5450, 33 },
    { 0, 0 },
    { 0x864e5452, 61 },
    { 0x88be5453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e5457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bee545c, 76 },
    { 0xa39e545d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23e5461, 90 },
    { 0x848e5462, 52 },
    { 0xfd3e5462, 43 },
    { 0, 0 },
    { 0xc22e5465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9e5468, 90 },
    { 0x824e5469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49e546c, 98 },
    { 0, 0 },
    { 0xdfe546e, 100 },
    { 0x10e546e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65de547c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2e5484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5487, 61 },
    { 0x9c9e5488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ce548b, 91 },
    { 0, 0 },
    { 0xbbce548d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83e5494, 98 },
    { 0x480e5494, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ee54a0, 98 },
    { 0, 0 },
    { 0xd29e54a2, 95 },
    { 0x78ce54a3, 23 },
    { 0xb2de54a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ee54a7, 90 },
    { 0x24ce54a8, 52 },
    { 0xa7ce54a7, 95 },
    { 0xb5e54aa, 99 },
    { 0x7b4e54aa, 33 },
    { 0, 0 },
    { 0xb7ee54ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8e54b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e54b7, 78 },
    { 0x19ee54b8, 90 },
    { 0xd5ae54b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe54bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de54bf, 78 },
    { 0, 0 },
    { 0x12e54c1, 52 },
    { 0, 0 },
    { 0xed8e54c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x579e54c6, 33 },
    { 0x4c0e54c7, 84 },
    { 0xa2ae54c7, 76 },
    { 0x742e54c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abe54cc, 95 },
    { 0, 0 },
    { 0xe80e54ce, 97 },
    { 0x653e54cf, 34 },
    { 0, 0 },
    { 0xae54d1, 69 },
    { 0, 0 },
    { 0xa7fe54d3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e54d8, 23 },
    { 0x9fe54d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ce54e2, 88 },
    { 0x87ae54e2, 90 },
    { 0xf34e54e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e54e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226e54ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dce54f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31e54f7, 55 },
    { 0x482e54f8, 91 },
    { 0x6a9e54f8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ee54ff, 90 },
    { 0xd79e5500, 52 },
    { 0, 0 },
    { 0x682e5502, 33 },
    { 0x528e5503, 23 },
    { 0xe39e5502, 95 },
    { 0xdb7e5503, 34 },
    { 0, 0 },
    { 0xeb3e5507, 23 },
    { 0x21be5507, 23 },
    { 0x530e5507, 61 },
    { 0x5e8e5509, 55 },
    { 0xdf3e550b, 90 },
    { 0x165e550c, 17 },
    { 0, 0 },
    { 0xf7ae550e, 33 },
    { 0x24de550f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbe5514, 95 },
    { 0x6ace5515, 61 },
    { 0xb04e5516, 100 },
    { 0xac0e5516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56e551e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36e5521, 90 },
    { 0, 0 },
    { 0x503e5523, 33 },
    { 0xfd7e5524, 90 },
    { 0, 0 },
    { 0x1dae5526, 75 },
    { 0, 0 },
    { 0x925e5528, 19 },
    { 0x19be5529, 100 },
    { 0, 0 },
    { 0xdd6e552b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94be5534, 90 },
    { 0x4b9e5535, 61 },
    { 0x93fe5536, 49 },
    { 0xd5fe5537, 23 },
    { 0x984e5535, 90 },
    { 0xb51e5536, 33 },
    { 0xe31e5535, 23 },
    { 0xf04e553b, 95 },
    { 0x14fe553b, 33 },
    { 0xb62e553d, 84 },
    { 0xcd4e553e, 95 },
    { 0x825e5536, 34 },
    { 0x174e553a, 95 },
    { 0xb1ae553c, 90 },
    { 0x73fe5540, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e5547, 88 },
    { 0x2dde5547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e554b, 61 },
    { 0, 0 },
    { 0x1c8e554d, 62 },
    { 0xb78e554e, 99 },
    { 0x5ae554f, 90 },
    { 0x222e554d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e5555, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce555a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e5561, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6e5564, 52 },
    { 0x4dee5565, 61 },
    { 0, 0 },
    { 0x87fe5567, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dce556f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549e5574, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbe5577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceae557a, 90 },
    { 0x896e557a, 97 },
    { 0x84e557b, 33 },
    { 0x531e557d, 90 },
    { 0xf38e557d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1e5584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ce5587, 85 },
    { 0, 0 },
    { 0x585e5589, 78 },
    { 0x613e558a, 95 },
    { 0x6dce5589, 98 },
    { 0x540e558a, 90 },
    { 0xfa6e558b, 65 },
    { 0x2bbe558b, 98 },
    { 0, 0 },
    { 0x65ce5590, 76 },
    { 0x959e5590, 85 },
    { 0x808e5592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4e5595, 76 },
    { 0xd66e5595, 76 },
    { 0, 0 },
    { 0x2fde5598, 33 },
    { 0x5e0e5598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce559d, 33 },
    { 0, 0 },
    { 0xa05e559f, 33 },
    { 0xd2ee559f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50e55a7, 23 },
    { 0x3e7e55a8, 90 },
    { 0xcc2e55a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de55ac, 34 },
    { 0xae1e55ad, 61 },
    { 0xb92e55ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201e55b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36e55b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7e55be, 98 },
    { 0, 0 },
    { 0x10ce55c0, 46 },
    { 0, 0 },
    { 0x552e55c2, 98 },
    { 0x4a8e55c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392e55c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1e55d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce55d9, 98 },
    { 0xcd8e55da, 99 },
    { 0xb11e55db, 55 },
    { 0x548e55d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce55e1, 68 },
    { 0x246e55e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde55e8, 33 },
    { 0x5de55e9, 95 },
    { 0, 0 },
    { 0x167e55eb, 23 },
    { 0x657e55eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e55f1, 90 },
    { 0, 0 },
    { 0x5e4e55f3, 76 },
    { 0, 0 },
    { 0x495e55f5, 99 },
    { 0, 0 },
    { 0x967e55f7, 95 },
    { 0, 0 },
    { 0xa8be55f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e55fc, 93 },
    { 0xf9e55fc, 90 },
    { 0xa3de55fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e5601, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94e5605, 34 },
    { 0x54ce5605, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173e560e, 88 },
    { 0xcbfe560f, 52 },
    { 0x5b7e560f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dae5616, 61 },
    { 0x623e5617, 93 },
    { 0xc75e5618, 55 },
    { 0x41ee5617, 61 },
    { 0xda5e5616, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcee5621, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade5625, 61 },
    { 0, 0 },
    { 0xebce5627, 33 },
    { 0xa4e5628, 33 },
    { 0xf8fe5629, 61 },
    { 0xc0de562a, 88 },
    { 0x481e562a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be5634, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e5638, 76 },
    { 0x9ebe5638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be5640, 100 },
    { 0x509e5641, 97 },
    { 0x81e5641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe564c, 88 },
    { 0, 0 },
    { 0x5bbe564e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e5652, 98 },
    { 0xb04e5653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee5658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e5661, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11e5669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce566c, 93 },
    { 0xd62e566c, 34 },
    { 0xf05e566e, 55 },
    { 0xbebe566c, 67 },
    { 0, 0 },
    { 0x8cbe5671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46e5674, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e5680, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e5684, 43 },
    { 0xbf8e5684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e568d, 90 },
    { 0x44fe568e, 76 },
    { 0xd6ce568d, 90 },
    { 0xa49e5690, 51 },
    { 0xd3de568f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7e5697, 34 },
    { 0xe2ce5697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ae56a0, 87 },
    { 0xa39e56a1, 61 },
    { 0, 0 },
    { 0xbb5e56a3, 33 },
    { 0, 0 },
    { 0xa65e56a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43e56a9, 98 },
    { 0x421e56aa, 95 },
    { 0, 0 },
    { 0xd9fe56ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fe56b4, 81 },
    { 0x26fe56b5, 99 },
    { 0x6eee56b6, 34 },
    { 0, 0 },
    { 0x829e56b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x785e56bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ae56c1, 90 },
    { 0, 0 },
    { 0xc37e56c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedce56c8, 90 },
    { 0x529e56c8, 55 },
    { 0, 0 },
    { 0xfeae56cb, 85 },
    { 0x741e56cc, 90 },
    { 0xba1e56cd, 40 },
    { 0xdaae56cc, 78 },
    { 0xf7fe56cf, 93 },
    { 0xee3e56d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e56d9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e56de, 46 },
    { 0, 0 },
    { 0xc34e56e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e56e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee56ee, 52 },
    { 0x527e56ef, 23 },
    { 0, 0 },
    { 0x5f4e56f1, 23 },
    { 0x106e56f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceae56f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34de56fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5e5701, 55 },
    { 0xb2e5702, 33 },
    { 0x202e5702, 43 },
    { 0x8bae5704, 61 },
    { 0xa27e5705, 76 },
    { 0, 0 },
    { 0xcb7e5707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8de570e, 43 },
    { 0xbb9e570e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e5714, 46 },
    { 0x77ce5715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5e571b, 76 },
    { 0, 0 },
    { 0xd1ee571d, 98 },
    { 0xf58e571d, 76 },
    { 0, 0 },
    { 0x6d1e5720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e5726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe5729, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47de572d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9e5738, 100 },
    { 0x717e5738, 90 },
    { 0, 0 },
    { 0x10e573b, 85 },
    { 0xf5ae573c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x985e573f, 85 },
    { 0x976e5740, 95 },
    { 0x542e5740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82e5749, 98 },
    { 0x87ee574a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e574d, 11 },
    { 0, 0 },
    { 0x628e574f, 88 },
    { 0xb45e5750, 84 },
    { 0, 0 },
    { 0x291e5752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x718e5755, 23 },
    { 0, 0 },
    { 0x439e5757, 98 },
    { 0xcc6e5758, 61 },
    { 0xc2e5759, 99 },
    { 0xc9e5759, 23 },
    { 0x494e5758, 98 },
    { 0x3dce575c, 90 },
    { 0x5fee5759, 75 },
    { 0xc7e575c, 90 },
    { 0xa8de575f, 33 },
    { 0x650e575f, 90 },
    { 0xc48e5761, 66 },
    { 0x83e5761, 33 },
    { 0xc9fe5763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e5766, 55 },
    { 0x48fe5767, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503e576f, 61 },
    { 0, 0 },
    { 0xb8be5771, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66be5777, 95 },
    { 0xfcee5778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9e577d, 91 },
    { 0xa20e577e, 93 },
    { 0, 0 },
    { 0x528e5780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e5783, 52 },
    { 0, 0 },
    { 0xf40e5785, 98 },
    { 0xd7ce5786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e578b, 61 },
    { 0x382e578c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ace578f, 88 },
    { 0, 0 },
    { 0xea7e5791, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde5797, 23 },
    { 0, 0 },
    { 0x381e5799, 52 },
    { 0x67de5799, 90 },
    { 0, 0 },
    { 0xaabe579c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ae57a9, 76 },
    { 0x604e57a9, 23 },
    { 0, 0 },
    { 0x90fe57ac, 78 },
    { 0, 0 },
    { 0xf6ce57ae, 61 },
    { 0x139e57ae, 23 },
    { 0, 0 },
    { 0x195e57b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532e57b4, 55 },
    { 0x389e57b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x633e57b8, 78 },
    { 0x112e57b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eae57c1, 85 },
    { 0x7f4e57c2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5e57c5, 98 },
    { 0xd28e57c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e57cb, 48 },
    { 0, 0 },
    { 0xef2e57cd, 88 },
    { 0, 0 },
    { 0x454e57cf, 61 },
    { 0x179e57d0, 101 },
    { 0xc7de57cf, 90 },
    { 0x18fe57d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827e57da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ee57e0, 90 },
    { 0, 0 },
    { 0xa02e57e2, 98 },
    { 0, 0 },
    { 0x90e57e4, 65 },
    { 0, 0 },
    { 0x79fe57e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ce57e9, 91 },
    { 0xe53e57ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e57ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1e57f0, 96 },
    { 0xc5ee57f1, 23 },
    { 0x86ae57f1, 65 },
    { 0x194e57f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x383e57f6, 93 },
    { 0, 0 },
    { 0xf7fe57f8, 48 },
    { 0, 0 },
    { 0x7d2e57fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e57fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e5803, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8de5808, 43 },
    { 0x50de5808, 61 },
    { 0x5cae5809, 23 },
    { 0x761e580a, 33 },
    { 0x2cde580c, 23 },
    { 0, 0 },
    { 0x804e580e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e5814, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ece5819, 36 },
    { 0x6cce581a, 98 },
    { 0, 0 },
    { 0x540e581c, 88 },
    { 0xfa3e581d, 87 },
    { 0x3b1e581e, 90 },
    { 0xac0e581f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fe5822, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbce5829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee582f, 98 },
    { 0, 0 },
    { 0x861e5831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0e5837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e583b, 95 },
    { 0xd96e583c, 23 },
    { 0x499e583c, 99 },
    { 0, 0 },
    { 0x95ee583f, 46 },
    { 0x90ae5840, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ee5845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae5848, 78 },
    { 0xa4fe5849, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6e584c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x86be584f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e5852, 95 },
    { 0, 0 },
    { 0x1f8e5854, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76e5858, 61 },
    { 0xfa8e5859, 98 },
    { 0x610e585a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ee585e, 55 },
    { 0, 0 },
    { 0x2a6e5860, 90 },
    { 0, 0 },
    { 0xbece5862, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x325e5865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede586f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9e5874, 98 },
    { 0x35de5874, 98 },
    { 0x3fbe5875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fe587a, 55 },
    { 0xf07e587b, 33 },
    { 0x2bce587a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460e5884, 33 },
    { 0, 0 },
    { 0x55ce5886, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e5889, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8e588c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e5893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e5897, 61 },
    { 0, 0 },
    { 0x5cfe5899, 99 },
    { 0x8c2e589a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe589d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e58a8, 52 },
    { 0x35de58a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e58af, 33 },
    { 0x4fae58b0, 55 },
    { 0x651e58b0, 61 },
    { 0xff8e58b2, 41 },
    { 0, 0 },
    { 0xa9e58b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e58b8, 95 },
    { 0xd1ee58b9, 91 },
    { 0xd41e58b9, 90 },
    { 0x10de58ba, 33 },
    { 0, 0 },
    { 0x581e58bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e58c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x860e58c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8e58c9, 95 },
    { 0x59ae58ca, 76 },
    { 0, 0 },
    { 0xb3be58cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e58cf, 88 },
    { 0x49de58cf, 78 },
    { 0x4d9e58d1, 90 },
    { 0xd7be58d2, 55 },
    { 0x5bee58d2, 34 },
    { 0x8a5e58d4, 66 },
    { 0xd13e58d4, 4 },
    { 0, 0 },
    { 0x1bae58d7, 90 },
    { 0x729e58d8, 52 },
    { 0, 0 },
    { 0xb97e58da, 85 },
    { 0, 0 },
    { 0xae7e58dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0be58df, 90 },
    { 0x768e58e0, 34 },
    { 0x959e58e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4e58e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5e58e9, 100 },
    { 0xa00e58ea, 61 },
    { 0x6f7e58ea, 33 },
    { 0, 0 },
    { 0xf1e58ed, 77 },
    { 0x21ee58ee, 33 },
    { 0x319e58ef, 91 },
    { 0x69be58ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9e58f3, 46 },
    { 0, 0 },
    { 0x989e58f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be58f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253e5901, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0e5904, 88 },
    { 0, 0 },
    { 0x55ce5906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e590a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4e590d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e5910, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e5918, 87 },
    { 0x30fe5918, 55 },
    { 0xd53e591a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e591d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0e5921, 3 },
    { 0x33de5922, 84 },
    { 0xb50e5923, 61 },
    { 0xf5be5923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5928, 34 },
    { 0x487e5928, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e592e, 98 },
    { 0xe35e592f, 61 },
    { 0x438e5930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee5933, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318e5938, 90 },
    { 0x351e5939, 23 },
    { 0x880e593a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0be593e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9e5943, 78 },
    { 0xf9ee5944, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ee5947, 61 },
    { 0xfade5948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e594b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00e5951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6be5955, 33 },
    { 0, 0 },
    { 0xce8e5957, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9e595e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1e5963, 90 },
    { 0xc2ae5964, 55 },
    { 0, 0 },
    { 0x4ace5966, 23 },
    { 0xb70e5966, 33 },
    { 0x7d2e5968, 61 },
    { 0xdfe5969, 43 },
    { 0xbb5e596a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe596e, 76 },
    { 0, 0 },
    { 0xa4fe5970, 95 },
    { 0, 0 },
    { 0x4b0e5972, 85 },
    { 0x1dee5973, 23 },
    { 0x449e5973, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe5978, 44 },
    { 0xafe5978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fe597c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfe597f, 43 },
    { 0xe3ee5980, 55 },
    { 0xb83e5981, 23 },
    { 0x9efe5982, 95 },
    { 0xb6ce5982, 91 },
    { 0xbe3e5984, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78e5991, 95 },
    { 0xb08e5991, 34 },
    { 0x273e5991, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ae599b, 93 },
    { 0x49fe599b, 90 },
    { 0x297e599b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e59a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e59a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e59a8, 55 },
    { 0x276e59a9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de59ad, 90 },
    { 0, 0 },
    { 0xcb2e59af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe59b4, 81 },
    { 0x228e59b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde59b9, 33 },
    { 0xbe7e59ba, 87 },
    { 0x1f2e59ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae59c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934e59c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7e59cc, 95 },
    { 0x569e59cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2e59d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e59dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960e59e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ce59e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2e59ee, 61 },
    { 0xca8e59ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae59fa, 43 },
    { 0, 0 },
    { 0x67ce59fc, 61 },
    { 0, 0 },
    { 0xc1be59fe, 90 },
    { 0x966e59ff, 23 },
    { 0, 0 },
    { 0x25de5a01, 93 },
    { 0x61ee5a02, 98 },
    { 0x613e5a03, 33 },
    { 0x688e5a04, 37 },
    { 0xc00e5a05, 91 },
    { 0x1d0e5a03, 43 },
    { 0xdb8e5a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aae5a0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be5a12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4e5a15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde5a18, 46 },
    { 0x627e5a19, 23 },
    { 0x3eae5a1a, 91 },
    { 0x4fee5a1b, 23 },
    { 0, 0 },
    { 0xc8be5a1d, 46 },
    { 0, 0 },
    { 0xac7e5a1f, 95 },
    { 0x7d6e5a20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9e5a24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ae5a27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e5a2a, 90 },
    { 0, 0 },
    { 0xa7e5a2c, 91 },
    { 0, 0 },
    { 0x72ce5a2e, 33 },
    { 0, 0 },
    { 0xe49e5a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04e5a33, 76 },
    { 0, 0 },
    { 0xc87e5a35, 34 },
    { 0x843e5a36, 98 },
    { 0xc0fe5a36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e5a3a, 93 },
    { 0x4d3e5a3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e5a46, 98 },
    { 0x6c8e5a47, 23 },
    { 0, 0 },
    { 0x83fe5a49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafae5a50, 98 },
    { 0, 0 },
    { 0x672e5a52, 61 },
    { 0, 0 },
    { 0xb1de5a54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e5a57, 46 },
    { 0x100e5a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe5a5c, 33 },
    { 0xd17e5a5d, 40 },
    { 0, 0 },
    { 0x500e5a5f, 88 },
    { 0x663e5a60, 76 },
    { 0x449e5a60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57e5a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e5a72, 90 },
    { 0x820e5a72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe5a77, 33 },
    { 0xeaae5a78, 43 },
    { 0x681e5a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5e5a7c, 95 },
    { 0x1fae5a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe5a86, 76 },
    { 0xaf2e5a87, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44be5a8b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e5a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5a91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae5a94, 90 },
    { 0, 0 },
    { 0x443e5a96, 46 },
    { 0xa50e5a96, 76 },
    { 0, 0 },
    { 0x3fbe5a99, 95 },
    { 0, 0 },
    { 0x234e5a9b, 90 },
    { 0, 0 },
    { 0x4aae5a9d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e5aa0, 33 },
    { 0, 0 },
    { 0xb07e5aa2, 34 },
    { 0, 0 },
    { 0xd64e5aa4, 95 },
    { 0xaf2e5aa4, 98 },
    { 0x323e5aa5, 95 },
    { 0x3ace5aa6, 98 },
    { 0xc30e5aa8, 90 },
    { 0, 0 },
    { 0x475e5aaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fe5aad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8e5ab1, 55 },
    { 0xd3ee5ab2, 33 },
    { 0x971e5ab3, 76 },
    { 0, 0 },
    { 0xa56e5ab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26e5ab8, 98 },
    { 0x901e5ab9, 44 },
    { 0x6b4e5aba, 78 },
    { 0xb6ce5abb, 99 },
    { 0x2a7e5abb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee5ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4e5aca, 61 },
    { 0x852e5aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191e5ad6, 55 },
    { 0x3fde5ad7, 55 },
    { 0x6c9e5ad8, 90 },
    { 0x6d7e5ad9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be5adc, 77 },
    { 0x417e5add, 83 },
    { 0x3b0e5add, 61 },
    { 0x20ee5adf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e5ae2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7e5aea, 61 },
    { 0x1dee5aeb, 34 },
    { 0x48be5aec, 33 },
    { 0xa00e5aed, 85 },
    { 0xc51e5aed, 90 },
    { 0xe5de5aeb, 52 },
    { 0xaa1e5af0, 43 },
    { 0x2be5af1, 98 },
    { 0xd31e5af2, 93 },
    { 0xd45e5af3, 23 },
    { 0xbbee5aed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e5af7, 52 },
    { 0x7dfe5af7, 90 },
    { 0x92e5af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee5afc, 98 },
    { 0xe02e5afd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae5b04, 33 },
    { 0x462e5b05, 78 },
    { 0x955e5b05, 90 },
    { 0x354e5b06, 96 },
    { 0xba5e5b08, 91 },
    { 0x200e5b09, 55 },
    { 0xa36e5b0a, 29 },
    { 0xea5e5b0a, 91 },
    { 0x336e5b08, 55 },
    { 0x71ee5b0d, 98 },
    { 0xad4e5b0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e5b12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e5b16, 61 },
    { 0x44ae5b17, 61 },
    { 0x5ede5b18, 90 },
    { 0x6b8e5b19, 76 },
    { 0xd2ce5b18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae5b1e, 78 },
    { 0, 0 },
    { 0x3c8e5b20, 23 },
    { 0x74fe5b21, 84 },
    { 0xf8fe5b20, 95 },
    { 0x74ce5b23, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8e5b2b, 46 },
    { 0x697e5b2c, 98 },
    { 0x1cee5b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301e5b34, 33 },
    { 0, 0 },
    { 0x4d0e5b36, 78 },
    { 0, 0 },
    { 0x24ee5b38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3e5b3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5b40, 40 },
    { 0x74ee5b40, 76 },
    { 0x763e5b42, 36 },
    { 0x667e5b43, 55 },
    { 0xddfe5b40, 91 },
    { 0xf51e5b45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93e5b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8e5b4c, 98 },
    { 0xa9fe5b4d, 89 },
    { 0x29ee5b4c, 76 },
    { 0xae8e5b4f, 76 },
    { 0, 0 },
    { 0x383e5b51, 85 },
    { 0x394e5b52, 73 },
    { 0, 0 },
    { 0xd3be5b54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5e5b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724e5b5b, 55 },
    { 0xb63e5b5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5b62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e5b66, 90 },
    { 0, 0 },
    { 0x2f1e5b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0e5b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e5b72, 95 },
    { 0x75fe5b73, 90 },
    { 0x311e5b73, 69 },
    { 0x1b8e5b74, 34 },
    { 0x8dae5b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902e5b7e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e5b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ae5b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e5b8b, 61 },
    { 0x1b2e5b8c, 95 },
    { 0x132e5b8b, 66 },
    { 0x86de5b8d, 76 },
    { 0xf01e5b8f, 78 },
    { 0xcb4e5b8f, 93 },
    { 0, 0 },
    { 0x39fe5b92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ede5b99, 61 },
    { 0, 0 },
    { 0xfefe5b9b, 95 },
    { 0, 0 },
    { 0x7f4e5b9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5e5ba0, 61 },
    { 0xdbee5ba1, 36 },
    { 0x60ce5ba2, 61 },
    { 0, 0 },
    { 0xca0e5ba4, 83 },
    { 0x6fbe5ba5, 33 },
    { 0xf4e5ba6, 85 },
    { 0xc82e5ba4, 76 },
    { 0xff0e5ba8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ee5bab, 34 },
    { 0xc80e5bab, 91 },
    { 0x60e5bad, 100 },
    { 0x4d9e5bad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be5bb5, 76 },
    { 0x979e5bb6, 61 },
    { 0x9a6e5bb6, 98 },
    { 0x45ce5bb8, 61 },
    { 0xa84e5bb6, 25 },
    { 0x5f4e5bba, 95 },
    { 0x565e5bb8, 61 },
    { 0x370e5bbb, 90 },
    { 0xcaee5bbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x153e5bc0, 81 },
    { 0xfd9e5bc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e5bc4, 66 },
    { 0, 0 },
    { 0x829e5bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50e5bca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90e5bcd, 91 },
    { 0x723e5bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8e5bd1, 87 },
    { 0, 0 },
    { 0x80fe5bd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e5bd8, 23 },
    { 0, 0 },
    { 0xf22e5bda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe5be0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcede5be5, 33 },
    { 0x36fe5be6, 17 },
    { 0, 0 },
    { 0xdf0e5be8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59e5bed, 17 },
    { 0x80de5bee, 98 },
    { 0x81e5bef, 95 },
    { 0x9d6e5bf0, 61 },
    { 0x808e5bf0, 84 },
    { 0x283e5bf1, 90 },
    { 0xa5ce5bf3, 33 },
    { 0x5cfe5bf3, 38 },
    { 0x6bee5bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee5bfd, 76 },
    { 0x342e5bfe, 46 },
    { 0x50ae5bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e5c04, 98 },
    { 0x77ae5c05, 100 },
    { 0, 0 },
    { 0x915e5c07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x876e5c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ae5c15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4e5c18, 91 },
    { 0x287e5c18, 93 },
    { 0xf1e5c1a, 61 },
    { 0, 0 },
    { 0xf3e5c1c, 95 },
    { 0, 0 },
    { 0x9dce5c1e, 95 },
    { 0xd9ce5c1f, 77 },
    { 0xcdce5c20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be5c23, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e5c29, 85 },
    { 0xe24e5c29, 83 },
    { 0xa4ee5c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e5c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409e5c3a, 90 },
    { 0x7d3e5c3b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e5c43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6e5c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e5c50, 95 },
    { 0, 0 },
    { 0xde6e5c52, 23 },
    { 0, 0 },
    { 0x7a1e5c54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x775e5c57, 34 },
    { 0xdfae5c58, 23 },
    { 0x5a5e5c59, 98 },
    { 0xc1de5c5a, 46 },
    { 0, 0 },
    { 0x4ee5c5c, 70 },
    { 0x981e5c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae5c6a, 95 },
    { 0, 0 },
    { 0x88fe5c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde5c70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e5c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce5c77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3e5c7f, 33 },
    { 0, 0 },
    { 0xfdce5c81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96e5c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbede5c89, 91 },
    { 0, 0 },
    { 0xf63e5c8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14e5c90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16e5c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ce5c9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918e5ca2, 33 },
    { 0, 0 },
    { 0xcd5e5ca4, 90 },
    { 0x8a7e5ca5, 78 },
    { 0x976e5ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ca9, 90 },
    { 0x372e5caa, 61 },
    { 0, 0 },
    { 0xadde5cac, 33 },
    { 0, 0 },
    { 0xb8fe5cae, 65 },
    { 0xbc5e5cae, 34 },
    { 0xca5e5caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e5cbb, 78 },
    { 0, 0 },
    { 0x10ee5cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ce5cc1, 52 },
    { 0, 0 },
    { 0x925e5cc3, 84 },
    { 0, 0 },
    { 0x38e5cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x376e5cc8, 33 },
    { 0x38fe5cc9, 52 },
    { 0, 0 },
    { 0x9f4e5ccb, 52 },
    { 0x8ebe5ccb, 78 },
    { 0, 0 },
    { 0xa56e5cce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfece5cd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde5cd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692e5cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce5ce1, 90 },
    { 0, 0 },
    { 0xac4e5ce3, 95 },
    { 0, 0 },
    { 0x589e5ce5, 51 },
    { 0xde5ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70e5ce9, 95 },
    { 0, 0 },
    { 0xd56e5ceb, 75 },
    { 0x2dbe5cec, 55 },
    { 0, 0 },
    { 0x98e5cee, 33 },
    { 0, 0 },
    { 0x6a6e5cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877e5cf5, 98 },
    { 0, 0 },
    { 0x4ebe5cf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e5d00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde5d04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3e5d08, 33 },
    { 0x5c3e5d09, 95 },
    { 0, 0 },
    { 0x7be5d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5d11, 4 },
    { 0x801e5d12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e5d15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e5d19, 65 },
    { 0xf1fe5d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e5d1f, 90 },
    { 0xc29e5d20, 90 },
    { 0x25be5d1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e5d26, 61 },
    { 0x927e5d26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de5d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486e5d2d, 33 },
    { 0, 0 },
    { 0x60ee5d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76e5d32, 33 },
    { 0xcd6e5d33, 76 },
    { 0x10fe5d34, 23 },
    { 0x1d3e5d34, 61 },
    { 0x19fe5d36, 23 },
    { 0, 0 },
    { 0x3fde5d38, 95 },
    { 0x15e5d38, 68 },
    { 0x8e8e5d3a, 33 },
    { 0xf74e5d39, 61 },
    { 0xee4e5d3c, 85 },
    { 0x46e5d3d, 95 },
    { 0x95e5d3e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e5d41, 67 },
    { 0x7a1e5d42, 91 },
    { 0xaaae5d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae5d47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cae5d4d, 33 },
    { 0, 0 },
    { 0x506e5d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ee5d52, 67 },
    { 0xf4fe5d53, 95 },
    { 0xb2ce5d52, 90 },
    { 0x93ae5d55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7e5d59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e5d61, 55 },
    { 0, 0 },
    { 0x1e6e5d63, 67 },
    { 0, 0 },
    { 0x3d2e5d65, 76 },
    { 0, 0 },
    { 0x421e5d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x860e5d6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be5d71, 91 },
    { 0xeee5d71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331e5d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e5d79, 83 },
    { 0x324e5d7a, 61 },
    { 0xdce5d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ee5d85, 66 },
    { 0, 0 },
    { 0x935e5d87, 95 },
    { 0xd94e5d88, 84 },
    { 0xa44e5d89, 33 },
    { 0xd7ee5d8a, 90 },
    { 0, 0 },
    { 0x1d8e5d8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3e5d8f, 90 },
    { 0, 0 },
    { 0xd5ce5d91, 23 },
    { 0, 0 },
    { 0x39ce5d93, 61 },
    { 0x9b6e5d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e5d99, 61 },
    { 0x4b6e5d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ce5d9d, 98 },
    { 0x4a7e5d9d, 90 },
    { 0x5d8e5d9e, 81 },
    { 0xe87e5da0, 90 },
    { 0xab6e5d9e, 90 },
    { 0x8aee5da2, 91 },
    { 0xdfce5d9e, 75 },
    { 0, 0 },
    { 0xc8fe5da5, 43 },
    { 0, 0 },
    { 0x21ee5da7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e5db1, 78 },
    { 0x9cce5db1, 33 },
    { 0xb6be5db2, 90 },
    { 0x44fe5db4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dde5db8, 90 },
    { 0x1f1e5db8, 43 },
    { 0xad5e5dba, 61 },
    { 0x227e5db9, 34 },
    { 0xa40e5dbc, 34 },
    { 0xf47e5dbd, 92 },
    { 0xe0e5db8, 96 },
    { 0, 0 },
    { 0x618e5dc0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede5dc4, 23 },
    { 0x378e5dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdde5dca, 99 },
    { 0x8c2e5dcb, 61 },
    { 0xd9be5dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce5dd0, 90 },
    { 0xadde5dd0, 33 },
    { 0, 0 },
    { 0x8b1e5dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de5dd7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e5dda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae5de2, 84 },
    { 0x16fe5de2, 69 },
    { 0, 0 },
    { 0xbd6e5de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58e5de8, 93 },
    { 0, 0 },
    { 0x382e5dea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ce5df0, 23 },
    { 0, 0 },
    { 0x4a3e5df2, 36 },
    { 0, 0 },
    { 0xbaae5df4, 90 },
    { 0xef8e5df4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64e5df9, 85 },
    { 0x4f4e5dfa, 23 },
    { 0x47e5dfb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcae5dff, 90 },
    { 0xeee5dff, 33 },
    { 0xcbce5e01, 61 },
    { 0xad7e5e01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e5e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be5e0c, 19 },
    { 0xa68e5e0d, 98 },
    { 0, 0 },
    { 0x39fe5e0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78e5e13, 33 },
    { 0x339e5e14, 98 },
    { 0x877e5e15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9de5e1b, 61 },
    { 0x292e5e1c, 84 },
    { 0x2b2e5e1d, 64 },
    { 0x8cbe5e1e, 33 },
    { 0, 0 },
    { 0xbc8e5e20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e5e2b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe5e2f, 61 },
    { 0x5ece5e30, 66 },
    { 0x762e5e31, 23 },
    { 0x3c8e5e31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e5e37, 34 },
    { 0xf00e5e37, 23 },
    { 0x53be5e39, 90 },
    { 0x2a8e5e3a, 66 },
    { 0xaf8e5e39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e5e44, 33 },
    { 0xd83e5e45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de5e48, 33 },
    { 0, 0 },
    { 0xf84e5e4a, 61 },
    { 0, 0 },
    { 0xfeae5e4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3e5e52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e5e56, 90 },
    { 0xdfae5e56, 91 },
    { 0xee2e5e58, 46 },
    { 0xaa3e5e58, 23 },
    { 0xf61e5e56, 90 },
    { 0x36e5e5b, 75 },
    { 0x71be5e5c, 61 },
    { 0x940e5e5d, 33 },
    { 0xabce5e5d, 43 },
    { 0x656e5e5c, 98 },
    { 0x634e5e60, 33 },
    { 0xcdde5e56, 43 },
    { 0xf93e5e5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91e5e66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e5e69, 90 },
    { 0, 0 },
    { 0x54ae5e6b, 34 },
    { 0, 0 },
    { 0x410e5e6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fce5e70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5e76, 90 },
    { 0x340e5e77, 98 },
    { 0x70ce5e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6e5e7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ee5e7f, 33 },
    { 0, 0 },
    { 0xd9ce5e81, 34 },
    { 0x5fe5e82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e5e85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e5e8e, 33 },
    { 0x992e5e8f, 61 },
    { 0, 0 },
    { 0xde2e5e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e5e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40de5e9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248e5ea2, 98 },
    { 0xb8fe5ea3, 96 },
    { 0, 0 },
    { 0x838e5ea5, 85 },
    { 0x198e5ea5, 78 },
    { 0xd0fe5ea5, 95 },
    { 0x44be5ea8, 81 },
    { 0, 0 },
    { 0x2dee5eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e5eae, 33 },
    { 0x7b2e5eae, 93 },
    { 0, 0 },
    { 0xc73e5eb1, 46 },
    { 0x32ce5eb2, 90 },
    { 0xffbe5eb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae5ebe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x425e5ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee5ec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe5ec7, 95 },
    { 0, 0 },
    { 0x229e5ec9, 98 },
    { 0, 0 },
    { 0x827e5ecb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ae5ed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee5ed3, 98 },
    { 0x9d5e5ed3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6e5ed8, 90 },
    { 0, 0 },
    { 0x492e5eda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e5ede, 61 },
    { 0, 0 },
    { 0x956e5ee0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece5ee4, 93 },
    { 0x843e5ee5, 33 },
    { 0, 0 },
    { 0x7f1e5ee7, 61 },
    { 0, 0 },
    { 0x62e5ee9, 61 },
    { 0, 0 },
    { 0xb4fe5eeb, 94 },
    { 0x5afe5eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce5ef2, 61 },
    { 0x314e5ef3, 33 },
    { 0, 0 },
    { 0xdcfe5ef5, 101 },
    { 0, 0 },
    { 0xa6be5ef7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee5efb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e5efe, 95 },
    { 0x96de5eff, 33 },
    { 0xff3e5f00, 98 },
    { 0xc4ce5f00, 46 },
    { 0xe8ee5f01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1de5f05, 88 },
    { 0x143e5f06, 90 },
    { 0x662e5f07, 61 },
    { 0, 0 },
    { 0xcf2e5f09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe5f0d, 33 },
    { 0xe3fe5f0e, 66 },
    { 0xae5f0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8be5f14, 33 },
    { 0x626e5f15, 52 },
    { 0xecbe5f15, 99 },
    { 0x7dae5f17, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ce5f1d, 87 },
    { 0x442e5f1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde5f22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcde5f2b, 23 },
    { 0x71ae5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee5f30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e5f36, 43 },
    { 0xe06e5f37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e5f3a, 90 },
    { 0x8ffe5f3a, 99 },
    { 0x507e5f3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ee5f3f, 98 },
    { 0x492e5f3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261e5f45, 90 },
    { 0x175e5f45, 90 },
    { 0x541e5f45, 23 },
    { 0x8a2e5f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2e5f4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x189e5f52, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e5f55, 34 },
    { 0x327e5f56, 12 },
    { 0, 0 },
    { 0xcd4e5f58, 76 },
    { 0xa9ee5f58, 98 },
    { 0, 0 },
    { 0x2e5e5f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ee5f5e, 33 },
    { 0, 0 },
    { 0x9eae5f60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304e5f64, 61 },
    { 0x2aae5f64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63e5f68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198e5f6b, 87 },
    { 0xa53e5f6b, 52 },
    { 0x2ade5f6b, 90 },
    { 0x25de5f6e, 76 },
    { 0x917e5f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee5f73, 61 },
    { 0, 0 },
    { 0xa5e5f75, 66 },
    { 0, 0 },
    { 0xf9de5f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fe5f7b, 95 },
    { 0xc8ee5f7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x561e5f7f, 61 },
    { 0xbc2e5f80, 92 },
    { 0xce8e5f80, 92 },
    { 0x8e3e5f7f, 55 },
    { 0xc6de5f81, 76 },
    { 0x42be5f84, 98 },
    { 0x82be5f84, 90 },
    { 0, 0 },
    { 0x522e5f87, 61 },
    { 0, 0 },
    { 0x38e5f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5e5f92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be5f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3e5f9e, 23 },
    { 0x493e5f9f, 66 },
    { 0x3fbe5fa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfee5fa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e5fa9, 90 },
    { 0x213e5faa, 56 },
    { 0x14be5fab, 95 },
    { 0x96ce5fac, 33 },
    { 0x1f0e5fac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e5fb4, 34 },
    { 0xe6ae5fb4, 61 },
    { 0xdd1e5fb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ae5fb9, 23 },
    { 0xd3de5fba, 61 },
    { 0x444e5fba, 84 },
    { 0x800e5fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e5fc0, 61 },
    { 0x575e5fc1, 95 },
    { 0, 0 },
    { 0x972e5fc3, 33 },
    { 0, 0 },
    { 0x4c2e5fc5, 33 },
    { 0xc86e5fc6, 34 },
    { 0x242e5fc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde5fca, 61 },
    { 0xa7ce5fcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710e5fcf, 43 },
    { 0, 0 },
    { 0x42e5fd1, 61 },
    { 0, 0 },
    { 0xa6ce5fd3, 90 },
    { 0x7d4e5fd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe5fe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fce5fed, 33 },
    { 0, 0 },
    { 0xe25e5fef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28e5ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e5ff6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ee6001, 61 },
    { 0, 0 },
    { 0x9afe6003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee600b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99e600e, 38 },
    { 0x6a1e600f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1e6016, 90 },
    { 0, 0 },
    { 0x9afe6018, 93 },
    { 0xac7e6018, 98 },
    { 0x72e6019, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5e601f, 61 },
    { 0xf7ce6020, 100 },
    { 0, 0 },
    { 0x10be6022, 95 },
    { 0, 0 },
    { 0xde6024, 98 },
    { 0xa67e6024, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdae6028, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee602b, 81 },
    { 0xae8e602b, 90 },
    { 0x298e602b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84e6032, 61 },
    { 0x749e6032, 61 },
    { 0x6bde6033, 95 },
    { 0xee0e6034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee603b, 68 },
    { 0x600e603b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e6041, 23 },
    { 0, 0 },
    { 0xb33e6043, 90 },
    { 0xcc4e6043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe6047, 91 },
    { 0, 0 },
    { 0xce8e6049, 76 },
    { 0x234e6049, 90 },
    { 0x104e6049, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e6052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece6058, 98 },
    { 0xae6059, 23 },
    { 0x18be6058, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe605f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833e6065, 34 },
    { 0x1bbe6066, 98 },
    { 0x9be6067, 23 },
    { 0, 0 },
    { 0xb5fe6069, 33 },
    { 0, 0 },
    { 0xd33e606b, 33 },
    { 0, 0 },
    { 0x6a3e606d, 90 },
    { 0, 0 },
    { 0xb05e606f, 76 },
    { 0xc0e606f, 86 },
    { 0x370e6071, 38 },
    { 0xb3be6072, 46 },
    { 0, 0 },
    { 0xf0ee6074, 80 },
    { 0, 0 },
    { 0x56ce6076, 100 },
    { 0, 0 },
    { 0x86ae6078, 90 },
    { 0, 0 },
    { 0x35ee607a, 90 },
    { 0xb79e607b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2e608b, 34 },
    { 0x5b3e608c, 99 },
    { 0xb54e608d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6e6091, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ee6099, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae609d, 46 },
    { 0x6dde609d, 46 },
    { 0, 0 },
    { 0x91ee60a0, 90 },
    { 0, 0 },
    { 0x100e60a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48de60a8, 33 },
    { 0xdd8e60a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880e60b3, 66 },
    { 0, 0 },
    { 0x82ae60b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552e60b9, 90 },
    { 0x751e60b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2e60c0, 34 },
    { 0xdf3e60c0, 95 },
    { 0xdc3e60c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7e60c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x513e60c9, 100 },
    { 0, 0 },
    { 0x4c6e60cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbe60ce, 67 },
    { 0x33ae60cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e60d2, 76 },
    { 0x447e60d2, 33 },
    { 0x5e7e60d3, 33 },
    { 0x116e60d2, 93 },
    { 0x4d1e60d6, 87 },
    { 0xc5be60d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dae60db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e60de, 66 },
    { 0, 0 },
    { 0x4e1e60e0, 52 },
    { 0, 0 },
    { 0x41de60e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fe60e8, 43 },
    { 0x82be60e9, 61 },
    { 0x619e60e8, 61 },
    { 0, 0 },
    { 0x6bee60ec, 90 },
    { 0x782e60ed, 23 },
    { 0, 0 },
    { 0x777e60ef, 86 },
    { 0xefee60f0, 40 },
    { 0, 0 },
    { 0xcd5e60f2, 48 },
    { 0, 0 },
    { 0xaece60f4, 95 },
    { 0, 0 },
    { 0x40ee60f6, 46 },
    { 0, 0 },
    { 0x9fe60f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e60fe, 90 },
    { 0x2a2e60ff, 85 },
    { 0x481e6100, 85 },
    { 0, 0 },
    { 0xbafe6102, 67 },
    { 0, 0 },
    { 0x41e6104, 76 },
    { 0xfd3e6105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee6108, 61 },
    { 0, 0 },
    { 0xae0e610a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e6110, 90 },
    { 0x95ae6111, 55 },
    { 0x766e6112, 95 },
    { 0xc43e6112, 95 },
    { 0x44ce6114, 55 },
    { 0, 0 },
    { 0xd9fe6116, 23 },
    { 0x62be6116, 76 },
    { 0x370e6118, 76 },
    { 0x1fae6116, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e611e, 95 },
    { 0x58fe611f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe6122, 48 },
    { 0, 0 },
    { 0x9d5e6124, 88 },
    { 0xaace6124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebe6129, 55 },
    { 0, 0 },
    { 0x455e612b, 23 },
    { 0x62ae612b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fe6135, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4e6139, 85 },
    { 0xa1fe613a, 60 },
    { 0xf4de6139, 33 },
    { 0xaf3e6139, 43 },
    { 0xad5e613b, 93 },
    { 0x9f6e613e, 77 },
    { 0x85be613f, 78 },
    { 0x8a6e613f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cae614c, 90 },
    { 0xd32e614d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde6151, 23 },
    { 0xc5de6151, 23 },
    { 0x7fae6153, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e6156, 46 },
    { 0, 0 },
    { 0xb10e6158, 81 },
    { 0, 0 },
    { 0x1d6e615a, 34 },
    { 0, 0 },
    { 0x811e615c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebe615f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece6163, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e6168, 43 },
    { 0, 0 },
    { 0xcae616a, 33 },
    { 0, 0 },
    { 0xc74e616c, 55 },
    { 0x46ee616d, 90 },
    { 0xce8e616e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fe6171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbe617b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2e6184, 66 },
    { 0xebfe6184, 95 },
    { 0xfe3e6184, 90 },
    { 0xa07e6185, 19 },
    { 0, 0 },
    { 0x851e6189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e618e, 55 },
    { 0xb11e618f, 83 },
    { 0, 0 },
    { 0xb27e6191, 33 },
    { 0xbf6e6191, 61 },
    { 0, 0 },
    { 0x2ebe6194, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17e6199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552e61a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e61a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fe61a7, 98 },
    { 0x698e61a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e61ad, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace61b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5e61b3, 61 },
    { 0xecde61b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842e61b9, 23 },
    { 0x9b5e61ba, 84 },
    { 0x5f5e61b9, 95 },
    { 0x2d0e61bc, 98 },
    { 0xadbe61ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7e61c0, 90 },
    { 0x906e61c1, 23 },
    { 0x81ae61c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e61c5, 23 },
    { 0xe9ce61c5, 91 },
    { 0x577e61c7, 90 },
    { 0x64de61c8, 90 },
    { 0x260e61c9, 95 },
    { 0, 0 },
    { 0x76fe61cb, 23 },
    { 0, 0 },
    { 0x309e61cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e61d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e61d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04e61e4, 55 },
    { 0, 0 },
    { 0x3be61e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4e61e9, 34 },
    { 0, 0 },
    { 0xdfe61eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce61f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61e6200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52e6203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e6209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e6211, 95 },
    { 0xf48e6212, 33 },
    { 0x66ce6212, 33 },
    { 0x2b3e6212, 8 },
    { 0, 0 },
    { 0x614e6216, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e621c, 33 },
    { 0x850e621d, 43 },
    { 0x78de621c, 90 },
    { 0x142e621f, 90 },
    { 0, 0 },
    { 0x98fe6221, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823e6225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660e622c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847e6230, 77 },
    { 0x6c9e6230, 23 },
    { 0x306e6232, 84 },
    { 0x8f8e6233, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6e623a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ee623f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ae6245, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841e624b, 23 },
    { 0x789e624c, 43 },
    { 0xf6de624d, 65 },
    { 0x2bfe624e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce6252, 98 },
    { 0, 0 },
    { 0x5a9e6254, 33 },
    { 0, 0 },
    { 0xf82e6256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x137e6259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e625f, 98 },
    { 0xfc2e6260, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7e6264, 90 },
    { 0x7a1e6264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de6268, 90 },
    { 0xdcae6268, 61 },
    { 0x33e626a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348e6270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bae6273, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57e627f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae6282, 100 },
    { 0xed3e6283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e6288, 33 },
    { 0xc40e6289, 98 },
    { 0, 0 },
    { 0x407e628b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e628e, 43 },
    { 0xdeae628f, 90 },
    { 0xeb3e6290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5e6295, 76 },
    { 0xf0de6296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e629e, 65 },
    { 0xe14e629f, 61 },
    { 0xcd4e62a0, 66 },
    { 0, 0 },
    { 0x3c5e62a2, 81 },
    { 0x1d4e62a2, 90 },
    { 0xf9be62a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843e62a9, 33 },
    { 0x931e62aa, 23 },
    { 0x71be62a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbe62b3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705e62ba, 61 },
    { 0xf3ce62ba, 90 },
    { 0x98fe62bc, 33 },
    { 0x11fe62bd, 33 },
    { 0, 0 },
    { 0xd52e62bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58e62c3, 90 },
    { 0, 0 },
    { 0x380e62c5, 61 },
    { 0x8dfe62c6, 43 },
    { 0x5f2e62c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e62ca, 95 },
    { 0xa30e62ca, 34 },
    { 0xc3fe62cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce62d8, 34 },
    { 0, 0 },
    { 0x7e3e62da, 91 },
    { 0x200e62db, 33 },
    { 0xeafe62dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e62e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e62e7, 55 },
    { 0x195e62e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4e62ee, 88 },
    { 0x6d7e62ee, 40 },
    { 0x77ae62ef, 85 },
    { 0x825e62f1, 91 },
    { 0xa42e62f2, 34 },
    { 0, 0 },
    { 0xa16e62f4, 99 },
    { 0x41de62f5, 66 },
    { 0x78ce62f6, 95 },
    { 0xccae62f4, 61 },
    { 0xdeee62f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fe62fe, 93 },
    { 0, 0 },
    { 0xdc9e6300, 66 },
    { 0x342e6301, 95 },
    { 0xd61e6302, 23 },
    { 0xc13e6303, 46 },
    { 0, 0 },
    { 0xb9e6305, 90 },
    { 0x2dbe6306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdee630f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1e6313, 90 },
    { 0xe05e6314, 33 },
    { 0xdf6e6313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0e631a, 90 },
    { 0x196e631b, 76 },
    { 0, 0 },
    { 0x56fe631d, 95 },
    { 0x848e631d, 95 },
    { 0x157e631f, 46 },
    { 0x144e631d, 98 },
    { 0, 0 },
    { 0x45de6322, 76 },
    { 0, 0 },
    { 0xd4de6324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe632a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03e632f, 55 },
    { 0, 0 },
    { 0x6e9e6331, 78 },
    { 0x22e6331, 61 },
    { 0x2ffe6333, 61 },
    { 0x15ae6334, 33 },
    { 0x83ee6335, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498e633b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e633e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e6341, 33 },
    { 0x4fbe6342, 17 },
    { 0x849e6343, 19 },
    { 0, 0 },
    { 0x9ace6345, 61 },
    { 0x444e6346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e6349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fe634f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de6357, 90 },
    { 0xc95e6358, 67 },
    { 0x20ee6359, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e635e, 95 },
    { 0x477e635f, 84 },
    { 0xf30e635f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee6366, 76 },
    { 0xfd3e6367, 34 },
    { 0, 0 },
    { 0xf15e6369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e6370, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae6374, 85 },
    { 0, 0 },
    { 0x373e6376, 61 },
    { 0, 0 },
    { 0x166e6378, 95 },
    { 0xd6de6379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fe6381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66e6384, 78 },
    { 0x53fe6385, 59 },
    { 0xef6e6386, 66 },
    { 0x213e6387, 95 },
    { 0xe3e6388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffe638c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfe6394, 53 },
    { 0xcd8e6395, 8 },
    { 0x21ce6394, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88e639a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3e63a0, 46 },
    { 0xb1fe63a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde63a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde63a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5e63aa, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919e63b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x383e63b7, 88 },
    { 0xf59e63b7, 33 },
    { 0xc8fe63b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25e63bd, 87 },
    { 0, 0 },
    { 0x660e63bf, 95 },
    { 0xc3be63c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce63c3, 33 },
    { 0x1a5e63c4, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e63c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0de63ca, 23 },
    { 0x5ebe63cb, 61 },
    { 0x4e9e63cb, 55 },
    { 0xaa4e63cd, 33 },
    { 0, 0 },
    { 0x26ce63cf, 95 },
    { 0x4a9e63d0, 76 },
    { 0xbb9e63d0, 95 },
    { 0xfc7e63d2, 34 },
    { 0xbe6e63d3, 43 },
    { 0x6b1e63d0, 61 },
    { 0x199e63d1, 95 },
    { 0x4a8e63d0, 53 },
    { 0x47ee63d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe63da, 76 },
    { 0, 0 },
    { 0x84ae63dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6e63e4, 2 },
    { 0x2c8e63e5, 43 },
    { 0x7a0e63e6, 52 },
    { 0xc3ee63e7, 85 },
    { 0x4de63e8, 55 },
    { 0x26ee63e6, 61 },
    { 0xbaee63ea, 95 },
    { 0x42e63e5, 43 },
    { 0x853e63e9, 98 },
    { 0xb5de63ed, 23 },
    { 0xece63ec, 66 },
    { 0xb31e63e6, 46 },
    { 0xf65e63f0, 90 },
    { 0xc95e63e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7e63f8, 61 },
    { 0xa02e63f8, 43 },
    { 0x383e63fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e63fd, 61 },
    { 0x2fe63fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fe6403, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6406, 61 },
    { 0x3cbe6407, 46 },
    { 0xa89e6407, 61 },
    { 0x3cde6409, 43 },
    { 0, 0 },
    { 0x70fe640b, 23 },
    { 0xe67e640c, 77 },
    { 0x4d5e640c, 76 },
    { 0xca3e640e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82be6411, 33 },
    { 0, 0 },
    { 0xc04e6413, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369e6417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5e641b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1e641e, 33 },
    { 0, 0 },
    { 0x4bbe6420, 76 },
    { 0, 0 },
    { 0xbe1e6422, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ce6426, 55 },
    { 0, 0 },
    { 0x993e6428, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x134e642b, 93 },
    { 0x595e642b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e642f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e6434, 90 },
    { 0xa6ae6434, 33 },
    { 0, 0 },
    { 0xc1e6437, 23 },
    { 0xfcbe6438, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4e643d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe6443, 67 },
    { 0x8b3e6444, 76 },
    { 0, 0 },
    { 0x293e6446, 34 },
    { 0xb2be6447, 76 },
    { 0, 0 },
    { 0xc79e6449, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e644c, 52 },
    { 0, 0 },
    { 0xcae644e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x744e6451, 98 },
    { 0x594e6452, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ee645c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x980e645f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76be6463, 78 },
    { 0, 0 },
    { 0x59e6465, 90 },
    { 0, 0 },
    { 0x862e6467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5e646a, 100 },
    { 0x9b6e646b, 100 },
    { 0, 0 },
    { 0x9cce646d, 61 },
    { 0xbd9e646d, 61 },
    { 0x4bde646d, 90 },
    { 0x189e646e, 43 },
    { 0, 0 },
    { 0x260e6472, 33 },
    { 0, 0 },
    { 0xe9ae6474, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85be6478, 33 },
    { 0x963e6479, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e6481, 95 },
    { 0xae9e6482, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e6488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e648c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ce6496, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e649a, 33 },
    { 0, 0 },
    { 0x16de649c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe64a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38de64a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee64a8, 67 },
    { 0xac2e64a8, 23 },
    { 0x4bee64aa, 98 },
    { 0xf58e64ab, 33 },
    { 0, 0 },
    { 0xb8ce64ad, 66 },
    { 0x635e64ad, 91 },
    { 0xc38e64af, 98 },
    { 0x45fe64ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88e64b3, 83 },
    { 0x8e0e64b3, 23 },
    { 0x621e64b5, 61 },
    { 0xa40e64b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ae64bf, 95 },
    { 0, 0 },
    { 0x10ce64c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5e64c4, 61 },
    { 0xa18e64c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36e64cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e64d0, 23 },
    { 0xdbce64d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e64d6, 33 },
    { 0, 0 },
    { 0x5f7e64d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e64dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebe64e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63e64e6, 23 },
    { 0x4d0e64e7, 99 },
    { 0x8cae64e8, 90 },
    { 0x7a5e64e8, 33 },
    { 0x2fae64ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57de64f1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce64f7, 76 },
    { 0, 0 },
    { 0x1dee64f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe64fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fe6504, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8de650f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95e6513, 34 },
    { 0, 0 },
    { 0xb15e6515, 55 },
    { 0x50be6516, 93 },
    { 0xb58e6516, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4e651a, 43 },
    { 0xa90e651b, 23 },
    { 0, 0 },
    { 0x1e3e651d, 91 },
    { 0x30e651e, 23 },
    { 0, 0 },
    { 0x379e6520, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x655e6523, 56 },
    { 0xeaae6524, 33 },
    { 0xce7e6524, 94 },
    { 0, 0 },
    { 0xb1ce6527, 91 },
    { 0xbffe6528, 90 },
    { 0, 0 },
    { 0x709e652a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5e652f, 88 },
    { 0x865e6530, 33 },
    { 0x3f2e6531, 76 },
    { 0x817e6530, 76 },
    { 0x7c9e652f, 33 },
    { 0, 0 },
    { 0x6b0e6535, 23 },
    { 0, 0 },
    { 0xea9e6537, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dae653a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ee653f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e6543, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6546, 61 },
    { 0xba9e6546, 23 },
    { 0, 0 },
    { 0x936e6549, 43 },
    { 0x2cbe654a, 90 },
    { 0x4a2e654b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118e6550, 76 },
    { 0x423e6551, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e6554, 90 },
    { 0x724e6555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15e6558, 86 },
    { 0xdd5e6559, 90 },
    { 0xc5be655a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce6561, 90 },
    { 0x4dfe6562, 33 },
    { 0xe16e6562, 46 },
    { 0x7f8e6561, 95 },
    { 0xb8de6565, 61 },
    { 0xa9fe6563, 90 },
    { 0x515e6567, 61 },
    { 0x9a5e6567, 33 },
    { 0xf7be6569, 98 },
    { 0x8de6563, 98 },
    { 0x918e656b, 34 },
    { 0, 0 },
    { 0x20ee656d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e6571, 87 },
    { 0, 0 },
    { 0xf1de6573, 86 },
    { 0x1b6e6574, 75 },
    { 0xd77e6575, 78 },
    { 0xc23e6575, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e6579, 33 },
    { 0xf6ee657a, 100 },
    { 0x775e657b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e6582, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfe6587, 76 },
    { 0, 0 },
    { 0x86ae6589, 95 },
    { 0xca7e658a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131e658e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32de6598, 52 },
    { 0xe6be6599, 99 },
    { 0x9cbe6599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e659f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e65aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aae65ad, 90 },
    { 0x6d2e65ae, 90 },
    { 0x9a6e65af, 42 },
    { 0xd0de65b0, 76 },
    { 0, 0 },
    { 0x95be65b2, 46 },
    { 0, 0 },
    { 0x7ade65b4, 99 },
    { 0x56ce65b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e65b8, 76 },
    { 0, 0 },
    { 0xbeae65ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdce65be, 61 },
    { 0x1d3e65bf, 61 },
    { 0x9ebe65bf, 81 },
    { 0xf85e65be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e65c4, 66 },
    { 0, 0 },
    { 0x954e65c6, 23 },
    { 0x688e65c6, 78 },
    { 0x53be65c6, 83 },
    { 0xea6e65c7, 43 },
    { 0, 0 },
    { 0xcb2e65cb, 61 },
    { 0x479e65cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe65cf, 68 },
    { 0, 0 },
    { 0x7d2e65d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe65d7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae65dc, 67 },
    { 0x611e65dd, 43 },
    { 0xa31e65de, 84 },
    { 0x666e65de, 33 },
    { 0xdcee65de, 90 },
    { 0x400e65e1, 23 },
    { 0xf41e65e0, 66 },
    { 0, 0 },
    { 0xffae65e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae65e7, 61 },
    { 0x9c5e65e8, 98 },
    { 0xb14e65e7, 76 },
    { 0x602e65ea, 77 },
    { 0, 0 },
    { 0xef2e65ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e65f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e65f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe65fb, 33 },
    { 0xb67e65fb, 92 },
    { 0x758e65fd, 94 },
    { 0xd8fe65fb, 44 },
    { 0, 0 },
    { 0xf77e6600, 76 },
    { 0x3ee6601, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x304e6604, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e6608, 64 },
    { 0xd24e6609, 74 },
    { 0xff3e660a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60e6612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de6617, 33 },
    { 0x80ae6618, 51 },
    { 0xc2fe6617, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ae661f, 88 },
    { 0xe83e6620, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaae6626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcae662d, 61 },
    { 0x33be662e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53e6631, 76 },
    { 0x676e6631, 91 },
    { 0x758e6633, 90 },
    { 0x32fe6634, 98 },
    { 0x512e6634, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e6638, 33 },
    { 0xd6de6639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6644, 77 },
    { 0x52ce6645, 95 },
    { 0x60e6645, 58 },
    { 0xf5de6644, 43 },
    { 0xa57e6648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18e664c, 43 },
    { 0x678e664d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75de6650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2e6655, 90 },
    { 0, 0 },
    { 0xc66e6657, 23 },
    { 0x965e6657, 34 },
    { 0, 0 },
    { 0xdd5e665a, 90 },
    { 0xbaae665b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e665e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ce6661, 46 },
    { 0, 0 },
    { 0xa29e6663, 100 },
    { 0xe8fe6664, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be6668, 68 },
    { 0x1f0e6668, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aee6671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbe6677, 95 },
    { 0xc49e6678, 90 },
    { 0x844e6679, 98 },
    { 0x9b3e6679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de667d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36e6683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae6686, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e6689, 33 },
    { 0x698e668a, 46 },
    { 0x2be668b, 61 },
    { 0xc09e668a, 76 },
    { 0x5b6e668d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e6690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0de6694, 93 },
    { 0xad2e6694, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eae669f, 33 },
    { 0xdf7e66a0, 37 },
    { 0xe3ce66a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x788e66a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616e66a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e66ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5e66af, 98 },
    { 0, 0 },
    { 0x2fbe66b1, 98 },
    { 0x419e66b2, 43 },
    { 0x89e66b3, 61 },
    { 0xcafe66b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e66b7, 33 },
    { 0x516e66b8, 100 },
    { 0xbd8e66b7, 33 },
    { 0, 0 },
    { 0x913e66bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e66c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950e66c8, 77 },
    { 0, 0 },
    { 0xd9ae66ca, 90 },
    { 0xd56e66ca, 61 },
    { 0x34de66cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9e66cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffe66d5, 90 },
    { 0x573e66d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22e66d9, 55 },
    { 0x9a5e66da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aae66e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e66e6, 98 },
    { 0xedfe66e7, 95 },
    { 0x823e66e6, 98 },
    { 0x7d5e66e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x913e66ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e66f6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e6702, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae6705, 33 },
    { 0xa11e6705, 98 },
    { 0x961e6707, 61 },
    { 0, 0 },
    { 0xea8e6709, 76 },
    { 0x4a4e670a, 94 },
    { 0, 0 },
    { 0x687e670c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e670f, 66 },
    { 0xf41e6710, 95 },
    { 0x33de6711, 61 },
    { 0, 0 },
    { 0xd8ce6713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750e6718, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ce671b, 90 },
    { 0xf12e671c, 90 },
    { 0x75e671d, 55 },
    { 0x98e671e, 90 },
    { 0, 0 },
    { 0x7e4e6720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0e6723, 90 },
    { 0x155e6723, 95 },
    { 0xd4ae6725, 95 },
    { 0x5b2e6723, 98 },
    { 0x859e6727, 96 },
    { 0x137e6724, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ee6730, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63e6735, 84 },
    { 0x8f2e6736, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5de6739, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e6742, 95 },
    { 0, 0 },
    { 0x312e6744, 68 },
    { 0, 0 },
    { 0x41de6746, 78 },
    { 0x58e6747, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e674e, 90 },
    { 0x8cfe674f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe6755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce675c, 33 },
    { 0, 0 },
    { 0x9e7e675e, 43 },
    { 0xb89e675e, 61 },
    { 0, 0 },
    { 0xa67e6761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7e6768, 33 },
    { 0xc3e6768, 90 },
    { 0, 0 },
    { 0x827e676b, 33 },
    { 0, 0 },
    { 0xc68e676d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e6773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4e677f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bde6784, 52 },
    { 0xe1ce6784, 95 },
    { 0x3b1e6786, 23 },
    { 0xd80e6787, 95 },
    { 0x90fe6786, 61 },
    { 0, 0 },
    { 0xc1ee678a, 90 },
    { 0x6ade678a, 76 },
    { 0, 0 },
    { 0x9eae678d, 90 },
    { 0x763e678d, 98 },
    { 0x9cae678f, 78 },
    { 0x6bae6790, 33 },
    { 0xab8e6791, 23 },
    { 0xae7e6791, 90 },
    { 0xb65e6793, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ce6798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede67a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8e67a6, 95 },
    { 0x994e67a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72e67aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151e67af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9e67b5, 85 },
    { 0xc7ae67b6, 66 },
    { 0xd31e67b7, 99 },
    { 0x70e67b8, 61 },
    { 0x3f1e67b9, 76 },
    { 0x1a5e67b5, 98 },
    { 0x92ae67b7, 90 },
    { 0xe99e67b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21e67c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ee67c3, 99 },
    { 0xa44e67c3, 90 },
    { 0xb80e67c5, 98 },
    { 0x585e67c6, 76 },
    { 0xa0ce67c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711e67cb, 33 },
    { 0, 0 },
    { 0xe7ce67cd, 55 },
    { 0x9e4e67ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe67d2, 90 },
    { 0, 0 },
    { 0x60ae67d4, 33 },
    { 0xc1ee67d4, 61 },
    { 0x183e67d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e67da, 90 },
    { 0xc88e67db, 90 },
    { 0x2f7e67dc, 23 },
    { 0x2abe67dd, 81 },
    { 0x71ee67dd, 23 },
    { 0xe0ce67df, 66 },
    { 0, 0 },
    { 0x72e67e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1e67e8, 66 },
    { 0x8dde67e9, 55 },
    { 0xc9e67e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece67ed, 90 },
    { 0xd91e67ee, 61 },
    { 0xfcbe67ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ce67f3, 75 },
    { 0x603e67f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642e67f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe67fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928e6801, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4be6804, 90 },
    { 0, 0 },
    { 0x4d5e6806, 95 },
    { 0, 0 },
    { 0x750e6808, 55 },
    { 0xfcee6809, 23 },
    { 0x30fe680a, 66 },
    { 0x5c3e680b, 88 },
    { 0xbcae680a, 95 },
    { 0x32ee6809, 23 },
    { 0x64ee680e, 69 },
    { 0xa71e680f, 61 },
    { 0x209e680f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609e681a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de681d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee6820, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3de6826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50e682f, 95 },
    { 0, 0 },
    { 0xc52e6831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24e6835, 61 },
    { 0, 0 },
    { 0x9ace6837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce683e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e6841, 93 },
    { 0x9f0e6842, 93 },
    { 0x5bde6843, 61 },
    { 0x10e6844, 55 },
    { 0x5a2e6841, 81 },
    { 0x324e6846, 61 },
    { 0x627e6846, 61 },
    { 0xe14e6848, 52 },
    { 0x8e6849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e684e, 33 },
    { 0xf5fe684f, 33 },
    { 0, 0 },
    { 0x856e6851, 98 },
    { 0, 0 },
    { 0x932e6853, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbe6857, 78 },
    { 0x9b9e6857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de685b, 90 },
    { 0xbb1e685c, 90 },
    { 0xd10e685c, 98 },
    { 0, 0 },
    { 0x46be685f, 84 },
    { 0xe2be6860, 76 },
    { 0x7e3e6860, 98 },
    { 0xa9ce6862, 36 },
    { 0x37be6863, 61 },
    { 0x2e9e6862, 75 },
    { 0x87de6865, 76 },
    { 0x904e6866, 66 },
    { 0xf7de6867, 90 },
    { 0x696e6868, 52 },
    { 0x29ce6861, 33 },
    { 0, 0 },
    { 0x162e686b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655e686f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e6872, 34 },
    { 0xf6ce6873, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae6879, 55 },
    { 0x6a5e687a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee687e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922e6883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ce6887, 98 },
    { 0xe62e6888, 43 },
    { 0x260e6889, 33 },
    { 0, 0 },
    { 0x36ee688b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ce688e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae6894, 76 },
    { 0x3dbe6894, 95 },
    { 0xd15e6895, 33 },
    { 0xaf2e6896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0e689a, 46 },
    { 0x57fe689b, 55 },
    { 0x955e689a, 76 },
    { 0x513e689a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e68a0, 67 },
    { 0x80ee68a0, 90 },
    { 0, 0 },
    { 0x5a1e68a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647e68a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e68ad, 46 },
    { 0x86ae68ae, 61 },
    { 0x1c2e68af, 76 },
    { 0xdb8e68ad, 67 },
    { 0xd86e68b1, 75 },
    { 0xa52e68b1, 90 },
    { 0x283e68b2, 33 },
    { 0xc0e68b2, 61 },
    { 0xd55e68b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e68b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4e68bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479e68bf, 99 },
    { 0, 0 },
    { 0x266e68c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bae68c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5e68ca, 48 },
    { 0xd99e68cb, 43 },
    { 0xa6fe68cc, 33 },
    { 0xe44e68ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x224e68d0, 98 },
    { 0xa84e68d1, 55 },
    { 0x9cde68d2, 61 },
    { 0xb88e68d2, 46 },
    { 0, 0 },
    { 0xafae68d5, 90 },
    { 0, 0 },
    { 0x17ce68d7, 95 },
    { 0, 0 },
    { 0x8bbe68d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e68de, 23 },
    { 0x49de68df, 61 },
    { 0x687e68de, 90 },
    { 0, 0 },
    { 0x1d9e68e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e68e6, 23 },
    { 0, 0 },
    { 0x1b4e68e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde68ed, 90 },
    { 0x5cde68ee, 78 },
    { 0xc33e68ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be68f3, 55 },
    { 0x2ede68f4, 99 },
    { 0x7b9e68f4, 69 },
    { 0x728e68f4, 76 },
    { 0xacee68f6, 46 },
    { 0, 0 },
    { 0x6fce68f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89e6902, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3be690e, 61 },
    { 0x7a2e690f, 98 },
    { 0xcfde690e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfee6914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de6919, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bce691c, 90 },
    { 0x674e691c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e6922, 34 },
    { 0xb02e6923, 61 },
    { 0x5a6e6923, 61 },
    { 0x8b9e6925, 56 },
    { 0x32be6926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee692a, 95 },
    { 0x218e692b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554e692f, 33 },
    { 0x76fe6930, 46 },
    { 0x46be6930, 61 },
    { 0, 0 },
    { 0x380e6933, 95 },
    { 0x5e2e6934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae6939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafe693e, 46 },
    { 0xaede693f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcce6943, 61 },
    { 0x3cae6944, 85 },
    { 0, 0 },
    { 0x8b1e6946, 43 },
    { 0, 0 },
    { 0x81de6948, 95 },
    { 0xfc7e6949, 53 },
    { 0xf46e694a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee694e, 90 },
    { 0, 0 },
    { 0x730e6950, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6953, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e6956, 99 },
    { 0xb96e6957, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802e695c, 95 },
    { 0x819e695d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe6963, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e6966, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce6971, 46 },
    { 0x691e6972, 77 },
    { 0x8dce6973, 60 },
    { 0xe31e6974, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ae6978, 61 },
    { 0x86ae6978, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888e697d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce6980, 93 },
    { 0xd89e6981, 90 },
    { 0x754e6981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e6986, 19 },
    { 0x59ae6987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaee698b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ee6993, 85 },
    { 0xffbe6994, 98 },
    { 0, 0 },
    { 0x8c7e6996, 23 },
    { 0x566e6996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1e699a, 61 },
    { 0, 0 },
    { 0x15e699c, 61 },
    { 0xc67e699d, 61 },
    { 0, 0 },
    { 0x6b0e699f, 99 },
    { 0x3e4e69a0, 97 },
    { 0x77ce69a0, 90 },
    { 0x9dbe699f, 95 },
    { 0xbe7e69a3, 90 },
    { 0x94ee69a4, 99 },
    { 0xf2e69a3, 53 },
    { 0x82de69a2, 98 },
    { 0, 0 },
    { 0x24fe69a8, 77 },
    { 0x7e3e69a8, 98 },
    { 0, 0 },
    { 0x694e69ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e69ae, 91 },
    { 0xd82e69af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ae69b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6e69b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531e69bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238e69c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e69cb, 99 },
    { 0x3d1e69cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9e69d0, 88 },
    { 0x17ae69d0, 95 },
    { 0x1c0e69d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe69d9, 61 },
    { 0x6ae69d9, 61 },
    { 0x897e69db, 90 },
    { 0x47e69dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e69df, 23 },
    { 0, 0 },
    { 0x60fe69e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fe69e6, 81 },
    { 0, 0 },
    { 0xd95e69e8, 90 },
    { 0, 0 },
    { 0x7fce69ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e69f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfe69f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e69ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960e6a06, 42 },
    { 0xe1e6a06, 98 },
    { 0x7bbe6a07, 95 },
    { 0x5f5e6a09, 95 },
    { 0xe7de6a0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de6a0d, 33 },
    { 0x80fe6a0e, 85 },
    { 0xe53e6a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce6a12, 100 },
    { 0xec8e6a12, 90 },
    { 0x79ee6a12, 61 },
    { 0x4c5e6a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358e6a1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5e6a22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1e6a25, 34 },
    { 0xfe3e6a26, 84 },
    { 0, 0 },
    { 0x546e6a28, 98 },
    { 0xdd1e6a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e6a2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75e6a31, 33 },
    { 0, 0 },
    { 0xdf7e6a33, 56 },
    { 0, 0 },
    { 0x53ee6a35, 90 },
    { 0x822e6a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe6a3c, 52 },
    { 0xd2fe6a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae6a41, 33 },
    { 0x6cbe6a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e6a47, 33 },
    { 0x2dde6a47, 23 },
    { 0xbd4e6a49, 95 },
    { 0xbaee6a4a, 76 },
    { 0xe2ee6a4b, 61 },
    { 0x66fe6a49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5e6a4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1e6a56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e6a5f, 90 },
    { 0xe61e6a5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee6a63, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e6a67, 95 },
    { 0x790e6a68, 78 },
    { 0xf6be6a68, 33 },
    { 0x59ae6a6a, 33 },
    { 0, 0 },
    { 0x91be6a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce6a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e6a76, 34 },
    { 0x8eae6a77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245e6a7d, 99 },
    { 0x4cbe6a7e, 90 },
    { 0xe6a7f, 34 },
    { 0x66de6a7f, 80 },
    { 0x84ce6a81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce6a84, 40 },
    { 0, 0 },
    { 0x5c7e6a86, 33 },
    { 0x4b5e6a87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee6a8a, 23 },
    { 0xe80e6a8b, 90 },
    { 0x5c5e6a8c, 69 },
    { 0x46ce6a8d, 61 },
    { 0x507e6a8d, 78 },
    { 0x17e6a8f, 91 },
    { 0, 0 },
    { 0x593e6a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0e6a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde6aa0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ee6aa4, 61 },
    { 0x719e6aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e6aaa, 33 },
    { 0x4b6e6aab, 100 },
    { 0x79e6aab, 90 },
    { 0, 0 },
    { 0x6f6e6aae, 66 },
    { 0xc63e6aaf, 61 },
    { 0, 0 },
    { 0xb1be6ab1, 90 },
    { 0x450e6ab2, 95 },
    { 0x289e6ab3, 33 },
    { 0x8aae6ab3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110e6ab9, 23 },
    { 0x894e6ab9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989e6abe, 99 },
    { 0, 0 },
    { 0x5f3e6ac0, 33 },
    { 0x5e7e6ac1, 68 },
    { 0, 0 },
    { 0x1d8e6ac3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586e6ac7, 90 },
    { 0x1ee6ac7, 90 },
    { 0x808e6ac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e6acd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ae6ad6, 50 },
    { 0, 0 },
    { 0x92fe6ad8, 61 },
    { 0xa99e6ad9, 61 },
    { 0x54de6ada, 68 },
    { 0xdb2e6ad9, 66 },
    { 0, 0 },
    { 0x48ee6add, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4e6ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x761e6aea, 61 },
    { 0xdc3e6aeb, 74 },
    { 0xf20e6aeb, 66 },
    { 0xeffe6aed, 46 },
    { 0xe5be6aee, 61 },
    { 0xb09e6aec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28e6af2, 33 },
    { 0, 0 },
    { 0xd16e6af4, 17 },
    { 0x3e5e6af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6e6afa, 33 },
    { 0, 0 },
    { 0x46be6afc, 90 },
    { 0, 0 },
    { 0xbf1e6afe, 67 },
    { 0x870e6aff, 61 },
    { 0x4a5e6aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2e6b04, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e6b08, 98 },
    { 0x127e6b09, 34 },
    { 0xe08e6b0a, 33 },
    { 0x97fe6b09, 97 },
    { 0x8dbe6b0c, 98 },
    { 0x67ee6b09, 91 },
    { 0xc45e6b08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ce6b12, 61 },
    { 0xc57e6b13, 81 },
    { 0x540e6b14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64be6b18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6b21, 43 },
    { 0x7dfe6b21, 87 },
    { 0x8d9e6b23, 23 },
    { 0xf7e6b24, 90 },
    { 0x801e6b23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee6b2a, 85 },
    { 0, 0 },
    { 0x8b4e6b2c, 91 },
    { 0, 0 },
    { 0x554e6b2e, 28 },
    { 0x501e6b2e, 23 },
    { 0, 0 },
    { 0xaede6b31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7e6b34, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de6b37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x972e6b3a, 43 },
    { 0x926e6b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cae6b40, 90 },
    { 0xa7ae6b41, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e6b4d, 33 },
    { 0xc7de6b4e, 61 },
    { 0x1d0e6b4f, 76 },
    { 0, 0 },
    { 0x24be6b51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6b56, 90 },
    { 0, 0 },
    { 0x37ce6b58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfe6b5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbe6b5f, 34 },
    { 0, 0 },
    { 0x671e6b61, 95 },
    { 0x14fe6b62, 98 },
    { 0xd28e6b63, 33 },
    { 0xe50e6b63, 65 },
    { 0, 0 },
    { 0x6a3e6b66, 33 },
    { 0, 0 },
    { 0x25ee6b68, 90 },
    { 0xaee6b68, 17 },
    { 0xd4ee6b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6b6e, 50 },
    { 0xb80e6b6e, 76 },
    { 0xab0e6b6e, 90 },
    { 0x793e6b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53e6b77, 90 },
    { 0, 0 },
    { 0x6a4e6b79, 61 },
    { 0x242e6b79, 33 },
    { 0x14ae6b7b, 100 },
    { 0, 0 },
    { 0x25ae6b7d, 65 },
    { 0xb81e6b7d, 95 },
    { 0x9ae6b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x210e6b82, 61 },
    { 0xf0ae6b83, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e6b88, 23 },
    { 0xbcbe6b88, 88 },
    { 0, 0 },
    { 0x8f7e6b8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce6b9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e6b9e, 75 },
    { 0, 0 },
    { 0xe28e6ba0, 84 },
    { 0, 0 },
    { 0xed9e6ba2, 90 },
    { 0, 0 },
    { 0x87de6ba4, 95 },
    { 0x915e6ba5, 99 },
    { 0xeb5e6ba6, 93 },
    { 0xd0ae6ba5, 95 },
    { 0x7c3e6ba8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x981e6bab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe6bae, 91 },
    { 0x9b1e6baf, 33 },
    { 0xede6bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6de6bb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652e6bb8, 90 },
    { 0x130e6bb9, 61 },
    { 0, 0 },
    { 0x84fe6bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e6bbe, 90 },
    { 0x6b0e6bbf, 95 },
    { 0, 0 },
    { 0x856e6bc1, 68 },
    { 0, 0 },
    { 0xdbee6bc3, 76 },
    { 0, 0 },
    { 0x4ade6bc5, 23 },
    { 0xb0be6bc6, 78 },
    { 0x759e6bc7, 65 },
    { 0x59e6bc7, 87 },
    { 0xf0de6bc9, 33 },
    { 0xfc2e6bca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0e6bcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae6bd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3e6bd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fe6bd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e6bdb, 55 },
    { 0x487e6bdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e6be1, 33 },
    { 0xac4e6be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46e6be5, 67 },
    { 0xec4e6be6, 61 },
    { 0xb4e6be7, 90 },
    { 0x782e6be6, 90 },
    { 0xabee6be5, 33 },
    { 0x2e3e6be5, 40 },
    { 0, 0 },
    { 0xb2ae6bec, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244e6bf1, 23 },
    { 0x746e6bf2, 98 },
    { 0x12ee6bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ae6bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de6bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x168e6bfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823e6c00, 33 },
    { 0x7d8e6c01, 99 },
    { 0xcfae6c02, 93 },
    { 0xd10e6c03, 61 },
    { 0x7f7e6c02, 61 },
    { 0x7c2e6c02, 95 },
    { 0x1c4e6c06, 90 },
    { 0xb5ae6c04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e6c0a, 23 },
    { 0xc19e6c0a, 23 },
    { 0x5b4e6c0c, 93 },
    { 0x41ae6c0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598e6c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e6c15, 33 },
    { 0xa62e6c16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63de6c1c, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ae6c20, 87 },
    { 0, 0 },
    { 0x832e6c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e6c28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9e6c2e, 98 },
    { 0x2ace6c2e, 78 },
    { 0, 0 },
    { 0x5f4e6c31, 90 },
    { 0xc0be6c32, 76 },
    { 0xfbee6c33, 52 },
    { 0, 0 },
    { 0x665e6c35, 55 },
    { 0xb6ee6c35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfae6c40, 61 },
    { 0xf14e6c41, 52 },
    { 0x834e6c41, 33 },
    { 0x5e1e6c43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e6c46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6c49, 90 },
    { 0xfe1e6c4a, 87 },
    { 0xd41e6c4b, 99 },
    { 0, 0 },
    { 0xbc6e6c4d, 58 },
    { 0xb52e6c4e, 90 },
    { 0xe70e6c4e, 99 },
    { 0x694e6c4e, 98 },
    { 0x4b5e6c4e, 95 },
    { 0, 0 },
    { 0xb1de6c53, 90 },
    { 0x591e6c54, 43 },
    { 0xfb7e6c54, 33 },
    { 0xa2ce6c55, 77 },
    { 0x2c9e6c53, 76 },
    { 0, 0 },
    { 0xb87e6c59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6c5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e6c62, 90 },
    { 0x3d5e6c63, 90 },
    { 0x433e6c64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2e6c69, 43 },
    { 0x2aee6c6a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce6c6e, 55 },
    { 0xc54e6c6f, 85 },
    { 0x35be6c6f, 64 },
    { 0x557e6c6e, 90 },
    { 0xa04e6c70, 99 },
    { 0xd2be6c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ee6c7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de6c7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e6c89, 55 },
    { 0x175e6c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0e6c90, 33 },
    { 0x333e6c91, 23 },
    { 0xe40e6c92, 93 },
    { 0xfe6e6c93, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e6c96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de6c9a, 69 },
    { 0x999e6c9b, 33 },
    { 0xa0de6c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7de6ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbe6ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9e6cad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127e6cb2, 98 },
    { 0x9bee6cb3, 34 },
    { 0, 0 },
    { 0x113e6cb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e6cb8, 91 },
    { 0, 0 },
    { 0xcfce6cba, 66 },
    { 0x620e6cbb, 52 },
    { 0xfabe6cbc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de6cbf, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de6cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfee6cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303e6cce, 61 },
    { 0, 0 },
    { 0xd15e6cd0, 90 },
    { 0x1c5e6cd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e6ce0, 69 },
    { 0xcb7e6ce1, 85 },
    { 0x80e6ce2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de6ce5, 98 },
    { 0, 0 },
    { 0x3e4e6ce7, 98 },
    { 0xf10e6ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e6ced, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9e6cf0, 90 },
    { 0xb59e6cf1, 33 },
    { 0x5d2e6cf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70be6cf5, 34 },
    { 0xd1fe6cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940e6d00, 98 },
    { 0x400e6d01, 33 },
    { 0x27ae6d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9e6d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffe6d0c, 33 },
    { 0x5d0e6d0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5e6d11, 84 },
    { 0x92fe6d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e6d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e6d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24e6d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17e6d38, 76 },
    { 0xf00e6d38, 33 },
    { 0x7b3e6d3a, 34 },
    { 0x137e6d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fe6d3e, 33 },
    { 0xb47e6d3f, 40 },
    { 0, 0 },
    { 0x5e7e6d41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855e6d46, 76 },
    { 0xc37e6d47, 46 },
    { 0xa38e6d47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276e6d4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0e6d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9e6d53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de6d5f, 90 },
    { 0x29ee6d5f, 33 },
    { 0xd8be6d61, 90 },
    { 0xbfae6d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e6d67, 55 },
    { 0xb44e6d68, 76 },
    { 0x904e6d69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eae6d70, 78 },
    { 0, 0 },
    { 0xf45e6d72, 84 },
    { 0x1c8e6d73, 61 },
    { 0xdc7e6d74, 90 },
    { 0x2a7e6d75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee6d78, 33 },
    { 0, 0 },
    { 0x39fe6d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e6d7d, 90 },
    { 0, 0 },
    { 0x96fe6d7f, 23 },
    { 0x352e6d80, 98 },
    { 0x574e6d81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8be6d84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747e6d88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e6d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4e6d93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce6d96, 68 },
    { 0, 0 },
    { 0x575e6d98, 99 },
    { 0x21de6d99, 76 },
    { 0x7d5e6d9a, 23 },
    { 0xd27e6d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99be6d9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffe6da5, 61 },
    { 0xbe3e6da6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x951e6da9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e6dad, 61 },
    { 0xff9e6dae, 100 },
    { 0x6d3e6daf, 40 },
    { 0x870e6dae, 84 },
    { 0xd25e6daf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ae6db4, 95 },
    { 0, 0 },
    { 0x659e6db6, 33 },
    { 0xca8e6db7, 90 },
    { 0x7efe6db8, 98 },
    { 0xec4e6db9, 33 },
    { 0xa8be6db7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800e6dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce6dc2, 43 },
    { 0xe36e6dc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e6dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fe6dcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e6dd3, 61 },
    { 0x84be6dd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae6de1, 52 },
    { 0x9ce6de2, 66 },
    { 0x99ce6de3, 95 },
    { 0x3bfe6de3, 46 },
    { 0x773e6de4, 61 },
    { 0x822e6de5, 90 },
    { 0xb9ae6de6, 43 },
    { 0, 0 },
    { 0x97be6de9, 65 },
    { 0x203e6dea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98e6ded, 100 },
    { 0x79de6dee, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce6df2, 90 },
    { 0, 0 },
    { 0x3a1e6df4, 33 },
    { 0xe6ee6df5, 61 },
    { 0xa79e6df6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e6dfc, 43 },
    { 0, 0 },
    { 0xf20e6dfe, 65 },
    { 0, 0 },
    { 0xf64e6e00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4e6e08, 38 },
    { 0, 0 },
    { 0xd39e6e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee6e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70e6e15, 23 },
    { 0xf09e6e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce6e19, 68 },
    { 0xc00e6e19, 90 },
    { 0x6d7e6e1b, 95 },
    { 0x65fe6e1b, 61 },
    { 0x33ce6e1d, 33 },
    { 0xa7e6e1e, 87 },
    { 0xc81e6e1f, 78 },
    { 0x454e6e1f, 88 },
    { 0x402e6e21, 34 },
    { 0xe65e6e1a, 55 },
    { 0, 0 },
    { 0x39be6e24, 61 },
    { 0x8c8e6e25, 23 },
    { 0x9aee6e26, 95 },
    { 0xceae6e27, 66 },
    { 0x8bce6e28, 38 },
    { 0x83fe6e29, 23 },
    { 0x496e6e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ee6e35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x877e6e38, 43 },
    { 0, 0 },
    { 0xab6e6e3a, 78 },
    { 0, 0 },
    { 0xbfde6e3c, 33 },
    { 0x2e7e6e3d, 90 },
    { 0x201e6e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09e6e41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e6e45, 23 },
    { 0x799e6e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be6e49, 98 },
    { 0x12fe6e49, 33 },
    { 0x9cae6e4b, 43 },
    { 0x4abe6e4c, 80 },
    { 0xeaae6e4d, 33 },
    { 0xe3e6e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ae6e55, 95 },
    { 0x9fe6e55, 75 },
    { 0x86e6e57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e6e5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x288e6e5f, 95 },
    { 0, 0 },
    { 0x73fe6e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e6e6c, 87 },
    { 0x9eae6e6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe6e78, 76 },
    { 0x7a2e6e79, 91 },
    { 0x436e6e79, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88e6e7f, 61 },
    { 0, 0 },
    { 0x9e4e6e81, 98 },
    { 0x209e6e81, 77 },
    { 0x3f3e6e83, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee6e87, 34 },
    { 0x772e6e88, 99 },
    { 0xbc8e6e87, 36 },
    { 0x44de6e8a, 23 },
    { 0x500e6e8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ae6e90, 46 },
    { 0xf02e6e90, 34 },
    { 0x85fe6e92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe6e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae6e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56e6e9d, 33 },
    { 0, 0 },
    { 0x704e6e9f, 99 },
    { 0, 0 },
    { 0xdce6ea1, 33 },
    { 0, 0 },
    { 0xffee6ea3, 23 },
    { 0xf46e6ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85e6ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6eab, 55 },
    { 0, 0 },
    { 0x2e1e6ead, 90 },
    { 0, 0 },
    { 0x8aee6eaf, 43 },
    { 0xcb5e6eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6be6eba, 93 },
    { 0xd16e6ebb, 17 },
    { 0xe0ae6ebc, 61 },
    { 0x1d9e6ebd, 93 },
    { 0x40ee6ebe, 95 },
    { 0, 0 },
    { 0xf85e6ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ae6ec5, 61 },
    { 0xe3de6ec5, 61 },
    { 0xd03e6ec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ee6ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de6ed5, 56 },
    { 0, 0 },
    { 0x420e6ed7, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x799e6eda, 98 },
    { 0, 0 },
    { 0x728e6edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be6edf, 95 },
    { 0xd8e6ee0, 66 },
    { 0, 0 },
    { 0xcc1e6ee2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4e6ee5, 99 },
    { 0, 0 },
    { 0xde8e6ee7, 33 },
    { 0, 0 },
    { 0x742e6ee9, 90 },
    { 0x7fe6eea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e6eed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e6ef4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e6efa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e6eff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263e6f04, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7e6f07, 91 },
    { 0x1f5e6f08, 95 },
    { 0x19de6f08, 99 },
    { 0, 0 },
    { 0x8c7e6f0b, 23 },
    { 0xb09e6f0c, 90 },
    { 0, 0 },
    { 0x83fe6f0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219e6f13, 91 },
    { 0x838e6f14, 95 },
    { 0x67fe6f14, 95 },
    { 0x8dee6f15, 98 },
    { 0xa2be6f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bee6f1c, 74 },
    { 0x1ce6f1c, 99 },
    { 0, 0 },
    { 0x8ae6f1f, 23 },
    { 0, 0 },
    { 0xffde6f21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593e6f26, 97 },
    { 0xa98e6f27, 98 },
    { 0x4d4e6f27, 76 },
    { 0xbb0e6f29, 33 },
    { 0, 0 },
    { 0x193e6f2b, 100 },
    { 0x95de6f2c, 98 },
    { 0x546e6f2d, 85 },
    { 0x996e6f2d, 78 },
    { 0x5e7e6f2f, 23 },
    { 0x3e3e6f30, 23 },
    { 0x155e6f2e, 90 },
    { 0x968e6f32, 84 },
    { 0x4b5e6f32, 76 },
    { 0x1dce6f30, 99 },
    { 0x80ce6f35, 90 },
    { 0xb3ae6f30, 95 },
    { 0xe4be6f37, 36 },
    { 0xae1e6f33, 66 },
    { 0x6cae6f37, 36 },
    { 0x4eee6f3a, 90 },
    { 0x270e6f3b, 98 },
    { 0xc2ae6f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae6f3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e6f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1e6f55, 61 },
    { 0x1aee6f56, 90 },
    { 0, 0 },
    { 0x229e6f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6e6f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de6f69, 90 },
    { 0xed9e6f69, 98 },
    { 0x930e6f6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe6f70, 76 },
    { 0x5b3e6f71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933e6f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fae6f7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e6f7e, 55 },
    { 0, 0 },
    { 0xf93e6f80, 95 },
    { 0x365e6f81, 95 },
    { 0x77be6f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e6f86, 93 },
    { 0xd26e6f87, 69 },
    { 0xd71e6f87, 99 },
    { 0x83e6f88, 34 },
    { 0x1dfe6f8a, 77 },
    { 0, 0 },
    { 0xc45e6f8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9e6f92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae6f95, 61 },
    { 0, 0 },
    { 0xfd6e6f97, 61 },
    { 0xdaae6f98, 23 },
    { 0x6de6f98, 33 },
    { 0, 0 },
    { 0xf8be6f9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e6fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe6fa6, 34 },
    { 0, 0 },
    { 0xebde6fa8, 38 },
    { 0xa7ae6fa9, 87 },
    { 0xef9e6faa, 95 },
    { 0x14fe6fa9, 93 },
    { 0xfee6faa, 100 },
    { 0x98be6faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce6fb1, 99 },
    { 0x911e6fb2, 98 },
    { 0xfcee6fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5e6fb9, 90 },
    { 0xfabe6fb9, 84 },
    { 0, 0 },
    { 0x8c8e6fbc, 90 },
    { 0, 0 },
    { 0xebbe6fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e6fc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9e6fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0e6fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e6fd4, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9e6fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ee6fda, 61 },
    { 0x1dde6fdb, 34 },
    { 0xfe6fdc, 90 },
    { 0, 0 },
    { 0x8dae6fde, 33 },
    { 0, 0 },
    { 0x23ee6fe0, 88 },
    { 0x62ee6fe0, 90 },
    { 0, 0 },
    { 0x749e6fe3, 75 },
    { 0, 0 },
    { 0x864e6fe5, 85 },
    { 0x79ce6fe6, 99 },
    { 0x64ee6fe7, 98 },
    { 0, 0 },
    { 0xb1fe6fe9, 61 },
    { 0x839e6fea, 76 },
    { 0x609e6feb, 33 },
    { 0, 0 },
    { 0x518e6fed, 61 },
    { 0xde2e6fee, 34 },
    { 0xb70e6fee, 46 },
    { 0x92ce6fef, 66 },
    { 0xa94e6ff1, 55 },
    { 0x2c4e6ff1, 98 },
    { 0x284e6ff3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20e6ffa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x166e6ffd, 84 },
    { 0, 0 },
    { 0xb97e6fff, 95 },
    { 0xade6fff, 33 },
    { 0x5ade7001, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade7005, 90 },
    { 0x19ae7005, 95 },
    { 0xb14e7007, 55 },
    { 0xeade7008, 93 },
    { 0xa0be7009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8e700f, 33 },
    { 0, 0 },
    { 0x619e7011, 43 },
    { 0xb78e7011, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ae7017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64e701f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76e7022, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94e7027, 100 },
    { 0x3dbe7028, 78 },
    { 0xe90e7029, 99 },
    { 0, 0 },
    { 0xd63e702b, 61 },
    { 0, 0 },
    { 0x7efe702d, 90 },
    { 0xf97e702e, 90 },
    { 0, 0 },
    { 0x3b7e7030, 33 },
    { 0x8d6e7030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e7034, 99 },
    { 0, 0 },
    { 0x671e7036, 88 },
    { 0xd49e7036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe703a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ee703d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfee7045, 53 },
    { 0, 0 },
    { 0xfa1e7047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be704b, 90 },
    { 0x3f8e704b, 85 },
    { 0x2b9e704d, 52 },
    { 0x4cae704c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80e7052, 46 },
    { 0x79fe7053, 87 },
    { 0x7b0e7054, 23 },
    { 0xa2fe7053, 84 },
    { 0x6ae7056, 33 },
    { 0x8bbe7053, 23 },
    { 0xae3e7058, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74e705f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310e7067, 98 },
    { 0xb1fe7067, 40 },
    { 0xb62e7069, 43 },
    { 0, 0 },
    { 0xd63e706b, 95 },
    { 0x1aee706c, 98 },
    { 0xdc6e706d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ce7072, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93e7076, 46 },
    { 0, 0 },
    { 0x79fe7078, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbe707e, 98 },
    { 0x8cfe707f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545e7089, 23 },
    { 0, 0 },
    { 0xa3ee708b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de708e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e7093, 46 },
    { 0xc4de7093, 85 },
    { 0, 0 },
    { 0x849e7096, 91 },
    { 0x830e7096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade709b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e709e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748e70ae, 99 },
    { 0xbcbe70ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de70b4, 90 },
    { 0x300e70b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2e70b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x290e70bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571e70bf, 43 },
    { 0x5d4e70c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee70c8, 23 },
    { 0xa8ce70c9, 34 },
    { 0x370e70ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3e70cd, 66 },
    { 0, 0 },
    { 0xf48e70cf, 76 },
    { 0, 0 },
    { 0xebae70d1, 61 },
    { 0x456e70d1, 84 },
    { 0, 0 },
    { 0x15ae70d4, 34 },
    { 0, 0 },
    { 0x88e70d6, 83 },
    { 0, 0 },
    { 0xcebe70d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7e70db, 76 },
    { 0x20e70dc, 84 },
    { 0xd4ae70dd, 90 },
    { 0, 0 },
    { 0xc0fe70df, 43 },
    { 0x6aae70df, 33 },
    { 0, 0 },
    { 0x34de70e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x176e70e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47e70e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ace70eb, 90 },
    { 0xe01e70ec, 91 },
    { 0x6aee70ed, 78 },
    { 0x540e70ee, 76 },
    { 0x35fe70eb, 55 },
    { 0x85be70f0, 85 },
    { 0x75de70f1, 90 },
    { 0, 0 },
    { 0x28de70f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee70f7, 61 },
    { 0x893e70f7, 95 },
    { 0, 0 },
    { 0xdb8e70fa, 95 },
    { 0x8afe70fb, 93 },
    { 0x44e70fc, 55 },
    { 0, 0 },
    { 0xbfee70fe, 46 },
    { 0xf36e70fe, 90 },
    { 0x37be70ff, 98 },
    { 0x8dfe7101, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e7106, 95 },
    { 0xc8ce7106, 55 },
    { 0x124e7107, 23 },
    { 0x30ee7109, 61 },
    { 0x971e7108, 61 },
    { 0xc0de710b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e710f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87e7114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04e7118, 93 },
    { 0x637e7118, 90 },
    { 0x6a5e711a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e7120, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbe7124, 90 },
    { 0, 0 },
    { 0xfb2e7126, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e712a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e7132, 76 },
    { 0xafbe7133, 95 },
    { 0x651e7134, 33 },
    { 0xad2e7135, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2e7139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae713c, 90 },
    { 0x3c5e713d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7e7141, 95 },
    { 0x8c1e7142, 33 },
    { 0x444e7143, 61 },
    { 0x90de7144, 33 },
    { 0x76e7145, 99 },
    { 0xb39e7146, 61 },
    { 0xdae7147, 61 },
    { 0x2d8e7148, 46 },
    { 0xa4e7149, 33 },
    { 0xb2ee714a, 90 },
    { 0xf36e714b, 23 },
    { 0xae5e714c, 90 },
    { 0xa2be714b, 90 },
    { 0x2a2e714e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abe7151, 23 },
    { 0x4aee7151, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70e7155, 23 },
    { 0, 0 },
    { 0xd74e7157, 38 },
    { 0x976e7158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4e715c, 55 },
    { 0, 0 },
    { 0x50ce715e, 90 },
    { 0xed5e715f, 76 },
    { 0x11ce7160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e716d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e7170, 61 },
    { 0xe6de7171, 33 },
    { 0xd2fe7170, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e7175, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e7178, 76 },
    { 0, 0 },
    { 0x21e717a, 87 },
    { 0x682e717a, 23 },
    { 0xea0e717c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x543e717f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0e7188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e718c, 90 },
    { 0, 0 },
    { 0xb89e718e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e7193, 61 },
    { 0x27be7194, 33 },
    { 0xb1e7194, 90 },
    { 0x4cfe7196, 67 },
    { 0, 0 },
    { 0xc18e7198, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841e719c, 46 },
    { 0, 0 },
    { 0x52ce719e, 23 },
    { 0x8b6e719f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61be71a3, 93 },
    { 0, 0 },
    { 0x275e71a5, 4 },
    { 0, 0 },
    { 0x6a0e71a7, 61 },
    { 0, 0 },
    { 0x2e5e71a9, 91 },
    { 0, 0 },
    { 0x570e71ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e71b4, 61 },
    { 0, 0 },
    { 0xee5e71b6, 76 },
    { 0x90e71b6, 91 },
    { 0xe99e71b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce71bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86e71ce, 34 },
    { 0, 0 },
    { 0x68e71d0, 90 },
    { 0x43fe71d1, 95 },
    { 0xfa5e71d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9e71d9, 87 },
    { 0, 0 },
    { 0x5c2e71db, 87 },
    { 0, 0 },
    { 0xd08e71dd, 85 },
    { 0xc4de71de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ee71e1, 61 },
    { 0xb5be71e2, 61 },
    { 0, 0 },
    { 0x370e71e4, 77 },
    { 0x63de71e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce71e9, 76 },
    { 0x14de71ea, 33 },
    { 0xbc9e71eb, 77 },
    { 0xc69e71eb, 23 },
    { 0x34be71ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x283e71f0, 78 },
    { 0xd3ae71f1, 98 },
    { 0xfbce71f2, 90 },
    { 0x646e71f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51e71f8, 90 },
    { 0xc8be71f9, 33 },
    { 0xdbee71f8, 90 },
    { 0x427e71fb, 76 },
    { 0, 0 },
    { 0xff9e71fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee7200, 90 },
    { 0, 0 },
    { 0xc99e7202, 95 },
    { 0xe2ae7203, 95 },
    { 0x97be7203, 95 },
    { 0x624e7203, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628e720b, 91 },
    { 0x965e720b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3e7213, 33 },
    { 0, 0 },
    { 0xe78e7215, 76 },
    { 0xcefe7216, 34 },
    { 0x1dde7217, 100 },
    { 0, 0 },
    { 0x827e7219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99e721c, 95 },
    { 0, 0 },
    { 0xe9be721e, 84 },
    { 0, 0 },
    { 0x1dee7220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74e722c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x475e722f, 98 },
    { 0, 0 },
    { 0x4b2e7231, 66 },
    { 0x505e7231, 33 },
    { 0, 0 },
    { 0x515e7234, 90 },
    { 0xa79e7234, 99 },
    { 0x1fe7236, 55 },
    { 0xa05e7237, 98 },
    { 0x5aae7238, 61 },
    { 0x98ce7236, 98 },
    { 0x6cee723a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee7240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e7244, 77 },
    { 0x853e7244, 90 },
    { 0, 0 },
    { 0x57ae7247, 33 },
    { 0, 0 },
    { 0xf26e7249, 90 },
    { 0, 0 },
    { 0xc09e724b, 33 },
    { 0x1e9e724c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2be7258, 38 },
    { 0, 0 },
    { 0xe51e725a, 55 },
    { 0x829e725a, 76 },
    { 0x3ece725c, 90 },
    { 0x2fbe725d, 51 },
    { 0, 0 },
    { 0x3ffe725f, 91 },
    { 0, 0 },
    { 0xbace7261, 98 },
    { 0, 0 },
    { 0x688e7263, 95 },
    { 0x37de7263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e7268, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e7272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32de7276, 43 },
    { 0, 0 },
    { 0xabbe7278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce727d, 93 },
    { 0, 0 },
    { 0xd41e727f, 90 },
    { 0xec8e727f, 61 },
    { 0xad0e7281, 23 },
    { 0x4bbe7282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e728c, 35 },
    { 0x296e728d, 97 },
    { 0xf4ee728c, 95 },
    { 0xda0e728f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e7292, 90 },
    { 0, 0 },
    { 0x183e7294, 33 },
    { 0x55be7294, 33 },
    { 0x462e7295, 23 },
    { 0xa09e7297, 43 },
    { 0xe70e7295, 34 },
    { 0x705e7299, 33 },
    { 0xf60e7294, 76 },
    { 0x83ee7294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e72a4, 90 },
    { 0xcd2e72a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde72ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41be72b5, 56 },
    { 0xf14e72b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fae72b9, 52 },
    { 0x5f8e72ba, 43 },
    { 0, 0 },
    { 0xd56e72bc, 77 },
    { 0, 0 },
    { 0x958e72be, 33 },
    { 0, 0 },
    { 0x75e72c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e72c7, 90 },
    { 0x7a0e72c8, 33 },
    { 0x468e72c7, 87 },
    { 0xdce72ca, 34 },
    { 0xb20e72cb, 33 },
    { 0xfe4e72cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e72d2, 88 },
    { 0, 0 },
    { 0x847e72d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38e72da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e72dd, 85 },
    { 0x4dde72de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae72e3, 61 },
    { 0, 0 },
    { 0x340e72e5, 95 },
    { 0, 0 },
    { 0x7dae72e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e72ec, 66 },
    { 0, 0 },
    { 0xb8ce72ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e72f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ae72fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2be7302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e7308, 90 },
    { 0, 0 },
    { 0x1a1e730a, 33 },
    { 0xf58e730a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4e7310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13e7314, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e7317, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cee731a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e731d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229e7321, 34 },
    { 0xb63e7321, 33 },
    { 0x595e7323, 90 },
    { 0xd6de7324, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258e7328, 46 },
    { 0, 0 },
    { 0x52be732a, 69 },
    { 0, 0 },
    { 0xdc6e732c, 95 },
    { 0x2b1e732c, 90 },
    { 0xae6e732d, 95 },
    { 0x128e732f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe7335, 98 },
    { 0x854e7336, 33 },
    { 0, 0 },
    { 0x261e7338, 78 },
    { 0, 0 },
    { 0x1dee733a, 98 },
    { 0x9c1e733b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35be7341, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e7344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e7347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ee7350, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7353, 76 },
    { 0xdf6e7354, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894e735b, 23 },
    { 0xe5ee735c, 90 },
    { 0x7fce735d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825e7369, 98 },
    { 0, 0 },
    { 0xb1ee736b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5e736f, 61 },
    { 0x6fce736f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ee7377, 23 },
    { 0xb7ce7377, 76 },
    { 0xc0e7378, 90 },
    { 0, 0 },
    { 0x1d3e737b, 93 },
    { 0xf9fe737c, 61 },
    { 0x26de737d, 76 },
    { 0, 0 },
    { 0x3d8e737f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae7382, 55 },
    { 0x674e7383, 48 },
    { 0, 0 },
    { 0x2cee7385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e738a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0e738d, 61 },
    { 0x5dce738e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfe7394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e739b, 23 },
    { 0xf77e739c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce73a0, 99 },
    { 0xd50e73a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce73a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e73b0, 43 },
    { 0, 0 },
    { 0x7dae73b2, 55 },
    { 0xa3ee73b2, 42 },
    { 0x4c2e73b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3de73bc, 33 },
    { 0x485e73bd, 61 },
    { 0xb95e73be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e73c1, 23 },
    { 0, 0 },
    { 0xf8be73c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43e73c7, 55 },
    { 0xc29e73c8, 94 },
    { 0x427e73c9, 78 },
    { 0x9d0e73ca, 95 },
    { 0, 0 },
    { 0xb7de73cc, 76 },
    { 0x564e73cd, 23 },
    { 0x872e73ce, 38 },
    { 0, 0 },
    { 0xa16e73d0, 61 },
    { 0xfbee73d1, 98 },
    { 0x9cae73d2, 34 },
    { 0x7b6e73d1, 90 },
    { 0x3e7e73d4, 33 },
    { 0xf05e73d4, 98 },
    { 0x231e73d4, 36 },
    { 0xbbbe73d6, 36 },
    { 0x65e73d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90be73db, 98 },
    { 0x3b1e73dc, 95 },
    { 0x410e73dd, 59 },
    { 0x24ae73de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e73e3, 88 },
    { 0, 0 },
    { 0xb0ce73e5, 98 },
    { 0x2e6e73e6, 90 },
    { 0xeb5e73e7, 81 },
    { 0x7b4e73e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26e73ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e73f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9e73f8, 85 },
    { 0xc9ae73f8, 98 },
    { 0x8de73fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ae73fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e7408, 90 },
    { 0xdeae7409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e740d, 33 },
    { 0xe23e740e, 55 },
    { 0xb02e740f, 90 },
    { 0, 0 },
    { 0xfcfe7411, 36 },
    { 0x8ebe7411, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbde741d, 77 },
    { 0xb3de741d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e7422, 23 },
    { 0x18be7422, 43 },
    { 0x2d2e7422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe7429, 90 },
    { 0, 0 },
    { 0x587e742b, 33 },
    { 0, 0 },
    { 0xbfee742d, 91 },
    { 0x987e742e, 98 },
    { 0x2e7e742e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe7433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7be7436, 61 },
    { 0x63e7437, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7e743a, 95 },
    { 0xcfee743b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aee7441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52e7444, 43 },
    { 0x1f5e7445, 23 },
    { 0x8bee7446, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e744a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5e7450, 65 },
    { 0x45ae7451, 43 },
    { 0x94e7451, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bae7457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de745d, 33 },
    { 0, 0 },
    { 0x178e745f, 95 },
    { 0xe32e745f, 91 },
    { 0xbc7e7461, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ce7466, 67 },
    { 0, 0 },
    { 0x9a2e7468, 61 },
    { 0x45ce7468, 95 },
    { 0x99e7468, 77 },
    { 0x4d3e7469, 90 },
    { 0x78ee746c, 91 },
    { 0x73fe746d, 46 },
    { 0xf70e746d, 33 },
    { 0xfcfe746e, 76 },
    { 0xcdfe746f, 66 },
    { 0x99de746e, 66 },
    { 0x919e746e, 61 },
    { 0, 0 },
    { 0x38ae7474, 23 },
    { 0x368e7475, 90 },
    { 0xeb6e7476, 98 },
    { 0x2cce7476, 100 },
    { 0xa6be7477, 98 },
    { 0xa43e7478, 95 },
    { 0xb0ce747a, 91 },
    { 0, 0 },
    { 0x1a4e747c, 96 },
    { 0, 0 },
    { 0xe08e747e, 34 },
    { 0xf43e747f, 67 },
    { 0xe6ae7480, 24 },
    { 0, 0 },
    { 0xef5e7482, 61 },
    { 0x5cfe7483, 27 },
    { 0x7e7e7484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e7489, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81e748d, 90 },
    { 0xab1e748e, 95 },
    { 0x172e748f, 85 },
    { 0x887e7490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48e7497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e749b, 90 },
    { 0, 0 },
    { 0xacee749d, 94 },
    { 0xa3ee749d, 33 },
    { 0x4b4e749f, 91 },
    { 0x3ae74a0, 92 },
    { 0, 0 },
    { 0xf6e74a2, 93 },
    { 0xeaee74a3, 95 },
    { 0x1b9e74a3, 90 },
    { 0x901e74a5, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e74aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e74ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e74b1, 100 },
    { 0x2ebe74b2, 90 },
    { 0, 0 },
    { 0xef4e74b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e74b7, 33 },
    { 0, 0 },
    { 0x624e74b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32e74bc, 61 },
    { 0, 0 },
    { 0xa0e74be, 90 },
    { 0x179e74bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce74c5, 95 },
    { 0x654e74c6, 23 },
    { 0, 0 },
    { 0x802e74c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e74cf, 61 },
    { 0x925e74d0, 88 },
    { 0x5fbe74d1, 90 },
    { 0xfc3e74d0, 95 },
    { 0xb2de74d2, 90 },
    { 0xf6be74d3, 95 },
    { 0x8e8e74d5, 61 },
    { 0x74ae74d0, 98 },
    { 0x186e74d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee74da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4e74e0, 66 },
    { 0x30e74e0, 85 },
    { 0, 0 },
    { 0xcb9e74e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499e74e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ae74ed, 55 },
    { 0x9b5e74ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce74f1, 95 },
    { 0, 0 },
    { 0x6a0e74f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943e74f6, 95 },
    { 0x563e74f7, 46 },
    { 0x43ce74f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717e74fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ee7502, 67 },
    { 0x88ae7502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7e7507, 90 },
    { 0, 0 },
    { 0xe67e7509, 95 },
    { 0xe85e7509, 90 },
    { 0x4a8e750b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de750f, 46 },
    { 0x266e7510, 33 },
    { 0x1cce7510, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e7515, 98 },
    { 0x8e4e7516, 90 },
    { 0, 0 },
    { 0x3bee7518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be7520, 93 },
    { 0, 0 },
    { 0x4a9e7522, 61 },
    { 0, 0 },
    { 0x668e7524, 76 },
    { 0x3dfe7525, 23 },
    { 0x9bbe7526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3e752a, 23 },
    { 0x90ce752b, 23 },
    { 0, 0 },
    { 0x860e752d, 78 },
    { 0xf9de752e, 95 },
    { 0xda6e752f, 61 },
    { 0xe5ce7530, 33 },
    { 0xb48e7530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e7539, 95 },
    { 0x250e753a, 34 },
    { 0x4dee753a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce753e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe7541, 98 },
    { 0, 0 },
    { 0xa4be7543, 61 },
    { 0xbc1e7544, 32 },
    { 0, 0 },
    { 0x659e7546, 34 },
    { 0xddde7546, 33 },
    { 0xb15e7547, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e754e, 84 },
    { 0x345e754f, 84 },
    { 0, 0 },
    { 0xe6ce7551, 33 },
    { 0x55ce7552, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9e7558, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7e755b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08e7561, 61 },
    { 0x725e7561, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee756c, 43 },
    { 0, 0 },
    { 0xef9e756e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e7576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e757a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98e757d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1e7582, 87 },
    { 0x284e7583, 87 },
    { 0xb69e7584, 52 },
    { 0xa85e7583, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938e758e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e7592, 23 },
    { 0x1f2e7592, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e7596, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e759c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x370e759f, 100 },
    { 0, 0 },
    { 0x7f4e75a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e75a7, 61 },
    { 0, 0 },
    { 0x7efe75a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fe75b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e75b4, 33 },
    { 0x598e75b5, 78 },
    { 0x78be75b6, 61 },
    { 0x801e75b6, 34 },
    { 0xda3e75b8, 95 },
    { 0xdf6e75b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde75c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be75c3, 23 },
    { 0x70ae75c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6de75c8, 53 },
    { 0, 0 },
    { 0x455e75ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee75ce, 87 },
    { 0, 0 },
    { 0x7d4e75d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae75d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e75d7, 93 },
    { 0x830e75d8, 95 },
    { 0, 0 },
    { 0xd40e75da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24e75de, 90 },
    { 0xaace75df, 23 },
    { 0x185e75de, 76 },
    { 0x505e75df, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81e75e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909e75ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ee75f2, 81 },
    { 0, 0 },
    { 0x1e5e75f4, 33 },
    { 0xd2de75f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e75f8, 33 },
    { 0x589e75f9, 44 },
    { 0xb8ee75fa, 53 },
    { 0, 0 },
    { 0x4c4e75fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bce7600, 46 },
    { 0x18de7600, 61 },
    { 0xb10e7602, 33 },
    { 0x43be7602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe7614, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce7619, 23 },
    { 0, 0 },
    { 0x8a4e761b, 52 },
    { 0x98ce761c, 84 },
    { 0xca2e761c, 61 },
    { 0xf45e761c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979e7623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e7630, 55 },
    { 0, 0 },
    { 0xcece7632, 66 },
    { 0x9f8e7633, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x202e7636, 46 },
    { 0x6dde7637, 52 },
    { 0xdc9e7638, 98 },
    { 0xe7639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee763c, 61 },
    { 0xcfae763d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e7641, 24 },
    { 0x2cde7642, 33 },
    { 0, 0 },
    { 0x352e7644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae7648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87e764e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ede7652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ee7655, 23 },
    { 0x9bde7656, 34 },
    { 0x107e7655, 33 },
    { 0x58ce7658, 55 },
    { 0x9ae7658, 61 },
    { 0x6d3e7657, 76 },
    { 0xba8e765b, 95 },
    { 0xb77e765c, 90 },
    { 0, 0 },
    { 0xf83e765e, 98 },
    { 0xdde765e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416e7664, 90 },
    { 0xa4ce7664, 90 },
    { 0x39ae7665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9e766a, 43 },
    { 0, 0 },
    { 0x5aae766c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccae7672, 33 },
    { 0xe50e7673, 61 },
    { 0xb67e7673, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04e7677, 33 },
    { 0xd64e7677, 95 },
    { 0x784e7679, 76 },
    { 0xd0fe767a, 61 },
    { 0xa31e767a, 11 },
    { 0x7aee767c, 66 },
    { 0x475e767d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2de7682, 61 },
    { 0, 0 },
    { 0xd65e7684, 33 },
    { 0x5ce7685, 65 },
    { 0xa35e7684, 87 },
    { 0x792e7687, 33 },
    { 0, 0 },
    { 0x1abe7689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e768e, 100 },
    { 0xa62e768e, 91 },
    { 0, 0 },
    { 0xd40e7691, 95 },
    { 0, 0 },
    { 0x19fe7693, 76 },
    { 0, 0 },
    { 0x1abe7695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eee769c, 76 },
    { 0x580e769c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e76a2, 52 },
    { 0x56de76a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530e76aa, 61 },
    { 0xf79e76aa, 76 },
    { 0xb9ae76aa, 33 },
    { 0, 0 },
    { 0xea5e76ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e76b4, 76 },
    { 0x8ffe76b5, 95 },
    { 0x373e76b6, 55 },
    { 0x922e76b6, 84 },
    { 0xd3ae76b5, 33 },
    { 0xa99e76b6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ae76c7, 98 },
    { 0, 0 },
    { 0x66fe76c9, 33 },
    { 0, 0 },
    { 0x460e76cb, 83 },
    { 0xeefe76cc, 93 },
    { 0x813e76cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1de76d0, 23 },
    { 0x258e76d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x738e76d4, 17 },
    { 0x552e76d4, 23 },
    { 0x321e76d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8e76db, 61 },
    { 0xa60e76dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e76e2, 61 },
    { 0, 0 },
    { 0x7dce76e4, 61 },
    { 0xd60e76e5, 33 },
    { 0, 0 },
    { 0x369e76e7, 47 },
    { 0, 0 },
    { 0x4c2e76e9, 95 },
    { 0, 0 },
    { 0xee9e76eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282e76f0, 88 },
    { 0x762e76f1, 90 },
    { 0x8bbe76f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de76f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6e76fe, 95 },
    { 0, 0 },
    { 0x1a4e7700, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e7703, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7706, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1e7709, 61 },
    { 0xa13e770a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe770d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76e7713, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42e7723, 77 },
    { 0x412e7723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7728, 61 },
    { 0, 0 },
    { 0xc1ee772a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4e772f, 17 },
    { 0x31e772f, 94 },
    { 0xd83e7731, 23 },
    { 0x7cce772f, 78 },
    { 0xd53e7730, 87 },
    { 0, 0 },
    { 0x9d4e7735, 78 },
    { 0x54ee7736, 61 },
    { 0x2c1e7737, 100 },
    { 0, 0 },
    { 0x95ee7739, 61 },
    { 0, 0 },
    { 0xb7ce773b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6e773e, 98 },
    { 0xf50e773e, 46 },
    { 0xc0ae7740, 98 },
    { 0xee7e7740, 33 },
    { 0xac4e773f, 61 },
    { 0xc74e7740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e7746, 61 },
    { 0x664e7747, 61 },
    { 0x21ae7748, 90 },
    { 0x97e7749, 76 },
    { 0xfd4e7749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7e774d, 85 },
    { 0xa3be774d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e775a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ee775d, 17 },
    { 0xa31e775d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245e7764, 40 },
    { 0x627e7765, 90 },
    { 0x4c4e7766, 69 },
    { 0xebe7765, 93 },
    { 0x49fe7764, 33 },
    { 0x7aae7769, 98 },
    { 0x423e7765, 98 },
    { 0, 0 },
    { 0x8d0e776c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1de7770, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314e7775, 98 },
    { 0xa1be7776, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312e777b, 87 },
    { 0x887e777c, 33 },
    { 0xc91e777d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853e7781, 33 },
    { 0, 0 },
    { 0x7c3e7783, 93 },
    { 0x5e5e7784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8e7787, 36 },
    { 0x762e7788, 55 },
    { 0xbb5e7789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1e778d, 23 },
    { 0, 0 },
    { 0xeb2e778f, 33 },
    { 0xf52e7790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971e7797, 43 },
    { 0x113e7797, 87 },
    { 0xffde7799, 61 },
    { 0, 0 },
    { 0xa15e779b, 87 },
    { 0x24fe779b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ce77a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bae77a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22e77a9, 23 },
    { 0x53ee77aa, 61 },
    { 0xc2e77aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e77af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74de77b2, 91 },
    { 0xc2de77b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24e77b8, 23 },
    { 0x9c5e77b8, 84 },
    { 0, 0 },
    { 0xaa1e77bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3e77bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e77c6, 33 },
    { 0xb62e77c6, 76 },
    { 0, 0 },
    { 0x993e77c9, 33 },
    { 0x457e77ca, 87 },
    { 0xa32e77cb, 61 },
    { 0x8c7e77ca, 90 },
    { 0, 0 },
    { 0x5efe77ce, 98 },
    { 0x5b5e77cf, 98 },
    { 0xdd7e77d0, 46 },
    { 0, 0 },
    { 0xf69e77d2, 95 },
    { 0, 0 },
    { 0x309e77d4, 88 },
    { 0xa1e77d5, 78 },
    { 0, 0 },
    { 0x226e77d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e77da, 98 },
    { 0, 0 },
    { 0xa9ce77dc, 95 },
    { 0, 0 },
    { 0x67ce77de, 90 },
    { 0xce4e77df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1e77e2, 19 },
    { 0, 0 },
    { 0xe2e77e4, 55 },
    { 0x99ae77e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4e77ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce77f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de77fb, 33 },
    { 0xecfe77fc, 91 },
    { 0x6d5e77fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2e7804, 93 },
    { 0xc03e7805, 81 },
    { 0xeb9e7806, 93 },
    { 0x296e7807, 23 },
    { 0xda5e7808, 81 },
    { 0x757e7804, 99 },
    { 0xc8ce7808, 23 },
    { 0, 0 },
    { 0xb6ee780c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04e7814, 36 },
    { 0x792e7815, 33 },
    { 0xc11e7815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e781d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884e7822, 76 },
    { 0x7fae7823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfe7829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efe782c, 23 },
    { 0, 0 },
    { 0xa00e782e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e7838, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2e7841, 92 },
    { 0x20e7841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e7847, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29e784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3de7856, 98 },
    { 0x966e7857, 93 },
    { 0xfdde7858, 95 },
    { 0xed2e7858, 33 },
    { 0xadce7858, 33 },
    { 0x10ae785b, 66 },
    { 0xc17e7859, 66 },
    { 0x603e785d, 38 },
    { 0x39ee785e, 88 },
    { 0x41be785e, 33 },
    { 0x86fe7860, 61 },
    { 0x6a0e7861, 93 },
    { 0x313e7858, 34 },
    { 0x7fce7863, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e7866, 90 },
    { 0xfd6e7866, 36 },
    { 0xef1e7868, 87 },
    { 0xd1e7869, 76 },
    { 0x30ce7868, 61 },
    { 0xf20e786b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe786e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2e7878, 55 },
    { 0xd52e7879, 75 },
    { 0x19fe787a, 99 },
    { 0x1bce787b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1e787f, 78 },
    { 0x812e7880, 34 },
    { 0xbe7881, 33 },
    { 0xdc7e7882, 33 },
    { 0x91ee7883, 95 },
    { 0xf53e7884, 23 },
    { 0x6f9e7885, 93 },
    { 0x2b0e7886, 100 },
    { 0x2a0e7887, 95 },
    { 0x869e7883, 23 },
    { 0x2fae7880, 95 },
    { 0xfde7883, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e788e, 95 },
    { 0x548e788f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e7894, 90 },
    { 0, 0 },
    { 0x971e7896, 76 },
    { 0x61ce7897, 87 },
    { 0, 0 },
    { 0x8cbe7899, 43 },
    { 0xdabe789a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e78a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e78a9, 33 },
    { 0x62ce78aa, 23 },
    { 0x240e78ab, 48 },
    { 0xf84e78ac, 90 },
    { 0, 0 },
    { 0x6f6e78ae, 95 },
    { 0, 0 },
    { 0x15ae78b0, 23 },
    { 0xac9e78b1, 98 },
    { 0, 0 },
    { 0x99ce78b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e78b7, 67 },
    { 0, 0 },
    { 0x81ae78b9, 90 },
    { 0, 0 },
    { 0xab9e78bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce78c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e78c3, 43 },
    { 0x334e78c4, 33 },
    { 0x65ae78c5, 90 },
    { 0x875e78c3, 33 },
    { 0, 0 },
    { 0x9ede78c8, 46 },
    { 0x2efe78c8, 61 },
    { 0x699e78c9, 34 },
    { 0x2d8e78c9, 99 },
    { 0xab6e78ca, 76 },
    { 0xf88e78cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e78d2, 95 },
    { 0, 0 },
    { 0xdf3e78d4, 33 },
    { 0, 0 },
    { 0xdffe78d6, 46 },
    { 0x399e78d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949e78df, 87 },
    { 0, 0 },
    { 0x541e78e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e78e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe78ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69e78f3, 46 },
    { 0xf6e78f4, 99 },
    { 0x4b9e78f5, 43 },
    { 0xfbee78f5, 34 },
    { 0xa71e78f7, 33 },
    { 0x26ce78f6, 61 },
    { 0, 0 },
    { 0x8fbe78fa, 33 },
    { 0xdc9e78fa, 61 },
    { 0x903e78fc, 95 },
    { 0x603e78fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e7902, 76 },
    { 0x323e7903, 65 },
    { 0x729e7902, 100 },
    { 0xf49e7905, 23 },
    { 0, 0 },
    { 0x2a8e7907, 95 },
    { 0, 0 },
    { 0xc54e7909, 90 },
    { 0x2a9e790a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9be790d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2e7912, 33 },
    { 0x7b5e7913, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49e7917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e791c, 61 },
    { 0x413e791d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88e7920, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6de7926, 43 },
    { 0xb3fe7927, 33 },
    { 0xabae7927, 34 },
    { 0x8c2e7929, 33 },
    { 0x652e792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819e7931, 84 },
    { 0, 0 },
    { 0x76ee7933, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cae7936, 76 },
    { 0xf0be7937, 61 },
    { 0x4b0e7937, 68 },
    { 0x6e5e7937, 66 },
    { 0x44fe7938, 80 },
    { 0x42ce7936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8e7942, 76 },
    { 0xdfde7942, 43 },
    { 0, 0 },
    { 0xd19e7945, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe7948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87e794d, 99 },
    { 0xe69e794e, 66 },
    { 0x89ee794f, 55 },
    { 0xbcde794e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae7953, 66 },
    { 0x584e7953, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e7957, 98 },
    { 0, 0 },
    { 0x9e9e7959, 95 },
    { 0, 0 },
    { 0x96de795b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5e795e, 61 },
    { 0x4dbe795f, 98 },
    { 0, 0 },
    { 0xa73e7961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37be7967, 76 },
    { 0xf27e7968, 33 },
    { 0x131e7967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7974, 17 },
    { 0, 0 },
    { 0xa64e7976, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670e797a, 34 },
    { 0, 0 },
    { 0x830e797c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedae797f, 55 },
    { 0xdace797f, 95 },
    { 0x76ce797f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8e7985, 77 },
    { 0, 0 },
    { 0xa6fe7987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97e798f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e7992, 90 },
    { 0x978e7993, 76 },
    { 0x8efe7992, 90 },
    { 0x2d8e7995, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ae799c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbe79a2, 78 },
    { 0x94ee79a2, 55 },
    { 0x460e79a3, 23 },
    { 0x798e79a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2e79a9, 23 },
    { 0x29be79aa, 84 },
    { 0, 0 },
    { 0xcd2e79ac, 61 },
    { 0xb1ce79ad, 61 },
    { 0x5c9e79ae, 33 },
    { 0xb13e79af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde79b3, 55 },
    { 0, 0 },
    { 0xfe0e79b5, 23 },
    { 0, 0 },
    { 0x1ce79b7, 95 },
    { 0xf36e79b7, 91 },
    { 0, 0 },
    { 0xda0e79ba, 55 },
    { 0, 0 },
    { 0xb01e79bc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee79bf, 95 },
    { 0, 0 },
    { 0x14e79c1, 91 },
    { 0x4bee79c2, 90 },
    { 0x182e79c2, 83 },
    { 0, 0 },
    { 0xce9e79c5, 95 },
    { 0x821e79c6, 90 },
    { 0x6cee79c7, 100 },
    { 0xd4be79c8, 40 },
    { 0xc8ee79c8, 87 },
    { 0xe9de79ca, 98 },
    { 0xe40e79cb, 98 },
    { 0x7a7e79cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e79d4, 77 },
    { 0x9e9e79d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ee79da, 38 },
    { 0xe49e79db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4e79e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e79e7, 95 },
    { 0, 0 },
    { 0xb59e79e9, 95 },
    { 0x5ade79e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce79f1, 81 },
    { 0x867e79f2, 95 },
    { 0x7c8e79f3, 95 },
    { 0xa83e79f4, 84 },
    { 0xb45e79f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48e79f8, 23 },
    { 0, 0 },
    { 0xc33e79fa, 33 },
    { 0xac6e79fb, 98 },
    { 0, 0 },
    { 0x67ae79fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x974e7a00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e7a03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e7a07, 90 },
    { 0x6a3e7a08, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4e7a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e7a0f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e7a14, 95 },
    { 0x75ae7a15, 95 },
    { 0xa37e7a16, 95 },
    { 0xb45e7a15, 76 },
    { 0x2a4e7a18, 66 },
    { 0x3b8e7a19, 46 },
    { 0x57ae7a18, 75 },
    { 0, 0 },
    { 0x27ee7a1c, 90 },
    { 0x1fbe7a1c, 61 },
    { 0x3ae7a1e, 90 },
    { 0, 0 },
    { 0x401e7a20, 94 },
    { 0xcf6e7a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7e7a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510e7a29, 23 },
    { 0, 0 },
    { 0x62ce7a2b, 86 },
    { 0, 0 },
    { 0xf18e7a2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x64de7a30, 55 },
    { 0x65de7a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20de7a34, 33 },
    { 0xe64e7a34, 77 },
    { 0, 0 },
    { 0xda8e7a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e7a3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31e7a41, 95 },
    { 0x270e7a42, 90 },
    { 0xe2fe7a43, 39 },
    { 0x366e7a43, 61 },
    { 0x71ee7a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965e7a4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e7a57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e7a5b, 84 },
    { 0, 0 },
    { 0x75be7a5d, 100 },
    { 0x510e7a5e, 95 },
    { 0x9fe7a5f, 98 },
    { 0xe86e7a5d, 33 },
    { 0xca3e7a61, 85 },
    { 0x764e7a62, 61 },
    { 0xeb3e7a63, 90 },
    { 0xbcce7a64, 33 },
    { 0x6e1e7a61, 93 },
    { 0xc69e7a66, 76 },
    { 0x121e7a66, 17 },
    { 0xa69e7a65, 69 },
    { 0xfbde7a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e7a6d, 76 },
    { 0, 0 },
    { 0xec8e7a6f, 91 },
    { 0xa78e7a6f, 90 },
    { 0, 0 },
    { 0x551e7a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4e7a75, 87 },
    { 0, 0 },
    { 0xcd7e7a77, 90 },
    { 0x161e7a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199e7a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ae7a87, 46 },
    { 0xe7be7a87, 61 },
    { 0x5eee7a89, 100 },
    { 0x1ce7a8a, 55 },
    { 0x6a1e7a8b, 95 },
    { 0xbe5e7a8c, 76 },
    { 0, 0 },
    { 0x8b1e7a8e, 85 },
    { 0x40ce7a8f, 33 },
    { 0x6cce7a8e, 46 },
    { 0x4e7e7a8f, 66 },
    { 0, 0 },
    { 0x5a5e7a93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fe7a99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e7a9d, 52 },
    { 0xeb7e7a9e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ae7aa1, 91 },
    { 0, 0 },
    { 0xaae7aa3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee7aaa, 61 },
    { 0x803e7aaa, 66 },
    { 0x616e7aac, 61 },
    { 0x443e7aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e7ab2, 55 },
    { 0, 0 },
    { 0x17de7ab4, 99 },
    { 0, 0 },
    { 0xbebe7ab6, 90 },
    { 0x930e7ab7, 61 },
    { 0xef5e7ab8, 33 },
    { 0, 0 },
    { 0xc2ae7aba, 33 },
    { 0x30de7abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece7ac3, 17 },
    { 0, 0 },
    { 0xe18e7ac5, 33 },
    { 0xf84e7ac6, 81 },
    { 0x987e7ac6, 77 },
    { 0x5ffe7ac6, 95 },
    { 0, 0 },
    { 0x146e7aca, 98 },
    { 0, 0 },
    { 0xb9fe7acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce7ad0, 23 },
    { 0xe23e7ad0, 76 },
    { 0xd2be7ad0, 23 },
    { 0xf13e7ad2, 17 },
    { 0x6cde7ad4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7ad7, 50 },
    { 0x68de7ad8, 61 },
    { 0x48ee7ad9, 43 },
    { 0x6f2e7ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1be7adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaae7ae2, 98 },
    { 0x2cce7ae3, 99 },
    { 0x597e7ae4, 61 },
    { 0x224e7ae5, 43 },
    { 0xc9be7ae6, 61 },
    { 0xf06e7ae6, 23 },
    { 0, 0 },
    { 0x16ce7ae9, 98 },
    { 0xfaee7ae9, 34 },
    { 0xf25e7aeb, 33 },
    { 0x38be7ae9, 76 },
    { 0xc58e7aed, 61 },
    { 0xc74e7aed, 98 },
    { 0xf7fe7aeb, 98 },
    { 0x701e7af0, 43 },
    { 0xd4ae7af1, 52 },
    { 0x4c0e7af2, 34 },
    { 0, 0 },
    { 0xb20e7af4, 17 },
    { 0x978e7af5, 33 },
    { 0x5c4e7af6, 90 },
    { 0xef5e7af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee7afd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fe7b0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be7b0d, 49 },
    { 0, 0 },
    { 0xb51e7b0f, 98 },
    { 0x3d9e7b10, 88 },
    { 0x8d3e7b11, 66 },
    { 0x1b3e7b12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae7b17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee7b1c, 23 },
    { 0x273e7b1c, 90 },
    { 0xca2e7b1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769e7b23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e7b29, 95 },
    { 0x349e7b2a, 23 },
    { 0xb39e7b29, 52 },
    { 0x416e7b2c, 88 },
    { 0, 0 },
    { 0x875e7b2e, 61 },
    { 0xe9be7b2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3e7b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee7b37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be7b3a, 23 },
    { 0, 0 },
    { 0x6b0e7b3c, 95 },
    { 0, 0 },
    { 0x301e7b3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709e7b44, 76 },
    { 0xd2ee7b45, 23 },
    { 0, 0 },
    { 0xdf0e7b47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9e7b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x844e7b4e, 95 },
    { 0, 0 },
    { 0xd27e7b50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447e7b56, 76 },
    { 0xe0ae7b57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7e7b5a, 78 },
    { 0xd48e7b5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e7b69, 61 },
    { 0x35fe7b6a, 93 },
    { 0x1f7e7b6b, 90 },
    { 0x33de7b6c, 43 },
    { 0, 0 },
    { 0xa60e7b6e, 66 },
    { 0xf82e7b6f, 90 },
    { 0xd77e7b6e, 43 },
    { 0x54be7b71, 61 },
    { 0x915e7b71, 55 },
    { 0, 0 },
    { 0xf0be7b74, 68 },
    { 0x88ce7b75, 76 },
    { 0x291e7b74, 33 },
    { 0x8ace7b77, 84 },
    { 0, 0 },
    { 0x8e8e7b79, 33 },
    { 0, 0 },
    { 0x2dee7b7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13e7b80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e7b83, 69 },
    { 0x8b6e7b84, 90 },
    { 0, 0 },
    { 0x828e7b86, 90 },
    { 0xfe7b86, 90 },
    { 0, 0 },
    { 0x174e7b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x513e7b8c, 85 },
    { 0xba3e7b8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35e7b91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7b97, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ce7b9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae7b9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e7ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5e7ba9, 61 },
    { 0x25e7baa, 90 },
    { 0x2fbe7bab, 95 },
    { 0x108e7bac, 75 },
    { 0x475e7bac, 61 },
    { 0x61de7bad, 87 },
    { 0xc23e7baf, 76 },
    { 0, 0 },
    { 0xd88e7bb1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e7bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e7bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37de7bbd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcade7bc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815e7bc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde7bd1, 90 },
    { 0, 0 },
    { 0x275e7bd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87e7bd6, 90 },
    { 0, 0 },
    { 0xd46e7bd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae7bdb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e7bde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe7be3, 90 },
    { 0x69ae7be3, 33 },
    { 0x8ebe7be3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e7be9, 98 },
    { 0, 0 },
    { 0xf6ce7beb, 93 },
    { 0, 0 },
    { 0x661e7bed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de7bf3, 33 },
    { 0xc1ae7bf4, 33 },
    { 0xdd4e7bf3, 91 },
    { 0xe7fe7bf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce7bfa, 90 },
    { 0x5eee7bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e7bff, 23 },
    { 0xc98e7c00, 76 },
    { 0xd2ee7c00, 90 },
    { 0, 0 },
    { 0x88ee7c03, 33 },
    { 0x73fe7c03, 33 },
    { 0xc1ce7c05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e7c09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee7c0c, 90 },
    { 0x7f6e7c0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e7c11, 95 },
    { 0x65ce7c12, 19 },
    { 0, 0 },
    { 0xbde7c14, 55 },
    { 0xbb4e7c15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee7c1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161e7c21, 76 },
    { 0x950e7c22, 90 },
    { 0x50fe7c22, 88 },
    { 0x399e7c24, 39 },
    { 0xacbe7c25, 43 },
    { 0x6a8e7c25, 43 },
    { 0x3b9e7c21, 69 },
    { 0xa47e7c28, 61 },
    { 0xc52e7c29, 55 },
    { 0x357e7c2a, 78 },
    { 0xe8fe7c27, 33 },
    { 0xaf2e7c2c, 98 },
    { 0x35e7c2d, 77 },
    { 0x565e7c2e, 94 },
    { 0x2d7e7c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e7c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e7c35, 34 },
    { 0x81be7c36, 100 },
    { 0x766e7c37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e7c3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee7c41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e7c49, 98 },
    { 0xd54e7c49, 55 },
    { 0x79de7c49, 95 },
    { 0x5c5e7c4b, 76 },
    { 0x893e7c4d, 91 },
    { 0, 0 },
    { 0x8d9e7c4f, 61 },
    { 0, 0 },
    { 0xaa5e7c51, 90 },
    { 0x693e7c52, 61 },
    { 0x6c2e7c53, 95 },
    { 0x97fe7c52, 93 },
    { 0x295e7c55, 61 },
    { 0, 0 },
    { 0xf85e7c57, 101 },
    { 0x664e7c57, 23 },
    { 0x6ee7c58, 33 },
    { 0xcb0e7c5a, 98 },
    { 0x7f2e7c57, 46 },
    { 0xbcde7c5c, 97 },
    { 0x13be7c5d, 90 },
    { 0x77e7c58, 90 },
    { 0x92be7c5f, 43 },
    { 0x7d0e7c60, 43 },
    { 0x799e7c59, 85 },
    { 0x3cfe7c62, 23 },
    { 0x939e7c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbe7c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe7c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe7c71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7c74, 56 },
    { 0x439e7c74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660e7c7a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdae7c7e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe7c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbe7c87, 90 },
    { 0x7b4e7c88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ee7c8d, 98 },
    { 0x69ee7c8e, 43 },
    { 0x2abe7c8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08e7c92, 33 },
    { 0, 0 },
    { 0x67ee7c94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x385e7c97, 33 },
    { 0, 0 },
    { 0x287e7c99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7e7c9d, 40 },
    { 0x35e7c9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e7ca5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfae7ca9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24e7cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ae7cb2, 65 },
    { 0x217e7cb3, 90 },
    { 0, 0 },
    { 0xf9ee7cb5, 33 },
    { 0x6c3e7cb6, 80 },
    { 0xd6fe7cb7, 23 },
    { 0x20ee7cb5, 99 },
    { 0, 0 },
    { 0x9ae7cba, 95 },
    { 0xb8ce7cbb, 34 },
    { 0xfd6e7cbb, 93 },
    { 0x8fe7cbb, 90 },
    { 0x64de7cbe, 33 },
    { 0xdafe7cbf, 90 },
    { 0x7c3e7cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809e7cc4, 90 },
    { 0x113e7cc4, 10 },
    { 0x9b7e7cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe7cc9, 95 },
    { 0, 0 },
    { 0x82ae7ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914e7cd5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3de7cd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe7cdf, 33 },
    { 0x506e7ce0, 98 },
    { 0xa82e7cdf, 23 },
    { 0x451e7ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce7ce7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade7ceb, 23 },
    { 0x101e7cec, 91 },
    { 0x4d2e7cec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e7cf0, 33 },
    { 0xcb5e7cf1, 98 },
    { 0x4e7cf1, 98 },
    { 0, 0 },
    { 0x4dae7cf4, 68 },
    { 0xdc8e7cf4, 81 },
    { 0, 0 },
    { 0xdd0e7cf7, 62 },
    { 0x482e7cf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffe7cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce7d00, 80 },
    { 0x697e7d00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08e7d04, 55 },
    { 0x3ee7d05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e7d08, 84 },
    { 0xd54e7d09, 34 },
    { 0, 0 },
    { 0xc66e7d0b, 23 },
    { 0x422e7d0b, 43 },
    { 0x249e7d0b, 95 },
    { 0x278e7d0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e7d11, 91 },
    { 0, 0 },
    { 0xe1ce7d13, 90 },
    { 0, 0 },
    { 0x3e9e7d15, 23 },
    { 0x5fde7d16, 61 },
    { 0xdbbe7d17, 43 },
    { 0xd63e7d16, 90 },
    { 0, 0 },
    { 0x2ffe7d1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84e7d1d, 76 },
    { 0x2a1e7d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71e7d22, 23 },
    { 0xe98e7d23, 95 },
    { 0x348e7d24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340e7d29, 75 },
    { 0x463e7d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e7d2d, 98 },
    { 0x66be7d2e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x925e7d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1e7d39, 33 },
    { 0x68de7d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e7d3e, 33 },
    { 0, 0 },
    { 0xae5e7d40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ee7d43, 95 },
    { 0x430e7d43, 33 },
    { 0xaa3e7d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bde7d48, 81 },
    { 0xd5ee7d48, 96 },
    { 0x5abe7d49, 33 },
    { 0, 0 },
    { 0xeb8e7d4c, 98 },
    { 0, 0 },
    { 0x7cfe7d4e, 93 },
    { 0xdd6e7d4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584e7d54, 90 },
    { 0x956e7d54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e7d5b, 23 },
    { 0x686e7d5c, 55 },
    { 0, 0 },
    { 0xcaae7d5e, 76 },
    { 0xd8ee7d5f, 23 },
    { 0, 0 },
    { 0x63ee7d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3e7d64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e7d6a, 90 },
    { 0xfc5e7d6b, 61 },
    { 0xe2de7d6c, 61 },
    { 0x4ee7d6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594e7d71, 65 },
    { 0xdabe7d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e7d76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e7d79, 34 },
    { 0x61ae7d79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e7d82, 95 },
    { 0xd1fe7d83, 33 },
    { 0, 0 },
    { 0xa5de7d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee7d90, 95 },
    { 0x6a5e7d90, 61 },
    { 0xecde7d92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ee7d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe7d9b, 23 },
    { 0, 0 },
    { 0x45be7d9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36e7da1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae7da5, 90 },
    { 0xe8ee7da5, 90 },
    { 0x805e7da7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ae7dab, 90 },
    { 0, 0 },
    { 0x17de7dad, 90 },
    { 0x111e7dae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62be7db4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ae7db7, 46 },
    { 0x58be7db8, 90 },
    { 0xde4e7db8, 67 },
    { 0x85e7dba, 46 },
    { 0x2eae7db8, 90 },
    { 0xc4be7dbc, 100 },
    { 0x669e7db9, 17 },
    { 0x8bae7db9, 76 },
    { 0x8b2e7dbf, 34 },
    { 0, 0 },
    { 0x2ae7dc1, 90 },
    { 0x3d2e7dc2, 33 },
    { 0, 0 },
    { 0x59ae7dc4, 55 },
    { 0, 0 },
    { 0xe9ee7dc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3e7dcd, 67 },
    { 0x4e4e7dce, 98 },
    { 0xe94e7dcf, 43 },
    { 0x3cbe7dcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cce7dd3, 23 },
    { 0, 0 },
    { 0x781e7dd5, 46 },
    { 0xd1ae7dd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354e7dda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e7ddd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9e7de7, 61 },
    { 0x82be7de7, 95 },
    { 0x94e7de8, 36 },
    { 0, 0 },
    { 0x99ce7deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e7def, 95 },
    { 0, 0 },
    { 0x9f2e7df1, 93 },
    { 0xf5de7df1, 95 },
    { 0xcc5e7df2, 61 },
    { 0xeb4e7df1, 43 },
    { 0, 0 },
    { 0x92ce7df6, 90 },
    { 0x4c1e7df7, 33 },
    { 0x178e7df6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e7dff, 98 },
    { 0x74ae7e00, 93 },
    { 0x9fce7e01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7e08, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e7e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee7e12, 23 },
    { 0, 0 },
    { 0xcdae7e14, 75 },
    { 0x80e7e15, 95 },
    { 0, 0 },
    { 0x50fe7e17, 90 },
    { 0, 0 },
    { 0xfbde7e19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e7e1d, 23 },
    { 0x1cfe7e1d, 98 },
    { 0, 0 },
    { 0x11ee7e20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56e7e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe7e31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e7e35, 90 },
    { 0x776e7e35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e7e3d, 66 },
    { 0, 0 },
    { 0xc3ee7e3f, 23 },
    { 0, 0 },
    { 0xd91e7e41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee7e44, 33 },
    { 0, 0 },
    { 0x504e7e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11e7e4b, 33 },
    { 0x8bfe7e4c, 33 },
    { 0, 0 },
    { 0xdabe7e4e, 90 },
    { 0, 0 },
    { 0x9a8e7e50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460e7e55, 98 },
    { 0x3ece7e55, 33 },
    { 0x975e7e55, 61 },
    { 0xc8de7e58, 98 },
    { 0xab5e7e58, 76 },
    { 0x1d4e7e5a, 23 },
    { 0x674e7e56, 33 },
    { 0x57de7e5b, 98 },
    { 0x113e7e5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e7e62, 33 },
    { 0x63ae7e63, 33 },
    { 0x789e7e64, 96 },
    { 0x9bbe7e65, 34 },
    { 0xb94e7e66, 95 },
    { 0x902e7e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfe7e6a, 67 },
    { 0, 0 },
    { 0xfafe7e6c, 23 },
    { 0x8cae7e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee7e71, 61 },
    { 0x56be7e72, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3e7e77, 52 },
    { 0x6e3e7e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3e7e7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0e7e81, 90 },
    { 0x5a2e7e82, 46 },
    { 0x18de7e83, 68 },
    { 0xd6e7e82, 90 },
    { 0xb9fe7e82, 95 },
    { 0x64ae7e86, 35 },
    { 0x1d8e7e87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7e8a, 93 },
    { 0xdc7e7e8b, 90 },
    { 0x12be7e8c, 65 },
    { 0x9e7e7e8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce7e92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71e7e99, 33 },
    { 0x5fee7e9a, 95 },
    { 0x6a4e7e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e7ea3, 90 },
    { 0xc89e7ea4, 76 },
    { 0, 0 },
    { 0xb3ae7ea6, 2 },
    { 0x3c5e7ea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2e7eab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x764e7eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593e7eb2, 76 },
    { 0x6de7eb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e7eb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130e7ebe, 23 },
    { 0, 0 },
    { 0x776e7ec0, 95 },
    { 0x1eae7ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4e7ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0e7ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e7ece, 61 },
    { 0, 0 },
    { 0x754e7ed0, 23 },
    { 0x982e7ed0, 43 },
    { 0x744e7ed1, 95 },
    { 0xed5e7ed3, 77 },
    { 0x3dee7ed4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55be7ed7, 40 },
    { 0x6b7e7ed8, 90 },
    { 0, 0 },
    { 0xa2e7eda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbe7ee0, 76 },
    { 0xbaae7ee0, 90 },
    { 0, 0 },
    { 0xe18e7ee3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe7eea, 23 },
    { 0x1e8e7eea, 75 },
    { 0x31ce7eec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe7efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e7f03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebe7f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9e7f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee7f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe7f17, 67 },
    { 0x179e7f18, 33 },
    { 0xde7f18, 99 },
    { 0, 0 },
    { 0xa2ae7f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e7f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00e7f21, 43 },
    { 0xfd1e7f21, 61 },
    { 0, 0 },
    { 0x1ae7f24, 90 },
    { 0x910e7f24, 43 },
    { 0x580e7f26, 52 },
    { 0, 0 },
    { 0x928e7f28, 98 },
    { 0x353e7f28, 90 },
    { 0x7d2e7f28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7f30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16be7f39, 61 },
    { 0xb94e7f3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e7f43, 43 },
    { 0xb37e7f44, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88e7f48, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e7f53, 93 },
    { 0xdc2e7f53, 61 },
    { 0xbfce7f55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ce7f58, 77 },
    { 0x7c2e7f59, 67 },
    { 0xff9e7f5a, 93 },
    { 0, 0 },
    { 0x554e7f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99e7f62, 55 },
    { 0x776e7f63, 83 },
    { 0x396e7f63, 34 },
    { 0xcf3e7f63, 90 },
    { 0x24ee7f66, 33 },
    { 0x90de7f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69de7f73, 61 },
    { 0xc62e7f73, 61 },
    { 0, 0 },
    { 0x265e7f76, 90 },
    { 0x509e7f77, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x260e7f7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e7f7e, 33 },
    { 0x7bae7f7f, 52 },
    { 0x739e7f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eee7f83, 23 },
    { 0x117e7f84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161e7f90, 61 },
    { 0, 0 },
    { 0x9ae7f92, 98 },
    { 0xb6fe7f92, 61 },
    { 0x54ee7f94, 95 },
    { 0xe4ee7f94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e7f99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e7f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7fa0, 61 },
    { 0xeebe7fa1, 98 },
    { 0, 0 },
    { 0x508e7fa3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2be7fa7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665e7fad, 35 },
    { 0x453e7fae, 43 },
    { 0xf91e7faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1e7fb2, 85 },
    { 0xe4e7fb3, 65 },
    { 0x47ae7fb4, 95 },
    { 0x32ae7fb4, 98 },
    { 0, 0 },
    { 0xee0e7fb7, 23 },
    { 0x12ce7fb8, 33 },
    { 0xbafe7fb9, 61 },
    { 0, 0 },
    { 0xe0e7fbb, 17 },
    { 0xb95e7fbc, 91 },
    { 0x532e7fbc, 61 },
    { 0x66fe7fbe, 66 },
    { 0, 0 },
    { 0xcc2e7fc0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9e7fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e7fd0, 68 },
    { 0x6e5e7fd1, 90 },
    { 0, 0 },
    { 0x468e7fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e7fd7, 33 },
    { 0, 0 },
    { 0x2e3e7fd9, 95 },
    { 0x8e2e7fda, 66 },
    { 0xdb3e7fdb, 93 },
    { 0, 0 },
    { 0x446e7fdd, 43 },
    { 0, 0 },
    { 0x7c4e7fdf, 34 },
    { 0, 0 },
    { 0x176e7fe1, 80 },
    { 0x850e7fe2, 90 },
    { 0x7f2e7fe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fe7fe8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e7fec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee7ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de7ff5, 95 },
    { 0x6e4e7ff5, 23 },
    { 0xcbae7ff7, 33 },
    { 0, 0 },
    { 0x390e7ff9, 46 },
    { 0x591e7ffa, 76 },
    { 0, 0 },
    { 0x208e7ffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395e8001, 90 },
    { 0x132e8001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde800e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee8014, 90 },
    { 0x795e8014, 98 },
    { 0xc35e8016, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ce801a, 33 },
    { 0, 0 },
    { 0x97be801c, 33 },
    { 0x962e801d, 78 },
    { 0, 0 },
    { 0x985e801f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e8025, 69 },
    { 0x1b4e8026, 33 },
    { 0, 0 },
    { 0x270e8028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98e802f, 34 },
    { 0xbe8030, 61 },
    { 0, 0 },
    { 0x9d2e8032, 33 },
    { 0x5b5e8032, 66 },
    { 0, 0 },
    { 0xd5fe8035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7e803a, 76 },
    { 0x95fe803b, 95 },
    { 0x412e803c, 33 },
    { 0, 0 },
    { 0xbb1e803e, 43 },
    { 0xb5ce803f, 76 },
    { 0x72de8040, 33 },
    { 0x263e8041, 77 },
    { 0xee8e8042, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9e8045, 100 },
    { 0x48be8046, 34 },
    { 0, 0 },
    { 0x373e8048, 61 },
    { 0xd7e8049, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fde804e, 77 },
    { 0x7dce804e, 28 },
    { 0xe13e804e, 34 },
    { 0x524e8051, 76 },
    { 0x75be8052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eee8055, 90 },
    { 0xaf3e8055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583e805a, 90 },
    { 0x69ee805b, 90 },
    { 0xbd2e805b, 55 },
    { 0x904e805b, 76 },
    { 0x7ece805a, 33 },
    { 0x941e805e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e8062, 95 },
    { 0xee9e8063, 90 },
    { 0x86de8063, 43 },
    { 0xacce8065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47e806b, 23 },
    { 0xf15e806c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de806f, 83 },
    { 0, 0 },
    { 0x8aee8071, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e8074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae8078, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ce8082, 84 },
    { 0x319e8083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ee8086, 90 },
    { 0, 0 },
    { 0x79ce8088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce808d, 99 },
    { 0, 0 },
    { 0xd24e808f, 95 },
    { 0x254e8090, 76 },
    { 0, 0 },
    { 0xa19e8092, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fce8095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760e80a0, 55 },
    { 0xee7e80a1, 61 },
    { 0x8d1e80a0, 61 },
    { 0x4d0e80a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e80a6, 68 },
    { 0, 0 },
    { 0xb4be80a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e80ad, 78 },
    { 0, 0 },
    { 0x988e80af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be80b3, 98 },
    { 0, 0 },
    { 0x46be80b5, 90 },
    { 0xf4ae80b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e80ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85de80be, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359e80c2, 98 },
    { 0x96ae80c3, 33 },
    { 0, 0 },
    { 0x8ce80c5, 43 },
    { 0, 0 },
    { 0x4abe80c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x222e80ca, 91 },
    { 0, 0 },
    { 0x86fe80cc, 33 },
    { 0x97ce80cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5e80d6, 98 },
    { 0x86de80d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fe80da, 90 },
    { 0x68fe80da, 33 },
    { 0, 0 },
    { 0x815e80dd, 90 },
    { 0x4dbe80de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x927e80e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6e80e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83e80ed, 33 },
    { 0xe30e80ee, 61 },
    { 0, 0 },
    { 0x9c5e80f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fe80f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49e80fc, 55 },
    { 0x7dfe80fd, 61 },
    { 0, 0 },
    { 0x6dbe80ff, 61 },
    { 0x539e8100, 98 },
    { 0x42de8101, 100 },
    { 0, 0 },
    { 0x835e8103, 34 },
    { 0xf8fe8103, 61 },
    { 0xe5e8105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e810a, 52 },
    { 0x9afe810a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e8112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x587e8115, 93 },
    { 0, 0 },
    { 0x22de8117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2e811c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e811f, 90 },
    { 0x5e1e811f, 34 },
    { 0x960e8121, 80 },
    { 0x92ee8122, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8125, 90 },
    { 0x3f5e8125, 43 },
    { 0xef2e8125, 90 },
    { 0x3c1e8128, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44e812c, 61 },
    { 0x264e812d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8130, 90 },
    { 0xdc8e8131, 98 },
    { 0xbb2e8132, 23 },
    { 0x313e8133, 33 },
    { 0x89be8132, 66 },
    { 0x715e8134, 98 },
    { 0x81be8136, 91 },
    { 0, 0 },
    { 0x340e8138, 42 },
    { 0xacae8139, 95 },
    { 0x822e813a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee8140, 90 },
    { 0xcede8140, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ce8144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3e8148, 23 },
    { 0, 0 },
    { 0xa84e814a, 90 },
    { 0xa4de814a, 91 },
    { 0x3ece814c, 33 },
    { 0, 0 },
    { 0xcf0e814e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84e8151, 46 },
    { 0x774e8152, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e8156, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264e8159, 95 },
    { 0x859e815a, 61 },
    { 0, 0 },
    { 0x72be815c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ee8162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0e8168, 91 },
    { 0x534e8169, 98 },
    { 0xd33e816a, 61 },
    { 0xb3de8169, 34 },
    { 0x845e816c, 53 },
    { 0x8e6e816d, 95 },
    { 0xd70e816e, 33 },
    { 0x3cae816e, 61 },
    { 0x7cde8170, 61 },
    { 0x1d8e8171, 43 },
    { 0xf1fe8168, 65 },
    { 0, 0 },
    { 0xa48e8174, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce817f, 55 },
    { 0, 0 },
    { 0x369e8181, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de8189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e818d, 76 },
    { 0x678e818d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e8191, 55 },
    { 0xb1fe8192, 61 },
    { 0xb7de8193, 77 },
    { 0x683e8194, 17 },
    { 0xfde8193, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace819a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30e819e, 33 },
    { 0x177e819f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfe81a3, 33 },
    { 0x84ae81a3, 65 },
    { 0x742e81a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3e81a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee81ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3e81b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15e81b4, 43 },
    { 0x77ce81b4, 61 },
    { 0xe99e81b6, 100 },
    { 0x7f3e81b5, 95 },
    { 0x8ace81b8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6e81bd, 95 },
    { 0, 0 },
    { 0x12fe81bf, 91 },
    { 0x921e81c0, 88 },
    { 0x3b0e81c1, 33 },
    { 0, 0 },
    { 0x93e81c3, 98 },
    { 0x17ee81c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee81c8, 33 },
    { 0, 0 },
    { 0x459e81ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20e81d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de81d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e81da, 55 },
    { 0xd23e81da, 90 },
    { 0, 0 },
    { 0x51fe81dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x220e81e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce81e4, 76 },
    { 0x658e81e5, 61 },
    { 0, 0 },
    { 0x897e81e7, 33 },
    { 0x978e81e7, 99 },
    { 0xc5fe81e8, 95 },
    { 0x981e81e9, 90 },
    { 0xb0ce81eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0e81ee, 61 },
    { 0x45de81ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8e81f4, 95 },
    { 0x8bde81f5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e81fb, 43 },
    { 0x27ce81fc, 61 },
    { 0, 0 },
    { 0x84ce81fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e8201, 90 },
    { 0xc92e8202, 34 },
    { 0xdece8203, 33 },
    { 0xdb7e8204, 55 },
    { 0x66fe8205, 37 },
    { 0x3f1e8206, 33 },
    { 0, 0 },
    { 0x319e8208, 90 },
    { 0, 0 },
    { 0xea7e820a, 23 },
    { 0, 0 },
    { 0x95ae820c, 85 },
    { 0x696e820d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e8213, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xface8216, 93 },
    { 0x37de8216, 90 },
    { 0, 0 },
    { 0xe5ae8219, 90 },
    { 0xbdbe821a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e821d, 46 },
    { 0x13ae821e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abe8222, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05e822a, 55 },
    { 0x23de822b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ae822f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921e8233, 100 },
    { 0x21be8234, 61 },
    { 0x36be8235, 67 },
    { 0x348e8234, 90 },
    { 0xe8de8237, 23 },
    { 0xd00e8237, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52e8242, 90 },
    { 0x78ee8243, 95 },
    { 0x8bee8244, 33 },
    { 0x5bce8243, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2e8248, 34 },
    { 0xb5ae8248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaee824c, 98 },
    { 0xbaee824d, 90 },
    { 0, 0 },
    { 0xff3e824f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913e8259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dce825c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae825f, 90 },
    { 0, 0 },
    { 0xa71e8261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e8266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefde8269, 33 },
    { 0, 0 },
    { 0xd97e826b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ce8273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe8276, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971e827b, 98 },
    { 0, 0 },
    { 0x726e827d, 61 },
    { 0x286e827d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be8283, 61 },
    { 0x911e8283, 46 },
    { 0x78e8285, 98 },
    { 0, 0 },
    { 0xc1ee8287, 55 },
    { 0, 0 },
    { 0xbd1e8289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e828e, 43 },
    { 0xf2fe828f, 61 },
    { 0, 0 },
    { 0xc85e8291, 36 },
    { 0, 0 },
    { 0x7b6e8293, 95 },
    { 0x70be8294, 95 },
    { 0, 0 },
    { 0x5b0e8296, 76 },
    { 0, 0 },
    { 0xca3e8298, 76 },
    { 0x112e8299, 33 },
    { 0xe3ee829a, 98 },
    { 0xd19e8299, 61 },
    { 0x149e8298, 61 },
    { 0xfebe829d, 95 },
    { 0x2cce829e, 23 },
    { 0xd4de829f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dce82a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de82ab, 61 },
    { 0x45fe82ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e82b1, 95 },
    { 0x9ffe82b2, 83 },
    { 0x969e82b3, 91 },
    { 0x5eee82b1, 87 },
    { 0xa37e82b2, 33 },
    { 0x507e82b6, 90 },
    { 0x315e82b7, 98 },
    { 0xdd1e82b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220e82bd, 93 },
    { 0, 0 },
    { 0x81e82bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97e82c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ae82c9, 43 },
    { 0, 0 },
    { 0x9cbe82cb, 93 },
    { 0xc45e82cc, 43 },
    { 0, 0 },
    { 0x1afe82ce, 46 },
    { 0x232e82cf, 23 },
    { 0, 0 },
    { 0x15fe82d1, 23 },
    { 0x5aae82d2, 34 },
    { 0x756e82d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33be82d9, 61 },
    { 0x592e82d9, 87 },
    { 0x59e82db, 61 },
    { 0xe67e82da, 61 },
    { 0x31be82d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7e82e2, 43 },
    { 0x6bae82e2, 93 },
    { 0x22fe82e4, 76 },
    { 0xb99e82e5, 33 },
    { 0x8d9e82e6, 23 },
    { 0xec7e82e7, 76 },
    { 0xb82e82e7, 78 },
    { 0x658e82e7, 66 },
    { 0, 0 },
    { 0x6e9e82eb, 90 },
    { 0xc59e82ec, 99 },
    { 0x7b5e82ed, 99 },
    { 0x548e82eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee82f8, 33 },
    { 0, 0 },
    { 0xda1e82fa, 34 },
    { 0, 0 },
    { 0xd17e82fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e8303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9e8307, 95 },
    { 0xaf9e8308, 55 },
    { 0x231e8308, 34 },
    { 0xc30e830a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e830d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e8317, 90 },
    { 0x6e5e8317, 84 },
    { 0, 0 },
    { 0xb81e831a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e831d, 78 },
    { 0, 0 },
    { 0x78de831f, 85 },
    { 0x821e831f, 91 },
    { 0xf66e8321, 90 },
    { 0xf70e8322, 61 },
    { 0, 0 },
    { 0x488e8324, 67 },
    { 0xcc3e8325, 90 },
    { 0x679e8325, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e832a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e832f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e8334, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae833c, 91 },
    { 0x746e833d, 23 },
    { 0, 0 },
    { 0x1bee833f, 95 },
    { 0, 0 },
    { 0x937e8341, 76 },
    { 0x42e8341, 33 },
    { 0x1d7e8342, 76 },
    { 0xf1fe8344, 98 },
    { 0, 0 },
    { 0x96ce8346, 55 },
    { 0x758e8347, 81 },
    { 0, 0 },
    { 0x889e8349, 76 },
    { 0x12ce834a, 93 },
    { 0x6c9e834b, 88 },
    { 0x9fae834c, 33 },
    { 0xcbce834a, 90 },
    { 0xec7e834e, 23 },
    { 0xce0e834f, 90 },
    { 0x8c1e8350, 43 },
    { 0x6f9e8351, 23 },
    { 0x3bde8350, 55 },
    { 0x5fee8350, 90 },
    { 0x4c5e8354, 75 },
    { 0, 0 },
    { 0xdbce8356, 98 },
    { 0xd84e8357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e835a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe835d, 76 },
    { 0x577e835e, 60 },
    { 0x4d3e835e, 81 },
    { 0x4f6e835e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117e836b, 80 },
    { 0xb4e836b, 95 },
    { 0x8a2e836b, 93 },
    { 0, 0 },
    { 0x687e836f, 90 },
    { 0, 0 },
    { 0xadde8371, 61 },
    { 0x468e8372, 93 },
    { 0x50be8371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de837e, 98 },
    { 0x698e837f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463e8386, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce8389, 90 },
    { 0xee6e8389, 85 },
    { 0x88ae838b, 99 },
    { 0x2a5e838c, 90 },
    { 0x318e838d, 90 },
    { 0x354e838e, 90 },
    { 0x4ffe838f, 98 },
    { 0xf65e838f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e8393, 40 },
    { 0x4abe8394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80be8397, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4e839c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e839f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6e83a2, 61 },
    { 0x255e83a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e83a7, 55 },
    { 0x130e83a8, 84 },
    { 0xd7fe83a7, 98 },
    { 0, 0 },
    { 0x29e83ab, 90 },
    { 0, 0 },
    { 0xdde83ad, 99 },
    { 0xbc2e83ae, 90 },
    { 0xf79e83ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e83b7, 90 },
    { 0, 0 },
    { 0xb03e83b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe83be, 33 },
    { 0x17ce83bf, 98 },
    { 0, 0 },
    { 0x76ae83c1, 33 },
    { 0xfe3e83c2, 61 },
    { 0x722e83c2, 95 },
    { 0x8cee83c4, 90 },
    { 0xf46e83c5, 78 },
    { 0xbcbe83c6, 90 },
    { 0xcfce83c1, 98 },
    { 0x24e83c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e83cc, 85 },
    { 0x5c8e83cd, 76 },
    { 0x85fe83ce, 46 },
    { 0x2a4e83cf, 33 },
    { 0xe4ee83d0, 23 },
    { 0x2a8e83d1, 33 },
    { 0x9e0e83ce, 33 },
    { 0x59ce83cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfce83d6, 75 },
    { 0, 0 },
    { 0x711e83d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e83df, 90 },
    { 0, 0 },
    { 0x80fe83e1, 33 },
    { 0xb0ce83e2, 23 },
    { 0xdbce83e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dee83e9, 95 },
    { 0xe1ee83e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e83f1, 43 },
    { 0xbe4e83f2, 76 },
    { 0, 0 },
    { 0xae7e83f4, 88 },
    { 0x445e83f5, 52 },
    { 0, 0 },
    { 0xba6e83f7, 68 },
    { 0xc17e83f8, 46 },
    { 0, 0 },
    { 0x500e83fa, 91 },
    { 0, 0 },
    { 0xc6ae83fc, 33 },
    { 0xac6e83fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3e8400, 55 },
    { 0xc11e8401, 33 },
    { 0x9aae8402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dde8411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae8416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e8420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2e842a, 76 },
    { 0xc81e842b, 98 },
    { 0x3aee842b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979e8435, 34 },
    { 0x13fe8436, 33 },
    { 0xe4be8435, 98 },
    { 0, 0 },
    { 0xa52e8439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e843c, 34 },
    { 0, 0 },
    { 0xd9fe843e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670e8443, 61 },
    { 0x86fe8444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0e8447, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be844a, 78 },
    { 0, 0 },
    { 0xafee844c, 95 },
    { 0xc3e844d, 88 },
    { 0xd83e844e, 61 },
    { 0x933e844e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9e8456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe845a, 95 },
    { 0xcf9e845b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85e8467, 90 },
    { 0x48fe8468, 55 },
    { 0xd4ce8469, 33 },
    { 0xa6ce846a, 67 },
    { 0xb9ee8468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19de846e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e8472, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28e8478, 82 },
    { 0xe1ae8479, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e847f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ece8490, 93 },
    { 0xb30e8491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e8495, 61 },
    { 0, 0 },
    { 0xbf3e8497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e849b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e849e, 33 },
    { 0xf1be849f, 98 },
    { 0x7c3e849e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1e84a3, 61 },
    { 0xb15e84a4, 98 },
    { 0xdfde84a4, 78 },
    { 0x57ae84a4, 34 },
    { 0x802e84a7, 97 },
    { 0, 0 },
    { 0x640e84a9, 99 },
    { 0x256e84a9, 95 },
    { 0, 0 },
    { 0x7a2e84ac, 95 },
    { 0x697e84ac, 61 },
    { 0x375e84ae, 55 },
    { 0x7d3e84af, 61 },
    { 0x600e84b0, 90 },
    { 0x1e7e84ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae84be, 98 },
    { 0x305e84be, 98 },
    { 0xf7ae84be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e84c4, 98 },
    { 0, 0 },
    { 0x332e84c6, 61 },
    { 0xf89e84c7, 61 },
    { 0x7e6e84c6, 90 },
    { 0xa2ee84c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e84cf, 78 },
    { 0xabce84d0, 52 },
    { 0xc6e84d0, 95 },
    { 0, 0 },
    { 0x227e84d3, 87 },
    { 0x8f2e84d4, 90 },
    { 0, 0 },
    { 0xafae84d6, 88 },
    { 0x7b6e84d7, 33 },
    { 0, 0 },
    { 0x9ebe84d9, 95 },
    { 0xf1de84da, 90 },
    { 0x7f9e84da, 33 },
    { 0xcabe84dc, 76 },
    { 0xcdce84d9, 67 },
    { 0x9ade84da, 61 },
    { 0xb88e84dc, 55 },
    { 0xff0e84e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be84e3, 95 },
    { 0x1d1e84e4, 76 },
    { 0x28be84e5, 94 },
    { 0x669e84e4, 95 },
    { 0, 0 },
    { 0xb9ae84e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de84ed, 67 },
    { 0xbcde84ee, 90 },
    { 0x818e84ee, 33 },
    { 0x956e84f0, 95 },
    { 0x324e84f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e84f5, 34 },
    { 0, 0 },
    { 0x850e84f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278e84ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e8502, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813e850c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692e8510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54e8519, 91 },
    { 0, 0 },
    { 0xadfe851b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce8520, 87 },
    { 0x56e8521, 61 },
    { 0xf15e8522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e8526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e852a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839e852e, 33 },
    { 0x7d5e852f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8e8535, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e8538, 75 },
    { 0x284e8538, 87 },
    { 0, 0 },
    { 0x3c0e853b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e853f, 84 },
    { 0, 0 },
    { 0x680e8541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50e8545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x450e8548, 98 },
    { 0, 0 },
    { 0x558e854a, 34 },
    { 0xcefe854a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6e854f, 90 },
    { 0x466e854f, 76 },
    { 0, 0 },
    { 0x211e8552, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e8556, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e855c, 98 },
    { 0x734e855c, 76 },
    { 0xbc6e855e, 53 },
    { 0, 0 },
    { 0xb3ee8560, 33 },
    { 0, 0 },
    { 0x768e8562, 33 },
    { 0, 0 },
    { 0x6dfe8564, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77e8567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9e856a, 76 },
    { 0x324e856b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106e856f, 88 },
    { 0x584e8570, 23 },
    { 0x3c5e8571, 85 },
    { 0x7fee8571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528e857a, 87 },
    { 0xdfe857b, 17 },
    { 0xb85e857c, 84 },
    { 0x948e857a, 61 },
    { 0xa45e857a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0e8582, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x225e8585, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e858a, 33 },
    { 0, 0 },
    { 0xa6ae858c, 66 },
    { 0x96ce858c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05e8590, 95 },
    { 0xeb2e8591, 76 },
    { 0xb47e8592, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8595, 95 },
    { 0xfd9e8595, 66 },
    { 0, 0 },
    { 0x85e8598, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e859c, 90 },
    { 0xa60e859c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e85a1, 90 },
    { 0, 0 },
    { 0x105e85a3, 94 },
    { 0x2b4e85a3, 95 },
    { 0, 0 },
    { 0x8efe85a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbe85a9, 90 },
    { 0x345e85a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be85ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898e85bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee85c0, 98 },
    { 0x190e85c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ae85c4, 90 },
    { 0, 0 },
    { 0x2c2e85c6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ce85cc, 40 },
    { 0, 0 },
    { 0x776e85ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbee85d2, 61 },
    { 0xfb5e85d3, 90 },
    { 0x3dae85d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e85d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ece85e4, 51 },
    { 0x26ae85e5, 91 },
    { 0, 0 },
    { 0x3cee85e7, 76 },
    { 0x915e85e7, 61 },
    { 0x213e85e9, 33 },
    { 0xd43e85e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e85f3, 23 },
    { 0x593e85f4, 17 },
    { 0x83ae85f4, 98 },
    { 0xad8e85f6, 23 },
    { 0x3bbe85f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31de85fb, 34 },
    { 0, 0 },
    { 0xe9ae85fd, 78 },
    { 0, 0 },
    { 0x247e85ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e8605, 77 },
    { 0x6d5e8606, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544e860c, 90 },
    { 0x692e860d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e8610, 91 },
    { 0x6b7e8611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeee8617, 76 },
    { 0x800e8618, 23 },
    { 0, 0 },
    { 0x73de861a, 46 },
    { 0xa9ae861b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbe861e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46de8625, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86de8628, 87 },
    { 0x5dbe8629, 23 },
    { 0x3c7e862a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x418e862d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1e8631, 76 },
    { 0x614e8632, 98 },
    { 0, 0 },
    { 0x1c2e8634, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284e8638, 17 },
    { 0, 0 },
    { 0x8efe863a, 76 },
    { 0x312e863b, 98 },
    { 0x8fbe863a, 74 },
    { 0x3dce863d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddae8640, 95 },
    { 0x770e8641, 76 },
    { 0x40e8642, 33 },
    { 0, 0 },
    { 0x25fe8644, 33 },
    { 0xe7ae8645, 98 },
    { 0xb57e8645, 95 },
    { 0x317e8647, 87 },
    { 0x21e8647, 95 },
    { 0xf24e8649, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0de864c, 95 },
    { 0x2a5e864d, 43 },
    { 0, 0 },
    { 0x380e864f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ce8652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x678e8655, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8e8659, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1e865d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2e8660, 36 },
    { 0, 0 },
    { 0xc58e8662, 95 },
    { 0, 0 },
    { 0x5cfe8664, 33 },
    { 0, 0 },
    { 0xd92e8666, 81 },
    { 0xd87e8667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e866e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x452e8671, 90 },
    { 0x70be8672, 95 },
    { 0, 0 },
    { 0xfcde8674, 76 },
    { 0x173e8675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e867a, 90 },
    { 0xe07e867a, 46 },
    { 0, 0 },
    { 0x69ce867d, 43 },
    { 0xd71e867e, 33 },
    { 0x63fe867f, 34 },
    { 0, 0 },
    { 0x952e8681, 43 },
    { 0x829e8681, 90 },
    { 0xd33e8682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8e8687, 90 },
    { 0xe88e8688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27e8690, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fee8693, 98 },
    { 0, 0 },
    { 0xc9be8695, 61 },
    { 0x35e8695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e869a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e86a4, 98 },
    { 0xa15e86a5, 23 },
    { 0x369e86a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ce86b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbde86b9, 61 },
    { 0, 0 },
    { 0xf8be86bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5e86c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75de86c7, 33 },
    { 0x91fe86c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5e86cd, 33 },
    { 0x576e86cd, 77 },
    { 0x409e86cf, 23 },
    { 0xf3ee86d0, 95 },
    { 0, 0 },
    { 0xb3ae86d2, 76 },
    { 0x2fee86d3, 40 },
    { 0x682e86d4, 90 },
    { 0x328e86d5, 23 },
    { 0xc20e86d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe86df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce86e2, 96 },
    { 0, 0 },
    { 0xedce86e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aee86ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ce86ed, 43 },
    { 0, 0 },
    { 0x2dee86ef, 90 },
    { 0x822e86ef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ee86f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60e86fc, 43 },
    { 0xdc3e86fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7e8701, 43 },
    { 0x619e8702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1e870a, 33 },
    { 0xcf1e870b, 90 },
    { 0x634e870a, 61 },
    { 0x44fe870c, 17 },
    { 0x90ee870e, 36 },
    { 0x57ce870f, 90 },
    { 0x903e870e, 99 },
    { 0x189e870e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e8718, 98 },
    { 0, 0 },
    { 0xaa0e871a, 90 },
    { 0x17de871a, 61 },
    { 0xb52e871c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be871f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e8722, 93 },
    { 0xdd7e8723, 33 },
    { 0xc5ce8724, 90 },
    { 0x89e8725, 34 },
    { 0, 0 },
    { 0xccfe8727, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2e872c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e872f, 34 },
    { 0xce8e8730, 95 },
    { 0, 0 },
    { 0x7b6e8732, 80 },
    { 0, 0 },
    { 0x5b1e8734, 90 },
    { 0, 0 },
    { 0x1b5e8736, 61 },
    { 0xf9de8737, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e873b, 61 },
    { 0x228e873c, 33 },
    { 0, 0 },
    { 0xdfde873e, 84 },
    { 0, 0 },
    { 0x76fe8740, 95 },
    { 0, 0 },
    { 0x22de8742, 36 },
    { 0x35ce8743, 43 },
    { 0xdee8743, 85 },
    { 0x3d3e8743, 76 },
    { 0x43ee8746, 90 },
    { 0xfc5e8747, 78 },
    { 0xaf6e8748, 61 },
    { 0xb67e8746, 90 },
    { 0x278e874a, 52 },
    { 0xe65e874b, 55 },
    { 0x8d9e874c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x901e874f, 61 },
    { 0xa2ce8750, 61 },
    { 0, 0 },
    { 0xdd9e8752, 96 },
    { 0, 0 },
    { 0x638e8754, 33 },
    { 0xf7ce8755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x983e8758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e8761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee8765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abe8768, 68 },
    { 0, 0 },
    { 0xec1e876a, 91 },
    { 0, 0 },
    { 0xcc5e876c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabe8770, 38 },
    { 0, 0 },
    { 0xb0be8772, 46 },
    { 0x2a9e8772, 23 },
    { 0xacce8774, 88 },
    { 0x2ace8775, 33 },
    { 0x8c3e8776, 42 },
    { 0, 0 },
    { 0x2e6e8778, 90 },
    { 0x5e1e8779, 61 },
    { 0, 0 },
    { 0x80be877b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee8781, 90 },
    { 0, 0 },
    { 0x35be8783, 99 },
    { 0, 0 },
    { 0x552e8785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9e878f, 77 },
    { 0x9a6e878f, 95 },
    { 0, 0 },
    { 0x509e8792, 23 },
    { 0, 0 },
    { 0x941e8794, 98 },
    { 0x556e8795, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34de8799, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714e87a3, 93 },
    { 0x9c8e87a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e87aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85e87b0, 33 },
    { 0x862e87b1, 78 },
    { 0xadce87b0, 61 },
    { 0, 0 },
    { 0xd6ae87b4, 46 },
    { 0xa64e87b5, 95 },
    { 0x21e87b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de87bd, 33 },
    { 0x7eae87be, 85 },
    { 0x402e87bf, 55 },
    { 0x662e87be, 95 },
    { 0xcd9e87be, 76 },
    { 0, 0 },
    { 0xcbee87c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e87cc, 90 },
    { 0x28fe87cc, 91 },
    { 0x78de87ce, 91 },
    { 0xb22e87cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e87d3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae87d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x572e87d9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1e87de, 33 },
    { 0xa5be87df, 85 },
    { 0x95fe87e0, 84 },
    { 0x992e87e0, 90 },
    { 0, 0 },
    { 0xa7fe87e3, 56 },
    { 0x406e87e4, 87 },
    { 0xd35e87e4, 95 },
    { 0xfa8e87e6, 95 },
    { 0x3bbe87e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fe87ec, 43 },
    { 0xad2e87ed, 33 },
    { 0x9dbe87ed, 61 },
    { 0x37de87ee, 61 },
    { 0xc6fe87f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e87f5, 99 },
    { 0xd8be87f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e87f9, 52 },
    { 0, 0 },
    { 0xc6ce87fb, 33 },
    { 0x35ce87fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688e8802, 98 },
    { 0x817e8802, 95 },
    { 0xe79e8804, 98 },
    { 0xc60e8805, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e880e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e8813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e8823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe8827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5e882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e8833, 90 },
    { 0x73de8834, 85 },
    { 0xef5e8833, 55 },
    { 0x55de8836, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e883c, 99 },
    { 0x3f6e883d, 76 },
    { 0x54ee883c, 46 },
    { 0xb0de883f, 33 },
    { 0, 0 },
    { 0x4d3e8841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ee8846, 52 },
    { 0x988e8847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5e884b, 99 },
    { 0xcf4e884c, 74 },
    { 0x7cde884c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e8854, 23 },
    { 0x8c5e8854, 90 },
    { 0xcb7e8855, 77 },
    { 0x1ace8857, 61 },
    { 0x5a1e8857, 66 },
    { 0xc17e8859, 101 },
    { 0xae1e8858, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1e8861, 23 },
    { 0xfcee8861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6e886a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dee886d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e8871, 55 },
    { 0x50ee8872, 90 },
    { 0x715e8872, 34 },
    { 0, 0 },
    { 0x660e8875, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce887a, 33 },
    { 0x8c2e887b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ae887f, 55 },
    { 0, 0 },
    { 0xa3e8881, 76 },
    { 0, 0 },
    { 0xfa0e8883, 75 },
    { 0x598e8883, 38 },
    { 0xd7ce8883, 23 },
    { 0, 0 },
    { 0xb0de8887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95de888a, 34 },
    { 0x34e888a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ce888f, 61 },
    { 0x586e8890, 85 },
    { 0xf85e888f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8897, 43 },
    { 0x2dbe8897, 76 },
    { 0x6bbe8899, 34 },
    { 0x688e889a, 61 },
    { 0x87be889a, 76 },
    { 0, 0 },
    { 0x207e889d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e88a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x423e88a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ee88a7, 34 },
    { 0xc5ae88a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e88ab, 69 },
    { 0, 0 },
    { 0x1e9e88ad, 61 },
    { 0, 0 },
    { 0x169e88af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe88b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527e88ba, 88 },
    { 0x8c0e88bb, 33 },
    { 0x432e88ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e88c1, 33 },
    { 0x47e88c2, 83 },
    { 0x8a4e88c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae88c6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce88cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e88ce, 99 },
    { 0, 0 },
    { 0x367e88d0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfe88d4, 96 },
    { 0xfe1e88d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e88d8, 90 },
    { 0, 0 },
    { 0xa48e88da, 33 },
    { 0xeace88db, 70 },
    { 0x98be88da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce88e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7e88ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eee88f1, 99 },
    { 0, 0 },
    { 0x5d0e88f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ce88f9, 61 },
    { 0xe17e88fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942e8904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e8911, 93 },
    { 0, 0 },
    { 0xbdde8913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce8916, 23 },
    { 0, 0 },
    { 0xc6e8918, 90 },
    { 0, 0 },
    { 0x849e891a, 96 },
    { 0x154e891b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1e891e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5e8921, 55 },
    { 0xdb7e8922, 98 },
    { 0xe55e8923, 90 },
    { 0x5cde8921, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe8928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e892b, 55 },
    { 0xb95e892c, 33 },
    { 0, 0 },
    { 0x75be892e, 76 },
    { 0x555e892e, 55 },
    { 0, 0 },
    { 0x48ee8931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521e8938, 33 },
    { 0x40ae8939, 61 },
    { 0, 0 },
    { 0x869e893b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204e8941, 90 },
    { 0xa89e8942, 90 },
    { 0x2f0e8943, 90 },
    { 0xfade8943, 61 },
    { 0xb71e8942, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdce8948, 87 },
    { 0x7f4e8949, 33 },
    { 0, 0 },
    { 0xce4e894b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64e894e, 87 },
    { 0, 0 },
    { 0xd1fe8950, 68 },
    { 0x859e8951, 85 },
    { 0x673e8950, 99 },
    { 0x4c3e8953, 33 },
    { 0, 0 },
    { 0x383e8955, 90 },
    { 0, 0 },
    { 0x96ee8957, 85 },
    { 0xefbe8958, 76 },
    { 0xa81e8959, 84 },
    { 0x4c9e8957, 61 },
    { 0xdbe895b, 93 },
    { 0x432e8959, 76 },
    { 0x3e1e8958, 66 },
    { 0x370e8958, 95 },
    { 0xc71e8957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fe8962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4e8965, 99 },
    { 0x696e8965, 61 },
    { 0, 0 },
    { 0x541e8968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504e896d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e8970, 95 },
    { 0, 0 },
    { 0x1abe8972, 87 },
    { 0xe36e8972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85e8976, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ee897a, 53 },
    { 0xf4ee897b, 100 },
    { 0xd5ae897c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e8983, 78 },
    { 0x962e8984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7e8987, 33 },
    { 0x4bfe8987, 31 },
    { 0x84ee8987, 95 },
    { 0x903e898a, 93 },
    { 0xcc5e898a, 55 },
    { 0x1b7e898c, 61 },
    { 0x956e898c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee8990, 90 },
    { 0, 0 },
    { 0x33be8992, 78 },
    { 0x1dae8993, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6e8996, 43 },
    { 0, 0 },
    { 0x154e8998, 61 },
    { 0xdc6e8999, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e899c, 55 },
    { 0, 0 },
    { 0xc70e899e, 90 },
    { 0x1a3e899f, 66 },
    { 0x671e899e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de89a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e89a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e89ad, 95 },
    { 0xdb8e89ad, 90 },
    { 0xcf5e89ae, 61 },
    { 0x93ce89b0, 76 },
    { 0, 0 },
    { 0xdc4e89b2, 46 },
    { 0x5a9e89b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9e89b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28e89ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e89c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede89cb, 100 },
    { 0, 0 },
    { 0xc75e89cd, 87 },
    { 0x987e89cd, 90 },
    { 0xaf2e89cf, 95 },
    { 0xc07e89cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68e89d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61be89d7, 61 },
    { 0x3b0e89d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de89dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e89e2, 43 },
    { 0xfa0e89e2, 33 },
    { 0, 0 },
    { 0x2e2e89e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1e89e9, 69 },
    { 0, 0 },
    { 0x7dbe89eb, 98 },
    { 0x178e89ec, 61 },
    { 0xbcae89eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x211e89f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e89f8, 43 },
    { 0x21fe89f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbe89fc, 43 },
    { 0, 0 },
    { 0xd6be89fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1e8a03, 90 },
    { 0xede8a04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246e8a0b, 61 },
    { 0x43e8a0b, 33 },
    { 0x615e8a0d, 23 },
    { 0, 0 },
    { 0x439e8a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0be8a14, 75 },
    { 0, 0 },
    { 0x39ae8a16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e8a1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae8a22, 98 },
    { 0xbb7e8a23, 95 },
    { 0, 0 },
    { 0x365e8a25, 17 },
    { 0, 0 },
    { 0x3cce8a27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e8a2b, 23 },
    { 0xc7fe8a2b, 81 },
    { 0xd49e8a2c, 90 },
    { 0xd55e8a2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae8a38, 94 },
    { 0x6c4e8a39, 33 },
    { 0, 0 },
    { 0x199e8a3b, 61 },
    { 0xf96e8a3c, 23 },
    { 0, 0 },
    { 0x41ee8a3e, 23 },
    { 0xa70e8a3f, 98 },
    { 0xb8ce8a40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e8a47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e8a4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaee8a51, 61 },
    { 0xc41e8a51, 98 },
    { 0x9d4e8a52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ae8a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee8a61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e8a66, 78 },
    { 0x8fae8a67, 68 },
    { 0xdaae8a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e8a6c, 91 },
    { 0xe99e8a6d, 88 },
    { 0x3c1e8a6c, 61 },
    { 0x78be8a6c, 95 },
    { 0xa2fe8a70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6e8a74, 64 },
    { 0xbf2e8a75, 61 },
    { 0x775e8a75, 82 },
    { 0xff3e8a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e8a7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e8a7f, 33 },
    { 0xab7e8a7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce8a84, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ae8a87, 61 },
    { 0xe0ae8a87, 95 },
    { 0x566e8a89, 55 },
    { 0xce2e8a8a, 61 },
    { 0x6b2e8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76e8a90, 55 },
    { 0x57be8a90, 46 },
    { 0, 0 },
    { 0x93ae8a93, 94 },
    { 0xa86e8a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ae8a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e8a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2e8aa1, 33 },
    { 0x820e8aa2, 55 },
    { 0xee8e8aa1, 98 },
    { 0xa46e8aa4, 78 },
    { 0x70de8aa5, 90 },
    { 0x6ede8aa6, 77 },
    { 0x381e8aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e8aac, 33 },
    { 0x1d1e8aac, 79 },
    { 0, 0 },
    { 0x3ede8aaf, 98 },
    { 0x3fe8aaf, 34 },
    { 0, 0 },
    { 0x953e8ab2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ee8ab5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee8abb, 88 },
    { 0x61ee8abc, 55 },
    { 0x606e8abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7e8ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede8ac6, 67 },
    { 0x2b2e8ac7, 55 },
    { 0x48ce8ac7, 46 },
    { 0x456e8ac8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e8ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ee8ad3, 65 },
    { 0, 0 },
    { 0xefae8ad5, 76 },
    { 0xd6fe8ad6, 95 },
    { 0xb32e8ad7, 55 },
    { 0x20e8ad8, 17 },
    { 0xe98e8ad8, 52 },
    { 0xca4e8ad7, 33 },
    { 0xe2ce8adb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e8ade, 95 },
    { 0, 0 },
    { 0x69ce8ae0, 61 },
    { 0xf6de8ae1, 95 },
    { 0, 0 },
    { 0x96ce8ae3, 52 },
    { 0xcee8ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e8ae7, 66 },
    { 0xb57e8ae8, 23 },
    { 0xc0ae8ae9, 98 },
    { 0x20be8ae9, 55 },
    { 0x25ce8ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fce8af1, 67 },
    { 0, 0 },
    { 0x860e8af3, 98 },
    { 0xa51e8af4, 69 },
    { 0, 0 },
    { 0x794e8af6, 98 },
    { 0x33ce8af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee8afa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e8afe, 55 },
    { 0x8e7e8aff, 34 },
    { 0x1d9e8b00, 97 },
    { 0xb98e8b00, 33 },
    { 0x848e8b02, 61 },
    { 0, 0 },
    { 0x9d8e8b04, 98 },
    { 0xa20e8b04, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75e8b0b, 98 },
    { 0x6bce8b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e8b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe8b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de8b19, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de8b1c, 46 },
    { 0x895e8b1d, 75 },
    { 0xe54e8b1d, 34 },
    { 0xb81e8b1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8b22, 61 },
    { 0x90de8b23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fe8b29, 90 },
    { 0x231e8b2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ee8b2e, 95 },
    { 0x816e8b2e, 76 },
    { 0x955e8b30, 33 },
    { 0xf19e8b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce8b35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e8b3b, 67 },
    { 0, 0 },
    { 0x557e8b3d, 34 },
    { 0x2dbe8b3e, 98 },
    { 0x2efe8b3d, 90 },
    { 0, 0 },
    { 0xb5ee8b41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee8b49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fe8b4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92e8b55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e8b59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e8b5e, 55 },
    { 0x457e8b5f, 33 },
    { 0xb09e8b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e8b69, 77 },
    { 0x91fe8b6a, 43 },
    { 0xc6fe8b6a, 66 },
    { 0, 0 },
    { 0x11ae8b6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2e8b72, 42 },
    { 0x6e2e8b73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be8b76, 93 },
    { 0x24e8b76, 76 },
    { 0x3e0e8b76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98de8b7d, 61 },
    { 0x2dde8b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e8b86, 90 },
    { 0xe51e8b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x496e8b8a, 98 },
    { 0x3bae8b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce8b8e, 90 },
    { 0, 0 },
    { 0x75ae8b90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8e8b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e8b96, 88 },
    { 0x9a4e8b96, 95 },
    { 0, 0 },
    { 0x458e8b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfee8b9e, 76 },
    { 0x7ede8b9e, 33 },
    { 0xc17e8ba0, 23 },
    { 0xf2ce8ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6e8ba5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ce8ba8, 67 },
    { 0xd29e8ba9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e8bad, 61 },
    { 0x311e8bae, 43 },
    { 0xeaee8bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e8bb5, 95 },
    { 0x237e8bb6, 36 },
    { 0x6e4e8bb7, 98 },
    { 0x9a4e8bb5, 85 },
    { 0, 0 },
    { 0xde8bba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219e8bbe, 66 },
    { 0x60ce8bbf, 61 },
    { 0, 0 },
    { 0x6b9e8bc1, 61 },
    { 0x61ce8bc2, 23 },
    { 0x153e8bc3, 61 },
    { 0, 0 },
    { 0xa68e8bc5, 98 },
    { 0, 0 },
    { 0xa39e8bc7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e8bcb, 17 },
    { 0x6afe8bcb, 99 },
    { 0x158e8bcd, 78 },
    { 0x3e2e8bce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34e8bd2, 90 },
    { 0, 0 },
    { 0x50be8bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de8bd7, 85 },
    { 0xb26e8bd8, 23 },
    { 0x13ee8bd7, 34 },
    { 0xd77e8bd8, 95 },
    { 0x640e8bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e8be7, 75 },
    { 0, 0 },
    { 0x36de8be9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17e8bed, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e8bf1, 95 },
    { 0xa74e8bf1, 23 },
    { 0x8f1e8bf3, 81 },
    { 0xe41e8bf3, 61 },
    { 0xf78e8bf5, 23 },
    { 0xe7fe8bf6, 94 },
    { 0x2bee8bf7, 33 },
    { 0x3dce8bf7, 23 },
    { 0xd5fe8bf9, 46 },
    { 0x48ee8bfa, 78 },
    { 0xf6de8bfb, 95 },
    { 0x67e8bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e8c00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672e8c05, 23 },
    { 0xc42e8c05, 96 },
    { 0x2e9e8c06, 33 },
    { 0x3eee8c08, 76 },
    { 0, 0 },
    { 0x7de8c0a, 61 },
    { 0x403e8c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe8c0e, 33 },
    { 0x37ce8c0f, 90 },
    { 0x63ee8c10, 46 },
    { 0x4ede8c11, 33 },
    { 0x4f4e8c10, 17 },
    { 0x169e8c0f, 33 },
    { 0xe9e8c14, 93 },
    { 0, 0 },
    { 0xeb1e8c16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e8c1d, 90 },
    { 0, 0 },
    { 0x3e6e8c1f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4e8c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1e8c2a, 48 },
    { 0, 0 },
    { 0x612e8c2c, 33 },
    { 0x945e8c2c, 95 },
    { 0x3a8e8c2d, 61 },
    { 0xdf0e8c2f, 95 },
    { 0xd11e8c2d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e8c38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e8c3e, 88 },
    { 0x52ee8c3f, 90 },
    { 0, 0 },
    { 0x908e8c41, 76 },
    { 0x375e8c41, 90 },
    { 0, 0 },
    { 0xa4ae8c44, 61 },
    { 0x50be8c45, 81 },
    { 0xce2e8c45, 61 },
    { 0xd3ce8c47, 23 },
    { 0xe4fe8c45, 61 },
    { 0xf47e8c44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456e8c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79e8c5f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761e8c64, 100 },
    { 0xa97e8c65, 61 },
    { 0x96be8c66, 61 },
    { 0xe51e8c67, 87 },
    { 0x62de8c65, 43 },
    { 0x50ae8c67, 33 },
    { 0x72de8c6a, 98 },
    { 0x78ae8c6b, 33 },
    { 0x9fae8c6c, 90 },
    { 0x594e8c6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e8c70, 23 },
    { 0xa3ae8c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21e8c74, 98 },
    { 0x21be8c75, 61 },
    { 0xe4ce8c76, 43 },
    { 0x131e8c75, 61 },
    { 0, 0 },
    { 0x2d7e8c79, 84 },
    { 0xa0fe8c7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e8c7e, 61 },
    { 0xc9e8c7f, 90 },
    { 0x864e8c7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeae8c85, 76 },
    { 0xacde8c86, 90 },
    { 0x3c7e8c87, 90 },
    { 0x5c0e8c88, 23 },
    { 0x5f5e8c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e8c8f, 33 },
    { 0x2d0e8c90, 98 },
    { 0, 0 },
    { 0xdbce8c92, 78 },
    { 0x9e2e8c93, 84 },
    { 0, 0 },
    { 0xfe0e8c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e8c98, 61 },
    { 0, 0 },
    { 0xa22e8c9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368e8ca0, 66 },
    { 0, 0 },
    { 0x6f7e8ca2, 93 },
    { 0x200e8ca3, 23 },
    { 0xf5fe8ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970e8cab, 33 },
    { 0xf3be8cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bae8caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be8cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2e8cb7, 98 },
    { 0xac5e8cb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be8cbe, 23 },
    { 0, 0 },
    { 0x212e8cc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964e8cc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x735e8cc7, 93 },
    { 0, 0 },
    { 0xb97e8cc9, 95 },
    { 0x718e8cca, 68 },
    { 0x561e8ccb, 90 },
    { 0xf75e8cc9, 55 },
    { 0x3e4e8ccb, 95 },
    { 0x712e8cce, 90 },
    { 0x7ffe8ccc, 90 },
    { 0x81fe8cd0, 46 },
    { 0x189e8cd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e8cd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fe8ce1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee8ce5, 46 },
    { 0xb97e8ce6, 85 },
    { 0, 0 },
    { 0xc5be8ce8, 93 },
    { 0xf3ee8ce9, 66 },
    { 0xbade8ce8, 65 },
    { 0xf41e8ceb, 23 },
    { 0xfb4e8cec, 99 },
    { 0xd96e8ced, 61 },
    { 0xf5be8cee, 33 },
    { 0xe30e8cef, 33 },
    { 0xc38e8ceb, 23 },
    { 0xbcfe8ceb, 95 },
    { 0x729e8cf1, 55 },
    { 0, 0 },
    { 0x5ebe8cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce8cf8, 66 },
    { 0, 0 },
    { 0x9fee8cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3e8d00, 90 },
    { 0x478e8d01, 61 },
    { 0xe26e8d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee8d09, 90 },
    { 0x259e8d0a, 55 },
    { 0xf4e8d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2e8d0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee8d14, 76 },
    { 0, 0 },
    { 0x450e8d16, 23 },
    { 0xbb9e8d17, 55 },
    { 0x5e3e8d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e8d1c, 61 },
    { 0xed3e8d1c, 78 },
    { 0x739e8d1d, 76 },
    { 0xd4ae8d1d, 76 },
    { 0x36be8d20, 52 },
    { 0xa83e8d21, 93 },
    { 0, 0 },
    { 0x20ee8d23, 43 },
    { 0xe04e8d23, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e8d27, 98 },
    { 0, 0 },
    { 0x854e8d29, 34 },
    { 0xa07e8d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7e8d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e8d37, 90 },
    { 0x36de8d37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e8d3c, 55 },
    { 0, 0 },
    { 0x92de8d3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e8d42, 67 },
    { 0, 0 },
    { 0x533e8d44, 67 },
    { 0, 0 },
    { 0x674e8d46, 95 },
    { 0, 0 },
    { 0xb87e8d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee8d50, 43 },
    { 0x930e8d51, 91 },
    { 0x485e8d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ae8d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x643e8d5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ee8d61, 23 },
    { 0xeae8d62, 46 },
    { 0x55be8d62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae8d68, 93 },
    { 0x844e8d69, 65 },
    { 0xfce8d6a, 33 },
    { 0xee6e8d6b, 95 },
    { 0xa6ae8d6c, 93 },
    { 0xc50e8d6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3e8d70, 52 },
    { 0x47e8d71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e8d75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08e8d79, 90 },
    { 0, 0 },
    { 0xfa5e8d7b, 100 },
    { 0xddfe8d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325e8d84, 96 },
    { 0, 0 },
    { 0x953e8d86, 90 },
    { 0x965e8d86, 33 },
    { 0xa89e8d87, 90 },
    { 0x711e8d89, 84 },
    { 0, 0 },
    { 0x9f7e8d8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de8d91, 95 },
    { 0x221e8d91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478e8d96, 33 },
    { 0x582e8d97, 78 },
    { 0, 0 },
    { 0xc74e8d99, 33 },
    { 0x31ce8d9a, 85 },
    { 0xcc2e8d9b, 33 },
    { 0xbe8d9a, 90 },
    { 0xa6be8d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce8da2, 43 },
    { 0x1fee8da3, 23 },
    { 0x24ee8da4, 98 },
    { 0x39ee8da2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebde8dab, 33 },
    { 0, 0 },
    { 0x224e8dad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89e8db6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6e8dc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6e8dc5, 83 },
    { 0xdebe8dc6, 98 },
    { 0xc5be8dc6, 61 },
    { 0xdcbe8dc8, 90 },
    { 0xcfe8dc8, 76 },
    { 0x819e8dca, 80 },
    { 0x761e8dca, 85 },
    { 0xe00e8dca, 84 },
    { 0xa48e8dcd, 76 },
    { 0xb6fe8dc7, 99 },
    { 0x7d9e8dcf, 17 },
    { 0xaede8dd0, 100 },
    { 0x666e8dd1, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e8dd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e8dd7, 61 },
    { 0x7bfe8dd8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e8ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e8de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae8dea, 34 },
    { 0x857e8dea, 98 },
    { 0x2ebe8dec, 98 },
    { 0x1a4e8ded, 34 },
    { 0x711e8dee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee8df2, 95 },
    { 0xe9e8df3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3e8df6, 76 },
    { 0, 0 },
    { 0x448e8df8, 61 },
    { 0xf86e8df9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce8dfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11de8e02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce8e06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39de8e0a, 90 },
    { 0x91be8e0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e8e11, 90 },
    { 0x3cbe8e12, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e8e18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e8e1b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812e8e21, 33 },
    { 0, 0 },
    { 0xeb5e8e23, 95 },
    { 0, 0 },
    { 0x9b3e8e25, 38 },
    { 0xf4fe8e25, 23 },
    { 0, 0 },
    { 0xe7e8e28, 23 },
    { 0x7bfe8e28, 23 },
    { 0, 0 },
    { 0x723e8e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4e8e30, 99 },
    { 0xd9ee8e31, 92 },
    { 0, 0 },
    { 0x75de8e33, 80 },
    { 0x90ce8e33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae8e37, 61 },
    { 0xfbfe8e37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x820e8e3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e8e46, 77 },
    { 0xcf1e8e47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9e8e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00e8e52, 95 },
    { 0xa2ce8e53, 98 },
    { 0xfe6e8e53, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e8e5a, 67 },
    { 0x97fe8e5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46de8e5f, 23 },
    { 0xdc8e8e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x784e8e63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe8e66, 61 },
    { 0, 0 },
    { 0x7d9e8e68, 78 },
    { 0x252e8e68, 33 },
    { 0xa2fe8e6a, 76 },
    { 0x5ae8e6b, 23 },
    { 0xff7e8e6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24e8e75, 68 },
    { 0x6b2e8e76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90de8e7c, 93 },
    { 0, 0 },
    { 0x4f8e8e7e, 23 },
    { 0xe99e8e7f, 87 },
    { 0xe1be8e80, 76 },
    { 0x9cce8e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfe8e89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfe8e8e, 87 },
    { 0xcd4e8e8e, 90 },
    { 0xce8e8f, 76 },
    { 0, 0 },
    { 0xadfe8e92, 90 },
    { 0x120e8e93, 99 },
    { 0xd1fe8e94, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e8e97, 33 },
    { 0x103e8e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be8e9f, 76 },
    { 0, 0 },
    { 0x76ce8ea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92be8ea5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22de8ead, 55 },
    { 0xa78e8ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79e8eb3, 93 },
    { 0, 0 },
    { 0x9c5e8eb5, 98 },
    { 0xb6ee8eb6, 33 },
    { 0xc04e8eb7, 91 },
    { 0x515e8eb8, 95 },
    { 0x670e8eb6, 34 },
    { 0x5e8e8eba, 68 },
    { 0x5a5e8eba, 76 },
    { 0x6eee8ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85de8ec3, 88 },
    { 0x3c9e8ec3, 61 },
    { 0x812e8ec4, 95 },
    { 0xe2be8ec6, 77 },
    { 0xba6e8ec6, 33 },
    { 0xc3ee8ec7, 90 },
    { 0xc82e8ec7, 98 },
    { 0xbeae8eca, 95 },
    { 0xf2ee8ec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e8ed2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e8ed8, 46 },
    { 0xf2fe8ed9, 61 },
    { 0x2d4e8ed8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6e8edd, 55 },
    { 0x85de8ede, 90 },
    { 0x8f3e8edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce8ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe8ee7, 99 },
    { 0xb5e8ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e8eeb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ae8ef3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7de8ef7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dce8efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e8f02, 87 },
    { 0, 0 },
    { 0xef8e8f04, 90 },
    { 0xf50e8f05, 52 },
    { 0x8b7e8f06, 61 },
    { 0, 0 },
    { 0xad7e8f08, 43 },
    { 0x3e2e8f08, 46 },
    { 0, 0 },
    { 0x4e9e8f0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e8f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e8f16, 95 },
    { 0x2a4e8f16, 98 },
    { 0xd79e8f18, 76 },
    { 0x245e8f17, 23 },
    { 0x80ce8f1a, 90 },
    { 0x89be8f1b, 90 },
    { 0xfa8e8f1c, 90 },
    { 0xd2de8f1c, 99 },
    { 0x28e8f1e, 93 },
    { 0x346e8f1f, 90 },
    { 0x261e8f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de8f26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe8f29, 23 },
    { 0xb8e8f2a, 23 },
    { 0x7f7e8f2a, 95 },
    { 0xd45e8f2c, 65 },
    { 0xcb5e8f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e8f32, 77 },
    { 0x856e8f33, 69 },
    { 0x433e8f34, 55 },
    { 0x73ae8f33, 23 },
    { 0xb4fe8f36, 87 },
    { 0, 0 },
    { 0xce1e8f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e8f3e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e8f47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e8f4d, 61 },
    { 0, 0 },
    { 0x382e8f4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37e8f53, 90 },
    { 0x65e8f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511e8f5e, 77 },
    { 0, 0 },
    { 0xaace8f60, 78 },
    { 0xf6ee8f61, 17 },
    { 0, 0 },
    { 0x358e8f63, 23 },
    { 0, 0 },
    { 0x33e8f65, 23 },
    { 0, 0 },
    { 0xe2de8f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce8f6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261e8f6f, 55 },
    { 0x42de8f70, 87 },
    { 0xbde8f71, 76 },
    { 0x146e8f72, 38 },
    { 0x377e8f73, 90 },
    { 0x89ce8f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0e8f7a, 95 },
    { 0, 0 },
    { 0xdc4e8f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02e8f7f, 43 },
    { 0x98e8f80, 33 },
    { 0x8b0e8f81, 81 },
    { 0x943e8f80, 84 },
    { 0xdd0e8f7f, 88 },
    { 0xa98e8f84, 33 },
    { 0x3b9e8f85, 84 },
    { 0xce3e8f86, 95 },
    { 0x5dce8f87, 85 },
    { 0x1c7e8f87, 23 },
    { 0xbb5e8f89, 95 },
    { 0x805e8f8a, 87 },
    { 0x10fe8f8b, 33 },
    { 0x775e8f86, 23 },
    { 0, 0 },
    { 0xa0ee8f8e, 52 },
    { 0x1d8e8f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95e8f96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe8f99, 76 },
    { 0, 0 },
    { 0xa45e8f9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e8fa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ee8fa5, 23 },
    { 0x75ee8fa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123e8faa, 55 },
    { 0x8a2e8fab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63e8fae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6e8fb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8fb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e8fba, 93 },
    { 0x40ce8fbb, 76 },
    { 0xe26e8fbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee8fbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e8fc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dee8fc7, 91 },
    { 0, 0 },
    { 0x865e8fc9, 90 },
    { 0, 0 },
    { 0x880e8fcb, 46 },
    { 0, 0 },
    { 0x763e8fcd, 85 },
    { 0x1bee8fce, 99 },
    { 0xd28e8fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e8fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1e8fd6, 23 },
    { 0xee8e8fd7, 98 },
    { 0x482e8fd6, 52 },
    { 0x404e8fd7, 23 },
    { 0x45ce8fda, 91 },
    { 0x593e8fdb, 34 },
    { 0x3e8e8fda, 95 },
    { 0x869e8fdd, 53 },
    { 0x9efe8fde, 84 },
    { 0xf03e8fde, 61 },
    { 0, 0 },
    { 0xc7ee8fe1, 33 },
    { 0xed7e8fe2, 95 },
    { 0x46ce8fe3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe8fe7, 61 },
    { 0xd88e8fe8, 95 },
    { 0x82ae8fe9, 43 },
    { 0x610e8fea, 61 },
    { 0x2bee8feb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426e8fef, 61 },
    { 0x9a3e8fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e8ff3, 81 },
    { 0xe84e8ff4, 99 },
    { 0, 0 },
    { 0x7e8e8ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae8ffa, 81 },
    { 0xc05e8ffb, 23 },
    { 0x8c8e8ffa, 98 },
    { 0x37de8ffd, 87 },
    { 0x40ce8ffe, 61 },
    { 0x315e8fff, 61 },
    { 0x68ae9000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x488e9003, 46 },
    { 0, 0 },
    { 0xad5e9005, 85 },
    { 0x44fe9006, 33 },
    { 0, 0 },
    { 0x8cbe9008, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2e9011, 90 },
    { 0xde7e9012, 23 },
    { 0xa0be9013, 23 },
    { 0x38fe9014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e9018, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe901e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e9021, 23 },
    { 0xeae9021, 99 },
    { 0x237e9022, 76 },
    { 0x6e1e9022, 77 },
    { 0xa2ae9022, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de902d, 90 },
    { 0x8cfe902d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde9034, 78 },
    { 0, 0 },
    { 0xd53e9036, 77 },
    { 0x80ce9037, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21de903a, 87 },
    { 0, 0 },
    { 0x73ee903c, 38 },
    { 0x5a4e903d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de9043, 33 },
    { 0, 0 },
    { 0xb09e9045, 93 },
    { 0, 0 },
    { 0xa1ee9047, 33 },
    { 0xf6ae9048, 23 },
    { 0x965e9048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189e904e, 84 },
    { 0x1abe904e, 90 },
    { 0xa14e904e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe9053, 95 },
    { 0x95ce9054, 61 },
    { 0, 0 },
    { 0xcb0e9056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ee905d, 90 },
    { 0x78de905e, 61 },
    { 0, 0 },
    { 0xefe9060, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e9064, 65 },
    { 0x278e9064, 90 },
    { 0x725e9066, 90 },
    { 0, 0 },
    { 0xfe1e9068, 33 },
    { 0xa96e9068, 33 },
    { 0xb57e9069, 34 },
    { 0, 0 },
    { 0x3dbe906c, 34 },
    { 0x83be906d, 90 },
    { 0xe33e906e, 33 },
    { 0, 0 },
    { 0xb1de9070, 33 },
    { 0, 0 },
    { 0x37e9072, 55 },
    { 0x389e9072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7e9076, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60e907c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce907f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fe9084, 33 },
    { 0, 0 },
    { 0x43de9086, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bee908b, 33 },
    { 0xab3e908b, 93 },
    { 0x96e908d, 98 },
    { 0xb19e908c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9093, 61 },
    { 0xc9be9093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e9099, 43 },
    { 0x7a2e909a, 76 },
    { 0x55e909a, 99 },
    { 0, 0 },
    { 0x626e909d, 46 },
    { 0x5bde909e, 23 },
    { 0xf06e909f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe90a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x165e90aa, 33 },
    { 0, 0 },
    { 0x42fe90ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e90b1, 34 },
    { 0x125e90b1, 33 },
    { 0x38ce90b1, 95 },
    { 0x80be90b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ee90bd, 17 },
    { 0x574e90be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59e90c3, 88 },
    { 0x98fe90c4, 53 },
    { 0xae0e90c5, 64 },
    { 0x7c2e90c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae90ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee90ce, 98 },
    { 0x8f8e90cf, 28 },
    { 0xb4ee90cf, 98 },
    { 0xb3ae90d0, 91 },
    { 0xade90d1, 95 },
    { 0x3bde90d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe90d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae90dd, 76 },
    { 0, 0 },
    { 0x58ce90df, 90 },
    { 0xc5ce90df, 76 },
    { 0x91fe90e1, 93 },
    { 0x2b3e90e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be90e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e90ea, 98 },
    { 0, 0 },
    { 0xc23e90ec, 55 },
    { 0, 0 },
    { 0x496e90ee, 33 },
    { 0x8b1e90ee, 36 },
    { 0x6b1e90f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45e90f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e90f9, 63 },
    { 0, 0 },
    { 0xe94e90fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e90ff, 23 },
    { 0x2d8e9100, 38 },
    { 0x239e9100, 76 },
    { 0xf54e9100, 33 },
    { 0xe0be9103, 23 },
    { 0x2dce9102, 98 },
    { 0, 0 },
    { 0xa64e9106, 95 },
    { 0x308e9106, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e910e, 32 },
    { 0x900e910f, 33 },
    { 0xafce910f, 67 },
    { 0xf87e9111, 99 },
    { 0x9dee9111, 55 },
    { 0xf89e9113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x216e9116, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbe911b, 90 },
    { 0x57ce911b, 61 },
    { 0xb8ae911c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ce9121, 43 },
    { 0x706e9122, 65 },
    { 0xc1be9123, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840e9129, 43 },
    { 0x3fee912a, 81 },
    { 0x6abe9129, 34 },
    { 0x38ce912c, 87 },
    { 0x9bfe912d, 55 },
    { 0xdfee912e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e9133, 61 },
    { 0xadde9133, 95 },
    { 0xd15e9135, 61 },
    { 0x3cae9136, 23 },
    { 0x21be9133, 68 },
    { 0x2c8e9136, 76 },
    { 0xc97e9138, 90 },
    { 0xccee913a, 33 },
    { 0, 0 },
    { 0x2d2e913c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e9144, 61 },
    { 0x4e3e9145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2e9148, 100 },
    { 0, 0 },
    { 0x8d4e914a, 51 },
    { 0x367e914b, 66 },
    { 0xe19e914b, 34 },
    { 0x9dfe914c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463e9157, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7e915b, 55 },
    { 0xc80e915b, 90 },
    { 0x963e915c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ee9161, 85 },
    { 0, 0 },
    { 0x788e9163, 75 },
    { 0, 0 },
    { 0x8d3e9165, 95 },
    { 0, 0 },
    { 0xc83e9167, 69 },
    { 0, 0 },
    { 0x18ae9169, 76 },
    { 0xa93e916a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee916e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733e9172, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e917a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e917e, 100 },
    { 0xa75e917e, 34 },
    { 0xc35e9180, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e9183, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763e9187, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae918a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e918d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e9192, 55 },
    { 0xaa4e9193, 100 },
    { 0xed5e9192, 61 },
    { 0, 0 },
    { 0x6c7e9196, 98 },
    { 0x604e9196, 55 },
    { 0x961e9197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931e919c, 43 },
    { 0, 0 },
    { 0xa3ae919e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbe91a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de91a6, 95 },
    { 0x42be91a6, 34 },
    { 0, 0 },
    { 0x357e91a9, 61 },
    { 0xd19e91a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749e91ae, 90 },
    { 0x83e91ae, 90 },
    { 0, 0 },
    { 0x65ae91b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47e91b4, 34 },
    { 0, 0 },
    { 0x820e91b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76e91b9, 91 },
    { 0, 0 },
    { 0xd60e91bb, 66 },
    { 0, 0 },
    { 0x6de91bd, 46 },
    { 0xfe91bd, 55 },
    { 0, 0 },
    { 0x8fe91c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde91ce, 82 },
    { 0xba0e91cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x596e91d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e91d7, 95 },
    { 0x41e91d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7e91de, 85 },
    { 0x1e3e91df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66e91ed, 33 },
    { 0x634e91ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738e91f4, 23 },
    { 0, 0 },
    { 0xf3de91f6, 78 },
    { 0x8d9e91f7, 61 },
    { 0xd40e91f6, 33 },
    { 0, 0 },
    { 0x9d7e91fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e9201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de9207, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ce920d, 90 },
    { 0, 0 },
    { 0x247e920f, 61 },
    { 0xcb5e9210, 84 },
    { 0x190e9211, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e9217, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e921a, 23 },
    { 0x3bfe921b, 99 },
    { 0, 0 },
    { 0xde6e921d, 88 },
    { 0xd6be921e, 76 },
    { 0x1d2e921f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ae9226, 91 },
    { 0xcafe9227, 98 },
    { 0xf74e9228, 90 },
    { 0xb95e9229, 76 },
    { 0, 0 },
    { 0x617e922b, 23 },
    { 0x544e922c, 99 },
    { 0x405e922d, 61 },
    { 0xffde922e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fae9231, 95 },
    { 0, 0 },
    { 0x197e9233, 55 },
    { 0, 0 },
    { 0xd1ce9235, 61 },
    { 0, 0 },
    { 0xf8fe9237, 78 },
    { 0x3b6e9238, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e923b, 97 },
    { 0xdfbe923c, 95 },
    { 0xb16e923d, 55 },
    { 0xf8e923c, 95 },
    { 0x965e923c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae9242, 23 },
    { 0, 0 },
    { 0xa7de9244, 23 },
    { 0xc28e9245, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ae9248, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8e925a, 76 },
    { 0xa44e925b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e9260, 46 },
    { 0xd0fe9261, 90 },
    { 0xfa6e9262, 55 },
    { 0xd27e9261, 33 },
    { 0, 0 },
    { 0x936e9265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e9269, 43 },
    { 0, 0 },
    { 0x2de926b, 91 },
    { 0, 0 },
    { 0x1a4e926d, 65 },
    { 0, 0 },
    { 0x4e2e926f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66e9273, 34 },
    { 0xb32e9274, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41de9278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4e927f, 61 },
    { 0x3a1e927f, 33 },
    { 0, 0 },
    { 0xec9e9282, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e9287, 90 },
    { 0x6ce9287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14e928b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e928e, 46 },
    { 0x4a8e928e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0e9292, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6e9296, 99 },
    { 0x1c2e9297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcece929b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452e92a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6e92a5, 46 },
    { 0x5ace92a6, 90 },
    { 0, 0 },
    { 0x3d8e92a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635e92ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ee92af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e92b2, 23 },
    { 0xc03e92b3, 55 },
    { 0, 0 },
    { 0x6f1e92b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e92b9, 99 },
    { 0xa1ce92ba, 17 },
    { 0x204e92bb, 90 },
    { 0x7bfe92bb, 67 },
    { 0xae0e92bc, 52 },
    { 0x97ce92bb, 23 },
    { 0x2f3e92bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e92c9, 87 },
    { 0, 0 },
    { 0x38ae92cb, 33 },
    { 0, 0 },
    { 0xc4de92cd, 43 },
    { 0xf8de92ce, 78 },
    { 0x70ce92cf, 91 },
    { 0xb3ce92d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844e92e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe92e7, 95 },
    { 0, 0 },
    { 0x924e92e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8e92ee, 23 },
    { 0xe68e92ef, 95 },
    { 0xad1e92f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ae92f3, 76 },
    { 0, 0 },
    { 0x885e92f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e92f8, 90 },
    { 0xfb9e92f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce92fd, 95 },
    { 0x43fe92fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e9303, 95 },
    { 0x8a4e9304, 95 },
    { 0x81fe9305, 90 },
    { 0x533e9306, 95 },
    { 0xf5ae9306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x990e930a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8e9313, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597e9317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e931c, 95 },
    { 0x6a1e931c, 98 },
    { 0x5fce931d, 90 },
    { 0xd44e931f, 90 },
    { 0x45be931d, 90 },
    { 0xa8ee9321, 90 },
    { 0x6aae9322, 43 },
    { 0xd90e9323, 34 },
    { 0x82ee9324, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dae9334, 90 },
    { 0xfa1e9335, 23 },
    { 0x19ae9336, 34 },
    { 0x450e9337, 23 },
    { 0x369e9337, 90 },
    { 0x992e9339, 81 },
    { 0x3f1e933a, 87 },
    { 0, 0 },
    { 0xa52e933c, 34 },
    { 0x147e933d, 91 },
    { 0xe22e933d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee9345, 90 },
    { 0x214e9345, 90 },
    { 0x4b6e9347, 55 },
    { 0, 0 },
    { 0xd82e9349, 23 },
    { 0, 0 },
    { 0x202e934b, 33 },
    { 0x6d6e934b, 58 },
    { 0x5d9e934b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7e9351, 61 },
    { 0, 0 },
    { 0xbb2e9353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6e9356, 46 },
    { 0, 0 },
    { 0x50de9358, 52 },
    { 0, 0 },
    { 0x37be935a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e935d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e9368, 76 },
    { 0xd52e9369, 76 },
    { 0xc43e936a, 98 },
    { 0x74e9369, 90 },
    { 0x886e936c, 55 },
    { 0, 0 },
    { 0xb15e936e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e9373, 61 },
    { 0xd65e9373, 76 },
    { 0x7dbe9375, 34 },
    { 0xdc7e9375, 68 },
    { 0x645e9377, 43 },
    { 0x7c2e9375, 76 },
    { 0xa2de9379, 100 },
    { 0, 0 },
    { 0x1b3e937b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ee937f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9382, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946e9388, 61 },
    { 0xb66e9389, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12e938c, 90 },
    { 0x476e938c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e9390, 78 },
    { 0xd47e9390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e9398, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aee93a0, 88 },
    { 0x8c5e93a1, 81 },
    { 0xac9e93a0, 97 },
    { 0x29e93a1, 23 },
    { 0xdf7e93a0, 33 },
    { 0x77ce93a2, 83 },
    { 0, 0 },
    { 0x35ee93a7, 23 },
    { 0x94ee93a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e93ab, 56 },
    { 0x190e93ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de93b2, 84 },
    { 0xd30e93b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18e93b9, 95 },
    { 0x405e93ba, 40 },
    { 0xaa2e93bb, 98 },
    { 0x67ee93bb, 61 },
    { 0, 0 },
    { 0x5b9e93be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1e93c2, 87 },
    { 0, 0 },
    { 0x734e93c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e93c7, 98 },
    { 0, 0 },
    { 0xac4e93c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30e93d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1e93d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace93df, 81 },
    { 0xa36e93df, 46 },
    { 0xe7be93e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbe93e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e93e8, 40 },
    { 0, 0 },
    { 0x310e93ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ee93ee, 33 },
    { 0, 0 },
    { 0xf88e93f0, 98 },
    { 0, 0 },
    { 0xfc8e93f2, 90 },
    { 0xb8ee93f3, 99 },
    { 0x374e93f4, 90 },
    { 0xcc6e93f2, 31 },
    { 0x145e93f6, 33 },
    { 0x1fae93f7, 61 },
    { 0x980e93f8, 64 },
    { 0x92ce93f9, 75 },
    { 0x3c2e93fa, 34 },
    { 0xe4de93f7, 98 },
    { 0xe65e93fb, 95 },
    { 0xef1e93fd, 36 },
    { 0xd96e93f7, 34 },
    { 0, 0 },
    { 0xcdae9400, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4e9404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9e9408, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae940c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de940f, 46 },
    { 0, 0 },
    { 0xa34e9411, 68 },
    { 0, 0 },
    { 0x251e9413, 76 },
    { 0, 0 },
    { 0x12de9415, 99 },
    { 0xb98e9415, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e941a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85e941f, 81 },
    { 0x65fe9420, 42 },
    { 0, 0 },
    { 0xa31e9422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae9426, 33 },
    { 0, 0 },
    { 0xa5de9428, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0e942b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aee942f, 96 },
    { 0x212e9430, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9e9433, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87e9439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e943c, 90 },
    { 0, 0 },
    { 0x27ee943e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e9441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51be9447, 91 },
    { 0xe62e9447, 34 },
    { 0x60ce9449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e944c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0e9452, 93 },
    { 0, 0 },
    { 0xfcde9454, 55 },
    { 0, 0 },
    { 0x1cae9456, 84 },
    { 0xad5e9457, 87 },
    { 0xc47e9458, 98 },
    { 0x954e9459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e945d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38e9461, 55 },
    { 0x8aee9461, 90 },
    { 0x258e9462, 59 },
    { 0xec3e9464, 76 },
    { 0xdd9e9462, 95 },
    { 0x7a9e9466, 34 },
    { 0xd6be9466, 90 },
    { 0x89ce9468, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e946b, 17 },
    { 0, 0 },
    { 0xb3de946d, 93 },
    { 0xc9de946d, 61 },
    { 0xfe3e946e, 23 },
    { 0xcf0e9470, 55 },
    { 0xa48e9470, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fe947b, 33 },
    { 0x4e4e947b, 61 },
    { 0, 0 },
    { 0x746e947e, 95 },
    { 0, 0 },
    { 0x178e9480, 61 },
    { 0x33ae9480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4e948c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e9490, 95 },
    { 0xbd4e9491, 61 },
    { 0, 0 },
    { 0x230e9493, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2e9497, 98 },
    { 0x941e9498, 23 },
    { 0x9ce9498, 95 },
    { 0xd87e9498, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7e949f, 87 },
    { 0xa5ee94a0, 61 },
    { 0xd06e94a1, 33 },
    { 0x3f7e94a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e94a5, 97 },
    { 0x132e94a6, 33 },
    { 0x78fe94a6, 84 },
    { 0x30ce94a8, 58 },
    { 0, 0 },
    { 0xc46e94aa, 90 },
    { 0x48ce94ab, 61 },
    { 0, 0 },
    { 0xb2de94ad, 43 },
    { 0, 0 },
    { 0x81fe94af, 34 },
    { 0, 0 },
    { 0xe76e94b1, 33 },
    { 0, 0 },
    { 0xb3ee94b3, 98 },
    { 0x6ede94b4, 61 },
    { 0, 0 },
    { 0xaf5e94b6, 61 },
    { 0, 0 },
    { 0xbece94b8, 33 },
    { 0x750e94b9, 90 },
    { 0x45e94ba, 65 },
    { 0x466e94bb, 43 },
    { 0xc6fe94bb, 33 },
    { 0x864e94bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1e94c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46e94c3, 34 },
    { 0, 0 },
    { 0x8d5e94c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e94ca, 55 },
    { 0x2bbe94ca, 95 },
    { 0, 0 },
    { 0x9c4e94cd, 17 },
    { 0xb0de94ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbe94d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee94dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140e94e1, 100 },
    { 0xdcae94e2, 23 },
    { 0xb8fe94e3, 66 },
    { 0, 0 },
    { 0x8eae94e5, 87 },
    { 0x3dbe94e6, 95 },
    { 0xa2de94e7, 33 },
    { 0xd92e94e8, 98 },
    { 0x8fee94e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53e94ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186e94f3, 52 },
    { 0x3ae94f4, 61 },
    { 0x805e94f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce94f9, 38 },
    { 0xb09e94f9, 23 },
    { 0, 0 },
    { 0x9cbe94fc, 56 },
    { 0xf93e94fd, 87 },
    { 0x501e94fe, 55 },
    { 0xaae94fd, 69 },
    { 0x573e94fe, 87 },
    { 0xa65e94ff, 33 },
    { 0, 0 },
    { 0xfe7e9503, 61 },
    { 0x113e9503, 61 },
    { 0x85ce9505, 61 },
    { 0, 0 },
    { 0x16ee9507, 33 },
    { 0, 0 },
    { 0x432e9509, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e950e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e9512, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75e951a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce951d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7be9521, 98 },
    { 0x805e9522, 55 },
    { 0x1a8e9523, 90 },
    { 0x52fe9524, 66 },
    { 0xbdfe9525, 61 },
    { 0x8f1e9521, 90 },
    { 0, 0 },
    { 0x416e9528, 46 },
    { 0xd77e9528, 61 },
    { 0, 0 },
    { 0x3dfe952b, 75 },
    { 0xbc2e952c, 84 },
    { 0x115e952d, 61 },
    { 0x97de952d, 75 },
    { 0x416e952f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e9532, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce9539, 98 },
    { 0x951e953a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e953f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be9544, 76 },
    { 0x8fce9544, 46 },
    { 0xe8ae9544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8e954a, 85 },
    { 0x5c2e954b, 78 },
    { 0x723e954c, 23 },
    { 0, 0 },
    { 0xc57e954e, 61 },
    { 0x928e954e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4e9552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0e9558, 87 },
    { 0, 0 },
    { 0xcbe955a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6e955e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe9565, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9e956d, 33 },
    { 0, 0 },
    { 0x193e956f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e9575, 52 },
    { 0xcf4e9576, 17 },
    { 0x9c4e9576, 95 },
    { 0xb19e9578, 84 },
    { 0, 0 },
    { 0x19ee957a, 100 },
    { 0x33fe957b, 23 },
    { 0x54ee957c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ade9580, 52 },
    { 0x570e9581, 23 },
    { 0xec0e9581, 33 },
    { 0xc1ee9583, 77 },
    { 0xb82e9583, 43 },
    { 0x81ae9585, 43 },
    { 0x952e9586, 61 },
    { 0x599e9587, 66 },
    { 0xf91e9588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be958d, 43 },
    { 0x538e958e, 46 },
    { 0x1a7e958f, 38 },
    { 0x2f8e958f, 61 },
    { 0x50e9590, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae9597, 17 },
    { 0xf8de9597, 34 },
    { 0xb84e9599, 33 },
    { 0, 0 },
    { 0xb3e959b, 90 },
    { 0, 0 },
    { 0x977e959d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x378e95a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4e95a3, 98 },
    { 0xc7de95a4, 52 },
    { 0x1dde95a4, 52 },
    { 0, 0 },
    { 0x6f1e95a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde95aa, 91 },
    { 0x7ece95ab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae95ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e95b7, 90 },
    { 0xf15e95b8, 99 },
    { 0xf39e95b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdece95bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950e95c1, 55 },
    { 0xfade95c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe95c7, 91 },
    { 0, 0 },
    { 0xdb9e95c9, 77 },
    { 0x6c7e95ca, 68 },
    { 0x5e6e95c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x134e95ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86e95d3, 61 },
    { 0, 0 },
    { 0x798e95d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e95d8, 90 },
    { 0x34fe95d9, 90 },
    { 0x8e9e95d8, 23 },
    { 0x3a7e95db, 23 },
    { 0x692e95dc, 85 },
    { 0xf9ee95dd, 99 },
    { 0, 0 },
    { 0x75be95df, 85 },
    { 0x207e95df, 95 },
    { 0xf01e95e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523e95e8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e95ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e95f1, 87 },
    { 0, 0 },
    { 0x4ce95f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e95f6, 43 },
    { 0xe95e95f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e95fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9e95fe, 87 },
    { 0x702e95fe, 33 },
    { 0, 0 },
    { 0x174e9601, 61 },
    { 0, 0 },
    { 0x3e9603, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfe960a, 61 },
    { 0, 0 },
    { 0xce7e960c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ce9611, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e961c, 90 },
    { 0xda0e961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e9623, 23 },
    { 0, 0 },
    { 0xc6fe9625, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be9632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e9635, 90 },
    { 0, 0 },
    { 0x64ce9637, 88 },
    { 0x8d4e9638, 17 },
    { 0x41de9639, 23 },
    { 0x47de963a, 34 },
    { 0, 0 },
    { 0xa5be963c, 93 },
    { 0x4ee963d, 98 },
    { 0x8be963d, 77 },
    { 0xfbe963f, 95 },
    { 0xc28e963c, 61 },
    { 0, 0 },
    { 0xf39e9642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ee9649, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8e964d, 33 },
    { 0x711e964d, 36 },
    { 0x2f1e964f, 98 },
    { 0x786e964d, 98 },
    { 0, 0 },
    { 0xe13e9652, 90 },
    { 0xe76e9653, 76 },
    { 0, 0 },
    { 0xc8e9655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643e965a, 61 },
    { 0xf50e965a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0e965e, 61 },
    { 0x6aae965e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15e9662, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e9667, 95 },
    { 0xfcae9668, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0e966c, 98 },
    { 0x443e966c, 61 },
    { 0x253e966e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7e9672, 61 },
    { 0, 0 },
    { 0xa90e9674, 90 },
    { 0xf7ae9675, 85 },
    { 0x751e9675, 33 },
    { 0xcf0e9677, 17 },
    { 0xebfe9678, 34 },
    { 0x9f1e9675, 61 },
    { 0x729e967a, 90 },
    { 0xe94e967a, 61 },
    { 0x47e967c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae9680, 55 },
    { 0xe59e9681, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e968a, 61 },
    { 0, 0 },
    { 0xf70e968c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee968f, 95 },
    { 0xbae9690, 33 },
    { 0, 0 },
    { 0x849e9692, 93 },
    { 0xc04e9693, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1e96a3, 90 },
    { 0, 0 },
    { 0xc71e96a5, 88 },
    { 0x294e96a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e96a9, 95 },
    { 0x725e96a9, 75 },
    { 0xac9e96ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ee96ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e96b1, 76 },
    { 0, 0 },
    { 0x449e96b3, 91 },
    { 0x109e96b3, 61 },
    { 0x809e96b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0be96bb, 61 },
    { 0xea1e96bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e96c3, 43 },
    { 0, 0 },
    { 0xd06e96c5, 95 },
    { 0, 0 },
    { 0xda8e96c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263e96cd, 98 },
    { 0x42e96cd, 98 },
    { 0, 0 },
    { 0x692e96d0, 33 },
    { 0, 0 },
    { 0x8a7e96d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e96d6, 88 },
    { 0x99fe96d6, 55 },
    { 0xbd5e96d8, 33 },
    { 0xc38e96d8, 98 },
    { 0xe95e96d6, 66 },
    { 0, 0 },
    { 0xb4ce96dc, 98 },
    { 0xf81e96dc, 84 },
    { 0x8bde96dd, 55 },
    { 0xf95e96df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fe96e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01e96e9, 38 },
    { 0, 0 },
    { 0x60fe96eb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x734e96ee, 85 },
    { 0x4e7e96ef, 90 },
    { 0xb7ae96f0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45e96f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494e96f7, 46 },
    { 0x24de96f8, 98 },
    { 0xfa7e96f8, 95 },
    { 0, 0 },
    { 0xc22e96fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e96ff, 95 },
    { 0xe99e9700, 90 },
    { 0, 0 },
    { 0x5e8e9702, 90 },
    { 0, 0 },
    { 0x83ae9704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ae9707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7e970a, 90 },
    { 0, 0 },
    { 0x6fde970c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7e9711, 95 },
    { 0, 0 },
    { 0x408e9713, 90 },
    { 0, 0 },
    { 0xa67e9715, 34 },
    { 0, 0 },
    { 0x5a6e9717, 23 },
    { 0x55ce9718, 90 },
    { 0x998e9719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5e971c, 78 },
    { 0, 0 },
    { 0x517e971e, 33 },
    { 0x809e971e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e9723, 46 },
    { 0, 0 },
    { 0x1fce9725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e9728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8e972f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbe9732, 95 },
    { 0, 0 },
    { 0xcb7e9734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e9739, 97 },
    { 0x6e9e9739, 85 },
    { 0, 0 },
    { 0xadee973c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde9741, 23 },
    { 0x7bee9742, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbe9747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9e974e, 46 },
    { 0x551e974f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe9754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e975a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8e9760, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae976c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43e9773, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x429e9776, 23 },
    { 0x87ae9777, 33 },
    { 0x4d8e9778, 33 },
    { 0, 0 },
    { 0x5cde977a, 95 },
    { 0x8a6e977a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee977f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e9782, 46 },
    { 0xbdbe9783, 84 },
    { 0xf58e9784, 99 },
    { 0x5f4e9784, 61 },
    { 0, 0 },
    { 0x197e9787, 33 },
    { 0x4d9e9788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de978c, 61 },
    { 0, 0 },
    { 0xa19e978e, 61 },
    { 0x535e978f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e9793, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbe9797, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88e979e, 85 },
    { 0xe31e979f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4e97a4, 75 },
    { 0x68ce97a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeee97a9, 33 },
    { 0x9c9e97a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de97b6, 95 },
    { 0, 0 },
    { 0x9f2e97b8, 23 },
    { 0xb4de97b9, 95 },
    { 0, 0 },
    { 0x2f8e97bb, 97 },
    { 0x6a5e97bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e97c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ce97c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e97cb, 34 },
    { 0, 0 },
    { 0xfebe97cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8e97d2, 93 },
    { 0, 0 },
    { 0xc18e97d4, 98 },
    { 0, 0 },
    { 0xe5be97d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43e97da, 33 },
    { 0xb3fe97db, 88 },
    { 0x9efe97da, 61 },
    { 0xe8be97dd, 90 },
    { 0x948e97de, 43 },
    { 0xceee97df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468e97e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce97e8, 93 },
    { 0x285e97e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2e97ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e97f0, 27 },
    { 0xc96e97f1, 76 },
    { 0x799e97f2, 90 },
    { 0x30de97f2, 55 },
    { 0xc5ae97f4, 23 },
    { 0x99e97f4, 95 },
    { 0, 0 },
    { 0x10ee97f7, 98 },
    { 0xf33e97f7, 33 },
    { 0, 0 },
    { 0x5abe97fa, 91 },
    { 0x878e97fb, 33 },
    { 0xc7fe97fb, 33 },
    { 0x7e9e97fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2e9801, 61 },
    { 0xf43e9802, 61 },
    { 0x478e9803, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9e9809, 61 },
    { 0x66ee9809, 90 },
    { 0xe67e980b, 98 },
    { 0x370e980c, 33 },
    { 0x649e980d, 98 },
    { 0xc9e980c, 90 },
    { 0xcd2e980e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287e9813, 87 },
    { 0xf09e9814, 33 },
    { 0xb64e9813, 90 },
    { 0, 0 },
    { 0x348e9817, 40 },
    { 0x2fbe9817, 34 },
    { 0, 0 },
    { 0x645e981a, 90 },
    { 0, 0 },
    { 0x687e981c, 55 },
    { 0x934e981c, 61 },
    { 0x7f2e981c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356e9825, 61 },
    { 0x2d0e9826, 23 },
    { 0x2b0e9825, 43 },
    { 0x7ede9828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfe982b, 95 },
    { 0xbade982c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21e9830, 61 },
    { 0x3d5e9830, 23 },
    { 0x9e9832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ae9835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde9838, 23 },
    { 0x179e9839, 99 },
    { 0xfbfe9839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6e983e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9844, 55 },
    { 0xd90e9845, 81 },
    { 0x89fe9845, 76 },
    { 0, 0 },
    { 0x3fae9848, 95 },
    { 0x2b8e9849, 100 },
    { 0xbf5e984a, 33 },
    { 0, 0 },
    { 0x7f2e984c, 33 },
    { 0xb63e984d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4e9851, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee9858, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69be9860, 61 },
    { 0x508e9860, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddae9865, 90 },
    { 0, 0 },
    { 0xf03e9867, 45 },
    { 0x702e9867, 90 },
    { 0xfcbe9869, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ce986c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cce9873, 81 },
    { 0, 0 },
    { 0x171e9875, 90 },
    { 0x770e9876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffe987e, 95 },
    { 0, 0 },
    { 0xd51e9880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e9886, 55 },
    { 0xb74e9887, 84 },
    { 0, 0 },
    { 0xa9be9889, 33 },
    { 0xb79e988a, 23 },
    { 0xf6ce988b, 33 },
    { 0x87ae988b, 67 },
    { 0, 0 },
    { 0xd73e988e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe9891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e9894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01e9897, 66 },
    { 0xc9ee9898, 95 },
    { 0x823e9899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e98a0, 55 },
    { 0x2cce98a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe98a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25e98a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5e98ab, 84 },
    { 0xfcae98ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892e98b4, 93 },
    { 0x4ede98b5, 52 },
    { 0, 0 },
    { 0xe3be98b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1e98bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x642e98be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6e98c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae98ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e98d3, 90 },
    { 0, 0 },
    { 0x6f1e98d5, 17 },
    { 0xec1e98d6, 98 },
    { 0x7bee98d7, 76 },
    { 0x820e98d8, 55 },
    { 0xd45e98d9, 78 },
    { 0x262e98d9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fce98dd, 95 },
    { 0xf73e98de, 23 },
    { 0x3d9e98df, 90 },
    { 0, 0 },
    { 0xfabe98e1, 52 },
    { 0x698e98e1, 19 },
    { 0x656e98e1, 66 },
    { 0xbe98e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x787e98e7, 34 },
    { 0, 0 },
    { 0x5afe98e9, 84 },
    { 0x855e98ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85e98ee, 95 },
    { 0, 0 },
    { 0xca5e98f0, 33 },
    { 0x366e98f0, 87 },
    { 0x838e98f2, 55 },
    { 0x9b1e98f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26e98f6, 84 },
    { 0x891e98f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e98fc, 76 },
    { 0, 0 },
    { 0xaf9e98fe, 98 },
    { 0, 0 },
    { 0x89fe9900, 55 },
    { 0, 0 },
    { 0x4aae9902, 34 },
    { 0, 0 },
    { 0xb54e9904, 76 },
    { 0x499e9905, 90 },
    { 0, 0 },
    { 0xbc2e9907, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cce990d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881e9913, 67 },
    { 0, 0 },
    { 0xc0be9915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde9919, 23 },
    { 0x541e991a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x658e991d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ae9921, 32 },
    { 0x9d6e9922, 98 },
    { 0, 0 },
    { 0x87e9924, 61 },
    { 0, 0 },
    { 0x2e3e9926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9929, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bce992c, 95 },
    { 0x3c6e992c, 95 },
    { 0x520e992e, 90 },
    { 0xbcce992d, 100 },
    { 0x9dfe9930, 90 },
    { 0xb5be9931, 76 },
    { 0, 0 },
    { 0x175e9933, 61 },
    { 0x198e9934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e9937, 96 },
    { 0x575e9938, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37e993d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e9940, 33 },
    { 0, 0 },
    { 0x6e3e9942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e9945, 24 },
    { 0x178e9946, 43 },
    { 0, 0 },
    { 0x7d5e9948, 23 },
    { 0xa41e9949, 99 },
    { 0x793e994a, 90 },
    { 0x48fe9948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece994f, 23 },
    { 0x5ae994f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e9954, 33 },
    { 0x51de9955, 65 },
    { 0x9e7e9954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e9959, 61 },
    { 0x8eee995a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ce995f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7e9963, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922e9969, 61 },
    { 0x38e996a, 90 },
    { 0xde8e9969, 33 },
    { 0xe65e996a, 90 },
    { 0x405e996d, 17 },
    { 0x2ebe996d, 66 },
    { 0x93e996d, 34 },
    { 0x410e996d, 69 },
    { 0x20de996f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e9978, 55 },
    { 0, 0 },
    { 0x6dfe997a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e9988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e998e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcde9998, 90 },
    { 0, 0 },
    { 0x49ee999a, 61 },
    { 0xec2e999b, 61 },
    { 0, 0 },
    { 0x753e999d, 53 },
    { 0x8a4e999e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e99a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55e99aa, 34 },
    { 0, 0 },
    { 0xd4ae99ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e99af, 55 },
    { 0, 0 },
    { 0x166e99b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x277e99b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987e99bc, 46 },
    { 0xfffe99bc, 48 },
    { 0xb0ce99be, 90 },
    { 0x67ce99bf, 17 },
    { 0xd84e99bf, 61 },
    { 0x3afe99c1, 23 },
    { 0, 0 },
    { 0x83fe99c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede99c6, 93 },
    { 0x110e99c7, 67 },
    { 0xc1ae99c8, 55 },
    { 0x5dae99c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08e99ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fe99d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be99db, 66 },
    { 0xc6e99dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e99df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fe99e2, 90 },
    { 0xe29e99e3, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e99e8, 61 },
    { 0xface99e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1e99ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde99f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece99f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e99f9, 95 },
    { 0x9e6e99fa, 100 },
    { 0, 0 },
    { 0xe63e99fc, 33 },
    { 0x650e99fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce9a01, 55 },
    { 0, 0 },
    { 0xd55e9a03, 76 },
    { 0x5e3e9a03, 33 },
    { 0x1dfe9a04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e9a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde9a12, 46 },
    { 0xd38e9a13, 33 },
    { 0x841e9a13, 23 },
    { 0xf4e9a12, 33 },
    { 0xc33e9a15, 46 },
    { 0xc52e9a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfe9a1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e9a1e, 40 },
    { 0, 0 },
    { 0x672e9a20, 80 },
    { 0xaf3e9a21, 77 },
    { 0, 0 },
    { 0xa6ee9a23, 95 },
    { 0, 0 },
    { 0xe86e9a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629e9a30, 90 },
    { 0, 0 },
    { 0x352e9a32, 61 },
    { 0x4b7e9a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269e9a3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e9a42, 99 },
    { 0xa02e9a42, 85 },
    { 0x164e9a43, 90 },
    { 0x13ee9a44, 90 },
    { 0x429e9a46, 90 },
    { 0x30fe9a47, 33 },
    { 0x822e9a48, 33 },
    { 0, 0 },
    { 0xb2fe9a4a, 55 },
    { 0xcf8e9a4b, 84 },
    { 0xbd9e9a4b, 33 },
    { 0, 0 },
    { 0x11ce9a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4e9a54, 33 },
    { 0x5c7e9a54, 43 },
    { 0xd38e9a54, 34 },
    { 0x29de9a57, 75 },
    { 0xf62e9a58, 95 },
    { 0, 0 },
    { 0x825e9a5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee9a5d, 94 },
    { 0, 0 },
    { 0x716e9a5f, 61 },
    { 0, 0 },
    { 0x349e9a61, 78 },
    { 0, 0 },
    { 0x19de9a63, 23 },
    { 0xb36e9a64, 90 },
    { 0xf7ee9a64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597e9a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9a6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3e9a70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5e9a75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x968e9a78, 76 },
    { 0xabee9a79, 55 },
    { 0x82fe9a7a, 66 },
    { 0x6cee9a7b, 76 },
    { 0xd1ae9a7c, 61 },
    { 0x63ce9a7c, 98 },
    { 0, 0 },
    { 0xffee9a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fce9a82, 61 },
    { 0xeede9a82, 33 },
    { 0, 0 },
    { 0x231e9a85, 100 },
    { 0x41fe9a85, 98 },
    { 0xc5ee9a87, 64 },
    { 0x622e9a88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e9a8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ee9a8e, 22 },
    { 0, 0 },
    { 0xe2e9a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ee9a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaee9a9a, 55 },
    { 0xc0ce9a9a, 85 },
    { 0xf5be9a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae9aa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffe9aa6, 93 },
    { 0x4be9aa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fe9aac, 38 },
    { 0xb94e9aad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e9ab4, 43 },
    { 0x10fe9ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90e9ab8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e9abe, 95 },
    { 0xd7de9abe, 85 },
    { 0x81de9ac0, 98 },
    { 0x6dae9ac1, 98 },
    { 0x518e9ac1, 55 },
    { 0xf63e9ac0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2e9ac7, 36 },
    { 0, 0 },
    { 0xecee9ac9, 81 },
    { 0, 0 },
    { 0x357e9acb, 95 },
    { 0xd26e9acc, 81 },
    { 0, 0 },
    { 0x1afe9ace, 90 },
    { 0x488e9ace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ace9ad4, 98 },
    { 0xbfee9ad4, 95 },
    { 0xdbfe9ad6, 61 },
    { 0x144e9ad7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9de9adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e9ade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae9ae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde9ae9, 43 },
    { 0x9ce9aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde9af0, 90 },
    { 0x41ae9af0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389e9af5, 90 },
    { 0, 0 },
    { 0x55de9af7, 90 },
    { 0xadde9af7, 43 },
    { 0, 0 },
    { 0x57ae9afa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bce9afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ce9b01, 80 },
    { 0, 0 },
    { 0x95be9b03, 76 },
    { 0x683e9b04, 61 },
    { 0xb79e9b05, 76 },
    { 0x6c7e9b05, 61 },
    { 0x370e9b07, 91 },
    { 0, 0 },
    { 0x8fbe9b09, 43 },
    { 0, 0 },
    { 0xe69e9b0b, 90 },
    { 0, 0 },
    { 0xb4de9b0d, 52 },
    { 0, 0 },
    { 0x2fbe9b0f, 23 },
    { 0x93be9b10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aae9b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ace9b20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77e9b23, 49 },
    { 0x150e9b24, 95 },
    { 0xbcde9b24, 34 },
    { 0, 0 },
    { 0x3fde9b27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ede9b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be9b32, 95 },
    { 0, 0 },
    { 0x1b4e9b34, 76 },
    { 0, 0 },
    { 0x9e8e9b36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e9b39, 33 },
    { 0x56ee9b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e9b3e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde9b42, 87 },
    { 0, 0 },
    { 0x303e9b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade9b47, 23 },
    { 0, 0 },
    { 0xe55e9b49, 100 },
    { 0xe1ee9b4a, 80 },
    { 0, 0 },
    { 0xf02e9b4c, 23 },
    { 0, 0 },
    { 0xe13e9b4e, 33 },
    { 0, 0 },
    { 0x9c9e9b50, 95 },
    { 0, 0 },
    { 0x2dde9b52, 34 },
    { 0xcd4e9b53, 84 },
    { 0x407e9b54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x802e9b57, 78 },
    { 0x8cbe9b58, 66 },
    { 0xc28e9b59, 61 },
    { 0, 0 },
    { 0x9f4e9b5b, 61 },
    { 0, 0 },
    { 0xecbe9b5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde9b61, 23 },
    { 0x14ee9b62, 98 },
    { 0xee4e9b63, 95 },
    { 0x9a9e9b63, 43 },
    { 0, 0 },
    { 0x49fe9b66, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1e9b6b, 33 },
    { 0x18de9b6b, 55 },
    { 0, 0 },
    { 0x20ae9b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ee9b75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae9b78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e9b7c, 98 },
    { 0, 0 },
    { 0xdaee9b7e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ae9b83, 61 },
    { 0, 0 },
    { 0x662e9b85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73e9b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9b8d, 95 },
    { 0x62ae9b8e, 98 },
    { 0, 0 },
    { 0x894e9b90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755e9b94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e9b9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75e9b9e, 99 },
    { 0x27be9b9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86e9ba2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce9ba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe9ba8, 33 },
    { 0, 0 },
    { 0xe89e9baa, 98 },
    { 0x888e9baa, 90 },
    { 0x3f9e9bab, 61 },
    { 0, 0 },
    { 0x9b0e9bae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e9bb2, 90 },
    { 0x875e9bb3, 43 },
    { 0x571e9bb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7e9bb7, 76 },
    { 0x29ce9bb7, 61 },
    { 0x1c2e9bb9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fe9bbc, 55 },
    { 0, 0 },
    { 0xc42e9bbe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2e9bc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0e9bc7, 52 },
    { 0, 0 },
    { 0x2fce9bc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104e9bce, 98 },
    { 0x740e9bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2be9bdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ce9be1, 43 },
    { 0x602e9be2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e9be9, 90 },
    { 0x112e9bea, 33 },
    { 0x2d3e9bea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9bf0, 90 },
    { 0xd0ce9bf1, 61 },
    { 0x828e9bf1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64e9bf7, 88 },
    { 0xfb4e9bf8, 23 },
    { 0xf7ee9bf9, 23 },
    { 0x995e9bfa, 88 },
    { 0x52ae9bfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e9bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e9c03, 87 },
    { 0, 0 },
    { 0xe81e9c05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cae9c0b, 95 },
    { 0xae9c0c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e9c14, 99 },
    { 0x51be9c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce9c1b, 23 },
    { 0, 0 },
    { 0xe5de9c1d, 91 },
    { 0x560e9c1e, 43 },
    { 0x9dce9c1d, 76 },
    { 0xdbee9c20, 98 },
    { 0x504e9c20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3e9c2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7e9c30, 92 },
    { 0xbdae9c30, 90 },
    { 0x1a3e9c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebee9c36, 98 },
    { 0x148e9c36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0e9c40, 43 },
    { 0x182e9c41, 23 },
    { 0x408e9c41, 33 },
    { 0xf84e9c42, 33 },
    { 0x27ee9c41, 91 },
    { 0, 0 },
    { 0x68fe9c46, 91 },
    { 0xf1e9c47, 23 },
    { 0x4de9c48, 23 },
    { 0, 0 },
    { 0xf4be9c4a, 91 },
    { 0x6aee9c4b, 69 },
    { 0x63fe9c4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e9c52, 90 },
    { 0x1c4e9c53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e9c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe9c60, 61 },
    { 0x55de9c61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ce9c64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e9c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede9c6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae9c70, 93 },
    { 0xd65e9c71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6e9c74, 23 },
    { 0, 0 },
    { 0xc04e9c76, 93 },
    { 0x8b7e9c77, 98 },
    { 0xa73e9c78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfde9c80, 33 },
    { 0x7f2e9c80, 99 },
    { 0xebfe9c81, 96 },
    { 0x984e9c81, 70 },
    { 0xae5e9c84, 43 },
    { 0xdfce9c84, 36 },
    { 0, 0 },
    { 0x9cae9c87, 95 },
    { 0x844e9c87, 98 },
    { 0x1b3e9c87, 60 },
    { 0xf44e9c88, 96 },
    { 0xcf7e9c87, 61 },
    { 0, 0 },
    { 0x328e9c8d, 8 },
    { 0x50e9c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e9c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce9c99, 99 },
    { 0xf78e9c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e9c9f, 38 },
    { 0, 0 },
    { 0xd4ce9ca1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3e9ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce9cae, 67 },
    { 0x293e9caf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae9cb5, 90 },
    { 0x288e9cb5, 61 },
    { 0, 0 },
    { 0x224e9cb8, 33 },
    { 0x254e9cb9, 61 },
    { 0xbd4e9cb9, 95 },
    { 0xd03e9cbb, 67 },
    { 0x2efe9cb8, 90 },
    { 0xb75e9cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e9cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6e9ccb, 34 },
    { 0xf1ee9ccc, 90 },
    { 0xf9ce9ccd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee9cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae9cd5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbe9cd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e9cdb, 90 },
    { 0xab3e9cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e9ce4, 61 },
    { 0x91ce9ce5, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ae9ce9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee9cef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e9cf2, 100 },
    { 0x2fee9cf3, 98 },
    { 0, 0 },
    { 0x4de9cf5, 88 },
    { 0x3a4e9cf6, 76 },
    { 0xe1ae9cf6, 38 },
    { 0, 0 },
    { 0xcb0e9cf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e9cfd, 61 },
    { 0xd2ce9cfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee9d03, 17 },
    { 0x5c9e9d04, 100 },
    { 0, 0 },
    { 0x914e9d06, 23 },
    { 0x898e9d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e9d0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e9d12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fe9d1d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e9d21, 76 },
    { 0, 0 },
    { 0xba6e9d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x723e9d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65e9d2b, 93 },
    { 0, 0 },
    { 0xfc5e9d2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be9d30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e9d34, 61 },
    { 0x52be9d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575e9d3b, 95 },
    { 0x661e9d3b, 95 },
    { 0xfe1e9d3d, 55 },
    { 0x3bce9d3e, 23 },
    { 0x74ae9d3d, 90 },
    { 0x3cfe9d3f, 34 },
    { 0x96e9d41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fde9d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e9d4d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35e9d53, 55 },
    { 0x64ee9d53, 33 },
    { 0x1fce9d55, 61 },
    { 0, 0 },
    { 0xa7de9d57, 98 },
    { 0xa58e9d57, 33 },
    { 0, 0 },
    { 0x589e9d5a, 43 },
    { 0x77be9d5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e9d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fe9d66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e9d6a, 61 },
    { 0, 0 },
    { 0x7d5e9d6c, 98 },
    { 0xf1be9d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5e9d70, 77 },
    { 0xa9be9d71, 84 },
    { 0, 0 },
    { 0x4cce9d73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0e9d7b, 91 },
    { 0x1b6e9d7c, 90 },
    { 0xf7ae9d7d, 96 },
    { 0xf99e9d7d, 46 },
    { 0xea5e9d7f, 17 },
    { 0xf19e9d7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e9d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9d89, 34 },
    { 0xb88e9d89, 61 },
    { 0x329e9d8b, 100 },
    { 0, 0 },
    { 0x518e9d8d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de9d90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1e9d96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x803e9d99, 92 },
    { 0x615e9d9a, 33 },
    { 0x90de9d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2e9da0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9da5, 33 },
    { 0, 0 },
    { 0x915e9da7, 99 },
    { 0x45be9da7, 90 },
    { 0x765e9da9, 33 },
    { 0x963e9daa, 55 },
    { 0x44fe9da9, 76 },
    { 0x363e9dac, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0e9db3, 59 },
    { 0xe11e9db4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe9db7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ee9dba, 68 },
    { 0, 0 },
    { 0xeb1e9dbc, 90 },
    { 0, 0 },
    { 0x715e9dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3e9dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62e9dc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ee9dd2, 98 },
    { 0x9d3e9dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce9dd9, 90 },
    { 0xa18e9dda, 61 },
    { 0xe2ee9ddb, 90 },
    { 0x96be9ddc, 33 },
    { 0, 0 },
    { 0xe8ee9dde, 98 },
    { 0x9f0e9ddf, 90 },
    { 0x4a9e9ddf, 23 },
    { 0xa8ae9ddf, 66 },
    { 0x966e9de0, 55 },
    { 0xdb4e9de1, 85 },
    { 0x55de9ddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe9de7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3e9df3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e9df7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e9dff, 55 },
    { 0xfe8e9e00, 90 },
    { 0, 0 },
    { 0x45ce9e02, 61 },
    { 0x329e9e03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9be9e0a, 23 },
    { 0, 0 },
    { 0xf51e9e0c, 33 },
    { 0xa03e9e0d, 61 },
    { 0x1e5e9e0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3e9e12, 34 },
    { 0, 0 },
    { 0xf24e9e14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e9e19, 90 },
    { 0, 0 },
    { 0x8ebe9e1b, 90 },
    { 0x620e9e1c, 95 },
    { 0, 0 },
    { 0x33be9e1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e9e21, 23 },
    { 0x52de9e22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x618e9e25, 52 },
    { 0x190e9e25, 23 },
    { 0x91fe9e27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4e9e2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x563e9e2d, 61 },
    { 0x5efe9e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e9e31, 61 },
    { 0x317e9e31, 67 },
    { 0xef8e9e31, 95 },
    { 0, 0 },
    { 0x7be9e35, 55 },
    { 0xb3ae9e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade9e3b, 81 },
    { 0, 0 },
    { 0x7f2e9e3d, 33 },
    { 0xadce9e3e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de9e41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x80be9e44, 55 },
    { 0xcf3e9e45, 43 },
    { 0x418e9e46, 61 },
    { 0x12ee9e47, 90 },
    { 0xf20e9e46, 86 },
    { 0x8ee9e47, 33 },
    { 0x7ade9e47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x100e9e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1e9e55, 66 },
    { 0, 0 },
    { 0x3a1e9e57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fee9e5b, 99 },
    { 0, 0 },
    { 0x6dbe9e5d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e9e63, 90 },
    { 0x63ae9e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e9e67, 68 },
    { 0x3ee9e67, 33 },
    { 0x7b6e9e68, 55 },
    { 0x8e9e6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde9e6f, 56 },
    { 0x7d9e9e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x502e9e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470e9e7a, 84 },
    { 0xfd3e9e7a, 33 },
    { 0xe87e9e7c, 23 },
    { 0x191e9e7d, 59 },
    { 0, 0 },
    { 0xeebe9e7f, 95 },
    { 0x4b0e9e80, 61 },
    { 0x549e9e81, 23 },
    { 0x4dbe9e82, 34 },
    { 0xc0fe9e81, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e9e8c, 61 },
    { 0x19ae9e8d, 46 },
    { 0, 0 },
    { 0x354e9e8f, 23 },
    { 0xd13e9e90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e9e94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430e9e98, 90 },
    { 0, 0 },
    { 0x7f1e9e9a, 93 },
    { 0, 0 },
    { 0x10ce9e9c, 93 },
    { 0x180e9e9c, 88 },
    { 0xcb4e9e9e, 98 },
    { 0xe3de9e9e, 92 },
    { 0x835e9ea0, 88 },
    { 0, 0 },
    { 0xf9ce9ea2, 99 },
    { 0x8f6e9ea2, 34 },
    { 0xda4e9ea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e9ea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495e9eb0, 23 },
    { 0, 0 },
    { 0xefde9eb2, 98 },
    { 0x855e9eb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde9eb6, 76 },
    { 0xdd3e9eb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe9ec4, 55 },
    { 0x194e9ec4, 61 },
    { 0xde0e9ec6, 87 },
    { 0xb44e9ec7, 91 },
    { 0xf00e9ec8, 23 },
    { 0x8dee9ec7, 34 },
    { 0x40e9eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30be9ece, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18e9ed5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x343e9ed8, 76 },
    { 0, 0 },
    { 0x15de9eda, 90 },
    { 0xf98e9edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ee9edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdce9ee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e9ee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabe9eed, 60 },
    { 0x128e9eee, 90 },
    { 0xb24e9eee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be9ef4, 43 },
    { 0x795e9ef5, 33 },
    { 0x443e9ef4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e9ef9, 23 },
    { 0, 0 },
    { 0xd93e9efb, 90 },
    { 0xe40e9efc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe9eff, 66 },
    { 0xe4e9f00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53e9f07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ee9f0a, 76 },
    { 0x3a1e9f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e9f0e, 90 },
    { 0, 0 },
    { 0x54ce9f10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e9f19, 33 },
    { 0, 0 },
    { 0x40be9f1b, 76 },
    { 0xaeae9f1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3e9f20, 90 },
    { 0xc8ce9f21, 70 },
    { 0, 0 },
    { 0x4cde9f23, 43 },
    { 0xe56e9f24, 95 },
    { 0x4d7e9f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19be9f2b, 61 },
    { 0, 0 },
    { 0xd1ae9f2d, 23 },
    { 0x99be9f2e, 90 },
    { 0x350e9f2f, 97 },
    { 0xb3fe9f2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ee9f37, 76 },
    { 0, 0 },
    { 0x274e9f39, 88 },
    { 0x64de9f3a, 52 },
    { 0, 0 },
    { 0x1fe9f3c, 66 },
    { 0, 0 },
    { 0x2a1e9f3e, 93 },
    { 0, 0 },
    { 0x2aae9f40, 40 },
    { 0, 0 },
    { 0xc1e9f42, 95 },
    { 0x87fe9f43, 66 },
    { 0x90de9f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e9f4a, 36 },
    { 0x434e9f4b, 55 },
    { 0xc47e9f4c, 76 },
    { 0xfc2e9f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe9f50, 100 },
    { 0, 0 },
    { 0xa10e9f52, 90 },
    { 0xa29e9f53, 24 },
    { 0xd81e9f54, 65 },
    { 0x8bce9f54, 95 },
    { 0x1ede9f56, 17 },
    { 0xa65e9f57, 90 },
    { 0x285e9f58, 33 },
    { 0xfe6e9f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59e9f5e, 87 },
    { 0x1bde9f5f, 76 },
    { 0x183e9f5f, 81 },
    { 0, 0 },
    { 0xd41e9f62, 52 },
    { 0x678e9f63, 33 },
    { 0, 0 },
    { 0xce1e9f65, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63de9f6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8e9f72, 17 },
    { 0xce8e9f72, 23 },
    { 0x583e9f72, 76 },
    { 0x9fe9f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4e9f7c, 98 },
    { 0xd8e9f7c, 85 },
    { 0x31fe9f7c, 98 },
    { 0xf34e9f7c, 23 },
    { 0x313e9f80, 90 },
    { 0x442e9f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce9f87, 90 },
    { 0, 0 },
    { 0xbb2e9f89, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e9f91, 34 },
    { 0, 0 },
    { 0x609e9f93, 43 },
    { 0xd46e9f94, 50 },
    { 0, 0 },
    { 0x8b4e9f96, 98 },
    { 0x378e9f97, 95 },
    { 0, 0 },
    { 0x72fe9f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e9f9f, 76 },
    { 0xf8be9fa0, 46 },
    { 0xd91e9fa0, 66 },
    { 0x564e9f9f, 90 },
    { 0x538e9fa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e9fa6, 34 },
    { 0x4c5e9fa7, 61 },
    { 0xdc0e9fa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce9fad, 61 },
    { 0x42ce9fae, 95 },
    { 0x935e9fae, 23 },
    { 0, 0 },
    { 0xb0ee9fb1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce9fb7, 55 },
    { 0x71ae9fb8, 95 },
    { 0x890e9fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce9fbc, 2 },
    { 0, 0 },
    { 0x6fbe9fbe, 23 },
    { 0, 0 },
    { 0xc54e9fc0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee9fc3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681e9fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e9fcb, 55 },
    { 0, 0 },
    { 0x449e9fcd, 17 },
    { 0, 0 },
    { 0x194e9fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ae9fd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e9fd7, 100 },
    { 0, 0 },
    { 0x793e9fd9, 84 },
    { 0x5dbe9fda, 65 },
    { 0x667e9fda, 33 },
    { 0, 0 },
    { 0xa18e9fdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e9fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92e9fe9, 90 },
    { 0, 0 },
    { 0x59de9feb, 90 },
    { 0x5c9e9feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ce9ff0, 90 },
    { 0x76ce9ff1, 90 },
    { 0, 0 },
    { 0x6a4e9ff3, 90 },
    { 0, 0 },
    { 0xac1e9ff5, 98 },
    { 0, 0 },
    { 0x820e9ff7, 90 },
    { 0xd49e9ff8, 33 },
    { 0xd0fe9ff7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e9ffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ea005, 61 },
    { 0, 0 },
    { 0xc72ea007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1ea00f, 61 },
    { 0x9efea00f, 17 },
    { 0xb33ea011, 98 },
    { 0xc28ea011, 43 },
    { 0x180ea010, 90 },
    { 0xa3eea014, 90 },
    { 0x20aea012, 33 },
    { 0x8c8ea016, 91 },
    { 0, 0 },
    { 0x89fea018, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55ea01d, 90 },
    { 0, 0 },
    { 0x71aea01f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ea026, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dea02c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adea02f, 90 },
    { 0, 0 },
    { 0x256ea031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeea034, 15 },
    { 0x3d5ea035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2ea03a, 81 },
    { 0xd6eea03b, 61 },
    { 0x98aea03c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ea040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2ea046, 40 },
    { 0, 0 },
    { 0xabdea048, 100 },
    { 0x7bcea049, 61 },
    { 0x529ea048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adea04f, 100 },
    { 0, 0 },
    { 0xae5ea051, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906ea058, 95 },
    { 0, 0 },
    { 0x5d7ea05a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfea062, 61 },
    { 0x4aaea063, 61 },
    { 0x28aea063, 61 },
    { 0x721ea065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaea06a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0ea073, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eea077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782ea07b, 56 },
    { 0xe7bea07b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eea080, 90 },
    { 0x9d0ea081, 98 },
    { 0x3b8ea081, 55 },
    { 0x3ceea082, 23 },
    { 0x2bbea084, 98 },
    { 0x47bea084, 95 },
    { 0xdfeea086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ea08c, 53 },
    { 0x587ea08d, 38 },
    { 0xba6ea08e, 46 },
    { 0xfbfea08f, 40 },
    { 0xf23ea08f, 76 },
    { 0xb3fea091, 61 },
    { 0, 0 },
    { 0xf76ea093, 90 },
    { 0xb9fea094, 33 },
    { 0xf11ea095, 92 },
    { 0, 0 },
    { 0xca3ea097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50ea09d, 84 },
    { 0, 0 },
    { 0x2c4ea09f, 33 },
    { 0xa2cea0a0, 23 },
    { 0xc48ea0a1, 55 },
    { 0, 0 },
    { 0xdc0ea0a3, 17 },
    { 0, 0 },
    { 0x172ea0a5, 33 },
    { 0xa34ea0a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eea0aa, 33 },
    { 0xeebea0aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66ea0b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763ea0b5, 87 },
    { 0, 0 },
    { 0xd07ea0b7, 93 },
    { 0x154ea0b7, 95 },
    { 0, 0 },
    { 0x959ea0ba, 90 },
    { 0x7ea0bb, 23 },
    { 0xabdea0bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5ea0c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ea0d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65ea0da, 61 },
    { 0, 0 },
    { 0xa85ea0dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36ea0e2, 76 },
    { 0xdb9ea0e3, 85 },
    { 0xf85ea0e4, 84 },
    { 0xd4aea0e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fea0ea, 78 },
    { 0x24fea0ea, 17 },
    { 0x5c1ea0ec, 85 },
    { 0x1e6ea0ed, 76 },
    { 0, 0 },
    { 0xa05ea0ef, 90 },
    { 0x3b1ea0f0, 90 },
    { 0, 0 },
    { 0xf74ea0f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78ea0f5, 100 },
    { 0, 0 },
    { 0x9ebea0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaea0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ea100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ea103, 33 },
    { 0xde9ea104, 93 },
    { 0x180ea105, 77 },
    { 0xdb9ea106, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ea10f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395ea114, 95 },
    { 0x6cea114, 98 },
    { 0x43bea116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ea11b, 33 },
    { 0xd00ea11b, 90 },
    { 0, 0 },
    { 0xd90ea11e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8ea121, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeea127, 95 },
    { 0xc70ea127, 33 },
    { 0x49fea128, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cea12e, 43 },
    { 0xaf5ea12f, 23 },
    { 0x2b8ea130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6ea133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ea137, 74 },
    { 0xbf5ea137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeea13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacea146, 56 },
    { 0xb8bea146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7ea14a, 95 },
    { 0xc83ea14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ea150, 34 },
    { 0x644ea150, 99 },
    { 0x11eea152, 33 },
    { 0, 0 },
    { 0x3f6ea154, 76 },
    { 0x5a5ea154, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9ea158, 98 },
    { 0x5acea159, 91 },
    { 0xcbbea15a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ea15f, 90 },
    { 0, 0 },
    { 0xc15ea161, 95 },
    { 0x36fea161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576ea169, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ea16f, 33 },
    { 0x29aea16f, 43 },
    { 0xb14ea171, 33 },
    { 0xd27ea171, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fea179, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaea17d, 61 },
    { 0x711ea17e, 95 },
    { 0, 0 },
    { 0xe9eea180, 78 },
    { 0xc4bea181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ea186, 33 },
    { 0, 0 },
    { 0xfbdea188, 95 },
    { 0, 0 },
    { 0x1d1ea18a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeea194, 90 },
    { 0x927ea194, 55 },
    { 0, 0 },
    { 0xd78ea197, 96 },
    { 0, 0 },
    { 0x356ea199, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07ea1a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eea1a3, 88 },
    { 0, 0 },
    { 0x420ea1a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48ea1aa, 52 },
    { 0, 0 },
    { 0xed5ea1ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ea1af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdea1b3, 33 },
    { 0x4acea1b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1ea1b8, 33 },
    { 0x6baea1b8, 90 },
    { 0, 0 },
    { 0xaf6ea1bb, 90 },
    { 0x2d8ea1bc, 61 },
    { 0, 0 },
    { 0x5c3ea1be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03ea1c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ea1ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cea1ce, 98 },
    { 0xd43ea1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ea1d7, 99 },
    { 0x6dea1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaea1dd, 61 },
    { 0x26bea1de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea1e2, 34 },
    { 0xb88ea1e3, 33 },
    { 0x6fdea1e4, 68 },
    { 0x1d0ea1e5, 61 },
    { 0x52ea1e3, 85 },
    { 0, 0 },
    { 0x65bea1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea1eb, 90 },
    { 0xc6cea1ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feea1f0, 23 },
    { 0x182ea1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eea1f6, 95 },
    { 0x1a4ea1f6, 65 },
    { 0x53eea1f8, 52 },
    { 0x284ea1f8, 85 },
    { 0xb23ea1f8, 33 },
    { 0, 0 },
    { 0x7b8ea1fc, 83 },
    { 0xf37ea1fd, 55 },
    { 0, 0 },
    { 0x6f8ea1ff, 78 },
    { 0x756ea200, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aea203, 88 },
    { 0, 0 },
    { 0x65dea205, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x240ea208, 66 },
    { 0x323ea209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bea20d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fea21a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989ea21f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ea224, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ea229, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ea231, 33 },
    { 0xa62ea232, 87 },
    { 0x8d5ea233, 98 },
    { 0x187ea234, 61 },
    { 0, 0 },
    { 0xbcfea236, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ea239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bea242, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cea246, 95 },
    { 0x811ea246, 90 },
    { 0xd3aea246, 91 },
    { 0, 0 },
    { 0x702ea24a, 95 },
    { 0x50cea24b, 61 },
    { 0, 0 },
    { 0x610ea24d, 93 },
    { 0x923ea24e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ea251, 85 },
    { 0xba7ea252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ea256, 23 },
    { 0xad6ea257, 61 },
    { 0x257ea256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ea25c, 69 },
    { 0x1dcea25d, 68 },
    { 0x928ea25d, 99 },
    { 0, 0 },
    { 0xe1eea260, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ea263, 33 },
    { 0x846ea264, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416ea26a, 88 },
    { 0xe5aea26b, 90 },
    { 0xd23ea26b, 98 },
    { 0x7b0ea26d, 34 },
    { 0x6cea26e, 95 },
    { 0x85fea26f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15ea272, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bea275, 43 },
    { 0, 0 },
    { 0xb6bea277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea27a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0ea27e, 93 },
    { 0xaedea27f, 61 },
    { 0xb73ea280, 85 },
    { 0x7f7ea27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eea284, 90 },
    { 0xa08ea284, 61 },
    { 0x7caea284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaea289, 88 },
    { 0x653ea289, 33 },
    { 0x4a5ea289, 23 },
    { 0xc7bea28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ea28f, 90 },
    { 0, 0 },
    { 0x473ea291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eea295, 95 },
    { 0xfbbea295, 92 },
    { 0xedaea295, 23 },
    { 0xf18ea295, 34 },
    { 0x98ea299, 55 },
    { 0xcd2ea295, 95 },
    { 0x8d8ea29b, 90 },
    { 0xc4dea29c, 98 },
    { 0, 0 },
    { 0x9ecea29e, 90 },
    { 0x52dea29e, 55 },
    { 0x437ea29f, 93 },
    { 0x2e8ea29e, 52 },
    { 0x628ea2a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6ea2a5, 33 },
    { 0, 0 },
    { 0x540ea2a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eea2ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ea2b0, 95 },
    { 0xda6ea2b1, 66 },
    { 0xbf0ea2b0, 61 },
    { 0x652ea2b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ea2b7, 100 },
    { 0, 0 },
    { 0x5a2ea2b9, 23 },
    { 0xefaea2ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dea2bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ea2c4, 90 },
    { 0x834ea2c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x334ea2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x913ea2cb, 23 },
    { 0x8bea2cb, 55 },
    { 0, 0 },
    { 0xeb0ea2ce, 36 },
    { 0x78cea2cf, 33 },
    { 0x79bea2cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ea2d6, 91 },
    { 0xeabea2d7, 90 },
    { 0, 0 },
    { 0xe2dea2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792ea2dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x957ea2e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561ea2e4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ea2e8, 55 },
    { 0x87bea2e8, 55 },
    { 0x9f1ea2ea, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abea2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aea301, 98 },
    { 0, 0 },
    { 0x114ea303, 98 },
    { 0, 0 },
    { 0x86eea305, 98 },
    { 0, 0 },
    { 0x663ea307, 95 },
    { 0, 0 },
    { 0x5b8ea309, 33 },
    { 0xd7bea30a, 77 },
    { 0x873ea309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ea30e, 46 },
    { 0x11ea30f, 43 },
    { 0x547ea310, 98 },
    { 0, 0 },
    { 0x42bea312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bea31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1ea325, 100 },
    { 0x937ea326, 95 },
    { 0x290ea326, 61 },
    { 0x135ea328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ea32c, 90 },
    { 0xcaea32d, 90 },
    { 0x225ea32d, 95 },
    { 0xe22ea32f, 76 },
    { 0, 0 },
    { 0xb45ea331, 98 },
    { 0x1ffea331, 33 },
    { 0, 0 },
    { 0xea7ea334, 98 },
    { 0x877ea334, 87 },
    { 0x257ea336, 55 },
    { 0x560ea337, 33 },
    { 0x7e2ea336, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22ea340, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ea343, 90 },
    { 0x311ea344, 87 },
    { 0x476ea343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ea349, 61 },
    { 0, 0 },
    { 0x2b5ea34b, 23 },
    { 0xa3aea34b, 36 },
    { 0x4d2ea34b, 33 },
    { 0, 0 },
    { 0x1e3ea34f, 100 },
    { 0x665ea350, 90 },
    { 0xacea34f, 76 },
    { 0x344ea350, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fea357, 81 },
    { 0x77fea358, 90 },
    { 0, 0 },
    { 0xc4bea35a, 95 },
    { 0x6a1ea35b, 90 },
    { 0x11eea35c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1ea362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3ea365, 76 },
    { 0, 0 },
    { 0x83eea367, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dea36a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6ea36d, 61 },
    { 0x554ea36d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656ea374, 90 },
    { 0xde8ea374, 43 },
    { 0, 0 },
    { 0xbe3ea377, 99 },
    { 0xd67ea378, 66 },
    { 0x2daea379, 61 },
    { 0x90fea37a, 61 },
    { 0xd78ea37b, 95 },
    { 0, 0 },
    { 0x87aea37d, 75 },
    { 0, 0 },
    { 0x8abea37f, 76 },
    { 0xa3cea380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ea386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cea38a, 98 },
    { 0, 0 },
    { 0x9f6ea38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ea38f, 23 },
    { 0x658ea38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bea393, 95 },
    { 0xf48ea394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fea39d, 43 },
    { 0xb0dea39e, 85 },
    { 0xeaeea39f, 90 },
    { 0xd6eea3a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cea3a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fea3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267ea3ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4ea3b4, 84 },
    { 0x7f8ea3b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eea3bb, 99 },
    { 0x31dea3bc, 88 },
    { 0x6eaea3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5ea3c8, 33 },
    { 0x41eea3c9, 95 },
    { 0x38cea3c8, 23 },
    { 0x105ea3cb, 101 },
    { 0x876ea3cc, 66 },
    { 0x26aea3cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aea3d0, 98 },
    { 0, 0 },
    { 0x6feea3d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedea3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ea3de, 98 },
    { 0x1b8ea3df, 90 },
    { 0, 0 },
    { 0xb2aea3e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ea3e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ea3e9, 82 },
    { 0, 0 },
    { 0x36fea3eb, 91 },
    { 0x865ea3ec, 66 },
    { 0, 0 },
    { 0x638ea3ee, 55 },
    { 0, 0 },
    { 0xb9ea3f0, 43 },
    { 0, 0 },
    { 0x79ea3f2, 99 },
    { 0, 0 },
    { 0xc44ea3f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dea3f7, 76 },
    { 0xf7fea3f8, 87 },
    { 0xa12ea3f9, 33 },
    { 0x9adea3fa, 33 },
    { 0xe19ea3fa, 61 },
    { 0x4f9ea3fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea401, 90 },
    { 0, 0 },
    { 0xfdeea403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fea409, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbea40f, 61 },
    { 0xbc1ea40f, 65 },
    { 0x93dea411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aea415, 85 },
    { 0x51bea416, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6ea41d, 64 },
    { 0, 0 },
    { 0x91aea41f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dea431, 23 },
    { 0x3aeea432, 76 },
    { 0x625ea432, 85 },
    { 0x585ea434, 61 },
    { 0xe36ea435, 90 },
    { 0xb7cea436, 23 },
    { 0xb2cea431, 61 },
    { 0xa82ea437, 40 },
    { 0xfa7ea439, 100 },
    { 0, 0 },
    { 0xcea43b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ea43e, 40 },
    { 0x76dea43e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5ea443, 33 },
    { 0x9a6ea443, 98 },
    { 0x6afea444, 43 },
    { 0x536ea446, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ea44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdea452, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ea45b, 23 },
    { 0, 0 },
    { 0x6cfea45d, 90 },
    { 0xfdaea45d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccea461, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ea464, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cea468, 81 },
    { 0xd19ea468, 90 },
    { 0x288ea469, 85 },
    { 0, 0 },
    { 0x18fea46c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebea470, 77 },
    { 0x82fea471, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2ea474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ea47b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8ea480, 90 },
    { 0x3dea481, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8ea484, 23 },
    { 0x5c7ea485, 95 },
    { 0xd01ea484, 98 },
    { 0xfceea487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bea48f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaea494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812ea498, 61 },
    { 0x390ea498, 46 },
    { 0x6ddea49a, 36 },
    { 0xadeea49a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ea49f, 61 },
    { 0, 0 },
    { 0x7bfea4a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18ea4a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8ea4a7, 76 },
    { 0xe62ea4a8, 33 },
    { 0x915ea4a9, 33 },
    { 0xe88ea4aa, 55 },
    { 0x1f5ea4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527ea4af, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbea4b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194ea4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x597ea4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aea4be, 33 },
    { 0x103ea4be, 34 },
    { 0x358ea4c0, 78 },
    { 0x7b7ea4c0, 84 },
    { 0x3a2ea4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea4c5, 23 },
    { 0, 0 },
    { 0x142ea4c7, 38 },
    { 0x5f5ea4c7, 98 },
    { 0x606ea4c8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0ea4cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ea4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eea4d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x344ea4da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ea4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365ea4ea, 98 },
    { 0x1c9ea4eb, 61 },
    { 0, 0 },
    { 0x8a8ea4ed, 43 },
    { 0x12fea4ee, 34 },
    { 0x840ea4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ea4f3, 33 },
    { 0x115ea4f4, 84 },
    { 0xeb5ea4f3, 95 },
    { 0x7b3ea4f6, 61 },
    { 0x42eea4f7, 23 },
    { 0, 0 },
    { 0xb9ea4f9, 95 },
    { 0x9b5ea4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46ea4fe, 98 },
    { 0x5b2ea4ff, 55 },
    { 0, 0 },
    { 0xb33ea501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ea504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104ea509, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cea50d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cea516, 97 },
    { 0x39bea517, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3daea51b, 98 },
    { 0xc6cea51c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bea520, 95 },
    { 0, 0 },
    { 0x35cea522, 61 },
    { 0x891ea523, 90 },
    { 0, 0 },
    { 0x7efea525, 33 },
    { 0x18eea526, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dea52a, 33 },
    { 0x76cea52b, 91 },
    { 0xb56ea52c, 90 },
    { 0x6e6ea52d, 52 },
    { 0x494ea52e, 91 },
    { 0xb45ea52f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceea533, 95 },
    { 0, 0 },
    { 0xf2fea535, 56 },
    { 0x9ea536, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4ea53a, 69 },
    { 0xcf6ea53a, 95 },
    { 0, 0 },
    { 0x31dea53d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cea540, 33 },
    { 0x47aea541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662ea547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ea54a, 66 },
    { 0, 0 },
    { 0x6b3ea54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cea54f, 76 },
    { 0x2c0ea550, 77 },
    { 0xcc7ea550, 33 },
    { 0x83cea54f, 90 },
    { 0x3ceea54f, 33 },
    { 0x8eea550, 95 },
    { 0xc53ea555, 55 },
    { 0x446ea556, 55 },
    { 0xddcea557, 78 },
    { 0x5d6ea557, 77 },
    { 0, 0 },
    { 0x202ea55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aea55e, 77 },
    { 0, 0 },
    { 0x964ea560, 16 },
    { 0x469ea560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ea564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5ea569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9ea570, 61 },
    { 0x5daea571, 55 },
    { 0x3c4ea570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ea57c, 61 },
    { 0xf69ea57d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcea581, 90 },
    { 0xd46ea582, 78 },
    { 0xcc0ea582, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ea587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bea58b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7ea592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cea596, 88 },
    { 0x128ea596, 23 },
    { 0, 0 },
    { 0x555ea599, 34 },
    { 0x59dea59a, 95 },
    { 0, 0 },
    { 0x1a7ea59c, 75 },
    { 0x7fbea59c, 98 },
    { 0xb14ea59c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2ea5a1, 33 },
    { 0x796ea5a2, 23 },
    { 0, 0 },
    { 0x9b9ea5a4, 81 },
    { 0, 0 },
    { 0xdeea5a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ea5ab, 95 },
    { 0x4f2ea5ab, 42 },
    { 0, 0 },
    { 0xe38ea5ae, 87 },
    { 0x4beea5af, 90 },
    { 0x851ea5af, 33 },
    { 0x9ea5ae, 33 },
    { 0, 0 },
    { 0x5ceea5b3, 23 },
    { 0xf6bea5b4, 43 },
    { 0x2b4ea5b5, 91 },
    { 0xba4ea5b6, 93 },
    { 0x77bea5b7, 53 },
    { 0xeb0ea5b8, 33 },
    { 0xa58ea5b4, 34 },
    { 0xb98ea5b9, 98 },
    { 0xa1eea5ba, 98 },
    { 0x175ea5bc, 90 },
    { 0x8e3ea5bc, 95 },
    { 0xd10ea5be, 77 },
    { 0, 0 },
    { 0x967ea5c0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea5c8, 90 },
    { 0x679ea5c8, 76 },
    { 0xa05ea5c8, 99 },
    { 0, 0 },
    { 0xd9aea5cc, 87 },
    { 0x36aea5cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796ea5d1, 46 },
    { 0, 0 },
    { 0x94bea5d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485ea5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6ea5dc, 33 },
    { 0xee3ea5dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ea5e1, 56 },
    { 0xf3bea5e2, 61 },
    { 0xf16ea5e1, 34 },
    { 0xf12ea5e3, 33 },
    { 0xfc5ea5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ea5e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cea5ec, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eea5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abea5f4, 95 },
    { 0x168ea5f5, 88 },
    { 0x43dea5f5, 76 },
    { 0xf69ea5f5, 98 },
    { 0x516ea5f8, 43 },
    { 0x53bea5f8, 43 },
    { 0x579ea5f4, 61 },
    { 0xfe9ea5f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cea5fe, 95 },
    { 0, 0 },
    { 0xaf2ea600, 33 },
    { 0x55eea601, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ea604, 77 },
    { 0, 0 },
    { 0x238ea606, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fea609, 23 },
    { 0x6f4ea60a, 40 },
    { 0x29dea60b, 98 },
    { 0x39ea60b, 95 },
    { 0xe93ea609, 76 },
    { 0x5e1ea60e, 91 },
    { 0x45ea60f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ea612, 61 },
    { 0x92cea612, 76 },
    { 0x20ea614, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09ea617, 95 },
    { 0xb0fea618, 61 },
    { 0xdd4ea619, 68 },
    { 0, 0 },
    { 0xf51ea61b, 84 },
    { 0, 0 },
    { 0x637ea61d, 46 },
    { 0x6f7ea61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dea624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aea627, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea62e, 34 },
    { 0, 0 },
    { 0x7c3ea630, 34 },
    { 0x198ea631, 95 },
    { 0x1ecea632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cea636, 61 },
    { 0x6a4ea636, 33 },
    { 0x1edea636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1ea640, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0ea646, 46 },
    { 0x202ea647, 61 },
    { 0x84cea648, 52 },
    { 0x300ea649, 93 },
    { 0xa42ea649, 33 },
    { 0xb3ea64a, 23 },
    { 0xebea64c, 91 },
    { 0xca1ea649, 33 },
    { 0xa5eea648, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eea656, 93 },
    { 0x600ea657, 23 },
    { 0xd4ea658, 23 },
    { 0x9caea659, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faea661, 43 },
    { 0x216ea662, 61 },
    { 0xc26ea663, 23 },
    { 0x48ea662, 90 },
    { 0, 0 },
    { 0x115ea666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ea669, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1ea66f, 33 },
    { 0x6dbea670, 55 },
    { 0xaea66f, 90 },
    { 0x324ea672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65ea679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ea67d, 95 },
    { 0, 0 },
    { 0x314ea67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eea68e, 90 },
    { 0xf34ea68f, 34 },
    { 0xe58ea690, 35 },
    { 0, 0 },
    { 0x234ea692, 33 },
    { 0x31fea693, 23 },
    { 0, 0 },
    { 0xe3eea695, 23 },
    { 0, 0 },
    { 0x578ea697, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52ea69d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aea6a1, 90 },
    { 0x580ea6a1, 43 },
    { 0, 0 },
    { 0x7dea6a4, 23 },
    { 0x363ea6a4, 76 },
    { 0x2e2ea6a6, 61 },
    { 0x2d9ea6a7, 95 },
    { 0x518ea6a6, 38 },
    { 0x1bfea6a9, 77 },
    { 0x896ea6aa, 90 },
    { 0xc58ea6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ea6ae, 90 },
    { 0xedbea6af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fea6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ea6b9, 60 },
    { 0x571ea6b9, 75 },
    { 0xb00ea6bb, 55 },
    { 0xf97ea6ba, 91 },
    { 0x8d9ea6bd, 98 },
    { 0, 0 },
    { 0xda7ea6bf, 85 },
    { 0x337ea6c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aea6c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632ea6c6, 99 },
    { 0xab9ea6c7, 93 },
    { 0x961ea6c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92ea6cc, 85 },
    { 0xc04ea6cd, 23 },
    { 0, 0 },
    { 0x478ea6cf, 61 },
    { 0xd4dea6cf, 76 },
    { 0x461ea6d1, 23 },
    { 0, 0 },
    { 0x9cfea6d3, 46 },
    { 0xde5ea6d4, 43 },
    { 0, 0 },
    { 0xdccea6d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498ea6db, 61 },
    { 0xaffea6dc, 98 },
    { 0x817ea6dc, 32 },
    { 0x660ea6de, 23 },
    { 0x28bea6de, 90 },
    { 0x6a3ea6e0, 23 },
    { 0xd7cea6db, 33 },
    { 0x757ea6e2, 98 },
    { 0x55fea6e2, 57 },
    { 0x27dea6e4, 90 },
    { 0x8edea6e3, 90 },
    { 0x85cea6e1, 91 },
    { 0x8c8ea6e7, 43 },
    { 0x6a4ea6de, 93 },
    { 0x35fea6e0, 26 },
    { 0, 0 },
    { 0xfd5ea6eb, 34 },
    { 0, 0 },
    { 0xbe2ea6ed, 61 },
    { 0, 0 },
    { 0x9b0ea6ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5ea6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bea6fb, 76 },
    { 0x474ea6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ea704, 33 },
    { 0x909ea705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cea709, 95 },
    { 0xe46ea709, 90 },
    { 0x187ea70b, 95 },
    { 0x37cea70c, 90 },
    { 0, 0 },
    { 0x9c9ea70e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4ea714, 81 },
    { 0, 0 },
    { 0xbd6ea716, 52 },
    { 0x9c5ea716, 76 },
    { 0x4cdea718, 23 },
    { 0xccbea716, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bea71d, 61 },
    { 0x2adea71e, 98 },
    { 0x4beea71f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ea723, 76 },
    { 0x3aaea724, 61 },
    { 0x71cea723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ea72b, 34 },
    { 0, 0 },
    { 0x62dea72d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2faea735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea738, 34 },
    { 0x3dcea739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeea742, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60ea746, 90 },
    { 0xd49ea747, 81 },
    { 0x1cdea748, 61 },
    { 0x190ea747, 84 },
    { 0xaebea747, 98 },
    { 0x298ea74a, 95 },
    { 0xec0ea74c, 81 },
    { 0x657ea74c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28ea751, 84 },
    { 0x5e4ea752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ea75a, 95 },
    { 0xd65ea75b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feea75f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ea763, 61 },
    { 0, 0 },
    { 0x58eea765, 55 },
    { 0xefeea765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fea76c, 76 },
    { 0, 0 },
    { 0x1edea76e, 66 },
    { 0x457ea76f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dea772, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355ea779, 33 },
    { 0x84dea77a, 65 },
    { 0, 0 },
    { 0x798ea77c, 90 },
    { 0x12aea77c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8ea780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fea786, 33 },
    { 0x5ea786, 55 },
    { 0x4c7ea786, 33 },
    { 0xfd3ea789, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fea78f, 33 },
    { 0, 0 },
    { 0xf27ea791, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2ea794, 56 },
    { 0x169ea795, 33 },
    { 0xd1ea795, 93 },
    { 0x233ea797, 36 },
    { 0xcc6ea798, 52 },
    { 0xb39ea798, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2ea79c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ea7a3, 66 },
    { 0xafdea7a3, 90 },
    { 0, 0 },
    { 0xa64ea7a6, 98 },
    { 0, 0 },
    { 0x444ea7a8, 90 },
    { 0, 0 },
    { 0x96eea7aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea7ae, 55 },
    { 0x122ea7af, 81 },
    { 0x70cea7b0, 99 },
    { 0xd60ea7b0, 95 },
    { 0xdbea7b2, 90 },
    { 0xc92ea7b2, 90 },
    { 0, 0 },
    { 0x3f4ea7b5, 67 },
    { 0x9eeea7b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ea7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea7bd, 33 },
    { 0xf4dea7be, 95 },
    { 0xa59ea7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aea7c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adea7c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8ea7ca, 95 },
    { 0, 0 },
    { 0xe56ea7cc, 93 },
    { 0xacbea7cd, 33 },
    { 0x80cea7ce, 67 },
    { 0xe9ea7cf, 90 },
    { 0x451ea7d0, 87 },
    { 0x47eea7cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63ea7d8, 23 },
    { 0, 0 },
    { 0xebeea7da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ea7dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ea7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bea7e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18ea7ec, 34 },
    { 0, 0 },
    { 0x61aea7ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cea7f1, 91 },
    { 0x64eea7f2, 17 },
    { 0, 0 },
    { 0x64ea7f4, 34 },
    { 0xf0eea7f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aea7f8, 66 },
    { 0, 0 },
    { 0x940ea7fa, 67 },
    { 0xd64ea7fb, 95 },
    { 0xdadea7fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ea806, 23 },
    { 0xf01ea807, 90 },
    { 0, 0 },
    { 0x169ea809, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaea80c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79ea810, 90 },
    { 0, 0 },
    { 0xd87ea812, 23 },
    { 0xd43ea812, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ea818, 95 },
    { 0x476ea819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268ea81e, 61 },
    { 0xa81ea81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aea823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ea827, 33 },
    { 0x9d3ea828, 95 },
    { 0x852ea828, 61 },
    { 0x17eea82a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ea82e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7ea831, 88 },
    { 0xadeea832, 77 },
    { 0xf6dea832, 17 },
    { 0xfcbea834, 95 },
    { 0x6caea835, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x457ea838, 90 },
    { 0xfa6ea839, 85 },
    { 0, 0 },
    { 0x67bea83b, 76 },
    { 0, 0 },
    { 0x5d4ea83d, 88 },
    { 0x8e0ea83d, 81 },
    { 0x95bea83f, 90 },
    { 0xb18ea83d, 43 },
    { 0, 0 },
    { 0x455ea842, 90 },
    { 0, 0 },
    { 0x236ea844, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bea84a, 46 },
    { 0x857ea84a, 95 },
    { 0x45eea84c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ea851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcea856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x895ea859, 100 },
    { 0x97aea85a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ea85e, 61 },
    { 0xd17ea85f, 78 },
    { 0x115ea85e, 92 },
    { 0x33ea85f, 95 },
    { 0x938ea85e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdea866, 95 },
    { 0, 0 },
    { 0x15ea868, 33 },
    { 0x4f6ea868, 33 },
    { 0x63fea86a, 33 },
    { 0x769ea868, 95 },
    { 0xc9cea86c, 74 },
    { 0x6beea86d, 78 },
    { 0, 0 },
    { 0xbafea86f, 78 },
    { 0x820ea86f, 76 },
    { 0xb3ea871, 90 },
    { 0x5adea870, 55 },
    { 0x920ea873, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477ea877, 98 },
    { 0xd98ea878, 33 },
    { 0x280ea879, 95 },
    { 0xd07ea878, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cea87d, 80 },
    { 0x262ea87e, 33 },
    { 0, 0 },
    { 0x393ea880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ea883, 90 },
    { 0, 0 },
    { 0xfe6ea885, 65 },
    { 0xf41ea886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bea88c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ea88f, 61 },
    { 0xb97ea890, 23 },
    { 0xf8bea891, 33 },
    { 0x9e1ea892, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dea898, 55 },
    { 0, 0 },
    { 0xd55ea89a, 98 },
    { 0x4a9ea89b, 78 },
    { 0x63eea89c, 66 },
    { 0x6c6ea89a, 90 },
    { 0x31cea89c, 84 },
    { 0x627ea89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaea8a3, 81 },
    { 0x597ea8a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aea8a7, 23 },
    { 0xac6ea8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabea8b7, 88 },
    { 0, 0 },
    { 0xe9fea8b9, 85 },
    { 0xba3ea8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fea8c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03ea8c4, 33 },
    { 0, 0 },
    { 0xbcea8c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cea8c9, 43 },
    { 0, 0 },
    { 0xfaeea8cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eea8ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0ea8d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cea8d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8ea8db, 55 },
    { 0xbe1ea8dc, 76 },
    { 0x6e3ea8dd, 55 },
    { 0, 0 },
    { 0x8a2ea8df, 46 },
    { 0xa31ea8df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea8e4, 90 },
    { 0x298ea8e5, 46 },
    { 0xa4bea8e4, 61 },
    { 0, 0 },
    { 0xe20ea8e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea8eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eea8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecea8f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ea8f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ea8fa, 46 },
    { 0x344ea8fb, 33 },
    { 0xf11ea8fb, 93 },
    { 0x432ea8fd, 76 },
    { 0x63ea8fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea901, 95 },
    { 0xa20ea901, 61 },
    { 0xc9ea903, 66 },
    { 0x59dea903, 23 },
    { 0xe29ea904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eea909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aea910, 91 },
    { 0x498ea911, 99 },
    { 0x162ea911, 90 },
    { 0xd93ea913, 100 },
    { 0, 0 },
    { 0x54cea915, 98 },
    { 0x243ea915, 93 },
    { 0xc30ea916, 66 },
    { 0, 0 },
    { 0x674ea919, 43 },
    { 0x108ea919, 90 },
    { 0xf67ea91b, 33 },
    { 0xbfaea91c, 17 },
    { 0xfabea91d, 23 },
    { 0, 0 },
    { 0x776ea91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315ea925, 77 },
    { 0x20bea926, 33 },
    { 0x142ea927, 17 },
    { 0x51dea927, 91 },
    { 0, 0 },
    { 0x6d4ea92a, 61 },
    { 0xc8aea92b, 98 },
    { 0x272ea92a, 61 },
    { 0x69aea92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcea932, 98 },
    { 0x484ea932, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccea93d, 46 },
    { 0, 0 },
    { 0x9ea93f, 90 },
    { 0x5cfea940, 95 },
    { 0x9f5ea941, 23 },
    { 0, 0 },
    { 0xab4ea943, 61 },
    { 0x553ea944, 87 },
    { 0x85dea945, 90 },
    { 0xcecea946, 90 },
    { 0xf19ea947, 95 },
    { 0x6a4ea948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3ea94c, 90 },
    { 0x312ea94c, 95 },
    { 0x564ea94d, 98 },
    { 0, 0 },
    { 0x31ea950, 67 },
    { 0x4b1ea951, 65 },
    { 0, 0 },
    { 0x93ea953, 98 },
    { 0x88cea954, 90 },
    { 0x6fdea953, 61 },
    { 0x50ea956, 78 },
    { 0x1e5ea957, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eea95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19eea962, 23 },
    { 0xd05ea962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ea967, 33 },
    { 0xc2eea967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ea96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ea96f, 36 },
    { 0xcb9ea96f, 61 },
    { 0x515ea971, 33 },
    { 0x588ea972, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2ea979, 95 },
    { 0x9a2ea97a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ea98b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3ea990, 76 },
    { 0x67eea991, 100 },
    { 0xcdaea992, 55 },
    { 0x7a1ea991, 61 },
    { 0, 0 },
    { 0xd95ea995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdea998, 55 },
    { 0x843ea998, 17 },
    { 0x77cea99a, 90 },
    { 0, 0 },
    { 0x967ea99c, 69 },
    { 0x165ea99d, 90 },
    { 0, 0 },
    { 0x60fea99f, 87 },
    { 0x659ea99f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ea9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ea9a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626ea9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cea9b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaea9b5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5daea9bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537ea9c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ea9ca, 85 },
    { 0, 0 },
    { 0x126ea9cc, 90 },
    { 0xfcdea9cd, 33 },
    { 0x303ea9ce, 56 },
    { 0xae0ea9cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afea9d9, 90 },
    { 0x5e8ea9d9, 34 },
    { 0, 0 },
    { 0x469ea9dc, 90 },
    { 0x519ea9dd, 43 },
    { 0xcaeea9dd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecea9e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ea9e6, 38 },
    { 0x5beea9e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6ea9ec, 91 },
    { 0x863ea9ed, 85 },
    { 0xdbcea9ee, 100 },
    { 0x2e6ea9ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdaea9f3, 91 },
    { 0x750ea9f4, 93 },
    { 0x9f5ea9f5, 55 },
    { 0, 0 },
    { 0x53dea9f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbea9fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebeaa01, 66 },
    { 0x3adeaa01, 23 },
    { 0, 0 },
    { 0x59beaa04, 23 },
    { 0x6cdeaa05, 87 },
    { 0, 0 },
    { 0x1bfeaa07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79eaa0a, 90 },
    { 0x80feaa0b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234eaa10, 34 },
    { 0xfe4eaa11, 46 },
    { 0, 0 },
    { 0x316eaa13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cceaa17, 23 },
    { 0xbabeaa18, 98 },
    { 0, 0 },
    { 0x888eaa1a, 33 },
    { 0, 0 },
    { 0xd6ceaa1c, 55 },
    { 0xab0eaa1d, 95 },
    { 0x4ceaa1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848eaa24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8beaa27, 69 },
    { 0xf17eaa27, 61 },
    { 0x368eaa29, 77 },
    { 0xfeceaa2a, 87 },
    { 0, 0 },
    { 0x83eeaa2c, 90 },
    { 0x48ceaa2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6feaa31, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eaa34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x779eaa37, 55 },
    { 0xbdbeaa37, 43 },
    { 0x60beaa38, 84 },
    { 0, 0 },
    { 0xd4ceaa3b, 93 },
    { 0, 0 },
    { 0xf46eaa3d, 61 },
    { 0xe64eaa3e, 98 },
    { 0xdc4eaa3f, 43 },
    { 0, 0 },
    { 0xb69eaa41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6eaa44, 55 },
    { 0x59ceaa45, 95 },
    { 0x5b7eaa45, 55 },
    { 0, 0 },
    { 0x8f3eaa48, 33 },
    { 0x8c0eaa49, 76 },
    { 0x2d8eaa4a, 99 },
    { 0xba1eaa4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213eaa55, 46 },
    { 0, 0 },
    { 0x81beaa57, 33 },
    { 0x60eaa58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7eaa5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89eaa5f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857eaa68, 95 },
    { 0x5fbeaa69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21eaa70, 61 },
    { 0x39ceaa71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55eaa74, 34 },
    { 0, 0 },
    { 0xe38eaa76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863eaa7b, 23 },
    { 0xe16eaa7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6eaa7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaeaa82, 90 },
    { 0xf2aeaa82, 34 },
    { 0x727eaa83, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aceaa87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbeaa8b, 33 },
    { 0xfd5eaa8c, 78 },
    { 0x5f1eaa8d, 61 },
    { 0xc11eaa8b, 98 },
    { 0, 0 },
    { 0x38feaa90, 95 },
    { 0x3feaa90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967eaa97, 23 },
    { 0x6d8eaa98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeeaa9b, 67 },
    { 0xb04eaa9c, 46 },
    { 0xa85eaa9b, 17 },
    { 0x843eaa9e, 90 },
    { 0x203eaa9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8eaaaa, 52 },
    { 0x83eaaaa, 61 },
    { 0x974eaaaa, 90 },
    { 0, 0 },
    { 0x87feaaae, 90 },
    { 0xc1beaaae, 77 },
    { 0xf79eaab0, 33 },
    { 0x1c1eaab1, 61 },
    { 0x850eaab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8eaabc, 76 },
    { 0xc8eeaabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x492eaac0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x80deaac3, 67 },
    { 0, 0 },
    { 0xbbceaac5, 66 },
    { 0, 0 },
    { 0x47feaac7, 43 },
    { 0, 0 },
    { 0x7b5eaac9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448eaace, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715eaad3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eeaadc, 61 },
    { 0, 0 },
    { 0x40aeaade, 46 },
    { 0xe6eeaade, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5eaae4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670eaae9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51eaaed, 53 },
    { 0xceaaee, 75 },
    { 0x7f3eaaee, 66 },
    { 0x9bbeaaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eeaaf3, 55 },
    { 0x9eeeaaf4, 64 },
    { 0xdc4eaaf4, 62 },
    { 0xd8deaaf6, 90 },
    { 0xd68eaaf7, 23 },
    { 0x41ceaaf8, 95 },
    { 0x20ceaaf9, 61 },
    { 0x397eaaf9, 91 },
    { 0x6e1eaafa, 95 },
    { 0xf9beaafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3eab00, 61 },
    { 0xf9ceab01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedeab05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x147eab08, 33 },
    { 0x997eab09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feab0f, 55 },
    { 0, 0 },
    { 0x452eab11, 43 },
    { 0, 0 },
    { 0x3d0eab13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0eab1b, 55 },
    { 0x54aeab1c, 17 },
    { 0x11beab1d, 100 },
    { 0, 0 },
    { 0x718eab1f, 90 },
    { 0x184eab1f, 98 },
    { 0, 0 },
    { 0x5c8eab22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eeab2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28eab2e, 90 },
    { 0xd9eab2f, 23 },
    { 0xb9eab30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9eab37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883eab43, 95 },
    { 0, 0 },
    { 0x8cceab45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490eab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afeab50, 61 },
    { 0x5beeab51, 23 },
    { 0xb71eab52, 76 },
    { 0x905eab53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88deab59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5eab5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478eab63, 95 },
    { 0x200eab64, 80 },
    { 0xa40eab65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eab6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19beab70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278eab76, 99 },
    { 0x994eab77, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36eab7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ceab7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabeab82, 95 },
    { 0xff7eab83, 93 },
    { 0x745eab84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60beab88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51eab8b, 38 },
    { 0xb09eab8c, 36 },
    { 0xe06eab8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feab90, 43 },
    { 0x8e7eab90, 85 },
    { 0x78feab92, 76 },
    { 0xc3deab92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9eab96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x588eab99, 90 },
    { 0, 0 },
    { 0xd6deab9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6eaba4, 61 },
    { 0, 0 },
    { 0xf66eaba6, 99 },
    { 0, 0 },
    { 0x952eaba8, 23 },
    { 0xd63eaba9, 53 },
    { 0, 0 },
    { 0xd6beabab, 33 },
    { 0x846eabac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaeabaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1eabb6, 61 },
    { 0x821eabb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41eabc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3eabc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8eabce, 95 },
    { 0xf59eabce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x767eabd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74beabd9, 23 },
    { 0xa3eabda, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5eabe1, 66 },
    { 0x795eabe2, 61 },
    { 0x976eabe3, 34 },
    { 0x810eabe4, 23 },
    { 0xe52eabe4, 90 },
    { 0xce2eabe6, 93 },
    { 0x19deabe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x513eabea, 90 },
    { 0, 0 },
    { 0xf6beabec, 23 },
    { 0xf0aeabed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfeabf3, 60 },
    { 0x2e5eabf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19aeabf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eeabfa, 90 },
    { 0xeffeabfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2eac03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5beac0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aeac0f, 90 },
    { 0x9fceac0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12eac18, 55 },
    { 0, 0 },
    { 0x4a8eac1a, 33 },
    { 0, 0 },
    { 0xe72eac1c, 90 },
    { 0xe8deac1c, 55 },
    { 0x670eac1e, 23 },
    { 0x7caeac1e, 55 },
    { 0xd31eac20, 52 },
    { 0x58ceac21, 56 },
    { 0x311eac22, 34 },
    { 0x98deac23, 90 },
    { 0xb66eac1d, 66 },
    { 0x31eac23, 55 },
    { 0xf23eac20, 23 },
    { 0xa60eac27, 76 },
    { 0x3abeac28, 76 },
    { 0xc30eac28, 52 },
    { 0xeb0eac2a, 76 },
    { 0xab8eac2b, 52 },
    { 0x3bdeac1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4deac30, 90 },
    { 0x4adeac30, 76 },
    { 0, 0 },
    { 0xe8eac33, 52 },
    { 0x9b0eac34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9eac39, 98 },
    { 0x675eac3a, 61 },
    { 0x694eac3b, 95 },
    { 0x7dfeac3c, 43 },
    { 0xc3eac3d, 90 },
    { 0x7c4eac3e, 38 },
    { 0x52ceac3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adeac44, 46 },
    { 0xe6ceac45, 90 },
    { 0, 0 },
    { 0xbd9eac47, 33 },
    { 0xf5eeac48, 46 },
    { 0, 0 },
    { 0x31aeac4a, 91 },
    { 0xb4eac4b, 55 },
    { 0xaafeac4c, 61 },
    { 0x40feac4d, 76 },
    { 0x871eac4e, 33 },
    { 0, 0 },
    { 0xeb0eac50, 43 },
    { 0xddceac51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9eac56, 33 },
    { 0xde4eac57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183eac5b, 98 },
    { 0, 0 },
    { 0x464eac5d, 40 },
    { 0, 0 },
    { 0xc4eeac5f, 90 },
    { 0, 0 },
    { 0xd29eac61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beac64, 61 },
    { 0xd8deac65, 93 },
    { 0x14eac66, 55 },
    { 0, 0 },
    { 0xc1deac68, 95 },
    { 0x5b5eac69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34eac72, 90 },
    { 0x4c3eac73, 76 },
    { 0xe87eac73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faeac77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1eac7a, 61 },
    { 0xe9feac7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3eac7e, 76 },
    { 0, 0 },
    { 0x783eac80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4eac84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ceac88, 61 },
    { 0x13beac88, 61 },
    { 0, 0 },
    { 0x95aeac8b, 95 },
    { 0xd38eac8c, 55 },
    { 0, 0 },
    { 0x466eac8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750eac93, 61 },
    { 0xd61eac93, 100 },
    { 0x443eac94, 55 },
    { 0x172eac96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aeac99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70eac9c, 52 },
    { 0x140eac9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922eaca2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eeacaa, 43 },
    { 0x735eacab, 34 },
    { 0, 0 },
    { 0x6b3eacad, 33 },
    { 0, 0 },
    { 0x83aeacaf, 90 },
    { 0x922eacaf, 34 },
    { 0x655eacb1, 91 },
    { 0, 0 },
    { 0x2e2eacb3, 90 },
    { 0xda1eacb4, 90 },
    { 0x71beacb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5eacbd, 33 },
    { 0xbbceacbd, 23 },
    { 0xc6aeacbd, 23 },
    { 0x717eacc0, 88 },
    { 0, 0 },
    { 0x247eacc2, 88 },
    { 0xfe0eacc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6eacc9, 23 },
    { 0x343eacca, 61 },
    { 0x4eaccb, 88 },
    { 0x2e7eaccb, 95 },
    { 0xacdeaccc, 95 },
    { 0x936eaccb, 33 },
    { 0, 0 },
    { 0xffeeacd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eacdb, 23 },
    { 0x85eeacdc, 100 },
    { 0x640eacdd, 55 },
    { 0xfffeacde, 81 },
    { 0, 0 },
    { 0x8aeace0, 23 },
    { 0x207eace0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0eace5, 95 },
    { 0, 0 },
    { 0xb57eace7, 99 },
    { 0xddbeace8, 95 },
    { 0x72feace9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08eaced, 33 },
    { 0x64feacee, 66 },
    { 0, 0 },
    { 0xd4feacf0, 33 },
    { 0x4d9eacf1, 33 },
    { 0xfb3eacf1, 95 },
    { 0xee4eacf3, 90 },
    { 0, 0 },
    { 0xb13eacf5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6eacf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12eacfc, 84 },
    { 0, 0 },
    { 0x51eacfe, 33 },
    { 0xb46eacff, 90 },
    { 0x3d1ead00, 61 },
    { 0x491eacfe, 93 },
    { 0xe6aead02, 66 },
    { 0xb53eacff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2ead08, 33 },
    { 0, 0 },
    { 0x98bead0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1ead0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ead13, 93 },
    { 0, 0 },
    { 0xf53ead15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119ead19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deead1c, 84 },
    { 0xd37ead1d, 33 },
    { 0xdd2ead1e, 55 },
    { 0x9f4ead1e, 23 },
    { 0x1c6ead20, 98 },
    { 0xbffead1d, 90 },
    { 0xe75ead1d, 98 },
    { 0x75bead1e, 61 },
    { 0xf5bead24, 55 },
    { 0xb4dead24, 87 },
    { 0xf6cead26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146ead2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ead32, 55 },
    { 0x7ccead33, 66 },
    { 0x7b6ead34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ead37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaead3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423ead42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x661ead45, 76 },
    { 0xd0ead46, 61 },
    { 0xbc0ead47, 65 },
    { 0, 0 },
    { 0x554ead49, 33 },
    { 0x84dead4a, 55 },
    { 0xf99ead4b, 98 },
    { 0xb16ead4c, 33 },
    { 0x816ead4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ead50, 36 },
    { 0, 0 },
    { 0xa7ead52, 40 },
    { 0x37cead52, 99 },
    { 0x50ead54, 55 },
    { 0xf2fead54, 95 },
    { 0x3beead56, 88 },
    { 0x44aead57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ead5d, 61 },
    { 0xd31ead5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ead68, 93 },
    { 0x2c2ead68, 78 },
    { 0xeffead68, 87 },
    { 0x971ead6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68ead6e, 55 },
    { 0xb69ead6f, 46 },
    { 0x1e4ead6f, 99 },
    { 0, 0 },
    { 0x8dbead72, 91 },
    { 0x573ead73, 93 },
    { 0xfd1ead74, 95 },
    { 0x202ead75, 52 },
    { 0xf01ead75, 76 },
    { 0x1dcead77, 95 },
    { 0xe31ead77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309ead7f, 67 },
    { 0, 0 },
    { 0x838ead81, 90 },
    { 0xb97ead81, 55 },
    { 0xd19ead82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70ead89, 90 },
    { 0xaa0ead8a, 40 },
    { 0x2deead8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432ead90, 100 },
    { 0xec0ead91, 48 },
    { 0x6e4ead92, 76 },
    { 0xb11ead92, 33 },
    { 0, 0 },
    { 0x3e7ead95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8ead9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2eada5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8eadaa, 99 },
    { 0x693eadab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eeadb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afeadb4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96eadb7, 84 },
    { 0x451eadb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8deadbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96eadc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4eadc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x684eadcc, 100 },
    { 0, 0 },
    { 0xd29eadce, 90 },
    { 0, 0 },
    { 0xffaeadd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9aeadd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83eadd8, 55 },
    { 0xc6eadd9, 23 },
    { 0x612eadda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87eaddd, 33 },
    { 0x909eadde, 61 },
    { 0xea6eaddf, 95 },
    { 0x38eeaddf, 96 },
    { 0xb4aeaddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2eade4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ceade8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdeadeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8eadee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30eadf5, 100 },
    { 0, 0 },
    { 0xac1eadf7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9eadfc, 98 },
    { 0x28eeadfc, 90 },
    { 0, 0 },
    { 0xc0aeadff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8feae06, 98 },
    { 0, 0 },
    { 0x2cbeae08, 33 },
    { 0xf52eae09, 76 },
    { 0, 0 },
    { 0xa25eae0b, 33 },
    { 0x671eae0b, 76 },
    { 0xc43eae0d, 34 },
    { 0xfc7eae0d, 95 },
    { 0x5f2eae0f, 33 },
    { 0x7fceae10, 61 },
    { 0xfc9eae0e, 61 },
    { 0, 0 },
    { 0x5aeae13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326eae17, 33 },
    { 0, 0 },
    { 0xf18eae19, 95 },
    { 0xcbdeae19, 81 },
    { 0, 0 },
    { 0x36aeae1c, 95 },
    { 0, 0 },
    { 0x9b9eae1e, 90 },
    { 0xdf8eae1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeeae23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ceae29, 90 },
    { 0xdf7eae29, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eeae31, 98 },
    { 0x7baeae32, 98 },
    { 0, 0 },
    { 0x5eceae34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdeae37, 23 },
    { 0, 0 },
    { 0x7c9eae39, 33 },
    { 0, 0 },
    { 0x76eae3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49beae3e, 34 },
    { 0xc3deae3f, 84 },
    { 0, 0 },
    { 0xfcdeae41, 61 },
    { 0x3a9eae42, 90 },
    { 0xb6eae41, 36 },
    { 0, 0 },
    { 0xc63eae45, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x868eae48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefeae4c, 75 },
    { 0, 0 },
    { 0x274eae4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530eae53, 95 },
    { 0x244eae54, 34 },
    { 0x17eae55, 55 },
    { 0xec8eae55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cceae59, 33 },
    { 0xc8aeae5a, 90 },
    { 0x5fdeae5a, 77 },
    { 0xfa1eae5b, 76 },
    { 0xb63eae59, 33 },
    { 0xc72eae5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eeae62, 55 },
    { 0x633eae63, 49 },
    { 0x2dceae63, 95 },
    { 0x72feae62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4eae68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7eae6e, 55 },
    { 0x84eae6f, 66 },
    { 0, 0 },
    { 0x66eae71, 46 },
    { 0, 0 },
    { 0x8f4eae73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eae76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98eae7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19deae80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312eae8a, 23 },
    { 0, 0 },
    { 0x4e3eae8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beeae91, 100 },
    { 0xcd6eae92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x572eae95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1eae99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852eaea1, 78 },
    { 0xcf8eaea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ceaea5, 55 },
    { 0x182eaea6, 23 },
    { 0xd08eaea5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8eaeac, 85 },
    { 0xd1feaead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10eaeb0, 46 },
    { 0x6a9eaeb0, 33 },
    { 0xaa6eaeb2, 34 },
    { 0x950eaeb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72eaeb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175eaebd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x438eaec0, 43 },
    { 0, 0 },
    { 0xc2beaec2, 90 },
    { 0x5c2eaec3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127eaec9, 78 },
    { 0, 0 },
    { 0x6bceaecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beeaed0, 34 },
    { 0x67eaed0, 78 },
    { 0xeeeeaed0, 95 },
    { 0, 0 },
    { 0xf2beaed4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7eaedd, 88 },
    { 0x840eaede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eaee8, 33 },
    { 0, 0 },
    { 0x402eaeea, 81 },
    { 0, 0 },
    { 0xdafeaeec, 100 },
    { 0x7a6eaeed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeeaef1, 95 },
    { 0, 0 },
    { 0x4c4eaef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6eaef6, 100 },
    { 0, 0 },
    { 0xafeaef8, 61 },
    { 0, 0 },
    { 0xbeaeaefa, 95 },
    { 0xd39eaefa, 99 },
    { 0xf9ceaefb, 23 },
    { 0, 0 },
    { 0xfaaeaefe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0eaf04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9eaf08, 90 },
    { 0, 0 },
    { 0xf34eaf0a, 61 },
    { 0x2eeeaf0a, 33 },
    { 0x650eaf0a, 98 },
    { 0x5dceaf0d, 33 },
    { 0x201eaf0a, 93 },
    { 0x57eeaf0f, 61 },
    { 0xd5deaf10, 33 },
    { 0xd7eeaf0e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfeaf17, 77 },
    { 0x40ceaf18, 33 },
    { 0x3a3eaf19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746eaf23, 33 },
    { 0x12eaf24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6eaf2a, 33 },
    { 0, 0 },
    { 0x2c0eaf2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118eaf30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abeaf37, 33 },
    { 0xc56eaf38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15beaf42, 82 },
    { 0x35deaf43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x592eaf46, 95 },
    { 0xf1eeaf46, 90 },
    { 0, 0 },
    { 0x58feaf49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97eaf4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ceaf4f, 97 },
    { 0x51eeaf4f, 61 },
    { 0xad2eaf51, 87 },
    { 0, 0 },
    { 0x565eaf53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x855eaf56, 46 },
    { 0x2d2eaf57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663eaf5e, 98 },
    { 0xb22eaf5e, 43 },
    { 0xe3eeaf5f, 55 },
    { 0x3c0eaf5e, 90 },
    { 0, 0 },
    { 0x3cbeaf63, 76 },
    { 0, 0 },
    { 0xe69eaf65, 95 },
    { 0, 0 },
    { 0x8c1eaf67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbeaf6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6eaf70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04eaf79, 55 },
    { 0x5e9eaf7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeeaf80, 52 },
    { 0xf13eaf80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117eaf89, 33 },
    { 0xbc5eaf8a, 90 },
    { 0x9a6eaf8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bceaf8e, 33 },
    { 0xd9aeaf8f, 67 },
    { 0xd37eaf90, 91 },
    { 0xd8eaf8e, 90 },
    { 0x679eaf92, 90 },
    { 0, 0 },
    { 0x313eaf94, 52 },
    { 0, 0 },
    { 0x3adeaf96, 90 },
    { 0x79feaf96, 95 },
    { 0x4f8eaf97, 33 },
    { 0xdf7eaf97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedeaf9d, 95 },
    { 0x69ceaf9d, 65 },
    { 0, 0 },
    { 0x93deafa0, 33 },
    { 0x13deafa0, 76 },
    { 0, 0 },
    { 0x1a8eafa3, 23 },
    { 0x916eafa4, 67 },
    { 0x183eafa4, 76 },
    { 0x14feafa5, 61 },
    { 0x95feafa7, 85 },
    { 0x9ebeafa8, 93 },
    { 0x22eeafa8, 92 },
    { 0x8cbeafa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40eafad, 75 },
    { 0, 0 },
    { 0x340eafaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01eafb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264eafb7, 33 },
    { 0xc90eafb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ceafbb, 33 },
    { 0x6ddeafbc, 94 },
    { 0x48deafbb, 40 },
    { 0x3d0eafbe, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0xceeafc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1eafc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6eafc9, 81 },
    { 0, 0 },
    { 0x42aeafcb, 34 },
    { 0x41aeafcc, 33 },
    { 0x2ffeafcc, 98 },
    { 0x101eafce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eafdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8beafde, 98 },
    { 0xdb5eafdf, 78 },
    { 0x2d5eafdf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50feafec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30eafef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9eaff4, 34 },
    { 0, 0 },
    { 0x46feaff6, 33 },
    { 0, 0 },
    { 0x409eaff8, 95 },
    { 0x1e3eaff8, 89 },
    { 0, 0 },
    { 0x45ceaffb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beafff, 95 },
    { 0, 0 },
    { 0x7eceb001, 95 },
    { 0xd1ceb002, 33 },
    { 0, 0 },
    { 0xc9beb004, 90 },
    { 0x39aeb004, 33 },
    { 0, 0 },
    { 0x4cceb007, 98 },
    { 0xdc7eb008, 75 },
    { 0xbc9eb009, 98 },
    { 0x889eb00a, 52 },
    { 0xe5eb00a, 76 },
    { 0x5a1eb00c, 95 },
    { 0xdaeb00d, 61 },
    { 0x7a1eb00c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10eb011, 88 },
    { 0x124eb011, 90 },
    { 0x953eb013, 90 },
    { 0, 0 },
    { 0xfcfeb015, 43 },
    { 0, 0 },
    { 0xfe5eb017, 52 },
    { 0x1e9eb018, 87 },
    { 0xa27eb018, 23 },
    { 0xa88eb01a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eb021, 98 },
    { 0x669eb022, 77 },
    { 0x44feb022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfeb029, 23 },
    { 0x7eceb02a, 61 },
    { 0x89beb02a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1feb033, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415eb039, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfeb03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3eb042, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aeb045, 84 },
    { 0x3a7eb045, 76 },
    { 0, 0 },
    { 0xaf0eb048, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36eb04b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9eb050, 61 },
    { 0, 0 },
    { 0xfceeb052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38eb055, 46 },
    { 0x732eb056, 33 },
    { 0x359eb056, 98 },
    { 0, 0 },
    { 0x11eb059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15beb05c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x78feb05f, 84 },
    { 0, 0 },
    { 0x9d3eb061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5eb064, 91 },
    { 0xac4eb065, 75 },
    { 0x9ceb065, 33 },
    { 0x93ceb067, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358eb06f, 48 },
    { 0xd4eeb06f, 55 },
    { 0xfe4eb071, 61 },
    { 0, 0 },
    { 0xdd8eb073, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3eb076, 61 },
    { 0x1c1eb076, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6eb07a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9eb07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ceb080, 89 },
    { 0xd3feb081, 34 },
    { 0x9c7eb082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbeb086, 95 },
    { 0xccaeb087, 55 },
    { 0xfdceb088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1eb08f, 33 },
    { 0xeeceb090, 33 },
    { 0x289eb08f, 33 },
    { 0x9cdeb092, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4eb095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x294eb098, 33 },
    { 0, 0 },
    { 0x14aeb09a, 55 },
    { 0xc0aeb09b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23eb0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29eb0a5, 17 },
    { 0, 0 },
    { 0x161eb0a7, 95 },
    { 0x265eb0a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7eb0ab, 95 },
    { 0x53feb0ac, 33 },
    { 0x8a4eb0ad, 90 },
    { 0x3ceb0ad, 100 },
    { 0xf19eb0af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1eb0b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713eb0b9, 90 },
    { 0x26beb0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63eb0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677eb0c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb0c7, 33 },
    { 0xe96eb0c8, 99 },
    { 0xd38eb0c8, 98 },
    { 0x717eb0ca, 67 },
    { 0x2f7eb0c7, 92 },
    { 0, 0 },
    { 0x1e5eb0cd, 95 },
    { 0xfcceb0cd, 33 },
    { 0xb49eb0cf, 90 },
    { 0, 0 },
    { 0x279eb0d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614eb0d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2eeb0d8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04eb0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedeb0df, 33 },
    { 0xeffeb0e0, 66 },
    { 0, 0 },
    { 0x24aeb0e2, 91 },
    { 0x8a3eb0e3, 95 },
    { 0, 0 },
    { 0x17beb0e5, 42 },
    { 0x58eeb0e6, 46 },
    { 0x2f0eb0e7, 33 },
    { 0xc9eb0e8, 90 },
    { 0, 0 },
    { 0xa50eb0ea, 81 },
    { 0, 0 },
    { 0xc53eb0ec, 85 },
    { 0xdceeb0ed, 34 },
    { 0, 0 },
    { 0xb57eb0ef, 33 },
    { 0, 0 },
    { 0x52eeb0f1, 52 },
    { 0, 0 },
    { 0xbeceb0f3, 75 },
    { 0, 0 },
    { 0x35feb0f5, 93 },
    { 0x8eb0f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7eb0ff, 40 },
    { 0xd1eeb0ff, 61 },
    { 0, 0 },
    { 0x542eb102, 90 },
    { 0, 0 },
    { 0xa8feb104, 76 },
    { 0x984eb105, 96 },
    { 0xa13eb106, 95 },
    { 0x283eb106, 90 },
    { 0xb25eb108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62eb10c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfeb10f, 68 },
    { 0x306eb110, 46 },
    { 0xf0beb111, 81 },
    { 0x7c1eb110, 100 },
    { 0, 0 },
    { 0x72feb114, 38 },
    { 0xc2eeb114, 90 },
    { 0, 0 },
    { 0x45eeb117, 90 },
    { 0x2c8eb117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4eb11b, 95 },
    { 0xf67eb11b, 90 },
    { 0x2f2eb11d, 38 },
    { 0xda3eb11e, 55 },
    { 0xc26eb11d, 33 },
    { 0x145eb120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb128, 90 },
    { 0, 0 },
    { 0x73aeb12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddeb12f, 66 },
    { 0x7a3eb12f, 98 },
    { 0x145eb131, 33 },
    { 0, 0 },
    { 0xf9beb133, 90 },
    { 0x29beb133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945eb137, 34 },
    { 0, 0 },
    { 0xcc0eb139, 33 },
    { 0x195eb13a, 52 },
    { 0x5dbeb13a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eeb140, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3eb145, 52 },
    { 0xf4ceb146, 35 },
    { 0x5d7eb146, 76 },
    { 0x2c9eb148, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x300eb14b, 55 },
    { 0xda8eb14c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7eb158, 61 },
    { 0, 0 },
    { 0x173eb15a, 43 },
    { 0, 0 },
    { 0x53eeb15c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ceb160, 90 },
    { 0x395eb161, 75 },
    { 0, 0 },
    { 0xf90eb163, 67 },
    { 0x275eb164, 76 },
    { 0xa5eb163, 33 },
    { 0, 0 },
    { 0xb25eb167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2eb16b, 17 },
    { 0, 0 },
    { 0xa6aeb16d, 95 },
    { 0xd5eb16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3eb172, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4eb176, 69 },
    { 0xd6deb176, 100 },
    { 0x156eb178, 88 },
    { 0x81eb178, 90 },
    { 0x20ceb17a, 98 },
    { 0x66eeb17b, 95 },
    { 0xaaeb17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98eb183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eb187, 100 },
    { 0xa2eb187, 34 },
    { 0xe54eb187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdeb18c, 61 },
    { 0x764eb18c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deb199, 90 },
    { 0x748eb19a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aeb1a7, 30 },
    { 0x6a0eb1a7, 34 },
    { 0xbeeb1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99feb1b1, 76 },
    { 0xf87eb1b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x703eb1b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59eb1bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefeeb1c0, 46 },
    { 0x442eb1c1, 33 },
    { 0x706eb1c2, 55 },
    { 0xd70eb1c3, 23 },
    { 0xae4eb1c3, 17 },
    { 0, 0 },
    { 0x934eb1c6, 23 },
    { 0x535eb1c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485eb1d5, 33 },
    { 0, 0 },
    { 0xf2deb1d7, 33 },
    { 0xb8aeb1d8, 51 },
    { 0, 0 },
    { 0x314eb1da, 33 },
    { 0xe43eb1db, 90 },
    { 0, 0 },
    { 0xbb3eb1dd, 76 },
    { 0x7e2eb1de, 61 },
    { 0x97eb1de, 91 },
    { 0xc04eb1e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfeb1e3, 45 },
    { 0x161eb1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eb1e7, 55 },
    { 0xb65eb1e8, 81 },
    { 0xceeb1e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ceb1f1, 77 },
    { 0xa40eb1f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787eb1fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51eb1fd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45deb208, 67 },
    { 0, 0 },
    { 0xc0aeb20a, 99 },
    { 0, 0 },
    { 0x9f4eb20c, 90 },
    { 0x53eb20d, 33 },
    { 0x9b0eb20c, 23 },
    { 0x66feb20f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0eb214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434eb218, 90 },
    { 0x212eb219, 23 },
    { 0, 0 },
    { 0xd69eb21b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40eb21e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2eb227, 43 },
    { 0, 0 },
    { 0x7e3eb229, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97beb238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833eb23c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11eb247, 52 },
    { 0, 0 },
    { 0xeafeb249, 33 },
    { 0xd0aeb24a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5eb255, 46 },
    { 0, 0 },
    { 0x3ddeb257, 37 },
    { 0xe57eb258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16eb25d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aeb262, 90 },
    { 0xd24eb263, 61 },
    { 0xe9feb263, 78 },
    { 0x215eb265, 95 },
    { 0, 0 },
    { 0xb8eeb267, 95 },
    { 0x417eb268, 52 },
    { 0, 0 },
    { 0x542eb26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aeb26d, 98 },
    { 0x8deeb26e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaeb272, 43 },
    { 0x18aeb273, 64 },
    { 0x878eb274, 98 },
    { 0xd4ceb275, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2eb278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66feb27e, 52 },
    { 0, 0 },
    { 0xf33eb280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7eb283, 61 },
    { 0x5e6eb284, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebeb28a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ceb28d, 43 },
    { 0x549eb28e, 46 },
    { 0x634eb28f, 44 },
    { 0, 0 },
    { 0x684eb291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65eb295, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67beb299, 43 },
    { 0x9d9eb29a, 23 },
    { 0x2dceb29b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ceb2a1, 90 },
    { 0xd10eb2a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491eb2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8eb2b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993eb2b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5eb2bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7eb2c5, 33 },
    { 0x68feb2c6, 88 },
    { 0x3c0eb2c5, 33 },
    { 0x849eb2c8, 55 },
    { 0x75aeb2c7, 34 },
    { 0, 0 },
    { 0x68beb2cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319eb2d0, 95 },
    { 0x88eb2d1, 61 },
    { 0x22deb2d2, 77 },
    { 0xdb4eb2d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6eb2d7, 46 },
    { 0x175eb2d8, 66 },
    { 0x1c6eb2d8, 90 },
    { 0xdceeb2da, 61 },
    { 0x5d0eb2da, 61 },
    { 0x2adeb2dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x807eb2df, 33 },
    { 0, 0 },
    { 0x8f1eb2e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949eb2e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aeb2ee, 98 },
    { 0x69beb2ef, 88 },
    { 0x8afeb2f0, 98 },
    { 0xcaceb2f1, 98 },
    { 0xa33eb2f1, 98 },
    { 0x7c9eb2f0, 23 },
    { 0x128eb2f4, 76 },
    { 0x13eeb2f5, 76 },
    { 0x158eb2f2, 90 },
    { 0xc14eb2f7, 98 },
    { 0x876eb2f6, 76 },
    { 0x1c8eb2f9, 76 },
    { 0xc2ceb2fa, 23 },
    { 0xfb8eb2f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aeb2fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5eb306, 77 },
    { 0x5baeb306, 78 },
    { 0xd7beb307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83eb30b, 46 },
    { 0x230eb30b, 91 },
    { 0xffaeb30c, 23 },
    { 0x369eb30b, 90 },
    { 0, 0 },
    { 0x52deb310, 98 },
    { 0, 0 },
    { 0x3baeb312, 99 },
    { 0x947eb312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb316, 23 },
    { 0x54eb317, 98 },
    { 0xa1feb318, 53 },
    { 0xcedeb318, 66 },
    { 0xe25eb316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77beb31d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ceb321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953eb32c, 93 },
    { 0xd20eb32d, 33 },
    { 0xb03eb32c, 23 },
    { 0, 0 },
    { 0x6c6eb330, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09eb333, 23 },
    { 0x40beb333, 100 },
    { 0x77feb335, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3feb338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29eb33b, 80 },
    { 0x47feb33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64feb344, 97 },
    { 0x557eb345, 23 },
    { 0x75deb346, 76 },
    { 0, 0 },
    { 0x8a8eb348, 84 },
    { 0, 0 },
    { 0xcb7eb34a, 33 },
    { 0xd25eb34a, 8 },
    { 0x476eb34a, 98 },
    { 0x38eb34c, 76 },
    { 0xec1eb34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171eb352, 95 },
    { 0, 0 },
    { 0x68feb354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02eb359, 69 },
    { 0x4f0eb35a, 46 },
    { 0x6feeb35b, 88 },
    { 0x7e2eb35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb36a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb36f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7eb374, 98 },
    { 0, 0 },
    { 0xa66eb376, 33 },
    { 0x768eb377, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaeb37d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509eb385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698eb38d, 99 },
    { 0, 0 },
    { 0x82beb38f, 23 },
    { 0x66feb38f, 23 },
    { 0, 0 },
    { 0x531eb392, 87 },
    { 0xfb3eb393, 33 },
    { 0xa5eeb394, 87 },
    { 0, 0 },
    { 0xe9aeb396, 33 },
    { 0xb53eb396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x381eb39a, 33 },
    { 0xd5eeb39b, 23 },
    { 0, 0 },
    { 0xe92eb39d, 98 },
    { 0xb1ceb39e, 77 },
    { 0x532eb39e, 33 },
    { 0xa62eb39e, 52 },
    { 0x4d8eb3a1, 95 },
    { 0x1caeb39e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cceb3a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaeb3ac, 77 },
    { 0xa37eb3ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4eb3b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eb3b4, 85 },
    { 0, 0 },
    { 0x46deb3b6, 23 },
    { 0x849eb3b7, 98 },
    { 0, 0 },
    { 0x108eb3b9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09eb3be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00eb3c4, 95 },
    { 0xaa4eb3c5, 23 },
    { 0xa4ceb3c6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x76beb3c9, 90 },
    { 0, 0 },
    { 0x49ceb3cb, 33 },
    { 0x201eb3cb, 43 },
    { 0x314eb3cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80feb3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0eb3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121eb3d8, 23 },
    { 0x91beb3d9, 100 },
    { 0x6e4eb3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eceb3e0, 90 },
    { 0, 0 },
    { 0x80eeb3e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3eb3e8, 90 },
    { 0x898eb3e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1eb3ec, 85 },
    { 0x31ceb3ed, 55 },
    { 0, 0 },
    { 0x7aceb3ef, 76 },
    { 0xa74eb3f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aceb3f5, 91 },
    { 0xc25eb3f6, 34 },
    { 0, 0 },
    { 0xc57eb3f8, 69 },
    { 0x864eb3f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efeb3fc, 99 },
    { 0, 0 },
    { 0xb1beb3fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9eb401, 46 },
    { 0x63eeb402, 66 },
    { 0x5b9eb401, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39feb406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3eb409, 38 },
    { 0xceb409, 61 },
    { 0x8b4eb40a, 66 },
    { 0x7b3eb40c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38eb411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2deb41a, 84 },
    { 0x484eb41a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458eb41f, 33 },
    { 0xeb5eb41f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48eb423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78aeb42b, 76 },
    { 0xc13eb42c, 91 },
    { 0xe2eb42c, 91 },
    { 0x4e7eb42e, 90 },
    { 0xf63eb42f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38eb435, 90 },
    { 0x928eb436, 78 },
    { 0x1c6eb437, 90 },
    { 0xe05eb435, 55 },
    { 0, 0 },
    { 0x230eb43a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8eb43e, 75 },
    { 0x170eb43f, 90 },
    { 0, 0 },
    { 0xc79eb441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78deb444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824eb448, 67 },
    { 0x465eb449, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x324eb44c, 33 },
    { 0, 0 },
    { 0x8deeb44e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x447eb451, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aeb454, 74 },
    { 0x40beb454, 98 },
    { 0x96eeb456, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0eb459, 95 },
    { 0x155eb459, 90 },
    { 0xe1beb45b, 23 },
    { 0x2faeb45b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366eb466, 4 },
    { 0, 0 },
    { 0xb18eb468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37eb46b, 55 },
    { 0x8c7eb46c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32aeb470, 33 },
    { 0, 0 },
    { 0x97aeb472, 61 },
    { 0x576eb473, 33 },
    { 0xc23eb474, 55 },
    { 0xdfdeb475, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187eb47b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beb481, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6eb485, 61 },
    { 0x37aeb486, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7eb48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46aeb492, 8 },
    { 0, 0 },
    { 0xa41eb494, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7eb497, 90 },
    { 0, 0 },
    { 0x1e6eb499, 99 },
    { 0xa38eb49a, 95 },
    { 0x468eb49b, 90 },
    { 0x53aeb49c, 100 },
    { 0x8bdeb49d, 90 },
    { 0xd1beb49e, 90 },
    { 0xa7deb49e, 61 },
    { 0x19ceb4a0, 33 },
    { 0x509eb49f, 23 },
    { 0xda6eb4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96feb4a6, 38 },
    { 0x95deb4a6, 95 },
    { 0x1c6eb4a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x69eb4ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5eb4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbeb4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eeb4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eceb4bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314eb4c5, 61 },
    { 0xecfeb4c6, 76 },
    { 0x791eb4c7, 95 },
    { 0xa6ceb4c6, 61 },
    { 0x389eb4c9, 23 },
    { 0x28feb4ca, 33 },
    { 0xbb6eb4cb, 33 },
    { 0x54eeb4cb, 61 },
    { 0x5eb4ca, 44 },
    { 0xdc5eb4ce, 61 },
    { 0x7d4eb4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462eb4d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffeb4da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394eb4e2, 78 },
    { 0x7d0eb4e3, 76 },
    { 0, 0 },
    { 0xacdeb4e5, 61 },
    { 0, 0 },
    { 0x9daeb4e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4eb4ea, 76 },
    { 0, 0 },
    { 0x8e7eb4ec, 43 },
    { 0x19beb4ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63deb4f5, 87 },
    { 0xfe4eb4f6, 91 },
    { 0xcafeb4f6, 95 },
    { 0x340eb4f8, 87 },
    { 0x5a9eb4f8, 76 },
    { 0x82aeb4f5, 98 },
    { 0x498eb4fb, 76 },
    { 0, 0 },
    { 0xc8deb4fd, 43 },
    { 0xb4feb4fd, 95 },
    { 0, 0 },
    { 0xbe1eb500, 23 },
    { 0, 0 },
    { 0x3cceb502, 46 },
    { 0x278eb503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21deb507, 64 },
    { 0xcbfeb507, 33 },
    { 0x6dfeb507, 98 },
    { 0, 0 },
    { 0xb43eb50b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44eb513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aeb516, 34 },
    { 0x725eb517, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8eb51e, 88 },
    { 0x6c5eb51e, 76 },
    { 0xd3eeb51e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aeb523, 23 },
    { 0, 0 },
    { 0x40eb525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318eb52a, 90 },
    { 0, 0 },
    { 0x728eb52c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94eb533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbeb537, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aeb53b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eeb541, 52 },
    { 0x32eeb542, 95 },
    { 0xcdaeb543, 95 },
    { 0xd5feb544, 98 },
    { 0x7cceb544, 85 },
    { 0xf0ceb545, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deb54a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffeb554, 76 },
    { 0x741eb554, 100 },
    { 0x1b7eb556, 23 },
    { 0x67feb557, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0eb55b, 84 },
    { 0xf21eb55b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x23deb55f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47eb563, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ceb56c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0eb571, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eeb57e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aeb583, 43 },
    { 0x651eb584, 36 },
    { 0xe41eb584, 33 },
    { 0x439eb586, 90 },
    { 0x6e7eb585, 33 },
    { 0, 0 },
    { 0xdc4eb589, 75 },
    { 0, 0 },
    { 0x555eb58b, 78 },
    { 0, 0 },
    { 0xf90eb58d, 17 },
    { 0xae2eb58e, 75 },
    { 0x755eb58e, 90 },
    { 0xc46eb58e, 52 },
    { 0x1e1eb591, 75 },
    { 0, 0 },
    { 0x93feb593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeeb59b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81eb5a0, 90 },
    { 0xfa6eb5a1, 33 },
    { 0x738eb5a0, 90 },
    { 0x910eb5a0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ceb5aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eeb5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112eb5b5, 90 },
    { 0xc14eb5b6, 99 },
    { 0xd9feb5b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaeb5bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3eb5be, 34 },
    { 0, 0 },
    { 0x774eb5c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aeb5c5, 61 },
    { 0xcbeb5c5, 90 },
    { 0x5eaeb5c7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20eb5cc, 61 },
    { 0x109eb5cc, 33 },
    { 0, 0 },
    { 0x572eb5cf, 100 },
    { 0x5b1eb5d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfeb5d6, 95 },
    { 0, 0 },
    { 0xceeeb5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66eb5db, 80 },
    { 0x7cfeb5dc, 33 },
    { 0x60beb5dc, 33 },
    { 0x9deeb5db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585eb5e3, 33 },
    { 0, 0 },
    { 0x70feb5e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323eb5e9, 61 },
    { 0, 0 },
    { 0xc44eb5eb, 95 },
    { 0xfa4eb5eb, 90 },
    { 0x13aeb5ed, 90 },
    { 0, 0 },
    { 0xf5eb5ef, 33 },
    { 0xe21eb5f0, 33 },
    { 0xcc1eb5ef, 90 },
    { 0x6faeb5f2, 43 },
    { 0x496eb5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aeb5f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feeb5fc, 78 },
    { 0xc80eb5fc, 33 },
    { 0x6cceb5fc, 98 },
    { 0x9d3eb5ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18eb606, 99 },
    { 0x8a1eb607, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4eb60e, 55 },
    { 0x9c2eb60f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88eb612, 33 },
    { 0xe75eb612, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71deb618, 95 },
    { 0, 0 },
    { 0xe4deb61a, 61 },
    { 0x904eb61b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237eb621, 100 },
    { 0x6bceb622, 98 },
    { 0x1edeb623, 88 },
    { 0, 0 },
    { 0x11beb625, 46 },
    { 0xd0eeb625, 61 },
    { 0x50beb627, 61 },
    { 0xe4deb628, 100 },
    { 0xc74eb627, 61 },
    { 0x1e5eb627, 99 },
    { 0x231eb629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53eb62f, 90 },
    { 0xb42eb630, 90 },
    { 0x4d9eb631, 76 },
    { 0xb7ceb632, 38 },
    { 0xf21eb62f, 95 },
    { 0, 0 },
    { 0xe5deb635, 98 },
    { 0xcaceb636, 95 },
    { 0, 0 },
    { 0xd5eeb638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51eb63f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9beb648, 95 },
    { 0xb99eb649, 95 },
    { 0, 0 },
    { 0xf16eb64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eeb653, 95 },
    { 0x604eb653, 46 },
    { 0x449eb655, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eeb659, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aeb65d, 33 },
    { 0x9c9eb65d, 33 },
    { 0x5edeb65f, 23 },
    { 0x7c9eb660, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73beb664, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6eb668, 17 },
    { 0, 0 },
    { 0x225eb66a, 33 },
    { 0x501eb66b, 90 },
    { 0, 0 },
    { 0x804eb66d, 61 },
    { 0, 0 },
    { 0xf7eeb66f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440eb673, 55 },
    { 0, 0 },
    { 0xadfeb675, 90 },
    { 0x81eb676, 23 },
    { 0xf2aeb677, 61 },
    { 0xe40eb675, 77 },
    { 0, 0 },
    { 0xaf2eb67a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddeb67f, 98 },
    { 0x49beb680, 90 },
    { 0, 0 },
    { 0xbbdeb682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1eb685, 69 },
    { 0x459eb686, 90 },
    { 0x9f3eb686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x455eb68a, 61 },
    { 0, 0 },
    { 0x8f0eb68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eb68f, 81 },
    { 0xcb3eb690, 43 },
    { 0x185eb690, 100 },
    { 0x155eb692, 98 },
    { 0x526eb692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dceb698, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadeb69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaeb69e, 46 },
    { 0x181eb69f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x588eb6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69eb6a5, 76 },
    { 0, 0 },
    { 0xbc6eb6a7, 98 },
    { 0x1d1eb6a7, 34 },
    { 0, 0 },
    { 0xb4ceb6aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50eb6af, 76 },
    { 0xdeceb6b0, 95 },
    { 0, 0 },
    { 0x823eb6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x416eb6b5, 61 },
    { 0x9aeeb6b6, 91 },
    { 0x6c2eb6b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07eb6ba, 34 },
    { 0xcceeb6bb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9eb6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eeb6c4, 23 },
    { 0x940eb6c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4beb6c9, 81 },
    { 0x47beb6ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9eb6cd, 90 },
    { 0x736eb6cd, 93 },
    { 0x595eb6cf, 55 },
    { 0xdafeb6d0, 90 },
    { 0x82aeb6d1, 34 },
    { 0x21beb6d0, 98 },
    { 0x2feb6d3, 67 },
    { 0x158eb6d3, 33 },
    { 0x1bceb6cf, 85 },
    { 0, 0 },
    { 0x278eb6d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5eb6db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1eb6df, 33 },
    { 0xdaeb6e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x796eb6e3, 34 },
    { 0xad4eb6e4, 61 },
    { 0, 0 },
    { 0xf62eb6e6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daeb6ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62eb6f0, 76 },
    { 0xeeeb6f1, 65 },
    { 0xe9ceb6f2, 34 },
    { 0, 0 },
    { 0x7b3eb6f4, 95 },
    { 0x704eb6f4, 55 },
    { 0x934eb6f6, 77 },
    { 0x4e7eb6f7, 76 },
    { 0x43beb6f8, 91 },
    { 0xd5eeb6f9, 76 },
    { 0xd7ceb6f9, 33 },
    { 0x18eb6f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aeb6fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfeb703, 43 },
    { 0, 0 },
    { 0xdc1eb705, 46 },
    { 0x7a0eb706, 61 },
    { 0xfe8eb706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fceb70c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eb714, 65 },
    { 0, 0 },
    { 0xfb5eb716, 23 },
    { 0x2a7eb717, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404eb71c, 33 },
    { 0xfabeb71d, 98 },
    { 0x76feb71c, 98 },
    { 0, 0 },
    { 0x35eeb720, 99 },
    { 0xe1eeb721, 55 },
    { 0x43feb722, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eb726, 90 },
    { 0, 0 },
    { 0x69beb728, 34 },
    { 0xe58eb729, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eeb72c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x479eb72f, 95 },
    { 0xc6eb730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eeb736, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10deb739, 95 },
    { 0, 0 },
    { 0xaf3eb73b, 61 },
    { 0, 0 },
    { 0x442eb73d, 91 },
    { 0x4ddeb73e, 78 },
    { 0x18aeb73f, 95 },
    { 0x205eb73f, 66 },
    { 0x2cdeb73f, 95 },
    { 0, 0 },
    { 0x7eeeb743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79eb74b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5beb751, 77 },
    { 0xfddeb752, 33 },
    { 0xdc1eb751, 87 },
    { 0xd6feb754, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47deb757, 23 },
    { 0x907eb758, 90 },
    { 0x121eb759, 95 },
    { 0x375eb75a, 61 },
    { 0xc86eb75b, 68 },
    { 0x876eb757, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282eb765, 90 },
    { 0x9eeb765, 61 },
    { 0x25feb765, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeeb76b, 66 },
    { 0, 0 },
    { 0x184eb76d, 95 },
    { 0xbcfeb76e, 90 },
    { 0x64feb76e, 98 },
    { 0, 0 },
    { 0x23deb771, 33 },
    { 0xaafeb772, 17 },
    { 0xa85eb773, 52 },
    { 0, 0 },
    { 0xb1eeb775, 87 },
    { 0x73eb776, 90 },
    { 0x22feb777, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859eb78d, 68 },
    { 0x719eb78e, 90 },
    { 0xe61eb78e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafceb799, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362eb79d, 33 },
    { 0, 0 },
    { 0xfbeeb79f, 95 },
    { 0xfb5eb7a0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72eb7a4, 95 },
    { 0xdfeb7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716eb7a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x183eb7ac, 43 },
    { 0, 0 },
    { 0x2b1eb7ae, 33 },
    { 0xf1beb7af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x570eb7b2, 46 },
    { 0x740eb7b3, 33 },
    { 0, 0 },
    { 0xc3eeb7b5, 61 },
    { 0xc07eb7b5, 66 },
    { 0, 0 },
    { 0xe52eb7b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836eb7be, 43 },
    { 0x752eb7be, 24 },
    { 0xeedeb7c0, 61 },
    { 0x939eb7c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3deb7c8, 89 },
    { 0x497eb7c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1beb7cd, 23 },
    { 0xe9feb7ce, 95 },
    { 0, 0 },
    { 0x9b4eb7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x205eb7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdeb7db, 76 },
    { 0xfe4eb7db, 93 },
    { 0x8caeb7dd, 90 },
    { 0xda3eb7de, 61 },
    { 0, 0 },
    { 0x80beb7e0, 69 },
    { 0xb1eb7e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3eb7f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328eb7f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491eb7fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7eb803, 4 },
    { 0x13ceb804, 90 },
    { 0x576eb804, 90 },
    { 0xabaeb804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eeb80d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37eb812, 46 },
    { 0xf9feb813, 69 },
    { 0x1fbeb814, 95 },
    { 0xbdaeb813, 33 },
    { 0x7feeb816, 33 },
    { 0xcddeb815, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23eb81a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebeb81f, 90 },
    { 0, 0 },
    { 0xd9deb821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6feb826, 43 },
    { 0xc23eb827, 98 },
    { 0x313eb828, 33 },
    { 0, 0 },
    { 0x581eb82a, 90 },
    { 0xa78eb82b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x108eb82e, 61 },
    { 0, 0 },
    { 0x412eb830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44eb835, 80 },
    { 0, 0 },
    { 0xccdeb837, 90 },
    { 0, 0 },
    { 0x1aeb839, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4eb83c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53deb845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeeb848, 33 },
    { 0x7daeb849, 81 },
    { 0, 0 },
    { 0xe4eeb84b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244eb84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3eb853, 61 },
    { 0xbe1eb854, 33 },
    { 0, 0 },
    { 0x343eb856, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeb85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1eb860, 23 },
    { 0, 0 },
    { 0x6caeb862, 97 },
    { 0x72beb863, 33 },
    { 0x9a8eb864, 95 },
    { 0x25eb863, 23 },
    { 0xa66eb865, 61 },
    { 0x193eb867, 90 },
    { 0x455eb863, 90 },
    { 0xb9eb869, 66 },
    { 0x2b3eb86a, 61 },
    { 0x643eb864, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943eb86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321eb875, 34 },
    { 0x93aeb876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4feb87f, 67 },
    { 0x86eb880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfeb883, 33 },
    { 0, 0 },
    { 0xf1eeb885, 34 },
    { 0x144eb886, 90 },
    { 0x68deb886, 90 },
    { 0, 0 },
    { 0xedeeb889, 55 },
    { 0x203eb88a, 100 },
    { 0, 0 },
    { 0x9e2eb88c, 23 },
    { 0xd01eb88c, 76 },
    { 0, 0 },
    { 0xe66eb88f, 8 },
    { 0xe97eb88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19eb893, 43 },
    { 0x8deeb894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x799eb897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02eb89a, 33 },
    { 0x7b2eb89b, 99 },
    { 0xccfeb89b, 33 },
    { 0x292eb89d, 88 },
    { 0x173eb89d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ceb8a1, 23 },
    { 0xb1beb8a2, 61 },
    { 0x16ceb8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786eb8a6, 52 },
    { 0, 0 },
    { 0xea0eb8a8, 34 },
    { 0xeffeb8a9, 76 },
    { 0, 0 },
    { 0x109eb8ab, 90 },
    { 0x196eb8ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18deb8b1, 23 },
    { 0, 0 },
    { 0x988eb8b3, 90 },
    { 0, 0 },
    { 0x156eb8b5, 61 },
    { 0, 0 },
    { 0x667eb8b7, 33 },
    { 0x6e7eb8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x795eb8bb, 33 },
    { 0x621eb8bc, 53 },
    { 0, 0 },
    { 0xf31eb8be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89beb8c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ceb8c5, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfeb8ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6eb8cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6eb8d4, 98 },
    { 0x773eb8d4, 95 },
    { 0, 0 },
    { 0xecaeb8d7, 87 },
    { 0x501eb8d8, 66 },
    { 0x318eb8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40deb8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685eb8e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58beb8e6, 93 },
    { 0xe17eb8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16eb8eb, 90 },
    { 0xed5eb8eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bceb8f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21eb8fa, 61 },
    { 0, 0 },
    { 0x8efeb8fc, 95 },
    { 0x7d8eb8fd, 87 },
    { 0x971eb8fc, 23 },
    { 0xb32eb8ff, 33 },
    { 0xed7eb8ff, 61 },
    { 0xb2deb901, 95 },
    { 0x62eeb8ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eeb90a, 80 },
    { 0, 0 },
    { 0x7a9eb90c, 84 },
    { 0, 0 },
    { 0x89aeb90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778eb913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3eb916, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4eb91f, 95 },
    { 0x8fdeb920, 17 },
    { 0, 0 },
    { 0x436eb922, 61 },
    { 0, 0 },
    { 0xc21eb924, 33 },
    { 0, 0 },
    { 0x1e9eb926, 90 },
    { 0, 0 },
    { 0xcddeb928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ceb92b, 23 },
    { 0x9eeb92c, 55 },
    { 0, 0 },
    { 0x1f3eb92e, 98 },
    { 0, 0 },
    { 0x7a7eb930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9eb933, 61 },
    { 0x656eb934, 80 },
    { 0x729eb935, 93 },
    { 0xa00eb934, 95 },
    { 0, 0 },
    { 0xae8eb938, 35 },
    { 0x562eb938, 33 },
    { 0x5c9eb93a, 33 },
    { 0x1a8eb93a, 76 },
    { 0x53feb93c, 23 },
    { 0x45beb93c, 43 },
    { 0xaceeb93a, 61 },
    { 0, 0 },
    { 0x610eb940, 95 },
    { 0, 0 },
    { 0x308eb942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceb948, 76 },
    { 0x320eb949, 61 },
    { 0xd01eb949, 90 },
    { 0, 0 },
    { 0xb8beb94c, 52 },
    { 0xa80eb94d, 99 },
    { 0xa92eb94e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaeb951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40eb954, 33 },
    { 0x2baeb955, 90 },
    { 0, 0 },
    { 0x6aeeb957, 90 },
    { 0, 0 },
    { 0x3b0eb959, 46 },
    { 0xe0ceb95a, 33 },
    { 0xde2eb95a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4eb95e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eeb961, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x31feb964, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eb969, 90 },
    { 0, 0 },
    { 0x188eb96b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2eb96e, 34 },
    { 0x2f7eb96f, 95 },
    { 0x5b7eb96f, 90 },
    { 0x6deb971, 80 },
    { 0xab7eb971, 78 },
    { 0xa22eb973, 91 },
    { 0x41eb973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06eb978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x392eb97b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130eb97f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32eb986, 23 },
    { 0, 0 },
    { 0xfcceb988, 76 },
    { 0x432eb989, 95 },
    { 0xe71eb988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36eb98f, 46 },
    { 0, 0 },
    { 0x7c0eb991, 72 },
    { 0, 0 },
    { 0x352eb993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ceb99c, 100 },
    { 0, 0 },
    { 0x278eb99e, 76 },
    { 0xc05eb99f, 33 },
    { 0x80feb9a0, 34 },
    { 0x646eb9a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aeb9a5, 98 },
    { 0xc3ceb9a6, 61 },
    { 0x9dbeb9a6, 61 },
    { 0x497eb9a8, 76 },
    { 0xaefeb9a9, 88 },
    { 0x87ceb9a9, 95 },
    { 0xfa5eb9a8, 33 },
    { 0x9f3eb9ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9eb9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabeb9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234eb9bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x810eb9be, 68 },
    { 0, 0 },
    { 0x9c8eb9c0, 98 },
    { 0, 0 },
    { 0xd69eb9c2, 87 },
    { 0x965eb9c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ceb9c6, 91 },
    { 0, 0 },
    { 0x160eb9c8, 99 },
    { 0x16eeb9c9, 76 },
    { 0x933eb9ca, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x564eb9cd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59eb9d4, 34 },
    { 0x9d9eb9d5, 87 },
    { 0x7c4eb9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52eb9e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1eb9e3, 95 },
    { 0, 0 },
    { 0xd59eb9e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94eb9ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3eb9f0, 87 },
    { 0, 0 },
    { 0xf37eb9f2, 23 },
    { 0x1d6eb9f3, 36 },
    { 0, 0 },
    { 0x4bceb9f5, 61 },
    { 0, 0 },
    { 0x49deb9f7, 40 },
    { 0xb5eeb9f8, 61 },
    { 0xf64eb9f8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aeb9ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11eba02, 84 },
    { 0xaa4eba03, 88 },
    { 0x768eba04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdeba08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458eba0c, 95 },
    { 0xc07eba0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ceba1d, 61 },
    { 0xf0eeba1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deba24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753eba2c, 90 },
    { 0x7abeba2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22eba32, 46 },
    { 0xfb9eba32, 95 },
    { 0x84deba34, 33 },
    { 0xcefeba35, 46 },
    { 0, 0 },
    { 0x913eba37, 55 },
    { 0x84eba37, 90 },
    { 0, 0 },
    { 0x8e9eba3a, 76 },
    { 0xd24eba3b, 43 },
    { 0x99aeba3c, 87 },
    { 0x13aeba3d, 97 },
    { 0xc46eba3d, 99 },
    { 0x5dfeba3c, 90 },
    { 0xfbeba3a, 65 },
    { 0x161eba3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754eba46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4eba49, 90 },
    { 0xf3beba4a, 91 },
    { 0x444eba4a, 66 },
    { 0x5dceba4c, 33 },
    { 0x68beba4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53eba53, 33 },
    { 0x45eeba54, 23 },
    { 0x59aeba55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8eba59, 76 },
    { 0xc86eba59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02eba5f, 90 },
    { 0xc09eba60, 61 },
    { 0x8c2eba5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93aeba66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40eba6e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8eba75, 61 },
    { 0, 0 },
    { 0x5a4eba77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30eba7a, 90 },
    { 0x39beba7b, 23 },
    { 0xe25eba7c, 90 },
    { 0x88beba7c, 61 },
    { 0x9aeba7d, 23 },
    { 0xd60eba7d, 90 },
    { 0x57aeba80, 46 },
    { 0x81deba7c, 93 },
    { 0x968eba82, 66 },
    { 0xe70eba7f, 90 },
    { 0xbb0eba7b, 61 },
    { 0x4deba85, 67 },
    { 0xe8eeba7e, 90 },
    { 0xa11eba85, 68 },
    { 0xec9eba81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1eba91, 33 },
    { 0, 0 },
    { 0x992eba93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaeba97, 64 },
    { 0x30eba97, 98 },
    { 0, 0 },
    { 0xb9deba9a, 97 },
    { 0x4bceba9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547ebaa0, 23 },
    { 0, 0 },
    { 0x376ebaa2, 81 },
    { 0x9cbebaa2, 33 },
    { 0x7fdebaa3, 51 },
    { 0xc1bebaa4, 55 },
    { 0, 0 },
    { 0xbf0ebaa7, 67 },
    { 0xa62ebaa8, 61 },
    { 0xdaaebaa8, 100 },
    { 0xa1ebaa7, 90 },
    { 0xaadebaab, 59 },
    { 0, 0 },
    { 0xc5febaad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfebab1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ebab9, 95 },
    { 0x743ebaba, 76 },
    { 0xac0ebabb, 95 },
    { 0x34febabc, 53 },
    { 0x297ebabc, 77 },
    { 0, 0 },
    { 0x802ebabf, 34 },
    { 0x86aebabf, 33 },
    { 0xc01ebac1, 95 },
    { 0, 0 },
    { 0x174ebac3, 33 },
    { 0x4c2ebac4, 91 },
    { 0xf4ebac4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ebac9, 95 },
    { 0, 0 },
    { 0x30febacb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ebad1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbebad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cebad9, 90 },
    { 0, 0 },
    { 0xc84ebadb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60febadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41ebae3, 99 },
    { 0x94bebae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660ebaeb, 90 },
    { 0x3e4ebaeb, 55 },
    { 0, 0 },
    { 0x34ebaee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bebaf1, 99 },
    { 0, 0 },
    { 0xcdfebaf3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aebaf6, 17 },
    { 0xf29ebaf7, 40 },
    { 0xd53ebaf8, 64 },
    { 0x11eebaf9, 48 },
    { 0x28aebaf7, 52 },
    { 0x120ebaf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x277ebafe, 87 },
    { 0, 0 },
    { 0x4cebb00, 33 },
    { 0xa01ebb01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ebb05, 98 },
    { 0x404ebb06, 59 },
    { 0, 0 },
    { 0xd08ebb08, 61 },
    { 0xbb3ebb08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5febb0c, 76 },
    { 0, 0 },
    { 0xebaebb0e, 55 },
    { 0x2b2ebb0f, 61 },
    { 0, 0 },
    { 0x136ebb11, 78 },
    { 0, 0 },
    { 0xd00ebb13, 98 },
    { 0xbdbebb14, 68 },
    { 0x287ebb15, 94 },
    { 0x1efebb16, 61 },
    { 0xefbebb17, 17 },
    { 0x2ccebb16, 80 },
    { 0x518ebb19, 98 },
    { 0x279ebb1a, 55 },
    { 0x1dbebb1a, 33 },
    { 0x33debb1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ebb22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ebb26, 33 },
    { 0, 0 },
    { 0x42cebb28, 84 },
    { 0x9cfebb29, 90 },
    { 0xd21ebb2a, 95 },
    { 0x5dfebb29, 76 },
    { 0xbdeebb2c, 33 },
    { 0xf24ebb2d, 98 },
    { 0xe12ebb2d, 61 },
    { 0xd62ebb2c, 75 },
    { 0x664ebb30, 76 },
    { 0xc72ebb31, 95 },
    { 0x559ebb31, 90 },
    { 0xd8aebb33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09ebb37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x608ebb3a, 95 },
    { 0x89cebb3b, 33 },
    { 0xb50ebb3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2febb40, 61 },
    { 0xfdeebb40, 93 },
    { 0x970ebb42, 99 },
    { 0x9ddebb43, 90 },
    { 0xcc0ebb44, 61 },
    { 0, 0 },
    { 0xb22ebb46, 90 },
    { 0xaf7ebb47, 98 },
    { 0xe46ebb47, 91 },
    { 0x6c8ebb48, 55 },
    { 0x8d1ebb49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9ebb50, 46 },
    { 0x42febb51, 88 },
    { 0x63febb51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1ebb57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ebb5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429ebb61, 67 },
    { 0, 0 },
    { 0xccaebb63, 84 },
    { 0xe42ebb63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1ebb6a, 85 },
    { 0xc43ebb6b, 91 },
    { 0, 0 },
    { 0xda0ebb6d, 67 },
    { 0xbeebb6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ebb71, 77 },
    { 0, 0 },
    { 0x588ebb73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ebb7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8ebb7e, 98 },
    { 0x22bebb7f, 42 },
    { 0x741ebb7e, 34 },
    { 0, 0 },
    { 0x9afebb82, 23 },
    { 0x61ebb82, 61 },
    { 0x3faebb82, 75 },
    { 0, 0 },
    { 0x944ebb86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ebb8c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6ebb92, 90 },
    { 0x24aebb92, 55 },
    { 0x113ebb93, 95 },
    { 0xff2ebb95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eebb9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ebba2, 90 },
    { 0, 0 },
    { 0x728ebba4, 61 },
    { 0x4c4ebba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495ebba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ebbad, 55 },
    { 0, 0 },
    { 0xd6aebbaf, 33 },
    { 0, 0 },
    { 0x5eebbb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7ebbb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ebbb8, 23 },
    { 0x3dcebbb9, 83 },
    { 0x69febbba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ebbbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33ebbc1, 90 },
    { 0xe6eebbc2, 75 },
    { 0x88febbc3, 52 },
    { 0x6a2ebbc4, 76 },
    { 0x2d9ebbc4, 95 },
    { 0xb59ebbc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aebbcc, 23 },
    { 0, 0 },
    { 0x3f2ebbce, 33 },
    { 0xf19ebbce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x177ebbd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17ebbd5, 52 },
    { 0xc7ebbd5, 34 },
    { 0xc47ebbd6, 84 },
    { 0xdd6ebbd6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ebbdc, 99 },
    { 0, 0 },
    { 0x74eebbde, 93 },
    { 0x6ebebbdf, 33 },
    { 0x408ebbe0, 77 },
    { 0xc9eebbe1, 95 },
    { 0x646ebbe2, 23 },
    { 0x164ebbe3, 85 },
    { 0, 0 },
    { 0x3d4ebbe5, 87 },
    { 0x90aebbe5, 61 },
    { 0, 0 },
    { 0xc65ebbe8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16febbed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0ebbf1, 90 },
    { 0, 0 },
    { 0x778ebbf3, 93 },
    { 0xaa2ebbf4, 90 },
    { 0x106ebbf5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64febbfa, 81 },
    { 0, 0 },
    { 0x7b6ebbfc, 90 },
    { 0, 0 },
    { 0xec6ebbfe, 76 },
    { 0, 0 },
    { 0x6bbebc00, 33 },
    { 0, 0 },
    { 0xff2ebc02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccebc07, 55 },
    { 0x385ebc08, 90 },
    { 0x5bebc09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ebc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ebc11, 98 },
    { 0xb7ebc11, 98 },
    { 0x2b4ebc12, 66 },
    { 0xae4ebc14, 55 },
    { 0, 0 },
    { 0xb7eebc16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1ebc1f, 33 },
    { 0xca7ebc20, 90 },
    { 0xe44ebc21, 98 },
    { 0, 0 },
    { 0x65aebc23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eebc29, 99 },
    { 0x5c3ebc2a, 23 },
    { 0xbaebc2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bebc2e, 98 },
    { 0xde6ebc2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ebc3b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5ebc3e, 33 },
    { 0xbdeebc3f, 33 },
    { 0, 0 },
    { 0x158ebc41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ebc47, 23 },
    { 0xf1bebc47, 23 },
    { 0x6d4ebc49, 90 },
    { 0x6c8ebc49, 61 },
    { 0xcbfebc4b, 66 },
    { 0x61cebc4b, 17 },
    { 0xf63ebc4d, 55 },
    { 0x3f5ebc4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbebc56, 90 },
    { 0xeb3ebc57, 66 },
    { 0, 0 },
    { 0x856ebc59, 33 },
    { 0x44eebc59, 33 },
    { 0, 0 },
    { 0xfa8ebc5c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42ebc64, 23 },
    { 0, 0 },
    { 0x4f6ebc66, 55 },
    { 0xda4ebc66, 61 },
    { 0xfaebc66, 95 },
    { 0x93aebc69, 46 },
    { 0xb3eebc6a, 95 },
    { 0x6eeebc6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eebc6f, 101 },
    { 0xc7debc70, 91 },
    { 0, 0 },
    { 0x43debc72, 90 },
    { 0x30debc72, 90 },
    { 0x91debc74, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ebc7a, 76 },
    { 0xa19ebc7b, 95 },
    { 0x10debc7b, 95 },
    { 0x9f2ebc7d, 23 },
    { 0, 0 },
    { 0x849ebc7f, 61 },
    { 0, 0 },
    { 0x7f1ebc81, 43 },
    { 0x6c4ebc81, 46 },
    { 0xbaebc81, 61 },
    { 0xff9ebc84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2ebc88, 90 },
    { 0xd32ebc89, 75 },
    { 0, 0 },
    { 0x77cebc8b, 90 },
    { 0, 0 },
    { 0xd85ebc8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aebc91, 61 },
    { 0, 0 },
    { 0xb5febc93, 61 },
    { 0xfc5ebc94, 95 },
    { 0x26bebc95, 61 },
    { 0x267ebc95, 90 },
    { 0xb06ebc94, 33 },
    { 0x34ebc98, 33 },
    { 0x7d1ebc94, 65 },
    { 0x728ebc9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220ebc9e, 66 },
    { 0x902ebc9f, 95 },
    { 0x357ebca0, 90 },
    { 0x895ebca1, 61 },
    { 0x20febca1, 66 },
    { 0xb11ebca3, 99 },
    { 0x2a7ebca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcebca7, 100 },
    { 0x51ebca8, 33 },
    { 0, 0 },
    { 0x3c4ebcaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aebcaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eebcb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6ebcc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289ebcc5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3ebccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65debccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ebcd4, 61 },
    { 0, 0 },
    { 0x40eebcd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cebcd9, 76 },
    { 0xa0ebcd9, 98 },
    { 0, 0 },
    { 0x757ebcdc, 34 },
    { 0xeb0ebcdd, 53 },
    { 0, 0 },
    { 0xf8ebcdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acebce2, 90 },
    { 0x1b7ebce3, 61 },
    { 0, 0 },
    { 0x501ebce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94ebce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbebced, 43 },
    { 0x371ebcee, 33 },
    { 0x577ebcef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfebcf7, 33 },
    { 0, 0 },
    { 0xadbebcf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feebcfe, 46 },
    { 0x988ebcff, 90 },
    { 0, 0 },
    { 0x80debd01, 33 },
    { 0, 0 },
    { 0x3c6ebd03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48ebd07, 23 },
    { 0xa9bebd08, 33 },
    { 0x780ebd08, 18 },
    { 0x4baebd0a, 97 },
    { 0xc55ebd0b, 98 },
    { 0x625ebd09, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ebd17, 87 },
    { 0, 0 },
    { 0x683ebd19, 43 },
    { 0, 0 },
    { 0xc7bebd1b, 78 },
    { 0x8afebd1b, 61 },
    { 0x65febd1c, 90 },
    { 0, 0 },
    { 0x17debd1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369ebd23, 95 },
    { 0, 0 },
    { 0xb97ebd25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x997ebd28, 33 },
    { 0xcceebd29, 93 },
    { 0x781ebd29, 55 },
    { 0xdf8ebd2b, 90 },
    { 0x8b0ebd2c, 96 },
    { 0x5cebd2d, 33 },
    { 0xfa4ebd2e, 66 },
    { 0xb7febd2d, 23 },
    { 0xf84ebd2a, 95 },
    { 0xa24ebd31, 65 },
    { 0, 0 },
    { 0xe9ebd33, 90 },
    { 0x861ebd33, 61 },
    { 0, 0 },
    { 0xa5aebd36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9ebd39, 75 },
    { 0xd1febd3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceebd3e, 90 },
    { 0x852ebd3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ebd4b, 95 },
    { 0, 0 },
    { 0xaa0ebd4d, 66 },
    { 0xd8febd4e, 90 },
    { 0x67debd4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ebd54, 34 },
    { 0xa12ebd55, 98 },
    { 0x8cbebd55, 91 },
    { 0x813ebd57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21debd5b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55eebd61, 43 },
    { 0x640ebd61, 61 },
    { 0, 0 },
    { 0x45eebd64, 90 },
    { 0x143ebd65, 33 },
    { 0, 0 },
    { 0xe63ebd67, 46 },
    { 0x47ebd68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3ebd6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ebd70, 55 },
    { 0, 0 },
    { 0x1bdebd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8ebd76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0febd79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0ebd7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eebd7f, 61 },
    { 0, 0 },
    { 0xd46ebd81, 98 },
    { 0xccbebd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ebd8c, 65 },
    { 0x19debd8c, 87 },
    { 0xb70ebd8c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0ebd94, 67 },
    { 0xfd5ebd95, 90 },
    { 0xcabebd96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebd9b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ebd9e, 90 },
    { 0, 0 },
    { 0xb5eebda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9ebda5, 90 },
    { 0x90cebda6, 98 },
    { 0x782ebda6, 33 },
    { 0, 0 },
    { 0x3e0ebda9, 33 },
    { 0x712ebdaa, 46 },
    { 0x610ebdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ebdaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ebdb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14debdb9, 76 },
    { 0, 0 },
    { 0x2e3ebdbb, 23 },
    { 0, 0 },
    { 0xe23ebdbd, 25 },
    { 0xf35ebdbe, 99 },
    { 0xceebdbf, 76 },
    { 0xcd5ebdc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ebdc4, 61 },
    { 0x2b6ebdc5, 75 },
    { 0, 0 },
    { 0x734ebdc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ebdd3, 84 },
    { 0x2c7ebdd4, 77 },
    { 0x740ebdd5, 33 },
    { 0xca1ebdd4, 90 },
    { 0xebebdd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1ebdde, 92 },
    { 0xf65ebddf, 90 },
    { 0x3ebebde0, 76 },
    { 0, 0 },
    { 0x1c9ebde2, 93 },
    { 0xf76ebde3, 52 },
    { 0, 0 },
    { 0xd8debde5, 55 },
    { 0x3dbebde6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x247ebde9, 90 },
    { 0x439ebdea, 98 },
    { 0xe84ebdea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3debdf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0debdf3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31ebdfb, 23 },
    { 0x6aebdfc, 46 },
    { 0x8e9ebdfd, 90 },
    { 0, 0 },
    { 0x2b5ebdff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9ebe02, 90 },
    { 0, 0 },
    { 0xd03ebe04, 98 },
    { 0x950ebe05, 85 },
    { 0xb3eebe06, 76 },
    { 0, 0 },
    { 0x6cebe08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ebe0c, 98 },
    { 0, 0 },
    { 0x3eebe0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aebe12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aebe18, 90 },
    { 0, 0 },
    { 0xc20ebe1a, 33 },
    { 0x67aebe1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937ebe24, 61 },
    { 0xb3ebe24, 61 },
    { 0x5ebe24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71febe2c, 93 },
    { 0x5f7ebe2d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9ebe36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bebe40, 23 },
    { 0xe51ebe41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4ebe46, 23 },
    { 0, 0 },
    { 0x5b5ebe48, 34 },
    { 0x583ebe49, 93 },
    { 0xcaebe4a, 90 },
    { 0, 0 },
    { 0xd46ebe4c, 95 },
    { 0x4e0ebe4d, 95 },
    { 0, 0 },
    { 0x14ebe4f, 61 },
    { 0x300ebe50, 81 },
    { 0xac8ebe4f, 67 },
    { 0, 0 },
    { 0x8a3ebe53, 88 },
    { 0, 0 },
    { 0xca8ebe55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ebe60, 96 },
    { 0, 0 },
    { 0x2febe62, 61 },
    { 0, 0 },
    { 0x60eebe64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ebe6b, 90 },
    { 0, 0 },
    { 0x8fbebe6d, 61 },
    { 0xc13ebe6e, 52 },
    { 0xb72ebe6f, 55 },
    { 0xd09ebe70, 84 },
    { 0x293ebe6f, 95 },
    { 0x4b2ebe71, 99 },
    { 0x637ebe6d, 95 },
    { 0xf53ebe74, 87 },
    { 0x2f7ebe74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6ebe79, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cebe7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ebe80, 76 },
    { 0x758ebe81, 90 },
    { 0xf22ebe81, 61 },
    { 0xdd8ebe82, 46 },
    { 0x7fbebe81, 90 },
    { 0, 0 },
    { 0xa58ebe86, 78 },
    { 0xddbebe87, 17 },
    { 0x1eaebe87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efebe8e, 33 },
    { 0, 0 },
    { 0xf3eebe90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acebe94, 17 },
    { 0x41bebe95, 85 },
    { 0, 0 },
    { 0x74eebe97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeebe9a, 95 },
    { 0x149ebe9b, 99 },
    { 0x428ebe9b, 43 },
    { 0x66eebe9d, 61 },
    { 0x41debe9e, 78 },
    { 0x103ebe9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892ebea3, 33 },
    { 0, 0 },
    { 0xa9eebea5, 33 },
    { 0xbdcebea5, 90 },
    { 0x94cebea7, 90 },
    { 0x33febea8, 46 },
    { 0x606ebea9, 95 },
    { 0x2f1ebea9, 61 },
    { 0x414ebeaa, 95 },
    { 0xcaeebeac, 61 },
    { 0, 0 },
    { 0xb45ebeae, 23 },
    { 0, 0 },
    { 0xd4debeb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51febeb7, 98 },
    { 0xee7ebeb8, 98 },
    { 0xa9cebeb8, 33 },
    { 0x2bfebeb9, 81 },
    { 0x5bbebebb, 77 },
    { 0xb0cebebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9ebec0, 90 },
    { 0x12ebec1, 33 },
    { 0xa0ebec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ebeca, 87 },
    { 0, 0 },
    { 0x5a6ebecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6ebed0, 93 },
    { 0, 0 },
    { 0x621ebed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abebed9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ebedd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23debee3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ebeec, 23 },
    { 0x627ebeed, 33 },
    { 0xb2ebeec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bebef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565ebef7, 90 },
    { 0xf24ebef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cebf01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ebf05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ebf08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ebf0b, 23 },
    { 0x5e2ebf0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfebf11, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52ebf15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5ebf1b, 43 },
    { 0xbeebf1c, 90 },
    { 0xb2febf1d, 20 },
    { 0xff9ebf1b, 23 },
    { 0x2eebf1b, 33 },
    { 0x5f5ebf20, 99 },
    { 0, 0 },
    { 0x9ecebf22, 98 },
    { 0x499ebf22, 34 },
    { 0xaedebf22, 98 },
    { 0x132ebf25, 76 },
    { 0, 0 },
    { 0x219ebf27, 23 },
    { 0x45cebf28, 81 },
    { 0x8e5ebf27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730ebf2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251ebf32, 95 },
    { 0x8debf33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ebf36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ebf39, 70 },
    { 0xc8cebf3a, 93 },
    { 0x3b0ebf3b, 33 },
    { 0, 0 },
    { 0xda1ebf3d, 76 },
    { 0, 0 },
    { 0x94cebf3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ebf42, 33 },
    { 0, 0 },
    { 0x39eebf44, 55 },
    { 0xad8ebf45, 33 },
    { 0x577ebf45, 90 },
    { 0xe7eebf47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebebf4a, 33 },
    { 0, 0 },
    { 0xd05ebf4c, 84 },
    { 0, 0 },
    { 0x313ebf4e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ebf51, 95 },
    { 0x7daebf51, 33 },
    { 0xfbebf53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ebf56, 46 },
    { 0, 0 },
    { 0x61cebf58, 87 },
    { 0x284ebf58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cebf5e, 64 },
    { 0x1fbebf5f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cebf62, 33 },
    { 0, 0 },
    { 0xfc9ebf64, 101 },
    { 0, 0 },
    { 0x78debf66, 85 },
    { 0xa9cebf66, 98 },
    { 0xcbeebf68, 92 },
    { 0xccbebf69, 33 },
    { 0x77febf69, 90 },
    { 0xabdebf69, 67 },
    { 0xaf6ebf6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ebf73, 92 },
    { 0x802ebf74, 61 },
    { 0, 0 },
    { 0xe84ebf76, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8ebf81, 33 },
    { 0x6c2ebf82, 52 },
    { 0xfe8ebf83, 78 },
    { 0xdeebf84, 98 },
    { 0x7e3ebf82, 99 },
    { 0xe8febf86, 52 },
    { 0x10eebf87, 55 },
    { 0x32debf88, 93 },
    { 0, 0 },
    { 0xcd3ebf8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acebf8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedebf92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950ebf96, 43 },
    { 0x769ebf97, 23 },
    { 0x77febf98, 23 },
    { 0, 0 },
    { 0xd0cebf9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bebf9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10debfa4, 33 },
    { 0x67cebfa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26febfa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ebfb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2ebfbf, 33 },
    { 0xa7bebfbf, 52 },
    { 0xca9ebfc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffebfc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cebfc7, 61 },
    { 0, 0 },
    { 0xa8febfc9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdebfcc, 93 },
    { 0x31eebfcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55aebfd5, 90 },
    { 0xeb7ebfd6, 55 },
    { 0x334ebfd7, 98 },
    { 0xe51ebfd8, 98 },
    { 0xe0bebfd8, 8 },
    { 0xd61ebfda, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cebfe1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0ebfe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ebfea, 61 },
    { 0x9e3ebfeb, 90 },
    { 0x3feebfec, 33 },
    { 0xc25ebfed, 46 },
    { 0x82febfee, 66 },
    { 0xf68ebfef, 33 },
    { 0, 0 },
    { 0x6b5ebff1, 67 },
    { 0, 0 },
    { 0x8caebff3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21ebff6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ebffb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bec000, 34 },
    { 0x500ec001, 33 },
    { 0x74aec002, 88 },
    { 0, 0 },
    { 0x80cec004, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ec00a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cec00e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5ec013, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fec01d, 95 },
    { 0x99cec01e, 33 },
    { 0, 0 },
    { 0x6cbec020, 90 },
    { 0, 0 },
    { 0x4eaec022, 43 },
    { 0x7b4ec023, 99 },
    { 0x1e3ec024, 79 },
    { 0, 0 },
    { 0x5b3ec026, 33 },
    { 0, 0 },
    { 0x1e5ec028, 93 },
    { 0xc38ec029, 61 },
    { 0x765ec02a, 95 },
    { 0, 0 },
    { 0x54cec02c, 95 },
    { 0x35bec02c, 33 },
    { 0xdf3ec02c, 61 },
    { 0xc3fec02f, 90 },
    { 0, 0 },
    { 0xdf8ec031, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dec037, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ec03c, 99 },
    { 0x7dfec03d, 46 },
    { 0xbbeec03d, 90 },
    { 0x714ec03f, 93 },
    { 0x624ec040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefec043, 33 },
    { 0xb76ec043, 43 },
    { 0, 0 },
    { 0x792ec046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41ec04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cec04f, 24 },
    { 0, 0 },
    { 0xf9cec051, 90 },
    { 0x9e5ec052, 90 },
    { 0xe24ec052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fec05d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeec060, 33 },
    { 0, 0 },
    { 0xe18ec062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ec067, 61 },
    { 0xae5ec068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6ec06b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eec06e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ec071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18ec075, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5ec079, 90 },
    { 0, 0 },
    { 0xaf0ec07b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eec07f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fec082, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ec086, 95 },
    { 0x364ec087, 52 },
    { 0x5b1ec088, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbec08b, 76 },
    { 0, 0 },
    { 0x16aec08d, 43 },
    { 0x3bec08e, 55 },
    { 0xbcfec08f, 77 },
    { 0x3fdec08f, 91 },
    { 0x498ec090, 91 },
    { 0x3ffec092, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adec098, 77 },
    { 0x506ec099, 98 },
    { 0x45cec09a, 98 },
    { 0x7a0ec099, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ec0a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cec0a7, 95 },
    { 0x9b8ec0a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dec0ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08ec0af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eec0b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ec0be, 68 },
    { 0, 0 },
    { 0x5c6ec0c0, 23 },
    { 0x694ec0c1, 76 },
    { 0xa6aec0c1, 95 },
    { 0x159ec0c3, 34 },
    { 0x357ec0c3, 90 },
    { 0x52aec0c3, 33 },
    { 0, 0 },
    { 0x942ec0c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbec0cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ec0d3, 61 },
    { 0, 0 },
    { 0x1a7ec0d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feec0dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x177ec0df, 23 },
    { 0, 0 },
    { 0x5e4ec0e1, 61 },
    { 0, 0 },
    { 0xc96ec0e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cec0e6, 33 },
    { 0x4e9ec0e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daec0ec, 23 },
    { 0x4dfec0ed, 33 },
    { 0x854ec0ee, 98 },
    { 0xdedec0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ec0f6, 98 },
    { 0x878ec0f7, 76 },
    { 0, 0 },
    { 0x3beec0f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ec0fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feec102, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cec10a, 33 },
    { 0, 0 },
    { 0x815ec10c, 90 },
    { 0x626ec10c, 91 },
    { 0xe8ec10e, 87 },
    { 0x235ec10f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefec114, 34 },
    { 0, 0 },
    { 0x7f3ec116, 55 },
    { 0xedfec117, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ec11a, 98 },
    { 0xba0ec11a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ec11e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6ec124, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x386ec127, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fec136, 90 },
    { 0x8f1ec136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7ec13b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5ec13e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cec148, 60 },
    { 0x289ec149, 98 },
    { 0xb51ec14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4ec150, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fec153, 33 },
    { 0, 0 },
    { 0x278ec155, 90 },
    { 0xacec156, 90 },
    { 0x9c9ec156, 85 },
    { 0x762ec155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bec15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcec162, 33 },
    { 0xcd2ec163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ec167, 23 },
    { 0x67fec167, 95 },
    { 0x5f8ec169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x699ec16c, 76 },
    { 0, 0 },
    { 0xa40ec16e, 98 },
    { 0x860ec16f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ec176, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ec17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aec17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcec182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bec187, 91 },
    { 0xe63ec188, 61 },
    { 0x551ec189, 100 },
    { 0xc09ec18a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ec18d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ec193, 95 },
    { 0, 0 },
    { 0x38ec195, 43 },
    { 0xadfec195, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fec19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fec1a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ec1a5, 95 },
    { 0x489ec1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dec1a9, 52 },
    { 0xb12ec1aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeec1af, 87 },
    { 0xfceec1b0, 61 },
    { 0, 0 },
    { 0x279ec1b2, 23 },
    { 0x1c2ec1b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620ec1b8, 93 },
    { 0xe79ec1b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1eec1bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aec1c5, 89 },
    { 0, 0 },
    { 0x5c2ec1c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ec1d0, 61 },
    { 0x2faec1d1, 34 },
    { 0xcb2ec1d0, 98 },
    { 0, 0 },
    { 0x43bec1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eec1d8, 69 },
    { 0, 0 },
    { 0x589ec1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceec1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3ec1e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5ec1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deec1ea, 55 },
    { 0x7caec1eb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec1f5, 95 },
    { 0xcf0ec1f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ec1fa, 76 },
    { 0xfd7ec1fb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eec203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aec206, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ec209, 65 },
    { 0, 0 },
    { 0xce6ec20b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ec212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ec215, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fec219, 85 },
    { 0xbf1ec219, 33 },
    { 0x244ec219, 61 },
    { 0, 0 },
    { 0xc86ec21d, 61 },
    { 0x293ec21e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2ec221, 61 },
    { 0x181ec222, 90 },
    { 0, 0 },
    { 0xf7ec224, 61 },
    { 0xc59ec225, 87 },
    { 0, 0 },
    { 0x353ec227, 17 },
    { 0xa14ec228, 98 },
    { 0x598ec229, 95 },
    { 0, 0 },
    { 0xdf7ec22b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8ec231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eec236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dec23b, 46 },
    { 0xc0dec23b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eec23f, 40 },
    { 0, 0 },
    { 0xcdeec241, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1ec250, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218ec258, 90 },
    { 0, 0 },
    { 0x3b2ec25a, 76 },
    { 0x81eec25b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfec25f, 55 },
    { 0, 0 },
    { 0x19cec261, 23 },
    { 0xac8ec261, 77 },
    { 0x6a6ec263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ec26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbec26e, 34 },
    { 0, 0 },
    { 0x279ec270, 34 },
    { 0x589ec271, 62 },
    { 0xffdec272, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec27d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fec283, 95 },
    { 0x4ceec283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dec287, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bec28a, 90 },
    { 0x7f7ec28b, 90 },
    { 0x4c4ec28c, 78 },
    { 0x17aec28d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ec290, 76 },
    { 0xaeaec290, 55 },
    { 0x641ec290, 21 },
    { 0x850ec290, 33 },
    { 0x211ec294, 95 },
    { 0x812ec295, 95 },
    { 0xc31ec291, 33 },
    { 0, 0 },
    { 0x1bdec298, 99 },
    { 0, 0 },
    { 0x3bdec29a, 55 },
    { 0x96aec29b, 34 },
    { 0x5fec29c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317ec2a0, 77 },
    { 0, 0 },
    { 0xce3ec2a2, 78 },
    { 0x377ec2a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855ec2a7, 77 },
    { 0, 0 },
    { 0x532ec2a9, 67 },
    { 0x896ec2a9, 98 },
    { 0xe42ec2a9, 43 },
    { 0x8dec2ab, 100 },
    { 0xb52ec2a9, 93 },
    { 0xe7aec2ae, 55 },
    { 0xca3ec2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1ec2b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ec2ba, 33 },
    { 0x1e5ec2bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289ec2c3, 90 },
    { 0x152ec2c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cec2c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ec2cd, 69 },
    { 0, 0 },
    { 0xc60ec2cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8ec2d9, 76 },
    { 0xf67ec2da, 76 },
    { 0x6bbec2db, 88 },
    { 0xa81ec2dc, 93 },
    { 0xc32ec2d9, 98 },
    { 0xd28ec2dc, 78 },
    { 0, 0 },
    { 0x737ec2e0, 33 },
    { 0x6ecec2e0, 69 },
    { 0x7aec2e2, 93 },
    { 0x24bec2e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fec2e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daec2ea, 52 },
    { 0x51fec2ea, 87 },
    { 0, 0 },
    { 0x50fec2ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aec2f3, 95 },
    { 0x121ec2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafec2fb, 90 },
    { 0, 0 },
    { 0x567ec2fd, 99 },
    { 0xec4ec2fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ec308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6ec311, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afec314, 99 },
    { 0x53cec315, 55 },
    { 0xce9ec316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ec31b, 55 },
    { 0x7b4ec31c, 95 },
    { 0x897ec31c, 76 },
    { 0xda3ec31e, 97 },
    { 0x836ec31b, 23 },
    { 0xd0eec31f, 76 },
    { 0x523ec321, 33 },
    { 0x1dcec322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1ec325, 34 },
    { 0, 0 },
    { 0x4d6ec327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4eec32c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeec335, 33 },
    { 0xd06ec336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec33a, 81 },
    { 0, 0 },
    { 0xa45ec33c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fec343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ec34a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52ec34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aec351, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1ec35d, 90 },
    { 0x9ddec35d, 81 },
    { 0xbd6ec35d, 33 },
    { 0, 0 },
    { 0x25aec361, 90 },
    { 0x40ec361, 61 },
    { 0xcedec361, 98 },
    { 0x527ec364, 68 },
    { 0xfb0ec361, 61 },
    { 0, 0 },
    { 0xbdfec367, 90 },
    { 0xb3dec367, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeec36c, 23 },
    { 0, 0 },
    { 0x53aec36e, 52 },
    { 0xf8bec36f, 43 },
    { 0, 0 },
    { 0x9dcec371, 55 },
    { 0x7d4ec371, 76 },
    { 0xb42ec372, 100 },
    { 0, 0 },
    { 0x4e2ec375, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969ec379, 98 },
    { 0, 0 },
    { 0x8d0ec37b, 33 },
    { 0, 0 },
    { 0x64ec37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dec381, 95 },
    { 0xa5eec382, 23 },
    { 0x69bec383, 98 },
    { 0xb38ec384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ec388, 83 },
    { 0x418ec388, 90 },
    { 0, 0 },
    { 0x1ceec38b, 98 },
    { 0, 0 },
    { 0x45cec38d, 100 },
    { 0xe08ec38d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccec399, 61 },
    { 0, 0 },
    { 0xcf3ec39b, 90 },
    { 0, 0 },
    { 0xb70ec39d, 83 },
    { 0xcbdec39e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ec3a1, 61 },
    { 0, 0 },
    { 0xad2ec3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2ec3a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34ec3a9, 55 },
    { 0xffbec3aa, 76 },
    { 0, 0 },
    { 0x555ec3ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcec3b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2ec3b6, 84 },
    { 0x14fec3b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dec3c0, 23 },
    { 0, 0 },
    { 0xf45ec3c2, 61 },
    { 0, 0 },
    { 0xbd2ec3c4, 55 },
    { 0x2e2ec3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddec3ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ec3cd, 55 },
    { 0x80bec3ce, 43 },
    { 0, 0 },
    { 0xd5aec3d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77ec3d5, 93 },
    { 0xd1dec3d6, 61 },
    { 0x711ec3d7, 43 },
    { 0x8acec3d6, 55 },
    { 0xf32ec3d7, 92 },
    { 0xfd0ec3d6, 33 },
    { 0x737ec3d5, 43 },
    { 0xa34ec3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcec3e5, 23 },
    { 0x21aec3e6, 67 },
    { 0xa7fec3e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bec3ea, 61 },
    { 0, 0 },
    { 0xffec3ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ec3ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eec3f8, 23 },
    { 0x15bec3f8, 90 },
    { 0xa3fec3f8, 23 },
    { 0xb9cec3fb, 43 },
    { 0x874ec3f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ec403, 66 },
    { 0, 0 },
    { 0xe0bec405, 75 },
    { 0, 0 },
    { 0xdc9ec407, 23 },
    { 0x126ec408, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ec40b, 52 },
    { 0xa19ec40b, 90 },
    { 0x826ec40d, 91 },
    { 0x672ec40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ec413, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ec416, 78 },
    { 0xa2eec417, 98 },
    { 0x279ec416, 55 },
    { 0, 0 },
    { 0x441ec41a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3ec421, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ec424, 55 },
    { 0xbcaec425, 23 },
    { 0xe98ec426, 34 },
    { 0x6caec424, 76 },
    { 0x6bdec428, 23 },
    { 0xe97ec428, 23 },
    { 0x893ec425, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x219ec42d, 76 },
    { 0x2c6ec42e, 76 },
    { 0x161ec42f, 78 },
    { 0, 0 },
    { 0x8e3ec431, 78 },
    { 0, 0 },
    { 0x27eec433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dec438, 87 },
    { 0x645ec438, 33 },
    { 0, 0 },
    { 0xb9cec43b, 95 },
    { 0xb96ec43c, 97 },
    { 0x6bcec43d, 33 },
    { 0xb4aec43e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aec441, 55 },
    { 0, 0 },
    { 0xd85ec443, 61 },
    { 0xeacec443, 95 },
    { 0xb4fec443, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adec44c, 65 },
    { 0x934ec44c, 23 },
    { 0x8eaec44e, 61 },
    { 0xc11ec44d, 43 },
    { 0, 0 },
    { 0xa70ec451, 43 },
    { 0x344ec452, 23 },
    { 0, 0 },
    { 0xca1ec454, 40 },
    { 0x71bec455, 61 },
    { 0xfeaec456, 33 },
    { 0xb31ec457, 98 },
    { 0, 0 },
    { 0xf28ec459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ec45d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ec466, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07ec46b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3ec46e, 51 },
    { 0x389ec46f, 33 },
    { 0x451ec470, 48 },
    { 0, 0 },
    { 0xc44ec472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ec478, 23 },
    { 0x6c4ec478, 61 },
    { 0xb0dec47a, 75 },
    { 0x8cbec47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x721ec47e, 61 },
    { 0x187ec47e, 33 },
    { 0xa7ec480, 98 },
    { 0xf67ec480, 95 },
    { 0xee2ec47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x426ec485, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107ec490, 23 },
    { 0xbc3ec490, 90 },
    { 0x535ec490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751ec49b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cec4a4, 23 },
    { 0xd26ec4a4, 23 },
    { 0x3deec4a5, 33 },
    { 0, 0 },
    { 0xf91ec4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ec4ad, 90 },
    { 0xa42ec4ad, 61 },
    { 0, 0 },
    { 0x872ec4b0, 91 },
    { 0xdeaec4b1, 38 },
    { 0x301ec4b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ec4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ec4ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aec4bf, 95 },
    { 0x193ec4c0, 90 },
    { 0x274ec4bf, 90 },
    { 0x231ec4c2, 23 },
    { 0xc34ec4c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ec4ca, 76 },
    { 0, 0 },
    { 0x7aeec4cc, 95 },
    { 0x9d3ec4cd, 34 },
    { 0xdceec4ce, 61 },
    { 0xc4ec4cf, 90 },
    { 0x2feec4d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48ec4d5, 78 },
    { 0xa53ec4d6, 90 },
    { 0x208ec4d7, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bec4dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8ec4e1, 61 },
    { 0x674ec4e2, 78 },
    { 0xd05ec4e3, 33 },
    { 0xf09ec4e3, 98 },
    { 0x51ec4e2, 55 },
    { 0x80aec4e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aec4e9, 95 },
    { 0xa5fec4ea, 76 },
    { 0xae0ec4ea, 38 },
    { 0x86bec4e9, 33 },
    { 0xfa3ec4ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242ec4f2, 98 },
    { 0x60aec4f3, 90 },
    { 0x6c5ec4f3, 81 },
    { 0xfc1ec4f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddec4fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aec4fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffec502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ec505, 61 },
    { 0x352ec505, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x25eec509, 61 },
    { 0x4c0ec50a, 61 },
    { 0x6ec509, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cec513, 99 },
    { 0x132ec513, 34 },
    { 0x9d5ec515, 100 },
    { 0x2ecec516, 93 },
    { 0xc14ec516, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350ec522, 61 },
    { 0x4a5ec522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bec527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aec52a, 95 },
    { 0, 0 },
    { 0xa0dec52c, 96 },
    { 0x376ec52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cec530, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ec533, 61 },
    { 0xc0ec533, 43 },
    { 0x72fec535, 81 },
    { 0xa10ec536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2ec539, 75 },
    { 0xdc6ec53a, 19 },
    { 0xe93ec53b, 33 },
    { 0, 0 },
    { 0x93eec53d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ec541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faec544, 66 },
    { 0xc7dec544, 65 },
    { 0xb2cec545, 100 },
    { 0, 0 },
    { 0x57cec548, 33 },
    { 0x6a7ec548, 81 },
    { 0x984ec54a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fec552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ec555, 52 },
    { 0, 0 },
    { 0xbbcec557, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bec55b, 95 },
    { 0, 0 },
    { 0x6faec55d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ec563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ec566, 98 },
    { 0, 0 },
    { 0x72cec568, 91 },
    { 0x7fdec569, 33 },
    { 0, 0 },
    { 0x52bec56b, 87 },
    { 0x325ec56c, 87 },
    { 0x8d8ec56d, 100 },
    { 0x6f5ec56d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ec572, 90 },
    { 0, 0 },
    { 0xe7ec574, 52 },
    { 0x517ec575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ec57a, 23 },
    { 0x6d8ec57a, 76 },
    { 0x97ec57b, 33 },
    { 0x783ec57d, 100 },
    { 0xe0cec57d, 99 },
    { 0x831ec57b, 33 },
    { 0xbb4ec580, 46 },
    { 0xc3fec581, 99 },
    { 0, 0 },
    { 0xfd5ec583, 33 },
    { 0x680ec583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdec588, 87 },
    { 0xe5aec589, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41ec591, 90 },
    { 0xbbcec592, 95 },
    { 0xdd6ec593, 61 },
    { 0, 0 },
    { 0xa2fec595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0ec598, 55 },
    { 0, 0 },
    { 0xc6dec59a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afec59f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefec5a3, 95 },
    { 0x1a6ec5a4, 33 },
    { 0, 0 },
    { 0xfc1ec5a6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ec5a9, 61 },
    { 0xf70ec5aa, 85 },
    { 0xfc1ec5ab, 55 },
    { 0x793ec5ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9ec5b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ec5b6, 99 },
    { 0x6ceec5b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00ec5c0, 90 },
    { 0x533ec5c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2aec5c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dec5c8, 90 },
    { 0x754ec5c9, 61 },
    { 0, 0 },
    { 0x657ec5cb, 95 },
    { 0xfddec5cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cec5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ec5d2, 23 },
    { 0x12ec5d2, 90 },
    { 0x5fbec5d3, 34 },
    { 0, 0 },
    { 0xebdec5d6, 61 },
    { 0xc79ec5d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccec5da, 23 },
    { 0, 0 },
    { 0x94dec5dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fec5e0, 33 },
    { 0x40dec5e0, 23 },
    { 0xcf1ec5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230ec5ea, 90 },
    { 0xb2cec5ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbec5f1, 81 },
    { 0, 0 },
    { 0x2e8ec5f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656ec5fc, 93 },
    { 0, 0 },
    { 0xee5ec5fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ec603, 76 },
    { 0x51ec604, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832ec60e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4ec612, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fec619, 46 },
    { 0xa36ec61a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0ec61e, 67 },
    { 0x3faec61e, 98 },
    { 0, 0 },
    { 0x558ec621, 68 },
    { 0x1c4ec621, 55 },
    { 0xebbec623, 45 },
    { 0x273ec624, 52 },
    { 0x393ec625, 61 },
    { 0xbbec623, 95 },
    { 0x69dec627, 98 },
    { 0xadec622, 90 },
    { 0x52aec629, 66 },
    { 0x465ec62a, 90 },
    { 0xbdfec629, 23 },
    { 0x5b1ec626, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ec62f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40ec632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ec638, 36 },
    { 0x4d0ec639, 90 },
    { 0xa18ec638, 33 },
    { 0x1f7ec63a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2ec63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddec643, 90 },
    { 0xb47ec644, 84 },
    { 0xe52ec645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1ec64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ec64f, 23 },
    { 0xe0bec64f, 95 },
    { 0, 0 },
    { 0xb40ec652, 33 },
    { 0, 0 },
    { 0x451ec654, 93 },
    { 0, 0 },
    { 0xd98ec656, 95 },
    { 0xa99ec656, 66 },
    { 0xe0eec658, 95 },
    { 0x21ec657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdec661, 76 },
    { 0, 0 },
    { 0xa27ec663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71ec668, 91 },
    { 0x5f9ec668, 98 },
    { 0xb26ec668, 77 },
    { 0xf41ec66b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaec66f, 93 },
    { 0xb3dec66f, 55 },
    { 0xb7eec66f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199ec676, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bec679, 33 },
    { 0x2d0ec679, 99 },
    { 0x74cec67b, 61 },
    { 0x180ec67c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbec687, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6ec68c, 98 },
    { 0x1fdec68d, 90 },
    { 0x2b3ec68e, 33 },
    { 0x1daec68f, 43 },
    { 0xb27ec68c, 100 },
    { 0xe7cec68f, 98 },
    { 0xc79ec692, 56 },
    { 0, 0 },
    { 0xdafec694, 68 },
    { 0xd1aec695, 67 },
    { 0x24eec694, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60ec69e, 90 },
    { 0x122ec69e, 99 },
    { 0x74fec69e, 100 },
    { 0x9a6ec6a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfec6af, 33 },
    { 0x2eec6af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372ec6b4, 61 },
    { 0, 0 },
    { 0xfd8ec6b6, 95 },
    { 0xcf6ec6b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcec6bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfec6c9, 61 },
    { 0, 0 },
    { 0xb67ec6cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935ec6d5, 23 },
    { 0, 0 },
    { 0x59eec6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04ec6db, 87 },
    { 0x998ec6db, 91 },
    { 0xb26ec6dd, 98 },
    { 0, 0 },
    { 0xa15ec6df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fec6e2, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764ec6ea, 88 },
    { 0, 0 },
    { 0x94aec6ec, 78 },
    { 0xe73ec6ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5ec6f0, 95 },
    { 0xae6ec6f1, 23 },
    { 0x1dcec6f2, 33 },
    { 0, 0 },
    { 0x874ec6f4, 36 },
    { 0x2a8ec6f4, 61 },
    { 0x521ec6f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dec6f9, 91 },
    { 0x498ec6fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfec6fd, 95 },
    { 0xa33ec6fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263ec703, 23 },
    { 0xf67ec704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22ec70a, 98 },
    { 0, 0 },
    { 0x8f3ec70c, 34 },
    { 0xe9dec70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1ec711, 98 },
    { 0x625ec712, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55ec717, 90 },
    { 0, 0 },
    { 0x9e9ec719, 90 },
    { 0x74dec719, 76 },
    { 0xca1ec71b, 61 },
    { 0x50ec71c, 87 },
    { 0x483ec71d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7ec720, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ec723, 93 },
    { 0x376ec724, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eec72b, 90 },
    { 0x18cec72b, 93 },
    { 0xa19ec72d, 66 },
    { 0xb31ec72d, 69 },
    { 0, 0 },
    { 0x1fdec730, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473ec734, 90 },
    { 0x40fec735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beec73a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x176ec73d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eec747, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bec74d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bec755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9ec75a, 90 },
    { 0x6acec75b, 61 },
    { 0x32aec75b, 61 },
    { 0xa46ec75d, 23 },
    { 0x539ec75b, 95 },
    { 0xb1aec75f, 33 },
    { 0, 0 },
    { 0x532ec761, 66 },
    { 0xd57ec762, 90 },
    { 0, 0 },
    { 0xc90ec764, 33 },
    { 0xa49ec765, 61 },
    { 0x5dcec766, 23 },
    { 0, 0 },
    { 0x7a2ec768, 99 },
    { 0x4eaec769, 33 },
    { 0x331ec76a, 96 },
    { 0x717ec769, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7ec76e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676ec773, 43 },
    { 0x8a4ec774, 88 },
    { 0x86eec775, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdec779, 66 },
    { 0x510ec779, 34 },
    { 0xea8ec77b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaec781, 34 },
    { 0, 0 },
    { 0xca0ec783, 87 },
    { 0xcbcec784, 34 },
    { 0xc55ec785, 87 },
    { 0, 0 },
    { 0xa66ec787, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ec78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324ec791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230ec799, 33 },
    { 0, 0 },
    { 0x782ec79b, 68 },
    { 0xec8ec79c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117ec7a0, 84 },
    { 0, 0 },
    { 0x413ec7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ec7a9, 99 },
    { 0x294ec7aa, 91 },
    { 0x4bec7a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaec7ae, 90 },
    { 0, 0 },
    { 0xbeaec7b0, 90 },
    { 0, 0 },
    { 0xb86ec7b2, 61 },
    { 0x4daec7b2, 66 },
    { 0, 0 },
    { 0x358ec7b5, 90 },
    { 0xebaec7b6, 43 },
    { 0xef2ec7b7, 61 },
    { 0x36cec7b6, 60 },
    { 0x7a2ec7b9, 78 },
    { 0x326ec7ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3ec7bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ec7c1, 55 },
    { 0, 0 },
    { 0xde0ec7c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434ec7c7, 55 },
    { 0, 0 },
    { 0x71bec7c9, 90 },
    { 0x9cdec7ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6ec7d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceec7e0, 93 },
    { 0xcc8ec7e1, 55 },
    { 0x5a4ec7e2, 95 },
    { 0xcfaec7e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aec7e6, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955ec7ea, 43 },
    { 0x54dec7ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ec7ef, 55 },
    { 0x82bec7f0, 55 },
    { 0x860ec7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddec7f4, 61 },
    { 0x419ec7f5, 23 },
    { 0x640ec7f5, 98 },
    { 0, 0 },
    { 0x423ec7f8, 98 },
    { 0x692ec7f9, 91 },
    { 0x444ec7f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ec7ff, 95 },
    { 0x14ec800, 52 },
    { 0xe27ec801, 90 },
    { 0x28fec801, 33 },
    { 0, 0 },
    { 0x6f9ec804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997ec80a, 91 },
    { 0, 0 },
    { 0xdf6ec80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeeec810, 66 },
    { 0xd95ec810, 46 },
    { 0x1f2ec810, 98 },
    { 0xba5ec813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311ec817, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4ec81b, 52 },
    { 0x8c1ec81b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ec822, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5ec825, 93 },
    { 0, 0 },
    { 0x1f5ec827, 97 },
    { 0, 0 },
    { 0xf33ec829, 33 },
    { 0x97bec82a, 61 },
    { 0x16fec82b, 55 },
    { 0xd4fec82c, 61 },
    { 0xb97ec82b, 90 },
    { 0x9a9ec82e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cec834, 61 },
    { 0, 0 },
    { 0x737ec836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dec83b, 23 },
    { 0x496ec83c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bec841, 66 },
    { 0, 0 },
    { 0xf89ec843, 55 },
    { 0, 0 },
    { 0xcd1ec845, 90 },
    { 0x5bfec845, 98 },
    { 0, 0 },
    { 0xd39ec848, 90 },
    { 0, 0 },
    { 0x3f3ec84a, 33 },
    { 0, 0 },
    { 0x8e9ec84c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bec854, 55 },
    { 0, 0 },
    { 0xe6ec856, 52 },
    { 0x38cec857, 91 },
    { 0xd39ec858, 55 },
    { 0, 0 },
    { 0x79bec85a, 95 },
    { 0x1daec85a, 61 },
    { 0x190ec85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175ec861, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13ec864, 55 },
    { 0x438ec864, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33ec872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bec876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ec879, 67 },
    { 0xb06ec87a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cec886, 61 },
    { 0x4edec887, 90 },
    { 0x535ec888, 23 },
    { 0xa03ec889, 33 },
    { 0xb0bec88a, 90 },
    { 0, 0 },
    { 0x6c4ec88c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852ec890, 61 },
    { 0x9e8ec891, 95 },
    { 0, 0 },
    { 0xd9dec893, 23 },
    { 0x76dec893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163ec899, 61 },
    { 0x5d5ec899, 69 },
    { 0, 0 },
    { 0x346ec89c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dec8a1, 95 },
    { 0x7deec8a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ec8a7, 90 },
    { 0, 0 },
    { 0xbf4ec8a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40ec8b1, 34 },
    { 0, 0 },
    { 0x81cec8b3, 98 },
    { 0xe02ec8b4, 34 },
    { 0x29fec8b4, 76 },
    { 0, 0 },
    { 0xa35ec8b7, 34 },
    { 0, 0 },
    { 0xfd1ec8b9, 95 },
    { 0x327ec8ba, 33 },
    { 0x5dbec8b9, 66 },
    { 0x7aeec8bc, 34 },
    { 0xc07ec8ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaec8c3, 23 },
    { 0, 0 },
    { 0xd23ec8c5, 76 },
    { 0, 0 },
    { 0x244ec8c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54ec8cd, 95 },
    { 0xb97ec8cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbec8d1, 52 },
    { 0x3a4ec8d1, 84 },
    { 0, 0 },
    { 0xa80ec8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ec8d7, 98 },
    { 0x855ec8d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafec8e0, 95 },
    { 0xd28ec8e1, 43 },
    { 0, 0 },
    { 0xeaaec8e3, 33 },
    { 0xfb6ec8e3, 99 },
    { 0x347ec8e4, 95 },
    { 0, 0 },
    { 0xc3ec8e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144ec8f0, 23 },
    { 0x37ec8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36ec8f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942ec8fe, 65 },
    { 0x5edec8fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ec902, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ec90c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdec910, 34 },
    { 0xde3ec911, 52 },
    { 0x714ec910, 95 },
    { 0xbbfec913, 33 },
    { 0xc87ec910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9ec917, 61 },
    { 0x5e1ec918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ec91e, 55 },
    { 0x7f4ec91f, 91 },
    { 0x95bec920, 93 },
    { 0x425ec91f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ec924, 90 },
    { 0, 0 },
    { 0xb3cec926, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8ec92c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9ec936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22ec93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ec940, 98 },
    { 0xd6bec941, 66 },
    { 0, 0 },
    { 0x71ec943, 95 },
    { 0x706ec944, 67 },
    { 0x906ec945, 99 },
    { 0, 0 },
    { 0x1bec947, 95 },
    { 0x529ec948, 76 },
    { 0, 0 },
    { 0xbebec94a, 61 },
    { 0xa19ec94b, 76 },
    { 0x585ec94b, 100 },
    { 0xc80ec94b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceec952, 46 },
    { 0, 0 },
    { 0xe76ec954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cec958, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afec95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2ec95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec961, 90 },
    { 0, 0 },
    { 0xe79ec963, 84 },
    { 0x28ec964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x907ec967, 90 },
    { 0x368ec968, 61 },
    { 0xf38ec968, 33 },
    { 0x2a6ec969, 52 },
    { 0x40fec96b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7ec96f, 46 },
    { 0, 0 },
    { 0x302ec971, 90 },
    { 0x671ec971, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eec975, 98 },
    { 0, 0 },
    { 0x9fcec977, 33 },
    { 0xcc0ec978, 90 },
    { 0xcfcec977, 91 },
    { 0, 0 },
    { 0xc67ec97b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3ec97e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12ec986, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ec989, 84 },
    { 0xebcec98a, 95 },
    { 0x9d1ec98b, 95 },
    { 0, 0 },
    { 0x34fec98d, 90 },
    { 0, 0 },
    { 0x80fec98f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5ec993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ec99a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28ec99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec9a2, 69 },
    { 0x83bec9a3, 98 },
    { 0x566ec9a2, 90 },
    { 0xe32ec9a2, 33 },
    { 0x37ec9a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bec9a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235ec9ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38ec9b2, 93 },
    { 0xa5fec9b2, 100 },
    { 0, 0 },
    { 0x422ec9b5, 91 },
    { 0xdeeec9b6, 43 },
    { 0x20dec9b6, 34 },
    { 0x113ec9b8, 90 },
    { 0xc78ec9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983ec9bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eec9c2, 33 },
    { 0x7ffec9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bec9c7, 65 },
    { 0x59fec9c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792ec9cd, 23 },
    { 0, 0 },
    { 0x777ec9cf, 23 },
    { 0x6eeec9d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6ec9d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ec9db, 23 },
    { 0xd84ec9db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dec9e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401ec9f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eca01, 55 },
    { 0, 0 },
    { 0x10deca03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3eca07, 33 },
    { 0x989eca08, 52 },
    { 0xcadeca08, 90 },
    { 0, 0 },
    { 0xc5beca0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x410eca0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5beca11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eca16, 88 },
    { 0xf01eca17, 61 },
    { 0x8b1eca16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3eca1d, 98 },
    { 0, 0 },
    { 0xb99eca1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2eca26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ceca2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x758eca2e, 90 },
    { 0, 0 },
    { 0x131eca30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eeca36, 90 },
    { 0x4f3eca36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14deca40, 98 },
    { 0, 0 },
    { 0xb9aeca42, 77 },
    { 0x3baeca43, 90 },
    { 0, 0 },
    { 0x6c7eca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4beca48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2eca4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3eca52, 76 },
    { 0xe7feca52, 93 },
    { 0, 0 },
    { 0x286eca55, 46 },
    { 0x2fceca56, 46 },
    { 0x96aeca57, 55 },
    { 0x19beca58, 33 },
    { 0xc26eca58, 76 },
    { 0x1e5eca5a, 46 },
    { 0x89deca58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422eca5f, 90 },
    { 0x5bbeca60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679eca65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24eca6a, 85 },
    { 0xe9feca6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ceca6e, 95 },
    { 0x80eeca6f, 91 },
    { 0x322eca6e, 61 },
    { 0xcaeeca71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0eca7e, 23 },
    { 0x6f0eca7f, 95 },
    { 0xfb8eca80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3eca83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36eca86, 90 },
    { 0xdf5eca86, 98 },
    { 0x7f0eca88, 34 },
    { 0x4e5eca88, 61 },
    { 0, 0 },
    { 0x55aeca8b, 23 },
    { 0, 0 },
    { 0xb2deca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935eca95, 61 },
    { 0x72beca95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53eca99, 40 },
    { 0x9aaeca9a, 42 },
    { 0xfc2eca9b, 61 },
    { 0x9efeca9c, 90 },
    { 0x923eca9d, 95 },
    { 0x1f1eca9c, 46 },
    { 0, 0 },
    { 0xf55ecaa0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26ecaaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ecab5, 61 },
    { 0x818ecab6, 33 },
    { 0xc37ecab5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9decabc, 97 },
    { 0xbb0ecabc, 78 },
    { 0xf34ecabe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eecac1, 98 },
    { 0, 0 },
    { 0xa82ecac3, 93 },
    { 0x6b6ecac4, 43 },
    { 0xde2ecac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ecac8, 97 },
    { 0, 0 },
    { 0xa8eecaca, 67 },
    { 0xcc5ecacb, 95 },
    { 0, 0 },
    { 0xa1fecacd, 98 },
    { 0x8acecacd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13decad5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5ecae2, 75 },
    { 0, 0 },
    { 0xc65ecae4, 88 },
    { 0, 0 },
    { 0x4e4ecae6, 87 },
    { 0x4a4ecae6, 76 },
    { 0x993ecae8, 36 },
    { 0x78aecae9, 76 },
    { 0x2d9ecae6, 23 },
    { 0, 0 },
    { 0xf30ecaec, 61 },
    { 0, 0 },
    { 0x12decaee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ecaf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fecaf4, 34 },
    { 0x5dbecaf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ecaf8, 66 },
    { 0xbceecaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ecb00, 93 },
    { 0, 0 },
    { 0xd9fecb02, 33 },
    { 0x6feecb02, 76 },
    { 0xdd3ecb02, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ecb0c, 99 },
    { 0x2beecb0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95ecb16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196ecb1b, 75 },
    { 0x4efecb1c, 61 },
    { 0, 0 },
    { 0x3edecb1e, 90 },
    { 0xee7ecb1f, 100 },
    { 0, 0 },
    { 0xccaecb21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ecb24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ecb2a, 34 },
    { 0x5a3ecb2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cecb32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01ecb39, 91 },
    { 0x956ecb39, 85 },
    { 0, 0 },
    { 0xe93ecb3c, 43 },
    { 0x3fbecb3d, 86 },
    { 0xf15ecb3c, 34 },
    { 0, 0 },
    { 0x94fecb40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ecb45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0ecb49, 66 },
    { 0xeb2ecb49, 61 },
    { 0xe30ecb4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90aecb4f, 43 },
    { 0x306ecb50, 97 },
    { 0, 0 },
    { 0xabbecb52, 90 },
    { 0xd26ecb53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccecb59, 34 },
    { 0x312ecb5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbecb5e, 23 },
    { 0x6eaecb5f, 55 },
    { 0, 0 },
    { 0x782ecb61, 98 },
    { 0, 0 },
    { 0x23decb63, 67 },
    { 0, 0 },
    { 0x7aeecb65, 61 },
    { 0xc02ecb66, 55 },
    { 0x7fbecb66, 36 },
    { 0x93fecb66, 23 },
    { 0x230ecb67, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58ecb6c, 60 },
    { 0xd99ecb6d, 100 },
    { 0x930ecb6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03ecb76, 52 },
    { 0xe20ecb77, 90 },
    { 0x1dcecb78, 93 },
    { 0x763ecb76, 61 },
    { 0x6b3ecb76, 99 },
    { 0xf47ecb7b, 90 },
    { 0xec4ecb7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34decb7f, 76 },
    { 0, 0 },
    { 0x379ecb81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6ecb84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ecb89, 90 },
    { 0x77becb8a, 67 },
    { 0x83becb8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fecb8e, 90 },
    { 0xcb4ecb8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeecb94, 33 },
    { 0x4c2ecb95, 98 },
    { 0x970ecb94, 100 },
    { 0x9b1ecb97, 90 },
    { 0xea1ecb97, 61 },
    { 0, 0 },
    { 0x1faecb9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ecb9e, 52 },
    { 0x326ecb9e, 23 },
    { 0, 0 },
    { 0x62becba1, 95 },
    { 0, 0 },
    { 0x715ecba3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ecba8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ecbac, 33 },
    { 0x78cecbac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ecbb1, 69 },
    { 0xb43ecbb2, 76 },
    { 0x722ecbb3, 33 },
    { 0x1eecbb4, 90 },
    { 0x409ecbb5, 55 },
    { 0xdb6ecbb5, 87 },
    { 0x3a4ecbb5, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97ecbbd, 98 },
    { 0xb34ecbbe, 87 },
    { 0x14eecbbf, 95 },
    { 0x4d9ecbc0, 52 },
    { 0x646ecbbe, 43 },
    { 0, 0 },
    { 0x380ecbc3, 43 },
    { 0, 0 },
    { 0x705ecbc5, 33 },
    { 0xa34ecbc5, 61 },
    { 0x502ecbc7, 85 },
    { 0xcb3ecbc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86ecbcc, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9ecbdb, 46 },
    { 0x673ecbdc, 43 },
    { 0x8b6ecbdb, 90 },
    { 0x89decbdc, 95 },
    { 0, 0 },
    { 0xbbcecbe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9ecbe6, 87 },
    { 0x8baecbe6, 61 },
    { 0xad0ecbe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddecbec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19ecbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddecbf7, 98 },
    { 0x2afecbf8, 93 },
    { 0xdc8ecbf8, 43 },
    { 0x841ecbfa, 90 },
    { 0x714ecbfb, 46 },
    { 0x57eecbf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ecbff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ecc04, 33 },
    { 0, 0 },
    { 0x9aeecc06, 53 },
    { 0x1dfecc07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41ecc0c, 34 },
    { 0x29fecc0d, 99 },
    { 0xce8ecc0e, 23 },
    { 0x6daecc0f, 67 },
    { 0x3c6ecc0f, 61 },
    { 0xe25ecc11, 46 },
    { 0x3e6ecc12, 17 },
    { 0x251ecc0c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ecc18, 55 },
    { 0, 0 },
    { 0x6ebecc1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2ecc1f, 76 },
    { 0x139ecc20, 98 },
    { 0xe75ecc21, 95 },
    { 0xed0ecc21, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ecc27, 52 },
    { 0, 0 },
    { 0x500ecc29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacecc2d, 65 },
    { 0x2d2ecc2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbecc35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33becc38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cecc3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76ecc43, 68 },
    { 0x9dfecc43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aecc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ecc4c, 93 },
    { 0, 0 },
    { 0xffecc4e, 76 },
    { 0x54cecc4e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbececc56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95ecc5b, 88 },
    { 0x726ecc5c, 33 },
    { 0x765ecc5b, 55 },
    { 0, 0 },
    { 0x3dfecc5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afecc62, 97 },
    { 0x1d2ecc63, 95 },
    { 0x48fecc64, 91 },
    { 0xd68ecc65, 75 },
    { 0xc53ecc66, 34 },
    { 0, 0 },
    { 0x6deecc68, 61 },
    { 0x6dcecc68, 33 },
    { 0xa81ecc69, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ecc71, 52 },
    { 0xebeecc71, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdecc75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2ecc79, 23 },
    { 0xf9decc7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4becc7e, 88 },
    { 0x30becc7f, 61 },
    { 0xfd0ecc80, 76 },
    { 0xe00ecc80, 87 },
    { 0, 0 },
    { 0x22becc83, 91 },
    { 0xd6decc84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fecc8c, 88 },
    { 0xc80ecc8d, 33 },
    { 0, 0 },
    { 0x6c6ecc8f, 61 },
    { 0x43fecc8f, 61 },
    { 0, 0 },
    { 0xb0ecc92, 61 },
    { 0xbc5ecc93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36becc9a, 95 },
    { 0, 0 },
    { 0xf1decc9c, 90 },
    { 0x767ecc9d, 100 },
    { 0, 0 },
    { 0xc1cecc9f, 61 },
    { 0x125ecca0, 90 },
    { 0x5ffecca1, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ecca8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfeccad, 43 },
    { 0x229eccae, 61 },
    { 0x59feccae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beccb2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94eccb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eccb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0beccbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42feccc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aceccc6, 76 },
    { 0, 0 },
    { 0xaaeccc8, 46 },
    { 0x21ceccc9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4aecccd, 90 },
    { 0x45deccce, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ceccd2, 88 },
    { 0xfbfeccd3, 43 },
    { 0x129eccd2, 90 },
    { 0x23eccd4, 34 },
    { 0xdcdeccd6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ceccd9, 91 },
    { 0x4f2eccda, 33 },
    { 0xd9beccda, 65 },
    { 0, 0 },
    { 0xa63eccdd, 61 },
    { 0, 0 },
    { 0x3edeccdf, 90 },
    { 0x4ddecce0, 61 },
    { 0x1acecce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0ecce7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1eccec, 68 },
    { 0xd42ecced, 76 },
    { 0x571eccee, 52 },
    { 0x426eccec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6eccf2, 23 },
    { 0x2cfeccf3, 90 },
    { 0xc95eccf4, 98 },
    { 0x78eccf4, 90 },
    { 0xb16eccf3, 61 },
    { 0, 0 },
    { 0xe6eccf8, 33 },
    { 0, 0 },
    { 0xb52eccfa, 33 },
    { 0xfe0eccfb, 76 },
    { 0x268eccfb, 95 },
    { 0x59beccfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20ecd03, 91 },
    { 0xe60ecd04, 33 },
    { 0, 0 },
    { 0xdc2ecd06, 46 },
    { 0x765ecd07, 23 },
    { 0x4b6ecd07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ecd0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ecd14, 33 },
    { 0x741ecd15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539ecd1d, 98 },
    { 0x7c3ecd1e, 88 },
    { 0xf66ecd1f, 17 },
    { 0xd99ecd1e, 55 },
    { 0x36decd21, 93 },
    { 0xe57ecd1d, 34 },
    { 0xbcbecd23, 33 },
    { 0xc96ecd21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2becd2a, 33 },
    { 0x321ecd2b, 61 },
    { 0x13cecd2b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9ecd2f, 90 },
    { 0x19eecd30, 95 },
    { 0x2efecd31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ecd36, 17 },
    { 0, 0 },
    { 0x8b5ecd38, 33 },
    { 0x766ecd39, 69 },
    { 0xed3ecd3a, 33 },
    { 0xca3ecd3a, 55 },
    { 0x229ecd39, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ecd41, 77 },
    { 0, 0 },
    { 0xb99ecd43, 95 },
    { 0x5c2ecd44, 99 },
    { 0, 0 },
    { 0x8f9ecd46, 52 },
    { 0, 0 },
    { 0x5dcecd48, 33 },
    { 0xea4ecd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eecd4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ecd55, 61 },
    { 0x91decd56, 33 },
    { 0xf5ecd57, 98 },
    { 0x8bdecd58, 76 },
    { 0, 0 },
    { 0xff3ecd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ecd63, 90 },
    { 0, 0 },
    { 0xfcecd65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ecd6a, 76 },
    { 0x847ecd6b, 46 },
    { 0x595ecd6c, 23 },
    { 0xaceecd6d, 98 },
    { 0x329ecd6e, 90 },
    { 0x405ecd6f, 99 },
    { 0, 0 },
    { 0xb7eecd71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854ecd78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120ecd7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eecd84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x140ecd87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195ecd8d, 55 },
    { 0x32ecd8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaecd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaecd9d, 78 },
    { 0xe3ecd9e, 95 },
    { 0, 0 },
    { 0x368ecda0, 33 },
    { 0xd26ecda1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4decda6, 61 },
    { 0, 0 },
    { 0x683ecda8, 78 },
    { 0x50ecda8, 43 },
    { 0xdacecdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaecdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ecdb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aecdb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13ecdb8, 61 },
    { 0xda3ecdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcecdbd, 55 },
    { 0xaebecdbd, 40 },
    { 0xccdecdbd, 90 },
    { 0xeaaecdc0, 90 },
    { 0x4bdecdc0, 76 },
    { 0xc1ecdc0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceecdc6, 87 },
    { 0xe69ecdc7, 61 },
    { 0xa30ecdc7, 84 },
    { 0xb6decdc9, 43 },
    { 0x2c0ecdca, 87 },
    { 0xe11ecdcb, 65 },
    { 0x82becdcb, 23 },
    { 0, 0 },
    { 0x441ecdce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fecdd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cecdde, 61 },
    { 0, 0 },
    { 0x646ecde0, 90 },
    { 0x9c7ecde1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ecde7, 93 },
    { 0xcf6ecde8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x870ecdeb, 90 },
    { 0, 0 },
    { 0x621ecded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ecdf2, 68 },
    { 0xd52ecdf3, 75 },
    { 0, 0 },
    { 0x715ecdf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ece02, 95 },
    { 0, 0 },
    { 0x1bdece04, 43 },
    { 0xa02ece05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaece08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565ece0e, 98 },
    { 0xb90ece0e, 23 },
    { 0x7a8ece0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eece13, 98 },
    { 0xc69ece14, 93 },
    { 0x6d5ece14, 98 },
    { 0xce2ece16, 99 },
    { 0x91fece17, 90 },
    { 0xef9ece17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7ece1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaece25, 61 },
    { 0xc5aece25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efece2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x794ece31, 84 },
    { 0x473ece32, 33 },
    { 0, 0 },
    { 0xe25ece34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaece37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cece3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475ece3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eece42, 33 },
    { 0, 0 },
    { 0x99ece44, 55 },
    { 0x8afece45, 33 },
    { 0xd1cece46, 17 },
    { 0xccfece47, 65 },
    { 0x341ece45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ece4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7ece52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eece5a, 90 },
    { 0xb49ece5b, 33 },
    { 0x847ece5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dece61, 46 },
    { 0x2f9ece61, 33 },
    { 0, 0 },
    { 0xce9ece64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cece68, 65 },
    { 0xd5ece69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dece6d, 90 },
    { 0, 0 },
    { 0x797ece6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aece72, 77 },
    { 0xb96ece72, 55 },
    { 0x991ece72, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ece79, 93 },
    { 0, 0 },
    { 0xd28ece7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfece82, 87 },
    { 0, 0 },
    { 0x8d7ece84, 95 },
    { 0x329ece84, 98 },
    { 0x67eece84, 3 },
    { 0x68aece87, 23 },
    { 0, 0 },
    { 0x855ece89, 90 },
    { 0x35bece89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ece90, 91 },
    { 0x3fbece91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41ece94, 66 },
    { 0x2dfece95, 84 },
    { 0, 0 },
    { 0xd72ece97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eeece9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ecea0, 34 },
    { 0xcefecea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3eceae, 17 },
    { 0, 0 },
    { 0x3b5eceb0, 93 },
    { 0x7b2eceb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbeceb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aeceb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ecebb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ecec4, 75 },
    { 0xa40ecec5, 55 },
    { 0x568ecec6, 33 },
    { 0x113ecec7, 23 },
    { 0x86ecec7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528eced2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738eced8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cecedc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23ecee4, 34 },
    { 0, 0 },
    { 0x52aecee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121ecee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517eceed, 84 },
    { 0xa49eceee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ecef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cecef9, 55 },
    { 0xc88ecefa, 93 },
    { 0, 0 },
    { 0x458ecefc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x483eceff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ecf08, 55 },
    { 0x623ecf08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ecf12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73eecf16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ecf19, 65 },
    { 0x2c9ecf1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16decf20, 78 },
    { 0x957ecf20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbecf24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ecf2a, 40 },
    { 0x196ecf2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42becf2f, 61 },
    { 0x16fecf2f, 95 },
    { 0xb7becf31, 40 },
    { 0xc31ecf32, 61 },
    { 0xf18ecf31, 95 },
    { 0xf25ecf34, 93 },
    { 0x3d1ecf34, 40 },
    { 0, 0 },
    { 0xe7ecf37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ecf3c, 69 },
    { 0xfa9ecf3c, 98 },
    { 0, 0 },
    { 0x3c3ecf3f, 85 },
    { 0xea6ecf40, 98 },
    { 0xcbbecf41, 95 },
    { 0xfe2ecf42, 23 },
    { 0x140ecf43, 33 },
    { 0xe70ecf44, 52 },
    { 0, 0 },
    { 0x80ecf46, 34 },
    { 0xbd0ecf47, 46 },
    { 0x528ecf47, 76 },
    { 0, 0 },
    { 0x7eeecf4a, 55 },
    { 0, 0 },
    { 0xdacecf4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ecf4f, 23 },
    { 0xdc2ecf50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65ecf54, 52 },
    { 0, 0 },
    { 0xc1aecf56, 98 },
    { 0x9ccecf57, 33 },
    { 0x1c3ecf56, 98 },
    { 0x582ecf59, 46 },
    { 0, 0 },
    { 0xd93ecf5b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ecf5f, 45 },
    { 0xbb4ecf60, 91 },
    { 0x5ececf61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ecf66, 98 },
    { 0x110ecf67, 66 },
    { 0x10decf68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7ecf6b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5ecf6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93ecf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaecf74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3ecf77, 61 },
    { 0x87becf77, 95 },
    { 0x5e7ecf79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27ecf85, 96 },
    { 0, 0 },
    { 0x802ecf87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ecf8a, 23 },
    { 0x2c5ecf8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03ecf8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800ecf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6ecfa0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928ecfa4, 99 },
    { 0x2f0ecfa5, 93 },
    { 0, 0 },
    { 0x984ecfa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edecfad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0ecfb3, 55 },
    { 0xdbbecfb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fecfb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ecfbb, 95 },
    { 0, 0 },
    { 0xb8aecfbd, 95 },
    { 0xbd6ecfbd, 61 },
    { 0, 0 },
    { 0x8d2ecfc0, 84 },
    { 0, 0 },
    { 0xc5becfc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ecfc5, 43 },
    { 0, 0 },
    { 0x213ecfc7, 93 },
    { 0, 0 },
    { 0x114ecfc9, 33 },
    { 0, 0 },
    { 0x18aecfcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989ecfd4, 98 },
    { 0xcbaecfd4, 95 },
    { 0x230ecfd6, 95 },
    { 0, 0 },
    { 0x851ecfd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53decfdd, 55 },
    { 0xb11ecfde, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fecfe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faecfe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63ecfe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635ecfef, 95 },
    { 0x75ecff0, 76 },
    { 0, 0 },
    { 0x30eecff2, 78 },
    { 0xfeaecff3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ecff6, 34 },
    { 0x7eeecff7, 98 },
    { 0xcefecff7, 76 },
    { 0x30becff9, 33 },
    { 0xf1cecff6, 66 },
    { 0, 0 },
    { 0x66eecffc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0ed002, 75 },
    { 0, 0 },
    { 0x95eed004, 90 },
    { 0x9e2ed005, 23 },
    { 0xf32ed006, 76 },
    { 0xba0ed007, 52 },
    { 0xdc0ed008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ed00b, 77 },
    { 0x9daed00c, 67 },
    { 0, 0 },
    { 0x45bed00e, 90 },
    { 0x8eed00f, 65 },
    { 0x713ed010, 90 },
    { 0x29ed011, 55 },
    { 0x452ed00f, 33 },
    { 0x367ed013, 91 },
    { 0x4aced011, 33 },
    { 0x32fed015, 55 },
    { 0x58bed016, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648ed01e, 43 },
    { 0, 0 },
    { 0xce1ed020, 61 },
    { 0x250ed021, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fed024, 83 },
    { 0x11eed025, 98 },
    { 0x3d9ed026, 100 },
    { 0x6c1ed027, 33 },
    { 0, 0 },
    { 0xceaed029, 90 },
    { 0, 0 },
    { 0x4cced02b, 91 },
    { 0xce8ed02b, 99 },
    { 0xdced02d, 95 },
    { 0xf5ced02e, 93 },
    { 0x824ed02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bed032, 33 },
    { 0x898ed033, 46 },
    { 0x776ed033, 95 },
    { 0, 0 },
    { 0xcb3ed036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ded039, 98 },
    { 0x2e1ed03a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6ed03d, 61 },
    { 0x7bced03e, 87 },
    { 0, 0 },
    { 0xdd7ed040, 81 },
    { 0xfb7ed040, 65 },
    { 0xf67ed042, 23 },
    { 0xd6ded043, 17 },
    { 0x921ed044, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ed04a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fed051, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x239ed054, 85 },
    { 0x424ed055, 95 },
    { 0x335ed056, 52 },
    { 0, 0 },
    { 0x1d0ed058, 98 },
    { 0, 0 },
    { 0xcdced05a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fed05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeed062, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aed065, 95 },
    { 0xb4ced066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefed069, 90 },
    { 0xf1fed069, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00ed06d, 98 },
    { 0x7d3ed06e, 85 },
    { 0x4f6ed06f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ed075, 87 },
    { 0x1a1ed076, 23 },
    { 0xc3bed077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ed07f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ed084, 76 },
    { 0x846ed085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567ed089, 76 },
    { 0x319ed08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ed08d, 33 },
    { 0x63ded08e, 46 },
    { 0, 0 },
    { 0x4feed090, 96 },
    { 0, 0 },
    { 0x3fded092, 23 },
    { 0, 0 },
    { 0xed2ed094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12ed097, 33 },
    { 0x695ed098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29ed09c, 23 },
    { 0xf93ed09d, 75 },
    { 0x45ced09c, 33 },
    { 0x2f1ed09d, 43 },
    { 0, 0 },
    { 0xbeed0a1, 40 },
    { 0x494ed0a2, 90 },
    { 0x826ed0a3, 33 },
    { 0x54ced0a4, 52 },
    { 0xac5ed0a5, 72 },
    { 0x3d5ed0a4, 61 },
    { 0xc60ed0a7, 33 },
    { 0x449ed0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404ed0ab, 37 },
    { 0xd8ced0ac, 33 },
    { 0, 0 },
    { 0x608ed0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedced0b2, 76 },
    { 0x9bed0b3, 76 },
    { 0, 0 },
    { 0x1a1ed0b5, 52 },
    { 0x1baed0b5, 55 },
    { 0x7a7ed0b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aed0bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69ed0be, 90 },
    { 0xbbbed0bf, 98 },
    { 0x847ed0c0, 78 },
    { 0xe06ed0c1, 90 },
    { 0xf5ded0c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841ed0ca, 95 },
    { 0x87ded0cb, 87 },
    { 0x821ed0cc, 90 },
    { 0x74ced0cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ed0d1, 66 },
    { 0x54ced0d2, 98 },
    { 0xbe9ed0d2, 99 },
    { 0xa10ed0d3, 98 },
    { 0x3ded0d2, 23 },
    { 0x617ed0d5, 55 },
    { 0, 0 },
    { 0x74fed0d8, 61 },
    { 0x26bed0d9, 34 },
    { 0, 0 },
    { 0x2e2ed0db, 76 },
    { 0x3d3ed0dc, 76 },
    { 0x261ed0dd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ed0e4, 61 },
    { 0xe4ced0e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffced0ea, 76 },
    { 0, 0 },
    { 0x920ed0ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19aed0f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8ed0f4, 33 },
    { 0xeb4ed0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ed0f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eded0fd, 48 },
    { 0xb35ed0fe, 90 },
    { 0x540ed0fe, 91 },
    { 0x31ded100, 65 },
    { 0x799ed0ff, 36 },
    { 0xd9aed0fd, 90 },
    { 0xae5ed103, 96 },
    { 0x5e8ed0ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bed10c, 23 },
    { 0, 0 },
    { 0x6c6ed10e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ed111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ed117, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ed11b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bced11e, 98 },
    { 0x2e0ed11f, 87 },
    { 0xffed120, 23 },
    { 0x2b8ed11f, 33 },
    { 0xe95ed122, 52 },
    { 0, 0 },
    { 0xc6eed124, 61 },
    { 0, 0 },
    { 0x64bed126, 87 },
    { 0x20ced127, 87 },
    { 0, 0 },
    { 0x7aaed129, 90 },
    { 0, 0 },
    { 0xcbded12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eed12e, 90 },
    { 0x5b9ed12f, 90 },
    { 0xf14ed12e, 61 },
    { 0x437ed131, 95 },
    { 0xde4ed130, 85 },
    { 0x1d4ed133, 95 },
    { 0xae4ed133, 91 },
    { 0xeabed135, 52 },
    { 0x41fed135, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ed139, 95 },
    { 0x753ed139, 90 },
    { 0xdb7ed13b, 95 },
    { 0, 0 },
    { 0x6ed13d, 101 },
    { 0, 0 },
    { 0x7cfed13f, 95 },
    { 0xffced13f, 96 },
    { 0, 0 },
    { 0xc05ed142, 93 },
    { 0xa69ed143, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ed14c, 98 },
    { 0x6fbed14c, 90 },
    { 0, 0 },
    { 0xdebed14f, 37 },
    { 0, 0 },
    { 0xc74ed151, 95 },
    { 0, 0 },
    { 0x7fced153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed157, 34 },
    { 0x289ed157, 33 },
    { 0x963ed157, 90 },
    { 0, 0 },
    { 0x530ed15b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ed15f, 99 },
    { 0x214ed160, 33 },
    { 0, 0 },
    { 0x60ded162, 78 },
    { 0x109ed163, 17 },
    { 0x827ed164, 34 },
    { 0, 0 },
    { 0xb9fed166, 78 },
    { 0xcf9ed166, 43 },
    { 0, 0 },
    { 0x381ed169, 23 },
    { 0, 0 },
    { 0x3ffed16b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ded16e, 61 },
    { 0xb39ed16f, 76 },
    { 0x86ed170, 61 },
    { 0, 0 },
    { 0xf51ed172, 93 },
    { 0, 0 },
    { 0xc9aed174, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfded178, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x633ed17b, 81 },
    { 0x8ffed17b, 61 },
    { 0x75bed17b, 95 },
    { 0x3f0ed17c, 55 },
    { 0x54ded17f, 99 },
    { 0xf81ed180, 85 },
    { 0x91eed180, 33 },
    { 0x8f3ed17b, 95 },
    { 0xa9bed183, 95 },
    { 0, 0 },
    { 0x38aed185, 80 },
    { 0x34aed186, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fed192, 33 },
    { 0, 0 },
    { 0x63ded194, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eced198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eced19b, 78 },
    { 0xbbded19b, 90 },
    { 0x34bed19c, 33 },
    { 0xe2eed19d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0ed1a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ed1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ed1a9, 98 },
    { 0xb3eed1aa, 23 },
    { 0xa02ed1ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dded1b7, 88 },
    { 0xb98ed1b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ed1bd, 33 },
    { 0, 0 },
    { 0x114ed1bf, 33 },
    { 0x278ed1c0, 87 },
    { 0x5f8ed1c0, 61 },
    { 0x2eded1bf, 61 },
    { 0x257ed1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfed1c6, 90 },
    { 0xa88ed1c6, 52 },
    { 0x1fced1c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63ed1cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fed1cf, 95 },
    { 0, 0 },
    { 0x9e7ed1d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ed1d6, 78 },
    { 0xf7eed1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x425ed1da, 90 },
    { 0x742ed1db, 98 },
    { 0, 0 },
    { 0xc5fed1dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed1e0, 95 },
    { 0xeeaed1e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09ed1e4, 90 },
    { 0xc5bed1e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fed1ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ed1f5, 78 },
    { 0x497ed1f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bed1fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ed203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78ed213, 99 },
    { 0x7bced213, 76 },
    { 0x4c5ed215, 76 },
    { 0, 0 },
    { 0x15fed217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed21a, 46 },
    { 0x471ed21b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5ed21e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aed222, 78 },
    { 0xd1ed223, 95 },
    { 0, 0 },
    { 0x9e3ed225, 34 },
    { 0x7abed226, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeed22a, 46 },
    { 0, 0 },
    { 0xabded22c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9ed22f, 33 },
    { 0, 0 },
    { 0x4c7ed231, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed234, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afed239, 87 },
    { 0xf93ed239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed23d, 61 },
    { 0xf42ed23d, 95 },
    { 0, 0 },
    { 0x748ed240, 43 },
    { 0xe5fed240, 23 },
    { 0x845ed242, 91 },
    { 0x801ed243, 95 },
    { 0xee6ed244, 78 },
    { 0, 0 },
    { 0x916ed246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ed24b, 33 },
    { 0xc3aed24c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ced251, 61 },
    { 0x6a7ed252, 61 },
    { 0xee5ed253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0ed256, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ed25d, 33 },
    { 0, 0 },
    { 0xf85ed25f, 90 },
    { 0, 0 },
    { 0x3bed261, 33 },
    { 0, 0 },
    { 0x20bed263, 87 },
    { 0, 0 },
    { 0x431ed265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bed26b, 95 },
    { 0x9b6ed26c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7ed271, 61 },
    { 0x44ed271, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaed276, 23 },
    { 0, 0 },
    { 0xf92ed278, 95 },
    { 0, 0 },
    { 0xf1ed27a, 33 },
    { 0xf9aed27a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3ed27e, 90 },
    { 0xd4aed27f, 61 },
    { 0x820ed27f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed28b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0ed292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3ed295, 80 },
    { 0x3f0ed296, 98 },
    { 0, 0 },
    { 0xa33ed298, 61 },
    { 0, 0 },
    { 0xc9ced29a, 91 },
    { 0xc4aed29b, 55 },
    { 0x942ed29c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4ed2a4, 55 },
    { 0x4a2ed2a5, 93 },
    { 0, 0 },
    { 0xd4ed2a7, 90 },
    { 0xb25ed2a7, 84 },
    { 0x4eaed2a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5ed2b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aed2bb, 85 },
    { 0x99aed2bb, 78 },
    { 0xa31ed2bc, 61 },
    { 0xbf0ed2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ed2c1, 61 },
    { 0x372ed2c2, 43 },
    { 0xccced2c3, 33 },
    { 0x5d0ed2c3, 34 },
    { 0xbc7ed2c5, 23 },
    { 0x7a4ed2c6, 72 },
    { 0x7b4ed2c7, 76 },
    { 0xf4fed2c6, 95 },
    { 0xc9ded2c4, 43 },
    { 0, 0 },
    { 0x2c5ed2cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed2cf, 76 },
    { 0xb8eed2d0, 76 },
    { 0xb04ed2d0, 33 },
    { 0x18eed2cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bed2d5, 66 },
    { 0x992ed2d6, 98 },
    { 0xf23ed2d7, 23 },
    { 0, 0 },
    { 0xb6aed2d9, 66 },
    { 0x28bed2da, 90 },
    { 0xbabed2db, 33 },
    { 0x6f3ed2db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabced2df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ed2e2, 84 },
    { 0xb1eed2e2, 33 },
    { 0xee6ed2e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30ed2e7, 33 },
    { 0x303ed2e7, 33 },
    { 0x3c8ed2e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ed2ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabed2f2, 98 },
    { 0, 0 },
    { 0xffaed2f4, 33 },
    { 0x514ed2f4, 90 },
    { 0, 0 },
    { 0xbb8ed2f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469ed2fb, 90 },
    { 0x42aed2fc, 100 },
    { 0xfaeed2fd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eed303, 17 },
    { 0, 0 },
    { 0xd6aed305, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3ed309, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04ed30c, 90 },
    { 0, 0 },
    { 0x14bed30e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ded314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ed319, 33 },
    { 0x7c8ed31a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ced31f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfed322, 61 },
    { 0x76ced323, 98 },
    { 0x3b8ed324, 55 },
    { 0xf62ed323, 33 },
    { 0, 0 },
    { 0xefbed327, 55 },
    { 0x9a1ed328, 76 },
    { 0, 0 },
    { 0x6d6ed32a, 55 },
    { 0x5d4ed32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbed332, 100 },
    { 0x83fed332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85ed33a, 98 },
    { 0x22ced33a, 33 },
    { 0, 0 },
    { 0x59ed33d, 52 },
    { 0, 0 },
    { 0x594ed33f, 84 },
    { 0xac6ed340, 33 },
    { 0x4d6ed341, 76 },
    { 0x60ed342, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ded34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38ed34d, 33 },
    { 0xfceed34e, 23 },
    { 0xcbed34f, 23 },
    { 0xc3fed34f, 99 },
    { 0x790ed350, 23 },
    { 0x9ebed34d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ced357, 41 },
    { 0x7deed358, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ed35c, 40 },
    { 0, 0 },
    { 0xcc7ed35e, 61 },
    { 0x958ed35e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ed362, 43 },
    { 0, 0 },
    { 0x57ded364, 100 },
    { 0x97ced364, 98 },
    { 0, 0 },
    { 0x497ed367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eed36a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ed36f, 43 },
    { 0x90ced370, 33 },
    { 0xa79ed371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0ed374, 61 },
    { 0, 0 },
    { 0xb5aed376, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2ed379, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed37c, 98 },
    { 0x4a3ed37d, 67 },
    { 0, 0 },
    { 0xb1bed37f, 34 },
    { 0xa40ed37f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaded383, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ced386, 90 },
    { 0x8b2ed386, 61 },
    { 0x52bed388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ed38f, 33 },
    { 0, 0 },
    { 0xc25ed391, 61 },
    { 0, 0 },
    { 0xf68ed393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x181ed396, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6ed39e, 93 },
    { 0xb38ed39f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ed3a4, 93 },
    { 0x107ed3a4, 98 },
    { 0x7ed3a6, 23 },
    { 0xfb2ed3a7, 33 },
    { 0x8c5ed3a8, 76 },
    { 0x47aed3a9, 61 },
    { 0x44eed3a7, 80 },
    { 0xc31ed3a8, 91 },
    { 0x65ded3a9, 98 },
    { 0x4d3ed3a6, 93 },
    { 0x880ed3ad, 23 },
    { 0xdfaed3ac, 75 },
    { 0x35aed3a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaed3b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfed3be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfed3c1, 98 },
    { 0x28bed3c2, 52 },
    { 0x541ed3c3, 90 },
    { 0x850ed3c2, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaded3ca, 33 },
    { 0xfc0ed3ca, 90 },
    { 0xfe2ed3cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bed3cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ed3d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ed3d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fed3db, 61 },
    { 0x7f9ed3db, 90 },
    { 0xc51ed3dd, 100 },
    { 0, 0 },
    { 0x8f1ed3df, 99 },
    { 0, 0 },
    { 0x160ed3e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x197ed3e4, 23 },
    { 0, 0 },
    { 0xfd8ed3e6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbed3ed, 90 },
    { 0x403ed3ee, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbced3f4, 85 },
    { 0x205ed3f5, 98 },
    { 0x6e0ed3f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ded400, 81 },
    { 0xd2ded401, 98 },
    { 0xc66ed402, 90 },
    { 0, 0 },
    { 0xf5ed404, 55 },
    { 0x7b2ed405, 84 },
    { 0xc50ed405, 76 },
    { 0x184ed407, 43 },
    { 0x57ded406, 95 },
    { 0x9dbed409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ced40d, 76 },
    { 0, 0 },
    { 0xad7ed40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153ed413, 84 },
    { 0x23ed413, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fed419, 55 },
    { 0x3c9ed419, 90 },
    { 0, 0 },
    { 0xc59ed41c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ed422, 93 },
    { 0x97ced422, 98 },
    { 0, 0 },
    { 0xa0ded425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0ed429, 43 },
    { 0xc81ed429, 95 },
    { 0xfe8ed42b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507ed431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fed435, 46 },
    { 0x54aed436, 55 },
    { 0xdbed437, 95 },
    { 0x728ed438, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ded445, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9ed449, 46 },
    { 0xdc9ed44a, 98 },
    { 0xeeded449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ed454, 23 },
    { 0xefaed454, 90 },
    { 0x15ced454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ded45a, 61 },
    { 0, 0 },
    { 0xa1ed45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5ed465, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ed470, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1ed475, 82 },
    { 0x50ded476, 76 },
    { 0x2bbed477, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889ed47d, 52 },
    { 0xf62ed47d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01ed482, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aed485, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ed488, 95 },
    { 0x45fed488, 46 },
    { 0x859ed48a, 33 },
    { 0xe2ded48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabed48e, 98 },
    { 0x967ed48f, 90 },
    { 0xe9ced490, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ed494, 95 },
    { 0x138ed494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ed49a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ed4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ed4a8, 99 },
    { 0, 0 },
    { 0xf74ed4aa, 61 },
    { 0xd2eed4aa, 85 },
    { 0x133ed4ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aed4b0, 33 },
    { 0x5aeed4b1, 33 },
    { 0x654ed4b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ed4bb, 95 },
    { 0, 0 },
    { 0x94eed4bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ed4c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ed4c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494ed4cb, 98 },
    { 0x504ed4cb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eced4d0, 85 },
    { 0x76eed4d1, 61 },
    { 0xe7aed4d2, 95 },
    { 0x34bed4d2, 67 },
    { 0x2cbed4d2, 61 },
    { 0x90ded4d5, 90 },
    { 0xf41ed4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fed4dc, 32 },
    { 0x682ed4dd, 55 },
    { 0x630ed4de, 17 },
    { 0x730ed4df, 83 },
    { 0, 0 },
    { 0x1cfed4e1, 90 },
    { 0xdafed4e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x124ed4e5, 33 },
    { 0x60eed4e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ed4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55ed4ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ed4f1, 98 },
    { 0xd53ed4f2, 98 },
    { 0x754ed4f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ed4f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20ed4fd, 98 },
    { 0x328ed4fe, 90 },
    { 0x62ed4ff, 33 },
    { 0xf2bed4ff, 43 },
    { 0x5b2ed4ff, 95 },
    { 0x719ed4ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aed507, 90 },
    { 0, 0 },
    { 0x6c7ed509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ed50c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ed510, 33 },
    { 0x36ed511, 33 },
    { 0x8c6ed512, 46 },
    { 0x2cbed511, 95 },
    { 0, 0 },
    { 0xfdeed515, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x706ed518, 33 },
    { 0x689ed519, 95 },
    { 0xdf4ed51a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722ed51e, 77 },
    { 0x92fed51f, 61 },
    { 0, 0 },
    { 0x542ed521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9ed526, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dded531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833ed537, 23 },
    { 0, 0 },
    { 0x581ed539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aed53d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ed540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927ed548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eed54c, 78 },
    { 0xd1eed54d, 33 },
    { 0xb4ded54e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bed552, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eed55a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed55d, 33 },
    { 0x7e7ed55e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ced561, 98 },
    { 0xc1ced561, 33 },
    { 0x1abed562, 78 },
    { 0x9b5ed561, 55 },
    { 0x36ded563, 95 },
    { 0x1d5ed562, 76 },
    { 0x8baed567, 88 },
    { 0x1aeed563, 33 },
    { 0x54aed569, 95 },
    { 0x6fced56a, 90 },
    { 0x769ed561, 91 },
    { 0xde0ed56c, 23 },
    { 0x4dbed56c, 87 },
    { 0x55eed56c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6ed576, 34 },
    { 0, 0 },
    { 0x3c1ed578, 91 },
    { 0x562ed578, 99 },
    { 0xb29ed57a, 76 },
    { 0x352ed57a, 78 },
    { 0xbc5ed57c, 98 },
    { 0xf75ed57d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44ed581, 55 },
    { 0x239ed581, 90 },
    { 0x361ed583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x761ed586, 93 },
    { 0x107ed586, 90 },
    { 0x520ed588, 92 },
    { 0x926ed589, 66 },
    { 0xee4ed58a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ed58d, 95 },
    { 0x782ed58e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734ed593, 66 },
    { 0, 0 },
    { 0xd78ed595, 65 },
    { 0, 0 },
    { 0x312ed597, 52 },
    { 0, 0 },
    { 0x512ed599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4ed5a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ed5a6, 84 },
    { 0xcaaed5a6, 76 },
    { 0x6eed5a8, 98 },
    { 0x465ed5a8, 78 },
    { 0x141ed5a8, 33 },
    { 0x5d3ed5aa, 61 },
    { 0xe93ed5a9, 23 },
    { 0xfe0ed5ad, 33 },
    { 0xb86ed5ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314ed5b2, 84 },
    { 0x351ed5b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923ed5b8, 90 },
    { 0, 0 },
    { 0xeebed5ba, 78 },
    { 0x322ed5ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ded5be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67ed5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541ed5c6, 69 },
    { 0xdaed5c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ed5ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ced5ce, 95 },
    { 0xe9ced5cf, 46 },
    { 0x1f7ed5ce, 91 },
    { 0x3b2ed5d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x647ed5d4, 61 },
    { 0x72ced5d5, 90 },
    { 0, 0 },
    { 0x677ed5d7, 61 },
    { 0, 0 },
    { 0xa70ed5d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70ed5de, 61 },
    { 0xf5bed5df, 91 },
    { 0, 0 },
    { 0x619ed5e1, 66 },
    { 0xc25ed5e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391ed5ee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fed5f3, 34 },
    { 0, 0 },
    { 0x39ded5f5, 88 },
    { 0x7a6ed5f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52ed604, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eed607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ed60d, 34 },
    { 0xd64ed60d, 76 },
    { 0, 0 },
    { 0x160ed610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fed614, 33 },
    { 0x843ed615, 76 },
    { 0x223ed614, 90 },
    { 0x4aaed617, 96 },
    { 0x294ed618, 33 },
    { 0xb1eed616, 61 },
    { 0, 0 },
    { 0xf54ed61b, 91 },
    { 0x4a8ed61c, 61 },
    { 0x20ed61c, 34 },
    { 0xa27ed61b, 33 },
    { 0xbcded61f, 46 },
    { 0x9ded61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x764ed623, 55 },
    { 0, 0 },
    { 0x3c0ed625, 84 },
    { 0, 0 },
    { 0xb3bed627, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9ed62b, 23 },
    { 0x2e6ed62b, 33 },
    { 0xf59ed62c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceed636, 43 },
    { 0x965ed637, 46 },
    { 0, 0 },
    { 0xcfaed639, 43 },
    { 0, 0 },
    { 0x113ed63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13ed63e, 100 },
    { 0xe21ed63e, 76 },
    { 0xbc5ed63f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed643, 98 },
    { 0, 0 },
    { 0x60ded645, 76 },
    { 0xe59ed646, 61 },
    { 0xb0aed647, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580ed64f, 33 },
    { 0xfd5ed650, 88 },
    { 0, 0 },
    { 0x910ed652, 33 },
    { 0xb17ed652, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ed657, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x657ed65a, 61 },
    { 0, 0 },
    { 0xb39ed65c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2ed660, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ed665, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ed668, 67 },
    { 0, 0 },
    { 0x40eed66a, 33 },
    { 0, 0 },
    { 0xf21ed66c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bed674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ed677, 99 },
    { 0xd10ed678, 35 },
    { 0x773ed679, 99 },
    { 0x57ded679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aed67d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411ed686, 85 },
    { 0x47aed686, 90 },
    { 0xeabed687, 36 },
    { 0xfeeed687, 90 },
    { 0x6d8ed68a, 46 },
    { 0x3e3ed68b, 90 },
    { 0x1e2ed68c, 84 },
    { 0x3d0ed68c, 33 },
    { 0xadfed68e, 95 },
    { 0xd7fed68e, 46 },
    { 0x52fed690, 95 },
    { 0xaaaed691, 52 },
    { 0x5ffed68b, 87 },
    { 0xef9ed68e, 94 },
    { 0x1beed68d, 34 },
    { 0x602ed695, 76 },
    { 0x651ed696, 33 },
    { 0xd8fed695, 93 },
    { 0x569ed691, 90 },
    { 0xf90ed694, 90 },
    { 0xb01ed69a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ed69d, 46 },
    { 0x21aed69e, 55 },
    { 0xf48ed69d, 75 },
    { 0, 0 },
    { 0xf2fed6a1, 61 },
    { 0x288ed6a2, 33 },
    { 0xbaaed6a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffed6a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ed6ab, 33 },
    { 0, 0 },
    { 0xf23ed6ad, 92 },
    { 0xca3ed6ae, 76 },
    { 0x5f2ed6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21ed6b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3ed6b9, 33 },
    { 0x8bfed6b9, 34 },
    { 0, 0 },
    { 0x9c5ed6bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51ed6c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eeed6cb, 90 },
    { 0, 0 },
    { 0xfe3ed6cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22ed6d7, 95 },
    { 0, 0 },
    { 0x2eded6d9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ed6de, 95 },
    { 0x68ded6df, 90 },
    { 0x2eed6de, 61 },
    { 0x3b6ed6e1, 33 },
    { 0x71ed6e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ced6e8, 33 },
    { 0x7e4ed6e9, 52 },
    { 0, 0 },
    { 0xe7ded6eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ed6f0, 17 },
    { 0x5f8ed6f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbed6f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ed6fb, 33 },
    { 0, 0 },
    { 0x1bfed6fd, 34 },
    { 0xb14ed6fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bded701, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124ed706, 90 },
    { 0, 0 },
    { 0x769ed708, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ed70c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x770ed70f, 34 },
    { 0xd9aed70f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x767ed713, 76 },
    { 0x651ed714, 98 },
    { 0x97eed715, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6ed718, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ed71e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ed729, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fed72e, 33 },
    { 0x35ced72f, 88 },
    { 0, 0 },
    { 0x266ed731, 43 },
    { 0xd0ced732, 55 },
    { 0, 0 },
    { 0x489ed734, 100 },
    { 0xa76ed735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ded73a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ed740, 90 },
    { 0x772ed740, 67 },
    { 0x452ed742, 48 },
    { 0x252ed742, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ed749, 33 },
    { 0xdc7ed749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ed74e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eed753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ed759, 98 },
    { 0xcc3ed759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ed75d, 87 },
    { 0x18eed75e, 43 },
    { 0x96ced75e, 33 },
    { 0x9dded75f, 76 },
    { 0xc6bed761, 90 },
    { 0x3ed762, 90 },
    { 0xb3fed760, 23 },
    { 0x517ed764, 90 },
    { 0x435ed765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eed76e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ed772, 98 },
    { 0x48fed772, 40 },
    { 0x47eed774, 98 },
    { 0x8bed773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ed778, 33 },
    { 0, 0 },
    { 0x79fed77a, 33 },
    { 0, 0 },
    { 0x146ed77c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ed780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ed785, 33 },
    { 0x57ced786, 34 },
    { 0x1dfed787, 91 },
    { 0x29fed787, 76 },
    { 0, 0 },
    { 0x6dbed78a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ded790, 33 },
    { 0x590ed791, 61 },
    { 0xda0ed792, 94 },
    { 0x326ed790, 33 },
    { 0, 0 },
    { 0x4ceed795, 52 },
    { 0xb3ced796, 76 },
    { 0, 0 },
    { 0xe95ed798, 80 },
    { 0x1afed799, 23 },
    { 0xfbed79a, 95 },
    { 0xba9ed79b, 23 },
    { 0x881ed79c, 99 },
    { 0, 0 },
    { 0x4aed79e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0ed7a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbed7a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebed7af, 55 },
    { 0, 0 },
    { 0x189ed7b1, 43 },
    { 0xdb8ed7b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbed7b6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857ed7ba, 23 },
    { 0xb15ed7bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ed7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed7c5, 43 },
    { 0x1f4ed7c5, 98 },
    { 0x9efed7c7, 96 },
    { 0, 0 },
    { 0x7bced7c9, 33 },
    { 0xf1fed7ca, 23 },
    { 0xb73ed7cb, 90 },
    { 0x307ed7ca, 100 },
    { 0x656ed7cd, 90 },
    { 0xd47ed7ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ed7d4, 81 },
    { 0, 0 },
    { 0x62ded7d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8ed7e0, 33 },
    { 0xcdeed7e1, 23 },
    { 0x4beed7e2, 99 },
    { 0xa23ed7e3, 61 },
    { 0x23bed7e3, 87 },
    { 0x284ed7e3, 81 },
    { 0x6f8ed7e4, 61 },
    { 0xb53ed7e7, 95 },
    { 0xb7fed7e2, 95 },
    { 0xe72ed7e4, 91 },
    { 0x792ed7ea, 98 },
    { 0x103ed7ea, 43 },
    { 0x3bced7e6, 98 },
    { 0x24eed7e3, 36 },
    { 0x385ed7ee, 95 },
    { 0xf42ed7ef, 95 },
    { 0xbeced7f0, 90 },
    { 0x861ed7f1, 61 },
    { 0x2d6ed7f1, 61 },
    { 0x460ed7f3, 46 },
    { 0xe12ed7f4, 88 },
    { 0xa1aed7f3, 98 },
    { 0, 0 },
    { 0x667ed7f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aced800, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3ed80c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x236ed80f, 90 },
    { 0, 0 },
    { 0xc8aed811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52aed818, 95 },
    { 0xeed819, 61 },
    { 0x664ed81a, 23 },
    { 0xd1ced81a, 88 },
    { 0xc85ed818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ed81f, 90 },
    { 0x5abed820, 75 },
    { 0x1dfed821, 95 },
    { 0x317ed821, 95 },
    { 0xd43ed823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6ed828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ed82f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bed832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ed836, 90 },
    { 0xbf4ed836, 34 },
    { 0xe92ed837, 76 },
    { 0xdcaed838, 33 },
    { 0xd22ed83a, 55 },
    { 0x435ed837, 100 },
    { 0xf73ed83c, 43 },
    { 0xbded83c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced844, 55 },
    { 0xae9ed845, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ed84b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ed84e, 33 },
    { 0x17ced84e, 78 },
    { 0, 0 },
    { 0x9ffed851, 33 },
    { 0x549ed851, 23 },
    { 0xa21ed852, 55 },
    { 0, 0 },
    { 0xb15ed855, 61 },
    { 0x492ed856, 90 },
    { 0xb07ed857, 85 },
    { 0, 0 },
    { 0xddfed859, 76 },
    { 0, 0 },
    { 0x422ed85b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6ed869, 65 },
    { 0x5daed86a, 66 },
    { 0, 0 },
    { 0x389ed86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dced876, 95 },
    { 0xe89ed876, 61 },
    { 0, 0 },
    { 0xbf8ed879, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826ed880, 95 },
    { 0xacaed880, 33 },
    { 0xf8fed882, 53 },
    { 0x75ced883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x775ed886, 88 },
    { 0, 0 },
    { 0x83ded888, 98 },
    { 0xc93ed888, 95 },
    { 0x89ed88a, 95 },
    { 0x3ffed88a, 23 },
    { 0x91bed889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ed88f, 33 },
    { 0xf92ed890, 34 },
    { 0xac4ed891, 34 },
    { 0x400ed890, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bed895, 90 },
    { 0x9a9ed895, 100 },
    { 0x3eaed897, 69 },
    { 0x5dded896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13ed89f, 90 },
    { 0x1c6ed8a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fed8a9, 87 },
    { 0x50ded8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ed8ad, 95 },
    { 0x39fed8ae, 99 },
    { 0, 0 },
    { 0x492ed8b0, 23 },
    { 0x7ceed8b1, 34 },
    { 0xa8aed8b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401ed8b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6ed8c0, 90 },
    { 0xed9ed8c1, 61 },
    { 0, 0 },
    { 0xc39ed8c3, 85 },
    { 0, 0 },
    { 0xab3ed8c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ed8c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ed8cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ced8d3, 61 },
    { 0x6a7ed8d4, 58 },
    { 0x8e8ed8d4, 47 },
    { 0, 0 },
    { 0x954ed8d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9deed8da, 23 },
    { 0, 0 },
    { 0x1aded8dc, 91 },
    { 0xfeded8dd, 33 },
    { 0x600ed8dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeed8e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ed8ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ed8ef, 23 },
    { 0xed5ed8f0, 96 },
    { 0, 0 },
    { 0x4bbed8f2, 77 },
    { 0x4cfed8f3, 65 },
    { 0, 0 },
    { 0xa7fed8f5, 100 },
    { 0x326ed8f5, 34 },
    { 0xc83ed8f5, 95 },
    { 0, 0 },
    { 0xd8ed8f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbed8fd, 99 },
    { 0x6e0ed8fe, 90 },
    { 0x2b3ed8ff, 55 },
    { 0x4ded900, 96 },
    { 0xe90ed901, 98 },
    { 0, 0 },
    { 0xa2aed903, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbed908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ed90e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beed91a, 84 },
    { 0, 0 },
    { 0xfcbed91c, 34 },
    { 0x8c1ed91d, 90 },
    { 0xc8aed91c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bed923, 90 },
    { 0x2e8ed924, 98 },
    { 0x788ed925, 87 },
    { 0xf49ed923, 85 },
    { 0x6d6ed923, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ed92a, 66 },
    { 0x5b2ed92b, 61 },
    { 0, 0 },
    { 0xc42ed92d, 66 },
    { 0xc68ed92e, 98 },
    { 0x883ed92f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ed932, 90 },
    { 0xb49ed933, 66 },
    { 0, 0 },
    { 0x8cfed935, 98 },
    { 0, 0 },
    { 0xdfeed937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdded93c, 90 },
    { 0, 0 },
    { 0x8d7ed93e, 33 },
    { 0, 0 },
    { 0x97ded940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x528ed943, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bed948, 23 },
    { 0, 0 },
    { 0x20fed94a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9ed954, 95 },
    { 0xc3ced955, 55 },
    { 0x55ed956, 23 },
    { 0x343ed955, 23 },
    { 0, 0 },
    { 0x95ed959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ed95d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ed960, 90 },
    { 0x1ced960, 90 },
    { 0, 0 },
    { 0x23eed963, 90 },
    { 0, 0 },
    { 0x5b8ed965, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefed96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4ed975, 23 },
    { 0xcf4ed976, 78 },
    { 0x678ed977, 76 },
    { 0x6a5ed978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bed97b, 33 },
    { 0, 0 },
    { 0x172ed97d, 76 },
    { 0x763ed97e, 90 },
    { 0x1b7ed97f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60ed982, 90 },
    { 0xe64ed982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183ed98b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9ed98e, 91 },
    { 0x673ed98f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4ed994, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24ed998, 67 },
    { 0xeaced998, 23 },
    { 0x2caed999, 95 },
    { 0x3c8ed99b, 90 },
    { 0x189ed998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ed9a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7ed9ae, 85 },
    { 0x2b1ed9af, 90 },
    { 0xdfded9ae, 95 },
    { 0, 0 },
    { 0xbbed9b2, 100 },
    { 0, 0 },
    { 0x8afed9b4, 68 },
    { 0xce1ed9b5, 98 },
    { 0xaffed9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3ed9b9, 76 },
    { 0x939ed9ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ed9be, 34 },
    { 0x7beed9be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daed9c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76ed9c9, 77 },
    { 0x3aded9ca, 76 },
    { 0x659ed9cb, 95 },
    { 0xddded9cc, 67 },
    { 0, 0 },
    { 0xd4bed9ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x333ed9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05ed9e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebed9e5, 95 },
    { 0x74ed9e5, 33 },
    { 0xb71ed9e7, 12 },
    { 0x5c1ed9e8, 43 },
    { 0, 0 },
    { 0x304ed9ea, 66 },
    { 0, 0 },
    { 0x661ed9ec, 85 },
    { 0x718ed9ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aed9f0, 46 },
    { 0xafbed9f0, 33 },
    { 0x1f2ed9f0, 90 },
    { 0x702ed9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3ed9f6, 55 },
    { 0, 0 },
    { 0x37eed9f8, 43 },
    { 0, 0 },
    { 0x705ed9fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ed9fd, 95 },
    { 0x81fed9fe, 90 },
    { 0xa5fed9ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x241eda02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0eda05, 88 },
    { 0, 0 },
    { 0xc2eeda07, 61 },
    { 0, 0 },
    { 0x547eda09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96feda0d, 100 },
    { 0x526eda0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3feda12, 33 },
    { 0xa64eda13, 90 },
    { 0x62feda14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8feda17, 90 },
    { 0xd70eda17, 61 },
    { 0xea6eda18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42eda1c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929eda21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92eda24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2eda28, 90 },
    { 0, 0 },
    { 0xf23eda2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beeda2d, 34 },
    { 0, 0 },
    { 0x939eda2f, 23 },
    { 0, 0 },
    { 0x5b9eda31, 61 },
    { 0, 0 },
    { 0x795eda33, 46 },
    { 0x12aeda34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1eda3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306eda3f, 98 },
    { 0, 0 },
    { 0xf9eda41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ceda46, 90 },
    { 0x430eda47, 23 },
    { 0x760eda48, 78 },
    { 0x1e1eda49, 33 },
    { 0x268eda4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeda4e, 33 },
    { 0, 0 },
    { 0x2dceda50, 23 },
    { 0x241eda51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eeda5a, 90 },
    { 0x4e9eda5b, 95 },
    { 0x4daeda5a, 46 },
    { 0x5b0eda5d, 90 },
    { 0x57deda5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aeda61, 91 },
    { 0xad7eda62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfeda68, 90 },
    { 0x20aeda69, 91 },
    { 0x987eda6a, 95 },
    { 0xa0deda6b, 65 },
    { 0xc93eda6b, 46 },
    { 0xb3ceda6c, 23 },
    { 0x392eda6e, 90 },
    { 0x76beda6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574eda73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752eda79, 23 },
    { 0xdfeda7a, 23 },
    { 0xee0eda7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66beda7e, 43 },
    { 0, 0 },
    { 0x608eda80, 33 },
    { 0, 0 },
    { 0x657eda82, 72 },
    { 0xb08eda82, 87 },
    { 0x716eda84, 86 },
    { 0xb0deda85, 33 },
    { 0x20deda83, 95 },
    { 0x6d2eda84, 90 },
    { 0xd30eda88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9eda8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57deda91, 90 },
    { 0x664eda91, 33 },
    { 0x8eeda93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08eda96, 46 },
    { 0x7a9eda97, 61 },
    { 0, 0 },
    { 0x365eda99, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eda9e, 91 },
    { 0xeebeda9e, 33 },
    { 0xb18eda9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242edaa7, 84 },
    { 0, 0 },
    { 0xe7fedaa9, 33 },
    { 0x810edaaa, 98 },
    { 0, 0 },
    { 0x456edaac, 67 },
    { 0xa41edaad, 98 },
    { 0, 0 },
    { 0x1aaedaaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fedab2, 61 },
    { 0xcffedab3, 61 },
    { 0xd26edab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04edabf, 49 },
    { 0x8caedac0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcedac3, 23 },
    { 0x83bedac4, 33 },
    { 0x84aedac5, 81 },
    { 0, 0 },
    { 0x22cedac7, 100 },
    { 0x160edac8, 90 },
    { 0, 0 },
    { 0xdc2edaca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93edad1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6edad4, 76 },
    { 0, 0 },
    { 0xe64edad6, 33 },
    { 0x7abedad7, 90 },
    { 0, 0 },
    { 0x934edad9, 33 },
    { 0x45bedada, 76 },
    { 0x98eedadb, 43 },
    { 0x895edadc, 52 },
    { 0xde6edad9, 23 },
    { 0, 0 },
    { 0xdc4edadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7edae2, 53 },
    { 0x915edae2, 67 },
    { 0xaabedae4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cedae7, 61 },
    { 0xc0cedae7, 23 },
    { 0xc35edae9, 95 },
    { 0x27dedae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fedaed, 43 },
    { 0xd41edaee, 23 },
    { 0, 0 },
    { 0x83eedaf0, 84 },
    { 0xaafedaf0, 76 },
    { 0x95eedaf0, 61 },
    { 0, 0 },
    { 0x717edaf4, 33 },
    { 0, 0 },
    { 0x3e2edaf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eedafa, 95 },
    { 0x70bedafa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8edb00, 94 },
    { 0xaefedb01, 100 },
    { 0, 0 },
    { 0x8b6edb03, 58 },
    { 0x5e5edb04, 99 },
    { 0x5aaedb05, 2 },
    { 0x76bedb06, 90 },
    { 0x953edb04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178edb0b, 66 },
    { 0xc2dedb0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fedb12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccaedb1e, 76 },
    { 0x65eedb1f, 23 },
    { 0x3bedb1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6edb23, 61 },
    { 0xa7edb24, 85 },
    { 0x716edb25, 33 },
    { 0x137edb26, 95 },
    { 0, 0 },
    { 0xb5fedb28, 23 },
    { 0x53eedb29, 43 },
    { 0xa45edb2a, 88 },
    { 0x969edb2b, 66 },
    { 0x2a3edb2c, 23 },
    { 0xcacedb2d, 61 },
    { 0x335edb2e, 91 },
    { 0x401edb2f, 66 },
    { 0x238edb2d, 61 },
    { 0x765edb2a, 33 },
    { 0, 0 },
    { 0x66fedb33, 54 },
    { 0, 0 },
    { 0xf8eedb35, 90 },
    { 0, 0 },
    { 0xa91edb37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5edb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2edb3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7edb48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06edb4b, 84 },
    { 0xc9edb4b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48edb50, 23 },
    { 0x967edb51, 46 },
    { 0xb1edb52, 91 },
    { 0x227edb53, 95 },
    { 0xd4bedb54, 61 },
    { 0xeccedb52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98edb58, 33 },
    { 0xac8edb59, 65 },
    { 0xf3cedb59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aedb62, 52 },
    { 0x8daedb63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x399edb66, 100 },
    { 0xf9eedb67, 76 },
    { 0x81dedb67, 43 },
    { 0xbbfedb67, 90 },
    { 0x3c4edb6a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66edb6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637edb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1edb7d, 17 },
    { 0x7caedb7e, 23 },
    { 0xa41edb7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7edb82, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cedb86, 33 },
    { 0x480edb87, 93 },
    { 0, 0 },
    { 0x38bedb89, 85 },
    { 0x7c9edb8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36edb90, 95 },
    { 0x5adedb90, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bedb94, 98 },
    { 0x1f4edb94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb98, 61 },
    { 0xfbeedb98, 95 },
    { 0, 0 },
    { 0x896edb9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5edb9f, 33 },
    { 0xf68edba0, 90 },
    { 0, 0 },
    { 0x280edba2, 68 },
    { 0x515edba3, 43 },
    { 0x496edba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeedba7, 90 },
    { 0, 0 },
    { 0xdcfedba9, 46 },
    { 0, 0 },
    { 0x120edbab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eedbae, 76 },
    { 0x833edbaf, 90 },
    { 0x317edbb0, 23 },
    { 0xd6cedbaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aedbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9faedbb7, 76 },
    { 0, 0 },
    { 0xdf3edbb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddedbc0, 90 },
    { 0x9a7edbc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbedbc9, 95 },
    { 0xdc9edbca, 66 },
    { 0x44dedbcb, 94 },
    { 0, 0 },
    { 0x521edbcd, 52 },
    { 0, 0 },
    { 0x304edbcf, 98 },
    { 0, 0 },
    { 0x2b0edbd1, 34 },
    { 0x4dbedbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71edbd7, 93 },
    { 0x5efedbd8, 23 },
    { 0xe11edbd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611edbdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6edbe8, 95 },
    { 0xd11edbe8, 87 },
    { 0, 0 },
    { 0xc97edbeb, 84 },
    { 0, 0 },
    { 0xd49edbed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edbf2, 66 },
    { 0x824edbf3, 90 },
    { 0, 0 },
    { 0x10fedbf5, 23 },
    { 0x3d9edbf6, 23 },
    { 0x836edbf7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aedbfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06edc01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bedc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8edc0a, 55 },
    { 0, 0 },
    { 0x79bedc0c, 23 },
    { 0, 0 },
    { 0x809edc0e, 23 },
    { 0x624edc0f, 90 },
    { 0, 0 },
    { 0x783edc11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43edc19, 76 },
    { 0x7a7edc19, 100 },
    { 0, 0 },
    { 0x796edc1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaedc20, 23 },
    { 0xd6cedc21, 91 },
    { 0x851edc21, 33 },
    { 0x653edc23, 93 },
    { 0x350edc21, 98 },
    { 0xa3cedc25, 34 },
    { 0, 0 },
    { 0xc6edc27, 76 },
    { 0x384edc27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996edc2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1edc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0edc37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eedc3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bedc42, 90 },
    { 0, 0 },
    { 0x5caedc44, 66 },
    { 0, 0 },
    { 0x6ededc46, 33 },
    { 0xab3edc47, 100 },
    { 0xfdedc47, 95 },
    { 0xf41edc49, 53 },
    { 0, 0 },
    { 0x9a0edc4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fedc4e, 95 },
    { 0x2b4edc4f, 34 },
    { 0x686edc4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eedc57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eedc5a, 87 },
    { 0x95bedc5b, 95 },
    { 0, 0 },
    { 0x528edc5d, 90 },
    { 0x7bedc5e, 90 },
    { 0xa73edc5d, 76 },
    { 0x6f6edc60, 95 },
    { 0x7b0edc60, 81 },
    { 0xcf2edc60, 95 },
    { 0x189edc61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8edc6b, 61 },
    { 0, 0 },
    { 0xec9edc6d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bedc70, 98 },
    { 0, 0 },
    { 0x989edc72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67edc75, 23 },
    { 0x7b1edc75, 67 },
    { 0x975edc77, 61 },
    { 0, 0 },
    { 0x76aedc79, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894edc7d, 95 },
    { 0xe82edc7e, 25 },
    { 0x67edc7f, 55 },
    { 0x5ededc80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdedc86, 88 },
    { 0x2e1edc87, 90 },
    { 0, 0 },
    { 0x6e4edc89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8edc8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69edc92, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344edc99, 61 },
    { 0xff7edc99, 90 },
    { 0, 0 },
    { 0x337edc9c, 33 },
    { 0, 0 },
    { 0x28bedc9e, 90 },
    { 0x8eaedc9f, 61 },
    { 0x49fedc9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52edca7, 66 },
    { 0xbccedca8, 68 },
    { 0x3b8edca9, 99 },
    { 0x5b1edca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deedcb7, 61 },
    { 0x60dedcb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x305edcbb, 90 },
    { 0xf91edcbc, 81 },
    { 0x312edcbd, 23 },
    { 0x2e7edcbe, 68 },
    { 0, 0 },
    { 0x750edcc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31edcc3, 76 },
    { 0x607edcc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6edcca, 95 },
    { 0xf33edccb, 75 },
    { 0, 0 },
    { 0xf07edccd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x992edcd0, 33 },
    { 0xd8dedcd0, 93 },
    { 0xae0edcd2, 83 },
    { 0x1e9edcd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eedcd6, 55 },
    { 0, 0 },
    { 0x75bedcd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80edcdb, 23 },
    { 0, 0 },
    { 0xc0fedcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9edce0, 67 },
    { 0xccaedce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bedce6, 34 },
    { 0, 0 },
    { 0x97aedce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bedceb, 76 },
    { 0xc98edcec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270edcf2, 33 },
    { 0, 0 },
    { 0xb5eedcf4, 90 },
    { 0xbb2edcf5, 43 },
    { 0x2f7edcf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x156edcf9, 33 },
    { 0xb37edcfa, 77 },
    { 0x920edcfb, 90 },
    { 0x887edcfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aedd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721edd07, 46 },
    { 0xb5fedd07, 61 },
    { 0xd10edd09, 61 },
    { 0xc8edd07, 23 },
    { 0x7b0edd0b, 61 },
    { 0x19edd0b, 23 },
    { 0x6caedd0d, 55 },
    { 0x2ceedd0b, 33 },
    { 0, 0 },
    { 0x531edd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768edd15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x175edd18, 34 },
    { 0, 0 },
    { 0x584edd1a, 90 },
    { 0xf6eedd1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eedd1e, 61 },
    { 0, 0 },
    { 0x492edd20, 12 },
    { 0x2c5edd21, 68 },
    { 0, 0 },
    { 0x930edd23, 91 },
    { 0x6c3edd23, 34 },
    { 0, 0 },
    { 0x8f4edd26, 66 },
    { 0x45dedd27, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x262edd2a, 90 },
    { 0xb9aedd2a, 23 },
    { 0xf94edd2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eedd33, 46 },
    { 0xc3bedd34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0edd38, 90 },
    { 0x5f4edd38, 61 },
    { 0xb7bedd38, 90 },
    { 0xf49edd39, 55 },
    { 0x74dedd3c, 90 },
    { 0x418edd3d, 61 },
    { 0x81cedd3e, 33 },
    { 0x733edd3f, 93 },
    { 0xb80edd40, 76 },
    { 0xffaedd3d, 98 },
    { 0, 0 },
    { 0xb2edd43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5edd47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247edd4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184edd50, 23 },
    { 0xb50edd51, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3edd56, 61 },
    { 0x5dfedd57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcedd5a, 91 },
    { 0xb8aedd5b, 90 },
    { 0xff4edd5c, 94 },
    { 0xfcdedd5b, 33 },
    { 0x9daedd5e, 100 },
    { 0xb99edd5f, 46 },
    { 0xa1eedd60, 33 },
    { 0x883edd5d, 52 },
    { 0x5e3edd62, 90 },
    { 0xb69edd63, 33 },
    { 0x1afedd64, 95 },
    { 0xdb9edd65, 90 },
    { 0, 0 },
    { 0x973edd67, 17 },
    { 0x7c7edd68, 90 },
    { 0x10fedd68, 90 },
    { 0x8bedd67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacedd6f, 38 },
    { 0, 0 },
    { 0xf8edd71, 100 },
    { 0, 0 },
    { 0x783edd73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cedd77, 65 },
    { 0xe3fedd78, 55 },
    { 0x616edd77, 95 },
    { 0xd51edd7a, 33 },
    { 0x323edd7b, 61 },
    { 0, 0 },
    { 0xb84edd7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aedd81, 43 },
    { 0, 0 },
    { 0x60fedd83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334edd8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2edd8f, 52 },
    { 0x4c8edd90, 99 },
    { 0, 0 },
    { 0x43eedd92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bedd96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15aedd9b, 23 },
    { 0xa5bedd9b, 76 },
    { 0, 0 },
    { 0x51eedd9e, 23 },
    { 0, 0 },
    { 0x2a8edda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600eddac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2eddb0, 84 },
    { 0x352eddb0, 33 },
    { 0x989eddb2, 61 },
    { 0x106eddb2, 90 },
    { 0x2bfeddb4, 90 },
    { 0x5c6eddb5, 61 },
    { 0xfa6eddb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802eddba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860eddbd, 85 },
    { 0, 0 },
    { 0x644eddbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbeddc3, 90 },
    { 0x490eddc4, 33 },
    { 0x944eddc5, 95 },
    { 0xba1eddc5, 34 },
    { 0, 0 },
    { 0x794eddc8, 95 },
    { 0x2ddeddc8, 34 },
    { 0x492eddca, 33 },
    { 0x5f4eddca, 68 },
    { 0x3daeddc9, 23 },
    { 0, 0 },
    { 0x43feddce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x499eddd1, 43 },
    { 0x3bfeddd2, 61 },
    { 0x8eceddd3, 67 },
    { 0x73beddd4, 98 },
    { 0xaa8eddd4, 33 },
    { 0xa02eddd4, 90 },
    { 0xecceddd7, 23 },
    { 0xa20eddd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4edddb, 90 },
    { 0, 0 },
    { 0xd9edddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dedde1, 93 },
    { 0, 0 },
    { 0xa23edde3, 81 },
    { 0x54fedde3, 34 },
    { 0, 0 },
    { 0x356edde6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822eddea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24eddf1, 90 },
    { 0x63aeddf2, 90 },
    { 0xe82eddf3, 43 },
    { 0x6fdeddf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2eddf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ede03, 90 },
    { 0x4bede04, 85 },
    { 0xf46ede05, 61 },
    { 0xc8fede05, 90 },
    { 0x904ede03, 98 },
    { 0, 0 },
    { 0xef7ede09, 87 },
    { 0x4ede09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ede0f, 65 },
    { 0x5eeede0f, 33 },
    { 0x929ede11, 78 },
    { 0x7bede12, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeede15, 68 },
    { 0x38ede15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x238ede19, 55 },
    { 0xf9dede19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbede20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afede24, 81 },
    { 0x1faede25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463ede29, 61 },
    { 0x934ede29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3ede33, 99 },
    { 0xb1fede34, 95 },
    { 0xd06ede35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceede38, 61 },
    { 0x20dede39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ede3c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaede41, 55 },
    { 0x12cede41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcede47, 90 },
    { 0x9ebede48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x151ede4b, 61 },
    { 0, 0 },
    { 0x18bede4d, 23 },
    { 0, 0 },
    { 0xd86ede4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ede58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aede5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ede62, 90 },
    { 0, 0 },
    { 0x479ede64, 55 },
    { 0, 0 },
    { 0x732ede66, 76 },
    { 0x1aeede67, 90 },
    { 0x74aede68, 34 },
    { 0xa4cede69, 76 },
    { 0xfe8ede69, 90 },
    { 0x4deede6b, 98 },
    { 0, 0 },
    { 0xcc1ede6d, 81 },
    { 0xdf9ede6e, 61 },
    { 0x64eede6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ede77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fede7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fede7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eede81, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aede87, 95 },
    { 0xd7fede88, 91 },
    { 0x71ede87, 34 },
    { 0x44cede8a, 23 },
    { 0xf68ede8b, 68 },
    { 0xc15ede8b, 52 },
    { 0x982ede8b, 93 },
    { 0xc1dede8e, 55 },
    { 0x412ede88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ede92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ede95, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ede9c, 98 },
    { 0, 0 },
    { 0x3ceede9e, 4 },
    { 0x495ede9f, 46 },
    { 0, 0 },
    { 0x1e0edea1, 52 },
    { 0xadfedea2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549edea8, 88 },
    { 0x18cedea9, 95 },
    { 0x525edeaa, 55 },
    { 0x917edea8, 23 },
    { 0xa6aedea8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797edeb3, 87 },
    { 0x7d8edeb3, 34 },
    { 0x737edeb4, 55 },
    { 0x704edeb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fedeb9, 43 },
    { 0x56cedeba, 77 },
    { 0, 0 },
    { 0x9c1edebc, 61 },
    { 0x261edebd, 91 },
    { 0xb5edebe, 23 },
    { 0x3a1edebf, 23 },
    { 0, 0 },
    { 0x16edec1, 90 },
    { 0xf57edec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18edec7, 95 },
    { 0, 0 },
    { 0x17fedec9, 81 },
    { 0x75dedeca, 69 },
    { 0xa47edec9, 90 },
    { 0x3eaedecc, 33 },
    { 0, 0 },
    { 0x3d6edece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5eded3, 95 },
    { 0x2f7eded4, 95 },
    { 0, 0 },
    { 0x73ceded6, 33 },
    { 0xc9deded7, 23 },
    { 0x47feded8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1edede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x107edee1, 90 },
    { 0x3f7edee2, 90 },
    { 0xae4edee3, 33 },
    { 0xe47edee4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2edee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615edeec, 61 },
    { 0, 0 },
    { 0x60cedeee, 95 },
    { 0x762edeef, 67 },
    { 0x9f0edef0, 90 },
    { 0xf77edef1, 55 },
    { 0x444edef0, 90 },
    { 0x35fedef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x794edef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2edefa, 66 },
    { 0x6e1edefa, 46 },
    { 0, 0 },
    { 0x7ffedefd, 33 },
    { 0, 0 },
    { 0x676edeff, 100 },
    { 0x638edf00, 33 },
    { 0xfaedf00, 61 },
    { 0, 0 },
    { 0x219edf03, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19edf09, 83 },
    { 0x938edf0a, 98 },
    { 0xf2eedf09, 95 },
    { 0x136edf09, 33 },
    { 0xfc3edf0d, 95 },
    { 0xeccedf0a, 95 },
    { 0x491edf0a, 23 },
    { 0x628edf10, 98 },
    { 0xe29edf11, 52 },
    { 0xf3dedf12, 33 },
    { 0xe2edf0b, 81 },
    { 0, 0 },
    { 0x5d9edf15, 43 },
    { 0xc2edf15, 87 },
    { 0xd82edf17, 69 },
    { 0x982edf18, 65 },
    { 0, 0 },
    { 0xd0bedf1a, 100 },
    { 0, 0 },
    { 0x6d6edf1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3edf20, 90 },
    { 0x275edf21, 98 },
    { 0xdadedf22, 90 },
    { 0xdd4edf23, 67 },
    { 0xf6aedf24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6edf27, 61 },
    { 0xc6bedf28, 93 },
    { 0xa87edf27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3edf32, 86 },
    { 0, 0 },
    { 0x55bedf34, 76 },
    { 0, 0 },
    { 0x9b3edf36, 34 },
    { 0x48aedf37, 88 },
    { 0, 0 },
    { 0xaf4edf39, 95 },
    { 0, 0 },
    { 0x31aedf3b, 75 },
    { 0x26fedf3c, 34 },
    { 0x671edf3c, 33 },
    { 0xaefedf3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6edf41, 36 },
    { 0, 0 },
    { 0x872edf43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bedf46, 55 },
    { 0, 0 },
    { 0xc33edf48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6edf55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7edf5c, 55 },
    { 0x6eaedf5d, 33 },
    { 0x262edf5e, 23 },
    { 0xe13edf5f, 33 },
    { 0x3feedf60, 23 },
    { 0x469edf60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dedf65, 43 },
    { 0x503edf66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4edf6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x794edf6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9edf71, 34 },
    { 0xdbcedf72, 61 },
    { 0x869edf72, 98 },
    { 0x5f4edf74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53edf78, 34 },
    { 0x29fedf78, 95 },
    { 0x152edf7a, 33 },
    { 0x31aedf7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127edf84, 23 },
    { 0x16cedf85, 55 },
    { 0x6f5edf86, 33 },
    { 0xe12edf87, 84 },
    { 0, 0 },
    { 0xce4edf89, 90 },
    { 0x6ecedf8a, 96 },
    { 0xde4edf8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494edf93, 98 },
    { 0, 0 },
    { 0xc4cedf95, 95 },
    { 0xf19edf96, 95 },
    { 0, 0 },
    { 0x898edf98, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7edfa0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aedfa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aedfa6, 90 },
    { 0x8d8edfa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8edfae, 61 },
    { 0x4f3edfaf, 91 },
    { 0x3c2edfb0, 55 },
    { 0, 0 },
    { 0xd07edfb2, 34 },
    { 0xd70edfb3, 76 },
    { 0x455edfb4, 98 },
    { 0, 0 },
    { 0x18cedfb6, 33 },
    { 0x81aedfb6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeedfba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128edfc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784edfc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cedfc8, 90 },
    { 0x673edfc9, 61 },
    { 0x9f5edfca, 33 },
    { 0xdf3edfca, 90 },
    { 0xff0edfca, 100 },
    { 0x747edfcd, 99 },
    { 0x630edfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75edfd6, 33 },
    { 0xabeedfd7, 33 },
    { 0xdffedfd7, 34 },
    { 0, 0 },
    { 0xef0edfda, 99 },
    { 0x4baedfdb, 61 },
    { 0x840edfdc, 66 },
    { 0, 0 },
    { 0x16cedfde, 43 },
    { 0x318edfdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fedfe4, 88 },
    { 0, 0 },
    { 0xcb2edfe6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaedfed, 69 },
    { 0x9f6edfee, 33 },
    { 0x742edfef, 23 },
    { 0xad8edff0, 38 },
    { 0x952edff1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbedff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3edff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fedffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x771edfff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464ee003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206ee007, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ee00c, 61 },
    { 0, 0 },
    { 0xb57ee00e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x8caee011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ee015, 23 },
    { 0, 0 },
    { 0xe59ee017, 90 },
    { 0, 0 },
    { 0x10bee019, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59ee020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2ee023, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dee026, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4ee02c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ee02f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ee032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3ee035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ee038, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aee041, 67 },
    { 0xe33ee042, 88 },
    { 0x14fee042, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dee04a, 43 },
    { 0, 0 },
    { 0xfeee04c, 69 },
    { 0, 0 },
    { 0x908ee04e, 78 },
    { 0xa70ee04e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4ee054, 33 },
    { 0x13eee054, 61 },
    { 0, 0 },
    { 0x906ee057, 98 },
    { 0x97dee057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ee05b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ee05e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee06a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadee06e, 95 },
    { 0xebfee06f, 61 },
    { 0, 0 },
    { 0x655ee071, 90 },
    { 0xdd3ee072, 61 },
    { 0xd51ee071, 95 },
    { 0xdbee074, 90 },
    { 0xd97ee075, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacee07a, 55 },
    { 0xaffee07b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ee080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20ee085, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abee088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211ee08c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aee091, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ee094, 66 },
    { 0x127ee095, 33 },
    { 0, 0 },
    { 0xf8ee097, 33 },
    { 0x80dee098, 90 },
    { 0xe95ee097, 98 },
    { 0x5b8ee09a, 95 },
    { 0, 0 },
    { 0xa62ee09c, 55 },
    { 0x20bee09d, 90 },
    { 0x2f8ee09d, 66 },
    { 0xd30ee09d, 98 },
    { 0, 0 },
    { 0x83cee0a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eee0a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ee0a9, 61 },
    { 0x8edee0a9, 69 },
    { 0xd08ee0ab, 98 },
    { 0x20cee0a9, 69 },
    { 0, 0 },
    { 0xceeee0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cee0b1, 61 },
    { 0xcbdee0b2, 61 },
    { 0x33ee0b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afee0bc, 90 },
    { 0xddbee0bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93eee0c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5ee0c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2ee0c6, 55 },
    { 0x7cee0c6, 76 },
    { 0, 0 },
    { 0x62bee0c9, 98 },
    { 0, 0 },
    { 0x293ee0cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9ee0d0, 81 },
    { 0x310ee0d1, 17 },
    { 0x963ee0d2, 76 },
    { 0, 0 },
    { 0x88aee0d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fee0df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ee0e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5ee0e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247ee0ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fee0ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622ee0f4, 34 },
    { 0xa66ee0f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecee0f8, 66 },
    { 0xa1eee0f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99eee103, 90 },
    { 0x4e7ee104, 66 },
    { 0xb2aee105, 90 },
    { 0x6e5ee104, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ee10a, 80 },
    { 0x9eee10b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53ee118, 68 },
    { 0, 0 },
    { 0x7f6ee11a, 46 },
    { 0x315ee11b, 33 },
    { 0, 0 },
    { 0xdedee11d, 101 },
    { 0xbb7ee11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80ee123, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1ee126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacee12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ee131, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dee138, 34 },
    { 0xeceee139, 84 },
    { 0xcebee13a, 52 },
    { 0, 0 },
    { 0x9bbee13c, 98 },
    { 0x2beee13d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aee143, 23 },
    { 0, 0 },
    { 0x66eee145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cee14a, 98 },
    { 0xf8dee14b, 99 },
    { 0x191ee14c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x176ee14f, 85 },
    { 0x905ee14f, 33 },
    { 0xe85ee151, 34 },
    { 0x66eee151, 98 },
    { 0xd05ee14f, 61 },
    { 0xc20ee154, 23 },
    { 0, 0 },
    { 0x5fcee156, 33 },
    { 0, 0 },
    { 0x6ccee158, 61 },
    { 0xa32ee159, 61 },
    { 0, 0 },
    { 0xe4fee15b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ee15e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ee164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee167, 61 },
    { 0, 0 },
    { 0xb4fee169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829ee16e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee173, 82 },
    { 0, 0 },
    { 0xb0aee175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1ee17a, 61 },
    { 0x3ceee17a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ee17f, 33 },
    { 0xf09ee17f, 95 },
    { 0xc10ee181, 80 },
    { 0x295ee182, 46 },
    { 0x6f6ee183, 61 },
    { 0x440ee17f, 98 },
    { 0x256ee185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cee18c, 99 },
    { 0x984ee18d, 52 },
    { 0x497ee18d, 100 },
    { 0x9d6ee18c, 98 },
    { 0x4ebee190, 90 },
    { 0x2e8ee18f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ee194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ee19b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eee1a7, 23 },
    { 0xd5cee1a8, 55 },
    { 0x569ee1a9, 66 },
    { 0x7e0ee1a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ee1af, 88 },
    { 0x9e6ee1b0, 98 },
    { 0, 0 },
    { 0xb4eee1b2, 87 },
    { 0x1ecee1b3, 8 },
    { 0x798ee1b2, 33 },
    { 0xbc7ee1b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9ee1b9, 91 },
    { 0x43aee1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32ee1bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cee1c2, 85 },
    { 0x2adee1c3, 61 },
    { 0x928ee1c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50ee1c8, 98 },
    { 0, 0 },
    { 0x659ee1ca, 98 },
    { 0xb94ee1cb, 76 },
    { 0xdf1ee1cc, 95 },
    { 0x686ee1cc, 17 },
    { 0xf1bee1ca, 90 },
    { 0xc00ee1cf, 90 },
    { 0x6e4ee1cc, 95 },
    { 0xda3ee1cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x872ee1d4, 98 },
    { 0xcf8ee1d5, 55 },
    { 0x525ee1d4, 95 },
    { 0x111ee1d7, 23 },
    { 0x3dbee1d8, 33 },
    { 0xb4fee1d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9ee1de, 90 },
    { 0x8adee1de, 33 },
    { 0xc81ee1e0, 61 },
    { 0, 0 },
    { 0x93cee1e2, 53 },
    { 0x45ee1e3, 98 },
    { 0x257ee1e4, 33 },
    { 0, 0 },
    { 0x7f7ee1e6, 85 },
    { 0x78aee1e6, 90 },
    { 0x3ebee1e7, 34 },
    { 0xcb3ee1e9, 55 },
    { 0x331ee1ea, 52 },
    { 0xc5eee1eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfee1f1, 43 },
    { 0x789ee1f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ee1f5, 95 },
    { 0, 0 },
    { 0x628ee1f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eee1fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x616ee1fe, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dee205, 90 },
    { 0, 0 },
    { 0x169ee207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dee20a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463ee20f, 90 },
    { 0x1c1ee20f, 33 },
    { 0x8beee211, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ee216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0ee21c, 95 },
    { 0x8e2ee21c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ee221, 90 },
    { 0xf32ee222, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ee226, 61 },
    { 0x638ee227, 78 },
    { 0x39bee226, 76 },
    { 0, 0 },
    { 0x45fee22a, 61 },
    { 0xe0dee22a, 55 },
    { 0xe8eee22b, 90 },
    { 0, 0 },
    { 0x1c9ee22e, 33 },
    { 0, 0 },
    { 0xc10ee230, 61 },
    { 0, 0 },
    { 0x15cee232, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7ee239, 33 },
    { 0xa23ee23a, 95 },
    { 0xba9ee23b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374ee242, 52 },
    { 0, 0 },
    { 0x24bee244, 43 },
    { 0x8f0ee245, 55 },
    { 0x442ee246, 55 },
    { 0xbdeee246, 78 },
    { 0xbbcee248, 33 },
    { 0x68cee247, 76 },
    { 0x3b4ee247, 61 },
    { 0xd6dee24b, 59 },
    { 0x467ee249, 23 },
    { 0x58eee245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ee250, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8ee25b, 97 },
    { 0x123ee25c, 33 },
    { 0x46bee25c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86ee268, 90 },
    { 0, 0 },
    { 0x935ee26a, 33 },
    { 0x89bee26b, 93 },
    { 0, 0 },
    { 0x880ee26d, 93 },
    { 0, 0 },
    { 0xbc2ee26f, 61 },
    { 0, 0 },
    { 0x148ee271, 34 },
    { 0xe92ee272, 90 },
    { 0x85eee271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cee279, 77 },
    { 0, 0 },
    { 0xdb8ee27b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ee281, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee285, 23 },
    { 0xc6aee285, 23 },
    { 0xb5ee287, 90 },
    { 0xb18ee288, 43 },
    { 0, 0 },
    { 0xeceee28a, 55 },
    { 0xb1cee28a, 34 },
    { 0x43ee28c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffee28f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ee293, 33 },
    { 0, 0 },
    { 0x4afee295, 76 },
    { 0x2b0ee295, 95 },
    { 0x5ccee297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eee29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940ee2a0, 90 },
    { 0x7e7ee2a1, 66 },
    { 0x5f3ee2a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ee2a5, 34 },
    { 0x774ee2a6, 40 },
    { 0x901ee2a6, 98 },
    { 0x88aee2a6, 55 },
    { 0x91eee2a9, 98 },
    { 0xbfeee2a9, 33 },
    { 0, 0 },
    { 0x333ee2ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eee2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bee2b8, 90 },
    { 0x57dee2b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aee2bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ee2c0, 84 },
    { 0x526ee2c1, 61 },
    { 0x583ee2c2, 98 },
    { 0x18dee2c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcee2c6, 90 },
    { 0, 0 },
    { 0x811ee2c8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dee2cb, 100 },
    { 0xbc1ee2cc, 98 },
    { 0x50cee2cc, 90 },
    { 0x378ee2ce, 33 },
    { 0x7f1ee2cf, 90 },
    { 0x30bee2d0, 33 },
    { 0, 0 },
    { 0x6d7ee2d2, 98 },
    { 0, 0 },
    { 0x755ee2d4, 93 },
    { 0, 0 },
    { 0x3cbee2d6, 77 },
    { 0, 0 },
    { 0xe32ee2d8, 40 },
    { 0x2f6ee2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x232ee2dc, 36 },
    { 0x954ee2dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee2e0, 66 },
    { 0x395ee2e1, 78 },
    { 0xa74ee2e1, 90 },
    { 0x7bdee2e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ee2e6, 34 },
    { 0xaa2ee2e7, 43 },
    { 0xce0ee2e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ee2eb, 85 },
    { 0x65dee2eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ee2f3, 67 },
    { 0x417ee2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ee2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443ee2ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968ee306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dee30e, 78 },
    { 0x573ee30f, 76 },
    { 0xc6eee310, 100 },
    { 0xed7ee310, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7ee315, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x826ee318, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aeee31d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dee323, 90 },
    { 0xc1ee323, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bee327, 61 },
    { 0x98ee327, 95 },
    { 0x8dbee328, 91 },
    { 0xcf3ee32a, 19 },
    { 0xca7ee32a, 65 },
    { 0x9acee32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eee331, 90 },
    { 0, 0 },
    { 0x3dee333, 90 },
    { 0x6aeee334, 98 },
    { 0xe17ee335, 76 },
    { 0x5f5ee334, 34 },
    { 0x62dee334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x363ee33a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aee33d, 61 },
    { 0x63cee33e, 34 },
    { 0x484ee33e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ee344, 43 },
    { 0xbcfee345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfee349, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5ee34d, 90 },
    { 0xbd2ee34d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ee351, 46 },
    { 0xf6aee352, 76 },
    { 0x12cee351, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ee356, 23 },
    { 0, 0 },
    { 0xf62ee358, 90 },
    { 0xc8bee359, 46 },
    { 0x774ee35a, 95 },
    { 0xc23ee35b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ee35f, 90 },
    { 0xd22ee360, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aee364, 90 },
    { 0x1caee365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54ee368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eee36b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5ee36f, 98 },
    { 0x6aaee36f, 33 },
    { 0xab5ee371, 93 },
    { 0xedfee36f, 34 },
    { 0x4dbee373, 98 },
    { 0xe12ee372, 95 },
    { 0x3eee373, 90 },
    { 0x35bee376, 90 },
    { 0xd2ee375, 61 },
    { 0, 0 },
    { 0x45dee379, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eee37c, 34 },
    { 0x25dee37d, 61 },
    { 0, 0 },
    { 0x2c3ee37f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bee382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ee38b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bee38e, 43 },
    { 0xc1aee38f, 61 },
    { 0xf5ee390, 98 },
    { 0x855ee38e, 33 },
    { 0xe51ee392, 76 },
    { 0xe60ee392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee397, 23 },
    { 0x773ee398, 90 },
    { 0xc75ee398, 23 },
    { 0x138ee398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ee39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cee3a3, 91 },
    { 0, 0 },
    { 0x50fee3a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee3a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ee3ab, 87 },
    { 0x7c5ee3ac, 34 },
    { 0x559ee3ac, 33 },
    { 0x31dee3ab, 34 },
    { 0, 0 },
    { 0xb7fee3b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fee3b4, 90 },
    { 0x129ee3b4, 46 },
    { 0xabaee3b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ee3bd, 84 },
    { 0, 0 },
    { 0xb02ee3bf, 85 },
    { 0x1dee3c0, 34 },
    { 0xbd5ee3c0, 76 },
    { 0, 0 },
    { 0xcfdee3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eee3ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddee3d0, 85 },
    { 0x777ee3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8ee3d4, 76 },
    { 0, 0 },
    { 0xfd6ee3d6, 84 },
    { 0x90cee3d7, 90 },
    { 0x5b6ee3d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22ee3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82ee3ea, 33 },
    { 0, 0 },
    { 0x54aee3ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fee3f8, 61 },
    { 0, 0 },
    { 0x9dee3fa, 90 },
    { 0xddbee3fb, 34 },
    { 0x65bee3fa, 98 },
    { 0xe5bee3fa, 95 },
    { 0x9ccee3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ee401, 34 },
    { 0x827ee402, 87 },
    { 0x61eee402, 100 },
    { 0, 0 },
    { 0x4c2ee405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97ee40a, 61 },
    { 0xb48ee40b, 99 },
    { 0, 0 },
    { 0xd74ee40d, 100 },
    { 0xe4eee40e, 91 },
    { 0xfe4ee40e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77ee416, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ee41e, 87 },
    { 0x49ee41f, 61 },
    { 0xbc3ee41f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ee426, 76 },
    { 0xd82ee426, 23 },
    { 0x2d4ee428, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee42b, 95 },
    { 0, 0 },
    { 0x5ee42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831ee432, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ee437, 23 },
    { 0x357ee438, 33 },
    { 0xe13ee439, 85 },
    { 0x699ee438, 76 },
    { 0x650ee43a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459ee440, 33 },
    { 0xd6bee440, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342ee445, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22ee44b, 34 },
    { 0, 0 },
    { 0xa68ee44d, 67 },
    { 0x954ee44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eee452, 49 },
    { 0x81bee453, 93 },
    { 0, 0 },
    { 0xb38ee455, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecee459, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448ee45d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdee463, 61 },
    { 0xe9bee463, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ee46a, 76 },
    { 0, 0 },
    { 0x3e4ee46c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cee470, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ee47b, 78 },
    { 0, 0 },
    { 0xe93ee47d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deee480, 95 },
    { 0xc66ee481, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ee484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ee487, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aee48f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eee497, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x608ee49a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29eee49e, 98 },
    { 0x846ee49e, 34 },
    { 0x976ee4a0, 93 },
    { 0x2daee49e, 23 },
    { 0xf08ee4a2, 87 },
    { 0x68ee4a2, 48 },
    { 0xed1ee4a2, 95 },
    { 0xe2bee4a5, 95 },
    { 0x7f3ee4a1, 90 },
    { 0x7ebee4a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01ee4aa, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ee4b1, 95 },
    { 0x22bee4b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ee4b5, 98 },
    { 0x9a3ee4b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddee4c0, 93 },
    { 0x356ee4c1, 98 },
    { 0x7b5ee4c2, 95 },
    { 0xccbee4c3, 78 },
    { 0xbadee4c4, 88 },
    { 0xbecee4c2, 78 },
    { 0xa79ee4c5, 23 },
    { 0, 0 },
    { 0x542ee4c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ee4cb, 23 },
    { 0x15aee4cc, 34 },
    { 0x91dee4cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ee4d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8ee4df, 69 },
    { 0x635ee4e0, 84 },
    { 0xe3cee4e1, 43 },
    { 0xbc8ee4e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fee4e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281ee4f2, 90 },
    { 0x9a7ee4f3, 43 },
    { 0x343ee4f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fee4f8, 98 },
    { 0x861ee4f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ee4fc, 90 },
    { 0x110ee4fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edee506, 90 },
    { 0xf37ee506, 56 },
    { 0xc7eee507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193ee50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ee511, 90 },
    { 0xdbfee512, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x750ee515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5ee518, 60 },
    { 0, 0 },
    { 0x395ee51a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370ee51f, 33 },
    { 0, 0 },
    { 0x56ee521, 55 },
    { 0x801ee522, 61 },
    { 0, 0 },
    { 0x62fee524, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6ee528, 87 },
    { 0x94bee529, 90 },
    { 0x62cee529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ee52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaee534, 85 },
    { 0xf63ee534, 95 },
    { 0xb9ee534, 78 },
    { 0xb8ee536, 33 },
    { 0, 0 },
    { 0x45ee539, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eee540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cee544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1ee547, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aee54f, 91 },
    { 0x671ee54f, 33 },
    { 0, 0 },
    { 0xb4cee552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caee556, 99 },
    { 0x209ee557, 55 },
    { 0x352ee557, 95 },
    { 0x659ee559, 90 },
    { 0xf4dee559, 61 },
    { 0x5baee556, 61 },
    { 0x7fdee55c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfee560, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cee565, 43 },
    { 0xd86ee566, 43 },
    { 0, 0 },
    { 0xd80ee568, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ee56c, 93 },
    { 0xa6bee56d, 33 },
    { 0x864ee56e, 76 },
    { 0x699ee56f, 90 },
    { 0x3fbee570, 76 },
    { 0x660ee56d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bee577, 84 },
    { 0xcaee577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ee57b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee580, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68ee58b, 78 },
    { 0, 0 },
    { 0xc17ee58d, 90 },
    { 0, 0 },
    { 0xb8bee58f, 97 },
    { 0, 0 },
    { 0xb79ee591, 23 },
    { 0, 0 },
    { 0xcddee593, 88 },
    { 0, 0 },
    { 0x4eeee595, 34 },
    { 0, 0 },
    { 0x6a3ee597, 61 },
    { 0, 0 },
    { 0x711ee599, 43 },
    { 0xc9cee59a, 33 },
    { 0x443ee59b, 76 },
    { 0xab0ee59a, 17 },
    { 0, 0 },
    { 0x1ceee59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fee5a2, 23 },
    { 0x961ee5a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956ee5a8, 47 },
    { 0, 0 },
    { 0x58fee5aa, 95 },
    { 0, 0 },
    { 0x741ee5ac, 93 },
    { 0, 0 },
    { 0x430ee5ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eee5b4, 33 },
    { 0xc6cee5b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee5bb, 95 },
    { 0x739ee5bc, 67 },
    { 0x898ee5bd, 87 },
    { 0, 0 },
    { 0xfb7ee5bf, 33 },
    { 0x56aee5bf, 90 },
    { 0xd65ee5c1, 91 },
    { 0x903ee5c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ee5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cee5c9, 61 },
    { 0x957ee5ca, 46 },
    { 0xf6bee5cb, 61 },
    { 0, 0 },
    { 0xc9ee5cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ee5d2, 98 },
    { 0xa9bee5d2, 52 },
    { 0, 0 },
    { 0x90bee5d5, 90 },
    { 0, 0 },
    { 0xbaee5d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ee5da, 84 },
    { 0, 0 },
    { 0x5d7ee5dc, 95 },
    { 0x2d5ee5dc, 61 },
    { 0xe2ee5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feee5e1, 99 },
    { 0, 0 },
    { 0xd80ee5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x622ee5e6, 90 },
    { 0, 0 },
    { 0xe7eee5e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ee5eb, 33 },
    { 0x4c8ee5ec, 33 },
    { 0xad5ee5eb, 98 },
    { 0, 0 },
    { 0xabbee5ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6ee5f8, 88 },
    { 0x182ee5f8, 55 },
    { 0x64bee5fa, 23 },
    { 0xa76ee5f8, 90 },
    { 0xd9cee5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ee606, 61 },
    { 0xc13ee606, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ee60a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6ee613, 61 },
    { 0x3e2ee613, 90 },
    { 0x83cee614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9dee619, 61 },
    { 0xa9fee61a, 95 },
    { 0, 0 },
    { 0x8e5ee61c, 61 },
    { 0x64cee61c, 90 },
    { 0, 0 },
    { 0xe92ee61f, 98 },
    { 0x28cee620, 76 },
    { 0x878ee621, 26 },
    { 0x377ee61f, 95 },
    { 0, 0 },
    { 0xe31ee624, 34 },
    { 0, 0 },
    { 0x9d4ee626, 98 },
    { 0xc56ee627, 85 },
    { 0, 0 },
    { 0x8eee629, 90 },
    { 0, 0 },
    { 0x6f8ee62b, 90 },
    { 0xf7fee62c, 76 },
    { 0x97cee62d, 61 },
    { 0x962ee62d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cee63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66ee63f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63ee646, 83 },
    { 0xda8ee647, 61 },
    { 0x16bee648, 90 },
    { 0, 0 },
    { 0xeb0ee64a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeee650, 66 },
    { 0xa32ee651, 33 },
    { 0xe84ee652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ee657, 55 },
    { 0, 0 },
    { 0x8e0ee659, 98 },
    { 0xee3ee65a, 93 },
    { 0, 0 },
    { 0x696ee65c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ee663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ee666, 55 },
    { 0xdb5ee667, 23 },
    { 0, 0 },
    { 0xfdbee669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ee66d, 98 },
    { 0, 0 },
    { 0x9e9ee66f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ee676, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x756ee679, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27ee67d, 33 },
    { 0xffdee67d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcee682, 52 },
    { 0x24dee682, 33 },
    { 0x38cee682, 55 },
    { 0x72eee683, 53 },
    { 0, 0 },
    { 0xf1ee687, 55 },
    { 0xa84ee688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dee68f, 23 },
    { 0xe5fee690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faee695, 76 },
    { 0x88aee696, 23 },
    { 0x1acee697, 85 },
    { 0x638ee697, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ee69c, 76 },
    { 0xfa1ee69c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faee6a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eee6a4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaee6aa, 91 },
    { 0, 0 },
    { 0x1cdee6ac, 78 },
    { 0, 0 },
    { 0xd94ee6ae, 23 },
    { 0xf45ee6af, 55 },
    { 0xfe4ee6b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bee6b3, 90 },
    { 0x86dee6b3, 61 },
    { 0, 0 },
    { 0x529ee6b6, 52 },
    { 0x7e9ee6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5ee6bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ee6be, 90 },
    { 0, 0 },
    { 0x3ceee6c0, 61 },
    { 0x84ee6c1, 91 },
    { 0xa46ee6c2, 46 },
    { 0x86aee6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aee6c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04ee6cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee6d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeee6d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ee6dc, 95 },
    { 0xf75ee6dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11ee6e1, 23 },
    { 0x5eeee6e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ee6e6, 93 },
    { 0x6c0ee6e7, 90 },
    { 0xb22ee6e8, 88 },
    { 0x22eee6e8, 98 },
    { 0xcfeee6e8, 23 },
    { 0x82eee6e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4ee6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30ee6f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cee6f8, 61 },
    { 0xe07ee6f9, 93 },
    { 0x339ee6f8, 76 },
    { 0xd7bee6f9, 90 },
    { 0x17aee6fc, 61 },
    { 0, 0 },
    { 0x14bee6fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118ee704, 91 },
    { 0, 0 },
    { 0x97aee706, 33 },
    { 0xe89ee706, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7ee712, 98 },
    { 0x2cfee713, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8ee717, 90 },
    { 0xa40ee718, 78 },
    { 0x8c5ee719, 98 },
    { 0xcddee71a, 77 },
    { 0x9a0ee71a, 65 },
    { 0x896ee71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ee721, 90 },
    { 0x52aee722, 24 },
    { 0x1f2ee723, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ee727, 65 },
    { 0x90eee728, 90 },
    { 0, 0 },
    { 0x1b3ee72a, 66 },
    { 0x926ee72a, 87 },
    { 0x983ee72b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aee731, 95 },
    { 0x96dee732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cee735, 64 },
    { 0x7a6ee735, 90 },
    { 0x70cee735, 61 },
    { 0xc7fee735, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77ee743, 95 },
    { 0xc26ee743, 95 },
    { 0, 0 },
    { 0xae5ee746, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ee749, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdee74d, 55 },
    { 0x96aee74e, 46 },
    { 0x6f2ee74e, 95 },
    { 0xc7cee74f, 100 },
    { 0x4feee74f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcee755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ee759, 76 },
    { 0x7fcee75a, 44 },
    { 0x33eee75b, 61 },
    { 0xb60ee75a, 46 },
    { 0xb79ee75a, 90 },
    { 0x45eee759, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dee764, 90 },
    { 0x61ee764, 23 },
    { 0xce9ee766, 33 },
    { 0x46fee766, 61 },
    { 0xe9dee764, 95 },
    { 0, 0 },
    { 0x2ccee76a, 90 },
    { 0, 0 },
    { 0x78cee76c, 95 },
    { 0x902ee76d, 90 },
    { 0xc21ee76e, 61 },
    { 0, 0 },
    { 0xf74ee770, 77 },
    { 0, 0 },
    { 0x46fee772, 67 },
    { 0x95aee773, 55 },
    { 0x1f5ee773, 90 },
    { 0x2ccee772, 61 },
    { 0xd7aee773, 43 },
    { 0xf9dee777, 98 },
    { 0xfe1ee777, 33 },
    { 0xfb1ee778, 93 },
    { 0xcb7ee77a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abee77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dee786, 46 },
    { 0, 0 },
    { 0x89cee788, 33 },
    { 0, 0 },
    { 0x859ee78a, 90 },
    { 0, 0 },
    { 0x734ee78c, 88 },
    { 0x97ee78c, 64 },
    { 0, 0 },
    { 0xe1cee78f, 90 },
    { 0x802ee78f, 78 },
    { 0xe37ee78f, 23 },
    { 0x3faee78f, 23 },
    { 0, 0 },
    { 0x6a7ee794, 15 },
    { 0, 0 },
    { 0x3fcee796, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eee7a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0ee7a4, 23 },
    { 0x324ee7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8ee7a8, 98 },
    { 0, 0 },
    { 0xe3eee7aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdee7ad, 61 },
    { 0, 0 },
    { 0xd72ee7af, 61 },
    { 0, 0 },
    { 0x12dee7b1, 61 },
    { 0x22eee7b2, 34 },
    { 0x518ee7b1, 100 },
    { 0x154ee7b2, 87 },
    { 0xab5ee7b5, 61 },
    { 0x439ee7b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfee7b9, 93 },
    { 0, 0 },
    { 0xbb6ee7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ee7c8, 61 },
    { 0xc51ee7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788ee7d0, 85 },
    { 0, 0 },
    { 0x3ee7d2, 66 },
    { 0x225ee7d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963ee7d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaee7dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ee7e1, 33 },
    { 0x29cee7e1, 23 },
    { 0x572ee7e3, 95 },
    { 0, 0 },
    { 0x9e7ee7e5, 90 },
    { 0x17aee7e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ee7f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9ee7f4, 76 },
    { 0x890ee7f5, 34 },
    { 0, 0 },
    { 0x3afee7f7, 23 },
    { 0xd29ee7f8, 90 },
    { 0x2d9ee7f7, 23 },
    { 0, 0 },
    { 0x6beee7fb, 34 },
    { 0, 0 },
    { 0xf92ee7fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee800, 33 },
    { 0xe8fee801, 33 },
    { 0, 0 },
    { 0x122ee803, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bee806, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dee809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00ee80c, 17 },
    { 0, 0 },
    { 0x8dfee80e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ee811, 90 },
    { 0, 0 },
    { 0xceee813, 90 },
    { 0, 0 },
    { 0x2f0ee815, 33 },
    { 0xa4cee816, 84 },
    { 0x48cee817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baee81c, 90 },
    { 0x3a3ee81d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ee826, 33 },
    { 0, 0 },
    { 0x667ee828, 90 },
    { 0x30aee829, 90 },
    { 0, 0 },
    { 0x209ee82b, 80 },
    { 0x8f2ee82b, 98 },
    { 0x58aee82d, 90 },
    { 0, 0 },
    { 0x2d2ee82f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ee834, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ee83a, 90 },
    { 0xfdcee83a, 61 },
    { 0x46fee83c, 61 },
    { 0xa96ee83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980ee847, 95 },
    { 0x26fee848, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0eee84e, 33 },
    { 0, 0 },
    { 0xe27ee850, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fee853, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cee858, 90 },
    { 0, 0 },
    { 0x6b2ee85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7ee861, 33 },
    { 0x30dee862, 98 },
    { 0, 0 },
    { 0x5cee864, 55 },
    { 0xc9dee865, 33 },
    { 0, 0 },
    { 0xb8eee867, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaee86b, 95 },
    { 0, 0 },
    { 0x45cee86d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ee870, 85 },
    { 0xdbee871, 91 },
    { 0xee5ee871, 90 },
    { 0x9dbee873, 67 },
    { 0xecee870, 96 },
    { 0, 0 },
    { 0xd4eee876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cee87b, 90 },
    { 0x5b7ee87c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203ee882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cee885, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceee88d, 95 },
    { 0, 0 },
    { 0x367ee88f, 90 },
    { 0xdd3ee890, 76 },
    { 0xdc6ee890, 90 },
    { 0x4f8ee892, 38 },
    { 0x147ee893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bee897, 34 },
    { 0x7afee898, 98 },
    { 0x441ee898, 95 },
    { 0xe8dee89a, 23 },
    { 0x444ee89b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ee89e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fee8a1, 61 },
    { 0xd1fee8a2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81ee8a7, 61 },
    { 0, 0 },
    { 0xec1ee8a9, 90 },
    { 0xe8ee8aa, 87 },
    { 0xd1fee8ab, 61 },
    { 0, 0 },
    { 0xb31ee8ad, 67 },
    { 0x713ee8ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaee8b1, 81 },
    { 0x70fee8b2, 33 },
    { 0x29eee8b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0ee8bc, 95 },
    { 0, 0 },
    { 0x78dee8be, 90 },
    { 0x578ee8bf, 61 },
    { 0x739ee8be, 90 },
    { 0xbbeee8c1, 55 },
    { 0, 0 },
    { 0x819ee8c3, 95 },
    { 0, 0 },
    { 0xd07ee8c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ee8cc, 67 },
    { 0x2e0ee8cd, 85 },
    { 0, 0 },
    { 0xa99ee8cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x479ee8d2, 76 },
    { 0x98dee8d3, 98 },
    { 0xa6cee8d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fee8d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959ee8df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bee8e3, 17 },
    { 0, 0 },
    { 0x51eee8e5, 61 },
    { 0xf48ee8e6, 34 },
    { 0x2e9ee8e7, 90 },
    { 0x627ee8e8, 66 },
    { 0x298ee8e7, 43 },
    { 0, 0 },
    { 0x68ee8eb, 93 },
    { 0, 0 },
    { 0x246ee8ed, 90 },
    { 0x9f7ee8ee, 33 },
    { 0xc94ee8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ee8f2, 66 },
    { 0x6b4ee8f3, 61 },
    { 0x7c5ee8f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4ee8f7, 33 },
    { 0xf7bee8f8, 23 },
    { 0, 0 },
    { 0xcf7ee8fa, 34 },
    { 0xf80ee8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acee901, 55 },
    { 0, 0 },
    { 0xeddee903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ee908, 23 },
    { 0, 0 },
    { 0x5afee90a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827ee911, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfee914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aee918, 17 },
    { 0xc20ee919, 87 },
    { 0x7ccee918, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceee922, 101 },
    { 0xa2bee923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1ee928, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ee92f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x763ee932, 88 },
    { 0, 0 },
    { 0xfc4ee934, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207ee93b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bee947, 95 },
    { 0xa7ee948, 23 },
    { 0x7a1ee948, 90 },
    { 0xbc3ee949, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449ee94e, 34 },
    { 0, 0 },
    { 0x55ee950, 90 },
    { 0, 0 },
    { 0xadeee952, 64 },
    { 0x589ee953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ee968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fee96d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eee973, 61 },
    { 0, 0 },
    { 0x872ee975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ee97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x781ee980, 78 },
    { 0xca9ee981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ee988, 76 },
    { 0x4eaee988, 61 },
    { 0xb31ee989, 98 },
    { 0x998ee98b, 34 },
    { 0x660ee98a, 43 },
    { 0x974ee98d, 91 },
    { 0x3e4ee98e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166ee994, 23 },
    { 0xacbee995, 87 },
    { 0x400ee994, 90 },
    { 0x817ee997, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ee99d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ee9a0, 76 },
    { 0, 0 },
    { 0x216ee9a2, 95 },
    { 0x82cee9a3, 99 },
    { 0xfeaee9a4, 46 },
    { 0xc6bee9a5, 34 },
    { 0, 0 },
    { 0xecdee9a7, 93 },
    { 0xec7ee9a8, 34 },
    { 0xeadee9a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaee9af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eee9b3, 61 },
    { 0, 0 },
    { 0xc43ee9b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbee9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ee9bd, 90 },
    { 0x9ccee9be, 93 },
    { 0xec8ee9bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ee9c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cee9ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dee9d1, 61 },
    { 0, 0 },
    { 0x522ee9d3, 90 },
    { 0x98eee9d4, 95 },
    { 0x576ee9d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bee9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ee9dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5ee9e0, 98 },
    { 0, 0 },
    { 0x9adee9e2, 90 },
    { 0x70cee9e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caee9e7, 99 },
    { 0x177ee9e7, 34 },
    { 0xf3fee9e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ee9f2, 88 },
    { 0, 0 },
    { 0xd4aee9f4, 84 },
    { 0x528ee9f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dee9fa, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cee9fd, 96 },
    { 0x683ee9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaeea05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419eea09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaeea12, 98 },
    { 0x644eea13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323eea17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92eea1b, 61 },
    { 0, 0 },
    { 0x68aeea1d, 33 },
    { 0xeebeea1e, 93 },
    { 0xd26eea1f, 61 },
    { 0x1dfeea1d, 52 },
    { 0x4bceea1f, 43 },
    { 0xf75eea22, 95 },
    { 0, 0 },
    { 0xa70eea24, 88 },
    { 0xd95eea25, 61 },
    { 0, 0 },
    { 0x38deea27, 95 },
    { 0x934eea28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373eea2c, 55 },
    { 0x626eea2d, 93 },
    { 0x483eea2e, 95 },
    { 0xbfaeea2f, 98 },
    { 0x3e0eea2c, 66 },
    { 0x88aeea2e, 88 },
    { 0x9b1eea32, 53 },
    { 0x6d1eea32, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4eea3e, 98 },
    { 0, 0 },
    { 0xd6aeea40, 55 },
    { 0x2a1eea41, 90 },
    { 0x56eeea42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ceea47, 55 },
    { 0x673eea48, 61 },
    { 0, 0 },
    { 0xb78eea4a, 33 },
    { 0x301eea4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aeea50, 61 },
    { 0xd6deea51, 93 },
    { 0, 0 },
    { 0xcabeea53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x225eea56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94eea5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x363eea5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78beea65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19eea68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ceea6e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7deea75, 23 },
    { 0x2c9eea76, 33 },
    { 0xa58eea77, 98 },
    { 0xa80eea78, 66 },
    { 0x5c4eea77, 34 },
    { 0x8e4eea7a, 98 },
    { 0xf4feea79, 96 },
    { 0x5dbeea7c, 55 },
    { 0x28beea7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aeea80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5deea86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408eea8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06eea92, 90 },
    { 0, 0 },
    { 0xb3deea94, 98 },
    { 0x7b4eea95, 23 },
    { 0x928eea95, 33 },
    { 0, 0 },
    { 0x986eea98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05eea9f, 61 },
    { 0, 0 },
    { 0xb9feeaa1, 66 },
    { 0, 0 },
    { 0x38ceeaa3, 34 },
    { 0, 0 },
    { 0x7a3eeaa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfeeaa8, 46 },
    { 0, 0 },
    { 0xbb4eeaaa, 98 },
    { 0, 0 },
    { 0xbbdeeaac, 66 },
    { 0, 0 },
    { 0xcd2eeaae, 88 },
    { 0, 0 },
    { 0x13eeab0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2eeab8, 23 },
    { 0, 0 },
    { 0xfc2eeaba, 84 },
    { 0x1e4eeaba, 95 },
    { 0, 0 },
    { 0x739eeabd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11beeac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ceeac6, 98 },
    { 0xa75eeac7, 33 },
    { 0xbaeeac7, 33 },
    { 0x79feeac9, 65 },
    { 0x213eeac8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5eeacd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beeead4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aeead9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1eeadc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9feeadf, 76 },
    { 0x614eeae0, 23 },
    { 0, 0 },
    { 0x4edeeae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833eeae7, 33 },
    { 0xfc2eeae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3feeaeb, 61 },
    { 0, 0 },
    { 0xa8eeeaed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eeaf4, 61 },
    { 0xca8eeaf5, 61 },
    { 0xd51eeaf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feeafa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beeafe, 34 },
    { 0x6a3eeafe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x445eeb02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eeb0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eeb13, 95 },
    { 0xbbdeeb14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x701eeb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8eeb1a, 61 },
    { 0x13beeb1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268eeb1f, 90 },
    { 0xa1eeeb20, 55 },
    { 0x4b5eeb21, 98 },
    { 0, 0 },
    { 0xde1eeb23, 90 },
    { 0xc1beeb23, 91 },
    { 0x141eeb24, 40 },
    { 0x747eeb26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64deeb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcceeb2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72eeb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x813eeb37, 61 },
    { 0x973eeb37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94eeb44, 60 },
    { 0x941eeb44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91beeb4a, 52 },
    { 0x14ceeb4b, 81 },
    { 0, 0 },
    { 0x5dfeeb4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42beeb50, 33 },
    { 0x7dbeeb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2beeb58, 90 },
    { 0x44eeeb59, 40 },
    { 0x7feeeb59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937eeb61, 33 },
    { 0, 0 },
    { 0xb7beeb63, 98 },
    { 0x623eeb63, 77 },
    { 0x99aeeb65, 100 },
    { 0x2d4eeb63, 95 },
    { 0x98feeb67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeeb6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ceeb73, 90 },
    { 0, 0 },
    { 0x795eeb75, 90 },
    { 0x85eeb76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ceeb79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x45beeb7c, 46 },
    { 0x775eeb7d, 95 },
    { 0, 0 },
    { 0xae7eeb7f, 95 },
    { 0, 0 },
    { 0x845eeb81, 55 },
    { 0x56deeb82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ceeb89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7eeb8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3eeb93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0eeb9b, 99 },
    { 0x8e7eeb9c, 76 },
    { 0x7deeeb9c, 19 },
    { 0x6dfeeb9e, 33 },
    { 0xbdfeeb9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eeeba2, 61 },
    { 0x247eeba3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5feeba6, 78 },
    { 0x4c4eeba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56beebaa, 34 },
    { 0x9a4eebab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ceebae, 61 },
    { 0, 0 },
    { 0xdd0eebb0, 90 },
    { 0x9ceeebb0, 33 },
    { 0, 0 },
    { 0x45feebb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4eebb8, 90 },
    { 0xee9eebb9, 62 },
    { 0, 0 },
    { 0x140eebbb, 43 },
    { 0, 0 },
    { 0x4ceebbd, 76 },
    { 0xe57eebbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30eebc9, 61 },
    { 0, 0 },
    { 0xf03eebcb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6eebcf, 90 },
    { 0xcc7eebd0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7eebd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2eebe1, 90 },
    { 0, 0 },
    { 0x4eeeebe3, 97 },
    { 0x629eebe4, 33 },
    { 0x276eebe5, 90 },
    { 0x9f0eebe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466eebec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eebf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818eebf8, 99 },
    { 0xbe4eebf9, 34 },
    { 0xa07eebfa, 23 },
    { 0xb7beebfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242eec00, 98 },
    { 0, 0 },
    { 0x961eec02, 98 },
    { 0xd8ceec03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5feec06, 23 },
    { 0, 0 },
    { 0xccfeec08, 76 },
    { 0, 0 },
    { 0xd90eec0a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79eec0f, 78 },
    { 0x4c9eec10, 34 },
    { 0xa9feec11, 61 },
    { 0, 0 },
    { 0x12aeec13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2eec17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12eec1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feeec26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eec2c, 93 },
    { 0x82deec2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eec33, 33 },
    { 0x5d0eec34, 33 },
    { 0x75beec35, 51 },
    { 0x5f2eec36, 91 },
    { 0x188eec36, 23 },
    { 0xe3feec36, 61 },
    { 0x4dbeec36, 95 },
    { 0x191eec38, 99 },
    { 0, 0 },
    { 0xd74eec3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68eec3f, 23 },
    { 0, 0 },
    { 0x8c5eec41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cceec47, 40 },
    { 0xe8feec48, 99 },
    { 0xad0eec49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975eec50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11beec57, 51 },
    { 0x5f2eec58, 33 },
    { 0x603eec58, 92 },
    { 0xcbfeec5a, 95 },
    { 0xcceeec59, 98 },
    { 0, 0 },
    { 0x504eec5d, 80 },
    { 0, 0 },
    { 0x84deec5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaeec62, 33 },
    { 0xeaeeec63, 52 },
    { 0, 0 },
    { 0x4b3eec65, 81 },
    { 0xf6eeec66, 100 },
    { 0, 0 },
    { 0x17deec68, 98 },
    { 0, 0 },
    { 0x17deec6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedeec6d, 43 },
    { 0, 0 },
    { 0x4c9eec6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32feec76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeec7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbceec81, 88 },
    { 0x900eec82, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7eec88, 100 },
    { 0, 0 },
    { 0x8deeec8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0eec8d, 61 },
    { 0x1dceec8d, 61 },
    { 0xa0beec8f, 90 },
    { 0xc3eec90, 85 },
    { 0xda7eec8e, 98 },
    { 0xd44eec92, 90 },
    { 0x95beec8e, 61 },
    { 0x4e8eec94, 23 },
    { 0xec9eec91, 90 },
    { 0, 0 },
    { 0xc81eec97, 33 },
    { 0xa1deec97, 67 },
    { 0, 0 },
    { 0x829eec9a, 90 },
    { 0, 0 },
    { 0x317eec9c, 33 },
    { 0, 0 },
    { 0x90beec9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42feeca2, 33 },
    { 0, 0 },
    { 0x732eeca4, 55 },
    { 0, 0 },
    { 0x26ceeca6, 90 },
    { 0xc2deeca7, 90 },
    { 0x21beeca7, 87 },
    { 0, 0 },
    { 0x88eecaa, 61 },
    { 0, 0 },
    { 0x48deecac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5eecb0, 76 },
    { 0x773eecb1, 17 },
    { 0xbbfeecb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaceecbb, 61 },
    { 0x2d5eecbb, 90 },
    { 0x34ceecbd, 90 },
    { 0x673eecbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813eecce, 95 },
    { 0, 0 },
    { 0x801eecd0, 95 },
    { 0xee9eecd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5eecd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7eecd7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5eecda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7eecde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0eece1, 43 },
    { 0xef6eece2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6beecea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeeced, 76 },
    { 0xa82eecee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8eecf4, 61 },
    { 0x862eecf5, 95 },
    { 0xc5deecf4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baeecfe, 56 },
    { 0, 0 },
    { 0xa29eed00, 23 },
    { 0xbb4eed01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93feed04, 99 },
    { 0x661eed05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0feed08, 95 },
    { 0, 0 },
    { 0x29beed0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0eed0e, 55 },
    { 0xd8eed0e, 66 },
    { 0x91aeed10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41feed1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eed1d, 91 },
    { 0x81eed1d, 90 },
    { 0, 0 },
    { 0xa14eed20, 95 },
    { 0x418eed21, 43 },
    { 0xac3eed22, 85 },
    { 0xa90eed22, 66 },
    { 0, 0 },
    { 0xd3beed25, 76 },
    { 0, 0 },
    { 0xdcbeed27, 61 },
    { 0, 0 },
    { 0xd94eed29, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deed2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbeed30, 23 },
    { 0xf60eed30, 76 },
    { 0x4a2eed31, 33 },
    { 0x203eed33, 33 },
    { 0xd6feed33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8eed37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11feed3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaeed3f, 43 },
    { 0, 0 },
    { 0xd4deed41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eed4d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31eed50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4deed55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x843eed58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7eed5d, 80 },
    { 0, 0 },
    { 0x2f7eed5f, 61 },
    { 0xe9eeed60, 61 },
    { 0x4dfeed61, 23 },
    { 0x77feed60, 99 },
    { 0, 0 },
    { 0xdfeed64, 65 },
    { 0xcdfeed64, 98 },
    { 0x311eed66, 76 },
    { 0, 0 },
    { 0xe9deed68, 99 },
    { 0xe69eed69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdeed6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773eed71, 84 },
    { 0x234eed72, 76 },
    { 0x290eed73, 100 },
    { 0xa4feed74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2eed7a, 55 },
    { 0x119eed7b, 95 },
    { 0x62eed7a, 77 },
    { 0, 0 },
    { 0x55eeed7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x330eed81, 61 },
    { 0x2cceed81, 99 },
    { 0xf92eed83, 90 },
    { 0, 0 },
    { 0x1b2eed85, 43 },
    { 0xc71eed86, 34 },
    { 0xc34eed86, 23 },
    { 0x609eed88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x895eed8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80feed8e, 34 },
    { 0x5ceeed8f, 69 },
    { 0xa86eed8e, 34 },
    { 0xfdfeed8e, 90 },
    { 0x1fbeed90, 55 },
    { 0x34feed93, 95 },
    { 0x226eed94, 90 },
    { 0x7ddeed95, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7eed99, 61 },
    { 0xf31eed9a, 33 },
    { 0xbcaeed9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53eeda1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eeda4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aeeda7, 87 },
    { 0, 0 },
    { 0x223eeda9, 93 },
    { 0, 0 },
    { 0xcadeedab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0beedb0, 61 },
    { 0xc63eedb1, 55 },
    { 0xb64eedb2, 33 },
    { 0x8d2eedb3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x97beedb6, 90 },
    { 0x1d4eedb6, 75 },
    { 0, 0 },
    { 0xde3eedb9, 93 },
    { 0, 0 },
    { 0xd9ceedbb, 95 },
    { 0x8dbeedbb, 61 },
    { 0x35feedbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2eedc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beedca, 95 },
    { 0, 0 },
    { 0xc11eedcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x687eedcf, 76 },
    { 0, 0 },
    { 0x147eedd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28eedd5, 77 },
    { 0xb50eedd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x515eedd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202eeddf, 52 },
    { 0, 0 },
    { 0x481eede1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaceede4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9eede8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eedec, 90 },
    { 0x317eeded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248eedf3, 43 },
    { 0xb6eedf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdeedfa, 56 },
    { 0xd05eedfb, 98 },
    { 0x10feedfb, 43 },
    { 0xc5aeedfd, 55 },
    { 0, 0 },
    { 0x6d0eedff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daeee08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x904eee0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56feee0f, 55 },
    { 0xaf0eee10, 97 },
    { 0x29eeee10, 98 },
    { 0, 0 },
    { 0x13feee13, 98 },
    { 0x293eee14, 97 },
    { 0x589eee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ceee1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1eee1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8beee21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x177eee24, 74 },
    { 0x1bdeee25, 98 },
    { 0xa47eee26, 76 },
    { 0, 0 },
    { 0x26eeee28, 98 },
    { 0x2afeee28, 90 },
    { 0, 0 },
    { 0x74aeee2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deee36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325eee3d, 52 },
    { 0x9e6eee3e, 43 },
    { 0x312eee3f, 95 },
    { 0x255eee3f, 90 },
    { 0xcd3eee3d, 43 },
    { 0x19eeee3f, 55 },
    { 0, 0 },
    { 0x11beee44, 42 },
    { 0, 0 },
    { 0x32aeee46, 98 },
    { 0x37beee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ceee4a, 61 },
    { 0x2dceee4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147eee4f, 33 },
    { 0, 0 },
    { 0xd6aeee51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44eee55, 95 },
    { 0, 0 },
    { 0xfb4eee57, 76 },
    { 0x75feee58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bceee5b, 95 },
    { 0, 0 },
    { 0x2dbeee5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4eee60, 78 },
    { 0xc0deee60, 95 },
    { 0x24feee60, 33 },
    { 0, 0 },
    { 0x6f1eee64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0eeee67, 43 },
    { 0xe0aeee68, 87 },
    { 0, 0 },
    { 0x300eee6a, 55 },
    { 0x46eeee6b, 88 },
    { 0x48deee6a, 69 },
    { 0x19eeee6d, 100 },
    { 0x3f8eee6c, 77 },
    { 0xad5eee6f, 95 },
    { 0x540eee6c, 98 },
    { 0x3edeee71, 23 },
    { 0, 0 },
    { 0xdfeee73, 95 },
    { 0xc3eee74, 95 },
    { 0xaf4eee75, 99 },
    { 0xddfeee74, 91 },
    { 0xc6eee77, 98 },
    { 0x72deee78, 61 },
    { 0x5f3eee74, 61 },
    { 0x3fbeee7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfeee7e, 90 },
    { 0x622eee7e, 61 },
    { 0x76beee80, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x857eee83, 90 },
    { 0, 0 },
    { 0xc79eee85, 23 },
    { 0xe27eee86, 78 },
    { 0, 0 },
    { 0x9b5eee88, 76 },
    { 0, 0 },
    { 0xdbeee8a, 61 },
    { 0x70eee8b, 90 },
    { 0x100eee8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ceee8f, 43 },
    { 0, 0 },
    { 0x9e4eee91, 98 },
    { 0, 0 },
    { 0x249eee93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aeee9b, 95 },
    { 0, 0 },
    { 0x5a9eee9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17eeea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69eeea9, 61 },
    { 0xfb9eeeaa, 85 },
    { 0x764eeeab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deeeb1, 61 },
    { 0, 0 },
    { 0xc9aeeeb3, 81 },
    { 0, 0 },
    { 0x500eeeb5, 90 },
    { 0xebbeeeb6, 100 },
    { 0x335eeeb7, 43 },
    { 0x913eeeb7, 46 },
    { 0x6b2eeeb9, 85 },
    { 0x933eeeb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0eeec3, 61 },
    { 0x699eeec3, 90 },
    { 0x966eeec3, 55 },
    { 0xc2feeec5, 76 },
    { 0x9f2eeec3, 23 },
    { 0x51eeec8, 33 },
    { 0, 0 },
    { 0xb16eeeca, 91 },
    { 0x8e1eeecb, 34 },
    { 0xe90eeecc, 61 },
    { 0xe6aeeecb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103eeed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5eeed6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454eeedc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825eeee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6eeeeb, 61 },
    { 0x24feeeec, 66 },
    { 0xc4beeeed, 95 },
    { 0x788eeeed, 34 },
    { 0xd01eeeef, 76 },
    { 0x6dceeef0, 77 },
    { 0, 0 },
    { 0x134eeef2, 61 },
    { 0x4faeeef3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706eeefb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x413eeefe, 90 },
    { 0xcffeeeff, 90 },
    { 0x13aeeeff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7eef03, 55 },
    { 0x214eef03, 87 },
    { 0x97aeef05, 23 },
    { 0x978eef06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189eef0e, 85 },
    { 0, 0 },
    { 0x9f6eef10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eeef1e, 43 },
    { 0x188eef1f, 95 },
    { 0x905eef20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3eef23, 23 },
    { 0, 0 },
    { 0xd4eef25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56beef28, 100 },
    { 0, 0 },
    { 0x8aeef2a, 33 },
    { 0, 0 },
    { 0x25ceef2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0eef32, 100 },
    { 0, 0 },
    { 0xeddeef34, 46 },
    { 0, 0 },
    { 0x6fbeef36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199eef3b, 61 },
    { 0xa3beef3c, 61 },
    { 0x860eef3d, 34 },
    { 0x13beef3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1eef42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eeef48, 65 },
    { 0xc67eef48, 67 },
    { 0xa7ceef49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeeef4d, 55 },
    { 0x115eef4d, 34 },
    { 0x7bfeef4d, 90 },
    { 0x27ceef50, 61 },
    { 0xab0eef50, 76 },
    { 0x9b4eef51, 98 },
    { 0x9a9eef53, 61 },
    { 0x10beef4e, 98 },
    { 0x8adeef4e, 43 },
    { 0xb5ceef50, 90 },
    { 0x898eef4d, 55 },
    { 0, 0 },
    { 0x431eef59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6eef61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30feef65, 61 },
    { 0x36eef66, 90 },
    { 0xbbceef65, 90 },
    { 0x4e1eef68, 76 },
    { 0xc57eef65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2eef6c, 43 },
    { 0xa87eef6d, 90 },
    { 0x1c1eef6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636eef73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2eef7b, 90 },
    { 0xe00eef7c, 101 },
    { 0xb6ceef7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aeef81, 91 },
    { 0xccbeef82, 61 },
    { 0xba0eef83, 61 },
    { 0xcc7eef84, 95 },
    { 0x8b7eef84, 61 },
    { 0x3cfeef86, 61 },
    { 0x234eef81, 98 },
    { 0xbe2eef83, 66 },
    { 0xc8eeef89, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02eef8f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eeef95, 33 },
    { 0, 0 },
    { 0x52deef97, 29 },
    { 0, 0 },
    { 0xf1beef99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4eef9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eeefa1, 33 },
    { 0x4d5eefa2, 33 },
    { 0x5f2eefa3, 61 },
    { 0xca6eefa4, 90 },
    { 0x156eefa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445eefac, 67 },
    { 0xb5ceefad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879eefb3, 55 },
    { 0x191eefb4, 67 },
    { 0xcf8eefb3, 66 },
    { 0x77aeefb3, 23 },
    { 0x23ceefb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x227eefba, 93 },
    { 0, 0 },
    { 0xec5eefbc, 33 },
    { 0, 0 },
    { 0x400eefbe, 61 },
    { 0x370eefbf, 90 },
    { 0x110eefbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659eefc4, 37 },
    { 0xeaeeefc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x806eefc8, 61 },
    { 0xba1eefc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881eefcf, 55 },
    { 0x1e7eefd0, 98 },
    { 0x4adeefcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x624eefd4, 61 },
    { 0, 0 },
    { 0xa9feefd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eefdb, 93 },
    { 0x2d4eefdc, 98 },
    { 0xbbaeefdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabeefe0, 90 },
    { 0xaefeefe1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883eefe7, 40 },
    { 0xc79eefe8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10deeff2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7eeff9, 55 },
    { 0xfc6eeff9, 33 },
    { 0x25eeff9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abef003, 99 },
    { 0xd50ef004, 72 },
    { 0x359ef004, 55 },
    { 0x972ef004, 23 },
    { 0x907ef007, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7ef00b, 75 },
    { 0, 0 },
    { 0xaa2ef00d, 96 },
    { 0, 0 },
    { 0x8c5ef00f, 90 },
    { 0xb6bef010, 76 },
    { 0x5acef011, 23 },
    { 0xe9fef012, 23 },
    { 0, 0 },
    { 0xf1eef014, 95 },
    { 0, 0 },
    { 0x201ef016, 66 },
    { 0xf28ef017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697ef020, 23 },
    { 0x9d9ef021, 61 },
    { 0, 0 },
    { 0xb14ef023, 95 },
    { 0, 0 },
    { 0x22ef025, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bef031, 90 },
    { 0xc6fef031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ef03a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17ef03e, 46 },
    { 0x97eef03e, 34 },
    { 0x9e2ef03f, 33 },
    { 0x845ef041, 55 },
    { 0xa15ef042, 55 },
    { 0x47bef043, 23 },
    { 0xc26ef044, 77 },
    { 0x199ef03f, 66 },
    { 0x4b0ef046, 94 },
    { 0x327ef045, 34 },
    { 0xdeef048, 90 },
    { 0x757ef047, 90 },
    { 0x22fef03e, 23 },
    { 0xde8ef04b, 76 },
    { 0x811ef03f, 61 },
    { 0x1ddef04d, 61 },
    { 0xaafef04e, 55 },
    { 0x99def04e, 43 },
    { 0x57ef04d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6ef055, 95 },
    { 0x2ffef056, 77 },
    { 0x4d3ef057, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fef05b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeef05f, 55 },
    { 0, 0 },
    { 0x3eef061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87def06f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aef077, 61 },
    { 0, 0 },
    { 0x4acef079, 78 },
    { 0, 0 },
    { 0x2d4ef07b, 43 },
    { 0xcb0ef07c, 78 },
    { 0xe31ef07c, 99 },
    { 0xf75ef07d, 72 },
    { 0x798ef07f, 61 },
    { 0x2aeef07f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x209ef083, 95 },
    { 0xf63ef084, 98 },
    { 0, 0 },
    { 0xa9def086, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ef08a, 100 },
    { 0xb77ef08a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bef091, 90 },
    { 0x235ef091, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ef09a, 61 },
    { 0xca5ef09b, 90 },
    { 0x403ef09c, 98 },
    { 0x8cef09a, 90 },
    { 0xcabef09e, 90 },
    { 0xed7ef09b, 61 },
    { 0x24def0a0, 90 },
    { 0xab0ef0a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddef0a6, 90 },
    { 0x9f5ef0a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x205ef0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcef0ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2ef0b6, 87 },
    { 0xe3eef0b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cef0bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fef0c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42def0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaef0cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7ef0d3, 98 },
    { 0xce6ef0d4, 100 },
    { 0x890ef0d5, 34 },
    { 0x93ef0d4, 34 },
    { 0x4a9ef0d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bef0da, 55 },
    { 0x39fef0da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98def0e0, 87 },
    { 0x93ef0e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5ef0e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88ef0e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ef0ef, 23 },
    { 0xb15ef0f0, 90 },
    { 0xfecef0ef, 66 },
    { 0xec2ef0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95ef0fa, 61 },
    { 0x387ef0fa, 99 },
    { 0x24bef0fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851ef102, 99 },
    { 0x954ef103, 87 },
    { 0, 0 },
    { 0x9b2ef105, 33 },
    { 0, 0 },
    { 0xce4ef107, 92 },
    { 0, 0 },
    { 0xa67ef109, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ef10c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ef10f, 55 },
    { 0, 0 },
    { 0x81bef111, 76 },
    { 0, 0 },
    { 0xd16ef113, 90 },
    { 0, 0 },
    { 0x420ef115, 78 },
    { 0x486ef116, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcef119, 17 },
    { 0x666ef11a, 23 },
    { 0xff3ef119, 100 },
    { 0x627ef11b, 99 },
    { 0xc96ef11d, 91 },
    { 0, 0 },
    { 0x81def11f, 90 },
    { 0x4d0ef120, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ef123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2def129, 61 },
    { 0xf31ef12a, 52 },
    { 0x48fef129, 43 },
    { 0, 0 },
    { 0x360ef12d, 23 },
    { 0x9fef12d, 61 },
    { 0, 0 },
    { 0xf2eef130, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cef135, 78 },
    { 0xaa9ef136, 98 },
    { 0x605ef137, 76 },
    { 0xc8bef137, 76 },
    { 0xfbfef139, 90 },
    { 0x435ef139, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ef13d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef140, 99 },
    { 0xba5ef141, 64 },
    { 0xe7ef142, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ef147, 40 },
    { 0, 0 },
    { 0x3e9ef149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ef14e, 23 },
    { 0x85fef14f, 100 },
    { 0, 0 },
    { 0x9aef151, 23 },
    { 0, 0 },
    { 0xafef153, 100 },
    { 0xbf6ef154, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x332ef157, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ef15b, 23 },
    { 0xa5bef15b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5ef15f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccef163, 33 },
    { 0xe3aef164, 61 },
    { 0xa50ef164, 55 },
    { 0x6a9ef164, 34 },
    { 0x589ef167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ef16f, 95 },
    { 0x86ef170, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ef174, 90 },
    { 0xfe8ef175, 67 },
    { 0xc55ef176, 61 },
    { 0x783ef177, 90 },
    { 0x5d4ef177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131ef17d, 38 },
    { 0, 0 },
    { 0xb8bef17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6ef182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ef185, 98 },
    { 0, 0 },
    { 0x358ef187, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ef18a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ef18e, 23 },
    { 0x33fef18f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cef198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x393ef19b, 85 },
    { 0, 0 },
    { 0x7d7ef19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ef1a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6def1a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ef1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ef1aa, 75 },
    { 0xf3cef1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13def1b1, 23 },
    { 0, 0 },
    { 0xf84ef1b3, 61 },
    { 0xe05ef1b3, 46 },
    { 0x362ef1b3, 90 },
    { 0, 0 },
    { 0x61def1b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bef1ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ef1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fef1c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ef1c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ef1cf, 95 },
    { 0xd11ef1d0, 80 },
    { 0x6e3ef1d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cef1d5, 43 },
    { 0, 0 },
    { 0xea8ef1d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ef1df, 23 },
    { 0, 0 },
    { 0x8d5ef1e1, 34 },
    { 0, 0 },
    { 0xddfef1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef1ef, 95 },
    { 0x214ef1f0, 95 },
    { 0x5fef1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccef1f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bef1fb, 33 },
    { 0x583ef1fb, 90 },
    { 0x3ebef1fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x808ef200, 76 },
    { 0x48eef201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfef206, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cef209, 34 },
    { 0, 0 },
    { 0x753ef20b, 65 },
    { 0x29fef20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02ef210, 96 },
    { 0, 0 },
    { 0xf5cef212, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39ef218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeef21b, 33 },
    { 0, 0 },
    { 0x65eef21d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eef222, 90 },
    { 0, 0 },
    { 0x21cef224, 67 },
    { 0x501ef225, 76 },
    { 0, 0 },
    { 0xcc2ef227, 76 },
    { 0, 0 },
    { 0x8e8ef229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fef22e, 23 },
    { 0, 0 },
    { 0x132ef230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ef234, 55 },
    { 0xeffef235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeef238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01ef23b, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ef23f, 91 },
    { 0x505ef240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03ef24c, 33 },
    { 0, 0 },
    { 0x49eef24e, 23 },
    { 0, 0 },
    { 0xd3eef250, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ef253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x798ef256, 100 },
    { 0x762ef256, 76 },
    { 0, 0 },
    { 0x282ef259, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ef260, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccef264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfef26e, 55 },
    { 0, 0 },
    { 0xfa9ef270, 98 },
    { 0x5ddef270, 98 },
    { 0, 0 },
    { 0x96def273, 23 },
    { 0x934ef274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fef27a, 98 },
    { 0, 0 },
    { 0x13fef27c, 33 },
    { 0xe9bef27d, 90 },
    { 0x967ef27e, 55 },
    { 0x58eef27e, 34 },
    { 0xd76ef27d, 61 },
    { 0x9c2ef27c, 76 },
    { 0x76aef27e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdef285, 46 },
    { 0xa39ef285, 23 },
    { 0x99cef286, 52 },
    { 0, 0 },
    { 0x11cef289, 90 },
    { 0, 0 },
    { 0xda2ef28b, 90 },
    { 0x675ef28c, 55 },
    { 0, 0 },
    { 0xa2fef28e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef291, 61 },
    { 0, 0 },
    { 0x4acef293, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceef296, 43 },
    { 0x153ef297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cef29a, 90 },
    { 0x4eef29b, 90 },
    { 0xdb1ef29b, 95 },
    { 0x957ef29c, 98 },
    { 0x324ef29b, 98 },
    { 0xd17ef29c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eef2a3, 78 },
    { 0xf43ef2a4, 90 },
    { 0xdd1ef2a5, 95 },
    { 0xeacef2a3, 95 },
    { 0x1c7ef2a6, 99 },
    { 0x250ef2a8, 76 },
    { 0xb1bef2a5, 98 },
    { 0xa2aef2aa, 95 },
    { 0x90fef2a6, 66 },
    { 0xe40ef2ac, 61 },
    { 0xca2ef2aa, 95 },
    { 0x7deef2ae, 34 },
    { 0, 0 },
    { 0x764ef2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ef2b8, 33 },
    { 0x2fef2b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef2bc, 23 },
    { 0xaa6ef2bd, 55 },
    { 0, 0 },
    { 0x14aef2bf, 34 },
    { 0xd9def2bf, 33 },
    { 0, 0 },
    { 0x594ef2c2, 93 },
    { 0x66aef2c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ef2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def2d3, 88 },
    { 0x43ef2d4, 95 },
    { 0x1b6ef2d5, 43 },
    { 0x9aef2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fef2db, 91 },
    { 0, 0 },
    { 0x474ef2dd, 43 },
    { 0, 0 },
    { 0x8eef2df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4ef2e6, 98 },
    { 0xd17ef2e7, 100 },
    { 0, 0 },
    { 0x7c0ef2e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafef2ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aef2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x228ef2f8, 33 },
    { 0xa82ef2f8, 43 },
    { 0, 0 },
    { 0xe56ef2fb, 55 },
    { 0, 0 },
    { 0xd6ef2fd, 61 },
    { 0xd3aef2fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fef307, 91 },
    { 0, 0 },
    { 0xfa1ef309, 23 },
    { 0, 0 },
    { 0x7d6ef30b, 33 },
    { 0x17aef30c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ef314, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaef317, 55 },
    { 0x783ef317, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54ef32b, 94 },
    { 0xe67ef32c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8ef330, 33 },
    { 0, 0 },
    { 0xa9cef332, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ef337, 33 },
    { 0xe41ef337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93ef33b, 48 },
    { 0xb85ef33b, 66 },
    { 0xc81ef33b, 87 },
    { 0xaa7ef33b, 61 },
    { 0x754ef33c, 61 },
    { 0xb61ef33c, 33 },
    { 0xad3ef341, 33 },
    { 0x3ccef342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ef349, 43 },
    { 0, 0 },
    { 0x4f6ef34b, 61 },
    { 0, 0 },
    { 0x76def34d, 93 },
    { 0, 0 },
    { 0xa3bef34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ef358, 90 },
    { 0x18fef359, 61 },
    { 0xc8bef358, 84 },
    { 0xcc6ef35b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09ef363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50ef367, 33 },
    { 0x59aef368, 33 },
    { 0x941ef369, 23 },
    { 0xaa9ef368, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x134ef36d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feef371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ef376, 33 },
    { 0x6b3ef376, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3ef37c, 55 },
    { 0, 0 },
    { 0xe63ef37e, 95 },
    { 0xe0aef37f, 33 },
    { 0xe33ef380, 95 },
    { 0x32bef381, 100 },
    { 0x2a9ef382, 95 },
    { 0, 0 },
    { 0x697ef384, 87 },
    { 0, 0 },
    { 0xa01ef386, 90 },
    { 0x305ef387, 43 },
    { 0x3d9ef388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbef390, 98 },
    { 0x3e2ef391, 55 },
    { 0, 0 },
    { 0x36eef393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdef39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7ef3a8, 61 },
    { 0x20cef3a9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ef3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6ef3af, 61 },
    { 0xb75ef3b0, 90 },
    { 0, 0 },
    { 0x27ef3b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef3b6, 67 },
    { 0x125ef3b7, 46 },
    { 0xecef3b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5ef3bb, 23 },
    { 0, 0 },
    { 0xee4ef3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4ef3c0, 33 },
    { 0, 0 },
    { 0x563ef3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ef3c9, 61 },
    { 0xc19ef3ca, 88 },
    { 0x93ef3c9, 61 },
    { 0x238ef3cc, 98 },
    { 0xc55ef3cc, 33 },
    { 0xb38ef3cd, 98 },
    { 0x84eef3c9, 55 },
    { 0x47aef3c9, 23 },
    { 0x7d6ef3cb, 23 },
    { 0x9aef3d2, 34 },
    { 0x5e1ef3d2, 98 },
    { 0xd9aef3d3, 43 },
    { 0x2d4ef3d5, 91 },
    { 0, 0 },
    { 0xaa7ef3d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5ef3dd, 23 },
    { 0x988ef3de, 46 },
    { 0x567ef3df, 43 },
    { 0xdeaef3df, 91 },
    { 0xa11ef3e1, 55 },
    { 0x1ffef3e2, 85 },
    { 0xa21ef3e0, 23 },
    { 0x4c3ef3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ef3e8, 95 },
    { 0x828ef3e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899ef3ec, 92 },
    { 0, 0 },
    { 0x6abef3ee, 61 },
    { 0x184ef3ee, 33 },
    { 0x15ef3f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75ef3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ef3fb, 61 },
    { 0, 0 },
    { 0x9c4ef3fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ef402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ef409, 98 },
    { 0, 0 },
    { 0xd7eef40b, 98 },
    { 0, 0 },
    { 0x5f3ef40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ef412, 61 },
    { 0, 0 },
    { 0x3a8ef414, 55 },
    { 0x7bfef414, 23 },
    { 0x2e0ef416, 43 },
    { 0x186ef417, 43 },
    { 0x207ef415, 99 },
    { 0x7e2ef419, 90 },
    { 0x17aef41a, 36 },
    { 0, 0 },
    { 0xf06ef41c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23ef424, 32 },
    { 0, 0 },
    { 0xa3fef426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aef430, 91 },
    { 0x20eef430, 33 },
    { 0xf49ef432, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ef436, 95 },
    { 0xb97ef437, 33 },
    { 0x26def438, 98 },
    { 0x52bef436, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884ef43f, 52 },
    { 0x6a7ef43f, 92 },
    { 0x333ef441, 23 },
    { 0x150ef442, 66 },
    { 0x19cef443, 98 },
    { 0xfb5ef444, 64 },
    { 0xb5def43f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515ef44f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ef455, 90 },
    { 0, 0 },
    { 0x2aaef457, 95 },
    { 0x609ef457, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ef45b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ef463, 33 },
    { 0x7f4ef464, 77 },
    { 0x6d1ef464, 76 },
    { 0x70fef465, 33 },
    { 0xc2bef467, 77 },
    { 0x179ef467, 95 },
    { 0xe20ef468, 61 },
    { 0x33aef46a, 89 },
    { 0, 0 },
    { 0xb18ef46c, 66 },
    { 0xf91ef46c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bef470, 67 },
    { 0, 0 },
    { 0x16eef472, 33 },
    { 0x11fef472, 23 },
    { 0, 0 },
    { 0x8fcef475, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51ef478, 34 },
    { 0, 0 },
    { 0x94cef47a, 33 },
    { 0, 0 },
    { 0x9d4ef47c, 95 },
    { 0, 0 },
    { 0x3bcef47e, 81 },
    { 0x2f8ef47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ef483, 33 },
    { 0, 0 },
    { 0xcefef485, 23 },
    { 0, 0 },
    { 0x297ef487, 40 },
    { 0xc95ef487, 76 },
    { 0x731ef489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37ef497, 98 },
    { 0x5cfef498, 33 },
    { 0x8d8ef499, 33 },
    { 0x544ef49a, 84 },
    { 0xca3ef49a, 33 },
    { 0xd8bef49c, 61 },
    { 0x927ef49a, 90 },
    { 0xf36ef49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13def4a1, 61 },
    { 0x9daef4a2, 90 },
    { 0, 0 },
    { 0x54fef4a4, 76 },
    { 0, 0 },
    { 0xc62ef4a6, 90 },
    { 0x98ef4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61eef4aa, 85 },
    { 0, 0 },
    { 0x552ef4ac, 66 },
    { 0x9aef4ad, 61 },
    { 0xae7ef4ad, 90 },
    { 0, 0 },
    { 0x940ef4b0, 90 },
    { 0x11ef4b1, 23 },
    { 0x278ef4b2, 33 },
    { 0, 0 },
    { 0x39def4b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabeef4b7, 33 },
    { 0x819ef4b8, 51 },
    { 0x2a8ef4b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5ef4bc, 95 },
    { 0xd8bef4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ef4c4, 33 },
    { 0x912ef4c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9ef4cb, 75 },
    { 0x896ef4cc, 23 },
    { 0xec8ef4cd, 91 },
    { 0x14fef4ce, 99 },
    { 0x3a8ef4cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1ef4d3, 98 },
    { 0x457ef4d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eef4d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ef4de, 33 },
    { 0xa39ef4de, 33 },
    { 0, 0 },
    { 0x59fef4e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618ef4e5, 55 },
    { 0, 0 },
    { 0x7b1ef4e7, 90 },
    { 0x220ef4e8, 61 },
    { 0x31aef4e8, 90 },
    { 0x7aeef4ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0def4f1, 61 },
    { 0xd3ef4f2, 90 },
    { 0x791ef4f3, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eef4f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef4fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ef4ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ef503, 33 },
    { 0x637ef503, 55 },
    { 0, 0 },
    { 0x85eef506, 100 },
    { 0x799ef507, 61 },
    { 0x813ef506, 76 },
    { 0, 0 },
    { 0xd37ef50a, 38 },
    { 0x8fbef50b, 98 },
    { 0xcf7ef50b, 23 },
    { 0, 0 },
    { 0x314ef50e, 61 },
    { 0x8fdef50e, 23 },
    { 0, 0 },
    { 0x371ef511, 23 },
    { 0, 0 },
    { 0xfdbef513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5ef518, 46 },
    { 0, 0 },
    { 0x25def51a, 23 },
    { 0, 0 },
    { 0x669ef51c, 46 },
    { 0x29bef51c, 33 },
    { 0xf2fef51e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1def522, 61 },
    { 0, 0 },
    { 0xa71ef524, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4ef527, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aef52f, 81 },
    { 0, 0 },
    { 0x965ef531, 23 },
    { 0x3c3ef531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bef536, 61 },
    { 0xde2ef537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ef53a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29ef53d, 43 },
    { 0, 0 },
    { 0xcb1ef53f, 91 },
    { 0x4cdef540, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bef543, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x350ef546, 98 },
    { 0x3e1ef546, 17 },
    { 0xdf2ef548, 43 },
    { 0, 0 },
    { 0x6a0ef54a, 23 },
    { 0xb45ef54b, 66 },
    { 0x482ef54c, 76 },
    { 0, 0 },
    { 0x10aef54e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78ef551, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7ef55a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfef55f, 23 },
    { 0, 0 },
    { 0x956ef561, 23 },
    { 0xb2aef562, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ef56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ef56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ef572, 95 },
    { 0xd46ef573, 84 },
    { 0xa3fef573, 23 },
    { 0, 0 },
    { 0x6a1ef576, 43 },
    { 0xcfeef577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9ef57d, 55 },
    { 0, 0 },
    { 0xbaaef57f, 33 },
    { 0, 0 },
    { 0xdceef581, 43 },
    { 0, 0 },
    { 0xe0cef583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cef588, 90 },
    { 0x70ef588, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef58e, 34 },
    { 0, 0 },
    { 0xea9ef590, 95 },
    { 0, 0 },
    { 0xde1ef592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06ef597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ef5a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedef5ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64ef5b0, 61 },
    { 0x906ef5b1, 33 },
    { 0x7b5ef5b0, 81 },
    { 0xa10ef5b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cef5b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2ef5bb, 90 },
    { 0x733ef5bb, 61 },
    { 0x434ef5bb, 90 },
    { 0, 0 },
    { 0xcedef5bf, 76 },
    { 0, 0 },
    { 0x3d4ef5c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eef5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aef5c9, 78 },
    { 0x683ef5c9, 46 },
    { 0x34cef5cb, 90 },
    { 0xdf8ef5cc, 95 },
    { 0x619ef5ca, 55 },
    { 0x7eaef5ca, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2ef5d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbef5d5, 90 },
    { 0, 0 },
    { 0xb45ef5d7, 46 },
    { 0x235ef5d7, 78 },
    { 0, 0 },
    { 0x7afef5da, 61 },
    { 0xb16ef5db, 33 },
    { 0x4dfef5da, 96 },
    { 0x37aef5dc, 43 },
    { 0xc1cef5da, 99 },
    { 0x24eef5dd, 101 },
    { 0x598ef5e0, 90 },
    { 0xe64ef5e1, 90 },
    { 0x679ef5e2, 61 },
    { 0x2ecef5e2, 74 },
    { 0, 0 },
    { 0xb8ef5e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efef5e8, 34 },
    { 0x826ef5e8, 33 },
    { 0xd1fef5ea, 76 },
    { 0, 0 },
    { 0x3e0ef5ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeef5f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95ef5f3, 34 },
    { 0x87bef5f3, 61 },
    { 0x901ef5f3, 23 },
    { 0xd10ef5f3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aef5fd, 23 },
    { 0x92ef5fd, 100 },
    { 0x693ef5ff, 23 },
    { 0x36aef5fd, 90 },
    { 0x7b8ef600, 90 },
    { 0xa9bef602, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x300ef605, 91 },
    { 0xd3ef606, 33 },
    { 0x249ef607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ef60a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89ef60f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60ef614, 55 },
    { 0, 0 },
    { 0xfd3ef616, 43 },
    { 0x706ef617, 55 },
    { 0xeaeef616, 98 },
    { 0x1e9ef618, 98 },
    { 0xbd2ef61a, 90 },
    { 0xc5cef61b, 66 },
    { 0, 0 },
    { 0x440ef61d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffef623, 33 },
    { 0xf00ef623, 17 },
    { 0, 0 },
    { 0xf5eef626, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x606ef629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ef62c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccef631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x803ef634, 23 },
    { 0, 0 },
    { 0x142ef636, 61 },
    { 0, 0 },
    { 0xe04ef638, 23 },
    { 0x685ef639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70ef63e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8ef648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bef64d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155ef652, 66 },
    { 0x3acef652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef656, 61 },
    { 0, 0 },
    { 0x6f6ef658, 75 },
    { 0x272ef659, 33 },
    { 0x50ef659, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eef65d, 55 },
    { 0xa51ef65e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x290ef661, 52 },
    { 0x864ef662, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ef665, 33 },
    { 0, 0 },
    { 0xbc4ef667, 76 },
    { 0x777ef668, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ef66b, 99 },
    { 0xcfcef66c, 91 },
    { 0x1a7ef66c, 61 },
    { 0x98eef66e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beef671, 59 },
    { 0x68cef672, 33 },
    { 0xa47ef673, 98 },
    { 0, 0 },
    { 0xefef675, 90 },
    { 0x3fef676, 101 },
    { 0, 0 },
    { 0x682ef678, 34 },
    { 0xf13ef679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ef68a, 76 },
    { 0x1e5ef68b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x309ef68e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8ef692, 90 },
    { 0xcacef692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7ef697, 99 },
    { 0x11fef697, 55 },
    { 0xf65ef698, 53 },
    { 0, 0 },
    { 0xa94ef69b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78def6a1, 34 },
    { 0xb3ef6a2, 87 },
    { 0xbd4ef6a2, 90 },
    { 0, 0 },
    { 0x63bef6a5, 69 },
    { 0x519ef6a6, 23 },
    { 0xc97ef6a6, 84 },
    { 0x63fef6a5, 95 },
    { 0x55bef6a5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cef6ad, 91 },
    { 0x19bef6ae, 33 },
    { 0, 0 },
    { 0xf58ef6b0, 98 },
    { 0, 0 },
    { 0xf93ef6b2, 33 },
    { 0xe86ef6b3, 95 },
    { 0xc50ef6b4, 33 },
    { 0, 0 },
    { 0x42aef6b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cef6c1, 61 },
    { 0x7d8ef6c1, 95 },
    { 0x367ef6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aef6c7, 25 },
    { 0, 0 },
    { 0x67ef6c9, 87 },
    { 0x2e5ef6c9, 23 },
    { 0x3f7ef6cb, 23 },
    { 0x999ef6cc, 76 },
    { 0x126ef6cc, 90 },
    { 0x37ef6ca, 95 },
    { 0x2eaef6cf, 23 },
    { 0x9e0ef6ce, 98 },
    { 0xbfeef6c9, 91 },
    { 0x62def6d0, 84 },
    { 0, 0 },
    { 0x77fef6d4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16ef6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ef6dc, 43 },
    { 0, 0 },
    { 0x1c6ef6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60ef6e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eef6e5, 90 },
    { 0xbc6ef6e6, 90 },
    { 0x8e5ef6e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8ef6ea, 98 },
    { 0x9a4ef6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8ef6ef, 33 },
    { 0x1d7ef6f0, 46 },
    { 0x65fef6f1, 93 },
    { 0xe8fef6f1, 98 },
    { 0xc27ef6f3, 95 },
    { 0xc43ef6f1, 94 },
    { 0x1bbef6f5, 43 },
    { 0x8ebef6f6, 76 },
    { 0, 0 },
    { 0x248ef6f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fef6fc, 33 },
    { 0x4dcef6fc, 76 },
    { 0, 0 },
    { 0x25fef6ff, 33 },
    { 0x80cef700, 46 },
    { 0xc15ef701, 76 },
    { 0xd9ef6ff, 99 },
    { 0xceef6ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383ef708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aef70b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeef70e, 23 },
    { 0, 0 },
    { 0x9b2ef710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44ef718, 95 },
    { 0xdf9ef719, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cef71e, 67 },
    { 0xab6ef71f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aef722, 93 },
    { 0x75eef723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bef726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29def729, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2ef72c, 91 },
    { 0x692ef72d, 93 },
    { 0xd1ef72e, 90 },
    { 0x7bfef72c, 95 },
    { 0x77cef730, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77ef73a, 61 },
    { 0x7baef73b, 88 },
    { 0x2d7ef73c, 55 },
    { 0x83bef73b, 61 },
    { 0xd49ef73a, 33 },
    { 0x6b8ef73a, 99 },
    { 0x57aef740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4ef744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963ef74b, 81 },
    { 0x542ef74c, 43 },
    { 0, 0 },
    { 0xf08ef74e, 52 },
    { 0x36eef74e, 91 },
    { 0xadeef74e, 84 },
    { 0x60fef751, 61 },
    { 0xae0ef751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ef762, 90 },
    { 0x2baef763, 93 },
    { 0x596ef763, 91 },
    { 0x925ef765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bef768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddef76b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aef76e, 90 },
    { 0xa6bef76f, 95 },
    { 0xd50ef770, 23 },
    { 0x83eef771, 43 },
    { 0xf95ef771, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x328ef775, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fef778, 61 },
    { 0xa34ef779, 34 },
    { 0x1d0ef77a, 61 },
    { 0xd18ef77b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ef77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5ef787, 43 },
    { 0xdb1ef787, 43 },
    { 0, 0 },
    { 0x3cbef78a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cef78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5ef790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eef794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07ef79c, 98 },
    { 0xd47ef79d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534ef7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabef7a6, 55 },
    { 0x8dbef7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bef7aa, 85 },
    { 0, 0 },
    { 0xebaef7ac, 95 },
    { 0, 0 },
    { 0xf84ef7ae, 33 },
    { 0x4f0ef7af, 33 },
    { 0xaa2ef7ae, 95 },
    { 0xaccef7ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869ef7b7, 95 },
    { 0, 0 },
    { 0xa1aef7b9, 85 },
    { 0, 0 },
    { 0x1c5ef7bb, 23 },
    { 0xd8ef7bc, 84 },
    { 0xdfcef7bb, 46 },
    { 0x892ef7be, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ef7c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x877ef7c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ef7c8, 100 },
    { 0x1cef7c9, 33 },
    { 0xd2def7ca, 90 },
    { 0x92aef7cb, 33 },
    { 0x3e4ef7c9, 61 },
    { 0x3e4ef7cd, 52 },
    { 0xc8aef7ce, 99 },
    { 0xb6cef7cb, 84 },
    { 0, 0 },
    { 0x378ef7d1, 74 },
    { 0x113ef7d2, 66 },
    { 0, 0 },
    { 0x237ef7d4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eef7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1ef7dc, 95 },
    { 0, 0 },
    { 0xcdcef7de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ef7e2, 98 },
    { 0x48def7e3, 99 },
    { 0x855ef7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ef7e9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652ef7ed, 91 },
    { 0xdeaef7ed, 66 },
    { 0xe78ef7ef, 98 },
    { 0xb50ef7ed, 33 },
    { 0, 0 },
    { 0x627ef7f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fef7f5, 90 },
    { 0xb00ef7f6, 33 },
    { 0x5fdef7f5, 33 },
    { 0xc7aef7f7, 95 },
    { 0, 0 },
    { 0x6efef7fa, 23 },
    { 0x79cef7fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ef801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aef806, 95 },
    { 0, 0 },
    { 0xceaef808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64ef80c, 90 },
    { 0, 0 },
    { 0x505ef80e, 97 },
    { 0xdfdef80e, 76 },
    { 0x8ecef80e, 46 },
    { 0, 0 },
    { 0x5bef812, 33 },
    { 0x417ef812, 95 },
    { 0x5b7ef814, 46 },
    { 0xa90ef814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef81c, 95 },
    { 0xb1def81c, 61 },
    { 0x67bef81e, 77 },
    { 0x4a1ef81d, 95 },
    { 0x843ef81c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ef823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fef826, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ef829, 90 },
    { 0x8beef82a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aef82e, 98 },
    { 0xc9ef82e, 23 },
    { 0, 0 },
    { 0x95ef831, 61 },
    { 0, 0 },
    { 0x3bdef833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ef836, 43 },
    { 0xac0ef837, 33 },
    { 0x5beef836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ef83e, 93 },
    { 0, 0 },
    { 0x2c1ef840, 46 },
    { 0x8cbef841, 67 },
    { 0xf2aef842, 76 },
    { 0x4afef843, 95 },
    { 0x291ef844, 98 },
    { 0x67cef845, 33 },
    { 0x511ef844, 55 },
    { 0x406ef845, 88 },
    { 0xbf6ef848, 90 },
    { 0xff7ef847, 23 },
    { 0x5b8ef842, 90 },
    { 0x9def84b, 23 },
    { 0xfc9ef842, 75 },
    { 0x5eaef845, 53 },
    { 0x23bef84e, 46 },
    { 0, 0 },
    { 0xee0ef850, 66 },
    { 0xa4ef851, 98 },
    { 0x351ef852, 90 },
    { 0xc95ef853, 34 },
    { 0x4adef854, 67 },
    { 0, 0 },
    { 0xdaaef856, 46 },
    { 0, 0 },
    { 0x2d4ef858, 61 },
    { 0xf7cef859, 61 },
    { 0x67aef85a, 99 },
    { 0x583ef859, 99 },
    { 0x9d8ef858, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ef85f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229ef865, 61 },
    { 0, 0 },
    { 0x61def867, 90 },
    { 0xe9eef867, 88 },
    { 0, 0 },
    { 0x8eeef86a, 61 },
    { 0x98fef86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2ef86e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332ef879, 100 },
    { 0x31ef87a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ef87f, 98 },
    { 0, 0 },
    { 0x467ef881, 84 },
    { 0, 0 },
    { 0x8c0ef883, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ef886, 33 },
    { 0, 0 },
    { 0x6d5ef888, 33 },
    { 0, 0 },
    { 0xc36ef88a, 87 },
    { 0x89def88a, 78 },
    { 0x95ef88c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cef891, 91 },
    { 0xb8ef892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7ef896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef89b, 46 },
    { 0, 0 },
    { 0xee4ef89d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497ef8a8, 92 },
    { 0, 0 },
    { 0x140ef8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ef8ad, 98 },
    { 0x7ffef8ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eef8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ef8b6, 93 },
    { 0, 0 },
    { 0x5beef8b8, 55 },
    { 0xcb5ef8b8, 77 },
    { 0x955ef8ba, 34 },
    { 0xb1aef8bb, 98 },
    { 0xa01ef8bb, 33 },
    { 0, 0 },
    { 0x1adef8be, 90 },
    { 0x44eef8be, 65 },
    { 0xe63ef8c0, 43 },
    { 0, 0 },
    { 0xfd7ef8c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ef8cc, 61 },
    { 0xf9cef8cd, 93 },
    { 0x97eef8ce, 90 },
    { 0x7fcef8ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x760ef8d2, 34 },
    { 0, 0 },
    { 0x2fbef8d4, 85 },
    { 0x5baef8d4, 34 },
    { 0xd84ef8d6, 33 },
    { 0xd8fef8d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ef8dc, 85 },
    { 0, 0 },
    { 0x79def8de, 95 },
    { 0, 0 },
    { 0xedcef8e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ef8e4, 23 },
    { 0xa19ef8e5, 34 },
    { 0x771ef8e6, 99 },
    { 0x3e5ef8e7, 61 },
    { 0x909ef8e4, 99 },
    { 0x9a2ef8e7, 93 },
    { 0x76aef8e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef8ed, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ef8f0, 98 },
    { 0x919ef8f1, 100 },
    { 0xf9aef8f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5ef8f5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eef8fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eef8ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcef902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894ef90a, 98 },
    { 0xb3eef90b, 23 },
    { 0, 0 },
    { 0x5eeef90d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8ef911, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72ef917, 61 },
    { 0, 0 },
    { 0x12bef919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ef91d, 33 },
    { 0x82def91e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0ef923, 23 },
    { 0x406ef924, 46 },
    { 0xe29ef925, 90 },
    { 0x292ef925, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdef933, 80 },
    { 0x7fdef934, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aef93b, 67 },
    { 0xc1def93b, 95 },
    { 0, 0 },
    { 0xfbbef93e, 90 },
    { 0x6cdef93f, 61 },
    { 0x9c8ef93f, 76 },
    { 0x21cef941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cef945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ef94e, 99 },
    { 0xf17ef94f, 33 },
    { 0xb49ef94f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70ef954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fef959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41ef95c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbef960, 93 },
    { 0xcecef961, 33 },
    { 0x12bef961, 95 },
    { 0x814ef961, 46 },
    { 0x5d3ef964, 34 },
    { 0x222ef965, 76 },
    { 0x952ef966, 86 },
    { 0, 0 },
    { 0x5abef968, 95 },
    { 0xe45ef968, 91 },
    { 0, 0 },
    { 0x6a0ef96b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5ef96f, 76 },
    { 0x899ef970, 90 },
    { 0x6f7ef971, 23 },
    { 0x46bef972, 66 },
    { 0, 0 },
    { 0x4a9ef974, 68 },
    { 0xfb4ef975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfef97a, 43 },
    { 0, 0 },
    { 0x804ef97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ef980, 65 },
    { 0x943ef981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0ef989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bef98d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ef990, 95 },
    { 0x282ef990, 85 },
    { 0x64cef991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef995, 73 },
    { 0x1e1ef996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bef999, 33 },
    { 0xe1bef999, 95 },
    { 0xdc5ef99b, 90 },
    { 0x18bef99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbef9a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1def9a4, 61 },
    { 0x693ef9a5, 98 },
    { 0x403ef9a4, 78 },
    { 0x951ef9a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef9b0, 61 },
    { 0x4ddef9b0, 95 },
    { 0x680ef9b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1ef9b9, 78 },
    { 0x5feef9ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25ef9c1, 61 },
    { 0, 0 },
    { 0x53aef9c3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaef9c7, 90 },
    { 0x843ef9c7, 93 },
    { 0xb13ef9c8, 33 },
    { 0, 0 },
    { 0xcf5ef9cb, 98 },
    { 0, 0 },
    { 0xb9def9cd, 52 },
    { 0xd63ef9ce, 61 },
    { 0xf25ef9cd, 66 },
    { 0x23eef9d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bef9d3, 98 },
    { 0xff5ef9d4, 46 },
    { 0xf59ef9d3, 43 },
    { 0xfdaef9d6, 97 },
    { 0x339ef9d7, 40 },
    { 0xcd8ef9d8, 90 },
    { 0xbf2ef9d9, 91 },
    { 0, 0 },
    { 0xcbaef9db, 68 },
    { 0xb9ef9dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ef9e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3ef9ee, 90 },
    { 0xed8ef9ef, 33 },
    { 0, 0 },
    { 0xd24ef9f1, 90 },
    { 0x185ef9f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2ef9f5, 33 },
    { 0x5c2ef9f6, 34 },
    { 0x771ef9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ef9fc, 33 },
    { 0x8bbef9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77efa00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71efa04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9efa07, 77 },
    { 0, 0 },
    { 0x10fefa09, 23 },
    { 0x38befa0a, 90 },
    { 0, 0 },
    { 0xe6cefa0c, 23 },
    { 0xd47efa0c, 90 },
    { 0x5cbefa0e, 23 },
    { 0xc77efa0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52efa13, 33 },
    { 0, 0 },
    { 0x5a4efa15, 33 },
    { 0x505efa15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79efa1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x581efa1f, 66 },
    { 0xb4befa20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fefa27, 76 },
    { 0x278efa27, 95 },
    { 0, 0 },
    { 0x5c7efa2a, 90 },
    { 0x2e9efa2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3efa36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x872efa39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652efa46, 23 },
    { 0x946efa47, 93 },
    { 0, 0 },
    { 0x7c9efa49, 90 },
    { 0, 0 },
    { 0xaf8efa4b, 90 },
    { 0xaf7efa4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449efa51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731efa57, 78 },
    { 0, 0 },
    { 0xcdbefa59, 20 },
    { 0x1a6efa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x376efa5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eefa60, 46 },
    { 0x9eeefa60, 43 },
    { 0x5eeefa60, 55 },
    { 0xdb7efa63, 36 },
    { 0, 0 },
    { 0x5bcefa65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7efa73, 90 },
    { 0xdd9efa74, 78 },
    { 0x5a1efa75, 33 },
    { 0x612efa75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbefa7b, 33 },
    { 0x44befa7c, 23 },
    { 0x982efa7d, 52 },
    { 0x158efa7e, 84 },
    { 0xf3befa7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3efa82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4efa85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fefa88, 78 },
    { 0x884efa89, 33 },
    { 0xd1fefa89, 95 },
    { 0x314efa8b, 95 },
    { 0xd79efa8c, 98 },
    { 0x5a5efa8c, 34 },
    { 0x9e2efa8e, 100 },
    { 0x5c1efa8e, 43 },
    { 0x296efa8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eefa97, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98befa9c, 80 },
    { 0xe35efa9d, 61 },
    { 0x9c9efa9e, 55 },
    { 0, 0 },
    { 0x3aefaa0, 61 },
    { 0x1d6efaa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270efaa5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acefaa9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1efaaf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1efab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148efabe, 76 },
    { 0xcbcefabf, 78 },
    { 0xd82efabf, 95 },
    { 0, 0 },
    { 0x939efac2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839efac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81defad1, 33 },
    { 0x8ebefad1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x843efad5, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3efada, 98 },
    { 0, 0 },
    { 0xb8eefadc, 52 },
    { 0, 0 },
    { 0xa62efade, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeefae3, 52 },
    { 0x17fefae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7efae8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8caefaf0, 95 },
    { 0xd1befaf1, 46 },
    { 0, 0 },
    { 0x4d5efaf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacefaf6, 76 },
    { 0, 0 },
    { 0x5d9efaf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508efafc, 98 },
    { 0, 0 },
    { 0xb9befafe, 17 },
    { 0x795efaff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9efb03, 93 },
    { 0, 0 },
    { 0xa49efb05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9efb0b, 76 },
    { 0x69befb0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdefb10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913efb19, 90 },
    { 0, 0 },
    { 0xbe3efb1b, 75 },
    { 0, 0 },
    { 0x5fdefb1d, 61 },
    { 0x667efb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fefb27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3efb2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fefb30, 90 },
    { 0x780efb31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cefb38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8beefb3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747efb43, 90 },
    { 0x5acefb43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9efb48, 34 },
    { 0x6bfefb49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cefb4c, 98 },
    { 0, 0 },
    { 0xed7efb4e, 61 },
    { 0x224efb4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88efb56, 64 },
    { 0xbf0efb56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aefb5b, 76 },
    { 0x252efb5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fefb5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5befb62, 95 },
    { 0, 0 },
    { 0xea8efb64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55aefb67, 81 },
    { 0x310efb67, 95 },
    { 0x902efb67, 44 },
    { 0, 0 },
    { 0x63fefb6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdefb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deefb74, 33 },
    { 0x5e4efb74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367efb79, 90 },
    { 0x68efb79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fefb7f, 80 },
    { 0x6beefb80, 55 },
    { 0x1a4efb81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811efb86, 33 },
    { 0x1edefb87, 55 },
    { 0x816efb88, 33 },
    { 0x241efb88, 34 },
    { 0x6a1efb8a, 43 },
    { 0x1b4efb8b, 55 },
    { 0, 0 },
    { 0xa77efb8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0efb90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0efb94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x434efb97, 88 },
    { 0xd76efb98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75efb9c, 61 },
    { 0x69efb9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05efba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x798efba3, 88 },
    { 0, 0 },
    { 0x668efba5, 95 },
    { 0, 0 },
    { 0x278efba7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12defbab, 52 },
    { 0xdb0efbab, 83 },
    { 0, 0 },
    { 0x63aefbae, 76 },
    { 0x77efbaf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68efbb4, 61 },
    { 0, 0 },
    { 0x5d4efbb6, 66 },
    { 0, 0 },
    { 0x1d8efbb8, 98 },
    { 0xc6aefbb8, 90 },
    { 0, 0 },
    { 0x572efbbb, 61 },
    { 0x1e0efbbc, 23 },
    { 0xef9efbbc, 78 },
    { 0xdd9efbbe, 98 },
    { 0x770efbbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaefbc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x417efbc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1efbca, 85 },
    { 0, 0 },
    { 0x166efbcc, 93 },
    { 0x4a5efbcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0efbd0, 93 },
    { 0xe90efbd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcefbd5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19efbdc, 84 },
    { 0, 0 },
    { 0x3e8efbde, 99 },
    { 0x33aefbdf, 98 },
    { 0, 0 },
    { 0xef8efbe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eefbef, 98 },
    { 0, 0 },
    { 0x6b2efbf1, 33 },
    { 0x41defbf2, 75 },
    { 0xdaaefbf3, 95 },
    { 0, 0 },
    { 0xe7cefbf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911efc00, 72 },
    { 0, 0 },
    { 0x747efc02, 43 },
    { 0x231efc02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635efc10, 23 },
    { 0, 0 },
    { 0xcbefc12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3befc16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18efc21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6efc25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4efc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cefc32, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cefc35, 76 },
    { 0, 0 },
    { 0x45cefc37, 46 },
    { 0, 0 },
    { 0xcb4efc39, 61 },
    { 0x366efc3a, 76 },
    { 0x66eefc3a, 23 },
    { 0x75aefc3c, 17 },
    { 0x356efc3c, 77 },
    { 0x721efc3d, 98 },
    { 0x8dbefc3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6efc43, 90 },
    { 0, 0 },
    { 0xbb4efc45, 33 },
    { 0x7c7efc45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67efc49, 43 },
    { 0, 0 },
    { 0x3eefc4b, 98 },
    { 0, 0 },
    { 0x469efc4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91efc52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x365efc55, 55 },
    { 0x89cefc55, 90 },
    { 0x466efc57, 76 },
    { 0, 0 },
    { 0x6b4efc59, 93 },
    { 0, 0 },
    { 0xe9efc5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feefc5f, 33 },
    { 0x79aefc60, 92 },
    { 0xbd8efc61, 95 },
    { 0x8b3efc62, 61 },
    { 0x38efc62, 87 },
    { 0xd21efc64, 61 },
    { 0x574efc62, 33 },
    { 0xfc9efc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x185efc69, 90 },
    { 0x438efc6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561efc6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960efc74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79efc77, 95 },
    { 0, 0 },
    { 0xdb0efc79, 43 },
    { 0x1b6efc7a, 46 },
    { 0x6b6efc7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9efc82, 61 },
    { 0xf89efc83, 66 },
    { 0xb00efc84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886efc88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaefc8e, 61 },
    { 0xef0efc8f, 88 },
    { 0, 0 },
    { 0x8fbefc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cefc94, 95 },
    { 0, 0 },
    { 0x2a7efc96, 66 },
    { 0, 0 },
    { 0x8bfefc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebefca0, 61 },
    { 0, 0 },
    { 0x276efca2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x995efca5, 43 },
    { 0, 0 },
    { 0x2dfefca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955efcad, 61 },
    { 0, 0 },
    { 0x326efcaf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28efcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aefcb8, 90 },
    { 0xd1aefcb8, 55 },
    { 0, 0 },
    { 0x6e9efcbb, 33 },
    { 0xd89efcbc, 95 },
    { 0x89eefcbd, 98 },
    { 0x812efcbd, 85 },
    { 0, 0 },
    { 0xc77efcc0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619efcc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faefcd1, 64 },
    { 0, 0 },
    { 0xb2efcd3, 98 },
    { 0x98befcd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484efcd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6efcdc, 99 },
    { 0xef6efcdc, 34 },
    { 0, 0 },
    { 0x99defcdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26efce5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289efce9, 33 },
    { 0xcc5efce9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eefced, 98 },
    { 0x46efced, 80 },
    { 0xa21efcef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caefcfa, 66 },
    { 0, 0 },
    { 0xe32efcfc, 61 },
    { 0xcf0efcfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294efd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4efd04, 91 },
    { 0xef6efd05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20efd0b, 34 },
    { 0x7dcefd0b, 53 },
    { 0x110efd0b, 84 },
    { 0, 0 },
    { 0x381efd0f, 33 },
    { 0, 0 },
    { 0xe4defd11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898efd19, 46 },
    { 0, 0 },
    { 0xea5efd1b, 95 },
    { 0x2cbefd1b, 33 },
    { 0x933efd1d, 77 },
    { 0x7c7efd1d, 85 },
    { 0xf71efd1e, 23 },
    { 0, 0 },
    { 0xd3aefd21, 66 },
    { 0xa5fefd21, 93 },
    { 0x12efd23, 61 },
    { 0xab7efd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219efd28, 90 },
    { 0x47fefd28, 90 },
    { 0xe97efd28, 90 },
    { 0x7f2efd2a, 61 },
    { 0x341efd2c, 33 },
    { 0x873efd2c, 61 },
    { 0xa91efd2b, 64 },
    { 0xeaaefd28, 95 },
    { 0, 0 },
    { 0x1ffefd31, 40 },
    { 0xe61efd32, 65 },
    { 0x105efd33, 61 },
    { 0x476efd34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aefd38, 33 },
    { 0x2deefd39, 55 },
    { 0x53befd3a, 61 },
    { 0x556efd3a, 43 },
    { 0x46efd3c, 61 },
    { 0, 0 },
    { 0xdfcefd3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28efd43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x910efd46, 83 },
    { 0x3e8efd47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x424efd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cefd4e, 90 },
    { 0xbc9efd4f, 48 },
    { 0xe54efd4f, 17 },
    { 0x6ebefd50, 98 },
    { 0xc2efd52, 61 },
    { 0x84cefd53, 41 },
    { 0xaf0efd54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2efd57, 66 },
    { 0xfc1efd58, 55 },
    { 0, 0 },
    { 0xabfefd5a, 55 },
    { 0xcb9efd5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9efd5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cefd62, 55 },
    { 0x42fefd63, 43 },
    { 0x639efd63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52efd68, 77 },
    { 0xb87efd68, 95 },
    { 0, 0 },
    { 0x518efd6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fefd6e, 33 },
    { 0, 0 },
    { 0xd46efd70, 33 },
    { 0, 0 },
    { 0x57cefd72, 90 },
    { 0, 0 },
    { 0x398efd74, 33 },
    { 0, 0 },
    { 0xcdbefd76, 98 },
    { 0xa2fefd76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5efd7b, 61 },
    { 0, 0 },
    { 0x2b9efd7d, 98 },
    { 0, 0 },
    { 0xe70efd7f, 95 },
    { 0, 0 },
    { 0x91defd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2efd88, 95 },
    { 0, 0 },
    { 0xb6befd8a, 90 },
    { 0xa8befd8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81efd91, 100 },
    { 0x97fefd92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efd9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370efda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88efda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceefda7, 78 },
    { 0, 0 },
    { 0x1caefda9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceefdaf, 55 },
    { 0x14fefdb0, 46 },
    { 0x3fbefdb1, 78 },
    { 0, 0 },
    { 0x29befdb3, 84 },
    { 0xd7eefdb4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x855efdb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33befdba, 23 },
    { 0xf0efdbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8efdbf, 95 },
    { 0x265efdc0, 33 },
    { 0x274efdbf, 90 },
    { 0, 0 },
    { 0x740efdc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfefdc6, 43 },
    { 0xcf5efdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aefdcd, 72 },
    { 0x61aefdce, 61 },
    { 0xe4defdcd, 95 },
    { 0xc35efdd0, 95 },
    { 0x635efdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cefdd8, 98 },
    { 0x9a0efdd8, 61 },
    { 0x5e6efdda, 46 },
    { 0x50befddb, 46 },
    { 0x4b2efdda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aefddf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78efde3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116efdf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18efdf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7defdfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5efe01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182efe0a, 38 },
    { 0xc49efe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9efe0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cefe11, 96 },
    { 0xed1efe12, 88 },
    { 0, 0 },
    { 0xcebefe14, 76 },
    { 0x440efe15, 90 },
    { 0x706efe16, 23 },
    { 0x485efe17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eefe1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77efe1e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8efe24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cefe2b, 61 },
    { 0xf8cefe2c, 95 },
    { 0, 0 },
    { 0x262efe2e, 23 },
    { 0x23fefe2f, 33 },
    { 0xb2befe30, 61 },
    { 0xea9efe30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2efe34, 46 },
    { 0x8f9efe35, 17 },
    { 0x5ecefe36, 23 },
    { 0x7b3efe37, 93 },
    { 0x324efe38, 88 },
    { 0, 0 },
    { 0x5ddefe3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eefe4e, 78 },
    { 0x6adefe4f, 33 },
    { 0xd60efe50, 97 },
    { 0x20aefe51, 76 },
    { 0, 0 },
    { 0x81befe53, 33 },
    { 0x832efe54, 98 },
    { 0x741efe54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890efe58, 98 },
    { 0, 0 },
    { 0x7d2efe5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205efe5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2efe62, 77 },
    { 0x43cefe62, 23 },
    { 0xdd9efe64, 90 },
    { 0x4b0efe65, 77 },
    { 0xf94efe66, 93 },
    { 0xeccefe67, 43 },
    { 0x422efe65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08efe6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efe6e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x934efe71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785efe76, 85 },
    { 0, 0 },
    { 0xcefe78, 98 },
    { 0, 0 },
    { 0xb36efe7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x709efe7d, 33 },
    { 0, 0 },
    { 0x689efe7f, 34 },
    { 0, 0 },
    { 0x929efe81, 93 },
    { 0x514efe81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4efe86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135efe8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5efe96, 33 },
    { 0xdceefe96, 95 },
    { 0, 0 },
    { 0x41cefe99, 90 },
    { 0, 0 },
    { 0x7a2efe9b, 61 },
    { 0, 0 },
    { 0xfc0efe9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cefea1, 90 },
    { 0, 0 },
    { 0xbbcefea3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684efea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9efeab, 89 },
    { 0xb69efeac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8efeaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56efeb2, 67 },
    { 0xa3befeb2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70efeb8, 90 },
    { 0x100efeb9, 52 },
    { 0x5d2efeba, 85 },
    { 0xaa0efeb9, 98 },
    { 0xc69efebc, 61 },
    { 0, 0 },
    { 0x8feefebe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89efec1, 23 },
    { 0, 0 },
    { 0x560efec3, 33 },
    { 0xa7befec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adefec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eefecb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fefecf, 87 },
    { 0x86efed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2efed3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x405efed6, 85 },
    { 0xe91efed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6efeda, 33 },
    { 0xd52efedb, 23 },
    { 0xf0befedb, 61 },
    { 0x906efedd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9efee0, 91 },
    { 0x306efee0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64efee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62efeea, 33 },
    { 0x4a2efeeb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aefef2, 23 },
    { 0x613efef3, 78 },
    { 0x201efef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8befef8, 23 },
    { 0x948efef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02efefc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33eff00, 23 },
    { 0xc3aeff01, 95 },
    { 0, 0 },
    { 0x7d3eff03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ceff06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eeff09, 46 },
    { 0xb77eff0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506eff0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5eff14, 76 },
    { 0x117eff14, 90 },
    { 0, 0 },
    { 0xd5eeff17, 98 },
    { 0x1c2eff18, 95 },
    { 0xfdceff19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976eff1d, 99 },
    { 0x313eff1e, 78 },
    { 0xb37eff1d, 46 },
    { 0x48eff20, 67 },
    { 0xd23eff1d, 61 },
    { 0x9abeff22, 55 },
    { 0x19eff23, 90 },
    { 0xb9beff23, 51 },
    { 0, 0 },
    { 0x217eff26, 87 },
    { 0, 0 },
    { 0xd46eff28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7deff30, 78 },
    { 0x38feff31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4eff34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x315eff37, 23 },
    { 0x50ceff38, 55 },
    { 0xa12eff37, 95 },
    { 0x124eff3a, 98 },
    { 0xb0ceff3a, 61 },
    { 0xc56eff3c, 90 },
    { 0xd51eff3b, 75 },
    { 0x8fceff3e, 61 },
    { 0, 0 },
    { 0x7a6eff40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8eff44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2eff47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x28feff4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eff4d, 33 },
    { 0xf9eff4e, 66 },
    { 0, 0 },
    { 0xa64eff50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5eff53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93eff57, 65 },
    { 0, 0 },
    { 0xac1eff59, 46 },
    { 0xd21eff5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ceff5e, 75 },
    { 0x9c1eff5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2eff64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aeff6a, 61 },
    { 0, 0 },
    { 0x18feff6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ceff71, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265eff75, 93 },
    { 0, 0 },
    { 0x896eff77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53beff7f, 33 },
    { 0xdb8eff7f, 76 },
    { 0x3caeff7f, 40 },
    { 0, 0 },
    { 0xbc3eff83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87deff8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70deff90, 83 },
    { 0x9adeff91, 61 },
    { 0, 0 },
    { 0x114eff93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caeff97, 11 },
    { 0xf26eff97, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfeff9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4effa2, 43 },
    { 0, 0 },
    { 0xad0effa4, 40 },
    { 0, 0 },
    { 0x8dfeffa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9effa9, 95 },
    { 0x648effa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348effae, 77 },
    { 0x8bbeffaf, 91 },
    { 0, 0 },
    { 0xe1aeffb1, 95 },
    { 0x17aeffb2, 34 },
    { 0x3e2effb3, 90 },
    { 0x21ceffb2, 33 },
    { 0xc99effb5, 90 },
    { 0xf5beffb2, 98 },
    { 0xe8effb7, 90 },
    { 0x32aeffb7, 90 },
    { 0x848effb9, 33 },
    { 0x9a4effba, 84 },
    { 0xafbeffba, 33 },
    { 0x327effb4, 90 },
    { 0xf07effb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973effc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2feffc7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896effcb, 95 },
    { 0x2f4effcc, 46 },
    { 0xcb6effcc, 61 },
    { 0xca0effce, 40 },
    { 0x812effcf, 23 },
    { 0, 0 },
    { 0xdaaeffd1, 98 },
    { 0x6b1effd1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1effd9, 68 },
    { 0x8afeffda, 23 },
    { 0xc4aeffdb, 43 },
    { 0x11effdc, 95 },
    { 0x666effdb, 95 },
    { 0xf8deffde, 80 },
    { 0x75ceffdf, 52 },
    { 0xf90effdc, 98 },
    { 0x7a2effe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6effe4, 91 },
    { 0x3a5effe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27effe8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8efff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aefff6, 66 },
    { 0x5aaefff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x187efffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0effff, 42 },
    { 0x11aeffff, 36 },
    { 0x9ff0000, 37 },
    { 0, 0 },
    { 0xeaf0003, 66 },
    { 0, 0 },
    { 0x945f0005, 66 },
    { 0x2acf0005, 99 },
    { 0, 0 },
    { 0x2dcf0008, 93 },
    { 0, 0 },
    { 0x932f000a, 23 },
    { 0xd37f000a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ef000f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ff0017, 90 },
    { 0xc50f0018, 68 },
    { 0xfabf0019, 90 },
    { 0x3daf001a, 48 },
    { 0, 0 },
    { 0xa0ef001c, 55 },
    { 0x277f001c, 76 },
    { 0xeb4f001e, 98 },
    { 0x5aaf001f, 46 },
    { 0x55df0020, 87 },
    { 0x2c1f0021, 76 },
    { 0xeaef001f, 76 },
    { 0x372f0023, 91 },
    { 0xa2df0022, 95 },
    { 0x8aef0025, 33 },
    { 0x836f0026, 95 },
    { 0x116f0027, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9f002b, 61 },
    { 0xc15f002c, 61 },
    { 0xa33f002d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebf0033, 34 },
    { 0x696f0034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdf0039, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6f0045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df0049, 34 },
    { 0, 0 },
    { 0x9acf004b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f0051, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f0054, 61 },
    { 0x21df0054, 43 },
    { 0x343f0055, 55 },
    { 0, 0 },
    { 0xca2f0058, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f005b, 78 },
    { 0x54af005b, 40 },
    { 0xfa8f005d, 23 },
    { 0x37cf005e, 90 },
    { 0x49df005e, 87 },
    { 0x44ef005c, 34 },
    { 0xe99f005c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87f0064, 23 },
    { 0xd7cf0064, 90 },
    { 0x228f0066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff0069, 33 },
    { 0x94ff006a, 34 },
    { 0xc39f006a, 46 },
    { 0x251f006b, 55 },
    { 0xba7f006a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65f0070, 75 },
    { 0, 0 },
    { 0x245f0072, 97 },
    { 0xee6f0073, 90 },
    { 0xf02f0074, 84 },
    { 0x441f0075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x958f0078, 36 },
    { 0x6edf0079, 77 },
    { 0x3fff0079, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37df0084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f008b, 90 },
    { 0x3a6f008b, 61 },
    { 0, 0 },
    { 0x63bf008e, 23 },
    { 0, 0 },
    { 0x4daf0090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361f0093, 90 },
    { 0x586f0094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef0098, 61 },
    { 0, 0 },
    { 0x587f009a, 34 },
    { 0, 0 },
    { 0xb1cf009c, 90 },
    { 0x9e6f009c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709f00a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f00a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caf00a9, 40 },
    { 0x91df00aa, 76 },
    { 0xc38f00ab, 78 },
    { 0xebbf00ac, 67 },
    { 0, 0 },
    { 0x764f00ae, 87 },
    { 0xf4cf00af, 46 },
    { 0x976f00b0, 100 },
    { 0xa5ef00b1, 84 },
    { 0x409f00b2, 98 },
    { 0x10ff00ae, 61 },
    { 0xc58f00b0, 17 },
    { 0x639f00b5, 33 },
    { 0x84ff00b6, 52 },
    { 0xba6f00b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4af00ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ff00c1, 52 },
    { 0xaaef00c2, 100 },
    { 0x23ef00c3, 23 },
    { 0x777f00c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f00c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f00cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812f00d4, 23 },
    { 0, 0 },
    { 0x70bf00d6, 95 },
    { 0xd8f00d6, 91 },
    { 0, 0 },
    { 0x715f00d9, 91 },
    { 0, 0 },
    { 0x30df00db, 90 },
    { 0xe5f00db, 91 },
    { 0xb98f00dd, 90 },
    { 0, 0 },
    { 0x850f00df, 33 },
    { 0, 0 },
    { 0x9e9f00e1, 95 },
    { 0, 0 },
    { 0xad2f00e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5f00e6, 90 },
    { 0, 0 },
    { 0x69cf00e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cf00eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff00f3, 93 },
    { 0x438f00f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf00fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73df0100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f010a, 90 },
    { 0x386f010b, 69 },
    { 0xb59f010c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225f0112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8f011a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ef011d, 33 },
    { 0, 0 },
    { 0x305f011f, 66 },
    { 0, 0 },
    { 0x1d0f0121, 88 },
    { 0, 0 },
    { 0xdcaf0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x664f0126, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21f012a, 99 },
    { 0xc88f012b, 61 },
    { 0x500f012a, 33 },
    { 0xa9bf012b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779f0131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f0136, 33 },
    { 0xe30f0137, 33 },
    { 0xd2f0138, 84 },
    { 0xf7cf0139, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f013c, 65 },
    { 0xf0ff013d, 93 },
    { 0x55df013d, 95 },
    { 0, 0 },
    { 0xff3f0140, 66 },
    { 0x42bf0140, 90 },
    { 0x3aaf0141, 76 },
    { 0x9f8f0140, 77 },
    { 0xe91f0140, 90 },
    { 0xb3ff0145, 55 },
    { 0xfaf0142, 43 },
    { 0x5ef0147, 91 },
    { 0xacbf0147, 95 },
    { 0xd38f0149, 77 },
    { 0xa24f014a, 90 },
    { 0xbaef0143, 43 },
    { 0x404f0145, 61 },
    { 0x81bf0145, 90 },
    { 0xd31f0148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6f0155, 33 },
    { 0, 0 },
    { 0xe73f0157, 34 },
    { 0, 0 },
    { 0x124f0159, 53 },
    { 0, 0 },
    { 0xf64f015b, 90 },
    { 0xb42f015b, 84 },
    { 0xa0ff015d, 95 },
    { 0xf46f015e, 88 },
    { 0, 0 },
    { 0xccbf0160, 99 },
    { 0, 0 },
    { 0xc2f0162, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef0169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x336f016c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff0173, 23 },
    { 0xc4bf0173, 95 },
    { 0xa82f0175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93f0179, 33 },
    { 0x815f017a, 17 },
    { 0x7edf017a, 34 },
    { 0, 0 },
    { 0x570f017d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158f0184, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f0188, 91 },
    { 0, 0 },
    { 0xd9f018a, 100 },
    { 0x5f5f018a, 34 },
    { 0, 0 },
    { 0x645f018d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f0192, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58f0197, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ff019b, 95 },
    { 0, 0 },
    { 0x7eaf019d, 95 },
    { 0xe26f019e, 77 },
    { 0, 0 },
    { 0x820f01a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f01ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x847f01ae, 33 },
    { 0xc49f01ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff01b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5f01ba, 23 },
    { 0, 0 },
    { 0x9e3f01bc, 95 },
    { 0x2f2f01bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf01cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df01cf, 98 },
    { 0x706f01d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf01d7, 33 },
    { 0xff4f01d8, 43 },
    { 0x56af01d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf01dc, 46 },
    { 0, 0 },
    { 0xe2cf01de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacf01e1, 68 },
    { 0xce5f01e2, 95 },
    { 0xd9df01e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f01e6, 90 },
    { 0x5ddf01e6, 55 },
    { 0xc3f01e7, 23 },
    { 0, 0 },
    { 0x5a5f01ea, 33 },
    { 0x558f01eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f01ee, 84 },
    { 0, 0 },
    { 0x882f01f0, 46 },
    { 0x6aef01f0, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df01f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdf01f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef0200, 93 },
    { 0, 0 },
    { 0x7b5f0202, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bf0209, 81 },
    { 0xc3af020a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x246f020d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f021d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f0222, 61 },
    { 0xd5af0223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2df0226, 93 },
    { 0, 0 },
    { 0x29af0228, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf022b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f0232, 33 },
    { 0xcaef0233, 76 },
    { 0x81af0234, 61 },
    { 0x39ef0235, 33 },
    { 0xdbbf0233, 93 },
    { 0xa52f0236, 90 },
    { 0x2e2f0232, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cf023d, 61 },
    { 0x548f023e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf0241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f0245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef0249, 90 },
    { 0xf4bf024a, 78 },
    { 0x5f5f024a, 98 },
    { 0x4e4f024c, 83 },
    { 0, 0 },
    { 0x86bf024e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399f0253, 61 },
    { 0, 0 },
    { 0x897f0255, 90 },
    { 0, 0 },
    { 0x7d5f0257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x150f025a, 46 },
    { 0x83ef025a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430f025f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7f0263, 33 },
    { 0xa8ff0263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cf0267, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15f026b, 93 },
    { 0x3faf026b, 98 },
    { 0x688f026d, 76 },
    { 0x73ff026c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f0272, 33 },
    { 0x92bf0272, 55 },
    { 0x4a4f0274, 93 },
    { 0x583f0275, 95 },
    { 0x8f7f0276, 99 },
    { 0xb0ff0277, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9f027a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f027d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fff0285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f028c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13f0291, 87 },
    { 0, 0 },
    { 0xa89f0293, 90 },
    { 0x6ebf0294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f0298, 33 },
    { 0x38bf0298, 33 },
    { 0x6d2f029a, 98 },
    { 0xb8bf0299, 33 },
    { 0x4ebf029b, 43 },
    { 0x837f0299, 91 },
    { 0, 0 },
    { 0xf88f029f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf02a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecf02a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ef02af, 99 },
    { 0x895f02b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccf02b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cf02b7, 88 },
    { 0xc29f02b7, 76 },
    { 0, 0 },
    { 0x638f02ba, 98 },
    { 0x707f02bb, 90 },
    { 0, 0 },
    { 0x155f02bd, 76 },
    { 0, 0 },
    { 0xe71f02bf, 90 },
    { 0, 0 },
    { 0x1d8f02c1, 33 },
    { 0x68ef02c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff02c6, 55 },
    { 0x56ff02c7, 34 },
    { 0, 0 },
    { 0x996f02c9, 95 },
    { 0x5b0f02c9, 99 },
    { 0x7bef02cb, 93 },
    { 0x65df02cc, 90 },
    { 0x580f02cc, 95 },
    { 0xab8f02c9, 78 },
    { 0x988f02cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f02d2, 95 },
    { 0xf4af02d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9f02dc, 61 },
    { 0x9aef02dd, 75 },
    { 0xbcef02de, 98 },
    { 0x5ff02df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f02e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f02e7, 91 },
    { 0x8eef02e8, 95 },
    { 0x3b8f02e8, 90 },
    { 0, 0 },
    { 0x34df02eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8f02ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbf02f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f02fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372f0300, 55 },
    { 0xf40f0300, 23 },
    { 0, 0 },
    { 0xca2f0303, 52 },
    { 0x917f0303, 23 },
    { 0, 0 },
    { 0x130f0306, 99 },
    { 0x940f0307, 33 },
    { 0xb1ef0308, 98 },
    { 0x5d6f0309, 23 },
    { 0, 0 },
    { 0x5f8f030b, 95 },
    { 0xae2f030b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef030f, 66 },
    { 0, 0 },
    { 0xb4af0311, 43 },
    { 0x597f0312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0316, 90 },
    { 0, 0 },
    { 0xdb8f0318, 34 },
    { 0x10af0319, 33 },
    { 0xcf1f0319, 23 },
    { 0x416f031b, 55 },
    { 0x1b9f031b, 34 },
    { 0x42bf0318, 33 },
    { 0x809f031e, 3 },
    { 0x6b9f031e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133f0324, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf0327, 61 },
    { 0, 0 },
    { 0x33af0329, 90 },
    { 0xb9af032a, 95 },
    { 0x602f032a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2f032f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf0335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f0339, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ff033d, 88 },
    { 0, 0 },
    { 0x570f033f, 95 },
    { 0xcfdf0340, 61 },
    { 0x901f0341, 61 },
    { 0x625f0342, 95 },
    { 0x194f0343, 61 },
    { 0xa5bf0343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df034a, 87 },
    { 0x314f034b, 93 },
    { 0, 0 },
    { 0xcf8f034d, 76 },
    { 0x92f034e, 93 },
    { 0x28ff034f, 90 },
    { 0xefcf0350, 38 },
    { 0x477f034d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ef0357, 81 },
    { 0x657f0357, 93 },
    { 0x6d8f0359, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bf035c, 90 },
    { 0x9f0f035d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f0365, 23 },
    { 0xcef0365, 90 },
    { 0x81bf0367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff036a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f036d, 75 },
    { 0x5f6f036d, 43 },
    { 0, 0 },
    { 0x3bef0370, 23 },
    { 0, 0 },
    { 0x903f0372, 95 },
    { 0x7e8f0373, 64 },
    { 0, 0 },
    { 0xd39f0375, 95 },
    { 0x989f0376, 98 },
    { 0xd16f0375, 91 },
    { 0x87bf0378, 93 },
    { 0xa9bf0378, 95 },
    { 0xf5af0378, 90 },
    { 0xd1ff037b, 23 },
    { 0x7d2f037b, 76 },
    { 0x6f6f037d, 33 },
    { 0x56af037b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550f0382, 33 },
    { 0, 0 },
    { 0x493f0384, 34 },
    { 0x2a8f0385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff0389, 99 },
    { 0, 0 },
    { 0x2f3f038b, 55 },
    { 0xffcf038c, 33 },
    { 0xa66f038b, 61 },
    { 0, 0 },
    { 0xe81f038f, 61 },
    { 0xffff038f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf0395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f039d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f03a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491f03a6, 61 },
    { 0xdabf03a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f03aa, 52 },
    { 0x37f03ab, 46 },
    { 0, 0 },
    { 0xb52f03ad, 33 },
    { 0, 0 },
    { 0x8d6f03af, 90 },
    { 0xea3f03b0, 90 },
    { 0x2c1f03b1, 76 },
    { 0x1b2f03b2, 80 },
    { 0xc60f03af, 95 },
    { 0, 0 },
    { 0x43ef03b5, 90 },
    { 0xf4f03b6, 59 },
    { 0xcfff03b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ff03bb, 68 },
    { 0xbbaf03bc, 66 },
    { 0xebbf03bd, 33 },
    { 0x929f03bd, 33 },
    { 0xcd7f03bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f03c2, 33 },
    { 0x29ff03c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765f03c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3f03c9, 93 },
    { 0x1ef03ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf03d4, 85 },
    { 0xd60f03d4, 90 },
    { 0xaa3f03d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef03d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f03de, 46 },
    { 0x731f03df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f03e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baf03e6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf03ea, 84 },
    { 0, 0 },
    { 0x5b0f03ec, 84 },
    { 0x1d5f03ec, 90 },
    { 0xd1ff03ee, 23 },
    { 0x9a8f03ed, 76 },
    { 0, 0 },
    { 0x1a8f03f1, 55 },
    { 0xbe1f03f2, 90 },
    { 0x9a6f03f3, 91 },
    { 0, 0 },
    { 0xe51f03f5, 94 },
    { 0xe5f03f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fff03f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847f03fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8f0401, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5f0404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f0408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677f0412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48f0415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f0419, 90 },
    { 0xacaf041a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf041d, 81 },
    { 0x8c3f041e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbf0424, 65 },
    { 0x78df0425, 90 },
    { 0xeb1f0426, 91 },
    { 0xe33f0426, 90 },
    { 0x22af0428, 95 },
    { 0x2d7f0428, 61 },
    { 0x867f0426, 43 },
    { 0x3faf0426, 61 },
    { 0, 0 },
    { 0x55df042d, 23 },
    { 0, 0 },
    { 0x99cf042f, 100 },
    { 0xa8ef042f, 34 },
    { 0, 0 },
    { 0x284f0432, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f0436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f043a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df043d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12f0440, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f044a, 46 },
    { 0x632f044b, 23 },
    { 0x611f044c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef044f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f0453, 55 },
    { 0x492f0453, 34 },
    { 0xa2ff0454, 95 },
    { 0x704f0453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabf045a, 91 },
    { 0, 0 },
    { 0x141f045c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acf0461, 90 },
    { 0x291f0461, 33 },
    { 0x6adf0461, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49f0468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5f046d, 33 },
    { 0xc59f046e, 55 },
    { 0x5d0f046f, 93 },
    { 0x4f4f046f, 90 },
    { 0x67ef046f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf0474, 90 },
    { 0x1c5f0475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33f047a, 23 },
    { 0x2c9f047b, 43 },
    { 0x799f047c, 33 },
    { 0, 0 },
    { 0x1faf047e, 23 },
    { 0x6caf047e, 23 },
    { 0x9e9f0480, 95 },
    { 0, 0 },
    { 0x1a2f0482, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf0486, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf0489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x223f048c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caf0491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef0498, 37 },
    { 0x863f0499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f04a6, 34 },
    { 0x554f04a7, 33 },
    { 0xb87f04a6, 91 },
    { 0x71ff04a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83f04ad, 52 },
    { 0xffdf04ae, 76 },
    { 0x586f04ae, 84 },
    { 0x937f04b0, 76 },
    { 0, 0 },
    { 0xb90f04b2, 98 },
    { 0x592f04b3, 90 },
    { 0, 0 },
    { 0x220f04b5, 33 },
    { 0x325f04b6, 69 },
    { 0x494f04b7, 46 },
    { 0xcf5f04b8, 90 },
    { 0x10ef04b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f04bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443f04c4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cf04c8, 95 },
    { 0, 0 },
    { 0x337f04ca, 91 },
    { 0x48ff04ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf04cf, 95 },
    { 0xff9f04d0, 98 },
    { 0x4c4f04d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1af04d5, 76 },
    { 0xaef04d6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bff04db, 90 },
    { 0xa34f04db, 90 },
    { 0x6dcf04dd, 90 },
    { 0x77df04de, 61 },
    { 0xb86f04df, 95 },
    { 0x674f04e0, 38 },
    { 0xd51f04e1, 98 },
    { 0x20bf04e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf04e6, 76 },
    { 0x54df04e7, 33 },
    { 0xba4f04e8, 23 },
    { 0, 0 },
    { 0xc6f04ea, 33 },
    { 0x8ebf04eb, 61 },
    { 0x116f04eb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf04f0, 55 },
    { 0x9f1f04f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4f04f6, 17 },
    { 0x4baf04f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2f04ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0502, 85 },
    { 0, 0 },
    { 0x665f0504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f050c, 34 },
    { 0x12ef050c, 90 },
    { 0x7c7f050e, 44 },
    { 0xf60f050f, 91 },
    { 0x47f0510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275f0517, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf051c, 100 },
    { 0x1af051c, 61 },
    { 0x20f051e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef0521, 33 },
    { 0, 0 },
    { 0x84ef0523, 33 },
    { 0x89bf0524, 85 },
    { 0x539f0524, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816f0529, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8f052d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4f0531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdff0534, 90 },
    { 0xa96f0535, 98 },
    { 0xb14f0535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300f053a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f0540, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f0543, 33 },
    { 0, 0 },
    { 0x3fbf0545, 90 },
    { 0, 0 },
    { 0x69f0547, 55 },
    { 0xb99f0548, 51 },
    { 0, 0 },
    { 0x60bf054a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x501f054d, 22 },
    { 0, 0 },
    { 0x1a6f054f, 61 },
    { 0, 0 },
    { 0x582f0551, 33 },
    { 0x856f0552, 90 },
    { 0x4d1f0552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3f0557, 23 },
    { 0xba8f0557, 95 },
    { 0x11ff0559, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f055c, 61 },
    { 0, 0 },
    { 0x2b5f055e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeef0562, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f0566, 23 },
    { 0x845f0566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f056b, 61 },
    { 0x3dcf056c, 61 },
    { 0, 0 },
    { 0x5c5f056e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9f0573, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cf0576, 64 },
    { 0x1ebf0577, 85 },
    { 0xcdcf0578, 90 },
    { 0xe2bf0579, 55 },
    { 0x63ef0579, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0580, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f0584, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cf0587, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95f058a, 46 },
    { 0x43cf058b, 91 },
    { 0, 0 },
    { 0xb90f058d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cf0590, 81 },
    { 0x4ff0591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdf0599, 84 },
    { 0x68ef059a, 99 },
    { 0, 0 },
    { 0xbddf059c, 66 },
    { 0x3a0f059d, 34 },
    { 0xe61f059d, 53 },
    { 0xab7f059d, 33 },
    { 0x514f05a0, 90 },
    { 0x3bf05a1, 90 },
    { 0xd64f05a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ef05a8, 43 },
    { 0, 0 },
    { 0x8ff05aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ef05af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52f05b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af05b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf05bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf05be, 33 },
    { 0, 0 },
    { 0x3c6f05c0, 46 },
    { 0xe84f05c1, 90 },
    { 0, 0 },
    { 0x778f05c3, 66 },
    { 0xec5f05c3, 34 },
    { 0x2ccf05c5, 23 },
    { 0, 0 },
    { 0x180f05c7, 23 },
    { 0, 0 },
    { 0x63ff05c9, 61 },
    { 0x910f05ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f05d5, 87 },
    { 0x1ddf05d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523f05df, 93 },
    { 0, 0 },
    { 0xf19f05e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f05e7, 95 },
    { 0, 0 },
    { 0xf2af05e9, 100 },
    { 0xdf4f05ea, 98 },
    { 0, 0 },
    { 0xbb3f05ec, 87 },
    { 0, 0 },
    { 0x626f05ee, 61 },
    { 0x795f05ef, 88 },
    { 0x13af05ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f05f8, 95 },
    { 0x4ef05f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ef05ff, 66 },
    { 0xa72f05ff, 23 },
    { 0, 0 },
    { 0x9aaf0602, 34 },
    { 0, 0 },
    { 0xc46f0604, 77 },
    { 0x7b3f0604, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff0609, 61 },
    { 0x7fcf0609, 61 },
    { 0x51bf060b, 53 },
    { 0xc6ff0609, 90 },
    { 0xe55f060c, 33 },
    { 0, 0 },
    { 0x30df060f, 22 },
    { 0xaebf0610, 87 },
    { 0x9cef0611, 33 },
    { 0, 0 },
    { 0x9e5f0613, 90 },
    { 0xb45f0614, 76 },
    { 0x288f0615, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5f061a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f061d, 34 },
    { 0xa7cf061e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0625, 38 },
    { 0x29af0626, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21f0629, 90 },
    { 0x778f062a, 23 },
    { 0x4cbf062a, 60 },
    { 0x8fcf0629, 77 },
    { 0xc5df062d, 61 },
    { 0x7bef062e, 98 },
    { 0x942f062f, 66 },
    { 0, 0 },
    { 0x8a8f0631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f0635, 23 },
    { 0x726f0635, 85 },
    { 0x7e0f0637, 90 },
    { 0xbdef0638, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4f063b, 33 },
    { 0xa93f063b, 90 },
    { 0x92af063d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f0640, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73af0647, 76 },
    { 0x71f0648, 66 },
    { 0x5a8f0647, 66 },
    { 0x523f0647, 98 },
    { 0x2f5f064a, 68 },
    { 0x851f064c, 90 },
    { 0xb89f0648, 55 },
    { 0, 0 },
    { 0x305f064f, 61 },
    { 0xf33f0650, 83 },
    { 0, 0 },
    { 0x389f0652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x471f0655, 87 },
    { 0, 0 },
    { 0x10bf0657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856f065f, 90 },
    { 0, 0 },
    { 0xfc3f0661, 91 },
    { 0xf87f0662, 68 },
    { 0x203f0663, 23 },
    { 0x414f0663, 90 },
    { 0x579f0662, 76 },
    { 0x69ef0666, 90 },
    { 0xadbf0664, 90 },
    { 0xd9ff0668, 95 },
    { 0xf90f0665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f066f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf0672, 53 },
    { 0x781f0673, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f0676, 90 },
    { 0x1fdf0677, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08f067d, 23 },
    { 0xdcaf067e, 99 },
    { 0x589f067d, 23 },
    { 0, 0 },
    { 0x6c4f0681, 90 },
    { 0, 0 },
    { 0x372f0683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0689, 85 },
    { 0x14f068a, 55 },
    { 0x5f6f068b, 77 },
    { 0x9dff068c, 55 },
    { 0x26af068b, 95 },
    { 0xee6f0689, 95 },
    { 0x9a7f068f, 52 },
    { 0, 0 },
    { 0xa7af0691, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f0697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6f069a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4f069d, 46 },
    { 0x732f069d, 61 },
    { 0xc68f069f, 95 },
    { 0x8ecf06a0, 98 },
    { 0xd2ff06a0, 61 },
    { 0x9faf06a1, 53 },
    { 0xf01f06a1, 76 },
    { 0x656f069f, 75 },
    { 0, 0 },
    { 0xfaaf06a6, 23 },
    { 0x2a2f06a6, 98 },
    { 0xd39f06a8, 90 },
    { 0x588f06a8, 91 },
    { 0xec6f06aa, 34 },
    { 0x8f06ab, 33 },
    { 0xa8af06ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cef06b1, 46 },
    { 0x4dcf06b1, 74 },
    { 0, 0 },
    { 0xd20f06b4, 95 },
    { 0x3c6f06b4, 83 },
    { 0xb14f06b6, 33 },
    { 0, 0 },
    { 0x967f06b8, 90 },
    { 0, 0 },
    { 0x8caf06ba, 98 },
    { 0xd22f06ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f06c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaf06c4, 67 },
    { 0x7f5f06c5, 98 },
    { 0x7f3f06c6, 23 },
    { 0xbfff06c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f06cd, 61 },
    { 0x7eff06ce, 98 },
    { 0x794f06ce, 98 },
    { 0x111f06d0, 34 },
    { 0x2faf06d0, 76 },
    { 0, 0 },
    { 0x288f06d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f06d8, 75 },
    { 0x5eff06d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf06dc, 84 },
    { 0x7d0f06dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ef06e1, 98 },
    { 0xa7bf06e2, 90 },
    { 0x44df06e2, 77 },
    { 0x37ff06e3, 95 },
    { 0xc81f06e2, 98 },
    { 0x9e9f06e6, 33 },
    { 0xe8cf06e7, 90 },
    { 0x272f06e7, 91 },
    { 0xee1f06e9, 52 },
    { 0x6af06e9, 34 },
    { 0xaef06eb, 52 },
    { 0xf73f06e1, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff06f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af06f6, 55 },
    { 0x272f06f6, 100 },
    { 0x21af06f8, 99 },
    { 0x829f06f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef06fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df0700, 67 },
    { 0x873f0701, 40 },
    { 0x64bf0701, 98 },
    { 0x734f0703, 95 },
    { 0x4ecf0703, 34 },
    { 0, 0 },
    { 0xc65f0706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f070b, 23 },
    { 0xf7f070c, 55 },
    { 0x211f070c, 95 },
    { 0x111f070d, 95 },
    { 0xcfdf070b, 95 },
    { 0x18ef0710, 90 },
    { 0, 0 },
    { 0xa6ff0712, 90 },
    { 0xa1ef0713, 90 },
    { 0x7d6f0714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47f0719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f071c, 23 },
    { 0x45f071d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60f0721, 91 },
    { 0x5eef0721, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x603f0725, 90 },
    { 0, 0 },
    { 0xc7f0727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945f072a, 90 },
    { 0xcd2f072a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4f072e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaf0731, 33 },
    { 0x139f0732, 76 },
    { 0xff3f0733, 98 },
    { 0x2e5f0731, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f0737, 95 },
    { 0x5b2f0738, 67 },
    { 0, 0 },
    { 0xae8f073a, 90 },
    { 0x37cf073b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef073e, 34 },
    { 0x638f073e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf074a, 61 },
    { 0, 0 },
    { 0x1dbf074c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20f0750, 88 },
    { 0x12df0750, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef0756, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f075b, 90 },
    { 0xd3ff075b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f0761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3f0766, 85 },
    { 0xabcf0766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f076d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ef0772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f0776, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5f0781, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6f0789, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4f0792, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f0795, 65 },
    { 0, 0 },
    { 0x8def0797, 61 },
    { 0, 0 },
    { 0x2eff0799, 78 },
    { 0, 0 },
    { 0x555f079b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8f07a4, 76 },
    { 0x8b8f07a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93f07a9, 93 },
    { 0x4bbf07aa, 90 },
    { 0x69bf07aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddf07b0, 88 },
    { 0x912f07b1, 52 },
    { 0xd61f07b2, 85 },
    { 0x8f3f07b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47df07b7, 61 },
    { 0x905f07b8, 90 },
    { 0x1a8f07b9, 66 },
    { 0xf16f07ba, 23 },
    { 0x2e6f07bb, 43 },
    { 0x7a0f07bc, 91 },
    { 0x23ef07bd, 95 },
    { 0x930f07bd, 76 },
    { 0, 0 },
    { 0x51df07c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f07ca, 33 },
    { 0x5cff07ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56af07d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f07d8, 61 },
    { 0x3d0f07d8, 75 },
    { 0x8f5f07da, 61 },
    { 0x3a3f07db, 76 },
    { 0x3df07dc, 98 },
    { 0x2b6f07dd, 78 },
    { 0, 0 },
    { 0x3cdf07df, 90 },
    { 0xdc9f07e0, 34 },
    { 0xd37f07e0, 43 },
    { 0xeb9f07e1, 76 },
    { 0, 0 },
    { 0x7df07e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f07eb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f07f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4f07f4, 38 },
    { 0x8b0f07f5, 61 },
    { 0xc1f07f5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff07f9, 33 },
    { 0x266f07fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ff0802, 43 },
    { 0x4fdf0803, 43 },
    { 0x32f0804, 23 },
    { 0x2f3f0805, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8def0808, 95 },
    { 0x788f0809, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f080c, 100 },
    { 0x12af080d, 95 },
    { 0xc1cf080d, 55 },
    { 0xcedf080f, 43 },
    { 0x4a9f0810, 95 },
    { 0x9def0811, 43 },
    { 0xff5f0811, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdf0816, 33 },
    { 0, 0 },
    { 0x18f0818, 98 },
    { 0, 0 },
    { 0xa01f081a, 98 },
    { 0x391f081a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0f081f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f0823, 55 },
    { 0x366f0824, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0827, 33 },
    { 0xe78f0828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f082c, 33 },
    { 0x2def082c, 91 },
    { 0x95f082e, 61 },
    { 0x2bbf082f, 33 },
    { 0x584f082e, 76 },
    { 0x58af082d, 76 },
    { 0x368f0832, 61 },
    { 0xe3ef0830, 33 },
    { 0xfe0f082c, 93 },
    { 0xc6df0835, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdf0838, 95 },
    { 0, 0 },
    { 0xa5ef083a, 43 },
    { 0x3d4f083b, 34 },
    { 0xa6f083a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43af0843, 95 },
    { 0, 0 },
    { 0x675f0845, 91 },
    { 0x8f0846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f084b, 52 },
    { 0xdcff084c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaf084f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5df0852, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f0857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff085b, 33 },
    { 0x29f085c, 61 },
    { 0xcf3f085c, 23 },
    { 0x72cf085c, 76 },
    { 0xa8f085d, 90 },
    { 0, 0 },
    { 0xfecf0861, 90 },
    { 0, 0 },
    { 0xf5ff0863, 51 },
    { 0x89ff0863, 99 },
    { 0xaaef0864, 76 },
    { 0x80f0865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f0871, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf0877, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff087a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f0881, 98 },
    { 0xa80f0882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706f0888, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3f088e, 61 },
    { 0, 0 },
    { 0xd33f0890, 55 },
    { 0x462f0890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f0899, 33 },
    { 0xadff089a, 91 },
    { 0x7d1f089b, 61 },
    { 0x7e4f089b, 64 },
    { 0xa04f089d, 48 },
    { 0x13bf089e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79f08a2, 23 },
    { 0x30ff08a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f08a6, 84 },
    { 0x199f08a7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f08aa, 23 },
    { 0, 0 },
    { 0xc92f08ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df08b1, 90 },
    { 0, 0 },
    { 0xf5df08b3, 95 },
    { 0x205f08b3, 90 },
    { 0, 0 },
    { 0x15ef08b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553f08c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00f08c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f08ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994f08d3, 12 },
    { 0x2f2f08d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4f08d8, 90 },
    { 0xb7ef08d8, 33 },
    { 0, 0 },
    { 0x376f08db, 33 },
    { 0x9e4f08dc, 98 },
    { 0, 0 },
    { 0xb20f08de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4f08e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff08e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414f08e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f08ed, 88 },
    { 0xa22f08ed, 98 },
    { 0x6edf08ef, 98 },
    { 0x811f08f0, 90 },
    { 0x122f08ef, 76 },
    { 0x3b1f08f2, 23 },
    { 0x780f08f3, 76 },
    { 0x25df08f4, 46 },
    { 0, 0 },
    { 0x9f2f08f6, 61 },
    { 0x85ef08f7, 34 },
    { 0x3b5f08f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f08ff, 33 },
    { 0x836f0900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3f0905, 95 },
    { 0, 0 },
    { 0x2a7f0907, 96 },
    { 0xeabf0908, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f090b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cf0910, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5f0914, 17 },
    { 0x9ef0915, 98 },
    { 0xd90f0916, 61 },
    { 0x48ff0917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88f091b, 77 },
    { 0x826f091c, 34 },
    { 0x6daf091b, 99 },
    { 0x410f091c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cf0922, 43 },
    { 0x1d0f0923, 87 },
    { 0xa2bf0923, 33 },
    { 0, 0 },
    { 0x643f0926, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cf0929, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f0930, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9f0935, 33 },
    { 0xf86f0936, 52 },
    { 0x208f0936, 78 },
    { 0x7bcf0937, 33 },
    { 0x7a9f0939, 61 },
    { 0x7aef0936, 90 },
    { 0x808f093b, 23 },
    { 0x1d8f093c, 90 },
    { 0, 0 },
    { 0x23ef093e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf0943, 93 },
    { 0, 0 },
    { 0xb0df0945, 61 },
    { 0, 0 },
    { 0xe0bf0947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183f094e, 93 },
    { 0xd74f094f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cf0955, 99 },
    { 0x5a1f0956, 90 },
    { 0, 0 },
    { 0xa01f0958, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baf095c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f095f, 90 },
    { 0x482f0960, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63df0966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6f096a, 55 },
    { 0x801f096b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf0973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6f097b, 38 },
    { 0x9aaf097c, 78 },
    { 0xfd8f097d, 98 },
    { 0xf51f097d, 76 },
    { 0x2f8f097e, 66 },
    { 0x2fbf0980, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1f098e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f0995, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff0998, 99 },
    { 0x18cf0999, 46 },
    { 0x702f099a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90f099f, 34 },
    { 0x15ef09a0, 95 },
    { 0, 0 },
    { 0xba9f09a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f09a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf09ac, 93 },
    { 0x310f09ad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f09b0, 43 },
    { 0x866f09b1, 78 },
    { 0xf9bf09b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f09b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff09bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f09c2, 46 },
    { 0x9e8f09c2, 98 },
    { 0x194f09c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1f09c9, 34 },
    { 0xc29f09c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f09d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf09d6, 34 },
    { 0x5e0f09d7, 43 },
    { 0x222f09d8, 40 },
    { 0xa0ef09d8, 61 },
    { 0, 0 },
    { 0x35cf09db, 23 },
    { 0x21cf09db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f09e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0f09ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2f09f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8f09f6, 36 },
    { 0x76af09f6, 33 },
    { 0, 0 },
    { 0x995f09f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af09fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443f0a07, 33 },
    { 0x49cf0a07, 34 },
    { 0, 0 },
    { 0x8b3f0a0a, 95 },
    { 0, 0 },
    { 0x45bf0a0c, 90 },
    { 0x317f0a0d, 23 },
    { 0x6c1f0a0d, 43 },
    { 0x985f0a0d, 93 },
    { 0xf9cf0a0d, 52 },
    { 0x963f0a10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df0a1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242f0a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6df0a27, 90 },
    { 0xae4f0a27, 61 },
    { 0, 0 },
    { 0x26df0a2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0a2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f0a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ef0a3c, 84 },
    { 0xa66f0a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6af0a43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3f0a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af0a4d, 55 },
    { 0, 0 },
    { 0xa61f0a4f, 33 },
    { 0xabff0a50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53df0a54, 98 },
    { 0, 0 },
    { 0x3acf0a56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0a5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f0a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6df0a65, 100 },
    { 0, 0 },
    { 0x7c9f0a67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f0a6d, 81 },
    { 0x79f0a6e, 90 },
    { 0x407f0a6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f0a72, 90 },
    { 0x889f0a72, 61 },
    { 0x89f0a74, 68 },
    { 0x8f5f0a73, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545f0a7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0a7f, 43 },
    { 0x186f0a80, 64 },
    { 0xadbf0a81, 76 },
    { 0xd2df0a82, 90 },
    { 0xbf7f0a83, 68 },
    { 0x7daf0a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f0a88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0a8f, 78 },
    { 0, 0 },
    { 0x61cf0a91, 90 },
    { 0x315f0a92, 46 },
    { 0x7eaf0a92, 34 },
    { 0, 0 },
    { 0xf0df0a95, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8af0aa1, 95 },
    { 0, 0 },
    { 0xdef0aa3, 67 },
    { 0x1c7f0aa4, 98 },
    { 0xd6f0aa4, 98 },
    { 0xeb1f0aa6, 61 },
    { 0, 0 },
    { 0x690f0aa8, 90 },
    { 0x724f0aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x663f0aac, 64 },
    { 0x8d6f0aad, 46 },
    { 0x39cf0aae, 84 },
    { 0x491f0aaf, 33 },
    { 0x1eef0aaf, 76 },
    { 0x8dff0ab0, 17 },
    { 0xc38f0aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30f0ab8, 85 },
    { 0xd73f0ab9, 34 },
    { 0, 0 },
    { 0x786f0abb, 23 },
    { 0x760f0abc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af0abf, 98 },
    { 0x8dcf0ac0, 78 },
    { 0xf73f0ac1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef0ac4, 61 },
    { 0, 0 },
    { 0x86f0ac6, 90 },
    { 0x2c7f0ac7, 95 },
    { 0xdff0ac7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f0acb, 33 },
    { 0x662f0acc, 33 },
    { 0xdf0f0acd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2af0ad0, 95 },
    { 0xfd3f0ad0, 67 },
    { 0x979f0ad1, 95 },
    { 0x4a0f0ad3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df0ad7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747f0adb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f0ae2, 75 },
    { 0, 0 },
    { 0x784f0ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcf0ae8, 98 },
    { 0xbcdf0ae9, 55 },
    { 0, 0 },
    { 0x268f0aeb, 34 },
    { 0x82ef0aec, 78 },
    { 0xa13f0aed, 46 },
    { 0xae0f0aee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f0af3, 98 },
    { 0, 0 },
    { 0xd2ef0af5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f0afa, 52 },
    { 0, 0 },
    { 0x909f0afc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff0b00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7f0b06, 34 },
    { 0xe36f0b07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ef0b0a, 61 },
    { 0x6baf0b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af0b0f, 61 },
    { 0x393f0b0f, 61 },
    { 0xc4f0b11, 33 },
    { 0xcb2f0b12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253f0b1a, 76 },
    { 0xaf0f0b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f0b20, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9f0b23, 90 },
    { 0, 0 },
    { 0xa87f0b25, 67 },
    { 0x150f0b26, 34 },
    { 0, 0 },
    { 0xdf6f0b28, 52 },
    { 0x6acf0b29, 90 },
    { 0xd8af0b2a, 61 },
    { 0, 0 },
    { 0xb18f0b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4af0b2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f0b36, 23 },
    { 0x938f0b37, 84 },
    { 0x5b5f0b37, 85 },
    { 0x944f0b37, 33 },
    { 0x2a8f0b3a, 90 },
    { 0, 0 },
    { 0x63cf0b3c, 33 },
    { 0x7baf0b3c, 81 },
    { 0x393f0b3e, 66 },
    { 0xb05f0b3e, 95 },
    { 0xc90f0b3d, 84 },
    { 0xb5df0b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f0b4a, 98 },
    { 0xfeaf0b4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f0b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f0b58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413f0b5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f0b61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283f0b65, 33 },
    { 0x716f0b66, 77 },
    { 0x649f0b66, 34 },
    { 0xf3cf0b68, 84 },
    { 0, 0 },
    { 0x6a2f0b6a, 76 },
    { 0x23cf0b6b, 99 },
    { 0x546f0b6b, 34 },
    { 0, 0 },
    { 0xf20f0b6e, 65 },
    { 0x771f0b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f0b74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff0b77, 33 },
    { 0x36af0b78, 33 },
    { 0x3d2f0b79, 55 },
    { 0x4e5f0b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aef0b7e, 43 },
    { 0xa5cf0b7f, 91 },
    { 0x90af0b7f, 33 },
    { 0x1daf0b80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f0b8a, 61 },
    { 0, 0 },
    { 0xbbdf0b8c, 90 },
    { 0x219f0b8c, 46 },
    { 0xdcaf0b8d, 90 },
    { 0, 0 },
    { 0xc15f0b90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f0b94, 91 },
    { 0x312f0b94, 90 },
    { 0x7f6f0b96, 90 },
    { 0, 0 },
    { 0x475f0b98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4f0ba1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803f0ba5, 100 },
    { 0x8dbf0ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f0ba9, 43 },
    { 0, 0 },
    { 0x48ff0bab, 95 },
    { 0x6d9f0bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5f0bb0, 46 },
    { 0, 0 },
    { 0x24cf0bb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff0bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f0bba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f0bbd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43f0bc2, 98 },
    { 0xf35f0bc2, 66 },
    { 0, 0 },
    { 0xb9bf0bc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf0bc8, 95 },
    { 0xe89f0bc9, 33 },
    { 0x630f0bca, 69 },
    { 0x4ff0bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf0bd6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01f0bda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f0be0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f0be3, 59 },
    { 0x28f0be4, 98 },
    { 0x752f0be5, 51 },
    { 0x719f0be6, 95 },
    { 0x542f0be6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdf0bec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18f0bf2, 34 },
    { 0x7def0bf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f0bf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x732f0bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f0bfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f0bff, 23 },
    { 0xa21f0c00, 99 },
    { 0, 0 },
    { 0x55bf0c02, 43 },
    { 0, 0 },
    { 0x4eef0c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbf0c09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df0c0c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f0c10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05f0c16, 99 },
    { 0x5c3f0c17, 61 },
    { 0xd3cf0c18, 98 },
    { 0x83f0c18, 23 },
    { 0x246f0c16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fff0c20, 84 },
    { 0xea3f0c20, 23 },
    { 0x327f0c22, 23 },
    { 0, 0 },
    { 0x780f0c24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9f0c2c, 90 },
    { 0x637f0c2d, 76 },
    { 0xca3f0c2c, 55 },
    { 0, 0 },
    { 0x3c3f0c30, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1df0c39, 95 },
    { 0, 0 },
    { 0x16af0c3b, 77 },
    { 0, 0 },
    { 0xe23f0c3d, 91 },
    { 0x7d9f0c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f0c45, 61 },
    { 0, 0 },
    { 0x232f0c47, 33 },
    { 0, 0 },
    { 0x864f0c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f0c4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3f0c51, 46 },
    { 0, 0 },
    { 0x676f0c53, 52 },
    { 0, 0 },
    { 0xb06f0c55, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f0c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941f0c5c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6df0c5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f0c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf0c67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f0c6c, 23 },
    { 0xb4ef0c6d, 98 },
    { 0, 0 },
    { 0x5f3f0c6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf0c73, 75 },
    { 0, 0 },
    { 0x78f0c75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaf0c7a, 80 },
    { 0xc18f0c7b, 90 },
    { 0xba5f0c7c, 90 },
    { 0, 0 },
    { 0x790f0c7e, 98 },
    { 0xe98f0c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabff0c84, 55 },
    { 0x7f9f0c85, 99 },
    { 0x79df0c85, 99 },
    { 0x4eef0c87, 66 },
    { 0, 0 },
    { 0x402f0c89, 33 },
    { 0, 0 },
    { 0x942f0c8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af0c90, 55 },
    { 0, 0 },
    { 0xf33f0c92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0c99, 85 },
    { 0xddff0c9a, 33 },
    { 0x596f0c9a, 43 },
    { 0x1f3f0c9b, 99 },
    { 0xecbf0c99, 46 },
    { 0x808f0c9e, 88 },
    { 0, 0 },
    { 0xc5f0ca0, 98 },
    { 0x7bcf0ca0, 85 },
    { 0x2a5f0ca2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f0ca5, 17 },
    { 0x56f0ca6, 33 },
    { 0x178f0ca5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0caf, 23 },
    { 0x9aef0cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3f0cb3, 90 },
    { 0xd3af0cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdf0cba, 55 },
    { 0xd4f0cbb, 76 },
    { 0x313f0cbb, 61 },
    { 0xaaff0cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acf0cc2, 93 },
    { 0x8b2f0cc2, 36 },
    { 0x2aff0cc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f0cc8, 85 },
    { 0xb7bf0cc8, 98 },
    { 0x8f2f0cca, 76 },
    { 0x648f0cc9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95f0ccf, 43 },
    { 0x8b1f0cd0, 23 },
    { 0, 0 },
    { 0xaaef0cd2, 40 },
    { 0xc61f0cd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x754f0cd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0f0cda, 61 },
    { 0x83bf0cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79f0ce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575f0ce6, 33 },
    { 0x7b5f0ce7, 23 },
    { 0x36f0ce8, 33 },
    { 0xd3ff0ce6, 98 },
    { 0xc20f0cea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2f0cf0, 46 },
    { 0xd7f0cf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf0cf4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10f0cf8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9f0cfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df0cff, 55 },
    { 0x613f0cff, 23 },
    { 0xfc9f0d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f0d05, 33 },
    { 0, 0 },
    { 0x364f0d07, 90 },
    { 0x4ecf0d07, 99 },
    { 0x5b6f0d07, 43 },
    { 0x62bf0d07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf0d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631f0d18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf0d20, 90 },
    { 0, 0 },
    { 0x254f0d22, 61 },
    { 0x17bf0d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bf0d27, 34 },
    { 0x7f6f0d28, 33 },
    { 0xe55f0d29, 33 },
    { 0x981f0d27, 55 },
    { 0xb4ef0d2a, 61 },
    { 0x699f0d27, 93 },
    { 0, 0 },
    { 0x7e1f0d2e, 100 },
    { 0xa9cf0d2e, 77 },
    { 0x715f0d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9f0d35, 61 },
    { 0x797f0d36, 93 },
    { 0x319f0d37, 33 },
    { 0xdf6f0d36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f0d3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933f0d41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ef0d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25df0d48, 90 },
    { 0xeb5f0d49, 35 },
    { 0x17df0d4a, 61 },
    { 0, 0 },
    { 0x867f0d4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f0d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ef0d52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf0d55, 66 },
    { 0x6d0f0d56, 76 },
    { 0xd6ff0d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f0d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf0d5f, 97 },
    { 0, 0 },
    { 0xcdf0d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f0d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f0d68, 61 },
    { 0x525f0d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f0d6c, 77 },
    { 0x624f0d6d, 23 },
    { 0x93cf0d6e, 61 },
    { 0, 0 },
    { 0x23af0d70, 23 },
    { 0xd9df0d70, 90 },
    { 0xa3df0d72, 90 },
    { 0xf7df0d71, 55 },
    { 0xf8ef0d72, 76 },
    { 0xf0cf0d72, 91 },
    { 0x332f0d76, 95 },
    { 0, 0 },
    { 0x19ff0d78, 53 },
    { 0, 0 },
    { 0xa82f0d7a, 55 },
    { 0x2ebf0d7b, 36 },
    { 0x9d9f0d7c, 90 },
    { 0x4bef0d7d, 98 },
    { 0xf5f0d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f0d85, 90 },
    { 0x78af0d85, 23 },
    { 0xc77f0d87, 95 },
    { 0xadef0d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daf0d8c, 95 },
    { 0x510f0d8d, 23 },
    { 0xe7cf0d8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x511f0d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf0d94, 76 },
    { 0x933f0d94, 61 },
    { 0x224f0d95, 76 },
    { 0xf9ff0d97, 33 },
    { 0, 0 },
    { 0xbd6f0d99, 85 },
    { 0xd4cf0d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fff0d9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbf0da7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x382f0daa, 74 },
    { 0xc60f0dab, 95 },
    { 0xef7f0dac, 80 },
    { 0x304f0dad, 76 },
    { 0x8b4f0dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ff0dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df0dc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9f0dce, 64 },
    { 0xd1ef0dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f0dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f0dd7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af0ddd, 90 },
    { 0, 0 },
    { 0xc7bf0ddf, 23 },
    { 0x9f0f0ddf, 95 },
    { 0xa97f0de1, 34 },
    { 0x7fdf0de2, 47 },
    { 0x2cdf0de2, 87 },
    { 0x60cf0de2, 101 },
    { 0xcef0de1, 90 },
    { 0x508f0de6, 33 },
    { 0x4b0f0de5, 93 },
    { 0xfaf0de8, 23 },
    { 0xd27f0de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0df0dec, 36 },
    { 0x124f0ded, 76 },
    { 0, 0 },
    { 0x1d6f0def, 66 },
    { 0, 0 },
    { 0xc2bf0df1, 61 },
    { 0, 0 },
    { 0xd9df0df3, 95 },
    { 0x6e3f0df4, 52 },
    { 0, 0 },
    { 0xe4cf0df6, 33 },
    { 0x57bf0df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bf0dfb, 33 },
    { 0, 0 },
    { 0x94f0dfd, 90 },
    { 0xfe1f0dfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f0e01, 98 },
    { 0x960f0e01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef0e05, 90 },
    { 0x33bf0e05, 90 },
    { 0, 0 },
    { 0x9fbf0e08, 98 },
    { 0x8b4f0e09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f0e0c, 17 },
    { 0, 0 },
    { 0xc37f0e0e, 91 },
    { 0, 0 },
    { 0xd39f0e10, 23 },
    { 0x77af0e10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x683f0e14, 66 },
    { 0x339f0e15, 99 },
    { 0, 0 },
    { 0xc80f0e17, 61 },
    { 0xfcf0e18, 43 },
    { 0x39af0e19, 33 },
    { 0x6e3f0e1a, 68 },
    { 0xe61f0e1a, 98 },
    { 0x82f0e1a, 23 },
    { 0x873f0e1d, 23 },
    { 0xce0f0e1e, 55 },
    { 0x489f0e19, 75 },
    { 0x59ef0e1f, 61 },
    { 0x253f0e21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff0e24, 95 },
    { 0, 0 },
    { 0x3a6f0e26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef0e2b, 90 },
    { 0x9fef0e2c, 95 },
    { 0x711f0e2d, 100 },
    { 0x38f0e2e, 77 },
    { 0x673f0e2f, 34 },
    { 0xf73f0e2e, 95 },
    { 0, 0 },
    { 0xc30f0e32, 95 },
    { 0xcebf0e33, 23 },
    { 0x351f0e34, 95 },
    { 0, 0 },
    { 0x283f0e36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36af0e3a, 66 },
    { 0x227f0e3b, 61 },
    { 0xfadf0e3c, 93 },
    { 0x388f0e3c, 61 },
    { 0xbdf0e3a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f0e44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ef0e47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0e4a, 76 },
    { 0xbacf0e4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf0e4e, 33 },
    { 0, 0 },
    { 0x6bdf0e50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819f0e55, 76 },
    { 0, 0 },
    { 0x384f0e57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df0e5a, 90 },
    { 0x9a9f0e5b, 55 },
    { 0, 0 },
    { 0x7abf0e5d, 96 },
    { 0, 0 },
    { 0x871f0e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef0e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890f0e6a, 95 },
    { 0x97df0e6b, 77 },
    { 0x8b7f0e6c, 97 },
    { 0, 0 },
    { 0xfa3f0e6e, 88 },
    { 0, 0 },
    { 0x4cbf0e70, 33 },
    { 0x6a5f0e71, 46 },
    { 0x472f0e72, 33 },
    { 0x652f0e72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818f0e7f, 90 },
    { 0x691f0e7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35f0e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f0e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f0e90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af0e98, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6f0ea2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10af0ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf0ead, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10df0eb0, 100 },
    { 0x150f0eb1, 55 },
    { 0x2c6f0eb1, 23 },
    { 0x6c9f0eb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df0eb9, 95 },
    { 0x2ef0eba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cf0ebe, 87 },
    { 0x370f0ebe, 33 },
    { 0x99f0ebf, 91 },
    { 0x9dff0ec1, 95 },
    { 0x88ff0ec0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef0ec7, 34 },
    { 0xb72f0ec8, 33 },
    { 0x694f0ec9, 87 },
    { 0, 0 },
    { 0xeb5f0ecb, 93 },
    { 0x366f0ecb, 98 },
    { 0, 0 },
    { 0xac8f0ece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcf0ed9, 90 },
    { 0x95af0eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4f0edf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f0ee3, 61 },
    { 0xc42f0ee4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f0ee8, 60 },
    { 0, 0 },
    { 0x207f0eea, 67 },
    { 0xa4ff0eeb, 46 },
    { 0x6edf0eeb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6f0eef, 66 },
    { 0, 0 },
    { 0x136f0ef1, 90 },
    { 0, 0 },
    { 0x587f0ef3, 76 },
    { 0, 0 },
    { 0xd0cf0ef5, 55 },
    { 0x7a8f0ef6, 95 },
    { 0x169f0ef6, 90 },
    { 0x48af0ef8, 85 },
    { 0x299f0ef8, 91 },
    { 0x9ebf0ef8, 84 },
    { 0xd91f0ef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af0efe, 65 },
    { 0x255f0eff, 23 },
    { 0xc1f0efe, 98 },
    { 0x425f0f01, 61 },
    { 0, 0 },
    { 0xebef0f03, 61 },
    { 0xe2ff0f03, 90 },
    { 0x3f6f0f03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f0f09, 76 },
    { 0xabdf0f0a, 76 },
    { 0x38af0f0a, 76 },
    { 0xb7af0f0a, 33 },
    { 0xec3f0f0d, 90 },
    { 0xa36f0f0e, 34 },
    { 0xa75f0f0f, 44 },
    { 0x15ff0f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9f0f13, 91 },
    { 0x5cef0f14, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f0f18, 90 },
    { 0x70bf0f19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527f0f1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df0f20, 98 },
    { 0xeeef0f21, 17 },
    { 0x209f0f22, 46 },
    { 0x28bf0f22, 33 },
    { 0, 0 },
    { 0x8fdf0f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f0f29, 98 },
    { 0x3cef0f2a, 84 },
    { 0, 0 },
    { 0x87f0f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf0f30, 61 },
    { 0x9af0f31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13f0f36, 93 },
    { 0, 0 },
    { 0x6d1f0f38, 61 },
    { 0, 0 },
    { 0x5f9f0f3a, 65 },
    { 0x630f0f3b, 95 },
    { 0x470f0f3c, 68 },
    { 0, 0 },
    { 0x267f0f3e, 33 },
    { 0xd64f0f3f, 91 },
    { 0x664f0f3f, 61 },
    { 0x978f0f3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f0f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdf0f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f0f4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef0f50, 93 },
    { 0x7bef0f50, 33 },
    { 0xdbaf0f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff0f55, 84 },
    { 0xf19f0f56, 98 },
    { 0, 0 },
    { 0x289f0f58, 76 },
    { 0, 0 },
    { 0xd72f0f5a, 65 },
    { 0x14bf0f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf0f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1f0f6a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55f0f72, 46 },
    { 0, 0 },
    { 0xb39f0f74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ef0f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388f0f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f0f7f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df0f85, 98 },
    { 0x67ff0f86, 93 },
    { 0x165f0f85, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7f0f8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x686f0f8d, 65 },
    { 0xa1cf0f8e, 55 },
    { 0x513f0f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cf0f92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbff0f96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99f0f99, 95 },
    { 0, 0 },
    { 0x890f0f9b, 76 },
    { 0, 0 },
    { 0xed3f0f9d, 95 },
    { 0x532f0f9d, 43 },
    { 0, 0 },
    { 0xdc9f0fa0, 93 },
    { 0xe9cf0fa1, 61 },
    { 0xf6df0fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f0fab, 86 },
    { 0x9dbf0fac, 91 },
    { 0, 0 },
    { 0x379f0fae, 36 },
    { 0xf5f0faf, 95 },
    { 0x365f0fb0, 90 },
    { 0, 0 },
    { 0x7e7f0fb2, 34 },
    { 0, 0 },
    { 0xb71f0fb4, 33 },
    { 0x1d8f0fb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1f0fb8, 98 },
    { 0, 0 },
    { 0x45df0fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f0fbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbef0fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5f0fc6, 90 },
    { 0x762f0fc6, 61 },
    { 0xc42f0fc6, 17 },
    { 0x20cf0fc9, 88 },
    { 0xc0af0fca, 52 },
    { 0x823f0fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4f0fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102f0fdb, 23 },
    { 0xaa9f0fdb, 90 },
    { 0x929f0fdc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cf0fe0, 90 },
    { 0, 0 },
    { 0x615f0fe2, 95 },
    { 0x14f0fe2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7f0fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f0feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af0ff2, 76 },
    { 0xfa5f0ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aef0ff9, 78 },
    { 0x7f6f0ff9, 23 },
    { 0xc83f0ffb, 23 },
    { 0xd0cf0ffb, 95 },
    { 0xa7ef0ffd, 98 },
    { 0x72bf0ffb, 91 },
    { 0, 0 },
    { 0x2e3f1000, 95 },
    { 0, 0 },
    { 0x920f1002, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644f1006, 100 },
    { 0, 0 },
    { 0xaaef1008, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06f100b, 17 },
    { 0xa3ff100c, 61 },
    { 0, 0 },
    { 0xb01f100e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259f1013, 55 },
    { 0x9cff1014, 46 },
    { 0xdfbf1014, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f1019, 98 },
    { 0x964f101a, 33 },
    { 0, 0 },
    { 0x43ff101c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25af1021, 46 },
    { 0, 0 },
    { 0x949f1023, 90 },
    { 0, 0 },
    { 0xfb8f1025, 23 },
    { 0x281f1026, 88 },
    { 0xd8ff1025, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8f102a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9f102d, 100 },
    { 0x196f102e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ef1033, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4f1036, 78 },
    { 0x266f1037, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f1041, 67 },
    { 0x495f1042, 35 },
    { 0x219f1043, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ef1048, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f104e, 34 },
    { 0x905f104f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f1054, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f105a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71f105e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x12af1061, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2f1066, 91 },
    { 0, 0 },
    { 0x4f0f1068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f106d, 95 },
    { 0xdccf106d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8f1071, 61 },
    { 0xc11f1072, 95 },
    { 0x840f1072, 90 },
    { 0xaeff1074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ef1078, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587f1083, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf108a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f1091, 58 },
    { 0x258f1092, 23 },
    { 0x8c4f1092, 55 },
    { 0, 0 },
    { 0x11f1095, 85 },
    { 0, 0 },
    { 0x9edf1097, 90 },
    { 0, 0 },
    { 0xc3ff1099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336f109d, 33 },
    { 0xbb8f109e, 76 },
    { 0x103f109f, 76 },
    { 0x10cf109e, 67 },
    { 0x2cdf10a1, 37 },
    { 0, 0 },
    { 0x573f10a3, 90 },
    { 0, 0 },
    { 0xd57f10a5, 23 },
    { 0, 0 },
    { 0x74df10a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790f10ab, 98 },
    { 0, 0 },
    { 0x7dcf10ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x232f10b0, 90 },
    { 0, 0 },
    { 0xf9bf10b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f10b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53f10be, 95 },
    { 0xd57f10bf, 93 },
    { 0xb57f10bf, 70 },
    { 0, 0 },
    { 0x3c4f10c2, 61 },
    { 0, 0 },
    { 0x42f10c4, 23 },
    { 0x4f10c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415f10cd, 68 },
    { 0, 0 },
    { 0xcf9f10cf, 46 },
    { 0, 0 },
    { 0x423f10d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55f10d6, 55 },
    { 0x438f10d7, 87 },
    { 0x113f10d6, 95 },
    { 0x2b2f10d9, 95 },
    { 0xc1ff10d9, 61 },
    { 0xdd7f10db, 61 },
    { 0, 0 },
    { 0xd70f10dd, 33 },
    { 0, 0 },
    { 0x37cf10df, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f10e3, 33 },
    { 0xbbbf10e3, 95 },
    { 0, 0 },
    { 0x5b4f10e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bf10ed, 23 },
    { 0x416f10ee, 91 },
    { 0, 0 },
    { 0xcbbf10f0, 90 },
    { 0xc0bf10f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f10f7, 98 },
    { 0x106f10f7, 33 },
    { 0xf78f10f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f10fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28f1104, 43 },
    { 0, 0 },
    { 0xf14f1106, 98 },
    { 0x8df1106, 34 },
    { 0, 0 },
    { 0x430f1109, 76 },
    { 0x768f110a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6f110e, 61 },
    { 0x39ff110e, 90 },
    { 0, 0 },
    { 0x27cf1111, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94f1115, 93 },
    { 0xd8ff1116, 100 },
    { 0x530f1117, 33 },
    { 0, 0 },
    { 0x371f1119, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddf1122, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7f112a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9df1133, 98 },
    { 0x9e3f1133, 68 },
    { 0, 0 },
    { 0x8e6f1136, 90 },
    { 0xac7f1137, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73f113b, 81 },
    { 0xcaef113c, 52 },
    { 0xc0cf113d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1140, 92 },
    { 0xc06f1141, 98 },
    { 0, 0 },
    { 0x508f1143, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f1149, 90 },
    { 0x2fff1149, 91 },
    { 0xe6bf1149, 23 },
    { 0x91bf114c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bf1153, 78 },
    { 0, 0 },
    { 0x4d0f1155, 100 },
    { 0x85ff1156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x828f1159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf1163, 83 },
    { 0, 0 },
    { 0xf05f1165, 55 },
    { 0, 0 },
    { 0xd4ef1167, 95 },
    { 0, 0 },
    { 0x2e1f1169, 90 },
    { 0, 0 },
    { 0x819f116b, 43 },
    { 0x732f116b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27f116f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabf1173, 55 },
    { 0, 0 },
    { 0x2caf1175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770f1179, 90 },
    { 0, 0 },
    { 0xa31f117b, 23 },
    { 0, 0 },
    { 0xecdf117d, 76 },
    { 0x74df117d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6f1181, 23 },
    { 0xc38f1182, 77 },
    { 0x5aff1181, 66 },
    { 0x43cf1183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53af1187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eff118b, 76 },
    { 0x90df118c, 46 },
    { 0x45f118c, 61 },
    { 0x50cf118e, 23 },
    { 0xe54f118d, 95 },
    { 0xffdf1190, 95 },
    { 0x50bf1191, 96 },
    { 0, 0 },
    { 0xd66f1193, 65 },
    { 0, 0 },
    { 0xc72f1195, 23 },
    { 0xc08f1196, 61 },
    { 0, 0 },
    { 0x1acf1198, 61 },
    { 0x6d3f1198, 98 },
    { 0, 0 },
    { 0x1f2f119b, 23 },
    { 0xbadf119b, 55 },
    { 0x6a3f119d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf11a6, 95 },
    { 0, 0 },
    { 0x2f7f11a8, 55 },
    { 0, 0 },
    { 0x92af11aa, 100 },
    { 0x706f11ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0df11ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef11b5, 46 },
    { 0xf2ff11b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2f11b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff11bc, 61 },
    { 0, 0 },
    { 0x49ff11be, 93 },
    { 0x9c8f11be, 85 },
    { 0x646f11c0, 90 },
    { 0x343f11c1, 23 },
    { 0xed5f11bf, 95 },
    { 0xa7ef11c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecf11c6, 95 },
    { 0xc22f11c7, 87 },
    { 0xddf11c8, 80 },
    { 0, 0 },
    { 0xfc5f11ca, 33 },
    { 0xbbdf11cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681f11d1, 52 },
    { 0x7b3f11d1, 53 },
    { 0x244f11d3, 34 },
    { 0x62f11d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7f11d9, 95 },
    { 0x4daf11da, 78 },
    { 0, 0 },
    { 0x959f11dc, 55 },
    { 0, 0 },
    { 0x149f11de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff11e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23f11e4, 33 },
    { 0x1f2f11e5, 52 },
    { 0, 0 },
    { 0x13f11e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ef11ec, 33 },
    { 0xc2df11ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52df11f1, 76 },
    { 0x7cf11f2, 90 },
    { 0x9baf11f3, 91 },
    { 0x6b9f11f2, 61 },
    { 0xf49f11f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f11f8, 98 },
    { 0xf31f11f9, 93 },
    { 0xba6f11fa, 66 },
    { 0x1f0f11fb, 95 },
    { 0x6c6f11f9, 98 },
    { 0x75ff11fd, 77 },
    { 0xeacf11fe, 95 },
    { 0, 0 },
    { 0x2a1f1200, 33 },
    { 0xde3f1201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2f1204, 90 },
    { 0x485f1204, 98 },
    { 0xc33f1206, 78 },
    { 0x784f1207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf120e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914f1212, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3f1216, 33 },
    { 0x647f1217, 90 },
    { 0xe11f1217, 98 },
    { 0, 0 },
    { 0x536f121a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427f1221, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ff1224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf1227, 52 },
    { 0x784f1228, 66 },
    { 0x531f1227, 33 },
    { 0x33af122a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66f122d, 52 },
    { 0x665f122e, 61 },
    { 0x21af122f, 98 },
    { 0x1ccf122f, 91 },
    { 0x15ef1230, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df123a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df123f, 23 },
    { 0x75df1240, 55 },
    { 0x728f1241, 66 },
    { 0xfb4f1242, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21af124d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20f1253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9df1259, 87 },
    { 0, 0 },
    { 0xebff125b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acf125e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255f1262, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f126c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6f126f, 40 },
    { 0xe86f1270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef1275, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314f127b, 90 },
    { 0, 0 },
    { 0x85ff127d, 100 },
    { 0x2ccf127e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f1281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df1285, 53 },
    { 0x51cf1285, 93 },
    { 0x340f1287, 76 },
    { 0x175f1287, 23 },
    { 0x2abf1289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7f1292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x554f1295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef1298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f129c, 46 },
    { 0xd66f129d, 33 },
    { 0x8adf129e, 95 },
    { 0xd80f129c, 55 },
    { 0x7a3f12a0, 95 },
    { 0x5f3f129c, 95 },
    { 0, 0 },
    { 0x514f12a3, 76 },
    { 0x88ff12a4, 76 },
    { 0x756f12a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f12a8, 99 },
    { 0xe2af12a9, 98 },
    { 0xc53f12aa, 87 },
    { 0xef1f12a9, 90 },
    { 0, 0 },
    { 0x108f12ad, 90 },
    { 0xa86f12ad, 27 },
    { 0x23df12ae, 55 },
    { 0xaeff12b0, 61 },
    { 0x26df12b1, 84 },
    { 0xe74f12b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f12b5, 87 },
    { 0xc54f12b6, 76 },
    { 0x3b4f12b7, 23 },
    { 0x865f12b5, 34 },
    { 0xfc9f12b7, 46 },
    { 0x998f12b7, 90 },
    { 0xf34f12b7, 100 },
    { 0x707f12b5, 46 },
    { 0xdaff12bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdf12c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f12c5, 95 },
    { 0, 0 },
    { 0xd8cf12c7, 98 },
    { 0, 0 },
    { 0xc17f12c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf12cc, 90 },
    { 0, 0 },
    { 0x1b8f12ce, 33 },
    { 0x3e4f12cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaef12d3, 33 },
    { 0x881f12d4, 23 },
    { 0, 0 },
    { 0x53f12d6, 90 },
    { 0, 0 },
    { 0x503f12d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af12db, 100 },
    { 0xeb8f12db, 78 },
    { 0x8a1f12dc, 98 },
    { 0x8b1f12de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaf12e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802f12ef, 90 },
    { 0, 0 },
    { 0x3f9f12f1, 61 },
    { 0xb27f12f2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf12fa, 61 },
    { 0x464f12fb, 99 },
    { 0, 0 },
    { 0xce6f12fd, 33 },
    { 0, 0 },
    { 0x5bcf12ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f1306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992f130a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77f130e, 95 },
    { 0xdd5f130f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf1313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x241f1316, 33 },
    { 0, 0 },
    { 0xcb9f1318, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8f131f, 61 },
    { 0x81df1320, 34 },
    { 0x8fef131f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46f1328, 34 },
    { 0xf6bf1328, 43 },
    { 0xc9ef132a, 61 },
    { 0x914f1328, 76 },
    { 0xdc1f132c, 93 },
    { 0xe16f132c, 90 },
    { 0xbf2f132b, 93 },
    { 0, 0 },
    { 0xc0bf1330, 60 },
    { 0, 0 },
    { 0x3baf1332, 87 },
    { 0xaa0f1332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5f133b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f1341, 23 },
    { 0x33ff1342, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593f1346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf134b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f1350, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x151f1353, 33 },
    { 0x958f1354, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f1357, 99 },
    { 0x65cf1358, 33 },
    { 0xf36f1359, 43 },
    { 0xe14f135a, 35 },
    { 0x38cf135b, 90 },
    { 0x99bf135c, 33 },
    { 0x8b8f135d, 23 },
    { 0x813f135e, 85 },
    { 0, 0 },
    { 0x78cf1360, 81 },
    { 0x66bf1361, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779f1365, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bf1369, 48 },
    { 0, 0 },
    { 0x941f136b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ff1375, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f137a, 95 },
    { 0xd2f137b, 33 },
    { 0xc86f137c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48f137f, 88 },
    { 0, 0 },
    { 0xd85f1381, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1384, 17 },
    { 0xc01f1385, 23 },
    { 0x4e1f1386, 95 },
    { 0xe4ef1387, 93 },
    { 0, 0 },
    { 0x976f1389, 55 },
    { 0, 0 },
    { 0x58bf138b, 55 },
    { 0xf80f138c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf1391, 98 },
    { 0, 0 },
    { 0x654f1393, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x560f1396, 33 },
    { 0xcecf1397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1f139c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf139f, 95 },
    { 0, 0 },
    { 0x224f13a1, 90 },
    { 0x685f13a2, 95 },
    { 0xa98f13a1, 33 },
    { 0x2eef13a2, 61 },
    { 0x710f13a5, 90 },
    { 0xcc2f13a6, 87 },
    { 0, 0 },
    { 0xa46f13a8, 90 },
    { 0, 0 },
    { 0x647f13aa, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9f13ad, 46 },
    { 0x22cf13ad, 95 },
    { 0, 0 },
    { 0xba3f13b0, 52 },
    { 0xa73f13b1, 67 },
    { 0x17bf13b2, 87 },
    { 0x870f13b2, 90 },
    { 0x83ff13b2, 90 },
    { 0xd19f13b5, 43 },
    { 0xa93f13b6, 56 },
    { 0xd60f13b5, 85 },
    { 0x2c0f13b3, 99 },
    { 0x9a2f13b8, 76 },
    { 0x6d1f13b8, 98 },
    { 0xe43f13ba, 65 },
    { 0xfcaf13bc, 76 },
    { 0x98f13b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f13c2, 55 },
    { 0x84af13c3, 34 },
    { 0, 0 },
    { 0xa10f13c5, 90 },
    { 0xd68f13c6, 61 },
    { 0x9e6f13c6, 90 },
    { 0x25af13c8, 23 },
    { 0, 0 },
    { 0x12cf13ca, 33 },
    { 0xd6df13cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4af13d0, 34 },
    { 0x170f13d1, 23 },
    { 0xa4ff13d2, 23 },
    { 0x4f4f13d3, 97 },
    { 0, 0 },
    { 0x284f13d5, 23 },
    { 0x12ef13d6, 76 },
    { 0, 0 },
    { 0x36cf13d8, 33 },
    { 0, 0 },
    { 0x6c4f13da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2f13e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff13e3, 46 },
    { 0xe83f13e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x338f13e7, 99 },
    { 0x87cf13e8, 99 },
    { 0xd13f13e9, 34 },
    { 0xc19f13e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff13f0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf13f3, 95 },
    { 0x360f13f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bf13f7, 43 },
    { 0x169f13f8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f13fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f1401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f1404, 76 },
    { 0x75af1404, 17 },
    { 0xd92f1405, 33 },
    { 0x2d8f1405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7f140e, 61 },
    { 0xb64f140e, 91 },
    { 0xb1ff140e, 33 },
    { 0xf94f1410, 91 },
    { 0, 0 },
    { 0x405f1413, 95 },
    { 0xc3ef1413, 87 },
    { 0x8b5f1415, 33 },
    { 0xf8ff1415, 90 },
    { 0xccaf1417, 61 },
    { 0x36af1415, 52 },
    { 0x7f5f1414, 78 },
    { 0xbbef141a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x154f141d, 55 },
    { 0xb2f141e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af1422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x814f1425, 95 },
    { 0xa8af1426, 40 },
    { 0xb1ff1427, 33 },
    { 0, 0 },
    { 0x895f1429, 43 },
    { 0, 0 },
    { 0xc93f142b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f1434, 90 },
    { 0x140f1435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x207f1438, 55 },
    { 0x3aaf1438, 55 },
    { 0, 0 },
    { 0x336f143b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f143e, 23 },
    { 0, 0 },
    { 0x5ddf1440, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f1447, 23 },
    { 0, 0 },
    { 0x441f1449, 78 },
    { 0xb48f1449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f144d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff1452, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6f1455, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ff1459, 98 },
    { 0xf9ef145a, 91 },
    { 0xf0cf145b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf145f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf1465, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df1469, 33 },
    { 0x36df146a, 23 },
    { 0xac4f146a, 86 },
    { 0xf25f1469, 90 },
    { 0x8e5f146b, 61 },
    { 0, 0 },
    { 0xf26f146f, 33 },
    { 0x866f1470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f1473, 66 },
    { 0x1d5f1474, 90 },
    { 0x274f1475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x763f1478, 98 },
    { 0x41df1479, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12f147c, 33 },
    { 0, 0 },
    { 0x44bf147e, 87 },
    { 0xee9f147f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faf1488, 98 },
    { 0, 0 },
    { 0xb55f148a, 91 },
    { 0x76df148b, 61 },
    { 0xc21f148b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dff148f, 98 },
    { 0x432f1490, 76 },
    { 0xed7f1491, 76 },
    { 0, 0 },
    { 0xb8ff1493, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f1498, 80 },
    { 0x662f1498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af149c, 77 },
    { 0xe2ff149d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8af14a0, 85 },
    { 0x988f14a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915f14a6, 23 },
    { 0x55cf14a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f14ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12f14b5, 47 },
    { 0, 0 },
    { 0xb84f14b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ef14ba, 90 },
    { 0x610f14ba, 90 },
    { 0xe4ff14bc, 61 },
    { 0x159f14bd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x611f14c0, 77 },
    { 0x435f14c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdf14c5, 52 },
    { 0x745f14c5, 90 },
    { 0, 0 },
    { 0x172f14c8, 66 },
    { 0xdb9f14c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdf14cf, 33 },
    { 0x3f0f14d0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f14d9, 34 },
    { 0xfc1f14da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465f14de, 95 },
    { 0x6c1f14de, 33 },
    { 0x252f14e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f14e3, 43 },
    { 0, 0 },
    { 0x205f14e5, 23 },
    { 0, 0 },
    { 0x3b3f14e7, 90 },
    { 0, 0 },
    { 0xcc7f14e9, 90 },
    { 0xea9f14e9, 61 },
    { 0, 0 },
    { 0x67af14ec, 90 },
    { 0, 0 },
    { 0x3f1f14ee, 99 },
    { 0x26cf14ef, 43 },
    { 0xdb9f14ef, 55 },
    { 0x82bf14f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f14fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55df1501, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f1504, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df150a, 33 },
    { 0x30f150b, 98 },
    { 0x682f150b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f150f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f1512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110f1518, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bf151d, 23 },
    { 0xc90f151d, 77 },
    { 0, 0 },
    { 0x2b4f1520, 98 },
    { 0x18ef1520, 33 },
    { 0xbcbf1522, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f1526, 52 },
    { 0x61cf1527, 23 },
    { 0x944f1527, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf152e, 95 },
    { 0x59f152f, 33 },
    { 0x6f9f152f, 90 },
    { 0, 0 },
    { 0xa49f1532, 90 },
    { 0, 0 },
    { 0x25ef1534, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45f153b, 33 },
    { 0, 0 },
    { 0xe4ef153d, 55 },
    { 0, 0 },
    { 0x43f153f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ff1546, 23 },
    { 0, 0 },
    { 0xf40f1548, 55 },
    { 0x866f1548, 23 },
    { 0xba8f154a, 52 },
    { 0x376f154a, 23 },
    { 0x150f154c, 99 },
    { 0x14af154d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef1551, 90 },
    { 0, 0 },
    { 0x957f1553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8df1559, 88 },
    { 0, 0 },
    { 0xd86f155b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbf155e, 52 },
    { 0x7a1f155e, 61 },
    { 0x83f155f, 51 },
    { 0x7e0f1561, 61 },
    { 0xf10f1562, 90 },
    { 0x839f1561, 76 },
    { 0, 0 },
    { 0x57f1565, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f156f, 90 },
    { 0xce1f1570, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5f1573, 90 },
    { 0xf2af1574, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f1579, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef157d, 95 },
    { 0x12cf157e, 23 },
    { 0x16af157e, 90 },
    { 0, 0 },
    { 0xd9bf1581, 33 },
    { 0, 0 },
    { 0x57af1583, 48 },
    { 0x633f1584, 90 },
    { 0x28ef1585, 33 },
    { 0x965f1584, 23 },
    { 0, 0 },
    { 0x821f1588, 33 },
    { 0x6e3f1589, 84 },
    { 0xc79f1589, 33 },
    { 0xa7df158b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00f158e, 33 },
    { 0x5c7f158f, 61 },
    { 0x451f158f, 33 },
    { 0xf7ff1591, 95 },
    { 0x876f1592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f1595, 23 },
    { 0, 0 },
    { 0x2f1597, 61 },
    { 0x176f1598, 23 },
    { 0x24af1598, 52 },
    { 0x1c4f159a, 88 },
    { 0xf68f1598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f159f, 95 },
    { 0xae3f15a0, 55 },
    { 0x88ef15a0, 76 },
    { 0xf6ef15a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x197f15a5, 77 },
    { 0, 0 },
    { 0xa4af15a7, 95 },
    { 0, 0 },
    { 0xf31f15a9, 95 },
    { 0x8a4f15a9, 98 },
    { 0x25ef15a9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7f15b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf15b4, 46 },
    { 0, 0 },
    { 0x233f15b6, 100 },
    { 0x596f15b7, 61 },
    { 0, 0 },
    { 0x81f15b9, 33 },
    { 0x647f15ba, 33 },
    { 0, 0 },
    { 0x3fff15bc, 33 },
    { 0x3e5f15bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af15c3, 66 },
    { 0xd77f15c3, 61 },
    { 0xf0af15c3, 23 },
    { 0x724f15c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667f15cc, 46 },
    { 0x149f15cc, 23 },
    { 0x2aff15ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf15d4, 34 },
    { 0x15af15d5, 80 },
    { 0xba7f15d6, 90 },
    { 0xeeef15d5, 56 },
    { 0xaf6f15d8, 55 },
    { 0xbdcf15d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f15e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddf15e6, 61 },
    { 0x165f15e7, 90 },
    { 0x93bf15e8, 90 },
    { 0x449f15e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f15ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f15f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f15fc, 23 },
    { 0, 0 },
    { 0xb62f15fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f1601, 23 },
    { 0x59ef1602, 67 },
    { 0, 0 },
    { 0xc98f1604, 43 },
    { 0xcbcf1605, 61 },
    { 0, 0 },
    { 0xafdf1607, 66 },
    { 0xe98f1608, 43 },
    { 0xa16f1609, 33 },
    { 0x5a3f1608, 34 },
    { 0, 0 },
    { 0x596f160c, 90 },
    { 0x798f160d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaf1610, 33 },
    { 0x3f1610, 95 },
    { 0, 0 },
    { 0x58ff1613, 33 },
    { 0xf6cf1613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ff161d, 90 },
    { 0x17ff161e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef1624, 65 },
    { 0, 0 },
    { 0x9a3f1626, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f162a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39f162f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5af1632, 93 },
    { 0xb83f1633, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f1637, 90 },
    { 0, 0 },
    { 0x868f1639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f1642, 61 },
    { 0x1cff1643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9f1648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f164c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f164f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139f1655, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51f1658, 98 },
    { 0x93bf1659, 84 },
    { 0xb16f1658, 23 },
    { 0, 0 },
    { 0x8f6f165c, 90 },
    { 0x3c8f165d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebf1664, 95 },
    { 0x810f1665, 91 },
    { 0x9e2f1665, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6f166e, 19 },
    { 0x34f166f, 87 },
    { 0, 0 },
    { 0x32f1671, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x471f1674, 84 },
    { 0, 0 },
    { 0x88ff1676, 61 },
    { 0xa2df1677, 84 },
    { 0xc9cf1677, 91 },
    { 0xa23f1679, 98 },
    { 0, 0 },
    { 0xa52f167b, 99 },
    { 0, 0 },
    { 0xceff167d, 98 },
    { 0, 0 },
    { 0xbdff167f, 61 },
    { 0xb39f1680, 95 },
    { 0, 0 },
    { 0x6fcf1682, 78 },
    { 0x8ecf1682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139f1689, 99 },
    { 0x32f168a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93df1691, 98 },
    { 0x87bf1692, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf169c, 34 },
    { 0x344f169d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800f16a5, 90 },
    { 0, 0 },
    { 0x12ef16a7, 78 },
    { 0, 0 },
    { 0x52cf16a9, 76 },
    { 0x7e3f16aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972f16af, 88 },
    { 0xf5ef16b0, 95 },
    { 0xe83f16af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bf16b6, 93 },
    { 0x6ef16b7, 95 },
    { 0xa1bf16b8, 90 },
    { 0x120f16b8, 98 },
    { 0x4c5f16ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47f16bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3f16c1, 33 },
    { 0x42df16c1, 43 },
    { 0x861f16c1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef16c7, 98 },
    { 0x79bf16c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf16d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5f16d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f16d6, 61 },
    { 0x2e8f16d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbf16e7, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f16ed, 76 },
    { 0, 0 },
    { 0x3daf16ef, 67 },
    { 0, 0 },
    { 0xce0f16f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef16fb, 23 },
    { 0x5ecf16fc, 76 },
    { 0x572f16fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767f1703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42af1707, 90 },
    { 0x4d4f1708, 23 },
    { 0xaf7f1709, 76 },
    { 0x9c4f1709, 90 },
    { 0, 0 },
    { 0xfa0f170c, 61 },
    { 0xddf170d, 98 },
    { 0, 0 },
    { 0x7cbf170f, 99 },
    { 0, 0 },
    { 0x75af1711, 23 },
    { 0x12df1711, 95 },
    { 0x7d9f1712, 77 },
    { 0x544f1714, 76 },
    { 0x3b4f1715, 68 },
    { 0x496f1715, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef171a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f1721, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f1727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815f172b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcff172e, 92 },
    { 0x2acf172f, 55 },
    { 0x302f172f, 98 },
    { 0x806f172e, 67 },
    { 0xd1f1732, 17 },
    { 0x24df1733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7f1736, 33 },
    { 0x754f1737, 91 },
    { 0x103f1738, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333f173e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f1741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f1745, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bf1748, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0f174b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef1752, 55 },
    { 0xd03f1753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f1756, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf175f, 97 },
    { 0xb44f1760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf1766, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f176a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f1775, 90 },
    { 0x1dff1776, 93 },
    { 0xc83f1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6f177b, 46 },
    { 0x192f177c, 98 },
    { 0, 0 },
    { 0xa52f177e, 75 },
    { 0x8bff177f, 78 },
    { 0, 0 },
    { 0x499f1781, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5f1789, 61 },
    { 0x145f1789, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43df178d, 78 },
    { 0x44f178e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259f1795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf179a, 23 },
    { 0x57ef179b, 95 },
    { 0x345f179a, 23 },
    { 0x22af179a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70f17a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28f17a3, 61 },
    { 0, 0 },
    { 0xc6ff17a5, 23 },
    { 0, 0 },
    { 0xc61f17a7, 33 },
    { 0, 0 },
    { 0xdb0f17a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f17ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cf17b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64f17b7, 46 },
    { 0x29ef17b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f17c0, 78 },
    { 0x46bf17c1, 46 },
    { 0xe82f17c2, 84 },
    { 0x74af17c3, 23 },
    { 0x991f17c3, 38 },
    { 0, 0 },
    { 0x18bf17c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88f17c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22f17cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4f17d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf17d5, 90 },
    { 0, 0 },
    { 0x8f1f17d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4f17dc, 90 },
    { 0xb19f17dc, 33 },
    { 0, 0 },
    { 0xf53f17df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef17e3, 68 },
    { 0x14cf17e4, 55 },
    { 0xdcf17e5, 33 },
    { 0xdadf17e3, 90 },
    { 0xcfef17e5, 33 },
    { 0, 0 },
    { 0x710f17e9, 98 },
    { 0x344f17ea, 66 },
    { 0x5e5f17eb, 90 },
    { 0x8b3f17ea, 77 },
    { 0xa37f17ed, 43 },
    { 0x114f17ea, 100 },
    { 0xeacf17ef, 33 },
    { 0, 0 },
    { 0x7a0f17f1, 33 },
    { 0x55f17f2, 17 },
    { 0xc9af17f2, 93 },
    { 0xad7f17f4, 33 },
    { 0x11bf17f4, 76 },
    { 0x2e5f17f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8f17fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f17ff, 87 },
    { 0x882f1800, 34 },
    { 0x4a0f1800, 98 },
    { 0x631f17ff, 87 },
    { 0, 0 },
    { 0x5bdf1804, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601f1809, 87 },
    { 0x344f180a, 61 },
    { 0xac0f180a, 34 },
    { 0xe49f180c, 34 },
    { 0x85df180b, 61 },
    { 0x2d3f180e, 33 },
    { 0xcb0f180f, 90 },
    { 0x90ef180c, 78 },
    { 0xf65f1811, 43 },
    { 0x48af1812, 66 },
    { 0x271f1813, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7f1816, 98 },
    { 0, 0 },
    { 0xc79f1818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44f181d, 33 },
    { 0x16df181e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2f1821, 55 },
    { 0x656f1821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3f1827, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bff182c, 90 },
    { 0xc4bf182d, 90 },
    { 0x994f182e, 90 },
    { 0x8b4f182f, 33 },
    { 0x4f1f182e, 33 },
    { 0xf97f1831, 23 },
    { 0x894f1831, 91 },
    { 0x2dcf182d, 43 },
    { 0xd6df1834, 76 },
    { 0x37cf1835, 46 },
    { 0x15ff1836, 81 },
    { 0x5cff1837, 95 },
    { 0x4a5f1833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x462f183b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424f1845, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ef1849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5f184d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f1852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df1858, 91 },
    { 0, 0 },
    { 0xc39f185a, 78 },
    { 0, 0 },
    { 0x9ddf185c, 60 },
    { 0xc55f185d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363f1861, 64 },
    { 0, 0 },
    { 0xa98f1863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf186b, 33 },
    { 0, 0 },
    { 0x48cf186d, 66 },
    { 0, 0 },
    { 0xd13f186f, 33 },
    { 0x3b2f186f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68f1875, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2f1879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf187d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af1881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f1885, 34 },
    { 0xa46f1885, 66 },
    { 0xc8df1887, 23 },
    { 0, 0 },
    { 0xeeff1889, 76 },
    { 0xa1f1889, 33 },
    { 0xac3f1889, 46 },
    { 0x99f1889, 98 },
    { 0x30af188d, 90 },
    { 0xb49f188d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x388f1891, 92 },
    { 0xd46f1892, 90 },
    { 0x620f1893, 90 },
    { 0x29cf1894, 23 },
    { 0xec9f1893, 90 },
    { 0xfcff1894, 34 },
    { 0xc29f1897, 85 },
    { 0x34ef1896, 91 },
    { 0x8abf1894, 55 },
    { 0x912f189a, 87 },
    { 0xf17f1897, 90 },
    { 0xd15f1893, 67 },
    { 0xb34f1892, 93 },
    { 0x1a7f1893, 95 },
    { 0x669f189f, 33 },
    { 0xe64f189a, 90 },
    { 0, 0 },
    { 0x881f18a2, 85 },
    { 0xe65f18a2, 55 },
    { 0x3a7f18a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff18aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0f18b0, 88 },
    { 0x7b1f18b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14f18b5, 85 },
    { 0x597f18b6, 87 },
    { 0xe91f18b6, 78 },
    { 0xe61f18b8, 33 },
    { 0x21ff18b7, 55 },
    { 0x7ccf18b9, 61 },
    { 0x571f18bb, 55 },
    { 0x2cef18b7, 90 },
    { 0x694f18bd, 33 },
    { 0x457f18be, 90 },
    { 0xf2df18bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67f18c4, 95 },
    { 0, 0 },
    { 0x3b6f18c6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f18ca, 17 },
    { 0xbf18cb, 66 },
    { 0, 0 },
    { 0x6f4f18cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06f18d3, 33 },
    { 0xa33f18d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859f18db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf18e1, 67 },
    { 0, 0 },
    { 0x424f18e3, 34 },
    { 0x385f18e3, 33 },
    { 0, 0 },
    { 0x62ef18e6, 23 },
    { 0xfd1f18e7, 23 },
    { 0xa18f18e8, 98 },
    { 0x54ef18e7, 66 },
    { 0xcf8f18ea, 65 },
    { 0xc09f18e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1f18f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1f18fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf1900, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86af1905, 97 },
    { 0x298f1906, 98 },
    { 0x979f1907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9f190f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30f1912, 33 },
    { 0, 0 },
    { 0x482f1914, 69 },
    { 0xeb5f1915, 76 },
    { 0, 0 },
    { 0xf0f1917, 55 },
    { 0, 0 },
    { 0x479f1919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eff191d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af1922, 46 },
    { 0xb85f1923, 63 },
    { 0, 0 },
    { 0xff2f1925, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62f192a, 76 },
    { 0xcbbf192a, 33 },
    { 0, 0 },
    { 0xe5df192d, 98 },
    { 0x632f192e, 68 },
    { 0, 0 },
    { 0x8acf1930, 61 },
    { 0x824f1930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f1934, 61 },
    { 0x7def1935, 23 },
    { 0xfc7f1936, 34 },
    { 0x55df1937, 34 },
    { 0xbb6f1938, 23 },
    { 0x6a4f1937, 34 },
    { 0xee6f193a, 55 },
    { 0x440f1937, 95 },
    { 0x1cf193c, 33 },
    { 0x9c7f193c, 43 },
    { 0, 0 },
    { 0x980f193f, 98 },
    { 0x2c3f193f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1f194a, 52 },
    { 0, 0 },
    { 0x777f194c, 43 },
    { 0x9caf194c, 23 },
    { 0xd93f194e, 100 },
    { 0x975f194d, 90 },
    { 0x1e3f194e, 98 },
    { 0x9aff1951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf1954, 90 },
    { 0xb8af1954, 95 },
    { 0x5aef1956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f1959, 95 },
    { 0, 0 },
    { 0xcaef195b, 43 },
    { 0xea9f195c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f195f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f1964, 77 },
    { 0xd82f1965, 68 },
    { 0xca1f1964, 78 },
    { 0xaacf1967, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef197a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f197d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6f1982, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31f1985, 55 },
    { 0, 0 },
    { 0x304f1987, 34 },
    { 0xa9df1988, 43 },
    { 0x7d7f1989, 95 },
    { 0x7ef198a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af198d, 17 },
    { 0x506f198e, 76 },
    { 0xbb7f198f, 61 },
    { 0xc72f1990, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f1995, 61 },
    { 0, 0 },
    { 0x920f1997, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf19a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587f19b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6f19b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eef19bb, 88 },
    { 0, 0 },
    { 0xa06f19bd, 66 },
    { 0x4ebf19bd, 84 },
    { 0x73f19bd, 61 },
    { 0x27bf19c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1f19c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddf19cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f19ce, 23 },
    { 0x9d0f19ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84f19d8, 78 },
    { 0xe33f19d8, 61 },
    { 0xc7ff19d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f19df, 78 },
    { 0xe25f19e0, 98 },
    { 0x91cf19e1, 23 },
    { 0, 0 },
    { 0xaeaf19e3, 95 },
    { 0xe1af19e4, 93 },
    { 0xb98f19e5, 90 },
    { 0xcb4f19e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff19ea, 87 },
    { 0xde5f19eb, 55 },
    { 0x3e7f19eb, 84 },
    { 0, 0 },
    { 0x137f19ee, 76 },
    { 0, 0 },
    { 0x4d4f19f0, 95 },
    { 0xb82f19f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994f19f6, 99 },
    { 0, 0 },
    { 0x6b1f19f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1f19fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef1a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f1a05, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f1a0f, 33 },
    { 0x77cf1a10, 43 },
    { 0xc91f1a11, 98 },
    { 0xf29f1a0f, 61 },
    { 0, 0 },
    { 0xe67f1a14, 76 },
    { 0x11ff1a15, 52 },
    { 0x52af1a15, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf1a19, 99 },
    { 0xc32f1a1a, 90 },
    { 0x960f1a1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352f1a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71f1a2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637f1a2e, 90 },
    { 0, 0 },
    { 0x22f1a30, 36 },
    { 0, 0 },
    { 0xd7ef1a32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f1a37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822f1a3c, 98 },
    { 0xa79f1a3c, 90 },
    { 0xfadf1a3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edf1a41, 52 },
    { 0xc3cf1a42, 93 },
    { 0x21f1a41, 2 },
    { 0xdcbf1a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39f1a4a, 98 },
    { 0xef7f1a4b, 33 },
    { 0xb3bf1a4c, 98 },
    { 0x40ef1a4d, 55 },
    { 0x5e6f1a4e, 36 },
    { 0x29af1a4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f1a55, 81 },
    { 0xb28f1a56, 90 },
    { 0, 0 },
    { 0x173f1a58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282f1a5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94f1a61, 76 },
    { 0, 0 },
    { 0xf12f1a63, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463f1a69, 66 },
    { 0, 0 },
    { 0x1f8f1a6b, 52 },
    { 0xad4f1a6b, 81 },
    { 0xa2cf1a6c, 33 },
    { 0xe3f1a6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f1a71, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f1a75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1f1a78, 43 },
    { 0x74af1a79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f1a7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f1a82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7f1a88, 85 },
    { 0, 0 },
    { 0x2abf1a8a, 95 },
    { 0x6a9f1a8b, 99 },
    { 0x518f1a8b, 98 },
    { 0x4c3f1a8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf1a91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f1a94, 23 },
    { 0xc54f1a94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499f1a9a, 33 },
    { 0xe69f1a9b, 90 },
    { 0xcc4f1a9c, 55 },
    { 0x11f1a9a, 33 },
    { 0, 0 },
    { 0x803f1a9f, 40 },
    { 0x629f1aa0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5f1aa9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f1aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245f1ab4, 53 },
    { 0x3e3f1ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62f1aba, 33 },
    { 0x5d7f1aba, 34 },
    { 0x613f1abc, 34 },
    { 0x947f1abd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f1ac0, 100 },
    { 0x3ecf1ac1, 53 },
    { 0x9ebf1ac1, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100f1aca, 98 },
    { 0x7b1f1acb, 66 },
    { 0x5c6f1acb, 90 },
    { 0x6c0f1aca, 65 },
    { 0x49ff1acc, 98 },
    { 0x4a6f1acd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x983f1ad2, 98 },
    { 0x912f1ad3, 68 },
    { 0, 0 },
    { 0x9c1f1ad5, 34 },
    { 0x313f1ad5, 61 },
    { 0, 0 },
    { 0xfd4f1ad8, 90 },
    { 0x3a3f1ad9, 23 },
    { 0x7bbf1ada, 33 },
    { 0xf1adb, 55 },
    { 0x308f1adb, 66 },
    { 0x74ff1add, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x35df1ae0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ef1ae6, 48 },
    { 0x1ecf1ae7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf1af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4af1af7, 81 },
    { 0xd03f1af7, 23 },
    { 0xa0cf1af9, 52 },
    { 0, 0 },
    { 0xce3f1afb, 61 },
    { 0x736f1afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdef1aff, 78 },
    { 0x784f1b00, 33 },
    { 0, 0 },
    { 0x9d3f1b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f1b05, 95 },
    { 0x23bf1b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff1b0b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df1b10, 76 },
    { 0x932f1b10, 61 },
    { 0, 0 },
    { 0x70df1b13, 23 },
    { 0x184f1b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f1b1c, 99 },
    { 0x6ecf1b1c, 90 },
    { 0xbc2f1b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820f1b27, 81 },
    { 0x96bf1b28, 33 },
    { 0xe7ef1b29, 76 },
    { 0x8d7f1b2a, 33 },
    { 0xf35f1b29, 25 },
    { 0x6c7f1b2b, 84 },
    { 0x9bbf1b2c, 46 },
    { 0xacdf1b2e, 67 },
    { 0xd16f1b2e, 53 },
    { 0xb1df1b30, 98 },
    { 0x466f1b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f1b35, 87 },
    { 0x368f1b36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05f1b3d, 93 },
    { 0, 0 },
    { 0x8b7f1b3f, 76 },
    { 0, 0 },
    { 0x4ddf1b41, 61 },
    { 0x4eaf1b41, 33 },
    { 0x5df1b41, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf1b46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57f1b49, 46 },
    { 0, 0 },
    { 0x87ef1b4b, 97 },
    { 0x6e9f1b4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04f1b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4f1b52, 78 },
    { 0x718f1b53, 100 },
    { 0xa28f1b52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf1b57, 33 },
    { 0x8f6f1b58, 90 },
    { 0xf87f1b57, 43 },
    { 0xf53f1b5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f1b5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x855f1b62, 98 },
    { 0xaf2f1b63, 43 },
    { 0xbaef1b62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf1b6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef1b72, 66 },
    { 0x5f5f1b73, 78 },
    { 0xa11f1b74, 85 },
    { 0, 0 },
    { 0xc4af1b76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x518f1b79, 98 },
    { 0xf4af1b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x438f1b7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf1b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f1b84, 66 },
    { 0x81af1b85, 87 },
    { 0x6baf1b84, 61 },
    { 0x43f1b87, 43 },
    { 0, 0 },
    { 0x9def1b89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64df1b91, 90 },
    { 0x890f1b92, 23 },
    { 0x98ff1b93, 23 },
    { 0x406f1b92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecff1b97, 90 },
    { 0x4bbf1b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68f1ba2, 46 },
    { 0, 0 },
    { 0xf5cf1ba4, 61 },
    { 0, 0 },
    { 0xebef1ba6, 33 },
    { 0, 0 },
    { 0xc1af1ba8, 84 },
    { 0xcbaf1ba9, 90 },
    { 0, 0 },
    { 0x179f1bab, 23 },
    { 0x5b5f1bac, 43 },
    { 0, 0 },
    { 0x1fef1bae, 53 },
    { 0x3d3f1baf, 90 },
    { 0xda3f1bb0, 61 },
    { 0x7b5f1baf, 61 },
    { 0, 0 },
    { 0xed6f1bb3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f1bbe, 33 },
    { 0x1f0f1bbe, 52 },
    { 0, 0 },
    { 0x420f1bc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f1bcc, 61 },
    { 0xeb3f1bcd, 99 },
    { 0x66df1bcd, 61 },
    { 0xe47f1bcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbf1bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ef1bda, 33 },
    { 0, 0 },
    { 0xf46f1bdc, 33 },
    { 0x94af1bdc, 77 },
    { 0x4acf1bde, 76 },
    { 0xb80f1bdf, 66 },
    { 0xb20f1bde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df1be5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0f1be9, 95 },
    { 0x27bf1bea, 75 },
    { 0xc24f1be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f1bf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf1bf6, 34 },
    { 0, 0 },
    { 0x3dbf1bf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf1bfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13f1bff, 98 },
    { 0xeb9f1c00, 91 },
    { 0, 0 },
    { 0xb8ef1c02, 33 },
    { 0x815f1c02, 69 },
    { 0x4d1f1c04, 33 },
    { 0, 0 },
    { 0x2eef1c06, 61 },
    { 0x6e0f1c06, 90 },
    { 0xa97f1c08, 55 },
    { 0x25f1c08, 33 },
    { 0x121f1c0a, 81 },
    { 0x86ff1c0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1c0e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1df1c12, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502f1c16, 98 },
    { 0xd86f1c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15f1c1b, 34 },
    { 0, 0 },
    { 0xf67f1c1d, 98 },
    { 0, 0 },
    { 0xe3bf1c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f1c23, 95 },
    { 0x260f1c24, 85 },
    { 0x818f1c25, 77 },
    { 0x537f1c25, 95 },
    { 0x64bf1c26, 61 },
    { 0xb24f1c28, 61 },
    { 0, 0 },
    { 0xbe6f1c2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf1c2f, 36 },
    { 0xad8f1c30, 95 },
    { 0x75f1c30, 77 },
    { 0xdff1c31, 95 },
    { 0x3fdf1c33, 61 },
    { 0xe4bf1c34, 2 },
    { 0xe12f1c35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x674f1c38, 43 },
    { 0x52f1c39, 85 },
    { 0xd6f1c3a, 43 },
    { 0x707f1c39, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df1c40, 61 },
    { 0, 0 },
    { 0x152f1c42, 43 },
    { 0, 0 },
    { 0x75f1c44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef1c49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf1c4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bef1c53, 33 },
    { 0x7bdf1c54, 61 },
    { 0x683f1c55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x792f1c58, 52 },
    { 0x775f1c59, 84 },
    { 0xbcff1c58, 90 },
    { 0, 0 },
    { 0xee6f1c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0f1c63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53f1c66, 90 },
    { 0x635f1c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271f1c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x764f1c6e, 90 },
    { 0xe12f1c6e, 55 },
    { 0x9f1c70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703f1c76, 66 },
    { 0xc05f1c77, 33 },
    { 0xa44f1c78, 90 },
    { 0xc76f1c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31f1c81, 90 },
    { 0x644f1c82, 65 },
    { 0xa53f1c83, 61 },
    { 0xebcf1c84, 65 },
    { 0x15cf1c83, 100 },
    { 0x342f1c86, 68 },
    { 0xf59f1c86, 23 },
    { 0x732f1c86, 33 },
    { 0xe5cf1c84, 95 },
    { 0xb15f1c86, 98 },
    { 0xd1f1c8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2f1c93, 34 },
    { 0, 0 },
    { 0xc09f1c95, 76 },
    { 0x8b7f1c96, 61 },
    { 0, 0 },
    { 0xf85f1c98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f1c9b, 33 },
    { 0x83df1c9c, 66 },
    { 0x981f1c9d, 33 },
    { 0xdf4f1c9c, 61 },
    { 0xd9bf1c9b, 98 },
    { 0, 0 },
    { 0x315f1ca1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf1ca4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef1cac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff1caf, 51 },
    { 0, 0 },
    { 0x3b6f1cb1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ef1cb7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebf1cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x115f1cc1, 37 },
    { 0xa1f1cc1, 3 },
    { 0xaedf1cc3, 46 },
    { 0x175f1cc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf1cc7, 90 },
    { 0x37ef1cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771f1cd4, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df1cd8, 23 },
    { 0, 0 },
    { 0xbf0f1cda, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64f1cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1f1ce2, 83 },
    { 0xe50f1ce2, 93 },
    { 0xf78f1ce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221f1ceb, 90 },
    { 0, 0 },
    { 0xae2f1ced, 90 },
    { 0x2b8f1cee, 43 },
    { 0, 0 },
    { 0x4f1cf0, 76 },
    { 0xac2f1cf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9f1cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4f1cf9, 77 },
    { 0x453f1cfa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2f1cff, 33 },
    { 0x1d7f1cff, 43 },
    { 0x2f7f1cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x672f1d04, 33 },
    { 0x378f1d05, 33 },
    { 0xb72f1d06, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743f1d0d, 23 },
    { 0, 0 },
    { 0x6ecf1d0f, 61 },
    { 0, 0 },
    { 0x27ff1d11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ef1d1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef1d1d, 81 },
    { 0xf37f1d1e, 95 },
    { 0, 0 },
    { 0x37ef1d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f1d23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x342f1d26, 98 },
    { 0x1eef1d26, 100 },
    { 0x505f1d28, 33 },
    { 0, 0 },
    { 0x6b2f1d2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709f1d2f, 93 },
    { 0, 0 },
    { 0x33ff1d31, 61 },
    { 0x8b6f1d32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965f1d37, 98 },
    { 0x2e8f1d37, 93 },
    { 0xd70f1d39, 33 },
    { 0, 0 },
    { 0x921f1d3b, 90 },
    { 0x58df1d3c, 99 },
    { 0x3af1d3d, 95 },
    { 0xf9df1d3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x93df1d41, 98 },
    { 0, 0 },
    { 0xf8bf1d43, 34 },
    { 0x573f1d43, 69 },
    { 0xc41f1d45, 46 },
    { 0x918f1d45, 23 },
    { 0, 0 },
    { 0x2f4f1d48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef1d4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf1d4f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf1d58, 61 },
    { 0x152f1d58, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681f1d62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef1d65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f1d6a, 33 },
    { 0x232f1d6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f1d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25f1d76, 90 },
    { 0, 0 },
    { 0xc42f1d78, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656f1d7d, 61 },
    { 0x9faf1d7d, 23 },
    { 0, 0 },
    { 0xe34f1d80, 90 },
    { 0, 0 },
    { 0x839f1d82, 91 },
    { 0x33ef1d82, 52 },
    { 0, 0 },
    { 0xf05f1d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf1d88, 66 },
    { 0xc66f1d89, 90 },
    { 0x60af1d89, 38 },
    { 0, 0 },
    { 0xaf7f1d8c, 66 },
    { 0, 0 },
    { 0xf5df1d8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f1d91, 98 },
    { 0x55ef1d92, 85 },
    { 0xf2bf1d93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef1d99, 61 },
    { 0, 0 },
    { 0x771f1d9b, 46 },
    { 0x6cdf1d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f1da2, 23 },
    { 0x3c3f1da3, 95 },
    { 0xa34f1da4, 69 },
    { 0x289f1da5, 91 },
    { 0, 0 },
    { 0xed9f1da7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f1dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff1db0, 61 },
    { 0xdf3f1db1, 95 },
    { 0, 0 },
    { 0x174f1db3, 34 },
    { 0xed9f1db4, 34 },
    { 0xe82f1db3, 90 },
    { 0, 0 },
    { 0x175f1db7, 85 },
    { 0xc11f1db8, 61 },
    { 0x9aff1db9, 90 },
    { 0x99ef1dba, 61 },
    { 0, 0 },
    { 0xb8df1dbc, 55 },
    { 0xe88f1dbd, 66 },
    { 0x116f1dbe, 33 },
    { 0x8ebf1dbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8f1dc5, 84 },
    { 0x4d7f1dc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acf1dc9, 95 },
    { 0x585f1dc9, 90 },
    { 0xcc5f1dcb, 23 },
    { 0, 0 },
    { 0x6d1f1dcd, 33 },
    { 0x49bf1dce, 95 },
    { 0, 0 },
    { 0xc0df1dd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f1dd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f1dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f1ddc, 98 },
    { 0x367f1ddd, 98 },
    { 0x3fdf1dde, 75 },
    { 0, 0 },
    { 0x462f1de0, 78 },
    { 0xffef1de1, 67 },
    { 0x735f1de2, 66 },
    { 0xb2df1de2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f1dee, 87 },
    { 0x863f1def, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf1df7, 98 },
    { 0xe77f1df8, 61 },
    { 0x6fdf1df7, 34 },
    { 0, 0 },
    { 0x99ff1dfb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f1dfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eff1e06, 33 },
    { 0x43af1e06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef1e0e, 61 },
    { 0, 0 },
    { 0x6a8f1e10, 76 },
    { 0xf24f1e11, 90 },
    { 0x38f1e11, 33 },
    { 0, 0 },
    { 0xba7f1e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1f1e1b, 43 },
    { 0x7d4f1e1c, 98 },
    { 0x777f1e1d, 55 },
    { 0x995f1e1e, 33 },
    { 0xf98f1e1d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f1e2b, 68 },
    { 0x2d6f1e2c, 43 },
    { 0x34df1e2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f1e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0f1e38, 77 },
    { 0xd9ef1e39, 33 },
    { 0, 0 },
    { 0x9fff1e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff1e3e, 52 },
    { 0x3d5f1e3f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750f1e46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74f1e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1e52, 67 },
    { 0xdb0f1e53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fef1e59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf1e5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2f1e67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ff1e6c, 23 },
    { 0xf2bf1e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df1e70, 87 },
    { 0x747f1e70, 88 },
    { 0x3b5f1e72, 61 },
    { 0xa63f1e70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5af1e7a, 34 },
    { 0, 0 },
    { 0xf9df1e7c, 55 },
    { 0xa00f1e7d, 43 },
    { 0xd43f1e7e, 34 },
    { 0x888f1e7e, 55 },
    { 0x609f1e80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff1e83, 23 },
    { 0x263f1e84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f1e88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bf1e90, 61 },
    { 0xe32f1e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf1e94, 33 },
    { 0, 0 },
    { 0x844f1e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79f1e9a, 77 },
    { 0x137f1e9b, 55 },
    { 0xca5f1e9c, 76 },
    { 0xe79f1e9d, 33 },
    { 0xef2f1e9d, 98 },
    { 0xb1df1e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af1ea2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29f1ea5, 90 },
    { 0xa24f1ea5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7af1eaa, 67 },
    { 0x20af1eab, 61 },
    { 0xc85f1eab, 95 },
    { 0x1b8f1ead, 34 },
    { 0x8f6f1eae, 46 },
    { 0xee5f1eae, 90 },
    { 0xef6f1ead, 98 },
    { 0x38bf1eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f1eba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf1ebd, 68 },
    { 0xefef1ebd, 61 },
    { 0, 0 },
    { 0x9e7f1ec0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47f1ece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef1ed3, 43 },
    { 0, 0 },
    { 0x218f1ed5, 83 },
    { 0x7dcf1ed6, 33 },
    { 0xfe0f1ed5, 33 },
    { 0x176f1ed8, 33 },
    { 0x3d7f1ed9, 78 },
    { 0x7bff1ed9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f1edd, 61 },
    { 0x440f1ede, 43 },
    { 0, 0 },
    { 0xb8af1ee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363f1ee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48f1ee9, 81 },
    { 0x1edf1eea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf1eed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f1ef7, 52 },
    { 0xa05f1ef7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef1efb, 33 },
    { 0xdf9f1efc, 76 },
    { 0x1c4f1efd, 17 },
    { 0x2fdf1efe, 98 },
    { 0, 0 },
    { 0x68af1f00, 61 },
    { 0xffbf1f01, 61 },
    { 0, 0 },
    { 0x17af1f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbf1f0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f1f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174f1f18, 23 },
    { 0x422f1f19, 61 },
    { 0x5a1f1f1a, 52 },
    { 0x907f1f1b, 33 },
    { 0x771f1f1a, 84 },
    { 0xa6af1f1d, 23 },
    { 0x95df1f1e, 97 },
    { 0x2f4f1f19, 34 },
    { 0x8a1f1f20, 17 },
    { 0xcc5f1f1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190f1f24, 23 },
    { 0x13bf1f25, 90 },
    { 0x1c3f1f26, 61 },
    { 0xd35f1f26, 33 },
    { 0xc86f1f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f1f2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f1f3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff1f3e, 95 },
    { 0xb1df1f3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef1f42, 61 },
    { 0, 0 },
    { 0x896f1f44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f1f49, 67 },
    { 0x454f1f4a, 55 },
    { 0x16df1f4a, 23 },
    { 0x961f1f49, 80 },
    { 0xe82f1f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64af1f51, 90 },
    { 0xcff1f52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f1f58, 95 },
    { 0x6e6f1f59, 98 },
    { 0x77af1f59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1f1f63, 52 },
    { 0x46bf1f63, 99 },
    { 0x88df1f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cf1f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x827f1f6d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cef1f70, 85 },
    { 0x73f1f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff1f75, 95 },
    { 0x9a1f1f75, 75 },
    { 0x7ecf1f77, 55 },
    { 0xd8df1f78, 33 },
    { 0xccef1f78, 98 },
    { 0x25bf1f79, 61 },
    { 0, 0 },
    { 0x1fbf1f7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401f1f80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff1f85, 43 },
    { 0xa3ef1f86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff1f8e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43f1f93, 66 },
    { 0x1f6f1f94, 85 },
    { 0xb03f1f95, 77 },
    { 0xdc3f1f93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f1f9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f1fa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0f1fa5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af1fa9, 100 },
    { 0x536f1faa, 23 },
    { 0xe90f1fab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf1fb5, 38 },
    { 0x53bf1fb5, 23 },
    { 0, 0 },
    { 0xf2af1fb8, 76 },
    { 0xe94f1fb9, 98 },
    { 0x599f1fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f1fbf, 87 },
    { 0x3c6f1fc0, 93 },
    { 0x3bf1fc1, 55 },
    { 0, 0 },
    { 0xfe4f1fc3, 66 },
    { 0, 0 },
    { 0xda9f1fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43f1fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193f1fcc, 84 },
    { 0, 0 },
    { 0x80ff1fce, 61 },
    { 0x57cf1fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acf1fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f1fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f1fe4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9f1feb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f1ffd, 61 },
    { 0x5a1f1ffe, 90 },
    { 0xf03f1ffd, 90 },
    { 0, 0 },
    { 0x65f2001, 90 },
    { 0xae3f2002, 91 },
    { 0xc77f2001, 90 },
    { 0x306f2003, 51 },
    { 0x663f2005, 90 },
    { 0x401f2006, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28f200e, 98 },
    { 0xe7cf200e, 91 },
    { 0, 0 },
    { 0x9d9f2011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9f2014, 90 },
    { 0x9c1f2015, 23 },
    { 0, 0 },
    { 0x4fdf2017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cf201a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff2021, 90 },
    { 0xa35f2022, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecf2029, 40 },
    { 0, 0 },
    { 0x1f3f202b, 33 },
    { 0xcc5f202b, 90 },
    { 0x88ef202d, 23 },
    { 0x57df202c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf2033, 97 },
    { 0xcf7f2034, 98 },
    { 0x5a4f2033, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af2038, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ef203b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abf203e, 76 },
    { 0x8e7f203f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1f204a, 55 },
    { 0xe2ff204b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ef2055, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf2059, 52 },
    { 0xf7af205a, 55 },
    { 0x7bff2059, 33 },
    { 0xe70f205c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf2061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25f2064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f2069, 69 },
    { 0x258f206a, 93 },
    { 0xbaaf206b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaf206e, 95 },
    { 0x415f206f, 61 },
    { 0xf41f2070, 55 },
    { 0x7f1f206e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x757f2074, 88 },
    { 0x3b2f2075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80f2078, 90 },
    { 0, 0 },
    { 0x3cef207a, 33 },
    { 0x9ecf207b, 75 },
    { 0x668f207c, 98 },
    { 0xe6bf207d, 23 },
    { 0xbe5f207e, 23 },
    { 0x405f207c, 91 },
    { 0xb91f207d, 100 },
    { 0xeb4f2080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f2084, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f2087, 61 },
    { 0xc3df2087, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f208c, 46 },
    { 0x722f208d, 100 },
    { 0x6f208e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f2092, 87 },
    { 0, 0 },
    { 0x546f2094, 100 },
    { 0, 0 },
    { 0x648f2096, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329f209d, 61 },
    { 0x1d2f209d, 98 },
    { 0, 0 },
    { 0xdbaf20a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adf20a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874f20ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22f20b4, 90 },
    { 0x7d6f20b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0f20b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ef20c0, 100 },
    { 0xdbf20c1, 23 },
    { 0xa75f20c2, 55 },
    { 0xd51f20c2, 95 },
    { 0, 0 },
    { 0xa0af20c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415f20cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84f20d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f20d4, 61 },
    { 0x6c3f20d5, 93 },
    { 0xbb9f20d6, 68 },
    { 0x63ff20d7, 80 },
    { 0xe7df20d8, 52 },
    { 0xbfdf20d8, 95 },
    { 0x947f20d8, 43 },
    { 0xddbf20db, 100 },
    { 0xaa2f20dc, 84 },
    { 0xcd5f20d7, 39 },
    { 0x210f20d4, 90 },
    { 0xd49f20d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f20e5, 90 },
    { 0x325f20e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f20e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18f20ee, 59 },
    { 0x672f20ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f20f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f20fe, 61 },
    { 0xf3f20ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f2103, 76 },
    { 0, 0 },
    { 0xf3cf2105, 23 },
    { 0x330f2105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fff2110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4af2115, 34 },
    { 0x7aef2115, 95 },
    { 0, 0 },
    { 0xee2f2118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4f211b, 88 },
    { 0x97ff211b, 61 },
    { 0x4cff211d, 90 },
    { 0x2eaf211e, 61 },
    { 0, 0 },
    { 0x12ff2120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df2126, 46 },
    { 0x781f2127, 98 },
    { 0, 0 },
    { 0xa95f2129, 93 },
    { 0xdd6f212a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6af212e, 98 },
    { 0, 0 },
    { 0x3cbf2130, 95 },
    { 0xef0f2130, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x235f2134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09f213a, 61 },
    { 0x827f213b, 90 },
    { 0x6d1f213b, 61 },
    { 0x248f213c, 98 },
    { 0x5ff213e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886f2147, 33 },
    { 0x78bf2147, 55 },
    { 0x9c4f2148, 90 },
    { 0, 0 },
    { 0x43bf214b, 55 },
    { 0, 0 },
    { 0xde5f214d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f2151, 95 },
    { 0xa20f2152, 81 },
    { 0, 0 },
    { 0xe51f2154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbf215a, 61 },
    { 0xa32f215b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f216a, 33 },
    { 0x277f216a, 34 },
    { 0xb34f216c, 85 },
    { 0, 0 },
    { 0x8cdf216e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2171, 76 },
    { 0, 0 },
    { 0x2ef2173, 3 },
    { 0, 0 },
    { 0xca3f2175, 90 },
    { 0, 0 },
    { 0xebaf2177, 90 },
    { 0, 0 },
    { 0x5e3f2179, 90 },
    { 0xdbf217a, 90 },
    { 0xa29f217a, 34 },
    { 0xb9cf217c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79f2182, 90 },
    { 0, 0 },
    { 0x3aff2184, 34 },
    { 0x261f2184, 46 },
    { 0x257f2186, 98 },
    { 0xd39f2186, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f218a, 90 },
    { 0, 0 },
    { 0x12f218c, 98 },
    { 0xbb3f218d, 78 },
    { 0x2e7f218d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cf2191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9caf2195, 55 },
    { 0, 0 },
    { 0x300f2197, 93 },
    { 0xd14f2198, 55 },
    { 0xcbf2198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f219d, 98 },
    { 0xa99f219d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff21a1, 90 },
    { 0x666f21a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bef21a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f21ae, 61 },
    { 0, 0 },
    { 0xcbdf21b0, 55 },
    { 0xc1ff21b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cf21b5, 90 },
    { 0x821f21b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6f21bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f21c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bf21ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaf21d1, 95 },
    { 0xfe9f21d2, 85 },
    { 0, 0 },
    { 0x9b3f21d4, 61 },
    { 0x2a1f21d5, 33 },
    { 0, 0 },
    { 0x607f21d7, 90 },
    { 0, 0 },
    { 0x105f21d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadf21dc, 90 },
    { 0xca1f21dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556f21e1, 90 },
    { 0, 0 },
    { 0x3e0f21e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f21e9, 96 },
    { 0x6cff21ea, 33 },
    { 0x80bf21eb, 23 },
    { 0x7b8f21ec, 69 },
    { 0x745f21ed, 55 },
    { 0, 0 },
    { 0x37bf21ef, 33 },
    { 0xbf5f21f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f21f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f21fc, 77 },
    { 0xbe5f21fd, 61 },
    { 0xef9f21fe, 90 },
    { 0xb94f21ff, 43 },
    { 0xb68f21ff, 43 },
    { 0x79f21fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4f2205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ef220b, 67 },
    { 0x44cf220c, 43 },
    { 0x73ff220d, 95 },
    { 0xab4f220d, 36 },
    { 0x84af220f, 43 },
    { 0x8c6f2210, 61 },
    { 0x40ff220f, 33 },
    { 0x9a6f2212, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f2216, 88 },
    { 0xf97f2216, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf221c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54f2220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63df2223, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ff222c, 77 },
    { 0, 0 },
    { 0x1edf222e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f2234, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f2238, 33 },
    { 0xf2af2238, 90 },
    { 0, 0 },
    { 0x59af223b, 66 },
    { 0x9caf223c, 90 },
    { 0x688f223d, 98 },
    { 0, 0 },
    { 0x49df223f, 93 },
    { 0xc24f223f, 34 },
    { 0xbc9f2241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf2247, 90 },
    { 0xb05f2248, 97 },
    { 0x39af2249, 95 },
    { 0x967f2248, 83 },
    { 0x6f1f224a, 78 },
    { 0xabdf2249, 69 },
    { 0x537f224a, 98 },
    { 0xc6af224b, 46 },
    { 0x666f224f, 95 },
    { 0x2f224b, 85 },
    { 0, 0 },
    { 0x793f2252, 99 },
    { 0x176f2252, 95 },
    { 0xc71f2254, 91 },
    { 0, 0 },
    { 0x19df2256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef225a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858f2262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338f2267, 90 },
    { 0xbadf2267, 33 },
    { 0xdf6f2269, 23 },
    { 0x1def2267, 55 },
    { 0x3c8f226b, 91 },
    { 0x8daf226c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6f2271, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf2275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f227e, 61 },
    { 0x999f227f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f2284, 61 },
    { 0x940f2284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf228a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495f228e, 90 },
    { 0, 0 },
    { 0xb07f2290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958f229c, 90 },
    { 0, 0 },
    { 0x539f229e, 76 },
    { 0x8b7f229e, 98 },
    { 0, 0 },
    { 0x14ef22a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4f22b8, 61 },
    { 0x538f22b9, 98 },
    { 0x80bf22b9, 90 },
    { 0xf5af22b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff22be, 34 },
    { 0, 0 },
    { 0x6a3f22c0, 66 },
    { 0, 0 },
    { 0x5fef22c2, 56 },
    { 0, 0 },
    { 0x746f22c4, 90 },
    { 0x92df22c5, 72 },
    { 0xdfcf22c5, 55 },
    { 0xbc4f22c7, 95 },
    { 0, 0 },
    { 0x575f22c9, 23 },
    { 0, 0 },
    { 0xa33f22cb, 90 },
    { 0xea0f22cb, 23 },
    { 0xddef22cb, 61 },
    { 0xfc8f22ce, 95 },
    { 0, 0 },
    { 0xbfff22d0, 76 },
    { 0, 0 },
    { 0xefff22d2, 98 },
    { 0xa36f22d3, 90 },
    { 0x9dff22d2, 55 },
    { 0x7e0f22d4, 33 },
    { 0x2faf22d6, 43 },
    { 0, 0 },
    { 0xf44f22d8, 41 },
    { 0xb22f22d9, 43 },
    { 0x80ef22d8, 55 },
    { 0x108f22db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8f22df, 66 },
    { 0x5e9f22e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ef22e4, 100 },
    { 0x553f22e5, 98 },
    { 0x8cdf22e5, 98 },
    { 0xe1f22e4, 55 },
    { 0x7f2f22e8, 61 },
    { 0, 0 },
    { 0x1d9f22ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f22f8, 95 },
    { 0x6eaf22f8, 90 },
    { 0x4ecf22fa, 98 },
    { 0x864f22fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4f2302, 66 },
    { 0, 0 },
    { 0x4b8f2304, 69 },
    { 0, 0 },
    { 0xaaaf2306, 43 },
    { 0x8c3f2307, 44 },
    { 0xdc6f2307, 95 },
    { 0, 0 },
    { 0x75f230a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7f230e, 66 },
    { 0xa13f230e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36f2312, 55 },
    { 0, 0 },
    { 0xed6f2314, 98 },
    { 0xe48f2315, 33 },
    { 0xbdef2316, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8f231d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7f2327, 55 },
    { 0x7c5f2327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x417f232b, 98 },
    { 0x6abf232c, 61 },
    { 0xebff232d, 76 },
    { 0xf78f232d, 90 },
    { 0x59f232e, 76 },
    { 0xceaf2330, 66 },
    { 0x7b3f2330, 55 },
    { 0x237f2332, 90 },
    { 0, 0 },
    { 0x3aef2334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dff2337, 95 },
    { 0x808f2338, 23 },
    { 0x3c8f2339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62f233d, 81 },
    { 0xeb9f233e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf2343, 34 },
    { 0, 0 },
    { 0x118f2345, 61 },
    { 0x92bf2345, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af2349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f234d, 66 },
    { 0, 0 },
    { 0x177f234f, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x70af2352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df2355, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f2359, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f235e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f2365, 95 },
    { 0xf8df2366, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49f236e, 61 },
    { 0xcb5f236f, 95 },
    { 0xd52f2370, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf2373, 61 },
    { 0x98df2373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf2378, 23 },
    { 0, 0 },
    { 0x997f237a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f2380, 90 },
    { 0xd43f2381, 95 },
    { 0, 0 },
    { 0x3d9f2383, 93 },
    { 0x361f2384, 76 },
    { 0, 0 },
    { 0xd5f2386, 85 },
    { 0x936f2387, 76 },
    { 0x9fcf2387, 23 },
    { 0xaa8f2386, 23 },
    { 0x9cef2386, 99 },
    { 0x1ef238b, 90 },
    { 0xf4ef238b, 84 },
    { 0x560f238c, 95 },
    { 0, 0 },
    { 0x8dcf238f, 99 },
    { 0, 0 },
    { 0xb12f2391, 53 },
    { 0, 0 },
    { 0xbf8f2393, 23 },
    { 0x711f2393, 95 },
    { 0x8cef2395, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f2399, 23 },
    { 0xfd0f239a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274f239f, 95 },
    { 0x87df23a0, 61 },
    { 0x6b0f23a1, 33 },
    { 0x798f239f, 23 },
    { 0, 0 },
    { 0x897f23a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf23a8, 46 },
    { 0x229f23a8, 98 },
    { 0x1a5f23a9, 98 },
    { 0, 0 },
    { 0x596f23ac, 61 },
    { 0xc3ef23ac, 95 },
    { 0x3af23ae, 33 },
    { 0, 0 },
    { 0x290f23b0, 34 },
    { 0x813f23b0, 61 },
    { 0, 0 },
    { 0xd5cf23b3, 23 },
    { 0xcc2f23b3, 99 },
    { 0, 0 },
    { 0xcaaf23b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f23bb, 33 },
    { 0, 0 },
    { 0x370f23bd, 87 },
    { 0, 0 },
    { 0x458f23bf, 34 },
    { 0xcdff23c0, 64 },
    { 0x7f23c1, 90 },
    { 0xf30f23c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f23c6, 90 },
    { 0x138f23c7, 84 },
    { 0x8eaf23c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470f23cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf23d2, 95 },
    { 0xeb2f23d2, 61 },
    { 0, 0 },
    { 0xcddf23d5, 81 },
    { 0x9dbf23d6, 90 },
    { 0xbe5f23d7, 68 },
    { 0xee6f23d6, 90 },
    { 0x7e3f23d9, 23 },
    { 0xda8f23d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f23e7, 90 },
    { 0xa49f23e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf23f4, 84 },
    { 0, 0 },
    { 0xb89f23f6, 33 },
    { 0, 0 },
    { 0x1ecf23f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8df23fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f2400, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f2408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f240e, 34 },
    { 0xe3df240e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f2413, 66 },
    { 0, 0 },
    { 0x3c8f2415, 98 },
    { 0x6cbf2416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35f241b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8f241e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92f2421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f2425, 68 },
    { 0xce3f2426, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f242d, 90 },
    { 0x20ef242e, 99 },
    { 0xb5ff242e, 66 },
    { 0xeff2430, 95 },
    { 0xc86f2431, 88 },
    { 0x56bf2431, 85 },
    { 0xf67f2432, 34 },
    { 0x28f2432, 90 },
    { 0xc4bf242f, 55 },
    { 0x9f9f2431, 99 },
    { 0x328f2437, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790f243c, 90 },
    { 0xdf6f243c, 55 },
    { 0x956f243e, 68 },
    { 0x922f243c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f2443, 95 },
    { 0x30f2444, 55 },
    { 0, 0 },
    { 0x70f2446, 33 },
    { 0x6cff2446, 23 },
    { 0xa6af2448, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcf244f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ff2458, 56 },
    { 0, 0 },
    { 0x322f245a, 61 },
    { 0x33cf245b, 98 },
    { 0x215f245b, 93 },
    { 0x7e1f245d, 88 },
    { 0xfbaf245e, 95 },
    { 0, 0 },
    { 0xa81f2460, 78 },
    { 0xe09f2461, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ff2464, 33 },
    { 0x6d2f2465, 66 },
    { 0, 0 },
    { 0x991f2467, 98 },
    { 0x179f2468, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf246c, 67 },
    { 0xe9cf246d, 52 },
    { 0x729f246e, 90 },
    { 0x8a8f246f, 61 },
    { 0x1c3f2470, 34 },
    { 0xc4af246f, 99 },
    { 0, 0 },
    { 0x8d9f2473, 78 },
    { 0x374f2474, 55 },
    { 0, 0 },
    { 0x3e2f2476, 87 },
    { 0, 0 },
    { 0x42f2478, 95 },
    { 0, 0 },
    { 0x266f247a, 43 },
    { 0xe9bf247b, 23 },
    { 0x2ecf247c, 67 },
    { 0x954f247d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf2481, 77 },
    { 0x225f2482, 90 },
    { 0x6cbf2483, 33 },
    { 0xb91f2484, 33 },
    { 0, 0 },
    { 0x277f2486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf248d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f2496, 68 },
    { 0x70cf2496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7f249b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edf24a3, 95 },
    { 0xcd6f24a4, 55 },
    { 0, 0 },
    { 0xb21f24a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f24ac, 84 },
    { 0, 0 },
    { 0x50af24ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96af24b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f24b5, 33 },
    { 0xc5ef24b6, 66 },
    { 0xa81f24b7, 34 },
    { 0, 0 },
    { 0x441f24b9, 43 },
    { 0x287f24b9, 100 },
    { 0, 0 },
    { 0x61ef24bc, 93 },
    { 0x575f24bc, 56 },
    { 0x1a6f24bd, 87 },
    { 0x23df24bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdf24c7, 98 },
    { 0x4a6f24c8, 61 },
    { 0xc2ff24c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf24d1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f24d4, 66 },
    { 0xd00f24d4, 43 },
    { 0xb8af24d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2af24dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf24e3, 91 },
    { 0x7a1f24e4, 61 },
    { 0x472f24e4, 34 },
    { 0, 0 },
    { 0xfa4f24e7, 91 },
    { 0, 0 },
    { 0xd6ef24e9, 33 },
    { 0xb8af24e9, 98 },
    { 0x796f24ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f24ee, 76 },
    { 0, 0 },
    { 0xc06f24f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff24f7, 90 },
    { 0xe3cf24f7, 46 },
    { 0, 0 },
    { 0xda2f24fa, 55 },
    { 0, 0 },
    { 0x16bf24fc, 95 },
    { 0, 0 },
    { 0x724f24fe, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ff250a, 34 },
    { 0x140f250b, 68 },
    { 0, 0 },
    { 0xa3ff250d, 90 },
    { 0x590f250d, 61 },
    { 0, 0 },
    { 0xdd2f2510, 33 },
    { 0x1d5f2511, 100 },
    { 0x9b6f2511, 99 },
    { 0x847f2512, 61 },
    { 0, 0 },
    { 0x527f2515, 61 },
    { 0xd78f2516, 99 },
    { 0x58ef2515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66f251b, 23 },
    { 0x33af251c, 55 },
    { 0xc93f251d, 52 },
    { 0x161f251b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebf2526, 98 },
    { 0x12bf2526, 33 },
    { 0x434f2526, 33 },
    { 0xa33f2529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f252e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cf2532, 61 },
    { 0, 0 },
    { 0xe27f2534, 95 },
    { 0xb3ef2534, 66 },
    { 0, 0 },
    { 0xcd1f2537, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24f253c, 76 },
    { 0, 0 },
    { 0xbb4f253e, 55 },
    { 0xa9ef253f, 77 },
    { 0x379f253f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75f2546, 66 },
    { 0x368f2547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf254a, 90 },
    { 0x36ef254b, 33 },
    { 0, 0 },
    { 0x33f254d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf2550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eef255b, 58 },
    { 0x4eaf255c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f255f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598f2565, 98 },
    { 0xb3f2566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9f2571, 51 },
    { 0x57f2571, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8f2576, 55 },
    { 0, 0 },
    { 0x7b0f2578, 23 },
    { 0xc25f2579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf257d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f2580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f2585, 93 },
    { 0xbabf2585, 76 },
    { 0xe72f2587, 90 },
    { 0x535f2588, 23 },
    { 0xb58f2589, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f2592, 65 },
    { 0x885f2593, 84 },
    { 0x25df2594, 90 },
    { 0xef2595, 33 },
    { 0xf0cf2596, 66 },
    { 0x6e0f2596, 80 },
    { 0x5b9f2596, 90 },
    { 0x1f3f2599, 90 },
    { 0xc75f259a, 34 },
    { 0x2bff2595, 99 },
    { 0x759f259c, 61 },
    { 0x470f259d, 90 },
    { 0xebcf2595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98f25a5, 95 },
    { 0, 0 },
    { 0x766f25a7, 52 },
    { 0x552f25a7, 95 },
    { 0xe3ff25a8, 95 },
    { 0x840f25aa, 90 },
    { 0xd0ef25a7, 76 },
    { 0, 0 },
    { 0xad9f25ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91f25b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf25bd, 61 },
    { 0xc9f25be, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4df25c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104f25cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651f25d1, 90 },
    { 0, 0 },
    { 0x34df25d3, 98 },
    { 0, 0 },
    { 0xcbdf25d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeff25d8, 76 },
    { 0xf9df25d8, 85 },
    { 0x2cf25da, 95 },
    { 0xb82f25d9, 40 },
    { 0x407f25dc, 61 },
    { 0x332f25dc, 55 },
    { 0xd2f25de, 56 },
    { 0, 0 },
    { 0x334f25e0, 90 },
    { 0, 0 },
    { 0x686f25e2, 23 },
    { 0x385f25e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f25e7, 75 },
    { 0x5bbf25e8, 98 },
    { 0, 0 },
    { 0xcbbf25ea, 90 },
    { 0x219f25ea, 61 },
    { 0xc4ff25ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821f25f3, 95 },
    { 0, 0 },
    { 0x873f25f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f25f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f25ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef2611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x334f2614, 33 },
    { 0x5c7f2615, 90 },
    { 0x791f2616, 95 },
    { 0, 0 },
    { 0xe1df2618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59f261f, 23 },
    { 0xf25f2620, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1f2625, 72 },
    { 0, 0 },
    { 0x63ff2627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5f2631, 52 },
    { 0xb8ff2631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cf2636, 33 },
    { 0x4abf2637, 95 },
    { 0xaabf2637, 100 },
    { 0x81ef2639, 61 },
    { 0, 0 },
    { 0xb45f263b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47df2640, 23 },
    { 0x23cf2641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f2645, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bff264d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f2652, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cf2655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f2659, 64 },
    { 0, 0 },
    { 0x90f265b, 90 },
    { 0x675f265c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcf265f, 76 },
    { 0x240f2660, 61 },
    { 0, 0 },
    { 0xa69f2662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184f2669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898f266f, 76 },
    { 0, 0 },
    { 0x653f2671, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f2676, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef267c, 68 },
    { 0x874f267d, 95 },
    { 0, 0 },
    { 0xcaff267f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02f2684, 98 },
    { 0x63ef2685, 76 },
    { 0xc8bf2685, 93 },
    { 0x59df2687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf268d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf2690, 53 },
    { 0xac2f2691, 33 },
    { 0x7b8f2692, 61 },
    { 0x754f2693, 33 },
    { 0xf74f2694, 52 },
    { 0, 0 },
    { 0xac2f2696, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f269b, 33 },
    { 0, 0 },
    { 0xe35f269d, 46 },
    { 0xd0bf269d, 90 },
    { 0xc78f269e, 55 },
    { 0, 0 },
    { 0xa1df26a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f26a4, 76 },
    { 0x372f26a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78f26aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f26ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf26b1, 66 },
    { 0x32ef26b2, 95 },
    { 0, 0 },
    { 0xcff26b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadf26b7, 34 },
    { 0, 0 },
    { 0x924f26b9, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff26be, 95 },
    { 0, 0 },
    { 0x3cf26c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef26c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df26c8, 33 },
    { 0, 0 },
    { 0x56ef26ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x959f26cd, 75 },
    { 0x554f26ce, 34 },
    { 0xdf7f26ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f26d5, 95 },
    { 0xfebf26d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f26dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f26e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9f26e7, 33 },
    { 0, 0 },
    { 0x104f26e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00f26f1, 23 },
    { 0x75ef26f2, 61 },
    { 0xe96f26f1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f26f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f26fa, 95 },
    { 0x484f26fa, 76 },
    { 0x147f26fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaff2701, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184f2705, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ff270b, 33 },
    { 0, 0 },
    { 0xb43f270d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f2710, 34 },
    { 0x44bf2711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2716, 61 },
    { 0x870f2717, 55 },
    { 0x19df2717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cf271e, 61 },
    { 0xe6cf271f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabf2726, 67 },
    { 0, 0 },
    { 0xd03f2728, 75 },
    { 0x34df2729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddf272c, 98 },
    { 0x552f272d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f2733, 95 },
    { 0x116f2734, 98 },
    { 0x8def2735, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bf273d, 46 },
    { 0, 0 },
    { 0x24ff273f, 85 },
    { 0xb2f273f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f2745, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cf274d, 61 },
    { 0, 0 },
    { 0x815f274f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf2753, 98 },
    { 0, 0 },
    { 0x4af2755, 90 },
    { 0x4adf2756, 23 },
    { 0xd5bf2756, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f2762, 83 },
    { 0x78bf2763, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f2766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef2769, 96 },
    { 0x47f276a, 61 },
    { 0xdc5f276b, 90 },
    { 0x3b6f276b, 96 },
    { 0xc31f276d, 33 },
    { 0, 0 },
    { 0xaa5f276f, 33 },
    { 0xbbf2770, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f2778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05f277b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26f277e, 93 },
    { 0xec2f277e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30f2783, 55 },
    { 0, 0 },
    { 0xda0f2785, 93 },
    { 0, 0 },
    { 0xd36f2787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ef278b, 98 },
    { 0, 0 },
    { 0x693f278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880f2795, 33 },
    { 0xbacf2796, 61 },
    { 0x66ff2796, 76 },
    { 0x3e9f2796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50f279b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f279e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7f27a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3df27ab, 98 },
    { 0xb81f27ac, 100 },
    { 0xa7bf27ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f27ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f27bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcff27c2, 95 },
    { 0x509f27c3, 90 },
    { 0x341f27c4, 90 },
    { 0xdbbf27c4, 93 },
    { 0x7ef27c6, 95 },
    { 0, 0 },
    { 0x700f27c8, 90 },
    { 0, 0 },
    { 0xae7f27ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af27d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af27dc, 23 },
    { 0xfcbf27dd, 77 },
    { 0, 0 },
    { 0x61bf27df, 90 },
    { 0x7d1f27e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f27e4, 90 },
    { 0x154f27e4, 55 },
    { 0xdc5f27e6, 67 },
    { 0, 0 },
    { 0x7e9f27e8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf27eb, 98 },
    { 0x951f27ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f27ef, 85 },
    { 0xf5ef27ef, 100 },
    { 0x242f27f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f27f8, 93 },
    { 0xfbbf27f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf2801, 90 },
    { 0, 0 },
    { 0x4a2f2803, 91 },
    { 0x739f2803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53df280a, 17 },
    { 0, 0 },
    { 0xc95f280c, 52 },
    { 0x344f280d, 61 },
    { 0xf22f280e, 61 },
    { 0x3aef280f, 23 },
    { 0x8a4f280f, 95 },
    { 0xd4af2811, 90 },
    { 0xd3df2810, 91 },
    { 0, 0 },
    { 0x6c5f2814, 23 },
    { 0x3eaf2814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f281d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f2820, 98 },
    { 0x1c2f2821, 40 },
    { 0x5fdf2821, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3f2826, 68 },
    { 0x579f2826, 93 },
    { 0x884f2828, 33 },
    { 0, 0 },
    { 0x78f282a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf282e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7f2838, 90 },
    { 0xc6df2839, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9f283f, 98 },
    { 0xef4f283f, 61 },
    { 0, 0 },
    { 0xfdf2842, 66 },
    { 0, 0 },
    { 0xa31f2844, 95 },
    { 0, 0 },
    { 0x214f2846, 61 },
    { 0x6bff2847, 33 },
    { 0xd8ff2848, 61 },
    { 0x4e9f2848, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20f284c, 33 },
    { 0, 0 },
    { 0xe02f284e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5f2851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f2855, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df285f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f2863, 90 },
    { 0xceff2863, 76 },
    { 0xe8ef2865, 81 },
    { 0x618f2863, 33 },
    { 0xde2f2867, 76 },
    { 0x282f2868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f286f, 61 },
    { 0xacbf286f, 98 },
    { 0xa44f2871, 43 },
    { 0xc1ff2872, 100 },
    { 0, 0 },
    { 0xe0bf2874, 90 },
    { 0xc7cf2875, 90 },
    { 0, 0 },
    { 0x179f2877, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f287b, 76 },
    { 0, 0 },
    { 0xacef287d, 61 },
    { 0x61cf287e, 95 },
    { 0x7f9f287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9f2885, 96 },
    { 0xdcaf2886, 90 },
    { 0x408f2887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff288e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0df2892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf2896, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaf289c, 34 },
    { 0xbdcf289c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf28a2, 95 },
    { 0x4e1f28a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df28a7, 33 },
    { 0xe63f28a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5f28af, 93 },
    { 0, 0 },
    { 0x6c1f28b1, 66 },
    { 0x95f28b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f28b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7f28b9, 61 },
    { 0xbb2f28ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9f28be, 84 },
    { 0xd6af28be, 90 },
    { 0x11ef28c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f28c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6def28cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828f28d3, 81 },
    { 0, 0 },
    { 0xa23f28d5, 98 },
    { 0xa12f28d6, 81 },
    { 0xe1af28d6, 58 },
    { 0x1fcf28d7, 43 },
    { 0, 0 },
    { 0x22af28da, 33 },
    { 0x7c4f28db, 76 },
    { 0x723f28dc, 43 },
    { 0, 0 },
    { 0x72f28de, 17 },
    { 0, 0 },
    { 0x36ff28e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90f28e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f28ea, 55 },
    { 0, 0 },
    { 0xbc8f28ec, 55 },
    { 0x3f3f28ed, 34 },
    { 0x1e3f28ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f28f2, 93 },
    { 0x9e8f28f3, 95 },
    { 0, 0 },
    { 0x54cf28f5, 95 },
    { 0xe13f28f6, 95 },
    { 0, 0 },
    { 0xd68f28f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f28fb, 90 },
    { 0x45bf28fc, 61 },
    { 0xccaf28fd, 52 },
    { 0x7d4f28fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9f2901, 95 },
    { 0x5d3f2902, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f2906, 23 },
    { 0x27ff2907, 100 },
    { 0xe60f2908, 61 },
    { 0x82ef2906, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cef290f, 55 },
    { 0xd2df2910, 76 },
    { 0xf47f290f, 95 },
    { 0, 0 },
    { 0x15cf2913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f2916, 36 },
    { 0xb70f2917, 61 },
    { 0x56af2918, 90 },
    { 0xe1ff2918, 98 },
    { 0, 0 },
    { 0x991f291b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f291e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379f292c, 33 },
    { 0, 0 },
    { 0x889f292e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcf2942, 33 },
    { 0x6e1f2943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386f2949, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339f2951, 33 },
    { 0xe45f2952, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22f2956, 98 },
    { 0, 0 },
    { 0x226f2958, 83 },
    { 0x4bbf2958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76f295c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f2960, 56 },
    { 0x1bdf2961, 90 },
    { 0xd34f2962, 88 },
    { 0x794f2962, 90 },
    { 0x616f2963, 23 },
    { 0x9b6f2963, 72 },
    { 0x96bf2966, 87 },
    { 0, 0 },
    { 0x33ff2968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf296c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2f2973, 46 },
    { 0x62df2974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x931f2977, 78 },
    { 0x724f2977, 23 },
    { 0xa80f2979, 84 },
    { 0x424f2979, 100 },
    { 0x88cf2979, 23 },
    { 0, 0 },
    { 0xda1f297d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455f2985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff2988, 23 },
    { 0, 0 },
    { 0x3eef298a, 61 },
    { 0, 0 },
    { 0x270f298c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf2990, 90 },
    { 0, 0 },
    { 0x1cff2992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585f2997, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f299a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909f29a2, 90 },
    { 0, 0 },
    { 0x5c1f29a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef29a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36f29ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff29b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf29b7, 78 },
    { 0xbe0f29b8, 87 },
    { 0xeedf29b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aef29bc, 55 },
    { 0xa10f29bd, 17 },
    { 0x85f29be, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f29c1, 33 },
    { 0x4b6f29c1, 33 },
    { 0, 0 },
    { 0x439f29c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f29c9, 87 },
    { 0xb22f29ca, 23 },
    { 0x54bf29cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dff29ce, 78 },
    { 0, 0 },
    { 0xce9f29d0, 41 },
    { 0x740f29d0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf29db, 90 },
    { 0x31f29dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719f29e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2f29e9, 33 },
    { 0x241f29ea, 95 },
    { 0x77f29e9, 98 },
    { 0xff29e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6f29f3, 68 },
    { 0xd74f29f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8f29f8, 55 },
    { 0x7cbf29f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f29fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f2a01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cf2a04, 33 },
    { 0, 0 },
    { 0x295f2a06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541f2a0a, 93 },
    { 0, 0 },
    { 0xcf2f2a0c, 93 },
    { 0, 0 },
    { 0xf07f2a0e, 23 },
    { 0x225f2a0f, 43 },
    { 0x7c7f2a10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73f2a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7f2a1c, 23 },
    { 0, 0 },
    { 0x885f2a1e, 95 },
    { 0, 0 },
    { 0x3baf2a20, 61 },
    { 0, 0 },
    { 0x933f2a22, 98 },
    { 0, 0 },
    { 0x50df2a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22df2a28, 65 },
    { 0x75ff2a29, 91 },
    { 0, 0 },
    { 0x4ecf2a2b, 43 },
    { 0xee9f2a2c, 61 },
    { 0xcedf2a2d, 90 },
    { 0x545f2a2c, 91 },
    { 0, 0 },
    { 0x88df2a30, 91 },
    { 0xc15f2a30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f2a35, 66 },
    { 0x2f5f2a36, 85 },
    { 0x872f2a35, 33 },
    { 0xd79f2a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f2a3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f2a43, 55 },
    { 0xab9f2a44, 98 },
    { 0x368f2a45, 33 },
    { 0xe0cf2a46, 98 },
    { 0x1f2a46, 95 },
    { 0xa16f2a48, 81 },
    { 0x3e3f2a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1f2a4d, 67 },
    { 0, 0 },
    { 0x6f7f2a4f, 52 },
    { 0, 0 },
    { 0xfc8f2a51, 76 },
    { 0, 0 },
    { 0xd4df2a53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f2a5a, 23 },
    { 0x5bff2a5b, 65 },
    { 0x877f2a5c, 80 },
    { 0xf3bf2a5a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ef2a63, 90 },
    { 0, 0 },
    { 0x206f2a65, 88 },
    { 0x7abf2a65, 100 },
    { 0xe9ef2a67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24f2a6a, 68 },
    { 0xb11f2a6b, 61 },
    { 0x9e4f2a6c, 99 },
    { 0, 0 },
    { 0xb56f2a6e, 85 },
    { 0, 0 },
    { 0x5daf2a70, 23 },
    { 0xfb2f2a71, 33 },
    { 0xbcf2a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbf2a75, 61 },
    { 0, 0 },
    { 0x890f2a77, 90 },
    { 0xa75f2a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9f2a7b, 90 },
    { 0x3e5f2a7c, 52 },
    { 0x6ddf2a7c, 61 },
    { 0x994f2a7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6f2a83, 91 },
    { 0, 0 },
    { 0xdfff2a85, 91 },
    { 0, 0 },
    { 0x896f2a87, 38 },
    { 0x38df2a88, 61 },
    { 0xa0bf2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160f2a8d, 99 },
    { 0x4b3f2a8e, 76 },
    { 0x5cef2a8f, 95 },
    { 0x267f2a90, 98 },
    { 0xb82f2a90, 87 },
    { 0xbdef2a91, 76 },
    { 0x98af2a93, 61 },
    { 0xa84f2a93, 55 },
    { 0x3c7f2a95, 33 },
    { 0x42df2a8e, 23 },
    { 0xb1cf2a8f, 99 },
    { 0x840f2a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f2a9d, 98 },
    { 0, 0 },
    { 0xac6f2a9f, 76 },
    { 0xf82f2aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f2aa5, 100 },
    { 0x653f2aa6, 61 },
    { 0x49af2aa7, 61 },
    { 0xc8ff2aa8, 90 },
    { 0x116f2aa7, 95 },
    { 0, 0 },
    { 0xcdef2aab, 77 },
    { 0x3bbf2aac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f2ab2, 76 },
    { 0xc4af2ab3, 33 },
    { 0, 0 },
    { 0xa2af2ab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf2ab9, 68 },
    { 0x4f4f2ab9, 85 },
    { 0, 0 },
    { 0x383f2abc, 33 },
    { 0, 0 },
    { 0x6e5f2abe, 90 },
    { 0xcb3f2abe, 84 },
    { 0x1cf2ac0, 52 },
    { 0xfd8f2abf, 95 },
    { 0xd81f2ac0, 61 },
    { 0, 0 },
    { 0xde6f2ac4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85f2ac8, 55 },
    { 0xf4cf2ac9, 98 },
    { 0x2fff2aca, 88 },
    { 0x5c7f2acb, 61 },
    { 0x82f2acb, 90 },
    { 0xb72f2acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf2ad2, 23 },
    { 0xdedf2ad2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f2ad8, 90 },
    { 0x5af2ad9, 8 },
    { 0, 0 },
    { 0xd81f2adb, 84 },
    { 0x4aaf2adc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f2ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af2aec, 98 },
    { 0, 0 },
    { 0x157f2aee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf2af6, 66 },
    { 0, 0 },
    { 0x838f2af8, 100 },
    { 0, 0 },
    { 0x9dff2afa, 98 },
    { 0x68af2afb, 40 },
    { 0, 0 },
    { 0x5e8f2afd, 61 },
    { 0, 0 },
    { 0x35ff2aff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f2b04, 61 },
    { 0, 0 },
    { 0x3b3f2b06, 61 },
    { 0xfc7f2b07, 95 },
    { 0, 0 },
    { 0x297f2b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ef2b0c, 43 },
    { 0x9bef2b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cf2b13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df2b16, 99 },
    { 0x6f2b17, 23 },
    { 0, 0 },
    { 0xf6df2b19, 55 },
    { 0, 0 },
    { 0x1bff2b1b, 55 },
    { 0xaddf2b1c, 33 },
    { 0, 0 },
    { 0x609f2b1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf2b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f2b2e, 66 },
    { 0, 0 },
    { 0x61bf2b30, 95 },
    { 0xa6f2b31, 90 },
    { 0x115f2b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8f2b37, 34 },
    { 0x4c2f2b37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2f2b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf2b3e, 23 },
    { 0x2bff2b3f, 90 },
    { 0x8dbf2b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ff2b43, 95 },
    { 0xd35f2b44, 17 },
    { 0, 0 },
    { 0x6d2f2b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbf2b4e, 100 },
    { 0xbedf2b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f2b53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f2b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f2b59, 76 },
    { 0, 0 },
    { 0x13ef2b5b, 90 },
    { 0, 0 },
    { 0xe73f2b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17f2b64, 93 },
    { 0, 0 },
    { 0x913f2b66, 90 },
    { 0, 0 },
    { 0x517f2b68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f2b77, 23 },
    { 0, 0 },
    { 0xb69f2b79, 61 },
    { 0, 0 },
    { 0x598f2b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4df2b7e, 95 },
    { 0, 0 },
    { 0xe82f2b80, 23 },
    { 0x852f2b81, 90 },
    { 0x582f2b81, 43 },
    { 0x6bcf2b83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964f2b8b, 78 },
    { 0x2cdf2b8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df2b8f, 98 },
    { 0x64ff2b8f, 76 },
    { 0, 0 },
    { 0x6c5f2b92, 23 },
    { 0, 0 },
    { 0xd2ff2b94, 17 },
    { 0x898f2b95, 33 },
    { 0x1f8f2b96, 55 },
    { 0x297f2b97, 98 },
    { 0x9e7f2b98, 23 },
    { 0xaa8f2b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbf2b9d, 61 },
    { 0, 0 },
    { 0x5c8f2b9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5f2ba4, 77 },
    { 0x171f2ba5, 95 },
    { 0, 0 },
    { 0x25f2ba7, 61 },
    { 0xd90f2ba8, 33 },
    { 0xb58f2ba9, 76 },
    { 0xc3ff2ba8, 76 },
    { 0x8eff2bab, 33 },
    { 0xc02f2bac, 61 },
    { 0x8faf2bad, 33 },
    { 0, 0 },
    { 0xbd0f2baf, 90 },
    { 0x581f2bb0, 66 },
    { 0xd15f2bb1, 98 },
    { 0xb2ef2bb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x595f2bb5, 33 },
    { 0x87ff2bb6, 61 },
    { 0, 0 },
    { 0xa3ef2bb8, 76 },
    { 0x9d0f2bb9, 33 },
    { 0x23cf2bba, 75 },
    { 0xd55f2bbb, 92 },
    { 0xb01f2bbb, 76 },
    { 0xbf2f2bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f2bc0, 76 },
    { 0xd25f2bc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df2bca, 61 },
    { 0xc39f2bcb, 34 },
    { 0xc8ff2bcb, 95 },
    { 0, 0 },
    { 0x4cef2bce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58df2bd5, 90 },
    { 0x69bf2bd6, 84 },
    { 0x16ff2bd6, 90 },
    { 0x5e6f2bd7, 85 },
    { 0x2fdf2bd9, 23 },
    { 0x4c6f2bda, 95 },
    { 0x749f2bda, 43 },
    { 0x7c9f2bdc, 34 },
    { 0x2c9f2bdc, 90 },
    { 0x7eef2bde, 61 },
    { 0xd7cf2bda, 23 },
    { 0xbeff2be0, 91 },
    { 0x2f7f2bd9, 91 },
    { 0x2abf2bdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x359f2be5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71af2be9, 46 },
    { 0x4b5f2bea, 93 },
    { 0x656f2beb, 95 },
    { 0xf72f2bec, 76 },
    { 0xec7f2bec, 34 },
    { 0x6fdf2bee, 81 },
    { 0xdf5f2bef, 80 },
    { 0, 0 },
    { 0x565f2bf1, 33 },
    { 0x3ef2bf2, 95 },
    { 0xed8f2bf3, 55 },
    { 0x49df2bf4, 90 },
    { 0x870f2bf2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5f2bfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacf2c03, 100 },
    { 0, 0 },
    { 0xd0bf2c05, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f2c0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f2c10, 90 },
    { 0xdcf2c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf2c14, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8f2c17, 98 },
    { 0, 0 },
    { 0x6d2f2c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789f2c1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af2c21, 33 },
    { 0, 0 },
    { 0x7aef2c23, 61 },
    { 0, 0 },
    { 0x4e1f2c25, 61 },
    { 0, 0 },
    { 0x59f2c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ef2c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f2c30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9f2c36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bf2c3a, 55 },
    { 0x3aef2c3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362f2c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f2c4a, 90 },
    { 0x874f2c4a, 55 },
    { 0, 0 },
    { 0xcfef2c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f2c54, 52 },
    { 0x2bef2c55, 69 },
    { 0x79ef2c55, 67 },
    { 0xd5af2c56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cf2c5a, 98 },
    { 0, 0 },
    { 0x597f2c5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f2c60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9f2c63, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f2c68, 17 },
    { 0xc9df2c68, 90 },
    { 0xcc9f2c69, 55 },
    { 0xc4af2c6b, 33 },
    { 0, 0 },
    { 0xd9cf2c6d, 90 },
    { 0x32f2c6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebf2c72, 61 },
    { 0, 0 },
    { 0x574f2c74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84f2c77, 64 },
    { 0x27f2c77, 90 },
    { 0, 0 },
    { 0xce6f2c7a, 95 },
    { 0xf0cf2c7b, 23 },
    { 0x11bf2c7b, 99 },
    { 0xbc8f2c7c, 55 },
    { 0, 0 },
    { 0xdf2c7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f2c84, 34 },
    { 0x523f2c85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f2c88, 87 },
    { 0x5e9f2c89, 33 },
    { 0x819f2c89, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x326f2c8d, 33 },
    { 0, 0 },
    { 0x597f2c8f, 76 },
    { 0, 0 },
    { 0x2aaf2c91, 33 },
    { 0xfb7f2c92, 33 },
    { 0x1ef2c92, 95 },
    { 0x20cf2c94, 95 },
    { 0x325f2c94, 95 },
    { 0xfa1f2c94, 43 },
    { 0x32bf2c97, 88 },
    { 0, 0 },
    { 0xaaaf2c99, 84 },
    { 0, 0 },
    { 0xbfbf2c9b, 61 },
    { 0xa48f2c9c, 95 },
    { 0x14bf2c9d, 94 },
    { 0x32cf2c9d, 33 },
    { 0xedff2c9c, 85 },
    { 0xf92f2ca0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f2ca6, 90 },
    { 0xe48f2ca6, 61 },
    { 0x279f2ca8, 90 },
    { 0xc0f2ca8, 90 },
    { 0, 0 },
    { 0xc36f2cab, 33 },
    { 0x79ef2cab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17f2cb7, 90 },
    { 0x73df2cb8, 51 },
    { 0xf08f2cb8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf2cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983f2cc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3f2cc7, 95 },
    { 0x9bf2cc7, 83 },
    { 0x604f2cc9, 33 },
    { 0, 0 },
    { 0x16cf2ccb, 100 },
    { 0, 0 },
    { 0x59f2ccd, 98 },
    { 0xd41f2cce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f2cd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf2cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859f2ce0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f2ce3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2ce8, 33 },
    { 0xba7f2ce9, 76 },
    { 0xc9ef2cea, 33 },
    { 0, 0 },
    { 0x1c8f2cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f2cf2, 33 },
    { 0x9af2cf3, 23 },
    { 0xb50f2cf2, 95 },
    { 0x19bf2cf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bf2cfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272f2d02, 90 },
    { 0, 0 },
    { 0xac9f2d04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2f2d0a, 98 },
    { 0, 0 },
    { 0xc56f2d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8f2d0f, 84 },
    { 0x323f2d0f, 76 },
    { 0, 0 },
    { 0xba2f2d12, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f2d17, 98 },
    { 0, 0 },
    { 0x939f2d19, 68 },
    { 0x1f8f2d1a, 98 },
    { 0x7fff2d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf2d20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f2d23, 75 },
    { 0, 0 },
    { 0xa73f2d25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f2d29, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x219f2d2c, 90 },
    { 0x9aff2d2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cff2d30, 98 },
    { 0x38df2d30, 33 },
    { 0xb90f2d32, 75 },
    { 0x16ef2d33, 95 },
    { 0xf64f2d34, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f2d39, 85 },
    { 0x135f2d3a, 23 },
    { 0x4a5f2d3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3f2d3e, 98 },
    { 0, 0 },
    { 0x5a7f2d40, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x692f2d43, 95 },
    { 0, 0 },
    { 0x825f2d45, 91 },
    { 0x617f2d46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f2d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f2d51, 61 },
    { 0xc0cf2d52, 95 },
    { 0x436f2d51, 99 },
    { 0x426f2d52, 95 },
    { 0, 0 },
    { 0x4b0f2d56, 95 },
    { 0, 0 },
    { 0x880f2d58, 75 },
    { 0xc50f2d59, 98 },
    { 0x650f2d5a, 33 },
    { 0xa9f2d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f2d62, 46 },
    { 0xdcf2d63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785f2d68, 98 },
    { 0x85af2d69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf2d71, 34 },
    { 0, 0 },
    { 0xdedf2d73, 43 },
    { 0xdbcf2d74, 55 },
    { 0xbaef2d73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f2d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdf2d7d, 53 },
    { 0, 0 },
    { 0xc44f2d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0f2d82, 99 },
    { 0, 0 },
    { 0x1ecf2d84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f2d88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f2d90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389f2d94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f2da1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x540f2da4, 55 },
    { 0x715f2da5, 76 },
    { 0x591f2da4, 33 },
    { 0xd38f2da7, 33 },
    { 0, 0 },
    { 0x6faf2da9, 93 },
    { 0xca9f2daa, 65 },
    { 0, 0 },
    { 0x84df2dac, 95 },
    { 0x90af2dad, 81 },
    { 0, 0 },
    { 0x261f2daf, 46 },
    { 0x157f2db0, 90 },
    { 0x5d0f2daf, 95 },
    { 0xdc2f2daf, 61 },
    { 0x44bf2db3, 84 },
    { 0x258f2db3, 95 },
    { 0, 0 },
    { 0xca7f2db6, 46 },
    { 0, 0 },
    { 0x1ff2db8, 46 },
    { 0x15ff2db8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf2dbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1f2dc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09f2dc3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94f2dc6, 93 },
    { 0xe9ef2dc7, 61 },
    { 0xc90f2dc7, 90 },
    { 0, 0 },
    { 0xf88f2dca, 46 },
    { 0xfe8f2dcb, 90 },
    { 0x32ff2dcc, 78 },
    { 0xcaff2dcb, 95 },
    { 0x38ef2dca, 95 },
    { 0, 0 },
    { 0x6eaf2dd0, 34 },
    { 0xae1f2dd1, 23 },
    { 0, 0 },
    { 0x28ff2dd3, 93 },
    { 0x833f2dd3, 23 },
    { 0x41bf2dd4, 23 },
    { 0xca3f2dd6, 75 },
    { 0, 0 },
    { 0x9bbf2dd8, 90 },
    { 0x565f2dd8, 61 },
    { 0, 0 },
    { 0xc66f2ddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff2dde, 78 },
    { 0x3eaf2dde, 95 },
    { 0xb2ff2de0, 61 },
    { 0x612f2de1, 77 },
    { 0x81bf2de2, 78 },
    { 0xbf6f2de1, 61 },
    { 0x20ff2de4, 90 },
    { 0x7cf2de2, 90 },
    { 0x548f2de3, 61 },
    { 0x314f2de7, 90 },
    { 0x882f2de6, 98 },
    { 0x7baf2de2, 90 },
    { 0xb44f2de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bf2dee, 95 },
    { 0xb90f2dee, 90 },
    { 0xa4af2def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af2df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1f2dfb, 23 },
    { 0x1b9f2dfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f2dff, 61 },
    { 0x9fef2e00, 23 },
    { 0xd6bf2e00, 33 },
    { 0xa51f2e00, 33 },
    { 0x2dff2e01, 98 },
    { 0x2adf2dff, 93 },
    { 0x924f2e02, 90 },
    { 0x53f2e05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634f2e12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ef2e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cf2e1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4f2e23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8f2e28, 38 },
    { 0x910f2e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f2e2c, 78 },
    { 0, 0 },
    { 0x481f2e2e, 77 },
    { 0x85ff2e2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bf2e32, 90 },
    { 0x9f7f2e33, 91 },
    { 0x97cf2e33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f2e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ff2e3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adf2e44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f2e48, 95 },
    { 0, 0 },
    { 0xe6f2e4a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff2e4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ef2e52, 76 },
    { 0, 0 },
    { 0x741f2e54, 91 },
    { 0, 0 },
    { 0xdc7f2e56, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbf2e5b, 90 },
    { 0x8bcf2e5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf2e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f2e63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f2e66, 90 },
    { 0xc0f2e67, 61 },
    { 0xd87f2e66, 34 },
    { 0x452f2e69, 77 },
    { 0x5c5f2e6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14f2e6e, 88 },
    { 0xc3ff2e6e, 33 },
    { 0x687f2e6f, 76 },
    { 0x3ebf2e71, 98 },
    { 0xfdf2e70, 90 },
    { 0x8df2e71, 90 },
    { 0x80bf2e70, 75 },
    { 0x860f2e74, 90 },
    { 0x191f2e76, 23 },
    { 0x557f2e76, 90 },
    { 0xec7f2e77, 95 },
    { 0, 0 },
    { 0xf1cf2e7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9f2e7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf2e81, 33 },
    { 0xde6f2e82, 75 },
    { 0xd2cf2e83, 33 },
    { 0xe77f2e82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f2e87, 61 },
    { 0, 0 },
    { 0x5bef2e89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9f2e8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cf2e8f, 100 },
    { 0x4c6f2e8f, 93 },
    { 0xfd7f2e90, 61 },
    { 0x3d5f2e92, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f2e95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1f2e9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f2e9e, 33 },
    { 0x565f2e9f, 95 },
    { 0, 0 },
    { 0xa21f2ea1, 17 },
    { 0, 0 },
    { 0x4c4f2ea3, 56 },
    { 0x62f2ea4, 100 },
    { 0x955f2ea4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1f2ea8, 55 },
    { 0x845f2ea9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bf2ead, 23 },
    { 0xe18f2eae, 98 },
    { 0xf2cf2eaf, 61 },
    { 0x825f2eb0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2eb5, 76 },
    { 0x4f6f2eb6, 90 },
    { 0x7ddf2eb7, 33 },
    { 0x4f9f2eb7, 66 },
    { 0x8a7f2eb5, 90 },
    { 0xe4f2eba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff2ec3, 95 },
    { 0xefdf2ec3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff2ecc, 95 },
    { 0, 0 },
    { 0x384f2ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcf2ed3, 24 },
    { 0, 0 },
    { 0xd7cf2ed5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4f2ed9, 95 },
    { 0x132f2ed9, 90 },
    { 0, 0 },
    { 0x56af2edc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef2ee1, 33 },
    { 0x7f4f2ee1, 34 },
    { 0, 0 },
    { 0xf49f2ee4, 90 },
    { 0x7b4f2ee5, 98 },
    { 0x20ef2ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f2ef2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f2ef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aff2ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadef2efd, 90 },
    { 0xd08f2efe, 61 },
    { 0xcb3f2eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef2f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276f2f0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f2f0e, 33 },
    { 0, 0 },
    { 0xf4af2f10, 69 },
    { 0, 0 },
    { 0x98ef2f12, 99 },
    { 0x980f2f12, 17 },
    { 0x596f2f14, 19 },
    { 0x6d4f2f15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x715f2f18, 85 },
    { 0xc24f2f18, 61 },
    { 0, 0 },
    { 0x92cf2f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2f2f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f2f23, 53 },
    { 0, 0 },
    { 0xd47f2f25, 55 },
    { 0, 0 },
    { 0x472f2f27, 43 },
    { 0xd21f2f28, 43 },
    { 0x44ff2f28, 93 },
    { 0x498f2f29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf2f2e, 93 },
    { 0x374f2f2f, 90 },
    { 0, 0 },
    { 0x3edf2f31, 95 },
    { 0, 0 },
    { 0xa1cf2f33, 84 },
    { 0xdd2f2f34, 61 },
    { 0, 0 },
    { 0xfe2f2f36, 68 },
    { 0xe22f2f37, 90 },
    { 0x371f2f37, 66 },
    { 0x90ef2f39, 61 },
    { 0, 0 },
    { 0xdc0f2f3b, 34 },
    { 0x151f2f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ef2f3f, 61 },
    { 0x802f2f40, 91 },
    { 0x4bdf2f41, 33 },
    { 0xc4df2f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bf2f45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f2f49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f2f51, 90 },
    { 0x6c8f2f52, 33 },
    { 0, 0 },
    { 0xb62f2f54, 98 },
    { 0, 0 },
    { 0x90bf2f56, 23 },
    { 0, 0 },
    { 0x153f2f58, 88 },
    { 0x1ddf2f59, 61 },
    { 0x9f1f2f59, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f2f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cf2f60, 37 },
    { 0x77bf2f61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f2f66, 98 },
    { 0xe52f2f67, 76 },
    { 0xafdf2f68, 90 },
    { 0xb0af2f68, 91 },
    { 0x46af2f6a, 95 },
    { 0x1bff2f6a, 98 },
    { 0x928f2f6b, 23 },
    { 0, 0 },
    { 0x468f2f6e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f2f72, 61 },
    { 0x60ef2f73, 46 },
    { 0xdcef2f74, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f2f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf2f7d, 90 },
    { 0x712f2f7e, 89 },
    { 0, 0 },
    { 0xe2ef2f80, 90 },
    { 0xafef2f81, 46 },
    { 0x53af2f81, 55 },
    { 0xea4f2f83, 61 },
    { 0xad9f2f83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9f2f8b, 68 },
    { 0x929f2f8b, 76 },
    { 0x3eff2f8b, 85 },
    { 0xedff2f8e, 85 },
    { 0x46af2f8f, 95 },
    { 0x3d6f2f8f, 61 },
    { 0x9ddf2f91, 66 },
    { 0, 0 },
    { 0xe1af2f93, 33 },
    { 0x8b1f2f94, 95 },
    { 0xaa1f2f94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f2f9d, 98 },
    { 0, 0 },
    { 0x240f2f9f, 84 },
    { 0xd7ff2f9f, 95 },
    { 0x665f2fa1, 90 },
    { 0xd8cf2fa2, 36 },
    { 0, 0 },
    { 0x891f2fa4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f2fa7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f2faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cf2faf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf2fb2, 91 },
    { 0, 0 },
    { 0xefef2fb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579f2fb8, 85 },
    { 0, 0 },
    { 0x23bf2fba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0f2fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bf2fc1, 33 },
    { 0, 0 },
    { 0xf98f2fc3, 52 },
    { 0xf8af2fc3, 66 },
    { 0x35df2fc4, 61 },
    { 0, 0 },
    { 0xf7af2fc7, 33 },
    { 0, 0 },
    { 0xc4cf2fc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdf2fcc, 87 },
    { 0x79ff2fcd, 95 },
    { 0x6dff2fce, 61 },
    { 0x386f2fcf, 76 },
    { 0x68ff2fcf, 61 },
    { 0x80cf2fd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214f2fd8, 90 },
    { 0, 0 },
    { 0x7a0f2fda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af2fe3, 95 },
    { 0x8b2f2fe4, 33 },
    { 0, 0 },
    { 0xe5cf2fe6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28f2fe9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f2fec, 34 },
    { 0x217f2fed, 76 },
    { 0x4eff2fed, 95 },
    { 0x8bff2fef, 85 },
    { 0x7baf2fed, 19 },
    { 0x864f2fef, 34 },
    { 0x7aaf2fef, 38 },
    { 0, 0 },
    { 0x3a1f2ff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebf2ff7, 23 },
    { 0, 0 },
    { 0x5a7f2ff9, 99 },
    { 0xb72f2ff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f2fff, 78 },
    { 0x8a5f2fff, 34 },
    { 0, 0 },
    { 0x883f3002, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff3007, 33 },
    { 0xea7f3008, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ef300e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ef3016, 95 },
    { 0x15cf3017, 93 },
    { 0x852f3016, 33 },
    { 0xe36f3019, 98 },
    { 0x6cf3018, 98 },
    { 0xb89f301b, 23 },
    { 0x3a6f301c, 80 },
    { 0xd3ff301d, 93 },
    { 0x8cf301d, 98 },
    { 0, 0 },
    { 0x31ff3020, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f3029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60f302f, 90 },
    { 0xc2f3030, 90 },
    { 0, 0 },
    { 0x45ef3032, 88 },
    { 0x412f3033, 93 },
    { 0, 0 },
    { 0xed0f3035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34df3038, 95 },
    { 0, 0 },
    { 0x9c3f303a, 35 },
    { 0x49af303b, 76 },
    { 0x9e4f303b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2f303f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39f3042, 84 },
    { 0xa3df3042, 90 },
    { 0, 0 },
    { 0x7e4f3045, 90 },
    { 0xb3f3046, 66 },
    { 0xbd7f3045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f304f, 95 },
    { 0, 0 },
    { 0x69af3051, 95 },
    { 0xd58f3052, 88 },
    { 0x4ebf3053, 66 },
    { 0, 0 },
    { 0x762f3055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f3059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbf3061, 87 },
    { 0x4c1f3062, 88 },
    { 0x6a8f3061, 34 },
    { 0, 0 },
    { 0xd04f3065, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf3068, 66 },
    { 0, 0 },
    { 0x73f306a, 90 },
    { 0, 0 },
    { 0x63af306c, 95 },
    { 0, 0 },
    { 0x16f306e, 98 },
    { 0x3fdf306f, 95 },
    { 0x23bf3070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f3076, 96 },
    { 0, 0 },
    { 0x226f3078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61f307b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56f307e, 34 },
    { 0xf4ef307f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13af3083, 61 },
    { 0, 0 },
    { 0x352f3085, 33 },
    { 0, 0 },
    { 0x84df3087, 98 },
    { 0x9eef3088, 61 },
    { 0xff6f3089, 34 },
    { 0x27df308a, 90 },
    { 0x16f308b, 70 },
    { 0x355f308b, 98 },
    { 0x88ef308c, 91 },
    { 0xdeff308e, 77 },
    { 0xda2f308a, 61 },
    { 0xe55f308c, 34 },
    { 0x222f308c, 23 },
    { 0x76bf3092, 33 },
    { 0, 0 },
    { 0xdf3094, 95 },
    { 0x80df3095, 95 },
    { 0, 0 },
    { 0x47cf3097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x833f309a, 61 },
    { 0, 0 },
    { 0x6d9f309c, 23 },
    { 0x112f309d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f30a1, 23 },
    { 0x94af30a2, 90 },
    { 0x19f30a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f30a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf30ae, 68 },
    { 0xe15f30af, 33 },
    { 0xb8cf30af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf30b5, 80 },
    { 0x1acf30b6, 68 },
    { 0xd56f30b6, 40 },
    { 0, 0 },
    { 0x274f30b9, 93 },
    { 0xb65f30ba, 66 },
    { 0x9cf30ba, 90 },
    { 0x605f30bc, 33 },
    { 0, 0 },
    { 0xb4f30be, 98 },
    { 0, 0 },
    { 0x8fdf30c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ff30c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ef30c6, 23 },
    { 0xb49f30c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f30cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf30cf, 98 },
    { 0, 0 },
    { 0x715f30d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432f30d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735f30db, 68 },
    { 0x8daf30dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92f30e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48f30e5, 81 },
    { 0x813f30e6, 34 },
    { 0x340f30e7, 61 },
    { 0x5acf30e7, 98 },
    { 0x649f30e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f30ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f30f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f30f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccf30fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f30fe, 23 },
    { 0xb44f30fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf3102, 33 },
    { 0x912f3102, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9f3106, 93 },
    { 0x518f3107, 94 },
    { 0x40ff3108, 90 },
    { 0x3b3f3109, 43 },
    { 0x8eaf310a, 33 },
    { 0, 0 },
    { 0xfc1f310c, 33 },
    { 0xbb7f310d, 90 },
    { 0x6a4f310e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ff3113, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df3116, 90 },
    { 0, 0 },
    { 0xb43f3118, 77 },
    { 0x2ff3119, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff311d, 90 },
    { 0x60cf311e, 17 },
    { 0, 0 },
    { 0xc09f3120, 55 },
    { 0, 0 },
    { 0x900f3122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf3127, 23 },
    { 0x471f3128, 90 },
    { 0xf66f3129, 61 },
    { 0, 0 },
    { 0x93ef312b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f312f, 55 },
    { 0xf35f312f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bf3136, 90 },
    { 0, 0 },
    { 0x4eef3138, 90 },
    { 0xb47f3139, 23 },
    { 0xec0f3139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bf313d, 95 },
    { 0x14cf313e, 90 },
    { 0x8a7f313e, 92 },
    { 0xcb5f3140, 61 },
    { 0, 0 },
    { 0x1baf3142, 23 },
    { 0x7ebf3143, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791f314d, 87 },
    { 0x5c3f314d, 76 },
    { 0xaf314f, 17 },
    { 0xcdff3150, 81 },
    { 0, 0 },
    { 0xbb9f3152, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854f3156, 90 },
    { 0, 0 },
    { 0xabdf3158, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af315d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df3160, 98 },
    { 0xcbf3160, 75 },
    { 0x884f3162, 90 },
    { 0x2d4f3161, 33 },
    { 0xc88f3164, 55 },
    { 0x927f3162, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff3168, 65 },
    { 0x2d6f3169, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf316c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5f3170, 91 },
    { 0, 0 },
    { 0xafff3172, 90 },
    { 0, 0 },
    { 0x374f3174, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f3177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcef317a, 55 },
    { 0xe06f317b, 76 },
    { 0xbb5f317b, 88 },
    { 0xec7f317c, 46 },
    { 0x326f317e, 23 },
    { 0, 0 },
    { 0xe6af3180, 33 },
    { 0x3f1f3181, 93 },
    { 0, 0 },
    { 0x56ff3183, 55 },
    { 0xddcf3183, 33 },
    { 0x46f3184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf3189, 75 },
    { 0, 0 },
    { 0x37df318b, 23 },
    { 0, 0 },
    { 0xfdff318d, 99 },
    { 0x19ff318e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02f3192, 33 },
    { 0, 0 },
    { 0x311f3194, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f3198, 64 },
    { 0xf8ef3198, 53 },
    { 0x6af3198, 61 },
    { 0, 0 },
    { 0xcb7f319c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f319f, 90 },
    { 0xbe5f31a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf31ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff31b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f31bc, 61 },
    { 0x8faf31bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4f31c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff31c9, 90 },
    { 0xda2f31ca, 76 },
    { 0x840f31cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f31d0, 55 },
    { 0, 0 },
    { 0x5dcf31d2, 23 },
    { 0xb21f31d3, 43 },
    { 0x759f31d4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ef31d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x474f31da, 90 },
    { 0x77ff31db, 88 },
    { 0xef9f31db, 91 },
    { 0, 0 },
    { 0xb39f31de, 33 },
    { 0xaddf31df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65f31e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65f31e8, 48 },
    { 0, 0 },
    { 0x435f31ea, 33 },
    { 0x391f31ea, 67 },
    { 0x998f31ec, 61 },
    { 0x312f31ec, 65 },
    { 0xca6f31ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df31f1, 52 },
    { 0x23df31f2, 93 },
    { 0x818f31f3, 98 },
    { 0x682f31f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df31f8, 33 },
    { 0xb4ff31f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9f3200, 91 },
    { 0x275f3200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f3205, 90 },
    { 0xbabf3205, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf320b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bf320e, 95 },
    { 0x603f320f, 52 },
    { 0x4b4f3210, 33 },
    { 0, 0 },
    { 0x2cf3212, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf3218, 34 },
    { 0, 0 },
    { 0x519f321a, 23 },
    { 0x5f0f321a, 33 },
    { 0xe87f321c, 61 },
    { 0x680f321d, 68 },
    { 0, 0 },
    { 0x308f321f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ef322c, 76 },
    { 0x1a8f322d, 90 },
    { 0, 0 },
    { 0xd65f322f, 46 },
    { 0x965f322f, 23 },
    { 0xf8df3231, 90 },
    { 0xd9ef3230, 86 },
    { 0x6b0f3233, 33 },
    { 0, 0 },
    { 0x9d4f3235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f323a, 76 },
    { 0x644f323b, 90 },
    { 0, 0 },
    { 0xb15f323d, 91 },
    { 0x53f323e, 90 },
    { 0x956f323f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f3243, 66 },
    { 0xe9f3244, 61 },
    { 0, 0 },
    { 0xd4af3246, 33 },
    { 0, 0 },
    { 0x6e0f3248, 33 },
    { 0x7e4f3249, 33 },
    { 0x166f324a, 46 },
    { 0xdb0f3248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abf324e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f3254, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f3257, 90 },
    { 0xbb4f3258, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f325c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651f3260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cf3265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f326b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef3270, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19df3274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bf3277, 98 },
    { 0x70ff3278, 76 },
    { 0x828f3279, 84 },
    { 0, 0 },
    { 0x951f327b, 66 },
    { 0x2cdf327b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f327f, 98 },
    { 0x7d7f3280, 33 },
    { 0x4f7f3281, 61 },
    { 0, 0 },
    { 0x9e0f3283, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef3286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f3289, 87 },
    { 0, 0 },
    { 0xdc3f328b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f3290, 91 },
    { 0xd95f3291, 84 },
    { 0, 0 },
    { 0x999f3293, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60f3297, 84 },
    { 0, 0 },
    { 0xb96f3299, 90 },
    { 0x225f329a, 98 },
    { 0x6d7f329b, 33 },
    { 0x1ef329b, 61 },
    { 0x1c0f329d, 99 },
    { 0xd06f329d, 23 },
    { 0x127f329b, 90 },
    { 0xb1ef329b, 61 },
    { 0x100f32a1, 87 },
    { 0xaa5f32a2, 98 },
    { 0x68f32a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f32a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90df32af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f32b2, 55 },
    { 0, 0 },
    { 0x6a2f32b4, 88 },
    { 0x30df32b4, 93 },
    { 0, 0 },
    { 0xbd2f32b7, 85 },
    { 0xf88f32b7, 76 },
    { 0xba9f32b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf32bc, 101 },
    { 0x6f6f32bd, 33 },
    { 0x94ef32bd, 67 },
    { 0, 0 },
    { 0xb62f32c0, 90 },
    { 0x24cf32c1, 90 },
    { 0, 0 },
    { 0xfbdf32c3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cf32c7, 55 },
    { 0xf17f32c7, 87 },
    { 0xa55f32c9, 98 },
    { 0x3f2f32ca, 90 },
    { 0x72ef32ca, 95 },
    { 0x129f32cc, 33 },
    { 0xffff32cd, 23 },
    { 0x4af32ce, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f32d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdf32d5, 90 },
    { 0x1a5f32d6, 84 },
    { 0xbe2f32d7, 46 },
    { 0x89ff32d8, 65 },
    { 0x109f32d8, 90 },
    { 0x83df32d5, 55 },
    { 0x7aff32da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf32de, 61 },
    { 0x4af32df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff32e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf32e6, 90 },
    { 0, 0 },
    { 0xdc6f32e8, 61 },
    { 0xceaf32e8, 46 },
    { 0x7faf32e8, 8 },
    { 0x961f32ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524f32ef, 53 },
    { 0x53ff32f0, 98 },
    { 0x88f32f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df32f8, 23 },
    { 0x22bf32f8, 87 },
    { 0xa6f32f8, 55 },
    { 0, 0 },
    { 0xff5f32fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f3302, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x183f3305, 61 },
    { 0x501f3306, 93 },
    { 0x62bf3307, 87 },
    { 0xb47f3307, 55 },
    { 0, 0 },
    { 0x3f4f330a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60af3312, 34 },
    { 0x4e6f3313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ff3316, 33 },
    { 0, 0 },
    { 0x2c9f3318, 98 },
    { 0, 0 },
    { 0x5e5f331a, 23 },
    { 0x581f331a, 23 },
    { 0, 0 },
    { 0xe7df331d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf3324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27f3328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73df332c, 46 },
    { 0x3aaf332d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41f3332, 61 },
    { 0x539f3333, 61 },
    { 0x8caf3334, 90 },
    { 0, 0 },
    { 0xf61f3336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36df333a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f333e, 76 },
    { 0, 0 },
    { 0x69bf3340, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8f3343, 33 },
    { 0x65cf3344, 34 },
    { 0xd0f3345, 34 },
    { 0xcf6f3345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af334a, 55 },
    { 0, 0 },
    { 0x298f334c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eff334f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918f335b, 77 },
    { 0x70df335c, 95 },
    { 0, 0 },
    { 0x2d3f335e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81f3361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ef3365, 88 },
    { 0x9f3366, 76 },
    { 0, 0 },
    { 0xd6df3368, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115f336f, 76 },
    { 0, 0 },
    { 0x869f3371, 61 },
    { 0x606f3372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x204f3375, 23 },
    { 0xd7cf3376, 91 },
    { 0xdd0f3377, 76 },
    { 0, 0 },
    { 0xf44f3379, 95 },
    { 0x717f3379, 61 },
    { 0, 0 },
    { 0x888f337c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9f337f, 95 },
    { 0xf56f3380, 95 },
    { 0xe7df3380, 33 },
    { 0, 0 },
    { 0xa22f3383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178f338c, 61 },
    { 0xe26f338d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f3390, 81 },
    { 0x721f3391, 93 },
    { 0xc2df3392, 85 },
    { 0x6e6f3392, 99 },
    { 0x139f3390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f3397, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87f339f, 43 },
    { 0xba1f33a0, 38 },
    { 0, 0 },
    { 0x30df33a2, 85 },
    { 0, 0 },
    { 0x2c7f33a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52f33a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4f33ac, 34 },
    { 0xaedf33ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f33b0, 99 },
    { 0xdc4f33b0, 34 },
    { 0, 0 },
    { 0x99cf33b3, 39 },
    { 0x878f33b4, 80 },
    { 0x13af33b5, 87 },
    { 0xdfff33b6, 95 },
    { 0, 0 },
    { 0x758f33b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf33bb, 78 },
    { 0x46bf33bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6f33bf, 85 },
    { 0xd38f33bf, 95 },
    { 0, 0 },
    { 0xb0af33c2, 55 },
    { 0, 0 },
    { 0xb7f33c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07f33c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f33cb, 36 },
    { 0, 0 },
    { 0x13cf33cd, 65 },
    { 0x7cf33ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af33d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7f33d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99f33da, 97 },
    { 0, 0 },
    { 0x8aff33dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f33e0, 34 },
    { 0, 0 },
    { 0x9a9f33e2, 61 },
    { 0xf30f33e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f33e8, 81 },
    { 0x98f33e8, 66 },
    { 0x320f33e8, 46 },
    { 0x6d7f33eb, 33 },
    { 0x278f33e8, 98 },
    { 0x890f33ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52f33fd, 33 },
    { 0x23df33fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1df3403, 55 },
    { 0x8af3404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf3408, 76 },
    { 0xee7f3408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4f340d, 81 },
    { 0x423f340d, 99 },
    { 0x67ff340f, 81 },
    { 0x6eef340f, 90 },
    { 0, 0 },
    { 0x839f3412, 46 },
    { 0x89ef3413, 98 },
    { 0xfd8f3414, 90 },
    { 0x7d3f3413, 55 },
    { 0x905f3416, 90 },
    { 0xe18f3416, 61 },
    { 0x6adf3418, 83 },
    { 0xf12f3419, 78 },
    { 0x3f6f3418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f341e, 55 },
    { 0xdfef341f, 46 },
    { 0x918f3420, 76 },
    { 0x92ff3421, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f3428, 98 },
    { 0xb7ef3429, 98 },
    { 0x737f3428, 65 },
    { 0xab7f342b, 61 },
    { 0, 0 },
    { 0x898f342d, 90 },
    { 0, 0 },
    { 0x13af342f, 90 },
    { 0x1e3f3430, 88 },
    { 0x38cf3431, 85 },
    { 0xf26f3430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3435, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f343d, 33 },
    { 0, 0 },
    { 0x21ef343f, 77 },
    { 0, 0 },
    { 0x53ff3441, 33 },
    { 0x10af3442, 100 },
    { 0xb17f3442, 33 },
    { 0xd2cf3443, 98 },
    { 0, 0 },
    { 0x924f3446, 23 },
    { 0, 0 },
    { 0xb93f3448, 99 },
    { 0x476f3449, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff344e, 64 },
    { 0xf21f344e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf3453, 76 },
    { 0x883f3454, 52 },
    { 0xa64f3455, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2f345d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89f3462, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf3467, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f346b, 66 },
    { 0, 0 },
    { 0x2fbf346d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf3474, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f3479, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f3481, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f3486, 55 },
    { 0x4fbf3486, 95 },
    { 0x268f3488, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf348d, 33 },
    { 0, 0 },
    { 0xe74f348f, 90 },
    { 0, 0 },
    { 0x72bf3491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56df3496, 61 },
    { 0xba5f3497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff349b, 98 },
    { 0xcef349c, 23 },
    { 0x8adf349b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f34a0, 23 },
    { 0xf7cf34a1, 55 },
    { 0x623f34a0, 88 },
    { 0xa68f34a1, 77 },
    { 0x5fef34a3, 33 },
    { 0xbd6f34a3, 61 },
    { 0x3ddf34a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f34aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9f34ad, 95 },
    { 0, 0 },
    { 0x735f34af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bf34b4, 88 },
    { 0xe3f34b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf34bd, 55 },
    { 0, 0 },
    { 0x74ff34bf, 88 },
    { 0, 0 },
    { 0xd1af34c1, 61 },
    { 0, 0 },
    { 0xcaf34c3, 33 },
    { 0, 0 },
    { 0x46cf34c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef34ce, 34 },
    { 0xaddf34cf, 76 },
    { 0x651f34cf, 61 },
    { 0, 0 },
    { 0x42cf34d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205f34d7, 95 },
    { 0x494f34d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f34db, 85 },
    { 0x1dbf34db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47f34e0, 99 },
    { 0xcb0f34e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf34e5, 100 },
    { 0x387f34e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf34e9, 40 },
    { 0x7ecf34ea, 23 },
    { 0x891f34e9, 52 },
    { 0x981f34eb, 34 },
    { 0x8cf34ed, 55 },
    { 0x419f34e9, 95 },
    { 0, 0 },
    { 0xfb4f34f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f34fb, 61 },
    { 0x8cbf34fc, 77 },
    { 0x3a1f34fd, 61 },
    { 0xf03f34fd, 43 },
    { 0xa34f34ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf3503, 64 },
    { 0xeb4f3504, 66 },
    { 0xb97f3504, 93 },
    { 0xb4cf3506, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f350b, 75 },
    { 0x978f350b, 90 },
    { 0x98bf350d, 33 },
    { 0xa58f350e, 81 },
    { 0xca4f350c, 76 },
    { 0, 0 },
    { 0x16af3511, 61 },
    { 0xc03f3511, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebf3517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9f351b, 56 },
    { 0x54ef351c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f3520, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaf3525, 90 },
    { 0xfd1f3526, 43 },
    { 0x8d2f3525, 93 },
    { 0x6b4f3528, 34 },
    { 0, 0 },
    { 0xf14f352a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76df352f, 78 },
    { 0xffcf3530, 43 },
    { 0xea6f3530, 43 },
    { 0, 0 },
    { 0x871f3533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def353c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f353f, 23 },
    { 0, 0 },
    { 0x370f3541, 61 },
    { 0, 0 },
    { 0xecf3543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65f354b, 34 },
    { 0x9abf354c, 76 },
    { 0x94f354d, 90 },
    { 0x124f354e, 40 },
    { 0x369f354f, 66 },
    { 0x3e0f3550, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f3555, 87 },
    { 0x18bf3556, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ef355b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9f3562, 55 },
    { 0x204f3563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7f3566, 23 },
    { 0, 0 },
    { 0xf59f3568, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f356d, 80 },
    { 0xc22f356d, 61 },
    { 0x205f356f, 61 },
    { 0xf7bf356f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ef3574, 61 },
    { 0xd9ef3574, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1f357c, 33 },
    { 0xdcbf357d, 33 },
    { 0xf38f357e, 55 },
    { 0xf82f357f, 98 },
    { 0xe9cf357e, 76 },
    { 0x515f3580, 99 },
    { 0xd8ff357d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df3585, 61 },
    { 0x16cf3586, 61 },
    { 0xc8bf3586, 61 },
    { 0xb20f3588, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2f3591, 61 },
    { 0xf59f3592, 61 },
    { 0xe45f3593, 55 },
    { 0x8d1f3593, 88 },
    { 0xb16f3593, 43 },
    { 0x420f3596, 61 },
    { 0x41ef3593, 95 },
    { 0xf25f3598, 33 },
    { 0x9bef3598, 61 },
    { 0, 0 },
    { 0xd53f359b, 88 },
    { 0, 0 },
    { 0xe7ff359d, 100 },
    { 0, 0 },
    { 0x9b3f359f, 66 },
    { 0, 0 },
    { 0xa3ff35a1, 99 },
    { 0xefdf35a2, 23 },
    { 0, 0 },
    { 0xb22f35a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af35a7, 95 },
    { 0xa32f35a8, 95 },
    { 0xb24f35a9, 61 },
    { 0x4d9f35aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f35ad, 67 },
    { 0xc6cf35ae, 33 },
    { 0, 0 },
    { 0x45df35b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f35b5, 95 },
    { 0, 0 },
    { 0x83af35b7, 99 },
    { 0, 0 },
    { 0xcaff35b9, 98 },
    { 0xa71f35b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ff35bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f35c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f35c6, 52 },
    { 0x9dff35c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f35cb, 24 },
    { 0xd56f35cc, 23 },
    { 0x418f35cb, 98 },
    { 0xa9df35cd, 90 },
    { 0, 0 },
    { 0x3aaf35d0, 61 },
    { 0xd44f35d1, 66 },
    { 0x301f35d2, 90 },
    { 0x9cbf35d3, 98 },
    { 0, 0 },
    { 0x8c2f35d5, 90 },
    { 0x665f35d5, 33 },
    { 0xe57f35d5, 61 },
    { 0x215f35d7, 61 },
    { 0xad7f35d7, 95 },
    { 0xfa4f35d8, 61 },
    { 0, 0 },
    { 0xcaff35dc, 33 },
    { 0, 0 },
    { 0xf5bf35de, 76 },
    { 0xccbf35df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26f35e2, 76 },
    { 0x714f35e2, 43 },
    { 0x82cf35e4, 98 },
    { 0x1f4f35e3, 17 },
    { 0x693f35e2, 43 },
    { 0x7f3f35e4, 87 },
    { 0x6caf35e8, 98 },
    { 0x7bff35e9, 98 },
    { 0, 0 },
    { 0x56f35eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf35ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525f35f2, 49 },
    { 0x80df35f3, 98 },
    { 0x7cf35f3, 43 },
    { 0x4c2f35f4, 90 },
    { 0, 0 },
    { 0xa44f35f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf3600, 98 },
    { 0x3ebf3600, 23 },
    { 0xd19f3600, 90 },
    { 0x9f1f3601, 95 },
    { 0x427f3604, 93 },
    { 0xd09f3605, 76 },
    { 0x7f7f3605, 76 },
    { 0x42af3603, 34 },
    { 0xfa6f3600, 98 },
    { 0xb89f3605, 55 },
    { 0x341f360a, 33 },
    { 0x7f7f360b, 90 },
    { 0x371f360c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51f3612, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130f361b, 80 },
    { 0x538f361c, 61 },
    { 0x469f361d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7f3626, 33 },
    { 0x868f3627, 34 },
    { 0x3bef3628, 95 },
    { 0x3ef3628, 38 },
    { 0, 0 },
    { 0x352f362b, 95 },
    { 0x173f362c, 61 },
    { 0, 0 },
    { 0x776f362e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cf3631, 90 },
    { 0x38bf3632, 69 },
    { 0x29ff3633, 87 },
    { 0x3b5f3634, 55 },
    { 0x182f3635, 78 },
    { 0xa5af3636, 43 },
    { 0xfd6f3632, 75 },
    { 0xfdff3636, 95 },
    { 0x913f3638, 23 },
    { 0, 0 },
    { 0xca0f363b, 88 },
    { 0xadbf363b, 95 },
    { 0xf82f363d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f3643, 85 },
    { 0x841f3644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68f3648, 61 },
    { 0xcb0f3649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f364c, 53 },
    { 0xd0f364d, 55 },
    { 0xa05f364d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af3651, 95 },
    { 0x605f3652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bf3655, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9caf3659, 61 },
    { 0xea5f3659, 52 },
    { 0xe85f365b, 61 },
    { 0, 0 },
    { 0x23bf365d, 90 },
    { 0x547f365d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7f366f, 95 },
    { 0xd0af3670, 90 },
    { 0x940f3671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11af3676, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x976f3679, 33 },
    { 0, 0 },
    { 0x572f367b, 76 },
    { 0xcaaf367c, 84 },
    { 0x628f367d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5df3681, 56 },
    { 0xed4f3681, 43 },
    { 0xc7ef3683, 76 },
    { 0x541f3681, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef3688, 90 },
    { 0x6a7f3689, 95 },
    { 0, 0 },
    { 0x37af368b, 23 },
    { 0xefbf368c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccf3692, 76 },
    { 0, 0 },
    { 0xbb4f3694, 77 },
    { 0xa5df3694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf3698, 33 },
    { 0x4eef3698, 33 },
    { 0xf04f369a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1f36a1, 78 },
    { 0xbb1f36a1, 76 },
    { 0xe8f36a2, 83 },
    { 0x245f36a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df36ad, 81 },
    { 0x280f36ad, 34 },
    { 0xc4af36af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0f36b7, 33 },
    { 0x812f36b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25f36c2, 84 },
    { 0x85af36c2, 46 },
    { 0, 0 },
    { 0xacff36c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83f36ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caf36cd, 99 },
    { 0, 0 },
    { 0x382f36cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bf36d3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf36dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522f36e3, 98 },
    { 0x15ff36e4, 33 },
    { 0x676f36e5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6f36ed, 23 },
    { 0xdf2f36ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f36f3, 61 },
    { 0xcf36f4, 76 },
    { 0x46f36f4, 90 },
    { 0xe6df36f6, 95 },
    { 0, 0 },
    { 0xf45f36f8, 77 },
    { 0, 0 },
    { 0x2f0f36fa, 61 },
    { 0xcb4f36fb, 52 },
    { 0x224f36fb, 98 },
    { 0xc4af36fb, 66 },
    { 0, 0 },
    { 0xf41f36ff, 34 },
    { 0x17cf3700, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f3703, 76 },
    { 0x957f3704, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7f370f, 33 },
    { 0xcd3f3710, 64 },
    { 0, 0 },
    { 0xa4cf3712, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81f3715, 78 },
    { 0x2ccf3716, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77f3719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f371d, 33 },
    { 0, 0 },
    { 0x19ef371f, 88 },
    { 0x3d3f3720, 91 },
    { 0x7b4f3721, 76 },
    { 0xbb4f3722, 98 },
    { 0xffff3721, 23 },
    { 0xc22f3721, 61 },
    { 0x4ebf3723, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47f372b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af3733, 76 },
    { 0xdd1f3734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f3738, 33 },
    { 0xbf7f3739, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339f373d, 85 },
    { 0x5f7f373e, 52 },
    { 0x20cf373f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dff3748, 23 },
    { 0, 0 },
    { 0x74f374a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08f374d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bf3751, 68 },
    { 0x218f3751, 61 },
    { 0xe9af3753, 95 },
    { 0x14ef3753, 23 },
    { 0xc4ef3753, 80 },
    { 0x3a9f3755, 23 },
    { 0, 0 },
    { 0xfbef3758, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55af375b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f375f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9f3762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f3765, 90 },
    { 0x67af3766, 99 },
    { 0, 0 },
    { 0xce6f3768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f376b, 98 },
    { 0x315f376c, 23 },
    { 0x428f376d, 95 },
    { 0x47bf376e, 76 },
    { 0x1fdf376f, 85 },
    { 0x2e5f376f, 90 },
    { 0x81f3771, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02f377e, 90 },
    { 0xe10f377f, 55 },
    { 0xb61f377e, 90 },
    { 0, 0 },
    { 0xa11f3782, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f3786, 66 },
    { 0, 0 },
    { 0xb07f3788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756f378c, 90 },
    { 0xcd0f378d, 61 },
    { 0xaaaf378d, 98 },
    { 0x161f378c, 61 },
    { 0x79f3790, 76 },
    { 0xf3cf3790, 33 },
    { 0xd87f378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df3797, 95 },
    { 0x14df3798, 95 },
    { 0x45af3799, 46 },
    { 0xb14f3799, 61 },
    { 0x807f3798, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af379f, 43 },
    { 0xe31f37a0, 77 },
    { 0x6a0f37a1, 33 },
    { 0, 0 },
    { 0xcb4f37a3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f37a8, 87 },
    { 0, 0 },
    { 0x99df37aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8f37b5, 95 },
    { 0x562f37b5, 91 },
    { 0, 0 },
    { 0xc92f37b8, 52 },
    { 0x177f37b8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f37be, 33 },
    { 0xc43f37bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f37c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecf37c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf37d0, 55 },
    { 0xb60f37d1, 61 },
    { 0, 0 },
    { 0x612f37d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1f37dd, 98 },
    { 0xf4cf37de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39f37e2, 23 },
    { 0xf9ff37e3, 61 },
    { 0xb99f37e2, 76 },
    { 0x68af37e5, 77 },
    { 0xb56f37e6, 46 },
    { 0xd19f37e6, 43 },
    { 0x75ef37e5, 87 },
    { 0xb27f37e9, 87 },
    { 0xde2f37e9, 33 },
    { 0x932f37eb, 67 },
    { 0xaadf37ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf37f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f37fa, 23 },
    { 0x363f37fb, 90 },
    { 0xcc6f37fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf3800, 95 },
    { 0x5e2f3800, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613f3812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cf3819, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0f3821, 76 },
    { 0x9df3822, 33 },
    { 0x90bf3823, 100 },
    { 0, 0 },
    { 0xe0df3825, 46 },
    { 0x2cef3826, 91 },
    { 0x6eef3827, 61 },
    { 0xa74f3827, 23 },
    { 0x884f3828, 90 },
    { 0x2bbf3825, 55 },
    { 0x770f3828, 90 },
    { 0, 0 },
    { 0xe6ef382d, 55 },
    { 0, 0 },
    { 0x329f382f, 23 },
    { 0x199f382f, 33 },
    { 0xc6ff382f, 93 },
    { 0xc0bf3832, 91 },
    { 0xa38f3832, 95 },
    { 0xa1df3834, 33 },
    { 0, 0 },
    { 0xc22f3836, 100 },
    { 0, 0 },
    { 0x67df3838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31f383e, 90 },
    { 0xe02f383f, 33 },
    { 0xc2ff3840, 98 },
    { 0xff3f3841, 95 },
    { 0xe96f3841, 55 },
    { 0xc78f383f, 95 },
    { 0x3cff3844, 66 },
    { 0xfeef3845, 90 },
    { 0xbe5f3841, 93 },
    { 0x139f3840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af3851, 98 },
    { 0xe17f3852, 55 },
    { 0x999f3852, 95 },
    { 0x1b4f3854, 76 },
    { 0x67cf3853, 95 },
    { 0xe38f3854, 83 },
    { 0, 0 },
    { 0x9a6f3858, 66 },
    { 0xf4df3859, 38 },
    { 0xf0ef385a, 66 },
    { 0x908f385b, 98 },
    { 0x65bf3858, 75 },
    { 0, 0 },
    { 0xb8cf385e, 61 },
    { 0x814f385e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f386e, 61 },
    { 0x309f386f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af3875, 61 },
    { 0xbecf3876, 23 },
    { 0xd92f3877, 90 },
    { 0x7bbf3878, 78 },
    { 0x52df3875, 33 },
    { 0, 0 },
    { 0x911f387b, 98 },
    { 0x5cff387c, 46 },
    { 0x306f387d, 43 },
    { 0xdb9f387e, 46 },
    { 0xa67f387f, 61 },
    { 0x5f8f387c, 90 },
    { 0xe0cf3881, 67 },
    { 0, 0 },
    { 0x70cf3883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f3887, 66 },
    { 0, 0 },
    { 0x845f3889, 34 },
    { 0x218f388a, 95 },
    { 0xcc9f388b, 84 },
    { 0xc35f388c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cf3891, 61 },
    { 0x80ef3892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446f3899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67af389d, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abf38a9, 95 },
    { 0x46df38a9, 90 },
    { 0x710f38a9, 23 },
    { 0, 0 },
    { 0x8c6f38ad, 90 },
    { 0x815f38ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85f38b2, 23 },
    { 0x56df38b3, 33 },
    { 0x4fbf38b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf38b9, 23 },
    { 0xf95f38ba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4f38c0, 91 },
    { 0, 0 },
    { 0x734f38c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906f38c8, 90 },
    { 0xb35f38c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98f38ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x220f38d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569f38d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06f38dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f38e3, 23 },
    { 0x2f4f38e4, 90 },
    { 0x3dff38e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f38ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922f38f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f38f6, 98 },
    { 0x2e1f38f6, 90 },
    { 0, 0 },
    { 0x880f38f9, 75 },
    { 0x208f38f9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82f38fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f3903, 61 },
    { 0x957f3903, 23 },
    { 0x76af3904, 90 },
    { 0x237f3906, 61 },
    { 0x378f3907, 61 },
    { 0x99cf3908, 33 },
    { 0x6b5f3909, 90 },
    { 0x4a7f390a, 36 },
    { 0x3e4f390b, 98 },
    { 0x903f390c, 78 },
    { 0x88bf390d, 74 },
    { 0x180f390d, 52 },
    { 0x310f390f, 33 },
    { 0xbcef390a, 61 },
    { 0xa25f3909, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff3917, 78 },
    { 0xa6bf3917, 99 },
    { 0xba7f3919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8f3921, 99 },
    { 0, 0 },
    { 0x116f3923, 61 },
    { 0x5d8f3924, 76 },
    { 0x22f3925, 61 },
    { 0xdd8f3926, 81 },
    { 0x8c6f3927, 40 },
    { 0x9f2f3927, 33 },
    { 0x57bf3929, 95 },
    { 0x14df392a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bf392e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7f3934, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5f3937, 77 },
    { 0, 0 },
    { 0x28af3939, 55 },
    { 0, 0 },
    { 0xf17f393b, 61 },
    { 0x7b7f393c, 55 },
    { 0xf17f393d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ef3941, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bf3946, 90 },
    { 0x835f3946, 98 },
    { 0xd6cf3948, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef394e, 33 },
    { 0, 0 },
    { 0x516f3950, 87 },
    { 0x55df3950, 67 },
    { 0x95cf3951, 85 },
    { 0x376f3953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8f3957, 55 },
    { 0x93af3957, 85 },
    { 0x52df3959, 33 },
    { 0xf09f3959, 48 },
    { 0x9e9f395b, 23 },
    { 0x880f3958, 61 },
    { 0xfcaf395b, 84 },
    { 0x15af395e, 76 },
    { 0xd4cf395b, 48 },
    { 0, 0 },
    { 0xec5f3961, 76 },
    { 0x755f3962, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf3965, 95 },
    { 0, 0 },
    { 0x6dff3967, 90 },
    { 0x96ef3967, 23 },
    { 0x78ff3969, 90 },
    { 0x44ff3969, 95 },
    { 0x644f396b, 99 },
    { 0x541f396c, 87 },
    { 0, 0 },
    { 0xcf0f396e, 46 },
    { 0, 0 },
    { 0x89f3970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f3974, 33 },
    { 0, 0 },
    { 0xd9cf3976, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f397a, 90 },
    { 0x117f397b, 75 },
    { 0, 0 },
    { 0xb6af397d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf3985, 52 },
    { 0x71df3986, 88 },
    { 0xdbcf3986, 90 },
    { 0xc93f3985, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedf398d, 100 },
    { 0xe06f398e, 46 },
    { 0, 0 },
    { 0xaf1f3990, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaf399b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf399e, 100 },
    { 0x3f6f399f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6af39a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ef39a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f39aa, 88 },
    { 0, 0 },
    { 0xcb3f39ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f39b1, 52 },
    { 0, 0 },
    { 0x20bf39b3, 90 },
    { 0, 0 },
    { 0x60df39b5, 98 },
    { 0, 0 },
    { 0xe5f39b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08f39c3, 38 },
    { 0, 0 },
    { 0xddbf39c5, 33 },
    { 0x39f39c6, 98 },
    { 0xae1f39c6, 76 },
    { 0x461f39c8, 43 },
    { 0xe64f39c8, 95 },
    { 0x9e4f39c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bf39cf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36f39d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40af39d7, 55 },
    { 0x9fcf39d8, 53 },
    { 0x25ff39d7, 76 },
    { 0xeedf39da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7f39dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf39e0, 52 },
    { 0x14af39e0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7f39e5, 33 },
    { 0, 0 },
    { 0xb42f39e7, 17 },
    { 0, 0 },
    { 0x861f39e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2f39ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f39f6, 90 },
    { 0, 0 },
    { 0xf41f39f8, 43 },
    { 0x4b4f39f8, 95 },
    { 0xb6df39fa, 95 },
    { 0, 0 },
    { 0xfc7f39fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff3a01, 61 },
    { 0xee0f3a02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229f3a08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524f3a0c, 61 },
    { 0x9b4f3a0c, 90 },
    { 0xa68f3a0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f3a11, 76 },
    { 0xd1af3a11, 76 },
    { 0xbdff3a11, 34 },
    { 0xe39f3a14, 98 },
    { 0x87bf3a15, 46 },
    { 0, 0 },
    { 0x26bf3a17, 53 },
    { 0xd20f3a18, 23 },
    { 0, 0 },
    { 0x9eff3a1a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6f3a1d, 99 },
    { 0xefdf3a1e, 90 },
    { 0x23df3a1f, 90 },
    { 0, 0 },
    { 0x1def3a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f3a2b, 61 },
    { 0x13cf3a2c, 43 },
    { 0x302f3a2d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ff3a30, 17 },
    { 0xde9f3a31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ef3a34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f3a38, 95 },
    { 0, 0 },
    { 0x14ef3a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bf3a3e, 55 },
    { 0x5fff3a3f, 85 },
    { 0, 0 },
    { 0x2ddf3a41, 34 },
    { 0x24ef3a42, 95 },
    { 0xaa4f3a43, 95 },
    { 0x90f3a44, 93 },
    { 0xee5f3a45, 23 },
    { 0, 0 },
    { 0xf1f3a47, 23 },
    { 0xddff3a48, 33 },
    { 0, 0 },
    { 0x3b7f3a4a, 33 },
    { 0xf1ef3a4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f3a4e, 67 },
    { 0x9a9f3a4e, 33 },
    { 0x56cf3a50, 95 },
    { 0x3edf3a50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f3a54, 55 },
    { 0x4d0f3a54, 84 },
    { 0x2f2f3a54, 95 },
    { 0x846f3a57, 76 },
    { 0xb91f3a56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f3a5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdf3a5f, 5 },
    { 0, 0 },
    { 0x808f3a61, 61 },
    { 0, 0 },
    { 0xe82f3a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509f3a6a, 90 },
    { 0xa4ff3a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x838f3a6e, 55 },
    { 0x4adf3a6f, 98 },
    { 0xd3ef3a70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef3a74, 33 },
    { 0, 0 },
    { 0x74cf3a76, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff3a7b, 98 },
    { 0x1d7f3a7c, 61 },
    { 0x9acf3a7b, 99 },
    { 0xb94f3a7e, 81 },
    { 0x58cf3a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff3a86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0f3a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf3a91, 61 },
    { 0xdbdf3a92, 93 },
    { 0xa29f3a93, 95 },
    { 0, 0 },
    { 0xd9df3a95, 33 },
    { 0xbcdf3a96, 91 },
    { 0xa22f3a97, 76 },
    { 0x9a8f3a96, 38 },
    { 0xabdf3a97, 90 },
    { 0x92ff3a9a, 98 },
    { 0x64ef3a9b, 77 },
    { 0, 0 },
    { 0x45bf3a9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112f3aa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24f3aa9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43f3aac, 90 },
    { 0xebef3aad, 81 },
    { 0x147f3aae, 61 },
    { 0xbe2f3aaf, 88 },
    { 0x2c7f3aad, 65 },
    { 0x602f3ab0, 99 },
    { 0x8cbf3ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af3abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cf3abf, 90 },
    { 0, 0 },
    { 0x515f3ac1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f3ac4, 46 },
    { 0x1d6f3ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x662f3ac8, 43 },
    { 0xb17f3ac9, 33 },
    { 0xa65f3aca, 98 },
    { 0x5fdf3aca, 23 },
    { 0xb7ef3aca, 52 },
    { 0xf81f3acd, 66 },
    { 0x97ff3ac9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f3ad1, 95 },
    { 0xcb9f3ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf3ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871f3ade, 23 },
    { 0x361f3adf, 90 },
    { 0xa88f3adf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf3ae3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98f3ae6, 91 },
    { 0xd41f3ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1f3aea, 98 },
    { 0xefbf3aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f3af3, 5 },
    { 0xd8ff3af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f3af8, 55 },
    { 0, 0 },
    { 0x8f6f3afa, 95 },
    { 0, 0 },
    { 0xea3f3afc, 52 },
    { 0x122f3afc, 53 },
    { 0x506f3afc, 90 },
    { 0, 0 },
    { 0x518f3b00, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af3b0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf3b11, 91 },
    { 0, 0 },
    { 0x8ddf3b13, 33 },
    { 0x4b2f3b14, 33 },
    { 0x9c9f3b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df3b1b, 61 },
    { 0xc7af3b1c, 98 },
    { 0xf4ef3b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f3b23, 84 },
    { 0x9c8f3b23, 17 },
    { 0x3e7f3b25, 23 },
    { 0x5edf3b26, 33 },
    { 0xd25f3b23, 94 },
    { 0x218f3b23, 48 },
    { 0x81df3b29, 95 },
    { 0x958f3b2a, 93 },
    { 0xe2bf3b2b, 84 },
    { 0x491f3b2c, 87 },
    { 0xc10f3b24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f3b31, 52 },
    { 0x96af3b32, 61 },
    { 0x846f3b31, 33 },
    { 0x23bf3b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2f3b37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x913f3b3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ff3b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89f3b40, 34 },
    { 0, 0 },
    { 0x2b7f3b42, 90 },
    { 0x389f3b42, 43 },
    { 0xfddf3b42, 34 },
    { 0x8b5f3b43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff3b4a, 46 },
    { 0xabff3b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862f3b50, 65 },
    { 0x71af3b51, 33 },
    { 0xfc7f3b50, 61 },
    { 0x4af3b50, 98 },
    { 0xcabf3b54, 90 },
    { 0x4adf3b55, 34 },
    { 0xa3af3b56, 46 },
    { 0x268f3b55, 99 },
    { 0xdf0f3b58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208f3b5b, 36 },
    { 0x434f3b5c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x869f3b5f, 78 },
    { 0, 0 },
    { 0x6d2f3b61, 89 },
    { 0, 0 },
    { 0x8fcf3b63, 75 },
    { 0, 0 },
    { 0xac6f3b65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954f3b6e, 55 },
    { 0x545f3b6e, 43 },
    { 0, 0 },
    { 0x5f3b71, 61 },
    { 0, 0 },
    { 0xa95f3b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef3b77, 46 },
    { 0, 0 },
    { 0x7acf3b79, 33 },
    { 0, 0 },
    { 0xc36f3b7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf3b83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21df3b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1f3b8b, 90 },
    { 0, 0 },
    { 0x8cdf3b8d, 23 },
    { 0x5a8f3b8e, 43 },
    { 0x397f3b8e, 23 },
    { 0x214f3b8f, 100 },
    { 0x892f3b8e, 37 },
    { 0x161f3b8d, 34 },
    { 0xb99f3b8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef3b96, 77 },
    { 0x14f3b96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f3b9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f3ba0, 84 },
    { 0x1f1f3ba1, 95 },
    { 0x96f3ba1, 33 },
    { 0x8df3ba1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48f3bab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ef3baf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6f3bb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f3bb9, 90 },
    { 0x4a5f3bba, 95 },
    { 0x8c3f3bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2f3bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1df3bc6, 67 },
    { 0xdb7f3bc7, 90 },
    { 0xc14f3bc8, 99 },
    { 0xaa5f3bc9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bf3bce, 81 },
    { 0, 0 },
    { 0x91bf3bd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7f3bd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf3bda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72af3bde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f3be6, 61 },
    { 0xea3f3be7, 76 },
    { 0, 0 },
    { 0x806f3be9, 66 },
    { 0x950f3bea, 85 },
    { 0xceff3beb, 61 },
    { 0x1cef3bec, 90 },
    { 0xce6f3bed, 55 },
    { 0x6f4f3bee, 93 },
    { 0x336f3bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef3bf6, 90 },
    { 0x9c7f3bf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ef3c00, 67 },
    { 0, 0 },
    { 0x6c7f3c02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baf3c0b, 90 },
    { 0x967f3c0c, 98 },
    { 0xbc3f3c0c, 52 },
    { 0x4ecf3c0d, 98 },
    { 0xb93f3c0f, 23 },
    { 0x7cef3c0e, 33 },
    { 0xd54f3c11, 95 },
    { 0xe2cf3c12, 78 },
    { 0x50ff3c11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f3c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142f3c1c, 88 },
    { 0x195f3c1d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ff3c22, 33 },
    { 0, 0 },
    { 0x43df3c24, 76 },
    { 0x51bf3c24, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df3c29, 95 },
    { 0, 0 },
    { 0x257f3c2b, 46 },
    { 0x95cf3c2c, 98 },
    { 0x1bdf3c2b, 84 },
    { 0, 0 },
    { 0xb13f3c2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f3c36, 23 },
    { 0xb2af3c37, 98 },
    { 0x17f3c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f3c3d, 76 },
    { 0, 0 },
    { 0x46cf3c3f, 90 },
    { 0x818f3c40, 52 },
    { 0xcfef3c41, 99 },
    { 0x89f3c42, 46 },
    { 0x6fcf3c42, 67 },
    { 0x580f3c44, 98 },
    { 0xb40f3c43, 95 },
    { 0x1f1f3c3f, 61 },
    { 0x8fbf3c47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edf3c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0f3c50, 90 },
    { 0x4b0f3c51, 90 },
    { 0x80bf3c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf3c55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f3c5a, 93 },
    { 0xd7cf3c5a, 87 },
    { 0x56af3c5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7af3c60, 83 },
    { 0x87ff3c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2af3c65, 33 },
    { 0x710f3c66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484f3c6b, 43 },
    { 0x758f3c6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7f3c6f, 34 },
    { 0xef8f3c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf3c7c, 4 },
    { 0, 0 },
    { 0x7f7f3c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f3c81, 95 },
    { 0x4d7f3c82, 61 },
    { 0xd14f3c83, 61 },
    { 0x50f3c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f3c87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x807f3c8a, 33 },
    { 0xb55f3c8b, 89 },
    { 0xa3bf3c8b, 87 },
    { 0x7df3c8d, 33 },
    { 0, 0 },
    { 0x8a1f3c8f, 33 },
    { 0x246f3c8f, 52 },
    { 0, 0 },
    { 0x248f3c92, 61 },
    { 0x3cff3c93, 77 },
    { 0x671f3c94, 61 },
    { 0xc6ff3c95, 76 },
    { 0x9c1f3c96, 40 },
    { 0xbcef3c95, 76 },
    { 0xe2ff3c94, 34 },
    { 0, 0 },
    { 0x53df3c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f3ca2, 87 },
    { 0xf61f3ca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9f3caa, 97 },
    { 0x89ff3cab, 84 },
    { 0x81f3cac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f3cb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8f3cb3, 90 },
    { 0x734f3cb4, 76 },
    { 0, 0 },
    { 0xfa7f3cb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f3cb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4f3cc1, 23 },
    { 0, 0 },
    { 0x7baf3cc3, 78 },
    { 0xd0af3cc4, 23 },
    { 0xbd2f3cc3, 33 },
    { 0, 0 },
    { 0xcbbf3cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ef3cca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf3ccf, 91 },
    { 0xd4ef3cd0, 61 },
    { 0x56ff3cd1, 66 },
    { 0x4a1f3cd2, 43 },
    { 0xaf3cd3, 67 },
    { 0, 0 },
    { 0xe24f3cd5, 23 },
    { 0xf46f3cd6, 95 },
    { 0x50f3cd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09f3cdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29df3cde, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61f3ce1, 78 },
    { 0xd63f3ce1, 52 },
    { 0x71df3ce2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cf3ce6, 23 },
    { 0, 0 },
    { 0xf8bf3ce8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af3cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cf3cf2, 91 },
    { 0, 0 },
    { 0xac1f3cf4, 55 },
    { 0xc1ef3cf4, 75 },
    { 0, 0 },
    { 0x6b0f3cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1f3cfa, 84 },
    { 0x524f3cfb, 93 },
    { 0x63df3cfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f3d01, 43 },
    { 0x218f3d02, 61 },
    { 0x868f3d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf3d06, 61 },
    { 0x3af3d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5f3d0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f3d10, 98 },
    { 0x9acf3d10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3f3d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f3d19, 90 },
    { 0xd2bf3d19, 95 },
    { 0x719f3d1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdf3d1f, 95 },
    { 0, 0 },
    { 0xffef3d21, 90 },
    { 0x493f3d22, 36 },
    { 0xacef3d23, 98 },
    { 0xd7bf3d24, 98 },
    { 0x3f9f3d24, 90 },
    { 0, 0 },
    { 0xa28f3d27, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82af3d2c, 95 },
    { 0, 0 },
    { 0xf99f3d2e, 78 },
    { 0xf6af3d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaf3d33, 98 },
    { 0x964f3d34, 61 },
    { 0xabf3d33, 33 },
    { 0, 0 },
    { 0xfd7f3d37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0f3d3b, 98 },
    { 0, 0 },
    { 0x144f3d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f3d43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f3d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f3d50, 91 },
    { 0x549f3d51, 23 },
    { 0x600f3d50, 33 },
    { 0, 0 },
    { 0xe85f3d54, 58 },
    { 0x2a9f3d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8f3d59, 98 },
    { 0x6a7f3d59, 77 },
    { 0xef2f3d5a, 85 },
    { 0x879f3d5c, 33 },
    { 0x147f3d5c, 95 },
    { 0x289f3d5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4f3d61, 76 },
    { 0, 0 },
    { 0xcbaf3d63, 23 },
    { 0xebdf3d63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f3d6a, 90 },
    { 0x133f3d6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1f3d70, 87 },
    { 0x1bef3d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ff3d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x108f3d77, 90 },
    { 0, 0 },
    { 0x1aef3d79, 90 },
    { 0x5eef3d7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eef3d82, 90 },
    { 0, 0 },
    { 0xf1bf3d84, 93 },
    { 0x263f3d85, 40 },
    { 0xd62f3d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f3d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f3d8e, 98 },
    { 0xe53f3d8e, 98 },
    { 0xe89f3d8e, 33 },
    { 0x6abf3d8f, 61 },
    { 0x55f3d92, 61 },
    { 0xee9f3d93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d96, 98 },
    { 0x915f3d97, 23 },
    { 0x5aef3d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7f3d9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49f3da8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f3dae, 99 },
    { 0, 0 },
    { 0x89ef3db0, 61 },
    { 0x22cf3db0, 34 },
    { 0xafbf3db2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2f3dbc, 43 },
    { 0xb9bf3dbc, 90 },
    { 0xb52f3dbe, 23 },
    { 0xe48f3dbf, 88 },
    { 0x7c3f3dbf, 76 },
    { 0x7b9f3dc1, 98 },
    { 0x115f3dbf, 90 },
    { 0x471f3dbf, 33 },
    { 0xa3df3dbc, 97 },
    { 0xbf4f3dc2, 61 },
    { 0x744f3dbc, 66 },
    { 0xce1f3dc2, 84 },
    { 0x4d7f3dc8, 78 },
    { 0x863f3dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff3dce, 82 },
    { 0, 0 },
    { 0xd81f3dd0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f3dd6, 66 },
    { 0x2b9f3dd7, 61 },
    { 0, 0 },
    { 0x8bcf3dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f3ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef3de3, 77 },
    { 0, 0 },
    { 0xa72f3de5, 90 },
    { 0xc5ef3de6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x702f3de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f3dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0f3def, 90 },
    { 0, 0 },
    { 0x199f3df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcf3df5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff3df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f3dfb, 23 },
    { 0x936f3dfc, 95 },
    { 0x6faf3dfd, 78 },
    { 0xb39f3dfb, 43 },
    { 0xc70f3dff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df3e03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f3e07, 90 },
    { 0, 0 },
    { 0xfcaf3e09, 85 },
    { 0xe6bf3e0a, 52 },
    { 0xe40f3e0a, 98 },
    { 0x3bcf3e0a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5f3e10, 55 },
    { 0, 0 },
    { 0x48df3e12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0f3e17, 33 },
    { 0x45ef3e18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f3e1c, 98 },
    { 0x9a9f3e1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0f3e22, 93 },
    { 0xcd6f3e22, 61 },
    { 0x75af3e24, 61 },
    { 0x897f3e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210f3e29, 90 },
    { 0x323f3e2a, 61 },
    { 0xce8f3e2a, 66 },
    { 0xc02f3e2b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5f3e31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f3e36, 43 },
    { 0xbd7f3e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8f3e3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9f3e3f, 87 },
    { 0, 0 },
    { 0x785f3e41, 99 },
    { 0, 0 },
    { 0x3f1f3e43, 33 },
    { 0x6f9f3e44, 55 },
    { 0, 0 },
    { 0xc07f3e46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x840f3e49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14f3e4e, 90 },
    { 0xf68f3e4e, 100 },
    { 0, 0 },
    { 0xf9ef3e51, 91 },
    { 0, 0 },
    { 0xcdaf3e53, 76 },
    { 0, 0 },
    { 0x616f3e55, 33 },
    { 0x2fbf3e56, 61 },
    { 0xa30f3e57, 34 },
    { 0x6b3f3e56, 33 },
    { 0, 0 },
    { 0xe15f3e5a, 23 },
    { 0x95ef3e5a, 90 },
    { 0xa98f3e5a, 34 },
    { 0, 0 },
    { 0x22f3e5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3e64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48df3e67, 61 },
    { 0xdaff3e68, 65 },
    { 0x6fdf3e68, 91 },
    { 0xccbf3e6a, 66 },
    { 0xdf4f3e6b, 78 },
    { 0x6caf3e67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81df3e70, 66 },
    { 0x982f3e70, 83 },
    { 0, 0 },
    { 0x5ddf3e73, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ff3e77, 23 },
    { 0, 0 },
    { 0x526f3e79, 55 },
    { 0, 0 },
    { 0x7c2f3e7b, 91 },
    { 0xf0f3e7c, 33 },
    { 0x2b7f3e7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9f3e80, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f3e84, 61 },
    { 0xf5af3e84, 84 },
    { 0x8edf3e86, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cf3e8a, 95 },
    { 0x8d4f3e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f3e91, 93 },
    { 0x8baf3e91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dff3e97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f3ea1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f3ea4, 55 },
    { 0, 0 },
    { 0x12af3ea6, 75 },
    { 0xe7cf3ea7, 76 },
    { 0, 0 },
    { 0x753f3ea9, 61 },
    { 0xe74f3eaa, 43 },
    { 0xc97f3eaa, 33 },
    { 0x6b8f3eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797f3eb1, 61 },
    { 0, 0 },
    { 0x13bf3eb3, 77 },
    { 0xeb4f3eb3, 95 },
    { 0xd51f3eb3, 91 },
    { 0xaa1f3eb5, 98 },
    { 0x674f3eb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f3ebc, 61 },
    { 0, 0 },
    { 0x647f3ebe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f3ec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612f3ec5, 93 },
    { 0, 0 },
    { 0xba4f3ec7, 95 },
    { 0xa97f3ec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f3ecf, 55 },
    { 0x4b1f3ed0, 100 },
    { 0, 0 },
    { 0xeddf3ed2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40af3ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaf3edd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x265f3ee0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef3ee3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbf3ee9, 33 },
    { 0x11cf3eea, 33 },
    { 0x25ef3eeb, 55 },
    { 0x5c4f3eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f3ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdf3ef6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff3efb, 23 },
    { 0, 0 },
    { 0x101f3efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1f3f00, 98 },
    { 0, 0 },
    { 0x28ff3f02, 98 },
    { 0, 0 },
    { 0x214f3f04, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df3f08, 95 },
    { 0x755f3f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f3f0c, 33 },
    { 0x261f3f0d, 97 },
    { 0, 0 },
    { 0x49af3f0f, 98 },
    { 0, 0 },
    { 0xf39f3f11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccf3f18, 76 },
    { 0x3def3f19, 33 },
    { 0xdd2f3f1a, 68 },
    { 0, 0 },
    { 0x9b8f3f1c, 33 },
    { 0x3ecf3f1c, 55 },
    { 0xbd5f3f1e, 90 },
    { 0xf80f3f1f, 68 },
    { 0, 0 },
    { 0x255f3f21, 95 },
    { 0, 0 },
    { 0x649f3f23, 90 },
    { 0x761f3f23, 90 },
    { 0x354f3f25, 55 },
    { 0x762f3f26, 17 },
    { 0xc59f3f23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66f3f2f, 61 },
    { 0x550f3f30, 34 },
    { 0, 0 },
    { 0x7aaf3f32, 99 },
    { 0x666f3f33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0f3f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccf3f3d, 5 },
    { 0, 0 },
    { 0xac4f3f3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ff3f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eef3f46, 97 },
    { 0x245f3f47, 33 },
    { 0x718f3f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f3f4f, 66 },
    { 0x41ff3f50, 90 },
    { 0x8d9f3f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df3f56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f3f5b, 90 },
    { 0xaebf3f5b, 98 },
    { 0, 0 },
    { 0x213f3f5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f3f61, 98 },
    { 0, 0 },
    { 0xf72f3f63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ff3f66, 43 },
    { 0xfadf3f67, 38 },
    { 0xe3df3f67, 55 },
    { 0x351f3f69, 23 },
    { 0x89f3f6a, 95 },
    { 0xd35f3f6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cf3f70, 85 },
    { 0xe21f3f71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f3f7e, 91 },
    { 0x352f3f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8f3f83, 76 },
    { 0, 0 },
    { 0xdb7f3f85, 33 },
    { 0x8ff3f85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7f3f8b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ff3f8f, 19 },
    { 0x385f3f8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af3f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ff3f97, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3f3f9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07f3f9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ef3fa3, 100 },
    { 0, 0 },
    { 0xdacf3fa5, 91 },
    { 0xed7f3fa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0df3fa9, 76 },
    { 0x29ff3faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef3fad, 78 },
    { 0, 0 },
    { 0x78ff3faf, 23 },
    { 0, 0 },
    { 0x7c0f3fb1, 66 },
    { 0, 0 },
    { 0x157f3fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273f3fba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770f3fbf, 90 },
    { 0, 0 },
    { 0x8fcf3fc1, 100 },
    { 0xfb4f3fc2, 86 },
    { 0x7ddf3fc2, 98 },
    { 0x969f3fc4, 99 },
    { 0, 0 },
    { 0x524f3fc6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x977f3fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bff3fcc, 66 },
    { 0, 0 },
    { 0x5e8f3fce, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f3fd3, 95 },
    { 0, 0 },
    { 0x996f3fd5, 98 },
    { 0x962f3fd6, 90 },
    { 0xcecf3fd6, 23 },
    { 0, 0 },
    { 0xdef3fd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f3fdd, 100 },
    { 0x7e1f3fde, 61 },
    { 0x7f3f3fdf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f3fe4, 76 },
    { 0x898f3fe4, 77 },
    { 0xe6bf3fe4, 34 },
    { 0xc1df3fe7, 33 },
    { 0xb52f3fe8, 93 },
    { 0xad5f3fe4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f3ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f3ff5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x700f3ff8, 23 },
    { 0xd29f3ff9, 90 },
    { 0x4e5f3ff9, 98 },
    { 0x164f3ffb, 61 },
    { 0xc74f3ffc, 76 },
    { 0, 0 },
    { 0x28cf3ffe, 76 },
    { 0, 0 },
    { 0x281f4000, 90 },
    { 0, 0 },
    { 0xd73f4002, 61 },
    { 0x2d3f4002, 61 },
    { 0x325f4004, 33 },
    { 0x847f4005, 33 },
    { 0x76cf4006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2f4009, 91 },
    { 0xed1f400a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f4013, 84 },
    { 0, 0 },
    { 0xb3ef4015, 23 },
    { 0xb1cf4016, 90 },
    { 0x7d6f4017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bf401a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9f401e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f4021, 95 },
    { 0, 0 },
    { 0x677f4023, 43 },
    { 0xbc8f4023, 76 },
    { 0x248f4023, 33 },
    { 0xb99f4026, 33 },
    { 0xcdef4026, 95 },
    { 0xd0cf4023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85f402c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7f4034, 76 },
    { 0xad8f4035, 90 },
    { 0x6b5f4036, 43 },
    { 0, 0 },
    { 0x5af4038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304f4046, 43 },
    { 0xeb0f4047, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841f404b, 33 },
    { 0xeb2f404b, 61 },
    { 0, 0 },
    { 0x9f0f404e, 61 },
    { 0, 0 },
    { 0x9c4f4050, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f4054, 61 },
    { 0x700f4054, 33 },
    { 0xcdcf4055, 33 },
    { 0xbdaf4054, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcf405a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef4062, 33 },
    { 0x5b2f4063, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f406d, 85 },
    { 0x61ff406e, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f4074, 90 },
    { 0x6aff4075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424f407a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f4090, 33 },
    { 0, 0 },
    { 0x24cf4092, 95 },
    { 0x32af4092, 76 },
    { 0xed4f4094, 90 },
    { 0x628f4094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ef4098, 95 },
    { 0, 0 },
    { 0x651f409a, 52 },
    { 0xc6f409b, 61 },
    { 0x65df409b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ff40a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f40a8, 90 },
    { 0xb3df40a9, 76 },
    { 0xebaf40a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71f40b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96f40b5, 46 },
    { 0xdfef40b5, 93 },
    { 0xb63f40b7, 95 },
    { 0xbc7f40b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25f40c1, 98 },
    { 0x551f40c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6f40c7, 61 },
    { 0xca0f40c8, 33 },
    { 0x549f40c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f40cc, 55 },
    { 0, 0 },
    { 0x5adf40ce, 61 },
    { 0x4f40ce, 61 },
    { 0xc74f40ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38f40d3, 34 },
    { 0xcdf40d4, 99 },
    { 0x7e3f40d4, 61 },
    { 0x1e2f40d4, 93 },
    { 0x2f6f40d3, 36 },
    { 0xce9f40d7, 91 },
    { 0x5a0f40d9, 61 },
    { 0xdb7f40da, 93 },
    { 0x42f40da, 88 },
    { 0x52ff40d9, 90 },
    { 0x9def40dd, 90 },
    { 0x994f40da, 55 },
    { 0xddf40dd, 66 },
    { 0x582f40d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af40e6, 46 },
    { 0x59ef40e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274f40eb, 95 },
    { 0xb7df40ec, 76 },
    { 0x305f40ec, 94 },
    { 0, 0 },
    { 0x1e7f40ef, 88 },
    { 0, 0 },
    { 0x659f40f1, 23 },
    { 0x710f40f1, 23 },
    { 0x54af40f3, 96 },
    { 0x8adf40f4, 2 },
    { 0, 0 },
    { 0xde3f40f6, 93 },
    { 0xf69f40f7, 23 },
    { 0x821f40f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98af40fb, 23 },
    { 0, 0 },
    { 0x3cbf40fd, 90 },
    { 0xf61f40fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dff4104, 66 },
    { 0, 0 },
    { 0x9b3f4106, 69 },
    { 0, 0 },
    { 0x7daf4108, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef410d, 33 },
    { 0x587f410d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53f4113, 34 },
    { 0xfd5f4114, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65f411c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f4121, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f4128, 55 },
    { 0x7d7f4129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff412c, 33 },
    { 0, 0 },
    { 0x45ef412e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f4131, 91 },
    { 0xa56f4131, 76 },
    { 0, 0 },
    { 0x7a4f4134, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccf4139, 61 },
    { 0xa8af413a, 55 },
    { 0x438f413b, 33 },
    { 0, 0 },
    { 0xe58f413d, 61 },
    { 0xea0f413e, 61 },
    { 0x1e6f413e, 19 },
    { 0xb89f4140, 33 },
    { 0x8a9f413f, 33 },
    { 0x747f4142, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f4145, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecf4149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x469f414c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653f4151, 33 },
    { 0xcd4f4152, 61 },
    { 0, 0 },
    { 0x82bf4154, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f415d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24df4161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397f4168, 90 },
    { 0x9acf4168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f416d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cf4174, 95 },
    { 0xacaf4175, 61 },
    { 0, 0 },
    { 0xf50f4177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f417b, 95 },
    { 0x89cf417c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f4180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff4184, 95 },
    { 0x8bef4184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17f4188, 38 },
    { 0, 0 },
    { 0x527f418a, 34 },
    { 0xab5f418b, 68 },
    { 0, 0 },
    { 0xb9ff418d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f4190, 34 },
    { 0, 0 },
    { 0x8b6f4192, 61 },
    { 0x2b1f4193, 93 },
    { 0x996f4194, 76 },
    { 0x4c8f4194, 66 },
    { 0x768f4194, 46 },
    { 0x501f4195, 98 },
    { 0, 0 },
    { 0xbf8f4199, 34 },
    { 0xe31f419a, 90 },
    { 0x8b5f4199, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf41a8, 55 },
    { 0, 0 },
    { 0x73df41aa, 33 },
    { 0x23ef41ab, 61 },
    { 0x9f5f41ac, 98 },
    { 0x9eef41ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ff41b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67f41b3, 66 },
    { 0, 0 },
    { 0x91bf41b5, 85 },
    { 0, 0 },
    { 0x43ef41b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f41be, 98 },
    { 0x950f41bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392f41ca, 55 },
    { 0xb79f41ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3f41cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f41d5, 99 },
    { 0, 0 },
    { 0x565f41d7, 33 },
    { 0x32df41d8, 97 },
    { 0xdd5f41d7, 23 },
    { 0xa95f41da, 61 },
    { 0xae0f41db, 76 },
    { 0xadbf41dc, 43 },
    { 0x634f41dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6f41e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2f41ec, 23 },
    { 0, 0 },
    { 0x830f41ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef41f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cf41f7, 90 },
    { 0, 0 },
    { 0xccdf41f9, 66 },
    { 0xf80f41fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf4205, 61 },
    { 0xae1f4205, 64 },
    { 0x9bef4207, 17 },
    { 0x43ef4208, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ef420b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bf4215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f4218, 90 },
    { 0x46ff4219, 100 },
    { 0x7b7f421a, 95 },
    { 0xc52f421b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f421f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaf4222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef4227, 52 },
    { 0x469f4227, 100 },
    { 0x3def4227, 33 },
    { 0, 0 },
    { 0xbc2f422b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f422e, 91 },
    { 0x8fbf422e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f4232, 66 },
    { 0xa1df4232, 90 },
    { 0x177f4232, 61 },
    { 0x954f4234, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcf423c, 93 },
    { 0x66ff423c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f4242, 98 },
    { 0xfdcf4242, 33 },
    { 0xbc7f4244, 76 },
    { 0x966f4242, 33 },
    { 0, 0 },
    { 0xe23f4247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f424d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f4253, 87 },
    { 0x61ff4253, 61 },
    { 0, 0 },
    { 0xa0f4256, 76 },
    { 0x1bff4257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff425c, 55 },
    { 0, 0 },
    { 0x1cbf425e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bf4261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f4265, 90 },
    { 0x94df4265, 99 },
    { 0x8f1f4267, 33 },
    { 0x1eff4267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbf426b, 77 },
    { 0x9f4f426b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19f426f, 34 },
    { 0x158f426f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0f4275, 61 },
    { 0, 0 },
    { 0x589f4277, 98 },
    { 0xc75f4277, 55 },
    { 0, 0 },
    { 0xc0df427a, 100 },
    { 0xdfdf427b, 33 },
    { 0xfb1f427c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4f4280, 87 },
    { 0xacbf4281, 34 },
    { 0xf6f4281, 90 },
    { 0, 0 },
    { 0x9cdf4284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f4289, 66 },
    { 0x693f428a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf428e, 67 },
    { 0x94f428f, 33 },
    { 0x63ef428e, 61 },
    { 0x845f4291, 75 },
    { 0xf87f428f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743f4298, 46 },
    { 0xc01f4299, 98 },
    { 0x139f429a, 96 },
    { 0x8faf429a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9f42a6, 34 },
    { 0, 0 },
    { 0x306f42a8, 90 },
    { 0, 0 },
    { 0x6bef42aa, 76 },
    { 0x597f42aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f42af, 61 },
    { 0xedff42b0, 90 },
    { 0, 0 },
    { 0x20bf42b2, 46 },
    { 0xf41f42b3, 36 },
    { 0xcc0f42b2, 98 },
    { 0xbbaf42b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f42b8, 95 },
    { 0xa2af42b9, 4 },
    { 0xc13f42ba, 95 },
    { 0x25af42ba, 95 },
    { 0xf92f42bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f42bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f42c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f42c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08f42ca, 93 },
    { 0x331f42cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19f42d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4f42d3, 75 },
    { 0x59df42d4, 43 },
    { 0, 0 },
    { 0xb3df42d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69f42db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x889f42de, 93 },
    { 0x6eff42df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624f42e3, 43 },
    { 0xbe0f42e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f42e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff42ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f42ef, 95 },
    { 0xac1f42f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ef42fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff42ff, 99 },
    { 0x6f4f4300, 52 },
    { 0xa55f42ff, 46 },
    { 0x58cf4302, 81 },
    { 0x4daf4300, 95 },
    { 0xaebf4304, 33 },
    { 0xdf7f4302, 90 },
    { 0xabcf42ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f430c, 23 },
    { 0x589f430d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f4314, 76 },
    { 0x85f4315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df4318, 90 },
    { 0, 0 },
    { 0xfc8f431a, 99 },
    { 0x42ff431a, 95 },
    { 0x7d8f431c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef431f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899f4323, 90 },
    { 0xacaf4323, 90 },
    { 0, 0 },
    { 0x689f4326, 61 },
    { 0, 0 },
    { 0x603f4328, 98 },
    { 0xb36f4329, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f432c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecf4330, 33 },
    { 0xabaf4331, 33 },
    { 0x12f4332, 34 },
    { 0xe3bf4331, 33 },
    { 0, 0 },
    { 0x12df4335, 95 },
    { 0xcb9f4336, 23 },
    { 0x2e2f4337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f433b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf433f, 95 },
    { 0x2eef433f, 95 },
    { 0x906f433f, 95 },
    { 0x1fcf4342, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef4348, 76 },
    { 0x715f4349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddff434d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf4354, 90 },
    { 0x65bf4354, 95 },
    { 0xb6ff4356, 33 },
    { 0xdfaf4354, 23 },
    { 0xe83f4358, 95 },
    { 0, 0 },
    { 0x13cf435a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f435e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf4361, 95 },
    { 0, 0 },
    { 0x72cf4363, 33 },
    { 0x82df4364, 69 },
    { 0x6af4365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f4368, 78 },
    { 0, 0 },
    { 0xb77f436a, 55 },
    { 0, 0 },
    { 0xb42f436c, 98 },
    { 0, 0 },
    { 0x47ef436e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4f4373, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f437a, 55 },
    { 0xde7f437b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5f4381, 83 },
    { 0x574f4382, 87 },
    { 0xef7f4383, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff438f, 90 },
    { 0xf7ff4390, 100 },
    { 0x7a5f4391, 87 },
    { 0x91ef4392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2f4396, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29f4399, 95 },
    { 0x1ff439a, 98 },
    { 0xca5f439b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9f43a0, 61 },
    { 0x7edf43a0, 61 },
    { 0, 0 },
    { 0x7a4f43a3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4f43aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f43ad, 93 },
    { 0x4e5f43ae, 61 },
    { 0x798f43ad, 91 },
    { 0x997f43b0, 66 },
    { 0xef9f43b1, 99 },
    { 0x747f43b2, 81 },
    { 0, 0 },
    { 0xe9bf43b4, 99 },
    { 0xcadf43b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17f43be, 23 },
    { 0xa7ef43be, 23 },
    { 0x212f43c0, 90 },
    { 0xb06f43c1, 87 },
    { 0, 0 },
    { 0x8f8f43c3, 78 },
    { 0x841f43c4, 99 },
    { 0xa30f43c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x563f43c8, 55 },
    { 0xc69f43c9, 93 },
    { 0x32ef43c9, 95 },
    { 0, 0 },
    { 0x3d3f43cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2f43d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f43d5, 90 },
    { 0, 0 },
    { 0x7e5f43d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f43da, 23 },
    { 0x6ff43da, 84 },
    { 0xfcdf43db, 90 },
    { 0x109f43dd, 55 },
    { 0, 0 },
    { 0x45af43df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ff43ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396f43f2, 52 },
    { 0, 0 },
    { 0x17cf43f4, 33 },
    { 0xd1df43f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513f43f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ff43fe, 90 },
    { 0x283f43ff, 61 },
    { 0xb77f4400, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f4408, 90 },
    { 0x6ccf4408, 90 },
    { 0, 0 },
    { 0x912f440b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af4412, 67 },
    { 0, 0 },
    { 0x8bdf4414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x893f4417, 52 },
    { 0x181f4418, 90 },
    { 0, 0 },
    { 0xecdf441a, 77 },
    { 0x66cf441b, 93 },
    { 0x7a5f441a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5f4426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf442b, 65 },
    { 0, 0 },
    { 0x7cf442d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cf4439, 34 },
    { 0x88ef4439, 33 },
    { 0x24ff443b, 46 },
    { 0, 0 },
    { 0x97df443d, 61 },
    { 0xb7df443d, 33 },
    { 0x8a1f443f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519f4443, 24 },
    { 0x108f4444, 34 },
    { 0xe3af4444, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153f4449, 33 },
    { 0xa57f444a, 88 },
    { 0, 0 },
    { 0xf69f444c, 23 },
    { 0x1eef444d, 61 },
    { 0x950f444e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf4452, 23 },
    { 0, 0 },
    { 0x33cf4454, 100 },
    { 0x1e0f4455, 95 },
    { 0x42bf4454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942f4460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ff4463, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aef4466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90f446a, 55 },
    { 0, 0 },
    { 0xba0f446c, 65 },
    { 0, 0 },
    { 0xca9f446e, 98 },
    { 0xf48f446f, 61 },
    { 0x96ef4470, 55 },
    { 0, 0 },
    { 0x39ef4472, 61 },
    { 0, 0 },
    { 0x709f4474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4479, 84 },
    { 0xab6f447a, 91 },
    { 0x8acf447b, 77 },
    { 0, 0 },
    { 0x659f447d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f4481, 90 },
    { 0x5f7f4482, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50df4485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777f448c, 61 },
    { 0x867f448d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ef4491, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f4498, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf449e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f44a4, 100 },
    { 0x8b9f44a4, 23 },
    { 0xca6f44a5, 90 },
    { 0xe2cf44a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef44ad, 90 },
    { 0x33f44ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41f44b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f44b5, 93 },
    { 0, 0 },
    { 0xa61f44b7, 61 },
    { 0, 0 },
    { 0x74ff44b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff44bd, 23 },
    { 0, 0 },
    { 0x3bdf44bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x688f44c2, 23 },
    { 0xf31f44c2, 90 },
    { 0x2d2f44c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf44c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f44ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf44d2, 80 },
    { 0x8b9f44d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cf44d7, 68 },
    { 0xe0f44d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccf44dc, 96 },
    { 0x4e9f44dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x679f44e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75f44e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271f44e8, 75 },
    { 0x7b0f44e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f44ee, 90 },
    { 0x908f44ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65f44f3, 91 },
    { 0, 0 },
    { 0x657f44f5, 95 },
    { 0x6b6f44f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f44fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f4500, 68 },
    { 0x959f4500, 43 },
    { 0, 0 },
    { 0xb9f4503, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f4506, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f450c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5f4513, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ff4516, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf451c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483f4524, 23 },
    { 0x73ff4525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8f452b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6f452e, 76 },
    { 0x92cf452e, 90 },
    { 0, 0 },
    { 0x548f4531, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19af453b, 100 },
    { 0xcbdf453b, 55 },
    { 0x924f453d, 52 },
    { 0, 0 },
    { 0x2c9f453f, 90 },
    { 0xb2ef4540, 90 },
    { 0x4e3f4541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3f4549, 36 },
    { 0, 0 },
    { 0xf73f454b, 90 },
    { 0xe6ef454b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf4550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbf4554, 87 },
    { 0, 0 },
    { 0x717f4556, 46 },
    { 0x917f4557, 96 },
    { 0, 0 },
    { 0x911f4559, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fef455c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef455f, 90 },
    { 0, 0 },
    { 0x66ff4561, 90 },
    { 0x8fff4561, 33 },
    { 0x394f4562, 43 },
    { 0x170f4564, 85 },
    { 0xeb6f4565, 86 },
    { 0x371f4561, 23 },
    { 0xf36f4563, 61 },
    { 0x65af4568, 85 },
    { 0x9bbf4569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57f456c, 34 },
    { 0x146f456d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7f4575, 90 },
    { 0, 0 },
    { 0x120f4577, 90 },
    { 0x7d1f4577, 98 },
    { 0xfc4f4579, 33 },
    { 0x29df4578, 99 },
    { 0x14df4577, 23 },
    { 0, 0 },
    { 0x21f457d, 33 },
    { 0xc26f457e, 61 },
    { 0x788f457d, 33 },
    { 0, 0 },
    { 0x355f4581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bf4586, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0f458c, 33 },
    { 0, 0 },
    { 0x3daf458e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3f4594, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cf4597, 46 },
    { 0, 0 },
    { 0xf9f4599, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f459d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79f45a3, 90 },
    { 0x52af45a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f45b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962f45b5, 61 },
    { 0, 0 },
    { 0xf1bf45b7, 23 },
    { 0x86cf45b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cf45bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18f45be, 66 },
    { 0, 0 },
    { 0x711f45c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5af45c4, 52 },
    { 0x910f45c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07f45c8, 64 },
    { 0, 0 },
    { 0x3fbf45ca, 84 },
    { 0, 0 },
    { 0x4c8f45cc, 23 },
    { 0x30ff45cd, 55 },
    { 0, 0 },
    { 0x491f45cf, 91 },
    { 0, 0 },
    { 0x768f45d1, 95 },
    { 0xacf45d2, 98 },
    { 0x1c0f45d3, 95 },
    { 0, 0 },
    { 0x48cf45d5, 98 },
    { 0x3cff45d6, 98 },
    { 0xd6ff45d5, 95 },
    { 0x570f45d8, 33 },
    { 0x4e9f45d8, 78 },
    { 0x3cf45da, 85 },
    { 0x9f3f45da, 23 },
    { 0x847f45dc, 98 },
    { 0x256f45d8, 43 },
    { 0, 0 },
    { 0x490f45df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f45e6, 33 },
    { 0x580f45e6, 90 },
    { 0x439f45e8, 33 },
    { 0x61af45e9, 55 },
    { 0x751f45e8, 95 },
    { 0, 0 },
    { 0x741f45ec, 76 },
    { 0x95af45ed, 98 },
    { 0xb15f45ee, 55 },
    { 0x5a9f45ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af45f3, 34 },
    { 0xcd1f45f4, 61 },
    { 0x665f45f4, 98 },
    { 0x37cf45f6, 61 },
    { 0x912f45f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef45fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f4604, 90 },
    { 0xd73f4605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf460d, 33 },
    { 0x42bf460e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f4618, 87 },
    { 0xf89f4618, 90 },
    { 0xc15f461a, 100 },
    { 0x2c0f4619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff4620, 100 },
    { 0, 0 },
    { 0x323f4622, 78 },
    { 0x578f4622, 90 },
    { 0x12bf4622, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4628, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f462b, 61 },
    { 0x6a7f462b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf4630, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7f4633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f463c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f463f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29f4643, 33 },
    { 0x610f4644, 93 },
    { 0, 0 },
    { 0xaaf4646, 34 },
    { 0, 0 },
    { 0x537f4648, 90 },
    { 0x5b4f4648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff464d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x918f4650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f4655, 33 },
    { 0, 0 },
    { 0x56f4657, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f465a, 90 },
    { 0, 0 },
    { 0x4b0f465c, 77 },
    { 0x918f465d, 90 },
    { 0xa5af465c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8af4665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x562f4668, 97 },
    { 0x467f4668, 61 },
    { 0x44ef4669, 90 },
    { 0x3e9f466b, 61 },
    { 0x904f466c, 84 },
    { 0x9f7f466a, 76 },
    { 0xa5cf466e, 33 },
    { 0x4cef466d, 84 },
    { 0xe7df4670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cf4674, 98 },
    { 0xe80f4675, 90 },
    { 0x24bf4676, 67 },
    { 0x41ff4676, 23 },
    { 0, 0 },
    { 0xb37f4679, 90 },
    { 0x5cf4679, 96 },
    { 0xbf3f467a, 66 },
    { 0x211f467c, 33 },
    { 0, 0 },
    { 0x2e6f467e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f4681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f4687, 38 },
    { 0x40df4688, 95 },
    { 0xf65f4689, 84 },
    { 0xec9f4689, 90 },
    { 0xdf5f4688, 98 },
    { 0x21ff468b, 84 },
    { 0xd9f4688, 95 },
    { 0x3f3f468e, 33 },
    { 0x44ff468f, 66 },
    { 0xe4f468f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x215f4693, 97 },
    { 0x1a9f4694, 95 },
    { 0xf97f4695, 43 },
    { 0, 0 },
    { 0x4c5f4697, 23 },
    { 0, 0 },
    { 0xd8ff4699, 55 },
    { 0xdc4f4699, 17 },
    { 0x55f469b, 52 },
    { 0x676f469b, 95 },
    { 0x42ff469b, 55 },
    { 0x4eaf469e, 61 },
    { 0x2a6f469f, 90 },
    { 0xfc8f46a0, 43 },
    { 0x68bf469e, 34 },
    { 0xdf6f469f, 55 },
    { 0x852f4699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x622f46a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf46af, 43 },
    { 0, 0 },
    { 0xa34f46b1, 46 },
    { 0x437f46b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf46b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340f46ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f46c2, 98 },
    { 0xddaf46c3, 61 },
    { 0, 0 },
    { 0xefdf46c5, 90 },
    { 0xab1f46c6, 23 },
    { 0x2e4f46c7, 90 },
    { 0x1e4f46c6, 95 },
    { 0xf25f46c9, 93 },
    { 0x22cf46ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f46cd, 61 },
    { 0x39f46cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f46d1, 33 },
    { 0, 0 },
    { 0xc2ff46d3, 23 },
    { 0, 0 },
    { 0x9f0f46d5, 17 },
    { 0x48ef46d6, 55 },
    { 0x810f46d7, 99 },
    { 0x715f46d8, 80 },
    { 0x55ff46d9, 33 },
    { 0x895f46da, 87 },
    { 0x94cf46d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df46e0, 55 },
    { 0x4f4f46e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963f46e8, 78 },
    { 0x5a1f46e9, 98 },
    { 0x54f46ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57af46f3, 90 },
    { 0xbb7f46f4, 97 },
    { 0x1a2f46f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf46f8, 90 },
    { 0xe41f46f9, 23 },
    { 0, 0 },
    { 0xbe4f46fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df4700, 33 },
    { 0x6c9f4700, 61 },
    { 0x9dcf4701, 33 },
    { 0x988f4703, 61 },
    { 0x869f4703, 80 },
    { 0x848f4705, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f4708, 34 },
    { 0, 0 },
    { 0xdd6f470a, 69 },
    { 0, 0 },
    { 0xa7ef470c, 61 },
    { 0x402f470d, 23 },
    { 0xaedf470e, 61 },
    { 0x415f470f, 33 },
    { 0x7c1f470f, 90 },
    { 0xa0ff4711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40f4714, 61 },
    { 0, 0 },
    { 0xf21f4716, 33 },
    { 0xc21f4717, 55 },
    { 0xee1f4718, 53 },
    { 0x4f6f4718, 23 },
    { 0x542f471a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17f471d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209f4725, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bf472b, 34 },
    { 0, 0 },
    { 0x1a5f472d, 98 },
    { 0, 0 },
    { 0x3df472f, 90 },
    { 0, 0 },
    { 0xadf4731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff4734, 95 },
    { 0x266f4734, 33 },
    { 0, 0 },
    { 0xdbff4737, 99 },
    { 0, 0 },
    { 0xbe7f4739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f473e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f4741, 67 },
    { 0, 0 },
    { 0x453f4743, 91 },
    { 0x84ef4744, 61 },
    { 0x39bf4744, 61 },
    { 0x302f4746, 61 },
    { 0xadaf4744, 95 },
    { 0xf78f4748, 84 },
    { 0x3ddf4748, 85 },
    { 0xa4bf474a, 23 },
    { 0x50df4748, 61 },
    { 0xf69f474c, 76 },
    { 0x3a7f474d, 43 },
    { 0x6f3f474c, 90 },
    { 0xf2cf474f, 98 },
    { 0xba2f4750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784f4758, 98 },
    { 0xf03f4759, 90 },
    { 0xffbf475a, 17 },
    { 0xeb1f4759, 23 },
    { 0x5a6f475c, 90 },
    { 0, 0 },
    { 0x9e5f475e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cef4762, 61 },
    { 0xbfaf4762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5df4768, 90 },
    { 0x4af4768, 91 },
    { 0, 0 },
    { 0xf16f476b, 93 },
    { 0, 0 },
    { 0x80cf476d, 98 },
    { 0xf8bf476d, 99 },
    { 0x5d1f476f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f4778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cf477f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedf4782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f4785, 90 },
    { 0x9df4785, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3f4789, 90 },
    { 0xa82f478a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df4790, 76 },
    { 0, 0 },
    { 0x30ef4792, 90 },
    { 0x5ccf4793, 45 },
    { 0x9f2f4793, 68 },
    { 0x59ff4793, 87 },
    { 0xb12f4793, 95 },
    { 0x559f4795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505f479b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ef479e, 33 },
    { 0, 0 },
    { 0xd6af47a0, 32 },
    { 0, 0 },
    { 0x130f47a2, 91 },
    { 0xa2ef47a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66df47a6, 23 },
    { 0xf27f47a7, 98 },
    { 0x505f47a8, 98 },
    { 0x3c0f47a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f47af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f47b2, 75 },
    { 0xeddf47b3, 90 },
    { 0, 0 },
    { 0x248f47b5, 69 },
    { 0xf30f47b6, 81 },
    { 0x9d9f47b6, 61 },
    { 0x98ef47b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6f47be, 34 },
    { 0xfe7f47bf, 78 },
    { 0, 0 },
    { 0x8e0f47c1, 80 },
    { 0x771f47c2, 23 },
    { 0xadef47c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23f47c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf47ca, 90 },
    { 0xc75f47cb, 91 },
    { 0xc14f47cc, 55 },
    { 0, 0 },
    { 0xb27f47ce, 99 },
    { 0x89af47ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df47d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f47d7, 23 },
    { 0, 0 },
    { 0x8e0f47d9, 33 },
    { 0x2ddf47da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf47e5, 61 },
    { 0xdfff47e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f47ec, 33 },
    { 0, 0 },
    { 0x720f47ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4f47f1, 23 },
    { 0, 0 },
    { 0xa6ff47f3, 65 },
    { 0xcd1f47f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402f47f8, 23 },
    { 0x41af47f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70f47fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f4801, 61 },
    { 0x64af4802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18f4805, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f480a, 90 },
    { 0x6ef480a, 61 },
    { 0xe64f480c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f480f, 43 },
    { 0x9e4f480f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13f4815, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f481a, 98 },
    { 0xde4f481b, 33 },
    { 0xc1ef481b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bf4822, 43 },
    { 0, 0 },
    { 0xe03f4824, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bf4829, 55 },
    { 0, 0 },
    { 0xb5f482b, 43 },
    { 0, 0 },
    { 0xef8f482d, 90 },
    { 0, 0 },
    { 0xd63f482f, 91 },
    { 0x754f482f, 90 },
    { 0xbeef4831, 61 },
    { 0xb4bf4832, 100 },
    { 0xc7cf482f, 95 },
    { 0x1b1f482f, 33 },
    { 0, 0 },
    { 0x5a4f4836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df483a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8f483f, 95 },
    { 0x715f4840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f4843, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f484c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2f4852, 40 },
    { 0xc26f4853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f4857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93af4861, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3f4868, 57 },
    { 0xacaf4869, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0f486e, 55 },
    { 0x486f486e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf4876, 23 },
    { 0x225f4877, 33 },
    { 0x3b0f4878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88af487f, 99 },
    { 0, 0 },
    { 0x5a1f4881, 23 },
    { 0, 0 },
    { 0x30f4883, 95 },
    { 0, 0 },
    { 0x38ef4885, 88 },
    { 0x736f4886, 95 },
    { 0xcf9f4887, 95 },
    { 0, 0 },
    { 0x254f4889, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f4894, 93 },
    { 0, 0 },
    { 0x5bbf4896, 68 },
    { 0, 0 },
    { 0x270f4898, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f489c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5f48a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdf48ab, 66 },
    { 0xc0ff48ac, 33 },
    { 0, 0 },
    { 0xe73f48ae, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37f48b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f48ba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x268f48bd, 52 },
    { 0xfa8f48bd, 93 },
    { 0xcc4f48bd, 61 },
    { 0, 0 },
    { 0x73ff48c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88f48c6, 33 },
    { 0x4bbf48c7, 33 },
    { 0, 0 },
    { 0x818f48c9, 90 },
    { 0, 0 },
    { 0x40af48cb, 76 },
    { 0x7a9f48cc, 95 },
    { 0xef9f48cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823f48d2, 46 },
    { 0x912f48d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4f48d7, 93 },
    { 0, 0 },
    { 0xf21f48d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07f48dd, 87 },
    { 0, 0 },
    { 0xcd8f48df, 98 },
    { 0x569f48e0, 67 },
    { 0x873f48e1, 95 },
    { 0xa76f48e0, 56 },
    { 0xe4ff48e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bf48e6, 46 },
    { 0xb7bf48e6, 98 },
    { 0x6c1f48e8, 87 },
    { 0x2d0f48e9, 23 },
    { 0, 0 },
    { 0x1ff48eb, 61 },
    { 0, 0 },
    { 0x98ff48ed, 90 },
    { 0xe30f48ee, 90 },
    { 0xc0f48ef, 77 },
    { 0x8dff48f0, 46 },
    { 0x6bff48f1, 76 },
    { 0x440f48ef, 93 },
    { 0x398f48f3, 87 },
    { 0xc87f48f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f48f8, 98 },
    { 0xea4f48f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf48fc, 84 },
    { 0, 0 },
    { 0x666f48fe, 93 },
    { 0x2fcf48fe, 95 },
    { 0, 0 },
    { 0x341f4901, 93 },
    { 0, 0 },
    { 0x110f4903, 34 },
    { 0xedaf4904, 33 },
    { 0xfef4903, 87 },
    { 0, 0 },
    { 0x7a7f4907, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f490c, 84 },
    { 0x3bff490d, 33 },
    { 0x680f490e, 85 },
    { 0, 0 },
    { 0xf7cf4910, 85 },
    { 0x4a3f4911, 34 },
    { 0xc31f4911, 55 },
    { 0xc70f4911, 90 },
    { 0x176f4914, 83 },
    { 0, 0 },
    { 0xed9f4916, 98 },
    { 0x61ff4917, 34 },
    { 0x20ef4916, 33 },
    { 0x5f8f4919, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x750f491c, 34 },
    { 0x3c9f491d, 33 },
    { 0x6a8f491d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f4922, 87 },
    { 0, 0 },
    { 0x772f4924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3f4930, 87 },
    { 0x2eff4931, 98 },
    { 0x71ff4930, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4936, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f4939, 77 },
    { 0, 0 },
    { 0x2a4f493b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f493e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f4942, 61 },
    { 0, 0 },
    { 0x2b0f4944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70df4949, 95 },
    { 0, 0 },
    { 0xd29f494b, 61 },
    { 0x685f494c, 33 },
    { 0xe4af494d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaf4950, 90 },
    { 0x63ff4951, 33 },
    { 0x79ef4951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f4955, 61 },
    { 0x3e3f4956, 99 },
    { 0xa70f4957, 23 },
    { 0x272f4956, 75 },
    { 0, 0 },
    { 0x4f3f495a, 85 },
    { 0xe3af495b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f4961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22f4964, 62 },
    { 0xa38f4965, 90 },
    { 0xbb9f4966, 99 },
    { 0xf44f4966, 34 },
    { 0x58df4966, 23 },
    { 0x76bf4964, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5f496e, 87 },
    { 0x3ddf496e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364f4973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60f4977, 88 },
    { 0, 0 },
    { 0x807f4979, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf497d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47f4987, 33 },
    { 0xf6ff4987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbf498b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cf498f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f4996, 99 },
    { 0x1eef4996, 65 },
    { 0xccbf4998, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff499c, 33 },
    { 0x575f499d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1f49a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef49a8, 98 },
    { 0xf3ef49a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7f49ae, 65 },
    { 0xe39f49af, 81 },
    { 0xba0f49b0, 67 },
    { 0xcfef49b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f49b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df49ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169f49c2, 61 },
    { 0, 0 },
    { 0x588f49c4, 81 },
    { 0x432f49c5, 95 },
    { 0, 0 },
    { 0x157f49c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ff49cb, 76 },
    { 0, 0 },
    { 0x943f49cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cf49d0, 52 },
    { 0x4bcf49d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaf49d4, 55 },
    { 0xb5ef49d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2f49d8, 76 },
    { 0x6bff49d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbf49dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486f49e1, 76 },
    { 0x44f49e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbf49e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cf49e9, 93 },
    { 0x492f49ea, 90 },
    { 0, 0 },
    { 0x3b0f49ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f49ef, 90 },
    { 0x117f49ef, 61 },
    { 0x90ef49f1, 34 },
    { 0x150f49f2, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bf49f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef49fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f4a00, 61 },
    { 0x342f4a01, 90 },
    { 0x8d2f4a02, 66 },
    { 0x980f4a02, 33 },
    { 0x75df4a04, 33 },
    { 0xd12f4a05, 55 },
    { 0x9baf4a04, 76 },
    { 0x7dbf4a00, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f4a0a, 23 },
    { 0, 0 },
    { 0xe5af4a0c, 33 },
    { 0xedbf4a0d, 98 },
    { 0xb3bf4a0e, 34 },
    { 0xdb3f4a0f, 77 },
    { 0x5faf4a0f, 98 },
    { 0x450f4a11, 98 },
    { 0x231f4a11, 85 },
    { 0x1e9f4a11, 34 },
    { 0x58ff4a13, 64 },
    { 0x62f4a12, 76 },
    { 0xff4f4a16, 43 },
    { 0x407f4a17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690f4a1b, 95 },
    { 0, 0 },
    { 0x8ecf4a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x461f4a20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f4a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3f4a2a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f4a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf4a33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f4a37, 69 },
    { 0x7e5f4a38, 95 },
    { 0x55f4a39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f4a40, 95 },
    { 0x309f4a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f4a46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f4a4a, 98 },
    { 0xd99f4a4b, 43 },
    { 0x444f4a4c, 95 },
    { 0x923f4a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f4a51, 33 },
    { 0x17ef4a52, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eef4a55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f4a62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeef4a68, 90 },
    { 0xe1af4a69, 33 },
    { 0x97ff4a6a, 55 },
    { 0x352f4a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af4a70, 55 },
    { 0x684f4a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f4a74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5f4a77, 17 },
    { 0, 0 },
    { 0x5f0f4a79, 23 },
    { 0x4d5f4a79, 78 },
    { 0x532f4a7a, 86 },
    { 0x4e1f4a79, 33 },
    { 0, 0 },
    { 0xbcf4a7e, 34 },
    { 0xde3f4a7f, 97 },
    { 0xf6f4a7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eef4a83, 61 },
    { 0x36ff4a84, 23 },
    { 0xb1bf4a85, 34 },
    { 0xb51f4a84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9f4a8f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ff4a92, 43 },
    { 0, 0 },
    { 0x69bf4a94, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40f4a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f4aa8, 85 },
    { 0xf94f4aa9, 55 },
    { 0, 0 },
    { 0x5ddf4aab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582f4ab8, 98 },
    { 0xaf0f4ab8, 43 },
    { 0x55cf4aba, 55 },
    { 0xbeaf4abb, 90 },
    { 0x9a6f4abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x103f4abf, 33 },
    { 0x21ef4ac0, 76 },
    { 0x5a5f4ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89f4ac5, 90 },
    { 0, 0 },
    { 0x7bbf4ac7, 97 },
    { 0, 0 },
    { 0xff4f4ac9, 93 },
    { 0xfedf4aca, 99 },
    { 0, 0 },
    { 0x734f4acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f4acf, 95 },
    { 0xbb8f4acf, 80 },
    { 0xae3f4ad1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ff4ad6, 33 },
    { 0, 0 },
    { 0x736f4ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faf4adc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3f4adf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bef4ae2, 34 },
    { 0xcacf4ae2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5f4ae6, 95 },
    { 0, 0 },
    { 0x6eef4ae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f4af1, 67 },
    { 0x79f4af2, 90 },
    { 0x4cff4af3, 95 },
    { 0, 0 },
    { 0x3a1f4af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80f4af9, 90 },
    { 0xe0df4af9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f4afd, 43 },
    { 0, 0 },
    { 0x8cff4aff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64f4b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ff4b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f4b10, 33 },
    { 0, 0 },
    { 0xbd9f4b12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f4b1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f4b1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f4b21, 83 },
    { 0xc81f4b22, 61 },
    { 0xca3f4b22, 95 },
    { 0, 0 },
    { 0x98af4b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff4b31, 55 },
    { 0, 0 },
    { 0x11af4b33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff4b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83f4b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f4b43, 77 },
    { 0x63cf4b44, 33 },
    { 0x25ef4b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f4b48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f4b4d, 33 },
    { 0, 0 },
    { 0x403f4b4f, 86 },
    { 0x383f4b50, 17 },
    { 0x913f4b51, 33 },
    { 0x5cdf4b52, 98 },
    { 0x9cdf4b53, 66 },
    { 0x43cf4b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04f4b59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9f4b60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f4b63, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81f4b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637f4b6d, 43 },
    { 0xaeef4b6e, 61 },
    { 0x4f2f4b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ff4b73, 99 },
    { 0x65af4b74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35af4b79, 100 },
    { 0, 0 },
    { 0x811f4b7b, 61 },
    { 0, 0 },
    { 0xb38f4b7d, 95 },
    { 0, 0 },
    { 0x658f4b7f, 23 },
    { 0xf88f4b7f, 34 },
    { 0x7e8f4b81, 33 },
    { 0, 0 },
    { 0x584f4b83, 33 },
    { 0, 0 },
    { 0xf70f4b85, 46 },
    { 0xa70f4b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f4b95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff4b9c, 55 },
    { 0, 0 },
    { 0xf91f4b9e, 78 },
    { 0, 0 },
    { 0x50bf4ba0, 34 },
    { 0x1bef4ba0, 23 },
    { 0xdb3f4ba2, 76 },
    { 0xb2cf4ba1, 96 },
    { 0xe17f4ba4, 78 },
    { 0x1faf4ba4, 61 },
    { 0xf04f4ba1, 98 },
    { 0x778f4ba1, 43 },
    { 0, 0 },
    { 0x7a0f4ba9, 98 },
    { 0xbaff4ba9, 95 },
    { 0, 0 },
    { 0xf65f4bac, 98 },
    { 0x958f4bac, 81 },
    { 0x749f4bae, 61 },
    { 0x47ff4baf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f4bb3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f4bb9, 33 },
    { 0x71ef4bba, 93 },
    { 0xcddf4bbb, 33 },
    { 0x781f4bbc, 33 },
    { 0x251f4bbd, 98 },
    { 0x89ef4bbd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f4bc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9f4bc4, 52 },
    { 0, 0 },
    { 0x872f4bc6, 46 },
    { 0x78ef4bc7, 46 },
    { 0x5d3f4bc8, 33 },
    { 0x33ff4bc9, 67 },
    { 0xf3cf4bc9, 98 },
    { 0xb1af4bc9, 90 },
    { 0x722f4bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31df4bd2, 91 },
    { 0xe66f4bd3, 65 },
    { 0xd53f4bd3, 55 },
    { 0, 0 },
    { 0xcd2f4bd6, 93 },
    { 0xafaf4bd7, 61 },
    { 0x434f4bd8, 90 },
    { 0xe25f4bd6, 46 },
    { 0, 0 },
    { 0x4b6f4bdb, 90 },
    { 0x2cef4bdc, 75 },
    { 0x218f4bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11f4be1, 46 },
    { 0x665f4be1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f4be6, 34 },
    { 0xcf6f4be7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf4bea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ef4bed, 61 },
    { 0x934f4bed, 98 },
    { 0xac9f4bee, 43 },
    { 0, 0 },
    { 0xf24f4bf1, 33 },
    { 0xd70f4bf2, 23 },
    { 0x248f4bf2, 69 },
    { 0x1b5f4bf4, 61 },
    { 0x9d4f4bf5, 33 },
    { 0xd39f4bf1, 61 },
    { 0x650f4bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49af4bfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f4bfe, 76 },
    { 0xdcf4bff, 90 },
    { 0xa36f4c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaf4c07, 95 },
    { 0x501f4c08, 75 },
    { 0x642f4c07, 76 },
    { 0, 0 },
    { 0x4af4c0b, 100 },
    { 0x800f4c0c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bf4c0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af4c12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf4c18, 65 },
    { 0, 0 },
    { 0x38ff4c1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f4c20, 98 },
    { 0x7aff4c21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bf4c26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4f4c29, 87 },
    { 0, 0 },
    { 0x84f4c2b, 98 },
    { 0, 0 },
    { 0x42af4c2d, 33 },
    { 0xc90f4c2d, 55 },
    { 0, 0 },
    { 0xad8f4c30, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af4c33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cf4c36, 23 },
    { 0x473f4c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f4c3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693f4c3f, 61 },
    { 0, 0 },
    { 0x2c3f4c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5f4c45, 77 },
    { 0xc20f4c46, 76 },
    { 0x5f3f4c47, 33 },
    { 0, 0 },
    { 0x2dbf4c49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cf4c4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9f4c51, 34 },
    { 0, 0 },
    { 0xec6f4c53, 100 },
    { 0, 0 },
    { 0x6e5f4c55, 23 },
    { 0x97df4c56, 55 },
    { 0, 0 },
    { 0xd65f4c58, 93 },
    { 0xf50f4c59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f4c5c, 34 },
    { 0, 0 },
    { 0x9c4f4c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bf4c62, 95 },
    { 0, 0 },
    { 0xaf7f4c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f4c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4c6b, 55 },
    { 0xd45f4c6b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f4c6f, 61 },
    { 0x718f4c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ff4c73, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ff4c76, 61 },
    { 0x165f4c76, 95 },
    { 0x8b1f4c78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832f4c81, 90 },
    { 0x894f4c82, 34 },
    { 0x824f4c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f4c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef4c90, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ff4c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f4c9a, 43 },
    { 0x5d0f4c9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ef4ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6f4cac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef4cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35f4cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f4cb8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaf4cbb, 100 },
    { 0, 0 },
    { 0xe43f4cbd, 55 },
    { 0xdb7f4cbe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5f4cc1, 95 },
    { 0xecdf4cc1, 67 },
    { 0xa8ef4cc3, 84 },
    { 0, 0 },
    { 0xe79f4cc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f4cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f4cd2, 61 },
    { 0xe22f4cd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df4cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973f4cdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8f4cdf, 99 },
    { 0xfcef4cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321f4ce5, 98 },
    { 0x143f4ce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74af4ce9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff4ced, 23 },
    { 0, 0 },
    { 0x3b6f4cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f4cf2, 90 },
    { 0, 0 },
    { 0xc56f4cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ff4cf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658f4cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295f4d01, 80 },
    { 0x75af4d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f4d05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72f4d0b, 33 },
    { 0, 0 },
    { 0xc8af4d0d, 95 },
    { 0, 0 },
    { 0x5bcf4d0f, 23 },
    { 0, 0 },
    { 0xb62f4d11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f4d1b, 65 },
    { 0x180f4d1c, 90 },
    { 0xdd1f4d1d, 88 },
    { 0x9d2f4d1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaef4d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69f4d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f4d2f, 91 },
    { 0xa9af4d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ef4d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f4d38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f4d3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936f4d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x548f4d43, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af4d4e, 23 },
    { 0, 0 },
    { 0x4c0f4d50, 85 },
    { 0, 0 },
    { 0xbc1f4d52, 55 },
    { 0x8f1f4d52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f4d59, 38 },
    { 0x2a0f4d59, 95 },
    { 0x6a7f4d5b, 43 },
    { 0x1df4d5b, 43 },
    { 0x2e8f4d5d, 90 },
    { 0x8bff4d5e, 46 },
    { 0xd3cf4d5f, 66 },
    { 0x2fef4d60, 33 },
    { 0x9d8f4d5e, 91 },
    { 0xb46f4d62, 91 },
    { 0x734f4d63, 90 },
    { 0, 0 },
    { 0x440f4d65, 98 },
    { 0x5d9f4d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88f4d69, 23 },
    { 0x624f4d69, 76 },
    { 0x744f4d6b, 88 },
    { 0, 0 },
    { 0xe3cf4d6d, 33 },
    { 0x7bff4d6d, 100 },
    { 0x7c4f4d6f, 43 },
    { 0x714f4d70, 88 },
    { 0xfaff4d6e, 23 },
    { 0, 0 },
    { 0x935f4d73, 78 },
    { 0x893f4d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f4d79, 98 },
    { 0x398f4d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf4d7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ff4d81, 33 },
    { 0x6e3f4d82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f4d85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f4d8a, 84 },
    { 0x7e7f4d8a, 34 },
    { 0x972f4d8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f4d91, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf4d99, 100 },
    { 0x6f8f4d99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87f4da0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df4da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f4db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49df4db3, 93 },
    { 0x888f4db4, 23 },
    { 0, 0 },
    { 0xc4af4db6, 23 },
    { 0x24af4db6, 96 },
    { 0x5f2f4db8, 23 },
    { 0x24cf4db9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cf4dc0, 95 },
    { 0, 0 },
    { 0x260f4dc2, 87 },
    { 0, 0 },
    { 0x60bf4dc4, 98 },
    { 0, 0 },
    { 0xd4df4dc6, 90 },
    { 0x53df4dc6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f4dcb, 34 },
    { 0x752f4dcb, 95 },
    { 0x983f4dcd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2f4dd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f4dde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf4de1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33f4df0, 33 },
    { 0xaf6f4df1, 100 },
    { 0xb88f4df2, 98 },
    { 0x9bef4df3, 61 },
    { 0xac1f4df4, 91 },
    { 0x499f4df5, 95 },
    { 0x17df4df5, 23 },
    { 0, 0 },
    { 0x239f4df8, 91 },
    { 0, 0 },
    { 0x3aaf4dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f4dff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3af4e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caf4e11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f4e16, 52 },
    { 0x55f4e17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f4e1b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbf4e22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f4e2c, 99 },
    { 0x48df4e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f4e30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff4e34, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f4e3b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4e40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649f4e44, 95 },
    { 0x5eff4e44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63f4e48, 90 },
    { 0x53ff4e48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff4e4e, 90 },
    { 0xe65f4e4e, 90 },
    { 0x52f4e4e, 33 },
    { 0, 0 },
    { 0xfa6f4e52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60af4e56, 76 },
    { 0, 0 },
    { 0x578f4e58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f4e5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdf4e60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf4e63, 56 },
    { 0, 0 },
    { 0xae9f4e65, 91 },
    { 0x45af4e65, 61 },
    { 0x8fdf4e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f4e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680f4e6e, 55 },
    { 0x977f4e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625f4e74, 90 },
    { 0x811f4e75, 33 },
    { 0x4adf4e74, 91 },
    { 0, 0 },
    { 0x3a3f4e78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf4e7d, 95 },
    { 0x74cf4e7d, 95 },
    { 0xce3f4e7f, 95 },
    { 0x85f4e80, 90 },
    { 0, 0 },
    { 0xd4bf4e82, 90 },
    { 0, 0 },
    { 0xfe2f4e84, 92 },
    { 0x62af4e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf4e88, 23 },
    { 0, 0 },
    { 0x54f4e8a, 95 },
    { 0x6f8f4e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8f4e8e, 88 },
    { 0x2cef4e8f, 67 },
    { 0x4c4f4e90, 95 },
    { 0xa88f4e91, 33 },
    { 0xe95f4e92, 61 },
    { 0xc41f4e91, 98 },
    { 0x8cef4e91, 61 },
    { 0xff3f4e95, 84 },
    { 0xca6f4e96, 78 },
    { 0x9d3f4e96, 33 },
    { 0, 0 },
    { 0xafff4e99, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cef4e9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417f4ea5, 34 },
    { 0x7df4ea6, 61 },
    { 0, 0 },
    { 0xf6bf4ea8, 33 },
    { 0, 0 },
    { 0xb05f4eaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10df4eaf, 76 },
    { 0, 0 },
    { 0x8adf4eb1, 68 },
    { 0, 0 },
    { 0xf5bf4eb3, 88 },
    { 0x339f4eb4, 95 },
    { 0x478f4eb5, 99 },
    { 0x2bf4eb6, 33 },
    { 0x738f4eb6, 34 },
    { 0xf9ef4eb8, 90 },
    { 0x2ccf4eb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125f4ebe, 84 },
    { 0, 0 },
    { 0xa42f4ec0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef4ec3, 55 },
    { 0x4fdf4ec3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8f4ec8, 91 },
    { 0xc93f4ec9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f4ecc, 80 },
    { 0x3d5f4ecc, 90 },
    { 0xd2ff4ece, 33 },
    { 0xc39f4ece, 61 },
    { 0x854f4ecc, 90 },
    { 0xb28f4ed1, 91 },
    { 0x4a6f4ecc, 100 },
    { 0x1b0f4ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f4ed6, 61 },
    { 0x615f4ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bf4edd, 56 },
    { 0, 0 },
    { 0xc71f4edf, 98 },
    { 0, 0 },
    { 0xabdf4ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2f4ee5, 95 },
    { 0x28af4ee5, 17 },
    { 0xe4af4ee7, 94 },
    { 0xde0f4ee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaf4eed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24af4ef3, 61 },
    { 0x297f4ef4, 90 },
    { 0xc5cf4ef5, 61 },
    { 0x970f4ef6, 33 },
    { 0x556f4ef7, 43 },
    { 0, 0 },
    { 0xff3f4ef9, 52 },
    { 0, 0 },
    { 0xda0f4efb, 95 },
    { 0, 0 },
    { 0x656f4efd, 75 },
    { 0x810f4efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ef4f02, 100 },
    { 0, 0 },
    { 0x46f4f04, 98 },
    { 0x951f4f04, 43 },
    { 0x7d3f4f04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef4f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f4f0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f4f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56df4f15, 23 },
    { 0, 0 },
    { 0xb15f4f17, 90 },
    { 0xdcef4f18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135f4f20, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x189f4f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f4f2a, 97 },
    { 0xfadf4f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baf4f2f, 100 },
    { 0, 0 },
    { 0x5dcf4f31, 61 },
    { 0x1bff4f32, 61 },
    { 0, 0 },
    { 0x2f0f4f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf4f37, 80 },
    { 0x54ef4f37, 67 },
    { 0x288f4f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98f4f3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f4f40, 61 },
    { 0x264f4f41, 90 },
    { 0x46bf4f41, 99 },
    { 0x329f4f43, 77 },
    { 0, 0 },
    { 0x681f4f45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf4f49, 76 },
    { 0x249f4f49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf4f50, 98 },
    { 0, 0 },
    { 0x79df4f52, 23 },
    { 0xc11f4f53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f4f5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f4f5f, 93 },
    { 0x152f4f60, 90 },
    { 0x40ef4f61, 87 },
    { 0x479f4f62, 23 },
    { 0x90f4f5f, 17 },
    { 0, 0 },
    { 0x44ff4f65, 95 },
    { 0x8c1f4f65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f4f6b, 98 },
    { 0x721f4f6c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8f4f70, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df4f80, 90 },
    { 0x48bf4f81, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58f4f87, 46 },
    { 0, 0 },
    { 0x28af4f89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991f4f90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85f4f93, 95 },
    { 0x688f4f93, 78 },
    { 0x689f4f95, 85 },
    { 0x160f4f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x798f4f99, 33 },
    { 0xd9af4f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cf4f9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cf4fa1, 46 },
    { 0x863f4fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f4fa6, 95 },
    { 0xb61f4fa6, 23 },
    { 0x209f4fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f4fab, 95 },
    { 0x16f4fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf4faf, 76 },
    { 0x40af4faf, 87 },
    { 0, 0 },
    { 0xeaaf4fb2, 61 },
    { 0, 0 },
    { 0xa8cf4fb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff4fbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f4fc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f4fca, 23 },
    { 0xaaaf4fca, 76 },
    { 0xfaaf4fcb, 77 },
    { 0, 0 },
    { 0xb4f4fce, 95 },
    { 0x9cef4fce, 98 },
    { 0xeccf4fce, 98 },
    { 0x5bff4fce, 33 },
    { 0, 0 },
    { 0xdf7f4fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624f4fd9, 96 },
    { 0, 0 },
    { 0xba9f4fdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f4fdf, 78 },
    { 0xa9ff4fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4fe3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x90af4fe6, 90 },
    { 0, 0 },
    { 0x2cf4fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f4feb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf4ff1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ff4ff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bf4ff9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f4ffd, 23 },
    { 0xecdf4ffe, 90 },
    { 0x938f4ffd, 75 },
    { 0x39ef4ffe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bf5004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7f500d, 98 },
    { 0x8f8f500e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef5011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f5019, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff501e, 34 },
    { 0xe06f501e, 91 },
    { 0x896f5020, 90 },
    { 0, 0 },
    { 0x463f5022, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daf5025, 67 },
    { 0, 0 },
    { 0x6dcf5027, 95 },
    { 0, 0 },
    { 0xc3f5029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f5034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79f5037, 100 },
    { 0xc29f5038, 40 },
    { 0xff1f5038, 90 },
    { 0xef503a, 84 },
    { 0x645f5038, 55 },
    { 0xc4af503c, 76 },
    { 0x756f503d, 78 },
    { 0x61ff503d, 52 },
    { 0x52df503c, 84 },
    { 0, 0 },
    { 0xd14f5041, 34 },
    { 0xdc6f5042, 33 },
    { 0x4ddf5043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1f5046, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff504b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431f5050, 90 },
    { 0, 0 },
    { 0xdc3f5052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f5059, 61 },
    { 0x88df5059, 95 },
    { 0x25bf505b, 61 },
    { 0x511f5059, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ef5062, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f5068, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4df506c, 55 },
    { 0x320f506c, 43 },
    { 0x69ef506e, 76 },
    { 0xfdcf506f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x716f5072, 81 },
    { 0x875f5073, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x880f5076, 97 },
    { 0, 0 },
    { 0x4eaf5078, 76 },
    { 0x50ff5078, 46 },
    { 0x8fdf507a, 33 },
    { 0xeb5f5078, 61 },
    { 0, 0 },
    { 0x8b6f507d, 33 },
    { 0, 0 },
    { 0x603f507f, 85 },
    { 0x81af507f, 55 },
    { 0xb26f5081, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edf508d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf5090, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f5096, 76 },
    { 0x540f5096, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423f509c, 95 },
    { 0xaaf509d, 76 },
    { 0, 0 },
    { 0x331f509f, 46 },
    { 0, 0 },
    { 0xf77f50a1, 34 },
    { 0x566f50a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ff50aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cf50b2, 98 },
    { 0xc9cf50b2, 34 },
    { 0, 0 },
    { 0xd5bf50b5, 90 },
    { 0, 0 },
    { 0x247f50b7, 43 },
    { 0, 0 },
    { 0xa5bf50b9, 98 },
    { 0xb9df50ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f50be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbf50c1, 87 },
    { 0xa01f50c2, 61 },
    { 0xb6ef50c3, 95 },
    { 0xdaff50c4, 75 },
    { 0xdcbf50c5, 78 },
    { 0xfb7f50c6, 75 },
    { 0xd7ef50c7, 33 },
    { 0x310f50c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cf50ce, 98 },
    { 0x777f50cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8f50d4, 95 },
    { 0xc2f50d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf50d9, 33 },
    { 0xf16f50da, 23 },
    { 0x5abf50da, 90 },
    { 0xe59f50dc, 66 },
    { 0x2f50da, 90 },
    { 0x8e9f50dd, 43 },
    { 0xbcbf50df, 61 },
    { 0xe0f50e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f50e3, 85 },
    { 0x6cf50e4, 34 },
    { 0, 0 },
    { 0x822f50e6, 65 },
    { 0, 0 },
    { 0x78f50e8, 91 },
    { 0, 0 },
    { 0x229f50ea, 95 },
    { 0x91f50eb, 43 },
    { 0, 0 },
    { 0x1cff50ed, 85 },
    { 0xc25f50ed, 90 },
    { 0, 0 },
    { 0x876f50f0, 55 },
    { 0xd9ff50f1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897f50f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f50fc, 90 },
    { 0xd49f50fd, 90 },
    { 0x119f50fd, 61 },
    { 0x3e1f50ff, 78 },
    { 0, 0 },
    { 0xab0f5101, 91 },
    { 0, 0 },
    { 0x41af5103, 99 },
    { 0x644f5103, 98 },
    { 0xd8f5104, 33 },
    { 0, 0 },
    { 0x6b6f5107, 17 },
    { 0xc9ff5108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f510b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4f5110, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f5116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6f5119, 95 },
    { 0, 0 },
    { 0x857f511b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff511e, 52 },
    { 0, 0 },
    { 0xc47f5120, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2f5124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78f512e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ef5132, 33 },
    { 0, 0 },
    { 0x4c6f5134, 95 },
    { 0xbe0f5135, 90 },
    { 0x807f5135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef513a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff513d, 67 },
    { 0, 0 },
    { 0x3d4f513f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37f5145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af5149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466f514e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f5152, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ff5156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78f5159, 87 },
    { 0xe86f5159, 95 },
    { 0, 0 },
    { 0x3d5f515c, 23 },
    { 0xba7f515c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f5165, 87 },
    { 0x2fff5165, 33 },
    { 0xccbf5167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2f516d, 100 },
    { 0, 0 },
    { 0x3f4f516f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93f5172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2f5176, 23 },
    { 0xb93f5177, 99 },
    { 0x1def5178, 77 },
    { 0x25cf5178, 90 },
    { 0xcd4f5177, 76 },
    { 0, 0 },
    { 0xfa7f517c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ff5182, 95 },
    { 0xd90f5183, 23 },
    { 0x64f5182, 93 },
    { 0x7ef5182, 34 },
    { 0x255f5186, 95 },
    { 0, 0 },
    { 0xbf1f5188, 95 },
    { 0, 0 },
    { 0x2e7f518a, 87 },
    { 0, 0 },
    { 0xf35f518c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301f5190, 90 },
    { 0x184f5190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x910f5194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11df5197, 40 },
    { 0, 0 },
    { 0x443f5199, 90 },
    { 0, 0 },
    { 0xa09f519b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70af519e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756f51a4, 52 },
    { 0xafdf51a4, 95 },
    { 0x5e7f51a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf51a9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f51ac, 95 },
    { 0x6bdf51ad, 33 },
    { 0x882f51ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f51b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88f51b4, 76 },
    { 0xbe0f51b5, 90 },
    { 0x443f51b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12f51b9, 76 },
    { 0x5a5f51ba, 98 },
    { 0x577f51ba, 61 },
    { 0xe91f51bb, 90 },
    { 0x918f51bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8f51c1, 61 },
    { 0, 0 },
    { 0x64df51c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813f51ce, 33 },
    { 0xaf1f51cf, 98 },
    { 0, 0 },
    { 0x80ff51d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f51d7, 23 },
    { 0, 0 },
    { 0xf9ff51d9, 90 },
    { 0xae1f51da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbf51e6, 76 },
    { 0, 0 },
    { 0xed9f51e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56f51eb, 98 },
    { 0x782f51ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655f51f1, 66 },
    { 0x873f51f1, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f51f9, 70 },
    { 0xcf3f51fa, 76 },
    { 0, 0 },
    { 0xb60f51fc, 23 },
    { 0, 0 },
    { 0x7b3f51fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2f5204, 23 },
    { 0x5aaf5204, 100 },
    { 0x171f5206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f520f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15af5217, 98 },
    { 0xc11f5218, 94 },
    { 0x9f5219, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1f521e, 33 },
    { 0x65df521f, 96 },
    { 0x42bf521f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908f5228, 61 },
    { 0, 0 },
    { 0x571f522a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x169f522d, 23 },
    { 0xf5f522d, 92 },
    { 0x1f0f522e, 76 },
    { 0x604f522f, 61 },
    { 0xe4af5231, 46 },
    { 0xd5bf5231, 33 },
    { 0x1aef522d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f5238, 61 },
    { 0x2d7f5239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf523d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf5242, 34 },
    { 0xb9ff5243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f5248, 77 },
    { 0, 0 },
    { 0xb09f524a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0f524e, 43 },
    { 0xb4bf524f, 33 },
    { 0x606f524f, 90 },
    { 0, 0 },
    { 0xfc9f5252, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61f5256, 90 },
    { 0x745f5256, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af525c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f5260, 78 },
    { 0, 0 },
    { 0x73cf5262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375f5266, 95 },
    { 0x4f7f5266, 34 },
    { 0, 0 },
    { 0xbaf5269, 67 },
    { 0, 0 },
    { 0xfa6f526b, 95 },
    { 0x175f526c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165f5272, 78 },
    { 0x3cef5273, 52 },
    { 0xad1f5273, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f5278, 77 },
    { 0, 0 },
    { 0x17bf527a, 34 },
    { 0x4abf527a, 90 },
    { 0x509f527c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f5282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4f5285, 87 },
    { 0x816f5286, 93 },
    { 0x77f5287, 61 },
    { 0x761f5288, 33 },
    { 0xa05f5289, 33 },
    { 0, 0 },
    { 0x4bf528b, 55 },
    { 0, 0 },
    { 0x440f528d, 55 },
    { 0x9baf528e, 93 },
    { 0x718f528e, 4 },
    { 0x811f5290, 76 },
    { 0, 0 },
    { 0xb05f5292, 33 },
    { 0x785f5293, 64 },
    { 0, 0 },
    { 0xebf5295, 93 },
    { 0x469f5296, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7f5299, 90 },
    { 0, 0 },
    { 0xb62f529b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x191f529e, 33 },
    { 0x244f529f, 33 },
    { 0xf77f52a0, 68 },
    { 0xee5f52a0, 95 },
    { 0x70ef52a1, 95 },
    { 0xb95f52a1, 46 },
    { 0, 0 },
    { 0xd6af52a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63f52ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f52ae, 33 },
    { 0, 0 },
    { 0xa7bf52b0, 33 },
    { 0x98bf52b1, 34 },
    { 0xf13f52b1, 33 },
    { 0x546f52b1, 33 },
    { 0xa41f52b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f52b7, 33 },
    { 0x541f52b8, 81 },
    { 0x435f52b9, 61 },
    { 0x309f52ba, 33 },
    { 0x28ef52bb, 23 },
    { 0x746f52bb, 98 },
    { 0x37df52ba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8f52c0, 69 },
    { 0, 0 },
    { 0x71ef52c2, 23 },
    { 0xd73f52c3, 88 },
    { 0, 0 },
    { 0xd9df52c5, 90 },
    { 0x38ef52c6, 61 },
    { 0x4f5f52c7, 46 },
    { 0x1d2f52c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf52cc, 78 },
    { 0x8bf52cd, 90 },
    { 0x26ff52ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf52d7, 76 },
    { 0x559f52d7, 33 },
    { 0xb03f52d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f52dc, 43 },
    { 0x68af52dd, 33 },
    { 0x8e1f52de, 76 },
    { 0, 0 },
    { 0xd1ff52e0, 61 },
    { 0x33bf52e1, 61 },
    { 0x44cf52e2, 90 },
    { 0x85af52e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6df52e9, 90 },
    { 0x5dbf52ea, 23 },
    { 0, 0 },
    { 0x99ef52ec, 43 },
    { 0x605f52ed, 33 },
    { 0x697f52ed, 77 },
    { 0x1b1f52ef, 23 },
    { 0, 0 },
    { 0xcc3f52f1, 55 },
    { 0, 0 },
    { 0x576f52f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8af52f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f52fa, 76 },
    { 0xf9cf52fa, 76 },
    { 0xcb8f52fb, 95 },
    { 0x716f52fd, 61 },
    { 0xe8ff52fe, 95 },
    { 0x720f52ff, 61 },
    { 0xd64f52fd, 55 },
    { 0xb43f52fd, 98 },
    { 0xbb6f5300, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcf5305, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bf530a, 95 },
    { 0xf23f530b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef530f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ef5312, 43 },
    { 0xe25f5312, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df5317, 90 },
    { 0xfe0f5318, 75 },
    { 0x221f5318, 23 },
    { 0x85ff5318, 61 },
    { 0x5b9f5318, 31 },
    { 0xa3f531a, 87 },
    { 0xe1ef531c, 33 },
    { 0xf1cf531b, 61 },
    { 0xa47f531f, 84 },
    { 0xd02f531c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27f5323, 95 },
    { 0, 0 },
    { 0x1e8f5325, 61 },
    { 0, 0 },
    { 0x3dff5327, 84 },
    { 0, 0 },
    { 0xd0f5329, 90 },
    { 0x286f532a, 85 },
    { 0x7a6f532a, 90 },
    { 0xf1ff532a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219f5331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af5334, 66 },
    { 0x9bdf5335, 93 },
    { 0xa2ef5336, 99 },
    { 0, 0 },
    { 0x7cef5338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f533b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf533e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7f5341, 55 },
    { 0xa15f5342, 95 },
    { 0x73bf5341, 84 },
    { 0x838f5344, 61 },
    { 0xc84f5344, 52 },
    { 0xc83f5346, 61 },
    { 0, 0 },
    { 0x781f5348, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f534d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780f535a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x142f535d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ff5365, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x290f5368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f536f, 76 },
    { 0x491f5370, 101 },
    { 0, 0 },
    { 0x2cbf5372, 33 },
    { 0x1e8f5372, 43 },
    { 0, 0 },
    { 0x6a3f5375, 98 },
    { 0, 0 },
    { 0x248f5377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf537a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdf537e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f5384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73f538d, 61 },
    { 0x526f538e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467f5391, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef5394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adf539e, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff53a2, 90 },
    { 0xd95f53a3, 90 },
    { 0xfccf53a4, 90 },
    { 0xf30f53a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729f53b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeef53b8, 43 },
    { 0, 0 },
    { 0xc03f53ba, 90 },
    { 0xb07f53bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f53bf, 91 },
    { 0, 0 },
    { 0xf0bf53c1, 95 },
    { 0xa81f53c2, 23 },
    { 0, 0 },
    { 0x5dcf53c4, 99 },
    { 0, 0 },
    { 0xfc0f53c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61f53c9, 85 },
    { 0xc6af53ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f53d7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2f53dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f53e0, 46 },
    { 0xf1df53e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5f53e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af53ed, 78 },
    { 0x304f53ee, 46 },
    { 0x7e0f53ee, 33 },
    { 0xca5f53f0, 33 },
    { 0xddcf53ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5df53f4, 23 },
    { 0x81ff53f5, 100 },
    { 0x879f53f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af53fd, 55 },
    { 0, 0 },
    { 0xa48f53ff, 33 },
    { 0, 0 },
    { 0x9d1f5401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fff5404, 33 },
    { 0x1b2f5405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5f540b, 95 },
    { 0xab6f540b, 93 },
    { 0xc28f540d, 33 },
    { 0xa04f540e, 23 },
    { 0x4cff540e, 95 },
    { 0xe8af5410, 90 },
    { 0, 0 },
    { 0x978f5412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3f5415, 91 },
    { 0x265f5416, 55 },
    { 0x4c0f5417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57af541a, 33 },
    { 0x986f541b, 78 },
    { 0x7e3f541c, 95 },
    { 0xa2df541b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55f5421, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ff5429, 95 },
    { 0, 0 },
    { 0xd1ff542b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f5431, 17 },
    { 0x3d6f5431, 98 },
    { 0x88ff5431, 84 },
    { 0, 0 },
    { 0x31ff5435, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4f543b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bf5442, 33 },
    { 0x214f5442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af5447, 99 },
    { 0xa85f5447, 55 },
    { 0, 0 },
    { 0xe1cf544a, 90 },
    { 0xc8df544b, 98 },
    { 0x7b7f544b, 61 },
    { 0xa52f544b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545f5451, 61 },
    { 0x1cbf5451, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cf5458, 100 },
    { 0, 0 },
    { 0xca4f545a, 90 },
    { 0xbc9f545a, 34 },
    { 0x95af545a, 90 },
    { 0x6a2f545a, 33 },
    { 0x2ccf545e, 81 },
    { 0, 0 },
    { 0x7a6f5460, 23 },
    { 0x916f5460, 90 },
    { 0x5a7f5462, 81 },
    { 0xd26f5462, 90 },
    { 0xc84f5460, 33 },
    { 0x79af5460, 61 },
    { 0x408f5466, 23 },
    { 0x68bf5467, 61 },
    { 0x7cdf5467, 55 },
    { 0xed2f5467, 90 },
    { 0xb06f5469, 61 },
    { 0, 0 },
    { 0xa8df546c, 82 },
    { 0x6ecf546d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f5471, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1f5477, 78 },
    { 0, 0 },
    { 0x31ff5479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf5481, 90 },
    { 0, 0 },
    { 0x6d8f5483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f5486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf5489, 61 },
    { 0, 0 },
    { 0x9adf548b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7f548e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0f5491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55f5497, 75 },
    { 0, 0 },
    { 0xd5bf5499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x351f549c, 91 },
    { 0xc51f549d, 61 },
    { 0x882f549c, 76 },
    { 0, 0 },
    { 0x4d0f54a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf54a4, 98 },
    { 0x220f54a5, 87 },
    { 0x92bf54a6, 98 },
    { 0, 0 },
    { 0x412f54a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4f54ac, 35 },
    { 0x981f54ad, 90 },
    { 0, 0 },
    { 0xdb6f54af, 93 },
    { 0x934f54b0, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f54b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f54b7, 61 },
    { 0xc61f54b8, 95 },
    { 0, 0 },
    { 0x15ff54ba, 95 },
    { 0x55ef54ba, 33 },
    { 0, 0 },
    { 0xd4ff54bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbf54c1, 78 },
    { 0, 0 },
    { 0x630f54c3, 34 },
    { 0, 0 },
    { 0x8cdf54c5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f54cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f54d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08f54d3, 98 },
    { 0xffcf54d4, 76 },
    { 0xaebf54d4, 33 },
    { 0xf31f54d6, 61 },
    { 0x5bff54d5, 33 },
    { 0xfbf54d6, 55 },
    { 0x5caf54d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff54e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x617f54e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x256f54e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f54ec, 90 },
    { 0, 0 },
    { 0x428f54ee, 99 },
    { 0x399f54ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0af54f7, 66 },
    { 0xe2f54f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf54fc, 43 },
    { 0, 0 },
    { 0xdbef54fe, 89 },
    { 0, 0 },
    { 0x5f5500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4f5503, 23 },
    { 0, 0 },
    { 0x3c8f5505, 33 },
    { 0xc94f5506, 76 },
    { 0, 0 },
    { 0x7d2f5508, 81 },
    { 0xa5df5508, 95 },
    { 0xed8f550a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106f550f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f5517, 78 },
    { 0xc5cf5518, 33 },
    { 0, 0 },
    { 0xb19f551a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52f5521, 90 },
    { 0xe7f5522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f5526, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef552a, 61 },
    { 0x654f552b, 93 },
    { 0x9a9f552b, 34 },
    { 0x545f552a, 61 },
    { 0x19bf552e, 100 },
    { 0, 0 },
    { 0x3f8f5530, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f5533, 98 },
    { 0, 0 },
    { 0x7d8f5535, 76 },
    { 0x6e0f5536, 46 },
    { 0x177f5536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf553b, 78 },
    { 0, 0 },
    { 0xe04f553d, 94 },
    { 0x7d6f553e, 61 },
    { 0, 0 },
    { 0xa11f5540, 33 },
    { 0xca7f5541, 78 },
    { 0x4c3f5540, 55 },
    { 0x3d5f5543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5547, 33 },
    { 0, 0 },
    { 0xf30f5549, 95 },
    { 0, 0 },
    { 0x427f554b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdf554e, 23 },
    { 0x8b6f554f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aef5552, 90 },
    { 0x638f5553, 88 },
    { 0xc10f5554, 61 },
    { 0x383f5553, 43 },
    { 0x17f5556, 76 },
    { 0, 0 },
    { 0x608f5558, 93 },
    { 0, 0 },
    { 0xc27f555a, 85 },
    { 0x6f1f555b, 61 },
    { 0x76bf555c, 78 },
    { 0, 0 },
    { 0x229f555e, 100 },
    { 0x5faf555f, 33 },
    { 0x85ef5560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77df5567, 61 },
    { 0, 0 },
    { 0xe82f5569, 66 },
    { 0, 0 },
    { 0x13bf556b, 34 },
    { 0, 0 },
    { 0x699f556d, 43 },
    { 0, 0 },
    { 0xdecf556f, 33 },
    { 0, 0 },
    { 0x2f2f5571, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f557a, 76 },
    { 0, 0 },
    { 0x474f557c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f5585, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641f558a, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f558f, 43 },
    { 0, 0 },
    { 0x1b0f5591, 23 },
    { 0, 0 },
    { 0x96ef5593, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cef5596, 99 },
    { 0x943f5597, 33 },
    { 0xeb5f5597, 33 },
    { 0, 0 },
    { 0xdc4f559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f55a1, 95 },
    { 0x3fbf55a2, 87 },
    { 0xb9bf55a3, 55 },
    { 0, 0 },
    { 0x21f55a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f55aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92f55ae, 85 },
    { 0x5a9f55af, 90 },
    { 0x8c8f55af, 90 },
    { 0x63f55af, 90 },
    { 0x472f55b1, 43 },
    { 0x195f55b0, 61 },
    { 0xfebf55b4, 90 },
    { 0, 0 },
    { 0xc75f55b6, 68 },
    { 0x971f55b6, 88 },
    { 0xe58f55b6, 76 },
    { 0x635f55b9, 43 },
    { 0x3b5f55ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f55bf, 61 },
    { 0, 0 },
    { 0xe4df55c1, 77 },
    { 0xa2f55c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8f55c7, 95 },
    { 0x6f5f55c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ff55d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f55dd, 98 },
    { 0xabf55de, 43 },
    { 0xb0df55de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbff55e2, 98 },
    { 0, 0 },
    { 0x185f55e4, 23 },
    { 0x872f55e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3f55e8, 98 },
    { 0xeef55e9, 36 },
    { 0x333f55ea, 61 },
    { 0xd7cf55eb, 33 },
    { 0x1dbf55ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f55f7, 76 },
    { 0x6c7f55f8, 76 },
    { 0xb82f55f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefff55fe, 34 },
    { 0xf2ef55ff, 66 },
    { 0xa0ff55ff, 100 },
    { 0x1ccf55ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef5604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f5608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f560c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2f5611, 85 },
    { 0xad0f5612, 33 },
    { 0x690f5613, 76 },
    { 0, 0 },
    { 0xc2bf5615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x464f5618, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90df561c, 23 },
    { 0x2b8f561d, 33 },
    { 0x5acf561e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbf5621, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf5625, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f562d, 33 },
    { 0x46df562d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df5631, 61 },
    { 0xfa9f5632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71f563e, 98 },
    { 0, 0 },
    { 0xb41f5640, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df5643, 100 },
    { 0xca4f5644, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2f564a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244f5656, 52 },
    { 0xa45f5657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df565b, 23 },
    { 0x5e5f565b, 33 },
    { 0, 0 },
    { 0xaf565e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f5662, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ef566a, 43 },
    { 0x5fcf566b, 55 },
    { 0xf1cf566b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f5673, 55 },
    { 0xae1f5674, 90 },
    { 0, 0 },
    { 0x60cf5676, 55 },
    { 0, 0 },
    { 0xcb9f5678, 78 },
    { 0, 0 },
    { 0x47ef567a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f567d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99af5680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1f5687, 33 },
    { 0, 0 },
    { 0xf18f5689, 43 },
    { 0, 0 },
    { 0x7b6f568b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f568f, 98 },
    { 0x7d8f5690, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x366f5693, 46 },
    { 0, 0 },
    { 0xda6f5695, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcf5699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f569e, 87 },
    { 0x3dff569f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ff56a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x950f56a6, 87 },
    { 0xdcbf56a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f56ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf56b1, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92f56b5, 66 },
    { 0x419f56b6, 61 },
    { 0xfa4f56b7, 99 },
    { 0, 0 },
    { 0x914f56b9, 78 },
    { 0xdccf56ba, 33 },
    { 0, 0 },
    { 0x80df56bc, 90 },
    { 0, 0 },
    { 0xcc8f56be, 76 },
    { 0x8b6f56bf, 90 },
    { 0x613f56c0, 76 },
    { 0x521f56c1, 95 },
    { 0xd87f56c1, 99 },
    { 0, 0 },
    { 0x99bf56c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ef56c9, 93 },
    { 0x412f56ca, 43 },
    { 0, 0 },
    { 0x5b3f56cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3f56cf, 91 },
    { 0, 0 },
    { 0xedf56d1, 95 },
    { 0, 0 },
    { 0xeacf56d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaf56d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af56e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff56e5, 33 },
    { 0x43af56e6, 66 },
    { 0xd2f56e7, 91 },
    { 0x668f56e7, 61 },
    { 0, 0 },
    { 0x5ff56ea, 90 },
    { 0x4fef56ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef56ee, 34 },
    { 0xf42f56ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf56f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660f56ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6f5702, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72f5706, 91 },
    { 0, 0 },
    { 0x59cf5708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff570b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1f570e, 33 },
    { 0, 0 },
    { 0x7d9f5710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804f5715, 72 },
    { 0, 0 },
    { 0xfbdf5717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f571a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bf571e, 90 },
    { 0, 0 },
    { 0xa67f5720, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f572f, 61 },
    { 0xa60f5730, 17 },
    { 0x78bf5731, 91 },
    { 0x8b0f5731, 55 },
    { 0x9c3f5733, 95 },
    { 0xac2f5734, 33 },
    { 0x8ecf5735, 19 },
    { 0x707f5736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cf5743, 75 },
    { 0, 0 },
    { 0x943f5745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1def574b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1f5750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff5755, 11 },
    { 0, 0 },
    { 0x588f5757, 90 },
    { 0xdaef5757, 98 },
    { 0x7eaf5758, 87 },
    { 0x377f575a, 98 },
    { 0, 0 },
    { 0x207f575c, 23 },
    { 0x563f575d, 99 },
    { 0, 0 },
    { 0x501f575f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19f5767, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4af576a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f576d, 98 },
    { 0, 0 },
    { 0x3f4f576f, 61 },
    { 0x4c6f5770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf577b, 23 },
    { 0, 0 },
    { 0x752f577d, 78 },
    { 0xdebf577d, 84 },
    { 0x72ef577e, 98 },
    { 0, 0 },
    { 0x91cf5781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef5786, 90 },
    { 0xc88f5787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f578b, 33 },
    { 0xdc5f578c, 61 },
    { 0xd3cf578d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddf5790, 76 },
    { 0, 0 },
    { 0x6a3f5792, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1f5797, 90 },
    { 0x77ef5798, 90 },
    { 0x97df5799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf579c, 33 },
    { 0x5a7f579d, 64 },
    { 0, 0 },
    { 0x3e6f579f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x148f57a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f57a7, 69 },
    { 0xe1bf57a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1f57b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3f57b8, 85 },
    { 0xdef57b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f57bd, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758f57c1, 55 },
    { 0, 0 },
    { 0xbfbf57c3, 46 },
    { 0x43cf57c3, 34 },
    { 0x269f57c4, 75 },
    { 0x572f57c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3f57cb, 23 },
    { 0, 0 },
    { 0x7b6f57cd, 95 },
    { 0, 0 },
    { 0xccbf57cf, 95 },
    { 0x344f57d0, 90 },
    { 0xb2df57d1, 67 },
    { 0, 0 },
    { 0x8bdf57d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0df57dd, 55 },
    { 0, 0 },
    { 0xc65f57df, 95 },
    { 0, 0 },
    { 0x7cbf57e1, 98 },
    { 0x7b1f57e1, 33 },
    { 0, 0 },
    { 0x10ff57e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffef57f3, 88 },
    { 0xc40f57f3, 33 },
    { 0xd1ef57f5, 66 },
    { 0x5adf57f5, 84 },
    { 0x983f57f7, 98 },
    { 0xb80f57f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f57fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08f5801, 55 },
    { 0, 0 },
    { 0xcf6f5803, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf5807, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f580b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f580f, 66 },
    { 0x896f580f, 78 },
    { 0xd46f5811, 90 },
    { 0x708f5812, 33 },
    { 0x84ef5812, 34 },
    { 0xbeaf5814, 93 },
    { 0xa67f5814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64af5818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8f5826, 85 },
    { 0, 0 },
    { 0xbedf5828, 95 },
    { 0, 0 },
    { 0x330f582a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65f5830, 90 },
    { 0, 0 },
    { 0x97df5832, 98 },
    { 0x5dcf5833, 98 },
    { 0xbd0f5832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f583c, 76 },
    { 0x8ccf583c, 61 },
    { 0x2adf583e, 93 },
    { 0xdbaf583d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf5843, 90 },
    { 0, 0 },
    { 0x16ff5845, 81 },
    { 0, 0 },
    { 0x4cbf5847, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f5852, 46 },
    { 0x2b9f5853, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f5858, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f585c, 95 },
    { 0x7f2f585d, 69 },
    { 0, 0 },
    { 0xf9ff585f, 33 },
    { 0, 0 },
    { 0x149f5861, 90 },
    { 0, 0 },
    { 0x3e4f5863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74df586f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df5879, 52 },
    { 0xaebf5879, 91 },
    { 0x98f587a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982f587f, 65 },
    { 0x120f587f, 95 },
    { 0x70ff587f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f5884, 33 },
    { 0, 0 },
    { 0xdb6f5886, 99 },
    { 0x4e6f5887, 23 },
    { 0x7aff5888, 87 },
    { 0x367f5889, 90 },
    { 0xc0cf588a, 98 },
    { 0x18af588b, 93 },
    { 0xdf3f588c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6f5891, 95 },
    { 0x273f5892, 43 },
    { 0, 0 },
    { 0x321f5894, 95 },
    { 0xd28f5895, 98 },
    { 0x919f5896, 61 },
    { 0, 0 },
    { 0x699f5898, 61 },
    { 0, 0 },
    { 0xe2ff589a, 66 },
    { 0xcf1f589b, 93 },
    { 0xd28f589b, 66 },
    { 0, 0 },
    { 0xb07f589e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f58a3, 23 },
    { 0x445f58a4, 33 },
    { 0xbcff58a4, 17 },
    { 0, 0 },
    { 0xb21f58a7, 33 },
    { 0, 0 },
    { 0xc7ef58a9, 55 },
    { 0xc7f58aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadf58ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1f58b1, 76 },
    { 0x5e5f58b1, 95 },
    { 0, 0 },
    { 0x298f58b4, 61 },
    { 0x223f58b5, 65 },
    { 0x12bf58b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f58b9, 43 },
    { 0x302f58ba, 23 },
    { 0x8daf58b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df58c2, 98 },
    { 0xce5f58c2, 91 },
    { 0x2e6f58c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cef58cc, 95 },
    { 0xb44f58cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ff58d4, 55 },
    { 0x33af58d5, 17 },
    { 0x665f58d6, 95 },
    { 0x568f58d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef58dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x583f58e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9f58e7, 61 },
    { 0xa01f58e7, 33 },
    { 0x7f7f58e9, 55 },
    { 0xa8cf58e7, 46 },
    { 0xd88f58e9, 61 },
    { 0, 0 },
    { 0x32ff58ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29f58f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf58f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f58fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977f5902, 95 },
    { 0, 0 },
    { 0xeef5904, 23 },
    { 0, 0 },
    { 0x127f5906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ff590a, 23 },
    { 0x66af590a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f590f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef5916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5f5919, 55 },
    { 0, 0 },
    { 0x80f591b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5f5920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff5927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f592a, 90 },
    { 0, 0 },
    { 0xd01f592c, 84 },
    { 0xc1bf592d, 90 },
    { 0, 0 },
    { 0xe2df592f, 97 },
    { 0, 0 },
    { 0xf43f5931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacff5935, 61 },
    { 0xbadf5936, 98 },
    { 0, 0 },
    { 0x8dff5938, 61 },
    { 0xefef5939, 52 },
    { 0x708f593a, 98 },
    { 0x40ff593b, 91 },
    { 0x834f593a, 55 },
    { 0x59ff593d, 98 },
    { 0x438f593d, 95 },
    { 0x574f593f, 90 },
    { 0, 0 },
    { 0x653f5941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f5944, 95 },
    { 0x588f5944, 100 },
    { 0x730f5946, 66 },
    { 0xd33f5946, 90 },
    { 0x910f5947, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97f594b, 95 },
    { 0, 0 },
    { 0x442f594d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379f5954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bf5957, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cf595a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf595d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f5962, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bf596b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddf596e, 90 },
    { 0x878f596e, 56 },
    { 0, 0 },
    { 0xb8ff5971, 77 },
    { 0, 0 },
    { 0x95df5973, 33 },
    { 0x225f5973, 65 },
    { 0x13ff5973, 33 },
    { 0x994f5976, 43 },
    { 0x375f5977, 98 },
    { 0x461f5978, 67 },
    { 0xff2f5979, 33 },
    { 0, 0 },
    { 0xae4f597b, 95 },
    { 0x687f597b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff5980, 98 },
    { 0x10af5980, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8f598a, 78 },
    { 0x878f598b, 90 },
    { 0, 0 },
    { 0xdb4f598d, 52 },
    { 0, 0 },
    { 0x5a0f598f, 43 },
    { 0x3d8f5990, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebf5993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af5998, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf599c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f59a1, 23 },
    { 0x5e9f59a2, 69 },
    { 0x7c3f59a2, 93 },
    { 0x8f5f59a4, 46 },
    { 0xef8f59a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f59b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bf59b5, 67 },
    { 0, 0 },
    { 0xbfff59b7, 23 },
    { 0x440f59b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f59bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418f59c2, 55 },
    { 0x8c2f59c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df59c7, 85 },
    { 0x61ef59c7, 66 },
    { 0xdb9f59c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af59cc, 76 },
    { 0xba0f59cc, 98 },
    { 0xc21f59cd, 69 },
    { 0, 0 },
    { 0x25bf59d0, 89 },
    { 0x5bbf59d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710f59d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38f59db, 23 },
    { 0x80df59db, 55 },
    { 0xa69f59dd, 98 },
    { 0x2adf59dd, 76 },
    { 0, 0 },
    { 0xab0f59e0, 95 },
    { 0xffdf59e0, 23 },
    { 0xa13f59e2, 95 },
    { 0xfb2f59e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ff59e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ef59eb, 33 },
    { 0xb6bf59ec, 85 },
    { 0xecff59ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3f59f0, 91 },
    { 0xdfff59f0, 76 },
    { 0x642f59f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f59f6, 33 },
    { 0x40ef59f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef59fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642f5a0c, 33 },
    { 0x739f5a0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f5a11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1f5a14, 95 },
    { 0x944f5a15, 93 },
    { 0, 0 },
    { 0x685f5a17, 90 },
    { 0x690f5a18, 43 },
    { 0x47cf5a19, 84 },
    { 0xeff5a19, 76 },
    { 0x74ff5a1b, 77 },
    { 0x767f5a1c, 95 },
    { 0x1fff5a1d, 76 },
    { 0x3dcf5a1d, 61 },
    { 0x570f5a1f, 34 },
    { 0x1d4f5a1d, 33 },
    { 0x7e1f5a1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729f5a2a, 46 },
    { 0x3c8f5a2b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254f5a30, 90 },
    { 0x94cf5a31, 83 },
    { 0, 0 },
    { 0xcfef5a33, 98 },
    { 0, 0 },
    { 0x9f5f5a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13af5a38, 95 },
    { 0x8a7f5a39, 23 },
    { 0xbb6f5a38, 90 },
    { 0xe65f5a3b, 61 },
    { 0xbdbf5a3c, 84 },
    { 0, 0 },
    { 0x143f5a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78af5a43, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1f5a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208f5a4d, 76 },
    { 0x569f5a4e, 90 },
    { 0, 0 },
    { 0x6f2f5a50, 95 },
    { 0x8aef5a50, 33 },
    { 0, 0 },
    { 0x83f5a53, 55 },
    { 0xb09f5a54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12f5a59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f5a5e, 100 },
    { 0x43bf5a5f, 91 },
    { 0, 0 },
    { 0x802f5a61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f5a67, 91 },
    { 0x9bef5a68, 90 },
    { 0x583f5a67, 98 },
    { 0x600f5a6a, 98 },
    { 0xdd6f5a6b, 38 },
    { 0, 0 },
    { 0x164f5a6d, 17 },
    { 0, 0 },
    { 0x5d0f5a6f, 76 },
    { 0, 0 },
    { 0xf09f5a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29f5a74, 17 },
    { 0x4a3f5a74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddf5a7a, 23 },
    { 0x656f5a7a, 99 },
    { 0, 0 },
    { 0xf0cf5a7d, 61 },
    { 0xb7bf5a7e, 55 },
    { 0x575f5a7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9f5a87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ef5a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f5a90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18f5a93, 96 },
    { 0x556f5a94, 17 },
    { 0, 0 },
    { 0x6eff5a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f5a9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff5aa5, 68 },
    { 0x9bdf5aa6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f5aa9, 93 },
    { 0x988f5aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81f5aae, 91 },
    { 0, 0 },
    { 0x8fbf5ab0, 76 },
    { 0, 0 },
    { 0x26ff5ab2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678f5ab9, 23 },
    { 0, 0 },
    { 0x2d9f5abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf5ac2, 23 },
    { 0x990f5ac3, 55 },
    { 0x1cef5ac2, 61 },
    { 0, 0 },
    { 0x8cf5ac6, 98 },
    { 0, 0 },
    { 0xd78f5ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f5ace, 98 },
    { 0xb72f5acf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27f5ad2, 61 },
    { 0, 0 },
    { 0x7f4f5ad4, 76 },
    { 0xc22f5ad5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9f5adb, 90 },
    { 0x765f5adc, 43 },
    { 0xd9af5add, 33 },
    { 0xb2df5ade, 90 },
    { 0xcbdf5adf, 95 },
    { 0, 0 },
    { 0xa08f5ae1, 33 },
    { 0, 0 },
    { 0x1dff5ae3, 90 },
    { 0x69cf5ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33f5ae9, 43 },
    { 0x72df5aea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6f5aef, 64 },
    { 0, 0 },
    { 0x113f5af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31f5afb, 95 },
    { 0, 0 },
    { 0x650f5afd, 100 },
    { 0xcfff5afe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f5b01, 61 },
    { 0x1df5b01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22f5b08, 90 },
    { 0, 0 },
    { 0x4aff5b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bf5b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f5b13, 95 },
    { 0x341f5b13, 97 },
    { 0x38f5b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f5b18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf5b1b, 76 },
    { 0, 0 },
    { 0xc59f5b1d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3f5b22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f5b25, 34 },
    { 0, 0 },
    { 0xb65f5b27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x294f5b2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df5b2e, 68 },
    { 0x4e2f5b2f, 91 },
    { 0x41af5b2e, 90 },
    { 0xc8bf5b31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165f5b35, 55 },
    { 0x98ff5b36, 43 },
    { 0x89ff5b37, 23 },
    { 0x372f5b36, 33 },
    { 0, 0 },
    { 0xed4f5b3a, 75 },
    { 0xa72f5b3a, 75 },
    { 0x860f5b3b, 95 },
    { 0x4aff5b3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef5b42, 77 },
    { 0x2bf5b43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4f5b47, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f5b4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79f5b50, 99 },
    { 0xecbf5b51, 76 },
    { 0xe1cf5b52, 100 },
    { 0xe96f5b52, 61 },
    { 0xb7af5b53, 67 },
    { 0x109f5b55, 90 },
    { 0x7bcf5b56, 90 },
    { 0x4dff5b56, 93 },
    { 0x707f5b58, 46 },
    { 0xeb0f5b52, 96 },
    { 0x6d6f5b52, 95 },
    { 0xe94f5b55, 65 },
    { 0xc12f5b55, 90 },
    { 0, 0 },
    { 0xd21f5b5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f5b63, 76 },
    { 0x625f5b63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9f5b68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ef5b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f5b77, 23 },
    { 0x37f5b77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaf5b7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0f5b82, 95 },
    { 0, 0 },
    { 0xd38f5b84, 23 },
    { 0x8a6f5b85, 76 },
    { 0xc9bf5b86, 90 },
    { 0xbb0f5b86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83f5b8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ef5b90, 90 },
    { 0x70ef5b91, 76 },
    { 0x6a3f5b91, 17 },
    { 0, 0 },
    { 0x4a2f5b94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f5b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3f5b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4f5ba4, 84 },
    { 0xda3f5ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5f5ba8, 33 },
    { 0, 0 },
    { 0x66f5baa, 91 },
    { 0xc2cf5bab, 23 },
    { 0xdabf5bab, 98 },
    { 0x22df5bad, 33 },
    { 0xb7ef5bac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293f5bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af5bba, 95 },
    { 0x6d9f5bba, 98 },
    { 0, 0 },
    { 0xe3cf5bbd, 33 },
    { 0xf20f5bbe, 23 },
    { 0x60f5bbf, 55 },
    { 0x729f5bc0, 91 },
    { 0, 0 },
    { 0x678f5bc2, 61 },
    { 0, 0 },
    { 0x368f5bc4, 52 },
    { 0xd47f5bc5, 90 },
    { 0x3ddf5bc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f5bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1f5bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf5bd1, 55 },
    { 0xf79f5bd2, 95 },
    { 0x5c6f5bd2, 34 },
    { 0x82cf5bd4, 90 },
    { 0, 0 },
    { 0xc66f5bd6, 66 },
    { 0x9e7f5bd6, 61 },
    { 0, 0 },
    { 0x93bf5bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff5bde, 78 },
    { 0xc2ef5bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff5be3, 23 },
    { 0x7def5be3, 90 },
    { 0x980f5be5, 43 },
    { 0x738f5be5, 90 },
    { 0xd34f5be7, 34 },
    { 0xa69f5be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcf5bee, 61 },
    { 0x6c1f5bef, 33 },
    { 0xc41f5bee, 61 },
    { 0x974f5bf0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f5bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddf5bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05f5bfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x827f5bfe, 28 },
    { 0x3cef5bfe, 95 },
    { 0xd3ef5c00, 100 },
    { 0x9d5f5bfe, 90 },
    { 0, 0 },
    { 0xb97f5c03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedff5c07, 61 },
    { 0, 0 },
    { 0xfe9f5c09, 90 },
    { 0xfbcf5c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f5c14, 84 },
    { 0x69f5c15, 23 },
    { 0, 0 },
    { 0x556f5c17, 66 },
    { 0x6c9f5c18, 76 },
    { 0x879f5c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f5c1d, 33 },
    { 0, 0 },
    { 0x25ef5c1f, 78 },
    { 0xf2f5c1f, 100 },
    { 0x41df5c21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf5c2a, 90 },
    { 0, 0 },
    { 0xbeff5c2c, 90 },
    { 0x2af5c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61f5c31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af5c34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f5c37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f5c3d, 55 },
    { 0, 0 },
    { 0x7eff5c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f5c4c, 95 },
    { 0, 0 },
    { 0x8a2f5c4e, 61 },
    { 0, 0 },
    { 0x126f5c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f5c53, 23 },
    { 0x1e6f5c54, 76 },
    { 0x3c5f5c55, 87 },
    { 0x6f1f5c56, 61 },
    { 0xde3f5c57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56af5c5e, 95 },
    { 0x5a8f5c5f, 90 },
    { 0, 0 },
    { 0x282f5c61, 43 },
    { 0, 0 },
    { 0x915f5c63, 33 },
    { 0xa69f5c63, 61 },
    { 0x249f5c64, 61 },
    { 0xa56f5c66, 98 },
    { 0x7aff5c67, 94 },
    { 0x1af5c68, 23 },
    { 0xe64f5c67, 76 },
    { 0xbbf5c6a, 93 },
    { 0x964f5c6b, 43 },
    { 0xd0ef5c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff5c70, 99 },
    { 0x5d1f5c70, 76 },
    { 0xcd6f5c72, 33 },
    { 0x468f5c70, 23 },
    { 0x91bf5c71, 87 },
    { 0x4a3f5c70, 95 },
    { 0x4b7f5c76, 33 },
    { 0xa08f5c76, 23 },
    { 0x718f5c78, 33 },
    { 0xf25f5c73, 90 },
    { 0xd12f5c7a, 66 },
    { 0x18af5c7b, 33 },
    { 0x78ff5c7b, 90 },
    { 0x6cef5c70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283f5c83, 77 },
    { 0xa11f5c83, 81 },
    { 0x9d6f5c83, 98 },
    { 0xaf5c86, 90 },
    { 0xf50f5c83, 34 },
    { 0xd13f5c86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304f5c8c, 55 },
    { 0, 0 },
    { 0x6ddf5c8e, 72 },
    { 0, 0 },
    { 0x46bf5c90, 95 },
    { 0xdacf5c90, 55 },
    { 0xbeef5c91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f5c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f5c99, 66 },
    { 0x17f5c9a, 98 },
    { 0x1f6f5c9a, 33 },
    { 0x54cf5c9b, 85 },
    { 0x111f5c9d, 90 },
    { 0x64bf5c9e, 98 },
    { 0x9c4f5c9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465f5ca5, 33 },
    { 0, 0 },
    { 0xbfaf5ca7, 90 },
    { 0x654f5ca7, 23 },
    { 0x7df5ca8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448f5cad, 90 },
    { 0xec4f5cad, 95 },
    { 0xe33f5cad, 90 },
    { 0x92f5cb0, 98 },
    { 0xc49f5cad, 90 },
    { 0x414f5caf, 84 },
    { 0xa4ef5cb3, 55 },
    { 0xa0bf5cb4, 88 },
    { 0x86bf5cb4, 23 },
    { 0, 0 },
    { 0x63bf5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89f5cbb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdf5cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef5cc3, 34 },
    { 0xcfff5cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abf5ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf5cd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817f5cd5, 46 },
    { 0, 0 },
    { 0x186f5cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccf5cde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acf5ce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f5ce7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600f5cec, 43 },
    { 0x56ff5ced, 81 },
    { 0, 0 },
    { 0x97af5cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247f5cf6, 23 },
    { 0xf95f5cf6, 95 },
    { 0x2e8f5cf6, 56 },
    { 0x256f5cf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f5d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3f5d05, 81 },
    { 0x956f5d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6f5d09, 61 },
    { 0x72af5d0a, 93 },
    { 0x86f5d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cf5d0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6f5d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12f5d1a, 46 },
    { 0xa1af5d1b, 52 },
    { 0x27f5d1a, 95 },
    { 0, 0 },
    { 0x678f5d1e, 97 },
    { 0, 0 },
    { 0xfeef5d20, 90 },
    { 0xdc4f5d21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0f5d27, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef5d2a, 85 },
    { 0x108f5d2b, 88 },
    { 0xf46f5d2b, 33 },
    { 0, 0 },
    { 0x49af5d2e, 98 },
    { 0, 0 },
    { 0x98bf5d30, 95 },
    { 0xc00f5d31, 92 },
    { 0xd87f5d30, 66 },
    { 0x93ff5d33, 95 },
    { 0x239f5d34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f5d3d, 61 },
    { 0x399f5d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf5d48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf5d4b, 90 },
    { 0x2b8f5d4c, 34 },
    { 0, 0 },
    { 0x62af5d4e, 80 },
    { 0, 0 },
    { 0x6fdf5d50, 76 },
    { 0, 0 },
    { 0x144f5d52, 99 },
    { 0, 0 },
    { 0xd2f5d54, 84 },
    { 0, 0 },
    { 0x963f5d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ef5d63, 90 },
    { 0, 0 },
    { 0x81ff5d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabf5d75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bf5d78, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f5d7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cf5d82, 91 },
    { 0, 0 },
    { 0xb1bf5d84, 90 },
    { 0x2d3f5d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf5d8a, 87 },
    { 0xeadf5d8b, 78 },
    { 0, 0 },
    { 0x526f5d8d, 23 },
    { 0x76af5d8d, 33 },
    { 0, 0 },
    { 0xd82f5d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f5d95, 95 },
    { 0, 0 },
    { 0x490f5d97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f5da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f5da5, 93 },
    { 0xa0bf5da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ef5da9, 33 },
    { 0, 0 },
    { 0x52df5dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff5db3, 85 },
    { 0x40af5db4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbf5db7, 90 },
    { 0, 0 },
    { 0xfc4f5db9, 61 },
    { 0x4df5dba, 98 },
    { 0x588f5dba, 46 },
    { 0x11ef5dbb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ff5dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff5dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91f5dd3, 23 },
    { 0, 0 },
    { 0xe8bf5dd5, 33 },
    { 0x1bdf5dd6, 33 },
    { 0xa75f5dd7, 61 },
    { 0xd8ff5dd7, 93 },
    { 0x1e0f5dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2f5dde, 76 },
    { 0x6fcf5ddf, 87 },
    { 0x53ff5dde, 87 },
    { 0xf60f5ddf, 98 },
    { 0x2e1f5de1, 66 },
    { 0xea0f5de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f5de7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8f5deb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f5dee, 52 },
    { 0x7cf5dee, 61 },
    { 0xfbff5def, 98 },
    { 0x86f5df1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ef5df5, 61 },
    { 0xff2f5df5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfff5dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01f5dff, 52 },
    { 0x928f5e00, 100 },
    { 0xc0df5dff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f5e0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef5e0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63af5e12, 93 },
    { 0x21df5e13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aff5e18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6f5e1c, 33 },
    { 0, 0 },
    { 0x353f5e1e, 95 },
    { 0x45df5e1f, 55 },
    { 0xe5df5e20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf5e25, 23 },
    { 0xfe4f5e25, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f5e2a, 55 },
    { 0xebdf5e2b, 68 },
    { 0, 0 },
    { 0x4d8f5e2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f5e34, 76 },
    { 0, 0 },
    { 0xe25f5e36, 55 },
    { 0, 0 },
    { 0x8ff5e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6af5e40, 95 },
    { 0, 0 },
    { 0xf5df5e42, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5f5e46, 84 },
    { 0x7d7f5e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96f5e4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf5e4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741f5e56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f5e59, 84 },
    { 0x3ccf5e5a, 61 },
    { 0x7c5f5e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf5e60, 95 },
    { 0x9c3f5e61, 95 },
    { 0, 0 },
    { 0x885f5e63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddf5e6a, 99 },
    { 0x99f5e6b, 49 },
    { 0x1acf5e6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f5e7b, 33 },
    { 0xaf3f5e7c, 55 },
    { 0xc07f5e7c, 95 },
    { 0x650f5e7e, 23 },
    { 0xef8f5e7f, 76 },
    { 0, 0 },
    { 0x8a7f5e81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aff5e8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f5e94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bf5e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f5e9c, 33 },
    { 0xb9cf5e9c, 99 },
    { 0x9dbf5e9e, 77 },
    { 0x6baf5e9e, 56 },
    { 0x7c0f5ea0, 23 },
    { 0x98f5ea1, 76 },
    { 0xbc4f5ea2, 61 },
    { 0x317f5ea2, 90 },
    { 0x502f5ea4, 90 },
    { 0x23f5ea4, 33 },
    { 0x8ef5e9c, 81 },
    { 0, 0 },
    { 0x9b2f5ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90f5eae, 90 },
    { 0x1cbf5eaf, 61 },
    { 0, 0 },
    { 0xcf6f5eb1, 87 },
    { 0, 0 },
    { 0x564f5eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bf5eb8, 61 },
    { 0x389f5eb8, 99 },
    { 0x73af5eba, 55 },
    { 0x987f5ebb, 90 },
    { 0xbe8f5ebc, 90 },
    { 0x216f5eb8, 46 },
    { 0x3e4f5ebe, 90 },
    { 0x88df5ebd, 43 },
    { 0x7a0f5ec0, 34 },
    { 0x8cff5ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef5ec8, 76 },
    { 0x2e0f5ec8, 90 },
    { 0, 0 },
    { 0x458f5ecb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f5ed2, 43 },
    { 0, 0 },
    { 0x207f5ed4, 69 },
    { 0x122f5ed5, 55 },
    { 0x2ef5ed5, 95 },
    { 0x665f5ed7, 61 },
    { 0x7baf5ed8, 90 },
    { 0x346f5ed8, 43 },
    { 0xb52f5eda, 91 },
    { 0xf9ef5edb, 33 },
    { 0, 0 },
    { 0x8a9f5edd, 33 },
    { 0, 0 },
    { 0x12bf5edf, 90 },
    { 0x3a8f5ee0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccf5ee3, 95 },
    { 0x24bf5ee3, 43 },
    { 0xa46f5ee4, 91 },
    { 0, 0 },
    { 0x168f5ee7, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cf5eea, 97 },
    { 0x1e0f5eeb, 95 },
    { 0xf66f5eea, 23 },
    { 0, 0 },
    { 0x56cf5eee, 88 },
    { 0x8c7f5eef, 33 },
    { 0xf06f5ef0, 95 },
    { 0x452f5ef0, 33 },
    { 0x5cdf5ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1f5ef5, 98 },
    { 0, 0 },
    { 0x52af5ef7, 33 },
    { 0x63df5ef8, 23 },
    { 0x219f5ef9, 84 },
    { 0x16df5efa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74f5efd, 23 },
    { 0x940f5efd, 81 },
    { 0, 0 },
    { 0x16df5f00, 56 },
    { 0xf96f5f01, 98 },
    { 0x9abf5f02, 55 },
    { 0xd7cf5f01, 98 },
    { 0x3aaf5f00, 46 },
    { 0x6c2f5f05, 77 },
    { 0, 0 },
    { 0x3f2f5f07, 51 },
    { 0x53af5f08, 98 },
    { 0xb68f5f09, 23 },
    { 0, 0 },
    { 0x986f5f0b, 90 },
    { 0, 0 },
    { 0xefbf5f0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f5f11, 99 },
    { 0x44f5f12, 17 },
    { 0x3f2f5f13, 33 },
    { 0xbdf5f13, 76 },
    { 0x73af5f15, 93 },
    { 0x6c9f5f16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cf5f23, 87 },
    { 0, 0 },
    { 0x573f5f25, 90 },
    { 0x7abf5f25, 43 },
    { 0, 0 },
    { 0xa37f5f28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdf5f2b, 33 },
    { 0xc9af5f2c, 99 },
    { 0, 0 },
    { 0x366f5f2e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5f32, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf5f36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef5f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f5f45, 33 },
    { 0xceff5f46, 33 },
    { 0, 0 },
    { 0x6ddf5f48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x230f5f4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf5f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x394f5f51, 85 },
    { 0x882f5f52, 66 },
    { 0x5a3f5f52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2f5f56, 90 },
    { 0, 0 },
    { 0xb95f5f58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f5f62, 23 },
    { 0, 0 },
    { 0x72df5f64, 81 },
    { 0, 0 },
    { 0x9b8f5f66, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f5f6a, 98 },
    { 0x73f5f6b, 95 },
    { 0x42bf5f6c, 34 },
    { 0x119f5f6d, 17 },
    { 0x38df5f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3f5f71, 88 },
    { 0x6e1f5f71, 90 },
    { 0x55ef5f72, 87 },
    { 0, 0 },
    { 0x786f5f75, 90 },
    { 0x61af5f76, 95 },
    { 0x773f5f76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123f5f83, 61 },
    { 0x170f5f84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206f5f89, 65 },
    { 0x188f5f8a, 78 },
    { 0xc39f5f8b, 52 },
    { 0x5eff5f8a, 90 },
    { 0x67ef5f8c, 87 },
    { 0xdff5f8e, 98 },
    { 0xcbbf5f8e, 23 },
    { 0x49ff5f8f, 76 },
    { 0x661f5f90, 67 },
    { 0xa58f5f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef5f9a, 36 },
    { 0x3d0f5f9b, 46 },
    { 0xd10f5f9a, 84 },
    { 0x9dbf5f9a, 91 },
    { 0, 0 },
    { 0x86f5f9f, 66 },
    { 0xe19f5fa0, 98 },
    { 0x720f5fa1, 23 },
    { 0xbdaf5fa1, 23 },
    { 0x70cf5fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698f5fa8, 90 },
    { 0xec5f5fa8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f5faf, 55 },
    { 0xf6ef5faf, 76 },
    { 0, 0 },
    { 0x30cf5fb2, 23 },
    { 0xca9f5fb2, 84 },
    { 0, 0 },
    { 0x276f5fb5, 33 },
    { 0, 0 },
    { 0x63bf5fb7, 61 },
    { 0xf66f5fb8, 55 },
    { 0x722f5fb9, 61 },
    { 0xb7f5fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f5fc0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf5fc3, 33 },
    { 0, 0 },
    { 0x6bef5fc5, 93 },
    { 0, 0 },
    { 0x7f7f5fc7, 33 },
    { 0x8bbf5fc8, 98 },
    { 0xe65f5fc9, 33 },
    { 0x977f5fc8, 50 },
    { 0xd88f5fc9, 55 },
    { 0xbacf5fcc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f5fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8f5fd5, 76 },
    { 0xb53f5fd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08f5fd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f5fe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x985f5fe3, 33 },
    { 0, 0 },
    { 0x713f5fe5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f5fea, 93 },
    { 0xb60f5fea, 61 },
    { 0x6af5fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f5fef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2f5ff2, 75 },
    { 0xc52f5ff3, 34 },
    { 0xbbcf5ff4, 99 },
    { 0xc07f5ff4, 34 },
    { 0x5ef5ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff5ffa, 95 },
    { 0x235f5ffb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af5fff, 81 },
    { 0, 0 },
    { 0xa15f6001, 61 },
    { 0x3e9f6002, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f6007, 46 },
    { 0x91ef6008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cff600e, 100 },
    { 0xeb9f600f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f6014, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931f6018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6f601c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f6026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cf6029, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acf602c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f6033, 90 },
    { 0x2e3f6034, 90 },
    { 0x444f6035, 23 },
    { 0x8fff6035, 61 },
    { 0xebff6037, 90 },
    { 0x8f1f6037, 55 },
    { 0xa7df6034, 76 },
    { 0x1cbf603a, 91 },
    { 0, 0 },
    { 0x56af603c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f6042, 33 },
    { 0xd1f6042, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08f6046, 94 },
    { 0xc30f6046, 95 },
    { 0, 0 },
    { 0x1ebf6049, 61 },
    { 0x734f604a, 90 },
    { 0x610f604a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799f6056, 55 },
    { 0xb64f6056, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9df605a, 90 },
    { 0x85df605b, 90 },
    { 0xddcf605c, 93 },
    { 0xd2df605a, 93 },
    { 0x588f605c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51f6062, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f6066, 40 },
    { 0, 0 },
    { 0x7af6068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f606b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250f6070, 90 },
    { 0x137f6070, 95 },
    { 0, 0 },
    { 0xb28f6073, 68 },
    { 0xca5f6073, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacf6077, 55 },
    { 0x309f6077, 98 },
    { 0x2d1f6077, 65 },
    { 0xbadf607a, 33 },
    { 0x4cbf607b, 86 },
    { 0x512f607c, 23 },
    { 0, 0 },
    { 0x20ff607e, 52 },
    { 0x5eef607f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201f6084, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f6088, 91 },
    { 0xe6df6089, 95 },
    { 0xc53f608a, 46 },
    { 0x8dbf608b, 90 },
    { 0x924f608c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f6091, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6df6095, 77 },
    { 0x846f6096, 23 },
    { 0, 0 },
    { 0x286f6098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f609c, 95 },
    { 0xd28f609c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0f60a4, 46 },
    { 0x544f60a5, 87 },
    { 0x8a0f60a5, 61 },
    { 0xd2df60a7, 84 },
    { 0xcaff60a8, 98 },
    { 0xad7f60a9, 77 },
    { 0xb2bf60a9, 98 },
    { 0x1cbf60a9, 78 },
    { 0xa5bf60aa, 90 },
    { 0x591f60ad, 33 },
    { 0xd3af60ab, 98 },
    { 0x4b1f60a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f60b7, 61 },
    { 0xe71f60b8, 87 },
    { 0, 0 },
    { 0x60df60ba, 98 },
    { 0xc5bf60bb, 84 },
    { 0x447f60bc, 90 },
    { 0x528f60bb, 61 },
    { 0x181f60be, 24 },
    { 0xe55f60ba, 95 },
    { 0x306f60bc, 43 },
    { 0x5f1f60c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93f60c8, 61 },
    { 0xb80f60c9, 53 },
    { 0, 0 },
    { 0xa8ff60cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f60d3, 90 },
    { 0, 0 },
    { 0x833f60d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf60d8, 91 },
    { 0xd10f60d8, 100 },
    { 0, 0 },
    { 0x711f60db, 95 },
    { 0x8e9f60dc, 61 },
    { 0xecaf60dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf60e0, 76 },
    { 0xa62f60e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af60e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f60ec, 55 },
    { 0x304f60ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2f60f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf60fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8f60fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8f6101, 43 },
    { 0, 0 },
    { 0x85ff6103, 33 },
    { 0x80bf6104, 61 },
    { 0xa8cf6105, 87 },
    { 0x261f6105, 46 },
    { 0x7e2f6106, 61 },
    { 0x5ef6108, 77 },
    { 0x89ff6109, 61 },
    { 0x227f6107, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2f610e, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342f6114, 98 },
    { 0xf3df6115, 78 },
    { 0x1c0f6114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf611a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaf611e, 23 },
    { 0x7c8f611f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acf6122, 34 },
    { 0, 0 },
    { 0xc80f6124, 55 },
    { 0x910f6124, 90 },
    { 0, 0 },
    { 0x26ef6127, 97 },
    { 0x7f7f6127, 34 },
    { 0x353f6129, 33 },
    { 0x4c9f6129, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f612d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf6131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2af6138, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff613d, 23 },
    { 0xc8bf613d, 88 },
    { 0xd6bf613e, 55 },
    { 0, 0 },
    { 0x5aaf6141, 95 },
    { 0, 0 },
    { 0x84bf6143, 55 },
    { 0x907f6144, 78 },
    { 0xc5ff6144, 23 },
    { 0, 0 },
    { 0x204f6147, 43 },
    { 0, 0 },
    { 0x834f6149, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f614c, 61 },
    { 0, 0 },
    { 0x11bf614e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473f6156, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f6159, 81 },
    { 0xa04f615a, 61 },
    { 0, 0 },
    { 0xc7f615c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f6161, 33 },
    { 0x56cf6162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f616d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f6170, 33 },
    { 0xb9cf6171, 43 },
    { 0x83df6172, 61 },
    { 0xf2ff6173, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef6176, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579f617b, 93 },
    { 0, 0 },
    { 0xa97f617d, 99 },
    { 0xb8bf617e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f6184, 100 },
    { 0x326f6185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aef6190, 99 },
    { 0xb54f6191, 61 },
    { 0xf44f6192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f6196, 95 },
    { 0, 0 },
    { 0x43df6198, 98 },
    { 0, 0 },
    { 0xe49f619a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41f61a3, 46 },
    { 0x5f9f61a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ff61ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f61b0, 33 },
    { 0x5dbf61b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f61b4, 76 },
    { 0, 0 },
    { 0x243f61b6, 55 },
    { 0x3d0f61b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3df61ba, 93 },
    { 0xa3bf61bb, 34 },
    { 0x633f61bc, 33 },
    { 0, 0 },
    { 0xebaf61be, 87 },
    { 0, 0 },
    { 0x129f61c0, 90 },
    { 0x367f61c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f61cb, 90 },
    { 0, 0 },
    { 0xd72f61cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef61d0, 55 },
    { 0x714f61d1, 23 },
    { 0x7f7f61d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f61d6, 99 },
    { 0x518f61d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f61da, 55 },
    { 0x3bdf61db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f61e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf61eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393f61f4, 50 },
    { 0x4d8f61f4, 46 },
    { 0, 0 },
    { 0xeaf61f7, 61 },
    { 0, 0 },
    { 0xd1ff61f9, 95 },
    { 0x554f61fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f61fd, 84 },
    { 0, 0 },
    { 0x480f61ff, 100 },
    { 0x6bbf6200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecf6203, 33 },
    { 0xc01f6204, 43 },
    { 0x3b8f6205, 68 },
    { 0x6f6204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552f620e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6214, 76 },
    { 0x8a7f6214, 66 },
    { 0, 0 },
    { 0x128f6217, 98 },
    { 0x72cf6218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25df621c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff621f, 61 },
    { 0x7c7f6220, 85 },
    { 0, 0 },
    { 0xe0bf6222, 76 },
    { 0, 0 },
    { 0xe81f6224, 95 },
    { 0xf37f6225, 33 },
    { 0, 0 },
    { 0x463f6227, 90 },
    { 0xa04f6227, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebf622b, 84 },
    { 0x2b5f622c, 78 },
    { 0xe21f622d, 61 },
    { 0x1dff622e, 85 },
    { 0xe1ef622f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf6234, 98 },
    { 0xe78f6235, 35 },
    { 0xbfcf6236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2f623b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x168f623e, 43 },
    { 0x352f623f, 91 },
    { 0xb17f6240, 76 },
    { 0x5aef6240, 90 },
    { 0xccff6242, 43 },
    { 0x1a4f623e, 46 },
    { 0x960f6244, 95 },
    { 0x4cdf6241, 43 },
    { 0x64bf6245, 88 },
    { 0x3e0f6247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f624a, 52 },
    { 0x36cf624b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6f6255, 90 },
    { 0, 0 },
    { 0x323f6257, 55 },
    { 0xc67f6257, 90 },
    { 0x59ef6259, 34 },
    { 0x7d7f625a, 99 },
    { 0xb47f625b, 33 },
    { 0, 0 },
    { 0x201f625d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedf6263, 66 },
    { 0x916f6263, 34 },
    { 0x9a2f6263, 91 },
    { 0xcf9f6266, 76 },
    { 0x203f6267, 61 },
    { 0x806f6264, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35f626c, 90 },
    { 0x5f4f626c, 90 },
    { 0, 0 },
    { 0xd81f626f, 93 },
    { 0, 0 },
    { 0x837f6271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf6277, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af627a, 61 },
    { 0x968f627b, 23 },
    { 0x626f627b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6282, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8df6286, 98 },
    { 0x4daf6286, 34 },
    { 0xb6f6287, 88 },
    { 0xc5cf6288, 61 },
    { 0xa79f6289, 34 },
    { 0, 0 },
    { 0x63f628c, 98 },
    { 0x7d3f628d, 23 },
    { 0x1e8f628e, 98 },
    { 0x2b5f628e, 87 },
    { 0x88ff6290, 100 },
    { 0x323f6291, 44 },
    { 0, 0 },
    { 0x88df6293, 90 },
    { 0x1dbf6294, 55 },
    { 0xf3df6294, 81 },
    { 0xcb8f6296, 90 },
    { 0x736f6297, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f629a, 90 },
    { 0, 0 },
    { 0xe65f629c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f62a7, 95 },
    { 0xb30f62a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf62b2, 23 },
    { 0x85ef62b2, 90 },
    { 0x97bf62b4, 76 },
    { 0xd1cf62b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3f62b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff62c0, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f62c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6def62cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4f62ce, 95 },
    { 0x472f62cf, 95 },
    { 0x47df62d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f62d4, 95 },
    { 0x8ccf62d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf62da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf62dd, 23 },
    { 0xb44f62dd, 95 },
    { 0xc15f62de, 55 },
    { 0xe95f62e0, 91 },
    { 0xf6ef62dd, 55 },
    { 0x4bff62de, 43 },
    { 0x1c6f62de, 52 },
    { 0x90af62e4, 46 },
    { 0xf54f62e5, 23 },
    { 0x94df62e4, 95 },
    { 0x33df62e5, 98 },
    { 0x6b9f62e5, 17 },
    { 0, 0 },
    { 0x493f62ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372f62ef, 61 },
    { 0xadf62ef, 85 },
    { 0x232f62f1, 33 },
    { 0x698f62f1, 43 },
    { 0xb37f62f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ff62f6, 98 },
    { 0, 0 },
    { 0xd9cf62f8, 81 },
    { 0x479f62f8, 55 },
    { 0xa7ff62fa, 90 },
    { 0x898f62fa, 95 },
    { 0x3a8f62fa, 95 },
    { 0, 0 },
    { 0x161f62fe, 67 },
    { 0, 0 },
    { 0xbaaf6300, 33 },
    { 0, 0 },
    { 0xfbef6302, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f6307, 66 },
    { 0, 0 },
    { 0x125f6309, 95 },
    { 0x4f7f630a, 43 },
    { 0, 0 },
    { 0xeadf630c, 93 },
    { 0xb40f630d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1f6310, 61 },
    { 0, 0 },
    { 0xfc3f6312, 33 },
    { 0, 0 },
    { 0x292f6314, 55 },
    { 0, 0 },
    { 0x6f4f6316, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf631b, 61 },
    { 0x4cff631c, 93 },
    { 0, 0 },
    { 0x342f631e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eef6323, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f6326, 76 },
    { 0, 0 },
    { 0xddbf6328, 61 },
    { 0x1e4f6329, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f632d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfff6334, 98 },
    { 0x272f6334, 90 },
    { 0, 0 },
    { 0x41ff6337, 52 },
    { 0x88cf6338, 33 },
    { 0x573f6337, 23 },
    { 0, 0 },
    { 0x310f633b, 76 },
    { 0x4bcf633c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecf6340, 34 },
    { 0xb87f6341, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f6344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f6348, 26 },
    { 0xca1f6349, 90 },
    { 0xd0ef634a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98f6351, 91 },
    { 0, 0 },
    { 0xaff6353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f6358, 69 },
    { 0, 0 },
    { 0xe88f635a, 76 },
    { 0, 0 },
    { 0x5bef635c, 76 },
    { 0x98cf635d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ef6360, 95 },
    { 0, 0 },
    { 0x56cf6362, 87 },
    { 0x4bf6363, 93 },
    { 0, 0 },
    { 0x6c1f6365, 34 },
    { 0, 0 },
    { 0x6abf6367, 93 },
    { 0, 0 },
    { 0x5fef6369, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621f636f, 43 },
    { 0, 0 },
    { 0xb19f6371, 17 },
    { 0x76af6372, 99 },
    { 0, 0 },
    { 0x462f6374, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df6377, 90 },
    { 0, 0 },
    { 0x5e4f6379, 33 },
    { 0x308f6379, 61 },
    { 0, 0 },
    { 0xa74f637c, 61 },
    { 0, 0 },
    { 0x80af637e, 61 },
    { 0x2c1f637e, 33 },
    { 0x1b0f6380, 90 },
    { 0xbe0f637f, 55 },
    { 0, 0 },
    { 0x699f6383, 52 },
    { 0, 0 },
    { 0xefef6385, 87 },
    { 0xbe8f6386, 33 },
    { 0x2b9f6385, 46 },
    { 0x5bcf6388, 78 },
    { 0xc24f6388, 33 },
    { 0, 0 },
    { 0xf40f638b, 99 },
    { 0, 0 },
    { 0xf04f638d, 23 },
    { 0x692f638d, 34 },
    { 0xe3cf638e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16df6393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aef639a, 66 },
    { 0, 0 },
    { 0x592f639c, 98 },
    { 0xd03f639c, 34 },
    { 0, 0 },
    { 0xe83f639f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f63a3, 76 },
    { 0, 0 },
    { 0x599f63a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f63a9, 61 },
    { 0x96cf63aa, 85 },
    { 0x5e8f63a9, 93 },
    { 0x6dcf63aa, 90 },
    { 0x4e7f63aa, 33 },
    { 0x946f63ae, 46 },
    { 0x793f63ae, 46 },
    { 0x5baf63b0, 47 },
    { 0x9ccf63b1, 33 },
    { 0x4fff63b1, 98 },
    { 0, 0 },
    { 0x9af63b4, 81 },
    { 0, 0 },
    { 0x339f63b6, 67 },
    { 0xd38f63b6, 78 },
    { 0x5a6f63b8, 98 },
    { 0x7b9f63b8, 33 },
    { 0x658f63ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61f63be, 90 },
    { 0x201f63bf, 78 },
    { 0xd1ef63c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9f63c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05f63c7, 61 },
    { 0, 0 },
    { 0xa95f63c9, 95 },
    { 0xe26f63ca, 90 },
    { 0, 0 },
    { 0x2cf63cc, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8f63d1, 52 },
    { 0xbf0f63d2, 33 },
    { 0x659f63d1, 43 },
    { 0, 0 },
    { 0x5edf63d5, 91 },
    { 0, 0 },
    { 0x120f63d7, 90 },
    { 0x4df63d8, 61 },
    { 0x8baf63d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f63dd, 87 },
    { 0xb1df63de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bf63e5, 23 },
    { 0x912f63e6, 93 },
    { 0, 0 },
    { 0x89bf63e8, 90 },
    { 0x60f63e9, 90 },
    { 0x60cf63ea, 99 },
    { 0, 0 },
    { 0x1fef63ec, 65 },
    { 0x67ef63ed, 90 },
    { 0, 0 },
    { 0x63cf63ef, 33 },
    { 0xaadf63f0, 95 },
    { 0x509f63f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2f63f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4f63f9, 90 },
    { 0, 0 },
    { 0x13ef63fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f640f, 34 },
    { 0xcd0f6410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f641d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caf6427, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf642f, 90 },
    { 0x4c9f6430, 34 },
    { 0xe8df6430, 33 },
    { 0x491f6431, 23 },
    { 0x48f6433, 90 },
    { 0x784f6434, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf6439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f643c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedf6440, 91 },
    { 0, 0 },
    { 0x1fcf6442, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfef644b, 78 },
    { 0x2daf644c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f6450, 34 },
    { 0x818f6451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f6455, 98 },
    { 0x26bf6456, 85 },
    { 0x290f6457, 46 },
    { 0x4d1f6458, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50f645c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf645f, 98 },
    { 0, 0 },
    { 0xe4f6461, 65 },
    { 0, 0 },
    { 0x287f6463, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbf6469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafff646d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0f6470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0f6474, 66 },
    { 0xc75f6474, 90 },
    { 0, 0 },
    { 0xe45f6477, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8f647a, 90 },
    { 0xf0ef647b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf6480, 23 },
    { 0xd9ef6481, 90 },
    { 0x325f6480, 55 },
    { 0x85f6483, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ef6486, 78 },
    { 0x810f6486, 46 },
    { 0x983f6486, 93 },
    { 0x3cef6489, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f648d, 91 },
    { 0x16f648e, 95 },
    { 0x8e3f648e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf6497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f649a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef64a7, 76 },
    { 0x226f64a7, 90 },
    { 0, 0 },
    { 0x76ff64aa, 33 },
    { 0x1ddf64ab, 61 },
    { 0x7ebf64ab, 23 },
    { 0x92af64ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58f64b1, 61 },
    { 0xd36f64b1, 84 },
    { 0, 0 },
    { 0x28ff64b4, 46 },
    { 0, 0 },
    { 0x867f64b6, 61 },
    { 0x245f64b7, 46 },
    { 0xb2df64b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf64c1, 90 },
    { 0x82df64c2, 99 },
    { 0, 0 },
    { 0xdfff64c4, 34 },
    { 0x2c4f64c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf64ca, 87 },
    { 0, 0 },
    { 0xdaff64cc, 66 },
    { 0x744f64cc, 52 },
    { 0x404f64ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996f64d2, 43 },
    { 0x3c9f64d2, 90 },
    { 0xf7cf64d4, 77 },
    { 0xc83f64d4, 34 },
    { 0x43df64d6, 61 },
    { 0, 0 },
    { 0x5d9f64d8, 65 },
    { 0x5fbf64d9, 76 },
    { 0xb07f64da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f64ed, 43 },
    { 0xaa0f64ee, 43 },
    { 0x72df64ef, 75 },
    { 0xb54f64ee, 46 },
    { 0xacaf64f1, 85 },
    { 0x15af64f1, 98 },
    { 0, 0 },
    { 0x460f64f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1f64fa, 95 },
    { 0x40df64fb, 83 },
    { 0, 0 },
    { 0x5b1f64fd, 23 },
    { 0xc0bf64fe, 23 },
    { 0xc83f64fe, 46 },
    { 0x99f6500, 90 },
    { 0x587f6501, 52 },
    { 0x7d2f6502, 76 },
    { 0x8baf6503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199f6509, 33 },
    { 0x847f6509, 33 },
    { 0xc11f650b, 33 },
    { 0, 0 },
    { 0x22f650d, 90 },
    { 0x9e3f650d, 61 },
    { 0x709f650d, 88 },
    { 0xb38f650e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf6513, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f6516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f6519, 95 },
    { 0, 0 },
    { 0x66af651b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f6522, 34 },
    { 0x58ff6523, 55 },
    { 0x4a8f6524, 61 },
    { 0xb37f6525, 53 },
    { 0x21cf6526, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf6529, 96 },
    { 0xbdcf652a, 33 },
    { 0xfb1f652b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3f6534, 76 },
    { 0xff6f6534, 95 },
    { 0, 0 },
    { 0x8f6537, 61 },
    { 0xb2df6538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x503f653b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f653e, 95 },
    { 0, 0 },
    { 0x248f6540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9f6545, 61 },
    { 0xa3ef6546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f654f, 43 },
    { 0xcd9f6550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0f6553, 91 },
    { 0x357f6554, 55 },
    { 0xad0f6555, 61 },
    { 0x844f6556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f6559, 95 },
    { 0, 0 },
    { 0xeecf655b, 95 },
    { 0xeb9f655c, 87 },
    { 0x3c7f655c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f6560, 100 },
    { 0, 0 },
    { 0x7fff6562, 33 },
    { 0xb6df6563, 61 },
    { 0x93bf6564, 78 },
    { 0xa13f6563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef6568, 100 },
    { 0x44ef6568, 61 },
    { 0, 0 },
    { 0x24ef656b, 95 },
    { 0x3cbf656b, 19 },
    { 0x937f656d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bf6572, 23 },
    { 0x6d0f6573, 76 },
    { 0x734f6574, 68 },
    { 0xcb4f6573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6f6579, 90 },
    { 0xa34f657a, 40 },
    { 0xc84f657b, 90 },
    { 0x428f657c, 85 },
    { 0x4d8f657d, 67 },
    { 0, 0 },
    { 0xf65f657f, 52 },
    { 0xca6f6580, 33 },
    { 0x515f6580, 61 },
    { 0x176f6582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af6586, 46 },
    { 0xfd8f6587, 90 },
    { 0xb2ff6587, 46 },
    { 0xbaaf6588, 77 },
    { 0xecaf6589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57f6593, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f6597, 33 },
    { 0, 0 },
    { 0x248f6599, 95 },
    { 0x2d7f6599, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf659e, 76 },
    { 0x52af659f, 23 },
    { 0x2c6f65a0, 90 },
    { 0xb11f65a0, 61 },
    { 0xbc1f65a2, 98 },
    { 0xcb6f659f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1f65a6, 91 },
    { 0x31ef65a7, 93 },
    { 0x935f65a8, 33 },
    { 0, 0 },
    { 0xe7df65aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f65b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f65b4, 92 },
    { 0x8a2f65b4, 76 },
    { 0x449f65b5, 98 },
    { 0xd73f65b7, 59 },
    { 0xf22f65b6, 61 },
    { 0, 0 },
    { 0xf65f65ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792f65c6, 33 },
    { 0x22af65c6, 34 },
    { 0x67df65c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f65cd, 90 },
    { 0x272f65ce, 78 },
    { 0x3cdf65cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489f65d4, 7 },
    { 0, 0 },
    { 0xe09f65d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf65d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15f65dc, 85 },
    { 0xddf65dd, 33 },
    { 0x420f65de, 90 },
    { 0x341f65df, 90 },
    { 0, 0 },
    { 0xd68f65e1, 33 },
    { 0, 0 },
    { 0x9a8f65e3, 61 },
    { 0, 0 },
    { 0xf3bf65e5, 95 },
    { 0, 0 },
    { 0xe08f65e7, 61 },
    { 0xb5df65e8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bff65ed, 61 },
    { 0x9bf65ee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5f65f4, 88 },
    { 0x5b0f65f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759f65f9, 87 },
    { 0xbef65fa, 98 },
    { 0x74ff65fa, 34 },
    { 0, 0 },
    { 0x883f65fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92f6603, 33 },
    { 0, 0 },
    { 0x8a5f6605, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef660a, 93 },
    { 0, 0 },
    { 0xa86f660c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f660f, 95 },
    { 0xea0f6610, 93 },
    { 0x1c0f6610, 34 },
    { 0x72ff6610, 95 },
    { 0xc0f6610, 98 },
    { 0x329f660f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4f6618, 24 },
    { 0, 0 },
    { 0xf26f661a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f661d, 76 },
    { 0xf6f661e, 98 },
    { 0xe95f661e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162f6625, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x354f6628, 61 },
    { 0x56bf6629, 61 },
    { 0x14f662a, 23 },
    { 0xeaef6629, 61 },
    { 0x752f662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x199f662f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09f6635, 95 },
    { 0xcc7f6636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ff663c, 90 },
    { 0xc22f663c, 34 },
    { 0, 0 },
    { 0x504f663f, 78 },
    { 0xc67f6640, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261f6648, 90 },
    { 0x3bcf6649, 61 },
    { 0x573f6649, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeef664f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f6652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f6656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf6659, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4f665d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f6666, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378f666d, 52 },
    { 0x918f666d, 90 },
    { 0, 0 },
    { 0x2edf6670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6675, 17 },
    { 0x12cf6676, 90 },
    { 0x79df6677, 100 },
    { 0xd9cf6678, 87 },
    { 0x330f6678, 93 },
    { 0x194f667a, 91 },
    { 0x4d7f667b, 76 },
    { 0xf93f667a, 90 },
    { 0, 0 },
    { 0x220f667e, 90 },
    { 0x903f667f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6f6684, 95 },
    { 0x23ef6684, 43 },
    { 0x8f3f6686, 98 },
    { 0xc8bf6687, 90 },
    { 0, 0 },
    { 0x104f6689, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf668d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1f6690, 95 },
    { 0, 0 },
    { 0xb4cf6692, 78 },
    { 0x553f6692, 90 },
    { 0x682f6694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11f6697, 78 },
    { 0, 0 },
    { 0x152f6699, 23 },
    { 0, 0 },
    { 0xde0f669b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df669e, 33 },
    { 0, 0 },
    { 0x5d6f66a0, 95 },
    { 0xf70f66a1, 55 },
    { 0x1f66a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05f66a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf66ad, 66 },
    { 0x585f66ae, 90 },
    { 0x32af66ad, 98 },
    { 0x9f3f66b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f66b5, 55 },
    { 0, 0 },
    { 0x986f66b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x150f66ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66be, 48 },
    { 0, 0 },
    { 0xc55f66c0, 23 },
    { 0xf97f66c0, 33 },
    { 0x43df66c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03f66c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f66ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f66d2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63f66d6, 98 },
    { 0x8d8f66d7, 99 },
    { 0xaa5f66d6, 55 },
    { 0x242f66d7, 98 },
    { 0x75f66d6, 23 },
    { 0x115f66db, 33 },
    { 0x606f66dc, 43 },
    { 0x5c7f66da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f66e1, 23 },
    { 0xf82f66e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7f66e7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef66ea, 33 },
    { 0xf81f66ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65f66ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x244f66f1, 98 },
    { 0, 0 },
    { 0xff2f66f3, 61 },
    { 0x19ef66f4, 91 },
    { 0x41ef66f5, 90 },
    { 0, 0 },
    { 0x76df66f7, 68 },
    { 0xd0af66f7, 90 },
    { 0xd67f66f7, 23 },
    { 0x88af66f7, 46 },
    { 0x8f6f66fa, 61 },
    { 0, 0 },
    { 0xad5f66fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7af6703, 23 },
    { 0xa06f6704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495f6708, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff670d, 87 },
    { 0xf15f670e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf6712, 90 },
    { 0x2e4f6713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890f6718, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f671e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ff6722, 34 },
    { 0x963f6723, 87 },
    { 0xb40f6723, 95 },
    { 0x5af6725, 55 },
    { 0xe1bf6722, 84 },
    { 0, 0 },
    { 0x652f6728, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f672d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0f6730, 33 },
    { 0xa0df6730, 98 },
    { 0x4d7f6731, 43 },
    { 0x49bf6733, 95 },
    { 0, 0 },
    { 0x7dbf6735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bf673e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f6744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf674a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8f6753, 90 },
    { 0, 0 },
    { 0x4d1f6755, 38 },
    { 0x192f6756, 91 },
    { 0x2d1f6755, 61 },
    { 0x282f6758, 23 },
    { 0, 0 },
    { 0x11ef675a, 33 },
    { 0x953f675a, 59 },
    { 0, 0 },
    { 0x7a8f675d, 23 },
    { 0x95f675d, 98 },
    { 0x514f675d, 90 },
    { 0x5a8f6760, 66 },
    { 0x66ef6761, 61 },
    { 0x8f6762, 95 },
    { 0, 0 },
    { 0xf2bf6764, 76 },
    { 0, 0 },
    { 0x38cf6766, 61 },
    { 0xb6cf6767, 23 },
    { 0, 0 },
    { 0x95cf6769, 99 },
    { 0x3ff676a, 61 },
    { 0x232f6769, 95 },
    { 0x242f676c, 61 },
    { 0x1daf676c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0f6770, 55 },
    { 0xf38f6771, 90 },
    { 0x768f6770, 43 },
    { 0x771f6772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f6779, 90 },
    { 0x7b5f677a, 100 },
    { 0xee9f677b, 33 },
    { 0, 0 },
    { 0xd48f677d, 33 },
    { 0x973f677e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26f6781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff6788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4f678b, 76 },
    { 0, 0 },
    { 0x48f678d, 23 },
    { 0xf9ef678e, 100 },
    { 0, 0 },
    { 0xa25f6790, 33 },
    { 0xa1df6790, 95 },
    { 0, 0 },
    { 0x23af6793, 76 },
    { 0, 0 },
    { 0x298f6795, 23 },
    { 0, 0 },
    { 0xf8ff6797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65df679f, 90 },
    { 0x13df679f, 34 },
    { 0x6c8f67a1, 34 },
    { 0x317f67a2, 23 },
    { 0, 0 },
    { 0x4e2f67a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7f67a7, 17 },
    { 0x7b0f67a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6f67ac, 90 },
    { 0x519f67ad, 90 },
    { 0x2b5f67ae, 61 },
    { 0x15af67af, 100 },
    { 0x357f67af, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf67b8, 43 },
    { 0xdb5f67b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff67c1, 61 },
    { 0, 0 },
    { 0xcf1f67c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5f67c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf67ca, 34 },
    { 0x25df67cb, 61 },
    { 0xb22f67cc, 90 },
    { 0xcdaf67cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f67d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf67d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f67da, 61 },
    { 0x704f67db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bf67de, 40 },
    { 0xf87f67df, 55 },
    { 0xedf67de, 43 },
    { 0x80af67e0, 83 },
    { 0x3ebf67e1, 61 },
    { 0, 0 },
    { 0xf40f67e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1f67e8, 83 },
    { 0, 0 },
    { 0xf00f67ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0f67ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f67f1, 76 },
    { 0x36bf67f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649f67f6, 93 },
    { 0x5c0f67f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef67fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f6800, 48 },
    { 0, 0 },
    { 0x566f6802, 73 },
    { 0x744f6803, 33 },
    { 0x784f6803, 90 },
    { 0x82cf6804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964f680a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x525f680d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f6814, 95 },
    { 0, 0 },
    { 0x777f6816, 61 },
    { 0xbc8f6816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf6826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f682b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5f682f, 77 },
    { 0, 0 },
    { 0x533f6831, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaef683f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ef6848, 76 },
    { 0x13bf6849, 90 },
    { 0x9def6849, 55 },
    { 0xc5f684b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccf684e, 55 },
    { 0x320f684f, 90 },
    { 0x95bf684f, 94 },
    { 0, 0 },
    { 0x239f6852, 43 },
    { 0x1c0f6852, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6f685d, 98 },
    { 0, 0 },
    { 0xaf5f685f, 88 },
    { 0x170f6860, 76 },
    { 0xdccf6861, 46 },
    { 0xd45f6862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f6866, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f686e, 90 },
    { 0x162f686f, 34 },
    { 0, 0 },
    { 0xafef6871, 23 },
    { 0, 0 },
    { 0x725f6873, 23 },
    { 0, 0 },
    { 0xfcff6875, 76 },
    { 0x704f6875, 99 },
    { 0, 0 },
    { 0x6c7f6878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bf687f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1f688b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedf688e, 61 },
    { 0, 0 },
    { 0xaa2f6890, 98 },
    { 0, 0 },
    { 0xc55f6892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf6896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f689b, 85 },
    { 0xd0cf689c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2f68a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f68a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f68a7, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f68ab, 33 },
    { 0x1b8f68ac, 34 },
    { 0x99ef68ac, 98 },
    { 0x4d5f68ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5f68b1, 55 },
    { 0xb03f68b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189f68b6, 76 },
    { 0x190f68b7, 66 },
    { 0, 0 },
    { 0x246f68b9, 52 },
    { 0x617f68ba, 93 },
    { 0, 0 },
    { 0xb97f68bc, 66 },
    { 0x8a5f68bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff68c9, 33 },
    { 0x1a4f68ca, 87 },
    { 0x7abf68cb, 88 },
    { 0, 0 },
    { 0x4d1f68cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef68db, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5f68de, 93 },
    { 0xbf9f68de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f68e3, 98 },
    { 0x960f68e4, 23 },
    { 0xccf68e3, 33 },
    { 0x27bf68e6, 23 },
    { 0xaccf68e6, 61 },
    { 0x213f68e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf68ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fff68ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ff68f3, 23 },
    { 0x6bdf68f4, 33 },
    { 0x43af68f4, 96 },
    { 0, 0 },
    { 0x44af68f7, 33 },
    { 0xd77f68f7, 90 },
    { 0xe48f68f7, 76 },
    { 0xa69f68fa, 55 },
    { 0x19df68fb, 61 },
    { 0x18bf68f8, 90 },
    { 0xdf7f68fd, 90 },
    { 0x964f68fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f6901, 61 },
    { 0, 0 },
    { 0x4fef6903, 33 },
    { 0x90ef6904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f6908, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df690f, 61 },
    { 0x658f6910, 43 },
    { 0x424f690f, 91 },
    { 0xef9f6912, 61 },
    { 0xf1ef6913, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5f6916, 76 },
    { 0xdddf6916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287f691c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15f691f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bf6922, 66 },
    { 0x4cbf6923, 67 },
    { 0, 0 },
    { 0xc0f6925, 90 },
    { 0x428f6926, 87 },
    { 0x31df6927, 77 },
    { 0xee5f6928, 34 },
    { 0, 0 },
    { 0x8c2f692a, 55 },
    { 0xb29f692b, 85 },
    { 0x8b0f692a, 33 },
    { 0xcbff692d, 90 },
    { 0x627f692d, 98 },
    { 0xbabf692d, 90 },
    { 0xe39f692e, 61 },
    { 0x62df692e, 99 },
    { 0xe4f6932, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f6937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef693d, 95 },
    { 0, 0 },
    { 0xee0f693f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaf6942, 34 },
    { 0, 0 },
    { 0x77bf6944, 90 },
    { 0x744f6945, 33 },
    { 0xdbf6946, 33 },
    { 0x2c8f6944, 61 },
    { 0xbb2f6948, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942f694d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x842f6950, 77 },
    { 0xd85f6951, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f6955, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909f6959, 95 },
    { 0x38df6959, 61 },
    { 0x9d2f6959, 98 },
    { 0x9f0f695c, 87 },
    { 0x4fef695c, 87 },
    { 0xc83f695d, 23 },
    { 0x7b8f695f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f6969, 55 },
    { 0xcdff6969, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27f696e, 33 },
    { 0xa9f696e, 34 },
    { 0xca0f6970, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108f697e, 55 },
    { 0, 0 },
    { 0xde4f6980, 95 },
    { 0, 0 },
    { 0xee2f6982, 90 },
    { 0xc6f6983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f698d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2f6990, 33 },
    { 0xa34f6990, 98 },
    { 0xd8f6992, 52 },
    { 0, 0 },
    { 0x79ef6994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f699a, 61 },
    { 0xbe6f699a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8df699e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f69a2, 98 },
    { 0x569f69a3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fef69a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f69aa, 95 },
    { 0x6f69ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f69b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39f69bb, 90 },
    { 0x2e0f69bc, 88 },
    { 0, 0 },
    { 0xddbf69be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39f69c2, 43 },
    { 0x57ef69c2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0f69c8, 90 },
    { 0x507f69c9, 93 },
    { 0xbaf69ca, 23 },
    { 0, 0 },
    { 0x9f1f69cc, 46 },
    { 0, 0 },
    { 0xd06f69ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50f69d2, 38 },
    { 0x20bf69d2, 33 },
    { 0x94f69d4, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff69dc, 98 },
    { 0x973f69dc, 97 },
    { 0, 0 },
    { 0xd65f69df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77df69e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f69e8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33f69f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0f69f7, 90 },
    { 0xf7cf69f8, 76 },
    { 0xa80f69f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606f6a00, 33 },
    { 0, 0 },
    { 0xc50f6a02, 33 },
    { 0, 0 },
    { 0x4cef6a04, 95 },
    { 0x725f6a05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15f6a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58af6a0f, 80 },
    { 0xadcf6a10, 66 },
    { 0x7daf6a10, 33 },
    { 0, 0 },
    { 0x2def6a13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9f6a18, 90 },
    { 0, 0 },
    { 0xcd6f6a1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f6a1d, 91 },
    { 0xeddf6a1d, 61 },
    { 0xb0cf6a1e, 100 },
    { 0xebdf6a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f6a2a, 46 },
    { 0x759f6a2b, 23 },
    { 0x9cf6a2c, 76 },
    { 0xab7f6a2d, 95 },
    { 0xe85f6a2a, 33 },
    { 0x2ebf6a2d, 95 },
    { 0, 0 },
    { 0x8e8f6a31, 90 },
    { 0x3e4f6a31, 40 },
    { 0x77ff6a31, 90 },
    { 0x38bf6a34, 98 },
    { 0xf45f6a34, 34 },
    { 0x10cf6a35, 23 },
    { 0, 0 },
    { 0x814f6a38, 33 },
    { 0xed5f6a39, 84 },
    { 0xd3f6a3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49af6a41, 36 },
    { 0, 0 },
    { 0xfd8f6a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f6a50, 100 },
    { 0xdd5f6a50, 61 },
    { 0x2f6f6a50, 93 },
    { 0xc04f6a53, 76 },
    { 0x56ef6a54, 43 },
    { 0x185f6a53, 33 },
    { 0, 0 },
    { 0x9c1f6a57, 90 },
    { 0xcdcf6a58, 76 },
    { 0, 0 },
    { 0x6d5f6a5a, 61 },
    { 0x9ecf6a5b, 100 },
    { 0, 0 },
    { 0xa0df6a5d, 46 },
    { 0, 0 },
    { 0xfaef6a5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f6a63, 68 },
    { 0x5d5f6a64, 98 },
    { 0x429f6a65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28f6a6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f6a73, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ff6a7f, 68 },
    { 0, 0 },
    { 0x4a7f6a81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75f6a89, 95 },
    { 0xd47f6a8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x801f6a8d, 90 },
    { 0x490f6a8e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xddef6a91, 98 },
    { 0, 0 },
    { 0xbf2f6a93, 53 },
    { 0x827f6a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17df6a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8f6a9b, 95 },
    { 0xfc9f6a9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245f6aa2, 33 },
    { 0x7fcf6aa2, 23 },
    { 0x4aaf6aa2, 90 },
    { 0xa4cf6aa4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8f6aac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaf6aaf, 52 },
    { 0xf92f6ab0, 95 },
    { 0xd5cf6aaf, 100 },
    { 0x64df6ab2, 33 },
    { 0x541f6ab2, 81 },
    { 0xdd8f6ab1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8f6ab8, 52 },
    { 0, 0 },
    { 0x2aff6aba, 83 },
    { 0x3e7f6aba, 90 },
    { 0x33bf6abb, 84 },
    { 0x1e1f6abd, 90 },
    { 0x37af6abe, 33 },
    { 0, 0 },
    { 0x491f6ac0, 76 },
    { 0xfbef6ac0, 33 },
    { 0, 0 },
    { 0x891f6ac3, 100 },
    { 0xc2f6ac4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ff6ac7, 90 },
    { 0x487f6ac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f6acf, 85 },
    { 0xa39f6ad0, 34 },
    { 0x493f6ad1, 44 },
    { 0, 0 },
    { 0xd5bf6ad3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46f6ad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f6adb, 55 },
    { 0, 0 },
    { 0x3b3f6add, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f6ae2, 98 },
    { 0xf3ef6ae3, 78 },
    { 0xc16f6ae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf6ae7, 52 },
    { 0xd79f6ae8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707f6aee, 98 },
    { 0, 0 },
    { 0x859f6af0, 98 },
    { 0, 0 },
    { 0x852f6af2, 55 },
    { 0x761f6af2, 95 },
    { 0x63ef6af4, 88 },
    { 0x208f6af5, 61 },
    { 0x7d2f6af4, 61 },
    { 0, 0 },
    { 0x63ff6af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cf6afe, 98 },
    { 0, 0 },
    { 0x254f6b00, 68 },
    { 0x36bf6b00, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af6b07, 34 },
    { 0, 0 },
    { 0x6c1f6b09, 93 },
    { 0xb4af6b0a, 93 },
    { 0x580f6b0b, 95 },
    { 0xeb5f6b0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164f6b14, 76 },
    { 0x2f8f6b14, 95 },
    { 0, 0 },
    { 0x92cf6b17, 90 },
    { 0x799f6b17, 33 },
    { 0xc1cf6b18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4f6b1d, 100 },
    { 0, 0 },
    { 0x463f6b1f, 93 },
    { 0, 0 },
    { 0x586f6b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df6b24, 46 },
    { 0xf79f6b25, 43 },
    { 0, 0 },
    { 0xac7f6b27, 36 },
    { 0x593f6b28, 94 },
    { 0, 0 },
    { 0x3af6b2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9f6b2e, 84 },
    { 0x807f6b2f, 23 },
    { 0, 0 },
    { 0x851f6b31, 33 },
    { 0x1fef6b32, 84 },
    { 0x462f6b31, 93 },
    { 0x984f6b31, 33 },
    { 0x6dbf6b35, 98 },
    { 0xff9f6b34, 90 },
    { 0x5e9f6b37, 95 },
    { 0, 0 },
    { 0x533f6b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6b40, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f6b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929f6b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5f6b52, 34 },
    { 0xd87f6b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf6b57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f6b5c, 61 },
    { 0x733f6b5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98f6b61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600f6b66, 98 },
    { 0x540f6b67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596f6b6c, 66 },
    { 0x548f6b6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61af6b74, 33 },
    { 0x24f6b74, 61 },
    { 0xeb1f6b75, 61 },
    { 0, 0 },
    { 0xe82f6b78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9f6b80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6b84, 52 },
    { 0x3a9f6b84, 33 },
    { 0x3a8f6b86, 93 },
    { 0x3c6f6b87, 76 },
    { 0, 0 },
    { 0x54af6b89, 95 },
    { 0xfdaf6b8a, 90 },
    { 0, 0 },
    { 0x977f6b8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f6b90, 55 },
    { 0, 0 },
    { 0x367f6b92, 55 },
    { 0xd55f6b93, 90 },
    { 0xcbcf6b94, 23 },
    { 0xa3f6b94, 17 },
    { 0, 0 },
    { 0x13df6b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94f6b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff6b9f, 37 },
    { 0x694f6ba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecf6ba3, 90 },
    { 0, 0 },
    { 0xa9cf6ba5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96f6bab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6f6bb0, 81 },
    { 0x675f6bb0, 23 },
    { 0xea8f6bb2, 52 },
    { 0, 0 },
    { 0xde4f6bb4, 95 },
    { 0, 0 },
    { 0x3f4f6bb6, 52 },
    { 0x207f6bb7, 33 },
    { 0x65bf6bb7, 88 },
    { 0x726f6bb7, 69 },
    { 0xfa5f6bb7, 23 },
    { 0, 0 },
    { 0x46cf6bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf6bc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x136f6bc3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x985f6bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f6bca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801f6bd0, 40 },
    { 0, 0 },
    { 0xd54f6bd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f6bd7, 85 },
    { 0x6d0f6bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25af6bde, 43 },
    { 0xa98f6bdf, 33 },
    { 0x5a6f6bdf, 61 },
    { 0x2b5f6be1, 95 },
    { 0x9df6be2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f6be8, 46 },
    { 0x8f6be8, 61 },
    { 0x562f6bea, 87 },
    { 0xf0cf6be9, 90 },
    { 0x8fbf6bec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41f6bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf6bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f6bfd, 96 },
    { 0x4b1f6bfd, 90 },
    { 0xb46f6bff, 84 },
    { 0xea9f6c00, 90 },
    { 0, 0 },
    { 0xc68f6c02, 98 },
    { 0, 0 },
    { 0xaaaf6c04, 65 },
    { 0, 0 },
    { 0x682f6c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ef6c0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155f6c14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef6c18, 61 },
    { 0xa00f6c19, 98 },
    { 0xa68f6c1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf6c1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df6c21, 46 },
    { 0xa99f6c22, 61 },
    { 0, 0 },
    { 0xd16f6c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def6c2d, 33 },
    { 0xc50f6c2e, 65 },
    { 0x44bf6c2f, 23 },
    { 0x4e2f6c2d, 33 },
    { 0xa9bf6c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6f6c35, 99 },
    { 0, 0 },
    { 0x578f6c37, 34 },
    { 0x683f6c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f6c3d, 81 },
    { 0, 0 },
    { 0x71cf6c3f, 95 },
    { 0xc0df6c40, 90 },
    { 0x38f6c3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2f6c46, 61 },
    { 0, 0 },
    { 0xfdff6c48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f6c4c, 55 },
    { 0xa5ef6c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f6c50, 61 },
    { 0, 0 },
    { 0x947f6c52, 61 },
    { 0xc85f6c53, 52 },
    { 0x76ff6c53, 98 },
    { 0, 0 },
    { 0x3dcf6c56, 61 },
    { 0xde6f6c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf6c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f6c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6f6c65, 97 },
    { 0x8ccf6c66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f6c69, 76 },
    { 0, 0 },
    { 0xff8f6c6b, 43 },
    { 0x49bf6c6c, 23 },
    { 0x592f6c6d, 23 },
    { 0x648f6c6e, 43 },
    { 0x4bf6c6f, 93 },
    { 0x9b8f6c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34f6c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75f6c7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f6c7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcf6c81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaff6c85, 99 },
    { 0x96f6c86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cf6c8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400f6c91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4f6c94, 33 },
    { 0x743f6c95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ef6ca0, 61 },
    { 0xd7f6ca1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f6ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf6ca8, 91 },
    { 0xb84f6ca9, 98 },
    { 0x67af6ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdf6cb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff6cb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f6cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6f6cc2, 95 },
    { 0x771f6cc3, 76 },
    { 0x39df6cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff6ccc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf6cd1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6f6cd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacef6cd8, 55 },
    { 0x977f6cd8, 98 },
    { 0, 0 },
    { 0x8ff6cdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf6ce0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f6ce7, 90 },
    { 0xff2f6ce8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6ceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971f6cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6cf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662f6cf8, 95 },
    { 0, 0 },
    { 0xbfdf6cfa, 34 },
    { 0, 0 },
    { 0xaaef6cfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529f6d0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff6d13, 93 },
    { 0xc0af6d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f6d19, 66 },
    { 0, 0 },
    { 0xf1f6d1b, 34 },
    { 0x1f1f6d1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faf6d23, 90 },
    { 0xbcbf6d23, 69 },
    { 0x4d2f6d25, 33 },
    { 0x70bf6d26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f6d2b, 76 },
    { 0x9f2f6d2c, 85 },
    { 0x96bf6d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26f6d32, 23 },
    { 0, 0 },
    { 0xe8bf6d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bf6d38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475f6d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361f6d43, 33 },
    { 0x5bbf6d44, 95 },
    { 0xa1bf6d45, 84 },
    { 0x824f6d46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x552f6d49, 52 },
    { 0, 0 },
    { 0xa52f6d4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbf6d4e, 99 },
    { 0xd34f6d4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f6d52, 76 },
    { 0x16ff6d53, 61 },
    { 0, 0 },
    { 0x2eef6d55, 91 },
    { 0x64cf6d56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f6d5a, 91 },
    { 0, 0 },
    { 0x7ecf6d5c, 33 },
    { 0xcf2f6d5c, 90 },
    { 0xf2cf6d5e, 34 },
    { 0, 0 },
    { 0x5a3f6d60, 76 },
    { 0, 0 },
    { 0xa11f6d62, 41 },
    { 0x3c3f6d63, 98 },
    { 0xdabf6d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1f6d6d, 66 },
    { 0xb87f6d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788f6d73, 94 },
    { 0x6a4f6d74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ff6d78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3f6d7e, 33 },
    { 0, 0 },
    { 0xc80f6d80, 100 },
    { 0, 0 },
    { 0x6c6f6d82, 95 },
    { 0, 0 },
    { 0xb7df6d84, 90 },
    { 0xd6af6d84, 76 },
    { 0x79f6d86, 61 },
    { 0xa36f6d87, 23 },
    { 0x752f6d88, 33 },
    { 0x7d2f6d88, 61 },
    { 0, 0 },
    { 0x516f6d8b, 100 },
    { 0, 0 },
    { 0x31bf6d8d, 61 },
    { 0, 0 },
    { 0x521f6d8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ff6d93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf6d9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f6da2, 61 },
    { 0x2caf6da2, 61 },
    { 0x4c4f6da4, 100 },
    { 0x424f6da5, 33 },
    { 0x628f6da4, 95 },
    { 0x1c3f6da7, 78 },
    { 0, 0 },
    { 0xfbaf6da9, 38 },
    { 0xd07f6daa, 55 },
    { 0, 0 },
    { 0xbf3f6dac, 78 },
    { 0xe16f6dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bf6db2, 64 },
    { 0x210f6db2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f6db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef6dc2, 90 },
    { 0xcccf6dc3, 90 },
    { 0x9caf6dc4, 99 },
    { 0x998f6dc5, 33 },
    { 0xb58f6dc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f6dc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ff6dcd, 84 },
    { 0x12bf6dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cf6dd6, 33 },
    { 0xe27f6dd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5f6ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef6dea, 98 },
    { 0x9dbf6dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f6def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf6df3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ef6df8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf6dff, 98 },
    { 0x4dff6e00, 81 },
    { 0xacf6dff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f6e0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6e10, 95 },
    { 0, 0 },
    { 0xe1bf6e12, 81 },
    { 0xee9f6e13, 93 },
    { 0xe62f6e14, 77 },
    { 0xa8ff6e15, 52 },
    { 0x2bbf6e15, 43 },
    { 0, 0 },
    { 0x3a1f6e18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32f6e21, 61 },
    { 0, 0 },
    { 0xdf0f6e23, 90 },
    { 0, 0 },
    { 0x707f6e25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x66af6e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296f6e2d, 33 },
    { 0xc99f6e2d, 23 },
    { 0, 0 },
    { 0x415f6e30, 46 },
    { 0x371f6e31, 88 },
    { 0, 0 },
    { 0xa4df6e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f6e37, 55 },
    { 0x243f6e37, 84 },
    { 0x295f6e38, 100 },
    { 0x5e6f6e3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f6e3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1daf6e41, 91 },
    { 0x972f6e42, 78 },
    { 0x5e5f6e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df6e46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ff6e4a, 95 },
    { 0xbf6e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5f6e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56f6e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16f6e58, 55 },
    { 0xbbaf6e59, 95 },
    { 0x353f6e5a, 91 },
    { 0x946f6e5a, 95 },
    { 0xf4ef6e58, 33 },
    { 0x790f6e5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f6e60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561f6e67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f6e6b, 90 },
    { 0x771f6e6c, 85 },
    { 0x500f6e6c, 75 },
    { 0, 0 },
    { 0x5a1f6e6f, 76 },
    { 0xf3bf6e70, 98 },
    { 0x869f6e70, 61 },
    { 0x4a5f6e72, 90 },
    { 0x3ecf6e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cf6e77, 65 },
    { 0x86f6e78, 76 },
    { 0x7aff6e79, 33 },
    { 0x521f6e79, 33 },
    { 0x391f6e7b, 90 },
    { 0xcfdf6e79, 43 },
    { 0, 0 },
    { 0xa99f6e7e, 98 },
    { 0, 0 },
    { 0x43bf6e80, 75 },
    { 0, 0 },
    { 0x953f6e82, 33 },
    { 0xb80f6e83, 52 },
    { 0, 0 },
    { 0x61bf6e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71f6e88, 68 },
    { 0xff0f6e89, 76 },
    { 0x97af6e88, 43 },
    { 0xf81f6e8b, 33 },
    { 0x6a4f6e8c, 23 },
    { 0x94f6e88, 87 },
    { 0x317f6e8a, 83 },
    { 0xe5ff6e8f, 55 },
    { 0xc56f6e90, 77 },
    { 0xe6ff6e91, 95 },
    { 0x8f1f6e91, 99 },
    { 0x669f6e93, 46 },
    { 0xf5af6e94, 90 },
    { 0, 0 },
    { 0x82af6e96, 23 },
    { 0x99cf6e97, 90 },
    { 0x550f6e98, 68 },
    { 0xcebf6e97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ff6e9c, 43 },
    { 0xc27f6e9d, 34 },
    { 0xd42f6e9d, 61 },
    { 0x6edf6e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f6ea9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf6ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df6ebc, 98 },
    { 0x586f6ebd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cf6ec1, 76 },
    { 0, 0 },
    { 0x15bf6ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff6ec7, 88 },
    { 0x2abf6ec8, 65 },
    { 0x3dff6ec8, 33 },
    { 0x23f6eca, 61 },
    { 0, 0 },
    { 0x197f6ecc, 75 },
    { 0xdf6f6ecc, 33 },
    { 0xde3f6ece, 34 },
    { 0xac8f6ecf, 93 },
    { 0x929f6ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f6ed5, 61 },
    { 0xda3f6ed6, 33 },
    { 0xfdcf6ed7, 96 },
    { 0, 0 },
    { 0x692f6ed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6f6edd, 98 },
    { 0x1f7f6edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07f6ee2, 23 },
    { 0xf40f6ee3, 78 },
    { 0xae6f6ee3, 33 },
    { 0xf4cf6ee4, 95 },
    { 0xd6df6ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f6eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f6ef1, 85 },
    { 0x731f6ef2, 61 },
    { 0x32ff6ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af6efa, 98 },
    { 0xbfef6efb, 77 },
    { 0xe6ff6efb, 66 },
    { 0, 0 },
    { 0xe33f6efe, 66 },
    { 0x2a8f6eff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df6f03, 95 },
    { 0x7f2f6f04, 78 },
    { 0xd74f6f03, 61 },
    { 0, 0 },
    { 0x7f6f6f07, 88 },
    { 0, 0 },
    { 0x4bef6f09, 43 },
    { 0x197f6f0a, 34 },
    { 0x67cf6f0a, 66 },
    { 0x11ef6f0c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f6f0f, 90 },
    { 0, 0 },
    { 0x64ef6f11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f6f18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fef6f1e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6f6f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff6f2c, 33 },
    { 0xb37f6f2d, 95 },
    { 0xbb9f6f2e, 23 },
    { 0x5faf6f2f, 67 },
    { 0xeddf6f30, 93 },
    { 0xb38f6f30, 23 },
    { 0x9cdf6f31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f6f38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f6f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f6f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff6f48, 78 },
    { 0, 0 },
    { 0x392f6f4a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bf6f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f6f57, 33 },
    { 0xf3ff6f57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f6f5d, 23 },
    { 0x7a5f6f5e, 23 },
    { 0xddef6f5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ff6f65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f6f6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938f6f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf6f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7f6f77, 95 },
    { 0x69af6f77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f6f7c, 84 },
    { 0x1b2f6f7c, 61 },
    { 0xb72f6f7e, 98 },
    { 0, 0 },
    { 0xa0af6f80, 100 },
    { 0x53df6f81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1f6f87, 98 },
    { 0x582f6f88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9f6f8c, 90 },
    { 0, 0 },
    { 0xf1f6f8e, 33 },
    { 0xf87f6f8f, 93 },
    { 0x8e7f6f90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f6f93, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f6f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5f6f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f6fa4, 33 },
    { 0, 0 },
    { 0x2d6f6fa6, 100 },
    { 0, 0 },
    { 0xd2df6fa8, 61 },
    { 0x415f6fa9, 43 },
    { 0xc65f6faa, 98 },
    { 0xa57f6fab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1f6faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3f6fb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ff6fb9, 90 },
    { 0, 0 },
    { 0xe5f6fbb, 90 },
    { 0xce9f6fbb, 46 },
    { 0x425f6fbd, 33 },
    { 0x3dff6fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5f6fc1, 90 },
    { 0x102f6fc1, 55 },
    { 0x6d7f6fc3, 76 },
    { 0xe4f6fc4, 23 },
    { 0x4b4f6fc5, 61 },
    { 0xeedf6fc3, 100 },
    { 0x18af6fc7, 77 },
    { 0x381f6fc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f6fcd, 99 },
    { 0x170f6fce, 23 },
    { 0, 0 },
    { 0x28bf6fd0, 61 },
    { 0x214f6fd1, 33 },
    { 0x1b7f6fd2, 78 },
    { 0, 0 },
    { 0x429f6fd4, 52 },
    { 0, 0 },
    { 0x788f6fd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f6fdc, 91 },
    { 0x23f6fdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f6fe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f6fe7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f6ff2, 55 },
    { 0x8aaf6ff2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50af6ff6, 98 },
    { 0x78af6ff7, 76 },
    { 0x78df6ff8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f6fff, 46 },
    { 0x66af7000, 61 },
    { 0, 0 },
    { 0xb30f7002, 43 },
    { 0x6b7f7002, 98 },
    { 0xbc1f7004, 69 },
    { 0xdcdf7004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f7009, 95 },
    { 0x227f700a, 61 },
    { 0xab9f7009, 55 },
    { 0xb74f700c, 61 },
    { 0, 0 },
    { 0x1def700e, 23 },
    { 0xa19f700f, 33 },
    { 0x7dbf700f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf7016, 33 },
    { 0xd75f7016, 90 },
    { 0x737f7016, 61 },
    { 0xc8df7019, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f701d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00f7020, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f7024, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f702a, 67 },
    { 0x9daf702b, 95 },
    { 0, 0 },
    { 0xb7af702d, 43 },
    { 0xb3f702e, 61 },
    { 0xf37f702f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68f7032, 69 },
    { 0, 0 },
    { 0xb97f7034, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549f703a, 90 },
    { 0x324f703b, 77 },
    { 0, 0 },
    { 0xddef703d, 55 },
    { 0, 0 },
    { 0xc27f703f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff7045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ef7048, 43 },
    { 0, 0 },
    { 0x6a3f704a, 75 },
    { 0xc4ff704b, 98 },
    { 0x92ff704b, 23 },
    { 0xa57f704d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f7051, 76 },
    { 0, 0 },
    { 0xeaef7053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74f7057, 98 },
    { 0, 0 },
    { 0xdf6f7059, 90 },
    { 0, 0 },
    { 0xf74f705b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f705f, 23 },
    { 0, 0 },
    { 0x66f7061, 98 },
    { 0, 0 },
    { 0xaa0f7063, 90 },
    { 0, 0 },
    { 0x894f7065, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f7068, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff706c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03f7073, 67 },
    { 0xe24f7074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1f7077, 55 },
    { 0, 0 },
    { 0x725f7079, 34 },
    { 0x470f707a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5f7084, 78 },
    { 0xebcf7084, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f708e, 91 },
    { 0, 0 },
    { 0xa40f7090, 87 },
    { 0, 0 },
    { 0xffff7092, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ef7095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef709a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df70a2, 61 },
    { 0x9edf70a3, 90 },
    { 0, 0 },
    { 0x37bf70a5, 98 },
    { 0xeaef70a6, 33 },
    { 0xa71f70a6, 23 },
    { 0, 0 },
    { 0x655f70a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12f70ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf70b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedef70b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff70bd, 76 },
    { 0x1eef70be, 33 },
    { 0x599f70be, 61 },
    { 0, 0 },
    { 0x31f70c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64f70c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccf70c9, 23 },
    { 0, 0 },
    { 0x8c9f70cb, 87 },
    { 0xf6bf70cc, 66 },
    { 0x198f70cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797f70d1, 43 },
    { 0x630f70d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff70d9, 88 },
    { 0xdb7f70d9, 90 },
    { 0xfb4f70db, 95 },
    { 0x8dbf70db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538f70e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf70e4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f70e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cf70ec, 61 },
    { 0xb29f70ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f70f0, 90 },
    { 0, 0 },
    { 0x94bf70f2, 66 },
    { 0xb5bf70f3, 76 },
    { 0x12ff70f3, 34 },
    { 0xdeff70f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f70fa, 90 },
    { 0xcf7f70fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46f70ff, 34 },
    { 0xb14f7100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0f7106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f710b, 69 },
    { 0xb16f710c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacf7111, 95 },
    { 0xa69f7112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edf7116, 55 },
    { 0x2edf7117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f711f, 90 },
    { 0xcbf7120, 100 },
    { 0x387f7121, 33 },
    { 0, 0 },
    { 0x7e2f7123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f7127, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37df712a, 67 },
    { 0, 0 },
    { 0xd76f712c, 43 },
    { 0xa34f712c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9f7131, 95 },
    { 0, 0 },
    { 0x795f7133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589f7137, 93 },
    { 0x844f7138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eef7141, 33 },
    { 0x77ff7142, 61 },
    { 0, 0 },
    { 0x5c3f7144, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f714c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0f714f, 23 },
    { 0x37cf714f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf7153, 98 },
    { 0, 0 },
    { 0x186f7155, 66 },
    { 0, 0 },
    { 0x7b5f7157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703f715c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487f7161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f716a, 77 },
    { 0xc11f716a, 83 },
    { 0, 0 },
    { 0x121f716d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7173, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f7179, 33 },
    { 0xfaff717a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ff717e, 95 },
    { 0xa9bf717e, 64 },
    { 0, 0 },
    { 0x584f7181, 95 },
    { 0xa46f7182, 88 },
    { 0x258f7183, 75 },
    { 0, 0 },
    { 0x87bf7185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6f7188, 91 },
    { 0x8dff7189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef7191, 90 },
    { 0xa2df7191, 53 },
    { 0xc6cf7193, 61 },
    { 0x707f7193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f719a, 46 },
    { 0x14ff719b, 98 },
    { 0x521f719c, 55 },
    { 0xdb1f719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421f71a1, 61 },
    { 0x589f71a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f71a6, 55 },
    { 0x297f71a6, 90 },
    { 0x1a0f71a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f71ab, 61 },
    { 0, 0 },
    { 0xb81f71ad, 85 },
    { 0xa56f71ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbf71b3, 23 },
    { 0xa03f71b3, 43 },
    { 0xecbf71b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef71ba, 43 },
    { 0x195f71bb, 95 },
    { 0x530f71bc, 58 },
    { 0xb25f71bd, 43 },
    { 0x117f71bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69f71c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf71c6, 34 },
    { 0xea4f71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x973f71ca, 98 },
    { 0xc93f71cb, 33 },
    { 0, 0 },
    { 0x43bf71cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182f71d1, 61 },
    { 0xca5f71d1, 90 },
    { 0x8aaf71d1, 90 },
    { 0x2a8f71d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6f71d7, 78 },
    { 0xeaf71d8, 90 },
    { 0x68bf71d9, 78 },
    { 0xf93f71d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42af71de, 33 },
    { 0x32cf71df, 33 },
    { 0x404f71df, 98 },
    { 0x24ff71e1, 52 },
    { 0x655f71e1, 55 },
    { 0x571f71e0, 33 },
    { 0x307f71e3, 23 },
    { 0x7cbf71e5, 68 },
    { 0x40f71e4, 34 },
    { 0x944f71e7, 93 },
    { 0xfc1f71e8, 55 },
    { 0x405f71e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8f71ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9f71f2, 33 },
    { 0, 0 },
    { 0x335f71f4, 84 },
    { 0, 0 },
    { 0xbcef71f6, 98 },
    { 0xdf8f71f7, 91 },
    { 0, 0 },
    { 0x355f71f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af7201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9af7205, 46 },
    { 0x73ff7206, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5f720a, 98 },
    { 0x127f720a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f720e, 77 },
    { 0x92bf720e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f7215, 90 },
    { 0xa40f7216, 95 },
    { 0xa2ef7216, 61 },
    { 0x828f7216, 87 },
    { 0, 0 },
    { 0x844f721a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ff721e, 90 },
    { 0x99bf721f, 46 },
    { 0xac0f721f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f7223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef7227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caf722d, 83 },
    { 0x114f722e, 61 },
    { 0, 0 },
    { 0xb30f7230, 61 },
    { 0xccff7230, 33 },
    { 0x6eef7232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf7236, 81 },
    { 0x10cf7237, 61 },
    { 0, 0 },
    { 0xfaaf7239, 90 },
    { 0x21cf723a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f723f, 87 },
    { 0xa0bf7240, 61 },
    { 0xfd3f7241, 61 },
    { 0xac1f7241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef7247, 61 },
    { 0xf34f7247, 90 },
    { 0x918f7248, 61 },
    { 0xbd3f724a, 46 },
    { 0xb61f724b, 95 },
    { 0xed6f724c, 23 },
    { 0x5f4f724c, 23 },
    { 0, 0 },
    { 0x6c0f724f, 43 },
    { 0x342f7250, 98 },
    { 0x19df7251, 33 },
    { 0x4bdf7251, 91 },
    { 0xeaff7253, 43 },
    { 0x87f7254, 43 },
    { 0x43af7251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x921f7258, 93 },
    { 0xa2df7258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df725f, 33 },
    { 0, 0 },
    { 0xe3f7261, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x539f7264, 95 },
    { 0, 0 },
    { 0x4a4f7266, 84 },
    { 0x608f7266, 55 },
    { 0xa54f7268, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaf726d, 95 },
    { 0, 0 },
    { 0xe68f726f, 93 },
    { 0x881f7270, 90 },
    { 0x3bef7271, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef7275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1f727a, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6af727e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf7282, 23 },
    { 0, 0 },
    { 0x899f7284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4f7287, 90 },
    { 0, 0 },
    { 0x11cf7289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f728f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f7294, 99 },
    { 0xf06f7295, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf7298, 95 },
    { 0, 0 },
    { 0x343f729a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff729f, 90 },
    { 0, 0 },
    { 0xdbcf72a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f72a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391f72aa, 90 },
    { 0, 0 },
    { 0xe75f72ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcf72b0, 33 },
    { 0x8c7f72b0, 61 },
    { 0, 0 },
    { 0x24ff72b3, 36 },
    { 0xa55f72b3, 66 },
    { 0x55ff72b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f72c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef72c4, 33 },
    { 0, 0 },
    { 0x4b3f72c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x802f72c9, 85 },
    { 0x4e8f72c9, 76 },
    { 0xec3f72cb, 23 },
    { 0xb9df72cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cf72d8, 91 },
    { 0x6e6f72d9, 66 },
    { 0xeef72d9, 98 },
    { 0x8c8f72db, 92 },
    { 0, 0 },
    { 0xc80f72dd, 66 },
    { 0x5a6f72de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff72eb, 100 },
    { 0x6acf72ec, 90 },
    { 0xa4df72ed, 90 },
    { 0x8aff72ed, 90 },
    { 0x11af72ec, 90 },
    { 0x440f72f0, 96 },
    { 0x77f72f1, 93 },
    { 0xd93f72f1, 96 },
    { 0xa3ef72ed, 90 },
    { 0, 0 },
    { 0xa27f72f5, 47 },
    { 0, 0 },
    { 0xe2cf72f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f72fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f72fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x879f7300, 95 },
    { 0, 0 },
    { 0x7c2f7302, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f7306, 23 },
    { 0, 0 },
    { 0xf48f7308, 98 },
    { 0xa5bf7309, 100 },
    { 0xd0cf730a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6af7312, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f7315, 99 },
    { 0x34df7316, 78 },
    { 0, 0 },
    { 0x42bf7318, 23 },
    { 0xe90f7319, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1f731c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ff7322, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f7325, 34 },
    { 0xd0f7325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710f732b, 61 },
    { 0, 0 },
    { 0xd64f732d, 81 },
    { 0x623f732e, 76 },
    { 0x36ff732d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f7333, 23 },
    { 0, 0 },
    { 0x8acf7335, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0f733a, 61 },
    { 0xcd0f733b, 80 },
    { 0, 0 },
    { 0xf86f733d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f7341, 73 },
    { 0, 0 },
    { 0xecdf7343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bf7346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cf7351, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf7355, 91 },
    { 0x5aef7356, 99 },
    { 0xa6bf7357, 95 },
    { 0, 0 },
    { 0xe97f7359, 98 },
    { 0x67bf735a, 88 },
    { 0x7c7f7359, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef7365, 33 },
    { 0xb28f7365, 98 },
    { 0xe0af7367, 23 },
    { 0xffcf7366, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f736b, 85 },
    { 0x117f736c, 80 },
    { 0xaf736d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48f7370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ef7375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99f737d, 23 },
    { 0, 0 },
    { 0x40af737f, 43 },
    { 0xf91f7380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982f7383, 90 },
    { 0xa6cf7384, 43 },
    { 0x3c4f7383, 98 },
    { 0x9a0f7383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620f738c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3f738f, 61 },
    { 0, 0 },
    { 0x9a6f7391, 46 },
    { 0, 0 },
    { 0x259f7393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18f7399, 90 },
    { 0, 0 },
    { 0xe3df739b, 93 },
    { 0x8ebf739c, 55 },
    { 0xc98f739d, 90 },
    { 0x407f739d, 17 },
    { 0xf78f739f, 55 },
    { 0x8cff73a0, 85 },
    { 0, 0 },
    { 0x735f73a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ef73a8, 52 },
    { 0x4c8f73a9, 95 },
    { 0xff4f73a9, 61 },
    { 0xba7f73aa, 46 },
    { 0x7c2f73ac, 98 },
    { 0x9b8f73aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf73b1, 69 },
    { 0x1d9f73b2, 90 },
    { 0xd7ef73b3, 90 },
    { 0xeabf73b3, 61 },
    { 0, 0 },
    { 0x350f73b6, 61 },
    { 0, 0 },
    { 0xee9f73b8, 95 },
    { 0x7b9f73b8, 33 },
    { 0, 0 },
    { 0x4caf73bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f73be, 33 },
    { 0x516f73bf, 100 },
    { 0xed7f73c0, 38 },
    { 0xaa4f73bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef73c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f73d3, 23 },
    { 0, 0 },
    { 0x1faf73d5, 91 },
    { 0, 0 },
    { 0x675f73d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5f73e0, 100 },
    { 0x52f73e0, 76 },
    { 0xf2f73e2, 100 },
    { 0x58df73e2, 99 },
    { 0x726f73e3, 17 },
    { 0xe08f73e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ff73ec, 95 },
    { 0x974f73ec, 76 },
    { 0x4ebf73ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0f73f2, 23 },
    { 0x767f73f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f73f6, 75 },
    { 0xfb5f73f7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4f73ff, 81 },
    { 0, 0 },
    { 0x1bdf7401, 100 },
    { 0, 0 },
    { 0xef6f7403, 61 },
    { 0xb7bf7403, 23 },
    { 0xb1ef7405, 55 },
    { 0x2ccf7406, 33 },
    { 0x8ddf7406, 23 },
    { 0x963f7408, 90 },
    { 0x6b8f7409, 43 },
    { 0x1e1f7409, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f740d, 80 },
    { 0xd64f740e, 88 },
    { 0x896f740f, 95 },
    { 0xee7f740d, 55 },
    { 0x711f7411, 94 },
    { 0x638f7411, 75 },
    { 0xe6cf7413, 61 },
    { 0x89f7413, 81 },
    { 0x420f7415, 76 },
    { 0xc66f7415, 33 },
    { 0x7e3f7414, 33 },
    { 0xa3af7414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceef741b, 23 },
    { 0xfb7f741b, 100 },
    { 0, 0 },
    { 0x7a3f741e, 95 },
    { 0, 0 },
    { 0xb36f7420, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0f7425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f7429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ff742c, 9 },
    { 0x82bf742c, 65 },
    { 0, 0 },
    { 0x479f742f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82f7439, 90 },
    { 0, 0 },
    { 0x335f743b, 33 },
    { 0, 0 },
    { 0x269f743d, 33 },
    { 0xe1bf743d, 17 },
    { 0xe4bf743e, 87 },
    { 0x6cef7440, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6f7444, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499f744c, 55 },
    { 0x2a1f744c, 61 },
    { 0, 0 },
    { 0x6d4f744f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599f7455, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf745d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaf7463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f7468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf746b, 33 },
    { 0xaa4f746c, 78 },
    { 0x27ff746d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef7472, 78 },
    { 0xc92f7472, 95 },
    { 0x58ff7474, 34 },
    { 0x51df7475, 93 },
    { 0x73af7476, 66 },
    { 0x592f7477, 98 },
    { 0, 0 },
    { 0x7ff7479, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f747c, 15 },
    { 0xd8ff747d, 52 },
    { 0x92f747c, 61 },
    { 0, 0 },
    { 0x67af7480, 55 },
    { 0x5f6f7480, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf7486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf748c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf7492, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af7496, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f7499, 88 },
    { 0x413f749a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af74a0, 34 },
    { 0, 0 },
    { 0xadef74a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef74a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef74a8, 91 },
    { 0x32bf74a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137f74ad, 68 },
    { 0, 0 },
    { 0xc7af74af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935f74b5, 90 },
    { 0, 0 },
    { 0x39ff74b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688f74bc, 55 },
    { 0x3e9f74bd, 90 },
    { 0, 0 },
    { 0x651f74bf, 81 },
    { 0x2f8f74bf, 52 },
    { 0x73df74c0, 34 },
    { 0xa1ef74c2, 56 },
    { 0x166f74c3, 43 },
    { 0xe22f74c0, 100 },
    { 0x768f74c5, 98 },
    { 0, 0 },
    { 0x741f74c7, 78 },
    { 0, 0 },
    { 0x7b2f74c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f74cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x875f74d2, 90 },
    { 0x8f8f74d2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121f74d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0df74db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f74de, 97 },
    { 0xc59f74de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f74e3, 81 },
    { 0xfaef74e3, 55 },
    { 0xf19f74e5, 66 },
    { 0, 0 },
    { 0x9cf74e7, 61 },
    { 0, 0 },
    { 0x82bf74e9, 90 },
    { 0, 0 },
    { 0x1fdf74eb, 33 },
    { 0x8e5f74eb, 66 },
    { 0, 0 },
    { 0x10bf74ee, 43 },
    { 0x109f74ef, 53 },
    { 0x996f74f0, 90 },
    { 0xa4af74f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef74f6, 23 },
    { 0x531f74f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f74fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf7500, 95 },
    { 0xbb4f7501, 90 },
    { 0, 0 },
    { 0x400f7503, 46 },
    { 0xa66f7504, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ff750c, 90 },
    { 0x87af750d, 34 },
    { 0x25bf750c, 90 },
    { 0xd41f750f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ef7515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x671f7518, 84 },
    { 0, 0 },
    { 0x56cf751a, 90 },
    { 0x181f751a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9f7523, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf752a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x585f752d, 95 },
    { 0, 0 },
    { 0x7f9f752f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f7532, 99 },
    { 0x154f7533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f753a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347f7542, 33 },
    { 0x5e9f7542, 84 },
    { 0x954f7542, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3baf7548, 46 },
    { 0, 0 },
    { 0x871f754a, 90 },
    { 0x497f754b, 93 },
    { 0, 0 },
    { 0xfa7f754d, 98 },
    { 0, 0 },
    { 0x581f754f, 80 },
    { 0xf01f7550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5f7556, 83 },
    { 0x842f7556, 75 },
    { 0xd27f7556, 33 },
    { 0xf3ff7559, 33 },
    { 0xe3df7559, 95 },
    { 0, 0 },
    { 0x93ff755c, 17 },
    { 0x953f755c, 68 },
    { 0x585f755e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f7563, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf7568, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7572, 33 },
    { 0x1c2f7573, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f7576, 90 },
    { 0xfa4f7577, 95 },
    { 0x98cf7578, 61 },
    { 0xaf3f7578, 100 },
    { 0, 0 },
    { 0x63bf757b, 93 },
    { 0xd6ef757b, 23 },
    { 0x45f757d, 61 },
    { 0x50f757e, 46 },
    { 0x766f757f, 34 },
    { 0, 0 },
    { 0xb8df7581, 90 },
    { 0xe5ef7582, 33 },
    { 0x446f7581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689f7588, 95 },
    { 0xc4ff7588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f758c, 33 },
    { 0, 0 },
    { 0xfc5f758e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810f7593, 90 },
    { 0, 0 },
    { 0xa39f7595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f759a, 34 },
    { 0, 0 },
    { 0xd1cf759c, 81 },
    { 0xc9f759d, 61 },
    { 0xb9cf759c, 8 },
    { 0, 0 },
    { 0xe92f75a0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00f75a5, 98 },
    { 0, 0 },
    { 0xf6ef75a7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f75aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf75ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f75b4, 96 },
    { 0xd7ff75b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef75b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218f75c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6f75c7, 45 },
    { 0xa8ff75c8, 95 },
    { 0xbedf75c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f75cc, 98 },
    { 0x9c1f75cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33f75d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f75d4, 98 },
    { 0xa18f75d4, 76 },
    { 0xc47f75d4, 61 },
    { 0xd0cf75d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3f75db, 55 },
    { 0x467f75db, 67 },
    { 0, 0 },
    { 0x4bef75de, 98 },
    { 0xe4bf75df, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bf75e5, 100 },
    { 0x764f75e5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830f75ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ef75ef, 95 },
    { 0x79ef75ef, 61 },
    { 0xc6df75f1, 43 },
    { 0x92af75f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bf75f9, 80 },
    { 0x5f0f75f9, 46 },
    { 0x96f75fb, 55 },
    { 0, 0 },
    { 0x46df75fd, 61 },
    { 0x411f75fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f7602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bf7607, 33 },
    { 0xf14f7607, 23 },
    { 0x9e4f7609, 33 },
    { 0xc28f760a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff7611, 55 },
    { 0, 0 },
    { 0x5df7613, 90 },
    { 0xe9df7614, 46 },
    { 0x2c1f7615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f761d, 99 },
    { 0x157f761e, 48 },
    { 0x5c4f761e, 81 },
    { 0, 0 },
    { 0x1ebf7621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f7625, 52 },
    { 0, 0 },
    { 0xdff7627, 33 },
    { 0x61bf7628, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58f762c, 34 },
    { 0, 0 },
    { 0x486f762e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaf7631, 90 },
    { 0x40f7631, 66 },
    { 0, 0 },
    { 0xffff7634, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bf763a, 95 },
    { 0, 0 },
    { 0x420f763c, 78 },
    { 0x58af763d, 33 },
    { 0, 0 },
    { 0x69f763f, 95 },
    { 0, 0 },
    { 0x6c3f7641, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f7646, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f764d, 23 },
    { 0xc6bf764d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff7654, 95 },
    { 0, 0 },
    { 0x552f7656, 55 },
    { 0xd8cf7656, 61 },
    { 0x882f7656, 44 },
    { 0xe10f7659, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0df765c, 98 },
    { 0xdd5f765c, 90 },
    { 0x367f765e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149f7662, 78 },
    { 0x165f7662, 75 },
    { 0xa5af7664, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef766a, 43 },
    { 0xff7f766a, 98 },
    { 0x2c7f766c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf7670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ef7678, 76 },
    { 0x3b5f7679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f767f, 95 },
    { 0, 0 },
    { 0x549f7681, 98 },
    { 0x1b5f7681, 90 },
    { 0x6f0f7683, 55 },
    { 0, 0 },
    { 0xf14f7685, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f7688, 90 },
    { 0, 0 },
    { 0x5cdf768a, 55 },
    { 0x93af768a, 23 },
    { 0, 0 },
    { 0x6cbf768d, 100 },
    { 0, 0 },
    { 0x671f768f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcf7692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f769d, 90 },
    { 0xfbef769d, 55 },
    { 0xe6bf769d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf76a5, 81 },
    { 0, 0 },
    { 0x742f76a7, 55 },
    { 0x3d7f76a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f76af, 66 },
    { 0xd96f76af, 90 },
    { 0xb75f76b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f76b6, 61 },
    { 0x602f76b6, 76 },
    { 0x27cf76b7, 76 },
    { 0xbb2f76b9, 61 },
    { 0x24df76b9, 23 },
    { 0, 0 },
    { 0x8c7f76bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f76c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f76c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672f76cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ef76d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acf76dc, 24 },
    { 0, 0 },
    { 0xcf2f76de, 76 },
    { 0x6dff76df, 65 },
    { 0, 0 },
    { 0xb34f76e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef76e6, 55 },
    { 0x38cf76e7, 23 },
    { 0, 0 },
    { 0xda2f76e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f76ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf76f2, 78 },
    { 0x4def76f3, 46 },
    { 0x85ff76f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f76f9, 34 },
    { 0x921f76fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293f7704, 36 },
    { 0, 0 },
    { 0xfb1f7706, 33 },
    { 0x4d5f7706, 23 },
    { 0x476f7708, 61 },
    { 0xdbcf7709, 98 },
    { 0xbecf770a, 23 },
    { 0, 0 },
    { 0x9cdf770c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9f7714, 85 },
    { 0x96ef7715, 93 },
    { 0xd66f7715, 66 },
    { 0x366f7717, 61 },
    { 0, 0 },
    { 0x4fcf7719, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66f771c, 88 },
    { 0x1bf771d, 33 },
    { 0x6b3f771e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ff7722, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0f7728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5f772b, 40 },
    { 0, 0 },
    { 0x3c5f772d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9f7735, 34 },
    { 0, 0 },
    { 0x494f7737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842f773d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ef7741, 76 },
    { 0x4e5f7741, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff7746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf7749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf7750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7f7755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f7758, 55 },
    { 0x2cf7758, 33 },
    { 0, 0 },
    { 0x8bdf775b, 33 },
    { 0, 0 },
    { 0x73df775d, 33 },
    { 0xa6df775e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816f7762, 33 },
    { 0xc2cf7763, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bf7767, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f776a, 96 },
    { 0xe04f776b, 33 },
    { 0xdcff776c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdf7770, 93 },
    { 0, 0 },
    { 0x215f7772, 98 },
    { 0x7bbf7772, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f7778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bf777e, 55 },
    { 0, 0 },
    { 0x17cf7780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f7783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6df7788, 90 },
    { 0, 0 },
    { 0x20af778a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf778f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7f7797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df779b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff77a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf77a5, 97 },
    { 0xb9af77a5, 43 },
    { 0x310f77a5, 99 },
    { 0x552f77a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x386f77ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f77ae, 61 },
    { 0x9b0f77af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43f77b4, 33 },
    { 0xb9af77b5, 90 },
    { 0, 0 },
    { 0xbd8f77b7, 84 },
    { 0x813f77b8, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839f77bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0f77c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaf77c5, 23 },
    { 0x941f77c6, 23 },
    { 0, 0 },
    { 0x18ff77c8, 47 },
    { 0x761f77c9, 90 },
    { 0xa23f77c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f77cf, 95 },
    { 0x38ef77cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adf77d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573f77e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895f77e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f77eb, 61 },
    { 0x8df77ec, 95 },
    { 0xf04f77ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f77f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f77f3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658f77f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f77fb, 95 },
    { 0x61cf77fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f7803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38f780b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f780f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x32af7812, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235f7816, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5f781b, 90 },
    { 0xcf9f781c, 68 },
    { 0xe9bf781d, 61 },
    { 0x4e5f781e, 98 },
    { 0x54f781f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517f7825, 23 },
    { 0x2c4f7826, 46 },
    { 0, 0 },
    { 0xe8f7828, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62f782c, 55 },
    { 0, 0 },
    { 0xb62f782e, 75 },
    { 0xbfaf782f, 33 },
    { 0, 0 },
    { 0x28bf7831, 76 },
    { 0xae6f7832, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580f7836, 98 },
    { 0, 0 },
    { 0x62f7838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x919f783b, 34 },
    { 0xcd3f783c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdf7842, 33 },
    { 0xecdf7843, 18 },
    { 0xa6af7844, 23 },
    { 0x4aaf7845, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405f784c, 61 },
    { 0xeeef784c, 23 },
    { 0xbe2f784e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf7851, 28 },
    { 0, 0 },
    { 0x2b3f7853, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1f7858, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8df785f, 33 },
    { 0x3e8f7860, 84 },
    { 0x89af7861, 98 },
    { 0xb73f7862, 98 },
    { 0, 0 },
    { 0x9ccf7864, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3f7868, 90 },
    { 0xb6cf7869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542f786e, 33 },
    { 0, 0 },
    { 0xf22f7870, 76 },
    { 0xc6ef7870, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bef7875, 95 },
    { 0xb80f7876, 65 },
    { 0xedff7876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf787e, 40 },
    { 0, 0 },
    { 0x746f7880, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48f7884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ef7887, 76 },
    { 0x3d1f7887, 61 },
    { 0, 0 },
    { 0x8d2f788a, 90 },
    { 0x2acf788b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadf7890, 90 },
    { 0x9d2f7891, 43 },
    { 0x683f7891, 99 },
    { 0xeb6f7893, 52 },
    { 0x29af7892, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f7897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5f789e, 90 },
    { 0x3f8f789f, 23 },
    { 0x3f1f78a0, 33 },
    { 0xafcf78a1, 90 },
    { 0, 0 },
    { 0xe1df78a3, 61 },
    { 0x5b4f78a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f78a7, 90 },
    { 0x465f78a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf78ab, 23 },
    { 0, 0 },
    { 0x114f78ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff78b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f78b3, 33 },
    { 0, 0 },
    { 0xdddf78b5, 34 },
    { 0xdb6f78b6, 55 },
    { 0x5abf78b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x317f78ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bf78be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09f78c6, 61 },
    { 0, 0 },
    { 0x5f9f78c8, 98 },
    { 0, 0 },
    { 0xc51f78ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04f78d3, 52 },
    { 0, 0 },
    { 0xec8f78d5, 33 },
    { 0xdb4f78d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ff78dc, 34 },
    { 0xa6f78dd, 23 },
    { 0x7ff78de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7f78e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf78e8, 61 },
    { 0xcacf78e9, 52 },
    { 0x2eaf78e9, 98 },
    { 0xc72f78ea, 67 },
    { 0, 0 },
    { 0x440f78ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f78f3, 33 },
    { 0, 0 },
    { 0xc56f78f5, 33 },
    { 0xe59f78f6, 77 },
    { 0x9c3f78f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cf78fa, 61 },
    { 0xdfbf78fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5f78ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f7906, 33 },
    { 0x221f7907, 93 },
    { 0xb63f7908, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638f790c, 90 },
    { 0x3faf790d, 23 },
    { 0x9b5f790d, 33 },
    { 0x3cef790f, 90 },
    { 0x52bf7910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f7913, 85 },
    { 0, 0 },
    { 0xd78f7915, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6f7919, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf791c, 98 },
    { 0x80ff791d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cf7921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f7924, 95 },
    { 0, 0 },
    { 0xcbff7926, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f7929, 69 },
    { 0xe54f792a, 95 },
    { 0xd3f792a, 90 },
    { 0xaf5f792a, 61 },
    { 0xbe5f792d, 61 },
    { 0xabaf792d, 43 },
    { 0x10ef7929, 34 },
    { 0, 0 },
    { 0xb11f7931, 95 },
    { 0xbddf7932, 55 },
    { 0, 0 },
    { 0xeeff7934, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f793a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2f793d, 95 },
    { 0xaedf793d, 88 },
    { 0x354f793f, 83 },
    { 0xe4ef7940, 76 },
    { 0x9acf7941, 95 },
    { 0, 0 },
    { 0x45af7943, 61 },
    { 0x76af7943, 33 },
    { 0xf92f7945, 95 },
    { 0x184f7946, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f794b, 90 },
    { 0x8b3f794c, 77 },
    { 0x5eaf794c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0f7957, 69 },
    { 0, 0 },
    { 0x9f0f7959, 19 },
    { 0x50af795a, 95 },
    { 0x3cbf795a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f795f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df7962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bf7965, 55 },
    { 0xbd9f7966, 76 },
    { 0x547f7967, 90 },
    { 0x311f7968, 80 },
    { 0x128f7969, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f796c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ff796f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbf7972, 98 },
    { 0x50bf7972, 91 },
    { 0x198f7972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f7979, 91 },
    { 0xadf797a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f797e, 90 },
    { 0x8aaf797f, 55 },
    { 0, 0 },
    { 0xe44f7981, 90 },
    { 0x3faf7982, 95 },
    { 0x17cf7983, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6f798a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f798e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f7992, 99 },
    { 0x95ef7993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aef7997, 43 },
    { 0, 0 },
    { 0xfccf7999, 52 },
    { 0x74f799a, 98 },
    { 0xd69f799b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdf799e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f79a4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cf79aa, 34 },
    { 0x742f79ab, 76 },
    { 0xe0bf79ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cf79b2, 55 },
    { 0, 0 },
    { 0x5f8f79b4, 98 },
    { 0, 0 },
    { 0x29ef79b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ff79ba, 93 },
    { 0xbf4f79bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bef79bf, 81 },
    { 0xe4af79c0, 85 },
    { 0, 0 },
    { 0x495f79c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f79c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f79cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f79ce, 55 },
    { 0x79af79ce, 23 },
    { 0xed8f79d0, 61 },
    { 0xfaff79d1, 77 },
    { 0x3bf79d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365f79d6, 73 },
    { 0xb1df79d7, 98 },
    { 0x36f79d7, 55 },
    { 0xc27f79d9, 34 },
    { 0xa21f79d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fef79dd, 66 },
    { 0, 0 },
    { 0x574f79df, 90 },
    { 0x208f79e0, 76 },
    { 0x28cf79e1, 87 },
    { 0x109f79e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcf79ea, 95 },
    { 0, 0 },
    { 0x447f79ec, 93 },
    { 0x830f79ed, 66 },
    { 0x6d5f79ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f79f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67f79f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f79fa, 9 },
    { 0, 0 },
    { 0x8ecf79fc, 85 },
    { 0x13cf79fd, 68 },
    { 0x793f79fe, 55 },
    { 0, 0 },
    { 0x77ef7a00, 84 },
    { 0x515f7a01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x261f7a04, 33 },
    { 0, 0 },
    { 0xe1cf7a06, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f7a10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71f7a16, 17 },
    { 0x2c7f7a17, 95 },
    { 0, 0 },
    { 0x45df7a19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7a1d, 61 },
    { 0xf42f7a1e, 34 },
    { 0, 0 },
    { 0x502f7a20, 95 },
    { 0x638f7a21, 84 },
    { 0x6f6f7a22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8f7a26, 34 },
    { 0, 0 },
    { 0xc9df7a28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f7a2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4f7a32, 33 },
    { 0xb1ff7a33, 90 },
    { 0, 0 },
    { 0x118f7a35, 87 },
    { 0, 0 },
    { 0x79af7a37, 75 },
    { 0xbd9f7a38, 34 },
    { 0, 0 },
    { 0x341f7a3a, 23 },
    { 0xe72f7a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef7a3f, 61 },
    { 0x4e8f7a40, 33 },
    { 0, 0 },
    { 0x599f7a42, 95 },
    { 0xca2f7a43, 23 },
    { 0x9b9f7a42, 61 },
    { 0x8e8f7a45, 52 },
    { 0xb5ff7a44, 76 },
    { 0x18bf7a47, 33 },
    { 0x9fef7a48, 91 },
    { 0xbe0f7a49, 61 },
    { 0, 0 },
    { 0x1c7f7a4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70df7a4f, 76 },
    { 0x631f7a50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faf7a59, 43 },
    { 0, 0 },
    { 0x202f7a5b, 76 },
    { 0x825f7a5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54f7a5f, 90 },
    { 0x73df7a60, 88 },
    { 0x17bf7a60, 51 },
    { 0x477f7a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcf7a65, 33 },
    { 0x931f7a66, 61 },
    { 0, 0 },
    { 0xd18f7a68, 55 },
    { 0x7dbf7a69, 23 },
    { 0xab7f7a6a, 23 },
    { 0, 0 },
    { 0x840f7a6c, 95 },
    { 0x81bf7a6d, 67 },
    { 0x24ff7a6e, 34 },
    { 0x4cdf7a6f, 43 },
    { 0xe17f7a70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2f7a74, 90 },
    { 0x690f7a75, 46 },
    { 0x149f7a75, 52 },
    { 0, 0 },
    { 0x275f7a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7a7e, 33 },
    { 0x8f6f7a7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddff7a82, 23 },
    { 0xe00f7a83, 17 },
    { 0xdb1f7a84, 61 },
    { 0, 0 },
    { 0xdc9f7a86, 85 },
    { 0xc55f7a86, 90 },
    { 0, 0 },
    { 0xdc4f7a89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf7a8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7a95, 33 },
    { 0xbd2f7a95, 43 },
    { 0x7e3f7a97, 61 },
    { 0x8a7f7a98, 78 },
    { 0x7eef7a98, 95 },
    { 0x14f7a99, 85 },
    { 0x179f7a9b, 55 },
    { 0x9cbf7a9c, 95 },
    { 0x397f7a9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864f7aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf7aad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6f7ab0, 90 },
    { 0, 0 },
    { 0x8a7f7ab2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7ab5, 84 },
    { 0, 0 },
    { 0x5c6f7ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff7abe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x877f7ac1, 33 },
    { 0, 0 },
    { 0xf94f7ac3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f7ac6, 33 },
    { 0x2f5f7ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85f7aca, 33 },
    { 0xb82f7acb, 98 },
    { 0x300f7acb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf7acf, 43 },
    { 0, 0 },
    { 0x118f7ad1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83af7ad5, 93 },
    { 0xabff7ad5, 23 },
    { 0x35af7ad7, 52 },
    { 0x836f7ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10af7adc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f7ae2, 61 },
    { 0xebef7ae3, 93 },
    { 0, 0 },
    { 0xa56f7ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f7ae9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2f7aee, 46 },
    { 0xfc0f7aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ff7af5, 23 },
    { 0x555f7af5, 55 },
    { 0xff9f7af7, 61 },
    { 0xc7cf7af7, 61 },
    { 0xc8bf7af9, 52 },
    { 0xb84f7afa, 55 },
    { 0x336f7afa, 76 },
    { 0x3b9f7afc, 76 },
    { 0xb3ef7afc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cf7b01, 95 },
    { 0x45f7b02, 90 },
    { 0xcf5f7b03, 90 },
    { 0xa8af7b02, 33 },
    { 0xb6f7b05, 34 },
    { 0xe88f7b06, 61 },
    { 0xa61f7b07, 90 },
    { 0x8eaf7b07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0f7b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x240f7b11, 43 },
    { 0, 0 },
    { 0xcdef7b13, 90 },
    { 0, 0 },
    { 0x20ef7b15, 80 },
    { 0, 0 },
    { 0x3d2f7b17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f7b1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x122f7b1d, 34 },
    { 0, 0 },
    { 0x5e2f7b1f, 61 },
    { 0, 0 },
    { 0x7c4f7b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746f7b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf7b2a, 23 },
    { 0xa63f7b2a, 78 },
    { 0x3fdf7b2c, 70 },
    { 0xd0ff7b2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5f7b34, 90 },
    { 0xe4ef7b34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f7b42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07f7b48, 46 },
    { 0xeecf7b49, 66 },
    { 0xa6ef7b4a, 52 },
    { 0x917f7b48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df7b53, 81 },
    { 0x2a2f7b53, 69 },
    { 0x739f7b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f7b59, 93 },
    { 0x525f7b5a, 51 },
    { 0x5ff7b5b, 88 },
    { 0x1d5f7b5b, 61 },
    { 0x9baf7b5b, 90 },
    { 0xc7af7b59, 81 },
    { 0xcfdf7b5f, 33 },
    { 0x9b1f7b60, 91 },
    { 0x39cf7b61, 23 },
    { 0, 0 },
    { 0xc0ff7b63, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6f7b68, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f7b6c, 90 },
    { 0, 0 },
    { 0x4f9f7b6e, 85 },
    { 0x95ef7b6f, 90 },
    { 0xb3bf7b6f, 90 },
    { 0x8fdf7b71, 95 },
    { 0x68ef7b72, 95 },
    { 0x3dcf7b73, 17 },
    { 0x62ef7b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acf7b78, 95 },
    { 0xd84f7b79, 23 },
    { 0xd98f7b7a, 33 },
    { 0xc11f7b79, 55 },
    { 0xa82f7b7b, 98 },
    { 0, 0 },
    { 0x5bdf7b7e, 98 },
    { 0x34af7b7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf7b86, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f7b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcf7b8e, 23 },
    { 0x878f7b8f, 90 },
    { 0x190f7b8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edf7b93, 100 },
    { 0, 0 },
    { 0x8dbf7b95, 91 },
    { 0, 0 },
    { 0x328f7b97, 95 },
    { 0x8f0f7b98, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5f7b9e, 90 },
    { 0xe59f7b9f, 55 },
    { 0x702f7ba0, 91 },
    { 0x4d7f7ba1, 80 },
    { 0xb0ff7ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcf7ba7, 76 },
    { 0xda2f7ba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x702f7bab, 55 },
    { 0, 0 },
    { 0xc0f7bad, 61 },
    { 0xe63f7bae, 34 },
    { 0, 0 },
    { 0xce7f7bb0, 88 },
    { 0x841f7bb1, 95 },
    { 0xa15f7bb2, 76 },
    { 0x461f7bb3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf7bb7, 90 },
    { 0x532f7bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf7bbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510f7bc3, 90 },
    { 0x8a9f7bc4, 33 },
    { 0x9c8f7bc4, 98 },
    { 0xfa9f7bc4, 95 },
    { 0x284f7bc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdf7bca, 68 },
    { 0x380f7bca, 33 },
    { 0xf3f7bcc, 23 },
    { 0x90f7bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f7bd4, 76 },
    { 0x508f7bd4, 33 },
    { 0x716f7bd6, 99 },
    { 0x1cef7bd5, 90 },
    { 0xe9ff7bd8, 95 },
    { 0x5a5f7bd8, 93 },
    { 0xed5f7bd5, 61 },
    { 0x5f5f7bdb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cf7bde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf7be3, 46 },
    { 0xd7df7be4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf7beb, 94 },
    { 0, 0 },
    { 0x8a8f7bed, 55 },
    { 0xad2f7bed, 93 },
    { 0, 0 },
    { 0x27bf7bf0, 95 },
    { 0x25bf7bf0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf7bf4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f7bf7, 98 },
    { 0xdfdf7bf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f7c02, 98 },
    { 0x71cf7c02, 43 },
    { 0, 0 },
    { 0x26bf7c05, 95 },
    { 0xe8f7c06, 61 },
    { 0xdaaf7c06, 95 },
    { 0x7a5f7c06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cf7c0e, 90 },
    { 0x1f1f7c0e, 23 },
    { 0xb64f7c10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ef7c16, 90 },
    { 0, 0 },
    { 0xc6cf7c18, 51 },
    { 0xdb6f7c19, 23 },
    { 0, 0 },
    { 0x903f7c1b, 61 },
    { 0, 0 },
    { 0xf27f7c1d, 90 },
    { 0, 0 },
    { 0x542f7c1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cff7c24, 68 },
    { 0xd62f7c25, 33 },
    { 0x6f1f7c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4f7c2c, 43 },
    { 0, 0 },
    { 0x6a1f7c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af7c31, 78 },
    { 0x954f7c32, 76 },
    { 0, 0 },
    { 0xc33f7c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bf7c40, 8 },
    { 0, 0 },
    { 0x2dcf7c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f7c45, 98 },
    { 0x7bff7c45, 100 },
    { 0x333f7c46, 95 },
    { 0x4bef7c45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf7c52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fff7c55, 33 },
    { 0xbf3f7c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ef7c59, 98 },
    { 0x3dff7c59, 34 },
    { 0, 0 },
    { 0xf86f7c5c, 43 },
    { 0x94df7c5c, 61 },
    { 0x36df7c5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f7c61, 55 },
    { 0, 0 },
    { 0x5f1f7c63, 91 },
    { 0xb68f7c63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f7c67, 95 },
    { 0, 0 },
    { 0x11ff7c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3df7c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef7c72, 47 },
    { 0x349f7c73, 84 },
    { 0, 0 },
    { 0x6a7f7c75, 76 },
    { 0xc66f7c76, 52 },
    { 0xd67f7c77, 93 },
    { 0x60ff7c78, 95 },
    { 0x41bf7c79, 76 },
    { 0x18ef7c77, 98 },
    { 0x147f7c7b, 95 },
    { 0x251f7c7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caf7c7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f7c82, 90 },
    { 0, 0 },
    { 0xc3bf7c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f7c8a, 33 },
    { 0, 0 },
    { 0xb8ff7c8c, 90 },
    { 0x95af7c8d, 23 },
    { 0xe94f7c8e, 90 },
    { 0x324f7c8e, 88 },
    { 0xf70f7c90, 76 },
    { 0xe8df7c8e, 55 },
    { 0x197f7c90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf7c96, 43 },
    { 0xdf0f7c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6f7c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5f7ca4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472f7ca8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff7cad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f7cb2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30f7cb5, 99 },
    { 0, 0 },
    { 0x8abf7cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f7cbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ff7cbf, 55 },
    { 0x5a7f7cc0, 66 },
    { 0, 0 },
    { 0x171f7cc2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f7cc7, 33 },
    { 0x3dff7cc8, 33 },
    { 0x9f8f7cc9, 98 },
    { 0xdd2f7cc7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf7ccf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbf7cdc, 90 },
    { 0xe88f7cdc, 52 },
    { 0x762f7cde, 55 },
    { 0x5f4f7cde, 33 },
    { 0x51ef7cde, 17 },
    { 0x3a2f7cdc, 99 },
    { 0x91f7ce2, 76 },
    { 0x72ef7ce3, 81 },
    { 0x5b9f7ce4, 55 },
    { 0xa97f7ce4, 24 },
    { 0x6c5f7ce6, 38 },
    { 0x7f8f7ce4, 90 },
    { 0x1aaf7cdd, 45 },
    { 0x856f7ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5f7cec, 93 },
    { 0, 0 },
    { 0xe87f7cee, 94 },
    { 0x1f4f7cee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9f7cf4, 34 },
    { 0, 0 },
    { 0x819f7cf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671f7cfa, 85 },
    { 0xe07f7cfa, 61 },
    { 0x1d2f7cfc, 33 },
    { 0x7af7cfd, 61 },
    { 0, 0 },
    { 0xb55f7cff, 76 },
    { 0xf11f7d00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f7d04, 98 },
    { 0, 0 },
    { 0x8fbf7d06, 23 },
    { 0x6d8f7d07, 91 },
    { 0, 0 },
    { 0xf4ff7d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf7d0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58f7d12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df7d17, 95 },
    { 0xeb9f7d18, 90 },
    { 0, 0 },
    { 0x69df7d1a, 90 },
    { 0, 0 },
    { 0x5e7f7d1c, 43 },
    { 0xfb4f7d1d, 95 },
    { 0xd8ef7d1d, 90 },
    { 0x631f7d1f, 61 },
    { 0xc98f7d20, 75 },
    { 0x24bf7d21, 98 },
    { 0, 0 },
    { 0xa71f7d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1f7d27, 90 },
    { 0, 0 },
    { 0x96ef7d29, 76 },
    { 0x8fcf7d2a, 61 },
    { 0xbcbf7d2a, 66 },
    { 0x44cf7d2c, 76 },
    { 0x723f7d2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8f7d30, 48 },
    { 0x504f7d30, 95 },
    { 0x9cff7d30, 23 },
    { 0, 0 },
    { 0x895f7d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf7d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7def7d3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfef7d40, 55 },
    { 0x63df7d40, 84 },
    { 0, 0 },
    { 0xa94f7d43, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aef7d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f7d49, 98 },
    { 0, 0 },
    { 0x1e8f7d4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff7d50, 22 },
    { 0x4f0f7d50, 33 },
    { 0xb38f7d50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf7d55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedf7d5c, 61 },
    { 0xc33f7d5d, 61 },
    { 0x39df7d5c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52f7d61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x489f7d64, 90 },
    { 0xdbbf7d65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28f7d69, 61 },
    { 0x22df7d6a, 61 },
    { 0, 0 },
    { 0x592f7d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00f7d78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f7d7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f7d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef7d86, 76 },
    { 0x6aff7d87, 61 },
    { 0x566f7d87, 43 },
    { 0, 0 },
    { 0x31f7d8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f7d92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f7d95, 33 },
    { 0x7e1f7d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f7d99, 90 },
    { 0xc2ef7d99, 100 },
    { 0, 0 },
    { 0x76cf7d9c, 90 },
    { 0, 0 },
    { 0xc7ff7d9e, 98 },
    { 0, 0 },
    { 0xa30f7da0, 93 },
    { 0xe59f7da1, 84 },
    { 0, 0 },
    { 0x3f0f7da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f7da6, 34 },
    { 0x87af7da7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f7db0, 46 },
    { 0, 0 },
    { 0xa81f7db2, 23 },
    { 0xd8ff7db3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0f7db6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af7dbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f7dc1, 98 },
    { 0x3a1f7dc2, 33 },
    { 0, 0 },
    { 0x59df7dc4, 34 },
    { 0x632f7dc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f7dca, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970f7dce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f7dd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f7dd4, 77 },
    { 0xd24f7dd4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16df7dd9, 23 },
    { 0, 0 },
    { 0xb13f7ddb, 78 },
    { 0x763f7ddc, 90 },
    { 0xb60f7ddc, 76 },
    { 0x8b8f7dde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f7de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f7de5, 34 },
    { 0x5a4f7de6, 55 },
    { 0, 0 },
    { 0x921f7de8, 97 },
    { 0x170f7de9, 95 },
    { 0, 0 },
    { 0x44ef7deb, 90 },
    { 0xb6ff7dec, 67 },
    { 0xf7bf7ded, 98 },
    { 0x98cf7dee, 95 },
    { 0x3af7dee, 34 },
    { 0xda2f7df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639f7df6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df7dfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f7dfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f7e00, 90 },
    { 0x866f7e01, 61 },
    { 0x286f7e02, 33 },
    { 0x14af7e02, 90 },
    { 0x801f7e02, 61 },
    { 0x68af7e05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df7e0c, 90 },
    { 0, 0 },
    { 0xa24f7e0e, 52 },
    { 0x598f7e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4f7e12, 23 },
    { 0x7f1f7e13, 33 },
    { 0x453f7e13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f7e22, 61 },
    { 0xd0ef7e23, 55 },
    { 0xbedf7e24, 34 },
    { 0x323f7e25, 68 },
    { 0xda4f7e26, 90 },
    { 0x2d4f7e22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf7e2a, 43 },
    { 0, 0 },
    { 0x591f7e2c, 95 },
    { 0, 0 },
    { 0x323f7e2e, 90 },
    { 0, 0 },
    { 0x4c9f7e30, 61 },
    { 0xa79f7e31, 33 },
    { 0, 0 },
    { 0xc3ef7e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df7e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0f7e3a, 91 },
    { 0xf5f7e3a, 67 },
    { 0x370f7e3c, 68 },
    { 0, 0 },
    { 0x489f7e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bf7e46, 34 },
    { 0xfaff7e47, 90 },
    { 0xa6df7e48, 98 },
    { 0, 0 },
    { 0x9d8f7e4a, 61 },
    { 0, 0 },
    { 0x628f7e4c, 34 },
    { 0x879f7e4d, 78 },
    { 0xdb0f7e4e, 23 },
    { 0x2c9f7e4e, 95 },
    { 0xe97f7e4d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9f7e56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f7e5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf7e60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405f7e65, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f7e6c, 43 },
    { 0, 0 },
    { 0xabaf7e6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ff7e75, 95 },
    { 0, 0 },
    { 0x774f7e77, 33 },
    { 0, 0 },
    { 0xf7af7e79, 83 },
    { 0xacff7e7a, 46 },
    { 0x6a0f7e7b, 76 },
    { 0x6f3f7e7c, 78 },
    { 0x9d5f7e7c, 98 },
    { 0xda7f7e7e, 91 },
    { 0xe7f7e7f, 23 },
    { 0x2d2f7e80, 95 },
    { 0x23af7e7b, 61 },
    { 0x7a3f7e79, 100 },
    { 0xbf8f7e83, 77 },
    { 0, 0 },
    { 0x71ff7e85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f7e89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f7e8c, 55 },
    { 0x577f7e8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff7e91, 95 },
    { 0xe57f7e92, 33 },
    { 0x3e2f7e93, 23 },
    { 0x65ef7e94, 58 },
    { 0, 0 },
    { 0xcbbf7e96, 33 },
    { 0, 0 },
    { 0x280f7e98, 33 },
    { 0, 0 },
    { 0xc2bf7e9a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf7e9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x831f7ea0, 56 },
    { 0xe71f7ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f7ea5, 95 },
    { 0x87bf7ea6, 87 },
    { 0x976f7ea7, 93 },
    { 0x634f7ea8, 23 },
    { 0xad7f7ea7, 78 },
    { 0x19af7eaa, 23 },
    { 0, 0 },
    { 0x13cf7eac, 77 },
    { 0x648f7eac, 33 },
    { 0x8dff7ead, 23 },
    { 0, 0 },
    { 0x702f7eb0, 76 },
    { 0xfdaf7eb1, 55 },
    { 0xef7f7eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abf7eb5, 95 },
    { 0x6a7f7eb6, 98 },
    { 0x51af7eb6, 90 },
    { 0, 0 },
    { 0xc7cf7eb9, 90 },
    { 0x77bf7eb9, 76 },
    { 0, 0 },
    { 0xd89f7ebc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389f7ec0, 53 },
    { 0, 0 },
    { 0x1c5f7ec2, 17 },
    { 0xec6f7ec2, 52 },
    { 0x198f7ec3, 78 },
    { 0x20bf7ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ef7ecd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731f7ed1, 55 },
    { 0, 0 },
    { 0xc9bf7ed3, 78 },
    { 0, 0 },
    { 0xbc8f7ed5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f7ed9, 95 },
    { 0, 0 },
    { 0x4d7f7edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39f7ee0, 95 },
    { 0xbf3f7ee1, 90 },
    { 0, 0 },
    { 0x811f7ee3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777f7ee8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3f7eef, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24f7ef2, 88 },
    { 0x481f7ef3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f7efa, 90 },
    { 0x4a6f7efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df7efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7f01, 90 },
    { 0xd4ef7f02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf7f08, 98 },
    { 0x5aff7f09, 34 },
    { 0x6b4f7f0a, 84 },
    { 0xe0bf7f08, 90 },
    { 0, 0 },
    { 0x5bf7f0d, 33 },
    { 0xdaaf7f0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef7f11, 98 },
    { 0x378f7f12, 90 },
    { 0, 0 },
    { 0xee4f7f14, 90 },
    { 0xed5f7f15, 66 },
    { 0x98cf7f14, 90 },
    { 0xed2f7f15, 99 },
    { 0x7bbf7f18, 90 },
    { 0x6bcf7f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf7f1e, 98 },
    { 0x2e1f7f1e, 98 },
    { 0xce8f7f20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf7f24, 34 },
    { 0, 0 },
    { 0xd63f7f26, 93 },
    { 0xbd5f7f27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8f7f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x814f7f38, 80 },
    { 0x168f7f39, 34 },
    { 0xf72f7f39, 95 },
    { 0x717f7f3b, 84 },
    { 0xcd9f7f3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef7f3f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f7f43, 90 },
    { 0x722f7f44, 23 },
    { 0xb98f7f45, 64 },
    { 0xd67f7f46, 34 },
    { 0xc06f7f44, 46 },
    { 0x152f7f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bf7f4b, 52 },
    { 0xe63f7f4b, 95 },
    { 0xa1ef7f4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf7f54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f7f57, 46 },
    { 0, 0 },
    { 0x145f7f59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faf7f5e, 90 },
    { 0, 0 },
    { 0x9f8f7f60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711f7f64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462f7f6a, 61 },
    { 0, 0 },
    { 0xe46f7f6c, 98 },
    { 0, 0 },
    { 0x681f7f6e, 100 },
    { 0, 0 },
    { 0xeaff7f70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef7f76, 33 },
    { 0x4b6f7f77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bf7f7a, 98 },
    { 0, 0 },
    { 0xb14f7f7c, 95 },
    { 0xbd2f7f7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f7f80, 61 },
    { 0x496f7f80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3f7f84, 55 },
    { 0xeb8f7f84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f7f8c, 90 },
    { 0x695f7f8d, 95 },
    { 0xc19f7f8d, 85 },
    { 0x372f7f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcf7f92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5f7f98, 84 },
    { 0x807f7f98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4f7f9f, 90 },
    { 0xb59f7fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f7fa3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f7fa6, 66 },
    { 0x1dbf7fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eef7fab, 99 },
    { 0, 0 },
    { 0xe48f7fad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486f7fb2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf7fb5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f7fb8, 95 },
    { 0x8fdf7fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaf7fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff7fc2, 90 },
    { 0x147f7fc3, 66 },
    { 0xeedf7fc3, 95 },
    { 0x499f7fc3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xedef7fc8, 100 },
    { 0xd4f7fc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f7fcc, 65 },
    { 0, 0 },
    { 0x546f7fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f7fd4, 90 },
    { 0xadcf7fd5, 96 },
    { 0xf78f7fd5, 90 },
    { 0, 0 },
    { 0xe03f7fd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff7fdb, 90 },
    { 0x94cf7fdc, 61 },
    { 0xc5ef7fdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df7fe0, 61 },
    { 0, 0 },
    { 0xadf7fe2, 33 },
    { 0x4eaf7fe3, 90 },
    { 0, 0 },
    { 0x87df7fe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26df7feb, 98 },
    { 0x96cf7feb, 76 },
    { 0xb2af7feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f7ffa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f8002, 55 },
    { 0, 0 },
    { 0x4a2f8004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767f8009, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45f800c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaff8010, 98 },
    { 0x7f8f8010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f8014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x358f8017, 61 },
    { 0x78bf8018, 46 },
    { 0xb5bf8019, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f801d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef8020, 55 },
    { 0xd46f8021, 95 },
    { 0x5b1f8022, 78 },
    { 0xdf6f8020, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf8027, 81 },
    { 0x186f8027, 36 },
    { 0, 0 },
    { 0xe35f802a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f802e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f8032, 66 },
    { 0x96df8033, 42 },
    { 0x6a7f8034, 43 },
    { 0xc8ff8035, 90 },
    { 0, 0 },
    { 0xbc8f8037, 90 },
    { 0xe0ff8037, 98 },
    { 0x381f8037, 99 },
    { 0xce6f8037, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x623f803d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33f8040, 33 },
    { 0x25ef8040, 95 },
    { 0x678f8042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60f8047, 98 },
    { 0x256f8048, 90 },
    { 0x441f8049, 95 },
    { 0x6d9f804a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f804e, 90 },
    { 0, 0 },
    { 0x33f8050, 23 },
    { 0x346f8051, 90 },
    { 0xba9f8052, 90 },
    { 0x25bf8051, 61 },
    { 0x58cf8051, 98 },
    { 0x413f8055, 33 },
    { 0x9c9f8056, 33 },
    { 0x282f8057, 52 },
    { 0x6d1f8056, 98 },
    { 0xe6df8051, 93 },
    { 0x233f805a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0f8065, 90 },
    { 0, 0 },
    { 0xff6f8067, 90 },
    { 0, 0 },
    { 0x4aef8069, 91 },
    { 0x514f8069, 90 },
    { 0xb6ff806b, 85 },
    { 0x912f8069, 95 },
    { 0, 0 },
    { 0xe21f806e, 34 },
    { 0x863f806f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f8074, 90 },
    { 0xfc7f8075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113f8079, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff8081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9f8088, 90 },
    { 0x593f8088, 61 },
    { 0x771f808a, 75 },
    { 0, 0 },
    { 0x51f808c, 23 },
    { 0xd18f808c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebf8090, 84 },
    { 0x435f8090, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491f8095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cf80a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f80a4, 100 },
    { 0xb5af80a5, 98 },
    { 0x151f80a6, 90 },
    { 0xe78f80a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f80ad, 33 },
    { 0xe70f80ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf80b1, 33 },
    { 0x4a8f80b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f80bd, 52 },
    { 0x598f80be, 98 },
    { 0x640f80be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef80c5, 23 },
    { 0, 0 },
    { 0x7ddf80c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14f80cb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5f80ce, 17 },
    { 0, 0 },
    { 0xba5f80d0, 98 },
    { 0x48bf80d1, 33 },
    { 0, 0 },
    { 0x89df80d3, 100 },
    { 0xf71f80d4, 78 },
    { 0xf68f80d4, 55 },
    { 0xf7ef80d4, 66 },
    { 0, 0 },
    { 0x286f80d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x561f80db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af80de, 97 },
    { 0x69df80df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f80e4, 67 },
    { 0x678f80e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f80e8, 55 },
    { 0xf7bf80e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f80ec, 100 },
    { 0, 0 },
    { 0x5ef80ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f80f2, 77 },
    { 0x92bf80f3, 23 },
    { 0xaa0f80f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f80f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aef8101, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f810e, 98 },
    { 0, 0 },
    { 0xea4f8110, 90 },
    { 0xcb7f8111, 88 },
    { 0x3a8f8112, 52 },
    { 0xbdaf8113, 90 },
    { 0xde4f8110, 95 },
    { 0xe00f8114, 34 },
    { 0x68af8112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf811d, 34 },
    { 0xd53f811e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f8121, 66 },
    { 0x6c4f8121, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f8126, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f812b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x476f812e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ff8131, 98 },
    { 0xf6f8132, 33 },
    { 0x8fef8131, 95 },
    { 0xc05f8133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6f8137, 61 },
    { 0x5e4f8137, 23 },
    { 0x3bcf8138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94af813c, 33 },
    { 0x42f813d, 81 },
    { 0xc49f813e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661f814a, 34 },
    { 0x8f0f814b, 87 },
    { 0xe22f814b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24f814f, 66 },
    { 0x90cf8150, 99 },
    { 0, 0 },
    { 0x149f8152, 43 },
    { 0xea2f8153, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976f8157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385f815b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x110f815e, 90 },
    { 0x831f815f, 84 },
    { 0x958f8160, 66 },
    { 0x4a1f8161, 78 },
    { 0xc71f8162, 98 },
    { 0, 0 },
    { 0xce5f8164, 34 },
    { 0xad7f8164, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x576f8168, 76 },
    { 0x362f8168, 76 },
    { 0xea8f816a, 90 },
    { 0x193f8168, 96 },
    { 0xa1df816c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f8175, 33 },
    { 0, 0 },
    { 0xe7bf8177, 43 },
    { 0, 0 },
    { 0xf5af8179, 81 },
    { 0, 0 },
    { 0x9adf817b, 55 },
    { 0x32ff817c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6f817f, 90 },
    { 0x182f8180, 76 },
    { 0xc60f8180, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42f8187, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f818b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170f818f, 85 },
    { 0xf0af818f, 100 },
    { 0x205f8191, 98 },
    { 0, 0 },
    { 0x86ff8193, 95 },
    { 0, 0 },
    { 0xbedf8195, 46 },
    { 0xa7cf8195, 23 },
    { 0xc17f8197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296f819d, 33 },
    { 0x248f819d, 98 },
    { 0xb6cf819d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8f81a2, 98 },
    { 0xbbdf81a2, 69 },
    { 0xdcff81a3, 95 },
    { 0, 0 },
    { 0x826f81a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8f81ab, 55 },
    { 0, 0 },
    { 0x4d8f81ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950f81b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdf81b4, 43 },
    { 0, 0 },
    { 0x9e6f81b6, 55 },
    { 0x764f81b6, 75 },
    { 0x13bf81b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6f81d3, 100 },
    { 0xd3f81d3, 29 },
    { 0x6cff81d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f81d9, 87 },
    { 0x68cf81d9, 61 },
    { 0, 0 },
    { 0x166f81dc, 90 },
    { 0x582f81dc, 98 },
    { 0xd2ff81dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f81e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f81e8, 95 },
    { 0x84cf81e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f81ec, 90 },
    { 0x8e5f81ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef81f0, 33 },
    { 0x10af81f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8def81f5, 61 },
    { 0, 0 },
    { 0xcc7f81f7, 23 },
    { 0x7cef81f8, 61 },
    { 0, 0 },
    { 0xd9ff81fa, 76 },
    { 0, 0 },
    { 0xd19f81fc, 46 },
    { 0, 0 },
    { 0x934f81fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2f8205, 95 },
    { 0, 0 },
    { 0xf45f8207, 78 },
    { 0x80ef8207, 98 },
    { 0xd19f8209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ef820e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f8214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9f8218, 90 },
    { 0x64af8219, 36 },
    { 0, 0 },
    { 0x6b5f821b, 34 },
    { 0x4aaf821c, 40 },
    { 0x62f821c, 23 },
    { 0x86af821e, 95 },
    { 0xd4ff821f, 23 },
    { 0x683f821e, 84 },
    { 0x35bf8221, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x869f8224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf8227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf822e, 23 },
    { 0x529f822f, 65 },
    { 0xe69f8230, 81 },
    { 0, 0 },
    { 0x791f8232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf8238, 100 },
    { 0x2d1f8238, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f823c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26f8240, 90 },
    { 0, 0 },
    { 0xeb5f8242, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff8247, 76 },
    { 0x75cf8247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0f824e, 43 },
    { 0xe1df824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f8252, 34 },
    { 0xc4af8253, 90 },
    { 0x90f8254, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51f8257, 90 },
    { 0, 0 },
    { 0x844f8259, 69 },
    { 0x8ecf8259, 33 },
    { 0xf3ef825b, 67 },
    { 0xe60f825c, 100 },
    { 0, 0 },
    { 0x976f825e, 46 },
    { 0xe54f825f, 61 },
    { 0xc1af8260, 98 },
    { 0x756f8261, 33 },
    { 0x25df8262, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f8265, 76 },
    { 0, 0 },
    { 0x106f8267, 68 },
    { 0x856f8267, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cff826b, 76 },
    { 0, 0 },
    { 0xac4f826d, 98 },
    { 0, 0 },
    { 0xa6af826f, 23 },
    { 0xa3ff8270, 34 },
    { 0xb07f8270, 55 },
    { 0xb60f8270, 90 },
    { 0x956f8273, 67 },
    { 0xc34f8270, 33 },
    { 0x8f5f8270, 98 },
    { 0xc8cf826f, 90 },
    { 0, 0 },
    { 0x43cf8278, 66 },
    { 0, 0 },
    { 0xd74f827a, 95 },
    { 0x185f827b, 65 },
    { 0xd54f827c, 23 },
    { 0x87df827c, 40 },
    { 0xe87f827b, 33 },
    { 0, 0 },
    { 0x31af8280, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cf8285, 23 },
    { 0xd96f8286, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f828a, 66 },
    { 0x365f828a, 33 },
    { 0, 0 },
    { 0x7eff828d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52df8292, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f8296, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917f829a, 95 },
    { 0xd9f829b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f829f, 76 },
    { 0, 0 },
    { 0xc86f82a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13f82a5, 52 },
    { 0xfedf82a5, 94 },
    { 0xb09f82a7, 61 },
    { 0xa17f82a8, 85 },
    { 0, 0 },
    { 0x853f82aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95df82af, 75 },
    { 0x9c5f82b0, 90 },
    { 0, 0 },
    { 0xc9ff82b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74f82b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55f82b9, 38 },
    { 0, 0 },
    { 0x1dff82bb, 84 },
    { 0, 0 },
    { 0x1abf82bd, 23 },
    { 0xd8bf82be, 93 },
    { 0, 0 },
    { 0x794f82c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f82c3, 33 },
    { 0x68bf82c3, 96 },
    { 0xa47f82c5, 90 },
    { 0xef5f82c4, 81 },
    { 0, 0 },
    { 0x513f82c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905f82ce, 43 },
    { 0x435f82cf, 93 },
    { 0x87bf82cf, 90 },
    { 0x3b5f82d0, 33 },
    { 0x3eff82ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0f82d6, 93 },
    { 0, 0 },
    { 0x824f82d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdf82dc, 93 },
    { 0xd56f82dd, 33 },
    { 0x430f82de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344f82e3, 90 },
    { 0xf82f82e4, 65 },
    { 0, 0 },
    { 0x672f82e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f82e9, 91 },
    { 0, 0 },
    { 0x476f82eb, 76 },
    { 0, 0 },
    { 0xe06f82ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bf82f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f82f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x582f82f6, 55 },
    { 0, 0 },
    { 0xec6f82f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faf82fb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f8300, 78 },
    { 0x40ef8300, 23 },
    { 0x859f8302, 91 },
    { 0xa2af8303, 61 },
    { 0xcd3f8301, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f830d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152f8312, 68 },
    { 0x7b7f8313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f8317, 34 },
    { 0xd86f8318, 76 },
    { 0, 0 },
    { 0x669f831a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999f8320, 61 },
    { 0x71ff8320, 100 },
    { 0xae4f8322, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f8329, 33 },
    { 0x870f832a, 98 },
    { 0x78bf832b, 61 },
    { 0xffff832a, 33 },
    { 0x397f832d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694f8338, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x581f833b, 61 },
    { 0, 0 },
    { 0xf37f833d, 23 },
    { 0xb61f833e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf8341, 61 },
    { 0x199f8342, 90 },
    { 0, 0 },
    { 0xb56f8344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef8349, 85 },
    { 0, 0 },
    { 0x84bf834b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8350, 17 },
    { 0x3fdf8351, 95 },
    { 0x74df8352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccf8356, 14 },
    { 0x3d7f8357, 98 },
    { 0x8d0f8357, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ff835e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf8366, 78 },
    { 0x3ef8366, 88 },
    { 0x935f8368, 77 },
    { 0xba9f8369, 23 },
    { 0, 0 },
    { 0xb6af836b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4f836f, 87 },
    { 0, 0 },
    { 0x7caf8371, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf8376, 61 },
    { 0, 0 },
    { 0xe28f8378, 90 },
    { 0x7cbf8378, 90 },
    { 0xb78f837a, 52 },
    { 0xa51f837a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaf837f, 81 },
    { 0, 0 },
    { 0x5cdf8381, 33 },
    { 0xcc1f8382, 33 },
    { 0x5a4f8383, 33 },
    { 0x9b8f8381, 34 },
    { 0xaabf8385, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff8388, 66 },
    { 0, 0 },
    { 0xa71f838a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf838d, 91 },
    { 0xe81f838e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f839c, 23 },
    { 0, 0 },
    { 0x1edf839e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f83a6, 90 },
    { 0xa32f83a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f83aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f83ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93af83b4, 90 },
    { 0, 0 },
    { 0xd9ff83b6, 100 },
    { 0xc3ff83b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cf83ba, 66 },
    { 0, 0 },
    { 0xc8cf83bc, 90 },
    { 0x1c9f83bd, 33 },
    { 0x506f83be, 85 },
    { 0xab5f83be, 19 },
    { 0x180f83be, 69 },
    { 0x14ff83be, 91 },
    { 0, 0 },
    { 0x8b1f83c3, 98 },
    { 0xa41f83c4, 46 },
    { 0xdcef83c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf83cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdf83d1, 98 },
    { 0x6e5f83d2, 33 },
    { 0x446f83d3, 90 },
    { 0x593f83d4, 98 },
    { 0xdd0f83d5, 52 },
    { 0, 0 },
    { 0x97af83d7, 88 },
    { 0xdd8f83d8, 95 },
    { 0x8e4f83d8, 76 },
    { 0x73ff83d8, 95 },
    { 0xe26f83d7, 61 },
    { 0xb5df83da, 61 },
    { 0, 0 },
    { 0x32bf83de, 90 },
    { 0x4d6f83df, 23 },
    { 0x9a0f83e0, 90 },
    { 0xb0ff83e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28f83e4, 77 },
    { 0, 0 },
    { 0x158f83e6, 51 },
    { 0, 0 },
    { 0x3daf83e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6f83ee, 76 },
    { 0, 0 },
    { 0xd08f83f0, 82 },
    { 0x73df83f1, 91 },
    { 0xed6f83f0, 90 },
    { 0x14f83f2, 17 },
    { 0x396f83f4, 76 },
    { 0xe4bf83f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f83fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df8400, 10 },
    { 0x55f8400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f8408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf8411, 90 },
    { 0xd81f8412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874f8417, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x630f841a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94f841d, 76 },
    { 0x6fdf841d, 98 },
    { 0xa40f841d, 33 },
    { 0x3c6f841d, 55 },
    { 0xe5af8421, 33 },
    { 0x1c9f8422, 33 },
    { 0xfeff8421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f8428, 81 },
    { 0x75f8429, 98 },
    { 0xf1df8429, 43 },
    { 0, 0 },
    { 0x772f842c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f8430, 33 },
    { 0x9eff8431, 33 },
    { 0, 0 },
    { 0x797f8433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f8437, 98 },
    { 0xc89f8438, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ef843b, 23 },
    { 0x87ef843c, 52 },
    { 0x8cdf843c, 75 },
    { 0x88ef843e, 76 },
    { 0xa17f843d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32f844e, 83 },
    { 0x1e4f844f, 90 },
    { 0, 0 },
    { 0x5e6f8451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcf8454, 81 },
    { 0x98cf8455, 90 },
    { 0x9d8f8454, 61 },
    { 0x590f8455, 93 },
    { 0, 0 },
    { 0xb95f8459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df8461, 81 },
    { 0x876f8461, 33 },
    { 0x4c5f8463, 88 },
    { 0xc73f8464, 90 },
    { 0x211f8465, 95 },
    { 0xa4bf8465, 90 },
    { 0x862f8467, 52 },
    { 0xde9f8467, 34 },
    { 0x7c7f8467, 95 },
    { 0x8acf8461, 23 },
    { 0, 0 },
    { 0x10bf846c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bf8471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af8474, 99 },
    { 0x2a0f8475, 66 },
    { 0xd58f8475, 84 },
    { 0x475f8477, 90 },
    { 0x9acf8477, 87 },
    { 0, 0 },
    { 0x9a8f847a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff847d, 52 },
    { 0xbddf847e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f8482, 76 },
    { 0, 0 },
    { 0x305f8484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x325f8487, 98 },
    { 0xcecf8488, 71 },
    { 0x8a6f8489, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf848c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f8493, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cf8496, 95 },
    { 0x8e0f8496, 100 },
    { 0, 0 },
    { 0x784f8499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9f849d, 46 },
    { 0, 0 },
    { 0x397f849f, 87 },
    { 0, 0 },
    { 0xd81f84a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f84a9, 23 },
    { 0x197f84aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2af84af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51af84b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f84bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bf84be, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f84c2, 90 },
    { 0xecbf84c3, 81 },
    { 0x6a9f84c3, 61 },
    { 0xc2ef84c4, 76 },
    { 0, 0 },
    { 0x115f84c7, 98 },
    { 0, 0 },
    { 0x2f3f84c9, 61 },
    { 0xc29f84c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f84ce, 75 },
    { 0, 0 },
    { 0x72bf84d0, 34 },
    { 0xed5f84d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4f84d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f84dc, 61 },
    { 0xc3bf84dd, 23 },
    { 0x706f84de, 95 },
    { 0x8d0f84df, 33 },
    { 0x82cf84e0, 55 },
    { 0x4e1f84df, 98 },
    { 0xf2af84e1, 34 },
    { 0x2f6f84e1, 100 },
    { 0x240f84e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f84e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdf84ea, 33 },
    { 0, 0 },
    { 0x66bf84ec, 85 },
    { 0x552f84ed, 98 },
    { 0xe2df84ee, 98 },
    { 0x539f84ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faf84f2, 68 },
    { 0x157f84f3, 98 },
    { 0xaa4f84f3, 93 },
    { 0xd8df84f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6f84f9, 92 },
    { 0xce9f84f9, 46 },
    { 0, 0 },
    { 0x91ef84fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88f8503, 87 },
    { 0xe9f8504, 43 },
    { 0x3a7f8503, 96 },
    { 0x8d3f8503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef850c, 33 },
    { 0x62f850d, 34 },
    { 0xd56f850c, 23 },
    { 0x8ef850f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241f8514, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62f8518, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f851b, 98 },
    { 0x44ff851b, 97 },
    { 0xc7ef851b, 61 },
    { 0x494f851b, 98 },
    { 0xe74f851c, 91 },
    { 0x2f9f851e, 85 },
    { 0x2e9f8521, 23 },
    { 0x26bf8522, 93 },
    { 0x5f9f8523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf8526, 95 },
    { 0xc2ef8526, 35 },
    { 0xcf9f8528, 55 },
    { 0xaaaf8528, 90 },
    { 0xd5ef852a, 98 },
    { 0xba3f8529, 99 },
    { 0, 0 },
    { 0x58af852d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27f8532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff8540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8545, 61 },
    { 0, 0 },
    { 0x7f6f8547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cf854b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f8550, 80 },
    { 0xce2f8551, 61 },
    { 0, 0 },
    { 0x38f8553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8556, 68 },
    { 0xb9af8557, 95 },
    { 0xff2f8557, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410f855c, 55 },
    { 0, 0 },
    { 0xc6bf855e, 43 },
    { 0x5ccf855f, 33 },
    { 0x7fef8560, 95 },
    { 0x8e8f8560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf8564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f8567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52f856d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ef8571, 66 },
    { 0x2df8571, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff8576, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4af857c, 61 },
    { 0xec9f857c, 36 },
    { 0xc8ff857e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231f8585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f858b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28af8596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff8599, 23 },
    { 0x2d8f859a, 61 },
    { 0x4e1f859b, 61 },
    { 0x872f859c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f85ae, 46 },
    { 0x55cf85af, 49 },
    { 0xae0f85b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdf85b3, 93 },
    { 0xa69f85b4, 90 },
    { 0x7f6f85b4, 55 },
    { 0x62cf85b6, 87 },
    { 0xa0cf85b7, 90 },
    { 0, 0 },
    { 0x70af85b9, 33 },
    { 0x9f3f85ba, 52 },
    { 0x41ef85bb, 98 },
    { 0xcfef85ba, 61 },
    { 0, 0 },
    { 0xb69f85be, 33 },
    { 0x20af85be, 93 },
    { 0x99bf85be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f85c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62f85cc, 69 },
    { 0x972f85cc, 98 },
    { 0xe50f85ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaef85d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59df85da, 90 },
    { 0x1bf85db, 61 },
    { 0x742f85dc, 23 },
    { 0x31df85da, 67 },
    { 0, 0 },
    { 0x655f85df, 33 },
    { 0x5aaf85e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78f85e5, 93 },
    { 0, 0 },
    { 0x41af85e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf85ea, 23 },
    { 0, 0 },
    { 0x771f85ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5f85f3, 95 },
    { 0, 0 },
    { 0xc94f85f5, 33 },
    { 0xf84f85f6, 61 },
    { 0x921f85f6, 61 },
    { 0xfe7f85f8, 76 },
    { 0x32f85f8, 84 },
    { 0x992f85f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x537f85fd, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bf8600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10f8607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df860a, 52 },
    { 0xb93f860b, 55 },
    { 0xcc7f860c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229f8612, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f8618, 90 },
    { 0, 0 },
    { 0x379f861a, 76 },
    { 0, 0 },
    { 0x8fbf861c, 33 },
    { 0xb81f861d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f8623, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38f8626, 61 },
    { 0xeff8626, 69 },
    { 0xba1f8628, 66 },
    { 0x83ef8627, 55 },
    { 0x749f862a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f8632, 40 },
    { 0xad3f8632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df8636, 95 },
    { 0x503f8637, 61 },
    { 0, 0 },
    { 0xc8ff8639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15f863d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bf8640, 23 },
    { 0, 0 },
    { 0x42f8642, 23 },
    { 0xccaf8642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf8646, 61 },
    { 0xa29f8647, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412f864e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf8651, 90 },
    { 0x278f8652, 77 },
    { 0x50af8653, 83 },
    { 0, 0 },
    { 0x1c5f8655, 43 },
    { 0x6bef8656, 33 },
    { 0, 0 },
    { 0xa0bf8658, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149f865f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f8662, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867f866c, 87 },
    { 0, 0 },
    { 0x113f866e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577f8672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03f8679, 93 },
    { 0x46ef867a, 34 },
    { 0xe2bf8679, 90 },
    { 0xdf7f867a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x202f867f, 95 },
    { 0x6e1f867f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bff8685, 55 },
    { 0x770f8686, 88 },
    { 0, 0 },
    { 0xf6ff8688, 98 },
    { 0, 0 },
    { 0xb48f868a, 33 },
    { 0, 0 },
    { 0x209f868c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bf8690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f8693, 98 },
    { 0, 0 },
    { 0x576f8695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bf869a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f86a0, 33 },
    { 0, 0 },
    { 0xa53f86a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f86a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ef86aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76f86ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5f86b0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f86b4, 91 },
    { 0, 0 },
    { 0x224f86b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f86bd, 91 },
    { 0xd69f86be, 95 },
    { 0x146f86be, 75 },
    { 0xc1af86c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x48df86c3, 55 },
    { 0xb9ef86c3, 48 },
    { 0, 0 },
    { 0xa5f86c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaf86c9, 90 },
    { 0x91df86c9, 98 },
    { 0, 0 },
    { 0x424f86cc, 95 },
    { 0x402f86cd, 33 },
    { 0xe60f86ce, 95 },
    { 0xcaff86cf, 80 },
    { 0x1ccf86cf, 36 },
    { 0xa53f86d0, 34 },
    { 0x9bcf86d2, 23 },
    { 0x33ff86d2, 100 },
    { 0x9bf86d4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454f86dc, 90 },
    { 0, 0 },
    { 0xbc4f86de, 85 },
    { 0xe05f86df, 43 },
    { 0, 0 },
    { 0xa57f86e1, 98 },
    { 0x15ef86e2, 95 },
    { 0xf9df86e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41f86ee, 98 },
    { 0xe01f86ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef86f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8700, 83 },
    { 0x129f8701, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af8705, 33 },
    { 0xf5df8705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbef870c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bf8710, 17 },
    { 0xdb4f8711, 23 },
    { 0xdfef8712, 23 },
    { 0, 0 },
    { 0xfeff8714, 46 },
    { 0, 0 },
    { 0xd40f8716, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f871d, 91 },
    { 0x920f871d, 43 },
    { 0, 0 },
    { 0xbc3f8720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f8723, 48 },
    { 0xa87f8724, 76 },
    { 0x3fbf8725, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f8728, 87 },
    { 0xbdaf8729, 55 },
    { 0, 0 },
    { 0x9c1f872b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf8730, 87 },
    { 0xe07f8730, 93 },
    { 0x619f8730, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3f8735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78f873b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df8742, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3f8747, 61 },
    { 0xab0f8748, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x173f874b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaf874e, 90 },
    { 0xcd0f874f, 23 },
    { 0, 0 },
    { 0x49df8751, 43 },
    { 0, 0 },
    { 0xd8af8753, 33 },
    { 0, 0 },
    { 0x7c7f8755, 95 },
    { 0x28ff8756, 52 },
    { 0xd22f8757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882f875c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf8763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x634f8766, 90 },
    { 0xa52f8767, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344f876b, 40 },
    { 0, 0 },
    { 0x4e0f876d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff8771, 55 },
    { 0x91f8772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69f8775, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26f877a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1f877d, 43 },
    { 0xb97f877d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbf8788, 76 },
    { 0x95f8789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804f878d, 93 },
    { 0x10cf878e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f8792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ff879b, 61 },
    { 0xf4f879b, 90 },
    { 0x729f879c, 17 },
    { 0xbcdf879e, 100 },
    { 0x337f879f, 43 },
    { 0x9c9f879e, 90 },
    { 0x8eff87a1, 43 },
    { 0x495f87a1, 90 },
    { 0, 0 },
    { 0xc9f87a4, 84 },
    { 0, 0 },
    { 0xa48f87a6, 66 },
    { 0x506f87a7, 93 },
    { 0xaccf87a8, 23 },
    { 0, 0 },
    { 0xbf1f87aa, 33 },
    { 0x208f87ab, 88 },
    { 0, 0 },
    { 0x41ff87ad, 33 },
    { 0x4e4f87ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bf87b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f87b8, 100 },
    { 0, 0 },
    { 0xe2ff87ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf87bd, 33 },
    { 0xaf87bd, 93 },
    { 0xe48f87be, 95 },
    { 0, 0 },
    { 0xa75f87c1, 88 },
    { 0xeef87c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f87c7, 95 },
    { 0xfd1f87c8, 33 },
    { 0xab2f87c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9def87cc, 43 },
    { 0, 0 },
    { 0xe96f87ce, 85 },
    { 0x235f87cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f87d2, 98 },
    { 0x555f87d3, 67 },
    { 0x294f87d4, 61 },
    { 0, 0 },
    { 0xd8f87d6, 99 },
    { 0xd02f87d6, 66 },
    { 0x75ff87d8, 55 },
    { 0x982f87d9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df87dd, 33 },
    { 0x1e8f87de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf87e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf87e9, 98 },
    { 0, 0 },
    { 0x37bf87eb, 95 },
    { 0xa04f87eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32f87f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15f87f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eff87f7, 23 },
    { 0xdef87f8, 34 },
    { 0xe83f87f9, 40 },
    { 0x9dcf87fa, 98 },
    { 0xb4ff87fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x365f87fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f8803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af8809, 38 },
    { 0x27cf880a, 91 },
    { 0x567f880b, 90 },
    { 0x76ff880a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2f8810, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ef8813, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384f8818, 75 },
    { 0xfbcf8819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cf881d, 95 },
    { 0x66ef881e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f882a, 95 },
    { 0x385f882b, 84 },
    { 0x9adf882a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf8831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff8835, 93 },
    { 0x62cf8835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32f883e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61f8843, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84f8846, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f884b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cf8851, 55 },
    { 0, 0 },
    { 0x92cf8853, 33 },
    { 0x9eff8854, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f8858, 95 },
    { 0x89ff8859, 98 },
    { 0xc0ef885a, 23 },
    { 0x989f885a, 61 },
    { 0, 0 },
    { 0x27ff885d, 21 },
    { 0xd7af885d, 55 },
    { 0, 0 },
    { 0xc28f8860, 55 },
    { 0x44df8860, 90 },
    { 0xe88f8860, 76 },
    { 0xe9f8863, 98 },
    { 0xe85f8863, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643f886c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f8879, 90 },
    { 0x35f887a, 65 },
    { 0x2a7f887b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f887f, 61 },
    { 0, 0 },
    { 0x2f6f8881, 88 },
    { 0x3f1f8881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f8887, 23 },
    { 0xabef8888, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdf888c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af8892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381f8897, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8df88a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ff88a7, 67 },
    { 0, 0 },
    { 0xb2bf88a9, 33 },
    { 0x169f88aa, 87 },
    { 0x4a4f88ab, 23 },
    { 0x3a0f88ac, 98 },
    { 0xce3f88ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ef88b0, 95 },
    { 0, 0 },
    { 0xd91f88b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f88b7, 93 },
    { 0xce6f88b8, 77 },
    { 0x466f88b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x832f88bc, 34 },
    { 0, 0 },
    { 0x19df88be, 61 },
    { 0, 0 },
    { 0x13ef88c0, 90 },
    { 0xe49f88c0, 76 },
    { 0x6caf88c2, 33 },
    { 0, 0 },
    { 0x9cff88c4, 87 },
    { 0x373f88c5, 33 },
    { 0, 0 },
    { 0x559f88c7, 43 },
    { 0x947f88c7, 33 },
    { 0xe3ef88c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ff88cc, 85 },
    { 0, 0 },
    { 0xebcf88ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbf88d1, 33 },
    { 0xd1f88d1, 23 },
    { 0x185f88d3, 95 },
    { 0x2cff88d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f88d7, 95 },
    { 0x823f88d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff88e6, 80 },
    { 0xe9df88e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cf88ea, 4 },
    { 0, 0 },
    { 0xe09f88ec, 90 },
    { 0x86ef88ed, 34 },
    { 0xa26f88ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f88f1, 69 },
    { 0xb36f88f2, 90 },
    { 0xb3ef88f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10f88f6, 23 },
    { 0xeecf88f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88fb, 46 },
    { 0x1f3f88fc, 80 },
    { 0, 0 },
    { 0x758f88fe, 40 },
    { 0xb23f88fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404f8906, 23 },
    { 0x70f8907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2f890d, 98 },
    { 0, 0 },
    { 0x7d4f890f, 33 },
    { 0x57cf8910, 23 },
    { 0xef7f8911, 68 },
    { 0x1acf8912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f8916, 76 },
    { 0xcc7f8916, 76 },
    { 0x7e6f8918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df891f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf8925, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x713f8928, 95 },
    { 0xaf6f8929, 46 },
    { 0xa68f892a, 46 },
    { 0x8fff892b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf8930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f8934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef8939, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1af893f, 90 },
    { 0xaf0f8940, 23 },
    { 0, 0 },
    { 0x901f8942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df8945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f8949, 44 },
    { 0x2e1f894a, 90 },
    { 0xccaf894a, 98 },
    { 0x16f894a, 76 },
    { 0, 0 },
    { 0xa4f894e, 90 },
    { 0x261f894f, 91 },
    { 0x71f894f, 61 },
    { 0x1aef8951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8f895a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6f895f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f8962, 87 },
    { 0, 0 },
    { 0x62ef8964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f8968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabf896b, 90 },
    { 0, 0 },
    { 0xf93f896d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8970, 23 },
    { 0x52ef8971, 98 },
    { 0, 0 },
    { 0x686f8973, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef897c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27f897f, 98 },
    { 0xa9ff897f, 23 },
    { 0x9d9f8981, 77 },
    { 0xe84f897f, 98 },
    { 0x906f8983, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f8987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f898a, 95 },
    { 0x93bf898a, 52 },
    { 0, 0 },
    { 0x8a1f898d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f8994, 55 },
    { 0, 0 },
    { 0x980f8996, 91 },
    { 0x1fcf8996, 17 },
    { 0x377f8996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f89a7, 78 },
    { 0x227f89a7, 76 },
    { 0, 0 },
    { 0x4cff89aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9df89b0, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f89b8, 80 },
    { 0x647f89b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867f89bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf89c0, 33 },
    { 0, 0 },
    { 0x2daf89c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1f89c7, 76 },
    { 0xef9f89c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f89ce, 82 },
    { 0, 0 },
    { 0x9d4f89d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af89d4, 61 },
    { 0, 0 },
    { 0x46ef89d6, 90 },
    { 0xaaff89d7, 67 },
    { 0x626f89d7, 38 },
    { 0, 0 },
    { 0x2eef89da, 78 },
    { 0x1a6f89da, 98 },
    { 0xb1bf89db, 40 },
    { 0, 0 },
    { 0x6ecf89de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf89e4, 33 },
    { 0x3bff89e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df89ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09f89f0, 61 },
    { 0xd03f89f1, 96 },
    { 0x3bdf89f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611f89f6, 91 },
    { 0xf1df89f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bf89fe, 61 },
    { 0x33df89ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578f8a03, 61 },
    { 0xb82f8a04, 93 },
    { 0x763f8a05, 76 },
    { 0, 0 },
    { 0xcf9f8a07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cf8a0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff8a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbf8a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf8a1e, 67 },
    { 0x534f8a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3f8a25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adf8a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef8a2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf8a33, 95 },
    { 0, 0 },
    { 0xcaff8a35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8a3c, 66 },
    { 0x53ff8a3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef8a42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f8a45, 84 },
    { 0x508f8a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bf8a49, 90 },
    { 0x587f8a49, 80 },
    { 0x353f8a4b, 86 },
    { 0x143f8a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff8a50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f8a5e, 61 },
    { 0xc38f8a5f, 34 },
    { 0x33ef8a5f, 55 },
    { 0x9e9f8a5f, 56 },
    { 0xf28f8a62, 90 },
    { 0xb64f8a60, 43 },
    { 0x414f8a64, 55 },
    { 0x52df8a64, 98 },
    { 0x4c3f8a65, 33 },
    { 0, 0 },
    { 0xf70f8a68, 97 },
    { 0x2e5f8a68, 61 },
    { 0xa61f8a6a, 46 },
    { 0x850f8a6b, 90 },
    { 0x7fdf8a6c, 23 },
    { 0, 0 },
    { 0x82df8a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5f8a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baf8a7a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6f8a7d, 33 },
    { 0x975f8a7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237f8a84, 23 },
    { 0xf51f8a85, 76 },
    { 0x1df8a86, 91 },
    { 0x973f8a87, 43 },
    { 0xeacf8a88, 23 },
    { 0x847f8a86, 33 },
    { 0, 0 },
    { 0xc28f8a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f8a8e, 64 },
    { 0x683f8a8f, 61 },
    { 0, 0 },
    { 0x2e1f8a91, 61 },
    { 0x7faf8a92, 84 },
    { 0xd43f8a93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f8a97, 55 },
    { 0x7c4f8a97, 33 },
    { 0, 0 },
    { 0xf70f8a9a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcf8a9f, 43 },
    { 0, 0 },
    { 0x558f8aa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f8aa4, 90 },
    { 0x359f8aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f8aac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ef8ab1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46f8ab7, 68 },
    { 0, 0 },
    { 0x9fff8ab9, 98 },
    { 0x3c2f8aba, 52 },
    { 0x459f8aba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40f8ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8ac5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff8acb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f8ace, 23 },
    { 0x744f8acf, 55 },
    { 0xfeef8acf, 66 },
    { 0x868f8ad1, 17 },
    { 0xea0f8ad2, 52 },
    { 0xe5ef8ad3, 91 },
    { 0x32f8ad0, 40 },
    { 0x45ef8ad5, 38 },
    { 0xb24f8ad6, 23 },
    { 0, 0 },
    { 0xebaf8ad8, 90 },
    { 0x644f8ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbf8ae0, 33 },
    { 0x61bf8ae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45df8ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f8ae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8aea, 78 },
    { 0xb14f8aeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cf8aef, 67 },
    { 0x5aaf8aef, 34 },
    { 0xbd8f8af1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6f8af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af8afc, 90 },
    { 0xef5f8afc, 33 },
    { 0xf02f8afe, 99 },
    { 0, 0 },
    { 0x684f8b00, 98 },
    { 0xe43f8b00, 61 },
    { 0x914f8b00, 61 },
    { 0xc07f8b03, 23 },
    { 0, 0 },
    { 0xab1f8b05, 95 },
    { 0xaa7f8b06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f8b0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaf8b14, 55 },
    { 0x9eef8b15, 98 },
    { 0x667f8b16, 90 },
    { 0, 0 },
    { 0x20f8b18, 23 },
    { 0x9a9f8b19, 46 },
    { 0, 0 },
    { 0xde0f8b1b, 100 },
    { 0xf19f8b1b, 98 },
    { 0, 0 },
    { 0x4bff8b1e, 33 },
    { 0, 0 },
    { 0x8b9f8b20, 23 },
    { 0x266f8b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2f8b24, 43 },
    { 0, 0 },
    { 0xffdf8b26, 61 },
    { 0xbb1f8b27, 85 },
    { 0xc4ef8b28, 83 },
    { 0x907f8b29, 95 },
    { 0x390f8b26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f8b2d, 84 },
    { 0xa8bf8b2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f8b31, 98 },
    { 0xed0f8b31, 90 },
    { 0, 0 },
    { 0xccaf8b34, 90 },
    { 0x11ef8b35, 98 },
    { 0x681f8b36, 33 },
    { 0x5cef8b37, 90 },
    { 0xfddf8b36, 23 },
    { 0, 0 },
    { 0xa34f8b3a, 99 },
    { 0xd31f8b3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff8b3e, 76 },
    { 0x131f8b3f, 33 },
    { 0x7d1f8b3f, 52 },
    { 0x7f4f8b3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf8b44, 23 },
    { 0x105f8b45, 100 },
    { 0, 0 },
    { 0x7fdf8b47, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aef8b4a, 59 },
    { 0x294f8b4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225f8b4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0af8b53, 23 },
    { 0xc0ff8b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b57, 98 },
    { 0, 0 },
    { 0x36f8b59, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f8b5e, 93 },
    { 0xdbbf8b5e, 81 },
    { 0, 0 },
    { 0xabf8b61, 90 },
    { 0x95af8b61, 23 },
    { 0xa0df8b62, 98 },
    { 0, 0 },
    { 0xeeef8b65, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ef8b68, 69 },
    { 0, 0 },
    { 0x25f8b6a, 66 },
    { 0xff4f8b6b, 81 },
    { 0x661f8b6c, 68 },
    { 0xc4df8b6a, 98 },
    { 0x13cf8b6e, 76 },
    { 0x8f3f8b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912f8b7c, 46 },
    { 0x601f8b7d, 43 },
    { 0, 0 },
    { 0x5eff8b7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef8b83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f8b87, 43 },
    { 0, 0 },
    { 0xf44f8b89, 33 },
    { 0x307f8b89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x906f8b8d, 33 },
    { 0xfc4f8b8e, 34 },
    { 0xb6ef8b8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78f8b92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59f8b99, 55 },
    { 0, 0 },
    { 0x1bff8b9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bf8ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8f8bac, 33 },
    { 0, 0 },
    { 0xdadf8bae, 90 },
    { 0x92ef8baf, 75 },
    { 0x658f8bb0, 43 },
    { 0x80ff8bb1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ff8bba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f8bc0, 36 },
    { 0xad3f8bc1, 81 },
    { 0xa05f8bc2, 76 },
    { 0x8b4f8bc3, 33 },
    { 0, 0 },
    { 0x10ff8bc5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccf8bca, 84 },
    { 0xa39f8bca, 40 },
    { 0xc68f8bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0f8bd3, 90 },
    { 0xe21f8bd3, 33 },
    { 0xa2df8bd5, 84 },
    { 0, 0 },
    { 0x4ecf8bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f8bde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f8be3, 66 },
    { 0x996f8be4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ff8be8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf8beb, 61 },
    { 0xa07f8bec, 76 },
    { 0xbdaf8bed, 56 },
    { 0, 0 },
    { 0x6cbf8bef, 23 },
    { 0, 0 },
    { 0x53f8bf1, 33 },
    { 0, 0 },
    { 0xc33f8bf3, 98 },
    { 0, 0 },
    { 0x8aff8bf5, 76 },
    { 0x31f8bf6, 66 },
    { 0x6eef8bf7, 61 },
    { 0, 0 },
    { 0x68ff8bf9, 55 },
    { 0, 0 },
    { 0xbf0f8bfb, 99 },
    { 0x620f8bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f8c00, 99 },
    { 0x877f8c01, 23 },
    { 0x9c0f8c01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476f8c0b, 90 },
    { 0xbcdf8c0b, 43 },
    { 0, 0 },
    { 0x188f8c0e, 85 },
    { 0x2d5f8c0f, 48 },
    { 0, 0 },
    { 0x583f8c11, 90 },
    { 0x3fcf8c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904f8c17, 76 },
    { 0xcecf8c18, 61 },
    { 0x354f8c19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f8c22, 66 },
    { 0, 0 },
    { 0xa89f8c24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ff8c29, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39df8c2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736f8c35, 90 },
    { 0x939f8c36, 98 },
    { 0xa7ff8c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef8c3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ff8c3f, 98 },
    { 0x7b0f8c40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f8c43, 99 },
    { 0, 0 },
    { 0xffff8c45, 55 },
    { 0x154f8c45, 98 },
    { 0x9a6f8c45, 95 },
    { 0xc96f8c47, 85 },
    { 0x115f8c49, 17 },
    { 0x95f8c49, 98 },
    { 0, 0 },
    { 0x1d4f8c4c, 33 },
    { 0xffff8c4c, 99 },
    { 0x86cf8c4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaf8c56, 33 },
    { 0x388f8c57, 87 },
    { 0x415f8c56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0f8c5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f8c60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33f8c65, 33 },
    { 0, 0 },
    { 0x74ef8c67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef8c6b, 33 },
    { 0, 0 },
    { 0x1c1f8c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8c74, 86 },
    { 0x42bf8c74, 61 },
    { 0xe65f8c75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8f8c7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8c82, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07f8c85, 61 },
    { 0xe5bf8c86, 76 },
    { 0xe97f8c85, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65f8c8e, 98 },
    { 0xa61f8c8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f8c92, 76 },
    { 0xed4f8c93, 95 },
    { 0x53cf8c93, 91 },
    { 0xe7ff8c95, 90 },
    { 0x1aaf8c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514f8c9c, 61 },
    { 0, 0 },
    { 0xc3cf8c9e, 33 },
    { 0xac5f8c9e, 23 },
    { 0x970f8ca0, 85 },
    { 0x8e7f8ca0, 61 },
    { 0x11f8ca2, 55 },
    { 0xa65f8ca3, 33 },
    { 0, 0 },
    { 0xe7bf8ca5, 96 },
    { 0x25bf8ca6, 90 },
    { 0xec4f8ca7, 34 },
    { 0x93df8ca8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bf8cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef8cb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529f8cb7, 23 },
    { 0x2dcf8cb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff8cbc, 91 },
    { 0x7f1f8cbc, 77 },
    { 0, 0 },
    { 0x4a9f8cbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceff8cc3, 61 },
    { 0x8cef8cc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6f8cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f8cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99f8ccf, 90 },
    { 0x7c5f8ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff8cd3, 90 },
    { 0xf49f8cd3, 61 },
    { 0x575f8cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95f8cd8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850f8cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334f8ce3, 66 },
    { 0x763f8ce3, 88 },
    { 0x52f8ce5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0af8ce9, 46 },
    { 0xa3df8cea, 33 },
    { 0x8e1f8ceb, 52 },
    { 0x534f8cec, 76 },
    { 0x630f8ceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2f8cf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f8cfb, 17 },
    { 0x5c8f8cfc, 90 },
    { 0xbbbf8cfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f8d03, 82 },
    { 0xdf2f8d03, 76 },
    { 0x1cbf8d05, 24 },
    { 0, 0 },
    { 0xb79f8d07, 90 },
    { 0x40ff8d08, 47 },
    { 0x313f8d09, 85 },
    { 0x712f8d09, 98 },
    { 0x560f8d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68f8d13, 90 },
    { 0xfd5f8d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaf8d1b, 90 },
    { 0x58af8d1b, 75 },
    { 0, 0 },
    { 0x9bff8d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711f8d22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f8d26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f8d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdef8d2f, 43 },
    { 0x4e0f8d2f, 33 },
    { 0x38af8d31, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f8d35, 33 },
    { 0xf68f8d35, 76 },
    { 0x1ccf8d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daf8d3b, 98 },
    { 0x3def8d3b, 33 },
    { 0x90ef8d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef8d41, 84 },
    { 0xed8f8d42, 93 },
    { 0xeecf8d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f8d4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650f8d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45f8d56, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f8d59, 77 },
    { 0xfaf8d5a, 33 },
    { 0, 0 },
    { 0x934f8d5c, 95 },
    { 0x785f8d5d, 55 },
    { 0x8e2f8d5e, 23 },
    { 0xc1bf8d5c, 90 },
    { 0xcd8f8d5c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59f8d63, 90 },
    { 0x143f8d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78f8d6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9f8d70, 81 },
    { 0x660f8d70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f8d74, 81 },
    { 0, 0 },
    { 0xe23f8d76, 77 },
    { 0xb70f8d77, 66 },
    { 0xec0f8d77, 34 },
    { 0x17df8d79, 90 },
    { 0x7df8d7a, 76 },
    { 0xe56f8d78, 33 },
    { 0x3cef8d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853f8d89, 23 },
    { 0xba7f8d8a, 61 },
    { 0, 0 },
    { 0x3aef8d8c, 90 },
    { 0, 0 },
    { 0x978f8d8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af8d92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf8d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f8d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf8da1, 94 },
    { 0xbbbf8da2, 33 },
    { 0x5a9f8da3, 98 },
    { 0xe73f8da4, 33 },
    { 0, 0 },
    { 0x8daf8da6, 88 },
    { 0xfd7f8da6, 91 },
    { 0xf72f8da8, 33 },
    { 0x1cef8da9, 76 },
    { 0x89df8daa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6f8daf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bf8dba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f8dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f8dcb, 33 },
    { 0x84ff8dcb, 55 },
    { 0x534f8dcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f8dd3, 23 },
    { 0x631f8dd4, 66 },
    { 0x163f8dd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf8ddb, 55 },
    { 0xbe0f8ddb, 78 },
    { 0x6f3f8ddd, 61 },
    { 0xd8ff8ddd, 84 },
    { 0x406f8ddf, 78 },
    { 0x382f8de0, 93 },
    { 0, 0 },
    { 0xd19f8de2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf8de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x654f8de8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaef8deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ef8dee, 52 },
    { 0x596f8def, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5f8df5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf8dfc, 46 },
    { 0x85ff8dfd, 34 },
    { 0, 0 },
    { 0xd9df8dff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f8e02, 38 },
    { 0x5c6f8e03, 33 },
    { 0x52ff8e02, 98 },
    { 0x931f8e05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df8e08, 84 },
    { 0, 0 },
    { 0xc2ff8e0a, 23 },
    { 0xc39f8e0b, 26 },
    { 0x778f8e0c, 61 },
    { 0x940f8e0d, 100 },
    { 0xadaf8e0a, 43 },
    { 0, 0 },
    { 0x547f8e10, 86 },
    { 0x117f8e11, 76 },
    { 0xed8f8e11, 23 },
    { 0xb3df8e13, 90 },
    { 0x7c8f8e14, 95 },
    { 0x3dbf8e14, 33 },
    { 0, 0 },
    { 0x983f8e17, 23 },
    { 0, 0 },
    { 0x443f8e19, 75 },
    { 0xdcff8e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0f8e1d, 90 },
    { 0xc24f8e1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bff8e21, 100 },
    { 0x39cf8e21, 91 },
    { 0xa7cf8e23, 61 },
    { 0x2e4f8e24, 55 },
    { 0, 0 },
    { 0x873f8e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f8e2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f8e32, 100 },
    { 0x3d8f8e33, 55 },
    { 0xa3cf8e32, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dff8e37, 90 },
    { 0x27cf8e37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03f8e3d, 77 },
    { 0x80ef8e3e, 66 },
    { 0x38ef8e3f, 33 },
    { 0, 0 },
    { 0x7caf8e41, 90 },
    { 0xfcbf8e42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f8e45, 43 },
    { 0xe4ff8e46, 100 },
    { 0x2e0f8e47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebf8e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f8e5b, 52 },
    { 0x45df8e5b, 85 },
    { 0xc1f8e5d, 90 },
    { 0, 0 },
    { 0xf3df8e5f, 43 },
    { 0x88ef8e5f, 34 },
    { 0xbfcf8e5f, 61 },
    { 0, 0 },
    { 0xba0f8e63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35df8e66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ff8e6b, 85 },
    { 0xf05f8e6c, 78 },
    { 0x35bf8e6c, 33 },
    { 0x493f8e6e, 66 },
    { 0x890f8e6b, 98 },
    { 0, 0 },
    { 0x96df8e71, 33 },
    { 0x71df8e72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df8e7b, 53 },
    { 0, 0 },
    { 0xc21f8e7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf8e83, 33 },
    { 0xff1f8e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93f8e87, 46 },
    { 0xd85f8e87, 61 },
    { 0, 0 },
    { 0xaa2f8e8a, 23 },
    { 0x4a2f8e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775f8e90, 46 },
    { 0x64f8e91, 66 },
    { 0xd27f8e92, 77 },
    { 0x4e1f8e92, 100 },
    { 0xc1ef8e92, 98 },
    { 0x62cf8e93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf8e98, 90 },
    { 0, 0 },
    { 0x5f3f8e9a, 46 },
    { 0xcddf8e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f8e9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f8ea3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f8ea6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bf8eaa, 90 },
    { 0, 0 },
    { 0x4b3f8eac, 33 },
    { 0x6aef8ead, 95 },
    { 0, 0 },
    { 0x200f8eaf, 95 },
    { 0x4eaf8eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x202f8eb3, 76 },
    { 0x30af8eb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8f8eba, 33 },
    { 0, 0 },
    { 0x400f8ebc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ef8ec0, 90 },
    { 0x47df8ec0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f8ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f8ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x896f8ece, 55 },
    { 0, 0 },
    { 0x693f8ed0, 91 },
    { 0x1cef8ed1, 98 },
    { 0, 0 },
    { 0xc88f8ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f8ed6, 93 },
    { 0, 0 },
    { 0x1a3f8ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff8edb, 23 },
    { 0xfa4f8edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f8ee3, 90 },
    { 0x47bf8ee4, 55 },
    { 0x10df8ee4, 98 },
    { 0, 0 },
    { 0xe55f8ee7, 91 },
    { 0, 0 },
    { 0xb77f8ee9, 69 },
    { 0, 0 },
    { 0x1acf8eeb, 99 },
    { 0x2ebf8eec, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881f8ef0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7af8ef8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f8eff, 91 },
    { 0x47af8eff, 95 },
    { 0x1edf8f01, 84 },
    { 0xa04f8f02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1f8f06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20f8f0a, 33 },
    { 0, 0 },
    { 0xa74f8f0c, 97 },
    { 0xd05f8f0c, 95 },
    { 0xeebf8f0e, 61 },
    { 0, 0 },
    { 0x248f8f10, 33 },
    { 0xf7ef8f11, 91 },
    { 0x845f8f11, 61 },
    { 0xbb0f8f13, 90 },
    { 0x3abf8f14, 33 },
    { 0x7b2f8f13, 93 },
    { 0x80ef8f16, 94 },
    { 0, 0 },
    { 0x4ef8f18, 55 },
    { 0x2aff8f19, 90 },
    { 0xc3cf8f18, 61 },
    { 0x5b4f8f19, 33 },
    { 0xd06f8f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40df8f22, 36 },
    { 0, 0 },
    { 0xfc3f8f24, 78 },
    { 0x884f8f24, 88 },
    { 0x2cff8f26, 8 },
    { 0x572f8f26, 90 },
    { 0x1b0f8f28, 85 },
    { 0xd26f8f24, 61 },
    { 0xa13f8f2a, 33 },
    { 0x88ff8f2b, 98 },
    { 0x5aaf8f2c, 33 },
    { 0x7fef8f2d, 61 },
    { 0xc4f8f2e, 98 },
    { 0xfaef8f29, 90 },
    { 0x730f8f30, 33 },
    { 0, 0 },
    { 0x271f8f32, 90 },
    { 0xc47f8f33, 33 },
    { 0, 0 },
    { 0x917f8f35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f8f38, 33 },
    { 0x49af8f38, 76 },
    { 0x333f8f39, 90 },
    { 0x2a6f8f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44f8f3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x531f8f41, 90 },
    { 0xccef8f42, 90 },
    { 0x2fcf8f41, 40 },
    { 0x486f8f44, 95 },
    { 0x93ef8f44, 100 },
    { 0xa8f8f44, 92 },
    { 0x743f8f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f8f4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x699f8f4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddf8f52, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f8f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f8f5e, 98 },
    { 0x39cf8f5f, 99 },
    { 0, 0 },
    { 0xb12f8f61, 65 },
    { 0x1f9f8f62, 61 },
    { 0x3d4f8f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51f8f67, 23 },
    { 0x67ff8f68, 33 },
    { 0x373f8f69, 33 },
    { 0x67ef8f6a, 61 },
    { 0xd71f8f6a, 76 },
    { 0xe24f8f6c, 33 },
    { 0xbacf8f69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf8f72, 95 },
    { 0, 0 },
    { 0x42ef8f74, 33 },
    { 0xf55f8f75, 61 },
    { 0x3b4f8f75, 69 },
    { 0, 0 },
    { 0x4a9f8f78, 61 },
    { 0x379f8f78, 76 },
    { 0xf71f8f79, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df8f81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1f8f85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4f8f8a, 98 },
    { 0, 0 },
    { 0xdbdf8f8c, 100 },
    { 0xc55f8f8d, 90 },
    { 0xe35f8f8e, 91 },
    { 0x6f4f8f8e, 93 },
    { 0, 0 },
    { 0x13cf8f91, 61 },
    { 0xf5af8f92, 80 },
    { 0xe36f8f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8f8f97, 43 },
    { 0x69bf8f98, 34 },
    { 0x25df8f99, 46 },
    { 0x9e5f8f99, 23 },
    { 0, 0 },
    { 0x94bf8f9c, 43 },
    { 0x707f8f9d, 23 },
    { 0x1a4f8f9e, 61 },
    { 0xf3bf8f9c, 23 },
    { 0x2f8fa0, 52 },
    { 0xc4af8fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101f8fa5, 83 },
    { 0x4daf8fa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207f8fab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef8fae, 90 },
    { 0x156f8fae, 61 },
    { 0x99ef8fae, 95 },
    { 0x603f8faf, 65 },
    { 0x77f8fb2, 85 },
    { 0x972f8fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f8fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bef8fbc, 33 },
    { 0, 0 },
    { 0xeadf8fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1f8fc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f8fc4, 33 },
    { 0xb21f8fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bf8fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71af8fd2, 66 },
    { 0, 0 },
    { 0x6bff8fd4, 55 },
    { 0, 0 },
    { 0xb11f8fd6, 93 },
    { 0xcedf8fd7, 23 },
    { 0, 0 },
    { 0x325f8fd9, 46 },
    { 0, 0 },
    { 0x203f8fdb, 43 },
    { 0xbdaf8fdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01f8fe4, 90 },
    { 0xdc7f8fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adf8ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbf8ff5, 35 },
    { 0x1a0f8ff5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1f8fff, 95 },
    { 0x41bf9000, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51f9003, 52 },
    { 0x90bf9004, 90 },
    { 0, 0 },
    { 0x961f9006, 43 },
    { 0x652f9007, 23 },
    { 0x710f9007, 43 },
    { 0x67af9009, 60 },
    { 0x204f9009, 98 },
    { 0x8a2f900b, 55 },
    { 0xb77f900c, 95 },
    { 0x801f900d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baf9011, 96 },
    { 0, 0 },
    { 0xd78f9013, 55 },
    { 0xe6af9014, 90 },
    { 0xd69f9015, 93 },
    { 0, 0 },
    { 0x5f9017, 84 },
    { 0x22df9017, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf901b, 76 },
    { 0x732f901c, 66 },
    { 0xb0df901c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f9020, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0f9023, 61 },
    { 0x3fdf9023, 61 },
    { 0xd8bf9025, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f902c, 90 },
    { 0xc64f902d, 52 },
    { 0x823f902e, 17 },
    { 0, 0 },
    { 0x2d1f9030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898f903a, 85 },
    { 0x873f903a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35f903e, 65 },
    { 0, 0 },
    { 0xed3f9040, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f9043, 33 },
    { 0, 0 },
    { 0x8b8f9045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ef904b, 34 },
    { 0x706f904c, 33 },
    { 0x8d7f904b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef9053, 93 },
    { 0, 0 },
    { 0xb16f9055, 23 },
    { 0x18ff9055, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af905c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af9065, 98 },
    { 0, 0 },
    { 0x795f9067, 91 },
    { 0, 0 },
    { 0x392f9069, 76 },
    { 0xe08f906a, 95 },
    { 0, 0 },
    { 0xf68f906c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x737f906f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0f9072, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85f9075, 55 },
    { 0x5def9076, 67 },
    { 0x4c5f9076, 61 },
    { 0x35f9075, 76 },
    { 0x4a5f9079, 78 },
    { 0xac4f9078, 84 },
    { 0x1e2f907b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ef9085, 93 },
    { 0xb2bf9085, 33 },
    { 0xf1ff9087, 61 },
    { 0, 0 },
    { 0x34ff9089, 61 },
    { 0, 0 },
    { 0x5e7f908b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af9092, 33 },
    { 0xe38f9093, 98 },
    { 0x208f9092, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf9097, 78 },
    { 0xe0bf9098, 61 },
    { 0x891f9099, 23 },
    { 0x5f5f9099, 90 },
    { 0xa8af909b, 55 },
    { 0, 0 },
    { 0xa7ef909d, 61 },
    { 0, 0 },
    { 0xbb9f909f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x213f90a2, 61 },
    { 0x145f90a3, 48 },
    { 0x489f90a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x410f90a7, 90 },
    { 0xff9f90a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f90ad, 55 },
    { 0, 0 },
    { 0x2b3f90af, 93 },
    { 0x34ef90b0, 61 },
    { 0xd9af90af, 76 },
    { 0xd3ff90b2, 23 },
    { 0x29df90b3, 90 },
    { 0, 0 },
    { 0x844f90b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf90ba, 90 },
    { 0xb1f90bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x249f90be, 95 },
    { 0x4abf90bf, 33 },
    { 0x3f8f90c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecf90c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23f90cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fff90d2, 43 },
    { 0x84cf90d3, 43 },
    { 0, 0 },
    { 0x47cf90d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f90d9, 90 },
    { 0xcdef90da, 78 },
    { 0xaedf90db, 33 },
    { 0xcd2f90db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328f90e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12f90e5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0f90ec, 90 },
    { 0x792f90ed, 85 },
    { 0x1baf90ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x402f90f1, 67 },
    { 0xea8f90f2, 40 },
    { 0, 0 },
    { 0xedff90f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f90f7, 43 },
    { 0, 0 },
    { 0x705f90f9, 33 },
    { 0x798f90f9, 98 },
    { 0xc3cf90fb, 33 },
    { 0x4e6f90fc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75f90ff, 33 },
    { 0xebaf9100, 90 },
    { 0, 0 },
    { 0x6d7f9102, 66 },
    { 0x361f9103, 90 },
    { 0xed8f9104, 23 },
    { 0xb89f9105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cf9111, 52 },
    { 0, 0 },
    { 0xbcff9113, 33 },
    { 0x5aaf9114, 76 },
    { 0x6bbf9113, 55 },
    { 0x8b9f9116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f9119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f911c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2af9126, 93 },
    { 0xbaff9127, 76 },
    { 0x98f9127, 23 },
    { 0x2c9f9126, 95 },
    { 0, 0 },
    { 0x7c4f912b, 84 },
    { 0, 0 },
    { 0x116f912d, 43 },
    { 0xc32f912e, 76 },
    { 0xc3af912e, 61 },
    { 0, 0 },
    { 0xc91f9131, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6f9135, 55 },
    { 0, 0 },
    { 0xeeff9137, 34 },
    { 0x8b5f9138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468f913d, 33 },
    { 0xdfbf913e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f9141, 23 },
    { 0xb1ef9142, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f9145, 78 },
    { 0xc47f9146, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378f914a, 83 },
    { 0x1f6f914b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34f914e, 95 },
    { 0x69ff914f, 23 },
    { 0x9baf9150, 95 },
    { 0, 0 },
    { 0x2daf9152, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f9159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660f915d, 90 },
    { 0x9adf915e, 55 },
    { 0xd7df915e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f9167, 67 },
    { 0xdabf9168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530f9170, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987f9174, 43 },
    { 0, 0 },
    { 0x490f9176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71df9179, 33 },
    { 0xbf8f917a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdf917d, 90 },
    { 0, 0 },
    { 0xf51f917f, 85 },
    { 0x5a2f9180, 90 },
    { 0x84bf9181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90f9189, 84 },
    { 0, 0 },
    { 0xc34f918b, 23 },
    { 0x9aef918c, 98 },
    { 0x570f918d, 55 },
    { 0xca1f918d, 33 },
    { 0x629f918c, 17 },
    { 0xb9bf918f, 91 },
    { 0xaebf9191, 87 },
    { 0xb66f9191, 90 },
    { 0x753f9192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cf9196, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4f919e, 90 },
    { 0x51ff919f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422f91a3, 66 },
    { 0, 0 },
    { 0x42bf91a5, 80 },
    { 0x110f91a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876f91ab, 61 },
    { 0x6bf91ac, 23 },
    { 0x80bf91ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f91b0, 76 },
    { 0, 0 },
    { 0xdc4f91b2, 84 },
    { 0xb6ff91b3, 88 },
    { 0x81bf91b4, 64 },
    { 0x9a7f91b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f91ba, 83 },
    { 0x30ff91ba, 55 },
    { 0x896f91bc, 38 },
    { 0xfcbf91bc, 90 },
    { 0x72af91bc, 95 },
    { 0xc7f91bf, 90 },
    { 0x296f91be, 33 },
    { 0, 0 },
    { 0xa68f91c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f91c5, 85 },
    { 0x67ff91c5, 84 },
    { 0x930f91c7, 33 },
    { 0, 0 },
    { 0xca9f91c9, 93 },
    { 0xb39f91ca, 17 },
    { 0x1e2f91cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x121f91ce, 99 },
    { 0x161f91cf, 43 },
    { 0xbacf91d0, 43 },
    { 0x36af91d0, 55 },
    { 0, 0 },
    { 0x620f91d3, 90 },
    { 0xf72f91d4, 34 },
    { 0x9eaf91d5, 85 },
    { 0xa53f91d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f91db, 98 },
    { 0, 0 },
    { 0xe66f91dd, 33 },
    { 0, 0 },
    { 0xc37f91df, 76 },
    { 0x9c5f91e0, 33 },
    { 0x60bf91e1, 90 },
    { 0xa7ef91e2, 61 },
    { 0xe9ef91e1, 93 },
    { 0x25f91df, 93 },
    { 0x7fef91e0, 95 },
    { 0x85ef91e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf91e9, 34 },
    { 0, 0 },
    { 0xb5af91eb, 98 },
    { 0xbe8f91ec, 99 },
    { 0x9d2f91eb, 53 },
    { 0x767f91ec, 75 },
    { 0x226f91ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f91f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f91fe, 55 },
    { 0xacef91ff, 98 },
    { 0x98af9200, 33 },
    { 0xc77f9201, 90 },
    { 0x219f91ff, 90 },
    { 0x6d8f9203, 67 },
    { 0x236f9204, 90 },
    { 0, 0 },
    { 0x7a7f9206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff9209, 95 },
    { 0x52f920a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdf920d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaf9210, 96 },
    { 0x41cf9210, 65 },
    { 0x2f9f9210, 95 },
    { 0x4e1f9212, 76 },
    { 0x546f9214, 43 },
    { 0xed5f9215, 61 },
    { 0x79df9215, 80 },
    { 0, 0 },
    { 0x969f9218, 33 },
    { 0x32bf9219, 40 },
    { 0xc9f9219, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f921f, 33 },
    { 0, 0 },
    { 0x4f5f9221, 90 },
    { 0xb88f9221, 23 },
    { 0x631f9223, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef9227, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480f922b, 95 },
    { 0x38cf922b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480f9233, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f9237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fff923b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df9240, 34 },
    { 0x55ef9241, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f9245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf924b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71f9251, 46 },
    { 0x482f9252, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf9256, 33 },
    { 0, 0 },
    { 0x516f9258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74f925c, 34 },
    { 0xdbcf925c, 95 },
    { 0xba6f925e, 90 },
    { 0xe9bf925e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x474f9262, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf926e, 99 },
    { 0, 0 },
    { 0xc09f9270, 55 },
    { 0xa54f9270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f9274, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f9277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eef927d, 34 },
    { 0, 0 },
    { 0x44df927f, 61 },
    { 0x934f9280, 61 },
    { 0x8faf9281, 55 },
    { 0, 0 },
    { 0xee3f9283, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f9287, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbf928f, 61 },
    { 0x2f9f928f, 55 },
    { 0, 0 },
    { 0xa1df9292, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9f9295, 81 },
    { 0x7cff9296, 98 },
    { 0x693f9296, 33 },
    { 0x9fdf9295, 33 },
    { 0x325f9299, 33 },
    { 0x6aff9299, 52 },
    { 0x58df929b, 23 },
    { 0x159f929b, 61 },
    { 0x380f929d, 17 },
    { 0, 0 },
    { 0xcb2f929f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92a4, 98 },
    { 0xd9f92a5, 88 },
    { 0x85df92a4, 98 },
    { 0x46df92a7, 90 },
    { 0xbacf92a7, 52 },
    { 0x5bdf92a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f92af, 90 },
    { 0x56af92b0, 95 },
    { 0x7eff92b1, 36 },
    { 0xcb0f92b2, 34 },
    { 0xec5f92b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f92b9, 33 },
    { 0xce9f92ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f92bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ef92c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df92c4, 33 },
    { 0x9fdf92c4, 90 },
    { 0x3ef92c5, 95 },
    { 0x83bf92c7, 91 },
    { 0x20ef92c8, 87 },
    { 0x875f92c8, 23 },
    { 0xab9f92ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cf92ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x127f92d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4f92d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2f92d8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf92e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f92f6, 90 },
    { 0xef7f92f6, 66 },
    { 0xbbcf92f7, 55 },
    { 0x144f92f9, 61 },
    { 0, 0 },
    { 0xce4f92fb, 17 },
    { 0x7cff92fc, 90 },
    { 0x37cf92fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f9305, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf9308, 76 },
    { 0xa31f9308, 34 },
    { 0x99cf930a, 87 },
    { 0x6e1f930b, 78 },
    { 0, 0 },
    { 0xbcbf930d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f9311, 61 },
    { 0xa6af9312, 93 },
    { 0xd51f9313, 23 },
    { 0, 0 },
    { 0x9c4f9315, 90 },
    { 0x515f9315, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1f931d, 23 },
    { 0xfd7f931d, 33 },
    { 0x6b9f931f, 33 },
    { 0, 0 },
    { 0x6b1f9321, 61 },
    { 0x64af9322, 23 },
    { 0xb57f9322, 75 },
    { 0, 0 },
    { 0x9f0f9325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749f932a, 34 },
    { 0x455f932a, 95 },
    { 0x9b4f932c, 98 },
    { 0x2b3f932c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f9331, 55 },
    { 0xb4ff9332, 90 },
    { 0xca8f9333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff9339, 76 },
    { 0, 0 },
    { 0xef9f933b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef9340, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30f9346, 52 },
    { 0xeb6f9347, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71f934d, 90 },
    { 0xe06f934d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f9351, 23 },
    { 0, 0 },
    { 0xe20f9353, 46 },
    { 0x7f6f9354, 90 },
    { 0x328f9353, 66 },
    { 0x334f9355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f935c, 55 },
    { 0, 0 },
    { 0x877f935e, 43 },
    { 0xd28f935f, 61 },
    { 0xab9f9360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f9364, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25f9369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f936c, 76 },
    { 0, 0 },
    { 0xbb7f936e, 90 },
    { 0x8e1f936e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3f9373, 33 },
    { 0xfe7f9374, 33 },
    { 0x685f9375, 95 },
    { 0xcedf9376, 76 },
    { 0, 0 },
    { 0x8dcf9378, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f937c, 23 },
    { 0x629f937d, 23 },
    { 0x971f937d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f9381, 90 },
    { 0x858f9381, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2f9388, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f938b, 34 },
    { 0x81df938c, 90 },
    { 0xbbbf938d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff9390, 76 },
    { 0x624f9391, 81 },
    { 0x1edf9392, 78 },
    { 0x6f1f9393, 23 },
    { 0x71ff9391, 61 },
    { 0xfcdf9392, 61 },
    { 0x62f9396, 77 },
    { 0x347f9393, 61 },
    { 0xaa2f9398, 77 },
    { 0x66f9399, 37 },
    { 0, 0 },
    { 0xd5ef939b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x118f939e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bff93a1, 65 },
    { 0x980f93a2, 33 },
    { 0x697f93a3, 46 },
    { 0x1fbf93a4, 23 },
    { 0xb71f93a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bf93a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567f93b4, 33 },
    { 0xdfaf93b5, 77 },
    { 0xd86f93b5, 90 },
    { 0x7f4f93b7, 68 },
    { 0xee1f93b4, 52 },
    { 0, 0 },
    { 0xb8cf93ba, 98 },
    { 0xc01f93bb, 93 },
    { 0x30af93ba, 95 },
    { 0xd37f93bd, 61 },
    { 0x6f9f93be, 66 },
    { 0, 0 },
    { 0x266f93c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x84af93c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df93c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daf93ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22df93d1, 85 },
    { 0, 0 },
    { 0xb6cf93d3, 43 },
    { 0, 0 },
    { 0x6cf93d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bf93db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04f93df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104f93e3, 33 },
    { 0xb3bf93e4, 61 },
    { 0xca1f93e5, 33 },
    { 0x361f93e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9f93ea, 98 },
    { 0xa98f93ea, 33 },
    { 0x806f93ec, 61 },
    { 0xc48f93ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x959f93f0, 33 },
    { 0x962f93f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f93f6, 34 },
    { 0, 0 },
    { 0xd3af93f8, 46 },
    { 0x729f93f8, 52 },
    { 0, 0 },
    { 0xedff93fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x100f93fe, 95 },
    { 0, 0 },
    { 0x8e2f9400, 100 },
    { 0xd84f9401, 78 },
    { 0x689f9402, 59 },
    { 0x449f9403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f940b, 74 },
    { 0x6eff940c, 99 },
    { 0x1ebf940c, 66 },
    { 0, 0 },
    { 0x704f940f, 98 },
    { 0xa51f9410, 95 },
    { 0xcb1f9411, 91 },
    { 0xc93f9410, 61 },
    { 0x8bbf9412, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64df9417, 61 },
    { 0xb42f9417, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71f941b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f9422, 67 },
    { 0, 0 },
    { 0x899f9424, 90 },
    { 0, 0 },
    { 0x90af9426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f942c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baf9433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f943d, 23 },
    { 0, 0 },
    { 0x101f943f, 85 },
    { 0x328f943f, 67 },
    { 0x657f9441, 43 },
    { 0x416f9441, 55 },
    { 0xee6f9443, 34 },
    { 0x13cf9444, 95 },
    { 0xbaaf9444, 66 },
    { 0x20ef9441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45f944c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff9452, 46 },
    { 0x932f9453, 33 },
    { 0xe87f9453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf945a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307f9460, 99 },
    { 0xc56f9461, 75 },
    { 0xae1f9461, 90 },
    { 0xe81f9462, 90 },
    { 0, 0 },
    { 0xc06f9465, 52 },
    { 0x993f9466, 61 },
    { 0x795f9467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f946e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf9473, 61 },
    { 0xf77f9474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f9479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769f947e, 3 },
    { 0xd35f947f, 95 },
    { 0x1d5f947f, 80 },
    { 0xac2f9481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcf9486, 33 },
    { 0x2c3f9487, 87 },
    { 0x7a0f9486, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6f948d, 75 },
    { 0x7cbf948e, 84 },
    { 0x4ef948d, 98 },
    { 0, 0 },
    { 0xeb1f9491, 100 },
    { 0x30df9492, 55 },
    { 0x941f9491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef9498, 61 },
    { 0, 0 },
    { 0xd95f949a, 46 },
    { 0xf2bf949b, 43 },
    { 0xee4f949c, 61 },
    { 0xe92f949b, 23 },
    { 0xb40f949e, 34 },
    { 0, 0 },
    { 0xd7ef94a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f94a3, 46 },
    { 0xed2f94a4, 90 },
    { 0x5a9f94a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f94a8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f94ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803f94b0, 81 },
    { 0, 0 },
    { 0x670f94b2, 40 },
    { 0x5bcf94b3, 33 },
    { 0xa4ef94b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62df94bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff94c2, 23 },
    { 0xecef94c3, 61 },
    { 0xf21f94c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecf94c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff94cc, 23 },
    { 0x42f94cc, 93 },
    { 0, 0 },
    { 0xc9f94cf, 43 },
    { 0, 0 },
    { 0x6a6f94d1, 84 },
    { 0x44ff94d1, 33 },
    { 0xa95f94d3, 61 },
    { 0x8e4f94d4, 90 },
    { 0x6a4f94d5, 76 },
    { 0x859f94d3, 34 },
    { 0xa8f94d3, 77 },
    { 0xa34f94d7, 33 },
    { 0x324f94d9, 90 },
    { 0xb5ff94d9, 33 },
    { 0xd03f94d9, 49 },
    { 0x513f94dc, 23 },
    { 0x28cf94dc, 90 },
    { 0xad1f94da, 43 },
    { 0x1d2f94dd, 98 },
    { 0x77df94e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4f94ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38f94ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x629f94f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f94f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcf94fc, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf94ff, 52 },
    { 0x5bf94ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0f9503, 92 },
    { 0, 0 },
    { 0xb9ff9505, 43 },
    { 0x75bf9506, 80 },
    { 0xedbf9507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dff950e, 101 },
    { 0, 0 },
    { 0x294f9510, 61 },
    { 0, 0 },
    { 0x36df9512, 46 },
    { 0, 0 },
    { 0xa80f9514, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f9522, 90 },
    { 0x73bf9523, 33 },
    { 0x543f9523, 34 },
    { 0x67cf9525, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904f952a, 55 },
    { 0xfc4f952b, 33 },
    { 0x41f952c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ef9530, 91 },
    { 0x250f9531, 90 },
    { 0xcacf9532, 93 },
    { 0x534f9532, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26f9536, 84 },
    { 0, 0 },
    { 0x7b8f9538, 61 },
    { 0, 0 },
    { 0xcdcf953a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf953d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf9541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12df9547, 99 },
    { 0, 0 },
    { 0xc40f9549, 99 },
    { 0xfb2f954a, 23 },
    { 0x2eaf954b, 90 },
    { 0xe26f954b, 55 },
    { 0, 0 },
    { 0x671f954e, 61 },
    { 0, 0 },
    { 0x686f9550, 33 },
    { 0xab1f9551, 61 },
    { 0xf7f9550, 81 },
    { 0xe4cf9553, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x218f9556, 33 },
    { 0x47cf9557, 100 },
    { 0x591f9558, 66 },
    { 0, 0 },
    { 0xfe5f955a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41f955e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff9565, 36 },
    { 0, 0 },
    { 0x1a2f9567, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0f956d, 61 },
    { 0x3f6f956d, 90 },
    { 0x894f956d, 43 },
    { 0x8eff9570, 91 },
    { 0x2c0f9570, 76 },
    { 0, 0 },
    { 0xf45f9573, 55 },
    { 0, 0 },
    { 0xf9575, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0f9579, 73 },
    { 0, 0 },
    { 0x4c8f957b, 33 },
    { 0x6f7f957b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f9581, 61 },
    { 0, 0 },
    { 0xbb0f9583, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152f9587, 65 },
    { 0, 0 },
    { 0xf1ef9589, 92 },
    { 0x1a6f9589, 90 },
    { 0xb31f9589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedf9594, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf9597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f959d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df95a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff95ae, 98 },
    { 0, 0 },
    { 0x1faf95b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514f95b5, 34 },
    { 0xf06f95b6, 85 },
    { 0xdc3f95b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f95bd, 23 },
    { 0xb07f95be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f95ca, 33 },
    { 0xebff95cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f95d3, 96 },
    { 0x253f95d4, 46 },
    { 0, 0 },
    { 0x2acf95d6, 43 },
    { 0xe26f95d7, 53 },
    { 0x1e7f95d7, 61 },
    { 0x1a3f95d7, 75 },
    { 0, 0 },
    { 0x77ff95db, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769f95eb, 33 },
    { 0, 0 },
    { 0x7fff95ed, 100 },
    { 0, 0 },
    { 0xa13f95ef, 33 },
    { 0xb9ef95f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf95f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9f95fa, 98 },
    { 0xcbcf95fb, 61 },
    { 0, 0 },
    { 0xc8cf95fd, 33 },
    { 0x385f95fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50f9601, 23 },
    { 0x4b3f9601, 86 },
    { 0x40ff9603, 76 },
    { 0xfcaf9601, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdf9608, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3f960b, 23 },
    { 0xfc9f960c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdf960f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff9613, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8f9617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fef961d, 43 },
    { 0x78ef961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899f9622, 98 },
    { 0xbc1f9622, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9628, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ff962e, 55 },
    { 0x82ef962e, 95 },
    { 0, 0 },
    { 0x658f9631, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef9636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190f963c, 55 },
    { 0x864f963d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f9648, 98 },
    { 0, 0 },
    { 0xe30f964a, 40 },
    { 0, 0 },
    { 0xc77f964c, 87 },
    { 0, 0 },
    { 0x314f964e, 98 },
    { 0xd6df964e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf9654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745f9659, 43 },
    { 0x3acf965a, 76 },
    { 0xef3f965a, 66 },
    { 0x4ff965c, 98 },
    { 0xf17f965d, 61 },
    { 0xff7f965d, 33 },
    { 0x238f965a, 84 },
    { 0x178f9660, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f9665, 33 },
    { 0, 0 },
    { 0x115f9667, 90 },
    { 0, 0 },
    { 0x46af9669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf966d, 98 },
    { 0xa2cf966e, 76 },
    { 0x4adf966e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f9676, 46 },
    { 0xdc8f9677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf967b, 78 },
    { 0x4eff967c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1f9681, 33 },
    { 0x15bf9682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ff9690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddf9695, 23 },
    { 0x939f9695, 61 },
    { 0x45bf9696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bf969d, 90 },
    { 0, 0 },
    { 0x2b5f969f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f96a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3f96a9, 90 },
    { 0x4ccf96aa, 55 },
    { 0, 0 },
    { 0x828f96ac, 52 },
    { 0xd6df96ac, 64 },
    { 0x46cf96ae, 23 },
    { 0xa3ef96ad, 91 },
    { 0x15df96b0, 90 },
    { 0x3e1f96b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af96b5, 98 },
    { 0xe37f96b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f96b9, 61 },
    { 0xa1ff96b9, 100 },
    { 0, 0 },
    { 0x481f96bc, 33 },
    { 0, 0 },
    { 0x4faf96be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f96c3, 97 },
    { 0xd04f96c3, 98 },
    { 0x55bf96c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbf96c8, 55 },
    { 0x15af96c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bf96cc, 84 },
    { 0x93f96cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaf96d5, 76 },
    { 0, 0 },
    { 0x9c9f96d7, 90 },
    { 0x115f96d8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ef96dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf96df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56f96e4, 76 },
    { 0x835f96e4, 81 },
    { 0x7def96e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105f96ec, 55 },
    { 0xc9f96ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15f96f0, 98 },
    { 0xfb9f96f0, 77 },
    { 0xbecf96f2, 84 },
    { 0x20af96f3, 66 },
    { 0, 0 },
    { 0xc52f96f5, 76 },
    { 0x8f96f6, 17 },
    { 0x914f96f7, 90 },
    { 0xd91f96f8, 66 },
    { 0, 0 },
    { 0x9f9f96fa, 87 },
    { 0xf35f96fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f96fe, 43 },
    { 0x508f96ff, 55 },
    { 0x3adf96ff, 34 },
    { 0xd5f9701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf9707, 91 },
    { 0, 0 },
    { 0xbc6f9709, 33 },
    { 0, 0 },
    { 0x22bf970b, 78 },
    { 0xa1af970b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cf970f, 76 },
    { 0x16f9710, 23 },
    { 0x214f9711, 95 },
    { 0x29f9712, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f971e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f9721, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3f9727, 42 },
    { 0xf87f9728, 33 },
    { 0x197f9729, 99 },
    { 0x254f9728, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcf9731, 46 },
    { 0x1aff9731, 61 },
    { 0x4af9733, 84 },
    { 0x5aaf9732, 95 },
    { 0x207f9732, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f973c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bf9742, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09f9748, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x493f974b, 61 },
    { 0, 0 },
    { 0x5faf974d, 90 },
    { 0, 0 },
    { 0xfcbf974f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32af9753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f9757, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f975e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccf9762, 43 },
    { 0x52df9763, 76 },
    { 0x9af9763, 66 },
    { 0x13ef9762, 34 },
    { 0x810f9766, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f976b, 90 },
    { 0xbfff976b, 61 },
    { 0xadf976c, 76 },
    { 0x148f976e, 98 },
    { 0xb5cf976f, 84 },
    { 0x30ef976f, 61 },
    { 0x586f9770, 87 },
    { 0x159f976d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf9775, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f977c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef9780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9f9786, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2af978a, 33 },
    { 0x606f978b, 95 },
    { 0, 0 },
    { 0x2c2f978d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3f9792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f9796, 98 },
    { 0xa8af9797, 77 },
    { 0, 0 },
    { 0x79af9799, 91 },
    { 0, 0 },
    { 0x1aaf979b, 90 },
    { 0xcd0f979c, 98 },
    { 0xb57f979b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf97a7, 81 },
    { 0, 0 },
    { 0xddef97a9, 90 },
    { 0x2aaf97a9, 97 },
    { 0xd2ff97a9, 23 },
    { 0xef7f97ab, 64 },
    { 0, 0 },
    { 0xd92f97ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f97b1, 52 },
    { 0xc1cf97b1, 17 },
    { 0x8a6f97b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47f97be, 30 },
    { 0x126f97bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2f97c4, 61 },
    { 0, 0 },
    { 0xd6cf97c6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3f97cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f97d0, 84 },
    { 0x168f97d1, 95 },
    { 0, 0 },
    { 0xf47f97d3, 95 },
    { 0xfc7f97d4, 77 },
    { 0, 0 },
    { 0xca7f97d6, 76 },
    { 0, 0 },
    { 0x8e8f97d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f97e0, 61 },
    { 0x58ff97e1, 95 },
    { 0, 0 },
    { 0x56f97e3, 43 },
    { 0, 0 },
    { 0xd9cf97e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356f97ea, 46 },
    { 0x3bcf97eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7f97f5, 76 },
    { 0xad1f97f6, 61 },
    { 0x9abf97f7, 52 },
    { 0x674f97f7, 61 },
    { 0x34f97f9, 90 },
    { 0x204f97f7, 55 },
    { 0, 0 },
    { 0xf39f97fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f9801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86f9804, 46 },
    { 0x8f3f9805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f9808, 95 },
    { 0x7c9f9809, 91 },
    { 0, 0 },
    { 0x88ef980b, 23 },
    { 0x956f980c, 70 },
    { 0x735f980c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f9813, 91 },
    { 0x612f9814, 33 },
    { 0, 0 },
    { 0xefef9816, 98 },
    { 0x265f9817, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cf981a, 76 },
    { 0xb5bf981b, 90 },
    { 0x45af981c, 99 },
    { 0x78cf981d, 76 },
    { 0xa53f981e, 61 },
    { 0x33cf981f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41df9824, 90 },
    { 0, 0 },
    { 0xd62f9826, 17 },
    { 0x674f9827, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaef982a, 76 },
    { 0, 0 },
    { 0x658f982c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bf9834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f983b, 33 },
    { 0xcb4f983c, 43 },
    { 0x7e4f983d, 95 },
    { 0xdb9f983e, 98 },
    { 0xa19f983e, 61 },
    { 0xc6f983f, 61 },
    { 0x1e2f983f, 91 },
    { 0x818f983f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef9846, 61 },
    { 0, 0 },
    { 0x93ef9848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1af984b, 100 },
    { 0x329f984c, 33 },
    { 0, 0 },
    { 0x986f984e, 23 },
    { 0, 0 },
    { 0x936f9850, 43 },
    { 0x91cf9851, 67 },
    { 0x3df9851, 33 },
    { 0, 0 },
    { 0x679f9854, 98 },
    { 0x13cf9855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf985d, 23 },
    { 0, 0 },
    { 0x538f985f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff9865, 43 },
    { 0x542f9866, 33 },
    { 0xec2f9867, 61 },
    { 0x40ef9866, 90 },
    { 0xfdaf9869, 85 },
    { 0x15cf986a, 43 },
    { 0xc9ff9868, 61 },
    { 0, 0 },
    { 0x296f986d, 98 },
    { 0x765f986d, 23 },
    { 0x852f986e, 87 },
    { 0xa69f986e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f9876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cf9881, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecf9884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf9888, 52 },
    { 0x58af9888, 64 },
    { 0x42bf988a, 61 },
    { 0xdc4f988a, 90 },
    { 0x544f988c, 90 },
    { 0x476f988b, 43 },
    { 0, 0 },
    { 0xe87f988f, 23 },
    { 0x630f9890, 40 },
    { 0x995f9891, 80 },
    { 0x648f9892, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578f989a, 66 },
    { 0, 0 },
    { 0xda3f989c, 23 },
    { 0xeddf989d, 91 },
    { 0xc00f989e, 55 },
    { 0x770f989f, 33 },
    { 0x501f989e, 53 },
    { 0x646f989d, 95 },
    { 0xf1cf98a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f98a9, 90 },
    { 0x23ff98aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7f98af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9f98b3, 81 },
    { 0, 0 },
    { 0x4aff98b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ef98bc, 34 },
    { 0x352f98bd, 90 },
    { 0, 0 },
    { 0x620f98bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f98ca, 64 },
    { 0, 0 },
    { 0x4baf98cc, 46 },
    { 0x68af98cc, 95 },
    { 0x185f98cc, 33 },
    { 0x783f98cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4f98d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936f98d9, 91 },
    { 0x54f98da, 100 },
    { 0, 0 },
    { 0x116f98dc, 34 },
    { 0xd6df98dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f98e1, 93 },
    { 0xccf98e2, 61 },
    { 0x5b3f98e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf98ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0df98ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f98f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f98f7, 95 },
    { 0, 0 },
    { 0xb58f98f9, 33 },
    { 0, 0 },
    { 0x712f98fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74f9908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f990b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ff990e, 98 },
    { 0, 0 },
    { 0x6eaf9910, 61 },
    { 0x454f9911, 98 },
    { 0, 0 },
    { 0xa24f9913, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df9918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef991b, 92 },
    { 0x507f991c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f9927, 75 },
    { 0, 0 },
    { 0xb0af9929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df9930, 77 },
    { 0xdc2f9930, 100 },
    { 0x692f9931, 34 },
    { 0xcf0f9931, 95 },
    { 0, 0 },
    { 0x1e8f9935, 90 },
    { 0xfcef9936, 100 },
    { 0, 0 },
    { 0xfd4f9938, 52 },
    { 0xf23f9938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781f9945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967f9949, 95 },
    { 0xa94f994a, 34 },
    { 0, 0 },
    { 0x29f994c, 90 },
    { 0x6a9f994d, 59 },
    { 0x356f994e, 90 },
    { 0x5eef994d, 95 },
    { 0xec3f994c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902f9956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff995b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915f9961, 90 },
    { 0xf09f9962, 98 },
    { 0xeb5f9963, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24f9966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af9969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f996d, 33 },
    { 0, 0 },
    { 0x450f996f, 55 },
    { 0xe51f9970, 90 },
    { 0x402f9971, 98 },
    { 0, 0 },
    { 0x7ecf9973, 88 },
    { 0xb59f9973, 66 },
    { 0, 0 },
    { 0xe97f9976, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f997a, 81 },
    { 0x80df997b, 23 },
    { 0x1f1f997c, 90 },
    { 0xc1ff997a, 33 },
    { 0xd89f997c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f9981, 99 },
    { 0, 0 },
    { 0xf9ef9983, 36 },
    { 0xbd1f9983, 99 },
    { 0, 0 },
    { 0x455f9986, 52 },
    { 0, 0 },
    { 0x548f9988, 23 },
    { 0x2cdf9988, 46 },
    { 0, 0 },
    { 0x14bf998b, 88 },
    { 0x4d0f998c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f9994, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f999a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef999e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9f99a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaf99a7, 91 },
    { 0xe67f99a8, 61 },
    { 0x27f99a7, 23 },
    { 0x63df99aa, 95 },
    { 0, 0 },
    { 0x94ef99ac, 95 },
    { 0x801f99ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f99b3, 98 },
    { 0x799f99b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef99bc, 23 },
    { 0x561f99bd, 93 },
    { 0xbddf99bc, 33 },
    { 0xdef99bf, 90 },
    { 0xb24f99c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17df99c5, 33 },
    { 0x608f99c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cf99ca, 33 },
    { 0x54f99cb, 90 },
    { 0xa83f99cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f99cf, 98 },
    { 0, 0 },
    { 0x631f99d1, 76 },
    { 0x1eef99d1, 80 },
    { 0x728f99d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f99d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f99dd, 76 },
    { 0x495f99de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f99e5, 95 },
    { 0xd58f99e6, 98 },
    { 0, 0 },
    { 0x47cf99e8, 90 },
    { 0, 0 },
    { 0xec0f99ea, 90 },
    { 0xf92f99eb, 100 },
    { 0xc49f99eb, 55 },
    { 0x491f99ed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9f99f0, 17 },
    { 0xaadf99f1, 91 },
    { 0xd5cf99f0, 98 },
    { 0x964f99f2, 98 },
    { 0, 0 },
    { 0x9dcf99f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f99fe, 95 },
    { 0x1cff99ff, 97 },
    { 0, 0 },
    { 0x188f9a01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef9a08, 33 },
    { 0, 0 },
    { 0x414f9a0a, 52 },
    { 0x93cf9a0b, 17 },
    { 0xde7f9a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df9a0f, 88 },
    { 0x9c9f9a10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1f9a17, 90 },
    { 0x66ff9a17, 90 },
    { 0x242f9a19, 90 },
    { 0x86df9a17, 99 },
    { 0x4e3f9a1a, 87 },
    { 0, 0 },
    { 0xd0f9a1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x387f9a20, 90 },
    { 0xddaf9a20, 33 },
    { 0xc59f9a21, 47 },
    { 0x68ff9a21, 55 },
    { 0x778f9a22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf9a2a, 93 },
    { 0x18af9a2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35f9a36, 90 },
    { 0x640f9a37, 33 },
    { 0x911f9a38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef9a3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15f9a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff9a49, 93 },
    { 0, 0 },
    { 0xa71f9a4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcf9a4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2f9a51, 23 },
    { 0x72bf9a52, 69 },
    { 0, 0 },
    { 0x2cbf9a54, 43 },
    { 0xca8f9a54, 77 },
    { 0x599f9a55, 88 },
    { 0x86cf9a57, 90 },
    { 0xae9f9a56, 55 },
    { 0, 0 },
    { 0x7a6f9a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f9a5d, 78 },
    { 0, 0 },
    { 0x6cef9a5f, 61 },
    { 0x522f9a60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf9a67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632f9a6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x213f9a72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f9a75, 23 },
    { 0xcf1f9a76, 90 },
    { 0x67cf9a76, 95 },
    { 0x476f9a78, 88 },
    { 0x26ff9a79, 84 },
    { 0x1a2f9a76, 55 },
    { 0x588f9a79, 95 },
    { 0x36bf9a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf9a82, 33 },
    { 0x59f9a83, 95 },
    { 0x44bf9a83, 33 },
    { 0xcdcf9a85, 90 },
    { 0x34cf9a86, 61 },
    { 0x9bdf9a86, 33 },
    { 0x30cf9a88, 84 },
    { 0xf17f9a88, 61 },
    { 0x300f9a89, 23 },
    { 0x829f9a8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cf9a90, 55 },
    { 0xf99f9a91, 34 },
    { 0xc64f9a92, 95 },
    { 0x68bf9a92, 98 },
    { 0xff4f9a93, 33 },
    { 0x984f9a95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f9a98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f9a9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f9aa1, 90 },
    { 0x4f4f9aa2, 46 },
    { 0, 0 },
    { 0x448f9aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1f9aaa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ef9ab0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf9ab7, 99 },
    { 0x76f9ab7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf9abd, 33 },
    { 0x1cbf9abe, 90 },
    { 0x15df9abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311f9ac9, 43 },
    { 0, 0 },
    { 0xed7f9acb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2f9ace, 76 },
    { 0x36f9acf, 66 },
    { 0x168f9acf, 33 },
    { 0x1bcf9acf, 33 },
    { 0xebef9ad2, 90 },
    { 0xd18f9ad3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f9ad7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f9ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14f9ae3, 93 },
    { 0, 0 },
    { 0x9cef9ae5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f9ae9, 33 },
    { 0xf8df9aea, 75 },
    { 0, 0 },
    { 0xcdf9aec, 66 },
    { 0xf53f9aed, 98 },
    { 0x926f9aee, 23 },
    { 0x192f9aef, 90 },
    { 0, 0 },
    { 0x1cef9af1, 61 },
    { 0, 0 },
    { 0xdf1f9af3, 69 },
    { 0xbbcf9af4, 33 },
    { 0xc5af9af5, 55 },
    { 0x8def9af6, 61 },
    { 0xb6af9af5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bf9afa, 33 },
    { 0xd5af9afb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4f9b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf9b07, 61 },
    { 0x4f9f9b07, 23 },
    { 0, 0 },
    { 0x5b6f9b0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff9b0d, 66 },
    { 0x29df9b0e, 55 },
    { 0x9e5f9b0d, 61 },
    { 0xccaf9b0d, 78 },
    { 0x5a1f9b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f9b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ef9b19, 90 },
    { 0x852f9b1a, 90 },
    { 0x9bcf9b1b, 66 },
    { 0xd15f9b1a, 90 },
    { 0xa40f9b19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef9b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f9b25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x536f9b28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff9b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f9b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f9b33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683f9b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f9b3c, 98 },
    { 0, 0 },
    { 0x4a8f9b3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806f9b42, 80 },
    { 0, 0 },
    { 0x9bff9b44, 69 },
    { 0x65ef9b44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9f9b49, 84 },
    { 0x33f9b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cf9b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bf9b50, 90 },
    { 0, 0 },
    { 0x871f9b52, 34 },
    { 0, 0 },
    { 0xa6af9b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaf9b58, 33 },
    { 0, 0 },
    { 0xb62f9b5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f9b5d, 95 },
    { 0x147f9b5e, 98 },
    { 0xed5f9b5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250f9b64, 33 },
    { 0x331f9b65, 84 },
    { 0, 0 },
    { 0xe6cf9b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f9b6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f9b6d, 75 },
    { 0x402f9b6e, 88 },
    { 0x1b1f9b6e, 90 },
    { 0x129f9b6f, 23 },
    { 0xdacf9b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664f9b7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657f9b81, 87 },
    { 0, 0 },
    { 0x3a2f9b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f9b86, 95 },
    { 0xca4f9b86, 34 },
    { 0xc1af9b88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9b8e, 95 },
    { 0xcc3f9b8e, 35 },
    { 0, 0 },
    { 0x860f9b91, 43 },
    { 0xe75f9b92, 93 },
    { 0xdbbf9b93, 77 },
    { 0x9e4f9b91, 96 },
    { 0x3c8f9b95, 90 },
    { 0xf2df9b96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f9b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edf9ba1, 33 },
    { 0, 0 },
    { 0xdfdf9ba3, 90 },
    { 0x3dcf9ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f9ba9, 66 },
    { 0xa7df9baa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f9baf, 61 },
    { 0x535f9bb0, 33 },
    { 0x290f9bb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995f9bb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83df9bb9, 85 },
    { 0xafdf9bb9, 95 },
    { 0, 0 },
    { 0x31bf9bbc, 93 },
    { 0x1f8f9bbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf9bcc, 90 },
    { 0x812f9bcd, 76 },
    { 0, 0 },
    { 0x98ff9bcf, 90 },
    { 0xb06f9bd0, 56 },
    { 0xf54f9bd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def9bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x191f9bdc, 43 },
    { 0xeedf9bdd, 46 },
    { 0x542f9bdc, 55 },
    { 0xe28f9bdf, 84 },
    { 0xf2af9be0, 76 },
    { 0xfaf9be0, 43 },
    { 0xbeef9bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3f9be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f9bee, 98 },
    { 0x5caf9bee, 81 },
    { 0xd47f9bf0, 99 },
    { 0xc81f9bf0, 93 },
    { 0, 0 },
    { 0xf5ff9bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef9bfa, 23 },
    { 0x5c2f9bfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef9c05, 61 },
    { 0x780f9c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04f9c0b, 23 },
    { 0x7adf9c0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff9c12, 43 },
    { 0xb07f9c13, 61 },
    { 0, 0 },
    { 0xac1f9c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f9c18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cf9c20, 90 },
    { 0, 0 },
    { 0xa08f9c22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f9c25, 66 },
    { 0, 0 },
    { 0x333f9c27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f9c2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbf9c32, 66 },
    { 0x615f9c33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67f9c3a, 33 },
    { 0x9bdf9c3a, 33 },
    { 0x6bcf9c3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcff9c3f, 88 },
    { 0x741f9c40, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f9c45, 90 },
    { 0x189f9c46, 40 },
    { 0x738f9c46, 93 },
    { 0, 0 },
    { 0x9a2f9c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f9c4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f9c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f9c56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318f9c5c, 33 },
    { 0, 0 },
    { 0x128f9c5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7f9c64, 23 },
    { 0xf77f9c65, 23 },
    { 0x25bf9c66, 77 },
    { 0x2a3f9c67, 100 },
    { 0xf47f9c64, 90 },
    { 0, 0 },
    { 0x86bf9c6a, 76 },
    { 0x1a2f9c6b, 76 },
    { 0, 0 },
    { 0x851f9c6d, 90 },
    { 0x2caf9c6e, 93 },
    { 0xf01f9c6f, 19 },
    { 0xe8ff9c6f, 61 },
    { 0xcedf9c6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78f9c7b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bf9c80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88f9c87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ff9c8a, 23 },
    { 0x53df9c8a, 76 },
    { 0, 0 },
    { 0xd21f9c8d, 61 },
    { 0xcf2f9c8d, 66 },
    { 0x515f9c8e, 98 },
    { 0, 0 },
    { 0x2dff9c91, 66 },
    { 0xc81f9c92, 23 },
    { 0, 0 },
    { 0xc98f9c94, 23 },
    { 0x47f9c94, 61 },
    { 0xb33f9c94, 46 },
    { 0x73ff9c97, 76 },
    { 0xe48f9c98, 90 },
    { 0x587f9c98, 23 },
    { 0xd01f9c96, 91 },
    { 0, 0 },
    { 0xc12f9c9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917f9ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef9ca3, 23 },
    { 0, 0 },
    { 0xe06f9ca5, 46 },
    { 0xb20f9ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38f9caa, 33 },
    { 0x84bf9cab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fef9cae, 65 },
    { 0, 0 },
    { 0x765f9cb0, 33 },
    { 0x416f9cb1, 61 },
    { 0, 0 },
    { 0x22df9cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f9cb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff9cba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61f9cbd, 90 },
    { 0xd50f9cbe, 100 },
    { 0x5cff9cbe, 67 },
    { 0xa9ff9cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82f9cc7, 87 },
    { 0xdaff9cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14f9ccb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x173f9cce, 23 },
    { 0x3f5f9ccf, 55 },
    { 0xf76f9cd0, 33 },
    { 0x8c5f9cd1, 38 },
    { 0xd9df9cd2, 23 },
    { 0x618f9cd3, 98 },
    { 0xdbbf9cd3, 91 },
    { 0x4fbf9cd5, 23 },
    { 0xa57f9cd3, 34 },
    { 0x58df9cd1, 55 },
    { 0x66df9cd8, 90 },
    { 0xfedf9cd9, 100 },
    { 0xff9f9cd3, 66 },
    { 0x664f9cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef9ce0, 46 },
    { 0x14cf9ce1, 90 },
    { 0x78bf9ce1, 61 },
    { 0, 0 },
    { 0xa75f9ce4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf9ce8, 84 },
    { 0xd04f9ce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ff9cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f9d00, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf9d05, 100 },
    { 0, 0 },
    { 0x1bf9d07, 97 },
    { 0xf6ff9d08, 81 },
    { 0xbadf9d07, 48 },
    { 0x11cf9d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f9d0e, 52 },
    { 0x858f9d0f, 66 },
    { 0x9f6f9d0e, 46 },
    { 0x306f9d11, 49 },
    { 0xeb7f9d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f9d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeef9d1f, 23 },
    { 0xd30f9d20, 95 },
    { 0, 0 },
    { 0x70ef9d22, 93 },
    { 0x10df9d23, 76 },
    { 0, 0 },
    { 0x499f9d25, 34 },
    { 0x2eaf9d25, 84 },
    { 0xc0f9d27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbf9d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebf9d2d, 90 },
    { 0x761f9d2e, 61 },
    { 0xbbff9d2f, 23 },
    { 0x9b5f9d2f, 33 },
    { 0xa32f9d2f, 33 },
    { 0x928f9d30, 33 },
    { 0x98cf9d33, 33 },
    { 0, 0 },
    { 0xc5ef9d35, 66 },
    { 0x485f9d36, 78 },
    { 0xf33f9d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9d3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285f9d41, 90 },
    { 0xdbff9d42, 95 },
    { 0x315f9d43, 23 },
    { 0xe68f9d42, 33 },
    { 0x31af9d43, 90 },
    { 0, 0 },
    { 0x329f9d47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364f9d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233f9d52, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf9d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6af9d5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f9d5d, 95 },
    { 0xbedf9d5d, 78 },
    { 0x1c9f9d5f, 98 },
    { 0x64bf9d60, 90 },
    { 0xc9af9d60, 90 },
    { 0xcddf9d62, 90 },
    { 0x4fff9d5e, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x962f9d66, 61 },
    { 0x36df9d67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf9d6a, 40 },
    { 0x30ff9d6a, 61 },
    { 0xddef9d6a, 95 },
    { 0xfbdf9d6d, 90 },
    { 0xc65f9d6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7f9d72, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f9d75, 43 },
    { 0xf31f9d76, 33 },
    { 0x20af9d77, 34 },
    { 0x99af9d78, 33 },
    { 0x5a2f9d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98af9d7d, 76 },
    { 0xe8cf9d7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x399f9d81, 90 },
    { 0xf69f9d82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff9d87, 61 },
    { 0xf3df9d88, 99 },
    { 0, 0 },
    { 0xddbf9d8a, 85 },
    { 0x5dff9d8b, 87 },
    { 0x3c8f9d8c, 55 },
    { 0, 0 },
    { 0x722f9d8e, 90 },
    { 0xdd9f9d8f, 34 },
    { 0x89ef9d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f9d93, 98 },
    { 0xb70f9d94, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3af9d9d, 33 },
    { 0x540f9d9e, 61 },
    { 0x70ef9d9d, 90 },
    { 0, 0 },
    { 0xc0af9da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50f9da7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1def9dae, 34 },
    { 0, 0 },
    { 0x571f9db0, 33 },
    { 0, 0 },
    { 0xc80f9db2, 90 },
    { 0x81df9db2, 91 },
    { 0, 0 },
    { 0x216f9db5, 23 },
    { 0, 0 },
    { 0x4e0f9db7, 76 },
    { 0, 0 },
    { 0x2df9db9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f9dbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ff9dc0, 61 },
    { 0x36df9dc1, 33 },
    { 0x6e0f9dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f9dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7f9dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef9dcb, 55 },
    { 0, 0 },
    { 0x6b6f9dcd, 98 },
    { 0x90ff9dce, 90 },
    { 0xc0df9dcf, 90 },
    { 0xa27f9dd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ff9dd3, 23 },
    { 0, 0 },
    { 0x2fff9dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1f9dda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0f9dde, 43 },
    { 0xcef9dde, 93 },
    { 0, 0 },
    { 0x955f9de1, 100 },
    { 0x7c3f9de1, 23 },
    { 0xddef9de3, 61 },
    { 0x7d7f9de2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbf9dee, 61 },
    { 0, 0 },
    { 0x665f9df0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf9df6, 61 },
    { 0x7f2f9df6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79f9dfa, 34 },
    { 0, 0 },
    { 0x9f2f9dfc, 61 },
    { 0xca1f9dfc, 61 },
    { 0x5f0f9dfc, 61 },
    { 0xd40f9dff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebf9e02, 17 },
    { 0, 0 },
    { 0x74bf9e04, 98 },
    { 0x68bf9e05, 33 },
    { 0, 0 },
    { 0xfa3f9e07, 23 },
    { 0x13ff9e07, 90 },
    { 0, 0 },
    { 0xb13f9e0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff9e11, 90 },
    { 0x5d7f9e12, 34 },
    { 0xae1f9e12, 76 },
    { 0x17f9e14, 90 },
    { 0xc52f9e15, 55 },
    { 0xc91f9e15, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f9e1d, 78 },
    { 0, 0 },
    { 0x5eef9e1f, 95 },
    { 0x9f7f9e1f, 33 },
    { 0x268f9e1f, 91 },
    { 0, 0 },
    { 0x900f9e23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf9e2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f9e2f, 23 },
    { 0xc7ff9e30, 90 },
    { 0x907f9e2f, 98 },
    { 0xde8f9e31, 84 },
    { 0, 0 },
    { 0xe17f9e34, 61 },
    { 0x230f9e35, 61 },
    { 0x428f9e34, 95 },
    { 0x3d0f9e37, 76 },
    { 0xee1f9e38, 90 },
    { 0, 0 },
    { 0x751f9e3a, 33 },
    { 0x2cdf9e3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eef9e41, 85 },
    { 0, 0 },
    { 0x46cf9e43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02f9e48, 98 },
    { 0x899f9e48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f9e4c, 55 },
    { 0x722f9e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadff9e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96f9e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3f9e57, 61 },
    { 0x847f9e58, 61 },
    { 0xd1af9e57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9e62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537f9e67, 33 },
    { 0x99ff9e67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f9e6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61af9e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f9e76, 91 },
    { 0x910f9e77, 23 },
    { 0x5d7f9e78, 7 },
    { 0x24cf9e79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f9e7d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf9e82, 77 },
    { 0xe39f9e82, 76 },
    { 0x95f9e84, 33 },
    { 0, 0 },
    { 0xa87f9e86, 55 },
    { 0, 0 },
    { 0x70bf9e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf9e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23f9e96, 81 },
    { 0x887f9e96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf9e9c, 23 },
    { 0, 0 },
    { 0xc05f9e9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eff9ea2, 23 },
    { 0x3f8f9ea2, 61 },
    { 0xa8ff9ea4, 67 },
    { 0xa06f9ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34df9ea9, 95 },
    { 0, 0 },
    { 0x61bf9eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170f9eb1, 55 },
    { 0xd8ef9eb2, 90 },
    { 0xdc5f9eb3, 95 },
    { 0, 0 },
    { 0xdb2f9eb5, 56 },
    { 0xa66f9eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f9ebe, 99 },
    { 0, 0 },
    { 0x812f9ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ef9ec4, 87 },
    { 0x13cf9ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f9ed0, 23 },
    { 0x694f9ed1, 93 },
    { 0xe6af9ed2, 61 },
    { 0x2c8f9ed2, 34 },
    { 0x232f9ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f9ed7, 93 },
    { 0xf6af9ed7, 98 },
    { 0, 0 },
    { 0x60df9eda, 98 },
    { 0, 0 },
    { 0x2faf9edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7f9ee0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2f9ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf9ee9, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaf9eec, 17 },
    { 0xfa7f9eed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cf9ef3, 33 },
    { 0, 0 },
    { 0x6fdf9ef5, 23 },
    { 0, 0 },
    { 0xdcaf9ef7, 76 },
    { 0xfdff9ef7, 95 },
    { 0x6aaf9ef9, 61 },
    { 0, 0 },
    { 0x3daf9efb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f9efe, 90 },
    { 0, 0 },
    { 0x1c8f9f00, 95 },
    { 0x486f9f01, 90 },
    { 0x87af9f01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff9f05, 55 },
    { 0xdf9f06, 77 },
    { 0x4aaf9f05, 43 },
    { 0x7c6f9f07, 95 },
    { 0x961f9f06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f9f0d, 76 },
    { 0xd57f9f0e, 61 },
    { 0xa04f9f0e, 95 },
    { 0xb41f9f10, 61 },
    { 0x455f9f11, 61 },
    { 0xfa9f9f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6f9f16, 76 },
    { 0x56ef9f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf9f1a, 76 },
    { 0, 0 },
    { 0xce9f9f1c, 43 },
    { 0, 0 },
    { 0x67ef9f1e, 67 },
    { 0x18ef9f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f9f23, 100 },
    { 0, 0 },
    { 0x87ef9f25, 99 },
    { 0x860f9f26, 64 },
    { 0xf7df9f26, 83 },
    { 0x6a4f9f25, 76 },
    { 0x843f9f29, 61 },
    { 0xdaf9f29, 89 },
    { 0xad4f9f2b, 34 },
    { 0x21ef9f2c, 61 },
    { 0x29bf9f2a, 55 },
    { 0x2c4f9f2d, 33 },
    { 0xd91f9f2f, 53 },
    { 0x9c8f9f30, 55 },
    { 0x3b1f9f30, 98 },
    { 0xac3f9f30, 91 },
    { 0x1b6f9f33, 90 },
    { 0xdf8f9f25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272f9f39, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f9f40, 23 },
    { 0, 0 },
    { 0x714f9f42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f9f46, 95 },
    { 0xb25f9f47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84df9f4c, 90 },
    { 0xefdf9f4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf9f50, 33 },
    { 0, 0 },
    { 0xcddf9f52, 93 },
    { 0, 0 },
    { 0xcdcf9f54, 90 },
    { 0x3b6f9f54, 23 },
    { 0x63af9f56, 99 },
    { 0x309f9f57, 85 },
    { 0x367f9f57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ef9f5c, 33 },
    { 0x7ddf9f5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70f9f65, 90 },
    { 0xbbdf9f65, 85 },
    { 0x5f8f9f67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff9f6b, 33 },
    { 0x4baf9f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfff9f70, 55 },
    { 0xaa9f9f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548f9f76, 52 },
    { 0xf70f9f76, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f9f82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f9f88, 23 },
    { 0, 0 },
    { 0x43ff9f8a, 85 },
    { 0xf52f9f8a, 95 },
    { 0xecbf9f8c, 76 },
    { 0, 0 },
    { 0xc76f9f8e, 98 },
    { 0xb3af9f8e, 98 },
    { 0x242f9f90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdf9f94, 66 },
    { 0x78af9f95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff9f9b, 23 },
    { 0, 0 },
    { 0xeb5f9f9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f9fa0, 91 },
    { 0xa40f9fa1, 98 },
    { 0x23af9fa2, 33 },
    { 0x691f9fa3, 33 },
    { 0x2aff9fa1, 48 },
    { 0x490f9fa4, 33 },
    { 0x434f9fa5, 76 },
    { 0xd8cf9fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4f9fad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3f9fb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cf9fb9, 90 },
    { 0, 0 },
    { 0x761f9fbb, 96 },
    { 0xadbf9fbc, 61 },
    { 0x1caf9fbd, 91 },
    { 0xedbf9fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcf9fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1f9fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ff9fd6, 88 },
    { 0x193f9fd7, 76 },
    { 0x41df9fd8, 81 },
    { 0xaeaf9fd8, 33 },
    { 0, 0 },
    { 0xcfdf9fdb, 76 },
    { 0xc77f9fdc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31df9fe3, 61 },
    { 0x72af9fe4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656f9fea, 23 },
    { 0, 0 },
    { 0x66ef9fec, 100 },
    { 0xcb5f9fec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7af9ff0, 23 },
    { 0x382f9ff0, 61 },
    { 0xce1f9ff2, 98 },
    { 0xfbef9ff3, 23 },
    { 0, 0 },
    { 0xb51f9ff5, 61 },
    { 0x153f9ff5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x573f9ff9, 95 },
    { 0xed2f9ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fa007, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fa00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211fa00f, 93 },
    { 0x556fa00f, 34 },
    { 0xa0cfa011, 43 },
    { 0x187fa011, 90 },
    { 0xa83fa013, 53 },
    { 0x5a1fa00f, 8 },
    { 0, 0 },
    { 0x2cfa016, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05fa01c, 33 },
    { 0x4dbfa01c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01fa021, 61 },
    { 0, 0 },
    { 0xb02fa023, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551fa029, 90 },
    { 0xb30fa02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x467fa02d, 61 },
    { 0x3e8fa02e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fa031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09fa036, 43 },
    { 0xdb1fa036, 90 },
    { 0xe3bfa038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fa03e, 33 },
    { 0x290fa03f, 61 },
    { 0x9fcfa040, 34 },
    { 0x4a6fa041, 55 },
    { 0xe21fa041, 56 },
    { 0x61ffa041, 90 },
    { 0x680fa043, 76 },
    { 0, 0 },
    { 0x165fa046, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa04f, 91 },
    { 0x9e4fa050, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937fa058, 23 },
    { 0, 0 },
    { 0xa25fa05a, 67 },
    { 0x3eafa05b, 78 },
    { 0x1c0fa05c, 43 },
    { 0x7d9fa05d, 33 },
    { 0x945fa05d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7fa061, 76 },
    { 0xab7fa061, 90 },
    { 0x7affa063, 23 },
    { 0xb32fa064, 98 },
    { 0xe31fa064, 52 },
    { 0x477fa066, 61 },
    { 0, 0 },
    { 0x204fa068, 84 },
    { 0x7b4fa069, 16 },
    { 0, 0 },
    { 0xbdbfa06b, 90 },
    { 0, 0 },
    { 0x52dfa06d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6fa079, 61 },
    { 0xfa5fa07a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6fa07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cfa083, 33 },
    { 0xe9fa083, 33 },
    { 0x646fa085, 33 },
    { 0x825fa085, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ffa08d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddfa097, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcfa09a, 95 },
    { 0xa5ffa09b, 100 },
    { 0x500fa09c, 61 },
    { 0xf5fa09b, 46 },
    { 0x7f8fa09e, 90 },
    { 0xc69fa09f, 76 },
    { 0x9fafa0a0, 69 },
    { 0xc43fa0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09fa0a4, 90 },
    { 0xc4dfa0a5, 34 },
    { 0x141fa0a5, 61 },
    { 0, 0 },
    { 0x600fa0a8, 43 },
    { 0x1ccfa0a8, 33 },
    { 0xba9fa0aa, 68 },
    { 0x7adfa0a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fa0ae, 91 },
    { 0x7e2fa0af, 55 },
    { 0, 0 },
    { 0x5c4fa0b1, 90 },
    { 0, 0 },
    { 0xc85fa0b3, 52 },
    { 0xb8cfa0b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551fa0b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defa0be, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dfa0c1, 68 },
    { 0xec7fa0c1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30fa0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cfa0cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210fa0d3, 98 },
    { 0x393fa0d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9fa0de, 90 },
    { 0, 0 },
    { 0xfa7fa0e0, 55 },
    { 0x773fa0e1, 76 },
    { 0, 0 },
    { 0xe4afa0e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ffa0e6, 66 },
    { 0, 0 },
    { 0x31fa0e8, 55 },
    { 0, 0 },
    { 0x516fa0ea, 23 },
    { 0x3b3fa0ea, 61 },
    { 0, 0 },
    { 0xb4fa0ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa0f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1fa0f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa0f9, 99 },
    { 0x798fa0f9, 84 },
    { 0x5cfa0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1fa0fe, 76 },
    { 0, 0 },
    { 0x53efa100, 90 },
    { 0, 0 },
    { 0x8fdfa102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa106, 61 },
    { 0, 0 },
    { 0xbcdfa108, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5fa10d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdfa110, 70 },
    { 0x660fa111, 33 },
    { 0x6e5fa112, 95 },
    { 0x798fa113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fa119, 78 },
    { 0x94dfa11a, 90 },
    { 0xaa8fa11b, 75 },
    { 0xa04fa11c, 66 },
    { 0x4c0fa11d, 95 },
    { 0x585fa11c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1fa124, 23 },
    { 0, 0 },
    { 0x787fa126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6fa129, 95 },
    { 0x8bfa12a, 65 },
    { 0x539fa12b, 43 },
    { 0, 0 },
    { 0x9fdfa12d, 23 },
    { 0, 0 },
    { 0x3bffa12f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22efa133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76efa136, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91efa13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cfa145, 43 },
    { 0xfdffa145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x980fa149, 90 },
    { 0, 0 },
    { 0x85dfa14b, 99 },
    { 0, 0 },
    { 0x734fa14d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ffa150, 76 },
    { 0xba2fa151, 95 },
    { 0x9d8fa152, 67 },
    { 0xca1fa151, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ffa15c, 40 },
    { 0x604fa15d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754fa163, 33 },
    { 0x80afa164, 91 },
    { 0xf77fa164, 77 },
    { 0, 0 },
    { 0x277fa167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01fa16e, 61 },
    { 0xbf2fa16f, 85 },
    { 0, 0 },
    { 0x272fa171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59fa174, 81 },
    { 0xac3fa175, 66 },
    { 0xd98fa176, 61 },
    { 0, 0 },
    { 0x4bcfa178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4affa17b, 33 },
    { 0, 0 },
    { 0xb08fa17d, 90 },
    { 0x714fa17e, 61 },
    { 0, 0 },
    { 0xfc8fa180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfa188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dfa18f, 43 },
    { 0xa35fa190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fa193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596fa198, 90 },
    { 0, 0 },
    { 0x893fa19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ffa19d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97fa1a9, 33 },
    { 0, 0 },
    { 0xeedfa1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fa1b1, 95 },
    { 0xe7efa1b2, 91 },
    { 0xe9bfa1b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76fa1b6, 99 },
    { 0xa30fa1b7, 93 },
    { 0, 0 },
    { 0x649fa1b9, 46 },
    { 0xfbdfa1ba, 35 },
    { 0x925fa1bb, 75 },
    { 0xa4efa1bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdffa1c3, 78 },
    { 0, 0 },
    { 0xca9fa1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfa1c9, 61 },
    { 0, 0 },
    { 0x654fa1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bafa1ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa1d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9fa1d7, 90 },
    { 0x86bfa1d8, 99 },
    { 0x4bbfa1d8, 95 },
    { 0, 0 },
    { 0xcf4fa1db, 33 },
    { 0xff0fa1dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bfa1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610fa1ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bafa1f0, 33 },
    { 0, 0 },
    { 0x2c6fa1f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fa1f9, 93 },
    { 0xb44fa1fa, 46 },
    { 0, 0 },
    { 0xe52fa1fc, 61 },
    { 0, 0 },
    { 0x950fa1fe, 61 },
    { 0x4ccfa1ff, 23 },
    { 0, 0 },
    { 0xbdffa201, 61 },
    { 0, 0 },
    { 0xb9bfa203, 78 },
    { 0xf43fa203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fa20d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x707fa210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56fa213, 90 },
    { 0x230fa214, 90 },
    { 0x8f5fa215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bffa218, 90 },
    { 0x142fa219, 84 },
    { 0xd77fa21a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dfa21f, 33 },
    { 0, 0 },
    { 0x48cfa221, 61 },
    { 0x506fa222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fa225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfa22c, 23 },
    { 0x31bfa22c, 61 },
    { 0x8d0fa22e, 23 },
    { 0xfe1fa22c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fa234, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6fa237, 33 },
    { 0x3a6fa238, 46 },
    { 0, 0 },
    { 0xff2fa23a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92fa23d, 90 },
    { 0xae9fa23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2fa246, 33 },
    { 0, 0 },
    { 0x8b1fa248, 76 },
    { 0x950fa249, 46 },
    { 0xf94fa24a, 90 },
    { 0, 0 },
    { 0xdb2fa24c, 61 },
    { 0x38fa24d, 95 },
    { 0x2ffa24d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24fa252, 95 },
    { 0x815fa253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82fa25b, 88 },
    { 0, 0 },
    { 0x7a0fa25d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24efa260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x237fa263, 75 },
    { 0x569fa264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70efa26c, 61 },
    { 0x822fa26d, 61 },
    { 0xed4fa26e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cfa27a, 61 },
    { 0x939fa27a, 23 },
    { 0x438fa27c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fa283, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5afa28f, 56 },
    { 0x27afa290, 98 },
    { 0x5f2fa291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6fa2a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71efa2a3, 98 },
    { 0xdb8fa2a4, 33 },
    { 0x714fa2a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0fa2ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5fa2b1, 90 },
    { 0x83ffa2b1, 90 },
    { 0x958fa2b1, 33 },
    { 0x304fa2b3, 98 },
    { 0xda2fa2b4, 61 },
    { 0, 0 },
    { 0xc1bfa2b7, 83 },
    { 0, 0 },
    { 0x859fa2b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55fa2bc, 95 },
    { 0x7b3fa2bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9fa2c2, 76 },
    { 0, 0 },
    { 0x729fa2c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8fa2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ffa2d6, 78 },
    { 0xb5efa2d6, 61 },
    { 0xd26fa2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afa2db, 33 },
    { 0xc45fa2dc, 95 },
    { 0x927fa2dc, 95 },
    { 0x26bfa2de, 60 },
    { 0xc55fa2df, 85 },
    { 0x73fa2e0, 33 },
    { 0xd6ffa2e1, 61 },
    { 0, 0 },
    { 0xaf0fa2e3, 23 },
    { 0xdfffa2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34fa2ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fa2f0, 91 },
    { 0x3e6fa2f1, 61 },
    { 0x81fa2f2, 63 },
    { 0, 0 },
    { 0x288fa2f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25fa2f8, 65 },
    { 0xa4cfa2f9, 23 },
    { 0x1c8fa2f9, 95 },
    { 0x955fa2fb, 33 },
    { 0xfa6fa2f8, 34 },
    { 0, 0 },
    { 0xd7cfa2fe, 90 },
    { 0xeb8fa2ff, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fa304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715fa309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99fa30c, 23 },
    { 0x5f9fa30d, 95 },
    { 0, 0 },
    { 0xd2efa30f, 61 },
    { 0xea3fa30f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dffa316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309fa31a, 95 },
    { 0x672fa31b, 90 },
    { 0, 0 },
    { 0xcd7fa31d, 76 },
    { 0x8befa31d, 93 },
    { 0x262fa31f, 23 },
    { 0, 0 },
    { 0x39cfa321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acfa326, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6fa32a, 95 },
    { 0, 0 },
    { 0x17efa32c, 67 },
    { 0x523fa32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x742fa330, 75 },
    { 0x7c0fa331, 98 },
    { 0x2a2fa331, 98 },
    { 0x54afa332, 100 },
    { 0x6a5fa334, 61 },
    { 0x170fa335, 98 },
    { 0x788fa333, 66 },
    { 0xa3ffa336, 91 },
    { 0x8b7fa338, 87 },
    { 0xad2fa339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x633fa33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7fa344, 90 },
    { 0x4b0fa345, 17 },
    { 0x752fa344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08fa34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0fa34d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150fa351, 43 },
    { 0, 0 },
    { 0x41afa353, 98 },
    { 0, 0 },
    { 0xe8efa355, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284fa35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89fa35d, 93 },
    { 0, 0 },
    { 0x6bffa35f, 90 },
    { 0xf87fa35f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03fa363, 78 },
    { 0x5e0fa364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fa368, 95 },
    { 0xccffa369, 68 },
    { 0x1e1fa369, 98 },
    { 0x8dcfa36b, 33 },
    { 0xd9ffa369, 95 },
    { 0x1aafa36a, 90 },
    { 0, 0 },
    { 0xc82fa36f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3afa372, 93 },
    { 0xf65fa372, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afa37a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fa37e, 77 },
    { 0xf07fa37f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8fa382, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadfa385, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efa38b, 75 },
    { 0xc8bfa38c, 52 },
    { 0, 0 },
    { 0x7b7fa38e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41fa394, 90 },
    { 0x8b2fa394, 90 },
    { 0xc08fa396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa39b, 34 },
    { 0x90cfa39b, 52 },
    { 0x50fa39b, 98 },
    { 0, 0 },
    { 0xc94fa39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fa3a2, 52 },
    { 0xefafa3a3, 90 },
    { 0x28efa3a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cfa3a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ffa3af, 90 },
    { 0x996fa3b0, 95 },
    { 0, 0 },
    { 0x457fa3b2, 98 },
    { 0, 0 },
    { 0xc86fa3b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8defa3b7, 61 },
    { 0x696fa3b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215fa3bc, 95 },
    { 0, 0 },
    { 0xbb8fa3be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1fa3c2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8fa3c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12afa3cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36fa3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61afa3d8, 34 },
    { 0xfccfa3d9, 95 },
    { 0, 0 },
    { 0xca2fa3db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fa3de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfefa3e1, 23 },
    { 0xd97fa3e2, 33 },
    { 0, 0 },
    { 0xfc1fa3e4, 61 },
    { 0, 0 },
    { 0xa9dfa3e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dfa3e9, 95 },
    { 0xcafa3ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa3ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fafa3f5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bfa3f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0fa3fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73fa3ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fa407, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cfa40c, 55 },
    { 0, 0 },
    { 0xba6fa40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406fa411, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dfa414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84fa419, 76 },
    { 0xc81fa41a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994fa41f, 95 },
    { 0, 0 },
    { 0x34afa421, 33 },
    { 0, 0 },
    { 0x714fa423, 17 },
    { 0, 0 },
    { 0x80afa425, 34 },
    { 0x392fa425, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07fa429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38fa42e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa431, 75 },
    { 0, 0 },
    { 0xcc6fa433, 90 },
    { 0x141fa433, 43 },
    { 0x1a6fa435, 33 },
    { 0, 0 },
    { 0xf6cfa437, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fa43b, 93 },
    { 0x6a8fa43c, 55 },
    { 0xf44fa43d, 85 },
    { 0x9e2fa43d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8fa441, 34 },
    { 0x7d1fa441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ffa448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ffa44b, 23 },
    { 0, 0 },
    { 0xb1afa44d, 91 },
    { 0x4d2fa44e, 95 },
    { 0xdb1fa44f, 33 },
    { 0xa79fa44d, 33 },
    { 0x8a3fa44e, 95 },
    { 0x20cfa452, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dfa459, 77 },
    { 0x8d0fa459, 87 },
    { 0x9b0fa45b, 93 },
    { 0x17efa45b, 43 },
    { 0x553fa45c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974fa461, 95 },
    { 0x403fa461, 83 },
    { 0, 0 },
    { 0x9f4fa464, 76 },
    { 0, 0 },
    { 0x408fa466, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fa47a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1fa47f, 76 },
    { 0x841fa480, 99 },
    { 0xbe6fa481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x219fa484, 90 },
    { 0x22bfa484, 61 },
    { 0, 0 },
    { 0xa08fa487, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa48b, 95 },
    { 0, 0 },
    { 0xf6cfa48d, 46 },
    { 0xaf5fa48d, 33 },
    { 0x636fa48f, 90 },
    { 0xee4fa490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84fa494, 78 },
    { 0x94efa494, 61 },
    { 0x4dbfa496, 95 },
    { 0xbefa497, 90 },
    { 0, 0 },
    { 0xe2cfa499, 78 },
    { 0xd01fa49a, 88 },
    { 0x903fa49b, 61 },
    { 0xb35fa49c, 93 },
    { 0, 0 },
    { 0x141fa49e, 90 },
    { 0, 0 },
    { 0x509fa4a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110fa4a6, 95 },
    { 0x518fa4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8fa4b1, 85 },
    { 0x989fa4b2, 61 },
    { 0, 0 },
    { 0x113fa4b4, 95 },
    { 0, 0 },
    { 0x46cfa4b6, 90 },
    { 0xeeffa4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfa4c1, 33 },
    { 0x777fa4c2, 61 },
    { 0x968fa4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bfa4ca, 76 },
    { 0xd25fa4cb, 93 },
    { 0, 0 },
    { 0x642fa4cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fa4d1, 95 },
    { 0x953fa4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7efa4d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cfa4d9, 46 },
    { 0, 0 },
    { 0xf19fa4db, 56 },
    { 0xc6afa4db, 90 },
    { 0xb02fa4dd, 36 },
    { 0, 0 },
    { 0x718fa4df, 61 },
    { 0, 0 },
    { 0x9adfa4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2fa4e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ffa4ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dfa4f4, 55 },
    { 0x203fa4f4, 99 },
    { 0x182fa4f6, 90 },
    { 0x189fa4f4, 83 },
    { 0xe01fa4f8, 55 },
    { 0x375fa4f8, 95 },
    { 0xd1cfa4fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x655fa4fd, 90 },
    { 0x12efa4fd, 70 },
    { 0x751fa4ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdefa502, 95 },
    { 0, 0 },
    { 0x4dbfa504, 99 },
    { 0x4b4fa505, 68 },
    { 0x6acfa506, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806fa50e, 98 },
    { 0x412fa50f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815fa514, 87 },
    { 0x9f8fa514, 75 },
    { 0xb54fa514, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98fa51e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fa523, 33 },
    { 0x1cffa523, 90 },
    { 0xdcefa525, 52 },
    { 0x118fa526, 33 },
    { 0xb82fa524, 46 },
    { 0x2b0fa528, 90 },
    { 0xaacfa529, 97 },
    { 0xcd3fa52a, 87 },
    { 0x701fa52b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904fa535, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cfa539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9fa53f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9fa546, 61 },
    { 0, 0 },
    { 0xac0fa548, 33 },
    { 0xdeefa548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fa54e, 24 },
    { 0xfa2fa54f, 98 },
    { 0, 0 },
    { 0xd27fa551, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa555, 93 },
    { 0, 0 },
    { 0xe1efa557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96fa55b, 52 },
    { 0xb60fa55b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9fa560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150fa568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763fa56c, 61 },
    { 0, 0 },
    { 0x22ffa56e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609fa573, 64 },
    { 0, 0 },
    { 0x3e4fa575, 78 },
    { 0, 0 },
    { 0xd1bfa577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcfa57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3efa581, 91 },
    { 0xb35fa581, 38 },
    { 0x457fa582, 23 },
    { 0, 0 },
    { 0x738fa585, 33 },
    { 0x776fa585, 23 },
    { 0x84dfa587, 61 },
    { 0xffffa588, 90 },
    { 0xbd0fa588, 55 },
    { 0, 0 },
    { 0xd95fa58b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa591, 61 },
    { 0, 0 },
    { 0xf37fa593, 90 },
    { 0, 0 },
    { 0xebdfa595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1fa59c, 94 },
    { 0, 0 },
    { 0x475fa59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ffa5a2, 76 },
    { 0x8a8fa5a2, 33 },
    { 0x5dbfa5a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5afa5ad, 43 },
    { 0x153fa5ae, 90 },
    { 0xc71fa5af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7fa5b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fa5b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cfa5bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7fa5bf, 61 },
    { 0x265fa5bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30fa5c4, 61 },
    { 0xdecfa5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856fa5ce, 23 },
    { 0, 0 },
    { 0xb10fa5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3fa5d3, 61 },
    { 0, 0 },
    { 0x6f7fa5d5, 65 },
    { 0x6dcfa5d5, 43 },
    { 0x765fa5d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fa5de, 61 },
    { 0x674fa5df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636fa5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecfa5ec, 86 },
    { 0, 0 },
    { 0xbbffa5ee, 98 },
    { 0x60ffa5ef, 55 },
    { 0x5c0fa5ee, 67 },
    { 0, 0 },
    { 0xefa5f2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14fa5fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51fa5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa601, 33 },
    { 0, 0 },
    { 0x679fa603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfa608, 33 },
    { 0xd66fa609, 80 },
    { 0x499fa608, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103fa60e, 98 },
    { 0, 0 },
    { 0xf32fa610, 23 },
    { 0, 0 },
    { 0x12bfa612, 36 },
    { 0x126fa612, 55 },
    { 0xf2cfa614, 33 },
    { 0, 0 },
    { 0x687fa616, 23 },
    { 0x9ddfa617, 66 },
    { 0x183fa618, 33 },
    { 0xb42fa619, 23 },
    { 0x1dafa617, 76 },
    { 0x748fa61a, 55 },
    { 0xdb2fa61c, 98 },
    { 0, 0 },
    { 0x796fa61e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa623, 100 },
    { 0xd7efa623, 95 },
    { 0x571fa625, 33 },
    { 0xc46fa626, 55 },
    { 0x910fa627, 91 },
    { 0xd4dfa628, 61 },
    { 0x9c9fa627, 95 },
    { 0x42fa626, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271fa62f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dfa632, 23 },
    { 0, 0 },
    { 0x14bfa634, 90 },
    { 0, 0 },
    { 0x40afa636, 98 },
    { 0x5b5fa637, 100 },
    { 0xf2ffa638, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fa63c, 90 },
    { 0, 0 },
    { 0x4fcfa63e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1fa641, 61 },
    { 0, 0 },
    { 0xe7bfa643, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9fa647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efa64e, 33 },
    { 0, 0 },
    { 0x751fa650, 33 },
    { 0, 0 },
    { 0x81cfa652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11fa65d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3fa661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2fa664, 84 },
    { 0, 0 },
    { 0x9e5fa666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ffa669, 33 },
    { 0x48bfa66a, 97 },
    { 0xcbbfa66b, 17 },
    { 0x73fa66c, 87 },
    { 0x4a9fa66d, 23 },
    { 0x424fa66b, 55 },
    { 0x295fa66c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3fa676, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfa67b, 84 },
    { 0xdafa67b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbfa681, 61 },
    { 0x5cefa681, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ffa685, 95 },
    { 0, 0 },
    { 0x41efa687, 34 },
    { 0xfdefa688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4fa68b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5fa68e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85fa692, 34 },
    { 0, 0 },
    { 0xeffa694, 95 },
    { 0, 0 },
    { 0x861fa696, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa69b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4affa69e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebfa6a2, 34 },
    { 0xa7ffa6a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa6ad, 76 },
    { 0xf62fa6ae, 93 },
    { 0, 0 },
    { 0xba4fa6b0, 46 },
    { 0, 0 },
    { 0x8e2fa6b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6fa6b5, 55 },
    { 0x73cfa6b5, 61 },
    { 0xb6dfa6b7, 61 },
    { 0x748fa6b8, 66 },
    { 0xff6fa6b7, 93 },
    { 0xdafa6ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82fa6be, 61 },
    { 0x6e7fa6bf, 100 },
    { 0x1abfa6bf, 90 },
    { 0x6bdfa6bf, 98 },
    { 0xc78fa6c0, 87 },
    { 0, 0 },
    { 0x63ffa6c4, 95 },
    { 0xb70fa6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fefa6c8, 17 },
    { 0, 0 },
    { 0x683fa6ca, 90 },
    { 0x5b1fa6cb, 61 },
    { 0x423fa6cc, 33 },
    { 0xec3fa6cb, 91 },
    { 0x836fa6ce, 76 },
    { 0, 0 },
    { 0x92fa6d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67fa6d4, 90 },
    { 0x85bfa6d5, 23 },
    { 0, 0 },
    { 0xde2fa6d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2fa6da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3fa6de, 76 },
    { 0xc9efa6de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2efa6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffa6ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366fa6f0, 95 },
    { 0x7e4fa6f1, 33 },
    { 0x89cfa6f2, 95 },
    { 0, 0 },
    { 0xb7fa6f4, 90 },
    { 0, 0 },
    { 0x8a7fa6f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10fa6fb, 90 },
    { 0xf5efa6fc, 33 },
    { 0xef4fa6fb, 61 },
    { 0xf1cfa6fd, 33 },
    { 0xc53fa6fc, 33 },
    { 0x531fa6fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffa703, 32 },
    { 0, 0 },
    { 0xdb5fa705, 95 },
    { 0x2c1fa705, 55 },
    { 0xf64fa707, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2fa70c, 51 },
    { 0x463fa70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cfa715, 85 },
    { 0xee7fa716, 33 },
    { 0x17bfa716, 61 },
    { 0xdbdfa715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74afa71e, 51 },
    { 0, 0 },
    { 0xcb9fa720, 95 },
    { 0x90fa720, 90 },
    { 0, 0 },
    { 0x1befa723, 33 },
    { 0x623fa723, 48 },
    { 0, 0 },
    { 0x1e3fa726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffa729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591fa72d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cfa73e, 55 },
    { 0, 0 },
    { 0xd08fa740, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fa744, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebfa747, 61 },
    { 0xa26fa747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fa74b, 90 },
    { 0x3cffa74c, 61 },
    { 0, 0 },
    { 0x630fa74e, 33 },
    { 0x9d1fa74e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48afa754, 93 },
    { 0x4a2fa754, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38fa75a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4afa75d, 76 },
    { 0, 0 },
    { 0x5c9fa75f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1befa765, 90 },
    { 0, 0 },
    { 0x3abfa767, 98 },
    { 0x65fa768, 33 },
    { 0xa1ffa769, 98 },
    { 0, 0 },
    { 0xb63fa76b, 46 },
    { 0x2e8fa76c, 98 },
    { 0x933fa76d, 61 },
    { 0xf11fa76b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dfa774, 95 },
    { 0x836fa775, 61 },
    { 0x870fa776, 55 },
    { 0xc2fa776, 90 },
    { 0x55fa778, 33 },
    { 0x83dfa779, 90 },
    { 0x3aafa779, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bfa77f, 90 },
    { 0x5b9fa780, 95 },
    { 0x94fa780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659fa785, 61 },
    { 0x64efa785, 95 },
    { 0, 0 },
    { 0xa43fa788, 99 },
    { 0x5abfa789, 81 },
    { 0x5d4fa789, 61 },
    { 0x625fa78b, 33 },
    { 0, 0 },
    { 0xb11fa78d, 52 },
    { 0x7abfa78d, 34 },
    { 0xc6ffa78f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41fa793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dfa797, 90 },
    { 0xa27fa798, 61 },
    { 0xf59fa799, 81 },
    { 0xdb9fa79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435fa7a2, 33 },
    { 0xe7afa7a3, 93 },
    { 0x20dfa7a4, 34 },
    { 0xc72fa7a5, 95 },
    { 0, 0 },
    { 0xa7fa7a7, 95 },
    { 0x81ffa7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfa7ab, 61 },
    { 0xa02fa7ac, 90 },
    { 0xcd8fa7ac, 33 },
    { 0xc00fa7ae, 55 },
    { 0xdf9fa7ae, 34 },
    { 0x27fa7b0, 34 },
    { 0, 0 },
    { 0x9a3fa7b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efa7b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa7b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1effa7c1, 88 },
    { 0, 0 },
    { 0x319fa7c3, 67 },
    { 0, 0 },
    { 0x924fa7c5, 91 },
    { 0xb68fa7c6, 40 },
    { 0xaa0fa7c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bfa7cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0fa7d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39afa7d7, 36 },
    { 0x6fbfa7d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454fa7dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3fa7e0, 43 },
    { 0x5dfa7e1, 46 },
    { 0xe62fa7e1, 90 },
    { 0xb30fa7e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x448fa7e6, 52 },
    { 0x61afa7e6, 33 },
    { 0x77efa7e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cfa7ec, 90 },
    { 0x46ffa7ed, 95 },
    { 0x598fa7ee, 65 },
    { 0x4f5fa7ee, 87 },
    { 0x152fa7f0, 23 },
    { 0xebffa7f0, 77 },
    { 0x481fa7f0, 98 },
    { 0x6e0fa7f0, 36 },
    { 0, 0 },
    { 0x250fa7f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16fa7f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfa7fc, 33 },
    { 0x966fa7fd, 61 },
    { 0xab3fa7fd, 90 },
    { 0x704fa7ff, 33 },
    { 0xa59fa7fe, 95 },
    { 0, 0 },
    { 0x4dbfa802, 33 },
    { 0x1aefa802, 66 },
    { 0xe9afa803, 23 },
    { 0xbc5fa805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0fa80a, 95 },
    { 0, 0 },
    { 0x5afa80c, 23 },
    { 0x611fa80c, 91 },
    { 0, 0 },
    { 0x40fa80f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93fa813, 95 },
    { 0x6afa813, 90 },
    { 0x457fa813, 33 },
    { 0xae8fa816, 99 },
    { 0x6fa817, 61 },
    { 0xd62fa818, 76 },
    { 0, 0 },
    { 0xd37fa81a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x677fa81d, 46 },
    { 0, 0 },
    { 0xda2fa81f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338fa829, 90 },
    { 0xc6ffa829, 76 },
    { 0, 0 },
    { 0xcd1fa82c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7fa830, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2fa834, 33 },
    { 0x93fa835, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fa83b, 55 },
    { 0x888fa83c, 85 },
    { 0xdfefa83b, 67 },
    { 0xcc4fa83e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fa841, 76 },
    { 0xf62fa842, 93 },
    { 0x3bcfa843, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35fa84b, 33 },
    { 0x686fa84c, 90 },
    { 0, 0 },
    { 0x48dfa84e, 23 },
    { 0, 0 },
    { 0xee0fa850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74fa853, 61 },
    { 0x9a0fa854, 100 },
    { 0x58cfa855, 98 },
    { 0xb3ffa856, 40 },
    { 0x316fa853, 92 },
    { 0, 0 },
    { 0xe94fa859, 100 },
    { 0xc36fa859, 61 },
    { 0xf46fa85a, 90 },
    { 0x92cfa859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecffa863, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1fa86d, 99 },
    { 0, 0 },
    { 0xb20fa86f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bfa873, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fa877, 95 },
    { 0, 0 },
    { 0x338fa879, 93 },
    { 0, 0 },
    { 0x87cfa87b, 33 },
    { 0x97bfa87c, 98 },
    { 0x858fa87d, 61 },
    { 0xcd7fa87e, 90 },
    { 0xeb0fa87f, 23 },
    { 0x8a9fa880, 95 },
    { 0x676fa880, 69 },
    { 0, 0 },
    { 0x545fa883, 90 },
    { 0xcd2fa884, 66 },
    { 0x5c8fa885, 95 },
    { 0, 0 },
    { 0x2e7fa887, 23 },
    { 0, 0 },
    { 0x4eefa889, 100 },
    { 0, 0 },
    { 0x66fa88b, 33 },
    { 0x6f2fa88b, 61 },
    { 0, 0 },
    { 0x11bfa88e, 87 },
    { 0, 0 },
    { 0x236fa890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1fa899, 46 },
    { 0x2a3fa89a, 49 },
    { 0x2bcfa899, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fa89e, 95 },
    { 0xa2ffa89f, 23 },
    { 0x6f6fa8a0, 99 },
    { 0x32dfa8a1, 98 },
    { 0xe2efa8a2, 66 },
    { 0x411fa89e, 90 },
    { 0x4e9fa8a4, 80 },
    { 0xbf6fa89e, 76 },
    { 0, 0 },
    { 0xe7dfa8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949fa8ac, 61 },
    { 0xea4fa8ac, 98 },
    { 0xc29fa8ae, 67 },
    { 0xeefa8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835fa8b7, 90 },
    { 0xdacfa8b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fa8bd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbfa8c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ffa8c6, 61 },
    { 0xd01fa8c7, 90 },
    { 0, 0 },
    { 0x32dfa8c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1fa8cc, 85 },
    { 0, 0 },
    { 0x6c7fa8ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fa8d2, 23 },
    { 0x992fa8d2, 95 },
    { 0, 0 },
    { 0x539fa8d5, 91 },
    { 0, 0 },
    { 0x7ffa8d7, 64 },
    { 0xc75fa8d8, 61 },
    { 0x677fa8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50fa8df, 33 },
    { 0, 0 },
    { 0xf1ffa8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41fa8e4, 85 },
    { 0, 0 },
    { 0xcc9fa8e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cfa8ea, 98 },
    { 0xda5fa8ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fa8ee, 61 },
    { 0x6fffa8ee, 90 },
    { 0xb82fa8f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6fa8fb, 61 },
    { 0xd48fa8fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x760fa8ff, 90 },
    { 0x66efa900, 55 },
    { 0, 0 },
    { 0xe94fa902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1fa90f, 95 },
    { 0, 0 },
    { 0x44cfa911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fa915, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fa919, 33 },
    { 0, 0 },
    { 0xf26fa91b, 93 },
    { 0x751fa91c, 33 },
    { 0xa40fa91d, 33 },
    { 0, 0 },
    { 0x2d9fa91f, 95 },
    { 0x399fa91f, 95 },
    { 0, 0 },
    { 0x7e5fa922, 87 },
    { 0, 0 },
    { 0x236fa924, 90 },
    { 0xb2fa924, 98 },
    { 0xdd9fa926, 98 },
    { 0x3ecfa927, 33 },
    { 0x528fa928, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bfa931, 61 },
    { 0, 0 },
    { 0x978fa933, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fa937, 56 },
    { 0xb6efa937, 33 },
    { 0xcfa939, 23 },
    { 0xeabfa939, 90 },
    { 0x840fa93b, 84 },
    { 0xb84fa93c, 33 },
    { 0xec0fa93d, 23 },
    { 0x123fa93e, 90 },
    { 0x1eefa93b, 99 },
    { 0x57ffa940, 78 },
    { 0xe01fa93b, 90 },
    { 0xe10fa938, 33 },
    { 0x811fa943, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafffa947, 88 },
    { 0xcf6fa947, 98 },
    { 0xa33fa949, 88 },
    { 0x987fa94a, 55 },
    { 0, 0 },
    { 0xaedfa94c, 90 },
    { 0x7cffa94d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ffa956, 90 },
    { 0xdcbfa957, 99 },
    { 0x338fa957, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfa962, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bfa965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533fa96c, 91 },
    { 0x144fa96d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46fa970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dafa974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787fa979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287fa97d, 78 },
    { 0, 0 },
    { 0xf84fa97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fa984, 46 },
    { 0x44bfa985, 56 },
    { 0xe86fa986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fa98b, 23 },
    { 0x73bfa98c, 90 },
    { 0x4b2fa98c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50fa990, 76 },
    { 0, 0 },
    { 0x33efa992, 55 },
    { 0, 0 },
    { 0x2e6fa994, 87 },
    { 0xd4ffa994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fa99c, 33 },
    { 0xb1bfa99d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfafa9aa, 81 },
    { 0, 0 },
    { 0x8f8fa9ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fa9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ffa9b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fa9b9, 33 },
    { 0x3b2fa9ba, 61 },
    { 0xbcafa9bb, 61 },
    { 0x3ebfa9b9, 33 },
    { 0x416fa9bd, 33 },
    { 0xb47fa9b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790fa9c2, 46 },
    { 0, 0 },
    { 0x7a0fa9c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931fa9c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1fa9ce, 95 },
    { 0, 0 },
    { 0xc6cfa9d0, 33 },
    { 0x912fa9d1, 87 },
    { 0xc29fa9d2, 90 },
    { 0xac8fa9d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bfa9d6, 78 },
    { 0xb22fa9d7, 95 },
    { 0xb11fa9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bfa9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804fa9e0, 55 },
    { 0x688fa9e0, 76 },
    { 0x442fa9e2, 76 },
    { 0x365fa9e0, 98 },
    { 0xcc1fa9e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71fa9e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cfa9ed, 55 },
    { 0x9eefa9ee, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4befa9f2, 40 },
    { 0xc78fa9f2, 91 },
    { 0xcfa9f4, 63 },
    { 0x4c4fa9f5, 90 },
    { 0xe37fa9f4, 100 },
    { 0xfa9fa9f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83afa9fa, 61 },
    { 0xdeefa9fb, 55 },
    { 0x18afa9fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x153fa9ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0faa05, 88 },
    { 0x8bdfaa06, 62 },
    { 0x105faa05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfaa0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffaa13, 52 },
    { 0, 0 },
    { 0x117faa15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2faa1a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9faa1d, 66 },
    { 0x903faa1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2faa24, 88 },
    { 0, 0 },
    { 0x4c0faa26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39efaa2e, 23 },
    { 0, 0 },
    { 0x68cfaa30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bfaa33, 90 },
    { 0xafcfaa34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x168faa37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x805faa3a, 34 },
    { 0x2fefaa3a, 80 },
    { 0, 0 },
    { 0x8acfaa3d, 87 },
    { 0xc6afaa3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5faa42, 99 },
    { 0x41afaa43, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ffaa46, 90 },
    { 0x2fdfaa46, 91 },
    { 0xabafaa48, 43 },
    { 0x272faa49, 76 },
    { 0x584faa4a, 85 },
    { 0x809faa48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72faa50, 83 },
    { 0, 0 },
    { 0xdd9faa52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ffaa55, 100 },
    { 0x85ffaa55, 23 },
    { 0xb43faa56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x850faa5a, 87 },
    { 0x970faa5b, 99 },
    { 0x3e1faa5c, 95 },
    { 0x7ecfaa5c, 33 },
    { 0x633faa5e, 100 },
    { 0x220faa5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12faa63, 76 },
    { 0xee7faa64, 98 },
    { 0x959faa64, 23 },
    { 0xbb7faa65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x573faa69, 43 },
    { 0, 0 },
    { 0x561faa6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39faa70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x614faa73, 90 },
    { 0xae0faa74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85efaa79, 76 },
    { 0, 0 },
    { 0x52ffaa7b, 68 },
    { 0xaa0faa7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819faa83, 48 },
    { 0, 0 },
    { 0xef3faa85, 23 },
    { 0, 0 },
    { 0xe63faa87, 99 },
    { 0x37ffaa88, 61 },
    { 0xca7faa88, 76 },
    { 0x26bfaa89, 85 },
    { 0x5d3faa8b, 46 },
    { 0xb5ffaa8c, 46 },
    { 0xdf4faa8c, 33 },
    { 0x6b9faa88, 76 },
    { 0x863faa8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0faa93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0faa96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384faa9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efaa9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549faaa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfaaa9, 61 },
    { 0x689faaaa, 55 },
    { 0x61faaab, 34 },
    { 0xb15faaa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9afaab2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40faab9, 87 },
    { 0xb97faab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91faabe, 95 },
    { 0, 0 },
    { 0x263faac0, 98 },
    { 0x404faac1, 77 },
    { 0x831faac0, 61 },
    { 0, 0 },
    { 0x534faac4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8afaaca, 90 },
    { 0xf0afaacb, 88 },
    { 0x6f1faacc, 77 },
    { 0xe53faacc, 34 },
    { 0x3abfaace, 56 },
    { 0xfeffaace, 100 },
    { 0x817faacb, 95 },
    { 0x82afaad1, 70 },
    { 0, 0 },
    { 0xdfdfaad3, 61 },
    { 0xdeafaad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfaad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93afaadb, 17 },
    { 0x74bfaadc, 90 },
    { 0xc1cfaadd, 33 },
    { 0, 0 },
    { 0xf21faadf, 33 },
    { 0x1aafaadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadafaae7, 23 },
    { 0x5efaae8, 78 },
    { 0, 0 },
    { 0x3b3faaea, 100 },
    { 0x19dfaaeb, 95 },
    { 0x91bfaaeb, 76 },
    { 0x3d9faaeb, 98 },
    { 0x3e8faaee, 90 },
    { 0x7bbfaaee, 99 },
    { 0x7c9faaef, 90 },
    { 0xb9dfaaf1, 61 },
    { 0x71bfaaf2, 77 },
    { 0x768faaf2, 90 },
    { 0x591faaf1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeffaaf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fab00, 90 },
    { 0, 0 },
    { 0x1d0fab02, 90 },
    { 0, 0 },
    { 0xdaafab04, 33 },
    { 0x6b4fab05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x474fab08, 33 },
    { 0x1e0fab09, 67 },
    { 0xa7bfab0a, 61 },
    { 0xfccfab09, 23 },
    { 0x2e7fab0c, 98 },
    { 0xf8dfab0c, 84 },
    { 0x2e6fab08, 98 },
    { 0xbd8fab0f, 33 },
    { 0xfa1fab10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3fab13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2fab18, 34 },
    { 0x48afab19, 34 },
    { 0xa77fab1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fab1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3fab24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ffab2a, 100 },
    { 0x8d3fab2b, 78 },
    { 0xca4fab2a, 61 },
    { 0x7d1fab2d, 76 },
    { 0x7ecfab2e, 100 },
    { 0x5c7fab2c, 33 },
    { 0, 0 },
    { 0x7b7fab31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fab37, 46 },
    { 0, 0 },
    { 0xc00fab39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36fab3c, 34 },
    { 0xa16fab3c, 99 },
    { 0xeb7fab3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbfab42, 95 },
    { 0xd05fab43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62fab48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5fab4b, 76 },
    { 0x102fab4c, 33 },
    { 0x795fab4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fab52, 83 },
    { 0xcb4fab53, 33 },
    { 0x1eafab54, 34 },
    { 0, 0 },
    { 0x326fab56, 78 },
    { 0x9e8fab57, 52 },
    { 0, 0 },
    { 0x970fab59, 81 },
    { 0, 0 },
    { 0xf24fab5b, 36 },
    { 0xaaffab5c, 33 },
    { 0xfe8fab5d, 23 },
    { 0x535fab5b, 46 },
    { 0, 0 },
    { 0xc8ffab60, 34 },
    { 0x5aafab61, 81 },
    { 0, 0 },
    { 0x190fab63, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3defab68, 33 },
    { 0, 0 },
    { 0x1a7fab6a, 33 },
    { 0x2a6fab6b, 100 },
    { 0, 0 },
    { 0x156fab6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fab72, 23 },
    { 0x2a2fab73, 85 },
    { 0xe48fab73, 90 },
    { 0x549fab75, 46 },
    { 0, 0 },
    { 0xe48fab77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fab7f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdfab83, 61 },
    { 0x46dfab84, 23 },
    { 0x235fab85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408fab8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62fab91, 82 },
    { 0, 0 },
    { 0xef1fab93, 46 },
    { 0, 0 },
    { 0x834fab95, 98 },
    { 0x50dfab96, 34 },
    { 0x9b9fab97, 55 },
    { 0x87bfab98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4fab9e, 76 },
    { 0xd46fab9f, 95 },
    { 0x35afaba0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaba7, 61 },
    { 0x6e7faba8, 83 },
    { 0x503faba8, 61 },
    { 0xfecfaba8, 99 },
    { 0x2e6faba7, 90 },
    { 0xd6ffabac, 61 },
    { 0x1bffaba9, 90 },
    { 0x3f7fabae, 90 },
    { 0x349fabaf, 48 },
    { 0, 0 },
    { 0xa94fabb1, 78 },
    { 0xbb6fabb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23fabb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x804fabb9, 95 },
    { 0, 0 },
    { 0x11efabbb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bfabc3, 61 },
    { 0xeb3fabc4, 61 },
    { 0xd77fabc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dfabca, 33 },
    { 0x79ffabca, 95 },
    { 0x734fabcc, 76 },
    { 0xfbafabcd, 66 },
    { 0x455fabce, 100 },
    { 0, 0 },
    { 0x30bfabd0, 33 },
    { 0x317fabd1, 55 },
    { 0x625fabd1, 67 },
    { 0, 0 },
    { 0xa3afabd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0fabd8, 61 },
    { 0, 0 },
    { 0x8dffabda, 76 },
    { 0, 0 },
    { 0xd03fabdc, 84 },
    { 0x9dcfabdc, 61 },
    { 0xcdafabde, 33 },
    { 0xd33fabdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdfabe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fabe7, 76 },
    { 0xb12fabe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x340fabeb, 33 },
    { 0x9c5fabeb, 90 },
    { 0, 0 },
    { 0x819fabee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fabf1, 61 },
    { 0x585fabf1, 90 },
    { 0x4effabf2, 67 },
    { 0, 0 },
    { 0x73dfabf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fabfa, 90 },
    { 0x9d8fabfb, 61 },
    { 0x88ffabfc, 55 },
    { 0x2a6fabfa, 95 },
    { 0xba4fabfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8fac01, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fac07, 93 },
    { 0x8f4fac08, 95 },
    { 0xf03fac08, 46 },
    { 0xaacfac07, 95 },
    { 0x5a6fac08, 61 },
    { 0, 0 },
    { 0xfc2fac0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90fac10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76fac15, 23 },
    { 0xa98fac15, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cfac1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bfac1f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3fac25, 90 },
    { 0x723fac25, 33 },
    { 0x2b5fac25, 90 },
    { 0, 0 },
    { 0x780fac29, 90 },
    { 0x327fac29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350fac2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afac31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270fac36, 90 },
    { 0x114fac37, 23 },
    { 0x3d9fac38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23fac3b, 85 },
    { 0xabffac3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9fac47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88fac52, 66 },
    { 0, 0 },
    { 0x64dfac54, 66 },
    { 0x5b7fac55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1fac5a, 98 },
    { 0x8e3fac5b, 61 },
    { 0xc25fac5c, 91 },
    { 0x31dfac5d, 61 },
    { 0x877fac5d, 36 },
    { 0, 0 },
    { 0xfbffac60, 34 },
    { 0xbdafac60, 98 },
    { 0xc4bfac61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0fac66, 23 },
    { 0x234fac66, 33 },
    { 0x1a8fac67, 55 },
    { 0xbbfac69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4fac6d, 95 },
    { 0x1b7fac6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580fac73, 95 },
    { 0, 0 },
    { 0x1b4fac75, 61 },
    { 0x9cafac76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x332fac79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfac7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0fac82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13fac87, 45 },
    { 0xc28fac88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1fac8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2fac8e, 46 },
    { 0xc39fac8e, 23 },
    { 0x902fac90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248fac94, 91 },
    { 0, 0 },
    { 0xca3fac96, 61 },
    { 0xc4bfac97, 23 },
    { 0x73ffac98, 76 },
    { 0xe8efac99, 67 },
    { 0x413fac9a, 33 },
    { 0x1a0fac9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8faca2, 67 },
    { 0, 0 },
    { 0xc89faca4, 34 },
    { 0x47bfaca4, 55 },
    { 0x7dafaca6, 55 },
    { 0x8d4faca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26facab, 90 },
    { 0x68efacac, 61 },
    { 0xaccfacab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0facb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569facb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2facbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccfacbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54facc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736facc9, 85 },
    { 0xca1facca, 61 },
    { 0xff4faccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ffacd0, 90 },
    { 0xc9facd1, 93 },
    { 0, 0 },
    { 0xa39facd3, 43 },
    { 0x275facd3, 55 },
    { 0x2adfacd5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1facdd, 23 },
    { 0, 0 },
    { 0xa88facdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122face6, 93 },
    { 0x6a0face7, 61 },
    { 0, 0 },
    { 0xce2face9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36facf8, 95 },
    { 0, 0 },
    { 0xc99facfa, 98 },
    { 0x7dffacfb, 61 },
    { 0x8effacfc, 46 },
    { 0xcd3facfd, 90 },
    { 0, 0 },
    { 0xbbffacff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238fad03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0fad0b, 23 },
    { 0xda3fad0c, 46 },
    { 0x994fad0b, 61 },
    { 0xfa9fad0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71fad12, 90 },
    { 0x15afad13, 61 },
    { 0xf12fad14, 33 },
    { 0xeafad13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dffad18, 61 },
    { 0, 0 },
    { 0x881fad1a, 61 },
    { 0x7eefad1b, 90 },
    { 0x4cafad1a, 61 },
    { 0x4fffad1d, 99 },
    { 0x44efad1c, 95 },
    { 0xbdfad1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6fad28, 66 },
    { 0x418fad28, 98 },
    { 0x5f3fad29, 55 },
    { 0x65bfad28, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5fad2e, 82 },
    { 0, 0 },
    { 0x13cfad30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6fad36, 55 },
    { 0, 0 },
    { 0xa33fad38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462fad3c, 33 },
    { 0x7dbfad3c, 95 },
    { 0, 0 },
    { 0xedafad3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124fad45, 88 },
    { 0x411fad46, 38 },
    { 0x9edfad47, 43 },
    { 0xc6cfad47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2fad4f, 84 },
    { 0xcbafad50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6fad53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cffad57, 76 },
    { 0x589fad57, 90 },
    { 0, 0 },
    { 0x9a7fad5a, 90 },
    { 0x466fad5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638fad65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfad71, 90 },
    { 0x567fad71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2fad76, 98 },
    { 0, 0 },
    { 0x5b6fad78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fad7f, 90 },
    { 0x406fad80, 40 },
    { 0xbc4fad80, 77 },
    { 0xb93fad80, 55 },
    { 0x3d8fad80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9fad87, 95 },
    { 0x8bffad88, 95 },
    { 0xd1ffad89, 76 },
    { 0, 0 },
    { 0x7dbfad8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fad90, 90 },
    { 0x92ffad91, 78 },
    { 0xaa2fad92, 61 },
    { 0xf03fad93, 61 },
    { 0, 0 },
    { 0x843fad95, 66 },
    { 0x384fad96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08fad9b, 64 },
    { 0, 0 },
    { 0x86fad9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13fada1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x535fada4, 61 },
    { 0x5d8fada5, 76 },
    { 0, 0 },
    { 0x137fada7, 55 },
    { 0, 0 },
    { 0x37fada9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8fadad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffadb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3fadb3, 100 },
    { 0, 0 },
    { 0xdd0fadb5, 61 },
    { 0x24efadb6, 98 },
    { 0xdafadb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bfadba, 33 },
    { 0xa2fadba, 61 },
    { 0x14bfadbc, 68 },
    { 0x629fadbc, 55 },
    { 0x4b7fadba, 34 },
    { 0xb18fadbe, 61 },
    { 0xbf1fadc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fadc7, 76 },
    { 0x14cfadc8, 85 },
    { 0, 0 },
    { 0x8dfadca, 77 },
    { 0x8fefadca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fadd3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53fadd7, 90 },
    { 0x2edfadd8, 93 },
    { 0, 0 },
    { 0x918fadda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54faddd, 98 },
    { 0x2d6fadde, 33 },
    { 0, 0 },
    { 0x32fade0, 99 },
    { 0x13bfade0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05fade4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fadea, 90 },
    { 0x5cafadea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x501fadee, 56 },
    { 0, 0 },
    { 0x995fadf0, 90 },
    { 0x9ffadf0, 96 },
    { 0x43cfadf2, 95 },
    { 0xcbefadf1, 98 },
    { 0, 0 },
    { 0xb5bfadf5, 61 },
    { 0, 0 },
    { 0xb54fadf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29fadfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6fadff, 33 },
    { 0, 0 },
    { 0x3d9fae01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21fae05, 38 },
    { 0x588fae06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cfae10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964fae16, 61 },
    { 0x4b9fae16, 90 },
    { 0xee8fae18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73fae1f, 65 },
    { 0, 0 },
    { 0x896fae21, 33 },
    { 0, 0 },
    { 0xc0dfae23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0fae26, 85 },
    { 0, 0 },
    { 0x40dfae28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbfae2b, 95 },
    { 0, 0 },
    { 0x5dcfae2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922fae32, 23 },
    { 0, 0 },
    { 0x1f7fae34, 91 },
    { 0x432fae35, 55 },
    { 0x267fae36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2fae39, 43 },
    { 0xa94fae3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35fae45, 90 },
    { 0x8ffae46, 76 },
    { 0xaa9fae47, 98 },
    { 0x49ffae47, 93 },
    { 0, 0 },
    { 0x3e7fae4a, 78 },
    { 0, 0 },
    { 0xff2fae4c, 98 },
    { 0, 0 },
    { 0x79bfae4e, 85 },
    { 0, 0 },
    { 0xa6bfae50, 90 },
    { 0, 0 },
    { 0xa23fae52, 90 },
    { 0x747fae53, 23 },
    { 0x53dfae54, 99 },
    { 0x4fffae53, 99 },
    { 0x1f5fae56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88efae60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28efae63, 100 },
    { 0, 0 },
    { 0x302fae65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffae69, 98 },
    { 0, 0 },
    { 0x736fae6b, 90 },
    { 0x8d3fae6c, 83 },
    { 0x632fae6d, 95 },
    { 0, 0 },
    { 0x2cfae6f, 46 },
    { 0x550fae70, 33 },
    { 0x7f5fae6f, 61 },
    { 0xdbefae6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11efae79, 34 },
    { 0x8dcfae7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cfae80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fae88, 90 },
    { 0x67bfae89, 23 },
    { 0, 0 },
    { 0x7c3fae8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791fae8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7fae95, 34 },
    { 0x29bfae96, 46 },
    { 0xcedfae96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x606fae9a, 23 },
    { 0xeeefae9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8fae9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7faea3, 90 },
    { 0, 0 },
    { 0x20faea5, 95 },
    { 0x8a5faea6, 23 },
    { 0, 0 },
    { 0x114faea8, 98 },
    { 0x657faea9, 95 },
    { 0x5a8faeaa, 61 },
    { 0, 0 },
    { 0x240faeac, 99 },
    { 0, 0 },
    { 0x41dfaeae, 76 },
    { 0xf9bfaeaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x916faeb2, 84 },
    { 0x193faeb3, 76 },
    { 0xebdfaeb3, 46 },
    { 0x18afaeb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743faeb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffaebe, 43 },
    { 0, 0 },
    { 0xef5faec0, 33 },
    { 0x802faec0, 90 },
    { 0xc70faec1, 90 },
    { 0xec4faec1, 61 },
    { 0x4b7faec1, 43 },
    { 0xe7ffaec5, 76 },
    { 0x49ffaec5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbafaecb, 61 },
    { 0x318faecc, 90 },
    { 0x525faecb, 43 },
    { 0, 0 },
    { 0x2bffaecf, 42 },
    { 0x6bbfaecf, 61 },
    { 0xfc6faed1, 61 },
    { 0x817faed1, 61 },
    { 0x9d7faed3, 78 },
    { 0xf91faed4, 98 },
    { 0x850faed3, 46 },
    { 0xf6dfaed4, 95 },
    { 0x27cfaed7, 33 },
    { 0xf5dfaed7, 23 },
    { 0x4f1faed9, 61 },
    { 0x231faed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5faedd, 85 },
    { 0, 0 },
    { 0x982faedf, 90 },
    { 0, 0 },
    { 0x71dfaee1, 23 },
    { 0, 0 },
    { 0xe92faee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71faee9, 46 },
    { 0x7cfaeea, 66 },
    { 0, 0 },
    { 0xfc5faeec, 23 },
    { 0x4a7faeed, 23 },
    { 0, 0 },
    { 0x4e5faeef, 66 },
    { 0, 0 },
    { 0xd7faef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4afaef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dfaefb, 23 },
    { 0x1a7faefc, 43 },
    { 0x752faefd, 90 },
    { 0, 0 },
    { 0x23cfaeff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bafaf03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9faf08, 34 },
    { 0x750faf09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dffaf0c, 33 },
    { 0x149faf0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x332faf10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43efaf1e, 33 },
    { 0x4e4faf1f, 88 },
    { 0x5ccfaf20, 90 },
    { 0x506faf20, 76 },
    { 0xe6faf22, 90 },
    { 0x5e3faf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574faf27, 55 },
    { 0x5e4faf28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0faf2b, 33 },
    { 0, 0 },
    { 0x25bfaf2d, 90 },
    { 0, 0 },
    { 0x4f8faf2f, 84 },
    { 0, 0 },
    { 0xbfdfaf31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x903faf34, 98 },
    { 0x790faf35, 23 },
    { 0x9f2faf35, 95 },
    { 0x2defaf37, 77 },
    { 0x8cffaf37, 46 },
    { 0xfb7faf39, 61 },
    { 0x2b9faf34, 87 },
    { 0, 0 },
    { 0xa87faf3c, 78 },
    { 0, 0 },
    { 0xa35faf3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48afaf42, 52 },
    { 0, 0 },
    { 0xd39faf44, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99efaf4a, 34 },
    { 0xc05faf4a, 67 },
    { 0xd3dfaf4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755faf50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298faf56, 72 },
    { 0, 0 },
    { 0xccfaf58, 55 },
    { 0x940faf59, 85 },
    { 0xe8dfaf5a, 90 },
    { 0xd3bfaf59, 46 },
    { 0x42bfaf5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592faf69, 90 },
    { 0, 0 },
    { 0xbfbfaf6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59faf73, 43 },
    { 0xa93faf73, 23 },
    { 0x98afaf73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dfaf7d, 43 },
    { 0x20efaf7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dfaf83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4faf90, 34 },
    { 0xfffaf91, 66 },
    { 0x8c6faf91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afaf96, 52 },
    { 0xb3faf97, 75 },
    { 0xd2efaf98, 90 },
    { 0xe9cfaf99, 100 },
    { 0x7b3faf9a, 78 },
    { 0x969faf99, 85 },
    { 0x39ffaf96, 90 },
    { 0xf67faf9b, 98 },
    { 0, 0 },
    { 0x7ebfaf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fafa2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cfafa7, 76 },
    { 0xe9dfafa7, 90 },
    { 0xc43fafa8, 46 },
    { 0, 0 },
    { 0x7fbfafab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3afafae, 95 },
    { 0x275fafaf, 98 },
    { 0x24ffafaf, 90 },
    { 0x158fafb1, 61 },
    { 0, 0 },
    { 0x3abfafb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fafb7, 76 },
    { 0x25cfafb7, 90 },
    { 0xcd9fafb9, 90 },
    { 0x263fafba, 55 },
    { 0x953fafb8, 87 },
    { 0xa09fafba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412fafc6, 100 },
    { 0x86cfafc6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x101fafca, 48 },
    { 0, 0 },
    { 0x9a3fafcc, 78 },
    { 0x9cfafcd, 95 },
    { 0x51ffafce, 76 },
    { 0x934fafce, 33 },
    { 0x4c6fafd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffafd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddfafd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x456fafd9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dfafdc, 43 },
    { 0x952fafdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffafe3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30fafe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4fafea, 46 },
    { 0x536fafeb, 97 },
    { 0, 0 },
    { 0x92cfafed, 61 },
    { 0xecbfafed, 65 },
    { 0x855fafee, 90 },
    { 0x167fafee, 64 },
    { 0, 0 },
    { 0x522faff2, 34 },
    { 0x4defaff3, 76 },
    { 0xac1faff4, 90 },
    { 0x6c6faff5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59faffe, 95 },
    { 0xb23fafff, 38 },
    { 0, 0 },
    { 0x247fb001, 76 },
    { 0x884fb002, 55 },
    { 0x6a2fb003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549fb008, 61 },
    { 0xbf7fb008, 98 },
    { 0x906fb00a, 55 },
    { 0x73bfb00b, 99 },
    { 0x72fb00b, 101 },
    { 0xfefb00d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x404fb010, 85 },
    { 0, 0 },
    { 0x4d8fb012, 46 },
    { 0x9fb013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3fb016, 23 },
    { 0x44dfb017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bafb01a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8fb01d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb020, 90 },
    { 0x5cfb021, 61 },
    { 0xdc5fb022, 100 },
    { 0x3acfb022, 33 },
    { 0xa35fb023, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10efb027, 67 },
    { 0xe33fb028, 90 },
    { 0x2fdfb029, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cfb02f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444fb03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeefb041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364fb044, 87 },
    { 0, 0 },
    { 0x619fb046, 98 },
    { 0xf8ffb047, 55 },
    { 0, 0 },
    { 0xddbfb049, 87 },
    { 0xe7afb049, 68 },
    { 0, 0 },
    { 0x3a6fb04c, 84 },
    { 0xe17fb04c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbfb050, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x25efb053, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914fb058, 66 },
    { 0xaa2fb058, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cfb05d, 90 },
    { 0, 0 },
    { 0xb23fb05f, 78 },
    { 0x34fb060, 17 },
    { 0x25afb061, 55 },
    { 0xca6fb062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffb066, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb069, 90 },
    { 0x54fb069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfb06e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ffb073, 43 },
    { 0, 0 },
    { 0x976fb075, 56 },
    { 0, 0 },
    { 0x3b7fb077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711fb07b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb07e, 23 },
    { 0, 0 },
    { 0xcc9fb080, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x615fb083, 33 },
    { 0x4efb083, 90 },
    { 0x5a8fb085, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fb08a, 77 },
    { 0xa4dfb08a, 61 },
    { 0x310fb08a, 33 },
    { 0x930fb08d, 52 },
    { 0x1fb08e, 61 },
    { 0, 0 },
    { 0x6e5fb090, 43 },
    { 0, 0 },
    { 0xdd6fb092, 55 },
    { 0x606fb093, 90 },
    { 0xdfbfb092, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dfb09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x967fb09f, 61 },
    { 0x417fb09f, 33 },
    { 0xffcfb0a1, 90 },
    { 0x61fb0a2, 53 },
    { 0, 0 },
    { 0x7b0fb0a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438fb0a8, 93 },
    { 0, 0 },
    { 0xbcdfb0aa, 53 },
    { 0xa50fb0aa, 55 },
    { 0xda5fb0ab, 90 },
    { 0, 0 },
    { 0xb1bfb0ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633fb0b6, 33 },
    { 0x7f3fb0b7, 95 },
    { 0x20ffb0b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9defb0be, 34 },
    { 0, 0 },
    { 0xfe0fb0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacfb0c6, 98 },
    { 0x545fb0c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0fb0d6, 46 },
    { 0, 0 },
    { 0x2b7fb0d8, 43 },
    { 0x33dfb0d9, 46 },
    { 0x5fafb0da, 23 },
    { 0x522fb0db, 76 },
    { 0, 0 },
    { 0x2e0fb0dd, 76 },
    { 0xfc6fb0de, 76 },
    { 0, 0 },
    { 0x55ffb0e0, 90 },
    { 0x398fb0e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bfb0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dffb0e7, 33 },
    { 0x17efb0e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0fb0eb, 23 },
    { 0, 0 },
    { 0xe5afb0ed, 23 },
    { 0x1e4fb0ed, 19 },
    { 0, 0 },
    { 0xacbfb0f0, 76 },
    { 0xaaffb0f0, 46 },
    { 0, 0 },
    { 0xf75fb0f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97fb0f6, 19 },
    { 0, 0 },
    { 0xf21fb0f8, 90 },
    { 0xccbfb0f8, 78 },
    { 0x935fb0f9, 75 },
    { 0xea8fb0fb, 98 },
    { 0x6dffb0fc, 33 },
    { 0xacbfb0fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60fb100, 99 },
    { 0, 0 },
    { 0x149fb102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ffb105, 98 },
    { 0x314fb106, 34 },
    { 0, 0 },
    { 0x540fb108, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bfb10c, 55 },
    { 0xbb0fb10d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78fb111, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553fb115, 23 },
    { 0xee5fb116, 23 },
    { 0, 0 },
    { 0x948fb118, 95 },
    { 0, 0 },
    { 0x1e8fb11a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdfb123, 98 },
    { 0, 0 },
    { 0x1f1fb125, 76 },
    { 0x82fb126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28fb129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ffb12d, 43 },
    { 0x3f3fb12d, 85 },
    { 0x913fb12d, 61 },
    { 0x7fafb130, 76 },
    { 0xa9ffb131, 98 },
    { 0x43efb12e, 66 },
    { 0, 0 },
    { 0xe79fb134, 90 },
    { 0x978fb134, 23 },
    { 0, 0 },
    { 0x273fb137, 23 },
    { 0x4bbfb138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fb142, 95 },
    { 0, 0 },
    { 0x16cfb144, 90 },
    { 0xca6fb145, 34 },
    { 0, 0 },
    { 0x26fb147, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7afb152, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438fb157, 63 },
    { 0, 0 },
    { 0xe24fb159, 43 },
    { 0x716fb15a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63fb15d, 100 },
    { 0, 0 },
    { 0x61fb15f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71fb164, 100 },
    { 0xa2cfb164, 43 },
    { 0x7f4fb166, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6fb16b, 23 },
    { 0x256fb16c, 98 },
    { 0x6adfb16b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16fb172, 90 },
    { 0x375fb173, 61 },
    { 0x391fb172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357fb178, 66 },
    { 0x7a5fb178, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x106fb17c, 33 },
    { 0, 0 },
    { 0x71fb17e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11fb182, 98 },
    { 0xf0fb182, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85fb188, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6fb18c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450fb194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x844fb197, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852fb19b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495fb1a3, 91 },
    { 0x7dbfb1a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fb1a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeffb1ab, 87 },
    { 0xf4bfb1ac, 66 },
    { 0, 0 },
    { 0x3e0fb1ae, 80 },
    { 0x5c7fb1ae, 34 },
    { 0x29bfb1b0, 88 },
    { 0, 0 },
    { 0x350fb1b2, 61 },
    { 0xa9afb1b3, 33 },
    { 0x231fb1b3, 76 },
    { 0x185fb1b5, 98 },
    { 0xb28fb1b3, 61 },
    { 0x544fb1b7, 95 },
    { 0x443fb1b2, 93 },
    { 0x42efb1b9, 33 },
    { 0x713fb1b9, 66 },
    { 0x154fb1bb, 95 },
    { 0x462fb1bc, 43 },
    { 0xa7afb1b9, 53 },
    { 0x8c4fb1bd, 33 },
    { 0x4b9fb1bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fb1c5, 77 },
    { 0, 0 },
    { 0xa86fb1c7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cfb1ca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fb1cf, 9 },
    { 0x8fafb1d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb1d5, 34 },
    { 0xd0dfb1d6, 33 },
    { 0x198fb1d7, 61 },
    { 0xcd6fb1d8, 100 },
    { 0x2b5fb1d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dfb1dc, 61 },
    { 0xeaefb1dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x907fb1e0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382fb1e5, 77 },
    { 0xc9afb1e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ffb1e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84fb1ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fb1f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbffb1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7effb1f8, 34 },
    { 0x509fb1f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfb1fc, 90 },
    { 0, 0 },
    { 0x5abfb1fe, 33 },
    { 0, 0 },
    { 0x18ffb200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fb204, 46 },
    { 0x4ffb205, 61 },
    { 0, 0 },
    { 0xc9dfb207, 65 },
    { 0xe5efb208, 87 },
    { 0, 0 },
    { 0x42cfb20a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52fb20d, 98 },
    { 0x793fb20e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09fb211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x139fb217, 23 },
    { 0x1afb218, 33 },
    { 0, 0 },
    { 0x8dafb21a, 66 },
    { 0xab3fb21b, 98 },
    { 0xf5efb21c, 61 },
    { 0x8c2fb21d, 90 },
    { 0x9f5fb21d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dafb223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cfb226, 23 },
    { 0x8b7fb227, 61 },
    { 0x9f9fb228, 90 },
    { 0x36dfb226, 98 },
    { 0x9f1fb22a, 90 },
    { 0xf63fb22a, 43 },
    { 0x809fb22c, 43 },
    { 0xa1fb22c, 98 },
    { 0x443fb22c, 91 },
    { 0xb18fb22f, 87 },
    { 0x5bafb229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557fb236, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb239, 95 },
    { 0x90ffb239, 95 },
    { 0, 0 },
    { 0x3befb23c, 66 },
    { 0x7d7fb23d, 78 },
    { 0, 0 },
    { 0x8d2fb23f, 33 },
    { 0xd1dfb23f, 76 },
    { 0x499fb241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3fb244, 95 },
    { 0x6dafb245, 43 },
    { 0xd41fb246, 64 },
    { 0xbbdfb246, 93 },
    { 0xc17fb248, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dfb24b, 95 },
    { 0xe61fb24c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94fb253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55fb257, 93 },
    { 0x747fb257, 61 },
    { 0x546fb257, 95 },
    { 0xbe0fb25a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffb25d, 33 },
    { 0xf4fb25e, 87 },
    { 0xadafb25f, 90 },
    { 0xfadfb260, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fb264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52fb26d, 23 },
    { 0x467fb26d, 23 },
    { 0x883fb26f, 90 },
    { 0x656fb270, 68 },
    { 0x787fb270, 98 },
    { 0xb45fb272, 23 },
    { 0x9b0fb273, 23 },
    { 0x563fb271, 98 },
    { 0, 0 },
    { 0xa92fb276, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6fb279, 84 },
    { 0, 0 },
    { 0x28cfb27b, 33 },
    { 0xcaafb27b, 90 },
    { 0x851fb27b, 23 },
    { 0x349fb27b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5fb284, 98 },
    { 0, 0 },
    { 0xb5afb286, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddfb289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2fb28c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfb290, 69 },
    { 0xfdcfb291, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810fb297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8fb29a, 65 },
    { 0xdccfb29a, 23 },
    { 0x508fb29c, 98 },
    { 0x181fb29a, 90 },
    { 0x1c2fb29d, 98 },
    { 0x652fb29f, 23 },
    { 0xa72fb29b, 33 },
    { 0x661fb2a1, 77 },
    { 0xb22fb2a2, 95 },
    { 0, 0 },
    { 0xc69fb2a4, 81 },
    { 0xc55fb2a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95fb2a9, 100 },
    { 0xf2afb2aa, 88 },
    { 0, 0 },
    { 0x252fb2ac, 61 },
    { 0xacafb2ad, 61 },
    { 0x955fb2ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dfb2b2, 66 },
    { 0, 0 },
    { 0xfdffb2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65fb2b8, 55 },
    { 0x807fb2b9, 66 },
    { 0x604fb2ba, 55 },
    { 0, 0 },
    { 0x379fb2bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585fb2c1, 23 },
    { 0, 0 },
    { 0x5edfb2c3, 34 },
    { 0xbc4fb2c3, 23 },
    { 0xb2fb2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3fb2c9, 98 },
    { 0x7cdfb2c9, 76 },
    { 0, 0 },
    { 0xe73fb2cc, 56 },
    { 0, 0 },
    { 0x3c9fb2ce, 88 },
    { 0x270fb2ce, 99 },
    { 0xcf1fb2d0, 98 },
    { 0xc9efb2cf, 100 },
    { 0x10efb2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98afb2d6, 61 },
    { 0, 0 },
    { 0xac7fb2d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0fb2dd, 84 },
    { 0x87fb2de, 61 },
    { 0xb44fb2df, 55 },
    { 0x61ffb2de, 49 },
    { 0xe28fb2e1, 91 },
    { 0xa7dfb2e0, 76 },
    { 0x7effb2de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedafb2e6, 93 },
    { 0x881fb2e6, 66 },
    { 0, 0 },
    { 0xf46fb2e9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ffb2ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x116fb2f0, 90 },
    { 0xabafb2f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dfb2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fb2fe, 93 },
    { 0xfc7fb2ff, 90 },
    { 0x6c1fb2ff, 95 },
    { 0x73ffb300, 90 },
    { 0x43cfb301, 33 },
    { 0x84fb303, 95 },
    { 0, 0 },
    { 0x62cfb305, 23 },
    { 0xebffb305, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fb30a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53fb30d, 90 },
    { 0x29dfb30e, 76 },
    { 0, 0 },
    { 0x81fb310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80fb315, 93 },
    { 0, 0 },
    { 0x1befb317, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7efb31a, 33 },
    { 0, 0 },
    { 0x142fb31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcfb322, 61 },
    { 0x4e2fb323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcfb326, 23 },
    { 0x1e5fb326, 33 },
    { 0x6d9fb328, 68 },
    { 0x368fb329, 61 },
    { 0xc29fb32a, 90 },
    { 0x5dafb32b, 61 },
    { 0x33ffb327, 61 },
    { 0x673fb326, 43 },
    { 0xe1ffb32e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abfb332, 55 },
    { 0, 0 },
    { 0xa52fb334, 99 },
    { 0xcc2fb334, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fb340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfb343, 61 },
    { 0, 0 },
    { 0x972fb345, 52 },
    { 0, 0 },
    { 0x54afb347, 23 },
    { 0, 0 },
    { 0x4ecfb349, 23 },
    { 0x25bfb34a, 91 },
    { 0x8cffb34b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4fb34e, 23 },
    { 0xd69fb34e, 55 },
    { 0xdf3fb34e, 76 },
    { 0x914fb351, 90 },
    { 0, 0 },
    { 0x769fb353, 61 },
    { 0, 0 },
    { 0xae3fb355, 80 },
    { 0, 0 },
    { 0xf15fb357, 34 },
    { 0xd05fb358, 55 },
    { 0x451fb358, 33 },
    { 0x48cfb35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0fb362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fb365, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8fb369, 23 },
    { 0, 0 },
    { 0xfdfb36b, 2 },
    { 0x342fb36b, 90 },
    { 0x2e5fb36d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ffb371, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cfb375, 76 },
    { 0x986fb375, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692fb37c, 90 },
    { 0, 0 },
    { 0xa7efb37e, 98 },
    { 0, 0 },
    { 0xadafb380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x182fb383, 33 },
    { 0xc97fb384, 34 },
    { 0xa1bfb383, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46efb389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fb392, 33 },
    { 0, 0 },
    { 0xff4fb394, 34 },
    { 0xfc0fb395, 33 },
    { 0x78dfb395, 34 },
    { 0x3afb394, 46 },
    { 0xda6fb397, 98 },
    { 0, 0 },
    { 0x9fcfb39a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafb3a9, 61 },
    { 0, 0 },
    { 0xd91fb3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02fb3ae, 85 },
    { 0, 0 },
    { 0xe2dfb3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366fb3b5, 61 },
    { 0x376fb3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1fb3ba, 61 },
    { 0x994fb3ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eafb3bf, 61 },
    { 0, 0 },
    { 0xc10fb3c1, 61 },
    { 0, 0 },
    { 0xc36fb3c3, 90 },
    { 0, 0 },
    { 0x472fb3c5, 23 },
    { 0, 0 },
    { 0x27cfb3c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9fb3ca, 23 },
    { 0xd00fb3ca, 33 },
    { 0xb5dfb3cc, 17 },
    { 0x156fb3cc, 23 },
    { 0, 0 },
    { 0xbc2fb3cf, 76 },
    { 0x83afb3d0, 33 },
    { 0, 0 },
    { 0x1b7fb3d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83fb3d9, 98 },
    { 0, 0 },
    { 0x8d1fb3db, 46 },
    { 0, 0 },
    { 0x885fb3dd, 98 },
    { 0, 0 },
    { 0x4a6fb3df, 81 },
    { 0x3bafb3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952fb3e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0fb3e7, 68 },
    { 0, 0 },
    { 0x91efb3e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x922fb3ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148fb3f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffb3f7, 98 },
    { 0xa91fb3f8, 55 },
    { 0x2d4fb3f8, 98 },
    { 0, 0 },
    { 0x9e6fb3fb, 93 },
    { 0x74cfb3fc, 90 },
    { 0xe09fb3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbfb400, 77 },
    { 0xdeafb401, 61 },
    { 0xe5bfb402, 55 },
    { 0, 0 },
    { 0xee2fb404, 87 },
    { 0xd89fb404, 98 },
    { 0x45bfb406, 90 },
    { 0x159fb407, 90 },
    { 0x3a2fb406, 23 },
    { 0x8e8fb408, 75 },
    { 0x85afb409, 95 },
    { 0x157fb40b, 95 },
    { 0x93cfb40b, 35 },
    { 0x158fb40b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcfb410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffb414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0fb41a, 87 },
    { 0x30dfb41b, 98 },
    { 0, 0 },
    { 0x2ccfb41d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bffb420, 23 },
    { 0x1abfb421, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaafb424, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ffb428, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1fb42c, 66 },
    { 0xf2efb42d, 77 },
    { 0xefffb42e, 84 },
    { 0xe76fb42d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb434, 88 },
    { 0x6edfb434, 33 },
    { 0x888fb434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ffb43f, 90 },
    { 0xd8ffb43f, 93 },
    { 0x638fb441, 58 },
    { 0, 0 },
    { 0x895fb443, 34 },
    { 0x80fb444, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfb448, 23 },
    { 0xd93fb449, 90 },
    { 0x85bfb449, 55 },
    { 0x35fb449, 83 },
    { 0x3c3fb44a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92efb44f, 91 },
    { 0x675fb44f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dfb455, 77 },
    { 0x9e1fb455, 77 },
    { 0x196fb455, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774fb45c, 88 },
    { 0, 0 },
    { 0xa4bfb45e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fb462, 23 },
    { 0, 0 },
    { 0xd83fb464, 34 },
    { 0, 0 },
    { 0xc9cfb466, 98 },
    { 0x537fb466, 91 },
    { 0x238fb468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77afb474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9fb478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x753fb47b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb47f, 33 },
    { 0xe88fb480, 98 },
    { 0xafb481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fb486, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dfb489, 53 },
    { 0, 0 },
    { 0x749fb48b, 93 },
    { 0x541fb48c, 33 },
    { 0x312fb48d, 33 },
    { 0xbfdfb48d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2fb495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4fb49a, 48 },
    { 0x6a1fb49b, 95 },
    { 0xf0dfb49c, 90 },
    { 0xaefb49c, 61 },
    { 0, 0 },
    { 0x917fb49f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86fb4a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeefb4aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fb4ad, 46 },
    { 0, 0 },
    { 0x63fb4af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aefb4b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29afb4bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51efb4c1, 98 },
    { 0, 0 },
    { 0xfe5fb4c3, 95 },
    { 0, 0 },
    { 0x53afb4c5, 46 },
    { 0xd69fb4c6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bfb4ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8fb4ce, 95 },
    { 0x56bfb4cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79fb4db, 23 },
    { 0, 0 },
    { 0x5f5fb4dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4fb4e2, 85 },
    { 0x8ecfb4e2, 2 },
    { 0, 0 },
    { 0xc49fb4e5, 61 },
    { 0xdc0fb4e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198fb4ec, 77 },
    { 0, 0 },
    { 0x2a2fb4ee, 78 },
    { 0x206fb4ee, 98 },
    { 0, 0 },
    { 0x4f5fb4f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672fb4fa, 55 },
    { 0xa4efb4fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dfb501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb50b, 90 },
    { 0, 0 },
    { 0x195fb50d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bffb510, 55 },
    { 0x3bffb511, 91 },
    { 0xfc2fb510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222fb516, 95 },
    { 0, 0 },
    { 0xb23fb518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158fb51d, 84 },
    { 0xbc4fb51e, 99 },
    { 0xa1afb51f, 95 },
    { 0x8a5fb520, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8fb525, 46 },
    { 0x81ffb526, 70 },
    { 0xbc2fb527, 84 },
    { 0, 0 },
    { 0x4f9fb529, 90 },
    { 0x382fb52a, 90 },
    { 0x7c7fb52a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cfb530, 33 },
    { 0xbb7fb531, 91 },
    { 0x25afb531, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7fb535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1fb53c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9fb540, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x82afb543, 61 },
    { 0xcefb543, 66 },
    { 0, 0 },
    { 0x91ffb546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bfb549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bfb54e, 90 },
    { 0x266fb54f, 61 },
    { 0, 0 },
    { 0x63efb551, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cfb554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5fb55b, 61 },
    { 0x2fffb55c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5fb560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebfb565, 99 },
    { 0xb34fb566, 75 },
    { 0xd50fb567, 77 },
    { 0, 0 },
    { 0xa0dfb569, 93 },
    { 0x767fb569, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fb570, 99 },
    { 0x914fb571, 61 },
    { 0, 0 },
    { 0x612fb573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x766fb576, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dfb579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29fb57e, 33 },
    { 0xdb2fb57f, 33 },
    { 0, 0 },
    { 0x4b3fb581, 87 },
    { 0x877fb582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745fb586, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbfb58a, 90 },
    { 0x664fb58b, 98 },
    { 0xcd0fb58c, 61 },
    { 0xfd3fb58d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaffb595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fb599, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754fb5a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2fb5a4, 98 },
    { 0x3efb5a5, 99 },
    { 0, 0 },
    { 0xe30fb5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300fb5ac, 77 },
    { 0, 0 },
    { 0x51afb5ae, 90 },
    { 0x7a8fb5ae, 23 },
    { 0xd9bfb5ae, 90 },
    { 0x75ffb5ae, 61 },
    { 0x970fb5b2, 90 },
    { 0, 0 },
    { 0x4bdfb5b4, 90 },
    { 0x3c1fb5b5, 33 },
    { 0xc32fb5b6, 98 },
    { 0x55bfb5b7, 93 },
    { 0xff8fb5b8, 65 },
    { 0x966fb5b8, 17 },
    { 0x9cefb5ba, 23 },
    { 0x1a2fb5bb, 90 },
    { 0xe86fb5b9, 90 },
    { 0xb41fb5b8, 84 },
    { 0x32bfb5bb, 90 },
    { 0x8effb5bf, 90 },
    { 0xe9ffb5bf, 100 },
    { 0x538fb5c0, 46 },
    { 0x30fb5bf, 61 },
    { 0x25cfb5c3, 76 },
    { 0, 0 },
    { 0xc47fb5c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdfb5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cfb5d0, 66 },
    { 0xef9fb5d1, 23 },
    { 0xa03fb5d1, 84 },
    { 0x3e6fb5d0, 76 },
    { 0x28bfb5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0fb5e1, 61 },
    { 0xfb8fb5e2, 95 },
    { 0x545fb5e2, 98 },
    { 0x78efb5e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x863fb5e7, 46 },
    { 0xa3efb5e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0fb5ee, 90 },
    { 0xc8afb5ef, 77 },
    { 0x285fb5ef, 95 },
    { 0x48afb5ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2fb5f4, 98 },
    { 0x2b4fb5f5, 33 },
    { 0xf34fb5f4, 37 },
    { 0xb10fb5f7, 61 },
    { 0xffefb5f5, 55 },
    { 0, 0 },
    { 0xde5fb5fa, 43 },
    { 0xad8fb5fa, 95 },
    { 0x127fb5fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcfb600, 33 },
    { 0xb73fb601, 95 },
    { 0, 0 },
    { 0x2b4fb603, 88 },
    { 0, 0 },
    { 0x6b3fb605, 23 },
    { 0x927fb606, 46 },
    { 0xf90fb606, 17 },
    { 0xcd1fb605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88fb60d, 85 },
    { 0, 0 },
    { 0xfaafb60f, 72 },
    { 0, 0 },
    { 0x24bfb611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528fb615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59fb61a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ffb61d, 66 },
    { 0xd5efb61e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6fb622, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ffb625, 23 },
    { 0x7b9fb625, 52 },
    { 0xf7ffb627, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2fb62a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdfb62d, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fb637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784fb63a, 90 },
    { 0xe98fb63b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dfb63f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x197fb645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79fb64b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6fb64f, 98 },
    { 0x4f7fb650, 33 },
    { 0x200fb651, 67 },
    { 0, 0 },
    { 0x406fb653, 76 },
    { 0x4d3fb654, 93 },
    { 0, 0 },
    { 0xaf1fb656, 38 },
    { 0x140fb657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2fb65b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fb660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125fb666, 98 },
    { 0x5e4fb666, 90 },
    { 0xcbdfb668, 98 },
    { 0x50afb667, 97 },
    { 0x254fb66a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80fb670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2fb675, 85 },
    { 0, 0 },
    { 0xca9fb677, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9fb67d, 23 },
    { 0, 0 },
    { 0x4cbfb67f, 23 },
    { 0x4d8fb67f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7fb683, 61 },
    { 0x658fb684, 61 },
    { 0, 0 },
    { 0x1ddfb686, 90 },
    { 0, 0 },
    { 0x1e8fb688, 61 },
    { 0x3a4fb689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fb68e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x948fb691, 98 },
    { 0x731fb691, 61 },
    { 0, 0 },
    { 0xf0dfb694, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb699, 90 },
    { 0xbadfb69a, 37 },
    { 0, 0 },
    { 0x15cfb69c, 76 },
    { 0, 0 },
    { 0x2b4fb69e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbefb6a2, 61 },
    { 0x635fb6a3, 61 },
    { 0xd5ffb6a4, 36 },
    { 0xf39fb6a5, 78 },
    { 0x73dfb6a6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97fb6ab, 87 },
    { 0xb46fb6ac, 99 },
    { 0, 0 },
    { 0xf0efb6ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x751fb6b1, 95 },
    { 0x18fb6b2, 95 },
    { 0x519fb6b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bfb6bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dfb6c4, 90 },
    { 0x173fb6c4, 78 },
    { 0, 0 },
    { 0x4e3fb6c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fb6ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadfb6cf, 99 },
    { 0, 0 },
    { 0x42bfb6d1, 90 },
    { 0x69cfb6d2, 90 },
    { 0x4cdfb6d3, 33 },
    { 0x88dfb6d2, 61 },
    { 0, 0 },
    { 0x27afb6d6, 61 },
    { 0x855fb6d7, 91 },
    { 0x9ddfb6d8, 46 },
    { 0, 0 },
    { 0x519fb6da, 33 },
    { 0, 0 },
    { 0xf9dfb6dc, 90 },
    { 0x559fb6dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fb6e8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4efb6eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ffb6ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bfb6f1, 93 },
    { 0, 0 },
    { 0xf8fb6f3, 90 },
    { 0x568fb6f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fb6f7, 98 },
    { 0x5c7fb6f8, 95 },
    { 0x241fb6f8, 61 },
    { 0, 0 },
    { 0xcfb6fb, 61 },
    { 0x36afb6fc, 33 },
    { 0xe3cfb6fd, 34 },
    { 0, 0 },
    { 0xfa2fb6ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4fb702, 46 },
    { 0xe2fb703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x800fb706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ffb70e, 34 },
    { 0x394fb70f, 90 },
    { 0, 0 },
    { 0xdb3fb711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bfb719, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fafb71c, 33 },
    { 0xf2efb71d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ffb721, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x568fb724, 23 },
    { 0, 0 },
    { 0xcf4fb726, 55 },
    { 0xdb5fb726, 92 },
    { 0xfdffb726, 33 },
    { 0x2c2fb728, 87 },
    { 0x341fb727, 95 },
    { 0x123fb72a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753fb731, 93 },
    { 0x9a5fb732, 33 },
    { 0, 0 },
    { 0xb75fb734, 90 },
    { 0, 0 },
    { 0xfe3fb736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bfb739, 61 },
    { 0xefbfb73a, 98 },
    { 0x2d9fb73b, 91 },
    { 0, 0 },
    { 0x559fb73d, 33 },
    { 0, 0 },
    { 0x19bfb73f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fb74f, 92 },
    { 0xa5ffb74f, 56 },
    { 0x2fefb751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dfb757, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83fb75a, 90 },
    { 0x44fb75b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89fb762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x432fb765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb76b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54fb770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8defb773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaffb77d, 23 },
    { 0xa0fb77e, 23 },
    { 0xbaafb77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fafb782, 95 },
    { 0x8b6fb782, 90 },
    { 0xa68fb784, 23 },
    { 0x302fb784, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefb788, 43 },
    { 0, 0 },
    { 0xddffb78a, 66 },
    { 0x180fb78b, 23 },
    { 0xc17fb78a, 46 },
    { 0, 0 },
    { 0x5edfb78e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x162fb791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6fb796, 66 },
    { 0, 0 },
    { 0x8adfb798, 90 },
    { 0, 0 },
    { 0x474fb79a, 75 },
    { 0, 0 },
    { 0xc8bfb79c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0afb79f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dfb7a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629fb7a8, 34 },
    { 0, 0 },
    { 0x39cfb7aa, 33 },
    { 0x67efb7aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4fb7b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x189fb7b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77fb7b8, 33 },
    { 0x860fb7b9, 23 },
    { 0xa8cfb7b8, 75 },
    { 0xb65fb7bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fb7be, 81 },
    { 0x377fb7be, 33 },
    { 0x371fb7be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ffb7c4, 95 },
    { 0x830fb7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2efb7c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90fb7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4fb7d2, 33 },
    { 0x664fb7d3, 23 },
    { 0xc3bfb7d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12fb7d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dfb7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fb7e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbfb7e5, 23 },
    { 0x83dfb7e6, 43 },
    { 0xd3efb7e6, 85 },
    { 0x527fb7e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bfb7eb, 23 },
    { 0x507fb7eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442fb7ef, 66 },
    { 0xaa8fb7f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bfb7fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1fb800, 93 },
    { 0, 0 },
    { 0xcccfb802, 99 },
    { 0x5bffb803, 65 },
    { 0xf3bfb804, 98 },
    { 0xd7dfb805, 90 },
    { 0, 0 },
    { 0xdfdfb807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3fb80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37fb810, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fb814, 93 },
    { 0, 0 },
    { 0xff3fb816, 34 },
    { 0, 0 },
    { 0xe93fb818, 42 },
    { 0x325fb818, 99 },
    { 0xf39fb819, 76 },
    { 0x44cfb81b, 90 },
    { 0, 0 },
    { 0x56dfb81d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42fb820, 36 },
    { 0xbfbfb821, 99 },
    { 0xb4efb822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30fb82c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adfb830, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6fb836, 23 },
    { 0x333fb837, 90 },
    { 0xe11fb837, 93 },
    { 0xfe9fb839, 67 },
    { 0, 0 },
    { 0x6a3fb83b, 52 },
    { 0xc61fb83c, 95 },
    { 0xc5bfb83b, 55 },
    { 0, 0 },
    { 0xa42fb83f, 93 },
    { 0xb5ffb83f, 43 },
    { 0, 0 },
    { 0x240fb842, 95 },
    { 0x19ffb843, 46 },
    { 0x15fb843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebfb848, 90 },
    { 0x47ffb849, 33 },
    { 0xd26fb849, 61 },
    { 0x46ffb84b, 95 },
    { 0x93afb84c, 98 },
    { 0xcc7fb84d, 34 },
    { 0xf14fb84e, 90 },
    { 0x5efb84e, 78 },
    { 0xa55fb84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134fb854, 85 },
    { 0x637fb855, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbfb858, 33 },
    { 0x26efb858, 45 },
    { 0xbd0fb85a, 90 },
    { 0xaaafb85b, 23 },
    { 0x2bbfb85c, 90 },
    { 0, 0 },
    { 0x7b5fb85e, 23 },
    { 0x4d6fb85e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb864, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdffb86a, 34 },
    { 0xfb2fb86b, 88 },
    { 0xa0fb86b, 23 },
    { 0x91ffb86d, 33 },
    { 0xb88fb86a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6fb874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5fb877, 33 },
    { 0x860fb877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aefb87d, 33 },
    { 0, 0 },
    { 0x4a2fb87f, 33 },
    { 0x587fb87f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ffb888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7fb88b, 90 },
    { 0, 0 },
    { 0x4bbfb88d, 76 },
    { 0x826fb88e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65fb892, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12fb895, 90 },
    { 0x860fb895, 32 },
    { 0xb8fb897, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34fb89d, 99 },
    { 0x5b9fb89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6fb8a2, 76 },
    { 0x5c5fb8a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83fb8a6, 33 },
    { 0xc47fb8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73fb8ab, 67 },
    { 0x8a1fb8ab, 98 },
    { 0, 0 },
    { 0x393fb8ae, 98 },
    { 0xca9fb8af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cfb8b7, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0fb8bb, 98 },
    { 0, 0 },
    { 0x7e9fb8bd, 65 },
    { 0xcd4fb8be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fb8c1, 33 },
    { 0, 0 },
    { 0xd17fb8c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6fb8ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x446fb8cd, 23 },
    { 0x88afb8ce, 33 },
    { 0x1a8fb8cd, 34 },
    { 0x316fb8d0, 55 },
    { 0xd96fb8d1, 61 },
    { 0x973fb8d2, 99 },
    { 0xf62fb8d3, 61 },
    { 0, 0 },
    { 0x2d3fb8d5, 33 },
    { 0, 0 },
    { 0xb55fb8d7, 34 },
    { 0x93dfb8d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dfb8e3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fb8e7, 33 },
    { 0x640fb8e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecffb8ec, 61 },
    { 0x628fb8ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1fb8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bafb8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bfb907, 90 },
    { 0, 0 },
    { 0x59ffb909, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37afb914, 99 },
    { 0x998fb914, 95 },
    { 0xa2dfb914, 95 },
    { 0, 0 },
    { 0xf4bfb918, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8fb91b, 33 },
    { 0x262fb91c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cfb925, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15afb931, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54fb938, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x539fb93b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44fb93e, 91 },
    { 0xeddfb93f, 76 },
    { 0x5c2fb940, 90 },
    { 0xa18fb941, 61 },
    { 0x151fb93e, 23 },
    { 0, 0 },
    { 0x7defb944, 33 },
    { 0x965fb945, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543fb950, 34 },
    { 0xa11fb951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30fb954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecfb95f, 91 },
    { 0x3affb95f, 95 },
    { 0xa42fb960, 34 },
    { 0, 0 },
    { 0x24bfb963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbfb968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2fb96c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbafb970, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcfb973, 91 },
    { 0x83efb974, 95 },
    { 0xb8afb974, 76 },
    { 0, 0 },
    { 0x59afb977, 23 },
    { 0xcfffb978, 95 },
    { 0x367fb979, 100 },
    { 0x715fb97a, 87 },
    { 0xfd7fb97b, 33 },
    { 0xe23fb97b, 33 },
    { 0x815fb97d, 84 },
    { 0xe2bfb97e, 61 },
    { 0, 0 },
    { 0x446fb980, 90 },
    { 0xb15fb980, 99 },
    { 0x40fb982, 77 },
    { 0x3edfb982, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafb986, 99 },
    { 0xc1cfb987, 61 },
    { 0xd26fb988, 55 },
    { 0xd37fb988, 55 },
    { 0x593fb98a, 99 },
    { 0x78afb98b, 91 },
    { 0xdddfb98b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07fb98f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cfb993, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9afb997, 88 },
    { 0x142fb997, 33 },
    { 0xf74fb999, 66 },
    { 0xf66fb99a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x944fb99d, 76 },
    { 0x7a8fb99e, 33 },
    { 0x3e2fb99f, 90 },
    { 0xc92fb9a0, 87 },
    { 0xaa1fb9a1, 90 },
    { 0, 0 },
    { 0x9e1fb9a3, 61 },
    { 0x5d7fb9a4, 90 },
    { 0, 0 },
    { 0x905fb9a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebfb9ad, 33 },
    { 0x9f0fb9ad, 98 },
    { 0, 0 },
    { 0x5ccfb9b0, 46 },
    { 0x5aafb9b1, 85 },
    { 0x654fb9b2, 94 },
    { 0xc99fb9b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763fb9b9, 90 },
    { 0, 0 },
    { 0x6dfb9bb, 98 },
    { 0x1bfb9bc, 61 },
    { 0x3b4fb9bc, 98 },
    { 0x80afb9be, 23 },
    { 0x445fb9bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdfb9c5, 87 },
    { 0x8f8fb9c5, 23 },
    { 0, 0 },
    { 0x5f7fb9c8, 95 },
    { 0x505fb9c9, 90 },
    { 0x9befb9c9, 23 },
    { 0x648fb9cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ffb9ce, 23 },
    { 0, 0 },
    { 0x64dfb9d0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07fb9d4, 91 },
    { 0x3dafb9d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1fb9db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2fb9e0, 52 },
    { 0, 0 },
    { 0x206fb9e2, 96 },
    { 0x9b2fb9e3, 33 },
    { 0x35afb9e4, 88 },
    { 0x27bfb9e5, 40 },
    { 0, 0 },
    { 0x9f8fb9e7, 91 },
    { 0x9d3fb9e8, 52 },
    { 0x6f5fb9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccffb9ec, 85 },
    { 0, 0 },
    { 0x4adfb9ee, 61 },
    { 0xfa3fb9ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993fb9f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcfb9fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395fb9fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60fba03, 90 },
    { 0xeaafba03, 75 },
    { 0x50dfba05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8fba08, 66 },
    { 0x675fba09, 90 },
    { 0x902fba0a, 98 },
    { 0x9a6fba0a, 95 },
    { 0x82fba0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffba10, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37fba13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3fba1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198fba1e, 43 },
    { 0x340fba1f, 76 },
    { 0xff8fba20, 95 },
    { 0x215fba21, 61 },
    { 0, 0 },
    { 0xa57fba23, 48 },
    { 0x875fba24, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ffba2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abfba2e, 33 },
    { 0xe72fba2f, 33 },
    { 0x3e7fba2f, 90 },
    { 0xd13fba30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ffba36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3fba39, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7efba3d, 87 },
    { 0x3f8fba3e, 90 },
    { 0xe8bfba3f, 23 },
    { 0xd0afba3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fffba43, 33 },
    { 0, 0 },
    { 0x2b4fba45, 85 },
    { 0x2a1fba46, 80 },
    { 0x76cfba47, 95 },
    { 0x79dfba48, 52 },
    { 0xdeafba49, 34 },
    { 0, 0 },
    { 0x1d2fba4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efba50, 23 },
    { 0x44cfba51, 61 },
    { 0, 0 },
    { 0x700fba53, 90 },
    { 0x1ccfba54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dfba57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fba5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x734fba60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40fba63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25fba67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfba6b, 47 },
    { 0, 0 },
    { 0xc4dfba6d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0fba75, 43 },
    { 0xa69fba75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddfba7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadfba8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8fba90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6fba9c, 53 },
    { 0x81ffba9d, 98 },
    { 0xe89fba9d, 76 },
    { 0x3e4fba9f, 43 },
    { 0x62dfba9f, 98 },
    { 0xe2cfba9e, 95 },
    { 0x355fbaa2, 43 },
    { 0, 0 },
    { 0x3bafbaa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70fbaa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edfbaac, 84 },
    { 0xd9afbaad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75afbab0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x783fbab3, 95 },
    { 0x772fbab4, 84 },
    { 0, 0 },
    { 0xfedfbab6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578fbaba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fbabe, 98 },
    { 0x752fbabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bfbac2, 94 },
    { 0x16bfbac3, 33 },
    { 0xb6dfbac4, 23 },
    { 0, 0 },
    { 0x789fbac6, 98 },
    { 0xe5efbac7, 90 },
    { 0xfecfbac8, 85 },
    { 0xa6afbac9, 95 },
    { 0xde8fbaca, 98 },
    { 0xee5fbacb, 95 },
    { 0xc1afbac9, 23 },
    { 0x60bfbacc, 33 },
    { 0x947fbac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0fbad1, 61 },
    { 0xf71fbad1, 91 },
    { 0x79efbad3, 95 },
    { 0xc7dfbad3, 90 },
    { 0, 0 },
    { 0xb9afbad6, 98 },
    { 0x35cfbad7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efbada, 95 },
    { 0x5edfbada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeefbae1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8fbae7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06fbaec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581fbaf1, 34 },
    { 0x39afbaf2, 33 },
    { 0x9ebfbaf1, 91 },
    { 0xc07fbaf4, 43 },
    { 0xa4efbaf5, 98 },
    { 0x8b6fbaf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36fbafa, 23 },
    { 0, 0 },
    { 0x389fbafc, 90 },
    { 0xeb1fbafd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7fbb01, 33 },
    { 0xbfefbb01, 33 },
    { 0xc79fbb03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399fbb07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cfbb0a, 55 },
    { 0x59cfbb0b, 34 },
    { 0x558fbb0c, 98 },
    { 0xf36fbb0d, 66 },
    { 0x89afbb0e, 100 },
    { 0x935fbb0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2fbb14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21fbb17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfbb1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cfbb26, 84 },
    { 0x544fbb27, 33 },
    { 0x717fbb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19efbb33, 66 },
    { 0x265fbb34, 95 },
    { 0x5b7fbb35, 33 },
    { 0, 0 },
    { 0x7a4fbb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dfbb3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ffbb40, 87 },
    { 0xe16fbb41, 23 },
    { 0xb7efbb42, 66 },
    { 0xb21fbb41, 66 },
    { 0, 0 },
    { 0x769fbb45, 76 },
    { 0x2b8fbb45, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32fbb49, 55 },
    { 0x2afbb4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dffbb4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16fbb51, 84 },
    { 0x209fbb51, 29 },
    { 0xcbafbb53, 90 },
    { 0, 0 },
    { 0xd0bfbb55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fbb59, 66 },
    { 0x591fbb5a, 90 },
    { 0xd92fbb5a, 90 },
    { 0x42fbb5a, 43 },
    { 0xebafbb59, 96 },
    { 0xf1efbb5e, 90 },
    { 0x440fbb5b, 61 },
    { 0x33bfbb5f, 90 },
    { 0xfccfbb61, 23 },
    { 0, 0 },
    { 0x428fbb63, 98 },
    { 0x324fbb63, 95 },
    { 0, 0 },
    { 0xba7fbb66, 33 },
    { 0xec4fbb66, 61 },
    { 0, 0 },
    { 0x4c7fbb69, 33 },
    { 0x7f6fbb69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x750fbb6d, 33 },
    { 0xc74fbb6e, 17 },
    { 0x7fafbb6d, 3 },
    { 0xdc5fbb70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbb73, 90 },
    { 0, 0 },
    { 0x8fffbb75, 33 },
    { 0x4d1fbb76, 98 },
    { 0, 0 },
    { 0x954fbb78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0fbb7d, 33 },
    { 0x490fbb7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3fbb81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47fbb85, 61 },
    { 0, 0 },
    { 0x63afbb87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ffbb8b, 98 },
    { 0, 0 },
    { 0x85ffbb8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43fbb91, 46 },
    { 0xb09fbb92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfbb97, 51 },
    { 0xea6fbb98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368fbba3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0fbbab, 23 },
    { 0xbfbfbbab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbfbbb0, 68 },
    { 0x2e5fbbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fffbbb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afbbb7, 93 },
    { 0, 0 },
    { 0xc36fbbb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1fbbc1, 95 },
    { 0x711fbbc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efbbca, 23 },
    { 0x2bbfbbca, 93 },
    { 0x65efbbcc, 76 },
    { 0x4edfbbcc, 80 },
    { 0xacffbbca, 78 },
    { 0xee6fbbcf, 95 },
    { 0x845fbbd0, 66 },
    { 0, 0 },
    { 0xeb3fbbd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0fbbd6, 77 },
    { 0xc1cfbbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190fbbdb, 90 },
    { 0xdaffbbdb, 90 },
    { 0x9d1fbbdb, 98 },
    { 0xd89fbbdd, 85 },
    { 0, 0 },
    { 0xd50fbbe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cfbbe3, 98 },
    { 0x901fbbe4, 100 },
    { 0x2f1fbbe5, 61 },
    { 0xc8cfbbe6, 23 },
    { 0, 0 },
    { 0x2abfbbe8, 23 },
    { 0x5c8fbbe8, 61 },
    { 0xb7fbbea, 91 },
    { 0, 0 },
    { 0x708fbbec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbbf2, 76 },
    { 0, 0 },
    { 0x92fbbf4, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71fbbf9, 90 },
    { 0xf59fbbfa, 61 },
    { 0xfb7fbbfb, 33 },
    { 0x6cafbbfc, 34 },
    { 0xda2fbbfc, 55 },
    { 0x644fbbfe, 44 },
    { 0xd44fbbfe, 98 },
    { 0xf75fbc00, 23 },
    { 0x2a9fbc01, 95 },
    { 0xbfbfbc02, 61 },
    { 0, 0 },
    { 0xff2fbc04, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9fbc08, 99 },
    { 0x2c1fbc08, 90 },
    { 0x181fbc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccfbc12, 55 },
    { 0, 0 },
    { 0x5a9fbc14, 23 },
    { 0x44afbc15, 52 },
    { 0, 0 },
    { 0xc7fbc17, 55 },
    { 0, 0 },
    { 0x782fbc19, 98 },
    { 0x2d4fbc1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bfbc1f, 90 },
    { 0xca7fbc20, 90 },
    { 0x51cfbc21, 95 },
    { 0x1a4fbc22, 34 },
    { 0xec9fbc23, 98 },
    { 0x60fbc21, 55 },
    { 0x87cfbc25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfbc2d, 98 },
    { 0, 0 },
    { 0x32dfbc2f, 33 },
    { 0x7c9fbc30, 93 },
    { 0, 0 },
    { 0xbd1fbc32, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7fbc37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1fbc3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65afbc3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aefbc40, 98 },
    { 0x307fbc41, 91 },
    { 0xa80fbc41, 85 },
    { 0xd06fbc41, 55 },
    { 0x3effbc44, 43 },
    { 0x406fbc42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ffbc4c, 67 },
    { 0x4b7fbc4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87afbc53, 52 },
    { 0xfeafbc54, 43 },
    { 0x7e6fbc54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934fbc5a, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x196fbc5d, 95 },
    { 0xae8fbc5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fbc63, 84 },
    { 0, 0 },
    { 0x2a0fbc65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223fbc69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78fbc6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x498fbc70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47afbc73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42fbc79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bfbc7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90fbc81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1fbc87, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3fbc8e, 75 },
    { 0xee6fbc8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fbc93, 95 },
    { 0, 0 },
    { 0x557fbc95, 76 },
    { 0xf36fbc96, 98 },
    { 0x535fbc97, 33 },
    { 0x562fbc98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fbc9b, 95 },
    { 0xd29fbc9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfbca3, 95 },
    { 0, 0 },
    { 0xfe2fbca5, 84 },
    { 0x69cfbca6, 68 },
    { 0x25afbca6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7fbcac, 36 },
    { 0x692fbcac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fbcb3, 68 },
    { 0, 0 },
    { 0x12efbcb5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45fbcb8, 23 },
    { 0x535fbcb9, 93 },
    { 0, 0 },
    { 0x99bfbcbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1fbcbe, 95 },
    { 0xa4bfbcbf, 33 },
    { 0x9defbcbe, 67 },
    { 0x3f2fbcc1, 33 },
    { 0, 0 },
    { 0x7f4fbcc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5fbcc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5afbcca, 91 },
    { 0xac3fbccb, 76 },
    { 0, 0 },
    { 0x98efbccd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x848fbcd0, 95 },
    { 0xdd8fbcd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fbcd4, 61 },
    { 0, 0 },
    { 0x313fbcd6, 95 },
    { 0xa2dfbcd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6fbcdc, 50 },
    { 0x653fbcdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x507fbce0, 95 },
    { 0, 0 },
    { 0xaf5fbce2, 90 },
    { 0xf3cfbce2, 78 },
    { 0x4f4fbce4, 98 },
    { 0, 0 },
    { 0xc5efbce6, 34 },
    { 0xc5dfbce6, 67 },
    { 0x6bbfbce8, 99 },
    { 0xda9fbce6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eefbcf2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7afbcfa, 65 },
    { 0x1a3fbcfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874fbd08, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775fbd0c, 93 },
    { 0xb8cfbd0d, 88 },
    { 0x8fbd0e, 90 },
    { 0xcaffbd0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cfbd16, 46 },
    { 0x9a3fbd17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dfbd1b, 33 },
    { 0x797fbd1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8efbd20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfbd23, 23 },
    { 0, 0 },
    { 0x711fbd25, 100 },
    { 0xa4efbd25, 56 },
    { 0, 0 },
    { 0x1d1fbd28, 33 },
    { 0x47dfbd29, 98 },
    { 0, 0 },
    { 0x54cfbd2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffbd2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6fbd32, 45 },
    { 0xbf2fbd32, 90 },
    { 0, 0 },
    { 0x1b1fbd35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5afbd38, 98 },
    { 0x4edfbd39, 61 },
    { 0x7bffbd39, 38 },
    { 0xd53fbd39, 23 },
    { 0x246fbd3c, 95 },
    { 0x29efbd3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bafbd40, 61 },
    { 0xc03fbd40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8fbd49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85fbd4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dfbd51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8fbd54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bfbd57, 78 },
    { 0xcbafbd58, 61 },
    { 0x172fbd59, 67 },
    { 0xd03fbd5a, 68 },
    { 0x306fbd5a, 40 },
    { 0x31afbd58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfbd5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215fbd66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562fbd6a, 98 },
    { 0xdd3fbd6a, 90 },
    { 0x7fcfbd6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0fbd6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efbd73, 98 },
    { 0x8c3fbd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4fbd79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bfbd82, 76 },
    { 0, 0 },
    { 0x70cfbd84, 33 },
    { 0xac2fbd85, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45afbd8b, 76 },
    { 0x63fbd8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bffbd90, 90 },
    { 0x470fbd91, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4fbd96, 34 },
    { 0, 0 },
    { 0xdecfbd98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bfbd9b, 90 },
    { 0x25dfbd9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fbda0, 98 },
    { 0, 0 },
    { 0x8a0fbda2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cfbda5, 8 },
    { 0x711fbda6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52fbda9, 90 },
    { 0x3fdfbda9, 90 },
    { 0, 0 },
    { 0x8e1fbdac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7fbdb0, 93 },
    { 0x2b1fbdb1, 95 },
    { 0xfa3fbdb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbfbdb7, 55 },
    { 0, 0 },
    { 0x58efbdb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedfbdcb, 78 },
    { 0x703fbdcb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5fbdcf, 33 },
    { 0x2e9fbdcf, 38 },
    { 0x3e3fbdd1, 94 },
    { 0x9abfbdd1, 61 },
    { 0x890fbdcf, 46 },
    { 0x7dffbdd4, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1fbddd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1fbde1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fbdeb, 43 },
    { 0xd37fbdec, 23 },
    { 0x628fbded, 34 },
    { 0xa54fbdee, 34 },
    { 0xbcbfbded, 33 },
    { 0, 0 },
    { 0x70efbdf1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72fbdf5, 75 },
    { 0, 0 },
    { 0xcb2fbdf7, 93 },
    { 0xd3bfbdf7, 23 },
    { 0, 0 },
    { 0xecdfbdfa, 66 },
    { 0x3b2fbdfa, 99 },
    { 0xd1cfbdfc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9fbe02, 52 },
    { 0x443fbe03, 99 },
    { 0x466fbe02, 76 },
    { 0, 0 },
    { 0x4a7fbe06, 76 },
    { 0x232fbe07, 90 },
    { 0xdfcfbe08, 95 },
    { 0x64dfbe08, 95 },
    { 0, 0 },
    { 0x866fbe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43fbe10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631fbe1a, 23 },
    { 0xe09fbe1a, 23 },
    { 0x985fbe1c, 65 },
    { 0x4a3fbe1d, 90 },
    { 0xbecfbe1b, 33 },
    { 0xb3dfbe1f, 55 },
    { 0x67dfbe20, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2fbe27, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650fbe2c, 98 },
    { 0x936fbe2c, 75 },
    { 0x375fbe2e, 61 },
    { 0xb69fbe2c, 66 },
    { 0x68ffbe30, 76 },
    { 0x1f4fbe31, 61 },
    { 0x3cbfbe32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3fbe35, 61 },
    { 0, 0 },
    { 0x819fbe37, 87 },
    { 0, 0 },
    { 0x9eefbe39, 90 },
    { 0x2c7fbe39, 33 },
    { 0, 0 },
    { 0xf9bfbe3c, 76 },
    { 0x41cfbe3d, 90 },
    { 0, 0 },
    { 0xb27fbe3f, 55 },
    { 0x1b9fbe40, 53 },
    { 0x416fbe41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbfbe46, 90 },
    { 0xf50fbe47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23fbe4a, 95 },
    { 0x2adfbe4a, 55 },
    { 0x873fbe4c, 90 },
    { 0x8bcfbe4c, 76 },
    { 0xf48fbe4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568fbe56, 100 },
    { 0, 0 },
    { 0x412fbe58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969fbe5c, 33 },
    { 0, 0 },
    { 0x5d8fbe5e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989fbe63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2fbe6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9fbe77, 66 },
    { 0x3dfbe78, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6fbe7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9fbe80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffbe84, 93 },
    { 0xf48fbe85, 90 },
    { 0xc5bfbe86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1fbe89, 90 },
    { 0x7c1fbe8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fbe90, 78 },
    { 0, 0 },
    { 0xf7ffbe92, 93 },
    { 0, 0 },
    { 0xf82fbe94, 33 },
    { 0x568fbe94, 23 },
    { 0xeeefbe95, 36 },
    { 0x330fbe95, 33 },
    { 0x2c6fbe96, 76 },
    { 0x8e9fbe99, 90 },
    { 0x8dfbe9a, 46 },
    { 0xef9fbe9b, 55 },
    { 0x938fbe9c, 33 },
    { 0xfcdfbe9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64fbea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efbea4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abfbea7, 90 },
    { 0x892fbea8, 46 },
    { 0, 0 },
    { 0x57ffbeaa, 98 },
    { 0xdd8fbeab, 33 },
    { 0xd81fbeac, 88 },
    { 0x611fbead, 93 },
    { 0xa42fbead, 33 },
    { 0x220fbeaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x923fbeb2, 98 },
    { 0, 0 },
    { 0xa15fbeb4, 67 },
    { 0x2a6fbeb4, 90 },
    { 0xdbefbeb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89fbeba, 76 },
    { 0xbe7fbebb, 33 },
    { 0x8b4fbebb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5fbec4, 66 },
    { 0, 0 },
    { 0xb64fbec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efbec9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dfbed0, 80 },
    { 0x8e8fbed1, 61 },
    { 0, 0 },
    { 0xdc7fbed3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fbed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fbedd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eafbee0, 90 },
    { 0, 0 },
    { 0xd03fbee2, 99 },
    { 0x17ffbee2, 43 },
    { 0xd31fbee4, 61 },
    { 0x95efbee4, 33 },
    { 0x811fbee6, 43 },
    { 0xd52fbee5, 73 },
    { 0xf5bfbee7, 33 },
    { 0x8cdfbee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bfbeed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fbef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fbef8, 85 },
    { 0xe25fbef9, 90 },
    { 0xed7fbef8, 90 },
    { 0x9ccfbefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242fbf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x482fbf05, 61 },
    { 0x19cfbf06, 90 },
    { 0x68dfbf05, 90 },
    { 0x6e1fbf08, 98 },
    { 0x448fbf06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303fbf10, 61 },
    { 0x57dfbf11, 93 },
    { 0xb2cfbf11, 33 },
    { 0xcd8fbf11, 61 },
    { 0xa1fbf14, 61 },
    { 0xda3fbf11, 66 },
    { 0xd9ffbf11, 100 },
    { 0x2c5fbf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x491fbf1a, 66 },
    { 0x77bfbf1b, 63 },
    { 0x3b1fbf1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6afbf23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafbf28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411fbf30, 98 },
    { 0x9bdfbf30, 96 },
    { 0, 0 },
    { 0xdbefbf33, 76 },
    { 0, 0 },
    { 0x457fbf35, 90 },
    { 0xeacfbf36, 61 },
    { 0xd25fbf36, 95 },
    { 0, 0 },
    { 0xe5bfbf39, 90 },
    { 0x5f0fbf3a, 65 },
    { 0xf5fbf3b, 40 },
    { 0xac7fbf3c, 91 },
    { 0, 0 },
    { 0x1affbf3e, 94 },
    { 0x277fbf3f, 23 },
    { 0, 0 },
    { 0x647fbf41, 98 },
    { 0x432fbf42, 84 },
    { 0x314fbf43, 23 },
    { 0, 0 },
    { 0x747fbf45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebfbf49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1fbf4f, 95 },
    { 0x6c1fbf50, 91 },
    { 0x536fbf51, 78 },
    { 0x425fbf52, 61 },
    { 0, 0 },
    { 0xd7ffbf54, 36 },
    { 0x619fbf55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbf5e, 34 },
    { 0xec7fbf5f, 100 },
    { 0x8d0fbf60, 95 },
    { 0x77dfbf60, 93 },
    { 0x7fbfbf62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9fbf66, 93 },
    { 0, 0 },
    { 0x5c9fbf68, 95 },
    { 0, 0 },
    { 0x6a2fbf6a, 33 },
    { 0xa13fbf6a, 61 },
    { 0, 0 },
    { 0x3d4fbf6d, 17 },
    { 0xf81fbf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfbf71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62fbf74, 67 },
    { 0, 0 },
    { 0x99bfbf76, 77 },
    { 0x96ffbf77, 98 },
    { 0x8e5fbf77, 46 },
    { 0, 0 },
    { 0xa4efbf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fbf7d, 93 },
    { 0xd30fbf7e, 66 },
    { 0xe4dfbf7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fafbf83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9fbf8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433fbf98, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1fbf9e, 99 },
    { 0, 0 },
    { 0xb9efbfa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33fbfa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10fbfa8, 43 },
    { 0xd17fbfa9, 95 },
    { 0x77bfbfaa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fbfb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88fbfb4, 90 },
    { 0x3e4fbfb4, 99 },
    { 0xefffbfb6, 90 },
    { 0xcc5fbfb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fbfbb, 98 },
    { 0xee0fbfbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeefbfc1, 33 },
    { 0xe69fbfc2, 98 },
    { 0xd20fbfc3, 52 },
    { 0x70dfbfc1, 93 },
    { 0x693fbfc5, 81 },
    { 0x9dbfbfc6, 76 },
    { 0x720fbfc5, 90 },
    { 0x927fbfc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491fbfd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13fbfd7, 90 },
    { 0x71cfbfd8, 61 },
    { 0x116fbfd8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9fbfdc, 90 },
    { 0, 0 },
    { 0xf10fbfde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cfbfec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefbfef, 90 },
    { 0xe46fbfef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fbff8, 93 },
    { 0x719fbff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x473fbffc, 98 },
    { 0x372fbffd, 66 },
    { 0xd13fbffc, 96 },
    { 0x543fbfff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbfc002, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bfc006, 87 },
    { 0, 0 },
    { 0x240fc008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0fc00d, 78 },
    { 0xdaefc00e, 95 },
    { 0x382fc00f, 95 },
    { 0x658fc010, 76 },
    { 0, 0 },
    { 0xd98fc012, 61 },
    { 0xf2cfc012, 67 },
    { 0x11afc012, 85 },
    { 0x9e2fc015, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766fc01a, 55 },
    { 0x80dfc01b, 23 },
    { 0xa62fc01b, 33 },
    { 0xa12fc01d, 34 },
    { 0x632fc01e, 98 },
    { 0xa1efc01f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cffc022, 90 },
    { 0xed8fc023, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931fc027, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3affc02a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812fc037, 34 },
    { 0, 0 },
    { 0xddefc039, 34 },
    { 0x1e9fc03a, 33 },
    { 0x191fc03a, 23 },
    { 0xdd1fc03c, 76 },
    { 0, 0 },
    { 0x939fc03e, 46 },
    { 0x609fc03f, 85 },
    { 0, 0 },
    { 0xedcfc041, 88 },
    { 0, 0 },
    { 0xfd1fc043, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fc04a, 33 },
    { 0x861fc04b, 33 },
    { 0x21cfc04c, 43 },
    { 0xd23fc04d, 95 },
    { 0x91ffc04d, 93 },
    { 0x800fc04d, 90 },
    { 0x6bdfc050, 98 },
    { 0xe37fc04e, 90 },
    { 0xa38fc04f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bffc056, 66 },
    { 0, 0 },
    { 0x85bfc058, 90 },
    { 0, 0 },
    { 0x5defc05a, 46 },
    { 0x462fc05b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfc05e, 67 },
    { 0x2edfc05f, 33 },
    { 0x640fc060, 91 },
    { 0x2bfc061, 90 },
    { 0xa80fc060, 90 },
    { 0xd7ffc060, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7fc066, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960fc06a, 76 },
    { 0, 0 },
    { 0x3cfc06c, 87 },
    { 0, 0 },
    { 0xf4ffc06e, 90 },
    { 0x278fc06e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56fc072, 83 },
    { 0xdebfc073, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4fc07d, 34 },
    { 0xc4cfc07e, 61 },
    { 0xc6dfc07e, 83 },
    { 0, 0 },
    { 0xdf1fc081, 52 },
    { 0, 0 },
    { 0xd0dfc083, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30fc087, 33 },
    { 0, 0 },
    { 0x82bfc089, 52 },
    { 0, 0 },
    { 0x574fc08b, 55 },
    { 0, 0 },
    { 0x78cfc08d, 33 },
    { 0x6c8fc08d, 33 },
    { 0, 0 },
    { 0xc66fc090, 98 },
    { 0x783fc091, 77 },
    { 0xf64fc091, 85 },
    { 0x944fc091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9fc09a, 33 },
    { 0, 0 },
    { 0x480fc09c, 90 },
    { 0xf9cfc09c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bfc0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3effc0a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307fc0ac, 98 },
    { 0xa44fc0ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x239fc0b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5fc0b3, 95 },
    { 0x1e5fc0b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9fc0b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfc0c2, 91 },
    { 0x946fc0c2, 61 },
    { 0xd93fc0c4, 23 },
    { 0xe58fc0c5, 98 },
    { 0x377fc0c6, 46 },
    { 0x4e2fc0c3, 90 },
    { 0xb78fc0c8, 33 },
    { 0x72efc0c8, 33 },
    { 0xa89fc0c9, 69 },
    { 0x414fc0cb, 76 },
    { 0x27bfc0c5, 23 },
    { 0xddbfc0cd, 91 },
    { 0, 0 },
    { 0xd06fc0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccfc0d5, 91 },
    { 0, 0 },
    { 0xddefc0d7, 33 },
    { 0, 0 },
    { 0x3bbfc0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3fc0dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfc0e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acfc0e3, 84 },
    { 0x84dfc0e4, 61 },
    { 0x4adfc0e4, 23 },
    { 0x85dfc0e6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27fc0e9, 34 },
    { 0xe9dfc0ea, 33 },
    { 0xbd8fc0ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abfc0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cfc0f1, 61 },
    { 0x999fc0f2, 55 },
    { 0xc00fc0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x951fc0f6, 77 },
    { 0x4fafc0f7, 23 },
    { 0xe6afc0f8, 23 },
    { 0, 0 },
    { 0xba5fc0fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efc0fd, 66 },
    { 0x86fc0fe, 61 },
    { 0x435fc0ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0fc10c, 90 },
    { 0x56dfc10d, 100 },
    { 0, 0 },
    { 0x687fc10f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efc116, 34 },
    { 0xe14fc117, 33 },
    { 0, 0 },
    { 0xa15fc119, 90 },
    { 0x9fdfc119, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfc11f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfc125, 46 },
    { 0x489fc125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436fc12a, 33 },
    { 0, 0 },
    { 0x53dfc12c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aafc131, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6efc134, 61 },
    { 0xfd7fc135, 55 },
    { 0xcabfc135, 52 },
    { 0x6ffc135, 33 },
    { 0xc32fc137, 23 },
    { 0x59dfc139, 67 },
    { 0x214fc139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x256fc13d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919fc143, 90 },
    { 0x5ddfc143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19fc149, 76 },
    { 0xf57fc14a, 91 },
    { 0x3f7fc149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1fc14f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2fc156, 33 },
    { 0x254fc157, 66 },
    { 0x797fc158, 33 },
    { 0x339fc159, 23 },
    { 0x4f0fc157, 61 },
    { 0xad6fc15b, 33 },
    { 0x747fc15c, 55 },
    { 0xc58fc15d, 87 },
    { 0xe90fc15d, 53 },
    { 0x2acfc15c, 23 },
    { 0x323fc160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc173, 61 },
    { 0, 0 },
    { 0x6aefc175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97fc179, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37fc17f, 90 },
    { 0x9c9fc180, 95 },
    { 0xb95fc181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60fc185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dfc189, 90 },
    { 0xbc0fc18a, 78 },
    { 0x8c0fc18b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3fc18e, 95 },
    { 0xaebfc18f, 90 },
    { 0xc59fc190, 65 },
    { 0xfb7fc18e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9afc196, 100 },
    { 0x3dbfc196, 95 },
    { 0x77dfc198, 61 },
    { 0, 0 },
    { 0xd73fc19a, 95 },
    { 0x235fc19b, 23 },
    { 0xe1cfc19b, 76 },
    { 0xf99fc19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafc1aa, 90 },
    { 0x664fc1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4fc1ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fc1b1, 62 },
    { 0x5cfc1b2, 90 },
    { 0xdbbfc1b3, 76 },
    { 0x3ecfc1b1, 98 },
    { 0x156fc1b1, 61 },
    { 0, 0 },
    { 0x93ffc1b7, 33 },
    { 0xb1cfc1b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544fc1bc, 95 },
    { 0x100fc1bd, 65 },
    { 0x6b7fc1bd, 61 },
    { 0, 0 },
    { 0x3f5fc1c0, 76 },
    { 0xd87fc1c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686fc1c9, 61 },
    { 0xd49fc1ca, 30 },
    { 0, 0 },
    { 0xc7afc1cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2fc1d6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcfc1d9, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6fc1dd, 23 },
    { 0x589fc1de, 61 },
    { 0x99dfc1de, 95 },
    { 0x540fc1df, 33 },
    { 0, 0 },
    { 0xbf3fc1e2, 43 },
    { 0x5a7fc1e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867fc1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafafc1f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fc1f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x182fc1fc, 33 },
    { 0x929fc1fd, 34 },
    { 0x76cfc1fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89efc203, 43 },
    { 0, 0 },
    { 0x4c8fc205, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fc208, 76 },
    { 0xa13fc209, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fc20c, 90 },
    { 0, 0 },
    { 0xc52fc20e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0efc214, 61 },
    { 0xfe1fc215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45fc219, 46 },
    { 0x319fc21a, 90 },
    { 0xc55fc21b, 55 },
    { 0x70efc21b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19fc222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32fc226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48fc229, 33 },
    { 0xaecfc22a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57efc22d, 90 },
    { 0x687fc22e, 98 },
    { 0xa86fc22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fc234, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4afc23b, 75 },
    { 0, 0 },
    { 0x5b8fc23d, 27 },
    { 0x2befc23e, 33 },
    { 0xec4fc23d, 90 },
    { 0xa60fc23f, 90 },
    { 0xe77fc241, 23 },
    { 0xfcefc242, 98 },
    { 0x95cfc243, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848fc249, 63 },
    { 0, 0 },
    { 0xd0efc24b, 46 },
    { 0x654fc24c, 45 },
    { 0xf6cfc24d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0afc257, 67 },
    { 0, 0 },
    { 0x83ffc259, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfc25c, 85 },
    { 0x550fc25d, 90 },
    { 0, 0 },
    { 0xf05fc25f, 85 },
    { 0xc33fc25f, 61 },
    { 0xd8efc25f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54fc265, 76 },
    { 0xf9afc266, 61 },
    { 0x51cfc267, 33 },
    { 0x15dfc267, 17 },
    { 0x5ffc269, 66 },
    { 0x878fc269, 95 },
    { 0x58efc26b, 76 },
    { 0x7a5fc26c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757fc27b, 88 },
    { 0x135fc27c, 23 },
    { 0x658fc27d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecfc283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ffc286, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fc28c, 61 },
    { 0x1eefc28c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bfc291, 77 },
    { 0, 0 },
    { 0x6d7fc293, 23 },
    { 0, 0 },
    { 0x3bfc295, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653fc299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81fc2a5, 95 },
    { 0x6affc2a6, 61 },
    { 0xa4ffc2a6, 76 },
    { 0x152fc2a8, 48 },
    { 0x5b2fc2a8, 95 },
    { 0xbe0fc2aa, 95 },
    { 0xbd8fc2aa, 61 },
    { 0x55dfc2aa, 33 },
    { 0xdafc2ac, 76 },
    { 0, 0 },
    { 0x9aefc2af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fc2b2, 61 },
    { 0, 0 },
    { 0xe3ffc2b4, 90 },
    { 0, 0 },
    { 0x4fafc2b6, 76 },
    { 0x552fc2b6, 90 },
    { 0x471fc2b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5fc2be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x968fc2c1, 33 },
    { 0xf4afc2c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40fc2cc, 90 },
    { 0x57cfc2cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffc2d2, 95 },
    { 0xffefc2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fc2d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7fc2de, 90 },
    { 0x932fc2df, 67 },
    { 0xc98fc2df, 40 },
    { 0x632fc2e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396fc2e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x623fc2e9, 98 },
    { 0xa51fc2ea, 90 },
    { 0x94ffc2e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dfc2ef, 95 },
    { 0xefffc2f0, 100 },
    { 0xe27fc2f0, 76 },
    { 0, 0 },
    { 0xc62fc2f3, 40 },
    { 0, 0 },
    { 0x5b6fc2f5, 33 },
    { 0, 0 },
    { 0xb96fc2f7, 33 },
    { 0xff2fc2f7, 95 },
    { 0x254fc2f8, 90 },
    { 0x1aefc2fa, 90 },
    { 0, 0 },
    { 0xb37fc2fc, 61 },
    { 0, 0 },
    { 0xf3cfc2fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27afc313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfc318, 98 },
    { 0xb7bfc318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dfc323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7fc326, 23 },
    { 0, 0 },
    { 0x6cafc328, 98 },
    { 0, 0 },
    { 0xd6cfc32a, 90 },
    { 0, 0 },
    { 0xd4dfc32c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cfc32f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5fc33d, 68 },
    { 0x92fc33e, 33 },
    { 0xce6fc33e, 90 },
    { 0x32dfc340, 23 },
    { 0x2dbfc340, 33 },
    { 0, 0 },
    { 0xc6efc343, 79 },
    { 0x75fc344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9fc34c, 87 },
    { 0, 0 },
    { 0x5c1fc34e, 99 },
    { 0x33bfc34f, 98 },
    { 0, 0 },
    { 0xfe2fc351, 33 },
    { 0, 0 },
    { 0xc70fc353, 95 },
    { 0x536fc353, 23 },
    { 0xc45fc355, 95 },
    { 0x7b4fc354, 33 },
    { 0, 0 },
    { 0xe8cfc358, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddfc366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44afc36d, 46 },
    { 0x82afc36e, 61 },
    { 0xf02fc36f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4befc373, 91 },
    { 0x80dfc374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9fc37a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2fc37e, 67 },
    { 0x2efc37f, 96 },
    { 0, 0 },
    { 0x163fc381, 23 },
    { 0, 0 },
    { 0x963fc383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dfc386, 65 },
    { 0x6e7fc387, 90 },
    { 0x1cdfc386, 95 },
    { 0x6b6fc389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x452fc38c, 66 },
    { 0, 0 },
    { 0xcd9fc38e, 87 },
    { 0, 0 },
    { 0x697fc390, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54fc396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27fc39a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238fc39e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedffc3a3, 98 },
    { 0x3ecfc3a4, 78 },
    { 0, 0 },
    { 0x19fc3a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77fc3ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230fc3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937fc3b6, 52 },
    { 0x1a0fc3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ffc3bb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbfc3be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x323fc3c1, 95 },
    { 0xf4ffc3c2, 98 },
    { 0x1c8fc3c1, 99 },
    { 0x99cfc3c4, 61 },
    { 0, 0 },
    { 0x3cfc3c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0fc3c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2fc3cf, 33 },
    { 0, 0 },
    { 0xf0efc3d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8fc3d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70fc3da, 66 },
    { 0x455fc3db, 77 },
    { 0xfc2fc3dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aafc3e3, 95 },
    { 0, 0 },
    { 0xcebfc3e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fc3ea, 23 },
    { 0xd75fc3eb, 90 },
    { 0x3afc3eb, 93 },
    { 0xc69fc3ed, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dffc3f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafc3f6, 33 },
    { 0, 0 },
    { 0x99bfc3f8, 34 },
    { 0x4a2fc3f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294fc3fd, 90 },
    { 0, 0 },
    { 0xacafc3ff, 78 },
    { 0xc5cfc3ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685fc406, 98 },
    { 0, 0 },
    { 0x6f4fc408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fc413, 61 },
    { 0xc7efc414, 93 },
    { 0, 0 },
    { 0xc19fc416, 90 },
    { 0, 0 },
    { 0xca5fc418, 23 },
    { 0x8c2fc419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bfc41c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaefc424, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fc42d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04fc435, 23 },
    { 0xdd8fc436, 33 },
    { 0x7b6fc436, 95 },
    { 0xff4fc437, 61 },
    { 0x334fc439, 44 },
    { 0x82fc439, 61 },
    { 0xa81fc43b, 33 },
    { 0, 0 },
    { 0x6abfc43d, 90 },
    { 0x47bfc43d, 66 },
    { 0x1bafc43f, 33 },
    { 0x463fc43d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51afc44b, 52 },
    { 0xcd6fc44c, 98 },
    { 0, 0 },
    { 0x237fc44e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fc451, 43 },
    { 0, 0 },
    { 0x281fc453, 33 },
    { 0, 0 },
    { 0x389fc455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fc45d, 55 },
    { 0xec2fc45e, 33 },
    { 0, 0 },
    { 0xe27fc460, 55 },
    { 0xe3fc461, 91 },
    { 0x5f4fc462, 93 },
    { 0x81afc463, 64 },
    { 0x5a3fc461, 61 },
    { 0x33ffc462, 90 },
    { 0x994fc466, 93 },
    { 0x22cfc466, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x955fc46a, 33 },
    { 0, 0 },
    { 0xdd6fc46c, 61 },
    { 0, 0 },
    { 0x172fc46e, 98 },
    { 0x660fc46e, 84 },
    { 0xf8efc46f, 33 },
    { 0, 0 },
    { 0x13efc472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fc475, 95 },
    { 0x4e7fc475, 95 },
    { 0, 0 },
    { 0x5c4fc478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817fc484, 95 },
    { 0x580fc485, 43 },
    { 0, 0 },
    { 0x1c5fc487, 33 },
    { 0xefefc487, 98 },
    { 0x297fc487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06fc48e, 33 },
    { 0, 0 },
    { 0xb68fc490, 90 },
    { 0xd9afc491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fc495, 93 },
    { 0, 0 },
    { 0x236fc497, 90 },
    { 0x849fc497, 99 },
    { 0xca4fc499, 83 },
    { 0x9f1fc499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0fc49e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ffc4a2, 90 },
    { 0, 0 },
    { 0xeb7fc4a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105fc4b0, 98 },
    { 0x81fc4b1, 55 },
    { 0x29bfc4b2, 66 },
    { 0x711fc4b1, 23 },
    { 0, 0 },
    { 0x99efc4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fefc4b8, 33 },
    { 0xc6efc4b8, 91 },
    { 0x9fcfc4ba, 33 },
    { 0x51dfc4bb, 61 },
    { 0x59ffc4bb, 95 },
    { 0x8ffc4bd, 55 },
    { 0x549fc4be, 61 },
    { 0x479fc4bf, 95 },
    { 0x9fafc4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80fc4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcfc4c8, 36 },
    { 0x6c3fc4c8, 95 },
    { 0xf84fc4ca, 43 },
    { 0x1abfc4ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74fc4cf, 93 },
    { 0xaedfc4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x160fc4d3, 60 },
    { 0, 0 },
    { 0xcfcfc4d5, 33 },
    { 0, 0 },
    { 0x5e7fc4d7, 98 },
    { 0xd09fc4d7, 90 },
    { 0x169fc4d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401fc4de, 52 },
    { 0x2eefc4df, 33 },
    { 0xd7afc4df, 81 },
    { 0x1ffc4e1, 76 },
    { 0xe80fc4e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0fc4e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdfc4ea, 98 },
    { 0, 0 },
    { 0x8cefc4ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3fc4f0, 33 },
    { 0xc76fc4f0, 34 },
    { 0xca6fc4f0, 98 },
    { 0xf95fc4f3, 81 },
    { 0, 0 },
    { 0x7c4fc4f5, 87 },
    { 0x7f2fc4f5, 33 },
    { 0x813fc4f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfc4fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69fc4ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2fc502, 23 },
    { 0x11ffc503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268fc509, 65 },
    { 0xee1fc50a, 90 },
    { 0xa88fc50b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61afc519, 55 },
    { 0x1b3fc51a, 85 },
    { 0, 0 },
    { 0x7b1fc51c, 76 },
    { 0x874fc51d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cfc522, 91 },
    { 0, 0 },
    { 0xab8fc524, 84 },
    { 0, 0 },
    { 0x917fc526, 99 },
    { 0, 0 },
    { 0x34ffc528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc52b, 61 },
    { 0x406fc52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x998fc52f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cfc539, 98 },
    { 0x80efc53a, 43 },
    { 0x1e5fc53a, 76 },
    { 0xf2bfc53c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fc540, 95 },
    { 0xd39fc540, 95 },
    { 0x47ffc542, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2fc545, 23 },
    { 0, 0 },
    { 0x5e4fc547, 100 },
    { 0x764fc547, 33 },
    { 0xb39fc547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6fc54f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dffc553, 61 },
    { 0, 0 },
    { 0x187fc555, 76 },
    { 0x295fc556, 46 },
    { 0x7e0fc557, 23 },
    { 0xecfc557, 23 },
    { 0, 0 },
    { 0xc57fc55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fc562, 61 },
    { 0x52ffc562, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddffc569, 52 },
    { 0x528fc569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ffc56d, 23 },
    { 0xa3ffc56d, 90 },
    { 0x6a3fc56d, 52 },
    { 0, 0 },
    { 0x684fc571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807fc577, 95 },
    { 0xcc9fc578, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5fc57b, 61 },
    { 0xd28fc57b, 90 },
    { 0xbe6fc57c, 76 },
    { 0, 0 },
    { 0x1d4fc57f, 98 },
    { 0x228fc57f, 61 },
    { 0, 0 },
    { 0x8e9fc582, 81 },
    { 0, 0 },
    { 0x1bafc584, 59 },
    { 0xdbefc584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fc588, 23 },
    { 0x235fc589, 43 },
    { 0xf2fc58a, 46 },
    { 0xb6dfc58a, 61 },
    { 0xe54fc588, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7fc591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x369fc594, 61 },
    { 0x7adfc595, 65 },
    { 0x9f0fc595, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cfc599, 77 },
    { 0x6a5fc59a, 90 },
    { 0x5ecfc59b, 61 },
    { 0xd83fc59b, 98 },
    { 0x8b8fc59d, 99 },
    { 0xb84fc599, 76 },
    { 0x47bfc59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fc5a4, 81 },
    { 0, 0 },
    { 0x8e6fc5a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35afc5aa, 91 },
    { 0x7cefc5aa, 76 },
    { 0xa16fc5ac, 77 },
    { 0x31dfc5ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1fc5b1, 34 },
    { 0xbb6fc5b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbefc5b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0afc5b8, 61 },
    { 0x7b5fc5b9, 77 },
    { 0xdc8fc5ba, 61 },
    { 0x72bfc5bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dfc5c1, 60 },
    { 0, 0 },
    { 0x8a0fc5c3, 61 },
    { 0, 0 },
    { 0x128fc5c5, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5fc5cb, 95 },
    { 0xc42fc5cb, 23 },
    { 0x157fc5cd, 98 },
    { 0x7c7fc5cd, 98 },
    { 0xaa8fc5cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4fc5d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fc5da, 90 },
    { 0xb89fc5db, 100 },
    { 0, 0 },
    { 0x4c7fc5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4fc5e0, 84 },
    { 0, 0 },
    { 0xf13fc5e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaefc5e8, 90 },
    { 0x42fc5e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4fc5ec, 33 },
    { 0x75ffc5ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7fc5f2, 88 },
    { 0xf9efc5f3, 61 },
    { 0x73afc5f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04fc5fa, 75 },
    { 0x8d5fc5fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96fc600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fc603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773fc607, 93 },
    { 0, 0 },
    { 0x3b0fc609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fc60c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dfc60f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9fc614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x907fc617, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914fc61c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fc623, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54fc627, 46 },
    { 0x376fc628, 33 },
    { 0x4ccfc629, 43 },
    { 0x4acfc62a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bfc62d, 61 },
    { 0x5c1fc62e, 23 },
    { 0xe15fc62f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x160fc632, 90 },
    { 0x5b2fc632, 46 },
    { 0x85ffc634, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc63b, 23 },
    { 0x16afc63c, 52 },
    { 0xf69fc63c, 34 },
    { 0xdd8fc63c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41fc642, 68 },
    { 0x5d6fc643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc649, 23 },
    { 0xa8bfc649, 90 },
    { 0xb93fc64b, 90 },
    { 0, 0 },
    { 0xc74fc64d, 55 },
    { 0x85ffc64e, 98 },
    { 0x777fc64f, 55 },
    { 0xc79fc650, 95 },
    { 0, 0 },
    { 0xbfbfc652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62fc661, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fc666, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590fc66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18fc670, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bffc677, 61 },
    { 0xdcbfc678, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22fc67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116fc681, 84 },
    { 0x129fc681, 53 },
    { 0xb0bfc683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x262fc686, 77 },
    { 0xd87fc687, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bfc68b, 66 },
    { 0, 0 },
    { 0xe88fc68d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bfc690, 61 },
    { 0x8e3fc691, 33 },
    { 0, 0 },
    { 0x7fc693, 94 },
    { 0xda7fc694, 33 },
    { 0x7c9fc695, 34 },
    { 0, 0 },
    { 0xa74fc697, 52 },
    { 0, 0 },
    { 0x323fc699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fc69c, 33 },
    { 0xc5bfc69d, 90 },
    { 0xf44fc69e, 61 },
    { 0, 0 },
    { 0xc40fc6a0, 53 },
    { 0x7fdfc6a0, 98 },
    { 0x2cffc6a0, 95 },
    { 0xa61fc6a3, 43 },
    { 0x189fc6a2, 90 },
    { 0x9b1fc6a5, 61 },
    { 0xf5efc6a6, 95 },
    { 0x614fc6a5, 98 },
    { 0xcdffc6a7, 61 },
    { 0xa47fc6a5, 33 },
    { 0x9d0fc6a6, 93 },
    { 0x991fc6ab, 46 },
    { 0x61ffc6ac, 23 },
    { 0, 0 },
    { 0x8c8fc6ae, 83 },
    { 0xba4fc6af, 95 },
    { 0x15cfc6ae, 66 },
    { 0, 0 },
    { 0xd71fc6b2, 85 },
    { 0x893fc6b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913fc6ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361fc6bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ffc6c2, 46 },
    { 0, 0 },
    { 0x35cfc6c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374fc6d1, 78 },
    { 0, 0 },
    { 0x124fc6d3, 65 },
    { 0xccfc6d4, 75 },
    { 0, 0 },
    { 0xcf9fc6d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cffc6da, 95 },
    { 0, 0 },
    { 0x314fc6dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecfc6e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x195fc6e7, 90 },
    { 0x6e7fc6e7, 61 },
    { 0xe3bfc6e9, 76 },
    { 0x99bfc6ea, 76 },
    { 0xf43fc6e8, 93 },
    { 0xc46fc6ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9fc6f2, 76 },
    { 0xe16fc6f2, 96 },
    { 0, 0 },
    { 0x96dfc6f5, 95 },
    { 0x123fc6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3fc6fd, 5 },
    { 0x839fc6fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x834fc701, 76 },
    { 0x160fc702, 90 },
    { 0xcd3fc701, 76 },
    { 0, 0 },
    { 0x41cfc705, 99 },
    { 0xb79fc705, 76 },
    { 0, 0 },
    { 0x46bfc708, 90 },
    { 0x6fdfc709, 75 },
    { 0, 0 },
    { 0xa3cfc70b, 33 },
    { 0xc05fc70c, 76 },
    { 0xcdbfc70d, 33 },
    { 0x436fc70b, 76 },
    { 0xcbafc70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90fc712, 78 },
    { 0, 0 },
    { 0x4a1fc714, 98 },
    { 0x975fc714, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1fc71d, 90 },
    { 0, 0 },
    { 0xcb4fc71f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306fc723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc726, 43 },
    { 0xf18fc727, 98 },
    { 0x476fc726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1fc72b, 61 },
    { 0, 0 },
    { 0xca4fc72d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fc733, 61 },
    { 0, 0 },
    { 0x96ffc735, 95 },
    { 0, 0 },
    { 0x403fc737, 43 },
    { 0x4d6fc738, 90 },
    { 0x571fc737, 95 },
    { 0x291fc73a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2fc73d, 76 },
    { 0x68fc73e, 98 },
    { 0, 0 },
    { 0xfaffc740, 69 },
    { 0xe87fc741, 23 },
    { 0xe6fc741, 33 },
    { 0xdb4fc743, 66 },
    { 0x825fc740, 61 },
    { 0, 0 },
    { 0x40bfc746, 88 },
    { 0x194fc746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227fc74b, 61 },
    { 0x9cdfc74b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fc752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdfc759, 83 },
    { 0, 0 },
    { 0xb87fc75b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ffc766, 66 },
    { 0x877fc767, 61 },
    { 0xbddfc767, 17 },
    { 0x328fc767, 33 },
    { 0xafffc767, 55 },
    { 0xdbdfc76b, 77 },
    { 0x97bfc76c, 61 },
    { 0x871fc76d, 90 },
    { 0xaf2fc768, 61 },
    { 0x5f4fc76f, 65 },
    { 0x1dffc76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccfc774, 33 },
    { 0x8c0fc775, 78 },
    { 0, 0 },
    { 0x497fc777, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fc77b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4fc77e, 33 },
    { 0, 0 },
    { 0x45afc780, 33 },
    { 0xf6cfc781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fc787, 76 },
    { 0x91ffc787, 33 },
    { 0, 0 },
    { 0x332fc78a, 33 },
    { 0, 0 },
    { 0xe89fc78c, 90 },
    { 0xfc8fc78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66afc795, 90 },
    { 0xae0fc796, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57fc79e, 61 },
    { 0x6efc79e, 93 },
    { 0xd42fc7a0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bfc7a4, 61 },
    { 0xe26fc7a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dfc7ac, 95 },
    { 0, 0 },
    { 0x8dffc7ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffc7b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5fc7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304fc7bd, 65 },
    { 0x5b9fc7be, 69 },
    { 0xab8fc7bf, 98 },
    { 0x71efc7c0, 90 },
    { 0x51dfc7c1, 90 },
    { 0xb16fc7c2, 61 },
    { 0xe18fc7c3, 39 },
    { 0xad1fc7c3, 84 },
    { 0x745fc7c3, 40 },
    { 0x314fc7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0fc7c9, 90 },
    { 0, 0 },
    { 0x779fc7cb, 43 },
    { 0xb4cfc7cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61fc7cf, 98 },
    { 0xf97fc7d0, 90 },
    { 0xd5bfc7d1, 100 },
    { 0xad8fc7d2, 23 },
    { 0xc79fc7d1, 76 },
    { 0x4fdfc7d4, 90 },
    { 0x44ffc7d5, 90 },
    { 0, 0 },
    { 0xb45fc7d7, 23 },
    { 0x123fc7d8, 81 },
    { 0, 0 },
    { 0x3edfc7da, 90 },
    { 0, 0 },
    { 0x8a5fc7dc, 90 },
    { 0x6cafc7dc, 90 },
    { 0xda9fc7de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764fc7e2, 61 },
    { 0, 0 },
    { 0x488fc7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8fc7e8, 93 },
    { 0x300fc7e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801fc7f2, 95 },
    { 0x5effc7f3, 67 },
    { 0xa42fc7f4, 52 },
    { 0xd9bfc7f5, 81 },
    { 0, 0 },
    { 0xad8fc7f7, 33 },
    { 0x496fc7f8, 95 },
    { 0x5f1fc7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2fc7fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fc800, 33 },
    { 0xb53fc801, 55 },
    { 0xdddfc801, 33 },
    { 0, 0 },
    { 0xac6fc804, 33 },
    { 0xe5dfc805, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490fc80b, 100 },
    { 0x6affc80b, 61 },
    { 0x3f2fc80b, 66 },
    { 0, 0 },
    { 0x851fc80f, 23 },
    { 0x750fc810, 76 },
    { 0xf9ffc80f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfc815, 85 },
    { 0xabffc816, 88 },
    { 0x4d5fc817, 90 },
    { 0xf74fc816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91fc81b, 95 },
    { 0, 0 },
    { 0x71cfc81d, 64 },
    { 0x282fc81d, 90 },
    { 0, 0 },
    { 0xa46fc820, 52 },
    { 0x47cfc820, 90 },
    { 0x505fc822, 33 },
    { 0xccfc820, 55 },
    { 0x8ebfc824, 33 },
    { 0, 0 },
    { 0xc71fc826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbfc82a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787fc831, 76 },
    { 0xe86fc832, 33 },
    { 0x2abfc833, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4fc836, 90 },
    { 0, 0 },
    { 0xa1afc838, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fc849, 61 },
    { 0, 0 },
    { 0x22cfc84b, 33 },
    { 0, 0 },
    { 0x984fc84d, 40 },
    { 0, 0 },
    { 0xa7efc84f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacffc859, 76 },
    { 0xe84fc85a, 90 },
    { 0xfb2fc859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fc866, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28fc869, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfc86d, 61 },
    { 0, 0 },
    { 0xd76fc86f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5fc872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cfc87b, 64 },
    { 0, 0 },
    { 0x64afc87d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x100fc880, 90 },
    { 0x24efc881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3fc884, 99 },
    { 0x346fc884, 68 },
    { 0x9fdfc886, 59 },
    { 0x9cdfc884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fc88a, 95 },
    { 0xc42fc88b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3fc890, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10efc89d, 98 },
    { 0x93efc89d, 33 },
    { 0, 0 },
    { 0xe92fc8a0, 93 },
    { 0xe2bfc8a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05fc8a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9fc8a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fc8b5, 78 },
    { 0x376fc8b6, 55 },
    { 0x96dfc8b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986fc8bd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3fc8c7, 33 },
    { 0, 0 },
    { 0x7fc8c9, 90 },
    { 0x249fc8ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efc8d0, 90 },
    { 0x81dfc8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cafc8d5, 66 },
    { 0, 0 },
    { 0x806fc8d7, 87 },
    { 0, 0 },
    { 0xb5efc8d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4fc8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734fc8eb, 67 },
    { 0, 0 },
    { 0xdaefc8ed, 33 },
    { 0, 0 },
    { 0x9f2fc8ef, 77 },
    { 0xf07fc8f0, 61 },
    { 0x1a4fc8f1, 33 },
    { 0x514fc8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bfc8f7, 80 },
    { 0x26bfc8f8, 90 },
    { 0xc28fc8f9, 55 },
    { 0, 0 },
    { 0xe85fc8fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20afc8fe, 23 },
    { 0xca9fc8ff, 95 },
    { 0, 0 },
    { 0x9d5fc901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421fc908, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834fc90c, 98 },
    { 0xb0efc90d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0fc911, 90 },
    { 0x3a6fc912, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7fc918, 33 },
    { 0, 0 },
    { 0x54fc91a, 100 },
    { 0, 0 },
    { 0x189fc91c, 75 },
    { 0, 0 },
    { 0x8f0fc91e, 93 },
    { 0, 0 },
    { 0x876fc920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7fc923, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bfc926, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76efc92c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fffc930, 69 },
    { 0xba0fc931, 65 },
    { 0x713fc931, 33 },
    { 0x694fc933, 69 },
    { 0x761fc933, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafc937, 84 },
    { 0x9d7fc937, 61 },
    { 0, 0 },
    { 0x2a6fc93a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecfc93e, 77 },
    { 0xe4afc93f, 76 },
    { 0xe59fc940, 98 },
    { 0x3ccfc941, 33 },
    { 0x784fc940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3fc945, 61 },
    { 0x695fc945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12fc949, 88 },
    { 0x678fc94a, 98 },
    { 0xf56fc94a, 84 },
    { 0, 0 },
    { 0x817fc94d, 85 },
    { 0x558fc94e, 90 },
    { 0x120fc94d, 95 },
    { 0x65bfc94d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdfc955, 95 },
    { 0, 0 },
    { 0x462fc957, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654fc95d, 61 },
    { 0x949fc95e, 23 },
    { 0xea1fc95e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1fc964, 55 },
    { 0x489fc965, 43 },
    { 0xcf2fc964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cfc96c, 36 },
    { 0xc06fc96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fc971, 33 },
    { 0, 0 },
    { 0xa90fc973, 76 },
    { 0x3d7fc973, 95 },
    { 0x5bffc973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fafc978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286fc97e, 33 },
    { 0x25fc97f, 97 },
    { 0x40efc97e, 34 },
    { 0xe16fc981, 85 },
    { 0xc80fc981, 60 },
    { 0x3befc983, 95 },
    { 0, 0 },
    { 0xaf1fc985, 90 },
    { 0xf74fc985, 95 },
    { 0xa01fc987, 98 },
    { 0x28cfc986, 98 },
    { 0xf3afc985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4fc991, 33 },
    { 0x553fc991, 43 },
    { 0, 0 },
    { 0xafafc994, 33 },
    { 0x367fc994, 75 },
    { 0xac2fc995, 17 },
    { 0xfb5fc997, 93 },
    { 0x561fc998, 84 },
    { 0xe2fc999, 33 },
    { 0x519fc994, 90 },
    { 0x1eafc994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19afc99e, 23 },
    { 0, 0 },
    { 0x65dfc9a0, 46 },
    { 0x610fc9a0, 55 },
    { 0xe98fc9a0, 98 },
    { 0, 0 },
    { 0x98efc9a4, 76 },
    { 0xa19fc9a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79fc9ab, 95 },
    { 0x17dfc9ac, 99 },
    { 0x860fc9ad, 23 },
    { 0, 0 },
    { 0x29dfc9af, 76 },
    { 0xd0afc9b0, 77 },
    { 0x852fc9b1, 61 },
    { 0x917fc9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x479fc9b5, 90 },
    { 0, 0 },
    { 0xd6afc9b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x735fc9ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ffc9bd, 90 },
    { 0, 0 },
    { 0x198fc9bf, 34 },
    { 0, 0 },
    { 0x5fafc9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cfc9c5, 68 },
    { 0x95afc9c5, 93 },
    { 0, 0 },
    { 0x827fc9c8, 43 },
    { 0x137fc9c9, 55 },
    { 0x276fc9ca, 61 },
    { 0x29bfc9c9, 43 },
    { 0xbaefc9cc, 65 },
    { 0xa3dfc9ca, 34 },
    { 0xffc9ce, 12 },
    { 0xec5fc9c9, 98 },
    { 0xcadfc9d0, 53 },
    { 0, 0 },
    { 0xfe4fc9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25fc9d8, 95 },
    { 0x10cfc9d9, 52 },
    { 0xb90fc9d9, 46 },
    { 0, 0 },
    { 0x2ccfc9dc, 23 },
    { 0x2f7fc9dc, 95 },
    { 0, 0 },
    { 0xfcdfc9df, 43 },
    { 0, 0 },
    { 0x634fc9e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3fc9e4, 23 },
    { 0xeb2fc9e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942fc9f0, 76 },
    { 0x7befc9f1, 90 },
    { 0, 0 },
    { 0x12afc9f3, 93 },
    { 0x55fc9f3, 76 },
    { 0, 0 },
    { 0xe4efc9f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801fc9fb, 90 },
    { 0, 0 },
    { 0x5c6fc9fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x644fca00, 33 },
    { 0x563fca01, 87 },
    { 0xfcafca02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fca08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61fca0b, 61 },
    { 0, 0 },
    { 0x41efca0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ffca15, 76 },
    { 0x401fca16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127fca1b, 95 },
    { 0, 0 },
    { 0xd9ffca1d, 97 },
    { 0xebfca1d, 76 },
    { 0x27efca1f, 90 },
    { 0xf76fca1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811fca28, 90 },
    { 0x8cffca28, 43 },
    { 0, 0 },
    { 0x56bfca2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fca30, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bfca35, 90 },
    { 0, 0 },
    { 0x53dfca37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22fca3a, 33 },
    { 0xc83fca3a, 68 },
    { 0xf70fca3b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79afca40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1afca46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fca4f, 34 },
    { 0, 0 },
    { 0x135fca51, 91 },
    { 0xa08fca52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1fca56, 46 },
    { 0x3aefca56, 76 },
    { 0, 0 },
    { 0x63afca59, 55 },
    { 0, 0 },
    { 0xd53fca5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9fca5e, 46 },
    { 0xdd6fca5f, 91 },
    { 0x190fca5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5fca63, 43 },
    { 0, 0 },
    { 0xe19fca65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2fca6a, 61 },
    { 0x2b5fca6b, 55 },
    { 0xb3ffca6b, 85 },
    { 0, 0 },
    { 0xf36fca6e, 93 },
    { 0, 0 },
    { 0x4bffca70, 23 },
    { 0xba3fca71, 61 },
    { 0xb00fca72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fca76, 90 },
    { 0x44dfca77, 90 },
    { 0x6a3fca78, 43 },
    { 0xe55fca79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfca80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x586fca83, 91 },
    { 0x92ffca83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ffca89, 31 },
    { 0, 0 },
    { 0xf7fca8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1fca91, 33 },
    { 0xa6afca91, 98 },
    { 0, 0 },
    { 0xb8fca94, 98 },
    { 0, 0 },
    { 0xf53fca96, 33 },
    { 0x91ffca97, 95 },
    { 0x5c0fca97, 55 },
    { 0x4afca99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70fca9f, 84 },
    { 0xa1ffcaa0, 76 },
    { 0x5f5fcaa1, 33 },
    { 0, 0 },
    { 0x483fcaa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bfcaac, 40 },
    { 0, 0 },
    { 0x36cfcaae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bffcab1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bfcab4, 100 },
    { 0, 0 },
    { 0xa54fcab6, 93 },
    { 0x61bfcab7, 76 },
    { 0xd8fcab8, 55 },
    { 0x283fcab7, 98 },
    { 0xc50fcaba, 93 },
    { 0xdcafcabb, 51 },
    { 0, 0 },
    { 0xe6bfcabd, 76 },
    { 0, 0 },
    { 0x565fcabf, 46 },
    { 0x3f5fcabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecfcac5, 61 },
    { 0x678fcac6, 98 },
    { 0x586fcac6, 55 },
    { 0xd77fcac7, 33 },
    { 0xc54fcac8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cfcacd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffcad3, 33 },
    { 0, 0 },
    { 0x1c1fcad5, 46 },
    { 0xcfcad6, 95 },
    { 0, 0 },
    { 0xb99fcad8, 33 },
    { 0x91ffcad8, 90 },
    { 0x38fcada, 33 },
    { 0xbcefcad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111fcae1, 77 },
    { 0xcb0fcae1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63fcae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bfcaec, 90 },
    { 0x613fcaec, 90 },
    { 0xa40fcaee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cfcaf2, 23 },
    { 0, 0 },
    { 0xa9fcaf4, 76 },
    { 0, 0 },
    { 0x29fcaf6, 88 },
    { 0xd57fcaf6, 33 },
    { 0, 0 },
    { 0x14afcaf9, 69 },
    { 0x956fcafa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713fcb00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7fcb07, 90 },
    { 0x917fcb08, 61 },
    { 0x803fcb09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2fcb0c, 98 },
    { 0xdecfcb0d, 33 },
    { 0xafffcb0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ffcb12, 98 },
    { 0x294fcb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865fcb18, 96 },
    { 0x75efcb19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2fcb1c, 76 },
    { 0x6bfcb1d, 95 },
    { 0x786fcb1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537fcb24, 90 },
    { 0xbbafcb25, 33 },
    { 0x124fcb26, 53 },
    { 0x84dfcb26, 69 },
    { 0x34bfcb27, 90 },
    { 0xbf5fcb27, 90 },
    { 0xdabfcb26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0fcb2f, 46 },
    { 0xd9efcb30, 33 },
    { 0, 0 },
    { 0xadffcb32, 93 },
    { 0xdb4fcb33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x347fcb36, 90 },
    { 0x7a6fcb37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31fcb3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fefcb43, 61 },
    { 0, 0 },
    { 0x666fcb45, 66 },
    { 0xe74fcb45, 99 },
    { 0x7dfcb47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fcb4a, 24 },
    { 0, 0 },
    { 0x191fcb4c, 90 },
    { 0, 0 },
    { 0xd9dfcb4e, 95 },
    { 0, 0 },
    { 0xc1fcb50, 52 },
    { 0, 0 },
    { 0xb3fcb52, 34 },
    { 0xf2afcb53, 61 },
    { 0xe53fcb54, 95 },
    { 0x1a5fcb55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0fcb5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16efcb5e, 100 },
    { 0, 0 },
    { 0xbbffcb60, 23 },
    { 0, 0 },
    { 0x80ffcb62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fcb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2fcb72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53fcb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cefcb7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dfcb7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fcb82, 90 },
    { 0x6a7fcb82, 46 },
    { 0x61cfcb84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258fcb88, 33 },
    { 0x5e6fcb88, 84 },
    { 0x6ccfcb8a, 99 },
    { 0xc4ffcb8b, 90 },
    { 0, 0 },
    { 0x4a9fcb8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fcb95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x124fcb98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41fcb9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfcb9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bfcba1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bafcba5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fcbaa, 76 },
    { 0xbdffcbab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22fcbaf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacffcbb3, 61 },
    { 0, 0 },
    { 0x1fafcbb5, 84 },
    { 0x496fcbb6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cfcbb9, 65 },
    { 0, 0 },
    { 0x7d6fcbbb, 23 },
    { 0xe43fcbbc, 48 },
    { 0x58afcbbd, 90 },
    { 0xb96fcbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fcbc1, 55 },
    { 0x767fcbc2, 23 },
    { 0, 0 },
    { 0x2ebfcbc4, 95 },
    { 0x7c1fcbc5, 33 },
    { 0xcafcbc6, 33 },
    { 0x4e0fcbc7, 95 },
    { 0x12bfcbc6, 84 },
    { 0, 0 },
    { 0xdc6fcbca, 55 },
    { 0xfddfcbca, 95 },
    { 0, 0 },
    { 0xcfffcbcd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66fcbd0, 90 },
    { 0xa0efcbd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179fcbd7, 23 },
    { 0, 0 },
    { 0xf57fcbd9, 77 },
    { 0x59dfcbd9, 99 },
    { 0x79cfcbd9, 55 },
    { 0xd0dfcbdb, 67 },
    { 0x329fcbda, 90 },
    { 0, 0 },
    { 0xc8afcbdf, 23 },
    { 0x1b4fcbdf, 52 },
    { 0, 0 },
    { 0xccdfcbe2, 90 },
    { 0, 0 },
    { 0x558fcbe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54fcbeb, 55 },
    { 0x1eefcbeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fcbef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ffcbf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefcbf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42fcc00, 24 },
    { 0, 0 },
    { 0xd42fcc02, 17 },
    { 0xc6cfcc02, 98 },
    { 0x4e9fcc04, 38 },
    { 0x7a7fcc03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfcc0a, 98 },
    { 0x842fcc0b, 90 },
    { 0xe76fcc0c, 90 },
    { 0xee5fcc0b, 98 },
    { 0xb69fcc0e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949fcc13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdfcc17, 23 },
    { 0, 0 },
    { 0xe67fcc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56fcc1c, 56 },
    { 0xd8afcc1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfcc24, 61 },
    { 0xbe0fcc24, 76 },
    { 0xa8fcc26, 52 },
    { 0, 0 },
    { 0x352fcc28, 77 },
    { 0x92fcc29, 95 },
    { 0x8dcfcc29, 33 },
    { 0xe6afcc2b, 61 },
    { 0x1d0fcc2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9fcc2f, 78 },
    { 0, 0 },
    { 0x9fcc31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6fcc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfcc3a, 33 },
    { 0x754fcc3b, 33 },
    { 0x177fcc3c, 88 },
    { 0x7b6fcc3c, 76 },
    { 0xb3efcc3e, 61 },
    { 0xc9efcc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4fcc42, 93 },
    { 0x5defcc43, 77 },
    { 0xe82fcc42, 61 },
    { 0x97cfcc43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6fcc4a, 76 },
    { 0, 0 },
    { 0x7f8fcc4c, 43 },
    { 0xe30fcc4d, 95 },
    { 0xa2afcc4e, 78 },
    { 0xbc9fcc4e, 98 },
    { 0x703fcc4c, 95 },
    { 0, 0 },
    { 0x388fcc52, 85 },
    { 0xe2bfcc53, 91 },
    { 0xbffcc54, 98 },
    { 0xb74fcc55, 90 },
    { 0, 0 },
    { 0xdb5fcc57, 90 },
    { 0x951fcc58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fcc5d, 78 },
    { 0xc96fcc5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906fcc63, 90 },
    { 0xbdbfcc64, 46 },
    { 0, 0 },
    { 0xe81fcc66, 90 },
    { 0xee1fcc67, 76 },
    { 0x653fcc68, 46 },
    { 0x4ecfcc66, 87 },
    { 0xef6fcc6a, 87 },
    { 0xde4fcc6b, 98 },
    { 0x180fcc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4efcc72, 68 },
    { 0, 0 },
    { 0x909fcc74, 23 },
    { 0x362fcc75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efcc7d, 61 },
    { 0, 0 },
    { 0x5e9fcc7f, 90 },
    { 0x5aafcc7f, 34 },
    { 0xf3fcc81, 46 },
    { 0x5f0fcc80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fcc8c, 64 },
    { 0, 0 },
    { 0x758fcc8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dfcc94, 33 },
    { 0xd50fcc94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4efcc99, 23 },
    { 0xe1afcc9a, 61 },
    { 0xaefcc9b, 66 },
    { 0xf7cfcc9b, 61 },
    { 0, 0 },
    { 0x834fcc9e, 52 },
    { 0, 0 },
    { 0x6cdfcca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8fcca4, 90 },
    { 0x87dfcca5, 46 },
    { 0, 0 },
    { 0x7bdfcca7, 90 },
    { 0, 0 },
    { 0x4fafcca9, 34 },
    { 0, 0 },
    { 0x7eafccab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6fccae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fccb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281fccb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfccc3, 67 },
    { 0x737fccc4, 95 },
    { 0x573fccc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abfccc8, 33 },
    { 0, 0 },
    { 0x9f3fccca, 33 },
    { 0x283fcccb, 93 },
    { 0x9a2fcccc, 33 },
    { 0x29dfcccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ffccd1, 34 },
    { 0, 0 },
    { 0x22dfccd3, 76 },
    { 0, 0 },
    { 0x726fccd5, 98 },
    { 0x2f8fccd6, 43 },
    { 0xbf5fccd6, 85 },
    { 0xde5fccd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939fccdd, 33 },
    { 0, 0 },
    { 0xe59fccdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59fcce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01fcce5, 98 },
    { 0x851fcce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67fcceb, 23 },
    { 0, 0 },
    { 0xa41fcced, 85 },
    { 0x658fccee, 46 },
    { 0xf45fccee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dafccf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23fccfa, 33 },
    { 0x193fccfb, 99 },
    { 0x7c8fccfc, 61 },
    { 0xccdfccfc, 78 },
    { 0xf97fccfe, 46 },
    { 0xcebfccff, 23 },
    { 0x9c0fcd00, 33 },
    { 0xe46fcd01, 34 },
    { 0, 0 },
    { 0x4e0fcd03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03fcd06, 87 },
    { 0x4cbfcd07, 61 },
    { 0xf5afcd08, 23 },
    { 0, 0 },
    { 0xd0bfcd0a, 95 },
    { 0x160fcd0b, 59 },
    { 0x85ffcd0a, 67 },
    { 0x5a4fcd0d, 43 },
    { 0x1e2fcd0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x850fcd11, 95 },
    { 0x2f2fcd12, 61 },
    { 0xa11fcd11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcd17, 33 },
    { 0x863fcd18, 85 },
    { 0, 0 },
    { 0x90bfcd1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fcd21, 85 },
    { 0, 0 },
    { 0xe0efcd23, 98 },
    { 0x4a1fcd24, 34 },
    { 0, 0 },
    { 0xa98fcd26, 98 },
    { 0x93efcd26, 98 },
    { 0xe7fcd26, 43 },
    { 0xde1fcd29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afcd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3fcd36, 95 },
    { 0xf01fcd37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4fcd3b, 66 },
    { 0x949fcd3c, 95 },
    { 0x322fcd3b, 90 },
    { 0x4b2fcd3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39fcd42, 99 },
    { 0x4f1fcd43, 93 },
    { 0xca7fcd44, 98 },
    { 0x145fcd45, 76 },
    { 0xd0bfcd44, 55 },
    { 0x495fcd47, 95 },
    { 0xd4dfcd47, 33 },
    { 0x238fcd47, 95 },
    { 0x749fcd4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6fcd54, 76 },
    { 0xcfafcd54, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04fcd5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dfcd5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8afcd61, 90 },
    { 0x5b1fcd61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fcd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bfcd6f, 33 },
    { 0x1b8fcd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fcd75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cfcd7c, 90 },
    { 0, 0 },
    { 0xee5fcd7e, 76 },
    { 0x419fcd7f, 61 },
    { 0, 0 },
    { 0x19efcd81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6fcd88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784fcd8c, 33 },
    { 0, 0 },
    { 0x76cfcd8e, 100 },
    { 0, 0 },
    { 0xc1dfcd90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278fcd95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7fcd9b, 33 },
    { 0x4fdfcd9c, 78 },
    { 0x6a7fcd9d, 51 },
    { 0x651fcd9e, 100 },
    { 0xd8efcd9f, 95 },
    { 0x9efcd9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x824fcda3, 33 },
    { 0, 0 },
    { 0xb17fcda5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8fcdaa, 90 },
    { 0x341fcdab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80fcdb3, 90 },
    { 0, 0 },
    { 0xf2afcdb5, 94 },
    { 0x853fcdb6, 76 },
    { 0, 0 },
    { 0xa2cfcdb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fcdbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4fcdbf, 80 },
    { 0xacefcdbf, 90 },
    { 0, 0 },
    { 0x838fcdc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0efcdce, 33 },
    { 0, 0 },
    { 0x5e6fcdd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61fcdd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93afcdd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487fcddc, 90 },
    { 0, 0 },
    { 0xbbdfcdde, 43 },
    { 0xbc4fcddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbfcde4, 93 },
    { 0, 0 },
    { 0x52efcde6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bfcdea, 33 },
    { 0, 0 },
    { 0x866fcdec, 23 },
    { 0x285fcded, 33 },
    { 0x44dfcdec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3fcdf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5fcdf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5fcdfd, 90 },
    { 0, 0 },
    { 0x8b3fcdff, 98 },
    { 0x106fce00, 55 },
    { 0, 0 },
    { 0x969fce02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190fce07, 93 },
    { 0x32bfce08, 95 },
    { 0, 0 },
    { 0x461fce0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fce0d, 90 },
    { 0, 0 },
    { 0xa38fce0f, 76 },
    { 0x5e6fce10, 75 },
    { 0x9c5fce11, 55 },
    { 0, 0 },
    { 0x7fcfce13, 34 },
    { 0xf65fce14, 94 },
    { 0, 0 },
    { 0x57efce16, 90 },
    { 0x18efce17, 14 },
    { 0xe1ffce18, 66 },
    { 0x8e2fce19, 90 },
    { 0, 0 },
    { 0x487fce1b, 66 },
    { 0xd9afce1b, 90 },
    { 0xff1fce1d, 72 },
    { 0, 0 },
    { 0x247fce1f, 61 },
    { 0xe05fce1f, 34 },
    { 0x71efce20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34fce26, 23 },
    { 0x3adfce26, 91 },
    { 0xb15fce28, 87 },
    { 0xb60fce29, 23 },
    { 0x17afce2a, 95 },
    { 0xcadfce2b, 90 },
    { 0x882fce2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fce2f, 95 },
    { 0x421fce30, 23 },
    { 0x7adfce31, 95 },
    { 0x2d8fce31, 61 },
    { 0x34cfce33, 33 },
    { 0xc6efce34, 33 },
    { 0xc7bfce35, 100 },
    { 0x628fce36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dafce3c, 33 },
    { 0x85fce3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23afce41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cfce44, 23 },
    { 0x522fce45, 98 },
    { 0x3a1fce44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7fce4a, 80 },
    { 0x24dfce4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cfce4e, 61 },
    { 0x3fffce4f, 52 },
    { 0, 0 },
    { 0x592fce51, 76 },
    { 0xd97fce51, 65 },
    { 0xc9dfce51, 61 },
    { 0xa6ffce54, 87 },
    { 0xa8ffce53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00fce59, 99 },
    { 0x843fce5a, 90 },
    { 0x403fce5b, 61 },
    { 0xe68fce5c, 91 },
    { 0xb9dfce5b, 83 },
    { 0xc2afce5e, 85 },
    { 0x863fce5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8fce64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fce67, 61 },
    { 0x7f2fce68, 33 },
    { 0x4f5fce69, 90 },
    { 0x8effce68, 87 },
    { 0xd65fce6b, 81 },
    { 0x8cafce6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccfce70, 76 },
    { 0x48bfce71, 61 },
    { 0, 0 },
    { 0x6befce73, 33 },
    { 0, 0 },
    { 0xceffce75, 55 },
    { 0, 0 },
    { 0xc98fce77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbfce7a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cfce7e, 99 },
    { 0x45dfce7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5fce85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6fce88, 46 },
    { 0, 0 },
    { 0x7a7fce8a, 68 },
    { 0xeaefce8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fce8f, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dfce97, 90 },
    { 0, 0 },
    { 0x6cdfce99, 95 },
    { 0x8bffce9a, 90 },
    { 0xf0dfce9a, 61 },
    { 0, 0 },
    { 0x109fce9d, 33 },
    { 0x485fce9d, 33 },
    { 0xa87fce9f, 23 },
    { 0, 0 },
    { 0xb90fcea1, 61 },
    { 0xc89fcea2, 43 },
    { 0xe74fcea3, 40 },
    { 0xa46fcea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bfceac, 78 },
    { 0, 0 },
    { 0xbdefceae, 55 },
    { 0, 0 },
    { 0x6f9fceb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16fcec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6fcec6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89fcecc, 23 },
    { 0xe51fcecd, 52 },
    { 0, 0 },
    { 0xfebfcecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfced3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6fced8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fcede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410fcee2, 85 },
    { 0xb10fcee2, 33 },
    { 0x674fcee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18fcee7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58fcef3, 60 },
    { 0, 0 },
    { 0xb4dfcef5, 81 },
    { 0xc59fcef6, 61 },
    { 0, 0 },
    { 0x8cefcef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9efcefe, 19 },
    { 0, 0 },
    { 0x6c5fcf00, 95 },
    { 0, 0 },
    { 0xc5ffcf02, 76 },
    { 0xc98fcf02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfcf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x496fcf0a, 23 },
    { 0x77afcf0b, 91 },
    { 0xd6ffcf0c, 65 },
    { 0x3cfcf0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cfcf15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fcf18, 90 },
    { 0xcf4fcf19, 90 },
    { 0xce5fcf1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fcf28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bfcf31, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcf34, 65 },
    { 0, 0 },
    { 0xaa0fcf36, 61 },
    { 0x3afcf36, 90 },
    { 0xa3ffcf37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfcf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9fcf42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3fcf45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bfcf4c, 87 },
    { 0xfb6fcf4d, 40 },
    { 0x7d8fcf4d, 55 },
    { 0, 0 },
    { 0xb4efcf50, 88 },
    { 0, 0 },
    { 0xdcefcf52, 81 },
    { 0, 0 },
    { 0x6b0fcf54, 33 },
    { 0, 0 },
    { 0xf76fcf56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32fcf5a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33fcf5d, 55 },
    { 0, 0 },
    { 0xa99fcf5f, 33 },
    { 0xa9dfcf60, 91 },
    { 0x826fcf5f, 95 },
    { 0x121fcf62, 29 },
    { 0x903fcf62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cfcf66, 56 },
    { 0, 0 },
    { 0x3e4fcf68, 98 },
    { 0x619fcf69, 33 },
    { 0, 0 },
    { 0x2cefcf6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ffcf70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebafcf74, 88 },
    { 0xd59fcf75, 61 },
    { 0xd83fcf76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fcf79, 90 },
    { 0x59cfcf79, 100 },
    { 0xf2dfcf7a, 95 },
    { 0x876fcf7c, 33 },
    { 0x616fcf7a, 93 },
    { 0, 0 },
    { 0x6c4fcf7f, 84 },
    { 0, 0 },
    { 0xe82fcf81, 85 },
    { 0xf2afcf81, 40 },
    { 0xb0bfcf83, 33 },
    { 0xac2fcf83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fcf8a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558fcf92, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fcf95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffcf98, 101 },
    { 0xeacfcf99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x828fcf9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7fcfa1, 23 },
    { 0xd42fcfa2, 68 },
    { 0, 0 },
    { 0x9f0fcfa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefcfa7, 88 },
    { 0x2a1fcfa8, 95 },
    { 0x107fcfa9, 61 },
    { 0x7effcfaa, 77 },
    { 0xc7cfcfa7, 33 },
    { 0x202fcfa7, 90 },
    { 0xa92fcfad, 61 },
    { 0x1f8fcfae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbfcfb3, 61 },
    { 0xfa2fcfb3, 76 },
    { 0xd00fcfb5, 90 },
    { 0x76fcfb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cfcfbc, 52 },
    { 0, 0 },
    { 0xf7fcfbe, 90 },
    { 0xe54fcfbf, 90 },
    { 0x70efcfbf, 90 },
    { 0x90dfcfbf, 43 },
    { 0xf87fcfbe, 98 },
    { 0xe8efcfc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adfcfc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213fcfcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fcfd2, 47 },
    { 0, 0 },
    { 0x575fcfd4, 55 },
    { 0x8cbfcfd5, 95 },
    { 0x6f3fcfd6, 90 },
    { 0x4a1fcfd6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cafcfdf, 76 },
    { 0x89bfcfe0, 33 },
    { 0, 0 },
    { 0xe8afcfe2, 23 },
    { 0x73afcfe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1fcfe9, 61 },
    { 0xe99fcfea, 95 },
    { 0, 0 },
    { 0xa3ffcfec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cfcfef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7fcff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x432fcff5, 33 },
    { 0, 0 },
    { 0xab3fcff7, 61 },
    { 0, 0 },
    { 0x121fcff9, 33 },
    { 0xb4fcffa, 98 },
    { 0xb02fcffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11fd001, 46 },
    { 0x60ffd002, 98 },
    { 0, 0 },
    { 0x283fd004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdffd007, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369fd00c, 85 },
    { 0x628fd00d, 91 },
    { 0xe02fd00e, 33 },
    { 0xe81fd00f, 77 },
    { 0xe95fd010, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2afd014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefffd017, 70 },
    { 0x4dbfd018, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bfd01b, 61 },
    { 0, 0 },
    { 0xa31fd01d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfd020, 61 },
    { 0xe47fd021, 93 },
    { 0xbd1fd022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eafd025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86fd031, 61 },
    { 0, 0 },
    { 0x88bfd033, 38 },
    { 0, 0 },
    { 0x9a1fd035, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464fd03a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x546fd03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68afd041, 38 },
    { 0, 0 },
    { 0xf74fd043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cfd048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82fd04f, 93 },
    { 0x82efd04f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd055, 55 },
    { 0xc2dfd055, 91 },
    { 0x460fd057, 77 },
    { 0x41afd055, 55 },
    { 0, 0 },
    { 0xa50fd05a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfd05d, 52 },
    { 0, 0 },
    { 0x965fd05f, 95 },
    { 0xc0afd060, 66 },
    { 0xe6afd061, 23 },
    { 0x8cbfd060, 93 },
    { 0xacefd063, 23 },
    { 0x11fd05f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1fd068, 80 },
    { 0xe91fd068, 87 },
    { 0x28cfd06a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dfd072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x328fd075, 33 },
    { 0, 0 },
    { 0x9aefd077, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbfd07f, 55 },
    { 0xd0afd080, 90 },
    { 0xffefd081, 78 },
    { 0xd4efd082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34efd085, 33 },
    { 0x927fd086, 76 },
    { 0x958fd087, 98 },
    { 0x9eefd085, 88 },
    { 0x5ddfd089, 46 },
    { 0x421fd08a, 97 },
    { 0xb37fd085, 76 },
    { 0xf07fd087, 95 },
    { 0, 0 },
    { 0x7b3fd08e, 23 },
    { 0xe3afd08f, 76 },
    { 0x87afd090, 61 },
    { 0xccfd08f, 55 },
    { 0, 0 },
    { 0x181fd093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fd097, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eefd09c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aefd0b1, 34 },
    { 0, 0 },
    { 0x3d4fd0b3, 98 },
    { 0xe19fd0b4, 33 },
    { 0xe83fd0b5, 93 },
    { 0xffdfd0b4, 52 },
    { 0x7ccfd0b6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bfd0bd, 91 },
    { 0, 0 },
    { 0x7dafd0bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715fd0c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efd0ca, 90 },
    { 0, 0 },
    { 0x219fd0cc, 59 },
    { 0, 0 },
    { 0x2a0fd0ce, 90 },
    { 0xe84fd0ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfd0d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bafd0da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49fd0e1, 34 },
    { 0, 0 },
    { 0x928fd0e3, 98 },
    { 0, 0 },
    { 0x585fd0e5, 90 },
    { 0, 0 },
    { 0xb90fd0e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7fd0eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774fd0ef, 23 },
    { 0, 0 },
    { 0x7bcfd0f1, 34 },
    { 0x4a2fd0f2, 98 },
    { 0xf4cfd0f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcfd0f7, 33 },
    { 0x309fd0f8, 98 },
    { 0xcafd0f9, 33 },
    { 0, 0 },
    { 0xe2dfd0fb, 36 },
    { 0xc17fd0fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdfd105, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fd10e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7fd113, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x586fd116, 93 },
    { 0, 0 },
    { 0x24afd118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67afd11c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0fd122, 61 },
    { 0x678fd122, 95 },
    { 0xd1afd124, 95 },
    { 0x2c5fd125, 68 },
    { 0, 0 },
    { 0x1f1fd127, 98 },
    { 0, 0 },
    { 0x53dfd129, 90 },
    { 0, 0 },
    { 0x963fd12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9fd132, 66 },
    { 0x178fd132, 17 },
    { 0x735fd134, 95 },
    { 0xeaefd135, 90 },
    { 0, 0 },
    { 0x626fd137, 55 },
    { 0x9d6fd137, 61 },
    { 0xdf2fd138, 91 },
    { 0xa7ffd13a, 98 },
    { 0xf6bfd13b, 88 },
    { 0xa0fd13b, 85 },
    { 0x2f5fd13a, 52 },
    { 0x591fd138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fd143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461fd14a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1fd156, 61 },
    { 0xee5fd156, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45fd15d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7fd160, 65 },
    { 0, 0 },
    { 0xa1efd162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fd166, 34 },
    { 0xf13fd167, 33 },
    { 0xca2fd168, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87efd16b, 90 },
    { 0xec0fd16c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00fd170, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6fd17c, 48 },
    { 0xdeafd17d, 78 },
    { 0x18bfd17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22fd183, 95 },
    { 0x7affd184, 23 },
    { 0x5aefd185, 33 },
    { 0xa14fd186, 23 },
    { 0x9cafd187, 33 },
    { 0x6affd187, 64 },
    { 0x12afd189, 43 },
    { 0xb8fd187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edfd18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95fd192, 77 },
    { 0xf8bfd193, 23 },
    { 0xa31fd193, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fd19a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68fd19d, 61 },
    { 0xe82fd19d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345fd1a3, 100 },
    { 0xdacfd1a4, 95 },
    { 0x9f1fd1a5, 75 },
    { 0x15cfd1a3, 55 },
    { 0x6b3fd1a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439fd1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5fd1b0, 95 },
    { 0xc64fd1b1, 90 },
    { 0xc0bfd1b1, 33 },
    { 0, 0 },
    { 0xed7fd1b4, 98 },
    { 0x271fd1b4, 33 },
    { 0, 0 },
    { 0xf06fd1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80fd1ba, 93 },
    { 0x300fd1bb, 55 },
    { 0, 0 },
    { 0x3a1fd1bd, 98 },
    { 0, 0 },
    { 0x525fd1bf, 34 },
    { 0, 0 },
    { 0xbbafd1c1, 61 },
    { 0xd6bfd1c2, 53 },
    { 0xa2dfd1c3, 34 },
    { 0x1ddfd1c4, 33 },
    { 0x8c3fd1c4, 95 },
    { 0x4f6fd1c6, 33 },
    { 0xe48fd1c6, 33 },
    { 0x155fd1c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407fd1cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cfd1d2, 23 },
    { 0xf3afd1d2, 23 },
    { 0, 0 },
    { 0x29dfd1d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bfd1d8, 77 },
    { 0x7d2fd1d9, 95 },
    { 0x51efd1d9, 90 },
    { 0, 0 },
    { 0x3b4fd1dc, 46 },
    { 0, 0 },
    { 0xea7fd1de, 2 },
    { 0x625fd1df, 76 },
    { 0x55afd1df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fd1e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242fd1ea, 55 },
    { 0, 0 },
    { 0xe3ffd1ec, 64 },
    { 0x112fd1ed, 61 },
    { 0x430fd1ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99fd1f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5fd1f7, 90 },
    { 0x3fcfd1f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0fd201, 33 },
    { 0x1a3fd202, 66 },
    { 0xe31fd203, 76 },
    { 0xfeafd204, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298fd20a, 90 },
    { 0, 0 },
    { 0x936fd20c, 67 },
    { 0, 0 },
    { 0x3a4fd20e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd216, 69 },
    { 0xb6dfd217, 33 },
    { 0, 0 },
    { 0x1cefd219, 98 },
    { 0, 0 },
    { 0xe6ffd21b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efd21e, 95 },
    { 0, 0 },
    { 0xe52fd220, 93 },
    { 0, 0 },
    { 0x524fd222, 55 },
    { 0x279fd223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dfd226, 90 },
    { 0, 0 },
    { 0xe3ffd228, 94 },
    { 0, 0 },
    { 0xc35fd22a, 65 },
    { 0, 0 },
    { 0xf4bfd22c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69fd230, 61 },
    { 0, 0 },
    { 0x185fd232, 83 },
    { 0, 0 },
    { 0xf5fd234, 61 },
    { 0, 0 },
    { 0x40cfd236, 61 },
    { 0xab1fd236, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143fd23d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2fd241, 66 },
    { 0xe0ffd241, 61 },
    { 0xc4efd243, 61 },
    { 0xf5efd243, 34 },
    { 0x7acfd242, 98 },
    { 0x63afd244, 88 },
    { 0x874fd245, 100 },
    { 0xcefd246, 91 },
    { 0x8f8fd246, 55 },
    { 0xa28fd241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973fd24e, 33 },
    { 0xa33fd24f, 95 },
    { 0x9c2fd24f, 90 },
    { 0xdf7fd251, 95 },
    { 0xd13fd251, 98 },
    { 0xfe5fd253, 34 },
    { 0xe21fd254, 43 },
    { 0x88cfd253, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fd25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61fd263, 68 },
    { 0x564fd263, 33 },
    { 0xafbfd263, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x655fd268, 66 },
    { 0xdaefd269, 93 },
    { 0x741fd26a, 61 },
    { 0x21efd268, 18 },
    { 0, 0 },
    { 0xac8fd26d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00fd275, 90 },
    { 0x414fd276, 34 },
    { 0, 0 },
    { 0xbbfd278, 100 },
    { 0x2c6fd279, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67afd27d, 77 },
    { 0xb6efd27d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115fd282, 46 },
    { 0x85fd282, 98 },
    { 0, 0 },
    { 0x3a6fd285, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31fd295, 76 },
    { 0x748fd296, 90 },
    { 0, 0 },
    { 0x395fd298, 98 },
    { 0x976fd299, 91 },
    { 0x4dfd299, 46 },
    { 0, 0 },
    { 0x8abfd29c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ffd2a5, 19 },
    { 0, 0 },
    { 0xc25fd2a7, 98 },
    { 0, 0 },
    { 0x45efd2a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7fd2ae, 90 },
    { 0x865fd2ae, 90 },
    { 0x3fffd2ae, 87 },
    { 0x5b3fd2af, 19 },
    { 0, 0 },
    { 0x8eafd2b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9fd2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1fd2ba, 61 },
    { 0xe38fd2bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9efd2c6, 33 },
    { 0, 0 },
    { 0xf99fd2c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780fd2cd, 23 },
    { 0, 0 },
    { 0x54cfd2cf, 91 },
    { 0xfb8fd2d0, 34 },
    { 0, 0 },
    { 0x1c5fd2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705fd2d6, 23 },
    { 0xe52fd2d7, 90 },
    { 0xe17fd2d7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dfd2dd, 98 },
    { 0, 0 },
    { 0xc0ffd2df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1fd2e4, 55 },
    { 0, 0 },
    { 0xb6efd2e6, 43 },
    { 0xe06fd2e7, 95 },
    { 0xe9fd2e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffd2eb, 100 },
    { 0x4a3fd2ec, 61 },
    { 0x857fd2ed, 23 },
    { 0xd70fd2ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadffd2f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4fd2fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2fd2ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19fd306, 93 },
    { 0xcd1fd307, 93 },
    { 0x3b7fd307, 98 },
    { 0xfbefd307, 55 },
    { 0, 0 },
    { 0xe64fd30b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9fd310, 90 },
    { 0xf8efd311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605fd318, 33 },
    { 0, 0 },
    { 0x83dfd31a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffd31f, 33 },
    { 0x568fd320, 33 },
    { 0x54fd320, 61 },
    { 0xa1fd322, 90 },
    { 0x89fd322, 55 },
    { 0xbe1fd324, 66 },
    { 0xa71fd324, 33 },
    { 0xb9afd324, 76 },
    { 0x83dfd327, 90 },
    { 0x513fd328, 95 },
    { 0xd2ffd329, 61 },
    { 0xae4fd321, 55 },
    { 0x5e4fd32b, 93 },
    { 0x93afd32c, 61 },
    { 0x578fd32c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3fd331, 87 },
    { 0, 0 },
    { 0xd3ffd333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186fd33b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5fd33f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fd345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54fd348, 76 },
    { 0xc6dfd348, 33 },
    { 0xacbfd34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2efd34e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cefd357, 90 },
    { 0, 0 },
    { 0x2d8fd359, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fd35d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504fd362, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cfd36c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffd372, 76 },
    { 0, 0 },
    { 0x287fd374, 98 },
    { 0xbc8fd375, 90 },
    { 0xb96fd375, 98 },
    { 0xaddfd377, 61 },
    { 0xb73fd377, 77 },
    { 0x934fd378, 98 },
    { 0, 0 },
    { 0x745fd37b, 33 },
    { 0xd41fd37b, 95 },
    { 0x751fd37c, 46 },
    { 0, 0 },
    { 0xdc5fd37f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3fd388, 46 },
    { 0xa42fd389, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1afd394, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3fd397, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fd39b, 23 },
    { 0xba6fd39c, 52 },
    { 0xc25fd39d, 95 },
    { 0xc50fd39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eafd3a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dffd3aa, 61 },
    { 0, 0 },
    { 0xf5bfd3ac, 84 },
    { 0x244fd3ac, 98 },
    { 0, 0 },
    { 0x3d4fd3af, 81 },
    { 0, 0 },
    { 0xb16fd3b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91fd3b7, 78 },
    { 0, 0 },
    { 0x244fd3b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51fd3bd, 33 },
    { 0xf6bfd3be, 90 },
    { 0x88fd3bd, 98 },
    { 0x863fd3c0, 34 },
    { 0xbffd3bf, 61 },
    { 0x3c6fd3c2, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04fd3c6, 95 },
    { 0x90bfd3c7, 23 },
    { 0x5eafd3c8, 53 },
    { 0x5bffd3c8, 17 },
    { 0, 0 },
    { 0x626fd3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3fd3d0, 93 },
    { 0, 0 },
    { 0x2a5fd3d2, 94 },
    { 0, 0 },
    { 0xbb4fd3d4, 88 },
    { 0x34efd3d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6fd3db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfd3df, 90 },
    { 0xf12fd3e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69fd3e7, 23 },
    { 0x7a0fd3e7, 95 },
    { 0xbddfd3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6fd3ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7fd3f1, 90 },
    { 0x447fd3f2, 33 },
    { 0xca1fd3f1, 61 },
    { 0x793fd3f4, 33 },
    { 0x22fd3f5, 95 },
    { 0x900fd3f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bfd3f9, 81 },
    { 0x1cdfd3fa, 84 },
    { 0, 0 },
    { 0x6c3fd3fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dfd400, 98 },
    { 0xc6efd401, 23 },
    { 0xf3ffd400, 87 },
    { 0, 0 },
    { 0xa6efd404, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12efd408, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x842fd40b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27fd40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6fd414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafd418, 76 },
    { 0xf1afd419, 95 },
    { 0xda5fd41a, 61 },
    { 0, 0 },
    { 0xa53fd41c, 98 },
    { 0xb49fd41d, 99 },
    { 0x336fd41d, 99 },
    { 0xdc3fd41e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8fd425, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x829fd428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ffd42d, 93 },
    { 0xcfefd42e, 36 },
    { 0x2f7fd42f, 34 },
    { 0xa0fd42f, 33 },
    { 0x218fd431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ffd434, 87 },
    { 0xcffd435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffd43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30fd43d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dfd440, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673fd445, 66 },
    { 0x3afd445, 33 },
    { 0x9dfd445, 82 },
    { 0x532fd447, 90 },
    { 0x636fd446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6fd44c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabafd452, 99 },
    { 0x559fd453, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebfd456, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fd45b, 98 },
    { 0xcb0fd45c, 91 },
    { 0x925fd45b, 23 },
    { 0, 0 },
    { 0x20ffd45f, 33 },
    { 0xb5dfd460, 76 },
    { 0xb27fd461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2fd467, 17 },
    { 0x28bfd468, 76 },
    { 0xc24fd468, 33 },
    { 0xfd8fd467, 36 },
    { 0xb1afd46b, 90 },
    { 0, 0 },
    { 0xf74fd46d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fd473, 95 },
    { 0x9f9fd473, 95 },
    { 0x434fd474, 95 },
    { 0x52efd473, 33 },
    { 0x259fd477, 61 },
    { 0x185fd478, 43 },
    { 0x5a5fd477, 34 },
    { 0x88cfd47a, 33 },
    { 0x833fd476, 95 },
    { 0xa9ffd476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40fd481, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6fd48c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x688fd48f, 23 },
    { 0, 0 },
    { 0xac1fd491, 95 },
    { 0x9ccfd492, 90 },
    { 0x16afd492, 61 },
    { 0xd10fd494, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55fd49d, 90 },
    { 0xa1cfd49e, 95 },
    { 0xe3dfd49e, 33 },
    { 0x5effd4a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd4a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fd4ad, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115fd4b3, 52 },
    { 0xd8bfd4b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dfd4ba, 90 },
    { 0xdfafd4bb, 67 },
    { 0x99dfd4bc, 23 },
    { 0x1dcfd4bb, 95 },
    { 0xcaffd4be, 33 },
    { 0x787fd4bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0fd4c5, 61 },
    { 0x313fd4c5, 98 },
    { 0, 0 },
    { 0x102fd4c8, 81 },
    { 0xe44fd4c9, 91 },
    { 0x9e1fd4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183fd4cf, 23 },
    { 0xc23fd4d0, 95 },
    { 0x18efd4d0, 100 },
    { 0x7c6fd4d2, 76 },
    { 0x871fd4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22afd4d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807fd4df, 90 },
    { 0x587fd4e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cfd4e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9fd4ea, 93 },
    { 0x451fd4eb, 61 },
    { 0, 0 },
    { 0x2e4fd4ed, 98 },
    { 0, 0 },
    { 0xc26fd4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fd4f7, 23 },
    { 0x16dfd4f8, 85 },
    { 0xfa3fd4f9, 34 },
    { 0x793fd4fa, 90 },
    { 0xd0cfd4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fd504, 95 },
    { 0x7cbfd504, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfd50c, 69 },
    { 0x99bfd50d, 90 },
    { 0x7fcfd50e, 33 },
    { 0, 0 },
    { 0x216fd510, 78 },
    { 0xe10fd510, 53 },
    { 0x519fd512, 61 },
    { 0xdb7fd511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9fd51b, 90 },
    { 0, 0 },
    { 0x9affd51d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606fd520, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cfd523, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757fd527, 61 },
    { 0xb10fd527, 33 },
    { 0x178fd529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aefd535, 23 },
    { 0x9f9fd535, 61 },
    { 0, 0 },
    { 0xa67fd538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42fd53b, 78 },
    { 0x598fd53b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9fd542, 33 },
    { 0x59fd542, 34 },
    { 0, 0 },
    { 0xacafd545, 43 },
    { 0x477fd546, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdfd549, 61 },
    { 0xe2dfd549, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43afd54f, 61 },
    { 0x3f8fd550, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffd554, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2befd557, 90 },
    { 0, 0 },
    { 0xeaafd559, 98 },
    { 0x8ffd55a, 98 },
    { 0xc93fd55b, 98 },
    { 0x524fd55a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87fd566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322fd56b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7afd572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9fd579, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fd57f, 93 },
    { 0, 0 },
    { 0xd16fd581, 33 },
    { 0x7ccfd582, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dfd586, 80 },
    { 0xeb3fd587, 90 },
    { 0x375fd588, 23 },
    { 0x6f5fd589, 90 },
    { 0x308fd58a, 55 },
    { 0xda9fd58b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520fd596, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72fd599, 78 },
    { 0xa87fd599, 88 },
    { 0x681fd599, 33 },
    { 0x8c8fd599, 61 },
    { 0x203fd59d, 52 },
    { 0x6b4fd59c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cfd5a2, 66 },
    { 0x477fd5a2, 34 },
    { 0xbfd5a4, 93 },
    { 0xd4dfd5a5, 76 },
    { 0x78efd5a6, 23 },
    { 0x602fd5a7, 34 },
    { 0xdd4fd5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28fd5ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdfd5ae, 33 },
    { 0x4d4fd5af, 95 },
    { 0, 0 },
    { 0x69afd5b1, 55 },
    { 0x8d3fd5b2, 74 },
    { 0, 0 },
    { 0x12afd5b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7afd5b7, 95 },
    { 0, 0 },
    { 0x5dbfd5b9, 95 },
    { 0, 0 },
    { 0xd1cfd5bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x570fd5be, 90 },
    { 0x793fd5be, 99 },
    { 0x7befd5bf, 55 },
    { 0x746fd5c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9fd5c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x495fd5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fd5cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x901fd5d0, 78 },
    { 0xf3ffd5d1, 43 },
    { 0, 0 },
    { 0xbf0fd5d3, 33 },
    { 0x199fd5d4, 76 },
    { 0x85fd5d4, 23 },
    { 0, 0 },
    { 0xb0bfd5d7, 81 },
    { 0xa14fd5d8, 33 },
    { 0, 0 },
    { 0xb05fd5da, 61 },
    { 0x4b9fd5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703fd5de, 87 },
    { 0xf02fd5df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424fd5e6, 95 },
    { 0x77fd5e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445fd5eb, 95 },
    { 0xefcfd5eb, 46 },
    { 0, 0 },
    { 0xbf8fd5ee, 46 },
    { 0xf59fd5ef, 34 },
    { 0xdf4fd5f0, 84 },
    { 0xee4fd5ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfd5f5, 33 },
    { 0, 0 },
    { 0x2eefd5f7, 78 },
    { 0x9b3fd5f8, 90 },
    { 0xbf2fd5f8, 55 },
    { 0xa18fd5fa, 78 },
    { 0xfa3fd5f8, 90 },
    { 0x6ccfd5fc, 68 },
    { 0x7cbfd5fd, 33 },
    { 0xf00fd5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92fd605, 91 },
    { 0xeebfd605, 90 },
    { 0x83ffd607, 90 },
    { 0, 0 },
    { 0x3fffd609, 33 },
    { 0x49bfd60a, 46 },
    { 0x146fd60a, 98 },
    { 0x63bfd609, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15efd610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2afd613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898fd617, 43 },
    { 0x150fd618, 98 },
    { 0x1dafd618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131fd61f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0efd623, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fd62b, 100 },
    { 0xbc5fd62c, 81 },
    { 0xecefd62d, 33 },
    { 0, 0 },
    { 0x334fd62f, 34 },
    { 0x713fd630, 95 },
    { 0xe3cfd630, 95 },
    { 0x13cfd632, 87 },
    { 0, 0 },
    { 0xceefd634, 46 },
    { 0xa3efd635, 61 },
    { 0, 0 },
    { 0x764fd637, 28 },
    { 0x145fd637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dfd640, 61 },
    { 0xc4afd641, 95 },
    { 0x859fd640, 97 },
    { 0xac9fd642, 99 },
    { 0, 0 },
    { 0xf7ffd645, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fffd64d, 33 },
    { 0x85afd64e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234fd655, 93 },
    { 0x17fd656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fd65d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dfd662, 33 },
    { 0xc3bfd663, 33 },
    { 0, 0 },
    { 0xa50fd665, 46 },
    { 0, 0 },
    { 0xdbffd667, 90 },
    { 0, 0 },
    { 0x28fd669, 98 },
    { 0, 0 },
    { 0x1dffd66b, 98 },
    { 0x483fd66b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbfd673, 84 },
    { 0x221fd674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bfd67a, 90 },
    { 0xe9bfd67b, 33 },
    { 0x29dfd67a, 98 },
    { 0x9a5fd67b, 33 },
    { 0, 0 },
    { 0x707fd67f, 55 },
    { 0x298fd67f, 33 },
    { 0x31ffd67f, 33 },
    { 0x372fd682, 17 },
    { 0, 0 },
    { 0x435fd684, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbfd687, 68 },
    { 0, 0 },
    { 0x17fd689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7fd68e, 34 },
    { 0xd1bfd68f, 55 },
    { 0, 0 },
    { 0x781fd691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32fd69c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fd69f, 76 },
    { 0xa2bfd6a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fd6a6, 61 },
    { 0x12cfd6a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fd6ab, 87 },
    { 0x225fd6ab, 33 },
    { 0x734fd6ab, 100 },
    { 0x8e9fd6ac, 33 },
    { 0x27cfd6af, 85 },
    { 0x256fd6ab, 33 },
    { 0x232fd6b0, 52 },
    { 0xa9dfd6ad, 34 },
    { 0, 0 },
    { 0x8c6fd6b4, 61 },
    { 0x51bfd6b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x582fd6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579fd6bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3fd6c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1fd6c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defd6cc, 33 },
    { 0x35afd6cc, 95 },
    { 0, 0 },
    { 0x2a0fd6cf, 43 },
    { 0, 0 },
    { 0x3f2fd6d1, 55 },
    { 0xf43fd6d1, 23 },
    { 0x27fd6d3, 33 },
    { 0, 0 },
    { 0x1fefd6d5, 43 },
    { 0x81cfd6d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78fd6d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6fd6e0, 67 },
    { 0xc93fd6e1, 85 },
    { 0xa73fd6e2, 91 },
    { 0x411fd6e3, 33 },
    { 0x119fd6e4, 91 },
    { 0x2c0fd6e2, 55 },
    { 0xd07fd6e1, 65 },
    { 0, 0 },
    { 0x30fd6e8, 33 },
    { 0, 0 },
    { 0x1b7fd6ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x303fd6ed, 85 },
    { 0x576fd6ee, 90 },
    { 0xf9dfd6ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2fd6f2, 95 },
    { 0x458fd6f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3fd6f8, 95 },
    { 0x386fd6f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afd6fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908fd701, 23 },
    { 0x874fd702, 93 },
    { 0xcbcfd702, 93 },
    { 0xe44fd704, 93 },
    { 0x702fd704, 33 },
    { 0x5adfd702, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdfd70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422fd713, 23 },
    { 0, 0 },
    { 0x9bdfd715, 33 },
    { 0x71cfd715, 91 },
    { 0xed7fd717, 33 },
    { 0x7a2fd718, 66 },
    { 0x2d0fd719, 90 },
    { 0xe66fd71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfd71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9fd725, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8fd728, 43 },
    { 0xa8ffd729, 55 },
    { 0xaf4fd728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996fd732, 52 },
    { 0xe85fd732, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27afd737, 93 },
    { 0xc60fd737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x383fd73b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6afd741, 78 },
    { 0x66efd742, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8befd746, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dfd749, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963fd750, 55 },
    { 0x539fd750, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7fd757, 99 },
    { 0x407fd758, 90 },
    { 0, 0 },
    { 0x71fd75a, 34 },
    { 0x151fd75a, 98 },
    { 0, 0 },
    { 0x653fd75d, 34 },
    { 0, 0 },
    { 0xbe3fd75f, 95 },
    { 0x970fd75f, 33 },
    { 0x459fd760, 23 },
    { 0xeb3fd75f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dfd767, 61 },
    { 0x79cfd767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16afd76b, 85 },
    { 0x1d2fd76c, 100 },
    { 0x854fd76d, 76 },
    { 0xfbafd76d, 90 },
    { 0xfa3fd76d, 98 },
    { 0x644fd76b, 23 },
    { 0, 0 },
    { 0x815fd772, 91 },
    { 0xbdffd773, 61 },
    { 0xd38fd774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6fd77a, 61 },
    { 0x9e6fd77b, 23 },
    { 0xa49fd77b, 61 },
    { 0, 0 },
    { 0x140fd77e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5fd783, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddfd787, 56 },
    { 0xf28fd788, 61 },
    { 0, 0 },
    { 0xc1bfd78a, 66 },
    { 0, 0 },
    { 0x27afd78c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520fd791, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfd795, 76 },
    { 0x844fd796, 66 },
    { 0xe80fd797, 65 },
    { 0x7f9fd798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd79e, 95 },
    { 0, 0 },
    { 0x444fd7a0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36fd7a5, 61 },
    { 0, 0 },
    { 0xf14fd7a7, 33 },
    { 0xcf1fd7a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fd7ab, 61 },
    { 0x2a7fd7ac, 33 },
    { 0x4a3fd7ad, 90 },
    { 0x397fd7ab, 55 },
    { 0x31afd7ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00fd7b2, 69 },
    { 0, 0 },
    { 0xd6cfd7b4, 98 },
    { 0x3ecfd7b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6efd7ba, 17 },
    { 0, 0 },
    { 0xf83fd7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfd7c3, 88 },
    { 0xdb7fd7c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5fd7c8, 78 },
    { 0, 0 },
    { 0x20dfd7ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391fd7cd, 90 },
    { 0xee2fd7cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5fd7d5, 98 },
    { 0xf50fd7d6, 61 },
    { 0xaedfd7d5, 66 },
    { 0x2d5fd7d8, 34 },
    { 0x1cfd7d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfd7e0, 95 },
    { 0xc1afd7e1, 98 },
    { 0xab3fd7e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbfd7e8, 75 },
    { 0x2a0fd7e9, 75 },
    { 0, 0 },
    { 0x6c9fd7eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefd7ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ffd7f6, 43 },
    { 0, 0 },
    { 0x47afd7f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63fd802, 95 },
    { 0x7e8fd803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8fd808, 23 },
    { 0x953fd809, 75 },
    { 0x108fd80a, 43 },
    { 0x566fd80b, 24 },
    { 0x8c5fd809, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efd80f, 77 },
    { 0x4dafd810, 76 },
    { 0xa8efd810, 43 },
    { 0x2a3fd810, 76 },
    { 0, 0 },
    { 0x9d6fd814, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322fd818, 43 },
    { 0xb6afd819, 23 },
    { 0xfefd81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28fd81d, 61 },
    { 0, 0 },
    { 0x4fffd81f, 55 },
    { 0, 0 },
    { 0xf5cfd821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86fd825, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fd828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cffd82b, 85 },
    { 0xfedfd82b, 76 },
    { 0x219fd82d, 90 },
    { 0, 0 },
    { 0x2bbfd82f, 76 },
    { 0, 0 },
    { 0x161fd831, 23 },
    { 0, 0 },
    { 0xab8fd833, 23 },
    { 0, 0 },
    { 0xa4bfd835, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fd83b, 90 },
    { 0x96fd83c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefd83f, 52 },
    { 0, 0 },
    { 0x19cfd841, 78 },
    { 0x75ffd842, 23 },
    { 0xb99fd843, 34 },
    { 0x796fd844, 33 },
    { 0, 0 },
    { 0xf2dfd846, 76 },
    { 0, 0 },
    { 0x7fcfd848, 99 },
    { 0x315fd849, 52 },
    { 0x3bfd849, 23 },
    { 0x33afd84b, 66 },
    { 0xb72fd84c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fd852, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd856, 33 },
    { 0, 0 },
    { 0xb8afd858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fd85d, 98 },
    { 0xd74fd85d, 95 },
    { 0, 0 },
    { 0x75cfd860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfd863, 43 },
    { 0xbbcfd864, 90 },
    { 0x32ffd865, 91 },
    { 0x83ffd866, 84 },
    { 0x4aafd864, 76 },
    { 0x20bfd868, 98 },
    { 0, 0 },
    { 0xa4cfd86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5fd86e, 33 },
    { 0x830fd86f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfd875, 77 },
    { 0x92bfd876, 95 },
    { 0xd20fd876, 61 },
    { 0xdeefd878, 76 },
    { 0x3a3fd879, 95 },
    { 0, 0 },
    { 0x889fd87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fd87f, 98 },
    { 0xb22fd880, 92 },
    { 0x728fd880, 55 },
    { 0x297fd882, 90 },
    { 0x596fd882, 95 },
    { 0xe76fd884, 43 },
    { 0xc0bfd882, 61 },
    { 0x27efd886, 52 },
    { 0x316fd87f, 98 },
    { 0x818fd888, 34 },
    { 0x330fd885, 61 },
    { 0, 0 },
    { 0x969fd88b, 43 },
    { 0xa11fd88c, 33 },
    { 0x5b1fd88d, 98 },
    { 0xe79fd88d, 61 },
    { 0xecafd88b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bfd893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaafd898, 23 },
    { 0x51afd898, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7fd8a0, 90 },
    { 0x2abfd8a1, 99 },
    { 0xd72fd8a2, 53 },
    { 0x36ffd8a3, 43 },
    { 0x1b8fd8a4, 90 },
    { 0xccefd8a1, 23 },
    { 0x215fd8a6, 95 },
    { 0xa49fd8a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bfd8ac, 33 },
    { 0xbfafd8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7fd8b7, 99 },
    { 0, 0 },
    { 0x583fd8b9, 69 },
    { 0, 0 },
    { 0x65afd8bb, 23 },
    { 0x6c5fd8bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9afd8bf, 98 },
    { 0, 0 },
    { 0x15ffd8c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfefd8c8, 38 },
    { 0, 0 },
    { 0x615fd8ca, 66 },
    { 0xa6cfd8cb, 90 },
    { 0x1e6fd8cc, 76 },
    { 0x7fbfd8cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6fd8d4, 55 },
    { 0xefefd8d5, 98 },
    { 0x33cfd8d6, 69 },
    { 0x73fd8d4, 76 },
    { 0xd02fd8d8, 55 },
    { 0, 0 },
    { 0xf9afd8da, 90 },
    { 0, 0 },
    { 0x731fd8dc, 64 },
    { 0, 0 },
    { 0xcf2fd8de, 81 },
    { 0x3f9fd8de, 46 },
    { 0, 0 },
    { 0x6fd8e1, 90 },
    { 0x439fd8e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2fd8e7, 99 },
    { 0, 0 },
    { 0xeb2fd8e9, 17 },
    { 0, 0 },
    { 0x94bfd8eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eafd8f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fd8fa, 61 },
    { 0xe84fd8fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0fd901, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ffd906, 55 },
    { 0, 0 },
    { 0xf8cfd908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x654fd90b, 55 },
    { 0x201fd90b, 33 },
    { 0x4c3fd90d, 78 },
    { 0x509fd90b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefd911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fd914, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fd918, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24fd922, 67 },
    { 0x497fd923, 98 },
    { 0x3befd924, 95 },
    { 0xed4fd925, 23 },
    { 0, 0 },
    { 0xef6fd927, 33 },
    { 0, 0 },
    { 0x26ffd929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555fd92e, 76 },
    { 0x34cfd92f, 98 },
    { 0xf15fd930, 61 },
    { 0x944fd931, 55 },
    { 0, 0 },
    { 0xf60fd933, 46 },
    { 0xdc6fd934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd938, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ffd93c, 68 },
    { 0, 0 },
    { 0x32bfd93e, 55 },
    { 0xb6afd93f, 90 },
    { 0x5c7fd93e, 95 },
    { 0xc54fd941, 95 },
    { 0x5dcfd93e, 76 },
    { 0x353fd940, 98 },
    { 0xe20fd93f, 75 },
    { 0x51ffd945, 91 },
    { 0xef2fd945, 66 },
    { 0, 0 },
    { 0x45afd948, 34 },
    { 0x8abfd948, 33 },
    { 0x4f3fd948, 61 },
    { 0, 0 },
    { 0x859fd94c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fd94f, 42 },
    { 0x603fd950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63fd955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x530fd958, 55 },
    { 0x305fd959, 93 },
    { 0xd3dfd95a, 46 },
    { 0x75efd95b, 34 },
    { 0x3dfd95c, 33 },
    { 0x331fd95d, 95 },
    { 0x107fd95e, 33 },
    { 0x3e1fd95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5fd962, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98fd969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137fd96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518fd973, 98 },
    { 0x37cfd973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bfd978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243fd97b, 100 },
    { 0, 0 },
    { 0xeabfd97d, 90 },
    { 0, 0 },
    { 0xce8fd97f, 90 },
    { 0x3f8fd980, 87 },
    { 0xa66fd980, 91 },
    { 0, 0 },
    { 0x577fd983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18fd986, 61 },
    { 0x7a3fd987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626fd98d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2fd992, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffd995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6fd999, 33 },
    { 0, 0 },
    { 0x743fd99b, 55 },
    { 0x3a6fd99c, 98 },
    { 0xc10fd99d, 98 },
    { 0x221fd99e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1fd9a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x422fd9a5, 87 },
    { 0x501fd9a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfd9a9, 55 },
    { 0xdd9fd9aa, 99 },
    { 0x4c7fd9aa, 90 },
    { 0x694fd9ab, 99 },
    { 0x463fd9ab, 85 },
    { 0xfeefd9ae, 46 },
    { 0xc48fd9af, 90 },
    { 0xea7fd9b0, 76 },
    { 0x226fd9b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82afd9b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348fd9ba, 76 },
    { 0xae6fd9bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4fd9be, 87 },
    { 0x5f5fd9be, 76 },
    { 0x66cfd9be, 76 },
    { 0x3b2fd9c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510fd9c5, 98 },
    { 0x528fd9c5, 69 },
    { 0, 0 },
    { 0x9cefd9c8, 95 },
    { 0x90fd9c9, 94 },
    { 0xfb8fd9ca, 43 },
    { 0, 0 },
    { 0x1afd9cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137fd9d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afd9d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9fd9da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cfd9dd, 61 },
    { 0xc01fd9de, 34 },
    { 0xc4cfd9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93fd9e3, 76 },
    { 0xd62fd9e3, 61 },
    { 0, 0 },
    { 0xc09fd9e6, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3afd9ea, 98 },
    { 0x873fd9eb, 90 },
    { 0, 0 },
    { 0x620fd9ed, 90 },
    { 0x5cffd9ee, 93 },
    { 0x104fd9ee, 90 },
    { 0xcc0fd9f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fd9f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3fd9fa, 66 },
    { 0, 0 },
    { 0x667fd9fc, 90 },
    { 0xbdbfd9fc, 61 },
    { 0, 0 },
    { 0x6fffd9ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5fda02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fda06, 76 },
    { 0xe98fda07, 93 },
    { 0xe62fda06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4fda0f, 40 },
    { 0x185fda10, 61 },
    { 0xeb8fda11, 98 },
    { 0xbfdfda11, 95 },
    { 0x812fda13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428fda17, 19 },
    { 0, 0 },
    { 0x3f6fda19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31afda1d, 85 },
    { 0xe85fda1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacefda21, 43 },
    { 0, 0 },
    { 0x8bdfda23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14fda29, 95 },
    { 0xc0bfda29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fda30, 23 },
    { 0x21fda30, 33 },
    { 0x3f8fda31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326fda36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fda39, 64 },
    { 0, 0 },
    { 0xe30fda3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fda48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68fda4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978fda4f, 98 },
    { 0x56ffda50, 87 },
    { 0xb8afda51, 81 },
    { 0xe0efda52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240fda58, 90 },
    { 0x4cdfda58, 23 },
    { 0, 0 },
    { 0x63dfda5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23fda5e, 43 },
    { 0, 0 },
    { 0x80ffda60, 43 },
    { 0, 0 },
    { 0x560fda62, 23 },
    { 0, 0 },
    { 0x4fcfda64, 33 },
    { 0xc0afda65, 78 },
    { 0x783fda66, 55 },
    { 0xbe1fda67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538fda6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dfda78, 80 },
    { 0, 0 },
    { 0x8dffda7a, 90 },
    { 0x63ffda7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fda84, 90 },
    { 0xe79fda85, 78 },
    { 0, 0 },
    { 0x231fda87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fda8b, 46 },
    { 0x3fda8c, 95 },
    { 0x2aafda8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fda90, 98 },
    { 0, 0 },
    { 0xd20fda92, 23 },
    { 0x907fda92, 23 },
    { 0, 0 },
    { 0x8c2fda95, 90 },
    { 0x672fda96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadefda99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dfda9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffdaa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992fdaa5, 44 },
    { 0x2d5fdaa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7fdaaa, 61 },
    { 0x914fdaaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9fdab0, 98 },
    { 0xb51fdab1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6fdab4, 23 },
    { 0, 0 },
    { 0xf1efdab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dfdaba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20fdabd, 61 },
    { 0x815fdabe, 76 },
    { 0x804fdabf, 61 },
    { 0xb16fdac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdac4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebfdac7, 98 },
    { 0, 0 },
    { 0x7a7fdac9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95fdad0, 42 },
    { 0xcfafdad1, 23 },
    { 0xb5fdad2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fdad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9fdadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325fdae3, 43 },
    { 0xb11fdae3, 76 },
    { 0x51afdae5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3fdaef, 98 },
    { 0xdc0fdaf0, 90 },
    { 0xbf2fdaf1, 95 },
    { 0, 0 },
    { 0x28dfdaf3, 78 },
    { 0xd26fdaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceefdaf9, 88 },
    { 0x6cefdafa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aefdafd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3efdb00, 76 },
    { 0, 0 },
    { 0x59cfdb02, 98 },
    { 0, 0 },
    { 0x6dcfdb04, 76 },
    { 0, 0 },
    { 0x502fdb06, 46 },
    { 0xef7fdb07, 61 },
    { 0xd35fdb08, 88 },
    { 0x6ddfdb09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73fdb0d, 55 },
    { 0xd1efdb0e, 91 },
    { 0x1f2fdb0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aafdb12, 43 },
    { 0xb19fdb13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fdb16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cfdb19, 33 },
    { 0xc3fdb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fdb28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4fdb2d, 98 },
    { 0x973fdb2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80fdb31, 98 },
    { 0, 0 },
    { 0x8e8fdb33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbfdb37, 93 },
    { 0x30cfdb37, 98 },
    { 0x5f0fdb39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edfdb3f, 90 },
    { 0x101fdb40, 23 },
    { 0xb7ffdb41, 61 },
    { 0xbecfdb40, 61 },
    { 0xcabfdb43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1fdb48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3fdb4e, 95 },
    { 0x71afdb4f, 90 },
    { 0xc7dfdb4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dfdb56, 33 },
    { 0x1b9fdb57, 90 },
    { 0x880fdb58, 55 },
    { 0x8cfdb59, 33 },
    { 0, 0 },
    { 0x1fefdb5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50afdb64, 76 },
    { 0xafafdb65, 95 },
    { 0x962fdb66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efdb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fdb6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x417fdb70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x836fdb73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5fdb77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22fdb7a, 76 },
    { 0xa20fdb7b, 33 },
    { 0, 0 },
    { 0xc8efdb7d, 93 },
    { 0x84efdb7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bfdb81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158fdb87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x233fdb8a, 83 },
    { 0xc59fdb8b, 91 },
    { 0xaaefdb8b, 95 },
    { 0x7fffdb8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8fdb93, 98 },
    { 0x41efdb94, 75 },
    { 0, 0 },
    { 0xe1efdb96, 33 },
    { 0, 0 },
    { 0xbefdb98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x414fdb9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x683fdb9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fdba3, 87 },
    { 0xfaafdba4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ffdba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40fdbaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864fdbb5, 43 },
    { 0x4bbfdbb6, 90 },
    { 0x88cfdbb6, 76 },
    { 0, 0 },
    { 0x4fdfdbb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84fdbbd, 76 },
    { 0xf88fdbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffdbc1, 61 },
    { 0, 0 },
    { 0x87dfdbc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bfdbc9, 33 },
    { 0xb2afdbca, 23 },
    { 0x700fdbc9, 61 },
    { 0x20efdbc9, 98 },
    { 0x21afdbcc, 100 },
    { 0x39ffdbce, 33 },
    { 0xd7bfdbcf, 61 },
    { 0xbb9fdbcf, 98 },
    { 0xa4cfdbd1, 98 },
    { 0xd95fdbce, 33 },
    { 0, 0 },
    { 0x5f7fdbd4, 87 },
    { 0x99fdbd5, 90 },
    { 0, 0 },
    { 0x67afdbd7, 93 },
    { 0x5fefdbd8, 96 },
    { 0xb8cfdbd9, 90 },
    { 0x454fdbd8, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5fdbe1, 33 },
    { 0x79efdbe2, 99 },
    { 0xf57fdbe3, 23 },
    { 0x262fdbe4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6fdbed, 95 },
    { 0xa5efdbee, 98 },
    { 0x973fdbef, 39 },
    { 0x172fdbf0, 61 },
    { 0xeb5fdbf1, 95 },
    { 0x7a2fdbf0, 17 },
    { 0xaf9fdbf3, 75 },
    { 0x1e4fdbee, 46 },
    { 0xae1fdbf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276fdbfa, 84 },
    { 0xab2fdbfb, 98 },
    { 0xdd1fdbfc, 95 },
    { 0x4bffdbfc, 15 },
    { 0x9c3fdbfc, 43 },
    { 0xc74fdbff, 78 },
    { 0x937fdbff, 61 },
    { 0x453fdbff, 23 },
    { 0x14efdc00, 99 },
    { 0, 0 },
    { 0x727fdc04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cffdc09, 85 },
    { 0x395fdc0a, 77 },
    { 0xea3fdc0b, 90 },
    { 0x487fdc0c, 34 },
    { 0x67bfdc09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dfdc11, 95 },
    { 0x322fdc12, 84 },
    { 0xc9cfdc13, 69 },
    { 0x94cfdc14, 93 },
    { 0xba6fdc14, 68 },
    { 0xa18fdc13, 33 },
    { 0xf1bfdc11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeffdc1d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676fdc22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5fdc27, 98 },
    { 0x303fdc28, 98 },
    { 0xa4ffdc29, 99 },
    { 0x3aefdc28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8fdc34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fdc37, 75 },
    { 0x701fdc37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bfdc3b, 46 },
    { 0x7d2fdc3c, 44 },
    { 0xb31fdc3d, 82 },
    { 0xe68fdc3e, 78 },
    { 0x5a3fdc3f, 90 },
    { 0x418fdc40, 61 },
    { 0x3e9fdc41, 99 },
    { 0x849fdc42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fdc45, 98 },
    { 0xcfffdc45, 24 },
    { 0, 0 },
    { 0xd3ffdc48, 83 },
    { 0xf5ffdc49, 90 },
    { 0, 0 },
    { 0x165fdc4b, 90 },
    { 0x651fdc4b, 76 },
    { 0, 0 },
    { 0x8c0fdc4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dafdc52, 98 },
    { 0xd39fdc53, 23 },
    { 0xdfcfdc53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cfdc59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdfdc5c, 43 },
    { 0x5defdc5d, 95 },
    { 0, 0 },
    { 0xc4fdc5f, 95 },
    { 0, 0 },
    { 0xebcfdc61, 95 },
    { 0x30bfdc62, 61 },
    { 0xdf5fdc62, 23 },
    { 0xe85fdc64, 91 },
    { 0xb59fdc65, 95 },
    { 0x34bfdc65, 85 },
    { 0x335fdc67, 95 },
    { 0xa45fdc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fdc6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502fdc71, 99 },
    { 0xe29fdc71, 95 },
    { 0x1d6fdc71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fdc79, 88 },
    { 0xf2cfdc7a, 90 },
    { 0x200fdc79, 91 },
    { 0xb40fdc7a, 23 },
    { 0, 0 },
    { 0x6ebfdc7e, 90 },
    { 0x2dafdc7f, 52 },
    { 0xd52fdc80, 98 },
    { 0xb68fdc81, 90 },
    { 0x613fdc80, 93 },
    { 0x358fdc83, 78 },
    { 0xc6efdc80, 33 },
    { 0x391fdc85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739fdc89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05fdc8e, 93 },
    { 0xd91fdc8f, 76 },
    { 0x17ffdc90, 76 },
    { 0x845fdc91, 98 },
    { 0xd29fdc92, 67 },
    { 0xf55fdc90, 34 },
    { 0xfdefdc91, 46 },
    { 0x270fdc95, 43 },
    { 0xf35fdc96, 76 },
    { 0x699fdc97, 90 },
    { 0x79efdc98, 88 },
    { 0x335fdc99, 68 },
    { 0x20dfdc96, 66 },
    { 0xeefdc97, 90 },
    { 0x879fdc9c, 33 },
    { 0x296fdc9d, 34 },
    { 0, 0 },
    { 0x87ffdc9f, 52 },
    { 0, 0 },
    { 0xe03fdca1, 77 },
    { 0x94fdca2, 64 },
    { 0xa36fdca2, 90 },
    { 0xb9cfdca1, 76 },
    { 0, 0 },
    { 0xb95fdca6, 66 },
    { 0x411fdca7, 34 },
    { 0x3b6fdca6, 90 },
    { 0xc79fdca9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00fdcae, 52 },
    { 0xc37fdcae, 43 },
    { 0, 0 },
    { 0x1effdcb1, 23 },
    { 0, 0 },
    { 0xcecfdcb3, 78 },
    { 0, 0 },
    { 0x6ecfdcb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41fdcb8, 98 },
    { 0x1d4fdcb9, 55 },
    { 0x45bfdcb9, 43 },
    { 0xcf8fdcb8, 23 },
    { 0x934fdcbc, 93 },
    { 0x136fdcbd, 99 },
    { 0xa5bfdcbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddfdcc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cfdcc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fdccc, 53 },
    { 0x72dfdccd, 98 },
    { 0xf7fdcce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98fdcd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96fdcdb, 90 },
    { 0, 0 },
    { 0x423fdcdd, 46 },
    { 0x23fdcde, 99 },
    { 0x391fdcdf, 46 },
    { 0xd37fdcdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811fdce4, 55 },
    { 0x27bfdce5, 90 },
    { 0xe85fdce6, 33 },
    { 0, 0 },
    { 0xde7fdce8, 55 },
    { 0xd95fdce9, 91 },
    { 0, 0 },
    { 0xb9cfdceb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0fdcf2, 98 },
    { 0x7ebfdcf3, 90 },
    { 0x12cfdcf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfdcf9, 84 },
    { 0, 0 },
    { 0x8fdfdcfb, 23 },
    { 0xbcefdcfc, 90 },
    { 0xb1afdcfd, 61 },
    { 0xfa2fdcfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddefdd01, 67 },
    { 0xc43fdd01, 95 },
    { 0, 0 },
    { 0xb28fdd04, 61 },
    { 0x1fefdd05, 93 },
    { 0xf1cfdd05, 4 },
    { 0x74efdd07, 61 },
    { 0x618fdd05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3fdd0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41fdd0f, 34 },
    { 0x928fdd10, 98 },
    { 0x533fdd11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9fdd14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01fdd20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514fdd27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfefdd2b, 95 },
    { 0x75dfdd2c, 33 },
    { 0x4dfdd2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eafdd33, 33 },
    { 0, 0 },
    { 0xf4ffdd35, 76 },
    { 0x66bfdd35, 33 },
    { 0x852fdd37, 33 },
    { 0x789fdd37, 90 },
    { 0xbbafdd39, 95 },
    { 0, 0 },
    { 0xf8efdd3b, 92 },
    { 0x35dfdd3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fdd47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647fdd4b, 95 },
    { 0, 0 },
    { 0x841fdd4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaffdd54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cffdd5a, 87 },
    { 0x2c3fdd5b, 91 },
    { 0xe0cfdd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3fdd5f, 46 },
    { 0, 0 },
    { 0xae8fdd61, 17 },
    { 0x4fffdd62, 93 },
    { 0x4e2fdd63, 85 },
    { 0xd7ffdd64, 61 },
    { 0x5cbfdd64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fdd6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ffdd6d, 95 },
    { 0xa33fdd6d, 43 },
    { 0x610fdd6d, 67 },
    { 0x73cfdd70, 46 },
    { 0xd98fdd71, 55 },
    { 0xd73fdd70, 33 },
    { 0x11cfdd6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276fdd7d, 33 },
    { 0, 0 },
    { 0x787fdd7f, 95 },
    { 0xf7ffdd7f, 23 },
    { 0xd56fdd81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabfdd84, 93 },
    { 0xd89fdd85, 93 },
    { 0, 0 },
    { 0x707fdd87, 61 },
    { 0x618fdd88, 46 },
    { 0x3c2fdd89, 84 },
    { 0xe99fdd8a, 56 },
    { 0x4a4fdd8b, 95 },
    { 0, 0 },
    { 0x18efdd8d, 78 },
    { 0xbb9fdd8e, 67 },
    { 0x57dfdd8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6affdd93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afdd9b, 99 },
    { 0x70fdd9c, 95 },
    { 0x2c8fdd9c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdda2, 61 },
    { 0xe32fdda3, 61 },
    { 0xcacfdda4, 98 },
    { 0, 0 },
    { 0xe9efdda6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52afddab, 61 },
    { 0xf5efddac, 90 },
    { 0xdbffddad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cfddb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x186fddb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4efddbd, 78 },
    { 0x4e8fddbd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5fddc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x954fddc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffddc9, 90 },
    { 0x5ecfddca, 65 },
    { 0x41bfddca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8fddd0, 95 },
    { 0xc45fddd0, 98 },
    { 0x5c6fddd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14fddd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1fddd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bfddde, 55 },
    { 0x690fdddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fdde2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fdded, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65fddf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cefddf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fddfb, 34 },
    { 0x1c0fddfc, 90 },
    { 0x82dfddfc, 34 },
    { 0xe8afddfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77fde07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711fde0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2fde15, 76 },
    { 0, 0 },
    { 0xd17fde17, 52 },
    { 0xf2dfde17, 46 },
    { 0x6fafde19, 12 },
    { 0x6e1fde1a, 61 },
    { 0x991fde1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962fde24, 76 },
    { 0x214fde24, 76 },
    { 0x5defde26, 43 },
    { 0xe92fde27, 61 },
    { 0x4dcfde28, 90 },
    { 0x82dfde28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9befde31, 55 },
    { 0xce5fde32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fde35, 34 },
    { 0xcfefde36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611fde3a, 68 },
    { 0xf8ffde3a, 100 },
    { 0x662fde3c, 34 },
    { 0x1d6fde3d, 33 },
    { 0xe12fde3d, 85 },
    { 0xcf3fde3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x213fde42, 95 },
    { 0, 0 },
    { 0x6c4fde44, 90 },
    { 0, 0 },
    { 0x881fde46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18fde4c, 75 },
    { 0x9cfde4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9fde50, 23 },
    { 0, 0 },
    { 0x4f8fde52, 98 },
    { 0x2e7fde53, 98 },
    { 0xd0efde54, 33 },
    { 0x103fde52, 84 },
    { 0, 0 },
    { 0x3d2fde57, 90 },
    { 0x6f3fde58, 98 },
    { 0, 0 },
    { 0x811fde5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12fde5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15efde63, 77 },
    { 0x8f8fde63, 95 },
    { 0, 0 },
    { 0x535fde66, 61 },
    { 0x8e1fde67, 43 },
    { 0, 0 },
    { 0x27dfde69, 43 },
    { 0x8befde69, 83 },
    { 0x75cfde6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1fde6e, 46 },
    { 0x823fde6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199fde73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8fde78, 23 },
    { 0xce3fde79, 90 },
    { 0, 0 },
    { 0x2bafde7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efde7e, 34 },
    { 0, 0 },
    { 0xf7bfde80, 33 },
    { 0xb15fde81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60fde84, 46 },
    { 0, 0 },
    { 0x68efde86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efde8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54efde93, 95 },
    { 0x7cafde93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1fde98, 90 },
    { 0xb65fde99, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x251fde9c, 95 },
    { 0, 0 },
    { 0x12efde9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3afdeab, 43 },
    { 0x151fdeac, 23 },
    { 0x287fdeac, 33 },
    { 0, 0 },
    { 0xbc5fdeaf, 95 },
    { 0, 0 },
    { 0x21bfdeb1, 95 },
    { 0x2cdfdeb1, 99 },
    { 0, 0 },
    { 0x507fdeb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91fdeb8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06fdebc, 90 },
    { 0, 0 },
    { 0x4fdfdebe, 93 },
    { 0x588fdebf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1fdec5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555fdeca, 91 },
    { 0xdf4fdeca, 90 },
    { 0xe95fdeca, 23 },
    { 0x754fdeca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ffded0, 98 },
    { 0, 0 },
    { 0xa40fded2, 55 },
    { 0x77afded2, 4 },
    { 0xabffded3, 87 },
    { 0xe9afded5, 95 },
    { 0x1fdfded5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cfdedf, 61 },
    { 0x857fdedf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3fdee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafdee7, 98 },
    { 0, 0 },
    { 0x4aafdee9, 90 },
    { 0x74afdee9, 67 },
    { 0, 0 },
    { 0x1aafdeec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112fdef2, 76 },
    { 0x20efdef2, 55 },
    { 0x893fdef4, 92 },
    { 0x68cfdef5, 90 },
    { 0, 0 },
    { 0x6f7fdef7, 55 },
    { 0, 0 },
    { 0x650fdef9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfdefc, 43 },
    { 0xc22fdefd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41afdf02, 33 },
    { 0xb9dfdf02, 90 },
    { 0x94ffdf04, 46 },
    { 0, 0 },
    { 0x1e1fdf06, 78 },
    { 0xa81fdf07, 67 },
    { 0x8adfdf08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55efdf17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9fdf1b, 33 },
    { 0xa22fdf1b, 43 },
    { 0x590fdf1c, 55 },
    { 0x140fdf1e, 33 },
    { 0xb21fdf1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafefdf22, 66 },
    { 0x441fdf23, 34 },
    { 0x9f5fdf22, 33 },
    { 0xecbfdf25, 61 },
    { 0x6c7fdf26, 61 },
    { 0, 0 },
    { 0xeddfdf28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dafdf2b, 95 },
    { 0x38efdf2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fdf33, 43 },
    { 0, 0 },
    { 0x97fdf35, 95 },
    { 0xd29fdf36, 93 },
    { 0, 0 },
    { 0xac7fdf38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fdf3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99fdf40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8fdf44, 43 },
    { 0, 0 },
    { 0xd1dfdf46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdfdf4a, 87 },
    { 0, 0 },
    { 0x824fdf4c, 68 },
    { 0x363fdf4c, 17 },
    { 0, 0 },
    { 0xca2fdf4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cfdf59, 34 },
    { 0, 0 },
    { 0x6bcfdf5b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17fdf5e, 23 },
    { 0x419fdf5e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cfdf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fdf6e, 46 },
    { 0x498fdf6f, 83 },
    { 0x987fdf70, 100 },
    { 0, 0 },
    { 0xf65fdf72, 95 },
    { 0x977fdf73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4fdf77, 23 },
    { 0, 0 },
    { 0xf5dfdf79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4fdf7c, 33 },
    { 0x436fdf7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x702fdf80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x996fdf83, 61 },
    { 0xadfdf83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39fdf87, 90 },
    { 0x9a1fdf88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8fdf93, 53 },
    { 0, 0 },
    { 0xcb1fdf95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ffdf99, 23 },
    { 0x72bfdf9a, 90 },
    { 0x1acfdf9b, 76 },
    { 0x6e7fdf9c, 61 },
    { 0xa19fdf9c, 85 },
    { 0xc2fdf9e, 52 },
    { 0x45fdf99, 33 },
    { 0x9c1fdf9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ffdfa4, 33 },
    { 0x302fdfa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9efdfa9, 46 },
    { 0xaf4fdfa9, 23 },
    { 0xf74fdfab, 76 },
    { 0xf56fdfab, 33 },
    { 0x9c4fdfad, 61 },
    { 0, 0 },
    { 0x617fdfaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02fdfb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfdfb7, 61 },
    { 0xcf2fdfb7, 51 },
    { 0xd03fdfb9, 40 },
    { 0, 0 },
    { 0xf51fdfbb, 100 },
    { 0x167fdfbc, 76 },
    { 0xf78fdfbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acfdfc0, 76 },
    { 0, 0 },
    { 0x8c6fdfc2, 46 },
    { 0x6d1fdfc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acfdfc9, 90 },
    { 0xfaafdfc9, 33 },
    { 0x869fdfc9, 61 },
    { 0xfb2fdfcc, 95 },
    { 0x266fdfcd, 46 },
    { 0xaf2fdfcc, 43 },
    { 0x199fdfca, 33 },
    { 0, 0 },
    { 0xcf6fdfd1, 61 },
    { 0xf0ffdfd2, 82 },
    { 0, 0 },
    { 0x1fdfdfd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cfdfe0, 88 },
    { 0xe9bfdfe0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aefdfe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fdfeb, 90 },
    { 0x4c6fdfec, 61 },
    { 0x6aefdfed, 90 },
    { 0, 0 },
    { 0xecefdfef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46fdff3, 55 },
    { 0, 0 },
    { 0xd57fdff5, 61 },
    { 0x405fdff6, 90 },
    { 0x5fafdff7, 61 },
    { 0x47cfdff8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7fdffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1fe000, 91 },
    { 0x369fe001, 90 },
    { 0x7fbfe002, 23 },
    { 0x80afe003, 78 },
    { 0x87ffe003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1fe012, 55 },
    { 0xc91fe013, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7efe016, 85 },
    { 0x685fe017, 77 },
    { 0x40efe016, 33 },
    { 0x4b0fe019, 43 },
    { 0xe32fe019, 61 },
    { 0, 0 },
    { 0x1eafe01c, 64 },
    { 0, 0 },
    { 0xbf5fe01e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0fe026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ffe029, 34 },
    { 0, 0 },
    { 0xe25fe02b, 76 },
    { 0, 0 },
    { 0x41fe02d, 23 },
    { 0x130fe02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x755fe031, 85 },
    { 0x1a4fe032, 90 },
    { 0, 0 },
    { 0x6cbfe034, 81 },
    { 0x173fe034, 95 },
    { 0, 0 },
    { 0x762fe037, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe03d, 61 },
    { 0xfaefe03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x386fe041, 33 },
    { 0, 0 },
    { 0xe7ffe043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1effe04a, 93 },
    { 0xf43fe04b, 55 },
    { 0x346fe04b, 98 },
    { 0x5b9fe04d, 17 },
    { 0xd06fe04e, 67 },
    { 0, 0 },
    { 0x57dfe050, 61 },
    { 0xf4cfe050, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86fe054, 33 },
    { 0x95ffe055, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4fe059, 78 },
    { 0, 0 },
    { 0xddffe05b, 55 },
    { 0xa7fe05c, 98 },
    { 0xf80fe05c, 61 },
    { 0xf0afe05d, 43 },
    { 0xa17fe05f, 76 },
    { 0x205fe060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5fe065, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64fe068, 90 },
    { 0xe2afe068, 98 },
    { 0xdbfe06a, 88 },
    { 0xde2fe06b, 61 },
    { 0xdfdfe06c, 78 },
    { 0x161fe06d, 90 },
    { 0x49cfe06e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x136fe071, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaffe077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fe07b, 98 },
    { 0xdc7fe07c, 78 },
    { 0x38cfe07d, 90 },
    { 0x58dfe07e, 34 },
    { 0x81fe07b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3fe083, 23 },
    { 0x6dafe084, 87 },
    { 0xc47fe083, 61 },
    { 0x49cfe086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28afe08b, 90 },
    { 0xf4afe08c, 61 },
    { 0, 0 },
    { 0x8c5fe08e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cfe097, 61 },
    { 0, 0 },
    { 0x63cfe099, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3fe09c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6afe0a1, 90 },
    { 0, 0 },
    { 0xeaafe0a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582fe0a7, 90 },
    { 0xa87fe0a7, 34 },
    { 0x3e9fe0a9, 87 },
    { 0x5ccfe0a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cfe0ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe0b0, 88 },
    { 0x1c8fe0b1, 23 },
    { 0x64cfe0b2, 55 },
    { 0xa3afe0b3, 33 },
    { 0, 0 },
    { 0xc92fe0b5, 90 },
    { 0x349fe0b6, 98 },
    { 0x8fafe0b6, 78 },
    { 0x133fe0b7, 100 },
    { 0x24cfe0b8, 90 },
    { 0xdebfe0b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbfe0bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225fe0c8, 90 },
    { 0xd5cfe0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646fe0d0, 33 },
    { 0x50cfe0d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825fe0d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426fe0de, 33 },
    { 0xdb7fe0df, 55 },
    { 0xefe0e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34afe0e7, 90 },
    { 0x7a6fe0e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cfe0ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cfe0f1, 52 },
    { 0xfc1fe0f2, 48 },
    { 0x9acfe0f1, 90 },
    { 0x3cfe0f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96afe0f9, 95 },
    { 0x684fe0fa, 90 },
    { 0x3ffe0fb, 78 },
    { 0xf79fe0fb, 33 },
    { 0x351fe0f9, 61 },
    { 0x93afe0fe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4fe103, 61 },
    { 0xf5afe103, 84 },
    { 0, 0 },
    { 0xc2fe106, 23 },
    { 0, 0 },
    { 0xa3ffe108, 23 },
    { 0xe03fe109, 23 },
    { 0, 0 },
    { 0xbedfe10b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535fe110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0fe117, 55 },
    { 0x94ffe118, 99 },
    { 0x65ffe118, 40 },
    { 0x28fe11a, 93 },
    { 0x1fafe11a, 76 },
    { 0xb5afe11c, 98 },
    { 0, 0 },
    { 0xdb8fe11e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efe123, 76 },
    { 0x721fe124, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08fe127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x299fe12a, 9 },
    { 0, 0 },
    { 0x5fcfe12c, 33 },
    { 0xef4fe12d, 23 },
    { 0x467fe12d, 40 },
    { 0xb75fe12f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30fe133, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfe137, 23 },
    { 0x16efe138, 98 },
    { 0xd57fe138, 23 },
    { 0, 0 },
    { 0x644fe13b, 23 },
    { 0x2bfe13c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593fe140, 90 },
    { 0x997fe141, 90 },
    { 0x4adfe142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdfe148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cefe14d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141fe156, 23 },
    { 0xf54fe157, 78 },
    { 0x209fe158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbfe160, 81 },
    { 0xea6fe161, 95 },
    { 0x86dfe161, 90 },
    { 0, 0 },
    { 0x512fe164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0fe167, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bfe171, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ffe174, 90 },
    { 0x3b7fe175, 95 },
    { 0xa1afe176, 43 },
    { 0xbc4fe177, 98 },
    { 0x1d2fe178, 87 },
    { 0, 0 },
    { 0x313fe17a, 98 },
    { 0, 0 },
    { 0xc6fe17c, 61 },
    { 0xb78fe17d, 98 },
    { 0, 0 },
    { 0x5e2fe17f, 61 },
    { 0xce8fe180, 81 },
    { 0x584fe181, 33 },
    { 0, 0 },
    { 0xfe6fe183, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cfe187, 100 },
    { 0xfe188, 65 },
    { 0xb9ffe188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fe190, 23 },
    { 0xd68fe191, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fe196, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x631fe199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1fe19d, 95 },
    { 0x368fe19e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x732fe1a1, 90 },
    { 0x307fe1a2, 43 },
    { 0x508fe1a1, 23 },
    { 0, 0 },
    { 0xa7bfe1a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeefe1a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929fe1ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bfe1b0, 66 },
    { 0xe85fe1b0, 61 },
    { 0x51bfe1b2, 33 },
    { 0x791fe1b3, 95 },
    { 0, 0 },
    { 0x677fe1b5, 23 },
    { 0x6c5fe1b5, 90 },
    { 0xde4fe1b7, 98 },
    { 0x57bfe1b5, 33 },
    { 0xe7ffe1b6, 43 },
    { 0x6cfe1b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bfe1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98fe1c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cfe1c5, 98 },
    { 0xa5cfe1c5, 55 },
    { 0x886fe1c7, 33 },
    { 0x4dcfe1c8, 95 },
    { 0xeb2fe1c9, 23 },
    { 0x926fe1c7, 87 },
    { 0xfd2fe1cb, 88 },
    { 0, 0 },
    { 0x7befe1cd, 55 },
    { 0x9edfe1cd, 23 },
    { 0x330fe1cf, 33 },
    { 0, 0 },
    { 0xfefe1d1, 91 },
    { 0xae4fe1d2, 33 },
    { 0, 0 },
    { 0xf95fe1d4, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x65efe1d7, 55 },
    { 0x3a0fe1d8, 67 },
    { 0, 0 },
    { 0xd35fe1da, 76 },
    { 0xce6fe1da, 90 },
    { 0, 0 },
    { 0xc8afe1dd, 46 },
    { 0, 0 },
    { 0x62bfe1df, 33 },
    { 0, 0 },
    { 0x5a9fe1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79fe1e6, 90 },
    { 0, 0 },
    { 0x3cafe1e8, 55 },
    { 0x827fe1e9, 61 },
    { 0xc74fe1e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781fe1f0, 23 },
    { 0, 0 },
    { 0x9b1fe1f2, 75 },
    { 0, 0 },
    { 0xeb9fe1f4, 33 },
    { 0xaa9fe1f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0fe1fd, 55 },
    { 0xa1efe1fe, 100 },
    { 0xf9efe1fe, 95 },
    { 0x476fe200, 61 },
    { 0x6fffe201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fe206, 33 },
    { 0x5a7fe207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7fe20a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x390fe20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfe211, 65 },
    { 0x4f9fe211, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fe218, 23 },
    { 0x484fe219, 34 },
    { 0x244fe21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddfe224, 34 },
    { 0x77cfe225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fe22a, 33 },
    { 0x8f0fe22b, 90 },
    { 0x765fe22b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebfe230, 61 },
    { 0x89bfe231, 67 },
    { 0xb87fe232, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59fe236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54fe23a, 33 },
    { 0x73afe23b, 33 },
    { 0xe69fe23a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96fe242, 81 },
    { 0x3a8fe242, 40 },
    { 0x6f8fe244, 90 },
    { 0xe3efe245, 90 },
    { 0x820fe246, 34 },
    { 0x324fe245, 23 },
    { 0x9e5fe248, 33 },
    { 0xaa0fe249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfe24c, 55 },
    { 0, 0 },
    { 0xfe6fe24e, 93 },
    { 0xe2bfe24f, 61 },
    { 0xf71fe24f, 99 },
    { 0x705fe24e, 23 },
    { 0x579fe252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efe257, 67 },
    { 0x18dfe258, 90 },
    { 0xa01fe259, 61 },
    { 0x63dfe259, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe263, 23 },
    { 0x1c8fe263, 23 },
    { 0x75dfe265, 61 },
    { 0xbf0fe263, 33 },
    { 0x6fe267, 33 },
    { 0x9d9fe267, 90 },
    { 0x74fe269, 33 },
    { 0x58fe26a, 96 },
    { 0x186fe264, 98 },
    { 0x4a0fe266, 75 },
    { 0xb9cfe26d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89fe270, 43 },
    { 0x63dfe271, 69 },
    { 0x5f2fe272, 61 },
    { 0x4d5fe273, 23 },
    { 0, 0 },
    { 0xeabfe275, 90 },
    { 0x3a9fe276, 98 },
    { 0x675fe276, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x285fe27a, 91 },
    { 0xf3cfe27a, 55 },
    { 0, 0 },
    { 0x73efe27d, 95 },
    { 0xd40fe27d, 98 },
    { 0xa44fe27f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78fe283, 98 },
    { 0x286fe283, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe287, 33 },
    { 0x6aafe288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcfe28c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bfe28f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8befe294, 55 },
    { 0x754fe295, 91 },
    { 0, 0 },
    { 0xe05fe297, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1fe29d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6fe2a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fe2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fe2ae, 81 },
    { 0, 0 },
    { 0xc17fe2b0, 90 },
    { 0, 0 },
    { 0x179fe2b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fe2b8, 40 },
    { 0, 0 },
    { 0xec6fe2ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98fe2bd, 24 },
    { 0, 0 },
    { 0xb63fe2bf, 76 },
    { 0x809fe2bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfe2c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fe2c8, 33 },
    { 0x652fe2c9, 55 },
    { 0x70ffe2c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12fe2d1, 61 },
    { 0xf5efe2d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cfe2d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5fe2db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dfe2de, 96 },
    { 0, 0 },
    { 0xdcdfe2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1fe2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85fe2e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe2eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x484fe2ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cfe2f1, 61 },
    { 0xb68fe2f2, 90 },
    { 0x957fe2f3, 76 },
    { 0xa36fe2f4, 98 },
    { 0x53efe2f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ffe2fb, 61 },
    { 0, 0 },
    { 0xfbffe2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0fe306, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbfe309, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ffe30f, 91 },
    { 0, 0 },
    { 0xe4afe311, 90 },
    { 0, 0 },
    { 0x7dcfe313, 55 },
    { 0x750fe314, 91 },
    { 0x427fe315, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fe31b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaefe31e, 46 },
    { 0x913fe31f, 90 },
    { 0xa14fe31f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bffe326, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fe32b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdfe331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cfe337, 84 },
    { 0xec1fe337, 33 },
    { 0, 0 },
    { 0xd2cfe33a, 98 },
    { 0x80fe33b, 23 },
    { 0x4edfe33a, 90 },
    { 0x1f5fe33d, 90 },
    { 0xa0cfe33d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66fe341, 96 },
    { 0, 0 },
    { 0x2d4fe343, 91 },
    { 0xc5efe344, 33 },
    { 0x69dfe343, 67 },
    { 0x99dfe346, 61 },
    { 0x3f6fe346, 46 },
    { 0xf72fe344, 98 },
    { 0xbf6fe349, 98 },
    { 0xe2afe347, 98 },
    { 0x4b7fe34b, 98 },
    { 0xdf2fe347, 80 },
    { 0, 0 },
    { 0xe43fe34e, 67 },
    { 0x83efe34e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x589fe352, 61 },
    { 0, 0 },
    { 0x90cfe354, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44fe357, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fe360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5fe363, 90 },
    { 0x6d2fe364, 98 },
    { 0xe17fe363, 55 },
    { 0x27dfe365, 90 },
    { 0x787fe367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bfe36c, 61 },
    { 0x38dfe36d, 78 },
    { 0xb9bfe36d, 95 },
    { 0x839fe36c, 92 },
    { 0x9aafe370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bfe375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63fe37e, 84 },
    { 0x9d3fe37f, 90 },
    { 0, 0 },
    { 0x9dbfe381, 61 },
    { 0, 0 },
    { 0x729fe383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dafe387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cfe38a, 55 },
    { 0, 0 },
    { 0x8e8fe38c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccefe390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dfe393, 98 },
    { 0xac8fe394, 96 },
    { 0, 0 },
    { 0x43ffe396, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfe399, 56 },
    { 0, 0 },
    { 0x166fe39b, 33 },
    { 0xb30fe39c, 90 },
    { 0, 0 },
    { 0x802fe39e, 33 },
    { 0x810fe39e, 90 },
    { 0, 0 },
    { 0x2befe3a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632fe3a5, 90 },
    { 0x499fe3a6, 90 },
    { 0, 0 },
    { 0x3c4fe3a8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9affe3ab, 90 },
    { 0xe3dfe3ac, 90 },
    { 0xd2cfe3ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4fe3b5, 76 },
    { 0xf5bfe3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68fe3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fe3bf, 66 },
    { 0, 0 },
    { 0xd81fe3c1, 33 },
    { 0xa5ffe3c1, 34 },
    { 0x6fbfe3c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bfe3c7, 34 },
    { 0xbb9fe3c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffe3cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98fe3cf, 76 },
    { 0, 0 },
    { 0x864fe3d1, 76 },
    { 0xf3ffe3d2, 93 },
    { 0xdccfe3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebfe3d7, 93 },
    { 0, 0 },
    { 0x255fe3d9, 95 },
    { 0xe4cfe3da, 87 },
    { 0, 0 },
    { 0x941fe3dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3fe3e2, 23 },
    { 0x4e3fe3e3, 61 },
    { 0x280fe3e4, 90 },
    { 0x409fe3e5, 61 },
    { 0x744fe3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174fe3e9, 75 },
    { 0xaa9fe3ea, 93 },
    { 0x755fe3eb, 90 },
    { 0xf41fe3ec, 66 },
    { 0xae2fe3ec, 35 },
    { 0x17afe3ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3fe3f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7fe3f7, 99 },
    { 0xc41fe3f8, 98 },
    { 0xf07fe3f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8fe3fc, 61 },
    { 0x5aafe3fd, 90 },
    { 0x54afe3fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8efe403, 95 },
    { 0x12bfe404, 84 },
    { 0xa60fe405, 23 },
    { 0xc15fe405, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bafe40b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x855fe41b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5fe421, 61 },
    { 0, 0 },
    { 0x18bfe423, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffe426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfe429, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9efe42c, 33 },
    { 0xe2afe42c, 61 },
    { 0, 0 },
    { 0x88fe42f, 38 },
    { 0xb9ffe430, 46 },
    { 0x25bfe430, 61 },
    { 0, 0 },
    { 0x38efe433, 43 },
    { 0xf52fe434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bfe437, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ffe441, 33 },
    { 0x8bbfe441, 23 },
    { 0x174fe443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28fe447, 33 },
    { 0, 0 },
    { 0xc3cfe449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdefe44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfe44f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x674fe452, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddfe455, 90 },
    { 0x6bafe456, 98 },
    { 0x9dbfe457, 51 },
    { 0xfb8fe458, 23 },
    { 0x7ecfe455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddfe461, 55 },
    { 0xdd9fe461, 55 },
    { 0x508fe463, 33 },
    { 0x54afe464, 61 },
    { 0x6dafe461, 80 },
    { 0x356fe464, 78 },
    { 0x205fe467, 84 },
    { 0x738fe465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0fe46c, 93 },
    { 0x866fe46c, 90 },
    { 0x85afe46d, 98 },
    { 0x2ccfe46f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fe478, 96 },
    { 0x834fe479, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ffe47c, 91 },
    { 0, 0 },
    { 0xaf1fe47e, 61 },
    { 0x542fe47f, 34 },
    { 0xe37fe47e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3fe485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549fe48c, 43 },
    { 0, 0 },
    { 0x600fe48e, 33 },
    { 0x4d7fe48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760fe493, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacfe497, 100 },
    { 0, 0 },
    { 0x848fe499, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ffe49c, 95 },
    { 0x5bffe49d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83efe4a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4fe4a5, 46 },
    { 0, 0 },
    { 0xb84fe4a7, 23 },
    { 0, 0 },
    { 0x724fe4a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ffe4ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dffe4b2, 87 },
    { 0x82fe4b3, 23 },
    { 0x7c0fe4b4, 95 },
    { 0, 0 },
    { 0x4ebfe4b6, 85 },
    { 0xcf8fe4b6, 98 },
    { 0xa54fe4b8, 61 },
    { 0x143fe4b9, 98 },
    { 0, 0 },
    { 0x813fe4bb, 91 },
    { 0x162fe4bb, 61 },
    { 0x6e0fe4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505fe4c2, 43 },
    { 0x6c0fe4c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cfe4c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25fe4d4, 95 },
    { 0, 0 },
    { 0xf6ffe4d6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50fe4da, 46 },
    { 0, 0 },
    { 0xc98fe4dc, 23 },
    { 0xe51fe4dd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafe4e9, 87 },
    { 0, 0 },
    { 0xd48fe4eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47fe4ee, 90 },
    { 0x7efe4ef, 61 },
    { 0x35bfe4ef, 93 },
    { 0x154fe4f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe4f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebafe500, 95 },
    { 0x292fe501, 46 },
    { 0x178fe500, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389fe509, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffe50f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cfe51c, 43 },
    { 0x7c4fe51d, 43 },
    { 0xf50fe51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cfe524, 76 },
    { 0x234fe524, 100 },
    { 0x645fe526, 90 },
    { 0xf6dfe524, 87 },
    { 0xfbbfe526, 98 },
    { 0x5f4fe529, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fe52c, 95 },
    { 0, 0 },
    { 0x407fe52e, 77 },
    { 0x958fe52f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebfe534, 23 },
    { 0, 0 },
    { 0xd1fe536, 33 },
    { 0xc47fe536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edfe53d, 67 },
    { 0, 0 },
    { 0x57bfe53f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ffe543, 95 },
    { 0x2e7fe543, 43 },
    { 0, 0 },
    { 0x3dcfe546, 33 },
    { 0, 0 },
    { 0xf06fe548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dfe54e, 95 },
    { 0xef3fe54f, 61 },
    { 0xe39fe550, 90 },
    { 0x297fe550, 66 },
    { 0x465fe552, 95 },
    { 0xbbefe553, 95 },
    { 0x874fe554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fe55b, 93 },
    { 0x9fafe55c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921fe560, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265fe564, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fe567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aafe56a, 88 },
    { 0xdb5fe56a, 61 },
    { 0xbbffe56c, 61 },
    { 0, 0 },
    { 0x786fe56e, 23 },
    { 0xc14fe56e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe572, 98 },
    { 0xd3dfe572, 90 },
    { 0xdbbfe574, 90 },
    { 0x48dfe574, 34 },
    { 0x8f6fe576, 76 },
    { 0xef3fe577, 91 },
    { 0xc9efe578, 90 },
    { 0x955fe579, 90 },
    { 0x3b1fe57a, 43 },
    { 0x3e4fe575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85afe57f, 33 },
    { 0xf34fe580, 34 },
    { 0xe34fe581, 23 },
    { 0x195fe582, 95 },
    { 0xb4efe583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0fe587, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7afe58b, 98 },
    { 0xe4ffe58c, 67 },
    { 0, 0 },
    { 0x2e5fe58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdefe59a, 90 },
    { 0x580fe59a, 43 },
    { 0xe85fe59c, 66 },
    { 0xbaefe59d, 33 },
    { 0x165fe59e, 33 },
    { 0x7a6fe59c, 90 },
    { 0xce3fe5a0, 76 },
    { 0xc71fe59f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fe5a5, 90 },
    { 0x57bfe5a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe5a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe5af, 34 },
    { 0xc43fe5b0, 90 },
    { 0xb6ffe5af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4fe5bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3fe5be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfe5c1, 61 },
    { 0x35afe5c2, 33 },
    { 0xcdffe5c2, 90 },
    { 0, 0 },
    { 0x8cafe5c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9fe5c9, 61 },
    { 0x33fe5ca, 99 },
    { 0, 0 },
    { 0x300fe5cc, 55 },
    { 0, 0 },
    { 0x2dafe5ce, 33 },
    { 0x747fe5cf, 81 },
    { 0xf8bfe5d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2fe5d6, 23 },
    { 0, 0 },
    { 0x3f4fe5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fe5dc, 96 },
    { 0xd28fe5dd, 61 },
    { 0x944fe5de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdfe5e1, 61 },
    { 0xf8bfe5e2, 53 },
    { 0x6d2fe5e3, 91 },
    { 0x7fbfe5e4, 99 },
    { 0xc8ffe5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fe5e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x866fe5eb, 76 },
    { 0xd73fe5eb, 55 },
    { 0, 0 },
    { 0xd42fe5ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1fe5f3, 67 },
    { 0, 0 },
    { 0xb0ffe5f5, 98 },
    { 0xcbdfe5f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecefe5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64efe602, 61 },
    { 0, 0 },
    { 0xb94fe604, 61 },
    { 0x18efe605, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfe608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fe60f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe612, 61 },
    { 0x642fe612, 90 },
    { 0, 0 },
    { 0x3cfe615, 33 },
    { 0x879fe615, 93 },
    { 0, 0 },
    { 0x7fefe618, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdfe61e, 81 },
    { 0, 0 },
    { 0x87bfe620, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x951fe623, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bfe626, 78 },
    { 0, 0 },
    { 0x625fe628, 34 },
    { 0x425fe628, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9efe62c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3efe634, 66 },
    { 0, 0 },
    { 0xb94fe636, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfe63c, 90 },
    { 0x701fe63d, 98 },
    { 0x4a1fe63e, 98 },
    { 0x607fe63e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24efe649, 55 },
    { 0x950fe64a, 64 },
    { 0x21fe649, 55 },
    { 0x738fe649, 90 },
    { 0x6d5fe64d, 98 },
    { 0, 0 },
    { 0x4c2fe64f, 78 },
    { 0x31efe650, 23 },
    { 0, 0 },
    { 0xc46fe652, 90 },
    { 0, 0 },
    { 0xffafe654, 66 },
    { 0x4e0fe654, 33 },
    { 0x6b3fe656, 61 },
    { 0x876fe657, 23 },
    { 0x484fe658, 90 },
    { 0xb51fe657, 33 },
    { 0x234fe65a, 61 },
    { 0x55fe656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23fe661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacfe664, 93 },
    { 0, 0 },
    { 0x1befe666, 66 },
    { 0, 0 },
    { 0x11cfe668, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dfe66f, 33 },
    { 0, 0 },
    { 0x8bdfe671, 34 },
    { 0, 0 },
    { 0xa53fe673, 90 },
    { 0, 0 },
    { 0xbbfe675, 90 },
    { 0, 0 },
    { 0x9e4fe677, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fe67c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218fe687, 61 },
    { 0x9c3fe687, 95 },
    { 0x205fe689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fe691, 88 },
    { 0x987fe691, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67fe695, 87 },
    { 0, 0 },
    { 0x805fe697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9fe69d, 85 },
    { 0x762fe69d, 84 },
    { 0x36dfe69f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0afe6a2, 55 },
    { 0x995fe6a3, 55 },
    { 0x6effe6a4, 95 },
    { 0x240fe6a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17efe6a9, 61 },
    { 0x2ffe6a9, 33 },
    { 0, 0 },
    { 0x57efe6ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fe6b6, 95 },
    { 0, 0 },
    { 0x82bfe6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x743fe6bb, 93 },
    { 0x275fe6bc, 61 },
    { 0, 0 },
    { 0xf50fe6be, 85 },
    { 0, 0 },
    { 0xb06fe6c0, 76 },
    { 0x5f9fe6c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1fe6c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810fe6cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dfe6d4, 33 },
    { 0xd40fe6d5, 34 },
    { 0x499fe6d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadfe6e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911fe6e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfe6ea, 90 },
    { 0, 0 },
    { 0xda2fe6ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabfe6ef, 66 },
    { 0xfadfe6f0, 42 },
    { 0x917fe6f1, 33 },
    { 0x4bafe6f0, 23 },
    { 0xae1fe6f2, 69 },
    { 0x6d3fe6f4, 95 },
    { 0x676fe6f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x685fe6f8, 76 },
    { 0x86dfe6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689fe6fd, 95 },
    { 0x566fe6fe, 100 },
    { 0x12afe6fe, 33 },
    { 0x8dfe6fd, 33 },
    { 0x3c3fe6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44efe706, 95 },
    { 0x615fe707, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fe70b, 85 },
    { 0x1d8fe70c, 61 },
    { 0x927fe70d, 95 },
    { 0xd75fe70e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1fe711, 75 },
    { 0, 0 },
    { 0xbccfe713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48fe719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe724, 23 },
    { 0xf71fe725, 90 },
    { 0, 0 },
    { 0xa64fe727, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffe72a, 90 },
    { 0x82efe72a, 95 },
    { 0x765fe72c, 23 },
    { 0x8d8fe72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4fe734, 95 },
    { 0, 0 },
    { 0x1a3fe736, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dfe739, 98 },
    { 0xcbdfe73a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bfe73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfe742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8afe745, 90 },
    { 0, 0 },
    { 0xa7fe747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297fe74b, 81 },
    { 0, 0 },
    { 0x2d7fe74d, 33 },
    { 0, 0 },
    { 0xcbefe74f, 23 },
    { 0x685fe74f, 55 },
    { 0x36afe751, 23 },
    { 0, 0 },
    { 0x54afe753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14fe758, 90 },
    { 0x404fe759, 61 },
    { 0x31cfe75a, 81 },
    { 0x11cfe75b, 34 },
    { 0xb63fe758, 34 },
    { 0xf7bfe75c, 34 },
    { 0xe17fe75e, 98 },
    { 0x442fe75e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe762, 87 },
    { 0, 0 },
    { 0x967fe764, 34 },
    { 0xb71fe765, 90 },
    { 0x6f0fe766, 55 },
    { 0xcedfe764, 83 },
    { 0x223fe768, 34 },
    { 0, 0 },
    { 0xf18fe76a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99fe770, 33 },
    { 0, 0 },
    { 0xc2cfe772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130fe784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617fe788, 33 },
    { 0, 0 },
    { 0xbe3fe78a, 90 },
    { 0xfe6fe78b, 90 },
    { 0x6c0fe78c, 33 },
    { 0x214fe78c, 90 },
    { 0x334fe78e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8fe792, 87 },
    { 0x846fe793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbfe797, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436fe79c, 90 },
    { 0, 0 },
    { 0x66afe79e, 90 },
    { 0, 0 },
    { 0x6fbfe7a0, 95 },
    { 0xefbfe7a1, 74 },
    { 0, 0 },
    { 0x81fe7a3, 91 },
    { 0x284fe7a3, 33 },
    { 0xc7cfe7a5, 98 },
    { 0x536fe7a5, 98 },
    { 0xbf8fe7a7, 95 },
    { 0x526fe7a8, 91 },
    { 0xe44fe7a8, 90 },
    { 0x697fe7aa, 61 },
    { 0x418fe7aa, 95 },
    { 0x50cfe7aa, 77 },
    { 0x42afe7ad, 90 },
    { 0xfbbfe7ae, 95 },
    { 0xcfdfe7af, 34 },
    { 0, 0 },
    { 0x24efe7b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616fe7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6fe7c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273fe7c6, 90 },
    { 0, 0 },
    { 0x4acfe7c8, 90 },
    { 0xf19fe7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccfe7cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fe7d0, 46 },
    { 0xe78fe7d0, 90 },
    { 0x98fe7d2, 33 },
    { 0, 0 },
    { 0x121fe7d4, 69 },
    { 0x35bfe7d4, 17 },
    { 0xf23fe7d6, 34 },
    { 0xe67fe7d6, 88 },
    { 0xd87fe7d8, 33 },
    { 0xb74fe7d9, 33 },
    { 0x292fe7d6, 52 },
    { 0, 0 },
    { 0x569fe7dc, 76 },
    { 0xe15fe7dc, 85 },
    { 0, 0 },
    { 0xe62fe7df, 90 },
    { 0, 0 },
    { 0xcdefe7e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1efe7e4, 23 },
    { 0, 0 },
    { 0x270fe7e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172fe7f1, 90 },
    { 0x453fe7f1, 33 },
    { 0x522fe7f1, 93 },
    { 0, 0 },
    { 0x525fe7f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538fe7ff, 90 },
    { 0x5bbfe7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefe804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69fe80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dfe810, 85 },
    { 0x6a8fe811, 23 },
    { 0x118fe811, 78 },
    { 0x73fe813, 90 },
    { 0xe7afe814, 55 },
    { 0x40afe815, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfe818, 33 },
    { 0, 0 },
    { 0x644fe81a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fe81e, 95 },
    { 0xc88fe81f, 84 },
    { 0xc1cfe820, 99 },
    { 0, 0 },
    { 0x36cfe822, 91 },
    { 0, 0 },
    { 0x4cafe824, 98 },
    { 0, 0 },
    { 0xb8ffe826, 34 },
    { 0x76dfe827, 61 },
    { 0x930fe827, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2fe82d, 34 },
    { 0, 0 },
    { 0xd47fe82f, 88 },
    { 0x611fe82f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x147fe833, 84 },
    { 0, 0 },
    { 0x29fe835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fe839, 52 },
    { 0x992fe83a, 61 },
    { 0, 0 },
    { 0x1b6fe83c, 78 },
    { 0, 0 },
    { 0xcfffe83e, 93 },
    { 0x18dfe83e, 95 },
    { 0x97afe840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154fe845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaefe848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4fe84b, 96 },
    { 0x809fe84b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fe84f, 76 },
    { 0, 0 },
    { 0xaf0fe851, 55 },
    { 0x977fe852, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafe855, 33 },
    { 0x394fe856, 99 },
    { 0, 0 },
    { 0x662fe858, 76 },
    { 0xe30fe859, 23 },
    { 0x339fe85a, 65 },
    { 0x610fe85a, 33 },
    { 0xe92fe859, 17 },
    { 0x3b4fe859, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cfe862, 23 },
    { 0x956fe863, 99 },
    { 0xf68fe864, 76 },
    { 0x45fe862, 90 },
    { 0x6bbfe866, 33 },
    { 0x3aafe866, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedfe873, 61 },
    { 0x94afe873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x185fe877, 34 },
    { 0, 0 },
    { 0xc40fe879, 34 },
    { 0xf46fe87a, 87 },
    { 0x9affe879, 68 },
    { 0x77afe87a, 76 },
    { 0xda8fe87d, 87 },
    { 0, 0 },
    { 0x942fe87f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcfe883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x565fe886, 84 },
    { 0, 0 },
    { 0x842fe888, 91 },
    { 0xa30fe889, 61 },
    { 0, 0 },
    { 0x471fe88b, 93 },
    { 0x82cfe88c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bfe890, 33 },
    { 0xe86fe890, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9efe897, 5 },
    { 0, 0 },
    { 0x296fe899, 93 },
    { 0xb9fe89a, 23 },
    { 0x1b3fe89b, 101 },
    { 0x303fe89b, 52 },
    { 0x559fe89c, 81 },
    { 0x727fe89e, 33 },
    { 0x6dfe89d, 61 },
    { 0x99cfe8a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953fe8a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x621fe8aa, 23 },
    { 0, 0 },
    { 0xa4cfe8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467fe8b0, 23 },
    { 0xcf9fe8b1, 98 },
    { 0x878fe8b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44fe8b8, 34 },
    { 0x769fe8b8, 100 },
    { 0x586fe8ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bfe8be, 40 },
    { 0xb43fe8be, 98 },
    { 0xa3dfe8c0, 61 },
    { 0, 0 },
    { 0x156fe8c2, 23 },
    { 0x872fe8c2, 33 },
    { 0x438fe8c4, 34 },
    { 0x1ecfe8c5, 33 },
    { 0, 0 },
    { 0x2cafe8c7, 78 },
    { 0x542fe8c8, 33 },
    { 0, 0 },
    { 0xa45fe8ca, 61 },
    { 0, 0 },
    { 0xf01fe8cc, 61 },
    { 0, 0 },
    { 0x218fe8ce, 23 },
    { 0xbcffe8cf, 95 },
    { 0xd63fe8ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13efe8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fefe8de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6fe8e5, 43 },
    { 0x256fe8e6, 95 },
    { 0x75cfe8e7, 95 },
    { 0x2acfe8e8, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22fe8ee, 46 },
    { 0xd6afe8ef, 90 },
    { 0, 0 },
    { 0xd09fe8f1, 95 },
    { 0, 0 },
    { 0x50dfe8f3, 84 },
    { 0x875fe8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667fe8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcfe8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bfe903, 61 },
    { 0x6b2fe904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977fe90c, 33 },
    { 0xa45fe90d, 61 },
    { 0, 0 },
    { 0xba3fe90f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391fe912, 90 },
    { 0xc40fe912, 90 },
    { 0xb00fe914, 55 },
    { 0x1d6fe913, 55 },
    { 0xd9ffe915, 65 },
    { 0x5a4fe915, 95 },
    { 0xe32fe914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318fe920, 95 },
    { 0x83afe920, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70fe928, 55 },
    { 0x288fe929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe92c, 43 },
    { 0, 0 },
    { 0x4d8fe92e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13fe937, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600fe93d, 95 },
    { 0, 0 },
    { 0xe9ffe93f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6fe944, 66 },
    { 0, 0 },
    { 0x382fe946, 46 },
    { 0x9ebfe946, 81 },
    { 0xf6ffe947, 95 },
    { 0, 0 },
    { 0x177fe94a, 33 },
    { 0xa90fe94a, 91 },
    { 0xfc3fe94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9fe950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fe954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efe959, 90 },
    { 0x413fe95a, 94 },
    { 0, 0 },
    { 0xb85fe95c, 34 },
    { 0x670fe95d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395fe964, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fefe968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2fe973, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x604fe976, 65 },
    { 0x871fe977, 90 },
    { 0xea3fe977, 99 },
    { 0xd94fe979, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0fe97d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860fe986, 61 },
    { 0x983fe987, 76 },
    { 0xc3afe988, 98 },
    { 0, 0 },
    { 0xe33fe98a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ffe98d, 23 },
    { 0xee9fe98e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6fe998, 77 },
    { 0x202fe999, 47 },
    { 0xc83fe99a, 67 },
    { 0x133fe99b, 33 },
    { 0x3ebfe99c, 48 },
    { 0x770fe99d, 90 },
    { 0x2e8fe99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ffe9a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecfe9a7, 43 },
    { 0, 0 },
    { 0x777fe9a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fe9ac, 61 },
    { 0x51fe9ac, 98 },
    { 0xe93fe9ae, 90 },
    { 0xdccfe9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552fe9ba, 90 },
    { 0xbdafe9bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ffe9c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66fe9c9, 82 },
    { 0x90afe9ca, 78 },
    { 0x359fe9cb, 90 },
    { 0x925fe9cc, 23 },
    { 0x698fe9c9, 43 },
    { 0x5d2fe9cd, 90 },
    { 0xa00fe9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfe9d4, 61 },
    { 0x6c2fe9d5, 36 },
    { 0xea8fe9d4, 33 },
    { 0x62fe9d5, 43 },
    { 0x4bafe9d4, 61 },
    { 0x6c9fe9d7, 81 },
    { 0, 0 },
    { 0x511fe9db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8fe9e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2fe9e3, 61 },
    { 0, 0 },
    { 0x4a1fe9e5, 55 },
    { 0, 0 },
    { 0x5a3fe9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efe9ee, 99 },
    { 0x712fe9ee, 33 },
    { 0, 0 },
    { 0x17dfe9f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4fe9f9, 76 },
    { 0x985fe9f9, 34 },
    { 0, 0 },
    { 0x9dcfe9fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29efe9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62fea06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dfea09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8fea12, 100 },
    { 0, 0 },
    { 0xc86fea14, 33 },
    { 0x790fea15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558fea1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83efea20, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6fea28, 96 },
    { 0, 0 },
    { 0x65ffea2a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14fea2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2effea32, 23 },
    { 0x539fea33, 46 },
    { 0xfbafea34, 99 },
    { 0x97efea33, 23 },
    { 0xd4cfea36, 76 },
    { 0x157fea37, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1fea3b, 23 },
    { 0, 0 },
    { 0x21ffea3d, 17 },
    { 0x60afea3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5fea42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fea46, 98 },
    { 0x137fea47, 77 },
    { 0x4efea47, 55 },
    { 0xb72fea49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abfea4c, 90 },
    { 0x9c4fea4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdfea52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216fea58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfea5b, 90 },
    { 0x269fea5c, 90 },
    { 0, 0 },
    { 0xa2ffea5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbefea63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cafea6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56efea6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496fea72, 90 },
    { 0xd4fea72, 95 },
    { 0, 0 },
    { 0xdc1fea75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424fea79, 61 },
    { 0xca5fea7a, 98 },
    { 0x262fea79, 43 },
    { 0, 0 },
    { 0x846fea7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37fea87, 95 },
    { 0xc7fea88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9efea8b, 81 },
    { 0xeb8fea8b, 100 },
    { 0, 0 },
    { 0xe0fea8e, 90 },
    { 0x948fea8f, 90 },
    { 0xa94fea8f, 61 },
    { 0x5fafea8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393fea98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcfea9d, 90 },
    { 0xc08fea9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdfeaac, 90 },
    { 0x722feaad, 76 },
    { 0x8ccfeaae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78feab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cfeab7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feabe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfeac3, 23 },
    { 0xa82feac4, 23 },
    { 0x9cafeac3, 61 },
    { 0, 0 },
    { 0x98efeac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9feaca, 87 },
    { 0xcfeacb, 93 },
    { 0x458feacb, 23 },
    { 0x431feacd, 98 },
    { 0, 0 },
    { 0x7c8feacf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27fead2, 100 },
    { 0x10afead2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58feadd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376feae2, 90 },
    { 0x559feae3, 90 },
    { 0x671feae2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7feae8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41feaeb, 95 },
    { 0xb0cfeaec, 43 },
    { 0xb52feaec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeffeaf0, 76 },
    { 0x6b2feaf1, 67 },
    { 0x49bfeaf0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481feaf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184feafc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadfeaff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681feb03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dfeb06, 35 },
    { 0xa28feb06, 90 },
    { 0, 0 },
    { 0x3acfeb09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x671feb0c, 84 },
    { 0x9a9feb0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608feb14, 77 },
    { 0xb09feb14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcfeb1b, 99 },
    { 0, 0 },
    { 0x810feb1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcffeb21, 94 },
    { 0x8f2feb22, 59 },
    { 0, 0 },
    { 0xc83feb24, 98 },
    { 0, 0 },
    { 0x2fbfeb26, 43 },
    { 0x552feb27, 66 },
    { 0xca7feb27, 98 },
    { 0xce9feb29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494feb2d, 61 },
    { 0xe4efeb2e, 34 },
    { 0xcf1feb2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bfeb35, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53feb3d, 43 },
    { 0xa87feb3d, 85 },
    { 0xbb0feb3e, 61 },
    { 0x3b3feb40, 95 },
    { 0, 0 },
    { 0x9b9feb42, 98 },
    { 0, 0 },
    { 0xb91feb44, 33 },
    { 0xf87feb45, 91 },
    { 0x8c0feb45, 76 },
    { 0xcf2feb44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493feb4b, 98 },
    { 0xf18feb4b, 33 },
    { 0xa2bfeb4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89feb51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9feb58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584feb5c, 98 },
    { 0, 0 },
    { 0xa29feb5e, 95 },
    { 0xa6efeb5e, 98 },
    { 0xec9feb60, 95 },
    { 0x99ffeb60, 33 },
    { 0xc1dfeb61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ffeb65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2feb69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efeb6f, 85 },
    { 0xa9efeb70, 84 },
    { 0x7a9feb71, 55 },
    { 0xf65feb6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffeb7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x868feb7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49feb80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecfeb86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddfeb8e, 23 },
    { 0xfe1feb8f, 84 },
    { 0x49cfeb8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02feb93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfeb99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cfeb9d, 78 },
    { 0x785feb9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07feba5, 78 },
    { 0, 0 },
    { 0xaf5feba7, 39 },
    { 0xa1afeba8, 90 },
    { 0, 0 },
    { 0x173febaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4febae, 66 },
    { 0, 0 },
    { 0x3a6febb0, 67 },
    { 0x545febb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aefebb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ffebba, 98 },
    { 0, 0 },
    { 0x342febbc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1febc0, 33 },
    { 0, 0 },
    { 0x9e0febc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ffebc5, 93 },
    { 0x51efebc5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00febc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18efebcd, 43 },
    { 0x85ffebcd, 90 },
    { 0, 0 },
    { 0xb0cfebd0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bfebd3, 66 },
    { 0xebffebd4, 90 },
    { 0, 0 },
    { 0xe5afebd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aafebda, 98 },
    { 0xd6afebdb, 61 },
    { 0xbf8febdc, 67 },
    { 0x857febdc, 61 },
    { 0x478febdc, 33 },
    { 0x5d8febdf, 98 },
    { 0x38efebe0, 81 },
    { 0xa19febde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64febe4, 90 },
    { 0xdcfebe5, 77 },
    { 0x3d6febe6, 93 },
    { 0x530febe6, 34 },
    { 0xa49febe8, 88 },
    { 0, 0 },
    { 0xff2febea, 90 },
    { 0xd15febeb, 95 },
    { 0x45febec, 98 },
    { 0xe90febed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31febf1, 90 },
    { 0, 0 },
    { 0xb72febf3, 33 },
    { 0xff5febf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8febf7, 61 },
    { 0x587febf7, 81 },
    { 0, 0 },
    { 0x15cfebfa, 33 },
    { 0x321febfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fec0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cfec12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10fec15, 46 },
    { 0, 0 },
    { 0xa40fec17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acfec1e, 23 },
    { 0xc87fec1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccfec29, 90 },
    { 0xe91fec2a, 55 },
    { 0, 0 },
    { 0xebefec2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfec2f, 95 },
    { 0x357fec2f, 93 },
    { 0x8a0fec31, 33 },
    { 0x4f4fec2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fec37, 90 },
    { 0x7fcfec38, 76 },
    { 0xab0fec39, 76 },
    { 0xefffec37, 48 },
    { 0, 0 },
    { 0xf2fec3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dfec3f, 55 },
    { 0x980fec3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fec45, 90 },
    { 0, 0 },
    { 0xca7fec47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791fec4d, 66 },
    { 0x9fdfec4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8fec55, 90 },
    { 0xd47fec56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07fec5a, 78 },
    { 0x8dcfec5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3fec64, 55 },
    { 0xe20fec65, 91 },
    { 0x2a6fec66, 33 },
    { 0xb47fec66, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5fec6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecfec70, 96 },
    { 0x25efec71, 95 },
    { 0xf20fec72, 34 },
    { 0x982fec71, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3fec78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65fec7c, 55 },
    { 0, 0 },
    { 0x3e0fec7e, 98 },
    { 0xb4ffec7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efec84, 95 },
    { 0, 0 },
    { 0x64bfec86, 90 },
    { 0xaddfec87, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269fec8c, 34 },
    { 0xc7cfec8c, 98 },
    { 0x537fec8c, 33 },
    { 0xf9cfec8f, 23 },
    { 0x898fec90, 90 },
    { 0x2a7fec91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66fec96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6fec9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6feca0, 77 },
    { 0x915feca0, 55 },
    { 0, 0 },
    { 0x910feca3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccfeca7, 23 },
    { 0x60feca8, 66 },
    { 0x58afeca9, 68 },
    { 0x49feca7, 61 },
    { 0x956feca9, 23 },
    { 0x91afeca9, 75 },
    { 0x259fecad, 46 },
    { 0x6efecae, 67 },
    { 0xc68fecac, 100 },
    { 0xf5bfecad, 61 },
    { 0x281fecb1, 95 },
    { 0x869fecac, 23 },
    { 0xc77feca8, 66 },
    { 0xda3fecae, 87 },
    { 0x250fecad, 76 },
    { 0xbf3fecb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5fecbf, 95 },
    { 0, 0 },
    { 0xaa4fecc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363fecc5, 98 },
    { 0xe3ffecc6, 23 },
    { 0x125fecc7, 23 },
    { 0xc8fecc8, 55 },
    { 0xe9afecc9, 61 },
    { 0x604fecca, 66 },
    { 0, 0 },
    { 0x26afeccc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1feccf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31afecd7, 90 },
    { 0x945fecd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acfecdb, 36 },
    { 0x29dfecdc, 90 },
    { 0xf09fecdb, 43 },
    { 0x311fecdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fece1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfece5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fece9, 55 },
    { 0xe53fece9, 52 },
    { 0x8ebfeceb, 61 },
    { 0, 0 },
    { 0xd93feced, 91 },
    { 0, 0 },
    { 0x1a5fecef, 33 },
    { 0xa95fecf0, 34 },
    { 0x95ffecf1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bfecf4, 52 },
    { 0x981fecf5, 95 },
    { 0xf76fecf6, 90 },
    { 0xa99fecf5, 98 },
    { 0x68afecf4, 93 },
    { 0x7e5fecf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118fed00, 65 },
    { 0, 0 },
    { 0x9a9fed02, 95 },
    { 0, 0 },
    { 0x9a9fed04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57fed08, 34 },
    { 0, 0 },
    { 0xcd1fed0a, 61 },
    { 0x5a0fed0b, 61 },
    { 0x687fed0c, 77 },
    { 0xabcfed0c, 33 },
    { 0x52bfed0d, 65 },
    { 0x291fed0c, 98 },
    { 0, 0 },
    { 0x597fed11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fed1c, 33 },
    { 0x2ddfed1c, 58 },
    { 0x3ddfed1e, 66 },
    { 0, 0 },
    { 0x798fed20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdfed23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ffed27, 100 },
    { 0x7ccfed27, 43 },
    { 0xeb3fed27, 76 },
    { 0x816fed2a, 61 },
    { 0x7d9fed2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6fed30, 46 },
    { 0x44fed31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52efed3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1fed3e, 23 },
    { 0x1d1fed3e, 65 },
    { 0x5a4fed40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808fed45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddfed48, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6fed4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x142fed51, 90 },
    { 0xe21fed52, 80 },
    { 0x127fed53, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88fed5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2fed60, 33 },
    { 0, 0 },
    { 0xf6dfed62, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4fed67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9fed6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffed74, 61 },
    { 0, 0 },
    { 0x8e6fed76, 55 },
    { 0x2ddfed76, 33 },
    { 0xa9fed76, 95 },
    { 0xf4dfed79, 87 },
    { 0xd5afed7a, 93 },
    { 0xf1ffed7a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2befed7e, 33 },
    { 0, 0 },
    { 0xdaafed80, 34 },
    { 0xf9ffed80, 61 },
    { 0x190fed81, 90 },
    { 0x5f2fed83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895fed87, 33 },
    { 0x712fed88, 61 },
    { 0x64efed87, 55 },
    { 0x6e9fed8a, 76 },
    { 0x603fed8b, 85 },
    { 0xb66fed8b, 67 },
    { 0x1c7fed87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23fed92, 95 },
    { 0x197fed93, 33 },
    { 0x9cefed92, 23 },
    { 0, 0 },
    { 0x84ffed96, 33 },
    { 0xcc4fed97, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x879fed9a, 33 },
    { 0x66dfed9b, 85 },
    { 0xd2afed9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835feda2, 76 },
    { 0x65efeda3, 33 },
    { 0xe00feda2, 76 },
    { 0x62feda5, 90 },
    { 0, 0 },
    { 0xcd2feda7, 43 },
    { 0x393feda8, 23 },
    { 0x791feda9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dffedb2, 99 },
    { 0xf41fedb3, 93 },
    { 0, 0 },
    { 0x1adfedb5, 90 },
    { 0x479fedb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143fedbb, 84 },
    { 0, 0 },
    { 0x627fedbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90fedc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdafedcb, 23 },
    { 0, 0 },
    { 0xafbfedcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fedd1, 90 },
    { 0x4ebfedd2, 55 },
    { 0, 0 },
    { 0xe65fedd4, 95 },
    { 0, 0 },
    { 0x401fedd6, 85 },
    { 0, 0 },
    { 0xe5efedd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3feddb, 90 },
    { 0, 0 },
    { 0xc45feddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56efede0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fafede3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583fede8, 90 },
    { 0, 0 },
    { 0x3affedea, 61 },
    { 0x1fafedea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187fedef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cfedf2, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefedf6, 81 },
    { 0xccafedf7, 97 },
    { 0xf92fedf7, 36 },
    { 0x7a5fedf9, 98 },
    { 0, 0 },
    { 0x245fedfb, 76 },
    { 0, 0 },
    { 0xa0fedfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8fee01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cfee07, 55 },
    { 0, 0 },
    { 0xa5bfee09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2fee0c, 61 },
    { 0, 0 },
    { 0x8f4fee0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeafee14, 43 },
    { 0xf87fee15, 90 },
    { 0x327fee15, 46 },
    { 0x5b3fee17, 33 },
    { 0xbf8fee16, 100 },
    { 0x9bafee14, 33 },
    { 0, 0 },
    { 0x595fee1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbfee21, 85 },
    { 0xfe2fee22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03fee26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8fee29, 98 },
    { 0xdbffee29, 66 },
    { 0, 0 },
    { 0x801fee2c, 23 },
    { 0x1c5fee2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cfee30, 90 },
    { 0xbabfee31, 55 },
    { 0x71dfee31, 46 },
    { 0xbb5fee33, 78 },
    { 0, 0 },
    { 0xe9cfee35, 78 },
    { 0, 0 },
    { 0xb65fee37, 66 },
    { 0xba6fee38, 33 },
    { 0, 0 },
    { 0x320fee3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9efee3e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fee41, 95 },
    { 0x3c5fee41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bfee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x702fee4a, 85 },
    { 0xa99fee4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12fee50, 61 },
    { 0x13cfee51, 88 },
    { 0, 0 },
    { 0xae0fee53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfee57, 61 },
    { 0x459fee58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ffee64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcafee67, 76 },
    { 0x911fee68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32afee6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fee70, 90 },
    { 0x827fee70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09fee7a, 76 },
    { 0xc09fee7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2fee7f, 61 },
    { 0x305fee7f, 76 },
    { 0x998fee81, 77 },
    { 0, 0 },
    { 0x19efee83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x457fee86, 95 },
    { 0, 0 },
    { 0x64bfee88, 93 },
    { 0x8ddfee89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3fee8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33fee90, 33 },
    { 0, 0 },
    { 0xda2fee92, 90 },
    { 0x47dfee93, 55 },
    { 0x542fee94, 64 },
    { 0x5e9fee95, 53 },
    { 0x272fee96, 95 },
    { 0xee4fee92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fee9d, 23 },
    { 0xcc7fee9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4feea2, 90 },
    { 0xa76feea2, 95 },
    { 0xf3efeea3, 99 },
    { 0x336feea4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddfeea9, 85 },
    { 0x5d5feea9, 75 },
    { 0x9d0feeab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efeeaf, 61 },
    { 0x4afeeb0, 33 },
    { 0x185feeb1, 43 },
    { 0x549feeb1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1efeeb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfeeb9, 33 },
    { 0xd94feeba, 78 },
    { 0x4b6feebb, 34 },
    { 0xe8feeba, 76 },
    { 0x5a1feebd, 90 },
    { 0x8e2feeb9, 90 },
    { 0, 0 },
    { 0xea8feec0, 75 },
    { 0, 0 },
    { 0xa18feec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3feec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3feecc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3efeecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbfeed4, 61 },
    { 0xd2ffeed5, 90 },
    { 0x47feed4, 5 },
    { 0xb98feed7, 98 },
    { 0x4dcfeed8, 61 },
    { 0x9fcfeed8, 57 },
    { 0xf50feeda, 33 },
    { 0x90cfeeda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffeee9, 90 },
    { 0x92efeee9, 99 },
    { 0x84afeeeb, 33 },
    { 0, 0 },
    { 0x824feeed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x970feef0, 33 },
    { 0x94efeef0, 12 },
    { 0xb35feef2, 55 },
    { 0x627feef0, 43 },
    { 0x762feef4, 33 },
    { 0x822feef4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6feef8, 78 },
    { 0x3b3feef9, 77 },
    { 0x9bffeef8, 33 },
    { 0x651feef8, 65 },
    { 0x741feefc, 98 },
    { 0x234feefc, 33 },
    { 0x510feef8, 95 },
    { 0, 0 },
    { 0xa4afef00, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103fef0e, 76 },
    { 0xc01fef0f, 61 },
    { 0, 0 },
    { 0x2b1fef11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cfef15, 93 },
    { 0x5cbfef16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8fef1c, 87 },
    { 0, 0 },
    { 0x97dfef1e, 23 },
    { 0xb8cfef1f, 95 },
    { 0x6dbfef1f, 33 },
    { 0, 0 },
    { 0xcd6fef22, 55 },
    { 0x2effef22, 90 },
    { 0, 0 },
    { 0x940fef25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bfef2c, 90 },
    { 0x864fef2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dfef32, 93 },
    { 0x428fef33, 55 },
    { 0, 0 },
    { 0x382fef35, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ffef39, 87 },
    { 0xcf8fef3a, 61 },
    { 0xe12fef3b, 76 },
    { 0, 0 },
    { 0xe5cfef3d, 85 },
    { 0x97ffef3d, 91 },
    { 0, 0 },
    { 0xbadfef40, 43 },
    { 0xadafef40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7fef4a, 95 },
    { 0, 0 },
    { 0xc79fef4c, 90 },
    { 0, 0 },
    { 0xb35fef4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4fef51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3fef54, 83 },
    { 0, 0 },
    { 0x563fef56, 17 },
    { 0x996fef56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6fef5c, 85 },
    { 0xe14fef5c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5fef64, 76 },
    { 0x2e0fef64, 76 },
    { 0x316fef66, 90 },
    { 0xff1fef67, 33 },
    { 0xffbfef64, 85 },
    { 0xd4afef68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719fef70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1fef73, 56 },
    { 0xa99fef73, 33 },
    { 0x2dfef75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3fef78, 98 },
    { 0, 0 },
    { 0xfb2fef7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x728fef7d, 41 },
    { 0xc7efef7e, 93 },
    { 0xa0dfef7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28fef82, 52 },
    { 0x69dfef83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dfef88, 65 },
    { 0, 0 },
    { 0xfabfef8a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06fef8f, 61 },
    { 0, 0 },
    { 0x11ffef91, 76 },
    { 0x300fef92, 23 },
    { 0x8a1fef91, 69 },
    { 0x9d8fef92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fefa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553fefa9, 61 },
    { 0, 0 },
    { 0x89fefab, 90 },
    { 0x918fefac, 90 },
    { 0xecfefad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5fefb1, 78 },
    { 0x763fefb2, 67 },
    { 0x1abfefb2, 69 },
    { 0x695fefb3, 61 },
    { 0xa87fefb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47fefbb, 95 },
    { 0, 0 },
    { 0x4bcfefbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bffefc0, 95 },
    { 0x314fefc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfefc5, 95 },
    { 0x5cffefc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cfefce, 66 },
    { 0x23afefce, 55 },
    { 0x83ffefce, 90 },
    { 0x151fefd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3befefd4, 90 },
    { 0xe58fefd5, 64 },
    { 0x907fefd6, 99 },
    { 0x18fefd5, 33 },
    { 0, 0 },
    { 0xae7fefd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794fefe4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ffefe7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261fefed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438feff1, 40 },
    { 0x1a0feff1, 90 },
    { 0x902feff3, 19 },
    { 0x3f2feff3, 78 },
    { 0x22afeff5, 78 },
    { 0, 0 },
    { 0x830feff7, 23 },
    { 0x92bfeff7, 61 },
    { 0x78ffeff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eff000, 98 },
    { 0, 0 },
    { 0x28ff002, 33 },
    { 0x9e4ff002, 61 },
    { 0x4abff002, 77 },
    { 0x295ff005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eff009, 85 },
    { 0xd8dff00a, 98 },
    { 0x481ff00b, 17 },
    { 0xbe6ff00c, 90 },
    { 0x267ff00c, 100 },
    { 0x942ff00e, 98 },
    { 0x15ff00f, 90 },
    { 0x6c4ff010, 100 },
    { 0xc39ff011, 33 },
    { 0x3beff00e, 33 },
    { 0xa24ff011, 34 },
    { 0xf0aff014, 95 },
    { 0x3e9ff014, 76 },
    { 0xf35ff016, 90 },
    { 0xb1cff017, 61 },
    { 0xe83ff018, 76 },
    { 0x1a3ff018, 91 },
    { 0x517ff01a, 77 },
    { 0xc72ff01b, 90 },
    { 0xc01ff01b, 81 },
    { 0x282ff01d, 78 },
    { 0x750ff01d, 23 },
    { 0x593ff00f, 100 },
    { 0x432ff020, 90 },
    { 0xbc8ff021, 86 },
    { 0x96bff01d, 91 },
    { 0x60bff01b, 55 },
    { 0x567ff017, 61 },
    { 0x443ff025, 90 },
    { 0xec8ff026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fff02a, 95 },
    { 0, 0 },
    { 0xaeeff02c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fff030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ff03a, 38 },
    { 0xddcff03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9ff043, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17ff046, 93 },
    { 0xed7ff046, 90 },
    { 0, 0 },
    { 0xcfff049, 88 },
    { 0, 0 },
    { 0x460ff04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdff04f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277ff053, 98 },
    { 0xe38ff054, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfff057, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dff05c, 46 },
    { 0x96aff05c, 95 },
    { 0x4e1ff05c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ff066, 95 },
    { 0x4a5ff066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ff06b, 81 },
    { 0x79dff06c, 98 },
    { 0x83ff06d, 84 },
    { 0, 0 },
    { 0x6d6ff06f, 90 },
    { 0, 0 },
    { 0xa44ff071, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0ff074, 61 },
    { 0x77fff075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x611ff078, 61 },
    { 0x7a3ff079, 55 },
    { 0xe07ff07a, 23 },
    { 0xdf9ff07b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ff07f, 55 },
    { 0xe37ff07f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ff083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff08b, 66 },
    { 0x572ff08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cff08f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcff092, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3ff096, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcff099, 90 },
    { 0xc3cff09a, 90 },
    { 0x129ff09b, 78 },
    { 0x658ff09c, 46 },
    { 0x28ff09c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ff0a6, 33 },
    { 0x2c3ff0a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5ff0ab, 78 },
    { 0x823ff0ac, 98 },
    { 0xb3bff0ad, 33 },
    { 0xd7cff0ad, 90 },
    { 0x479ff0af, 98 },
    { 0, 0 },
    { 0x878ff0b1, 98 },
    { 0xa76ff0b2, 91 },
    { 0xe62ff0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ff0b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfff0c2, 43 },
    { 0x56bff0c2, 99 },
    { 0x85aff0c2, 75 },
    { 0xbe4ff0c4, 76 },
    { 0x1ebff0c6, 98 },
    { 0x96bff0c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187ff0cb, 67 },
    { 0x745ff0cc, 23 },
    { 0x37bff0cd, 95 },
    { 0xac7ff0ce, 90 },
    { 0xf5eff0cc, 43 },
    { 0x251ff0d0, 55 },
    { 0, 0 },
    { 0xaf5ff0d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff0d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ff0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feff0dc, 76 },
    { 0xe5fff0dd, 33 },
    { 0x638ff0dc, 90 },
    { 0, 0 },
    { 0xf61ff0e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ff0e4, 33 },
    { 0x305ff0e4, 84 },
    { 0x829ff0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff0ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ff0ee, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cff0f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3ff0f8, 17 },
    { 0xf2dff0f9, 23 },
    { 0xb4ff0fa, 33 },
    { 0x8d2ff0fb, 85 },
    { 0x841ff0fb, 93 },
    { 0x13eff0fc, 33 },
    { 0xdfaff0fa, 90 },
    { 0x580ff0ff, 33 },
    { 0x75bff100, 68 },
    { 0x290ff101, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ff105, 95 },
    { 0x41fff106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6ff109, 100 },
    { 0x1e7ff10a, 33 },
    { 0xeb6ff10a, 36 },
    { 0xa4cff10a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122ff116, 62 },
    { 0x162ff117, 12 },
    { 0xb18ff117, 90 },
    { 0x8a1ff117, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3ff11c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ff122, 34 },
    { 0x9c8ff123, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eff12b, 23 },
    { 0x42aff12b, 55 },
    { 0, 0 },
    { 0xae4ff12e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7ff131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307ff137, 43 },
    { 0x4abff138, 23 },
    { 0xcf8ff139, 55 },
    { 0xb51ff13a, 34 },
    { 0x487ff13b, 85 },
    { 0xdb1ff13c, 23 },
    { 0x177ff13a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff149, 98 },
    { 0x738ff149, 38 },
    { 0xf8ff14b, 23 },
    { 0, 0 },
    { 0x1a4ff14d, 33 },
    { 0, 0 },
    { 0xd5cff14f, 55 },
    { 0x88cff150, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bff155, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccff15b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x886ff15e, 34 },
    { 0xdd4ff15f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4ff167, 99 },
    { 0, 0 },
    { 0x368ff169, 100 },
    { 0x30dff16a, 61 },
    { 0, 0 },
    { 0xf99ff16c, 98 },
    { 0xbd3ff16d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fff174, 90 },
    { 0x2bdff175, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x455ff178, 95 },
    { 0x4bff179, 98 },
    { 0, 0 },
    { 0xf5eff17b, 95 },
    { 0x728ff17c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ff17f, 61 },
    { 0xbaff180, 98 },
    { 0xe36ff181, 84 },
    { 0x5d0ff182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7ff186, 36 },
    { 0x495ff186, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ff18a, 23 },
    { 0xd80ff18b, 90 },
    { 0xc64ff18c, 98 },
    { 0, 0 },
    { 0x6b6ff18e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ff191, 95 },
    { 0, 0 },
    { 0xf19ff193, 81 },
    { 0, 0 },
    { 0xcb5ff195, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926ff19c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adff19f, 90 },
    { 0xac3ff19f, 95 },
    { 0xef9ff1a1, 98 },
    { 0x667ff1a2, 76 },
    { 0x13aff1a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ff1a6, 23 },
    { 0, 0 },
    { 0xe43ff1a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99ff1ac, 81 },
    { 0xb4ff1ad, 33 },
    { 0xc7aff1ad, 34 },
    { 0, 0 },
    { 0xaf1ff1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ff1b3, 33 },
    { 0xc91ff1b4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9ff1b8, 33 },
    { 0xe95ff1b9, 88 },
    { 0x6d9ff1ba, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9ff1bf, 85 },
    { 0, 0 },
    { 0xaf0ff1c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5ff1c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400ff1ca, 61 },
    { 0x6eeff1ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2ff1d0, 61 },
    { 0x492ff1d1, 95 },
    { 0, 0 },
    { 0xf6bff1d3, 52 },
    { 0x940ff1d3, 90 },
    { 0xba5ff1d5, 81 },
    { 0, 0 },
    { 0x9f1ff1d7, 91 },
    { 0xbd7ff1d8, 93 },
    { 0x85dff1d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7ff1de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deff1ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ff1f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fff1fb, 77 },
    { 0xa96ff1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cff200, 55 },
    { 0, 0 },
    { 0x879ff202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5eff205, 76 },
    { 0x9e1ff206, 76 },
    { 0, 0 },
    { 0x73aff208, 93 },
    { 0, 0 },
    { 0xa70ff20a, 23 },
    { 0, 0 },
    { 0xeb3ff20c, 61 },
    { 0, 0 },
    { 0x1c0ff20e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ff214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x739ff217, 84 },
    { 0x26aff218, 33 },
    { 0x39aff219, 93 },
    { 0xf8aff218, 90 },
    { 0xfb5ff21b, 90 },
    { 0xcd9ff219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ff221, 33 },
    { 0x9e7ff222, 65 },
    { 0x625ff222, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aff226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aff22c, 33 },
    { 0, 0 },
    { 0x9e0ff22e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ff235, 78 },
    { 0x9edff236, 74 },
    { 0, 0 },
    { 0xe57ff238, 90 },
    { 0xc1aff239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91ff23d, 78 },
    { 0xee1ff23e, 43 },
    { 0x8c8ff23d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ff243, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaff246, 90 },
    { 0, 0 },
    { 0x426ff248, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ff24b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97ff253, 23 },
    { 0x76bff253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cff25b, 43 },
    { 0x5c2ff25c, 33 },
    { 0, 0 },
    { 0x55bff25e, 33 },
    { 0xc63ff25f, 100 },
    { 0x85cff25f, 95 },
    { 0x6c9ff25f, 55 },
    { 0xcd5ff262, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ff265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aff268, 65 },
    { 0x3d9ff269, 78 },
    { 0x10cff26a, 100 },
    { 0x276ff26b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edff279, 61 },
    { 0xb18ff27a, 76 },
    { 0, 0 },
    { 0x53cff27c, 34 },
    { 0x318ff27d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff280, 61 },
    { 0xd75ff280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3ff284, 90 },
    { 0, 0 },
    { 0x6caff286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ff28a, 34 },
    { 0x717ff28b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87ff294, 33 },
    { 0, 0 },
    { 0x3aeff296, 98 },
    { 0x262ff297, 78 },
    { 0x3c3ff298, 66 },
    { 0xb2aff296, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeff29e, 76 },
    { 0x52ff29f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdff2a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993ff2b1, 33 },
    { 0x413ff2b2, 90 },
    { 0x862ff2b2, 90 },
    { 0x24dff2b4, 23 },
    { 0, 0 },
    { 0x1c5ff2b6, 67 },
    { 0xe85ff2b6, 91 },
    { 0xaa9ff2b8, 23 },
    { 0xd9fff2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eff2bc, 95 },
    { 0xfcbff2bd, 90 },
    { 0, 0 },
    { 0x49dff2bf, 95 },
    { 0x56cff2c0, 98 },
    { 0xcc8ff2c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dff2c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81ff2c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ff2cc, 95 },
    { 0xd55ff2cd, 90 },
    { 0xb2ff2cd, 90 },
    { 0xcc5ff2cd, 33 },
    { 0x476ff2ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1ff2d5, 98 },
    { 0, 0 },
    { 0x9a4ff2d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ff2dc, 33 },
    { 0, 0 },
    { 0x22eff2de, 95 },
    { 0x8e8ff2df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ff2e3, 98 },
    { 0, 0 },
    { 0xbd2ff2e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaff2e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1ff2ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ff2f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94ff2fa, 90 },
    { 0x121ff2fb, 61 },
    { 0, 0 },
    { 0x988ff2fd, 77 },
    { 0xb85ff2fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ff301, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eff305, 86 },
    { 0x12fff306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9ff30b, 46 },
    { 0, 0 },
    { 0xd89ff30d, 91 },
    { 0xd61ff30d, 37 },
    { 0x677ff30f, 34 },
    { 0xe41ff310, 34 },
    { 0, 0 },
    { 0xe34ff312, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ff316, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x226ff319, 34 },
    { 0xdedff319, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ff320, 34 },
    { 0, 0 },
    { 0xb7cff322, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fff326, 90 },
    { 0xb35ff326, 55 },
    { 0x46bff328, 95 },
    { 0x705ff329, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ff32d, 33 },
    { 0xc34ff32d, 95 },
    { 0x530ff32f, 98 },
    { 0x6f6ff32f, 33 },
    { 0x3e9ff331, 99 },
    { 0xc15ff331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568ff336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eff339, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16ff344, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ff347, 46 },
    { 0x691ff348, 17 },
    { 0x294ff347, 61 },
    { 0x727ff34a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabff350, 91 },
    { 0, 0 },
    { 0x108ff352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edff355, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ff35a, 98 },
    { 0x996ff35b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfff360, 61 },
    { 0x912ff361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756ff365, 76 },
    { 0xb04ff366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7ff36b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x739ff36e, 61 },
    { 0xd70ff36f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679ff375, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fff378, 35 },
    { 0, 0 },
    { 0xea6ff37a, 42 },
    { 0xb41ff37a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff37e, 95 },
    { 0xf1cff37e, 76 },
    { 0xc81ff380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdff383, 23 },
    { 0, 0 },
    { 0x4cff385, 99 },
    { 0x6d1ff386, 90 },
    { 0, 0 },
    { 0x7aaff388, 87 },
    { 0xc0dff389, 61 },
    { 0, 0 },
    { 0xd6bff38b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0ff38e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff392, 98 },
    { 0, 0 },
    { 0x9dcff394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ff399, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ff39d, 33 },
    { 0xaa8ff39e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6ff3a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ff3a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ff3aa, 91 },
    { 0xe35ff3aa, 61 },
    { 0, 0 },
    { 0x21aff3ad, 76 },
    { 0xddbff3ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6ff3b1, 90 },
    { 0x4edff3b1, 52 },
    { 0xd0eff3b3, 84 },
    { 0x22fff3b4, 34 },
    { 0, 0 },
    { 0xf61ff3b6, 33 },
    { 0x375ff3b6, 34 },
    { 0x485ff3b8, 76 },
    { 0x86cff3b9, 87 },
    { 0x8b9ff3b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deff3be, 34 },
    { 0xe79ff3bf, 33 },
    { 0x659ff3c0, 55 },
    { 0, 0 },
    { 0x1ffff3c2, 86 },
    { 0x45aff3c2, 23 },
    { 0x7a3ff3c4, 67 },
    { 0x3d7ff3c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256ff3ca, 90 },
    { 0, 0 },
    { 0x5b2ff3cc, 61 },
    { 0, 0 },
    { 0xaeaff3ce, 76 },
    { 0, 0 },
    { 0x19dff3d0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ff3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ff3d6, 95 },
    { 0x378ff3d7, 93 },
    { 0x4e3ff3d7, 55 },
    { 0xed1ff3d9, 93 },
    { 0xa05ff3d6, 95 },
    { 0x9d8ff3db, 23 },
    { 0x656ff3d7, 34 },
    { 0xda8ff3db, 95 },
    { 0xeefff3de, 90 },
    { 0xb90ff3df, 98 },
    { 0x1a8ff3de, 36 },
    { 0x25cff3e1, 61 },
    { 0, 0 },
    { 0xeceff3e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500ff3e9, 36 },
    { 0x453ff3ea, 98 },
    { 0xf77ff3e9, 90 },
    { 0xb14ff3ec, 101 },
    { 0xdf2ff3e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eff3f1, 24 },
    { 0, 0 },
    { 0xccff3f3, 33 },
    { 0xde9ff3f4, 61 },
    { 0, 0 },
    { 0x6a7ff3f6, 34 },
    { 0, 0 },
    { 0xf3bff3f8, 80 },
    { 0x43dff3f9, 99 },
    { 0x9ebff3f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ff400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61ff40b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377ff40f, 23 },
    { 0xa13ff410, 23 },
    { 0, 0 },
    { 0x6b4ff412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ff41a, 34 },
    { 0, 0 },
    { 0x34dff41c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562ff420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23ff423, 95 },
    { 0xe79ff423, 55 },
    { 0x3fdff425, 33 },
    { 0x2bcff426, 61 },
    { 0x33eff423, 23 },
    { 0x6c8ff427, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fff42b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ff42e, 95 },
    { 0x176ff42e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7ff432, 33 },
    { 0xd4dff432, 52 },
    { 0xa32ff434, 33 },
    { 0, 0 },
    { 0xffbff436, 52 },
    { 0xb89ff437, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaff43c, 90 },
    { 0xd69ff43d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ff441, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afff446, 98 },
    { 0x7fff447, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04ff453, 96 },
    { 0x25eff453, 95 },
    { 0xfafff454, 55 },
    { 0xacfff455, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738ff45b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fff465, 90 },
    { 0xc0ff466, 67 },
    { 0xf5cff466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dff46c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4ff471, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddff474, 95 },
    { 0x16ff475, 55 },
    { 0x199ff476, 61 },
    { 0x142ff477, 46 },
    { 0x609ff474, 95 },
    { 0x5cff479, 76 },
    { 0, 0 },
    { 0x300ff47b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edff47f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fff482, 33 },
    { 0x2edff483, 55 },
    { 0xf21ff484, 90 },
    { 0x763ff485, 90 },
    { 0xa9fff486, 33 },
    { 0x8daff486, 90 },
    { 0x737ff486, 66 },
    { 0, 0 },
    { 0x9efff48a, 55 },
    { 0x637ff48b, 90 },
    { 0xf93ff48c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ff492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31ff497, 98 },
    { 0x7acff498, 23 },
    { 0, 0 },
    { 0xb4eff49a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ff49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33ff4a2, 98 },
    { 0xc49ff4a3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dff4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ff4b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ff4bd, 90 },
    { 0x92ff4bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ff4c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdff4c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdff4c9, 98 },
    { 0, 0 },
    { 0x816ff4cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ff4d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ff4d9, 66 },
    { 0xfeaff4da, 61 },
    { 0xc3cff4da, 61 },
    { 0x927ff4dc, 95 },
    { 0x1cfff4dd, 23 },
    { 0, 0 },
    { 0xae1ff4df, 39 },
    { 0x4a0ff4e0, 55 },
    { 0, 0 },
    { 0xb19ff4e2, 23 },
    { 0, 0 },
    { 0x805ff4e4, 90 },
    { 0xa1bff4e4, 90 },
    { 0, 0 },
    { 0xa21ff4e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46ff4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacff4f3, 95 },
    { 0, 0 },
    { 0xa34ff4f5, 61 },
    { 0, 0 },
    { 0xbacff4f7, 61 },
    { 0, 0 },
    { 0x199ff4f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0ff4fc, 15 },
    { 0, 0 },
    { 0xeeaff4fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464ff509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccff50c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff50f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x872ff512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ff517, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ff51b, 61 },
    { 0xf5aff51b, 56 },
    { 0xb8fff51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cff520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0ff523, 76 },
    { 0xebdff523, 61 },
    { 0xdc6ff524, 95 },
    { 0xdfcff523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eff52a, 67 },
    { 0x906ff52b, 61 },
    { 0x597ff52c, 53 },
    { 0, 0 },
    { 0xf36ff52e, 90 },
    { 0, 0 },
    { 0xdacff530, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78ff53a, 36 },
    { 0xbd3ff53a, 90 },
    { 0xa73ff53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ff54a, 61 },
    { 0xcb2ff54b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41aff551, 93 },
    { 0x287ff551, 95 },
    { 0xbd8ff553, 33 },
    { 0, 0 },
    { 0x647ff555, 33 },
    { 0xab9ff556, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ff566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ff571, 34 },
    { 0x3a2ff572, 55 },
    { 0xeaeff571, 61 },
    { 0x1e3ff572, 55 },
    { 0xd87ff575, 95 },
    { 0xcd7ff575, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ff57d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ff581, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bff586, 88 },
    { 0xc34ff587, 46 },
    { 0xc18ff587, 61 },
    { 0xe06ff587, 34 },
    { 0xf37ff58a, 36 },
    { 0xc92ff586, 90 },
    { 0x3aff58c, 68 },
    { 0xa8dff58a, 90 },
    { 0x80cff58c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ff596, 90 },
    { 0x957ff597, 90 },
    { 0x306ff596, 90 },
    { 0x44bff598, 95 },
    { 0xa1dff598, 99 },
    { 0x3d4ff596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aff5a9, 95 },
    { 0x3aeff5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0ff5ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dff5b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486ff5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922ff5be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ff5c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ff5ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ff5d7, 90 },
    { 0x520ff5d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ff5db, 59 },
    { 0x73dff5db, 90 },
    { 0x4fbff5dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aff5e6, 90 },
    { 0x57ff5e7, 90 },
    { 0x4c4ff5e8, 61 },
    { 0x494ff5e6, 43 },
    { 0xf89ff5e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ff5ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bff5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffff5fb, 76 },
    { 0x307ff5fb, 98 },
    { 0, 0 },
    { 0x533ff5fe, 23 },
    { 0, 0 },
    { 0x3b1ff600, 33 },
    { 0, 0 },
    { 0x19bff602, 43 },
    { 0x57fff603, 93 },
    { 0x92cff603, 61 },
    { 0x763ff604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bff608, 78 },
    { 0xad7ff609, 55 },
    { 0x188ff608, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ff60d, 61 },
    { 0x491ff60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedff611, 33 },
    { 0, 0 },
    { 0x2d9ff613, 85 },
    { 0xdcdff614, 66 },
    { 0xb63ff613, 23 },
    { 0xfb6ff616, 95 },
    { 0xa5ff613, 23 },
    { 0, 0 },
    { 0xb69ff619, 87 },
    { 0xf9cff619, 98 },
    { 0xd1fff61b, 98 },
    { 0xe14ff61b, 23 },
    { 0x28bff61c, 93 },
    { 0x57bff61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ff624, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ff62c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbff630, 91 },
    { 0x8d1ff631, 90 },
    { 0x6d1ff630, 37 },
    { 0x2b4ff633, 34 },
    { 0xf1dff634, 93 },
    { 0, 0 },
    { 0x44ff636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x873ff639, 95 },
    { 0x5b0ff63a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cff63d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fff643, 81 },
    { 0x190ff644, 52 },
    { 0x947ff644, 78 },
    { 0x381ff646, 88 },
    { 0xab9ff647, 34 },
    { 0xfccff646, 90 },
    { 0xd3aff649, 95 },
    { 0x9bcff644, 95 },
    { 0xf75ff64b, 23 },
    { 0x94bff643, 65 },
    { 0x4b4ff645, 95 },
    { 0xbe1ff64c, 66 },
    { 0, 0 },
    { 0xe02ff650, 98 },
    { 0x942ff650, 76 },
    { 0x94dff651, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375ff656, 33 },
    { 0, 0 },
    { 0x230ff658, 61 },
    { 0, 0 },
    { 0x33dff65a, 90 },
    { 0xaa5ff65a, 61 },
    { 0, 0 },
    { 0x9b9ff65d, 97 },
    { 0, 0 },
    { 0x792ff65f, 90 },
    { 0xd15ff660, 76 },
    { 0xd1eff661, 23 },
    { 0, 0 },
    { 0x775ff663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff666, 61 },
    { 0xef9ff666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dff66d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99ff672, 61 },
    { 0x7aff673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ff676, 46 },
    { 0x4b0ff677, 23 },
    { 0, 0 },
    { 0xfebff679, 67 },
    { 0xbddff67a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4ff67e, 34 },
    { 0x17cff67e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aff682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bff685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aff68b, 91 },
    { 0, 0 },
    { 0x675ff68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bff694, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9ff69a, 90 },
    { 0x6f2ff69b, 98 },
    { 0x8f6ff69c, 98 },
    { 0x633ff69d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ff6aa, 46 },
    { 0x426ff6ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff6af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ff6b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ff6b8, 95 },
    { 0xfe1ff6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aff6bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ff6c5, 46 },
    { 0x70ff6c6, 81 },
    { 0x5a8ff6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff6d3, 99 },
    { 0xb9aff6d4, 43 },
    { 0x905ff6d5, 61 },
    { 0xe5eff6d6, 67 },
    { 0x5c8ff6d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eff6dd, 33 },
    { 0xc7bff6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bff6e1, 43 },
    { 0, 0 },
    { 0xea0ff6e3, 90 },
    { 0x31aff6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ff6ea, 61 },
    { 0, 0 },
    { 0x2fcff6ec, 90 },
    { 0xfbbff6ec, 93 },
    { 0xa2fff6ee, 52 },
    { 0, 0 },
    { 0x943ff6f0, 33 },
    { 0, 0 },
    { 0x66ff6f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cff6f8, 61 },
    { 0x98dff6f9, 90 },
    { 0, 0 },
    { 0x4dfff6fb, 36 },
    { 0xde9ff6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315ff701, 33 },
    { 0x49cff702, 61 },
    { 0, 0 },
    { 0x37cff704, 55 },
    { 0, 0 },
    { 0x197ff706, 33 },
    { 0x9ffff706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaff70a, 99 },
    { 0, 0 },
    { 0xea5ff70c, 61 },
    { 0x70fff70d, 55 },
    { 0x7faff70e, 34 },
    { 0x3d0ff70f, 100 },
    { 0x2bcff710, 43 },
    { 0x8f1ff711, 77 },
    { 0x69ff711, 77 },
    { 0x3b6ff70f, 90 },
    { 0xbcff714, 90 },
    { 0xd6bff70d, 33 },
    { 0x14eff716, 98 },
    { 0, 0 },
    { 0x401ff718, 61 },
    { 0x6faff718, 61 },
    { 0xb6fff71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606ff71d, 61 },
    { 0x678ff71e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621ff722, 90 },
    { 0x674ff723, 33 },
    { 0xf0cff724, 91 },
    { 0x3deff725, 90 },
    { 0xaeff726, 61 },
    { 0xadbff724, 100 },
    { 0, 0 },
    { 0x2acff729, 66 },
    { 0x8e6ff72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cff72e, 87 },
    { 0x305ff72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47ff734, 100 },
    { 0, 0 },
    { 0x5dff736, 33 },
    { 0, 0 },
    { 0x3adff738, 33 },
    { 0x26fff739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedff73d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dff742, 97 },
    { 0, 0 },
    { 0xf85ff744, 61 },
    { 0, 0 },
    { 0xe76ff746, 67 },
    { 0x1adff746, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420ff74d, 90 },
    { 0x40cff74e, 76 },
    { 0x72fff74e, 81 },
    { 0x584ff750, 90 },
    { 0, 0 },
    { 0xc28ff752, 90 },
    { 0x22ff753, 43 },
    { 0, 0 },
    { 0x8c6ff755, 67 },
    { 0x12ff756, 95 },
    { 0, 0 },
    { 0xffdff758, 98 },
    { 0xd2aff759, 55 },
    { 0xbb0ff759, 61 },
    { 0, 0 },
    { 0xc6ff75c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bff75f, 90 },
    { 0, 0 },
    { 0xfe0ff761, 61 },
    { 0x633ff761, 12 },
    { 0, 0 },
    { 0xc45ff764, 76 },
    { 0xa65ff764, 90 },
    { 0xadeff766, 43 },
    { 0xeb1ff766, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff76c, 98 },
    { 0xf6bff76c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaff770, 33 },
    { 0x97dff771, 90 },
    { 0xd00ff772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eff777, 90 },
    { 0x344ff778, 77 },
    { 0, 0 },
    { 0xef0ff77a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cff781, 33 },
    { 0, 0 },
    { 0xb49ff783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7ff786, 95 },
    { 0, 0 },
    { 0x1abff788, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ff78d, 55 },
    { 0x67dff78e, 33 },
    { 0xdd7ff78d, 98 },
    { 0x85ff790, 67 },
    { 0x364ff791, 46 },
    { 0x8ffff792, 76 },
    { 0x215ff792, 76 },
    { 0xbdaff791, 67 },
    { 0x765ff793, 95 },
    { 0x2f3ff793, 87 },
    { 0xbc5ff793, 33 },
    { 0x410ff792, 34 },
    { 0x2c7ff799, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaff7a6, 90 },
    { 0x187ff7a7, 90 },
    { 0x8d4ff7a8, 90 },
    { 0x5dcff7a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ff7ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fff7b1, 98 },
    { 0xe8aff7b2, 33 },
    { 0x303ff7b1, 64 },
    { 0, 0 },
    { 0x934ff7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cff7c1, 91 },
    { 0x444ff7c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ff7ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ff7d0, 23 },
    { 0x2cff7d1, 100 },
    { 0xf9dff7d2, 88 },
    { 0x2b1ff7d3, 66 },
    { 0xeb7ff7d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ff7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efff7e0, 90 },
    { 0, 0 },
    { 0x365ff7e2, 90 },
    { 0x5a2ff7e2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237ff7e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601ff7f3, 76 },
    { 0x184ff7f4, 95 },
    { 0x557ff7f5, 61 },
    { 0x92fff7f5, 34 },
    { 0xef8ff7f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96ff7fe, 78 },
    { 0xaafff7ff, 61 },
    { 0xfff7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bff804, 33 },
    { 0x27dff805, 76 },
    { 0, 0 },
    { 0xba3ff807, 95 },
    { 0x580ff808, 74 },
    { 0x22bff809, 88 },
    { 0x649ff808, 78 },
    { 0x43ff809, 100 },
    { 0xc04ff80c, 61 },
    { 0x2b1ff80d, 90 },
    { 0x1d2ff80d, 93 },
    { 0x11bff80c, 85 },
    { 0x8fbff80c, 33 },
    { 0xce4ff80d, 90 },
    { 0xf31ff811, 33 },
    { 0x2ccff813, 66 },
    { 0xccfff814, 98 },
    { 0xb69ff815, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ff81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff820, 90 },
    { 0, 0 },
    { 0xc60ff822, 66 },
    { 0, 0 },
    { 0x785ff824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ff82b, 46 },
    { 0xd8ff82b, 78 },
    { 0x479ff82b, 98 },
    { 0x41fff82e, 46 },
    { 0x52ff82e, 52 },
    { 0, 0 },
    { 0xc33ff831, 46 },
    { 0, 0 },
    { 0xd1bff833, 33 },
    { 0, 0 },
    { 0x9aeff835, 98 },
    { 0xa00ff835, 98 },
    { 0x406ff836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fff83c, 33 },
    { 0xcb3ff83c, 75 },
    { 0x89ff83e, 84 },
    { 0x7dcff83f, 83 },
    { 0x1eeff840, 33 },
    { 0xbe3ff841, 61 },
    { 0xc89ff83e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87ff84c, 90 },
    { 0, 0 },
    { 0xf48ff84e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cff851, 88 },
    { 0xeccff851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2ff856, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baff860, 67 },
    { 0x77dff860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ff864, 85 },
    { 0x9fdff865, 61 },
    { 0x759ff866, 95 },
    { 0xb68ff864, 100 },
    { 0x367ff868, 23 },
    { 0, 0 },
    { 0xac9ff86a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x540ff86d, 93 },
    { 0x8afff86e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4ff872, 90 },
    { 0x3ceff872, 87 },
    { 0x25ff874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfff877, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecff882, 87 },
    { 0xadff882, 95 },
    { 0x35ff884, 55 },
    { 0x1eaff883, 46 },
    { 0x39eff886, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cff88c, 76 },
    { 0x2ceff88d, 43 },
    { 0xc6dff88d, 55 },
    { 0, 0 },
    { 0xa1bff890, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377ff895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9ff898, 91 },
    { 0xe7dff899, 46 },
    { 0xc3cff89a, 40 },
    { 0x778ff89a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ff89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaff8a6, 100 },
    { 0x748ff8a6, 66 },
    { 0x5a4ff8a7, 61 },
    { 0x7dff8a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ff8ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18ff8af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3ff8b5, 69 },
    { 0, 0 },
    { 0xabfff8b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaff8bb, 90 },
    { 0x4e0ff8bb, 52 },
    { 0x508ff8bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40ff8c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333ff8c8, 98 },
    { 0x55eff8c8, 33 },
    { 0, 0 },
    { 0x4bfff8cb, 33 },
    { 0, 0 },
    { 0x8d0ff8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ff8d2, 98 },
    { 0x73fff8d3, 90 },
    { 0xd46ff8d4, 98 },
    { 0x7b7ff8d5, 76 },
    { 0xcd1ff8d5, 100 },
    { 0x339ff8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fff8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4ff8e2, 98 },
    { 0x230ff8e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabff8e9, 77 },
    { 0x70dff8ea, 34 },
    { 0x448ff8e9, 98 },
    { 0x174ff8ec, 90 },
    { 0x561ff8ec, 55 },
    { 0, 0 },
    { 0x9eeff8ef, 66 },
    { 0xbb1ff8f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfff8f3, 52 },
    { 0x821ff8f4, 33 },
    { 0x296ff8f5, 33 },
    { 0xaf0ff8f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8ff8fb, 95 },
    { 0, 0 },
    { 0x5cff8fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2ff900, 33 },
    { 0xfa3ff901, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cff908, 33 },
    { 0x27cff909, 61 },
    { 0x3dff90a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acff910, 33 },
    { 0x2f5ff910, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351ff915, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49ff919, 94 },
    { 0xe85ff91a, 61 },
    { 0x31ff91b, 33 },
    { 0x68bff91a, 61 },
    { 0x43aff91d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ff924, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aff927, 34 },
    { 0xf23ff928, 52 },
    { 0, 0 },
    { 0xcd3ff92a, 34 },
    { 0xe93ff92b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5ff931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ff936, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ff93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x271ff93f, 91 },
    { 0, 0 },
    { 0xb83ff941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceff944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x615ff947, 98 },
    { 0, 0 },
    { 0xffcff949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ff94c, 76 },
    { 0x13fff94d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8ff953, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9ff95a, 23 },
    { 0x4e0ff95b, 34 },
    { 0xe67ff95c, 90 },
    { 0x96aff95a, 61 },
    { 0xde5ff95e, 33 },
    { 0, 0 },
    { 0xc8fff960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cff965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ff969, 95 },
    { 0, 0 },
    { 0x55ff96b, 61 },
    { 0xf0dff96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff97a, 23 },
    { 0x1c2ff97a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceff97f, 23 },
    { 0, 0 },
    { 0x8e9ff981, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fff984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840ff98a, 99 },
    { 0xd8cff98b, 33 },
    { 0xbb2ff98c, 84 },
    { 0xb2fff98c, 61 },
    { 0xf2ff98d, 76 },
    { 0, 0 },
    { 0x7d3ff990, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75ff9a4, 61 },
    { 0xb75ff9a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1ff9a8, 23 },
    { 0xcfaff9a8, 93 },
    { 0x87fff9aa, 97 },
    { 0x575ff9aa, 33 },
    { 0x81fff9ac, 98 },
    { 0xa4bff9a9, 76 },
    { 0xb0fff9aa, 98 },
    { 0xb94ff9ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dff9b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44ff9bc, 90 },
    { 0x8caff9bd, 47 },
    { 0x2d2ff9be, 33 },
    { 0xd15ff9bc, 90 },
    { 0x436ff9c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5ff9c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbff9cc, 67 },
    { 0xa9bff9cd, 23 },
    { 0xd30ff9cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55ff9d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcff9d6, 90 },
    { 0x83aff9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dff9da, 40 },
    { 0x6ddff9db, 61 },
    { 0x36aff9da, 98 },
    { 0x9ff9dd, 95 },
    { 0x86ff9de, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfff9e4, 90 },
    { 0, 0 },
    { 0xd49ff9e6, 87 },
    { 0, 0 },
    { 0x844ff9e8, 61 },
    { 0x233ff9e8, 33 },
    { 0x48bff9e8, 95 },
    { 0x7edff9eb, 61 },
    { 0xe66ff9ec, 90 },
    { 0x723ff9ed, 34 },
    { 0, 0 },
    { 0x115ff9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fff9f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff9f8, 95 },
    { 0, 0 },
    { 0xf7aff9fa, 90 },
    { 0xa0bff9fb, 33 },
    { 0, 0 },
    { 0x8beff9fd, 100 },
    { 0xa4bff9fe, 61 },
    { 0x6b3ff9ff, 33 },
    { 0xc3affa00, 33 },
    { 0x2b1ffa01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17effa05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139ffa0b, 90 },
    { 0x552ffa0c, 96 },
    { 0, 0 },
    { 0xe3effa0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86ffa11, 91 },
    { 0xb67ffa11, 61 },
    { 0x683ffa13, 84 },
    { 0xee8ffa14, 23 },
    { 0x246ffa15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4ffa1f, 52 },
    { 0xa50ffa20, 23 },
    { 0xfc2ffa21, 61 },
    { 0, 0 },
    { 0xc3ffa23, 66 },
    { 0, 0 },
    { 0xc0bffa25, 95 },
    { 0, 0 },
    { 0x290ffa27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02ffa2b, 33 },
    { 0x99bffa2c, 99 },
    { 0x1f0ffa2b, 90 },
    { 0xdbbffa2e, 23 },
    { 0x5b4ffa2d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151ffa37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ffa3e, 98 },
    { 0xc5fffa3f, 87 },
    { 0xdbbffa3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dffa44, 52 },
    { 0x276ffa44, 38 },
    { 0x3d4ffa46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ffa4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0ffa4e, 33 },
    { 0, 0 },
    { 0x89ffa50, 90 },
    { 0x4c4ffa50, 90 },
    { 0xc9effa52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72ffa55, 31 },
    { 0xa0effa56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dffa5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaffa5e, 23 },
    { 0x457ffa5f, 33 },
    { 0xaa4ffa60, 43 },
    { 0, 0 },
    { 0x4a8ffa62, 84 },
    { 0, 0 },
    { 0xa43ffa64, 95 },
    { 0, 0 },
    { 0x1ffa66, 77 },
    { 0x159ffa67, 61 },
    { 0x1daffa68, 95 },
    { 0x533ffa69, 61 },
    { 0x146ffa69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85ffa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865ffa74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905ffa7b, 83 },
    { 0x53cffa7c, 77 },
    { 0, 0 },
    { 0xe3fffa7e, 34 },
    { 0x706ffa7e, 96 },
    { 0x4bcffa7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ffa83, 23 },
    { 0x6a9ffa83, 55 },
    { 0, 0 },
    { 0x485ffa86, 33 },
    { 0, 0 },
    { 0x723ffa88, 98 },
    { 0xd5dffa89, 91 },
    { 0, 0 },
    { 0xda7ffa8b, 61 },
    { 0xc09ffa8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524ffa90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcffa94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86effa99, 78 },
    { 0xd85ffa99, 46 },
    { 0xd81ffa9b, 90 },
    { 0x8fcffa9b, 63 },
    { 0x4faffa9a, 61 },
    { 0x469ffa9e, 99 },
    { 0, 0 },
    { 0xef2ffaa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2effaa8, 23 },
    { 0x83ffaa9, 91 },
    { 0x348ffaaa, 61 },
    { 0, 0 },
    { 0x845ffaac, 90 },
    { 0, 0 },
    { 0xb52ffaae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ffab1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9ffab7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ffabd, 61 },
    { 0xc0effabe, 52 },
    { 0x263ffabe, 95 },
    { 0x665ffabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceffac5, 66 },
    { 0x1f1ffac5, 33 },
    { 0xc9fffac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451ffacd, 66 },
    { 0, 0 },
    { 0x4c7ffacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36ffad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1ffad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ffadd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ffae0, 61 },
    { 0, 0 },
    { 0xd32ffae2, 65 },
    { 0, 0 },
    { 0xcf3ffae4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849ffaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddffaf2, 90 },
    { 0x46dffaf3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeffafa, 43 },
    { 0xf8bffafa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ffb0b, 23 },
    { 0x10fffb0c, 43 },
    { 0xe52ffb0c, 93 },
    { 0x31ffb0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ffb11, 69 },
    { 0, 0 },
    { 0xf5affb13, 76 },
    { 0, 0 },
    { 0xff1ffb15, 93 },
    { 0x8a9ffb15, 33 },
    { 0x5f2ffb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ffb1a, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778ffb1f, 65 },
    { 0xcd3ffb1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfffb27, 95 },
    { 0x419ffb27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7ffb2f, 84 },
    { 0, 0 },
    { 0xeedffb31, 23 },
    { 0x804ffb32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x210ffb35, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ffb3d, 90 },
    { 0, 0 },
    { 0x4bffb3f, 76 },
    { 0, 0 },
    { 0xac7ffb41, 95 },
    { 0, 0 },
    { 0xb1cffb43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ffb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfffb4c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6ffb50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64ffb53, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437ffb61, 33 },
    { 0, 0 },
    { 0xb3cffb63, 98 },
    { 0x897ffb64, 95 },
    { 0xbc0ffb65, 95 },
    { 0x6d5ffb66, 17 },
    { 0, 0 },
    { 0xa4effb68, 66 },
    { 0, 0 },
    { 0x9a0ffb6a, 90 },
    { 0x119ffb6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ffb6e, 100 },
    { 0xf95ffb6f, 77 },
    { 0xf7effb6e, 90 },
    { 0xf06ffb6f, 85 },
    { 0x278ffb71, 43 },
    { 0, 0 },
    { 0x969ffb74, 83 },
    { 0x71fffb75, 55 },
    { 0x17affb74, 43 },
    { 0x123ffb75, 33 },
    { 0x982ffb75, 90 },
    { 0x130ffb78, 84 },
    { 0xa11ffb7a, 33 },
    { 0xf73ffb79, 91 },
    { 0xe19ffb78, 85 },
    { 0xb4affb78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165ffb81, 95 },
    { 0x448ffb82, 67 },
    { 0, 0 },
    { 0xf38ffb84, 100 },
    { 0, 0 },
    { 0xc82ffb86, 76 },
    { 0, 0 },
    { 0xfa0ffb88, 55 },
    { 0xc39ffb89, 46 },
    { 0x51cffb89, 33 },
    { 0xe17ffb89, 76 },
    { 0xc51ffb8c, 98 },
    { 0x684ffb8d, 98 },
    { 0x6abffb8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dffb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9affb96, 61 },
    { 0, 0 },
    { 0xfa0ffb98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8ffb9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0ffba1, 33 },
    { 0, 0 },
    { 0xb51ffba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426ffba9, 78 },
    { 0x5ffbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2ffbae, 43 },
    { 0x53effbaf, 34 },
    { 0x45dffbaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173ffbb7, 33 },
    { 0xf29ffbb8, 61 },
    { 0x2ccffbb7, 98 },
    { 0x442ffbba, 95 },
    { 0x5ffffbbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x213ffbbe, 90 },
    { 0x4b2ffbbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ffbc4, 80 },
    { 0, 0 },
    { 0xcc3ffbc6, 78 },
    { 0x6faffbc7, 93 },
    { 0x5bbffbc8, 87 },
    { 0x3dcffbc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcffbcf, 95 },
    { 0, 0 },
    { 0x85affbd1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ffbd8, 99 },
    { 0x3c4ffbd9, 90 },
    { 0x735ffbd9, 95 },
    { 0xc6effbdb, 90 },
    { 0, 0 },
    { 0xa7affbdd, 84 },
    { 0x7a3ffbde, 61 },
    { 0xaa0ffbdd, 90 },
    { 0, 0 },
    { 0x242ffbe1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13ffbe4, 61 },
    { 0x30fffbe5, 23 },
    { 0x95bffbe5, 90 },
    { 0x5dbffbe6, 36 },
    { 0x66bffbe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x314ffbeb, 98 },
    { 0, 0 },
    { 0x7f4ffbed, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ffbf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaffbf9, 98 },
    { 0x154ffbfa, 55 },
    { 0, 0 },
    { 0x85cffbfc, 98 },
    { 0, 0 },
    { 0x1beffbfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962ffc05, 95 },
    { 0x757ffc06, 61 },
    { 0xf94ffc05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ffc0b, 81 },
    { 0, 0 },
    { 0xb85ffc0d, 23 },
    { 0xa0affc0e, 61 },
    { 0, 0 },
    { 0x47effc10, 76 },
    { 0, 0 },
    { 0xdc1ffc12, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758ffc18, 52 },
    { 0xc8cffc19, 76 },
    { 0, 0 },
    { 0x20dffc1b, 69 },
    { 0, 0 },
    { 0x6eaffc1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366ffc29, 98 },
    { 0x2feffc2a, 90 },
    { 0, 0 },
    { 0x2f5ffc2c, 93 },
    { 0xeaffc2d, 52 },
    { 0xbb1ffc2d, 90 },
    { 0x6caffc2f, 99 },
    { 0, 0 },
    { 0xc27ffc31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ffc35, 100 },
    { 0xbb2ffc35, 95 },
    { 0x2c7ffc35, 84 },
    { 0xc87ffc36, 61 },
    { 0x115ffc37, 76 },
    { 0, 0 },
    { 0x2eeffc3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fffc43, 93 },
    { 0, 0 },
    { 0x924ffc45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86effc48, 33 },
    { 0xa1fffc48, 98 },
    { 0, 0 },
    { 0x561ffc4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9affc50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681ffc54, 95 },
    { 0xc99ffc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x544ffc58, 83 },
    { 0x7f7ffc59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ffc5c, 52 },
    { 0x14affc5d, 61 },
    { 0, 0 },
    { 0x6f9ffc5f, 97 },
    { 0x9b6ffc60, 34 },
    { 0x4a9ffc60, 46 },
    { 0xb16ffc62, 98 },
    { 0xad1ffc63, 61 },
    { 0x31ffc64, 33 },
    { 0, 0 },
    { 0x407ffc66, 98 },
    { 0x2c3ffc67, 61 },
    { 0xe54ffc68, 90 },
    { 0x9eeffc66, 66 },
    { 0x362ffc6a, 23 },
    { 0xf06ffc6a, 55 },
    { 0xdd2ffc6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ffc72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37ffc76, 81 },
    { 0x9fcffc77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ffc7b, 53 },
    { 0x53ffc7c, 91 },
    { 0x448ffc7b, 55 },
    { 0xe56ffc7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ffc82, 93 },
    { 0xa9cffc83, 91 },
    { 0x683ffc83, 23 },
    { 0xde1ffc85, 98 },
    { 0x720ffc84, 43 },
    { 0x13affc87, 33 },
    { 0xd9fffc88, 34 },
    { 0x8ffc84, 98 },
    { 0x963ffc8a, 76 },
    { 0x16affc8b, 90 },
    { 0x8e2ffc8b, 34 },
    { 0x73effc8a, 36 },
    { 0, 0 },
    { 0x8a7ffc8f, 87 },
    { 0x638ffc90, 33 },
    { 0x9a3ffc91, 95 },
    { 0x2bcffc90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x523ffc95, 23 },
    { 0xbc7ffc96, 80 },
    { 0, 0 },
    { 0x5b4ffc98, 61 },
    { 0, 0 },
    { 0xca8ffc9a, 34 },
    { 0xb6effc9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750ffc9f, 34 },
    { 0xd1bffc9f, 87 },
    { 0xcdaffca1, 91 },
    { 0x6e5ffc9f, 55 },
    { 0xa19ffca3, 90 },
    { 0xe2cffca3, 61 },
    { 0x2a4ffca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acffca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ffcb3, 90 },
    { 0x259ffcb4, 43 },
    { 0xce6ffcb3, 17 },
    { 0x1a7ffcb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172ffcbb, 52 },
    { 0x6d2ffcbc, 76 },
    { 0xc2affcbd, 33 },
    { 0x399ffcbe, 96 },
    { 0x308ffcbe, 61 },
    { 0xf11ffcc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfffcc6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132ffccb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3affcce, 23 },
    { 0x477ffcce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dffcd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ffcdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416ffcdf, 90 },
    { 0, 0 },
    { 0xdb2ffce1, 43 },
    { 0, 0 },
    { 0x7b7ffce3, 90 },
    { 0x4c2ffce4, 95 },
    { 0x6adffce4, 77 },
    { 0x6ffce5, 95 },
    { 0x659ffce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cffcea, 81 },
    { 0xa33ffcea, 34 },
    { 0x8a2ffcec, 78 },
    { 0, 0 },
    { 0x89dffcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ffcf1, 98 },
    { 0, 0 },
    { 0xf0ffcf3, 33 },
    { 0x9c8ffcf3, 23 },
    { 0xafbffcf5, 93 },
    { 0x383ffcf6, 23 },
    { 0x957ffcf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ffcfe, 33 },
    { 0x675ffcff, 78 },
    { 0, 0 },
    { 0x4cfffd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adffd04, 95 },
    { 0x68ffd05, 66 },
    { 0x867ffd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ffd0c, 90 },
    { 0, 0 },
    { 0xa5bffd0e, 23 },
    { 0xefbffd0f, 98 },
    { 0, 0 },
    { 0x18ffd11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ffd15, 90 },
    { 0x358ffd16, 98 },
    { 0xc09ffd17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27affd1d, 90 },
    { 0x2b8ffd1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ffd26, 61 },
    { 0xfd2ffd27, 61 },
    { 0x571ffd26, 78 },
    { 0x160ffd29, 76 },
    { 0, 0 },
    { 0x9f1ffd2b, 40 },
    { 0x7d0ffd2c, 66 },
    { 0xf9affd2d, 66 },
    { 0x724ffd2e, 33 },
    { 0, 0 },
    { 0x22cffd30, 55 },
    { 0x66effd30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaffd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05ffd3d, 99 },
    { 0x5c2ffd3e, 90 },
    { 0x5ffd3f, 33 },
    { 0xb66ffd3d, 61 },
    { 0xe66ffd3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dffd44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cffd4c, 95 },
    { 0x6bdffd4c, 33 },
    { 0xb9effd4e, 61 },
    { 0, 0 },
    { 0x2f5ffd50, 46 },
    { 0xba2ffd51, 17 },
    { 0xccffd51, 43 },
    { 0xc58ffd53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffffd56, 66 },
    { 0x574ffd57, 68 },
    { 0, 0 },
    { 0xb59ffd59, 90 },
    { 0xcdcffd59, 76 },
    { 0xe49ffd5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ffd5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65affd64, 23 },
    { 0x108ffd64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374ffd69, 98 },
    { 0x7d7ffd69, 76 },
    { 0, 0 },
    { 0x579ffd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667ffd70, 100 },
    { 0xdf0ffd71, 98 },
    { 0, 0 },
    { 0x403ffd73, 46 },
    { 0x38dffd74, 34 },
    { 0x97affd75, 87 },
    { 0xfb7ffd76, 23 },
    { 0xae7ffd75, 84 },
    { 0x704ffd73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dffd7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ffd7e, 90 },
    { 0x516ffd7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ffd84, 34 },
    { 0, 0 },
    { 0x3d4ffd86, 34 },
    { 0x7a5ffd86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116ffd8e, 55 },
    { 0x467ffd8f, 98 },
    { 0x624ffd90, 90 },
    { 0xe19ffd91, 95 },
    { 0x338ffd90, 90 },
    { 0, 0 },
    { 0xd5fffd94, 90 },
    { 0x533ffd95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7ffd98, 90 },
    { 0x79fffd98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ffd9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ffda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ffda6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2ffdb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadffdb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fffdb6, 52 },
    { 0x69effdb7, 61 },
    { 0xa77ffdb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ffdbc, 33 },
    { 0, 0 },
    { 0xb14ffdbe, 61 },
    { 0, 0 },
    { 0xdfffdc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fffdc4, 93 },
    { 0x8d2ffdc5, 90 },
    { 0, 0 },
    { 0xb78ffdc7, 90 },
    { 0, 0 },
    { 0xb43ffdc9, 95 },
    { 0x55cffdca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cffdcd, 90 },
    { 0xf12ffdce, 55 },
    { 0xb95ffdcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ffdd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3ffddd, 76 },
    { 0xde7ffddd, 23 },
    { 0x629ffddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cffde5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cffde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84ffded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46ffdfb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661ffdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ffe03, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dffe09, 55 },
    { 0x6bffe0a, 76 },
    { 0, 0 },
    { 0xb10ffe0c, 33 },
    { 0, 0 },
    { 0x355ffe0e, 55 },
    { 0x5b6ffe0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdffe14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fffe1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbffe1f, 93 },
    { 0x102ffe20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44ffe23, 23 },
    { 0xf35ffe24, 93 },
    { 0, 0 },
    { 0x734ffe26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbffe29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fffe2f, 61 },
    { 0xd37ffe30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ffe33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceffe3a, 90 },
    { 0x2e7ffe3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fffe41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ffe44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ffe47, 95 },
    { 0xbf6ffe47, 95 },
    { 0, 0 },
    { 0xa19ffe4a, 43 },
    { 0x238ffe4b, 61 },
    { 0, 0 },
    { 0xc58ffe4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ffe53, 87 },
    { 0, 0 },
    { 0x2abffe55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0ffe58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ffe5f, 33 },
    { 0xb74ffe5f, 90 },
    { 0x743ffe61, 33 },
    { 0, 0 },
    { 0x37effe63, 95 },
    { 0x732ffe64, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528ffe6b, 52 },
    { 0x453ffe6b, 55 },
    { 0x5f6ffe6d, 34 },
    { 0, 0 },
    { 0xfafffe6f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ffe76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ffe79, 66 },
    { 0x265ffe7a, 55 },
    { 0xbf1ffe7b, 33 },
    { 0x4d8ffe7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ffe80, 90 },
    { 0, 0 },
    { 0xe2fffe82, 33 },
    { 0x9e9ffe82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2ffe87, 61 },
    { 0x6f6ffe88, 90 },
    { 0xe2fffe89, 85 },
    { 0x154ffe88, 90 },
    { 0x3edffe89, 98 },
    { 0x1d4ffe8a, 67 },
    { 0x46effe87, 55 },
    { 0x9cfffe8e, 33 },
    { 0xd61ffe8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5ffe96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bffe9a, 46 },
    { 0x7e8ffe9b, 46 },
    { 0xe55ffe9b, 76 },
    { 0x6a4ffe9b, 23 },
    { 0xef5ffe9e, 95 },
    { 0xef4ffe9f, 93 },
    { 0x50fffe9f, 61 },
    { 0x6a4ffea0, 76 },
    { 0x353ffea0, 33 },
    { 0x1eeffea3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1ffeae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bffeb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bffeb5, 80 },
    { 0x1e7ffeb6, 61 },
    { 0x20fffeb7, 23 },
    { 0x78affeb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30ffebd, 95 },
    { 0xfdeffebe, 61 },
    { 0, 0 },
    { 0xfeffec0, 76 },
    { 0, 0 },
    { 0x9a9ffec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dffec9, 90 },
    { 0xa06ffec9, 66 },
    { 0xb6dffec9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdffece, 84 },
    { 0, 0 },
    { 0x460ffed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ffed7, 95 },
    { 0x2baffed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07ffedb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50effedf, 17 },
    { 0xd39ffee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ffee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08ffeea, 33 },
    { 0x55cffeeb, 46 },
    { 0x75bffeec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3ffef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ffef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x733ffef7, 43 },
    { 0xcdcffef7, 67 },
    { 0, 0 },
    { 0x690ffefa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bffefd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90fff06, 90 },
    { 0xb4cfff06, 55 },
    { 0x8affff06, 76 },
    { 0xb5ffff09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9fff0c, 90 },
    { 0x28fff0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dfff11, 75 },
    { 0x5a4fff11, 23 },
    { 0xccbfff11, 33 },
    { 0, 0 },
    { 0xe82fff15, 61 },
    { 0xb5cfff16, 33 },
    { 0x5e7fff17, 61 },
    { 0xb27fff16, 61 },
    { 0x69afff15, 33 },
    { 0x5adfff1a, 78 },
    { 0x7eefff1b, 90 },
    { 0x9e3fff1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbfff1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1fff24, 55 },
    { 0x6fffff24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fff2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9fff2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2fff34, 95 },
    { 0x62bfff35, 100 },
    { 0, 0 },
    { 0xea6fff37, 55 },
    { 0xbebfff38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570fff3c, 95 },
    { 0, 0 },
    { 0xc8ffff3e, 95 },
    { 0xf5dfff3f, 91 },
    { 0xa39fff3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729fff44, 40 },
    { 0x663fff45, 87 },
    { 0x231fff46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x334fff49, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaefff4e, 61 },
    { 0xb5dfff4f, 95 },
    { 0, 0 },
    { 0x9d0fff51, 95 },
    { 0x82cfff52, 78 },
    { 0x51dfff53, 61 },
    { 0, 0 },
    { 0xbc3fff55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6fff5b, 61 },
    { 0, 0 },
    { 0x930fff5d, 84 },
    { 0, 0 },
    { 0x938fff5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfff62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x126fff65, 34 },
    { 0, 0 },
    { 0x500fff67, 61 },
    { 0, 0 },
    { 0x56ffff69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1fff6f, 93 },
    { 0x3a0fff70, 98 },
    { 0x273fff71, 95 },
    { 0x128fff72, 88 },
    { 0x220fff72, 98 },
    { 0xa0bfff70, 23 },
    { 0x4b6fff75, 90 },
    { 0x134fff75, 93 },
    { 0xd79fff77, 33 },
    { 0xd7fff74, 52 },
    { 0x2f5fff79, 23 },
    { 0xb4efff70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0fff7e, 90 },
    { 0xf85fff7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cfff82, 88 },
    { 0xb84fff82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594fff87, 61 },
    { 0, 0 },
    { 0x6d8fff89, 23 },
    { 0, 0 },
    { 0x450fff8b, 68 },
    { 0xc58fff8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfff93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903fff97, 33 },
    { 0, 0 },
    { 0x245fff99, 34 },
    { 0x487fff9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8fff9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52fffa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69fffaa, 100 },
    { 0x416fffaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767fffaf, 23 },
    { 0x886fffb0, 90 },
    { 0xef1fffb1, 90 },
    { 0xeccfffb2, 78 },
    { 0, 0 },
    { 0x81cfffb4, 77 },
    { 0, 0 },
    { 0x70efffb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fffbc, 61 },
    { 0xf48fffbd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92fffc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fffc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1fffc8, 97 },
    { 0, 0 },
    { 0x9effffca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8fffce, 75 },
    { 0xc3dfffcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81fffd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4fffd5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cfffdc, 95 },
    { 0x52dfffdd, 76 },
    { 0x310fffde, 88 },
    { 0xa49fffdc, 66 },
    { 0, 0 },
    { 0x64afffe1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656fffe7, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47fffef, 90 },
    { 0, 0 },
    { 0xa8fffff1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x861ffff4, 99 },
    { 0, 0 },
    { 0xd86ffff6, 95 },
    { 0, 0 },
    { 0x2affff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dffffb, 98 },
    { 0xfb6ffffc, 34 },
    { 0x8a0ffffd, 95 },
    { 0xd8fffffe, 84 },
    { 0x32dfffff, 84 }
};


static inline label brown_get_wc(uint32_t key) {
    size_t i = key & 0xfffff;
    for (;;) {
        if (brown[i].hash == key) return brown[i].value;
        if (brown[i].value == 0) return 0;
        i = (i + 1) & 0xfffff;
    }
}

static int extract_invariant(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        partial_hash_t *pi_hashes)
{
size_t i;

for (i=0; i<n_items; i++) {

    uint32_t codep_buf[MAX_STR];
    size_t codep_buf_len;

    uint32_t trans_buf[MAX_STR];
    size_t trans_buf_len;

    const partial_hash_t hash_2_x_x = hash32_data(1, field_buf[2], field_len[2]);

    const partial_hash_t hash_1_x_x_names = names_get_wc(hash32_data(1, field_buf[1], field_len[1]));

    const partial_hash_t hash_0_x_x_brown = brown_get_wc(hash32_data(1, field_buf[0], field_len[0]));

    const partial_hash_t hash_1_x_x = hash32_data(1, field_buf[1], field_len[1]);

    codep_buf_len = MAX_STR;
    if (utf8_decode(codep_buf, &codep_buf_len, field_buf[0], field_len[0]) < 0)
        return -1;

    trans_buf_len = MAX_STR;
    unicode_translate_normalize(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_pre1_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 1, 2);

    const partial_hash_t hash_0_pre2_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 2, 2);

    const partial_hash_t hash_0_pre3_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 3, 2);

    const partial_hash_t hash_0_pre4_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 4, 2);

    const partial_hash_t hash_0_suf1_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 1, 2);

    const partial_hash_t hash_0_suf2_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 2, 2);

    const partial_hash_t hash_0_suf3_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 3, 2);

    const partial_hash_t hash_0_suf4_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 4, 2);

    trans_buf_len = MAX_STR;
    unicode_translate_abstract(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_x_abstract =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    trans_buf_len = MAX_STR;
    unicode_translate_delexicalize(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_x_delexicalize =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    codep_buf_len = MAX_STR;
    if (utf8_decode(codep_buf, &codep_buf_len, field_buf[1], field_len[1]) < 0)
        return -1;

    trans_buf_len = MAX_STR;
    unicode_translate_normalize(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_1_x_normalize =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    pi_hashes[0] = hash_0_pre1_normalize;
    pi_hashes[1] = hash_0_pre2_normalize;
    pi_hashes[2] = hash_0_pre3_normalize;
    pi_hashes[3] = hash_0_pre4_normalize;
    pi_hashes[4] = hash_0_suf1_normalize;
    pi_hashes[5] = hash_0_suf2_normalize;
    pi_hashes[6] = hash_0_suf3_normalize;
    pi_hashes[7] = hash_0_suf4_normalize;
    pi_hashes[8] = hash_0_x_abstract;
    pi_hashes[9] = hash_0_x_delexicalize;
    pi_hashes[10] = hash_0_x_x_brown;
    pi_hashes[11] = hash_1_x_normalize;
    pi_hashes[12] = hash_1_x_x;
    pi_hashes[13] = hash_1_x_x_names;
    pi_hashes[14] = hash_2_x_x;

    field_buf += n_fields;
    field_len += n_fields;
    pi_hashes += 15;
}
return 0;
}

#define N_INVARIANTS    15
#define N_FEATURES      31

static void extract_features(
        const label *history,
        label tag,
        size_t i,
        size_t n_items,
        const partial_hash_t *pi_hashes,
        feat_hash_t *feature_hashes)
{
    feature_hashes[0] = hash32_fmix(hash32_mix(1, hash32_mix(tag, ((i>=1)? history[i-1] : suc_ne_N_TAGS))));
    feature_hashes[1] = hash32_fmix(hash32_mix(2, hash32_mix(tag, hash32_mix(((i>=1)? history[i-1] : suc_ne_N_TAGS), ((i>=2)? history[i-2] : suc_ne_N_TAGS)))));
    feature_hashes[2] = hash32_fmix(hash32_mix(3, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 14] : 0x01694c52), tag)));
    feature_hashes[3] = hash32_fmix(hash32_mix(4, hash32_mix(pi_hashes[i*N_INVARIANTS + 14], tag)));
    feature_hashes[4] = hash32_fmix(hash32_mix(5, hash32_mix(((i < n_items-2)? pi_hashes[(i+2)*N_INVARIANTS + 14] : 0x01694c52), tag)));
    feature_hashes[5] = hash32_fmix(hash32_mix(6, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 13] : 0xb4c24b1a), tag)));
    feature_hashes[6] = hash32_fmix(hash32_mix(7, hash32_mix(pi_hashes[i*N_INVARIANTS + 13], tag)));
    feature_hashes[7] = hash32_fmix(hash32_mix(8, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 13] : 0xb4c24b1a), tag)));
    feature_hashes[8] = hash32_fmix(hash32_mix(9, hash32_mix(pi_hashes[i*N_INVARIANTS + 13], hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 13] : 0xb4c24b1a), tag))));
    feature_hashes[9] = hash32_fmix(hash32_mix(10, hash32_mix(pi_hashes[i*N_INVARIANTS + 13], hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 13] : 0xb4c24b1a), tag))));
    feature_hashes[10] = hash32_fmix(hash32_mix(11, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 10] : 0xdcdeaf63), tag)));
    feature_hashes[11] = hash32_fmix(hash32_mix(12, hash32_mix(pi_hashes[i*N_INVARIANTS + 10], tag)));
    feature_hashes[12] = hash32_fmix(hash32_mix(13, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 10] : 0xdcdeaf63), tag)));
    feature_hashes[13] = hash32_fmix(hash32_mix(14, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 10] : 0xdcdeaf63), hash32_mix(((i < n_items-2)? pi_hashes[(i+2)*N_INVARIANTS + 10] : 0xdcdeaf63), tag))));
    feature_hashes[14] = hash32_fmix(hash32_mix(15, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 10] : 0xdcdeaf63), hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 10] : 0xdcdeaf63), tag))));
    feature_hashes[15] = hash32_fmix(hash32_mix(16, hash32_mix(pi_hashes[i*N_INVARIANTS + 9], tag)));
    feature_hashes[16] = hash32_fmix(hash32_mix(17, hash32_mix(pi_hashes[i*N_INVARIANTS + 8], tag)));
    feature_hashes[17] = hash32_fmix(hash32_mix(18, hash32_mix(pi_hashes[i*N_INVARIANTS + 12], tag)));
    feature_hashes[18] = hash32_fmix(hash32_mix(19, hash32_mix(pi_hashes[i*N_INVARIANTS + 12], hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 12] : 0xd50987d6), tag))));
    feature_hashes[19] = hash32_fmix(hash32_mix(20, hash32_mix(pi_hashes[i*N_INVARIANTS + 12], hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 12] : 0xd50987d6), tag))));
    feature_hashes[20] = hash32_fmix(hash32_mix(21, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 12] : 0xd50987d6), tag)));
    feature_hashes[21] = hash32_fmix(hash32_mix(22, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 12] : 0xd50987d6), tag)));
    feature_hashes[22] = hash32_fmix(hash32_mix(23, hash32_mix(pi_hashes[i*N_INVARIANTS + 11], tag)));
    feature_hashes[23] = hash32_fmix(hash32_mix(24, hash32_mix(pi_hashes[i*N_INVARIANTS + 0], tag)));
    feature_hashes[24] = hash32_fmix(hash32_mix(25, hash32_mix(pi_hashes[i*N_INVARIANTS + 1], tag)));
    feature_hashes[25] = hash32_fmix(hash32_mix(26, hash32_mix(pi_hashes[i*N_INVARIANTS + 2], tag)));
    feature_hashes[26] = hash32_fmix(hash32_mix(27, hash32_mix(pi_hashes[i*N_INVARIANTS + 3], tag)));
    feature_hashes[27] = hash32_fmix(hash32_mix(28, hash32_mix(pi_hashes[i*N_INVARIANTS + 4], tag)));
    feature_hashes[28] = hash32_fmix(hash32_mix(29, hash32_mix(pi_hashes[i*N_INVARIANTS + 5], tag)));
    feature_hashes[29] = hash32_fmix(hash32_mix(30, hash32_mix(pi_hashes[i*N_INVARIANTS + 6], tag)));
    feature_hashes[30] = hash32_fmix(hash32_mix(31, hash32_mix(pi_hashes[i*N_INVARIANTS + 7], tag)));
}


#if BEAM_SIZE == 1
static void beam_search(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        const real *weights,
        size_t weights_len,
        int use_lexicon,
        int use_dropout,
        feat_hash_t dropout_seed,
        label *result)
{
    size_t i;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    const label *tags;
    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    for (i=0; i<n_items; i++) {
        real max_score = -REAL_MAX;
        label max_tag = 0;

        if (use_lexicon) {
            size_t j;
            tags = get_tags(hash32_fmix(invariant_hashes[N_INVARIANTS*i + 11]));
            if (tags[0] == 1) {
                //printf("Only tag available: %d\n", tags[1]);
                max_tag = (label)tags[1];
            } else {
                //printf("%d tags available\n", tags[0]);
                for (j=1; j<tags[0]+1; j++) {
                    extract_features(
                            result, tags[j], i, n_items,
                            invariant_hashes, feature_hashes);
                    const real score = get_score(
                            feature_hashes, N_FEATURES, weights, weights_len,
                            use_dropout, dropout_seed);
                    if (score > max_score) {
                        max_score = score;
                        max_tag = (label)tags[j];
                    }
                }
            }
        } else {
            label tag;
            for (tag=0; tag<N_TAGS; tag++) {
                extract_features(
                        result, tag, i, n_items,
                        invariant_hashes, feature_hashes);
                const real score = get_score(
                        feature_hashes, N_FEATURES, weights, weights_len,
                        use_dropout, dropout_seed);
                if (score > max_score) {
                    max_score = score;
                    max_tag = (label)tag;
                }
            }
        }
        result[i] = max_tag;
    }
}
#endif

#if BEAM_SIZE > 1
static void beam_search(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        const real *weights,
        size_t weights_len,
        int use_lexicon,
        int use_dropout,
        feat_hash_t dropout_seed,
        label *result)
{
    size_t i;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    real beam_scores[BEAM_SIZE];
    label new_beams[BEAM_SIZE][n_items];
    label beams[BEAM_SIZE][n_items];
    // this is the actual beam size, whereas BEAM_SIZE is the maximum size
    size_t beam_size = 1;
    const label *tags;
    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    beam_scores[0] = (real)0.0;

    for (i=0; i<n_items; i++) {
        size_t k;
        real max_score[BEAM_SIZE];
        label max_tag[BEAM_SIZE];
        size_t max_beam[BEAM_SIZE];
        for (k=0; k<BEAM_SIZE; k++) {
            max_score[k] = -REAL_MAX;
            max_tag[k] = 0;
            max_beam[k] = 0;
        }

        if (use_lexicon) {
            size_t j;
            tags = get_tags(hash32_fmix(invariant_hashes[N_INVARIANTS*i + 11]));
            /* if (tags[0] == 1) {
                for (k=0; k<beam_size; k++) {
                    max_tag[k] = tags[1];
                    max_beam[k] = 0;
                    max_score[k] = beam_scores[k];
                }
            } else */ {
                for (j=1; j<tags[0]+1; j++) {
                    for (k=0; k<beam_size; k++) {
                        extract_features(
                                beams[k], tags[j], i, n_items,
                                invariant_hashes, feature_hashes);
                        const real score = beam_scores[k] + get_score(
                                feature_hashes, N_FEATURES,
                                weights, weights_len,
                                use_dropout, dropout_seed);
                        if (score > max_score[BEAM_SIZE-1]) {
                            size_t l,m;
                            for (l=0; score < max_score[l]; l++);
                            for (m=BEAM_SIZE-1; m>l; m--) {
                                max_score[m] = max_score[m-1];
                                max_tag[m] = max_tag[m-1];
                                max_beam[m] = max_beam[m-1];
                            }
                            max_score[l] = score;
                            max_tag[l] = (label)tags[j];
                            max_beam[l] = k;
                        }
                    }
                }
            }
        } else {
            label tag;
            for (tag=0; tag<N_TAGS; tag++) {
                for (k=0; k<beam_size; k++) {
                    extract_features(
                            beams[k], tag, i, n_items,
                            invariant_hashes, feature_hashes);
                    const real score = beam_scores[k] + get_score(
                            feature_hashes, N_FEATURES, weights, weights_len,
                            use_dropout, dropout_seed);
                    if (score > max_score[BEAM_SIZE-1]) {
                        size_t l,m;
                        for (l=0; score < max_score[l]; l++);
                        for (m=BEAM_SIZE-1; m>l; m--) {
                            max_score[m] = max_score[m-1];
                            max_tag[m] = max_tag[m-1];
                            max_beam[m] = max_beam[m-1];
                        }
                        max_score[l] = score;
                        max_tag[l] = tag;
                        max_beam[l] = k;
                    }
                }
            }
        }
        for (k=0; k<BEAM_SIZE && max_score[k] != -REAL_MAX; k++) {
            beams[k][i] = max_tag[k];
            beam_scores[k] = max_score[k];
        }
        beam_size = k;
        if (i > 0) {
            for (k=0; k<beam_size; k++)
                if (max_beam[k] != k)
                    memcpy(new_beams[k], beams[max_beam[k]], i*sizeof(label));
            for (k=0; k<beam_size; k++)
                if (max_beam[k] != k)
                    memcpy(beams[k], new_beams[k], i*sizeof(label));
        }
    }
    memcpy(result, beams[0], n_items*sizeof(label));
}
#endif
#ifndef __SEQ_C__
#define __SEQ_C__

// If the output should be aligned, set this to 1.
#define ALIGN_FIELDS    0
#define ALIGNMENT       (sizeof(size_t))

// If fields should be zero-terminated, set this to 1.
#define ZERO_TERMINATE  1

static int read_sequence(
        FILE *file,
        uint8_t **field_buf,
        size_t *field_len,
        size_t n_fields,
        size_t *n_items,
        uint8_t *buf,
        size_t *buf_len)
{
    const size_t max_items = *n_items;
    size_t field, item, field_p = 0, buf_p = 0;
    const size_t max_len = *buf_len;

    for (item=0; item<max_items; item++) {
        for (field=0; field<n_fields; field++) {
#if ALIGN_FIELDS
            while (buf_p % ALIGNMENT) buf[buf_p++] = 0;
            if (unlikely(buf_p >= max_len)) return -1;
#endif
            const size_t buf_p_start = buf_p;
            for (;;) {
                const int c = getc_unlocked(file);
                if (likely(c > '\n')) {
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = c;
                } else if (unlikely(c == '\t')) {
                    if (unlikely(field == n_fields-1)) return -1;
                    field_buf[field_p] = buf + buf_p_start;
                    if ((field_len[field_p] = buf_p - buf_p_start) >= MAX_STR) {
                        field_len[field_p] = MAX_STR-1;
                        buf[buf_p_start + MAX_STR-1] = 0;
                    }
                    field_p++;
#if ZERO_TERMINATE
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = 0;
#endif
                    break;
                } else if (unlikely(c == '\n')) {
                    if (unlikely(field == 0 && buf_p == buf_p_start)) {
                        *n_items = item;
                        *buf_len = buf_p;
                        return 0;
                    }
                    if (unlikely(field != n_fields-1)) return -1;
                    field_buf[field_p] = buf + buf_p_start;
                    if ((field_len[field_p] = buf_p - buf_p_start) >= MAX_STR) {
                        field_len[field_p] = MAX_STR-1;
                        buf[buf_p_start + MAX_STR-1] = 0;
                    }
                    field_p++;
#if ZERO_TERMINATE
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = 0;
#endif
                    break;
                } else if (unlikely(c == EOF)) return -1;
                // For the time being, silently drop ASCII values below 10
                // except newline and tab.
            }
        }
    }
    return -1;
}

#endif


#ifndef __SEARCH_C__
#define __SEARCH_C__

#include <string.h>

static void adjust_weights(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        real *weights,
        size_t weights_len,
        const label *labels,
        real weight_diff,
        double t,
        double *average_weights,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    size_t i, j;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    feat_hash_t mask = (feat_hash_t)weights_len - 1;

    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    for (i=0; i<n_items; i++) {
        extract_features(
                labels, labels[i], i, n_items,
                invariant_hashes, feature_hashes);
        for (j=0; j<N_FEATURES; j++) {
            const feat_hash_t h = feature_hashes[j];
            if ((!use_dropout) ||
                (hash32_mix(dropout_seed, h) >= DROPOUT_CONSTANT)) {
                const size_t idx = h & mask;
                average_weights[idx*2] +=
                    (t - average_weights[idx*2+1]) * (double)weights[idx];
                average_weights[idx*2+1] = t;
                weights[idx] += weight_diff;
            }
        }
    }
}

static size_t train_sequence(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        real *weights,
        size_t weights_len,
        const label *gold,
        double t,
        double *average_weights,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    label crap[n_items];
    beam_search(field_buf, field_len, n_fields, n_items,
                  weights, weights_len, 0, use_dropout, dropout_seed, crap);
    if (!memcmp(crap, gold, n_items*sizeof(label))) return 0;
    size_t n_errs = 0;
    size_t i;
    for (i=0; i<n_items; i++)
        n_errs += (gold[i] != crap[i]);
    adjust_weights(field_buf, field_len, n_fields, n_items,
                   weights, weights_len, gold, (real)1.0,
                   t, average_weights,
                   use_dropout, dropout_seed);
    adjust_weights(field_buf, field_len, n_fields, n_items,
                   weights, weights_len, crap, (real)(-1.0),
                   t, average_weights,
                   use_dropout, dropout_seed);
    return n_errs;
}

#endif

#ifndef __TAG_C__
#define __TAG_C__

static int tag(
        FILE *infile,
        const real *weights,
        size_t weights_len,
        FILE *outfile,
        size_t n_fields,
        double *error_rate)
{
    size_t i;

    const size_t max_items = 0x400;
    const size_t max_fields = max_items*n_fields;
    size_t max_len = 0x10000;

    uint8_t *field_buf[max_fields];
    size_t field_len[max_fields];
    size_t n_items;
    uint8_t buf[max_len];

    size_t n_total = 0, n_errors = 0;

    for (;;) {
        n_items = max_items;
        size_t buf_len = max_len;
        const int rv = read_sequence(
                infile, field_buf, field_len, n_fields, &n_items,
                buf, &buf_len);
        if (rv < 0) {
            if (!feof(infile)) {
                fprintf(stderr, "Error at %ld!\n", ftell(infile));
                return -1;
            }
            break;
        }

        label result[n_items];
        beam_search(
                (const uint8_t**)field_buf, field_len, n_fields,
                n_items, weights, weights_len, 1, 0, 0, result);

        if (outfile != NULL) {
            for (i=0; i<n_items; i++) {
                size_t j;
                for (j=0; j<N_TRAIN_FIELDS; j++) {
                    if (j == COL_TAG)
                        fputs((char*)tag_str[result[i]], outfile);
                    else if (j < COL_TAG)
                        fputs((char*)field_buf[i*n_fields+j], outfile);
                    else
                        fputs((char*)field_buf[i*n_fields+j-1], outfile);
                    if (j < N_TRAIN_FIELDS-1)
                        fputc('\t', outfile);
                }
                fputc('\n', outfile);
            }
            fputc('\n', outfile);
        }

        if (error_rate != NULL) {
            for (i=0; i<n_items; i++) {
                const int tag = tagset_from_str(
                        (const char*)field_buf[i*n_fields + COL_TAG]);
                if (tag < 0) {
                    fprintf(stderr, "Invalid tag: '%s'\n",
                            field_buf[i*n_fields + COL_TAG]);
                }
                if (result[i] != tag) n_errors++;
                n_total++;
            }
        }

    }

    if (error_rate != NULL) {
        if (n_total > 0) *error_rate = (double)n_errors / (double)n_total;
        else *error_rate = -1.0;
    }

    fflush(outfile);

    return 0;
}

#endif

#ifndef __TRAIN_C__
#define __TRAIN_C__

#include <stdlib.h>
#include <errno.h>

static void shuffle(size_t *items, size_t len) {
    size_t i;
    for (i=0; i<len; i++) {
        size_t j = random() % len;
        size_t t = items[i];
        items[i] = items[j];
        items[j] = t;
    }
}

static int train(
        const char *train_filename,
        const char *tune_filename,
        const char *model_filename)
{
    FILE *train_file = fopen(train_filename, "rb");
    FILE *tune_file = fopen(tune_filename, "rb");
    const int use_dropout = DROPOUT_RATE != 0;

    if (train_file == NULL) {
        perror("unable to open training data file");
        exit(1);
    }

    if (tune_file == NULL) {
        perror("unable to open tuning data file");
        exit(1);
    }

    size_t i;

    const size_t max_items = 0x400;
    const size_t max_fields = max_items*N_TRAIN_FIELDS;
    const size_t max_len = 0x10000;
    
    double best_error_ever = 1.0;

    FILE *model = NULL;

    feat_hash_t dropout_seed = 1;

    uint8_t *field_buf[max_fields];
    size_t field_len[max_fields];
    size_t n_items;
    uint8_t buf[max_len];

    // First, get file offsets of sentence starts
    size_t max_sents = 0x100000;
    size_t n_sents;
    long *sent_offsets = malloc(sizeof(long)*max_sents);

    for (n_sents=0; ; n_sents++) {
        if (n_sents >= max_sents) {
            return 1;
        }
        sent_offsets[n_sents] = ftell(train_file);
        n_items = max_items;
        size_t buf_len = max_len;
        const int rv = read_sequence(
                train_file, field_buf, field_len, N_TRAIN_FIELDS, &n_items,
                buf, &buf_len);
        if (rv < 0) {
            if (!feof(train_file)) {
                fprintf(stderr, "Error at %s:%ld (bytes)!\n",
                        train_filename, ftell(train_file));
                return 1;
            }
            rewind(train_file);
            break;
        }
    }

    sent_offsets = realloc(sent_offsets, sizeof(long)*n_sents);
    size_t sent_order[n_sents];
    for (i=0; i<n_sents; i++) sent_order[i] = i;

    fprintf(stderr, "Training data contains %zd sentences\n", n_sents);

    size_t weights_len;
    for (weights_len=MIN_WEIGHTS_LEN;
         weights_len<=MAX_WEIGHTS_LEN;
         weights_len*=2)
    {
        fprintf(stderr, "Trying weight vector of size 0x%zx\n", weights_len);
        real *weights = malloc(sizeof(real)*weights_len);
        // Even elements contain averages of the weights vector,
        // odd elements contain the time (i.e. value of t) of the last update
        // of the average.
        // This is less elegant, but good for cache locality since these values
        // are accessed randomly at the same time.
        // TODO: might also want to get the weight vector itself into the same
        // area.
        double *average_weights = malloc(sizeof(double)*weights_len*2);
        double t = 0.0;

        for (i=0; i<weights_len; i++) weights[i] = (real)0.0;
        for (i=0; i<weights_len*2; i++) average_weights[i] = 0.0;

        size_t iter;
        double tune_error_avg = 1.0;
        double best_error = 1.0;

        const int max_patience = 5;
        int patience_left = max_patience;

        double tune_error = 1.0;
        for (iter=0; ; iter++) {
            shuffle(sent_order, n_sents);

            fprintf(stderr, "Iteration %zd...\n", iter+1);
            size_t n_errs = 0;
            size_t n_total = 0;
            size_t sent;
            for (sent=0; sent<n_sents; sent++) {
                fseek(train_file, sent_offsets[sent_order[sent]], SEEK_SET);

                n_items = max_items;
                size_t buf_len = max_len;
                const int rv = read_sequence(
                        train_file, field_buf, field_len, N_TRAIN_FIELDS,
                        &n_items, buf, &buf_len);
                if (rv < 0) {
                    fprintf(stderr, "Error at %s:%ld (bytes)!\n",
                            train_filename, ftell(train_file));
                    return 1;
                }

                label gold[n_items];
                for (i=0; i<n_items; i++) {
                    const int tag = tagset_from_str(
                            (const char*)field_buf[i*N_TRAIN_FIELDS + COL_TAG]);
                    if (tag < 0) {
                        fprintf(stderr, "Invalid tag: '%s'\n",
                                field_buf[i*N_TRAIN_FIELDS + COL_TAG]);
                    }
                    gold[i] = tag;
                }

                n_total += n_items;
                n_errs += train_sequence(
                        (const uint8_t**)field_buf, field_len, N_TRAIN_FIELDS,
                        n_items, weights, weights_len, gold,
                        t, average_weights,
                        use_dropout, dropout_seed);
                t += 1.0;
                dropout_seed++;
            }

            fprintf(stderr, "  Training error: %.2f%%\n",
                    100.0*(double)n_errs/(double)n_total);

            for (i=0; i<weights_len; i++) {
                average_weights[i*2] +=
                    (t - average_weights[i*2+1]) * (double)weights[i];
                average_weights[i*2+1] = t;
            }

            tune_error = 1.0;

            real *real_average_weights = malloc(sizeof(real)*weights_len);
            for (i=0; i<weights_len; i++)
                real_average_weights[i] = (real)average_weights[i*2];
            tag(tune_file, real_average_weights, weights_len, NULL, 
                N_TRAIN_FIELDS, &tune_error);
            rewind(tune_file);

            fprintf(stderr, "  Tuning error:   %.2f%%\n", 100.0*tune_error);

            if (tune_error < best_error_ever) {
                best_error_ever = tune_error;

                fprintf(stderr, "  Best so far, writing...\n");

                model = fopen(model_filename, "wb");
                if (model == NULL) {
                    perror("unable to open model file for writing");
                    exit(1);
                }
                fwrite(real_average_weights, sizeof(real), weights_len, model);
                fclose(model);
            }

            free(real_average_weights);

            if (tune_error < best_error) {
                best_error = tune_error;
                patience_left = max_patience;
            }
            if (iter == 0) {
                tune_error_avg = tune_error;
            } else {
                if (tune_error > 0.99*tune_error_avg) {
                    if (! (--patience_left)) break;
                }
                tune_error_avg = tune_error_avg*0.5 + tune_error*0.5;
            }
        }

        for (i=0; i<weights_len; i++)
            weights[i] = (real)average_weights[i*2];

        free(average_weights);

#ifdef POST_TRAINING_COMPRESSION
        fprintf(stderr, "Finding optimal feature compression...\n");

        model = fopen(model_filename, "rb");
        if (model == NULL) {
            perror("unable to open model file for reading");
            exit(1);
        }
        fread(weights, sizeof(real), weights_len, model);
        fclose(model);

        size_t compressed_len = weights_len;
        size_t compression;
        for (compression=2; ; compression*=2) {
            compressed_len /= 2;
            real *folded_weights = malloc(sizeof(real)*compressed_len);
            for (i=0; i<compressed_len; i++)
                folded_weights[i] = weights[i] + weights[compressed_len+i];
            tune_error = 1.0;
            tag(tune_file, folded_weights, compressed_len, NULL, N_TRAIN_FIELDS,
                &tune_error);
            rewind(tune_file);
            fprintf(stderr, "  %zdx compression tuning error: %.2f%%\n",
                    compression, 100.0*tune_error);
            if (tune_error > 1.0025 * best_error) {
                free(folded_weights);
                fprintf(stderr, "Selected %zdx compression: 0x%zx features\n",
                        compression/2, compressed_len*2);
                compressed_len *= 2;
                break;
            }
            free(weights);
            weights = folded_weights;
            if (tune_error < best_error) best_error = tune_error;
            if (tune_error < best_error_ever) best_error_ever = tune_error;
        }

        model = fopen(model_filename, "wb");
        if (model == NULL) {
            perror("unable to open model file for writing");
            exit(1);
        }
        fwrite(weights, sizeof(real), compressed_len, model);
        fclose(model);
#endif

        free(weights);

        if (best_error > best_error_ever*1.0025) {
            fprintf(stderr, "Error no longer decreasing with compression.\n");
            break;
        }
    }

    fclose(train_file);
    fclose(tune_file);

    return 0;
}

#endif

#ifndef __RUN_C__
#define __RUN_C__

static int run(
        const char *data_filename,
        const char *model_filename,
        int evaluate)
{
    FILE *file = (!strcmp(data_filename, "-"))? stdin :
                 fopen(data_filename, "rb");
    if (file == NULL) {
        fprintf(stderr, "Error: can not open input file %s\n", data_filename);
        return -1;
    }
    FILE *model = fopen(model_filename, "rb");
    if (model == NULL) {
        fprintf(stderr, "Error: can not open model file %s\n", model_filename);
        return -1;
    }

    fseek(model, 0, SEEK_END);
    const size_t weights_len = ftell(model) / sizeof(real);

    if (weights_len & (weights_len-1)) {
        fprintf(stderr, "Model file size not power of 2!\n");
        return -1;
    }
    rewind(model);

    real *weights = malloc(sizeof(real)*weights_len);
    if (weights == NULL) {
        fprintf(stderr, "Error: unable to allocate weight vector memory\n");
        return -1;
    }
    if (fread(weights, sizeof(real), weights_len, model) != weights_len) {
        fprintf(stderr, "Error: unable to read weights vector\n");
        return -1;
    }
    fclose(model);

    double error_rate = 1.0;
    if (tag(file, weights, weights_len, stdout,
            (evaluate)? N_TRAIN_FIELDS : N_TAG_FIELDS,
            (evaluate)? &error_rate : NULL)) {
        fprintf(stderr, "Tagging failed!\n");
        fclose(file);
        free(weights);
        return 1;
    }

    if (evaluate)
        fprintf(stderr, "Error rate: %.2f%%\n", 100.0*error_rate);

    fclose(file);
    free(weights);

    return 0;
}

#endif

int main(int argc, const char **argv) {
    if (argc == 5 && !strcmp(argv[1], "train")) {
        if (train(argv[2], argv[3], argv[4])) return 1;
    } else if (argc == 4 && !strcmp(argv[1], "tag")) {
        if (run(argv[2], argv[3], 0)) return 1;
    } else if (argc == 5 && !strcmp(argv[1], "tag") &&
               !strcmp(argv[4], "evaluate")) {
        if (run(argv[2], argv[3], 1)) return 1;
    } else {
        fprintf(stderr,
                "Usage:\n"
                "    %s train train.txt tune.txt model.bin\n"
                "    %s tag input.txt model.bin [evaluate]\n\n"
                "For tagging, the input file may be \"-\" to use stdin\n\n",
                argv[0], argv[0]);
        return 1;
    }
    return 0;
}

#define PY_SSIZE_T_CLEAN
#include <Python.h>

static PyObject *py_tag(PyObject *self, PyObject *args) {
    PyObject *seq;
    Py_ssize_t buf_len;
    const char *buf;
    if (!PyArg_ParseTuple(args, "y#O", &buf, &buf_len, &seq)) return NULL;
    if (buf_len & (buf_len-1)) {
        PyErr_SetString(PyExc_ValueError,
                "Weights vector length must be power of 2");
        return NULL;
    }
    int clear_seq = 0;
    if (PyList_Check(seq)) {
        seq = PyList_AsTuple(seq);
        clear_seq = 1;
    } else if (!PyTuple_Check(seq)) {
        PyErr_SetString(PyExc_TypeError, "Expected a list or tuple");
        return NULL;
    }
    const real *weights = (real*)buf;
    size_t weights_len = buf_len / sizeof(*weights);
    Py_ssize_t seq_len = PyTuple_Size(seq), i, j;
    uint8_t *field_buf[seq_len*N_TAG_FIELDS];
    size_t field_len[seq_len*N_TAG_FIELDS];
    PyObject *field_bytes[seq_len*N_TAG_FIELDS];

    for (i=0; i<seq_len; i++) {
        PyObject *row = PyTuple_GetItem(seq, i);
        if (PyUnicode_Check(row)) {
            if (N_TAG_FIELDS != 1) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Expected %d fields for token, found single string",
                        N_TAG_FIELDS);
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            PyObject *buf = PyUnicode_AsEncodedString(row, "utf-8", NULL);
            if (PyBytes_Size(buf) >= MAX_STR) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Input string too long: %zd bytes",
                        PyBytes_Size(buf));
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            field_buf[i*N_TAG_FIELDS + 0] = (uint8_t*)PyBytes_AsString(buf);
            field_len[i*N_TAG_FIELDS + 0] = PyBytes_Size(buf);
            field_bytes[i*N_TAG_FIELDS + 0] = buf;
        } else {
            int clear_row = 0;
            if (PyList_Check(row)) {
                row = PyList_AsTuple(row);
                clear_row = 1;
            }
            if (!PyTuple_Check(row)) {
                PyErr_SetString(PyExc_TypeError,
                        "Expected tuple, list or str for token");
                return NULL;
            }
            if (PyTuple_Size(row) != N_TAG_FIELDS) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Expected %d fields for token, found %zd",
                        N_TAG_FIELDS, PyTuple_Size(row));
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            for (j=0; j<N_TAG_FIELDS; j++) {
                PyObject *str = PyTuple_GetItem(row, j);
                if (PyBytes_Check(str)) {
                    if (PyBytes_Size(str) >= MAX_STR) {
                        char msg[0x100];
                        snprintf(msg, sizeof(msg),
                                "Input string too long: %zd bytes",
                                PyBytes_Size(str));
                        PyErr_SetString(PyExc_ValueError, msg);
                        return NULL;
                    }
                    field_buf[i*N_TAG_FIELDS + j] =
                        (uint8_t*)PyBytes_AsString(str);
                    field_len[i*N_TAG_FIELDS + j] = PyBytes_Size(str);
                    field_bytes[i*N_TAG_FIELDS + j] = NULL;
                } else if(PyUnicode_Check(str)) {
                    PyObject *buf = PyUnicode_AsEncodedString(
                            str, "utf-8", NULL);
                    if (PyBytes_Size(buf) >= MAX_STR) {
                        char msg[0x100];
                        snprintf(msg, sizeof(msg),
                                "Input string too long: %zd bytes",
                                PyBytes_Size(buf));
                        PyErr_SetString(PyExc_ValueError, msg);
                        return NULL;
                    }
                    field_buf[i*N_TAG_FIELDS + j] =
                        (uint8_t*)PyBytes_AsString(buf);
                    field_len[i*N_TAG_FIELDS + j] = PyBytes_Size(buf);
                    field_bytes[i*N_TAG_FIELDS + j] = buf;
                } else {
                    PyErr_SetString(PyExc_TypeError, "Expected bytes or str");
                    return NULL;
                }
            }
            if (clear_row) Py_CLEAR(row);
        }
    }

    label result[seq_len];
    beam_search(
            (const uint8_t**)field_buf, field_len, N_TAG_FIELDS,
            seq_len, weights, weights_len, 1, 0, 0, result);

    PyObject *tags = PyTuple_New(seq_len);
    for (i=0; i<seq_len; i++)
        PyTuple_SetItem(tags, i, PyUnicode_FromString(tag_str[result[i]]));

    for (i=0; i<seq_len; i++) {
        for (j=0; j<N_TAG_FIELDS; j++) {
            Py_CLEAR(field_bytes[i*N_TAG_FIELDS + j]);
        }
    }
    if (clear_seq) Py_CLEAR(seq);

    return tags;
}

static PyMethodDef py_methods[] = {
    { "tag", py_tag, METH_VARARGS, "Tag one sentence" },
    {NULL, NULL, 0, NULL}
};

static struct PyModuleDef py_module = {
    PyModuleDef_HEAD_INIT,
    TAGGER_NAME,
    NULL,
    -1,
    py_methods
};

PyMODINIT_FUNC
PyInit_TAGGER_NAME(void) {
        return PyModule_Create(&py_module);
}

